!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2010 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2010* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB ,&
&  0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB, 0.63068E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.65526E+00_JPRB, 0.65400E+00_JPRB, 0.65286E+00_JPRB, 0.65184E+00_JPRB, 0.65086E+00_JPRB, 0.65012E+00_JPRB, 0.64950E+00_JPRB ,&
&  0.64859E+00_JPRB, 0.64763E+00_JPRB, 0.64658E+00_JPRB, 0.64563E+00_JPRB, 0.64468E+00_JPRB, 0.64364E+00_JPRB, 0.64241E+00_JPRB ,&
&  0.64106E+00_JPRB, 0.63979E+00_JPRB, 0.63860E+00_JPRB, 0.63719E+00_JPRB, 0.63568E+00_JPRB, 0.63395E+00_JPRB, 0.63204E+00_JPRB ,&
&  0.62989E+00_JPRB, 0.62765E+00_JPRB, 0.62528E+00_JPRB, 0.62285E+00_JPRB, 0.62015E+00_JPRB, 0.61694E+00_JPRB, 0.61411E+00_JPRB ,&
&  0.61203E+00_JPRB, 0.60994E+00_JPRB, 0.60796E+00_JPRB, 0.60591E+00_JPRB, 0.60418E+00_JPRB, 0.60295E+00_JPRB, 0.60170E+00_JPRB ,&
&  0.60067E+00_JPRB, 0.59985E+00_JPRB, 0.59989E+00_JPRB, 0.60022E+00_JPRB, 0.60076E+00_JPRB, 0.60210E+00_JPRB, 0.60425E+00_JPRB ,&
&  0.60615E+00_JPRB, 0.60786E+00_JPRB, 0.60991E+00_JPRB, 0.61215E+00_JPRB, 0.61447E+00_JPRB, 0.61679E+00_JPRB, 0.61900E+00_JPRB ,&
&  0.62071E+00_JPRB, 0.62213E+00_JPRB, 0.62361E+00_JPRB, 0.62488E+00_JPRB, 0.62557E+00_JPRB, 0.62618E+00_JPRB, 0.62680E+00_JPRB ,&
&  0.62731E+00_JPRB, 0.62768E+00_JPRB, 0.62822E+00_JPRB, 0.62916E+00_JPRB, 0.63008E+00_JPRB, 0.63108E+00_JPRB, 0.63241E+00_JPRB ,&
&  0.63416E+00_JPRB, 0.63545E+00_JPRB, 0.63704E+00_JPRB, 0.63899E+00_JPRB, 0.64147E+00_JPRB, 0.64440E+00_JPRB, 0.64688E+00_JPRB ,&
&  0.64949E+00_JPRB, 0.65227E+00_JPRB, 0.65523E+00_JPRB, 0.65743E+00_JPRB, 0.65984E+00_JPRB, 0.66214E+00_JPRB, 0.66433E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66633E+00_JPRB, 0.66790E+00_JPRB, 0.66955E+00_JPRB, 0.66912E+00_JPRB, 0.66883E+00_JPRB, 0.66860E+00_JPRB ,&
&  0.66944E+00_JPRB, 0.66790E+00_JPRB, 0.66716E+00_JPRB, 0.66606E+00_JPRB, 0.66554E+00_JPRB, 0.66390E+00_JPRB, 0.66267E+00_JPRB ,&
&  0.66130E+00_JPRB, 0.66023E+00_JPRB, 0.65890E+00_JPRB, 0.65754E+00_JPRB, 0.65628E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.67616E+00_JPRB, 0.67371E+00_JPRB, 0.67108E+00_JPRB, 0.66841E+00_JPRB, 0.66559E+00_JPRB, 0.66342E+00_JPRB, 0.66126E+00_JPRB ,&
&  0.65896E+00_JPRB, 0.65702E+00_JPRB, 0.65521E+00_JPRB, 0.65370E+00_JPRB, 0.65223E+00_JPRB, 0.65161E+00_JPRB, 0.65058E+00_JPRB ,&
&  0.64918E+00_JPRB, 0.64710E+00_JPRB, 0.64537E+00_JPRB, 0.64264E+00_JPRB, 0.63954E+00_JPRB, 0.63534E+00_JPRB, 0.63144E+00_JPRB ,&
&  0.62701E+00_JPRB, 0.62168E+00_JPRB, 0.61686E+00_JPRB, 0.61280E+00_JPRB, 0.60870E+00_JPRB, 0.60391E+00_JPRB, 0.60045E+00_JPRB ,&
&  0.59842E+00_JPRB, 0.59574E+00_JPRB, 0.59329E+00_JPRB, 0.59114E+00_JPRB, 0.58832E+00_JPRB, 0.58621E+00_JPRB, 0.58419E+00_JPRB ,&
&  0.58098E+00_JPRB, 0.57804E+00_JPRB, 0.57835E+00_JPRB, 0.57914E+00_JPRB, 0.58088E+00_JPRB, 0.58437E+00_JPRB, 0.59084E+00_JPRB ,&
&  0.59928E+00_JPRB, 0.60724E+00_JPRB, 0.61572E+00_JPRB, 0.62340E+00_JPRB, 0.62953E+00_JPRB, 0.63448E+00_JPRB, 0.63862E+00_JPRB ,&
&  0.64140E+00_JPRB, 0.64346E+00_JPRB, 0.64447E+00_JPRB, 0.64434E+00_JPRB, 0.64317E+00_JPRB, 0.64263E+00_JPRB, 0.64255E+00_JPRB ,&
&  0.64273E+00_JPRB, 0.64026E+00_JPRB, 0.63936E+00_JPRB, 0.63985E+00_JPRB, 0.64148E+00_JPRB, 0.64324E+00_JPRB, 0.64657E+00_JPRB ,&
&  0.65119E+00_JPRB, 0.65670E+00_JPRB, 0.66220E+00_JPRB, 0.66938E+00_JPRB, 0.67766E+00_JPRB, 0.68686E+00_JPRB, 0.69287E+00_JPRB ,&
&  0.70024E+00_JPRB, 0.70820E+00_JPRB, 0.71652E+00_JPRB, 0.71892E+00_JPRB, 0.72194E+00_JPRB, 0.72466E+00_JPRB, 0.72805E+00_JPRB ,&
&  0.72598E+00_JPRB, 0.72603E+00_JPRB, 0.72645E+00_JPRB, 0.72882E+00_JPRB, 0.72586E+00_JPRB, 0.72481E+00_JPRB, 0.72360E+00_JPRB ,&
&  0.72512E+00_JPRB, 0.71919E+00_JPRB, 0.71571E+00_JPRB, 0.71105E+00_JPRB, 0.70804E+00_JPRB, 0.70139E+00_JPRB, 0.69607E+00_JPRB ,&
&  0.69148E+00_JPRB, 0.68848E+00_JPRB, 0.68476E+00_JPRB, 0.68156E+00_JPRB, 0.67890E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.70046E+00_JPRB, 0.69525E+00_JPRB, 0.68761E+00_JPRB, 0.67810E+00_JPRB, 0.66919E+00_JPRB, 0.66259E+00_JPRB, 0.65561E+00_JPRB ,&
&  0.64992E+00_JPRB, 0.64514E+00_JPRB, 0.64179E+00_JPRB, 0.63934E+00_JPRB, 0.63699E+00_JPRB, 0.63760E+00_JPRB, 0.63818E+00_JPRB ,&
&  0.63899E+00_JPRB, 0.63873E+00_JPRB, 0.64079E+00_JPRB, 0.64040E+00_JPRB, 0.63858E+00_JPRB, 0.63413E+00_JPRB, 0.62818E+00_JPRB ,&
&  0.62083E+00_JPRB, 0.61233E+00_JPRB, 0.60265E+00_JPRB, 0.59160E+00_JPRB, 0.58114E+00_JPRB, 0.54297E+00_JPRB, 0.53294E+00_JPRB ,&
&  0.53225E+00_JPRB, 0.52698E+00_JPRB, 0.52358E+00_JPRB, 0.52231E+00_JPRB, 0.52241E+00_JPRB, 0.51762E+00_JPRB, 0.51217E+00_JPRB ,&
&  0.48347E+00_JPRB, 0.48001E+00_JPRB, 0.47659E+00_JPRB, 0.47480E+00_JPRB, 0.47956E+00_JPRB, 0.49640E+00_JPRB, 0.52148E+00_JPRB ,&
&  0.56631E+00_JPRB, 0.58548E+00_JPRB, 0.60152E+00_JPRB, 0.62017E+00_JPRB, 0.63535E+00_JPRB, 0.64704E+00_JPRB, 0.65849E+00_JPRB ,&
&  0.66817E+00_JPRB, 0.67767E+00_JPRB, 0.68476E+00_JPRB, 0.68860E+00_JPRB, 0.68936E+00_JPRB, 0.68953E+00_JPRB, 0.68919E+00_JPRB ,&
&  0.68860E+00_JPRB, 0.68738E+00_JPRB, 0.68696E+00_JPRB, 0.68620E+00_JPRB, 0.68460E+00_JPRB, 0.68481E+00_JPRB, 0.68913E+00_JPRB ,&
&  0.69626E+00_JPRB, 0.70703E+00_JPRB, 0.71715E+00_JPRB, 0.73197E+00_JPRB, 0.74636E+00_JPRB, 0.76482E+00_JPRB, 0.77317E+00_JPRB ,&
&  0.78346E+00_JPRB, 0.79039E+00_JPRB, 0.79936E+00_JPRB, 0.80018E+00_JPRB, 0.80109E+00_JPRB, 0.80050E+00_JPRB, 0.80223E+00_JPRB ,&
&  0.79865E+00_JPRB, 0.79640E+00_JPRB, 0.79301E+00_JPRB, 0.79319E+00_JPRB, 0.78838E+00_JPRB, 0.78792E+00_JPRB, 0.78528E+00_JPRB ,&
&  0.78764E+00_JPRB, 0.78097E+00_JPRB, 0.77806E+00_JPRB, 0.76949E+00_JPRB, 0.76226E+00_JPRB, 0.74952E+00_JPRB, 0.73825E+00_JPRB ,&
&  0.72855E+00_JPRB, 0.72126E+00_JPRB, 0.71507E+00_JPRB, 0.70981E+00_JPRB, 0.70482E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.71678E+00_JPRB, 0.70269E+00_JPRB, 0.68738E+00_JPRB, 0.67459E+00_JPRB, 0.66269E+00_JPRB, 0.65392E+00_JPRB, 0.64537E+00_JPRB ,&
&  0.63891E+00_JPRB, 0.63277E+00_JPRB, 0.62830E+00_JPRB, 0.62439E+00_JPRB, 0.62156E+00_JPRB, 0.62009E+00_JPRB, 0.61880E+00_JPRB ,&
&  0.61877E+00_JPRB, 0.61855E+00_JPRB, 0.61988E+00_JPRB, 0.62059E+00_JPRB, 0.62171E+00_JPRB, 0.61917E+00_JPRB, 0.61370E+00_JPRB ,&
&  0.60590E+00_JPRB, 0.59665E+00_JPRB, 0.56511E+00_JPRB, 0.55140E+00_JPRB, 0.52478E+00_JPRB, 0.50479E+00_JPRB, 0.49391E+00_JPRB ,&
&  0.47735E+00_JPRB, 0.46834E+00_JPRB, 0.45135E+00_JPRB, 0.43939E+00_JPRB, 0.43408E+00_JPRB, 0.42094E+00_JPRB, 0.41411E+00_JPRB ,&
&  0.40456E+00_JPRB, 0.39018E+00_JPRB, 0.38426E+00_JPRB, 0.38253E+00_JPRB, 0.38777E+00_JPRB, 0.40292E+00_JPRB, 0.42870E+00_JPRB ,&
&  0.47678E+00_JPRB, 0.53031E+00_JPRB, 0.60366E+00_JPRB, 0.62534E+00_JPRB, 0.62889E+00_JPRB, 0.64617E+00_JPRB, 0.66036E+00_JPRB ,&
&  0.66928E+00_JPRB, 0.68935E+00_JPRB, 0.71035E+00_JPRB, 0.72016E+00_JPRB, 0.72669E+00_JPRB, 0.72964E+00_JPRB, 0.73227E+00_JPRB ,&
&  0.73466E+00_JPRB, 0.74265E+00_JPRB, 0.74691E+00_JPRB, 0.74978E+00_JPRB, 0.75238E+00_JPRB, 0.75533E+00_JPRB, 0.76425E+00_JPRB ,&
&  0.77205E+00_JPRB, 0.79181E+00_JPRB, 0.80958E+00_JPRB, 0.83903E+00_JPRB, 0.85282E+00_JPRB, 0.86979E+00_JPRB, 0.87534E+00_JPRB ,&
&  0.88349E+00_JPRB, 0.88850E+00_JPRB, 0.89238E+00_JPRB, 0.88528E+00_JPRB, 0.88467E+00_JPRB, 0.89131E+00_JPRB, 0.89029E+00_JPRB ,&
&  0.88770E+00_JPRB, 0.88568E+00_JPRB, 0.88263E+00_JPRB, 0.87998E+00_JPRB, 0.87802E+00_JPRB, 0.87971E+00_JPRB, 0.87907E+00_JPRB ,&
&  0.88432E+00_JPRB, 0.88007E+00_JPRB, 0.87608E+00_JPRB, 0.86607E+00_JPRB, 0.85328E+00_JPRB, 0.83488E+00_JPRB, 0.81105E+00_JPRB ,&
&  0.79017E+00_JPRB, 0.77390E+00_JPRB, 0.75687E+00_JPRB, 0.74372E+00_JPRB, 0.72996E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.67892E+00_JPRB, 0.66118E+00_JPRB, 0.64602E+00_JPRB, 0.63485E+00_JPRB, 0.62501E+00_JPRB, 0.61845E+00_JPRB, 0.61240E+00_JPRB ,&
&  0.60914E+00_JPRB, 0.60558E+00_JPRB, 0.60421E+00_JPRB, 0.60255E+00_JPRB, 0.60255E+00_JPRB, 0.60343E+00_JPRB, 0.60533E+00_JPRB ,&
&  0.60797E+00_JPRB, 0.60960E+00_JPRB, 0.61170E+00_JPRB, 0.61037E+00_JPRB, 0.60957E+00_JPRB, 0.60684E+00_JPRB, 0.60421E+00_JPRB ,&
&  0.59566E+00_JPRB, 0.58519E+00_JPRB, 0.56262E+00_JPRB, 0.54769E+00_JPRB, 0.52236E+00_JPRB, 0.50457E+00_JPRB, 0.48914E+00_JPRB ,&
&  0.48169E+00_JPRB, 0.45366E+00_JPRB, 0.45879E+00_JPRB, 0.43915E+00_JPRB, 0.42058E+00_JPRB, 0.40254E+00_JPRB, 0.38276E+00_JPRB ,&
&  0.37289E+00_JPRB, 0.35332E+00_JPRB, 0.34542E+00_JPRB, 0.34518E+00_JPRB, 0.34803E+00_JPRB, 0.36268E+00_JPRB, 0.38537E+00_JPRB ,&
&  0.43271E+00_JPRB, 0.54953E+00_JPRB, 0.59055E+00_JPRB, 0.60896E+00_JPRB, 0.60445E+00_JPRB, 0.60665E+00_JPRB, 0.60113E+00_JPRB ,&
&  0.62294E+00_JPRB, 0.65231E+00_JPRB, 0.68222E+00_JPRB, 0.70307E+00_JPRB, 0.72796E+00_JPRB, 0.74322E+00_JPRB, 0.75701E+00_JPRB ,&
&  0.77258E+00_JPRB, 0.79510E+00_JPRB, 0.81442E+00_JPRB, 0.83130E+00_JPRB, 0.85065E+00_JPRB, 0.86953E+00_JPRB, 0.89247E+00_JPRB ,&
&  0.90673E+00_JPRB, 0.94112E+00_JPRB, 0.95421E+00_JPRB, 0.97268E+00_JPRB, 0.97721E+00_JPRB, 0.97683E+00_JPRB, 0.97982E+00_JPRB ,&
&  0.98602E+00_JPRB, 0.99383E+00_JPRB, 0.10008E+01_JPRB, 0.10006E+01_JPRB, 0.10010E+01_JPRB, 0.98776E+00_JPRB, 0.98299E+00_JPRB ,&
&  0.98353E+00_JPRB, 0.98201E+00_JPRB, 0.97867E+00_JPRB, 0.97105E+00_JPRB, 0.96092E+00_JPRB, 0.94987E+00_JPRB, 0.94354E+00_JPRB ,&
&  0.93595E+00_JPRB, 0.92269E+00_JPRB, 0.90854E+00_JPRB, 0.89289E+00_JPRB, 0.87691E+00_JPRB, 0.85987E+00_JPRB, 0.83221E+00_JPRB ,&
&  0.79856E+00_JPRB, 0.76912E+00_JPRB, 0.74169E+00_JPRB, 0.72053E+00_JPRB, 0.69843E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.64852E+00_JPRB, 0.63059E+00_JPRB, 0.61477E+00_JPRB, 0.60514E+00_JPRB, 0.59613E+00_JPRB, 0.59175E+00_JPRB, 0.58674E+00_JPRB ,&
&  0.58415E+00_JPRB, 0.58200E+00_JPRB, 0.58126E+00_JPRB, 0.58102E+00_JPRB, 0.58190E+00_JPRB, 0.58335E+00_JPRB, 0.58601E+00_JPRB ,&
&  0.58938E+00_JPRB, 0.59405E+00_JPRB, 0.59949E+00_JPRB, 0.60125E+00_JPRB, 0.60273E+00_JPRB, 0.60219E+00_JPRB, 0.60088E+00_JPRB ,&
&  0.59430E+00_JPRB, 0.58545E+00_JPRB, 0.56469E+00_JPRB, 0.54826E+00_JPRB, 0.53341E+00_JPRB, 0.51491E+00_JPRB, 0.51137E+00_JPRB ,&
&  0.50743E+00_JPRB, 0.49599E+00_JPRB, 0.47909E+00_JPRB, 0.47025E+00_JPRB, 0.45672E+00_JPRB, 0.40696E+00_JPRB, 0.40237E+00_JPRB ,&
&  0.38436E+00_JPRB, 0.35464E+00_JPRB, 0.34502E+00_JPRB, 0.33880E+00_JPRB, 0.34045E+00_JPRB, 0.35264E+00_JPRB, 0.37242E+00_JPRB ,&
&  0.41695E+00_JPRB, 0.55012E+00_JPRB, 0.58285E+00_JPRB, 0.59352E+00_JPRB, 0.57854E+00_JPRB, 0.58277E+00_JPRB, 0.58400E+00_JPRB ,&
&  0.59947E+00_JPRB, 0.61494E+00_JPRB, 0.63402E+00_JPRB, 0.65105E+00_JPRB, 0.66859E+00_JPRB, 0.67928E+00_JPRB, 0.69936E+00_JPRB ,&
&  0.73503E+00_JPRB, 0.77900E+00_JPRB, 0.81833E+00_JPRB, 0.86273E+00_JPRB, 0.91401E+00_JPRB, 0.10056E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.11078E+01_JPRB, 0.10951E+01_JPRB, 0.10922E+01_JPRB, 0.10851E+01_JPRB, 0.10843E+01_JPRB, 0.10832E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.11201E+01_JPRB, 0.11019E+01_JPRB, 0.10661E+01_JPRB, 0.10514E+01_JPRB, 0.10253E+01_JPRB, 0.10235E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.10364E+01_JPRB, 0.10292E+01_JPRB, 0.10134E+01_JPRB, 0.99851E+00_JPRB, 0.98215E+00_JPRB, 0.97207E+00_JPRB ,&
&  0.96033E+00_JPRB, 0.94111E+00_JPRB, 0.91999E+00_JPRB, 0.90222E+00_JPRB, 0.88567E+00_JPRB, 0.87140E+00_JPRB, 0.85010E+00_JPRB ,&
&  0.81257E+00_JPRB, 0.76587E+00_JPRB, 0.72974E+00_JPRB, 0.69662E+00_JPRB, 0.66952E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.62737E+00_JPRB, 0.61393E+00_JPRB, 0.60512E+00_JPRB, 0.60189E+00_JPRB, 0.59856E+00_JPRB, 0.59701E+00_JPRB, 0.59207E+00_JPRB ,&
&  0.58716E+00_JPRB, 0.58233E+00_JPRB, 0.57808E+00_JPRB, 0.57454E+00_JPRB, 0.57251E+00_JPRB, 0.57095E+00_JPRB, 0.57056E+00_JPRB ,&
&  0.57094E+00_JPRB, 0.56554E+00_JPRB, 0.56542E+00_JPRB, 0.57078E+00_JPRB, 0.57806E+00_JPRB, 0.58425E+00_JPRB, 0.59165E+00_JPRB ,&
&  0.58998E+00_JPRB, 0.58188E+00_JPRB, 0.57100E+00_JPRB, 0.55595E+00_JPRB, 0.54327E+00_JPRB, 0.53683E+00_JPRB, 0.52832E+00_JPRB ,&
&  0.51478E+00_JPRB, 0.50163E+00_JPRB, 0.50767E+00_JPRB, 0.50401E+00_JPRB, 0.47808E+00_JPRB, 0.46429E+00_JPRB, 0.42953E+00_JPRB ,&
&  0.39567E+00_JPRB, 0.35900E+00_JPRB, 0.34945E+00_JPRB, 0.34513E+00_JPRB, 0.35182E+00_JPRB, 0.36450E+00_JPRB, 0.39354E+00_JPRB ,&
&  0.44711E+00_JPRB, 0.55221E+00_JPRB, 0.57781E+00_JPRB, 0.59009E+00_JPRB, 0.59576E+00_JPRB, 0.59422E+00_JPRB, 0.59390E+00_JPRB ,&
&  0.60462E+00_JPRB, 0.61568E+00_JPRB, 0.63341E+00_JPRB, 0.65297E+00_JPRB, 0.67701E+00_JPRB, 0.69763E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.75999E+00_JPRB, 0.79111E+00_JPRB, 0.82304E+00_JPRB, 0.90562E+00_JPRB, 0.10519E+01_JPRB, 0.11986E+01_JPRB, 0.12513E+01_JPRB ,&
&  0.12360E+01_JPRB, 0.12212E+01_JPRB, 0.11751E+01_JPRB, 0.11454E+01_JPRB, 0.11325E+01_JPRB, 0.11301E+01_JPRB, 0.11362E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.11765E+01_JPRB, 0.12504E+01_JPRB, 0.11953E+01_JPRB, 0.11624E+01_JPRB, 0.11282E+01_JPRB, 0.10873E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10359E+01_JPRB, 0.10235E+01_JPRB, 0.10071E+01_JPRB, 0.99343E+00_JPRB, 0.98310E+00_JPRB, 0.98458E+00_JPRB ,&
&  0.97813E+00_JPRB, 0.96325E+00_JPRB, 0.93386E+00_JPRB, 0.91618E+00_JPRB, 0.89849E+00_JPRB, 0.87965E+00_JPRB, 0.85911E+00_JPRB ,&
&  0.81848E+00_JPRB, 0.76467E+00_JPRB, 0.72146E+00_JPRB, 0.68320E+00_JPRB, 0.65120E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.62861E+00_JPRB, 0.62032E+00_JPRB, 0.62001E+00_JPRB, 0.62555E+00_JPRB, 0.62905E+00_JPRB, 0.61908E+00_JPRB, 0.60744E+00_JPRB ,&
&  0.59900E+00_JPRB, 0.59610E+00_JPRB, 0.59314E+00_JPRB, 0.58746E+00_JPRB, 0.58305E+00_JPRB, 0.57896E+00_JPRB, 0.57517E+00_JPRB ,&
&  0.57157E+00_JPRB, 0.56812E+00_JPRB, 0.56470E+00_JPRB, 0.55904E+00_JPRB, 0.55739E+00_JPRB, 0.56258E+00_JPRB, 0.57304E+00_JPRB ,&
&  0.58327E+00_JPRB, 0.58492E+00_JPRB, 0.58190E+00_JPRB, 0.57125E+00_JPRB, 0.55705E+00_JPRB, 0.54526E+00_JPRB, 0.54036E+00_JPRB ,&
&  0.52920E+00_JPRB, 0.51623E+00_JPRB, 0.50710E+00_JPRB, 0.51613E+00_JPRB, 0.52210E+00_JPRB, 0.49549E+00_JPRB, 0.45199E+00_JPRB ,&
&  0.41353E+00_JPRB, 0.38308E+00_JPRB, 0.36859E+00_JPRB, 0.36604E+00_JPRB, 0.37785E+00_JPRB, 0.40175E+00_JPRB, 0.43616E+00_JPRB ,&
&  0.49036E+00_JPRB, 0.53908E+00_JPRB, 0.56062E+00_JPRB, 0.57873E+00_JPRB, 0.58079E+00_JPRB, 0.57746E+00_JPRB, 0.57473E+00_JPRB ,&
&  0.57898E+00_JPRB, 0.58850E+00_JPRB, 0.60332E+00_JPRB, 0.61798E+00_JPRB, 0.64140E+00_JPRB, 0.68211E+00_JPRB, 0.76375E+00_JPRB ,&
&  0.95564E+00_JPRB, 0.98417E+00_JPRB, 0.91482E+00_JPRB, 0.10135E+01_JPRB, 0.11676E+01_JPRB, 0.12932E+01_JPRB, 0.13740E+01_JPRB ,&
&  0.13690E+01_JPRB, 0.12956E+01_JPRB, 0.12371E+01_JPRB, 0.12028E+01_JPRB, 0.11813E+01_JPRB, 0.11680E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.12072E+01_JPRB, 0.13009E+01_JPRB, 0.11472E+01_JPRB, 0.10743E+01_JPRB, 0.10539E+01_JPRB, 0.10263E+01_JPRB ,&
&  0.99885E+00_JPRB, 0.98348E+00_JPRB, 0.98711E+00_JPRB, 0.10009E+01_JPRB, 0.96252E+00_JPRB, 0.92782E+00_JPRB, 0.90408E+00_JPRB ,&
&  0.90341E+00_JPRB, 0.89548E+00_JPRB, 0.88999E+00_JPRB, 0.89740E+00_JPRB, 0.89296E+00_JPRB, 0.86220E+00_JPRB, 0.84146E+00_JPRB ,&
&  0.81115E+00_JPRB, 0.76117E+00_JPRB, 0.72516E+00_JPRB, 0.69040E+00_JPRB, 0.65341E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.62059E+00_JPRB, 0.63366E+00_JPRB, 0.65246E+00_JPRB, 0.66290E+00_JPRB, 0.66426E+00_JPRB, 0.63279E+00_JPRB, 0.60083E+00_JPRB ,&
&  0.58200E+00_JPRB, 0.57544E+00_JPRB, 0.57681E+00_JPRB, 0.57765E+00_JPRB, 0.57722E+00_JPRB, 0.57735E+00_JPRB, 0.57617E+00_JPRB ,&
&  0.56998E+00_JPRB, 0.56173E+00_JPRB, 0.55471E+00_JPRB, 0.54927E+00_JPRB, 0.54656E+00_JPRB, 0.54888E+00_JPRB, 0.55578E+00_JPRB ,&
&  0.56611E+00_JPRB, 0.57330E+00_JPRB, 0.57593E+00_JPRB, 0.57168E+00_JPRB, 0.56757E+00_JPRB, 0.56372E+00_JPRB, 0.55198E+00_JPRB ,&
&  0.53268E+00_JPRB, 0.52944E+00_JPRB, 0.52816E+00_JPRB, 0.52999E+00_JPRB, 0.54017E+00_JPRB, 0.52447E+00_JPRB, 0.47885E+00_JPRB ,&
&  0.43596E+00_JPRB, 0.40776E+00_JPRB, 0.39915E+00_JPRB, 0.40849E+00_JPRB, 0.43565E+00_JPRB, 0.47350E+00_JPRB, 0.49267E+00_JPRB ,&
&  0.51069E+00_JPRB, 0.52680E+00_JPRB, 0.53869E+00_JPRB, 0.54940E+00_JPRB, 0.55725E+00_JPRB, 0.56080E+00_JPRB, 0.56158E+00_JPRB ,&
&  0.56108E+00_JPRB, 0.56310E+00_JPRB, 0.56891E+00_JPRB, 0.56103E+00_JPRB, 0.58479E+00_JPRB, 0.70084E+00_JPRB, 0.82508E+00_JPRB ,&
&  0.95128E+00_JPRB, 0.99285E+00_JPRB, 0.10435E+01_JPRB, 0.11637E+01_JPRB, 0.12594E+01_JPRB, 0.13292E+01_JPRB, 0.13733E+01_JPRB ,&
&  0.13742E+01_JPRB, 0.12992E+01_JPRB, 0.12353E+01_JPRB, 0.12157E+01_JPRB, 0.12100E+01_JPRB, 0.12059E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.10875E+01_JPRB, 0.10137E+01_JPRB, 0.94757E+00_JPRB, 0.93704E+00_JPRB, 0.96300E+00_JPRB, 0.10049E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.10076E+01_JPRB, 0.10067E+01_JPRB, 0.10249E+01_JPRB, 0.93521E+00_JPRB, 0.87713E+00_JPRB, 0.83916E+00_JPRB ,&
&  0.83301E+00_JPRB, 0.83326E+00_JPRB, 0.83134E+00_JPRB, 0.85459E+00_JPRB, 0.85770E+00_JPRB, 0.84381E+00_JPRB, 0.80213E+00_JPRB ,&
&  0.74584E+00_JPRB, 0.69965E+00_JPRB, 0.66556E+00_JPRB, 0.63908E+00_JPRB, 0.62629E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.69364E+00_JPRB, 0.70077E+00_JPRB, 0.70180E+00_JPRB, 0.70182E+00_JPRB, 0.69020E+00_JPRB, 0.67266E+00_JPRB, 0.65475E+00_JPRB ,&
&  0.64736E+00_JPRB, 0.63263E+00_JPRB, 0.61661E+00_JPRB, 0.59576E+00_JPRB, 0.57816E+00_JPRB, 0.56311E+00_JPRB, 0.56023E+00_JPRB ,&
&  0.56356E+00_JPRB, 0.56680E+00_JPRB, 0.56346E+00_JPRB, 0.55450E+00_JPRB, 0.54700E+00_JPRB, 0.54492E+00_JPRB, 0.54913E+00_JPRB ,&
&  0.55725E+00_JPRB, 0.56727E+00_JPRB, 0.57368E+00_JPRB, 0.57564E+00_JPRB, 0.57568E+00_JPRB, 0.57417E+00_JPRB, 0.56857E+00_JPRB ,&
&  0.55492E+00_JPRB, 0.54793E+00_JPRB, 0.54801E+00_JPRB, 0.55273E+00_JPRB, 0.56531E+00_JPRB, 0.55061E+00_JPRB, 0.50689E+00_JPRB ,&
&  0.45950E+00_JPRB, 0.44396E+00_JPRB, 0.45392E+00_JPRB, 0.49354E+00_JPRB, 0.51656E+00_JPRB, 0.53529E+00_JPRB, 0.54820E+00_JPRB ,&
&  0.54523E+00_JPRB, 0.53903E+00_JPRB, 0.54497E+00_JPRB, 0.56392E+00_JPRB, 0.56685E+00_JPRB, 0.56789E+00_JPRB, 0.56561E+00_JPRB ,&
&  0.56254E+00_JPRB, 0.56480E+00_JPRB, 0.56999E+00_JPRB, 0.56625E+00_JPRB, 0.66803E+00_JPRB, 0.85858E+00_JPRB, 0.93561E+00_JPRB ,&
&  0.10457E+01_JPRB, 0.11303E+01_JPRB, 0.12530E+01_JPRB, 0.12843E+01_JPRB, 0.12689E+01_JPRB, 0.12935E+01_JPRB, 0.13665E+01_JPRB ,&
&  0.13645E+01_JPRB, 0.13331E+01_JPRB, 0.12968E+01_JPRB, 0.12889E+01_JPRB, 0.12445E+01_JPRB, 0.11939E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.11341E+01_JPRB, 0.11116E+01_JPRB, 0.10593E+01_JPRB, 0.99589E+00_JPRB, 0.97306E+00_JPRB, 0.96903E+00_JPRB, 0.98752E+00_JPRB ,&
&  0.99475E+00_JPRB, 0.10109E+01_JPRB, 0.10211E+01_JPRB, 0.10361E+01_JPRB, 0.95864E+00_JPRB, 0.86445E+00_JPRB, 0.80957E+00_JPRB ,&
&  0.79311E+00_JPRB, 0.78824E+00_JPRB, 0.77286E+00_JPRB, 0.78939E+00_JPRB, 0.80676E+00_JPRB, 0.79751E+00_JPRB, 0.74451E+00_JPRB ,&
&  0.69858E+00_JPRB, 0.67342E+00_JPRB, 0.66085E+00_JPRB, 0.67357E+00_JPRB, 0.68723E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.74282E+00_JPRB, 0.75171E+00_JPRB, 0.74491E+00_JPRB, 0.73541E+00_JPRB, 0.73119E+00_JPRB, 0.71610E+00_JPRB, 0.68633E+00_JPRB ,&
&  0.66942E+00_JPRB, 0.63750E+00_JPRB, 0.61792E+00_JPRB, 0.60467E+00_JPRB, 0.59507E+00_JPRB, 0.58064E+00_JPRB, 0.55802E+00_JPRB ,&
&  0.54990E+00_JPRB, 0.55447E+00_JPRB, 0.56510E+00_JPRB, 0.56580E+00_JPRB, 0.56169E+00_JPRB, 0.55681E+00_JPRB, 0.55585E+00_JPRB ,&
&  0.56013E+00_JPRB, 0.56919E+00_JPRB, 0.58045E+00_JPRB, 0.58330E+00_JPRB, 0.58027E+00_JPRB, 0.57765E+00_JPRB, 0.57950E+00_JPRB ,&
&  0.57347E+00_JPRB, 0.56640E+00_JPRB, 0.57157E+00_JPRB, 0.57489E+00_JPRB, 0.58979E+00_JPRB, 0.59847E+00_JPRB, 0.57701E+00_JPRB ,&
&  0.51351E+00_JPRB, 0.51537E+00_JPRB, 0.56284E+00_JPRB, 0.58381E+00_JPRB, 0.60075E+00_JPRB, 0.60085E+00_JPRB, 0.60022E+00_JPRB ,&
&  0.59913E+00_JPRB, 0.59982E+00_JPRB, 0.59815E+00_JPRB, 0.59819E+00_JPRB, 0.59712E+00_JPRB, 0.59569E+00_JPRB, 0.59333E+00_JPRB ,&
&  0.59633E+00_JPRB, 0.56404E+00_JPRB, 0.56588E+00_JPRB, 0.60268E+00_JPRB, 0.78890E+00_JPRB, 0.89014E+00_JPRB, 0.97206E+00_JPRB ,&
&  0.10922E+01_JPRB, 0.11792E+01_JPRB, 0.12867E+01_JPRB, 0.13536E+01_JPRB, 0.13639E+01_JPRB, 0.13632E+01_JPRB, 0.14212E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.14388E+01_JPRB, 0.14164E+01_JPRB, 0.13691E+01_JPRB, 0.12814E+01_JPRB, 0.12451E+01_JPRB, 0.12243E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.12365E+01_JPRB, 0.11828E+01_JPRB, 0.11140E+01_JPRB, 0.10646E+01_JPRB, 0.10670E+01_JPRB, 0.10775E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.10964E+01_JPRB, 0.10875E+01_JPRB, 0.10658E+01_JPRB, 0.10077E+01_JPRB, 0.95817E+00_JPRB, 0.94363E+00_JPRB ,&
&  0.91785E+00_JPRB, 0.86981E+00_JPRB, 0.79603E+00_JPRB, 0.78001E+00_JPRB, 0.78736E+00_JPRB, 0.77137E+00_JPRB, 0.73606E+00_JPRB ,&
&  0.71955E+00_JPRB, 0.70623E+00_JPRB, 0.69726E+00_JPRB, 0.71326E+00_JPRB, 0.72484E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.75179E+00_JPRB, 0.75776E+00_JPRB, 0.74985E+00_JPRB, 0.73917E+00_JPRB, 0.74723E+00_JPRB, 0.73494E+00_JPRB, 0.73133E+00_JPRB ,&
&  0.73953E+00_JPRB, 0.72528E+00_JPRB, 0.74363E+00_JPRB, 0.66528E+00_JPRB, 0.61076E+00_JPRB, 0.59156E+00_JPRB, 0.56942E+00_JPRB ,&
&  0.56266E+00_JPRB, 0.57038E+00_JPRB, 0.57555E+00_JPRB, 0.57949E+00_JPRB, 0.58165E+00_JPRB, 0.57118E+00_JPRB, 0.56672E+00_JPRB ,&
&  0.56946E+00_JPRB, 0.57881E+00_JPRB, 0.59218E+00_JPRB, 0.59460E+00_JPRB, 0.59414E+00_JPRB, 0.59312E+00_JPRB, 0.59188E+00_JPRB ,&
&  0.58368E+00_JPRB, 0.59210E+00_JPRB, 0.60988E+00_JPRB, 0.60441E+00_JPRB, 0.61207E+00_JPRB, 0.63487E+00_JPRB, 0.63585E+00_JPRB ,&
&  0.64795E+00_JPRB, 0.62082E+00_JPRB, 0.65337E+00_JPRB, 0.66844E+00_JPRB, 0.67099E+00_JPRB, 0.66032E+00_JPRB, 0.64961E+00_JPRB ,&
&  0.64625E+00_JPRB, 0.64918E+00_JPRB, 0.65455E+00_JPRB, 0.66011E+00_JPRB, 0.65876E+00_JPRB, 0.66635E+00_JPRB, 0.67518E+00_JPRB ,&
&  0.69005E+00_JPRB, 0.63934E+00_JPRB, 0.72657E+00_JPRB, 0.77520E+00_JPRB, 0.90859E+00_JPRB, 0.10654E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.13768E+01_JPRB, 0.13654E+01_JPRB, 0.14306E+01_JPRB, 0.14431E+01_JPRB, 0.14521E+01_JPRB, 0.14889E+01_JPRB, 0.15504E+01_JPRB ,&
&  0.16504E+01_JPRB, 0.15667E+01_JPRB, 0.15972E+01_JPRB, 0.14839E+01_JPRB, 0.14031E+01_JPRB, 0.13940E+01_JPRB, 0.13820E+01_JPRB ,&
&  0.13925E+01_JPRB, 0.13972E+01_JPRB, 0.13758E+01_JPRB, 0.12328E+01_JPRB, 0.12030E+01_JPRB, 0.11954E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.11437E+01_JPRB, 0.10899E+01_JPRB, 0.11457E+01_JPRB, 0.10468E+01_JPRB, 0.98982E+00_JPRB, 0.99537E+00_JPRB ,&
&  0.10547E+01_JPRB, 0.95057E+00_JPRB, 0.87996E+00_JPRB, 0.83817E+00_JPRB, 0.80156E+00_JPRB, 0.76836E+00_JPRB, 0.77035E+00_JPRB ,&
&  0.73734E+00_JPRB, 0.73841E+00_JPRB, 0.76242E+00_JPRB, 0.75439E+00_JPRB, 0.74522E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.76263E+00_JPRB, 0.75434E+00_JPRB, 0.74449E+00_JPRB, 0.73877E+00_JPRB, 0.76484E+00_JPRB, 0.77644E+00_JPRB, 0.77698E+00_JPRB ,&
&  0.79471E+00_JPRB, 0.78680E+00_JPRB, 0.78155E+00_JPRB, 0.76253E+00_JPRB, 0.74020E+00_JPRB, 0.68359E+00_JPRB, 0.59628E+00_JPRB ,&
&  0.56862E+00_JPRB, 0.57720E+00_JPRB, 0.59831E+00_JPRB, 0.60760E+00_JPRB, 0.59888E+00_JPRB, 0.58532E+00_JPRB, 0.57907E+00_JPRB ,&
&  0.58462E+00_JPRB, 0.59113E+00_JPRB, 0.60338E+00_JPRB, 0.61141E+00_JPRB, 0.61332E+00_JPRB, 0.61587E+00_JPRB, 0.62020E+00_JPRB ,&
&  0.61443E+00_JPRB, 0.63092E+00_JPRB, 0.65894E+00_JPRB, 0.63123E+00_JPRB, 0.66928E+00_JPRB, 0.71285E+00_JPRB, 0.72459E+00_JPRB ,&
&  0.75563E+00_JPRB, 0.76489E+00_JPRB, 0.75452E+00_JPRB, 0.75185E+00_JPRB, 0.73180E+00_JPRB, 0.71354E+00_JPRB, 0.70773E+00_JPRB ,&
&  0.71114E+00_JPRB, 0.71630E+00_JPRB, 0.72277E+00_JPRB, 0.72840E+00_JPRB, 0.73269E+00_JPRB, 0.77365E+00_JPRB, 0.83624E+00_JPRB ,&
&  0.87184E+00_JPRB, 0.93042E+00_JPRB, 0.10390E+01_JPRB, 0.10510E+01_JPRB, 0.11480E+01_JPRB, 0.12511E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.13959E+01_JPRB, 0.15537E+01_JPRB, 0.16709E+01_JPRB, 0.17765E+01_JPRB, 0.18269E+01_JPRB, 0.18770E+01_JPRB ,&
&  0.20089E+01_JPRB, 0.20783E+01_JPRB, 0.18652E+01_JPRB, 0.17423E+01_JPRB, 0.16369E+01_JPRB, 0.16248E+01_JPRB, 0.15614E+01_JPRB ,&
&  0.15684E+01_JPRB, 0.16078E+01_JPRB, 0.15932E+01_JPRB, 0.15560E+01_JPRB, 0.14717E+01_JPRB, 0.13711E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.11041E+01_JPRB, 0.10463E+01_JPRB, 0.10728E+01_JPRB, 0.10792E+01_JPRB, 0.97547E+00_JPRB, 0.10216E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.10627E+01_JPRB, 0.97725E+00_JPRB, 0.89388E+00_JPRB, 0.85668E+00_JPRB, 0.82613E+00_JPRB, 0.77083E+00_JPRB ,&
&  0.72750E+00_JPRB, 0.76840E+00_JPRB, 0.78426E+00_JPRB, 0.77516E+00_JPRB, 0.76593E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.77680E+00_JPRB, 0.75827E+00_JPRB, 0.73975E+00_JPRB, 0.74336E+00_JPRB, 0.77485E+00_JPRB, 0.78965E+00_JPRB, 0.81069E+00_JPRB ,&
&  0.82184E+00_JPRB, 0.82449E+00_JPRB, 0.81480E+00_JPRB, 0.78080E+00_JPRB, 0.76533E+00_JPRB, 0.76544E+00_JPRB, 0.72888E+00_JPRB ,&
&  0.66189E+00_JPRB, 0.65449E+00_JPRB, 0.63864E+00_JPRB, 0.64177E+00_JPRB, 0.64330E+00_JPRB, 0.62295E+00_JPRB, 0.61255E+00_JPRB ,&
&  0.62285E+00_JPRB, 0.61768E+00_JPRB, 0.61824E+00_JPRB, 0.62277E+00_JPRB, 0.62977E+00_JPRB, 0.64192E+00_JPRB, 0.65367E+00_JPRB ,&
&  0.65229E+00_JPRB, 0.68839E+00_JPRB, 0.72771E+00_JPRB, 0.71267E+00_JPRB, 0.77813E+00_JPRB, 0.85363E+00_JPRB, 0.88945E+00_JPRB ,&
&  0.90619E+00_JPRB, 0.88423E+00_JPRB, 0.84883E+00_JPRB, 0.81917E+00_JPRB, 0.78898E+00_JPRB, 0.77922E+00_JPRB, 0.78291E+00_JPRB ,&
&  0.79153E+00_JPRB, 0.79630E+00_JPRB, 0.79967E+00_JPRB, 0.81159E+00_JPRB, 0.81782E+00_JPRB, 0.91847E+00_JPRB, 0.10736E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.13319E+01_JPRB, 0.14340E+01_JPRB, 0.15067E+01_JPRB, 0.15927E+01_JPRB, 0.16140E+01_JPRB, 0.15521E+01_JPRB ,&
&  0.16046E+01_JPRB, 0.15680E+01_JPRB, 0.19874E+01_JPRB, 0.20574E+01_JPRB, 0.20615E+01_JPRB, 0.21516E+01_JPRB, 0.21239E+01_JPRB ,&
&  0.21250E+01_JPRB, 0.20085E+01_JPRB, 0.19330E+01_JPRB, 0.18635E+01_JPRB, 0.18075E+01_JPRB, 0.18615E+01_JPRB, 0.17781E+01_JPRB ,&
&  0.17962E+01_JPRB, 0.17953E+01_JPRB, 0.15915E+01_JPRB, 0.14238E+01_JPRB, 0.12339E+01_JPRB, 0.12928E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.10094E+01_JPRB, 0.10671E+01_JPRB, 0.11257E+01_JPRB, 0.11544E+01_JPRB, 0.12126E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.13089E+01_JPRB, 0.11537E+01_JPRB, 0.10273E+01_JPRB, 0.95700E+00_JPRB, 0.90461E+00_JPRB, 0.80377E+00_JPRB ,&
&  0.78075E+00_JPRB, 0.81492E+00_JPRB, 0.82987E+00_JPRB, 0.82051E+00_JPRB, 0.79715E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.80974E+00_JPRB, 0.78745E+00_JPRB, 0.75942E+00_JPRB, 0.76724E+00_JPRB, 0.79174E+00_JPRB, 0.80075E+00_JPRB, 0.82461E+00_JPRB ,&
&  0.85061E+00_JPRB, 0.85219E+00_JPRB, 0.81573E+00_JPRB, 0.78691E+00_JPRB, 0.77540E+00_JPRB, 0.78704E+00_JPRB, 0.81910E+00_JPRB ,&
&  0.74861E+00_JPRB, 0.72513E+00_JPRB, 0.66858E+00_JPRB, 0.67614E+00_JPRB, 0.74646E+00_JPRB, 0.70162E+00_JPRB, 0.69543E+00_JPRB ,&
&  0.67854E+00_JPRB, 0.67426E+00_JPRB, 0.65785E+00_JPRB, 0.64751E+00_JPRB, 0.64726E+00_JPRB, 0.66430E+00_JPRB, 0.69987E+00_JPRB ,&
&  0.72658E+00_JPRB, 0.81169E+00_JPRB, 0.89614E+00_JPRB, 0.89079E+00_JPRB, 0.96241E+00_JPRB, 0.10187E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10456E+01_JPRB, 0.97559E+00_JPRB, 0.90904E+00_JPRB, 0.87637E+00_JPRB, 0.86101E+00_JPRB, 0.86575E+00_JPRB, 0.87586E+00_JPRB ,&
&  0.88972E+00_JPRB, 0.90520E+00_JPRB, 0.91502E+00_JPRB, 0.93428E+00_JPRB, 0.94923E+00_JPRB, 0.10813E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.18364E+01_JPRB, 0.20020E+01_JPRB, 0.23096E+01_JPRB, 0.23934E+01_JPRB, 0.23067E+01_JPRB, 0.20816E+01_JPRB ,&
&  0.19409E+01_JPRB, 0.18211E+01_JPRB, 0.19600E+01_JPRB, 0.21099E+01_JPRB, 0.20930E+01_JPRB, 0.22711E+01_JPRB, 0.22153E+01_JPRB ,&
&  0.20216E+01_JPRB, 0.18855E+01_JPRB, 0.17928E+01_JPRB, 0.18087E+01_JPRB, 0.17645E+01_JPRB, 0.18413E+01_JPRB, 0.18265E+01_JPRB ,&
&  0.18373E+01_JPRB, 0.16270E+01_JPRB, 0.12563E+01_JPRB, 0.11727E+01_JPRB, 0.94284E+00_JPRB, 0.95995E+00_JPRB, 0.11745E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.11786E+01_JPRB, 0.11964E+01_JPRB, 0.12791E+01_JPRB, 0.13208E+01_JPRB, 0.15973E+01_JPRB, 0.17181E+01_JPRB ,&
&  0.15306E+01_JPRB, 0.15215E+01_JPRB, 0.14520E+01_JPRB, 0.13398E+01_JPRB, 0.12248E+01_JPRB, 0.10729E+01_JPRB, 0.95751E+00_JPRB ,&
&  0.92341E+00_JPRB, 0.89973E+00_JPRB, 0.89505E+00_JPRB, 0.86746E+00_JPRB, 0.82968E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.86604E+00_JPRB, 0.84565E+00_JPRB, 0.80389E+00_JPRB, 0.80483E+00_JPRB, 0.81148E+00_JPRB, 0.82097E+00_JPRB, 0.84956E+00_JPRB ,&
&  0.87746E+00_JPRB, 0.85636E+00_JPRB, 0.81090E+00_JPRB, 0.79356E+00_JPRB, 0.79506E+00_JPRB, 0.80829E+00_JPRB, 0.84722E+00_JPRB ,&
&  0.91130E+00_JPRB, 0.89696E+00_JPRB, 0.79132E+00_JPRB, 0.69640E+00_JPRB, 0.74005E+00_JPRB, 0.73363E+00_JPRB, 0.77293E+00_JPRB ,&
&  0.75557E+00_JPRB, 0.79641E+00_JPRB, 0.76539E+00_JPRB, 0.75027E+00_JPRB, 0.74287E+00_JPRB, 0.77475E+00_JPRB, 0.84653E+00_JPRB ,&
&  0.94807E+00_JPRB, 0.11177E+01_JPRB, 0.12716E+01_JPRB, 0.12743E+01_JPRB, 0.13194E+01_JPRB, 0.12185E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.10669E+01_JPRB, 0.99340E+00_JPRB, 0.96112E+00_JPRB, 0.95430E+00_JPRB, 0.95967E+00_JPRB, 0.98449E+00_JPRB ,&
&  0.10232E+01_JPRB, 0.10397E+01_JPRB, 0.10197E+01_JPRB, 0.10111E+01_JPRB, 0.10568E+01_JPRB, 0.11667E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.16626E+01_JPRB, 0.18817E+01_JPRB, 0.19913E+01_JPRB, 0.23856E+01_JPRB, 0.26082E+01_JPRB, 0.26369E+01_JPRB, 0.25204E+01_JPRB ,&
&  0.23688E+01_JPRB, 0.22193E+01_JPRB, 0.22828E+01_JPRB, 0.23210E+01_JPRB, 0.23440E+01_JPRB, 0.23640E+01_JPRB, 0.22455E+01_JPRB ,&
&  0.19838E+01_JPRB, 0.18283E+01_JPRB, 0.17772E+01_JPRB, 0.17064E+01_JPRB, 0.16797E+01_JPRB, 0.15950E+01_JPRB, 0.16317E+01_JPRB ,&
&  0.17192E+01_JPRB, 0.10628E+01_JPRB, 0.88582E+00_JPRB, 0.10229E+01_JPRB, 0.88658E+00_JPRB, 0.91098E+00_JPRB, 0.10358E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10481E+01_JPRB, 0.12151E+01_JPRB, 0.14078E+01_JPRB, 0.16297E+01_JPRB, 0.20354E+01_JPRB, 0.20685E+01_JPRB ,&
&  0.19223E+01_JPRB, 0.17418E+01_JPRB, 0.17683E+01_JPRB, 0.17116E+01_JPRB, 0.15631E+01_JPRB, 0.13340E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.10542E+01_JPRB, 0.10097E+01_JPRB, 0.98258E+00_JPRB, 0.93974E+00_JPRB, 0.88651E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.94770E+00_JPRB, 0.91736E+00_JPRB, 0.84557E+00_JPRB, 0.82910E+00_JPRB, 0.84131E+00_JPRB, 0.86959E+00_JPRB, 0.89989E+00_JPRB ,&
&  0.90466E+00_JPRB, 0.86023E+00_JPRB, 0.82683E+00_JPRB, 0.81884E+00_JPRB, 0.81953E+00_JPRB, 0.82044E+00_JPRB, 0.83118E+00_JPRB ,&
&  0.85334E+00_JPRB, 0.89691E+00_JPRB, 0.78795E+00_JPRB, 0.75147E+00_JPRB, 0.69521E+00_JPRB, 0.73030E+00_JPRB, 0.78474E+00_JPRB ,&
&  0.85238E+00_JPRB, 0.85257E+00_JPRB, 0.83494E+00_JPRB, 0.85043E+00_JPRB, 0.86664E+00_JPRB, 0.92081E+00_JPRB, 0.10896E+01_JPRB ,&
&  0.12956E+01_JPRB, 0.15552E+01_JPRB, 0.16163E+01_JPRB, 0.17043E+01_JPRB, 0.15663E+01_JPRB, 0.13667E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.12514E+01_JPRB, 0.11420E+01_JPRB, 0.10561E+01_JPRB, 0.10227E+01_JPRB, 0.10141E+01_JPRB, 0.10063E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10650E+01_JPRB, 0.10565E+01_JPRB, 0.10455E+01_JPRB, 0.11159E+01_JPRB, 0.12136E+01_JPRB, 0.12314E+01_JPRB ,&
&  0.14932E+01_JPRB, 0.14932E+01_JPRB, 0.13307E+01_JPRB, 0.18849E+01_JPRB, 0.27471E+01_JPRB, 0.27695E+01_JPRB, 0.24127E+01_JPRB ,&
&  0.24964E+01_JPRB, 0.24587E+01_JPRB, 0.25498E+01_JPRB, 0.26529E+01_JPRB, 0.26191E+01_JPRB, 0.24733E+01_JPRB, 0.22747E+01_JPRB ,&
&  0.20349E+01_JPRB, 0.18928E+01_JPRB, 0.18264E+01_JPRB, 0.19447E+01_JPRB, 0.19059E+01_JPRB, 0.18515E+01_JPRB, 0.17738E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.13587E+01_JPRB, 0.10090E+01_JPRB, 0.10303E+01_JPRB, 0.83750E+00_JPRB, 0.84691E+00_JPRB, 0.97284E+00_JPRB ,&
&  0.95976E+00_JPRB, 0.10391E+01_JPRB, 0.12284E+01_JPRB, 0.14577E+01_JPRB, 0.20713E+01_JPRB, 0.25203E+01_JPRB, 0.26598E+01_JPRB ,&
&  0.25265E+01_JPRB, 0.21731E+01_JPRB, 0.22036E+01_JPRB, 0.20244E+01_JPRB, 0.18844E+01_JPRB, 0.16674E+01_JPRB, 0.14214E+01_JPRB ,&
&  0.12551E+01_JPRB, 0.11751E+01_JPRB, 0.11211E+01_JPRB, 0.10519E+01_JPRB, 0.97821E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.99629E+00_JPRB, 0.93668E+00_JPRB, 0.83979E+00_JPRB, 0.82543E+00_JPRB, 0.84871E+00_JPRB, 0.90908E+00_JPRB, 0.94318E+00_JPRB ,&
&  0.92023E+00_JPRB, 0.86955E+00_JPRB, 0.84084E+00_JPRB, 0.83109E+00_JPRB, 0.82125E+00_JPRB, 0.80609E+00_JPRB, 0.80274E+00_JPRB ,&
&  0.80465E+00_JPRB, 0.80782E+00_JPRB, 0.73696E+00_JPRB, 0.69421E+00_JPRB, 0.65856E+00_JPRB, 0.68471E+00_JPRB, 0.74698E+00_JPRB ,&
&  0.80404E+00_JPRB, 0.85166E+00_JPRB, 0.91603E+00_JPRB, 0.96772E+00_JPRB, 0.99016E+00_JPRB, 0.11152E+01_JPRB, 0.13897E+01_JPRB ,&
&  0.18603E+01_JPRB, 0.18346E+01_JPRB, 0.19149E+01_JPRB, 0.17991E+01_JPRB, 0.16325E+01_JPRB, 0.14703E+01_JPRB, 0.13312E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.10788E+01_JPRB, 0.10081E+01_JPRB, 0.97475E+00_JPRB, 0.99515E+00_JPRB, 0.10287E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.11209E+01_JPRB, 0.10831E+01_JPRB, 0.11108E+01_JPRB, 0.12158E+01_JPRB, 0.12756E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.12237E+01_JPRB, 0.16063E+01_JPRB, 0.19070E+01_JPRB, 0.24368E+01_JPRB, 0.27706E+01_JPRB, 0.26803E+01_JPRB ,&
&  0.27114E+01_JPRB, 0.26402E+01_JPRB, 0.26480E+01_JPRB, 0.22035E+01_JPRB, 0.24610E+01_JPRB, 0.23556E+01_JPRB, 0.21684E+01_JPRB ,&
&  0.19990E+01_JPRB, 0.19119E+01_JPRB, 0.19157E+01_JPRB, 0.20065E+01_JPRB, 0.20991E+01_JPRB, 0.21627E+01_JPRB, 0.16692E+01_JPRB ,&
&  0.13462E+01_JPRB, 0.14849E+01_JPRB, 0.13611E+01_JPRB, 0.11652E+01_JPRB, 0.10079E+01_JPRB, 0.10307E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.11102E+01_JPRB, 0.13424E+01_JPRB, 0.18332E+01_JPRB, 0.26927E+01_JPRB, 0.32582E+01_JPRB, 0.32465E+01_JPRB ,&
&  0.28485E+01_JPRB, 0.28989E+01_JPRB, 0.26704E+01_JPRB, 0.23443E+01_JPRB, 0.20997E+01_JPRB, 0.18979E+01_JPRB, 0.16973E+01_JPRB ,&
&  0.15049E+01_JPRB, 0.13789E+01_JPRB, 0.12809E+01_JPRB, 0.11699E+01_JPRB, 0.10558E+01_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.10348E+01_JPRB, 0.96939E+00_JPRB, 0.87615E+00_JPRB, 0.86324E+00_JPRB, 0.90913E+00_JPRB, 0.96998E+00_JPRB, 0.97918E+00_JPRB ,&
&  0.90862E+00_JPRB, 0.86245E+00_JPRB, 0.83368E+00_JPRB, 0.83220E+00_JPRB, 0.82560E+00_JPRB, 0.80403E+00_JPRB, 0.78539E+00_JPRB ,&
&  0.80106E+00_JPRB, 0.76435E+00_JPRB, 0.70031E+00_JPRB, 0.72023E+00_JPRB, 0.72827E+00_JPRB, 0.67679E+00_JPRB, 0.71603E+00_JPRB ,&
&  0.77832E+00_JPRB, 0.85700E+00_JPRB, 0.92981E+00_JPRB, 0.10543E+01_JPRB, 0.12186E+01_JPRB, 0.14718E+01_JPRB, 0.17911E+01_JPRB ,&
&  0.19105E+01_JPRB, 0.19885E+01_JPRB, 0.20165E+01_JPRB, 0.18577E+01_JPRB, 0.15659E+01_JPRB, 0.13483E+01_JPRB, 0.12279E+01_JPRB ,&
&  0.11108E+01_JPRB, 0.10098E+01_JPRB, 0.97957E+00_JPRB, 0.10060E+01_JPRB, 0.10743E+01_JPRB, 0.11299E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.11573E+01_JPRB, 0.11762E+01_JPRB, 0.13016E+01_JPRB, 0.10889E+01_JPRB, 0.10907E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.14163E+01_JPRB, 0.14892E+01_JPRB, 0.15705E+01_JPRB, 0.19791E+01_JPRB, 0.18587E+01_JPRB, 0.21460E+01_JPRB, 0.26061E+01_JPRB ,&
&  0.28408E+01_JPRB, 0.28641E+01_JPRB, 0.28714E+01_JPRB, 0.18620E+01_JPRB, 0.16180E+01_JPRB, 0.21099E+01_JPRB, 0.20873E+01_JPRB ,&
&  0.19748E+01_JPRB, 0.20247E+01_JPRB, 0.20405E+01_JPRB, 0.22196E+01_JPRB, 0.17463E+01_JPRB, 0.15120E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.11778E+01_JPRB, 0.11132E+01_JPRB, 0.13026E+01_JPRB, 0.12109E+01_JPRB, 0.12768E+01_JPRB, 0.12351E+01_JPRB, 0.11722E+01_JPRB ,&
&  0.12605E+01_JPRB, 0.14509E+01_JPRB, 0.18575E+01_JPRB, 0.26883E+01_JPRB, 0.37008E+01_JPRB, 0.34837E+01_JPRB, 0.38038E+01_JPRB ,&
&  0.37365E+01_JPRB, 0.33742E+01_JPRB, 0.27940E+01_JPRB, 0.25416E+01_JPRB, 0.23194E+01_JPRB, 0.21307E+01_JPRB, 0.19307E+01_JPRB ,&
&  0.17250E+01_JPRB, 0.15600E+01_JPRB, 0.14045E+01_JPRB, 0.12491E+01_JPRB, 0.11145E+01_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.11206E+01_JPRB, 0.10518E+01_JPRB, 0.93942E+00_JPRB, 0.93430E+00_JPRB, 0.98758E+00_JPRB, 0.10123E+01_JPRB, 0.96073E+00_JPRB ,&
&  0.90172E+00_JPRB, 0.85261E+00_JPRB, 0.83591E+00_JPRB, 0.82770E+00_JPRB, 0.81578E+00_JPRB, 0.81037E+00_JPRB, 0.79798E+00_JPRB ,&
&  0.83506E+00_JPRB, 0.83449E+00_JPRB, 0.75360E+00_JPRB, 0.70869E+00_JPRB, 0.74938E+00_JPRB, 0.70098E+00_JPRB, 0.67898E+00_JPRB ,&
&  0.78914E+00_JPRB, 0.91291E+00_JPRB, 0.10537E+01_JPRB, 0.12677E+01_JPRB, 0.15190E+01_JPRB, 0.19562E+01_JPRB, 0.21812E+01_JPRB ,&
&  0.21694E+01_JPRB, 0.21388E+01_JPRB, 0.18861E+01_JPRB, 0.16026E+01_JPRB, 0.13449E+01_JPRB, 0.11920E+01_JPRB, 0.10855E+01_JPRB ,&
&  0.10042E+01_JPRB, 0.95691E+00_JPRB, 0.97741E+00_JPRB, 0.10370E+01_JPRB, 0.11077E+01_JPRB, 0.11397E+01_JPRB, 0.11499E+01_JPRB ,&
&  0.11960E+01_JPRB, 0.12856E+01_JPRB, 0.13253E+01_JPRB, 0.12941E+01_JPRB, 0.12346E+01_JPRB, 0.11086E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.13921E+01_JPRB, 0.13720E+01_JPRB, 0.15064E+01_JPRB, 0.18304E+01_JPRB, 0.21280E+01_JPRB, 0.21571E+01_JPRB, 0.26130E+01_JPRB ,&
&  0.21919E+01_JPRB, 0.18329E+01_JPRB, 0.15526E+01_JPRB, 0.12013E+01_JPRB, 0.14784E+01_JPRB, 0.20759E+01_JPRB, 0.21576E+01_JPRB ,&
&  0.21208E+01_JPRB, 0.21580E+01_JPRB, 0.22233E+01_JPRB, 0.19742E+01_JPRB, 0.15313E+01_JPRB, 0.12841E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.16605E+01_JPRB, 0.17553E+01_JPRB, 0.17015E+01_JPRB, 0.14897E+01_JPRB, 0.13702E+01_JPRB, 0.15732E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.18397E+01_JPRB, 0.20980E+01_JPRB, 0.33365E+01_JPRB, 0.44570E+01_JPRB, 0.46646E+01_JPRB, 0.45982E+01_JPRB, 0.48067E+01_JPRB ,&
&  0.42105E+01_JPRB, 0.37950E+01_JPRB, 0.32527E+01_JPRB, 0.27966E+01_JPRB, 0.24849E+01_JPRB, 0.22688E+01_JPRB, 0.20979E+01_JPRB ,&
&  0.18982E+01_JPRB, 0.17467E+01_JPRB, 0.15750E+01_JPRB, 0.13653E+01_JPRB, 0.12066E+01_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.12301E+01_JPRB, 0.11316E+01_JPRB, 0.10030E+01_JPRB, 0.99402E+00_JPRB, 0.10424E+01_JPRB, 0.10160E+01_JPRB, 0.93563E+00_JPRB ,&
&  0.90448E+00_JPRB, 0.86010E+00_JPRB, 0.84517E+00_JPRB, 0.82883E+00_JPRB, 0.81129E+00_JPRB, 0.80248E+00_JPRB, 0.81953E+00_JPRB ,&
&  0.88871E+00_JPRB, 0.98866E+00_JPRB, 0.89096E+00_JPRB, 0.80798E+00_JPRB, 0.77954E+00_JPRB, 0.75583E+00_JPRB, 0.67613E+00_JPRB ,&
&  0.84431E+00_JPRB, 0.10442E+01_JPRB, 0.12280E+01_JPRB, 0.15089E+01_JPRB, 0.19601E+01_JPRB, 0.20775E+01_JPRB, 0.21347E+01_JPRB ,&
&  0.20991E+01_JPRB, 0.19360E+01_JPRB, 0.16435E+01_JPRB, 0.13831E+01_JPRB, 0.12057E+01_JPRB, 0.11065E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.97042E+00_JPRB, 0.98572E+00_JPRB, 0.10241E+01_JPRB, 0.10720E+01_JPRB, 0.11530E+01_JPRB, 0.12172E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.13404E+01_JPRB, 0.14151E+01_JPRB, 0.14395E+01_JPRB, 0.14218E+01_JPRB, 0.14265E+01_JPRB, 0.13019E+01_JPRB, 0.16509E+01_JPRB ,&
&  0.15826E+01_JPRB, 0.16140E+01_JPRB, 0.16038E+01_JPRB, 0.20170E+01_JPRB, 0.20866E+01_JPRB, 0.20753E+01_JPRB, 0.23373E+01_JPRB ,&
&  0.18132E+01_JPRB, 0.16511E+01_JPRB, 0.17421E+01_JPRB, 0.17668E+01_JPRB, 0.15896E+01_JPRB, 0.17101E+01_JPRB, 0.23838E+01_JPRB ,&
&  0.19473E+01_JPRB, 0.21461E+01_JPRB, 0.22813E+01_JPRB, 0.22585E+01_JPRB, 0.11246E+01_JPRB, 0.91825E+00_JPRB, 0.14716E+01_JPRB ,&
&  0.16490E+01_JPRB, 0.11605E+01_JPRB, 0.91063E+00_JPRB, 0.11282E+01_JPRB, 0.93691E+00_JPRB, 0.11877E+01_JPRB, 0.18766E+01_JPRB ,&
&  0.24524E+01_JPRB, 0.32631E+01_JPRB, 0.55157E+01_JPRB, 0.57408E+01_JPRB, 0.51470E+01_JPRB, 0.54700E+01_JPRB, 0.47857E+01_JPRB ,&
&  0.42913E+01_JPRB, 0.33601E+01_JPRB, 0.31749E+01_JPRB, 0.28254E+01_JPRB, 0.25565E+01_JPRB, 0.23860E+01_JPRB, 0.22285E+01_JPRB ,&
&  0.20447E+01_JPRB, 0.18723E+01_JPRB, 0.16887E+01_JPRB, 0.14987E+01_JPRB, 0.13480E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.12987E+01_JPRB, 0.11852E+01_JPRB, 0.10691E+01_JPRB, 0.10426E+01_JPRB, 0.10657E+01_JPRB, 0.98525E+00_JPRB, 0.92761E+00_JPRB ,&
&  0.88306E+00_JPRB, 0.85975E+00_JPRB, 0.83993E+00_JPRB, 0.81528E+00_JPRB, 0.79625E+00_JPRB, 0.80755E+00_JPRB, 0.84219E+00_JPRB ,&
&  0.90332E+00_JPRB, 0.10169E+01_JPRB, 0.11408E+01_JPRB, 0.10452E+01_JPRB, 0.97966E+00_JPRB, 0.90321E+00_JPRB, 0.84124E+00_JPRB ,&
&  0.10149E+01_JPRB, 0.12954E+01_JPRB, 0.14810E+01_JPRB, 0.16380E+01_JPRB, 0.18086E+01_JPRB, 0.18181E+01_JPRB, 0.19231E+01_JPRB ,&
&  0.18878E+01_JPRB, 0.16383E+01_JPRB, 0.14315E+01_JPRB, 0.12591E+01_JPRB, 0.11489E+01_JPRB, 0.10614E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.10114E+01_JPRB, 0.10227E+01_JPRB, 0.10328E+01_JPRB, 0.11165E+01_JPRB, 0.12430E+01_JPRB, 0.13179E+01_JPRB, 0.13875E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.14711E+01_JPRB, 0.14767E+01_JPRB, 0.14948E+01_JPRB, 0.15799E+01_JPRB, 0.14863E+01_JPRB, 0.13506E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.14470E+01_JPRB, 0.19790E+01_JPRB, 0.21249E+01_JPRB, 0.20038E+01_JPRB, 0.20983E+01_JPRB, 0.20934E+01_JPRB ,&
&  0.20392E+01_JPRB, 0.21549E+01_JPRB, 0.19948E+01_JPRB, 0.20973E+01_JPRB, 0.20044E+01_JPRB, 0.15769E+01_JPRB, 0.19985E+01_JPRB ,&
&  0.21313E+01_JPRB, 0.20705E+01_JPRB, 0.16325E+01_JPRB, 0.12161E+01_JPRB, 0.12778E+01_JPRB, 0.10704E+01_JPRB, 0.81350E+00_JPRB ,&
&  0.77988E+00_JPRB, 0.75251E+00_JPRB, 0.70148E+00_JPRB, 0.73561E+00_JPRB, 0.76239E+00_JPRB, 0.11775E+01_JPRB, 0.19896E+01_JPRB ,&
&  0.35844E+01_JPRB, 0.53149E+01_JPRB, 0.64398E+01_JPRB, 0.61749E+01_JPRB, 0.53729E+01_JPRB, 0.54294E+01_JPRB, 0.47988E+01_JPRB ,&
&  0.35481E+01_JPRB, 0.30198E+01_JPRB, 0.28454E+01_JPRB, 0.27265E+01_JPRB, 0.26295E+01_JPRB, 0.25272E+01_JPRB, 0.23563E+01_JPRB ,&
&  0.21414E+01_JPRB, 0.19700E+01_JPRB, 0.18020E+01_JPRB, 0.15977E+01_JPRB, 0.14386E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.13462E+01_JPRB, 0.12319E+01_JPRB, 0.11612E+01_JPRB, 0.11098E+01_JPRB, 0.10763E+01_JPRB, 0.96480E+00_JPRB, 0.91978E+00_JPRB ,&
&  0.86175E+00_JPRB, 0.84583E+00_JPRB, 0.81210E+00_JPRB, 0.79498E+00_JPRB, 0.78722E+00_JPRB, 0.81949E+00_JPRB, 0.88015E+00_JPRB ,&
&  0.95406E+00_JPRB, 0.11124E+01_JPRB, 0.13560E+01_JPRB, 0.14084E+01_JPRB, 0.14814E+01_JPRB, 0.14146E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.13769E+01_JPRB, 0.17099E+01_JPRB, 0.17652E+01_JPRB, 0.17638E+01_JPRB, 0.18362E+01_JPRB, 0.18235E+01_JPRB, 0.17471E+01_JPRB ,&
&  0.16449E+01_JPRB, 0.14475E+01_JPRB, 0.12870E+01_JPRB, 0.11764E+01_JPRB, 0.10996E+01_JPRB, 0.10450E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10261E+01_JPRB, 0.10586E+01_JPRB, 0.11840E+01_JPRB, 0.13055E+01_JPRB, 0.13715E+01_JPRB, 0.14539E+01_JPRB ,&
&  0.15232E+01_JPRB, 0.15300E+01_JPRB, 0.15205E+01_JPRB, 0.14970E+01_JPRB, 0.12970E+01_JPRB, 0.11487E+01_JPRB, 0.13790E+01_JPRB ,&
&  0.15804E+01_JPRB, 0.18912E+01_JPRB, 0.20958E+01_JPRB, 0.23941E+01_JPRB, 0.20781E+01_JPRB, 0.20038E+01_JPRB, 0.20147E+01_JPRB ,&
&  0.20253E+01_JPRB, 0.24977E+01_JPRB, 0.20810E+01_JPRB, 0.19550E+01_JPRB, 0.21954E+01_JPRB, 0.17694E+01_JPRB, 0.17621E+01_JPRB ,&
&  0.20661E+01_JPRB, 0.19801E+01_JPRB, 0.16476E+01_JPRB, 0.13510E+01_JPRB, 0.17990E+01_JPRB, 0.19911E+01_JPRB, 0.10216E+01_JPRB ,&
&  0.85652E+00_JPRB, 0.80674E+00_JPRB, 0.76887E+00_JPRB, 0.74333E+00_JPRB, 0.79869E+00_JPRB, 0.13233E+01_JPRB, 0.28527E+01_JPRB ,&
&  0.43781E+01_JPRB, 0.66194E+01_JPRB, 0.69087E+01_JPRB, 0.64662E+01_JPRB, 0.54625E+01_JPRB, 0.50495E+01_JPRB, 0.40716E+01_JPRB ,&
&  0.34737E+01_JPRB, 0.30672E+01_JPRB, 0.28425E+01_JPRB, 0.27287E+01_JPRB, 0.26230E+01_JPRB, 0.25178E+01_JPRB, 0.23741E+01_JPRB ,&
&  0.21877E+01_JPRB, 0.20227E+01_JPRB, 0.18555E+01_JPRB, 0.16462E+01_JPRB, 0.14776E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.13616E+01_JPRB, 0.12774E+01_JPRB, 0.12381E+01_JPRB, 0.11936E+01_JPRB, 0.10985E+01_JPRB, 0.98356E+00_JPRB, 0.89337E+00_JPRB ,&
&  0.84718E+00_JPRB, 0.82153E+00_JPRB, 0.80074E+00_JPRB, 0.79591E+00_JPRB, 0.80651E+00_JPRB, 0.84900E+00_JPRB, 0.91024E+00_JPRB ,&
&  0.10263E+01_JPRB, 0.12145E+01_JPRB, 0.14173E+01_JPRB, 0.17687E+01_JPRB, 0.18742E+01_JPRB, 0.17316E+01_JPRB, 0.19245E+01_JPRB ,&
&  0.20052E+01_JPRB, 0.21719E+01_JPRB, 0.21049E+01_JPRB, 0.19517E+01_JPRB, 0.18456E+01_JPRB, 0.17780E+01_JPRB, 0.16573E+01_JPRB ,&
&  0.14892E+01_JPRB, 0.13293E+01_JPRB, 0.11986E+01_JPRB, 0.11393E+01_JPRB, 0.10903E+01_JPRB, 0.10531E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10065E+01_JPRB, 0.10932E+01_JPRB, 0.12265E+01_JPRB, 0.13302E+01_JPRB, 0.14274E+01_JPRB, 0.15341E+01_JPRB ,&
&  0.15750E+01_JPRB, 0.15775E+01_JPRB, 0.15570E+01_JPRB, 0.15279E+01_JPRB, 0.12803E+01_JPRB, 0.14600E+01_JPRB, 0.15303E+01_JPRB ,&
&  0.16829E+01_JPRB, 0.18654E+01_JPRB, 0.19343E+01_JPRB, 0.21890E+01_JPRB, 0.20654E+01_JPRB, 0.19594E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.20835E+01_JPRB, 0.20361E+01_JPRB, 0.17801E+01_JPRB, 0.18418E+01_JPRB, 0.20970E+01_JPRB, 0.23734E+01_JPRB, 0.16368E+01_JPRB ,&
&  0.14947E+01_JPRB, 0.18918E+01_JPRB, 0.17150E+01_JPRB, 0.15856E+01_JPRB, 0.20200E+01_JPRB, 0.28204E+01_JPRB, 0.20971E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.87002E+00_JPRB, 0.90975E+00_JPRB, 0.98845E+00_JPRB, 0.10131E+01_JPRB, 0.17083E+01_JPRB, 0.59233E+01_JPRB ,&
&  0.63041E+01_JPRB, 0.76619E+01_JPRB, 0.69341E+01_JPRB, 0.59588E+01_JPRB, 0.53022E+01_JPRB, 0.46988E+01_JPRB, 0.39648E+01_JPRB ,&
&  0.34164E+01_JPRB, 0.30827E+01_JPRB, 0.28217E+01_JPRB, 0.26344E+01_JPRB, 0.24502E+01_JPRB, 0.22685E+01_JPRB, 0.21146E+01_JPRB ,&
&  0.20117E+01_JPRB, 0.18962E+01_JPRB, 0.17806E+01_JPRB, 0.16172E+01_JPRB, 0.14704E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.13382E+01_JPRB, 0.12776E+01_JPRB, 0.12495E+01_JPRB, 0.11993E+01_JPRB, 0.11123E+01_JPRB, 0.10331E+01_JPRB, 0.92505E+00_JPRB ,&
&  0.85341E+00_JPRB, 0.81484E+00_JPRB, 0.80025E+00_JPRB, 0.80895E+00_JPRB, 0.84009E+00_JPRB, 0.88928E+00_JPRB, 0.94150E+00_JPRB ,&
&  0.11062E+01_JPRB, 0.12649E+01_JPRB, 0.15253E+01_JPRB, 0.18819E+01_JPRB, 0.20630E+01_JPRB, 0.18948E+01_JPRB, 0.20777E+01_JPRB ,&
&  0.26224E+01_JPRB, 0.25594E+01_JPRB, 0.22232E+01_JPRB, 0.20558E+01_JPRB, 0.19501E+01_JPRB, 0.19046E+01_JPRB, 0.16316E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.13589E+01_JPRB, 0.12395E+01_JPRB, 0.11797E+01_JPRB, 0.11181E+01_JPRB, 0.11114E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.10051E+01_JPRB, 0.10365E+01_JPRB, 0.11385E+01_JPRB, 0.12489E+01_JPRB, 0.13549E+01_JPRB, 0.14608E+01_JPRB, 0.15807E+01_JPRB ,&
&  0.16365E+01_JPRB, 0.16274E+01_JPRB, 0.15833E+01_JPRB, 0.14880E+01_JPRB, 0.14856E+01_JPRB, 0.15232E+01_JPRB, 0.16405E+01_JPRB ,&
&  0.16991E+01_JPRB, 0.18211E+01_JPRB, 0.18918E+01_JPRB, 0.19622E+01_JPRB, 0.19195E+01_JPRB, 0.18819E+01_JPRB, 0.18044E+01_JPRB ,&
&  0.18928E+01_JPRB, 0.18156E+01_JPRB, 0.15792E+01_JPRB, 0.17770E+01_JPRB, 0.21079E+01_JPRB, 0.23346E+01_JPRB, 0.21759E+01_JPRB ,&
&  0.19385E+01_JPRB, 0.17111E+01_JPRB, 0.17181E+01_JPRB, 0.20274E+01_JPRB, 0.23687E+01_JPRB, 0.29309E+01_JPRB, 0.38694E+01_JPRB ,&
&  0.41252E+01_JPRB, 0.23989E+01_JPRB, 0.19999E+01_JPRB, 0.27502E+01_JPRB, 0.24472E+01_JPRB, 0.26785E+01_JPRB, 0.46223E+01_JPRB ,&
&  0.62846E+01_JPRB, 0.72263E+01_JPRB, 0.66893E+01_JPRB, 0.56057E+01_JPRB, 0.50501E+01_JPRB, 0.42913E+01_JPRB, 0.37272E+01_JPRB ,&
&  0.31878E+01_JPRB, 0.27553E+01_JPRB, 0.24515E+01_JPRB, 0.22230E+01_JPRB, 0.20167E+01_JPRB, 0.18635E+01_JPRB, 0.18017E+01_JPRB ,&
&  0.17619E+01_JPRB, 0.17036E+01_JPRB, 0.16387E+01_JPRB, 0.15178E+01_JPRB, 0.14090E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.12864E+01_JPRB, 0.12475E+01_JPRB, 0.12169E+01_JPRB, 0.11671E+01_JPRB, 0.11465E+01_JPRB, 0.11182E+01_JPRB, 0.10204E+01_JPRB ,&
&  0.93989E+00_JPRB, 0.82853E+00_JPRB, 0.82121E+00_JPRB, 0.84678E+00_JPRB, 0.88754E+00_JPRB, 0.94055E+00_JPRB, 0.10400E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.13519E+01_JPRB, 0.16821E+01_JPRB, 0.20707E+01_JPRB, 0.24416E+01_JPRB, 0.30613E+01_JPRB, 0.21445E+01_JPRB ,&
&  0.22163E+01_JPRB, 0.23725E+01_JPRB, 0.21489E+01_JPRB, 0.20866E+01_JPRB, 0.19852E+01_JPRB, 0.19433E+01_JPRB, 0.16352E+01_JPRB ,&
&  0.14672E+01_JPRB, 0.13855E+01_JPRB, 0.12804E+01_JPRB, 0.11867E+01_JPRB, 0.11383E+01_JPRB, 0.11072E+01_JPRB, 0.10303E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.10859E+01_JPRB, 0.11761E+01_JPRB, 0.12718E+01_JPRB, 0.13786E+01_JPRB, 0.14978E+01_JPRB, 0.15809E+01_JPRB ,&
&  0.16607E+01_JPRB, 0.16736E+01_JPRB, 0.16340E+01_JPRB, 0.15189E+01_JPRB, 0.15396E+01_JPRB, 0.15659E+01_JPRB, 0.15973E+01_JPRB ,&
&  0.15233E+01_JPRB, 0.14748E+01_JPRB, 0.16778E+01_JPRB, 0.18276E+01_JPRB, 0.18011E+01_JPRB, 0.16864E+01_JPRB, 0.16047E+01_JPRB ,&
&  0.17532E+01_JPRB, 0.18352E+01_JPRB, 0.19126E+01_JPRB, 0.19347E+01_JPRB, 0.21619E+01_JPRB, 0.23452E+01_JPRB, 0.24613E+01_JPRB ,&
&  0.25283E+01_JPRB, 0.23934E+01_JPRB, 0.24464E+01_JPRB, 0.26717E+01_JPRB, 0.28234E+01_JPRB, 0.32552E+01_JPRB, 0.37968E+01_JPRB ,&
&  0.46030E+01_JPRB, 0.56484E+01_JPRB, 0.58839E+01_JPRB, 0.48600E+01_JPRB, 0.45125E+01_JPRB, 0.41443E+01_JPRB, 0.49298E+01_JPRB ,&
&  0.61823E+01_JPRB, 0.59713E+01_JPRB, 0.55587E+01_JPRB, 0.55055E+01_JPRB, 0.41770E+01_JPRB, 0.35172E+01_JPRB, 0.30753E+01_JPRB ,&
&  0.27553E+01_JPRB, 0.23806E+01_JPRB, 0.20857E+01_JPRB, 0.18620E+01_JPRB, 0.16550E+01_JPRB, 0.15650E+01_JPRB, 0.15689E+01_JPRB ,&
&  0.15533E+01_JPRB, 0.15322E+01_JPRB, 0.14919E+01_JPRB, 0.14115E+01_JPRB, 0.13431E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.12324E+01_JPRB, 0.11882E+01_JPRB, 0.11610E+01_JPRB, 0.11347E+01_JPRB, 0.11695E+01_JPRB, 0.11730E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.88256E+00_JPRB, 0.86537E+00_JPRB, 0.89230E+00_JPRB, 0.94348E+00_JPRB, 0.10140E+01_JPRB, 0.11336E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.14448E+01_JPRB, 0.18018E+01_JPRB, 0.22390E+01_JPRB, 0.26241E+01_JPRB, 0.29199E+01_JPRB, 0.29166E+01_JPRB ,&
&  0.18783E+01_JPRB, 0.24076E+01_JPRB, 0.21079E+01_JPRB, 0.21460E+01_JPRB, 0.21289E+01_JPRB, 0.20376E+01_JPRB, 0.16868E+01_JPRB ,&
&  0.14649E+01_JPRB, 0.13830E+01_JPRB, 0.13202E+01_JPRB, 0.12830E+01_JPRB, 0.11980E+01_JPRB, 0.11059E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.11810E+01_JPRB, 0.12716E+01_JPRB, 0.13576E+01_JPRB, 0.14634E+01_JPRB, 0.15635E+01_JPRB, 0.16280E+01_JPRB ,&
&  0.17001E+01_JPRB, 0.17145E+01_JPRB, 0.16311E+01_JPRB, 0.15977E+01_JPRB, 0.15872E+01_JPRB, 0.16126E+01_JPRB, 0.16126E+01_JPRB ,&
&  0.15327E+01_JPRB, 0.14694E+01_JPRB, 0.15916E+01_JPRB, 0.16063E+01_JPRB, 0.15403E+01_JPRB, 0.15883E+01_JPRB, 0.15687E+01_JPRB ,&
&  0.18102E+01_JPRB, 0.18780E+01_JPRB, 0.23160E+01_JPRB, 0.20975E+01_JPRB, 0.21432E+01_JPRB, 0.23629E+01_JPRB, 0.23633E+01_JPRB ,&
&  0.22364E+01_JPRB, 0.24689E+01_JPRB, 0.26709E+01_JPRB, 0.31485E+01_JPRB, 0.35676E+01_JPRB, 0.40037E+01_JPRB, 0.41595E+01_JPRB ,&
&  0.45684E+01_JPRB, 0.50932E+01_JPRB, 0.57558E+01_JPRB, 0.47415E+01_JPRB, 0.44739E+01_JPRB, 0.44663E+01_JPRB, 0.51633E+01_JPRB ,&
&  0.57150E+01_JPRB, 0.53435E+01_JPRB, 0.51926E+01_JPRB, 0.44989E+01_JPRB, 0.34105E+01_JPRB, 0.29444E+01_JPRB, 0.25697E+01_JPRB ,&
&  0.22851E+01_JPRB, 0.19910E+01_JPRB, 0.17577E+01_JPRB, 0.15642E+01_JPRB, 0.14458E+01_JPRB, 0.14262E+01_JPRB, 0.14419E+01_JPRB ,&
&  0.14408E+01_JPRB, 0.14282E+01_JPRB, 0.14038E+01_JPRB, 0.13406E+01_JPRB, 0.12812E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.11988E+01_JPRB, 0.11761E+01_JPRB, 0.11623E+01_JPRB, 0.11224E+01_JPRB, 0.11620E+01_JPRB, 0.12644E+01_JPRB, 0.13388E+01_JPRB ,&
&  0.12903E+01_JPRB, 0.91980E+00_JPRB, 0.91120E+00_JPRB, 0.93941E+00_JPRB, 0.98754E+00_JPRB, 0.10811E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.15584E+01_JPRB, 0.19413E+01_JPRB, 0.23938E+01_JPRB, 0.27612E+01_JPRB, 0.31937E+01_JPRB, 0.32899E+01_JPRB ,&
&  0.23168E+01_JPRB, 0.19159E+01_JPRB, 0.20450E+01_JPRB, 0.21077E+01_JPRB, 0.20171E+01_JPRB, 0.20562E+01_JPRB, 0.19995E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.15591E+01_JPRB, 0.15391E+01_JPRB, 0.14851E+01_JPRB, 0.12107E+01_JPRB, 0.11756E+01_JPRB, 0.11819E+01_JPRB ,&
&  0.12295E+01_JPRB, 0.12995E+01_JPRB, 0.13816E+01_JPRB, 0.14609E+01_JPRB, 0.15608E+01_JPRB, 0.16318E+01_JPRB, 0.17003E+01_JPRB ,&
&  0.17403E+01_JPRB, 0.17669E+01_JPRB, 0.17138E+01_JPRB, 0.16374E+01_JPRB, 0.16258E+01_JPRB, 0.16148E+01_JPRB, 0.15787E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.15920E+01_JPRB, 0.16150E+01_JPRB, 0.15577E+01_JPRB, 0.14862E+01_JPRB, 0.16531E+01_JPRB, 0.17435E+01_JPRB ,&
&  0.19163E+01_JPRB, 0.21125E+01_JPRB, 0.26245E+01_JPRB, 0.21230E+01_JPRB, 0.21308E+01_JPRB, 0.22884E+01_JPRB, 0.23514E+01_JPRB ,&
&  0.23819E+01_JPRB, 0.26851E+01_JPRB, 0.30907E+01_JPRB, 0.35585E+01_JPRB, 0.40356E+01_JPRB, 0.40935E+01_JPRB, 0.50119E+01_JPRB ,&
&  0.47468E+01_JPRB, 0.54812E+01_JPRB, 0.50361E+01_JPRB, 0.48955E+01_JPRB, 0.42357E+01_JPRB, 0.43435E+01_JPRB, 0.44704E+01_JPRB ,&
&  0.47206E+01_JPRB, 0.43692E+01_JPRB, 0.38751E+01_JPRB, 0.33677E+01_JPRB, 0.26790E+01_JPRB, 0.23689E+01_JPRB, 0.20646E+01_JPRB ,&
&  0.17938E+01_JPRB, 0.16070E+01_JPRB, 0.15211E+01_JPRB, 0.14553E+01_JPRB, 0.14292E+01_JPRB, 0.14220E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.13790E+01_JPRB, 0.13490E+01_JPRB, 0.12880E+01_JPRB, 0.12364E+01_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.11710E+01_JPRB, 0.11463E+01_JPRB, 0.11130E+01_JPRB, 0.10342E+01_JPRB, 0.98779E+00_JPRB, 0.99098E+00_JPRB, 0.97933E+00_JPRB ,&
&  0.96489E+00_JPRB, 0.93874E+00_JPRB, 0.95524E+00_JPRB, 0.99916E+00_JPRB, 0.10570E+01_JPRB, 0.11547E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.14583E+01_JPRB, 0.16477E+01_JPRB, 0.19473E+01_JPRB, 0.22726E+01_JPRB, 0.25536E+01_JPRB, 0.27799E+01_JPRB, 0.29011E+01_JPRB ,&
&  0.23477E+01_JPRB, 0.16888E+01_JPRB, 0.20828E+01_JPRB, 0.22211E+01_JPRB, 0.21722E+01_JPRB, 0.22875E+01_JPRB, 0.22120E+01_JPRB ,&
&  0.18553E+01_JPRB, 0.17339E+01_JPRB, 0.17489E+01_JPRB, 0.15861E+01_JPRB, 0.12731E+01_JPRB, 0.12766E+01_JPRB, 0.13138E+01_JPRB ,&
&  0.13729E+01_JPRB, 0.14230E+01_JPRB, 0.14885E+01_JPRB, 0.15656E+01_JPRB, 0.16356E+01_JPRB, 0.17305E+01_JPRB, 0.17970E+01_JPRB ,&
&  0.18223E+01_JPRB, 0.18288E+01_JPRB, 0.18123E+01_JPRB, 0.17293E+01_JPRB, 0.16664E+01_JPRB, 0.16294E+01_JPRB, 0.15877E+01_JPRB ,&
&  0.15829E+01_JPRB, 0.15728E+01_JPRB, 0.16285E+01_JPRB, 0.17225E+01_JPRB, 0.18723E+01_JPRB, 0.18076E+01_JPRB, 0.19794E+01_JPRB ,&
&  0.20977E+01_JPRB, 0.23784E+01_JPRB, 0.24653E+01_JPRB, 0.27328E+01_JPRB, 0.18486E+01_JPRB, 0.20895E+01_JPRB, 0.20898E+01_JPRB ,&
&  0.27016E+01_JPRB, 0.29917E+01_JPRB, 0.34677E+01_JPRB, 0.39936E+01_JPRB, 0.46691E+01_JPRB, 0.47089E+01_JPRB, 0.50385E+01_JPRB ,&
&  0.49384E+01_JPRB, 0.47298E+01_JPRB, 0.43203E+01_JPRB, 0.43416E+01_JPRB, 0.41394E+01_JPRB, 0.45026E+01_JPRB, 0.39792E+01_JPRB ,&
&  0.39104E+01_JPRB, 0.34987E+01_JPRB, 0.31146E+01_JPRB, 0.27990E+01_JPRB, 0.22319E+01_JPRB, 0.18505E+01_JPRB, 0.16116E+01_JPRB ,&
&  0.14984E+01_JPRB, 0.14502E+01_JPRB, 0.14136E+01_JPRB, 0.13940E+01_JPRB, 0.13970E+01_JPRB, 0.14013E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13504E+01_JPRB, 0.13117E+01_JPRB, 0.12571E+01_JPRB, 0.12119E+01_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.11318E+01_JPRB, 0.10995E+01_JPRB, 0.10513E+01_JPRB, 0.99184E+00_JPRB, 0.96342E+00_JPRB, 0.94635E+00_JPRB, 0.94629E+00_JPRB ,&
&  0.95144E+00_JPRB, 0.97577E+00_JPRB, 0.99570E+00_JPRB, 0.10308E+01_JPRB, 0.10773E+01_JPRB, 0.11622E+01_JPRB, 0.12450E+01_JPRB ,&
&  0.13488E+01_JPRB, 0.14984E+01_JPRB, 0.17687E+01_JPRB, 0.21049E+01_JPRB, 0.23851E+01_JPRB, 0.25420E+01_JPRB, 0.24301E+01_JPRB ,&
&  0.23222E+01_JPRB, 0.22843E+01_JPRB, 0.21346E+01_JPRB, 0.17220E+01_JPRB, 0.16959E+01_JPRB, 0.21061E+01_JPRB, 0.18939E+01_JPRB ,&
&  0.16594E+01_JPRB, 0.15896E+01_JPRB, 0.15188E+01_JPRB, 0.14412E+01_JPRB, 0.13695E+01_JPRB, 0.13757E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.14610E+01_JPRB, 0.15128E+01_JPRB, 0.15835E+01_JPRB, 0.16570E+01_JPRB, 0.17350E+01_JPRB, 0.18485E+01_JPRB, 0.19163E+01_JPRB ,&
&  0.19202E+01_JPRB, 0.19104E+01_JPRB, 0.18776E+01_JPRB, 0.18108E+01_JPRB, 0.17130E+01_JPRB, 0.16968E+01_JPRB, 0.17124E+01_JPRB ,&
&  0.17363E+01_JPRB, 0.17638E+01_JPRB, 0.18084E+01_JPRB, 0.19455E+01_JPRB, 0.20658E+01_JPRB, 0.19359E+01_JPRB, 0.19712E+01_JPRB ,&
&  0.22227E+01_JPRB, 0.25330E+01_JPRB, 0.23357E+01_JPRB, 0.27400E+01_JPRB, 0.17004E+01_JPRB, 0.20511E+01_JPRB, 0.25779E+01_JPRB ,&
&  0.28447E+01_JPRB, 0.32781E+01_JPRB, 0.37896E+01_JPRB, 0.43766E+01_JPRB, 0.49803E+01_JPRB, 0.47117E+01_JPRB, 0.47421E+01_JPRB ,&
&  0.43426E+01_JPRB, 0.37817E+01_JPRB, 0.36726E+01_JPRB, 0.36031E+01_JPRB, 0.37887E+01_JPRB, 0.38433E+01_JPRB, 0.30571E+01_JPRB ,&
&  0.25811E+01_JPRB, 0.28088E+01_JPRB, 0.27513E+01_JPRB, 0.25519E+01_JPRB, 0.19056E+01_JPRB, 0.14937E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.13901E+01_JPRB, 0.13612E+01_JPRB, 0.13538E+01_JPRB, 0.13607E+01_JPRB, 0.13761E+01_JPRB, 0.13800E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.13227E+01_JPRB, 0.12791E+01_JPRB, 0.12267E+01_JPRB, 0.11766E+01_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.10824E+01_JPRB, 0.10470E+01_JPRB, 0.99450E+00_JPRB, 0.94711E+00_JPRB, 0.89137E+00_JPRB, 0.86526E+00_JPRB, 0.86479E+00_JPRB ,&
&  0.88859E+00_JPRB, 0.92224E+00_JPRB, 0.94548E+00_JPRB, 0.95351E+00_JPRB, 0.98423E+00_JPRB, 0.10033E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.12275E+01_JPRB, 0.14574E+01_JPRB, 0.18214E+01_JPRB, 0.21748E+01_JPRB, 0.25069E+01_JPRB, 0.23971E+01_JPRB ,&
&  0.22756E+01_JPRB, 0.21906E+01_JPRB, 0.21406E+01_JPRB, 0.21070E+01_JPRB, 0.18745E+01_JPRB, 0.20372E+01_JPRB, 0.20300E+01_JPRB ,&
&  0.20157E+01_JPRB, 0.18516E+01_JPRB, 0.15864E+01_JPRB, 0.15096E+01_JPRB, 0.14655E+01_JPRB, 0.14680E+01_JPRB, 0.15038E+01_JPRB ,&
&  0.15490E+01_JPRB, 0.15955E+01_JPRB, 0.16732E+01_JPRB, 0.17708E+01_JPRB, 0.18875E+01_JPRB, 0.19980E+01_JPRB, 0.20463E+01_JPRB ,&
&  0.20344E+01_JPRB, 0.20193E+01_JPRB, 0.19922E+01_JPRB, 0.19078E+01_JPRB, 0.18800E+01_JPRB, 0.19105E+01_JPRB, 0.19595E+01_JPRB ,&
&  0.20261E+01_JPRB, 0.19848E+01_JPRB, 0.20645E+01_JPRB, 0.22268E+01_JPRB, 0.22455E+01_JPRB, 0.21768E+01_JPRB, 0.22057E+01_JPRB ,&
&  0.23866E+01_JPRB, 0.26660E+01_JPRB, 0.19917E+01_JPRB, 0.24117E+01_JPRB, 0.29117E+01_JPRB, 0.26228E+01_JPRB, 0.27285E+01_JPRB ,&
&  0.31461E+01_JPRB, 0.35152E+01_JPRB, 0.39739E+01_JPRB, 0.44606E+01_JPRB, 0.50163E+01_JPRB, 0.54755E+01_JPRB, 0.39466E+01_JPRB ,&
&  0.35793E+01_JPRB, 0.30863E+01_JPRB, 0.29898E+01_JPRB, 0.30315E+01_JPRB, 0.31379E+01_JPRB, 0.29925E+01_JPRB, 0.23374E+01_JPRB ,&
&  0.21081E+01_JPRB, 0.22860E+01_JPRB, 0.21906E+01_JPRB, 0.21751E+01_JPRB, 0.19814E+01_JPRB, 0.13073E+01_JPRB, 0.13256E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.13129E+01_JPRB, 0.13140E+01_JPRB, 0.13223E+01_JPRB, 0.13336E+01_JPRB, 0.13355E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.13062E+01_JPRB, 0.12631E+01_JPRB, 0.12158E+01_JPRB, 0.11666E+01_JPRB, 0.11216E+01_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.10405E+01_JPRB, 0.99590E+00_JPRB, 0.93879E+00_JPRB, 0.87788E+00_JPRB, 0.82258E+00_JPRB, 0.80143E+00_JPRB, 0.79344E+00_JPRB ,&
&  0.80268E+00_JPRB, 0.83188E+00_JPRB, 0.84952E+00_JPRB, 0.84736E+00_JPRB, 0.84284E+00_JPRB, 0.85727E+00_JPRB, 0.90695E+00_JPRB ,&
&  0.97107E+00_JPRB, 0.10531E+01_JPRB, 0.12178E+01_JPRB, 0.15679E+01_JPRB, 0.20139E+01_JPRB, 0.23864E+01_JPRB, 0.23758E+01_JPRB ,&
&  0.22515E+01_JPRB, 0.22576E+01_JPRB, 0.22625E+01_JPRB, 0.23042E+01_JPRB, 0.23482E+01_JPRB, 0.24473E+01_JPRB, 0.25632E+01_JPRB ,&
&  0.23078E+01_JPRB, 0.18490E+01_JPRB, 0.16554E+01_JPRB, 0.15348E+01_JPRB, 0.15194E+01_JPRB, 0.15414E+01_JPRB, 0.15752E+01_JPRB ,&
&  0.16170E+01_JPRB, 0.16839E+01_JPRB, 0.17788E+01_JPRB, 0.19232E+01_JPRB, 0.20732E+01_JPRB, 0.21602E+01_JPRB, 0.21959E+01_JPRB ,&
&  0.22097E+01_JPRB, 0.22167E+01_JPRB, 0.22576E+01_JPRB, 0.20750E+01_JPRB, 0.21528E+01_JPRB, 0.22258E+01_JPRB, 0.22849E+01_JPRB ,&
&  0.22769E+01_JPRB, 0.22107E+01_JPRB, 0.23611E+01_JPRB, 0.24495E+01_JPRB, 0.25388E+01_JPRB, 0.25381E+01_JPRB, 0.25172E+01_JPRB ,&
&  0.26252E+01_JPRB, 0.24669E+01_JPRB, 0.18147E+01_JPRB, 0.18690E+01_JPRB, 0.20921E+01_JPRB, 0.20877E+01_JPRB, 0.28448E+01_JPRB ,&
&  0.30994E+01_JPRB, 0.33266E+01_JPRB, 0.35758E+01_JPRB, 0.38224E+01_JPRB, 0.40900E+01_JPRB, 0.42495E+01_JPRB, 0.33631E+01_JPRB ,&
&  0.28117E+01_JPRB, 0.25222E+01_JPRB, 0.24426E+01_JPRB, 0.23796E+01_JPRB, 0.23127E+01_JPRB, 0.21292E+01_JPRB, 0.20186E+01_JPRB ,&
&  0.19482E+01_JPRB, 0.18881E+01_JPRB, 0.17729E+01_JPRB, 0.16014E+01_JPRB, 0.13996E+01_JPRB, 0.12488E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12810E+01_JPRB, 0.12601E+01_JPRB, 0.12616E+01_JPRB, 0.12612E+01_JPRB, 0.12650E+01_JPRB, 0.12596E+01_JPRB, 0.12442E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.11794E+01_JPRB, 0.11449E+01_JPRB, 0.11066E+01_JPRB, 0.10698E+01_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.10004E+01_JPRB, 0.94656E+00_JPRB, 0.88118E+00_JPRB, 0.82752E+00_JPRB, 0.81921E+00_JPRB, 0.82258E+00_JPRB, 0.80720E+00_JPRB ,&
&  0.80094E+00_JPRB, 0.79724E+00_JPRB, 0.81745E+00_JPRB, 0.82139E+00_JPRB, 0.82601E+00_JPRB, 0.84328E+00_JPRB, 0.86648E+00_JPRB ,&
&  0.88584E+00_JPRB, 0.90254E+00_JPRB, 0.98370E+00_JPRB, 0.11608E+01_JPRB, 0.14968E+01_JPRB, 0.19450E+01_JPRB, 0.22849E+01_JPRB ,&
&  0.24874E+01_JPRB, 0.26575E+01_JPRB, 0.28181E+01_JPRB, 0.29673E+01_JPRB, 0.28455E+01_JPRB, 0.29767E+01_JPRB, 0.33338E+01_JPRB ,&
&  0.24719E+01_JPRB, 0.16843E+01_JPRB, 0.16800E+01_JPRB, 0.15144E+01_JPRB, 0.15143E+01_JPRB, 0.15723E+01_JPRB, 0.16189E+01_JPRB ,&
&  0.16809E+01_JPRB, 0.17622E+01_JPRB, 0.18645E+01_JPRB, 0.20059E+01_JPRB, 0.21428E+01_JPRB, 0.22356E+01_JPRB, 0.22987E+01_JPRB ,&
&  0.23498E+01_JPRB, 0.24152E+01_JPRB, 0.25204E+01_JPRB, 0.24973E+01_JPRB, 0.24670E+01_JPRB, 0.25727E+01_JPRB, 0.26249E+01_JPRB ,&
&  0.26827E+01_JPRB, 0.26953E+01_JPRB, 0.24292E+01_JPRB, 0.24863E+01_JPRB, 0.27442E+01_JPRB, 0.26259E+01_JPRB, 0.26314E+01_JPRB ,&
&  0.28029E+01_JPRB, 0.24771E+01_JPRB, 0.17159E+01_JPRB, 0.17295E+01_JPRB, 0.19933E+01_JPRB, 0.23643E+01_JPRB, 0.27807E+01_JPRB ,&
&  0.30225E+01_JPRB, 0.31638E+01_JPRB, 0.31760E+01_JPRB, 0.32013E+01_JPRB, 0.30606E+01_JPRB, 0.28785E+01_JPRB, 0.26610E+01_JPRB ,&
&  0.22820E+01_JPRB, 0.21482E+01_JPRB, 0.20234E+01_JPRB, 0.19579E+01_JPRB, 0.19537E+01_JPRB, 0.18684E+01_JPRB, 0.17777E+01_JPRB ,&
&  0.18957E+01_JPRB, 0.19625E+01_JPRB, 0.21415E+01_JPRB, 0.16947E+01_JPRB, 0.13052E+01_JPRB, 0.12304E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.12440E+01_JPRB, 0.12343E+01_JPRB, 0.12237E+01_JPRB, 0.12162E+01_JPRB, 0.11939E+01_JPRB, 0.11594E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.10974E+01_JPRB, 0.10815E+01_JPRB, 0.10591E+01_JPRB, 0.10271E+01_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.93573E+00_JPRB, 0.86085E+00_JPRB, 0.82157E+00_JPRB, 0.86264E+00_JPRB, 0.83526E+00_JPRB, 0.86089E+00_JPRB, 0.85259E+00_JPRB ,&
&  0.85550E+00_JPRB, 0.85975E+00_JPRB, 0.88176E+00_JPRB, 0.91160E+00_JPRB, 0.94465E+00_JPRB, 0.97141E+00_JPRB, 0.98546E+00_JPRB ,&
&  0.99958E+00_JPRB, 0.10128E+01_JPRB, 0.10329E+01_JPRB, 0.10734E+01_JPRB, 0.11544E+01_JPRB, 0.12964E+01_JPRB, 0.15687E+01_JPRB ,&
&  0.18521E+01_JPRB, 0.20176E+01_JPRB, 0.20937E+01_JPRB, 0.21274E+01_JPRB, 0.23909E+01_JPRB, 0.29852E+01_JPRB, 0.34622E+01_JPRB ,&
&  0.20706E+01_JPRB, 0.17562E+01_JPRB, 0.15238E+01_JPRB, 0.13747E+01_JPRB, 0.14101E+01_JPRB, 0.15317E+01_JPRB, 0.15874E+01_JPRB ,&
&  0.16225E+01_JPRB, 0.16950E+01_JPRB, 0.17908E+01_JPRB, 0.19056E+01_JPRB, 0.19998E+01_JPRB, 0.20841E+01_JPRB, 0.21785E+01_JPRB ,&
&  0.22868E+01_JPRB, 0.24376E+01_JPRB, 0.25993E+01_JPRB, 0.27708E+01_JPRB, 0.27743E+01_JPRB, 0.30707E+01_JPRB, 0.31217E+01_JPRB ,&
&  0.31486E+01_JPRB, 0.30105E+01_JPRB, 0.26731E+01_JPRB, 0.27133E+01_JPRB, 0.28631E+01_JPRB, 0.28498E+01_JPRB, 0.27067E+01_JPRB ,&
&  0.27579E+01_JPRB, 0.24047E+01_JPRB, 0.19216E+01_JPRB, 0.20077E+01_JPRB, 0.21512E+01_JPRB, 0.24051E+01_JPRB, 0.24371E+01_JPRB ,&
&  0.26135E+01_JPRB, 0.26278E+01_JPRB, 0.25610E+01_JPRB, 0.24241E+01_JPRB, 0.22703E+01_JPRB, 0.21968E+01_JPRB, 0.21694E+01_JPRB ,&
&  0.19784E+01_JPRB, 0.18937E+01_JPRB, 0.18076E+01_JPRB, 0.18380E+01_JPRB, 0.19243E+01_JPRB, 0.16332E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.17656E+01_JPRB, 0.18681E+01_JPRB, 0.16370E+01_JPRB, 0.18241E+01_JPRB, 0.12427E+01_JPRB, 0.11389E+01_JPRB, 0.11591E+01_JPRB ,&
&  0.11970E+01_JPRB, 0.11670E+01_JPRB, 0.11627E+01_JPRB, 0.11509E+01_JPRB, 0.11501E+01_JPRB, 0.11322E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10382E+01_JPRB, 0.10166E+01_JPRB, 0.10146E+01_JPRB, 0.10031E+01_JPRB, 0.98504E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.85394E+00_JPRB, 0.81020E+00_JPRB, 0.86328E+00_JPRB, 0.83381E+00_JPRB, 0.83531E+00_JPRB, 0.85486E+00_JPRB, 0.87550E+00_JPRB ,&
&  0.88711E+00_JPRB, 0.90948E+00_JPRB, 0.94690E+00_JPRB, 0.10005E+01_JPRB, 0.10500E+01_JPRB, 0.10883E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.11871E+01_JPRB, 0.12007E+01_JPRB, 0.12138E+01_JPRB, 0.12491E+01_JPRB, 0.12915E+01_JPRB, 0.13612E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.14919E+01_JPRB, 0.15039E+01_JPRB, 0.15611E+01_JPRB, 0.16284E+01_JPRB, 0.18158E+01_JPRB, 0.22570E+01_JPRB ,&
&  0.17424E+01_JPRB, 0.16990E+01_JPRB, 0.15387E+01_JPRB, 0.14357E+01_JPRB, 0.14534E+01_JPRB, 0.14055E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.15097E+01_JPRB, 0.15206E+01_JPRB, 0.15650E+01_JPRB, 0.15975E+01_JPRB, 0.16446E+01_JPRB, 0.17173E+01_JPRB, 0.17950E+01_JPRB ,&
&  0.19070E+01_JPRB, 0.20498E+01_JPRB, 0.21660E+01_JPRB, 0.23534E+01_JPRB, 0.25381E+01_JPRB, 0.27232E+01_JPRB, 0.28025E+01_JPRB ,&
&  0.28508E+01_JPRB, 0.28663E+01_JPRB, 0.26782E+01_JPRB, 0.26917E+01_JPRB, 0.26855E+01_JPRB, 0.25688E+01_JPRB, 0.23144E+01_JPRB ,&
&  0.20962E+01_JPRB, 0.21148E+01_JPRB, 0.21827E+01_JPRB, 0.21417E+01_JPRB, 0.21714E+01_JPRB, 0.21633E+01_JPRB, 0.21662E+01_JPRB ,&
&  0.22086E+01_JPRB, 0.21019E+01_JPRB, 0.19800E+01_JPRB, 0.18156E+01_JPRB, 0.17432E+01_JPRB, 0.17318E+01_JPRB, 0.17546E+01_JPRB ,&
&  0.16970E+01_JPRB, 0.16400E+01_JPRB, 0.16663E+01_JPRB, 0.18464E+01_JPRB, 0.20970E+01_JPRB, 0.27236E+01_JPRB, 0.20947E+01_JPRB ,&
&  0.15553E+01_JPRB, 0.14263E+01_JPRB, 0.10116E+01_JPRB, 0.12986E+01_JPRB, 0.10506E+01_JPRB, 0.10075E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.10672E+01_JPRB, 0.11122E+01_JPRB, 0.10779E+01_JPRB, 0.10465E+01_JPRB, 0.10308E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.98306E+00_JPRB, 0.92882E+00_JPRB, 0.87097E+00_JPRB, 0.88146E+00_JPRB, 0.83208E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.72766E+00_JPRB, 0.75338E+00_JPRB, 0.85651E+00_JPRB, 0.80024E+00_JPRB, 0.80437E+00_JPRB, 0.82805E+00_JPRB, 0.86244E+00_JPRB ,&
&  0.89983E+00_JPRB, 0.94656E+00_JPRB, 0.99637E+00_JPRB, 0.10626E+01_JPRB, 0.11217E+01_JPRB, 0.11730E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12665E+01_JPRB, 0.13062E+01_JPRB, 0.13447E+01_JPRB, 0.13678E+01_JPRB, 0.13823E+01_JPRB, 0.14188E+01_JPRB, 0.14653E+01_JPRB ,&
&  0.15572E+01_JPRB, 0.16056E+01_JPRB, 0.16629E+01_JPRB, 0.17550E+01_JPRB, 0.18303E+01_JPRB, 0.19507E+01_JPRB, 0.17827E+01_JPRB ,&
&  0.19768E+01_JPRB, 0.16623E+01_JPRB, 0.15402E+01_JPRB, 0.14926E+01_JPRB, 0.14307E+01_JPRB, 0.13780E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.13976E+01_JPRB, 0.13393E+01_JPRB, 0.13423E+01_JPRB, 0.13152E+01_JPRB, 0.13334E+01_JPRB, 0.14031E+01_JPRB, 0.14439E+01_JPRB ,&
&  0.15484E+01_JPRB, 0.16808E+01_JPRB, 0.18174E+01_JPRB, 0.19177E+01_JPRB, 0.20110E+01_JPRB, 0.20102E+01_JPRB, 0.20020E+01_JPRB ,&
&  0.20399E+01_JPRB, 0.21322E+01_JPRB, 0.20881E+01_JPRB, 0.21256E+01_JPRB, 0.21059E+01_JPRB, 0.19419E+01_JPRB, 0.15773E+01_JPRB ,&
&  0.14302E+01_JPRB, 0.17218E+01_JPRB, 0.18494E+01_JPRB, 0.22833E+01_JPRB, 0.21111E+01_JPRB, 0.19251E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.18368E+01_JPRB, 0.16811E+01_JPRB, 0.15609E+01_JPRB, 0.13829E+01_JPRB, 0.13575E+01_JPRB, 0.13679E+01_JPRB, 0.14020E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.13293E+01_JPRB, 0.13513E+01_JPRB, 0.15565E+01_JPRB, 0.22115E+01_JPRB, 0.23025E+01_JPRB, 0.25113E+01_JPRB ,&
&  0.14390E+01_JPRB, 0.88382E+00_JPRB, 0.95841E+00_JPRB, 0.95116E+00_JPRB, 0.88210E+00_JPRB, 0.84075E+00_JPRB, 0.90052E+00_JPRB ,&
&  0.97994E+00_JPRB, 0.98222E+00_JPRB, 0.95999E+00_JPRB, 0.96687E+00_JPRB, 0.96471E+00_JPRB, 0.96989E+00_JPRB, 0.89272E+00_JPRB ,&
&  0.87198E+00_JPRB, 0.81307E+00_JPRB, 0.77718E+00_JPRB, 0.75932E+00_JPRB, 0.71822E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.67357E+00_JPRB, 0.74798E+00_JPRB, 0.82538E+00_JPRB, 0.76970E+00_JPRB, 0.78946E+00_JPRB, 0.82335E+00_JPRB, 0.86910E+00_JPRB ,&
&  0.91983E+00_JPRB, 0.97780E+00_JPRB, 0.10374E+01_JPRB, 0.11063E+01_JPRB, 0.11628E+01_JPRB, 0.12162E+01_JPRB, 0.12728E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.13613E+01_JPRB, 0.14121E+01_JPRB, 0.14373E+01_JPRB, 0.14728E+01_JPRB, 0.14948E+01_JPRB, 0.15282E+01_JPRB ,&
&  0.16237E+01_JPRB, 0.16951E+01_JPRB, 0.17776E+01_JPRB, 0.18638E+01_JPRB, 0.19183E+01_JPRB, 0.21338E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.19778E+01_JPRB, 0.16500E+01_JPRB, 0.15168E+01_JPRB, 0.15087E+01_JPRB, 0.14556E+01_JPRB, 0.14275E+01_JPRB, 0.14133E+01_JPRB ,&
&  0.14050E+01_JPRB, 0.14099E+01_JPRB, 0.13916E+01_JPRB, 0.13535E+01_JPRB, 0.12148E+01_JPRB, 0.12503E+01_JPRB, 0.12929E+01_JPRB ,&
&  0.13585E+01_JPRB, 0.14446E+01_JPRB, 0.15096E+01_JPRB, 0.15522E+01_JPRB, 0.15761E+01_JPRB, 0.15496E+01_JPRB, 0.15184E+01_JPRB ,&
&  0.14837E+01_JPRB, 0.14943E+01_JPRB, 0.14089E+01_JPRB, 0.14529E+01_JPRB, 0.15594E+01_JPRB, 0.13444E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.94758E+00_JPRB, 0.12979E+01_JPRB, 0.18011E+01_JPRB, 0.22190E+01_JPRB, 0.19466E+01_JPRB, 0.17822E+01_JPRB, 0.16905E+01_JPRB ,&
&  0.15422E+01_JPRB, 0.13720E+01_JPRB, 0.12360E+01_JPRB, 0.10961E+01_JPRB, 0.10460E+01_JPRB, 0.10327E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.10367E+01_JPRB, 0.10299E+01_JPRB, 0.10646E+01_JPRB, 0.11481E+01_JPRB, 0.14474E+01_JPRB, 0.16789E+01_JPRB, 0.29585E+01_JPRB ,&
&  0.15288E+01_JPRB, 0.10554E+01_JPRB, 0.15998E+01_JPRB, 0.60102E+00_JPRB, 0.68307E+00_JPRB, 0.71669E+00_JPRB, 0.75414E+00_JPRB ,&
&  0.70525E+00_JPRB, 0.89041E+00_JPRB, 0.12399E+01_JPRB, 0.12584E+01_JPRB, 0.13126E+01_JPRB, 0.13870E+01_JPRB, 0.86963E+00_JPRB ,&
&  0.72921E+00_JPRB, 0.72531E+00_JPRB, 0.73199E+00_JPRB, 0.67899E+00_JPRB, 0.66772E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.62919E+00_JPRB, 0.68964E+00_JPRB, 0.76790E+00_JPRB, 0.82854E+00_JPRB, 0.82420E+00_JPRB, 0.83804E+00_JPRB, 0.86877E+00_JPRB ,&
&  0.90427E+00_JPRB, 0.94654E+00_JPRB, 0.98832E+00_JPRB, 0.10414E+01_JPRB, 0.10804E+01_JPRB, 0.11303E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.12930E+01_JPRB, 0.13368E+01_JPRB, 0.13853E+01_JPRB, 0.14402E+01_JPRB, 0.14913E+01_JPRB, 0.15410E+01_JPRB, 0.15951E+01_JPRB ,&
&  0.16337E+01_JPRB, 0.16992E+01_JPRB, 0.17618E+01_JPRB, 0.19024E+01_JPRB, 0.20318E+01_JPRB, 0.24049E+01_JPRB, 0.17693E+01_JPRB ,&
&  0.18902E+01_JPRB, 0.16178E+01_JPRB, 0.15060E+01_JPRB, 0.15182E+01_JPRB, 0.14888E+01_JPRB, 0.14565E+01_JPRB, 0.14543E+01_JPRB ,&
&  0.15130E+01_JPRB, 0.15507E+01_JPRB, 0.15114E+01_JPRB, 0.15000E+01_JPRB, 0.13680E+01_JPRB, 0.13232E+01_JPRB, 0.14268E+01_JPRB ,&
&  0.15213E+01_JPRB, 0.16021E+01_JPRB, 0.16291E+01_JPRB, 0.16753E+01_JPRB, 0.16800E+01_JPRB, 0.16485E+01_JPRB, 0.16060E+01_JPRB ,&
&  0.15538E+01_JPRB, 0.15325E+01_JPRB, 0.15386E+01_JPRB, 0.15796E+01_JPRB, 0.15969E+01_JPRB, 0.14121E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.91269E+00_JPRB, 0.92537E+00_JPRB, 0.14955E+01_JPRB, 0.19586E+01_JPRB, 0.17421E+01_JPRB, 0.16175E+01_JPRB, 0.14432E+01_JPRB ,&
&  0.13414E+01_JPRB, 0.11871E+01_JPRB, 0.10732E+01_JPRB, 0.95294E+00_JPRB, 0.90299E+00_JPRB, 0.88458E+00_JPRB, 0.92621E+00_JPRB ,&
&  0.94975E+00_JPRB, 0.92673E+00_JPRB, 0.93226E+00_JPRB, 0.96794E+00_JPRB, 0.12442E+01_JPRB, 0.17074E+01_JPRB, 0.17833E+01_JPRB ,&
&  0.17818E+01_JPRB, 0.11928E+01_JPRB, 0.99240E+00_JPRB, 0.98639E+00_JPRB, 0.66317E+00_JPRB, 0.68508E+00_JPRB, 0.72824E+00_JPRB ,&
&  0.91604E+00_JPRB, 0.99222E+00_JPRB, 0.13248E+01_JPRB, 0.35453E+01_JPRB, 0.40357E+01_JPRB, 0.50237E+01_JPRB, 0.20466E+01_JPRB ,&
&  0.99345E+00_JPRB, 0.68761E+00_JPRB, 0.62578E+00_JPRB, 0.63008E+00_JPRB, 0.61990E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.59305E+00_JPRB, 0.64167E+00_JPRB, 0.68478E+00_JPRB, 0.74917E+00_JPRB, 0.77871E+00_JPRB, 0.79400E+00_JPRB, 0.80243E+00_JPRB ,&
&  0.79805E+00_JPRB, 0.80834E+00_JPRB, 0.83713E+00_JPRB, 0.90470E+00_JPRB, 0.97873E+00_JPRB, 0.10557E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.13549E+01_JPRB, 0.13743E+01_JPRB, 0.14166E+01_JPRB, 0.14857E+01_JPRB, 0.15509E+01_JPRB, 0.15927E+01_JPRB ,&
&  0.16357E+01_JPRB, 0.17192E+01_JPRB, 0.18192E+01_JPRB, 0.19558E+01_JPRB, 0.21324E+01_JPRB, 0.23273E+01_JPRB, 0.17132E+01_JPRB ,&
&  0.18586E+01_JPRB, 0.16255E+01_JPRB, 0.15980E+01_JPRB, 0.16119E+01_JPRB, 0.15344E+01_JPRB, 0.14650E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.15891E+01_JPRB, 0.17071E+01_JPRB, 0.17090E+01_JPRB, 0.16102E+01_JPRB, 0.16625E+01_JPRB, 0.15440E+01_JPRB, 0.17296E+01_JPRB ,&
&  0.18799E+01_JPRB, 0.19618E+01_JPRB, 0.20009E+01_JPRB, 0.20373E+01_JPRB, 0.20807E+01_JPRB, 0.21470E+01_JPRB, 0.22259E+01_JPRB ,&
&  0.22645E+01_JPRB, 0.22705E+01_JPRB, 0.22680E+01_JPRB, 0.22561E+01_JPRB, 0.20045E+01_JPRB, 0.20576E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.99141E+00_JPRB, 0.13562E+01_JPRB, 0.16960E+01_JPRB, 0.15754E+01_JPRB, 0.14430E+01_JPRB, 0.13104E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.10634E+01_JPRB, 0.97274E+00_JPRB, 0.88553E+00_JPRB, 0.81840E+00_JPRB, 0.78776E+00_JPRB, 0.78873E+00_JPRB ,&
&  0.79661E+00_JPRB, 0.79530E+00_JPRB, 0.80369E+00_JPRB, 0.86528E+00_JPRB, 0.10596E+01_JPRB, 0.15319E+01_JPRB, 0.22354E+01_JPRB ,&
&  0.25886E+01_JPRB, 0.12384E+01_JPRB, 0.93890E+00_JPRB, 0.83286E+00_JPRB, 0.78620E+00_JPRB, 0.80820E+00_JPRB, 0.84763E+00_JPRB ,&
&  0.99237E+00_JPRB, 0.12874E+01_JPRB, 0.11893E+01_JPRB, 0.13595E+01_JPRB, 0.26202E+01_JPRB, 0.28222E+01_JPRB, 0.16498E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.76819E+00_JPRB, 0.63288E+00_JPRB, 0.59424E+00_JPRB, 0.58670E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.59197E+00_JPRB, 0.60813E+00_JPRB, 0.63179E+00_JPRB, 0.66159E+00_JPRB, 0.66376E+00_JPRB, 0.67009E+00_JPRB, 0.66153E+00_JPRB ,&
&  0.66361E+00_JPRB, 0.68841E+00_JPRB, 0.74698E+00_JPRB, 0.84546E+00_JPRB, 0.92299E+00_JPRB, 0.10287E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.12715E+01_JPRB, 0.12944E+01_JPRB, 0.13362E+01_JPRB, 0.13896E+01_JPRB, 0.14548E+01_JPRB, 0.15198E+01_JPRB ,&
&  0.15994E+01_JPRB, 0.17495E+01_JPRB, 0.18907E+01_JPRB, 0.20640E+01_JPRB, 0.22345E+01_JPRB, 0.23769E+01_JPRB, 0.26915E+01_JPRB ,&
&  0.14413E+01_JPRB, 0.17804E+01_JPRB, 0.17976E+01_JPRB, 0.17585E+01_JPRB, 0.14980E+01_JPRB, 0.13958E+01_JPRB, 0.15060E+01_JPRB ,&
&  0.18029E+01_JPRB, 0.20284E+01_JPRB, 0.22397E+01_JPRB, 0.17156E+01_JPRB, 0.17782E+01_JPRB, 0.18090E+01_JPRB, 0.20214E+01_JPRB ,&
&  0.20996E+01_JPRB, 0.21703E+01_JPRB, 0.22651E+01_JPRB, 0.23718E+01_JPRB, 0.24800E+01_JPRB, 0.26196E+01_JPRB, 0.27505E+01_JPRB ,&
&  0.28401E+01_JPRB, 0.28537E+01_JPRB, 0.29066E+01_JPRB, 0.25931E+01_JPRB, 0.22506E+01_JPRB, 0.23092E+01_JPRB, 0.22167E+01_JPRB ,&
&  0.18640E+01_JPRB, 0.13279E+01_JPRB, 0.13542E+01_JPRB, 0.15891E+01_JPRB, 0.15134E+01_JPRB, 0.13962E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.93935E+00_JPRB, 0.87219E+00_JPRB, 0.80494E+00_JPRB, 0.75492E+00_JPRB, 0.74581E+00_JPRB, 0.76727E+00_JPRB ,&
&  0.77855E+00_JPRB, 0.77749E+00_JPRB, 0.77216E+00_JPRB, 0.79135E+00_JPRB, 0.86578E+00_JPRB, 0.93949E+00_JPRB, 0.10036E+01_JPRB ,&
&  0.97432E+00_JPRB, 0.94174E+00_JPRB, 0.90480E+00_JPRB, 0.91123E+00_JPRB, 0.91029E+00_JPRB, 0.89417E+00_JPRB, 0.91547E+00_JPRB ,&
&  0.91985E+00_JPRB, 0.91273E+00_JPRB, 0.10100E+01_JPRB, 0.12279E+01_JPRB, 0.12740E+01_JPRB, 0.13349E+01_JPRB, 0.11089E+01_JPRB ,&
&  0.89817E+00_JPRB, 0.76717E+00_JPRB, 0.68006E+00_JPRB, 0.62356E+00_JPRB, 0.57997E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.57924E+00_JPRB, 0.57981E+00_JPRB, 0.57994E+00_JPRB, 0.59543E+00_JPRB, 0.59878E+00_JPRB, 0.60352E+00_JPRB, 0.60311E+00_JPRB ,&
&  0.63455E+00_JPRB, 0.69409E+00_JPRB, 0.78560E+00_JPRB, 0.89142E+00_JPRB, 0.97592E+00_JPRB, 0.10785E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.11758E+01_JPRB, 0.11958E+01_JPRB, 0.12492E+01_JPRB, 0.13071E+01_JPRB, 0.13872E+01_JPRB, 0.14745E+01_JPRB ,&
&  0.15730E+01_JPRB, 0.17045E+01_JPRB, 0.18815E+01_JPRB, 0.20924E+01_JPRB, 0.23214E+01_JPRB, 0.25258E+01_JPRB, 0.28093E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.14257E+01_JPRB, 0.19108E+01_JPRB, 0.20111E+01_JPRB, 0.15512E+01_JPRB, 0.13929E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.18632E+01_JPRB, 0.19041E+01_JPRB, 0.18914E+01_JPRB, 0.18251E+01_JPRB, 0.19767E+01_JPRB, 0.21083E+01_JPRB ,&
&  0.21510E+01_JPRB, 0.22034E+01_JPRB, 0.23102E+01_JPRB, 0.24431E+01_JPRB, 0.25671E+01_JPRB, 0.26916E+01_JPRB, 0.28203E+01_JPRB ,&
&  0.29505E+01_JPRB, 0.30037E+01_JPRB, 0.30797E+01_JPRB, 0.25989E+01_JPRB, 0.26627E+01_JPRB, 0.27884E+01_JPRB, 0.31391E+01_JPRB ,&
&  0.26550E+01_JPRB, 0.16775E+01_JPRB, 0.13583E+01_JPRB, 0.15318E+01_JPRB, 0.14764E+01_JPRB, 0.13912E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.10170E+01_JPRB, 0.89353E+00_JPRB, 0.84164E+00_JPRB, 0.82440E+00_JPRB, 0.80648E+00_JPRB, 0.81291E+00_JPRB, 0.83070E+00_JPRB ,&
&  0.83359E+00_JPRB, 0.83362E+00_JPRB, 0.81965E+00_JPRB, 0.81870E+00_JPRB, 0.86964E+00_JPRB, 0.92184E+00_JPRB, 0.95757E+00_JPRB ,&
&  0.96592E+00_JPRB, 0.96890E+00_JPRB, 0.98824E+00_JPRB, 0.10158E+01_JPRB, 0.10472E+01_JPRB, 0.10547E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.94277E+00_JPRB, 0.88066E+00_JPRB, 0.85975E+00_JPRB, 0.84645E+00_JPRB, 0.83766E+00_JPRB, 0.86285E+00_JPRB, 0.81175E+00_JPRB ,&
&  0.78083E+00_JPRB, 0.76848E+00_JPRB, 0.73457E+00_JPRB, 0.66277E+00_JPRB, 0.59321E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.59924E+00_JPRB, 0.59689E+00_JPRB, 0.58940E+00_JPRB, 0.59331E+00_JPRB, 0.58768E+00_JPRB, 0.59777E+00_JPRB, 0.61579E+00_JPRB ,&
&  0.65185E+00_JPRB, 0.71661E+00_JPRB, 0.80005E+00_JPRB, 0.87787E+00_JPRB, 0.96941E+00_JPRB, 0.10467E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10914E+01_JPRB, 0.11096E+01_JPRB, 0.11640E+01_JPRB, 0.12225E+01_JPRB, 0.13037E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.15430E+01_JPRB, 0.17133E+01_JPRB, 0.19289E+01_JPRB, 0.21895E+01_JPRB, 0.25332E+01_JPRB, 0.29380E+01_JPRB, 0.34180E+01_JPRB ,&
&  0.25490E+01_JPRB, 0.13667E+01_JPRB, 0.23095E+01_JPRB, 0.24142E+01_JPRB, 0.18366E+01_JPRB, 0.16399E+01_JPRB, 0.13959E+01_JPRB ,&
&  0.13400E+01_JPRB, 0.16969E+01_JPRB, 0.17237E+01_JPRB, 0.17541E+01_JPRB, 0.17916E+01_JPRB, 0.19395E+01_JPRB, 0.20544E+01_JPRB ,&
&  0.21174E+01_JPRB, 0.21734E+01_JPRB, 0.22397E+01_JPRB, 0.23141E+01_JPRB, 0.24170E+01_JPRB, 0.25283E+01_JPRB, 0.26652E+01_JPRB ,&
&  0.28253E+01_JPRB, 0.29147E+01_JPRB, 0.29771E+01_JPRB, 0.25424E+01_JPRB, 0.26913E+01_JPRB, 0.25665E+01_JPRB, 0.24603E+01_JPRB ,&
&  0.22620E+01_JPRB, 0.14627E+01_JPRB, 0.11870E+01_JPRB, 0.14622E+01_JPRB, 0.14142E+01_JPRB, 0.11137E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.91588E+00_JPRB, 0.90468E+00_JPRB, 0.92283E+00_JPRB, 0.88432E+00_JPRB, 0.86659E+00_JPRB, 0.86102E+00_JPRB ,&
&  0.89772E+00_JPRB, 0.89132E+00_JPRB, 0.87133E+00_JPRB, 0.88625E+00_JPRB, 0.91895E+00_JPRB, 0.96791E+00_JPRB, 0.10071E+01_JPRB ,&
&  0.10453E+01_JPRB, 0.10728E+01_JPRB, 0.10998E+01_JPRB, 0.11398E+01_JPRB, 0.11708E+01_JPRB, 0.11156E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.99945E+00_JPRB, 0.96699E+00_JPRB, 0.93186E+00_JPRB, 0.90453E+00_JPRB, 0.86231E+00_JPRB, 0.86485E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.91765E+00_JPRB, 0.91859E+00_JPRB, 0.84117E+00_JPRB, 0.70483E+00_JPRB, 0.62435E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.61078E+00_JPRB, 0.58934E+00_JPRB, 0.58257E+00_JPRB, 0.59997E+00_JPRB, 0.59890E+00_JPRB, 0.60808E+00_JPRB, 0.63060E+00_JPRB ,&
&  0.66569E+00_JPRB, 0.71377E+00_JPRB, 0.75622E+00_JPRB, 0.81414E+00_JPRB, 0.89227E+00_JPRB, 0.97122E+00_JPRB, 0.99913E+00_JPRB ,&
&  0.99891E+00_JPRB, 0.10069E+01_JPRB, 0.10301E+01_JPRB, 0.10649E+01_JPRB, 0.11286E+01_JPRB, 0.12023E+01_JPRB, 0.12890E+01_JPRB ,&
&  0.14190E+01_JPRB, 0.16003E+01_JPRB, 0.17989E+01_JPRB, 0.20184E+01_JPRB, 0.23211E+01_JPRB, 0.28047E+01_JPRB, 0.35503E+01_JPRB ,&
&  0.55972E+01_JPRB, 0.60179E+01_JPRB, 0.20390E+01_JPRB, 0.25407E+01_JPRB, 0.22654E+01_JPRB, 0.19618E+01_JPRB, 0.15533E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.16450E+01_JPRB, 0.16330E+01_JPRB, 0.16723E+01_JPRB, 0.16405E+01_JPRB, 0.17640E+01_JPRB, 0.18763E+01_JPRB ,&
&  0.20089E+01_JPRB, 0.20805E+01_JPRB, 0.21073E+01_JPRB, 0.21152E+01_JPRB, 0.21568E+01_JPRB, 0.22367E+01_JPRB, 0.23761E+01_JPRB ,&
&  0.25781E+01_JPRB, 0.27800E+01_JPRB, 0.29207E+01_JPRB, 0.26565E+01_JPRB, 0.27464E+01_JPRB, 0.24255E+01_JPRB, 0.21779E+01_JPRB ,&
&  0.17393E+01_JPRB, 0.15190E+01_JPRB, 0.14820E+01_JPRB, 0.15004E+01_JPRB, 0.13248E+01_JPRB, 0.11872E+01_JPRB, 0.12683E+01_JPRB ,&
&  0.11782E+01_JPRB, 0.10546E+01_JPRB, 0.99216E+00_JPRB, 0.96315E+00_JPRB, 0.94556E+00_JPRB, 0.91833E+00_JPRB, 0.92506E+00_JPRB ,&
&  0.95586E+00_JPRB, 0.96149E+00_JPRB, 0.93896E+00_JPRB, 0.96546E+00_JPRB, 0.99490E+00_JPRB, 0.10332E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.11082E+01_JPRB, 0.11522E+01_JPRB, 0.11840E+01_JPRB, 0.12016E+01_JPRB, 0.11523E+01_JPRB, 0.10976E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.10522E+01_JPRB, 0.10442E+01_JPRB, 0.10322E+01_JPRB, 0.99138E+00_JPRB, 0.10033E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.10594E+01_JPRB, 0.95954E+00_JPRB, 0.74005E+00_JPRB, 0.64911E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.65392E+00_JPRB, 0.63386E+00_JPRB, 0.63100E+00_JPRB, 0.63989E+00_JPRB, 0.64418E+00_JPRB, 0.63297E+00_JPRB, 0.63567E+00_JPRB ,&
&  0.64916E+00_JPRB, 0.66877E+00_JPRB, 0.67197E+00_JPRB, 0.71197E+00_JPRB, 0.79949E+00_JPRB, 0.89193E+00_JPRB, 0.91576E+00_JPRB ,&
&  0.91507E+00_JPRB, 0.90961E+00_JPRB, 0.90986E+00_JPRB, 0.93365E+00_JPRB, 0.97110E+00_JPRB, 0.10537E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.13851E+01_JPRB, 0.14871E+01_JPRB, 0.15632E+01_JPRB, 0.17269E+01_JPRB, 0.20474E+01_JPRB, 0.29022E+01_JPRB ,&
&  0.44910E+01_JPRB, 0.89119E+01_JPRB, 0.23898E+01_JPRB, 0.24856E+01_JPRB, 0.25044E+01_JPRB, 0.22788E+01_JPRB, 0.20022E+01_JPRB ,&
&  0.19559E+01_JPRB, 0.17571E+01_JPRB, 0.16741E+01_JPRB, 0.16054E+01_JPRB, 0.14886E+01_JPRB, 0.15306E+01_JPRB, 0.16114E+01_JPRB ,&
&  0.17880E+01_JPRB, 0.18989E+01_JPRB, 0.19568E+01_JPRB, 0.19357E+01_JPRB, 0.19059E+01_JPRB, 0.18945E+01_JPRB, 0.19357E+01_JPRB ,&
&  0.20944E+01_JPRB, 0.23944E+01_JPRB, 0.27060E+01_JPRB, 0.26868E+01_JPRB, 0.28383E+01_JPRB, 0.25338E+01_JPRB, 0.23283E+01_JPRB ,&
&  0.19671E+01_JPRB, 0.18464E+01_JPRB, 0.18673E+01_JPRB, 0.17537E+01_JPRB, 0.14760E+01_JPRB, 0.13456E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.13095E+01_JPRB, 0.11965E+01_JPRB, 0.10764E+01_JPRB, 0.99289E+00_JPRB, 0.98419E+00_JPRB, 0.99937E+00_JPRB, 0.98151E+00_JPRB ,&
&  0.10166E+01_JPRB, 0.10615E+01_JPRB, 0.10418E+01_JPRB, 0.10810E+01_JPRB, 0.11039E+01_JPRB, 0.11185E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11457E+01_JPRB, 0.11915E+01_JPRB, 0.12188E+01_JPRB, 0.11852E+01_JPRB, 0.11053E+01_JPRB, 0.10621E+01_JPRB, 0.10855E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11758E+01_JPRB, 0.11086E+01_JPRB, 0.10550E+01_JPRB, 0.10694E+01_JPRB, 0.10549E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.11275E+01_JPRB, 0.11485E+01_JPRB, 0.83905E+00_JPRB, 0.71613E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.73804E+00_JPRB, 0.71043E+00_JPRB, 0.68763E+00_JPRB, 0.66816E+00_JPRB, 0.65582E+00_JPRB, 0.65999E+00_JPRB, 0.66519E+00_JPRB ,&
&  0.66219E+00_JPRB, 0.66593E+00_JPRB, 0.66307E+00_JPRB, 0.67076E+00_JPRB, 0.69762E+00_JPRB, 0.75150E+00_JPRB, 0.81253E+00_JPRB ,&
&  0.82378E+00_JPRB, 0.80017E+00_JPRB, 0.79236E+00_JPRB, 0.78567E+00_JPRB, 0.80850E+00_JPRB, 0.85093E+00_JPRB, 0.93860E+00_JPRB ,&
&  0.10694E+01_JPRB, 0.11929E+01_JPRB, 0.12459E+01_JPRB, 0.12466E+01_JPRB, 0.12857E+01_JPRB, 0.13978E+01_JPRB, 0.21163E+01_JPRB ,&
&  0.36776E+01_JPRB, 0.80613E+01_JPRB, 0.50511E+01_JPRB, 0.38368E+01_JPRB, 0.27848E+01_JPRB, 0.23982E+01_JPRB, 0.23195E+01_JPRB ,&
&  0.26113E+01_JPRB, 0.24692E+01_JPRB, 0.21377E+01_JPRB, 0.16120E+01_JPRB, 0.13917E+01_JPRB, 0.13183E+01_JPRB, 0.13692E+01_JPRB ,&
&  0.15195E+01_JPRB, 0.16532E+01_JPRB, 0.17386E+01_JPRB, 0.17533E+01_JPRB, 0.17113E+01_JPRB, 0.16774E+01_JPRB, 0.16581E+01_JPRB ,&
&  0.16440E+01_JPRB, 0.17562E+01_JPRB, 0.22194E+01_JPRB, 0.26717E+01_JPRB, 0.29180E+01_JPRB, 0.27298E+01_JPRB, 0.26009E+01_JPRB ,&
&  0.25575E+01_JPRB, 0.23397E+01_JPRB, 0.22104E+01_JPRB, 0.19553E+01_JPRB, 0.16080E+01_JPRB, 0.16378E+01_JPRB, 0.14462E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.12851E+01_JPRB, 0.11800E+01_JPRB, 0.10617E+01_JPRB, 0.10630E+01_JPRB, 0.10606E+01_JPRB, 0.10318E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.11128E+01_JPRB, 0.10990E+01_JPRB, 0.11130E+01_JPRB, 0.11233E+01_JPRB, 0.11440E+01_JPRB, 0.11590E+01_JPRB ,&
&  0.11694E+01_JPRB, 0.12115E+01_JPRB, 0.11510E+01_JPRB, 0.10844E+01_JPRB, 0.10576E+01_JPRB, 0.10268E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10756E+01_JPRB, 0.11619E+01_JPRB, 0.11390E+01_JPRB, 0.11365E+01_JPRB, 0.10957E+01_JPRB, 0.10891E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.10012E+01_JPRB, 0.99613E+00_JPRB, 0.91352E+00_JPRB, 0.80518E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.79817E+00_JPRB, 0.76909E+00_JPRB, 0.71002E+00_JPRB, 0.66179E+00_JPRB, 0.65251E+00_JPRB, 0.65955E+00_JPRB, 0.67318E+00_JPRB ,&
&  0.67998E+00_JPRB, 0.68262E+00_JPRB, 0.68081E+00_JPRB, 0.67505E+00_JPRB, 0.68063E+00_JPRB, 0.66476E+00_JPRB, 0.69024E+00_JPRB ,&
&  0.72553E+00_JPRB, 0.73678E+00_JPRB, 0.74642E+00_JPRB, 0.71476E+00_JPRB, 0.70886E+00_JPRB, 0.70664E+00_JPRB, 0.74918E+00_JPRB ,&
&  0.85035E+00_JPRB, 0.99375E+00_JPRB, 0.10852E+01_JPRB, 0.11073E+01_JPRB, 0.11228E+01_JPRB, 0.12210E+01_JPRB, 0.16845E+01_JPRB ,&
&  0.29960E+01_JPRB, 0.68304E+01_JPRB, 0.29581E+01_JPRB, 0.48314E+01_JPRB, 0.33441E+01_JPRB, 0.25889E+01_JPRB, 0.20520E+01_JPRB ,&
&  0.21864E+01_JPRB, 0.26366E+01_JPRB, 0.20371E+01_JPRB, 0.15787E+01_JPRB, 0.13342E+01_JPRB, 0.12581E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.12814E+01_JPRB, 0.13969E+01_JPRB, 0.15001E+01_JPRB, 0.15482E+01_JPRB, 0.15223E+01_JPRB, 0.14675E+01_JPRB, 0.14531E+01_JPRB ,&
&  0.14411E+01_JPRB, 0.14411E+01_JPRB, 0.16379E+01_JPRB, 0.19103E+01_JPRB, 0.26411E+01_JPRB, 0.29147E+01_JPRB, 0.29910E+01_JPRB ,&
&  0.30614E+01_JPRB, 0.29056E+01_JPRB, 0.25261E+01_JPRB, 0.21784E+01_JPRB, 0.18607E+01_JPRB, 0.17772E+01_JPRB, 0.15621E+01_JPRB ,&
&  0.14249E+01_JPRB, 0.13272E+01_JPRB, 0.12334E+01_JPRB, 0.11369E+01_JPRB, 0.10933E+01_JPRB, 0.10786E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.11336E+01_JPRB, 0.11246E+01_JPRB, 0.11108E+01_JPRB, 0.11122E+01_JPRB, 0.11275E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.12139E+01_JPRB, 0.11482E+01_JPRB, 0.10264E+01_JPRB, 0.10112E+01_JPRB, 0.10302E+01_JPRB, 0.10389E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11854E+01_JPRB, 0.11765E+01_JPRB, 0.11337E+01_JPRB, 0.11281E+01_JPRB, 0.11642E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.97580E+00_JPRB, 0.94554E+00_JPRB, 0.91238E+00_JPRB, 0.86122E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.80573E+00_JPRB, 0.75356E+00_JPRB, 0.69267E+00_JPRB, 0.66388E+00_JPRB, 0.66670E+00_JPRB, 0.65540E+00_JPRB, 0.66619E+00_JPRB ,&
&  0.69069E+00_JPRB, 0.69383E+00_JPRB, 0.69369E+00_JPRB, 0.68335E+00_JPRB, 0.67421E+00_JPRB, 0.65430E+00_JPRB, 0.63448E+00_JPRB ,&
&  0.65466E+00_JPRB, 0.71379E+00_JPRB, 0.73820E+00_JPRB, 0.72164E+00_JPRB, 0.70154E+00_JPRB, 0.67984E+00_JPRB, 0.67862E+00_JPRB ,&
&  0.71620E+00_JPRB, 0.80390E+00_JPRB, 0.90525E+00_JPRB, 0.96314E+00_JPRB, 0.99007E+00_JPRB, 0.10345E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.22729E+01_JPRB, 0.70460E+01_JPRB, 0.28225E+01_JPRB, 0.52435E+01_JPRB, 0.38093E+01_JPRB, 0.29460E+01_JPRB, 0.21457E+01_JPRB ,&
&  0.20003E+01_JPRB, 0.21553E+01_JPRB, 0.18770E+01_JPRB, 0.15910E+01_JPRB, 0.13850E+01_JPRB, 0.12865E+01_JPRB, 0.12554E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.12288E+01_JPRB, 0.12604E+01_JPRB, 0.12950E+01_JPRB, 0.12975E+01_JPRB, 0.12744E+01_JPRB, 0.12560E+01_JPRB ,&
&  0.12536E+01_JPRB, 0.12610E+01_JPRB, 0.13664E+01_JPRB, 0.16033E+01_JPRB, 0.20751E+01_JPRB, 0.27646E+01_JPRB, 0.35219E+01_JPRB ,&
&  0.38286E+01_JPRB, 0.35990E+01_JPRB, 0.29815E+01_JPRB, 0.25338E+01_JPRB, 0.22084E+01_JPRB, 0.20531E+01_JPRB, 0.18040E+01_JPRB ,&
&  0.16012E+01_JPRB, 0.14634E+01_JPRB, 0.13447E+01_JPRB, 0.12598E+01_JPRB, 0.11789E+01_JPRB, 0.11512E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11752E+01_JPRB, 0.11694E+01_JPRB, 0.11403E+01_JPRB, 0.11012E+01_JPRB, 0.10973E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.11715E+01_JPRB, 0.11548E+01_JPRB, 0.10510E+01_JPRB, 0.10108E+01_JPRB, 0.10282E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.12238E+01_JPRB, 0.12250E+01_JPRB, 0.12155E+01_JPRB, 0.11972E+01_JPRB, 0.12562E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.98968E+00_JPRB, 0.94997E+00_JPRB, 0.89351E+00_JPRB, 0.86570E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.82562E+00_JPRB, 0.75598E+00_JPRB, 0.71048E+00_JPRB, 0.67881E+00_JPRB, 0.67706E+00_JPRB, 0.67021E+00_JPRB, 0.66883E+00_JPRB ,&
&  0.68886E+00_JPRB, 0.70089E+00_JPRB, 0.71341E+00_JPRB, 0.70168E+00_JPRB, 0.67991E+00_JPRB, 0.67496E+00_JPRB, 0.65783E+00_JPRB ,&
&  0.64667E+00_JPRB, 0.68496E+00_JPRB, 0.72516E+00_JPRB, 0.73513E+00_JPRB, 0.71911E+00_JPRB, 0.68417E+00_JPRB, 0.66812E+00_JPRB ,&
&  0.67652E+00_JPRB, 0.70324E+00_JPRB, 0.75518E+00_JPRB, 0.81460E+00_JPRB, 0.85704E+00_JPRB, 0.88952E+00_JPRB, 0.97437E+00_JPRB ,&
&  0.14116E+01_JPRB, 0.23002E+01_JPRB, 0.40734E+01_JPRB, 0.51131E+01_JPRB, 0.40554E+01_JPRB, 0.30966E+01_JPRB, 0.24029E+01_JPRB ,&
&  0.20989E+01_JPRB, 0.19513E+01_JPRB, 0.17824E+01_JPRB, 0.15794E+01_JPRB, 0.14173E+01_JPRB, 0.12943E+01_JPRB, 0.12444E+01_JPRB ,&
&  0.11914E+01_JPRB, 0.11497E+01_JPRB, 0.11158E+01_JPRB, 0.11014E+01_JPRB, 0.10914E+01_JPRB, 0.10844E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.11111E+01_JPRB, 0.11623E+01_JPRB, 0.13421E+01_JPRB, 0.18342E+01_JPRB, 0.25202E+01_JPRB, 0.35851E+01_JPRB ,&
&  0.43554E+01_JPRB, 0.39710E+01_JPRB, 0.33549E+01_JPRB, 0.28622E+01_JPRB, 0.25090E+01_JPRB, 0.23109E+01_JPRB, 0.21038E+01_JPRB ,&
&  0.19150E+01_JPRB, 0.17186E+01_JPRB, 0.15523E+01_JPRB, 0.14389E+01_JPRB, 0.13075E+01_JPRB, 0.12404E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.11968E+01_JPRB, 0.11960E+01_JPRB, 0.12264E+01_JPRB, 0.11817E+01_JPRB, 0.11334E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.11150E+01_JPRB, 0.11280E+01_JPRB, 0.10751E+01_JPRB, 0.10575E+01_JPRB, 0.10667E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.11428E+01_JPRB, 0.12127E+01_JPRB, 0.12493E+01_JPRB, 0.12781E+01_JPRB, 0.12091E+01_JPRB, 0.12786E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10166E+01_JPRB, 0.98143E+00_JPRB, 0.93748E+00_JPRB, 0.90370E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.92647E+00_JPRB, 0.84531E+00_JPRB, 0.77450E+00_JPRB, 0.73177E+00_JPRB, 0.69901E+00_JPRB, 0.68433E+00_JPRB, 0.67679E+00_JPRB ,&
&  0.68577E+00_JPRB, 0.69518E+00_JPRB, 0.71046E+00_JPRB, 0.70766E+00_JPRB, 0.68806E+00_JPRB, 0.68901E+00_JPRB, 0.70224E+00_JPRB ,&
&  0.69448E+00_JPRB, 0.70137E+00_JPRB, 0.71712E+00_JPRB, 0.70901E+00_JPRB, 0.69348E+00_JPRB, 0.67527E+00_JPRB, 0.65880E+00_JPRB ,&
&  0.66189E+00_JPRB, 0.66848E+00_JPRB, 0.68562E+00_JPRB, 0.71912E+00_JPRB, 0.76558E+00_JPRB, 0.80484E+00_JPRB, 0.83844E+00_JPRB ,&
&  0.99573E+00_JPRB, 0.25422E+01_JPRB, 0.37485E+01_JPRB, 0.44158E+01_JPRB, 0.39864E+01_JPRB, 0.30734E+01_JPRB, 0.25462E+01_JPRB ,&
&  0.20593E+01_JPRB, 0.18226E+01_JPRB, 0.16662E+01_JPRB, 0.15261E+01_JPRB, 0.13796E+01_JPRB, 0.12868E+01_JPRB, 0.12384E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.11237E+01_JPRB, 0.10509E+01_JPRB, 0.98972E+00_JPRB, 0.96249E+00_JPRB, 0.95502E+00_JPRB, 0.95734E+00_JPRB ,&
&  0.96713E+00_JPRB, 0.98412E+00_JPRB, 0.10187E+01_JPRB, 0.10946E+01_JPRB, 0.14916E+01_JPRB, 0.17274E+01_JPRB, 0.28674E+01_JPRB ,&
&  0.34817E+01_JPRB, 0.30455E+01_JPRB, 0.27625E+01_JPRB, 0.25631E+01_JPRB, 0.23556E+01_JPRB, 0.21677E+01_JPRB, 0.20555E+01_JPRB ,&
&  0.19409E+01_JPRB, 0.18044E+01_JPRB, 0.16868E+01_JPRB, 0.15997E+01_JPRB, 0.15065E+01_JPRB, 0.14320E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.12983E+01_JPRB, 0.12521E+01_JPRB, 0.12306E+01_JPRB, 0.12486E+01_JPRB, 0.12626E+01_JPRB, 0.12149E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.10851E+01_JPRB, 0.11029E+01_JPRB, 0.10729E+01_JPRB, 0.10782E+01_JPRB, 0.11038E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.11164E+01_JPRB, 0.11729E+01_JPRB, 0.12401E+01_JPRB, 0.12799E+01_JPRB, 0.12759E+01_JPRB, 0.13339E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.10428E+01_JPRB, 0.98763E+00_JPRB, 0.94501E+00_JPRB, 0.96502E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.98047E+00_JPRB, 0.92668E+00_JPRB, 0.86362E+00_JPRB, 0.81680E+00_JPRB, 0.77711E+00_JPRB, 0.74853E+00_JPRB, 0.74048E+00_JPRB ,&
&  0.73413E+00_JPRB, 0.72338E+00_JPRB, 0.72800E+00_JPRB, 0.72542E+00_JPRB, 0.72093E+00_JPRB, 0.71422E+00_JPRB, 0.72559E+00_JPRB ,&
&  0.73464E+00_JPRB, 0.73226E+00_JPRB, 0.72639E+00_JPRB, 0.71545E+00_JPRB, 0.70209E+00_JPRB, 0.68519E+00_JPRB, 0.67396E+00_JPRB ,&
&  0.66684E+00_JPRB, 0.66737E+00_JPRB, 0.67441E+00_JPRB, 0.68956E+00_JPRB, 0.72115E+00_JPRB, 0.76347E+00_JPRB, 0.80416E+00_JPRB ,&
&  0.84778E+00_JPRB, 0.15232E+01_JPRB, 0.21809E+01_JPRB, 0.33508E+01_JPRB, 0.36729E+01_JPRB, 0.30630E+01_JPRB, 0.25142E+01_JPRB ,&
&  0.20809E+01_JPRB, 0.17888E+01_JPRB, 0.15982E+01_JPRB, 0.14603E+01_JPRB, 0.13222E+01_JPRB, 0.12406E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.10710E+01_JPRB, 0.10023E+01_JPRB, 0.94754E+00_JPRB, 0.92164E+00_JPRB, 0.91583E+00_JPRB, 0.90648E+00_JPRB ,&
&  0.89790E+00_JPRB, 0.90425E+00_JPRB, 0.91187E+00_JPRB, 0.93179E+00_JPRB, 0.11250E+01_JPRB, 0.13408E+01_JPRB, 0.18147E+01_JPRB ,&
&  0.18219E+01_JPRB, 0.17334E+01_JPRB, 0.17152E+01_JPRB, 0.16954E+01_JPRB, 0.17123E+01_JPRB, 0.17203E+01_JPRB, 0.17459E+01_JPRB ,&
&  0.17674E+01_JPRB, 0.17501E+01_JPRB, 0.17042E+01_JPRB, 0.16665E+01_JPRB, 0.16222E+01_JPRB, 0.15789E+01_JPRB, 0.14945E+01_JPRB ,&
&  0.14249E+01_JPRB, 0.13677E+01_JPRB, 0.13301E+01_JPRB, 0.12889E+01_JPRB, 0.12768E+01_JPRB, 0.12494E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.10826E+01_JPRB, 0.10862E+01_JPRB, 0.10898E+01_JPRB, 0.10840E+01_JPRB, 0.10748E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11743E+01_JPRB, 0.12067E+01_JPRB, 0.12798E+01_JPRB, 0.12817E+01_JPRB, 0.13136E+01_JPRB, 0.12451E+01_JPRB ,&
&  0.11673E+01_JPRB, 0.10817E+01_JPRB, 0.10017E+01_JPRB, 0.95841E+00_JPRB, 0.10015E+01_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.10810E+01_JPRB, 0.10234E+01_JPRB, 0.96662E+00_JPRB, 0.89647E+00_JPRB, 0.84030E+00_JPRB, 0.81349E+00_JPRB, 0.79420E+00_JPRB ,&
&  0.77110E+00_JPRB, 0.75008E+00_JPRB, 0.74596E+00_JPRB, 0.74959E+00_JPRB, 0.75162E+00_JPRB, 0.75074E+00_JPRB, 0.74263E+00_JPRB ,&
&  0.74012E+00_JPRB, 0.74368E+00_JPRB, 0.73642E+00_JPRB, 0.72404E+00_JPRB, 0.71042E+00_JPRB, 0.69885E+00_JPRB, 0.69004E+00_JPRB ,&
&  0.68497E+00_JPRB, 0.68771E+00_JPRB, 0.68980E+00_JPRB, 0.70100E+00_JPRB, 0.72224E+00_JPRB, 0.75133E+00_JPRB, 0.79140E+00_JPRB ,&
&  0.83222E+00_JPRB, 0.91009E+00_JPRB, 0.13554E+01_JPRB, 0.19932E+01_JPRB, 0.25358E+01_JPRB, 0.25758E+01_JPRB, 0.23059E+01_JPRB ,&
&  0.20535E+01_JPRB, 0.18006E+01_JPRB, 0.16042E+01_JPRB, 0.14415E+01_JPRB, 0.13067E+01_JPRB, 0.12038E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10127E+01_JPRB, 0.96518E+00_JPRB, 0.92209E+00_JPRB, 0.90042E+00_JPRB, 0.88339E+00_JPRB, 0.86673E+00_JPRB ,&
&  0.85261E+00_JPRB, 0.84484E+00_JPRB, 0.84133E+00_JPRB, 0.84256E+00_JPRB, 0.86587E+00_JPRB, 0.93720E+00_JPRB, 0.10707E+01_JPRB ,&
&  0.11373E+01_JPRB, 0.11758E+01_JPRB, 0.11817E+01_JPRB, 0.11770E+01_JPRB, 0.11943E+01_JPRB, 0.12377E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.14014E+01_JPRB, 0.14761E+01_JPRB, 0.15317E+01_JPRB, 0.15675E+01_JPRB, 0.15668E+01_JPRB, 0.15399E+01_JPRB, 0.14956E+01_JPRB ,&
&  0.14490E+01_JPRB, 0.13821E+01_JPRB, 0.13203E+01_JPRB, 0.12720E+01_JPRB, 0.12402E+01_JPRB, 0.12177E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.10918E+01_JPRB, 0.10704E+01_JPRB, 0.10690E+01_JPRB, 0.10693E+01_JPRB, 0.10592E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10955E+01_JPRB, 0.11007E+01_JPRB, 0.12346E+01_JPRB, 0.12745E+01_JPRB, 0.12312E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.10788E+01_JPRB, 0.10251E+01_JPRB, 0.98495E+00_JPRB, 0.10797E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.10039E+01_JPRB, 0.99061E+00_JPRB, 0.94809E+00_JPRB, 0.89382E+00_JPRB, 0.84257E+00_JPRB, 0.80007E+00_JPRB, 0.77586E+00_JPRB ,&
&  0.75740E+00_JPRB, 0.74494E+00_JPRB, 0.74819E+00_JPRB, 0.75972E+00_JPRB, 0.77351E+00_JPRB, 0.77686E+00_JPRB, 0.77128E+00_JPRB ,&
&  0.75956E+00_JPRB, 0.74154E+00_JPRB, 0.72948E+00_JPRB, 0.72177E+00_JPRB, 0.71126E+00_JPRB, 0.70640E+00_JPRB, 0.70414E+00_JPRB ,&
&  0.70705E+00_JPRB, 0.71210E+00_JPRB, 0.71509E+00_JPRB, 0.72450E+00_JPRB, 0.74052E+00_JPRB, 0.76600E+00_JPRB, 0.79599E+00_JPRB ,&
&  0.81490E+00_JPRB, 0.74467E+00_JPRB, 0.91818E+00_JPRB, 0.12973E+01_JPRB, 0.17291E+01_JPRB, 0.19552E+01_JPRB, 0.19486E+01_JPRB ,&
&  0.18233E+01_JPRB, 0.16947E+01_JPRB, 0.15578E+01_JPRB, 0.14043E+01_JPRB, 0.12690E+01_JPRB, 0.11604E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.99478E+00_JPRB, 0.94382E+00_JPRB, 0.91315E+00_JPRB, 0.89068E+00_JPRB, 0.86984E+00_JPRB, 0.83808E+00_JPRB, 0.81542E+00_JPRB ,&
&  0.80580E+00_JPRB, 0.79980E+00_JPRB, 0.79796E+00_JPRB, 0.79766E+00_JPRB, 0.81005E+00_JPRB, 0.83318E+00_JPRB, 0.85685E+00_JPRB ,&
&  0.87809E+00_JPRB, 0.90222E+00_JPRB, 0.91468E+00_JPRB, 0.91514E+00_JPRB, 0.91484E+00_JPRB, 0.95415E+00_JPRB, 0.99218E+00_JPRB ,&
&  0.10452E+01_JPRB, 0.11270E+01_JPRB, 0.12245E+01_JPRB, 0.12848E+01_JPRB, 0.12960E+01_JPRB, 0.12695E+01_JPRB, 0.12208E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.11483E+01_JPRB, 0.11109E+01_JPRB, 0.10818E+01_JPRB, 0.10650E+01_JPRB, 0.10616E+01_JPRB, 0.10592E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.10417E+01_JPRB, 0.10330E+01_JPRB, 0.10452E+01_JPRB, 0.10605E+01_JPRB, 0.10617E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10386E+01_JPRB, 0.10325E+01_JPRB, 0.10354E+01_JPRB, 0.11090E+01_JPRB, 0.10987E+01_JPRB, 0.10959E+01_JPRB, 0.10731E+01_JPRB ,&
&  0.10506E+01_JPRB, 0.10175E+01_JPRB, 0.97567E+00_JPRB, 0.94072E+00_JPRB, 0.93569E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.88130E+00_JPRB, 0.91405E+00_JPRB, 0.91360E+00_JPRB, 0.87927E+00_JPRB, 0.84092E+00_JPRB, 0.81417E+00_JPRB, 0.79582E+00_JPRB ,&
&  0.78858E+00_JPRB, 0.78053E+00_JPRB, 0.78196E+00_JPRB, 0.79266E+00_JPRB, 0.80459E+00_JPRB, 0.80716E+00_JPRB, 0.79003E+00_JPRB ,&
&  0.76726E+00_JPRB, 0.74850E+00_JPRB, 0.73814E+00_JPRB, 0.73315E+00_JPRB, 0.72736E+00_JPRB, 0.72447E+00_JPRB, 0.73270E+00_JPRB ,&
&  0.74274E+00_JPRB, 0.75506E+00_JPRB, 0.76862E+00_JPRB, 0.78025E+00_JPRB, 0.78961E+00_JPRB, 0.79888E+00_JPRB, 0.82108E+00_JPRB ,&
&  0.83868E+00_JPRB, 0.67893E+00_JPRB, 0.73607E+00_JPRB, 0.94560E+00_JPRB, 0.12033E+01_JPRB, 0.14969E+01_JPRB, 0.16210E+01_JPRB ,&
&  0.15880E+01_JPRB, 0.15042E+01_JPRB, 0.14022E+01_JPRB, 0.12923E+01_JPRB, 0.11833E+01_JPRB, 0.10916E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.95350E+00_JPRB, 0.90944E+00_JPRB, 0.87974E+00_JPRB, 0.85121E+00_JPRB, 0.83070E+00_JPRB, 0.81281E+00_JPRB, 0.79709E+00_JPRB ,&
&  0.78349E+00_JPRB, 0.77020E+00_JPRB, 0.76549E+00_JPRB, 0.76636E+00_JPRB, 0.76217E+00_JPRB, 0.76607E+00_JPRB, 0.76440E+00_JPRB ,&
&  0.77802E+00_JPRB, 0.79810E+00_JPRB, 0.80538E+00_JPRB, 0.80455E+00_JPRB, 0.80747E+00_JPRB, 0.82030E+00_JPRB, 0.83947E+00_JPRB ,&
&  0.86021E+00_JPRB, 0.90490E+00_JPRB, 0.95237E+00_JPRB, 0.99914E+00_JPRB, 0.10286E+01_JPRB, 0.10336E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.10235E+01_JPRB, 0.10147E+01_JPRB, 0.10003E+01_JPRB, 0.98081E+00_JPRB, 0.95808E+00_JPRB, 0.94439E+00_JPRB, 0.94429E+00_JPRB ,&
&  0.95358E+00_JPRB, 0.97129E+00_JPRB, 0.98422E+00_JPRB, 0.10068E+01_JPRB, 0.10165E+01_JPRB, 0.10096E+01_JPRB, 0.10021E+01_JPRB ,&
&  0.99305E+00_JPRB, 0.98009E+00_JPRB, 0.96766E+00_JPRB, 0.97266E+00_JPRB, 0.10358E+01_JPRB, 0.10375E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.99618E+00_JPRB, 0.95397E+00_JPRB, 0.92831E+00_JPRB, 0.75995E+00_JPRB, 0.84110E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.83251E+00_JPRB, 0.86244E+00_JPRB, 0.86997E+00_JPRB, 0.85735E+00_JPRB, 0.83328E+00_JPRB, 0.81905E+00_JPRB, 0.80964E+00_JPRB ,&
&  0.80339E+00_JPRB, 0.79851E+00_JPRB, 0.80700E+00_JPRB, 0.81299E+00_JPRB, 0.81405E+00_JPRB, 0.80488E+00_JPRB, 0.79504E+00_JPRB ,&
&  0.78005E+00_JPRB, 0.76176E+00_JPRB, 0.74810E+00_JPRB, 0.73617E+00_JPRB, 0.73416E+00_JPRB, 0.74167E+00_JPRB, 0.75750E+00_JPRB ,&
&  0.77824E+00_JPRB, 0.79302E+00_JPRB, 0.80545E+00_JPRB, 0.81882E+00_JPRB, 0.82240E+00_JPRB, 0.82399E+00_JPRB, 0.82882E+00_JPRB ,&
&  0.83778E+00_JPRB, 0.65698E+00_JPRB, 0.71136E+00_JPRB, 0.79654E+00_JPRB, 0.90130E+00_JPRB, 0.10652E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.11941E+01_JPRB, 0.11411E+01_JPRB, 0.10825E+01_JPRB, 0.10238E+01_JPRB, 0.97264E+00_JPRB, 0.92121E+00_JPRB ,&
&  0.88040E+00_JPRB, 0.85463E+00_JPRB, 0.84144E+00_JPRB, 0.82827E+00_JPRB, 0.81327E+00_JPRB, 0.79519E+00_JPRB, 0.77524E+00_JPRB ,&
&  0.75508E+00_JPRB, 0.74178E+00_JPRB, 0.73154E+00_JPRB, 0.72390E+00_JPRB, 0.72284E+00_JPRB, 0.72556E+00_JPRB, 0.72783E+00_JPRB ,&
&  0.73589E+00_JPRB, 0.74261E+00_JPRB, 0.74494E+00_JPRB, 0.74285E+00_JPRB, 0.73982E+00_JPRB, 0.74006E+00_JPRB, 0.75508E+00_JPRB ,&
&  0.76567E+00_JPRB, 0.78524E+00_JPRB, 0.81077E+00_JPRB, 0.85059E+00_JPRB, 0.88486E+00_JPRB, 0.89969E+00_JPRB, 0.90576E+00_JPRB ,&
&  0.90806E+00_JPRB, 0.89977E+00_JPRB, 0.88229E+00_JPRB, 0.86368E+00_JPRB, 0.84955E+00_JPRB, 0.84449E+00_JPRB, 0.83885E+00_JPRB ,&
&  0.84136E+00_JPRB, 0.85032E+00_JPRB, 0.85799E+00_JPRB, 0.87390E+00_JPRB, 0.89467E+00_JPRB, 0.90327E+00_JPRB, 0.89068E+00_JPRB ,&
&  0.88071E+00_JPRB, 0.87866E+00_JPRB, 0.87546E+00_JPRB, 0.91479E+00_JPRB, 0.92996E+00_JPRB, 0.93247E+00_JPRB, 0.91857E+00_JPRB ,&
&  0.90501E+00_JPRB, 0.89168E+00_JPRB, 0.68258E+00_JPRB, 0.77797E+00_JPRB, 0.79976E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.82285E+00_JPRB, 0.81773E+00_JPRB, 0.81007E+00_JPRB, 0.81190E+00_JPRB, 0.81403E+00_JPRB, 0.81143E+00_JPRB, 0.80938E+00_JPRB ,&
&  0.80600E+00_JPRB, 0.80168E+00_JPRB, 0.80931E+00_JPRB, 0.81660E+00_JPRB, 0.81756E+00_JPRB, 0.81111E+00_JPRB, 0.80426E+00_JPRB ,&
&  0.79900E+00_JPRB, 0.79018E+00_JPRB, 0.78111E+00_JPRB, 0.77704E+00_JPRB, 0.77908E+00_JPRB, 0.79256E+00_JPRB, 0.81337E+00_JPRB ,&
&  0.82893E+00_JPRB, 0.82808E+00_JPRB, 0.83177E+00_JPRB, 0.84323E+00_JPRB, 0.84644E+00_JPRB, 0.84892E+00_JPRB, 0.85655E+00_JPRB ,&
&  0.73829E+00_JPRB, 0.61760E+00_JPRB, 0.68722E+00_JPRB, 0.75413E+00_JPRB, 0.81280E+00_JPRB, 0.88544E+00_JPRB, 0.97999E+00_JPRB ,&
&  0.10184E+01_JPRB, 0.10034E+01_JPRB, 0.98186E+00_JPRB, 0.95137E+00_JPRB, 0.91657E+00_JPRB, 0.89132E+00_JPRB, 0.87103E+00_JPRB ,&
&  0.86451E+00_JPRB, 0.85195E+00_JPRB, 0.83256E+00_JPRB, 0.81637E+00_JPRB, 0.80109E+00_JPRB, 0.78130E+00_JPRB, 0.76212E+00_JPRB ,&
&  0.74251E+00_JPRB, 0.72554E+00_JPRB, 0.71845E+00_JPRB, 0.71368E+00_JPRB, 0.71398E+00_JPRB, 0.71245E+00_JPRB, 0.70645E+00_JPRB ,&
&  0.70209E+00_JPRB, 0.70243E+00_JPRB, 0.71051E+00_JPRB, 0.71773E+00_JPRB, 0.71907E+00_JPRB, 0.71628E+00_JPRB, 0.72396E+00_JPRB ,&
&  0.72976E+00_JPRB, 0.74213E+00_JPRB, 0.75859E+00_JPRB, 0.78596E+00_JPRB, 0.80786E+00_JPRB, 0.81636E+00_JPRB, 0.82158E+00_JPRB ,&
&  0.82908E+00_JPRB, 0.82441E+00_JPRB, 0.80879E+00_JPRB, 0.79711E+00_JPRB, 0.79084E+00_JPRB, 0.78260E+00_JPRB, 0.77881E+00_JPRB ,&
&  0.78323E+00_JPRB, 0.78634E+00_JPRB, 0.78337E+00_JPRB, 0.78802E+00_JPRB, 0.79897E+00_JPRB, 0.81254E+00_JPRB, 0.82384E+00_JPRB ,&
&  0.81278E+00_JPRB, 0.81922E+00_JPRB, 0.81989E+00_JPRB, 0.83711E+00_JPRB, 0.85787E+00_JPRB, 0.85334E+00_JPRB, 0.84610E+00_JPRB ,&
&  0.83726E+00_JPRB, 0.81272E+00_JPRB, 0.81167E+00_JPRB, 0.81121E+00_JPRB, 0.81358E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.77235E+00_JPRB, 0.76404E+00_JPRB, 0.75834E+00_JPRB, 0.76702E+00_JPRB, 0.78224E+00_JPRB, 0.79338E+00_JPRB, 0.79332E+00_JPRB ,&
&  0.79297E+00_JPRB, 0.79730E+00_JPRB, 0.80539E+00_JPRB, 0.80767E+00_JPRB, 0.80089E+00_JPRB, 0.78440E+00_JPRB, 0.77591E+00_JPRB ,&
&  0.77369E+00_JPRB, 0.77446E+00_JPRB, 0.77417E+00_JPRB, 0.77825E+00_JPRB, 0.79749E+00_JPRB, 0.82771E+00_JPRB, 0.85633E+00_JPRB ,&
&  0.87037E+00_JPRB, 0.86480E+00_JPRB, 0.85856E+00_JPRB, 0.85361E+00_JPRB, 0.85262E+00_JPRB, 0.85780E+00_JPRB, 0.85881E+00_JPRB ,&
&  0.85203E+00_JPRB, 0.70851E+00_JPRB, 0.76815E+00_JPRB, 0.79279E+00_JPRB, 0.82252E+00_JPRB, 0.85801E+00_JPRB, 0.91251E+00_JPRB ,&
&  0.93263E+00_JPRB, 0.92061E+00_JPRB, 0.89386E+00_JPRB, 0.87405E+00_JPRB, 0.85599E+00_JPRB, 0.84750E+00_JPRB, 0.84112E+00_JPRB ,&
&  0.83807E+00_JPRB, 0.83439E+00_JPRB, 0.81181E+00_JPRB, 0.77577E+00_JPRB, 0.75468E+00_JPRB, 0.73965E+00_JPRB, 0.72623E+00_JPRB ,&
&  0.71357E+00_JPRB, 0.69807E+00_JPRB, 0.68326E+00_JPRB, 0.67113E+00_JPRB, 0.67106E+00_JPRB, 0.66793E+00_JPRB, 0.65424E+00_JPRB ,&
&  0.64809E+00_JPRB, 0.64510E+00_JPRB, 0.65196E+00_JPRB, 0.66180E+00_JPRB, 0.66547E+00_JPRB, 0.66022E+00_JPRB, 0.66916E+00_JPRB ,&
&  0.69016E+00_JPRB, 0.70371E+00_JPRB, 0.71585E+00_JPRB, 0.73183E+00_JPRB, 0.74067E+00_JPRB, 0.74167E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.75206E+00_JPRB, 0.75153E+00_JPRB, 0.74413E+00_JPRB, 0.73819E+00_JPRB, 0.73949E+00_JPRB, 0.74024E+00_JPRB, 0.74977E+00_JPRB ,&
&  0.75878E+00_JPRB, 0.76169E+00_JPRB, 0.75742E+00_JPRB, 0.75976E+00_JPRB, 0.76546E+00_JPRB, 0.77557E+00_JPRB, 0.78253E+00_JPRB ,&
&  0.78989E+00_JPRB, 0.78711E+00_JPRB, 0.77514E+00_JPRB, 0.77896E+00_JPRB, 0.78941E+00_JPRB, 0.78967E+00_JPRB, 0.77844E+00_JPRB ,&
&  0.76778E+00_JPRB, 0.75865E+00_JPRB, 0.76086E+00_JPRB, 0.76203E+00_JPRB, 0.76776E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.75782E+00_JPRB, 0.75413E+00_JPRB, 0.75317E+00_JPRB, 0.76450E+00_JPRB, 0.78215E+00_JPRB, 0.78611E+00_JPRB, 0.78743E+00_JPRB ,&
&  0.79966E+00_JPRB, 0.80895E+00_JPRB, 0.80858E+00_JPRB, 0.80707E+00_JPRB, 0.80121E+00_JPRB, 0.78737E+00_JPRB, 0.77670E+00_JPRB ,&
&  0.78183E+00_JPRB, 0.78722E+00_JPRB, 0.78914E+00_JPRB, 0.81281E+00_JPRB, 0.84417E+00_JPRB, 0.88147E+00_JPRB, 0.90672E+00_JPRB ,&
&  0.91556E+00_JPRB, 0.90811E+00_JPRB, 0.90440E+00_JPRB, 0.89925E+00_JPRB, 0.89402E+00_JPRB, 0.89588E+00_JPRB, 0.89036E+00_JPRB ,&
&  0.88028E+00_JPRB, 0.79686E+00_JPRB, 0.82915E+00_JPRB, 0.83920E+00_JPRB, 0.85605E+00_JPRB, 0.88283E+00_JPRB, 0.92087E+00_JPRB ,&
&  0.92802E+00_JPRB, 0.92130E+00_JPRB, 0.90571E+00_JPRB, 0.87898E+00_JPRB, 0.86045E+00_JPRB, 0.85045E+00_JPRB, 0.83266E+00_JPRB ,&
&  0.80478E+00_JPRB, 0.76755E+00_JPRB, 0.73434E+00_JPRB, 0.71165E+00_JPRB, 0.70086E+00_JPRB, 0.69691E+00_JPRB, 0.69041E+00_JPRB ,&
&  0.68643E+00_JPRB, 0.68299E+00_JPRB, 0.67076E+00_JPRB, 0.65969E+00_JPRB, 0.64807E+00_JPRB, 0.63974E+00_JPRB, 0.62556E+00_JPRB ,&
&  0.61693E+00_JPRB, 0.60549E+00_JPRB, 0.60419E+00_JPRB, 0.62346E+00_JPRB, 0.64391E+00_JPRB, 0.64830E+00_JPRB, 0.66133E+00_JPRB ,&
&  0.67907E+00_JPRB, 0.68681E+00_JPRB, 0.68868E+00_JPRB, 0.68619E+00_JPRB, 0.67932E+00_JPRB, 0.67925E+00_JPRB, 0.68756E+00_JPRB ,&
&  0.69496E+00_JPRB, 0.70527E+00_JPRB, 0.70905E+00_JPRB, 0.71393E+00_JPRB, 0.72068E+00_JPRB, 0.72577E+00_JPRB, 0.73666E+00_JPRB ,&
&  0.74227E+00_JPRB, 0.74639E+00_JPRB, 0.75785E+00_JPRB, 0.76928E+00_JPRB, 0.77418E+00_JPRB, 0.77678E+00_JPRB, 0.78622E+00_JPRB ,&
&  0.79007E+00_JPRB, 0.78841E+00_JPRB, 0.78021E+00_JPRB, 0.76878E+00_JPRB, 0.76178E+00_JPRB, 0.75001E+00_JPRB, 0.73113E+00_JPRB ,&
&  0.71372E+00_JPRB, 0.71157E+00_JPRB, 0.72096E+00_JPRB, 0.73112E+00_JPRB, 0.74844E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.74288E+00_JPRB, 0.75449E+00_JPRB, 0.76777E+00_JPRB, 0.78063E+00_JPRB, 0.79397E+00_JPRB, 0.79361E+00_JPRB, 0.79522E+00_JPRB ,&
&  0.81504E+00_JPRB, 0.81859E+00_JPRB, 0.81827E+00_JPRB, 0.80980E+00_JPRB, 0.80358E+00_JPRB, 0.80340E+00_JPRB, 0.79433E+00_JPRB ,&
&  0.78969E+00_JPRB, 0.79576E+00_JPRB, 0.81363E+00_JPRB, 0.84120E+00_JPRB, 0.86811E+00_JPRB, 0.90373E+00_JPRB, 0.93005E+00_JPRB ,&
&  0.93399E+00_JPRB, 0.92300E+00_JPRB, 0.92325E+00_JPRB, 0.92668E+00_JPRB, 0.92691E+00_JPRB, 0.91008E+00_JPRB, 0.90597E+00_JPRB ,&
&  0.88554E+00_JPRB, 0.89944E+00_JPRB, 0.76926E+00_JPRB, 0.83999E+00_JPRB, 0.88469E+00_JPRB, 0.92187E+00_JPRB, 0.94009E+00_JPRB ,&
&  0.93453E+00_JPRB, 0.92125E+00_JPRB, 0.89339E+00_JPRB, 0.86261E+00_JPRB, 0.85203E+00_JPRB, 0.83229E+00_JPRB, 0.80909E+00_JPRB ,&
&  0.76006E+00_JPRB, 0.69718E+00_JPRB, 0.66513E+00_JPRB, 0.65322E+00_JPRB, 0.64713E+00_JPRB, 0.64746E+00_JPRB, 0.65155E+00_JPRB ,&
&  0.65409E+00_JPRB, 0.65353E+00_JPRB, 0.64132E+00_JPRB, 0.62358E+00_JPRB, 0.60191E+00_JPRB, 0.59010E+00_JPRB, 0.58709E+00_JPRB ,&
&  0.58349E+00_JPRB, 0.57015E+00_JPRB, 0.57307E+00_JPRB, 0.60251E+00_JPRB, 0.62689E+00_JPRB, 0.64460E+00_JPRB, 0.65289E+00_JPRB ,&
&  0.65927E+00_JPRB, 0.65394E+00_JPRB, 0.65077E+00_JPRB, 0.64211E+00_JPRB, 0.61904E+00_JPRB, 0.61300E+00_JPRB, 0.63414E+00_JPRB ,&
&  0.65787E+00_JPRB, 0.67146E+00_JPRB, 0.67970E+00_JPRB, 0.69205E+00_JPRB, 0.70517E+00_JPRB, 0.71275E+00_JPRB, 0.71915E+00_JPRB ,&
&  0.72373E+00_JPRB, 0.72835E+00_JPRB, 0.74187E+00_JPRB, 0.75690E+00_JPRB, 0.76419E+00_JPRB, 0.76570E+00_JPRB, 0.76582E+00_JPRB ,&
&  0.76304E+00_JPRB, 0.75551E+00_JPRB, 0.74670E+00_JPRB, 0.73843E+00_JPRB, 0.71913E+00_JPRB, 0.69542E+00_JPRB, 0.67773E+00_JPRB ,&
&  0.66862E+00_JPRB, 0.67851E+00_JPRB, 0.68625E+00_JPRB, 0.69886E+00_JPRB, 0.72571E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.78010E+00_JPRB, 0.79409E+00_JPRB, 0.80562E+00_JPRB, 0.82651E+00_JPRB, 0.84535E+00_JPRB, 0.84256E+00_JPRB, 0.83583E+00_JPRB ,&
&  0.83142E+00_JPRB, 0.82736E+00_JPRB, 0.82058E+00_JPRB, 0.81127E+00_JPRB, 0.80498E+00_JPRB, 0.80448E+00_JPRB, 0.80500E+00_JPRB ,&
&  0.80637E+00_JPRB, 0.82619E+00_JPRB, 0.85876E+00_JPRB, 0.88415E+00_JPRB, 0.91187E+00_JPRB, 0.95465E+00_JPRB, 0.97458E+00_JPRB ,&
&  0.95739E+00_JPRB, 0.95609E+00_JPRB, 0.94529E+00_JPRB, 0.93096E+00_JPRB, 0.93308E+00_JPRB, 0.92427E+00_JPRB, 0.89683E+00_JPRB ,&
&  0.89131E+00_JPRB, 0.88782E+00_JPRB, 0.88394E+00_JPRB, 0.91331E+00_JPRB, 0.93776E+00_JPRB, 0.94097E+00_JPRB, 0.94165E+00_JPRB ,&
&  0.91922E+00_JPRB, 0.88210E+00_JPRB, 0.84351E+00_JPRB, 0.82337E+00_JPRB, 0.82071E+00_JPRB, 0.80099E+00_JPRB, 0.77319E+00_JPRB ,&
&  0.71561E+00_JPRB, 0.66799E+00_JPRB, 0.64136E+00_JPRB, 0.62524E+00_JPRB, 0.61968E+00_JPRB, 0.61642E+00_JPRB, 0.61123E+00_JPRB ,&
&  0.61584E+00_JPRB, 0.60971E+00_JPRB, 0.58050E+00_JPRB, 0.54958E+00_JPRB, 0.54351E+00_JPRB, 0.55201E+00_JPRB, 0.57180E+00_JPRB ,&
&  0.58672E+00_JPRB, 0.58162E+00_JPRB, 0.58798E+00_JPRB, 0.59533E+00_JPRB, 0.61249E+00_JPRB, 0.63026E+00_JPRB, 0.63247E+00_JPRB ,&
&  0.63365E+00_JPRB, 0.62935E+00_JPRB, 0.61692E+00_JPRB, 0.60849E+00_JPRB, 0.59733E+00_JPRB, 0.59287E+00_JPRB, 0.60428E+00_JPRB ,&
&  0.62163E+00_JPRB, 0.63114E+00_JPRB, 0.64177E+00_JPRB, 0.65535E+00_JPRB, 0.66678E+00_JPRB, 0.67559E+00_JPRB, 0.68255E+00_JPRB ,&
&  0.69293E+00_JPRB, 0.70780E+00_JPRB, 0.72275E+00_JPRB, 0.73583E+00_JPRB, 0.73604E+00_JPRB, 0.72197E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.67966E+00_JPRB, 0.66638E+00_JPRB, 0.66333E+00_JPRB, 0.66900E+00_JPRB, 0.66423E+00_JPRB, 0.64484E+00_JPRB, 0.63524E+00_JPRB ,&
&  0.64613E+00_JPRB, 0.67005E+00_JPRB, 0.69283E+00_JPRB, 0.72384E+00_JPRB, 0.75642E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.81623E+00_JPRB, 0.84767E+00_JPRB, 0.87153E+00_JPRB, 0.87974E+00_JPRB, 0.87735E+00_JPRB, 0.86867E+00_JPRB, 0.84642E+00_JPRB ,&
&  0.82218E+00_JPRB, 0.80519E+00_JPRB, 0.78710E+00_JPRB, 0.76079E+00_JPRB, 0.73891E+00_JPRB, 0.73438E+00_JPRB, 0.74696E+00_JPRB ,&
&  0.79319E+00_JPRB, 0.83413E+00_JPRB, 0.85048E+00_JPRB, 0.86376E+00_JPRB, 0.91307E+00_JPRB, 0.95439E+00_JPRB, 0.96296E+00_JPRB ,&
&  0.95714E+00_JPRB, 0.95831E+00_JPRB, 0.92176E+00_JPRB, 0.88873E+00_JPRB, 0.88932E+00_JPRB, 0.89550E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.90547E+00_JPRB, 0.92437E+00_JPRB, 0.92001E+00_JPRB, 0.89883E+00_JPRB, 0.89390E+00_JPRB, 0.88395E+00_JPRB, 0.87158E+00_JPRB ,&
&  0.84931E+00_JPRB, 0.80093E+00_JPRB, 0.76762E+00_JPRB, 0.76352E+00_JPRB, 0.75629E+00_JPRB, 0.74958E+00_JPRB, 0.73656E+00_JPRB ,&
&  0.69646E+00_JPRB, 0.66016E+00_JPRB, 0.63245E+00_JPRB, 0.61332E+00_JPRB, 0.60110E+00_JPRB, 0.59133E+00_JPRB, 0.58074E+00_JPRB ,&
&  0.57134E+00_JPRB, 0.54240E+00_JPRB, 0.51284E+00_JPRB, 0.50553E+00_JPRB, 0.51051E+00_JPRB, 0.51743E+00_JPRB, 0.54467E+00_JPRB ,&
&  0.56958E+00_JPRB, 0.56785E+00_JPRB, 0.56402E+00_JPRB, 0.56666E+00_JPRB, 0.57607E+00_JPRB, 0.57742E+00_JPRB, 0.56836E+00_JPRB ,&
&  0.56428E+00_JPRB, 0.56262E+00_JPRB, 0.55356E+00_JPRB, 0.54687E+00_JPRB, 0.54654E+00_JPRB, 0.55067E+00_JPRB, 0.56498E+00_JPRB ,&
&  0.57361E+00_JPRB, 0.57453E+00_JPRB, 0.58778E+00_JPRB, 0.60490E+00_JPRB, 0.61565E+00_JPRB, 0.62430E+00_JPRB, 0.63059E+00_JPRB ,&
&  0.64454E+00_JPRB, 0.66443E+00_JPRB, 0.67950E+00_JPRB, 0.68162E+00_JPRB, 0.65884E+00_JPRB, 0.63473E+00_JPRB, 0.61588E+00_JPRB ,&
&  0.60770E+00_JPRB, 0.61230E+00_JPRB, 0.61999E+00_JPRB, 0.62742E+00_JPRB, 0.63418E+00_JPRB, 0.63874E+00_JPRB, 0.63824E+00_JPRB ,&
&  0.65800E+00_JPRB, 0.68927E+00_JPRB, 0.71444E+00_JPRB, 0.74448E+00_JPRB, 0.77592E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.88157E+00_JPRB, 0.89901E+00_JPRB, 0.91054E+00_JPRB, 0.91091E+00_JPRB, 0.89511E+00_JPRB, 0.86827E+00_JPRB, 0.84542E+00_JPRB ,&
&  0.81938E+00_JPRB, 0.79409E+00_JPRB, 0.77009E+00_JPRB, 0.73718E+00_JPRB, 0.71939E+00_JPRB, 0.72256E+00_JPRB, 0.77124E+00_JPRB ,&
&  0.83648E+00_JPRB, 0.84134E+00_JPRB, 0.83515E+00_JPRB, 0.86001E+00_JPRB, 0.91186E+00_JPRB, 0.94317E+00_JPRB, 0.96367E+00_JPRB ,&
&  0.96553E+00_JPRB, 0.93190E+00_JPRB, 0.89890E+00_JPRB, 0.88615E+00_JPRB, 0.91047E+00_JPRB, 0.92937E+00_JPRB, 0.93540E+00_JPRB ,&
&  0.93183E+00_JPRB, 0.91198E+00_JPRB, 0.89478E+00_JPRB, 0.88042E+00_JPRB, 0.88562E+00_JPRB, 0.86077E+00_JPRB, 0.80797E+00_JPRB ,&
&  0.74161E+00_JPRB, 0.71468E+00_JPRB, 0.70125E+00_JPRB, 0.68585E+00_JPRB, 0.67498E+00_JPRB, 0.67539E+00_JPRB, 0.66661E+00_JPRB ,&
&  0.65301E+00_JPRB, 0.63663E+00_JPRB, 0.62056E+00_JPRB, 0.60185E+00_JPRB, 0.58699E+00_JPRB, 0.56616E+00_JPRB, 0.54667E+00_JPRB ,&
&  0.52871E+00_JPRB, 0.50647E+00_JPRB, 0.48581E+00_JPRB, 0.47097E+00_JPRB, 0.46402E+00_JPRB, 0.47425E+00_JPRB, 0.51433E+00_JPRB ,&
&  0.54584E+00_JPRB, 0.54293E+00_JPRB, 0.54084E+00_JPRB, 0.54560E+00_JPRB, 0.53956E+00_JPRB, 0.53037E+00_JPRB, 0.52564E+00_JPRB ,&
&  0.52128E+00_JPRB, 0.51792E+00_JPRB, 0.51697E+00_JPRB, 0.51957E+00_JPRB, 0.52219E+00_JPRB, 0.52038E+00_JPRB, 0.52225E+00_JPRB ,&
&  0.52611E+00_JPRB, 0.53089E+00_JPRB, 0.54779E+00_JPRB, 0.57221E+00_JPRB, 0.59330E+00_JPRB, 0.60875E+00_JPRB, 0.62307E+00_JPRB ,&
&  0.64474E+00_JPRB, 0.65798E+00_JPRB, 0.64550E+00_JPRB, 0.63375E+00_JPRB, 0.62794E+00_JPRB, 0.63343E+00_JPRB, 0.64407E+00_JPRB ,&
&  0.66463E+00_JPRB, 0.67735E+00_JPRB, 0.68355E+00_JPRB, 0.68497E+00_JPRB, 0.68807E+00_JPRB, 0.68893E+00_JPRB, 0.70170E+00_JPRB ,&
&  0.73581E+00_JPRB, 0.76511E+00_JPRB, 0.77235E+00_JPRB, 0.80487E+00_JPRB, 0.84660E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.91971E+00_JPRB, 0.92376E+00_JPRB, 0.91373E+00_JPRB, 0.90917E+00_JPRB, 0.89838E+00_JPRB, 0.88453E+00_JPRB, 0.86213E+00_JPRB ,&
&  0.83588E+00_JPRB, 0.80770E+00_JPRB, 0.78086E+00_JPRB, 0.75875E+00_JPRB, 0.76224E+00_JPRB, 0.79430E+00_JPRB, 0.82148E+00_JPRB ,&
&  0.81039E+00_JPRB, 0.80621E+00_JPRB, 0.82285E+00_JPRB, 0.85300E+00_JPRB, 0.88579E+00_JPRB, 0.91603E+00_JPRB, 0.93746E+00_JPRB ,&
&  0.95463E+00_JPRB, 0.93629E+00_JPRB, 0.94067E+00_JPRB, 0.94358E+00_JPRB, 0.93835E+00_JPRB, 0.92115E+00_JPRB, 0.89303E+00_JPRB ,&
&  0.86489E+00_JPRB, 0.83763E+00_JPRB, 0.80408E+00_JPRB, 0.74630E+00_JPRB, 0.82018E+00_JPRB, 0.74570E+00_JPRB, 0.71771E+00_JPRB ,&
&  0.69042E+00_JPRB, 0.67178E+00_JPRB, 0.65021E+00_JPRB, 0.62746E+00_JPRB, 0.61961E+00_JPRB, 0.61489E+00_JPRB, 0.60665E+00_JPRB ,&
&  0.59960E+00_JPRB, 0.59156E+00_JPRB, 0.57956E+00_JPRB, 0.56485E+00_JPRB, 0.55124E+00_JPRB, 0.53700E+00_JPRB, 0.52397E+00_JPRB ,&
&  0.50484E+00_JPRB, 0.47690E+00_JPRB, 0.44751E+00_JPRB, 0.42572E+00_JPRB, 0.41420E+00_JPRB, 0.42352E+00_JPRB, 0.45689E+00_JPRB ,&
&  0.49116E+00_JPRB, 0.49726E+00_JPRB, 0.50758E+00_JPRB, 0.51138E+00_JPRB, 0.50370E+00_JPRB, 0.49392E+00_JPRB, 0.47665E+00_JPRB ,&
&  0.46395E+00_JPRB, 0.46271E+00_JPRB, 0.46474E+00_JPRB, 0.46952E+00_JPRB, 0.47453E+00_JPRB, 0.48138E+00_JPRB, 0.49083E+00_JPRB ,&
&  0.49939E+00_JPRB, 0.50223E+00_JPRB, 0.51437E+00_JPRB, 0.54107E+00_JPRB, 0.57238E+00_JPRB, 0.60442E+00_JPRB, 0.63215E+00_JPRB ,&
&  0.65123E+00_JPRB, 0.66481E+00_JPRB, 0.67336E+00_JPRB, 0.68857E+00_JPRB, 0.70436E+00_JPRB, 0.72000E+00_JPRB, 0.72793E+00_JPRB ,&
&  0.72522E+00_JPRB, 0.72905E+00_JPRB, 0.74678E+00_JPRB, 0.77546E+00_JPRB, 0.79538E+00_JPRB, 0.82929E+00_JPRB, 0.85054E+00_JPRB ,&
&  0.87297E+00_JPRB, 0.87867E+00_JPRB, 0.87325E+00_JPRB, 0.88576E+00_JPRB, 0.90244E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.90378E+00_JPRB, 0.91426E+00_JPRB, 0.91197E+00_JPRB, 0.91066E+00_JPRB, 0.91203E+00_JPRB, 0.90462E+00_JPRB, 0.89048E+00_JPRB ,&
&  0.87627E+00_JPRB, 0.84953E+00_JPRB, 0.81861E+00_JPRB, 0.80446E+00_JPRB, 0.81136E+00_JPRB, 0.81071E+00_JPRB, 0.79189E+00_JPRB ,&
&  0.78996E+00_JPRB, 0.80332E+00_JPRB, 0.82589E+00_JPRB, 0.85106E+00_JPRB, 0.88499E+00_JPRB, 0.92025E+00_JPRB, 0.94646E+00_JPRB ,&
&  0.95693E+00_JPRB, 0.95533E+00_JPRB, 0.94806E+00_JPRB, 0.92156E+00_JPRB, 0.88600E+00_JPRB, 0.85910E+00_JPRB, 0.83854E+00_JPRB ,&
&  0.81270E+00_JPRB, 0.76388E+00_JPRB, 0.69532E+00_JPRB, 0.69004E+00_JPRB, 0.62815E+00_JPRB, 0.60951E+00_JPRB, 0.63212E+00_JPRB ,&
&  0.64760E+00_JPRB, 0.62969E+00_JPRB, 0.60521E+00_JPRB, 0.58484E+00_JPRB, 0.58042E+00_JPRB, 0.57849E+00_JPRB, 0.57280E+00_JPRB ,&
&  0.56834E+00_JPRB, 0.56106E+00_JPRB, 0.54644E+00_JPRB, 0.53310E+00_JPRB, 0.52045E+00_JPRB, 0.50785E+00_JPRB, 0.49896E+00_JPRB ,&
&  0.46995E+00_JPRB, 0.42601E+00_JPRB, 0.39324E+00_JPRB, 0.37868E+00_JPRB, 0.37066E+00_JPRB, 0.37470E+00_JPRB, 0.38829E+00_JPRB ,&
&  0.39968E+00_JPRB, 0.40121E+00_JPRB, 0.39628E+00_JPRB, 0.38358E+00_JPRB, 0.36284E+00_JPRB, 0.31995E+00_JPRB, 0.29493E+00_JPRB ,&
&  0.31211E+00_JPRB, 0.34441E+00_JPRB, 0.37820E+00_JPRB, 0.39044E+00_JPRB, 0.39958E+00_JPRB, 0.40100E+00_JPRB, 0.38534E+00_JPRB ,&
&  0.35576E+00_JPRB, 0.32809E+00_JPRB, 0.32337E+00_JPRB, 0.32550E+00_JPRB, 0.33116E+00_JPRB, 0.34129E+00_JPRB, 0.35788E+00_JPRB ,&
&  0.38262E+00_JPRB, 0.42016E+00_JPRB, 0.44261E+00_JPRB, 0.44527E+00_JPRB, 0.43790E+00_JPRB, 0.42680E+00_JPRB, 0.42441E+00_JPRB ,&
&  0.43892E+00_JPRB, 0.49905E+00_JPRB, 0.61996E+00_JPRB, 0.76499E+00_JPRB, 0.76674E+00_JPRB, 0.81135E+00_JPRB, 0.84603E+00_JPRB ,&
&  0.87601E+00_JPRB, 0.88963E+00_JPRB, 0.87656E+00_JPRB, 0.87753E+00_JPRB, 0.88388E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.86193E+00_JPRB, 0.88173E+00_JPRB, 0.89243E+00_JPRB, 0.90616E+00_JPRB, 0.90612E+00_JPRB, 0.87573E+00_JPRB, 0.86108E+00_JPRB ,&
&  0.86549E+00_JPRB, 0.86017E+00_JPRB, 0.84869E+00_JPRB, 0.84132E+00_JPRB, 0.83207E+00_JPRB, 0.82212E+00_JPRB, 0.81383E+00_JPRB ,&
&  0.80542E+00_JPRB, 0.80005E+00_JPRB, 0.80668E+00_JPRB, 0.82026E+00_JPRB, 0.83134E+00_JPRB, 0.83733E+00_JPRB, 0.84228E+00_JPRB ,&
&  0.84184E+00_JPRB, 0.80897E+00_JPRB, 0.76368E+00_JPRB, 0.73057E+00_JPRB, 0.71436E+00_JPRB, 0.70077E+00_JPRB, 0.66736E+00_JPRB ,&
&  0.59804E+00_JPRB, 0.47667E+00_JPRB, 0.42168E+00_JPRB, 0.46397E+00_JPRB, 0.58032E+00_JPRB, 0.54710E+00_JPRB, 0.56819E+00_JPRB ,&
&  0.59908E+00_JPRB, 0.61516E+00_JPRB, 0.61081E+00_JPRB, 0.60003E+00_JPRB, 0.59610E+00_JPRB, 0.59938E+00_JPRB, 0.59593E+00_JPRB ,&
&  0.59234E+00_JPRB, 0.57616E+00_JPRB, 0.56630E+00_JPRB, 0.55453E+00_JPRB, 0.53618E+00_JPRB, 0.50998E+00_JPRB, 0.49381E+00_JPRB ,&
&  0.45469E+00_JPRB, 0.41631E+00_JPRB, 0.40483E+00_JPRB, 0.40496E+00_JPRB, 0.37410E+00_JPRB, 0.36179E+00_JPRB, 0.34677E+00_JPRB ,&
&  0.30854E+00_JPRB, 0.28775E+00_JPRB, 0.25669E+00_JPRB, 0.23750E+00_JPRB, 0.22415E+00_JPRB, 0.20980E+00_JPRB, 0.20782E+00_JPRB ,&
&  0.21344E+00_JPRB, 0.22696E+00_JPRB, 0.24798E+00_JPRB, 0.27971E+00_JPRB, 0.30786E+00_JPRB, 0.26935E+00_JPRB, 0.25289E+00_JPRB ,&
&  0.23828E+00_JPRB, 0.23489E+00_JPRB, 0.24129E+00_JPRB, 0.24862E+00_JPRB, 0.25595E+00_JPRB, 0.26457E+00_JPRB, 0.27319E+00_JPRB ,&
&  0.27952E+00_JPRB, 0.28266E+00_JPRB, 0.28325E+00_JPRB, 0.28244E+00_JPRB, 0.28095E+00_JPRB, 0.28143E+00_JPRB, 0.28480E+00_JPRB ,&
&  0.29155E+00_JPRB, 0.30428E+00_JPRB, 0.32827E+00_JPRB, 0.36371E+00_JPRB, 0.40955E+00_JPRB, 0.47003E+00_JPRB, 0.56184E+00_JPRB ,&
&  0.68632E+00_JPRB, 0.84967E+00_JPRB, 0.89580E+00_JPRB, 0.90509E+00_JPRB, 0.87372E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.86575E+00_JPRB, 0.86288E+00_JPRB, 0.86996E+00_JPRB, 0.85946E+00_JPRB, 0.84031E+00_JPRB, 0.83123E+00_JPRB, 0.84812E+00_JPRB ,&
&  0.86773E+00_JPRB, 0.83794E+00_JPRB, 0.81597E+00_JPRB, 0.79660E+00_JPRB, 0.78250E+00_JPRB, 0.76839E+00_JPRB, 0.74635E+00_JPRB ,&
&  0.72617E+00_JPRB, 0.71859E+00_JPRB, 0.71636E+00_JPRB, 0.71108E+00_JPRB, 0.70069E+00_JPRB, 0.69053E+00_JPRB, 0.67280E+00_JPRB ,&
&  0.63953E+00_JPRB, 0.60444E+00_JPRB, 0.58626E+00_JPRB, 0.56247E+00_JPRB, 0.54562E+00_JPRB, 0.52614E+00_JPRB, 0.49851E+00_JPRB ,&
&  0.45286E+00_JPRB, 0.39487E+00_JPRB, 0.36066E+00_JPRB, 0.39836E+00_JPRB, 0.49180E+00_JPRB, 0.48503E+00_JPRB, 0.50522E+00_JPRB ,&
&  0.54037E+00_JPRB, 0.56171E+00_JPRB, 0.57224E+00_JPRB, 0.57547E+00_JPRB, 0.58452E+00_JPRB, 0.59037E+00_JPRB, 0.58659E+00_JPRB ,&
&  0.57113E+00_JPRB, 0.51855E+00_JPRB, 0.47488E+00_JPRB, 0.37079E+00_JPRB, 0.30027E+00_JPRB, 0.26292E+00_JPRB, 0.23301E+00_JPRB ,&
&  0.21336E+00_JPRB, 0.20217E+00_JPRB, 0.19978E+00_JPRB, 0.19870E+00_JPRB, 0.20441E+00_JPRB, 0.21146E+00_JPRB, 0.21194E+00_JPRB ,&
&  0.20794E+00_JPRB, 0.19978E+00_JPRB, 0.18858E+00_JPRB, 0.18238E+00_JPRB, 0.17906E+00_JPRB, 0.17941E+00_JPRB, 0.18171E+00_JPRB ,&
&  0.18311E+00_JPRB, 0.18667E+00_JPRB, 0.19928E+00_JPRB, 0.20989E+00_JPRB, 0.22343E+00_JPRB, 0.20693E+00_JPRB, 0.19288E+00_JPRB ,&
&  0.18739E+00_JPRB, 0.19246E+00_JPRB, 0.20003E+00_JPRB, 0.20566E+00_JPRB, 0.21153E+00_JPRB, 0.21632E+00_JPRB, 0.22111E+00_JPRB ,&
&  0.22612E+00_JPRB, 0.22977E+00_JPRB, 0.23140E+00_JPRB, 0.23319E+00_JPRB, 0.23477E+00_JPRB, 0.23719E+00_JPRB, 0.24008E+00_JPRB ,&
&  0.24502E+00_JPRB, 0.25467E+00_JPRB, 0.26714E+00_JPRB, 0.28780E+00_JPRB, 0.30717E+00_JPRB, 0.32639E+00_JPRB, 0.34899E+00_JPRB ,&
&  0.36949E+00_JPRB, 0.45041E+00_JPRB, 0.64337E+00_JPRB, 0.96217E+00_JPRB, 0.95560E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.81214E+00_JPRB, 0.82423E+00_JPRB, 0.84085E+00_JPRB, 0.81727E+00_JPRB, 0.80198E+00_JPRB, 0.80516E+00_JPRB, 0.80581E+00_JPRB ,&
&  0.78237E+00_JPRB, 0.74654E+00_JPRB, 0.69680E+00_JPRB, 0.65522E+00_JPRB, 0.59060E+00_JPRB, 0.50739E+00_JPRB, 0.46949E+00_JPRB ,&
&  0.43457E+00_JPRB, 0.43065E+00_JPRB, 0.43249E+00_JPRB, 0.44843E+00_JPRB, 0.45695E+00_JPRB, 0.45338E+00_JPRB, 0.43110E+00_JPRB ,&
&  0.40373E+00_JPRB, 0.37676E+00_JPRB, 0.35909E+00_JPRB, 0.34539E+00_JPRB, 0.34528E+00_JPRB, 0.34334E+00_JPRB, 0.33681E+00_JPRB ,&
&  0.32879E+00_JPRB, 0.33379E+00_JPRB, 0.35160E+00_JPRB, 0.37925E+00_JPRB, 0.42740E+00_JPRB, 0.40726E+00_JPRB, 0.41490E+00_JPRB ,&
&  0.44535E+00_JPRB, 0.46731E+00_JPRB, 0.47598E+00_JPRB, 0.47909E+00_JPRB, 0.47172E+00_JPRB, 0.49483E+00_JPRB, 0.43803E+00_JPRB ,&
&  0.33140E+00_JPRB, 0.27227E+00_JPRB, 0.22993E+00_JPRB, 0.20217E+00_JPRB, 0.18311E+00_JPRB, 0.17188E+00_JPRB, 0.15931E+00_JPRB ,&
&  0.15343E+00_JPRB, 0.14899E+00_JPRB, 0.14814E+00_JPRB, 0.15031E+00_JPRB, 0.15344E+00_JPRB, 0.15740E+00_JPRB, 0.15897E+00_JPRB ,&
&  0.16132E+00_JPRB, 0.16160E+00_JPRB, 0.16105E+00_JPRB, 0.16119E+00_JPRB, 0.16019E+00_JPRB, 0.16104E+00_JPRB, 0.16110E+00_JPRB ,&
&  0.16097E+00_JPRB, 0.16070E+00_JPRB, 0.16244E+00_JPRB, 0.16345E+00_JPRB, 0.16443E+00_JPRB, 0.16146E+00_JPRB, 0.16011E+00_JPRB ,&
&  0.16067E+00_JPRB, 0.16309E+00_JPRB, 0.16681E+00_JPRB, 0.17097E+00_JPRB, 0.17667E+00_JPRB, 0.18106E+00_JPRB, 0.18620E+00_JPRB ,&
&  0.19065E+00_JPRB, 0.19507E+00_JPRB, 0.19938E+00_JPRB, 0.20377E+00_JPRB, 0.20857E+00_JPRB, 0.21334E+00_JPRB, 0.21825E+00_JPRB ,&
&  0.22341E+00_JPRB, 0.23202E+00_JPRB, 0.24246E+00_JPRB, 0.26067E+00_JPRB, 0.28515E+00_JPRB, 0.31130E+00_JPRB, 0.35759E+00_JPRB ,&
&  0.41537E+00_JPRB, 0.64295E+00_JPRB, 0.94517E+00_JPRB, 0.77727E+00_JPRB, 0.79462E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.80155E+00_JPRB, 0.81971E+00_JPRB, 0.83550E+00_JPRB, 0.83271E+00_JPRB, 0.81525E+00_JPRB, 0.75755E+00_JPRB, 0.69403E+00_JPRB ,&
&  0.61529E+00_JPRB, 0.54945E+00_JPRB, 0.47992E+00_JPRB, 0.41998E+00_JPRB, 0.36943E+00_JPRB, 0.33044E+00_JPRB, 0.30636E+00_JPRB ,&
&  0.28909E+00_JPRB, 0.28146E+00_JPRB, 0.27810E+00_JPRB, 0.28253E+00_JPRB, 0.28458E+00_JPRB, 0.28993E+00_JPRB, 0.28418E+00_JPRB ,&
&  0.27710E+00_JPRB, 0.26929E+00_JPRB, 0.26721E+00_JPRB, 0.27584E+00_JPRB, 0.28924E+00_JPRB, 0.30223E+00_JPRB, 0.31918E+00_JPRB ,&
&  0.33749E+00_JPRB, 0.34336E+00_JPRB, 0.35783E+00_JPRB, 0.36141E+00_JPRB, 0.34698E+00_JPRB, 0.34892E+00_JPRB, 0.35487E+00_JPRB ,&
&  0.36813E+00_JPRB, 0.38666E+00_JPRB, 0.39721E+00_JPRB, 0.41685E+00_JPRB, 0.36287E+00_JPRB, 0.30280E+00_JPRB, 0.25058E+00_JPRB ,&
&  0.21162E+00_JPRB, 0.18928E+00_JPRB, 0.17321E+00_JPRB, 0.16507E+00_JPRB, 0.15692E+00_JPRB, 0.15250E+00_JPRB, 0.14793E+00_JPRB ,&
&  0.14595E+00_JPRB, 0.14610E+00_JPRB, 0.14584E+00_JPRB, 0.14583E+00_JPRB, 0.14663E+00_JPRB, 0.14777E+00_JPRB, 0.14751E+00_JPRB ,&
&  0.14705E+00_JPRB, 0.14698E+00_JPRB, 0.14667E+00_JPRB, 0.14639E+00_JPRB, 0.14556E+00_JPRB, 0.14553E+00_JPRB, 0.14388E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14165E+00_JPRB, 0.14088E+00_JPRB, 0.14092E+00_JPRB, 0.14147E+00_JPRB, 0.14198E+00_JPRB, 0.14281E+00_JPRB ,&
&  0.14426E+00_JPRB, 0.14556E+00_JPRB, 0.14771E+00_JPRB, 0.14969E+00_JPRB, 0.15288E+00_JPRB, 0.15564E+00_JPRB, 0.15949E+00_JPRB ,&
&  0.16360E+00_JPRB, 0.16855E+00_JPRB, 0.17434E+00_JPRB, 0.18089E+00_JPRB, 0.18635E+00_JPRB, 0.19165E+00_JPRB, 0.19737E+00_JPRB ,&
&  0.20180E+00_JPRB, 0.20943E+00_JPRB, 0.21792E+00_JPRB, 0.23212E+00_JPRB, 0.26188E+00_JPRB, 0.28521E+00_JPRB, 0.33347E+00_JPRB ,&
&  0.41298E+00_JPRB, 0.66498E+00_JPRB, 0.89781E+00_JPRB, 0.75190E+00_JPRB, 0.78327E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.67914E+00_JPRB, 0.71015E+00_JPRB, 0.73393E+00_JPRB, 0.73566E+00_JPRB, 0.73401E+00_JPRB, 0.69643E+00_JPRB, 0.63801E+00_JPRB ,&
&  0.53173E+00_JPRB, 0.46225E+00_JPRB, 0.41794E+00_JPRB, 0.37606E+00_JPRB, 0.33823E+00_JPRB, 0.31240E+00_JPRB, 0.29489E+00_JPRB ,&
&  0.28165E+00_JPRB, 0.27431E+00_JPRB, 0.26876E+00_JPRB, 0.26411E+00_JPRB, 0.25901E+00_JPRB, 0.25412E+00_JPRB, 0.24541E+00_JPRB ,&
&  0.24007E+00_JPRB, 0.23678E+00_JPRB, 0.23522E+00_JPRB, 0.23763E+00_JPRB, 0.24452E+00_JPRB, 0.25114E+00_JPRB, 0.26162E+00_JPRB ,&
&  0.27384E+00_JPRB, 0.28798E+00_JPRB, 0.29666E+00_JPRB, 0.30030E+00_JPRB, 0.30341E+00_JPRB, 0.30189E+00_JPRB, 0.27983E+00_JPRB ,&
&  0.28473E+00_JPRB, 0.30652E+00_JPRB, 0.32469E+00_JPRB, 0.33359E+00_JPRB, 0.28823E+00_JPRB, 0.24691E+00_JPRB, 0.20622E+00_JPRB ,&
&  0.18719E+00_JPRB, 0.17151E+00_JPRB, 0.16322E+00_JPRB, 0.15684E+00_JPRB, 0.15189E+00_JPRB, 0.14930E+00_JPRB, 0.14706E+00_JPRB ,&
&  0.14519E+00_JPRB, 0.14464E+00_JPRB, 0.14348E+00_JPRB, 0.14318E+00_JPRB, 0.14247E+00_JPRB, 0.14258E+00_JPRB, 0.14150E+00_JPRB ,&
&  0.14038E+00_JPRB, 0.13948E+00_JPRB, 0.13892E+00_JPRB, 0.13865E+00_JPRB, 0.13778E+00_JPRB, 0.13693E+00_JPRB, 0.13491E+00_JPRB ,&
&  0.13355E+00_JPRB, 0.13223E+00_JPRB, 0.13128E+00_JPRB, 0.13150E+00_JPRB, 0.13223E+00_JPRB, 0.13441E+00_JPRB, 0.13563E+00_JPRB ,&
&  0.13590E+00_JPRB, 0.13602E+00_JPRB, 0.13603E+00_JPRB, 0.13684E+00_JPRB, 0.13803E+00_JPRB, 0.14007E+00_JPRB, 0.14293E+00_JPRB ,&
&  0.14637E+00_JPRB, 0.15168E+00_JPRB, 0.15694E+00_JPRB, 0.16273E+00_JPRB, 0.16805E+00_JPRB, 0.17309E+00_JPRB, 0.17992E+00_JPRB ,&
&  0.18685E+00_JPRB, 0.19516E+00_JPRB, 0.20758E+00_JPRB, 0.22062E+00_JPRB, 0.24421E+00_JPRB, 0.26361E+00_JPRB, 0.29405E+00_JPRB ,&
&  0.36338E+00_JPRB, 0.50894E+00_JPRB, 0.59494E+00_JPRB, 0.60783E+00_JPRB, 0.64121E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.49660E+00_JPRB, 0.50819E+00_JPRB, 0.52327E+00_JPRB, 0.53238E+00_JPRB, 0.54274E+00_JPRB, 0.51018E+00_JPRB, 0.48446E+00_JPRB ,&
&  0.43171E+00_JPRB, 0.40141E+00_JPRB, 0.37617E+00_JPRB, 0.35299E+00_JPRB, 0.33231E+00_JPRB, 0.31363E+00_JPRB, 0.29623E+00_JPRB ,&
&  0.28508E+00_JPRB, 0.27843E+00_JPRB, 0.26946E+00_JPRB, 0.25930E+00_JPRB, 0.24841E+00_JPRB, 0.23887E+00_JPRB, 0.23407E+00_JPRB ,&
&  0.22932E+00_JPRB, 0.22592E+00_JPRB, 0.22450E+00_JPRB, 0.22440E+00_JPRB, 0.22618E+00_JPRB, 0.22624E+00_JPRB, 0.22676E+00_JPRB ,&
&  0.22330E+00_JPRB, 0.23055E+00_JPRB, 0.24016E+00_JPRB, 0.24457E+00_JPRB, 0.25284E+00_JPRB, 0.25162E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.23966E+00_JPRB, 0.23124E+00_JPRB, 0.22155E+00_JPRB, 0.20956E+00_JPRB, 0.19915E+00_JPRB, 0.19155E+00_JPRB, 0.18403E+00_JPRB ,&
&  0.17959E+00_JPRB, 0.17399E+00_JPRB, 0.16718E+00_JPRB, 0.16127E+00_JPRB, 0.15614E+00_JPRB, 0.15201E+00_JPRB, 0.14821E+00_JPRB ,&
&  0.14555E+00_JPRB, 0.14406E+00_JPRB, 0.14265E+00_JPRB, 0.14248E+00_JPRB, 0.14150E+00_JPRB, 0.14046E+00_JPRB, 0.13942E+00_JPRB ,&
&  0.13889E+00_JPRB, 0.13767E+00_JPRB, 0.13669E+00_JPRB, 0.13582E+00_JPRB, 0.13441E+00_JPRB, 0.13320E+00_JPRB, 0.13180E+00_JPRB ,&
&  0.13063E+00_JPRB, 0.13006E+00_JPRB, 0.12977E+00_JPRB, 0.12977E+00_JPRB, 0.13016E+00_JPRB, 0.13017E+00_JPRB, 0.13051E+00_JPRB ,&
&  0.13090E+00_JPRB, 0.13145E+00_JPRB, 0.13146E+00_JPRB, 0.13185E+00_JPRB, 0.13281E+00_JPRB, 0.13412E+00_JPRB, 0.13727E+00_JPRB ,&
&  0.14085E+00_JPRB, 0.14543E+00_JPRB, 0.15037E+00_JPRB, 0.15587E+00_JPRB, 0.16133E+00_JPRB, 0.16665E+00_JPRB, 0.17258E+00_JPRB ,&
&  0.18026E+00_JPRB, 0.18721E+00_JPRB, 0.19544E+00_JPRB, 0.20802E+00_JPRB, 0.23590E+00_JPRB, 0.25078E+00_JPRB, 0.26527E+00_JPRB ,&
&  0.29084E+00_JPRB, 0.32478E+00_JPRB, 0.42828E+00_JPRB, 0.46063E+00_JPRB, 0.47793E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.24057E+00_JPRB, 0.24039E+00_JPRB, 0.24084E+00_JPRB, 0.24693E+00_JPRB, 0.25706E+00_JPRB, 0.33438E+00_JPRB, 0.32502E+00_JPRB ,&
&  0.31743E+00_JPRB, 0.28950E+00_JPRB, 0.28617E+00_JPRB, 0.27922E+00_JPRB, 0.26977E+00_JPRB, 0.26229E+00_JPRB, 0.25794E+00_JPRB ,&
&  0.25314E+00_JPRB, 0.24779E+00_JPRB, 0.24492E+00_JPRB, 0.24176E+00_JPRB, 0.23863E+00_JPRB, 0.23500E+00_JPRB, 0.23219E+00_JPRB ,&
&  0.22951E+00_JPRB, 0.22763E+00_JPRB, 0.22558E+00_JPRB, 0.22145E+00_JPRB, 0.21966E+00_JPRB, 0.21625E+00_JPRB, 0.21120E+00_JPRB ,&
&  0.20459E+00_JPRB, 0.20114E+00_JPRB, 0.19732E+00_JPRB, 0.19394E+00_JPRB, 0.19716E+00_JPRB, 0.19636E+00_JPRB, 0.19428E+00_JPRB ,&
&  0.19070E+00_JPRB, 0.18700E+00_JPRB, 0.18215E+00_JPRB, 0.17748E+00_JPRB, 0.17363E+00_JPRB, 0.17049E+00_JPRB, 0.16781E+00_JPRB ,&
&  0.16568E+00_JPRB, 0.16391E+00_JPRB, 0.16138E+00_JPRB, 0.15909E+00_JPRB, 0.15665E+00_JPRB, 0.15453E+00_JPRB, 0.15230E+00_JPRB ,&
&  0.15057E+00_JPRB, 0.14894E+00_JPRB, 0.14725E+00_JPRB, 0.14551E+00_JPRB, 0.14415E+00_JPRB, 0.14236E+00_JPRB, 0.14102E+00_JPRB ,&
&  0.13987E+00_JPRB, 0.13837E+00_JPRB, 0.13726E+00_JPRB, 0.13591E+00_JPRB, 0.13492E+00_JPRB, 0.13410E+00_JPRB, 0.13330E+00_JPRB ,&
&  0.13222E+00_JPRB, 0.13150E+00_JPRB, 0.13124E+00_JPRB, 0.13106E+00_JPRB, 0.13076E+00_JPRB, 0.13084E+00_JPRB, 0.13122E+00_JPRB ,&
&  0.13174E+00_JPRB, 0.13236E+00_JPRB, 0.13334E+00_JPRB, 0.13446E+00_JPRB, 0.13614E+00_JPRB, 0.13838E+00_JPRB, 0.14189E+00_JPRB ,&
&  0.14569E+00_JPRB, 0.14988E+00_JPRB, 0.15412E+00_JPRB, 0.15866E+00_JPRB, 0.16329E+00_JPRB, 0.16796E+00_JPRB, 0.17335E+00_JPRB ,&
&  0.17915E+00_JPRB, 0.18368E+00_JPRB, 0.18795E+00_JPRB, 0.19251E+00_JPRB, 0.20234E+00_JPRB, 0.20967E+00_JPRB, 0.21518E+00_JPRB ,&
&  0.22140E+00_JPRB, 0.22997E+00_JPRB, 0.23465E+00_JPRB, 0.23775E+00_JPRB, 0.23938E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.21197E+00_JPRB, 0.21472E+00_JPRB, 0.21658E+00_JPRB, 0.21775E+00_JPRB, 0.21746E+00_JPRB, 0.21836E+00_JPRB, 0.21930E+00_JPRB ,&
&  0.22056E+00_JPRB, 0.22215E+00_JPRB, 0.22329E+00_JPRB, 0.22432E+00_JPRB, 0.22478E+00_JPRB, 0.22477E+00_JPRB, 0.22476E+00_JPRB ,&
&  0.22453E+00_JPRB, 0.22426E+00_JPRB, 0.22050E+00_JPRB, 0.21711E+00_JPRB, 0.21465E+00_JPRB, 0.21321E+00_JPRB, 0.21196E+00_JPRB ,&
&  0.21094E+00_JPRB, 0.20966E+00_JPRB, 0.20820E+00_JPRB, 0.20606E+00_JPRB, 0.20275E+00_JPRB, 0.19952E+00_JPRB, 0.19619E+00_JPRB ,&
&  0.19318E+00_JPRB, 0.19078E+00_JPRB, 0.18856E+00_JPRB, 0.18634E+00_JPRB, 0.18506E+00_JPRB, 0.18356E+00_JPRB, 0.18199E+00_JPRB ,&
&  0.17995E+00_JPRB, 0.17774E+00_JPRB, 0.17564E+00_JPRB, 0.17349E+00_JPRB, 0.17142E+00_JPRB, 0.16937E+00_JPRB, 0.16788E+00_JPRB ,&
&  0.16633E+00_JPRB, 0.16517E+00_JPRB, 0.16388E+00_JPRB, 0.16271E+00_JPRB, 0.16148E+00_JPRB, 0.16032E+00_JPRB, 0.15894E+00_JPRB ,&
&  0.15799E+00_JPRB, 0.15701E+00_JPRB, 0.15601E+00_JPRB, 0.15509E+00_JPRB, 0.15417E+00_JPRB, 0.15328E+00_JPRB, 0.15227E+00_JPRB ,&
&  0.15134E+00_JPRB, 0.15074E+00_JPRB, 0.15012E+00_JPRB, 0.14942E+00_JPRB, 0.14878E+00_JPRB, 0.14835E+00_JPRB, 0.14797E+00_JPRB ,&
&  0.14763E+00_JPRB, 0.14762E+00_JPRB, 0.14780E+00_JPRB, 0.14798E+00_JPRB, 0.14818E+00_JPRB, 0.14859E+00_JPRB, 0.14920E+00_JPRB ,&
&  0.14993E+00_JPRB, 0.15071E+00_JPRB, 0.15171E+00_JPRB, 0.15312E+00_JPRB, 0.15469E+00_JPRB, 0.15648E+00_JPRB, 0.15843E+00_JPRB ,&
&  0.16054E+00_JPRB, 0.16278E+00_JPRB, 0.16513E+00_JPRB, 0.16737E+00_JPRB, 0.16983E+00_JPRB, 0.17226E+00_JPRB, 0.17463E+00_JPRB ,&
&  0.17710E+00_JPRB, 0.17955E+00_JPRB, 0.18209E+00_JPRB, 0.18442E+00_JPRB, 0.18809E+00_JPRB, 0.19049E+00_JPRB, 0.19296E+00_JPRB ,&
&  0.19524E+00_JPRB, 0.19788E+00_JPRB, 0.20055E+00_JPRB, 0.20383E+00_JPRB, 0.20710E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB, 0.17576E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB, 0.66266E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.71782E+00_JPRB, 0.71508E+00_JPRB, 0.71206E+00_JPRB, 0.70931E+00_JPRB, 0.70599E+00_JPRB, 0.70309E+00_JPRB, 0.70012E+00_JPRB ,&
&  0.69748E+00_JPRB, 0.69468E+00_JPRB, 0.69168E+00_JPRB, 0.68841E+00_JPRB, 0.68519E+00_JPRB, 0.68205E+00_JPRB, 0.67900E+00_JPRB ,&
&  0.67594E+00_JPRB, 0.67304E+00_JPRB, 0.67004E+00_JPRB, 0.66670E+00_JPRB, 0.66386E+00_JPRB, 0.66057E+00_JPRB, 0.65637E+00_JPRB ,&
&  0.65028E+00_JPRB, 0.64382E+00_JPRB, 0.63791E+00_JPRB, 0.63369E+00_JPRB, 0.63058E+00_JPRB, 0.62802E+00_JPRB, 0.62577E+00_JPRB ,&
&  0.62432E+00_JPRB, 0.62353E+00_JPRB, 0.62301E+00_JPRB, 0.62206E+00_JPRB, 0.62159E+00_JPRB, 0.62254E+00_JPRB, 0.62298E+00_JPRB ,&
&  0.62292E+00_JPRB, 0.62299E+00_JPRB, 0.62399E+00_JPRB, 0.62482E+00_JPRB, 0.62550E+00_JPRB, 0.62668E+00_JPRB, 0.62898E+00_JPRB ,&
&  0.63133E+00_JPRB, 0.63339E+00_JPRB, 0.63587E+00_JPRB, 0.63860E+00_JPRB, 0.64141E+00_JPRB, 0.64419E+00_JPRB, 0.64800E+00_JPRB ,&
&  0.65222E+00_JPRB, 0.65598E+00_JPRB, 0.65945E+00_JPRB, 0.66226E+00_JPRB, 0.66462E+00_JPRB, 0.66589E+00_JPRB, 0.66674E+00_JPRB ,&
&  0.66674E+00_JPRB, 0.66783E+00_JPRB, 0.66806E+00_JPRB, 0.66839E+00_JPRB, 0.66794E+00_JPRB, 0.66893E+00_JPRB, 0.66954E+00_JPRB ,&
&  0.67140E+00_JPRB, 0.67262E+00_JPRB, 0.67487E+00_JPRB, 0.67706E+00_JPRB, 0.68045E+00_JPRB, 0.68419E+00_JPRB, 0.68822E+00_JPRB ,&
&  0.69268E+00_JPRB, 0.69784E+00_JPRB, 0.70381E+00_JPRB, 0.70753E+00_JPRB, 0.71125E+00_JPRB, 0.71491E+00_JPRB, 0.71884E+00_JPRB ,&
&  0.72185E+00_JPRB, 0.72464E+00_JPRB, 0.72748E+00_JPRB, 0.72975E+00_JPRB, 0.73118E+00_JPRB, 0.73236E+00_JPRB, 0.73352E+00_JPRB ,&
&  0.73416E+00_JPRB, 0.73466E+00_JPRB, 0.73493E+00_JPRB, 0.73527E+00_JPRB, 0.73443E+00_JPRB, 0.73294E+00_JPRB, 0.73116E+00_JPRB ,&
&  0.72968E+00_JPRB, 0.72748E+00_JPRB, 0.72530E+00_JPRB, 0.72296E+00_JPRB, 0.72070E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.81898E+00_JPRB, 0.81415E+00_JPRB, 0.80906E+00_JPRB, 0.80450E+00_JPRB, 0.79787E+00_JPRB, 0.79115E+00_JPRB, 0.78410E+00_JPRB ,&
&  0.77660E+00_JPRB, 0.76720E+00_JPRB, 0.75685E+00_JPRB, 0.74582E+00_JPRB, 0.73483E+00_JPRB, 0.72547E+00_JPRB, 0.71807E+00_JPRB ,&
&  0.71095E+00_JPRB, 0.70341E+00_JPRB, 0.69627E+00_JPRB, 0.68724E+00_JPRB, 0.67814E+00_JPRB, 0.66983E+00_JPRB, 0.66347E+00_JPRB ,&
&  0.65459E+00_JPRB, 0.64485E+00_JPRB, 0.63481E+00_JPRB, 0.62456E+00_JPRB, 0.61485E+00_JPRB, 0.60523E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.58707E+00_JPRB, 0.58062E+00_JPRB, 0.57467E+00_JPRB, 0.56842E+00_JPRB, 0.56190E+00_JPRB, 0.55987E+00_JPRB, 0.55769E+00_JPRB ,&
&  0.55571E+00_JPRB, 0.55514E+00_JPRB, 0.55902E+00_JPRB, 0.56246E+00_JPRB, 0.56609E+00_JPRB, 0.56995E+00_JPRB, 0.57674E+00_JPRB ,&
&  0.58319E+00_JPRB, 0.59070E+00_JPRB, 0.60023E+00_JPRB, 0.60989E+00_JPRB, 0.61787E+00_JPRB, 0.62384E+00_JPRB, 0.63192E+00_JPRB ,&
&  0.63780E+00_JPRB, 0.64574E+00_JPRB, 0.65140E+00_JPRB, 0.65428E+00_JPRB, 0.65417E+00_JPRB, 0.65434E+00_JPRB, 0.65586E+00_JPRB ,&
&  0.65635E+00_JPRB, 0.66078E+00_JPRB, 0.66507E+00_JPRB, 0.66961E+00_JPRB, 0.67151E+00_JPRB, 0.67683E+00_JPRB, 0.68076E+00_JPRB ,&
&  0.68804E+00_JPRB, 0.69876E+00_JPRB, 0.70977E+00_JPRB, 0.72191E+00_JPRB, 0.73670E+00_JPRB, 0.75404E+00_JPRB, 0.76827E+00_JPRB ,&
&  0.78169E+00_JPRB, 0.79366E+00_JPRB, 0.80390E+00_JPRB, 0.81013E+00_JPRB, 0.81598E+00_JPRB, 0.82153E+00_JPRB, 0.82704E+00_JPRB ,&
&  0.82943E+00_JPRB, 0.83220E+00_JPRB, 0.83520E+00_JPRB, 0.83765E+00_JPRB, 0.83736E+00_JPRB, 0.83676E+00_JPRB, 0.83681E+00_JPRB ,&
&  0.83707E+00_JPRB, 0.83688E+00_JPRB, 0.83715E+00_JPRB, 0.83837E+00_JPRB, 0.83880E+00_JPRB, 0.83806E+00_JPRB, 0.83712E+00_JPRB ,&
&  0.83636E+00_JPRB, 0.83389E+00_JPRB, 0.83040E+00_JPRB, 0.82686E+00_JPRB, 0.82376E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.86198E+00_JPRB, 0.86001E+00_JPRB, 0.85797E+00_JPRB, 0.85582E+00_JPRB, 0.85147E+00_JPRB, 0.84666E+00_JPRB, 0.84045E+00_JPRB ,&
&  0.83325E+00_JPRB, 0.82303E+00_JPRB, 0.81195E+00_JPRB, 0.79964E+00_JPRB, 0.78758E+00_JPRB, 0.77671E+00_JPRB, 0.76627E+00_JPRB ,&
&  0.75589E+00_JPRB, 0.74293E+00_JPRB, 0.72668E+00_JPRB, 0.70591E+00_JPRB, 0.68764E+00_JPRB, 0.67260E+00_JPRB, 0.65714E+00_JPRB ,&
&  0.63430E+00_JPRB, 0.61002E+00_JPRB, 0.58841E+00_JPRB, 0.56644E+00_JPRB, 0.55044E+00_JPRB, 0.51152E+00_JPRB, 0.49877E+00_JPRB ,&
&  0.48913E+00_JPRB, 0.48012E+00_JPRB, 0.47571E+00_JPRB, 0.47220E+00_JPRB, 0.47319E+00_JPRB, 0.46634E+00_JPRB, 0.46239E+00_JPRB ,&
&  0.44544E+00_JPRB, 0.44818E+00_JPRB, 0.45261E+00_JPRB, 0.45826E+00_JPRB, 0.46953E+00_JPRB, 0.48223E+00_JPRB, 0.50206E+00_JPRB ,&
&  0.53779E+00_JPRB, 0.55358E+00_JPRB, 0.57742E+00_JPRB, 0.59584E+00_JPRB, 0.60748E+00_JPRB, 0.61392E+00_JPRB, 0.62193E+00_JPRB ,&
&  0.63047E+00_JPRB, 0.64379E+00_JPRB, 0.65154E+00_JPRB, 0.65665E+00_JPRB, 0.65657E+00_JPRB, 0.65506E+00_JPRB, 0.65530E+00_JPRB ,&
&  0.65493E+00_JPRB, 0.65970E+00_JPRB, 0.66801E+00_JPRB, 0.68441E+00_JPRB, 0.70749E+00_JPRB, 0.73588E+00_JPRB, 0.76412E+00_JPRB ,&
&  0.79028E+00_JPRB, 0.81721E+00_JPRB, 0.83898E+00_JPRB, 0.85982E+00_JPRB, 0.88251E+00_JPRB, 0.90149E+00_JPRB, 0.91772E+00_JPRB ,&
&  0.92876E+00_JPRB, 0.93409E+00_JPRB, 0.93224E+00_JPRB, 0.92907E+00_JPRB, 0.92305E+00_JPRB, 0.91807E+00_JPRB, 0.91706E+00_JPRB ,&
&  0.91593E+00_JPRB, 0.91682E+00_JPRB, 0.91864E+00_JPRB, 0.91642E+00_JPRB, 0.91306E+00_JPRB, 0.90941E+00_JPRB, 0.90645E+00_JPRB ,&
&  0.90300E+00_JPRB, 0.89992E+00_JPRB, 0.89711E+00_JPRB, 0.89404E+00_JPRB, 0.89102E+00_JPRB, 0.88586E+00_JPRB, 0.88128E+00_JPRB ,&
&  0.87637E+00_JPRB, 0.87397E+00_JPRB, 0.87013E+00_JPRB, 0.86701E+00_JPRB, 0.86399E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.80144E+00_JPRB, 0.80065E+00_JPRB, 0.80193E+00_JPRB, 0.80211E+00_JPRB, 0.80440E+00_JPRB, 0.80575E+00_JPRB, 0.80807E+00_JPRB ,&
&  0.80932E+00_JPRB, 0.81064E+00_JPRB, 0.80993E+00_JPRB, 0.80792E+00_JPRB, 0.80374E+00_JPRB, 0.79923E+00_JPRB, 0.79361E+00_JPRB ,&
&  0.78557E+00_JPRB, 0.77053E+00_JPRB, 0.74842E+00_JPRB, 0.72371E+00_JPRB, 0.69562E+00_JPRB, 0.66488E+00_JPRB, 0.63218E+00_JPRB ,&
&  0.59895E+00_JPRB, 0.56981E+00_JPRB, 0.53212E+00_JPRB, 0.51373E+00_JPRB, 0.49046E+00_JPRB, 0.47307E+00_JPRB, 0.46079E+00_JPRB ,&
&  0.44654E+00_JPRB, 0.43552E+00_JPRB, 0.42354E+00_JPRB, 0.41149E+00_JPRB, 0.40430E+00_JPRB, 0.39294E+00_JPRB, 0.38757E+00_JPRB ,&
&  0.38378E+00_JPRB, 0.38211E+00_JPRB, 0.38265E+00_JPRB, 0.38601E+00_JPRB, 0.39329E+00_JPRB, 0.40318E+00_JPRB, 0.42083E+00_JPRB ,&
&  0.45920E+00_JPRB, 0.51257E+00_JPRB, 0.59424E+00_JPRB, 0.61108E+00_JPRB, 0.61117E+00_JPRB, 0.62373E+00_JPRB, 0.63807E+00_JPRB ,&
&  0.65247E+00_JPRB, 0.66740E+00_JPRB, 0.67921E+00_JPRB, 0.68382E+00_JPRB, 0.69001E+00_JPRB, 0.69568E+00_JPRB, 0.70244E+00_JPRB ,&
&  0.71022E+00_JPRB, 0.72638E+00_JPRB, 0.74976E+00_JPRB, 0.77770E+00_JPRB, 0.80857E+00_JPRB, 0.83757E+00_JPRB, 0.86115E+00_JPRB ,&
&  0.87926E+00_JPRB, 0.89590E+00_JPRB, 0.92015E+00_JPRB, 0.94582E+00_JPRB, 0.97235E+00_JPRB, 0.98797E+00_JPRB, 0.10013E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.10122E+01_JPRB, 0.10043E+01_JPRB, 0.99352E+00_JPRB, 0.98508E+00_JPRB, 0.99000E+00_JPRB, 0.98822E+00_JPRB ,&
&  0.98642E+00_JPRB, 0.97773E+00_JPRB, 0.96228E+00_JPRB, 0.94629E+00_JPRB, 0.93552E+00_JPRB, 0.92619E+00_JPRB, 0.91970E+00_JPRB ,&
&  0.91230E+00_JPRB, 0.90345E+00_JPRB, 0.89283E+00_JPRB, 0.87889E+00_JPRB, 0.86457E+00_JPRB, 0.84871E+00_JPRB, 0.83608E+00_JPRB ,&
&  0.82419E+00_JPRB, 0.81668E+00_JPRB, 0.80984E+00_JPRB, 0.80598E+00_JPRB, 0.80231E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.75842E+00_JPRB, 0.76524E+00_JPRB, 0.77287E+00_JPRB, 0.77768E+00_JPRB, 0.78136E+00_JPRB, 0.78385E+00_JPRB, 0.78488E+00_JPRB ,&
&  0.78525E+00_JPRB, 0.78580E+00_JPRB, 0.78688E+00_JPRB, 0.78793E+00_JPRB, 0.78905E+00_JPRB, 0.79045E+00_JPRB, 0.79088E+00_JPRB ,&
&  0.79067E+00_JPRB, 0.78066E+00_JPRB, 0.75969E+00_JPRB, 0.73548E+00_JPRB, 0.70840E+00_JPRB, 0.67303E+00_JPRB, 0.63170E+00_JPRB ,&
&  0.59632E+00_JPRB, 0.57083E+00_JPRB, 0.54170E+00_JPRB, 0.52111E+00_JPRB, 0.50113E+00_JPRB, 0.49040E+00_JPRB, 0.47555E+00_JPRB ,&
&  0.47148E+00_JPRB, 0.43509E+00_JPRB, 0.42864E+00_JPRB, 0.40481E+00_JPRB, 0.38815E+00_JPRB, 0.37497E+00_JPRB, 0.36565E+00_JPRB ,&
&  0.36330E+00_JPRB, 0.35980E+00_JPRB, 0.35996E+00_JPRB, 0.36455E+00_JPRB, 0.37059E+00_JPRB, 0.38352E+00_JPRB, 0.39862E+00_JPRB ,&
&  0.43361E+00_JPRB, 0.54574E+00_JPRB, 0.58798E+00_JPRB, 0.60005E+00_JPRB, 0.60654E+00_JPRB, 0.62506E+00_JPRB, 0.64477E+00_JPRB ,&
&  0.67366E+00_JPRB, 0.69502E+00_JPRB, 0.71668E+00_JPRB, 0.72944E+00_JPRB, 0.75727E+00_JPRB, 0.78225E+00_JPRB, 0.80381E+00_JPRB ,&
&  0.82562E+00_JPRB, 0.84530E+00_JPRB, 0.86603E+00_JPRB, 0.88063E+00_JPRB, 0.88313E+00_JPRB, 0.88772E+00_JPRB, 0.89155E+00_JPRB ,&
&  0.90144E+00_JPRB, 0.91128E+00_JPRB, 0.94271E+00_JPRB, 0.98397E+00_JPRB, 0.10292E+01_JPRB, 0.10591E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.10963E+01_JPRB, 0.11070E+01_JPRB, 0.11197E+01_JPRB, 0.11080E+01_JPRB, 0.10853E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.10273E+01_JPRB, 0.10027E+01_JPRB, 0.98486E+00_JPRB, 0.96904E+00_JPRB, 0.95606E+00_JPRB, 0.93989E+00_JPRB ,&
&  0.91904E+00_JPRB, 0.89485E+00_JPRB, 0.86879E+00_JPRB, 0.83827E+00_JPRB, 0.81177E+00_JPRB, 0.78476E+00_JPRB, 0.76819E+00_JPRB ,&
&  0.75413E+00_JPRB, 0.74951E+00_JPRB, 0.74504E+00_JPRB, 0.74812E+00_JPRB, 0.75032E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.72345E+00_JPRB, 0.73906E+00_JPRB, 0.74961E+00_JPRB, 0.75329E+00_JPRB, 0.75643E+00_JPRB, 0.76158E+00_JPRB, 0.76379E+00_JPRB ,&
&  0.76606E+00_JPRB, 0.76629E+00_JPRB, 0.76831E+00_JPRB, 0.76933E+00_JPRB, 0.77215E+00_JPRB, 0.77343E+00_JPRB, 0.77375E+00_JPRB ,&
&  0.77346E+00_JPRB, 0.77079E+00_JPRB, 0.76107E+00_JPRB, 0.74186E+00_JPRB, 0.71586E+00_JPRB, 0.68611E+00_JPRB, 0.64547E+00_JPRB ,&
&  0.60539E+00_JPRB, 0.57805E+00_JPRB, 0.55218E+00_JPRB, 0.53221E+00_JPRB, 0.52087E+00_JPRB, 0.50779E+00_JPRB, 0.50522E+00_JPRB ,&
&  0.50238E+00_JPRB, 0.48972E+00_JPRB, 0.47242E+00_JPRB, 0.45125E+00_JPRB, 0.43090E+00_JPRB, 0.38462E+00_JPRB, 0.37899E+00_JPRB ,&
&  0.36772E+00_JPRB, 0.35732E+00_JPRB, 0.35461E+00_JPRB, 0.35613E+00_JPRB, 0.36094E+00_JPRB, 0.37520E+00_JPRB, 0.39478E+00_JPRB ,&
&  0.42252E+00_JPRB, 0.55478E+00_JPRB, 0.58763E+00_JPRB, 0.60273E+00_JPRB, 0.62835E+00_JPRB, 0.65450E+00_JPRB, 0.67621E+00_JPRB ,&
&  0.69941E+00_JPRB, 0.72179E+00_JPRB, 0.75452E+00_JPRB, 0.78652E+00_JPRB, 0.81968E+00_JPRB, 0.84812E+00_JPRB, 0.86607E+00_JPRB ,&
&  0.88347E+00_JPRB, 0.89050E+00_JPRB, 0.92732E+00_JPRB, 0.93996E+00_JPRB, 0.93408E+00_JPRB, 0.96689E+00_JPRB, 0.98817E+00_JPRB ,&
&  0.98688E+00_JPRB, 0.97635E+00_JPRB, 0.99956E+00_JPRB, 0.10557E+01_JPRB, 0.11350E+01_JPRB, 0.11972E+01_JPRB, 0.12163E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12130E+01_JPRB, 0.11968E+01_JPRB, 0.11893E+01_JPRB, 0.11536E+01_JPRB, 0.11085E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.10522E+01_JPRB, 0.10522E+01_JPRB, 0.10411E+01_JPRB, 0.10302E+01_JPRB, 0.10093E+01_JPRB, 0.98854E+00_JPRB, 0.95635E+00_JPRB ,&
&  0.90896E+00_JPRB, 0.85532E+00_JPRB, 0.81210E+00_JPRB, 0.77637E+00_JPRB, 0.75379E+00_JPRB, 0.73027E+00_JPRB, 0.71732E+00_JPRB ,&
&  0.70478E+00_JPRB, 0.70059E+00_JPRB, 0.69662E+00_JPRB, 0.70096E+00_JPRB, 0.70854E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.70779E+00_JPRB, 0.72687E+00_JPRB, 0.74448E+00_JPRB, 0.75612E+00_JPRB, 0.75915E+00_JPRB, 0.76135E+00_JPRB, 0.76124E+00_JPRB ,&
&  0.76262E+00_JPRB, 0.76394E+00_JPRB, 0.76440E+00_JPRB, 0.76118E+00_JPRB, 0.75787E+00_JPRB, 0.75307E+00_JPRB, 0.75118E+00_JPRB ,&
&  0.74833E+00_JPRB, 0.73754E+00_JPRB, 0.73006E+00_JPRB, 0.72162E+00_JPRB, 0.70631E+00_JPRB, 0.68429E+00_JPRB, 0.65389E+00_JPRB ,&
&  0.61611E+00_JPRB, 0.58432E+00_JPRB, 0.56125E+00_JPRB, 0.54185E+00_JPRB, 0.52765E+00_JPRB, 0.51888E+00_JPRB, 0.51481E+00_JPRB ,&
&  0.51611E+00_JPRB, 0.50868E+00_JPRB, 0.50290E+00_JPRB, 0.47563E+00_JPRB, 0.44289E+00_JPRB, 0.43082E+00_JPRB, 0.40386E+00_JPRB ,&
&  0.37906E+00_JPRB, 0.36065E+00_JPRB, 0.35615E+00_JPRB, 0.35646E+00_JPRB, 0.36412E+00_JPRB, 0.38167E+00_JPRB, 0.42038E+00_JPRB ,&
&  0.48529E+00_JPRB, 0.59571E+00_JPRB, 0.62200E+00_JPRB, 0.64983E+00_JPRB, 0.67829E+00_JPRB, 0.70564E+00_JPRB, 0.72504E+00_JPRB ,&
&  0.74439E+00_JPRB, 0.76487E+00_JPRB, 0.80351E+00_JPRB, 0.84338E+00_JPRB, 0.89942E+00_JPRB, 0.94802E+00_JPRB, 0.98634E+00_JPRB ,&
&  0.10217E+01_JPRB, 0.99047E+00_JPRB, 0.10039E+01_JPRB, 0.10173E+01_JPRB, 0.10630E+01_JPRB, 0.11066E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.11432E+01_JPRB, 0.11495E+01_JPRB, 0.11802E+01_JPRB, 0.12496E+01_JPRB, 0.13263E+01_JPRB, 0.13553E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.13150E+01_JPRB, 0.13648E+01_JPRB, 0.13028E+01_JPRB, 0.12611E+01_JPRB, 0.12011E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.10692E+01_JPRB, 0.10655E+01_JPRB, 0.10607E+01_JPRB, 0.10382E+01_JPRB, 0.10054E+01_JPRB, 0.94623E+00_JPRB ,&
&  0.86812E+00_JPRB, 0.80836E+00_JPRB, 0.76599E+00_JPRB, 0.74160E+00_JPRB, 0.72258E+00_JPRB, 0.70187E+00_JPRB, 0.68318E+00_JPRB ,&
&  0.66631E+00_JPRB, 0.65964E+00_JPRB, 0.66409E+00_JPRB, 0.67968E+00_JPRB, 0.69534E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.72337E+00_JPRB, 0.74385E+00_JPRB, 0.75636E+00_JPRB, 0.75370E+00_JPRB, 0.73847E+00_JPRB, 0.71612E+00_JPRB, 0.70129E+00_JPRB ,&
&  0.69751E+00_JPRB, 0.70088E+00_JPRB, 0.70579E+00_JPRB, 0.70429E+00_JPRB, 0.70453E+00_JPRB, 0.70593E+00_JPRB, 0.70952E+00_JPRB ,&
&  0.71221E+00_JPRB, 0.71529E+00_JPRB, 0.71794E+00_JPRB, 0.70983E+00_JPRB, 0.69575E+00_JPRB, 0.67633E+00_JPRB, 0.65299E+00_JPRB ,&
&  0.62507E+00_JPRB, 0.59946E+00_JPRB, 0.57652E+00_JPRB, 0.56022E+00_JPRB, 0.54551E+00_JPRB, 0.53906E+00_JPRB, 0.53468E+00_JPRB ,&
&  0.53090E+00_JPRB, 0.53022E+00_JPRB, 0.52567E+00_JPRB, 0.50879E+00_JPRB, 0.47909E+00_JPRB, 0.44866E+00_JPRB, 0.41838E+00_JPRB ,&
&  0.39459E+00_JPRB, 0.38029E+00_JPRB, 0.37251E+00_JPRB, 0.37158E+00_JPRB, 0.38320E+00_JPRB, 0.40966E+00_JPRB, 0.46946E+00_JPRB ,&
&  0.59040E+00_JPRB, 0.62686E+00_JPRB, 0.64675E+00_JPRB, 0.66800E+00_JPRB, 0.68945E+00_JPRB, 0.71084E+00_JPRB, 0.73398E+00_JPRB ,&
&  0.75999E+00_JPRB, 0.78514E+00_JPRB, 0.82136E+00_JPRB, 0.85081E+00_JPRB, 0.89429E+00_JPRB, 0.91564E+00_JPRB, 0.10118E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.12186E+01_JPRB, 0.11369E+01_JPRB, 0.11563E+01_JPRB, 0.12104E+01_JPRB, 0.12637E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.13447E+01_JPRB, 0.13498E+01_JPRB, 0.13523E+01_JPRB, 0.13562E+01_JPRB, 0.13810E+01_JPRB, 0.14065E+01_JPRB, 0.14106E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.14092E+01_JPRB, 0.14563E+01_JPRB, 0.12384E+01_JPRB, 0.11415E+01_JPRB, 0.11159E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.10497E+01_JPRB, 0.10475E+01_JPRB, 0.10691E+01_JPRB, 0.10819E+01_JPRB, 0.10216E+01_JPRB, 0.93917E+00_JPRB, 0.85023E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.76291E+00_JPRB, 0.73168E+00_JPRB, 0.71948E+00_JPRB, 0.70772E+00_JPRB, 0.68749E+00_JPRB, 0.65625E+00_JPRB ,&
&  0.64807E+00_JPRB, 0.65609E+00_JPRB, 0.66843E+00_JPRB, 0.69543E+00_JPRB, 0.71209E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.70841E+00_JPRB, 0.74546E+00_JPRB, 0.75072E+00_JPRB, 0.72528E+00_JPRB, 0.69907E+00_JPRB, 0.65484E+00_JPRB, 0.61797E+00_JPRB ,&
&  0.60203E+00_JPRB, 0.59622E+00_JPRB, 0.59759E+00_JPRB, 0.60082E+00_JPRB, 0.60803E+00_JPRB, 0.62009E+00_JPRB, 0.63318E+00_JPRB ,&
&  0.64060E+00_JPRB, 0.64554E+00_JPRB, 0.65175E+00_JPRB, 0.66035E+00_JPRB, 0.66188E+00_JPRB, 0.65014E+00_JPRB, 0.63406E+00_JPRB ,&
&  0.61029E+00_JPRB, 0.59103E+00_JPRB, 0.57562E+00_JPRB, 0.56513E+00_JPRB, 0.56120E+00_JPRB, 0.56330E+00_JPRB, 0.56740E+00_JPRB ,&
&  0.56577E+00_JPRB, 0.56076E+00_JPRB, 0.55726E+00_JPRB, 0.54576E+00_JPRB, 0.51611E+00_JPRB, 0.48101E+00_JPRB, 0.44101E+00_JPRB ,&
&  0.41437E+00_JPRB, 0.40449E+00_JPRB, 0.40761E+00_JPRB, 0.42440E+00_JPRB, 0.47118E+00_JPRB, 0.55982E+00_JPRB, 0.59425E+00_JPRB ,&
&  0.62495E+00_JPRB, 0.64511E+00_JPRB, 0.66313E+00_JPRB, 0.68492E+00_JPRB, 0.70349E+00_JPRB, 0.72693E+00_JPRB, 0.75324E+00_JPRB ,&
&  0.79374E+00_JPRB, 0.83556E+00_JPRB, 0.89081E+00_JPRB, 0.83041E+00_JPRB, 0.85481E+00_JPRB, 0.10032E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.12659E+01_JPRB, 0.12885E+01_JPRB, 0.13970E+01_JPRB, 0.14498E+01_JPRB, 0.15186E+01_JPRB, 0.15217E+01_JPRB ,&
&  0.14937E+01_JPRB, 0.14495E+01_JPRB, 0.14084E+01_JPRB, 0.13923E+01_JPRB, 0.14185E+01_JPRB, 0.14624E+01_JPRB, 0.14386E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.13766E+01_JPRB, 0.12608E+01_JPRB, 0.11086E+01_JPRB, 0.10508E+01_JPRB, 0.10374E+01_JPRB, 0.10463E+01_JPRB ,&
&  0.10390E+01_JPRB, 0.10453E+01_JPRB, 0.10591E+01_JPRB, 0.10715E+01_JPRB, 0.97364E+00_JPRB, 0.86621E+00_JPRB, 0.77345E+00_JPRB ,&
&  0.72617E+00_JPRB, 0.70817E+00_JPRB, 0.69096E+00_JPRB, 0.69789E+00_JPRB, 0.68794E+00_JPRB, 0.66767E+00_JPRB, 0.64535E+00_JPRB ,&
&  0.63799E+00_JPRB, 0.63787E+00_JPRB, 0.65696E+00_JPRB, 0.67935E+00_JPRB, 0.69233E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.75614E+00_JPRB, 0.76314E+00_JPRB, 0.73088E+00_JPRB, 0.70668E+00_JPRB, 0.68014E+00_JPRB, 0.65383E+00_JPRB, 0.62588E+00_JPRB ,&
&  0.61742E+00_JPRB, 0.59129E+00_JPRB, 0.56948E+00_JPRB, 0.55115E+00_JPRB, 0.54292E+00_JPRB, 0.54073E+00_JPRB, 0.55267E+00_JPRB ,&
&  0.57746E+00_JPRB, 0.60841E+00_JPRB, 0.62812E+00_JPRB, 0.62925E+00_JPRB, 0.62643E+00_JPRB, 0.61455E+00_JPRB, 0.60618E+00_JPRB ,&
&  0.58995E+00_JPRB, 0.58087E+00_JPRB, 0.58353E+00_JPRB, 0.59566E+00_JPRB, 0.60943E+00_JPRB, 0.61166E+00_JPRB, 0.60038E+00_JPRB ,&
&  0.58463E+00_JPRB, 0.57522E+00_JPRB, 0.57503E+00_JPRB, 0.57301E+00_JPRB, 0.55056E+00_JPRB, 0.50752E+00_JPRB, 0.46454E+00_JPRB ,&
&  0.43600E+00_JPRB, 0.43306E+00_JPRB, 0.46874E+00_JPRB, 0.55379E+00_JPRB, 0.58780E+00_JPRB, 0.61105E+00_JPRB, 0.62811E+00_JPRB ,&
&  0.62260E+00_JPRB, 0.63086E+00_JPRB, 0.67135E+00_JPRB, 0.74475E+00_JPRB, 0.76183E+00_JPRB, 0.79000E+00_JPRB, 0.82288E+00_JPRB ,&
&  0.87250E+00_JPRB, 0.88245E+00_JPRB, 0.92437E+00_JPRB, 0.91574E+00_JPRB, 0.10756E+01_JPRB, 0.14433E+01_JPRB, 0.14056E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14916E+01_JPRB, 0.16068E+01_JPRB, 0.16493E+01_JPRB, 0.16483E+01_JPRB, 0.16546E+01_JPRB, 0.16193E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.14819E+01_JPRB, 0.14313E+01_JPRB, 0.14421E+01_JPRB, 0.14457E+01_JPRB, 0.14541E+01_JPRB, 0.14455E+01_JPRB ,&
&  0.14284E+01_JPRB, 0.13923E+01_JPRB, 0.12950E+01_JPRB, 0.11709E+01_JPRB, 0.11047E+01_JPRB, 0.10468E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.98582E+00_JPRB, 0.10022E+01_JPRB, 0.10393E+01_JPRB, 0.10718E+01_JPRB, 0.99926E+00_JPRB, 0.86742E+00_JPRB, 0.75284E+00_JPRB ,&
&  0.69088E+00_JPRB, 0.66696E+00_JPRB, 0.64129E+00_JPRB, 0.63615E+00_JPRB, 0.63943E+00_JPRB, 0.63909E+00_JPRB, 0.63629E+00_JPRB ,&
&  0.63638E+00_JPRB, 0.64613E+00_JPRB, 0.68191E+00_JPRB, 0.72027E+00_JPRB, 0.74011E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.75297E+00_JPRB, 0.76015E+00_JPRB, 0.73941E+00_JPRB, 0.72649E+00_JPRB, 0.71286E+00_JPRB, 0.68290E+00_JPRB, 0.63960E+00_JPRB ,&
&  0.63132E+00_JPRB, 0.57991E+00_JPRB, 0.55652E+00_JPRB, 0.54705E+00_JPRB, 0.54804E+00_JPRB, 0.54341E+00_JPRB, 0.53405E+00_JPRB ,&
&  0.54168E+00_JPRB, 0.56627E+00_JPRB, 0.60648E+00_JPRB, 0.62504E+00_JPRB, 0.62960E+00_JPRB, 0.62008E+00_JPRB, 0.60113E+00_JPRB ,&
&  0.58189E+00_JPRB, 0.57868E+00_JPRB, 0.59764E+00_JPRB, 0.63319E+00_JPRB, 0.65010E+00_JPRB, 0.64665E+00_JPRB, 0.63387E+00_JPRB ,&
&  0.61679E+00_JPRB, 0.60353E+00_JPRB, 0.59491E+00_JPRB, 0.58918E+00_JPRB, 0.57521E+00_JPRB, 0.55809E+00_JPRB, 0.51483E+00_JPRB ,&
&  0.48110E+00_JPRB, 0.48829E+00_JPRB, 0.56600E+00_JPRB, 0.60934E+00_JPRB, 0.63324E+00_JPRB, 0.64298E+00_JPRB, 0.65630E+00_JPRB ,&
&  0.68044E+00_JPRB, 0.71241E+00_JPRB, 0.75264E+00_JPRB, 0.80194E+00_JPRB, 0.84221E+00_JPRB, 0.88498E+00_JPRB, 0.93373E+00_JPRB ,&
&  0.10257E+01_JPRB, 0.88101E+00_JPRB, 0.86244E+00_JPRB, 0.94686E+00_JPRB, 0.13260E+01_JPRB, 0.15132E+01_JPRB, 0.15927E+01_JPRB ,&
&  0.16790E+01_JPRB, 0.16428E+01_JPRB, 0.16891E+01_JPRB, 0.17792E+01_JPRB, 0.18231E+01_JPRB, 0.18229E+01_JPRB, 0.17838E+01_JPRB ,&
&  0.17222E+01_JPRB, 0.16600E+01_JPRB, 0.15755E+01_JPRB, 0.15335E+01_JPRB, 0.14890E+01_JPRB, 0.14959E+01_JPRB, 0.14728E+01_JPRB ,&
&  0.14334E+01_JPRB, 0.14053E+01_JPRB, 0.13135E+01_JPRB, 0.12117E+01_JPRB, 0.11322E+01_JPRB, 0.10807E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.10123E+01_JPRB, 0.10155E+01_JPRB, 0.10403E+01_JPRB, 0.10950E+01_JPRB, 0.10731E+01_JPRB, 0.96275E+00_JPRB, 0.85594E+00_JPRB ,&
&  0.75784E+00_JPRB, 0.70281E+00_JPRB, 0.65365E+00_JPRB, 0.64229E+00_JPRB, 0.65083E+00_JPRB, 0.66210E+00_JPRB, 0.67737E+00_JPRB ,&
&  0.69768E+00_JPRB, 0.71053E+00_JPRB, 0.72297E+00_JPRB, 0.74426E+00_JPRB, 0.74669E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.75298E+00_JPRB, 0.76912E+00_JPRB, 0.75687E+00_JPRB, 0.75221E+00_JPRB, 0.74441E+00_JPRB, 0.70235E+00_JPRB, 0.67400E+00_JPRB ,&
&  0.69849E+00_JPRB, 0.65315E+00_JPRB, 0.67930E+00_JPRB, 0.60438E+00_JPRB, 0.57404E+00_JPRB, 0.56719E+00_JPRB, 0.55213E+00_JPRB ,&
&  0.55438E+00_JPRB, 0.57506E+00_JPRB, 0.60107E+00_JPRB, 0.62227E+00_JPRB, 0.63995E+00_JPRB, 0.62372E+00_JPRB, 0.60632E+00_JPRB ,&
&  0.58650E+00_JPRB, 0.58583E+00_JPRB, 0.61168E+00_JPRB, 0.63553E+00_JPRB, 0.65717E+00_JPRB, 0.66624E+00_JPRB, 0.66138E+00_JPRB ,&
&  0.63265E+00_JPRB, 0.62507E+00_JPRB, 0.62452E+00_JPRB, 0.61332E+00_JPRB, 0.60036E+00_JPRB, 0.58964E+00_JPRB, 0.57396E+00_JPRB ,&
&  0.55951E+00_JPRB, 0.56336E+00_JPRB, 0.61202E+00_JPRB, 0.64514E+00_JPRB, 0.66163E+00_JPRB, 0.66513E+00_JPRB, 0.67840E+00_JPRB ,&
&  0.69614E+00_JPRB, 0.72445E+00_JPRB, 0.77393E+00_JPRB, 0.84576E+00_JPRB, 0.91442E+00_JPRB, 0.99186E+00_JPRB, 0.10793E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.97392E+00_JPRB, 0.11326E+01_JPRB, 0.11868E+01_JPRB, 0.14022E+01_JPRB, 0.16450E+01_JPRB, 0.16853E+01_JPRB ,&
&  0.19029E+01_JPRB, 0.18694E+01_JPRB, 0.19164E+01_JPRB, 0.19453E+01_JPRB, 0.19882E+01_JPRB, 0.20068E+01_JPRB, 0.19891E+01_JPRB ,&
&  0.20097E+01_JPRB, 0.17919E+01_JPRB, 0.17837E+01_JPRB, 0.16899E+01_JPRB, 0.16112E+01_JPRB, 0.15905E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.14666E+01_JPRB, 0.14299E+01_JPRB, 0.13722E+01_JPRB, 0.12157E+01_JPRB, 0.11673E+01_JPRB, 0.11274E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.10061E+01_JPRB, 0.97174E+00_JPRB, 0.94644E+00_JPRB, 0.10628E+01_JPRB, 0.10777E+01_JPRB, 0.97714E+00_JPRB, 0.85695E+00_JPRB ,&
&  0.77749E+00_JPRB, 0.71956E+00_JPRB, 0.73806E+00_JPRB, 0.73705E+00_JPRB, 0.71646E+00_JPRB, 0.71467E+00_JPRB, 0.76374E+00_JPRB ,&
&  0.74651E+00_JPRB, 0.75472E+00_JPRB, 0.78433E+00_JPRB, 0.78032E+00_JPRB, 0.76057E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.77146E+00_JPRB, 0.79561E+00_JPRB, 0.77602E+00_JPRB, 0.77607E+00_JPRB, 0.77803E+00_JPRB, 0.74040E+00_JPRB, 0.70028E+00_JPRB ,&
&  0.70619E+00_JPRB, 0.70615E+00_JPRB, 0.71019E+00_JPRB, 0.71174E+00_JPRB, 0.71508E+00_JPRB, 0.66022E+00_JPRB, 0.56598E+00_JPRB ,&
&  0.54144E+00_JPRB, 0.56357E+00_JPRB, 0.60655E+00_JPRB, 0.63732E+00_JPRB, 0.63999E+00_JPRB, 0.62741E+00_JPRB, 0.62025E+00_JPRB ,&
&  0.61784E+00_JPRB, 0.61875E+00_JPRB, 0.63848E+00_JPRB, 0.65291E+00_JPRB, 0.66096E+00_JPRB, 0.67621E+00_JPRB, 0.68588E+00_JPRB ,&
&  0.66638E+00_JPRB, 0.66837E+00_JPRB, 0.66335E+00_JPRB, 0.63116E+00_JPRB, 0.63908E+00_JPRB, 0.62268E+00_JPRB, 0.60373E+00_JPRB ,&
&  0.60572E+00_JPRB, 0.62679E+00_JPRB, 0.64959E+00_JPRB, 0.67792E+00_JPRB, 0.69624E+00_JPRB, 0.70299E+00_JPRB, 0.71223E+00_JPRB ,&
&  0.73416E+00_JPRB, 0.77511E+00_JPRB, 0.82793E+00_JPRB, 0.90084E+00_JPRB, 0.99737E+00_JPRB, 0.10788E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.12957E+01_JPRB, 0.13436E+01_JPRB, 0.14036E+01_JPRB, 0.13345E+01_JPRB, 0.15267E+01_JPRB, 0.17614E+01_JPRB, 0.18041E+01_JPRB ,&
&  0.19288E+01_JPRB, 0.18683E+01_JPRB, 0.20158E+01_JPRB, 0.21897E+01_JPRB, 0.23257E+01_JPRB, 0.22979E+01_JPRB, 0.22306E+01_JPRB ,&
&  0.22652E+01_JPRB, 0.21957E+01_JPRB, 0.19622E+01_JPRB, 0.18601E+01_JPRB, 0.17545E+01_JPRB, 0.17130E+01_JPRB, 0.16014E+01_JPRB ,&
&  0.15360E+01_JPRB, 0.15039E+01_JPRB, 0.14455E+01_JPRB, 0.13731E+01_JPRB, 0.12862E+01_JPRB, 0.12042E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.10404E+01_JPRB, 0.92632E+00_JPRB, 0.89389E+00_JPRB, 0.97390E+00_JPRB, 0.10551E+01_JPRB, 0.93842E+00_JPRB, 0.80879E+00_JPRB ,&
&  0.75902E+00_JPRB, 0.76357E+00_JPRB, 0.83283E+00_JPRB, 0.83297E+00_JPRB, 0.81609E+00_JPRB, 0.80573E+00_JPRB, 0.77248E+00_JPRB ,&
&  0.73114E+00_JPRB, 0.78684E+00_JPRB, 0.79952E+00_JPRB, 0.78659E+00_JPRB, 0.76756E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.80971E+00_JPRB, 0.84809E+00_JPRB, 0.82332E+00_JPRB, 0.81678E+00_JPRB, 0.82229E+00_JPRB, 0.77212E+00_JPRB, 0.75211E+00_JPRB ,&
&  0.75009E+00_JPRB, 0.74644E+00_JPRB, 0.73810E+00_JPRB, 0.73599E+00_JPRB, 0.73613E+00_JPRB, 0.72594E+00_JPRB, 0.66365E+00_JPRB ,&
&  0.59590E+00_JPRB, 0.62517E+00_JPRB, 0.62477E+00_JPRB, 0.65896E+00_JPRB, 0.69413E+00_JPRB, 0.66890E+00_JPRB, 0.65227E+00_JPRB ,&
&  0.65992E+00_JPRB, 0.65312E+00_JPRB, 0.65440E+00_JPRB, 0.66846E+00_JPRB, 0.69217E+00_JPRB, 0.73001E+00_JPRB, 0.74823E+00_JPRB ,&
&  0.72180E+00_JPRB, 0.71411E+00_JPRB, 0.69466E+00_JPRB, 0.68144E+00_JPRB, 0.67970E+00_JPRB, 0.67093E+00_JPRB, 0.65751E+00_JPRB ,&
&  0.65821E+00_JPRB, 0.67424E+00_JPRB, 0.69226E+00_JPRB, 0.72258E+00_JPRB, 0.74927E+00_JPRB, 0.76737E+00_JPRB, 0.78660E+00_JPRB ,&
&  0.83528E+00_JPRB, 0.89444E+00_JPRB, 0.94058E+00_JPRB, 0.10115E+01_JPRB, 0.10999E+01_JPRB, 0.12880E+01_JPRB, 0.14415E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.15444E+01_JPRB, 0.15723E+01_JPRB, 0.16267E+01_JPRB, 0.18096E+01_JPRB, 0.19427E+01_JPRB, 0.20306E+01_JPRB ,&
&  0.21710E+01_JPRB, 0.21084E+01_JPRB, 0.24925E+01_JPRB, 0.26281E+01_JPRB, 0.25821E+01_JPRB, 0.25888E+01_JPRB, 0.24371E+01_JPRB ,&
&  0.23678E+01_JPRB, 0.21925E+01_JPRB, 0.20887E+01_JPRB, 0.19896E+01_JPRB, 0.18855E+01_JPRB, 0.18651E+01_JPRB, 0.17412E+01_JPRB ,&
&  0.17018E+01_JPRB, 0.16429E+01_JPRB, 0.14301E+01_JPRB, 0.12755E+01_JPRB, 0.11044E+01_JPRB, 0.11618E+01_JPRB, 0.11222E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.88287E+00_JPRB, 0.95278E+00_JPRB, 0.10638E+01_JPRB, 0.11734E+01_JPRB, 0.11744E+01_JPRB, 0.97360E+00_JPRB ,&
&  0.86034E+00_JPRB, 0.87396E+00_JPRB, 0.88383E+00_JPRB, 0.90192E+00_JPRB, 0.91156E+00_JPRB, 0.91920E+00_JPRB, 0.83265E+00_JPRB ,&
&  0.77369E+00_JPRB, 0.80972E+00_JPRB, 0.81992E+00_JPRB, 0.80722E+00_JPRB, 0.79857E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.86024E+00_JPRB, 0.90908E+00_JPRB, 0.90584E+00_JPRB, 0.87556E+00_JPRB, 0.86761E+00_JPRB, 0.81238E+00_JPRB, 0.79119E+00_JPRB ,&
&  0.79548E+00_JPRB, 0.79109E+00_JPRB, 0.76726E+00_JPRB, 0.76275E+00_JPRB, 0.76101E+00_JPRB, 0.75119E+00_JPRB, 0.73556E+00_JPRB ,&
&  0.63927E+00_JPRB, 0.67807E+00_JPRB, 0.63625E+00_JPRB, 0.66517E+00_JPRB, 0.78424E+00_JPRB, 0.75264E+00_JPRB, 0.73418E+00_JPRB ,&
&  0.71629E+00_JPRB, 0.70766E+00_JPRB, 0.68879E+00_JPRB, 0.69539E+00_JPRB, 0.72434E+00_JPRB, 0.78751E+00_JPRB, 0.84869E+00_JPRB ,&
&  0.83822E+00_JPRB, 0.81008E+00_JPRB, 0.77979E+00_JPRB, 0.75940E+00_JPRB, 0.73648E+00_JPRB, 0.73057E+00_JPRB, 0.74377E+00_JPRB ,&
&  0.75095E+00_JPRB, 0.75296E+00_JPRB, 0.77274E+00_JPRB, 0.81026E+00_JPRB, 0.84119E+00_JPRB, 0.86399E+00_JPRB, 0.91312E+00_JPRB ,&
&  0.98988E+00_JPRB, 0.10517E+01_JPRB, 0.10884E+01_JPRB, 0.11536E+01_JPRB, 0.12252E+01_JPRB, 0.14035E+01_JPRB, 0.15778E+01_JPRB ,&
&  0.18348E+01_JPRB, 0.18550E+01_JPRB, 0.19783E+01_JPRB, 0.22393E+01_JPRB, 0.23799E+01_JPRB, 0.24039E+01_JPRB, 0.23889E+01_JPRB ,&
&  0.24459E+01_JPRB, 0.24018E+01_JPRB, 0.25428E+01_JPRB, 0.27094E+01_JPRB, 0.26403E+01_JPRB, 0.27314E+01_JPRB, 0.25487E+01_JPRB ,&
&  0.22610E+01_JPRB, 0.20997E+01_JPRB, 0.20245E+01_JPRB, 0.19666E+01_JPRB, 0.18541E+01_JPRB, 0.18912E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.18003E+01_JPRB, 0.15979E+01_JPRB, 0.12112E+01_JPRB, 0.11171E+01_JPRB, 0.89385E+00_JPRB, 0.91875E+00_JPRB, 0.11444E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.10534E+01_JPRB, 0.11106E+01_JPRB, 0.12470E+01_JPRB, 0.14115E+01_JPRB, 0.16373E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.11375E+01_JPRB, 0.10759E+01_JPRB, 0.10616E+01_JPRB, 0.10589E+01_JPRB, 0.10567E+01_JPRB, 0.10803E+01_JPRB, 0.10387E+01_JPRB ,&
&  0.97839E+00_JPRB, 0.92224E+00_JPRB, 0.89435E+00_JPRB, 0.86776E+00_JPRB, 0.85826E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.91336E+00_JPRB, 0.95452E+00_JPRB, 0.96844E+00_JPRB, 0.93343E+00_JPRB, 0.91061E+00_JPRB, 0.86516E+00_JPRB, 0.83299E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.82585E+00_JPRB, 0.79787E+00_JPRB, 0.79255E+00_JPRB, 0.78600E+00_JPRB, 0.76835E+00_JPRB, 0.76305E+00_JPRB ,&
&  0.80048E+00_JPRB, 0.82103E+00_JPRB, 0.73197E+00_JPRB, 0.67567E+00_JPRB, 0.77407E+00_JPRB, 0.76289E+00_JPRB, 0.78803E+00_JPRB ,&
&  0.76494E+00_JPRB, 0.79397E+00_JPRB, 0.79130E+00_JPRB, 0.80541E+00_JPRB, 0.86663E+00_JPRB, 0.94682E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10536E+01_JPRB, 0.10146E+01_JPRB, 0.98196E+00_JPRB, 0.98014E+00_JPRB, 0.92541E+00_JPRB, 0.93634E+00_JPRB ,&
&  0.92095E+00_JPRB, 0.89160E+00_JPRB, 0.88107E+00_JPRB, 0.90663E+00_JPRB, 0.93560E+00_JPRB, 0.97069E+00_JPRB, 0.10388E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.11354E+01_JPRB, 0.11368E+01_JPRB, 0.11566E+01_JPRB, 0.12314E+01_JPRB, 0.13591E+01_JPRB, 0.15173E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.18270E+01_JPRB, 0.19047E+01_JPRB, 0.22615E+01_JPRB, 0.24661E+01_JPRB, 0.24593E+01_JPRB, 0.24614E+01_JPRB ,&
&  0.25492E+01_JPRB, 0.26036E+01_JPRB, 0.27574E+01_JPRB, 0.28352E+01_JPRB, 0.28593E+01_JPRB, 0.28225E+01_JPRB, 0.25631E+01_JPRB ,&
&  0.22120E+01_JPRB, 0.20074E+01_JPRB, 0.19454E+01_JPRB, 0.18464E+01_JPRB, 0.17706E+01_JPRB, 0.16378E+01_JPRB, 0.16993E+01_JPRB ,&
&  0.18276E+01_JPRB, 0.10524E+01_JPRB, 0.85045E+00_JPRB, 0.99871E+00_JPRB, 0.87153E+00_JPRB, 0.89977E+00_JPRB, 0.10366E+01_JPRB ,&
&  0.10106E+01_JPRB, 0.10094E+01_JPRB, 0.12139E+01_JPRB, 0.13829E+01_JPRB, 0.16097E+01_JPRB, 0.19296E+01_JPRB, 0.17368E+01_JPRB ,&
&  0.15102E+01_JPRB, 0.13912E+01_JPRB, 0.13922E+01_JPRB, 0.13457E+01_JPRB, 0.13599E+01_JPRB, 0.13309E+01_JPRB, 0.12764E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.11205E+01_JPRB, 0.10463E+01_JPRB, 0.98385E+00_JPRB, 0.93676E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.10161E+01_JPRB, 0.10349E+01_JPRB, 0.10290E+01_JPRB, 0.99464E+00_JPRB, 0.95680E+00_JPRB, 0.93225E+00_JPRB, 0.89199E+00_JPRB ,&
&  0.87287E+00_JPRB, 0.85621E+00_JPRB, 0.82901E+00_JPRB, 0.82056E+00_JPRB, 0.79954E+00_JPRB, 0.77786E+00_JPRB, 0.77418E+00_JPRB ,&
&  0.80237E+00_JPRB, 0.84412E+00_JPRB, 0.71693E+00_JPRB, 0.68409E+00_JPRB, 0.66154E+00_JPRB, 0.71631E+00_JPRB, 0.76905E+00_JPRB ,&
&  0.83064E+00_JPRB, 0.84890E+00_JPRB, 0.88445E+00_JPRB, 0.95532E+00_JPRB, 0.10750E+01_JPRB, 0.11510E+01_JPRB, 0.12611E+01_JPRB ,&
&  0.13003E+01_JPRB, 0.13583E+01_JPRB, 0.12628E+01_JPRB, 0.12712E+01_JPRB, 0.12811E+01_JPRB, 0.11984E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.10679E+01_JPRB, 0.10387E+01_JPRB, 0.10209E+01_JPRB, 0.10244E+01_JPRB, 0.10295E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.11182E+01_JPRB, 0.11108E+01_JPRB, 0.11138E+01_JPRB, 0.11630E+01_JPRB, 0.12571E+01_JPRB, 0.13225E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.14874E+01_JPRB, 0.12774E+01_JPRB, 0.18522E+01_JPRB, 0.25050E+01_JPRB, 0.23757E+01_JPRB, 0.21369E+01_JPRB ,&
&  0.23487E+01_JPRB, 0.25278E+01_JPRB, 0.27630E+01_JPRB, 0.28561E+01_JPRB, 0.28650E+01_JPRB, 0.27552E+01_JPRB, 0.25261E+01_JPRB ,&
&  0.22389E+01_JPRB, 0.21173E+01_JPRB, 0.20994E+01_JPRB, 0.21988E+01_JPRB, 0.21023E+01_JPRB, 0.19946E+01_JPRB, 0.18935E+01_JPRB ,&
&  0.16427E+01_JPRB, 0.14179E+01_JPRB, 0.10191E+01_JPRB, 0.10609E+01_JPRB, 0.86129E+00_JPRB, 0.87384E+00_JPRB, 0.10023E+01_JPRB ,&
&  0.97942E+00_JPRB, 0.10637E+01_JPRB, 0.12921E+01_JPRB, 0.15192E+01_JPRB, 0.20249E+01_JPRB, 0.22158E+01_JPRB, 0.21588E+01_JPRB ,&
&  0.18930E+01_JPRB, 0.16423E+01_JPRB, 0.16910E+01_JPRB, 0.16703E+01_JPRB, 0.16774E+01_JPRB, 0.16055E+01_JPRB, 0.15120E+01_JPRB ,&
&  0.13982E+01_JPRB, 0.12955E+01_JPRB, 0.12247E+01_JPRB, 0.11325E+01_JPRB, 0.10440E+01_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.11451E+01_JPRB, 0.11250E+01_JPRB, 0.10797E+01_JPRB, 0.10194E+01_JPRB, 0.96804E+00_JPRB, 0.94706E+00_JPRB, 0.92542E+00_JPRB ,&
&  0.91111E+00_JPRB, 0.88416E+00_JPRB, 0.86066E+00_JPRB, 0.83861E+00_JPRB, 0.81677E+00_JPRB, 0.79545E+00_JPRB, 0.79546E+00_JPRB ,&
&  0.81894E+00_JPRB, 0.77205E+00_JPRB, 0.64728E+00_JPRB, 0.60032E+00_JPRB, 0.58007E+00_JPRB, 0.63466E+00_JPRB, 0.71010E+00_JPRB ,&
&  0.80230E+00_JPRB, 0.86158E+00_JPRB, 0.97484E+00_JPRB, 0.11271E+01_JPRB, 0.12937E+01_JPRB, 0.14292E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.18231E+01_JPRB, 0.16713E+01_JPRB, 0.16844E+01_JPRB, 0.15968E+01_JPRB, 0.15577E+01_JPRB, 0.14280E+01_JPRB, 0.13019E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.11337E+01_JPRB, 0.10696E+01_JPRB, 0.10187E+01_JPRB, 0.99966E+00_JPRB, 0.10171E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.11252E+01_JPRB, 0.10915E+01_JPRB, 0.11155E+01_JPRB, 0.12021E+01_JPRB, 0.12832E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.13511E+01_JPRB, 0.14108E+01_JPRB, 0.19549E+01_JPRB, 0.21438E+01_JPRB, 0.24010E+01_JPRB, 0.24226E+01_JPRB, 0.23246E+01_JPRB ,&
&  0.24777E+01_JPRB, 0.25542E+01_JPRB, 0.26838E+01_JPRB, 0.22296E+01_JPRB, 0.25519E+01_JPRB, 0.25192E+01_JPRB, 0.23497E+01_JPRB ,&
&  0.21998E+01_JPRB, 0.22158E+01_JPRB, 0.21777E+01_JPRB, 0.22557E+01_JPRB, 0.23425E+01_JPRB, 0.24403E+01_JPRB, 0.19013E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.16407E+01_JPRB, 0.14776E+01_JPRB, 0.12443E+01_JPRB, 0.10661E+01_JPRB, 0.10951E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.12775E+01_JPRB, 0.15286E+01_JPRB, 0.20264E+01_JPRB, 0.26050E+01_JPRB, 0.28224E+01_JPRB, 0.26179E+01_JPRB ,&
&  0.21901E+01_JPRB, 0.22397E+01_JPRB, 0.21530E+01_JPRB, 0.20569E+01_JPRB, 0.20090E+01_JPRB, 0.19027E+01_JPRB, 0.17457E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.14545E+01_JPRB, 0.13520E+01_JPRB, 0.12493E+01_JPRB, 0.11740E+01_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.12271E+01_JPRB, 0.11921E+01_JPRB, 0.11336E+01_JPRB, 0.10696E+01_JPRB, 0.10116E+01_JPRB, 0.98336E+00_JPRB, 0.96237E+00_JPRB ,&
&  0.95789E+00_JPRB, 0.90364E+00_JPRB, 0.87917E+00_JPRB, 0.85402E+00_JPRB, 0.83390E+00_JPRB, 0.82162E+00_JPRB, 0.82781E+00_JPRB ,&
&  0.84200E+00_JPRB, 0.73571E+00_JPRB, 0.60828E+00_JPRB, 0.61284E+00_JPRB, 0.64271E+00_JPRB, 0.64075E+00_JPRB, 0.70627E+00_JPRB ,&
&  0.79259E+00_JPRB, 0.89836E+00_JPRB, 0.10467E+01_JPRB, 0.13152E+01_JPRB, 0.15699E+01_JPRB, 0.17960E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.20412E+01_JPRB, 0.20909E+01_JPRB, 0.20262E+01_JPRB, 0.18271E+01_JPRB, 0.15858E+01_JPRB, 0.14027E+01_JPRB, 0.12965E+01_JPRB ,&
&  0.12150E+01_JPRB, 0.11289E+01_JPRB, 0.10745E+01_JPRB, 0.10490E+01_JPRB, 0.10500E+01_JPRB, 0.10854E+01_JPRB, 0.11397E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11394E+01_JPRB, 0.11194E+01_JPRB, 0.11796E+01_JPRB, 0.11624E+01_JPRB, 0.11742E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.14985E+01_JPRB, 0.17748E+01_JPRB, 0.19076E+01_JPRB, 0.21390E+01_JPRB, 0.19183E+01_JPRB, 0.20362E+01_JPRB, 0.23893E+01_JPRB ,&
&  0.25966E+01_JPRB, 0.26228E+01_JPRB, 0.27246E+01_JPRB, 0.19483E+01_JPRB, 0.17278E+01_JPRB, 0.23311E+01_JPRB, 0.23018E+01_JPRB ,&
&  0.21653E+01_JPRB, 0.22812E+01_JPRB, 0.23658E+01_JPRB, 0.26051E+01_JPRB, 0.20315E+01_JPRB, 0.18065E+01_JPRB, 0.13761E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.12361E+01_JPRB, 0.14090E+01_JPRB, 0.13230E+01_JPRB, 0.13556E+01_JPRB, 0.13499E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.15388E+01_JPRB, 0.20213E+01_JPRB, 0.28570E+01_JPRB, 0.36325E+01_JPRB, 0.32817E+01_JPRB, 0.32506E+01_JPRB ,&
&  0.31050E+01_JPRB, 0.29165E+01_JPRB, 0.25934E+01_JPRB, 0.25258E+01_JPRB, 0.23709E+01_JPRB, 0.21420E+01_JPRB, 0.18773E+01_JPRB ,&
&  0.16633E+01_JPRB, 0.15351E+01_JPRB, 0.14188E+01_JPRB, 0.13234E+01_JPRB, 0.12617E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.13062E+01_JPRB, 0.12555E+01_JPRB, 0.11974E+01_JPRB, 0.11317E+01_JPRB, 0.10753E+01_JPRB, 0.10369E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.10081E+01_JPRB, 0.94291E+00_JPRB, 0.88674E+00_JPRB, 0.86038E+00_JPRB, 0.85008E+00_JPRB, 0.84888E+00_JPRB, 0.85939E+00_JPRB ,&
&  0.88447E+00_JPRB, 0.84631E+00_JPRB, 0.66848E+00_JPRB, 0.62142E+00_JPRB, 0.70256E+00_JPRB, 0.68831E+00_JPRB, 0.71075E+00_JPRB ,&
&  0.82629E+00_JPRB, 0.98352E+00_JPRB, 0.12262E+01_JPRB, 0.16262E+01_JPRB, 0.19569E+01_JPRB, 0.22844E+01_JPRB, 0.24616E+01_JPRB ,&
&  0.25310E+01_JPRB, 0.23189E+01_JPRB, 0.19899E+01_JPRB, 0.16708E+01_JPRB, 0.14405E+01_JPRB, 0.12943E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.11511E+01_JPRB, 0.11074E+01_JPRB, 0.10849E+01_JPRB, 0.10647E+01_JPRB, 0.10930E+01_JPRB, 0.11096E+01_JPRB, 0.11491E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.11547E+01_JPRB, 0.11749E+01_JPRB, 0.12128E+01_JPRB, 0.12174E+01_JPRB, 0.11827E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.15782E+01_JPRB, 0.17990E+01_JPRB, 0.20036E+01_JPRB, 0.21951E+01_JPRB, 0.22556E+01_JPRB, 0.25518E+01_JPRB ,&
&  0.21840E+01_JPRB, 0.18023E+01_JPRB, 0.15339E+01_JPRB, 0.12721E+01_JPRB, 0.15864E+01_JPRB, 0.22656E+01_JPRB, 0.23846E+01_JPRB ,&
&  0.24209E+01_JPRB, 0.25532E+01_JPRB, 0.26579E+01_JPRB, 0.23335E+01_JPRB, 0.19069E+01_JPRB, 0.15231E+01_JPRB, 0.18092E+01_JPRB ,&
&  0.18860E+01_JPRB, 0.19032E+01_JPRB, 0.17811E+01_JPRB, 0.15766E+01_JPRB, 0.13812E+01_JPRB, 0.16253E+01_JPRB, 0.16374E+01_JPRB ,&
&  0.17094E+01_JPRB, 0.19248E+01_JPRB, 0.32490E+01_JPRB, 0.43373E+01_JPRB, 0.44379E+01_JPRB, 0.40764E+01_JPRB, 0.41240E+01_JPRB ,&
&  0.37307E+01_JPRB, 0.35062E+01_JPRB, 0.31372E+01_JPRB, 0.27551E+01_JPRB, 0.24085E+01_JPRB, 0.21139E+01_JPRB, 0.18760E+01_JPRB ,&
&  0.17156E+01_JPRB, 0.15944E+01_JPRB, 0.14705E+01_JPRB, 0.14005E+01_JPRB, 0.13484E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.13976E+01_JPRB, 0.13374E+01_JPRB, 0.12805E+01_JPRB, 0.12190E+01_JPRB, 0.11635E+01_JPRB, 0.10999E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10522E+01_JPRB, 0.98100E+00_JPRB, 0.91374E+00_JPRB, 0.88656E+00_JPRB, 0.89473E+00_JPRB, 0.90433E+00_JPRB, 0.91162E+00_JPRB ,&
&  0.93121E+00_JPRB, 0.10405E+01_JPRB, 0.81461E+00_JPRB, 0.70779E+00_JPRB, 0.73321E+00_JPRB, 0.73948E+00_JPRB, 0.70502E+00_JPRB ,&
&  0.88864E+00_JPRB, 0.11438E+01_JPRB, 0.14433E+01_JPRB, 0.18817E+01_JPRB, 0.24252E+01_JPRB, 0.24343E+01_JPRB, 0.25161E+01_JPRB ,&
&  0.25052E+01_JPRB, 0.21980E+01_JPRB, 0.18706E+01_JPRB, 0.16071E+01_JPRB, 0.14211E+01_JPRB, 0.13012E+01_JPRB, 0.12392E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.11775E+01_JPRB, 0.11519E+01_JPRB, 0.11497E+01_JPRB, 0.11408E+01_JPRB, 0.11442E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.11561E+01_JPRB, 0.11711E+01_JPRB, 0.12367E+01_JPRB, 0.13301E+01_JPRB, 0.12264E+01_JPRB, 0.15093E+01_JPRB ,&
&  0.13997E+01_JPRB, 0.15740E+01_JPRB, 0.17557E+01_JPRB, 0.22061E+01_JPRB, 0.23360E+01_JPRB, 0.22887E+01_JPRB, 0.24303E+01_JPRB ,&
&  0.19643E+01_JPRB, 0.17566E+01_JPRB, 0.18595E+01_JPRB, 0.19058E+01_JPRB, 0.17288E+01_JPRB, 0.18175E+01_JPRB, 0.26992E+01_JPRB ,&
&  0.22413E+01_JPRB, 0.24763E+01_JPRB, 0.26135E+01_JPRB, 0.25827E+01_JPRB, 0.13371E+01_JPRB, 0.10765E+01_JPRB, 0.16090E+01_JPRB ,&
&  0.17868E+01_JPRB, 0.12545E+01_JPRB, 0.94184E+00_JPRB, 0.11303E+01_JPRB, 0.91423E+00_JPRB, 0.11738E+01_JPRB, 0.17271E+01_JPRB ,&
&  0.20812E+01_JPRB, 0.25891E+01_JPRB, 0.44041E+01_JPRB, 0.50813E+01_JPRB, 0.49788E+01_JPRB, 0.50500E+01_JPRB, 0.44507E+01_JPRB ,&
&  0.40858E+01_JPRB, 0.33967E+01_JPRB, 0.32605E+01_JPRB, 0.28443E+01_JPRB, 0.24757E+01_JPRB, 0.22111E+01_JPRB, 0.19975E+01_JPRB ,&
&  0.18317E+01_JPRB, 0.16897E+01_JPRB, 0.15802E+01_JPRB, 0.15116E+01_JPRB, 0.14564E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.15320E+01_JPRB, 0.14521E+01_JPRB, 0.13772E+01_JPRB, 0.13046E+01_JPRB, 0.12308E+01_JPRB, 0.11730E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10147E+01_JPRB, 0.95808E+00_JPRB, 0.91532E+00_JPRB, 0.91461E+00_JPRB, 0.92586E+00_JPRB, 0.91854E+00_JPRB ,&
&  0.91205E+00_JPRB, 0.94243E+00_JPRB, 0.95558E+00_JPRB, 0.87935E+00_JPRB, 0.82900E+00_JPRB, 0.80566E+00_JPRB, 0.83621E+00_JPRB ,&
&  0.11084E+01_JPRB, 0.14950E+01_JPRB, 0.18093E+01_JPRB, 0.21380E+01_JPRB, 0.24043E+01_JPRB, 0.23767E+01_JPRB, 0.25424E+01_JPRB ,&
&  0.24508E+01_JPRB, 0.22088E+01_JPRB, 0.19410E+01_JPRB, 0.16802E+01_JPRB, 0.15017E+01_JPRB, 0.13753E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.12468E+01_JPRB, 0.12247E+01_JPRB, 0.11980E+01_JPRB, 0.11637E+01_JPRB, 0.11399E+01_JPRB, 0.11408E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.11399E+01_JPRB, 0.11596E+01_JPRB, 0.11955E+01_JPRB, 0.13010E+01_JPRB, 0.13173E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.12494E+01_JPRB, 0.17311E+01_JPRB, 0.20574E+01_JPRB, 0.21267E+01_JPRB, 0.21837E+01_JPRB, 0.21168E+01_JPRB ,&
&  0.20613E+01_JPRB, 0.21967E+01_JPRB, 0.21083E+01_JPRB, 0.22624E+01_JPRB, 0.21772E+01_JPRB, 0.16965E+01_JPRB, 0.21522E+01_JPRB ,&
&  0.23226E+01_JPRB, 0.22976E+01_JPRB, 0.18944E+01_JPRB, 0.13970E+01_JPRB, 0.14992E+01_JPRB, 0.12466E+01_JPRB, 0.91141E+00_JPRB ,&
&  0.86391E+00_JPRB, 0.82515E+00_JPRB, 0.75485E+00_JPRB, 0.75803E+00_JPRB, 0.76178E+00_JPRB, 0.11579E+01_JPRB, 0.18120E+01_JPRB ,&
&  0.28307E+01_JPRB, 0.40586E+01_JPRB, 0.52732E+01_JPRB, 0.57335E+01_JPRB, 0.52087E+01_JPRB, 0.51366E+01_JPRB, 0.46560E+01_JPRB ,&
&  0.37184E+01_JPRB, 0.32815E+01_JPRB, 0.30826E+01_JPRB, 0.27518E+01_JPRB, 0.24794E+01_JPRB, 0.22889E+01_JPRB, 0.21296E+01_JPRB ,&
&  0.19693E+01_JPRB, 0.18266E+01_JPRB, 0.17261E+01_JPRB, 0.16617E+01_JPRB, 0.16040E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.16131E+01_JPRB, 0.15301E+01_JPRB, 0.14535E+01_JPRB, 0.13858E+01_JPRB, 0.13195E+01_JPRB, 0.12617E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.11334E+01_JPRB, 0.10394E+01_JPRB, 0.97905E+00_JPRB, 0.94157E+00_JPRB, 0.93168E+00_JPRB, 0.93770E+00_JPRB, 0.93540E+00_JPRB ,&
&  0.92877E+00_JPRB, 0.93681E+00_JPRB, 0.99731E+00_JPRB, 0.10081E+01_JPRB, 0.10344E+01_JPRB, 0.10725E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.15545E+01_JPRB, 0.20597E+01_JPRB, 0.23401E+01_JPRB, 0.25456E+01_JPRB, 0.27445E+01_JPRB, 0.27652E+01_JPRB, 0.26753E+01_JPRB ,&
&  0.24659E+01_JPRB, 0.22488E+01_JPRB, 0.19988E+01_JPRB, 0.17252E+01_JPRB, 0.15259E+01_JPRB, 0.14086E+01_JPRB, 0.13380E+01_JPRB ,&
&  0.12872E+01_JPRB, 0.12767E+01_JPRB, 0.12416E+01_JPRB, 0.11867E+01_JPRB, 0.11352E+01_JPRB, 0.11389E+01_JPRB, 0.11618E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.11666E+01_JPRB, 0.12052E+01_JPRB, 0.12154E+01_JPRB, 0.11294E+01_JPRB, 0.10006E+01_JPRB, 0.11052E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.15221E+01_JPRB, 0.16327E+01_JPRB, 0.20835E+01_JPRB, 0.21657E+01_JPRB, 0.20670E+01_JPRB, 0.20968E+01_JPRB ,&
&  0.21096E+01_JPRB, 0.24290E+01_JPRB, 0.20196E+01_JPRB, 0.20303E+01_JPRB, 0.23236E+01_JPRB, 0.18732E+01_JPRB, 0.18967E+01_JPRB ,&
&  0.21959E+01_JPRB, 0.20710E+01_JPRB, 0.17901E+01_JPRB, 0.14150E+01_JPRB, 0.18787E+01_JPRB, 0.20419E+01_JPRB, 0.10999E+01_JPRB ,&
&  0.92762E+00_JPRB, 0.86661E+00_JPRB, 0.82086E+00_JPRB, 0.79052E+00_JPRB, 0.83900E+00_JPRB, 0.14011E+01_JPRB, 0.28062E+01_JPRB ,&
&  0.35160E+01_JPRB, 0.53645E+01_JPRB, 0.62435E+01_JPRB, 0.63822E+01_JPRB, 0.55085E+01_JPRB, 0.46594E+01_JPRB, 0.38006E+01_JPRB ,&
&  0.35259E+01_JPRB, 0.34202E+01_JPRB, 0.31268E+01_JPRB, 0.29031E+01_JPRB, 0.26720E+01_JPRB, 0.24614E+01_JPRB, 0.22700E+01_JPRB ,&
&  0.20779E+01_JPRB, 0.19363E+01_JPRB, 0.18314E+01_JPRB, 0.17808E+01_JPRB, 0.17002E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.16725E+01_JPRB, 0.16048E+01_JPRB, 0.15062E+01_JPRB, 0.14314E+01_JPRB, 0.13606E+01_JPRB, 0.12859E+01_JPRB, 0.12384E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.10755E+01_JPRB, 0.10087E+01_JPRB, 0.98083E+00_JPRB, 0.96525E+00_JPRB, 0.94987E+00_JPRB, 0.94346E+00_JPRB ,&
&  0.93425E+00_JPRB, 0.94175E+00_JPRB, 0.10128E+01_JPRB, 0.12062E+01_JPRB, 0.13057E+01_JPRB, 0.13727E+01_JPRB, 0.15551E+01_JPRB ,&
&  0.21690E+01_JPRB, 0.25081E+01_JPRB, 0.25824E+01_JPRB, 0.26679E+01_JPRB, 0.25997E+01_JPRB, 0.25267E+01_JPRB, 0.23670E+01_JPRB ,&
&  0.21777E+01_JPRB, 0.20008E+01_JPRB, 0.18235E+01_JPRB, 0.16322E+01_JPRB, 0.14847E+01_JPRB, 0.14065E+01_JPRB, 0.13583E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.13148E+01_JPRB, 0.12602E+01_JPRB, 0.11718E+01_JPRB, 0.11193E+01_JPRB, 0.11363E+01_JPRB, 0.11579E+01_JPRB ,&
&  0.11648E+01_JPRB, 0.12212E+01_JPRB, 0.12561E+01_JPRB, 0.12358E+01_JPRB, 0.10511E+01_JPRB, 0.11687E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.14020E+01_JPRB, 0.14217E+01_JPRB, 0.17597E+01_JPRB, 0.20120E+01_JPRB, 0.19720E+01_JPRB, 0.19279E+01_JPRB ,&
&  0.21319E+01_JPRB, 0.20377E+01_JPRB, 0.18143E+01_JPRB, 0.18813E+01_JPRB, 0.20353E+01_JPRB, 0.23269E+01_JPRB, 0.16238E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.18032E+01_JPRB, 0.16425E+01_JPRB, 0.13760E+01_JPRB, 0.17889E+01_JPRB, 0.25946E+01_JPRB, 0.21571E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.88425E+00_JPRB, 0.95867E+00_JPRB, 0.10691E+01_JPRB, 0.10111E+01_JPRB, 0.17230E+01_JPRB, 0.55071E+01_JPRB ,&
&  0.52126E+01_JPRB, 0.66569E+01_JPRB, 0.65857E+01_JPRB, 0.60738E+01_JPRB, 0.52809E+01_JPRB, 0.43449E+01_JPRB, 0.38515E+01_JPRB ,&
&  0.35443E+01_JPRB, 0.33934E+01_JPRB, 0.32513E+01_JPRB, 0.30897E+01_JPRB, 0.28759E+01_JPRB, 0.25852E+01_JPRB, 0.23269E+01_JPRB ,&
&  0.21278E+01_JPRB, 0.19871E+01_JPRB, 0.18734E+01_JPRB, 0.17955E+01_JPRB, 0.17239E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.17446E+01_JPRB, 0.16315E+01_JPRB, 0.15153E+01_JPRB, 0.14396E+01_JPRB, 0.13569E+01_JPRB, 0.12797E+01_JPRB, 0.12651E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.11259E+01_JPRB, 0.10483E+01_JPRB, 0.10135E+01_JPRB, 0.98904E+00_JPRB, 0.96633E+00_JPRB, 0.96030E+00_JPRB ,&
&  0.95717E+00_JPRB, 0.97506E+00_JPRB, 0.10263E+01_JPRB, 0.11937E+01_JPRB, 0.13387E+01_JPRB, 0.13024E+01_JPRB, 0.15747E+01_JPRB ,&
&  0.24027E+01_JPRB, 0.26609E+01_JPRB, 0.25662E+01_JPRB, 0.25301E+01_JPRB, 0.24336E+01_JPRB, 0.24004E+01_JPRB, 0.21715E+01_JPRB ,&
&  0.18912E+01_JPRB, 0.17027E+01_JPRB, 0.15626E+01_JPRB, 0.15001E+01_JPRB, 0.14514E+01_JPRB, 0.14053E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.12719E+01_JPRB, 0.11996E+01_JPRB, 0.11152E+01_JPRB, 0.11082E+01_JPRB, 0.11450E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.12854E+01_JPRB, 0.12952E+01_JPRB, 0.12148E+01_JPRB, 0.11925E+01_JPRB, 0.12032E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12748E+01_JPRB, 0.13602E+01_JPRB, 0.14845E+01_JPRB, 0.16348E+01_JPRB, 0.18158E+01_JPRB, 0.18678E+01_JPRB, 0.18274E+01_JPRB ,&
&  0.19085E+01_JPRB, 0.18163E+01_JPRB, 0.16133E+01_JPRB, 0.17056E+01_JPRB, 0.18592E+01_JPRB, 0.21532E+01_JPRB, 0.20711E+01_JPRB ,&
&  0.17738E+01_JPRB, 0.14983E+01_JPRB, 0.14303E+01_JPRB, 0.15126E+01_JPRB, 0.17811E+01_JPRB, 0.24495E+01_JPRB, 0.36917E+01_JPRB ,&
&  0.37631E+01_JPRB, 0.25325E+01_JPRB, 0.21200E+01_JPRB, 0.28435E+01_JPRB, 0.25366E+01_JPRB, 0.26128E+01_JPRB, 0.41221E+01_JPRB ,&
&  0.55325E+01_JPRB, 0.66456E+01_JPRB, 0.64108E+01_JPRB, 0.56950E+01_JPRB, 0.50712E+01_JPRB, 0.42294E+01_JPRB, 0.38838E+01_JPRB ,&
&  0.36516E+01_JPRB, 0.34389E+01_JPRB, 0.33740E+01_JPRB, 0.32209E+01_JPRB, 0.29396E+01_JPRB, 0.25899E+01_JPRB, 0.22795E+01_JPRB ,&
&  0.20911E+01_JPRB, 0.19794E+01_JPRB, 0.19022E+01_JPRB, 0.18527E+01_JPRB, 0.18069E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.17310E+01_JPRB, 0.15859E+01_JPRB, 0.14815E+01_JPRB, 0.13979E+01_JPRB, 0.13104E+01_JPRB, 0.12607E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.11589E+01_JPRB, 0.10870E+01_JPRB, 0.10289E+01_JPRB, 0.98952E+00_JPRB, 0.97842E+00_JPRB, 0.98392E+00_JPRB ,&
&  0.98041E+00_JPRB, 0.99898E+00_JPRB, 0.10356E+01_JPRB, 0.12049E+01_JPRB, 0.14967E+01_JPRB, 0.19696E+01_JPRB, 0.15615E+01_JPRB ,&
&  0.23066E+01_JPRB, 0.29010E+01_JPRB, 0.26924E+01_JPRB, 0.24477E+01_JPRB, 0.22316E+01_JPRB, 0.22139E+01_JPRB, 0.19681E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.14947E+01_JPRB, 0.14041E+01_JPRB, 0.13688E+01_JPRB, 0.13636E+01_JPRB, 0.13339E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.12309E+01_JPRB, 0.11841E+01_JPRB, 0.11298E+01_JPRB, 0.11017E+01_JPRB, 0.11152E+01_JPRB, 0.11586E+01_JPRB, 0.12217E+01_JPRB ,&
&  0.13001E+01_JPRB, 0.13394E+01_JPRB, 0.13295E+01_JPRB, 0.12397E+01_JPRB, 0.12419E+01_JPRB, 0.12467E+01_JPRB, 0.12647E+01_JPRB ,&
&  0.12227E+01_JPRB, 0.11909E+01_JPRB, 0.13836E+01_JPRB, 0.15958E+01_JPRB, 0.17065E+01_JPRB, 0.16537E+01_JPRB, 0.15843E+01_JPRB ,&
&  0.17358E+01_JPRB, 0.18228E+01_JPRB, 0.18596E+01_JPRB, 0.17555E+01_JPRB, 0.19128E+01_JPRB, 0.21014E+01_JPRB, 0.22378E+01_JPRB ,&
&  0.21773E+01_JPRB, 0.18338E+01_JPRB, 0.17040E+01_JPRB, 0.17400E+01_JPRB, 0.18127E+01_JPRB, 0.23916E+01_JPRB, 0.33969E+01_JPRB ,&
&  0.44872E+01_JPRB, 0.60895E+01_JPRB, 0.61120E+01_JPRB, 0.48133E+01_JPRB, 0.42572E+01_JPRB, 0.41597E+01_JPRB, 0.42728E+01_JPRB ,&
&  0.59678E+01_JPRB, 0.63990E+01_JPRB, 0.59882E+01_JPRB, 0.63748E+01_JPRB, 0.50223E+01_JPRB, 0.41878E+01_JPRB, 0.37760E+01_JPRB ,&
&  0.36446E+01_JPRB, 0.34579E+01_JPRB, 0.32681E+01_JPRB, 0.30457E+01_JPRB, 0.27091E+01_JPRB, 0.23915E+01_JPRB, 0.21733E+01_JPRB ,&
&  0.20401E+01_JPRB, 0.19629E+01_JPRB, 0.19057E+01_JPRB, 0.18703E+01_JPRB, 0.18300E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.16245E+01_JPRB, 0.14923E+01_JPRB, 0.13621E+01_JPRB, 0.12968E+01_JPRB, 0.12442E+01_JPRB, 0.12367E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.13456E+01_JPRB, 0.11807E+01_JPRB, 0.10806E+01_JPRB, 0.10077E+01_JPRB, 0.98201E+00_JPRB, 0.98870E+00_JPRB, 0.99763E+00_JPRB ,&
&  0.10070E+01_JPRB, 0.10380E+01_JPRB, 0.11257E+01_JPRB, 0.12473E+01_JPRB, 0.15517E+01_JPRB, 0.19309E+01_JPRB, 0.20359E+01_JPRB ,&
&  0.25078E+01_JPRB, 0.36922E+01_JPRB, 0.28340E+01_JPRB, 0.23610E+01_JPRB, 0.22080E+01_JPRB, 0.24291E+01_JPRB, 0.20341E+01_JPRB ,&
&  0.16015E+01_JPRB, 0.14793E+01_JPRB, 0.14442E+01_JPRB, 0.13403E+01_JPRB, 0.13031E+01_JPRB, 0.12563E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.11544E+01_JPRB, 0.11161E+01_JPRB, 0.11164E+01_JPRB, 0.11498E+01_JPRB, 0.12072E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.13718E+01_JPRB, 0.13033E+01_JPRB, 0.12778E+01_JPRB, 0.12738E+01_JPRB, 0.12917E+01_JPRB, 0.13018E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.12339E+01_JPRB, 0.13483E+01_JPRB, 0.14029E+01_JPRB, 0.14213E+01_JPRB, 0.15037E+01_JPRB, 0.14534E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.16621E+01_JPRB, 0.19612E+01_JPRB, 0.17716E+01_JPRB, 0.19601E+01_JPRB, 0.21240E+01_JPRB, 0.20390E+01_JPRB ,&
&  0.17941E+01_JPRB, 0.17355E+01_JPRB, 0.16476E+01_JPRB, 0.17627E+01_JPRB, 0.21345E+01_JPRB, 0.28926E+01_JPRB, 0.36913E+01_JPRB ,&
&  0.45600E+01_JPRB, 0.54407E+01_JPRB, 0.61065E+01_JPRB, 0.48914E+01_JPRB, 0.44640E+01_JPRB, 0.44303E+01_JPRB, 0.48762E+01_JPRB ,&
&  0.61481E+01_JPRB, 0.63503E+01_JPRB, 0.63620E+01_JPRB, 0.57433E+01_JPRB, 0.45096E+01_JPRB, 0.38143E+01_JPRB, 0.34812E+01_JPRB ,&
&  0.32874E+01_JPRB, 0.31111E+01_JPRB, 0.27892E+01_JPRB, 0.25300E+01_JPRB, 0.23438E+01_JPRB, 0.21859E+01_JPRB, 0.20231E+01_JPRB ,&
&  0.19241E+01_JPRB, 0.18460E+01_JPRB, 0.18098E+01_JPRB, 0.17914E+01_JPRB, 0.17329E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.14434E+01_JPRB, 0.13483E+01_JPRB, 0.12861E+01_JPRB, 0.12652E+01_JPRB, 0.12339E+01_JPRB, 0.12574E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.11394E+01_JPRB, 0.10371E+01_JPRB, 0.10016E+01_JPRB, 0.99447E+00_JPRB, 0.99322E+00_JPRB, 0.10074E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.11073E+01_JPRB, 0.12351E+01_JPRB, 0.13929E+01_JPRB, 0.17266E+01_JPRB, 0.22155E+01_JPRB, 0.26967E+01_JPRB ,&
&  0.31674E+01_JPRB, 0.31173E+01_JPRB, 0.27388E+01_JPRB, 0.22704E+01_JPRB, 0.20967E+01_JPRB, 0.22524E+01_JPRB, 0.21596E+01_JPRB ,&
&  0.18398E+01_JPRB, 0.15716E+01_JPRB, 0.14577E+01_JPRB, 0.13349E+01_JPRB, 0.12527E+01_JPRB, 0.12259E+01_JPRB, 0.11933E+01_JPRB ,&
&  0.11500E+01_JPRB, 0.11149E+01_JPRB, 0.11022E+01_JPRB, 0.11348E+01_JPRB, 0.11912E+01_JPRB, 0.12781E+01_JPRB, 0.14076E+01_JPRB ,&
&  0.14486E+01_JPRB, 0.13914E+01_JPRB, 0.13429E+01_JPRB, 0.12846E+01_JPRB, 0.12955E+01_JPRB, 0.13097E+01_JPRB, 0.13103E+01_JPRB ,&
&  0.12982E+01_JPRB, 0.13284E+01_JPRB, 0.13587E+01_JPRB, 0.13522E+01_JPRB, 0.13028E+01_JPRB, 0.14377E+01_JPRB, 0.14592E+01_JPRB ,&
&  0.15983E+01_JPRB, 0.17119E+01_JPRB, 0.20952E+01_JPRB, 0.17528E+01_JPRB, 0.20035E+01_JPRB, 0.20626E+01_JPRB, 0.19251E+01_JPRB ,&
&  0.17327E+01_JPRB, 0.16905E+01_JPRB, 0.17602E+01_JPRB, 0.19758E+01_JPRB, 0.26333E+01_JPRB, 0.34472E+01_JPRB, 0.49457E+01_JPRB ,&
&  0.50078E+01_JPRB, 0.56446E+01_JPRB, 0.53020E+01_JPRB, 0.52143E+01_JPRB, 0.47110E+01_JPRB, 0.46765E+01_JPRB, 0.49339E+01_JPRB ,&
&  0.55403E+01_JPRB, 0.55138E+01_JPRB, 0.51105E+01_JPRB, 0.45332E+01_JPRB, 0.37413E+01_JPRB, 0.31974E+01_JPRB, 0.29122E+01_JPRB ,&
&  0.26431E+01_JPRB, 0.24017E+01_JPRB, 0.22108E+01_JPRB, 0.21312E+01_JPRB, 0.20374E+01_JPRB, 0.19277E+01_JPRB, 0.18347E+01_JPRB ,&
&  0.17793E+01_JPRB, 0.17294E+01_JPRB, 0.16862E+01_JPRB, 0.16316E+01_JPRB, 0.15560E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.12962E+01_JPRB, 0.12455E+01_JPRB, 0.12184E+01_JPRB, 0.11803E+01_JPRB, 0.11439E+01_JPRB, 0.11393E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.10336E+01_JPRB, 0.10186E+01_JPRB, 0.10188E+01_JPRB, 0.10085E+01_JPRB, 0.99783E+00_JPRB, 0.10134E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.11775E+01_JPRB, 0.13215E+01_JPRB, 0.15639E+01_JPRB, 0.18865E+01_JPRB, 0.23567E+01_JPRB, 0.28674E+01_JPRB ,&
&  0.29330E+01_JPRB, 0.20915E+01_JPRB, 0.23253E+01_JPRB, 0.22290E+01_JPRB, 0.19962E+01_JPRB, 0.20592E+01_JPRB, 0.20145E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.15221E+01_JPRB, 0.14966E+01_JPRB, 0.14204E+01_JPRB, 0.12235E+01_JPRB, 0.11946E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.11072E+01_JPRB, 0.11216E+01_JPRB, 0.11944E+01_JPRB, 0.12910E+01_JPRB, 0.14200E+01_JPRB, 0.15022E+01_JPRB ,&
&  0.14812E+01_JPRB, 0.14446E+01_JPRB, 0.14280E+01_JPRB, 0.13685E+01_JPRB, 0.13678E+01_JPRB, 0.13798E+01_JPRB, 0.13747E+01_JPRB ,&
&  0.13624E+01_JPRB, 0.13464E+01_JPRB, 0.13829E+01_JPRB, 0.14462E+01_JPRB, 0.14940E+01_JPRB, 0.14116E+01_JPRB, 0.15281E+01_JPRB ,&
&  0.16407E+01_JPRB, 0.18741E+01_JPRB, 0.20661E+01_JPRB, 0.23440E+01_JPRB, 0.17543E+01_JPRB, 0.18854E+01_JPRB, 0.16731E+01_JPRB ,&
&  0.17831E+01_JPRB, 0.19185E+01_JPRB, 0.22322E+01_JPRB, 0.26673E+01_JPRB, 0.35879E+01_JPRB, 0.44397E+01_JPRB, 0.58165E+01_JPRB ,&
&  0.51753E+01_JPRB, 0.49517E+01_JPRB, 0.47781E+01_JPRB, 0.51911E+01_JPRB, 0.49711E+01_JPRB, 0.51984E+01_JPRB, 0.48873E+01_JPRB ,&
&  0.49351E+01_JPRB, 0.44410E+01_JPRB, 0.39518E+01_JPRB, 0.36041E+01_JPRB, 0.29911E+01_JPRB, 0.24137E+01_JPRB, 0.20901E+01_JPRB ,&
&  0.19068E+01_JPRB, 0.18515E+01_JPRB, 0.18157E+01_JPRB, 0.17868E+01_JPRB, 0.17343E+01_JPRB, 0.16841E+01_JPRB, 0.16431E+01_JPRB ,&
&  0.16034E+01_JPRB, 0.15509E+01_JPRB, 0.14991E+01_JPRB, 0.14445E+01_JPRB, 0.13773E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.12000E+01_JPRB, 0.11616E+01_JPRB, 0.11390E+01_JPRB, 0.11071E+01_JPRB, 0.10849E+01_JPRB, 0.10782E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.10387E+01_JPRB, 0.10370E+01_JPRB, 0.10247E+01_JPRB, 0.10013E+01_JPRB, 0.10021E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.13118E+01_JPRB, 0.14984E+01_JPRB, 0.17008E+01_JPRB, 0.20131E+01_JPRB, 0.22691E+01_JPRB, 0.23862E+01_JPRB ,&
&  0.25598E+01_JPRB, 0.26108E+01_JPRB, 0.22235E+01_JPRB, 0.16007E+01_JPRB, 0.14730E+01_JPRB, 0.17611E+01_JPRB, 0.15771E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.13522E+01_JPRB, 0.13168E+01_JPRB, 0.12750E+01_JPRB, 0.12052E+01_JPRB, 0.11696E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.11340E+01_JPRB, 0.11506E+01_JPRB, 0.12151E+01_JPRB, 0.13453E+01_JPRB, 0.14745E+01_JPRB, 0.15566E+01_JPRB, 0.15678E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.15759E+01_JPRB, 0.15678E+01_JPRB, 0.15227E+01_JPRB, 0.14814E+01_JPRB, 0.14924E+01_JPRB, 0.14948E+01_JPRB ,&
&  0.14909E+01_JPRB, 0.14802E+01_JPRB, 0.14747E+01_JPRB, 0.15195E+01_JPRB, 0.15431E+01_JPRB, 0.14404E+01_JPRB, 0.15069E+01_JPRB ,&
&  0.17484E+01_JPRB, 0.21122E+01_JPRB, 0.21369E+01_JPRB, 0.25242E+01_JPRB, 0.16008E+01_JPRB, 0.17708E+01_JPRB, 0.19621E+01_JPRB ,&
&  0.20458E+01_JPRB, 0.24609E+01_JPRB, 0.28835E+01_JPRB, 0.34504E+01_JPRB, 0.43167E+01_JPRB, 0.48548E+01_JPRB, 0.54147E+01_JPRB ,&
&  0.46231E+01_JPRB, 0.42028E+01_JPRB, 0.42485E+01_JPRB, 0.45974E+01_JPRB, 0.47733E+01_JPRB, 0.47826E+01_JPRB, 0.41026E+01_JPRB ,&
&  0.34668E+01_JPRB, 0.35444E+01_JPRB, 0.31007E+01_JPRB, 0.29315E+01_JPRB, 0.22241E+01_JPRB, 0.17567E+01_JPRB, 0.16765E+01_JPRB ,&
&  0.16020E+01_JPRB, 0.15490E+01_JPRB, 0.15390E+01_JPRB, 0.15211E+01_JPRB, 0.14992E+01_JPRB, 0.14800E+01_JPRB, 0.14456E+01_JPRB ,&
&  0.14119E+01_JPRB, 0.13806E+01_JPRB, 0.13442E+01_JPRB, 0.13044E+01_JPRB, 0.12559E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.11292E+01_JPRB, 0.11031E+01_JPRB, 0.10932E+01_JPRB, 0.10549E+01_JPRB, 0.10240E+01_JPRB, 0.10203E+01_JPRB, 0.10316E+01_JPRB ,&
&  0.10263E+01_JPRB, 0.10187E+01_JPRB, 0.10049E+01_JPRB, 0.99518E+00_JPRB, 0.10103E+01_JPRB, 0.10629E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.12709E+01_JPRB, 0.14320E+01_JPRB, 0.16273E+01_JPRB, 0.18200E+01_JPRB, 0.20728E+01_JPRB, 0.21790E+01_JPRB, 0.21650E+01_JPRB ,&
&  0.22337E+01_JPRB, 0.23249E+01_JPRB, 0.22782E+01_JPRB, 0.22275E+01_JPRB, 0.18416E+01_JPRB, 0.19755E+01_JPRB, 0.20126E+01_JPRB ,&
&  0.19100E+01_JPRB, 0.15460E+01_JPRB, 0.13095E+01_JPRB, 0.12541E+01_JPRB, 0.12010E+01_JPRB, 0.11810E+01_JPRB, 0.11933E+01_JPRB ,&
&  0.12315E+01_JPRB, 0.13006E+01_JPRB, 0.14094E+01_JPRB, 0.15422E+01_JPRB, 0.16317E+01_JPRB, 0.16660E+01_JPRB, 0.16819E+01_JPRB ,&
&  0.17048E+01_JPRB, 0.17223E+01_JPRB, 0.17402E+01_JPRB, 0.17021E+01_JPRB, 0.16940E+01_JPRB, 0.16965E+01_JPRB, 0.17247E+01_JPRB ,&
&  0.17528E+01_JPRB, 0.16743E+01_JPRB, 0.16811E+01_JPRB, 0.17507E+01_JPRB, 0.17316E+01_JPRB, 0.16912E+01_JPRB, 0.17577E+01_JPRB ,&
&  0.20106E+01_JPRB, 0.24424E+01_JPRB, 0.20211E+01_JPRB, 0.21864E+01_JPRB, 0.25699E+01_JPRB, 0.22160E+01_JPRB, 0.21278E+01_JPRB ,&
&  0.24346E+01_JPRB, 0.27997E+01_JPRB, 0.31920E+01_JPRB, 0.37119E+01_JPRB, 0.44895E+01_JPRB, 0.54689E+01_JPRB, 0.45890E+01_JPRB ,&
&  0.39435E+01_JPRB, 0.35907E+01_JPRB, 0.37116E+01_JPRB, 0.38945E+01_JPRB, 0.39521E+01_JPRB, 0.38713E+01_JPRB, 0.33290E+01_JPRB ,&
&  0.28889E+01_JPRB, 0.29292E+01_JPRB, 0.26629E+01_JPRB, 0.23829E+01_JPRB, 0.21790E+01_JPRB, 0.15790E+01_JPRB, 0.14863E+01_JPRB ,&
&  0.13844E+01_JPRB, 0.13718E+01_JPRB, 0.13758E+01_JPRB, 0.13696E+01_JPRB, 0.13549E+01_JPRB, 0.13273E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.12754E+01_JPRB, 0.12504E+01_JPRB, 0.12294E+01_JPRB, 0.12079E+01_JPRB, 0.11675E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.10792E+01_JPRB, 0.10641E+01_JPRB, 0.10500E+01_JPRB, 0.10192E+01_JPRB, 0.98761E+00_JPRB, 0.98303E+00_JPRB, 0.98010E+00_JPRB ,&
&  0.95998E+00_JPRB, 0.93842E+00_JPRB, 0.91368E+00_JPRB, 0.93180E+00_JPRB, 0.97285E+00_JPRB, 0.10507E+01_JPRB, 0.11400E+01_JPRB ,&
&  0.12442E+01_JPRB, 0.13900E+01_JPRB, 0.15917E+01_JPRB, 0.17585E+01_JPRB, 0.19926E+01_JPRB, 0.21206E+01_JPRB, 0.21959E+01_JPRB ,&
&  0.22907E+01_JPRB, 0.23982E+01_JPRB, 0.24754E+01_JPRB, 0.25237E+01_JPRB, 0.25208E+01_JPRB, 0.25711E+01_JPRB, 0.27176E+01_JPRB ,&
&  0.22678E+01_JPRB, 0.16382E+01_JPRB, 0.14284E+01_JPRB, 0.13028E+01_JPRB, 0.12674E+01_JPRB, 0.12714E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.14771E+01_JPRB, 0.15868E+01_JPRB, 0.16745E+01_JPRB, 0.17513E+01_JPRB, 0.18033E+01_JPRB, 0.18334E+01_JPRB ,&
&  0.18884E+01_JPRB, 0.19440E+01_JPRB, 0.20331E+01_JPRB, 0.19377E+01_JPRB, 0.20676E+01_JPRB, 0.20792E+01_JPRB, 0.20938E+01_JPRB ,&
&  0.20484E+01_JPRB, 0.19242E+01_JPRB, 0.19851E+01_JPRB, 0.20452E+01_JPRB, 0.21096E+01_JPRB, 0.21242E+01_JPRB, 0.21416E+01_JPRB ,&
&  0.23964E+01_JPRB, 0.24774E+01_JPRB, 0.19529E+01_JPRB, 0.18374E+01_JPRB, 0.19609E+01_JPRB, 0.18086E+01_JPRB, 0.22839E+01_JPRB ,&
&  0.24964E+01_JPRB, 0.27783E+01_JPRB, 0.30734E+01_JPRB, 0.35294E+01_JPRB, 0.40584E+01_JPRB, 0.44693E+01_JPRB, 0.39307E+01_JPRB ,&
&  0.31373E+01_JPRB, 0.28997E+01_JPRB, 0.29403E+01_JPRB, 0.29572E+01_JPRB, 0.29770E+01_JPRB, 0.29725E+01_JPRB, 0.28151E+01_JPRB ,&
&  0.25884E+01_JPRB, 0.23430E+01_JPRB, 0.21241E+01_JPRB, 0.17078E+01_JPRB, 0.14806E+01_JPRB, 0.13651E+01_JPRB, 0.12612E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.12676E+01_JPRB, 0.12772E+01_JPRB, 0.12718E+01_JPRB, 0.12599E+01_JPRB, 0.12280E+01_JPRB, 0.12057E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.11737E+01_JPRB, 0.11547E+01_JPRB, 0.11393E+01_JPRB, 0.11066E+01_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.10048E+01_JPRB, 0.97888E+00_JPRB, 0.96051E+00_JPRB, 0.94237E+00_JPRB, 0.91208E+00_JPRB, 0.89126E+00_JPRB, 0.88374E+00_JPRB ,&
&  0.84556E+00_JPRB, 0.84528E+00_JPRB, 0.82974E+00_JPRB, 0.84443E+00_JPRB, 0.88722E+00_JPRB, 0.92924E+00_JPRB, 0.98908E+00_JPRB ,&
&  0.10384E+01_JPRB, 0.11082E+01_JPRB, 0.12246E+01_JPRB, 0.14730E+01_JPRB, 0.17571E+01_JPRB, 0.19783E+01_JPRB, 0.22393E+01_JPRB ,&
&  0.23634E+01_JPRB, 0.24769E+01_JPRB, 0.26785E+01_JPRB, 0.29596E+01_JPRB, 0.31684E+01_JPRB, 0.33515E+01_JPRB, 0.33217E+01_JPRB ,&
&  0.24722E+01_JPRB, 0.15220E+01_JPRB, 0.14456E+01_JPRB, 0.13166E+01_JPRB, 0.13297E+01_JPRB, 0.13692E+01_JPRB, 0.14136E+01_JPRB ,&
&  0.14930E+01_JPRB, 0.15695E+01_JPRB, 0.16509E+01_JPRB, 0.17203E+01_JPRB, 0.18126E+01_JPRB, 0.19147E+01_JPRB, 0.19601E+01_JPRB ,&
&  0.19966E+01_JPRB, 0.20395E+01_JPRB, 0.20809E+01_JPRB, 0.20335E+01_JPRB, 0.21136E+01_JPRB, 0.22478E+01_JPRB, 0.23493E+01_JPRB ,&
&  0.24468E+01_JPRB, 0.24462E+01_JPRB, 0.21108E+01_JPRB, 0.21553E+01_JPRB, 0.24129E+01_JPRB, 0.22968E+01_JPRB, 0.23917E+01_JPRB ,&
&  0.27601E+01_JPRB, 0.28491E+01_JPRB, 0.18611E+01_JPRB, 0.17265E+01_JPRB, 0.18567E+01_JPRB, 0.19974E+01_JPRB, 0.22020E+01_JPRB ,&
&  0.23469E+01_JPRB, 0.25913E+01_JPRB, 0.28586E+01_JPRB, 0.30904E+01_JPRB, 0.32273E+01_JPRB, 0.30993E+01_JPRB, 0.29288E+01_JPRB ,&
&  0.24552E+01_JPRB, 0.22872E+01_JPRB, 0.23152E+01_JPRB, 0.23657E+01_JPRB, 0.24545E+01_JPRB, 0.25621E+01_JPRB, 0.23240E+01_JPRB ,&
&  0.22062E+01_JPRB, 0.21275E+01_JPRB, 0.22728E+01_JPRB, 0.18593E+01_JPRB, 0.13445E+01_JPRB, 0.12076E+01_JPRB, 0.11244E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.12064E+01_JPRB, 0.12201E+01_JPRB, 0.12262E+01_JPRB, 0.12187E+01_JPRB, 0.12010E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.11264E+01_JPRB, 0.10956E+01_JPRB, 0.10644E+01_JPRB, 0.10362E+01_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.87596E+00_JPRB, 0.85036E+00_JPRB, 0.84034E+00_JPRB, 0.79626E+00_JPRB, 0.81090E+00_JPRB, 0.78034E+00_JPRB, 0.81805E+00_JPRB ,&
&  0.79333E+00_JPRB, 0.79289E+00_JPRB, 0.79666E+00_JPRB, 0.81606E+00_JPRB, 0.83553E+00_JPRB, 0.85856E+00_JPRB, 0.87773E+00_JPRB ,&
&  0.90163E+00_JPRB, 0.95508E+00_JPRB, 0.10352E+01_JPRB, 0.11329E+01_JPRB, 0.12623E+01_JPRB, 0.13952E+01_JPRB, 0.15172E+01_JPRB ,&
&  0.16788E+01_JPRB, 0.18238E+01_JPRB, 0.19220E+01_JPRB, 0.21476E+01_JPRB, 0.27083E+01_JPRB, 0.33938E+01_JPRB, 0.34951E+01_JPRB ,&
&  0.22236E+01_JPRB, 0.15444E+01_JPRB, 0.13373E+01_JPRB, 0.12272E+01_JPRB, 0.12736E+01_JPRB, 0.13897E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.14556E+01_JPRB, 0.15005E+01_JPRB, 0.15687E+01_JPRB, 0.16420E+01_JPRB, 0.17387E+01_JPRB, 0.18453E+01_JPRB, 0.18882E+01_JPRB ,&
&  0.18778E+01_JPRB, 0.19082E+01_JPRB, 0.19143E+01_JPRB, 0.20287E+01_JPRB, 0.22493E+01_JPRB, 0.25154E+01_JPRB, 0.26844E+01_JPRB ,&
&  0.27784E+01_JPRB, 0.27379E+01_JPRB, 0.23647E+01_JPRB, 0.23720E+01_JPRB, 0.24895E+01_JPRB, 0.24869E+01_JPRB, 0.24478E+01_JPRB ,&
&  0.26941E+01_JPRB, 0.26553E+01_JPRB, 0.20105E+01_JPRB, 0.19502E+01_JPRB, 0.19711E+01_JPRB, 0.20329E+01_JPRB, 0.19893E+01_JPRB ,&
&  0.21711E+01_JPRB, 0.23588E+01_JPRB, 0.24935E+01_JPRB, 0.25236E+01_JPRB, 0.24501E+01_JPRB, 0.23006E+01_JPRB, 0.22909E+01_JPRB ,&
&  0.21231E+01_JPRB, 0.20832E+01_JPRB, 0.21039E+01_JPRB, 0.21502E+01_JPRB, 0.24656E+01_JPRB, 0.22206E+01_JPRB, 0.21769E+01_JPRB ,&
&  0.18781E+01_JPRB, 0.18172E+01_JPRB, 0.15120E+01_JPRB, 0.17666E+01_JPRB, 0.12345E+01_JPRB, 0.10155E+01_JPRB, 0.10228E+01_JPRB ,&
&  0.10473E+01_JPRB, 0.10872E+01_JPRB, 0.10864E+01_JPRB, 0.10988E+01_JPRB, 0.11025E+01_JPRB, 0.10868E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.10357E+01_JPRB, 0.99301E+00_JPRB, 0.95752E+00_JPRB, 0.92022E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.83574E+00_JPRB, 0.83217E+00_JPRB, 0.80633E+00_JPRB, 0.76706E+00_JPRB, 0.74182E+00_JPRB, 0.72360E+00_JPRB, 0.73308E+00_JPRB ,&
&  0.74404E+00_JPRB, 0.76971E+00_JPRB, 0.79293E+00_JPRB, 0.81606E+00_JPRB, 0.82646E+00_JPRB, 0.83346E+00_JPRB, 0.85285E+00_JPRB ,&
&  0.87910E+00_JPRB, 0.90440E+00_JPRB, 0.93951E+00_JPRB, 0.98754E+00_JPRB, 0.10374E+01_JPRB, 0.11011E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.11895E+01_JPRB, 0.12319E+01_JPRB, 0.13118E+01_JPRB, 0.16159E+01_JPRB, 0.19527E+01_JPRB, 0.24179E+01_JPRB ,&
&  0.18238E+01_JPRB, 0.16442E+01_JPRB, 0.13848E+01_JPRB, 0.12616E+01_JPRB, 0.12811E+01_JPRB, 0.12555E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.12802E+01_JPRB, 0.12900E+01_JPRB, 0.13051E+01_JPRB, 0.13284E+01_JPRB, 0.13886E+01_JPRB, 0.13450E+01_JPRB ,&
&  0.13233E+01_JPRB, 0.13710E+01_JPRB, 0.14346E+01_JPRB, 0.16250E+01_JPRB, 0.18980E+01_JPRB, 0.21477E+01_JPRB, 0.21666E+01_JPRB ,&
&  0.21816E+01_JPRB, 0.23092E+01_JPRB, 0.22580E+01_JPRB, 0.22573E+01_JPRB, 0.22316E+01_JPRB, 0.21946E+01_JPRB, 0.22436E+01_JPRB ,&
&  0.21765E+01_JPRB, 0.21521E+01_JPRB, 0.21864E+01_JPRB, 0.20127E+01_JPRB, 0.19103E+01_JPRB, 0.18226E+01_JPRB, 0.18198E+01_JPRB ,&
&  0.18889E+01_JPRB, 0.19967E+01_JPRB, 0.20576E+01_JPRB, 0.19909E+01_JPRB, 0.19174E+01_JPRB, 0.18219E+01_JPRB, 0.18626E+01_JPRB ,&
&  0.18743E+01_JPRB, 0.17764E+01_JPRB, 0.17757E+01_JPRB, 0.19167E+01_JPRB, 0.20866E+01_JPRB, 0.33873E+01_JPRB, 0.29810E+01_JPRB ,&
&  0.17181E+01_JPRB, 0.13863E+01_JPRB, 0.94978E+00_JPRB, 0.13723E+01_JPRB, 0.95616E+00_JPRB, 0.88414E+00_JPRB, 0.90764E+00_JPRB ,&
&  0.92585E+00_JPRB, 0.94284E+00_JPRB, 0.95326E+00_JPRB, 0.95341E+00_JPRB, 0.95232E+00_JPRB, 0.95299E+00_JPRB, 0.93752E+00_JPRB ,&
&  0.92476E+00_JPRB, 0.90569E+00_JPRB, 0.90013E+00_JPRB, 0.86333E+00_JPRB, 0.84544E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.79299E+00_JPRB, 0.78974E+00_JPRB, 0.75549E+00_JPRB, 0.70875E+00_JPRB, 0.68767E+00_JPRB, 0.69219E+00_JPRB, 0.70962E+00_JPRB ,&
&  0.72745E+00_JPRB, 0.75091E+00_JPRB, 0.77882E+00_JPRB, 0.80648E+00_JPRB, 0.83179E+00_JPRB, 0.85921E+00_JPRB, 0.89191E+00_JPRB ,&
&  0.92273E+00_JPRB, 0.96038E+00_JPRB, 0.99964E+00_JPRB, 0.10414E+01_JPRB, 0.10804E+01_JPRB, 0.11464E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.13434E+01_JPRB, 0.14354E+01_JPRB, 0.15330E+01_JPRB, 0.16615E+01_JPRB, 0.17861E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.19195E+01_JPRB, 0.16429E+01_JPRB, 0.14089E+01_JPRB, 0.12768E+01_JPRB, 0.12030E+01_JPRB, 0.11454E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.10091E+01_JPRB, 0.96433E+00_JPRB, 0.92760E+00_JPRB, 0.90550E+00_JPRB, 0.95809E+00_JPRB, 0.95156E+00_JPRB ,&
&  0.10192E+01_JPRB, 0.11462E+01_JPRB, 0.12514E+01_JPRB, 0.14225E+01_JPRB, 0.15673E+01_JPRB, 0.16628E+01_JPRB, 0.16058E+01_JPRB ,&
&  0.16162E+01_JPRB, 0.17335E+01_JPRB, 0.16986E+01_JPRB, 0.17229E+01_JPRB, 0.17130E+01_JPRB, 0.16639E+01_JPRB, 0.16502E+01_JPRB ,&
&  0.15085E+01_JPRB, 0.16177E+01_JPRB, 0.18584E+01_JPRB, 0.20876E+01_JPRB, 0.18527E+01_JPRB, 0.17023E+01_JPRB, 0.17104E+01_JPRB ,&
&  0.16665E+01_JPRB, 0.16954E+01_JPRB, 0.17126E+01_JPRB, 0.16300E+01_JPRB, 0.15602E+01_JPRB, 0.15007E+01_JPRB, 0.15461E+01_JPRB ,&
&  0.15867E+01_JPRB, 0.14802E+01_JPRB, 0.15422E+01_JPRB, 0.17786E+01_JPRB, 0.21327E+01_JPRB, 0.23146E+01_JPRB, 0.27946E+01_JPRB ,&
&  0.16202E+01_JPRB, 0.94220E+00_JPRB, 0.95018E+00_JPRB, 0.10022E+01_JPRB, 0.82343E+00_JPRB, 0.74492E+00_JPRB, 0.79382E+00_JPRB ,&
&  0.83276E+00_JPRB, 0.85685E+00_JPRB, 0.84166E+00_JPRB, 0.86452E+00_JPRB, 0.91280E+00_JPRB, 0.90705E+00_JPRB, 0.85617E+00_JPRB ,&
&  0.86560E+00_JPRB, 0.82868E+00_JPRB, 0.81149E+00_JPRB, 0.80686E+00_JPRB, 0.80229E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.74264E+00_JPRB, 0.72865E+00_JPRB, 0.69160E+00_JPRB, 0.64842E+00_JPRB, 0.63118E+00_JPRB, 0.64689E+00_JPRB, 0.67354E+00_JPRB ,&
&  0.69756E+00_JPRB, 0.72625E+00_JPRB, 0.75603E+00_JPRB, 0.78939E+00_JPRB, 0.82456E+00_JPRB, 0.85862E+00_JPRB, 0.88562E+00_JPRB ,&
&  0.90711E+00_JPRB, 0.93546E+00_JPRB, 0.96231E+00_JPRB, 0.98861E+00_JPRB, 0.10087E+01_JPRB, 0.10211E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.11630E+01_JPRB, 0.12464E+01_JPRB, 0.13388E+01_JPRB, 0.14227E+01_JPRB, 0.15359E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.16763E+01_JPRB, 0.15701E+01_JPRB, 0.14135E+01_JPRB, 0.12802E+01_JPRB, 0.11813E+01_JPRB, 0.11085E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.10489E+01_JPRB, 0.99611E+00_JPRB, 0.10050E+01_JPRB, 0.86266E+00_JPRB, 0.90020E+00_JPRB, 0.95267E+00_JPRB ,&
&  0.10612E+01_JPRB, 0.11860E+01_JPRB, 0.12932E+01_JPRB, 0.13852E+01_JPRB, 0.14274E+01_JPRB, 0.14573E+01_JPRB, 0.13945E+01_JPRB ,&
&  0.13911E+01_JPRB, 0.13817E+01_JPRB, 0.12825E+01_JPRB, 0.12842E+01_JPRB, 0.13149E+01_JPRB, 0.11735E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.12471E+01_JPRB, 0.17702E+01_JPRB, 0.20813E+01_JPRB, 0.18049E+01_JPRB, 0.17176E+01_JPRB, 0.16242E+01_JPRB ,&
&  0.15095E+01_JPRB, 0.14926E+01_JPRB, 0.14729E+01_JPRB, 0.14105E+01_JPRB, 0.12965E+01_JPRB, 0.12286E+01_JPRB, 0.12668E+01_JPRB ,&
&  0.13107E+01_JPRB, 0.12299E+01_JPRB, 0.12243E+01_JPRB, 0.14495E+01_JPRB, 0.16213E+01_JPRB, 0.15543E+01_JPRB, 0.26375E+01_JPRB ,&
&  0.15913E+01_JPRB, 0.11340E+01_JPRB, 0.15520E+01_JPRB, 0.64767E+00_JPRB, 0.69620E+00_JPRB, 0.63930E+00_JPRB, 0.65514E+00_JPRB ,&
&  0.55917E+00_JPRB, 0.65639E+00_JPRB, 0.10293E+01_JPRB, 0.13249E+01_JPRB, 0.15133E+01_JPRB, 0.15122E+01_JPRB, 0.91429E+00_JPRB ,&
&  0.82038E+00_JPRB, 0.79972E+00_JPRB, 0.78547E+00_JPRB, 0.76579E+00_JPRB, 0.74434E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.68358E+00_JPRB, 0.64504E+00_JPRB, 0.62412E+00_JPRB, 0.60801E+00_JPRB, 0.59661E+00_JPRB, 0.61659E+00_JPRB, 0.63465E+00_JPRB ,&
&  0.66222E+00_JPRB, 0.69529E+00_JPRB, 0.73494E+00_JPRB, 0.77086E+00_JPRB, 0.81809E+00_JPRB, 0.86590E+00_JPRB, 0.90840E+00_JPRB ,&
&  0.94716E+00_JPRB, 0.98331E+00_JPRB, 0.10196E+01_JPRB, 0.10445E+01_JPRB, 0.10685E+01_JPRB, 0.10904E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.11913E+01_JPRB, 0.12895E+01_JPRB, 0.13999E+01_JPRB, 0.14949E+01_JPRB, 0.16496E+01_JPRB, 0.10530E+01_JPRB ,&
&  0.14408E+01_JPRB, 0.14553E+01_JPRB, 0.14073E+01_JPRB, 0.13046E+01_JPRB, 0.12124E+01_JPRB, 0.11545E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.11875E+01_JPRB, 0.11441E+01_JPRB, 0.11652E+01_JPRB, 0.11059E+01_JPRB, 0.10393E+01_JPRB, 0.11362E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.13272E+01_JPRB, 0.14017E+01_JPRB, 0.14572E+01_JPRB, 0.14947E+01_JPRB, 0.15289E+01_JPRB, 0.15379E+01_JPRB ,&
&  0.15367E+01_JPRB, 0.14881E+01_JPRB, 0.14195E+01_JPRB, 0.14096E+01_JPRB, 0.14088E+01_JPRB, 0.12211E+01_JPRB, 0.11294E+01_JPRB ,&
&  0.88839E+00_JPRB, 0.92394E+00_JPRB, 0.15028E+01_JPRB, 0.19789E+01_JPRB, 0.18180E+01_JPRB, 0.17269E+01_JPRB, 0.15526E+01_JPRB ,&
&  0.13918E+01_JPRB, 0.13101E+01_JPRB, 0.12640E+01_JPRB, 0.12036E+01_JPRB, 0.11150E+01_JPRB, 0.10573E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.96640E+00_JPRB, 0.94538E+00_JPRB, 0.10587E+01_JPRB, 0.11462E+01_JPRB, 0.13747E+01_JPRB, 0.14627E+01_JPRB ,&
&  0.16814E+01_JPRB, 0.14975E+01_JPRB, 0.11222E+01_JPRB, 0.10013E+01_JPRB, 0.68942E+00_JPRB, 0.59522E+00_JPRB, 0.56758E+00_JPRB ,&
&  0.58416E+00_JPRB, 0.52783E+00_JPRB, 0.10290E+01_JPRB, 0.38368E+01_JPRB, 0.37647E+01_JPRB, 0.47526E+01_JPRB, 0.25184E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.80273E+00_JPRB, 0.71714E+00_JPRB, 0.71235E+00_JPRB, 0.69667E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.60009E+00_JPRB, 0.57644E+00_JPRB, 0.55073E+00_JPRB, 0.55668E+00_JPRB, 0.56335E+00_JPRB, 0.57206E+00_JPRB, 0.60270E+00_JPRB ,&
&  0.66570E+00_JPRB, 0.68358E+00_JPRB, 0.71542E+00_JPRB, 0.76773E+00_JPRB, 0.84452E+00_JPRB, 0.91368E+00_JPRB, 0.98084E+00_JPRB ,&
&  0.10520E+01_JPRB, 0.11102E+01_JPRB, 0.11451E+01_JPRB, 0.11676E+01_JPRB, 0.12038E+01_JPRB, 0.12456E+01_JPRB, 0.12978E+01_JPRB ,&
&  0.13326E+01_JPRB, 0.13989E+01_JPRB, 0.15119E+01_JPRB, 0.16335E+01_JPRB, 0.17469E+01_JPRB, 0.17873E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.13223E+01_JPRB, 0.13527E+01_JPRB, 0.13262E+01_JPRB, 0.12456E+01_JPRB, 0.11854E+01_JPRB, 0.12209E+01_JPRB ,&
&  0.13075E+01_JPRB, 0.13763E+01_JPRB, 0.13280E+01_JPRB, 0.12551E+01_JPRB, 0.13640E+01_JPRB, 0.12699E+01_JPRB, 0.14356E+01_JPRB ,&
&  0.15129E+01_JPRB, 0.15788E+01_JPRB, 0.16601E+01_JPRB, 0.17062E+01_JPRB, 0.17668E+01_JPRB, 0.18416E+01_JPRB, 0.19093E+01_JPRB ,&
&  0.19319E+01_JPRB, 0.19096E+01_JPRB, 0.19365E+01_JPRB, 0.20485E+01_JPRB, 0.18776E+01_JPRB, 0.20404E+01_JPRB, 0.16757E+01_JPRB ,&
&  0.11743E+01_JPRB, 0.10390E+01_JPRB, 0.13737E+01_JPRB, 0.17820E+01_JPRB, 0.17461E+01_JPRB, 0.16941E+01_JPRB, 0.15452E+01_JPRB ,&
&  0.13343E+01_JPRB, 0.11994E+01_JPRB, 0.11450E+01_JPRB, 0.10916E+01_JPRB, 0.10237E+01_JPRB, 0.96429E+00_JPRB, 0.92927E+00_JPRB ,&
&  0.87696E+00_JPRB, 0.82142E+00_JPRB, 0.85167E+00_JPRB, 0.87008E+00_JPRB, 0.93248E+00_JPRB, 0.12189E+01_JPRB, 0.18346E+01_JPRB ,&
&  0.28953E+01_JPRB, 0.15732E+01_JPRB, 0.11660E+01_JPRB, 0.95152E+00_JPRB, 0.73974E+00_JPRB, 0.63670E+00_JPRB, 0.58180E+00_JPRB ,&
&  0.56795E+00_JPRB, 0.67949E+00_JPRB, 0.81458E+00_JPRB, 0.15003E+01_JPRB, 0.26253E+01_JPRB, 0.26712E+01_JPRB, 0.18175E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.79920E+00_JPRB, 0.65008E+00_JPRB, 0.62310E+00_JPRB, 0.60284E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.54888E+00_JPRB, 0.53679E+00_JPRB, 0.53448E+00_JPRB, 0.53091E+00_JPRB, 0.56753E+00_JPRB, 0.59272E+00_JPRB, 0.61736E+00_JPRB ,&
&  0.65660E+00_JPRB, 0.68380E+00_JPRB, 0.72258E+00_JPRB, 0.79551E+00_JPRB, 0.87957E+00_JPRB, 0.95559E+00_JPRB, 0.10059E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10732E+01_JPRB, 0.11002E+01_JPRB, 0.11267E+01_JPRB, 0.11885E+01_JPRB, 0.12462E+01_JPRB, 0.13366E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.15663E+01_JPRB, 0.17060E+01_JPRB, 0.18344E+01_JPRB, 0.19783E+01_JPRB, 0.21093E+01_JPRB, 0.22132E+01_JPRB ,&
&  0.96472E+00_JPRB, 0.12569E+01_JPRB, 0.13017E+01_JPRB, 0.12952E+01_JPRB, 0.12278E+01_JPRB, 0.12407E+01_JPRB, 0.13924E+01_JPRB ,&
&  0.15697E+01_JPRB, 0.15942E+01_JPRB, 0.17235E+01_JPRB, 0.14014E+01_JPRB, 0.15205E+01_JPRB, 0.15360E+01_JPRB, 0.16699E+01_JPRB ,&
&  0.17612E+01_JPRB, 0.18261E+01_JPRB, 0.19160E+01_JPRB, 0.20119E+01_JPRB, 0.21197E+01_JPRB, 0.22365E+01_JPRB, 0.23825E+01_JPRB ,&
&  0.25181E+01_JPRB, 0.26870E+01_JPRB, 0.29167E+01_JPRB, 0.28687E+01_JPRB, 0.26562E+01_JPRB, 0.28158E+01_JPRB, 0.26723E+01_JPRB ,&
&  0.22669E+01_JPRB, 0.16689E+01_JPRB, 0.16466E+01_JPRB, 0.17298E+01_JPRB, 0.16828E+01_JPRB, 0.16414E+01_JPRB, 0.15181E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.11992E+01_JPRB, 0.11068E+01_JPRB, 0.10442E+01_JPRB, 0.97288E+00_JPRB, 0.92148E+00_JPRB, 0.88875E+00_JPRB ,&
&  0.85969E+00_JPRB, 0.87084E+00_JPRB, 0.88804E+00_JPRB, 0.82699E+00_JPRB, 0.80080E+00_JPRB, 0.89410E+00_JPRB, 0.10808E+01_JPRB ,&
&  0.12922E+01_JPRB, 0.11190E+01_JPRB, 0.95579E+00_JPRB, 0.94020E+00_JPRB, 0.86198E+00_JPRB, 0.77777E+00_JPRB, 0.73941E+00_JPRB ,&
&  0.73257E+00_JPRB, 0.71668E+00_JPRB, 0.82288E+00_JPRB, 0.12365E+01_JPRB, 0.15721E+01_JPRB, 0.17936E+01_JPRB, 0.14861E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.80798E+00_JPRB, 0.65264E+00_JPRB, 0.61329E+00_JPRB, 0.59309E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.58466E+00_JPRB, 0.57427E+00_JPRB, 0.57563E+00_JPRB, 0.57280E+00_JPRB, 0.60764E+00_JPRB, 0.64736E+00_JPRB, 0.66012E+00_JPRB ,&
&  0.65919E+00_JPRB, 0.68285E+00_JPRB, 0.70723E+00_JPRB, 0.74958E+00_JPRB, 0.82987E+00_JPRB, 0.87927E+00_JPRB, 0.93659E+00_JPRB ,&
&  0.99422E+00_JPRB, 0.10154E+01_JPRB, 0.10164E+01_JPRB, 0.10458E+01_JPRB, 0.11228E+01_JPRB, 0.12087E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.14633E+01_JPRB, 0.16393E+01_JPRB, 0.18582E+01_JPRB, 0.21018E+01_JPRB, 0.23746E+01_JPRB, 0.26412E+01_JPRB, 0.27850E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.11118E+01_JPRB, 0.12802E+01_JPRB, 0.12614E+01_JPRB, 0.12220E+01_JPRB, 0.12965E+01_JPRB, 0.14224E+01_JPRB ,&
&  0.14786E+01_JPRB, 0.13395E+01_JPRB, 0.13996E+01_JPRB, 0.16808E+01_JPRB, 0.16276E+01_JPRB, 0.17145E+01_JPRB, 0.18164E+01_JPRB ,&
&  0.19027E+01_JPRB, 0.19970E+01_JPRB, 0.20757E+01_JPRB, 0.21445E+01_JPRB, 0.22354E+01_JPRB, 0.23535E+01_JPRB, 0.25152E+01_JPRB ,&
&  0.27234E+01_JPRB, 0.30365E+01_JPRB, 0.33443E+01_JPRB, 0.30547E+01_JPRB, 0.31342E+01_JPRB, 0.33148E+01_JPRB, 0.37560E+01_JPRB ,&
&  0.33955E+01_JPRB, 0.24264E+01_JPRB, 0.18821E+01_JPRB, 0.17155E+01_JPRB, 0.15181E+01_JPRB, 0.14657E+01_JPRB, 0.13747E+01_JPRB ,&
&  0.12718E+01_JPRB, 0.11849E+01_JPRB, 0.11004E+01_JPRB, 0.10282E+01_JPRB, 0.94110E+00_JPRB, 0.90701E+00_JPRB, 0.89006E+00_JPRB ,&
&  0.92495E+00_JPRB, 0.95366E+00_JPRB, 0.92200E+00_JPRB, 0.87116E+00_JPRB, 0.85032E+00_JPRB, 0.89492E+00_JPRB, 0.97914E+00_JPRB ,&
&  0.10248E+01_JPRB, 0.10161E+01_JPRB, 0.10184E+01_JPRB, 0.10274E+01_JPRB, 0.10274E+01_JPRB, 0.10205E+01_JPRB, 0.98370E+00_JPRB ,&
&  0.92466E+00_JPRB, 0.87281E+00_JPRB, 0.86973E+00_JPRB, 0.85517E+00_JPRB, 0.10103E+01_JPRB, 0.13030E+01_JPRB, 0.13383E+01_JPRB ,&
&  0.11456E+01_JPRB, 0.89041E+00_JPRB, 0.72576E+00_JPRB, 0.67481E+00_JPRB, 0.64992E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.69605E+00_JPRB, 0.65807E+00_JPRB, 0.64991E+00_JPRB, 0.63454E+00_JPRB, 0.65002E+00_JPRB, 0.67905E+00_JPRB, 0.70033E+00_JPRB ,&
&  0.70764E+00_JPRB, 0.70846E+00_JPRB, 0.71341E+00_JPRB, 0.73996E+00_JPRB, 0.77433E+00_JPRB, 0.81142E+00_JPRB, 0.85777E+00_JPRB ,&
&  0.89109E+00_JPRB, 0.91342E+00_JPRB, 0.90234E+00_JPRB, 0.89126E+00_JPRB, 0.93853E+00_JPRB, 0.10102E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.13839E+01_JPRB, 0.16082E+01_JPRB, 0.19558E+01_JPRB, 0.24770E+01_JPRB, 0.31099E+01_JPRB, 0.37730E+01_JPRB ,&
&  0.30566E+01_JPRB, 0.13841E+01_JPRB, 0.14791E+01_JPRB, 0.15214E+01_JPRB, 0.13686E+01_JPRB, 0.14491E+01_JPRB, 0.14869E+01_JPRB ,&
&  0.14506E+01_JPRB, 0.14015E+01_JPRB, 0.12892E+01_JPRB, 0.13595E+01_JPRB, 0.15424E+01_JPRB, 0.16768E+01_JPRB, 0.18016E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.19669E+01_JPRB, 0.20011E+01_JPRB, 0.20363E+01_JPRB, 0.21116E+01_JPRB, 0.22273E+01_JPRB, 0.23882E+01_JPRB ,&
&  0.26147E+01_JPRB, 0.28976E+01_JPRB, 0.32543E+01_JPRB, 0.29586E+01_JPRB, 0.33165E+01_JPRB, 0.33327E+01_JPRB, 0.33826E+01_JPRB ,&
&  0.31448E+01_JPRB, 0.22248E+01_JPRB, 0.17457E+01_JPRB, 0.16582E+01_JPRB, 0.13841E+01_JPRB, 0.10839E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.11743E+01_JPRB, 0.11475E+01_JPRB, 0.11129E+01_JPRB, 0.10220E+01_JPRB, 0.94206E+00_JPRB, 0.91760E+00_JPRB, 0.96655E+00_JPRB ,&
&  0.10266E+01_JPRB, 0.10616E+01_JPRB, 0.10051E+01_JPRB, 0.99279E+00_JPRB, 0.97216E+00_JPRB, 0.10046E+01_JPRB, 0.10549E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.11330E+01_JPRB, 0.11559E+01_JPRB, 0.11885E+01_JPRB, 0.12245E+01_JPRB, 0.11590E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.10257E+01_JPRB, 0.96897E+00_JPRB, 0.92935E+00_JPRB, 0.89867E+00_JPRB, 0.92710E+00_JPRB, 0.10293E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.89729E+00_JPRB, 0.78001E+00_JPRB, 0.75029E+00_JPRB, 0.75633E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.80515E+00_JPRB, 0.71731E+00_JPRB, 0.67081E+00_JPRB, 0.66643E+00_JPRB, 0.65342E+00_JPRB, 0.66048E+00_JPRB, 0.67124E+00_JPRB ,&
&  0.66921E+00_JPRB, 0.68972E+00_JPRB, 0.68606E+00_JPRB, 0.70352E+00_JPRB, 0.72573E+00_JPRB, 0.73563E+00_JPRB, 0.76612E+00_JPRB ,&
&  0.79546E+00_JPRB, 0.82766E+00_JPRB, 0.85903E+00_JPRB, 0.83569E+00_JPRB, 0.85166E+00_JPRB, 0.88328E+00_JPRB, 0.94143E+00_JPRB ,&
&  0.10444E+01_JPRB, 0.11815E+01_JPRB, 0.13418E+01_JPRB, 0.15368E+01_JPRB, 0.18205E+01_JPRB, 0.23299E+01_JPRB, 0.32645E+01_JPRB ,&
&  0.55597E+01_JPRB, 0.47259E+01_JPRB, 0.14090E+01_JPRB, 0.17901E+01_JPRB, 0.17240E+01_JPRB, 0.17281E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.16078E+01_JPRB, 0.16894E+01_JPRB, 0.14047E+01_JPRB, 0.12240E+01_JPRB, 0.12553E+01_JPRB, 0.14599E+01_JPRB, 0.16345E+01_JPRB ,&
&  0.17347E+01_JPRB, 0.18060E+01_JPRB, 0.18262E+01_JPRB, 0.18462E+01_JPRB, 0.18835E+01_JPRB, 0.19630E+01_JPRB, 0.20936E+01_JPRB ,&
&  0.22946E+01_JPRB, 0.25794E+01_JPRB, 0.29558E+01_JPRB, 0.28724E+01_JPRB, 0.34748E+01_JPRB, 0.33558E+01_JPRB, 0.31528E+01_JPRB ,&
&  0.26528E+01_JPRB, 0.22152E+01_JPRB, 0.19335E+01_JPRB, 0.16287E+01_JPRB, 0.12172E+01_JPRB, 0.10171E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.10036E+01_JPRB, 0.97290E+00_JPRB, 0.96462E+00_JPRB, 0.92383E+00_JPRB, 0.99113E+00_JPRB, 0.10805E+01_JPRB ,&
&  0.11281E+01_JPRB, 0.11264E+01_JPRB, 0.11000E+01_JPRB, 0.10910E+01_JPRB, 0.10876E+01_JPRB, 0.11024E+01_JPRB, 0.11417E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11994E+01_JPRB, 0.12136E+01_JPRB, 0.12489E+01_JPRB, 0.12831E+01_JPRB, 0.12097E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10838E+01_JPRB, 0.10972E+01_JPRB, 0.10609E+01_JPRB, 0.10288E+01_JPRB, 0.96914E+00_JPRB, 0.92981E+00_JPRB, 0.91910E+00_JPRB ,&
&  0.99913E+00_JPRB, 0.93528E+00_JPRB, 0.87737E+00_JPRB, 0.81129E+00_JPRB, 0.83436E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.85090E+00_JPRB, 0.79948E+00_JPRB, 0.71554E+00_JPRB, 0.68518E+00_JPRB, 0.67813E+00_JPRB, 0.66433E+00_JPRB, 0.64531E+00_JPRB ,&
&  0.62175E+00_JPRB, 0.61331E+00_JPRB, 0.64121E+00_JPRB, 0.67414E+00_JPRB, 0.70087E+00_JPRB, 0.70617E+00_JPRB, 0.71987E+00_JPRB ,&
&  0.72902E+00_JPRB, 0.75438E+00_JPRB, 0.80205E+00_JPRB, 0.82117E+00_JPRB, 0.82281E+00_JPRB, 0.83232E+00_JPRB, 0.85122E+00_JPRB ,&
&  0.88564E+00_JPRB, 0.94658E+00_JPRB, 0.10541E+01_JPRB, 0.11771E+01_JPRB, 0.13409E+01_JPRB, 0.16305E+01_JPRB, 0.22374E+01_JPRB ,&
&  0.34662E+01_JPRB, 0.63924E+01_JPRB, 0.15310E+01_JPRB, 0.21350E+01_JPRB, 0.21027E+01_JPRB, 0.19552E+01_JPRB, 0.18188E+01_JPRB ,&
&  0.18259E+01_JPRB, 0.18049E+01_JPRB, 0.16294E+01_JPRB, 0.13209E+01_JPRB, 0.10551E+01_JPRB, 0.11400E+01_JPRB, 0.13432E+01_JPRB ,&
&  0.15174E+01_JPRB, 0.15907E+01_JPRB, 0.16116E+01_JPRB, 0.15944E+01_JPRB, 0.16024E+01_JPRB, 0.16285E+01_JPRB, 0.16971E+01_JPRB ,&
&  0.18375E+01_JPRB, 0.20735E+01_JPRB, 0.24037E+01_JPRB, 0.24920E+01_JPRB, 0.32057E+01_JPRB, 0.34148E+01_JPRB, 0.32868E+01_JPRB ,&
&  0.27068E+01_JPRB, 0.24965E+01_JPRB, 0.22577E+01_JPRB, 0.17772E+01_JPRB, 0.13303E+01_JPRB, 0.10977E+01_JPRB, 0.10984E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.99034E+00_JPRB, 0.96894E+00_JPRB, 0.95216E+00_JPRB, 0.96563E+00_JPRB, 0.10402E+01_JPRB, 0.11096E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.11403E+01_JPRB, 0.11181E+01_JPRB, 0.11141E+01_JPRB, 0.11218E+01_JPRB, 0.11341E+01_JPRB, 0.11735E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.12237E+01_JPRB, 0.12357E+01_JPRB, 0.12140E+01_JPRB, 0.12330E+01_JPRB, 0.11965E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11504E+01_JPRB, 0.12022E+01_JPRB, 0.11259E+01_JPRB, 0.10685E+01_JPRB, 0.10742E+01_JPRB, 0.10414E+01_JPRB, 0.10063E+01_JPRB ,&
&  0.10399E+01_JPRB, 0.10524E+01_JPRB, 0.10219E+01_JPRB, 0.89400E+00_JPRB, 0.89861E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.90040E+00_JPRB, 0.85163E+00_JPRB, 0.79276E+00_JPRB, 0.72595E+00_JPRB, 0.72337E+00_JPRB, 0.72849E+00_JPRB, 0.69435E+00_JPRB ,&
&  0.64002E+00_JPRB, 0.63333E+00_JPRB, 0.64196E+00_JPRB, 0.66587E+00_JPRB, 0.68417E+00_JPRB, 0.69068E+00_JPRB, 0.71894E+00_JPRB ,&
&  0.71449E+00_JPRB, 0.71181E+00_JPRB, 0.73611E+00_JPRB, 0.77532E+00_JPRB, 0.79067E+00_JPRB, 0.80163E+00_JPRB, 0.82457E+00_JPRB ,&
&  0.83601E+00_JPRB, 0.83117E+00_JPRB, 0.85539E+00_JPRB, 0.91278E+00_JPRB, 0.10213E+01_JPRB, 0.12158E+01_JPRB, 0.15963E+01_JPRB ,&
&  0.27329E+01_JPRB, 0.67089E+01_JPRB, 0.40244E+01_JPRB, 0.37929E+01_JPRB, 0.27798E+01_JPRB, 0.22582E+01_JPRB, 0.20616E+01_JPRB ,&
&  0.22614E+01_JPRB, 0.21181E+01_JPRB, 0.19144E+01_JPRB, 0.14279E+01_JPRB, 0.11388E+01_JPRB, 0.10117E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.13045E+01_JPRB, 0.13657E+01_JPRB, 0.13696E+01_JPRB, 0.13717E+01_JPRB, 0.13924E+01_JPRB, 0.14150E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.16348E+01_JPRB, 0.18942E+01_JPRB, 0.21988E+01_JPRB, 0.25900E+01_JPRB, 0.30553E+01_JPRB, 0.35214E+01_JPRB ,&
&  0.33813E+01_JPRB, 0.30397E+01_JPRB, 0.25562E+01_JPRB, 0.19139E+01_JPRB, 0.14040E+01_JPRB, 0.13256E+01_JPRB, 0.11385E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.10083E+01_JPRB, 0.99749E+00_JPRB, 0.10160E+01_JPRB, 0.10460E+01_JPRB, 0.10955E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11457E+01_JPRB, 0.11422E+01_JPRB, 0.11344E+01_JPRB, 0.11305E+01_JPRB, 0.11310E+01_JPRB, 0.11341E+01_JPRB, 0.11477E+01_JPRB ,&
&  0.11623E+01_JPRB, 0.11949E+01_JPRB, 0.11616E+01_JPRB, 0.11558E+01_JPRB, 0.11656E+01_JPRB, 0.11591E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.11941E+01_JPRB, 0.11878E+01_JPRB, 0.11615E+01_JPRB, 0.11421E+01_JPRB, 0.11457E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.11158E+01_JPRB, 0.10993E+01_JPRB, 0.10198E+01_JPRB, 0.97328E+00_JPRB, 0.96130E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.95689E+00_JPRB, 0.91648E+00_JPRB, 0.86936E+00_JPRB, 0.81123E+00_JPRB, 0.77754E+00_JPRB, 0.77751E+00_JPRB, 0.78588E+00_JPRB ,&
&  0.75376E+00_JPRB, 0.72579E+00_JPRB, 0.71370E+00_JPRB, 0.72750E+00_JPRB, 0.75111E+00_JPRB, 0.75835E+00_JPRB, 0.76310E+00_JPRB ,&
&  0.75380E+00_JPRB, 0.74960E+00_JPRB, 0.73668E+00_JPRB, 0.74304E+00_JPRB, 0.79675E+00_JPRB, 0.80841E+00_JPRB, 0.80534E+00_JPRB ,&
&  0.82552E+00_JPRB, 0.81724E+00_JPRB, 0.80149E+00_JPRB, 0.81343E+00_JPRB, 0.86185E+00_JPRB, 0.99273E+00_JPRB, 0.12914E+01_JPRB ,&
&  0.24947E+01_JPRB, 0.61417E+01_JPRB, 0.27143E+01_JPRB, 0.44110E+01_JPRB, 0.30795E+01_JPRB, 0.25069E+01_JPRB, 0.20071E+01_JPRB ,&
&  0.20346E+01_JPRB, 0.23563E+01_JPRB, 0.18551E+01_JPRB, 0.15157E+01_JPRB, 0.12484E+01_JPRB, 0.11075E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.11924E+01_JPRB, 0.11978E+01_JPRB, 0.12090E+01_JPRB, 0.11932E+01_JPRB, 0.11940E+01_JPRB, 0.12164E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.13163E+01_JPRB, 0.14796E+01_JPRB, 0.17095E+01_JPRB, 0.22855E+01_JPRB, 0.29761E+01_JPRB, 0.37304E+01_JPRB ,&
&  0.37620E+01_JPRB, 0.37739E+01_JPRB, 0.29173E+01_JPRB, 0.20825E+01_JPRB, 0.15649E+01_JPRB, 0.13906E+01_JPRB, 0.12276E+01_JPRB ,&
&  0.11404E+01_JPRB, 0.10968E+01_JPRB, 0.10471E+01_JPRB, 0.10443E+01_JPRB, 0.11080E+01_JPRB, 0.11599E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.11481E+01_JPRB, 0.11389E+01_JPRB, 0.11251E+01_JPRB, 0.11359E+01_JPRB, 0.11373E+01_JPRB, 0.11293E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11454E+01_JPRB, 0.11308E+01_JPRB, 0.11357E+01_JPRB, 0.11431E+01_JPRB, 0.11409E+01_JPRB, 0.11021E+01_JPRB ,&
&  0.11315E+01_JPRB, 0.11669E+01_JPRB, 0.11774E+01_JPRB, 0.11700E+01_JPRB, 0.11589E+01_JPRB, 0.12049E+01_JPRB, 0.11645E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.11346E+01_JPRB, 0.10700E+01_JPRB, 0.10368E+01_JPRB, 0.10171E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.10225E+01_JPRB, 0.96780E+00_JPRB, 0.92685E+00_JPRB, 0.88123E+00_JPRB, 0.84566E+00_JPRB, 0.82437E+00_JPRB, 0.82336E+00_JPRB ,&
&  0.81046E+00_JPRB, 0.77762E+00_JPRB, 0.75508E+00_JPRB, 0.75472E+00_JPRB, 0.75456E+00_JPRB, 0.76202E+00_JPRB, 0.76790E+00_JPRB ,&
&  0.76439E+00_JPRB, 0.75445E+00_JPRB, 0.75108E+00_JPRB, 0.75360E+00_JPRB, 0.78195E+00_JPRB, 0.82717E+00_JPRB, 0.84512E+00_JPRB ,&
&  0.83608E+00_JPRB, 0.82493E+00_JPRB, 0.80332E+00_JPRB, 0.78854E+00_JPRB, 0.80115E+00_JPRB, 0.88615E+00_JPRB, 0.11431E+01_JPRB ,&
&  0.23727E+01_JPRB, 0.71577E+01_JPRB, 0.28420E+01_JPRB, 0.48282E+01_JPRB, 0.33127E+01_JPRB, 0.26327E+01_JPRB, 0.20654E+01_JPRB ,&
&  0.20321E+01_JPRB, 0.22111E+01_JPRB, 0.18954E+01_JPRB, 0.16259E+01_JPRB, 0.14118E+01_JPRB, 0.12641E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.12056E+01_JPRB, 0.11732E+01_JPRB, 0.11554E+01_JPRB, 0.11263E+01_JPRB, 0.11151E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.11247E+01_JPRB, 0.11964E+01_JPRB, 0.15124E+01_JPRB, 0.19268E+01_JPRB, 0.28626E+01_JPRB, 0.42501E+01_JPRB ,&
&  0.42432E+01_JPRB, 0.44015E+01_JPRB, 0.32386E+01_JPRB, 0.22837E+01_JPRB, 0.16869E+01_JPRB, 0.14489E+01_JPRB, 0.13464E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.11762E+01_JPRB, 0.11190E+01_JPRB, 0.11026E+01_JPRB, 0.11372E+01_JPRB, 0.11488E+01_JPRB, 0.11656E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.11251E+01_JPRB, 0.11037E+01_JPRB, 0.10958E+01_JPRB, 0.11111E+01_JPRB, 0.11199E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.11101E+01_JPRB, 0.11123E+01_JPRB, 0.11250E+01_JPRB, 0.11410E+01_JPRB, 0.11405E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11582E+01_JPRB, 0.11569E+01_JPRB, 0.11936E+01_JPRB, 0.12077E+01_JPRB, 0.12950E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.11476E+01_JPRB, 0.11014E+01_JPRB, 0.10797E+01_JPRB, 0.10692E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.10765E+01_JPRB, 0.10350E+01_JPRB, 0.10005E+01_JPRB, 0.95956E+00_JPRB, 0.90907E+00_JPRB, 0.88030E+00_JPRB, 0.85926E+00_JPRB ,&
&  0.84541E+00_JPRB, 0.81292E+00_JPRB, 0.78821E+00_JPRB, 0.78294E+00_JPRB, 0.78597E+00_JPRB, 0.77946E+00_JPRB, 0.77888E+00_JPRB ,&
&  0.78636E+00_JPRB, 0.79026E+00_JPRB, 0.78144E+00_JPRB, 0.77147E+00_JPRB, 0.77103E+00_JPRB, 0.80828E+00_JPRB, 0.83097E+00_JPRB ,&
&  0.84088E+00_JPRB, 0.82802E+00_JPRB, 0.80275E+00_JPRB, 0.77397E+00_JPRB, 0.77030E+00_JPRB, 0.81263E+00_JPRB, 0.10108E+01_JPRB ,&
&  0.20272E+01_JPRB, 0.27881E+01_JPRB, 0.50186E+01_JPRB, 0.51253E+01_JPRB, 0.35202E+01_JPRB, 0.27745E+01_JPRB, 0.23867E+01_JPRB ,&
&  0.22969E+01_JPRB, 0.21596E+01_JPRB, 0.18977E+01_JPRB, 0.16970E+01_JPRB, 0.15301E+01_JPRB, 0.13905E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.12208E+01_JPRB, 0.11898E+01_JPRB, 0.11729E+01_JPRB, 0.11594E+01_JPRB, 0.11443E+01_JPRB, 0.11360E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.11281E+01_JPRB, 0.11337E+01_JPRB, 0.12515E+01_JPRB, 0.17493E+01_JPRB, 0.25927E+01_JPRB, 0.38769E+01_JPRB ,&
&  0.46026E+01_JPRB, 0.43013E+01_JPRB, 0.31086E+01_JPRB, 0.23353E+01_JPRB, 0.17619E+01_JPRB, 0.14578E+01_JPRB, 0.13650E+01_JPRB ,&
&  0.13171E+01_JPRB, 0.12727E+01_JPRB, 0.12015E+01_JPRB, 0.11665E+01_JPRB, 0.11575E+01_JPRB, 0.11448E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.11333E+01_JPRB, 0.11195E+01_JPRB, 0.11053E+01_JPRB, 0.10923E+01_JPRB, 0.10800E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.10661E+01_JPRB, 0.10865E+01_JPRB, 0.11065E+01_JPRB, 0.11605E+01_JPRB, 0.11763E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.11530E+01_JPRB, 0.11502E+01_JPRB, 0.12080E+01_JPRB, 0.12102E+01_JPRB, 0.13333E+01_JPRB, 0.12351E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.11747E+01_JPRB, 0.11370E+01_JPRB, 0.11023E+01_JPRB, 0.10865E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.10889E+01_JPRB, 0.10437E+01_JPRB, 0.10109E+01_JPRB, 0.98633E+00_JPRB, 0.95162E+00_JPRB, 0.91662E+00_JPRB, 0.90392E+00_JPRB ,&
&  0.89585E+00_JPRB, 0.87141E+00_JPRB, 0.84277E+00_JPRB, 0.80763E+00_JPRB, 0.79592E+00_JPRB, 0.79943E+00_JPRB, 0.80539E+00_JPRB ,&
&  0.80545E+00_JPRB, 0.80766E+00_JPRB, 0.80713E+00_JPRB, 0.80491E+00_JPRB, 0.80218E+00_JPRB, 0.82412E+00_JPRB, 0.83593E+00_JPRB ,&
&  0.83091E+00_JPRB, 0.83202E+00_JPRB, 0.80989E+00_JPRB, 0.78653E+00_JPRB, 0.77627E+00_JPRB, 0.77091E+00_JPRB, 0.81508E+00_JPRB ,&
&  0.15073E+01_JPRB, 0.34445E+01_JPRB, 0.49774E+01_JPRB, 0.45216E+01_JPRB, 0.34343E+01_JPRB, 0.26612E+01_JPRB, 0.24425E+01_JPRB ,&
&  0.22661E+01_JPRB, 0.21037E+01_JPRB, 0.18875E+01_JPRB, 0.17172E+01_JPRB, 0.15832E+01_JPRB, 0.14697E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.12759E+01_JPRB, 0.12382E+01_JPRB, 0.12050E+01_JPRB, 0.11902E+01_JPRB, 0.11770E+01_JPRB, 0.11798E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.11818E+01_JPRB, 0.11759E+01_JPRB, 0.12078E+01_JPRB, 0.14924E+01_JPRB, 0.16859E+01_JPRB, 0.27644E+01_JPRB ,&
&  0.38371E+01_JPRB, 0.31507E+01_JPRB, 0.24693E+01_JPRB, 0.20829E+01_JPRB, 0.17616E+01_JPRB, 0.14782E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.12532E+01_JPRB, 0.11992E+01_JPRB, 0.11666E+01_JPRB, 0.11380E+01_JPRB, 0.11242E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.10945E+01_JPRB, 0.10935E+01_JPRB, 0.10929E+01_JPRB, 0.10775E+01_JPRB, 0.10663E+01_JPRB, 0.10550E+01_JPRB ,&
&  0.10459E+01_JPRB, 0.10284E+01_JPRB, 0.10498E+01_JPRB, 0.10646E+01_JPRB, 0.11441E+01_JPRB, 0.12081E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.11804E+01_JPRB, 0.11768E+01_JPRB, 0.11779E+01_JPRB, 0.12272E+01_JPRB, 0.13230E+01_JPRB, 0.14550E+01_JPRB, 0.13151E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.12192E+01_JPRB, 0.11826E+01_JPRB, 0.11407E+01_JPRB, 0.11222E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.11361E+01_JPRB, 0.10822E+01_JPRB, 0.10374E+01_JPRB, 0.99577E+00_JPRB, 0.97799E+00_JPRB, 0.95158E+00_JPRB, 0.92199E+00_JPRB ,&
&  0.90498E+00_JPRB, 0.90081E+00_JPRB, 0.89022E+00_JPRB, 0.86862E+00_JPRB, 0.84935E+00_JPRB, 0.84140E+00_JPRB, 0.84005E+00_JPRB ,&
&  0.83845E+00_JPRB, 0.83811E+00_JPRB, 0.83637E+00_JPRB, 0.82843E+00_JPRB, 0.83046E+00_JPRB, 0.83914E+00_JPRB, 0.84883E+00_JPRB ,&
&  0.83247E+00_JPRB, 0.81831E+00_JPRB, 0.80699E+00_JPRB, 0.79484E+00_JPRB, 0.79095E+00_JPRB, 0.77366E+00_JPRB, 0.76075E+00_JPRB ,&
&  0.10632E+01_JPRB, 0.22094E+01_JPRB, 0.31790E+01_JPRB, 0.39443E+01_JPRB, 0.33761E+01_JPRB, 0.27475E+01_JPRB, 0.24655E+01_JPRB ,&
&  0.22454E+01_JPRB, 0.20077E+01_JPRB, 0.18070E+01_JPRB, 0.16759E+01_JPRB, 0.15934E+01_JPRB, 0.14982E+01_JPRB, 0.13786E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.12323E+01_JPRB, 0.12138E+01_JPRB, 0.12145E+01_JPRB, 0.12097E+01_JPRB, 0.12169E+01_JPRB, 0.12180E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.11970E+01_JPRB, 0.11760E+01_JPRB, 0.11612E+01_JPRB, 0.12868E+01_JPRB, 0.13982E+01_JPRB, 0.17426E+01_JPRB ,&
&  0.18930E+01_JPRB, 0.19449E+01_JPRB, 0.18538E+01_JPRB, 0.17771E+01_JPRB, 0.16640E+01_JPRB, 0.14720E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.12030E+01_JPRB, 0.11581E+01_JPRB, 0.11375E+01_JPRB, 0.11165E+01_JPRB, 0.10980E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.10745E+01_JPRB, 0.10661E+01_JPRB, 0.10699E+01_JPRB, 0.10592E+01_JPRB, 0.10345E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.10050E+01_JPRB, 0.98812E+00_JPRB, 0.99930E+00_JPRB, 0.10395E+01_JPRB, 0.10821E+01_JPRB, 0.11114E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.12271E+01_JPRB, 0.12059E+01_JPRB, 0.12805E+01_JPRB, 0.13965E+01_JPRB, 0.15116E+01_JPRB, 0.13843E+01_JPRB ,&
&  0.13077E+01_JPRB, 0.12608E+01_JPRB, 0.12184E+01_JPRB, 0.11593E+01_JPRB, 0.11791E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.11409E+01_JPRB, 0.11084E+01_JPRB, 0.10703E+01_JPRB, 0.10217E+01_JPRB, 0.99783E+00_JPRB, 0.97647E+00_JPRB, 0.95506E+00_JPRB ,&
&  0.94359E+00_JPRB, 0.93240E+00_JPRB, 0.92384E+00_JPRB, 0.90683E+00_JPRB, 0.88956E+00_JPRB, 0.87867E+00_JPRB, 0.86644E+00_JPRB ,&
&  0.86644E+00_JPRB, 0.86666E+00_JPRB, 0.85898E+00_JPRB, 0.85054E+00_JPRB, 0.84615E+00_JPRB, 0.84230E+00_JPRB, 0.84592E+00_JPRB ,&
&  0.83835E+00_JPRB, 0.82806E+00_JPRB, 0.82763E+00_JPRB, 0.81947E+00_JPRB, 0.80539E+00_JPRB, 0.79344E+00_JPRB, 0.76319E+00_JPRB ,&
&  0.83496E+00_JPRB, 0.12269E+01_JPRB, 0.19439E+01_JPRB, 0.27522E+01_JPRB, 0.26962E+01_JPRB, 0.25488E+01_JPRB, 0.22919E+01_JPRB ,&
&  0.20549E+01_JPRB, 0.18705E+01_JPRB, 0.16655E+01_JPRB, 0.15242E+01_JPRB, 0.14696E+01_JPRB, 0.14160E+01_JPRB, 0.13593E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.12442E+01_JPRB, 0.12141E+01_JPRB, 0.12161E+01_JPRB, 0.12074E+01_JPRB, 0.11998E+01_JPRB, 0.11908E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.11495E+01_JPRB, 0.11184E+01_JPRB, 0.10901E+01_JPRB, 0.10671E+01_JPRB, 0.10661E+01_JPRB, 0.11455E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.13149E+01_JPRB, 0.13420E+01_JPRB, 0.13782E+01_JPRB, 0.13999E+01_JPRB, 0.13718E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.11894E+01_JPRB, 0.11392E+01_JPRB, 0.11390E+01_JPRB, 0.11192E+01_JPRB, 0.10934E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.10406E+01_JPRB, 0.10231E+01_JPRB, 0.10133E+01_JPRB, 0.10041E+01_JPRB, 0.98813E+00_JPRB, 0.96890E+00_JPRB ,&
&  0.96109E+00_JPRB, 0.95189E+00_JPRB, 0.95826E+00_JPRB, 0.98717E+00_JPRB, 0.10268E+01_JPRB, 0.10549E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.11614E+01_JPRB, 0.11534E+01_JPRB, 0.12646E+01_JPRB, 0.14303E+01_JPRB, 0.14445E+01_JPRB, 0.13770E+01_JPRB ,&
&  0.13146E+01_JPRB, 0.12589E+01_JPRB, 0.12214E+01_JPRB, 0.11536E+01_JPRB, 0.11307E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.10156E+01_JPRB, 0.10300E+01_JPRB, 0.10113E+01_JPRB, 0.97469E+00_JPRB, 0.95414E+00_JPRB, 0.94506E+00_JPRB, 0.94237E+00_JPRB ,&
&  0.93723E+00_JPRB, 0.92547E+00_JPRB, 0.92901E+00_JPRB, 0.93386E+00_JPRB, 0.92919E+00_JPRB, 0.91531E+00_JPRB, 0.90515E+00_JPRB ,&
&  0.90164E+00_JPRB, 0.90462E+00_JPRB, 0.90919E+00_JPRB, 0.89864E+00_JPRB, 0.87668E+00_JPRB, 0.86136E+00_JPRB, 0.84832E+00_JPRB ,&
&  0.84715E+00_JPRB, 0.84762E+00_JPRB, 0.84565E+00_JPRB, 0.84666E+00_JPRB, 0.82411E+00_JPRB, 0.79845E+00_JPRB, 0.77716E+00_JPRB ,&
&  0.77533E+00_JPRB, 0.80295E+00_JPRB, 0.11560E+01_JPRB, 0.16494E+01_JPRB, 0.18816E+01_JPRB, 0.18492E+01_JPRB, 0.16633E+01_JPRB ,&
&  0.15435E+01_JPRB, 0.14675E+01_JPRB, 0.14049E+01_JPRB, 0.13286E+01_JPRB, 0.12770E+01_JPRB, 0.12425E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.11894E+01_JPRB, 0.11603E+01_JPRB, 0.11368E+01_JPRB, 0.11069E+01_JPRB, 0.10822E+01_JPRB, 0.10715E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10380E+01_JPRB, 0.10265E+01_JPRB, 0.10142E+01_JPRB, 0.98029E+00_JPRB, 0.96468E+00_JPRB, 0.96416E+00_JPRB ,&
&  0.99358E+00_JPRB, 0.10233E+01_JPRB, 0.10394E+01_JPRB, 0.10953E+01_JPRB, 0.11392E+01_JPRB, 0.11774E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.11565E+01_JPRB, 0.11419E+01_JPRB, 0.11218E+01_JPRB, 0.11140E+01_JPRB, 0.10975E+01_JPRB, 0.10619E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.10069E+01_JPRB, 0.99463E+00_JPRB, 0.97355E+00_JPRB, 0.95911E+00_JPRB, 0.94391E+00_JPRB, 0.92192E+00_JPRB, 0.90278E+00_JPRB ,&
&  0.88942E+00_JPRB, 0.88632E+00_JPRB, 0.88945E+00_JPRB, 0.90681E+00_JPRB, 0.93473E+00_JPRB, 0.96958E+00_JPRB, 0.10054E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.10708E+01_JPRB, 0.11103E+01_JPRB, 0.12154E+01_JPRB, 0.12497E+01_JPRB, 0.12659E+01_JPRB, 0.12847E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.12131E+01_JPRB, 0.11567E+01_JPRB, 0.10727E+01_JPRB, 0.96988E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.96408E+00_JPRB, 0.96897E+00_JPRB, 0.95931E+00_JPRB, 0.94290E+00_JPRB, 0.93082E+00_JPRB, 0.92281E+00_JPRB, 0.92440E+00_JPRB ,&
&  0.92177E+00_JPRB, 0.91535E+00_JPRB, 0.91736E+00_JPRB, 0.91853E+00_JPRB, 0.91939E+00_JPRB, 0.91527E+00_JPRB, 0.91875E+00_JPRB ,&
&  0.91860E+00_JPRB, 0.91948E+00_JPRB, 0.92220E+00_JPRB, 0.91535E+00_JPRB, 0.89489E+00_JPRB, 0.88009E+00_JPRB, 0.86538E+00_JPRB ,&
&  0.86407E+00_JPRB, 0.86366E+00_JPRB, 0.86412E+00_JPRB, 0.85737E+00_JPRB, 0.83599E+00_JPRB, 0.81685E+00_JPRB, 0.80433E+00_JPRB ,&
&  0.80439E+00_JPRB, 0.69429E+00_JPRB, 0.84736E+00_JPRB, 0.12069E+01_JPRB, 0.13866E+01_JPRB, 0.12984E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.12302E+01_JPRB, 0.12104E+01_JPRB, 0.11713E+01_JPRB, 0.11055E+01_JPRB, 0.10754E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10169E+01_JPRB, 0.10068E+01_JPRB, 0.99509E+00_JPRB, 0.98825E+00_JPRB, 0.96985E+00_JPRB, 0.95120E+00_JPRB, 0.94597E+00_JPRB ,&
&  0.94859E+00_JPRB, 0.94332E+00_JPRB, 0.94229E+00_JPRB, 0.94017E+00_JPRB, 0.92728E+00_JPRB, 0.93500E+00_JPRB, 0.93739E+00_JPRB ,&
&  0.94213E+00_JPRB, 0.94437E+00_JPRB, 0.94556E+00_JPRB, 0.95215E+00_JPRB, 0.97445E+00_JPRB, 0.10235E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10450E+01_JPRB, 0.10503E+01_JPRB, 0.10527E+01_JPRB, 0.10428E+01_JPRB, 0.10180E+01_JPRB, 0.98882E+00_JPRB ,&
&  0.96953E+00_JPRB, 0.95202E+00_JPRB, 0.93555E+00_JPRB, 0.91638E+00_JPRB, 0.89539E+00_JPRB, 0.87609E+00_JPRB, 0.86097E+00_JPRB ,&
&  0.85123E+00_JPRB, 0.84743E+00_JPRB, 0.85453E+00_JPRB, 0.87301E+00_JPRB, 0.89799E+00_JPRB, 0.92765E+00_JPRB, 0.95974E+00_JPRB ,&
&  0.99716E+00_JPRB, 0.10387E+01_JPRB, 0.10766E+01_JPRB, 0.10979E+01_JPRB, 0.11845E+01_JPRB, 0.12080E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.11568E+01_JPRB, 0.10802E+01_JPRB, 0.85382E+00_JPRB, 0.93363E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.93588E+00_JPRB, 0.93475E+00_JPRB, 0.93233E+00_JPRB, 0.91685E+00_JPRB, 0.89867E+00_JPRB, 0.89080E+00_JPRB, 0.89591E+00_JPRB ,&
&  0.89015E+00_JPRB, 0.87585E+00_JPRB, 0.87987E+00_JPRB, 0.88242E+00_JPRB, 0.88852E+00_JPRB, 0.89079E+00_JPRB, 0.89352E+00_JPRB ,&
&  0.89651E+00_JPRB, 0.89959E+00_JPRB, 0.89728E+00_JPRB, 0.90517E+00_JPRB, 0.89743E+00_JPRB, 0.89154E+00_JPRB, 0.88830E+00_JPRB ,&
&  0.87952E+00_JPRB, 0.87056E+00_JPRB, 0.86288E+00_JPRB, 0.85208E+00_JPRB, 0.84327E+00_JPRB, 0.83252E+00_JPRB, 0.81578E+00_JPRB ,&
&  0.82641E+00_JPRB, 0.69043E+00_JPRB, 0.81826E+00_JPRB, 0.10481E+01_JPRB, 0.10479E+01_JPRB, 0.10142E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.10590E+01_JPRB, 0.10519E+01_JPRB, 0.10222E+01_JPRB, 0.95903E+00_JPRB, 0.92979E+00_JPRB, 0.91478E+00_JPRB ,&
&  0.89789E+00_JPRB, 0.88493E+00_JPRB, 0.87633E+00_JPRB, 0.87715E+00_JPRB, 0.87737E+00_JPRB, 0.87404E+00_JPRB, 0.86874E+00_JPRB ,&
&  0.86983E+00_JPRB, 0.87648E+00_JPRB, 0.87736E+00_JPRB, 0.87798E+00_JPRB, 0.87864E+00_JPRB, 0.86931E+00_JPRB, 0.86267E+00_JPRB ,&
&  0.85671E+00_JPRB, 0.85422E+00_JPRB, 0.85101E+00_JPRB, 0.85363E+00_JPRB, 0.86265E+00_JPRB, 0.89570E+00_JPRB, 0.92587E+00_JPRB ,&
&  0.93653E+00_JPRB, 0.93970E+00_JPRB, 0.94477E+00_JPRB, 0.95062E+00_JPRB, 0.95663E+00_JPRB, 0.95592E+00_JPRB, 0.94342E+00_JPRB ,&
&  0.92828E+00_JPRB, 0.90886E+00_JPRB, 0.89598E+00_JPRB, 0.88544E+00_JPRB, 0.87190E+00_JPRB, 0.86202E+00_JPRB, 0.85877E+00_JPRB ,&
&  0.84959E+00_JPRB, 0.83636E+00_JPRB, 0.83817E+00_JPRB, 0.85286E+00_JPRB, 0.86711E+00_JPRB, 0.87809E+00_JPRB, 0.89959E+00_JPRB ,&
&  0.93725E+00_JPRB, 0.97836E+00_JPRB, 0.10102E+01_JPRB, 0.10644E+01_JPRB, 0.10965E+01_JPRB, 0.11155E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10794E+01_JPRB, 0.79853E+00_JPRB, 0.90838E+00_JPRB, 0.93046E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.92934E+00_JPRB, 0.91703E+00_JPRB, 0.90212E+00_JPRB, 0.88263E+00_JPRB, 0.86402E+00_JPRB, 0.85253E+00_JPRB, 0.85689E+00_JPRB ,&
&  0.85461E+00_JPRB, 0.84169E+00_JPRB, 0.85129E+00_JPRB, 0.86387E+00_JPRB, 0.87373E+00_JPRB, 0.88270E+00_JPRB, 0.88525E+00_JPRB ,&
&  0.88813E+00_JPRB, 0.88454E+00_JPRB, 0.88743E+00_JPRB, 0.89946E+00_JPRB, 0.91811E+00_JPRB, 0.91977E+00_JPRB, 0.91664E+00_JPRB ,&
&  0.89409E+00_JPRB, 0.88257E+00_JPRB, 0.87190E+00_JPRB, 0.85882E+00_JPRB, 0.84664E+00_JPRB, 0.84143E+00_JPRB, 0.84261E+00_JPRB ,&
&  0.72871E+00_JPRB, 0.64450E+00_JPRB, 0.79221E+00_JPRB, 0.90548E+00_JPRB, 0.89271E+00_JPRB, 0.87639E+00_JPRB, 0.89421E+00_JPRB ,&
&  0.90123E+00_JPRB, 0.89664E+00_JPRB, 0.89826E+00_JPRB, 0.89631E+00_JPRB, 0.86780E+00_JPRB, 0.83227E+00_JPRB, 0.81599E+00_JPRB ,&
&  0.80698E+00_JPRB, 0.81271E+00_JPRB, 0.81563E+00_JPRB, 0.82380E+00_JPRB, 0.82415E+00_JPRB, 0.81659E+00_JPRB, 0.81357E+00_JPRB ,&
&  0.81200E+00_JPRB, 0.81948E+00_JPRB, 0.82564E+00_JPRB, 0.82875E+00_JPRB, 0.82789E+00_JPRB, 0.82565E+00_JPRB, 0.81933E+00_JPRB ,&
&  0.81363E+00_JPRB, 0.80903E+00_JPRB, 0.80387E+00_JPRB, 0.80573E+00_JPRB, 0.81239E+00_JPRB, 0.82848E+00_JPRB, 0.85350E+00_JPRB ,&
&  0.86909E+00_JPRB, 0.87533E+00_JPRB, 0.87945E+00_JPRB, 0.88106E+00_JPRB, 0.89219E+00_JPRB, 0.89896E+00_JPRB, 0.89089E+00_JPRB ,&
&  0.87868E+00_JPRB, 0.86863E+00_JPRB, 0.86812E+00_JPRB, 0.86618E+00_JPRB, 0.86600E+00_JPRB, 0.86316E+00_JPRB, 0.85480E+00_JPRB ,&
&  0.84414E+00_JPRB, 0.82914E+00_JPRB, 0.81803E+00_JPRB, 0.81790E+00_JPRB, 0.82925E+00_JPRB, 0.83899E+00_JPRB, 0.85959E+00_JPRB ,&
&  0.89498E+00_JPRB, 0.93768E+00_JPRB, 0.97275E+00_JPRB, 0.10195E+01_JPRB, 0.10503E+01_JPRB, 0.10670E+01_JPRB, 0.10651E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10076E+01_JPRB, 0.99127E+00_JPRB, 0.96212E+00_JPRB, 0.94410E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.88057E+00_JPRB, 0.87246E+00_JPRB, 0.85661E+00_JPRB, 0.82847E+00_JPRB, 0.79293E+00_JPRB, 0.79995E+00_JPRB, 0.81051E+00_JPRB ,&
&  0.80343E+00_JPRB, 0.79341E+00_JPRB, 0.80378E+00_JPRB, 0.82876E+00_JPRB, 0.85296E+00_JPRB, 0.86650E+00_JPRB, 0.86810E+00_JPRB ,&
&  0.86234E+00_JPRB, 0.86371E+00_JPRB, 0.87174E+00_JPRB, 0.88897E+00_JPRB, 0.88829E+00_JPRB, 0.87049E+00_JPRB, 0.85953E+00_JPRB ,&
&  0.85652E+00_JPRB, 0.85455E+00_JPRB, 0.85174E+00_JPRB, 0.83461E+00_JPRB, 0.82314E+00_JPRB, 0.81972E+00_JPRB, 0.81969E+00_JPRB ,&
&  0.81880E+00_JPRB, 0.68549E+00_JPRB, 0.81571E+00_JPRB, 0.82038E+00_JPRB, 0.82100E+00_JPRB, 0.80232E+00_JPRB, 0.79909E+00_JPRB ,&
&  0.80227E+00_JPRB, 0.81775E+00_JPRB, 0.81484E+00_JPRB, 0.79745E+00_JPRB, 0.77803E+00_JPRB, 0.75317E+00_JPRB, 0.73644E+00_JPRB ,&
&  0.72812E+00_JPRB, 0.73232E+00_JPRB, 0.74334E+00_JPRB, 0.75546E+00_JPRB, 0.76781E+00_JPRB, 0.76983E+00_JPRB, 0.77316E+00_JPRB ,&
&  0.77427E+00_JPRB, 0.77734E+00_JPRB, 0.77737E+00_JPRB, 0.77989E+00_JPRB, 0.77919E+00_JPRB, 0.77417E+00_JPRB, 0.76510E+00_JPRB ,&
&  0.75247E+00_JPRB, 0.74433E+00_JPRB, 0.74246E+00_JPRB, 0.74478E+00_JPRB, 0.75392E+00_JPRB, 0.77143E+00_JPRB, 0.79107E+00_JPRB ,&
&  0.81200E+00_JPRB, 0.82620E+00_JPRB, 0.83455E+00_JPRB, 0.83817E+00_JPRB, 0.84780E+00_JPRB, 0.85740E+00_JPRB, 0.85543E+00_JPRB ,&
&  0.84529E+00_JPRB, 0.83365E+00_JPRB, 0.82707E+00_JPRB, 0.81945E+00_JPRB, 0.81596E+00_JPRB, 0.81312E+00_JPRB, 0.80639E+00_JPRB ,&
&  0.79696E+00_JPRB, 0.79477E+00_JPRB, 0.79599E+00_JPRB, 0.78571E+00_JPRB, 0.78862E+00_JPRB, 0.80020E+00_JPRB, 0.82115E+00_JPRB ,&
&  0.84755E+00_JPRB, 0.88187E+00_JPRB, 0.91524E+00_JPRB, 0.94526E+00_JPRB, 0.96523E+00_JPRB, 0.97472E+00_JPRB, 0.97256E+00_JPRB ,&
&  0.95689E+00_JPRB, 0.93313E+00_JPRB, 0.91310E+00_JPRB, 0.90032E+00_JPRB, 0.89759E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.83159E+00_JPRB, 0.83120E+00_JPRB, 0.81995E+00_JPRB, 0.78018E+00_JPRB, 0.74969E+00_JPRB, 0.75797E+00_JPRB, 0.76579E+00_JPRB ,&
&  0.76512E+00_JPRB, 0.78025E+00_JPRB, 0.80532E+00_JPRB, 0.83058E+00_JPRB, 0.84743E+00_JPRB, 0.84701E+00_JPRB, 0.85053E+00_JPRB ,&
&  0.84737E+00_JPRB, 0.84485E+00_JPRB, 0.85458E+00_JPRB, 0.85951E+00_JPRB, 0.84888E+00_JPRB, 0.82805E+00_JPRB, 0.82595E+00_JPRB ,&
&  0.83218E+00_JPRB, 0.82779E+00_JPRB, 0.80943E+00_JPRB, 0.79686E+00_JPRB, 0.78660E+00_JPRB, 0.79208E+00_JPRB, 0.80016E+00_JPRB ,&
&  0.79685E+00_JPRB, 0.76413E+00_JPRB, 0.80480E+00_JPRB, 0.81145E+00_JPRB, 0.81003E+00_JPRB, 0.78820E+00_JPRB, 0.79041E+00_JPRB ,&
&  0.80013E+00_JPRB, 0.80617E+00_JPRB, 0.78898E+00_JPRB, 0.76899E+00_JPRB, 0.75203E+00_JPRB, 0.73690E+00_JPRB, 0.71111E+00_JPRB ,&
&  0.70230E+00_JPRB, 0.70025E+00_JPRB, 0.71449E+00_JPRB, 0.72963E+00_JPRB, 0.74715E+00_JPRB, 0.75460E+00_JPRB, 0.76507E+00_JPRB ,&
&  0.76644E+00_JPRB, 0.75956E+00_JPRB, 0.75256E+00_JPRB, 0.74607E+00_JPRB, 0.74324E+00_JPRB, 0.73618E+00_JPRB, 0.72599E+00_JPRB ,&
&  0.71710E+00_JPRB, 0.71451E+00_JPRB, 0.71844E+00_JPRB, 0.72743E+00_JPRB, 0.73983E+00_JPRB, 0.76068E+00_JPRB, 0.77824E+00_JPRB ,&
&  0.79951E+00_JPRB, 0.80764E+00_JPRB, 0.81470E+00_JPRB, 0.82881E+00_JPRB, 0.83782E+00_JPRB, 0.83718E+00_JPRB, 0.83416E+00_JPRB ,&
&  0.82918E+00_JPRB, 0.82061E+00_JPRB, 0.82232E+00_JPRB, 0.81845E+00_JPRB, 0.81622E+00_JPRB, 0.80542E+00_JPRB, 0.78848E+00_JPRB ,&
&  0.78164E+00_JPRB, 0.77678E+00_JPRB, 0.76824E+00_JPRB, 0.76538E+00_JPRB, 0.76317E+00_JPRB, 0.78183E+00_JPRB, 0.81462E+00_JPRB ,&
&  0.83348E+00_JPRB, 0.85877E+00_JPRB, 0.89374E+00_JPRB, 0.91678E+00_JPRB, 0.91815E+00_JPRB, 0.91923E+00_JPRB, 0.90937E+00_JPRB ,&
&  0.89183E+00_JPRB, 0.87910E+00_JPRB, 0.86558E+00_JPRB, 0.85935E+00_JPRB, 0.85361E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.78247E+00_JPRB, 0.80378E+00_JPRB, 0.77057E+00_JPRB, 0.75160E+00_JPRB, 0.72997E+00_JPRB, 0.72345E+00_JPRB, 0.71368E+00_JPRB ,&
&  0.72472E+00_JPRB, 0.74228E+00_JPRB, 0.76643E+00_JPRB, 0.77452E+00_JPRB, 0.78135E+00_JPRB, 0.78398E+00_JPRB, 0.80042E+00_JPRB ,&
&  0.79521E+00_JPRB, 0.78664E+00_JPRB, 0.78957E+00_JPRB, 0.77910E+00_JPRB, 0.77033E+00_JPRB, 0.77552E+00_JPRB, 0.78498E+00_JPRB ,&
&  0.79537E+00_JPRB, 0.78658E+00_JPRB, 0.76900E+00_JPRB, 0.74223E+00_JPRB, 0.72946E+00_JPRB, 0.72667E+00_JPRB, 0.73477E+00_JPRB ,&
&  0.74187E+00_JPRB, 0.77718E+00_JPRB, 0.71025E+00_JPRB, 0.76868E+00_JPRB, 0.77183E+00_JPRB, 0.76393E+00_JPRB, 0.76020E+00_JPRB ,&
&  0.76377E+00_JPRB, 0.75333E+00_JPRB, 0.74627E+00_JPRB, 0.74000E+00_JPRB, 0.71793E+00_JPRB, 0.70037E+00_JPRB, 0.68229E+00_JPRB ,&
&  0.66658E+00_JPRB, 0.65690E+00_JPRB, 0.66291E+00_JPRB, 0.68144E+00_JPRB, 0.70017E+00_JPRB, 0.71054E+00_JPRB, 0.71675E+00_JPRB ,&
&  0.71999E+00_JPRB, 0.71242E+00_JPRB, 0.70149E+00_JPRB, 0.70021E+00_JPRB, 0.70505E+00_JPRB, 0.70861E+00_JPRB, 0.71019E+00_JPRB ,&
&  0.70903E+00_JPRB, 0.68559E+00_JPRB, 0.67171E+00_JPRB, 0.68770E+00_JPRB, 0.70615E+00_JPRB, 0.72724E+00_JPRB, 0.75475E+00_JPRB ,&
&  0.77626E+00_JPRB, 0.78506E+00_JPRB, 0.78538E+00_JPRB, 0.79263E+00_JPRB, 0.79978E+00_JPRB, 0.80898E+00_JPRB, 0.81575E+00_JPRB ,&
&  0.80386E+00_JPRB, 0.80005E+00_JPRB, 0.79988E+00_JPRB, 0.79865E+00_JPRB, 0.78813E+00_JPRB, 0.76368E+00_JPRB, 0.73032E+00_JPRB ,&
&  0.71467E+00_JPRB, 0.69916E+00_JPRB, 0.68793E+00_JPRB, 0.68411E+00_JPRB, 0.68533E+00_JPRB, 0.71212E+00_JPRB, 0.75084E+00_JPRB ,&
&  0.78962E+00_JPRB, 0.82143E+00_JPRB, 0.84891E+00_JPRB, 0.87386E+00_JPRB, 0.87327E+00_JPRB, 0.85284E+00_JPRB, 0.84453E+00_JPRB ,&
&  0.83045E+00_JPRB, 0.81444E+00_JPRB, 0.80252E+00_JPRB, 0.79266E+00_JPRB, 0.78207E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.78141E+00_JPRB, 0.78254E+00_JPRB, 0.76705E+00_JPRB, 0.75563E+00_JPRB, 0.73779E+00_JPRB, 0.71493E+00_JPRB, 0.69525E+00_JPRB ,&
&  0.69723E+00_JPRB, 0.71332E+00_JPRB, 0.72871E+00_JPRB, 0.73969E+00_JPRB, 0.75524E+00_JPRB, 0.75834E+00_JPRB, 0.76444E+00_JPRB ,&
&  0.75964E+00_JPRB, 0.72225E+00_JPRB, 0.71622E+00_JPRB, 0.72365E+00_JPRB, 0.72542E+00_JPRB, 0.73168E+00_JPRB, 0.74294E+00_JPRB ,&
&  0.75319E+00_JPRB, 0.74862E+00_JPRB, 0.74026E+00_JPRB, 0.71033E+00_JPRB, 0.70341E+00_JPRB, 0.70828E+00_JPRB, 0.71353E+00_JPRB ,&
&  0.72445E+00_JPRB, 0.74493E+00_JPRB, 0.74711E+00_JPRB, 0.75747E+00_JPRB, 0.76383E+00_JPRB, 0.76543E+00_JPRB, 0.76071E+00_JPRB ,&
&  0.73415E+00_JPRB, 0.70807E+00_JPRB, 0.69791E+00_JPRB, 0.68183E+00_JPRB, 0.65883E+00_JPRB, 0.64284E+00_JPRB, 0.63588E+00_JPRB ,&
&  0.63301E+00_JPRB, 0.63899E+00_JPRB, 0.64562E+00_JPRB, 0.65984E+00_JPRB, 0.66519E+00_JPRB, 0.66957E+00_JPRB, 0.67535E+00_JPRB ,&
&  0.67548E+00_JPRB, 0.67192E+00_JPRB, 0.67199E+00_JPRB, 0.68145E+00_JPRB, 0.68637E+00_JPRB, 0.68283E+00_JPRB, 0.67725E+00_JPRB ,&
&  0.66282E+00_JPRB, 0.64809E+00_JPRB, 0.65468E+00_JPRB, 0.67008E+00_JPRB, 0.67810E+00_JPRB, 0.70508E+00_JPRB, 0.73948E+00_JPRB ,&
&  0.76866E+00_JPRB, 0.78755E+00_JPRB, 0.78765E+00_JPRB, 0.78375E+00_JPRB, 0.78209E+00_JPRB, 0.78950E+00_JPRB, 0.80144E+00_JPRB ,&
&  0.78513E+00_JPRB, 0.77550E+00_JPRB, 0.77284E+00_JPRB, 0.75267E+00_JPRB, 0.71534E+00_JPRB, 0.68312E+00_JPRB, 0.65940E+00_JPRB ,&
&  0.64051E+00_JPRB, 0.62999E+00_JPRB, 0.63382E+00_JPRB, 0.63670E+00_JPRB, 0.64885E+00_JPRB, 0.68461E+00_JPRB, 0.72249E+00_JPRB ,&
&  0.76398E+00_JPRB, 0.78585E+00_JPRB, 0.79345E+00_JPRB, 0.80259E+00_JPRB, 0.80961E+00_JPRB, 0.79973E+00_JPRB, 0.79130E+00_JPRB ,&
&  0.78899E+00_JPRB, 0.78038E+00_JPRB, 0.77088E+00_JPRB, 0.76107E+00_JPRB, 0.76844E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.77396E+00_JPRB, 0.76595E+00_JPRB, 0.75125E+00_JPRB, 0.72337E+00_JPRB, 0.70498E+00_JPRB, 0.67780E+00_JPRB, 0.66273E+00_JPRB ,&
&  0.66544E+00_JPRB, 0.66740E+00_JPRB, 0.67302E+00_JPRB, 0.68239E+00_JPRB, 0.68034E+00_JPRB, 0.67563E+00_JPRB, 0.67066E+00_JPRB ,&
&  0.67319E+00_JPRB, 0.67341E+00_JPRB, 0.68187E+00_JPRB, 0.69725E+00_JPRB, 0.71340E+00_JPRB, 0.72280E+00_JPRB, 0.73302E+00_JPRB ,&
&  0.74089E+00_JPRB, 0.73483E+00_JPRB, 0.73018E+00_JPRB, 0.69908E+00_JPRB, 0.68095E+00_JPRB, 0.68593E+00_JPRB, 0.69362E+00_JPRB ,&
&  0.70415E+00_JPRB, 0.71596E+00_JPRB, 0.72484E+00_JPRB, 0.73257E+00_JPRB, 0.71878E+00_JPRB, 0.71422E+00_JPRB, 0.70280E+00_JPRB ,&
&  0.65909E+00_JPRB, 0.61657E+00_JPRB, 0.59372E+00_JPRB, 0.58551E+00_JPRB, 0.57883E+00_JPRB, 0.58437E+00_JPRB, 0.59260E+00_JPRB ,&
&  0.60697E+00_JPRB, 0.61993E+00_JPRB, 0.62316E+00_JPRB, 0.62608E+00_JPRB, 0.62767E+00_JPRB, 0.62728E+00_JPRB, 0.62865E+00_JPRB ,&
&  0.62700E+00_JPRB, 0.62654E+00_JPRB, 0.63397E+00_JPRB, 0.63406E+00_JPRB, 0.62431E+00_JPRB, 0.62044E+00_JPRB, 0.61701E+00_JPRB ,&
&  0.61339E+00_JPRB, 0.61012E+00_JPRB, 0.60654E+00_JPRB, 0.61698E+00_JPRB, 0.64352E+00_JPRB, 0.67373E+00_JPRB, 0.69941E+00_JPRB ,&
&  0.71435E+00_JPRB, 0.72670E+00_JPRB, 0.73009E+00_JPRB, 0.73201E+00_JPRB, 0.73158E+00_JPRB, 0.72278E+00_JPRB, 0.71923E+00_JPRB ,&
&  0.71125E+00_JPRB, 0.69161E+00_JPRB, 0.66541E+00_JPRB, 0.62507E+00_JPRB, 0.58605E+00_JPRB, 0.56375E+00_JPRB, 0.56328E+00_JPRB ,&
&  0.57545E+00_JPRB, 0.58799E+00_JPRB, 0.59423E+00_JPRB, 0.60201E+00_JPRB, 0.61632E+00_JPRB, 0.63355E+00_JPRB, 0.65202E+00_JPRB ,&
&  0.66376E+00_JPRB, 0.66420E+00_JPRB, 0.66244E+00_JPRB, 0.67143E+00_JPRB, 0.68344E+00_JPRB, 0.70242E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.74082E+00_JPRB, 0.74089E+00_JPRB, 0.74163E+00_JPRB, 0.74328E+00_JPRB, 0.76339E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.76599E+00_JPRB, 0.76357E+00_JPRB, 0.73363E+00_JPRB, 0.70099E+00_JPRB, 0.68208E+00_JPRB, 0.66167E+00_JPRB, 0.64547E+00_JPRB ,&
&  0.64422E+00_JPRB, 0.64160E+00_JPRB, 0.63938E+00_JPRB, 0.63651E+00_JPRB, 0.63463E+00_JPRB, 0.63162E+00_JPRB, 0.61961E+00_JPRB ,&
&  0.61502E+00_JPRB, 0.62136E+00_JPRB, 0.64361E+00_JPRB, 0.66850E+00_JPRB, 0.68844E+00_JPRB, 0.71464E+00_JPRB, 0.72610E+00_JPRB ,&
&  0.73832E+00_JPRB, 0.75103E+00_JPRB, 0.75295E+00_JPRB, 0.71927E+00_JPRB, 0.69157E+00_JPRB, 0.69249E+00_JPRB, 0.70391E+00_JPRB ,&
&  0.70469E+00_JPRB, 0.70849E+00_JPRB, 0.71119E+00_JPRB, 0.70907E+00_JPRB, 0.68895E+00_JPRB, 0.65091E+00_JPRB, 0.62035E+00_JPRB ,&
&  0.59458E+00_JPRB, 0.57447E+00_JPRB, 0.56756E+00_JPRB, 0.55469E+00_JPRB, 0.55164E+00_JPRB, 0.56152E+00_JPRB, 0.57580E+00_JPRB ,&
&  0.59085E+00_JPRB, 0.59804E+00_JPRB, 0.60520E+00_JPRB, 0.60131E+00_JPRB, 0.59712E+00_JPRB, 0.60047E+00_JPRB, 0.60094E+00_JPRB ,&
&  0.60121E+00_JPRB, 0.59252E+00_JPRB, 0.58936E+00_JPRB, 0.58174E+00_JPRB, 0.57715E+00_JPRB, 0.57643E+00_JPRB, 0.57821E+00_JPRB ,&
&  0.57864E+00_JPRB, 0.58374E+00_JPRB, 0.58527E+00_JPRB, 0.58776E+00_JPRB, 0.59756E+00_JPRB, 0.61458E+00_JPRB, 0.63147E+00_JPRB ,&
&  0.64402E+00_JPRB, 0.65379E+00_JPRB, 0.66133E+00_JPRB, 0.66722E+00_JPRB, 0.66719E+00_JPRB, 0.66235E+00_JPRB, 0.64824E+00_JPRB ,&
&  0.61752E+00_JPRB, 0.58472E+00_JPRB, 0.55981E+00_JPRB, 0.54001E+00_JPRB, 0.53672E+00_JPRB, 0.54828E+00_JPRB, 0.56664E+00_JPRB ,&
&  0.58042E+00_JPRB, 0.58410E+00_JPRB, 0.57754E+00_JPRB, 0.57507E+00_JPRB, 0.57909E+00_JPRB, 0.58795E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.59373E+00_JPRB, 0.59067E+00_JPRB, 0.59512E+00_JPRB, 0.60329E+00_JPRB, 0.61869E+00_JPRB, 0.63780E+00_JPRB, 0.65817E+00_JPRB ,&
&  0.67826E+00_JPRB, 0.69375E+00_JPRB, 0.71348E+00_JPRB, 0.73332E+00_JPRB, 0.75841E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.74606E+00_JPRB, 0.71723E+00_JPRB, 0.68702E+00_JPRB, 0.66158E+00_JPRB, 0.64966E+00_JPRB, 0.63237E+00_JPRB, 0.61793E+00_JPRB ,&
&  0.61721E+00_JPRB, 0.62505E+00_JPRB, 0.62031E+00_JPRB, 0.61422E+00_JPRB, 0.60503E+00_JPRB, 0.59763E+00_JPRB, 0.58885E+00_JPRB ,&
&  0.58492E+00_JPRB, 0.59612E+00_JPRB, 0.62155E+00_JPRB, 0.63403E+00_JPRB, 0.63999E+00_JPRB, 0.67033E+00_JPRB, 0.70181E+00_JPRB ,&
&  0.71136E+00_JPRB, 0.72552E+00_JPRB, 0.72938E+00_JPRB, 0.70333E+00_JPRB, 0.68624E+00_JPRB, 0.67961E+00_JPRB, 0.68348E+00_JPRB ,&
&  0.67918E+00_JPRB, 0.68377E+00_JPRB, 0.67526E+00_JPRB, 0.60677E+00_JPRB, 0.60606E+00_JPRB, 0.57580E+00_JPRB, 0.56513E+00_JPRB ,&
&  0.55940E+00_JPRB, 0.56409E+00_JPRB, 0.55928E+00_JPRB, 0.54881E+00_JPRB, 0.54446E+00_JPRB, 0.54771E+00_JPRB, 0.54979E+00_JPRB ,&
&  0.54633E+00_JPRB, 0.53624E+00_JPRB, 0.54090E+00_JPRB, 0.54730E+00_JPRB, 0.55099E+00_JPRB, 0.55840E+00_JPRB, 0.56935E+00_JPRB ,&
&  0.55957E+00_JPRB, 0.55012E+00_JPRB, 0.54131E+00_JPRB, 0.54161E+00_JPRB, 0.54796E+00_JPRB, 0.56146E+00_JPRB, 0.56525E+00_JPRB ,&
&  0.56253E+00_JPRB, 0.56107E+00_JPRB, 0.56225E+00_JPRB, 0.56897E+00_JPRB, 0.58374E+00_JPRB, 0.59819E+00_JPRB, 0.59931E+00_JPRB ,&
&  0.60050E+00_JPRB, 0.60518E+00_JPRB, 0.61061E+00_JPRB, 0.61201E+00_JPRB, 0.60758E+00_JPRB, 0.59227E+00_JPRB, 0.57474E+00_JPRB ,&
&  0.55817E+00_JPRB, 0.54050E+00_JPRB, 0.52610E+00_JPRB, 0.52211E+00_JPRB, 0.53624E+00_JPRB, 0.56641E+00_JPRB, 0.58352E+00_JPRB ,&
&  0.58792E+00_JPRB, 0.59011E+00_JPRB, 0.59315E+00_JPRB, 0.59360E+00_JPRB, 0.59503E+00_JPRB, 0.59244E+00_JPRB, 0.58508E+00_JPRB ,&
&  0.57320E+00_JPRB, 0.56222E+00_JPRB, 0.56056E+00_JPRB, 0.56655E+00_JPRB, 0.57340E+00_JPRB, 0.58727E+00_JPRB, 0.60525E+00_JPRB ,&
&  0.62823E+00_JPRB, 0.66696E+00_JPRB, 0.69936E+00_JPRB, 0.72420E+00_JPRB, 0.74139E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.72233E+00_JPRB, 0.69497E+00_JPRB, 0.67656E+00_JPRB, 0.66242E+00_JPRB, 0.64395E+00_JPRB, 0.62992E+00_JPRB, 0.62431E+00_JPRB ,&
&  0.63253E+00_JPRB, 0.63117E+00_JPRB, 0.62158E+00_JPRB, 0.61370E+00_JPRB, 0.59815E+00_JPRB, 0.58423E+00_JPRB, 0.57866E+00_JPRB ,&
&  0.58716E+00_JPRB, 0.60323E+00_JPRB, 0.62297E+00_JPRB, 0.63067E+00_JPRB, 0.62964E+00_JPRB, 0.63801E+00_JPRB, 0.65732E+00_JPRB ,&
&  0.68598E+00_JPRB, 0.70597E+00_JPRB, 0.71261E+00_JPRB, 0.69875E+00_JPRB, 0.68064E+00_JPRB, 0.67225E+00_JPRB, 0.65813E+00_JPRB ,&
&  0.64161E+00_JPRB, 0.64453E+00_JPRB, 0.57763E+00_JPRB, 0.57001E+00_JPRB, 0.54718E+00_JPRB, 0.53261E+00_JPRB, 0.52875E+00_JPRB ,&
&  0.54184E+00_JPRB, 0.54536E+00_JPRB, 0.53628E+00_JPRB, 0.52334E+00_JPRB, 0.51477E+00_JPRB, 0.50366E+00_JPRB, 0.49364E+00_JPRB ,&
&  0.48714E+00_JPRB, 0.49005E+00_JPRB, 0.49911E+00_JPRB, 0.50858E+00_JPRB, 0.51959E+00_JPRB, 0.53032E+00_JPRB, 0.52712E+00_JPRB ,&
&  0.51551E+00_JPRB, 0.51360E+00_JPRB, 0.52104E+00_JPRB, 0.53617E+00_JPRB, 0.55250E+00_JPRB, 0.55980E+00_JPRB, 0.56255E+00_JPRB ,&
&  0.55339E+00_JPRB, 0.53578E+00_JPRB, 0.52096E+00_JPRB, 0.50885E+00_JPRB, 0.48150E+00_JPRB, 0.41907E+00_JPRB, 0.39679E+00_JPRB ,&
&  0.40841E+00_JPRB, 0.46624E+00_JPRB, 0.53433E+00_JPRB, 0.52666E+00_JPRB, 0.52393E+00_JPRB, 0.50964E+00_JPRB, 0.47234E+00_JPRB ,&
&  0.42787E+00_JPRB, 0.36715E+00_JPRB, 0.34732E+00_JPRB, 0.34440E+00_JPRB, 0.34428E+00_JPRB, 0.35147E+00_JPRB, 0.36441E+00_JPRB ,&
&  0.38129E+00_JPRB, 0.40689E+00_JPRB, 0.42356E+00_JPRB, 0.42966E+00_JPRB, 0.42756E+00_JPRB, 0.41140E+00_JPRB, 0.39716E+00_JPRB ,&
&  0.38820E+00_JPRB, 0.41404E+00_JPRB, 0.47413E+00_JPRB, 0.55644E+00_JPRB, 0.55775E+00_JPRB, 0.58468E+00_JPRB, 0.61083E+00_JPRB ,&
&  0.63563E+00_JPRB, 0.67634E+00_JPRB, 0.70234E+00_JPRB, 0.70757E+00_JPRB, 0.72519E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.68203E+00_JPRB, 0.66874E+00_JPRB, 0.65954E+00_JPRB, 0.65310E+00_JPRB, 0.64579E+00_JPRB, 0.64416E+00_JPRB, 0.64330E+00_JPRB ,&
&  0.63751E+00_JPRB, 0.62933E+00_JPRB, 0.63073E+00_JPRB, 0.61414E+00_JPRB, 0.59007E+00_JPRB, 0.57398E+00_JPRB, 0.57194E+00_JPRB ,&
&  0.57591E+00_JPRB, 0.58318E+00_JPRB, 0.59220E+00_JPRB, 0.60043E+00_JPRB, 0.60515E+00_JPRB, 0.61677E+00_JPRB, 0.63750E+00_JPRB ,&
&  0.66159E+00_JPRB, 0.67508E+00_JPRB, 0.67900E+00_JPRB, 0.67058E+00_JPRB, 0.65798E+00_JPRB, 0.64514E+00_JPRB, 0.63391E+00_JPRB ,&
&  0.62349E+00_JPRB, 0.52156E+00_JPRB, 0.45478E+00_JPRB, 0.46222E+00_JPRB, 0.52895E+00_JPRB, 0.51271E+00_JPRB, 0.51177E+00_JPRB ,&
&  0.52441E+00_JPRB, 0.52375E+00_JPRB, 0.51004E+00_JPRB, 0.49263E+00_JPRB, 0.47649E+00_JPRB, 0.46487E+00_JPRB, 0.45683E+00_JPRB ,&
&  0.45344E+00_JPRB, 0.46059E+00_JPRB, 0.46850E+00_JPRB, 0.48067E+00_JPRB, 0.48346E+00_JPRB, 0.47041E+00_JPRB, 0.45461E+00_JPRB ,&
&  0.43952E+00_JPRB, 0.43402E+00_JPRB, 0.43662E+00_JPRB, 0.44381E+00_JPRB, 0.45321E+00_JPRB, 0.46047E+00_JPRB, 0.45777E+00_JPRB ,&
&  0.42962E+00_JPRB, 0.40624E+00_JPRB, 0.37799E+00_JPRB, 0.35134E+00_JPRB, 0.32836E+00_JPRB, 0.31305E+00_JPRB, 0.30490E+00_JPRB ,&
&  0.30306E+00_JPRB, 0.31121E+00_JPRB, 0.33302E+00_JPRB, 0.38178E+00_JPRB, 0.44511E+00_JPRB, 0.36596E+00_JPRB, 0.32754E+00_JPRB ,&
&  0.30673E+00_JPRB, 0.29363E+00_JPRB, 0.28865E+00_JPRB, 0.28979E+00_JPRB, 0.29450E+00_JPRB, 0.30111E+00_JPRB, 0.30632E+00_JPRB ,&
&  0.31055E+00_JPRB, 0.31393E+00_JPRB, 0.31524E+00_JPRB, 0.31573E+00_JPRB, 0.31703E+00_JPRB, 0.31816E+00_JPRB, 0.31700E+00_JPRB ,&
&  0.31584E+00_JPRB, 0.32297E+00_JPRB, 0.34171E+00_JPRB, 0.37093E+00_JPRB, 0.40444E+00_JPRB, 0.44120E+00_JPRB, 0.49415E+00_JPRB ,&
&  0.55979E+00_JPRB, 0.63286E+00_JPRB, 0.65221E+00_JPRB, 0.66625E+00_JPRB, 0.67661E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.62240E+00_JPRB, 0.62537E+00_JPRB, 0.62872E+00_JPRB, 0.63041E+00_JPRB, 0.62464E+00_JPRB, 0.61707E+00_JPRB, 0.61246E+00_JPRB ,&
&  0.61783E+00_JPRB, 0.61833E+00_JPRB, 0.59550E+00_JPRB, 0.57790E+00_JPRB, 0.56795E+00_JPRB, 0.56450E+00_JPRB, 0.56182E+00_JPRB ,&
&  0.55574E+00_JPRB, 0.55361E+00_JPRB, 0.55464E+00_JPRB, 0.56557E+00_JPRB, 0.57460E+00_JPRB, 0.58243E+00_JPRB, 0.58962E+00_JPRB ,&
&  0.58442E+00_JPRB, 0.59629E+00_JPRB, 0.61755E+00_JPRB, 0.61629E+00_JPRB, 0.61317E+00_JPRB, 0.60561E+00_JPRB, 0.59077E+00_JPRB ,&
&  0.54156E+00_JPRB, 0.47708E+00_JPRB, 0.41729E+00_JPRB, 0.42999E+00_JPRB, 0.50510E+00_JPRB, 0.51268E+00_JPRB, 0.49877E+00_JPRB ,&
&  0.50958E+00_JPRB, 0.50460E+00_JPRB, 0.48646E+00_JPRB, 0.47280E+00_JPRB, 0.45711E+00_JPRB, 0.44695E+00_JPRB, 0.43962E+00_JPRB ,&
&  0.43661E+00_JPRB, 0.43605E+00_JPRB, 0.42028E+00_JPRB, 0.37062E+00_JPRB, 0.34292E+00_JPRB, 0.32561E+00_JPRB, 0.30759E+00_JPRB ,&
&  0.29319E+00_JPRB, 0.28473E+00_JPRB, 0.28379E+00_JPRB, 0.28418E+00_JPRB, 0.28955E+00_JPRB, 0.29247E+00_JPRB, 0.29508E+00_JPRB ,&
&  0.29537E+00_JPRB, 0.29431E+00_JPRB, 0.29035E+00_JPRB, 0.28590E+00_JPRB, 0.28275E+00_JPRB, 0.28245E+00_JPRB, 0.28209E+00_JPRB ,&
&  0.28482E+00_JPRB, 0.28857E+00_JPRB, 0.30532E+00_JPRB, 0.32287E+00_JPRB, 0.31979E+00_JPRB, 0.30473E+00_JPRB, 0.29074E+00_JPRB ,&
&  0.28038E+00_JPRB, 0.27513E+00_JPRB, 0.27178E+00_JPRB, 0.27126E+00_JPRB, 0.27450E+00_JPRB, 0.27916E+00_JPRB, 0.28028E+00_JPRB ,&
&  0.27995E+00_JPRB, 0.28053E+00_JPRB, 0.28051E+00_JPRB, 0.28086E+00_JPRB, 0.28145E+00_JPRB, 0.28420E+00_JPRB, 0.28939E+00_JPRB ,&
&  0.29554E+00_JPRB, 0.30662E+00_JPRB, 0.31898E+00_JPRB, 0.33542E+00_JPRB, 0.34747E+00_JPRB, 0.35667E+00_JPRB, 0.36955E+00_JPRB ,&
&  0.38797E+00_JPRB, 0.42772E+00_JPRB, 0.49683E+00_JPRB, 0.62079E+00_JPRB, 0.61418E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.58955E+00_JPRB, 0.60035E+00_JPRB, 0.61216E+00_JPRB, 0.60354E+00_JPRB, 0.59003E+00_JPRB, 0.58241E+00_JPRB, 0.58241E+00_JPRB ,&
&  0.58252E+00_JPRB, 0.58002E+00_JPRB, 0.56833E+00_JPRB, 0.55504E+00_JPRB, 0.53693E+00_JPRB, 0.48937E+00_JPRB, 0.47217E+00_JPRB ,&
&  0.45487E+00_JPRB, 0.45761E+00_JPRB, 0.46173E+00_JPRB, 0.46784E+00_JPRB, 0.48125E+00_JPRB, 0.49172E+00_JPRB, 0.49316E+00_JPRB ,&
&  0.48262E+00_JPRB, 0.46402E+00_JPRB, 0.44599E+00_JPRB, 0.43212E+00_JPRB, 0.43161E+00_JPRB, 0.43124E+00_JPRB, 0.42382E+00_JPRB ,&
&  0.41021E+00_JPRB, 0.40434E+00_JPRB, 0.40415E+00_JPRB, 0.42403E+00_JPRB, 0.47518E+00_JPRB, 0.49394E+00_JPRB, 0.46985E+00_JPRB ,&
&  0.47424E+00_JPRB, 0.46882E+00_JPRB, 0.46454E+00_JPRB, 0.46545E+00_JPRB, 0.45484E+00_JPRB, 0.43358E+00_JPRB, 0.39962E+00_JPRB ,&
&  0.35739E+00_JPRB, 0.33554E+00_JPRB, 0.31460E+00_JPRB, 0.29421E+00_JPRB, 0.27785E+00_JPRB, 0.26847E+00_JPRB, 0.26062E+00_JPRB ,&
&  0.25557E+00_JPRB, 0.25122E+00_JPRB, 0.24845E+00_JPRB, 0.24865E+00_JPRB, 0.25062E+00_JPRB, 0.25163E+00_JPRB, 0.25281E+00_JPRB ,&
&  0.25456E+00_JPRB, 0.25563E+00_JPRB, 0.25628E+00_JPRB, 0.25784E+00_JPRB, 0.25953E+00_JPRB, 0.26248E+00_JPRB, 0.26637E+00_JPRB ,&
&  0.27119E+00_JPRB, 0.27489E+00_JPRB, 0.27836E+00_JPRB, 0.28492E+00_JPRB, 0.28554E+00_JPRB, 0.28417E+00_JPRB, 0.27604E+00_JPRB ,&
&  0.27004E+00_JPRB, 0.26742E+00_JPRB, 0.26468E+00_JPRB, 0.26395E+00_JPRB, 0.26518E+00_JPRB, 0.26728E+00_JPRB, 0.26942E+00_JPRB ,&
&  0.27089E+00_JPRB, 0.27087E+00_JPRB, 0.26992E+00_JPRB, 0.26971E+00_JPRB, 0.27024E+00_JPRB, 0.27035E+00_JPRB, 0.27418E+00_JPRB ,&
&  0.28276E+00_JPRB, 0.29528E+00_JPRB, 0.30903E+00_JPRB, 0.32758E+00_JPRB, 0.34523E+00_JPRB, 0.36212E+00_JPRB, 0.39020E+00_JPRB ,&
&  0.42901E+00_JPRB, 0.53495E+00_JPRB, 0.60302E+00_JPRB, 0.54460E+00_JPRB, 0.56944E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.56268E+00_JPRB, 0.56999E+00_JPRB, 0.58048E+00_JPRB, 0.58480E+00_JPRB, 0.59264E+00_JPRB, 0.58088E+00_JPRB, 0.57361E+00_JPRB ,&
&  0.54342E+00_JPRB, 0.52216E+00_JPRB, 0.49157E+00_JPRB, 0.45385E+00_JPRB, 0.42847E+00_JPRB, 0.40907E+00_JPRB, 0.40199E+00_JPRB ,&
&  0.39249E+00_JPRB, 0.38675E+00_JPRB, 0.38042E+00_JPRB, 0.37769E+00_JPRB, 0.37542E+00_JPRB, 0.37595E+00_JPRB, 0.37514E+00_JPRB ,&
&  0.37114E+00_JPRB, 0.36537E+00_JPRB, 0.35851E+00_JPRB, 0.36091E+00_JPRB, 0.37359E+00_JPRB, 0.38817E+00_JPRB, 0.40556E+00_JPRB ,&
&  0.42582E+00_JPRB, 0.43622E+00_JPRB, 0.44653E+00_JPRB, 0.45064E+00_JPRB, 0.43523E+00_JPRB, 0.43796E+00_JPRB, 0.44155E+00_JPRB ,&
&  0.44940E+00_JPRB, 0.45511E+00_JPRB, 0.45160E+00_JPRB, 0.44616E+00_JPRB, 0.39459E+00_JPRB, 0.36221E+00_JPRB, 0.33358E+00_JPRB ,&
&  0.31234E+00_JPRB, 0.29890E+00_JPRB, 0.28700E+00_JPRB, 0.27679E+00_JPRB, 0.26576E+00_JPRB, 0.25884E+00_JPRB, 0.25367E+00_JPRB ,&
&  0.24961E+00_JPRB, 0.24801E+00_JPRB, 0.24657E+00_JPRB, 0.24625E+00_JPRB, 0.24690E+00_JPRB, 0.24635E+00_JPRB, 0.24642E+00_JPRB ,&
&  0.24640E+00_JPRB, 0.24589E+00_JPRB, 0.24624E+00_JPRB, 0.24780E+00_JPRB, 0.25003E+00_JPRB, 0.25355E+00_JPRB, 0.25451E+00_JPRB ,&
&  0.25558E+00_JPRB, 0.25643E+00_JPRB, 0.25655E+00_JPRB, 0.25874E+00_JPRB, 0.25995E+00_JPRB, 0.26011E+00_JPRB, 0.25852E+00_JPRB ,&
&  0.25680E+00_JPRB, 0.25546E+00_JPRB, 0.25553E+00_JPRB, 0.25618E+00_JPRB, 0.25791E+00_JPRB, 0.26043E+00_JPRB, 0.26089E+00_JPRB ,&
&  0.26175E+00_JPRB, 0.26116E+00_JPRB, 0.26056E+00_JPRB, 0.26282E+00_JPRB, 0.26541E+00_JPRB, 0.26818E+00_JPRB, 0.27181E+00_JPRB ,&
&  0.27346E+00_JPRB, 0.28103E+00_JPRB, 0.29497E+00_JPRB, 0.31665E+00_JPRB, 0.35191E+00_JPRB, 0.36893E+00_JPRB, 0.40888E+00_JPRB ,&
&  0.46663E+00_JPRB, 0.60456E+00_JPRB, 0.66851E+00_JPRB, 0.53056E+00_JPRB, 0.54710E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.50458E+00_JPRB, 0.51679E+00_JPRB, 0.53331E+00_JPRB, 0.54356E+00_JPRB, 0.55708E+00_JPRB, 0.55939E+00_JPRB, 0.55279E+00_JPRB ,&
&  0.53474E+00_JPRB, 0.50663E+00_JPRB, 0.49034E+00_JPRB, 0.46924E+00_JPRB, 0.44806E+00_JPRB, 0.42637E+00_JPRB, 0.41668E+00_JPRB ,&
&  0.40503E+00_JPRB, 0.39139E+00_JPRB, 0.37756E+00_JPRB, 0.36545E+00_JPRB, 0.35371E+00_JPRB, 0.34299E+00_JPRB, 0.33686E+00_JPRB ,&
&  0.33060E+00_JPRB, 0.33022E+00_JPRB, 0.32958E+00_JPRB, 0.33353E+00_JPRB, 0.34489E+00_JPRB, 0.36341E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.42772E+00_JPRB, 0.43199E+00_JPRB, 0.41748E+00_JPRB, 0.41131E+00_JPRB, 0.40382E+00_JPRB, 0.39760E+00_JPRB, 0.38618E+00_JPRB ,&
&  0.39361E+00_JPRB, 0.40360E+00_JPRB, 0.41870E+00_JPRB, 0.40567E+00_JPRB, 0.35953E+00_JPRB, 0.33999E+00_JPRB, 0.31638E+00_JPRB ,&
&  0.30522E+00_JPRB, 0.29544E+00_JPRB, 0.28756E+00_JPRB, 0.27961E+00_JPRB, 0.27024E+00_JPRB, 0.26479E+00_JPRB, 0.25875E+00_JPRB ,&
&  0.25482E+00_JPRB, 0.25354E+00_JPRB, 0.25122E+00_JPRB, 0.25160E+00_JPRB, 0.25170E+00_JPRB, 0.25104E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.24949E+00_JPRB, 0.24836E+00_JPRB, 0.24798E+00_JPRB, 0.24871E+00_JPRB, 0.24959E+00_JPRB, 0.25042E+00_JPRB, 0.24807E+00_JPRB ,&
&  0.24682E+00_JPRB, 0.24560E+00_JPRB, 0.24557E+00_JPRB, 0.24729E+00_JPRB, 0.24791E+00_JPRB, 0.24908E+00_JPRB, 0.24942E+00_JPRB ,&
&  0.24967E+00_JPRB, 0.25039E+00_JPRB, 0.25001E+00_JPRB, 0.25081E+00_JPRB, 0.25105E+00_JPRB, 0.25222E+00_JPRB, 0.25295E+00_JPRB ,&
&  0.25373E+00_JPRB, 0.25599E+00_JPRB, 0.25834E+00_JPRB, 0.26185E+00_JPRB, 0.26654E+00_JPRB, 0.26931E+00_JPRB, 0.27393E+00_JPRB ,&
&  0.27926E+00_JPRB, 0.28868E+00_JPRB, 0.30726E+00_JPRB, 0.32798E+00_JPRB, 0.35405E+00_JPRB, 0.36513E+00_JPRB, 0.37196E+00_JPRB ,&
&  0.38278E+00_JPRB, 0.41338E+00_JPRB, 0.46382E+00_JPRB, 0.48904E+00_JPRB, 0.49637E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.49733E+00_JPRB, 0.49960E+00_JPRB, 0.50123E+00_JPRB, 0.49852E+00_JPRB, 0.49822E+00_JPRB, 0.49842E+00_JPRB, 0.49579E+00_JPRB ,&
&  0.48774E+00_JPRB, 0.48571E+00_JPRB, 0.48065E+00_JPRB, 0.47292E+00_JPRB, 0.46516E+00_JPRB, 0.45227E+00_JPRB, 0.43569E+00_JPRB ,&
&  0.41363E+00_JPRB, 0.39393E+00_JPRB, 0.37938E+00_JPRB, 0.36551E+00_JPRB, 0.35198E+00_JPRB, 0.33879E+00_JPRB, 0.33407E+00_JPRB ,&
&  0.32968E+00_JPRB, 0.32793E+00_JPRB, 0.32755E+00_JPRB, 0.33075E+00_JPRB, 0.33816E+00_JPRB, 0.34747E+00_JPRB, 0.35777E+00_JPRB ,&
&  0.36906E+00_JPRB, 0.37971E+00_JPRB, 0.38077E+00_JPRB, 0.37303E+00_JPRB, 0.36602E+00_JPRB, 0.36178E+00_JPRB, 0.35845E+00_JPRB ,&
&  0.35453E+00_JPRB, 0.34643E+00_JPRB, 0.33682E+00_JPRB, 0.32657E+00_JPRB, 0.31627E+00_JPRB, 0.30877E+00_JPRB, 0.30002E+00_JPRB ,&
&  0.29360E+00_JPRB, 0.28768E+00_JPRB, 0.28115E+00_JPRB, 0.27551E+00_JPRB, 0.27168E+00_JPRB, 0.26854E+00_JPRB, 0.26526E+00_JPRB ,&
&  0.26382E+00_JPRB, 0.26236E+00_JPRB, 0.26075E+00_JPRB, 0.25927E+00_JPRB, 0.25731E+00_JPRB, 0.25590E+00_JPRB, 0.25464E+00_JPRB ,&
&  0.25372E+00_JPRB, 0.25288E+00_JPRB, 0.25240E+00_JPRB, 0.25186E+00_JPRB, 0.25141E+00_JPRB, 0.25033E+00_JPRB, 0.24910E+00_JPRB ,&
&  0.24842E+00_JPRB, 0.24842E+00_JPRB, 0.24768E+00_JPRB, 0.24736E+00_JPRB, 0.24707E+00_JPRB, 0.24624E+00_JPRB, 0.24684E+00_JPRB ,&
&  0.24822E+00_JPRB, 0.24974E+00_JPRB, 0.24989E+00_JPRB, 0.25095E+00_JPRB, 0.25222E+00_JPRB, 0.25384E+00_JPRB, 0.25658E+00_JPRB ,&
&  0.25864E+00_JPRB, 0.26080E+00_JPRB, 0.26356E+00_JPRB, 0.26582E+00_JPRB, 0.26948E+00_JPRB, 0.27400E+00_JPRB, 0.27962E+00_JPRB ,&
&  0.28735E+00_JPRB, 0.29669E+00_JPRB, 0.31007E+00_JPRB, 0.32649E+00_JPRB, 0.35066E+00_JPRB, 0.35859E+00_JPRB, 0.36576E+00_JPRB ,&
&  0.37091E+00_JPRB, 0.38044E+00_JPRB, 0.46280E+00_JPRB, 0.47712E+00_JPRB, 0.49196E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.37441E+00_JPRB, 0.38417E+00_JPRB, 0.39205E+00_JPRB, 0.39934E+00_JPRB, 0.39659E+00_JPRB, 0.46100E+00_JPRB, 0.44015E+00_JPRB ,&
&  0.43008E+00_JPRB, 0.40347E+00_JPRB, 0.40558E+00_JPRB, 0.40198E+00_JPRB, 0.39599E+00_JPRB, 0.38965E+00_JPRB, 0.38418E+00_JPRB ,&
&  0.37857E+00_JPRB, 0.37110E+00_JPRB, 0.36678E+00_JPRB, 0.36116E+00_JPRB, 0.35568E+00_JPRB, 0.35010E+00_JPRB, 0.34670E+00_JPRB ,&
&  0.34418E+00_JPRB, 0.34269E+00_JPRB, 0.34157E+00_JPRB, 0.33953E+00_JPRB, 0.34137E+00_JPRB, 0.34210E+00_JPRB, 0.33979E+00_JPRB ,&
&  0.33544E+00_JPRB, 0.33151E+00_JPRB, 0.32683E+00_JPRB, 0.32205E+00_JPRB, 0.31962E+00_JPRB, 0.31730E+00_JPRB, 0.31484E+00_JPRB ,&
&  0.31134E+00_JPRB, 0.30742E+00_JPRB, 0.30325E+00_JPRB, 0.29984E+00_JPRB, 0.29649E+00_JPRB, 0.29297E+00_JPRB, 0.28962E+00_JPRB ,&
&  0.28603E+00_JPRB, 0.28245E+00_JPRB, 0.27885E+00_JPRB, 0.27659E+00_JPRB, 0.27469E+00_JPRB, 0.27294E+00_JPRB, 0.27135E+00_JPRB ,&
&  0.27076E+00_JPRB, 0.27003E+00_JPRB, 0.26910E+00_JPRB, 0.26792E+00_JPRB, 0.26687E+00_JPRB, 0.26574E+00_JPRB, 0.26486E+00_JPRB ,&
&  0.26482E+00_JPRB, 0.26424E+00_JPRB, 0.26364E+00_JPRB, 0.26285E+00_JPRB, 0.26251E+00_JPRB, 0.26164E+00_JPRB, 0.26056E+00_JPRB ,&
&  0.25959E+00_JPRB, 0.25871E+00_JPRB, 0.25857E+00_JPRB, 0.25839E+00_JPRB, 0.25818E+00_JPRB, 0.25801E+00_JPRB, 0.25848E+00_JPRB ,&
&  0.25908E+00_JPRB, 0.25982E+00_JPRB, 0.26101E+00_JPRB, 0.26236E+00_JPRB, 0.26356E+00_JPRB, 0.26475E+00_JPRB, 0.26644E+00_JPRB ,&
&  0.26855E+00_JPRB, 0.27066E+00_JPRB, 0.27305E+00_JPRB, 0.27506E+00_JPRB, 0.27871E+00_JPRB, 0.28268E+00_JPRB, 0.28708E+00_JPRB ,&
&  0.29232E+00_JPRB, 0.29962E+00_JPRB, 0.30822E+00_JPRB, 0.31604E+00_JPRB, 0.32472E+00_JPRB, 0.33318E+00_JPRB, 0.34012E+00_JPRB ,&
&  0.34611E+00_JPRB, 0.35230E+00_JPRB, 0.35716E+00_JPRB, 0.36270E+00_JPRB, 0.36864E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.33260E+00_JPRB, 0.33570E+00_JPRB, 0.33815E+00_JPRB, 0.33977E+00_JPRB, 0.33951E+00_JPRB, 0.34082E+00_JPRB, 0.34081E+00_JPRB ,&
&  0.34055E+00_JPRB, 0.34006E+00_JPRB, 0.34074E+00_JPRB, 0.34151E+00_JPRB, 0.34197E+00_JPRB, 0.34226E+00_JPRB, 0.34230E+00_JPRB ,&
&  0.34171E+00_JPRB, 0.34063E+00_JPRB, 0.33598E+00_JPRB, 0.33337E+00_JPRB, 0.33174E+00_JPRB, 0.33084E+00_JPRB, 0.33030E+00_JPRB ,&
&  0.32925E+00_JPRB, 0.32807E+00_JPRB, 0.32627E+00_JPRB, 0.32610E+00_JPRB, 0.32531E+00_JPRB, 0.32425E+00_JPRB, 0.32269E+00_JPRB ,&
&  0.32114E+00_JPRB, 0.31991E+00_JPRB, 0.31843E+00_JPRB, 0.31692E+00_JPRB, 0.31513E+00_JPRB, 0.31294E+00_JPRB, 0.31008E+00_JPRB ,&
&  0.30715E+00_JPRB, 0.30421E+00_JPRB, 0.30178E+00_JPRB, 0.29948E+00_JPRB, 0.29750E+00_JPRB, 0.29587E+00_JPRB, 0.29465E+00_JPRB ,&
&  0.29329E+00_JPRB, 0.29172E+00_JPRB, 0.29013E+00_JPRB, 0.28879E+00_JPRB, 0.28746E+00_JPRB, 0.28631E+00_JPRB, 0.28569E+00_JPRB ,&
&  0.28534E+00_JPRB, 0.28492E+00_JPRB, 0.28440E+00_JPRB, 0.28391E+00_JPRB, 0.28331E+00_JPRB, 0.28276E+00_JPRB, 0.28220E+00_JPRB ,&
&  0.28173E+00_JPRB, 0.28149E+00_JPRB, 0.28110E+00_JPRB, 0.28071E+00_JPRB, 0.28046E+00_JPRB, 0.28004E+00_JPRB, 0.27958E+00_JPRB ,&
&  0.27905E+00_JPRB, 0.27882E+00_JPRB, 0.27853E+00_JPRB, 0.27815E+00_JPRB, 0.27780E+00_JPRB, 0.27756E+00_JPRB, 0.27759E+00_JPRB ,&
&  0.27758E+00_JPRB, 0.27750E+00_JPRB, 0.27758E+00_JPRB, 0.27841E+00_JPRB, 0.27929E+00_JPRB, 0.28035E+00_JPRB, 0.28154E+00_JPRB ,&
&  0.28306E+00_JPRB, 0.28466E+00_JPRB, 0.28647E+00_JPRB, 0.28819E+00_JPRB, 0.29053E+00_JPRB, 0.29284E+00_JPRB, 0.29515E+00_JPRB ,&
&  0.29737E+00_JPRB, 0.29967E+00_JPRB, 0.30201E+00_JPRB, 0.30444E+00_JPRB, 0.30772E+00_JPRB, 0.31032E+00_JPRB, 0.31327E+00_JPRB ,&
&  0.31609E+00_JPRB, 0.31931E+00_JPRB, 0.32251E+00_JPRB, 0.32535E+00_JPRB, 0.32815E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB, 0.30239E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB, 0.66249E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.67476E+00_JPRB, 0.67441E+00_JPRB, 0.67416E+00_JPRB, 0.67415E+00_JPRB, 0.67368E+00_JPRB, 0.67284E+00_JPRB, 0.67161E+00_JPRB ,&
&  0.67019E+00_JPRB, 0.66863E+00_JPRB, 0.66647E+00_JPRB, 0.66426E+00_JPRB, 0.66210E+00_JPRB, 0.65997E+00_JPRB, 0.65734E+00_JPRB ,&
&  0.65455E+00_JPRB, 0.65139E+00_JPRB, 0.64813E+00_JPRB, 0.64480E+00_JPRB, 0.64223E+00_JPRB, 0.64006E+00_JPRB, 0.63859E+00_JPRB ,&
&  0.63721E+00_JPRB, 0.63594E+00_JPRB, 0.63468E+00_JPRB, 0.63378E+00_JPRB, 0.63345E+00_JPRB, 0.63293E+00_JPRB, 0.63250E+00_JPRB ,&
&  0.63219E+00_JPRB, 0.63222E+00_JPRB, 0.63200E+00_JPRB, 0.63174E+00_JPRB, 0.63151E+00_JPRB, 0.63198E+00_JPRB, 0.63228E+00_JPRB ,&
&  0.63277E+00_JPRB, 0.63311E+00_JPRB, 0.63333E+00_JPRB, 0.63342E+00_JPRB, 0.63374E+00_JPRB, 0.63417E+00_JPRB, 0.63454E+00_JPRB ,&
&  0.63507E+00_JPRB, 0.63595E+00_JPRB, 0.63765E+00_JPRB, 0.63981E+00_JPRB, 0.64195E+00_JPRB, 0.64424E+00_JPRB, 0.64673E+00_JPRB ,&
&  0.64931E+00_JPRB, 0.65163E+00_JPRB, 0.65425E+00_JPRB, 0.65692E+00_JPRB, 0.65992E+00_JPRB, 0.66277E+00_JPRB, 0.66575E+00_JPRB ,&
&  0.66858E+00_JPRB, 0.67155E+00_JPRB, 0.67491E+00_JPRB, 0.67838E+00_JPRB, 0.68163E+00_JPRB, 0.68446E+00_JPRB, 0.68753E+00_JPRB ,&
&  0.69076E+00_JPRB, 0.69393E+00_JPRB, 0.69634E+00_JPRB, 0.69896E+00_JPRB, 0.70164E+00_JPRB, 0.70428E+00_JPRB, 0.70618E+00_JPRB ,&
&  0.70816E+00_JPRB, 0.71035E+00_JPRB, 0.71263E+00_JPRB, 0.71338E+00_JPRB, 0.71391E+00_JPRB, 0.71417E+00_JPRB, 0.71388E+00_JPRB ,&
&  0.71264E+00_JPRB, 0.71123E+00_JPRB, 0.70977E+00_JPRB, 0.70800E+00_JPRB, 0.70561E+00_JPRB, 0.70291E+00_JPRB, 0.69984E+00_JPRB ,&
&  0.69659E+00_JPRB, 0.69274E+00_JPRB, 0.68898E+00_JPRB, 0.68602E+00_JPRB, 0.68401E+00_JPRB, 0.68181E+00_JPRB, 0.67953E+00_JPRB ,&
&  0.67730E+00_JPRB, 0.67641E+00_JPRB, 0.67614E+00_JPRB, 0.67581E+00_JPRB, 0.67540E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.67521E+00_JPRB, 0.67547E+00_JPRB, 0.67595E+00_JPRB, 0.67585E+00_JPRB, 0.67395E+00_JPRB, 0.67215E+00_JPRB, 0.67012E+00_JPRB ,&
&  0.66852E+00_JPRB, 0.66714E+00_JPRB, 0.66465E+00_JPRB, 0.66141E+00_JPRB, 0.65813E+00_JPRB, 0.65518E+00_JPRB, 0.65212E+00_JPRB ,&
&  0.64922E+00_JPRB, 0.64616E+00_JPRB, 0.64302E+00_JPRB, 0.63758E+00_JPRB, 0.63220E+00_JPRB, 0.62748E+00_JPRB, 0.62483E+00_JPRB ,&
&  0.62220E+00_JPRB, 0.61984E+00_JPRB, 0.61688E+00_JPRB, 0.61371E+00_JPRB, 0.61072E+00_JPRB, 0.60804E+00_JPRB, 0.60526E+00_JPRB ,&
&  0.60318E+00_JPRB, 0.60161E+00_JPRB, 0.60035E+00_JPRB, 0.59879E+00_JPRB, 0.59957E+00_JPRB, 0.60065E+00_JPRB, 0.60161E+00_JPRB ,&
&  0.60248E+00_JPRB, 0.60308E+00_JPRB, 0.60519E+00_JPRB, 0.60721E+00_JPRB, 0.60909E+00_JPRB, 0.61201E+00_JPRB, 0.61392E+00_JPRB ,&
&  0.61470E+00_JPRB, 0.61576E+00_JPRB, 0.61949E+00_JPRB, 0.62277E+00_JPRB, 0.62588E+00_JPRB, 0.62887E+00_JPRB, 0.63302E+00_JPRB ,&
&  0.63712E+00_JPRB, 0.64130E+00_JPRB, 0.64589E+00_JPRB, 0.65139E+00_JPRB, 0.65703E+00_JPRB, 0.66335E+00_JPRB, 0.66968E+00_JPRB ,&
&  0.67565E+00_JPRB, 0.68174E+00_JPRB, 0.68793E+00_JPRB, 0.69408E+00_JPRB, 0.69982E+00_JPRB, 0.70476E+00_JPRB, 0.71033E+00_JPRB ,&
&  0.71628E+00_JPRB, 0.72337E+00_JPRB, 0.72799E+00_JPRB, 0.73298E+00_JPRB, 0.73839E+00_JPRB, 0.74596E+00_JPRB, 0.74904E+00_JPRB ,&
&  0.75175E+00_JPRB, 0.75416E+00_JPRB, 0.75681E+00_JPRB, 0.75804E+00_JPRB, 0.75923E+00_JPRB, 0.76030E+00_JPRB, 0.76074E+00_JPRB ,&
&  0.76029E+00_JPRB, 0.75870E+00_JPRB, 0.75597E+00_JPRB, 0.75175E+00_JPRB, 0.74689E+00_JPRB, 0.74177E+00_JPRB, 0.73556E+00_JPRB ,&
&  0.72982E+00_JPRB, 0.72257E+00_JPRB, 0.71525E+00_JPRB, 0.70943E+00_JPRB, 0.70514E+00_JPRB, 0.69911E+00_JPRB, 0.69285E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.68248E+00_JPRB, 0.67988E+00_JPRB, 0.67764E+00_JPRB, 0.67635E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.67748E+00_JPRB, 0.67731E+00_JPRB, 0.67668E+00_JPRB, 0.67520E+00_JPRB, 0.67186E+00_JPRB, 0.66880E+00_JPRB, 0.66558E+00_JPRB ,&
&  0.66286E+00_JPRB, 0.65995E+00_JPRB, 0.65749E+00_JPRB, 0.65524E+00_JPRB, 0.65312E+00_JPRB, 0.65164E+00_JPRB, 0.65008E+00_JPRB ,&
&  0.64892E+00_JPRB, 0.64732E+00_JPRB, 0.64586E+00_JPRB, 0.64305E+00_JPRB, 0.63818E+00_JPRB, 0.63069E+00_JPRB, 0.62266E+00_JPRB ,&
&  0.61503E+00_JPRB, 0.60839E+00_JPRB, 0.60153E+00_JPRB, 0.59505E+00_JPRB, 0.58979E+00_JPRB, 0.55520E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.54625E+00_JPRB, 0.54775E+00_JPRB, 0.55253E+00_JPRB, 0.55883E+00_JPRB, 0.56432E+00_JPRB, 0.55669E+00_JPRB, 0.55298E+00_JPRB ,&
&  0.52877E+00_JPRB, 0.53084E+00_JPRB, 0.53331E+00_JPRB, 0.53673E+00_JPRB, 0.54410E+00_JPRB, 0.55137E+00_JPRB, 0.56325E+00_JPRB ,&
&  0.58874E+00_JPRB, 0.59402E+00_JPRB, 0.60030E+00_JPRB, 0.60776E+00_JPRB, 0.61434E+00_JPRB, 0.62014E+00_JPRB, 0.62690E+00_JPRB ,&
&  0.63279E+00_JPRB, 0.63879E+00_JPRB, 0.64476E+00_JPRB, 0.65145E+00_JPRB, 0.65895E+00_JPRB, 0.66691E+00_JPRB, 0.67525E+00_JPRB ,&
&  0.68329E+00_JPRB, 0.69237E+00_JPRB, 0.70163E+00_JPRB, 0.71113E+00_JPRB, 0.72054E+00_JPRB, 0.73028E+00_JPRB, 0.74148E+00_JPRB ,&
&  0.75219E+00_JPRB, 0.76351E+00_JPRB, 0.77111E+00_JPRB, 0.77918E+00_JPRB, 0.78457E+00_JPRB, 0.79115E+00_JPRB, 0.79287E+00_JPRB ,&
&  0.79508E+00_JPRB, 0.79271E+00_JPRB, 0.79230E+00_JPRB, 0.79028E+00_JPRB, 0.78874E+00_JPRB, 0.78801E+00_JPRB, 0.78622E+00_JPRB ,&
&  0.78538E+00_JPRB, 0.78308E+00_JPRB, 0.78103E+00_JPRB, 0.77685E+00_JPRB, 0.77102E+00_JPRB, 0.76345E+00_JPRB, 0.75547E+00_JPRB ,&
&  0.74930E+00_JPRB, 0.74422E+00_JPRB, 0.73901E+00_JPRB, 0.73411E+00_JPRB, 0.72722E+00_JPRB, 0.71865E+00_JPRB, 0.70935E+00_JPRB ,&
&  0.70119E+00_JPRB, 0.69373E+00_JPRB, 0.68746E+00_JPRB, 0.68236E+00_JPRB, 0.67935E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.68122E+00_JPRB, 0.68141E+00_JPRB, 0.68074E+00_JPRB, 0.67975E+00_JPRB, 0.67638E+00_JPRB, 0.67338E+00_JPRB, 0.67018E+00_JPRB ,&
&  0.66694E+00_JPRB, 0.66392E+00_JPRB, 0.66072E+00_JPRB, 0.65831E+00_JPRB, 0.65548E+00_JPRB, 0.65347E+00_JPRB, 0.65047E+00_JPRB ,&
&  0.64910E+00_JPRB, 0.64665E+00_JPRB, 0.64408E+00_JPRB, 0.63900E+00_JPRB, 0.63382E+00_JPRB, 0.62775E+00_JPRB, 0.61686E+00_JPRB ,&
&  0.60421E+00_JPRB, 0.59262E+00_JPRB, 0.56388E+00_JPRB, 0.55609E+00_JPRB, 0.54169E+00_JPRB, 0.53366E+00_JPRB, 0.53078E+00_JPRB ,&
&  0.52563E+00_JPRB, 0.52199E+00_JPRB, 0.51195E+00_JPRB, 0.50226E+00_JPRB, 0.49405E+00_JPRB, 0.48069E+00_JPRB, 0.47333E+00_JPRB ,&
&  0.46482E+00_JPRB, 0.46104E+00_JPRB, 0.46262E+00_JPRB, 0.46790E+00_JPRB, 0.48061E+00_JPRB, 0.49604E+00_JPRB, 0.51624E+00_JPRB ,&
&  0.54425E+00_JPRB, 0.56973E+00_JPRB, 0.59529E+00_JPRB, 0.60046E+00_JPRB, 0.60970E+00_JPRB, 0.61846E+00_JPRB, 0.62544E+00_JPRB ,&
&  0.62342E+00_JPRB, 0.62563E+00_JPRB, 0.64130E+00_JPRB, 0.64829E+00_JPRB, 0.65583E+00_JPRB, 0.66389E+00_JPRB, 0.67223E+00_JPRB ,&
&  0.68090E+00_JPRB, 0.69052E+00_JPRB, 0.70059E+00_JPRB, 0.71305E+00_JPRB, 0.73011E+00_JPRB, 0.74854E+00_JPRB, 0.76435E+00_JPRB ,&
&  0.77835E+00_JPRB, 0.79485E+00_JPRB, 0.80680E+00_JPRB, 0.81786E+00_JPRB, 0.82790E+00_JPRB, 0.84151E+00_JPRB, 0.85197E+00_JPRB ,&
&  0.85796E+00_JPRB, 0.85416E+00_JPRB, 0.84467E+00_JPRB, 0.82646E+00_JPRB, 0.81731E+00_JPRB, 0.82410E+00_JPRB, 0.82058E+00_JPRB ,&
&  0.81895E+00_JPRB, 0.81385E+00_JPRB, 0.80997E+00_JPRB, 0.80363E+00_JPRB, 0.79700E+00_JPRB, 0.78808E+00_JPRB, 0.77652E+00_JPRB ,&
&  0.76672E+00_JPRB, 0.76077E+00_JPRB, 0.75462E+00_JPRB, 0.74804E+00_JPRB, 0.73752E+00_JPRB, 0.72646E+00_JPRB, 0.71600E+00_JPRB ,&
&  0.70670E+00_JPRB, 0.69857E+00_JPRB, 0.69101E+00_JPRB, 0.68604E+00_JPRB, 0.68304E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.69255E+00_JPRB, 0.69250E+00_JPRB, 0.68971E+00_JPRB, 0.68729E+00_JPRB, 0.68311E+00_JPRB, 0.67940E+00_JPRB, 0.67474E+00_JPRB ,&
&  0.67038E+00_JPRB, 0.66606E+00_JPRB, 0.66206E+00_JPRB, 0.65859E+00_JPRB, 0.65519E+00_JPRB, 0.65214E+00_JPRB, 0.64992E+00_JPRB ,&
&  0.64800E+00_JPRB, 0.64502E+00_JPRB, 0.64051E+00_JPRB, 0.63499E+00_JPRB, 0.63016E+00_JPRB, 0.62504E+00_JPRB, 0.61824E+00_JPRB ,&
&  0.61102E+00_JPRB, 0.59986E+00_JPRB, 0.58120E+00_JPRB, 0.57215E+00_JPRB, 0.56062E+00_JPRB, 0.55692E+00_JPRB, 0.55051E+00_JPRB ,&
&  0.55025E+00_JPRB, 0.51062E+00_JPRB, 0.50802E+00_JPRB, 0.49181E+00_JPRB, 0.47985E+00_JPRB, 0.47053E+00_JPRB, 0.45423E+00_JPRB ,&
&  0.44688E+00_JPRB, 0.43779E+00_JPRB, 0.43450E+00_JPRB, 0.43907E+00_JPRB, 0.44763E+00_JPRB, 0.46560E+00_JPRB, 0.48862E+00_JPRB ,&
&  0.51688E+00_JPRB, 0.57447E+00_JPRB, 0.58943E+00_JPRB, 0.60111E+00_JPRB, 0.61190E+00_JPRB, 0.62034E+00_JPRB, 0.61087E+00_JPRB ,&
&  0.61060E+00_JPRB, 0.61112E+00_JPRB, 0.61938E+00_JPRB, 0.63251E+00_JPRB, 0.64637E+00_JPRB, 0.65226E+00_JPRB, 0.65976E+00_JPRB ,&
&  0.66631E+00_JPRB, 0.67665E+00_JPRB, 0.68766E+00_JPRB, 0.70531E+00_JPRB, 0.73364E+00_JPRB, 0.76361E+00_JPRB, 0.78927E+00_JPRB ,&
&  0.81087E+00_JPRB, 0.82428E+00_JPRB, 0.83191E+00_JPRB, 0.84501E+00_JPRB, 0.86289E+00_JPRB, 0.88750E+00_JPRB, 0.90220E+00_JPRB ,&
&  0.91103E+00_JPRB, 0.91761E+00_JPRB, 0.91479E+00_JPRB, 0.90694E+00_JPRB, 0.88795E+00_JPRB, 0.86987E+00_JPRB, 0.86049E+00_JPRB ,&
&  0.86392E+00_JPRB, 0.85747E+00_JPRB, 0.84691E+00_JPRB, 0.83032E+00_JPRB, 0.81872E+00_JPRB, 0.80497E+00_JPRB, 0.79058E+00_JPRB ,&
&  0.77818E+00_JPRB, 0.77005E+00_JPRB, 0.75988E+00_JPRB, 0.75074E+00_JPRB, 0.73774E+00_JPRB, 0.72643E+00_JPRB, 0.71785E+00_JPRB ,&
&  0.70945E+00_JPRB, 0.70327E+00_JPRB, 0.69699E+00_JPRB, 0.69415E+00_JPRB, 0.69342E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.70269E+00_JPRB, 0.70059E+00_JPRB, 0.69734E+00_JPRB, 0.69443E+00_JPRB, 0.69111E+00_JPRB, 0.68892E+00_JPRB, 0.68727E+00_JPRB ,&
&  0.68478E+00_JPRB, 0.68227E+00_JPRB, 0.67961E+00_JPRB, 0.67675E+00_JPRB, 0.67379E+00_JPRB, 0.67119E+00_JPRB, 0.66873E+00_JPRB ,&
&  0.66441E+00_JPRB, 0.65909E+00_JPRB, 0.65188E+00_JPRB, 0.64532E+00_JPRB, 0.63944E+00_JPRB, 0.63663E+00_JPRB, 0.63167E+00_JPRB ,&
&  0.62666E+00_JPRB, 0.61814E+00_JPRB, 0.60412E+00_JPRB, 0.59377E+00_JPRB, 0.59088E+00_JPRB, 0.58255E+00_JPRB, 0.58274E+00_JPRB ,&
&  0.57865E+00_JPRB, 0.57148E+00_JPRB, 0.55574E+00_JPRB, 0.55065E+00_JPRB, 0.54174E+00_JPRB, 0.48511E+00_JPRB, 0.47142E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.43382E+00_JPRB, 0.42644E+00_JPRB, 0.42433E+00_JPRB, 0.42865E+00_JPRB, 0.44698E+00_JPRB, 0.47160E+00_JPRB ,&
&  0.49754E+00_JPRB, 0.56748E+00_JPRB, 0.58536E+00_JPRB, 0.60036E+00_JPRB, 0.60984E+00_JPRB, 0.61451E+00_JPRB, 0.60914E+00_JPRB ,&
&  0.60684E+00_JPRB, 0.60750E+00_JPRB, 0.61773E+00_JPRB, 0.63399E+00_JPRB, 0.64698E+00_JPRB, 0.64772E+00_JPRB, 0.64771E+00_JPRB ,&
&  0.64373E+00_JPRB, 0.65233E+00_JPRB, 0.66896E+00_JPRB, 0.69269E+00_JPRB, 0.72675E+00_JPRB, 0.75298E+00_JPRB, 0.78761E+00_JPRB ,&
&  0.81802E+00_JPRB, 0.83909E+00_JPRB, 0.87146E+00_JPRB, 0.90537E+00_JPRB, 0.92943E+00_JPRB, 0.94525E+00_JPRB, 0.94752E+00_JPRB ,&
&  0.94306E+00_JPRB, 0.96219E+00_JPRB, 0.97026E+00_JPRB, 0.96760E+00_JPRB, 0.95652E+00_JPRB, 0.93403E+00_JPRB, 0.90903E+00_JPRB ,&
&  0.89700E+00_JPRB, 0.90175E+00_JPRB, 0.89177E+00_JPRB, 0.86200E+00_JPRB, 0.83653E+00_JPRB, 0.81609E+00_JPRB, 0.79988E+00_JPRB ,&
&  0.78829E+00_JPRB, 0.78048E+00_JPRB, 0.76875E+00_JPRB, 0.75780E+00_JPRB, 0.74565E+00_JPRB, 0.73385E+00_JPRB, 0.72635E+00_JPRB ,&
&  0.71987E+00_JPRB, 0.71702E+00_JPRB, 0.71314E+00_JPRB, 0.70952E+00_JPRB, 0.70732E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.71258E+00_JPRB, 0.71130E+00_JPRB, 0.70827E+00_JPRB, 0.70507E+00_JPRB, 0.70344E+00_JPRB, 0.70398E+00_JPRB, 0.70184E+00_JPRB ,&
&  0.69811E+00_JPRB, 0.69502E+00_JPRB, 0.68985E+00_JPRB, 0.68758E+00_JPRB, 0.68455E+00_JPRB, 0.68097E+00_JPRB, 0.67785E+00_JPRB ,&
&  0.67332E+00_JPRB, 0.65988E+00_JPRB, 0.65173E+00_JPRB, 0.64957E+00_JPRB, 0.64800E+00_JPRB, 0.64809E+00_JPRB, 0.64814E+00_JPRB ,&
&  0.64330E+00_JPRB, 0.63459E+00_JPRB, 0.62570E+00_JPRB, 0.61084E+00_JPRB, 0.59989E+00_JPRB, 0.59998E+00_JPRB, 0.60526E+00_JPRB ,&
&  0.60078E+00_JPRB, 0.58370E+00_JPRB, 0.58629E+00_JPRB, 0.58474E+00_JPRB, 0.56403E+00_JPRB, 0.55574E+00_JPRB, 0.50564E+00_JPRB ,&
&  0.47160E+00_JPRB, 0.44320E+00_JPRB, 0.43253E+00_JPRB, 0.42594E+00_JPRB, 0.43230E+00_JPRB, 0.44881E+00_JPRB, 0.48544E+00_JPRB ,&
&  0.52354E+00_JPRB, 0.56305E+00_JPRB, 0.57601E+00_JPRB, 0.59007E+00_JPRB, 0.59698E+00_JPRB, 0.61010E+00_JPRB, 0.62016E+00_JPRB ,&
&  0.62703E+00_JPRB, 0.63363E+00_JPRB, 0.64640E+00_JPRB, 0.65908E+00_JPRB, 0.68378E+00_JPRB, 0.70512E+00_JPRB, 0.71761E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.73995E+00_JPRB, 0.75600E+00_JPRB, 0.77528E+00_JPRB, 0.80144E+00_JPRB, 0.81608E+00_JPRB, 0.83108E+00_JPRB ,&
&  0.84846E+00_JPRB, 0.89039E+00_JPRB, 0.94968E+00_JPRB, 0.10202E+01_JPRB, 0.10412E+01_JPRB, 0.10377E+01_JPRB, 0.10071E+01_JPRB ,&
&  0.96895E+00_JPRB, 0.99151E+00_JPRB, 0.11135E+01_JPRB, 0.11001E+01_JPRB, 0.10887E+01_JPRB, 0.10699E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.95796E+00_JPRB, 0.93302E+00_JPRB, 0.90815E+00_JPRB, 0.86480E+00_JPRB, 0.82839E+00_JPRB, 0.80483E+00_JPRB, 0.79272E+00_JPRB ,&
&  0.78612E+00_JPRB, 0.78423E+00_JPRB, 0.77524E+00_JPRB, 0.76772E+00_JPRB, 0.75609E+00_JPRB, 0.74320E+00_JPRB, 0.73558E+00_JPRB ,&
&  0.72844E+00_JPRB, 0.72503E+00_JPRB, 0.72316E+00_JPRB, 0.72040E+00_JPRB, 0.71712E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.72249E+00_JPRB, 0.72500E+00_JPRB, 0.72635E+00_JPRB, 0.73255E+00_JPRB, 0.73937E+00_JPRB, 0.73936E+00_JPRB, 0.73351E+00_JPRB ,&
&  0.71980E+00_JPRB, 0.70369E+00_JPRB, 0.69011E+00_JPRB, 0.68046E+00_JPRB, 0.67435E+00_JPRB, 0.67061E+00_JPRB, 0.66724E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.66307E+00_JPRB, 0.65770E+00_JPRB, 0.64975E+00_JPRB, 0.64671E+00_JPRB, 0.64828E+00_JPRB, 0.65079E+00_JPRB ,&
&  0.65114E+00_JPRB, 0.64629E+00_JPRB, 0.63859E+00_JPRB, 0.62568E+00_JPRB, 0.61240E+00_JPRB, 0.61632E+00_JPRB, 0.60912E+00_JPRB ,&
&  0.60235E+00_JPRB, 0.60069E+00_JPRB, 0.60242E+00_JPRB, 0.60101E+00_JPRB, 0.59652E+00_JPRB, 0.57790E+00_JPRB, 0.53601E+00_JPRB ,&
&  0.49346E+00_JPRB, 0.45945E+00_JPRB, 0.43984E+00_JPRB, 0.43409E+00_JPRB, 0.44404E+00_JPRB, 0.47073E+00_JPRB, 0.50945E+00_JPRB ,&
&  0.55491E+00_JPRB, 0.57249E+00_JPRB, 0.58792E+00_JPRB, 0.60483E+00_JPRB, 0.61962E+00_JPRB, 0.64033E+00_JPRB, 0.66169E+00_JPRB ,&
&  0.68617E+00_JPRB, 0.71973E+00_JPRB, 0.76513E+00_JPRB, 0.82546E+00_JPRB, 0.89289E+00_JPRB, 0.88722E+00_JPRB, 0.94418E+00_JPRB ,&
&  0.10908E+01_JPRB, 0.10978E+01_JPRB, 0.10600E+01_JPRB, 0.10120E+01_JPRB, 0.98362E+00_JPRB, 0.97568E+00_JPRB, 0.98358E+00_JPRB ,&
&  0.99753E+00_JPRB, 0.10249E+01_JPRB, 0.10693E+01_JPRB, 0.11195E+01_JPRB, 0.11379E+01_JPRB, 0.11337E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.10471E+01_JPRB, 0.10693E+01_JPRB, 0.13069E+01_JPRB, 0.11973E+01_JPRB, 0.11043E+01_JPRB, 0.10620E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.96736E+00_JPRB, 0.93339E+00_JPRB, 0.89327E+00_JPRB, 0.85886E+00_JPRB, 0.81947E+00_JPRB, 0.78368E+00_JPRB, 0.75274E+00_JPRB ,&
&  0.74309E+00_JPRB, 0.74016E+00_JPRB, 0.73809E+00_JPRB, 0.73698E+00_JPRB, 0.73034E+00_JPRB, 0.72777E+00_JPRB, 0.72672E+00_JPRB ,&
&  0.71814E+00_JPRB, 0.71389E+00_JPRB, 0.71593E+00_JPRB, 0.71853E+00_JPRB, 0.72040E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.69228E+00_JPRB, 0.71103E+00_JPRB, 0.74295E+00_JPRB, 0.76297E+00_JPRB, 0.76978E+00_JPRB, 0.75050E+00_JPRB, 0.72663E+00_JPRB ,&
&  0.70135E+00_JPRB, 0.67219E+00_JPRB, 0.65411E+00_JPRB, 0.64412E+00_JPRB, 0.63860E+00_JPRB, 0.63487E+00_JPRB, 0.63233E+00_JPRB ,&
&  0.63116E+00_JPRB, 0.63007E+00_JPRB, 0.63239E+00_JPRB, 0.64000E+00_JPRB, 0.64627E+00_JPRB, 0.64654E+00_JPRB, 0.64577E+00_JPRB ,&
&  0.64667E+00_JPRB, 0.64595E+00_JPRB, 0.63838E+00_JPRB, 0.62467E+00_JPRB, 0.61936E+00_JPRB, 0.63112E+00_JPRB, 0.63238E+00_JPRB ,&
&  0.62743E+00_JPRB, 0.61729E+00_JPRB, 0.61751E+00_JPRB, 0.62526E+00_JPRB, 0.62369E+00_JPRB, 0.60995E+00_JPRB, 0.56998E+00_JPRB ,&
&  0.52025E+00_JPRB, 0.48001E+00_JPRB, 0.46635E+00_JPRB, 0.48149E+00_JPRB, 0.51451E+00_JPRB, 0.56202E+00_JPRB, 0.58038E+00_JPRB ,&
&  0.59363E+00_JPRB, 0.60419E+00_JPRB, 0.61619E+00_JPRB, 0.63933E+00_JPRB, 0.66982E+00_JPRB, 0.69938E+00_JPRB, 0.73018E+00_JPRB ,&
&  0.77565E+00_JPRB, 0.82494E+00_JPRB, 0.88607E+00_JPRB, 0.86142E+00_JPRB, 0.89613E+00_JPRB, 0.98064E+00_JPRB, 0.10268E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.11025E+01_JPRB, 0.11400E+01_JPRB, 0.11878E+01_JPRB, 0.11772E+01_JPRB, 0.11573E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11297E+01_JPRB, 0.11399E+01_JPRB, 0.11913E+01_JPRB, 0.12450E+01_JPRB, 0.12474E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.10618E+01_JPRB, 0.97659E+00_JPRB, 0.91903E+00_JPRB, 0.91409E+00_JPRB, 0.93474E+00_JPRB, 0.94673E+00_JPRB ,&
&  0.94451E+00_JPRB, 0.93616E+00_JPRB, 0.90960E+00_JPRB, 0.88651E+00_JPRB, 0.81731E+00_JPRB, 0.76694E+00_JPRB, 0.72763E+00_JPRB ,&
&  0.71266E+00_JPRB, 0.70256E+00_JPRB, 0.69674E+00_JPRB, 0.70507E+00_JPRB, 0.69875E+00_JPRB, 0.69929E+00_JPRB, 0.70004E+00_JPRB ,&
&  0.69501E+00_JPRB, 0.68614E+00_JPRB, 0.68337E+00_JPRB, 0.68255E+00_JPRB, 0.68514E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.76616E+00_JPRB, 0.78060E+00_JPRB, 0.79218E+00_JPRB, 0.79630E+00_JPRB, 0.78206E+00_JPRB, 0.76954E+00_JPRB, 0.75799E+00_JPRB ,&
&  0.75105E+00_JPRB, 0.69873E+00_JPRB, 0.66062E+00_JPRB, 0.63392E+00_JPRB, 0.61442E+00_JPRB, 0.59638E+00_JPRB, 0.59117E+00_JPRB ,&
&  0.59359E+00_JPRB, 0.59940E+00_JPRB, 0.60811E+00_JPRB, 0.61837E+00_JPRB, 0.62754E+00_JPRB, 0.63110E+00_JPRB, 0.63709E+00_JPRB ,&
&  0.64734E+00_JPRB, 0.65517E+00_JPRB, 0.65459E+00_JPRB, 0.65519E+00_JPRB, 0.65772E+00_JPRB, 0.66754E+00_JPRB, 0.66821E+00_JPRB ,&
&  0.66544E+00_JPRB, 0.65855E+00_JPRB, 0.65450E+00_JPRB, 0.66373E+00_JPRB, 0.66844E+00_JPRB, 0.66043E+00_JPRB, 0.61069E+00_JPRB ,&
&  0.53797E+00_JPRB, 0.50976E+00_JPRB, 0.52597E+00_JPRB, 0.58042E+00_JPRB, 0.60044E+00_JPRB, 0.61479E+00_JPRB, 0.62964E+00_JPRB ,&
&  0.62196E+00_JPRB, 0.61757E+00_JPRB, 0.63682E+00_JPRB, 0.68755E+00_JPRB, 0.71301E+00_JPRB, 0.73982E+00_JPRB, 0.77591E+00_JPRB ,&
&  0.83812E+00_JPRB, 0.88846E+00_JPRB, 0.94591E+00_JPRB, 0.91370E+00_JPRB, 0.10215E+01_JPRB, 0.12455E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.12259E+01_JPRB, 0.12899E+01_JPRB, 0.13310E+01_JPRB, 0.13337E+01_JPRB, 0.13161E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.12875E+01_JPRB, 0.12806E+01_JPRB, 0.12938E+01_JPRB, 0.13627E+01_JPRB, 0.13626E+01_JPRB, 0.13362E+01_JPRB, 0.13001E+01_JPRB ,&
&  0.12359E+01_JPRB, 0.11424E+01_JPRB, 0.10552E+01_JPRB, 0.98996E+00_JPRB, 0.97110E+00_JPRB, 0.97430E+00_JPRB, 0.96882E+00_JPRB ,&
&  0.94771E+00_JPRB, 0.94219E+00_JPRB, 0.94068E+00_JPRB, 0.93403E+00_JPRB, 0.86378E+00_JPRB, 0.78359E+00_JPRB, 0.72407E+00_JPRB ,&
&  0.68276E+00_JPRB, 0.65668E+00_JPRB, 0.63622E+00_JPRB, 0.64073E+00_JPRB, 0.65709E+00_JPRB, 0.67831E+00_JPRB, 0.67561E+00_JPRB ,&
&  0.67605E+00_JPRB, 0.67427E+00_JPRB, 0.68415E+00_JPRB, 0.71352E+00_JPRB, 0.74254E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.80897E+00_JPRB, 0.82304E+00_JPRB, 0.83210E+00_JPRB, 0.81939E+00_JPRB, 0.79919E+00_JPRB, 0.78451E+00_JPRB, 0.76898E+00_JPRB ,&
&  0.75987E+00_JPRB, 0.68162E+00_JPRB, 0.64326E+00_JPRB, 0.62405E+00_JPRB, 0.61044E+00_JPRB, 0.58520E+00_JPRB, 0.56572E+00_JPRB ,&
&  0.56515E+00_JPRB, 0.58024E+00_JPRB, 0.60628E+00_JPRB, 0.62638E+00_JPRB, 0.64532E+00_JPRB, 0.65970E+00_JPRB, 0.66582E+00_JPRB ,&
&  0.67217E+00_JPRB, 0.68025E+00_JPRB, 0.68523E+00_JPRB, 0.69140E+00_JPRB, 0.70035E+00_JPRB, 0.72012E+00_JPRB, 0.74693E+00_JPRB ,&
&  0.76511E+00_JPRB, 0.77118E+00_JPRB, 0.74848E+00_JPRB, 0.74997E+00_JPRB, 0.75574E+00_JPRB, 0.75990E+00_JPRB, 0.75806E+00_JPRB ,&
&  0.59454E+00_JPRB, 0.57860E+00_JPRB, 0.63615E+00_JPRB, 0.63611E+00_JPRB, 0.64638E+00_JPRB, 0.64689E+00_JPRB, 0.64526E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.66348E+00_JPRB, 0.69011E+00_JPRB, 0.72428E+00_JPRB, 0.75122E+00_JPRB, 0.77708E+00_JPRB, 0.81301E+00_JPRB ,&
&  0.89971E+00_JPRB, 0.85477E+00_JPRB, 0.85575E+00_JPRB, 0.94542E+00_JPRB, 0.11355E+01_JPRB, 0.12830E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.14504E+01_JPRB, 0.14475E+01_JPRB, 0.14496E+01_JPRB, 0.14819E+01_JPRB, 0.14926E+01_JPRB, 0.14879E+01_JPRB, 0.15160E+01_JPRB ,&
&  0.15151E+01_JPRB, 0.15350E+01_JPRB, 0.15619E+01_JPRB, 0.15868E+01_JPRB, 0.15277E+01_JPRB, 0.14794E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.13563E+01_JPRB, 0.12747E+01_JPRB, 0.11793E+01_JPRB, 0.11030E+01_JPRB, 0.10485E+01_JPRB, 0.10423E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.10095E+01_JPRB, 0.99412E+00_JPRB, 0.97239E+00_JPRB, 0.95008E+00_JPRB, 0.90101E+00_JPRB, 0.85184E+00_JPRB, 0.82729E+00_JPRB ,&
&  0.75898E+00_JPRB, 0.70741E+00_JPRB, 0.64911E+00_JPRB, 0.64877E+00_JPRB, 0.68741E+00_JPRB, 0.70562E+00_JPRB, 0.70901E+00_JPRB ,&
&  0.73457E+00_JPRB, 0.75102E+00_JPRB, 0.75621E+00_JPRB, 0.77259E+00_JPRB, 0.79206E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.83230E+00_JPRB, 0.83218E+00_JPRB, 0.82719E+00_JPRB, 0.81574E+00_JPRB, 0.81429E+00_JPRB, 0.80627E+00_JPRB, 0.80867E+00_JPRB ,&
&  0.83128E+00_JPRB, 0.78512E+00_JPRB, 0.82593E+00_JPRB, 0.70195E+00_JPRB, 0.63554E+00_JPRB, 0.60169E+00_JPRB, 0.58012E+00_JPRB ,&
&  0.58310E+00_JPRB, 0.60631E+00_JPRB, 0.62833E+00_JPRB, 0.64612E+00_JPRB, 0.68043E+00_JPRB, 0.68752E+00_JPRB, 0.68831E+00_JPRB ,&
&  0.69460E+00_JPRB, 0.70214E+00_JPRB, 0.72670E+00_JPRB, 0.73269E+00_JPRB, 0.76079E+00_JPRB, 0.80012E+00_JPRB, 0.87352E+00_JPRB ,&
&  0.84002E+00_JPRB, 0.84177E+00_JPRB, 0.88740E+00_JPRB, 0.90187E+00_JPRB, 0.90965E+00_JPRB, 0.89623E+00_JPRB, 0.87075E+00_JPRB ,&
&  0.85175E+00_JPRB, 0.74458E+00_JPRB, 0.75882E+00_JPRB, 0.74838E+00_JPRB, 0.73780E+00_JPRB, 0.72777E+00_JPRB, 0.70952E+00_JPRB ,&
&  0.70066E+00_JPRB, 0.70740E+00_JPRB, 0.72802E+00_JPRB, 0.75580E+00_JPRB, 0.79359E+00_JPRB, 0.83647E+00_JPRB, 0.87805E+00_JPRB ,&
&  0.97923E+00_JPRB, 0.91411E+00_JPRB, 0.10879E+01_JPRB, 0.11798E+01_JPRB, 0.13293E+01_JPRB, 0.15337E+01_JPRB, 0.16693E+01_JPRB ,&
&  0.18747E+01_JPRB, 0.18358E+01_JPRB, 0.18275E+01_JPRB, 0.17972E+01_JPRB, 0.17518E+01_JPRB, 0.17301E+01_JPRB, 0.17405E+01_JPRB ,&
&  0.17965E+01_JPRB, 0.17996E+01_JPRB, 0.19244E+01_JPRB, 0.18782E+01_JPRB, 0.17488E+01_JPRB, 0.16612E+01_JPRB, 0.15657E+01_JPRB ,&
&  0.14853E+01_JPRB, 0.14303E+01_JPRB, 0.13721E+01_JPRB, 0.12358E+01_JPRB, 0.11991E+01_JPRB, 0.11759E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.99833E+00_JPRB, 0.92657E+00_JPRB, 0.93287E+00_JPRB, 0.87923E+00_JPRB, 0.83835E+00_JPRB, 0.83183E+00_JPRB ,&
&  0.80343E+00_JPRB, 0.73186E+00_JPRB, 0.73175E+00_JPRB, 0.75979E+00_JPRB, 0.75058E+00_JPRB, 0.73761E+00_JPRB, 0.76881E+00_JPRB ,&
&  0.77947E+00_JPRB, 0.79533E+00_JPRB, 0.83561E+00_JPRB, 0.83809E+00_JPRB, 0.84244E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.84472E+00_JPRB, 0.82400E+00_JPRB, 0.81091E+00_JPRB, 0.80915E+00_JPRB, 0.82490E+00_JPRB, 0.85512E+00_JPRB, 0.84554E+00_JPRB ,&
&  0.86784E+00_JPRB, 0.87668E+00_JPRB, 0.86474E+00_JPRB, 0.84039E+00_JPRB, 0.80237E+00_JPRB, 0.70848E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.58284E+00_JPRB, 0.61796E+00_JPRB, 0.65414E+00_JPRB, 0.66898E+00_JPRB, 0.69782E+00_JPRB, 0.70030E+00_JPRB, 0.70211E+00_JPRB ,&
&  0.70782E+00_JPRB, 0.72010E+00_JPRB, 0.74677E+00_JPRB, 0.78808E+00_JPRB, 0.82128E+00_JPRB, 0.89199E+00_JPRB, 0.97134E+00_JPRB ,&
&  0.90779E+00_JPRB, 0.86709E+00_JPRB, 0.87120E+00_JPRB, 0.85036E+00_JPRB, 0.92053E+00_JPRB, 0.94216E+00_JPRB, 0.94013E+00_JPRB ,&
&  0.92330E+00_JPRB, 0.89705E+00_JPRB, 0.85801E+00_JPRB, 0.83225E+00_JPRB, 0.80940E+00_JPRB, 0.79466E+00_JPRB, 0.78958E+00_JPRB ,&
&  0.80223E+00_JPRB, 0.82133E+00_JPRB, 0.85200E+00_JPRB, 0.90142E+00_JPRB, 0.95829E+00_JPRB, 0.10075E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.13439E+01_JPRB, 0.15347E+01_JPRB, 0.15099E+01_JPRB, 0.16674E+01_JPRB, 0.18251E+01_JPRB, 0.18728E+01_JPRB ,&
&  0.20023E+01_JPRB, 0.19812E+01_JPRB, 0.20616E+01_JPRB, 0.21166E+01_JPRB, 0.21585E+01_JPRB, 0.21499E+01_JPRB, 0.20848E+01_JPRB ,&
&  0.20644E+01_JPRB, 0.22262E+01_JPRB, 0.22606E+01_JPRB, 0.21619E+01_JPRB, 0.19792E+01_JPRB, 0.18550E+01_JPRB, 0.17145E+01_JPRB ,&
&  0.16599E+01_JPRB, 0.16559E+01_JPRB, 0.15955E+01_JPRB, 0.15305E+01_JPRB, 0.14201E+01_JPRB, 0.12980E+01_JPRB, 0.12005E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.95172E+00_JPRB, 0.87943E+00_JPRB, 0.86108E+00_JPRB, 0.85143E+00_JPRB, 0.80472E+00_JPRB, 0.80782E+00_JPRB ,&
&  0.81594E+00_JPRB, 0.80022E+00_JPRB, 0.84185E+00_JPRB, 0.86608E+00_JPRB, 0.84681E+00_JPRB, 0.82013E+00_JPRB, 0.78345E+00_JPRB ,&
&  0.75902E+00_JPRB, 0.80502E+00_JPRB, 0.84900E+00_JPRB, 0.86750E+00_JPRB, 0.86338E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.86235E+00_JPRB, 0.83803E+00_JPRB, 0.80811E+00_JPRB, 0.80749E+00_JPRB, 0.84193E+00_JPRB, 0.87580E+00_JPRB, 0.89686E+00_JPRB ,&
&  0.89132E+00_JPRB, 0.89143E+00_JPRB, 0.87919E+00_JPRB, 0.86334E+00_JPRB, 0.83520E+00_JPRB, 0.80101E+00_JPRB, 0.72714E+00_JPRB ,&
&  0.64484E+00_JPRB, 0.70374E+00_JPRB, 0.67421E+00_JPRB, 0.68678E+00_JPRB, 0.74631E+00_JPRB, 0.73460E+00_JPRB, 0.73131E+00_JPRB ,&
&  0.74349E+00_JPRB, 0.75730E+00_JPRB, 0.79180E+00_JPRB, 0.82327E+00_JPRB, 0.87331E+00_JPRB, 0.10051E+01_JPRB, 0.10027E+01_JPRB ,&
&  0.89597E+00_JPRB, 0.83785E+00_JPRB, 0.84706E+00_JPRB, 0.88202E+00_JPRB, 0.92564E+00_JPRB, 0.96732E+00_JPRB, 0.96080E+00_JPRB ,&
&  0.93682E+00_JPRB, 0.90373E+00_JPRB, 0.84814E+00_JPRB, 0.81614E+00_JPRB, 0.80351E+00_JPRB, 0.80630E+00_JPRB, 0.81450E+00_JPRB ,&
&  0.82427E+00_JPRB, 0.84782E+00_JPRB, 0.89304E+00_JPRB, 0.96730E+00_JPRB, 0.10489E+01_JPRB, 0.12280E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.16081E+01_JPRB, 0.17642E+01_JPRB, 0.19304E+01_JPRB, 0.21278E+01_JPRB, 0.21760E+01_JPRB, 0.22023E+01_JPRB ,&
&  0.22632E+01_JPRB, 0.22224E+01_JPRB, 0.25035E+01_JPRB, 0.25466E+01_JPRB, 0.25222E+01_JPRB, 0.25414E+01_JPRB, 0.24129E+01_JPRB ,&
&  0.23041E+01_JPRB, 0.22214E+01_JPRB, 0.21956E+01_JPRB, 0.21284E+01_JPRB, 0.20486E+01_JPRB, 0.20407E+01_JPRB, 0.19098E+01_JPRB ,&
&  0.18900E+01_JPRB, 0.18371E+01_JPRB, 0.15806E+01_JPRB, 0.13852E+01_JPRB, 0.11915E+01_JPRB, 0.12101E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.90632E+00_JPRB, 0.89236E+00_JPRB, 0.87186E+00_JPRB, 0.87945E+00_JPRB, 0.91949E+00_JPRB, 0.93090E+00_JPRB ,&
&  0.94299E+00_JPRB, 0.97439E+00_JPRB, 0.98045E+00_JPRB, 0.10343E+01_JPRB, 0.99898E+00_JPRB, 0.93880E+00_JPRB, 0.81029E+00_JPRB ,&
&  0.77208E+00_JPRB, 0.80780E+00_JPRB, 0.84684E+00_JPRB, 0.87125E+00_JPRB, 0.87584E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.90146E+00_JPRB, 0.87870E+00_JPRB, 0.84917E+00_JPRB, 0.84574E+00_JPRB, 0.87729E+00_JPRB, 0.91193E+00_JPRB, 0.93131E+00_JPRB ,&
&  0.92246E+00_JPRB, 0.92031E+00_JPRB, 0.89815E+00_JPRB, 0.88047E+00_JPRB, 0.85988E+00_JPRB, 0.82517E+00_JPRB, 0.82200E+00_JPRB ,&
&  0.70167E+00_JPRB, 0.77661E+00_JPRB, 0.67975E+00_JPRB, 0.67896E+00_JPRB, 0.81655E+00_JPRB, 0.79560E+00_JPRB, 0.80225E+00_JPRB ,&
&  0.78912E+00_JPRB, 0.81585E+00_JPRB, 0.82870E+00_JPRB, 0.86219E+00_JPRB, 0.98026E+00_JPRB, 0.10692E+01_JPRB, 0.10288E+01_JPRB ,&
&  0.95153E+00_JPRB, 0.90079E+00_JPRB, 0.91608E+00_JPRB, 0.92891E+00_JPRB, 0.93346E+00_JPRB, 0.95439E+00_JPRB, 0.95591E+00_JPRB ,&
&  0.91962E+00_JPRB, 0.90383E+00_JPRB, 0.87709E+00_JPRB, 0.87277E+00_JPRB, 0.88095E+00_JPRB, 0.90014E+00_JPRB, 0.91405E+00_JPRB ,&
&  0.93512E+00_JPRB, 0.97111E+00_JPRB, 0.10168E+01_JPRB, 0.11058E+01_JPRB, 0.11915E+01_JPRB, 0.14160E+01_JPRB, 0.17246E+01_JPRB ,&
&  0.20961E+01_JPRB, 0.22102E+01_JPRB, 0.24120E+01_JPRB, 0.27992E+01_JPRB, 0.28646E+01_JPRB, 0.27824E+01_JPRB, 0.26037E+01_JPRB ,&
&  0.25418E+01_JPRB, 0.24812E+01_JPRB, 0.25143E+01_JPRB, 0.26034E+01_JPRB, 0.25977E+01_JPRB, 0.26825E+01_JPRB, 0.25706E+01_JPRB ,&
&  0.23041E+01_JPRB, 0.21783E+01_JPRB, 0.21174E+01_JPRB, 0.20934E+01_JPRB, 0.19906E+01_JPRB, 0.19939E+01_JPRB, 0.18887E+01_JPRB ,&
&  0.18808E+01_JPRB, 0.16491E+01_JPRB, 0.12593E+01_JPRB, 0.11576E+01_JPRB, 0.95147E+00_JPRB, 0.97815E+00_JPRB, 0.12074E+01_JPRB ,&
&  0.11763E+01_JPRB, 0.10582E+01_JPRB, 0.97239E+00_JPRB, 0.93315E+00_JPRB, 0.95273E+00_JPRB, 0.12156E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.12170E+01_JPRB, 0.12716E+01_JPRB, 0.13131E+01_JPRB, 0.12596E+01_JPRB, 0.11780E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.99094E+00_JPRB, 0.96884E+00_JPRB, 0.96383E+00_JPRB, 0.94732E+00_JPRB, 0.92468E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.99608E+00_JPRB, 0.96088E+00_JPRB, 0.92829E+00_JPRB, 0.92207E+00_JPRB, 0.94130E+00_JPRB, 0.97061E+00_JPRB, 0.98685E+00_JPRB ,&
&  0.97239E+00_JPRB, 0.96107E+00_JPRB, 0.93189E+00_JPRB, 0.90694E+00_JPRB, 0.88377E+00_JPRB, 0.85804E+00_JPRB, 0.84374E+00_JPRB ,&
&  0.87194E+00_JPRB, 0.93077E+00_JPRB, 0.77415E+00_JPRB, 0.68090E+00_JPRB, 0.75582E+00_JPRB, 0.77794E+00_JPRB, 0.83099E+00_JPRB ,&
&  0.81653E+00_JPRB, 0.88388E+00_JPRB, 0.89418E+00_JPRB, 0.94725E+00_JPRB, 0.10951E+01_JPRB, 0.11357E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.10733E+01_JPRB, 0.10620E+01_JPRB, 0.10771E+01_JPRB, 0.10863E+01_JPRB, 0.99187E+00_JPRB, 0.98459E+00_JPRB ,&
&  0.96733E+00_JPRB, 0.96308E+00_JPRB, 0.95105E+00_JPRB, 0.96367E+00_JPRB, 0.99205E+00_JPRB, 0.10203E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.11258E+01_JPRB, 0.11464E+01_JPRB, 0.12104E+01_JPRB, 0.13110E+01_JPRB, 0.14884E+01_JPRB, 0.16405E+01_JPRB ,&
&  0.19366E+01_JPRB, 0.21928E+01_JPRB, 0.23875E+01_JPRB, 0.28454E+01_JPRB, 0.30198E+01_JPRB, 0.29660E+01_JPRB, 0.28804E+01_JPRB ,&
&  0.28293E+01_JPRB, 0.27573E+01_JPRB, 0.28254E+01_JPRB, 0.28573E+01_JPRB, 0.28866E+01_JPRB, 0.28644E+01_JPRB, 0.26598E+01_JPRB ,&
&  0.23723E+01_JPRB, 0.21777E+01_JPRB, 0.20710E+01_JPRB, 0.19321E+01_JPRB, 0.18090E+01_JPRB, 0.16840E+01_JPRB, 0.16865E+01_JPRB ,&
&  0.18187E+01_JPRB, 0.11040E+01_JPRB, 0.90569E+00_JPRB, 0.10333E+01_JPRB, 0.89923E+00_JPRB, 0.93620E+00_JPRB, 0.10621E+01_JPRB ,&
&  0.10536E+01_JPRB, 0.10421E+01_JPRB, 0.10879E+01_JPRB, 0.11164E+01_JPRB, 0.12433E+01_JPRB, 0.16546E+01_JPRB, 0.17717E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.15410E+01_JPRB, 0.16113E+01_JPRB, 0.15825E+01_JPRB, 0.15347E+01_JPRB, 0.14805E+01_JPRB, 0.13746E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.12199E+01_JPRB, 0.11506E+01_JPRB, 0.10845E+01_JPRB, 0.10354E+01_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.11477E+01_JPRB, 0.11068E+01_JPRB, 0.10754E+01_JPRB, 0.10557E+01_JPRB, 0.10544E+01_JPRB, 0.10729E+01_JPRB, 0.10761E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.10279E+01_JPRB, 0.98560E+00_JPRB, 0.94929E+00_JPRB, 0.92034E+00_JPRB, 0.90637E+00_JPRB, 0.89113E+00_JPRB ,&
&  0.87111E+00_JPRB, 0.93251E+00_JPRB, 0.75384E+00_JPRB, 0.69412E+00_JPRB, 0.65839E+00_JPRB, 0.73570E+00_JPRB, 0.81088E+00_JPRB ,&
&  0.84689E+00_JPRB, 0.89598E+00_JPRB, 0.95608E+00_JPRB, 0.11590E+01_JPRB, 0.13545E+01_JPRB, 0.13411E+01_JPRB, 0.14413E+01_JPRB ,&
&  0.14576E+01_JPRB, 0.14171E+01_JPRB, 0.12717E+01_JPRB, 0.12798E+01_JPRB, 0.11611E+01_JPRB, 0.10522E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11011E+01_JPRB, 0.10928E+01_JPRB, 0.10816E+01_JPRB, 0.10926E+01_JPRB, 0.10933E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.11972E+01_JPRB, 0.12561E+01_JPRB, 0.13064E+01_JPRB, 0.13961E+01_JPRB, 0.15188E+01_JPRB, 0.15607E+01_JPRB ,&
&  0.17597E+01_JPRB, 0.17750E+01_JPRB, 0.15883E+01_JPRB, 0.21136E+01_JPRB, 0.29958E+01_JPRB, 0.30255E+01_JPRB, 0.28146E+01_JPRB ,&
&  0.30018E+01_JPRB, 0.30297E+01_JPRB, 0.31528E+01_JPRB, 0.32889E+01_JPRB, 0.32392E+01_JPRB, 0.30857E+01_JPRB, 0.28395E+01_JPRB ,&
&  0.25105E+01_JPRB, 0.22850E+01_JPRB, 0.21306E+01_JPRB, 0.21497E+01_JPRB, 0.19868E+01_JPRB, 0.19004E+01_JPRB, 0.18378E+01_JPRB ,&
&  0.16314E+01_JPRB, 0.14276E+01_JPRB, 0.10550E+01_JPRB, 0.10836E+01_JPRB, 0.89428E+00_JPRB, 0.92285E+00_JPRB, 0.10542E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.11586E+01_JPRB, 0.12790E+01_JPRB, 0.13527E+01_JPRB, 0.17357E+01_JPRB, 0.21024E+01_JPRB, 0.24234E+01_JPRB ,&
&  0.23807E+01_JPRB, 0.19955E+01_JPRB, 0.20585E+01_JPRB, 0.19721E+01_JPRB, 0.19428E+01_JPRB, 0.18753E+01_JPRB, 0.17770E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.15040E+01_JPRB, 0.13750E+01_JPRB, 0.12706E+01_JPRB, 0.11988E+01_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.12589E+01_JPRB, 0.12043E+01_JPRB, 0.11695E+01_JPRB, 0.11504E+01_JPRB, 0.11489E+01_JPRB, 0.11631E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.10810E+01_JPRB, 0.10303E+01_JPRB, 0.98573E+00_JPRB, 0.95413E+00_JPRB, 0.94035E+00_JPRB, 0.92584E+00_JPRB ,&
&  0.88789E+00_JPRB, 0.83341E+00_JPRB, 0.68863E+00_JPRB, 0.62388E+00_JPRB, 0.60528E+00_JPRB, 0.66262E+00_JPRB, 0.77678E+00_JPRB ,&
&  0.87228E+00_JPRB, 0.95748E+00_JPRB, 0.11439E+01_JPRB, 0.14259E+01_JPRB, 0.16300E+01_JPRB, 0.17086E+01_JPRB, 0.17979E+01_JPRB ,&
&  0.20789E+01_JPRB, 0.16632E+01_JPRB, 0.15850E+01_JPRB, 0.14444E+01_JPRB, 0.13575E+01_JPRB, 0.13059E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.11645E+01_JPRB, 0.11278E+01_JPRB, 0.11231E+01_JPRB, 0.11379E+01_JPRB, 0.11742E+01_JPRB, 0.12613E+01_JPRB ,&
&  0.13493E+01_JPRB, 0.13958E+01_JPRB, 0.13675E+01_JPRB, 0.13691E+01_JPRB, 0.14883E+01_JPRB, 0.15829E+01_JPRB, 0.16015E+01_JPRB ,&
&  0.15272E+01_JPRB, 0.15508E+01_JPRB, 0.20079E+01_JPRB, 0.23933E+01_JPRB, 0.27938E+01_JPRB, 0.30613E+01_JPRB, 0.31444E+01_JPRB ,&
&  0.32689E+01_JPRB, 0.31997E+01_JPRB, 0.32439E+01_JPRB, 0.29316E+01_JPRB, 0.32138E+01_JPRB, 0.31218E+01_JPRB, 0.29170E+01_JPRB ,&
&  0.27013E+01_JPRB, 0.25862E+01_JPRB, 0.23680E+01_JPRB, 0.23046E+01_JPRB, 0.23448E+01_JPRB, 0.23818E+01_JPRB, 0.18828E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.16573E+01_JPRB, 0.13869E+01_JPRB, 0.12199E+01_JPRB, 0.11026E+01_JPRB, 0.11476E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.13994E+01_JPRB, 0.14523E+01_JPRB, 0.16861E+01_JPRB, 0.21280E+01_JPRB, 0.25288E+01_JPRB, 0.27695E+01_JPRB ,&
&  0.25180E+01_JPRB, 0.26245E+01_JPRB, 0.25398E+01_JPRB, 0.24352E+01_JPRB, 0.24140E+01_JPRB, 0.23295E+01_JPRB, 0.21792E+01_JPRB ,&
&  0.19704E+01_JPRB, 0.17701E+01_JPRB, 0.16035E+01_JPRB, 0.14602E+01_JPRB, 0.13433E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.14861E+01_JPRB, 0.14131E+01_JPRB, 0.13609E+01_JPRB, 0.13280E+01_JPRB, 0.13204E+01_JPRB, 0.13234E+01_JPRB, 0.12873E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.11545E+01_JPRB, 0.10973E+01_JPRB, 0.10473E+01_JPRB, 0.10005E+01_JPRB, 0.97118E+00_JPRB, 0.94896E+00_JPRB ,&
&  0.92761E+00_JPRB, 0.80573E+00_JPRB, 0.65132E+00_JPRB, 0.65046E+00_JPRB, 0.67954E+00_JPRB, 0.65764E+00_JPRB, 0.74376E+00_JPRB ,&
&  0.86215E+00_JPRB, 0.10564E+01_JPRB, 0.12932E+01_JPRB, 0.17059E+01_JPRB, 0.19591E+01_JPRB, 0.21310E+01_JPRB, 0.22046E+01_JPRB ,&
&  0.21251E+01_JPRB, 0.19976E+01_JPRB, 0.18237E+01_JPRB, 0.16281E+01_JPRB, 0.14108E+01_JPRB, 0.13368E+01_JPRB, 0.12765E+01_JPRB ,&
&  0.12266E+01_JPRB, 0.11793E+01_JPRB, 0.11686E+01_JPRB, 0.12110E+01_JPRB, 0.12732E+01_JPRB, 0.13524E+01_JPRB, 0.14669E+01_JPRB ,&
&  0.15002E+01_JPRB, 0.14699E+01_JPRB, 0.13642E+01_JPRB, 0.14004E+01_JPRB, 0.13652E+01_JPRB, 0.14087E+01_JPRB, 0.14187E+01_JPRB ,&
&  0.17891E+01_JPRB, 0.19675E+01_JPRB, 0.20109E+01_JPRB, 0.24444E+01_JPRB, 0.23195E+01_JPRB, 0.25264E+01_JPRB, 0.31962E+01_JPRB ,&
&  0.33536E+01_JPRB, 0.33372E+01_JPRB, 0.34337E+01_JPRB, 0.27143E+01_JPRB, 0.22754E+01_JPRB, 0.30874E+01_JPRB, 0.30315E+01_JPRB ,&
&  0.29238E+01_JPRB, 0.29213E+01_JPRB, 0.27776E+01_JPRB, 0.28699E+01_JPRB, 0.23592E+01_JPRB, 0.19838E+01_JPRB, 0.14890E+01_JPRB ,&
&  0.14335E+01_JPRB, 0.13726E+01_JPRB, 0.15104E+01_JPRB, 0.14075E+01_JPRB, 0.14866E+01_JPRB, 0.13755E+01_JPRB, 0.13514E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.16478E+01_JPRB, 0.18463E+01_JPRB, 0.22330E+01_JPRB, 0.27481E+01_JPRB, 0.26555E+01_JPRB, 0.29959E+01_JPRB ,&
&  0.30834E+01_JPRB, 0.30120E+01_JPRB, 0.26598E+01_JPRB, 0.26919E+01_JPRB, 0.25885E+01_JPRB, 0.24754E+01_JPRB, 0.23300E+01_JPRB ,&
&  0.21486E+01_JPRB, 0.19766E+01_JPRB, 0.18303E+01_JPRB, 0.16969E+01_JPRB, 0.15821E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.16862E+01_JPRB, 0.16190E+01_JPRB, 0.15642E+01_JPRB, 0.15292E+01_JPRB, 0.15066E+01_JPRB, 0.14650E+01_JPRB, 0.13751E+01_JPRB ,&
&  0.12745E+01_JPRB, 0.12018E+01_JPRB, 0.11316E+01_JPRB, 0.10731E+01_JPRB, 0.10124E+01_JPRB, 0.98547E+00_JPRB, 0.97662E+00_JPRB ,&
&  0.96271E+00_JPRB, 0.91731E+00_JPRB, 0.72282E+00_JPRB, 0.67198E+00_JPRB, 0.73650E+00_JPRB, 0.70585E+00_JPRB, 0.72100E+00_JPRB ,&
&  0.91256E+00_JPRB, 0.11891E+01_JPRB, 0.15159E+01_JPRB, 0.20486E+01_JPRB, 0.22924E+01_JPRB, 0.25487E+01_JPRB, 0.25918E+01_JPRB ,&
&  0.26548E+01_JPRB, 0.22574E+01_JPRB, 0.19016E+01_JPRB, 0.16251E+01_JPRB, 0.14199E+01_JPRB, 0.13004E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11977E+01_JPRB, 0.12563E+01_JPRB, 0.12844E+01_JPRB, 0.13508E+01_JPRB, 0.14059E+01_JPRB, 0.14805E+01_JPRB ,&
&  0.15032E+01_JPRB, 0.14651E+01_JPRB, 0.14372E+01_JPRB, 0.15191E+01_JPRB, 0.15310E+01_JPRB, 0.14868E+01_JPRB, 0.15525E+01_JPRB ,&
&  0.17208E+01_JPRB, 0.18113E+01_JPRB, 0.19481E+01_JPRB, 0.22425E+01_JPRB, 0.26211E+01_JPRB, 0.26698E+01_JPRB, 0.31050E+01_JPRB ,&
&  0.25561E+01_JPRB, 0.22716E+01_JPRB, 0.19905E+01_JPRB, 0.16993E+01_JPRB, 0.20188E+01_JPRB, 0.31757E+01_JPRB, 0.31917E+01_JPRB ,&
&  0.30488E+01_JPRB, 0.31472E+01_JPRB, 0.31451E+01_JPRB, 0.27401E+01_JPRB, 0.22598E+01_JPRB, 0.18316E+01_JPRB, 0.22164E+01_JPRB ,&
&  0.22720E+01_JPRB, 0.25035E+01_JPRB, 0.22501E+01_JPRB, 0.18105E+01_JPRB, 0.16561E+01_JPRB, 0.20253E+01_JPRB, 0.20832E+01_JPRB ,&
&  0.21812E+01_JPRB, 0.23339E+01_JPRB, 0.32712E+01_JPRB, 0.33939E+01_JPRB, 0.33069E+01_JPRB, 0.32860E+01_JPRB, 0.35636E+01_JPRB ,&
&  0.32909E+01_JPRB, 0.32059E+01_JPRB, 0.30553E+01_JPRB, 0.28399E+01_JPRB, 0.26577E+01_JPRB, 0.24846E+01_JPRB, 0.23265E+01_JPRB ,&
&  0.21993E+01_JPRB, 0.20779E+01_JPRB, 0.19459E+01_JPRB, 0.18445E+01_JPRB, 0.17576E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.18100E+01_JPRB, 0.17507E+01_JPRB, 0.17050E+01_JPRB, 0.16540E+01_JPRB, 0.15922E+01_JPRB, 0.15037E+01_JPRB, 0.13974E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.12259E+01_JPRB, 0.11599E+01_JPRB, 0.11036E+01_JPRB, 0.10449E+01_JPRB, 0.10121E+01_JPRB, 0.99592E+00_JPRB ,&
&  0.99389E+00_JPRB, 0.11236E+01_JPRB, 0.87007E+00_JPRB, 0.74211E+00_JPRB, 0.76319E+00_JPRB, 0.77099E+00_JPRB, 0.74226E+00_JPRB ,&
&  0.99737E+00_JPRB, 0.13579E+01_JPRB, 0.16390E+01_JPRB, 0.22091E+01_JPRB, 0.28065E+01_JPRB, 0.26844E+01_JPRB, 0.26403E+01_JPRB ,&
&  0.25642E+01_JPRB, 0.21674E+01_JPRB, 0.18688E+01_JPRB, 0.16535E+01_JPRB, 0.14709E+01_JPRB, 0.13683E+01_JPRB, 0.13086E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.13490E+01_JPRB, 0.13448E+01_JPRB, 0.13617E+01_JPRB, 0.13896E+01_JPRB, 0.14725E+01_JPRB, 0.15543E+01_JPRB ,&
&  0.15213E+01_JPRB, 0.14869E+01_JPRB, 0.15193E+01_JPRB, 0.16435E+01_JPRB, 0.16941E+01_JPRB, 0.16110E+01_JPRB, 0.18312E+01_JPRB ,&
&  0.17921E+01_JPRB, 0.19301E+01_JPRB, 0.19654E+01_JPRB, 0.22906E+01_JPRB, 0.25232E+01_JPRB, 0.25781E+01_JPRB, 0.28011E+01_JPRB ,&
&  0.21780E+01_JPRB, 0.19818E+01_JPRB, 0.20489E+01_JPRB, 0.20749E+01_JPRB, 0.19692E+01_JPRB, 0.22974E+01_JPRB, 0.33700E+01_JPRB ,&
&  0.27321E+01_JPRB, 0.29463E+01_JPRB, 0.29948E+01_JPRB, 0.29700E+01_JPRB, 0.16522E+01_JPRB, 0.13593E+01_JPRB, 0.22878E+01_JPRB ,&
&  0.25701E+01_JPRB, 0.16713E+01_JPRB, 0.11963E+01_JPRB, 0.14067E+01_JPRB, 0.11514E+01_JPRB, 0.16342E+01_JPRB, 0.25539E+01_JPRB ,&
&  0.30215E+01_JPRB, 0.33712E+01_JPRB, 0.47127E+01_JPRB, 0.42255E+01_JPRB, 0.38721E+01_JPRB, 0.43251E+01_JPRB, 0.38580E+01_JPRB ,&
&  0.35653E+01_JPRB, 0.30606E+01_JPRB, 0.30293E+01_JPRB, 0.27007E+01_JPRB, 0.24808E+01_JPRB, 0.23426E+01_JPRB, 0.22674E+01_JPRB ,&
&  0.22224E+01_JPRB, 0.21644E+01_JPRB, 0.20819E+01_JPRB, 0.19835E+01_JPRB, 0.18828E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.18464E+01_JPRB, 0.18023E+01_JPRB, 0.17642E+01_JPRB, 0.17020E+01_JPRB, 0.16154E+01_JPRB, 0.15091E+01_JPRB, 0.14061E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.12589E+01_JPRB, 0.11949E+01_JPRB, 0.11402E+01_JPRB, 0.10843E+01_JPRB, 0.10416E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.99393E+00_JPRB, 0.10276E+01_JPRB, 0.10805E+01_JPRB, 0.95923E+00_JPRB, 0.91997E+00_JPRB, 0.93063E+00_JPRB, 0.94839E+00_JPRB ,&
&  0.12362E+01_JPRB, 0.15661E+01_JPRB, 0.18891E+01_JPRB, 0.23109E+01_JPRB, 0.26403E+01_JPRB, 0.25383E+01_JPRB, 0.26147E+01_JPRB ,&
&  0.24158E+01_JPRB, 0.21145E+01_JPRB, 0.18621E+01_JPRB, 0.16835E+01_JPRB, 0.15601E+01_JPRB, 0.14844E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.14202E+01_JPRB, 0.14106E+01_JPRB, 0.13688E+01_JPRB, 0.13381E+01_JPRB, 0.14018E+01_JPRB, 0.15254E+01_JPRB, 0.15830E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.15389E+01_JPRB, 0.15682E+01_JPRB, 0.16634E+01_JPRB, 0.17920E+01_JPRB, 0.17791E+01_JPRB, 0.15918E+01_JPRB ,&
&  0.14972E+01_JPRB, 0.16178E+01_JPRB, 0.20522E+01_JPRB, 0.21616E+01_JPRB, 0.22808E+01_JPRB, 0.24558E+01_JPRB, 0.25700E+01_JPRB ,&
&  0.24963E+01_JPRB, 0.24738E+01_JPRB, 0.22185E+01_JPRB, 0.23688E+01_JPRB, 0.23470E+01_JPRB, 0.21802E+01_JPRB, 0.28638E+01_JPRB ,&
&  0.29150E+01_JPRB, 0.27274E+01_JPRB, 0.22231E+01_JPRB, 0.17859E+01_JPRB, 0.18374E+01_JPRB, 0.15406E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.10501E+01_JPRB, 0.93760E+00_JPRB, 0.92534E+00_JPRB, 0.90175E+00_JPRB, 0.14850E+01_JPRB, 0.26663E+01_JPRB ,&
&  0.41672E+01_JPRB, 0.50199E+01_JPRB, 0.55216E+01_JPRB, 0.50139E+01_JPRB, 0.43223E+01_JPRB, 0.46152E+01_JPRB, 0.41284E+01_JPRB ,&
&  0.31864E+01_JPRB, 0.28620E+01_JPRB, 0.27725E+01_JPRB, 0.25540E+01_JPRB, 0.24393E+01_JPRB, 0.23392E+01_JPRB, 0.22527E+01_JPRB ,&
&  0.21713E+01_JPRB, 0.21101E+01_JPRB, 0.20457E+01_JPRB, 0.19710E+01_JPRB, 0.19056E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.18376E+01_JPRB, 0.17977E+01_JPRB, 0.17455E+01_JPRB, 0.16706E+01_JPRB, 0.15774E+01_JPRB, 0.14871E+01_JPRB, 0.14170E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13211E+01_JPRB, 0.12539E+01_JPRB, 0.11910E+01_JPRB, 0.11277E+01_JPRB, 0.10728E+01_JPRB, 0.10407E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.10171E+01_JPRB, 0.11421E+01_JPRB, 0.11200E+01_JPRB, 0.12019E+01_JPRB, 0.13108E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.16686E+01_JPRB, 0.19696E+01_JPRB, 0.23472E+01_JPRB, 0.25387E+01_JPRB, 0.27691E+01_JPRB, 0.28121E+01_JPRB, 0.26937E+01_JPRB ,&
&  0.24102E+01_JPRB, 0.21883E+01_JPRB, 0.19857E+01_JPRB, 0.18316E+01_JPRB, 0.16819E+01_JPRB, 0.15699E+01_JPRB, 0.14957E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.14398E+01_JPRB, 0.13643E+01_JPRB, 0.13539E+01_JPRB, 0.14333E+01_JPRB, 0.15303E+01_JPRB, 0.15881E+01_JPRB ,&
&  0.16071E+01_JPRB, 0.16167E+01_JPRB, 0.16612E+01_JPRB, 0.17512E+01_JPRB, 0.16036E+01_JPRB, 0.14152E+01_JPRB, 0.15741E+01_JPRB ,&
&  0.17268E+01_JPRB, 0.19722E+01_JPRB, 0.19815E+01_JPRB, 0.23224E+01_JPRB, 0.23485E+01_JPRB, 0.23185E+01_JPRB, 0.25330E+01_JPRB ,&
&  0.25677E+01_JPRB, 0.27666E+01_JPRB, 0.22106E+01_JPRB, 0.22360E+01_JPRB, 0.27905E+01_JPRB, 0.24630E+01_JPRB, 0.25373E+01_JPRB ,&
&  0.28034E+01_JPRB, 0.24739E+01_JPRB, 0.22147E+01_JPRB, 0.20586E+01_JPRB, 0.29669E+01_JPRB, 0.30440E+01_JPRB, 0.14956E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.11019E+01_JPRB, 0.10044E+01_JPRB, 0.90570E+00_JPRB, 0.92724E+00_JPRB, 0.16207E+01_JPRB, 0.39280E+01_JPRB ,&
&  0.45553E+01_JPRB, 0.66165E+01_JPRB, 0.66992E+01_JPRB, 0.59908E+01_JPRB, 0.48153E+01_JPRB, 0.45877E+01_JPRB, 0.36365E+01_JPRB ,&
&  0.31666E+01_JPRB, 0.29348E+01_JPRB, 0.26902E+01_JPRB, 0.24852E+01_JPRB, 0.22791E+01_JPRB, 0.21536E+01_JPRB, 0.20914E+01_JPRB ,&
&  0.20389E+01_JPRB, 0.19928E+01_JPRB, 0.19567E+01_JPRB, 0.19175E+01_JPRB, 0.18809E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.17255E+01_JPRB, 0.16738E+01_JPRB, 0.16186E+01_JPRB, 0.15501E+01_JPRB, 0.14892E+01_JPRB, 0.14492E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.13643E+01_JPRB, 0.12983E+01_JPRB, 0.12226E+01_JPRB, 0.11503E+01_JPRB, 0.10906E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10383E+01_JPRB, 0.11114E+01_JPRB, 0.12623E+01_JPRB, 0.14466E+01_JPRB, 0.15387E+01_JPRB, 0.18374E+01_JPRB ,&
&  0.22688E+01_JPRB, 0.26298E+01_JPRB, 0.30040E+01_JPRB, 0.30837E+01_JPRB, 0.30864E+01_JPRB, 0.29686E+01_JPRB, 0.27391E+01_JPRB ,&
&  0.25063E+01_JPRB, 0.23441E+01_JPRB, 0.21524E+01_JPRB, 0.19583E+01_JPRB, 0.17820E+01_JPRB, 0.16545E+01_JPRB, 0.15613E+01_JPRB ,&
&  0.15334E+01_JPRB, 0.14477E+01_JPRB, 0.13723E+01_JPRB, 0.13742E+01_JPRB, 0.14503E+01_JPRB, 0.15604E+01_JPRB, 0.16373E+01_JPRB ,&
&  0.16701E+01_JPRB, 0.16721E+01_JPRB, 0.17099E+01_JPRB, 0.17836E+01_JPRB, 0.15005E+01_JPRB, 0.16911E+01_JPRB, 0.17221E+01_JPRB ,&
&  0.17837E+01_JPRB, 0.18851E+01_JPRB, 0.18814E+01_JPRB, 0.21736E+01_JPRB, 0.22955E+01_JPRB, 0.22692E+01_JPRB, 0.23243E+01_JPRB ,&
&  0.25230E+01_JPRB, 0.22719E+01_JPRB, 0.20583E+01_JPRB, 0.22892E+01_JPRB, 0.26637E+01_JPRB, 0.33609E+01_JPRB, 0.23377E+01_JPRB ,&
&  0.21048E+01_JPRB, 0.24744E+01_JPRB, 0.24284E+01_JPRB, 0.23423E+01_JPRB, 0.30366E+01_JPRB, 0.37623E+01_JPRB, 0.32165E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.10690E+01_JPRB, 0.11309E+01_JPRB, 0.11705E+01_JPRB, 0.11455E+01_JPRB, 0.18465E+01_JPRB, 0.63911E+01_JPRB ,&
&  0.56769E+01_JPRB, 0.74152E+01_JPRB, 0.69115E+01_JPRB, 0.58433E+01_JPRB, 0.48065E+01_JPRB, 0.42538E+01_JPRB, 0.36635E+01_JPRB ,&
&  0.30537E+01_JPRB, 0.26846E+01_JPRB, 0.23782E+01_JPRB, 0.21662E+01_JPRB, 0.20463E+01_JPRB, 0.19596E+01_JPRB, 0.19045E+01_JPRB ,&
&  0.18571E+01_JPRB, 0.18216E+01_JPRB, 0.18026E+01_JPRB, 0.17917E+01_JPRB, 0.17695E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.16092E+01_JPRB, 0.15467E+01_JPRB, 0.14854E+01_JPRB, 0.14525E+01_JPRB, 0.14355E+01_JPRB, 0.14146E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.13369E+01_JPRB, 0.12800E+01_JPRB, 0.12093E+01_JPRB, 0.11409E+01_JPRB, 0.10905E+01_JPRB, 0.10620E+01_JPRB ,&
&  0.10588E+01_JPRB, 0.10619E+01_JPRB, 0.11091E+01_JPRB, 0.12848E+01_JPRB, 0.15859E+01_JPRB, 0.16386E+01_JPRB, 0.19663E+01_JPRB ,&
&  0.31590E+01_JPRB, 0.37082E+01_JPRB, 0.35181E+01_JPRB, 0.34642E+01_JPRB, 0.34298E+01_JPRB, 0.34243E+01_JPRB, 0.30572E+01_JPRB ,&
&  0.27038E+01_JPRB, 0.23962E+01_JPRB, 0.21170E+01_JPRB, 0.19102E+01_JPRB, 0.17490E+01_JPRB, 0.16392E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.15070E+01_JPRB, 0.14316E+01_JPRB, 0.13762E+01_JPRB, 0.13886E+01_JPRB, 0.14855E+01_JPRB, 0.15891E+01_JPRB, 0.16720E+01_JPRB ,&
&  0.16920E+01_JPRB, 0.16909E+01_JPRB, 0.17005E+01_JPRB, 0.16646E+01_JPRB, 0.16848E+01_JPRB, 0.17217E+01_JPRB, 0.17895E+01_JPRB ,&
&  0.18007E+01_JPRB, 0.19054E+01_JPRB, 0.20253E+01_JPRB, 0.20794E+01_JPRB, 0.21571E+01_JPRB, 0.22235E+01_JPRB, 0.22261E+01_JPRB ,&
&  0.22992E+01_JPRB, 0.21549E+01_JPRB, 0.20815E+01_JPRB, 0.24374E+01_JPRB, 0.27172E+01_JPRB, 0.30485E+01_JPRB, 0.29683E+01_JPRB ,&
&  0.25807E+01_JPRB, 0.21372E+01_JPRB, 0.21725E+01_JPRB, 0.25519E+01_JPRB, 0.29178E+01_JPRB, 0.32768E+01_JPRB, 0.48541E+01_JPRB ,&
&  0.48355E+01_JPRB, 0.32513E+01_JPRB, 0.27373E+01_JPRB, 0.35763E+01_JPRB, 0.30709E+01_JPRB, 0.29830E+01_JPRB, 0.44158E+01_JPRB ,&
&  0.58376E+01_JPRB, 0.72619E+01_JPRB, 0.66036E+01_JPRB, 0.52013E+01_JPRB, 0.45919E+01_JPRB, 0.39551E+01_JPRB, 0.32849E+01_JPRB ,&
&  0.27113E+01_JPRB, 0.24108E+01_JPRB, 0.22014E+01_JPRB, 0.19997E+01_JPRB, 0.18406E+01_JPRB, 0.17589E+01_JPRB, 0.17242E+01_JPRB ,&
&  0.16944E+01_JPRB, 0.16821E+01_JPRB, 0.16821E+01_JPRB, 0.16833E+01_JPRB, 0.16556E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.14686E+01_JPRB, 0.14253E+01_JPRB, 0.14073E+01_JPRB, 0.13753E+01_JPRB, 0.13412E+01_JPRB, 0.13153E+01_JPRB, 0.13079E+01_JPRB ,&
&  0.12969E+01_JPRB, 0.12565E+01_JPRB, 0.12105E+01_JPRB, 0.11571E+01_JPRB, 0.10986E+01_JPRB, 0.10651E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.10774E+01_JPRB, 0.11663E+01_JPRB, 0.13883E+01_JPRB, 0.17218E+01_JPRB, 0.22535E+01_JPRB, 0.20635E+01_JPRB ,&
&  0.31278E+01_JPRB, 0.43363E+01_JPRB, 0.41551E+01_JPRB, 0.39724E+01_JPRB, 0.37647E+01_JPRB, 0.36135E+01_JPRB, 0.31233E+01_JPRB ,&
&  0.26760E+01_JPRB, 0.23522E+01_JPRB, 0.20443E+01_JPRB, 0.18321E+01_JPRB, 0.17037E+01_JPRB, 0.16171E+01_JPRB, 0.15277E+01_JPRB ,&
&  0.14600E+01_JPRB, 0.13948E+01_JPRB, 0.13605E+01_JPRB, 0.13886E+01_JPRB, 0.14627E+01_JPRB, 0.15677E+01_JPRB, 0.16391E+01_JPRB ,&
&  0.16695E+01_JPRB, 0.16721E+01_JPRB, 0.16690E+01_JPRB, 0.15878E+01_JPRB, 0.16426E+01_JPRB, 0.16955E+01_JPRB, 0.17525E+01_JPRB ,&
&  0.16959E+01_JPRB, 0.16339E+01_JPRB, 0.18514E+01_JPRB, 0.20453E+01_JPRB, 0.21162E+01_JPRB, 0.20524E+01_JPRB, 0.20043E+01_JPRB ,&
&  0.21822E+01_JPRB, 0.22981E+01_JPRB, 0.25982E+01_JPRB, 0.26457E+01_JPRB, 0.27883E+01_JPRB, 0.29469E+01_JPRB, 0.30747E+01_JPRB ,&
&  0.31367E+01_JPRB, 0.27646E+01_JPRB, 0.26620E+01_JPRB, 0.27230E+01_JPRB, 0.27294E+01_JPRB, 0.30445E+01_JPRB, 0.42580E+01_JPRB ,&
&  0.55116E+01_JPRB, 0.63611E+01_JPRB, 0.65962E+01_JPRB, 0.55756E+01_JPRB, 0.47719E+01_JPRB, 0.43249E+01_JPRB, 0.45214E+01_JPRB ,&
&  0.61932E+01_JPRB, 0.65608E+01_JPRB, 0.56828E+01_JPRB, 0.54924E+01_JPRB, 0.41282E+01_JPRB, 0.36343E+01_JPRB, 0.30610E+01_JPRB ,&
&  0.26365E+01_JPRB, 0.23095E+01_JPRB, 0.20487E+01_JPRB, 0.18574E+01_JPRB, 0.17260E+01_JPRB, 0.16221E+01_JPRB, 0.15635E+01_JPRB ,&
&  0.15367E+01_JPRB, 0.15487E+01_JPRB, 0.15652E+01_JPRB, 0.15590E+01_JPRB, 0.15174E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.13846E+01_JPRB, 0.13665E+01_JPRB, 0.13457E+01_JPRB, 0.13020E+01_JPRB, 0.12696E+01_JPRB, 0.12486E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.11947E+01_JPRB, 0.11745E+01_JPRB, 0.11354E+01_JPRB, 0.10902E+01_JPRB, 0.10542E+01_JPRB, 0.10307E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.11952E+01_JPRB, 0.13931E+01_JPRB, 0.16175E+01_JPRB, 0.19787E+01_JPRB, 0.25286E+01_JPRB, 0.28889E+01_JPRB ,&
&  0.32935E+01_JPRB, 0.51674E+01_JPRB, 0.43542E+01_JPRB, 0.40086E+01_JPRB, 0.37607E+01_JPRB, 0.34932E+01_JPRB, 0.29428E+01_JPRB ,&
&  0.23506E+01_JPRB, 0.20905E+01_JPRB, 0.19165E+01_JPRB, 0.17633E+01_JPRB, 0.16623E+01_JPRB, 0.15428E+01_JPRB, 0.14770E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.13904E+01_JPRB, 0.13642E+01_JPRB, 0.14013E+01_JPRB, 0.14813E+01_JPRB, 0.15534E+01_JPRB, 0.16224E+01_JPRB ,&
&  0.16469E+01_JPRB, 0.16369E+01_JPRB, 0.15837E+01_JPRB, 0.15556E+01_JPRB, 0.15662E+01_JPRB, 0.16238E+01_JPRB, 0.16883E+01_JPRB ,&
&  0.16828E+01_JPRB, 0.16495E+01_JPRB, 0.17815E+01_JPRB, 0.18245E+01_JPRB, 0.18217E+01_JPRB, 0.19265E+01_JPRB, 0.18788E+01_JPRB ,&
&  0.21320E+01_JPRB, 0.20868E+01_JPRB, 0.27184E+01_JPRB, 0.24685E+01_JPRB, 0.25295E+01_JPRB, 0.27238E+01_JPRB, 0.26815E+01_JPRB ,&
&  0.24367E+01_JPRB, 0.24709E+01_JPRB, 0.24357E+01_JPRB, 0.25537E+01_JPRB, 0.28448E+01_JPRB, 0.33848E+01_JPRB, 0.44642E+01_JPRB ,&
&  0.49403E+01_JPRB, 0.48856E+01_JPRB, 0.49884E+01_JPRB, 0.41885E+01_JPRB, 0.40414E+01_JPRB, 0.39260E+01_JPRB, 0.42727E+01_JPRB ,&
&  0.57974E+01_JPRB, 0.58414E+01_JPRB, 0.52811E+01_JPRB, 0.45665E+01_JPRB, 0.35100E+01_JPRB, 0.30494E+01_JPRB, 0.25691E+01_JPRB ,&
&  0.22481E+01_JPRB, 0.19716E+01_JPRB, 0.18126E+01_JPRB, 0.16869E+01_JPRB, 0.15934E+01_JPRB, 0.15023E+01_JPRB, 0.14630E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.14510E+01_JPRB, 0.14552E+01_JPRB, 0.14375E+01_JPRB, 0.14089E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.13663E+01_JPRB, 0.13716E+01_JPRB, 0.13713E+01_JPRB, 0.13925E+01_JPRB, 0.14371E+01_JPRB, 0.15028E+01_JPRB, 0.15345E+01_JPRB ,&
&  0.13885E+01_JPRB, 0.11027E+01_JPRB, 0.10788E+01_JPRB, 0.10512E+01_JPRB, 0.10221E+01_JPRB, 0.10262E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.12054E+01_JPRB, 0.13565E+01_JPRB, 0.15347E+01_JPRB, 0.18285E+01_JPRB, 0.22547E+01_JPRB, 0.29629E+01_JPRB, 0.37483E+01_JPRB ,&
&  0.39704E+01_JPRB, 0.43387E+01_JPRB, 0.43211E+01_JPRB, 0.36924E+01_JPRB, 0.33266E+01_JPRB, 0.31317E+01_JPRB, 0.28663E+01_JPRB ,&
&  0.23199E+01_JPRB, 0.20264E+01_JPRB, 0.19493E+01_JPRB, 0.19159E+01_JPRB, 0.15310E+01_JPRB, 0.14635E+01_JPRB, 0.14319E+01_JPRB ,&
&  0.14170E+01_JPRB, 0.14159E+01_JPRB, 0.14202E+01_JPRB, 0.14729E+01_JPRB, 0.15358E+01_JPRB, 0.15910E+01_JPRB, 0.16340E+01_JPRB ,&
&  0.16495E+01_JPRB, 0.16335E+01_JPRB, 0.15999E+01_JPRB, 0.15098E+01_JPRB, 0.15172E+01_JPRB, 0.15537E+01_JPRB, 0.15974E+01_JPRB ,&
&  0.16311E+01_JPRB, 0.17000E+01_JPRB, 0.17429E+01_JPRB, 0.17248E+01_JPRB, 0.16551E+01_JPRB, 0.18541E+01_JPRB, 0.18974E+01_JPRB ,&
&  0.20601E+01_JPRB, 0.22304E+01_JPRB, 0.28873E+01_JPRB, 0.21948E+01_JPRB, 0.23137E+01_JPRB, 0.24660E+01_JPRB, 0.24398E+01_JPRB ,&
&  0.22697E+01_JPRB, 0.22777E+01_JPRB, 0.22401E+01_JPRB, 0.23731E+01_JPRB, 0.29443E+01_JPRB, 0.36441E+01_JPRB, 0.49570E+01_JPRB ,&
&  0.42440E+01_JPRB, 0.43404E+01_JPRB, 0.37527E+01_JPRB, 0.37965E+01_JPRB, 0.37378E+01_JPRB, 0.36290E+01_JPRB, 0.36569E+01_JPRB ,&
&  0.46459E+01_JPRB, 0.42581E+01_JPRB, 0.37039E+01_JPRB, 0.31404E+01_JPRB, 0.24567E+01_JPRB, 0.21070E+01_JPRB, 0.17882E+01_JPRB ,&
&  0.16333E+01_JPRB, 0.15534E+01_JPRB, 0.15403E+01_JPRB, 0.14917E+01_JPRB, 0.14595E+01_JPRB, 0.14267E+01_JPRB, 0.14171E+01_JPRB ,&
&  0.14018E+01_JPRB, 0.13927E+01_JPRB, 0.13805E+01_JPRB, 0.13691E+01_JPRB, 0.13606E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.13917E+01_JPRB, 0.13778E+01_JPRB, 0.13653E+01_JPRB, 0.13577E+01_JPRB, 0.13493E+01_JPRB, 0.13178E+01_JPRB, 0.12509E+01_JPRB ,&
&  0.11582E+01_JPRB, 0.10893E+01_JPRB, 0.10699E+01_JPRB, 0.10558E+01_JPRB, 0.10594E+01_JPRB, 0.11022E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.12839E+01_JPRB, 0.14550E+01_JPRB, 0.16891E+01_JPRB, 0.20060E+01_JPRB, 0.24204E+01_JPRB, 0.30844E+01_JPRB, 0.39016E+01_JPRB ,&
&  0.40770E+01_JPRB, 0.30503E+01_JPRB, 0.34399E+01_JPRB, 0.33036E+01_JPRB, 0.30774E+01_JPRB, 0.28338E+01_JPRB, 0.23924E+01_JPRB ,&
&  0.19652E+01_JPRB, 0.18361E+01_JPRB, 0.17797E+01_JPRB, 0.16598E+01_JPRB, 0.14448E+01_JPRB, 0.14482E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.14871E+01_JPRB, 0.15059E+01_JPRB, 0.15086E+01_JPRB, 0.15576E+01_JPRB, 0.15940E+01_JPRB, 0.16445E+01_JPRB, 0.17058E+01_JPRB ,&
&  0.17250E+01_JPRB, 0.17193E+01_JPRB, 0.16958E+01_JPRB, 0.16065E+01_JPRB, 0.15886E+01_JPRB, 0.16218E+01_JPRB, 0.16467E+01_JPRB ,&
&  0.16676E+01_JPRB, 0.16526E+01_JPRB, 0.17096E+01_JPRB, 0.18264E+01_JPRB, 0.19638E+01_JPRB, 0.18407E+01_JPRB, 0.20529E+01_JPRB ,&
&  0.21891E+01_JPRB, 0.24383E+01_JPRB, 0.23828E+01_JPRB, 0.26780E+01_JPRB, 0.18833E+01_JPRB, 0.20893E+01_JPRB, 0.20035E+01_JPRB ,&
&  0.21857E+01_JPRB, 0.21307E+01_JPRB, 0.22516E+01_JPRB, 0.26373E+01_JPRB, 0.33956E+01_JPRB, 0.42938E+01_JPRB, 0.45801E+01_JPRB ,&
&  0.33470E+01_JPRB, 0.30714E+01_JPRB, 0.32828E+01_JPRB, 0.37254E+01_JPRB, 0.37903E+01_JPRB, 0.37020E+01_JPRB, 0.32549E+01_JPRB ,&
&  0.33896E+01_JPRB, 0.29250E+01_JPRB, 0.25741E+01_JPRB, 0.21839E+01_JPRB, 0.16387E+01_JPRB, 0.14375E+01_JPRB, 0.13841E+01_JPRB ,&
&  0.13947E+01_JPRB, 0.14166E+01_JPRB, 0.14249E+01_JPRB, 0.14307E+01_JPRB, 0.14127E+01_JPRB, 0.13920E+01_JPRB, 0.13785E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.13730E+01_JPRB, 0.13731E+01_JPRB, 0.13723E+01_JPRB, 0.13817E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.13333E+01_JPRB, 0.13049E+01_JPRB, 0.12792E+01_JPRB, 0.12557E+01_JPRB, 0.12395E+01_JPRB, 0.11964E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.10855E+01_JPRB, 0.10838E+01_JPRB, 0.10960E+01_JPRB, 0.11321E+01_JPRB, 0.11734E+01_JPRB, 0.12479E+01_JPRB ,&
&  0.13763E+01_JPRB, 0.15594E+01_JPRB, 0.18043E+01_JPRB, 0.21022E+01_JPRB, 0.24805E+01_JPRB, 0.29398E+01_JPRB, 0.33328E+01_JPRB ,&
&  0.37371E+01_JPRB, 0.38647E+01_JPRB, 0.32260E+01_JPRB, 0.23130E+01_JPRB, 0.21791E+01_JPRB, 0.21576E+01_JPRB, 0.18135E+01_JPRB ,&
&  0.17211E+01_JPRB, 0.16394E+01_JPRB, 0.15716E+01_JPRB, 0.15230E+01_JPRB, 0.15078E+01_JPRB, 0.15267E+01_JPRB, 0.15585E+01_JPRB ,&
&  0.15831E+01_JPRB, 0.16002E+01_JPRB, 0.16261E+01_JPRB, 0.16866E+01_JPRB, 0.17336E+01_JPRB, 0.17869E+01_JPRB, 0.18312E+01_JPRB ,&
&  0.18523E+01_JPRB, 0.18809E+01_JPRB, 0.18565E+01_JPRB, 0.18134E+01_JPRB, 0.17615E+01_JPRB, 0.17882E+01_JPRB, 0.18255E+01_JPRB ,&
&  0.18603E+01_JPRB, 0.18609E+01_JPRB, 0.18831E+01_JPRB, 0.20282E+01_JPRB, 0.21471E+01_JPRB, 0.19855E+01_JPRB, 0.20693E+01_JPRB ,&
&  0.23559E+01_JPRB, 0.25543E+01_JPRB, 0.20532E+01_JPRB, 0.23898E+01_JPRB, 0.15412E+01_JPRB, 0.19210E+01_JPRB, 0.21613E+01_JPRB ,&
&  0.21589E+01_JPRB, 0.22903E+01_JPRB, 0.24874E+01_JPRB, 0.29896E+01_JPRB, 0.36986E+01_JPRB, 0.42080E+01_JPRB, 0.37516E+01_JPRB ,&
&  0.27353E+01_JPRB, 0.26815E+01_JPRB, 0.29266E+01_JPRB, 0.31786E+01_JPRB, 0.32472E+01_JPRB, 0.30287E+01_JPRB, 0.23156E+01_JPRB ,&
&  0.18302E+01_JPRB, 0.20893E+01_JPRB, 0.20307E+01_JPRB, 0.19275E+01_JPRB, 0.14172E+01_JPRB, 0.12309E+01_JPRB, 0.12876E+01_JPRB ,&
&  0.13600E+01_JPRB, 0.14118E+01_JPRB, 0.14354E+01_JPRB, 0.14295E+01_JPRB, 0.14058E+01_JPRB, 0.13846E+01_JPRB, 0.13798E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.13675E+01_JPRB, 0.13643E+01_JPRB, 0.13662E+01_JPRB, 0.13573E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.12172E+01_JPRB, 0.11788E+01_JPRB, 0.11557E+01_JPRB, 0.11400E+01_JPRB, 0.11361E+01_JPRB, 0.11097E+01_JPRB, 0.10675E+01_JPRB ,&
&  0.10427E+01_JPRB, 0.10354E+01_JPRB, 0.10521E+01_JPRB, 0.10939E+01_JPRB, 0.11513E+01_JPRB, 0.12086E+01_JPRB, 0.13154E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.16491E+01_JPRB, 0.18979E+01_JPRB, 0.22605E+01_JPRB, 0.25657E+01_JPRB, 0.28616E+01_JPRB, 0.29631E+01_JPRB ,&
&  0.31151E+01_JPRB, 0.31499E+01_JPRB, 0.29389E+01_JPRB, 0.27140E+01_JPRB, 0.21345E+01_JPRB, 0.20949E+01_JPRB, 0.21927E+01_JPRB ,&
&  0.22736E+01_JPRB, 0.18875E+01_JPRB, 0.16245E+01_JPRB, 0.15996E+01_JPRB, 0.15912E+01_JPRB, 0.16269E+01_JPRB, 0.16699E+01_JPRB ,&
&  0.17078E+01_JPRB, 0.17424E+01_JPRB, 0.17747E+01_JPRB, 0.18293E+01_JPRB, 0.18877E+01_JPRB, 0.19130E+01_JPRB, 0.19462E+01_JPRB ,&
&  0.19527E+01_JPRB, 0.19702E+01_JPRB, 0.19769E+01_JPRB, 0.19564E+01_JPRB, 0.19819E+01_JPRB, 0.20350E+01_JPRB, 0.21268E+01_JPRB ,&
&  0.22202E+01_JPRB, 0.21718E+01_JPRB, 0.22952E+01_JPRB, 0.24790E+01_JPRB, 0.24893E+01_JPRB, 0.24160E+01_JPRB, 0.24359E+01_JPRB ,&
&  0.26520E+01_JPRB, 0.28633E+01_JPRB, 0.17853E+01_JPRB, 0.20078E+01_JPRB, 0.23720E+01_JPRB, 0.21555E+01_JPRB, 0.21041E+01_JPRB ,&
&  0.22584E+01_JPRB, 0.24609E+01_JPRB, 0.27655E+01_JPRB, 0.31320E+01_JPRB, 0.37272E+01_JPRB, 0.47788E+01_JPRB, 0.33321E+01_JPRB ,&
&  0.22918E+01_JPRB, 0.22166E+01_JPRB, 0.23663E+01_JPRB, 0.24853E+01_JPRB, 0.24933E+01_JPRB, 0.21846E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.14009E+01_JPRB, 0.18643E+01_JPRB, 0.19336E+01_JPRB, 0.19719E+01_JPRB, 0.17047E+01_JPRB, 0.11676E+01_JPRB, 0.12875E+01_JPRB ,&
&  0.13498E+01_JPRB, 0.13884E+01_JPRB, 0.13754E+01_JPRB, 0.13727E+01_JPRB, 0.13699E+01_JPRB, 0.13612E+01_JPRB, 0.13598E+01_JPRB ,&
&  0.13566E+01_JPRB, 0.13534E+01_JPRB, 0.13517E+01_JPRB, 0.13190E+01_JPRB, 0.12617E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.10876E+01_JPRB, 0.10581E+01_JPRB, 0.10425E+01_JPRB, 0.10198E+01_JPRB, 0.10051E+01_JPRB, 0.98477E+00_JPRB, 0.97422E+00_JPRB ,&
&  0.97334E+00_JPRB, 0.98067E+00_JPRB, 0.10074E+01_JPRB, 0.10493E+01_JPRB, 0.11085E+01_JPRB, 0.11897E+01_JPRB, 0.13320E+01_JPRB ,&
&  0.14761E+01_JPRB, 0.16968E+01_JPRB, 0.19683E+01_JPRB, 0.22867E+01_JPRB, 0.25374E+01_JPRB, 0.26369E+01_JPRB, 0.27826E+01_JPRB ,&
&  0.28816E+01_JPRB, 0.29089E+01_JPRB, 0.28506E+01_JPRB, 0.27656E+01_JPRB, 0.26481E+01_JPRB, 0.26520E+01_JPRB, 0.28131E+01_JPRB ,&
&  0.25816E+01_JPRB, 0.19454E+01_JPRB, 0.17624E+01_JPRB, 0.16799E+01_JPRB, 0.16895E+01_JPRB, 0.17219E+01_JPRB, 0.17726E+01_JPRB ,&
&  0.18121E+01_JPRB, 0.18387E+01_JPRB, 0.18634E+01_JPRB, 0.18963E+01_JPRB, 0.19712E+01_JPRB, 0.20085E+01_JPRB, 0.20560E+01_JPRB ,&
&  0.20955E+01_JPRB, 0.21396E+01_JPRB, 0.22218E+01_JPRB, 0.21040E+01_JPRB, 0.22554E+01_JPRB, 0.23735E+01_JPRB, 0.24821E+01_JPRB ,&
&  0.25072E+01_JPRB, 0.24736E+01_JPRB, 0.26479E+01_JPRB, 0.27625E+01_JPRB, 0.28646E+01_JPRB, 0.29109E+01_JPRB, 0.28739E+01_JPRB ,&
&  0.29921E+01_JPRB, 0.28208E+01_JPRB, 0.17521E+01_JPRB, 0.16600E+01_JPRB, 0.18478E+01_JPRB, 0.17638E+01_JPRB, 0.21531E+01_JPRB ,&
&  0.23083E+01_JPRB, 0.25371E+01_JPRB, 0.27842E+01_JPRB, 0.31145E+01_JPRB, 0.35251E+01_JPRB, 0.43874E+01_JPRB, 0.29951E+01_JPRB ,&
&  0.17716E+01_JPRB, 0.16862E+01_JPRB, 0.17313E+01_JPRB, 0.17830E+01_JPRB, 0.18489E+01_JPRB, 0.17145E+01_JPRB, 0.15136E+01_JPRB ,&
&  0.14903E+01_JPRB, 0.15721E+01_JPRB, 0.16135E+01_JPRB, 0.14647E+01_JPRB, 0.11922E+01_JPRB, 0.10961E+01_JPRB, 0.11591E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.12536E+01_JPRB, 0.12568E+01_JPRB, 0.12705E+01_JPRB, 0.12681E+01_JPRB, 0.12637E+01_JPRB, 0.12596E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.12376E+01_JPRB, 0.12129E+01_JPRB, 0.11614E+01_JPRB, 0.11176E+01_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.91998E+00_JPRB, 0.92046E+00_JPRB, 0.90569E+00_JPRB, 0.88655E+00_JPRB, 0.87658E+00_JPRB, 0.85658E+00_JPRB, 0.88137E+00_JPRB ,&
&  0.88220E+00_JPRB, 0.91033E+00_JPRB, 0.94079E+00_JPRB, 0.97477E+00_JPRB, 0.10335E+01_JPRB, 0.10966E+01_JPRB, 0.11865E+01_JPRB ,&
&  0.13387E+01_JPRB, 0.15779E+01_JPRB, 0.18875E+01_JPRB, 0.22740E+01_JPRB, 0.24285E+01_JPRB, 0.25358E+01_JPRB, 0.26905E+01_JPRB ,&
&  0.28761E+01_JPRB, 0.30295E+01_JPRB, 0.31172E+01_JPRB, 0.32327E+01_JPRB, 0.33556E+01_JPRB, 0.33164E+01_JPRB, 0.32434E+01_JPRB ,&
&  0.26301E+01_JPRB, 0.17908E+01_JPRB, 0.18102E+01_JPRB, 0.16691E+01_JPRB, 0.17238E+01_JPRB, 0.17900E+01_JPRB, 0.18328E+01_JPRB ,&
&  0.18724E+01_JPRB, 0.18955E+01_JPRB, 0.19311E+01_JPRB, 0.19803E+01_JPRB, 0.20558E+01_JPRB, 0.21283E+01_JPRB, 0.22052E+01_JPRB ,&
&  0.22625E+01_JPRB, 0.23198E+01_JPRB, 0.23610E+01_JPRB, 0.23053E+01_JPRB, 0.24038E+01_JPRB, 0.25814E+01_JPRB, 0.27322E+01_JPRB ,&
&  0.28536E+01_JPRB, 0.28407E+01_JPRB, 0.25762E+01_JPRB, 0.26571E+01_JPRB, 0.28495E+01_JPRB, 0.27630E+01_JPRB, 0.28951E+01_JPRB ,&
&  0.31962E+01_JPRB, 0.29192E+01_JPRB, 0.16748E+01_JPRB, 0.15680E+01_JPRB, 0.17583E+01_JPRB, 0.19242E+01_JPRB, 0.20797E+01_JPRB ,&
&  0.22116E+01_JPRB, 0.23330E+01_JPRB, 0.25007E+01_JPRB, 0.26967E+01_JPRB, 0.27963E+01_JPRB, 0.29188E+01_JPRB, 0.23975E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.14068E+01_JPRB, 0.14967E+01_JPRB, 0.15969E+01_JPRB, 0.18322E+01_JPRB, 0.17884E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.15378E+01_JPRB, 0.17915E+01_JPRB, 0.21358E+01_JPRB, 0.17945E+01_JPRB, 0.11377E+01_JPRB, 0.10008E+01_JPRB, 0.10002E+01_JPRB ,&
&  0.10448E+01_JPRB, 0.10510E+01_JPRB, 0.10752E+01_JPRB, 0.10931E+01_JPRB, 0.11158E+01_JPRB, 0.11020E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.10364E+01_JPRB, 0.10077E+01_JPRB, 0.96747E+00_JPRB, 0.93701E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.77566E+00_JPRB, 0.73820E+00_JPRB, 0.69701E+00_JPRB, 0.69144E+00_JPRB, 0.69434E+00_JPRB, 0.67970E+00_JPRB, 0.71850E+00_JPRB ,&
&  0.69511E+00_JPRB, 0.72853E+00_JPRB, 0.74836E+00_JPRB, 0.75602E+00_JPRB, 0.81121E+00_JPRB, 0.86581E+00_JPRB, 0.93499E+00_JPRB ,&
&  0.10298E+01_JPRB, 0.11780E+01_JPRB, 0.13674E+01_JPRB, 0.16065E+01_JPRB, 0.18992E+01_JPRB, 0.20531E+01_JPRB, 0.22058E+01_JPRB ,&
&  0.24150E+01_JPRB, 0.26896E+01_JPRB, 0.30752E+01_JPRB, 0.35192E+01_JPRB, 0.40269E+01_JPRB, 0.38561E+01_JPRB, 0.33128E+01_JPRB ,&
&  0.23869E+01_JPRB, 0.17476E+01_JPRB, 0.16052E+01_JPRB, 0.14878E+01_JPRB, 0.15640E+01_JPRB, 0.17263E+01_JPRB, 0.17904E+01_JPRB ,&
&  0.18361E+01_JPRB, 0.18861E+01_JPRB, 0.19393E+01_JPRB, 0.20034E+01_JPRB, 0.20825E+01_JPRB, 0.21726E+01_JPRB, 0.22238E+01_JPRB ,&
&  0.22545E+01_JPRB, 0.22882E+01_JPRB, 0.23173E+01_JPRB, 0.24317E+01_JPRB, 0.25985E+01_JPRB, 0.28048E+01_JPRB, 0.28346E+01_JPRB ,&
&  0.28367E+01_JPRB, 0.27497E+01_JPRB, 0.24151E+01_JPRB, 0.23854E+01_JPRB, 0.24735E+01_JPRB, 0.25305E+01_JPRB, 0.26219E+01_JPRB ,&
&  0.28923E+01_JPRB, 0.24625E+01_JPRB, 0.17260E+01_JPRB, 0.17988E+01_JPRB, 0.19100E+01_JPRB, 0.20527E+01_JPRB, 0.19183E+01_JPRB ,&
&  0.19567E+01_JPRB, 0.19998E+01_JPRB, 0.20911E+01_JPRB, 0.21076E+01_JPRB, 0.21294E+01_JPRB, 0.20796E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.14786E+01_JPRB, 0.13916E+01_JPRB, 0.13662E+01_JPRB, 0.14191E+01_JPRB, 0.17884E+01_JPRB, 0.17678E+01_JPRB, 0.16201E+01_JPRB ,&
&  0.14674E+01_JPRB, 0.16338E+01_JPRB, 0.14371E+01_JPRB, 0.17191E+01_JPRB, 0.10018E+01_JPRB, 0.86295E+00_JPRB, 0.84301E+00_JPRB ,&
&  0.87131E+00_JPRB, 0.88039E+00_JPRB, 0.89890E+00_JPRB, 0.88834E+00_JPRB, 0.90339E+00_JPRB, 0.93810E+00_JPRB, 0.94512E+00_JPRB ,&
&  0.89650E+00_JPRB, 0.88027E+00_JPRB, 0.84329E+00_JPRB, 0.80196E+00_JPRB, 0.79035E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.64509E+00_JPRB, 0.63360E+00_JPRB, 0.62790E+00_JPRB, 0.62204E+00_JPRB, 0.63211E+00_JPRB, 0.63729E+00_JPRB, 0.64330E+00_JPRB ,&
&  0.64807E+00_JPRB, 0.66686E+00_JPRB, 0.68306E+00_JPRB, 0.70452E+00_JPRB, 0.72702E+00_JPRB, 0.77005E+00_JPRB, 0.81875E+00_JPRB ,&
&  0.87700E+00_JPRB, 0.94255E+00_JPRB, 0.10162E+01_JPRB, 0.11230E+01_JPRB, 0.12746E+01_JPRB, 0.13881E+01_JPRB, 0.15125E+01_JPRB ,&
&  0.16750E+01_JPRB, 0.19176E+01_JPRB, 0.21958E+01_JPRB, 0.25694E+01_JPRB, 0.30006E+01_JPRB, 0.31274E+01_JPRB, 0.29766E+01_JPRB ,&
&  0.21158E+01_JPRB, 0.18284E+01_JPRB, 0.16374E+01_JPRB, 0.14885E+01_JPRB, 0.15026E+01_JPRB, 0.14601E+01_JPRB, 0.14758E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.15193E+01_JPRB, 0.15072E+01_JPRB, 0.15222E+01_JPRB, 0.15346E+01_JPRB, 0.15482E+01_JPRB, 0.15341E+01_JPRB ,&
&  0.15286E+01_JPRB, 0.15817E+01_JPRB, 0.18166E+01_JPRB, 0.21229E+01_JPRB, 0.23430E+01_JPRB, 0.24312E+01_JPRB, 0.23039E+01_JPRB ,&
&  0.21321E+01_JPRB, 0.20077E+01_JPRB, 0.18867E+01_JPRB, 0.18649E+01_JPRB, 0.19457E+01_JPRB, 0.20051E+01_JPRB, 0.21827E+01_JPRB ,&
&  0.21164E+01_JPRB, 0.18996E+01_JPRB, 0.18961E+01_JPRB, 0.19232E+01_JPRB, 0.19550E+01_JPRB, 0.19363E+01_JPRB, 0.18228E+01_JPRB ,&
&  0.18468E+01_JPRB, 0.17822E+01_JPRB, 0.17760E+01_JPRB, 0.16913E+01_JPRB, 0.15688E+01_JPRB, 0.14463E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.12142E+01_JPRB, 0.12355E+01_JPRB, 0.14097E+01_JPRB, 0.17388E+01_JPRB, 0.28118E+01_JPRB, 0.24694E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.12104E+01_JPRB, 0.84043E+00_JPRB, 0.10961E+01_JPRB, 0.80830E+00_JPRB, 0.74442E+00_JPRB, 0.73896E+00_JPRB ,&
&  0.74425E+00_JPRB, 0.73913E+00_JPRB, 0.71890E+00_JPRB, 0.71494E+00_JPRB, 0.72607E+00_JPRB, 0.73686E+00_JPRB, 0.72850E+00_JPRB ,&
&  0.69050E+00_JPRB, 0.67394E+00_JPRB, 0.70397E+00_JPRB, 0.65801E+00_JPRB, 0.64699E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.58624E+00_JPRB, 0.58586E+00_JPRB, 0.60144E+00_JPRB, 0.61646E+00_JPRB, 0.62649E+00_JPRB, 0.63680E+00_JPRB, 0.65204E+00_JPRB ,&
&  0.66406E+00_JPRB, 0.67863E+00_JPRB, 0.69407E+00_JPRB, 0.71833E+00_JPRB, 0.75151E+00_JPRB, 0.80079E+00_JPRB, 0.85746E+00_JPRB ,&
&  0.92734E+00_JPRB, 0.10029E+01_JPRB, 0.10885E+01_JPRB, 0.11816E+01_JPRB, 0.12966E+01_JPRB, 0.14551E+01_JPRB, 0.16725E+01_JPRB ,&
&  0.18002E+01_JPRB, 0.19627E+01_JPRB, 0.21671E+01_JPRB, 0.23551E+01_JPRB, 0.24071E+01_JPRB, 0.23832E+01_JPRB, 0.19996E+01_JPRB ,&
&  0.22252E+01_JPRB, 0.18348E+01_JPRB, 0.15962E+01_JPRB, 0.14311E+01_JPRB, 0.13248E+01_JPRB, 0.12334E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.10792E+01_JPRB, 0.10177E+01_JPRB, 0.97696E+00_JPRB, 0.94216E+00_JPRB, 0.95544E+00_JPRB, 0.93847E+00_JPRB ,&
&  0.10189E+01_JPRB, 0.12050E+01_JPRB, 0.14436E+01_JPRB, 0.17405E+01_JPRB, 0.19061E+01_JPRB, 0.18744E+01_JPRB, 0.15858E+01_JPRB ,&
&  0.14134E+01_JPRB, 0.13286E+01_JPRB, 0.13058E+01_JPRB, 0.13805E+01_JPRB, 0.14255E+01_JPRB, 0.15173E+01_JPRB, 0.15747E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.15092E+01_JPRB, 0.16154E+01_JPRB, 0.20356E+01_JPRB, 0.20084E+01_JPRB, 0.18575E+01_JPRB, 0.18006E+01_JPRB ,&
&  0.17053E+01_JPRB, 0.16424E+01_JPRB, 0.16099E+01_JPRB, 0.15180E+01_JPRB, 0.13852E+01_JPRB, 0.12483E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.10760E+01_JPRB, 0.10592E+01_JPRB, 0.11797E+01_JPRB, 0.15232E+01_JPRB, 0.19231E+01_JPRB, 0.27229E+01_JPRB ,&
&  0.14710E+01_JPRB, 0.87221E+00_JPRB, 0.86179E+00_JPRB, 0.83629E+00_JPRB, 0.73749E+00_JPRB, 0.67177E+00_JPRB, 0.71957E+00_JPRB ,&
&  0.78396E+00_JPRB, 0.85092E+00_JPRB, 0.84256E+00_JPRB, 0.73230E+00_JPRB, 0.68962E+00_JPRB, 0.67127E+00_JPRB, 0.64744E+00_JPRB ,&
&  0.62739E+00_JPRB, 0.58993E+00_JPRB, 0.60523E+00_JPRB, 0.58840E+00_JPRB, 0.58792E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.54534E+00_JPRB, 0.54392E+00_JPRB, 0.55317E+00_JPRB, 0.56267E+00_JPRB, 0.56650E+00_JPRB, 0.57108E+00_JPRB, 0.57601E+00_JPRB ,&
&  0.58204E+00_JPRB, 0.58476E+00_JPRB, 0.58948E+00_JPRB, 0.59923E+00_JPRB, 0.61893E+00_JPRB, 0.66117E+00_JPRB, 0.71182E+00_JPRB ,&
&  0.75889E+00_JPRB, 0.80573E+00_JPRB, 0.85491E+00_JPRB, 0.89865E+00_JPRB, 0.94427E+00_JPRB, 0.97597E+00_JPRB, 0.10516E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.12667E+01_JPRB, 0.13892E+01_JPRB, 0.14871E+01_JPRB, 0.15890E+01_JPRB, 0.16260E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.18523E+01_JPRB, 0.16829E+01_JPRB, 0.14975E+01_JPRB, 0.13403E+01_JPRB, 0.12346E+01_JPRB, 0.11402E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.99347E+00_JPRB, 0.10570E+01_JPRB, 0.10052E+01_JPRB, 0.10709E+01_JPRB, 0.86805E+00_JPRB, 0.88525E+00_JPRB, 0.10270E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.12558E+01_JPRB, 0.13906E+01_JPRB, 0.15352E+01_JPRB, 0.16167E+01_JPRB, 0.15828E+01_JPRB, 0.14229E+01_JPRB ,&
&  0.13280E+01_JPRB, 0.12886E+01_JPRB, 0.12370E+01_JPRB, 0.13275E+01_JPRB, 0.13194E+01_JPRB, 0.12852E+01_JPRB, 0.13099E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.12579E+01_JPRB, 0.14856E+01_JPRB, 0.19480E+01_JPRB, 0.19092E+01_JPRB, 0.18379E+01_JPRB, 0.17647E+01_JPRB ,&
&  0.16599E+01_JPRB, 0.15819E+01_JPRB, 0.15196E+01_JPRB, 0.14177E+01_JPRB, 0.12967E+01_JPRB, 0.11786E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.99166E+00_JPRB, 0.95310E+00_JPRB, 0.97173E+00_JPRB, 0.11004E+01_JPRB, 0.13063E+01_JPRB, 0.22618E+01_JPRB ,&
&  0.14414E+01_JPRB, 0.11293E+01_JPRB, 0.12195E+01_JPRB, 0.52931E+00_JPRB, 0.60709E+00_JPRB, 0.62057E+00_JPRB, 0.72693E+00_JPRB ,&
&  0.88196E+00_JPRB, 0.13843E+01_JPRB, 0.20871E+01_JPRB, 0.18675E+01_JPRB, 0.14337E+01_JPRB, 0.93802E+00_JPRB, 0.58338E+00_JPRB ,&
&  0.56345E+00_JPRB, 0.55828E+00_JPRB, 0.56302E+00_JPRB, 0.55600E+00_JPRB, 0.55334E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.49081E+00_JPRB, 0.48178E+00_JPRB, 0.49114E+00_JPRB, 0.50224E+00_JPRB, 0.49956E+00_JPRB, 0.50039E+00_JPRB, 0.49605E+00_JPRB ,&
&  0.48733E+00_JPRB, 0.48131E+00_JPRB, 0.50550E+00_JPRB, 0.55135E+00_JPRB, 0.61673E+00_JPRB, 0.70290E+00_JPRB, 0.79166E+00_JPRB ,&
&  0.85918E+00_JPRB, 0.94144E+00_JPRB, 0.10196E+01_JPRB, 0.10857E+01_JPRB, 0.11098E+01_JPRB, 0.11291E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.11739E+01_JPRB, 0.12645E+01_JPRB, 0.13613E+01_JPRB, 0.15170E+01_JPRB, 0.17796E+01_JPRB, 0.11176E+01_JPRB ,&
&  0.15094E+01_JPRB, 0.14744E+01_JPRB, 0.14043E+01_JPRB, 0.12899E+01_JPRB, 0.12347E+01_JPRB, 0.11505E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.11281E+01_JPRB, 0.12083E+01_JPRB, 0.11901E+01_JPRB, 0.12310E+01_JPRB, 0.12492E+01_JPRB, 0.12302E+01_JPRB, 0.13876E+01_JPRB ,&
&  0.15035E+01_JPRB, 0.16561E+01_JPRB, 0.18097E+01_JPRB, 0.19178E+01_JPRB, 0.20089E+01_JPRB, 0.20403E+01_JPRB, 0.20178E+01_JPRB ,&
&  0.18881E+01_JPRB, 0.17972E+01_JPRB, 0.18284E+01_JPRB, 0.19424E+01_JPRB, 0.17506E+01_JPRB, 0.16634E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.11656E+01_JPRB, 0.11291E+01_JPRB, 0.13141E+01_JPRB, 0.17150E+01_JPRB, 0.17930E+01_JPRB, 0.17274E+01_JPRB, 0.15891E+01_JPRB ,&
&  0.14892E+01_JPRB, 0.13939E+01_JPRB, 0.13188E+01_JPRB, 0.12337E+01_JPRB, 0.11473E+01_JPRB, 0.10665E+01_JPRB, 0.98953E+00_JPRB ,&
&  0.95016E+00_JPRB, 0.90644E+00_JPRB, 0.85829E+00_JPRB, 0.88173E+00_JPRB, 0.98150E+00_JPRB, 0.12647E+01_JPRB, 0.14035E+01_JPRB ,&
&  0.15384E+01_JPRB, 0.10747E+01_JPRB, 0.89059E+00_JPRB, 0.82454E+00_JPRB, 0.60438E+00_JPRB, 0.61219E+00_JPRB, 0.77243E+00_JPRB ,&
&  0.11886E+01_JPRB, 0.17174E+01_JPRB, 0.28517E+01_JPRB, 0.50761E+01_JPRB, 0.28183E+01_JPRB, 0.21831E+01_JPRB, 0.56061E+00_JPRB ,&
&  0.53904E+00_JPRB, 0.53910E+00_JPRB, 0.53591E+00_JPRB, 0.51958E+00_JPRB, 0.51461E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.47188E+00_JPRB, 0.46388E+00_JPRB, 0.47349E+00_JPRB, 0.47224E+00_JPRB, 0.47641E+00_JPRB, 0.48434E+00_JPRB, 0.48912E+00_JPRB ,&
&  0.49390E+00_JPRB, 0.51195E+00_JPRB, 0.60842E+00_JPRB, 0.74432E+00_JPRB, 0.87770E+00_JPRB, 0.98340E+00_JPRB, 0.10406E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.11575E+01_JPRB, 0.11855E+01_JPRB, 0.12081E+01_JPRB, 0.12646E+01_JPRB, 0.13499E+01_JPRB, 0.14229E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.15709E+01_JPRB, 0.16583E+01_JPRB, 0.17913E+01_JPRB, 0.19179E+01_JPRB, 0.19007E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.14170E+01_JPRB, 0.14359E+01_JPRB, 0.13499E+01_JPRB, 0.12617E+01_JPRB, 0.12285E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.14151E+01_JPRB, 0.14513E+01_JPRB, 0.14007E+01_JPRB, 0.14062E+01_JPRB, 0.17819E+01_JPRB, 0.17269E+01_JPRB, 0.19817E+01_JPRB ,&
&  0.21539E+01_JPRB, 0.23407E+01_JPRB, 0.24620E+01_JPRB, 0.26045E+01_JPRB, 0.27243E+01_JPRB, 0.28042E+01_JPRB, 0.29202E+01_JPRB ,&
&  0.29521E+01_JPRB, 0.30025E+01_JPRB, 0.29894E+01_JPRB, 0.28915E+01_JPRB, 0.24050E+01_JPRB, 0.25442E+01_JPRB, 0.18353E+01_JPRB ,&
&  0.13925E+01_JPRB, 0.12609E+01_JPRB, 0.14132E+01_JPRB, 0.15366E+01_JPRB, 0.15318E+01_JPRB, 0.14066E+01_JPRB, 0.12302E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.11487E+01_JPRB, 0.11305E+01_JPRB, 0.10787E+01_JPRB, 0.10477E+01_JPRB, 0.99102E+00_JPRB, 0.94579E+00_JPRB ,&
&  0.92010E+00_JPRB, 0.87599E+00_JPRB, 0.82855E+00_JPRB, 0.86161E+00_JPRB, 0.10003E+01_JPRB, 0.13818E+01_JPRB, 0.20218E+01_JPRB ,&
&  0.21775E+01_JPRB, 0.11585E+01_JPRB, 0.95469E+00_JPRB, 0.80678E+00_JPRB, 0.73110E+00_JPRB, 0.73337E+00_JPRB, 0.80344E+00_JPRB ,&
&  0.10228E+01_JPRB, 0.18012E+01_JPRB, 0.26680E+01_JPRB, 0.19893E+01_JPRB, 0.22456E+01_JPRB, 0.15534E+01_JPRB, 0.65595E+00_JPRB ,&
&  0.52256E+00_JPRB, 0.50571E+00_JPRB, 0.50976E+00_JPRB, 0.50064E+00_JPRB, 0.48781E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.51954E+00_JPRB, 0.50808E+00_JPRB, 0.50702E+00_JPRB, 0.51786E+00_JPRB, 0.52484E+00_JPRB, 0.55063E+00_JPRB, 0.58250E+00_JPRB ,&
&  0.60062E+00_JPRB, 0.67634E+00_JPRB, 0.84587E+00_JPRB, 0.99432E+00_JPRB, 0.10657E+01_JPRB, 0.11203E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.11149E+01_JPRB, 0.11032E+01_JPRB, 0.11395E+01_JPRB, 0.12074E+01_JPRB, 0.13038E+01_JPRB, 0.14299E+01_JPRB ,&
&  0.15723E+01_JPRB, 0.17142E+01_JPRB, 0.18447E+01_JPRB, 0.19663E+01_JPRB, 0.21311E+01_JPRB, 0.21566E+01_JPRB, 0.21086E+01_JPRB ,&
&  0.96129E+00_JPRB, 0.14551E+01_JPRB, 0.14665E+01_JPRB, 0.14004E+01_JPRB, 0.13106E+01_JPRB, 0.13314E+01_JPRB, 0.15292E+01_JPRB ,&
&  0.16887E+01_JPRB, 0.16895E+01_JPRB, 0.19497E+01_JPRB, 0.17269E+01_JPRB, 0.20243E+01_JPRB, 0.21830E+01_JPRB, 0.23777E+01_JPRB ,&
&  0.25569E+01_JPRB, 0.26846E+01_JPRB, 0.28204E+01_JPRB, 0.29521E+01_JPRB, 0.30839E+01_JPRB, 0.31967E+01_JPRB, 0.33425E+01_JPRB ,&
&  0.35099E+01_JPRB, 0.37439E+01_JPRB, 0.39741E+01_JPRB, 0.38024E+01_JPRB, 0.32583E+01_JPRB, 0.30738E+01_JPRB, 0.25170E+01_JPRB ,&
&  0.18015E+01_JPRB, 0.12653E+01_JPRB, 0.13121E+01_JPRB, 0.13802E+01_JPRB, 0.13106E+01_JPRB, 0.12125E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.10152E+01_JPRB, 0.97438E+00_JPRB, 0.10003E+01_JPRB, 0.10239E+01_JPRB, 0.10034E+01_JPRB, 0.96829E+00_JPRB ,&
&  0.95476E+00_JPRB, 0.92170E+00_JPRB, 0.88529E+00_JPRB, 0.87553E+00_JPRB, 0.93520E+00_JPRB, 0.10722E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.10602E+01_JPRB, 0.93946E+00_JPRB, 0.91172E+00_JPRB, 0.87281E+00_JPRB, 0.84549E+00_JPRB, 0.82313E+00_JPRB ,&
&  0.86396E+00_JPRB, 0.10323E+01_JPRB, 0.14347E+01_JPRB, 0.18238E+01_JPRB, 0.14430E+01_JPRB, 0.11420E+01_JPRB, 0.66714E+00_JPRB ,&
&  0.54150E+00_JPRB, 0.52287E+00_JPRB, 0.53987E+00_JPRB, 0.54318E+00_JPRB, 0.54287E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.62823E+00_JPRB, 0.61423E+00_JPRB, 0.59578E+00_JPRB, 0.61216E+00_JPRB, 0.64635E+00_JPRB, 0.69026E+00_JPRB, 0.71051E+00_JPRB ,&
&  0.73798E+00_JPRB, 0.84698E+00_JPRB, 0.99019E+00_JPRB, 0.11180E+01_JPRB, 0.11467E+01_JPRB, 0.11236E+01_JPRB, 0.10422E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.10292E+01_JPRB, 0.10379E+01_JPRB, 0.10539E+01_JPRB, 0.11035E+01_JPRB, 0.12068E+01_JPRB, 0.13618E+01_JPRB ,&
&  0.15637E+01_JPRB, 0.17883E+01_JPRB, 0.20419E+01_JPRB, 0.23204E+01_JPRB, 0.25858E+01_JPRB, 0.27410E+01_JPRB, 0.26364E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.11795E+01_JPRB, 0.15735E+01_JPRB, 0.15640E+01_JPRB, 0.14863E+01_JPRB, 0.14678E+01_JPRB, 0.15639E+01_JPRB ,&
&  0.16603E+01_JPRB, 0.16507E+01_JPRB, 0.18115E+01_JPRB, 0.20339E+01_JPRB, 0.21005E+01_JPRB, 0.23348E+01_JPRB, 0.24498E+01_JPRB ,&
&  0.25633E+01_JPRB, 0.26910E+01_JPRB, 0.28376E+01_JPRB, 0.29826E+01_JPRB, 0.31248E+01_JPRB, 0.32569E+01_JPRB, 0.34424E+01_JPRB ,&
&  0.37017E+01_JPRB, 0.39748E+01_JPRB, 0.42012E+01_JPRB, 0.37848E+01_JPRB, 0.40037E+01_JPRB, 0.35455E+01_JPRB, 0.30823E+01_JPRB ,&
&  0.19464E+01_JPRB, 0.13234E+01_JPRB, 0.13346E+01_JPRB, 0.14168E+01_JPRB, 0.12736E+01_JPRB, 0.11378E+01_JPRB, 0.10053E+01_JPRB ,&
&  0.94043E+00_JPRB, 0.87900E+00_JPRB, 0.92316E+00_JPRB, 0.97879E+00_JPRB, 0.97534E+00_JPRB, 0.10206E+01_JPRB, 0.97952E+00_JPRB ,&
&  0.98543E+00_JPRB, 0.99293E+00_JPRB, 0.97542E+00_JPRB, 0.98041E+00_JPRB, 0.10030E+01_JPRB, 0.10399E+01_JPRB, 0.10463E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.10211E+01_JPRB, 0.10089E+01_JPRB, 0.10025E+01_JPRB, 0.98087E+00_JPRB, 0.95152E+00_JPRB, 0.89594E+00_JPRB ,&
&  0.83541E+00_JPRB, 0.84017E+00_JPRB, 0.93845E+00_JPRB, 0.10668E+01_JPRB, 0.10055E+01_JPRB, 0.86986E+00_JPRB, 0.69619E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.70299E+00_JPRB, 0.70309E+00_JPRB, 0.68265E+00_JPRB, 0.65866E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.74690E+00_JPRB, 0.71768E+00_JPRB, 0.67835E+00_JPRB, 0.66956E+00_JPRB, 0.72351E+00_JPRB, 0.77336E+00_JPRB, 0.77692E+00_JPRB ,&
&  0.78852E+00_JPRB, 0.85840E+00_JPRB, 0.98280E+00_JPRB, 0.10804E+01_JPRB, 0.11138E+01_JPRB, 0.11086E+01_JPRB, 0.10254E+01_JPRB ,&
&  0.10012E+01_JPRB, 0.99195E+00_JPRB, 0.97818E+00_JPRB, 0.98411E+00_JPRB, 0.10058E+01_JPRB, 0.10516E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.12678E+01_JPRB, 0.14838E+01_JPRB, 0.17977E+01_JPRB, 0.22483E+01_JPRB, 0.28894E+01_JPRB, 0.36490E+01_JPRB, 0.43160E+01_JPRB ,&
&  0.32668E+01_JPRB, 0.12704E+01_JPRB, 0.14822E+01_JPRB, 0.18662E+01_JPRB, 0.17199E+01_JPRB, 0.17183E+01_JPRB, 0.16621E+01_JPRB ,&
&  0.16983E+01_JPRB, 0.17127E+01_JPRB, 0.17364E+01_JPRB, 0.19228E+01_JPRB, 0.20899E+01_JPRB, 0.22870E+01_JPRB, 0.23639E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.25271E+01_JPRB, 0.26242E+01_JPRB, 0.27257E+01_JPRB, 0.28425E+01_JPRB, 0.29725E+01_JPRB, 0.31836E+01_JPRB ,&
&  0.34551E+01_JPRB, 0.37102E+01_JPRB, 0.38910E+01_JPRB, 0.33479E+01_JPRB, 0.35253E+01_JPRB, 0.28661E+01_JPRB, 0.22454E+01_JPRB ,&
&  0.18362E+01_JPRB, 0.15414E+01_JPRB, 0.14340E+01_JPRB, 0.14770E+01_JPRB, 0.12898E+01_JPRB, 0.10197E+01_JPRB, 0.10036E+01_JPRB ,&
&  0.91740E+00_JPRB, 0.89665E+00_JPRB, 0.91596E+00_JPRB, 0.88055E+00_JPRB, 0.94570E+00_JPRB, 0.98151E+00_JPRB, 0.10132E+01_JPRB ,&
&  0.10084E+01_JPRB, 0.10322E+01_JPRB, 0.10557E+01_JPRB, 0.10969E+01_JPRB, 0.10950E+01_JPRB, 0.11064E+01_JPRB, 0.10908E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.11072E+01_JPRB, 0.11167E+01_JPRB, 0.11283E+01_JPRB, 0.11237E+01_JPRB, 0.10324E+01_JPRB, 0.96926E+00_JPRB ,&
&  0.92976E+00_JPRB, 0.90655E+00_JPRB, 0.86165E+00_JPRB, 0.89731E+00_JPRB, 0.89478E+00_JPRB, 0.89852E+00_JPRB, 0.92856E+00_JPRB ,&
&  0.93943E+00_JPRB, 0.94021E+00_JPRB, 0.88595E+00_JPRB, 0.82758E+00_JPRB, 0.79043E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.82210E+00_JPRB, 0.76505E+00_JPRB, 0.73133E+00_JPRB, 0.74086E+00_JPRB, 0.79174E+00_JPRB, 0.84398E+00_JPRB, 0.84139E+00_JPRB ,&
&  0.82680E+00_JPRB, 0.84970E+00_JPRB, 0.92355E+00_JPRB, 0.98737E+00_JPRB, 0.99734E+00_JPRB, 0.10051E+01_JPRB, 0.10303E+01_JPRB ,&
&  0.98776E+00_JPRB, 0.96350E+00_JPRB, 0.95520E+00_JPRB, 0.93996E+00_JPRB, 0.94668E+00_JPRB, 0.95147E+00_JPRB, 0.97742E+00_JPRB ,&
&  0.10439E+01_JPRB, 0.11626E+01_JPRB, 0.13783E+01_JPRB, 0.17044E+01_JPRB, 0.21877E+01_JPRB, 0.30860E+01_JPRB, 0.44807E+01_JPRB ,&
&  0.66335E+01_JPRB, 0.43474E+01_JPRB, 0.14560E+01_JPRB, 0.19048E+01_JPRB, 0.20040E+01_JPRB, 0.20169E+01_JPRB, 0.17387E+01_JPRB ,&
&  0.18515E+01_JPRB, 0.19040E+01_JPRB, 0.17006E+01_JPRB, 0.17877E+01_JPRB, 0.19481E+01_JPRB, 0.20910E+01_JPRB, 0.21737E+01_JPRB ,&
&  0.22061E+01_JPRB, 0.22670E+01_JPRB, 0.23138E+01_JPRB, 0.23482E+01_JPRB, 0.23932E+01_JPRB, 0.24739E+01_JPRB, 0.26222E+01_JPRB ,&
&  0.29117E+01_JPRB, 0.32644E+01_JPRB, 0.35675E+01_JPRB, 0.31023E+01_JPRB, 0.31191E+01_JPRB, 0.24977E+01_JPRB, 0.21475E+01_JPRB ,&
&  0.19100E+01_JPRB, 0.19068E+01_JPRB, 0.18482E+01_JPRB, 0.15851E+01_JPRB, 0.13099E+01_JPRB, 0.11345E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.93116E+00_JPRB, 0.90967E+00_JPRB, 0.94169E+00_JPRB, 0.96703E+00_JPRB, 0.98804E+00_JPRB, 0.10096E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.10664E+01_JPRB, 0.11171E+01_JPRB, 0.11487E+01_JPRB, 0.11763E+01_JPRB, 0.12255E+01_JPRB, 0.12304E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.12233E+01_JPRB, 0.12313E+01_JPRB, 0.12413E+01_JPRB, 0.11787E+01_JPRB, 0.10844E+01_JPRB, 0.10716E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.10364E+01_JPRB, 0.97662E+00_JPRB, 0.97606E+00_JPRB, 0.97460E+00_JPRB, 0.10306E+01_JPRB, 0.10997E+01_JPRB ,&
&  0.11288E+01_JPRB, 0.10957E+01_JPRB, 0.10199E+01_JPRB, 0.88871E+00_JPRB, 0.87340E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.88546E+00_JPRB, 0.86487E+00_JPRB, 0.83438E+00_JPRB, 0.81398E+00_JPRB, 0.83826E+00_JPRB, 0.87423E+00_JPRB, 0.87340E+00_JPRB ,&
&  0.84649E+00_JPRB, 0.83031E+00_JPRB, 0.86763E+00_JPRB, 0.90675E+00_JPRB, 0.90430E+00_JPRB, 0.92471E+00_JPRB, 0.93662E+00_JPRB ,&
&  0.97312E+00_JPRB, 0.97063E+00_JPRB, 0.95381E+00_JPRB, 0.93029E+00_JPRB, 0.92898E+00_JPRB, 0.91677E+00_JPRB, 0.92037E+00_JPRB ,&
&  0.94528E+00_JPRB, 0.10011E+01_JPRB, 0.11174E+01_JPRB, 0.12988E+01_JPRB, 0.15262E+01_JPRB, 0.19497E+01_JPRB, 0.28037E+01_JPRB ,&
&  0.37727E+01_JPRB, 0.48467E+01_JPRB, 0.15793E+01_JPRB, 0.21215E+01_JPRB, 0.25757E+01_JPRB, 0.24565E+01_JPRB, 0.21853E+01_JPRB ,&
&  0.21755E+01_JPRB, 0.21481E+01_JPRB, 0.19707E+01_JPRB, 0.17131E+01_JPRB, 0.16994E+01_JPRB, 0.17886E+01_JPRB, 0.17939E+01_JPRB ,&
&  0.18447E+01_JPRB, 0.19279E+01_JPRB, 0.20506E+01_JPRB, 0.20739E+01_JPRB, 0.20636E+01_JPRB, 0.20432E+01_JPRB, 0.20882E+01_JPRB ,&
&  0.22902E+01_JPRB, 0.26335E+01_JPRB, 0.31490E+01_JPRB, 0.29125E+01_JPRB, 0.29057E+01_JPRB, 0.26014E+01_JPRB, 0.23437E+01_JPRB ,&
&  0.21392E+01_JPRB, 0.21754E+01_JPRB, 0.21022E+01_JPRB, 0.17882E+01_JPRB, 0.14452E+01_JPRB, 0.12827E+01_JPRB, 0.12712E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.10183E+01_JPRB, 0.10005E+01_JPRB, 0.10099E+01_JPRB, 0.10605E+01_JPRB, 0.10592E+01_JPRB, 0.10474E+01_JPRB ,&
&  0.10850E+01_JPRB, 0.10819E+01_JPRB, 0.11163E+01_JPRB, 0.11614E+01_JPRB, 0.12087E+01_JPRB, 0.12750E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.13130E+01_JPRB, 0.13257E+01_JPRB, 0.13225E+01_JPRB, 0.12693E+01_JPRB, 0.11946E+01_JPRB, 0.11499E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.11524E+01_JPRB, 0.11626E+01_JPRB, 0.10694E+01_JPRB, 0.10134E+01_JPRB, 0.10399E+01_JPRB, 0.10460E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.12999E+01_JPRB, 0.12419E+01_JPRB, 0.96458E+00_JPRB, 0.94672E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.99269E+00_JPRB, 0.94712E+00_JPRB, 0.92100E+00_JPRB, 0.89655E+00_JPRB, 0.87630E+00_JPRB, 0.90918E+00_JPRB, 0.90141E+00_JPRB ,&
&  0.87710E+00_JPRB, 0.84298E+00_JPRB, 0.85894E+00_JPRB, 0.90748E+00_JPRB, 0.91029E+00_JPRB, 0.89288E+00_JPRB, 0.88155E+00_JPRB ,&
&  0.88216E+00_JPRB, 0.89954E+00_JPRB, 0.90382E+00_JPRB, 0.91879E+00_JPRB, 0.92157E+00_JPRB, 0.91230E+00_JPRB, 0.90728E+00_JPRB ,&
&  0.92660E+00_JPRB, 0.95422E+00_JPRB, 0.10039E+01_JPRB, 0.10666E+01_JPRB, 0.11537E+01_JPRB, 0.12836E+01_JPRB, 0.17139E+01_JPRB ,&
&  0.22871E+01_JPRB, 0.38356E+01_JPRB, 0.30811E+01_JPRB, 0.36753E+01_JPRB, 0.34420E+01_JPRB, 0.31326E+01_JPRB, 0.27833E+01_JPRB ,&
&  0.29885E+01_JPRB, 0.27465E+01_JPRB, 0.25643E+01_JPRB, 0.18928E+01_JPRB, 0.15526E+01_JPRB, 0.14825E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.14138E+01_JPRB, 0.14278E+01_JPRB, 0.16086E+01_JPRB, 0.16881E+01_JPRB, 0.17160E+01_JPRB, 0.17176E+01_JPRB, 0.17240E+01_JPRB ,&
&  0.18273E+01_JPRB, 0.20023E+01_JPRB, 0.24830E+01_JPRB, 0.28141E+01_JPRB, 0.27249E+01_JPRB, 0.27086E+01_JPRB, 0.26649E+01_JPRB ,&
&  0.27033E+01_JPRB, 0.25130E+01_JPRB, 0.22984E+01_JPRB, 0.19644E+01_JPRB, 0.15987E+01_JPRB, 0.15504E+01_JPRB, 0.13550E+01_JPRB ,&
&  0.11972E+01_JPRB, 0.10962E+01_JPRB, 0.10803E+01_JPRB, 0.10946E+01_JPRB, 0.11304E+01_JPRB, 0.11213E+01_JPRB, 0.10934E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.11118E+01_JPRB, 0.11071E+01_JPRB, 0.11411E+01_JPRB, 0.11721E+01_JPRB, 0.12145E+01_JPRB, 0.12508E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.13382E+01_JPRB, 0.13027E+01_JPRB, 0.12585E+01_JPRB, 0.11957E+01_JPRB, 0.11555E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.11482E+01_JPRB, 0.11322E+01_JPRB, 0.10894E+01_JPRB, 0.10585E+01_JPRB, 0.10381E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.11136E+01_JPRB, 0.10940E+01_JPRB, 0.10397E+01_JPRB, 0.10235E+01_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.10614E+01_JPRB, 0.10357E+01_JPRB, 0.98555E+00_JPRB, 0.97376E+00_JPRB, 0.93938E+00_JPRB, 0.96287E+00_JPRB, 0.96718E+00_JPRB ,&
&  0.93318E+00_JPRB, 0.89892E+00_JPRB, 0.87335E+00_JPRB, 0.89274E+00_JPRB, 0.91453E+00_JPRB, 0.90379E+00_JPRB, 0.86738E+00_JPRB ,&
&  0.85527E+00_JPRB, 0.85244E+00_JPRB, 0.85146E+00_JPRB, 0.85643E+00_JPRB, 0.87399E+00_JPRB, 0.89955E+00_JPRB, 0.89905E+00_JPRB ,&
&  0.89692E+00_JPRB, 0.93762E+00_JPRB, 0.98676E+00_JPRB, 0.10047E+01_JPRB, 0.10283E+01_JPRB, 0.10774E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.38118E+01_JPRB, 0.24394E+01_JPRB, 0.47326E+01_JPRB, 0.37489E+01_JPRB, 0.33361E+01_JPRB, 0.27289E+01_JPRB ,&
&  0.25341E+01_JPRB, 0.26612E+01_JPRB, 0.21718E+01_JPRB, 0.18013E+01_JPRB, 0.14938E+01_JPRB, 0.13339E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.11378E+01_JPRB, 0.12336E+01_JPRB, 0.13410E+01_JPRB, 0.14013E+01_JPRB, 0.14203E+01_JPRB, 0.14524E+01_JPRB ,&
&  0.15160E+01_JPRB, 0.15971E+01_JPRB, 0.19105E+01_JPRB, 0.22954E+01_JPRB, 0.26498E+01_JPRB, 0.28086E+01_JPRB, 0.30124E+01_JPRB ,&
&  0.30843E+01_JPRB, 0.29444E+01_JPRB, 0.25273E+01_JPRB, 0.21240E+01_JPRB, 0.18222E+01_JPRB, 0.17085E+01_JPRB, 0.15531E+01_JPRB ,&
&  0.13946E+01_JPRB, 0.12481E+01_JPRB, 0.12106E+01_JPRB, 0.11861E+01_JPRB, 0.12191E+01_JPRB, 0.12117E+01_JPRB, 0.11537E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.11040E+01_JPRB, 0.11236E+01_JPRB, 0.11514E+01_JPRB, 0.11828E+01_JPRB, 0.11884E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.12888E+01_JPRB, 0.12787E+01_JPRB, 0.12760E+01_JPRB, 0.12251E+01_JPRB, 0.11884E+01_JPRB, 0.11664E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.11896E+01_JPRB, 0.11448E+01_JPRB, 0.10970E+01_JPRB, 0.10785E+01_JPRB, 0.10807E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.10165E+01_JPRB, 0.10542E+01_JPRB, 0.10997E+01_JPRB, 0.10660E+01_JPRB, 0.10620E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.10896E+01_JPRB, 0.10816E+01_JPRB, 0.10176E+01_JPRB, 0.99887E+00_JPRB, 0.96958E+00_JPRB, 0.95932E+00_JPRB, 0.96808E+00_JPRB ,&
&  0.95832E+00_JPRB, 0.93213E+00_JPRB, 0.89902E+00_JPRB, 0.88333E+00_JPRB, 0.89491E+00_JPRB, 0.90872E+00_JPRB, 0.87739E+00_JPRB ,&
&  0.84075E+00_JPRB, 0.82697E+00_JPRB, 0.81864E+00_JPRB, 0.82115E+00_JPRB, 0.83349E+00_JPRB, 0.85062E+00_JPRB, 0.86163E+00_JPRB ,&
&  0.85837E+00_JPRB, 0.90318E+00_JPRB, 0.96461E+00_JPRB, 0.99939E+00_JPRB, 0.10054E+01_JPRB, 0.10501E+01_JPRB, 0.12045E+01_JPRB ,&
&  0.16049E+01_JPRB, 0.52899E+01_JPRB, 0.27694E+01_JPRB, 0.49472E+01_JPRB, 0.38828E+01_JPRB, 0.33313E+01_JPRB, 0.26483E+01_JPRB ,&
&  0.24108E+01_JPRB, 0.22695E+01_JPRB, 0.19691E+01_JPRB, 0.17202E+01_JPRB, 0.15257E+01_JPRB, 0.13595E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.11143E+01_JPRB, 0.11368E+01_JPRB, 0.12103E+01_JPRB, 0.12506E+01_JPRB, 0.12563E+01_JPRB, 0.12589E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.13575E+01_JPRB, 0.15452E+01_JPRB, 0.20638E+01_JPRB, 0.24228E+01_JPRB, 0.28047E+01_JPRB, 0.38934E+01_JPRB ,&
&  0.37832E+01_JPRB, 0.35757E+01_JPRB, 0.27782E+01_JPRB, 0.23105E+01_JPRB, 0.20025E+01_JPRB, 0.18147E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.14567E+01_JPRB, 0.13886E+01_JPRB, 0.13407E+01_JPRB, 0.13073E+01_JPRB, 0.12798E+01_JPRB, 0.12287E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.11497E+01_JPRB, 0.11635E+01_JPRB, 0.11821E+01_JPRB, 0.12030E+01_JPRB, 0.11946E+01_JPRB, 0.11850E+01_JPRB ,&
&  0.11966E+01_JPRB, 0.12395E+01_JPRB, 0.12436E+01_JPRB, 0.12562E+01_JPRB, 0.12456E+01_JPRB, 0.12203E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.12546E+01_JPRB, 0.12551E+01_JPRB, 0.11846E+01_JPRB, 0.11619E+01_JPRB, 0.11386E+01_JPRB, 0.11769E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10113E+01_JPRB, 0.10726E+01_JPRB, 0.10814E+01_JPRB, 0.10971E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.11087E+01_JPRB, 0.10877E+01_JPRB, 0.10534E+01_JPRB, 0.10044E+01_JPRB, 0.98593E+00_JPRB, 0.96489E+00_JPRB, 0.95168E+00_JPRB ,&
&  0.94599E+00_JPRB, 0.93036E+00_JPRB, 0.91405E+00_JPRB, 0.89114E+00_JPRB, 0.88142E+00_JPRB, 0.89280E+00_JPRB, 0.88770E+00_JPRB ,&
&  0.86968E+00_JPRB, 0.84565E+00_JPRB, 0.83663E+00_JPRB, 0.82699E+00_JPRB, 0.81636E+00_JPRB, 0.81637E+00_JPRB, 0.82724E+00_JPRB ,&
&  0.83408E+00_JPRB, 0.85506E+00_JPRB, 0.89400E+00_JPRB, 0.93105E+00_JPRB, 0.94117E+00_JPRB, 0.97201E+00_JPRB, 0.11586E+01_JPRB ,&
&  0.17220E+01_JPRB, 0.18536E+01_JPRB, 0.40935E+01_JPRB, 0.48242E+01_JPRB, 0.39762E+01_JPRB, 0.34435E+01_JPRB, 0.28893E+01_JPRB ,&
&  0.25363E+01_JPRB, 0.21192E+01_JPRB, 0.17792E+01_JPRB, 0.15587E+01_JPRB, 0.14319E+01_JPRB, 0.12896E+01_JPRB, 0.12080E+01_JPRB ,&
&  0.11673E+01_JPRB, 0.11431E+01_JPRB, 0.11428E+01_JPRB, 0.11597E+01_JPRB, 0.11577E+01_JPRB, 0.11517E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.12180E+01_JPRB, 0.13312E+01_JPRB, 0.16578E+01_JPRB, 0.22518E+01_JPRB, 0.28881E+01_JPRB, 0.38759E+01_JPRB ,&
&  0.43730E+01_JPRB, 0.40854E+01_JPRB, 0.30346E+01_JPRB, 0.24626E+01_JPRB, 0.21502E+01_JPRB, 0.19296E+01_JPRB, 0.18169E+01_JPRB ,&
&  0.17175E+01_JPRB, 0.16065E+01_JPRB, 0.15194E+01_JPRB, 0.15124E+01_JPRB, 0.14315E+01_JPRB, 0.13637E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.12185E+01_JPRB, 0.11821E+01_JPRB, 0.11600E+01_JPRB, 0.11749E+01_JPRB, 0.11806E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.11713E+01_JPRB, 0.11909E+01_JPRB, 0.12112E+01_JPRB, 0.12093E+01_JPRB, 0.12389E+01_JPRB, 0.12485E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.12764E+01_JPRB, 0.12292E+01_JPRB, 0.12314E+01_JPRB, 0.11852E+01_JPRB, 0.12320E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.10163E+01_JPRB, 0.10212E+01_JPRB, 0.10721E+01_JPRB, 0.11115E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.10995E+01_JPRB, 0.10811E+01_JPRB, 0.10579E+01_JPRB, 0.10305E+01_JPRB, 0.10060E+01_JPRB, 0.99090E+00_JPRB, 0.97441E+00_JPRB ,&
&  0.97187E+00_JPRB, 0.96967E+00_JPRB, 0.95657E+00_JPRB, 0.92695E+00_JPRB, 0.90203E+00_JPRB, 0.89582E+00_JPRB, 0.89945E+00_JPRB ,&
&  0.89242E+00_JPRB, 0.88068E+00_JPRB, 0.88304E+00_JPRB, 0.86518E+00_JPRB, 0.84308E+00_JPRB, 0.82890E+00_JPRB, 0.82432E+00_JPRB ,&
&  0.82996E+00_JPRB, 0.84209E+00_JPRB, 0.87211E+00_JPRB, 0.90234E+00_JPRB, 0.90928E+00_JPRB, 0.91036E+00_JPRB, 0.95206E+00_JPRB ,&
&  0.15038E+01_JPRB, 0.30719E+01_JPRB, 0.47263E+01_JPRB, 0.44184E+01_JPRB, 0.37873E+01_JPRB, 0.31759E+01_JPRB, 0.28503E+01_JPRB ,&
&  0.23661E+01_JPRB, 0.20653E+01_JPRB, 0.17649E+01_JPRB, 0.15282E+01_JPRB, 0.13626E+01_JPRB, 0.12601E+01_JPRB, 0.11971E+01_JPRB ,&
&  0.11471E+01_JPRB, 0.11218E+01_JPRB, 0.11186E+01_JPRB, 0.11156E+01_JPRB, 0.11063E+01_JPRB, 0.10893E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.11482E+01_JPRB, 0.12185E+01_JPRB, 0.13983E+01_JPRB, 0.20017E+01_JPRB, 0.23319E+01_JPRB, 0.34211E+01_JPRB ,&
&  0.45468E+01_JPRB, 0.38675E+01_JPRB, 0.30800E+01_JPRB, 0.25673E+01_JPRB, 0.22477E+01_JPRB, 0.20218E+01_JPRB, 0.18843E+01_JPRB ,&
&  0.17487E+01_JPRB, 0.16563E+01_JPRB, 0.15804E+01_JPRB, 0.15062E+01_JPRB, 0.14591E+01_JPRB, 0.13864E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.12587E+01_JPRB, 0.12134E+01_JPRB, 0.11626E+01_JPRB, 0.11512E+01_JPRB, 0.11524E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.11576E+01_JPRB, 0.11861E+01_JPRB, 0.11805E+01_JPRB, 0.12219E+01_JPRB, 0.12621E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.12392E+01_JPRB, 0.12581E+01_JPRB, 0.12511E+01_JPRB, 0.12934E+01_JPRB, 0.13667E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.10615E+01_JPRB, 0.10273E+01_JPRB, 0.10327E+01_JPRB, 0.10980E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.11202E+01_JPRB, 0.11061E+01_JPRB, 0.10786E+01_JPRB, 0.10551E+01_JPRB, 0.10439E+01_JPRB, 0.10253E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.98625E+00_JPRB, 0.97457E+00_JPRB, 0.95791E+00_JPRB, 0.93656E+00_JPRB, 0.91788E+00_JPRB, 0.90994E+00_JPRB ,&
&  0.90771E+00_JPRB, 0.89684E+00_JPRB, 0.89980E+00_JPRB, 0.90031E+00_JPRB, 0.89625E+00_JPRB, 0.88388E+00_JPRB, 0.87173E+00_JPRB ,&
&  0.86441E+00_JPRB, 0.85090E+00_JPRB, 0.84956E+00_JPRB, 0.87455E+00_JPRB, 0.89626E+00_JPRB, 0.90364E+00_JPRB, 0.91311E+00_JPRB ,&
&  0.11796E+01_JPRB, 0.23824E+01_JPRB, 0.34348E+01_JPRB, 0.36794E+01_JPRB, 0.34393E+01_JPRB, 0.29952E+01_JPRB, 0.25637E+01_JPRB ,&
&  0.21846E+01_JPRB, 0.18892E+01_JPRB, 0.16978E+01_JPRB, 0.15474E+01_JPRB, 0.14125E+01_JPRB, 0.13013E+01_JPRB, 0.12210E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.11154E+01_JPRB, 0.10931E+01_JPRB, 0.10816E+01_JPRB, 0.10722E+01_JPRB, 0.10640E+01_JPRB, 0.10698E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.11072E+01_JPRB, 0.11479E+01_JPRB, 0.12151E+01_JPRB, 0.15193E+01_JPRB, 0.20227E+01_JPRB, 0.27223E+01_JPRB ,&
&  0.29026E+01_JPRB, 0.29381E+01_JPRB, 0.27027E+01_JPRB, 0.23808E+01_JPRB, 0.21195E+01_JPRB, 0.19188E+01_JPRB, 0.17873E+01_JPRB ,&
&  0.16608E+01_JPRB, 0.15644E+01_JPRB, 0.14913E+01_JPRB, 0.14103E+01_JPRB, 0.13291E+01_JPRB, 0.12866E+01_JPRB, 0.12877E+01_JPRB ,&
&  0.12995E+01_JPRB, 0.12816E+01_JPRB, 0.12336E+01_JPRB, 0.11771E+01_JPRB, 0.11169E+01_JPRB, 0.10920E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10826E+01_JPRB, 0.10970E+01_JPRB, 0.11340E+01_JPRB, 0.11715E+01_JPRB, 0.11971E+01_JPRB, 0.12092E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.12509E+01_JPRB, 0.12311E+01_JPRB, 0.12848E+01_JPRB, 0.13323E+01_JPRB, 0.13964E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.11986E+01_JPRB, 0.11189E+01_JPRB, 0.10782E+01_JPRB, 0.10590E+01_JPRB, 0.11064E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11427E+01_JPRB, 0.11275E+01_JPRB, 0.11003E+01_JPRB, 0.10760E+01_JPRB, 0.10464E+01_JPRB, 0.10224E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.99091E+00_JPRB, 0.97087E+00_JPRB, 0.96016E+00_JPRB, 0.95069E+00_JPRB, 0.94552E+00_JPRB, 0.93429E+00_JPRB, 0.91790E+00_JPRB ,&
&  0.90066E+00_JPRB, 0.89006E+00_JPRB, 0.88585E+00_JPRB, 0.88879E+00_JPRB, 0.89775E+00_JPRB, 0.90372E+00_JPRB, 0.89861E+00_JPRB ,&
&  0.88928E+00_JPRB, 0.87430E+00_JPRB, 0.85706E+00_JPRB, 0.87476E+00_JPRB, 0.89442E+00_JPRB, 0.90265E+00_JPRB, 0.89956E+00_JPRB ,&
&  0.10578E+01_JPRB, 0.15932E+01_JPRB, 0.22770E+01_JPRB, 0.24702E+01_JPRB, 0.24697E+01_JPRB, 0.24725E+01_JPRB, 0.22511E+01_JPRB ,&
&  0.20166E+01_JPRB, 0.17685E+01_JPRB, 0.15527E+01_JPRB, 0.14327E+01_JPRB, 0.13551E+01_JPRB, 0.12746E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.10931E+01_JPRB, 0.10651E+01_JPRB, 0.10538E+01_JPRB, 0.10491E+01_JPRB, 0.10495E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.10602E+01_JPRB, 0.10731E+01_JPRB, 0.11080E+01_JPRB, 0.11833E+01_JPRB, 0.14047E+01_JPRB, 0.17372E+01_JPRB ,&
&  0.19562E+01_JPRB, 0.21101E+01_JPRB, 0.22169E+01_JPRB, 0.21934E+01_JPRB, 0.20343E+01_JPRB, 0.18479E+01_JPRB, 0.17108E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.14808E+01_JPRB, 0.13754E+01_JPRB, 0.12917E+01_JPRB, 0.12194E+01_JPRB, 0.11880E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.12151E+01_JPRB, 0.11770E+01_JPRB, 0.11311E+01_JPRB, 0.10737E+01_JPRB, 0.10369E+01_JPRB, 0.10288E+01_JPRB ,&
&  0.10351E+01_JPRB, 0.10541E+01_JPRB, 0.10841E+01_JPRB, 0.11192E+01_JPRB, 0.11487E+01_JPRB, 0.11648E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.11886E+01_JPRB, 0.11408E+01_JPRB, 0.12207E+01_JPRB, 0.13488E+01_JPRB, 0.13317E+01_JPRB, 0.12993E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.11537E+01_JPRB, 0.10925E+01_JPRB, 0.10566E+01_JPRB, 0.11024E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.99336E+00_JPRB, 0.10149E+01_JPRB, 0.10182E+01_JPRB, 0.10135E+01_JPRB, 0.99553E+00_JPRB, 0.96919E+00_JPRB, 0.94885E+00_JPRB ,&
&  0.93869E+00_JPRB, 0.92130E+00_JPRB, 0.91304E+00_JPRB, 0.90947E+00_JPRB, 0.91209E+00_JPRB, 0.91818E+00_JPRB, 0.91689E+00_JPRB ,&
&  0.90580E+00_JPRB, 0.88984E+00_JPRB, 0.87868E+00_JPRB, 0.87704E+00_JPRB, 0.88612E+00_JPRB, 0.89649E+00_JPRB, 0.90325E+00_JPRB ,&
&  0.90581E+00_JPRB, 0.89859E+00_JPRB, 0.89472E+00_JPRB, 0.90657E+00_JPRB, 0.91782E+00_JPRB, 0.91240E+00_JPRB, 0.89738E+00_JPRB ,&
&  0.96235E+00_JPRB, 0.10948E+01_JPRB, 0.13535E+01_JPRB, 0.15821E+01_JPRB, 0.17832E+01_JPRB, 0.18567E+01_JPRB, 0.18042E+01_JPRB ,&
&  0.16807E+01_JPRB, 0.15660E+01_JPRB, 0.14331E+01_JPRB, 0.13147E+01_JPRB, 0.12279E+01_JPRB, 0.11785E+01_JPRB, 0.11475E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.10615E+01_JPRB, 0.10248E+01_JPRB, 0.10038E+01_JPRB, 0.99556E+00_JPRB, 0.98830E+00_JPRB, 0.98052E+00_JPRB ,&
&  0.96990E+00_JPRB, 0.96238E+00_JPRB, 0.96696E+00_JPRB, 0.98686E+00_JPRB, 0.10242E+01_JPRB, 0.11175E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.14086E+01_JPRB, 0.15468E+01_JPRB, 0.16807E+01_JPRB, 0.17855E+01_JPRB, 0.17770E+01_JPRB, 0.17032E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.13462E+01_JPRB, 0.12215E+01_JPRB, 0.11452E+01_JPRB, 0.11181E+01_JPRB, 0.11173E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11483E+01_JPRB, 0.11368E+01_JPRB, 0.10965E+01_JPRB, 0.10552E+01_JPRB, 0.10271E+01_JPRB, 0.99761E+00_JPRB, 0.98812E+00_JPRB ,&
&  0.98725E+00_JPRB, 0.99832E+00_JPRB, 0.10211E+01_JPRB, 0.10446E+01_JPRB, 0.10763E+01_JPRB, 0.10984E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.11039E+01_JPRB, 0.11021E+01_JPRB, 0.11674E+01_JPRB, 0.11866E+01_JPRB, 0.11983E+01_JPRB, 0.11871E+01_JPRB ,&
&  0.11606E+01_JPRB, 0.11078E+01_JPRB, 0.10472E+01_JPRB, 0.10138E+01_JPRB, 0.93717E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.92769E+00_JPRB, 0.94627E+00_JPRB, 0.95042E+00_JPRB, 0.94732E+00_JPRB, 0.94072E+00_JPRB, 0.92275E+00_JPRB, 0.90163E+00_JPRB ,&
&  0.87944E+00_JPRB, 0.86975E+00_JPRB, 0.86576E+00_JPRB, 0.86928E+00_JPRB, 0.87439E+00_JPRB, 0.88245E+00_JPRB, 0.89402E+00_JPRB ,&
&  0.90482E+00_JPRB, 0.90931E+00_JPRB, 0.91252E+00_JPRB, 0.90971E+00_JPRB, 0.90720E+00_JPRB, 0.90888E+00_JPRB, 0.91271E+00_JPRB ,&
&  0.92113E+00_JPRB, 0.92342E+00_JPRB, 0.91907E+00_JPRB, 0.92851E+00_JPRB, 0.94478E+00_JPRB, 0.93618E+00_JPRB, 0.91997E+00_JPRB ,&
&  0.93812E+00_JPRB, 0.87919E+00_JPRB, 0.93979E+00_JPRB, 0.11396E+01_JPRB, 0.13336E+01_JPRB, 0.14880E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.14621E+01_JPRB, 0.13738E+01_JPRB, 0.12901E+01_JPRB, 0.12080E+01_JPRB, 0.11485E+01_JPRB, 0.10945E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10077E+01_JPRB, 0.97168E+00_JPRB, 0.93969E+00_JPRB, 0.91251E+00_JPRB, 0.88677E+00_JPRB, 0.87088E+00_JPRB ,&
&  0.86576E+00_JPRB, 0.87340E+00_JPRB, 0.88550E+00_JPRB, 0.90395E+00_JPRB, 0.93986E+00_JPRB, 0.98689E+00_JPRB, 0.10602E+01_JPRB ,&
&  0.11379E+01_JPRB, 0.12217E+01_JPRB, 0.13040E+01_JPRB, 0.13740E+01_JPRB, 0.14312E+01_JPRB, 0.14355E+01_JPRB, 0.13743E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.11876E+01_JPRB, 0.11272E+01_JPRB, 0.10861E+01_JPRB, 0.10774E+01_JPRB, 0.10877E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10487E+01_JPRB, 0.10095E+01_JPRB, 0.97322E+00_JPRB, 0.95348E+00_JPRB, 0.94151E+00_JPRB, 0.92788E+00_JPRB ,&
&  0.91699E+00_JPRB, 0.91566E+00_JPRB, 0.92512E+00_JPRB, 0.94740E+00_JPRB, 0.97678E+00_JPRB, 0.10089E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10374E+01_JPRB, 0.10467E+01_JPRB, 0.10583E+01_JPRB, 0.10539E+01_JPRB, 0.11083E+01_JPRB, 0.11279E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.11124E+01_JPRB, 0.10783E+01_JPRB, 0.10328E+01_JPRB, 0.85242E+00_JPRB, 0.90618E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.89202E+00_JPRB, 0.89259E+00_JPRB, 0.89331E+00_JPRB, 0.88610E+00_JPRB, 0.87968E+00_JPRB, 0.86911E+00_JPRB, 0.84945E+00_JPRB ,&
&  0.83124E+00_JPRB, 0.82391E+00_JPRB, 0.82324E+00_JPRB, 0.82403E+00_JPRB, 0.82645E+00_JPRB, 0.83213E+00_JPRB, 0.84298E+00_JPRB ,&
&  0.85614E+00_JPRB, 0.86978E+00_JPRB, 0.88578E+00_JPRB, 0.90021E+00_JPRB, 0.90249E+00_JPRB, 0.89815E+00_JPRB, 0.89726E+00_JPRB ,&
&  0.90249E+00_JPRB, 0.90072E+00_JPRB, 0.89287E+00_JPRB, 0.89468E+00_JPRB, 0.90577E+00_JPRB, 0.89941E+00_JPRB, 0.88283E+00_JPRB ,&
&  0.88419E+00_JPRB, 0.77118E+00_JPRB, 0.82856E+00_JPRB, 0.92763E+00_JPRB, 0.10671E+01_JPRB, 0.11849E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.11631E+01_JPRB, 0.11081E+01_JPRB, 0.10457E+01_JPRB, 0.10094E+01_JPRB, 0.98605E+00_JPRB, 0.96273E+00_JPRB ,&
&  0.92900E+00_JPRB, 0.89337E+00_JPRB, 0.85359E+00_JPRB, 0.81682E+00_JPRB, 0.78776E+00_JPRB, 0.77148E+00_JPRB, 0.76400E+00_JPRB ,&
&  0.76139E+00_JPRB, 0.77121E+00_JPRB, 0.77787E+00_JPRB, 0.79019E+00_JPRB, 0.81106E+00_JPRB, 0.84310E+00_JPRB, 0.88473E+00_JPRB ,&
&  0.93797E+00_JPRB, 0.99826E+00_JPRB, 0.10567E+01_JPRB, 0.10970E+01_JPRB, 0.11230E+01_JPRB, 0.11404E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.10469E+01_JPRB, 0.10386E+01_JPRB, 0.10368E+01_JPRB, 0.10242E+01_JPRB, 0.10118E+01_JPRB, 0.99337E+00_JPRB ,&
&  0.96510E+00_JPRB, 0.93189E+00_JPRB, 0.90365E+00_JPRB, 0.87626E+00_JPRB, 0.85838E+00_JPRB, 0.84915E+00_JPRB, 0.83092E+00_JPRB ,&
&  0.81913E+00_JPRB, 0.81811E+00_JPRB, 0.82764E+00_JPRB, 0.85010E+00_JPRB, 0.88069E+00_JPRB, 0.91047E+00_JPRB, 0.92945E+00_JPRB ,&
&  0.93879E+00_JPRB, 0.94785E+00_JPRB, 0.95297E+00_JPRB, 0.99026E+00_JPRB, 0.10071E+01_JPRB, 0.10195E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.10319E+01_JPRB, 0.10309E+01_JPRB, 0.79929E+00_JPRB, 0.86903E+00_JPRB, 0.87972E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.89935E+00_JPRB, 0.87460E+00_JPRB, 0.85977E+00_JPRB, 0.85024E+00_JPRB, 0.84295E+00_JPRB, 0.83439E+00_JPRB, 0.81904E+00_JPRB ,&
&  0.80293E+00_JPRB, 0.79425E+00_JPRB, 0.78699E+00_JPRB, 0.77818E+00_JPRB, 0.77949E+00_JPRB, 0.78546E+00_JPRB, 0.79662E+00_JPRB ,&
&  0.80876E+00_JPRB, 0.82406E+00_JPRB, 0.83849E+00_JPRB, 0.85650E+00_JPRB, 0.86769E+00_JPRB, 0.87496E+00_JPRB, 0.87826E+00_JPRB ,&
&  0.88028E+00_JPRB, 0.87996E+00_JPRB, 0.87138E+00_JPRB, 0.86048E+00_JPRB, 0.85869E+00_JPRB, 0.86323E+00_JPRB, 0.85911E+00_JPRB ,&
&  0.76225E+00_JPRB, 0.68900E+00_JPRB, 0.75785E+00_JPRB, 0.83915E+00_JPRB, 0.92742E+00_JPRB, 0.10049E+01_JPRB, 0.10415E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.98355E+00_JPRB, 0.94545E+00_JPRB, 0.90142E+00_JPRB, 0.87951E+00_JPRB, 0.86858E+00_JPRB, 0.85606E+00_JPRB ,&
&  0.84466E+00_JPRB, 0.83038E+00_JPRB, 0.81002E+00_JPRB, 0.77813E+00_JPRB, 0.75398E+00_JPRB, 0.73624E+00_JPRB, 0.73045E+00_JPRB ,&
&  0.73043E+00_JPRB, 0.73306E+00_JPRB, 0.73630E+00_JPRB, 0.73952E+00_JPRB, 0.74757E+00_JPRB, 0.76074E+00_JPRB, 0.77769E+00_JPRB ,&
&  0.80319E+00_JPRB, 0.83257E+00_JPRB, 0.87103E+00_JPRB, 0.89751E+00_JPRB, 0.91382E+00_JPRB, 0.92696E+00_JPRB, 0.94289E+00_JPRB ,&
&  0.94951E+00_JPRB, 0.94251E+00_JPRB, 0.91871E+00_JPRB, 0.90670E+00_JPRB, 0.90441E+00_JPRB, 0.90045E+00_JPRB, 0.88799E+00_JPRB ,&
&  0.86525E+00_JPRB, 0.83731E+00_JPRB, 0.80832E+00_JPRB, 0.79084E+00_JPRB, 0.78710E+00_JPRB, 0.78455E+00_JPRB, 0.77830E+00_JPRB ,&
&  0.77324E+00_JPRB, 0.77283E+00_JPRB, 0.77893E+00_JPRB, 0.79121E+00_JPRB, 0.80794E+00_JPRB, 0.82837E+00_JPRB, 0.84528E+00_JPRB ,&
&  0.86168E+00_JPRB, 0.87750E+00_JPRB, 0.88852E+00_JPRB, 0.91233E+00_JPRB, 0.93566E+00_JPRB, 0.94895E+00_JPRB, 0.95058E+00_JPRB ,&
&  0.95649E+00_JPRB, 0.95008E+00_JPRB, 0.94041E+00_JPRB, 0.93365E+00_JPRB, 0.92047E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.85139E+00_JPRB, 0.82904E+00_JPRB, 0.81512E+00_JPRB, 0.80817E+00_JPRB, 0.79753E+00_JPRB, 0.77717E+00_JPRB, 0.76049E+00_JPRB ,&
&  0.74471E+00_JPRB, 0.73538E+00_JPRB, 0.73078E+00_JPRB, 0.72936E+00_JPRB, 0.73512E+00_JPRB, 0.74084E+00_JPRB, 0.74805E+00_JPRB ,&
&  0.75977E+00_JPRB, 0.77841E+00_JPRB, 0.79404E+00_JPRB, 0.80787E+00_JPRB, 0.81988E+00_JPRB, 0.83156E+00_JPRB, 0.83383E+00_JPRB ,&
&  0.83547E+00_JPRB, 0.84214E+00_JPRB, 0.84271E+00_JPRB, 0.82895E+00_JPRB, 0.81515E+00_JPRB, 0.81993E+00_JPRB, 0.81835E+00_JPRB ,&
&  0.81020E+00_JPRB, 0.69785E+00_JPRB, 0.76061E+00_JPRB, 0.79589E+00_JPRB, 0.83707E+00_JPRB, 0.88043E+00_JPRB, 0.90382E+00_JPRB ,&
&  0.89760E+00_JPRB, 0.87177E+00_JPRB, 0.84486E+00_JPRB, 0.81398E+00_JPRB, 0.80435E+00_JPRB, 0.79549E+00_JPRB, 0.78331E+00_JPRB ,&
&  0.77504E+00_JPRB, 0.76726E+00_JPRB, 0.75605E+00_JPRB, 0.74143E+00_JPRB, 0.72354E+00_JPRB, 0.71001E+00_JPRB, 0.69782E+00_JPRB ,&
&  0.69313E+00_JPRB, 0.68406E+00_JPRB, 0.67633E+00_JPRB, 0.67698E+00_JPRB, 0.68927E+00_JPRB, 0.69607E+00_JPRB, 0.69843E+00_JPRB ,&
&  0.70230E+00_JPRB, 0.70932E+00_JPRB, 0.72407E+00_JPRB, 0.74410E+00_JPRB, 0.76048E+00_JPRB, 0.77255E+00_JPRB, 0.78863E+00_JPRB ,&
&  0.81289E+00_JPRB, 0.82946E+00_JPRB, 0.82879E+00_JPRB, 0.81389E+00_JPRB, 0.80360E+00_JPRB, 0.80101E+00_JPRB, 0.78786E+00_JPRB ,&
&  0.76651E+00_JPRB, 0.75116E+00_JPRB, 0.74520E+00_JPRB, 0.74256E+00_JPRB, 0.73792E+00_JPRB, 0.73168E+00_JPRB, 0.73069E+00_JPRB ,&
&  0.73184E+00_JPRB, 0.73196E+00_JPRB, 0.73106E+00_JPRB, 0.73399E+00_JPRB, 0.74055E+00_JPRB, 0.74921E+00_JPRB, 0.76109E+00_JPRB ,&
&  0.77804E+00_JPRB, 0.79372E+00_JPRB, 0.80223E+00_JPRB, 0.81595E+00_JPRB, 0.83209E+00_JPRB, 0.84062E+00_JPRB, 0.85764E+00_JPRB ,&
&  0.86111E+00_JPRB, 0.86038E+00_JPRB, 0.86046E+00_JPRB, 0.86572E+00_JPRB, 0.86631E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.80451E+00_JPRB, 0.78299E+00_JPRB, 0.76934E+00_JPRB, 0.75664E+00_JPRB, 0.74757E+00_JPRB, 0.72881E+00_JPRB, 0.71798E+00_JPRB ,&
&  0.71072E+00_JPRB, 0.70584E+00_JPRB, 0.70400E+00_JPRB, 0.70363E+00_JPRB, 0.71386E+00_JPRB, 0.72068E+00_JPRB, 0.72350E+00_JPRB ,&
&  0.73143E+00_JPRB, 0.74939E+00_JPRB, 0.76604E+00_JPRB, 0.77972E+00_JPRB, 0.79009E+00_JPRB, 0.79715E+00_JPRB, 0.79615E+00_JPRB ,&
&  0.79903E+00_JPRB, 0.80631E+00_JPRB, 0.80953E+00_JPRB, 0.79842E+00_JPRB, 0.78469E+00_JPRB, 0.77759E+00_JPRB, 0.77919E+00_JPRB ,&
&  0.76841E+00_JPRB, 0.73556E+00_JPRB, 0.75412E+00_JPRB, 0.76797E+00_JPRB, 0.78487E+00_JPRB, 0.79990E+00_JPRB, 0.81534E+00_JPRB ,&
&  0.82088E+00_JPRB, 0.80192E+00_JPRB, 0.77583E+00_JPRB, 0.75763E+00_JPRB, 0.76137E+00_JPRB, 0.75754E+00_JPRB, 0.74502E+00_JPRB ,&
&  0.73709E+00_JPRB, 0.72994E+00_JPRB, 0.72014E+00_JPRB, 0.70567E+00_JPRB, 0.69296E+00_JPRB, 0.68375E+00_JPRB, 0.67695E+00_JPRB ,&
&  0.66396E+00_JPRB, 0.65320E+00_JPRB, 0.64738E+00_JPRB, 0.64123E+00_JPRB, 0.63655E+00_JPRB, 0.64134E+00_JPRB, 0.64600E+00_JPRB ,&
&  0.64722E+00_JPRB, 0.65035E+00_JPRB, 0.66050E+00_JPRB, 0.67727E+00_JPRB, 0.69255E+00_JPRB, 0.70940E+00_JPRB, 0.72834E+00_JPRB ,&
&  0.74721E+00_JPRB, 0.76403E+00_JPRB, 0.77002E+00_JPRB, 0.77356E+00_JPRB, 0.76665E+00_JPRB, 0.75604E+00_JPRB, 0.73544E+00_JPRB ,&
&  0.71266E+00_JPRB, 0.70702E+00_JPRB, 0.70716E+00_JPRB, 0.71263E+00_JPRB, 0.71081E+00_JPRB, 0.70795E+00_JPRB, 0.70629E+00_JPRB ,&
&  0.70218E+00_JPRB, 0.69845E+00_JPRB, 0.69329E+00_JPRB, 0.68849E+00_JPRB, 0.69178E+00_JPRB, 0.69782E+00_JPRB, 0.71161E+00_JPRB ,&
&  0.73017E+00_JPRB, 0.74990E+00_JPRB, 0.76983E+00_JPRB, 0.78398E+00_JPRB, 0.78947E+00_JPRB, 0.79289E+00_JPRB, 0.79731E+00_JPRB ,&
&  0.79926E+00_JPRB, 0.79868E+00_JPRB, 0.80579E+00_JPRB, 0.81072E+00_JPRB, 0.81539E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.74233E+00_JPRB, 0.71711E+00_JPRB, 0.69824E+00_JPRB, 0.69109E+00_JPRB, 0.68716E+00_JPRB, 0.67448E+00_JPRB, 0.66837E+00_JPRB ,&
&  0.67379E+00_JPRB, 0.67523E+00_JPRB, 0.68185E+00_JPRB, 0.68105E+00_JPRB, 0.68017E+00_JPRB, 0.68280E+00_JPRB, 0.67518E+00_JPRB ,&
&  0.67528E+00_JPRB, 0.68455E+00_JPRB, 0.69551E+00_JPRB, 0.71027E+00_JPRB, 0.72261E+00_JPRB, 0.73185E+00_JPRB, 0.73943E+00_JPRB ,&
&  0.74221E+00_JPRB, 0.74839E+00_JPRB, 0.75164E+00_JPRB, 0.74694E+00_JPRB, 0.73518E+00_JPRB, 0.72585E+00_JPRB, 0.72404E+00_JPRB ,&
&  0.72219E+00_JPRB, 0.72257E+00_JPRB, 0.67890E+00_JPRB, 0.70469E+00_JPRB, 0.71308E+00_JPRB, 0.72022E+00_JPRB, 0.72700E+00_JPRB ,&
&  0.72576E+00_JPRB, 0.71326E+00_JPRB, 0.70604E+00_JPRB, 0.69168E+00_JPRB, 0.68969E+00_JPRB, 0.69537E+00_JPRB, 0.69118E+00_JPRB ,&
&  0.68333E+00_JPRB, 0.66958E+00_JPRB, 0.65334E+00_JPRB, 0.64155E+00_JPRB, 0.63842E+00_JPRB, 0.63546E+00_JPRB, 0.62607E+00_JPRB ,&
&  0.62001E+00_JPRB, 0.61898E+00_JPRB, 0.61713E+00_JPRB, 0.61089E+00_JPRB, 0.59564E+00_JPRB, 0.58202E+00_JPRB, 0.57983E+00_JPRB ,&
&  0.58410E+00_JPRB, 0.59226E+00_JPRB, 0.60782E+00_JPRB, 0.62779E+00_JPRB, 0.64898E+00_JPRB, 0.66881E+00_JPRB, 0.68655E+00_JPRB ,&
&  0.69393E+00_JPRB, 0.69802E+00_JPRB, 0.70212E+00_JPRB, 0.70741E+00_JPRB, 0.70822E+00_JPRB, 0.69955E+00_JPRB, 0.67786E+00_JPRB ,&
&  0.65515E+00_JPRB, 0.64881E+00_JPRB, 0.65265E+00_JPRB, 0.66175E+00_JPRB, 0.66619E+00_JPRB, 0.66565E+00_JPRB, 0.65908E+00_JPRB ,&
&  0.65475E+00_JPRB, 0.64615E+00_JPRB, 0.63695E+00_JPRB, 0.63236E+00_JPRB, 0.63189E+00_JPRB, 0.63803E+00_JPRB, 0.65595E+00_JPRB ,&
&  0.67757E+00_JPRB, 0.70152E+00_JPRB, 0.72233E+00_JPRB, 0.72999E+00_JPRB, 0.73446E+00_JPRB, 0.73349E+00_JPRB, 0.72436E+00_JPRB ,&
&  0.72885E+00_JPRB, 0.74831E+00_JPRB, 0.75857E+00_JPRB, 0.76035E+00_JPRB, 0.75794E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.68667E+00_JPRB, 0.65804E+00_JPRB, 0.64634E+00_JPRB, 0.64814E+00_JPRB, 0.65144E+00_JPRB, 0.64679E+00_JPRB, 0.65360E+00_JPRB ,&
&  0.66447E+00_JPRB, 0.67279E+00_JPRB, 0.67721E+00_JPRB, 0.67307E+00_JPRB, 0.67029E+00_JPRB, 0.66709E+00_JPRB, 0.65206E+00_JPRB ,&
&  0.64696E+00_JPRB, 0.64517E+00_JPRB, 0.64949E+00_JPRB, 0.65699E+00_JPRB, 0.66528E+00_JPRB, 0.67797E+00_JPRB, 0.68736E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.69702E+00_JPRB, 0.70108E+00_JPRB, 0.70112E+00_JPRB, 0.69725E+00_JPRB, 0.68758E+00_JPRB, 0.68015E+00_JPRB ,&
&  0.67565E+00_JPRB, 0.66571E+00_JPRB, 0.65519E+00_JPRB, 0.65418E+00_JPRB, 0.65785E+00_JPRB, 0.66170E+00_JPRB, 0.66323E+00_JPRB ,&
&  0.65339E+00_JPRB, 0.63879E+00_JPRB, 0.63850E+00_JPRB, 0.63788E+00_JPRB, 0.64218E+00_JPRB, 0.65399E+00_JPRB, 0.65665E+00_JPRB ,&
&  0.65172E+00_JPRB, 0.63563E+00_JPRB, 0.61484E+00_JPRB, 0.61041E+00_JPRB, 0.61223E+00_JPRB, 0.60383E+00_JPRB, 0.59588E+00_JPRB ,&
&  0.59444E+00_JPRB, 0.59145E+00_JPRB, 0.58549E+00_JPRB, 0.57727E+00_JPRB, 0.56419E+00_JPRB, 0.54727E+00_JPRB, 0.53872E+00_JPRB ,&
&  0.53839E+00_JPRB, 0.55453E+00_JPRB, 0.58349E+00_JPRB, 0.61581E+00_JPRB, 0.64601E+00_JPRB, 0.66051E+00_JPRB, 0.65984E+00_JPRB ,&
&  0.64742E+00_JPRB, 0.64425E+00_JPRB, 0.64952E+00_JPRB, 0.65425E+00_JPRB, 0.65549E+00_JPRB, 0.64437E+00_JPRB, 0.62603E+00_JPRB ,&
&  0.61254E+00_JPRB, 0.60287E+00_JPRB, 0.60200E+00_JPRB, 0.60725E+00_JPRB, 0.61215E+00_JPRB, 0.61330E+00_JPRB, 0.60734E+00_JPRB ,&
&  0.60123E+00_JPRB, 0.59471E+00_JPRB, 0.59055E+00_JPRB, 0.59262E+00_JPRB, 0.59705E+00_JPRB, 0.60188E+00_JPRB, 0.61552E+00_JPRB ,&
&  0.63524E+00_JPRB, 0.65458E+00_JPRB, 0.65881E+00_JPRB, 0.65457E+00_JPRB, 0.65413E+00_JPRB, 0.65130E+00_JPRB, 0.64856E+00_JPRB ,&
&  0.65865E+00_JPRB, 0.68146E+00_JPRB, 0.69800E+00_JPRB, 0.70714E+00_JPRB, 0.70725E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.60791E+00_JPRB, 0.59102E+00_JPRB, 0.59167E+00_JPRB, 0.59935E+00_JPRB, 0.60371E+00_JPRB, 0.61152E+00_JPRB, 0.63079E+00_JPRB ,&
&  0.64166E+00_JPRB, 0.64851E+00_JPRB, 0.65106E+00_JPRB, 0.64567E+00_JPRB, 0.64456E+00_JPRB, 0.63719E+00_JPRB, 0.62423E+00_JPRB ,&
&  0.61877E+00_JPRB, 0.61769E+00_JPRB, 0.61940E+00_JPRB, 0.62341E+00_JPRB, 0.62590E+00_JPRB, 0.62916E+00_JPRB, 0.63284E+00_JPRB ,&
&  0.63485E+00_JPRB, 0.63487E+00_JPRB, 0.63391E+00_JPRB, 0.63401E+00_JPRB, 0.63288E+00_JPRB, 0.62495E+00_JPRB, 0.61590E+00_JPRB ,&
&  0.60632E+00_JPRB, 0.59760E+00_JPRB, 0.59484E+00_JPRB, 0.59889E+00_JPRB, 0.60370E+00_JPRB, 0.60653E+00_JPRB, 0.60436E+00_JPRB ,&
&  0.60067E+00_JPRB, 0.60030E+00_JPRB, 0.60457E+00_JPRB, 0.61167E+00_JPRB, 0.61988E+00_JPRB, 0.62931E+00_JPRB, 0.63426E+00_JPRB ,&
&  0.62807E+00_JPRB, 0.60947E+00_JPRB, 0.59791E+00_JPRB, 0.59702E+00_JPRB, 0.59223E+00_JPRB, 0.57568E+00_JPRB, 0.56445E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.55030E+00_JPRB, 0.54073E+00_JPRB, 0.52802E+00_JPRB, 0.51656E+00_JPRB, 0.51222E+00_JPRB, 0.51099E+00_JPRB ,&
&  0.51387E+00_JPRB, 0.53288E+00_JPRB, 0.56820E+00_JPRB, 0.60210E+00_JPRB, 0.61667E+00_JPRB, 0.60997E+00_JPRB, 0.59253E+00_JPRB ,&
&  0.58421E+00_JPRB, 0.58050E+00_JPRB, 0.58194E+00_JPRB, 0.58202E+00_JPRB, 0.57884E+00_JPRB, 0.56975E+00_JPRB, 0.56199E+00_JPRB ,&
&  0.55114E+00_JPRB, 0.53936E+00_JPRB, 0.53473E+00_JPRB, 0.53220E+00_JPRB, 0.52945E+00_JPRB, 0.52982E+00_JPRB, 0.52932E+00_JPRB ,&
&  0.53029E+00_JPRB, 0.53428E+00_JPRB, 0.54556E+00_JPRB, 0.55105E+00_JPRB, 0.55083E+00_JPRB, 0.54024E+00_JPRB, 0.54159E+00_JPRB ,&
&  0.54492E+00_JPRB, 0.54288E+00_JPRB, 0.53827E+00_JPRB, 0.53812E+00_JPRB, 0.53823E+00_JPRB, 0.54049E+00_JPRB, 0.55289E+00_JPRB ,&
&  0.58033E+00_JPRB, 0.60397E+00_JPRB, 0.62332E+00_JPRB, 0.62888E+00_JPRB, 0.62506E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.54945E+00_JPRB, 0.55492E+00_JPRB, 0.56845E+00_JPRB, 0.57965E+00_JPRB, 0.58334E+00_JPRB, 0.59308E+00_JPRB, 0.60408E+00_JPRB ,&
&  0.61260E+00_JPRB, 0.61386E+00_JPRB, 0.60982E+00_JPRB, 0.59963E+00_JPRB, 0.58771E+00_JPRB, 0.57783E+00_JPRB, 0.57401E+00_JPRB ,&
&  0.57623E+00_JPRB, 0.57928E+00_JPRB, 0.58486E+00_JPRB, 0.58803E+00_JPRB, 0.59194E+00_JPRB, 0.59100E+00_JPRB, 0.59179E+00_JPRB ,&
&  0.59394E+00_JPRB, 0.59517E+00_JPRB, 0.59570E+00_JPRB, 0.59187E+00_JPRB, 0.58871E+00_JPRB, 0.58591E+00_JPRB, 0.57689E+00_JPRB ,&
&  0.56702E+00_JPRB, 0.56075E+00_JPRB, 0.56470E+00_JPRB, 0.57579E+00_JPRB, 0.58266E+00_JPRB, 0.57929E+00_JPRB, 0.57752E+00_JPRB ,&
&  0.57785E+00_JPRB, 0.58371E+00_JPRB, 0.59312E+00_JPRB, 0.60614E+00_JPRB, 0.61436E+00_JPRB, 0.62499E+00_JPRB, 0.62557E+00_JPRB ,&
&  0.61912E+00_JPRB, 0.60963E+00_JPRB, 0.60645E+00_JPRB, 0.60482E+00_JPRB, 0.57419E+00_JPRB, 0.54044E+00_JPRB, 0.52396E+00_JPRB ,&
&  0.51507E+00_JPRB, 0.50426E+00_JPRB, 0.48839E+00_JPRB, 0.47744E+00_JPRB, 0.47657E+00_JPRB, 0.48181E+00_JPRB, 0.49343E+00_JPRB ,&
&  0.50161E+00_JPRB, 0.51411E+00_JPRB, 0.52993E+00_JPRB, 0.54047E+00_JPRB, 0.54088E+00_JPRB, 0.52845E+00_JPRB, 0.51674E+00_JPRB ,&
&  0.50970E+00_JPRB, 0.50756E+00_JPRB, 0.50592E+00_JPRB, 0.50392E+00_JPRB, 0.50207E+00_JPRB, 0.50150E+00_JPRB, 0.50055E+00_JPRB ,&
&  0.49636E+00_JPRB, 0.48906E+00_JPRB, 0.48025E+00_JPRB, 0.47553E+00_JPRB, 0.47525E+00_JPRB, 0.47597E+00_JPRB, 0.47811E+00_JPRB ,&
&  0.48702E+00_JPRB, 0.49824E+00_JPRB, 0.50697E+00_JPRB, 0.51019E+00_JPRB, 0.50628E+00_JPRB, 0.49760E+00_JPRB, 0.49235E+00_JPRB ,&
&  0.48855E+00_JPRB, 0.48589E+00_JPRB, 0.48241E+00_JPRB, 0.48103E+00_JPRB, 0.48123E+00_JPRB, 0.48944E+00_JPRB, 0.50631E+00_JPRB ,&
&  0.52665E+00_JPRB, 0.55256E+00_JPRB, 0.56411E+00_JPRB, 0.56251E+00_JPRB, 0.55552E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.51788E+00_JPRB, 0.53118E+00_JPRB, 0.54780E+00_JPRB, 0.55941E+00_JPRB, 0.56423E+00_JPRB, 0.56940E+00_JPRB, 0.57157E+00_JPRB ,&
&  0.56578E+00_JPRB, 0.56316E+00_JPRB, 0.55749E+00_JPRB, 0.54962E+00_JPRB, 0.54280E+00_JPRB, 0.53949E+00_JPRB, 0.54002E+00_JPRB ,&
&  0.54223E+00_JPRB, 0.54416E+00_JPRB, 0.54086E+00_JPRB, 0.53928E+00_JPRB, 0.54350E+00_JPRB, 0.54921E+00_JPRB, 0.55304E+00_JPRB ,&
&  0.55488E+00_JPRB, 0.55672E+00_JPRB, 0.56176E+00_JPRB, 0.56491E+00_JPRB, 0.56499E+00_JPRB, 0.55820E+00_JPRB, 0.55300E+00_JPRB ,&
&  0.54774E+00_JPRB, 0.54587E+00_JPRB, 0.55178E+00_JPRB, 0.54414E+00_JPRB, 0.55842E+00_JPRB, 0.55973E+00_JPRB, 0.56469E+00_JPRB ,&
&  0.57246E+00_JPRB, 0.58624E+00_JPRB, 0.59756E+00_JPRB, 0.60653E+00_JPRB, 0.60915E+00_JPRB, 0.61208E+00_JPRB, 0.60195E+00_JPRB ,&
&  0.58808E+00_JPRB, 0.56947E+00_JPRB, 0.55063E+00_JPRB, 0.52956E+00_JPRB, 0.50834E+00_JPRB, 0.49192E+00_JPRB, 0.48421E+00_JPRB ,&
&  0.47485E+00_JPRB, 0.46077E+00_JPRB, 0.44746E+00_JPRB, 0.44247E+00_JPRB, 0.44724E+00_JPRB, 0.45744E+00_JPRB, 0.47280E+00_JPRB ,&
&  0.48219E+00_JPRB, 0.48448E+00_JPRB, 0.48134E+00_JPRB, 0.47745E+00_JPRB, 0.47336E+00_JPRB, 0.46567E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.45758E+00_JPRB, 0.45214E+00_JPRB, 0.44951E+00_JPRB, 0.45100E+00_JPRB, 0.45388E+00_JPRB, 0.45593E+00_JPRB, 0.45491E+00_JPRB ,&
&  0.45254E+00_JPRB, 0.44784E+00_JPRB, 0.43981E+00_JPRB, 0.43596E+00_JPRB, 0.43702E+00_JPRB, 0.44191E+00_JPRB, 0.44951E+00_JPRB ,&
&  0.45189E+00_JPRB, 0.45814E+00_JPRB, 0.46396E+00_JPRB, 0.46572E+00_JPRB, 0.46789E+00_JPRB, 0.46938E+00_JPRB, 0.46903E+00_JPRB ,&
&  0.46686E+00_JPRB, 0.46156E+00_JPRB, 0.45468E+00_JPRB, 0.45239E+00_JPRB, 0.45425E+00_JPRB, 0.46188E+00_JPRB, 0.47167E+00_JPRB ,&
&  0.48113E+00_JPRB, 0.49578E+00_JPRB, 0.50684E+00_JPRB, 0.50929E+00_JPRB, 0.51148E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.49599E+00_JPRB, 0.50788E+00_JPRB, 0.52169E+00_JPRB, 0.53104E+00_JPRB, 0.52980E+00_JPRB, 0.52947E+00_JPRB, 0.53522E+00_JPRB ,&
&  0.52829E+00_JPRB, 0.52053E+00_JPRB, 0.51448E+00_JPRB, 0.50856E+00_JPRB, 0.50678E+00_JPRB, 0.50586E+00_JPRB, 0.50922E+00_JPRB ,&
&  0.51173E+00_JPRB, 0.51094E+00_JPRB, 0.50963E+00_JPRB, 0.51075E+00_JPRB, 0.51779E+00_JPRB, 0.52651E+00_JPRB, 0.53385E+00_JPRB ,&
&  0.53560E+00_JPRB, 0.53961E+00_JPRB, 0.54582E+00_JPRB, 0.55112E+00_JPRB, 0.55373E+00_JPRB, 0.55516E+00_JPRB, 0.55371E+00_JPRB ,&
&  0.55081E+00_JPRB, 0.54737E+00_JPRB, 0.53024E+00_JPRB, 0.54256E+00_JPRB, 0.54706E+00_JPRB, 0.54754E+00_JPRB, 0.55194E+00_JPRB ,&
&  0.55933E+00_JPRB, 0.56847E+00_JPRB, 0.57305E+00_JPRB, 0.57473E+00_JPRB, 0.56997E+00_JPRB, 0.56032E+00_JPRB, 0.53553E+00_JPRB ,&
&  0.51474E+00_JPRB, 0.49338E+00_JPRB, 0.47998E+00_JPRB, 0.47150E+00_JPRB, 0.46453E+00_JPRB, 0.45869E+00_JPRB, 0.45368E+00_JPRB ,&
&  0.44670E+00_JPRB, 0.44055E+00_JPRB, 0.43677E+00_JPRB, 0.43657E+00_JPRB, 0.44041E+00_JPRB, 0.45102E+00_JPRB, 0.46409E+00_JPRB ,&
&  0.46951E+00_JPRB, 0.46112E+00_JPRB, 0.45270E+00_JPRB, 0.44330E+00_JPRB, 0.42719E+00_JPRB, 0.40080E+00_JPRB, 0.38428E+00_JPRB ,&
&  0.38073E+00_JPRB, 0.39330E+00_JPRB, 0.41364E+00_JPRB, 0.42006E+00_JPRB, 0.42609E+00_JPRB, 0.42361E+00_JPRB, 0.41804E+00_JPRB ,&
&  0.40789E+00_JPRB, 0.38563E+00_JPRB, 0.37467E+00_JPRB, 0.37188E+00_JPRB, 0.37020E+00_JPRB, 0.37142E+00_JPRB, 0.37441E+00_JPRB ,&
&  0.37919E+00_JPRB, 0.38592E+00_JPRB, 0.39057E+00_JPRB, 0.39229E+00_JPRB, 0.39498E+00_JPRB, 0.39519E+00_JPRB, 0.39397E+00_JPRB ,&
&  0.39063E+00_JPRB, 0.39805E+00_JPRB, 0.41728E+00_JPRB, 0.44471E+00_JPRB, 0.45140E+00_JPRB, 0.45968E+00_JPRB, 0.46860E+00_JPRB ,&
&  0.47416E+00_JPRB, 0.47997E+00_JPRB, 0.48095E+00_JPRB, 0.48174E+00_JPRB, 0.48692E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.47001E+00_JPRB, 0.47711E+00_JPRB, 0.48584E+00_JPRB, 0.49703E+00_JPRB, 0.49931E+00_JPRB, 0.50080E+00_JPRB, 0.50968E+00_JPRB ,&
&  0.50959E+00_JPRB, 0.50630E+00_JPRB, 0.50314E+00_JPRB, 0.50378E+00_JPRB, 0.50639E+00_JPRB, 0.50929E+00_JPRB, 0.50945E+00_JPRB ,&
&  0.50073E+00_JPRB, 0.49585E+00_JPRB, 0.49903E+00_JPRB, 0.50250E+00_JPRB, 0.50740E+00_JPRB, 0.51393E+00_JPRB, 0.51982E+00_JPRB ,&
&  0.52323E+00_JPRB, 0.52727E+00_JPRB, 0.53033E+00_JPRB, 0.53696E+00_JPRB, 0.54677E+00_JPRB, 0.55366E+00_JPRB, 0.55941E+00_JPRB ,&
&  0.54294E+00_JPRB, 0.49478E+00_JPRB, 0.46489E+00_JPRB, 0.47064E+00_JPRB, 0.51706E+00_JPRB, 0.52786E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.53139E+00_JPRB, 0.53874E+00_JPRB, 0.53833E+00_JPRB, 0.53275E+00_JPRB, 0.52070E+00_JPRB, 0.50125E+00_JPRB, 0.47767E+00_JPRB ,&
&  0.46211E+00_JPRB, 0.45216E+00_JPRB, 0.44530E+00_JPRB, 0.43810E+00_JPRB, 0.43440E+00_JPRB, 0.42950E+00_JPRB, 0.42170E+00_JPRB ,&
&  0.41848E+00_JPRB, 0.41752E+00_JPRB, 0.41878E+00_JPRB, 0.41986E+00_JPRB, 0.42129E+00_JPRB, 0.42183E+00_JPRB, 0.41658E+00_JPRB ,&
&  0.40460E+00_JPRB, 0.38833E+00_JPRB, 0.37298E+00_JPRB, 0.35622E+00_JPRB, 0.34221E+00_JPRB, 0.33383E+00_JPRB, 0.32685E+00_JPRB ,&
&  0.32437E+00_JPRB, 0.33131E+00_JPRB, 0.35035E+00_JPRB, 0.37138E+00_JPRB, 0.39997E+00_JPRB, 0.37419E+00_JPRB, 0.35095E+00_JPRB ,&
&  0.34052E+00_JPRB, 0.33537E+00_JPRB, 0.33258E+00_JPRB, 0.33301E+00_JPRB, 0.33500E+00_JPRB, 0.33557E+00_JPRB, 0.33577E+00_JPRB ,&
&  0.33746E+00_JPRB, 0.34035E+00_JPRB, 0.34229E+00_JPRB, 0.34401E+00_JPRB, 0.34663E+00_JPRB, 0.34960E+00_JPRB, 0.35111E+00_JPRB ,&
&  0.35201E+00_JPRB, 0.35739E+00_JPRB, 0.36770E+00_JPRB, 0.37975E+00_JPRB, 0.39146E+00_JPRB, 0.40361E+00_JPRB, 0.42091E+00_JPRB ,&
&  0.44175E+00_JPRB, 0.46548E+00_JPRB, 0.46582E+00_JPRB, 0.46720E+00_JPRB, 0.46740E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.45519E+00_JPRB, 0.46646E+00_JPRB, 0.47198E+00_JPRB, 0.47749E+00_JPRB, 0.47991E+00_JPRB, 0.48135E+00_JPRB, 0.48934E+00_JPRB ,&
&  0.49422E+00_JPRB, 0.49820E+00_JPRB, 0.50080E+00_JPRB, 0.50261E+00_JPRB, 0.49922E+00_JPRB, 0.49309E+00_JPRB, 0.49173E+00_JPRB ,&
&  0.49078E+00_JPRB, 0.49135E+00_JPRB, 0.49053E+00_JPRB, 0.49061E+00_JPRB, 0.49116E+00_JPRB, 0.49592E+00_JPRB, 0.50308E+00_JPRB ,&
&  0.50484E+00_JPRB, 0.50910E+00_JPRB, 0.52247E+00_JPRB, 0.53024E+00_JPRB, 0.53515E+00_JPRB, 0.53420E+00_JPRB, 0.52783E+00_JPRB ,&
&  0.49834E+00_JPRB, 0.46970E+00_JPRB, 0.44358E+00_JPRB, 0.44477E+00_JPRB, 0.49566E+00_JPRB, 0.50113E+00_JPRB, 0.50110E+00_JPRB ,&
&  0.49964E+00_JPRB, 0.50363E+00_JPRB, 0.50381E+00_JPRB, 0.48901E+00_JPRB, 0.47827E+00_JPRB, 0.46395E+00_JPRB, 0.45118E+00_JPRB ,&
&  0.43953E+00_JPRB, 0.43206E+00_JPRB, 0.41826E+00_JPRB, 0.38999E+00_JPRB, 0.37341E+00_JPRB, 0.36495E+00_JPRB, 0.35116E+00_JPRB ,&
&  0.34055E+00_JPRB, 0.33761E+00_JPRB, 0.33686E+00_JPRB, 0.33532E+00_JPRB, 0.33912E+00_JPRB, 0.34123E+00_JPRB, 0.33925E+00_JPRB ,&
&  0.33482E+00_JPRB, 0.33122E+00_JPRB, 0.32701E+00_JPRB, 0.32153E+00_JPRB, 0.31872E+00_JPRB, 0.31809E+00_JPRB, 0.31899E+00_JPRB ,&
&  0.32376E+00_JPRB, 0.33051E+00_JPRB, 0.34508E+00_JPRB, 0.35510E+00_JPRB, 0.34953E+00_JPRB, 0.33737E+00_JPRB, 0.32733E+00_JPRB ,&
&  0.32026E+00_JPRB, 0.31750E+00_JPRB, 0.31537E+00_JPRB, 0.31308E+00_JPRB, 0.31380E+00_JPRB, 0.31616E+00_JPRB, 0.31760E+00_JPRB ,&
&  0.31775E+00_JPRB, 0.31986E+00_JPRB, 0.32120E+00_JPRB, 0.32263E+00_JPRB, 0.32384E+00_JPRB, 0.32583E+00_JPRB, 0.32856E+00_JPRB ,&
&  0.33095E+00_JPRB, 0.33651E+00_JPRB, 0.34253E+00_JPRB, 0.34945E+00_JPRB, 0.35386E+00_JPRB, 0.35722E+00_JPRB, 0.36442E+00_JPRB ,&
&  0.37067E+00_JPRB, 0.38639E+00_JPRB, 0.41849E+00_JPRB, 0.45323E+00_JPRB, 0.44470E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.44254E+00_JPRB, 0.45149E+00_JPRB, 0.45924E+00_JPRB, 0.46528E+00_JPRB, 0.46932E+00_JPRB, 0.47031E+00_JPRB, 0.47327E+00_JPRB ,&
&  0.48108E+00_JPRB, 0.48576E+00_JPRB, 0.48962E+00_JPRB, 0.48862E+00_JPRB, 0.48807E+00_JPRB, 0.47020E+00_JPRB, 0.46654E+00_JPRB ,&
&  0.46029E+00_JPRB, 0.45588E+00_JPRB, 0.44949E+00_JPRB, 0.44923E+00_JPRB, 0.44891E+00_JPRB, 0.45119E+00_JPRB, 0.45211E+00_JPRB ,&
&  0.45299E+00_JPRB, 0.45204E+00_JPRB, 0.44925E+00_JPRB, 0.44760E+00_JPRB, 0.44872E+00_JPRB, 0.44972E+00_JPRB, 0.44647E+00_JPRB ,&
&  0.44186E+00_JPRB, 0.44055E+00_JPRB, 0.44349E+00_JPRB, 0.44984E+00_JPRB, 0.48882E+00_JPRB, 0.48272E+00_JPRB, 0.48100E+00_JPRB ,&
&  0.47766E+00_JPRB, 0.47331E+00_JPRB, 0.47247E+00_JPRB, 0.46554E+00_JPRB, 0.45566E+00_JPRB, 0.43237E+00_JPRB, 0.41950E+00_JPRB ,&
&  0.38563E+00_JPRB, 0.36737E+00_JPRB, 0.35143E+00_JPRB, 0.34113E+00_JPRB, 0.33016E+00_JPRB, 0.32378E+00_JPRB, 0.31826E+00_JPRB ,&
&  0.31400E+00_JPRB, 0.31049E+00_JPRB, 0.30734E+00_JPRB, 0.30808E+00_JPRB, 0.31002E+00_JPRB, 0.31089E+00_JPRB, 0.31187E+00_JPRB ,&
&  0.31241E+00_JPRB, 0.31293E+00_JPRB, 0.31353E+00_JPRB, 0.31505E+00_JPRB, 0.31712E+00_JPRB, 0.31987E+00_JPRB, 0.32340E+00_JPRB ,&
&  0.32903E+00_JPRB, 0.33330E+00_JPRB, 0.33469E+00_JPRB, 0.33769E+00_JPRB, 0.33737E+00_JPRB, 0.33084E+00_JPRB, 0.32370E+00_JPRB ,&
&  0.31613E+00_JPRB, 0.31218E+00_JPRB, 0.30824E+00_JPRB, 0.30542E+00_JPRB, 0.30471E+00_JPRB, 0.30460E+00_JPRB, 0.30669E+00_JPRB ,&
&  0.30902E+00_JPRB, 0.31118E+00_JPRB, 0.31271E+00_JPRB, 0.31431E+00_JPRB, 0.31639E+00_JPRB, 0.31786E+00_JPRB, 0.31980E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32574E+00_JPRB, 0.33081E+00_JPRB, 0.33674E+00_JPRB, 0.34473E+00_JPRB, 0.35615E+00_JPRB, 0.37554E+00_JPRB ,&
&  0.40011E+00_JPRB, 0.45396E+00_JPRB, 0.45732E+00_JPRB, 0.42921E+00_JPRB, 0.43581E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.41874E+00_JPRB, 0.42608E+00_JPRB, 0.43314E+00_JPRB, 0.44076E+00_JPRB, 0.45033E+00_JPRB, 0.45458E+00_JPRB, 0.45655E+00_JPRB ,&
&  0.45223E+00_JPRB, 0.44813E+00_JPRB, 0.43767E+00_JPRB, 0.42884E+00_JPRB, 0.41971E+00_JPRB, 0.41250E+00_JPRB, 0.40651E+00_JPRB ,&
&  0.40248E+00_JPRB, 0.39945E+00_JPRB, 0.39440E+00_JPRB, 0.39082E+00_JPRB, 0.38746E+00_JPRB, 0.38778E+00_JPRB, 0.38848E+00_JPRB ,&
&  0.38838E+00_JPRB, 0.38910E+00_JPRB, 0.38759E+00_JPRB, 0.39278E+00_JPRB, 0.40217E+00_JPRB, 0.41331E+00_JPRB, 0.42786E+00_JPRB ,&
&  0.44816E+00_JPRB, 0.45889E+00_JPRB, 0.47434E+00_JPRB, 0.47419E+00_JPRB, 0.47236E+00_JPRB, 0.46865E+00_JPRB, 0.46551E+00_JPRB ,&
&  0.46070E+00_JPRB, 0.45377E+00_JPRB, 0.44848E+00_JPRB, 0.44160E+00_JPRB, 0.41115E+00_JPRB, 0.38793E+00_JPRB, 0.36997E+00_JPRB ,&
&  0.35504E+00_JPRB, 0.34090E+00_JPRB, 0.32815E+00_JPRB, 0.32182E+00_JPRB, 0.31632E+00_JPRB, 0.31305E+00_JPRB, 0.30959E+00_JPRB ,&
&  0.30585E+00_JPRB, 0.30480E+00_JPRB, 0.30291E+00_JPRB, 0.30215E+00_JPRB, 0.30335E+00_JPRB, 0.30316E+00_JPRB, 0.30395E+00_JPRB ,&
&  0.30473E+00_JPRB, 0.30611E+00_JPRB, 0.30925E+00_JPRB, 0.31159E+00_JPRB, 0.31395E+00_JPRB, 0.31613E+00_JPRB, 0.31707E+00_JPRB ,&
&  0.31912E+00_JPRB, 0.32093E+00_JPRB, 0.32084E+00_JPRB, 0.31999E+00_JPRB, 0.31797E+00_JPRB, 0.31326E+00_JPRB, 0.30865E+00_JPRB ,&
&  0.30365E+00_JPRB, 0.30004E+00_JPRB, 0.29882E+00_JPRB, 0.29771E+00_JPRB, 0.29870E+00_JPRB, 0.29949E+00_JPRB, 0.30025E+00_JPRB ,&
&  0.30222E+00_JPRB, 0.30411E+00_JPRB, 0.30643E+00_JPRB, 0.30988E+00_JPRB, 0.31240E+00_JPRB, 0.31449E+00_JPRB, 0.31706E+00_JPRB ,&
&  0.31679E+00_JPRB, 0.32130E+00_JPRB, 0.32701E+00_JPRB, 0.33509E+00_JPRB, 0.35017E+00_JPRB, 0.36083E+00_JPRB, 0.38447E+00_JPRB ,&
&  0.41788E+00_JPRB, 0.47973E+00_JPRB, 0.47173E+00_JPRB, 0.40670E+00_JPRB, 0.41193E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.40577E+00_JPRB, 0.40941E+00_JPRB, 0.41275E+00_JPRB, 0.41696E+00_JPRB, 0.42207E+00_JPRB, 0.42681E+00_JPRB, 0.43081E+00_JPRB ,&
&  0.43351E+00_JPRB, 0.43141E+00_JPRB, 0.43272E+00_JPRB, 0.42770E+00_JPRB, 0.42268E+00_JPRB, 0.41604E+00_JPRB, 0.40965E+00_JPRB ,&
&  0.40334E+00_JPRB, 0.39421E+00_JPRB, 0.38454E+00_JPRB, 0.37606E+00_JPRB, 0.36881E+00_JPRB, 0.36342E+00_JPRB, 0.35931E+00_JPRB ,&
&  0.35646E+00_JPRB, 0.35736E+00_JPRB, 0.35947E+00_JPRB, 0.36559E+00_JPRB, 0.37581E+00_JPRB, 0.38809E+00_JPRB, 0.40705E+00_JPRB ,&
&  0.43339E+00_JPRB, 0.43548E+00_JPRB, 0.44173E+00_JPRB, 0.44089E+00_JPRB, 0.43671E+00_JPRB, 0.43509E+00_JPRB, 0.43208E+00_JPRB ,&
&  0.43284E+00_JPRB, 0.42361E+00_JPRB, 0.43319E+00_JPRB, 0.41495E+00_JPRB, 0.38633E+00_JPRB, 0.37096E+00_JPRB, 0.35237E+00_JPRB ,&
&  0.33954E+00_JPRB, 0.32855E+00_JPRB, 0.32041E+00_JPRB, 0.31743E+00_JPRB, 0.31322E+00_JPRB, 0.31130E+00_JPRB, 0.30707E+00_JPRB ,&
&  0.30283E+00_JPRB, 0.30105E+00_JPRB, 0.29778E+00_JPRB, 0.29781E+00_JPRB, 0.29806E+00_JPRB, 0.29742E+00_JPRB, 0.29754E+00_JPRB ,&
&  0.29801E+00_JPRB, 0.29935E+00_JPRB, 0.30176E+00_JPRB, 0.30392E+00_JPRB, 0.30522E+00_JPRB, 0.30596E+00_JPRB, 0.30574E+00_JPRB ,&
&  0.30627E+00_JPRB, 0.30590E+00_JPRB, 0.30493E+00_JPRB, 0.30347E+00_JPRB, 0.30116E+00_JPRB, 0.29873E+00_JPRB, 0.29635E+00_JPRB ,&
&  0.29437E+00_JPRB, 0.29352E+00_JPRB, 0.29271E+00_JPRB, 0.29278E+00_JPRB, 0.29324E+00_JPRB, 0.29507E+00_JPRB, 0.29679E+00_JPRB ,&
&  0.29879E+00_JPRB, 0.30174E+00_JPRB, 0.30427E+00_JPRB, 0.30741E+00_JPRB, 0.31018E+00_JPRB, 0.31152E+00_JPRB, 0.31582E+00_JPRB ,&
&  0.32143E+00_JPRB, 0.32933E+00_JPRB, 0.33868E+00_JPRB, 0.34739E+00_JPRB, 0.35842E+00_JPRB, 0.37391E+00_JPRB, 0.39439E+00_JPRB ,&
&  0.41465E+00_JPRB, 0.43435E+00_JPRB, 0.42673E+00_JPRB, 0.40142E+00_JPRB, 0.40172E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.40299E+00_JPRB, 0.40198E+00_JPRB, 0.40315E+00_JPRB, 0.40486E+00_JPRB, 0.40965E+00_JPRB, 0.41583E+00_JPRB, 0.42246E+00_JPRB ,&
&  0.42231E+00_JPRB, 0.42581E+00_JPRB, 0.42460E+00_JPRB, 0.42127E+00_JPRB, 0.41596E+00_JPRB, 0.41020E+00_JPRB, 0.40539E+00_JPRB ,&
&  0.39869E+00_JPRB, 0.39008E+00_JPRB, 0.38176E+00_JPRB, 0.37313E+00_JPRB, 0.36506E+00_JPRB, 0.35672E+00_JPRB, 0.35239E+00_JPRB ,&
&  0.34900E+00_JPRB, 0.34820E+00_JPRB, 0.35025E+00_JPRB, 0.35270E+00_JPRB, 0.35799E+00_JPRB, 0.36526E+00_JPRB, 0.37497E+00_JPRB ,&
&  0.38420E+00_JPRB, 0.39519E+00_JPRB, 0.40105E+00_JPRB, 0.40090E+00_JPRB, 0.40078E+00_JPRB, 0.39880E+00_JPRB, 0.39540E+00_JPRB ,&
&  0.38854E+00_JPRB, 0.38100E+00_JPRB, 0.37206E+00_JPRB, 0.36262E+00_JPRB, 0.35296E+00_JPRB, 0.34426E+00_JPRB, 0.33617E+00_JPRB ,&
&  0.32962E+00_JPRB, 0.32405E+00_JPRB, 0.31883E+00_JPRB, 0.31441E+00_JPRB, 0.31101E+00_JPRB, 0.30853E+00_JPRB, 0.30527E+00_JPRB ,&
&  0.30264E+00_JPRB, 0.30029E+00_JPRB, 0.29746E+00_JPRB, 0.29557E+00_JPRB, 0.29384E+00_JPRB, 0.29274E+00_JPRB, 0.29181E+00_JPRB ,&
&  0.29188E+00_JPRB, 0.29198E+00_JPRB, 0.29292E+00_JPRB, 0.29385E+00_JPRB, 0.29487E+00_JPRB, 0.29496E+00_JPRB, 0.29481E+00_JPRB ,&
&  0.29441E+00_JPRB, 0.29398E+00_JPRB, 0.29231E+00_JPRB, 0.29098E+00_JPRB, 0.29002E+00_JPRB, 0.28897E+00_JPRB, 0.28881E+00_JPRB ,&
&  0.28958E+00_JPRB, 0.29039E+00_JPRB, 0.29097E+00_JPRB, 0.29267E+00_JPRB, 0.29434E+00_JPRB, 0.29684E+00_JPRB, 0.29991E+00_JPRB ,&
&  0.30209E+00_JPRB, 0.30468E+00_JPRB, 0.30755E+00_JPRB, 0.31088E+00_JPRB, 0.31494E+00_JPRB, 0.31909E+00_JPRB, 0.32404E+00_JPRB ,&
&  0.32966E+00_JPRB, 0.33533E+00_JPRB, 0.34175E+00_JPRB, 0.34887E+00_JPRB, 0.35752E+00_JPRB, 0.36322E+00_JPRB, 0.37224E+00_JPRB ,&
&  0.37874E+00_JPRB, 0.37392E+00_JPRB, 0.41404E+00_JPRB, 0.40755E+00_JPRB, 0.40371E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.34707E+00_JPRB, 0.35351E+00_JPRB, 0.35766E+00_JPRB, 0.36156E+00_JPRB, 0.35273E+00_JPRB, 0.38980E+00_JPRB, 0.38782E+00_JPRB ,&
&  0.38890E+00_JPRB, 0.37796E+00_JPRB, 0.38011E+00_JPRB, 0.37975E+00_JPRB, 0.37754E+00_JPRB, 0.37405E+00_JPRB, 0.37073E+00_JPRB ,&
&  0.36777E+00_JPRB, 0.36338E+00_JPRB, 0.35804E+00_JPRB, 0.35341E+00_JPRB, 0.34980E+00_JPRB, 0.34675E+00_JPRB, 0.34408E+00_JPRB ,&
&  0.34345E+00_JPRB, 0.34479E+00_JPRB, 0.34730E+00_JPRB, 0.34531E+00_JPRB, 0.34577E+00_JPRB, 0.34725E+00_JPRB, 0.34924E+00_JPRB ,&
&  0.35102E+00_JPRB, 0.35271E+00_JPRB, 0.35268E+00_JPRB, 0.35084E+00_JPRB, 0.35019E+00_JPRB, 0.34815E+00_JPRB, 0.34618E+00_JPRB ,&
&  0.34363E+00_JPRB, 0.34082E+00_JPRB, 0.33746E+00_JPRB, 0.33478E+00_JPRB, 0.33179E+00_JPRB, 0.32812E+00_JPRB, 0.32461E+00_JPRB ,&
&  0.32161E+00_JPRB, 0.31889E+00_JPRB, 0.31574E+00_JPRB, 0.31282E+00_JPRB, 0.31002E+00_JPRB, 0.30718E+00_JPRB, 0.30458E+00_JPRB ,&
&  0.30260E+00_JPRB, 0.30092E+00_JPRB, 0.29927E+00_JPRB, 0.29753E+00_JPRB, 0.29586E+00_JPRB, 0.29426E+00_JPRB, 0.29299E+00_JPRB ,&
&  0.29294E+00_JPRB, 0.29226E+00_JPRB, 0.29160E+00_JPRB, 0.29133E+00_JPRB, 0.29153E+00_JPRB, 0.29105E+00_JPRB, 0.29087E+00_JPRB ,&
&  0.29083E+00_JPRB, 0.29088E+00_JPRB, 0.29112E+00_JPRB, 0.29143E+00_JPRB, 0.29186E+00_JPRB, 0.29244E+00_JPRB, 0.29339E+00_JPRB ,&
&  0.29452E+00_JPRB, 0.29589E+00_JPRB, 0.29738E+00_JPRB, 0.29888E+00_JPRB, 0.30043E+00_JPRB, 0.30206E+00_JPRB, 0.30390E+00_JPRB ,&
&  0.30573E+00_JPRB, 0.30751E+00_JPRB, 0.30922E+00_JPRB, 0.31090E+00_JPRB, 0.31383E+00_JPRB, 0.31661E+00_JPRB, 0.31958E+00_JPRB ,&
&  0.32280E+00_JPRB, 0.32662E+00_JPRB, 0.33065E+00_JPRB, 0.33505E+00_JPRB, 0.34008E+00_JPRB, 0.34106E+00_JPRB, 0.34176E+00_JPRB ,&
&  0.34197E+00_JPRB, 0.34504E+00_JPRB, 0.34622E+00_JPRB, 0.34520E+00_JPRB, 0.34434E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.32080E+00_JPRB, 0.32078E+00_JPRB, 0.32067E+00_JPRB, 0.32048E+00_JPRB, 0.31857E+00_JPRB, 0.31877E+00_JPRB, 0.31906E+00_JPRB ,&
&  0.31975E+00_JPRB, 0.32146E+00_JPRB, 0.32342E+00_JPRB, 0.32488E+00_JPRB, 0.32589E+00_JPRB, 0.32655E+00_JPRB, 0.32708E+00_JPRB ,&
&  0.32721E+00_JPRB, 0.32758E+00_JPRB, 0.32368E+00_JPRB, 0.32226E+00_JPRB, 0.32173E+00_JPRB, 0.32166E+00_JPRB, 0.32184E+00_JPRB ,&
&  0.32192E+00_JPRB, 0.32185E+00_JPRB, 0.32119E+00_JPRB, 0.32153E+00_JPRB, 0.32191E+00_JPRB, 0.32215E+00_JPRB, 0.32241E+00_JPRB ,&
&  0.32266E+00_JPRB, 0.32290E+00_JPRB, 0.32284E+00_JPRB, 0.32260E+00_JPRB, 0.32251E+00_JPRB, 0.32208E+00_JPRB, 0.32124E+00_JPRB ,&
&  0.32021E+00_JPRB, 0.31888E+00_JPRB, 0.31757E+00_JPRB, 0.31637E+00_JPRB, 0.31533E+00_JPRB, 0.31444E+00_JPRB, 0.31359E+00_JPRB ,&
&  0.31253E+00_JPRB, 0.31111E+00_JPRB, 0.30960E+00_JPRB, 0.30812E+00_JPRB, 0.30669E+00_JPRB, 0.30540E+00_JPRB, 0.30402E+00_JPRB ,&
&  0.30298E+00_JPRB, 0.30211E+00_JPRB, 0.30125E+00_JPRB, 0.30051E+00_JPRB, 0.29977E+00_JPRB, 0.29906E+00_JPRB, 0.29837E+00_JPRB ,&
&  0.29742E+00_JPRB, 0.29693E+00_JPRB, 0.29639E+00_JPRB, 0.29617E+00_JPRB, 0.29629E+00_JPRB, 0.29638E+00_JPRB, 0.29658E+00_JPRB ,&
&  0.29684E+00_JPRB, 0.29715E+00_JPRB, 0.29731E+00_JPRB, 0.29755E+00_JPRB, 0.29791E+00_JPRB, 0.29837E+00_JPRB, 0.29892E+00_JPRB ,&
&  0.29951E+00_JPRB, 0.30016E+00_JPRB, 0.30087E+00_JPRB, 0.30166E+00_JPRB, 0.30249E+00_JPRB, 0.30339E+00_JPRB, 0.30425E+00_JPRB ,&
&  0.30513E+00_JPRB, 0.30594E+00_JPRB, 0.30676E+00_JPRB, 0.30739E+00_JPRB, 0.30835E+00_JPRB, 0.30926E+00_JPRB, 0.31035E+00_JPRB ,&
&  0.31144E+00_JPRB, 0.31262E+00_JPRB, 0.31365E+00_JPRB, 0.31478E+00_JPRB, 0.31603E+00_JPRB, 0.31573E+00_JPRB, 0.31556E+00_JPRB ,&
&  0.31545E+00_JPRB, 0.31571E+00_JPRB, 0.31555E+00_JPRB, 0.31584E+00_JPRB, 0.31674E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB, 0.30391E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB, 0.75398E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.76993E+00_JPRB, 0.77009E+00_JPRB, 0.77020E+00_JPRB, 0.77005E+00_JPRB, 0.76982E+00_JPRB, 0.76976E+00_JPRB, 0.76962E+00_JPRB ,&
&  0.76946E+00_JPRB, 0.76923E+00_JPRB, 0.76931E+00_JPRB, 0.76894E+00_JPRB, 0.76860E+00_JPRB, 0.76799E+00_JPRB, 0.76742E+00_JPRB ,&
&  0.76678E+00_JPRB, 0.76622E+00_JPRB, 0.76562E+00_JPRB, 0.76521E+00_JPRB, 0.76489E+00_JPRB, 0.76484E+00_JPRB, 0.76474E+00_JPRB ,&
&  0.76450E+00_JPRB, 0.76386E+00_JPRB, 0.76343E+00_JPRB, 0.76252E+00_JPRB, 0.76150E+00_JPRB, 0.75987E+00_JPRB, 0.75801E+00_JPRB ,&
&  0.75556E+00_JPRB, 0.75331E+00_JPRB, 0.75147E+00_JPRB, 0.74991E+00_JPRB, 0.74867E+00_JPRB, 0.74779E+00_JPRB, 0.74706E+00_JPRB ,&
&  0.74620E+00_JPRB, 0.74558E+00_JPRB, 0.74505E+00_JPRB, 0.74452E+00_JPRB, 0.74360E+00_JPRB, 0.74267E+00_JPRB, 0.74172E+00_JPRB ,&
&  0.74068E+00_JPRB, 0.73943E+00_JPRB, 0.73854E+00_JPRB, 0.73777E+00_JPRB, 0.73680E+00_JPRB, 0.73507E+00_JPRB, 0.73339E+00_JPRB ,&
&  0.73185E+00_JPRB, 0.73019E+00_JPRB, 0.72838E+00_JPRB, 0.72666E+00_JPRB, 0.72544E+00_JPRB, 0.72429E+00_JPRB, 0.72312E+00_JPRB ,&
&  0.72289E+00_JPRB, 0.72367E+00_JPRB, 0.72444E+00_JPRB, 0.72498E+00_JPRB, 0.72601E+00_JPRB, 0.72718E+00_JPRB, 0.72846E+00_JPRB ,&
&  0.72974E+00_JPRB, 0.73130E+00_JPRB, 0.73342E+00_JPRB, 0.73537E+00_JPRB, 0.73756E+00_JPRB, 0.73979E+00_JPRB, 0.74230E+00_JPRB ,&
&  0.74465E+00_JPRB, 0.74704E+00_JPRB, 0.74914E+00_JPRB, 0.75146E+00_JPRB, 0.75298E+00_JPRB, 0.75447E+00_JPRB, 0.75554E+00_JPRB ,&
&  0.75667E+00_JPRB, 0.75757E+00_JPRB, 0.75861E+00_JPRB, 0.75945E+00_JPRB, 0.76054E+00_JPRB, 0.76142E+00_JPRB, 0.76235E+00_JPRB ,&
&  0.76323E+00_JPRB, 0.76421E+00_JPRB, 0.76498E+00_JPRB, 0.76569E+00_JPRB, 0.76618E+00_JPRB, 0.76692E+00_JPRB, 0.76749E+00_JPRB ,&
&  0.76802E+00_JPRB, 0.76857E+00_JPRB, 0.76913E+00_JPRB, 0.76964E+00_JPRB, 0.76992E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.76977E+00_JPRB, 0.77202E+00_JPRB, 0.77312E+00_JPRB, 0.77418E+00_JPRB, 0.77484E+00_JPRB, 0.77618E+00_JPRB, 0.77706E+00_JPRB ,&
&  0.77788E+00_JPRB, 0.77903E+00_JPRB, 0.78104E+00_JPRB, 0.78177E+00_JPRB, 0.78213E+00_JPRB, 0.78181E+00_JPRB, 0.78143E+00_JPRB ,&
&  0.78058E+00_JPRB, 0.77931E+00_JPRB, 0.77800E+00_JPRB, 0.77695E+00_JPRB, 0.77602E+00_JPRB, 0.77541E+00_JPRB, 0.77435E+00_JPRB ,&
&  0.77238E+00_JPRB, 0.76998E+00_JPRB, 0.76725E+00_JPRB, 0.76279E+00_JPRB, 0.75773E+00_JPRB, 0.75242E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.74044E+00_JPRB, 0.73565E+00_JPRB, 0.73121E+00_JPRB, 0.72711E+00_JPRB, 0.72574E+00_JPRB, 0.72434E+00_JPRB, 0.72288E+00_JPRB ,&
&  0.72148E+00_JPRB, 0.71895E+00_JPRB, 0.71821E+00_JPRB, 0.71779E+00_JPRB, 0.71784E+00_JPRB, 0.71902E+00_JPRB, 0.71982E+00_JPRB ,&
&  0.72034E+00_JPRB, 0.72002E+00_JPRB, 0.71944E+00_JPRB, 0.71988E+00_JPRB, 0.72103E+00_JPRB, 0.72123E+00_JPRB, 0.71996E+00_JPRB ,&
&  0.71890E+00_JPRB, 0.71709E+00_JPRB, 0.71493E+00_JPRB, 0.71241E+00_JPRB, 0.71125E+00_JPRB, 0.70956E+00_JPRB, 0.70765E+00_JPRB ,&
&  0.70652E+00_JPRB, 0.70648E+00_JPRB, 0.70674E+00_JPRB, 0.70725E+00_JPRB, 0.70865E+00_JPRB, 0.71035E+00_JPRB, 0.71207E+00_JPRB ,&
&  0.71393E+00_JPRB, 0.71703E+00_JPRB, 0.72042E+00_JPRB, 0.72361E+00_JPRB, 0.72705E+00_JPRB, 0.73077E+00_JPRB, 0.73466E+00_JPRB ,&
&  0.73793E+00_JPRB, 0.74051E+00_JPRB, 0.74192E+00_JPRB, 0.74431E+00_JPRB, 0.74655E+00_JPRB, 0.74948E+00_JPRB, 0.75191E+00_JPRB ,&
&  0.75485E+00_JPRB, 0.75697E+00_JPRB, 0.75841E+00_JPRB, 0.75982E+00_JPRB, 0.76107E+00_JPRB, 0.76195E+00_JPRB, 0.76239E+00_JPRB ,&
&  0.76257E+00_JPRB, 0.76277E+00_JPRB, 0.76280E+00_JPRB, 0.76267E+00_JPRB, 0.76256E+00_JPRB, 0.76252E+00_JPRB, 0.76254E+00_JPRB ,&
&  0.76297E+00_JPRB, 0.76418E+00_JPRB, 0.76587E+00_JPRB, 0.76703E+00_JPRB, 0.76827E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.76085E+00_JPRB, 0.76268E+00_JPRB, 0.76407E+00_JPRB, 0.76643E+00_JPRB, 0.77061E+00_JPRB, 0.77474E+00_JPRB, 0.77677E+00_JPRB ,&
&  0.77976E+00_JPRB, 0.78330E+00_JPRB, 0.78758E+00_JPRB, 0.78983E+00_JPRB, 0.79148E+00_JPRB, 0.79187E+00_JPRB, 0.79117E+00_JPRB ,&
&  0.78934E+00_JPRB, 0.78690E+00_JPRB, 0.78520E+00_JPRB, 0.78347E+00_JPRB, 0.78150E+00_JPRB, 0.77931E+00_JPRB, 0.77344E+00_JPRB ,&
&  0.76821E+00_JPRB, 0.76479E+00_JPRB, 0.75741E+00_JPRB, 0.74861E+00_JPRB, 0.74166E+00_JPRB, 0.69247E+00_JPRB, 0.67640E+00_JPRB ,&
&  0.67502E+00_JPRB, 0.67080E+00_JPRB, 0.67211E+00_JPRB, 0.67439E+00_JPRB, 0.68136E+00_JPRB, 0.67364E+00_JPRB, 0.66844E+00_JPRB ,&
&  0.63856E+00_JPRB, 0.63929E+00_JPRB, 0.63462E+00_JPRB, 0.63228E+00_JPRB, 0.63563E+00_JPRB, 0.63801E+00_JPRB, 0.64814E+00_JPRB ,&
&  0.67559E+00_JPRB, 0.68338E+00_JPRB, 0.68685E+00_JPRB, 0.69003E+00_JPRB, 0.69487E+00_JPRB, 0.69819E+00_JPRB, 0.70064E+00_JPRB ,&
&  0.70176E+00_JPRB, 0.70179E+00_JPRB, 0.70213E+00_JPRB, 0.70166E+00_JPRB, 0.70124E+00_JPRB, 0.69981E+00_JPRB, 0.69898E+00_JPRB ,&
&  0.69850E+00_JPRB, 0.69974E+00_JPRB, 0.70179E+00_JPRB, 0.70399E+00_JPRB, 0.70697E+00_JPRB, 0.70904E+00_JPRB, 0.71106E+00_JPRB ,&
&  0.71317E+00_JPRB, 0.71512E+00_JPRB, 0.71718E+00_JPRB, 0.71940E+00_JPRB, 0.72216E+00_JPRB, 0.72405E+00_JPRB, 0.72653E+00_JPRB ,&
&  0.72869E+00_JPRB, 0.73145E+00_JPRB, 0.73388E+00_JPRB, 0.73919E+00_JPRB, 0.74376E+00_JPRB, 0.74974E+00_JPRB, 0.75510E+00_JPRB ,&
&  0.76080E+00_JPRB, 0.76466E+00_JPRB, 0.76790E+00_JPRB, 0.77021E+00_JPRB, 0.77261E+00_JPRB, 0.77443E+00_JPRB, 0.77515E+00_JPRB ,&
&  0.77509E+00_JPRB, 0.77369E+00_JPRB, 0.77192E+00_JPRB, 0.76942E+00_JPRB, 0.76709E+00_JPRB, 0.76404E+00_JPRB, 0.76137E+00_JPRB ,&
&  0.75822E+00_JPRB, 0.75706E+00_JPRB, 0.75704E+00_JPRB, 0.75825E+00_JPRB, 0.75943E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.76819E+00_JPRB, 0.76757E+00_JPRB, 0.76699E+00_JPRB, 0.76977E+00_JPRB, 0.77351E+00_JPRB, 0.77971E+00_JPRB, 0.78101E+00_JPRB ,&
&  0.78606E+00_JPRB, 0.79127E+00_JPRB, 0.79729E+00_JPRB, 0.79979E+00_JPRB, 0.80193E+00_JPRB, 0.80097E+00_JPRB, 0.79825E+00_JPRB ,&
&  0.79545E+00_JPRB, 0.79097E+00_JPRB, 0.78820E+00_JPRB, 0.78415E+00_JPRB, 0.77845E+00_JPRB, 0.77423E+00_JPRB, 0.76633E+00_JPRB ,&
&  0.76157E+00_JPRB, 0.75554E+00_JPRB, 0.70888E+00_JPRB, 0.69532E+00_JPRB, 0.67262E+00_JPRB, 0.65945E+00_JPRB, 0.65299E+00_JPRB ,&
&  0.64392E+00_JPRB, 0.63509E+00_JPRB, 0.62277E+00_JPRB, 0.60762E+00_JPRB, 0.59121E+00_JPRB, 0.56753E+00_JPRB, 0.55684E+00_JPRB ,&
&  0.54655E+00_JPRB, 0.54430E+00_JPRB, 0.54631E+00_JPRB, 0.55251E+00_JPRB, 0.56594E+00_JPRB, 0.57980E+00_JPRB, 0.59719E+00_JPRB ,&
&  0.62127E+00_JPRB, 0.64552E+00_JPRB, 0.65782E+00_JPRB, 0.65716E+00_JPRB, 0.67742E+00_JPRB, 0.68803E+00_JPRB, 0.69428E+00_JPRB ,&
&  0.69699E+00_JPRB, 0.69548E+00_JPRB, 0.70292E+00_JPRB, 0.70370E+00_JPRB, 0.70402E+00_JPRB, 0.70463E+00_JPRB, 0.70629E+00_JPRB ,&
&  0.70995E+00_JPRB, 0.71388E+00_JPRB, 0.71695E+00_JPRB, 0.71980E+00_JPRB, 0.72157E+00_JPRB, 0.72219E+00_JPRB, 0.72272E+00_JPRB ,&
&  0.72362E+00_JPRB, 0.72454E+00_JPRB, 0.72554E+00_JPRB, 0.72643E+00_JPRB, 0.72817E+00_JPRB, 0.72928E+00_JPRB, 0.72993E+00_JPRB ,&
&  0.73210E+00_JPRB, 0.73547E+00_JPRB, 0.73932E+00_JPRB, 0.73906E+00_JPRB, 0.74412E+00_JPRB, 0.75621E+00_JPRB, 0.76316E+00_JPRB ,&
&  0.77308E+00_JPRB, 0.78033E+00_JPRB, 0.78833E+00_JPRB, 0.79467E+00_JPRB, 0.80007E+00_JPRB, 0.80246E+00_JPRB, 0.80230E+00_JPRB ,&
&  0.80135E+00_JPRB, 0.79897E+00_JPRB, 0.79719E+00_JPRB, 0.79330E+00_JPRB, 0.78984E+00_JPRB, 0.78461E+00_JPRB, 0.78034E+00_JPRB ,&
&  0.77461E+00_JPRB, 0.77158E+00_JPRB, 0.76638E+00_JPRB, 0.76665E+00_JPRB, 0.76674E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.78067E+00_JPRB, 0.77836E+00_JPRB, 0.77908E+00_JPRB, 0.78491E+00_JPRB, 0.79612E+00_JPRB, 0.80904E+00_JPRB, 0.80943E+00_JPRB ,&
&  0.81031E+00_JPRB, 0.80953E+00_JPRB, 0.81273E+00_JPRB, 0.81231E+00_JPRB, 0.81140E+00_JPRB, 0.80772E+00_JPRB, 0.80459E+00_JPRB ,&
&  0.80155E+00_JPRB, 0.79385E+00_JPRB, 0.78406E+00_JPRB, 0.77829E+00_JPRB, 0.77555E+00_JPRB, 0.77468E+00_JPRB, 0.76869E+00_JPRB ,&
&  0.76072E+00_JPRB, 0.74339E+00_JPRB, 0.70895E+00_JPRB, 0.68806E+00_JPRB, 0.66423E+00_JPRB, 0.65594E+00_JPRB, 0.64173E+00_JPRB ,&
&  0.64555E+00_JPRB, 0.59298E+00_JPRB, 0.59419E+00_JPRB, 0.57168E+00_JPRB, 0.55602E+00_JPRB, 0.53841E+00_JPRB, 0.51912E+00_JPRB ,&
&  0.51325E+00_JPRB, 0.50592E+00_JPRB, 0.50458E+00_JPRB, 0.51356E+00_JPRB, 0.52545E+00_JPRB, 0.54652E+00_JPRB, 0.56906E+00_JPRB ,&
&  0.58386E+00_JPRB, 0.64037E+00_JPRB, 0.65671E+00_JPRB, 0.66399E+00_JPRB, 0.68222E+00_JPRB, 0.69472E+00_JPRB, 0.69681E+00_JPRB ,&
&  0.69868E+00_JPRB, 0.69946E+00_JPRB, 0.70904E+00_JPRB, 0.71517E+00_JPRB, 0.72449E+00_JPRB, 0.72639E+00_JPRB, 0.72839E+00_JPRB ,&
&  0.73343E+00_JPRB, 0.73955E+00_JPRB, 0.74571E+00_JPRB, 0.75154E+00_JPRB, 0.75625E+00_JPRB, 0.75928E+00_JPRB, 0.76050E+00_JPRB ,&
&  0.76064E+00_JPRB, 0.75958E+00_JPRB, 0.75920E+00_JPRB, 0.75979E+00_JPRB, 0.76026E+00_JPRB, 0.76295E+00_JPRB, 0.76451E+00_JPRB ,&
&  0.76649E+00_JPRB, 0.76667E+00_JPRB, 0.76531E+00_JPRB, 0.76611E+00_JPRB, 0.77120E+00_JPRB, 0.77614E+00_JPRB, 0.78584E+00_JPRB ,&
&  0.79933E+00_JPRB, 0.80742E+00_JPRB, 0.81575E+00_JPRB, 0.82308E+00_JPRB, 0.82948E+00_JPRB, 0.83051E+00_JPRB, 0.82899E+00_JPRB ,&
&  0.82703E+00_JPRB, 0.82459E+00_JPRB, 0.82187E+00_JPRB, 0.81682E+00_JPRB, 0.81165E+00_JPRB, 0.80382E+00_JPRB, 0.79669E+00_JPRB ,&
&  0.78845E+00_JPRB, 0.78479E+00_JPRB, 0.77803E+00_JPRB, 0.77788E+00_JPRB, 0.77888E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.80394E+00_JPRB, 0.80346E+00_JPRB, 0.80725E+00_JPRB, 0.81421E+00_JPRB, 0.82225E+00_JPRB, 0.83250E+00_JPRB, 0.83684E+00_JPRB ,&
&  0.84039E+00_JPRB, 0.84032E+00_JPRB, 0.83780E+00_JPRB, 0.82547E+00_JPRB, 0.81660E+00_JPRB, 0.81168E+00_JPRB, 0.80644E+00_JPRB ,&
&  0.79844E+00_JPRB, 0.79204E+00_JPRB, 0.78749E+00_JPRB, 0.78479E+00_JPRB, 0.78070E+00_JPRB, 0.78083E+00_JPRB, 0.77094E+00_JPRB ,&
&  0.74863E+00_JPRB, 0.72842E+00_JPRB, 0.69816E+00_JPRB, 0.67465E+00_JPRB, 0.65847E+00_JPRB, 0.64347E+00_JPRB, 0.64306E+00_JPRB ,&
&  0.63819E+00_JPRB, 0.63407E+00_JPRB, 0.62925E+00_JPRB, 0.62581E+00_JPRB, 0.62662E+00_JPRB, 0.53882E+00_JPRB, 0.51708E+00_JPRB ,&
&  0.50233E+00_JPRB, 0.49033E+00_JPRB, 0.48704E+00_JPRB, 0.48688E+00_JPRB, 0.49478E+00_JPRB, 0.52089E+00_JPRB, 0.55674E+00_JPRB ,&
&  0.57829E+00_JPRB, 0.65311E+00_JPRB, 0.66720E+00_JPRB, 0.68016E+00_JPRB, 0.70301E+00_JPRB, 0.71749E+00_JPRB, 0.72111E+00_JPRB ,&
&  0.73104E+00_JPRB, 0.74986E+00_JPRB, 0.77369E+00_JPRB, 0.79785E+00_JPRB, 0.80354E+00_JPRB, 0.81494E+00_JPRB, 0.81925E+00_JPRB ,&
&  0.81908E+00_JPRB, 0.81217E+00_JPRB, 0.81021E+00_JPRB, 0.81004E+00_JPRB, 0.80739E+00_JPRB, 0.80809E+00_JPRB, 0.80333E+00_JPRB ,&
&  0.80536E+00_JPRB, 0.80194E+00_JPRB, 0.80987E+00_JPRB, 0.81378E+00_JPRB, 0.82024E+00_JPRB, 0.82665E+00_JPRB, 0.82870E+00_JPRB ,&
&  0.82754E+00_JPRB, 0.82671E+00_JPRB, 0.81956E+00_JPRB, 0.80759E+00_JPRB, 0.80296E+00_JPRB, 0.79758E+00_JPRB, 0.80085E+00_JPRB ,&
&  0.80767E+00_JPRB, 0.82129E+00_JPRB, 0.82818E+00_JPRB, 0.83535E+00_JPRB, 0.84177E+00_JPRB, 0.84174E+00_JPRB, 0.84429E+00_JPRB ,&
&  0.84518E+00_JPRB, 0.84655E+00_JPRB, 0.84423E+00_JPRB, 0.83913E+00_JPRB, 0.83091E+00_JPRB, 0.81895E+00_JPRB, 0.80611E+00_JPRB ,&
&  0.79849E+00_JPRB, 0.79724E+00_JPRB, 0.79447E+00_JPRB, 0.79638E+00_JPRB, 0.80016E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.82543E+00_JPRB, 0.82414E+00_JPRB, 0.83121E+00_JPRB, 0.83946E+00_JPRB, 0.85046E+00_JPRB, 0.86101E+00_JPRB, 0.86444E+00_JPRB ,&
&  0.87478E+00_JPRB, 0.87267E+00_JPRB, 0.85815E+00_JPRB, 0.84047E+00_JPRB, 0.82026E+00_JPRB, 0.80539E+00_JPRB, 0.79851E+00_JPRB ,&
&  0.79740E+00_JPRB, 0.78722E+00_JPRB, 0.78078E+00_JPRB, 0.77602E+00_JPRB, 0.77170E+00_JPRB, 0.77015E+00_JPRB, 0.75737E+00_JPRB ,&
&  0.73183E+00_JPRB, 0.71592E+00_JPRB, 0.69847E+00_JPRB, 0.67449E+00_JPRB, 0.65315E+00_JPRB, 0.64019E+00_JPRB, 0.64213E+00_JPRB ,&
&  0.64287E+00_JPRB, 0.63317E+00_JPRB, 0.63352E+00_JPRB, 0.62884E+00_JPRB, 0.61527E+00_JPRB, 0.60800E+00_JPRB, 0.54014E+00_JPRB ,&
&  0.51584E+00_JPRB, 0.49105E+00_JPRB, 0.48180E+00_JPRB, 0.48054E+00_JPRB, 0.49698E+00_JPRB, 0.52509E+00_JPRB, 0.57578E+00_JPRB ,&
&  0.62012E+00_JPRB, 0.66094E+00_JPRB, 0.69119E+00_JPRB, 0.71282E+00_JPRB, 0.73540E+00_JPRB, 0.75700E+00_JPRB, 0.77245E+00_JPRB ,&
&  0.80082E+00_JPRB, 0.83987E+00_JPRB, 0.88074E+00_JPRB, 0.92013E+00_JPRB, 0.94727E+00_JPRB, 0.95821E+00_JPRB, 0.96776E+00_JPRB ,&
&  0.96441E+00_JPRB, 0.88816E+00_JPRB, 0.85887E+00_JPRB, 0.84123E+00_JPRB, 0.82888E+00_JPRB, 0.82967E+00_JPRB, 0.81977E+00_JPRB ,&
&  0.82001E+00_JPRB, 0.82663E+00_JPRB, 0.83004E+00_JPRB, 0.83458E+00_JPRB, 0.84112E+00_JPRB, 0.84369E+00_JPRB, 0.84948E+00_JPRB ,&
&  0.86077E+00_JPRB, 0.87854E+00_JPRB, 0.91262E+00_JPRB, 0.88620E+00_JPRB, 0.85307E+00_JPRB, 0.82103E+00_JPRB, 0.80412E+00_JPRB ,&
&  0.80186E+00_JPRB, 0.80779E+00_JPRB, 0.81893E+00_JPRB, 0.82354E+00_JPRB, 0.82938E+00_JPRB, 0.83447E+00_JPRB, 0.85079E+00_JPRB ,&
&  0.85699E+00_JPRB, 0.86230E+00_JPRB, 0.86294E+00_JPRB, 0.85946E+00_JPRB, 0.84917E+00_JPRB, 0.83452E+00_JPRB, 0.82020E+00_JPRB ,&
&  0.81514E+00_JPRB, 0.81679E+00_JPRB, 0.81790E+00_JPRB, 0.82161E+00_JPRB, 0.82459E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.83287E+00_JPRB, 0.84046E+00_JPRB, 0.85881E+00_JPRB, 0.87726E+00_JPRB, 0.89096E+00_JPRB, 0.87909E+00_JPRB, 0.86371E+00_JPRB ,&
&  0.86007E+00_JPRB, 0.86024E+00_JPRB, 0.84647E+00_JPRB, 0.83234E+00_JPRB, 0.81040E+00_JPRB, 0.79274E+00_JPRB, 0.78694E+00_JPRB ,&
&  0.78465E+00_JPRB, 0.77602E+00_JPRB, 0.76359E+00_JPRB, 0.74744E+00_JPRB, 0.74030E+00_JPRB, 0.73857E+00_JPRB, 0.72875E+00_JPRB ,&
&  0.71657E+00_JPRB, 0.70944E+00_JPRB, 0.69983E+00_JPRB, 0.68280E+00_JPRB, 0.66484E+00_JPRB, 0.65473E+00_JPRB, 0.64917E+00_JPRB ,&
&  0.64507E+00_JPRB, 0.64427E+00_JPRB, 0.64887E+00_JPRB, 0.64455E+00_JPRB, 0.64211E+00_JPRB, 0.63560E+00_JPRB, 0.58725E+00_JPRB ,&
&  0.53613E+00_JPRB, 0.50145E+00_JPRB, 0.48170E+00_JPRB, 0.48027E+00_JPRB, 0.50295E+00_JPRB, 0.55025E+00_JPRB, 0.60405E+00_JPRB ,&
&  0.65316E+00_JPRB, 0.70246E+00_JPRB, 0.74135E+00_JPRB, 0.76546E+00_JPRB, 0.78650E+00_JPRB, 0.81399E+00_JPRB, 0.84817E+00_JPRB ,&
&  0.89667E+00_JPRB, 0.95759E+00_JPRB, 0.10139E+01_JPRB, 0.10691E+01_JPRB, 0.11080E+01_JPRB, 0.10711E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.13995E+01_JPRB, 0.11632E+01_JPRB, 0.96927E+00_JPRB, 0.89719E+00_JPRB, 0.88233E+00_JPRB, 0.88759E+00_JPRB, 0.89505E+00_JPRB ,&
&  0.90784E+00_JPRB, 0.91151E+00_JPRB, 0.90557E+00_JPRB, 0.89975E+00_JPRB, 0.90327E+00_JPRB, 0.91129E+00_JPRB, 0.93250E+00_JPRB ,&
&  0.97103E+00_JPRB, 0.11548E+01_JPRB, 0.14308E+01_JPRB, 0.10966E+01_JPRB, 0.97746E+00_JPRB, 0.88921E+00_JPRB, 0.81080E+00_JPRB ,&
&  0.77946E+00_JPRB, 0.77211E+00_JPRB, 0.78026E+00_JPRB, 0.78904E+00_JPRB, 0.79210E+00_JPRB, 0.79942E+00_JPRB, 0.81179E+00_JPRB ,&
&  0.82976E+00_JPRB, 0.84743E+00_JPRB, 0.85872E+00_JPRB, 0.86468E+00_JPRB, 0.86012E+00_JPRB, 0.84419E+00_JPRB, 0.82170E+00_JPRB ,&
&  0.81957E+00_JPRB, 0.82951E+00_JPRB, 0.83667E+00_JPRB, 0.84078E+00_JPRB, 0.83715E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.83132E+00_JPRB, 0.86670E+00_JPRB, 0.91391E+00_JPRB, 0.92530E+00_JPRB, 0.91769E+00_JPRB, 0.86928E+00_JPRB, 0.82033E+00_JPRB ,&
&  0.79310E+00_JPRB, 0.78482E+00_JPRB, 0.78092E+00_JPRB, 0.76830E+00_JPRB, 0.75484E+00_JPRB, 0.74618E+00_JPRB, 0.73925E+00_JPRB ,&
&  0.73314E+00_JPRB, 0.72787E+00_JPRB, 0.72156E+00_JPRB, 0.71068E+00_JPRB, 0.70859E+00_JPRB, 0.71089E+00_JPRB, 0.70918E+00_JPRB ,&
&  0.70568E+00_JPRB, 0.70531E+00_JPRB, 0.69374E+00_JPRB, 0.67580E+00_JPRB, 0.66785E+00_JPRB, 0.66870E+00_JPRB, 0.67315E+00_JPRB ,&
&  0.66329E+00_JPRB, 0.65185E+00_JPRB, 0.65748E+00_JPRB, 0.66855E+00_JPRB, 0.66224E+00_JPRB, 0.65432E+00_JPRB, 0.60433E+00_JPRB ,&
&  0.54791E+00_JPRB, 0.51152E+00_JPRB, 0.50022E+00_JPRB, 0.52997E+00_JPRB, 0.58639E+00_JPRB, 0.66036E+00_JPRB, 0.70557E+00_JPRB ,&
&  0.73323E+00_JPRB, 0.77198E+00_JPRB, 0.81002E+00_JPRB, 0.85341E+00_JPRB, 0.90995E+00_JPRB, 0.97565E+00_JPRB, 0.10508E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.11482E+01_JPRB, 0.11527E+01_JPRB, 0.10442E+01_JPRB, 0.10175E+01_JPRB, 0.10196E+01_JPRB, 0.11153E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.11334E+01_JPRB, 0.10655E+01_JPRB, 0.10662E+01_JPRB, 0.10576E+01_JPRB, 0.10750E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.10690E+01_JPRB, 0.10341E+01_JPRB, 0.99405E+00_JPRB, 0.96160E+00_JPRB, 0.95814E+00_JPRB, 0.97991E+00_JPRB, 0.10021E+01_JPRB ,&
&  0.10565E+01_JPRB, 0.11758E+01_JPRB, 0.11343E+01_JPRB, 0.88545E+00_JPRB, 0.84653E+00_JPRB, 0.82913E+00_JPRB, 0.80902E+00_JPRB ,&
&  0.77617E+00_JPRB, 0.75683E+00_JPRB, 0.75587E+00_JPRB, 0.76031E+00_JPRB, 0.75555E+00_JPRB, 0.75192E+00_JPRB, 0.75495E+00_JPRB ,&
&  0.76344E+00_JPRB, 0.77919E+00_JPRB, 0.79990E+00_JPRB, 0.83226E+00_JPRB, 0.84516E+00_JPRB, 0.83902E+00_JPRB, 0.81763E+00_JPRB ,&
&  0.80758E+00_JPRB, 0.81241E+00_JPRB, 0.81603E+00_JPRB, 0.81599E+00_JPRB, 0.81904E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.95263E+00_JPRB, 0.96298E+00_JPRB, 0.96390E+00_JPRB, 0.96425E+00_JPRB, 0.91545E+00_JPRB, 0.88799E+00_JPRB, 0.84427E+00_JPRB ,&
&  0.83236E+00_JPRB, 0.80893E+00_JPRB, 0.76763E+00_JPRB, 0.71950E+00_JPRB, 0.69124E+00_JPRB, 0.67751E+00_JPRB, 0.67709E+00_JPRB ,&
&  0.68509E+00_JPRB, 0.69484E+00_JPRB, 0.69763E+00_JPRB, 0.68901E+00_JPRB, 0.69286E+00_JPRB, 0.69639E+00_JPRB, 0.69441E+00_JPRB ,&
&  0.69535E+00_JPRB, 0.70131E+00_JPRB, 0.69993E+00_JPRB, 0.69310E+00_JPRB, 0.69065E+00_JPRB, 0.69133E+00_JPRB, 0.68683E+00_JPRB ,&
&  0.67242E+00_JPRB, 0.66040E+00_JPRB, 0.66638E+00_JPRB, 0.69047E+00_JPRB, 0.67246E+00_JPRB, 0.66520E+00_JPRB, 0.60491E+00_JPRB ,&
&  0.55204E+00_JPRB, 0.54006E+00_JPRB, 0.57362E+00_JPRB, 0.65173E+00_JPRB, 0.68594E+00_JPRB, 0.72178E+00_JPRB, 0.74677E+00_JPRB ,&
&  0.74834E+00_JPRB, 0.76843E+00_JPRB, 0.83029E+00_JPRB, 0.94962E+00_JPRB, 0.10125E+01_JPRB, 0.10967E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.12996E+01_JPRB, 0.13346E+01_JPRB, 0.12681E+01_JPRB, 0.12942E+01_JPRB, 0.14082E+01_JPRB, 0.12628E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.11936E+01_JPRB, 0.12098E+01_JPRB, 0.12858E+01_JPRB, 0.12353E+01_JPRB, 0.12660E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.10955E+01_JPRB, 0.10521E+01_JPRB, 0.10341E+01_JPRB, 0.10251E+01_JPRB, 0.10330E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.11291E+01_JPRB, 0.10411E+01_JPRB, 0.88499E+00_JPRB, 0.82523E+00_JPRB, 0.80005E+00_JPRB, 0.78296E+00_JPRB ,&
&  0.76380E+00_JPRB, 0.75181E+00_JPRB, 0.75504E+00_JPRB, 0.75883E+00_JPRB, 0.75656E+00_JPRB, 0.74511E+00_JPRB, 0.73391E+00_JPRB ,&
&  0.72869E+00_JPRB, 0.73337E+00_JPRB, 0.73619E+00_JPRB, 0.76842E+00_JPRB, 0.80117E+00_JPRB, 0.82269E+00_JPRB, 0.81666E+00_JPRB ,&
&  0.81179E+00_JPRB, 0.82588E+00_JPRB, 0.84856E+00_JPRB, 0.89178E+00_JPRB, 0.91949E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.99296E+00_JPRB, 0.10088E+01_JPRB, 0.10164E+01_JPRB, 0.10033E+01_JPRB, 0.96103E+00_JPRB, 0.91190E+00_JPRB, 0.81681E+00_JPRB ,&
&  0.80918E+00_JPRB, 0.76463E+00_JPRB, 0.71982E+00_JPRB, 0.68734E+00_JPRB, 0.67548E+00_JPRB, 0.65893E+00_JPRB, 0.65179E+00_JPRB ,&
&  0.65204E+00_JPRB, 0.67093E+00_JPRB, 0.68807E+00_JPRB, 0.68472E+00_JPRB, 0.69051E+00_JPRB, 0.69461E+00_JPRB, 0.69017E+00_JPRB ,&
&  0.69748E+00_JPRB, 0.71179E+00_JPRB, 0.72529E+00_JPRB, 0.73310E+00_JPRB, 0.72035E+00_JPRB, 0.71080E+00_JPRB, 0.70164E+00_JPRB ,&
&  0.69524E+00_JPRB, 0.69114E+00_JPRB, 0.69268E+00_JPRB, 0.70646E+00_JPRB, 0.69001E+00_JPRB, 0.67128E+00_JPRB, 0.64920E+00_JPRB ,&
&  0.58205E+00_JPRB, 0.59472E+00_JPRB, 0.66353E+00_JPRB, 0.69617E+00_JPRB, 0.72923E+00_JPRB, 0.76404E+00_JPRB, 0.81730E+00_JPRB ,&
&  0.86430E+00_JPRB, 0.91455E+00_JPRB, 0.97858E+00_JPRB, 0.10629E+01_JPRB, 0.11732E+01_JPRB, 0.12951E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.15151E+01_JPRB, 0.12618E+01_JPRB, 0.12333E+01_JPRB, 0.14320E+01_JPRB, 0.17635E+01_JPRB, 0.15716E+01_JPRB, 0.14680E+01_JPRB ,&
&  0.13870E+01_JPRB, 0.13137E+01_JPRB, 0.12884E+01_JPRB, 0.13739E+01_JPRB, 0.13983E+01_JPRB, 0.14667E+01_JPRB, 0.15107E+01_JPRB ,&
&  0.13762E+01_JPRB, 0.12855E+01_JPRB, 0.11983E+01_JPRB, 0.11633E+01_JPRB, 0.11473E+01_JPRB, 0.11411E+01_JPRB, 0.11336E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.10924E+01_JPRB, 0.10272E+01_JPRB, 0.93947E+00_JPRB, 0.87631E+00_JPRB, 0.83956E+00_JPRB, 0.81414E+00_JPRB ,&
&  0.79425E+00_JPRB, 0.77401E+00_JPRB, 0.76014E+00_JPRB, 0.75637E+00_JPRB, 0.76044E+00_JPRB, 0.77606E+00_JPRB, 0.81282E+00_JPRB ,&
&  0.79443E+00_JPRB, 0.78326E+00_JPRB, 0.76371E+00_JPRB, 0.78472E+00_JPRB, 0.82638E+00_JPRB, 0.86323E+00_JPRB, 0.87061E+00_JPRB ,&
&  0.89041E+00_JPRB, 0.89883E+00_JPRB, 0.89323E+00_JPRB, 0.92244E+00_JPRB, 0.94721E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.10292E+01_JPRB, 0.10514E+01_JPRB, 0.10577E+01_JPRB, 0.10588E+01_JPRB, 0.10080E+01_JPRB, 0.92830E+00_JPRB, 0.85364E+00_JPRB ,&
&  0.91274E+00_JPRB, 0.84096E+00_JPRB, 0.82628E+00_JPRB, 0.71545E+00_JPRB, 0.67936E+00_JPRB, 0.66363E+00_JPRB, 0.64316E+00_JPRB ,&
&  0.62245E+00_JPRB, 0.64381E+00_JPRB, 0.67189E+00_JPRB, 0.68300E+00_JPRB, 0.69461E+00_JPRB, 0.70291E+00_JPRB, 0.69706E+00_JPRB ,&
&  0.70805E+00_JPRB, 0.73123E+00_JPRB, 0.76911E+00_JPRB, 0.77146E+00_JPRB, 0.74983E+00_JPRB, 0.74660E+00_JPRB, 0.74535E+00_JPRB ,&
&  0.72404E+00_JPRB, 0.71614E+00_JPRB, 0.73232E+00_JPRB, 0.73952E+00_JPRB, 0.73233E+00_JPRB, 0.69560E+00_JPRB, 0.67167E+00_JPRB ,&
&  0.67363E+00_JPRB, 0.66936E+00_JPRB, 0.71860E+00_JPRB, 0.74895E+00_JPRB, 0.78690E+00_JPRB, 0.83206E+00_JPRB, 0.87970E+00_JPRB ,&
&  0.94178E+00_JPRB, 0.10214E+01_JPRB, 0.11228E+01_JPRB, 0.12599E+01_JPRB, 0.14069E+01_JPRB, 0.15811E+01_JPRB, 0.17157E+01_JPRB ,&
&  0.18878E+01_JPRB, 0.14600E+01_JPRB, 0.17679E+01_JPRB, 0.18923E+01_JPRB, 0.21317E+01_JPRB, 0.21111E+01_JPRB, 0.20198E+01_JPRB ,&
&  0.20037E+01_JPRB, 0.17606E+01_JPRB, 0.16079E+01_JPRB, 0.15072E+01_JPRB, 0.15140E+01_JPRB, 0.16259E+01_JPRB, 0.17158E+01_JPRB ,&
&  0.18122E+01_JPRB, 0.17322E+01_JPRB, 0.15936E+01_JPRB, 0.14949E+01_JPRB, 0.14240E+01_JPRB, 0.13747E+01_JPRB, 0.13138E+01_JPRB ,&
&  0.12502E+01_JPRB, 0.11812E+01_JPRB, 0.11107E+01_JPRB, 0.10144E+01_JPRB, 0.96346E+00_JPRB, 0.91770E+00_JPRB, 0.87001E+00_JPRB ,&
&  0.84532E+00_JPRB, 0.80287E+00_JPRB, 0.76128E+00_JPRB, 0.73997E+00_JPRB, 0.72607E+00_JPRB, 0.73127E+00_JPRB, 0.76379E+00_JPRB ,&
&  0.78322E+00_JPRB, 0.78436E+00_JPRB, 0.84324E+00_JPRB, 0.89532E+00_JPRB, 0.92536E+00_JPRB, 0.95379E+00_JPRB, 0.10015E+01_JPRB ,&
&  0.93837E+00_JPRB, 0.94461E+00_JPRB, 0.99705E+00_JPRB, 0.10083E+01_JPRB, 0.10162E+01_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.10983E+01_JPRB, 0.11268E+01_JPRB, 0.11205E+01_JPRB, 0.11173E+01_JPRB, 0.10351E+01_JPRB, 0.99888E+00_JPRB, 0.91257E+00_JPRB ,&
&  0.93365E+00_JPRB, 0.89846E+00_JPRB, 0.81751E+00_JPRB, 0.79047E+00_JPRB, 0.79477E+00_JPRB, 0.71347E+00_JPRB, 0.64774E+00_JPRB ,&
&  0.61629E+00_JPRB, 0.62463E+00_JPRB, 0.67589E+00_JPRB, 0.70329E+00_JPRB, 0.70956E+00_JPRB, 0.71806E+00_JPRB, 0.71110E+00_JPRB ,&
&  0.76134E+00_JPRB, 0.77559E+00_JPRB, 0.81248E+00_JPRB, 0.82985E+00_JPRB, 0.82026E+00_JPRB, 0.82343E+00_JPRB, 0.83455E+00_JPRB ,&
&  0.79487E+00_JPRB, 0.78191E+00_JPRB, 0.77130E+00_JPRB, 0.73876E+00_JPRB, 0.79069E+00_JPRB, 0.76549E+00_JPRB, 0.73140E+00_JPRB ,&
&  0.72732E+00_JPRB, 0.75019E+00_JPRB, 0.78578E+00_JPRB, 0.82573E+00_JPRB, 0.85801E+00_JPRB, 0.89881E+00_JPRB, 0.95553E+00_JPRB ,&
&  0.10252E+01_JPRB, 0.11310E+01_JPRB, 0.12419E+01_JPRB, 0.14268E+01_JPRB, 0.16002E+01_JPRB, 0.16283E+01_JPRB, 0.19043E+01_JPRB ,&
&  0.20544E+01_JPRB, 0.22555E+01_JPRB, 0.24824E+01_JPRB, 0.23088E+01_JPRB, 0.25491E+01_JPRB, 0.24958E+01_JPRB, 0.22745E+01_JPRB ,&
&  0.22795E+01_JPRB, 0.20801E+01_JPRB, 0.20142E+01_JPRB, 0.20104E+01_JPRB, 0.21176E+01_JPRB, 0.20625E+01_JPRB, 0.20349E+01_JPRB ,&
&  0.21041E+01_JPRB, 0.23254E+01_JPRB, 0.21618E+01_JPRB, 0.21077E+01_JPRB, 0.19637E+01_JPRB, 0.18463E+01_JPRB, 0.16935E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.14702E+01_JPRB, 0.13567E+01_JPRB, 0.12652E+01_JPRB, 0.11409E+01_JPRB, 0.10435E+01_JPRB, 0.96415E+00_JPRB ,&
&  0.88231E+00_JPRB, 0.78782E+00_JPRB, 0.75568E+00_JPRB, 0.73548E+00_JPRB, 0.72863E+00_JPRB, 0.70468E+00_JPRB, 0.72023E+00_JPRB ,&
&  0.76644E+00_JPRB, 0.84276E+00_JPRB, 0.95656E+00_JPRB, 0.10280E+01_JPRB, 0.11187E+01_JPRB, 0.12048E+01_JPRB, 0.11060E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.10833E+01_JPRB, 0.10979E+01_JPRB, 0.10967E+01_JPRB, 0.10943E+01_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.12587E+01_JPRB, 0.12671E+01_JPRB, 0.12403E+01_JPRB, 0.11940E+01_JPRB, 0.10809E+01_JPRB, 0.10262E+01_JPRB, 0.95738E+00_JPRB ,&
&  0.95911E+00_JPRB, 0.91633E+00_JPRB, 0.83876E+00_JPRB, 0.80861E+00_JPRB, 0.80712E+00_JPRB, 0.78927E+00_JPRB, 0.71825E+00_JPRB ,&
&  0.67120E+00_JPRB, 0.69836E+00_JPRB, 0.69611E+00_JPRB, 0.75970E+00_JPRB, 0.81370E+00_JPRB, 0.78634E+00_JPRB, 0.80567E+00_JPRB ,&
&  0.86413E+00_JPRB, 0.87306E+00_JPRB, 0.87475E+00_JPRB, 0.87683E+00_JPRB, 0.88162E+00_JPRB, 0.89130E+00_JPRB, 0.88488E+00_JPRB ,&
&  0.83589E+00_JPRB, 0.80309E+00_JPRB, 0.77303E+00_JPRB, 0.76855E+00_JPRB, 0.82602E+00_JPRB, 0.84973E+00_JPRB, 0.82804E+00_JPRB ,&
&  0.80762E+00_JPRB, 0.80240E+00_JPRB, 0.81633E+00_JPRB, 0.86946E+00_JPRB, 0.91450E+00_JPRB, 0.95206E+00_JPRB, 0.10165E+01_JPRB ,&
&  0.11023E+01_JPRB, 0.12086E+01_JPRB, 0.13106E+01_JPRB, 0.14444E+01_JPRB, 0.14988E+01_JPRB, 0.17273E+01_JPRB, 0.21008E+01_JPRB ,&
&  0.22640E+01_JPRB, 0.24640E+01_JPRB, 0.26737E+01_JPRB, 0.29263E+01_JPRB, 0.31120E+01_JPRB, 0.29127E+01_JPRB, 0.27017E+01_JPRB ,&
&  0.26240E+01_JPRB, 0.25079E+01_JPRB, 0.30275E+01_JPRB, 0.28383E+01_JPRB, 0.26301E+01_JPRB, 0.25699E+01_JPRB, 0.24713E+01_JPRB ,&
&  0.24687E+01_JPRB, 0.25028E+01_JPRB, 0.24779E+01_JPRB, 0.24476E+01_JPRB, 0.23967E+01_JPRB, 0.23802E+01_JPRB, 0.22317E+01_JPRB ,&
&  0.21398E+01_JPRB, 0.19645E+01_JPRB, 0.16053E+01_JPRB, 0.13281E+01_JPRB, 0.10983E+01_JPRB, 0.10993E+01_JPRB, 0.10173E+01_JPRB ,&
&  0.90337E+00_JPRB, 0.78761E+00_JPRB, 0.76705E+00_JPRB, 0.74598E+00_JPRB, 0.74847E+00_JPRB, 0.73936E+00_JPRB, 0.73325E+00_JPRB ,&
&  0.78223E+00_JPRB, 0.90950E+00_JPRB, 0.10036E+01_JPRB, 0.11533E+01_JPRB, 0.13148E+01_JPRB, 0.14400E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.12183E+01_JPRB, 0.13244E+01_JPRB, 0.13445E+01_JPRB, 0.13325E+01_JPRB, 0.12935E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.15476E+01_JPRB, 0.14836E+01_JPRB, 0.14018E+01_JPRB, 0.12860E+01_JPRB, 0.11320E+01_JPRB, 0.10404E+01_JPRB, 0.96785E+00_JPRB ,&
&  0.95762E+00_JPRB, 0.91876E+00_JPRB, 0.85406E+00_JPRB, 0.82502E+00_JPRB, 0.81689E+00_JPRB, 0.80912E+00_JPRB, 0.81023E+00_JPRB ,&
&  0.73455E+00_JPRB, 0.81476E+00_JPRB, 0.77931E+00_JPRB, 0.86679E+00_JPRB, 0.10754E+01_JPRB, 0.92556E+00_JPRB, 0.93653E+00_JPRB ,&
&  0.93173E+00_JPRB, 0.96338E+00_JPRB, 0.94011E+00_JPRB, 0.93451E+00_JPRB, 0.97154E+00_JPRB, 0.99589E+00_JPRB, 0.97823E+00_JPRB ,&
&  0.92788E+00_JPRB, 0.86528E+00_JPRB, 0.83908E+00_JPRB, 0.83676E+00_JPRB, 0.89226E+00_JPRB, 0.93469E+00_JPRB, 0.94806E+00_JPRB ,&
&  0.93010E+00_JPRB, 0.91255E+00_JPRB, 0.89856E+00_JPRB, 0.92230E+00_JPRB, 0.96311E+00_JPRB, 0.10082E+01_JPRB, 0.10855E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.12705E+01_JPRB, 0.13705E+01_JPRB, 0.14912E+01_JPRB, 0.15208E+01_JPRB, 0.18198E+01_JPRB, 0.22886E+01_JPRB ,&
&  0.26596E+01_JPRB, 0.26644E+01_JPRB, 0.28630E+01_JPRB, 0.34522E+01_JPRB, 0.37310E+01_JPRB, 0.36134E+01_JPRB, 0.33578E+01_JPRB ,&
&  0.30979E+01_JPRB, 0.29020E+01_JPRB, 0.30029E+01_JPRB, 0.29784E+01_JPRB, 0.27745E+01_JPRB, 0.28443E+01_JPRB, 0.27826E+01_JPRB ,&
&  0.26778E+01_JPRB, 0.26503E+01_JPRB, 0.27036E+01_JPRB, 0.27406E+01_JPRB, 0.26091E+01_JPRB, 0.26417E+01_JPRB, 0.25413E+01_JPRB ,&
&  0.24573E+01_JPRB, 0.21094E+01_JPRB, 0.15392E+01_JPRB, 0.12590E+01_JPRB, 0.96990E+00_JPRB, 0.95580E+00_JPRB, 0.11223E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.10176E+01_JPRB, 0.92511E+00_JPRB, 0.87094E+00_JPRB, 0.84018E+00_JPRB, 0.88029E+00_JPRB, 0.91749E+00_JPRB ,&
&  0.95307E+00_JPRB, 0.11114E+01_JPRB, 0.13493E+01_JPRB, 0.15986E+01_JPRB, 0.17446E+01_JPRB, 0.17931E+01_JPRB, 0.16352E+01_JPRB ,&
&  0.17298E+01_JPRB, 0.16967E+01_JPRB, 0.16910E+01_JPRB, 0.16895E+01_JPRB, 0.16393E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.18328E+01_JPRB, 0.17251E+01_JPRB, 0.15666E+01_JPRB, 0.13763E+01_JPRB, 0.11624E+01_JPRB, 0.10353E+01_JPRB, 0.98477E+00_JPRB ,&
&  0.97200E+00_JPRB, 0.93725E+00_JPRB, 0.87819E+00_JPRB, 0.84340E+00_JPRB, 0.83068E+00_JPRB, 0.82577E+00_JPRB, 0.85312E+00_JPRB ,&
&  0.93721E+00_JPRB, 0.10763E+01_JPRB, 0.94652E+00_JPRB, 0.84414E+00_JPRB, 0.93559E+00_JPRB, 0.91912E+00_JPRB, 0.95099E+00_JPRB ,&
&  0.95509E+00_JPRB, 0.10427E+01_JPRB, 0.98886E+00_JPRB, 0.10098E+01_JPRB, 0.11222E+01_JPRB, 0.11797E+01_JPRB, 0.12024E+01_JPRB ,&
&  0.11895E+01_JPRB, 0.11435E+01_JPRB, 0.10894E+01_JPRB, 0.10988E+01_JPRB, 0.11525E+01_JPRB, 0.10983E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.10462E+01_JPRB, 0.10062E+01_JPRB, 0.10156E+01_JPRB, 0.10658E+01_JPRB, 0.11511E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.12773E+01_JPRB, 0.13369E+01_JPRB, 0.13697E+01_JPRB, 0.14690E+01_JPRB, 0.16222E+01_JPRB, 0.18365E+01_JPRB, 0.19704E+01_JPRB ,&
&  0.22474E+01_JPRB, 0.23718E+01_JPRB, 0.25127E+01_JPRB, 0.30644E+01_JPRB, 0.34701E+01_JPRB, 0.35580E+01_JPRB, 0.35472E+01_JPRB ,&
&  0.34648E+01_JPRB, 0.33666E+01_JPRB, 0.34303E+01_JPRB, 0.32916E+01_JPRB, 0.31854E+01_JPRB, 0.31967E+01_JPRB, 0.31590E+01_JPRB ,&
&  0.29340E+01_JPRB, 0.28591E+01_JPRB, 0.29070E+01_JPRB, 0.28277E+01_JPRB, 0.27339E+01_JPRB, 0.25964E+01_JPRB, 0.25963E+01_JPRB ,&
&  0.26068E+01_JPRB, 0.15903E+01_JPRB, 0.12351E+01_JPRB, 0.13165E+01_JPRB, 0.10368E+01_JPRB, 0.93725E+00_JPRB, 0.10419E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.10050E+01_JPRB, 0.10425E+01_JPRB, 0.10722E+01_JPRB, 0.10929E+01_JPRB, 0.12432E+01_JPRB, 0.13835E+01_JPRB ,&
&  0.15420E+01_JPRB, 0.15785E+01_JPRB, 0.19425E+01_JPRB, 0.21751E+01_JPRB, 0.22371E+01_JPRB, 0.22271E+01_JPRB, 0.21781E+01_JPRB ,&
&  0.21168E+01_JPRB, 0.20457E+01_JPRB, 0.19971E+01_JPRB, 0.19699E+01_JPRB, 0.19214E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.20966E+01_JPRB, 0.19749E+01_JPRB, 0.17656E+01_JPRB, 0.15126E+01_JPRB, 0.12374E+01_JPRB, 0.10778E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10110E+01_JPRB, 0.95603E+00_JPRB, 0.91014E+00_JPRB, 0.89064E+00_JPRB, 0.87904E+00_JPRB, 0.89000E+00_JPRB ,&
&  0.90374E+00_JPRB, 0.10722E+01_JPRB, 0.90842E+00_JPRB, 0.78697E+00_JPRB, 0.72075E+00_JPRB, 0.79081E+00_JPRB, 0.85563E+00_JPRB ,&
&  0.94443E+00_JPRB, 0.10454E+01_JPRB, 0.10847E+01_JPRB, 0.12643E+01_JPRB, 0.14209E+01_JPRB, 0.14924E+01_JPRB, 0.16072E+01_JPRB ,&
&  0.16194E+01_JPRB, 0.16119E+01_JPRB, 0.14676E+01_JPRB, 0.14756E+01_JPRB, 0.14234E+01_JPRB, 0.13017E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.12753E+01_JPRB, 0.12315E+01_JPRB, 0.11938E+01_JPRB, 0.12026E+01_JPRB, 0.12428E+01_JPRB, 0.12607E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.12528E+01_JPRB, 0.13033E+01_JPRB, 0.13883E+01_JPRB, 0.14725E+01_JPRB, 0.15961E+01_JPRB, 0.17615E+01_JPRB, 0.17997E+01_JPRB ,&
&  0.19415E+01_JPRB, 0.18455E+01_JPRB, 0.17185E+01_JPRB, 0.22489E+01_JPRB, 0.31916E+01_JPRB, 0.35345E+01_JPRB, 0.33969E+01_JPRB ,&
&  0.36868E+01_JPRB, 0.37775E+01_JPRB, 0.38267E+01_JPRB, 0.37094E+01_JPRB, 0.35650E+01_JPRB, 0.35044E+01_JPRB, 0.34782E+01_JPRB ,&
&  0.32941E+01_JPRB, 0.32388E+01_JPRB, 0.31959E+01_JPRB, 0.31530E+01_JPRB, 0.30218E+01_JPRB, 0.29863E+01_JPRB, 0.29861E+01_JPRB ,&
&  0.27383E+01_JPRB, 0.22968E+01_JPRB, 0.16011E+01_JPRB, 0.15547E+01_JPRB, 0.11476E+01_JPRB, 0.98811E+00_JPRB, 0.10636E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.10538E+01_JPRB, 0.11744E+01_JPRB, 0.12791E+01_JPRB, 0.16046E+01_JPRB, 0.19281E+01_JPRB, 0.22391E+01_JPRB ,&
&  0.23845E+01_JPRB, 0.22847E+01_JPRB, 0.26670E+01_JPRB, 0.27409E+01_JPRB, 0.27351E+01_JPRB, 0.27387E+01_JPRB, 0.27239E+01_JPRB ,&
&  0.26393E+01_JPRB, 0.25144E+01_JPRB, 0.24054E+01_JPRB, 0.22964E+01_JPRB, 0.21986E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.22381E+01_JPRB, 0.20841E+01_JPRB, 0.18420E+01_JPRB, 0.15705E+01_JPRB, 0.12952E+01_JPRB, 0.11120E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11343E+01_JPRB, 0.10995E+01_JPRB, 0.10328E+01_JPRB, 0.98944E+00_JPRB, 0.96608E+00_JPRB, 0.94327E+00_JPRB ,&
&  0.91747E+00_JPRB, 0.91156E+00_JPRB, 0.76562E+00_JPRB, 0.66586E+00_JPRB, 0.64449E+00_JPRB, 0.67313E+00_JPRB, 0.77784E+00_JPRB ,&
&  0.90327E+00_JPRB, 0.11324E+01_JPRB, 0.13731E+01_JPRB, 0.16207E+01_JPRB, 0.19124E+01_JPRB, 0.21809E+01_JPRB, 0.22789E+01_JPRB ,&
&  0.23720E+01_JPRB, 0.20314E+01_JPRB, 0.20137E+01_JPRB, 0.18771E+01_JPRB, 0.17687E+01_JPRB, 0.16666E+01_JPRB, 0.15898E+01_JPRB ,&
&  0.14835E+01_JPRB, 0.14184E+01_JPRB, 0.13531E+01_JPRB, 0.13158E+01_JPRB, 0.12983E+01_JPRB, 0.12852E+01_JPRB, 0.12786E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.14021E+01_JPRB, 0.14666E+01_JPRB, 0.15170E+01_JPRB, 0.16475E+01_JPRB, 0.18217E+01_JPRB, 0.18743E+01_JPRB ,&
&  0.17780E+01_JPRB, 0.17964E+01_JPRB, 0.22207E+01_JPRB, 0.24944E+01_JPRB, 0.28592E+01_JPRB, 0.33324E+01_JPRB, 0.36400E+01_JPRB ,&
&  0.38444E+01_JPRB, 0.37260E+01_JPRB, 0.37241E+01_JPRB, 0.34316E+01_JPRB, 0.38156E+01_JPRB, 0.37578E+01_JPRB, 0.36310E+01_JPRB ,&
&  0.34806E+01_JPRB, 0.35246E+01_JPRB, 0.33300E+01_JPRB, 0.32028E+01_JPRB, 0.32149E+01_JPRB, 0.33927E+01_JPRB, 0.30358E+01_JPRB ,&
&  0.25861E+01_JPRB, 0.28491E+01_JPRB, 0.25013E+01_JPRB, 0.20744E+01_JPRB, 0.16304E+01_JPRB, 0.14405E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.12151E+01_JPRB, 0.13610E+01_JPRB, 0.17680E+01_JPRB, 0.24314E+01_JPRB, 0.29291E+01_JPRB, 0.31531E+01_JPRB ,&
&  0.31490E+01_JPRB, 0.37360E+01_JPRB, 0.36744E+01_JPRB, 0.35024E+01_JPRB, 0.34126E+01_JPRB, 0.33738E+01_JPRB, 0.32671E+01_JPRB ,&
&  0.30325E+01_JPRB, 0.28375E+01_JPRB, 0.26711E+01_JPRB, 0.25198E+01_JPRB, 0.23659E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.22889E+01_JPRB, 0.21486E+01_JPRB, 0.19135E+01_JPRB, 0.16256E+01_JPRB, 0.13621E+01_JPRB, 0.11862E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.12084E+01_JPRB, 0.12192E+01_JPRB, 0.12090E+01_JPRB, 0.11765E+01_JPRB, 0.11216E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.86494E+00_JPRB, 0.70703E+00_JPRB, 0.66480E+00_JPRB, 0.66305E+00_JPRB, 0.67000E+00_JPRB, 0.74348E+00_JPRB ,&
&  0.90963E+00_JPRB, 0.13130E+01_JPRB, 0.17430E+01_JPRB, 0.22299E+01_JPRB, 0.26784E+01_JPRB, 0.29912E+01_JPRB, 0.29690E+01_JPRB ,&
&  0.27664E+01_JPRB, 0.27284E+01_JPRB, 0.25472E+01_JPRB, 0.23059E+01_JPRB, 0.21108E+01_JPRB, 0.19366E+01_JPRB, 0.17818E+01_JPRB ,&
&  0.16355E+01_JPRB, 0.15120E+01_JPRB, 0.14605E+01_JPRB, 0.14345E+01_JPRB, 0.14229E+01_JPRB, 0.14198E+01_JPRB, 0.14284E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.14978E+01_JPRB, 0.15332E+01_JPRB, 0.16152E+01_JPRB, 0.15409E+01_JPRB, 0.16496E+01_JPRB, 0.17668E+01_JPRB ,&
&  0.20180E+01_JPRB, 0.20256E+01_JPRB, 0.20484E+01_JPRB, 0.23248E+01_JPRB, 0.22740E+01_JPRB, 0.25960E+01_JPRB, 0.33427E+01_JPRB ,&
&  0.36019E+01_JPRB, 0.35876E+01_JPRB, 0.37417E+01_JPRB, 0.31414E+01_JPRB, 0.28629E+01_JPRB, 0.39541E+01_JPRB, 0.38881E+01_JPRB ,&
&  0.37119E+01_JPRB, 0.36235E+01_JPRB, 0.33999E+01_JPRB, 0.34338E+01_JPRB, 0.29862E+01_JPRB, 0.26806E+01_JPRB, 0.20579E+01_JPRB ,&
&  0.22367E+01_JPRB, 0.21993E+01_JPRB, 0.24457E+01_JPRB, 0.22600E+01_JPRB, 0.21763E+01_JPRB, 0.19697E+01_JPRB, 0.16719E+01_JPRB ,&
&  0.15234E+01_JPRB, 0.15176E+01_JPRB, 0.18315E+01_JPRB, 0.28041E+01_JPRB, 0.39631E+01_JPRB, 0.38378E+01_JPRB, 0.44696E+01_JPRB ,&
&  0.48094E+01_JPRB, 0.48575E+01_JPRB, 0.42137E+01_JPRB, 0.40975E+01_JPRB, 0.38425E+01_JPRB, 0.35944E+01_JPRB, 0.33037E+01_JPRB ,&
&  0.30657E+01_JPRB, 0.28945E+01_JPRB, 0.27408E+01_JPRB, 0.25776E+01_JPRB, 0.24140E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.23344E+01_JPRB, 0.22010E+01_JPRB, 0.19701E+01_JPRB, 0.16843E+01_JPRB, 0.14270E+01_JPRB, 0.12651E+01_JPRB, 0.12317E+01_JPRB ,&
&  0.12562E+01_JPRB, 0.12830E+01_JPRB, 0.12938E+01_JPRB, 0.12838E+01_JPRB, 0.12623E+01_JPRB, 0.12356E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.97626E+00_JPRB, 0.76232E+00_JPRB, 0.67402E+00_JPRB, 0.73157E+00_JPRB, 0.71805E+00_JPRB, 0.75388E+00_JPRB ,&
&  0.10462E+01_JPRB, 0.15808E+01_JPRB, 0.22420E+01_JPRB, 0.28681E+01_JPRB, 0.34050E+01_JPRB, 0.35469E+01_JPRB, 0.34074E+01_JPRB ,&
&  0.33886E+01_JPRB, 0.30970E+01_JPRB, 0.26899E+01_JPRB, 0.23452E+01_JPRB, 0.21091E+01_JPRB, 0.19487E+01_JPRB, 0.17809E+01_JPRB ,&
&  0.16155E+01_JPRB, 0.15095E+01_JPRB, 0.14586E+01_JPRB, 0.14288E+01_JPRB, 0.14263E+01_JPRB, 0.14136E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.15031E+01_JPRB, 0.15753E+01_JPRB, 0.16695E+01_JPRB, 0.16039E+01_JPRB, 0.16403E+01_JPRB, 0.18322E+01_JPRB ,&
&  0.19517E+01_JPRB, 0.19534E+01_JPRB, 0.19917E+01_JPRB, 0.21212E+01_JPRB, 0.23846E+01_JPRB, 0.25264E+01_JPRB, 0.30827E+01_JPRB ,&
&  0.29504E+01_JPRB, 0.27611E+01_JPRB, 0.26324E+01_JPRB, 0.23574E+01_JPRB, 0.27627E+01_JPRB, 0.38802E+01_JPRB, 0.40773E+01_JPRB ,&
&  0.38839E+01_JPRB, 0.37308E+01_JPRB, 0.35965E+01_JPRB, 0.32504E+01_JPRB, 0.27238E+01_JPRB, 0.21437E+01_JPRB, 0.25793E+01_JPRB ,&
&  0.28140E+01_JPRB, 0.29628E+01_JPRB, 0.28156E+01_JPRB, 0.23566E+01_JPRB, 0.21713E+01_JPRB, 0.25623E+01_JPRB, 0.23967E+01_JPRB ,&
&  0.20832E+01_JPRB, 0.21016E+01_JPRB, 0.32938E+01_JPRB, 0.47845E+01_JPRB, 0.53788E+01_JPRB, 0.51126E+01_JPRB, 0.56799E+01_JPRB ,&
&  0.54082E+01_JPRB, 0.51535E+01_JPRB, 0.47289E+01_JPRB, 0.42164E+01_JPRB, 0.37788E+01_JPRB, 0.34185E+01_JPRB, 0.31262E+01_JPRB ,&
&  0.29621E+01_JPRB, 0.28694E+01_JPRB, 0.27552E+01_JPRB, 0.26344E+01_JPRB, 0.24728E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.23674E+01_JPRB, 0.22175E+01_JPRB, 0.19763E+01_JPRB, 0.16884E+01_JPRB, 0.14797E+01_JPRB, 0.13448E+01_JPRB, 0.12903E+01_JPRB ,&
&  0.12929E+01_JPRB, 0.12979E+01_JPRB, 0.12985E+01_JPRB, 0.12857E+01_JPRB, 0.12706E+01_JPRB, 0.12547E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.12048E+01_JPRB, 0.92153E+00_JPRB, 0.78221E+00_JPRB, 0.79059E+00_JPRB, 0.82949E+00_JPRB, 0.84682E+00_JPRB ,&
&  0.13002E+01_JPRB, 0.20991E+01_JPRB, 0.27554E+01_JPRB, 0.33189E+01_JPRB, 0.38446E+01_JPRB, 0.34674E+01_JPRB, 0.33574E+01_JPRB ,&
&  0.32474E+01_JPRB, 0.29761E+01_JPRB, 0.26539E+01_JPRB, 0.23315E+01_JPRB, 0.20406E+01_JPRB, 0.18655E+01_JPRB, 0.17488E+01_JPRB ,&
&  0.16289E+01_JPRB, 0.15346E+01_JPRB, 0.14842E+01_JPRB, 0.14746E+01_JPRB, 0.14630E+01_JPRB, 0.14556E+01_JPRB, 0.14707E+01_JPRB ,&
&  0.14936E+01_JPRB, 0.15266E+01_JPRB, 0.15669E+01_JPRB, 0.16968E+01_JPRB, 0.17494E+01_JPRB, 0.17548E+01_JPRB, 0.20515E+01_JPRB ,&
&  0.20254E+01_JPRB, 0.20446E+01_JPRB, 0.20232E+01_JPRB, 0.21489E+01_JPRB, 0.22839E+01_JPRB, 0.24495E+01_JPRB, 0.28795E+01_JPRB ,&
&  0.27860E+01_JPRB, 0.27747E+01_JPRB, 0.29278E+01_JPRB, 0.29609E+01_JPRB, 0.28339E+01_JPRB, 0.31828E+01_JPRB, 0.42577E+01_JPRB ,&
&  0.35504E+01_JPRB, 0.35594E+01_JPRB, 0.35834E+01_JPRB, 0.35259E+01_JPRB, 0.19981E+01_JPRB, 0.15843E+01_JPRB, 0.24508E+01_JPRB ,&
&  0.26125E+01_JPRB, 0.16666E+01_JPRB, 0.12522E+01_JPRB, 0.16772E+01_JPRB, 0.13819E+01_JPRB, 0.17786E+01_JPRB, 0.27329E+01_JPRB ,&
&  0.30592E+01_JPRB, 0.29261E+01_JPRB, 0.44249E+01_JPRB, 0.53539E+01_JPRB, 0.55670E+01_JPRB, 0.60296E+01_JPRB, 0.60603E+01_JPRB ,&
&  0.57762E+01_JPRB, 0.47346E+01_JPRB, 0.46246E+01_JPRB, 0.40731E+01_JPRB, 0.36285E+01_JPRB, 0.33500E+01_JPRB, 0.31278E+01_JPRB ,&
&  0.28917E+01_JPRB, 0.26925E+01_JPRB, 0.25931E+01_JPRB, 0.25705E+01_JPRB, 0.24906E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.23385E+01_JPRB, 0.21864E+01_JPRB, 0.19377E+01_JPRB, 0.16601E+01_JPRB, 0.14926E+01_JPRB, 0.13750E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.12930E+01_JPRB, 0.12857E+01_JPRB, 0.12721E+01_JPRB, 0.12590E+01_JPRB, 0.12588E+01_JPRB, 0.12584E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.11730E+01_JPRB, 0.10941E+01_JPRB, 0.97618E+00_JPRB, 0.95840E+00_JPRB, 0.96928E+00_JPRB, 0.10942E+01_JPRB ,&
&  0.15749E+01_JPRB, 0.25125E+01_JPRB, 0.32169E+01_JPRB, 0.34544E+01_JPRB, 0.33122E+01_JPRB, 0.30088E+01_JPRB, 0.31418E+01_JPRB ,&
&  0.29607E+01_JPRB, 0.27075E+01_JPRB, 0.24352E+01_JPRB, 0.22216E+01_JPRB, 0.20293E+01_JPRB, 0.18956E+01_JPRB, 0.18051E+01_JPRB ,&
&  0.16850E+01_JPRB, 0.15906E+01_JPRB, 0.15543E+01_JPRB, 0.15285E+01_JPRB, 0.15084E+01_JPRB, 0.15018E+01_JPRB, 0.15023E+01_JPRB ,&
&  0.15163E+01_JPRB, 0.15428E+01_JPRB, 0.15932E+01_JPRB, 0.16926E+01_JPRB, 0.19131E+01_JPRB, 0.19610E+01_JPRB, 0.18835E+01_JPRB ,&
&  0.18223E+01_JPRB, 0.18730E+01_JPRB, 0.21132E+01_JPRB, 0.21073E+01_JPRB, 0.21585E+01_JPRB, 0.23291E+01_JPRB, 0.26521E+01_JPRB ,&
&  0.30306E+01_JPRB, 0.32879E+01_JPRB, 0.29932E+01_JPRB, 0.30865E+01_JPRB, 0.30025E+01_JPRB, 0.26140E+01_JPRB, 0.32002E+01_JPRB ,&
&  0.34982E+01_JPRB, 0.31101E+01_JPRB, 0.25835E+01_JPRB, 0.21615E+01_JPRB, 0.23388E+01_JPRB, 0.19175E+01_JPRB, 0.12260E+01_JPRB ,&
&  0.10844E+01_JPRB, 0.10073E+01_JPRB, 0.92026E+00_JPRB, 0.10053E+01_JPRB, 0.10314E+01_JPRB, 0.15937E+01_JPRB, 0.27179E+01_JPRB ,&
&  0.39141E+01_JPRB, 0.42892E+01_JPRB, 0.51961E+01_JPRB, 0.55430E+01_JPRB, 0.54623E+01_JPRB, 0.59692E+01_JPRB, 0.61264E+01_JPRB ,&
&  0.49167E+01_JPRB, 0.43437E+01_JPRB, 0.42184E+01_JPRB, 0.37173E+01_JPRB, 0.34379E+01_JPRB, 0.32315E+01_JPRB, 0.29766E+01_JPRB ,&
&  0.26658E+01_JPRB, 0.24008E+01_JPRB, 0.23170E+01_JPRB, 0.23733E+01_JPRB, 0.24291E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.22898E+01_JPRB, 0.21467E+01_JPRB, 0.19263E+01_JPRB, 0.16832E+01_JPRB, 0.15428E+01_JPRB, 0.14463E+01_JPRB, 0.13822E+01_JPRB ,&
&  0.13504E+01_JPRB, 0.13236E+01_JPRB, 0.13049E+01_JPRB, 0.12928E+01_JPRB, 0.12904E+01_JPRB, 0.12971E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.11904E+01_JPRB, 0.11900E+01_JPRB, 0.11365E+01_JPRB, 0.11632E+01_JPRB, 0.11583E+01_JPRB, 0.13578E+01_JPRB ,&
&  0.19496E+01_JPRB, 0.29458E+01_JPRB, 0.35611E+01_JPRB, 0.35110E+01_JPRB, 0.32924E+01_JPRB, 0.30712E+01_JPRB, 0.28215E+01_JPRB ,&
&  0.25516E+01_JPRB, 0.23944E+01_JPRB, 0.22382E+01_JPRB, 0.21193E+01_JPRB, 0.20198E+01_JPRB, 0.19421E+01_JPRB, 0.18451E+01_JPRB ,&
&  0.17420E+01_JPRB, 0.16866E+01_JPRB, 0.16247E+01_JPRB, 0.15846E+01_JPRB, 0.15478E+01_JPRB, 0.15206E+01_JPRB, 0.15060E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.15771E+01_JPRB, 0.16686E+01_JPRB, 0.18401E+01_JPRB, 0.17957E+01_JPRB, 0.16414E+01_JPRB, 0.17665E+01_JPRB ,&
&  0.18741E+01_JPRB, 0.20205E+01_JPRB, 0.20041E+01_JPRB, 0.21466E+01_JPRB, 0.21457E+01_JPRB, 0.21890E+01_JPRB, 0.25173E+01_JPRB ,&
&  0.29119E+01_JPRB, 0.33298E+01_JPRB, 0.28272E+01_JPRB, 0.27884E+01_JPRB, 0.31937E+01_JPRB, 0.27774E+01_JPRB, 0.28304E+01_JPRB ,&
&  0.31982E+01_JPRB, 0.28294E+01_JPRB, 0.26049E+01_JPRB, 0.24486E+01_JPRB, 0.34378E+01_JPRB, 0.36536E+01_JPRB, 0.18566E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.11014E+01_JPRB, 0.98278E+00_JPRB, 0.93420E+00_JPRB, 0.10205E+01_JPRB, 0.17113E+01_JPRB, 0.34636E+01_JPRB ,&
&  0.41552E+01_JPRB, 0.53287E+01_JPRB, 0.60268E+01_JPRB, 0.61052E+01_JPRB, 0.57640E+01_JPRB, 0.55937E+01_JPRB, 0.48937E+01_JPRB ,&
&  0.47713E+01_JPRB, 0.43203E+01_JPRB, 0.37895E+01_JPRB, 0.33384E+01_JPRB, 0.29792E+01_JPRB, 0.26083E+01_JPRB, 0.23121E+01_JPRB ,&
&  0.21149E+01_JPRB, 0.20521E+01_JPRB, 0.20622E+01_JPRB, 0.21724E+01_JPRB, 0.23423E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.22418E+01_JPRB, 0.21624E+01_JPRB, 0.19689E+01_JPRB, 0.17510E+01_JPRB, 0.16157E+01_JPRB, 0.15226E+01_JPRB, 0.14618E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.13535E+01_JPRB, 0.13106E+01_JPRB, 0.12898E+01_JPRB, 0.12967E+01_JPRB, 0.12994E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.12654E+01_JPRB, 0.12045E+01_JPRB, 0.11805E+01_JPRB, 0.12521E+01_JPRB, 0.12954E+01_JPRB, 0.13572E+01_JPRB, 0.16556E+01_JPRB ,&
&  0.24727E+01_JPRB, 0.35332E+01_JPRB, 0.37837E+01_JPRB, 0.36257E+01_JPRB, 0.32513E+01_JPRB, 0.29681E+01_JPRB, 0.23704E+01_JPRB ,&
&  0.21008E+01_JPRB, 0.20963E+01_JPRB, 0.20432E+01_JPRB, 0.20728E+01_JPRB, 0.20657E+01_JPRB, 0.19960E+01_JPRB, 0.18924E+01_JPRB ,&
&  0.18038E+01_JPRB, 0.17071E+01_JPRB, 0.16407E+01_JPRB, 0.15879E+01_JPRB, 0.15443E+01_JPRB, 0.15083E+01_JPRB, 0.15193E+01_JPRB ,&
&  0.15783E+01_JPRB, 0.16449E+01_JPRB, 0.17746E+01_JPRB, 0.19026E+01_JPRB, 0.16018E+01_JPRB, 0.17267E+01_JPRB, 0.17829E+01_JPRB ,&
&  0.18245E+01_JPRB, 0.18785E+01_JPRB, 0.18256E+01_JPRB, 0.19215E+01_JPRB, 0.20654E+01_JPRB, 0.21547E+01_JPRB, 0.23722E+01_JPRB ,&
&  0.28712E+01_JPRB, 0.27293E+01_JPRB, 0.25042E+01_JPRB, 0.25625E+01_JPRB, 0.29009E+01_JPRB, 0.36707E+01_JPRB, 0.26165E+01_JPRB ,&
&  0.24339E+01_JPRB, 0.28333E+01_JPRB, 0.27930E+01_JPRB, 0.29497E+01_JPRB, 0.39060E+01_JPRB, 0.50232E+01_JPRB, 0.47303E+01_JPRB ,&
&  0.19362E+01_JPRB, 0.12742E+01_JPRB, 0.12581E+01_JPRB, 0.12305E+01_JPRB, 0.11966E+01_JPRB, 0.17803E+01_JPRB, 0.61806E+01_JPRB ,&
&  0.53146E+01_JPRB, 0.64147E+01_JPRB, 0.65182E+01_JPRB, 0.59555E+01_JPRB, 0.56535E+01_JPRB, 0.52294E+01_JPRB, 0.46720E+01_JPRB ,&
&  0.40832E+01_JPRB, 0.35499E+01_JPRB, 0.30175E+01_JPRB, 0.25057E+01_JPRB, 0.21120E+01_JPRB, 0.18645E+01_JPRB, 0.17699E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.18125E+01_JPRB, 0.18917E+01_JPRB, 0.20217E+01_JPRB, 0.22026E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.21986E+01_JPRB, 0.21667E+01_JPRB, 0.19810E+01_JPRB, 0.17887E+01_JPRB, 0.16589E+01_JPRB, 0.15834E+01_JPRB, 0.14939E+01_JPRB ,&
&  0.14546E+01_JPRB, 0.13709E+01_JPRB, 0.12945E+01_JPRB, 0.12539E+01_JPRB, 0.12562E+01_JPRB, 0.12529E+01_JPRB, 0.12542E+01_JPRB ,&
&  0.12361E+01_JPRB, 0.12062E+01_JPRB, 0.11896E+01_JPRB, 0.12195E+01_JPRB, 0.13552E+01_JPRB, 0.12780E+01_JPRB, 0.15533E+01_JPRB ,&
&  0.28212E+01_JPRB, 0.42292E+01_JPRB, 0.37459E+01_JPRB, 0.34638E+01_JPRB, 0.31177E+01_JPRB, 0.29293E+01_JPRB, 0.21133E+01_JPRB ,&
&  0.19317E+01_JPRB, 0.19693E+01_JPRB, 0.19863E+01_JPRB, 0.20449E+01_JPRB, 0.20306E+01_JPRB, 0.19658E+01_JPRB, 0.18802E+01_JPRB ,&
&  0.17645E+01_JPRB, 0.16754E+01_JPRB, 0.16373E+01_JPRB, 0.15959E+01_JPRB, 0.15495E+01_JPRB, 0.15384E+01_JPRB, 0.15784E+01_JPRB ,&
&  0.16319E+01_JPRB, 0.17051E+01_JPRB, 0.18519E+01_JPRB, 0.18169E+01_JPRB, 0.17441E+01_JPRB, 0.17502E+01_JPRB, 0.18047E+01_JPRB ,&
&  0.17950E+01_JPRB, 0.18229E+01_JPRB, 0.18380E+01_JPRB, 0.18436E+01_JPRB, 0.19691E+01_JPRB, 0.21573E+01_JPRB, 0.23229E+01_JPRB ,&
&  0.25605E+01_JPRB, 0.24420E+01_JPRB, 0.22208E+01_JPRB, 0.23351E+01_JPRB, 0.27144E+01_JPRB, 0.33753E+01_JPRB, 0.33244E+01_JPRB ,&
&  0.30181E+01_JPRB, 0.25121E+01_JPRB, 0.27268E+01_JPRB, 0.36487E+01_JPRB, 0.43359E+01_JPRB, 0.49658E+01_JPRB, 0.69112E+01_JPRB ,&
&  0.68826E+01_JPRB, 0.37391E+01_JPRB, 0.29628E+01_JPRB, 0.38155E+01_JPRB, 0.26526E+01_JPRB, 0.21676E+01_JPRB, 0.39625E+01_JPRB ,&
&  0.51459E+01_JPRB, 0.61176E+01_JPRB, 0.62913E+01_JPRB, 0.57126E+01_JPRB, 0.54829E+01_JPRB, 0.48664E+01_JPRB, 0.39203E+01_JPRB ,&
&  0.31599E+01_JPRB, 0.25063E+01_JPRB, 0.20621E+01_JPRB, 0.18297E+01_JPRB, 0.17363E+01_JPRB, 0.16891E+01_JPRB, 0.16722E+01_JPRB ,&
&  0.16896E+01_JPRB, 0.17442E+01_JPRB, 0.18397E+01_JPRB, 0.19976E+01_JPRB, 0.21521E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.21430E+01_JPRB, 0.20898E+01_JPRB, 0.19277E+01_JPRB, 0.17763E+01_JPRB, 0.16733E+01_JPRB, 0.15887E+01_JPRB, 0.14822E+01_JPRB ,&
&  0.15663E+01_JPRB, 0.13883E+01_JPRB, 0.12649E+01_JPRB, 0.12143E+01_JPRB, 0.11931E+01_JPRB, 0.11958E+01_JPRB, 0.12040E+01_JPRB ,&
&  0.12054E+01_JPRB, 0.11988E+01_JPRB, 0.12079E+01_JPRB, 0.12478E+01_JPRB, 0.13791E+01_JPRB, 0.17866E+01_JPRB, 0.17105E+01_JPRB ,&
&  0.28686E+01_JPRB, 0.48118E+01_JPRB, 0.38336E+01_JPRB, 0.33919E+01_JPRB, 0.30296E+01_JPRB, 0.28715E+01_JPRB, 0.21117E+01_JPRB ,&
&  0.19028E+01_JPRB, 0.18970E+01_JPRB, 0.19088E+01_JPRB, 0.19303E+01_JPRB, 0.19366E+01_JPRB, 0.18937E+01_JPRB, 0.17707E+01_JPRB ,&
&  0.16772E+01_JPRB, 0.16226E+01_JPRB, 0.16110E+01_JPRB, 0.15901E+01_JPRB, 0.15738E+01_JPRB, 0.15911E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.16632E+01_JPRB, 0.18086E+01_JPRB, 0.19240E+01_JPRB, 0.17859E+01_JPRB, 0.17744E+01_JPRB, 0.18053E+01_JPRB, 0.18380E+01_JPRB ,&
&  0.17471E+01_JPRB, 0.16486E+01_JPRB, 0.17589E+01_JPRB, 0.18729E+01_JPRB, 0.19709E+01_JPRB, 0.20518E+01_JPRB, 0.21073E+01_JPRB ,&
&  0.23231E+01_JPRB, 0.23908E+01_JPRB, 0.24799E+01_JPRB, 0.23816E+01_JPRB, 0.26283E+01_JPRB, 0.31312E+01_JPRB, 0.33628E+01_JPRB ,&
&  0.35826E+01_JPRB, 0.33945E+01_JPRB, 0.38432E+01_JPRB, 0.44374E+01_JPRB, 0.46665E+01_JPRB, 0.47647E+01_JPRB, 0.54842E+01_JPRB ,&
&  0.63403E+01_JPRB, 0.76038E+01_JPRB, 0.67930E+01_JPRB, 0.53000E+01_JPRB, 0.41319E+01_JPRB, 0.32024E+01_JPRB, 0.40142E+01_JPRB ,&
&  0.53541E+01_JPRB, 0.55300E+01_JPRB, 0.54879E+01_JPRB, 0.65227E+01_JPRB, 0.48253E+01_JPRB, 0.35573E+01_JPRB, 0.25817E+01_JPRB ,&
&  0.20133E+01_JPRB, 0.18103E+01_JPRB, 0.17681E+01_JPRB, 0.17135E+01_JPRB, 0.16873E+01_JPRB, 0.16648E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.17361E+01_JPRB, 0.18239E+01_JPRB, 0.19261E+01_JPRB, 0.20444E+01_JPRB, 0.21357E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.20184E+01_JPRB, 0.19329E+01_JPRB, 0.18179E+01_JPRB, 0.17055E+01_JPRB, 0.16030E+01_JPRB, 0.15096E+01_JPRB, 0.14493E+01_JPRB ,&
&  0.16319E+01_JPRB, 0.13243E+01_JPRB, 0.12022E+01_JPRB, 0.11652E+01_JPRB, 0.11556E+01_JPRB, 0.11535E+01_JPRB, 0.11704E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.12145E+01_JPRB, 0.12677E+01_JPRB, 0.13666E+01_JPRB, 0.16111E+01_JPRB, 0.21991E+01_JPRB, 0.27141E+01_JPRB ,&
&  0.31915E+01_JPRB, 0.56353E+01_JPRB, 0.38783E+01_JPRB, 0.33110E+01_JPRB, 0.30193E+01_JPRB, 0.29181E+01_JPRB, 0.23141E+01_JPRB ,&
&  0.19386E+01_JPRB, 0.18533E+01_JPRB, 0.18372E+01_JPRB, 0.18109E+01_JPRB, 0.17891E+01_JPRB, 0.17264E+01_JPRB, 0.16702E+01_JPRB ,&
&  0.16248E+01_JPRB, 0.15995E+01_JPRB, 0.15870E+01_JPRB, 0.15769E+01_JPRB, 0.15695E+01_JPRB, 0.16071E+01_JPRB, 0.16645E+01_JPRB ,&
&  0.17861E+01_JPRB, 0.19413E+01_JPRB, 0.18788E+01_JPRB, 0.17974E+01_JPRB, 0.17693E+01_JPRB, 0.18142E+01_JPRB, 0.18434E+01_JPRB ,&
&  0.17724E+01_JPRB, 0.16873E+01_JPRB, 0.17742E+01_JPRB, 0.18088E+01_JPRB, 0.18246E+01_JPRB, 0.19674E+01_JPRB, 0.19807E+01_JPRB ,&
&  0.22523E+01_JPRB, 0.23151E+01_JPRB, 0.28446E+01_JPRB, 0.26493E+01_JPRB, 0.25647E+01_JPRB, 0.29964E+01_JPRB, 0.30495E+01_JPRB ,&
&  0.28853E+01_JPRB, 0.33928E+01_JPRB, 0.40697E+01_JPRB, 0.47827E+01_JPRB, 0.50278E+01_JPRB, 0.51538E+01_JPRB, 0.50780E+01_JPRB ,&
&  0.46627E+01_JPRB, 0.49031E+01_JPRB, 0.41520E+01_JPRB, 0.34842E+01_JPRB, 0.37087E+01_JPRB, 0.34583E+01_JPRB, 0.44717E+01_JPRB ,&
&  0.56366E+01_JPRB, 0.56595E+01_JPRB, 0.57640E+01_JPRB, 0.51434E+01_JPRB, 0.32616E+01_JPRB, 0.22710E+01_JPRB, 0.17831E+01_JPRB ,&
&  0.16787E+01_JPRB, 0.17078E+01_JPRB, 0.17310E+01_JPRB, 0.17072E+01_JPRB, 0.16998E+01_JPRB, 0.17103E+01_JPRB, 0.17595E+01_JPRB ,&
&  0.18497E+01_JPRB, 0.19513E+01_JPRB, 0.20098E+01_JPRB, 0.20405E+01_JPRB, 0.20644E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.18842E+01_JPRB, 0.18077E+01_JPRB, 0.17224E+01_JPRB, 0.16488E+01_JPRB, 0.16046E+01_JPRB, 0.15170E+01_JPRB, 0.15281E+01_JPRB ,&
&  0.16832E+01_JPRB, 0.12444E+01_JPRB, 0.11396E+01_JPRB, 0.11198E+01_JPRB, 0.11265E+01_JPRB, 0.11500E+01_JPRB, 0.12032E+01_JPRB ,&
&  0.12580E+01_JPRB, 0.13213E+01_JPRB, 0.14368E+01_JPRB, 0.16774E+01_JPRB, 0.21622E+01_JPRB, 0.30393E+01_JPRB, 0.38466E+01_JPRB ,&
&  0.47647E+01_JPRB, 0.46890E+01_JPRB, 0.38800E+01_JPRB, 0.29884E+01_JPRB, 0.27912E+01_JPRB, 0.27626E+01_JPRB, 0.26743E+01_JPRB ,&
&  0.21443E+01_JPRB, 0.19310E+01_JPRB, 0.18548E+01_JPRB, 0.17671E+01_JPRB, 0.16540E+01_JPRB, 0.16338E+01_JPRB, 0.16266E+01_JPRB ,&
&  0.16153E+01_JPRB, 0.15965E+01_JPRB, 0.15849E+01_JPRB, 0.15877E+01_JPRB, 0.16252E+01_JPRB, 0.17209E+01_JPRB, 0.18276E+01_JPRB ,&
&  0.19443E+01_JPRB, 0.20000E+01_JPRB, 0.19086E+01_JPRB, 0.18076E+01_JPRB, 0.17806E+01_JPRB, 0.18226E+01_JPRB, 0.18348E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.18085E+01_JPRB, 0.18353E+01_JPRB, 0.18312E+01_JPRB, 0.17635E+01_JPRB, 0.19450E+01_JPRB, 0.20102E+01_JPRB ,&
&  0.22429E+01_JPRB, 0.24418E+01_JPRB, 0.30475E+01_JPRB, 0.25462E+01_JPRB, 0.27286E+01_JPRB, 0.28687E+01_JPRB, 0.29597E+01_JPRB ,&
&  0.30377E+01_JPRB, 0.37286E+01_JPRB, 0.44614E+01_JPRB, 0.48493E+01_JPRB, 0.53624E+01_JPRB, 0.51263E+01_JPRB, 0.53830E+01_JPRB ,&
&  0.40488E+01_JPRB, 0.38810E+01_JPRB, 0.29780E+01_JPRB, 0.32439E+01_JPRB, 0.34275E+01_JPRB, 0.34548E+01_JPRB, 0.39919E+01_JPRB ,&
&  0.48907E+01_JPRB, 0.46682E+01_JPRB, 0.40320E+01_JPRB, 0.30417E+01_JPRB, 0.19513E+01_JPRB, 0.16192E+01_JPRB, 0.15335E+01_JPRB ,&
&  0.15661E+01_JPRB, 0.16903E+01_JPRB, 0.17273E+01_JPRB, 0.17497E+01_JPRB, 0.17551E+01_JPRB, 0.17793E+01_JPRB, 0.18290E+01_JPRB ,&
&  0.19056E+01_JPRB, 0.19643E+01_JPRB, 0.19879E+01_JPRB, 0.19710E+01_JPRB, 0.19432E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.17331E+01_JPRB, 0.16663E+01_JPRB, 0.16097E+01_JPRB, 0.15323E+01_JPRB, 0.13946E+01_JPRB, 0.12917E+01_JPRB, 0.12316E+01_JPRB ,&
&  0.11840E+01_JPRB, 0.11430E+01_JPRB, 0.11525E+01_JPRB, 0.11670E+01_JPRB, 0.11969E+01_JPRB, 0.12496E+01_JPRB, 0.13262E+01_JPRB ,&
&  0.14154E+01_JPRB, 0.15593E+01_JPRB, 0.18264E+01_JPRB, 0.22270E+01_JPRB, 0.27282E+01_JPRB, 0.34696E+01_JPRB, 0.45038E+01_JPRB ,&
&  0.47160E+01_JPRB, 0.30278E+01_JPRB, 0.31668E+01_JPRB, 0.28202E+01_JPRB, 0.26721E+01_JPRB, 0.24324E+01_JPRB, 0.22568E+01_JPRB ,&
&  0.21039E+01_JPRB, 0.20140E+01_JPRB, 0.19742E+01_JPRB, 0.18606E+01_JPRB, 0.16758E+01_JPRB, 0.16792E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.16753E+01_JPRB, 0.16824E+01_JPRB, 0.17047E+01_JPRB, 0.17515E+01_JPRB, 0.18412E+01_JPRB, 0.19470E+01_JPRB, 0.20297E+01_JPRB ,&
&  0.20711E+01_JPRB, 0.20995E+01_JPRB, 0.20446E+01_JPRB, 0.19722E+01_JPRB, 0.19115E+01_JPRB, 0.19136E+01_JPRB, 0.18814E+01_JPRB ,&
&  0.18558E+01_JPRB, 0.18218E+01_JPRB, 0.18718E+01_JPRB, 0.19741E+01_JPRB, 0.20421E+01_JPRB, 0.19512E+01_JPRB, 0.21417E+01_JPRB ,&
&  0.23295E+01_JPRB, 0.25893E+01_JPRB, 0.27465E+01_JPRB, 0.33391E+01_JPRB, 0.24181E+01_JPRB, 0.26396E+01_JPRB, 0.26794E+01_JPRB ,&
&  0.35791E+01_JPRB, 0.40160E+01_JPRB, 0.44263E+01_JPRB, 0.47902E+01_JPRB, 0.55102E+01_JPRB, 0.50592E+01_JPRB, 0.47038E+01_JPRB ,&
&  0.37373E+01_JPRB, 0.29768E+01_JPRB, 0.25679E+01_JPRB, 0.28987E+01_JPRB, 0.32492E+01_JPRB, 0.37418E+01_JPRB, 0.37294E+01_JPRB ,&
&  0.38794E+01_JPRB, 0.32135E+01_JPRB, 0.24935E+01_JPRB, 0.21817E+01_JPRB, 0.16496E+01_JPRB, 0.15141E+01_JPRB, 0.15419E+01_JPRB ,&
&  0.16057E+01_JPRB, 0.17087E+01_JPRB, 0.17387E+01_JPRB, 0.17434E+01_JPRB, 0.17507E+01_JPRB, 0.17656E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.18242E+01_JPRB, 0.18562E+01_JPRB, 0.18631E+01_JPRB, 0.18333E+01_JPRB, 0.17860E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.15968E+01_JPRB, 0.15299E+01_JPRB, 0.14220E+01_JPRB, 0.12881E+01_JPRB, 0.11851E+01_JPRB, 0.11282E+01_JPRB, 0.11053E+01_JPRB ,&
&  0.11250E+01_JPRB, 0.11585E+01_JPRB, 0.11982E+01_JPRB, 0.12515E+01_JPRB, 0.13062E+01_JPRB, 0.13647E+01_JPRB, 0.14787E+01_JPRB ,&
&  0.16094E+01_JPRB, 0.18227E+01_JPRB, 0.21415E+01_JPRB, 0.25014E+01_JPRB, 0.28300E+01_JPRB, 0.31699E+01_JPRB, 0.35054E+01_JPRB ,&
&  0.38743E+01_JPRB, 0.38624E+01_JPRB, 0.32964E+01_JPRB, 0.22439E+01_JPRB, 0.19768E+01_JPRB, 0.20696E+01_JPRB, 0.20005E+01_JPRB ,&
&  0.19634E+01_JPRB, 0.19057E+01_JPRB, 0.18596E+01_JPRB, 0.18253E+01_JPRB, 0.18058E+01_JPRB, 0.18170E+01_JPRB, 0.18227E+01_JPRB ,&
&  0.18454E+01_JPRB, 0.18806E+01_JPRB, 0.19258E+01_JPRB, 0.19820E+01_JPRB, 0.20335E+01_JPRB, 0.20960E+01_JPRB, 0.21680E+01_JPRB ,&
&  0.22170E+01_JPRB, 0.22892E+01_JPRB, 0.23193E+01_JPRB, 0.22920E+01_JPRB, 0.21929E+01_JPRB, 0.21510E+01_JPRB, 0.20883E+01_JPRB ,&
&  0.20596E+01_JPRB, 0.20266E+01_JPRB, 0.20413E+01_JPRB, 0.21482E+01_JPRB, 0.22167E+01_JPRB, 0.20473E+01_JPRB, 0.21217E+01_JPRB ,&
&  0.24741E+01_JPRB, 0.28430E+01_JPRB, 0.26448E+01_JPRB, 0.32743E+01_JPRB, 0.20929E+01_JPRB, 0.26022E+01_JPRB, 0.32376E+01_JPRB ,&
&  0.34520E+01_JPRB, 0.37908E+01_JPRB, 0.42181E+01_JPRB, 0.45738E+01_JPRB, 0.46989E+01_JPRB, 0.43541E+01_JPRB, 0.40392E+01_JPRB ,&
&  0.33455E+01_JPRB, 0.26110E+01_JPRB, 0.23082E+01_JPRB, 0.24310E+01_JPRB, 0.28324E+01_JPRB, 0.32797E+01_JPRB, 0.27011E+01_JPRB ,&
&  0.21128E+01_JPRB, 0.22144E+01_JPRB, 0.20418E+01_JPRB, 0.21370E+01_JPRB, 0.17393E+01_JPRB, 0.14832E+01_JPRB, 0.15661E+01_JPRB ,&
&  0.16638E+01_JPRB, 0.16850E+01_JPRB, 0.17108E+01_JPRB, 0.16990E+01_JPRB, 0.17050E+01_JPRB, 0.17073E+01_JPRB, 0.17094E+01_JPRB ,&
&  0.17225E+01_JPRB, 0.17494E+01_JPRB, 0.17526E+01_JPRB, 0.17059E+01_JPRB, 0.16464E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.14212E+01_JPRB, 0.12988E+01_JPRB, 0.11652E+01_JPRB, 0.10692E+01_JPRB, 0.10260E+01_JPRB, 0.10209E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.10824E+01_JPRB, 0.11320E+01_JPRB, 0.11937E+01_JPRB, 0.12718E+01_JPRB, 0.13465E+01_JPRB, 0.14292E+01_JPRB, 0.15444E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.18666E+01_JPRB, 0.21182E+01_JPRB, 0.23963E+01_JPRB, 0.26212E+01_JPRB, 0.28326E+01_JPRB, 0.30280E+01_JPRB ,&
&  0.32162E+01_JPRB, 0.32690E+01_JPRB, 0.31323E+01_JPRB, 0.29186E+01_JPRB, 0.23272E+01_JPRB, 0.24020E+01_JPRB, 0.25399E+01_JPRB ,&
&  0.25933E+01_JPRB, 0.21341E+01_JPRB, 0.19686E+01_JPRB, 0.19326E+01_JPRB, 0.19380E+01_JPRB, 0.19608E+01_JPRB, 0.19780E+01_JPRB ,&
&  0.20074E+01_JPRB, 0.20390E+01_JPRB, 0.20677E+01_JPRB, 0.20929E+01_JPRB, 0.21235E+01_JPRB, 0.21945E+01_JPRB, 0.23307E+01_JPRB ,&
&  0.23822E+01_JPRB, 0.24308E+01_JPRB, 0.24624E+01_JPRB, 0.24377E+01_JPRB, 0.25281E+01_JPRB, 0.26051E+01_JPRB, 0.25593E+01_JPRB ,&
&  0.25070E+01_JPRB, 0.23602E+01_JPRB, 0.24194E+01_JPRB, 0.25667E+01_JPRB, 0.25651E+01_JPRB, 0.24353E+01_JPRB, 0.24806E+01_JPRB ,&
&  0.28119E+01_JPRB, 0.31983E+01_JPRB, 0.23948E+01_JPRB, 0.27822E+01_JPRB, 0.33166E+01_JPRB, 0.29439E+01_JPRB, 0.27991E+01_JPRB ,&
&  0.26788E+01_JPRB, 0.28949E+01_JPRB, 0.35105E+01_JPRB, 0.37925E+01_JPRB, 0.37649E+01_JPRB, 0.43649E+01_JPRB, 0.35683E+01_JPRB ,&
&  0.29182E+01_JPRB, 0.22434E+01_JPRB, 0.20006E+01_JPRB, 0.19904E+01_JPRB, 0.21904E+01_JPRB, 0.22642E+01_JPRB, 0.16497E+01_JPRB ,&
&  0.14429E+01_JPRB, 0.18129E+01_JPRB, 0.18535E+01_JPRB, 0.20802E+01_JPRB, 0.20101E+01_JPRB, 0.14187E+01_JPRB, 0.14975E+01_JPRB ,&
&  0.15668E+01_JPRB, 0.16153E+01_JPRB, 0.16274E+01_JPRB, 0.16262E+01_JPRB, 0.16231E+01_JPRB, 0.16206E+01_JPRB, 0.16253E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.16397E+01_JPRB, 0.16306E+01_JPRB, 0.15808E+01_JPRB, 0.15082E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.11398E+01_JPRB, 0.10294E+01_JPRB, 0.95224E+00_JPRB, 0.92811E+00_JPRB, 0.93919E+00_JPRB, 0.95826E+00_JPRB, 0.97121E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.10451E+01_JPRB, 0.11073E+01_JPRB, 0.11638E+01_JPRB, 0.12316E+01_JPRB, 0.12990E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.16302E+01_JPRB, 0.18127E+01_JPRB, 0.20727E+01_JPRB, 0.22714E+01_JPRB, 0.24451E+01_JPRB, 0.26920E+01_JPRB ,&
&  0.29492E+01_JPRB, 0.31052E+01_JPRB, 0.32751E+01_JPRB, 0.33695E+01_JPRB, 0.32701E+01_JPRB, 0.31393E+01_JPRB, 0.32546E+01_JPRB ,&
&  0.26795E+01_JPRB, 0.20883E+01_JPRB, 0.20178E+01_JPRB, 0.19940E+01_JPRB, 0.20296E+01_JPRB, 0.20385E+01_JPRB, 0.20484E+01_JPRB ,&
&  0.20720E+01_JPRB, 0.20861E+01_JPRB, 0.20981E+01_JPRB, 0.21203E+01_JPRB, 0.21722E+01_JPRB, 0.22751E+01_JPRB, 0.24191E+01_JPRB ,&
&  0.24661E+01_JPRB, 0.25296E+01_JPRB, 0.26333E+01_JPRB, 0.24353E+01_JPRB, 0.25990E+01_JPRB, 0.28455E+01_JPRB, 0.29899E+01_JPRB ,&
&  0.29705E+01_JPRB, 0.27975E+01_JPRB, 0.28632E+01_JPRB, 0.29350E+01_JPRB, 0.29728E+01_JPRB, 0.29461E+01_JPRB, 0.29696E+01_JPRB ,&
&  0.31538E+01_JPRB, 0.30427E+01_JPRB, 0.23167E+01_JPRB, 0.22796E+01_JPRB, 0.24340E+01_JPRB, 0.21021E+01_JPRB, 0.22283E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.21461E+01_JPRB, 0.26867E+01_JPRB, 0.31207E+01_JPRB, 0.33527E+01_JPRB, 0.38648E+01_JPRB, 0.32820E+01_JPRB ,&
&  0.22987E+01_JPRB, 0.17550E+01_JPRB, 0.14909E+01_JPRB, 0.13864E+01_JPRB, 0.14865E+01_JPRB, 0.15368E+01_JPRB, 0.14010E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.14316E+01_JPRB, 0.14802E+01_JPRB, 0.14376E+01_JPRB, 0.13056E+01_JPRB, 0.12861E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.14258E+01_JPRB, 0.14473E+01_JPRB, 0.14500E+01_JPRB, 0.14569E+01_JPRB, 0.14583E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.14711E+01_JPRB, 0.14628E+01_JPRB, 0.14274E+01_JPRB, 0.13696E+01_JPRB, 0.12670E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.90112E+00_JPRB, 0.83742E+00_JPRB, 0.81997E+00_JPRB, 0.83026E+00_JPRB, 0.84329E+00_JPRB, 0.85754E+00_JPRB, 0.87363E+00_JPRB ,&
&  0.88693E+00_JPRB, 0.89824E+00_JPRB, 0.91607E+00_JPRB, 0.93034E+00_JPRB, 0.98340E+00_JPRB, 0.10460E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.13451E+01_JPRB, 0.14357E+01_JPRB, 0.15800E+01_JPRB, 0.16810E+01_JPRB, 0.17858E+01_JPRB, 0.19523E+01_JPRB ,&
&  0.21696E+01_JPRB, 0.24269E+01_JPRB, 0.26797E+01_JPRB, 0.29619E+01_JPRB, 0.33396E+01_JPRB, 0.36929E+01_JPRB, 0.37745E+01_JPRB ,&
&  0.28174E+01_JPRB, 0.19285E+01_JPRB, 0.20109E+01_JPRB, 0.18834E+01_JPRB, 0.19391E+01_JPRB, 0.19837E+01_JPRB, 0.19911E+01_JPRB ,&
&  0.20166E+01_JPRB, 0.20311E+01_JPRB, 0.20437E+01_JPRB, 0.20602E+01_JPRB, 0.21238E+01_JPRB, 0.22141E+01_JPRB, 0.23345E+01_JPRB ,&
&  0.23997E+01_JPRB, 0.24536E+01_JPRB, 0.24680E+01_JPRB, 0.24531E+01_JPRB, 0.25925E+01_JPRB, 0.28544E+01_JPRB, 0.30455E+01_JPRB ,&
&  0.31470E+01_JPRB, 0.32349E+01_JPRB, 0.29704E+01_JPRB, 0.29978E+01_JPRB, 0.29990E+01_JPRB, 0.27782E+01_JPRB, 0.28684E+01_JPRB ,&
&  0.30352E+01_JPRB, 0.28024E+01_JPRB, 0.19267E+01_JPRB, 0.19433E+01_JPRB, 0.19988E+01_JPRB, 0.19289E+01_JPRB, 0.18954E+01_JPRB ,&
&  0.15741E+01_JPRB, 0.16255E+01_JPRB, 0.18807E+01_JPRB, 0.24522E+01_JPRB, 0.27602E+01_JPRB, 0.29245E+01_JPRB, 0.25963E+01_JPRB ,&
&  0.18025E+01_JPRB, 0.14182E+01_JPRB, 0.12165E+01_JPRB, 0.12247E+01_JPRB, 0.14336E+01_JPRB, 0.16883E+01_JPRB, 0.13910E+01_JPRB ,&
&  0.14293E+01_JPRB, 0.16767E+01_JPRB, 0.21019E+01_JPRB, 0.16533E+01_JPRB, 0.11663E+01_JPRB, 0.11246E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.11872E+01_JPRB, 0.12000E+01_JPRB, 0.12069E+01_JPRB, 0.12148E+01_JPRB, 0.12333E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.11950E+01_JPRB, 0.11622E+01_JPRB, 0.10747E+01_JPRB, 0.98017E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.74724E+00_JPRB, 0.73807E+00_JPRB, 0.74097E+00_JPRB, 0.75751E+00_JPRB, 0.75864E+00_JPRB, 0.76037E+00_JPRB, 0.79470E+00_JPRB ,&
&  0.78486E+00_JPRB, 0.77691E+00_JPRB, 0.78413E+00_JPRB, 0.76519E+00_JPRB, 0.79856E+00_JPRB, 0.83347E+00_JPRB, 0.89063E+00_JPRB ,&
&  0.94904E+00_JPRB, 0.98763E+00_JPRB, 0.10185E+01_JPRB, 0.10624E+01_JPRB, 0.11472E+01_JPRB, 0.12142E+01_JPRB, 0.12717E+01_JPRB ,&
&  0.13764E+01_JPRB, 0.15347E+01_JPRB, 0.17066E+01_JPRB, 0.20865E+01_JPRB, 0.29527E+01_JPRB, 0.36486E+01_JPRB, 0.36540E+01_JPRB ,&
&  0.27085E+01_JPRB, 0.18628E+01_JPRB, 0.17254E+01_JPRB, 0.15669E+01_JPRB, 0.16271E+01_JPRB, 0.17705E+01_JPRB, 0.17993E+01_JPRB ,&
&  0.17748E+01_JPRB, 0.17794E+01_JPRB, 0.17769E+01_JPRB, 0.17815E+01_JPRB, 0.17971E+01_JPRB, 0.18053E+01_JPRB, 0.17812E+01_JPRB ,&
&  0.17136E+01_JPRB, 0.16283E+01_JPRB, 0.16668E+01_JPRB, 0.19747E+01_JPRB, 0.22578E+01_JPRB, 0.26998E+01_JPRB, 0.26444E+01_JPRB ,&
&  0.25278E+01_JPRB, 0.24475E+01_JPRB, 0.24383E+01_JPRB, 0.24221E+01_JPRB, 0.23681E+01_JPRB, 0.22937E+01_JPRB, 0.23568E+01_JPRB ,&
&  0.24764E+01_JPRB, 0.23079E+01_JPRB, 0.16370E+01_JPRB, 0.17426E+01_JPRB, 0.17643E+01_JPRB, 0.18146E+01_JPRB, 0.16031E+01_JPRB ,&
&  0.13149E+01_JPRB, 0.12701E+01_JPRB, 0.13691E+01_JPRB, 0.16023E+01_JPRB, 0.19974E+01_JPRB, 0.21463E+01_JPRB, 0.19124E+01_JPRB ,&
&  0.14910E+01_JPRB, 0.12134E+01_JPRB, 0.10757E+01_JPRB, 0.11074E+01_JPRB, 0.14473E+01_JPRB, 0.17114E+01_JPRB, 0.18784E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.14860E+01_JPRB, 0.13287E+01_JPRB, 0.16512E+01_JPRB, 0.98507E+00_JPRB, 0.88422E+00_JPRB, 0.87693E+00_JPRB ,&
&  0.89021E+00_JPRB, 0.88311E+00_JPRB, 0.87460E+00_JPRB, 0.85530E+00_JPRB, 0.83660E+00_JPRB, 0.89927E+00_JPRB, 0.95909E+00_JPRB ,&
&  0.90714E+00_JPRB, 0.93798E+00_JPRB, 0.89523E+00_JPRB, 0.82575E+00_JPRB, 0.79091E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.70270E+00_JPRB, 0.71249E+00_JPRB, 0.71568E+00_JPRB, 0.72633E+00_JPRB, 0.74781E+00_JPRB, 0.76549E+00_JPRB, 0.78889E+00_JPRB ,&
&  0.81495E+00_JPRB, 0.83027E+00_JPRB, 0.84622E+00_JPRB, 0.86695E+00_JPRB, 0.88395E+00_JPRB, 0.91650E+00_JPRB, 0.94732E+00_JPRB ,&
&  0.98614E+00_JPRB, 0.10175E+01_JPRB, 0.10503E+01_JPRB, 0.11009E+01_JPRB, 0.11638E+01_JPRB, 0.12568E+01_JPRB, 0.13574E+01_JPRB ,&
&  0.14910E+01_JPRB, 0.16500E+01_JPRB, 0.18408E+01_JPRB, 0.20016E+01_JPRB, 0.20893E+01_JPRB, 0.21509E+01_JPRB, 0.24642E+01_JPRB ,&
&  0.20641E+01_JPRB, 0.19247E+01_JPRB, 0.17873E+01_JPRB, 0.15923E+01_JPRB, 0.15673E+01_JPRB, 0.14490E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.13795E+01_JPRB, 0.13208E+01_JPRB, 0.12864E+01_JPRB, 0.12569E+01_JPRB, 0.12207E+01_JPRB, 0.11537E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.11094E+01_JPRB, 0.12413E+01_JPRB, 0.15253E+01_JPRB, 0.18060E+01_JPRB, 0.19035E+01_JPRB, 0.17756E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.16867E+01_JPRB, 0.16177E+01_JPRB, 0.15616E+01_JPRB, 0.16360E+01_JPRB, 0.16951E+01_JPRB, 0.17431E+01_JPRB ,&
&  0.16620E+01_JPRB, 0.15135E+01_JPRB, 0.14660E+01_JPRB, 0.14977E+01_JPRB, 0.15320E+01_JPRB, 0.15331E+01_JPRB, 0.14169E+01_JPRB ,&
&  0.12165E+01_JPRB, 0.10910E+01_JPRB, 0.10539E+01_JPRB, 0.11250E+01_JPRB, 0.13555E+01_JPRB, 0.13963E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.98264E+00_JPRB, 0.96208E+00_JPRB, 0.10295E+01_JPRB, 0.12946E+01_JPRB, 0.28077E+01_JPRB, 0.37379E+01_JPRB ,&
&  0.13995E+01_JPRB, 0.11361E+01_JPRB, 0.73079E+00_JPRB, 0.10244E+01_JPRB, 0.75509E+00_JPRB, 0.71818E+00_JPRB, 0.71723E+00_JPRB ,&
&  0.71078E+00_JPRB, 0.70951E+00_JPRB, 0.70936E+00_JPRB, 0.69616E+00_JPRB, 0.68969E+00_JPRB, 0.70111E+00_JPRB, 0.74618E+00_JPRB ,&
&  0.74568E+00_JPRB, 0.75173E+00_JPRB, 0.73013E+00_JPRB, 0.72672E+00_JPRB, 0.71182E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.69935E+00_JPRB, 0.71648E+00_JPRB, 0.73960E+00_JPRB, 0.76865E+00_JPRB, 0.79131E+00_JPRB, 0.81946E+00_JPRB, 0.84416E+00_JPRB ,&
&  0.87495E+00_JPRB, 0.90125E+00_JPRB, 0.92515E+00_JPRB, 0.95130E+00_JPRB, 0.98217E+00_JPRB, 0.10170E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.11549E+01_JPRB, 0.12069E+01_JPRB, 0.12656E+01_JPRB, 0.13343E+01_JPRB, 0.14181E+01_JPRB, 0.14995E+01_JPRB ,&
&  0.15778E+01_JPRB, 0.16611E+01_JPRB, 0.17525E+01_JPRB, 0.18262E+01_JPRB, 0.18580E+01_JPRB, 0.19481E+01_JPRB, 0.17961E+01_JPRB ,&
&  0.20235E+01_JPRB, 0.18189E+01_JPRB, 0.16747E+01_JPRB, 0.15186E+01_JPRB, 0.13870E+01_JPRB, 0.12698E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.11731E+01_JPRB, 0.11082E+01_JPRB, 0.10804E+01_JPRB, 0.10671E+01_JPRB, 0.96823E+00_JPRB, 0.95364E+00_JPRB, 0.98616E+00_JPRB ,&
&  0.10546E+01_JPRB, 0.11728E+01_JPRB, 0.12438E+01_JPRB, 0.13294E+01_JPRB, 0.14443E+01_JPRB, 0.13876E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.12274E+01_JPRB, 0.11608E+01_JPRB, 0.95645E+00_JPRB, 0.10630E+01_JPRB, 0.11194E+01_JPRB, 0.11662E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.11436E+01_JPRB, 0.99734E+00_JPRB, 0.10694E+01_JPRB, 0.13413E+01_JPRB, 0.13382E+01_JPRB, 0.12862E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.10614E+01_JPRB, 0.98575E+00_JPRB, 0.95530E+00_JPRB, 0.10137E+01_JPRB, 0.10131E+01_JPRB, 0.95278E+00_JPRB ,&
&  0.87979E+00_JPRB, 0.87101E+00_JPRB, 0.90058E+00_JPRB, 0.97319E+00_JPRB, 0.12476E+01_JPRB, 0.15757E+01_JPRB, 0.25417E+01_JPRB ,&
&  0.14531E+01_JPRB, 0.78924E+00_JPRB, 0.83961E+00_JPRB, 0.82779E+00_JPRB, 0.72630E+00_JPRB, 0.67452E+00_JPRB, 0.69796E+00_JPRB ,&
&  0.72626E+00_JPRB, 0.73734E+00_JPRB, 0.69479E+00_JPRB, 0.69448E+00_JPRB, 0.69318E+00_JPRB, 0.69345E+00_JPRB, 0.69799E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.69517E+00_JPRB, 0.70206E+00_JPRB, 0.69588E+00_JPRB, 0.69330E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.61336E+00_JPRB, 0.62509E+00_JPRB, 0.64237E+00_JPRB, 0.65813E+00_JPRB, 0.66954E+00_JPRB, 0.68695E+00_JPRB, 0.70105E+00_JPRB ,&
&  0.71685E+00_JPRB, 0.72612E+00_JPRB, 0.73961E+00_JPRB, 0.75271E+00_JPRB, 0.76768E+00_JPRB, 0.78820E+00_JPRB, 0.81026E+00_JPRB ,&
&  0.83036E+00_JPRB, 0.86199E+00_JPRB, 0.89523E+00_JPRB, 0.92071E+00_JPRB, 0.97322E+00_JPRB, 0.10692E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12783E+01_JPRB, 0.14094E+01_JPRB, 0.15325E+01_JPRB, 0.16085E+01_JPRB, 0.16368E+01_JPRB, 0.17701E+01_JPRB, 0.15193E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.17094E+01_JPRB, 0.15493E+01_JPRB, 0.14565E+01_JPRB, 0.13719E+01_JPRB, 0.12943E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.15234E+01_JPRB, 0.14502E+01_JPRB, 0.14692E+01_JPRB, 0.12215E+01_JPRB, 0.11624E+01_JPRB, 0.12648E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.14291E+01_JPRB, 0.14159E+01_JPRB, 0.14733E+01_JPRB, 0.15401E+01_JPRB, 0.15654E+01_JPRB, 0.15681E+01_JPRB ,&
&  0.15082E+01_JPRB, 0.14513E+01_JPRB, 0.13533E+01_JPRB, 0.15533E+01_JPRB, 0.14264E+01_JPRB, 0.12922E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.97188E+00_JPRB, 0.93874E+00_JPRB, 0.10056E+01_JPRB, 0.11184E+01_JPRB, 0.11174E+01_JPRB, 0.11250E+01_JPRB, 0.11067E+01_JPRB ,&
&  0.10699E+01_JPRB, 0.10421E+01_JPRB, 0.98645E+00_JPRB, 0.96759E+00_JPRB, 0.97133E+00_JPRB, 0.93698E+00_JPRB, 0.86698E+00_JPRB ,&
&  0.83435E+00_JPRB, 0.86076E+00_JPRB, 0.87234E+00_JPRB, 0.91049E+00_JPRB, 0.10539E+01_JPRB, 0.11101E+01_JPRB, 0.16770E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.10329E+01_JPRB, 0.13569E+01_JPRB, 0.54624E+00_JPRB, 0.63499E+00_JPRB, 0.73474E+00_JPRB, 0.92764E+00_JPRB ,&
&  0.96780E+00_JPRB, 0.12347E+01_JPRB, 0.15665E+01_JPRB, 0.12475E+01_JPRB, 0.96686E+00_JPRB, 0.68048E+00_JPRB, 0.64922E+00_JPRB ,&
&  0.64036E+00_JPRB, 0.62941E+00_JPRB, 0.63027E+00_JPRB, 0.62065E+00_JPRB, 0.61587E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.52647E+00_JPRB, 0.53145E+00_JPRB, 0.54319E+00_JPRB, 0.55190E+00_JPRB, 0.56086E+00_JPRB, 0.56888E+00_JPRB, 0.57581E+00_JPRB ,&
&  0.59326E+00_JPRB, 0.60515E+00_JPRB, 0.63407E+00_JPRB, 0.66113E+00_JPRB, 0.68733E+00_JPRB, 0.73557E+00_JPRB, 0.78119E+00_JPRB ,&
&  0.84946E+00_JPRB, 0.93685E+00_JPRB, 0.98478E+00_JPRB, 0.10406E+01_JPRB, 0.10765E+01_JPRB, 0.11203E+01_JPRB, 0.11592E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.15216E+01_JPRB, 0.16895E+01_JPRB, 0.18679E+01_JPRB, 0.18880E+01_JPRB, 0.20604E+01_JPRB, 0.15306E+01_JPRB ,&
&  0.17953E+01_JPRB, 0.18097E+01_JPRB, 0.17021E+01_JPRB, 0.16443E+01_JPRB, 0.16118E+01_JPRB, 0.15914E+01_JPRB, 0.15817E+01_JPRB ,&
&  0.16422E+01_JPRB, 0.17147E+01_JPRB, 0.16312E+01_JPRB, 0.16284E+01_JPRB, 0.16278E+01_JPRB, 0.15500E+01_JPRB, 0.17044E+01_JPRB ,&
&  0.18264E+01_JPRB, 0.19234E+01_JPRB, 0.20463E+01_JPRB, 0.22020E+01_JPRB, 0.23111E+01_JPRB, 0.23764E+01_JPRB, 0.23944E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.24541E+01_JPRB, 0.26327E+01_JPRB, 0.26123E+01_JPRB, 0.21479E+01_JPRB, 0.18175E+01_JPRB, 0.14686E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.92416E+00_JPRB, 0.92208E+00_JPRB, 0.98156E+00_JPRB, 0.96369E+00_JPRB, 0.95626E+00_JPRB, 0.94940E+00_JPRB ,&
&  0.95021E+00_JPRB, 0.96965E+00_JPRB, 0.97222E+00_JPRB, 0.96946E+00_JPRB, 0.94114E+00_JPRB, 0.87306E+00_JPRB, 0.80678E+00_JPRB ,&
&  0.80727E+00_JPRB, 0.81536E+00_JPRB, 0.83806E+00_JPRB, 0.89576E+00_JPRB, 0.10120E+01_JPRB, 0.12353E+01_JPRB, 0.12384E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.86052E+00_JPRB, 0.80274E+00_JPRB, 0.82806E+00_JPRB, 0.68650E+00_JPRB, 0.93290E+00_JPRB, 0.13443E+01_JPRB ,&
&  0.19637E+01_JPRB, 0.21937E+01_JPRB, 0.28560E+01_JPRB, 0.41608E+01_JPRB, 0.26309E+01_JPRB, 0.17071E+01_JPRB, 0.60221E+00_JPRB ,&
&  0.59215E+00_JPRB, 0.56741E+00_JPRB, 0.55954E+00_JPRB, 0.54241E+00_JPRB, 0.53406E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.43120E+00_JPRB, 0.42500E+00_JPRB, 0.43721E+00_JPRB, 0.45634E+00_JPRB, 0.47168E+00_JPRB, 0.50558E+00_JPRB, 0.54698E+00_JPRB ,&
&  0.59301E+00_JPRB, 0.65265E+00_JPRB, 0.70922E+00_JPRB, 0.75737E+00_JPRB, 0.83389E+00_JPRB, 0.93585E+00_JPRB, 0.99944E+00_JPRB ,&
&  0.11027E+01_JPRB, 0.11839E+01_JPRB, 0.12433E+01_JPRB, 0.12893E+01_JPRB, 0.13516E+01_JPRB, 0.14354E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.17828E+01_JPRB, 0.19519E+01_JPRB, 0.20627E+01_JPRB, 0.21206E+01_JPRB, 0.21362E+01_JPRB, 0.16309E+01_JPRB ,&
&  0.19121E+01_JPRB, 0.20502E+01_JPRB, 0.20682E+01_JPRB, 0.19969E+01_JPRB, 0.19085E+01_JPRB, 0.18555E+01_JPRB, 0.18710E+01_JPRB ,&
&  0.18984E+01_JPRB, 0.18674E+01_JPRB, 0.17687E+01_JPRB, 0.17193E+01_JPRB, 0.20174E+01_JPRB, 0.20267E+01_JPRB, 0.22060E+01_JPRB ,&
&  0.23594E+01_JPRB, 0.24885E+01_JPRB, 0.26617E+01_JPRB, 0.28427E+01_JPRB, 0.30303E+01_JPRB, 0.31910E+01_JPRB, 0.33310E+01_JPRB ,&
&  0.34621E+01_JPRB, 0.36106E+01_JPRB, 0.37560E+01_JPRB, 0.32589E+01_JPRB, 0.26451E+01_JPRB, 0.22224E+01_JPRB, 0.15133E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.94342E+00_JPRB, 0.96287E+00_JPRB, 0.96002E+00_JPRB, 0.88928E+00_JPRB, 0.85054E+00_JPRB, 0.82552E+00_JPRB ,&
&  0.84128E+00_JPRB, 0.87850E+00_JPRB, 0.90287E+00_JPRB, 0.90350E+00_JPRB, 0.86379E+00_JPRB, 0.80706E+00_JPRB, 0.76303E+00_JPRB ,&
&  0.79278E+00_JPRB, 0.80665E+00_JPRB, 0.82377E+00_JPRB, 0.91723E+00_JPRB, 0.10605E+01_JPRB, 0.13707E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.15668E+01_JPRB, 0.86999E+00_JPRB, 0.76653E+00_JPRB, 0.73072E+00_JPRB, 0.71148E+00_JPRB, 0.77942E+00_JPRB, 0.93986E+00_JPRB ,&
&  0.13135E+01_JPRB, 0.21185E+01_JPRB, 0.23435E+01_JPRB, 0.15448E+01_JPRB, 0.20466E+01_JPRB, 0.13552E+01_JPRB, 0.58016E+00_JPRB ,&
&  0.55403E+00_JPRB, 0.53403E+00_JPRB, 0.50544E+00_JPRB, 0.47282E+00_JPRB, 0.44833E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.43208E+00_JPRB, 0.41333E+00_JPRB, 0.42530E+00_JPRB, 0.45120E+00_JPRB, 0.48174E+00_JPRB, 0.52955E+00_JPRB, 0.59038E+00_JPRB ,&
&  0.64836E+00_JPRB, 0.70998E+00_JPRB, 0.81274E+00_JPRB, 0.85081E+00_JPRB, 0.89374E+00_JPRB, 0.96783E+00_JPRB, 0.10355E+01_JPRB ,&
&  0.11182E+01_JPRB, 0.12016E+01_JPRB, 0.12864E+01_JPRB, 0.13696E+01_JPRB, 0.14708E+01_JPRB, 0.15780E+01_JPRB, 0.16709E+01_JPRB ,&
&  0.17957E+01_JPRB, 0.19230E+01_JPRB, 0.20925E+01_JPRB, 0.22702E+01_JPRB, 0.24337E+01_JPRB, 0.24607E+01_JPRB, 0.25453E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.22357E+01_JPRB, 0.22538E+01_JPRB, 0.21967E+01_JPRB, 0.20778E+01_JPRB, 0.19791E+01_JPRB, 0.19848E+01_JPRB ,&
&  0.19981E+01_JPRB, 0.18954E+01_JPRB, 0.19306E+01_JPRB, 0.19113E+01_JPRB, 0.22404E+01_JPRB, 0.23276E+01_JPRB, 0.24610E+01_JPRB ,&
&  0.26071E+01_JPRB, 0.27392E+01_JPRB, 0.28855E+01_JPRB, 0.30620E+01_JPRB, 0.32439E+01_JPRB, 0.34152E+01_JPRB, 0.35853E+01_JPRB ,&
&  0.37299E+01_JPRB, 0.38666E+01_JPRB, 0.40313E+01_JPRB, 0.33804E+01_JPRB, 0.28438E+01_JPRB, 0.23376E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.94530E+00_JPRB, 0.96831E+00_JPRB, 0.99654E+00_JPRB, 0.90190E+00_JPRB, 0.84174E+00_JPRB, 0.76687E+00_JPRB ,&
&  0.76428E+00_JPRB, 0.78148E+00_JPRB, 0.78968E+00_JPRB, 0.78524E+00_JPRB, 0.78876E+00_JPRB, 0.74275E+00_JPRB, 0.72591E+00_JPRB ,&
&  0.75538E+00_JPRB, 0.75980E+00_JPRB, 0.78207E+00_JPRB, 0.83243E+00_JPRB, 0.91393E+00_JPRB, 0.97477E+00_JPRB, 0.97154E+00_JPRB ,&
&  0.96237E+00_JPRB, 0.84699E+00_JPRB, 0.79410E+00_JPRB, 0.80018E+00_JPRB, 0.76634E+00_JPRB, 0.73496E+00_JPRB, 0.74998E+00_JPRB ,&
&  0.83360E+00_JPRB, 0.94480E+00_JPRB, 0.10349E+01_JPRB, 0.10108E+01_JPRB, 0.10332E+01_JPRB, 0.82656E+00_JPRB, 0.63208E+00_JPRB ,&
&  0.59298E+00_JPRB, 0.58244E+00_JPRB, 0.53579E+00_JPRB, 0.50214E+00_JPRB, 0.46465E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.53361E+00_JPRB, 0.47071E+00_JPRB, 0.46669E+00_JPRB, 0.51101E+00_JPRB, 0.55615E+00_JPRB, 0.60416E+00_JPRB, 0.65148E+00_JPRB ,&
&  0.70490E+00_JPRB, 0.73963E+00_JPRB, 0.80189E+00_JPRB, 0.84164E+00_JPRB, 0.87698E+00_JPRB, 0.90756E+00_JPRB, 0.94985E+00_JPRB ,&
&  0.10221E+01_JPRB, 0.11080E+01_JPRB, 0.11936E+01_JPRB, 0.13064E+01_JPRB, 0.14282E+01_JPRB, 0.15439E+01_JPRB, 0.16729E+01_JPRB ,&
&  0.18025E+01_JPRB, 0.19478E+01_JPRB, 0.20982E+01_JPRB, 0.22710E+01_JPRB, 0.24281E+01_JPRB, 0.24894E+01_JPRB, 0.24455E+01_JPRB ,&
&  0.12199E+01_JPRB, 0.19157E+01_JPRB, 0.23940E+01_JPRB, 0.22882E+01_JPRB, 0.21695E+01_JPRB, 0.20249E+01_JPRB, 0.19766E+01_JPRB ,&
&  0.18387E+01_JPRB, 0.16557E+01_JPRB, 0.17138E+01_JPRB, 0.21581E+01_JPRB, 0.23057E+01_JPRB, 0.24289E+01_JPRB, 0.25095E+01_JPRB ,&
&  0.26111E+01_JPRB, 0.26868E+01_JPRB, 0.27734E+01_JPRB, 0.28665E+01_JPRB, 0.29775E+01_JPRB, 0.30837E+01_JPRB, 0.31740E+01_JPRB ,&
&  0.32426E+01_JPRB, 0.32747E+01_JPRB, 0.32567E+01_JPRB, 0.25896E+01_JPRB, 0.25370E+01_JPRB, 0.21883E+01_JPRB, 0.18986E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.10019E+01_JPRB, 0.98807E+00_JPRB, 0.10169E+01_JPRB, 0.91722E+00_JPRB, 0.84607E+00_JPRB, 0.75167E+00_JPRB ,&
&  0.74658E+00_JPRB, 0.74397E+00_JPRB, 0.74865E+00_JPRB, 0.73946E+00_JPRB, 0.74018E+00_JPRB, 0.69465E+00_JPRB, 0.72000E+00_JPRB ,&
&  0.74715E+00_JPRB, 0.75412E+00_JPRB, 0.77774E+00_JPRB, 0.82056E+00_JPRB, 0.87503E+00_JPRB, 0.92269E+00_JPRB, 0.93466E+00_JPRB ,&
&  0.92877E+00_JPRB, 0.90203E+00_JPRB, 0.89064E+00_JPRB, 0.88050E+00_JPRB, 0.86509E+00_JPRB, 0.84868E+00_JPRB, 0.82520E+00_JPRB ,&
&  0.79813E+00_JPRB, 0.80337E+00_JPRB, 0.83232E+00_JPRB, 0.76662E+00_JPRB, 0.80559E+00_JPRB, 0.73391E+00_JPRB, 0.71910E+00_JPRB ,&
&  0.70080E+00_JPRB, 0.68369E+00_JPRB, 0.63719E+00_JPRB, 0.61307E+00_JPRB, 0.57796E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.64952E+00_JPRB, 0.57130E+00_JPRB, 0.51155E+00_JPRB, 0.52711E+00_JPRB, 0.59477E+00_JPRB, 0.65255E+00_JPRB, 0.68596E+00_JPRB ,&
&  0.72643E+00_JPRB, 0.74253E+00_JPRB, 0.75509E+00_JPRB, 0.77125E+00_JPRB, 0.80580E+00_JPRB, 0.83133E+00_JPRB, 0.88419E+00_JPRB ,&
&  0.93152E+00_JPRB, 0.97721E+00_JPRB, 0.10284E+01_JPRB, 0.11173E+01_JPRB, 0.12325E+01_JPRB, 0.13488E+01_JPRB, 0.14620E+01_JPRB ,&
&  0.15970E+01_JPRB, 0.17643E+01_JPRB, 0.19906E+01_JPRB, 0.22589E+01_JPRB, 0.26306E+01_JPRB, 0.30407E+01_JPRB, 0.33516E+01_JPRB ,&
&  0.23326E+01_JPRB, 0.10300E+01_JPRB, 0.20961E+01_JPRB, 0.24346E+01_JPRB, 0.22174E+01_JPRB, 0.20871E+01_JPRB, 0.19183E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.16068E+01_JPRB, 0.15546E+01_JPRB, 0.18625E+01_JPRB, 0.22258E+01_JPRB, 0.23623E+01_JPRB, 0.24154E+01_JPRB ,&
&  0.24628E+01_JPRB, 0.24834E+01_JPRB, 0.25182E+01_JPRB, 0.25713E+01_JPRB, 0.26365E+01_JPRB, 0.26817E+01_JPRB, 0.27069E+01_JPRB ,&
&  0.27205E+01_JPRB, 0.26953E+01_JPRB, 0.25954E+01_JPRB, 0.19248E+01_JPRB, 0.17704E+01_JPRB, 0.15905E+01_JPRB, 0.14672E+01_JPRB ,&
&  0.13280E+01_JPRB, 0.11015E+01_JPRB, 0.10318E+01_JPRB, 0.10280E+01_JPRB, 0.94462E+00_JPRB, 0.82748E+00_JPRB, 0.77874E+00_JPRB ,&
&  0.77078E+00_JPRB, 0.75037E+00_JPRB, 0.72449E+00_JPRB, 0.70462E+00_JPRB, 0.73721E+00_JPRB, 0.73968E+00_JPRB, 0.74139E+00_JPRB ,&
&  0.74289E+00_JPRB, 0.78746E+00_JPRB, 0.83273E+00_JPRB, 0.87250E+00_JPRB, 0.91389E+00_JPRB, 0.96135E+00_JPRB, 0.97058E+00_JPRB ,&
&  0.98856E+00_JPRB, 0.98075E+00_JPRB, 0.97506E+00_JPRB, 0.95112E+00_JPRB, 0.92882E+00_JPRB, 0.89040E+00_JPRB, 0.88016E+00_JPRB ,&
&  0.87527E+00_JPRB, 0.87523E+00_JPRB, 0.85993E+00_JPRB, 0.84090E+00_JPRB, 0.85736E+00_JPRB, 0.84493E+00_JPRB, 0.84870E+00_JPRB ,&
&  0.84201E+00_JPRB, 0.81994E+00_JPRB, 0.76375E+00_JPRB, 0.71706E+00_JPRB, 0.69503E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.72042E+00_JPRB, 0.66658E+00_JPRB, 0.59774E+00_JPRB, 0.55076E+00_JPRB, 0.57070E+00_JPRB, 0.64057E+00_JPRB, 0.69799E+00_JPRB ,&
&  0.72105E+00_JPRB, 0.72348E+00_JPRB, 0.71585E+00_JPRB, 0.71697E+00_JPRB, 0.73468E+00_JPRB, 0.73818E+00_JPRB, 0.76670E+00_JPRB ,&
&  0.81266E+00_JPRB, 0.84994E+00_JPRB, 0.88115E+00_JPRB, 0.93231E+00_JPRB, 0.99391E+00_JPRB, 0.10970E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.12778E+01_JPRB, 0.13683E+01_JPRB, 0.15139E+01_JPRB, 0.17222E+01_JPRB, 0.20165E+01_JPRB, 0.25297E+01_JPRB, 0.34207E+01_JPRB ,&
&  0.50327E+01_JPRB, 0.33537E+01_JPRB, 0.13978E+01_JPRB, 0.23818E+01_JPRB, 0.24644E+01_JPRB, 0.22991E+01_JPRB, 0.19547E+01_JPRB ,&
&  0.19129E+01_JPRB, 0.16545E+01_JPRB, 0.14551E+01_JPRB, 0.15517E+01_JPRB, 0.17439E+01_JPRB, 0.20126E+01_JPRB, 0.21772E+01_JPRB ,&
&  0.22161E+01_JPRB, 0.22348E+01_JPRB, 0.22522E+01_JPRB, 0.22915E+01_JPRB, 0.23525E+01_JPRB, 0.24137E+01_JPRB, 0.24280E+01_JPRB ,&
&  0.24403E+01_JPRB, 0.24654E+01_JPRB, 0.23960E+01_JPRB, 0.18357E+01_JPRB, 0.17305E+01_JPRB, 0.16771E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.13337E+01_JPRB, 0.12441E+01_JPRB, 0.10988E+01_JPRB, 0.97746E+00_JPRB, 0.90544E+00_JPRB, 0.89549E+00_JPRB ,&
&  0.86641E+00_JPRB, 0.81169E+00_JPRB, 0.78097E+00_JPRB, 0.75683E+00_JPRB, 0.77679E+00_JPRB, 0.80906E+00_JPRB, 0.81881E+00_JPRB ,&
&  0.84677E+00_JPRB, 0.88473E+00_JPRB, 0.91108E+00_JPRB, 0.93508E+00_JPRB, 0.97049E+00_JPRB, 0.10049E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.10508E+01_JPRB, 0.10539E+01_JPRB, 0.10489E+01_JPRB, 0.10329E+01_JPRB, 0.98773E+00_JPRB, 0.94461E+00_JPRB, 0.95182E+00_JPRB ,&
&  0.93919E+00_JPRB, 0.93435E+00_JPRB, 0.90864E+00_JPRB, 0.91485E+00_JPRB, 0.90733E+00_JPRB, 0.92236E+00_JPRB, 0.94797E+00_JPRB ,&
&  0.97108E+00_JPRB, 0.94785E+00_JPRB, 0.86136E+00_JPRB, 0.76835E+00_JPRB, 0.74304E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.75759E+00_JPRB, 0.73868E+00_JPRB, 0.69667E+00_JPRB, 0.61560E+00_JPRB, 0.58327E+00_JPRB, 0.59751E+00_JPRB, 0.66397E+00_JPRB ,&
&  0.69441E+00_JPRB, 0.70470E+00_JPRB, 0.69043E+00_JPRB, 0.67584E+00_JPRB, 0.67010E+00_JPRB, 0.67691E+00_JPRB, 0.70363E+00_JPRB ,&
&  0.73836E+00_JPRB, 0.74144E+00_JPRB, 0.73630E+00_JPRB, 0.74509E+00_JPRB, 0.78578E+00_JPRB, 0.84634E+00_JPRB, 0.94590E+00_JPRB ,&
&  0.10287E+01_JPRB, 0.10794E+01_JPRB, 0.11357E+01_JPRB, 0.12690E+01_JPRB, 0.14313E+01_JPRB, 0.15203E+01_JPRB, 0.17742E+01_JPRB ,&
&  0.28475E+01_JPRB, 0.44104E+01_JPRB, 0.16722E+01_JPRB, 0.25124E+01_JPRB, 0.28712E+01_JPRB, 0.25540E+01_JPRB, 0.23076E+01_JPRB ,&
&  0.22198E+01_JPRB, 0.18232E+01_JPRB, 0.14223E+01_JPRB, 0.13513E+01_JPRB, 0.14224E+01_JPRB, 0.15188E+01_JPRB, 0.16696E+01_JPRB ,&
&  0.18132E+01_JPRB, 0.19067E+01_JPRB, 0.19851E+01_JPRB, 0.20483E+01_JPRB, 0.21263E+01_JPRB, 0.22342E+01_JPRB, 0.23285E+01_JPRB ,&
&  0.23941E+01_JPRB, 0.24908E+01_JPRB, 0.25336E+01_JPRB, 0.20523E+01_JPRB, 0.19703E+01_JPRB, 0.20894E+01_JPRB, 0.19894E+01_JPRB ,&
&  0.17623E+01_JPRB, 0.16291E+01_JPRB, 0.14494E+01_JPRB, 0.12308E+01_JPRB, 0.10676E+01_JPRB, 0.10023E+01_JPRB, 0.98404E+00_JPRB ,&
&  0.92973E+00_JPRB, 0.88479E+00_JPRB, 0.87257E+00_JPRB, 0.87065E+00_JPRB, 0.87222E+00_JPRB, 0.88153E+00_JPRB, 0.90651E+00_JPRB ,&
&  0.94936E+00_JPRB, 0.98692E+00_JPRB, 0.10109E+01_JPRB, 0.10232E+01_JPRB, 0.10372E+01_JPRB, 0.10561E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.10780E+01_JPRB, 0.10819E+01_JPRB, 0.10706E+01_JPRB, 0.10452E+01_JPRB, 0.10285E+01_JPRB, 0.10239E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10010E+01_JPRB, 0.92975E+00_JPRB, 0.90965E+00_JPRB, 0.93625E+00_JPRB, 0.90780E+00_JPRB, 0.94850E+00_JPRB ,&
&  0.10138E+01_JPRB, 0.10514E+01_JPRB, 0.99244E+00_JPRB, 0.79386E+00_JPRB, 0.76736E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.79066E+00_JPRB, 0.77969E+00_JPRB, 0.76392E+00_JPRB, 0.70679E+00_JPRB, 0.65052E+00_JPRB, 0.62287E+00_JPRB, 0.63761E+00_JPRB ,&
&  0.67589E+00_JPRB, 0.70264E+00_JPRB, 0.70313E+00_JPRB, 0.69244E+00_JPRB, 0.69060E+00_JPRB, 0.70138E+00_JPRB, 0.71862E+00_JPRB ,&
&  0.73028E+00_JPRB, 0.72579E+00_JPRB, 0.71077E+00_JPRB, 0.71331E+00_JPRB, 0.71829E+00_JPRB, 0.73479E+00_JPRB, 0.78451E+00_JPRB ,&
&  0.85459E+00_JPRB, 0.89616E+00_JPRB, 0.91038E+00_JPRB, 0.95570E+00_JPRB, 0.10790E+01_JPRB, 0.11274E+01_JPRB, 0.11972E+01_JPRB ,&
&  0.16089E+01_JPRB, 0.33756E+01_JPRB, 0.36117E+01_JPRB, 0.41163E+01_JPRB, 0.36190E+01_JPRB, 0.29822E+01_JPRB, 0.28321E+01_JPRB ,&
&  0.28445E+01_JPRB, 0.23219E+01_JPRB, 0.18328E+01_JPRB, 0.14270E+01_JPRB, 0.13539E+01_JPRB, 0.13704E+01_JPRB, 0.13737E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.15823E+01_JPRB, 0.16764E+01_JPRB, 0.17634E+01_JPRB, 0.18473E+01_JPRB, 0.19412E+01_JPRB, 0.20798E+01_JPRB ,&
&  0.22399E+01_JPRB, 0.24426E+01_JPRB, 0.26807E+01_JPRB, 0.24882E+01_JPRB, 0.22074E+01_JPRB, 0.24961E+01_JPRB, 0.25807E+01_JPRB ,&
&  0.24873E+01_JPRB, 0.20499E+01_JPRB, 0.16919E+01_JPRB, 0.14124E+01_JPRB, 0.11996E+01_JPRB, 0.12169E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.97525E+00_JPRB, 0.95873E+00_JPRB, 0.96479E+00_JPRB, 0.96951E+00_JPRB, 0.96061E+00_JPRB, 0.97283E+00_JPRB ,&
&  0.10042E+01_JPRB, 0.10333E+01_JPRB, 0.10517E+01_JPRB, 0.10648E+01_JPRB, 0.10818E+01_JPRB, 0.10936E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10690E+01_JPRB, 0.10310E+01_JPRB, 0.10394E+01_JPRB, 0.10422E+01_JPRB, 0.10404E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.10164E+01_JPRB, 0.97682E+00_JPRB, 0.95203E+00_JPRB, 0.95734E+00_JPRB, 0.93809E+00_JPRB, 0.89183E+00_JPRB ,&
&  0.89548E+00_JPRB, 0.87847E+00_JPRB, 0.85135E+00_JPRB, 0.82413E+00_JPRB, 0.80454E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.83913E+00_JPRB, 0.82753E+00_JPRB, 0.82079E+00_JPRB, 0.79147E+00_JPRB, 0.75430E+00_JPRB, 0.72139E+00_JPRB, 0.70388E+00_JPRB ,&
&  0.70521E+00_JPRB, 0.71909E+00_JPRB, 0.73064E+00_JPRB, 0.71987E+00_JPRB, 0.71568E+00_JPRB, 0.73015E+00_JPRB, 0.74007E+00_JPRB ,&
&  0.74810E+00_JPRB, 0.74778E+00_JPRB, 0.74240E+00_JPRB, 0.74128E+00_JPRB, 0.73718E+00_JPRB, 0.72449E+00_JPRB, 0.72595E+00_JPRB ,&
&  0.75503E+00_JPRB, 0.78422E+00_JPRB, 0.79850E+00_JPRB, 0.80869E+00_JPRB, 0.87653E+00_JPRB, 0.93070E+00_JPRB, 0.10030E+01_JPRB ,&
&  0.12073E+01_JPRB, 0.28871E+01_JPRB, 0.25043E+01_JPRB, 0.47688E+01_JPRB, 0.40112E+01_JPRB, 0.33304E+01_JPRB, 0.28234E+01_JPRB ,&
&  0.27131E+01_JPRB, 0.27955E+01_JPRB, 0.21857E+01_JPRB, 0.17544E+01_JPRB, 0.14983E+01_JPRB, 0.14067E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.12798E+01_JPRB, 0.13017E+01_JPRB, 0.13521E+01_JPRB, 0.14291E+01_JPRB, 0.15274E+01_JPRB, 0.16136E+01_JPRB, 0.17234E+01_JPRB ,&
&  0.18499E+01_JPRB, 0.20559E+01_JPRB, 0.23751E+01_JPRB, 0.23961E+01_JPRB, 0.24045E+01_JPRB, 0.27474E+01_JPRB, 0.31009E+01_JPRB ,&
&  0.30801E+01_JPRB, 0.26207E+01_JPRB, 0.20029E+01_JPRB, 0.15977E+01_JPRB, 0.13910E+01_JPRB, 0.13335E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.10632E+01_JPRB, 0.10453E+01_JPRB, 0.10333E+01_JPRB, 0.10324E+01_JPRB, 0.10295E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10681E+01_JPRB, 0.10751E+01_JPRB, 0.10752E+01_JPRB, 0.10793E+01_JPRB, 0.10813E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10709E+01_JPRB, 0.10236E+01_JPRB, 0.10072E+01_JPRB, 0.10172E+01_JPRB, 0.10176E+01_JPRB, 0.10171E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.10609E+01_JPRB, 0.10252E+01_JPRB, 0.97379E+00_JPRB, 0.98112E+00_JPRB, 0.99230E+00_JPRB, 0.90124E+00_JPRB ,&
&  0.88530E+00_JPRB, 0.87425E+00_JPRB, 0.87043E+00_JPRB, 0.86428E+00_JPRB, 0.85184E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.85436E+00_JPRB, 0.84088E+00_JPRB, 0.83646E+00_JPRB, 0.82163E+00_JPRB, 0.79498E+00_JPRB, 0.76787E+00_JPRB, 0.75284E+00_JPRB ,&
&  0.74180E+00_JPRB, 0.74326E+00_JPRB, 0.75266E+00_JPRB, 0.74421E+00_JPRB, 0.74642E+00_JPRB, 0.75718E+00_JPRB, 0.76336E+00_JPRB ,&
&  0.76014E+00_JPRB, 0.75797E+00_JPRB, 0.75040E+00_JPRB, 0.74588E+00_JPRB, 0.74330E+00_JPRB, 0.73111E+00_JPRB, 0.72373E+00_JPRB ,&
&  0.72648E+00_JPRB, 0.73997E+00_JPRB, 0.75237E+00_JPRB, 0.76328E+00_JPRB, 0.79872E+00_JPRB, 0.83640E+00_JPRB, 0.89134E+00_JPRB ,&
&  0.10992E+01_JPRB, 0.32124E+01_JPRB, 0.23381E+01_JPRB, 0.44397E+01_JPRB, 0.38607E+01_JPRB, 0.31411E+01_JPRB, 0.25871E+01_JPRB ,&
&  0.25229E+01_JPRB, 0.24968E+01_JPRB, 0.22101E+01_JPRB, 0.19843E+01_JPRB, 0.17739E+01_JPRB, 0.15644E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12223E+01_JPRB, 0.12155E+01_JPRB, 0.12355E+01_JPRB, 0.12898E+01_JPRB, 0.13558E+01_JPRB, 0.14203E+01_JPRB ,&
&  0.14963E+01_JPRB, 0.16143E+01_JPRB, 0.18373E+01_JPRB, 0.22019E+01_JPRB, 0.22506E+01_JPRB, 0.25271E+01_JPRB, 0.35383E+01_JPRB ,&
&  0.36639E+01_JPRB, 0.32977E+01_JPRB, 0.24133E+01_JPRB, 0.17872E+01_JPRB, 0.15358E+01_JPRB, 0.14296E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.12273E+01_JPRB, 0.11529E+01_JPRB, 0.11357E+01_JPRB, 0.10950E+01_JPRB, 0.10643E+01_JPRB, 0.10718E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.10886E+01_JPRB, 0.10949E+01_JPRB, 0.10815E+01_JPRB, 0.10645E+01_JPRB, 0.10434E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.10571E+01_JPRB, 0.10377E+01_JPRB, 0.10102E+01_JPRB, 0.10113E+01_JPRB, 0.10210E+01_JPRB, 0.10236E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.10935E+01_JPRB, 0.10626E+01_JPRB, 0.10408E+01_JPRB, 0.10167E+01_JPRB, 0.10327E+01_JPRB, 0.90498E+00_JPRB ,&
&  0.87738E+00_JPRB, 0.86795E+00_JPRB, 0.87092E+00_JPRB, 0.87855E+00_JPRB, 0.87453E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.87563E+00_JPRB, 0.85524E+00_JPRB, 0.84875E+00_JPRB, 0.84123E+00_JPRB, 0.82601E+00_JPRB, 0.80836E+00_JPRB, 0.80333E+00_JPRB ,&
&  0.79923E+00_JPRB, 0.79604E+00_JPRB, 0.78586E+00_JPRB, 0.76890E+00_JPRB, 0.76732E+00_JPRB, 0.76701E+00_JPRB, 0.76526E+00_JPRB ,&
&  0.76199E+00_JPRB, 0.76069E+00_JPRB, 0.74920E+00_JPRB, 0.74320E+00_JPRB, 0.74462E+00_JPRB, 0.74013E+00_JPRB, 0.72897E+00_JPRB ,&
&  0.71828E+00_JPRB, 0.71697E+00_JPRB, 0.72866E+00_JPRB, 0.74194E+00_JPRB, 0.77011E+00_JPRB, 0.78764E+00_JPRB, 0.83280E+00_JPRB ,&
&  0.10913E+01_JPRB, 0.11081E+01_JPRB, 0.28339E+01_JPRB, 0.38204E+01_JPRB, 0.36909E+01_JPRB, 0.31189E+01_JPRB, 0.27270E+01_JPRB ,&
&  0.25395E+01_JPRB, 0.23367E+01_JPRB, 0.21807E+01_JPRB, 0.19780E+01_JPRB, 0.17590E+01_JPRB, 0.15205E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.12261E+01_JPRB, 0.11991E+01_JPRB, 0.11991E+01_JPRB, 0.12125E+01_JPRB, 0.12238E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.13265E+01_JPRB, 0.14164E+01_JPRB, 0.16885E+01_JPRB, 0.20829E+01_JPRB, 0.24234E+01_JPRB, 0.31731E+01_JPRB ,&
&  0.39238E+01_JPRB, 0.37439E+01_JPRB, 0.27811E+01_JPRB, 0.20150E+01_JPRB, 0.16634E+01_JPRB, 0.15196E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.12324E+01_JPRB, 0.11790E+01_JPRB, 0.11348E+01_JPRB, 0.10962E+01_JPRB, 0.10748E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.11291E+01_JPRB, 0.11012E+01_JPRB, 0.10768E+01_JPRB, 0.10557E+01_JPRB, 0.10316E+01_JPRB, 0.10217E+01_JPRB ,&
&  0.10242E+01_JPRB, 0.10439E+01_JPRB, 0.10443E+01_JPRB, 0.10246E+01_JPRB, 0.10253E+01_JPRB, 0.10475E+01_JPRB, 0.10717E+01_JPRB ,&
&  0.10988E+01_JPRB, 0.11023E+01_JPRB, 0.10685E+01_JPRB, 0.10656E+01_JPRB, 0.10165E+01_JPRB, 0.10265E+01_JPRB, 0.92049E+00_JPRB ,&
&  0.87986E+00_JPRB, 0.87147E+00_JPRB, 0.88205E+00_JPRB, 0.89075E+00_JPRB, 0.90324E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.89521E+00_JPRB, 0.86401E+00_JPRB, 0.85239E+00_JPRB, 0.85296E+00_JPRB, 0.84712E+00_JPRB, 0.84908E+00_JPRB, 0.85027E+00_JPRB ,&
&  0.84212E+00_JPRB, 0.82104E+00_JPRB, 0.80361E+00_JPRB, 0.78533E+00_JPRB, 0.77895E+00_JPRB, 0.77178E+00_JPRB, 0.77069E+00_JPRB ,&
&  0.76294E+00_JPRB, 0.75768E+00_JPRB, 0.74816E+00_JPRB, 0.74516E+00_JPRB, 0.74456E+00_JPRB, 0.73802E+00_JPRB, 0.71526E+00_JPRB ,&
&  0.70744E+00_JPRB, 0.70838E+00_JPRB, 0.72217E+00_JPRB, 0.73930E+00_JPRB, 0.76060E+00_JPRB, 0.77079E+00_JPRB, 0.79727E+00_JPRB ,&
&  0.10397E+01_JPRB, 0.18365E+01_JPRB, 0.29329E+01_JPRB, 0.30946E+01_JPRB, 0.30692E+01_JPRB, 0.27676E+01_JPRB, 0.25071E+01_JPRB ,&
&  0.22626E+01_JPRB, 0.21560E+01_JPRB, 0.21381E+01_JPRB, 0.19289E+01_JPRB, 0.16148E+01_JPRB, 0.13666E+01_JPRB, 0.12667E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.11631E+01_JPRB, 0.11401E+01_JPRB, 0.11381E+01_JPRB, 0.11386E+01_JPRB, 0.11410E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.11786E+01_JPRB, 0.12066E+01_JPRB, 0.13383E+01_JPRB, 0.18315E+01_JPRB, 0.19817E+01_JPRB, 0.28338E+01_JPRB ,&
&  0.40224E+01_JPRB, 0.36453E+01_JPRB, 0.29047E+01_JPRB, 0.22375E+01_JPRB, 0.18509E+01_JPRB, 0.16555E+01_JPRB, 0.15067E+01_JPRB ,&
&  0.13924E+01_JPRB, 0.12965E+01_JPRB, 0.12164E+01_JPRB, 0.11710E+01_JPRB, 0.11277E+01_JPRB, 0.10923E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.11164E+01_JPRB, 0.10995E+01_JPRB, 0.10687E+01_JPRB, 0.10440E+01_JPRB, 0.10376E+01_JPRB, 0.10315E+01_JPRB ,&
&  0.10330E+01_JPRB, 0.10400E+01_JPRB, 0.10560E+01_JPRB, 0.10141E+01_JPRB, 0.10171E+01_JPRB, 0.10490E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10545E+01_JPRB, 0.10742E+01_JPRB, 0.10610E+01_JPRB, 0.11017E+01_JPRB, 0.10849E+01_JPRB, 0.95873E+00_JPRB ,&
&  0.88894E+00_JPRB, 0.86954E+00_JPRB, 0.88855E+00_JPRB, 0.91392E+00_JPRB, 0.94583E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.92777E+00_JPRB, 0.87805E+00_JPRB, 0.85095E+00_JPRB, 0.84848E+00_JPRB, 0.85433E+00_JPRB, 0.85112E+00_JPRB, 0.85373E+00_JPRB ,&
&  0.84424E+00_JPRB, 0.82511E+00_JPRB, 0.80412E+00_JPRB, 0.78634E+00_JPRB, 0.77734E+00_JPRB, 0.77105E+00_JPRB, 0.76620E+00_JPRB ,&
&  0.76182E+00_JPRB, 0.75684E+00_JPRB, 0.74869E+00_JPRB, 0.75087E+00_JPRB, 0.75244E+00_JPRB, 0.74947E+00_JPRB, 0.72468E+00_JPRB ,&
&  0.71076E+00_JPRB, 0.71279E+00_JPRB, 0.72316E+00_JPRB, 0.73235E+00_JPRB, 0.74607E+00_JPRB, 0.75994E+00_JPRB, 0.78360E+00_JPRB ,&
&  0.91725E+00_JPRB, 0.16268E+01_JPRB, 0.22458E+01_JPRB, 0.26415E+01_JPRB, 0.27589E+01_JPRB, 0.26233E+01_JPRB, 0.23603E+01_JPRB ,&
&  0.21587E+01_JPRB, 0.20446E+01_JPRB, 0.19392E+01_JPRB, 0.16815E+01_JPRB, 0.14087E+01_JPRB, 0.12577E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.11412E+01_JPRB, 0.11260E+01_JPRB, 0.11094E+01_JPRB, 0.10963E+01_JPRB, 0.10843E+01_JPRB, 0.10843E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.11025E+01_JPRB, 0.11139E+01_JPRB, 0.11263E+01_JPRB, 0.13897E+01_JPRB, 0.17416E+01_JPRB, 0.23065E+01_JPRB ,&
&  0.25677E+01_JPRB, 0.26212E+01_JPRB, 0.24280E+01_JPRB, 0.21685E+01_JPRB, 0.19326E+01_JPRB, 0.17369E+01_JPRB, 0.15696E+01_JPRB ,&
&  0.14281E+01_JPRB, 0.13035E+01_JPRB, 0.12004E+01_JPRB, 0.11360E+01_JPRB, 0.10986E+01_JPRB, 0.10625E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10567E+01_JPRB, 0.10557E+01_JPRB, 0.10459E+01_JPRB, 0.10261E+01_JPRB, 0.10096E+01_JPRB, 0.99924E+00_JPRB ,&
&  0.10027E+01_JPRB, 0.10036E+01_JPRB, 0.10240E+01_JPRB, 0.10139E+01_JPRB, 0.10017E+01_JPRB, 0.99664E+00_JPRB, 0.10042E+01_JPRB ,&
&  0.10239E+01_JPRB, 0.10451E+01_JPRB, 0.10347E+01_JPRB, 0.10561E+01_JPRB, 0.10815E+01_JPRB, 0.10802E+01_JPRB, 0.97801E+00_JPRB ,&
&  0.90907E+00_JPRB, 0.88025E+00_JPRB, 0.87760E+00_JPRB, 0.89647E+00_JPRB, 0.93907E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.95036E+00_JPRB, 0.91262E+00_JPRB, 0.87614E+00_JPRB, 0.85484E+00_JPRB, 0.84637E+00_JPRB, 0.84276E+00_JPRB, 0.84022E+00_JPRB ,&
&  0.83504E+00_JPRB, 0.82610E+00_JPRB, 0.80431E+00_JPRB, 0.78738E+00_JPRB, 0.77537E+00_JPRB, 0.77076E+00_JPRB, 0.76680E+00_JPRB ,&
&  0.76476E+00_JPRB, 0.76152E+00_JPRB, 0.75451E+00_JPRB, 0.75229E+00_JPRB, 0.75326E+00_JPRB, 0.74705E+00_JPRB, 0.72752E+00_JPRB ,&
&  0.70686E+00_JPRB, 0.70716E+00_JPRB, 0.71606E+00_JPRB, 0.72465E+00_JPRB, 0.73776E+00_JPRB, 0.74986E+00_JPRB, 0.77439E+00_JPRB ,&
&  0.89209E+00_JPRB, 0.12795E+01_JPRB, 0.18801E+01_JPRB, 0.22101E+01_JPRB, 0.23517E+01_JPRB, 0.23657E+01_JPRB, 0.22045E+01_JPRB ,&
&  0.19993E+01_JPRB, 0.17874E+01_JPRB, 0.16158E+01_JPRB, 0.14343E+01_JPRB, 0.13043E+01_JPRB, 0.12055E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.10886E+01_JPRB, 0.10747E+01_JPRB, 0.10572E+01_JPRB, 0.10438E+01_JPRB, 0.10355E+01_JPRB, 0.10330E+01_JPRB ,&
&  0.10367E+01_JPRB, 0.10478E+01_JPRB, 0.10608E+01_JPRB, 0.10709E+01_JPRB, 0.11011E+01_JPRB, 0.12375E+01_JPRB, 0.15397E+01_JPRB ,&
&  0.18147E+01_JPRB, 0.19717E+01_JPRB, 0.19407E+01_JPRB, 0.18786E+01_JPRB, 0.18173E+01_JPRB, 0.16924E+01_JPRB, 0.15632E+01_JPRB ,&
&  0.14356E+01_JPRB, 0.13176E+01_JPRB, 0.11963E+01_JPRB, 0.10984E+01_JPRB, 0.10352E+01_JPRB, 0.99852E+00_JPRB, 0.97909E+00_JPRB ,&
&  0.97769E+00_JPRB, 0.98833E+00_JPRB, 0.99006E+00_JPRB, 0.99342E+00_JPRB, 0.98535E+00_JPRB, 0.97028E+00_JPRB, 0.95502E+00_JPRB ,&
&  0.94046E+00_JPRB, 0.92818E+00_JPRB, 0.93361E+00_JPRB, 0.93302E+00_JPRB, 0.92852E+00_JPRB, 0.93132E+00_JPRB, 0.93771E+00_JPRB ,&
&  0.94760E+00_JPRB, 0.97714E+00_JPRB, 0.95961E+00_JPRB, 0.10257E+01_JPRB, 0.11035E+01_JPRB, 0.10458E+01_JPRB, 0.98382E+00_JPRB ,&
&  0.93134E+00_JPRB, 0.89743E+00_JPRB, 0.88381E+00_JPRB, 0.89999E+00_JPRB, 0.94247E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.85146E+00_JPRB, 0.84563E+00_JPRB, 0.83931E+00_JPRB, 0.83832E+00_JPRB, 0.83648E+00_JPRB, 0.82603E+00_JPRB, 0.81692E+00_JPRB ,&
&  0.81346E+00_JPRB, 0.80747E+00_JPRB, 0.79988E+00_JPRB, 0.78889E+00_JPRB, 0.77481E+00_JPRB, 0.76405E+00_JPRB, 0.76071E+00_JPRB ,&
&  0.75514E+00_JPRB, 0.75221E+00_JPRB, 0.75049E+00_JPRB, 0.74489E+00_JPRB, 0.73827E+00_JPRB, 0.72845E+00_JPRB, 0.71173E+00_JPRB ,&
&  0.68997E+00_JPRB, 0.68216E+00_JPRB, 0.69200E+00_JPRB, 0.70242E+00_JPRB, 0.71120E+00_JPRB, 0.71616E+00_JPRB, 0.73670E+00_JPRB ,&
&  0.85826E+00_JPRB, 0.10986E+01_JPRB, 0.13615E+01_JPRB, 0.16749E+01_JPRB, 0.18611E+01_JPRB, 0.18132E+01_JPRB, 0.17652E+01_JPRB ,&
&  0.16674E+01_JPRB, 0.15627E+01_JPRB, 0.14591E+01_JPRB, 0.13449E+01_JPRB, 0.12454E+01_JPRB, 0.11669E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.10846E+01_JPRB, 0.10556E+01_JPRB, 0.10329E+01_JPRB, 0.10074E+01_JPRB, 0.98757E+00_JPRB, 0.97346E+00_JPRB, 0.97094E+00_JPRB ,&
&  0.97393E+00_JPRB, 0.98153E+00_JPRB, 0.99262E+00_JPRB, 0.10043E+01_JPRB, 0.10212E+01_JPRB, 0.10643E+01_JPRB, 0.11975E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.15380E+01_JPRB, 0.16100E+01_JPRB, 0.15817E+01_JPRB, 0.15443E+01_JPRB, 0.14973E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.12764E+01_JPRB, 0.11926E+01_JPRB, 0.11261E+01_JPRB, 0.10683E+01_JPRB, 0.10256E+01_JPRB, 0.99278E+00_JPRB, 0.96805E+00_JPRB ,&
&  0.95294E+00_JPRB, 0.94680E+00_JPRB, 0.94195E+00_JPRB, 0.93741E+00_JPRB, 0.92902E+00_JPRB, 0.90935E+00_JPRB, 0.88186E+00_JPRB ,&
&  0.85209E+00_JPRB, 0.84154E+00_JPRB, 0.84775E+00_JPRB, 0.84743E+00_JPRB, 0.84623E+00_JPRB, 0.85501E+00_JPRB, 0.87280E+00_JPRB ,&
&  0.88457E+00_JPRB, 0.90583E+00_JPRB, 0.91888E+00_JPRB, 0.99057E+00_JPRB, 0.10027E+01_JPRB, 0.98072E+00_JPRB, 0.96766E+00_JPRB ,&
&  0.93988E+00_JPRB, 0.90351E+00_JPRB, 0.87297E+00_JPRB, 0.86793E+00_JPRB, 0.81630E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.83173E+00_JPRB, 0.82540E+00_JPRB, 0.81646E+00_JPRB, 0.81418E+00_JPRB, 0.81490E+00_JPRB, 0.80145E+00_JPRB, 0.78885E+00_JPRB ,&
&  0.78082E+00_JPRB, 0.77962E+00_JPRB, 0.78107E+00_JPRB, 0.78008E+00_JPRB, 0.77205E+00_JPRB, 0.75838E+00_JPRB, 0.74975E+00_JPRB ,&
&  0.73850E+00_JPRB, 0.72903E+00_JPRB, 0.72606E+00_JPRB, 0.72450E+00_JPRB, 0.71787E+00_JPRB, 0.70971E+00_JPRB, 0.70246E+00_JPRB ,&
&  0.68663E+00_JPRB, 0.67513E+00_JPRB, 0.67259E+00_JPRB, 0.67683E+00_JPRB, 0.68259E+00_JPRB, 0.68375E+00_JPRB, 0.71349E+00_JPRB ,&
&  0.85082E+00_JPRB, 0.90607E+00_JPRB, 0.92828E+00_JPRB, 0.11572E+01_JPRB, 0.13515E+01_JPRB, 0.13818E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.14512E+01_JPRB, 0.14428E+01_JPRB, 0.13929E+01_JPRB, 0.12887E+01_JPRB, 0.11999E+01_JPRB, 0.11282E+01_JPRB, 0.10620E+01_JPRB ,&
&  0.10044E+01_JPRB, 0.95424E+00_JPRB, 0.92191E+00_JPRB, 0.89628E+00_JPRB, 0.88247E+00_JPRB, 0.86467E+00_JPRB, 0.85650E+00_JPRB ,&
&  0.85750E+00_JPRB, 0.86811E+00_JPRB, 0.88175E+00_JPRB, 0.89463E+00_JPRB, 0.91668E+00_JPRB, 0.95127E+00_JPRB, 0.10184E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.12561E+01_JPRB, 0.13452E+01_JPRB, 0.13649E+01_JPRB, 0.13379E+01_JPRB, 0.12866E+01_JPRB, 0.12126E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.10600E+01_JPRB, 0.10080E+01_JPRB, 0.97794E+00_JPRB, 0.95571E+00_JPRB, 0.93816E+00_JPRB, 0.92130E+00_JPRB ,&
&  0.90679E+00_JPRB, 0.89764E+00_JPRB, 0.89166E+00_JPRB, 0.88330E+00_JPRB, 0.87174E+00_JPRB, 0.84738E+00_JPRB, 0.81276E+00_JPRB ,&
&  0.78624E+00_JPRB, 0.77170E+00_JPRB, 0.77192E+00_JPRB, 0.77294E+00_JPRB, 0.77665E+00_JPRB, 0.79372E+00_JPRB, 0.81525E+00_JPRB ,&
&  0.83065E+00_JPRB, 0.85028E+00_JPRB, 0.87416E+00_JPRB, 0.90149E+00_JPRB, 0.94929E+00_JPRB, 0.94434E+00_JPRB, 0.94413E+00_JPRB ,&
&  0.93289E+00_JPRB, 0.90958E+00_JPRB, 0.88427E+00_JPRB, 0.75538E+00_JPRB, 0.81672E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.81706E+00_JPRB, 0.81054E+00_JPRB, 0.79458E+00_JPRB, 0.78393E+00_JPRB, 0.77653E+00_JPRB, 0.76764E+00_JPRB, 0.75426E+00_JPRB ,&
&  0.74523E+00_JPRB, 0.74187E+00_JPRB, 0.73956E+00_JPRB, 0.73910E+00_JPRB, 0.73819E+00_JPRB, 0.73103E+00_JPRB, 0.72048E+00_JPRB ,&
&  0.71016E+00_JPRB, 0.70129E+00_JPRB, 0.70229E+00_JPRB, 0.70097E+00_JPRB, 0.69493E+00_JPRB, 0.68990E+00_JPRB, 0.68357E+00_JPRB ,&
&  0.66827E+00_JPRB, 0.65191E+00_JPRB, 0.63930E+00_JPRB, 0.64235E+00_JPRB, 0.64949E+00_JPRB, 0.64581E+00_JPRB, 0.67214E+00_JPRB ,&
&  0.80149E+00_JPRB, 0.77534E+00_JPRB, 0.81453E+00_JPRB, 0.94451E+00_JPRB, 0.10274E+01_JPRB, 0.11040E+01_JPRB, 0.11605E+01_JPRB ,&
&  0.12074E+01_JPRB, 0.12403E+01_JPRB, 0.11687E+01_JPRB, 0.10785E+01_JPRB, 0.98946E+00_JPRB, 0.91300E+00_JPRB, 0.85724E+00_JPRB ,&
&  0.82003E+00_JPRB, 0.80002E+00_JPRB, 0.78630E+00_JPRB, 0.77032E+00_JPRB, 0.75768E+00_JPRB, 0.74615E+00_JPRB, 0.73979E+00_JPRB ,&
&  0.73735E+00_JPRB, 0.74111E+00_JPRB, 0.75068E+00_JPRB, 0.77077E+00_JPRB, 0.79320E+00_JPRB, 0.81876E+00_JPRB, 0.85373E+00_JPRB ,&
&  0.90617E+00_JPRB, 0.98374E+00_JPRB, 0.10623E+01_JPRB, 0.10993E+01_JPRB, 0.10929E+01_JPRB, 0.10596E+01_JPRB, 0.10096E+01_JPRB ,&
&  0.95696E+00_JPRB, 0.91279E+00_JPRB, 0.88280E+00_JPRB, 0.86203E+00_JPRB, 0.84252E+00_JPRB, 0.82595E+00_JPRB, 0.81267E+00_JPRB ,&
&  0.80210E+00_JPRB, 0.79053E+00_JPRB, 0.78530E+00_JPRB, 0.77828E+00_JPRB, 0.76688E+00_JPRB, 0.74887E+00_JPRB, 0.73105E+00_JPRB ,&
&  0.71498E+00_JPRB, 0.70281E+00_JPRB, 0.69792E+00_JPRB, 0.70171E+00_JPRB, 0.70915E+00_JPRB, 0.72541E+00_JPRB, 0.74795E+00_JPRB ,&
&  0.76610E+00_JPRB, 0.78362E+00_JPRB, 0.79896E+00_JPRB, 0.84612E+00_JPRB, 0.87665E+00_JPRB, 0.89648E+00_JPRB, 0.90641E+00_JPRB ,&
&  0.90817E+00_JPRB, 0.90811E+00_JPRB, 0.72443E+00_JPRB, 0.79434E+00_JPRB, 0.81008E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.81580E+00_JPRB, 0.79437E+00_JPRB, 0.76846E+00_JPRB, 0.74864E+00_JPRB, 0.73627E+00_JPRB, 0.72954E+00_JPRB, 0.72276E+00_JPRB ,&
&  0.71485E+00_JPRB, 0.71337E+00_JPRB, 0.71353E+00_JPRB, 0.71363E+00_JPRB, 0.71311E+00_JPRB, 0.70841E+00_JPRB, 0.69888E+00_JPRB ,&
&  0.68822E+00_JPRB, 0.67557E+00_JPRB, 0.67320E+00_JPRB, 0.67203E+00_JPRB, 0.66525E+00_JPRB, 0.66035E+00_JPRB, 0.65935E+00_JPRB ,&
&  0.65243E+00_JPRB, 0.64148E+00_JPRB, 0.62208E+00_JPRB, 0.61606E+00_JPRB, 0.61909E+00_JPRB, 0.61968E+00_JPRB, 0.63148E+00_JPRB ,&
&  0.64955E+00_JPRB, 0.66302E+00_JPRB, 0.73686E+00_JPRB, 0.79058E+00_JPRB, 0.82582E+00_JPRB, 0.86158E+00_JPRB, 0.90003E+00_JPRB ,&
&  0.89528E+00_JPRB, 0.89652E+00_JPRB, 0.86097E+00_JPRB, 0.80531E+00_JPRB, 0.76492E+00_JPRB, 0.73648E+00_JPRB, 0.71725E+00_JPRB ,&
&  0.70991E+00_JPRB, 0.70432E+00_JPRB, 0.69757E+00_JPRB, 0.69105E+00_JPRB, 0.68761E+00_JPRB, 0.68109E+00_JPRB, 0.67746E+00_JPRB ,&
&  0.67885E+00_JPRB, 0.68355E+00_JPRB, 0.68688E+00_JPRB, 0.69353E+00_JPRB, 0.70668E+00_JPRB, 0.72677E+00_JPRB, 0.75000E+00_JPRB ,&
&  0.77430E+00_JPRB, 0.80396E+00_JPRB, 0.84667E+00_JPRB, 0.87908E+00_JPRB, 0.88656E+00_JPRB, 0.86815E+00_JPRB, 0.83849E+00_JPRB ,&
&  0.80996E+00_JPRB, 0.78839E+00_JPRB, 0.77176E+00_JPRB, 0.75767E+00_JPRB, 0.74117E+00_JPRB, 0.72608E+00_JPRB, 0.71172E+00_JPRB ,&
&  0.70264E+00_JPRB, 0.69203E+00_JPRB, 0.68348E+00_JPRB, 0.67531E+00_JPRB, 0.66971E+00_JPRB, 0.66748E+00_JPRB, 0.66741E+00_JPRB ,&
&  0.66648E+00_JPRB, 0.66142E+00_JPRB, 0.65891E+00_JPRB, 0.66278E+00_JPRB, 0.66767E+00_JPRB, 0.67722E+00_JPRB, 0.69169E+00_JPRB ,&
&  0.70975E+00_JPRB, 0.72765E+00_JPRB, 0.74529E+00_JPRB, 0.78353E+00_JPRB, 0.82258E+00_JPRB, 0.85360E+00_JPRB, 0.87012E+00_JPRB ,&
&  0.87955E+00_JPRB, 0.87182E+00_JPRB, 0.86748E+00_JPRB, 0.85238E+00_JPRB, 0.83275E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.75194E+00_JPRB, 0.72555E+00_JPRB, 0.70319E+00_JPRB, 0.68680E+00_JPRB, 0.67640E+00_JPRB, 0.67237E+00_JPRB, 0.66761E+00_JPRB ,&
&  0.66425E+00_JPRB, 0.65989E+00_JPRB, 0.65719E+00_JPRB, 0.65953E+00_JPRB, 0.66065E+00_JPRB, 0.66086E+00_JPRB, 0.66136E+00_JPRB ,&
&  0.65482E+00_JPRB, 0.64086E+00_JPRB, 0.62977E+00_JPRB, 0.62639E+00_JPRB, 0.62684E+00_JPRB, 0.62243E+00_JPRB, 0.62069E+00_JPRB ,&
&  0.62370E+00_JPRB, 0.61551E+00_JPRB, 0.60156E+00_JPRB, 0.59237E+00_JPRB, 0.59297E+00_JPRB, 0.59072E+00_JPRB, 0.59110E+00_JPRB ,&
&  0.63453E+00_JPRB, 0.59417E+00_JPRB, 0.65532E+00_JPRB, 0.66747E+00_JPRB, 0.67974E+00_JPRB, 0.68987E+00_JPRB, 0.69530E+00_JPRB ,&
&  0.69276E+00_JPRB, 0.68375E+00_JPRB, 0.67317E+00_JPRB, 0.66046E+00_JPRB, 0.64879E+00_JPRB, 0.63655E+00_JPRB, 0.62691E+00_JPRB ,&
&  0.62313E+00_JPRB, 0.61937E+00_JPRB, 0.61972E+00_JPRB, 0.62285E+00_JPRB, 0.62930E+00_JPRB, 0.63124E+00_JPRB, 0.63309E+00_JPRB ,&
&  0.63576E+00_JPRB, 0.63765E+00_JPRB, 0.63781E+00_JPRB, 0.63715E+00_JPRB, 0.63621E+00_JPRB, 0.64147E+00_JPRB, 0.64981E+00_JPRB ,&
&  0.65914E+00_JPRB, 0.67376E+00_JPRB, 0.69432E+00_JPRB, 0.71068E+00_JPRB, 0.71709E+00_JPRB, 0.71254E+00_JPRB, 0.70101E+00_JPRB ,&
&  0.69282E+00_JPRB, 0.69039E+00_JPRB, 0.68918E+00_JPRB, 0.68221E+00_JPRB, 0.67249E+00_JPRB, 0.66341E+00_JPRB, 0.65124E+00_JPRB ,&
&  0.63622E+00_JPRB, 0.62372E+00_JPRB, 0.61600E+00_JPRB, 0.61206E+00_JPRB, 0.61116E+00_JPRB, 0.60975E+00_JPRB, 0.60855E+00_JPRB ,&
&  0.60993E+00_JPRB, 0.61002E+00_JPRB, 0.61180E+00_JPRB, 0.61724E+00_JPRB, 0.62231E+00_JPRB, 0.62924E+00_JPRB, 0.63869E+00_JPRB ,&
&  0.65191E+00_JPRB, 0.66732E+00_JPRB, 0.68171E+00_JPRB, 0.70585E+00_JPRB, 0.74005E+00_JPRB, 0.76697E+00_JPRB, 0.78226E+00_JPRB ,&
&  0.79559E+00_JPRB, 0.79989E+00_JPRB, 0.79478E+00_JPRB, 0.78665E+00_JPRB, 0.77156E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.69087E+00_JPRB, 0.67443E+00_JPRB, 0.66139E+00_JPRB, 0.65127E+00_JPRB, 0.64412E+00_JPRB, 0.64131E+00_JPRB, 0.63797E+00_JPRB ,&
&  0.63484E+00_JPRB, 0.63207E+00_JPRB, 0.62462E+00_JPRB, 0.61751E+00_JPRB, 0.61698E+00_JPRB, 0.62288E+00_JPRB, 0.62737E+00_JPRB ,&
&  0.62406E+00_JPRB, 0.61283E+00_JPRB, 0.60222E+00_JPRB, 0.59965E+00_JPRB, 0.60215E+00_JPRB, 0.60430E+00_JPRB, 0.60376E+00_JPRB ,&
&  0.60542E+00_JPRB, 0.60656E+00_JPRB, 0.59820E+00_JPRB, 0.58063E+00_JPRB, 0.57258E+00_JPRB, 0.56817E+00_JPRB, 0.57510E+00_JPRB ,&
&  0.59407E+00_JPRB, 0.60851E+00_JPRB, 0.61701E+00_JPRB, 0.61195E+00_JPRB, 0.61139E+00_JPRB, 0.60929E+00_JPRB, 0.60372E+00_JPRB ,&
&  0.59839E+00_JPRB, 0.59165E+00_JPRB, 0.58613E+00_JPRB, 0.58213E+00_JPRB, 0.57972E+00_JPRB, 0.57839E+00_JPRB, 0.57956E+00_JPRB ,&
&  0.57731E+00_JPRB, 0.57302E+00_JPRB, 0.57424E+00_JPRB, 0.58062E+00_JPRB, 0.58930E+00_JPRB, 0.59457E+00_JPRB, 0.59949E+00_JPRB ,&
&  0.60361E+00_JPRB, 0.60592E+00_JPRB, 0.60891E+00_JPRB, 0.60965E+00_JPRB, 0.60587E+00_JPRB, 0.60522E+00_JPRB, 0.60542E+00_JPRB ,&
&  0.60821E+00_JPRB, 0.61364E+00_JPRB, 0.62316E+00_JPRB, 0.63437E+00_JPRB, 0.63936E+00_JPRB, 0.63586E+00_JPRB, 0.63093E+00_JPRB ,&
&  0.63067E+00_JPRB, 0.63487E+00_JPRB, 0.63963E+00_JPRB, 0.64214E+00_JPRB, 0.64075E+00_JPRB, 0.63246E+00_JPRB, 0.62237E+00_JPRB ,&
&  0.61166E+00_JPRB, 0.60078E+00_JPRB, 0.59106E+00_JPRB, 0.58483E+00_JPRB, 0.57997E+00_JPRB, 0.57663E+00_JPRB, 0.57462E+00_JPRB ,&
&  0.57407E+00_JPRB, 0.57456E+00_JPRB, 0.57839E+00_JPRB, 0.58546E+00_JPRB, 0.58985E+00_JPRB, 0.59891E+00_JPRB, 0.61018E+00_JPRB ,&
&  0.62208E+00_JPRB, 0.63427E+00_JPRB, 0.64881E+00_JPRB, 0.66899E+00_JPRB, 0.69231E+00_JPRB, 0.71172E+00_JPRB, 0.72664E+00_JPRB ,&
&  0.73632E+00_JPRB, 0.73636E+00_JPRB, 0.72869E+00_JPRB, 0.71909E+00_JPRB, 0.70670E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.63508E+00_JPRB, 0.62632E+00_JPRB, 0.61919E+00_JPRB, 0.61301E+00_JPRB, 0.60760E+00_JPRB, 0.60510E+00_JPRB, 0.60269E+00_JPRB ,&
&  0.60044E+00_JPRB, 0.59736E+00_JPRB, 0.59526E+00_JPRB, 0.59104E+00_JPRB, 0.58771E+00_JPRB, 0.59061E+00_JPRB, 0.59634E+00_JPRB ,&
&  0.59488E+00_JPRB, 0.58773E+00_JPRB, 0.58205E+00_JPRB, 0.58174E+00_JPRB, 0.58497E+00_JPRB, 0.58733E+00_JPRB, 0.59001E+00_JPRB ,&
&  0.58961E+00_JPRB, 0.58872E+00_JPRB, 0.58668E+00_JPRB, 0.57034E+00_JPRB, 0.55504E+00_JPRB, 0.54754E+00_JPRB, 0.54395E+00_JPRB ,&
&  0.54693E+00_JPRB, 0.56418E+00_JPRB, 0.54389E+00_JPRB, 0.55367E+00_JPRB, 0.54957E+00_JPRB, 0.54299E+00_JPRB, 0.53879E+00_JPRB ,&
&  0.53645E+00_JPRB, 0.53403E+00_JPRB, 0.53183E+00_JPRB, 0.53281E+00_JPRB, 0.53429E+00_JPRB, 0.53400E+00_JPRB, 0.53471E+00_JPRB ,&
&  0.53369E+00_JPRB, 0.53569E+00_JPRB, 0.53785E+00_JPRB, 0.54309E+00_JPRB, 0.54697E+00_JPRB, 0.54949E+00_JPRB, 0.55288E+00_JPRB ,&
&  0.55773E+00_JPRB, 0.56226E+00_JPRB, 0.56650E+00_JPRB, 0.57012E+00_JPRB, 0.57166E+00_JPRB, 0.57149E+00_JPRB, 0.56738E+00_JPRB ,&
&  0.56410E+00_JPRB, 0.56413E+00_JPRB, 0.56799E+00_JPRB, 0.57640E+00_JPRB, 0.58409E+00_JPRB, 0.58600E+00_JPRB, 0.58583E+00_JPRB ,&
&  0.58675E+00_JPRB, 0.59047E+00_JPRB, 0.59131E+00_JPRB, 0.59001E+00_JPRB, 0.58786E+00_JPRB, 0.58500E+00_JPRB, 0.58012E+00_JPRB ,&
&  0.57360E+00_JPRB, 0.56807E+00_JPRB, 0.56233E+00_JPRB, 0.55055E+00_JPRB, 0.54172E+00_JPRB, 0.54094E+00_JPRB, 0.54279E+00_JPRB ,&
&  0.54252E+00_JPRB, 0.54111E+00_JPRB, 0.54253E+00_JPRB, 0.54849E+00_JPRB, 0.54912E+00_JPRB, 0.55682E+00_JPRB, 0.57156E+00_JPRB ,&
&  0.58645E+00_JPRB, 0.59971E+00_JPRB, 0.61111E+00_JPRB, 0.62325E+00_JPRB, 0.63556E+00_JPRB, 0.64567E+00_JPRB, 0.65439E+00_JPRB ,&
&  0.65849E+00_JPRB, 0.65668E+00_JPRB, 0.65050E+00_JPRB, 0.64670E+00_JPRB, 0.64338E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.60219E+00_JPRB, 0.60062E+00_JPRB, 0.59878E+00_JPRB, 0.59636E+00_JPRB, 0.59282E+00_JPRB, 0.58956E+00_JPRB, 0.58453E+00_JPRB ,&
&  0.57941E+00_JPRB, 0.57532E+00_JPRB, 0.57348E+00_JPRB, 0.57271E+00_JPRB, 0.57383E+00_JPRB, 0.57798E+00_JPRB, 0.58495E+00_JPRB ,&
&  0.58490E+00_JPRB, 0.57775E+00_JPRB, 0.57427E+00_JPRB, 0.57415E+00_JPRB, 0.57643E+00_JPRB, 0.57804E+00_JPRB, 0.58053E+00_JPRB ,&
&  0.58204E+00_JPRB, 0.58231E+00_JPRB, 0.58100E+00_JPRB, 0.57301E+00_JPRB, 0.55543E+00_JPRB, 0.54497E+00_JPRB, 0.53622E+00_JPRB ,&
&  0.53176E+00_JPRB, 0.53006E+00_JPRB, 0.52615E+00_JPRB, 0.52506E+00_JPRB, 0.52388E+00_JPRB, 0.52199E+00_JPRB, 0.52189E+00_JPRB ,&
&  0.52006E+00_JPRB, 0.51761E+00_JPRB, 0.51643E+00_JPRB, 0.51702E+00_JPRB, 0.51881E+00_JPRB, 0.52036E+00_JPRB, 0.52010E+00_JPRB ,&
&  0.52086E+00_JPRB, 0.52437E+00_JPRB, 0.52513E+00_JPRB, 0.52503E+00_JPRB, 0.52464E+00_JPRB, 0.52400E+00_JPRB, 0.52397E+00_JPRB ,&
&  0.52639E+00_JPRB, 0.53177E+00_JPRB, 0.53657E+00_JPRB, 0.54156E+00_JPRB, 0.54399E+00_JPRB, 0.54521E+00_JPRB, 0.54531E+00_JPRB ,&
&  0.54211E+00_JPRB, 0.53790E+00_JPRB, 0.53640E+00_JPRB, 0.53915E+00_JPRB, 0.54492E+00_JPRB, 0.55121E+00_JPRB, 0.55468E+00_JPRB ,&
&  0.55721E+00_JPRB, 0.55812E+00_JPRB, 0.55588E+00_JPRB, 0.55466E+00_JPRB, 0.55194E+00_JPRB, 0.54736E+00_JPRB, 0.54475E+00_JPRB ,&
&  0.54224E+00_JPRB, 0.53989E+00_JPRB, 0.53750E+00_JPRB, 0.52820E+00_JPRB, 0.52044E+00_JPRB, 0.51604E+00_JPRB, 0.51336E+00_JPRB ,&
&  0.51250E+00_JPRB, 0.51170E+00_JPRB, 0.51152E+00_JPRB, 0.51276E+00_JPRB, 0.51509E+00_JPRB, 0.52246E+00_JPRB, 0.53668E+00_JPRB ,&
&  0.55731E+00_JPRB, 0.57631E+00_JPRB, 0.58672E+00_JPRB, 0.59421E+00_JPRB, 0.59935E+00_JPRB, 0.60211E+00_JPRB, 0.60477E+00_JPRB ,&
&  0.60857E+00_JPRB, 0.60994E+00_JPRB, 0.60795E+00_JPRB, 0.60537E+00_JPRB, 0.60421E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.57045E+00_JPRB, 0.57103E+00_JPRB, 0.56982E+00_JPRB, 0.56855E+00_JPRB, 0.56702E+00_JPRB, 0.56555E+00_JPRB, 0.56336E+00_JPRB ,&
&  0.55833E+00_JPRB, 0.55181E+00_JPRB, 0.54801E+00_JPRB, 0.54648E+00_JPRB, 0.55234E+00_JPRB, 0.56077E+00_JPRB, 0.56827E+00_JPRB ,&
&  0.56711E+00_JPRB, 0.55745E+00_JPRB, 0.55435E+00_JPRB, 0.55619E+00_JPRB, 0.55867E+00_JPRB, 0.56140E+00_JPRB, 0.56506E+00_JPRB ,&
&  0.56906E+00_JPRB, 0.57309E+00_JPRB, 0.57474E+00_JPRB, 0.57160E+00_JPRB, 0.56423E+00_JPRB, 0.54987E+00_JPRB, 0.53776E+00_JPRB ,&
&  0.52990E+00_JPRB, 0.52412E+00_JPRB, 0.51980E+00_JPRB, 0.51867E+00_JPRB, 0.51637E+00_JPRB, 0.51263E+00_JPRB, 0.50752E+00_JPRB ,&
&  0.50331E+00_JPRB, 0.50156E+00_JPRB, 0.50433E+00_JPRB, 0.50658E+00_JPRB, 0.50770E+00_JPRB, 0.50728E+00_JPRB, 0.50402E+00_JPRB ,&
&  0.50194E+00_JPRB, 0.50094E+00_JPRB, 0.50008E+00_JPRB, 0.50104E+00_JPRB, 0.49810E+00_JPRB, 0.49574E+00_JPRB, 0.49753E+00_JPRB ,&
&  0.50198E+00_JPRB, 0.50609E+00_JPRB, 0.51005E+00_JPRB, 0.51343E+00_JPRB, 0.51525E+00_JPRB, 0.51785E+00_JPRB, 0.51931E+00_JPRB ,&
&  0.51819E+00_JPRB, 0.51421E+00_JPRB, 0.51054E+00_JPRB, 0.51088E+00_JPRB, 0.51315E+00_JPRB, 0.51585E+00_JPRB, 0.51938E+00_JPRB ,&
&  0.51990E+00_JPRB, 0.51471E+00_JPRB, 0.50958E+00_JPRB, 0.50502E+00_JPRB, 0.50079E+00_JPRB, 0.49756E+00_JPRB, 0.49600E+00_JPRB ,&
&  0.49562E+00_JPRB, 0.49244E+00_JPRB, 0.48633E+00_JPRB, 0.48133E+00_JPRB, 0.48086E+00_JPRB, 0.48034E+00_JPRB, 0.47676E+00_JPRB ,&
&  0.47754E+00_JPRB, 0.48040E+00_JPRB, 0.47939E+00_JPRB, 0.47752E+00_JPRB, 0.47955E+00_JPRB, 0.48529E+00_JPRB, 0.49718E+00_JPRB ,&
&  0.51303E+00_JPRB, 0.53104E+00_JPRB, 0.54763E+00_JPRB, 0.56176E+00_JPRB, 0.57014E+00_JPRB, 0.57219E+00_JPRB, 0.57053E+00_JPRB ,&
&  0.57002E+00_JPRB, 0.57098E+00_JPRB, 0.57023E+00_JPRB, 0.56874E+00_JPRB, 0.56918E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.54003E+00_JPRB, 0.54026E+00_JPRB, 0.54073E+00_JPRB, 0.54206E+00_JPRB, 0.54296E+00_JPRB, 0.54196E+00_JPRB, 0.54157E+00_JPRB ,&
&  0.53825E+00_JPRB, 0.53350E+00_JPRB, 0.53076E+00_JPRB, 0.53264E+00_JPRB, 0.54227E+00_JPRB, 0.55416E+00_JPRB, 0.56106E+00_JPRB ,&
&  0.55610E+00_JPRB, 0.54304E+00_JPRB, 0.53973E+00_JPRB, 0.54196E+00_JPRB, 0.54420E+00_JPRB, 0.54831E+00_JPRB, 0.55436E+00_JPRB ,&
&  0.55989E+00_JPRB, 0.56361E+00_JPRB, 0.56769E+00_JPRB, 0.56923E+00_JPRB, 0.56905E+00_JPRB, 0.56146E+00_JPRB, 0.55008E+00_JPRB ,&
&  0.53917E+00_JPRB, 0.53122E+00_JPRB, 0.52555E+00_JPRB, 0.52180E+00_JPRB, 0.51673E+00_JPRB, 0.51186E+00_JPRB, 0.50558E+00_JPRB ,&
&  0.50285E+00_JPRB, 0.50304E+00_JPRB, 0.50440E+00_JPRB, 0.50273E+00_JPRB, 0.50010E+00_JPRB, 0.49451E+00_JPRB, 0.48958E+00_JPRB ,&
&  0.48727E+00_JPRB, 0.48783E+00_JPRB, 0.48833E+00_JPRB, 0.48665E+00_JPRB, 0.48245E+00_JPRB, 0.47935E+00_JPRB, 0.48027E+00_JPRB ,&
&  0.48591E+00_JPRB, 0.48561E+00_JPRB, 0.48464E+00_JPRB, 0.48586E+00_JPRB, 0.48854E+00_JPRB, 0.49083E+00_JPRB, 0.49024E+00_JPRB ,&
&  0.48939E+00_JPRB, 0.48900E+00_JPRB, 0.48892E+00_JPRB, 0.49049E+00_JPRB, 0.49047E+00_JPRB, 0.48826E+00_JPRB, 0.48258E+00_JPRB ,&
&  0.47398E+00_JPRB, 0.46327E+00_JPRB, 0.45894E+00_JPRB, 0.45578E+00_JPRB, 0.45268E+00_JPRB, 0.45210E+00_JPRB, 0.45363E+00_JPRB ,&
&  0.45233E+00_JPRB, 0.44617E+00_JPRB, 0.44348E+00_JPRB, 0.44429E+00_JPRB, 0.44383E+00_JPRB, 0.44659E+00_JPRB, 0.45107E+00_JPRB ,&
&  0.46216E+00_JPRB, 0.46851E+00_JPRB, 0.47039E+00_JPRB, 0.46815E+00_JPRB, 0.46828E+00_JPRB, 0.46997E+00_JPRB, 0.47446E+00_JPRB ,&
&  0.48127E+00_JPRB, 0.48878E+00_JPRB, 0.49666E+00_JPRB, 0.50535E+00_JPRB, 0.51600E+00_JPRB, 0.52613E+00_JPRB, 0.53015E+00_JPRB ,&
&  0.53064E+00_JPRB, 0.53140E+00_JPRB, 0.53310E+00_JPRB, 0.53502E+00_JPRB, 0.53840E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.50376E+00_JPRB, 0.50634E+00_JPRB, 0.50891E+00_JPRB, 0.51074E+00_JPRB, 0.51368E+00_JPRB, 0.51877E+00_JPRB, 0.52082E+00_JPRB ,&
&  0.51879E+00_JPRB, 0.51646E+00_JPRB, 0.51630E+00_JPRB, 0.52004E+00_JPRB, 0.52733E+00_JPRB, 0.53474E+00_JPRB, 0.54252E+00_JPRB ,&
&  0.53789E+00_JPRB, 0.52459E+00_JPRB, 0.51976E+00_JPRB, 0.52179E+00_JPRB, 0.52672E+00_JPRB, 0.53217E+00_JPRB, 0.53881E+00_JPRB ,&
&  0.54512E+00_JPRB, 0.54917E+00_JPRB, 0.55341E+00_JPRB, 0.55729E+00_JPRB, 0.55973E+00_JPRB, 0.55770E+00_JPRB, 0.55314E+00_JPRB ,&
&  0.54766E+00_JPRB, 0.54179E+00_JPRB, 0.53616E+00_JPRB, 0.52058E+00_JPRB, 0.51816E+00_JPRB, 0.50940E+00_JPRB, 0.50229E+00_JPRB ,&
&  0.49757E+00_JPRB, 0.49361E+00_JPRB, 0.49102E+00_JPRB, 0.48778E+00_JPRB, 0.48303E+00_JPRB, 0.47503E+00_JPRB, 0.47219E+00_JPRB ,&
&  0.47045E+00_JPRB, 0.47073E+00_JPRB, 0.46526E+00_JPRB, 0.46205E+00_JPRB, 0.45850E+00_JPRB, 0.45561E+00_JPRB, 0.45925E+00_JPRB ,&
&  0.46401E+00_JPRB, 0.46116E+00_JPRB, 0.45700E+00_JPRB, 0.45686E+00_JPRB, 0.46137E+00_JPRB, 0.46312E+00_JPRB, 0.46400E+00_JPRB ,&
&  0.46558E+00_JPRB, 0.46575E+00_JPRB, 0.46064E+00_JPRB, 0.45519E+00_JPRB, 0.44670E+00_JPRB, 0.43535E+00_JPRB, 0.43188E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.42314E+00_JPRB, 0.42035E+00_JPRB, 0.41943E+00_JPRB, 0.41915E+00_JPRB, 0.42027E+00_JPRB, 0.42069E+00_JPRB ,&
&  0.42106E+00_JPRB, 0.41759E+00_JPRB, 0.41253E+00_JPRB, 0.40839E+00_JPRB, 0.40268E+00_JPRB, 0.40595E+00_JPRB, 0.41661E+00_JPRB ,&
&  0.42961E+00_JPRB, 0.43770E+00_JPRB, 0.44297E+00_JPRB, 0.44641E+00_JPRB, 0.44945E+00_JPRB, 0.45070E+00_JPRB, 0.45294E+00_JPRB ,&
&  0.45594E+00_JPRB, 0.45725E+00_JPRB, 0.45599E+00_JPRB, 0.45684E+00_JPRB, 0.46071E+00_JPRB, 0.46646E+00_JPRB, 0.47324E+00_JPRB ,&
&  0.47990E+00_JPRB, 0.48554E+00_JPRB, 0.49028E+00_JPRB, 0.49613E+00_JPRB, 0.50012E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.47407E+00_JPRB, 0.47869E+00_JPRB, 0.48319E+00_JPRB, 0.48805E+00_JPRB, 0.49477E+00_JPRB, 0.49840E+00_JPRB, 0.49964E+00_JPRB ,&
&  0.50097E+00_JPRB, 0.50139E+00_JPRB, 0.50268E+00_JPRB, 0.50696E+00_JPRB, 0.51344E+00_JPRB, 0.51837E+00_JPRB, 0.52451E+00_JPRB ,&
&  0.52447E+00_JPRB, 0.51508E+00_JPRB, 0.50682E+00_JPRB, 0.50651E+00_JPRB, 0.51219E+00_JPRB, 0.51807E+00_JPRB, 0.52535E+00_JPRB ,&
&  0.53249E+00_JPRB, 0.53754E+00_JPRB, 0.54028E+00_JPRB, 0.54160E+00_JPRB, 0.54337E+00_JPRB, 0.54268E+00_JPRB, 0.53953E+00_JPRB ,&
&  0.53502E+00_JPRB, 0.52524E+00_JPRB, 0.50566E+00_JPRB, 0.50316E+00_JPRB, 0.49444E+00_JPRB, 0.48698E+00_JPRB, 0.48186E+00_JPRB ,&
&  0.47779E+00_JPRB, 0.47429E+00_JPRB, 0.47014E+00_JPRB, 0.46625E+00_JPRB, 0.46170E+00_JPRB, 0.45387E+00_JPRB, 0.44949E+00_JPRB ,&
&  0.44543E+00_JPRB, 0.44213E+00_JPRB, 0.43871E+00_JPRB, 0.43820E+00_JPRB, 0.43788E+00_JPRB, 0.44006E+00_JPRB, 0.44095E+00_JPRB ,&
&  0.43732E+00_JPRB, 0.43314E+00_JPRB, 0.43001E+00_JPRB, 0.43160E+00_JPRB, 0.43404E+00_JPRB, 0.43571E+00_JPRB, 0.43630E+00_JPRB ,&
&  0.43340E+00_JPRB, 0.42807E+00_JPRB, 0.42249E+00_JPRB, 0.41629E+00_JPRB, 0.40916E+00_JPRB, 0.38355E+00_JPRB, 0.37278E+00_JPRB ,&
&  0.37060E+00_JPRB, 0.38013E+00_JPRB, 0.39162E+00_JPRB, 0.39116E+00_JPRB, 0.39476E+00_JPRB, 0.39535E+00_JPRB, 0.39275E+00_JPRB ,&
&  0.38279E+00_JPRB, 0.36955E+00_JPRB, 0.36153E+00_JPRB, 0.35818E+00_JPRB, 0.35571E+00_JPRB, 0.35801E+00_JPRB, 0.36495E+00_JPRB ,&
&  0.37610E+00_JPRB, 0.38802E+00_JPRB, 0.39691E+00_JPRB, 0.40087E+00_JPRB, 0.39948E+00_JPRB, 0.39640E+00_JPRB, 0.39228E+00_JPRB ,&
&  0.38827E+00_JPRB, 0.39489E+00_JPRB, 0.41013E+00_JPRB, 0.42962E+00_JPRB, 0.43230E+00_JPRB, 0.43804E+00_JPRB, 0.44203E+00_JPRB ,&
&  0.44780E+00_JPRB, 0.45508E+00_JPRB, 0.46120E+00_JPRB, 0.46739E+00_JPRB, 0.47111E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.45067E+00_JPRB, 0.45593E+00_JPRB, 0.46301E+00_JPRB, 0.47085E+00_JPRB, 0.47653E+00_JPRB, 0.47934E+00_JPRB, 0.48233E+00_JPRB ,&
&  0.48597E+00_JPRB, 0.48754E+00_JPRB, 0.48858E+00_JPRB, 0.49125E+00_JPRB, 0.49688E+00_JPRB, 0.50021E+00_JPRB, 0.50544E+00_JPRB ,&
&  0.50761E+00_JPRB, 0.50518E+00_JPRB, 0.49803E+00_JPRB, 0.49605E+00_JPRB, 0.49983E+00_JPRB, 0.50477E+00_JPRB, 0.50997E+00_JPRB ,&
&  0.51521E+00_JPRB, 0.51996E+00_JPRB, 0.52210E+00_JPRB, 0.52247E+00_JPRB, 0.52240E+00_JPRB, 0.52203E+00_JPRB, 0.51924E+00_JPRB ,&
&  0.51287E+00_JPRB, 0.47217E+00_JPRB, 0.44592E+00_JPRB, 0.45144E+00_JPRB, 0.46236E+00_JPRB, 0.45566E+00_JPRB, 0.45414E+00_JPRB ,&
&  0.45266E+00_JPRB, 0.44879E+00_JPRB, 0.44307E+00_JPRB, 0.43825E+00_JPRB, 0.43246E+00_JPRB, 0.42484E+00_JPRB, 0.41879E+00_JPRB ,&
&  0.41698E+00_JPRB, 0.42224E+00_JPRB, 0.42348E+00_JPRB, 0.42466E+00_JPRB, 0.42216E+00_JPRB, 0.42303E+00_JPRB, 0.42081E+00_JPRB ,&
&  0.41433E+00_JPRB, 0.40706E+00_JPRB, 0.40426E+00_JPRB, 0.40351E+00_JPRB, 0.40466E+00_JPRB, 0.40741E+00_JPRB, 0.40647E+00_JPRB ,&
&  0.39376E+00_JPRB, 0.38200E+00_JPRB, 0.36306E+00_JPRB, 0.34468E+00_JPRB, 0.33148E+00_JPRB, 0.32564E+00_JPRB, 0.32143E+00_JPRB ,&
&  0.32100E+00_JPRB, 0.32799E+00_JPRB, 0.34688E+00_JPRB, 0.36914E+00_JPRB, 0.38028E+00_JPRB, 0.36162E+00_JPRB, 0.34682E+00_JPRB ,&
&  0.33805E+00_JPRB, 0.33322E+00_JPRB, 0.32904E+00_JPRB, 0.32789E+00_JPRB, 0.32873E+00_JPRB, 0.33132E+00_JPRB, 0.33493E+00_JPRB ,&
&  0.34017E+00_JPRB, 0.34508E+00_JPRB, 0.34790E+00_JPRB, 0.34924E+00_JPRB, 0.35005E+00_JPRB, 0.35095E+00_JPRB, 0.35036E+00_JPRB ,&
&  0.35047E+00_JPRB, 0.35465E+00_JPRB, 0.36235E+00_JPRB, 0.37183E+00_JPRB, 0.38031E+00_JPRB, 0.39004E+00_JPRB, 0.40366E+00_JPRB ,&
&  0.41839E+00_JPRB, 0.42930E+00_JPRB, 0.43348E+00_JPRB, 0.44016E+00_JPRB, 0.44577E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.43964E+00_JPRB, 0.44708E+00_JPRB, 0.45198E+00_JPRB, 0.46192E+00_JPRB, 0.46501E+00_JPRB, 0.46839E+00_JPRB, 0.47145E+00_JPRB ,&
&  0.47827E+00_JPRB, 0.48206E+00_JPRB, 0.48003E+00_JPRB, 0.47958E+00_JPRB, 0.47869E+00_JPRB, 0.47699E+00_JPRB, 0.47815E+00_JPRB ,&
&  0.47848E+00_JPRB, 0.48077E+00_JPRB, 0.47819E+00_JPRB, 0.47547E+00_JPRB, 0.47749E+00_JPRB, 0.48246E+00_JPRB, 0.48575E+00_JPRB ,&
&  0.48021E+00_JPRB, 0.47828E+00_JPRB, 0.48659E+00_JPRB, 0.48821E+00_JPRB, 0.48895E+00_JPRB, 0.49060E+00_JPRB, 0.49007E+00_JPRB ,&
&  0.47553E+00_JPRB, 0.44900E+00_JPRB, 0.42152E+00_JPRB, 0.42484E+00_JPRB, 0.43968E+00_JPRB, 0.43219E+00_JPRB, 0.43035E+00_JPRB ,&
&  0.42754E+00_JPRB, 0.42286E+00_JPRB, 0.41850E+00_JPRB, 0.41616E+00_JPRB, 0.41338E+00_JPRB, 0.40827E+00_JPRB, 0.40669E+00_JPRB ,&
&  0.40515E+00_JPRB, 0.41478E+00_JPRB, 0.40860E+00_JPRB, 0.38447E+00_JPRB, 0.36751E+00_JPRB, 0.35806E+00_JPRB, 0.34568E+00_JPRB ,&
&  0.33101E+00_JPRB, 0.32320E+00_JPRB, 0.32237E+00_JPRB, 0.31995E+00_JPRB, 0.32096E+00_JPRB, 0.32212E+00_JPRB, 0.32323E+00_JPRB ,&
&  0.32373E+00_JPRB, 0.32317E+00_JPRB, 0.31824E+00_JPRB, 0.31191E+00_JPRB, 0.30850E+00_JPRB, 0.30859E+00_JPRB, 0.31184E+00_JPRB ,&
&  0.31865E+00_JPRB, 0.32539E+00_JPRB, 0.34083E+00_JPRB, 0.34869E+00_JPRB, 0.33909E+00_JPRB, 0.33480E+00_JPRB, 0.33002E+00_JPRB ,&
&  0.31843E+00_JPRB, 0.31419E+00_JPRB, 0.31070E+00_JPRB, 0.30886E+00_JPRB, 0.30990E+00_JPRB, 0.31364E+00_JPRB, 0.31516E+00_JPRB ,&
&  0.31778E+00_JPRB, 0.32092E+00_JPRB, 0.32222E+00_JPRB, 0.32321E+00_JPRB, 0.32408E+00_JPRB, 0.32634E+00_JPRB, 0.32866E+00_JPRB ,&
&  0.33044E+00_JPRB, 0.33370E+00_JPRB, 0.33762E+00_JPRB, 0.34273E+00_JPRB, 0.34723E+00_JPRB, 0.34983E+00_JPRB, 0.35823E+00_JPRB ,&
&  0.36765E+00_JPRB, 0.38673E+00_JPRB, 0.41737E+00_JPRB, 0.44106E+00_JPRB, 0.43424E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.44043E+00_JPRB, 0.44471E+00_JPRB, 0.44506E+00_JPRB, 0.44835E+00_JPRB, 0.45384E+00_JPRB, 0.46009E+00_JPRB, 0.46548E+00_JPRB ,&
&  0.47095E+00_JPRB, 0.47062E+00_JPRB, 0.46559E+00_JPRB, 0.45760E+00_JPRB, 0.45098E+00_JPRB, 0.43417E+00_JPRB, 0.42566E+00_JPRB ,&
&  0.41855E+00_JPRB, 0.41864E+00_JPRB, 0.41901E+00_JPRB, 0.42080E+00_JPRB, 0.42168E+00_JPRB, 0.42676E+00_JPRB, 0.42802E+00_JPRB ,&
&  0.42499E+00_JPRB, 0.41946E+00_JPRB, 0.41377E+00_JPRB, 0.41262E+00_JPRB, 0.41526E+00_JPRB, 0.41700E+00_JPRB, 0.41368E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.40984E+00_JPRB, 0.41035E+00_JPRB, 0.41179E+00_JPRB, 0.42121E+00_JPRB, 0.41503E+00_JPRB, 0.41033E+00_JPRB ,&
&  0.40329E+00_JPRB, 0.39737E+00_JPRB, 0.39344E+00_JPRB, 0.39393E+00_JPRB, 0.39756E+00_JPRB, 0.39583E+00_JPRB, 0.39865E+00_JPRB ,&
&  0.38807E+00_JPRB, 0.37932E+00_JPRB, 0.36401E+00_JPRB, 0.35153E+00_JPRB, 0.33763E+00_JPRB, 0.32815E+00_JPRB, 0.31951E+00_JPRB ,&
&  0.31221E+00_JPRB, 0.30586E+00_JPRB, 0.29997E+00_JPRB, 0.29824E+00_JPRB, 0.29928E+00_JPRB, 0.29912E+00_JPRB, 0.29830E+00_JPRB ,&
&  0.29865E+00_JPRB, 0.29955E+00_JPRB, 0.30023E+00_JPRB, 0.30131E+00_JPRB, 0.30320E+00_JPRB, 0.30621E+00_JPRB, 0.31035E+00_JPRB ,&
&  0.31673E+00_JPRB, 0.32166E+00_JPRB, 0.32428E+00_JPRB, 0.32594E+00_JPRB, 0.32279E+00_JPRB, 0.31706E+00_JPRB, 0.31153E+00_JPRB ,&
&  0.30562E+00_JPRB, 0.30291E+00_JPRB, 0.30033E+00_JPRB, 0.29912E+00_JPRB, 0.29881E+00_JPRB, 0.29969E+00_JPRB, 0.30166E+00_JPRB ,&
&  0.30360E+00_JPRB, 0.30623E+00_JPRB, 0.30794E+00_JPRB, 0.31001E+00_JPRB, 0.31246E+00_JPRB, 0.31459E+00_JPRB, 0.31679E+00_JPRB ,&
&  0.31929E+00_JPRB, 0.32126E+00_JPRB, 0.32467E+00_JPRB, 0.32895E+00_JPRB, 0.33725E+00_JPRB, 0.34648E+00_JPRB, 0.36269E+00_JPRB ,&
&  0.38110E+00_JPRB, 0.41207E+00_JPRB, 0.42015E+00_JPRB, 0.43053E+00_JPRB, 0.43910E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.43546E+00_JPRB, 0.43812E+00_JPRB, 0.44184E+00_JPRB, 0.44752E+00_JPRB, 0.45478E+00_JPRB, 0.46161E+00_JPRB, 0.46336E+00_JPRB ,&
&  0.45347E+00_JPRB, 0.44059E+00_JPRB, 0.42203E+00_JPRB, 0.40761E+00_JPRB, 0.39327E+00_JPRB, 0.38267E+00_JPRB, 0.37421E+00_JPRB ,&
&  0.36843E+00_JPRB, 0.36817E+00_JPRB, 0.36826E+00_JPRB, 0.36943E+00_JPRB, 0.36799E+00_JPRB, 0.36880E+00_JPRB, 0.36876E+00_JPRB ,&
&  0.36713E+00_JPRB, 0.36691E+00_JPRB, 0.36683E+00_JPRB, 0.37020E+00_JPRB, 0.37707E+00_JPRB, 0.38327E+00_JPRB, 0.38940E+00_JPRB ,&
&  0.39668E+00_JPRB, 0.39827E+00_JPRB, 0.40228E+00_JPRB, 0.40063E+00_JPRB, 0.39717E+00_JPRB, 0.39276E+00_JPRB, 0.38647E+00_JPRB ,&
&  0.38250E+00_JPRB, 0.37885E+00_JPRB, 0.37795E+00_JPRB, 0.37830E+00_JPRB, 0.37826E+00_JPRB, 0.38197E+00_JPRB, 0.37695E+00_JPRB ,&
&  0.37010E+00_JPRB, 0.35932E+00_JPRB, 0.34606E+00_JPRB, 0.33601E+00_JPRB, 0.32592E+00_JPRB, 0.31869E+00_JPRB, 0.31224E+00_JPRB ,&
&  0.30633E+00_JPRB, 0.30241E+00_JPRB, 0.29828E+00_JPRB, 0.29519E+00_JPRB, 0.29419E+00_JPRB, 0.29198E+00_JPRB, 0.29070E+00_JPRB ,&
&  0.29014E+00_JPRB, 0.29055E+00_JPRB, 0.29254E+00_JPRB, 0.29430E+00_JPRB, 0.29677E+00_JPRB, 0.29946E+00_JPRB, 0.30168E+00_JPRB ,&
&  0.30469E+00_JPRB, 0.30717E+00_JPRB, 0.30750E+00_JPRB, 0.30694E+00_JPRB, 0.30466E+00_JPRB, 0.30156E+00_JPRB, 0.29897E+00_JPRB ,&
&  0.29615E+00_JPRB, 0.29421E+00_JPRB, 0.29358E+00_JPRB, 0.29259E+00_JPRB, 0.29311E+00_JPRB, 0.29437E+00_JPRB, 0.29521E+00_JPRB ,&
&  0.29708E+00_JPRB, 0.29860E+00_JPRB, 0.30043E+00_JPRB, 0.30378E+00_JPRB, 0.30675E+00_JPRB, 0.30939E+00_JPRB, 0.31201E+00_JPRB ,&
&  0.31194E+00_JPRB, 0.31375E+00_JPRB, 0.31633E+00_JPRB, 0.32130E+00_JPRB, 0.33619E+00_JPRB, 0.34576E+00_JPRB, 0.36791E+00_JPRB ,&
&  0.38517E+00_JPRB, 0.41800E+00_JPRB, 0.41577E+00_JPRB, 0.42380E+00_JPRB, 0.43139E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.42957E+00_JPRB, 0.43667E+00_JPRB, 0.44330E+00_JPRB, 0.44863E+00_JPRB, 0.45216E+00_JPRB, 0.45546E+00_JPRB, 0.45294E+00_JPRB ,&
&  0.44686E+00_JPRB, 0.43249E+00_JPRB, 0.42131E+00_JPRB, 0.40981E+00_JPRB, 0.40030E+00_JPRB, 0.39102E+00_JPRB, 0.38245E+00_JPRB ,&
&  0.37410E+00_JPRB, 0.36707E+00_JPRB, 0.36109E+00_JPRB, 0.35578E+00_JPRB, 0.34861E+00_JPRB, 0.34225E+00_JPRB, 0.33713E+00_JPRB ,&
&  0.33288E+00_JPRB, 0.33277E+00_JPRB, 0.33531E+00_JPRB, 0.33993E+00_JPRB, 0.34863E+00_JPRB, 0.35701E+00_JPRB, 0.36632E+00_JPRB ,&
&  0.38038E+00_JPRB, 0.38272E+00_JPRB, 0.38156E+00_JPRB, 0.38097E+00_JPRB, 0.37596E+00_JPRB, 0.36961E+00_JPRB, 0.36068E+00_JPRB ,&
&  0.36316E+00_JPRB, 0.36118E+00_JPRB, 0.36939E+00_JPRB, 0.36928E+00_JPRB, 0.36792E+00_JPRB, 0.36917E+00_JPRB, 0.36206E+00_JPRB ,&
&  0.35763E+00_JPRB, 0.34814E+00_JPRB, 0.34076E+00_JPRB, 0.33245E+00_JPRB, 0.32366E+00_JPRB, 0.31852E+00_JPRB, 0.31169E+00_JPRB ,&
&  0.30680E+00_JPRB, 0.30333E+00_JPRB, 0.29869E+00_JPRB, 0.29590E+00_JPRB, 0.29338E+00_JPRB, 0.29069E+00_JPRB, 0.28929E+00_JPRB ,&
&  0.28822E+00_JPRB, 0.28785E+00_JPRB, 0.28808E+00_JPRB, 0.28834E+00_JPRB, 0.28894E+00_JPRB, 0.28998E+00_JPRB, 0.29073E+00_JPRB ,&
&  0.29159E+00_JPRB, 0.29194E+00_JPRB, 0.29158E+00_JPRB, 0.29121E+00_JPRB, 0.28994E+00_JPRB, 0.28914E+00_JPRB, 0.28800E+00_JPRB ,&
&  0.28728E+00_JPRB, 0.28761E+00_JPRB, 0.28787E+00_JPRB, 0.28860E+00_JPRB, 0.28930E+00_JPRB, 0.29099E+00_JPRB, 0.29254E+00_JPRB ,&
&  0.29420E+00_JPRB, 0.29638E+00_JPRB, 0.29809E+00_JPRB, 0.30037E+00_JPRB, 0.30304E+00_JPRB, 0.30469E+00_JPRB, 0.30780E+00_JPRB ,&
&  0.31079E+00_JPRB, 0.31461E+00_JPRB, 0.32149E+00_JPRB, 0.32893E+00_JPRB, 0.33895E+00_JPRB, 0.35044E+00_JPRB, 0.36459E+00_JPRB ,&
&  0.38255E+00_JPRB, 0.40668E+00_JPRB, 0.40758E+00_JPRB, 0.41744E+00_JPRB, 0.42361E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.41670E+00_JPRB, 0.42228E+00_JPRB, 0.42698E+00_JPRB, 0.43118E+00_JPRB, 0.43505E+00_JPRB, 0.43635E+00_JPRB, 0.43544E+00_JPRB ,&
&  0.42612E+00_JPRB, 0.41942E+00_JPRB, 0.41105E+00_JPRB, 0.40333E+00_JPRB, 0.39617E+00_JPRB, 0.38695E+00_JPRB, 0.37949E+00_JPRB ,&
&  0.37189E+00_JPRB, 0.36423E+00_JPRB, 0.35715E+00_JPRB, 0.34877E+00_JPRB, 0.34009E+00_JPRB, 0.33173E+00_JPRB, 0.32684E+00_JPRB ,&
&  0.32409E+00_JPRB, 0.32411E+00_JPRB, 0.32747E+00_JPRB, 0.33146E+00_JPRB, 0.33658E+00_JPRB, 0.34310E+00_JPRB, 0.35015E+00_JPRB ,&
&  0.35455E+00_JPRB, 0.35685E+00_JPRB, 0.35617E+00_JPRB, 0.35417E+00_JPRB, 0.35186E+00_JPRB, 0.35011E+00_JPRB, 0.34736E+00_JPRB ,&
&  0.34382E+00_JPRB, 0.34180E+00_JPRB, 0.34185E+00_JPRB, 0.34398E+00_JPRB, 0.34461E+00_JPRB, 0.34329E+00_JPRB, 0.33936E+00_JPRB ,&
&  0.33468E+00_JPRB, 0.32986E+00_JPRB, 0.32471E+00_JPRB, 0.31986E+00_JPRB, 0.31590E+00_JPRB, 0.31162E+00_JPRB, 0.30691E+00_JPRB ,&
&  0.30400E+00_JPRB, 0.30096E+00_JPRB, 0.29761E+00_JPRB, 0.29449E+00_JPRB, 0.29188E+00_JPRB, 0.28987E+00_JPRB, 0.28818E+00_JPRB ,&
&  0.28694E+00_JPRB, 0.28600E+00_JPRB, 0.28529E+00_JPRB, 0.28477E+00_JPRB, 0.28427E+00_JPRB, 0.28339E+00_JPRB, 0.28253E+00_JPRB ,&
&  0.28167E+00_JPRB, 0.28140E+00_JPRB, 0.28084E+00_JPRB, 0.28087E+00_JPRB, 0.28116E+00_JPRB, 0.28132E+00_JPRB, 0.28187E+00_JPRB ,&
&  0.28307E+00_JPRB, 0.28434E+00_JPRB, 0.28487E+00_JPRB, 0.28604E+00_JPRB, 0.28718E+00_JPRB, 0.28869E+00_JPRB, 0.29074E+00_JPRB ,&
&  0.29196E+00_JPRB, 0.29331E+00_JPRB, 0.29489E+00_JPRB, 0.29657E+00_JPRB, 0.29957E+00_JPRB, 0.30283E+00_JPRB, 0.30645E+00_JPRB ,&
&  0.31112E+00_JPRB, 0.31561E+00_JPRB, 0.32077E+00_JPRB, 0.32694E+00_JPRB, 0.33843E+00_JPRB, 0.34528E+00_JPRB, 0.35362E+00_JPRB ,&
&  0.36146E+00_JPRB, 0.36369E+00_JPRB, 0.39954E+00_JPRB, 0.40262E+00_JPRB, 0.40865E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.34188E+00_JPRB, 0.34867E+00_JPRB, 0.35280E+00_JPRB, 0.35692E+00_JPRB, 0.35165E+00_JPRB, 0.39100E+00_JPRB, 0.39007E+00_JPRB ,&
&  0.38939E+00_JPRB, 0.37649E+00_JPRB, 0.37520E+00_JPRB, 0.37087E+00_JPRB, 0.36546E+00_JPRB, 0.35933E+00_JPRB, 0.35401E+00_JPRB ,&
&  0.34907E+00_JPRB, 0.34346E+00_JPRB, 0.33898E+00_JPRB, 0.33482E+00_JPRB, 0.33141E+00_JPRB, 0.32841E+00_JPRB, 0.32631E+00_JPRB ,&
&  0.32585E+00_JPRB, 0.32753E+00_JPRB, 0.33103E+00_JPRB, 0.33265E+00_JPRB, 0.33309E+00_JPRB, 0.33344E+00_JPRB, 0.33385E+00_JPRB ,&
&  0.33365E+00_JPRB, 0.33329E+00_JPRB, 0.33175E+00_JPRB, 0.32900E+00_JPRB, 0.32698E+00_JPRB, 0.32566E+00_JPRB, 0.32471E+00_JPRB ,&
&  0.32387E+00_JPRB, 0.32314E+00_JPRB, 0.32235E+00_JPRB, 0.32200E+00_JPRB, 0.32121E+00_JPRB, 0.31989E+00_JPRB, 0.31774E+00_JPRB ,&
&  0.31551E+00_JPRB, 0.31314E+00_JPRB, 0.31017E+00_JPRB, 0.30780E+00_JPRB, 0.30548E+00_JPRB, 0.30313E+00_JPRB, 0.30076E+00_JPRB ,&
&  0.29943E+00_JPRB, 0.29813E+00_JPRB, 0.29647E+00_JPRB, 0.29438E+00_JPRB, 0.29283E+00_JPRB, 0.29119E+00_JPRB, 0.28978E+00_JPRB ,&
&  0.28889E+00_JPRB, 0.28793E+00_JPRB, 0.28695E+00_JPRB, 0.28594E+00_JPRB, 0.28522E+00_JPRB, 0.28436E+00_JPRB, 0.28355E+00_JPRB ,&
&  0.28281E+00_JPRB, 0.28195E+00_JPRB, 0.28174E+00_JPRB, 0.28163E+00_JPRB, 0.28171E+00_JPRB, 0.28173E+00_JPRB, 0.28205E+00_JPRB ,&
&  0.28256E+00_JPRB, 0.28333E+00_JPRB, 0.28430E+00_JPRB, 0.28517E+00_JPRB, 0.28602E+00_JPRB, 0.28689E+00_JPRB, 0.28821E+00_JPRB ,&
&  0.28960E+00_JPRB, 0.29106E+00_JPRB, 0.29256E+00_JPRB, 0.29369E+00_JPRB, 0.29638E+00_JPRB, 0.29891E+00_JPRB, 0.30158E+00_JPRB ,&
&  0.30465E+00_JPRB, 0.30834E+00_JPRB, 0.31237E+00_JPRB, 0.31673E+00_JPRB, 0.32260E+00_JPRB, 0.32598E+00_JPRB, 0.32835E+00_JPRB ,&
&  0.33096E+00_JPRB, 0.33557E+00_JPRB, 0.33673E+00_JPRB, 0.33783E+00_JPRB, 0.33958E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.31466E+00_JPRB, 0.31492E+00_JPRB, 0.31490E+00_JPRB, 0.31465E+00_JPRB, 0.31174E+00_JPRB, 0.31163E+00_JPRB, 0.31136E+00_JPRB ,&
&  0.31136E+00_JPRB, 0.31150E+00_JPRB, 0.31221E+00_JPRB, 0.31307E+00_JPRB, 0.31395E+00_JPRB, 0.31480E+00_JPRB, 0.31559E+00_JPRB ,&
&  0.31609E+00_JPRB, 0.31672E+00_JPRB, 0.31521E+00_JPRB, 0.31423E+00_JPRB, 0.31406E+00_JPRB, 0.31410E+00_JPRB, 0.31402E+00_JPRB ,&
&  0.31388E+00_JPRB, 0.31368E+00_JPRB, 0.31309E+00_JPRB, 0.31269E+00_JPRB, 0.31251E+00_JPRB, 0.31227E+00_JPRB, 0.31190E+00_JPRB ,&
&  0.31148E+00_JPRB, 0.31115E+00_JPRB, 0.31076E+00_JPRB, 0.31029E+00_JPRB, 0.31021E+00_JPRB, 0.31016E+00_JPRB, 0.30961E+00_JPRB ,&
&  0.30896E+00_JPRB, 0.30794E+00_JPRB, 0.30702E+00_JPRB, 0.30616E+00_JPRB, 0.30548E+00_JPRB, 0.30518E+00_JPRB, 0.30484E+00_JPRB ,&
&  0.30435E+00_JPRB, 0.30368E+00_JPRB, 0.30304E+00_JPRB, 0.30221E+00_JPRB, 0.30146E+00_JPRB, 0.30077E+00_JPRB, 0.29996E+00_JPRB ,&
&  0.29906E+00_JPRB, 0.29815E+00_JPRB, 0.29719E+00_JPRB, 0.29611E+00_JPRB, 0.29505E+00_JPRB, 0.29396E+00_JPRB, 0.29277E+00_JPRB ,&
&  0.29159E+00_JPRB, 0.29111E+00_JPRB, 0.29054E+00_JPRB, 0.29007E+00_JPRB, 0.28971E+00_JPRB, 0.28944E+00_JPRB, 0.28916E+00_JPRB ,&
&  0.28895E+00_JPRB, 0.28872E+00_JPRB, 0.28861E+00_JPRB, 0.28850E+00_JPRB, 0.28851E+00_JPRB, 0.28857E+00_JPRB, 0.28877E+00_JPRB ,&
&  0.28900E+00_JPRB, 0.28929E+00_JPRB, 0.28968E+00_JPRB, 0.29035E+00_JPRB, 0.29101E+00_JPRB, 0.29178E+00_JPRB, 0.29254E+00_JPRB ,&
&  0.29338E+00_JPRB, 0.29421E+00_JPRB, 0.29506E+00_JPRB, 0.29587E+00_JPRB, 0.29698E+00_JPRB, 0.29805E+00_JPRB, 0.29923E+00_JPRB ,&
&  0.30040E+00_JPRB, 0.30167E+00_JPRB, 0.30292E+00_JPRB, 0.30427E+00_JPRB, 0.30582E+00_JPRB, 0.30625E+00_JPRB, 0.30671E+00_JPRB ,&
&  0.30727E+00_JPRB, 0.30810E+00_JPRB, 0.30884E+00_JPRB, 0.30985E+00_JPRB, 0.31156E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB, 0.29932E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB, 0.77634E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.77321E+00_JPRB, 0.77411E+00_JPRB, 0.77485E+00_JPRB, 0.77527E+00_JPRB, 0.77557E+00_JPRB, 0.77577E+00_JPRB, 0.77586E+00_JPRB ,&
&  0.77569E+00_JPRB, 0.77595E+00_JPRB, 0.77645E+00_JPRB, 0.77693E+00_JPRB, 0.77733E+00_JPRB, 0.77762E+00_JPRB, 0.77758E+00_JPRB ,&
&  0.77805E+00_JPRB, 0.78080E+00_JPRB, 0.78253E+00_JPRB, 0.78326E+00_JPRB, 0.78384E+00_JPRB, 0.78427E+00_JPRB, 0.78443E+00_JPRB ,&
&  0.78468E+00_JPRB, 0.78494E+00_JPRB, 0.78516E+00_JPRB, 0.78526E+00_JPRB, 0.78540E+00_JPRB, 0.78561E+00_JPRB, 0.78590E+00_JPRB ,&
&  0.78644E+00_JPRB, 0.78772E+00_JPRB, 0.78948E+00_JPRB, 0.79140E+00_JPRB, 0.79368E+00_JPRB, 0.79678E+00_JPRB, 0.79941E+00_JPRB ,&
&  0.80129E+00_JPRB, 0.80224E+00_JPRB, 0.80118E+00_JPRB, 0.79848E+00_JPRB, 0.79486E+00_JPRB, 0.79112E+00_JPRB, 0.78752E+00_JPRB ,&
&  0.78341E+00_JPRB, 0.77964E+00_JPRB, 0.77628E+00_JPRB, 0.77337E+00_JPRB, 0.77064E+00_JPRB, 0.76831E+00_JPRB, 0.76621E+00_JPRB ,&
&  0.76468E+00_JPRB, 0.76311E+00_JPRB, 0.76174E+00_JPRB, 0.76044E+00_JPRB, 0.75907E+00_JPRB, 0.75722E+00_JPRB, 0.75526E+00_JPRB ,&
&  0.75386E+00_JPRB, 0.75309E+00_JPRB, 0.75257E+00_JPRB, 0.75196E+00_JPRB, 0.75164E+00_JPRB, 0.75216E+00_JPRB, 0.75305E+00_JPRB ,&
&  0.75386E+00_JPRB, 0.75501E+00_JPRB, 0.75638E+00_JPRB, 0.75724E+00_JPRB, 0.75775E+00_JPRB, 0.75776E+00_JPRB, 0.75753E+00_JPRB ,&
&  0.75717E+00_JPRB, 0.75740E+00_JPRB, 0.75772E+00_JPRB, 0.75865E+00_JPRB, 0.75962E+00_JPRB, 0.76083E+00_JPRB, 0.76223E+00_JPRB ,&
&  0.76330E+00_JPRB, 0.76446E+00_JPRB, 0.76543E+00_JPRB, 0.76668E+00_JPRB, 0.76777E+00_JPRB, 0.76854E+00_JPRB, 0.76901E+00_JPRB ,&
&  0.76935E+00_JPRB, 0.76947E+00_JPRB, 0.76960E+00_JPRB, 0.76955E+00_JPRB, 0.76968E+00_JPRB, 0.76989E+00_JPRB, 0.77008E+00_JPRB ,&
&  0.77026E+00_JPRB, 0.77067E+00_JPRB, 0.77100E+00_JPRB, 0.77176E+00_JPRB, 0.77244E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.77715E+00_JPRB, 0.77846E+00_JPRB, 0.77957E+00_JPRB, 0.78009E+00_JPRB, 0.78027E+00_JPRB, 0.77978E+00_JPRB, 0.77864E+00_JPRB ,&
&  0.77662E+00_JPRB, 0.77520E+00_JPRB, 0.77455E+00_JPRB, 0.77370E+00_JPRB, 0.77221E+00_JPRB, 0.77369E+00_JPRB, 0.77631E+00_JPRB ,&
&  0.77526E+00_JPRB, 0.77347E+00_JPRB, 0.77107E+00_JPRB, 0.76889E+00_JPRB, 0.76650E+00_JPRB, 0.76381E+00_JPRB, 0.76199E+00_JPRB ,&
&  0.76116E+00_JPRB, 0.75967E+00_JPRB, 0.75734E+00_JPRB, 0.75411E+00_JPRB, 0.75312E+00_JPRB, 0.75216E+00_JPRB, 0.75102E+00_JPRB ,&
&  0.74948E+00_JPRB, 0.74584E+00_JPRB, 0.74247E+00_JPRB, 0.74044E+00_JPRB, 0.74478E+00_JPRB, 0.74917E+00_JPRB, 0.75392E+00_JPRB ,&
&  0.75869E+00_JPRB, 0.76226E+00_JPRB, 0.76703E+00_JPRB, 0.76959E+00_JPRB, 0.76904E+00_JPRB, 0.77146E+00_JPRB, 0.77319E+00_JPRB ,&
&  0.77238E+00_JPRB, 0.76907E+00_JPRB, 0.76668E+00_JPRB, 0.76635E+00_JPRB, 0.76631E+00_JPRB, 0.76589E+00_JPRB, 0.76559E+00_JPRB ,&
&  0.76603E+00_JPRB, 0.76604E+00_JPRB, 0.76525E+00_JPRB, 0.76404E+00_JPRB, 0.76195E+00_JPRB, 0.75917E+00_JPRB, 0.75579E+00_JPRB ,&
&  0.75310E+00_JPRB, 0.74987E+00_JPRB, 0.74663E+00_JPRB, 0.74340E+00_JPRB, 0.74101E+00_JPRB, 0.73789E+00_JPRB, 0.73450E+00_JPRB ,&
&  0.73282E+00_JPRB, 0.73401E+00_JPRB, 0.73642E+00_JPRB, 0.73692E+00_JPRB, 0.73750E+00_JPRB, 0.73718E+00_JPRB, 0.73862E+00_JPRB ,&
&  0.74025E+00_JPRB, 0.74403E+00_JPRB, 0.74796E+00_JPRB, 0.75357E+00_JPRB, 0.75884E+00_JPRB, 0.76397E+00_JPRB, 0.76786E+00_JPRB ,&
&  0.77151E+00_JPRB, 0.77380E+00_JPRB, 0.77505E+00_JPRB, 0.77543E+00_JPRB, 0.77619E+00_JPRB, 0.77643E+00_JPRB, 0.77593E+00_JPRB ,&
&  0.77503E+00_JPRB, 0.77460E+00_JPRB, 0.77387E+00_JPRB, 0.77279E+00_JPRB, 0.77219E+00_JPRB, 0.77138E+00_JPRB, 0.77098E+00_JPRB ,&
&  0.77064E+00_JPRB, 0.77119E+00_JPRB, 0.77191E+00_JPRB, 0.77344E+00_JPRB, 0.77531E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.76466E+00_JPRB, 0.76750E+00_JPRB, 0.76970E+00_JPRB, 0.77012E+00_JPRB, 0.77036E+00_JPRB, 0.77077E+00_JPRB, 0.77098E+00_JPRB ,&
&  0.77121E+00_JPRB, 0.77126E+00_JPRB, 0.77200E+00_JPRB, 0.77216E+00_JPRB, 0.77208E+00_JPRB, 0.77704E+00_JPRB, 0.77819E+00_JPRB ,&
&  0.77693E+00_JPRB, 0.77401E+00_JPRB, 0.77091E+00_JPRB, 0.76667E+00_JPRB, 0.76125E+00_JPRB, 0.75424E+00_JPRB, 0.74198E+00_JPRB ,&
&  0.73454E+00_JPRB, 0.72698E+00_JPRB, 0.71847E+00_JPRB, 0.71659E+00_JPRB, 0.71434E+00_JPRB, 0.68558E+00_JPRB, 0.68502E+00_JPRB ,&
&  0.71186E+00_JPRB, 0.72306E+00_JPRB, 0.72609E+00_JPRB, 0.73024E+00_JPRB, 0.69978E+00_JPRB, 0.66403E+00_JPRB, 0.64020E+00_JPRB ,&
&  0.60146E+00_JPRB, 0.63257E+00_JPRB, 0.65137E+00_JPRB, 0.65586E+00_JPRB, 0.65226E+00_JPRB, 0.64602E+00_JPRB, 0.66825E+00_JPRB ,&
&  0.72314E+00_JPRB, 0.73306E+00_JPRB, 0.74679E+00_JPRB, 0.76172E+00_JPRB, 0.77682E+00_JPRB, 0.78737E+00_JPRB, 0.79961E+00_JPRB ,&
&  0.80646E+00_JPRB, 0.81127E+00_JPRB, 0.81040E+00_JPRB, 0.80729E+00_JPRB, 0.79818E+00_JPRB, 0.79078E+00_JPRB, 0.78342E+00_JPRB ,&
&  0.77991E+00_JPRB, 0.77628E+00_JPRB, 0.77298E+00_JPRB, 0.76711E+00_JPRB, 0.75978E+00_JPRB, 0.75176E+00_JPRB, 0.74358E+00_JPRB ,&
&  0.73570E+00_JPRB, 0.72692E+00_JPRB, 0.72155E+00_JPRB, 0.71751E+00_JPRB, 0.71943E+00_JPRB, 0.71944E+00_JPRB, 0.72348E+00_JPRB ,&
&  0.72715E+00_JPRB, 0.73294E+00_JPRB, 0.73914E+00_JPRB, 0.74779E+00_JPRB, 0.75453E+00_JPRB, 0.75992E+00_JPRB, 0.76278E+00_JPRB ,&
&  0.76515E+00_JPRB, 0.76591E+00_JPRB, 0.76589E+00_JPRB, 0.76470E+00_JPRB, 0.76419E+00_JPRB, 0.76390E+00_JPRB, 0.76309E+00_JPRB ,&
&  0.76277E+00_JPRB, 0.76253E+00_JPRB, 0.76264E+00_JPRB, 0.76240E+00_JPRB, 0.76184E+00_JPRB, 0.75972E+00_JPRB, 0.75730E+00_JPRB ,&
&  0.75477E+00_JPRB, 0.75450E+00_JPRB, 0.75513E+00_JPRB, 0.75769E+00_JPRB, 0.76044E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.77455E+00_JPRB, 0.77351E+00_JPRB, 0.77351E+00_JPRB, 0.77346E+00_JPRB, 0.77352E+00_JPRB, 0.77349E+00_JPRB, 0.77336E+00_JPRB ,&
&  0.77402E+00_JPRB, 0.77524E+00_JPRB, 0.77582E+00_JPRB, 0.77524E+00_JPRB, 0.77606E+00_JPRB, 0.77768E+00_JPRB, 0.77807E+00_JPRB ,&
&  0.77494E+00_JPRB, 0.77053E+00_JPRB, 0.76640E+00_JPRB, 0.76079E+00_JPRB, 0.75539E+00_JPRB, 0.75050E+00_JPRB, 0.73983E+00_JPRB ,&
&  0.73296E+00_JPRB, 0.72253E+00_JPRB, 0.68373E+00_JPRB, 0.68800E+00_JPRB, 0.68053E+00_JPRB, 0.67926E+00_JPRB, 0.68350E+00_JPRB ,&
&  0.67189E+00_JPRB, 0.67028E+00_JPRB, 0.65536E+00_JPRB, 0.63247E+00_JPRB, 0.61945E+00_JPRB, 0.59808E+00_JPRB, 0.59797E+00_JPRB ,&
&  0.59376E+00_JPRB, 0.59978E+00_JPRB, 0.61340E+00_JPRB, 0.61746E+00_JPRB, 0.61314E+00_JPRB, 0.61056E+00_JPRB, 0.62183E+00_JPRB ,&
&  0.65186E+00_JPRB, 0.69615E+00_JPRB, 0.75460E+00_JPRB, 0.78615E+00_JPRB, 0.81748E+00_JPRB, 0.84359E+00_JPRB, 0.86846E+00_JPRB ,&
&  0.88732E+00_JPRB, 0.90270E+00_JPRB, 0.90615E+00_JPRB, 0.90645E+00_JPRB, 0.89619E+00_JPRB, 0.87813E+00_JPRB, 0.85431E+00_JPRB ,&
&  0.83071E+00_JPRB, 0.81363E+00_JPRB, 0.80121E+00_JPRB, 0.79546E+00_JPRB, 0.79350E+00_JPRB, 0.79197E+00_JPRB, 0.79172E+00_JPRB ,&
&  0.78776E+00_JPRB, 0.77203E+00_JPRB, 0.75924E+00_JPRB, 0.74456E+00_JPRB, 0.74053E+00_JPRB, 0.73854E+00_JPRB, 0.74331E+00_JPRB ,&
&  0.74917E+00_JPRB, 0.76107E+00_JPRB, 0.77461E+00_JPRB, 0.77878E+00_JPRB, 0.78576E+00_JPRB, 0.79459E+00_JPRB, 0.79328E+00_JPRB ,&
&  0.79201E+00_JPRB, 0.78866E+00_JPRB, 0.78302E+00_JPRB, 0.77657E+00_JPRB, 0.76958E+00_JPRB, 0.76321E+00_JPRB, 0.75808E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.75698E+00_JPRB, 0.76249E+00_JPRB, 0.76879E+00_JPRB, 0.77616E+00_JPRB, 0.78041E+00_JPRB, 0.78034E+00_JPRB ,&
&  0.77603E+00_JPRB, 0.77346E+00_JPRB, 0.77154E+00_JPRB, 0.77330E+00_JPRB, 0.77460E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.79462E+00_JPRB, 0.79471E+00_JPRB, 0.79832E+00_JPRB, 0.80345E+00_JPRB, 0.80798E+00_JPRB, 0.80929E+00_JPRB, 0.80145E+00_JPRB ,&
&  0.79467E+00_JPRB, 0.78817E+00_JPRB, 0.78313E+00_JPRB, 0.77974E+00_JPRB, 0.77868E+00_JPRB, 0.77525E+00_JPRB, 0.77103E+00_JPRB ,&
&  0.76740E+00_JPRB, 0.76603E+00_JPRB, 0.76626E+00_JPRB, 0.76613E+00_JPRB, 0.76817E+00_JPRB, 0.76744E+00_JPRB, 0.76402E+00_JPRB ,&
&  0.76058E+00_JPRB, 0.74546E+00_JPRB, 0.71499E+00_JPRB, 0.70576E+00_JPRB, 0.69464E+00_JPRB, 0.69577E+00_JPRB, 0.69600E+00_JPRB ,&
&  0.69201E+00_JPRB, 0.62081E+00_JPRB, 0.63095E+00_JPRB, 0.61251E+00_JPRB, 0.60117E+00_JPRB, 0.60035E+00_JPRB, 0.58465E+00_JPRB ,&
&  0.57459E+00_JPRB, 0.56108E+00_JPRB, 0.56006E+00_JPRB, 0.56661E+00_JPRB, 0.56467E+00_JPRB, 0.57597E+00_JPRB, 0.59171E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.73863E+00_JPRB, 0.79377E+00_JPRB, 0.82696E+00_JPRB, 0.85536E+00_JPRB, 0.88783E+00_JPRB, 0.92109E+00_JPRB ,&
&  0.94818E+00_JPRB, 0.98074E+00_JPRB, 0.99775E+00_JPRB, 0.99643E+00_JPRB, 0.10108E+01_JPRB, 0.10052E+01_JPRB, 0.98362E+00_JPRB ,&
&  0.94744E+00_JPRB, 0.89662E+00_JPRB, 0.86042E+00_JPRB, 0.83844E+00_JPRB, 0.82236E+00_JPRB, 0.81122E+00_JPRB, 0.80542E+00_JPRB ,&
&  0.81204E+00_JPRB, 0.80841E+00_JPRB, 0.80124E+00_JPRB, 0.78423E+00_JPRB, 0.77494E+00_JPRB, 0.76937E+00_JPRB, 0.77121E+00_JPRB ,&
&  0.77652E+00_JPRB, 0.78919E+00_JPRB, 0.80547E+00_JPRB, 0.81767E+00_JPRB, 0.82394E+00_JPRB, 0.82304E+00_JPRB, 0.82724E+00_JPRB ,&
&  0.82512E+00_JPRB, 0.81907E+00_JPRB, 0.81260E+00_JPRB, 0.80612E+00_JPRB, 0.79987E+00_JPRB, 0.79611E+00_JPRB, 0.79859E+00_JPRB ,&
&  0.80229E+00_JPRB, 0.80409E+00_JPRB, 0.80395E+00_JPRB, 0.80580E+00_JPRB, 0.81010E+00_JPRB, 0.81379E+00_JPRB, 0.81486E+00_JPRB ,&
&  0.80845E+00_JPRB, 0.80178E+00_JPRB, 0.79653E+00_JPRB, 0.79674E+00_JPRB, 0.79634E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.78729E+00_JPRB, 0.79590E+00_JPRB, 0.81415E+00_JPRB, 0.82956E+00_JPRB, 0.83896E+00_JPRB, 0.83797E+00_JPRB, 0.82535E+00_JPRB ,&
&  0.81379E+00_JPRB, 0.80535E+00_JPRB, 0.79624E+00_JPRB, 0.78780E+00_JPRB, 0.77831E+00_JPRB, 0.77084E+00_JPRB, 0.76577E+00_JPRB ,&
&  0.76305E+00_JPRB, 0.76397E+00_JPRB, 0.76523E+00_JPRB, 0.76632E+00_JPRB, 0.76880E+00_JPRB, 0.77207E+00_JPRB, 0.77209E+00_JPRB ,&
&  0.77116E+00_JPRB, 0.75878E+00_JPRB, 0.73247E+00_JPRB, 0.71383E+00_JPRB, 0.70800E+00_JPRB, 0.69657E+00_JPRB, 0.70301E+00_JPRB ,&
&  0.70316E+00_JPRB, 0.70167E+00_JPRB, 0.69453E+00_JPRB, 0.69052E+00_JPRB, 0.66254E+00_JPRB, 0.56548E+00_JPRB, 0.56974E+00_JPRB ,&
&  0.55495E+00_JPRB, 0.53479E+00_JPRB, 0.53042E+00_JPRB, 0.52907E+00_JPRB, 0.53018E+00_JPRB, 0.55624E+00_JPRB, 0.58540E+00_JPRB ,&
&  0.61187E+00_JPRB, 0.74736E+00_JPRB, 0.78564E+00_JPRB, 0.82729E+00_JPRB, 0.86476E+00_JPRB, 0.91022E+00_JPRB, 0.95864E+00_JPRB ,&
&  0.99185E+00_JPRB, 0.10277E+01_JPRB, 0.10588E+01_JPRB, 0.10842E+01_JPRB, 0.10996E+01_JPRB, 0.11118E+01_JPRB, 0.11223E+01_JPRB ,&
&  0.11130E+01_JPRB, 0.10228E+01_JPRB, 0.96357E+00_JPRB, 0.92844E+00_JPRB, 0.89382E+00_JPRB, 0.87074E+00_JPRB, 0.85379E+00_JPRB ,&
&  0.85139E+00_JPRB, 0.83525E+00_JPRB, 0.83520E+00_JPRB, 0.82451E+00_JPRB, 0.81842E+00_JPRB, 0.81334E+00_JPRB, 0.80688E+00_JPRB ,&
&  0.79752E+00_JPRB, 0.80147E+00_JPRB, 0.82083E+00_JPRB, 0.83026E+00_JPRB, 0.83840E+00_JPRB, 0.83272E+00_JPRB, 0.82120E+00_JPRB ,&
&  0.80214E+00_JPRB, 0.79843E+00_JPRB, 0.78968E+00_JPRB, 0.78718E+00_JPRB, 0.78776E+00_JPRB, 0.79916E+00_JPRB, 0.81407E+00_JPRB ,&
&  0.82313E+00_JPRB, 0.82460E+00_JPRB, 0.82282E+00_JPRB, 0.82240E+00_JPRB, 0.82020E+00_JPRB, 0.81580E+00_JPRB, 0.80845E+00_JPRB ,&
&  0.80134E+00_JPRB, 0.79795E+00_JPRB, 0.79093E+00_JPRB, 0.78646E+00_JPRB, 0.78573E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.77602E+00_JPRB, 0.80017E+00_JPRB, 0.82954E+00_JPRB, 0.84873E+00_JPRB, 0.85596E+00_JPRB, 0.84254E+00_JPRB, 0.82331E+00_JPRB ,&
&  0.80826E+00_JPRB, 0.79746E+00_JPRB, 0.78815E+00_JPRB, 0.78172E+00_JPRB, 0.77473E+00_JPRB, 0.76787E+00_JPRB, 0.76262E+00_JPRB ,&
&  0.75935E+00_JPRB, 0.74725E+00_JPRB, 0.74434E+00_JPRB, 0.74943E+00_JPRB, 0.75286E+00_JPRB, 0.75983E+00_JPRB, 0.76617E+00_JPRB ,&
&  0.76419E+00_JPRB, 0.75566E+00_JPRB, 0.74801E+00_JPRB, 0.72828E+00_JPRB, 0.70892E+00_JPRB, 0.70378E+00_JPRB, 0.71242E+00_JPRB ,&
&  0.69712E+00_JPRB, 0.67005E+00_JPRB, 0.69576E+00_JPRB, 0.71315E+00_JPRB, 0.67215E+00_JPRB, 0.63032E+00_JPRB, 0.57742E+00_JPRB ,&
&  0.56029E+00_JPRB, 0.52917E+00_JPRB, 0.52060E+00_JPRB, 0.51809E+00_JPRB, 0.52747E+00_JPRB, 0.55107E+00_JPRB, 0.59805E+00_JPRB ,&
&  0.65202E+00_JPRB, 0.75023E+00_JPRB, 0.78611E+00_JPRB, 0.83805E+00_JPRB, 0.88253E+00_JPRB, 0.92153E+00_JPRB, 0.95080E+00_JPRB ,&
&  0.99775E+00_JPRB, 0.10649E+01_JPRB, 0.11166E+01_JPRB, 0.11444E+01_JPRB, 0.11780E+01_JPRB, 0.12192E+01_JPRB, 0.12891E+01_JPRB ,&
&  0.13841E+01_JPRB, 0.12060E+01_JPRB, 0.11025E+01_JPRB, 0.10352E+01_JPRB, 0.99049E+00_JPRB, 0.97628E+00_JPRB, 0.92437E+00_JPRB ,&
&  0.90756E+00_JPRB, 0.90266E+00_JPRB, 0.88963E+00_JPRB, 0.88460E+00_JPRB, 0.87886E+00_JPRB, 0.87213E+00_JPRB, 0.86045E+00_JPRB ,&
&  0.84514E+00_JPRB, 0.85272E+00_JPRB, 0.93843E+00_JPRB, 0.95783E+00_JPRB, 0.94108E+00_JPRB, 0.88528E+00_JPRB, 0.82400E+00_JPRB ,&
&  0.78138E+00_JPRB, 0.76429E+00_JPRB, 0.75770E+00_JPRB, 0.75524E+00_JPRB, 0.76333E+00_JPRB, 0.77516E+00_JPRB, 0.79397E+00_JPRB ,&
&  0.79198E+00_JPRB, 0.80139E+00_JPRB, 0.81139E+00_JPRB, 0.81675E+00_JPRB, 0.81563E+00_JPRB, 0.80783E+00_JPRB, 0.80202E+00_JPRB ,&
&  0.79715E+00_JPRB, 0.79178E+00_JPRB, 0.78268E+00_JPRB, 0.77210E+00_JPRB, 0.76938E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.75302E+00_JPRB, 0.79437E+00_JPRB, 0.84312E+00_JPRB, 0.87355E+00_JPRB, 0.87337E+00_JPRB, 0.84578E+00_JPRB, 0.81526E+00_JPRB ,&
&  0.79479E+00_JPRB, 0.78548E+00_JPRB, 0.77677E+00_JPRB, 0.77077E+00_JPRB, 0.76592E+00_JPRB, 0.76084E+00_JPRB, 0.75575E+00_JPRB ,&
&  0.75191E+00_JPRB, 0.74823E+00_JPRB, 0.74168E+00_JPRB, 0.73273E+00_JPRB, 0.72886E+00_JPRB, 0.73359E+00_JPRB, 0.74326E+00_JPRB ,&
&  0.75287E+00_JPRB, 0.75379E+00_JPRB, 0.75380E+00_JPRB, 0.74306E+00_JPRB, 0.72754E+00_JPRB, 0.70958E+00_JPRB, 0.69644E+00_JPRB ,&
&  0.69062E+00_JPRB, 0.69457E+00_JPRB, 0.68879E+00_JPRB, 0.71162E+00_JPRB, 0.70110E+00_JPRB, 0.65273E+00_JPRB, 0.60308E+00_JPRB ,&
&  0.57296E+00_JPRB, 0.53784E+00_JPRB, 0.52325E+00_JPRB, 0.52816E+00_JPRB, 0.56203E+00_JPRB, 0.60295E+00_JPRB, 0.66408E+00_JPRB ,&
&  0.77162E+00_JPRB, 0.84242E+00_JPRB, 0.89295E+00_JPRB, 0.93580E+00_JPRB, 0.96956E+00_JPRB, 0.10081E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.11173E+01_JPRB, 0.11818E+01_JPRB, 0.12331E+01_JPRB, 0.12313E+01_JPRB, 0.12502E+01_JPRB, 0.12051E+01_JPRB, 0.13718E+01_JPRB ,&
&  0.17943E+01_JPRB, 0.15352E+01_JPRB, 0.13403E+01_JPRB, 0.12332E+01_JPRB, 0.11714E+01_JPRB, 0.11154E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.10128E+01_JPRB, 0.98296E+00_JPRB, 0.96772E+00_JPRB, 0.92111E+00_JPRB, 0.87479E+00_JPRB, 0.87082E+00_JPRB, 0.92381E+00_JPRB ,&
&  0.10582E+01_JPRB, 0.12498E+01_JPRB, 0.14734E+01_JPRB, 0.10569E+01_JPRB, 0.92766E+00_JPRB, 0.83417E+00_JPRB, 0.78759E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.77108E+00_JPRB, 0.77360E+00_JPRB, 0.77016E+00_JPRB, 0.76650E+00_JPRB, 0.76651E+00_JPRB, 0.76424E+00_JPRB ,&
&  0.75922E+00_JPRB, 0.76844E+00_JPRB, 0.78281E+00_JPRB, 0.79450E+00_JPRB, 0.80660E+00_JPRB, 0.80788E+00_JPRB, 0.80551E+00_JPRB ,&
&  0.80202E+00_JPRB, 0.79083E+00_JPRB, 0.77671E+00_JPRB, 0.75332E+00_JPRB, 0.74219E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.74573E+00_JPRB, 0.79339E+00_JPRB, 0.85808E+00_JPRB, 0.89508E+00_JPRB, 0.89201E+00_JPRB, 0.84832E+00_JPRB, 0.79180E+00_JPRB ,&
&  0.75179E+00_JPRB, 0.72763E+00_JPRB, 0.71930E+00_JPRB, 0.71614E+00_JPRB, 0.71648E+00_JPRB, 0.71755E+00_JPRB, 0.71693E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71359E+00_JPRB, 0.71171E+00_JPRB, 0.71134E+00_JPRB, 0.71061E+00_JPRB, 0.71213E+00_JPRB, 0.71761E+00_JPRB ,&
&  0.72575E+00_JPRB, 0.72988E+00_JPRB, 0.73064E+00_JPRB, 0.72859E+00_JPRB, 0.72988E+00_JPRB, 0.73572E+00_JPRB, 0.73542E+00_JPRB ,&
&  0.71579E+00_JPRB, 0.69694E+00_JPRB, 0.70241E+00_JPRB, 0.72012E+00_JPRB, 0.71474E+00_JPRB, 0.67511E+00_JPRB, 0.61938E+00_JPRB ,&
&  0.58249E+00_JPRB, 0.55720E+00_JPRB, 0.57466E+00_JPRB, 0.62242E+00_JPRB, 0.68534E+00_JPRB, 0.81844E+00_JPRB, 0.90894E+00_JPRB ,&
&  0.98495E+00_JPRB, 0.10281E+01_JPRB, 0.10555E+01_JPRB, 0.10862E+01_JPRB, 0.11461E+01_JPRB, 0.12117E+01_JPRB, 0.12715E+01_JPRB ,&
&  0.13340E+01_JPRB, 0.13996E+01_JPRB, 0.14740E+01_JPRB, 0.11996E+01_JPRB, 0.11974E+01_JPRB, 0.13443E+01_JPRB, 0.15061E+01_JPRB ,&
&  0.15870E+01_JPRB, 0.14931E+01_JPRB, 0.14369E+01_JPRB, 0.14556E+01_JPRB, 0.13572E+01_JPRB, 0.12757E+01_JPRB, 0.12163E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.11476E+01_JPRB, 0.10617E+01_JPRB, 0.96854E+00_JPRB, 0.95459E+00_JPRB, 0.95947E+00_JPRB, 0.98077E+00_JPRB ,&
&  0.10309E+01_JPRB, 0.10650E+01_JPRB, 0.97612E+00_JPRB, 0.78785E+00_JPRB, 0.73977E+00_JPRB, 0.72218E+00_JPRB, 0.72621E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.74603E+00_JPRB, 0.75885E+00_JPRB, 0.76423E+00_JPRB, 0.76476E+00_JPRB, 0.75317E+00_JPRB, 0.73957E+00_JPRB ,&
&  0.72663E+00_JPRB, 0.72590E+00_JPRB, 0.73522E+00_JPRB, 0.73546E+00_JPRB, 0.75299E+00_JPRB, 0.76943E+00_JPRB, 0.78601E+00_JPRB ,&
&  0.78330E+00_JPRB, 0.77412E+00_JPRB, 0.76356E+00_JPRB, 0.74014E+00_JPRB, 0.72536E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.78949E+00_JPRB, 0.83660E+00_JPRB, 0.87786E+00_JPRB, 0.89947E+00_JPRB, 0.89154E+00_JPRB, 0.85914E+00_JPRB, 0.78716E+00_JPRB ,&
&  0.75803E+00_JPRB, 0.71876E+00_JPRB, 0.70186E+00_JPRB, 0.68904E+00_JPRB, 0.67927E+00_JPRB, 0.67163E+00_JPRB, 0.67072E+00_JPRB ,&
&  0.67596E+00_JPRB, 0.68147E+00_JPRB, 0.68596E+00_JPRB, 0.69192E+00_JPRB, 0.69564E+00_JPRB, 0.70198E+00_JPRB, 0.70975E+00_JPRB ,&
&  0.71485E+00_JPRB, 0.71837E+00_JPRB, 0.72322E+00_JPRB, 0.72759E+00_JPRB, 0.73301E+00_JPRB, 0.73846E+00_JPRB, 0.73551E+00_JPRB ,&
&  0.72013E+00_JPRB, 0.70305E+00_JPRB, 0.71018E+00_JPRB, 0.72675E+00_JPRB, 0.72249E+00_JPRB, 0.69589E+00_JPRB, 0.63708E+00_JPRB ,&
&  0.59928E+00_JPRB, 0.61104E+00_JPRB, 0.70345E+00_JPRB, 0.85719E+00_JPRB, 0.92208E+00_JPRB, 0.98640E+00_JPRB, 0.10222E+01_JPRB ,&
&  0.97419E+00_JPRB, 0.97912E+00_JPRB, 0.10454E+01_JPRB, 0.12344E+01_JPRB, 0.12798E+01_JPRB, 0.13492E+01_JPRB, 0.14475E+01_JPRB ,&
&  0.16115E+01_JPRB, 0.16904E+01_JPRB, 0.17158E+01_JPRB, 0.15064E+01_JPRB, 0.17582E+01_JPRB, 0.21682E+01_JPRB, 0.19568E+01_JPRB ,&
&  0.18896E+01_JPRB, 0.18269E+01_JPRB, 0.18569E+01_JPRB, 0.17268E+01_JPRB, 0.15379E+01_JPRB, 0.14188E+01_JPRB, 0.13813E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.12771E+01_JPRB, 0.11241E+01_JPRB, 0.10680E+01_JPRB, 0.10727E+01_JPRB, 0.10461E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.98802E+00_JPRB, 0.92041E+00_JPRB, 0.86242E+00_JPRB, 0.80074E+00_JPRB, 0.76020E+00_JPRB, 0.75080E+00_JPRB, 0.75395E+00_JPRB ,&
&  0.75822E+00_JPRB, 0.76301E+00_JPRB, 0.77385E+00_JPRB, 0.78734E+00_JPRB, 0.78786E+00_JPRB, 0.76089E+00_JPRB, 0.75325E+00_JPRB ,&
&  0.74310E+00_JPRB, 0.72554E+00_JPRB, 0.71109E+00_JPRB, 0.71706E+00_JPRB, 0.72360E+00_JPRB, 0.73946E+00_JPRB, 0.75810E+00_JPRB ,&
&  0.77370E+00_JPRB, 0.78117E+00_JPRB, 0.77644E+00_JPRB, 0.77905E+00_JPRB, 0.77243E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.84006E+00_JPRB, 0.85803E+00_JPRB, 0.88569E+00_JPRB, 0.90607E+00_JPRB, 0.91600E+00_JPRB, 0.85719E+00_JPRB, 0.79999E+00_JPRB ,&
&  0.81358E+00_JPRB, 0.74150E+00_JPRB, 0.70073E+00_JPRB, 0.68944E+00_JPRB, 0.68661E+00_JPRB, 0.66784E+00_JPRB, 0.65538E+00_JPRB ,&
&  0.65254E+00_JPRB, 0.66412E+00_JPRB, 0.68193E+00_JPRB, 0.69513E+00_JPRB, 0.70796E+00_JPRB, 0.71930E+00_JPRB, 0.72483E+00_JPRB ,&
&  0.72934E+00_JPRB, 0.73383E+00_JPRB, 0.73755E+00_JPRB, 0.73952E+00_JPRB, 0.73816E+00_JPRB, 0.73945E+00_JPRB, 0.73535E+00_JPRB ,&
&  0.72115E+00_JPRB, 0.71007E+00_JPRB, 0.71620E+00_JPRB, 0.73503E+00_JPRB, 0.74157E+00_JPRB, 0.73704E+00_JPRB, 0.70758E+00_JPRB ,&
&  0.64306E+00_JPRB, 0.69945E+00_JPRB, 0.90578E+00_JPRB, 0.97317E+00_JPRB, 0.10191E+01_JPRB, 0.10781E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.12081E+01_JPRB, 0.12425E+01_JPRB, 0.12918E+01_JPRB, 0.13587E+01_JPRB, 0.14549E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.19992E+01_JPRB, 0.16495E+01_JPRB, 0.15538E+01_JPRB, 0.16622E+01_JPRB, 0.23326E+01_JPRB, 0.24993E+01_JPRB, 0.22553E+01_JPRB ,&
&  0.21468E+01_JPRB, 0.20359E+01_JPRB, 0.19397E+01_JPRB, 0.17750E+01_JPRB, 0.16515E+01_JPRB, 0.16087E+01_JPRB, 0.15708E+01_JPRB ,&
&  0.14608E+01_JPRB, 0.14003E+01_JPRB, 0.12306E+01_JPRB, 0.11914E+01_JPRB, 0.11870E+01_JPRB, 0.11302E+01_JPRB, 0.10476E+01_JPRB ,&
&  0.97491E+00_JPRB, 0.91592E+00_JPRB, 0.87204E+00_JPRB, 0.83959E+00_JPRB, 0.81395E+00_JPRB, 0.79888E+00_JPRB, 0.78832E+00_JPRB ,&
&  0.78332E+00_JPRB, 0.78409E+00_JPRB, 0.78035E+00_JPRB, 0.79471E+00_JPRB, 0.80103E+00_JPRB, 0.78103E+00_JPRB, 0.79132E+00_JPRB ,&
&  0.79221E+00_JPRB, 0.78046E+00_JPRB, 0.75488E+00_JPRB, 0.76091E+00_JPRB, 0.78071E+00_JPRB, 0.81639E+00_JPRB, 0.84709E+00_JPRB ,&
&  0.89099E+00_JPRB, 0.88105E+00_JPRB, 0.84150E+00_JPRB, 0.83120E+00_JPRB, 0.82671E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.87931E+00_JPRB, 0.89184E+00_JPRB, 0.90153E+00_JPRB, 0.91909E+00_JPRB, 0.93384E+00_JPRB, 0.88503E+00_JPRB, 0.84679E+00_JPRB ,&
&  0.90333E+00_JPRB, 0.81361E+00_JPRB, 0.85948E+00_JPRB, 0.76749E+00_JPRB, 0.72089E+00_JPRB, 0.69546E+00_JPRB, 0.67793E+00_JPRB ,&
&  0.66536E+00_JPRB, 0.68414E+00_JPRB, 0.70925E+00_JPRB, 0.73405E+00_JPRB, 0.77359E+00_JPRB, 0.76551E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.74860E+00_JPRB, 0.74491E+00_JPRB, 0.75571E+00_JPRB, 0.75280E+00_JPRB, 0.74858E+00_JPRB, 0.74362E+00_JPRB, 0.72817E+00_JPRB ,&
&  0.69606E+00_JPRB, 0.69078E+00_JPRB, 0.71229E+00_JPRB, 0.73341E+00_JPRB, 0.75485E+00_JPRB, 0.76302E+00_JPRB, 0.77278E+00_JPRB ,&
&  0.78939E+00_JPRB, 0.81424E+00_JPRB, 0.94855E+00_JPRB, 0.98999E+00_JPRB, 0.10365E+01_JPRB, 0.10702E+01_JPRB, 0.11115E+01_JPRB ,&
&  0.11573E+01_JPRB, 0.11997E+01_JPRB, 0.12274E+01_JPRB, 0.12812E+01_JPRB, 0.13821E+01_JPRB, 0.15594E+01_JPRB, 0.18795E+01_JPRB ,&
&  0.22959E+01_JPRB, 0.17684E+01_JPRB, 0.21166E+01_JPRB, 0.21314E+01_JPRB, 0.24290E+01_JPRB, 0.26564E+01_JPRB, 0.26749E+01_JPRB ,&
&  0.28708E+01_JPRB, 0.25125E+01_JPRB, 0.22486E+01_JPRB, 0.19806E+01_JPRB, 0.17734E+01_JPRB, 0.17306E+01_JPRB, 0.16910E+01_JPRB ,&
&  0.17185E+01_JPRB, 0.16573E+01_JPRB, 0.14201E+01_JPRB, 0.13001E+01_JPRB, 0.12588E+01_JPRB, 0.12347E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.10631E+01_JPRB, 0.10094E+01_JPRB, 0.94995E+00_JPRB, 0.90744E+00_JPRB, 0.88496E+00_JPRB, 0.85365E+00_JPRB ,&
&  0.84791E+00_JPRB, 0.83376E+00_JPRB, 0.78856E+00_JPRB, 0.79307E+00_JPRB, 0.79402E+00_JPRB, 0.75786E+00_JPRB, 0.76911E+00_JPRB ,&
&  0.79954E+00_JPRB, 0.80089E+00_JPRB, 0.85287E+00_JPRB, 0.92305E+00_JPRB, 0.94449E+00_JPRB, 0.94773E+00_JPRB, 0.10387E+01_JPRB ,&
&  0.96072E+00_JPRB, 0.95664E+00_JPRB, 0.95009E+00_JPRB, 0.90637E+00_JPRB, 0.88226E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.93954E+00_JPRB, 0.92835E+00_JPRB, 0.92134E+00_JPRB, 0.93318E+00_JPRB, 0.94922E+00_JPRB, 0.95544E+00_JPRB, 0.92329E+00_JPRB ,&
&  0.97690E+00_JPRB, 0.94328E+00_JPRB, 0.92496E+00_JPRB, 0.91538E+00_JPRB, 0.89685E+00_JPRB, 0.79908E+00_JPRB, 0.69779E+00_JPRB ,&
&  0.67894E+00_JPRB, 0.69434E+00_JPRB, 0.74299E+00_JPRB, 0.77012E+00_JPRB, 0.80317E+00_JPRB, 0.80066E+00_JPRB, 0.78950E+00_JPRB ,&
&  0.78154E+00_JPRB, 0.76834E+00_JPRB, 0.77444E+00_JPRB, 0.77325E+00_JPRB, 0.76268E+00_JPRB, 0.76388E+00_JPRB, 0.76398E+00_JPRB ,&
&  0.71896E+00_JPRB, 0.69300E+00_JPRB, 0.69425E+00_JPRB, 0.69277E+00_JPRB, 0.75637E+00_JPRB, 0.78361E+00_JPRB, 0.80479E+00_JPRB ,&
&  0.82984E+00_JPRB, 0.86746E+00_JPRB, 0.93040E+00_JPRB, 0.98169E+00_JPRB, 0.10161E+01_JPRB, 0.10410E+01_JPRB, 0.10802E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.11586E+01_JPRB, 0.12176E+01_JPRB, 0.13202E+01_JPRB, 0.14796E+01_JPRB, 0.15776E+01_JPRB, 0.20743E+01_JPRB ,&
&  0.22564E+01_JPRB, 0.25185E+01_JPRB, 0.28225E+01_JPRB, 0.26391E+01_JPRB, 0.28731E+01_JPRB, 0.30112E+01_JPRB, 0.28913E+01_JPRB ,&
&  0.28082E+01_JPRB, 0.24791E+01_JPRB, 0.24011E+01_JPRB, 0.23496E+01_JPRB, 0.23428E+01_JPRB, 0.21840E+01_JPRB, 0.20185E+01_JPRB ,&
&  0.21141E+01_JPRB, 0.23663E+01_JPRB, 0.18823E+01_JPRB, 0.16931E+01_JPRB, 0.15423E+01_JPRB, 0.14966E+01_JPRB, 0.14165E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.13945E+01_JPRB, 0.13373E+01_JPRB, 0.12567E+01_JPRB, 0.11101E+01_JPRB, 0.10446E+01_JPRB, 0.97073E+00_JPRB ,&
&  0.93474E+00_JPRB, 0.88534E+00_JPRB, 0.81680E+00_JPRB, 0.80539E+00_JPRB, 0.78256E+00_JPRB, 0.75791E+00_JPRB, 0.78012E+00_JPRB ,&
&  0.86583E+00_JPRB, 0.94882E+00_JPRB, 0.11331E+01_JPRB, 0.11947E+01_JPRB, 0.12325E+01_JPRB, 0.12740E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.99214E+00_JPRB, 0.10727E+01_JPRB, 0.10369E+01_JPRB, 0.99281E+00_JPRB, 0.95778E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.10447E+01_JPRB, 0.10140E+01_JPRB, 0.98348E+00_JPRB, 0.97705E+00_JPRB, 0.98733E+00_JPRB, 0.98639E+00_JPRB, 0.99342E+00_JPRB ,&
&  0.10147E+01_JPRB, 0.10048E+01_JPRB, 0.97367E+00_JPRB, 0.95635E+00_JPRB, 0.93610E+00_JPRB, 0.93430E+00_JPRB, 0.83117E+00_JPRB ,&
&  0.75583E+00_JPRB, 0.78185E+00_JPRB, 0.76523E+00_JPRB, 0.80579E+00_JPRB, 0.87764E+00_JPRB, 0.85696E+00_JPRB, 0.85242E+00_JPRB ,&
&  0.83457E+00_JPRB, 0.80692E+00_JPRB, 0.80284E+00_JPRB, 0.81011E+00_JPRB, 0.82567E+00_JPRB, 0.87001E+00_JPRB, 0.89481E+00_JPRB ,&
&  0.81717E+00_JPRB, 0.72242E+00_JPRB, 0.68045E+00_JPRB, 0.68796E+00_JPRB, 0.74707E+00_JPRB, 0.80466E+00_JPRB, 0.83839E+00_JPRB ,&
&  0.86409E+00_JPRB, 0.89172E+00_JPRB, 0.93133E+00_JPRB, 0.98486E+00_JPRB, 0.10126E+01_JPRB, 0.10400E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.11159E+01_JPRB, 0.11802E+01_JPRB, 0.12664E+01_JPRB, 0.14055E+01_JPRB, 0.14420E+01_JPRB, 0.16936E+01_JPRB, 0.21430E+01_JPRB ,&
&  0.24008E+01_JPRB, 0.26394E+01_JPRB, 0.29812E+01_JPRB, 0.34406E+01_JPRB, 0.36528E+01_JPRB, 0.34386E+01_JPRB, 0.31645E+01_JPRB ,&
&  0.29854E+01_JPRB, 0.26475E+01_JPRB, 0.29450E+01_JPRB, 0.27278E+01_JPRB, 0.25545E+01_JPRB, 0.25484E+01_JPRB, 0.24252E+01_JPRB ,&
&  0.23900E+01_JPRB, 0.24586E+01_JPRB, 0.23602E+01_JPRB, 0.22835E+01_JPRB, 0.20996E+01_JPRB, 0.20588E+01_JPRB, 0.19161E+01_JPRB ,&
&  0.19328E+01_JPRB, 0.19072E+01_JPRB, 0.16336E+01_JPRB, 0.13906E+01_JPRB, 0.11680E+01_JPRB, 0.11253E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.97480E+00_JPRB, 0.90994E+00_JPRB, 0.89649E+00_JPRB, 0.86704E+00_JPRB, 0.85064E+00_JPRB, 0.88923E+00_JPRB, 0.91227E+00_JPRB ,&
&  0.10195E+01_JPRB, 0.12264E+01_JPRB, 0.13279E+01_JPRB, 0.13853E+01_JPRB, 0.14416E+01_JPRB, 0.14927E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11530E+01_JPRB, 0.11336E+01_JPRB, 0.10987E+01_JPRB, 0.10678E+01_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.11958E+01_JPRB, 0.11394E+01_JPRB, 0.10881E+01_JPRB, 0.10568E+01_JPRB, 0.10385E+01_JPRB, 0.10294E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.10490E+01_JPRB, 0.10237E+01_JPRB, 0.10034E+01_JPRB, 0.98233E+00_JPRB, 0.98540E+00_JPRB, 0.10047E+01_JPRB ,&
&  0.87069E+00_JPRB, 0.91242E+00_JPRB, 0.80096E+00_JPRB, 0.82987E+00_JPRB, 0.10603E+01_JPRB, 0.95167E+00_JPRB, 0.91865E+00_JPRB ,&
&  0.86728E+00_JPRB, 0.87075E+00_JPRB, 0.87046E+00_JPRB, 0.93733E+00_JPRB, 0.99783E+00_JPRB, 0.10775E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.97538E+00_JPRB, 0.82608E+00_JPRB, 0.75705E+00_JPRB, 0.73848E+00_JPRB, 0.77913E+00_JPRB, 0.81844E+00_JPRB, 0.89560E+00_JPRB ,&
&  0.93528E+00_JPRB, 0.96351E+00_JPRB, 0.99097E+00_JPRB, 0.10339E+01_JPRB, 0.10626E+01_JPRB, 0.10997E+01_JPRB, 0.11534E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.13022E+01_JPRB, 0.13889E+01_JPRB, 0.15213E+01_JPRB, 0.15862E+01_JPRB, 0.18148E+01_JPRB, 0.22801E+01_JPRB ,&
&  0.25975E+01_JPRB, 0.28018E+01_JPRB, 0.33469E+01_JPRB, 0.43966E+01_JPRB, 0.48025E+01_JPRB, 0.46525E+01_JPRB, 0.41271E+01_JPRB ,&
&  0.35882E+01_JPRB, 0.30707E+01_JPRB, 0.29805E+01_JPRB, 0.28083E+01_JPRB, 0.26075E+01_JPRB, 0.27044E+01_JPRB, 0.26198E+01_JPRB ,&
&  0.24899E+01_JPRB, 0.24716E+01_JPRB, 0.24347E+01_JPRB, 0.24731E+01_JPRB, 0.23484E+01_JPRB, 0.23552E+01_JPRB, 0.22808E+01_JPRB ,&
&  0.23080E+01_JPRB, 0.21140E+01_JPRB, 0.16001E+01_JPRB, 0.13511E+01_JPRB, 0.11223E+01_JPRB, 0.10614E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.11279E+01_JPRB, 0.11112E+01_JPRB, 0.11402E+01_JPRB, 0.11323E+01_JPRB, 0.11881E+01_JPRB, 0.14128E+01_JPRB, 0.12962E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.13887E+01_JPRB, 0.16278E+01_JPRB, 0.18097E+01_JPRB, 0.18998E+01_JPRB, 0.18397E+01_JPRB, 0.14863E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.13591E+01_JPRB, 0.13209E+01_JPRB, 0.12679E+01_JPRB, 0.12299E+01_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.14327E+01_JPRB, 0.13645E+01_JPRB, 0.12877E+01_JPRB, 0.11990E+01_JPRB, 0.11479E+01_JPRB, 0.11349E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11038E+01_JPRB, 0.10774E+01_JPRB, 0.10577E+01_JPRB, 0.10515E+01_JPRB, 0.10624E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.11198E+01_JPRB, 0.11952E+01_JPRB, 0.99549E+00_JPRB, 0.86077E+00_JPRB, 0.94740E+00_JPRB, 0.90338E+00_JPRB, 0.92701E+00_JPRB ,&
&  0.91464E+00_JPRB, 0.10262E+01_JPRB, 0.10532E+01_JPRB, 0.11620E+01_JPRB, 0.12957E+01_JPRB, 0.14644E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.13516E+01_JPRB, 0.11354E+01_JPRB, 0.97479E+00_JPRB, 0.93161E+00_JPRB, 0.96945E+00_JPRB, 0.91881E+00_JPRB, 0.98321E+00_JPRB ,&
&  0.10138E+01_JPRB, 0.10463E+01_JPRB, 0.10815E+01_JPRB, 0.11094E+01_JPRB, 0.11568E+01_JPRB, 0.12227E+01_JPRB, 0.12979E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.14505E+01_JPRB, 0.14842E+01_JPRB, 0.15417E+01_JPRB, 0.16855E+01_JPRB, 0.18820E+01_JPRB, 0.19925E+01_JPRB ,&
&  0.23349E+01_JPRB, 0.26536E+01_JPRB, 0.32165E+01_JPRB, 0.42265E+01_JPRB, 0.47921E+01_JPRB, 0.49410E+01_JPRB, 0.47456E+01_JPRB ,&
&  0.43331E+01_JPRB, 0.38947E+01_JPRB, 0.37698E+01_JPRB, 0.33952E+01_JPRB, 0.31631E+01_JPRB, 0.31027E+01_JPRB, 0.29982E+01_JPRB ,&
&  0.28081E+01_JPRB, 0.26875E+01_JPRB, 0.26716E+01_JPRB, 0.25836E+01_JPRB, 0.25392E+01_JPRB, 0.24641E+01_JPRB, 0.25028E+01_JPRB ,&
&  0.27603E+01_JPRB, 0.18668E+01_JPRB, 0.14711E+01_JPRB, 0.15369E+01_JPRB, 0.12709E+01_JPRB, 0.11332E+01_JPRB, 0.12026E+01_JPRB ,&
&  0.10916E+01_JPRB, 0.12191E+01_JPRB, 0.14799E+01_JPRB, 0.15142E+01_JPRB, 0.17994E+01_JPRB, 0.21708E+01_JPRB, 0.19508E+01_JPRB ,&
&  0.18968E+01_JPRB, 0.18656E+01_JPRB, 0.22510E+01_JPRB, 0.25507E+01_JPRB, 0.25827E+01_JPRB, 0.23553E+01_JPRB, 0.20379E+01_JPRB ,&
&  0.17897E+01_JPRB, 0.16592E+01_JPRB, 0.15956E+01_JPRB, 0.15406E+01_JPRB, 0.14986E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.17007E+01_JPRB, 0.16137E+01_JPRB, 0.15374E+01_JPRB, 0.14548E+01_JPRB, 0.13930E+01_JPRB, 0.13460E+01_JPRB, 0.12938E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.11695E+01_JPRB, 0.11387E+01_JPRB, 0.11280E+01_JPRB, 0.11283E+01_JPRB, 0.11424E+01_JPRB, 0.11521E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.12665E+01_JPRB, 0.10407E+01_JPRB, 0.89316E+00_JPRB, 0.79224E+00_JPRB, 0.82844E+00_JPRB, 0.88277E+00_JPRB ,&
&  0.99033E+00_JPRB, 0.11039E+01_JPRB, 0.13180E+01_JPRB, 0.16304E+01_JPRB, 0.18519E+01_JPRB, 0.20053E+01_JPRB, 0.20702E+01_JPRB ,&
&  0.19052E+01_JPRB, 0.17383E+01_JPRB, 0.14007E+01_JPRB, 0.13198E+01_JPRB, 0.12166E+01_JPRB, 0.11071E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.11780E+01_JPRB, 0.11997E+01_JPRB, 0.12487E+01_JPRB, 0.12953E+01_JPRB, 0.13355E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.14724E+01_JPRB, 0.15197E+01_JPRB, 0.15672E+01_JPRB, 0.16808E+01_JPRB, 0.18743E+01_JPRB, 0.19786E+01_JPRB ,&
&  0.23157E+01_JPRB, 0.23966E+01_JPRB, 0.25498E+01_JPRB, 0.38033E+01_JPRB, 0.54223E+01_JPRB, 0.54845E+01_JPRB, 0.50382E+01_JPRB ,&
&  0.49514E+01_JPRB, 0.48866E+01_JPRB, 0.49778E+01_JPRB, 0.48207E+01_JPRB, 0.43698E+01_JPRB, 0.41248E+01_JPRB, 0.38531E+01_JPRB ,&
&  0.35159E+01_JPRB, 0.32890E+01_JPRB, 0.31576E+01_JPRB, 0.31479E+01_JPRB, 0.30586E+01_JPRB, 0.30561E+01_JPRB, 0.31021E+01_JPRB ,&
&  0.30585E+01_JPRB, 0.26773E+01_JPRB, 0.18767E+01_JPRB, 0.18136E+01_JPRB, 0.14564E+01_JPRB, 0.13116E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.12558E+01_JPRB, 0.14950E+01_JPRB, 0.18065E+01_JPRB, 0.18778E+01_JPRB, 0.26851E+01_JPRB, 0.30499E+01_JPRB, 0.30057E+01_JPRB ,&
&  0.28557E+01_JPRB, 0.26825E+01_JPRB, 0.31406E+01_JPRB, 0.33153E+01_JPRB, 0.32707E+01_JPRB, 0.30317E+01_JPRB, 0.26826E+01_JPRB ,&
&  0.23178E+01_JPRB, 0.20926E+01_JPRB, 0.19767E+01_JPRB, 0.18950E+01_JPRB, 0.17949E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.19737E+01_JPRB, 0.18614E+01_JPRB, 0.17547E+01_JPRB, 0.16717E+01_JPRB, 0.15992E+01_JPRB, 0.15188E+01_JPRB, 0.14165E+01_JPRB ,&
&  0.13145E+01_JPRB, 0.12643E+01_JPRB, 0.12434E+01_JPRB, 0.12194E+01_JPRB, 0.12035E+01_JPRB, 0.12038E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.12083E+01_JPRB, 0.98249E+00_JPRB, 0.82162E+00_JPRB, 0.72422E+00_JPRB, 0.73586E+00_JPRB, 0.82549E+00_JPRB ,&
&  0.97312E+00_JPRB, 0.13034E+01_JPRB, 0.18176E+01_JPRB, 0.21976E+01_JPRB, 0.25260E+01_JPRB, 0.27911E+01_JPRB, 0.28334E+01_JPRB ,&
&  0.29857E+01_JPRB, 0.23906E+01_JPRB, 0.21641E+01_JPRB, 0.18184E+01_JPRB, 0.16104E+01_JPRB, 0.14984E+01_JPRB, 0.13980E+01_JPRB ,&
&  0.13020E+01_JPRB, 0.12747E+01_JPRB, 0.12681E+01_JPRB, 0.13022E+01_JPRB, 0.13315E+01_JPRB, 0.13819E+01_JPRB, 0.14677E+01_JPRB ,&
&  0.15701E+01_JPRB, 0.16302E+01_JPRB, 0.16106E+01_JPRB, 0.16130E+01_JPRB, 0.18964E+01_JPRB, 0.22586E+01_JPRB, 0.24527E+01_JPRB ,&
&  0.24655E+01_JPRB, 0.27188E+01_JPRB, 0.43885E+01_JPRB, 0.51411E+01_JPRB, 0.55235E+01_JPRB, 0.60968E+01_JPRB, 0.61360E+01_JPRB ,&
&  0.56604E+01_JPRB, 0.51840E+01_JPRB, 0.49548E+01_JPRB, 0.43891E+01_JPRB, 0.48825E+01_JPRB, 0.48545E+01_JPRB, 0.46206E+01_JPRB ,&
&  0.42576E+01_JPRB, 0.40048E+01_JPRB, 0.37574E+01_JPRB, 0.36386E+01_JPRB, 0.36267E+01_JPRB, 0.37050E+01_JPRB, 0.35036E+01_JPRB ,&
&  0.31131E+01_JPRB, 0.33115E+01_JPRB, 0.28238E+01_JPRB, 0.22996E+01_JPRB, 0.19024E+01_JPRB, 0.17828E+01_JPRB, 0.16933E+01_JPRB ,&
&  0.15978E+01_JPRB, 0.17270E+01_JPRB, 0.22502E+01_JPRB, 0.28798E+01_JPRB, 0.37904E+01_JPRB, 0.42946E+01_JPRB, 0.41534E+01_JPRB ,&
&  0.37829E+01_JPRB, 0.40827E+01_JPRB, 0.39651E+01_JPRB, 0.37192E+01_JPRB, 0.35334E+01_JPRB, 0.32391E+01_JPRB, 0.29686E+01_JPRB ,&
&  0.26799E+01_JPRB, 0.24867E+01_JPRB, 0.23510E+01_JPRB, 0.22295E+01_JPRB, 0.21011E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.21119E+01_JPRB, 0.19946E+01_JPRB, 0.18771E+01_JPRB, 0.17715E+01_JPRB, 0.16624E+01_JPRB, 0.15671E+01_JPRB, 0.14659E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.13566E+01_JPRB, 0.13116E+01_JPRB, 0.12714E+01_JPRB, 0.12466E+01_JPRB, 0.12274E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.12326E+01_JPRB, 0.11193E+01_JPRB, 0.92583E+00_JPRB, 0.85705E+00_JPRB, 0.81508E+00_JPRB, 0.76453E+00_JPRB, 0.82462E+00_JPRB ,&
&  0.10548E+01_JPRB, 0.16221E+01_JPRB, 0.22387E+01_JPRB, 0.28190E+01_JPRB, 0.33422E+01_JPRB, 0.37093E+01_JPRB, 0.37090E+01_JPRB ,&
&  0.33032E+01_JPRB, 0.32438E+01_JPRB, 0.28920E+01_JPRB, 0.25452E+01_JPRB, 0.21862E+01_JPRB, 0.19314E+01_JPRB, 0.17386E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.15027E+01_JPRB, 0.14827E+01_JPRB, 0.14960E+01_JPRB, 0.15461E+01_JPRB, 0.16195E+01_JPRB, 0.17158E+01_JPRB ,&
&  0.17584E+01_JPRB, 0.17755E+01_JPRB, 0.17232E+01_JPRB, 0.17745E+01_JPRB, 0.19519E+01_JPRB, 0.25204E+01_JPRB, 0.27183E+01_JPRB ,&
&  0.32925E+01_JPRB, 0.38056E+01_JPRB, 0.40040E+01_JPRB, 0.49667E+01_JPRB, 0.44063E+01_JPRB, 0.53064E+01_JPRB, 0.63290E+01_JPRB ,&
&  0.57723E+01_JPRB, 0.51758E+01_JPRB, 0.50132E+01_JPRB, 0.42057E+01_JPRB, 0.37884E+01_JPRB, 0.52224E+01_JPRB, 0.50241E+01_JPRB ,&
&  0.47112E+01_JPRB, 0.46931E+01_JPRB, 0.43488E+01_JPRB, 0.41486E+01_JPRB, 0.37904E+01_JPRB, 0.34705E+01_JPRB, 0.26708E+01_JPRB ,&
&  0.27644E+01_JPRB, 0.25871E+01_JPRB, 0.27138E+01_JPRB, 0.24219E+01_JPRB, 0.22755E+01_JPRB, 0.20974E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.18519E+01_JPRB, 0.20097E+01_JPRB, 0.34169E+01_JPRB, 0.52511E+01_JPRB, 0.61551E+01_JPRB, 0.52512E+01_JPRB, 0.57169E+01_JPRB ,&
&  0.54176E+01_JPRB, 0.46348E+01_JPRB, 0.37770E+01_JPRB, 0.34385E+01_JPRB, 0.31494E+01_JPRB, 0.29082E+01_JPRB, 0.27285E+01_JPRB ,&
&  0.25671E+01_JPRB, 0.24243E+01_JPRB, 0.23203E+01_JPRB, 0.22652E+01_JPRB, 0.21995E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.21531E+01_JPRB, 0.20365E+01_JPRB, 0.19098E+01_JPRB, 0.18024E+01_JPRB, 0.16985E+01_JPRB, 0.16040E+01_JPRB, 0.15286E+01_JPRB ,&
&  0.14780E+01_JPRB, 0.14190E+01_JPRB, 0.13342E+01_JPRB, 0.12749E+01_JPRB, 0.12462E+01_JPRB, 0.12435E+01_JPRB, 0.12302E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.11538E+01_JPRB, 0.97490E+00_JPRB, 0.89966E+00_JPRB, 0.94551E+00_JPRB, 0.89498E+00_JPRB, 0.91578E+00_JPRB ,&
&  0.13462E+01_JPRB, 0.22260E+01_JPRB, 0.28506E+01_JPRB, 0.33643E+01_JPRB, 0.40519E+01_JPRB, 0.44124E+01_JPRB, 0.42113E+01_JPRB ,&
&  0.41800E+01_JPRB, 0.36544E+01_JPRB, 0.32697E+01_JPRB, 0.29359E+01_JPRB, 0.26020E+01_JPRB, 0.22589E+01_JPRB, 0.19620E+01_JPRB ,&
&  0.17999E+01_JPRB, 0.17028E+01_JPRB, 0.16648E+01_JPRB, 0.16264E+01_JPRB, 0.16800E+01_JPRB, 0.17477E+01_JPRB, 0.18100E+01_JPRB ,&
&  0.18339E+01_JPRB, 0.18357E+01_JPRB, 0.17998E+01_JPRB, 0.18257E+01_JPRB, 0.21747E+01_JPRB, 0.26850E+01_JPRB, 0.30976E+01_JPRB ,&
&  0.33644E+01_JPRB, 0.36486E+01_JPRB, 0.39833E+01_JPRB, 0.45824E+01_JPRB, 0.55759E+01_JPRB, 0.57733E+01_JPRB, 0.63668E+01_JPRB ,&
&  0.50862E+01_JPRB, 0.42509E+01_JPRB, 0.35424E+01_JPRB, 0.28195E+01_JPRB, 0.36943E+01_JPRB, 0.57571E+01_JPRB, 0.58889E+01_JPRB ,&
&  0.55875E+01_JPRB, 0.55787E+01_JPRB, 0.52693E+01_JPRB, 0.43742E+01_JPRB, 0.36914E+01_JPRB, 0.30278E+01_JPRB, 0.35334E+01_JPRB ,&
&  0.35075E+01_JPRB, 0.34229E+01_JPRB, 0.30521E+01_JPRB, 0.25117E+01_JPRB, 0.22146E+01_JPRB, 0.24244E+01_JPRB, 0.22729E+01_JPRB ,&
&  0.22284E+01_JPRB, 0.27970E+01_JPRB, 0.61870E+01_JPRB, 0.79594E+01_JPRB, 0.74881E+01_JPRB, 0.62561E+01_JPRB, 0.67038E+01_JPRB ,&
&  0.53797E+01_JPRB, 0.43373E+01_JPRB, 0.36037E+01_JPRB, 0.30631E+01_JPRB, 0.27695E+01_JPRB, 0.26055E+01_JPRB, 0.25310E+01_JPRB ,&
&  0.24879E+01_JPRB, 0.24563E+01_JPRB, 0.23779E+01_JPRB, 0.22928E+01_JPRB, 0.22217E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.21969E+01_JPRB, 0.20778E+01_JPRB, 0.19712E+01_JPRB, 0.18568E+01_JPRB, 0.17422E+01_JPRB, 0.16585E+01_JPRB, 0.16008E+01_JPRB ,&
&  0.15220E+01_JPRB, 0.14448E+01_JPRB, 0.13711E+01_JPRB, 0.13245E+01_JPRB, 0.13042E+01_JPRB, 0.12913E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.12726E+01_JPRB, 0.10902E+01_JPRB, 0.10136E+01_JPRB, 0.10372E+01_JPRB, 0.10491E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.19495E+01_JPRB, 0.29276E+01_JPRB, 0.31709E+01_JPRB, 0.34689E+01_JPRB, 0.43762E+01_JPRB, 0.40207E+01_JPRB, 0.39539E+01_JPRB ,&
&  0.40035E+01_JPRB, 0.35049E+01_JPRB, 0.31938E+01_JPRB, 0.28979E+01_JPRB, 0.26382E+01_JPRB, 0.24087E+01_JPRB, 0.21822E+01_JPRB ,&
&  0.20002E+01_JPRB, 0.18851E+01_JPRB, 0.18445E+01_JPRB, 0.18721E+01_JPRB, 0.18836E+01_JPRB, 0.18882E+01_JPRB, 0.18853E+01_JPRB ,&
&  0.18661E+01_JPRB, 0.18305E+01_JPRB, 0.17835E+01_JPRB, 0.19466E+01_JPRB, 0.23828E+01_JPRB, 0.28446E+01_JPRB, 0.37925E+01_JPRB ,&
&  0.37534E+01_JPRB, 0.40572E+01_JPRB, 0.40968E+01_JPRB, 0.54015E+01_JPRB, 0.61461E+01_JPRB, 0.61557E+01_JPRB, 0.62712E+01_JPRB ,&
&  0.48289E+01_JPRB, 0.40929E+01_JPRB, 0.37913E+01_JPRB, 0.36532E+01_JPRB, 0.38280E+01_JPRB, 0.45456E+01_JPRB, 0.60093E+01_JPRB ,&
&  0.49552E+01_JPRB, 0.50164E+01_JPRB, 0.50087E+01_JPRB, 0.46215E+01_JPRB, 0.27186E+01_JPRB, 0.21219E+01_JPRB, 0.31903E+01_JPRB ,&
&  0.32763E+01_JPRB, 0.20017E+01_JPRB, 0.14601E+01_JPRB, 0.17956E+01_JPRB, 0.14303E+01_JPRB, 0.17716E+01_JPRB, 0.26678E+01_JPRB ,&
&  0.31623E+01_JPRB, 0.44265E+01_JPRB, 0.79007E+01_JPRB, 0.73922E+01_JPRB, 0.68605E+01_JPRB, 0.70097E+01_JPRB, 0.60362E+01_JPRB ,&
&  0.48474E+01_JPRB, 0.37226E+01_JPRB, 0.34723E+01_JPRB, 0.29915E+01_JPRB, 0.27300E+01_JPRB, 0.25891E+01_JPRB, 0.25950E+01_JPRB ,&
&  0.26831E+01_JPRB, 0.26970E+01_JPRB, 0.26174E+01_JPRB, 0.24644E+01_JPRB, 0.23126E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.22632E+01_JPRB, 0.21543E+01_JPRB, 0.20464E+01_JPRB, 0.19391E+01_JPRB, 0.18403E+01_JPRB, 0.17576E+01_JPRB, 0.16499E+01_JPRB ,&
&  0.15160E+01_JPRB, 0.14345E+01_JPRB, 0.14025E+01_JPRB, 0.13705E+01_JPRB, 0.13509E+01_JPRB, 0.13493E+01_JPRB, 0.13286E+01_JPRB ,&
&  0.12990E+01_JPRB, 0.12962E+01_JPRB, 0.12952E+01_JPRB, 0.12208E+01_JPRB, 0.11882E+01_JPRB, 0.11499E+01_JPRB, 0.15155E+01_JPRB ,&
&  0.27437E+01_JPRB, 0.36347E+01_JPRB, 0.35663E+01_JPRB, 0.34283E+01_JPRB, 0.36363E+01_JPRB, 0.35034E+01_JPRB, 0.38754E+01_JPRB ,&
&  0.36555E+01_JPRB, 0.33088E+01_JPRB, 0.29984E+01_JPRB, 0.27371E+01_JPRB, 0.25330E+01_JPRB, 0.23580E+01_JPRB, 0.22413E+01_JPRB ,&
&  0.21130E+01_JPRB, 0.19928E+01_JPRB, 0.19638E+01_JPRB, 0.18992E+01_JPRB, 0.18566E+01_JPRB, 0.18247E+01_JPRB, 0.18616E+01_JPRB ,&
&  0.18468E+01_JPRB, 0.18241E+01_JPRB, 0.18373E+01_JPRB, 0.20852E+01_JPRB, 0.27517E+01_JPRB, 0.33423E+01_JPRB, 0.34865E+01_JPRB ,&
&  0.35612E+01_JPRB, 0.37836E+01_JPRB, 0.46238E+01_JPRB, 0.53502E+01_JPRB, 0.58716E+01_JPRB, 0.61432E+01_JPRB, 0.57880E+01_JPRB ,&
&  0.54036E+01_JPRB, 0.49678E+01_JPRB, 0.40520E+01_JPRB, 0.39684E+01_JPRB, 0.42014E+01_JPRB, 0.39664E+01_JPRB, 0.46530E+01_JPRB ,&
&  0.48086E+01_JPRB, 0.43211E+01_JPRB, 0.34841E+01_JPRB, 0.28559E+01_JPRB, 0.30708E+01_JPRB, 0.23117E+01_JPRB, 0.15333E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.12294E+01_JPRB, 0.10966E+01_JPRB, 0.11367E+01_JPRB, 0.11488E+01_JPRB, 0.17094E+01_JPRB, 0.29452E+01_JPRB ,&
&  0.45946E+01_JPRB, 0.60486E+01_JPRB, 0.72902E+01_JPRB, 0.64793E+01_JPRB, 0.56871E+01_JPRB, 0.60931E+01_JPRB, 0.57232E+01_JPRB ,&
&  0.38501E+01_JPRB, 0.32529E+01_JPRB, 0.31682E+01_JPRB, 0.28607E+01_JPRB, 0.26587E+01_JPRB, 0.26402E+01_JPRB, 0.27854E+01_JPRB ,&
&  0.29368E+01_JPRB, 0.29358E+01_JPRB, 0.28124E+01_JPRB, 0.25905E+01_JPRB, 0.23951E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.22404E+01_JPRB, 0.21032E+01_JPRB, 0.20113E+01_JPRB, 0.19519E+01_JPRB, 0.18912E+01_JPRB, 0.17862E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.14653E+01_JPRB, 0.14132E+01_JPRB, 0.14043E+01_JPRB, 0.13832E+01_JPRB, 0.13608E+01_JPRB, 0.13509E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.12906E+01_JPRB, 0.12872E+01_JPRB, 0.13632E+01_JPRB, 0.13667E+01_JPRB, 0.13891E+01_JPRB, 0.13764E+01_JPRB, 0.19971E+01_JPRB ,&
&  0.35334E+01_JPRB, 0.41569E+01_JPRB, 0.37613E+01_JPRB, 0.32531E+01_JPRB, 0.34567E+01_JPRB, 0.36446E+01_JPRB, 0.36578E+01_JPRB ,&
&  0.32165E+01_JPRB, 0.28216E+01_JPRB, 0.24220E+01_JPRB, 0.22050E+01_JPRB, 0.21736E+01_JPRB, 0.21550E+01_JPRB, 0.21184E+01_JPRB ,&
&  0.20130E+01_JPRB, 0.19425E+01_JPRB, 0.18646E+01_JPRB, 0.17991E+01_JPRB, 0.17786E+01_JPRB, 0.18034E+01_JPRB, 0.18555E+01_JPRB ,&
&  0.18788E+01_JPRB, 0.18960E+01_JPRB, 0.20226E+01_JPRB, 0.24312E+01_JPRB, 0.28873E+01_JPRB, 0.30271E+01_JPRB, 0.35251E+01_JPRB ,&
&  0.40760E+01_JPRB, 0.45449E+01_JPRB, 0.45049E+01_JPRB, 0.55684E+01_JPRB, 0.57655E+01_JPRB, 0.55737E+01_JPRB, 0.54161E+01_JPRB ,&
&  0.51432E+01_JPRB, 0.52238E+01_JPRB, 0.40481E+01_JPRB, 0.36759E+01_JPRB, 0.44703E+01_JPRB, 0.42543E+01_JPRB, 0.43185E+01_JPRB ,&
&  0.47676E+01_JPRB, 0.41701E+01_JPRB, 0.35740E+01_JPRB, 0.34103E+01_JPRB, 0.48717E+01_JPRB, 0.47046E+01_JPRB, 0.19117E+01_JPRB ,&
&  0.14089E+01_JPRB, 0.12485E+01_JPRB, 0.10734E+01_JPRB, 0.10188E+01_JPRB, 0.11581E+01_JPRB, 0.18577E+01_JPRB, 0.39792E+01_JPRB ,&
&  0.47434E+01_JPRB, 0.67664E+01_JPRB, 0.69389E+01_JPRB, 0.63250E+01_JPRB, 0.52796E+01_JPRB, 0.49291E+01_JPRB, 0.42787E+01_JPRB ,&
&  0.37872E+01_JPRB, 0.32861E+01_JPRB, 0.28002E+01_JPRB, 0.25022E+01_JPRB, 0.24006E+01_JPRB, 0.25013E+01_JPRB, 0.27456E+01_JPRB ,&
&  0.28766E+01_JPRB, 0.28913E+01_JPRB, 0.27889E+01_JPRB, 0.26017E+01_JPRB, 0.23954E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.21285E+01_JPRB, 0.20656E+01_JPRB, 0.20276E+01_JPRB, 0.19542E+01_JPRB, 0.18552E+01_JPRB, 0.17208E+01_JPRB, 0.15501E+01_JPRB ,&
&  0.14636E+01_JPRB, 0.14285E+01_JPRB, 0.14228E+01_JPRB, 0.13853E+01_JPRB, 0.13402E+01_JPRB, 0.13151E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.12801E+01_JPRB, 0.12561E+01_JPRB, 0.12932E+01_JPRB, 0.14262E+01_JPRB, 0.15734E+01_JPRB, 0.18297E+01_JPRB, 0.26055E+01_JPRB ,&
&  0.41636E+01_JPRB, 0.44360E+01_JPRB, 0.38735E+01_JPRB, 0.34529E+01_JPRB, 0.32722E+01_JPRB, 0.33225E+01_JPRB, 0.29260E+01_JPRB ,&
&  0.25139E+01_JPRB, 0.22174E+01_JPRB, 0.19811E+01_JPRB, 0.19740E+01_JPRB, 0.20393E+01_JPRB, 0.20828E+01_JPRB, 0.20415E+01_JPRB ,&
&  0.19965E+01_JPRB, 0.18947E+01_JPRB, 0.18383E+01_JPRB, 0.17885E+01_JPRB, 0.17965E+01_JPRB, 0.18559E+01_JPRB, 0.19166E+01_JPRB ,&
&  0.19763E+01_JPRB, 0.20642E+01_JPRB, 0.23010E+01_JPRB, 0.26889E+01_JPRB, 0.26510E+01_JPRB, 0.32744E+01_JPRB, 0.36788E+01_JPRB ,&
&  0.40381E+01_JPRB, 0.42407E+01_JPRB, 0.42340E+01_JPRB, 0.49209E+01_JPRB, 0.54290E+01_JPRB, 0.53248E+01_JPRB, 0.49460E+01_JPRB ,&
&  0.49720E+01_JPRB, 0.43150E+01_JPRB, 0.37819E+01_JPRB, 0.34844E+01_JPRB, 0.38745E+01_JPRB, 0.51604E+01_JPRB, 0.39669E+01_JPRB ,&
&  0.38603E+01_JPRB, 0.44616E+01_JPRB, 0.41361E+01_JPRB, 0.42457E+01_JPRB, 0.51372E+01_JPRB, 0.62621E+01_JPRB, 0.48694E+01_JPRB ,&
&  0.18156E+01_JPRB, 0.12672E+01_JPRB, 0.13020E+01_JPRB, 0.12613E+01_JPRB, 0.11689E+01_JPRB, 0.19778E+01_JPRB, 0.74519E+01_JPRB ,&
&  0.60698E+01_JPRB, 0.79711E+01_JPRB, 0.74930E+01_JPRB, 0.63600E+01_JPRB, 0.55053E+01_JPRB, 0.44640E+01_JPRB, 0.38645E+01_JPRB ,&
&  0.31217E+01_JPRB, 0.26642E+01_JPRB, 0.24271E+01_JPRB, 0.23071E+01_JPRB, 0.23061E+01_JPRB, 0.23712E+01_JPRB, 0.25433E+01_JPRB ,&
&  0.26549E+01_JPRB, 0.26732E+01_JPRB, 0.25684E+01_JPRB, 0.24042E+01_JPRB, 0.22449E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.20509E+01_JPRB, 0.20207E+01_JPRB, 0.19684E+01_JPRB, 0.18626E+01_JPRB, 0.17555E+01_JPRB, 0.16099E+01_JPRB, 0.14937E+01_JPRB ,&
&  0.14674E+01_JPRB, 0.14397E+01_JPRB, 0.14133E+01_JPRB, 0.13726E+01_JPRB, 0.13216E+01_JPRB, 0.12845E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.12339E+01_JPRB, 0.12413E+01_JPRB, 0.13865E+01_JPRB, 0.18092E+01_JPRB, 0.22683E+01_JPRB, 0.28003E+01_JPRB ,&
&  0.46872E+01_JPRB, 0.46876E+01_JPRB, 0.32885E+01_JPRB, 0.29997E+01_JPRB, 0.29830E+01_JPRB, 0.30492E+01_JPRB, 0.24356E+01_JPRB ,&
&  0.21441E+01_JPRB, 0.19966E+01_JPRB, 0.18623E+01_JPRB, 0.18787E+01_JPRB, 0.19341E+01_JPRB, 0.20014E+01_JPRB, 0.20130E+01_JPRB ,&
&  0.19337E+01_JPRB, 0.18952E+01_JPRB, 0.19011E+01_JPRB, 0.18889E+01_JPRB, 0.18913E+01_JPRB, 0.19595E+01_JPRB, 0.20297E+01_JPRB ,&
&  0.21281E+01_JPRB, 0.22791E+01_JPRB, 0.25589E+01_JPRB, 0.27277E+01_JPRB, 0.29952E+01_JPRB, 0.33184E+01_JPRB, 0.37115E+01_JPRB ,&
&  0.37882E+01_JPRB, 0.39950E+01_JPRB, 0.42623E+01_JPRB, 0.45334E+01_JPRB, 0.48029E+01_JPRB, 0.49324E+01_JPRB, 0.46645E+01_JPRB ,&
&  0.45805E+01_JPRB, 0.39631E+01_JPRB, 0.34457E+01_JPRB, 0.32926E+01_JPRB, 0.36043E+01_JPRB, 0.47571E+01_JPRB, 0.49327E+01_JPRB ,&
&  0.48699E+01_JPRB, 0.38423E+01_JPRB, 0.39531E+01_JPRB, 0.49249E+01_JPRB, 0.50039E+01_JPRB, 0.56113E+01_JPRB, 0.75837E+01_JPRB ,&
&  0.67584E+01_JPRB, 0.34100E+01_JPRB, 0.27367E+01_JPRB, 0.35696E+01_JPRB, 0.22347E+01_JPRB, 0.21408E+01_JPRB, 0.43718E+01_JPRB ,&
&  0.59386E+01_JPRB, 0.72157E+01_JPRB, 0.68875E+01_JPRB, 0.58629E+01_JPRB, 0.52316E+01_JPRB, 0.38989E+01_JPRB, 0.30579E+01_JPRB ,&
&  0.24464E+01_JPRB, 0.22295E+01_JPRB, 0.21247E+01_JPRB, 0.20183E+01_JPRB, 0.19883E+01_JPRB, 0.20858E+01_JPRB, 0.22277E+01_JPRB ,&
&  0.23456E+01_JPRB, 0.23773E+01_JPRB, 0.23367E+01_JPRB, 0.22356E+01_JPRB, 0.21204E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.19489E+01_JPRB, 0.19285E+01_JPRB, 0.18896E+01_JPRB, 0.17784E+01_JPRB, 0.16609E+01_JPRB, 0.15387E+01_JPRB, 0.14974E+01_JPRB ,&
&  0.14612E+01_JPRB, 0.14286E+01_JPRB, 0.13926E+01_JPRB, 0.13374E+01_JPRB, 0.13075E+01_JPRB, 0.12756E+01_JPRB, 0.12796E+01_JPRB ,&
&  0.12930E+01_JPRB, 0.12696E+01_JPRB, 0.13323E+01_JPRB, 0.16757E+01_JPRB, 0.23479E+01_JPRB, 0.37401E+01_JPRB, 0.31966E+01_JPRB ,&
&  0.46493E+01_JPRB, 0.49920E+01_JPRB, 0.32589E+01_JPRB, 0.30003E+01_JPRB, 0.29392E+01_JPRB, 0.29471E+01_JPRB, 0.24301E+01_JPRB ,&
&  0.21229E+01_JPRB, 0.20022E+01_JPRB, 0.19368E+01_JPRB, 0.18821E+01_JPRB, 0.19038E+01_JPRB, 0.19465E+01_JPRB, 0.19002E+01_JPRB ,&
&  0.19061E+01_JPRB, 0.19452E+01_JPRB, 0.19587E+01_JPRB, 0.19589E+01_JPRB, 0.19948E+01_JPRB, 0.20651E+01_JPRB, 0.21537E+01_JPRB ,&
&  0.23031E+01_JPRB, 0.24961E+01_JPRB, 0.27836E+01_JPRB, 0.28456E+01_JPRB, 0.30483E+01_JPRB, 0.34026E+01_JPRB, 0.35953E+01_JPRB ,&
&  0.35172E+01_JPRB, 0.34369E+01_JPRB, 0.38805E+01_JPRB, 0.42754E+01_JPRB, 0.43964E+01_JPRB, 0.43838E+01_JPRB, 0.42412E+01_JPRB ,&
&  0.43448E+01_JPRB, 0.41203E+01_JPRB, 0.40931E+01_JPRB, 0.35709E+01_JPRB, 0.38649E+01_JPRB, 0.46645E+01_JPRB, 0.47166E+01_JPRB ,&
&  0.55034E+01_JPRB, 0.51865E+01_JPRB, 0.53096E+01_JPRB, 0.52433E+01_JPRB, 0.49507E+01_JPRB, 0.51891E+01_JPRB, 0.58186E+01_JPRB ,&
&  0.62411E+01_JPRB, 0.69594E+01_JPRB, 0.66045E+01_JPRB, 0.40154E+01_JPRB, 0.28263E+01_JPRB, 0.27215E+01_JPRB, 0.35149E+01_JPRB ,&
&  0.51999E+01_JPRB, 0.55236E+01_JPRB, 0.51566E+01_JPRB, 0.63030E+01_JPRB, 0.40641E+01_JPRB, 0.28481E+01_JPRB, 0.22177E+01_JPRB ,&
&  0.19358E+01_JPRB, 0.18608E+01_JPRB, 0.17965E+01_JPRB, 0.16858E+01_JPRB, 0.16690E+01_JPRB, 0.17485E+01_JPRB, 0.18728E+01_JPRB ,&
&  0.20050E+01_JPRB, 0.20896E+01_JPRB, 0.21257E+01_JPRB, 0.20678E+01_JPRB, 0.19941E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.17912E+01_JPRB, 0.17902E+01_JPRB, 0.17292E+01_JPRB, 0.16402E+01_JPRB, 0.15652E+01_JPRB, 0.15020E+01_JPRB, 0.14922E+01_JPRB ,&
&  0.14090E+01_JPRB, 0.13673E+01_JPRB, 0.13350E+01_JPRB, 0.13024E+01_JPRB, 0.13133E+01_JPRB, 0.13339E+01_JPRB, 0.13776E+01_JPRB ,&
&  0.14379E+01_JPRB, 0.15409E+01_JPRB, 0.18294E+01_JPRB, 0.24973E+01_JPRB, 0.34395E+01_JPRB, 0.47624E+01_JPRB, 0.48811E+01_JPRB ,&
&  0.45461E+01_JPRB, 0.54228E+01_JPRB, 0.31822E+01_JPRB, 0.31751E+01_JPRB, 0.32324E+01_JPRB, 0.36178E+01_JPRB, 0.30364E+01_JPRB ,&
&  0.23721E+01_JPRB, 0.21949E+01_JPRB, 0.21171E+01_JPRB, 0.19549E+01_JPRB, 0.19022E+01_JPRB, 0.18755E+01_JPRB, 0.18803E+01_JPRB ,&
&  0.19352E+01_JPRB, 0.19906E+01_JPRB, 0.20208E+01_JPRB, 0.20603E+01_JPRB, 0.21286E+01_JPRB, 0.22044E+01_JPRB, 0.23431E+01_JPRB ,&
&  0.25494E+01_JPRB, 0.27719E+01_JPRB, 0.28686E+01_JPRB, 0.29467E+01_JPRB, 0.30461E+01_JPRB, 0.33052E+01_JPRB, 0.34891E+01_JPRB ,&
&  0.34912E+01_JPRB, 0.34207E+01_JPRB, 0.37113E+01_JPRB, 0.37985E+01_JPRB, 0.38059E+01_JPRB, 0.40851E+01_JPRB, 0.39170E+01_JPRB ,&
&  0.41952E+01_JPRB, 0.38978E+01_JPRB, 0.43757E+01_JPRB, 0.38956E+01_JPRB, 0.40032E+01_JPRB, 0.43725E+01_JPRB, 0.39964E+01_JPRB ,&
&  0.41153E+01_JPRB, 0.48341E+01_JPRB, 0.50912E+01_JPRB, 0.51937E+01_JPRB, 0.51739E+01_JPRB, 0.51280E+01_JPRB, 0.49431E+01_JPRB ,&
&  0.43927E+01_JPRB, 0.43392E+01_JPRB, 0.38600E+01_JPRB, 0.25457E+01_JPRB, 0.20922E+01_JPRB, 0.23922E+01_JPRB, 0.28713E+01_JPRB ,&
&  0.39084E+01_JPRB, 0.42581E+01_JPRB, 0.46814E+01_JPRB, 0.40035E+01_JPRB, 0.25414E+01_JPRB, 0.19078E+01_JPRB, 0.17379E+01_JPRB ,&
&  0.16694E+01_JPRB, 0.16055E+01_JPRB, 0.15083E+01_JPRB, 0.14178E+01_JPRB, 0.14252E+01_JPRB, 0.15152E+01_JPRB, 0.16267E+01_JPRB ,&
&  0.17459E+01_JPRB, 0.18217E+01_JPRB, 0.18275E+01_JPRB, 0.17971E+01_JPRB, 0.17941E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.17261E+01_JPRB, 0.17242E+01_JPRB, 0.17114E+01_JPRB, 0.17188E+01_JPRB, 0.17387E+01_JPRB, 0.17776E+01_JPRB, 0.17495E+01_JPRB ,&
&  0.15712E+01_JPRB, 0.13051E+01_JPRB, 0.13183E+01_JPRB, 0.13626E+01_JPRB, 0.14643E+01_JPRB, 0.15652E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.19273E+01_JPRB, 0.22363E+01_JPRB, 0.27667E+01_JPRB, 0.35625E+01_JPRB, 0.45640E+01_JPRB, 0.57872E+01_JPRB, 0.61242E+01_JPRB ,&
&  0.58362E+01_JPRB, 0.40276E+01_JPRB, 0.33723E+01_JPRB, 0.32507E+01_JPRB, 0.30724E+01_JPRB, 0.34274E+01_JPRB, 0.32661E+01_JPRB ,&
&  0.26355E+01_JPRB, 0.23695E+01_JPRB, 0.22764E+01_JPRB, 0.21165E+01_JPRB, 0.19668E+01_JPRB, 0.19849E+01_JPRB, 0.20352E+01_JPRB ,&
&  0.21005E+01_JPRB, 0.21418E+01_JPRB, 0.21599E+01_JPRB, 0.21987E+01_JPRB, 0.22895E+01_JPRB, 0.23831E+01_JPRB, 0.25658E+01_JPRB ,&
&  0.27760E+01_JPRB, 0.29443E+01_JPRB, 0.29970E+01_JPRB, 0.29282E+01_JPRB, 0.30073E+01_JPRB, 0.31615E+01_JPRB, 0.32880E+01_JPRB ,&
&  0.33771E+01_JPRB, 0.35115E+01_JPRB, 0.36043E+01_JPRB, 0.36451E+01_JPRB, 0.34639E+01_JPRB, 0.38346E+01_JPRB, 0.37274E+01_JPRB ,&
&  0.38529E+01_JPRB, 0.37051E+01_JPRB, 0.41911E+01_JPRB, 0.36147E+01_JPRB, 0.39183E+01_JPRB, 0.39329E+01_JPRB, 0.38337E+01_JPRB ,&
&  0.40320E+01_JPRB, 0.47655E+01_JPRB, 0.50432E+01_JPRB, 0.50535E+01_JPRB, 0.51293E+01_JPRB, 0.45574E+01_JPRB, 0.47875E+01_JPRB ,&
&  0.37574E+01_JPRB, 0.37870E+01_JPRB, 0.25615E+01_JPRB, 0.19187E+01_JPRB, 0.15907E+01_JPRB, 0.19702E+01_JPRB, 0.25898E+01_JPRB ,&
&  0.33688E+01_JPRB, 0.33021E+01_JPRB, 0.27439E+01_JPRB, 0.24369E+01_JPRB, 0.19529E+01_JPRB, 0.16412E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.15409E+01_JPRB, 0.14526E+01_JPRB, 0.13712E+01_JPRB, 0.13491E+01_JPRB, 0.13687E+01_JPRB, 0.14763E+01_JPRB, 0.15584E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.16719E+01_JPRB, 0.16759E+01_JPRB, 0.16894E+01_JPRB, 0.17021E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.16701E+01_JPRB, 0.16364E+01_JPRB, 0.16211E+01_JPRB, 0.16026E+01_JPRB, 0.15749E+01_JPRB, 0.14962E+01_JPRB, 0.14354E+01_JPRB ,&
&  0.13710E+01_JPRB, 0.13319E+01_JPRB, 0.14400E+01_JPRB, 0.15920E+01_JPRB, 0.17841E+01_JPRB, 0.19952E+01_JPRB, 0.22487E+01_JPRB ,&
&  0.25747E+01_JPRB, 0.29888E+01_JPRB, 0.35137E+01_JPRB, 0.39915E+01_JPRB, 0.44988E+01_JPRB, 0.51609E+01_JPRB, 0.56126E+01_JPRB ,&
&  0.49648E+01_JPRB, 0.30631E+01_JPRB, 0.32452E+01_JPRB, 0.32606E+01_JPRB, 0.30938E+01_JPRB, 0.30930E+01_JPRB, 0.28589E+01_JPRB ,&
&  0.25260E+01_JPRB, 0.24145E+01_JPRB, 0.23755E+01_JPRB, 0.22954E+01_JPRB, 0.21215E+01_JPRB, 0.21550E+01_JPRB, 0.21978E+01_JPRB ,&
&  0.22153E+01_JPRB, 0.22018E+01_JPRB, 0.22071E+01_JPRB, 0.23116E+01_JPRB, 0.24986E+01_JPRB, 0.25947E+01_JPRB, 0.27484E+01_JPRB ,&
&  0.28797E+01_JPRB, 0.30525E+01_JPRB, 0.31024E+01_JPRB, 0.30528E+01_JPRB, 0.31110E+01_JPRB, 0.31755E+01_JPRB, 0.32351E+01_JPRB ,&
&  0.33108E+01_JPRB, 0.33459E+01_JPRB, 0.34636E+01_JPRB, 0.37430E+01_JPRB, 0.38950E+01_JPRB, 0.35227E+01_JPRB, 0.36173E+01_JPRB ,&
&  0.36528E+01_JPRB, 0.36735E+01_JPRB, 0.36197E+01_JPRB, 0.45012E+01_JPRB, 0.34617E+01_JPRB, 0.35693E+01_JPRB, 0.33631E+01_JPRB ,&
&  0.39621E+01_JPRB, 0.43867E+01_JPRB, 0.47229E+01_JPRB, 0.48272E+01_JPRB, 0.47196E+01_JPRB, 0.43153E+01_JPRB, 0.45960E+01_JPRB ,&
&  0.39803E+01_JPRB, 0.33809E+01_JPRB, 0.23876E+01_JPRB, 0.17561E+01_JPRB, 0.17773E+01_JPRB, 0.30044E+01_JPRB, 0.32049E+01_JPRB ,&
&  0.31424E+01_JPRB, 0.24263E+01_JPRB, 0.21934E+01_JPRB, 0.24460E+01_JPRB, 0.19473E+01_JPRB, 0.15434E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.14424E+01_JPRB, 0.14158E+01_JPRB, 0.13570E+01_JPRB, 0.13558E+01_JPRB, 0.14192E+01_JPRB, 0.15176E+01_JPRB, 0.15884E+01_JPRB ,&
&  0.16213E+01_JPRB, 0.16087E+01_JPRB, 0.16182E+01_JPRB, 0.16353E+01_JPRB, 0.16806E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.15352E+01_JPRB, 0.14732E+01_JPRB, 0.14245E+01_JPRB, 0.13766E+01_JPRB, 0.13280E+01_JPRB, 0.12985E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.14309E+01_JPRB, 0.15628E+01_JPRB, 0.17264E+01_JPRB, 0.19348E+01_JPRB, 0.21897E+01_JPRB, 0.24669E+01_JPRB ,&
&  0.28218E+01_JPRB, 0.31586E+01_JPRB, 0.33398E+01_JPRB, 0.34659E+01_JPRB, 0.36190E+01_JPRB, 0.38491E+01_JPRB, 0.40498E+01_JPRB ,&
&  0.42170E+01_JPRB, 0.40275E+01_JPRB, 0.35217E+01_JPRB, 0.25703E+01_JPRB, 0.22765E+01_JPRB, 0.25658E+01_JPRB, 0.25004E+01_JPRB ,&
&  0.24331E+01_JPRB, 0.23417E+01_JPRB, 0.22757E+01_JPRB, 0.22142E+01_JPRB, 0.21959E+01_JPRB, 0.22289E+01_JPRB, 0.22588E+01_JPRB ,&
&  0.22534E+01_JPRB, 0.22655E+01_JPRB, 0.23262E+01_JPRB, 0.24598E+01_JPRB, 0.26534E+01_JPRB, 0.27648E+01_JPRB, 0.28765E+01_JPRB ,&
&  0.29623E+01_JPRB, 0.31096E+01_JPRB, 0.31645E+01_JPRB, 0.31911E+01_JPRB, 0.31464E+01_JPRB, 0.31795E+01_JPRB, 0.33104E+01_JPRB ,&
&  0.34310E+01_JPRB, 0.35496E+01_JPRB, 0.37208E+01_JPRB, 0.39649E+01_JPRB, 0.40652E+01_JPRB, 0.35961E+01_JPRB, 0.34535E+01_JPRB ,&
&  0.36125E+01_JPRB, 0.37705E+01_JPRB, 0.35601E+01_JPRB, 0.45892E+01_JPRB, 0.30283E+01_JPRB, 0.33880E+01_JPRB, 0.35248E+01_JPRB ,&
&  0.35304E+01_JPRB, 0.38118E+01_JPRB, 0.40995E+01_JPRB, 0.43015E+01_JPRB, 0.40821E+01_JPRB, 0.37428E+01_JPRB, 0.44343E+01_JPRB ,&
&  0.40634E+01_JPRB, 0.29867E+01_JPRB, 0.21951E+01_JPRB, 0.15525E+01_JPRB, 0.19959E+01_JPRB, 0.34670E+01_JPRB, 0.27919E+01_JPRB ,&
&  0.20167E+01_JPRB, 0.20004E+01_JPRB, 0.21967E+01_JPRB, 0.24362E+01_JPRB, 0.19760E+01_JPRB, 0.14040E+01_JPRB, 0.13762E+01_JPRB ,&
&  0.14111E+01_JPRB, 0.14183E+01_JPRB, 0.14421E+01_JPRB, 0.14689E+01_JPRB, 0.15137E+01_JPRB, 0.15728E+01_JPRB, 0.16257E+01_JPRB ,&
&  0.16287E+01_JPRB, 0.16198E+01_JPRB, 0.16481E+01_JPRB, 0.16424E+01_JPRB, 0.16006E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.12728E+01_JPRB, 0.12024E+01_JPRB, 0.11517E+01_JPRB, 0.11358E+01_JPRB, 0.11432E+01_JPRB, 0.11681E+01_JPRB, 0.12165E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.14140E+01_JPRB, 0.15263E+01_JPRB, 0.16414E+01_JPRB, 0.17874E+01_JPRB, 0.19443E+01_JPRB, 0.21268E+01_JPRB ,&
&  0.23239E+01_JPRB, 0.25288E+01_JPRB, 0.26112E+01_JPRB, 0.25992E+01_JPRB, 0.26184E+01_JPRB, 0.26497E+01_JPRB, 0.28428E+01_JPRB ,&
&  0.32553E+01_JPRB, 0.35988E+01_JPRB, 0.37386E+01_JPRB, 0.34594E+01_JPRB, 0.27235E+01_JPRB, 0.27040E+01_JPRB, 0.29475E+01_JPRB ,&
&  0.29604E+01_JPRB, 0.25693E+01_JPRB, 0.22719E+01_JPRB, 0.22459E+01_JPRB, 0.22481E+01_JPRB, 0.22862E+01_JPRB, 0.23151E+01_JPRB ,&
&  0.23215E+01_JPRB, 0.23507E+01_JPRB, 0.24301E+01_JPRB, 0.25206E+01_JPRB, 0.26562E+01_JPRB, 0.27935E+01_JPRB, 0.29359E+01_JPRB ,&
&  0.29996E+01_JPRB, 0.30703E+01_JPRB, 0.30260E+01_JPRB, 0.29154E+01_JPRB, 0.29170E+01_JPRB, 0.30874E+01_JPRB, 0.33646E+01_JPRB ,&
&  0.35882E+01_JPRB, 0.36228E+01_JPRB, 0.38823E+01_JPRB, 0.41154E+01_JPRB, 0.40348E+01_JPRB, 0.37803E+01_JPRB, 0.37345E+01_JPRB ,&
&  0.38170E+01_JPRB, 0.41561E+01_JPRB, 0.31460E+01_JPRB, 0.38363E+01_JPRB, 0.39778E+01_JPRB, 0.35530E+01_JPRB, 0.31789E+01_JPRB ,&
&  0.29742E+01_JPRB, 0.27787E+01_JPRB, 0.24861E+01_JPRB, 0.26731E+01_JPRB, 0.28385E+01_JPRB, 0.31356E+01_JPRB, 0.37438E+01_JPRB ,&
&  0.35414E+01_JPRB, 0.26052E+01_JPRB, 0.18260E+01_JPRB, 0.14625E+01_JPRB, 0.17826E+01_JPRB, 0.25194E+01_JPRB, 0.20077E+01_JPRB ,&
&  0.16833E+01_JPRB, 0.18480E+01_JPRB, 0.19507E+01_JPRB, 0.22629E+01_JPRB, 0.21759E+01_JPRB, 0.13018E+01_JPRB, 0.13151E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.14154E+01_JPRB, 0.14328E+01_JPRB, 0.14511E+01_JPRB, 0.14644E+01_JPRB, 0.14995E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.15351E+01_JPRB, 0.15255E+01_JPRB, 0.14919E+01_JPRB, 0.14315E+01_JPRB, 0.13458E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.99210E+00_JPRB, 0.98277E+00_JPRB, 0.97267E+00_JPRB, 0.99775E+00_JPRB, 0.10255E+01_JPRB, 0.10374E+01_JPRB, 0.10678E+01_JPRB ,&
&  0.11056E+01_JPRB, 0.11463E+01_JPRB, 0.11868E+01_JPRB, 0.12257E+01_JPRB, 0.13392E+01_JPRB, 0.14595E+01_JPRB, 0.15955E+01_JPRB ,&
&  0.16654E+01_JPRB, 0.17189E+01_JPRB, 0.17035E+01_JPRB, 0.16339E+01_JPRB, 0.15682E+01_JPRB, 0.14319E+01_JPRB, 0.13035E+01_JPRB ,&
&  0.13660E+01_JPRB, 0.17569E+01_JPRB, 0.23951E+01_JPRB, 0.30380E+01_JPRB, 0.36437E+01_JPRB, 0.32478E+01_JPRB, 0.35906E+01_JPRB ,&
&  0.30186E+01_JPRB, 0.22615E+01_JPRB, 0.21527E+01_JPRB, 0.21941E+01_JPRB, 0.22795E+01_JPRB, 0.22788E+01_JPRB, 0.23156E+01_JPRB ,&
&  0.23378E+01_JPRB, 0.23662E+01_JPRB, 0.24179E+01_JPRB, 0.25232E+01_JPRB, 0.26058E+01_JPRB, 0.26586E+01_JPRB, 0.27293E+01_JPRB ,&
&  0.27843E+01_JPRB, 0.27853E+01_JPRB, 0.27919E+01_JPRB, 0.25947E+01_JPRB, 0.27430E+01_JPRB, 0.30565E+01_JPRB, 0.33298E+01_JPRB ,&
&  0.34816E+01_JPRB, 0.35596E+01_JPRB, 0.38684E+01_JPRB, 0.39921E+01_JPRB, 0.40390E+01_JPRB, 0.40145E+01_JPRB, 0.40784E+01_JPRB ,&
&  0.40923E+01_JPRB, 0.37544E+01_JPRB, 0.28062E+01_JPRB, 0.31591E+01_JPRB, 0.29351E+01_JPRB, 0.24977E+01_JPRB, 0.26149E+01_JPRB ,&
&  0.23913E+01_JPRB, 0.21179E+01_JPRB, 0.17622E+01_JPRB, 0.17731E+01_JPRB, 0.20150E+01_JPRB, 0.23229E+01_JPRB, 0.30187E+01_JPRB ,&
&  0.25154E+01_JPRB, 0.19463E+01_JPRB, 0.14564E+01_JPRB, 0.13838E+01_JPRB, 0.15140E+01_JPRB, 0.17378E+01_JPRB, 0.17864E+01_JPRB ,&
&  0.17477E+01_JPRB, 0.16663E+01_JPRB, 0.17831E+01_JPRB, 0.20292E+01_JPRB, 0.13875E+01_JPRB, 0.11333E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.12134E+01_JPRB, 0.12175E+01_JPRB, 0.12092E+01_JPRB, 0.12225E+01_JPRB, 0.12505E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.11526E+01_JPRB, 0.10851E+01_JPRB, 0.10393E+01_JPRB, 0.10058E+01_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.75454E+00_JPRB, 0.75839E+00_JPRB, 0.76587E+00_JPRB, 0.78054E+00_JPRB, 0.78690E+00_JPRB, 0.78984E+00_JPRB, 0.77986E+00_JPRB ,&
&  0.77340E+00_JPRB, 0.76187E+00_JPRB, 0.76093E+00_JPRB, 0.76723E+00_JPRB, 0.80296E+00_JPRB, 0.86631E+00_JPRB, 0.94608E+00_JPRB ,&
&  0.96702E+00_JPRB, 0.95955E+00_JPRB, 0.90757E+00_JPRB, 0.86071E+00_JPRB, 0.86620E+00_JPRB, 0.84866E+00_JPRB, 0.80824E+00_JPRB ,&
&  0.78459E+00_JPRB, 0.86873E+00_JPRB, 0.10996E+01_JPRB, 0.14659E+01_JPRB, 0.21261E+01_JPRB, 0.25769E+01_JPRB, 0.34240E+01_JPRB ,&
&  0.33695E+01_JPRB, 0.20178E+01_JPRB, 0.19820E+01_JPRB, 0.18679E+01_JPRB, 0.19650E+01_JPRB, 0.20191E+01_JPRB, 0.20330E+01_JPRB ,&
&  0.20814E+01_JPRB, 0.21287E+01_JPRB, 0.21769E+01_JPRB, 0.22267E+01_JPRB, 0.22528E+01_JPRB, 0.22311E+01_JPRB, 0.22150E+01_JPRB ,&
&  0.21990E+01_JPRB, 0.21540E+01_JPRB, 0.21811E+01_JPRB, 0.22968E+01_JPRB, 0.26114E+01_JPRB, 0.29671E+01_JPRB, 0.30157E+01_JPRB ,&
&  0.30536E+01_JPRB, 0.32553E+01_JPRB, 0.32055E+01_JPRB, 0.34370E+01_JPRB, 0.35262E+01_JPRB, 0.34451E+01_JPRB, 0.36874E+01_JPRB ,&
&  0.38199E+01_JPRB, 0.32434E+01_JPRB, 0.22440E+01_JPRB, 0.24823E+01_JPRB, 0.23982E+01_JPRB, 0.22679E+01_JPRB, 0.23601E+01_JPRB ,&
&  0.21928E+01_JPRB, 0.19961E+01_JPRB, 0.17957E+01_JPRB, 0.16905E+01_JPRB, 0.17291E+01_JPRB, 0.16791E+01_JPRB, 0.17734E+01_JPRB ,&
&  0.16283E+01_JPRB, 0.15816E+01_JPRB, 0.14897E+01_JPRB, 0.14667E+01_JPRB, 0.17043E+01_JPRB, 0.20729E+01_JPRB, 0.17866E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.17546E+01_JPRB, 0.20496E+01_JPRB, 0.22022E+01_JPRB, 0.12113E+01_JPRB, 0.91835E+00_JPRB, 0.86329E+00_JPRB ,&
&  0.87327E+00_JPRB, 0.86635E+00_JPRB, 0.85600E+00_JPRB, 0.83902E+00_JPRB, 0.87536E+00_JPRB, 0.93942E+00_JPRB, 0.91308E+00_JPRB ,&
&  0.89976E+00_JPRB, 0.83143E+00_JPRB, 0.80026E+00_JPRB, 0.78842E+00_JPRB, 0.77897E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.68421E+00_JPRB, 0.67650E+00_JPRB, 0.66932E+00_JPRB, 0.66162E+00_JPRB, 0.67329E+00_JPRB, 0.68078E+00_JPRB, 0.69822E+00_JPRB ,&
&  0.71792E+00_JPRB, 0.72623E+00_JPRB, 0.74070E+00_JPRB, 0.74842E+00_JPRB, 0.77447E+00_JPRB, 0.79690E+00_JPRB, 0.83514E+00_JPRB ,&
&  0.87390E+00_JPRB, 0.89661E+00_JPRB, 0.92098E+00_JPRB, 0.94610E+00_JPRB, 0.95960E+00_JPRB, 0.97663E+00_JPRB, 0.96407E+00_JPRB ,&
&  0.94577E+00_JPRB, 0.94755E+00_JPRB, 0.98799E+00_JPRB, 0.11534E+01_JPRB, 0.14671E+01_JPRB, 0.18712E+01_JPRB, 0.26956E+01_JPRB ,&
&  0.28496E+01_JPRB, 0.18408E+01_JPRB, 0.15338E+01_JPRB, 0.13503E+01_JPRB, 0.13946E+01_JPRB, 0.14724E+01_JPRB, 0.14500E+01_JPRB ,&
&  0.14436E+01_JPRB, 0.14360E+01_JPRB, 0.14188E+01_JPRB, 0.13827E+01_JPRB, 0.13011E+01_JPRB, 0.12045E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.10901E+01_JPRB, 0.12328E+01_JPRB, 0.16067E+01_JPRB, 0.19100E+01_JPRB, 0.24064E+01_JPRB, 0.22778E+01_JPRB ,&
&  0.23027E+01_JPRB, 0.23864E+01_JPRB, 0.23616E+01_JPRB, 0.25374E+01_JPRB, 0.26208E+01_JPRB, 0.27232E+01_JPRB, 0.27441E+01_JPRB ,&
&  0.29171E+01_JPRB, 0.26065E+01_JPRB, 0.19320E+01_JPRB, 0.22107E+01_JPRB, 0.21827E+01_JPRB, 0.22339E+01_JPRB, 0.21482E+01_JPRB ,&
&  0.20230E+01_JPRB, 0.18743E+01_JPRB, 0.17507E+01_JPRB, 0.16639E+01_JPRB, 0.16725E+01_JPRB, 0.16405E+01_JPRB, 0.15144E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.15218E+01_JPRB, 0.14999E+01_JPRB, 0.14706E+01_JPRB, 0.18217E+01_JPRB, 0.20274E+01_JPRB, 0.25871E+01_JPRB ,&
&  0.17562E+01_JPRB, 0.16050E+01_JPRB, 0.12411E+01_JPRB, 0.15662E+01_JPRB, 0.95415E+00_JPRB, 0.72097E+00_JPRB, 0.70958E+00_JPRB ,&
&  0.68886E+00_JPRB, 0.66419E+00_JPRB, 0.64332E+00_JPRB, 0.63786E+00_JPRB, 0.65113E+00_JPRB, 0.68818E+00_JPRB, 0.68122E+00_JPRB ,&
&  0.68177E+00_JPRB, 0.68282E+00_JPRB, 0.69336E+00_JPRB, 0.71094E+00_JPRB, 0.70266E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.71809E+00_JPRB, 0.72259E+00_JPRB, 0.73660E+00_JPRB, 0.75922E+00_JPRB, 0.77395E+00_JPRB, 0.78742E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.82866E+00_JPRB, 0.86049E+00_JPRB, 0.88933E+00_JPRB, 0.91730E+00_JPRB, 0.94575E+00_JPRB, 0.96885E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.10471E+01_JPRB, 0.10869E+01_JPRB, 0.11127E+01_JPRB, 0.11425E+01_JPRB, 0.11693E+01_JPRB, 0.11601E+01_JPRB, 0.11734E+01_JPRB ,&
&  0.11909E+01_JPRB, 0.12359E+01_JPRB, 0.12760E+01_JPRB, 0.13522E+01_JPRB, 0.14513E+01_JPRB, 0.14991E+01_JPRB, 0.17885E+01_JPRB ,&
&  0.18100E+01_JPRB, 0.17940E+01_JPRB, 0.14812E+01_JPRB, 0.12638E+01_JPRB, 0.11979E+01_JPRB, 0.10705E+01_JPRB, 0.99474E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.97443E+00_JPRB, 0.94131E+00_JPRB, 0.89046E+00_JPRB, 0.83135E+00_JPRB, 0.86892E+00_JPRB, 0.93438E+00_JPRB ,&
&  0.10926E+01_JPRB, 0.11833E+01_JPRB, 0.13178E+01_JPRB, 0.14444E+01_JPRB, 0.15942E+01_JPRB, 0.16766E+01_JPRB, 0.16498E+01_JPRB ,&
&  0.16521E+01_JPRB, 0.16340E+01_JPRB, 0.15672E+01_JPRB, 0.17952E+01_JPRB, 0.19702E+01_JPRB, 0.20583E+01_JPRB, 0.20929E+01_JPRB ,&
&  0.21066E+01_JPRB, 0.19515E+01_JPRB, 0.19466E+01_JPRB, 0.19784E+01_JPRB, 0.19389E+01_JPRB, 0.19851E+01_JPRB, 0.19025E+01_JPRB ,&
&  0.17822E+01_JPRB, 0.16936E+01_JPRB, 0.16668E+01_JPRB, 0.16791E+01_JPRB, 0.17277E+01_JPRB, 0.17466E+01_JPRB, 0.16306E+01_JPRB ,&
&  0.15442E+01_JPRB, 0.14989E+01_JPRB, 0.14678E+01_JPRB, 0.14696E+01_JPRB, 0.16429E+01_JPRB, 0.34559E+01_JPRB, 0.44367E+01_JPRB ,&
&  0.16797E+01_JPRB, 0.11846E+01_JPRB, 0.70650E+00_JPRB, 0.86191E+00_JPRB, 0.72638E+00_JPRB, 0.68898E+00_JPRB, 0.75761E+00_JPRB ,&
&  0.76888E+00_JPRB, 0.75190E+00_JPRB, 0.68325E+00_JPRB, 0.61816E+00_JPRB, 0.64047E+00_JPRB, 0.65408E+00_JPRB, 0.65330E+00_JPRB ,&
&  0.67012E+00_JPRB, 0.67650E+00_JPRB, 0.68839E+00_JPRB, 0.71222E+00_JPRB, 0.70923E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.76089E+00_JPRB, 0.78069E+00_JPRB, 0.80568E+00_JPRB, 0.83832E+00_JPRB, 0.87040E+00_JPRB, 0.89689E+00_JPRB, 0.91673E+00_JPRB ,&
&  0.94714E+00_JPRB, 0.98751E+00_JPRB, 0.10149E+01_JPRB, 0.10299E+01_JPRB, 0.10599E+01_JPRB, 0.10856E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.11593E+01_JPRB, 0.12219E+01_JPRB, 0.12753E+01_JPRB, 0.13254E+01_JPRB, 0.13682E+01_JPRB, 0.14148E+01_JPRB, 0.14589E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.15656E+01_JPRB, 0.16193E+01_JPRB, 0.16492E+01_JPRB, 0.17190E+01_JPRB, 0.18656E+01_JPRB, 0.16387E+01_JPRB ,&
&  0.19495E+01_JPRB, 0.16828E+01_JPRB, 0.14416E+01_JPRB, 0.12843E+01_JPRB, 0.11786E+01_JPRB, 0.11092E+01_JPRB, 0.10723E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.11303E+01_JPRB, 0.11677E+01_JPRB, 0.11300E+01_JPRB, 0.10462E+01_JPRB, 0.11378E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.15311E+01_JPRB, 0.15830E+01_JPRB, 0.15945E+01_JPRB, 0.16586E+01_JPRB, 0.17488E+01_JPRB, 0.17872E+01_JPRB ,&
&  0.17816E+01_JPRB, 0.17248E+01_JPRB, 0.15584E+01_JPRB, 0.18593E+01_JPRB, 0.18694E+01_JPRB, 0.18098E+01_JPRB, 0.16748E+01_JPRB ,&
&  0.15403E+01_JPRB, 0.14404E+01_JPRB, 0.15386E+01_JPRB, 0.17165E+01_JPRB, 0.16941E+01_JPRB, 0.16728E+01_JPRB, 0.15725E+01_JPRB ,&
&  0.14702E+01_JPRB, 0.14506E+01_JPRB, 0.14853E+01_JPRB, 0.15620E+01_JPRB, 0.16304E+01_JPRB, 0.16399E+01_JPRB, 0.15391E+01_JPRB ,&
&  0.14378E+01_JPRB, 0.13399E+01_JPRB, 0.12859E+01_JPRB, 0.12781E+01_JPRB, 0.14296E+01_JPRB, 0.17773E+01_JPRB, 0.26263E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.80534E+00_JPRB, 0.80093E+00_JPRB, 0.72605E+00_JPRB, 0.73532E+00_JPRB, 0.69781E+00_JPRB, 0.85048E+00_JPRB ,&
&  0.10589E+01_JPRB, 0.12052E+01_JPRB, 0.11789E+01_JPRB, 0.91475E+00_JPRB, 0.75398E+00_JPRB, 0.70790E+00_JPRB, 0.70139E+00_JPRB ,&
&  0.71420E+00_JPRB, 0.71536E+00_JPRB, 0.72180E+00_JPRB, 0.73197E+00_JPRB, 0.74659E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.73933E+00_JPRB, 0.75407E+00_JPRB, 0.77251E+00_JPRB, 0.79236E+00_JPRB, 0.81649E+00_JPRB, 0.84151E+00_JPRB, 0.86039E+00_JPRB ,&
&  0.88251E+00_JPRB, 0.90947E+00_JPRB, 0.93546E+00_JPRB, 0.95485E+00_JPRB, 0.97737E+00_JPRB, 0.10158E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.11615E+01_JPRB, 0.12095E+01_JPRB, 0.12859E+01_JPRB, 0.13622E+01_JPRB, 0.14477E+01_JPRB, 0.15142E+01_JPRB ,&
&  0.15860E+01_JPRB, 0.16939E+01_JPRB, 0.17504E+01_JPRB, 0.17510E+01_JPRB, 0.18023E+01_JPRB, 0.20310E+01_JPRB, 0.17299E+01_JPRB ,&
&  0.19083E+01_JPRB, 0.17428E+01_JPRB, 0.15425E+01_JPRB, 0.14536E+01_JPRB, 0.13853E+01_JPRB, 0.13718E+01_JPRB, 0.13940E+01_JPRB ,&
&  0.14543E+01_JPRB, 0.15653E+01_JPRB, 0.14794E+01_JPRB, 0.14787E+01_JPRB, 0.14772E+01_JPRB, 0.16305E+01_JPRB, 0.17992E+01_JPRB ,&
&  0.18523E+01_JPRB, 0.19703E+01_JPRB, 0.20223E+01_JPRB, 0.20772E+01_JPRB, 0.21750E+01_JPRB, 0.22734E+01_JPRB, 0.23389E+01_JPRB ,&
&  0.23119E+01_JPRB, 0.22670E+01_JPRB, 0.21081E+01_JPRB, 0.22404E+01_JPRB, 0.21410E+01_JPRB, 0.18113E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.12442E+01_JPRB, 0.13351E+01_JPRB, 0.14593E+01_JPRB, 0.13952E+01_JPRB, 0.13315E+01_JPRB, 0.11706E+01_JPRB ,&
&  0.11107E+01_JPRB, 0.11089E+01_JPRB, 0.11782E+01_JPRB, 0.12774E+01_JPRB, 0.13394E+01_JPRB, 0.13341E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.10877E+01_JPRB, 0.10630E+01_JPRB, 0.10955E+01_JPRB, 0.11417E+01_JPRB, 0.11586E+01_JPRB, 0.15807E+01_JPRB ,&
&  0.11190E+01_JPRB, 0.97599E+00_JPRB, 0.11237E+01_JPRB, 0.54007E+00_JPRB, 0.70497E+00_JPRB, 0.88326E+00_JPRB, 0.11531E+01_JPRB ,&
&  0.13513E+01_JPRB, 0.18693E+01_JPRB, 0.22970E+01_JPRB, 0.17501E+01_JPRB, 0.12368E+01_JPRB, 0.75733E+00_JPRB, 0.66609E+00_JPRB ,&
&  0.67996E+00_JPRB, 0.68741E+00_JPRB, 0.69940E+00_JPRB, 0.70314E+00_JPRB, 0.72052E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.67331E+00_JPRB, 0.66995E+00_JPRB, 0.67162E+00_JPRB, 0.66312E+00_JPRB, 0.66000E+00_JPRB, 0.66197E+00_JPRB, 0.65218E+00_JPRB ,&
&  0.66037E+00_JPRB, 0.68042E+00_JPRB, 0.68944E+00_JPRB, 0.71651E+00_JPRB, 0.73674E+00_JPRB, 0.79386E+00_JPRB, 0.87942E+00_JPRB ,&
&  0.95120E+00_JPRB, 0.10906E+01_JPRB, 0.11821E+01_JPRB, 0.12860E+01_JPRB, 0.13790E+01_JPRB, 0.14797E+01_JPRB, 0.15658E+01_JPRB ,&
&  0.16782E+01_JPRB, 0.18210E+01_JPRB, 0.19667E+01_JPRB, 0.20752E+01_JPRB, 0.21170E+01_JPRB, 0.24421E+01_JPRB, 0.18939E+01_JPRB ,&
&  0.19858E+01_JPRB, 0.19303E+01_JPRB, 0.17908E+01_JPRB, 0.17085E+01_JPRB, 0.16249E+01_JPRB, 0.15734E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.15551E+01_JPRB, 0.15639E+01_JPRB, 0.15548E+01_JPRB, 0.16643E+01_JPRB, 0.17938E+01_JPRB, 0.18886E+01_JPRB, 0.20226E+01_JPRB ,&
&  0.21413E+01_JPRB, 0.22539E+01_JPRB, 0.24011E+01_JPRB, 0.25575E+01_JPRB, 0.27217E+01_JPRB, 0.28279E+01_JPRB, 0.29472E+01_JPRB ,&
&  0.30010E+01_JPRB, 0.30414E+01_JPRB, 0.28565E+01_JPRB, 0.25312E+01_JPRB, 0.21847E+01_JPRB, 0.18074E+01_JPRB, 0.14350E+01_JPRB ,&
&  0.11591E+01_JPRB, 0.11146E+01_JPRB, 0.11310E+01_JPRB, 0.11597E+01_JPRB, 0.10547E+01_JPRB, 0.98103E+00_JPRB, 0.93709E+00_JPRB ,&
&  0.89111E+00_JPRB, 0.83021E+00_JPRB, 0.87278E+00_JPRB, 0.94658E+00_JPRB, 0.97739E+00_JPRB, 0.96938E+00_JPRB, 0.94972E+00_JPRB ,&
&  0.91482E+00_JPRB, 0.87106E+00_JPRB, 0.89258E+00_JPRB, 0.93936E+00_JPRB, 0.10332E+01_JPRB, 0.11832E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.11447E+01_JPRB, 0.80274E+00_JPRB, 0.76437E+00_JPRB, 0.75804E+00_JPRB, 0.74057E+00_JPRB, 0.94054E+00_JPRB, 0.12853E+01_JPRB ,&
&  0.19031E+01_JPRB, 0.21211E+01_JPRB, 0.23384E+01_JPRB, 0.28793E+01_JPRB, 0.25051E+01_JPRB, 0.18730E+01_JPRB, 0.60612E+00_JPRB ,&
&  0.60754E+00_JPRB, 0.61443E+00_JPRB, 0.64442E+00_JPRB, 0.65540E+00_JPRB, 0.66176E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.56749E+00_JPRB, 0.55039E+00_JPRB, 0.54242E+00_JPRB, 0.53858E+00_JPRB, 0.52301E+00_JPRB, 0.52521E+00_JPRB, 0.52425E+00_JPRB ,&
&  0.54446E+00_JPRB, 0.57305E+00_JPRB, 0.62225E+00_JPRB, 0.68516E+00_JPRB, 0.77190E+00_JPRB, 0.84722E+00_JPRB, 0.94318E+00_JPRB ,&
&  0.10142E+01_JPRB, 0.11027E+01_JPRB, 0.11923E+01_JPRB, 0.12829E+01_JPRB, 0.13842E+01_JPRB, 0.14741E+01_JPRB, 0.15960E+01_JPRB ,&
&  0.17085E+01_JPRB, 0.18418E+01_JPRB, 0.20072E+01_JPRB, 0.21530E+01_JPRB, 0.22664E+01_JPRB, 0.23477E+01_JPRB, 0.18017E+01_JPRB ,&
&  0.20763E+01_JPRB, 0.21073E+01_JPRB, 0.20964E+01_JPRB, 0.19969E+01_JPRB, 0.18486E+01_JPRB, 0.17135E+01_JPRB, 0.16715E+01_JPRB ,&
&  0.16541E+01_JPRB, 0.16226E+01_JPRB, 0.15816E+01_JPRB, 0.15886E+01_JPRB, 0.18627E+01_JPRB, 0.19497E+01_JPRB, 0.20755E+01_JPRB ,&
&  0.22087E+01_JPRB, 0.23584E+01_JPRB, 0.25118E+01_JPRB, 0.26735E+01_JPRB, 0.27885E+01_JPRB, 0.29189E+01_JPRB, 0.30575E+01_JPRB ,&
&  0.31554E+01_JPRB, 0.32382E+01_JPRB, 0.31154E+01_JPRB, 0.24819E+01_JPRB, 0.20988E+01_JPRB, 0.17759E+01_JPRB, 0.12941E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.10036E+01_JPRB, 0.10205E+01_JPRB, 0.10147E+01_JPRB, 0.92730E+00_JPRB, 0.85069E+00_JPRB, 0.79414E+00_JPRB ,&
&  0.70917E+00_JPRB, 0.68854E+00_JPRB, 0.68143E+00_JPRB, 0.73153E+00_JPRB, 0.75809E+00_JPRB, 0.72530E+00_JPRB, 0.71414E+00_JPRB ,&
&  0.68860E+00_JPRB, 0.66990E+00_JPRB, 0.72813E+00_JPRB, 0.82773E+00_JPRB, 0.10411E+01_JPRB, 0.12783E+01_JPRB, 0.14494E+01_JPRB ,&
&  0.14504E+01_JPRB, 0.81415E+00_JPRB, 0.73565E+00_JPRB, 0.70881E+00_JPRB, 0.71785E+00_JPRB, 0.76942E+00_JPRB, 0.86337E+00_JPRB ,&
&  0.10788E+01_JPRB, 0.14584E+01_JPRB, 0.12721E+01_JPRB, 0.97567E+00_JPRB, 0.10209E+01_JPRB, 0.94434E+00_JPRB, 0.55149E+00_JPRB ,&
&  0.55498E+00_JPRB, 0.55189E+00_JPRB, 0.56977E+00_JPRB, 0.57613E+00_JPRB, 0.57995E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.50048E+00_JPRB, 0.48018E+00_JPRB, 0.45976E+00_JPRB, 0.45082E+00_JPRB, 0.44438E+00_JPRB, 0.43753E+00_JPRB, 0.45221E+00_JPRB ,&
&  0.48833E+00_JPRB, 0.52343E+00_JPRB, 0.58245E+00_JPRB, 0.67307E+00_JPRB, 0.76979E+00_JPRB, 0.83864E+00_JPRB, 0.91124E+00_JPRB ,&
&  0.97782E+00_JPRB, 0.10469E+01_JPRB, 0.11046E+01_JPRB, 0.11633E+01_JPRB, 0.12410E+01_JPRB, 0.13417E+01_JPRB, 0.14803E+01_JPRB ,&
&  0.16438E+01_JPRB, 0.18045E+01_JPRB, 0.20004E+01_JPRB, 0.22327E+01_JPRB, 0.24567E+01_JPRB, 0.25493E+01_JPRB, 0.27261E+01_JPRB ,&
&  0.18181E+01_JPRB, 0.22528E+01_JPRB, 0.22356E+01_JPRB, 0.21502E+01_JPRB, 0.20410E+01_JPRB, 0.19387E+01_JPRB, 0.18680E+01_JPRB ,&
&  0.18140E+01_JPRB, 0.16553E+01_JPRB, 0.15952E+01_JPRB, 0.15624E+01_JPRB, 0.17105E+01_JPRB, 0.18298E+01_JPRB, 0.19526E+01_JPRB ,&
&  0.20769E+01_JPRB, 0.22216E+01_JPRB, 0.23737E+01_JPRB, 0.25225E+01_JPRB, 0.26437E+01_JPRB, 0.27849E+01_JPRB, 0.28870E+01_JPRB ,&
&  0.30094E+01_JPRB, 0.31094E+01_JPRB, 0.31431E+01_JPRB, 0.26000E+01_JPRB, 0.21264E+01_JPRB, 0.18592E+01_JPRB, 0.14293E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.92248E+00_JPRB, 0.94796E+00_JPRB, 0.97505E+00_JPRB, 0.91449E+00_JPRB, 0.85260E+00_JPRB, 0.77451E+00_JPRB ,&
&  0.71436E+00_JPRB, 0.63983E+00_JPRB, 0.60838E+00_JPRB, 0.62372E+00_JPRB, 0.66740E+00_JPRB, 0.63411E+00_JPRB, 0.59407E+00_JPRB ,&
&  0.56921E+00_JPRB, 0.56357E+00_JPRB, 0.62959E+00_JPRB, 0.70979E+00_JPRB, 0.81830E+00_JPRB, 0.87976E+00_JPRB, 0.91861E+00_JPRB ,&
&  0.96483E+00_JPRB, 0.78539E+00_JPRB, 0.74986E+00_JPRB, 0.74968E+00_JPRB, 0.73816E+00_JPRB, 0.73945E+00_JPRB, 0.75473E+00_JPRB ,&
&  0.78974E+00_JPRB, 0.85700E+00_JPRB, 0.87267E+00_JPRB, 0.81166E+00_JPRB, 0.73008E+00_JPRB, 0.67558E+00_JPRB, 0.62799E+00_JPRB ,&
&  0.63004E+00_JPRB, 0.61615E+00_JPRB, 0.60080E+00_JPRB, 0.57301E+00_JPRB, 0.53853E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.51392E+00_JPRB, 0.47935E+00_JPRB, 0.43417E+00_JPRB, 0.39700E+00_JPRB, 0.42374E+00_JPRB, 0.42814E+00_JPRB, 0.44916E+00_JPRB ,&
&  0.46042E+00_JPRB, 0.49436E+00_JPRB, 0.53276E+00_JPRB, 0.58913E+00_JPRB, 0.66752E+00_JPRB, 0.74871E+00_JPRB, 0.81971E+00_JPRB ,&
&  0.88477E+00_JPRB, 0.94625E+00_JPRB, 0.99237E+00_JPRB, 0.10156E+01_JPRB, 0.10538E+01_JPRB, 0.11163E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.13129E+01_JPRB, 0.14615E+01_JPRB, 0.16592E+01_JPRB, 0.19077E+01_JPRB, 0.21924E+01_JPRB, 0.23591E+01_JPRB, 0.24877E+01_JPRB ,&
&  0.13270E+01_JPRB, 0.20471E+01_JPRB, 0.24244E+01_JPRB, 0.22425E+01_JPRB, 0.21656E+01_JPRB, 0.20928E+01_JPRB, 0.20165E+01_JPRB ,&
&  0.18491E+01_JPRB, 0.16331E+01_JPRB, 0.15328E+01_JPRB, 0.16462E+01_JPRB, 0.15360E+01_JPRB, 0.16535E+01_JPRB, 0.17410E+01_JPRB ,&
&  0.18327E+01_JPRB, 0.19293E+01_JPRB, 0.20442E+01_JPRB, 0.21959E+01_JPRB, 0.23553E+01_JPRB, 0.24917E+01_JPRB, 0.26105E+01_JPRB ,&
&  0.27683E+01_JPRB, 0.29062E+01_JPRB, 0.29453E+01_JPRB, 0.23135E+01_JPRB, 0.21196E+01_JPRB, 0.19977E+01_JPRB, 0.18003E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.94072E+00_JPRB, 0.94823E+00_JPRB, 0.98582E+00_JPRB, 0.94219E+00_JPRB, 0.89430E+00_JPRB, 0.83781E+00_JPRB ,&
&  0.75885E+00_JPRB, 0.69495E+00_JPRB, 0.64621E+00_JPRB, 0.62521E+00_JPRB, 0.64595E+00_JPRB, 0.64412E+00_JPRB, 0.61859E+00_JPRB ,&
&  0.59033E+00_JPRB, 0.59604E+00_JPRB, 0.64445E+00_JPRB, 0.68706E+00_JPRB, 0.74988E+00_JPRB, 0.78626E+00_JPRB, 0.83313E+00_JPRB ,&
&  0.84527E+00_JPRB, 0.77255E+00_JPRB, 0.76656E+00_JPRB, 0.77869E+00_JPRB, 0.78215E+00_JPRB, 0.78416E+00_JPRB, 0.78053E+00_JPRB ,&
&  0.77405E+00_JPRB, 0.79151E+00_JPRB, 0.79851E+00_JPRB, 0.80360E+00_JPRB, 0.77991E+00_JPRB, 0.75254E+00_JPRB, 0.73488E+00_JPRB ,&
&  0.70998E+00_JPRB, 0.67829E+00_JPRB, 0.65053E+00_JPRB, 0.60335E+00_JPRB, 0.55598E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.54069E+00_JPRB, 0.49993E+00_JPRB, 0.45534E+00_JPRB, 0.43897E+00_JPRB, 0.46783E+00_JPRB, 0.49439E+00_JPRB, 0.49272E+00_JPRB ,&
&  0.48874E+00_JPRB, 0.48036E+00_JPRB, 0.47999E+00_JPRB, 0.49817E+00_JPRB, 0.55246E+00_JPRB, 0.63453E+00_JPRB, 0.70477E+00_JPRB ,&
&  0.76638E+00_JPRB, 0.81257E+00_JPRB, 0.84802E+00_JPRB, 0.85886E+00_JPRB, 0.84986E+00_JPRB, 0.87049E+00_JPRB, 0.91097E+00_JPRB ,&
&  0.98386E+00_JPRB, 0.10838E+01_JPRB, 0.12189E+01_JPRB, 0.14230E+01_JPRB, 0.17259E+01_JPRB, 0.20971E+01_JPRB, 0.24782E+01_JPRB ,&
&  0.17392E+01_JPRB, 0.75666E+00_JPRB, 0.22359E+01_JPRB, 0.26293E+01_JPRB, 0.24592E+01_JPRB, 0.24254E+01_JPRB, 0.21939E+01_JPRB ,&
&  0.19787E+01_JPRB, 0.19072E+01_JPRB, 0.17159E+01_JPRB, 0.17214E+01_JPRB, 0.15374E+01_JPRB, 0.15279E+01_JPRB, 0.15306E+01_JPRB ,&
&  0.15575E+01_JPRB, 0.16018E+01_JPRB, 0.16768E+01_JPRB, 0.17728E+01_JPRB, 0.18900E+01_JPRB, 0.20840E+01_JPRB, 0.22244E+01_JPRB ,&
&  0.23554E+01_JPRB, 0.24710E+01_JPRB, 0.24950E+01_JPRB, 0.18714E+01_JPRB, 0.15808E+01_JPRB, 0.14350E+01_JPRB, 0.13223E+01_JPRB ,&
&  0.11807E+01_JPRB, 0.10063E+01_JPRB, 0.98241E+00_JPRB, 0.10132E+01_JPRB, 0.98688E+00_JPRB, 0.92010E+00_JPRB, 0.91511E+00_JPRB ,&
&  0.82973E+00_JPRB, 0.76424E+00_JPRB, 0.71011E+00_JPRB, 0.66773E+00_JPRB, 0.66975E+00_JPRB, 0.67586E+00_JPRB, 0.67983E+00_JPRB ,&
&  0.66470E+00_JPRB, 0.66331E+00_JPRB, 0.68532E+00_JPRB, 0.69067E+00_JPRB, 0.70005E+00_JPRB, 0.71133E+00_JPRB, 0.72444E+00_JPRB ,&
&  0.77155E+00_JPRB, 0.76978E+00_JPRB, 0.78038E+00_JPRB, 0.80280E+00_JPRB, 0.81498E+00_JPRB, 0.79939E+00_JPRB, 0.80280E+00_JPRB ,&
&  0.81304E+00_JPRB, 0.82482E+00_JPRB, 0.81509E+00_JPRB, 0.82625E+00_JPRB, 0.81243E+00_JPRB, 0.80163E+00_JPRB, 0.80079E+00_JPRB ,&
&  0.80192E+00_JPRB, 0.78655E+00_JPRB, 0.74128E+00_JPRB, 0.64913E+00_JPRB, 0.59246E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.58515E+00_JPRB, 0.51970E+00_JPRB, 0.47930E+00_JPRB, 0.48245E+00_JPRB, 0.50939E+00_JPRB, 0.52012E+00_JPRB, 0.52299E+00_JPRB ,&
&  0.51429E+00_JPRB, 0.50197E+00_JPRB, 0.48256E+00_JPRB, 0.47506E+00_JPRB, 0.48554E+00_JPRB, 0.50907E+00_JPRB, 0.56600E+00_JPRB ,&
&  0.62212E+00_JPRB, 0.67509E+00_JPRB, 0.71578E+00_JPRB, 0.71421E+00_JPRB, 0.68000E+00_JPRB, 0.67331E+00_JPRB, 0.69097E+00_JPRB ,&
&  0.72079E+00_JPRB, 0.77870E+00_JPRB, 0.86085E+00_JPRB, 0.97721E+00_JPRB, 0.11658E+01_JPRB, 0.14941E+01_JPRB, 0.20190E+01_JPRB ,&
&  0.29956E+01_JPRB, 0.20234E+01_JPRB, 0.12320E+01_JPRB, 0.24447E+01_JPRB, 0.29349E+01_JPRB, 0.30754E+01_JPRB, 0.25304E+01_JPRB ,&
&  0.24255E+01_JPRB, 0.22393E+01_JPRB, 0.20157E+01_JPRB, 0.19368E+01_JPRB, 0.16627E+01_JPRB, 0.15565E+01_JPRB, 0.14604E+01_JPRB ,&
&  0.13898E+01_JPRB, 0.13972E+01_JPRB, 0.14413E+01_JPRB, 0.14841E+01_JPRB, 0.15297E+01_JPRB, 0.16534E+01_JPRB, 0.18166E+01_JPRB ,&
&  0.19867E+01_JPRB, 0.20873E+01_JPRB, 0.21169E+01_JPRB, 0.16483E+01_JPRB, 0.14362E+01_JPRB, 0.13361E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.11702E+01_JPRB, 0.11169E+01_JPRB, 0.11049E+01_JPRB, 0.10671E+01_JPRB, 0.10182E+01_JPRB, 0.97302E+00_JPRB, 0.98504E+00_JPRB ,&
&  0.91562E+00_JPRB, 0.84937E+00_JPRB, 0.80066E+00_JPRB, 0.77005E+00_JPRB, 0.75902E+00_JPRB, 0.75589E+00_JPRB, 0.74542E+00_JPRB ,&
&  0.74996E+00_JPRB, 0.75055E+00_JPRB, 0.75371E+00_JPRB, 0.73698E+00_JPRB, 0.71893E+00_JPRB, 0.71001E+00_JPRB, 0.69618E+00_JPRB ,&
&  0.72713E+00_JPRB, 0.76473E+00_JPRB, 0.78648E+00_JPRB, 0.81660E+00_JPRB, 0.82088E+00_JPRB, 0.81567E+00_JPRB, 0.83725E+00_JPRB ,&
&  0.83989E+00_JPRB, 0.85506E+00_JPRB, 0.83690E+00_JPRB, 0.84721E+00_JPRB, 0.82842E+00_JPRB, 0.81035E+00_JPRB, 0.80823E+00_JPRB ,&
&  0.84128E+00_JPRB, 0.87994E+00_JPRB, 0.85314E+00_JPRB, 0.68668E+00_JPRB, 0.62258E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.57013E+00_JPRB, 0.54164E+00_JPRB, 0.50013E+00_JPRB, 0.50662E+00_JPRB, 0.51263E+00_JPRB, 0.52345E+00_JPRB, 0.52760E+00_JPRB ,&
&  0.53061E+00_JPRB, 0.53283E+00_JPRB, 0.53569E+00_JPRB, 0.52494E+00_JPRB, 0.49673E+00_JPRB, 0.48568E+00_JPRB, 0.48929E+00_JPRB ,&
&  0.50743E+00_JPRB, 0.54649E+00_JPRB, 0.59050E+00_JPRB, 0.59821E+00_JPRB, 0.59097E+00_JPRB, 0.58071E+00_JPRB, 0.57616E+00_JPRB ,&
&  0.58344E+00_JPRB, 0.60071E+00_JPRB, 0.62871E+00_JPRB, 0.68714E+00_JPRB, 0.79216E+00_JPRB, 0.96123E+00_JPRB, 0.12719E+01_JPRB ,&
&  0.20712E+01_JPRB, 0.22562E+01_JPRB, 0.96834E+00_JPRB, 0.19993E+01_JPRB, 0.30657E+01_JPRB, 0.33018E+01_JPRB, 0.31598E+01_JPRB ,&
&  0.29886E+01_JPRB, 0.26534E+01_JPRB, 0.23645E+01_JPRB, 0.20640E+01_JPRB, 0.17284E+01_JPRB, 0.15483E+01_JPRB, 0.14026E+01_JPRB ,&
&  0.12831E+01_JPRB, 0.12585E+01_JPRB, 0.12937E+01_JPRB, 0.13222E+01_JPRB, 0.13225E+01_JPRB, 0.13554E+01_JPRB, 0.14703E+01_JPRB ,&
&  0.16509E+01_JPRB, 0.18412E+01_JPRB, 0.19235E+01_JPRB, 0.15755E+01_JPRB, 0.14220E+01_JPRB, 0.14319E+01_JPRB, 0.13901E+01_JPRB ,&
&  0.13100E+01_JPRB, 0.12336E+01_JPRB, 0.11724E+01_JPRB, 0.11179E+01_JPRB, 0.10557E+01_JPRB, 0.10341E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.93056E+00_JPRB, 0.88097E+00_JPRB, 0.84956E+00_JPRB, 0.82844E+00_JPRB, 0.81640E+00_JPRB, 0.80351E+00_JPRB ,&
&  0.79679E+00_JPRB, 0.80449E+00_JPRB, 0.81518E+00_JPRB, 0.80486E+00_JPRB, 0.77923E+00_JPRB, 0.74924E+00_JPRB, 0.72225E+00_JPRB ,&
&  0.72535E+00_JPRB, 0.74661E+00_JPRB, 0.78648E+00_JPRB, 0.80476E+00_JPRB, 0.81967E+00_JPRB, 0.83918E+00_JPRB, 0.84790E+00_JPRB ,&
&  0.86138E+00_JPRB, 0.88267E+00_JPRB, 0.84133E+00_JPRB, 0.82965E+00_JPRB, 0.84586E+00_JPRB, 0.81526E+00_JPRB, 0.78502E+00_JPRB ,&
&  0.81234E+00_JPRB, 0.87629E+00_JPRB, 0.96923E+00_JPRB, 0.69161E+00_JPRB, 0.61982E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.56603E+00_JPRB, 0.53004E+00_JPRB, 0.52278E+00_JPRB, 0.52236E+00_JPRB, 0.52767E+00_JPRB, 0.53926E+00_JPRB, 0.55261E+00_JPRB ,&
&  0.55713E+00_JPRB, 0.55835E+00_JPRB, 0.55861E+00_JPRB, 0.55388E+00_JPRB, 0.53900E+00_JPRB, 0.52367E+00_JPRB, 0.50825E+00_JPRB ,&
&  0.50335E+00_JPRB, 0.50830E+00_JPRB, 0.52713E+00_JPRB, 0.53642E+00_JPRB, 0.53664E+00_JPRB, 0.53292E+00_JPRB, 0.53372E+00_JPRB ,&
&  0.53872E+00_JPRB, 0.54056E+00_JPRB, 0.54272E+00_JPRB, 0.56533E+00_JPRB, 0.61417E+00_JPRB, 0.71128E+00_JPRB, 0.90064E+00_JPRB ,&
&  0.14513E+01_JPRB, 0.19354E+01_JPRB, 0.18921E+01_JPRB, 0.30869E+01_JPRB, 0.37908E+01_JPRB, 0.37458E+01_JPRB, 0.36857E+01_JPRB ,&
&  0.34718E+01_JPRB, 0.30894E+01_JPRB, 0.28652E+01_JPRB, 0.22625E+01_JPRB, 0.18502E+01_JPRB, 0.15600E+01_JPRB, 0.13494E+01_JPRB ,&
&  0.11807E+01_JPRB, 0.11036E+01_JPRB, 0.11124E+01_JPRB, 0.11562E+01_JPRB, 0.11860E+01_JPRB, 0.11955E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.13674E+01_JPRB, 0.16035E+01_JPRB, 0.17951E+01_JPRB, 0.16656E+01_JPRB, 0.14107E+01_JPRB, 0.15229E+01_JPRB, 0.16419E+01_JPRB ,&
&  0.18143E+01_JPRB, 0.14593E+01_JPRB, 0.12493E+01_JPRB, 0.11637E+01_JPRB, 0.10862E+01_JPRB, 0.11514E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.10014E+01_JPRB, 0.94465E+00_JPRB, 0.90702E+00_JPRB, 0.87737E+00_JPRB, 0.85606E+00_JPRB, 0.84068E+00_JPRB ,&
&  0.83247E+00_JPRB, 0.82926E+00_JPRB, 0.83914E+00_JPRB, 0.84106E+00_JPRB, 0.83142E+00_JPRB, 0.80656E+00_JPRB, 0.77195E+00_JPRB ,&
&  0.74602E+00_JPRB, 0.74720E+00_JPRB, 0.74676E+00_JPRB, 0.76827E+00_JPRB, 0.79520E+00_JPRB, 0.80871E+00_JPRB, 0.79915E+00_JPRB ,&
&  0.80572E+00_JPRB, 0.83220E+00_JPRB, 0.84034E+00_JPRB, 0.84422E+00_JPRB, 0.84400E+00_JPRB, 0.83426E+00_JPRB, 0.77369E+00_JPRB ,&
&  0.74297E+00_JPRB, 0.72291E+00_JPRB, 0.68588E+00_JPRB, 0.65256E+00_JPRB, 0.60445E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.57255E+00_JPRB, 0.55019E+00_JPRB, 0.54378E+00_JPRB, 0.54747E+00_JPRB, 0.55216E+00_JPRB, 0.55880E+00_JPRB, 0.57170E+00_JPRB ,&
&  0.57852E+00_JPRB, 0.58021E+00_JPRB, 0.57903E+00_JPRB, 0.57624E+00_JPRB, 0.56602E+00_JPRB, 0.55374E+00_JPRB, 0.53983E+00_JPRB ,&
&  0.52759E+00_JPRB, 0.50453E+00_JPRB, 0.49323E+00_JPRB, 0.50032E+00_JPRB, 0.50343E+00_JPRB, 0.50402E+00_JPRB, 0.50601E+00_JPRB ,&
&  0.50668E+00_JPRB, 0.51197E+00_JPRB, 0.51226E+00_JPRB, 0.52202E+00_JPRB, 0.54565E+00_JPRB, 0.60955E+00_JPRB, 0.72736E+00_JPRB ,&
&  0.10912E+01_JPRB, 0.17612E+01_JPRB, 0.15674E+01_JPRB, 0.35733E+01_JPRB, 0.40298E+01_JPRB, 0.39718E+01_JPRB, 0.35838E+01_JPRB ,&
&  0.33134E+01_JPRB, 0.34104E+01_JPRB, 0.26302E+01_JPRB, 0.21152E+01_JPRB, 0.18254E+01_JPRB, 0.15750E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.10037E+01_JPRB, 0.96185E+00_JPRB, 0.97667E+00_JPRB, 0.10271E+01_JPRB, 0.10737E+01_JPRB, 0.11131E+01_JPRB ,&
&  0.11856E+01_JPRB, 0.13586E+01_JPRB, 0.16065E+01_JPRB, 0.15648E+01_JPRB, 0.14743E+01_JPRB, 0.16291E+01_JPRB, 0.19339E+01_JPRB ,&
&  0.22697E+01_JPRB, 0.19195E+01_JPRB, 0.14223E+01_JPRB, 0.12296E+01_JPRB, 0.11751E+01_JPRB, 0.12128E+01_JPRB, 0.11934E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.10741E+01_JPRB, 0.10049E+01_JPRB, 0.95685E+00_JPRB, 0.92221E+00_JPRB, 0.88044E+00_JPRB, 0.85311E+00_JPRB ,&
&  0.84124E+00_JPRB, 0.83610E+00_JPRB, 0.83491E+00_JPRB, 0.82910E+00_JPRB, 0.82761E+00_JPRB, 0.82481E+00_JPRB, 0.81466E+00_JPRB ,&
&  0.79234E+00_JPRB, 0.76113E+00_JPRB, 0.72934E+00_JPRB, 0.72898E+00_JPRB, 0.75301E+00_JPRB, 0.76287E+00_JPRB, 0.76361E+00_JPRB ,&
&  0.79413E+00_JPRB, 0.81373E+00_JPRB, 0.81529E+00_JPRB, 0.80399E+00_JPRB, 0.82588E+00_JPRB, 0.87275E+00_JPRB, 0.78987E+00_JPRB ,&
&  0.75702E+00_JPRB, 0.72678E+00_JPRB, 0.69660E+00_JPRB, 0.66631E+00_JPRB, 0.61983E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.60849E+00_JPRB, 0.59112E+00_JPRB, 0.58008E+00_JPRB, 0.57664E+00_JPRB, 0.57070E+00_JPRB, 0.57099E+00_JPRB, 0.57996E+00_JPRB ,&
&  0.59505E+00_JPRB, 0.60035E+00_JPRB, 0.59904E+00_JPRB, 0.59656E+00_JPRB, 0.58977E+00_JPRB, 0.57821E+00_JPRB, 0.56420E+00_JPRB ,&
&  0.55184E+00_JPRB, 0.53412E+00_JPRB, 0.51036E+00_JPRB, 0.49180E+00_JPRB, 0.49353E+00_JPRB, 0.49850E+00_JPRB, 0.50054E+00_JPRB ,&
&  0.50085E+00_JPRB, 0.50798E+00_JPRB, 0.51094E+00_JPRB, 0.52149E+00_JPRB, 0.53102E+00_JPRB, 0.56779E+00_JPRB, 0.67845E+00_JPRB ,&
&  0.96507E+00_JPRB, 0.21168E+01_JPRB, 0.16875E+01_JPRB, 0.33326E+01_JPRB, 0.37921E+01_JPRB, 0.36817E+01_JPRB, 0.32623E+01_JPRB ,&
&  0.30150E+01_JPRB, 0.28224E+01_JPRB, 0.23230E+01_JPRB, 0.19016E+01_JPRB, 0.16339E+01_JPRB, 0.14312E+01_JPRB, 0.12606E+01_JPRB ,&
&  0.11286E+01_JPRB, 0.10419E+01_JPRB, 0.98751E+00_JPRB, 0.95071E+00_JPRB, 0.94826E+00_JPRB, 0.97145E+00_JPRB, 0.10126E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.11588E+01_JPRB, 0.13736E+01_JPRB, 0.15770E+01_JPRB, 0.14702E+01_JPRB, 0.16304E+01_JPRB, 0.22203E+01_JPRB ,&
&  0.25584E+01_JPRB, 0.25482E+01_JPRB, 0.17978E+01_JPRB, 0.13896E+01_JPRB, 0.12623E+01_JPRB, 0.12781E+01_JPRB, 0.12726E+01_JPRB ,&
&  0.12151E+01_JPRB, 0.11659E+01_JPRB, 0.10902E+01_JPRB, 0.10105E+01_JPRB, 0.95096E+00_JPRB, 0.90604E+00_JPRB, 0.85957E+00_JPRB ,&
&  0.82712E+00_JPRB, 0.81454E+00_JPRB, 0.80557E+00_JPRB, 0.79456E+00_JPRB, 0.78903E+00_JPRB, 0.79223E+00_JPRB, 0.79774E+00_JPRB ,&
&  0.80084E+00_JPRB, 0.78712E+00_JPRB, 0.74613E+00_JPRB, 0.71728E+00_JPRB, 0.71554E+00_JPRB, 0.72356E+00_JPRB, 0.72672E+00_JPRB ,&
&  0.76920E+00_JPRB, 0.78928E+00_JPRB, 0.78701E+00_JPRB, 0.80229E+00_JPRB, 0.81846E+00_JPRB, 0.92154E+00_JPRB, 0.81441E+00_JPRB ,&
&  0.76446E+00_JPRB, 0.73139E+00_JPRB, 0.71394E+00_JPRB, 0.69155E+00_JPRB, 0.64987E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.64052E+00_JPRB, 0.61926E+00_JPRB, 0.60812E+00_JPRB, 0.59367E+00_JPRB, 0.58168E+00_JPRB, 0.57750E+00_JPRB, 0.58367E+00_JPRB ,&
&  0.60034E+00_JPRB, 0.61433E+00_JPRB, 0.60907E+00_JPRB, 0.60613E+00_JPRB, 0.60377E+00_JPRB, 0.59433E+00_JPRB, 0.58096E+00_JPRB ,&
&  0.56763E+00_JPRB, 0.55534E+00_JPRB, 0.53927E+00_JPRB, 0.52017E+00_JPRB, 0.50679E+00_JPRB, 0.50604E+00_JPRB, 0.50413E+00_JPRB ,&
&  0.50304E+00_JPRB, 0.50782E+00_JPRB, 0.51224E+00_JPRB, 0.51996E+00_JPRB, 0.52830E+00_JPRB, 0.55039E+00_JPRB, 0.65239E+00_JPRB ,&
&  0.99320E+00_JPRB, 0.76193E+00_JPRB, 0.21135E+01_JPRB, 0.29316E+01_JPRB, 0.30755E+01_JPRB, 0.28689E+01_JPRB, 0.27566E+01_JPRB ,&
&  0.26023E+01_JPRB, 0.23241E+01_JPRB, 0.20629E+01_JPRB, 0.18316E+01_JPRB, 0.15431E+01_JPRB, 0.13521E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.10786E+01_JPRB, 0.10159E+01_JPRB, 0.95333E+00_JPRB, 0.90660E+00_JPRB, 0.89129E+00_JPRB, 0.91318E+00_JPRB ,&
&  0.95176E+00_JPRB, 0.10084E+01_JPRB, 0.11638E+01_JPRB, 0.13702E+01_JPRB, 0.14629E+01_JPRB, 0.15928E+01_JPRB, 0.18897E+01_JPRB ,&
&  0.23844E+01_JPRB, 0.26188E+01_JPRB, 0.20882E+01_JPRB, 0.16532E+01_JPRB, 0.14129E+01_JPRB, 0.13549E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.12356E+01_JPRB, 0.11509E+01_JPRB, 0.10561E+01_JPRB, 0.97526E+00_JPRB, 0.92165E+00_JPRB, 0.88142E+00_JPRB ,&
&  0.84155E+00_JPRB, 0.80679E+00_JPRB, 0.78403E+00_JPRB, 0.76874E+00_JPRB, 0.75951E+00_JPRB, 0.75616E+00_JPRB, 0.76164E+00_JPRB ,&
&  0.77640E+00_JPRB, 0.78287E+00_JPRB, 0.77064E+00_JPRB, 0.73929E+00_JPRB, 0.72740E+00_JPRB, 0.73101E+00_JPRB, 0.73867E+00_JPRB ,&
&  0.77199E+00_JPRB, 0.78235E+00_JPRB, 0.78389E+00_JPRB, 0.80561E+00_JPRB, 0.79153E+00_JPRB, 0.83997E+00_JPRB, 0.79582E+00_JPRB ,&
&  0.75275E+00_JPRB, 0.73060E+00_JPRB, 0.72393E+00_JPRB, 0.70754E+00_JPRB, 0.67523E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.66477E+00_JPRB, 0.64329E+00_JPRB, 0.62803E+00_JPRB, 0.60493E+00_JPRB, 0.59216E+00_JPRB, 0.58802E+00_JPRB, 0.59363E+00_JPRB ,&
&  0.60051E+00_JPRB, 0.60691E+00_JPRB, 0.60241E+00_JPRB, 0.59748E+00_JPRB, 0.59021E+00_JPRB, 0.58438E+00_JPRB, 0.57539E+00_JPRB ,&
&  0.57006E+00_JPRB, 0.56539E+00_JPRB, 0.55713E+00_JPRB, 0.54550E+00_JPRB, 0.53054E+00_JPRB, 0.52294E+00_JPRB, 0.51524E+00_JPRB ,&
&  0.50855E+00_JPRB, 0.50562E+00_JPRB, 0.50668E+00_JPRB, 0.51543E+00_JPRB, 0.52757E+00_JPRB, 0.54705E+00_JPRB, 0.61446E+00_JPRB ,&
&  0.10152E+01_JPRB, 0.15136E+01_JPRB, 0.24655E+01_JPRB, 0.25771E+01_JPRB, 0.23203E+01_JPRB, 0.20116E+01_JPRB, 0.19703E+01_JPRB ,&
&  0.18944E+01_JPRB, 0.17911E+01_JPRB, 0.16599E+01_JPRB, 0.15679E+01_JPRB, 0.14678E+01_JPRB, 0.13993E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.10969E+01_JPRB, 0.98940E+00_JPRB, 0.90201E+00_JPRB, 0.84316E+00_JPRB, 0.81511E+00_JPRB, 0.81798E+00_JPRB ,&
&  0.83483E+00_JPRB, 0.86061E+00_JPRB, 0.95405E+00_JPRB, 0.11444E+01_JPRB, 0.14137E+01_JPRB, 0.14557E+01_JPRB, 0.18412E+01_JPRB ,&
&  0.23788E+01_JPRB, 0.23393E+01_JPRB, 0.21178E+01_JPRB, 0.19213E+01_JPRB, 0.16667E+01_JPRB, 0.15610E+01_JPRB, 0.15291E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.13030E+01_JPRB, 0.11603E+01_JPRB, 0.10518E+01_JPRB, 0.97641E+00_JPRB, 0.93104E+00_JPRB, 0.90858E+00_JPRB ,&
&  0.88475E+00_JPRB, 0.85102E+00_JPRB, 0.81273E+00_JPRB, 0.78134E+00_JPRB, 0.76267E+00_JPRB, 0.75538E+00_JPRB, 0.75477E+00_JPRB ,&
&  0.75880E+00_JPRB, 0.76406E+00_JPRB, 0.78100E+00_JPRB, 0.75749E+00_JPRB, 0.76413E+00_JPRB, 0.78636E+00_JPRB, 0.79452E+00_JPRB ,&
&  0.78538E+00_JPRB, 0.79400E+00_JPRB, 0.79669E+00_JPRB, 0.80837E+00_JPRB, 0.85086E+00_JPRB, 0.86776E+00_JPRB, 0.80646E+00_JPRB ,&
&  0.74459E+00_JPRB, 0.72154E+00_JPRB, 0.72620E+00_JPRB, 0.71903E+00_JPRB, 0.70305E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.68918E+00_JPRB, 0.66412E+00_JPRB, 0.64157E+00_JPRB, 0.61978E+00_JPRB, 0.60212E+00_JPRB, 0.59463E+00_JPRB, 0.59421E+00_JPRB ,&
&  0.59627E+00_JPRB, 0.59446E+00_JPRB, 0.58924E+00_JPRB, 0.58094E+00_JPRB, 0.57487E+00_JPRB, 0.56921E+00_JPRB, 0.56427E+00_JPRB ,&
&  0.55669E+00_JPRB, 0.55773E+00_JPRB, 0.55405E+00_JPRB, 0.54626E+00_JPRB, 0.53723E+00_JPRB, 0.52630E+00_JPRB, 0.51591E+00_JPRB ,&
&  0.50896E+00_JPRB, 0.50604E+00_JPRB, 0.50582E+00_JPRB, 0.51018E+00_JPRB, 0.52365E+00_JPRB, 0.54074E+00_JPRB, 0.57661E+00_JPRB ,&
&  0.87230E+00_JPRB, 0.14635E+01_JPRB, 0.18680E+01_JPRB, 0.21353E+01_JPRB, 0.18953E+01_JPRB, 0.16076E+01_JPRB, 0.14436E+01_JPRB ,&
&  0.13744E+01_JPRB, 0.13635E+01_JPRB, 0.13023E+01_JPRB, 0.12233E+01_JPRB, 0.11280E+01_JPRB, 0.10791E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.96586E+00_JPRB, 0.91006E+00_JPRB, 0.86058E+00_JPRB, 0.83150E+00_JPRB, 0.80558E+00_JPRB, 0.77711E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.75631E+00_JPRB, 0.76281E+00_JPRB, 0.79677E+00_JPRB, 0.91833E+00_JPRB, 0.11729E+01_JPRB, 0.13728E+01_JPRB, 0.16240E+01_JPRB ,&
&  0.16856E+01_JPRB, 0.17422E+01_JPRB, 0.17856E+01_JPRB, 0.17701E+01_JPRB, 0.16691E+01_JPRB, 0.15977E+01_JPRB, 0.15823E+01_JPRB ,&
&  0.15029E+01_JPRB, 0.13148E+01_JPRB, 0.11317E+01_JPRB, 0.10180E+01_JPRB, 0.96024E+00_JPRB, 0.93347E+00_JPRB, 0.91286E+00_JPRB ,&
&  0.90045E+00_JPRB, 0.88423E+00_JPRB, 0.85553E+00_JPRB, 0.82039E+00_JPRB, 0.78793E+00_JPRB, 0.76306E+00_JPRB, 0.74937E+00_JPRB ,&
&  0.73964E+00_JPRB, 0.73701E+00_JPRB, 0.75270E+00_JPRB, 0.76651E+00_JPRB, 0.77730E+00_JPRB, 0.78891E+00_JPRB, 0.80004E+00_JPRB ,&
&  0.81270E+00_JPRB, 0.83743E+00_JPRB, 0.80832E+00_JPRB, 0.81962E+00_JPRB, 0.84311E+00_JPRB, 0.83702E+00_JPRB, 0.77964E+00_JPRB ,&
&  0.72616E+00_JPRB, 0.70270E+00_JPRB, 0.70656E+00_JPRB, 0.71135E+00_JPRB, 0.71216E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.77276E+00_JPRB, 0.71786E+00_JPRB, 0.67127E+00_JPRB, 0.63429E+00_JPRB, 0.61224E+00_JPRB, 0.59894E+00_JPRB, 0.58936E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.57827E+00_JPRB, 0.56778E+00_JPRB, 0.55980E+00_JPRB, 0.55403E+00_JPRB, 0.55103E+00_JPRB, 0.54803E+00_JPRB ,&
&  0.53804E+00_JPRB, 0.53645E+00_JPRB, 0.53682E+00_JPRB, 0.53223E+00_JPRB, 0.52569E+00_JPRB, 0.51818E+00_JPRB, 0.50834E+00_JPRB ,&
&  0.50128E+00_JPRB, 0.49876E+00_JPRB, 0.50020E+00_JPRB, 0.50433E+00_JPRB, 0.50914E+00_JPRB, 0.52274E+00_JPRB, 0.56167E+00_JPRB ,&
&  0.81259E+00_JPRB, 0.11442E+01_JPRB, 0.14124E+01_JPRB, 0.15353E+01_JPRB, 0.13116E+01_JPRB, 0.11866E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.11269E+01_JPRB, 0.10654E+01_JPRB, 0.96302E+00_JPRB, 0.86884E+00_JPRB, 0.81721E+00_JPRB, 0.78267E+00_JPRB ,&
&  0.75177E+00_JPRB, 0.72689E+00_JPRB, 0.70470E+00_JPRB, 0.70089E+00_JPRB, 0.69846E+00_JPRB, 0.69116E+00_JPRB, 0.68394E+00_JPRB ,&
&  0.68076E+00_JPRB, 0.68339E+00_JPRB, 0.69436E+00_JPRB, 0.74539E+00_JPRB, 0.87708E+00_JPRB, 0.10200E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.12861E+01_JPRB, 0.13667E+01_JPRB, 0.14275E+01_JPRB, 0.14292E+01_JPRB, 0.14168E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.13643E+01_JPRB, 0.12505E+01_JPRB, 0.11007E+01_JPRB, 0.98943E+00_JPRB, 0.92890E+00_JPRB, 0.89852E+00_JPRB, 0.88040E+00_JPRB ,&
&  0.86434E+00_JPRB, 0.84828E+00_JPRB, 0.82463E+00_JPRB, 0.79290E+00_JPRB, 0.75884E+00_JPRB, 0.73491E+00_JPRB, 0.71741E+00_JPRB ,&
&  0.70749E+00_JPRB, 0.71008E+00_JPRB, 0.72553E+00_JPRB, 0.75076E+00_JPRB, 0.76900E+00_JPRB, 0.78005E+00_JPRB, 0.78298E+00_JPRB ,&
&  0.78566E+00_JPRB, 0.81248E+00_JPRB, 0.78301E+00_JPRB, 0.82621E+00_JPRB, 0.88276E+00_JPRB, 0.82684E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.71496E+00_JPRB, 0.68235E+00_JPRB, 0.67209E+00_JPRB, 0.69125E+00_JPRB, 0.80201E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.71746E+00_JPRB, 0.69636E+00_JPRB, 0.66372E+00_JPRB, 0.63252E+00_JPRB, 0.61149E+00_JPRB, 0.59808E+00_JPRB, 0.58968E+00_JPRB ,&
&  0.58149E+00_JPRB, 0.57187E+00_JPRB, 0.56053E+00_JPRB, 0.54646E+00_JPRB, 0.53742E+00_JPRB, 0.53307E+00_JPRB, 0.53003E+00_JPRB ,&
&  0.52484E+00_JPRB, 0.51509E+00_JPRB, 0.51219E+00_JPRB, 0.50947E+00_JPRB, 0.50815E+00_JPRB, 0.50611E+00_JPRB, 0.50050E+00_JPRB ,&
&  0.49351E+00_JPRB, 0.49000E+00_JPRB, 0.49283E+00_JPRB, 0.49433E+00_JPRB, 0.49652E+00_JPRB, 0.50385E+00_JPRB, 0.53033E+00_JPRB ,&
&  0.69209E+00_JPRB, 0.86777E+00_JPRB, 0.10565E+01_JPRB, 0.12076E+01_JPRB, 0.11170E+01_JPRB, 0.10309E+01_JPRB, 0.98100E+00_JPRB ,&
&  0.92072E+00_JPRB, 0.88196E+00_JPRB, 0.82028E+00_JPRB, 0.74976E+00_JPRB, 0.69073E+00_JPRB, 0.66743E+00_JPRB, 0.65350E+00_JPRB ,&
&  0.64203E+00_JPRB, 0.63254E+00_JPRB, 0.62519E+00_JPRB, 0.62161E+00_JPRB, 0.62189E+00_JPRB, 0.62009E+00_JPRB, 0.61797E+00_JPRB ,&
&  0.61871E+00_JPRB, 0.62373E+00_JPRB, 0.63033E+00_JPRB, 0.64660E+00_JPRB, 0.70015E+00_JPRB, 0.79243E+00_JPRB, 0.89073E+00_JPRB ,&
&  0.95206E+00_JPRB, 0.10052E+01_JPRB, 0.10804E+01_JPRB, 0.11402E+01_JPRB, 0.11735E+01_JPRB, 0.11800E+01_JPRB, 0.11644E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.11001E+01_JPRB, 0.10141E+01_JPRB, 0.92205E+00_JPRB, 0.86160E+00_JPRB, 0.82309E+00_JPRB, 0.79454E+00_JPRB ,&
&  0.77280E+00_JPRB, 0.74679E+00_JPRB, 0.72089E+00_JPRB, 0.69940E+00_JPRB, 0.68552E+00_JPRB, 0.67283E+00_JPRB, 0.66223E+00_JPRB ,&
&  0.66465E+00_JPRB, 0.67985E+00_JPRB, 0.70241E+00_JPRB, 0.71796E+00_JPRB, 0.72623E+00_JPRB, 0.73324E+00_JPRB, 0.73414E+00_JPRB ,&
&  0.73616E+00_JPRB, 0.75315E+00_JPRB, 0.76081E+00_JPRB, 0.80661E+00_JPRB, 0.80357E+00_JPRB, 0.78036E+00_JPRB, 0.73708E+00_JPRB ,&
&  0.69806E+00_JPRB, 0.66921E+00_JPRB, 0.64652E+00_JPRB, 0.65736E+00_JPRB, 0.70807E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.65143E+00_JPRB, 0.64563E+00_JPRB, 0.63630E+00_JPRB, 0.62182E+00_JPRB, 0.61171E+00_JPRB, 0.60244E+00_JPRB, 0.59063E+00_JPRB ,&
&  0.57831E+00_JPRB, 0.56760E+00_JPRB, 0.55744E+00_JPRB, 0.54510E+00_JPRB, 0.53186E+00_JPRB, 0.52704E+00_JPRB, 0.52565E+00_JPRB ,&
&  0.52136E+00_JPRB, 0.51297E+00_JPRB, 0.50521E+00_JPRB, 0.50335E+00_JPRB, 0.50247E+00_JPRB, 0.50171E+00_JPRB, 0.49797E+00_JPRB ,&
&  0.49212E+00_JPRB, 0.49090E+00_JPRB, 0.49375E+00_JPRB, 0.49389E+00_JPRB, 0.48947E+00_JPRB, 0.49139E+00_JPRB, 0.51403E+00_JPRB ,&
&  0.62954E+00_JPRB, 0.64212E+00_JPRB, 0.76212E+00_JPRB, 0.94872E+00_JPRB, 0.88215E+00_JPRB, 0.84717E+00_JPRB, 0.78107E+00_JPRB ,&
&  0.71438E+00_JPRB, 0.67667E+00_JPRB, 0.65092E+00_JPRB, 0.62314E+00_JPRB, 0.59681E+00_JPRB, 0.58258E+00_JPRB, 0.58223E+00_JPRB ,&
&  0.58955E+00_JPRB, 0.59282E+00_JPRB, 0.59266E+00_JPRB, 0.58502E+00_JPRB, 0.57957E+00_JPRB, 0.57494E+00_JPRB, 0.57202E+00_JPRB ,&
&  0.57012E+00_JPRB, 0.57331E+00_JPRB, 0.57913E+00_JPRB, 0.58479E+00_JPRB, 0.60252E+00_JPRB, 0.65189E+00_JPRB, 0.72883E+00_JPRB ,&
&  0.80093E+00_JPRB, 0.86439E+00_JPRB, 0.91603E+00_JPRB, 0.96397E+00_JPRB, 0.99464E+00_JPRB, 0.10197E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.95709E+00_JPRB, 0.90439E+00_JPRB, 0.84583E+00_JPRB, 0.79242E+00_JPRB, 0.74979E+00_JPRB, 0.71633E+00_JPRB ,&
&  0.69076E+00_JPRB, 0.67315E+00_JPRB, 0.65954E+00_JPRB, 0.64500E+00_JPRB, 0.63663E+00_JPRB, 0.63601E+00_JPRB, 0.64033E+00_JPRB ,&
&  0.64911E+00_JPRB, 0.65947E+00_JPRB, 0.66640E+00_JPRB, 0.66873E+00_JPRB, 0.66737E+00_JPRB, 0.66754E+00_JPRB, 0.66855E+00_JPRB ,&
&  0.67777E+00_JPRB, 0.69711E+00_JPRB, 0.71301E+00_JPRB, 0.76255E+00_JPRB, 0.79532E+00_JPRB, 0.78235E+00_JPRB, 0.74711E+00_JPRB ,&
&  0.70711E+00_JPRB, 0.67056E+00_JPRB, 0.64961E+00_JPRB, 0.58032E+00_JPRB, 0.65490E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.61124E+00_JPRB, 0.60966E+00_JPRB, 0.60584E+00_JPRB, 0.60115E+00_JPRB, 0.59603E+00_JPRB, 0.58757E+00_JPRB, 0.57531E+00_JPRB ,&
&  0.56322E+00_JPRB, 0.55192E+00_JPRB, 0.54390E+00_JPRB, 0.53585E+00_JPRB, 0.52418E+00_JPRB, 0.51668E+00_JPRB, 0.51527E+00_JPRB ,&
&  0.51337E+00_JPRB, 0.50761E+00_JPRB, 0.50084E+00_JPRB, 0.49949E+00_JPRB, 0.49712E+00_JPRB, 0.49379E+00_JPRB, 0.48908E+00_JPRB ,&
&  0.48428E+00_JPRB, 0.48258E+00_JPRB, 0.48264E+00_JPRB, 0.48311E+00_JPRB, 0.47950E+00_JPRB, 0.48162E+00_JPRB, 0.50530E+00_JPRB ,&
&  0.59787E+00_JPRB, 0.54943E+00_JPRB, 0.64808E+00_JPRB, 0.75108E+00_JPRB, 0.68928E+00_JPRB, 0.64172E+00_JPRB, 0.60299E+00_JPRB ,&
&  0.57298E+00_JPRB, 0.55571E+00_JPRB, 0.54603E+00_JPRB, 0.53675E+00_JPRB, 0.52945E+00_JPRB, 0.52285E+00_JPRB, 0.52293E+00_JPRB ,&
&  0.52666E+00_JPRB, 0.53684E+00_JPRB, 0.54336E+00_JPRB, 0.54378E+00_JPRB, 0.53871E+00_JPRB, 0.53271E+00_JPRB, 0.52726E+00_JPRB ,&
&  0.52393E+00_JPRB, 0.52411E+00_JPRB, 0.52675E+00_JPRB, 0.53230E+00_JPRB, 0.54394E+00_JPRB, 0.56884E+00_JPRB, 0.61403E+00_JPRB ,&
&  0.67262E+00_JPRB, 0.72358E+00_JPRB, 0.76811E+00_JPRB, 0.81600E+00_JPRB, 0.85901E+00_JPRB, 0.88593E+00_JPRB, 0.88837E+00_JPRB ,&
&  0.86741E+00_JPRB, 0.82584E+00_JPRB, 0.79049E+00_JPRB, 0.75655E+00_JPRB, 0.72405E+00_JPRB, 0.69736E+00_JPRB, 0.67893E+00_JPRB ,&
&  0.66692E+00_JPRB, 0.65874E+00_JPRB, 0.65340E+00_JPRB, 0.64878E+00_JPRB, 0.64798E+00_JPRB, 0.64838E+00_JPRB, 0.65053E+00_JPRB ,&
&  0.64825E+00_JPRB, 0.64305E+00_JPRB, 0.62997E+00_JPRB, 0.61274E+00_JPRB, 0.60252E+00_JPRB, 0.59893E+00_JPRB, 0.60011E+00_JPRB ,&
&  0.60748E+00_JPRB, 0.62281E+00_JPRB, 0.63962E+00_JPRB, 0.69002E+00_JPRB, 0.71383E+00_JPRB, 0.72538E+00_JPRB, 0.72080E+00_JPRB ,&
&  0.69806E+00_JPRB, 0.66598E+00_JPRB, 0.54583E+00_JPRB, 0.60426E+00_JPRB, 0.61680E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.60747E+00_JPRB, 0.59894E+00_JPRB, 0.59109E+00_JPRB, 0.58326E+00_JPRB, 0.57416E+00_JPRB, 0.56374E+00_JPRB, 0.55238E+00_JPRB ,&
&  0.54212E+00_JPRB, 0.53303E+00_JPRB, 0.52425E+00_JPRB, 0.51680E+00_JPRB, 0.50981E+00_JPRB, 0.50643E+00_JPRB, 0.50553E+00_JPRB ,&
&  0.50605E+00_JPRB, 0.50527E+00_JPRB, 0.50321E+00_JPRB, 0.50282E+00_JPRB, 0.50211E+00_JPRB, 0.49914E+00_JPRB, 0.49488E+00_JPRB ,&
&  0.48828E+00_JPRB, 0.48251E+00_JPRB, 0.47999E+00_JPRB, 0.48172E+00_JPRB, 0.48028E+00_JPRB, 0.48377E+00_JPRB, 0.49875E+00_JPRB ,&
&  0.49999E+00_JPRB, 0.49433E+00_JPRB, 0.55507E+00_JPRB, 0.63036E+00_JPRB, 0.58638E+00_JPRB, 0.54468E+00_JPRB, 0.52753E+00_JPRB ,&
&  0.51329E+00_JPRB, 0.50373E+00_JPRB, 0.49702E+00_JPRB, 0.49317E+00_JPRB, 0.49304E+00_JPRB, 0.49281E+00_JPRB, 0.49294E+00_JPRB ,&
&  0.49455E+00_JPRB, 0.49982E+00_JPRB, 0.50546E+00_JPRB, 0.50906E+00_JPRB, 0.50851E+00_JPRB, 0.50543E+00_JPRB, 0.49993E+00_JPRB ,&
&  0.49484E+00_JPRB, 0.49150E+00_JPRB, 0.49102E+00_JPRB, 0.49502E+00_JPRB, 0.50313E+00_JPRB, 0.51750E+00_JPRB, 0.54270E+00_JPRB ,&
&  0.57927E+00_JPRB, 0.61727E+00_JPRB, 0.65134E+00_JPRB, 0.68656E+00_JPRB, 0.72726E+00_JPRB, 0.76395E+00_JPRB, 0.78442E+00_JPRB ,&
&  0.77508E+00_JPRB, 0.74525E+00_JPRB, 0.71031E+00_JPRB, 0.68160E+00_JPRB, 0.66010E+00_JPRB, 0.64606E+00_JPRB, 0.63752E+00_JPRB ,&
&  0.63416E+00_JPRB, 0.63203E+00_JPRB, 0.63031E+00_JPRB, 0.62938E+00_JPRB, 0.62831E+00_JPRB, 0.62446E+00_JPRB, 0.61845E+00_JPRB ,&
&  0.60873E+00_JPRB, 0.59389E+00_JPRB, 0.57701E+00_JPRB, 0.56325E+00_JPRB, 0.55410E+00_JPRB, 0.55101E+00_JPRB, 0.55266E+00_JPRB ,&
&  0.56018E+00_JPRB, 0.57628E+00_JPRB, 0.59554E+00_JPRB, 0.63654E+00_JPRB, 0.66516E+00_JPRB, 0.68480E+00_JPRB, 0.69367E+00_JPRB ,&
&  0.68556E+00_JPRB, 0.66250E+00_JPRB, 0.63940E+00_JPRB, 0.62609E+00_JPRB, 0.61940E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.59193E+00_JPRB, 0.57690E+00_JPRB, 0.56155E+00_JPRB, 0.55075E+00_JPRB, 0.54243E+00_JPRB, 0.53449E+00_JPRB, 0.52610E+00_JPRB ,&
&  0.51663E+00_JPRB, 0.50764E+00_JPRB, 0.49925E+00_JPRB, 0.49141E+00_JPRB, 0.48735E+00_JPRB, 0.48532E+00_JPRB, 0.48572E+00_JPRB ,&
&  0.48847E+00_JPRB, 0.49334E+00_JPRB, 0.49506E+00_JPRB, 0.49433E+00_JPRB, 0.49481E+00_JPRB, 0.49430E+00_JPRB, 0.49195E+00_JPRB ,&
&  0.48690E+00_JPRB, 0.48090E+00_JPRB, 0.47727E+00_JPRB, 0.47732E+00_JPRB, 0.47723E+00_JPRB, 0.47936E+00_JPRB, 0.48316E+00_JPRB ,&
&  0.49734E+00_JPRB, 0.46816E+00_JPRB, 0.51036E+00_JPRB, 0.55846E+00_JPRB, 0.54357E+00_JPRB, 0.52308E+00_JPRB, 0.50585E+00_JPRB ,&
&  0.49164E+00_JPRB, 0.48351E+00_JPRB, 0.47890E+00_JPRB, 0.47909E+00_JPRB, 0.48204E+00_JPRB, 0.48411E+00_JPRB, 0.48554E+00_JPRB ,&
&  0.48393E+00_JPRB, 0.48463E+00_JPRB, 0.48346E+00_JPRB, 0.48260E+00_JPRB, 0.48280E+00_JPRB, 0.48373E+00_JPRB, 0.48358E+00_JPRB ,&
&  0.48123E+00_JPRB, 0.47710E+00_JPRB, 0.47303E+00_JPRB, 0.47212E+00_JPRB, 0.47364E+00_JPRB, 0.47834E+00_JPRB, 0.48807E+00_JPRB ,&
&  0.50405E+00_JPRB, 0.52409E+00_JPRB, 0.54706E+00_JPRB, 0.57416E+00_JPRB, 0.60998E+00_JPRB, 0.64538E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.67979E+00_JPRB, 0.67199E+00_JPRB, 0.65386E+00_JPRB, 0.63211E+00_JPRB, 0.61258E+00_JPRB, 0.59398E+00_JPRB, 0.57735E+00_JPRB ,&
&  0.56498E+00_JPRB, 0.55508E+00_JPRB, 0.54822E+00_JPRB, 0.54471E+00_JPRB, 0.54163E+00_JPRB, 0.53686E+00_JPRB, 0.53233E+00_JPRB ,&
&  0.52765E+00_JPRB, 0.52336E+00_JPRB, 0.51999E+00_JPRB, 0.51667E+00_JPRB, 0.51282E+00_JPRB, 0.50977E+00_JPRB, 0.51009E+00_JPRB ,&
&  0.51514E+00_JPRB, 0.52576E+00_JPRB, 0.54390E+00_JPRB, 0.57095E+00_JPRB, 0.59776E+00_JPRB, 0.61904E+00_JPRB, 0.63177E+00_JPRB ,&
&  0.63438E+00_JPRB, 0.63130E+00_JPRB, 0.62196E+00_JPRB, 0.61260E+00_JPRB, 0.60447E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.56103E+00_JPRB, 0.54982E+00_JPRB, 0.53820E+00_JPRB, 0.52887E+00_JPRB, 0.52159E+00_JPRB, 0.51392E+00_JPRB, 0.50548E+00_JPRB ,&
&  0.49875E+00_JPRB, 0.49119E+00_JPRB, 0.48544E+00_JPRB, 0.48035E+00_JPRB, 0.47648E+00_JPRB, 0.47516E+00_JPRB, 0.47586E+00_JPRB ,&
&  0.47855E+00_JPRB, 0.48429E+00_JPRB, 0.48709E+00_JPRB, 0.48528E+00_JPRB, 0.48683E+00_JPRB, 0.48891E+00_JPRB, 0.48901E+00_JPRB ,&
&  0.48598E+00_JPRB, 0.48208E+00_JPRB, 0.47784E+00_JPRB, 0.47512E+00_JPRB, 0.47491E+00_JPRB, 0.47702E+00_JPRB, 0.47585E+00_JPRB ,&
&  0.47576E+00_JPRB, 0.48172E+00_JPRB, 0.50181E+00_JPRB, 0.51949E+00_JPRB, 0.51912E+00_JPRB, 0.50789E+00_JPRB, 0.49444E+00_JPRB ,&
&  0.48026E+00_JPRB, 0.47245E+00_JPRB, 0.47130E+00_JPRB, 0.47304E+00_JPRB, 0.47596E+00_JPRB, 0.47795E+00_JPRB, 0.48060E+00_JPRB ,&
&  0.47825E+00_JPRB, 0.47673E+00_JPRB, 0.47358E+00_JPRB, 0.47054E+00_JPRB, 0.47030E+00_JPRB, 0.47127E+00_JPRB, 0.47232E+00_JPRB ,&
&  0.47182E+00_JPRB, 0.47258E+00_JPRB, 0.47299E+00_JPRB, 0.47161E+00_JPRB, 0.47267E+00_JPRB, 0.47487E+00_JPRB, 0.47842E+00_JPRB ,&
&  0.48417E+00_JPRB, 0.49263E+00_JPRB, 0.50269E+00_JPRB, 0.51719E+00_JPRB, 0.53974E+00_JPRB, 0.56455E+00_JPRB, 0.58418E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.59490E+00_JPRB, 0.58858E+00_JPRB, 0.57687E+00_JPRB, 0.56536E+00_JPRB, 0.55451E+00_JPRB, 0.54188E+00_JPRB ,&
&  0.52800E+00_JPRB, 0.51796E+00_JPRB, 0.51055E+00_JPRB, 0.50534E+00_JPRB, 0.50049E+00_JPRB, 0.49696E+00_JPRB, 0.49569E+00_JPRB ,&
&  0.49656E+00_JPRB, 0.49776E+00_JPRB, 0.49748E+00_JPRB, 0.49643E+00_JPRB, 0.49598E+00_JPRB, 0.49357E+00_JPRB, 0.49224E+00_JPRB ,&
&  0.49439E+00_JPRB, 0.50031E+00_JPRB, 0.51149E+00_JPRB, 0.53157E+00_JPRB, 0.54900E+00_JPRB, 0.56314E+00_JPRB, 0.57362E+00_JPRB ,&
&  0.58043E+00_JPRB, 0.58354E+00_JPRB, 0.58220E+00_JPRB, 0.57817E+00_JPRB, 0.57130E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.51623E+00_JPRB, 0.51095E+00_JPRB, 0.50467E+00_JPRB, 0.49847E+00_JPRB, 0.49269E+00_JPRB, 0.48693E+00_JPRB, 0.48160E+00_JPRB ,&
&  0.47675E+00_JPRB, 0.47365E+00_JPRB, 0.47160E+00_JPRB, 0.46999E+00_JPRB, 0.46651E+00_JPRB, 0.46493E+00_JPRB, 0.46558E+00_JPRB ,&
&  0.46804E+00_JPRB, 0.47058E+00_JPRB, 0.47216E+00_JPRB, 0.47135E+00_JPRB, 0.47344E+00_JPRB, 0.47534E+00_JPRB, 0.47524E+00_JPRB ,&
&  0.47399E+00_JPRB, 0.47257E+00_JPRB, 0.46980E+00_JPRB, 0.46662E+00_JPRB, 0.46653E+00_JPRB, 0.46762E+00_JPRB, 0.46509E+00_JPRB ,&
&  0.45957E+00_JPRB, 0.47035E+00_JPRB, 0.46359E+00_JPRB, 0.48624E+00_JPRB, 0.49312E+00_JPRB, 0.48635E+00_JPRB, 0.47617E+00_JPRB ,&
&  0.46945E+00_JPRB, 0.46702E+00_JPRB, 0.46891E+00_JPRB, 0.47105E+00_JPRB, 0.47240E+00_JPRB, 0.47328E+00_JPRB, 0.47330E+00_JPRB ,&
&  0.46496E+00_JPRB, 0.45902E+00_JPRB, 0.45699E+00_JPRB, 0.45665E+00_JPRB, 0.45610E+00_JPRB, 0.45652E+00_JPRB, 0.45851E+00_JPRB ,&
&  0.46030E+00_JPRB, 0.46160E+00_JPRB, 0.46287E+00_JPRB, 0.46530E+00_JPRB, 0.46681E+00_JPRB, 0.46748E+00_JPRB, 0.46932E+00_JPRB ,&
&  0.47249E+00_JPRB, 0.47683E+00_JPRB, 0.48123E+00_JPRB, 0.48577E+00_JPRB, 0.49536E+00_JPRB, 0.50930E+00_JPRB, 0.52128E+00_JPRB ,&
&  0.52745E+00_JPRB, 0.52879E+00_JPRB, 0.52674E+00_JPRB, 0.52034E+00_JPRB, 0.51105E+00_JPRB, 0.50321E+00_JPRB, 0.49653E+00_JPRB ,&
&  0.48784E+00_JPRB, 0.48258E+00_JPRB, 0.47850E+00_JPRB, 0.47357E+00_JPRB, 0.46940E+00_JPRB, 0.46692E+00_JPRB, 0.46597E+00_JPRB ,&
&  0.46692E+00_JPRB, 0.46973E+00_JPRB, 0.47133E+00_JPRB, 0.47342E+00_JPRB, 0.47676E+00_JPRB, 0.48006E+00_JPRB, 0.47971E+00_JPRB ,&
&  0.47799E+00_JPRB, 0.47749E+00_JPRB, 0.47969E+00_JPRB, 0.48647E+00_JPRB, 0.49512E+00_JPRB, 0.50367E+00_JPRB, 0.51188E+00_JPRB ,&
&  0.51860E+00_JPRB, 0.52283E+00_JPRB, 0.52392E+00_JPRB, 0.52363E+00_JPRB, 0.52078E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.47826E+00_JPRB, 0.47665E+00_JPRB, 0.47458E+00_JPRB, 0.47124E+00_JPRB, 0.46754E+00_JPRB, 0.46409E+00_JPRB, 0.46112E+00_JPRB ,&
&  0.45907E+00_JPRB, 0.45844E+00_JPRB, 0.45888E+00_JPRB, 0.45900E+00_JPRB, 0.45877E+00_JPRB, 0.45950E+00_JPRB, 0.45977E+00_JPRB ,&
&  0.46128E+00_JPRB, 0.46139E+00_JPRB, 0.45871E+00_JPRB, 0.45796E+00_JPRB, 0.46038E+00_JPRB, 0.46507E+00_JPRB, 0.46666E+00_JPRB ,&
&  0.46411E+00_JPRB, 0.46350E+00_JPRB, 0.46282E+00_JPRB, 0.46027E+00_JPRB, 0.45867E+00_JPRB, 0.45825E+00_JPRB, 0.45517E+00_JPRB ,&
&  0.45062E+00_JPRB, 0.45468E+00_JPRB, 0.45680E+00_JPRB, 0.46531E+00_JPRB, 0.46933E+00_JPRB, 0.47053E+00_JPRB, 0.47118E+00_JPRB ,&
&  0.46855E+00_JPRB, 0.46677E+00_JPRB, 0.46532E+00_JPRB, 0.46495E+00_JPRB, 0.46247E+00_JPRB, 0.45878E+00_JPRB, 0.45207E+00_JPRB ,&
&  0.44022E+00_JPRB, 0.43506E+00_JPRB, 0.43624E+00_JPRB, 0.44039E+00_JPRB, 0.43963E+00_JPRB, 0.44005E+00_JPRB, 0.44311E+00_JPRB ,&
&  0.44719E+00_JPRB, 0.45032E+00_JPRB, 0.45226E+00_JPRB, 0.45345E+00_JPRB, 0.45500E+00_JPRB, 0.45645E+00_JPRB, 0.45723E+00_JPRB ,&
&  0.45830E+00_JPRB, 0.46038E+00_JPRB, 0.46315E+00_JPRB, 0.46638E+00_JPRB, 0.46963E+00_JPRB, 0.47456E+00_JPRB, 0.47930E+00_JPRB ,&
&  0.48146E+00_JPRB, 0.48007E+00_JPRB, 0.47727E+00_JPRB, 0.47199E+00_JPRB, 0.46547E+00_JPRB, 0.46048E+00_JPRB, 0.45677E+00_JPRB ,&
&  0.45102E+00_JPRB, 0.44610E+00_JPRB, 0.44245E+00_JPRB, 0.44116E+00_JPRB, 0.44101E+00_JPRB, 0.44191E+00_JPRB, 0.44378E+00_JPRB ,&
&  0.44540E+00_JPRB, 0.44849E+00_JPRB, 0.45512E+00_JPRB, 0.45951E+00_JPRB, 0.46166E+00_JPRB, 0.46479E+00_JPRB, 0.46608E+00_JPRB ,&
&  0.46405E+00_JPRB, 0.46218E+00_JPRB, 0.46091E+00_JPRB, 0.46307E+00_JPRB, 0.46605E+00_JPRB, 0.46782E+00_JPRB, 0.46961E+00_JPRB ,&
&  0.47340E+00_JPRB, 0.47730E+00_JPRB, 0.47849E+00_JPRB, 0.47891E+00_JPRB, 0.47936E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.44198E+00_JPRB, 0.44238E+00_JPRB, 0.44190E+00_JPRB, 0.44081E+00_JPRB, 0.43963E+00_JPRB, 0.43866E+00_JPRB, 0.43873E+00_JPRB ,&
&  0.43892E+00_JPRB, 0.43962E+00_JPRB, 0.44063E+00_JPRB, 0.44145E+00_JPRB, 0.44320E+00_JPRB, 0.44526E+00_JPRB, 0.44699E+00_JPRB ,&
&  0.44876E+00_JPRB, 0.44791E+00_JPRB, 0.44455E+00_JPRB, 0.44151E+00_JPRB, 0.44223E+00_JPRB, 0.44780E+00_JPRB, 0.45388E+00_JPRB ,&
&  0.45525E+00_JPRB, 0.45611E+00_JPRB, 0.45625E+00_JPRB, 0.45478E+00_JPRB, 0.45314E+00_JPRB, 0.45311E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.44566E+00_JPRB, 0.44579E+00_JPRB, 0.44536E+00_JPRB, 0.44523E+00_JPRB, 0.44784E+00_JPRB, 0.45347E+00_JPRB, 0.45524E+00_JPRB ,&
&  0.45557E+00_JPRB, 0.45357E+00_JPRB, 0.44947E+00_JPRB, 0.44514E+00_JPRB, 0.43981E+00_JPRB, 0.43099E+00_JPRB, 0.42100E+00_JPRB ,&
&  0.41295E+00_JPRB, 0.41216E+00_JPRB, 0.41755E+00_JPRB, 0.42320E+00_JPRB, 0.42177E+00_JPRB, 0.42023E+00_JPRB, 0.42160E+00_JPRB ,&
&  0.42528E+00_JPRB, 0.43011E+00_JPRB, 0.43324E+00_JPRB, 0.43256E+00_JPRB, 0.43138E+00_JPRB, 0.43328E+00_JPRB, 0.43600E+00_JPRB ,&
&  0.43724E+00_JPRB, 0.43861E+00_JPRB, 0.43962E+00_JPRB, 0.44118E+00_JPRB, 0.44205E+00_JPRB, 0.44153E+00_JPRB, 0.44202E+00_JPRB ,&
&  0.44091E+00_JPRB, 0.43826E+00_JPRB, 0.43381E+00_JPRB, 0.42804E+00_JPRB, 0.42339E+00_JPRB, 0.42019E+00_JPRB, 0.41956E+00_JPRB ,&
&  0.41837E+00_JPRB, 0.41939E+00_JPRB, 0.41951E+00_JPRB, 0.41730E+00_JPRB, 0.41443E+00_JPRB, 0.41457E+00_JPRB, 0.41825E+00_JPRB ,&
&  0.42316E+00_JPRB, 0.42703E+00_JPRB, 0.43315E+00_JPRB, 0.43614E+00_JPRB, 0.43824E+00_JPRB, 0.44082E+00_JPRB, 0.44149E+00_JPRB ,&
&  0.43873E+00_JPRB, 0.43638E+00_JPRB, 0.43572E+00_JPRB, 0.43842E+00_JPRB, 0.43944E+00_JPRB, 0.43897E+00_JPRB, 0.43662E+00_JPRB ,&
&  0.43611E+00_JPRB, 0.43825E+00_JPRB, 0.43921E+00_JPRB, 0.43924E+00_JPRB, 0.44024E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.41483E+00_JPRB, 0.41729E+00_JPRB, 0.41982E+00_JPRB, 0.42206E+00_JPRB, 0.42348E+00_JPRB, 0.42448E+00_JPRB, 0.42362E+00_JPRB ,&
&  0.42347E+00_JPRB, 0.42517E+00_JPRB, 0.42740E+00_JPRB, 0.42927E+00_JPRB, 0.43047E+00_JPRB, 0.43263E+00_JPRB, 0.43482E+00_JPRB ,&
&  0.43627E+00_JPRB, 0.43517E+00_JPRB, 0.43103E+00_JPRB, 0.42885E+00_JPRB, 0.42766E+00_JPRB, 0.43297E+00_JPRB, 0.44185E+00_JPRB ,&
&  0.44584E+00_JPRB, 0.44818E+00_JPRB, 0.45098E+00_JPRB, 0.45143E+00_JPRB, 0.45173E+00_JPRB, 0.45191E+00_JPRB, 0.44774E+00_JPRB ,&
&  0.44280E+00_JPRB, 0.43890E+00_JPRB, 0.43570E+00_JPRB, 0.43285E+00_JPRB, 0.43409E+00_JPRB, 0.43362E+00_JPRB, 0.43356E+00_JPRB ,&
&  0.43344E+00_JPRB, 0.43107E+00_JPRB, 0.42682E+00_JPRB, 0.41997E+00_JPRB, 0.41166E+00_JPRB, 0.40105E+00_JPRB, 0.39346E+00_JPRB ,&
&  0.39014E+00_JPRB, 0.39376E+00_JPRB, 0.40257E+00_JPRB, 0.40634E+00_JPRB, 0.40256E+00_JPRB, 0.40123E+00_JPRB, 0.40427E+00_JPRB ,&
&  0.40921E+00_JPRB, 0.41483E+00_JPRB, 0.41705E+00_JPRB, 0.41625E+00_JPRB, 0.41420E+00_JPRB, 0.41585E+00_JPRB, 0.42031E+00_JPRB ,&
&  0.42414E+00_JPRB, 0.42603E+00_JPRB, 0.42509E+00_JPRB, 0.42398E+00_JPRB, 0.42127E+00_JPRB, 0.41561E+00_JPRB, 0.40813E+00_JPRB ,&
&  0.40129E+00_JPRB, 0.39705E+00_JPRB, 0.39376E+00_JPRB, 0.39054E+00_JPRB, 0.38933E+00_JPRB, 0.38975E+00_JPRB, 0.39206E+00_JPRB ,&
&  0.39301E+00_JPRB, 0.39491E+00_JPRB, 0.39513E+00_JPRB, 0.39103E+00_JPRB, 0.38778E+00_JPRB, 0.38928E+00_JPRB, 0.39092E+00_JPRB ,&
&  0.39363E+00_JPRB, 0.39643E+00_JPRB, 0.39921E+00_JPRB, 0.40026E+00_JPRB, 0.40187E+00_JPRB, 0.40475E+00_JPRB, 0.40654E+00_JPRB ,&
&  0.40747E+00_JPRB, 0.40813E+00_JPRB, 0.41005E+00_JPRB, 0.41319E+00_JPRB, 0.41615E+00_JPRB, 0.41855E+00_JPRB, 0.41803E+00_JPRB ,&
&  0.41689E+00_JPRB, 0.41547E+00_JPRB, 0.41589E+00_JPRB, 0.41614E+00_JPRB, 0.41466E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.40769E+00_JPRB, 0.40758E+00_JPRB, 0.41100E+00_JPRB, 0.41465E+00_JPRB, 0.41542E+00_JPRB, 0.41498E+00_JPRB, 0.41228E+00_JPRB ,&
&  0.41047E+00_JPRB, 0.41163E+00_JPRB, 0.41349E+00_JPRB, 0.41490E+00_JPRB, 0.41731E+00_JPRB, 0.42052E+00_JPRB, 0.42298E+00_JPRB ,&
&  0.42309E+00_JPRB, 0.42100E+00_JPRB, 0.41744E+00_JPRB, 0.41746E+00_JPRB, 0.41720E+00_JPRB, 0.41949E+00_JPRB, 0.42736E+00_JPRB ,&
&  0.43309E+00_JPRB, 0.43697E+00_JPRB, 0.44021E+00_JPRB, 0.44066E+00_JPRB, 0.44022E+00_JPRB, 0.44217E+00_JPRB, 0.43885E+00_JPRB ,&
&  0.43257E+00_JPRB, 0.42716E+00_JPRB, 0.42137E+00_JPRB, 0.41417E+00_JPRB, 0.41723E+00_JPRB, 0.41531E+00_JPRB, 0.41274E+00_JPRB ,&
&  0.40857E+00_JPRB, 0.40274E+00_JPRB, 0.39566E+00_JPRB, 0.38888E+00_JPRB, 0.38372E+00_JPRB, 0.37891E+00_JPRB, 0.37296E+00_JPRB ,&
&  0.37086E+00_JPRB, 0.37326E+00_JPRB, 0.37919E+00_JPRB, 0.38132E+00_JPRB, 0.37818E+00_JPRB, 0.37869E+00_JPRB, 0.38534E+00_JPRB ,&
&  0.39251E+00_JPRB, 0.39606E+00_JPRB, 0.39601E+00_JPRB, 0.39651E+00_JPRB, 0.39715E+00_JPRB, 0.39864E+00_JPRB, 0.39940E+00_JPRB ,&
&  0.39829E+00_JPRB, 0.39647E+00_JPRB, 0.39370E+00_JPRB, 0.38757E+00_JPRB, 0.38036E+00_JPRB, 0.37383E+00_JPRB, 0.37125E+00_JPRB ,&
&  0.37385E+00_JPRB, 0.37410E+00_JPRB, 0.37506E+00_JPRB, 0.37715E+00_JPRB, 0.37785E+00_JPRB, 0.37726E+00_JPRB, 0.37644E+00_JPRB ,&
&  0.37800E+00_JPRB, 0.37671E+00_JPRB, 0.37763E+00_JPRB, 0.37475E+00_JPRB, 0.37016E+00_JPRB, 0.36535E+00_JPRB, 0.36073E+00_JPRB ,&
&  0.35814E+00_JPRB, 0.35750E+00_JPRB, 0.35955E+00_JPRB, 0.36038E+00_JPRB, 0.36252E+00_JPRB, 0.36502E+00_JPRB, 0.36717E+00_JPRB ,&
&  0.36849E+00_JPRB, 0.37025E+00_JPRB, 0.37352E+00_JPRB, 0.37971E+00_JPRB, 0.38612E+00_JPRB, 0.39163E+00_JPRB, 0.39710E+00_JPRB ,&
&  0.40200E+00_JPRB, 0.40537E+00_JPRB, 0.41119E+00_JPRB, 0.41219E+00_JPRB, 0.41112E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.40594E+00_JPRB, 0.40311E+00_JPRB, 0.40296E+00_JPRB, 0.40362E+00_JPRB, 0.40275E+00_JPRB, 0.40153E+00_JPRB, 0.39857E+00_JPRB ,&
&  0.39607E+00_JPRB, 0.39645E+00_JPRB, 0.39787E+00_JPRB, 0.40057E+00_JPRB, 0.40526E+00_JPRB, 0.41023E+00_JPRB, 0.40976E+00_JPRB ,&
&  0.40666E+00_JPRB, 0.40536E+00_JPRB, 0.40752E+00_JPRB, 0.40928E+00_JPRB, 0.40936E+00_JPRB, 0.41007E+00_JPRB, 0.41397E+00_JPRB ,&
&  0.41881E+00_JPRB, 0.42107E+00_JPRB, 0.42414E+00_JPRB, 0.42380E+00_JPRB, 0.42012E+00_JPRB, 0.42074E+00_JPRB, 0.41898E+00_JPRB ,&
&  0.41402E+00_JPRB, 0.40781E+00_JPRB, 0.39826E+00_JPRB, 0.39891E+00_JPRB, 0.39524E+00_JPRB, 0.39215E+00_JPRB, 0.38741E+00_JPRB ,&
&  0.38210E+00_JPRB, 0.37746E+00_JPRB, 0.37234E+00_JPRB, 0.36823E+00_JPRB, 0.36625E+00_JPRB, 0.36608E+00_JPRB, 0.36376E+00_JPRB ,&
&  0.36244E+00_JPRB, 0.36472E+00_JPRB, 0.36610E+00_JPRB, 0.36229E+00_JPRB, 0.36143E+00_JPRB, 0.36752E+00_JPRB, 0.37280E+00_JPRB ,&
&  0.37517E+00_JPRB, 0.37530E+00_JPRB, 0.37248E+00_JPRB, 0.37168E+00_JPRB, 0.37065E+00_JPRB, 0.36867E+00_JPRB, 0.36512E+00_JPRB ,&
&  0.35957E+00_JPRB, 0.35675E+00_JPRB, 0.35478E+00_JPRB, 0.35092E+00_JPRB, 0.34383E+00_JPRB, 0.32763E+00_JPRB, 0.31875E+00_JPRB ,&
&  0.31972E+00_JPRB, 0.32894E+00_JPRB, 0.34300E+00_JPRB, 0.35069E+00_JPRB, 0.35808E+00_JPRB, 0.36357E+00_JPRB, 0.36391E+00_JPRB ,&
&  0.36352E+00_JPRB, 0.34392E+00_JPRB, 0.33303E+00_JPRB, 0.32355E+00_JPRB, 0.31500E+00_JPRB, 0.30981E+00_JPRB, 0.30784E+00_JPRB ,&
&  0.31063E+00_JPRB, 0.31623E+00_JPRB, 0.32178E+00_JPRB, 0.32481E+00_JPRB, 0.32489E+00_JPRB, 0.32244E+00_JPRB, 0.31944E+00_JPRB ,&
&  0.32001E+00_JPRB, 0.32787E+00_JPRB, 0.34230E+00_JPRB, 0.35510E+00_JPRB, 0.36500E+00_JPRB, 0.37564E+00_JPRB, 0.38436E+00_JPRB ,&
&  0.39042E+00_JPRB, 0.39612E+00_JPRB, 0.40283E+00_JPRB, 0.40516E+00_JPRB, 0.40600E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.39269E+00_JPRB, 0.38856E+00_JPRB, 0.38834E+00_JPRB, 0.38846E+00_JPRB, 0.38698E+00_JPRB, 0.38647E+00_JPRB, 0.38539E+00_JPRB ,&
&  0.38481E+00_JPRB, 0.38283E+00_JPRB, 0.38104E+00_JPRB, 0.38164E+00_JPRB, 0.38392E+00_JPRB, 0.38645E+00_JPRB, 0.38714E+00_JPRB ,&
&  0.38705E+00_JPRB, 0.38858E+00_JPRB, 0.39211E+00_JPRB, 0.39561E+00_JPRB, 0.39614E+00_JPRB, 0.39641E+00_JPRB, 0.39832E+00_JPRB ,&
&  0.40069E+00_JPRB, 0.40157E+00_JPRB, 0.40449E+00_JPRB, 0.40226E+00_JPRB, 0.39777E+00_JPRB, 0.39634E+00_JPRB, 0.39501E+00_JPRB ,&
&  0.38794E+00_JPRB, 0.36366E+00_JPRB, 0.34827E+00_JPRB, 0.35365E+00_JPRB, 0.36884E+00_JPRB, 0.36765E+00_JPRB, 0.36530E+00_JPRB ,&
&  0.36287E+00_JPRB, 0.35954E+00_JPRB, 0.35676E+00_JPRB, 0.35574E+00_JPRB, 0.35609E+00_JPRB, 0.35623E+00_JPRB, 0.35498E+00_JPRB ,&
&  0.35570E+00_JPRB, 0.36133E+00_JPRB, 0.36755E+00_JPRB, 0.36359E+00_JPRB, 0.35811E+00_JPRB, 0.35039E+00_JPRB, 0.34548E+00_JPRB ,&
&  0.34091E+00_JPRB, 0.33642E+00_JPRB, 0.33340E+00_JPRB, 0.33248E+00_JPRB, 0.33220E+00_JPRB, 0.32990E+00_JPRB, 0.32597E+00_JPRB ,&
&  0.31935E+00_JPRB, 0.30973E+00_JPRB, 0.30019E+00_JPRB, 0.29078E+00_JPRB, 0.28439E+00_JPRB, 0.27970E+00_JPRB, 0.27586E+00_JPRB ,&
&  0.27815E+00_JPRB, 0.29123E+00_JPRB, 0.31152E+00_JPRB, 0.33208E+00_JPRB, 0.34828E+00_JPRB, 0.32712E+00_JPRB, 0.31721E+00_JPRB ,&
&  0.30543E+00_JPRB, 0.29502E+00_JPRB, 0.28765E+00_JPRB, 0.28241E+00_JPRB, 0.27849E+00_JPRB, 0.27612E+00_JPRB, 0.27572E+00_JPRB ,&
&  0.27887E+00_JPRB, 0.28324E+00_JPRB, 0.28725E+00_JPRB, 0.28861E+00_JPRB, 0.28831E+00_JPRB, 0.28663E+00_JPRB, 0.28572E+00_JPRB ,&
&  0.28738E+00_JPRB, 0.29379E+00_JPRB, 0.30399E+00_JPRB, 0.31885E+00_JPRB, 0.33185E+00_JPRB, 0.34513E+00_JPRB, 0.36002E+00_JPRB ,&
&  0.37536E+00_JPRB, 0.38846E+00_JPRB, 0.39203E+00_JPRB, 0.39550E+00_JPRB, 0.39580E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.38416E+00_JPRB, 0.38141E+00_JPRB, 0.37932E+00_JPRB, 0.37951E+00_JPRB, 0.37948E+00_JPRB, 0.38166E+00_JPRB, 0.38040E+00_JPRB ,&
&  0.37581E+00_JPRB, 0.37116E+00_JPRB, 0.36990E+00_JPRB, 0.37006E+00_JPRB, 0.36901E+00_JPRB, 0.36750E+00_JPRB, 0.36834E+00_JPRB ,&
&  0.36676E+00_JPRB, 0.36898E+00_JPRB, 0.37217E+00_JPRB, 0.37389E+00_JPRB, 0.37456E+00_JPRB, 0.37470E+00_JPRB, 0.37503E+00_JPRB ,&
&  0.36881E+00_JPRB, 0.36677E+00_JPRB, 0.37254E+00_JPRB, 0.36960E+00_JPRB, 0.36552E+00_JPRB, 0.36205E+00_JPRB, 0.36093E+00_JPRB ,&
&  0.35189E+00_JPRB, 0.33934E+00_JPRB, 0.32680E+00_JPRB, 0.33383E+00_JPRB, 0.35076E+00_JPRB, 0.34716E+00_JPRB, 0.34485E+00_JPRB ,&
&  0.34470E+00_JPRB, 0.34405E+00_JPRB, 0.34505E+00_JPRB, 0.34570E+00_JPRB, 0.34703E+00_JPRB, 0.34742E+00_JPRB, 0.34909E+00_JPRB ,&
&  0.35084E+00_JPRB, 0.35781E+00_JPRB, 0.36099E+00_JPRB, 0.33982E+00_JPRB, 0.32237E+00_JPRB, 0.30910E+00_JPRB, 0.29620E+00_JPRB ,&
&  0.28469E+00_JPRB, 0.28312E+00_JPRB, 0.27922E+00_JPRB, 0.27152E+00_JPRB, 0.27185E+00_JPRB, 0.27268E+00_JPRB, 0.27169E+00_JPRB ,&
&  0.26921E+00_JPRB, 0.26772E+00_JPRB, 0.26628E+00_JPRB, 0.26383E+00_JPRB, 0.26302E+00_JPRB, 0.26453E+00_JPRB, 0.26871E+00_JPRB ,&
&  0.27511E+00_JPRB, 0.28184E+00_JPRB, 0.29722E+00_JPRB, 0.31000E+00_JPRB, 0.30515E+00_JPRB, 0.29875E+00_JPRB, 0.29193E+00_JPRB ,&
&  0.27987E+00_JPRB, 0.27404E+00_JPRB, 0.26817E+00_JPRB, 0.26338E+00_JPRB, 0.26112E+00_JPRB, 0.26117E+00_JPRB, 0.26178E+00_JPRB ,&
&  0.26290E+00_JPRB, 0.26512E+00_JPRB, 0.26599E+00_JPRB, 0.26567E+00_JPRB, 0.26493E+00_JPRB, 0.26573E+00_JPRB, 0.26777E+00_JPRB ,&
&  0.27131E+00_JPRB, 0.27734E+00_JPRB, 0.28450E+00_JPRB, 0.29340E+00_JPRB, 0.30119E+00_JPRB, 0.30847E+00_JPRB, 0.31999E+00_JPRB ,&
&  0.32835E+00_JPRB, 0.34828E+00_JPRB, 0.37486E+00_JPRB, 0.39706E+00_JPRB, 0.38943E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.36341E+00_JPRB, 0.36102E+00_JPRB, 0.35831E+00_JPRB, 0.35785E+00_JPRB, 0.35861E+00_JPRB, 0.36107E+00_JPRB, 0.35845E+00_JPRB ,&
&  0.35359E+00_JPRB, 0.35168E+00_JPRB, 0.35291E+00_JPRB, 0.35232E+00_JPRB, 0.35207E+00_JPRB, 0.34233E+00_JPRB, 0.33680E+00_JPRB ,&
&  0.32982E+00_JPRB, 0.32784E+00_JPRB, 0.32618E+00_JPRB, 0.32719E+00_JPRB, 0.32928E+00_JPRB, 0.33052E+00_JPRB, 0.32613E+00_JPRB ,&
&  0.32161E+00_JPRB, 0.31748E+00_JPRB, 0.31299E+00_JPRB, 0.31107E+00_JPRB, 0.31242E+00_JPRB, 0.31364E+00_JPRB, 0.31364E+00_JPRB ,&
&  0.31442E+00_JPRB, 0.31749E+00_JPRB, 0.32043E+00_JPRB, 0.32433E+00_JPRB, 0.33716E+00_JPRB, 0.33464E+00_JPRB, 0.33339E+00_JPRB ,&
&  0.33400E+00_JPRB, 0.33472E+00_JPRB, 0.33613E+00_JPRB, 0.33718E+00_JPRB, 0.34205E+00_JPRB, 0.34469E+00_JPRB, 0.34676E+00_JPRB ,&
&  0.33344E+00_JPRB, 0.32671E+00_JPRB, 0.31945E+00_JPRB, 0.31010E+00_JPRB, 0.29770E+00_JPRB, 0.28794E+00_JPRB, 0.27907E+00_JPRB ,&
&  0.27161E+00_JPRB, 0.26454E+00_JPRB, 0.25787E+00_JPRB, 0.25530E+00_JPRB, 0.25432E+00_JPRB, 0.25275E+00_JPRB, 0.25251E+00_JPRB ,&
&  0.25306E+00_JPRB, 0.25388E+00_JPRB, 0.25437E+00_JPRB, 0.25604E+00_JPRB, 0.25875E+00_JPRB, 0.26307E+00_JPRB, 0.26767E+00_JPRB ,&
&  0.27443E+00_JPRB, 0.27908E+00_JPRB, 0.28469E+00_JPRB, 0.28985E+00_JPRB, 0.28839E+00_JPRB, 0.28291E+00_JPRB, 0.27721E+00_JPRB ,&
&  0.26981E+00_JPRB, 0.26554E+00_JPRB, 0.26112E+00_JPRB, 0.25788E+00_JPRB, 0.25597E+00_JPRB, 0.25475E+00_JPRB, 0.25485E+00_JPRB ,&
&  0.25470E+00_JPRB, 0.25506E+00_JPRB, 0.25440E+00_JPRB, 0.25466E+00_JPRB, 0.25560E+00_JPRB, 0.25773E+00_JPRB, 0.26054E+00_JPRB ,&
&  0.26489E+00_JPRB, 0.26992E+00_JPRB, 0.27577E+00_JPRB, 0.28208E+00_JPRB, 0.29150E+00_JPRB, 0.30349E+00_JPRB, 0.32019E+00_JPRB ,&
&  0.33647E+00_JPRB, 0.36424E+00_JPRB, 0.37505E+00_JPRB, 0.36947E+00_JPRB, 0.36725E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.34649E+00_JPRB, 0.34789E+00_JPRB, 0.34884E+00_JPRB, 0.35104E+00_JPRB, 0.35368E+00_JPRB, 0.35452E+00_JPRB, 0.35467E+00_JPRB ,&
&  0.34525E+00_JPRB, 0.33796E+00_JPRB, 0.33115E+00_JPRB, 0.32102E+00_JPRB, 0.31298E+00_JPRB, 0.30591E+00_JPRB, 0.30085E+00_JPRB ,&
&  0.29546E+00_JPRB, 0.29295E+00_JPRB, 0.28979E+00_JPRB, 0.28886E+00_JPRB, 0.28830E+00_JPRB, 0.28874E+00_JPRB, 0.28703E+00_JPRB ,&
&  0.28523E+00_JPRB, 0.28661E+00_JPRB, 0.28741E+00_JPRB, 0.29188E+00_JPRB, 0.29860E+00_JPRB, 0.30613E+00_JPRB, 0.31353E+00_JPRB ,&
&  0.32298E+00_JPRB, 0.32686E+00_JPRB, 0.33009E+00_JPRB, 0.33090E+00_JPRB, 0.32988E+00_JPRB, 0.32967E+00_JPRB, 0.32790E+00_JPRB ,&
&  0.32806E+00_JPRB, 0.32800E+00_JPRB, 0.32988E+00_JPRB, 0.33121E+00_JPRB, 0.32958E+00_JPRB, 0.32780E+00_JPRB, 0.32286E+00_JPRB ,&
&  0.31503E+00_JPRB, 0.30883E+00_JPRB, 0.30230E+00_JPRB, 0.29711E+00_JPRB, 0.28923E+00_JPRB, 0.28238E+00_JPRB, 0.27573E+00_JPRB ,&
&  0.27006E+00_JPRB, 0.26591E+00_JPRB, 0.26083E+00_JPRB, 0.25680E+00_JPRB, 0.25490E+00_JPRB, 0.25253E+00_JPRB, 0.25149E+00_JPRB ,&
&  0.25145E+00_JPRB, 0.25191E+00_JPRB, 0.25438E+00_JPRB, 0.25635E+00_JPRB, 0.25942E+00_JPRB, 0.26300E+00_JPRB, 0.26506E+00_JPRB ,&
&  0.26883E+00_JPRB, 0.27209E+00_JPRB, 0.27390E+00_JPRB, 0.27531E+00_JPRB, 0.27437E+00_JPRB, 0.27105E+00_JPRB, 0.26780E+00_JPRB ,&
&  0.26335E+00_JPRB, 0.26027E+00_JPRB, 0.25828E+00_JPRB, 0.25606E+00_JPRB, 0.25484E+00_JPRB, 0.25374E+00_JPRB, 0.25248E+00_JPRB ,&
&  0.25150E+00_JPRB, 0.25101E+00_JPRB, 0.25056E+00_JPRB, 0.25176E+00_JPRB, 0.25302E+00_JPRB, 0.25539E+00_JPRB, 0.25824E+00_JPRB ,&
&  0.26080E+00_JPRB, 0.26479E+00_JPRB, 0.26910E+00_JPRB, 0.27505E+00_JPRB, 0.28614E+00_JPRB, 0.29466E+00_JPRB, 0.31343E+00_JPRB ,&
&  0.32492E+00_JPRB, 0.34660E+00_JPRB, 0.35600E+00_JPRB, 0.34677E+00_JPRB, 0.34633E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.33523E+00_JPRB, 0.33750E+00_JPRB, 0.33814E+00_JPRB, 0.34004E+00_JPRB, 0.34173E+00_JPRB, 0.34226E+00_JPRB, 0.34198E+00_JPRB ,&
&  0.33977E+00_JPRB, 0.33428E+00_JPRB, 0.32977E+00_JPRB, 0.32314E+00_JPRB, 0.31702E+00_JPRB, 0.31026E+00_JPRB, 0.30436E+00_JPRB ,&
&  0.29792E+00_JPRB, 0.29201E+00_JPRB, 0.28591E+00_JPRB, 0.28073E+00_JPRB, 0.27689E+00_JPRB, 0.27372E+00_JPRB, 0.27054E+00_JPRB ,&
&  0.26781E+00_JPRB, 0.26926E+00_JPRB, 0.27219E+00_JPRB, 0.27762E+00_JPRB, 0.28596E+00_JPRB, 0.29339E+00_JPRB, 0.30158E+00_JPRB ,&
&  0.31528E+00_JPRB, 0.31808E+00_JPRB, 0.31821E+00_JPRB, 0.32060E+00_JPRB, 0.32025E+00_JPRB, 0.32089E+00_JPRB, 0.31132E+00_JPRB ,&
&  0.31498E+00_JPRB, 0.31731E+00_JPRB, 0.32583E+00_JPRB, 0.32872E+00_JPRB, 0.32295E+00_JPRB, 0.31894E+00_JPRB, 0.31119E+00_JPRB ,&
&  0.30545E+00_JPRB, 0.30073E+00_JPRB, 0.29759E+00_JPRB, 0.29578E+00_JPRB, 0.29035E+00_JPRB, 0.28634E+00_JPRB, 0.27960E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27074E+00_JPRB, 0.26637E+00_JPRB, 0.26373E+00_JPRB, 0.26154E+00_JPRB, 0.25934E+00_JPRB, 0.25765E+00_JPRB ,&
&  0.25672E+00_JPRB, 0.25594E+00_JPRB, 0.25671E+00_JPRB, 0.25757E+00_JPRB, 0.25925E+00_JPRB, 0.26059E+00_JPRB, 0.26081E+00_JPRB ,&
&  0.26108E+00_JPRB, 0.26160E+00_JPRB, 0.26185E+00_JPRB, 0.26272E+00_JPRB, 0.26220E+00_JPRB, 0.26116E+00_JPRB, 0.25988E+00_JPRB ,&
&  0.25843E+00_JPRB, 0.25781E+00_JPRB, 0.25670E+00_JPRB, 0.25600E+00_JPRB, 0.25383E+00_JPRB, 0.25235E+00_JPRB, 0.25154E+00_JPRB ,&
&  0.25078E+00_JPRB, 0.25126E+00_JPRB, 0.25137E+00_JPRB, 0.25230E+00_JPRB, 0.25375E+00_JPRB, 0.25497E+00_JPRB, 0.25724E+00_JPRB ,&
&  0.26007E+00_JPRB, 0.26336E+00_JPRB, 0.26919E+00_JPRB, 0.27512E+00_JPRB, 0.28168E+00_JPRB, 0.28935E+00_JPRB, 0.29541E+00_JPRB ,&
&  0.30425E+00_JPRB, 0.31798E+00_JPRB, 0.31989E+00_JPRB, 0.32594E+00_JPRB, 0.33167E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.31820E+00_JPRB, 0.32269E+00_JPRB, 0.32561E+00_JPRB, 0.32754E+00_JPRB, 0.32950E+00_JPRB, 0.32878E+00_JPRB, 0.32708E+00_JPRB ,&
&  0.32267E+00_JPRB, 0.32118E+00_JPRB, 0.31832E+00_JPRB, 0.31511E+00_JPRB, 0.31143E+00_JPRB, 0.30673E+00_JPRB, 0.30268E+00_JPRB ,&
&  0.29806E+00_JPRB, 0.29296E+00_JPRB, 0.28705E+00_JPRB, 0.28038E+00_JPRB, 0.27429E+00_JPRB, 0.26854E+00_JPRB, 0.26557E+00_JPRB ,&
&  0.26428E+00_JPRB, 0.26536E+00_JPRB, 0.26865E+00_JPRB, 0.27246E+00_JPRB, 0.27704E+00_JPRB, 0.28219E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.29396E+00_JPRB, 0.30031E+00_JPRB, 0.30413E+00_JPRB, 0.30570E+00_JPRB, 0.30431E+00_JPRB, 0.30437E+00_JPRB, 0.30353E+00_JPRB ,&
&  0.30242E+00_JPRB, 0.30082E+00_JPRB, 0.30054E+00_JPRB, 0.30119E+00_JPRB, 0.30086E+00_JPRB, 0.29940E+00_JPRB, 0.29698E+00_JPRB ,&
&  0.29457E+00_JPRB, 0.29250E+00_JPRB, 0.28960E+00_JPRB, 0.28713E+00_JPRB, 0.28529E+00_JPRB, 0.28288E+00_JPRB, 0.27955E+00_JPRB ,&
&  0.27661E+00_JPRB, 0.27372E+00_JPRB, 0.27033E+00_JPRB, 0.26755E+00_JPRB, 0.26479E+00_JPRB, 0.26279E+00_JPRB, 0.26099E+00_JPRB ,&
&  0.26027E+00_JPRB, 0.25948E+00_JPRB, 0.25928E+00_JPRB, 0.25918E+00_JPRB, 0.25968E+00_JPRB, 0.25932E+00_JPRB, 0.25880E+00_JPRB ,&
&  0.25818E+00_JPRB, 0.25802E+00_JPRB, 0.25739E+00_JPRB, 0.25717E+00_JPRB, 0.25692E+00_JPRB, 0.25606E+00_JPRB, 0.25559E+00_JPRB ,&
&  0.25553E+00_JPRB, 0.25530E+00_JPRB, 0.25451E+00_JPRB, 0.25434E+00_JPRB, 0.25383E+00_JPRB, 0.25312E+00_JPRB, 0.25340E+00_JPRB ,&
&  0.25315E+00_JPRB, 0.25334E+00_JPRB, 0.25365E+00_JPRB, 0.25413E+00_JPRB, 0.25530E+00_JPRB, 0.25675E+00_JPRB, 0.25874E+00_JPRB ,&
&  0.26187E+00_JPRB, 0.26468E+00_JPRB, 0.26746E+00_JPRB, 0.27067E+00_JPRB, 0.27710E+00_JPRB, 0.27948E+00_JPRB, 0.28393E+00_JPRB ,&
&  0.28771E+00_JPRB, 0.28376E+00_JPRB, 0.30721E+00_JPRB, 0.30850E+00_JPRB, 0.31147E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.27405E+00_JPRB, 0.27918E+00_JPRB, 0.28263E+00_JPRB, 0.28530E+00_JPRB, 0.28013E+00_JPRB, 0.30045E+00_JPRB, 0.29863E+00_JPRB ,&
&  0.29912E+00_JPRB, 0.29449E+00_JPRB, 0.29657E+00_JPRB, 0.29575E+00_JPRB, 0.29359E+00_JPRB, 0.29065E+00_JPRB, 0.28757E+00_JPRB ,&
&  0.28433E+00_JPRB, 0.28038E+00_JPRB, 0.27683E+00_JPRB, 0.27335E+00_JPRB, 0.27048E+00_JPRB, 0.26793E+00_JPRB, 0.26659E+00_JPRB ,&
&  0.26674E+00_JPRB, 0.26843E+00_JPRB, 0.27130E+00_JPRB, 0.27265E+00_JPRB, 0.27415E+00_JPRB, 0.27574E+00_JPRB, 0.27747E+00_JPRB ,&
&  0.27881E+00_JPRB, 0.28074E+00_JPRB, 0.28173E+00_JPRB, 0.28126E+00_JPRB, 0.28045E+00_JPRB, 0.28039E+00_JPRB, 0.28064E+00_JPRB ,&
&  0.28160E+00_JPRB, 0.28225E+00_JPRB, 0.28270E+00_JPRB, 0.28326E+00_JPRB, 0.28380E+00_JPRB, 0.28349E+00_JPRB, 0.28232E+00_JPRB ,&
&  0.28132E+00_JPRB, 0.28050E+00_JPRB, 0.27866E+00_JPRB, 0.27692E+00_JPRB, 0.27527E+00_JPRB, 0.27354E+00_JPRB, 0.27172E+00_JPRB ,&
&  0.27042E+00_JPRB, 0.26912E+00_JPRB, 0.26750E+00_JPRB, 0.26563E+00_JPRB, 0.26390E+00_JPRB, 0.26239E+00_JPRB, 0.26122E+00_JPRB ,&
&  0.26103E+00_JPRB, 0.26040E+00_JPRB, 0.25984E+00_JPRB, 0.25929E+00_JPRB, 0.25917E+00_JPRB, 0.25864E+00_JPRB, 0.25824E+00_JPRB ,&
&  0.25793E+00_JPRB, 0.25753E+00_JPRB, 0.25708E+00_JPRB, 0.25663E+00_JPRB, 0.25632E+00_JPRB, 0.25592E+00_JPRB, 0.25550E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25499E+00_JPRB, 0.25476E+00_JPRB, 0.25426E+00_JPRB, 0.25376E+00_JPRB, 0.25316E+00_JPRB, 0.25311E+00_JPRB ,&
&  0.25322E+00_JPRB, 0.25347E+00_JPRB, 0.25396E+00_JPRB, 0.25408E+00_JPRB, 0.25499E+00_JPRB, 0.25586E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25799E+00_JPRB, 0.25971E+00_JPRB, 0.26162E+00_JPRB, 0.26373E+00_JPRB, 0.26722E+00_JPRB, 0.26815E+00_JPRB, 0.26890E+00_JPRB ,&
&  0.26987E+00_JPRB, 0.27392E+00_JPRB, 0.27467E+00_JPRB, 0.27416E+00_JPRB, 0.27449E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.26366E+00_JPRB, 0.26392E+00_JPRB, 0.26386E+00_JPRB, 0.26361E+00_JPRB, 0.26122E+00_JPRB, 0.26018E+00_JPRB, 0.25928E+00_JPRB ,&
&  0.25861E+00_JPRB, 0.25828E+00_JPRB, 0.25875E+00_JPRB, 0.25937E+00_JPRB, 0.26001E+00_JPRB, 0.26069E+00_JPRB, 0.26129E+00_JPRB ,&
&  0.26161E+00_JPRB, 0.26196E+00_JPRB, 0.26075E+00_JPRB, 0.26016E+00_JPRB, 0.25996E+00_JPRB, 0.26000E+00_JPRB, 0.25997E+00_JPRB ,&
&  0.26007E+00_JPRB, 0.26014E+00_JPRB, 0.26011E+00_JPRB, 0.26052E+00_JPRB, 0.26121E+00_JPRB, 0.26178E+00_JPRB, 0.26236E+00_JPRB ,&
&  0.26294E+00_JPRB, 0.26379E+00_JPRB, 0.26457E+00_JPRB, 0.26531E+00_JPRB, 0.26632E+00_JPRB, 0.26734E+00_JPRB, 0.26799E+00_JPRB ,&
&  0.26851E+00_JPRB, 0.26874E+00_JPRB, 0.26903E+00_JPRB, 0.26927E+00_JPRB, 0.26963E+00_JPRB, 0.27005E+00_JPRB, 0.27017E+00_JPRB ,&
&  0.27014E+00_JPRB, 0.27001E+00_JPRB, 0.26979E+00_JPRB, 0.26919E+00_JPRB, 0.26854E+00_JPRB, 0.26792E+00_JPRB, 0.26729E+00_JPRB ,&
&  0.26675E+00_JPRB, 0.26613E+00_JPRB, 0.26544E+00_JPRB, 0.26472E+00_JPRB, 0.26396E+00_JPRB, 0.26323E+00_JPRB, 0.26248E+00_JPRB ,&
&  0.26171E+00_JPRB, 0.26122E+00_JPRB, 0.26068E+00_JPRB, 0.26021E+00_JPRB, 0.25984E+00_JPRB, 0.25945E+00_JPRB, 0.25908E+00_JPRB ,&
&  0.25871E+00_JPRB, 0.25827E+00_JPRB, 0.25787E+00_JPRB, 0.25743E+00_JPRB, 0.25703E+00_JPRB, 0.25658E+00_JPRB, 0.25621E+00_JPRB ,&
&  0.25586E+00_JPRB, 0.25554E+00_JPRB, 0.25528E+00_JPRB, 0.25515E+00_JPRB, 0.25506E+00_JPRB, 0.25507E+00_JPRB, 0.25509E+00_JPRB ,&
&  0.25513E+00_JPRB, 0.25514E+00_JPRB, 0.25514E+00_JPRB, 0.25495E+00_JPRB, 0.25516E+00_JPRB, 0.25538E+00_JPRB, 0.25571E+00_JPRB ,&
&  0.25601E+00_JPRB, 0.25654E+00_JPRB, 0.25709E+00_JPRB, 0.25771E+00_JPRB, 0.25871E+00_JPRB, 0.25906E+00_JPRB, 0.25936E+00_JPRB ,&
&  0.25967E+00_JPRB, 0.26037E+00_JPRB, 0.26094E+00_JPRB, 0.26124E+00_JPRB, 0.26188E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB, 0.25920E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB, 0.98674E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.95793E+00_JPRB, 0.95338E+00_JPRB, 0.94846E+00_JPRB, 0.94331E+00_JPRB, 0.93762E+00_JPRB, 0.93127E+00_JPRB, 0.92397E+00_JPRB ,&
&  0.91706E+00_JPRB, 0.91047E+00_JPRB, 0.90464E+00_JPRB, 0.89952E+00_JPRB, 0.89501E+00_JPRB, 0.89152E+00_JPRB, 0.89026E+00_JPRB ,&
&  0.89020E+00_JPRB, 0.89017E+00_JPRB, 0.89061E+00_JPRB, 0.89089E+00_JPRB, 0.89068E+00_JPRB, 0.89022E+00_JPRB, 0.88962E+00_JPRB ,&
&  0.88932E+00_JPRB, 0.88925E+00_JPRB, 0.88918E+00_JPRB, 0.88969E+00_JPRB, 0.89042E+00_JPRB, 0.89126E+00_JPRB, 0.89199E+00_JPRB ,&
&  0.89342E+00_JPRB, 0.89515E+00_JPRB, 0.89749E+00_JPRB, 0.90023E+00_JPRB, 0.90568E+00_JPRB, 0.91144E+00_JPRB, 0.91747E+00_JPRB ,&
&  0.92319E+00_JPRB, 0.92903E+00_JPRB, 0.93430E+00_JPRB, 0.93898E+00_JPRB, 0.94331E+00_JPRB, 0.94848E+00_JPRB, 0.95461E+00_JPRB ,&
&  0.96139E+00_JPRB, 0.96877E+00_JPRB, 0.97689E+00_JPRB, 0.98506E+00_JPRB, 0.99266E+00_JPRB, 0.10004E+01_JPRB, 0.10086E+01_JPRB ,&
&  0.10167E+01_JPRB, 0.10248E+01_JPRB, 0.10328E+01_JPRB, 0.10408E+01_JPRB, 0.10488E+01_JPRB, 0.10579E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10891E+01_JPRB, 0.10994E+01_JPRB, 0.11105E+01_JPRB, 0.11255E+01_JPRB, 0.11330E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11472E+01_JPRB, 0.11574E+01_JPRB, 0.11636E+01_JPRB, 0.11700E+01_JPRB, 0.11760E+01_JPRB, 0.11811E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.11865E+01_JPRB, 0.11887E+01_JPRB, 0.11885E+01_JPRB, 0.11868E+01_JPRB, 0.11836E+01_JPRB, 0.11800E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.11592E+01_JPRB, 0.11533E+01_JPRB, 0.11450E+01_JPRB, 0.11366E+01_JPRB, 0.11281E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10974E+01_JPRB, 0.10855E+01_JPRB, 0.10729E+01_JPRB, 0.10605E+01_JPRB, 0.10477E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.10045E+01_JPRB, 0.98891E+00_JPRB, 0.97357E+00_JPRB, 0.96231E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.93277E+00_JPRB, 0.92046E+00_JPRB, 0.91497E+00_JPRB, 0.91114E+00_JPRB, 0.90748E+00_JPRB, 0.90257E+00_JPRB, 0.89714E+00_JPRB ,&
&  0.89297E+00_JPRB, 0.88757E+00_JPRB, 0.88316E+00_JPRB, 0.88188E+00_JPRB, 0.87928E+00_JPRB, 0.87785E+00_JPRB, 0.87642E+00_JPRB ,&
&  0.87544E+00_JPRB, 0.87417E+00_JPRB, 0.87346E+00_JPRB, 0.87123E+00_JPRB, 0.86785E+00_JPRB, 0.86338E+00_JPRB, 0.85976E+00_JPRB ,&
&  0.85698E+00_JPRB, 0.85501E+00_JPRB, 0.85218E+00_JPRB, 0.84976E+00_JPRB, 0.85057E+00_JPRB, 0.85073E+00_JPRB, 0.85009E+00_JPRB ,&
&  0.85373E+00_JPRB, 0.85642E+00_JPRB, 0.85637E+00_JPRB, 0.85384E+00_JPRB, 0.85051E+00_JPRB, 0.84881E+00_JPRB, 0.85154E+00_JPRB ,&
&  0.85612E+00_JPRB, 0.87022E+00_JPRB, 0.88213E+00_JPRB, 0.89342E+00_JPRB, 0.90393E+00_JPRB, 0.91669E+00_JPRB, 0.93172E+00_JPRB ,&
&  0.94766E+00_JPRB, 0.96116E+00_JPRB, 0.97605E+00_JPRB, 0.99383E+00_JPRB, 0.10127E+01_JPRB, 0.10325E+01_JPRB, 0.10538E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10923E+01_JPRB, 0.11148E+01_JPRB, 0.11390E+01_JPRB, 0.11602E+01_JPRB, 0.11866E+01_JPRB, 0.12172E+01_JPRB ,&
&  0.12543E+01_JPRB, 0.12747E+01_JPRB, 0.12974E+01_JPRB, 0.13205E+01_JPRB, 0.13485E+01_JPRB, 0.13658E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.14024E+01_JPRB, 0.13996E+01_JPRB, 0.13942E+01_JPRB, 0.13906E+01_JPRB, 0.13847E+01_JPRB, 0.13780E+01_JPRB ,&
&  0.13724E+01_JPRB, 0.13692E+01_JPRB, 0.13604E+01_JPRB, 0.13500E+01_JPRB, 0.13361E+01_JPRB, 0.13217E+01_JPRB, 0.12970E+01_JPRB ,&
&  0.12740E+01_JPRB, 0.12562E+01_JPRB, 0.12438E+01_JPRB, 0.12288E+01_JPRB, 0.12142E+01_JPRB, 0.12007E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.11599E+01_JPRB, 0.11442E+01_JPRB, 0.11273E+01_JPRB, 0.11134E+01_JPRB, 0.10940E+01_JPRB, 0.10724E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.10269E+01_JPRB, 0.10030E+01_JPRB, 0.97539E+00_JPRB, 0.94882E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.94995E+00_JPRB, 0.92139E+00_JPRB, 0.90559E+00_JPRB, 0.90038E+00_JPRB, 0.89700E+00_JPRB, 0.89368E+00_JPRB, 0.89031E+00_JPRB ,&
&  0.88461E+00_JPRB, 0.87962E+00_JPRB, 0.87976E+00_JPRB, 0.88072E+00_JPRB, 0.87989E+00_JPRB, 0.87839E+00_JPRB, 0.87530E+00_JPRB ,&
&  0.87149E+00_JPRB, 0.86567E+00_JPRB, 0.85934E+00_JPRB, 0.85014E+00_JPRB, 0.84032E+00_JPRB, 0.82865E+00_JPRB, 0.81238E+00_JPRB ,&
&  0.79969E+00_JPRB, 0.78845E+00_JPRB, 0.77393E+00_JPRB, 0.76950E+00_JPRB, 0.76810E+00_JPRB, 0.73529E+00_JPRB, 0.72889E+00_JPRB ,&
&  0.74078E+00_JPRB, 0.74476E+00_JPRB, 0.74702E+00_JPRB, 0.74904E+00_JPRB, 0.74380E+00_JPRB, 0.72723E+00_JPRB, 0.72082E+00_JPRB ,&
&  0.69447E+00_JPRB, 0.71599E+00_JPRB, 0.73129E+00_JPRB, 0.74505E+00_JPRB, 0.75841E+00_JPRB, 0.77619E+00_JPRB, 0.81379E+00_JPRB ,&
&  0.88467E+00_JPRB, 0.91431E+00_JPRB, 0.94929E+00_JPRB, 0.98352E+00_JPRB, 0.10265E+01_JPRB, 0.10707E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.11488E+01_JPRB, 0.11895E+01_JPRB, 0.12368E+01_JPRB, 0.12952E+01_JPRB, 0.13450E+01_JPRB, 0.13903E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.14606E+01_JPRB, 0.14870E+01_JPRB, 0.15067E+01_JPRB, 0.15186E+01_JPRB, 0.15321E+01_JPRB, 0.15396E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.15535E+01_JPRB, 0.15545E+01_JPRB, 0.15439E+01_JPRB, 0.15229E+01_JPRB, 0.15093E+01_JPRB, 0.15042E+01_JPRB, 0.14947E+01_JPRB ,&
&  0.14828E+01_JPRB, 0.14693E+01_JPRB, 0.14515E+01_JPRB, 0.14298E+01_JPRB, 0.14006E+01_JPRB, 0.13748E+01_JPRB, 0.13432E+01_JPRB ,&
&  0.13165E+01_JPRB, 0.12946E+01_JPRB, 0.12763E+01_JPRB, 0.12552E+01_JPRB, 0.12332E+01_JPRB, 0.12114E+01_JPRB, 0.11871E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.11471E+01_JPRB, 0.11324E+01_JPRB, 0.11169E+01_JPRB, 0.11058E+01_JPRB, 0.10909E+01_JPRB, 0.10777E+01_JPRB ,&
&  0.10631E+01_JPRB, 0.10504E+01_JPRB, 0.10307E+01_JPRB, 0.10059E+01_JPRB, 0.97497E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.98527E+00_JPRB, 0.95596E+00_JPRB, 0.92280E+00_JPRB, 0.90331E+00_JPRB, 0.89678E+00_JPRB, 0.89011E+00_JPRB, 0.88389E+00_JPRB ,&
&  0.87977E+00_JPRB, 0.88087E+00_JPRB, 0.88293E+00_JPRB, 0.88254E+00_JPRB, 0.87956E+00_JPRB, 0.87425E+00_JPRB, 0.86707E+00_JPRB ,&
&  0.85757E+00_JPRB, 0.84605E+00_JPRB, 0.83634E+00_JPRB, 0.82668E+00_JPRB, 0.81613E+00_JPRB, 0.80560E+00_JPRB, 0.78889E+00_JPRB ,&
&  0.77386E+00_JPRB, 0.76023E+00_JPRB, 0.71282E+00_JPRB, 0.71219E+00_JPRB, 0.70398E+00_JPRB, 0.70263E+00_JPRB, 0.70772E+00_JPRB ,&
&  0.70408E+00_JPRB, 0.68949E+00_JPRB, 0.67336E+00_JPRB, 0.65260E+00_JPRB, 0.63756E+00_JPRB, 0.62092E+00_JPRB, 0.61415E+00_JPRB ,&
&  0.61252E+00_JPRB, 0.61779E+00_JPRB, 0.63073E+00_JPRB, 0.64471E+00_JPRB, 0.65973E+00_JPRB, 0.67281E+00_JPRB, 0.69540E+00_JPRB ,&
&  0.75141E+00_JPRB, 0.82740E+00_JPRB, 0.92198E+00_JPRB, 0.97499E+00_JPRB, 0.10403E+01_JPRB, 0.11017E+01_JPRB, 0.11499E+01_JPRB ,&
&  0.11987E+01_JPRB, 0.12503E+01_JPRB, 0.13076E+01_JPRB, 0.13690E+01_JPRB, 0.14319E+01_JPRB, 0.14871E+01_JPRB, 0.15536E+01_JPRB ,&
&  0.16093E+01_JPRB, 0.16590E+01_JPRB, 0.16859E+01_JPRB, 0.17015E+01_JPRB, 0.17051E+01_JPRB, 0.16926E+01_JPRB, 0.16782E+01_JPRB ,&
&  0.16681E+01_JPRB, 0.16542E+01_JPRB, 0.16354E+01_JPRB, 0.15925E+01_JPRB, 0.15720E+01_JPRB, 0.15647E+01_JPRB, 0.15412E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.14753E+01_JPRB, 0.14399E+01_JPRB, 0.13903E+01_JPRB, 0.13462E+01_JPRB, 0.13296E+01_JPRB, 0.13144E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.12854E+01_JPRB, 0.12698E+01_JPRB, 0.12450E+01_JPRB, 0.12081E+01_JPRB, 0.11727E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.10953E+01_JPRB, 0.10749E+01_JPRB, 0.10542E+01_JPRB, 0.10412E+01_JPRB, 0.10304E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.10269E+01_JPRB, 0.10253E+01_JPRB, 0.10204E+01_JPRB, 0.10061E+01_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.10778E+01_JPRB, 0.10649E+01_JPRB, 0.10359E+01_JPRB, 0.99710E+00_JPRB, 0.96510E+00_JPRB, 0.94062E+00_JPRB, 0.92709E+00_JPRB ,&
&  0.91786E+00_JPRB, 0.91304E+00_JPRB, 0.90705E+00_JPRB, 0.90084E+00_JPRB, 0.89167E+00_JPRB, 0.88138E+00_JPRB, 0.87026E+00_JPRB ,&
&  0.85343E+00_JPRB, 0.83660E+00_JPRB, 0.82051E+00_JPRB, 0.80986E+00_JPRB, 0.80107E+00_JPRB, 0.79446E+00_JPRB, 0.78819E+00_JPRB ,&
&  0.78181E+00_JPRB, 0.77391E+00_JPRB, 0.74827E+00_JPRB, 0.73978E+00_JPRB, 0.72955E+00_JPRB, 0.72918E+00_JPRB, 0.72351E+00_JPRB ,&
&  0.73588E+00_JPRB, 0.66169E+00_JPRB, 0.65674E+00_JPRB, 0.63898E+00_JPRB, 0.62558E+00_JPRB, 0.60981E+00_JPRB, 0.59365E+00_JPRB ,&
&  0.58595E+00_JPRB, 0.57542E+00_JPRB, 0.57250E+00_JPRB, 0.58429E+00_JPRB, 0.59484E+00_JPRB, 0.61601E+00_JPRB, 0.63776E+00_JPRB ,&
&  0.67203E+00_JPRB, 0.82848E+00_JPRB, 0.90826E+00_JPRB, 0.96614E+00_JPRB, 0.10273E+01_JPRB, 0.10803E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.11474E+01_JPRB, 0.11932E+01_JPRB, 0.12373E+01_JPRB, 0.12632E+01_JPRB, 0.13745E+01_JPRB, 0.14558E+01_JPRB, 0.15479E+01_JPRB ,&
&  0.16342E+01_JPRB, 0.17230E+01_JPRB, 0.18019E+01_JPRB, 0.18646E+01_JPRB, 0.19114E+01_JPRB, 0.19181E+01_JPRB, 0.18965E+01_JPRB ,&
&  0.18821E+01_JPRB, 0.18674E+01_JPRB, 0.18587E+01_JPRB, 0.18009E+01_JPRB, 0.17655E+01_JPRB, 0.17386E+01_JPRB, 0.16758E+01_JPRB ,&
&  0.16049E+01_JPRB, 0.15412E+01_JPRB, 0.14765E+01_JPRB, 0.14077E+01_JPRB, 0.13432E+01_JPRB, 0.13054E+01_JPRB, 0.12859E+01_JPRB ,&
&  0.12780E+01_JPRB, 0.12616E+01_JPRB, 0.12343E+01_JPRB, 0.11944E+01_JPRB, 0.11538E+01_JPRB, 0.11264E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10405E+01_JPRB, 0.10216E+01_JPRB, 0.10087E+01_JPRB, 0.10047E+01_JPRB, 0.10069E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.10363E+01_JPRB, 0.10501E+01_JPRB, 0.10642E+01_JPRB, 0.10751E+01_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.10792E+01_JPRB, 0.10863E+01_JPRB, 0.10853E+01_JPRB, 0.10753E+01_JPRB, 0.10565E+01_JPRB, 0.10330E+01_JPRB, 0.10099E+01_JPRB ,&
&  0.98800E+00_JPRB, 0.97057E+00_JPRB, 0.95329E+00_JPRB, 0.93895E+00_JPRB, 0.92208E+00_JPRB, 0.90927E+00_JPRB, 0.89500E+00_JPRB ,&
&  0.88525E+00_JPRB, 0.87962E+00_JPRB, 0.86806E+00_JPRB, 0.85507E+00_JPRB, 0.83486E+00_JPRB, 0.81753E+00_JPRB, 0.80391E+00_JPRB ,&
&  0.79148E+00_JPRB, 0.78100E+00_JPRB, 0.75656E+00_JPRB, 0.73960E+00_JPRB, 0.73042E+00_JPRB, 0.71891E+00_JPRB, 0.72432E+00_JPRB ,&
&  0.71877E+00_JPRB, 0.72171E+00_JPRB, 0.72508E+00_JPRB, 0.71780E+00_JPRB, 0.70795E+00_JPRB, 0.58627E+00_JPRB, 0.55710E+00_JPRB ,&
&  0.54958E+00_JPRB, 0.54161E+00_JPRB, 0.54086E+00_JPRB, 0.54891E+00_JPRB, 0.56260E+00_JPRB, 0.60072E+00_JPRB, 0.62924E+00_JPRB ,&
&  0.64490E+00_JPRB, 0.80841E+00_JPRB, 0.87683E+00_JPRB, 0.93646E+00_JPRB, 0.98341E+00_JPRB, 0.10174E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.10598E+01_JPRB, 0.10917E+01_JPRB, 0.11140E+01_JPRB, 0.11600E+01_JPRB, 0.12467E+01_JPRB, 0.13859E+01_JPRB, 0.15230E+01_JPRB ,&
&  0.16757E+01_JPRB, 0.17780E+01_JPRB, 0.18762E+01_JPRB, 0.19458E+01_JPRB, 0.20271E+01_JPRB, 0.20800E+01_JPRB, 0.21006E+01_JPRB ,&
&  0.21252E+01_JPRB, 0.21000E+01_JPRB, 0.21155E+01_JPRB, 0.20777E+01_JPRB, 0.20427E+01_JPRB, 0.19988E+01_JPRB, 0.18790E+01_JPRB ,&
&  0.17369E+01_JPRB, 0.16409E+01_JPRB, 0.15516E+01_JPRB, 0.14379E+01_JPRB, 0.13537E+01_JPRB, 0.12931E+01_JPRB, 0.12677E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.12391E+01_JPRB, 0.12002E+01_JPRB, 0.11569E+01_JPRB, 0.11279E+01_JPRB, 0.11082E+01_JPRB, 0.10783E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.10268E+01_JPRB, 0.10111E+01_JPRB, 0.99697E+00_JPRB, 0.98712E+00_JPRB, 0.98397E+00_JPRB, 0.98668E+00_JPRB ,&
&  0.99449E+00_JPRB, 0.10068E+01_JPRB, 0.10246E+01_JPRB, 0.10467E+01_JPRB, 0.10711E+01_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.10643E+01_JPRB, 0.10773E+01_JPRB, 0.10816E+01_JPRB, 0.10762E+01_JPRB, 0.10583E+01_JPRB, 0.10361E+01_JPRB, 0.10162E+01_JPRB ,&
&  0.99688E+00_JPRB, 0.97868E+00_JPRB, 0.96132E+00_JPRB, 0.94757E+00_JPRB, 0.93506E+00_JPRB, 0.92503E+00_JPRB, 0.91692E+00_JPRB ,&
&  0.91040E+00_JPRB, 0.88264E+00_JPRB, 0.87010E+00_JPRB, 0.86716E+00_JPRB, 0.86665E+00_JPRB, 0.86212E+00_JPRB, 0.85510E+00_JPRB ,&
&  0.82996E+00_JPRB, 0.79537E+00_JPRB, 0.76285E+00_JPRB, 0.73468E+00_JPRB, 0.71114E+00_JPRB, 0.69890E+00_JPRB, 0.70107E+00_JPRB ,&
&  0.70274E+00_JPRB, 0.69297E+00_JPRB, 0.69329E+00_JPRB, 0.71151E+00_JPRB, 0.71155E+00_JPRB, 0.64322E+00_JPRB, 0.55620E+00_JPRB ,&
&  0.54510E+00_JPRB, 0.52424E+00_JPRB, 0.52425E+00_JPRB, 0.53176E+00_JPRB, 0.54901E+00_JPRB, 0.57652E+00_JPRB, 0.62150E+00_JPRB ,&
&  0.67143E+00_JPRB, 0.78520E+00_JPRB, 0.85488E+00_JPRB, 0.90453E+00_JPRB, 0.94818E+00_JPRB, 0.98304E+00_JPRB, 0.99803E+00_JPRB ,&
&  0.10084E+01_JPRB, 0.10337E+01_JPRB, 0.10766E+01_JPRB, 0.11298E+01_JPRB, 0.12214E+01_JPRB, 0.14066E+01_JPRB, 0.15830E+01_JPRB ,&
&  0.18234E+01_JPRB, 0.19797E+01_JPRB, 0.20139E+01_JPRB, 0.19880E+01_JPRB, 0.20704E+01_JPRB, 0.21990E+01_JPRB, 0.22455E+01_JPRB ,&
&  0.23445E+01_JPRB, 0.24557E+01_JPRB, 0.24262E+01_JPRB, 0.23064E+01_JPRB, 0.22299E+01_JPRB, 0.21810E+01_JPRB, 0.20134E+01_JPRB ,&
&  0.18255E+01_JPRB, 0.17522E+01_JPRB, 0.17559E+01_JPRB, 0.15721E+01_JPRB, 0.14443E+01_JPRB, 0.13253E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.12073E+01_JPRB, 0.11688E+01_JPRB, 0.11197E+01_JPRB, 0.10825E+01_JPRB, 0.10627E+01_JPRB, 0.10493E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.10038E+01_JPRB, 0.98862E+00_JPRB, 0.97827E+00_JPRB, 0.96737E+00_JPRB, 0.96179E+00_JPRB, 0.96071E+00_JPRB ,&
&  0.96540E+00_JPRB, 0.97751E+00_JPRB, 0.99573E+00_JPRB, 0.10189E+01_JPRB, 0.10460E+01_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.10751E+01_JPRB, 0.10839E+01_JPRB, 0.10710E+01_JPRB, 0.10432E+01_JPRB, 0.10045E+01_JPRB, 0.95601E+00_JPRB, 0.92024E+00_JPRB ,&
&  0.89890E+00_JPRB, 0.88902E+00_JPRB, 0.87900E+00_JPRB, 0.86924E+00_JPRB, 0.86460E+00_JPRB, 0.86289E+00_JPRB, 0.86368E+00_JPRB ,&
&  0.86642E+00_JPRB, 0.86876E+00_JPRB, 0.86637E+00_JPRB, 0.85230E+00_JPRB, 0.84715E+00_JPRB, 0.84684E+00_JPRB, 0.84994E+00_JPRB ,&
&  0.84143E+00_JPRB, 0.81104E+00_JPRB, 0.77435E+00_JPRB, 0.74677E+00_JPRB, 0.71940E+00_JPRB, 0.70659E+00_JPRB, 0.69024E+00_JPRB ,&
&  0.66746E+00_JPRB, 0.66114E+00_JPRB, 0.67815E+00_JPRB, 0.70790E+00_JPRB, 0.69973E+00_JPRB, 0.71294E+00_JPRB, 0.59218E+00_JPRB ,&
&  0.55273E+00_JPRB, 0.52988E+00_JPRB, 0.52475E+00_JPRB, 0.53894E+00_JPRB, 0.57769E+00_JPRB, 0.62343E+00_JPRB, 0.66336E+00_JPRB ,&
&  0.70288E+00_JPRB, 0.77708E+00_JPRB, 0.82046E+00_JPRB, 0.85506E+00_JPRB, 0.89605E+00_JPRB, 0.93742E+00_JPRB, 0.96790E+00_JPRB ,&
&  0.98988E+00_JPRB, 0.10315E+01_JPRB, 0.10973E+01_JPRB, 0.12018E+01_JPRB, 0.14170E+01_JPRB, 0.14298E+01_JPRB, 0.15205E+01_JPRB ,&
&  0.20163E+01_JPRB, 0.22785E+01_JPRB, 0.21857E+01_JPRB, 0.21084E+01_JPRB, 0.21062E+01_JPRB, 0.21996E+01_JPRB, 0.23449E+01_JPRB ,&
&  0.25803E+01_JPRB, 0.27695E+01_JPRB, 0.27519E+01_JPRB, 0.24865E+01_JPRB, 0.23525E+01_JPRB, 0.22720E+01_JPRB, 0.20454E+01_JPRB ,&
&  0.17927E+01_JPRB, 0.18220E+01_JPRB, 0.18937E+01_JPRB, 0.15996E+01_JPRB, 0.13972E+01_JPRB, 0.12676E+01_JPRB, 0.12084E+01_JPRB ,&
&  0.11695E+01_JPRB, 0.11370E+01_JPRB, 0.11021E+01_JPRB, 0.10710E+01_JPRB, 0.10336E+01_JPRB, 0.99800E+00_JPRB, 0.96671E+00_JPRB ,&
&  0.94333E+00_JPRB, 0.92637E+00_JPRB, 0.92252E+00_JPRB, 0.92578E+00_JPRB, 0.92689E+00_JPRB, 0.92844E+00_JPRB, 0.93813E+00_JPRB ,&
&  0.95349E+00_JPRB, 0.96133E+00_JPRB, 0.97946E+00_JPRB, 0.10060E+01_JPRB, 0.10424E+01_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.10401E+01_JPRB, 0.10396E+01_JPRB, 0.10210E+01_JPRB, 0.98789E+00_JPRB, 0.94494E+00_JPRB, 0.88561E+00_JPRB, 0.83747E+00_JPRB ,&
&  0.80902E+00_JPRB, 0.79225E+00_JPRB, 0.78139E+00_JPRB, 0.77543E+00_JPRB, 0.77491E+00_JPRB, 0.77988E+00_JPRB, 0.77994E+00_JPRB ,&
&  0.78241E+00_JPRB, 0.78522E+00_JPRB, 0.79054E+00_JPRB, 0.79868E+00_JPRB, 0.80287E+00_JPRB, 0.80541E+00_JPRB, 0.80900E+00_JPRB ,&
&  0.81235E+00_JPRB, 0.79769E+00_JPRB, 0.76624E+00_JPRB, 0.74282E+00_JPRB, 0.74213E+00_JPRB, 0.75456E+00_JPRB, 0.73108E+00_JPRB ,&
&  0.68414E+00_JPRB, 0.66229E+00_JPRB, 0.66004E+00_JPRB, 0.71581E+00_JPRB, 0.75500E+00_JPRB, 0.76397E+00_JPRB, 0.64300E+00_JPRB ,&
&  0.57651E+00_JPRB, 0.54959E+00_JPRB, 0.55401E+00_JPRB, 0.58934E+00_JPRB, 0.64894E+00_JPRB, 0.70928E+00_JPRB, 0.78682E+00_JPRB ,&
&  0.83415E+00_JPRB, 0.87603E+00_JPRB, 0.88906E+00_JPRB, 0.91475E+00_JPRB, 0.95844E+00_JPRB, 0.99976E+00_JPRB, 0.10451E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.11294E+01_JPRB, 0.12610E+01_JPRB, 0.11736E+01_JPRB, 0.12430E+01_JPRB, 0.13407E+01_JPRB, 0.15145E+01_JPRB ,&
&  0.18489E+01_JPRB, 0.20030E+01_JPRB, 0.20762E+01_JPRB, 0.22609E+01_JPRB, 0.21989E+01_JPRB, 0.21686E+01_JPRB, 0.22557E+01_JPRB ,&
&  0.26201E+01_JPRB, 0.29287E+01_JPRB, 0.28564E+01_JPRB, 0.25906E+01_JPRB, 0.24896E+01_JPRB, 0.24232E+01_JPRB, 0.21445E+01_JPRB ,&
&  0.17970E+01_JPRB, 0.16006E+01_JPRB, 0.14326E+01_JPRB, 0.12409E+01_JPRB, 0.11623E+01_JPRB, 0.11158E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10498E+01_JPRB, 0.10367E+01_JPRB, 0.97306E+00_JPRB, 0.91380E+00_JPRB, 0.84755E+00_JPRB, 0.80213E+00_JPRB ,&
&  0.77351E+00_JPRB, 0.77093E+00_JPRB, 0.78655E+00_JPRB, 0.81433E+00_JPRB, 0.86875E+00_JPRB, 0.91824E+00_JPRB, 0.95296E+00_JPRB ,&
&  0.97033E+00_JPRB, 0.95585E+00_JPRB, 0.95368E+00_JPRB, 0.96859E+00_JPRB, 0.10131E+01_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.98094E+00_JPRB, 0.93984E+00_JPRB, 0.91298E+00_JPRB, 0.91458E+00_JPRB, 0.86259E+00_JPRB, 0.80916E+00_JPRB, 0.77654E+00_JPRB ,&
&  0.77526E+00_JPRB, 0.77090E+00_JPRB, 0.76814E+00_JPRB, 0.73642E+00_JPRB, 0.70881E+00_JPRB, 0.69460E+00_JPRB, 0.68956E+00_JPRB ,&
&  0.70276E+00_JPRB, 0.71987E+00_JPRB, 0.73480E+00_JPRB, 0.73521E+00_JPRB, 0.73380E+00_JPRB, 0.73784E+00_JPRB, 0.75092E+00_JPRB ,&
&  0.76783E+00_JPRB, 0.77244E+00_JPRB, 0.76130E+00_JPRB, 0.75232E+00_JPRB, 0.76949E+00_JPRB, 0.78883E+00_JPRB, 0.75582E+00_JPRB ,&
&  0.72013E+00_JPRB, 0.68945E+00_JPRB, 0.68162E+00_JPRB, 0.72168E+00_JPRB, 0.79187E+00_JPRB, 0.81353E+00_JPRB, 0.69393E+00_JPRB ,&
&  0.60996E+00_JPRB, 0.59489E+00_JPRB, 0.64340E+00_JPRB, 0.76426E+00_JPRB, 0.82112E+00_JPRB, 0.87761E+00_JPRB, 0.94700E+00_JPRB ,&
&  0.90307E+00_JPRB, 0.90405E+00_JPRB, 0.93437E+00_JPRB, 0.10772E+01_JPRB, 0.11230E+01_JPRB, 0.11478E+01_JPRB, 0.11681E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.12461E+01_JPRB, 0.14178E+01_JPRB, 0.12765E+01_JPRB, 0.14190E+01_JPRB, 0.17109E+01_JPRB, 0.17491E+01_JPRB ,&
&  0.19101E+01_JPRB, 0.20048E+01_JPRB, 0.22847E+01_JPRB, 0.22894E+01_JPRB, 0.21768E+01_JPRB, 0.21725E+01_JPRB, 0.23683E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.31800E+01_JPRB, 0.30781E+01_JPRB, 0.29232E+01_JPRB, 0.28117E+01_JPRB, 0.26556E+01_JPRB, 0.23444E+01_JPRB ,&
&  0.19190E+01_JPRB, 0.16008E+01_JPRB, 0.13598E+01_JPRB, 0.11546E+01_JPRB, 0.10572E+01_JPRB, 0.10206E+01_JPRB, 0.98210E+00_JPRB ,&
&  0.97300E+00_JPRB, 0.96458E+00_JPRB, 0.95526E+00_JPRB, 0.88651E+00_JPRB, 0.82563E+00_JPRB, 0.77624E+00_JPRB, 0.74006E+00_JPRB ,&
&  0.72857E+00_JPRB, 0.71556E+00_JPRB, 0.70952E+00_JPRB, 0.73673E+00_JPRB, 0.81375E+00_JPRB, 0.90481E+00_JPRB, 0.95119E+00_JPRB ,&
&  0.93873E+00_JPRB, 0.90393E+00_JPRB, 0.92510E+00_JPRB, 0.97201E+00_JPRB, 0.99758E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.98978E+00_JPRB, 0.98097E+00_JPRB, 0.97261E+00_JPRB, 0.95879E+00_JPRB, 0.93032E+00_JPRB, 0.86569E+00_JPRB, 0.82530E+00_JPRB ,&
&  0.83388E+00_JPRB, 0.80783E+00_JPRB, 0.79084E+00_JPRB, 0.75109E+00_JPRB, 0.71306E+00_JPRB, 0.67496E+00_JPRB, 0.65747E+00_JPRB ,&
&  0.64230E+00_JPRB, 0.65998E+00_JPRB, 0.69627E+00_JPRB, 0.70853E+00_JPRB, 0.71090E+00_JPRB, 0.71184E+00_JPRB, 0.72917E+00_JPRB ,&
&  0.74909E+00_JPRB, 0.76893E+00_JPRB, 0.77840E+00_JPRB, 0.78822E+00_JPRB, 0.80371E+00_JPRB, 0.82125E+00_JPRB, 0.81069E+00_JPRB ,&
&  0.79544E+00_JPRB, 0.76368E+00_JPRB, 0.73880E+00_JPRB, 0.76164E+00_JPRB, 0.82413E+00_JPRB, 0.88180E+00_JPRB, 0.84184E+00_JPRB ,&
&  0.68370E+00_JPRB, 0.69166E+00_JPRB, 0.86439E+00_JPRB, 0.93335E+00_JPRB, 0.99689E+00_JPRB, 0.10363E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11764E+01_JPRB, 0.12100E+01_JPRB, 0.12363E+01_JPRB, 0.12591E+01_JPRB, 0.12915E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.13953E+01_JPRB, 0.11809E+01_JPRB, 0.11558E+01_JPRB, 0.12830E+01_JPRB, 0.17109E+01_JPRB, 0.19020E+01_JPRB, 0.19169E+01_JPRB ,&
&  0.20316E+01_JPRB, 0.20647E+01_JPRB, 0.22060E+01_JPRB, 0.24089E+01_JPRB, 0.24940E+01_JPRB, 0.26587E+01_JPRB, 0.28858E+01_JPRB ,&
&  0.32184E+01_JPRB, 0.35565E+01_JPRB, 0.35534E+01_JPRB, 0.34273E+01_JPRB, 0.32074E+01_JPRB, 0.29190E+01_JPRB, 0.24829E+01_JPRB ,&
&  0.19693E+01_JPRB, 0.16069E+01_JPRB, 0.13078E+01_JPRB, 0.11036E+01_JPRB, 0.98742E+00_JPRB, 0.93620E+00_JPRB, 0.90040E+00_JPRB ,&
&  0.90009E+00_JPRB, 0.89757E+00_JPRB, 0.88569E+00_JPRB, 0.83421E+00_JPRB, 0.79300E+00_JPRB, 0.79667E+00_JPRB, 0.80601E+00_JPRB ,&
&  0.81529E+00_JPRB, 0.81144E+00_JPRB, 0.76790E+00_JPRB, 0.79065E+00_JPRB, 0.86590E+00_JPRB, 0.94390E+00_JPRB, 0.97220E+00_JPRB ,&
&  0.96770E+00_JPRB, 0.93093E+00_JPRB, 0.94377E+00_JPRB, 0.97581E+00_JPRB, 0.98426E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.10143E+01_JPRB, 0.10181E+01_JPRB, 0.10022E+01_JPRB, 0.99057E+00_JPRB, 0.96945E+00_JPRB, 0.89001E+00_JPRB, 0.86384E+00_JPRB ,&
&  0.93604E+00_JPRB, 0.87711E+00_JPRB, 0.92675E+00_JPRB, 0.81411E+00_JPRB, 0.78295E+00_JPRB, 0.74191E+00_JPRB, 0.71351E+00_JPRB ,&
&  0.67700E+00_JPRB, 0.67020E+00_JPRB, 0.69459E+00_JPRB, 0.72048E+00_JPRB, 0.72386E+00_JPRB, 0.71430E+00_JPRB, 0.71988E+00_JPRB ,&
&  0.73228E+00_JPRB, 0.76297E+00_JPRB, 0.80558E+00_JPRB, 0.82704E+00_JPRB, 0.84076E+00_JPRB, 0.86148E+00_JPRB, 0.86834E+00_JPRB ,&
&  0.82964E+00_JPRB, 0.81861E+00_JPRB, 0.81115E+00_JPRB, 0.82233E+00_JPRB, 0.86052E+00_JPRB, 0.92208E+00_JPRB, 0.96054E+00_JPRB ,&
&  0.97037E+00_JPRB, 0.85966E+00_JPRB, 0.97593E+00_JPRB, 0.10134E+01_JPRB, 0.10814E+01_JPRB, 0.11224E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.12309E+01_JPRB, 0.12767E+01_JPRB, 0.13200E+01_JPRB, 0.13675E+01_JPRB, 0.14311E+01_JPRB, 0.14653E+01_JPRB ,&
&  0.16045E+01_JPRB, 0.12374E+01_JPRB, 0.13552E+01_JPRB, 0.14938E+01_JPRB, 0.19920E+01_JPRB, 0.22367E+01_JPRB, 0.22707E+01_JPRB ,&
&  0.26356E+01_JPRB, 0.25537E+01_JPRB, 0.27702E+01_JPRB, 0.29608E+01_JPRB, 0.32155E+01_JPRB, 0.35486E+01_JPRB, 0.37386E+01_JPRB ,&
&  0.37754E+01_JPRB, 0.38655E+01_JPRB, 0.40271E+01_JPRB, 0.36859E+01_JPRB, 0.32410E+01_JPRB, 0.27907E+01_JPRB, 0.22560E+01_JPRB ,&
&  0.18993E+01_JPRB, 0.15841E+01_JPRB, 0.13156E+01_JPRB, 0.11292E+01_JPRB, 0.99139E+00_JPRB, 0.90553E+00_JPRB, 0.86231E+00_JPRB ,&
&  0.84225E+00_JPRB, 0.82511E+00_JPRB, 0.82338E+00_JPRB, 0.80283E+00_JPRB, 0.77898E+00_JPRB, 0.79042E+00_JPRB, 0.83162E+00_JPRB ,&
&  0.84283E+00_JPRB, 0.85750E+00_JPRB, 0.91175E+00_JPRB, 0.10282E+01_JPRB, 0.10557E+01_JPRB, 0.10866E+01_JPRB, 0.11976E+01_JPRB ,&
&  0.10732E+01_JPRB, 0.10615E+01_JPRB, 0.11042E+01_JPRB, 0.10679E+01_JPRB, 0.10228E+01_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.10885E+01_JPRB, 0.10775E+01_JPRB, 0.10428E+01_JPRB, 0.10239E+01_JPRB, 0.10206E+01_JPRB, 0.97620E+00_JPRB, 0.93534E+00_JPRB ,&
&  0.10000E+01_JPRB, 0.98881E+00_JPRB, 0.98771E+00_JPRB, 0.96834E+00_JPRB, 0.94995E+00_JPRB, 0.87467E+00_JPRB, 0.82138E+00_JPRB ,&
&  0.77450E+00_JPRB, 0.76277E+00_JPRB, 0.79532E+00_JPRB, 0.80635E+00_JPRB, 0.80299E+00_JPRB, 0.77984E+00_JPRB, 0.76783E+00_JPRB ,&
&  0.75696E+00_JPRB, 0.76870E+00_JPRB, 0.81155E+00_JPRB, 0.84999E+00_JPRB, 0.85243E+00_JPRB, 0.85623E+00_JPRB, 0.87943E+00_JPRB ,&
&  0.84633E+00_JPRB, 0.84619E+00_JPRB, 0.84880E+00_JPRB, 0.80510E+00_JPRB, 0.90184E+00_JPRB, 0.94792E+00_JPRB, 0.97546E+00_JPRB ,&
&  0.10224E+01_JPRB, 0.10639E+01_JPRB, 0.10883E+01_JPRB, 0.11014E+01_JPRB, 0.11304E+01_JPRB, 0.11503E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.12110E+01_JPRB, 0.12504E+01_JPRB, 0.13007E+01_JPRB, 0.13649E+01_JPRB, 0.14824E+01_JPRB, 0.14927E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.18142E+01_JPRB, 0.18721E+01_JPRB, 0.20425E+01_JPRB, 0.19244E+01_JPRB, 0.24366E+01_JPRB, 0.27379E+01_JPRB, 0.26019E+01_JPRB ,&
&  0.27256E+01_JPRB, 0.27073E+01_JPRB, 0.32527E+01_JPRB, 0.37055E+01_JPRB, 0.41102E+01_JPRB, 0.44701E+01_JPRB, 0.45847E+01_JPRB ,&
&  0.44897E+01_JPRB, 0.46543E+01_JPRB, 0.45181E+01_JPRB, 0.40198E+01_JPRB, 0.34068E+01_JPRB, 0.29643E+01_JPRB, 0.23916E+01_JPRB ,&
&  0.19195E+01_JPRB, 0.15399E+01_JPRB, 0.13060E+01_JPRB, 0.11890E+01_JPRB, 0.99041E+00_JPRB, 0.92728E+00_JPRB, 0.86287E+00_JPRB ,&
&  0.80135E+00_JPRB, 0.76924E+00_JPRB, 0.78936E+00_JPRB, 0.81236E+00_JPRB, 0.79210E+00_JPRB, 0.80346E+00_JPRB, 0.81572E+00_JPRB ,&
&  0.89021E+00_JPRB, 0.10522E+01_JPRB, 0.13539E+01_JPRB, 0.14252E+01_JPRB, 0.14061E+01_JPRB, 0.13953E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.11712E+01_JPRB, 0.12796E+01_JPRB, 0.12247E+01_JPRB, 0.11460E+01_JPRB, 0.10902E+01_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.11479E+01_JPRB, 0.11264E+01_JPRB, 0.11021E+01_JPRB, 0.10824E+01_JPRB, 0.10717E+01_JPRB, 0.10460E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.10202E+01_JPRB, 0.10145E+01_JPRB, 0.10041E+01_JPRB, 0.98235E+00_JPRB, 0.99280E+00_JPRB, 0.91531E+00_JPRB ,&
&  0.85221E+00_JPRB, 0.86004E+00_JPRB, 0.84668E+00_JPRB, 0.88737E+00_JPRB, 0.94228E+00_JPRB, 0.91868E+00_JPRB, 0.89809E+00_JPRB ,&
&  0.87092E+00_JPRB, 0.83587E+00_JPRB, 0.83433E+00_JPRB, 0.85223E+00_JPRB, 0.86209E+00_JPRB, 0.87508E+00_JPRB, 0.89596E+00_JPRB ,&
&  0.84424E+00_JPRB, 0.79811E+00_JPRB, 0.77560E+00_JPRB, 0.78343E+00_JPRB, 0.85492E+00_JPRB, 0.96322E+00_JPRB, 0.10279E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.10695E+01_JPRB, 0.11175E+01_JPRB, 0.11467E+01_JPRB, 0.11686E+01_JPRB, 0.12036E+01_JPRB, 0.12425E+01_JPRB ,&
&  0.12775E+01_JPRB, 0.13196E+01_JPRB, 0.13718E+01_JPRB, 0.14593E+01_JPRB, 0.14784E+01_JPRB, 0.16284E+01_JPRB, 0.19940E+01_JPRB ,&
&  0.21894E+01_JPRB, 0.22868E+01_JPRB, 0.24021E+01_JPRB, 0.26286E+01_JPRB, 0.29685E+01_JPRB, 0.29638E+01_JPRB, 0.29735E+01_JPRB ,&
&  0.32172E+01_JPRB, 0.33336E+01_JPRB, 0.41521E+01_JPRB, 0.45460E+01_JPRB, 0.47579E+01_JPRB, 0.50846E+01_JPRB, 0.50254E+01_JPRB ,&
&  0.49516E+01_JPRB, 0.48000E+01_JPRB, 0.46687E+01_JPRB, 0.41962E+01_JPRB, 0.35311E+01_JPRB, 0.30723E+01_JPRB, 0.24023E+01_JPRB ,&
&  0.19605E+01_JPRB, 0.15680E+01_JPRB, 0.11908E+01_JPRB, 0.10372E+01_JPRB, 0.89624E+00_JPRB, 0.87146E+00_JPRB, 0.82961E+00_JPRB ,&
&  0.77658E+00_JPRB, 0.79047E+00_JPRB, 0.84504E+00_JPRB, 0.95198E+00_JPRB, 0.95433E+00_JPRB, 0.86651E+00_JPRB, 0.92369E+00_JPRB ,&
&  0.10461E+01_JPRB, 0.14182E+01_JPRB, 0.15828E+01_JPRB, 0.17347E+01_JPRB, 0.16518E+01_JPRB, 0.16287E+01_JPRB, 0.13592E+01_JPRB ,&
&  0.13643E+01_JPRB, 0.14661E+01_JPRB, 0.13515E+01_JPRB, 0.12506E+01_JPRB, 0.11877E+01_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.11988E+01_JPRB, 0.11657E+01_JPRB, 0.11537E+01_JPRB, 0.11596E+01_JPRB, 0.11627E+01_JPRB, 0.11233E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.10717E+01_JPRB, 0.10569E+01_JPRB, 0.10486E+01_JPRB, 0.10287E+01_JPRB, 0.10389E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.99295E+00_JPRB, 0.10337E+01_JPRB, 0.92490E+00_JPRB, 0.93734E+00_JPRB, 0.11544E+01_JPRB, 0.10739E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.99893E+00_JPRB, 0.95960E+00_JPRB, 0.89537E+00_JPRB, 0.87974E+00_JPRB, 0.89126E+00_JPRB, 0.92436E+00_JPRB, 0.93743E+00_JPRB ,&
&  0.86777E+00_JPRB, 0.80056E+00_JPRB, 0.78149E+00_JPRB, 0.79037E+00_JPRB, 0.84505E+00_JPRB, 0.93302E+00_JPRB, 0.10492E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.11161E+01_JPRB, 0.11600E+01_JPRB, 0.12131E+01_JPRB, 0.12698E+01_JPRB, 0.13397E+01_JPRB, 0.13833E+01_JPRB ,&
&  0.14119E+01_JPRB, 0.14377E+01_JPRB, 0.14821E+01_JPRB, 0.15816E+01_JPRB, 0.15577E+01_JPRB, 0.17815E+01_JPRB, 0.25163E+01_JPRB ,&
&  0.29845E+01_JPRB, 0.27899E+01_JPRB, 0.30584E+01_JPRB, 0.39725E+01_JPRB, 0.43150E+01_JPRB, 0.42743E+01_JPRB, 0.41436E+01_JPRB ,&
&  0.42688E+01_JPRB, 0.44592E+01_JPRB, 0.47223E+01_JPRB, 0.51022E+01_JPRB, 0.53269E+01_JPRB, 0.54889E+01_JPRB, 0.52920E+01_JPRB ,&
&  0.50074E+01_JPRB, 0.48327E+01_JPRB, 0.45791E+01_JPRB, 0.41536E+01_JPRB, 0.33984E+01_JPRB, 0.28924E+01_JPRB, 0.22704E+01_JPRB ,&
&  0.18937E+01_JPRB, 0.14399E+01_JPRB, 0.99342E+00_JPRB, 0.83265E+00_JPRB, 0.77114E+00_JPRB, 0.84549E+00_JPRB, 0.11210E+01_JPRB ,&
&  0.86163E+00_JPRB, 0.96975E+00_JPRB, 0.11117E+01_JPRB, 0.12952E+01_JPRB, 0.14071E+01_JPRB, 0.13934E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.14577E+01_JPRB, 0.19373E+01_JPRB, 0.21021E+01_JPRB, 0.19112E+01_JPRB, 0.18818E+01_JPRB, 0.17572E+01_JPRB ,&
&  0.18255E+01_JPRB, 0.16524E+01_JPRB, 0.14840E+01_JPRB, 0.13364E+01_JPRB, 0.12693E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.12190E+01_JPRB, 0.12127E+01_JPRB, 0.12280E+01_JPRB, 0.12611E+01_JPRB, 0.12835E+01_JPRB, 0.12221E+01_JPRB, 0.12012E+01_JPRB ,&
&  0.11743E+01_JPRB, 0.11384E+01_JPRB, 0.11098E+01_JPRB, 0.10988E+01_JPRB, 0.10832E+01_JPRB, 0.10760E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.11619E+01_JPRB, 0.12775E+01_JPRB, 0.11413E+01_JPRB, 0.95544E+00_JPRB, 0.10367E+01_JPRB, 0.10227E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.10391E+01_JPRB, 0.10632E+01_JPRB, 0.10217E+01_JPRB, 0.98328E+00_JPRB, 0.10222E+01_JPRB, 0.10846E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.96618E+00_JPRB, 0.91713E+00_JPRB, 0.93399E+00_JPRB, 0.10291E+01_JPRB, 0.99325E+00_JPRB, 0.11177E+01_JPRB ,&
&  0.11842E+01_JPRB, 0.12397E+01_JPRB, 0.12971E+01_JPRB, 0.13727E+01_JPRB, 0.14424E+01_JPRB, 0.15178E+01_JPRB, 0.15608E+01_JPRB ,&
&  0.15872E+01_JPRB, 0.15844E+01_JPRB, 0.15477E+01_JPRB, 0.16224E+01_JPRB, 0.17323E+01_JPRB, 0.19373E+01_JPRB, 0.24558E+01_JPRB ,&
&  0.28893E+01_JPRB, 0.30080E+01_JPRB, 0.36366E+01_JPRB, 0.46660E+01_JPRB, 0.49472E+01_JPRB, 0.48728E+01_JPRB, 0.50766E+01_JPRB ,&
&  0.53865E+01_JPRB, 0.56990E+01_JPRB, 0.60245E+01_JPRB, 0.61026E+01_JPRB, 0.60580E+01_JPRB, 0.56506E+01_JPRB, 0.52311E+01_JPRB ,&
&  0.48117E+01_JPRB, 0.46764E+01_JPRB, 0.45299E+01_JPRB, 0.39021E+01_JPRB, 0.31667E+01_JPRB, 0.26013E+01_JPRB, 0.23056E+01_JPRB ,&
&  0.22178E+01_JPRB, 0.11414E+01_JPRB, 0.79967E+00_JPRB, 0.66911E+00_JPRB, 0.65392E+00_JPRB, 0.70956E+00_JPRB, 0.90345E+00_JPRB ,&
&  0.84294E+00_JPRB, 0.11160E+01_JPRB, 0.16205E+01_JPRB, 0.18727E+01_JPRB, 0.22540E+01_JPRB, 0.27154E+01_JPRB, 0.20826E+01_JPRB ,&
&  0.16914E+01_JPRB, 0.18089E+01_JPRB, 0.24171E+01_JPRB, 0.25294E+01_JPRB, 0.22419E+01_JPRB, 0.21759E+01_JPRB, 0.21898E+01_JPRB ,&
&  0.20182E+01_JPRB, 0.18963E+01_JPRB, 0.16376E+01_JPRB, 0.13824E+01_JPRB, 0.12717E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.13406E+01_JPRB, 0.13317E+01_JPRB, 0.13677E+01_JPRB, 0.14038E+01_JPRB, 0.13941E+01_JPRB, 0.13322E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.12561E+01_JPRB, 0.12048E+01_JPRB, 0.11640E+01_JPRB, 0.11442E+01_JPRB, 0.11278E+01_JPRB, 0.11210E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.12502E+01_JPRB, 0.10784E+01_JPRB, 0.95196E+00_JPRB, 0.83579E+00_JPRB, 0.88600E+00_JPRB, 0.96170E+00_JPRB ,&
&  0.10818E+01_JPRB, 0.11361E+01_JPRB, 0.12193E+01_JPRB, 0.13225E+01_JPRB, 0.14382E+01_JPRB, 0.14020E+01_JPRB, 0.14834E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.13402E+01_JPRB, 0.11676E+01_JPRB, 0.12642E+01_JPRB, 0.12714E+01_JPRB, 0.11932E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.14519E+01_JPRB, 0.15047E+01_JPRB, 0.15492E+01_JPRB, 0.15933E+01_JPRB, 0.16279E+01_JPRB, 0.16101E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.15051E+01_JPRB, 0.14982E+01_JPRB, 0.15473E+01_JPRB, 0.16135E+01_JPRB, 0.18680E+01_JPRB, 0.21184E+01_JPRB, 0.25912E+01_JPRB ,&
&  0.31218E+01_JPRB, 0.31282E+01_JPRB, 0.33366E+01_JPRB, 0.46172E+01_JPRB, 0.59885E+01_JPRB, 0.59849E+01_JPRB, 0.60749E+01_JPRB ,&
&  0.64450E+01_JPRB, 0.65234E+01_JPRB, 0.63904E+01_JPRB, 0.60459E+01_JPRB, 0.57605E+01_JPRB, 0.54362E+01_JPRB, 0.50594E+01_JPRB ,&
&  0.46421E+01_JPRB, 0.44991E+01_JPRB, 0.43474E+01_JPRB, 0.38866E+01_JPRB, 0.31897E+01_JPRB, 0.28250E+01_JPRB, 0.25151E+01_JPRB ,&
&  0.22269E+01_JPRB, 0.14476E+01_JPRB, 0.93495E+00_JPRB, 0.77267E+00_JPRB, 0.73846E+00_JPRB, 0.77388E+00_JPRB, 0.97453E+00_JPRB ,&
&  0.10399E+01_JPRB, 0.14657E+01_JPRB, 0.21763E+01_JPRB, 0.25251E+01_JPRB, 0.38513E+01_JPRB, 0.45761E+01_JPRB, 0.41408E+01_JPRB ,&
&  0.32141E+01_JPRB, 0.26209E+01_JPRB, 0.30349E+01_JPRB, 0.29955E+01_JPRB, 0.27129E+01_JPRB, 0.24560E+01_JPRB, 0.23884E+01_JPRB ,&
&  0.22179E+01_JPRB, 0.20859E+01_JPRB, 0.18924E+01_JPRB, 0.15865E+01_JPRB, 0.14064E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.16004E+01_JPRB, 0.15336E+01_JPRB, 0.15051E+01_JPRB, 0.14898E+01_JPRB, 0.14070E+01_JPRB, 0.13474E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.12690E+01_JPRB, 0.12256E+01_JPRB, 0.12124E+01_JPRB, 0.12022E+01_JPRB, 0.12017E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.11003E+01_JPRB, 0.11315E+01_JPRB, 0.96026E+00_JPRB, 0.83304E+00_JPRB, 0.78710E+00_JPRB, 0.87516E+00_JPRB, 0.10239E+01_JPRB ,&
&  0.12347E+01_JPRB, 0.14565E+01_JPRB, 0.17385E+01_JPRB, 0.19189E+01_JPRB, 0.20609E+01_JPRB, 0.21018E+01_JPRB, 0.21754E+01_JPRB ,&
&  0.23378E+01_JPRB, 0.18642E+01_JPRB, 0.17757E+01_JPRB, 0.17255E+01_JPRB, 0.17204E+01_JPRB, 0.16953E+01_JPRB, 0.16974E+01_JPRB ,&
&  0.16584E+01_JPRB, 0.16429E+01_JPRB, 0.16306E+01_JPRB, 0.16131E+01_JPRB, 0.15934E+01_JPRB, 0.15682E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.15436E+01_JPRB, 0.15597E+01_JPRB, 0.18996E+01_JPRB, 0.26210E+01_JPRB, 0.30883E+01_JPRB, 0.33198E+01_JPRB ,&
&  0.32097E+01_JPRB, 0.34254E+01_JPRB, 0.53270E+01_JPRB, 0.59731E+01_JPRB, 0.64243E+01_JPRB, 0.72766E+01_JPRB, 0.75512E+01_JPRB ,&
&  0.72164E+01_JPRB, 0.64808E+01_JPRB, 0.57906E+01_JPRB, 0.51041E+01_JPRB, 0.56016E+01_JPRB, 0.53493E+01_JPRB, 0.48480E+01_JPRB ,&
&  0.43680E+01_JPRB, 0.43154E+01_JPRB, 0.39388E+01_JPRB, 0.35433E+01_JPRB, 0.32518E+01_JPRB, 0.32559E+01_JPRB, 0.26220E+01_JPRB ,&
&  0.21119E+01_JPRB, 0.20430E+01_JPRB, 0.15183E+01_JPRB, 0.12525E+01_JPRB, 0.11718E+01_JPRB, 0.11973E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.16126E+01_JPRB, 0.21228E+01_JPRB, 0.28797E+01_JPRB, 0.36552E+01_JPRB, 0.51979E+01_JPRB, 0.61013E+01_JPRB, 0.58444E+01_JPRB ,&
&  0.45734E+01_JPRB, 0.42241E+01_JPRB, 0.37332E+01_JPRB, 0.34711E+01_JPRB, 0.31613E+01_JPRB, 0.27960E+01_JPRB, 0.25912E+01_JPRB ,&
&  0.23603E+01_JPRB, 0.21970E+01_JPRB, 0.20663E+01_JPRB, 0.18580E+01_JPRB, 0.17224E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.19354E+01_JPRB, 0.18215E+01_JPRB, 0.16598E+01_JPRB, 0.15423E+01_JPRB, 0.14275E+01_JPRB, 0.13550E+01_JPRB, 0.13288E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.12737E+01_JPRB, 0.12783E+01_JPRB, 0.12930E+01_JPRB, 0.12864E+01_JPRB, 0.12628E+01_JPRB, 0.11732E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10111E+01_JPRB, 0.88363E+00_JPRB, 0.87776E+00_JPRB, 0.97789E+00_JPRB, 0.10868E+01_JPRB, 0.12592E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.19801E+01_JPRB, 0.23517E+01_JPRB, 0.28757E+01_JPRB, 0.31737E+01_JPRB, 0.32558E+01_JPRB, 0.31884E+01_JPRB ,&
&  0.29077E+01_JPRB, 0.28405E+01_JPRB, 0.26823E+01_JPRB, 0.24400E+01_JPRB, 0.22066E+01_JPRB, 0.20357E+01_JPRB, 0.18944E+01_JPRB ,&
&  0.17716E+01_JPRB, 0.16730E+01_JPRB, 0.16253E+01_JPRB, 0.15889E+01_JPRB, 0.16113E+01_JPRB, 0.16279E+01_JPRB, 0.16644E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.16258E+01_JPRB, 0.18832E+01_JPRB, 0.27407E+01_JPRB, 0.34360E+01_JPRB, 0.39242E+01_JPRB, 0.37302E+01_JPRB ,&
&  0.41837E+01_JPRB, 0.46870E+01_JPRB, 0.49982E+01_JPRB, 0.58172E+01_JPRB, 0.51225E+01_JPRB, 0.59817E+01_JPRB, 0.70714E+01_JPRB ,&
&  0.68873E+01_JPRB, 0.61930E+01_JPRB, 0.57647E+01_JPRB, 0.52914E+01_JPRB, 0.49197E+01_JPRB, 0.58676E+01_JPRB, 0.51090E+01_JPRB ,&
&  0.44288E+01_JPRB, 0.41634E+01_JPRB, 0.37541E+01_JPRB, 0.36984E+01_JPRB, 0.31394E+01_JPRB, 0.27849E+01_JPRB, 0.20177E+01_JPRB ,&
&  0.20815E+01_JPRB, 0.18878E+01_JPRB, 0.19151E+01_JPRB, 0.17519E+01_JPRB, 0.18259E+01_JPRB, 0.17911E+01_JPRB, 0.19155E+01_JPRB ,&
&  0.24293E+01_JPRB, 0.30543E+01_JPRB, 0.45811E+01_JPRB, 0.56791E+01_JPRB, 0.79407E+01_JPRB, 0.72068E+01_JPRB, 0.70117E+01_JPRB ,&
&  0.59947E+01_JPRB, 0.54009E+01_JPRB, 0.43871E+01_JPRB, 0.41763E+01_JPRB, 0.36835E+01_JPRB, 0.32464E+01_JPRB, 0.28570E+01_JPRB ,&
&  0.25055E+01_JPRB, 0.22750E+01_JPRB, 0.21143E+01_JPRB, 0.19977E+01_JPRB, 0.19822E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.18699E+01_JPRB, 0.17652E+01_JPRB, 0.16502E+01_JPRB, 0.15291E+01_JPRB, 0.13941E+01_JPRB, 0.13192E+01_JPRB, 0.12753E+01_JPRB ,&
&  0.12526E+01_JPRB, 0.12654E+01_JPRB, 0.12827E+01_JPRB, 0.12901E+01_JPRB, 0.12920E+01_JPRB, 0.12852E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10128E+01_JPRB, 0.88668E+00_JPRB, 0.87740E+00_JPRB, 0.11200E+01_JPRB, 0.13716E+01_JPRB, 0.14035E+01_JPRB ,&
&  0.20121E+01_JPRB, 0.26290E+01_JPRB, 0.29993E+01_JPRB, 0.36983E+01_JPRB, 0.44330E+01_JPRB, 0.48576E+01_JPRB, 0.44785E+01_JPRB ,&
&  0.42723E+01_JPRB, 0.38337E+01_JPRB, 0.32352E+01_JPRB, 0.27545E+01_JPRB, 0.23379E+01_JPRB, 0.20213E+01_JPRB, 0.17821E+01_JPRB ,&
&  0.16328E+01_JPRB, 0.15435E+01_JPRB, 0.14839E+01_JPRB, 0.14717E+01_JPRB, 0.15194E+01_JPRB, 0.15050E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.16877E+01_JPRB, 0.18217E+01_JPRB, 0.23398E+01_JPRB, 0.32896E+01_JPRB, 0.42164E+01_JPRB, 0.45999E+01_JPRB, 0.45579E+01_JPRB ,&
&  0.44581E+01_JPRB, 0.45600E+01_JPRB, 0.49148E+01_JPRB, 0.51483E+01_JPRB, 0.55359E+01_JPRB, 0.60571E+01_JPRB, 0.68858E+01_JPRB ,&
&  0.54978E+01_JPRB, 0.45543E+01_JPRB, 0.38670E+01_JPRB, 0.39270E+01_JPRB, 0.49086E+01_JPRB, 0.60716E+01_JPRB, 0.58532E+01_JPRB ,&
&  0.51055E+01_JPRB, 0.45766E+01_JPRB, 0.40797E+01_JPRB, 0.36155E+01_JPRB, 0.36634E+01_JPRB, 0.24671E+01_JPRB, 0.27044E+01_JPRB ,&
&  0.27887E+01_JPRB, 0.27958E+01_JPRB, 0.25569E+01_JPRB, 0.21980E+01_JPRB, 0.21914E+01_JPRB, 0.26255E+01_JPRB, 0.31816E+01_JPRB ,&
&  0.40969E+01_JPRB, 0.49842E+01_JPRB, 0.84106E+01_JPRB, 0.92208E+01_JPRB, 0.95340E+01_JPRB, 0.79452E+01_JPRB, 0.72181E+01_JPRB ,&
&  0.58085E+01_JPRB, 0.53588E+01_JPRB, 0.51215E+01_JPRB, 0.46416E+01_JPRB, 0.40738E+01_JPRB, 0.36348E+01_JPRB, 0.31265E+01_JPRB ,&
&  0.27201E+01_JPRB, 0.23924E+01_JPRB, 0.21452E+01_JPRB, 0.19856E+01_JPRB, 0.19318E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.18138E+01_JPRB, 0.17298E+01_JPRB, 0.16061E+01_JPRB, 0.14252E+01_JPRB, 0.13200E+01_JPRB, 0.12821E+01_JPRB, 0.12834E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.13059E+01_JPRB, 0.12984E+01_JPRB, 0.12877E+01_JPRB, 0.12981E+01_JPRB, 0.12775E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.10942E+01_JPRB, 0.95291E+00_JPRB, 0.10043E+01_JPRB, 0.13172E+01_JPRB, 0.17117E+01_JPRB, 0.17700E+01_JPRB ,&
&  0.27497E+01_JPRB, 0.31020E+01_JPRB, 0.31011E+01_JPRB, 0.37098E+01_JPRB, 0.55203E+01_JPRB, 0.54130E+01_JPRB, 0.49925E+01_JPRB ,&
&  0.45901E+01_JPRB, 0.38172E+01_JPRB, 0.30883E+01_JPRB, 0.25323E+01_JPRB, 0.21287E+01_JPRB, 0.18682E+01_JPRB, 0.16743E+01_JPRB ,&
&  0.15618E+01_JPRB, 0.14843E+01_JPRB, 0.14613E+01_JPRB, 0.14980E+01_JPRB, 0.14904E+01_JPRB, 0.14972E+01_JPRB, 0.16353E+01_JPRB ,&
&  0.17950E+01_JPRB, 0.19882E+01_JPRB, 0.24921E+01_JPRB, 0.32845E+01_JPRB, 0.42171E+01_JPRB, 0.48085E+01_JPRB, 0.57623E+01_JPRB ,&
&  0.51988E+01_JPRB, 0.53076E+01_JPRB, 0.51315E+01_JPRB, 0.60877E+01_JPRB, 0.59173E+01_JPRB, 0.63120E+01_JPRB, 0.66752E+01_JPRB ,&
&  0.49149E+01_JPRB, 0.41341E+01_JPRB, 0.42383E+01_JPRB, 0.50464E+01_JPRB, 0.45938E+01_JPRB, 0.49684E+01_JPRB, 0.62225E+01_JPRB ,&
&  0.49888E+01_JPRB, 0.45708E+01_JPRB, 0.43797E+01_JPRB, 0.44894E+01_JPRB, 0.28471E+01_JPRB, 0.21527E+01_JPRB, 0.32966E+01_JPRB ,&
&  0.31458E+01_JPRB, 0.18719E+01_JPRB, 0.14253E+01_JPRB, 0.17644E+01_JPRB, 0.14890E+01_JPRB, 0.20861E+01_JPRB, 0.39715E+01_JPRB ,&
&  0.52479E+01_JPRB, 0.65350E+01_JPRB, 0.10556E+02_JPRB, 0.89380E+01_JPRB, 0.84255E+01_JPRB, 0.78817E+01_JPRB, 0.65820E+01_JPRB ,&
&  0.58494E+01_JPRB, 0.46648E+01_JPRB, 0.46851E+01_JPRB, 0.42035E+01_JPRB, 0.38835E+01_JPRB, 0.36469E+01_JPRB, 0.32442E+01_JPRB ,&
&  0.28132E+01_JPRB, 0.24327E+01_JPRB, 0.21369E+01_JPRB, 0.19575E+01_JPRB, 0.18790E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.16758E+01_JPRB, 0.15750E+01_JPRB, 0.15173E+01_JPRB, 0.14161E+01_JPRB, 0.13371E+01_JPRB, 0.13122E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.12958E+01_JPRB, 0.12527E+01_JPRB, 0.12475E+01_JPRB, 0.12631E+01_JPRB, 0.12521E+01_JPRB, 0.11888E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.10953E+01_JPRB, 0.11620E+01_JPRB, 0.13045E+01_JPRB, 0.18038E+01_JPRB, 0.20439E+01_JPRB, 0.26082E+01_JPRB ,&
&  0.33622E+01_JPRB, 0.32364E+01_JPRB, 0.31888E+01_JPRB, 0.36680E+01_JPRB, 0.41902E+01_JPRB, 0.43275E+01_JPRB, 0.44258E+01_JPRB ,&
&  0.37633E+01_JPRB, 0.31180E+01_JPRB, 0.27147E+01_JPRB, 0.24135E+01_JPRB, 0.20771E+01_JPRB, 0.18104E+01_JPRB, 0.16304E+01_JPRB ,&
&  0.15467E+01_JPRB, 0.15066E+01_JPRB, 0.15493E+01_JPRB, 0.14935E+01_JPRB, 0.15016E+01_JPRB, 0.15497E+01_JPRB, 0.17087E+01_JPRB ,&
&  0.18994E+01_JPRB, 0.21273E+01_JPRB, 0.25241E+01_JPRB, 0.32885E+01_JPRB, 0.47341E+01_JPRB, 0.56374E+01_JPRB, 0.55234E+01_JPRB ,&
&  0.51241E+01_JPRB, 0.50349E+01_JPRB, 0.56352E+01_JPRB, 0.61414E+01_JPRB, 0.57224E+01_JPRB, 0.62483E+01_JPRB, 0.62440E+01_JPRB ,&
&  0.55017E+01_JPRB, 0.49619E+01_JPRB, 0.44481E+01_JPRB, 0.51952E+01_JPRB, 0.51707E+01_JPRB, 0.47803E+01_JPRB, 0.53591E+01_JPRB ,&
&  0.52012E+01_JPRB, 0.44934E+01_JPRB, 0.36828E+01_JPRB, 0.31408E+01_JPRB, 0.31657E+01_JPRB, 0.28686E+01_JPRB, 0.21391E+01_JPRB ,&
&  0.18117E+01_JPRB, 0.15176E+01_JPRB, 0.13064E+01_JPRB, 0.13490E+01_JPRB, 0.12521E+01_JPRB, 0.20092E+01_JPRB, 0.36320E+01_JPRB ,&
&  0.61858E+01_JPRB, 0.75065E+01_JPRB, 0.92244E+01_JPRB, 0.82615E+01_JPRB, 0.76654E+01_JPRB, 0.80475E+01_JPRB, 0.76296E+01_JPRB ,&
&  0.50424E+01_JPRB, 0.42806E+01_JPRB, 0.42717E+01_JPRB, 0.39936E+01_JPRB, 0.37363E+01_JPRB, 0.34776E+01_JPRB, 0.30831E+01_JPRB ,&
&  0.26316E+01_JPRB, 0.22253E+01_JPRB, 0.20097E+01_JPRB, 0.19237E+01_JPRB, 0.18246E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.13918E+01_JPRB, 0.13667E+01_JPRB, 0.13858E+01_JPRB, 0.13632E+01_JPRB, 0.13281E+01_JPRB, 0.13364E+01_JPRB, 0.13289E+01_JPRB ,&
&  0.12798E+01_JPRB, 0.12112E+01_JPRB, 0.12004E+01_JPRB, 0.12257E+01_JPRB, 0.12301E+01_JPRB, 0.12255E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.11267E+01_JPRB, 0.11165E+01_JPRB, 0.12431E+01_JPRB, 0.15500E+01_JPRB, 0.24337E+01_JPRB, 0.25900E+01_JPRB, 0.34866E+01_JPRB ,&
&  0.36856E+01_JPRB, 0.31117E+01_JPRB, 0.31330E+01_JPRB, 0.31114E+01_JPRB, 0.35482E+01_JPRB, 0.33283E+01_JPRB, 0.29190E+01_JPRB ,&
&  0.23205E+01_JPRB, 0.22559E+01_JPRB, 0.21562E+01_JPRB, 0.20070E+01_JPRB, 0.18272E+01_JPRB, 0.16868E+01_JPRB, 0.16194E+01_JPRB ,&
&  0.16174E+01_JPRB, 0.16277E+01_JPRB, 0.15522E+01_JPRB, 0.15271E+01_JPRB, 0.15809E+01_JPRB, 0.16494E+01_JPRB, 0.18041E+01_JPRB ,&
&  0.20122E+01_JPRB, 0.22502E+01_JPRB, 0.26225E+01_JPRB, 0.37622E+01_JPRB, 0.45219E+01_JPRB, 0.47500E+01_JPRB, 0.53273E+01_JPRB ,&
&  0.56397E+01_JPRB, 0.58977E+01_JPRB, 0.54075E+01_JPRB, 0.62678E+01_JPRB, 0.56435E+01_JPRB, 0.57390E+01_JPRB, 0.57984E+01_JPRB ,&
&  0.52340E+01_JPRB, 0.51410E+01_JPRB, 0.44580E+01_JPRB, 0.45385E+01_JPRB, 0.52986E+01_JPRB, 0.53181E+01_JPRB, 0.50858E+01_JPRB ,&
&  0.51740E+01_JPRB, 0.45304E+01_JPRB, 0.39476E+01_JPRB, 0.37209E+01_JPRB, 0.48904E+01_JPRB, 0.55640E+01_JPRB, 0.26628E+01_JPRB ,&
&  0.18427E+01_JPRB, 0.17045E+01_JPRB, 0.13503E+01_JPRB, 0.11881E+01_JPRB, 0.13273E+01_JPRB, 0.20752E+01_JPRB, 0.45877E+01_JPRB ,&
&  0.53899E+01_JPRB, 0.88476E+01_JPRB, 0.93914E+01_JPRB, 0.95346E+01_JPRB, 0.85983E+01_JPRB, 0.76860E+01_JPRB, 0.58676E+01_JPRB ,&
&  0.57266E+01_JPRB, 0.50452E+01_JPRB, 0.43004E+01_JPRB, 0.38803E+01_JPRB, 0.34447E+01_JPRB, 0.30162E+01_JPRB, 0.26823E+01_JPRB ,&
&  0.24155E+01_JPRB, 0.21072E+01_JPRB, 0.19129E+01_JPRB, 0.17676E+01_JPRB, 0.15517E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.11493E+01_JPRB, 0.11897E+01_JPRB, 0.12611E+01_JPRB, 0.12797E+01_JPRB, 0.12660E+01_JPRB, 0.12863E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.11920E+01_JPRB, 0.11760E+01_JPRB, 0.11923E+01_JPRB, 0.12253E+01_JPRB, 0.12665E+01_JPRB, 0.12927E+01_JPRB, 0.12475E+01_JPRB ,&
&  0.11921E+01_JPRB, 0.11877E+01_JPRB, 0.13267E+01_JPRB, 0.20721E+01_JPRB, 0.29245E+01_JPRB, 0.37616E+01_JPRB, 0.42222E+01_JPRB ,&
&  0.38340E+01_JPRB, 0.30626E+01_JPRB, 0.30034E+01_JPRB, 0.30255E+01_JPRB, 0.28377E+01_JPRB, 0.27824E+01_JPRB, 0.20999E+01_JPRB ,&
&  0.17938E+01_JPRB, 0.17728E+01_JPRB, 0.17052E+01_JPRB, 0.16906E+01_JPRB, 0.16559E+01_JPRB, 0.16176E+01_JPRB, 0.16461E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.16281E+01_JPRB, 0.16239E+01_JPRB, 0.16564E+01_JPRB, 0.17258E+01_JPRB, 0.17886E+01_JPRB, 0.19240E+01_JPRB ,&
&  0.21464E+01_JPRB, 0.24429E+01_JPRB, 0.29635E+01_JPRB, 0.41614E+01_JPRB, 0.40337E+01_JPRB, 0.51701E+01_JPRB, 0.54199E+01_JPRB ,&
&  0.55154E+01_JPRB, 0.55546E+01_JPRB, 0.51681E+01_JPRB, 0.55465E+01_JPRB, 0.54113E+01_JPRB, 0.54846E+01_JPRB, 0.52677E+01_JPRB ,&
&  0.51126E+01_JPRB, 0.45811E+01_JPRB, 0.48023E+01_JPRB, 0.46813E+01_JPRB, 0.49543E+01_JPRB, 0.66278E+01_JPRB, 0.51315E+01_JPRB ,&
&  0.45061E+01_JPRB, 0.48724E+01_JPRB, 0.44405E+01_JPRB, 0.44408E+01_JPRB, 0.55699E+01_JPRB, 0.69859E+01_JPRB, 0.60112E+01_JPRB ,&
&  0.23650E+01_JPRB, 0.16426E+01_JPRB, 0.15172E+01_JPRB, 0.13473E+01_JPRB, 0.11785E+01_JPRB, 0.19721E+01_JPRB, 0.94463E+01_JPRB ,&
&  0.66278E+01_JPRB, 0.92052E+01_JPRB, 0.93202E+01_JPRB, 0.89257E+01_JPRB, 0.84469E+01_JPRB, 0.70228E+01_JPRB, 0.59325E+01_JPRB ,&
&  0.47998E+01_JPRB, 0.39100E+01_JPRB, 0.32580E+01_JPRB, 0.27681E+01_JPRB, 0.24555E+01_JPRB, 0.22927E+01_JPRB, 0.22311E+01_JPRB ,&
&  0.21287E+01_JPRB, 0.19032E+01_JPRB, 0.16876E+01_JPRB, 0.14156E+01_JPRB, 0.11925E+01_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.11578E+01_JPRB, 0.11802E+01_JPRB, 0.12244E+01_JPRB, 0.12880E+01_JPRB, 0.13276E+01_JPRB, 0.13173E+01_JPRB, 0.12738E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.13180E+01_JPRB, 0.13893E+01_JPRB, 0.14303E+01_JPRB, 0.14778E+01_JPRB, 0.14588E+01_JPRB, 0.13985E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.13628E+01_JPRB, 0.17555E+01_JPRB, 0.26514E+01_JPRB, 0.33081E+01_JPRB, 0.38815E+01_JPRB, 0.45323E+01_JPRB ,&
&  0.43606E+01_JPRB, 0.29442E+01_JPRB, 0.23460E+01_JPRB, 0.24852E+01_JPRB, 0.25647E+01_JPRB, 0.27497E+01_JPRB, 0.19572E+01_JPRB ,&
&  0.17107E+01_JPRB, 0.16443E+01_JPRB, 0.15864E+01_JPRB, 0.15957E+01_JPRB, 0.16163E+01_JPRB, 0.16274E+01_JPRB, 0.16623E+01_JPRB ,&
&  0.16754E+01_JPRB, 0.17280E+01_JPRB, 0.17946E+01_JPRB, 0.18600E+01_JPRB, 0.19113E+01_JPRB, 0.19564E+01_JPRB, 0.20943E+01_JPRB ,&
&  0.23488E+01_JPRB, 0.27074E+01_JPRB, 0.32462E+01_JPRB, 0.39459E+01_JPRB, 0.45136E+01_JPRB, 0.48171E+01_JPRB, 0.52204E+01_JPRB ,&
&  0.53182E+01_JPRB, 0.53964E+01_JPRB, 0.52622E+01_JPRB, 0.50661E+01_JPRB, 0.49516E+01_JPRB, 0.52497E+01_JPRB, 0.49847E+01_JPRB ,&
&  0.49128E+01_JPRB, 0.48450E+01_JPRB, 0.49810E+01_JPRB, 0.48795E+01_JPRB, 0.50757E+01_JPRB, 0.63935E+01_JPRB, 0.64433E+01_JPRB ,&
&  0.54589E+01_JPRB, 0.44997E+01_JPRB, 0.43981E+01_JPRB, 0.49459E+01_JPRB, 0.55617E+01_JPRB, 0.63638E+01_JPRB, 0.93117E+01_JPRB ,&
&  0.76611E+01_JPRB, 0.34204E+01_JPRB, 0.19675E+01_JPRB, 0.20179E+01_JPRB, 0.12289E+01_JPRB, 0.13801E+01_JPRB, 0.36662E+01_JPRB ,&
&  0.51228E+01_JPRB, 0.63718E+01_JPRB, 0.66667E+01_JPRB, 0.63203E+01_JPRB, 0.59520E+01_JPRB, 0.47020E+01_JPRB, 0.34820E+01_JPRB ,&
&  0.25480E+01_JPRB, 0.20324E+01_JPRB, 0.17666E+01_JPRB, 0.16656E+01_JPRB, 0.17358E+01_JPRB, 0.18506E+01_JPRB, 0.19264E+01_JPRB ,&
&  0.18845E+01_JPRB, 0.16769E+01_JPRB, 0.13798E+01_JPRB, 0.11607E+01_JPRB, 0.11151E+01_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.13398E+01_JPRB, 0.13336E+01_JPRB, 0.13633E+01_JPRB, 0.14281E+01_JPRB, 0.14588E+01_JPRB, 0.14810E+01_JPRB, 0.15974E+01_JPRB ,&
&  0.16871E+01_JPRB, 0.17402E+01_JPRB, 0.17817E+01_JPRB, 0.18119E+01_JPRB, 0.17895E+01_JPRB, 0.16894E+01_JPRB, 0.16099E+01_JPRB ,&
&  0.16285E+01_JPRB, 0.19346E+01_JPRB, 0.26079E+01_JPRB, 0.35604E+01_JPRB, 0.40931E+01_JPRB, 0.53350E+01_JPRB, 0.42817E+01_JPRB ,&
&  0.41242E+01_JPRB, 0.34928E+01_JPRB, 0.24007E+01_JPRB, 0.25193E+01_JPRB, 0.25577E+01_JPRB, 0.26768E+01_JPRB, 0.20015E+01_JPRB ,&
&  0.17593E+01_JPRB, 0.16926E+01_JPRB, 0.16230E+01_JPRB, 0.16292E+01_JPRB, 0.16542E+01_JPRB, 0.16984E+01_JPRB, 0.17386E+01_JPRB ,&
&  0.18232E+01_JPRB, 0.19400E+01_JPRB, 0.20349E+01_JPRB, 0.20770E+01_JPRB, 0.21042E+01_JPRB, 0.21499E+01_JPRB, 0.23445E+01_JPRB ,&
&  0.26152E+01_JPRB, 0.29751E+01_JPRB, 0.35135E+01_JPRB, 0.40423E+01_JPRB, 0.44209E+01_JPRB, 0.44945E+01_JPRB, 0.48307E+01_JPRB ,&
&  0.47398E+01_JPRB, 0.45408E+01_JPRB, 0.47290E+01_JPRB, 0.47969E+01_JPRB, 0.47652E+01_JPRB, 0.47993E+01_JPRB, 0.44927E+01_JPRB ,&
&  0.48399E+01_JPRB, 0.54636E+01_JPRB, 0.62476E+01_JPRB, 0.55918E+01_JPRB, 0.55390E+01_JPRB, 0.63511E+01_JPRB, 0.66222E+01_JPRB ,&
&  0.61159E+01_JPRB, 0.52587E+01_JPRB, 0.51005E+01_JPRB, 0.47873E+01_JPRB, 0.49327E+01_JPRB, 0.54483E+01_JPRB, 0.68853E+01_JPRB ,&
&  0.74511E+01_JPRB, 0.76064E+01_JPRB, 0.39445E+01_JPRB, 0.17398E+01_JPRB, 0.13554E+01_JPRB, 0.17796E+01_JPRB, 0.25336E+01_JPRB ,&
&  0.33424E+01_JPRB, 0.36980E+01_JPRB, 0.39165E+01_JPRB, 0.45463E+01_JPRB, 0.39204E+01_JPRB, 0.29932E+01_JPRB, 0.22873E+01_JPRB ,&
&  0.18268E+01_JPRB, 0.14964E+01_JPRB, 0.13389E+01_JPRB, 0.13550E+01_JPRB, 0.15219E+01_JPRB, 0.16784E+01_JPRB, 0.17395E+01_JPRB ,&
&  0.15930E+01_JPRB, 0.13514E+01_JPRB, 0.12424E+01_JPRB, 0.12332E+01_JPRB, 0.12853E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.15549E+01_JPRB, 0.15984E+01_JPRB, 0.16772E+01_JPRB, 0.17872E+01_JPRB, 0.18896E+01_JPRB, 0.20145E+01_JPRB, 0.21528E+01_JPRB ,&
&  0.22346E+01_JPRB, 0.22406E+01_JPRB, 0.22277E+01_JPRB, 0.21703E+01_JPRB, 0.21389E+01_JPRB, 0.21304E+01_JPRB, 0.22642E+01_JPRB ,&
&  0.26678E+01_JPRB, 0.33071E+01_JPRB, 0.39799E+01_JPRB, 0.44760E+01_JPRB, 0.47356E+01_JPRB, 0.53976E+01_JPRB, 0.53614E+01_JPRB ,&
&  0.38466E+01_JPRB, 0.45251E+01_JPRB, 0.26499E+01_JPRB, 0.28028E+01_JPRB, 0.29687E+01_JPRB, 0.33127E+01_JPRB, 0.24553E+01_JPRB ,&
&  0.19193E+01_JPRB, 0.18376E+01_JPRB, 0.18036E+01_JPRB, 0.18115E+01_JPRB, 0.17940E+01_JPRB, 0.18605E+01_JPRB, 0.19709E+01_JPRB ,&
&  0.20909E+01_JPRB, 0.21719E+01_JPRB, 0.22273E+01_JPRB, 0.22554E+01_JPRB, 0.23081E+01_JPRB, 0.23978E+01_JPRB, 0.26636E+01_JPRB ,&
&  0.29538E+01_JPRB, 0.33497E+01_JPRB, 0.37851E+01_JPRB, 0.42290E+01_JPRB, 0.41894E+01_JPRB, 0.42086E+01_JPRB, 0.44272E+01_JPRB ,&
&  0.44101E+01_JPRB, 0.42535E+01_JPRB, 0.44153E+01_JPRB, 0.43481E+01_JPRB, 0.42392E+01_JPRB, 0.44999E+01_JPRB, 0.42651E+01_JPRB ,&
&  0.51356E+01_JPRB, 0.54842E+01_JPRB, 0.68114E+01_JPRB, 0.61538E+01_JPRB, 0.56404E+01_JPRB, 0.59225E+01_JPRB, 0.55325E+01_JPRB ,&
&  0.48528E+01_JPRB, 0.47805E+01_JPRB, 0.45903E+01_JPRB, 0.42904E+01_JPRB, 0.43433E+01_JPRB, 0.46947E+01_JPRB, 0.54054E+01_JPRB ,&
&  0.57288E+01_JPRB, 0.54593E+01_JPRB, 0.37802E+01_JPRB, 0.18616E+01_JPRB, 0.14861E+01_JPRB, 0.16445E+01_JPRB, 0.21016E+01_JPRB ,&
&  0.27212E+01_JPRB, 0.29154E+01_JPRB, 0.29295E+01_JPRB, 0.26780E+01_JPRB, 0.21360E+01_JPRB, 0.17171E+01_JPRB, 0.14458E+01_JPRB ,&
&  0.13608E+01_JPRB, 0.13127E+01_JPRB, 0.13013E+01_JPRB, 0.14232E+01_JPRB, 0.15476E+01_JPRB, 0.15791E+01_JPRB, 0.14110E+01_JPRB ,&
&  0.12992E+01_JPRB, 0.12807E+01_JPRB, 0.13616E+01_JPRB, 0.14410E+01_JPRB, 0.15087E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.17876E+01_JPRB, 0.18931E+01_JPRB, 0.20343E+01_JPRB, 0.22328E+01_JPRB, 0.24960E+01_JPRB, 0.27953E+01_JPRB, 0.28246E+01_JPRB ,&
&  0.27943E+01_JPRB, 0.27208E+01_JPRB, 0.28274E+01_JPRB, 0.29411E+01_JPRB, 0.30751E+01_JPRB, 0.32815E+01_JPRB, 0.35922E+01_JPRB ,&
&  0.39861E+01_JPRB, 0.43793E+01_JPRB, 0.47734E+01_JPRB, 0.51769E+01_JPRB, 0.55919E+01_JPRB, 0.63706E+01_JPRB, 0.66284E+01_JPRB ,&
&  0.60346E+01_JPRB, 0.36229E+01_JPRB, 0.31794E+01_JPRB, 0.28264E+01_JPRB, 0.29145E+01_JPRB, 0.32644E+01_JPRB, 0.30686E+01_JPRB ,&
&  0.24571E+01_JPRB, 0.22133E+01_JPRB, 0.21225E+01_JPRB, 0.20726E+01_JPRB, 0.19789E+01_JPRB, 0.20805E+01_JPRB, 0.21873E+01_JPRB ,&
&  0.22558E+01_JPRB, 0.22986E+01_JPRB, 0.23516E+01_JPRB, 0.23969E+01_JPRB, 0.24770E+01_JPRB, 0.26408E+01_JPRB, 0.29552E+01_JPRB ,&
&  0.32327E+01_JPRB, 0.35511E+01_JPRB, 0.38666E+01_JPRB, 0.39928E+01_JPRB, 0.39202E+01_JPRB, 0.38991E+01_JPRB, 0.39829E+01_JPRB ,&
&  0.40730E+01_JPRB, 0.42340E+01_JPRB, 0.42557E+01_JPRB, 0.41808E+01_JPRB, 0.39669E+01_JPRB, 0.44212E+01_JPRB, 0.45768E+01_JPRB ,&
&  0.53078E+01_JPRB, 0.56426E+01_JPRB, 0.68111E+01_JPRB, 0.58620E+01_JPRB, 0.53221E+01_JPRB, 0.50357E+01_JPRB, 0.46643E+01_JPRB ,&
&  0.41155E+01_JPRB, 0.40910E+01_JPRB, 0.38235E+01_JPRB, 0.33065E+01_JPRB, 0.31327E+01_JPRB, 0.31581E+01_JPRB, 0.40383E+01_JPRB ,&
&  0.44456E+01_JPRB, 0.43977E+01_JPRB, 0.26949E+01_JPRB, 0.18231E+01_JPRB, 0.15603E+01_JPRB, 0.15632E+01_JPRB, 0.19356E+01_JPRB ,&
&  0.24642E+01_JPRB, 0.23946E+01_JPRB, 0.20881E+01_JPRB, 0.19518E+01_JPRB, 0.16451E+01_JPRB, 0.13694E+01_JPRB, 0.11787E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.12010E+01_JPRB, 0.12341E+01_JPRB, 0.13258E+01_JPRB, 0.13586E+01_JPRB, 0.14058E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.14315E+01_JPRB, 0.15009E+01_JPRB, 0.15687E+01_JPRB, 0.16378E+01_JPRB, 0.16982E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.18576E+01_JPRB, 0.19818E+01_JPRB, 0.21297E+01_JPRB, 0.23024E+01_JPRB, 0.24911E+01_JPRB, 0.26507E+01_JPRB, 0.27648E+01_JPRB ,&
&  0.29359E+01_JPRB, 0.31302E+01_JPRB, 0.33345E+01_JPRB, 0.35056E+01_JPRB, 0.36320E+01_JPRB, 0.37934E+01_JPRB, 0.39791E+01_JPRB ,&
&  0.41556E+01_JPRB, 0.43459E+01_JPRB, 0.45888E+01_JPRB, 0.48886E+01_JPRB, 0.52775E+01_JPRB, 0.57855E+01_JPRB, 0.60534E+01_JPRB ,&
&  0.49037E+01_JPRB, 0.29702E+01_JPRB, 0.32672E+01_JPRB, 0.29840E+01_JPRB, 0.29237E+01_JPRB, 0.29882E+01_JPRB, 0.28072E+01_JPRB ,&
&  0.25338E+01_JPRB, 0.24437E+01_JPRB, 0.24411E+01_JPRB, 0.23444E+01_JPRB, 0.21798E+01_JPRB, 0.22368E+01_JPRB, 0.22920E+01_JPRB ,&
&  0.23422E+01_JPRB, 0.23916E+01_JPRB, 0.24363E+01_JPRB, 0.24703E+01_JPRB, 0.25696E+01_JPRB, 0.27488E+01_JPRB, 0.30311E+01_JPRB ,&
&  0.33184E+01_JPRB, 0.35264E+01_JPRB, 0.37053E+01_JPRB, 0.38115E+01_JPRB, 0.37329E+01_JPRB, 0.36638E+01_JPRB, 0.37025E+01_JPRB ,&
&  0.38181E+01_JPRB, 0.39253E+01_JPRB, 0.41303E+01_JPRB, 0.43555E+01_JPRB, 0.46136E+01_JPRB, 0.44802E+01_JPRB, 0.50005E+01_JPRB ,&
&  0.54705E+01_JPRB, 0.55455E+01_JPRB, 0.58796E+01_JPRB, 0.67302E+01_JPRB, 0.45544E+01_JPRB, 0.42319E+01_JPRB, 0.37414E+01_JPRB ,&
&  0.38873E+01_JPRB, 0.36815E+01_JPRB, 0.31682E+01_JPRB, 0.24844E+01_JPRB, 0.22135E+01_JPRB, 0.20926E+01_JPRB, 0.26748E+01_JPRB ,&
&  0.34146E+01_JPRB, 0.33492E+01_JPRB, 0.23600E+01_JPRB, 0.19201E+01_JPRB, 0.17521E+01_JPRB, 0.19686E+01_JPRB, 0.22771E+01_JPRB ,&
&  0.23770E+01_JPRB, 0.20039E+01_JPRB, 0.18317E+01_JPRB, 0.19908E+01_JPRB, 0.15648E+01_JPRB, 0.11177E+01_JPRB, 0.93215E+00_JPRB ,&
&  0.93359E+00_JPRB, 0.98226E+00_JPRB, 0.10672E+01_JPRB, 0.11832E+01_JPRB, 0.12346E+01_JPRB, 0.12966E+01_JPRB, 0.13299E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.15504E+01_JPRB, 0.16281E+01_JPRB, 0.16776E+01_JPRB, 0.17635E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.17092E+01_JPRB, 0.18273E+01_JPRB, 0.19284E+01_JPRB, 0.20368E+01_JPRB, 0.21340E+01_JPRB, 0.22264E+01_JPRB, 0.23582E+01_JPRB ,&
&  0.25165E+01_JPRB, 0.26994E+01_JPRB, 0.28734E+01_JPRB, 0.30098E+01_JPRB, 0.31543E+01_JPRB, 0.32511E+01_JPRB, 0.33359E+01_JPRB ,&
&  0.33714E+01_JPRB, 0.34323E+01_JPRB, 0.34812E+01_JPRB, 0.35711E+01_JPRB, 0.37099E+01_JPRB, 0.38694E+01_JPRB, 0.40244E+01_JPRB ,&
&  0.40969E+01_JPRB, 0.38566E+01_JPRB, 0.32063E+01_JPRB, 0.22959E+01_JPRB, 0.20606E+01_JPRB, 0.23582E+01_JPRB, 0.23715E+01_JPRB ,&
&  0.23526E+01_JPRB, 0.22915E+01_JPRB, 0.22222E+01_JPRB, 0.22442E+01_JPRB, 0.22477E+01_JPRB, 0.22687E+01_JPRB, 0.22825E+01_JPRB ,&
&  0.23125E+01_JPRB, 0.23602E+01_JPRB, 0.23828E+01_JPRB, 0.24131E+01_JPRB, 0.24952E+01_JPRB, 0.26541E+01_JPRB, 0.28861E+01_JPRB ,&
&  0.31705E+01_JPRB, 0.33615E+01_JPRB, 0.34817E+01_JPRB, 0.35794E+01_JPRB, 0.35058E+01_JPRB, 0.34851E+01_JPRB, 0.35707E+01_JPRB ,&
&  0.37427E+01_JPRB, 0.39045E+01_JPRB, 0.41992E+01_JPRB, 0.45707E+01_JPRB, 0.47776E+01_JPRB, 0.45984E+01_JPRB, 0.48394E+01_JPRB ,&
&  0.51397E+01_JPRB, 0.51767E+01_JPRB, 0.51568E+01_JPRB, 0.64628E+01_JPRB, 0.42434E+01_JPRB, 0.42307E+01_JPRB, 0.40373E+01_JPRB ,&
&  0.38058E+01_JPRB, 0.36946E+01_JPRB, 0.27489E+01_JPRB, 0.19836E+01_JPRB, 0.17350E+01_JPRB, 0.16578E+01_JPRB, 0.21594E+01_JPRB ,&
&  0.27694E+01_JPRB, 0.24281E+01_JPRB, 0.19181E+01_JPRB, 0.17340E+01_JPRB, 0.17627E+01_JPRB, 0.22668E+01_JPRB, 0.25160E+01_JPRB ,&
&  0.20571E+01_JPRB, 0.17787E+01_JPRB, 0.17256E+01_JPRB, 0.21759E+01_JPRB, 0.18232E+01_JPRB, 0.98898E+00_JPRB, 0.80389E+00_JPRB ,&
&  0.81394E+00_JPRB, 0.87342E+00_JPRB, 0.94272E+00_JPRB, 0.10059E+01_JPRB, 0.10566E+01_JPRB, 0.11175E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.13836E+01_JPRB, 0.14687E+01_JPRB, 0.15503E+01_JPRB, 0.16214E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.12713E+01_JPRB, 0.13744E+01_JPRB, 0.14715E+01_JPRB, 0.15382E+01_JPRB, 0.15703E+01_JPRB, 0.15954E+01_JPRB, 0.16017E+01_JPRB ,&
&  0.16177E+01_JPRB, 0.16157E+01_JPRB, 0.16689E+01_JPRB, 0.17958E+01_JPRB, 0.20176E+01_JPRB, 0.21498E+01_JPRB, 0.22938E+01_JPRB ,&
&  0.22915E+01_JPRB, 0.22946E+01_JPRB, 0.22944E+01_JPRB, 0.23300E+01_JPRB, 0.24058E+01_JPRB, 0.25195E+01_JPRB, 0.27976E+01_JPRB ,&
&  0.33553E+01_JPRB, 0.37054E+01_JPRB, 0.35989E+01_JPRB, 0.31721E+01_JPRB, 0.25430E+01_JPRB, 0.24106E+01_JPRB, 0.27631E+01_JPRB ,&
&  0.27636E+01_JPRB, 0.24298E+01_JPRB, 0.22083E+01_JPRB, 0.21659E+01_JPRB, 0.21566E+01_JPRB, 0.21612E+01_JPRB, 0.21700E+01_JPRB ,&
&  0.21963E+01_JPRB, 0.22280E+01_JPRB, 0.22424E+01_JPRB, 0.22711E+01_JPRB, 0.23427E+01_JPRB, 0.24499E+01_JPRB, 0.25848E+01_JPRB ,&
&  0.27790E+01_JPRB, 0.29177E+01_JPRB, 0.29858E+01_JPRB, 0.30149E+01_JPRB, 0.30669E+01_JPRB, 0.32171E+01_JPRB, 0.34622E+01_JPRB ,&
&  0.37038E+01_JPRB, 0.38255E+01_JPRB, 0.41780E+01_JPRB, 0.44876E+01_JPRB, 0.44724E+01_JPRB, 0.44080E+01_JPRB, 0.44963E+01_JPRB ,&
&  0.45443E+01_JPRB, 0.45200E+01_JPRB, 0.40093E+01_JPRB, 0.55459E+01_JPRB, 0.56930E+01_JPRB, 0.48833E+01_JPRB, 0.40809E+01_JPRB ,&
&  0.37142E+01_JPRB, 0.34130E+01_JPRB, 0.27955E+01_JPRB, 0.21261E+01_JPRB, 0.18309E+01_JPRB, 0.20265E+01_JPRB, 0.23587E+01_JPRB ,&
&  0.24687E+01_JPRB, 0.19568E+01_JPRB, 0.17319E+01_JPRB, 0.16962E+01_JPRB, 0.16544E+01_JPRB, 0.22106E+01_JPRB, 0.21092E+01_JPRB ,&
&  0.16144E+01_JPRB, 0.14854E+01_JPRB, 0.15642E+01_JPRB, 0.20775E+01_JPRB, 0.27531E+01_JPRB, 0.90102E+00_JPRB, 0.69599E+00_JPRB ,&
&  0.71166E+00_JPRB, 0.74634E+00_JPRB, 0.79065E+00_JPRB, 0.81072E+00_JPRB, 0.81172E+00_JPRB, 0.85194E+00_JPRB, 0.91689E+00_JPRB ,&
&  0.95129E+00_JPRB, 0.98003E+00_JPRB, 0.10164E+01_JPRB, 0.11148E+01_JPRB, 0.11846E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.79452E+00_JPRB, 0.84603E+00_JPRB, 0.88321E+00_JPRB, 0.91899E+00_JPRB, 0.93385E+00_JPRB, 0.92332E+00_JPRB, 0.87676E+00_JPRB ,&
&  0.84230E+00_JPRB, 0.81308E+00_JPRB, 0.81395E+00_JPRB, 0.80516E+00_JPRB, 0.85180E+00_JPRB, 0.93833E+00_JPRB, 0.98726E+00_JPRB ,&
&  0.11327E+01_JPRB, 0.10462E+01_JPRB, 0.10310E+01_JPRB, 0.10529E+01_JPRB, 0.10490E+01_JPRB, 0.10930E+01_JPRB, 0.12699E+01_JPRB ,&
&  0.15424E+01_JPRB, 0.18508E+01_JPRB, 0.23020E+01_JPRB, 0.30644E+01_JPRB, 0.33597E+01_JPRB, 0.28202E+01_JPRB, 0.31467E+01_JPRB ,&
&  0.27679E+01_JPRB, 0.21905E+01_JPRB, 0.20431E+01_JPRB, 0.19203E+01_JPRB, 0.19271E+01_JPRB, 0.19321E+01_JPRB, 0.19746E+01_JPRB ,&
&  0.20236E+01_JPRB, 0.20714E+01_JPRB, 0.21035E+01_JPRB, 0.21322E+01_JPRB, 0.21742E+01_JPRB, 0.22245E+01_JPRB, 0.22994E+01_JPRB ,&
&  0.23519E+01_JPRB, 0.23733E+01_JPRB, 0.24352E+01_JPRB, 0.24503E+01_JPRB, 0.26709E+01_JPRB, 0.29543E+01_JPRB, 0.32296E+01_JPRB ,&
&  0.34138E+01_JPRB, 0.35523E+01_JPRB, 0.38535E+01_JPRB, 0.39451E+01_JPRB, 0.39636E+01_JPRB, 0.39542E+01_JPRB, 0.38351E+01_JPRB ,&
&  0.35987E+01_JPRB, 0.29636E+01_JPRB, 0.27819E+01_JPRB, 0.43290E+01_JPRB, 0.42778E+01_JPRB, 0.35176E+01_JPRB, 0.36256E+01_JPRB ,&
&  0.33708E+01_JPRB, 0.31836E+01_JPRB, 0.27794E+01_JPRB, 0.23034E+01_JPRB, 0.19690E+01_JPRB, 0.19089E+01_JPRB, 0.22251E+01_JPRB ,&
&  0.19372E+01_JPRB, 0.15859E+01_JPRB, 0.16109E+01_JPRB, 0.17055E+01_JPRB, 0.17038E+01_JPRB, 0.19136E+01_JPRB, 0.18213E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.13071E+01_JPRB, 0.15225E+01_JPRB, 0.20265E+01_JPRB, 0.14229E+01_JPRB, 0.73493E+00_JPRB, 0.60196E+00_JPRB ,&
&  0.61000E+00_JPRB, 0.62219E+00_JPRB, 0.63298E+00_JPRB, 0.63852E+00_JPRB, 0.64332E+00_JPRB, 0.66188E+00_JPRB, 0.67328E+00_JPRB ,&
&  0.68348E+00_JPRB, 0.71246E+00_JPRB, 0.72447E+00_JPRB, 0.74367E+00_JPRB, 0.77451E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.66803E+00_JPRB, 0.67116E+00_JPRB, 0.66637E+00_JPRB, 0.66980E+00_JPRB, 0.67761E+00_JPRB, 0.69721E+00_JPRB, 0.71040E+00_JPRB ,&
&  0.71312E+00_JPRB, 0.72327E+00_JPRB, 0.70702E+00_JPRB, 0.71441E+00_JPRB, 0.74513E+00_JPRB, 0.77183E+00_JPRB, 0.79964E+00_JPRB ,&
&  0.86660E+00_JPRB, 0.88202E+00_JPRB, 0.87977E+00_JPRB, 0.88835E+00_JPRB, 0.87867E+00_JPRB, 0.94344E+00_JPRB, 0.94576E+00_JPRB ,&
&  0.91360E+00_JPRB, 0.93695E+00_JPRB, 0.11034E+01_JPRB, 0.15466E+01_JPRB, 0.19980E+01_JPRB, 0.21002E+01_JPRB, 0.25475E+01_JPRB ,&
&  0.25775E+01_JPRB, 0.17500E+01_JPRB, 0.17008E+01_JPRB, 0.14420E+01_JPRB, 0.14360E+01_JPRB, 0.14914E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.15139E+01_JPRB, 0.15103E+01_JPRB, 0.15247E+01_JPRB, 0.15230E+01_JPRB, 0.15199E+01_JPRB, 0.15130E+01_JPRB ,&
&  0.14301E+01_JPRB, 0.14500E+01_JPRB, 0.15842E+01_JPRB, 0.18640E+01_JPRB, 0.21857E+01_JPRB, 0.25616E+01_JPRB, 0.27123E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.29522E+01_JPRB, 0.28324E+01_JPRB, 0.29295E+01_JPRB, 0.31337E+01_JPRB, 0.29191E+01_JPRB, 0.27533E+01_JPRB ,&
&  0.25738E+01_JPRB, 0.20489E+01_JPRB, 0.20676E+01_JPRB, 0.30389E+01_JPRB, 0.32552E+01_JPRB, 0.31270E+01_JPRB, 0.31667E+01_JPRB ,&
&  0.30122E+01_JPRB, 0.27841E+01_JPRB, 0.23944E+01_JPRB, 0.21067E+01_JPRB, 0.18655E+01_JPRB, 0.16513E+01_JPRB, 0.17213E+01_JPRB ,&
&  0.14494E+01_JPRB, 0.14377E+01_JPRB, 0.16518E+01_JPRB, 0.18296E+01_JPRB, 0.19992E+01_JPRB, 0.23497E+01_JPRB, 0.18160E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.13234E+01_JPRB, 0.19164E+01_JPRB, 0.25571E+01_JPRB, 0.11935E+01_JPRB, 0.72489E+00_JPRB, 0.52248E+00_JPRB ,&
&  0.53814E+00_JPRB, 0.55654E+00_JPRB, 0.56129E+00_JPRB, 0.56643E+00_JPRB, 0.58054E+00_JPRB, 0.59171E+00_JPRB, 0.61298E+00_JPRB ,&
&  0.62431E+00_JPRB, 0.63334E+00_JPRB, 0.64348E+00_JPRB, 0.66151E+00_JPRB, 0.66047E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.72412E+00_JPRB, 0.73767E+00_JPRB, 0.74835E+00_JPRB, 0.77203E+00_JPRB, 0.80644E+00_JPRB, 0.82232E+00_JPRB, 0.84807E+00_JPRB ,&
&  0.86659E+00_JPRB, 0.89477E+00_JPRB, 0.92319E+00_JPRB, 0.95687E+00_JPRB, 0.98601E+00_JPRB, 0.10043E+01_JPRB, 0.10421E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.10798E+01_JPRB, 0.11008E+01_JPRB, 0.11416E+01_JPRB, 0.11497E+01_JPRB, 0.11974E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.11942E+01_JPRB, 0.12143E+01_JPRB, 0.13173E+01_JPRB, 0.14417E+01_JPRB, 0.16751E+01_JPRB, 0.21580E+01_JPRB ,&
&  0.20292E+01_JPRB, 0.14842E+01_JPRB, 0.12712E+01_JPRB, 0.10723E+01_JPRB, 0.10516E+01_JPRB, 0.10585E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.10018E+01_JPRB, 0.94735E+00_JPRB, 0.91796E+00_JPRB, 0.90097E+00_JPRB, 0.92787E+00_JPRB, 0.10161E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.12081E+01_JPRB, 0.13881E+01_JPRB, 0.16371E+01_JPRB, 0.18290E+01_JPRB, 0.22646E+01_JPRB, 0.22707E+01_JPRB ,&
&  0.21848E+01_JPRB, 0.21303E+01_JPRB, 0.18874E+01_JPRB, 0.19851E+01_JPRB, 0.21597E+01_JPRB, 0.20709E+01_JPRB, 0.18447E+01_JPRB ,&
&  0.18176E+01_JPRB, 0.17404E+01_JPRB, 0.18808E+01_JPRB, 0.25462E+01_JPRB, 0.27954E+01_JPRB, 0.28663E+01_JPRB, 0.27349E+01_JPRB ,&
&  0.25403E+01_JPRB, 0.22684E+01_JPRB, 0.20236E+01_JPRB, 0.18984E+01_JPRB, 0.18317E+01_JPRB, 0.16623E+01_JPRB, 0.15036E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.14913E+01_JPRB, 0.16909E+01_JPRB, 0.18667E+01_JPRB, 0.23575E+01_JPRB, 0.21604E+01_JPRB, 0.24042E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.12737E+01_JPRB, 0.10047E+01_JPRB, 0.13432E+01_JPRB, 0.84167E+00_JPRB, 0.58447E+00_JPRB, 0.55412E+00_JPRB ,&
&  0.56839E+00_JPRB, 0.57761E+00_JPRB, 0.59185E+00_JPRB, 0.60235E+00_JPRB, 0.61562E+00_JPRB, 0.64014E+00_JPRB, 0.66601E+00_JPRB ,&
&  0.69180E+00_JPRB, 0.68481E+00_JPRB, 0.68812E+00_JPRB, 0.69739E+00_JPRB, 0.71634E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.78057E+00_JPRB, 0.80911E+00_JPRB, 0.83806E+00_JPRB, 0.86748E+00_JPRB, 0.89441E+00_JPRB, 0.92053E+00_JPRB, 0.94738E+00_JPRB ,&
&  0.97392E+00_JPRB, 0.10041E+01_JPRB, 0.10363E+01_JPRB, 0.10720E+01_JPRB, 0.11052E+01_JPRB, 0.11277E+01_JPRB, 0.11571E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.12068E+01_JPRB, 0.12107E+01_JPRB, 0.12582E+01_JPRB, 0.13036E+01_JPRB, 0.13299E+01_JPRB, 0.13696E+01_JPRB ,&
&  0.14072E+01_JPRB, 0.13696E+01_JPRB, 0.13534E+01_JPRB, 0.13901E+01_JPRB, 0.14252E+01_JPRB, 0.13813E+01_JPRB, 0.15986E+01_JPRB ,&
&  0.13057E+01_JPRB, 0.13303E+01_JPRB, 0.11806E+01_JPRB, 0.10223E+01_JPRB, 0.97068E+00_JPRB, 0.84907E+00_JPRB, 0.77955E+00_JPRB ,&
&  0.81475E+00_JPRB, 0.81191E+00_JPRB, 0.80881E+00_JPRB, 0.82829E+00_JPRB, 0.90905E+00_JPRB, 0.10695E+01_JPRB, 0.11862E+01_JPRB ,&
&  0.13422E+01_JPRB, 0.14974E+01_JPRB, 0.16810E+01_JPRB, 0.18803E+01_JPRB, 0.20930E+01_JPRB, 0.22069E+01_JPRB, 0.21399E+01_JPRB ,&
&  0.20167E+01_JPRB, 0.18971E+01_JPRB, 0.14921E+01_JPRB, 0.16681E+01_JPRB, 0.19293E+01_JPRB, 0.18867E+01_JPRB, 0.16382E+01_JPRB ,&
&  0.14992E+01_JPRB, 0.15354E+01_JPRB, 0.17797E+01_JPRB, 0.20782E+01_JPRB, 0.22567E+01_JPRB, 0.23216E+01_JPRB, 0.22114E+01_JPRB ,&
&  0.20733E+01_JPRB, 0.18289E+01_JPRB, 0.17186E+01_JPRB, 0.17533E+01_JPRB, 0.17379E+01_JPRB, 0.16424E+01_JPRB, 0.14418E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.14874E+01_JPRB, 0.16326E+01_JPRB, 0.18111E+01_JPRB, 0.20934E+01_JPRB, 0.31984E+01_JPRB, 0.32134E+01_JPRB ,&
&  0.13199E+01_JPRB, 0.10491E+01_JPRB, 0.61987E+00_JPRB, 0.71233E+00_JPRB, 0.73216E+00_JPRB, 0.66533E+00_JPRB, 0.71000E+00_JPRB ,&
&  0.74660E+00_JPRB, 0.71600E+00_JPRB, 0.70722E+00_JPRB, 0.68465E+00_JPRB, 0.68732E+00_JPRB, 0.68549E+00_JPRB, 0.70157E+00_JPRB ,&
&  0.71202E+00_JPRB, 0.71621E+00_JPRB, 0.72900E+00_JPRB, 0.74568E+00_JPRB, 0.75978E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.81233E+00_JPRB, 0.84480E+00_JPRB, 0.87757E+00_JPRB, 0.90799E+00_JPRB, 0.93819E+00_JPRB, 0.96744E+00_JPRB, 0.99776E+00_JPRB ,&
&  0.10282E+01_JPRB, 0.10608E+01_JPRB, 0.10931E+01_JPRB, 0.11326E+01_JPRB, 0.11706E+01_JPRB, 0.11956E+01_JPRB, 0.12405E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.13318E+01_JPRB, 0.13741E+01_JPRB, 0.14432E+01_JPRB, 0.15022E+01_JPRB, 0.15449E+01_JPRB, 0.16084E+01_JPRB ,&
&  0.16350E+01_JPRB, 0.16400E+01_JPRB, 0.16470E+01_JPRB, 0.16954E+01_JPRB, 0.17306E+01_JPRB, 0.19134E+01_JPRB, 0.16435E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.13966E+01_JPRB, 0.12660E+01_JPRB, 0.11706E+01_JPRB, 0.10915E+01_JPRB, 0.10216E+01_JPRB, 0.97671E+00_JPRB ,&
&  0.98597E+00_JPRB, 0.96094E+00_JPRB, 0.96797E+00_JPRB, 0.98904E+00_JPRB, 0.10578E+01_JPRB, 0.12248E+01_JPRB, 0.13608E+01_JPRB ,&
&  0.15121E+01_JPRB, 0.16883E+01_JPRB, 0.18487E+01_JPRB, 0.20099E+01_JPRB, 0.21874E+01_JPRB, 0.23608E+01_JPRB, 0.24868E+01_JPRB ,&
&  0.24826E+01_JPRB, 0.24225E+01_JPRB, 0.20845E+01_JPRB, 0.22798E+01_JPRB, 0.24301E+01_JPRB, 0.22197E+01_JPRB, 0.17564E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.12910E+01_JPRB, 0.13792E+01_JPRB, 0.16517E+01_JPRB, 0.17606E+01_JPRB, 0.17764E+01_JPRB, 0.16982E+01_JPRB ,&
&  0.15963E+01_JPRB, 0.14551E+01_JPRB, 0.14063E+01_JPRB, 0.14387E+01_JPRB, 0.15151E+01_JPRB, 0.15604E+01_JPRB, 0.14747E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.14351E+01_JPRB, 0.14391E+01_JPRB, 0.14671E+01_JPRB, 0.16418E+01_JPRB, 0.16189E+01_JPRB, 0.17911E+01_JPRB ,&
&  0.10297E+01_JPRB, 0.80102E+00_JPRB, 0.78539E+00_JPRB, 0.70306E+00_JPRB, 0.77472E+00_JPRB, 0.79089E+00_JPRB, 0.95846E+00_JPRB ,&
&  0.10941E+01_JPRB, 0.11471E+01_JPRB, 0.10781E+01_JPRB, 0.89994E+00_JPRB, 0.76329E+00_JPRB, 0.71099E+00_JPRB, 0.71453E+00_JPRB ,&
&  0.72043E+00_JPRB, 0.72350E+00_JPRB, 0.73879E+00_JPRB, 0.76242E+00_JPRB, 0.78462E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.77793E+00_JPRB, 0.80881E+00_JPRB, 0.84277E+00_JPRB, 0.87193E+00_JPRB, 0.90768E+00_JPRB, 0.95742E+00_JPRB, 0.10073E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10860E+01_JPRB, 0.11155E+01_JPRB, 0.11550E+01_JPRB, 0.12051E+01_JPRB, 0.12751E+01_JPRB, 0.13633E+01_JPRB ,&
&  0.14202E+01_JPRB, 0.15017E+01_JPRB, 0.15661E+01_JPRB, 0.16680E+01_JPRB, 0.17369E+01_JPRB, 0.18507E+01_JPRB, 0.19425E+01_JPRB ,&
&  0.19909E+01_JPRB, 0.20151E+01_JPRB, 0.20465E+01_JPRB, 0.21071E+01_JPRB, 0.21059E+01_JPRB, 0.22204E+01_JPRB, 0.17907E+01_JPRB ,&
&  0.17859E+01_JPRB, 0.16789E+01_JPRB, 0.15281E+01_JPRB, 0.14540E+01_JPRB, 0.13881E+01_JPRB, 0.13436E+01_JPRB, 0.13251E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.12497E+01_JPRB, 0.11390E+01_JPRB, 0.11106E+01_JPRB, 0.11503E+01_JPRB, 0.12875E+01_JPRB, 0.14334E+01_JPRB ,&
&  0.15910E+01_JPRB, 0.17472E+01_JPRB, 0.18995E+01_JPRB, 0.20953E+01_JPRB, 0.23535E+01_JPRB, 0.26257E+01_JPRB, 0.28894E+01_JPRB ,&
&  0.29772E+01_JPRB, 0.30164E+01_JPRB, 0.27744E+01_JPRB, 0.27904E+01_JPRB, 0.28641E+01_JPRB, 0.24142E+01_JPRB, 0.17895E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.11728E+01_JPRB, 0.12853E+01_JPRB, 0.14165E+01_JPRB, 0.13528E+01_JPRB, 0.12859E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10278E+01_JPRB, 0.10394E+01_JPRB, 0.11150E+01_JPRB, 0.12165E+01_JPRB, 0.13410E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.13629E+01_JPRB, 0.12933E+01_JPRB, 0.12433E+01_JPRB, 0.12464E+01_JPRB, 0.12520E+01_JPRB, 0.11645E+01_JPRB, 0.12922E+01_JPRB ,&
&  0.95779E+00_JPRB, 0.90344E+00_JPRB, 0.89395E+00_JPRB, 0.62249E+00_JPRB, 0.79073E+00_JPRB, 0.97880E+00_JPRB, 0.12401E+01_JPRB ,&
&  0.13621E+01_JPRB, 0.18074E+01_JPRB, 0.22004E+01_JPRB, 0.15782E+01_JPRB, 0.11850E+01_JPRB, 0.76812E+00_JPRB, 0.70125E+00_JPRB ,&
&  0.70785E+00_JPRB, 0.70294E+00_JPRB, 0.71148E+00_JPRB, 0.73088E+00_JPRB, 0.75155E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.71289E+00_JPRB, 0.73597E+00_JPRB, 0.76626E+00_JPRB, 0.77755E+00_JPRB, 0.79259E+00_JPRB, 0.81701E+00_JPRB, 0.84308E+00_JPRB ,&
&  0.86758E+00_JPRB, 0.89156E+00_JPRB, 0.91248E+00_JPRB, 0.94167E+00_JPRB, 0.97993E+00_JPRB, 0.10674E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.14297E+01_JPRB, 0.15746E+01_JPRB, 0.16645E+01_JPRB, 0.17930E+01_JPRB, 0.19364E+01_JPRB, 0.20440E+01_JPRB ,&
&  0.21638E+01_JPRB, 0.22256E+01_JPRB, 0.22841E+01_JPRB, 0.24226E+01_JPRB, 0.23777E+01_JPRB, 0.24404E+01_JPRB, 0.18552E+01_JPRB ,&
&  0.18972E+01_JPRB, 0.18664E+01_JPRB, 0.17705E+01_JPRB, 0.17200E+01_JPRB, 0.16472E+01_JPRB, 0.15437E+01_JPRB, 0.14689E+01_JPRB ,&
&  0.14150E+01_JPRB, 0.12847E+01_JPRB, 0.11375E+01_JPRB, 0.11298E+01_JPRB, 0.11449E+01_JPRB, 0.12046E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.15488E+01_JPRB, 0.17043E+01_JPRB, 0.18546E+01_JPRB, 0.20738E+01_JPRB, 0.23686E+01_JPRB, 0.26828E+01_JPRB, 0.30110E+01_JPRB ,&
&  0.32290E+01_JPRB, 0.33923E+01_JPRB, 0.34335E+01_JPRB, 0.29482E+01_JPRB, 0.28187E+01_JPRB, 0.22454E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.11631E+01_JPRB, 0.11892E+01_JPRB, 0.12215E+01_JPRB, 0.10992E+01_JPRB, 0.10096E+01_JPRB, 0.90875E+00_JPRB ,&
&  0.83476E+00_JPRB, 0.81673E+00_JPRB, 0.84421E+00_JPRB, 0.93121E+00_JPRB, 0.10254E+01_JPRB, 0.11429E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.10897E+01_JPRB, 0.10655E+01_JPRB, 0.10807E+01_JPRB, 0.11321E+01_JPRB, 0.12043E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.94830E+00_JPRB, 0.77710E+00_JPRB, 0.74523E+00_JPRB, 0.79812E+00_JPRB, 0.83683E+00_JPRB, 0.10796E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.21267E+01_JPRB, 0.22342E+01_JPRB, 0.26793E+01_JPRB, 0.35353E+01_JPRB, 0.25994E+01_JPRB, 0.18581E+01_JPRB, 0.62773E+00_JPRB ,&
&  0.64782E+00_JPRB, 0.64981E+00_JPRB, 0.65637E+00_JPRB, 0.66389E+00_JPRB, 0.68219E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.58908E+00_JPRB, 0.62021E+00_JPRB, 0.61457E+00_JPRB, 0.62121E+00_JPRB, 0.63089E+00_JPRB, 0.64676E+00_JPRB, 0.67843E+00_JPRB ,&
&  0.69181E+00_JPRB, 0.74240E+00_JPRB, 0.79973E+00_JPRB, 0.85122E+00_JPRB, 0.91568E+00_JPRB, 0.10052E+01_JPRB, 0.11222E+01_JPRB ,&
&  0.12575E+01_JPRB, 0.13444E+01_JPRB, 0.14349E+01_JPRB, 0.15122E+01_JPRB, 0.16315E+01_JPRB, 0.17793E+01_JPRB, 0.18889E+01_JPRB ,&
&  0.20507E+01_JPRB, 0.21727E+01_JPRB, 0.23072E+01_JPRB, 0.24929E+01_JPRB, 0.23922E+01_JPRB, 0.23537E+01_JPRB, 0.16464E+01_JPRB ,&
&  0.18861E+01_JPRB, 0.19269E+01_JPRB, 0.18951E+01_JPRB, 0.18501E+01_JPRB, 0.17534E+01_JPRB, 0.15990E+01_JPRB, 0.14956E+01_JPRB ,&
&  0.13967E+01_JPRB, 0.12875E+01_JPRB, 0.11562E+01_JPRB, 0.10770E+01_JPRB, 0.11435E+01_JPRB, 0.11174E+01_JPRB, 0.12596E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.16183E+01_JPRB, 0.17963E+01_JPRB, 0.20085E+01_JPRB, 0.22891E+01_JPRB, 0.26110E+01_JPRB, 0.29392E+01_JPRB ,&
&  0.31843E+01_JPRB, 0.33884E+01_JPRB, 0.33538E+01_JPRB, 0.28188E+01_JPRB, 0.25349E+01_JPRB, 0.20024E+01_JPRB, 0.14504E+01_JPRB ,&
&  0.12243E+01_JPRB, 0.11543E+01_JPRB, 0.11744E+01_JPRB, 0.11412E+01_JPRB, 0.10002E+01_JPRB, 0.88377E+00_JPRB, 0.78362E+00_JPRB ,&
&  0.70034E+00_JPRB, 0.69339E+00_JPRB, 0.75598E+00_JPRB, 0.84383E+00_JPRB, 0.92062E+00_JPRB, 0.99831E+00_JPRB, 0.10014E+01_JPRB ,&
&  0.96497E+00_JPRB, 0.90487E+00_JPRB, 0.89801E+00_JPRB, 0.97766E+00_JPRB, 0.11554E+01_JPRB, 0.13356E+01_JPRB, 0.14426E+01_JPRB ,&
&  0.10140E+01_JPRB, 0.77905E+00_JPRB, 0.75118E+00_JPRB, 0.76270E+00_JPRB, 0.79823E+00_JPRB, 0.87176E+00_JPRB, 0.99184E+00_JPRB ,&
&  0.11733E+01_JPRB, 0.15625E+01_JPRB, 0.14956E+01_JPRB, 0.12147E+01_JPRB, 0.17007E+01_JPRB, 0.14279E+01_JPRB, 0.56970E+00_JPRB ,&
&  0.55187E+00_JPRB, 0.54161E+00_JPRB, 0.55282E+00_JPRB, 0.56124E+00_JPRB, 0.58020E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.54697E+00_JPRB, 0.53674E+00_JPRB, 0.51953E+00_JPRB, 0.53191E+00_JPRB, 0.53636E+00_JPRB, 0.56003E+00_JPRB, 0.58858E+00_JPRB ,&
&  0.62849E+00_JPRB, 0.70220E+00_JPRB, 0.78126E+00_JPRB, 0.85181E+00_JPRB, 0.90577E+00_JPRB, 0.96849E+00_JPRB, 0.10272E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.11276E+01_JPRB, 0.11955E+01_JPRB, 0.12836E+01_JPRB, 0.13975E+01_JPRB, 0.15308E+01_JPRB, 0.16922E+01_JPRB ,&
&  0.18728E+01_JPRB, 0.20631E+01_JPRB, 0.22986E+01_JPRB, 0.24181E+01_JPRB, 0.24991E+01_JPRB, 0.25744E+01_JPRB, 0.25715E+01_JPRB ,&
&  0.16830E+01_JPRB, 0.20708E+01_JPRB, 0.19903E+01_JPRB, 0.18876E+01_JPRB, 0.18003E+01_JPRB, 0.16592E+01_JPRB, 0.15341E+01_JPRB ,&
&  0.14416E+01_JPRB, 0.12961E+01_JPRB, 0.12233E+01_JPRB, 0.10704E+01_JPRB, 0.10702E+01_JPRB, 0.10805E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.14598E+01_JPRB, 0.16984E+01_JPRB, 0.19255E+01_JPRB, 0.21485E+01_JPRB, 0.23948E+01_JPRB, 0.27393E+01_JPRB ,&
&  0.31122E+01_JPRB, 0.33625E+01_JPRB, 0.35115E+01_JPRB, 0.30554E+01_JPRB, 0.24912E+01_JPRB, 0.21990E+01_JPRB, 0.17160E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.10883E+01_JPRB, 0.11197E+01_JPRB, 0.11153E+01_JPRB, 0.99121E+00_JPRB, 0.88527E+00_JPRB, 0.78350E+00_JPRB ,&
&  0.68614E+00_JPRB, 0.63143E+00_JPRB, 0.65185E+00_JPRB, 0.75127E+00_JPRB, 0.80229E+00_JPRB, 0.83219E+00_JPRB, 0.82281E+00_JPRB ,&
&  0.80764E+00_JPRB, 0.76411E+00_JPRB, 0.72927E+00_JPRB, 0.74777E+00_JPRB, 0.80353E+00_JPRB, 0.85014E+00_JPRB, 0.84368E+00_JPRB ,&
&  0.81273E+00_JPRB, 0.79733E+00_JPRB, 0.79228E+00_JPRB, 0.80314E+00_JPRB, 0.80658E+00_JPRB, 0.81862E+00_JPRB, 0.83468E+00_JPRB ,&
&  0.85513E+00_JPRB, 0.89102E+00_JPRB, 0.86559E+00_JPRB, 0.83359E+00_JPRB, 0.88865E+00_JPRB, 0.93557E+00_JPRB, 0.62498E+00_JPRB ,&
&  0.54261E+00_JPRB, 0.51049E+00_JPRB, 0.51666E+00_JPRB, 0.51268E+00_JPRB, 0.55602E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.55588E+00_JPRB, 0.53660E+00_JPRB, 0.52656E+00_JPRB, 0.51883E+00_JPRB, 0.50492E+00_JPRB, 0.53530E+00_JPRB, 0.54788E+00_JPRB ,&
&  0.58264E+00_JPRB, 0.63739E+00_JPRB, 0.70598E+00_JPRB, 0.75585E+00_JPRB, 0.78142E+00_JPRB, 0.82732E+00_JPRB, 0.86821E+00_JPRB ,&
&  0.89956E+00_JPRB, 0.92508E+00_JPRB, 0.96029E+00_JPRB, 0.10052E+01_JPRB, 0.10798E+01_JPRB, 0.11924E+01_JPRB, 0.13597E+01_JPRB ,&
&  0.15568E+01_JPRB, 0.17710E+01_JPRB, 0.20056E+01_JPRB, 0.22303E+01_JPRB, 0.23997E+01_JPRB, 0.25330E+01_JPRB, 0.24879E+01_JPRB ,&
&  0.13553E+01_JPRB, 0.19333E+01_JPRB, 0.22305E+01_JPRB, 0.20366E+01_JPRB, 0.18708E+01_JPRB, 0.17078E+01_JPRB, 0.15466E+01_JPRB ,&
&  0.14034E+01_JPRB, 0.12646E+01_JPRB, 0.11998E+01_JPRB, 0.11741E+01_JPRB, 0.10351E+01_JPRB, 0.10372E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.12610E+01_JPRB, 0.15142E+01_JPRB, 0.17946E+01_JPRB, 0.19503E+01_JPRB, 0.21051E+01_JPRB, 0.23938E+01_JPRB ,&
&  0.28272E+01_JPRB, 0.31559E+01_JPRB, 0.33892E+01_JPRB, 0.26097E+01_JPRB, 0.22693E+01_JPRB, 0.22103E+01_JPRB, 0.20224E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.11070E+01_JPRB, 0.11252E+01_JPRB, 0.11353E+01_JPRB, 0.10197E+01_JPRB, 0.94275E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.74095E+00_JPRB, 0.66059E+00_JPRB, 0.60138E+00_JPRB, 0.63356E+00_JPRB, 0.66692E+00_JPRB, 0.68816E+00_JPRB, 0.66071E+00_JPRB ,&
&  0.69864E+00_JPRB, 0.68984E+00_JPRB, 0.68814E+00_JPRB, 0.69960E+00_JPRB, 0.77263E+00_JPRB, 0.80781E+00_JPRB, 0.82491E+00_JPRB ,&
&  0.84486E+00_JPRB, 0.85999E+00_JPRB, 0.85972E+00_JPRB, 0.86360E+00_JPRB, 0.86491E+00_JPRB, 0.85655E+00_JPRB, 0.84290E+00_JPRB ,&
&  0.81872E+00_JPRB, 0.80320E+00_JPRB, 0.76960E+00_JPRB, 0.73275E+00_JPRB, 0.68000E+00_JPRB, 0.68215E+00_JPRB, 0.61649E+00_JPRB ,&
&  0.57234E+00_JPRB, 0.54289E+00_JPRB, 0.52440E+00_JPRB, 0.52108E+00_JPRB, 0.53222E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.57429E+00_JPRB, 0.57021E+00_JPRB, 0.56457E+00_JPRB, 0.55303E+00_JPRB, 0.53820E+00_JPRB, 0.53880E+00_JPRB, 0.52807E+00_JPRB ,&
&  0.53063E+00_JPRB, 0.56057E+00_JPRB, 0.60528E+00_JPRB, 0.64160E+00_JPRB, 0.67587E+00_JPRB, 0.70198E+00_JPRB, 0.73543E+00_JPRB ,&
&  0.74947E+00_JPRB, 0.75326E+00_JPRB, 0.75260E+00_JPRB, 0.75584E+00_JPRB, 0.77570E+00_JPRB, 0.83701E+00_JPRB, 0.94164E+00_JPRB ,&
&  0.10927E+01_JPRB, 0.12944E+01_JPRB, 0.15502E+01_JPRB, 0.18380E+01_JPRB, 0.21411E+01_JPRB, 0.24459E+01_JPRB, 0.26152E+01_JPRB ,&
&  0.16646E+01_JPRB, 0.80456E+00_JPRB, 0.20881E+01_JPRB, 0.23054E+01_JPRB, 0.20799E+01_JPRB, 0.19331E+01_JPRB, 0.16838E+01_JPRB ,&
&  0.14583E+01_JPRB, 0.13677E+01_JPRB, 0.12450E+01_JPRB, 0.11559E+01_JPRB, 0.10328E+01_JPRB, 0.10142E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.11048E+01_JPRB, 0.12801E+01_JPRB, 0.15802E+01_JPRB, 0.17822E+01_JPRB, 0.18623E+01_JPRB, 0.20515E+01_JPRB ,&
&  0.24257E+01_JPRB, 0.27377E+01_JPRB, 0.28596E+01_JPRB, 0.20181E+01_JPRB, 0.15241E+01_JPRB, 0.14067E+01_JPRB, 0.13323E+01_JPRB ,&
&  0.12938E+01_JPRB, 0.11459E+01_JPRB, 0.11362E+01_JPRB, 0.11575E+01_JPRB, 0.10804E+01_JPRB, 0.97822E+00_JPRB, 0.96056E+00_JPRB ,&
&  0.87407E+00_JPRB, 0.78738E+00_JPRB, 0.70478E+00_JPRB, 0.66321E+00_JPRB, 0.63799E+00_JPRB, 0.64579E+00_JPRB, 0.64328E+00_JPRB ,&
&  0.64713E+00_JPRB, 0.67071E+00_JPRB, 0.68293E+00_JPRB, 0.70124E+00_JPRB, 0.75095E+00_JPRB, 0.80201E+00_JPRB, 0.83688E+00_JPRB ,&
&  0.86340E+00_JPRB, 0.89221E+00_JPRB, 0.90823E+00_JPRB, 0.91282E+00_JPRB, 0.91051E+00_JPRB, 0.88678E+00_JPRB, 0.88459E+00_JPRB ,&
&  0.88233E+00_JPRB, 0.87056E+00_JPRB, 0.83158E+00_JPRB, 0.80480E+00_JPRB, 0.74010E+00_JPRB, 0.69284E+00_JPRB, 0.62345E+00_JPRB ,&
&  0.59522E+00_JPRB, 0.55426E+00_JPRB, 0.52719E+00_JPRB, 0.52517E+00_JPRB, 0.54030E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.58612E+00_JPRB, 0.58598E+00_JPRB, 0.58835E+00_JPRB, 0.59157E+00_JPRB, 0.58740E+00_JPRB, 0.58893E+00_JPRB, 0.56987E+00_JPRB ,&
&  0.54083E+00_JPRB, 0.53232E+00_JPRB, 0.55458E+00_JPRB, 0.55938E+00_JPRB, 0.56171E+00_JPRB, 0.58773E+00_JPRB, 0.61739E+00_JPRB ,&
&  0.63482E+00_JPRB, 0.64239E+00_JPRB, 0.64154E+00_JPRB, 0.64116E+00_JPRB, 0.63104E+00_JPRB, 0.63549E+00_JPRB, 0.67180E+00_JPRB ,&
&  0.73859E+00_JPRB, 0.86041E+00_JPRB, 0.10444E+01_JPRB, 0.12795E+01_JPRB, 0.15633E+01_JPRB, 0.19737E+01_JPRB, 0.23755E+01_JPRB ,&
&  0.31153E+01_JPRB, 0.20752E+01_JPRB, 0.11618E+01_JPRB, 0.21061E+01_JPRB, 0.23610E+01_JPRB, 0.23220E+01_JPRB, 0.19047E+01_JPRB ,&
&  0.17624E+01_JPRB, 0.16572E+01_JPRB, 0.15335E+01_JPRB, 0.13697E+01_JPRB, 0.11432E+01_JPRB, 0.10539E+01_JPRB, 0.97574E+00_JPRB ,&
&  0.93595E+00_JPRB, 0.95941E+00_JPRB, 0.10561E+01_JPRB, 0.12666E+01_JPRB, 0.15384E+01_JPRB, 0.16773E+01_JPRB, 0.17624E+01_JPRB ,&
&  0.20214E+01_JPRB, 0.23410E+01_JPRB, 0.24085E+01_JPRB, 0.17024E+01_JPRB, 0.13576E+01_JPRB, 0.12745E+01_JPRB, 0.12584E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12411E+01_JPRB, 0.12673E+01_JPRB, 0.11947E+01_JPRB, 0.10990E+01_JPRB, 0.10281E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.96506E+00_JPRB, 0.89817E+00_JPRB, 0.83033E+00_JPRB, 0.75011E+00_JPRB, 0.67723E+00_JPRB, 0.66046E+00_JPRB, 0.65245E+00_JPRB ,&
&  0.66087E+00_JPRB, 0.68524E+00_JPRB, 0.69125E+00_JPRB, 0.71598E+00_JPRB, 0.74439E+00_JPRB, 0.77606E+00_JPRB, 0.80831E+00_JPRB ,&
&  0.83623E+00_JPRB, 0.86749E+00_JPRB, 0.89561E+00_JPRB, 0.90209E+00_JPRB, 0.89285E+00_JPRB, 0.87728E+00_JPRB, 0.89566E+00_JPRB ,&
&  0.90232E+00_JPRB, 0.91126E+00_JPRB, 0.88394E+00_JPRB, 0.88055E+00_JPRB, 0.82359E+00_JPRB, 0.77026E+00_JPRB, 0.70902E+00_JPRB ,&
&  0.69228E+00_JPRB, 0.64775E+00_JPRB, 0.61287E+00_JPRB, 0.57144E+00_JPRB, 0.57612E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.60919E+00_JPRB, 0.60147E+00_JPRB, 0.60194E+00_JPRB, 0.60486E+00_JPRB, 0.60427E+00_JPRB, 0.60695E+00_JPRB, 0.60834E+00_JPRB ,&
&  0.57838E+00_JPRB, 0.55015E+00_JPRB, 0.53303E+00_JPRB, 0.52213E+00_JPRB, 0.51680E+00_JPRB, 0.53234E+00_JPRB, 0.54196E+00_JPRB ,&
&  0.54276E+00_JPRB, 0.54370E+00_JPRB, 0.56702E+00_JPRB, 0.57499E+00_JPRB, 0.57394E+00_JPRB, 0.56657E+00_JPRB, 0.56658E+00_JPRB ,&
&  0.58546E+00_JPRB, 0.63194E+00_JPRB, 0.72582E+00_JPRB, 0.85638E+00_JPRB, 0.10062E+01_JPRB, 0.12751E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.22235E+01_JPRB, 0.22986E+01_JPRB, 0.10126E+01_JPRB, 0.18130E+01_JPRB, 0.24868E+01_JPRB, 0.25509E+01_JPRB, 0.23404E+01_JPRB ,&
&  0.21504E+01_JPRB, 0.19800E+01_JPRB, 0.20132E+01_JPRB, 0.18127E+01_JPRB, 0.14033E+01_JPRB, 0.11579E+01_JPRB, 0.99304E+00_JPRB ,&
&  0.89716E+00_JPRB, 0.86247E+00_JPRB, 0.88789E+00_JPRB, 0.99360E+00_JPRB, 0.12127E+01_JPRB, 0.14185E+01_JPRB, 0.15201E+01_JPRB ,&
&  0.16455E+01_JPRB, 0.19537E+01_JPRB, 0.21563E+01_JPRB, 0.15859E+01_JPRB, 0.13467E+01_JPRB, 0.13772E+01_JPRB, 0.13701E+01_JPRB ,&
&  0.13263E+01_JPRB, 0.13299E+01_JPRB, 0.13189E+01_JPRB, 0.12351E+01_JPRB, 0.11292E+01_JPRB, 0.10857E+01_JPRB, 0.10959E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.97051E+00_JPRB, 0.91454E+00_JPRB, 0.84663E+00_JPRB, 0.76437E+00_JPRB, 0.69698E+00_JPRB, 0.69077E+00_JPRB ,&
&  0.69374E+00_JPRB, 0.71424E+00_JPRB, 0.71751E+00_JPRB, 0.73420E+00_JPRB, 0.75108E+00_JPRB, 0.76960E+00_JPRB, 0.78722E+00_JPRB ,&
&  0.80980E+00_JPRB, 0.83453E+00_JPRB, 0.85753E+00_JPRB, 0.85632E+00_JPRB, 0.85417E+00_JPRB, 0.85594E+00_JPRB, 0.86470E+00_JPRB ,&
&  0.88374E+00_JPRB, 0.90275E+00_JPRB, 0.87846E+00_JPRB, 0.88099E+00_JPRB, 0.89190E+00_JPRB, 0.85901E+00_JPRB, 0.81656E+00_JPRB ,&
&  0.84587E+00_JPRB, 0.87496E+00_JPRB, 0.80809E+00_JPRB, 0.65944E+00_JPRB, 0.62707E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.65183E+00_JPRB, 0.63786E+00_JPRB, 0.62809E+00_JPRB, 0.62597E+00_JPRB, 0.62211E+00_JPRB, 0.62050E+00_JPRB, 0.62816E+00_JPRB ,&
&  0.61714E+00_JPRB, 0.59707E+00_JPRB, 0.57224E+00_JPRB, 0.54540E+00_JPRB, 0.52453E+00_JPRB, 0.52666E+00_JPRB, 0.53418E+00_JPRB ,&
&  0.53657E+00_JPRB, 0.53225E+00_JPRB, 0.53156E+00_JPRB, 0.53858E+00_JPRB, 0.53784E+00_JPRB, 0.53308E+00_JPRB, 0.52784E+00_JPRB ,&
&  0.52772E+00_JPRB, 0.53882E+00_JPRB, 0.57391E+00_JPRB, 0.63616E+00_JPRB, 0.71570E+00_JPRB, 0.85175E+00_JPRB, 0.10544E+01_JPRB ,&
&  0.15178E+01_JPRB, 0.18295E+01_JPRB, 0.19714E+01_JPRB, 0.29751E+01_JPRB, 0.31245E+01_JPRB, 0.29352E+01_JPRB, 0.28776E+01_JPRB ,&
&  0.27612E+01_JPRB, 0.25646E+01_JPRB, 0.27073E+01_JPRB, 0.21364E+01_JPRB, 0.16784E+01_JPRB, 0.13037E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.92837E+00_JPRB, 0.84391E+00_JPRB, 0.81740E+00_JPRB, 0.83844E+00_JPRB, 0.94571E+00_JPRB, 0.11363E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.16171E+01_JPRB, 0.19211E+01_JPRB, 0.16566E+01_JPRB, 0.14001E+01_JPRB, 0.15611E+01_JPRB, 0.17138E+01_JPRB ,&
&  0.18938E+01_JPRB, 0.15406E+01_JPRB, 0.13995E+01_JPRB, 0.12954E+01_JPRB, 0.11647E+01_JPRB, 0.12340E+01_JPRB, 0.11434E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10224E+01_JPRB, 0.96906E+00_JPRB, 0.91716E+00_JPRB, 0.85581E+00_JPRB, 0.78912E+00_JPRB, 0.75269E+00_JPRB ,&
&  0.74905E+00_JPRB, 0.75726E+00_JPRB, 0.75658E+00_JPRB, 0.76332E+00_JPRB, 0.76426E+00_JPRB, 0.77627E+00_JPRB, 0.78940E+00_JPRB ,&
&  0.80889E+00_JPRB, 0.82091E+00_JPRB, 0.80215E+00_JPRB, 0.79883E+00_JPRB, 0.80002E+00_JPRB, 0.79931E+00_JPRB, 0.79530E+00_JPRB ,&
&  0.80890E+00_JPRB, 0.84082E+00_JPRB, 0.85862E+00_JPRB, 0.87329E+00_JPRB, 0.88583E+00_JPRB, 0.86526E+00_JPRB, 0.81491E+00_JPRB ,&
&  0.79450E+00_JPRB, 0.79392E+00_JPRB, 0.77849E+00_JPRB, 0.71007E+00_JPRB, 0.67679E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.67297E+00_JPRB, 0.65968E+00_JPRB, 0.65084E+00_JPRB, 0.64115E+00_JPRB, 0.62971E+00_JPRB, 0.62628E+00_JPRB, 0.63171E+00_JPRB ,&
&  0.62983E+00_JPRB, 0.61727E+00_JPRB, 0.59941E+00_JPRB, 0.58138E+00_JPRB, 0.55728E+00_JPRB, 0.54406E+00_JPRB, 0.53654E+00_JPRB ,&
&  0.53148E+00_JPRB, 0.52941E+00_JPRB, 0.52310E+00_JPRB, 0.52096E+00_JPRB, 0.51922E+00_JPRB, 0.50896E+00_JPRB, 0.50176E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.50284E+00_JPRB, 0.51674E+00_JPRB, 0.54840E+00_JPRB, 0.59791E+00_JPRB, 0.68443E+00_JPRB, 0.79813E+00_JPRB ,&
&  0.10887E+01_JPRB, 0.15437E+01_JPRB, 0.14282E+01_JPRB, 0.31225E+01_JPRB, 0.30940E+01_JPRB, 0.30329E+01_JPRB, 0.28599E+01_JPRB ,&
&  0.28414E+01_JPRB, 0.31492E+01_JPRB, 0.27442E+01_JPRB, 0.22623E+01_JPRB, 0.17649E+01_JPRB, 0.14059E+01_JPRB, 0.11394E+01_JPRB ,&
&  0.97322E+00_JPRB, 0.87561E+00_JPRB, 0.82831E+00_JPRB, 0.80532E+00_JPRB, 0.80639E+00_JPRB, 0.90559E+00_JPRB, 0.10727E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.13620E+01_JPRB, 0.16478E+01_JPRB, 0.15741E+01_JPRB, 0.14807E+01_JPRB, 0.16893E+01_JPRB, 0.20802E+01_JPRB ,&
&  0.24487E+01_JPRB, 0.19930E+01_JPRB, 0.15836E+01_JPRB, 0.14042E+01_JPRB, 0.12966E+01_JPRB, 0.12779E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.10999E+01_JPRB, 0.10476E+01_JPRB, 0.10045E+01_JPRB, 0.96482E+00_JPRB, 0.91727E+00_JPRB, 0.86900E+00_JPRB, 0.81855E+00_JPRB ,&
&  0.79765E+00_JPRB, 0.79259E+00_JPRB, 0.79142E+00_JPRB, 0.79913E+00_JPRB, 0.79938E+00_JPRB, 0.80651E+00_JPRB, 0.81194E+00_JPRB ,&
&  0.82488E+00_JPRB, 0.82579E+00_JPRB, 0.78511E+00_JPRB, 0.76421E+00_JPRB, 0.76055E+00_JPRB, 0.75488E+00_JPRB, 0.75065E+00_JPRB ,&
&  0.77201E+00_JPRB, 0.80033E+00_JPRB, 0.81763E+00_JPRB, 0.82716E+00_JPRB, 0.87077E+00_JPRB, 0.88203E+00_JPRB, 0.82396E+00_JPRB ,&
&  0.78883E+00_JPRB, 0.76048E+00_JPRB, 0.73532E+00_JPRB, 0.70443E+00_JPRB, 0.69004E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.67911E+00_JPRB, 0.66889E+00_JPRB, 0.65951E+00_JPRB, 0.64721E+00_JPRB, 0.63036E+00_JPRB, 0.62116E+00_JPRB, 0.62325E+00_JPRB ,&
&  0.62769E+00_JPRB, 0.62006E+00_JPRB, 0.60823E+00_JPRB, 0.59143E+00_JPRB, 0.56856E+00_JPRB, 0.55554E+00_JPRB, 0.54646E+00_JPRB ,&
&  0.53321E+00_JPRB, 0.52204E+00_JPRB, 0.50677E+00_JPRB, 0.50024E+00_JPRB, 0.49962E+00_JPRB, 0.49248E+00_JPRB, 0.47804E+00_JPRB ,&
&  0.47935E+00_JPRB, 0.48836E+00_JPRB, 0.50003E+00_JPRB, 0.51612E+00_JPRB, 0.55110E+00_JPRB, 0.61548E+00_JPRB, 0.71686E+00_JPRB ,&
&  0.92831E+00_JPRB, 0.18155E+01_JPRB, 0.13144E+01_JPRB, 0.24091E+01_JPRB, 0.24858E+01_JPRB, 0.26627E+01_JPRB, 0.27177E+01_JPRB ,&
&  0.27924E+01_JPRB, 0.28390E+01_JPRB, 0.26159E+01_JPRB, 0.21362E+01_JPRB, 0.16661E+01_JPRB, 0.13610E+01_JPRB, 0.11672E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.94086E+00_JPRB, 0.87117E+00_JPRB, 0.82476E+00_JPRB, 0.79392E+00_JPRB, 0.79446E+00_JPRB, 0.87378E+00_JPRB ,&
&  0.10180E+01_JPRB, 0.11537E+01_JPRB, 0.13729E+01_JPRB, 0.15964E+01_JPRB, 0.14783E+01_JPRB, 0.16191E+01_JPRB, 0.21891E+01_JPRB ,&
&  0.25788E+01_JPRB, 0.26388E+01_JPRB, 0.19123E+01_JPRB, 0.15875E+01_JPRB, 0.14506E+01_JPRB, 0.13613E+01_JPRB, 0.12266E+01_JPRB ,&
&  0.11026E+01_JPRB, 0.10402E+01_JPRB, 0.10045E+01_JPRB, 0.97716E+00_JPRB, 0.94470E+00_JPRB, 0.89816E+00_JPRB, 0.84962E+00_JPRB ,&
&  0.80878E+00_JPRB, 0.80025E+00_JPRB, 0.80103E+00_JPRB, 0.80511E+00_JPRB, 0.81424E+00_JPRB, 0.82342E+00_JPRB, 0.83005E+00_JPRB ,&
&  0.83338E+00_JPRB, 0.83473E+00_JPRB, 0.79465E+00_JPRB, 0.76475E+00_JPRB, 0.75166E+00_JPRB, 0.74123E+00_JPRB, 0.73266E+00_JPRB ,&
&  0.74791E+00_JPRB, 0.76625E+00_JPRB, 0.78399E+00_JPRB, 0.81766E+00_JPRB, 0.86301E+00_JPRB, 0.90961E+00_JPRB, 0.80332E+00_JPRB ,&
&  0.79273E+00_JPRB, 0.76633E+00_JPRB, 0.73914E+00_JPRB, 0.70943E+00_JPRB, 0.69440E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.67647E+00_JPRB, 0.66713E+00_JPRB, 0.65747E+00_JPRB, 0.64665E+00_JPRB, 0.63367E+00_JPRB, 0.62250E+00_JPRB, 0.61683E+00_JPRB ,&
&  0.61497E+00_JPRB, 0.61524E+00_JPRB, 0.60836E+00_JPRB, 0.59055E+00_JPRB, 0.57083E+00_JPRB, 0.55276E+00_JPRB, 0.54229E+00_JPRB ,&
&  0.53292E+00_JPRB, 0.52037E+00_JPRB, 0.50501E+00_JPRB, 0.49094E+00_JPRB, 0.48667E+00_JPRB, 0.48707E+00_JPRB, 0.47912E+00_JPRB ,&
&  0.48465E+00_JPRB, 0.49316E+00_JPRB, 0.50099E+00_JPRB, 0.50950E+00_JPRB, 0.53661E+00_JPRB, 0.59083E+00_JPRB, 0.68405E+00_JPRB ,&
&  0.86744E+00_JPRB, 0.75195E+00_JPRB, 0.14748E+01_JPRB, 0.18072E+01_JPRB, 0.19836E+01_JPRB, 0.21386E+01_JPRB, 0.24672E+01_JPRB ,&
&  0.27194E+01_JPRB, 0.26278E+01_JPRB, 0.23438E+01_JPRB, 0.19545E+01_JPRB, 0.16013E+01_JPRB, 0.13425E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.97856E+00_JPRB, 0.90494E+00_JPRB, 0.84200E+00_JPRB, 0.80013E+00_JPRB, 0.77633E+00_JPRB, 0.77972E+00_JPRB ,&
&  0.83263E+00_JPRB, 0.93854E+00_JPRB, 0.10870E+01_JPRB, 0.13482E+01_JPRB, 0.14629E+01_JPRB, 0.15458E+01_JPRB, 0.17592E+01_JPRB ,&
&  0.22648E+01_JPRB, 0.25879E+01_JPRB, 0.21477E+01_JPRB, 0.17785E+01_JPRB, 0.15853E+01_JPRB, 0.13998E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.10255E+01_JPRB, 0.97575E+00_JPRB, 0.94622E+00_JPRB, 0.92986E+00_JPRB, 0.90734E+00_JPRB, 0.87394E+00_JPRB ,&
&  0.84354E+00_JPRB, 0.82419E+00_JPRB, 0.81744E+00_JPRB, 0.81715E+00_JPRB, 0.81911E+00_JPRB, 0.82681E+00_JPRB, 0.82836E+00_JPRB ,&
&  0.82292E+00_JPRB, 0.81583E+00_JPRB, 0.79972E+00_JPRB, 0.77129E+00_JPRB, 0.75770E+00_JPRB, 0.74233E+00_JPRB, 0.72829E+00_JPRB ,&
&  0.73454E+00_JPRB, 0.74184E+00_JPRB, 0.75719E+00_JPRB, 0.80064E+00_JPRB, 0.82942E+00_JPRB, 0.86215E+00_JPRB, 0.78763E+00_JPRB ,&
&  0.77023E+00_JPRB, 0.74551E+00_JPRB, 0.72251E+00_JPRB, 0.70014E+00_JPRB, 0.69070E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.67607E+00_JPRB, 0.66636E+00_JPRB, 0.65440E+00_JPRB, 0.64244E+00_JPRB, 0.62878E+00_JPRB, 0.61614E+00_JPRB, 0.60787E+00_JPRB ,&
&  0.60292E+00_JPRB, 0.60167E+00_JPRB, 0.60153E+00_JPRB, 0.58862E+00_JPRB, 0.57170E+00_JPRB, 0.55576E+00_JPRB, 0.54575E+00_JPRB ,&
&  0.53804E+00_JPRB, 0.52899E+00_JPRB, 0.51790E+00_JPRB, 0.50465E+00_JPRB, 0.49797E+00_JPRB, 0.49390E+00_JPRB, 0.49083E+00_JPRB ,&
&  0.49606E+00_JPRB, 0.50062E+00_JPRB, 0.50667E+00_JPRB, 0.51572E+00_JPRB, 0.53822E+00_JPRB, 0.58722E+00_JPRB, 0.67245E+00_JPRB ,&
&  0.86485E+00_JPRB, 0.13171E+01_JPRB, 0.17081E+01_JPRB, 0.16008E+01_JPRB, 0.16090E+01_JPRB, 0.16621E+01_JPRB, 0.19607E+01_JPRB ,&
&  0.21664E+01_JPRB, 0.22295E+01_JPRB, 0.20289E+01_JPRB, 0.17540E+01_JPRB, 0.15095E+01_JPRB, 0.13189E+01_JPRB, 0.11803E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.93957E+00_JPRB, 0.85744E+00_JPRB, 0.80002E+00_JPRB, 0.76307E+00_JPRB, 0.73962E+00_JPRB, 0.72998E+00_JPRB ,&
&  0.73829E+00_JPRB, 0.77688E+00_JPRB, 0.86622E+00_JPRB, 0.10455E+01_JPRB, 0.14110E+01_JPRB, 0.14054E+01_JPRB, 0.16705E+01_JPRB ,&
&  0.21786E+01_JPRB, 0.21455E+01_JPRB, 0.19784E+01_JPRB, 0.17605E+01_JPRB, 0.15626E+01_JPRB, 0.13860E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.10466E+01_JPRB, 0.97671E+00_JPRB, 0.92564E+00_JPRB, 0.89404E+00_JPRB, 0.88210E+00_JPRB, 0.87277E+00_JPRB ,&
&  0.85931E+00_JPRB, 0.84353E+00_JPRB, 0.83228E+00_JPRB, 0.82731E+00_JPRB, 0.82208E+00_JPRB, 0.81877E+00_JPRB, 0.80226E+00_JPRB ,&
&  0.78530E+00_JPRB, 0.77238E+00_JPRB, 0.78574E+00_JPRB, 0.75832E+00_JPRB, 0.74838E+00_JPRB, 0.73741E+00_JPRB, 0.71950E+00_JPRB ,&
&  0.70516E+00_JPRB, 0.71955E+00_JPRB, 0.73962E+00_JPRB, 0.77724E+00_JPRB, 0.85395E+00_JPRB, 0.87684E+00_JPRB, 0.79371E+00_JPRB ,&
&  0.76232E+00_JPRB, 0.72738E+00_JPRB, 0.69778E+00_JPRB, 0.68430E+00_JPRB, 0.68761E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.66959E+00_JPRB, 0.65630E+00_JPRB, 0.64327E+00_JPRB, 0.63101E+00_JPRB, 0.61615E+00_JPRB, 0.60366E+00_JPRB, 0.59229E+00_JPRB ,&
&  0.58416E+00_JPRB, 0.57991E+00_JPRB, 0.58051E+00_JPRB, 0.58136E+00_JPRB, 0.57831E+00_JPRB, 0.56904E+00_JPRB, 0.55317E+00_JPRB ,&
&  0.54242E+00_JPRB, 0.53289E+00_JPRB, 0.52147E+00_JPRB, 0.50768E+00_JPRB, 0.49440E+00_JPRB, 0.48820E+00_JPRB, 0.48731E+00_JPRB ,&
&  0.49156E+00_JPRB, 0.49677E+00_JPRB, 0.50072E+00_JPRB, 0.51251E+00_JPRB, 0.53433E+00_JPRB, 0.58098E+00_JPRB, 0.65132E+00_JPRB ,&
&  0.78970E+00_JPRB, 0.12419E+01_JPRB, 0.13624E+01_JPRB, 0.13680E+01_JPRB, 0.14038E+01_JPRB, 0.14173E+01_JPRB, 0.15609E+01_JPRB ,&
&  0.16989E+01_JPRB, 0.17868E+01_JPRB, 0.17341E+01_JPRB, 0.15369E+01_JPRB, 0.13648E+01_JPRB, 0.12450E+01_JPRB, 0.11270E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.91865E+00_JPRB, 0.84458E+00_JPRB, 0.79431E+00_JPRB, 0.75581E+00_JPRB, 0.72667E+00_JPRB, 0.70937E+00_JPRB ,&
&  0.70419E+00_JPRB, 0.70981E+00_JPRB, 0.73800E+00_JPRB, 0.80451E+00_JPRB, 0.10669E+01_JPRB, 0.12951E+01_JPRB, 0.14984E+01_JPRB ,&
&  0.15732E+01_JPRB, 0.16166E+01_JPRB, 0.16306E+01_JPRB, 0.15488E+01_JPRB, 0.14271E+01_JPRB, 0.12837E+01_JPRB, 0.11752E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.10305E+01_JPRB, 0.96053E+00_JPRB, 0.89613E+00_JPRB, 0.85105E+00_JPRB, 0.82750E+00_JPRB, 0.81510E+00_JPRB ,&
&  0.80275E+00_JPRB, 0.79571E+00_JPRB, 0.79018E+00_JPRB, 0.78363E+00_JPRB, 0.77474E+00_JPRB, 0.76240E+00_JPRB, 0.74607E+00_JPRB ,&
&  0.72705E+00_JPRB, 0.70938E+00_JPRB, 0.70977E+00_JPRB, 0.70571E+00_JPRB, 0.70104E+00_JPRB, 0.69192E+00_JPRB, 0.67996E+00_JPRB ,&
&  0.68016E+00_JPRB, 0.72282E+00_JPRB, 0.73492E+00_JPRB, 0.77792E+00_JPRB, 0.82509E+00_JPRB, 0.82905E+00_JPRB, 0.77966E+00_JPRB ,&
&  0.73948E+00_JPRB, 0.70308E+00_JPRB, 0.67475E+00_JPRB, 0.66239E+00_JPRB, 0.67747E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.66828E+00_JPRB, 0.64939E+00_JPRB, 0.63011E+00_JPRB, 0.60868E+00_JPRB, 0.59045E+00_JPRB, 0.57817E+00_JPRB, 0.56934E+00_JPRB ,&
&  0.56391E+00_JPRB, 0.56115E+00_JPRB, 0.55915E+00_JPRB, 0.55706E+00_JPRB, 0.55415E+00_JPRB, 0.55045E+00_JPRB, 0.54243E+00_JPRB ,&
&  0.53229E+00_JPRB, 0.51997E+00_JPRB, 0.50996E+00_JPRB, 0.50159E+00_JPRB, 0.48650E+00_JPRB, 0.47688E+00_JPRB, 0.47759E+00_JPRB ,&
&  0.48211E+00_JPRB, 0.48743E+00_JPRB, 0.49234E+00_JPRB, 0.50268E+00_JPRB, 0.52792E+00_JPRB, 0.57507E+00_JPRB, 0.63072E+00_JPRB ,&
&  0.74912E+00_JPRB, 0.90313E+00_JPRB, 0.10427E+01_JPRB, 0.10731E+01_JPRB, 0.10950E+01_JPRB, 0.11540E+01_JPRB, 0.12394E+01_JPRB ,&
&  0.12917E+01_JPRB, 0.14030E+01_JPRB, 0.13861E+01_JPRB, 0.12583E+01_JPRB, 0.11207E+01_JPRB, 0.10208E+01_JPRB, 0.95663E+00_JPRB ,&
&  0.90855E+00_JPRB, 0.86453E+00_JPRB, 0.82710E+00_JPRB, 0.79970E+00_JPRB, 0.77356E+00_JPRB, 0.74703E+00_JPRB, 0.72120E+00_JPRB ,&
&  0.70346E+00_JPRB, 0.69209E+00_JPRB, 0.68974E+00_JPRB, 0.70645E+00_JPRB, 0.77379E+00_JPRB, 0.89217E+00_JPRB, 0.10522E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.12551E+01_JPRB, 0.13170E+01_JPRB, 0.13105E+01_JPRB, 0.12575E+01_JPRB, 0.11952E+01_JPRB, 0.11280E+01_JPRB ,&
&  0.10400E+01_JPRB, 0.94020E+00_JPRB, 0.86487E+00_JPRB, 0.81215E+00_JPRB, 0.77675E+00_JPRB, 0.75539E+00_JPRB, 0.74337E+00_JPRB ,&
&  0.73418E+00_JPRB, 0.73248E+00_JPRB, 0.74452E+00_JPRB, 0.75718E+00_JPRB, 0.75543E+00_JPRB, 0.73589E+00_JPRB, 0.70825E+00_JPRB ,&
&  0.68336E+00_JPRB, 0.66367E+00_JPRB, 0.65309E+00_JPRB, 0.64492E+00_JPRB, 0.63862E+00_JPRB, 0.63462E+00_JPRB, 0.62866E+00_JPRB ,&
&  0.62639E+00_JPRB, 0.65816E+00_JPRB, 0.68725E+00_JPRB, 0.77606E+00_JPRB, 0.82434E+00_JPRB, 0.78424E+00_JPRB, 0.73625E+00_JPRB ,&
&  0.69922E+00_JPRB, 0.67433E+00_JPRB, 0.65032E+00_JPRB, 0.63783E+00_JPRB, 0.68705E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.59134E+00_JPRB, 0.58646E+00_JPRB, 0.57440E+00_JPRB, 0.56172E+00_JPRB, 0.55083E+00_JPRB, 0.54451E+00_JPRB, 0.53912E+00_JPRB ,&
&  0.53559E+00_JPRB, 0.53326E+00_JPRB, 0.53175E+00_JPRB, 0.52898E+00_JPRB, 0.52425E+00_JPRB, 0.51711E+00_JPRB, 0.50807E+00_JPRB ,&
&  0.49965E+00_JPRB, 0.49132E+00_JPRB, 0.48406E+00_JPRB, 0.47626E+00_JPRB, 0.46638E+00_JPRB, 0.46077E+00_JPRB, 0.46299E+00_JPRB ,&
&  0.46902E+00_JPRB, 0.47626E+00_JPRB, 0.48345E+00_JPRB, 0.49293E+00_JPRB, 0.52320E+00_JPRB, 0.56649E+00_JPRB, 0.60674E+00_JPRB ,&
&  0.70950E+00_JPRB, 0.74729E+00_JPRB, 0.81134E+00_JPRB, 0.87817E+00_JPRB, 0.95049E+00_JPRB, 0.97633E+00_JPRB, 0.10008E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.10725E+01_JPRB, 0.10738E+01_JPRB, 0.10242E+01_JPRB, 0.92459E+00_JPRB, 0.82791E+00_JPRB, 0.76888E+00_JPRB ,&
&  0.73970E+00_JPRB, 0.72588E+00_JPRB, 0.71324E+00_JPRB, 0.70124E+00_JPRB, 0.69115E+00_JPRB, 0.68111E+00_JPRB, 0.67084E+00_JPRB ,&
&  0.66100E+00_JPRB, 0.65322E+00_JPRB, 0.64856E+00_JPRB, 0.64766E+00_JPRB, 0.65857E+00_JPRB, 0.69793E+00_JPRB, 0.78024E+00_JPRB ,&
&  0.89643E+00_JPRB, 0.10044E+01_JPRB, 0.10597E+01_JPRB, 0.10925E+01_JPRB, 0.11159E+01_JPRB, 0.10782E+01_JPRB, 0.97573E+00_JPRB ,&
&  0.87957E+00_JPRB, 0.79879E+00_JPRB, 0.74070E+00_JPRB, 0.69985E+00_JPRB, 0.67253E+00_JPRB, 0.65623E+00_JPRB, 0.64812E+00_JPRB ,&
&  0.64465E+00_JPRB, 0.64369E+00_JPRB, 0.64685E+00_JPRB, 0.64953E+00_JPRB, 0.65077E+00_JPRB, 0.64693E+00_JPRB, 0.63694E+00_JPRB ,&
&  0.62264E+00_JPRB, 0.61240E+00_JPRB, 0.60457E+00_JPRB, 0.59641E+00_JPRB, 0.58871E+00_JPRB, 0.58059E+00_JPRB, 0.57403E+00_JPRB ,&
&  0.57068E+00_JPRB, 0.58526E+00_JPRB, 0.61127E+00_JPRB, 0.67761E+00_JPRB, 0.67119E+00_JPRB, 0.65563E+00_JPRB, 0.63537E+00_JPRB ,&
&  0.62220E+00_JPRB, 0.61460E+00_JPRB, 0.60023E+00_JPRB, 0.59167E+00_JPRB, 0.59184E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.55266E+00_JPRB, 0.54667E+00_JPRB, 0.53577E+00_JPRB, 0.52481E+00_JPRB, 0.51875E+00_JPRB, 0.51589E+00_JPRB, 0.51438E+00_JPRB ,&
&  0.51140E+00_JPRB, 0.50690E+00_JPRB, 0.50171E+00_JPRB, 0.49828E+00_JPRB, 0.49383E+00_JPRB, 0.48760E+00_JPRB, 0.47992E+00_JPRB ,&
&  0.47264E+00_JPRB, 0.46587E+00_JPRB, 0.46051E+00_JPRB, 0.45472E+00_JPRB, 0.45026E+00_JPRB, 0.44992E+00_JPRB, 0.45148E+00_JPRB ,&
&  0.45767E+00_JPRB, 0.46610E+00_JPRB, 0.47434E+00_JPRB, 0.49165E+00_JPRB, 0.51414E+00_JPRB, 0.54890E+00_JPRB, 0.58744E+00_JPRB ,&
&  0.67198E+00_JPRB, 0.62393E+00_JPRB, 0.67282E+00_JPRB, 0.77068E+00_JPRB, 0.83700E+00_JPRB, 0.86149E+00_JPRB, 0.83840E+00_JPRB ,&
&  0.82542E+00_JPRB, 0.83062E+00_JPRB, 0.83305E+00_JPRB, 0.82733E+00_JPRB, 0.79541E+00_JPRB, 0.72779E+00_JPRB, 0.66018E+00_JPRB ,&
&  0.61405E+00_JPRB, 0.59726E+00_JPRB, 0.58943E+00_JPRB, 0.57860E+00_JPRB, 0.56760E+00_JPRB, 0.56143E+00_JPRB, 0.56021E+00_JPRB ,&
&  0.55931E+00_JPRB, 0.55776E+00_JPRB, 0.55506E+00_JPRB, 0.55200E+00_JPRB, 0.55157E+00_JPRB, 0.56293E+00_JPRB, 0.59934E+00_JPRB ,&
&  0.66960E+00_JPRB, 0.74842E+00_JPRB, 0.81956E+00_JPRB, 0.87468E+00_JPRB, 0.88501E+00_JPRB, 0.86099E+00_JPRB, 0.81539E+00_JPRB ,&
&  0.76225E+00_JPRB, 0.71360E+00_JPRB, 0.67601E+00_JPRB, 0.64625E+00_JPRB, 0.62224E+00_JPRB, 0.60323E+00_JPRB, 0.58929E+00_JPRB ,&
&  0.58121E+00_JPRB, 0.57552E+00_JPRB, 0.57102E+00_JPRB, 0.56547E+00_JPRB, 0.56063E+00_JPRB, 0.55702E+00_JPRB, 0.55525E+00_JPRB ,&
&  0.55254E+00_JPRB, 0.54937E+00_JPRB, 0.54786E+00_JPRB, 0.54744E+00_JPRB, 0.54619E+00_JPRB, 0.54349E+00_JPRB, 0.54077E+00_JPRB ,&
&  0.54014E+00_JPRB, 0.54525E+00_JPRB, 0.55072E+00_JPRB, 0.59951E+00_JPRB, 0.61413E+00_JPRB, 0.60001E+00_JPRB, 0.58601E+00_JPRB ,&
&  0.57838E+00_JPRB, 0.56801E+00_JPRB, 0.56290E+00_JPRB, 0.52431E+00_JPRB, 0.55191E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.50964E+00_JPRB, 0.50967E+00_JPRB, 0.50542E+00_JPRB, 0.49658E+00_JPRB, 0.48980E+00_JPRB, 0.48509E+00_JPRB, 0.48249E+00_JPRB ,&
&  0.48131E+00_JPRB, 0.47978E+00_JPRB, 0.47682E+00_JPRB, 0.47138E+00_JPRB, 0.46666E+00_JPRB, 0.46092E+00_JPRB, 0.45495E+00_JPRB ,&
&  0.44983E+00_JPRB, 0.44516E+00_JPRB, 0.44236E+00_JPRB, 0.43888E+00_JPRB, 0.43752E+00_JPRB, 0.43888E+00_JPRB, 0.44082E+00_JPRB ,&
&  0.44389E+00_JPRB, 0.44706E+00_JPRB, 0.45779E+00_JPRB, 0.47426E+00_JPRB, 0.50263E+00_JPRB, 0.52180E+00_JPRB, 0.55635E+00_JPRB ,&
&  0.63502E+00_JPRB, 0.55303E+00_JPRB, 0.61283E+00_JPRB, 0.68781E+00_JPRB, 0.73835E+00_JPRB, 0.76524E+00_JPRB, 0.72184E+00_JPRB ,&
&  0.68738E+00_JPRB, 0.66785E+00_JPRB, 0.66011E+00_JPRB, 0.65720E+00_JPRB, 0.65443E+00_JPRB, 0.63568E+00_JPRB, 0.61103E+00_JPRB ,&
&  0.58466E+00_JPRB, 0.56319E+00_JPRB, 0.54485E+00_JPRB, 0.52580E+00_JPRB, 0.50879E+00_JPRB, 0.49771E+00_JPRB, 0.49222E+00_JPRB ,&
&  0.49074E+00_JPRB, 0.49069E+00_JPRB, 0.49035E+00_JPRB, 0.49089E+00_JPRB, 0.49310E+00_JPRB, 0.49755E+00_JPRB, 0.50607E+00_JPRB ,&
&  0.52923E+00_JPRB, 0.56807E+00_JPRB, 0.60996E+00_JPRB, 0.64890E+00_JPRB, 0.67719E+00_JPRB, 0.68692E+00_JPRB, 0.68604E+00_JPRB ,&
&  0.67190E+00_JPRB, 0.64926E+00_JPRB, 0.62424E+00_JPRB, 0.60068E+00_JPRB, 0.58003E+00_JPRB, 0.56225E+00_JPRB, 0.54602E+00_JPRB ,&
&  0.53208E+00_JPRB, 0.51945E+00_JPRB, 0.50979E+00_JPRB, 0.50294E+00_JPRB, 0.49667E+00_JPRB, 0.49121E+00_JPRB, 0.48820E+00_JPRB ,&
&  0.48643E+00_JPRB, 0.48560E+00_JPRB, 0.48561E+00_JPRB, 0.48809E+00_JPRB, 0.49176E+00_JPRB, 0.49481E+00_JPRB, 0.49613E+00_JPRB ,&
&  0.49537E+00_JPRB, 0.49874E+00_JPRB, 0.50106E+00_JPRB, 0.52382E+00_JPRB, 0.53233E+00_JPRB, 0.53302E+00_JPRB, 0.53092E+00_JPRB ,&
&  0.52927E+00_JPRB, 0.52120E+00_JPRB, 0.46700E+00_JPRB, 0.49342E+00_JPRB, 0.50383E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.47718E+00_JPRB, 0.47521E+00_JPRB, 0.47466E+00_JPRB, 0.47280E+00_JPRB, 0.47029E+00_JPRB, 0.46741E+00_JPRB, 0.46373E+00_JPRB ,&
&  0.45926E+00_JPRB, 0.45640E+00_JPRB, 0.45375E+00_JPRB, 0.45024E+00_JPRB, 0.44652E+00_JPRB, 0.44347E+00_JPRB, 0.44062E+00_JPRB ,&
&  0.43749E+00_JPRB, 0.43394E+00_JPRB, 0.43134E+00_JPRB, 0.42950E+00_JPRB, 0.42752E+00_JPRB, 0.42676E+00_JPRB, 0.42772E+00_JPRB ,&
&  0.42764E+00_JPRB, 0.43066E+00_JPRB, 0.44335E+00_JPRB, 0.46098E+00_JPRB, 0.48439E+00_JPRB, 0.50566E+00_JPRB, 0.52764E+00_JPRB ,&
&  0.52569E+00_JPRB, 0.47685E+00_JPRB, 0.52088E+00_JPRB, 0.57866E+00_JPRB, 0.63686E+00_JPRB, 0.67609E+00_JPRB, 0.64681E+00_JPRB ,&
&  0.60559E+00_JPRB, 0.57153E+00_JPRB, 0.55163E+00_JPRB, 0.54161E+00_JPRB, 0.54209E+00_JPRB, 0.54199E+00_JPRB, 0.54125E+00_JPRB ,&
&  0.53539E+00_JPRB, 0.52239E+00_JPRB, 0.50710E+00_JPRB, 0.49237E+00_JPRB, 0.48132E+00_JPRB, 0.47269E+00_JPRB, 0.46553E+00_JPRB ,&
&  0.46160E+00_JPRB, 0.46016E+00_JPRB, 0.45959E+00_JPRB, 0.45962E+00_JPRB, 0.46056E+00_JPRB, 0.46372E+00_JPRB, 0.46757E+00_JPRB ,&
&  0.47203E+00_JPRB, 0.48123E+00_JPRB, 0.49911E+00_JPRB, 0.52092E+00_JPRB, 0.54270E+00_JPRB, 0.55829E+00_JPRB, 0.56692E+00_JPRB ,&
&  0.57073E+00_JPRB, 0.56907E+00_JPRB, 0.56128E+00_JPRB, 0.54798E+00_JPRB, 0.53181E+00_JPRB, 0.51649E+00_JPRB, 0.50232E+00_JPRB ,&
&  0.48901E+00_JPRB, 0.47533E+00_JPRB, 0.46285E+00_JPRB, 0.45533E+00_JPRB, 0.45154E+00_JPRB, 0.44791E+00_JPRB, 0.44480E+00_JPRB ,&
&  0.44323E+00_JPRB, 0.44280E+00_JPRB, 0.44297E+00_JPRB, 0.44390E+00_JPRB, 0.44583E+00_JPRB, 0.44907E+00_JPRB, 0.45226E+00_JPRB ,&
&  0.45462E+00_JPRB, 0.46024E+00_JPRB, 0.46754E+00_JPRB, 0.48604E+00_JPRB, 0.49757E+00_JPRB, 0.50161E+00_JPRB, 0.50071E+00_JPRB ,&
&  0.49576E+00_JPRB, 0.48908E+00_JPRB, 0.48242E+00_JPRB, 0.47721E+00_JPRB, 0.47677E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.45184E+00_JPRB, 0.45043E+00_JPRB, 0.44933E+00_JPRB, 0.44744E+00_JPRB, 0.44576E+00_JPRB, 0.44438E+00_JPRB, 0.44111E+00_JPRB ,&
&  0.44003E+00_JPRB, 0.43642E+00_JPRB, 0.43295E+00_JPRB, 0.43139E+00_JPRB, 0.42945E+00_JPRB, 0.42684E+00_JPRB, 0.42489E+00_JPRB ,&
&  0.42260E+00_JPRB, 0.41983E+00_JPRB, 0.41770E+00_JPRB, 0.41494E+00_JPRB, 0.41097E+00_JPRB, 0.40813E+00_JPRB, 0.40846E+00_JPRB ,&
&  0.40911E+00_JPRB, 0.41195E+00_JPRB, 0.42243E+00_JPRB, 0.43360E+00_JPRB, 0.44790E+00_JPRB, 0.46319E+00_JPRB, 0.47652E+00_JPRB ,&
&  0.50048E+00_JPRB, 0.44374E+00_JPRB, 0.48482E+00_JPRB, 0.51910E+00_JPRB, 0.56865E+00_JPRB, 0.60567E+00_JPRB, 0.59205E+00_JPRB ,&
&  0.55581E+00_JPRB, 0.51762E+00_JPRB, 0.49394E+00_JPRB, 0.48283E+00_JPRB, 0.48677E+00_JPRB, 0.49210E+00_JPRB, 0.49101E+00_JPRB ,&
&  0.48490E+00_JPRB, 0.47182E+00_JPRB, 0.45693E+00_JPRB, 0.44746E+00_JPRB, 0.44217E+00_JPRB, 0.43788E+00_JPRB, 0.43366E+00_JPRB ,&
&  0.43058E+00_JPRB, 0.42917E+00_JPRB, 0.42838E+00_JPRB, 0.42796E+00_JPRB, 0.42852E+00_JPRB, 0.43030E+00_JPRB, 0.43283E+00_JPRB ,&
&  0.43470E+00_JPRB, 0.43517E+00_JPRB, 0.43699E+00_JPRB, 0.44235E+00_JPRB, 0.45075E+00_JPRB, 0.45942E+00_JPRB, 0.46576E+00_JPRB ,&
&  0.46950E+00_JPRB, 0.47195E+00_JPRB, 0.47254E+00_JPRB, 0.46984E+00_JPRB, 0.46362E+00_JPRB, 0.45508E+00_JPRB, 0.44661E+00_JPRB ,&
&  0.43848E+00_JPRB, 0.43068E+00_JPRB, 0.42160E+00_JPRB, 0.41677E+00_JPRB, 0.41511E+00_JPRB, 0.41435E+00_JPRB, 0.41396E+00_JPRB ,&
&  0.41379E+00_JPRB, 0.41332E+00_JPRB, 0.41308E+00_JPRB, 0.41294E+00_JPRB, 0.41302E+00_JPRB, 0.41379E+00_JPRB, 0.41594E+00_JPRB ,&
&  0.41968E+00_JPRB, 0.42649E+00_JPRB, 0.43542E+00_JPRB, 0.45045E+00_JPRB, 0.46336E+00_JPRB, 0.47028E+00_JPRB, 0.47264E+00_JPRB ,&
&  0.46980E+00_JPRB, 0.46475E+00_JPRB, 0.45897E+00_JPRB, 0.45368E+00_JPRB, 0.45295E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.43375E+00_JPRB, 0.43009E+00_JPRB, 0.42758E+00_JPRB, 0.42378E+00_JPRB, 0.42111E+00_JPRB, 0.41925E+00_JPRB, 0.41719E+00_JPRB ,&
&  0.41580E+00_JPRB, 0.41638E+00_JPRB, 0.41536E+00_JPRB, 0.41472E+00_JPRB, 0.41356E+00_JPRB, 0.41051E+00_JPRB, 0.40681E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.40282E+00_JPRB, 0.40108E+00_JPRB, 0.39902E+00_JPRB, 0.39680E+00_JPRB, 0.39390E+00_JPRB, 0.39362E+00_JPRB ,&
&  0.39426E+00_JPRB, 0.39711E+00_JPRB, 0.40185E+00_JPRB, 0.40743E+00_JPRB, 0.41687E+00_JPRB, 0.43017E+00_JPRB, 0.44319E+00_JPRB ,&
&  0.45963E+00_JPRB, 0.44752E+00_JPRB, 0.47397E+00_JPRB, 0.49016E+00_JPRB, 0.51851E+00_JPRB, 0.54274E+00_JPRB, 0.54395E+00_JPRB ,&
&  0.52975E+00_JPRB, 0.50418E+00_JPRB, 0.48394E+00_JPRB, 0.47328E+00_JPRB, 0.47274E+00_JPRB, 0.47086E+00_JPRB, 0.46133E+00_JPRB ,&
&  0.44716E+00_JPRB, 0.43089E+00_JPRB, 0.41873E+00_JPRB, 0.41204E+00_JPRB, 0.40908E+00_JPRB, 0.40747E+00_JPRB, 0.40476E+00_JPRB ,&
&  0.40191E+00_JPRB, 0.39927E+00_JPRB, 0.39781E+00_JPRB, 0.39705E+00_JPRB, 0.39640E+00_JPRB, 0.39645E+00_JPRB, 0.39719E+00_JPRB ,&
&  0.39800E+00_JPRB, 0.39841E+00_JPRB, 0.39840E+00_JPRB, 0.39903E+00_JPRB, 0.40061E+00_JPRB, 0.40281E+00_JPRB, 0.40500E+00_JPRB ,&
&  0.40590E+00_JPRB, 0.40670E+00_JPRB, 0.40701E+00_JPRB, 0.40713E+00_JPRB, 0.40532E+00_JPRB, 0.40134E+00_JPRB, 0.39655E+00_JPRB ,&
&  0.39267E+00_JPRB, 0.38971E+00_JPRB, 0.38717E+00_JPRB, 0.38539E+00_JPRB, 0.38478E+00_JPRB, 0.38540E+00_JPRB, 0.38647E+00_JPRB ,&
&  0.38756E+00_JPRB, 0.38864E+00_JPRB, 0.38939E+00_JPRB, 0.38997E+00_JPRB, 0.38986E+00_JPRB, 0.39021E+00_JPRB, 0.39152E+00_JPRB ,&
&  0.39536E+00_JPRB, 0.40048E+00_JPRB, 0.40790E+00_JPRB, 0.41995E+00_JPRB, 0.43190E+00_JPRB, 0.44013E+00_JPRB, 0.44577E+00_JPRB ,&
&  0.44712E+00_JPRB, 0.44551E+00_JPRB, 0.44268E+00_JPRB, 0.44036E+00_JPRB, 0.43794E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.40727E+00_JPRB, 0.40249E+00_JPRB, 0.39909E+00_JPRB, 0.39665E+00_JPRB, 0.39667E+00_JPRB, 0.39545E+00_JPRB, 0.39241E+00_JPRB ,&
&  0.39023E+00_JPRB, 0.39178E+00_JPRB, 0.39481E+00_JPRB, 0.39610E+00_JPRB, 0.39528E+00_JPRB, 0.39350E+00_JPRB, 0.39020E+00_JPRB ,&
&  0.38588E+00_JPRB, 0.38366E+00_JPRB, 0.38333E+00_JPRB, 0.38407E+00_JPRB, 0.38358E+00_JPRB, 0.38131E+00_JPRB, 0.37940E+00_JPRB ,&
&  0.37971E+00_JPRB, 0.38128E+00_JPRB, 0.38258E+00_JPRB, 0.38549E+00_JPRB, 0.39242E+00_JPRB, 0.40514E+00_JPRB, 0.41661E+00_JPRB ,&
&  0.42474E+00_JPRB, 0.42917E+00_JPRB, 0.42414E+00_JPRB, 0.44354E+00_JPRB, 0.45939E+00_JPRB, 0.47998E+00_JPRB, 0.49468E+00_JPRB ,&
&  0.49878E+00_JPRB, 0.49193E+00_JPRB, 0.47670E+00_JPRB, 0.46403E+00_JPRB, 0.45481E+00_JPRB, 0.44262E+00_JPRB, 0.42462E+00_JPRB ,&
&  0.40640E+00_JPRB, 0.39194E+00_JPRB, 0.38320E+00_JPRB, 0.37883E+00_JPRB, 0.37783E+00_JPRB, 0.37761E+00_JPRB, 0.37512E+00_JPRB ,&
&  0.37226E+00_JPRB, 0.36961E+00_JPRB, 0.36843E+00_JPRB, 0.36741E+00_JPRB, 0.36615E+00_JPRB, 0.36514E+00_JPRB, 0.36476E+00_JPRB ,&
&  0.36479E+00_JPRB, 0.36481E+00_JPRB, 0.36512E+00_JPRB, 0.36576E+00_JPRB, 0.36601E+00_JPRB, 0.36556E+00_JPRB, 0.36498E+00_JPRB ,&
&  0.36440E+00_JPRB, 0.36403E+00_JPRB, 0.36368E+00_JPRB, 0.36314E+00_JPRB, 0.36252E+00_JPRB, 0.36167E+00_JPRB, 0.35996E+00_JPRB ,&
&  0.35774E+00_JPRB, 0.35699E+00_JPRB, 0.35716E+00_JPRB, 0.35675E+00_JPRB, 0.35624E+00_JPRB, 0.35691E+00_JPRB, 0.35877E+00_JPRB ,&
&  0.36066E+00_JPRB, 0.36297E+00_JPRB, 0.36485E+00_JPRB, 0.36607E+00_JPRB, 0.36595E+00_JPRB, 0.36628E+00_JPRB, 0.36710E+00_JPRB ,&
&  0.36909E+00_JPRB, 0.37237E+00_JPRB, 0.37717E+00_JPRB, 0.38388E+00_JPRB, 0.39178E+00_JPRB, 0.39826E+00_JPRB, 0.40423E+00_JPRB ,&
&  0.40918E+00_JPRB, 0.41173E+00_JPRB, 0.41220E+00_JPRB, 0.41257E+00_JPRB, 0.41127E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.37788E+00_JPRB, 0.37424E+00_JPRB, 0.37169E+00_JPRB, 0.37140E+00_JPRB, 0.37326E+00_JPRB, 0.37335E+00_JPRB, 0.37121E+00_JPRB ,&
&  0.36932E+00_JPRB, 0.37078E+00_JPRB, 0.37446E+00_JPRB, 0.37675E+00_JPRB, 0.37775E+00_JPRB, 0.37853E+00_JPRB, 0.37701E+00_JPRB ,&
&  0.37291E+00_JPRB, 0.36901E+00_JPRB, 0.36933E+00_JPRB, 0.37098E+00_JPRB, 0.37178E+00_JPRB, 0.36981E+00_JPRB, 0.36752E+00_JPRB ,&
&  0.36746E+00_JPRB, 0.36755E+00_JPRB, 0.36985E+00_JPRB, 0.37232E+00_JPRB, 0.37814E+00_JPRB, 0.38868E+00_JPRB, 0.39777E+00_JPRB ,&
&  0.40365E+00_JPRB, 0.40301E+00_JPRB, 0.40224E+00_JPRB, 0.40750E+00_JPRB, 0.41415E+00_JPRB, 0.42873E+00_JPRB, 0.44510E+00_JPRB ,&
&  0.45403E+00_JPRB, 0.45310E+00_JPRB, 0.44025E+00_JPRB, 0.42592E+00_JPRB, 0.41325E+00_JPRB, 0.39935E+00_JPRB, 0.38569E+00_JPRB ,&
&  0.37315E+00_JPRB, 0.36434E+00_JPRB, 0.35919E+00_JPRB, 0.35668E+00_JPRB, 0.35554E+00_JPRB, 0.35422E+00_JPRB, 0.35122E+00_JPRB ,&
&  0.34755E+00_JPRB, 0.34464E+00_JPRB, 0.34253E+00_JPRB, 0.34106E+00_JPRB, 0.33977E+00_JPRB, 0.33916E+00_JPRB, 0.33840E+00_JPRB ,&
&  0.33804E+00_JPRB, 0.33841E+00_JPRB, 0.33917E+00_JPRB, 0.33900E+00_JPRB, 0.33856E+00_JPRB, 0.33756E+00_JPRB, 0.33681E+00_JPRB ,&
&  0.33609E+00_JPRB, 0.33428E+00_JPRB, 0.33291E+00_JPRB, 0.33303E+00_JPRB, 0.33322E+00_JPRB, 0.33381E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33345E+00_JPRB, 0.33329E+00_JPRB, 0.33374E+00_JPRB, 0.33422E+00_JPRB, 0.33453E+00_JPRB, 0.33483E+00_JPRB, 0.33555E+00_JPRB ,&
&  0.33747E+00_JPRB, 0.33973E+00_JPRB, 0.34147E+00_JPRB, 0.34299E+00_JPRB, 0.34420E+00_JPRB, 0.34584E+00_JPRB, 0.34608E+00_JPRB ,&
&  0.34628E+00_JPRB, 0.34760E+00_JPRB, 0.35085E+00_JPRB, 0.35545E+00_JPRB, 0.36030E+00_JPRB, 0.36424E+00_JPRB, 0.36826E+00_JPRB ,&
&  0.37193E+00_JPRB, 0.37501E+00_JPRB, 0.37700E+00_JPRB, 0.37775E+00_JPRB, 0.37839E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.35040E+00_JPRB, 0.34909E+00_JPRB, 0.34841E+00_JPRB, 0.34896E+00_JPRB, 0.35107E+00_JPRB, 0.35127E+00_JPRB, 0.35089E+00_JPRB ,&
&  0.35060E+00_JPRB, 0.35284E+00_JPRB, 0.35694E+00_JPRB, 0.35960E+00_JPRB, 0.36034E+00_JPRB, 0.36043E+00_JPRB, 0.35981E+00_JPRB ,&
&  0.35904E+00_JPRB, 0.35840E+00_JPRB, 0.35724E+00_JPRB, 0.35770E+00_JPRB, 0.35741E+00_JPRB, 0.35595E+00_JPRB, 0.35636E+00_JPRB ,&
&  0.35607E+00_JPRB, 0.35649E+00_JPRB, 0.35924E+00_JPRB, 0.36187E+00_JPRB, 0.36739E+00_JPRB, 0.37723E+00_JPRB, 0.38510E+00_JPRB ,&
&  0.38851E+00_JPRB, 0.38385E+00_JPRB, 0.37888E+00_JPRB, 0.37948E+00_JPRB, 0.38273E+00_JPRB, 0.38981E+00_JPRB, 0.39862E+00_JPRB ,&
&  0.40364E+00_JPRB, 0.39898E+00_JPRB, 0.38777E+00_JPRB, 0.37694E+00_JPRB, 0.36788E+00_JPRB, 0.36032E+00_JPRB, 0.35379E+00_JPRB ,&
&  0.34781E+00_JPRB, 0.34364E+00_JPRB, 0.34154E+00_JPRB, 0.34084E+00_JPRB, 0.33993E+00_JPRB, 0.33644E+00_JPRB, 0.33233E+00_JPRB ,&
&  0.32730E+00_JPRB, 0.32371E+00_JPRB, 0.32082E+00_JPRB, 0.31975E+00_JPRB, 0.31972E+00_JPRB, 0.31930E+00_JPRB, 0.31806E+00_JPRB ,&
&  0.31658E+00_JPRB, 0.31604E+00_JPRB, 0.31550E+00_JPRB, 0.31408E+00_JPRB, 0.31229E+00_JPRB, 0.31156E+00_JPRB, 0.31037E+00_JPRB ,&
&  0.30838E+00_JPRB, 0.30755E+00_JPRB, 0.30813E+00_JPRB, 0.30959E+00_JPRB, 0.30970E+00_JPRB, 0.31063E+00_JPRB, 0.31228E+00_JPRB ,&
&  0.31133E+00_JPRB, 0.30863E+00_JPRB, 0.30707E+00_JPRB, 0.30739E+00_JPRB, 0.30919E+00_JPRB, 0.31174E+00_JPRB, 0.31306E+00_JPRB ,&
&  0.31426E+00_JPRB, 0.31450E+00_JPRB, 0.31569E+00_JPRB, 0.31741E+00_JPRB, 0.31958E+00_JPRB, 0.32117E+00_JPRB, 0.32188E+00_JPRB ,&
&  0.32236E+00_JPRB, 0.32296E+00_JPRB, 0.32528E+00_JPRB, 0.32891E+00_JPRB, 0.33223E+00_JPRB, 0.33596E+00_JPRB, 0.33898E+00_JPRB ,&
&  0.34127E+00_JPRB, 0.34375E+00_JPRB, 0.34566E+00_JPRB, 0.34703E+00_JPRB, 0.34874E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.33242E+00_JPRB, 0.33335E+00_JPRB, 0.33453E+00_JPRB, 0.33486E+00_JPRB, 0.33605E+00_JPRB, 0.33677E+00_JPRB, 0.33817E+00_JPRB ,&
&  0.33893E+00_JPRB, 0.34081E+00_JPRB, 0.34332E+00_JPRB, 0.34347E+00_JPRB, 0.34319E+00_JPRB, 0.34306E+00_JPRB, 0.34373E+00_JPRB ,&
&  0.34601E+00_JPRB, 0.34696E+00_JPRB, 0.34641E+00_JPRB, 0.34642E+00_JPRB, 0.34514E+00_JPRB, 0.34389E+00_JPRB, 0.34553E+00_JPRB ,&
&  0.34572E+00_JPRB, 0.34635E+00_JPRB, 0.34879E+00_JPRB, 0.35126E+00_JPRB, 0.35653E+00_JPRB, 0.36356E+00_JPRB, 0.37087E+00_JPRB ,&
&  0.37494E+00_JPRB, 0.37202E+00_JPRB, 0.36464E+00_JPRB, 0.35939E+00_JPRB, 0.35842E+00_JPRB, 0.35974E+00_JPRB, 0.36283E+00_JPRB ,&
&  0.36262E+00_JPRB, 0.35675E+00_JPRB, 0.34849E+00_JPRB, 0.34344E+00_JPRB, 0.34081E+00_JPRB, 0.33705E+00_JPRB, 0.33180E+00_JPRB ,&
&  0.32760E+00_JPRB, 0.32590E+00_JPRB, 0.32569E+00_JPRB, 0.32458E+00_JPRB, 0.32231E+00_JPRB, 0.31856E+00_JPRB, 0.31389E+00_JPRB ,&
&  0.30971E+00_JPRB, 0.30562E+00_JPRB, 0.30295E+00_JPRB, 0.30119E+00_JPRB, 0.30092E+00_JPRB, 0.30088E+00_JPRB, 0.29976E+00_JPRB ,&
&  0.29751E+00_JPRB, 0.29598E+00_JPRB, 0.29398E+00_JPRB, 0.29047E+00_JPRB, 0.28796E+00_JPRB, 0.28710E+00_JPRB, 0.28594E+00_JPRB ,&
&  0.28432E+00_JPRB, 0.28388E+00_JPRB, 0.28385E+00_JPRB, 0.28349E+00_JPRB, 0.28450E+00_JPRB, 0.28631E+00_JPRB, 0.28389E+00_JPRB ,&
&  0.28158E+00_JPRB, 0.28002E+00_JPRB, 0.28015E+00_JPRB, 0.28198E+00_JPRB, 0.28496E+00_JPRB, 0.28816E+00_JPRB, 0.29073E+00_JPRB ,&
&  0.29181E+00_JPRB, 0.29213E+00_JPRB, 0.29250E+00_JPRB, 0.29284E+00_JPRB, 0.29436E+00_JPRB, 0.29727E+00_JPRB, 0.30054E+00_JPRB ,&
&  0.30211E+00_JPRB, 0.30470E+00_JPRB, 0.30766E+00_JPRB, 0.31140E+00_JPRB, 0.31370E+00_JPRB, 0.31598E+00_JPRB, 0.31823E+00_JPRB ,&
&  0.32002E+00_JPRB, 0.32223E+00_JPRB, 0.32524E+00_JPRB, 0.32766E+00_JPRB, 0.33014E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.31864E+00_JPRB, 0.32060E+00_JPRB, 0.32167E+00_JPRB, 0.32318E+00_JPRB, 0.32602E+00_JPRB, 0.32765E+00_JPRB, 0.32768E+00_JPRB ,&
&  0.32793E+00_JPRB, 0.32952E+00_JPRB, 0.33078E+00_JPRB, 0.33137E+00_JPRB, 0.32906E+00_JPRB, 0.32873E+00_JPRB, 0.33056E+00_JPRB ,&
&  0.33294E+00_JPRB, 0.33497E+00_JPRB, 0.33511E+00_JPRB, 0.33508E+00_JPRB, 0.33403E+00_JPRB, 0.33247E+00_JPRB, 0.33377E+00_JPRB ,&
&  0.33480E+00_JPRB, 0.33523E+00_JPRB, 0.33690E+00_JPRB, 0.33896E+00_JPRB, 0.34311E+00_JPRB, 0.34807E+00_JPRB, 0.35263E+00_JPRB ,&
&  0.35635E+00_JPRB, 0.35581E+00_JPRB, 0.34813E+00_JPRB, 0.33970E+00_JPRB, 0.33963E+00_JPRB, 0.34040E+00_JPRB, 0.34016E+00_JPRB ,&
&  0.33796E+00_JPRB, 0.33488E+00_JPRB, 0.33279E+00_JPRB, 0.33101E+00_JPRB, 0.32853E+00_JPRB, 0.32383E+00_JPRB, 0.31858E+00_JPRB ,&
&  0.31412E+00_JPRB, 0.31240E+00_JPRB, 0.31079E+00_JPRB, 0.30801E+00_JPRB, 0.30391E+00_JPRB, 0.30046E+00_JPRB, 0.29687E+00_JPRB ,&
&  0.29095E+00_JPRB, 0.28532E+00_JPRB, 0.28215E+00_JPRB, 0.28111E+00_JPRB, 0.28119E+00_JPRB, 0.28050E+00_JPRB, 0.27839E+00_JPRB ,&
&  0.27597E+00_JPRB, 0.27310E+00_JPRB, 0.27129E+00_JPRB, 0.27072E+00_JPRB, 0.27187E+00_JPRB, 0.27013E+00_JPRB, 0.26520E+00_JPRB ,&
&  0.26243E+00_JPRB, 0.26098E+00_JPRB, 0.26146E+00_JPRB, 0.26042E+00_JPRB, 0.25949E+00_JPRB, 0.25827E+00_JPRB, 0.25878E+00_JPRB ,&
&  0.26001E+00_JPRB, 0.26015E+00_JPRB, 0.26295E+00_JPRB, 0.26797E+00_JPRB, 0.27249E+00_JPRB, 0.27512E+00_JPRB, 0.27671E+00_JPRB ,&
&  0.27727E+00_JPRB, 0.27677E+00_JPRB, 0.27631E+00_JPRB, 0.27712E+00_JPRB, 0.27960E+00_JPRB, 0.28295E+00_JPRB, 0.28611E+00_JPRB ,&
&  0.28693E+00_JPRB, 0.28823E+00_JPRB, 0.29137E+00_JPRB, 0.29606E+00_JPRB, 0.29880E+00_JPRB, 0.30090E+00_JPRB, 0.30270E+00_JPRB ,&
&  0.30364E+00_JPRB, 0.30540E+00_JPRB, 0.30950E+00_JPRB, 0.31233E+00_JPRB, 0.31518E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.30427E+00_JPRB, 0.30561E+00_JPRB, 0.30787E+00_JPRB, 0.31035E+00_JPRB, 0.31158E+00_JPRB, 0.31114E+00_JPRB, 0.31024E+00_JPRB ,&
&  0.31069E+00_JPRB, 0.31309E+00_JPRB, 0.31566E+00_JPRB, 0.31651E+00_JPRB, 0.31696E+00_JPRB, 0.31747E+00_JPRB, 0.31937E+00_JPRB ,&
&  0.32206E+00_JPRB, 0.32517E+00_JPRB, 0.32513E+00_JPRB, 0.32486E+00_JPRB, 0.32409E+00_JPRB, 0.32278E+00_JPRB, 0.32386E+00_JPRB ,&
&  0.32683E+00_JPRB, 0.32687E+00_JPRB, 0.32673E+00_JPRB, 0.32626E+00_JPRB, 0.32862E+00_JPRB, 0.33287E+00_JPRB, 0.33606E+00_JPRB ,&
&  0.33656E+00_JPRB, 0.33063E+00_JPRB, 0.32207E+00_JPRB, 0.32196E+00_JPRB, 0.32367E+00_JPRB, 0.32528E+00_JPRB, 0.32531E+00_JPRB ,&
&  0.32480E+00_JPRB, 0.32377E+00_JPRB, 0.32244E+00_JPRB, 0.32091E+00_JPRB, 0.31869E+00_JPRB, 0.31352E+00_JPRB, 0.30765E+00_JPRB ,&
&  0.30294E+00_JPRB, 0.29902E+00_JPRB, 0.29533E+00_JPRB, 0.29024E+00_JPRB, 0.28510E+00_JPRB, 0.28134E+00_JPRB, 0.27695E+00_JPRB ,&
&  0.27170E+00_JPRB, 0.26812E+00_JPRB, 0.26694E+00_JPRB, 0.26666E+00_JPRB, 0.26654E+00_JPRB, 0.26551E+00_JPRB, 0.26420E+00_JPRB ,&
&  0.26197E+00_JPRB, 0.26024E+00_JPRB, 0.25850E+00_JPRB, 0.25732E+00_JPRB, 0.25083E+00_JPRB, 0.24296E+00_JPRB, 0.23712E+00_JPRB ,&
&  0.23667E+00_JPRB, 0.23913E+00_JPRB, 0.24192E+00_JPRB, 0.24299E+00_JPRB, 0.24562E+00_JPRB, 0.24663E+00_JPRB, 0.24599E+00_JPRB ,&
&  0.24250E+00_JPRB, 0.23977E+00_JPRB, 0.24096E+00_JPRB, 0.24241E+00_JPRB, 0.24317E+00_JPRB, 0.24486E+00_JPRB, 0.24648E+00_JPRB ,&
&  0.24957E+00_JPRB, 0.25166E+00_JPRB, 0.25399E+00_JPRB, 0.25661E+00_JPRB, 0.25841E+00_JPRB, 0.25870E+00_JPRB, 0.25674E+00_JPRB ,&
&  0.25704E+00_JPRB, 0.26319E+00_JPRB, 0.27362E+00_JPRB, 0.28333E+00_JPRB, 0.28622E+00_JPRB, 0.28880E+00_JPRB, 0.29099E+00_JPRB ,&
&  0.29231E+00_JPRB, 0.29620E+00_JPRB, 0.29958E+00_JPRB, 0.30153E+00_JPRB, 0.30278E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.29403E+00_JPRB, 0.29368E+00_JPRB, 0.29248E+00_JPRB, 0.29143E+00_JPRB, 0.28986E+00_JPRB, 0.28837E+00_JPRB, 0.28911E+00_JPRB ,&
&  0.29197E+00_JPRB, 0.29572E+00_JPRB, 0.29933E+00_JPRB, 0.30209E+00_JPRB, 0.30420E+00_JPRB, 0.30526E+00_JPRB, 0.30687E+00_JPRB ,&
&  0.30870E+00_JPRB, 0.31194E+00_JPRB, 0.31330E+00_JPRB, 0.31444E+00_JPRB, 0.31354E+00_JPRB, 0.31262E+00_JPRB, 0.31362E+00_JPRB ,&
&  0.31683E+00_JPRB, 0.31820E+00_JPRB, 0.31732E+00_JPRB, 0.31523E+00_JPRB, 0.31605E+00_JPRB, 0.31876E+00_JPRB, 0.31915E+00_JPRB ,&
&  0.31295E+00_JPRB, 0.29868E+00_JPRB, 0.28917E+00_JPRB, 0.29324E+00_JPRB, 0.30275E+00_JPRB, 0.30702E+00_JPRB, 0.31020E+00_JPRB ,&
&  0.31212E+00_JPRB, 0.31212E+00_JPRB, 0.31080E+00_JPRB, 0.30846E+00_JPRB, 0.30489E+00_JPRB, 0.29912E+00_JPRB, 0.29254E+00_JPRB ,&
&  0.28683E+00_JPRB, 0.28407E+00_JPRB, 0.28056E+00_JPRB, 0.27435E+00_JPRB, 0.26845E+00_JPRB, 0.26488E+00_JPRB, 0.26034E+00_JPRB ,&
&  0.25558E+00_JPRB, 0.25220E+00_JPRB, 0.25143E+00_JPRB, 0.25080E+00_JPRB, 0.25041E+00_JPRB, 0.24937E+00_JPRB, 0.24640E+00_JPRB ,&
&  0.24329E+00_JPRB, 0.23865E+00_JPRB, 0.23290E+00_JPRB, 0.22759E+00_JPRB, 0.22410E+00_JPRB, 0.22034E+00_JPRB, 0.21554E+00_JPRB ,&
&  0.21315E+00_JPRB, 0.21599E+00_JPRB, 0.22577E+00_JPRB, 0.23603E+00_JPRB, 0.23845E+00_JPRB, 0.22688E+00_JPRB, 0.22359E+00_JPRB ,&
&  0.22348E+00_JPRB, 0.22344E+00_JPRB, 0.22389E+00_JPRB, 0.22558E+00_JPRB, 0.22638E+00_JPRB, 0.22596E+00_JPRB, 0.22654E+00_JPRB ,&
&  0.22889E+00_JPRB, 0.23128E+00_JPRB, 0.23247E+00_JPRB, 0.23395E+00_JPRB, 0.23492E+00_JPRB, 0.23477E+00_JPRB, 0.23441E+00_JPRB ,&
&  0.23520E+00_JPRB, 0.23866E+00_JPRB, 0.24414E+00_JPRB, 0.25185E+00_JPRB, 0.25708E+00_JPRB, 0.26191E+00_JPRB, 0.26785E+00_JPRB ,&
&  0.27553E+00_JPRB, 0.28244E+00_JPRB, 0.28725E+00_JPRB, 0.29158E+00_JPRB, 0.29332E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.28432E+00_JPRB, 0.28520E+00_JPRB, 0.28110E+00_JPRB, 0.27762E+00_JPRB, 0.27585E+00_JPRB, 0.27541E+00_JPRB, 0.27695E+00_JPRB ,&
&  0.28050E+00_JPRB, 0.28335E+00_JPRB, 0.28619E+00_JPRB, 0.28869E+00_JPRB, 0.28978E+00_JPRB, 0.28953E+00_JPRB, 0.29061E+00_JPRB ,&
&  0.29171E+00_JPRB, 0.29464E+00_JPRB, 0.29669E+00_JPRB, 0.29989E+00_JPRB, 0.30013E+00_JPRB, 0.30083E+00_JPRB, 0.30209E+00_JPRB ,&
&  0.30157E+00_JPRB, 0.30180E+00_JPRB, 0.30316E+00_JPRB, 0.30069E+00_JPRB, 0.29924E+00_JPRB, 0.29708E+00_JPRB, 0.29567E+00_JPRB ,&
&  0.28906E+00_JPRB, 0.28178E+00_JPRB, 0.27518E+00_JPRB, 0.27972E+00_JPRB, 0.28778E+00_JPRB, 0.28932E+00_JPRB, 0.29397E+00_JPRB ,&
&  0.29561E+00_JPRB, 0.29358E+00_JPRB, 0.29154E+00_JPRB, 0.28775E+00_JPRB, 0.28421E+00_JPRB, 0.28015E+00_JPRB, 0.27689E+00_JPRB ,&
&  0.27319E+00_JPRB, 0.27375E+00_JPRB, 0.26891E+00_JPRB, 0.25426E+00_JPRB, 0.24720E+00_JPRB, 0.24278E+00_JPRB, 0.23524E+00_JPRB ,&
&  0.22741E+00_JPRB, 0.22563E+00_JPRB, 0.22532E+00_JPRB, 0.22239E+00_JPRB, 0.22200E+00_JPRB, 0.22229E+00_JPRB, 0.22213E+00_JPRB ,&
&  0.22052E+00_JPRB, 0.22022E+00_JPRB, 0.21882E+00_JPRB, 0.21671E+00_JPRB, 0.21459E+00_JPRB, 0.21371E+00_JPRB, 0.21415E+00_JPRB ,&
&  0.21575E+00_JPRB, 0.21722E+00_JPRB, 0.22307E+00_JPRB, 0.22645E+00_JPRB, 0.22053E+00_JPRB, 0.21840E+00_JPRB, 0.21974E+00_JPRB ,&
&  0.21531E+00_JPRB, 0.21455E+00_JPRB, 0.21458E+00_JPRB, 0.21452E+00_JPRB, 0.21491E+00_JPRB, 0.21583E+00_JPRB, 0.21555E+00_JPRB ,&
&  0.21623E+00_JPRB, 0.21785E+00_JPRB, 0.21833E+00_JPRB, 0.21841E+00_JPRB, 0.21811E+00_JPRB, 0.21863E+00_JPRB, 0.22005E+00_JPRB ,&
&  0.22161E+00_JPRB, 0.22464E+00_JPRB, 0.22778E+00_JPRB, 0.23244E+00_JPRB, 0.23611E+00_JPRB, 0.23904E+00_JPRB, 0.24452E+00_JPRB ,&
&  0.24900E+00_JPRB, 0.25924E+00_JPRB, 0.27619E+00_JPRB, 0.28501E+00_JPRB, 0.27953E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.27361E+00_JPRB, 0.27229E+00_JPRB, 0.27055E+00_JPRB, 0.26902E+00_JPRB, 0.27003E+00_JPRB, 0.27134E+00_JPRB, 0.27319E+00_JPRB ,&
&  0.27525E+00_JPRB, 0.27592E+00_JPRB, 0.27599E+00_JPRB, 0.27346E+00_JPRB, 0.27218E+00_JPRB, 0.26652E+00_JPRB, 0.26462E+00_JPRB ,&
&  0.26322E+00_JPRB, 0.26559E+00_JPRB, 0.26863E+00_JPRB, 0.27319E+00_JPRB, 0.27494E+00_JPRB, 0.27681E+00_JPRB, 0.27567E+00_JPRB ,&
&  0.27332E+00_JPRB, 0.27055E+00_JPRB, 0.26779E+00_JPRB, 0.26564E+00_JPRB, 0.26501E+00_JPRB, 0.26305E+00_JPRB, 0.26108E+00_JPRB ,&
&  0.26119E+00_JPRB, 0.26286E+00_JPRB, 0.26359E+00_JPRB, 0.26437E+00_JPRB, 0.27021E+00_JPRB, 0.27045E+00_JPRB, 0.27289E+00_JPRB ,&
&  0.27374E+00_JPRB, 0.27465E+00_JPRB, 0.27426E+00_JPRB, 0.27294E+00_JPRB, 0.27170E+00_JPRB, 0.26529E+00_JPRB, 0.26263E+00_JPRB ,&
&  0.25555E+00_JPRB, 0.25148E+00_JPRB, 0.24437E+00_JPRB, 0.23776E+00_JPRB, 0.23007E+00_JPRB, 0.22431E+00_JPRB, 0.21948E+00_JPRB ,&
&  0.21563E+00_JPRB, 0.21275E+00_JPRB, 0.21041E+00_JPRB, 0.21011E+00_JPRB, 0.21015E+00_JPRB, 0.20963E+00_JPRB, 0.20981E+00_JPRB ,&
&  0.21086E+00_JPRB, 0.21198E+00_JPRB, 0.21187E+00_JPRB, 0.21252E+00_JPRB, 0.21305E+00_JPRB, 0.21385E+00_JPRB, 0.21541E+00_JPRB ,&
&  0.21782E+00_JPRB, 0.21972E+00_JPRB, 0.22017E+00_JPRB, 0.22235E+00_JPRB, 0.22162E+00_JPRB, 0.21873E+00_JPRB, 0.21601E+00_JPRB ,&
&  0.21275E+00_JPRB, 0.21135E+00_JPRB, 0.20969E+00_JPRB, 0.20871E+00_JPRB, 0.20851E+00_JPRB, 0.20864E+00_JPRB, 0.20948E+00_JPRB ,&
&  0.21039E+00_JPRB, 0.21138E+00_JPRB, 0.21175E+00_JPRB, 0.21222E+00_JPRB, 0.21262E+00_JPRB, 0.21354E+00_JPRB, 0.21538E+00_JPRB ,&
&  0.21792E+00_JPRB, 0.22084E+00_JPRB, 0.22368E+00_JPRB, 0.22712E+00_JPRB, 0.23279E+00_JPRB, 0.24017E+00_JPRB, 0.25126E+00_JPRB ,&
&  0.25946E+00_JPRB, 0.27064E+00_JPRB, 0.26919E+00_JPRB, 0.27279E+00_JPRB, 0.27429E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.26556E+00_JPRB, 0.26548E+00_JPRB, 0.26448E+00_JPRB, 0.26395E+00_JPRB, 0.26446E+00_JPRB, 0.26535E+00_JPRB, 0.26664E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26105E+00_JPRB, 0.25679E+00_JPRB, 0.25207E+00_JPRB, 0.24783E+00_JPRB, 0.24479E+00_JPRB, 0.24308E+00_JPRB ,&
&  0.24294E+00_JPRB, 0.24537E+00_JPRB, 0.24706E+00_JPRB, 0.24934E+00_JPRB, 0.24939E+00_JPRB, 0.24954E+00_JPRB, 0.24786E+00_JPRB ,&
&  0.24609E+00_JPRB, 0.24542E+00_JPRB, 0.24567E+00_JPRB, 0.24715E+00_JPRB, 0.24977E+00_JPRB, 0.25132E+00_JPRB, 0.25330E+00_JPRB ,&
&  0.25603E+00_JPRB, 0.25495E+00_JPRB, 0.25556E+00_JPRB, 0.25484E+00_JPRB, 0.25540E+00_JPRB, 0.25670E+00_JPRB, 0.25743E+00_JPRB ,&
&  0.25889E+00_JPRB, 0.26027E+00_JPRB, 0.26046E+00_JPRB, 0.25968E+00_JPRB, 0.25515E+00_JPRB, 0.25294E+00_JPRB, 0.24828E+00_JPRB ,&
&  0.24257E+00_JPRB, 0.23737E+00_JPRB, 0.23106E+00_JPRB, 0.22654E+00_JPRB, 0.22099E+00_JPRB, 0.21774E+00_JPRB, 0.21523E+00_JPRB ,&
&  0.21290E+00_JPRB, 0.21208E+00_JPRB, 0.21088E+00_JPRB, 0.20952E+00_JPRB, 0.20919E+00_JPRB, 0.20777E+00_JPRB, 0.20771E+00_JPRB ,&
&  0.20814E+00_JPRB, 0.20851E+00_JPRB, 0.20970E+00_JPRB, 0.21018E+00_JPRB, 0.21101E+00_JPRB, 0.21173E+00_JPRB, 0.21214E+00_JPRB ,&
&  0.21326E+00_JPRB, 0.21485E+00_JPRB, 0.21474E+00_JPRB, 0.21514E+00_JPRB, 0.21423E+00_JPRB, 0.21218E+00_JPRB, 0.21000E+00_JPRB ,&
&  0.20758E+00_JPRB, 0.20571E+00_JPRB, 0.20517E+00_JPRB, 0.20433E+00_JPRB, 0.20447E+00_JPRB, 0.20487E+00_JPRB, 0.20498E+00_JPRB ,&
&  0.20566E+00_JPRB, 0.20627E+00_JPRB, 0.20721E+00_JPRB, 0.20940E+00_JPRB, 0.21086E+00_JPRB, 0.21279E+00_JPRB, 0.21483E+00_JPRB ,&
&  0.21641E+00_JPRB, 0.21938E+00_JPRB, 0.22198E+00_JPRB, 0.22557E+00_JPRB, 0.23397E+00_JPRB, 0.24089E+00_JPRB, 0.25331E+00_JPRB ,&
&  0.25762E+00_JPRB, 0.26632E+00_JPRB, 0.26302E+00_JPRB, 0.26466E+00_JPRB, 0.26600E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.25997E+00_JPRB, 0.26091E+00_JPRB, 0.26070E+00_JPRB, 0.26172E+00_JPRB, 0.26247E+00_JPRB, 0.26360E+00_JPRB, 0.26348E+00_JPRB ,&
&  0.26312E+00_JPRB, 0.25937E+00_JPRB, 0.25736E+00_JPRB, 0.25395E+00_JPRB, 0.25083E+00_JPRB, 0.24743E+00_JPRB, 0.24467E+00_JPRB ,&
&  0.24293E+00_JPRB, 0.24155E+00_JPRB, 0.24030E+00_JPRB, 0.23855E+00_JPRB, 0.23631E+00_JPRB, 0.23403E+00_JPRB, 0.23196E+00_JPRB ,&
&  0.23006E+00_JPRB, 0.23021E+00_JPRB, 0.23165E+00_JPRB, 0.23399E+00_JPRB, 0.23832E+00_JPRB, 0.24131E+00_JPRB, 0.24517E+00_JPRB ,&
&  0.25086E+00_JPRB, 0.24662E+00_JPRB, 0.24682E+00_JPRB, 0.24733E+00_JPRB, 0.24723E+00_JPRB, 0.24721E+00_JPRB, 0.24500E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.25022E+00_JPRB, 0.25413E+00_JPRB, 0.25282E+00_JPRB, 0.24770E+00_JPRB, 0.24419E+00_JPRB, 0.23901E+00_JPRB ,&
&  0.23563E+00_JPRB, 0.23085E+00_JPRB, 0.22763E+00_JPRB, 0.22455E+00_JPRB, 0.22061E+00_JPRB, 0.21902E+00_JPRB, 0.21628E+00_JPRB ,&
&  0.21468E+00_JPRB, 0.21439E+00_JPRB, 0.21331E+00_JPRB, 0.21334E+00_JPRB, 0.21240E+00_JPRB, 0.21104E+00_JPRB, 0.21020E+00_JPRB ,&
&  0.20947E+00_JPRB, 0.20866E+00_JPRB, 0.20840E+00_JPRB, 0.20809E+00_JPRB, 0.20820E+00_JPRB, 0.20814E+00_JPRB, 0.20808E+00_JPRB ,&
&  0.20821E+00_JPRB, 0.20832E+00_JPRB, 0.20778E+00_JPRB, 0.20765E+00_JPRB, 0.20661E+00_JPRB, 0.20569E+00_JPRB, 0.20424E+00_JPRB ,&
&  0.20333E+00_JPRB, 0.20273E+00_JPRB, 0.20179E+00_JPRB, 0.20119E+00_JPRB, 0.20096E+00_JPRB, 0.20146E+00_JPRB, 0.20213E+00_JPRB ,&
&  0.20292E+00_JPRB, 0.20445E+00_JPRB, 0.20568E+00_JPRB, 0.20736E+00_JPRB, 0.20919E+00_JPRB, 0.21059E+00_JPRB, 0.21266E+00_JPRB ,&
&  0.21468E+00_JPRB, 0.21678E+00_JPRB, 0.22045E+00_JPRB, 0.22442E+00_JPRB, 0.22965E+00_JPRB, 0.23493E+00_JPRB, 0.23712E+00_JPRB ,&
&  0.23940E+00_JPRB, 0.24525E+00_JPRB, 0.24797E+00_JPRB, 0.25747E+00_JPRB, 0.25902E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.24895E+00_JPRB, 0.25076E+00_JPRB, 0.25250E+00_JPRB, 0.25417E+00_JPRB, 0.25622E+00_JPRB, 0.25709E+00_JPRB, 0.25717E+00_JPRB ,&
&  0.25479E+00_JPRB, 0.25442E+00_JPRB, 0.25295E+00_JPRB, 0.25096E+00_JPRB, 0.24935E+00_JPRB, 0.24576E+00_JPRB, 0.24268E+00_JPRB ,&
&  0.23966E+00_JPRB, 0.23676E+00_JPRB, 0.23428E+00_JPRB, 0.23082E+00_JPRB, 0.22756E+00_JPRB, 0.22426E+00_JPRB, 0.22356E+00_JPRB ,&
&  0.22339E+00_JPRB, 0.22415E+00_JPRB, 0.22622E+00_JPRB, 0.22772E+00_JPRB, 0.22992E+00_JPRB, 0.23223E+00_JPRB, 0.23569E+00_JPRB ,&
&  0.23883E+00_JPRB, 0.24108E+00_JPRB, 0.24127E+00_JPRB, 0.23990E+00_JPRB, 0.23899E+00_JPRB, 0.23888E+00_JPRB, 0.23866E+00_JPRB ,&
&  0.23810E+00_JPRB, 0.23802E+00_JPRB, 0.23762E+00_JPRB, 0.23734E+00_JPRB, 0.23600E+00_JPRB, 0.23337E+00_JPRB, 0.23062E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22574E+00_JPRB, 0.22348E+00_JPRB, 0.22148E+00_JPRB, 0.22032E+00_JPRB, 0.21912E+00_JPRB, 0.21754E+00_JPRB ,&
&  0.21664E+00_JPRB, 0.21568E+00_JPRB, 0.21462E+00_JPRB, 0.21385E+00_JPRB, 0.21302E+00_JPRB, 0.21245E+00_JPRB, 0.21178E+00_JPRB ,&
&  0.21131E+00_JPRB, 0.21050E+00_JPRB, 0.20981E+00_JPRB, 0.20904E+00_JPRB, 0.20870E+00_JPRB, 0.20778E+00_JPRB, 0.20690E+00_JPRB ,&
&  0.20579E+00_JPRB, 0.20510E+00_JPRB, 0.20381E+00_JPRB, 0.20303E+00_JPRB, 0.20211E+00_JPRB, 0.20126E+00_JPRB, 0.20065E+00_JPRB ,&
&  0.20061E+00_JPRB, 0.20048E+00_JPRB, 0.19980E+00_JPRB, 0.19957E+00_JPRB, 0.19950E+00_JPRB, 0.19992E+00_JPRB, 0.20143E+00_JPRB ,&
&  0.20239E+00_JPRB, 0.20344E+00_JPRB, 0.20443E+00_JPRB, 0.20517E+00_JPRB, 0.20650E+00_JPRB, 0.20776E+00_JPRB, 0.20905E+00_JPRB ,&
&  0.21059E+00_JPRB, 0.21188E+00_JPRB, 0.21340E+00_JPRB, 0.21515E+00_JPRB, 0.21970E+00_JPRB, 0.22252E+00_JPRB, 0.22533E+00_JPRB ,&
&  0.22702E+00_JPRB, 0.22452E+00_JPRB, 0.23951E+00_JPRB, 0.24236E+00_JPRB, 0.24457E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.22366E+00_JPRB, 0.22666E+00_JPRB, 0.22832E+00_JPRB, 0.22943E+00_JPRB, 0.22104E+00_JPRB, 0.23258E+00_JPRB, 0.22914E+00_JPRB ,&
&  0.22869E+00_JPRB, 0.22841E+00_JPRB, 0.23077E+00_JPRB, 0.23125E+00_JPRB, 0.23091E+00_JPRB, 0.23024E+00_JPRB, 0.22923E+00_JPRB ,&
&  0.22823E+00_JPRB, 0.22687E+00_JPRB, 0.22608E+00_JPRB, 0.22520E+00_JPRB, 0.22451E+00_JPRB, 0.22378E+00_JPRB, 0.22329E+00_JPRB ,&
&  0.22338E+00_JPRB, 0.22436E+00_JPRB, 0.22607E+00_JPRB, 0.22530E+00_JPRB, 0.22517E+00_JPRB, 0.22549E+00_JPRB, 0.22606E+00_JPRB ,&
&  0.22624E+00_JPRB, 0.22623E+00_JPRB, 0.22583E+00_JPRB, 0.22455E+00_JPRB, 0.22473E+00_JPRB, 0.22500E+00_JPRB, 0.22534E+00_JPRB ,&
&  0.22545E+00_JPRB, 0.22529E+00_JPRB, 0.22443E+00_JPRB, 0.22362E+00_JPRB, 0.22279E+00_JPRB, 0.22175E+00_JPRB, 0.22045E+00_JPRB ,&
&  0.21939E+00_JPRB, 0.21857E+00_JPRB, 0.21753E+00_JPRB, 0.21650E+00_JPRB, 0.21561E+00_JPRB, 0.21474E+00_JPRB, 0.21391E+00_JPRB ,&
&  0.21337E+00_JPRB, 0.21298E+00_JPRB, 0.21255E+00_JPRB, 0.21192E+00_JPRB, 0.21121E+00_JPRB, 0.21055E+00_JPRB, 0.20996E+00_JPRB ,&
&  0.20958E+00_JPRB, 0.20892E+00_JPRB, 0.20826E+00_JPRB, 0.20753E+00_JPRB, 0.20698E+00_JPRB, 0.20605E+00_JPRB, 0.20524E+00_JPRB ,&
&  0.20444E+00_JPRB, 0.20371E+00_JPRB, 0.20315E+00_JPRB, 0.20272E+00_JPRB, 0.20234E+00_JPRB, 0.20205E+00_JPRB, 0.20182E+00_JPRB ,&
&  0.20164E+00_JPRB, 0.20149E+00_JPRB, 0.20143E+00_JPRB, 0.20129E+00_JPRB, 0.20111E+00_JPRB, 0.20103E+00_JPRB, 0.20131E+00_JPRB ,&
&  0.20153E+00_JPRB, 0.20168E+00_JPRB, 0.20174E+00_JPRB, 0.20132E+00_JPRB, 0.20176E+00_JPRB, 0.20213E+00_JPRB, 0.20260E+00_JPRB ,&
&  0.20298E+00_JPRB, 0.20386E+00_JPRB, 0.20494E+00_JPRB, 0.20626E+00_JPRB, 0.20887E+00_JPRB, 0.21111E+00_JPRB, 0.21352E+00_JPRB ,&
&  0.21623E+00_JPRB, 0.22020E+00_JPRB, 0.22170E+00_JPRB, 0.22262E+00_JPRB, 0.22323E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.21224E+00_JPRB, 0.21212E+00_JPRB, 0.21181E+00_JPRB, 0.21080E+00_JPRB, 0.20791E+00_JPRB, 0.20733E+00_JPRB, 0.20725E+00_JPRB ,&
&  0.20741E+00_JPRB, 0.20751E+00_JPRB, 0.20857E+00_JPRB, 0.20970E+00_JPRB, 0.21084E+00_JPRB, 0.21173E+00_JPRB, 0.21250E+00_JPRB ,&
&  0.21293E+00_JPRB, 0.21336E+00_JPRB, 0.21201E+00_JPRB, 0.21134E+00_JPRB, 0.21112E+00_JPRB, 0.21109E+00_JPRB, 0.21107E+00_JPRB ,&
&  0.21098E+00_JPRB, 0.21095E+00_JPRB, 0.21080E+00_JPRB, 0.21105E+00_JPRB, 0.21143E+00_JPRB, 0.21173E+00_JPRB, 0.21210E+00_JPRB ,&
&  0.21238E+00_JPRB, 0.21263E+00_JPRB, 0.21280E+00_JPRB, 0.21293E+00_JPRB, 0.21335E+00_JPRB, 0.21359E+00_JPRB, 0.21355E+00_JPRB ,&
&  0.21340E+00_JPRB, 0.21304E+00_JPRB, 0.21263E+00_JPRB, 0.21220E+00_JPRB, 0.21189E+00_JPRB, 0.21159E+00_JPRB, 0.21128E+00_JPRB ,&
&  0.21097E+00_JPRB, 0.21064E+00_JPRB, 0.21029E+00_JPRB, 0.20989E+00_JPRB, 0.20950E+00_JPRB, 0.20922E+00_JPRB, 0.20891E+00_JPRB ,&
&  0.20865E+00_JPRB, 0.20840E+00_JPRB, 0.20813E+00_JPRB, 0.20778E+00_JPRB, 0.20740E+00_JPRB, 0.20702E+00_JPRB, 0.20659E+00_JPRB ,&
&  0.20605E+00_JPRB, 0.20567E+00_JPRB, 0.20530E+00_JPRB, 0.20496E+00_JPRB, 0.20468E+00_JPRB, 0.20438E+00_JPRB, 0.20411E+00_JPRB ,&
&  0.20383E+00_JPRB, 0.20356E+00_JPRB, 0.20323E+00_JPRB, 0.20294E+00_JPRB, 0.20266E+00_JPRB, 0.20242E+00_JPRB, 0.20220E+00_JPRB ,&
&  0.20200E+00_JPRB, 0.20179E+00_JPRB, 0.20161E+00_JPRB, 0.20160E+00_JPRB, 0.20154E+00_JPRB, 0.20155E+00_JPRB, 0.20155E+00_JPRB ,&
&  0.20157E+00_JPRB, 0.20157E+00_JPRB, 0.20155E+00_JPRB, 0.20140E+00_JPRB, 0.20155E+00_JPRB, 0.20178E+00_JPRB, 0.20215E+00_JPRB ,&
&  0.20258E+00_JPRB, 0.20314E+00_JPRB, 0.20373E+00_JPRB, 0.20437E+00_JPRB, 0.20512E+00_JPRB, 0.20574E+00_JPRB, 0.20644E+00_JPRB ,&
&  0.20722E+00_JPRB, 0.20803E+00_JPRB, 0.20860E+00_JPRB, 0.20928E+00_JPRB, 0.21032E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB, 0.20520E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB, 0.10110E+01_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.10281E+01_JPRB, 0.10264E+01_JPRB, 0.10250E+01_JPRB, 0.10242E+01_JPRB, 0.10237E+01_JPRB, 0.10232E+01_JPRB, 0.10213E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.10158E+01_JPRB, 0.10127E+01_JPRB, 0.10090E+01_JPRB, 0.10043E+01_JPRB, 0.99863E+00_JPRB, 0.99352E+00_JPRB ,&
&  0.98876E+00_JPRB, 0.98479E+00_JPRB, 0.98064E+00_JPRB, 0.97776E+00_JPRB, 0.97462E+00_JPRB, 0.97145E+00_JPRB, 0.96811E+00_JPRB ,&
&  0.96544E+00_JPRB, 0.96292E+00_JPRB, 0.96046E+00_JPRB, 0.95830E+00_JPRB, 0.95740E+00_JPRB, 0.95621E+00_JPRB, 0.95495E+00_JPRB ,&
&  0.95387E+00_JPRB, 0.95326E+00_JPRB, 0.95280E+00_JPRB, 0.95233E+00_JPRB, 0.95207E+00_JPRB, 0.95242E+00_JPRB, 0.95277E+00_JPRB ,&
&  0.95340E+00_JPRB, 0.95497E+00_JPRB, 0.95731E+00_JPRB, 0.95945E+00_JPRB, 0.96183E+00_JPRB, 0.96518E+00_JPRB, 0.96955E+00_JPRB ,&
&  0.97346E+00_JPRB, 0.97729E+00_JPRB, 0.98158E+00_JPRB, 0.98648E+00_JPRB, 0.99113E+00_JPRB, 0.99594E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10051E+01_JPRB, 0.10088E+01_JPRB, 0.10125E+01_JPRB, 0.10144E+01_JPRB, 0.10172E+01_JPRB, 0.10195E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.10233E+01_JPRB, 0.10239E+01_JPRB, 0.10249E+01_JPRB, 0.10257E+01_JPRB, 0.10266E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10292E+01_JPRB, 0.10301E+01_JPRB, 0.10315E+01_JPRB, 0.10324E+01_JPRB, 0.10336E+01_JPRB, 0.10339E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10365E+01_JPRB, 0.10366E+01_JPRB, 0.10368E+01_JPRB, 0.10368E+01_JPRB, 0.10370E+01_JPRB, 0.10372E+01_JPRB ,&
&  0.10375E+01_JPRB, 0.10376E+01_JPRB, 0.10376E+01_JPRB, 0.10369E+01_JPRB, 0.10357E+01_JPRB, 0.10346E+01_JPRB, 0.10337E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10327E+01_JPRB, 0.10327E+01_JPRB, 0.10331E+01_JPRB, 0.10343E+01_JPRB, 0.10345E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.10345E+01_JPRB, 0.10342E+01_JPRB, 0.10331E+01_JPRB, 0.10316E+01_JPRB, 0.10299E+01_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.10276E+01_JPRB, 0.10257E+01_JPRB, 0.10247E+01_JPRB, 0.10247E+01_JPRB, 0.10252E+01_JPRB, 0.10252E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.10150E+01_JPRB, 0.10072E+01_JPRB, 0.99859E+00_JPRB, 0.98820E+00_JPRB, 0.97855E+00_JPRB, 0.97240E+00_JPRB ,&
&  0.96613E+00_JPRB, 0.95982E+00_JPRB, 0.95357E+00_JPRB, 0.94861E+00_JPRB, 0.94290E+00_JPRB, 0.93690E+00_JPRB, 0.93098E+00_JPRB ,&
&  0.92654E+00_JPRB, 0.92157E+00_JPRB, 0.91598E+00_JPRB, 0.90894E+00_JPRB, 0.90422E+00_JPRB, 0.89919E+00_JPRB, 0.89462E+00_JPRB ,&
&  0.88859E+00_JPRB, 0.88559E+00_JPRB, 0.88406E+00_JPRB, 0.88407E+00_JPRB, 0.88903E+00_JPRB, 0.89291E+00_JPRB, 0.89369E+00_JPRB ,&
&  0.89259E+00_JPRB, 0.88689E+00_JPRB, 0.88745E+00_JPRB, 0.88894E+00_JPRB, 0.89228E+00_JPRB, 0.89932E+00_JPRB, 0.90862E+00_JPRB ,&
&  0.92232E+00_JPRB, 0.93916E+00_JPRB, 0.95725E+00_JPRB, 0.97187E+00_JPRB, 0.98632E+00_JPRB, 0.99925E+00_JPRB, 0.10087E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.10305E+01_JPRB, 0.10406E+01_JPRB, 0.10497E+01_JPRB, 0.10608E+01_JPRB, 0.10694E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.10767E+01_JPRB, 0.10759E+01_JPRB, 0.10754E+01_JPRB, 0.10728E+01_JPRB, 0.10713E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10717E+01_JPRB, 0.10717E+01_JPRB, 0.10737E+01_JPRB, 0.10761E+01_JPRB, 0.10788E+01_JPRB, 0.10784E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10787E+01_JPRB, 0.10764E+01_JPRB, 0.10718E+01_JPRB, 0.10680E+01_JPRB, 0.10658E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.10566E+01_JPRB, 0.10550E+01_JPRB, 0.10524E+01_JPRB, 0.10486E+01_JPRB, 0.10451E+01_JPRB, 0.10430E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.10386E+01_JPRB, 0.10379E+01_JPRB, 0.10375E+01_JPRB, 0.10405E+01_JPRB, 0.10409E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10399E+01_JPRB, 0.10398E+01_JPRB, 0.10378E+01_JPRB, 0.10346E+01_JPRB, 0.10304E+01_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.10374E+01_JPRB, 0.10385E+01_JPRB, 0.10427E+01_JPRB, 0.10436E+01_JPRB, 0.10430E+01_JPRB, 0.10393E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10306E+01_JPRB, 0.10195E+01_JPRB, 0.10023E+01_JPRB, 0.98762E+00_JPRB, 0.97338E+00_JPRB, 0.96365E+00_JPRB, 0.95522E+00_JPRB ,&
&  0.94665E+00_JPRB, 0.93973E+00_JPRB, 0.93185E+00_JPRB, 0.92252E+00_JPRB, 0.91083E+00_JPRB, 0.89747E+00_JPRB, 0.88010E+00_JPRB ,&
&  0.86753E+00_JPRB, 0.85560E+00_JPRB, 0.84591E+00_JPRB, 0.83984E+00_JPRB, 0.83904E+00_JPRB, 0.80847E+00_JPRB, 0.80375E+00_JPRB ,&
&  0.81952E+00_JPRB, 0.80347E+00_JPRB, 0.78718E+00_JPRB, 0.76040E+00_JPRB, 0.75110E+00_JPRB, 0.76391E+00_JPRB, 0.77404E+00_JPRB ,&
&  0.76242E+00_JPRB, 0.80777E+00_JPRB, 0.81301E+00_JPRB, 0.81502E+00_JPRB, 0.81169E+00_JPRB, 0.80257E+00_JPRB, 0.81585E+00_JPRB ,&
&  0.85043E+00_JPRB, 0.86472E+00_JPRB, 0.89580E+00_JPRB, 0.93172E+00_JPRB, 0.95923E+00_JPRB, 0.98742E+00_JPRB, 0.10156E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10639E+01_JPRB, 0.10796E+01_JPRB, 0.10941E+01_JPRB, 0.11168E+01_JPRB, 0.11388E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.11795E+01_JPRB, 0.11887E+01_JPRB, 0.11978E+01_JPRB, 0.12104E+01_JPRB, 0.12211E+01_JPRB, 0.12322E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.12474E+01_JPRB, 0.12512E+01_JPRB, 0.12521E+01_JPRB, 0.12511E+01_JPRB, 0.12455E+01_JPRB, 0.12438E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.12239E+01_JPRB, 0.12095E+01_JPRB, 0.11911E+01_JPRB, 0.11763E+01_JPRB, 0.11643E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.11396E+01_JPRB, 0.11274E+01_JPRB, 0.11163E+01_JPRB, 0.11032E+01_JPRB, 0.10909E+01_JPRB, 0.10782E+01_JPRB, 0.10692E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.10575E+01_JPRB, 0.10552E+01_JPRB, 0.10520E+01_JPRB, 0.10523E+01_JPRB, 0.10512E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10491E+01_JPRB, 0.10423E+01_JPRB, 0.10383E+01_JPRB, 0.10358E+01_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.10353E+01_JPRB, 0.10407E+01_JPRB, 0.10523E+01_JPRB, 0.10528E+01_JPRB, 0.10512E+01_JPRB, 0.10423E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.10161E+01_JPRB, 0.99003E+00_JPRB, 0.97280E+00_JPRB, 0.95760E+00_JPRB, 0.94813E+00_JPRB, 0.94104E+00_JPRB ,&
&  0.93521E+00_JPRB, 0.92759E+00_JPRB, 0.91774E+00_JPRB, 0.90318E+00_JPRB, 0.88478E+00_JPRB, 0.86381E+00_JPRB, 0.83464E+00_JPRB ,&
&  0.81026E+00_JPRB, 0.79295E+00_JPRB, 0.77056E+00_JPRB, 0.77931E+00_JPRB, 0.76815E+00_JPRB, 0.75965E+00_JPRB, 0.75387E+00_JPRB ,&
&  0.73686E+00_JPRB, 0.71864E+00_JPRB, 0.69082E+00_JPRB, 0.67664E+00_JPRB, 0.66700E+00_JPRB, 0.65515E+00_JPRB, 0.67500E+00_JPRB ,&
&  0.69066E+00_JPRB, 0.73232E+00_JPRB, 0.75084E+00_JPRB, 0.75644E+00_JPRB, 0.76307E+00_JPRB, 0.76060E+00_JPRB, 0.77067E+00_JPRB ,&
&  0.78325E+00_JPRB, 0.80047E+00_JPRB, 0.85883E+00_JPRB, 0.89874E+00_JPRB, 0.92850E+00_JPRB, 0.95633E+00_JPRB, 0.97778E+00_JPRB ,&
&  0.10049E+01_JPRB, 0.10219E+01_JPRB, 0.10384E+01_JPRB, 0.10586E+01_JPRB, 0.10916E+01_JPRB, 0.11299E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.12126E+01_JPRB, 0.12367E+01_JPRB, 0.12644E+01_JPRB, 0.13020E+01_JPRB, 0.13439E+01_JPRB, 0.13884E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.14770E+01_JPRB, 0.15129E+01_JPRB, 0.15441E+01_JPRB, 0.15600E+01_JPRB, 0.15626E+01_JPRB, 0.15583E+01_JPRB ,&
&  0.15334E+01_JPRB, 0.15044E+01_JPRB, 0.14642E+01_JPRB, 0.14044E+01_JPRB, 0.13583E+01_JPRB, 0.13330E+01_JPRB, 0.12888E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.12162E+01_JPRB, 0.11837E+01_JPRB, 0.11496E+01_JPRB, 0.11192E+01_JPRB, 0.10914E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10489E+01_JPRB, 0.10436E+01_JPRB, 0.10401E+01_JPRB, 0.10407E+01_JPRB, 0.10395E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10417E+01_JPRB, 0.10333E+01_JPRB, 0.10306E+01_JPRB, 0.10299E+01_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.10374E+01_JPRB, 0.10403E+01_JPRB, 0.10397E+01_JPRB, 0.10267E+01_JPRB, 0.10203E+01_JPRB, 0.10138E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.10006E+01_JPRB, 0.97641E+00_JPRB, 0.96026E+00_JPRB, 0.94297E+00_JPRB, 0.93326E+00_JPRB, 0.92431E+00_JPRB ,&
&  0.91740E+00_JPRB, 0.90847E+00_JPRB, 0.90172E+00_JPRB, 0.88692E+00_JPRB, 0.87224E+00_JPRB, 0.84986E+00_JPRB, 0.82510E+00_JPRB ,&
&  0.79918E+00_JPRB, 0.77285E+00_JPRB, 0.75175E+00_JPRB, 0.75419E+00_JPRB, 0.74398E+00_JPRB, 0.73681E+00_JPRB, 0.72690E+00_JPRB ,&
&  0.71496E+00_JPRB, 0.66884E+00_JPRB, 0.68631E+00_JPRB, 0.66885E+00_JPRB, 0.64989E+00_JPRB, 0.62600E+00_JPRB, 0.60584E+00_JPRB ,&
&  0.61055E+00_JPRB, 0.61435E+00_JPRB, 0.63101E+00_JPRB, 0.65796E+00_JPRB, 0.67093E+00_JPRB, 0.69351E+00_JPRB, 0.70773E+00_JPRB ,&
&  0.69916E+00_JPRB, 0.79880E+00_JPRB, 0.85096E+00_JPRB, 0.88341E+00_JPRB, 0.91508E+00_JPRB, 0.93975E+00_JPRB, 0.95549E+00_JPRB ,&
&  0.98507E+00_JPRB, 0.10158E+01_JPRB, 0.10530E+01_JPRB, 0.10924E+01_JPRB, 0.11500E+01_JPRB, 0.11911E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.12610E+01_JPRB, 0.12779E+01_JPRB, 0.13046E+01_JPRB, 0.13463E+01_JPRB, 0.13989E+01_JPRB, 0.14571E+01_JPRB, 0.15204E+01_JPRB ,&
&  0.15911E+01_JPRB, 0.16578E+01_JPRB, 0.17274E+01_JPRB, 0.17885E+01_JPRB, 0.18370E+01_JPRB, 0.18672E+01_JPRB, 0.18948E+01_JPRB ,&
&  0.18721E+01_JPRB, 0.18391E+01_JPRB, 0.17851E+01_JPRB, 0.17100E+01_JPRB, 0.16162E+01_JPRB, 0.15132E+01_JPRB, 0.14111E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.12939E+01_JPRB, 0.12392E+01_JPRB, 0.11867E+01_JPRB, 0.11423E+01_JPRB, 0.11066E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.10571E+01_JPRB, 0.10497E+01_JPRB, 0.10436E+01_JPRB, 0.10406E+01_JPRB, 0.10332E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.10324E+01_JPRB, 0.10295E+01_JPRB, 0.10296E+01_JPRB, 0.10322E+01_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.10472E+01_JPRB, 0.10501E+01_JPRB, 0.10413E+01_JPRB, 0.10314E+01_JPRB, 0.10271E+01_JPRB, 0.10145E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.10103E+01_JPRB, 0.10089E+01_JPRB, 0.99618E+00_JPRB, 0.97999E+00_JPRB, 0.95863E+00_JPRB, 0.94321E+00_JPRB, 0.92598E+00_JPRB ,&
&  0.91098E+00_JPRB, 0.89254E+00_JPRB, 0.87167E+00_JPRB, 0.85168E+00_JPRB, 0.83601E+00_JPRB, 0.81687E+00_JPRB, 0.80282E+00_JPRB ,&
&  0.78722E+00_JPRB, 0.76764E+00_JPRB, 0.75312E+00_JPRB, 0.75229E+00_JPRB, 0.74077E+00_JPRB, 0.72798E+00_JPRB, 0.72821E+00_JPRB ,&
&  0.73100E+00_JPRB, 0.72303E+00_JPRB, 0.71418E+00_JPRB, 0.69610E+00_JPRB, 0.65744E+00_JPRB, 0.58695E+00_JPRB, 0.60515E+00_JPRB ,&
&  0.59887E+00_JPRB, 0.58611E+00_JPRB, 0.59112E+00_JPRB, 0.60458E+00_JPRB, 0.61844E+00_JPRB, 0.65690E+00_JPRB, 0.66107E+00_JPRB ,&
&  0.64499E+00_JPRB, 0.78886E+00_JPRB, 0.81927E+00_JPRB, 0.85649E+00_JPRB, 0.89073E+00_JPRB, 0.91426E+00_JPRB, 0.93526E+00_JPRB ,&
&  0.96044E+00_JPRB, 0.99502E+00_JPRB, 0.10301E+01_JPRB, 0.10818E+01_JPRB, 0.11396E+01_JPRB, 0.12122E+01_JPRB, 0.12767E+01_JPRB ,&
&  0.13160E+01_JPRB, 0.13421E+01_JPRB, 0.13779E+01_JPRB, 0.14081E+01_JPRB, 0.14429E+01_JPRB, 0.15015E+01_JPRB, 0.15765E+01_JPRB ,&
&  0.16805E+01_JPRB, 0.17462E+01_JPRB, 0.18825E+01_JPRB, 0.19906E+01_JPRB, 0.20472E+01_JPRB, 0.21228E+01_JPRB, 0.21953E+01_JPRB ,&
&  0.21711E+01_JPRB, 0.21594E+01_JPRB, 0.21171E+01_JPRB, 0.19594E+01_JPRB, 0.17874E+01_JPRB, 0.16690E+01_JPRB, 0.15633E+01_JPRB ,&
&  0.14793E+01_JPRB, 0.13955E+01_JPRB, 0.13119E+01_JPRB, 0.12466E+01_JPRB, 0.11951E+01_JPRB, 0.11576E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.10971E+01_JPRB, 0.10789E+01_JPRB, 0.10704E+01_JPRB, 0.10531E+01_JPRB, 0.10384E+01_JPRB, 0.10231E+01_JPRB ,&
&  0.10187E+01_JPRB, 0.10174E+01_JPRB, 0.10240E+01_JPRB, 0.10303E+01_JPRB, 0.10341E+01_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.10649E+01_JPRB, 0.10847E+01_JPRB, 0.10869E+01_JPRB, 0.10863E+01_JPRB, 0.10842E+01_JPRB, 0.10631E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.10280E+01_JPRB, 0.10180E+01_JPRB, 0.10046E+01_JPRB, 0.98731E+00_JPRB, 0.97306E+00_JPRB, 0.95970E+00_JPRB ,&
&  0.94698E+00_JPRB, 0.89142E+00_JPRB, 0.86331E+00_JPRB, 0.83970E+00_JPRB, 0.81644E+00_JPRB, 0.80389E+00_JPRB, 0.79723E+00_JPRB ,&
&  0.78564E+00_JPRB, 0.77887E+00_JPRB, 0.77354E+00_JPRB, 0.76345E+00_JPRB, 0.75039E+00_JPRB, 0.73288E+00_JPRB, 0.72590E+00_JPRB ,&
&  0.73121E+00_JPRB, 0.71500E+00_JPRB, 0.70110E+00_JPRB, 0.67934E+00_JPRB, 0.63756E+00_JPRB, 0.61836E+00_JPRB, 0.61928E+00_JPRB ,&
&  0.61217E+00_JPRB, 0.58238E+00_JPRB, 0.58448E+00_JPRB, 0.58839E+00_JPRB, 0.59088E+00_JPRB, 0.60934E+00_JPRB, 0.64370E+00_JPRB ,&
&  0.67540E+00_JPRB, 0.78365E+00_JPRB, 0.82928E+00_JPRB, 0.88000E+00_JPRB, 0.91581E+00_JPRB, 0.94532E+00_JPRB, 0.97092E+00_JPRB ,&
&  0.10006E+01_JPRB, 0.10432E+01_JPRB, 0.10905E+01_JPRB, 0.11538E+01_JPRB, 0.12241E+01_JPRB, 0.12835E+01_JPRB, 0.13421E+01_JPRB ,&
&  0.14180E+01_JPRB, 0.14070E+01_JPRB, 0.14377E+01_JPRB, 0.14514E+01_JPRB, 0.14834E+01_JPRB, 0.15598E+01_JPRB, 0.15942E+01_JPRB ,&
&  0.16853E+01_JPRB, 0.18225E+01_JPRB, 0.20450E+01_JPRB, 0.22617E+01_JPRB, 0.23636E+01_JPRB, 0.24696E+01_JPRB, 0.25028E+01_JPRB ,&
&  0.23827E+01_JPRB, 0.23133E+01_JPRB, 0.23711E+01_JPRB, 0.21820E+01_JPRB, 0.19750E+01_JPRB, 0.17998E+01_JPRB, 0.16779E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.14448E+01_JPRB, 0.13717E+01_JPRB, 0.13130E+01_JPRB, 0.12532E+01_JPRB, 0.12005E+01_JPRB, 0.11921E+01_JPRB ,&
&  0.11847E+01_JPRB, 0.11739E+01_JPRB, 0.11559E+01_JPRB, 0.11343E+01_JPRB, 0.10953E+01_JPRB, 0.10562E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10140E+01_JPRB, 0.10092E+01_JPRB, 0.10178E+01_JPRB, 0.10256E+01_JPRB, 0.10412E+01_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.10761E+01_JPRB, 0.10934E+01_JPRB, 0.10942E+01_JPRB, 0.10974E+01_JPRB, 0.11034E+01_JPRB, 0.10467E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.98097E+00_JPRB, 0.98989E+00_JPRB, 0.99385E+00_JPRB, 0.99017E+00_JPRB, 0.97786E+00_JPRB, 0.96309E+00_JPRB, 0.95120E+00_JPRB ,&
&  0.94050E+00_JPRB, 0.91441E+00_JPRB, 0.86715E+00_JPRB, 0.80367E+00_JPRB, 0.78478E+00_JPRB, 0.77806E+00_JPRB, 0.77994E+00_JPRB ,&
&  0.77936E+00_JPRB, 0.77377E+00_JPRB, 0.77566E+00_JPRB, 0.76119E+00_JPRB, 0.75127E+00_JPRB, 0.72276E+00_JPRB, 0.70012E+00_JPRB ,&
&  0.70007E+00_JPRB, 0.69389E+00_JPRB, 0.70176E+00_JPRB, 0.68836E+00_JPRB, 0.67124E+00_JPRB, 0.65873E+00_JPRB, 0.60106E+00_JPRB ,&
&  0.59191E+00_JPRB, 0.57559E+00_JPRB, 0.57019E+00_JPRB, 0.57046E+00_JPRB, 0.59376E+00_JPRB, 0.61353E+00_JPRB, 0.64112E+00_JPRB ,&
&  0.66117E+00_JPRB, 0.78587E+00_JPRB, 0.82688E+00_JPRB, 0.85840E+00_JPRB, 0.88692E+00_JPRB, 0.92608E+00_JPRB, 0.98104E+00_JPRB ,&
&  0.10382E+01_JPRB, 0.10959E+01_JPRB, 0.11436E+01_JPRB, 0.12171E+01_JPRB, 0.12910E+01_JPRB, 0.12912E+01_JPRB, 0.14061E+01_JPRB ,&
&  0.18116E+01_JPRB, 0.17138E+01_JPRB, 0.16309E+01_JPRB, 0.15959E+01_JPRB, 0.16165E+01_JPRB, 0.16549E+01_JPRB, 0.17206E+01_JPRB ,&
&  0.18805E+01_JPRB, 0.21369E+01_JPRB, 0.25146E+01_JPRB, 0.27736E+01_JPRB, 0.27827E+01_JPRB, 0.28111E+01_JPRB, 0.27652E+01_JPRB ,&
&  0.25734E+01_JPRB, 0.24440E+01_JPRB, 0.25396E+01_JPRB, 0.21001E+01_JPRB, 0.17909E+01_JPRB, 0.16254E+01_JPRB, 0.15514E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.14016E+01_JPRB, 0.13581E+01_JPRB, 0.13017E+01_JPRB, 0.12279E+01_JPRB, 0.11850E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.11594E+01_JPRB, 0.11497E+01_JPRB, 0.11340E+01_JPRB, 0.11051E+01_JPRB, 0.10728E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10446E+01_JPRB, 0.10384E+01_JPRB, 0.10392E+01_JPRB, 0.10510E+01_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.10757E+01_JPRB, 0.10896E+01_JPRB, 0.10889E+01_JPRB, 0.10729E+01_JPRB, 0.10512E+01_JPRB, 0.97132E+00_JPRB, 0.88480E+00_JPRB ,&
&  0.82169E+00_JPRB, 0.82588E+00_JPRB, 0.85584E+00_JPRB, 0.88105E+00_JPRB, 0.88270E+00_JPRB, 0.86319E+00_JPRB, 0.85320E+00_JPRB ,&
&  0.86013E+00_JPRB, 0.85880E+00_JPRB, 0.82777E+00_JPRB, 0.80456E+00_JPRB, 0.76655E+00_JPRB, 0.75787E+00_JPRB, 0.75935E+00_JPRB ,&
&  0.76245E+00_JPRB, 0.76043E+00_JPRB, 0.76137E+00_JPRB, 0.75945E+00_JPRB, 0.75968E+00_JPRB, 0.74981E+00_JPRB, 0.73123E+00_JPRB ,&
&  0.70800E+00_JPRB, 0.70378E+00_JPRB, 0.70656E+00_JPRB, 0.71188E+00_JPRB, 0.70423E+00_JPRB, 0.69027E+00_JPRB, 0.61572E+00_JPRB ,&
&  0.58731E+00_JPRB, 0.56787E+00_JPRB, 0.56906E+00_JPRB, 0.58382E+00_JPRB, 0.62192E+00_JPRB, 0.62905E+00_JPRB, 0.71944E+00_JPRB ,&
&  0.78435E+00_JPRB, 0.82647E+00_JPRB, 0.84196E+00_JPRB, 0.85477E+00_JPRB, 0.88425E+00_JPRB, 0.95158E+00_JPRB, 0.10400E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.11543E+01_JPRB, 0.11584E+01_JPRB, 0.10383E+01_JPRB, 0.10526E+01_JPRB, 0.11695E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.15239E+01_JPRB, 0.15717E+01_JPRB, 0.15843E+01_JPRB, 0.16689E+01_JPRB, 0.16947E+01_JPRB, 0.17151E+01_JPRB, 0.17322E+01_JPRB ,&
&  0.19546E+01_JPRB, 0.22770E+01_JPRB, 0.27697E+01_JPRB, 0.30226E+01_JPRB, 0.29485E+01_JPRB, 0.29663E+01_JPRB, 0.28421E+01_JPRB ,&
&  0.27374E+01_JPRB, 0.24386E+01_JPRB, 0.21735E+01_JPRB, 0.18637E+01_JPRB, 0.16167E+01_JPRB, 0.14746E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.12690E+01_JPRB, 0.12681E+01_JPRB, 0.12373E+01_JPRB, 0.11673E+01_JPRB, 0.11012E+01_JPRB, 0.10513E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.99784E+00_JPRB, 0.99561E+00_JPRB, 0.97232E+00_JPRB, 0.97292E+00_JPRB, 0.98755E+00_JPRB, 0.99180E+00_JPRB ,&
&  0.98934E+00_JPRB, 0.98751E+00_JPRB, 0.10015E+01_JPRB, 0.10221E+01_JPRB, 0.10429E+01_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.11175E+01_JPRB, 0.10891E+01_JPRB, 0.10494E+01_JPRB, 0.10410E+01_JPRB, 0.10119E+01_JPRB, 0.98704E+00_JPRB, 0.89257E+00_JPRB ,&
&  0.77497E+00_JPRB, 0.74298E+00_JPRB, 0.77308E+00_JPRB, 0.78662E+00_JPRB, 0.76968E+00_JPRB, 0.76474E+00_JPRB, 0.77258E+00_JPRB ,&
&  0.78153E+00_JPRB, 0.81138E+00_JPRB, 0.80906E+00_JPRB, 0.78867E+00_JPRB, 0.77432E+00_JPRB, 0.76592E+00_JPRB, 0.76765E+00_JPRB ,&
&  0.76697E+00_JPRB, 0.76542E+00_JPRB, 0.75653E+00_JPRB, 0.74202E+00_JPRB, 0.73966E+00_JPRB, 0.73739E+00_JPRB, 0.73333E+00_JPRB ,&
&  0.73106E+00_JPRB, 0.72392E+00_JPRB, 0.72988E+00_JPRB, 0.73192E+00_JPRB, 0.72203E+00_JPRB, 0.69030E+00_JPRB, 0.62131E+00_JPRB ,&
&  0.58799E+00_JPRB, 0.57340E+00_JPRB, 0.60417E+00_JPRB, 0.67881E+00_JPRB, 0.73602E+00_JPRB, 0.78599E+00_JPRB, 0.82934E+00_JPRB ,&
&  0.80763E+00_JPRB, 0.80910E+00_JPRB, 0.84951E+00_JPRB, 0.96815E+00_JPRB, 0.10276E+01_JPRB, 0.11046E+01_JPRB, 0.11802E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.13253E+01_JPRB, 0.13386E+01_JPRB, 0.11803E+01_JPRB, 0.12707E+01_JPRB, 0.15892E+01_JPRB, 0.15486E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.17715E+01_JPRB, 0.18758E+01_JPRB, 0.18536E+01_JPRB, 0.17527E+01_JPRB, 0.17796E+01_JPRB, 0.19338E+01_JPRB ,&
&  0.21670E+01_JPRB, 0.25165E+01_JPRB, 0.31732E+01_JPRB, 0.35318E+01_JPRB, 0.34660E+01_JPRB, 0.33941E+01_JPRB, 0.33097E+01_JPRB ,&
&  0.32490E+01_JPRB, 0.29225E+01_JPRB, 0.25038E+01_JPRB, 0.21221E+01_JPRB, 0.17931E+01_JPRB, 0.15363E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.12951E+01_JPRB, 0.13041E+01_JPRB, 0.12793E+01_JPRB, 0.11975E+01_JPRB, 0.10967E+01_JPRB, 0.10134E+01_JPRB ,&
&  0.95975E+00_JPRB, 0.91041E+00_JPRB, 0.86275E+00_JPRB, 0.84827E+00_JPRB, 0.89815E+00_JPRB, 0.96344E+00_JPRB, 0.98554E+00_JPRB ,&
&  0.96812E+00_JPRB, 0.96230E+00_JPRB, 0.10110E+01_JPRB, 0.10624E+01_JPRB, 0.10819E+01_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.11719E+01_JPRB, 0.11292E+01_JPRB, 0.10901E+01_JPRB, 0.10542E+01_JPRB, 0.10195E+01_JPRB, 0.95671E+00_JPRB, 0.85392E+00_JPRB ,&
&  0.75553E+00_JPRB, 0.72149E+00_JPRB, 0.72931E+00_JPRB, 0.71159E+00_JPRB, 0.68600E+00_JPRB, 0.67151E+00_JPRB, 0.67604E+00_JPRB ,&
&  0.69177E+00_JPRB, 0.73081E+00_JPRB, 0.75986E+00_JPRB, 0.76931E+00_JPRB, 0.76586E+00_JPRB, 0.75991E+00_JPRB, 0.76733E+00_JPRB ,&
&  0.77871E+00_JPRB, 0.79021E+00_JPRB, 0.79485E+00_JPRB, 0.78600E+00_JPRB, 0.77585E+00_JPRB, 0.77115E+00_JPRB, 0.76679E+00_JPRB ,&
&  0.76552E+00_JPRB, 0.76137E+00_JPRB, 0.74460E+00_JPRB, 0.74293E+00_JPRB, 0.72092E+00_JPRB, 0.71416E+00_JPRB, 0.67166E+00_JPRB ,&
&  0.62068E+00_JPRB, 0.62901E+00_JPRB, 0.75538E+00_JPRB, 0.80870E+00_JPRB, 0.82622E+00_JPRB, 0.86575E+00_JPRB, 0.92128E+00_JPRB ,&
&  0.97026E+00_JPRB, 0.10139E+01_JPRB, 0.10562E+01_JPRB, 0.11070E+01_JPRB, 0.11675E+01_JPRB, 0.12738E+01_JPRB, 0.14021E+01_JPRB ,&
&  0.15623E+01_JPRB, 0.13585E+01_JPRB, 0.12997E+01_JPRB, 0.14306E+01_JPRB, 0.18940E+01_JPRB, 0.21195E+01_JPRB, 0.21663E+01_JPRB ,&
&  0.22125E+01_JPRB, 0.21554E+01_JPRB, 0.21038E+01_JPRB, 0.21151E+01_JPRB, 0.22147E+01_JPRB, 0.23941E+01_JPRB, 0.26822E+01_JPRB ,&
&  0.29286E+01_JPRB, 0.33761E+01_JPRB, 0.40322E+01_JPRB, 0.41826E+01_JPRB, 0.40978E+01_JPRB, 0.40283E+01_JPRB, 0.38765E+01_JPRB ,&
&  0.37018E+01_JPRB, 0.33589E+01_JPRB, 0.28090E+01_JPRB, 0.23479E+01_JPRB, 0.19605E+01_JPRB, 0.16675E+01_JPRB, 0.14342E+01_JPRB ,&
&  0.13254E+01_JPRB, 0.13089E+01_JPRB, 0.13147E+01_JPRB, 0.12970E+01_JPRB, 0.12452E+01_JPRB, 0.11622E+01_JPRB, 0.10670E+01_JPRB ,&
&  0.95572E+00_JPRB, 0.89993E+00_JPRB, 0.84753E+00_JPRB, 0.87778E+00_JPRB, 0.95577E+00_JPRB, 0.10327E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10528E+01_JPRB, 0.10597E+01_JPRB, 0.11016E+01_JPRB, 0.11514E+01_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.11905E+01_JPRB, 0.11411E+01_JPRB, 0.10799E+01_JPRB, 0.10078E+01_JPRB, 0.94217E+00_JPRB, 0.88859E+00_JPRB, 0.82466E+00_JPRB ,&
&  0.85630E+00_JPRB, 0.77680E+00_JPRB, 0.84450E+00_JPRB, 0.73230E+00_JPRB, 0.70175E+00_JPRB, 0.69224E+00_JPRB, 0.67342E+00_JPRB ,&
&  0.66439E+00_JPRB, 0.67422E+00_JPRB, 0.69108E+00_JPRB, 0.71010E+00_JPRB, 0.71298E+00_JPRB, 0.71435E+00_JPRB, 0.72690E+00_JPRB ,&
&  0.74841E+00_JPRB, 0.77944E+00_JPRB, 0.81321E+00_JPRB, 0.82836E+00_JPRB, 0.83192E+00_JPRB, 0.82526E+00_JPRB, 0.82329E+00_JPRB ,&
&  0.77963E+00_JPRB, 0.76458E+00_JPRB, 0.76417E+00_JPRB, 0.75916E+00_JPRB, 0.75313E+00_JPRB, 0.73633E+00_JPRB, 0.74586E+00_JPRB ,&
&  0.72784E+00_JPRB, 0.70704E+00_JPRB, 0.82100E+00_JPRB, 0.84620E+00_JPRB, 0.88292E+00_JPRB, 0.92397E+00_JPRB, 0.96740E+00_JPRB ,&
&  0.10108E+01_JPRB, 0.10581E+01_JPRB, 0.11114E+01_JPRB, 0.11606E+01_JPRB, 0.12628E+01_JPRB, 0.14464E+01_JPRB, 0.16228E+01_JPRB ,&
&  0.18059E+01_JPRB, 0.14964E+01_JPRB, 0.16772E+01_JPRB, 0.18309E+01_JPRB, 0.22136E+01_JPRB, 0.26123E+01_JPRB, 0.27395E+01_JPRB ,&
&  0.30042E+01_JPRB, 0.29572E+01_JPRB, 0.28826E+01_JPRB, 0.28119E+01_JPRB, 0.29957E+01_JPRB, 0.32631E+01_JPRB, 0.35629E+01_JPRB ,&
&  0.39125E+01_JPRB, 0.43495E+01_JPRB, 0.50335E+01_JPRB, 0.49969E+01_JPRB, 0.48504E+01_JPRB, 0.46812E+01_JPRB, 0.44017E+01_JPRB ,&
&  0.40484E+01_JPRB, 0.36233E+01_JPRB, 0.30940E+01_JPRB, 0.25060E+01_JPRB, 0.21046E+01_JPRB, 0.17676E+01_JPRB, 0.14941E+01_JPRB ,&
&  0.13131E+01_JPRB, 0.12246E+01_JPRB, 0.12290E+01_JPRB, 0.12175E+01_JPRB, 0.11998E+01_JPRB, 0.11380E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.93734E+00_JPRB, 0.96391E+00_JPRB, 0.10261E+01_JPRB, 0.11578E+01_JPRB, 0.12156E+01_JPRB, 0.12194E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11886E+01_JPRB, 0.12594E+01_JPRB, 0.12389E+01_JPRB, 0.12220E+01_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.12399E+01_JPRB, 0.11530E+01_JPRB, 0.10207E+01_JPRB, 0.94387E+00_JPRB, 0.92937E+00_JPRB, 0.91175E+00_JPRB, 0.87284E+00_JPRB ,&
&  0.89607E+00_JPRB, 0.87880E+00_JPRB, 0.90065E+00_JPRB, 0.93291E+00_JPRB, 0.96070E+00_JPRB, 0.86553E+00_JPRB, 0.73649E+00_JPRB ,&
&  0.69976E+00_JPRB, 0.71752E+00_JPRB, 0.73834E+00_JPRB, 0.74634E+00_JPRB, 0.76087E+00_JPRB, 0.76010E+00_JPRB, 0.76482E+00_JPRB ,&
&  0.77185E+00_JPRB, 0.78563E+00_JPRB, 0.82443E+00_JPRB, 0.83871E+00_JPRB, 0.83794E+00_JPRB, 0.83786E+00_JPRB, 0.84490E+00_JPRB ,&
&  0.80037E+00_JPRB, 0.78159E+00_JPRB, 0.76801E+00_JPRB, 0.74038E+00_JPRB, 0.78022E+00_JPRB, 0.79322E+00_JPRB, 0.80120E+00_JPRB ,&
&  0.81113E+00_JPRB, 0.83203E+00_JPRB, 0.86691E+00_JPRB, 0.90547E+00_JPRB, 0.94599E+00_JPRB, 0.98662E+00_JPRB, 0.10242E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.10998E+01_JPRB, 0.11460E+01_JPRB, 0.12235E+01_JPRB, 0.14365E+01_JPRB, 0.15821E+01_JPRB, 0.18566E+01_JPRB ,&
&  0.19910E+01_JPRB, 0.21980E+01_JPRB, 0.24792E+01_JPRB, 0.24457E+01_JPRB, 0.26381E+01_JPRB, 0.29182E+01_JPRB, 0.28839E+01_JPRB ,&
&  0.29609E+01_JPRB, 0.29415E+01_JPRB, 0.31234E+01_JPRB, 0.34018E+01_JPRB, 0.37834E+01_JPRB, 0.41080E+01_JPRB, 0.43775E+01_JPRB ,&
&  0.48041E+01_JPRB, 0.55805E+01_JPRB, 0.58528E+01_JPRB, 0.55994E+01_JPRB, 0.53179E+01_JPRB, 0.50449E+01_JPRB, 0.46377E+01_JPRB ,&
&  0.42199E+01_JPRB, 0.37436E+01_JPRB, 0.31864E+01_JPRB, 0.26164E+01_JPRB, 0.21093E+01_JPRB, 0.18321E+01_JPRB, 0.15572E+01_JPRB ,&
&  0.13032E+01_JPRB, 0.11388E+01_JPRB, 0.11574E+01_JPRB, 0.11929E+01_JPRB, 0.11867E+01_JPRB, 0.10988E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.11819E+01_JPRB, 0.15457E+01_JPRB, 0.16741E+01_JPRB, 0.16901E+01_JPRB, 0.15930E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.12967E+01_JPRB, 0.12920E+01_JPRB, 0.13006E+01_JPRB, 0.12858E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.13046E+01_JPRB, 0.11689E+01_JPRB, 0.98713E+00_JPRB, 0.92595E+00_JPRB, 0.94096E+00_JPRB, 0.93260E+00_JPRB, 0.92091E+00_JPRB ,&
&  0.93062E+00_JPRB, 0.93541E+00_JPRB, 0.94312E+00_JPRB, 0.95809E+00_JPRB, 0.97233E+00_JPRB, 0.10056E+01_JPRB, 0.91820E+00_JPRB ,&
&  0.82423E+00_JPRB, 0.82000E+00_JPRB, 0.79963E+00_JPRB, 0.82377E+00_JPRB, 0.92095E+00_JPRB, 0.93066E+00_JPRB, 0.93240E+00_JPRB ,&
&  0.95024E+00_JPRB, 0.92828E+00_JPRB, 0.93019E+00_JPRB, 0.90831E+00_JPRB, 0.87725E+00_JPRB, 0.87647E+00_JPRB, 0.87686E+00_JPRB ,&
&  0.81156E+00_JPRB, 0.76637E+00_JPRB, 0.74045E+00_JPRB, 0.72958E+00_JPRB, 0.76888E+00_JPRB, 0.82521E+00_JPRB, 0.85288E+00_JPRB ,&
&  0.86737E+00_JPRB, 0.89055E+00_JPRB, 0.92567E+00_JPRB, 0.96997E+00_JPRB, 0.10168E+01_JPRB, 0.10575E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.11396E+01_JPRB, 0.11843E+01_JPRB, 0.12900E+01_JPRB, 0.14287E+01_JPRB, 0.17139E+01_JPRB, 0.21536E+01_JPRB ,&
&  0.23953E+01_JPRB, 0.25837E+01_JPRB, 0.29059E+01_JPRB, 0.33452E+01_JPRB, 0.36250E+01_JPRB, 0.35222E+01_JPRB, 0.33930E+01_JPRB ,&
&  0.34308E+01_JPRB, 0.34659E+01_JPRB, 0.40081E+01_JPRB, 0.44085E+01_JPRB, 0.46761E+01_JPRB, 0.49997E+01_JPRB, 0.51744E+01_JPRB ,&
&  0.55228E+01_JPRB, 0.58840E+01_JPRB, 0.58990E+01_JPRB, 0.56352E+01_JPRB, 0.53676E+01_JPRB, 0.51243E+01_JPRB, 0.46885E+01_JPRB ,&
&  0.42411E+01_JPRB, 0.36779E+01_JPRB, 0.29901E+01_JPRB, 0.24262E+01_JPRB, 0.19125E+01_JPRB, 0.17805E+01_JPRB, 0.16262E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.11874E+01_JPRB, 0.12308E+01_JPRB, 0.13625E+01_JPRB, 0.13610E+01_JPRB, 0.12401E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.14476E+01_JPRB, 0.17232E+01_JPRB, 0.19220E+01_JPRB, 0.19309E+01_JPRB, 0.17586E+01_JPRB, 0.12992E+01_JPRB ,&
&  0.12390E+01_JPRB, 0.13629E+01_JPRB, 0.13724E+01_JPRB, 0.13695E+01_JPRB, 0.13547E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.13986E+01_JPRB, 0.12211E+01_JPRB, 0.10545E+01_JPRB, 0.95879E+00_JPRB, 0.96365E+00_JPRB, 0.96892E+00_JPRB, 0.96870E+00_JPRB ,&
&  0.98605E+00_JPRB, 0.10018E+01_JPRB, 0.10149E+01_JPRB, 0.10252E+01_JPRB, 0.10292E+01_JPRB, 0.10301E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.92611E+00_JPRB, 0.91810E+00_JPRB, 0.88086E+00_JPRB, 0.93834E+00_JPRB, 0.12251E+01_JPRB, 0.12323E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.12492E+01_JPRB, 0.12390E+01_JPRB, 0.11925E+01_JPRB, 0.11296E+01_JPRB, 0.10748E+01_JPRB, 0.10556E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.89401E+00_JPRB, 0.82583E+00_JPRB, 0.80870E+00_JPRB, 0.81021E+00_JPRB, 0.84998E+00_JPRB, 0.87915E+00_JPRB, 0.94327E+00_JPRB ,&
&  0.97150E+00_JPRB, 0.10112E+01_JPRB, 0.10569E+01_JPRB, 0.11008E+01_JPRB, 0.11298E+01_JPRB, 0.11563E+01_JPRB, 0.11867E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.12678E+01_JPRB, 0.13405E+01_JPRB, 0.14723E+01_JPRB, 0.15322E+01_JPRB, 0.18851E+01_JPRB, 0.26686E+01_JPRB ,&
&  0.32287E+01_JPRB, 0.33391E+01_JPRB, 0.41026E+01_JPRB, 0.55278E+01_JPRB, 0.56762E+01_JPRB, 0.53254E+01_JPRB, 0.49332E+01_JPRB ,&
&  0.47459E+01_JPRB, 0.46340E+01_JPRB, 0.48299E+01_JPRB, 0.51224E+01_JPRB, 0.53364E+01_JPRB, 0.56832E+01_JPRB, 0.58252E+01_JPRB ,&
&  0.57733E+01_JPRB, 0.57006E+01_JPRB, 0.55342E+01_JPRB, 0.53146E+01_JPRB, 0.49567E+01_JPRB, 0.47153E+01_JPRB, 0.44098E+01_JPRB ,&
&  0.40255E+01_JPRB, 0.35101E+01_JPRB, 0.27203E+01_JPRB, 0.22381E+01_JPRB, 0.16758E+01_JPRB, 0.15411E+01_JPRB, 0.18845E+01_JPRB ,&
&  0.14565E+01_JPRB, 0.13321E+01_JPRB, 0.14545E+01_JPRB, 0.16761E+01_JPRB, 0.17348E+01_JPRB, 0.17147E+01_JPRB, 0.14885E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.14425E+01_JPRB, 0.20076E+01_JPRB, 0.21785E+01_JPRB, 0.21383E+01_JPRB, 0.19348E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.15705E+01_JPRB, 0.15012E+01_JPRB, 0.14902E+01_JPRB, 0.14850E+01_JPRB, 0.14679E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.14719E+01_JPRB, 0.12858E+01_JPRB, 0.11353E+01_JPRB, 0.10254E+01_JPRB, 0.10120E+01_JPRB, 0.10313E+01_JPRB, 0.10474E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.10815E+01_JPRB, 0.10897E+01_JPRB, 0.10982E+01_JPRB, 0.10977E+01_JPRB, 0.10689E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.10328E+01_JPRB, 0.10096E+01_JPRB, 0.10006E+01_JPRB, 0.12248E+01_JPRB, 0.13601E+01_JPRB, 0.15280E+01_JPRB ,&
&  0.16465E+01_JPRB, 0.17701E+01_JPRB, 0.17303E+01_JPRB, 0.16291E+01_JPRB, 0.15292E+01_JPRB, 0.14517E+01_JPRB, 0.13001E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.10627E+01_JPRB, 0.10313E+01_JPRB, 0.10781E+01_JPRB, 0.11492E+01_JPRB, 0.10461E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.12279E+01_JPRB, 0.12588E+01_JPRB, 0.12830E+01_JPRB, 0.13362E+01_JPRB, 0.14019E+01_JPRB, 0.14672E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.15517E+01_JPRB, 0.15482E+01_JPRB, 0.16391E+01_JPRB, 0.17518E+01_JPRB, 0.19846E+01_JPRB, 0.26033E+01_JPRB ,&
&  0.34463E+01_JPRB, 0.38514E+01_JPRB, 0.49186E+01_JPRB, 0.66864E+01_JPRB, 0.68239E+01_JPRB, 0.64446E+01_JPRB, 0.63315E+01_JPRB ,&
&  0.62228E+01_JPRB, 0.60209E+01_JPRB, 0.58910E+01_JPRB, 0.57465E+01_JPRB, 0.59170E+01_JPRB, 0.61888E+01_JPRB, 0.61733E+01_JPRB ,&
&  0.57560E+01_JPRB, 0.52843E+01_JPRB, 0.50394E+01_JPRB, 0.46910E+01_JPRB, 0.43768E+01_JPRB, 0.39921E+01_JPRB, 0.40126E+01_JPRB ,&
&  0.40517E+01_JPRB, 0.28478E+01_JPRB, 0.23054E+01_JPRB, 0.21764E+01_JPRB, 0.16348E+01_JPRB, 0.14432E+01_JPRB, 0.16320E+01_JPRB ,&
&  0.13792E+01_JPRB, 0.13040E+01_JPRB, 0.15966E+01_JPRB, 0.18985E+01_JPRB, 0.22420E+01_JPRB, 0.24332E+01_JPRB, 0.23592E+01_JPRB ,&
&  0.20402E+01_JPRB, 0.20316E+01_JPRB, 0.26349E+01_JPRB, 0.26730E+01_JPRB, 0.24314E+01_JPRB, 0.21502E+01_JPRB, 0.18938E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.17428E+01_JPRB, 0.16983E+01_JPRB, 0.16589E+01_JPRB, 0.16142E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.16259E+01_JPRB, 0.14364E+01_JPRB, 0.12570E+01_JPRB, 0.11493E+01_JPRB, 0.11348E+01_JPRB, 0.11503E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.11335E+01_JPRB, 0.11435E+01_JPRB, 0.11452E+01_JPRB, 0.11375E+01_JPRB, 0.11025E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.10981E+01_JPRB, 0.10385E+01_JPRB, 0.10740E+01_JPRB, 0.11600E+01_JPRB, 0.14396E+01_JPRB, 0.16957E+01_JPRB ,&
&  0.19678E+01_JPRB, 0.21494E+01_JPRB, 0.22550E+01_JPRB, 0.23525E+01_JPRB, 0.23368E+01_JPRB, 0.21898E+01_JPRB, 0.20217E+01_JPRB ,&
&  0.18830E+01_JPRB, 0.18270E+01_JPRB, 0.16146E+01_JPRB, 0.17075E+01_JPRB, 0.16682E+01_JPRB, 0.14931E+01_JPRB, 0.16344E+01_JPRB ,&
&  0.16264E+01_JPRB, 0.16543E+01_JPRB, 0.16573E+01_JPRB, 0.16908E+01_JPRB, 0.17321E+01_JPRB, 0.17390E+01_JPRB, 0.17210E+01_JPRB ,&
&  0.17053E+01_JPRB, 0.17119E+01_JPRB, 0.17530E+01_JPRB, 0.17515E+01_JPRB, 0.18513E+01_JPRB, 0.22141E+01_JPRB, 0.28799E+01_JPRB ,&
&  0.37666E+01_JPRB, 0.39148E+01_JPRB, 0.42150E+01_JPRB, 0.61822E+01_JPRB, 0.77141E+01_JPRB, 0.75185E+01_JPRB, 0.72514E+01_JPRB ,&
&  0.72467E+01_JPRB, 0.67813E+01_JPRB, 0.62954E+01_JPRB, 0.60666E+01_JPRB, 0.64772E+01_JPRB, 0.65278E+01_JPRB, 0.62712E+01_JPRB ,&
&  0.57400E+01_JPRB, 0.51880E+01_JPRB, 0.49145E+01_JPRB, 0.46106E+01_JPRB, 0.41106E+01_JPRB, 0.39058E+01_JPRB, 0.39694E+01_JPRB ,&
&  0.39107E+01_JPRB, 0.34211E+01_JPRB, 0.26081E+01_JPRB, 0.23432E+01_JPRB, 0.19188E+01_JPRB, 0.17558E+01_JPRB, 0.19514E+01_JPRB ,&
&  0.17464E+01_JPRB, 0.18007E+01_JPRB, 0.19828E+01_JPRB, 0.21321E+01_JPRB, 0.34749E+01_JPRB, 0.38725E+01_JPRB, 0.41803E+01_JPRB ,&
&  0.38102E+01_JPRB, 0.33549E+01_JPRB, 0.39055E+01_JPRB, 0.36695E+01_JPRB, 0.30924E+01_JPRB, 0.25362E+01_JPRB, 0.21376E+01_JPRB ,&
&  0.19965E+01_JPRB, 0.19959E+01_JPRB, 0.19236E+01_JPRB, 0.18526E+01_JPRB, 0.17959E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.17724E+01_JPRB, 0.15708E+01_JPRB, 0.13886E+01_JPRB, 0.12683E+01_JPRB, 0.12209E+01_JPRB, 0.12233E+01_JPRB, 0.12175E+01_JPRB ,&
&  0.11811E+01_JPRB, 0.11648E+01_JPRB, 0.11605E+01_JPRB, 0.11392E+01_JPRB, 0.11122E+01_JPRB, 0.10810E+01_JPRB, 0.10385E+01_JPRB ,&
&  0.10135E+01_JPRB, 0.10787E+01_JPRB, 0.10410E+01_JPRB, 0.10757E+01_JPRB, 0.12207E+01_JPRB, 0.16130E+01_JPRB, 0.20606E+01_JPRB ,&
&  0.24344E+01_JPRB, 0.26630E+01_JPRB, 0.28754E+01_JPRB, 0.31159E+01_JPRB, 0.33520E+01_JPRB, 0.34121E+01_JPRB, 0.33220E+01_JPRB ,&
&  0.35410E+01_JPRB, 0.29687E+01_JPRB, 0.29916E+01_JPRB, 0.27309E+01_JPRB, 0.25986E+01_JPRB, 0.24523E+01_JPRB, 0.23212E+01_JPRB ,&
&  0.22125E+01_JPRB, 0.21226E+01_JPRB, 0.20279E+01_JPRB, 0.19741E+01_JPRB, 0.19491E+01_JPRB, 0.19347E+01_JPRB, 0.19484E+01_JPRB ,&
&  0.19765E+01_JPRB, 0.19338E+01_JPRB, 0.18698E+01_JPRB, 0.18669E+01_JPRB, 0.23263E+01_JPRB, 0.32081E+01_JPRB, 0.39629E+01_JPRB ,&
&  0.39570E+01_JPRB, 0.39054E+01_JPRB, 0.66877E+01_JPRB, 0.75165E+01_JPRB, 0.73518E+01_JPRB, 0.80353E+01_JPRB, 0.81025E+01_JPRB ,&
&  0.74150E+01_JPRB, 0.64319E+01_JPRB, 0.56208E+01_JPRB, 0.55569E+01_JPRB, 0.68617E+01_JPRB, 0.67886E+01_JPRB, 0.64203E+01_JPRB ,&
&  0.57995E+01_JPRB, 0.53217E+01_JPRB, 0.47986E+01_JPRB, 0.44055E+01_JPRB, 0.41793E+01_JPRB, 0.43488E+01_JPRB, 0.40837E+01_JPRB ,&
&  0.38759E+01_JPRB, 0.42256E+01_JPRB, 0.34855E+01_JPRB, 0.27462E+01_JPRB, 0.24253E+01_JPRB, 0.23910E+01_JPRB, 0.24283E+01_JPRB ,&
&  0.23887E+01_JPRB, 0.24958E+01_JPRB, 0.26514E+01_JPRB, 0.34495E+01_JPRB, 0.54509E+01_JPRB, 0.56143E+01_JPRB, 0.53570E+01_JPRB ,&
&  0.50178E+01_JPRB, 0.57069E+01_JPRB, 0.50619E+01_JPRB, 0.43404E+01_JPRB, 0.35728E+01_JPRB, 0.29373E+01_JPRB, 0.24809E+01_JPRB ,&
&  0.22298E+01_JPRB, 0.21952E+01_JPRB, 0.21382E+01_JPRB, 0.20396E+01_JPRB, 0.19611E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.19637E+01_JPRB, 0.17186E+01_JPRB, 0.15846E+01_JPRB, 0.14557E+01_JPRB, 0.13826E+01_JPRB, 0.13233E+01_JPRB, 0.12264E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.11508E+01_JPRB, 0.11291E+01_JPRB, 0.11084E+01_JPRB, 0.10863E+01_JPRB, 0.10500E+01_JPRB, 0.10157E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.10631E+01_JPRB, 0.10807E+01_JPRB, 0.12341E+01_JPRB, 0.14419E+01_JPRB, 0.17705E+01_JPRB, 0.22701E+01_JPRB ,&
&  0.26957E+01_JPRB, 0.30097E+01_JPRB, 0.32235E+01_JPRB, 0.38273E+01_JPRB, 0.45048E+01_JPRB, 0.50987E+01_JPRB, 0.51681E+01_JPRB ,&
&  0.47471E+01_JPRB, 0.46988E+01_JPRB, 0.44602E+01_JPRB, 0.40390E+01_JPRB, 0.36876E+01_JPRB, 0.34096E+01_JPRB, 0.31021E+01_JPRB ,&
&  0.27789E+01_JPRB, 0.24773E+01_JPRB, 0.22563E+01_JPRB, 0.21354E+01_JPRB, 0.20678E+01_JPRB, 0.20849E+01_JPRB, 0.20695E+01_JPRB ,&
&  0.20105E+01_JPRB, 0.19952E+01_JPRB, 0.19572E+01_JPRB, 0.23139E+01_JPRB, 0.32109E+01_JPRB, 0.44730E+01_JPRB, 0.47483E+01_JPRB ,&
&  0.52076E+01_JPRB, 0.51647E+01_JPRB, 0.62137E+01_JPRB, 0.72075E+01_JPRB, 0.60738E+01_JPRB, 0.69029E+01_JPRB, 0.73186E+01_JPRB ,&
&  0.65577E+01_JPRB, 0.60552E+01_JPRB, 0.64712E+01_JPRB, 0.67196E+01_JPRB, 0.67302E+01_JPRB, 0.78684E+01_JPRB, 0.73479E+01_JPRB ,&
&  0.62829E+01_JPRB, 0.55172E+01_JPRB, 0.47153E+01_JPRB, 0.45635E+01_JPRB, 0.40129E+01_JPRB, 0.39643E+01_JPRB, 0.33069E+01_JPRB ,&
&  0.35587E+01_JPRB, 0.30931E+01_JPRB, 0.32935E+01_JPRB, 0.29561E+01_JPRB, 0.29841E+01_JPRB, 0.28555E+01_JPRB, 0.27739E+01_JPRB ,&
&  0.29289E+01_JPRB, 0.31878E+01_JPRB, 0.37940E+01_JPRB, 0.56076E+01_JPRB, 0.77281E+01_JPRB, 0.63657E+01_JPRB, 0.70759E+01_JPRB ,&
&  0.82824E+01_JPRB, 0.78411E+01_JPRB, 0.56572E+01_JPRB, 0.47654E+01_JPRB, 0.38741E+01_JPRB, 0.32515E+01_JPRB, 0.28976E+01_JPRB ,&
&  0.25955E+01_JPRB, 0.24530E+01_JPRB, 0.23668E+01_JPRB, 0.22735E+01_JPRB, 0.21722E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.20522E+01_JPRB, 0.18145E+01_JPRB, 0.17002E+01_JPRB, 0.16428E+01_JPRB, 0.15333E+01_JPRB, 0.13474E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.11745E+01_JPRB, 0.11019E+01_JPRB, 0.10554E+01_JPRB, 0.10443E+01_JPRB, 0.10444E+01_JPRB, 0.10244E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.11968E+01_JPRB, 0.12878E+01_JPRB, 0.14196E+01_JPRB, 0.16825E+01_JPRB, 0.19834E+01_JPRB, 0.22374E+01_JPRB ,&
&  0.31537E+01_JPRB, 0.34251E+01_JPRB, 0.34714E+01_JPRB, 0.42001E+01_JPRB, 0.52926E+01_JPRB, 0.63773E+01_JPRB, 0.67049E+01_JPRB ,&
&  0.70134E+01_JPRB, 0.66407E+01_JPRB, 0.53566E+01_JPRB, 0.44346E+01_JPRB, 0.38056E+01_JPRB, 0.34280E+01_JPRB, 0.30336E+01_JPRB ,&
&  0.25795E+01_JPRB, 0.21998E+01_JPRB, 0.20201E+01_JPRB, 0.19178E+01_JPRB, 0.18848E+01_JPRB, 0.17861E+01_JPRB, 0.17933E+01_JPRB ,&
&  0.18680E+01_JPRB, 0.19868E+01_JPRB, 0.21182E+01_JPRB, 0.27156E+01_JPRB, 0.40293E+01_JPRB, 0.52378E+01_JPRB, 0.55692E+01_JPRB ,&
&  0.52423E+01_JPRB, 0.50223E+01_JPRB, 0.60379E+01_JPRB, 0.66723E+01_JPRB, 0.69157E+01_JPRB, 0.66642E+01_JPRB, 0.61720E+01_JPRB ,&
&  0.49532E+01_JPRB, 0.47715E+01_JPRB, 0.53022E+01_JPRB, 0.60136E+01_JPRB, 0.70295E+01_JPRB, 0.86638E+01_JPRB, 0.83477E+01_JPRB ,&
&  0.69632E+01_JPRB, 0.59370E+01_JPRB, 0.51357E+01_JPRB, 0.43483E+01_JPRB, 0.44764E+01_JPRB, 0.34589E+01_JPRB, 0.40978E+01_JPRB ,&
&  0.42918E+01_JPRB, 0.41398E+01_JPRB, 0.34435E+01_JPRB, 0.28980E+01_JPRB, 0.26194E+01_JPRB, 0.31368E+01_JPRB, 0.32590E+01_JPRB ,&
&  0.38120E+01_JPRB, 0.43355E+01_JPRB, 0.76862E+01_JPRB, 0.91002E+01_JPRB, 0.90448E+01_JPRB, 0.78949E+01_JPRB, 0.96181E+01_JPRB ,&
&  0.88579E+01_JPRB, 0.76313E+01_JPRB, 0.60591E+01_JPRB, 0.49633E+01_JPRB, 0.40751E+01_JPRB, 0.35473E+01_JPRB, 0.32155E+01_JPRB ,&
&  0.28739E+01_JPRB, 0.26971E+01_JPRB, 0.25681E+01_JPRB, 0.24418E+01_JPRB, 0.22902E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.19778E+01_JPRB, 0.18454E+01_JPRB, 0.17956E+01_JPRB, 0.16495E+01_JPRB, 0.14265E+01_JPRB, 0.12751E+01_JPRB, 0.12328E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.11453E+01_JPRB, 0.10784E+01_JPRB, 0.10366E+01_JPRB, 0.10460E+01_JPRB, 0.10639E+01_JPRB, 0.10908E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.14861E+01_JPRB, 0.15984E+01_JPRB, 0.17773E+01_JPRB, 0.19877E+01_JPRB, 0.22381E+01_JPRB, 0.25705E+01_JPRB ,&
&  0.35988E+01_JPRB, 0.33861E+01_JPRB, 0.30322E+01_JPRB, 0.36568E+01_JPRB, 0.55469E+01_JPRB, 0.63290E+01_JPRB, 0.67382E+01_JPRB ,&
&  0.65676E+01_JPRB, 0.55658E+01_JPRB, 0.45228E+01_JPRB, 0.36586E+01_JPRB, 0.31015E+01_JPRB, 0.27381E+01_JPRB, 0.22806E+01_JPRB ,&
&  0.19712E+01_JPRB, 0.18331E+01_JPRB, 0.18137E+01_JPRB, 0.17974E+01_JPRB, 0.17556E+01_JPRB, 0.17460E+01_JPRB, 0.18061E+01_JPRB ,&
&  0.18974E+01_JPRB, 0.20198E+01_JPRB, 0.22245E+01_JPRB, 0.29711E+01_JPRB, 0.42741E+01_JPRB, 0.53695E+01_JPRB, 0.62336E+01_JPRB ,&
&  0.53927E+01_JPRB, 0.54269E+01_JPRB, 0.62071E+01_JPRB, 0.77132E+01_JPRB, 0.65992E+01_JPRB, 0.62653E+01_JPRB, 0.57501E+01_JPRB ,&
&  0.49435E+01_JPRB, 0.52602E+01_JPRB, 0.63917E+01_JPRB, 0.70916E+01_JPRB, 0.65205E+01_JPRB, 0.74150E+01_JPRB, 0.89448E+01_JPRB ,&
&  0.67495E+01_JPRB, 0.58233E+01_JPRB, 0.53631E+01_JPRB, 0.53917E+01_JPRB, 0.35910E+01_JPRB, 0.25675E+01_JPRB, 0.39615E+01_JPRB ,&
&  0.40424E+01_JPRB, 0.22954E+01_JPRB, 0.16979E+01_JPRB, 0.20570E+01_JPRB, 0.15912E+01_JPRB, 0.20503E+01_JPRB, 0.35410E+01_JPRB ,&
&  0.47911E+01_JPRB, 0.59546E+01_JPRB, 0.12323E+02_JPRB, 0.11158E+02_JPRB, 0.10905E+02_JPRB, 0.10330E+02_JPRB, 0.10874E+02_JPRB ,&
&  0.95907E+01_JPRB, 0.61687E+01_JPRB, 0.55582E+01_JPRB, 0.48365E+01_JPRB, 0.39150E+01_JPRB, 0.32529E+01_JPRB, 0.27939E+01_JPRB ,&
&  0.24412E+01_JPRB, 0.22439E+01_JPRB, 0.21400E+01_JPRB, 0.20978E+01_JPRB, 0.20995E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.16798E+01_JPRB, 0.16279E+01_JPRB, 0.15532E+01_JPRB, 0.13875E+01_JPRB, 0.12636E+01_JPRB, 0.12116E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.10815E+01_JPRB, 0.10451E+01_JPRB, 0.10593E+01_JPRB, 0.10832E+01_JPRB, 0.11122E+01_JPRB, 0.11704E+01_JPRB ,&
&  0.12766E+01_JPRB, 0.15001E+01_JPRB, 0.19860E+01_JPRB, 0.23713E+01_JPRB, 0.26285E+01_JPRB, 0.23862E+01_JPRB, 0.30619E+01_JPRB ,&
&  0.36079E+01_JPRB, 0.29728E+01_JPRB, 0.27770E+01_JPRB, 0.34609E+01_JPRB, 0.45581E+01_JPRB, 0.51147E+01_JPRB, 0.52522E+01_JPRB ,&
&  0.47054E+01_JPRB, 0.40161E+01_JPRB, 0.35058E+01_JPRB, 0.31742E+01_JPRB, 0.27117E+01_JPRB, 0.21624E+01_JPRB, 0.18906E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.17965E+01_JPRB, 0.18486E+01_JPRB, 0.18057E+01_JPRB, 0.18243E+01_JPRB, 0.18413E+01_JPRB, 0.19152E+01_JPRB ,&
&  0.19942E+01_JPRB, 0.21169E+01_JPRB, 0.24350E+01_JPRB, 0.34661E+01_JPRB, 0.52361E+01_JPRB, 0.66327E+01_JPRB, 0.57108E+01_JPRB ,&
&  0.47058E+01_JPRB, 0.46338E+01_JPRB, 0.65013E+01_JPRB, 0.71234E+01_JPRB, 0.61148E+01_JPRB, 0.61806E+01_JPRB, 0.56863E+01_JPRB ,&
&  0.59350E+01_JPRB, 0.64188E+01_JPRB, 0.62210E+01_JPRB, 0.68455E+01_JPRB, 0.72655E+01_JPRB, 0.70280E+01_JPRB, 0.74268E+01_JPRB ,&
&  0.66966E+01_JPRB, 0.55321E+01_JPRB, 0.44506E+01_JPRB, 0.38822E+01_JPRB, 0.44645E+01_JPRB, 0.33438E+01_JPRB, 0.21927E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.14044E+01_JPRB, 0.11405E+01_JPRB, 0.11585E+01_JPRB, 0.11187E+01_JPRB, 0.18602E+01_JPRB, 0.34512E+01_JPRB ,&
&  0.70813E+01_JPRB, 0.99333E+01_JPRB, 0.13291E+02_JPRB, 0.13398E+02_JPRB, 0.11796E+02_JPRB, 0.11264E+02_JPRB, 0.12053E+02_JPRB ,&
&  0.74451E+01_JPRB, 0.53808E+01_JPRB, 0.50415E+01_JPRB, 0.40663E+01_JPRB, 0.30077E+01_JPRB, 0.24066E+01_JPRB, 0.20633E+01_JPRB ,&
&  0.18831E+01_JPRB, 0.17780E+01_JPRB, 0.17081E+01_JPRB, 0.17072E+01_JPRB, 0.17314E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.14193E+01_JPRB, 0.13821E+01_JPRB, 0.13498E+01_JPRB, 0.12601E+01_JPRB, 0.11752E+01_JPRB, 0.11711E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.10752E+01_JPRB, 0.10774E+01_JPRB, 0.11114E+01_JPRB, 0.11518E+01_JPRB, 0.12213E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.17002E+01_JPRB, 0.23552E+01_JPRB, 0.30340E+01_JPRB, 0.35112E+01_JPRB, 0.29239E+01_JPRB, 0.34759E+01_JPRB ,&
&  0.33785E+01_JPRB, 0.27772E+01_JPRB, 0.28221E+01_JPRB, 0.34068E+01_JPRB, 0.41734E+01_JPRB, 0.39974E+01_JPRB, 0.36134E+01_JPRB ,&
&  0.32907E+01_JPRB, 0.30191E+01_JPRB, 0.27843E+01_JPRB, 0.25646E+01_JPRB, 0.21651E+01_JPRB, 0.20265E+01_JPRB, 0.20191E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.20478E+01_JPRB, 0.20079E+01_JPRB, 0.19735E+01_JPRB, 0.19770E+01_JPRB, 0.20065E+01_JPRB, 0.20765E+01_JPRB ,&
&  0.21724E+01_JPRB, 0.24034E+01_JPRB, 0.30747E+01_JPRB, 0.46737E+01_JPRB, 0.57377E+01_JPRB, 0.56970E+01_JPRB, 0.54646E+01_JPRB ,&
&  0.52946E+01_JPRB, 0.57452E+01_JPRB, 0.61852E+01_JPRB, 0.72270E+01_JPRB, 0.60515E+01_JPRB, 0.58334E+01_JPRB, 0.55586E+01_JPRB ,&
&  0.57018E+01_JPRB, 0.64962E+01_JPRB, 0.61973E+01_JPRB, 0.67179E+01_JPRB, 0.86006E+01_JPRB, 0.83168E+01_JPRB, 0.68532E+01_JPRB ,&
&  0.64801E+01_JPRB, 0.53670E+01_JPRB, 0.48073E+01_JPRB, 0.44624E+01_JPRB, 0.64282E+01_JPRB, 0.65960E+01_JPRB, 0.24037E+01_JPRB ,&
&  0.14692E+01_JPRB, 0.11720E+01_JPRB, 0.89107E+00_JPRB, 0.87975E+00_JPRB, 0.11018E+01_JPRB, 0.19230E+01_JPRB, 0.55324E+01_JPRB ,&
&  0.65707E+01_JPRB, 0.10738E+02_JPRB, 0.12134E+02_JPRB, 0.12877E+02_JPRB, 0.11576E+02_JPRB, 0.11026E+02_JPRB, 0.81239E+01_JPRB ,&
&  0.63258E+01_JPRB, 0.48553E+01_JPRB, 0.36936E+01_JPRB, 0.28504E+01_JPRB, 0.22238E+01_JPRB, 0.18867E+01_JPRB, 0.17272E+01_JPRB ,&
&  0.16575E+01_JPRB, 0.16208E+01_JPRB, 0.15626E+01_JPRB, 0.15029E+01_JPRB, 0.14322E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.11885E+01_JPRB, 0.12111E+01_JPRB, 0.12412E+01_JPRB, 0.12334E+01_JPRB, 0.12210E+01_JPRB, 0.12401E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.11499E+01_JPRB, 0.11683E+01_JPRB, 0.12097E+01_JPRB, 0.12744E+01_JPRB, 0.13666E+01_JPRB, 0.14720E+01_JPRB ,&
&  0.16820E+01_JPRB, 0.21541E+01_JPRB, 0.28937E+01_JPRB, 0.38725E+01_JPRB, 0.42191E+01_JPRB, 0.38454E+01_JPRB, 0.38698E+01_JPRB ,&
&  0.33489E+01_JPRB, 0.28339E+01_JPRB, 0.27706E+01_JPRB, 0.32029E+01_JPRB, 0.32584E+01_JPRB, 0.30062E+01_JPRB, 0.25500E+01_JPRB ,&
&  0.25390E+01_JPRB, 0.25100E+01_JPRB, 0.24256E+01_JPRB, 0.23677E+01_JPRB, 0.23597E+01_JPRB, 0.23853E+01_JPRB, 0.24180E+01_JPRB ,&
&  0.24280E+01_JPRB, 0.23591E+01_JPRB, 0.22696E+01_JPRB, 0.22123E+01_JPRB, 0.22029E+01_JPRB, 0.22528E+01_JPRB, 0.23588E+01_JPRB ,&
&  0.25621E+01_JPRB, 0.30694E+01_JPRB, 0.42191E+01_JPRB, 0.56461E+01_JPRB, 0.53698E+01_JPRB, 0.59513E+01_JPRB, 0.56396E+01_JPRB ,&
&  0.54254E+01_JPRB, 0.56925E+01_JPRB, 0.56681E+01_JPRB, 0.63414E+01_JPRB, 0.60939E+01_JPRB, 0.59506E+01_JPRB, 0.55239E+01_JPRB ,&
&  0.61848E+01_JPRB, 0.64825E+01_JPRB, 0.71086E+01_JPRB, 0.75412E+01_JPRB, 0.86665E+01_JPRB, 0.10255E+02_JPRB, 0.68711E+01_JPRB ,&
&  0.56214E+01_JPRB, 0.56821E+01_JPRB, 0.52200E+01_JPRB, 0.48916E+01_JPRB, 0.58443E+01_JPRB, 0.77738E+01_JPRB, 0.58208E+01_JPRB ,&
&  0.19241E+01_JPRB, 0.12427E+01_JPRB, 0.11224E+01_JPRB, 0.11137E+01_JPRB, 0.10058E+01_JPRB, 0.19812E+01_JPRB, 0.82937E+01_JPRB ,&
&  0.69252E+01_JPRB, 0.92491E+01_JPRB, 0.10451E+02_JPRB, 0.10848E+02_JPRB, 0.10360E+02_JPRB, 0.75136E+01_JPRB, 0.49970E+01_JPRB ,&
&  0.31745E+01_JPRB, 0.23241E+01_JPRB, 0.20097E+01_JPRB, 0.17655E+01_JPRB, 0.14786E+01_JPRB, 0.13379E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.12903E+01_JPRB, 0.12516E+01_JPRB, 0.12191E+01_JPRB, 0.11757E+01_JPRB, 0.11734E+01_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.11288E+01_JPRB, 0.12128E+01_JPRB, 0.12879E+01_JPRB, 0.13509E+01_JPRB, 0.13804E+01_JPRB, 0.13665E+01_JPRB, 0.13363E+01_JPRB ,&
&  0.13222E+01_JPRB, 0.13087E+01_JPRB, 0.13329E+01_JPRB, 0.13795E+01_JPRB, 0.14861E+01_JPRB, 0.16442E+01_JPRB, 0.18973E+01_JPRB ,&
&  0.23060E+01_JPRB, 0.28983E+01_JPRB, 0.36552E+01_JPRB, 0.44056E+01_JPRB, 0.45320E+01_JPRB, 0.39321E+01_JPRB, 0.37988E+01_JPRB ,&
&  0.34244E+01_JPRB, 0.29594E+01_JPRB, 0.25883E+01_JPRB, 0.28596E+01_JPRB, 0.30690E+01_JPRB, 0.30517E+01_JPRB, 0.26768E+01_JPRB ,&
&  0.26883E+01_JPRB, 0.26982E+01_JPRB, 0.27011E+01_JPRB, 0.27302E+01_JPRB, 0.27601E+01_JPRB, 0.27886E+01_JPRB, 0.27647E+01_JPRB ,&
&  0.26592E+01_JPRB, 0.25458E+01_JPRB, 0.24938E+01_JPRB, 0.25052E+01_JPRB, 0.25707E+01_JPRB, 0.26659E+01_JPRB, 0.28683E+01_JPRB ,&
&  0.32976E+01_JPRB, 0.40830E+01_JPRB, 0.51308E+01_JPRB, 0.55776E+01_JPRB, 0.55549E+01_JPRB, 0.55054E+01_JPRB, 0.53850E+01_JPRB ,&
&  0.53353E+01_JPRB, 0.56083E+01_JPRB, 0.57295E+01_JPRB, 0.57980E+01_JPRB, 0.58144E+01_JPRB, 0.60252E+01_JPRB, 0.58257E+01_JPRB ,&
&  0.66952E+01_JPRB, 0.73667E+01_JPRB, 0.72711E+01_JPRB, 0.72884E+01_JPRB, 0.76262E+01_JPRB, 0.84999E+01_JPRB, 0.78533E+01_JPRB ,&
&  0.63594E+01_JPRB, 0.51982E+01_JPRB, 0.48443E+01_JPRB, 0.50852E+01_JPRB, 0.52122E+01_JPRB, 0.59403E+01_JPRB, 0.71859E+01_JPRB ,&
&  0.63733E+01_JPRB, 0.25293E+01_JPRB, 0.16446E+01_JPRB, 0.23024E+01_JPRB, 0.15207E+01_JPRB, 0.20131E+01_JPRB, 0.39204E+01_JPRB ,&
&  0.49242E+01_JPRB, 0.59901E+01_JPRB, 0.69280E+01_JPRB, 0.74318E+01_JPRB, 0.61971E+01_JPRB, 0.42622E+01_JPRB, 0.27986E+01_JPRB ,&
&  0.19312E+01_JPRB, 0.16043E+01_JPRB, 0.15255E+01_JPRB, 0.13169E+01_JPRB, 0.11719E+01_JPRB, 0.11380E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.10465E+01_JPRB, 0.10685E+01_JPRB, 0.10708E+01_JPRB, 0.10681E+01_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.12254E+01_JPRB, 0.13157E+01_JPRB, 0.14357E+01_JPRB, 0.15026E+01_JPRB, 0.15132E+01_JPRB, 0.15122E+01_JPRB, 0.15394E+01_JPRB ,&
&  0.15725E+01_JPRB, 0.16133E+01_JPRB, 0.16905E+01_JPRB, 0.18170E+01_JPRB, 0.20368E+01_JPRB, 0.23098E+01_JPRB, 0.26704E+01_JPRB ,&
&  0.31285E+01_JPRB, 0.36520E+01_JPRB, 0.42223E+01_JPRB, 0.47875E+01_JPRB, 0.51597E+01_JPRB, 0.54219E+01_JPRB, 0.35587E+01_JPRB ,&
&  0.29657E+01_JPRB, 0.33047E+01_JPRB, 0.27438E+01_JPRB, 0.30040E+01_JPRB, 0.31666E+01_JPRB, 0.32136E+01_JPRB, 0.29653E+01_JPRB ,&
&  0.29264E+01_JPRB, 0.29222E+01_JPRB, 0.29348E+01_JPRB, 0.29317E+01_JPRB, 0.29276E+01_JPRB, 0.29152E+01_JPRB, 0.28489E+01_JPRB ,&
&  0.27754E+01_JPRB, 0.27557E+01_JPRB, 0.28081E+01_JPRB, 0.29040E+01_JPRB, 0.30427E+01_JPRB, 0.32501E+01_JPRB, 0.35954E+01_JPRB ,&
&  0.40931E+01_JPRB, 0.47011E+01_JPRB, 0.52626E+01_JPRB, 0.51691E+01_JPRB, 0.51647E+01_JPRB, 0.51200E+01_JPRB, 0.52748E+01_JPRB ,&
&  0.51705E+01_JPRB, 0.49042E+01_JPRB, 0.53288E+01_JPRB, 0.57784E+01_JPRB, 0.58455E+01_JPRB, 0.58674E+01_JPRB, 0.59156E+01_JPRB ,&
&  0.69736E+01_JPRB, 0.75038E+01_JPRB, 0.76325E+01_JPRB, 0.67039E+01_JPRB, 0.64787E+01_JPRB, 0.70448E+01_JPRB, 0.72746E+01_JPRB ,&
&  0.64655E+01_JPRB, 0.55658E+01_JPRB, 0.50403E+01_JPRB, 0.46706E+01_JPRB, 0.43807E+01_JPRB, 0.46361E+01_JPRB, 0.50796E+01_JPRB ,&
&  0.53929E+01_JPRB, 0.60563E+01_JPRB, 0.48413E+01_JPRB, 0.29184E+01_JPRB, 0.21892E+01_JPRB, 0.26487E+01_JPRB, 0.33895E+01_JPRB ,&
&  0.37038E+01_JPRB, 0.39496E+01_JPRB, 0.39259E+01_JPRB, 0.49901E+01_JPRB, 0.30483E+01_JPRB, 0.21451E+01_JPRB, 0.15639E+01_JPRB ,&
&  0.13350E+01_JPRB, 0.12965E+01_JPRB, 0.12870E+01_JPRB, 0.11750E+01_JPRB, 0.10839E+01_JPRB, 0.11057E+01_JPRB, 0.10436E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.11052E+01_JPRB, 0.11234E+01_JPRB, 0.11403E+01_JPRB, 0.11823E+01_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.14221E+01_JPRB, 0.14957E+01_JPRB, 0.15905E+01_JPRB, 0.16458E+01_JPRB, 0.17163E+01_JPRB, 0.17779E+01_JPRB, 0.18804E+01_JPRB ,&
&  0.19989E+01_JPRB, 0.21412E+01_JPRB, 0.23021E+01_JPRB, 0.25107E+01_JPRB, 0.27871E+01_JPRB, 0.30863E+01_JPRB, 0.34290E+01_JPRB ,&
&  0.37669E+01_JPRB, 0.40810E+01_JPRB, 0.43914E+01_JPRB, 0.47240E+01_JPRB, 0.49768E+01_JPRB, 0.52573E+01_JPRB, 0.45008E+01_JPRB ,&
&  0.27244E+01_JPRB, 0.38717E+01_JPRB, 0.28683E+01_JPRB, 0.32002E+01_JPRB, 0.32934E+01_JPRB, 0.34588E+01_JPRB, 0.31384E+01_JPRB ,&
&  0.29593E+01_JPRB, 0.29223E+01_JPRB, 0.28933E+01_JPRB, 0.29100E+01_JPRB, 0.29217E+01_JPRB, 0.29233E+01_JPRB, 0.29145E+01_JPRB ,&
&  0.29447E+01_JPRB, 0.30338E+01_JPRB, 0.31730E+01_JPRB, 0.33295E+01_JPRB, 0.35526E+01_JPRB, 0.38457E+01_JPRB, 0.41325E+01_JPRB ,&
&  0.44438E+01_JPRB, 0.47683E+01_JPRB, 0.48947E+01_JPRB, 0.48056E+01_JPRB, 0.47838E+01_JPRB, 0.49478E+01_JPRB, 0.52820E+01_JPRB ,&
&  0.52392E+01_JPRB, 0.50837E+01_JPRB, 0.53623E+01_JPRB, 0.54752E+01_JPRB, 0.54173E+01_JPRB, 0.58506E+01_JPRB, 0.58692E+01_JPRB ,&
&  0.67808E+01_JPRB, 0.67564E+01_JPRB, 0.74709E+01_JPRB, 0.62546E+01_JPRB, 0.56070E+01_JPRB, 0.58642E+01_JPRB, 0.55573E+01_JPRB ,&
&  0.48663E+01_JPRB, 0.46865E+01_JPRB, 0.42386E+01_JPRB, 0.36424E+01_JPRB, 0.36237E+01_JPRB, 0.37348E+01_JPRB, 0.41176E+01_JPRB ,&
&  0.44827E+01_JPRB, 0.45933E+01_JPRB, 0.34947E+01_JPRB, 0.23217E+01_JPRB, 0.21301E+01_JPRB, 0.24964E+01_JPRB, 0.28345E+01_JPRB ,&
&  0.28705E+01_JPRB, 0.29292E+01_JPRB, 0.30544E+01_JPRB, 0.26794E+01_JPRB, 0.16744E+01_JPRB, 0.13122E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11966E+01_JPRB, 0.11967E+01_JPRB, 0.11609E+01_JPRB, 0.10759E+01_JPRB, 0.11233E+01_JPRB, 0.11272E+01_JPRB ,&
&  0.12007E+01_JPRB, 0.12384E+01_JPRB, 0.12931E+01_JPRB, 0.13408E+01_JPRB, 0.13758E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.16429E+01_JPRB, 0.17489E+01_JPRB, 0.19183E+01_JPRB, 0.20710E+01_JPRB, 0.21935E+01_JPRB, 0.22942E+01_JPRB, 0.23744E+01_JPRB ,&
&  0.24416E+01_JPRB, 0.25365E+01_JPRB, 0.27426E+01_JPRB, 0.29569E+01_JPRB, 0.32257E+01_JPRB, 0.35246E+01_JPRB, 0.38139E+01_JPRB ,&
&  0.40666E+01_JPRB, 0.43092E+01_JPRB, 0.45522E+01_JPRB, 0.48777E+01_JPRB, 0.52463E+01_JPRB, 0.58616E+01_JPRB, 0.55929E+01_JPRB ,&
&  0.39939E+01_JPRB, 0.30511E+01_JPRB, 0.30073E+01_JPRB, 0.30783E+01_JPRB, 0.30142E+01_JPRB, 0.32956E+01_JPRB, 0.33759E+01_JPRB ,&
&  0.30717E+01_JPRB, 0.29487E+01_JPRB, 0.29183E+01_JPRB, 0.29220E+01_JPRB, 0.28772E+01_JPRB, 0.29232E+01_JPRB, 0.29809E+01_JPRB ,&
&  0.31034E+01_JPRB, 0.32681E+01_JPRB, 0.34392E+01_JPRB, 0.36437E+01_JPRB, 0.38647E+01_JPRB, 0.41069E+01_JPRB, 0.43201E+01_JPRB ,&
&  0.44943E+01_JPRB, 0.46569E+01_JPRB, 0.46761E+01_JPRB, 0.45426E+01_JPRB, 0.46205E+01_JPRB, 0.48102E+01_JPRB, 0.49480E+01_JPRB ,&
&  0.50374E+01_JPRB, 0.52888E+01_JPRB, 0.55228E+01_JPRB, 0.55363E+01_JPRB, 0.52960E+01_JPRB, 0.57748E+01_JPRB, 0.57985E+01_JPRB ,&
&  0.61787E+01_JPRB, 0.61251E+01_JPRB, 0.68633E+01_JPRB, 0.57601E+01_JPRB, 0.51568E+01_JPRB, 0.49304E+01_JPRB, 0.44932E+01_JPRB ,&
&  0.40259E+01_JPRB, 0.38665E+01_JPRB, 0.35415E+01_JPRB, 0.27862E+01_JPRB, 0.29716E+01_JPRB, 0.28635E+01_JPRB, 0.36752E+01_JPRB ,&
&  0.42089E+01_JPRB, 0.46171E+01_JPRB, 0.29245E+01_JPRB, 0.22985E+01_JPRB, 0.20850E+01_JPRB, 0.22920E+01_JPRB, 0.23709E+01_JPRB ,&
&  0.22923E+01_JPRB, 0.20605E+01_JPRB, 0.19063E+01_JPRB, 0.17267E+01_JPRB, 0.12018E+01_JPRB, 0.10561E+01_JPRB, 0.10204E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.11447E+01_JPRB, 0.11539E+01_JPRB, 0.11334E+01_JPRB, 0.11431E+01_JPRB, 0.12276E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.14162E+01_JPRB, 0.14793E+01_JPRB, 0.15119E+01_JPRB, 0.15557E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.16640E+01_JPRB, 0.17744E+01_JPRB, 0.19526E+01_JPRB, 0.21171E+01_JPRB, 0.22510E+01_JPRB, 0.23306E+01_JPRB, 0.23697E+01_JPRB ,&
&  0.24446E+01_JPRB, 0.25644E+01_JPRB, 0.27451E+01_JPRB, 0.29163E+01_JPRB, 0.31184E+01_JPRB, 0.33283E+01_JPRB, 0.35387E+01_JPRB ,&
&  0.37338E+01_JPRB, 0.39101E+01_JPRB, 0.41231E+01_JPRB, 0.43882E+01_JPRB, 0.46922E+01_JPRB, 0.51690E+01_JPRB, 0.51887E+01_JPRB ,&
&  0.40559E+01_JPRB, 0.24685E+01_JPRB, 0.27104E+01_JPRB, 0.27432E+01_JPRB, 0.28090E+01_JPRB, 0.31480E+01_JPRB, 0.32343E+01_JPRB ,&
&  0.28256E+01_JPRB, 0.27711E+01_JPRB, 0.28576E+01_JPRB, 0.28567E+01_JPRB, 0.27947E+01_JPRB, 0.29320E+01_JPRB, 0.30504E+01_JPRB ,&
&  0.32012E+01_JPRB, 0.33441E+01_JPRB, 0.34839E+01_JPRB, 0.36405E+01_JPRB, 0.37877E+01_JPRB, 0.39552E+01_JPRB, 0.41430E+01_JPRB ,&
&  0.43388E+01_JPRB, 0.44861E+01_JPRB, 0.44693E+01_JPRB, 0.44358E+01_JPRB, 0.44622E+01_JPRB, 0.45678E+01_JPRB, 0.46681E+01_JPRB ,&
&  0.47977E+01_JPRB, 0.49228E+01_JPRB, 0.52986E+01_JPRB, 0.56382E+01_JPRB, 0.57773E+01_JPRB, 0.54936E+01_JPRB, 0.55948E+01_JPRB ,&
&  0.56105E+01_JPRB, 0.53662E+01_JPRB, 0.56113E+01_JPRB, 0.60680E+01_JPRB, 0.43592E+01_JPRB, 0.42536E+01_JPRB, 0.36010E+01_JPRB ,&
&  0.36870E+01_JPRB, 0.34344E+01_JPRB, 0.30720E+01_JPRB, 0.26624E+01_JPRB, 0.24322E+01_JPRB, 0.23880E+01_JPRB, 0.30260E+01_JPRB ,&
&  0.37823E+01_JPRB, 0.33715E+01_JPRB, 0.24185E+01_JPRB, 0.20763E+01_JPRB, 0.20551E+01_JPRB, 0.24937E+01_JPRB, 0.24739E+01_JPRB ,&
&  0.21232E+01_JPRB, 0.18318E+01_JPRB, 0.17172E+01_JPRB, 0.20752E+01_JPRB, 0.13645E+01_JPRB, 0.99476E+00_JPRB, 0.92983E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.10803E+01_JPRB, 0.11447E+01_JPRB, 0.11908E+01_JPRB, 0.12334E+01_JPRB, 0.13008E+01_JPRB, 0.13292E+01_JPRB ,&
&  0.14117E+01_JPRB, 0.14311E+01_JPRB, 0.15078E+01_JPRB, 0.15471E+01_JPRB, 0.15990E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.16222E+01_JPRB, 0.17079E+01_JPRB, 0.17988E+01_JPRB, 0.19037E+01_JPRB, 0.20177E+01_JPRB, 0.21042E+01_JPRB, 0.21583E+01_JPRB ,&
&  0.22282E+01_JPRB, 0.23270E+01_JPRB, 0.24467E+01_JPRB, 0.25496E+01_JPRB, 0.27017E+01_JPRB, 0.28443E+01_JPRB, 0.29883E+01_JPRB ,&
&  0.31309E+01_JPRB, 0.32877E+01_JPRB, 0.34847E+01_JPRB, 0.36530E+01_JPRB, 0.37445E+01_JPRB, 0.38354E+01_JPRB, 0.39388E+01_JPRB ,&
&  0.40300E+01_JPRB, 0.34275E+01_JPRB, 0.29998E+01_JPRB, 0.23182E+01_JPRB, 0.18915E+01_JPRB, 0.23796E+01_JPRB, 0.23513E+01_JPRB ,&
&  0.23437E+01_JPRB, 0.23142E+01_JPRB, 0.23417E+01_JPRB, 0.24318E+01_JPRB, 0.25792E+01_JPRB, 0.27316E+01_JPRB, 0.28589E+01_JPRB ,&
&  0.29704E+01_JPRB, 0.30611E+01_JPRB, 0.31390E+01_JPRB, 0.32320E+01_JPRB, 0.33437E+01_JPRB, 0.34484E+01_JPRB, 0.35540E+01_JPRB ,&
&  0.36980E+01_JPRB, 0.38463E+01_JPRB, 0.39465E+01_JPRB, 0.40661E+01_JPRB, 0.40944E+01_JPRB, 0.42440E+01_JPRB, 0.44054E+01_JPRB ,&
&  0.45334E+01_JPRB, 0.46324E+01_JPRB, 0.49130E+01_JPRB, 0.52060E+01_JPRB, 0.52455E+01_JPRB, 0.47961E+01_JPRB, 0.47994E+01_JPRB ,&
&  0.48243E+01_JPRB, 0.45390E+01_JPRB, 0.44447E+01_JPRB, 0.56219E+01_JPRB, 0.35904E+01_JPRB, 0.38458E+01_JPRB, 0.37616E+01_JPRB ,&
&  0.33011E+01_JPRB, 0.31181E+01_JPRB, 0.27406E+01_JPRB, 0.22051E+01_JPRB, 0.19922E+01_JPRB, 0.19954E+01_JPRB, 0.25330E+01_JPRB ,&
&  0.29154E+01_JPRB, 0.24155E+01_JPRB, 0.18186E+01_JPRB, 0.16127E+01_JPRB, 0.17836E+01_JPRB, 0.24348E+01_JPRB, 0.25150E+01_JPRB ,&
&  0.19731E+01_JPRB, 0.18009E+01_JPRB, 0.16187E+01_JPRB, 0.24805E+01_JPRB, 0.19185E+01_JPRB, 0.95764E+00_JPRB, 0.80647E+00_JPRB ,&
&  0.84087E+00_JPRB, 0.91490E+00_JPRB, 0.10274E+01_JPRB, 0.11049E+01_JPRB, 0.11402E+01_JPRB, 0.11869E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.13489E+01_JPRB, 0.14373E+01_JPRB, 0.14982E+01_JPRB, 0.15587E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.12650E+01_JPRB, 0.13338E+01_JPRB, 0.13881E+01_JPRB, 0.14181E+01_JPRB, 0.14369E+01_JPRB, 0.14366E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.13703E+01_JPRB, 0.14090E+01_JPRB, 0.15209E+01_JPRB, 0.17685E+01_JPRB, 0.19817E+01_JPRB, 0.21197E+01_JPRB, 0.22524E+01_JPRB ,&
&  0.23798E+01_JPRB, 0.24231E+01_JPRB, 0.23531E+01_JPRB, 0.23227E+01_JPRB, 0.22326E+01_JPRB, 0.21270E+01_JPRB, 0.23192E+01_JPRB ,&
&  0.26782E+01_JPRB, 0.30775E+01_JPRB, 0.33163E+01_JPRB, 0.35314E+01_JPRB, 0.28411E+01_JPRB, 0.23314E+01_JPRB, 0.25814E+01_JPRB ,&
&  0.28730E+01_JPRB, 0.24922E+01_JPRB, 0.20112E+01_JPRB, 0.20049E+01_JPRB, 0.20390E+01_JPRB, 0.21373E+01_JPRB, 0.22964E+01_JPRB ,&
&  0.24051E+01_JPRB, 0.24798E+01_JPRB, 0.25506E+01_JPRB, 0.26090E+01_JPRB, 0.26686E+01_JPRB, 0.27238E+01_JPRB, 0.27904E+01_JPRB ,&
&  0.28294E+01_JPRB, 0.28446E+01_JPRB, 0.29474E+01_JPRB, 0.31472E+01_JPRB, 0.33428E+01_JPRB, 0.35813E+01_JPRB, 0.37697E+01_JPRB ,&
&  0.38824E+01_JPRB, 0.39920E+01_JPRB, 0.44235E+01_JPRB, 0.46923E+01_JPRB, 0.44489E+01_JPRB, 0.40804E+01_JPRB, 0.41115E+01_JPRB ,&
&  0.40140E+01_JPRB, 0.36738E+01_JPRB, 0.29918E+01_JPRB, 0.43206E+01_JPRB, 0.43654E+01_JPRB, 0.37930E+01_JPRB, 0.32037E+01_JPRB ,&
&  0.28695E+01_JPRB, 0.27196E+01_JPRB, 0.24490E+01_JPRB, 0.21098E+01_JPRB, 0.18197E+01_JPRB, 0.22061E+01_JPRB, 0.24322E+01_JPRB ,&
&  0.24690E+01_JPRB, 0.19535E+01_JPRB, 0.14603E+01_JPRB, 0.13141E+01_JPRB, 0.13853E+01_JPRB, 0.20582E+01_JPRB, 0.20674E+01_JPRB ,&
&  0.16417E+01_JPRB, 0.14963E+01_JPRB, 0.13782E+01_JPRB, 0.20275E+01_JPRB, 0.31663E+01_JPRB, 0.98387E+00_JPRB, 0.69796E+00_JPRB ,&
&  0.70396E+00_JPRB, 0.73834E+00_JPRB, 0.77578E+00_JPRB, 0.78600E+00_JPRB, 0.78791E+00_JPRB, 0.82641E+00_JPRB, 0.89004E+00_JPRB ,&
&  0.96833E+00_JPRB, 0.10390E+01_JPRB, 0.10852E+01_JPRB, 0.11523E+01_JPRB, 0.12073E+01_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.87115E+00_JPRB, 0.90454E+00_JPRB, 0.91119E+00_JPRB, 0.92499E+00_JPRB, 0.92163E+00_JPRB, 0.92227E+00_JPRB, 0.89110E+00_JPRB ,&
&  0.87155E+00_JPRB, 0.89904E+00_JPRB, 0.97966E+00_JPRB, 0.11277E+01_JPRB, 0.11819E+01_JPRB, 0.12248E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.12307E+01_JPRB, 0.11815E+01_JPRB, 0.10978E+01_JPRB, 0.10628E+01_JPRB, 0.10834E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.15860E+01_JPRB, 0.18458E+01_JPRB, 0.21562E+01_JPRB, 0.28895E+01_JPRB, 0.26324E+01_JPRB, 0.26173E+01_JPRB ,&
&  0.30768E+01_JPRB, 0.22834E+01_JPRB, 0.18675E+01_JPRB, 0.16268E+01_JPRB, 0.15146E+01_JPRB, 0.14602E+01_JPRB, 0.15157E+01_JPRB ,&
&  0.15477E+01_JPRB, 0.16022E+01_JPRB, 0.16681E+01_JPRB, 0.16911E+01_JPRB, 0.17080E+01_JPRB, 0.16757E+01_JPRB, 0.16220E+01_JPRB ,&
&  0.15389E+01_JPRB, 0.16025E+01_JPRB, 0.17286E+01_JPRB, 0.18875E+01_JPRB, 0.22116E+01_JPRB, 0.24383E+01_JPRB, 0.26143E+01_JPRB ,&
&  0.26980E+01_JPRB, 0.28703E+01_JPRB, 0.32361E+01_JPRB, 0.33555E+01_JPRB, 0.34172E+01_JPRB, 0.32378E+01_JPRB, 0.30785E+01_JPRB ,&
&  0.28273E+01_JPRB, 0.22969E+01_JPRB, 0.19772E+01_JPRB, 0.29884E+01_JPRB, 0.31452E+01_JPRB, 0.25776E+01_JPRB, 0.26990E+01_JPRB ,&
&  0.24999E+01_JPRB, 0.23633E+01_JPRB, 0.22772E+01_JPRB, 0.19849E+01_JPRB, 0.16480E+01_JPRB, 0.18384E+01_JPRB, 0.22523E+01_JPRB ,&
&  0.19912E+01_JPRB, 0.14700E+01_JPRB, 0.11585E+01_JPRB, 0.11144E+01_JPRB, 0.12109E+01_JPRB, 0.16532E+01_JPRB, 0.18219E+01_JPRB ,&
&  0.16857E+01_JPRB, 0.12689E+01_JPRB, 0.14195E+01_JPRB, 0.21981E+01_JPRB, 0.16273E+01_JPRB, 0.84159E+00_JPRB, 0.58314E+00_JPRB ,&
&  0.56624E+00_JPRB, 0.57867E+00_JPRB, 0.59742E+00_JPRB, 0.60482E+00_JPRB, 0.60891E+00_JPRB, 0.62079E+00_JPRB, 0.66395E+00_JPRB ,&
&  0.70269E+00_JPRB, 0.74294E+00_JPRB, 0.77599E+00_JPRB, 0.82115E+00_JPRB, 0.84989E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.81191E+00_JPRB, 0.83238E+00_JPRB, 0.85784E+00_JPRB, 0.87442E+00_JPRB, 0.89641E+00_JPRB, 0.91677E+00_JPRB, 0.96323E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10483E+01_JPRB, 0.11014E+01_JPRB, 0.11578E+01_JPRB, 0.11755E+01_JPRB, 0.11917E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.12283E+01_JPRB, 0.12421E+01_JPRB, 0.12250E+01_JPRB, 0.12186E+01_JPRB, 0.12104E+01_JPRB, 0.12903E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12783E+01_JPRB, 0.12901E+01_JPRB, 0.13605E+01_JPRB, 0.15828E+01_JPRB, 0.18119E+01_JPRB, 0.19142E+01_JPRB ,&
&  0.23895E+01_JPRB, 0.18632E+01_JPRB, 0.16244E+01_JPRB, 0.12099E+01_JPRB, 0.11211E+01_JPRB, 0.11171E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.10398E+01_JPRB, 0.10598E+01_JPRB, 0.11102E+01_JPRB, 0.10985E+01_JPRB, 0.11283E+01_JPRB, 0.11388E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.11881E+01_JPRB, 0.12451E+01_JPRB, 0.13816E+01_JPRB, 0.14870E+01_JPRB, 0.16927E+01_JPRB, 0.17722E+01_JPRB ,&
&  0.17890E+01_JPRB, 0.18551E+01_JPRB, 0.17682E+01_JPRB, 0.18278E+01_JPRB, 0.20450E+01_JPRB, 0.20567E+01_JPRB, 0.19763E+01_JPRB ,&
&  0.19792E+01_JPRB, 0.18113E+01_JPRB, 0.14610E+01_JPRB, 0.21799E+01_JPRB, 0.24621E+01_JPRB, 0.23112E+01_JPRB, 0.22821E+01_JPRB ,&
&  0.21357E+01_JPRB, 0.19285E+01_JPRB, 0.17792E+01_JPRB, 0.17142E+01_JPRB, 0.15265E+01_JPRB, 0.14540E+01_JPRB, 0.16118E+01_JPRB ,&
&  0.14269E+01_JPRB, 0.11962E+01_JPRB, 0.10813E+01_JPRB, 0.11566E+01_JPRB, 0.15077E+01_JPRB, 0.24174E+01_JPRB, 0.21682E+01_JPRB ,&
&  0.15171E+01_JPRB, 0.12244E+01_JPRB, 0.17733E+01_JPRB, 0.26485E+01_JPRB, 0.14134E+01_JPRB, 0.86193E+00_JPRB, 0.55857E+00_JPRB ,&
&  0.52136E+00_JPRB, 0.54361E+00_JPRB, 0.57056E+00_JPRB, 0.59330E+00_JPRB, 0.62169E+00_JPRB, 0.64779E+00_JPRB, 0.66938E+00_JPRB ,&
&  0.69037E+00_JPRB, 0.71755E+00_JPRB, 0.74226E+00_JPRB, 0.77285E+00_JPRB, 0.80020E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.95575E+00_JPRB, 0.10051E+01_JPRB, 0.10423E+01_JPRB, 0.10735E+01_JPRB, 0.11009E+01_JPRB, 0.11300E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.12313E+01_JPRB, 0.12615E+01_JPRB, 0.12995E+01_JPRB, 0.13471E+01_JPRB, 0.13899E+01_JPRB, 0.14078E+01_JPRB ,&
&  0.14126E+01_JPRB, 0.14585E+01_JPRB, 0.14797E+01_JPRB, 0.15386E+01_JPRB, 0.15579E+01_JPRB, 0.15976E+01_JPRB, 0.15998E+01_JPRB ,&
&  0.15869E+01_JPRB, 0.15806E+01_JPRB, 0.15841E+01_JPRB, 0.16001E+01_JPRB, 0.16103E+01_JPRB, 0.17354E+01_JPRB, 0.19250E+01_JPRB ,&
&  0.19065E+01_JPRB, 0.18145E+01_JPRB, 0.13053E+01_JPRB, 0.10532E+01_JPRB, 0.10423E+01_JPRB, 0.10495E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.10061E+01_JPRB, 0.10169E+01_JPRB, 0.10314E+01_JPRB, 0.10225E+01_JPRB, 0.10704E+01_JPRB, 0.11688E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.12848E+01_JPRB, 0.13079E+01_JPRB, 0.14291E+01_JPRB, 0.14299E+01_JPRB, 0.16549E+01_JPRB, 0.17617E+01_JPRB ,&
&  0.17795E+01_JPRB, 0.16875E+01_JPRB, 0.13478E+01_JPRB, 0.13629E+01_JPRB, 0.14921E+01_JPRB, 0.14663E+01_JPRB, 0.14276E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.17026E+01_JPRB, 0.15684E+01_JPRB, 0.19987E+01_JPRB, 0.21426E+01_JPRB, 0.21041E+01_JPRB, 0.18785E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.13354E+01_JPRB, 0.13300E+01_JPRB, 0.13508E+01_JPRB, 0.12800E+01_JPRB, 0.12146E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.10627E+01_JPRB, 0.10899E+01_JPRB, 0.12517E+01_JPRB, 0.19608E+01_JPRB, 0.23224E+01_JPRB, 0.27775E+01_JPRB ,&
&  0.12618E+01_JPRB, 0.12194E+01_JPRB, 0.97625E+00_JPRB, 0.12504E+01_JPRB, 0.99757E+00_JPRB, 0.64943E+00_JPRB, 0.55080E+00_JPRB ,&
&  0.58306E+00_JPRB, 0.63115E+00_JPRB, 0.67181E+00_JPRB, 0.70321E+00_JPRB, 0.73284E+00_JPRB, 0.76446E+00_JPRB, 0.78262E+00_JPRB ,&
&  0.80538E+00_JPRB, 0.83150E+00_JPRB, 0.85696E+00_JPRB, 0.89402E+00_JPRB, 0.92009E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.10360E+01_JPRB, 0.10768E+01_JPRB, 0.11130E+01_JPRB, 0.11473E+01_JPRB, 0.11854E+01_JPRB, 0.12267E+01_JPRB, 0.12648E+01_JPRB ,&
&  0.12972E+01_JPRB, 0.13316E+01_JPRB, 0.13614E+01_JPRB, 0.13906E+01_JPRB, 0.14298E+01_JPRB, 0.14683E+01_JPRB, 0.14933E+01_JPRB ,&
&  0.15200E+01_JPRB, 0.15643E+01_JPRB, 0.16098E+01_JPRB, 0.16539E+01_JPRB, 0.16765E+01_JPRB, 0.16973E+01_JPRB, 0.16933E+01_JPRB ,&
&  0.16756E+01_JPRB, 0.16664E+01_JPRB, 0.16805E+01_JPRB, 0.17072E+01_JPRB, 0.17512E+01_JPRB, 0.17512E+01_JPRB, 0.18968E+01_JPRB ,&
&  0.17022E+01_JPRB, 0.18432E+01_JPRB, 0.14171E+01_JPRB, 0.12271E+01_JPRB, 0.12305E+01_JPRB, 0.11482E+01_JPRB, 0.11300E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11247E+01_JPRB, 0.11186E+01_JPRB, 0.11134E+01_JPRB, 0.11486E+01_JPRB, 0.12191E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.13401E+01_JPRB, 0.14355E+01_JPRB, 0.15075E+01_JPRB, 0.16000E+01_JPRB, 0.17899E+01_JPRB, 0.19427E+01_JPRB, 0.20127E+01_JPRB ,&
&  0.19867E+01_JPRB, 0.19011E+01_JPRB, 0.14882E+01_JPRB, 0.15400E+01_JPRB, 0.16336E+01_JPRB, 0.15488E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.13912E+01_JPRB, 0.16144E+01_JPRB, 0.17023E+01_JPRB, 0.16514E+01_JPRB, 0.14879E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.89123E+00_JPRB, 0.91598E+00_JPRB, 0.10446E+01_JPRB, 0.10169E+01_JPRB, 0.88376E+00_JPRB, 0.88528E+00_JPRB ,&
&  0.86975E+00_JPRB, 0.84784E+00_JPRB, 0.90381E+00_JPRB, 0.11459E+01_JPRB, 0.16505E+01_JPRB, 0.28494E+01_JPRB, 0.26605E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.10541E+01_JPRB, 0.64068E+00_JPRB, 0.68051E+00_JPRB, 0.69816E+00_JPRB, 0.59108E+00_JPRB, 0.60297E+00_JPRB ,&
&  0.71126E+00_JPRB, 0.81628E+00_JPRB, 0.83915E+00_JPRB, 0.79254E+00_JPRB, 0.78460E+00_JPRB, 0.80413E+00_JPRB, 0.83395E+00_JPRB ,&
&  0.86481E+00_JPRB, 0.89448E+00_JPRB, 0.92549E+00_JPRB, 0.95980E+00_JPRB, 0.99588E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.10712E+01_JPRB, 0.11124E+01_JPRB, 0.11488E+01_JPRB, 0.11819E+01_JPRB, 0.12221E+01_JPRB, 0.12662E+01_JPRB, 0.13088E+01_JPRB ,&
&  0.13475E+01_JPRB, 0.13835E+01_JPRB, 0.14230E+01_JPRB, 0.14614E+01_JPRB, 0.15002E+01_JPRB, 0.15424E+01_JPRB, 0.15922E+01_JPRB ,&
&  0.16385E+01_JPRB, 0.16732E+01_JPRB, 0.17297E+01_JPRB, 0.17960E+01_JPRB, 0.18416E+01_JPRB, 0.18700E+01_JPRB, 0.18833E+01_JPRB ,&
&  0.18982E+01_JPRB, 0.19436E+01_JPRB, 0.19719E+01_JPRB, 0.19947E+01_JPRB, 0.20378E+01_JPRB, 0.22136E+01_JPRB, 0.19875E+01_JPRB ,&
&  0.21471E+01_JPRB, 0.18210E+01_JPRB, 0.15026E+01_JPRB, 0.13983E+01_JPRB, 0.13677E+01_JPRB, 0.13461E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.12504E+01_JPRB, 0.12143E+01_JPRB, 0.11926E+01_JPRB, 0.12154E+01_JPRB, 0.12701E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.14434E+01_JPRB, 0.15859E+01_JPRB, 0.17515E+01_JPRB, 0.19293E+01_JPRB, 0.21426E+01_JPRB, 0.23137E+01_JPRB, 0.24175E+01_JPRB ,&
&  0.24322E+01_JPRB, 0.23998E+01_JPRB, 0.20654E+01_JPRB, 0.21636E+01_JPRB, 0.21641E+01_JPRB, 0.18639E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.11681E+01_JPRB, 0.12369E+01_JPRB, 0.13236E+01_JPRB, 0.12257E+01_JPRB, 0.10548E+01_JPRB, 0.89570E+00_JPRB ,&
&  0.71472E+00_JPRB, 0.65604E+00_JPRB, 0.65693E+00_JPRB, 0.76338E+00_JPRB, 0.76420E+00_JPRB, 0.69512E+00_JPRB, 0.71657E+00_JPRB ,&
&  0.72474E+00_JPRB, 0.72760E+00_JPRB, 0.80157E+00_JPRB, 0.10210E+01_JPRB, 0.13612E+01_JPRB, 0.13747E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.98276E+00_JPRB, 0.83857E+00_JPRB, 0.78520E+00_JPRB, 0.68136E+00_JPRB, 0.71463E+00_JPRB, 0.72025E+00_JPRB, 0.81820E+00_JPRB ,&
&  0.98054E+00_JPRB, 0.11973E+01_JPRB, 0.11727E+01_JPRB, 0.97387E+00_JPRB, 0.83021E+00_JPRB, 0.81082E+00_JPRB, 0.84032E+00_JPRB ,&
&  0.87396E+00_JPRB, 0.90490E+00_JPRB, 0.94268E+00_JPRB, 0.98387E+00_JPRB, 0.10266E+01_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.10755E+01_JPRB, 0.11145E+01_JPRB, 0.11476E+01_JPRB, 0.11767E+01_JPRB, 0.12166E+01_JPRB, 0.12587E+01_JPRB, 0.13024E+01_JPRB ,&
&  0.13449E+01_JPRB, 0.13869E+01_JPRB, 0.14329E+01_JPRB, 0.14773E+01_JPRB, 0.15277E+01_JPRB, 0.15807E+01_JPRB, 0.16387E+01_JPRB ,&
&  0.16932E+01_JPRB, 0.17468E+01_JPRB, 0.17947E+01_JPRB, 0.18846E+01_JPRB, 0.19864E+01_JPRB, 0.20603E+01_JPRB, 0.21113E+01_JPRB ,&
&  0.21348E+01_JPRB, 0.21908E+01_JPRB, 0.22580E+01_JPRB, 0.23144E+01_JPRB, 0.23296E+01_JPRB, 0.24780E+01_JPRB, 0.22310E+01_JPRB ,&
&  0.21563E+01_JPRB, 0.18867E+01_JPRB, 0.16480E+01_JPRB, 0.15883E+01_JPRB, 0.15425E+01_JPRB, 0.15219E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.14569E+01_JPRB, 0.14080E+01_JPRB, 0.13355E+01_JPRB, 0.12865E+01_JPRB, 0.12770E+01_JPRB, 0.13157E+01_JPRB, 0.13967E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.16714E+01_JPRB, 0.18590E+01_JPRB, 0.20619E+01_JPRB, 0.22930E+01_JPRB, 0.24961E+01_JPRB, 0.26815E+01_JPRB ,&
&  0.28214E+01_JPRB, 0.29042E+01_JPRB, 0.26830E+01_JPRB, 0.26474E+01_JPRB, 0.25375E+01_JPRB, 0.18819E+01_JPRB, 0.14633E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.11016E+01_JPRB, 0.11451E+01_JPRB, 0.11469E+01_JPRB, 0.95330E+00_JPRB, 0.83389E+00_JPRB, 0.69509E+00_JPRB ,&
&  0.58749E+00_JPRB, 0.54649E+00_JPRB, 0.55344E+00_JPRB, 0.61378E+00_JPRB, 0.60337E+00_JPRB, 0.57767E+00_JPRB, 0.61463E+00_JPRB ,&
&  0.63350E+00_JPRB, 0.69109E+00_JPRB, 0.78551E+00_JPRB, 0.93722E+00_JPRB, 0.10889E+01_JPRB, 0.10925E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.96527E+00_JPRB, 0.94061E+00_JPRB, 0.84172E+00_JPRB, 0.63726E+00_JPRB, 0.80155E+00_JPRB, 0.91015E+00_JPRB, 0.10451E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.15804E+01_JPRB, 0.18071E+01_JPRB, 0.14254E+01_JPRB, 0.11302E+01_JPRB, 0.85966E+00_JPRB, 0.83996E+00_JPRB ,&
&  0.87664E+00_JPRB, 0.89603E+00_JPRB, 0.93377E+00_JPRB, 0.97990E+00_JPRB, 0.10293E+01_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.10286E+01_JPRB, 0.10715E+01_JPRB, 0.10999E+01_JPRB, 0.11121E+01_JPRB, 0.11459E+01_JPRB, 0.11985E+01_JPRB, 0.12373E+01_JPRB ,&
&  0.12789E+01_JPRB, 0.13136E+01_JPRB, 0.13612E+01_JPRB, 0.14063E+01_JPRB, 0.14547E+01_JPRB, 0.15053E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.16178E+01_JPRB, 0.17048E+01_JPRB, 0.17659E+01_JPRB, 0.18680E+01_JPRB, 0.19791E+01_JPRB, 0.21144E+01_JPRB, 0.21967E+01_JPRB ,&
&  0.22235E+01_JPRB, 0.22942E+01_JPRB, 0.23782E+01_JPRB, 0.24739E+01_JPRB, 0.24736E+01_JPRB, 0.24984E+01_JPRB, 0.19186E+01_JPRB ,&
&  0.20390E+01_JPRB, 0.19331E+01_JPRB, 0.18217E+01_JPRB, 0.17635E+01_JPRB, 0.16910E+01_JPRB, 0.16254E+01_JPRB, 0.15585E+01_JPRB ,&
&  0.15245E+01_JPRB, 0.14575E+01_JPRB, 0.13385E+01_JPRB, 0.13370E+01_JPRB, 0.13256E+01_JPRB, 0.13297E+01_JPRB, 0.14177E+01_JPRB ,&
&  0.15087E+01_JPRB, 0.16488E+01_JPRB, 0.18265E+01_JPRB, 0.20381E+01_JPRB, 0.22661E+01_JPRB, 0.24895E+01_JPRB, 0.27079E+01_JPRB ,&
&  0.29120E+01_JPRB, 0.30758E+01_JPRB, 0.31042E+01_JPRB, 0.25966E+01_JPRB, 0.22395E+01_JPRB, 0.17070E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.11148E+01_JPRB, 0.11004E+01_JPRB, 0.10686E+01_JPRB, 0.85956E+00_JPRB, 0.72804E+00_JPRB, 0.60610E+00_JPRB ,&
&  0.55899E+00_JPRB, 0.52391E+00_JPRB, 0.52411E+00_JPRB, 0.52718E+00_JPRB, 0.53004E+00_JPRB, 0.53290E+00_JPRB, 0.56159E+00_JPRB ,&
&  0.59190E+00_JPRB, 0.65838E+00_JPRB, 0.78174E+00_JPRB, 0.95423E+00_JPRB, 0.11144E+01_JPRB, 0.11922E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.97753E+00_JPRB, 0.85406E+00_JPRB, 0.81041E+00_JPRB, 0.83858E+00_JPRB, 0.88302E+00_JPRB, 0.10612E+01_JPRB, 0.12718E+01_JPRB ,&
&  0.15311E+01_JPRB, 0.15591E+01_JPRB, 0.18418E+01_JPRB, 0.23949E+01_JPRB, 0.21604E+01_JPRB, 0.18719E+01_JPRB, 0.76655E+00_JPRB ,&
&  0.82081E+00_JPRB, 0.84366E+00_JPRB, 0.89075E+00_JPRB, 0.93851E+00_JPRB, 0.99745E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.82471E+00_JPRB, 0.86521E+00_JPRB, 0.90089E+00_JPRB, 0.91708E+00_JPRB, 0.96616E+00_JPRB, 0.10191E+01_JPRB, 0.10578E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.11485E+01_JPRB, 0.11923E+01_JPRB, 0.12487E+01_JPRB, 0.12967E+01_JPRB, 0.13447E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.14451E+01_JPRB, 0.15192E+01_JPRB, 0.15960E+01_JPRB, 0.17030E+01_JPRB, 0.18156E+01_JPRB, 0.19548E+01_JPRB, 0.20829E+01_JPRB ,&
&  0.21807E+01_JPRB, 0.22799E+01_JPRB, 0.23968E+01_JPRB, 0.24756E+01_JPRB, 0.24490E+01_JPRB, 0.23940E+01_JPRB, 0.17402E+01_JPRB ,&
&  0.19618E+01_JPRB, 0.19924E+01_JPRB, 0.19740E+01_JPRB, 0.19150E+01_JPRB, 0.18258E+01_JPRB, 0.17218E+01_JPRB, 0.16496E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.14829E+01_JPRB, 0.13660E+01_JPRB, 0.12712E+01_JPRB, 0.13404E+01_JPRB, 0.13358E+01_JPRB, 0.14004E+01_JPRB ,&
&  0.14916E+01_JPRB, 0.16097E+01_JPRB, 0.17547E+01_JPRB, 0.19602E+01_JPRB, 0.21879E+01_JPRB, 0.24113E+01_JPRB, 0.26385E+01_JPRB ,&
&  0.28079E+01_JPRB, 0.28932E+01_JPRB, 0.29223E+01_JPRB, 0.23810E+01_JPRB, 0.20325E+01_JPRB, 0.16854E+01_JPRB, 0.13343E+01_JPRB ,&
&  0.11656E+01_JPRB, 0.11025E+01_JPRB, 0.11033E+01_JPRB, 0.10239E+01_JPRB, 0.84349E+00_JPRB, 0.68982E+00_JPRB, 0.58228E+00_JPRB ,&
&  0.54003E+00_JPRB, 0.50266E+00_JPRB, 0.50144E+00_JPRB, 0.50874E+00_JPRB, 0.52459E+00_JPRB, 0.53912E+00_JPRB, 0.56500E+00_JPRB ,&
&  0.60151E+00_JPRB, 0.65748E+00_JPRB, 0.75065E+00_JPRB, 0.90115E+00_JPRB, 0.11110E+01_JPRB, 0.13406E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.85544E+00_JPRB, 0.79197E+00_JPRB, 0.76278E+00_JPRB, 0.77604E+00_JPRB, 0.83142E+00_JPRB, 0.90376E+00_JPRB ,&
&  0.10350E+01_JPRB, 0.11095E+01_JPRB, 0.99631E+00_JPRB, 0.83295E+00_JPRB, 0.10904E+01_JPRB, 0.10848E+01_JPRB, 0.61911E+00_JPRB ,&
&  0.67877E+00_JPRB, 0.66898E+00_JPRB, 0.70384E+00_JPRB, 0.74602E+00_JPRB, 0.79735E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.62575E+00_JPRB, 0.68757E+00_JPRB, 0.71881E+00_JPRB, 0.75737E+00_JPRB, 0.79088E+00_JPRB, 0.83925E+00_JPRB, 0.88626E+00_JPRB ,&
&  0.93413E+00_JPRB, 0.98798E+00_JPRB, 0.10303E+01_JPRB, 0.10849E+01_JPRB, 0.11333E+01_JPRB, 0.11778E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.12638E+01_JPRB, 0.13273E+01_JPRB, 0.13929E+01_JPRB, 0.14688E+01_JPRB, 0.15503E+01_JPRB, 0.16578E+01_JPRB, 0.18074E+01_JPRB ,&
&  0.19881E+01_JPRB, 0.21643E+01_JPRB, 0.23235E+01_JPRB, 0.24569E+01_JPRB, 0.25288E+01_JPRB, 0.26298E+01_JPRB, 0.26525E+01_JPRB ,&
&  0.17671E+01_JPRB, 0.21656E+01_JPRB, 0.20849E+01_JPRB, 0.19706E+01_JPRB, 0.18789E+01_JPRB, 0.17888E+01_JPRB, 0.17351E+01_JPRB ,&
&  0.16634E+01_JPRB, 0.14954E+01_JPRB, 0.14127E+01_JPRB, 0.13015E+01_JPRB, 0.12977E+01_JPRB, 0.13115E+01_JPRB, 0.13630E+01_JPRB ,&
&  0.14476E+01_JPRB, 0.15641E+01_JPRB, 0.16913E+01_JPRB, 0.18285E+01_JPRB, 0.20213E+01_JPRB, 0.22623E+01_JPRB, 0.24769E+01_JPRB ,&
&  0.26988E+01_JPRB, 0.29256E+01_JPRB, 0.30573E+01_JPRB, 0.26054E+01_JPRB, 0.21983E+01_JPRB, 0.19981E+01_JPRB, 0.15641E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.10292E+01_JPRB, 0.10400E+01_JPRB, 0.10061E+01_JPRB, 0.84146E+00_JPRB, 0.70654E+00_JPRB, 0.61171E+00_JPRB ,&
&  0.57004E+00_JPRB, 0.52965E+00_JPRB, 0.51819E+00_JPRB, 0.52547E+00_JPRB, 0.53928E+00_JPRB, 0.55787E+00_JPRB, 0.59757E+00_JPRB ,&
&  0.62833E+00_JPRB, 0.65777E+00_JPRB, 0.69449E+00_JPRB, 0.73277E+00_JPRB, 0.80373E+00_JPRB, 0.91564E+00_JPRB, 0.92862E+00_JPRB ,&
&  0.88352E+00_JPRB, 0.84081E+00_JPRB, 0.79611E+00_JPRB, 0.78015E+00_JPRB, 0.75029E+00_JPRB, 0.73611E+00_JPRB, 0.74618E+00_JPRB ,&
&  0.76525E+00_JPRB, 0.77722E+00_JPRB, 0.73256E+00_JPRB, 0.69547E+00_JPRB, 0.68200E+00_JPRB, 0.64964E+00_JPRB, 0.59602E+00_JPRB ,&
&  0.57684E+00_JPRB, 0.56019E+00_JPRB, 0.54286E+00_JPRB, 0.55471E+00_JPRB, 0.58600E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.52824E+00_JPRB, 0.55222E+00_JPRB, 0.58146E+00_JPRB, 0.61188E+00_JPRB, 0.64056E+00_JPRB, 0.66951E+00_JPRB, 0.71316E+00_JPRB ,&
&  0.76410E+00_JPRB, 0.80451E+00_JPRB, 0.82890E+00_JPRB, 0.86681E+00_JPRB, 0.91209E+00_JPRB, 0.96833E+00_JPRB, 0.10231E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.11515E+01_JPRB, 0.12112E+01_JPRB, 0.12730E+01_JPRB, 0.13365E+01_JPRB, 0.14072E+01_JPRB, 0.15133E+01_JPRB ,&
&  0.16507E+01_JPRB, 0.18315E+01_JPRB, 0.20573E+01_JPRB, 0.22785E+01_JPRB, 0.24813E+01_JPRB, 0.26803E+01_JPRB, 0.27486E+01_JPRB ,&
&  0.15837E+01_JPRB, 0.20883E+01_JPRB, 0.23941E+01_JPRB, 0.21266E+01_JPRB, 0.19152E+01_JPRB, 0.17850E+01_JPRB, 0.17078E+01_JPRB ,&
&  0.15887E+01_JPRB, 0.14293E+01_JPRB, 0.13527E+01_JPRB, 0.13801E+01_JPRB, 0.12623E+01_JPRB, 0.12652E+01_JPRB, 0.12860E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.14208E+01_JPRB, 0.15048E+01_JPRB, 0.16082E+01_JPRB, 0.17613E+01_JPRB, 0.19508E+01_JPRB, 0.21891E+01_JPRB ,&
&  0.24900E+01_JPRB, 0.28260E+01_JPRB, 0.31358E+01_JPRB, 0.25561E+01_JPRB, 0.23529E+01_JPRB, 0.22321E+01_JPRB, 0.19544E+01_JPRB ,&
&  0.13262E+01_JPRB, 0.10406E+01_JPRB, 0.10328E+01_JPRB, 0.10073E+01_JPRB, 0.88349E+00_JPRB, 0.77859E+00_JPRB, 0.70218E+00_JPRB ,&
&  0.64997E+00_JPRB, 0.60897E+00_JPRB, 0.58880E+00_JPRB, 0.58398E+00_JPRB, 0.58882E+00_JPRB, 0.61282E+00_JPRB, 0.64490E+00_JPRB ,&
&  0.67467E+00_JPRB, 0.68296E+00_JPRB, 0.71683E+00_JPRB, 0.73390E+00_JPRB, 0.78222E+00_JPRB, 0.82527E+00_JPRB, 0.85360E+00_JPRB ,&
&  0.86461E+00_JPRB, 0.85526E+00_JPRB, 0.84670E+00_JPRB, 0.83752E+00_JPRB, 0.82983E+00_JPRB, 0.82007E+00_JPRB, 0.80771E+00_JPRB ,&
&  0.79017E+00_JPRB, 0.79271E+00_JPRB, 0.77116E+00_JPRB, 0.75341E+00_JPRB, 0.74894E+00_JPRB, 0.71031E+00_JPRB, 0.68925E+00_JPRB ,&
&  0.66789E+00_JPRB, 0.61573E+00_JPRB, 0.58008E+00_JPRB, 0.55327E+00_JPRB, 0.53280E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.55757E+00_JPRB, 0.52610E+00_JPRB, 0.53111E+00_JPRB, 0.53918E+00_JPRB, 0.56146E+00_JPRB, 0.58477E+00_JPRB, 0.58196E+00_JPRB ,&
&  0.59461E+00_JPRB, 0.61456E+00_JPRB, 0.64239E+00_JPRB, 0.66954E+00_JPRB, 0.70157E+00_JPRB, 0.73749E+00_JPRB, 0.77705E+00_JPRB ,&
&  0.83081E+00_JPRB, 0.89686E+00_JPRB, 0.96903E+00_JPRB, 0.10363E+01_JPRB, 0.11036E+01_JPRB, 0.11586E+01_JPRB, 0.12148E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.14129E+01_JPRB, 0.15939E+01_JPRB, 0.18229E+01_JPRB, 0.21211E+01_JPRB, 0.24775E+01_JPRB, 0.27290E+01_JPRB ,&
&  0.18989E+01_JPRB, 0.93945E+00_JPRB, 0.22298E+01_JPRB, 0.24663E+01_JPRB, 0.21022E+01_JPRB, 0.19303E+01_JPRB, 0.17270E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.15446E+01_JPRB, 0.14071E+01_JPRB, 0.13662E+01_JPRB, 0.12600E+01_JPRB, 0.12398E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.12479E+01_JPRB, 0.13288E+01_JPRB, 0.13817E+01_JPRB, 0.14611E+01_JPRB, 0.16026E+01_JPRB, 0.18164E+01_JPRB ,&
&  0.21217E+01_JPRB, 0.24905E+01_JPRB, 0.28583E+01_JPRB, 0.22761E+01_JPRB, 0.18662E+01_JPRB, 0.15654E+01_JPRB, 0.13517E+01_JPRB ,&
&  0.12295E+01_JPRB, 0.10582E+01_JPRB, 0.10198E+01_JPRB, 0.10017E+01_JPRB, 0.92911E+00_JPRB, 0.81956E+00_JPRB, 0.79494E+00_JPRB ,&
&  0.74413E+00_JPRB, 0.70849E+00_JPRB, 0.68726E+00_JPRB, 0.67295E+00_JPRB, 0.67099E+00_JPRB, 0.68609E+00_JPRB, 0.70457E+00_JPRB ,&
&  0.72957E+00_JPRB, 0.74760E+00_JPRB, 0.77474E+00_JPRB, 0.80611E+00_JPRB, 0.82062E+00_JPRB, 0.83174E+00_JPRB, 0.85381E+00_JPRB ,&
&  0.85989E+00_JPRB, 0.87433E+00_JPRB, 0.88904E+00_JPRB, 0.89392E+00_JPRB, 0.89626E+00_JPRB, 0.88159E+00_JPRB, 0.88472E+00_JPRB ,&
&  0.87990E+00_JPRB, 0.87141E+00_JPRB, 0.84829E+00_JPRB, 0.84793E+00_JPRB, 0.84461E+00_JPRB, 0.83326E+00_JPRB, 0.84130E+00_JPRB ,&
&  0.82708E+00_JPRB, 0.79031E+00_JPRB, 0.72106E+00_JPRB, 0.64679E+00_JPRB, 0.60569E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.67692E+00_JPRB, 0.62532E+00_JPRB, 0.58621E+00_JPRB, 0.56581E+00_JPRB, 0.55340E+00_JPRB, 0.55001E+00_JPRB, 0.54991E+00_JPRB ,&
&  0.55552E+00_JPRB, 0.56147E+00_JPRB, 0.55226E+00_JPRB, 0.56079E+00_JPRB, 0.57912E+00_JPRB, 0.60297E+00_JPRB, 0.63363E+00_JPRB ,&
&  0.65772E+00_JPRB, 0.68488E+00_JPRB, 0.71599E+00_JPRB, 0.75890E+00_JPRB, 0.81294E+00_JPRB, 0.87332E+00_JPRB, 0.93245E+00_JPRB ,&
&  0.99461E+00_JPRB, 0.10667E+01_JPRB, 0.11718E+01_JPRB, 0.13232E+01_JPRB, 0.15367E+01_JPRB, 0.18577E+01_JPRB, 0.22308E+01_JPRB ,&
&  0.27619E+01_JPRB, 0.19758E+01_JPRB, 0.13184E+01_JPRB, 0.22495E+01_JPRB, 0.23586E+01_JPRB, 0.22132E+01_JPRB, 0.18328E+01_JPRB ,&
&  0.18109E+01_JPRB, 0.18038E+01_JPRB, 0.15956E+01_JPRB, 0.14499E+01_JPRB, 0.12725E+01_JPRB, 0.12298E+01_JPRB, 0.11889E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.11235E+01_JPRB, 0.11558E+01_JPRB, 0.11894E+01_JPRB, 0.12187E+01_JPRB, 0.13065E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.17527E+01_JPRB, 0.20980E+01_JPRB, 0.24862E+01_JPRB, 0.21220E+01_JPRB, 0.16920E+01_JPRB, 0.13696E+01_JPRB, 0.12449E+01_JPRB ,&
&  0.11702E+01_JPRB, 0.11352E+01_JPRB, 0.10756E+01_JPRB, 0.10112E+01_JPRB, 0.93467E+00_JPRB, 0.86296E+00_JPRB, 0.84984E+00_JPRB ,&
&  0.82311E+00_JPRB, 0.78358E+00_JPRB, 0.75388E+00_JPRB, 0.72639E+00_JPRB, 0.72228E+00_JPRB, 0.73213E+00_JPRB, 0.74535E+00_JPRB ,&
&  0.76094E+00_JPRB, 0.78075E+00_JPRB, 0.80949E+00_JPRB, 0.84219E+00_JPRB, 0.85437E+00_JPRB, 0.85491E+00_JPRB, 0.86131E+00_JPRB ,&
&  0.86803E+00_JPRB, 0.87570E+00_JPRB, 0.89903E+00_JPRB, 0.91295E+00_JPRB, 0.91456E+00_JPRB, 0.89863E+00_JPRB, 0.91458E+00_JPRB ,&
&  0.91476E+00_JPRB, 0.91893E+00_JPRB, 0.89932E+00_JPRB, 0.90789E+00_JPRB, 0.90229E+00_JPRB, 0.91682E+00_JPRB, 0.93300E+00_JPRB ,&
&  0.94578E+00_JPRB, 0.93178E+00_JPRB, 0.88516E+00_JPRB, 0.76702E+00_JPRB, 0.71831E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.72842E+00_JPRB, 0.69543E+00_JPRB, 0.67065E+00_JPRB, 0.64792E+00_JPRB, 0.62532E+00_JPRB, 0.60356E+00_JPRB, 0.58662E+00_JPRB ,&
&  0.56672E+00_JPRB, 0.56434E+00_JPRB, 0.56145E+00_JPRB, 0.55567E+00_JPRB, 0.54876E+00_JPRB, 0.55209E+00_JPRB, 0.56358E+00_JPRB ,&
&  0.57322E+00_JPRB, 0.58143E+00_JPRB, 0.59397E+00_JPRB, 0.60207E+00_JPRB, 0.61350E+00_JPRB, 0.63665E+00_JPRB, 0.67223E+00_JPRB ,&
&  0.72132E+00_JPRB, 0.77805E+00_JPRB, 0.85357E+00_JPRB, 0.94923E+00_JPRB, 0.10484E+01_JPRB, 0.12048E+01_JPRB, 0.13999E+01_JPRB ,&
&  0.18759E+01_JPRB, 0.19403E+01_JPRB, 0.10918E+01_JPRB, 0.19646E+01_JPRB, 0.25008E+01_JPRB, 0.23638E+01_JPRB, 0.21810E+01_JPRB ,&
&  0.21788E+01_JPRB, 0.20443E+01_JPRB, 0.18242E+01_JPRB, 0.15380E+01_JPRB, 0.12893E+01_JPRB, 0.12135E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11013E+01_JPRB, 0.10553E+01_JPRB, 0.10429E+01_JPRB, 0.10413E+01_JPRB, 0.10436E+01_JPRB, 0.10766E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.14125E+01_JPRB, 0.17364E+01_JPRB, 0.21012E+01_JPRB, 0.19781E+01_JPRB, 0.15796E+01_JPRB, 0.14005E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.12798E+01_JPRB, 0.12407E+01_JPRB, 0.11082E+01_JPRB, 0.10128E+01_JPRB, 0.93155E+00_JPRB, 0.87756E+00_JPRB, 0.87943E+00_JPRB ,&
&  0.86115E+00_JPRB, 0.83002E+00_JPRB, 0.80556E+00_JPRB, 0.77328E+00_JPRB, 0.75639E+00_JPRB, 0.74980E+00_JPRB, 0.75643E+00_JPRB ,&
&  0.76874E+00_JPRB, 0.78297E+00_JPRB, 0.80129E+00_JPRB, 0.82312E+00_JPRB, 0.83674E+00_JPRB, 0.84642E+00_JPRB, 0.85131E+00_JPRB ,&
&  0.86061E+00_JPRB, 0.87641E+00_JPRB, 0.89575E+00_JPRB, 0.90749E+00_JPRB, 0.91560E+00_JPRB, 0.91931E+00_JPRB, 0.91728E+00_JPRB ,&
&  0.92007E+00_JPRB, 0.91937E+00_JPRB, 0.87844E+00_JPRB, 0.87122E+00_JPRB, 0.90025E+00_JPRB, 0.90127E+00_JPRB, 0.89846E+00_JPRB ,&
&  0.92977E+00_JPRB, 0.95315E+00_JPRB, 0.93651E+00_JPRB, 0.81749E+00_JPRB, 0.76614E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.74529E+00_JPRB, 0.71938E+00_JPRB, 0.69895E+00_JPRB, 0.68341E+00_JPRB, 0.66484E+00_JPRB, 0.64618E+00_JPRB, 0.63082E+00_JPRB ,&
&  0.61042E+00_JPRB, 0.59818E+00_JPRB, 0.59001E+00_JPRB, 0.58091E+00_JPRB, 0.57151E+00_JPRB, 0.56258E+00_JPRB, 0.55554E+00_JPRB ,&
&  0.54943E+00_JPRB, 0.54468E+00_JPRB, 0.54210E+00_JPRB, 0.53648E+00_JPRB, 0.53191E+00_JPRB, 0.53293E+00_JPRB, 0.54213E+00_JPRB ,&
&  0.56845E+00_JPRB, 0.60854E+00_JPRB, 0.65968E+00_JPRB, 0.71795E+00_JPRB, 0.76958E+00_JPRB, 0.83955E+00_JPRB, 0.92508E+00_JPRB ,&
&  0.12866E+01_JPRB, 0.15154E+01_JPRB, 0.17891E+01_JPRB, 0.29208E+01_JPRB, 0.29874E+01_JPRB, 0.26680E+01_JPRB, 0.26430E+01_JPRB ,&
&  0.26998E+01_JPRB, 0.25123E+01_JPRB, 0.22788E+01_JPRB, 0.17752E+01_JPRB, 0.14171E+01_JPRB, 0.12272E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.10227E+01_JPRB, 0.97025E+00_JPRB, 0.95578E+00_JPRB, 0.94367E+00_JPRB, 0.93963E+00_JPRB, 0.98291E+00_JPRB, 0.10486E+01_JPRB ,&
&  0.11787E+01_JPRB, 0.14054E+01_JPRB, 0.17382E+01_JPRB, 0.19283E+01_JPRB, 0.15711E+01_JPRB, 0.14920E+01_JPRB, 0.15099E+01_JPRB ,&
&  0.17425E+01_JPRB, 0.14812E+01_JPRB, 0.11984E+01_JPRB, 0.10337E+01_JPRB, 0.92962E+00_JPRB, 0.89780E+00_JPRB, 0.87010E+00_JPRB ,&
&  0.85465E+00_JPRB, 0.83704E+00_JPRB, 0.82015E+00_JPRB, 0.80004E+00_JPRB, 0.78557E+00_JPRB, 0.77149E+00_JPRB, 0.76575E+00_JPRB ,&
&  0.76748E+00_JPRB, 0.77394E+00_JPRB, 0.78127E+00_JPRB, 0.79302E+00_JPRB, 0.80840E+00_JPRB, 0.82191E+00_JPRB, 0.83255E+00_JPRB ,&
&  0.84375E+00_JPRB, 0.86296E+00_JPRB, 0.85373E+00_JPRB, 0.86048E+00_JPRB, 0.87986E+00_JPRB, 0.88632E+00_JPRB, 0.86121E+00_JPRB ,&
&  0.84924E+00_JPRB, 0.86652E+00_JPRB, 0.86461E+00_JPRB, 0.85978E+00_JPRB, 0.86820E+00_JPRB, 0.87185E+00_JPRB, 0.83146E+00_JPRB ,&
&  0.82449E+00_JPRB, 0.82628E+00_JPRB, 0.81860E+00_JPRB, 0.79610E+00_JPRB, 0.77272E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.75334E+00_JPRB, 0.73310E+00_JPRB, 0.71220E+00_JPRB, 0.69185E+00_JPRB, 0.66921E+00_JPRB, 0.65188E+00_JPRB, 0.64241E+00_JPRB ,&
&  0.62787E+00_JPRB, 0.61526E+00_JPRB, 0.60620E+00_JPRB, 0.60186E+00_JPRB, 0.59810E+00_JPRB, 0.58718E+00_JPRB, 0.56748E+00_JPRB ,&
&  0.54802E+00_JPRB, 0.53481E+00_JPRB, 0.52689E+00_JPRB, 0.51889E+00_JPRB, 0.51086E+00_JPRB, 0.50693E+00_JPRB, 0.51013E+00_JPRB ,&
&  0.52064E+00_JPRB, 0.53820E+00_JPRB, 0.57177E+00_JPRB, 0.61141E+00_JPRB, 0.64755E+00_JPRB, 0.68291E+00_JPRB, 0.71017E+00_JPRB ,&
&  0.91224E+00_JPRB, 0.12862E+01_JPRB, 0.13026E+01_JPRB, 0.24825E+01_JPRB, 0.26615E+01_JPRB, 0.26114E+01_JPRB, 0.26328E+01_JPRB ,&
&  0.26562E+01_JPRB, 0.29207E+01_JPRB, 0.24847E+01_JPRB, 0.20176E+01_JPRB, 0.16168E+01_JPRB, 0.13537E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.10311E+01_JPRB, 0.94844E+00_JPRB, 0.90612E+00_JPRB, 0.89209E+00_JPRB, 0.88836E+00_JPRB, 0.90980E+00_JPRB, 0.97115E+00_JPRB ,&
&  0.10589E+01_JPRB, 0.12082E+01_JPRB, 0.14473E+01_JPRB, 0.15942E+01_JPRB, 0.15251E+01_JPRB, 0.15471E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.20767E+01_JPRB, 0.18784E+01_JPRB, 0.13658E+01_JPRB, 0.10845E+01_JPRB, 0.94636E+00_JPRB, 0.89493E+00_JPRB, 0.85438E+00_JPRB ,&
&  0.83713E+00_JPRB, 0.82048E+00_JPRB, 0.80694E+00_JPRB, 0.79678E+00_JPRB, 0.78654E+00_JPRB, 0.78087E+00_JPRB, 0.77588E+00_JPRB ,&
&  0.77017E+00_JPRB, 0.76958E+00_JPRB, 0.77025E+00_JPRB, 0.77207E+00_JPRB, 0.77597E+00_JPRB, 0.78640E+00_JPRB, 0.80223E+00_JPRB ,&
&  0.81859E+00_JPRB, 0.83771E+00_JPRB, 0.83079E+00_JPRB, 0.82085E+00_JPRB, 0.83079E+00_JPRB, 0.83303E+00_JPRB, 0.81487E+00_JPRB ,&
&  0.82138E+00_JPRB, 0.83992E+00_JPRB, 0.84195E+00_JPRB, 0.82562E+00_JPRB, 0.84277E+00_JPRB, 0.86994E+00_JPRB, 0.81098E+00_JPRB ,&
&  0.78989E+00_JPRB, 0.78298E+00_JPRB, 0.78371E+00_JPRB, 0.77445E+00_JPRB, 0.76458E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.73756E+00_JPRB, 0.72731E+00_JPRB, 0.71015E+00_JPRB, 0.69003E+00_JPRB, 0.66903E+00_JPRB, 0.65128E+00_JPRB, 0.64175E+00_JPRB ,&
&  0.63060E+00_JPRB, 0.61910E+00_JPRB, 0.60865E+00_JPRB, 0.60245E+00_JPRB, 0.59916E+00_JPRB, 0.58854E+00_JPRB, 0.57037E+00_JPRB ,&
&  0.55127E+00_JPRB, 0.53406E+00_JPRB, 0.51962E+00_JPRB, 0.51065E+00_JPRB, 0.50133E+00_JPRB, 0.49720E+00_JPRB, 0.49750E+00_JPRB ,&
&  0.50056E+00_JPRB, 0.51038E+00_JPRB, 0.53107E+00_JPRB, 0.56251E+00_JPRB, 0.59390E+00_JPRB, 0.62026E+00_JPRB, 0.63807E+00_JPRB ,&
&  0.75427E+00_JPRB, 0.15140E+01_JPRB, 0.12098E+01_JPRB, 0.19078E+01_JPRB, 0.20816E+01_JPRB, 0.22103E+01_JPRB, 0.23866E+01_JPRB ,&
&  0.24604E+01_JPRB, 0.25710E+01_JPRB, 0.24086E+01_JPRB, 0.21057E+01_JPRB, 0.17586E+01_JPRB, 0.15124E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.11708E+01_JPRB, 0.10667E+01_JPRB, 0.99297E+00_JPRB, 0.93892E+00_JPRB, 0.90515E+00_JPRB, 0.90512E+00_JPRB, 0.95710E+00_JPRB ,&
&  0.10560E+01_JPRB, 0.11627E+01_JPRB, 0.12960E+01_JPRB, 0.14598E+01_JPRB, 0.13852E+01_JPRB, 0.15176E+01_JPRB, 0.19095E+01_JPRB ,&
&  0.22470E+01_JPRB, 0.22905E+01_JPRB, 0.16437E+01_JPRB, 0.11935E+01_JPRB, 0.98947E+00_JPRB, 0.89196E+00_JPRB, 0.83308E+00_JPRB ,&
&  0.80101E+00_JPRB, 0.78454E+00_JPRB, 0.77471E+00_JPRB, 0.77194E+00_JPRB, 0.77277E+00_JPRB, 0.77598E+00_JPRB, 0.77487E+00_JPRB ,&
&  0.77259E+00_JPRB, 0.77077E+00_JPRB, 0.76894E+00_JPRB, 0.76613E+00_JPRB, 0.76098E+00_JPRB, 0.75942E+00_JPRB, 0.76616E+00_JPRB ,&
&  0.78094E+00_JPRB, 0.79948E+00_JPRB, 0.80133E+00_JPRB, 0.79490E+00_JPRB, 0.79656E+00_JPRB, 0.79677E+00_JPRB, 0.78756E+00_JPRB ,&
&  0.79907E+00_JPRB, 0.80631E+00_JPRB, 0.80783E+00_JPRB, 0.81533E+00_JPRB, 0.81996E+00_JPRB, 0.87744E+00_JPRB, 0.80360E+00_JPRB ,&
&  0.77657E+00_JPRB, 0.76423E+00_JPRB, 0.75790E+00_JPRB, 0.74639E+00_JPRB, 0.74102E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.70841E+00_JPRB, 0.69990E+00_JPRB, 0.68429E+00_JPRB, 0.66657E+00_JPRB, 0.65381E+00_JPRB, 0.64003E+00_JPRB, 0.62928E+00_JPRB ,&
&  0.61992E+00_JPRB, 0.61142E+00_JPRB, 0.60441E+00_JPRB, 0.59644E+00_JPRB, 0.58628E+00_JPRB, 0.57277E+00_JPRB, 0.55988E+00_JPRB ,&
&  0.54639E+00_JPRB, 0.53333E+00_JPRB, 0.52005E+00_JPRB, 0.50626E+00_JPRB, 0.49513E+00_JPRB, 0.48781E+00_JPRB, 0.48319E+00_JPRB ,&
&  0.48320E+00_JPRB, 0.48767E+00_JPRB, 0.50029E+00_JPRB, 0.52430E+00_JPRB, 0.55492E+00_JPRB, 0.58405E+00_JPRB, 0.61310E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.68312E+00_JPRB, 0.13708E+01_JPRB, 0.16322E+01_JPRB, 0.16047E+01_JPRB, 0.16498E+01_JPRB, 0.19792E+01_JPRB ,&
&  0.21826E+01_JPRB, 0.22638E+01_JPRB, 0.22639E+01_JPRB, 0.21270E+01_JPRB, 0.18636E+01_JPRB, 0.16669E+01_JPRB, 0.15137E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.13013E+01_JPRB, 0.11913E+01_JPRB, 0.10847E+01_JPRB, 0.10065E+01_JPRB, 0.98821E+00_JPRB, 0.10142E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.11842E+01_JPRB, 0.12326E+01_JPRB, 0.13185E+01_JPRB, 0.13107E+01_JPRB, 0.14612E+01_JPRB, 0.16704E+01_JPRB ,&
&  0.22153E+01_JPRB, 0.23221E+01_JPRB, 0.18147E+01_JPRB, 0.13731E+01_JPRB, 0.10925E+01_JPRB, 0.94269E+00_JPRB, 0.83921E+00_JPRB ,&
&  0.78481E+00_JPRB, 0.75607E+00_JPRB, 0.73931E+00_JPRB, 0.73752E+00_JPRB, 0.74921E+00_JPRB, 0.76618E+00_JPRB, 0.77853E+00_JPRB ,&
&  0.79069E+00_JPRB, 0.79782E+00_JPRB, 0.79461E+00_JPRB, 0.78677E+00_JPRB, 0.76954E+00_JPRB, 0.74679E+00_JPRB, 0.73393E+00_JPRB ,&
&  0.73651E+00_JPRB, 0.75216E+00_JPRB, 0.76510E+00_JPRB, 0.76027E+00_JPRB, 0.76286E+00_JPRB, 0.76610E+00_JPRB, 0.76644E+00_JPRB ,&
&  0.77437E+00_JPRB, 0.77720E+00_JPRB, 0.77738E+00_JPRB, 0.79308E+00_JPRB, 0.78876E+00_JPRB, 0.83539E+00_JPRB, 0.79537E+00_JPRB ,&
&  0.76679E+00_JPRB, 0.74747E+00_JPRB, 0.73367E+00_JPRB, 0.71977E+00_JPRB, 0.71036E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.68058E+00_JPRB, 0.67036E+00_JPRB, 0.65275E+00_JPRB, 0.63497E+00_JPRB, 0.62385E+00_JPRB, 0.61576E+00_JPRB, 0.61003E+00_JPRB ,&
&  0.60593E+00_JPRB, 0.59660E+00_JPRB, 0.58717E+00_JPRB, 0.57460E+00_JPRB, 0.55902E+00_JPRB, 0.54600E+00_JPRB, 0.53769E+00_JPRB ,&
&  0.52748E+00_JPRB, 0.52190E+00_JPRB, 0.51520E+00_JPRB, 0.50290E+00_JPRB, 0.48942E+00_JPRB, 0.47845E+00_JPRB, 0.47160E+00_JPRB ,&
&  0.46865E+00_JPRB, 0.47180E+00_JPRB, 0.48020E+00_JPRB, 0.49810E+00_JPRB, 0.51707E+00_JPRB, 0.53955E+00_JPRB, 0.57990E+00_JPRB ,&
&  0.73594E+00_JPRB, 0.12947E+01_JPRB, 0.16696E+01_JPRB, 0.14858E+01_JPRB, 0.13616E+01_JPRB, 0.12690E+01_JPRB, 0.14754E+01_JPRB ,&
&  0.16535E+01_JPRB, 0.18369E+01_JPRB, 0.19446E+01_JPRB, 0.19784E+01_JPRB, 0.18459E+01_JPRB, 0.17478E+01_JPRB, 0.16491E+01_JPRB ,&
&  0.15474E+01_JPRB, 0.14310E+01_JPRB, 0.12739E+01_JPRB, 0.11228E+01_JPRB, 0.10365E+01_JPRB, 0.10230E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.11353E+01_JPRB, 0.11926E+01_JPRB, 0.11992E+01_JPRB, 0.12606E+01_JPRB, 0.13690E+01_JPRB, 0.13616E+01_JPRB, 0.16851E+01_JPRB ,&
&  0.21259E+01_JPRB, 0.19466E+01_JPRB, 0.16705E+01_JPRB, 0.13768E+01_JPRB, 0.11481E+01_JPRB, 0.96790E+00_JPRB, 0.86438E+00_JPRB ,&
&  0.79370E+00_JPRB, 0.74872E+00_JPRB, 0.72024E+00_JPRB, 0.70910E+00_JPRB, 0.72402E+00_JPRB, 0.75510E+00_JPRB, 0.78182E+00_JPRB ,&
&  0.81442E+00_JPRB, 0.84529E+00_JPRB, 0.85928E+00_JPRB, 0.85186E+00_JPRB, 0.82357E+00_JPRB, 0.78347E+00_JPRB, 0.74548E+00_JPRB ,&
&  0.72393E+00_JPRB, 0.71809E+00_JPRB, 0.73907E+00_JPRB, 0.72645E+00_JPRB, 0.72964E+00_JPRB, 0.74196E+00_JPRB, 0.74210E+00_JPRB ,&
&  0.73979E+00_JPRB, 0.75201E+00_JPRB, 0.76603E+00_JPRB, 0.76765E+00_JPRB, 0.79900E+00_JPRB, 0.84445E+00_JPRB, 0.80402E+00_JPRB ,&
&  0.76684E+00_JPRB, 0.73614E+00_JPRB, 0.71690E+00_JPRB, 0.69271E+00_JPRB, 0.68268E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.66937E+00_JPRB, 0.65195E+00_JPRB, 0.62487E+00_JPRB, 0.60167E+00_JPRB, 0.58781E+00_JPRB, 0.57927E+00_JPRB, 0.57590E+00_JPRB ,&
&  0.57428E+00_JPRB, 0.56580E+00_JPRB, 0.55573E+00_JPRB, 0.54119E+00_JPRB, 0.52846E+00_JPRB, 0.51720E+00_JPRB, 0.50718E+00_JPRB ,&
&  0.49870E+00_JPRB, 0.49384E+00_JPRB, 0.48919E+00_JPRB, 0.48408E+00_JPRB, 0.47661E+00_JPRB, 0.46613E+00_JPRB, 0.45955E+00_JPRB ,&
&  0.45595E+00_JPRB, 0.45643E+00_JPRB, 0.46138E+00_JPRB, 0.47333E+00_JPRB, 0.48808E+00_JPRB, 0.50711E+00_JPRB, 0.53697E+00_JPRB ,&
&  0.63868E+00_JPRB, 0.10837E+01_JPRB, 0.11671E+01_JPRB, 0.11370E+01_JPRB, 0.11544E+01_JPRB, 0.11277E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.13290E+01_JPRB, 0.14537E+01_JPRB, 0.15653E+01_JPRB, 0.16065E+01_JPRB, 0.15432E+01_JPRB, 0.14806E+01_JPRB, 0.13965E+01_JPRB ,&
&  0.13052E+01_JPRB, 0.12053E+01_JPRB, 0.11027E+01_JPRB, 0.10072E+01_JPRB, 0.93276E+00_JPRB, 0.91567E+00_JPRB, 0.94080E+00_JPRB ,&
&  0.99283E+00_JPRB, 0.10299E+01_JPRB, 0.10498E+01_JPRB, 0.11098E+01_JPRB, 0.12598E+01_JPRB, 0.13540E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.15192E+01_JPRB, 0.14679E+01_JPRB, 0.13842E+01_JPRB, 0.12771E+01_JPRB, 0.11300E+01_JPRB, 0.98810E+00_JPRB, 0.87571E+00_JPRB ,&
&  0.79549E+00_JPRB, 0.74257E+00_JPRB, 0.71110E+00_JPRB, 0.69962E+00_JPRB, 0.70952E+00_JPRB, 0.73889E+00_JPRB, 0.77684E+00_JPRB ,&
&  0.82097E+00_JPRB, 0.86327E+00_JPRB, 0.88821E+00_JPRB, 0.89613E+00_JPRB, 0.88299E+00_JPRB, 0.85394E+00_JPRB, 0.81008E+00_JPRB ,&
&  0.76931E+00_JPRB, 0.73767E+00_JPRB, 0.72845E+00_JPRB, 0.72505E+00_JPRB, 0.72577E+00_JPRB, 0.72795E+00_JPRB, 0.73041E+00_JPRB ,&
&  0.73825E+00_JPRB, 0.76384E+00_JPRB, 0.76304E+00_JPRB, 0.76327E+00_JPRB, 0.78087E+00_JPRB, 0.80336E+00_JPRB, 0.77529E+00_JPRB ,&
&  0.74752E+00_JPRB, 0.71815E+00_JPRB, 0.69517E+00_JPRB, 0.67042E+00_JPRB, 0.67398E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.70215E+00_JPRB, 0.66638E+00_JPRB, 0.62220E+00_JPRB, 0.58851E+00_JPRB, 0.56547E+00_JPRB, 0.54963E+00_JPRB, 0.54156E+00_JPRB ,&
&  0.53908E+00_JPRB, 0.52840E+00_JPRB, 0.51166E+00_JPRB, 0.49735E+00_JPRB, 0.48974E+00_JPRB, 0.48395E+00_JPRB, 0.47633E+00_JPRB ,&
&  0.47011E+00_JPRB, 0.46451E+00_JPRB, 0.46023E+00_JPRB, 0.45849E+00_JPRB, 0.45762E+00_JPRB, 0.45572E+00_JPRB, 0.45059E+00_JPRB ,&
&  0.44312E+00_JPRB, 0.44062E+00_JPRB, 0.44198E+00_JPRB, 0.45031E+00_JPRB, 0.46316E+00_JPRB, 0.47965E+00_JPRB, 0.50617E+00_JPRB ,&
&  0.60961E+00_JPRB, 0.70492E+00_JPRB, 0.78681E+00_JPRB, 0.80490E+00_JPRB, 0.83924E+00_JPRB, 0.91690E+00_JPRB, 0.98536E+00_JPRB ,&
&  0.10498E+01_JPRB, 0.11393E+01_JPRB, 0.12534E+01_JPRB, 0.13067E+01_JPRB, 0.12629E+01_JPRB, 0.11828E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.99740E+00_JPRB, 0.95023E+00_JPRB, 0.90436E+00_JPRB, 0.87238E+00_JPRB, 0.86474E+00_JPRB, 0.87395E+00_JPRB ,&
&  0.88154E+00_JPRB, 0.86683E+00_JPRB, 0.89781E+00_JPRB, 0.94826E+00_JPRB, 0.10342E+01_JPRB, 0.10965E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.12136E+01_JPRB, 0.12148E+01_JPRB, 0.11888E+01_JPRB, 0.11143E+01_JPRB, 0.10235E+01_JPRB, 0.92925E+00_JPRB ,&
&  0.84141E+00_JPRB, 0.78373E+00_JPRB, 0.75411E+00_JPRB, 0.74087E+00_JPRB, 0.74008E+00_JPRB, 0.74763E+00_JPRB, 0.76476E+00_JPRB ,&
&  0.79048E+00_JPRB, 0.81472E+00_JPRB, 0.83446E+00_JPRB, 0.84323E+00_JPRB, 0.84012E+00_JPRB, 0.83374E+00_JPRB, 0.82412E+00_JPRB ,&
&  0.81138E+00_JPRB, 0.79456E+00_JPRB, 0.77712E+00_JPRB, 0.75953E+00_JPRB, 0.74465E+00_JPRB, 0.73468E+00_JPRB, 0.72747E+00_JPRB ,&
&  0.72073E+00_JPRB, 0.72884E+00_JPRB, 0.71916E+00_JPRB, 0.76334E+00_JPRB, 0.78956E+00_JPRB, 0.77126E+00_JPRB, 0.74565E+00_JPRB ,&
&  0.71391E+00_JPRB, 0.68549E+00_JPRB, 0.66025E+00_JPRB, 0.63733E+00_JPRB, 0.69034E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.60359E+00_JPRB, 0.59622E+00_JPRB, 0.57665E+00_JPRB, 0.55244E+00_JPRB, 0.53481E+00_JPRB, 0.51980E+00_JPRB, 0.50781E+00_JPRB ,&
&  0.50013E+00_JPRB, 0.49008E+00_JPRB, 0.47683E+00_JPRB, 0.46694E+00_JPRB, 0.46513E+00_JPRB, 0.46216E+00_JPRB, 0.45714E+00_JPRB ,&
&  0.44848E+00_JPRB, 0.44015E+00_JPRB, 0.43444E+00_JPRB, 0.43250E+00_JPRB, 0.43338E+00_JPRB, 0.43557E+00_JPRB, 0.43630E+00_JPRB ,&
&  0.43341E+00_JPRB, 0.42836E+00_JPRB, 0.42758E+00_JPRB, 0.43005E+00_JPRB, 0.43696E+00_JPRB, 0.44869E+00_JPRB, 0.47324E+00_JPRB ,&
&  0.54824E+00_JPRB, 0.55737E+00_JPRB, 0.59467E+00_JPRB, 0.61728E+00_JPRB, 0.65608E+00_JPRB, 0.72724E+00_JPRB, 0.80823E+00_JPRB ,&
&  0.85490E+00_JPRB, 0.90084E+00_JPRB, 0.97810E+00_JPRB, 0.10432E+01_JPRB, 0.10449E+01_JPRB, 0.10038E+01_JPRB, 0.94381E+00_JPRB ,&
&  0.88289E+00_JPRB, 0.83745E+00_JPRB, 0.79910E+00_JPRB, 0.76701E+00_JPRB, 0.74425E+00_JPRB, 0.73185E+00_JPRB, 0.72550E+00_JPRB ,&
&  0.71268E+00_JPRB, 0.71343E+00_JPRB, 0.73995E+00_JPRB, 0.78364E+00_JPRB, 0.83486E+00_JPRB, 0.88646E+00_JPRB, 0.94346E+00_JPRB ,&
&  0.99468E+00_JPRB, 0.10273E+01_JPRB, 0.10502E+01_JPRB, 0.10656E+01_JPRB, 0.10508E+01_JPRB, 0.10247E+01_JPRB, 0.96460E+00_JPRB ,&
&  0.89592E+00_JPRB, 0.83935E+00_JPRB, 0.80256E+00_JPRB, 0.77442E+00_JPRB, 0.75465E+00_JPRB, 0.74269E+00_JPRB, 0.73654E+00_JPRB ,&
&  0.73648E+00_JPRB, 0.74052E+00_JPRB, 0.74212E+00_JPRB, 0.74035E+00_JPRB, 0.73729E+00_JPRB, 0.73510E+00_JPRB, 0.73260E+00_JPRB ,&
&  0.73052E+00_JPRB, 0.72830E+00_JPRB, 0.72370E+00_JPRB, 0.71372E+00_JPRB, 0.69968E+00_JPRB, 0.68568E+00_JPRB, 0.67318E+00_JPRB ,&
&  0.66282E+00_JPRB, 0.66531E+00_JPRB, 0.66313E+00_JPRB, 0.68777E+00_JPRB, 0.69065E+00_JPRB, 0.68584E+00_JPRB, 0.67496E+00_JPRB ,&
&  0.65628E+00_JPRB, 0.63667E+00_JPRB, 0.60992E+00_JPRB, 0.58605E+00_JPRB, 0.58368E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.54179E+00_JPRB, 0.54294E+00_JPRB, 0.53795E+00_JPRB, 0.52154E+00_JPRB, 0.50463E+00_JPRB, 0.49468E+00_JPRB, 0.48394E+00_JPRB ,&
&  0.47264E+00_JPRB, 0.46344E+00_JPRB, 0.45097E+00_JPRB, 0.44486E+00_JPRB, 0.44220E+00_JPRB, 0.44129E+00_JPRB, 0.43878E+00_JPRB ,&
&  0.43275E+00_JPRB, 0.42382E+00_JPRB, 0.41706E+00_JPRB, 0.41431E+00_JPRB, 0.41521E+00_JPRB, 0.41740E+00_JPRB, 0.41914E+00_JPRB ,&
&  0.41757E+00_JPRB, 0.41612E+00_JPRB, 0.41434E+00_JPRB, 0.41333E+00_JPRB, 0.41473E+00_JPRB, 0.42406E+00_JPRB, 0.44270E+00_JPRB ,&
&  0.49889E+00_JPRB, 0.47025E+00_JPRB, 0.48857E+00_JPRB, 0.52387E+00_JPRB, 0.56287E+00_JPRB, 0.62258E+00_JPRB, 0.68872E+00_JPRB ,&
&  0.73148E+00_JPRB, 0.74267E+00_JPRB, 0.77158E+00_JPRB, 0.82175E+00_JPRB, 0.83081E+00_JPRB, 0.82537E+00_JPRB, 0.80183E+00_JPRB ,&
&  0.76217E+00_JPRB, 0.71334E+00_JPRB, 0.66596E+00_JPRB, 0.63174E+00_JPRB, 0.61469E+00_JPRB, 0.60549E+00_JPRB, 0.59808E+00_JPRB ,&
&  0.58362E+00_JPRB, 0.57914E+00_JPRB, 0.59271E+00_JPRB, 0.61207E+00_JPRB, 0.63552E+00_JPRB, 0.67538E+00_JPRB, 0.73139E+00_JPRB ,&
&  0.77619E+00_JPRB, 0.82426E+00_JPRB, 0.87053E+00_JPRB, 0.90551E+00_JPRB, 0.93367E+00_JPRB, 0.93715E+00_JPRB, 0.90838E+00_JPRB ,&
&  0.86765E+00_JPRB, 0.82209E+00_JPRB, 0.77967E+00_JPRB, 0.74134E+00_JPRB, 0.71081E+00_JPRB, 0.69259E+00_JPRB, 0.68186E+00_JPRB ,&
&  0.67455E+00_JPRB, 0.66885E+00_JPRB, 0.65864E+00_JPRB, 0.64409E+00_JPRB, 0.62938E+00_JPRB, 0.61702E+00_JPRB, 0.60600E+00_JPRB ,&
&  0.59623E+00_JPRB, 0.58636E+00_JPRB, 0.57978E+00_JPRB, 0.57774E+00_JPRB, 0.57539E+00_JPRB, 0.57464E+00_JPRB, 0.57518E+00_JPRB ,&
&  0.57413E+00_JPRB, 0.58009E+00_JPRB, 0.57663E+00_JPRB, 0.62368E+00_JPRB, 0.63507E+00_JPRB, 0.62699E+00_JPRB, 0.61681E+00_JPRB ,&
&  0.60053E+00_JPRB, 0.58389E+00_JPRB, 0.56732E+00_JPRB, 0.52052E+00_JPRB, 0.54088E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.49489E+00_JPRB, 0.49671E+00_JPRB, 0.49324E+00_JPRB, 0.48440E+00_JPRB, 0.47293E+00_JPRB, 0.46507E+00_JPRB, 0.45446E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.43718E+00_JPRB, 0.42999E+00_JPRB, 0.42612E+00_JPRB, 0.42582E+00_JPRB, 0.42174E+00_JPRB, 0.41809E+00_JPRB ,&
&  0.41472E+00_JPRB, 0.40871E+00_JPRB, 0.40478E+00_JPRB, 0.40163E+00_JPRB, 0.40148E+00_JPRB, 0.40200E+00_JPRB, 0.40176E+00_JPRB ,&
&  0.39934E+00_JPRB, 0.39561E+00_JPRB, 0.39242E+00_JPRB, 0.39027E+00_JPRB, 0.39180E+00_JPRB, 0.39902E+00_JPRB, 0.41343E+00_JPRB ,&
&  0.45309E+00_JPRB, 0.41307E+00_JPRB, 0.43464E+00_JPRB, 0.45893E+00_JPRB, 0.49641E+00_JPRB, 0.53330E+00_JPRB, 0.55702E+00_JPRB ,&
&  0.58454E+00_JPRB, 0.60990E+00_JPRB, 0.63486E+00_JPRB, 0.66136E+00_JPRB, 0.66533E+00_JPRB, 0.65724E+00_JPRB, 0.64719E+00_JPRB ,&
&  0.62992E+00_JPRB, 0.60705E+00_JPRB, 0.58329E+00_JPRB, 0.55854E+00_JPRB, 0.54208E+00_JPRB, 0.53171E+00_JPRB, 0.52011E+00_JPRB ,&
&  0.50528E+00_JPRB, 0.49204E+00_JPRB, 0.48378E+00_JPRB, 0.48336E+00_JPRB, 0.49279E+00_JPRB, 0.51263E+00_JPRB, 0.54993E+00_JPRB ,&
&  0.58925E+00_JPRB, 0.63281E+00_JPRB, 0.67957E+00_JPRB, 0.71693E+00_JPRB, 0.74440E+00_JPRB, 0.75466E+00_JPRB, 0.75318E+00_JPRB ,&
&  0.73918E+00_JPRB, 0.72054E+00_JPRB, 0.70297E+00_JPRB, 0.68141E+00_JPRB, 0.65478E+00_JPRB, 0.63050E+00_JPRB, 0.61144E+00_JPRB ,&
&  0.59678E+00_JPRB, 0.58157E+00_JPRB, 0.56454E+00_JPRB, 0.54608E+00_JPRB, 0.52691E+00_JPRB, 0.50981E+00_JPRB, 0.49551E+00_JPRB ,&
&  0.48343E+00_JPRB, 0.47296E+00_JPRB, 0.46701E+00_JPRB, 0.46697E+00_JPRB, 0.47159E+00_JPRB, 0.47762E+00_JPRB, 0.48251E+00_JPRB ,&
&  0.48819E+00_JPRB, 0.49770E+00_JPRB, 0.50280E+00_JPRB, 0.52528E+00_JPRB, 0.53460E+00_JPRB, 0.53727E+00_JPRB, 0.53873E+00_JPRB ,&
&  0.53586E+00_JPRB, 0.52697E+00_JPRB, 0.46637E+00_JPRB, 0.49161E+00_JPRB, 0.49415E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.45914E+00_JPRB, 0.45236E+00_JPRB, 0.44602E+00_JPRB, 0.44570E+00_JPRB, 0.44294E+00_JPRB, 0.43788E+00_JPRB, 0.43057E+00_JPRB ,&
&  0.41945E+00_JPRB, 0.41192E+00_JPRB, 0.40652E+00_JPRB, 0.40491E+00_JPRB, 0.40735E+00_JPRB, 0.40511E+00_JPRB, 0.40010E+00_JPRB ,&
&  0.39637E+00_JPRB, 0.39212E+00_JPRB, 0.38909E+00_JPRB, 0.38556E+00_JPRB, 0.38265E+00_JPRB, 0.38127E+00_JPRB, 0.38035E+00_JPRB ,&
&  0.37820E+00_JPRB, 0.37446E+00_JPRB, 0.37109E+00_JPRB, 0.36915E+00_JPRB, 0.37119E+00_JPRB, 0.37734E+00_JPRB, 0.38697E+00_JPRB ,&
&  0.38801E+00_JPRB, 0.37324E+00_JPRB, 0.39487E+00_JPRB, 0.42247E+00_JPRB, 0.44271E+00_JPRB, 0.46510E+00_JPRB, 0.48474E+00_JPRB ,&
&  0.50339E+00_JPRB, 0.51026E+00_JPRB, 0.51886E+00_JPRB, 0.52870E+00_JPRB, 0.53581E+00_JPRB, 0.53479E+00_JPRB, 0.53129E+00_JPRB ,&
&  0.52711E+00_JPRB, 0.52193E+00_JPRB, 0.51562E+00_JPRB, 0.50517E+00_JPRB, 0.49357E+00_JPRB, 0.48082E+00_JPRB, 0.46808E+00_JPRB ,&
&  0.45676E+00_JPRB, 0.44646E+00_JPRB, 0.43918E+00_JPRB, 0.43606E+00_JPRB, 0.43779E+00_JPRB, 0.44237E+00_JPRB, 0.45144E+00_JPRB ,&
&  0.46267E+00_JPRB, 0.48060E+00_JPRB, 0.51172E+00_JPRB, 0.54269E+00_JPRB, 0.56678E+00_JPRB, 0.58255E+00_JPRB, 0.59437E+00_JPRB ,&
&  0.59591E+00_JPRB, 0.59214E+00_JPRB, 0.58735E+00_JPRB, 0.57925E+00_JPRB, 0.56721E+00_JPRB, 0.55124E+00_JPRB, 0.53250E+00_JPRB ,&
&  0.51591E+00_JPRB, 0.50162E+00_JPRB, 0.48799E+00_JPRB, 0.47488E+00_JPRB, 0.46196E+00_JPRB, 0.44982E+00_JPRB, 0.43938E+00_JPRB ,&
&  0.43013E+00_JPRB, 0.42153E+00_JPRB, 0.41544E+00_JPRB, 0.41335E+00_JPRB, 0.41595E+00_JPRB, 0.42091E+00_JPRB, 0.42719E+00_JPRB ,&
&  0.43309E+00_JPRB, 0.44235E+00_JPRB, 0.45051E+00_JPRB, 0.46356E+00_JPRB, 0.47576E+00_JPRB, 0.47950E+00_JPRB, 0.48237E+00_JPRB ,&
&  0.48403E+00_JPRB, 0.47966E+00_JPRB, 0.47634E+00_JPRB, 0.46966E+00_JPRB, 0.46264E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.42065E+00_JPRB, 0.41247E+00_JPRB, 0.41012E+00_JPRB, 0.41269E+00_JPRB, 0.41609E+00_JPRB, 0.41493E+00_JPRB, 0.41014E+00_JPRB ,&
&  0.39989E+00_JPRB, 0.39077E+00_JPRB, 0.38365E+00_JPRB, 0.38055E+00_JPRB, 0.38240E+00_JPRB, 0.38264E+00_JPRB, 0.37970E+00_JPRB ,&
&  0.37646E+00_JPRB, 0.37064E+00_JPRB, 0.36559E+00_JPRB, 0.36052E+00_JPRB, 0.35672E+00_JPRB, 0.35495E+00_JPRB, 0.35414E+00_JPRB ,&
&  0.35250E+00_JPRB, 0.35036E+00_JPRB, 0.34948E+00_JPRB, 0.34900E+00_JPRB, 0.35051E+00_JPRB, 0.35484E+00_JPRB, 0.36097E+00_JPRB ,&
&  0.37321E+00_JPRB, 0.36099E+00_JPRB, 0.38292E+00_JPRB, 0.38857E+00_JPRB, 0.39704E+00_JPRB, 0.40935E+00_JPRB, 0.42598E+00_JPRB ,&
&  0.44104E+00_JPRB, 0.44700E+00_JPRB, 0.44985E+00_JPRB, 0.45184E+00_JPRB, 0.45258E+00_JPRB, 0.45546E+00_JPRB, 0.45443E+00_JPRB ,&
&  0.45037E+00_JPRB, 0.44816E+00_JPRB, 0.44691E+00_JPRB, 0.44439E+00_JPRB, 0.44181E+00_JPRB, 0.43696E+00_JPRB, 0.43064E+00_JPRB ,&
&  0.42413E+00_JPRB, 0.41758E+00_JPRB, 0.41187E+00_JPRB, 0.40633E+00_JPRB, 0.40175E+00_JPRB, 0.39909E+00_JPRB, 0.39867E+00_JPRB ,&
&  0.39939E+00_JPRB, 0.40295E+00_JPRB, 0.41307E+00_JPRB, 0.42765E+00_JPRB, 0.44326E+00_JPRB, 0.45690E+00_JPRB, 0.46799E+00_JPRB ,&
&  0.47445E+00_JPRB, 0.47727E+00_JPRB, 0.47650E+00_JPRB, 0.47359E+00_JPRB, 0.46812E+00_JPRB, 0.46152E+00_JPRB, 0.45256E+00_JPRB ,&
&  0.44414E+00_JPRB, 0.43650E+00_JPRB, 0.42849E+00_JPRB, 0.42092E+00_JPRB, 0.41270E+00_JPRB, 0.40390E+00_JPRB, 0.39599E+00_JPRB ,&
&  0.38984E+00_JPRB, 0.38548E+00_JPRB, 0.38232E+00_JPRB, 0.38064E+00_JPRB, 0.38032E+00_JPRB, 0.38234E+00_JPRB, 0.38512E+00_JPRB ,&
&  0.38879E+00_JPRB, 0.39444E+00_JPRB, 0.40135E+00_JPRB, 0.40920E+00_JPRB, 0.41927E+00_JPRB, 0.42589E+00_JPRB, 0.43118E+00_JPRB ,&
&  0.43412E+00_JPRB, 0.43328E+00_JPRB, 0.43297E+00_JPRB, 0.43042E+00_JPRB, 0.42681E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.38351E+00_JPRB, 0.37932E+00_JPRB, 0.38120E+00_JPRB, 0.38571E+00_JPRB, 0.39238E+00_JPRB, 0.39146E+00_JPRB, 0.38899E+00_JPRB ,&
&  0.38297E+00_JPRB, 0.37508E+00_JPRB, 0.36743E+00_JPRB, 0.36044E+00_JPRB, 0.35753E+00_JPRB, 0.35804E+00_JPRB, 0.35779E+00_JPRB ,&
&  0.35623E+00_JPRB, 0.35190E+00_JPRB, 0.34593E+00_JPRB, 0.34021E+00_JPRB, 0.33607E+00_JPRB, 0.33373E+00_JPRB, 0.33186E+00_JPRB ,&
&  0.33076E+00_JPRB, 0.32902E+00_JPRB, 0.32970E+00_JPRB, 0.32958E+00_JPRB, 0.33018E+00_JPRB, 0.33389E+00_JPRB, 0.33802E+00_JPRB ,&
&  0.34189E+00_JPRB, 0.35956E+00_JPRB, 0.35534E+00_JPRB, 0.35363E+00_JPRB, 0.35779E+00_JPRB, 0.36708E+00_JPRB, 0.38142E+00_JPRB ,&
&  0.39553E+00_JPRB, 0.40263E+00_JPRB, 0.40491E+00_JPRB, 0.40232E+00_JPRB, 0.39918E+00_JPRB, 0.39811E+00_JPRB, 0.39925E+00_JPRB ,&
&  0.39926E+00_JPRB, 0.39581E+00_JPRB, 0.39092E+00_JPRB, 0.38667E+00_JPRB, 0.38498E+00_JPRB, 0.38380E+00_JPRB, 0.38189E+00_JPRB ,&
&  0.37907E+00_JPRB, 0.37552E+00_JPRB, 0.37026E+00_JPRB, 0.36428E+00_JPRB, 0.35862E+00_JPRB, 0.35500E+00_JPRB, 0.35389E+00_JPRB ,&
&  0.35494E+00_JPRB, 0.35771E+00_JPRB, 0.36249E+00_JPRB, 0.36900E+00_JPRB, 0.37565E+00_JPRB, 0.38304E+00_JPRB, 0.38899E+00_JPRB ,&
&  0.39306E+00_JPRB, 0.39488E+00_JPRB, 0.39491E+00_JPRB, 0.39387E+00_JPRB, 0.39192E+00_JPRB, 0.38913E+00_JPRB, 0.38606E+00_JPRB ,&
&  0.38307E+00_JPRB, 0.38002E+00_JPRB, 0.37681E+00_JPRB, 0.37333E+00_JPRB, 0.36850E+00_JPRB, 0.36335E+00_JPRB, 0.35944E+00_JPRB ,&
&  0.35706E+00_JPRB, 0.35562E+00_JPRB, 0.35461E+00_JPRB, 0.35351E+00_JPRB, 0.35299E+00_JPRB, 0.35364E+00_JPRB, 0.35551E+00_JPRB ,&
&  0.35840E+00_JPRB, 0.36236E+00_JPRB, 0.36725E+00_JPRB, 0.37244E+00_JPRB, 0.37758E+00_JPRB, 0.38296E+00_JPRB, 0.38840E+00_JPRB ,&
&  0.39185E+00_JPRB, 0.39189E+00_JPRB, 0.39189E+00_JPRB, 0.39173E+00_JPRB, 0.38877E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.35209E+00_JPRB, 0.35187E+00_JPRB, 0.35662E+00_JPRB, 0.36178E+00_JPRB, 0.36763E+00_JPRB, 0.36955E+00_JPRB, 0.36717E+00_JPRB ,&
&  0.36405E+00_JPRB, 0.35990E+00_JPRB, 0.35366E+00_JPRB, 0.34596E+00_JPRB, 0.33903E+00_JPRB, 0.33713E+00_JPRB, 0.33726E+00_JPRB ,&
&  0.33743E+00_JPRB, 0.33514E+00_JPRB, 0.33096E+00_JPRB, 0.32636E+00_JPRB, 0.32118E+00_JPRB, 0.31658E+00_JPRB, 0.31307E+00_JPRB ,&
&  0.31188E+00_JPRB, 0.31124E+00_JPRB, 0.31184E+00_JPRB, 0.31245E+00_JPRB, 0.31335E+00_JPRB, 0.31597E+00_JPRB, 0.32238E+00_JPRB ,&
&  0.32195E+00_JPRB, 0.32955E+00_JPRB, 0.31966E+00_JPRB, 0.32263E+00_JPRB, 0.32599E+00_JPRB, 0.33141E+00_JPRB, 0.33933E+00_JPRB ,&
&  0.34893E+00_JPRB, 0.35577E+00_JPRB, 0.35980E+00_JPRB, 0.35960E+00_JPRB, 0.35737E+00_JPRB, 0.35747E+00_JPRB, 0.35825E+00_JPRB ,&
&  0.35856E+00_JPRB, 0.35823E+00_JPRB, 0.35603E+00_JPRB, 0.35125E+00_JPRB, 0.34473E+00_JPRB, 0.33912E+00_JPRB, 0.33509E+00_JPRB ,&
&  0.33393E+00_JPRB, 0.33296E+00_JPRB, 0.33068E+00_JPRB, 0.32803E+00_JPRB, 0.32524E+00_JPRB, 0.32335E+00_JPRB, 0.32334E+00_JPRB ,&
&  0.32437E+00_JPRB, 0.32531E+00_JPRB, 0.32603E+00_JPRB, 0.32814E+00_JPRB, 0.33187E+00_JPRB, 0.33571E+00_JPRB, 0.33920E+00_JPRB ,&
&  0.34157E+00_JPRB, 0.34192E+00_JPRB, 0.34135E+00_JPRB, 0.34021E+00_JPRB, 0.33936E+00_JPRB, 0.33860E+00_JPRB, 0.33849E+00_JPRB ,&
&  0.33825E+00_JPRB, 0.33745E+00_JPRB, 0.33575E+00_JPRB, 0.33323E+00_JPRB, 0.33083E+00_JPRB, 0.32880E+00_JPRB, 0.32772E+00_JPRB ,&
&  0.32752E+00_JPRB, 0.32760E+00_JPRB, 0.32725E+00_JPRB, 0.32666E+00_JPRB, 0.32695E+00_JPRB, 0.32764E+00_JPRB, 0.32911E+00_JPRB ,&
&  0.33117E+00_JPRB, 0.33374E+00_JPRB, 0.33712E+00_JPRB, 0.34043E+00_JPRB, 0.34116E+00_JPRB, 0.34267E+00_JPRB, 0.34653E+00_JPRB ,&
&  0.35033E+00_JPRB, 0.35224E+00_JPRB, 0.35306E+00_JPRB, 0.35355E+00_JPRB, 0.35378E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.32738E+00_JPRB, 0.32971E+00_JPRB, 0.33390E+00_JPRB, 0.34005E+00_JPRB, 0.34382E+00_JPRB, 0.34797E+00_JPRB, 0.34801E+00_JPRB ,&
&  0.34539E+00_JPRB, 0.34169E+00_JPRB, 0.33718E+00_JPRB, 0.33145E+00_JPRB, 0.32540E+00_JPRB, 0.32088E+00_JPRB, 0.31926E+00_JPRB ,&
&  0.31904E+00_JPRB, 0.31765E+00_JPRB, 0.31446E+00_JPRB, 0.31025E+00_JPRB, 0.30556E+00_JPRB, 0.30164E+00_JPRB, 0.29856E+00_JPRB ,&
&  0.29757E+00_JPRB, 0.29800E+00_JPRB, 0.29918E+00_JPRB, 0.29992E+00_JPRB, 0.30009E+00_JPRB, 0.30255E+00_JPRB, 0.30736E+00_JPRB ,&
&  0.30622E+00_JPRB, 0.30479E+00_JPRB, 0.29741E+00_JPRB, 0.29677E+00_JPRB, 0.29870E+00_JPRB, 0.30153E+00_JPRB, 0.30527E+00_JPRB ,&
&  0.30974E+00_JPRB, 0.31402E+00_JPRB, 0.31838E+00_JPRB, 0.32180E+00_JPRB, 0.32404E+00_JPRB, 0.32584E+00_JPRB, 0.32700E+00_JPRB ,&
&  0.32763E+00_JPRB, 0.32684E+00_JPRB, 0.32458E+00_JPRB, 0.32049E+00_JPRB, 0.31530E+00_JPRB, 0.31164E+00_JPRB, 0.30991E+00_JPRB ,&
&  0.30824E+00_JPRB, 0.30780E+00_JPRB, 0.30728E+00_JPRB, 0.30599E+00_JPRB, 0.30427E+00_JPRB, 0.30346E+00_JPRB, 0.30369E+00_JPRB ,&
&  0.30382E+00_JPRB, 0.30303E+00_JPRB, 0.30154E+00_JPRB, 0.30112E+00_JPRB, 0.30218E+00_JPRB, 0.30422E+00_JPRB, 0.30604E+00_JPRB ,&
&  0.30736E+00_JPRB, 0.30715E+00_JPRB, 0.30625E+00_JPRB, 0.30430E+00_JPRB, 0.30290E+00_JPRB, 0.30302E+00_JPRB, 0.30435E+00_JPRB ,&
&  0.30523E+00_JPRB, 0.30518E+00_JPRB, 0.30517E+00_JPRB, 0.30372E+00_JPRB, 0.30225E+00_JPRB, 0.30196E+00_JPRB, 0.30214E+00_JPRB ,&
&  0.30196E+00_JPRB, 0.30156E+00_JPRB, 0.30116E+00_JPRB, 0.30109E+00_JPRB, 0.30169E+00_JPRB, 0.30188E+00_JPRB, 0.30288E+00_JPRB ,&
&  0.30552E+00_JPRB, 0.30859E+00_JPRB, 0.31135E+00_JPRB, 0.31302E+00_JPRB, 0.31371E+00_JPRB, 0.31388E+00_JPRB, 0.31663E+00_JPRB ,&
&  0.32062E+00_JPRB, 0.32366E+00_JPRB, 0.32408E+00_JPRB, 0.32388E+00_JPRB, 0.32541E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.30685E+00_JPRB, 0.30972E+00_JPRB, 0.31431E+00_JPRB, 0.31933E+00_JPRB, 0.32253E+00_JPRB, 0.32734E+00_JPRB, 0.32837E+00_JPRB ,&
&  0.32773E+00_JPRB, 0.32589E+00_JPRB, 0.32158E+00_JPRB, 0.31627E+00_JPRB, 0.31111E+00_JPRB, 0.30675E+00_JPRB, 0.30370E+00_JPRB ,&
&  0.30192E+00_JPRB, 0.30031E+00_JPRB, 0.29840E+00_JPRB, 0.29551E+00_JPRB, 0.29205E+00_JPRB, 0.28890E+00_JPRB, 0.28645E+00_JPRB ,&
&  0.28563E+00_JPRB, 0.28569E+00_JPRB, 0.28618E+00_JPRB, 0.28621E+00_JPRB, 0.28594E+00_JPRB, 0.28728E+00_JPRB, 0.28933E+00_JPRB ,&
&  0.28848E+00_JPRB, 0.28424E+00_JPRB, 0.28091E+00_JPRB, 0.28057E+00_JPRB, 0.28068E+00_JPRB, 0.28119E+00_JPRB, 0.28232E+00_JPRB ,&
&  0.28348E+00_JPRB, 0.28458E+00_JPRB, 0.28680E+00_JPRB, 0.28946E+00_JPRB, 0.29197E+00_JPRB, 0.29536E+00_JPRB, 0.29716E+00_JPRB ,&
&  0.29753E+00_JPRB, 0.29674E+00_JPRB, 0.29512E+00_JPRB, 0.29248E+00_JPRB, 0.29064E+00_JPRB, 0.29013E+00_JPRB, 0.28933E+00_JPRB ,&
&  0.28992E+00_JPRB, 0.29049E+00_JPRB, 0.29088E+00_JPRB, 0.28941E+00_JPRB, 0.28700E+00_JPRB, 0.28638E+00_JPRB, 0.28664E+00_JPRB ,&
&  0.28647E+00_JPRB, 0.28509E+00_JPRB, 0.28276E+00_JPRB, 0.28122E+00_JPRB, 0.28002E+00_JPRB, 0.27979E+00_JPRB, 0.27965E+00_JPRB ,&
&  0.27932E+00_JPRB, 0.27810E+00_JPRB, 0.27730E+00_JPRB, 0.27701E+00_JPRB, 0.27695E+00_JPRB, 0.27745E+00_JPRB, 0.27727E+00_JPRB ,&
&  0.27765E+00_JPRB, 0.27802E+00_JPRB, 0.27696E+00_JPRB, 0.27634E+00_JPRB, 0.27590E+00_JPRB, 0.27598E+00_JPRB, 0.27590E+00_JPRB ,&
&  0.27544E+00_JPRB, 0.27489E+00_JPRB, 0.27466E+00_JPRB, 0.27495E+00_JPRB, 0.27455E+00_JPRB, 0.27362E+00_JPRB, 0.27445E+00_JPRB ,&
&  0.27786E+00_JPRB, 0.28151E+00_JPRB, 0.28459E+00_JPRB, 0.28647E+00_JPRB, 0.28703E+00_JPRB, 0.28844E+00_JPRB, 0.29147E+00_JPRB ,&
&  0.29423E+00_JPRB, 0.29582E+00_JPRB, 0.29863E+00_JPRB, 0.30086E+00_JPRB, 0.30320E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.28892E+00_JPRB, 0.29286E+00_JPRB, 0.29587E+00_JPRB, 0.29956E+00_JPRB, 0.30247E+00_JPRB, 0.30747E+00_JPRB, 0.30962E+00_JPRB ,&
&  0.30964E+00_JPRB, 0.30942E+00_JPRB, 0.30790E+00_JPRB, 0.30519E+00_JPRB, 0.30096E+00_JPRB, 0.29655E+00_JPRB, 0.29234E+00_JPRB ,&
&  0.28905E+00_JPRB, 0.28634E+00_JPRB, 0.28418E+00_JPRB, 0.28223E+00_JPRB, 0.27959E+00_JPRB, 0.27714E+00_JPRB, 0.27529E+00_JPRB ,&
&  0.27456E+00_JPRB, 0.27406E+00_JPRB, 0.27389E+00_JPRB, 0.27393E+00_JPRB, 0.27363E+00_JPRB, 0.27321E+00_JPRB, 0.27322E+00_JPRB ,&
&  0.27192E+00_JPRB, 0.26819E+00_JPRB, 0.26596E+00_JPRB, 0.26520E+00_JPRB, 0.26451E+00_JPRB, 0.26502E+00_JPRB, 0.26485E+00_JPRB ,&
&  0.26487E+00_JPRB, 0.26527E+00_JPRB, 0.26713E+00_JPRB, 0.26962E+00_JPRB, 0.27050E+00_JPRB, 0.27187E+00_JPRB, 0.27389E+00_JPRB ,&
&  0.27481E+00_JPRB, 0.27406E+00_JPRB, 0.27210E+00_JPRB, 0.27166E+00_JPRB, 0.27265E+00_JPRB, 0.27279E+00_JPRB, 0.27315E+00_JPRB ,&
&  0.27372E+00_JPRB, 0.27473E+00_JPRB, 0.27512E+00_JPRB, 0.27301E+00_JPRB, 0.27091E+00_JPRB, 0.27011E+00_JPRB, 0.26998E+00_JPRB ,&
&  0.26956E+00_JPRB, 0.26833E+00_JPRB, 0.26585E+00_JPRB, 0.26345E+00_JPRB, 0.26017E+00_JPRB, 0.25752E+00_JPRB, 0.25613E+00_JPRB ,&
&  0.25647E+00_JPRB, 0.25692E+00_JPRB, 0.25826E+00_JPRB, 0.25944E+00_JPRB, 0.26019E+00_JPRB, 0.26023E+00_JPRB, 0.25867E+00_JPRB ,&
&  0.25663E+00_JPRB, 0.25568E+00_JPRB, 0.25449E+00_JPRB, 0.25328E+00_JPRB, 0.25347E+00_JPRB, 0.25330E+00_JPRB, 0.25292E+00_JPRB ,&
&  0.25257E+00_JPRB, 0.25324E+00_JPRB, 0.25390E+00_JPRB, 0.25362E+00_JPRB, 0.25275E+00_JPRB, 0.25173E+00_JPRB, 0.25233E+00_JPRB ,&
&  0.25399E+00_JPRB, 0.25640E+00_JPRB, 0.25910E+00_JPRB, 0.26193E+00_JPRB, 0.26478E+00_JPRB, 0.26737E+00_JPRB, 0.27009E+00_JPRB ,&
&  0.27345E+00_JPRB, 0.27476E+00_JPRB, 0.27721E+00_JPRB, 0.28078E+00_JPRB, 0.28420E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.26844E+00_JPRB, 0.27384E+00_JPRB, 0.27694E+00_JPRB, 0.27961E+00_JPRB, 0.28237E+00_JPRB, 0.28715E+00_JPRB, 0.29048E+00_JPRB ,&
&  0.29124E+00_JPRB, 0.29141E+00_JPRB, 0.29113E+00_JPRB, 0.29009E+00_JPRB, 0.28829E+00_JPRB, 0.28651E+00_JPRB, 0.28351E+00_JPRB ,&
&  0.27973E+00_JPRB, 0.27622E+00_JPRB, 0.27308E+00_JPRB, 0.27014E+00_JPRB, 0.26744E+00_JPRB, 0.26501E+00_JPRB, 0.26321E+00_JPRB ,&
&  0.26285E+00_JPRB, 0.26292E+00_JPRB, 0.26223E+00_JPRB, 0.26219E+00_JPRB, 0.26218E+00_JPRB, 0.26124E+00_JPRB, 0.25959E+00_JPRB ,&
&  0.25746E+00_JPRB, 0.25402E+00_JPRB, 0.25049E+00_JPRB, 0.24960E+00_JPRB, 0.24945E+00_JPRB, 0.24863E+00_JPRB, 0.24746E+00_JPRB ,&
&  0.24880E+00_JPRB, 0.25098E+00_JPRB, 0.25362E+00_JPRB, 0.25514E+00_JPRB, 0.25619E+00_JPRB, 0.25795E+00_JPRB, 0.26016E+00_JPRB ,&
&  0.26074E+00_JPRB, 0.25807E+00_JPRB, 0.25554E+00_JPRB, 0.25523E+00_JPRB, 0.25532E+00_JPRB, 0.25499E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.25608E+00_JPRB, 0.25760E+00_JPRB, 0.25852E+00_JPRB, 0.25725E+00_JPRB, 0.25505E+00_JPRB, 0.25345E+00_JPRB, 0.25242E+00_JPRB ,&
&  0.25185E+00_JPRB, 0.25119E+00_JPRB, 0.25007E+00_JPRB, 0.24829E+00_JPRB, 0.24625E+00_JPRB, 0.24417E+00_JPRB, 0.24200E+00_JPRB ,&
&  0.24011E+00_JPRB, 0.24015E+00_JPRB, 0.24275E+00_JPRB, 0.24436E+00_JPRB, 0.24393E+00_JPRB, 0.24254E+00_JPRB, 0.24085E+00_JPRB ,&
&  0.23972E+00_JPRB, 0.23698E+00_JPRB, 0.23462E+00_JPRB, 0.23280E+00_JPRB, 0.23162E+00_JPRB, 0.23002E+00_JPRB, 0.22953E+00_JPRB ,&
&  0.23017E+00_JPRB, 0.23230E+00_JPRB, 0.23298E+00_JPRB, 0.23295E+00_JPRB, 0.23245E+00_JPRB, 0.23206E+00_JPRB, 0.23311E+00_JPRB ,&
&  0.23457E+00_JPRB, 0.23696E+00_JPRB, 0.23964E+00_JPRB, 0.24319E+00_JPRB, 0.24626E+00_JPRB, 0.24995E+00_JPRB, 0.25294E+00_JPRB ,&
&  0.25477E+00_JPRB, 0.25533E+00_JPRB, 0.25670E+00_JPRB, 0.25895E+00_JPRB, 0.26261E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.24716E+00_JPRB, 0.25310E+00_JPRB, 0.25819E+00_JPRB, 0.26095E+00_JPRB, 0.26272E+00_JPRB, 0.26535E+00_JPRB, 0.26798E+00_JPRB ,&
&  0.26995E+00_JPRB, 0.27119E+00_JPRB, 0.27186E+00_JPRB, 0.27189E+00_JPRB, 0.27107E+00_JPRB, 0.26958E+00_JPRB, 0.26776E+00_JPRB ,&
&  0.26570E+00_JPRB, 0.26275E+00_JPRB, 0.26000E+00_JPRB, 0.25793E+00_JPRB, 0.25624E+00_JPRB, 0.25434E+00_JPRB, 0.25286E+00_JPRB ,&
&  0.25144E+00_JPRB, 0.25118E+00_JPRB, 0.25069E+00_JPRB, 0.25000E+00_JPRB, 0.24925E+00_JPRB, 0.24807E+00_JPRB, 0.24711E+00_JPRB ,&
&  0.24435E+00_JPRB, 0.23983E+00_JPRB, 0.23819E+00_JPRB, 0.23693E+00_JPRB, 0.23711E+00_JPRB, 0.23648E+00_JPRB, 0.23557E+00_JPRB ,&
&  0.23729E+00_JPRB, 0.24043E+00_JPRB, 0.24291E+00_JPRB, 0.24423E+00_JPRB, 0.24517E+00_JPRB, 0.24670E+00_JPRB, 0.24841E+00_JPRB ,&
&  0.24627E+00_JPRB, 0.24333E+00_JPRB, 0.24114E+00_JPRB, 0.24015E+00_JPRB, 0.24012E+00_JPRB, 0.24106E+00_JPRB, 0.24293E+00_JPRB ,&
&  0.24458E+00_JPRB, 0.24550E+00_JPRB, 0.24506E+00_JPRB, 0.24344E+00_JPRB, 0.24147E+00_JPRB, 0.24056E+00_JPRB, 0.24006E+00_JPRB ,&
&  0.23977E+00_JPRB, 0.23924E+00_JPRB, 0.23859E+00_JPRB, 0.23712E+00_JPRB, 0.23031E+00_JPRB, 0.22703E+00_JPRB, 0.22438E+00_JPRB ,&
&  0.22516E+00_JPRB, 0.22430E+00_JPRB, 0.22121E+00_JPRB, 0.22777E+00_JPRB, 0.23468E+00_JPRB, 0.22911E+00_JPRB, 0.22276E+00_JPRB ,&
&  0.21400E+00_JPRB, 0.21627E+00_JPRB, 0.21357E+00_JPRB, 0.20818E+00_JPRB, 0.20690E+00_JPRB, 0.20734E+00_JPRB, 0.20701E+00_JPRB ,&
&  0.20799E+00_JPRB, 0.20794E+00_JPRB, 0.20919E+00_JPRB, 0.20975E+00_JPRB, 0.21011E+00_JPRB, 0.21128E+00_JPRB, 0.21240E+00_JPRB ,&
&  0.21511E+00_JPRB, 0.21951E+00_JPRB, 0.22424E+00_JPRB, 0.22498E+00_JPRB, 0.22833E+00_JPRB, 0.23110E+00_JPRB, 0.23279E+00_JPRB ,&
&  0.23413E+00_JPRB, 0.23616E+00_JPRB, 0.23802E+00_JPRB, 0.23966E+00_JPRB, 0.24243E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.23257E+00_JPRB, 0.23518E+00_JPRB, 0.23833E+00_JPRB, 0.24147E+00_JPRB, 0.24375E+00_JPRB, 0.24605E+00_JPRB, 0.24810E+00_JPRB ,&
&  0.24988E+00_JPRB, 0.25110E+00_JPRB, 0.25194E+00_JPRB, 0.25247E+00_JPRB, 0.25285E+00_JPRB, 0.25264E+00_JPRB, 0.25200E+00_JPRB ,&
&  0.25133E+00_JPRB, 0.25010E+00_JPRB, 0.24839E+00_JPRB, 0.24698E+00_JPRB, 0.24602E+00_JPRB, 0.24475E+00_JPRB, 0.24295E+00_JPRB ,&
&  0.24112E+00_JPRB, 0.23949E+00_JPRB, 0.23814E+00_JPRB, 0.23672E+00_JPRB, 0.23655E+00_JPRB, 0.23715E+00_JPRB, 0.23771E+00_JPRB ,&
&  0.23230E+00_JPRB, 0.22685E+00_JPRB, 0.22870E+00_JPRB, 0.22977E+00_JPRB, 0.22636E+00_JPRB, 0.22776E+00_JPRB, 0.22859E+00_JPRB ,&
&  0.22969E+00_JPRB, 0.23127E+00_JPRB, 0.23288E+00_JPRB, 0.23366E+00_JPRB, 0.23424E+00_JPRB, 0.23518E+00_JPRB, 0.23481E+00_JPRB ,&
&  0.23279E+00_JPRB, 0.23098E+00_JPRB, 0.22931E+00_JPRB, 0.22755E+00_JPRB, 0.22710E+00_JPRB, 0.22897E+00_JPRB, 0.23014E+00_JPRB ,&
&  0.22904E+00_JPRB, 0.22733E+00_JPRB, 0.22569E+00_JPRB, 0.22603E+00_JPRB, 0.22691E+00_JPRB, 0.22733E+00_JPRB, 0.22484E+00_JPRB ,&
&  0.22398E+00_JPRB, 0.22095E+00_JPRB, 0.21824E+00_JPRB, 0.21554E+00_JPRB, 0.21444E+00_JPRB, 0.21059E+00_JPRB, 0.20636E+00_JPRB ,&
&  0.20559E+00_JPRB, 0.21022E+00_JPRB, 0.21684E+00_JPRB, 0.21814E+00_JPRB, 0.20351E+00_JPRB, 0.20611E+00_JPRB, 0.20730E+00_JPRB ,&
&  0.20251E+00_JPRB, 0.19665E+00_JPRB, 0.19237E+00_JPRB, 0.19053E+00_JPRB, 0.18990E+00_JPRB, 0.18935E+00_JPRB, 0.19002E+00_JPRB ,&
&  0.19120E+00_JPRB, 0.19178E+00_JPRB, 0.19243E+00_JPRB, 0.19386E+00_JPRB, 0.19525E+00_JPRB, 0.19670E+00_JPRB, 0.19802E+00_JPRB ,&
&  0.20091E+00_JPRB, 0.20434E+00_JPRB, 0.20685E+00_JPRB, 0.21006E+00_JPRB, 0.21346E+00_JPRB, 0.21676E+00_JPRB, 0.21976E+00_JPRB ,&
&  0.22301E+00_JPRB, 0.22437E+00_JPRB, 0.22615E+00_JPRB, 0.22941E+00_JPRB, 0.23100E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.22560E+00_JPRB, 0.22863E+00_JPRB, 0.23062E+00_JPRB, 0.23254E+00_JPRB, 0.23403E+00_JPRB, 0.23563E+00_JPRB, 0.23641E+00_JPRB ,&
&  0.23717E+00_JPRB, 0.23725E+00_JPRB, 0.23780E+00_JPRB, 0.23786E+00_JPRB, 0.23816E+00_JPRB, 0.23699E+00_JPRB, 0.23653E+00_JPRB ,&
&  0.23612E+00_JPRB, 0.23663E+00_JPRB, 0.23671E+00_JPRB, 0.23660E+00_JPRB, 0.23573E+00_JPRB, 0.23459E+00_JPRB, 0.23283E+00_JPRB ,&
&  0.22935E+00_JPRB, 0.22675E+00_JPRB, 0.22688E+00_JPRB, 0.22622E+00_JPRB, 0.22663E+00_JPRB, 0.22653E+00_JPRB, 0.22701E+00_JPRB ,&
&  0.22206E+00_JPRB, 0.22064E+00_JPRB, 0.22160E+00_JPRB, 0.22030E+00_JPRB, 0.22167E+00_JPRB, 0.22239E+00_JPRB, 0.22441E+00_JPRB ,&
&  0.22491E+00_JPRB, 0.22525E+00_JPRB, 0.22623E+00_JPRB, 0.22574E+00_JPRB, 0.22615E+00_JPRB, 0.22690E+00_JPRB, 0.22584E+00_JPRB ,&
&  0.22236E+00_JPRB, 0.21973E+00_JPRB, 0.21591E+00_JPRB, 0.20709E+00_JPRB, 0.20561E+00_JPRB, 0.20645E+00_JPRB, 0.20521E+00_JPRB ,&
&  0.20225E+00_JPRB, 0.20312E+00_JPRB, 0.20262E+00_JPRB, 0.20065E+00_JPRB, 0.20281E+00_JPRB, 0.20604E+00_JPRB, 0.20614E+00_JPRB ,&
&  0.20458E+00_JPRB, 0.20414E+00_JPRB, 0.20339E+00_JPRB, 0.20194E+00_JPRB, 0.20016E+00_JPRB, 0.19946E+00_JPRB, 0.19881E+00_JPRB ,&
&  0.19845E+00_JPRB, 0.19669E+00_JPRB, 0.20013E+00_JPRB, 0.20046E+00_JPRB, 0.19460E+00_JPRB, 0.19791E+00_JPRB, 0.19567E+00_JPRB ,&
&  0.18768E+00_JPRB, 0.18417E+00_JPRB, 0.18216E+00_JPRB, 0.18044E+00_JPRB, 0.18011E+00_JPRB, 0.18016E+00_JPRB, 0.17993E+00_JPRB ,&
&  0.18096E+00_JPRB, 0.18228E+00_JPRB, 0.18288E+00_JPRB, 0.18370E+00_JPRB, 0.18486E+00_JPRB, 0.18658E+00_JPRB, 0.18884E+00_JPRB ,&
&  0.19082E+00_JPRB, 0.19352E+00_JPRB, 0.19480E+00_JPRB, 0.19693E+00_JPRB, 0.19848E+00_JPRB, 0.20032E+00_JPRB, 0.20483E+00_JPRB ,&
&  0.20831E+00_JPRB, 0.21512E+00_JPRB, 0.22414E+00_JPRB, 0.22611E+00_JPRB, 0.22260E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.22028E+00_JPRB, 0.22256E+00_JPRB, 0.22463E+00_JPRB, 0.22634E+00_JPRB, 0.22813E+00_JPRB, 0.22963E+00_JPRB, 0.23041E+00_JPRB ,&
&  0.23059E+00_JPRB, 0.22989E+00_JPRB, 0.22994E+00_JPRB, 0.22830E+00_JPRB, 0.22674E+00_JPRB, 0.22245E+00_JPRB, 0.22224E+00_JPRB ,&
&  0.22133E+00_JPRB, 0.22218E+00_JPRB, 0.22168E+00_JPRB, 0.22196E+00_JPRB, 0.22102E+00_JPRB, 0.22044E+00_JPRB, 0.21836E+00_JPRB ,&
&  0.21541E+00_JPRB, 0.21334E+00_JPRB, 0.21138E+00_JPRB, 0.21093E+00_JPRB, 0.21187E+00_JPRB, 0.21248E+00_JPRB, 0.21206E+00_JPRB ,&
&  0.21213E+00_JPRB, 0.21237E+00_JPRB, 0.21273E+00_JPRB, 0.21254E+00_JPRB, 0.21806E+00_JPRB, 0.22050E+00_JPRB, 0.22180E+00_JPRB ,&
&  0.22158E+00_JPRB, 0.22184E+00_JPRB, 0.22197E+00_JPRB, 0.22093E+00_JPRB, 0.21991E+00_JPRB, 0.21500E+00_JPRB, 0.21198E+00_JPRB ,&
&  0.20387E+00_JPRB, 0.19793E+00_JPRB, 0.19349E+00_JPRB, 0.19074E+00_JPRB, 0.18958E+00_JPRB, 0.18960E+00_JPRB, 0.18917E+00_JPRB ,&
&  0.18744E+00_JPRB, 0.18580E+00_JPRB, 0.18474E+00_JPRB, 0.18555E+00_JPRB, 0.18689E+00_JPRB, 0.18837E+00_JPRB, 0.19009E+00_JPRB ,&
&  0.19155E+00_JPRB, 0.19191E+00_JPRB, 0.19183E+00_JPRB, 0.19232E+00_JPRB, 0.19269E+00_JPRB, 0.19327E+00_JPRB, 0.19364E+00_JPRB ,&
&  0.19474E+00_JPRB, 0.19365E+00_JPRB, 0.19435E+00_JPRB, 0.19585E+00_JPRB, 0.19309E+00_JPRB, 0.18872E+00_JPRB, 0.18445E+00_JPRB ,&
&  0.18038E+00_JPRB, 0.17809E+00_JPRB, 0.17619E+00_JPRB, 0.17495E+00_JPRB, 0.17466E+00_JPRB, 0.17471E+00_JPRB, 0.17531E+00_JPRB ,&
&  0.17609E+00_JPRB, 0.17722E+00_JPRB, 0.17792E+00_JPRB, 0.17889E+00_JPRB, 0.18009E+00_JPRB, 0.18144E+00_JPRB, 0.18304E+00_JPRB ,&
&  0.18504E+00_JPRB, 0.18692E+00_JPRB, 0.18823E+00_JPRB, 0.19006E+00_JPRB, 0.19353E+00_JPRB, 0.19802E+00_JPRB, 0.20476E+00_JPRB ,&
&  0.21201E+00_JPRB, 0.21696E+00_JPRB, 0.21298E+00_JPRB, 0.21645E+00_JPRB, 0.21910E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.22014E+00_JPRB, 0.22207E+00_JPRB, 0.22364E+00_JPRB, 0.22529E+00_JPRB, 0.22661E+00_JPRB, 0.22785E+00_JPRB, 0.22866E+00_JPRB ,&
&  0.22579E+00_JPRB, 0.22428E+00_JPRB, 0.22143E+00_JPRB, 0.21909E+00_JPRB, 0.21620E+00_JPRB, 0.21289E+00_JPRB, 0.21191E+00_JPRB ,&
&  0.21012E+00_JPRB, 0.20978E+00_JPRB, 0.20819E+00_JPRB, 0.20748E+00_JPRB, 0.20609E+00_JPRB, 0.20518E+00_JPRB, 0.20366E+00_JPRB ,&
&  0.20096E+00_JPRB, 0.19963E+00_JPRB, 0.19837E+00_JPRB, 0.19865E+00_JPRB, 0.20024E+00_JPRB, 0.20192E+00_JPRB, 0.20374E+00_JPRB ,&
&  0.20638E+00_JPRB, 0.20740E+00_JPRB, 0.20995E+00_JPRB, 0.21191E+00_JPRB, 0.21550E+00_JPRB, 0.21854E+00_JPRB, 0.21930E+00_JPRB ,&
&  0.21935E+00_JPRB, 0.21695E+00_JPRB, 0.21610E+00_JPRB, 0.21344E+00_JPRB, 0.20687E+00_JPRB, 0.20344E+00_JPRB, 0.19830E+00_JPRB ,&
&  0.19254E+00_JPRB, 0.18823E+00_JPRB, 0.18520E+00_JPRB, 0.18427E+00_JPRB, 0.18320E+00_JPRB, 0.18287E+00_JPRB, 0.18274E+00_JPRB ,&
&  0.18190E+00_JPRB, 0.18092E+00_JPRB, 0.17971E+00_JPRB, 0.17903E+00_JPRB, 0.17950E+00_JPRB, 0.18042E+00_JPRB, 0.18243E+00_JPRB ,&
&  0.18443E+00_JPRB, 0.18556E+00_JPRB, 0.18680E+00_JPRB, 0.18720E+00_JPRB, 0.18733E+00_JPRB, 0.18759E+00_JPRB, 0.18762E+00_JPRB ,&
&  0.18840E+00_JPRB, 0.18855E+00_JPRB, 0.18794E+00_JPRB, 0.18696E+00_JPRB, 0.18437E+00_JPRB, 0.18094E+00_JPRB, 0.17830E+00_JPRB ,&
&  0.17604E+00_JPRB, 0.17457E+00_JPRB, 0.17408E+00_JPRB, 0.17354E+00_JPRB, 0.17384E+00_JPRB, 0.17428E+00_JPRB, 0.17466E+00_JPRB ,&
&  0.17533E+00_JPRB, 0.17593E+00_JPRB, 0.17651E+00_JPRB, 0.17790E+00_JPRB, 0.17903E+00_JPRB, 0.18073E+00_JPRB, 0.18250E+00_JPRB ,&
&  0.18373E+00_JPRB, 0.18560E+00_JPRB, 0.18719E+00_JPRB, 0.18893E+00_JPRB, 0.19428E+00_JPRB, 0.19838E+00_JPRB, 0.20676E+00_JPRB ,&
&  0.21178E+00_JPRB, 0.21227E+00_JPRB, 0.21135E+00_JPRB, 0.21802E+00_JPRB, 0.21910E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.22032E+00_JPRB, 0.22112E+00_JPRB, 0.22116E+00_JPRB, 0.22223E+00_JPRB, 0.22295E+00_JPRB, 0.22375E+00_JPRB, 0.22267E+00_JPRB ,&
&  0.22176E+00_JPRB, 0.22041E+00_JPRB, 0.21933E+00_JPRB, 0.21609E+00_JPRB, 0.21355E+00_JPRB, 0.21037E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20515E+00_JPRB, 0.20279E+00_JPRB, 0.20055E+00_JPRB, 0.19865E+00_JPRB, 0.19683E+00_JPRB, 0.19518E+00_JPRB, 0.19355E+00_JPRB ,&
&  0.19108E+00_JPRB, 0.19039E+00_JPRB, 0.19033E+00_JPRB, 0.19170E+00_JPRB, 0.19461E+00_JPRB, 0.19635E+00_JPRB, 0.19849E+00_JPRB ,&
&  0.20062E+00_JPRB, 0.19957E+00_JPRB, 0.20308E+00_JPRB, 0.20643E+00_JPRB, 0.20919E+00_JPRB, 0.21157E+00_JPRB, 0.21355E+00_JPRB ,&
&  0.21422E+00_JPRB, 0.20804E+00_JPRB, 0.20909E+00_JPRB, 0.20390E+00_JPRB, 0.19878E+00_JPRB, 0.19682E+00_JPRB, 0.19263E+00_JPRB ,&
&  0.19112E+00_JPRB, 0.18780E+00_JPRB, 0.18565E+00_JPRB, 0.18372E+00_JPRB, 0.18130E+00_JPRB, 0.18074E+00_JPRB, 0.17981E+00_JPRB ,&
&  0.17933E+00_JPRB, 0.17954E+00_JPRB, 0.17872E+00_JPRB, 0.17851E+00_JPRB, 0.17802E+00_JPRB, 0.17758E+00_JPRB, 0.17812E+00_JPRB ,&
&  0.17896E+00_JPRB, 0.17991E+00_JPRB, 0.18082E+00_JPRB, 0.18095E+00_JPRB, 0.18087E+00_JPRB, 0.18084E+00_JPRB, 0.18072E+00_JPRB ,&
&  0.18105E+00_JPRB, 0.18107E+00_JPRB, 0.18061E+00_JPRB, 0.18007E+00_JPRB, 0.17880E+00_JPRB, 0.17754E+00_JPRB, 0.17629E+00_JPRB ,&
&  0.17519E+00_JPRB, 0.17459E+00_JPRB, 0.17407E+00_JPRB, 0.17393E+00_JPRB, 0.17394E+00_JPRB, 0.17431E+00_JPRB, 0.17469E+00_JPRB ,&
&  0.17503E+00_JPRB, 0.17589E+00_JPRB, 0.17652E+00_JPRB, 0.17756E+00_JPRB, 0.17881E+00_JPRB, 0.17993E+00_JPRB, 0.18163E+00_JPRB ,&
&  0.18348E+00_JPRB, 0.18549E+00_JPRB, 0.18840E+00_JPRB, 0.19088E+00_JPRB, 0.19403E+00_JPRB, 0.19800E+00_JPRB, 0.20079E+00_JPRB ,&
&  0.20378E+00_JPRB, 0.20630E+00_JPRB, 0.20607E+00_JPRB, 0.21418E+00_JPRB, 0.21870E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.21113E+00_JPRB, 0.21211E+00_JPRB, 0.21261E+00_JPRB, 0.21303E+00_JPRB, 0.21422E+00_JPRB, 0.21420E+00_JPRB, 0.21396E+00_JPRB ,&
&  0.21162E+00_JPRB, 0.21081E+00_JPRB, 0.20943E+00_JPRB, 0.20741E+00_JPRB, 0.20534E+00_JPRB, 0.20173E+00_JPRB, 0.19959E+00_JPRB ,&
&  0.19756E+00_JPRB, 0.19539E+00_JPRB, 0.19405E+00_JPRB, 0.19220E+00_JPRB, 0.19060E+00_JPRB, 0.18873E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18716E+00_JPRB, 0.18705E+00_JPRB, 0.18794E+00_JPRB, 0.18858E+00_JPRB, 0.18985E+00_JPRB, 0.19120E+00_JPRB, 0.19296E+00_JPRB ,&
&  0.19323E+00_JPRB, 0.19397E+00_JPRB, 0.19475E+00_JPRB, 0.19530E+00_JPRB, 0.19736E+00_JPRB, 0.19883E+00_JPRB, 0.19938E+00_JPRB ,&
&  0.19891E+00_JPRB, 0.19793E+00_JPRB, 0.19600E+00_JPRB, 0.19377E+00_JPRB, 0.19128E+00_JPRB, 0.18859E+00_JPRB, 0.18601E+00_JPRB ,&
&  0.18405E+00_JPRB, 0.18221E+00_JPRB, 0.18080E+00_JPRB, 0.17966E+00_JPRB, 0.17897E+00_JPRB, 0.17842E+00_JPRB, 0.17786E+00_JPRB ,&
&  0.17757E+00_JPRB, 0.17734E+00_JPRB, 0.17697E+00_JPRB, 0.17703E+00_JPRB, 0.17697E+00_JPRB, 0.17708E+00_JPRB, 0.17720E+00_JPRB ,&
&  0.17744E+00_JPRB, 0.17749E+00_JPRB, 0.17744E+00_JPRB, 0.17715E+00_JPRB, 0.17686E+00_JPRB, 0.17651E+00_JPRB, 0.17614E+00_JPRB ,&
&  0.17575E+00_JPRB, 0.17572E+00_JPRB, 0.17538E+00_JPRB, 0.17514E+00_JPRB, 0.17481E+00_JPRB, 0.17423E+00_JPRB, 0.17404E+00_JPRB ,&
&  0.17410E+00_JPRB, 0.17422E+00_JPRB, 0.17400E+00_JPRB, 0.17412E+00_JPRB, 0.17433E+00_JPRB, 0.17465E+00_JPRB, 0.17559E+00_JPRB ,&
&  0.17616E+00_JPRB, 0.17674E+00_JPRB, 0.17728E+00_JPRB, 0.17806E+00_JPRB, 0.17943E+00_JPRB, 0.18079E+00_JPRB, 0.18231E+00_JPRB ,&
&  0.18414E+00_JPRB, 0.18585E+00_JPRB, 0.18756E+00_JPRB, 0.18924E+00_JPRB, 0.19204E+00_JPRB, 0.19394E+00_JPRB, 0.19641E+00_JPRB ,&
&  0.19850E+00_JPRB, 0.19673E+00_JPRB, 0.20552E+00_JPRB, 0.20618E+00_JPRB, 0.20745E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.19440E+00_JPRB, 0.19431E+00_JPRB, 0.19322E+00_JPRB, 0.19200E+00_JPRB, 0.18407E+00_JPRB, 0.19062E+00_JPRB, 0.18996E+00_JPRB ,&
&  0.19066E+00_JPRB, 0.19126E+00_JPRB, 0.19300E+00_JPRB, 0.19340E+00_JPRB, 0.19324E+00_JPRB, 0.19326E+00_JPRB, 0.19310E+00_JPRB ,&
&  0.19287E+00_JPRB, 0.19210E+00_JPRB, 0.19178E+00_JPRB, 0.19120E+00_JPRB, 0.19060E+00_JPRB, 0.18954E+00_JPRB, 0.18839E+00_JPRB ,&
&  0.18786E+00_JPRB, 0.18792E+00_JPRB, 0.18834E+00_JPRB, 0.18636E+00_JPRB, 0.18543E+00_JPRB, 0.18481E+00_JPRB, 0.18447E+00_JPRB ,&
&  0.18395E+00_JPRB, 0.18378E+00_JPRB, 0.18379E+00_JPRB, 0.18374E+00_JPRB, 0.18531E+00_JPRB, 0.18581E+00_JPRB, 0.18620E+00_JPRB ,&
&  0.18630E+00_JPRB, 0.18616E+00_JPRB, 0.18544E+00_JPRB, 0.18477E+00_JPRB, 0.18409E+00_JPRB, 0.18286E+00_JPRB, 0.18159E+00_JPRB ,&
&  0.18050E+00_JPRB, 0.17967E+00_JPRB, 0.17871E+00_JPRB, 0.17798E+00_JPRB, 0.17739E+00_JPRB, 0.17682E+00_JPRB, 0.17645E+00_JPRB ,&
&  0.17636E+00_JPRB, 0.17637E+00_JPRB, 0.17637E+00_JPRB, 0.17637E+00_JPRB, 0.17634E+00_JPRB, 0.17634E+00_JPRB, 0.17636E+00_JPRB ,&
&  0.17677E+00_JPRB, 0.17677E+00_JPRB, 0.17659E+00_JPRB, 0.17628E+00_JPRB, 0.17618E+00_JPRB, 0.17583E+00_JPRB, 0.17559E+00_JPRB ,&
&  0.17539E+00_JPRB, 0.17514E+00_JPRB, 0.17502E+00_JPRB, 0.17499E+00_JPRB, 0.17499E+00_JPRB, 0.17497E+00_JPRB, 0.17507E+00_JPRB ,&
&  0.17523E+00_JPRB, 0.17544E+00_JPRB, 0.17586E+00_JPRB, 0.17619E+00_JPRB, 0.17650E+00_JPRB, 0.17677E+00_JPRB, 0.17733E+00_JPRB ,&
&  0.17792E+00_JPRB, 0.17850E+00_JPRB, 0.17907E+00_JPRB, 0.17944E+00_JPRB, 0.18029E+00_JPRB, 0.18114E+00_JPRB, 0.18207E+00_JPRB ,&
&  0.18308E+00_JPRB, 0.18434E+00_JPRB, 0.18566E+00_JPRB, 0.18698E+00_JPRB, 0.18874E+00_JPRB, 0.19012E+00_JPRB, 0.19147E+00_JPRB ,&
&  0.19298E+00_JPRB, 0.19579E+00_JPRB, 0.19645E+00_JPRB, 0.19649E+00_JPRB, 0.19574E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.18956E+00_JPRB, 0.18950E+00_JPRB, 0.18933E+00_JPRB, 0.18886E+00_JPRB, 0.18799E+00_JPRB, 0.18778E+00_JPRB, 0.18751E+00_JPRB ,&
&  0.18712E+00_JPRB, 0.18633E+00_JPRB, 0.18631E+00_JPRB, 0.18621E+00_JPRB, 0.18608E+00_JPRB, 0.18594E+00_JPRB, 0.18564E+00_JPRB ,&
&  0.18506E+00_JPRB, 0.18415E+00_JPRB, 0.18150E+00_JPRB, 0.18022E+00_JPRB, 0.17939E+00_JPRB, 0.17894E+00_JPRB, 0.17856E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17822E+00_JPRB, 0.17810E+00_JPRB, 0.17860E+00_JPRB, 0.17883E+00_JPRB, 0.17892E+00_JPRB, 0.17898E+00_JPRB ,&
&  0.17899E+00_JPRB, 0.17910E+00_JPRB, 0.17911E+00_JPRB, 0.17911E+00_JPRB, 0.17925E+00_JPRB, 0.17926E+00_JPRB, 0.17906E+00_JPRB ,&
&  0.17883E+00_JPRB, 0.17855E+00_JPRB, 0.17828E+00_JPRB, 0.17794E+00_JPRB, 0.17770E+00_JPRB, 0.17750E+00_JPRB, 0.17743E+00_JPRB ,&
&  0.17727E+00_JPRB, 0.17714E+00_JPRB, 0.17701E+00_JPRB, 0.17695E+00_JPRB, 0.17685E+00_JPRB, 0.17681E+00_JPRB, 0.17667E+00_JPRB ,&
&  0.17678E+00_JPRB, 0.17682E+00_JPRB, 0.17690E+00_JPRB, 0.17695E+00_JPRB, 0.17698E+00_JPRB, 0.17696E+00_JPRB, 0.17689E+00_JPRB ,&
&  0.17670E+00_JPRB, 0.17659E+00_JPRB, 0.17645E+00_JPRB, 0.17635E+00_JPRB, 0.17633E+00_JPRB, 0.17631E+00_JPRB, 0.17631E+00_JPRB ,&
&  0.17632E+00_JPRB, 0.17644E+00_JPRB, 0.17648E+00_JPRB, 0.17654E+00_JPRB, 0.17662E+00_JPRB, 0.17670E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17687E+00_JPRB, 0.17696E+00_JPRB, 0.17708E+00_JPRB, 0.17724E+00_JPRB, 0.17744E+00_JPRB, 0.17771E+00_JPRB, 0.17804E+00_JPRB ,&
&  0.17839E+00_JPRB, 0.17874E+00_JPRB, 0.17908E+00_JPRB, 0.17942E+00_JPRB, 0.17997E+00_JPRB, 0.18051E+00_JPRB, 0.18116E+00_JPRB ,&
&  0.18180E+00_JPRB, 0.18243E+00_JPRB, 0.18301E+00_JPRB, 0.18362E+00_JPRB, 0.18445E+00_JPRB, 0.18486E+00_JPRB, 0.18519E+00_JPRB ,&
&  0.18575E+00_JPRB, 0.18633E+00_JPRB, 0.18690E+00_JPRB, 0.18739E+00_JPRB, 0.18818E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB, 0.17871E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB, 0.91610E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.92224E+00_JPRB, 0.91907E+00_JPRB, 0.91618E+00_JPRB, 0.91316E+00_JPRB, 0.91005E+00_JPRB, 0.90751E+00_JPRB, 0.90513E+00_JPRB ,&
&  0.90298E+00_JPRB, 0.90173E+00_JPRB, 0.90103E+00_JPRB, 0.90065E+00_JPRB, 0.90071E+00_JPRB, 0.90057E+00_JPRB, 0.90058E+00_JPRB ,&
&  0.90051E+00_JPRB, 0.90065E+00_JPRB, 0.90066E+00_JPRB, 0.90019E+00_JPRB, 0.89932E+00_JPRB, 0.89815E+00_JPRB, 0.89667E+00_JPRB ,&
&  0.89457E+00_JPRB, 0.89218E+00_JPRB, 0.88980E+00_JPRB, 0.88727E+00_JPRB, 0.88485E+00_JPRB, 0.88209E+00_JPRB, 0.87916E+00_JPRB ,&
&  0.87650E+00_JPRB, 0.87483E+00_JPRB, 0.87342E+00_JPRB, 0.87195E+00_JPRB, 0.87108E+00_JPRB, 0.87187E+00_JPRB, 0.87252E+00_JPRB ,&
&  0.87321E+00_JPRB, 0.87410E+00_JPRB, 0.87638E+00_JPRB, 0.87864E+00_JPRB, 0.88076E+00_JPRB, 0.88309E+00_JPRB, 0.88570E+00_JPRB ,&
&  0.88722E+00_JPRB, 0.88819E+00_JPRB, 0.88882E+00_JPRB, 0.88956E+00_JPRB, 0.88979E+00_JPRB, 0.89028E+00_JPRB, 0.89063E+00_JPRB ,&
&  0.89077E+00_JPRB, 0.89093E+00_JPRB, 0.89063E+00_JPRB, 0.88933E+00_JPRB, 0.88581E+00_JPRB, 0.88320E+00_JPRB, 0.88205E+00_JPRB ,&
&  0.88220E+00_JPRB, 0.88273E+00_JPRB, 0.88378E+00_JPRB, 0.88542E+00_JPRB, 0.88811E+00_JPRB, 0.89120E+00_JPRB, 0.89483E+00_JPRB ,&
&  0.89878E+00_JPRB, 0.90353E+00_JPRB, 0.90943E+00_JPRB, 0.91551E+00_JPRB, 0.92215E+00_JPRB, 0.92992E+00_JPRB, 0.93710E+00_JPRB ,&
&  0.94455E+00_JPRB, 0.95236E+00_JPRB, 0.95893E+00_JPRB, 0.96514E+00_JPRB, 0.97107E+00_JPRB, 0.97700E+00_JPRB, 0.98135E+00_JPRB ,&
&  0.98333E+00_JPRB, 0.98469E+00_JPRB, 0.98565E+00_JPRB, 0.98464E+00_JPRB, 0.98229E+00_JPRB, 0.98034E+00_JPRB, 0.97828E+00_JPRB ,&
&  0.97571E+00_JPRB, 0.97247E+00_JPRB, 0.96938E+00_JPRB, 0.96594E+00_JPRB, 0.96119E+00_JPRB, 0.95598E+00_JPRB, 0.95104E+00_JPRB ,&
&  0.94616E+00_JPRB, 0.94079E+00_JPRB, 0.93583E+00_JPRB, 0.93106E+00_JPRB, 0.92660E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.95640E+00_JPRB, 0.94507E+00_JPRB, 0.93471E+00_JPRB, 0.92515E+00_JPRB, 0.91724E+00_JPRB, 0.91102E+00_JPRB, 0.90556E+00_JPRB ,&
&  0.90071E+00_JPRB, 0.89806E+00_JPRB, 0.89562E+00_JPRB, 0.89338E+00_JPRB, 0.89128E+00_JPRB, 0.88892E+00_JPRB, 0.88615E+00_JPRB ,&
&  0.88354E+00_JPRB, 0.88175E+00_JPRB, 0.88004E+00_JPRB, 0.87774E+00_JPRB, 0.87340E+00_JPRB, 0.86685E+00_JPRB, 0.85916E+00_JPRB ,&
&  0.85035E+00_JPRB, 0.83977E+00_JPRB, 0.82759E+00_JPRB, 0.81461E+00_JPRB, 0.80767E+00_JPRB, 0.79969E+00_JPRB, 0.79203E+00_JPRB ,&
&  0.78527E+00_JPRB, 0.78294E+00_JPRB, 0.78101E+00_JPRB, 0.77967E+00_JPRB, 0.77955E+00_JPRB, 0.78142E+00_JPRB, 0.78235E+00_JPRB ,&
&  0.78290E+00_JPRB, 0.78504E+00_JPRB, 0.79238E+00_JPRB, 0.80040E+00_JPRB, 0.80981E+00_JPRB, 0.82142E+00_JPRB, 0.83464E+00_JPRB ,&
&  0.84482E+00_JPRB, 0.85154E+00_JPRB, 0.85896E+00_JPRB, 0.86596E+00_JPRB, 0.87108E+00_JPRB, 0.87510E+00_JPRB, 0.87829E+00_JPRB ,&
&  0.88297E+00_JPRB, 0.88615E+00_JPRB, 0.88897E+00_JPRB, 0.89063E+00_JPRB, 0.88973E+00_JPRB, 0.88837E+00_JPRB, 0.88611E+00_JPRB ,&
&  0.88282E+00_JPRB, 0.88251E+00_JPRB, 0.88492E+00_JPRB, 0.88792E+00_JPRB, 0.89378E+00_JPRB, 0.90218E+00_JPRB, 0.91157E+00_JPRB ,&
&  0.92398E+00_JPRB, 0.94059E+00_JPRB, 0.96080E+00_JPRB, 0.98313E+00_JPRB, 0.10089E+01_JPRB, 0.10410E+01_JPRB, 0.10673E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.11113E+01_JPRB, 0.11290E+01_JPRB, 0.11413E+01_JPRB, 0.11497E+01_JPRB, 0.11573E+01_JPRB, 0.11566E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.11503E+01_JPRB, 0.11474E+01_JPRB, 0.11409E+01_JPRB, 0.11329E+01_JPRB, 0.11247E+01_JPRB, 0.11168E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.10934E+01_JPRB, 0.10818E+01_JPRB, 0.10706E+01_JPRB, 0.10578E+01_JPRB, 0.10454E+01_JPRB, 0.10331E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.10074E+01_JPRB, 0.99460E+00_JPRB, 0.98206E+00_JPRB, 0.96942E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.99802E+00_JPRB, 0.97935E+00_JPRB, 0.96232E+00_JPRB, 0.94719E+00_JPRB, 0.93389E+00_JPRB, 0.92306E+00_JPRB, 0.91387E+00_JPRB ,&
&  0.90591E+00_JPRB, 0.90086E+00_JPRB, 0.89589E+00_JPRB, 0.89061E+00_JPRB, 0.88531E+00_JPRB, 0.88395E+00_JPRB, 0.88050E+00_JPRB ,&
&  0.87539E+00_JPRB, 0.86898E+00_JPRB, 0.86294E+00_JPRB, 0.85419E+00_JPRB, 0.84114E+00_JPRB, 0.82540E+00_JPRB, 0.80587E+00_JPRB ,&
&  0.79348E+00_JPRB, 0.77778E+00_JPRB, 0.76123E+00_JPRB, 0.74436E+00_JPRB, 0.73253E+00_JPRB, 0.70666E+00_JPRB, 0.69745E+00_JPRB ,&
&  0.69146E+00_JPRB, 0.67373E+00_JPRB, 0.66400E+00_JPRB, 0.65942E+00_JPRB, 0.67498E+00_JPRB, 0.69632E+00_JPRB, 0.70823E+00_JPRB ,&
&  0.69601E+00_JPRB, 0.70661E+00_JPRB, 0.70370E+00_JPRB, 0.70085E+00_JPRB, 0.70249E+00_JPRB, 0.70782E+00_JPRB, 0.73446E+00_JPRB ,&
&  0.78234E+00_JPRB, 0.80183E+00_JPRB, 0.81449E+00_JPRB, 0.82747E+00_JPRB, 0.83713E+00_JPRB, 0.84644E+00_JPRB, 0.85442E+00_JPRB ,&
&  0.86329E+00_JPRB, 0.87134E+00_JPRB, 0.88029E+00_JPRB, 0.88427E+00_JPRB, 0.89039E+00_JPRB, 0.89628E+00_JPRB, 0.90109E+00_JPRB ,&
&  0.89674E+00_JPRB, 0.89538E+00_JPRB, 0.89818E+00_JPRB, 0.90546E+00_JPRB, 0.91717E+00_JPRB, 0.93642E+00_JPRB, 0.95740E+00_JPRB ,&
&  0.97999E+00_JPRB, 0.99807E+00_JPRB, 0.10271E+01_JPRB, 0.10665E+01_JPRB, 0.11142E+01_JPRB, 0.11732E+01_JPRB, 0.12325E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.13386E+01_JPRB, 0.13837E+01_JPRB, 0.14053E+01_JPRB, 0.14158E+01_JPRB, 0.14226E+01_JPRB, 0.14214E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.14095E+01_JPRB, 0.14004E+01_JPRB, 0.13875E+01_JPRB, 0.13701E+01_JPRB, 0.13474E+01_JPRB, 0.13216E+01_JPRB ,&
&  0.12913E+01_JPRB, 0.12619E+01_JPRB, 0.12329E+01_JPRB, 0.12046E+01_JPRB, 0.11763E+01_JPRB, 0.11503E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.10820E+01_JPRB, 0.10608E+01_JPRB, 0.10401E+01_JPRB, 0.10193E+01_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.10371E+01_JPRB, 0.10166E+01_JPRB, 0.99498E+00_JPRB, 0.97527E+00_JPRB, 0.95874E+00_JPRB, 0.94599E+00_JPRB, 0.93651E+00_JPRB ,&
&  0.92784E+00_JPRB, 0.91736E+00_JPRB, 0.90613E+00_JPRB, 0.89411E+00_JPRB, 0.88331E+00_JPRB, 0.87753E+00_JPRB, 0.87260E+00_JPRB ,&
&  0.86872E+00_JPRB, 0.85988E+00_JPRB, 0.84868E+00_JPRB, 0.83237E+00_JPRB, 0.81451E+00_JPRB, 0.79810E+00_JPRB, 0.77766E+00_JPRB ,&
&  0.76237E+00_JPRB, 0.74650E+00_JPRB, 0.72338E+00_JPRB, 0.71043E+00_JPRB, 0.67944E+00_JPRB, 0.65841E+00_JPRB, 0.64495E+00_JPRB ,&
&  0.63497E+00_JPRB, 0.63631E+00_JPRB, 0.64146E+00_JPRB, 0.65129E+00_JPRB, 0.65632E+00_JPRB, 0.65355E+00_JPRB, 0.65447E+00_JPRB ,&
&  0.64714E+00_JPRB, 0.64211E+00_JPRB, 0.63843E+00_JPRB, 0.63783E+00_JPRB, 0.64128E+00_JPRB, 0.64987E+00_JPRB, 0.66796E+00_JPRB ,&
&  0.70573E+00_JPRB, 0.75511E+00_JPRB, 0.79985E+00_JPRB, 0.82384E+00_JPRB, 0.82818E+00_JPRB, 0.83788E+00_JPRB, 0.84091E+00_JPRB ,&
&  0.84758E+00_JPRB, 0.85171E+00_JPRB, 0.86121E+00_JPRB, 0.87254E+00_JPRB, 0.88484E+00_JPRB, 0.89582E+00_JPRB, 0.90816E+00_JPRB ,&
&  0.91260E+00_JPRB, 0.91706E+00_JPRB, 0.91735E+00_JPRB, 0.92913E+00_JPRB, 0.95208E+00_JPRB, 0.97579E+00_JPRB, 0.98698E+00_JPRB ,&
&  0.10062E+01_JPRB, 0.10323E+01_JPRB, 0.10805E+01_JPRB, 0.11531E+01_JPRB, 0.12321E+01_JPRB, 0.13056E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.15223E+01_JPRB, 0.15978E+01_JPRB, 0.16274E+01_JPRB, 0.16256E+01_JPRB, 0.16316E+01_JPRB, 0.16180E+01_JPRB ,&
&  0.16084E+01_JPRB, 0.15929E+01_JPRB, 0.15845E+01_JPRB, 0.15760E+01_JPRB, 0.15520E+01_JPRB, 0.15154E+01_JPRB, 0.14766E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.13885E+01_JPRB, 0.13446E+01_JPRB, 0.13002E+01_JPRB, 0.12575E+01_JPRB, 0.12159E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11200E+01_JPRB, 0.10952E+01_JPRB, 0.10751E+01_JPRB, 0.10562E+01_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.10771E+01_JPRB, 0.10633E+01_JPRB, 0.10494E+01_JPRB, 0.10310E+01_JPRB, 0.10122E+01_JPRB, 0.99487E+00_JPRB, 0.98110E+00_JPRB ,&
&  0.96921E+00_JPRB, 0.95604E+00_JPRB, 0.94278E+00_JPRB, 0.92669E+00_JPRB, 0.91129E+00_JPRB, 0.89694E+00_JPRB, 0.88480E+00_JPRB ,&
&  0.87403E+00_JPRB, 0.85863E+00_JPRB, 0.84375E+00_JPRB, 0.82605E+00_JPRB, 0.80946E+00_JPRB, 0.79127E+00_JPRB, 0.76997E+00_JPRB ,&
&  0.75385E+00_JPRB, 0.73045E+00_JPRB, 0.71039E+00_JPRB, 0.69934E+00_JPRB, 0.67084E+00_JPRB, 0.65582E+00_JPRB, 0.64694E+00_JPRB ,&
&  0.66216E+00_JPRB, 0.65840E+00_JPRB, 0.67514E+00_JPRB, 0.65651E+00_JPRB, 0.64388E+00_JPRB, 0.63232E+00_JPRB, 0.61953E+00_JPRB ,&
&  0.61083E+00_JPRB, 0.60090E+00_JPRB, 0.60210E+00_JPRB, 0.60814E+00_JPRB, 0.61232E+00_JPRB, 0.62591E+00_JPRB, 0.64343E+00_JPRB ,&
&  0.68197E+00_JPRB, 0.78795E+00_JPRB, 0.81102E+00_JPRB, 0.83523E+00_JPRB, 0.84132E+00_JPRB, 0.84919E+00_JPRB, 0.85476E+00_JPRB ,&
&  0.86130E+00_JPRB, 0.86513E+00_JPRB, 0.86980E+00_JPRB, 0.87574E+00_JPRB, 0.89687E+00_JPRB, 0.91176E+00_JPRB, 0.92950E+00_JPRB ,&
&  0.93612E+00_JPRB, 0.94448E+00_JPRB, 0.95010E+00_JPRB, 0.96560E+00_JPRB, 0.99732E+00_JPRB, 0.10126E+01_JPRB, 0.10174E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10909E+01_JPRB, 0.11568E+01_JPRB, 0.12368E+01_JPRB, 0.12974E+01_JPRB, 0.13488E+01_JPRB, 0.14259E+01_JPRB ,&
&  0.14795E+01_JPRB, 0.15681E+01_JPRB, 0.16611E+01_JPRB, 0.17036E+01_JPRB, 0.17090E+01_JPRB, 0.16951E+01_JPRB, 0.16772E+01_JPRB ,&
&  0.16716E+01_JPRB, 0.16446E+01_JPRB, 0.16282E+01_JPRB, 0.16192E+01_JPRB, 0.15899E+01_JPRB, 0.15545E+01_JPRB, 0.15177E+01_JPRB ,&
&  0.14710E+01_JPRB, 0.14290E+01_JPRB, 0.13937E+01_JPRB, 0.13529E+01_JPRB, 0.13093E+01_JPRB, 0.12668E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11540E+01_JPRB, 0.11243E+01_JPRB, 0.11049E+01_JPRB, 0.10876E+01_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.11089E+01_JPRB, 0.10951E+01_JPRB, 0.10821E+01_JPRB, 0.10696E+01_JPRB, 0.10556E+01_JPRB, 0.10414E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.10015E+01_JPRB, 0.98799E+00_JPRB, 0.97076E+00_JPRB, 0.95255E+00_JPRB, 0.93203E+00_JPRB, 0.91467E+00_JPRB ,&
&  0.89614E+00_JPRB, 0.87651E+00_JPRB, 0.84988E+00_JPRB, 0.82521E+00_JPRB, 0.80775E+00_JPRB, 0.79157E+00_JPRB, 0.77215E+00_JPRB ,&
&  0.75536E+00_JPRB, 0.73307E+00_JPRB, 0.70758E+00_JPRB, 0.68717E+00_JPRB, 0.66054E+00_JPRB, 0.65483E+00_JPRB, 0.69031E+00_JPRB ,&
&  0.70696E+00_JPRB, 0.70649E+00_JPRB, 0.69985E+00_JPRB, 0.68800E+00_JPRB, 0.67667E+00_JPRB, 0.62759E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.61578E+00_JPRB, 0.59785E+00_JPRB, 0.59691E+00_JPRB, 0.59913E+00_JPRB, 0.60775E+00_JPRB, 0.62747E+00_JPRB, 0.64821E+00_JPRB ,&
&  0.68627E+00_JPRB, 0.80863E+00_JPRB, 0.83346E+00_JPRB, 0.85682E+00_JPRB, 0.86497E+00_JPRB, 0.87257E+00_JPRB, 0.87958E+00_JPRB ,&
&  0.88464E+00_JPRB, 0.89371E+00_JPRB, 0.90311E+00_JPRB, 0.91724E+00_JPRB, 0.93387E+00_JPRB, 0.94380E+00_JPRB, 0.96675E+00_JPRB ,&
&  0.98324E+00_JPRB, 0.99823E+00_JPRB, 0.10093E+01_JPRB, 0.10129E+01_JPRB, 0.10291E+01_JPRB, 0.10243E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.11735E+01_JPRB, 0.12493E+01_JPRB, 0.13189E+01_JPRB, 0.13796E+01_JPRB, 0.14470E+01_JPRB, 0.15297E+01_JPRB ,&
&  0.15891E+01_JPRB, 0.16749E+01_JPRB, 0.17738E+01_JPRB, 0.18069E+01_JPRB, 0.18045E+01_JPRB, 0.17967E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.18020E+01_JPRB, 0.18096E+01_JPRB, 0.17575E+01_JPRB, 0.17252E+01_JPRB, 0.16576E+01_JPRB, 0.15724E+01_JPRB, 0.15034E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.13462E+01_JPRB, 0.13003E+01_JPRB, 0.12665E+01_JPRB, 0.12417E+01_JPRB, 0.12254E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.11800E+01_JPRB, 0.11604E+01_JPRB, 0.11407E+01_JPRB, 0.11232E+01_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.11139E+01_JPRB, 0.11033E+01_JPRB, 0.10888E+01_JPRB, 0.10765E+01_JPRB, 0.10612E+01_JPRB, 0.10503E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10327E+01_JPRB, 0.10290E+01_JPRB, 0.10239E+01_JPRB, 0.10188E+01_JPRB, 0.10092E+01_JPRB, 0.99651E+00_JPRB, 0.97943E+00_JPRB ,&
&  0.95492E+00_JPRB, 0.90074E+00_JPRB, 0.86096E+00_JPRB, 0.84089E+00_JPRB, 0.82100E+00_JPRB, 0.80467E+00_JPRB, 0.78837E+00_JPRB ,&
&  0.76569E+00_JPRB, 0.74599E+00_JPRB, 0.72282E+00_JPRB, 0.69275E+00_JPRB, 0.67751E+00_JPRB, 0.68759E+00_JPRB, 0.69325E+00_JPRB ,&
&  0.70096E+00_JPRB, 0.72030E+00_JPRB, 0.71747E+00_JPRB, 0.70366E+00_JPRB, 0.70143E+00_JPRB, 0.70341E+00_JPRB, 0.66167E+00_JPRB ,&
&  0.62755E+00_JPRB, 0.60064E+00_JPRB, 0.59696E+00_JPRB, 0.59757E+00_JPRB, 0.60729E+00_JPRB, 0.62819E+00_JPRB, 0.67926E+00_JPRB ,&
&  0.76390E+00_JPRB, 0.86920E+00_JPRB, 0.87238E+00_JPRB, 0.87791E+00_JPRB, 0.88891E+00_JPRB, 0.89935E+00_JPRB, 0.91077E+00_JPRB ,&
&  0.92359E+00_JPRB, 0.94329E+00_JPRB, 0.96780E+00_JPRB, 0.99325E+00_JPRB, 0.10218E+01_JPRB, 0.10392E+01_JPRB, 0.10550E+01_JPRB ,&
&  0.10868E+01_JPRB, 0.10753E+01_JPRB, 0.10770E+01_JPRB, 0.10691E+01_JPRB, 0.10614E+01_JPRB, 0.10418E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.12337E+01_JPRB, 0.13147E+01_JPRB, 0.13789E+01_JPRB, 0.14485E+01_JPRB, 0.15433E+01_JPRB, 0.16493E+01_JPRB ,&
&  0.17459E+01_JPRB, 0.18724E+01_JPRB, 0.20450E+01_JPRB, 0.20249E+01_JPRB, 0.19902E+01_JPRB, 0.19692E+01_JPRB, 0.19736E+01_JPRB ,&
&  0.19704E+01_JPRB, 0.19885E+01_JPRB, 0.18922E+01_JPRB, 0.17509E+01_JPRB, 0.16256E+01_JPRB, 0.14855E+01_JPRB, 0.14040E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.12588E+01_JPRB, 0.12148E+01_JPRB, 0.11895E+01_JPRB, 0.11735E+01_JPRB, 0.11560E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11333E+01_JPRB, 0.11332E+01_JPRB, 0.11279E+01_JPRB, 0.11219E+01_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.10859E+01_JPRB, 0.10879E+01_JPRB, 0.10834E+01_JPRB, 0.10716E+01_JPRB, 0.10419E+01_JPRB, 0.10019E+01_JPRB, 0.97477E+00_JPRB ,&
&  0.95770E+00_JPRB, 0.95505E+00_JPRB, 0.94978E+00_JPRB, 0.94704E+00_JPRB, 0.95077E+00_JPRB, 0.96029E+00_JPRB, 0.96871E+00_JPRB ,&
&  0.96898E+00_JPRB, 0.95383E+00_JPRB, 0.92540E+00_JPRB, 0.88259E+00_JPRB, 0.84811E+00_JPRB, 0.82300E+00_JPRB, 0.80792E+00_JPRB ,&
&  0.79607E+00_JPRB, 0.77977E+00_JPRB, 0.75991E+00_JPRB, 0.72755E+00_JPRB, 0.70534E+00_JPRB, 0.68882E+00_JPRB, 0.69661E+00_JPRB ,&
&  0.71860E+00_JPRB, 0.72601E+00_JPRB, 0.71950E+00_JPRB, 0.73422E+00_JPRB, 0.73550E+00_JPRB, 0.74011E+00_JPRB, 0.68621E+00_JPRB ,&
&  0.64637E+00_JPRB, 0.62823E+00_JPRB, 0.61521E+00_JPRB, 0.61253E+00_JPRB, 0.62778E+00_JPRB, 0.65791E+00_JPRB, 0.71875E+00_JPRB ,&
&  0.84080E+00_JPRB, 0.87948E+00_JPRB, 0.89425E+00_JPRB, 0.91340E+00_JPRB, 0.92892E+00_JPRB, 0.93735E+00_JPRB, 0.95574E+00_JPRB ,&
&  0.99285E+00_JPRB, 0.10495E+01_JPRB, 0.11049E+01_JPRB, 0.11173E+01_JPRB, 0.11200E+01_JPRB, 0.11263E+01_JPRB, 0.12583E+01_JPRB ,&
&  0.16150E+01_JPRB, 0.14606E+01_JPRB, 0.12772E+01_JPRB, 0.11684E+01_JPRB, 0.11161E+01_JPRB, 0.10958E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.12186E+01_JPRB, 0.12966E+01_JPRB, 0.13992E+01_JPRB, 0.14787E+01_JPRB, 0.15657E+01_JPRB, 0.16837E+01_JPRB, 0.18498E+01_JPRB ,&
&  0.20205E+01_JPRB, 0.22353E+01_JPRB, 0.26666E+01_JPRB, 0.23827E+01_JPRB, 0.22151E+01_JPRB, 0.21537E+01_JPRB, 0.21282E+01_JPRB ,&
&  0.20528E+01_JPRB, 0.20445E+01_JPRB, 0.18928E+01_JPRB, 0.16931E+01_JPRB, 0.15909E+01_JPRB, 0.14792E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.13177E+01_JPRB, 0.12439E+01_JPRB, 0.12024E+01_JPRB, 0.11713E+01_JPRB, 0.11608E+01_JPRB, 0.11492E+01_JPRB, 0.11348E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.11016E+01_JPRB, 0.10908E+01_JPRB, 0.10828E+01_JPRB, 0.10806E+01_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.10460E+01_JPRB, 0.10667E+01_JPRB, 0.10807E+01_JPRB, 0.10556E+01_JPRB, 0.10140E+01_JPRB, 0.94443E+00_JPRB, 0.88950E+00_JPRB ,&
&  0.84863E+00_JPRB, 0.83563E+00_JPRB, 0.82989E+00_JPRB, 0.83211E+00_JPRB, 0.84241E+00_JPRB, 0.86010E+00_JPRB, 0.87290E+00_JPRB ,&
&  0.88629E+00_JPRB, 0.88776E+00_JPRB, 0.88716E+00_JPRB, 0.87356E+00_JPRB, 0.84883E+00_JPRB, 0.82038E+00_JPRB, 0.80061E+00_JPRB ,&
&  0.78855E+00_JPRB, 0.78040E+00_JPRB, 0.76896E+00_JPRB, 0.75084E+00_JPRB, 0.73765E+00_JPRB, 0.72004E+00_JPRB, 0.72136E+00_JPRB ,&
&  0.71186E+00_JPRB, 0.71840E+00_JPRB, 0.73687E+00_JPRB, 0.74403E+00_JPRB, 0.75944E+00_JPRB, 0.76745E+00_JPRB, 0.72554E+00_JPRB ,&
&  0.68039E+00_JPRB, 0.65875E+00_JPRB, 0.65275E+00_JPRB, 0.65629E+00_JPRB, 0.70656E+00_JPRB, 0.82057E+00_JPRB, 0.87961E+00_JPRB ,&
&  0.91265E+00_JPRB, 0.93217E+00_JPRB, 0.95442E+00_JPRB, 0.98091E+00_JPRB, 0.99283E+00_JPRB, 0.10075E+01_JPRB, 0.10483E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.12311E+01_JPRB, 0.13091E+01_JPRB, 0.11744E+01_JPRB, 0.11116E+01_JPRB, 0.11623E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.13304E+01_JPRB, 0.12992E+01_JPRB, 0.12589E+01_JPRB, 0.12891E+01_JPRB, 0.12993E+01_JPRB, 0.13106E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.14233E+01_JPRB, 0.15881E+01_JPRB, 0.16767E+01_JPRB, 0.18155E+01_JPRB, 0.20168E+01_JPRB, 0.21949E+01_JPRB ,&
&  0.24015E+01_JPRB, 0.25505E+01_JPRB, 0.26157E+01_JPRB, 0.25127E+01_JPRB, 0.24353E+01_JPRB, 0.24270E+01_JPRB, 0.23711E+01_JPRB ,&
&  0.21815E+01_JPRB, 0.20429E+01_JPRB, 0.19268E+01_JPRB, 0.18135E+01_JPRB, 0.16594E+01_JPRB, 0.15301E+01_JPRB, 0.13978E+01_JPRB ,&
&  0.12704E+01_JPRB, 0.11768E+01_JPRB, 0.11288E+01_JPRB, 0.10957E+01_JPRB, 0.10844E+01_JPRB, 0.10796E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.10410E+01_JPRB, 0.10166E+01_JPRB, 0.10043E+01_JPRB, 0.10108E+01_JPRB, 0.10371E+01_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.10927E+01_JPRB, 0.10847E+01_JPRB, 0.10438E+01_JPRB, 0.10429E+01_JPRB, 0.10065E+01_JPRB, 0.96694E+00_JPRB, 0.91058E+00_JPRB ,&
&  0.86968E+00_JPRB, 0.82700E+00_JPRB, 0.79578E+00_JPRB, 0.75722E+00_JPRB, 0.73536E+00_JPRB, 0.72787E+00_JPRB, 0.75075E+00_JPRB ,&
&  0.78263E+00_JPRB, 0.80618E+00_JPRB, 0.82118E+00_JPRB, 0.82510E+00_JPRB, 0.82121E+00_JPRB, 0.80825E+00_JPRB, 0.79400E+00_JPRB ,&
&  0.78134E+00_JPRB, 0.77832E+00_JPRB, 0.77031E+00_JPRB, 0.76071E+00_JPRB, 0.74979E+00_JPRB, 0.73814E+00_JPRB, 0.73201E+00_JPRB ,&
&  0.71996E+00_JPRB, 0.71601E+00_JPRB, 0.72462E+00_JPRB, 0.74496E+00_JPRB, 0.77004E+00_JPRB, 0.78273E+00_JPRB, 0.74783E+00_JPRB ,&
&  0.71236E+00_JPRB, 0.70457E+00_JPRB, 0.71930E+00_JPRB, 0.82111E+00_JPRB, 0.86584E+00_JPRB, 0.89115E+00_JPRB, 0.92561E+00_JPRB ,&
&  0.91477E+00_JPRB, 0.91512E+00_JPRB, 0.93997E+00_JPRB, 0.10195E+01_JPRB, 0.10488E+01_JPRB, 0.10762E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.12298E+01_JPRB, 0.13633E+01_JPRB, 0.14703E+01_JPRB, 0.13483E+01_JPRB, 0.14444E+01_JPRB, 0.17099E+01_JPRB, 0.15652E+01_JPRB ,&
&  0.14896E+01_JPRB, 0.14276E+01_JPRB, 0.14605E+01_JPRB, 0.15021E+01_JPRB, 0.15015E+01_JPRB, 0.15479E+01_JPRB, 0.15922E+01_JPRB ,&
&  0.15759E+01_JPRB, 0.16671E+01_JPRB, 0.18660E+01_JPRB, 0.20573E+01_JPRB, 0.22384E+01_JPRB, 0.24742E+01_JPRB, 0.27193E+01_JPRB ,&
&  0.30182E+01_JPRB, 0.32767E+01_JPRB, 0.34475E+01_JPRB, 0.33957E+01_JPRB, 0.32949E+01_JPRB, 0.31195E+01_JPRB, 0.28689E+01_JPRB ,&
&  0.25321E+01_JPRB, 0.22935E+01_JPRB, 0.21369E+01_JPRB, 0.20129E+01_JPRB, 0.18599E+01_JPRB, 0.16582E+01_JPRB, 0.13971E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.10722E+01_JPRB, 0.10213E+01_JPRB, 0.10073E+01_JPRB, 0.10135E+01_JPRB, 0.10195E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.99868E+00_JPRB, 0.98902E+00_JPRB, 0.98256E+00_JPRB, 0.99576E+00_JPRB, 0.10630E+01_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.11368E+01_JPRB, 0.11086E+01_JPRB, 0.10790E+01_JPRB, 0.10689E+01_JPRB, 0.10497E+01_JPRB, 0.99101E+00_JPRB, 0.90318E+00_JPRB ,&
&  0.90414E+00_JPRB, 0.86251E+00_JPRB, 0.84893E+00_JPRB, 0.83263E+00_JPRB, 0.80766E+00_JPRB, 0.75950E+00_JPRB, 0.72358E+00_JPRB ,&
&  0.72434E+00_JPRB, 0.74852E+00_JPRB, 0.77557E+00_JPRB, 0.79729E+00_JPRB, 0.80568E+00_JPRB, 0.80106E+00_JPRB, 0.79190E+00_JPRB ,&
&  0.78349E+00_JPRB, 0.78035E+00_JPRB, 0.78069E+00_JPRB, 0.78120E+00_JPRB, 0.77627E+00_JPRB, 0.76978E+00_JPRB, 0.76785E+00_JPRB ,&
&  0.76256E+00_JPRB, 0.75580E+00_JPRB, 0.74989E+00_JPRB, 0.76006E+00_JPRB, 0.77175E+00_JPRB, 0.79714E+00_JPRB, 0.81319E+00_JPRB ,&
&  0.75269E+00_JPRB, 0.76890E+00_JPRB, 0.86497E+00_JPRB, 0.89442E+00_JPRB, 0.92515E+00_JPRB, 0.95182E+00_JPRB, 0.97462E+00_JPRB ,&
&  0.99834E+00_JPRB, 0.10313E+01_JPRB, 0.10694E+01_JPRB, 0.11060E+01_JPRB, 0.11325E+01_JPRB, 0.11796E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.13446E+01_JPRB, 0.13151E+01_JPRB, 0.14286E+01_JPRB, 0.18967E+01_JPRB, 0.19834E+01_JPRB, 0.19545E+01_JPRB ,&
&  0.19313E+01_JPRB, 0.18351E+01_JPRB, 0.18006E+01_JPRB, 0.18462E+01_JPRB, 0.18810E+01_JPRB, 0.19880E+01_JPRB, 0.21422E+01_JPRB ,&
&  0.21531E+01_JPRB, 0.23008E+01_JPRB, 0.24842E+01_JPRB, 0.26368E+01_JPRB, 0.28081E+01_JPRB, 0.30734E+01_JPRB, 0.33834E+01_JPRB ,&
&  0.37081E+01_JPRB, 0.39781E+01_JPRB, 0.40793E+01_JPRB, 0.39885E+01_JPRB, 0.37862E+01_JPRB, 0.35401E+01_JPRB, 0.32253E+01_JPRB ,&
&  0.28173E+01_JPRB, 0.25383E+01_JPRB, 0.23588E+01_JPRB, 0.21877E+01_JPRB, 0.20312E+01_JPRB, 0.18216E+01_JPRB, 0.15086E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.10405E+01_JPRB, 0.10120E+01_JPRB, 0.10089E+01_JPRB, 0.10199E+01_JPRB, 0.10413E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.10458E+01_JPRB, 0.10406E+01_JPRB, 0.10771E+01_JPRB, 0.11219E+01_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.10979E+01_JPRB, 0.10861E+01_JPRB, 0.10900E+01_JPRB, 0.11027E+01_JPRB, 0.10766E+01_JPRB, 0.10172E+01_JPRB, 0.95978E+00_JPRB ,&
&  0.10747E+01_JPRB, 0.10720E+01_JPRB, 0.11222E+01_JPRB, 0.95835E+00_JPRB, 0.90277E+00_JPRB, 0.87248E+00_JPRB, 0.82591E+00_JPRB ,&
&  0.79013E+00_JPRB, 0.77048E+00_JPRB, 0.76467E+00_JPRB, 0.78064E+00_JPRB, 0.79018E+00_JPRB, 0.78762E+00_JPRB, 0.78416E+00_JPRB ,&
&  0.78023E+00_JPRB, 0.78167E+00_JPRB, 0.79088E+00_JPRB, 0.79214E+00_JPRB, 0.79144E+00_JPRB, 0.79244E+00_JPRB, 0.79677E+00_JPRB ,&
&  0.77466E+00_JPRB, 0.76831E+00_JPRB, 0.77174E+00_JPRB, 0.77283E+00_JPRB, 0.78744E+00_JPRB, 0.80086E+00_JPRB, 0.82893E+00_JPRB ,&
&  0.88144E+00_JPRB, 0.86052E+00_JPRB, 0.96350E+00_JPRB, 0.98980E+00_JPRB, 0.10182E+01_JPRB, 0.10437E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.11147E+01_JPRB, 0.11506E+01_JPRB, 0.11802E+01_JPRB, 0.12131E+01_JPRB, 0.12952E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.16108E+01_JPRB, 0.14598E+01_JPRB, 0.15914E+01_JPRB, 0.16734E+01_JPRB, 0.19812E+01_JPRB, 0.22960E+01_JPRB, 0.23002E+01_JPRB ,&
&  0.26147E+01_JPRB, 0.24586E+01_JPRB, 0.23237E+01_JPRB, 0.22098E+01_JPRB, 0.22849E+01_JPRB, 0.24414E+01_JPRB, 0.26161E+01_JPRB ,&
&  0.27653E+01_JPRB, 0.28848E+01_JPRB, 0.31084E+01_JPRB, 0.32789E+01_JPRB, 0.34669E+01_JPRB, 0.37046E+01_JPRB, 0.39332E+01_JPRB ,&
&  0.41400E+01_JPRB, 0.42684E+01_JPRB, 0.43328E+01_JPRB, 0.40767E+01_JPRB, 0.38598E+01_JPRB, 0.36337E+01_JPRB, 0.31751E+01_JPRB ,&
&  0.28149E+01_JPRB, 0.25568E+01_JPRB, 0.23481E+01_JPRB, 0.21977E+01_JPRB, 0.19292E+01_JPRB, 0.17068E+01_JPRB, 0.14880E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.10917E+01_JPRB, 0.10984E+01_JPRB, 0.11194E+01_JPRB, 0.11224E+01_JPRB, 0.11328E+01_JPRB, 0.11394E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.11076E+01_JPRB, 0.11427E+01_JPRB, 0.11351E+01_JPRB, 0.11167E+01_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.11703E+01_JPRB, 0.11847E+01_JPRB, 0.11877E+01_JPRB, 0.11666E+01_JPRB, 0.11331E+01_JPRB, 0.11414E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.12366E+01_JPRB, 0.12362E+01_JPRB, 0.12360E+01_JPRB, 0.12427E+01_JPRB, 0.11401E+01_JPRB, 0.97095E+00_JPRB ,&
&  0.88891E+00_JPRB, 0.87572E+00_JPRB, 0.87774E+00_JPRB, 0.87858E+00_JPRB, 0.88149E+00_JPRB, 0.87885E+00_JPRB, 0.88194E+00_JPRB ,&
&  0.88206E+00_JPRB, 0.87250E+00_JPRB, 0.87270E+00_JPRB, 0.86715E+00_JPRB, 0.85557E+00_JPRB, 0.84910E+00_JPRB, 0.84971E+00_JPRB ,&
&  0.81284E+00_JPRB, 0.79170E+00_JPRB, 0.77844E+00_JPRB, 0.76303E+00_JPRB, 0.79481E+00_JPRB, 0.82489E+00_JPRB, 0.87075E+00_JPRB ,&
&  0.93459E+00_JPRB, 0.99946E+00_JPRB, 0.10450E+01_JPRB, 0.10869E+01_JPRB, 0.11211E+01_JPRB, 0.11520E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12411E+01_JPRB, 0.12771E+01_JPRB, 0.13045E+01_JPRB, 0.13672E+01_JPRB, 0.15174E+01_JPRB, 0.18327E+01_JPRB ,&
&  0.19671E+01_JPRB, 0.21445E+01_JPRB, 0.22751E+01_JPRB, 0.20773E+01_JPRB, 0.22784E+01_JPRB, 0.25591E+01_JPRB, 0.25761E+01_JPRB ,&
&  0.26603E+01_JPRB, 0.26536E+01_JPRB, 0.27412E+01_JPRB, 0.28773E+01_JPRB, 0.32277E+01_JPRB, 0.34827E+01_JPRB, 0.35918E+01_JPRB ,&
&  0.36715E+01_JPRB, 0.39720E+01_JPRB, 0.41884E+01_JPRB, 0.42836E+01_JPRB, 0.42962E+01_JPRB, 0.43360E+01_JPRB, 0.44162E+01_JPRB ,&
&  0.44824E+01_JPRB, 0.45495E+01_JPRB, 0.45241E+01_JPRB, 0.43667E+01_JPRB, 0.41224E+01_JPRB, 0.37993E+01_JPRB, 0.33570E+01_JPRB ,&
&  0.29713E+01_JPRB, 0.25095E+01_JPRB, 0.21357E+01_JPRB, 0.19151E+01_JPRB, 0.17126E+01_JPRB, 0.15122E+01_JPRB, 0.13753E+01_JPRB ,&
&  0.12350E+01_JPRB, 0.12209E+01_JPRB, 0.13244E+01_JPRB, 0.13373E+01_JPRB, 0.13384E+01_JPRB, 0.13016E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.12020E+01_JPRB, 0.12209E+01_JPRB, 0.12033E+01_JPRB, 0.11789E+01_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.13697E+01_JPRB, 0.13522E+01_JPRB, 0.13250E+01_JPRB, 0.12914E+01_JPRB, 0.12822E+01_JPRB, 0.12877E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.13129E+01_JPRB, 0.13425E+01_JPRB, 0.13426E+01_JPRB, 0.13160E+01_JPRB, 0.12817E+01_JPRB, 0.12553E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.99188E+00_JPRB, 0.98855E+00_JPRB, 0.10376E+01_JPRB, 0.10991E+01_JPRB, 0.11101E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.10876E+01_JPRB, 0.10785E+01_JPRB, 0.10674E+01_JPRB, 0.10441E+01_JPRB, 0.10204E+01_JPRB, 0.99532E+00_JPRB ,&
&  0.91646E+00_JPRB, 0.86658E+00_JPRB, 0.84320E+00_JPRB, 0.83157E+00_JPRB, 0.86237E+00_JPRB, 0.91907E+00_JPRB, 0.98641E+00_JPRB ,&
&  0.10595E+01_JPRB, 0.11203E+01_JPRB, 0.11788E+01_JPRB, 0.12307E+01_JPRB, 0.12662E+01_JPRB, 0.12989E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.13667E+01_JPRB, 0.14060E+01_JPRB, 0.14300E+01_JPRB, 0.14722E+01_JPRB, 0.15499E+01_JPRB, 0.20305E+01_JPRB, 0.26256E+01_JPRB ,&
&  0.25744E+01_JPRB, 0.26164E+01_JPRB, 0.27940E+01_JPRB, 0.30462E+01_JPRB, 0.31557E+01_JPRB, 0.30773E+01_JPRB, 0.31252E+01_JPRB ,&
&  0.33969E+01_JPRB, 0.37201E+01_JPRB, 0.43687E+01_JPRB, 0.44119E+01_JPRB, 0.45031E+01_JPRB, 0.46314E+01_JPRB, 0.45685E+01_JPRB ,&
&  0.46456E+01_JPRB, 0.46857E+01_JPRB, 0.48333E+01_JPRB, 0.47711E+01_JPRB, 0.46619E+01_JPRB, 0.46831E+01_JPRB, 0.46629E+01_JPRB ,&
&  0.47188E+01_JPRB, 0.47382E+01_JPRB, 0.44850E+01_JPRB, 0.42653E+01_JPRB, 0.38674E+01_JPRB, 0.38690E+01_JPRB, 0.35692E+01_JPRB ,&
&  0.29908E+01_JPRB, 0.22256E+01_JPRB, 0.20288E+01_JPRB, 0.18158E+01_JPRB, 0.16807E+01_JPRB, 0.15434E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.13940E+01_JPRB, 0.15017E+01_JPRB, 0.15231E+01_JPRB, 0.15341E+01_JPRB, 0.15459E+01_JPRB, 0.15024E+01_JPRB, 0.13523E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.13792E+01_JPRB, 0.14022E+01_JPRB, 0.13928E+01_JPRB, 0.13803E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.16452E+01_JPRB, 0.15839E+01_JPRB, 0.15338E+01_JPRB, 0.15034E+01_JPRB, 0.14748E+01_JPRB, 0.14369E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.14380E+01_JPRB, 0.14397E+01_JPRB, 0.13903E+01_JPRB, 0.13322E+01_JPRB, 0.12901E+01_JPRB, 0.12836E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.11773E+01_JPRB, 0.12155E+01_JPRB, 0.13481E+01_JPRB, 0.16432E+01_JPRB, 0.16197E+01_JPRB, 0.16572E+01_JPRB ,&
&  0.16112E+01_JPRB, 0.15524E+01_JPRB, 0.15136E+01_JPRB, 0.14812E+01_JPRB, 0.14206E+01_JPRB, 0.13740E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.11585E+01_JPRB, 0.10514E+01_JPRB, 0.10148E+01_JPRB, 0.99956E+00_JPRB, 0.10423E+01_JPRB, 0.10971E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.12564E+01_JPRB, 0.13282E+01_JPRB, 0.13940E+01_JPRB, 0.14533E+01_JPRB, 0.14918E+01_JPRB, 0.15263E+01_JPRB, 0.15552E+01_JPRB ,&
&  0.15888E+01_JPRB, 0.16149E+01_JPRB, 0.16485E+01_JPRB, 0.16922E+01_JPRB, 0.19218E+01_JPRB, 0.28329E+01_JPRB, 0.37959E+01_JPRB ,&
&  0.37223E+01_JPRB, 0.37828E+01_JPRB, 0.48669E+01_JPRB, 0.58818E+01_JPRB, 0.52983E+01_JPRB, 0.46885E+01_JPRB, 0.44809E+01_JPRB ,&
&  0.47735E+01_JPRB, 0.52331E+01_JPRB, 0.57157E+01_JPRB, 0.58273E+01_JPRB, 0.56940E+01_JPRB, 0.57078E+01_JPRB, 0.56273E+01_JPRB ,&
&  0.54618E+01_JPRB, 0.53582E+01_JPRB, 0.51939E+01_JPRB, 0.50248E+01_JPRB, 0.48760E+01_JPRB, 0.49625E+01_JPRB, 0.49260E+01_JPRB ,&
&  0.48282E+01_JPRB, 0.45940E+01_JPRB, 0.42202E+01_JPRB, 0.40134E+01_JPRB, 0.33479E+01_JPRB, 0.32525E+01_JPRB, 0.36628E+01_JPRB ,&
&  0.30336E+01_JPRB, 0.24079E+01_JPRB, 0.20773E+01_JPRB, 0.19095E+01_JPRB, 0.18789E+01_JPRB, 0.20154E+01_JPRB, 0.20299E+01_JPRB ,&
&  0.17856E+01_JPRB, 0.18366E+01_JPRB, 0.19878E+01_JPRB, 0.20330E+01_JPRB, 0.20258E+01_JPRB, 0.19820E+01_JPRB, 0.18026E+01_JPRB ,&
&  0.17882E+01_JPRB, 0.17321E+01_JPRB, 0.17330E+01_JPRB, 0.17241E+01_JPRB, 0.16975E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.19270E+01_JPRB, 0.18164E+01_JPRB, 0.17512E+01_JPRB, 0.16738E+01_JPRB, 0.15864E+01_JPRB, 0.15046E+01_JPRB, 0.14556E+01_JPRB ,&
&  0.14889E+01_JPRB, 0.15073E+01_JPRB, 0.14846E+01_JPRB, 0.14238E+01_JPRB, 0.13722E+01_JPRB, 0.13390E+01_JPRB, 0.13500E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.14036E+01_JPRB, 0.14282E+01_JPRB, 0.14591E+01_JPRB, 0.18128E+01_JPRB, 0.20699E+01_JPRB, 0.22934E+01_JPRB ,&
&  0.23525E+01_JPRB, 0.23843E+01_JPRB, 0.23530E+01_JPRB, 0.22667E+01_JPRB, 0.21576E+01_JPRB, 0.20458E+01_JPRB, 0.19023E+01_JPRB ,&
&  0.17272E+01_JPRB, 0.15427E+01_JPRB, 0.14406E+01_JPRB, 0.14208E+01_JPRB, 0.14722E+01_JPRB, 0.13756E+01_JPRB, 0.14662E+01_JPRB ,&
&  0.15501E+01_JPRB, 0.16372E+01_JPRB, 0.16873E+01_JPRB, 0.17235E+01_JPRB, 0.17324E+01_JPRB, 0.17412E+01_JPRB, 0.17469E+01_JPRB ,&
&  0.17840E+01_JPRB, 0.17724E+01_JPRB, 0.16954E+01_JPRB, 0.18264E+01_JPRB, 0.23037E+01_JPRB, 0.35025E+01_JPRB, 0.47180E+01_JPRB ,&
&  0.52017E+01_JPRB, 0.62953E+01_JPRB, 0.78558E+01_JPRB, 0.83145E+01_JPRB, 0.71627E+01_JPRB, 0.63758E+01_JPRB, 0.63100E+01_JPRB ,&
&  0.66704E+01_JPRB, 0.71193E+01_JPRB, 0.72785E+01_JPRB, 0.68704E+01_JPRB, 0.68032E+01_JPRB, 0.66965E+01_JPRB, 0.64804E+01_JPRB ,&
&  0.61193E+01_JPRB, 0.57056E+01_JPRB, 0.54252E+01_JPRB, 0.51929E+01_JPRB, 0.50211E+01_JPRB, 0.46346E+01_JPRB, 0.45154E+01_JPRB ,&
&  0.46058E+01_JPRB, 0.38704E+01_JPRB, 0.34923E+01_JPRB, 0.36706E+01_JPRB, 0.30290E+01_JPRB, 0.29740E+01_JPRB, 0.32176E+01_JPRB ,&
&  0.27534E+01_JPRB, 0.23277E+01_JPRB, 0.21556E+01_JPRB, 0.20459E+01_JPRB, 0.21366E+01_JPRB, 0.26441E+01_JPRB, 0.30005E+01_JPRB ,&
&  0.28032E+01_JPRB, 0.26117E+01_JPRB, 0.29229E+01_JPRB, 0.30666E+01_JPRB, 0.31257E+01_JPRB, 0.29667E+01_JPRB, 0.27191E+01_JPRB ,&
&  0.25400E+01_JPRB, 0.24211E+01_JPRB, 0.23211E+01_JPRB, 0.22056E+01_JPRB, 0.20802E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.21181E+01_JPRB, 0.20557E+01_JPRB, 0.19536E+01_JPRB, 0.18148E+01_JPRB, 0.16937E+01_JPRB, 0.16013E+01_JPRB, 0.15240E+01_JPRB ,&
&  0.14954E+01_JPRB, 0.14585E+01_JPRB, 0.14264E+01_JPRB, 0.13830E+01_JPRB, 0.13642E+01_JPRB, 0.13651E+01_JPRB, 0.13743E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.14797E+01_JPRB, 0.14288E+01_JPRB, 0.14677E+01_JPRB, 0.17374E+01_JPRB, 0.25548E+01_JPRB, 0.31533E+01_JPRB ,&
&  0.33968E+01_JPRB, 0.34012E+01_JPRB, 0.33839E+01_JPRB, 0.34567E+01_JPRB, 0.34120E+01_JPRB, 0.31879E+01_JPRB, 0.30501E+01_JPRB ,&
&  0.28936E+01_JPRB, 0.26488E+01_JPRB, 0.22266E+01_JPRB, 0.21772E+01_JPRB, 0.20649E+01_JPRB, 0.18708E+01_JPRB, 0.20178E+01_JPRB ,&
&  0.20538E+01_JPRB, 0.20351E+01_JPRB, 0.19692E+01_JPRB, 0.19067E+01_JPRB, 0.18602E+01_JPRB, 0.17588E+01_JPRB, 0.16859E+01_JPRB ,&
&  0.16812E+01_JPRB, 0.17374E+01_JPRB, 0.18616E+01_JPRB, 0.22867E+01_JPRB, 0.33363E+01_JPRB, 0.48758E+01_JPRB, 0.60702E+01_JPRB ,&
&  0.71149E+01_JPRB, 0.76920E+01_JPRB, 0.73253E+01_JPRB, 0.79292E+01_JPRB, 0.88228E+01_JPRB, 0.83599E+01_JPRB, 0.78406E+01_JPRB ,&
&  0.77882E+01_JPRB, 0.76815E+01_JPRB, 0.74140E+01_JPRB, 0.72096E+01_JPRB, 0.73209E+01_JPRB, 0.71606E+01_JPRB, 0.67593E+01_JPRB ,&
&  0.58363E+01_JPRB, 0.54975E+01_JPRB, 0.54116E+01_JPRB, 0.52932E+01_JPRB, 0.47315E+01_JPRB, 0.44881E+01_JPRB, 0.45472E+01_JPRB ,&
&  0.44492E+01_JPRB, 0.43935E+01_JPRB, 0.35219E+01_JPRB, 0.33551E+01_JPRB, 0.26111E+01_JPRB, 0.25371E+01_JPRB, 0.28903E+01_JPRB ,&
&  0.26551E+01_JPRB, 0.24743E+01_JPRB, 0.22777E+01_JPRB, 0.23492E+01_JPRB, 0.32335E+01_JPRB, 0.41741E+01_JPRB, 0.51491E+01_JPRB ,&
&  0.48332E+01_JPRB, 0.39539E+01_JPRB, 0.41586E+01_JPRB, 0.42362E+01_JPRB, 0.43379E+01_JPRB, 0.40611E+01_JPRB, 0.36023E+01_JPRB ,&
&  0.32356E+01_JPRB, 0.29469E+01_JPRB, 0.26584E+01_JPRB, 0.23914E+01_JPRB, 0.22182E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.21683E+01_JPRB, 0.21068E+01_JPRB, 0.19863E+01_JPRB, 0.18331E+01_JPRB, 0.17138E+01_JPRB, 0.16206E+01_JPRB, 0.15430E+01_JPRB ,&
&  0.14655E+01_JPRB, 0.14021E+01_JPRB, 0.13624E+01_JPRB, 0.13422E+01_JPRB, 0.13397E+01_JPRB, 0.13459E+01_JPRB, 0.13538E+01_JPRB ,&
&  0.13603E+01_JPRB, 0.14807E+01_JPRB, 0.14515E+01_JPRB, 0.14871E+01_JPRB, 0.19100E+01_JPRB, 0.28801E+01_JPRB, 0.38040E+01_JPRB ,&
&  0.41931E+01_JPRB, 0.41402E+01_JPRB, 0.42017E+01_JPRB, 0.43720E+01_JPRB, 0.46364E+01_JPRB, 0.47459E+01_JPRB, 0.47418E+01_JPRB ,&
&  0.45691E+01_JPRB, 0.35818E+01_JPRB, 0.32035E+01_JPRB, 0.28916E+01_JPRB, 0.27537E+01_JPRB, 0.26496E+01_JPRB, 0.25223E+01_JPRB ,&
&  0.23340E+01_JPRB, 0.21043E+01_JPRB, 0.19302E+01_JPRB, 0.18005E+01_JPRB, 0.17312E+01_JPRB, 0.16496E+01_JPRB, 0.17020E+01_JPRB ,&
&  0.18638E+01_JPRB, 0.20686E+01_JPRB, 0.27390E+01_JPRB, 0.38641E+01_JPRB, 0.57402E+01_JPRB, 0.74058E+01_JPRB, 0.76129E+01_JPRB ,&
&  0.65584E+01_JPRB, 0.68307E+01_JPRB, 0.98825E+01_JPRB, 0.90152E+01_JPRB, 0.90266E+01_JPRB, 0.97931E+01_JPRB, 0.91676E+01_JPRB ,&
&  0.82056E+01_JPRB, 0.74441E+01_JPRB, 0.69948E+01_JPRB, 0.68836E+01_JPRB, 0.80939E+01_JPRB, 0.79072E+01_JPRB, 0.70471E+01_JPRB ,&
&  0.60724E+01_JPRB, 0.58674E+01_JPRB, 0.55038E+01_JPRB, 0.50993E+01_JPRB, 0.47992E+01_JPRB, 0.50198E+01_JPRB, 0.47321E+01_JPRB ,&
&  0.44634E+01_JPRB, 0.49652E+01_JPRB, 0.40052E+01_JPRB, 0.31134E+01_JPRB, 0.26115E+01_JPRB, 0.27120E+01_JPRB, 0.29270E+01_JPRB ,&
&  0.29555E+01_JPRB, 0.29413E+01_JPRB, 0.30830E+01_JPRB, 0.41231E+01_JPRB, 0.62384E+01_JPRB, 0.77331E+01_JPRB, 0.71535E+01_JPRB ,&
&  0.63923E+01_JPRB, 0.64972E+01_JPRB, 0.60818E+01_JPRB, 0.59538E+01_JPRB, 0.57602E+01_JPRB, 0.49811E+01_JPRB, 0.40234E+01_JPRB ,&
&  0.32166E+01_JPRB, 0.27106E+01_JPRB, 0.24456E+01_JPRB, 0.22981E+01_JPRB, 0.22161E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.22607E+01_JPRB, 0.21333E+01_JPRB, 0.19906E+01_JPRB, 0.18443E+01_JPRB, 0.17084E+01_JPRB, 0.15957E+01_JPRB, 0.15024E+01_JPRB ,&
&  0.14102E+01_JPRB, 0.13444E+01_JPRB, 0.13252E+01_JPRB, 0.13326E+01_JPRB, 0.13509E+01_JPRB, 0.13421E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.14075E+01_JPRB, 0.15017E+01_JPRB, 0.14765E+01_JPRB, 0.16391E+01_JPRB, 0.20729E+01_JPRB, 0.27628E+01_JPRB, 0.37313E+01_JPRB ,&
&  0.43052E+01_JPRB, 0.41248E+01_JPRB, 0.41138E+01_JPRB, 0.48990E+01_JPRB, 0.60154E+01_JPRB, 0.67200E+01_JPRB, 0.68513E+01_JPRB ,&
&  0.59892E+01_JPRB, 0.52336E+01_JPRB, 0.44997E+01_JPRB, 0.38654E+01_JPRB, 0.34237E+01_JPRB, 0.30216E+01_JPRB, 0.25511E+01_JPRB ,&
&  0.21387E+01_JPRB, 0.18756E+01_JPRB, 0.17516E+01_JPRB, 0.17047E+01_JPRB, 0.17271E+01_JPRB, 0.17901E+01_JPRB, 0.19791E+01_JPRB ,&
&  0.23201E+01_JPRB, 0.29388E+01_JPRB, 0.40614E+01_JPRB, 0.54864E+01_JPRB, 0.60212E+01_JPRB, 0.77569E+01_JPRB, 0.75477E+01_JPRB ,&
&  0.80044E+01_JPRB, 0.84743E+01_JPRB, 0.84508E+01_JPRB, 0.90035E+01_JPRB, 0.77786E+01_JPRB, 0.84313E+01_JPRB, 0.87407E+01_JPRB ,&
&  0.82705E+01_JPRB, 0.78235E+01_JPRB, 0.79230E+01_JPRB, 0.73074E+01_JPRB, 0.73827E+01_JPRB, 0.88870E+01_JPRB, 0.78562E+01_JPRB ,&
&  0.67030E+01_JPRB, 0.61830E+01_JPRB, 0.55328E+01_JPRB, 0.53026E+01_JPRB, 0.44831E+01_JPRB, 0.44828E+01_JPRB, 0.39891E+01_JPRB ,&
&  0.38693E+01_JPRB, 0.32419E+01_JPRB, 0.35778E+01_JPRB, 0.30595E+01_JPRB, 0.29364E+01_JPRB, 0.29468E+01_JPRB, 0.30812E+01_JPRB ,&
&  0.34647E+01_JPRB, 0.39746E+01_JPRB, 0.52682E+01_JPRB, 0.76555E+01_JPRB, 0.10449E+02_JPRB, 0.88450E+01_JPRB, 0.85781E+01_JPRB ,&
&  0.80068E+01_JPRB, 0.77025E+01_JPRB, 0.64674E+01_JPRB, 0.59745E+01_JPRB, 0.52677E+01_JPRB, 0.46813E+01_JPRB, 0.40771E+01_JPRB ,&
&  0.34613E+01_JPRB, 0.29500E+01_JPRB, 0.26676E+01_JPRB, 0.25027E+01_JPRB, 0.23782E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.21840E+01_JPRB, 0.20212E+01_JPRB, 0.18869E+01_JPRB, 0.17570E+01_JPRB, 0.16338E+01_JPRB, 0.15087E+01_JPRB, 0.13847E+01_JPRB ,&
&  0.12899E+01_JPRB, 0.12489E+01_JPRB, 0.12636E+01_JPRB, 0.12930E+01_JPRB, 0.13232E+01_JPRB, 0.13641E+01_JPRB, 0.13967E+01_JPRB ,&
&  0.14728E+01_JPRB, 0.15671E+01_JPRB, 0.15544E+01_JPRB, 0.16572E+01_JPRB, 0.20773E+01_JPRB, 0.26846E+01_JPRB, 0.32602E+01_JPRB ,&
&  0.41815E+01_JPRB, 0.36486E+01_JPRB, 0.36355E+01_JPRB, 0.47725E+01_JPRB, 0.73570E+01_JPRB, 0.91694E+01_JPRB, 0.91039E+01_JPRB ,&
&  0.88973E+01_JPRB, 0.73558E+01_JPRB, 0.56144E+01_JPRB, 0.43040E+01_JPRB, 0.32516E+01_JPRB, 0.24982E+01_JPRB, 0.19634E+01_JPRB ,&
&  0.16903E+01_JPRB, 0.15747E+01_JPRB, 0.15618E+01_JPRB, 0.15905E+01_JPRB, 0.16902E+01_JPRB, 0.18010E+01_JPRB, 0.22598E+01_JPRB ,&
&  0.27983E+01_JPRB, 0.35330E+01_JPRB, 0.44319E+01_JPRB, 0.50552E+01_JPRB, 0.52930E+01_JPRB, 0.68590E+01_JPRB, 0.80728E+01_JPRB ,&
&  0.74771E+01_JPRB, 0.75684E+01_JPRB, 0.82965E+01_JPRB, 0.86275E+01_JPRB, 0.90499E+01_JPRB, 0.82942E+01_JPRB, 0.80156E+01_JPRB ,&
&  0.61765E+01_JPRB, 0.52627E+01_JPRB, 0.49624E+01_JPRB, 0.53371E+01_JPRB, 0.71725E+01_JPRB, 0.93095E+01_JPRB, 0.86643E+01_JPRB ,&
&  0.74853E+01_JPRB, 0.67643E+01_JPRB, 0.60339E+01_JPRB, 0.48388E+01_JPRB, 0.45502E+01_JPRB, 0.38933E+01_JPRB, 0.46794E+01_JPRB ,&
&  0.46775E+01_JPRB, 0.44105E+01_JPRB, 0.37364E+01_JPRB, 0.29401E+01_JPRB, 0.25288E+01_JPRB, 0.32073E+01_JPRB, 0.36478E+01_JPRB ,&
&  0.44420E+01_JPRB, 0.54899E+01_JPRB, 0.98188E+01_JPRB, 0.11297E+02_JPRB, 0.11400E+02_JPRB, 0.89580E+01_JPRB, 0.93769E+01_JPRB ,&
&  0.82842E+01_JPRB, 0.79300E+01_JPRB, 0.66651E+01_JPRB, 0.51006E+01_JPRB, 0.41002E+01_JPRB, 0.35436E+01_JPRB, 0.32610E+01_JPRB ,&
&  0.31503E+01_JPRB, 0.30321E+01_JPRB, 0.28254E+01_JPRB, 0.25989E+01_JPRB, 0.24022E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.18516E+01_JPRB, 0.16922E+01_JPRB, 0.15854E+01_JPRB, 0.14980E+01_JPRB, 0.14010E+01_JPRB, 0.13083E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.11741E+01_JPRB, 0.12089E+01_JPRB, 0.12720E+01_JPRB, 0.13354E+01_JPRB, 0.13984E+01_JPRB, 0.14395E+01_JPRB ,&
&  0.15161E+01_JPRB, 0.16915E+01_JPRB, 0.16998E+01_JPRB, 0.18340E+01_JPRB, 0.22233E+01_JPRB, 0.26299E+01_JPRB, 0.31726E+01_JPRB ,&
&  0.39231E+01_JPRB, 0.33371E+01_JPRB, 0.35341E+01_JPRB, 0.47502E+01_JPRB, 0.84853E+01_JPRB, 0.97553E+01_JPRB, 0.91267E+01_JPRB ,&
&  0.88331E+01_JPRB, 0.66610E+01_JPRB, 0.44124E+01_JPRB, 0.32009E+01_JPRB, 0.24366E+01_JPRB, 0.18970E+01_JPRB, 0.16521E+01_JPRB ,&
&  0.15784E+01_JPRB, 0.15725E+01_JPRB, 0.16033E+01_JPRB, 0.16635E+01_JPRB, 0.17675E+01_JPRB, 0.20584E+01_JPRB, 0.26365E+01_JPRB ,&
&  0.32384E+01_JPRB, 0.38360E+01_JPRB, 0.42485E+01_JPRB, 0.44160E+01_JPRB, 0.47135E+01_JPRB, 0.62166E+01_JPRB, 0.85693E+01_JPRB ,&
&  0.77835E+01_JPRB, 0.80609E+01_JPRB, 0.83930E+01_JPRB, 0.89787E+01_JPRB, 0.83783E+01_JPRB, 0.78019E+01_JPRB, 0.72668E+01_JPRB ,&
&  0.54886E+01_JPRB, 0.49871E+01_JPRB, 0.56329E+01_JPRB, 0.65022E+01_JPRB, 0.67505E+01_JPRB, 0.77847E+01_JPRB, 0.94842E+01_JPRB ,&
&  0.72742E+01_JPRB, 0.66388E+01_JPRB, 0.62348E+01_JPRB, 0.58333E+01_JPRB, 0.34890E+01_JPRB, 0.25656E+01_JPRB, 0.37523E+01_JPRB ,&
&  0.36389E+01_JPRB, 0.21460E+01_JPRB, 0.17328E+01_JPRB, 0.23183E+01_JPRB, 0.17232E+01_JPRB, 0.23066E+01_JPRB, 0.42512E+01_JPRB ,&
&  0.56494E+01_JPRB, 0.73871E+01_JPRB, 0.13935E+02_JPRB, 0.11950E+02_JPRB, 0.11385E+02_JPRB, 0.11046E+02_JPRB, 0.10591E+02_JPRB ,&
&  0.95857E+01_JPRB, 0.60950E+01_JPRB, 0.44121E+01_JPRB, 0.33788E+01_JPRB, 0.28258E+01_JPRB, 0.27489E+01_JPRB, 0.26877E+01_JPRB ,&
&  0.26350E+01_JPRB, 0.25014E+01_JPRB, 0.23717E+01_JPRB, 0.21828E+01_JPRB, 0.20315E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.14185E+01_JPRB, 0.13136E+01_JPRB, 0.12828E+01_JPRB, 0.12548E+01_JPRB, 0.11885E+01_JPRB, 0.11423E+01_JPRB, 0.11039E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.11576E+01_JPRB, 0.12231E+01_JPRB, 0.13100E+01_JPRB, 0.14100E+01_JPRB, 0.14943E+01_JPRB, 0.15252E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.17470E+01_JPRB, 0.20189E+01_JPRB, 0.23404E+01_JPRB, 0.28411E+01_JPRB, 0.27696E+01_JPRB, 0.37448E+01_JPRB ,&
&  0.37658E+01_JPRB, 0.31481E+01_JPRB, 0.36578E+01_JPRB, 0.50413E+01_JPRB, 0.70571E+01_JPRB, 0.75091E+01_JPRB, 0.68105E+01_JPRB ,&
&  0.55211E+01_JPRB, 0.39554E+01_JPRB, 0.29498E+01_JPRB, 0.23266E+01_JPRB, 0.19396E+01_JPRB, 0.17313E+01_JPRB, 0.17313E+01_JPRB ,&
&  0.16904E+01_JPRB, 0.17125E+01_JPRB, 0.17721E+01_JPRB, 0.17718E+01_JPRB, 0.19769E+01_JPRB, 0.23390E+01_JPRB, 0.28995E+01_JPRB ,&
&  0.33996E+01_JPRB, 0.37886E+01_JPRB, 0.39919E+01_JPRB, 0.40606E+01_JPRB, 0.53528E+01_JPRB, 0.66808E+01_JPRB, 0.68439E+01_JPRB ,&
&  0.67581E+01_JPRB, 0.69675E+01_JPRB, 0.86723E+01_JPRB, 0.86319E+01_JPRB, 0.79420E+01_JPRB, 0.80832E+01_JPRB, 0.71851E+01_JPRB ,&
&  0.67340E+01_JPRB, 0.67305E+01_JPRB, 0.64270E+01_JPRB, 0.71246E+01_JPRB, 0.78579E+01_JPRB, 0.75281E+01_JPRB, 0.78459E+01_JPRB ,&
&  0.75277E+01_JPRB, 0.63247E+01_JPRB, 0.48508E+01_JPRB, 0.38179E+01_JPRB, 0.37768E+01_JPRB, 0.31620E+01_JPRB, 0.21753E+01_JPRB ,&
&  0.18271E+01_JPRB, 0.15810E+01_JPRB, 0.12283E+01_JPRB, 0.14129E+01_JPRB, 0.13657E+01_JPRB, 0.20787E+01_JPRB, 0.42205E+01_JPRB ,&
&  0.81503E+01_JPRB, 0.10993E+02_JPRB, 0.14032E+02_JPRB, 0.11813E+02_JPRB, 0.10867E+02_JPRB, 0.11333E+02_JPRB, 0.11143E+02_JPRB ,&
&  0.53701E+01_JPRB, 0.35129E+01_JPRB, 0.28017E+01_JPRB, 0.25207E+01_JPRB, 0.24176E+01_JPRB, 0.23382E+01_JPRB, 0.22500E+01_JPRB ,&
&  0.21049E+01_JPRB, 0.19744E+01_JPRB, 0.19113E+01_JPRB, 0.18119E+01_JPRB, 0.16124E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.12050E+01_JPRB, 0.11825E+01_JPRB, 0.11869E+01_JPRB, 0.11572E+01_JPRB, 0.10927E+01_JPRB, 0.11021E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.12414E+01_JPRB, 0.13327E+01_JPRB, 0.14583E+01_JPRB, 0.16208E+01_JPRB, 0.16952E+01_JPRB, 0.16990E+01_JPRB ,&
&  0.17446E+01_JPRB, 0.19706E+01_JPRB, 0.24231E+01_JPRB, 0.30571E+01_JPRB, 0.36863E+01_JPRB, 0.32616E+01_JPRB, 0.43185E+01_JPRB ,&
&  0.35023E+01_JPRB, 0.30122E+01_JPRB, 0.35371E+01_JPRB, 0.44986E+01_JPRB, 0.55008E+01_JPRB, 0.51561E+01_JPRB, 0.39867E+01_JPRB ,&
&  0.29756E+01_JPRB, 0.25226E+01_JPRB, 0.22554E+01_JPRB, 0.20038E+01_JPRB, 0.18001E+01_JPRB, 0.17496E+01_JPRB, 0.17853E+01_JPRB ,&
&  0.17704E+01_JPRB, 0.18250E+01_JPRB, 0.18478E+01_JPRB, 0.20153E+01_JPRB, 0.23519E+01_JPRB, 0.27326E+01_JPRB, 0.31682E+01_JPRB ,&
&  0.34552E+01_JPRB, 0.36954E+01_JPRB, 0.39194E+01_JPRB, 0.46190E+01_JPRB, 0.50967E+01_JPRB, 0.53051E+01_JPRB, 0.62636E+01_JPRB ,&
&  0.74286E+01_JPRB, 0.82488E+01_JPRB, 0.78152E+01_JPRB, 0.89192E+01_JPRB, 0.82911E+01_JPRB, 0.76451E+01_JPRB, 0.71246E+01_JPRB ,&
&  0.68246E+01_JPRB, 0.75103E+01_JPRB, 0.70834E+01_JPRB, 0.74381E+01_JPRB, 0.93737E+01_JPRB, 0.93105E+01_JPRB, 0.77447E+01_JPRB ,&
&  0.75394E+01_JPRB, 0.59955E+01_JPRB, 0.48462E+01_JPRB, 0.40104E+01_JPRB, 0.51894E+01_JPRB, 0.56927E+01_JPRB, 0.29406E+01_JPRB ,&
&  0.20007E+01_JPRB, 0.15718E+01_JPRB, 0.12180E+01_JPRB, 0.12307E+01_JPRB, 0.13923E+01_JPRB, 0.21146E+01_JPRB, 0.60301E+01_JPRB ,&
&  0.76326E+01_JPRB, 0.12813E+02_JPRB, 0.14769E+02_JPRB, 0.13114E+02_JPRB, 0.11353E+02_JPRB, 0.95364E+01_JPRB, 0.55597E+01_JPRB ,&
&  0.30961E+01_JPRB, 0.21049E+01_JPRB, 0.20510E+01_JPRB, 0.22459E+01_JPRB, 0.22995E+01_JPRB, 0.21597E+01_JPRB, 0.19578E+01_JPRB ,&
&  0.18208E+01_JPRB, 0.17651E+01_JPRB, 0.16651E+01_JPRB, 0.15621E+01_JPRB, 0.13775E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.11946E+01_JPRB, 0.12269E+01_JPRB, 0.12620E+01_JPRB, 0.11863E+01_JPRB, 0.11711E+01_JPRB, 0.11975E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.12768E+01_JPRB, 0.13958E+01_JPRB, 0.15184E+01_JPRB, 0.17228E+01_JPRB, 0.19193E+01_JPRB, 0.19412E+01_JPRB, 0.19307E+01_JPRB ,&
&  0.20335E+01_JPRB, 0.23979E+01_JPRB, 0.30542E+01_JPRB, 0.39262E+01_JPRB, 0.42140E+01_JPRB, 0.39929E+01_JPRB, 0.44737E+01_JPRB ,&
&  0.34271E+01_JPRB, 0.29970E+01_JPRB, 0.31883E+01_JPRB, 0.37724E+01_JPRB, 0.38933E+01_JPRB, 0.36311E+01_JPRB, 0.26356E+01_JPRB ,&
&  0.22020E+01_JPRB, 0.21644E+01_JPRB, 0.19438E+01_JPRB, 0.19838E+01_JPRB, 0.18303E+01_JPRB, 0.18356E+01_JPRB, 0.18206E+01_JPRB ,&
&  0.18767E+01_JPRB, 0.19683E+01_JPRB, 0.20848E+01_JPRB, 0.23344E+01_JPRB, 0.27118E+01_JPRB, 0.30662E+01_JPRB, 0.33896E+01_JPRB ,&
&  0.35146E+01_JPRB, 0.38115E+01_JPRB, 0.42888E+01_JPRB, 0.50199E+01_JPRB, 0.44098E+01_JPRB, 0.57251E+01_JPRB, 0.62936E+01_JPRB ,&
&  0.71380E+01_JPRB, 0.75682E+01_JPRB, 0.71484E+01_JPRB, 0.77544E+01_JPRB, 0.80529E+01_JPRB, 0.75654E+01_JPRB, 0.70327E+01_JPRB ,&
&  0.75774E+01_JPRB, 0.76546E+01_JPRB, 0.79894E+01_JPRB, 0.84033E+01_JPRB, 0.99566E+01_JPRB, 0.12399E+02_JPRB, 0.78174E+01_JPRB ,&
&  0.62028E+01_JPRB, 0.61170E+01_JPRB, 0.51191E+01_JPRB, 0.45534E+01_JPRB, 0.57159E+01_JPRB, 0.68305E+01_JPRB, 0.64209E+01_JPRB ,&
&  0.27489E+01_JPRB, 0.16387E+01_JPRB, 0.15401E+01_JPRB, 0.15368E+01_JPRB, 0.13712E+01_JPRB, 0.22986E+01_JPRB, 0.10317E+02_JPRB ,&
&  0.81269E+01_JPRB, 0.12954E+02_JPRB, 0.13656E+02_JPRB, 0.12037E+02_JPRB, 0.10834E+02_JPRB, 0.68658E+01_JPRB, 0.36631E+01_JPRB ,&
&  0.17976E+01_JPRB, 0.17607E+01_JPRB, 0.19879E+01_JPRB, 0.19331E+01_JPRB, 0.19017E+01_JPRB, 0.17785E+01_JPRB, 0.15600E+01_JPRB ,&
&  0.15242E+01_JPRB, 0.15004E+01_JPRB, 0.13921E+01_JPRB, 0.12883E+01_JPRB, 0.12203E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.12582E+01_JPRB, 0.13052E+01_JPRB, 0.13076E+01_JPRB, 0.13019E+01_JPRB, 0.13223E+01_JPRB, 0.13470E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.16316E+01_JPRB, 0.18589E+01_JPRB, 0.21529E+01_JPRB, 0.23191E+01_JPRB, 0.23037E+01_JPRB, 0.23360E+01_JPRB ,&
&  0.25946E+01_JPRB, 0.31199E+01_JPRB, 0.38256E+01_JPRB, 0.43989E+01_JPRB, 0.43501E+01_JPRB, 0.39705E+01_JPRB, 0.41290E+01_JPRB ,&
&  0.35495E+01_JPRB, 0.29764E+01_JPRB, 0.25177E+01_JPRB, 0.28159E+01_JPRB, 0.31129E+01_JPRB, 0.32692E+01_JPRB, 0.25482E+01_JPRB ,&
&  0.23514E+01_JPRB, 0.22897E+01_JPRB, 0.21629E+01_JPRB, 0.21567E+01_JPRB, 0.21107E+01_JPRB, 0.21208E+01_JPRB, 0.21208E+01_JPRB ,&
&  0.21062E+01_JPRB, 0.22339E+01_JPRB, 0.24458E+01_JPRB, 0.27318E+01_JPRB, 0.30707E+01_JPRB, 0.33558E+01_JPRB, 0.36228E+01_JPRB ,&
&  0.38932E+01_JPRB, 0.42905E+01_JPRB, 0.47473E+01_JPRB, 0.47162E+01_JPRB, 0.48376E+01_JPRB, 0.53715E+01_JPRB, 0.60360E+01_JPRB ,&
&  0.67104E+01_JPRB, 0.72003E+01_JPRB, 0.72897E+01_JPRB, 0.72593E+01_JPRB, 0.74168E+01_JPRB, 0.74997E+01_JPRB, 0.75013E+01_JPRB ,&
&  0.82449E+01_JPRB, 0.86347E+01_JPRB, 0.83723E+01_JPRB, 0.86169E+01_JPRB, 0.91805E+01_JPRB, 0.96556E+01_JPRB, 0.82448E+01_JPRB ,&
&  0.66617E+01_JPRB, 0.54219E+01_JPRB, 0.49559E+01_JPRB, 0.55086E+01_JPRB, 0.67219E+01_JPRB, 0.70781E+01_JPRB, 0.92709E+01_JPRB ,&
&  0.91380E+01_JPRB, 0.36840E+01_JPRB, 0.25582E+01_JPRB, 0.34300E+01_JPRB, 0.20127E+01_JPRB, 0.22651E+01_JPRB, 0.49153E+01_JPRB ,&
&  0.82015E+01_JPRB, 0.11075E+02_JPRB, 0.11045E+02_JPRB, 0.92410E+01_JPRB, 0.83277E+01_JPRB, 0.45211E+01_JPRB, 0.21823E+01_JPRB ,&
&  0.16012E+01_JPRB, 0.16919E+01_JPRB, 0.16957E+01_JPRB, 0.15173E+01_JPRB, 0.14538E+01_JPRB, 0.15070E+01_JPRB, 0.13356E+01_JPRB ,&
&  0.13423E+01_JPRB, 0.13424E+01_JPRB, 0.13490E+01_JPRB, 0.13043E+01_JPRB, 0.12437E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.13990E+01_JPRB, 0.14372E+01_JPRB, 0.15081E+01_JPRB, 0.15546E+01_JPRB, 0.15685E+01_JPRB, 0.16123E+01_JPRB, 0.16827E+01_JPRB ,&
&  0.18608E+01_JPRB, 0.21131E+01_JPRB, 0.23984E+01_JPRB, 0.26714E+01_JPRB, 0.27499E+01_JPRB, 0.27630E+01_JPRB, 0.29455E+01_JPRB ,&
&  0.33126E+01_JPRB, 0.38314E+01_JPRB, 0.43674E+01_JPRB, 0.47925E+01_JPRB, 0.48617E+01_JPRB, 0.49443E+01_JPRB, 0.39660E+01_JPRB ,&
&  0.32848E+01_JPRB, 0.31991E+01_JPRB, 0.25801E+01_JPRB, 0.27797E+01_JPRB, 0.29918E+01_JPRB, 0.30901E+01_JPRB, 0.28184E+01_JPRB ,&
&  0.26534E+01_JPRB, 0.25729E+01_JPRB, 0.24560E+01_JPRB, 0.24115E+01_JPRB, 0.24361E+01_JPRB, 0.24730E+01_JPRB, 0.25243E+01_JPRB ,&
&  0.26293E+01_JPRB, 0.27368E+01_JPRB, 0.29176E+01_JPRB, 0.31704E+01_JPRB, 0.34341E+01_JPRB, 0.37126E+01_JPRB, 0.40508E+01_JPRB ,&
&  0.43340E+01_JPRB, 0.45389E+01_JPRB, 0.47554E+01_JPRB, 0.45475E+01_JPRB, 0.47497E+01_JPRB, 0.51873E+01_JPRB, 0.56990E+01_JPRB ,&
&  0.58470E+01_JPRB, 0.58990E+01_JPRB, 0.65875E+01_JPRB, 0.73407E+01_JPRB, 0.76328E+01_JPRB, 0.75316E+01_JPRB, 0.75897E+01_JPRB ,&
&  0.84357E+01_JPRB, 0.87788E+01_JPRB, 0.86428E+01_JPRB, 0.77241E+01_JPRB, 0.72867E+01_JPRB, 0.71775E+01_JPRB, 0.72857E+01_JPRB ,&
&  0.66139E+01_JPRB, 0.59015E+01_JPRB, 0.57481E+01_JPRB, 0.63166E+01_JPRB, 0.69878E+01_JPRB, 0.72334E+01_JPRB, 0.83809E+01_JPRB ,&
&  0.94116E+01_JPRB, 0.84293E+01_JPRB, 0.65963E+01_JPRB, 0.42594E+01_JPRB, 0.32242E+01_JPRB, 0.33101E+01_JPRB, 0.46275E+01_JPRB ,&
&  0.73005E+01_JPRB, 0.78115E+01_JPRB, 0.73625E+01_JPRB, 0.80458E+01_JPRB, 0.53774E+01_JPRB, 0.28215E+01_JPRB, 0.16355E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.14327E+01_JPRB, 0.14257E+01_JPRB, 0.13402E+01_JPRB, 0.13585E+01_JPRB, 0.13977E+01_JPRB, 0.13345E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14319E+01_JPRB, 0.13814E+01_JPRB, 0.13364E+01_JPRB, 0.13614E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.16580E+01_JPRB, 0.17389E+01_JPRB, 0.18101E+01_JPRB, 0.18510E+01_JPRB, 0.19334E+01_JPRB, 0.19849E+01_JPRB, 0.21208E+01_JPRB ,&
&  0.23640E+01_JPRB, 0.26297E+01_JPRB, 0.28757E+01_JPRB, 0.30195E+01_JPRB, 0.30756E+01_JPRB, 0.31889E+01_JPRB, 0.34472E+01_JPRB ,&
&  0.37437E+01_JPRB, 0.41007E+01_JPRB, 0.44214E+01_JPRB, 0.47636E+01_JPRB, 0.49850E+01_JPRB, 0.54768E+01_JPRB, 0.48180E+01_JPRB ,&
&  0.30900E+01_JPRB, 0.39817E+01_JPRB, 0.28067E+01_JPRB, 0.30737E+01_JPRB, 0.32159E+01_JPRB, 0.33865E+01_JPRB, 0.30671E+01_JPRB ,&
&  0.28493E+01_JPRB, 0.28063E+01_JPRB, 0.27440E+01_JPRB, 0.26541E+01_JPRB, 0.26121E+01_JPRB, 0.27303E+01_JPRB, 0.28876E+01_JPRB ,&
&  0.30266E+01_JPRB, 0.31266E+01_JPRB, 0.32646E+01_JPRB, 0.34659E+01_JPRB, 0.36889E+01_JPRB, 0.39456E+01_JPRB, 0.42231E+01_JPRB ,&
&  0.43775E+01_JPRB, 0.45343E+01_JPRB, 0.45090E+01_JPRB, 0.44419E+01_JPRB, 0.45261E+01_JPRB, 0.48127E+01_JPRB, 0.51817E+01_JPRB ,&
&  0.53953E+01_JPRB, 0.55765E+01_JPRB, 0.62321E+01_JPRB, 0.66516E+01_JPRB, 0.67357E+01_JPRB, 0.70295E+01_JPRB, 0.68986E+01_JPRB ,&
&  0.77107E+01_JPRB, 0.74754E+01_JPRB, 0.74973E+01_JPRB, 0.64133E+01_JPRB, 0.56620E+01_JPRB, 0.54953E+01_JPRB, 0.55229E+01_JPRB ,&
&  0.52612E+01_JPRB, 0.55263E+01_JPRB, 0.57043E+01_JPRB, 0.60771E+01_JPRB, 0.67103E+01_JPRB, 0.73292E+01_JPRB, 0.72178E+01_JPRB ,&
&  0.70030E+01_JPRB, 0.50653E+01_JPRB, 0.38389E+01_JPRB, 0.34096E+01_JPRB, 0.28983E+01_JPRB, 0.30071E+01_JPRB, 0.43589E+01_JPRB ,&
&  0.60094E+01_JPRB, 0.60901E+01_JPRB, 0.61346E+01_JPRB, 0.52949E+01_JPRB, 0.30152E+01_JPRB, 0.17961E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.12926E+01_JPRB, 0.12917E+01_JPRB, 0.12817E+01_JPRB, 0.13379E+01_JPRB, 0.12790E+01_JPRB, 0.13463E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14323E+01_JPRB, 0.14702E+01_JPRB, 0.14823E+01_JPRB, 0.15512E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.19387E+01_JPRB, 0.21194E+01_JPRB, 0.23070E+01_JPRB, 0.24469E+01_JPRB, 0.24510E+01_JPRB, 0.24705E+01_JPRB, 0.25581E+01_JPRB ,&
&  0.26778E+01_JPRB, 0.28349E+01_JPRB, 0.30009E+01_JPRB, 0.31444E+01_JPRB, 0.32625E+01_JPRB, 0.34574E+01_JPRB, 0.37467E+01_JPRB ,&
&  0.40699E+01_JPRB, 0.44046E+01_JPRB, 0.47346E+01_JPRB, 0.51336E+01_JPRB, 0.56249E+01_JPRB, 0.63937E+01_JPRB, 0.62267E+01_JPRB ,&
&  0.43753E+01_JPRB, 0.33397E+01_JPRB, 0.32797E+01_JPRB, 0.31400E+01_JPRB, 0.30566E+01_JPRB, 0.33083E+01_JPRB, 0.33275E+01_JPRB ,&
&  0.30915E+01_JPRB, 0.29406E+01_JPRB, 0.28729E+01_JPRB, 0.27788E+01_JPRB, 0.26855E+01_JPRB, 0.28530E+01_JPRB, 0.30251E+01_JPRB ,&
&  0.31762E+01_JPRB, 0.32756E+01_JPRB, 0.33856E+01_JPRB, 0.35413E+01_JPRB, 0.37186E+01_JPRB, 0.38928E+01_JPRB, 0.41026E+01_JPRB ,&
&  0.41687E+01_JPRB, 0.42541E+01_JPRB, 0.42668E+01_JPRB, 0.42153E+01_JPRB, 0.42788E+01_JPRB, 0.43609E+01_JPRB, 0.44756E+01_JPRB ,&
&  0.47304E+01_JPRB, 0.51493E+01_JPRB, 0.55619E+01_JPRB, 0.57816E+01_JPRB, 0.57191E+01_JPRB, 0.60737E+01_JPRB, 0.60684E+01_JPRB ,&
&  0.62266E+01_JPRB, 0.58273E+01_JPRB, 0.59453E+01_JPRB, 0.52183E+01_JPRB, 0.48410E+01_JPRB, 0.45517E+01_JPRB, 0.44850E+01_JPRB ,&
&  0.43829E+01_JPRB, 0.43771E+01_JPRB, 0.42543E+01_JPRB, 0.45230E+01_JPRB, 0.53599E+01_JPRB, 0.58712E+01_JPRB, 0.60699E+01_JPRB ,&
&  0.51140E+01_JPRB, 0.39028E+01_JPRB, 0.28711E+01_JPRB, 0.29651E+01_JPRB, 0.26014E+01_JPRB, 0.29524E+01_JPRB, 0.37268E+01_JPRB ,&
&  0.44998E+01_JPRB, 0.37570E+01_JPRB, 0.37697E+01_JPRB, 0.32698E+01_JPRB, 0.19577E+01_JPRB, 0.13270E+01_JPRB, 0.10927E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10719E+01_JPRB, 0.10985E+01_JPRB, 0.10946E+01_JPRB, 0.11674E+01_JPRB, 0.12254E+01_JPRB, 0.13247E+01_JPRB ,&
&  0.13621E+01_JPRB, 0.14360E+01_JPRB, 0.15225E+01_JPRB, 0.16100E+01_JPRB, 0.17550E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.19246E+01_JPRB, 0.20698E+01_JPRB, 0.22253E+01_JPRB, 0.22610E+01_JPRB, 0.23016E+01_JPRB, 0.23548E+01_JPRB, 0.24392E+01_JPRB ,&
&  0.26080E+01_JPRB, 0.27654E+01_JPRB, 0.29297E+01_JPRB, 0.30697E+01_JPRB, 0.31892E+01_JPRB, 0.33847E+01_JPRB, 0.36250E+01_JPRB ,&
&  0.38265E+01_JPRB, 0.40135E+01_JPRB, 0.42058E+01_JPRB, 0.44686E+01_JPRB, 0.47437E+01_JPRB, 0.51289E+01_JPRB, 0.53106E+01_JPRB ,&
&  0.41504E+01_JPRB, 0.25345E+01_JPRB, 0.27295E+01_JPRB, 0.28029E+01_JPRB, 0.28995E+01_JPRB, 0.32052E+01_JPRB, 0.32881E+01_JPRB ,&
&  0.29638E+01_JPRB, 0.28526E+01_JPRB, 0.28600E+01_JPRB, 0.27497E+01_JPRB, 0.25450E+01_JPRB, 0.27253E+01_JPRB, 0.29046E+01_JPRB ,&
&  0.30719E+01_JPRB, 0.32140E+01_JPRB, 0.33355E+01_JPRB, 0.34651E+01_JPRB, 0.35752E+01_JPRB, 0.36753E+01_JPRB, 0.37720E+01_JPRB ,&
&  0.38518E+01_JPRB, 0.38375E+01_JPRB, 0.38540E+01_JPRB, 0.39301E+01_JPRB, 0.39221E+01_JPRB, 0.38647E+01_JPRB, 0.38061E+01_JPRB ,&
&  0.39265E+01_JPRB, 0.41589E+01_JPRB, 0.45288E+01_JPRB, 0.48272E+01_JPRB, 0.50285E+01_JPRB, 0.49317E+01_JPRB, 0.49015E+01_JPRB ,&
&  0.46379E+01_JPRB, 0.42685E+01_JPRB, 0.44924E+01_JPRB, 0.47255E+01_JPRB, 0.37495E+01_JPRB, 0.36346E+01_JPRB, 0.34094E+01_JPRB ,&
&  0.34359E+01_JPRB, 0.30992E+01_JPRB, 0.28712E+01_JPRB, 0.30649E+01_JPRB, 0.39893E+01_JPRB, 0.42585E+01_JPRB, 0.44447E+01_JPRB ,&
&  0.38085E+01_JPRB, 0.26101E+01_JPRB, 0.23970E+01_JPRB, 0.28051E+01_JPRB, 0.28776E+01_JPRB, 0.39516E+01_JPRB, 0.35206E+01_JPRB ,&
&  0.27728E+01_JPRB, 0.22540E+01_JPRB, 0.24709E+01_JPRB, 0.24415E+01_JPRB, 0.15904E+01_JPRB, 0.11420E+01_JPRB, 0.99250E+00_JPRB ,&
&  0.93074E+00_JPRB, 0.91312E+00_JPRB, 0.96101E+00_JPRB, 0.94491E+00_JPRB, 0.10108E+01_JPRB, 0.10665E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.13023E+01_JPRB, 0.14019E+01_JPRB, 0.14882E+01_JPRB, 0.16080E+01_JPRB, 0.17467E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.13519E+01_JPRB, 0.14398E+01_JPRB, 0.14710E+01_JPRB, 0.16446E+01_JPRB, 0.18475E+01_JPRB, 0.20088E+01_JPRB, 0.21562E+01_JPRB ,&
&  0.23476E+01_JPRB, 0.25834E+01_JPRB, 0.27666E+01_JPRB, 0.28956E+01_JPRB, 0.30277E+01_JPRB, 0.32042E+01_JPRB, 0.32541E+01_JPRB ,&
&  0.32430E+01_JPRB, 0.32304E+01_JPRB, 0.32369E+01_JPRB, 0.32298E+01_JPRB, 0.32471E+01_JPRB, 0.32598E+01_JPRB, 0.35123E+01_JPRB ,&
&  0.34640E+01_JPRB, 0.31777E+01_JPRB, 0.28747E+01_JPRB, 0.21719E+01_JPRB, 0.17683E+01_JPRB, 0.22144E+01_JPRB, 0.22106E+01_JPRB ,&
&  0.24184E+01_JPRB, 0.24119E+01_JPRB, 0.22912E+01_JPRB, 0.23456E+01_JPRB, 0.24013E+01_JPRB, 0.25204E+01_JPRB, 0.26859E+01_JPRB ,&
&  0.28076E+01_JPRB, 0.29386E+01_JPRB, 0.30619E+01_JPRB, 0.31638E+01_JPRB, 0.32460E+01_JPRB, 0.33252E+01_JPRB, 0.33379E+01_JPRB ,&
&  0.33418E+01_JPRB, 0.33193E+01_JPRB, 0.33282E+01_JPRB, 0.34690E+01_JPRB, 0.34748E+01_JPRB, 0.34628E+01_JPRB, 0.33313E+01_JPRB ,&
&  0.33232E+01_JPRB, 0.33991E+01_JPRB, 0.36234E+01_JPRB, 0.37809E+01_JPRB, 0.37672E+01_JPRB, 0.35677E+01_JPRB, 0.34445E+01_JPRB ,&
&  0.32884E+01_JPRB, 0.31084E+01_JPRB, 0.32341E+01_JPRB, 0.36043E+01_JPRB, 0.27387E+01_JPRB, 0.28933E+01_JPRB, 0.29631E+01_JPRB ,&
&  0.25906E+01_JPRB, 0.23078E+01_JPRB, 0.21344E+01_JPRB, 0.20367E+01_JPRB, 0.25626E+01_JPRB, 0.30628E+01_JPRB, 0.36111E+01_JPRB ,&
&  0.29612E+01_JPRB, 0.20685E+01_JPRB, 0.20268E+01_JPRB, 0.25612E+01_JPRB, 0.29752E+01_JPRB, 0.38771E+01_JPRB, 0.28906E+01_JPRB ,&
&  0.19328E+01_JPRB, 0.19394E+01_JPRB, 0.22043E+01_JPRB, 0.25396E+01_JPRB, 0.18061E+01_JPRB, 0.10723E+01_JPRB, 0.89819E+00_JPRB ,&
&  0.85796E+00_JPRB, 0.85486E+00_JPRB, 0.86095E+00_JPRB, 0.85395E+00_JPRB, 0.89635E+00_JPRB, 0.94014E+00_JPRB, 0.95227E+00_JPRB ,&
&  0.97160E+00_JPRB, 0.98578E+00_JPRB, 0.10344E+01_JPRB, 0.11193E+01_JPRB, 0.12493E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.87170E+00_JPRB, 0.88214E+00_JPRB, 0.87077E+00_JPRB, 0.91125E+00_JPRB, 0.99018E+00_JPRB, 0.11498E+01_JPRB, 0.13100E+01_JPRB ,&
&  0.14909E+01_JPRB, 0.17083E+01_JPRB, 0.20141E+01_JPRB, 0.23003E+01_JPRB, 0.24208E+01_JPRB, 0.23950E+01_JPRB, 0.21701E+01_JPRB ,&
&  0.18646E+01_JPRB, 0.15372E+01_JPRB, 0.13314E+01_JPRB, 0.12412E+01_JPRB, 0.12912E+01_JPRB, 0.13100E+01_JPRB, 0.13890E+01_JPRB ,&
&  0.15439E+01_JPRB, 0.18052E+01_JPRB, 0.20281E+01_JPRB, 0.25609E+01_JPRB, 0.27309E+01_JPRB, 0.21274E+01_JPRB, 0.21340E+01_JPRB ,&
&  0.27338E+01_JPRB, 0.27155E+01_JPRB, 0.18561E+01_JPRB, 0.17676E+01_JPRB, 0.19055E+01_JPRB, 0.20163E+01_JPRB, 0.21951E+01_JPRB ,&
&  0.22854E+01_JPRB, 0.23787E+01_JPRB, 0.24504E+01_JPRB, 0.24861E+01_JPRB, 0.25015E+01_JPRB, 0.24673E+01_JPRB, 0.24184E+01_JPRB ,&
&  0.23653E+01_JPRB, 0.23060E+01_JPRB, 0.23762E+01_JPRB, 0.25084E+01_JPRB, 0.26636E+01_JPRB, 0.27430E+01_JPRB, 0.27095E+01_JPRB ,&
&  0.26768E+01_JPRB, 0.26834E+01_JPRB, 0.28995E+01_JPRB, 0.29122E+01_JPRB, 0.27052E+01_JPRB, 0.25555E+01_JPRB, 0.23344E+01_JPRB ,&
&  0.21096E+01_JPRB, 0.18048E+01_JPRB, 0.15278E+01_JPRB, 0.23838E+01_JPRB, 0.27386E+01_JPRB, 0.24371E+01_JPRB, 0.21256E+01_JPRB ,&
&  0.19559E+01_JPRB, 0.18066E+01_JPRB, 0.16695E+01_JPRB, 0.16511E+01_JPRB, 0.19070E+01_JPRB, 0.28329E+01_JPRB, 0.29655E+01_JPRB ,&
&  0.24865E+01_JPRB, 0.16613E+01_JPRB, 0.14905E+01_JPRB, 0.17533E+01_JPRB, 0.20818E+01_JPRB, 0.25993E+01_JPRB, 0.19332E+01_JPRB ,&
&  0.15053E+01_JPRB, 0.17893E+01_JPRB, 0.20205E+01_JPRB, 0.24859E+01_JPRB, 0.25931E+01_JPRB, 0.87761E+00_JPRB, 0.81328E+00_JPRB ,&
&  0.78755E+00_JPRB, 0.77281E+00_JPRB, 0.78832E+00_JPRB, 0.77743E+00_JPRB, 0.78358E+00_JPRB, 0.76609E+00_JPRB, 0.74309E+00_JPRB ,&
&  0.74002E+00_JPRB, 0.75728E+00_JPRB, 0.78195E+00_JPRB, 0.80106E+00_JPRB, 0.84160E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.78518E+00_JPRB, 0.78564E+00_JPRB, 0.79029E+00_JPRB, 0.80000E+00_JPRB, 0.80860E+00_JPRB, 0.81761E+00_JPRB, 0.85280E+00_JPRB ,&
&  0.91175E+00_JPRB, 0.99935E+00_JPRB, 0.11222E+01_JPRB, 0.12113E+01_JPRB, 0.11923E+01_JPRB, 0.10930E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.99763E+00_JPRB, 0.97558E+00_JPRB, 0.10076E+01_JPRB, 0.10639E+01_JPRB, 0.11336E+01_JPRB, 0.11491E+01_JPRB, 0.12155E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.12600E+01_JPRB, 0.13767E+01_JPRB, 0.14779E+01_JPRB, 0.21975E+01_JPRB, 0.21295E+01_JPRB, 0.18520E+01_JPRB ,&
&  0.22248E+01_JPRB, 0.22397E+01_JPRB, 0.19070E+01_JPRB, 0.14996E+01_JPRB, 0.13801E+01_JPRB, 0.13412E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.13551E+01_JPRB, 0.14394E+01_JPRB, 0.15078E+01_JPRB, 0.15854E+01_JPRB, 0.16351E+01_JPRB, 0.16426E+01_JPRB, 0.16831E+01_JPRB ,&
&  0.17109E+01_JPRB, 0.17419E+01_JPRB, 0.17790E+01_JPRB, 0.16204E+01_JPRB, 0.17408E+01_JPRB, 0.18586E+01_JPRB, 0.19207E+01_JPRB ,&
&  0.19225E+01_JPRB, 0.19209E+01_JPRB, 0.20422E+01_JPRB, 0.19564E+01_JPRB, 0.18479E+01_JPRB, 0.16802E+01_JPRB, 0.15356E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.10991E+01_JPRB, 0.83688E+00_JPRB, 0.12411E+01_JPRB, 0.15567E+01_JPRB, 0.14147E+01_JPRB, 0.15227E+01_JPRB ,&
&  0.14712E+01_JPRB, 0.14065E+01_JPRB, 0.13410E+01_JPRB, 0.13531E+01_JPRB, 0.13883E+01_JPRB, 0.18521E+01_JPRB, 0.21978E+01_JPRB ,&
&  0.18315E+01_JPRB, 0.13171E+01_JPRB, 0.11424E+01_JPRB, 0.12000E+01_JPRB, 0.13786E+01_JPRB, 0.16376E+01_JPRB, 0.18217E+01_JPRB ,&
&  0.16904E+01_JPRB, 0.14081E+01_JPRB, 0.17120E+01_JPRB, 0.21289E+01_JPRB, 0.12179E+01_JPRB, 0.68876E+00_JPRB, 0.64397E+00_JPRB ,&
&  0.63746E+00_JPRB, 0.64539E+00_JPRB, 0.66729E+00_JPRB, 0.68168E+00_JPRB, 0.69087E+00_JPRB, 0.69551E+00_JPRB, 0.71172E+00_JPRB ,&
&  0.72531E+00_JPRB, 0.74175E+00_JPRB, 0.75087E+00_JPRB, 0.76344E+00_JPRB, 0.77740E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.99342E+00_JPRB, 0.10144E+01_JPRB, 0.10451E+01_JPRB, 0.10773E+01_JPRB, 0.11011E+01_JPRB, 0.11343E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.12855E+01_JPRB, 0.13308E+01_JPRB, 0.13485E+01_JPRB, 0.13617E+01_JPRB, 0.13771E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.14210E+01_JPRB, 0.14978E+01_JPRB, 0.15309E+01_JPRB, 0.15718E+01_JPRB, 0.16223E+01_JPRB, 0.16268E+01_JPRB ,&
&  0.16076E+01_JPRB, 0.15943E+01_JPRB, 0.16046E+01_JPRB, 0.15931E+01_JPRB, 0.16755E+01_JPRB, 0.16965E+01_JPRB, 0.16410E+01_JPRB ,&
&  0.17159E+01_JPRB, 0.16811E+01_JPRB, 0.17724E+01_JPRB, 0.13300E+01_JPRB, 0.12629E+01_JPRB, 0.12580E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11714E+01_JPRB, 0.12297E+01_JPRB, 0.12870E+01_JPRB, 0.13674E+01_JPRB, 0.13967E+01_JPRB, 0.14083E+01_JPRB ,&
&  0.14280E+01_JPRB, 0.14640E+01_JPRB, 0.14887E+01_JPRB, 0.14408E+01_JPRB, 0.14122E+01_JPRB, 0.15803E+01_JPRB, 0.16341E+01_JPRB ,&
&  0.16313E+01_JPRB, 0.16276E+01_JPRB, 0.13691E+01_JPRB, 0.12754E+01_JPRB, 0.13101E+01_JPRB, 0.11787E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.10739E+01_JPRB, 0.71094E+00_JPRB, 0.93705E+00_JPRB, 0.11290E+01_JPRB, 0.12123E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.10792E+01_JPRB, 0.10442E+01_JPRB, 0.11145E+01_JPRB, 0.11842E+01_JPRB, 0.11938E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.10774E+01_JPRB, 0.10063E+01_JPRB, 0.11680E+01_JPRB, 0.16708E+01_JPRB, 0.26168E+01_JPRB, 0.23282E+01_JPRB ,&
&  0.16597E+01_JPRB, 0.14815E+01_JPRB, 0.23500E+01_JPRB, 0.31830E+01_JPRB, 0.12338E+01_JPRB, 0.79072E+00_JPRB, 0.61174E+00_JPRB ,&
&  0.64299E+00_JPRB, 0.65840E+00_JPRB, 0.66903E+00_JPRB, 0.69599E+00_JPRB, 0.73812E+00_JPRB, 0.76079E+00_JPRB, 0.77099E+00_JPRB ,&
&  0.80053E+00_JPRB, 0.84360E+00_JPRB, 0.87672E+00_JPRB, 0.91792E+00_JPRB, 0.95540E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.11893E+01_JPRB, 0.12228E+01_JPRB, 0.12495E+01_JPRB, 0.12790E+01_JPRB, 0.13151E+01_JPRB, 0.13509E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.14351E+01_JPRB, 0.14470E+01_JPRB, 0.14694E+01_JPRB, 0.14764E+01_JPRB, 0.15442E+01_JPRB, 0.15709E+01_JPRB ,&
&  0.16371E+01_JPRB, 0.16772E+01_JPRB, 0.16936E+01_JPRB, 0.16994E+01_JPRB, 0.17178E+01_JPRB, 0.17055E+01_JPRB, 0.16916E+01_JPRB ,&
&  0.17202E+01_JPRB, 0.17796E+01_JPRB, 0.18359E+01_JPRB, 0.18873E+01_JPRB, 0.19031E+01_JPRB, 0.19847E+01_JPRB, 0.20997E+01_JPRB ,&
&  0.17088E+01_JPRB, 0.18598E+01_JPRB, 0.17192E+01_JPRB, 0.14369E+01_JPRB, 0.14173E+01_JPRB, 0.14201E+01_JPRB, 0.13538E+01_JPRB ,&
&  0.13030E+01_JPRB, 0.13200E+01_JPRB, 0.13362E+01_JPRB, 0.13467E+01_JPRB, 0.13739E+01_JPRB, 0.14077E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.14548E+01_JPRB, 0.14835E+01_JPRB, 0.15172E+01_JPRB, 0.15727E+01_JPRB, 0.15161E+01_JPRB, 0.16987E+01_JPRB, 0.17323E+01_JPRB ,&
&  0.17192E+01_JPRB, 0.16102E+01_JPRB, 0.12477E+01_JPRB, 0.11829E+01_JPRB, 0.12104E+01_JPRB, 0.11565E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.11666E+01_JPRB, 0.93386E+00_JPRB, 0.10519E+01_JPRB, 0.11149E+01_JPRB, 0.11429E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10039E+01_JPRB, 0.89238E+00_JPRB, 0.86082E+00_JPRB, 0.89019E+00_JPRB, 0.91145E+00_JPRB, 0.91259E+00_JPRB, 0.96057E+00_JPRB ,&
&  0.98254E+00_JPRB, 0.97609E+00_JPRB, 0.10111E+01_JPRB, 0.10908E+01_JPRB, 0.17588E+01_JPRB, 0.22265E+01_JPRB, 0.30680E+01_JPRB ,&
&  0.13890E+01_JPRB, 0.13804E+01_JPRB, 0.12319E+01_JPRB, 0.16894E+01_JPRB, 0.10279E+01_JPRB, 0.66461E+00_JPRB, 0.64450E+00_JPRB ,&
&  0.71441E+00_JPRB, 0.74839E+00_JPRB, 0.75909E+00_JPRB, 0.78314E+00_JPRB, 0.83771E+00_JPRB, 0.87353E+00_JPRB, 0.90859E+00_JPRB ,&
&  0.92893E+00_JPRB, 0.98447E+00_JPRB, 0.10567E+01_JPRB, 0.11125E+01_JPRB, 0.11435E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.12802E+01_JPRB, 0.13235E+01_JPRB, 0.13627E+01_JPRB, 0.14019E+01_JPRB, 0.14400E+01_JPRB, 0.14786E+01_JPRB, 0.15116E+01_JPRB ,&
&  0.15388E+01_JPRB, 0.15576E+01_JPRB, 0.15680E+01_JPRB, 0.15842E+01_JPRB, 0.15948E+01_JPRB, 0.16512E+01_JPRB, 0.16709E+01_JPRB ,&
&  0.17334E+01_JPRB, 0.17612E+01_JPRB, 0.17574E+01_JPRB, 0.17960E+01_JPRB, 0.17769E+01_JPRB, 0.17439E+01_JPRB, 0.17380E+01_JPRB ,&
&  0.17528E+01_JPRB, 0.17557E+01_JPRB, 0.17592E+01_JPRB, 0.17730E+01_JPRB, 0.17828E+01_JPRB, 0.17522E+01_JPRB, 0.19037E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.20525E+01_JPRB, 0.19983E+01_JPRB, 0.17380E+01_JPRB, 0.16239E+01_JPRB, 0.14860E+01_JPRB, 0.14584E+01_JPRB ,&
&  0.14546E+01_JPRB, 0.14139E+01_JPRB, 0.13991E+01_JPRB, 0.14078E+01_JPRB, 0.14326E+01_JPRB, 0.14715E+01_JPRB, 0.14965E+01_JPRB ,&
&  0.15362E+01_JPRB, 0.15644E+01_JPRB, 0.15898E+01_JPRB, 0.16170E+01_JPRB, 0.16803E+01_JPRB, 0.17031E+01_JPRB, 0.16951E+01_JPRB ,&
&  0.16784E+01_JPRB, 0.16679E+01_JPRB, 0.13305E+01_JPRB, 0.12881E+01_JPRB, 0.13639E+01_JPRB, 0.13105E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.11687E+01_JPRB, 0.11564E+01_JPRB, 0.11161E+01_JPRB, 0.10906E+01_JPRB, 0.10473E+01_JPRB, 0.97581E+00_JPRB ,&
&  0.83629E+00_JPRB, 0.72880E+00_JPRB, 0.70445E+00_JPRB, 0.74700E+00_JPRB, 0.77021E+00_JPRB, 0.78297E+00_JPRB, 0.77431E+00_JPRB ,&
&  0.79677E+00_JPRB, 0.84380E+00_JPRB, 0.93799E+00_JPRB, 0.11468E+01_JPRB, 0.14663E+01_JPRB, 0.28020E+01_JPRB, 0.29481E+01_JPRB ,&
&  0.13416E+01_JPRB, 0.12311E+01_JPRB, 0.77759E+00_JPRB, 0.89710E+00_JPRB, 0.90165E+00_JPRB, 0.75797E+00_JPRB, 0.77186E+00_JPRB ,&
&  0.88585E+00_JPRB, 0.95226E+00_JPRB, 0.98052E+00_JPRB, 0.92890E+00_JPRB, 0.95747E+00_JPRB, 0.95487E+00_JPRB, 0.97155E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.10601E+01_JPRB, 0.11223E+01_JPRB, 0.11810E+01_JPRB, 0.12321E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.13143E+01_JPRB, 0.13599E+01_JPRB, 0.14076E+01_JPRB, 0.14605E+01_JPRB, 0.15149E+01_JPRB, 0.15579E+01_JPRB, 0.15996E+01_JPRB ,&
&  0.16395E+01_JPRB, 0.16689E+01_JPRB, 0.16797E+01_JPRB, 0.16883E+01_JPRB, 0.16980E+01_JPRB, 0.17512E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.17955E+01_JPRB, 0.18249E+01_JPRB, 0.18437E+01_JPRB, 0.18689E+01_JPRB, 0.18602E+01_JPRB, 0.18187E+01_JPRB, 0.18364E+01_JPRB ,&
&  0.18344E+01_JPRB, 0.18442E+01_JPRB, 0.18542E+01_JPRB, 0.19004E+01_JPRB, 0.19640E+01_JPRB, 0.21400E+01_JPRB, 0.18922E+01_JPRB ,&
&  0.20793E+01_JPRB, 0.21972E+01_JPRB, 0.21038E+01_JPRB, 0.18990E+01_JPRB, 0.16961E+01_JPRB, 0.16427E+01_JPRB, 0.16143E+01_JPRB ,&
&  0.16087E+01_JPRB, 0.15408E+01_JPRB, 0.15032E+01_JPRB, 0.14740E+01_JPRB, 0.14664E+01_JPRB, 0.14961E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.15461E+01_JPRB, 0.16018E+01_JPRB, 0.16596E+01_JPRB, 0.17202E+01_JPRB, 0.17988E+01_JPRB, 0.18711E+01_JPRB, 0.19104E+01_JPRB ,&
&  0.19304E+01_JPRB, 0.19430E+01_JPRB, 0.17655E+01_JPRB, 0.17542E+01_JPRB, 0.18187E+01_JPRB, 0.16617E+01_JPRB, 0.14204E+01_JPRB ,&
&  0.13194E+01_JPRB, 0.11503E+01_JPRB, 0.10494E+01_JPRB, 0.11759E+01_JPRB, 0.10931E+01_JPRB, 0.99213E+00_JPRB, 0.86910E+00_JPRB ,&
&  0.72815E+00_JPRB, 0.65599E+00_JPRB, 0.65884E+00_JPRB, 0.70543E+00_JPRB, 0.70582E+00_JPRB, 0.72380E+00_JPRB, 0.76686E+00_JPRB ,&
&  0.83331E+00_JPRB, 0.90976E+00_JPRB, 0.10227E+01_JPRB, 0.12215E+01_JPRB, 0.14858E+01_JPRB, 0.15164E+01_JPRB, 0.16677E+01_JPRB ,&
&  0.12035E+01_JPRB, 0.99441E+00_JPRB, 0.10081E+01_JPRB, 0.97471E+00_JPRB, 0.99578E+00_JPRB, 0.95505E+00_JPRB, 0.10544E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.13498E+01_JPRB, 0.13652E+01_JPRB, 0.11606E+01_JPRB, 0.10652E+01_JPRB, 0.10292E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.10992E+01_JPRB, 0.11538E+01_JPRB, 0.12152E+01_JPRB, 0.12685E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.13057E+01_JPRB, 0.13534E+01_JPRB, 0.14036E+01_JPRB, 0.14536E+01_JPRB, 0.14964E+01_JPRB, 0.15327E+01_JPRB, 0.15625E+01_JPRB ,&
&  0.16015E+01_JPRB, 0.16403E+01_JPRB, 0.16706E+01_JPRB, 0.16897E+01_JPRB, 0.17100E+01_JPRB, 0.17496E+01_JPRB, 0.17968E+01_JPRB ,&
&  0.18528E+01_JPRB, 0.18969E+01_JPRB, 0.19672E+01_JPRB, 0.19796E+01_JPRB, 0.19920E+01_JPRB, 0.20213E+01_JPRB, 0.20893E+01_JPRB ,&
&  0.21255E+01_JPRB, 0.21294E+01_JPRB, 0.21429E+01_JPRB, 0.22103E+01_JPRB, 0.22775E+01_JPRB, 0.24107E+01_JPRB, 0.21802E+01_JPRB ,&
&  0.23021E+01_JPRB, 0.23846E+01_JPRB, 0.22616E+01_JPRB, 0.21192E+01_JPRB, 0.19078E+01_JPRB, 0.18304E+01_JPRB, 0.17889E+01_JPRB ,&
&  0.17443E+01_JPRB, 0.16935E+01_JPRB, 0.16200E+01_JPRB, 0.15669E+01_JPRB, 0.15297E+01_JPRB, 0.15085E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.15592E+01_JPRB, 0.16003E+01_JPRB, 0.16765E+01_JPRB, 0.17615E+01_JPRB, 0.18832E+01_JPRB, 0.20007E+01_JPRB, 0.20773E+01_JPRB ,&
&  0.21657E+01_JPRB, 0.22495E+01_JPRB, 0.21120E+01_JPRB, 0.21294E+01_JPRB, 0.22349E+01_JPRB, 0.20133E+01_JPRB, 0.16143E+01_JPRB ,&
&  0.12236E+01_JPRB, 0.11233E+01_JPRB, 0.11415E+01_JPRB, 0.12133E+01_JPRB, 0.10800E+01_JPRB, 0.95365E+00_JPRB, 0.87002E+00_JPRB ,&
&  0.75998E+00_JPRB, 0.67570E+00_JPRB, 0.69181E+00_JPRB, 0.69053E+00_JPRB, 0.68720E+00_JPRB, 0.72983E+00_JPRB, 0.78998E+00_JPRB ,&
&  0.86837E+00_JPRB, 0.99344E+00_JPRB, 0.11314E+01_JPRB, 0.12876E+01_JPRB, 0.13985E+01_JPRB, 0.13570E+01_JPRB, 0.14062E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.12103E+01_JPRB, 0.11903E+01_JPRB, 0.91819E+00_JPRB, 0.11707E+01_JPRB, 0.12427E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.13292E+01_JPRB, 0.17520E+01_JPRB, 0.21131E+01_JPRB, 0.16709E+01_JPRB, 0.13665E+01_JPRB, 0.10731E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.10909E+01_JPRB, 0.11395E+01_JPRB, 0.11983E+01_JPRB, 0.12536E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.12570E+01_JPRB, 0.13095E+01_JPRB, 0.13648E+01_JPRB, 0.14103E+01_JPRB, 0.14443E+01_JPRB, 0.14474E+01_JPRB, 0.14496E+01_JPRB ,&
&  0.14655E+01_JPRB, 0.14926E+01_JPRB, 0.15288E+01_JPRB, 0.15683E+01_JPRB, 0.16011E+01_JPRB, 0.16357E+01_JPRB, 0.16829E+01_JPRB ,&
&  0.17396E+01_JPRB, 0.18074E+01_JPRB, 0.18962E+01_JPRB, 0.19801E+01_JPRB, 0.20801E+01_JPRB, 0.21758E+01_JPRB, 0.22829E+01_JPRB ,&
&  0.23794E+01_JPRB, 0.24829E+01_JPRB, 0.25271E+01_JPRB, 0.25970E+01_JPRB, 0.26373E+01_JPRB, 0.27427E+01_JPRB, 0.22010E+01_JPRB ,&
&  0.24083E+01_JPRB, 0.24643E+01_JPRB, 0.24208E+01_JPRB, 0.22597E+01_JPRB, 0.20425E+01_JPRB, 0.19310E+01_JPRB, 0.18475E+01_JPRB ,&
&  0.17972E+01_JPRB, 0.17151E+01_JPRB, 0.15776E+01_JPRB, 0.15842E+01_JPRB, 0.15860E+01_JPRB, 0.14965E+01_JPRB, 0.15317E+01_JPRB ,&
&  0.15657E+01_JPRB, 0.16139E+01_JPRB, 0.16767E+01_JPRB, 0.17830E+01_JPRB, 0.19260E+01_JPRB, 0.20548E+01_JPRB, 0.21827E+01_JPRB ,&
&  0.23169E+01_JPRB, 0.24580E+01_JPRB, 0.25659E+01_JPRB, 0.23286E+01_JPRB, 0.23176E+01_JPRB, 0.21003E+01_JPRB, 0.16550E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.11121E+01_JPRB, 0.11554E+01_JPRB, 0.12275E+01_JPRB, 0.10902E+01_JPRB, 0.93136E+00_JPRB, 0.85078E+00_JPRB ,&
&  0.76469E+00_JPRB, 0.71958E+00_JPRB, 0.71510E+00_JPRB, 0.69390E+00_JPRB, 0.69251E+00_JPRB, 0.73788E+00_JPRB, 0.80002E+00_JPRB ,&
&  0.87843E+00_JPRB, 0.98917E+00_JPRB, 0.11276E+01_JPRB, 0.12799E+01_JPRB, 0.14320E+01_JPRB, 0.14844E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.11915E+01_JPRB, 0.12230E+01_JPRB, 0.13172E+01_JPRB, 0.13357E+01_JPRB, 0.15658E+01_JPRB, 0.18041E+01_JPRB ,&
&  0.20283E+01_JPRB, 0.18729E+01_JPRB, 0.21792E+01_JPRB, 0.29945E+01_JPRB, 0.27555E+01_JPRB, 0.22832E+01_JPRB, 0.97086E+00_JPRB ,&
&  0.98506E+00_JPRB, 0.99744E+00_JPRB, 0.10513E+01_JPRB, 0.11211E+01_JPRB, 0.11999E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.10085E+01_JPRB, 0.10791E+01_JPRB, 0.11517E+01_JPRB, 0.12478E+01_JPRB, 0.13341E+01_JPRB, 0.13548E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.13489E+01_JPRB, 0.13759E+01_JPRB, 0.14148E+01_JPRB, 0.14513E+01_JPRB, 0.14835E+01_JPRB, 0.15226E+01_JPRB ,&
&  0.15748E+01_JPRB, 0.16360E+01_JPRB, 0.17088E+01_JPRB, 0.17888E+01_JPRB, 0.18719E+01_JPRB, 0.19889E+01_JPRB, 0.21297E+01_JPRB ,&
&  0.22828E+01_JPRB, 0.24356E+01_JPRB, 0.25539E+01_JPRB, 0.26702E+01_JPRB, 0.27697E+01_JPRB, 0.27576E+01_JPRB, 0.19729E+01_JPRB ,&
&  0.24193E+01_JPRB, 0.24658E+01_JPRB, 0.25107E+01_JPRB, 0.23621E+01_JPRB, 0.21119E+01_JPRB, 0.19654E+01_JPRB, 0.18940E+01_JPRB ,&
&  0.18185E+01_JPRB, 0.17117E+01_JPRB, 0.15880E+01_JPRB, 0.14935E+01_JPRB, 0.15690E+01_JPRB, 0.14696E+01_JPRB, 0.15140E+01_JPRB ,&
&  0.15593E+01_JPRB, 0.16116E+01_JPRB, 0.16659E+01_JPRB, 0.17588E+01_JPRB, 0.19045E+01_JPRB, 0.20500E+01_JPRB, 0.22137E+01_JPRB ,&
&  0.23704E+01_JPRB, 0.25035E+01_JPRB, 0.25766E+01_JPRB, 0.22612E+01_JPRB, 0.21546E+01_JPRB, 0.20204E+01_JPRB, 0.14951E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.11984E+01_JPRB, 0.12741E+01_JPRB, 0.12634E+01_JPRB, 0.11286E+01_JPRB, 0.98746E+00_JPRB, 0.88411E+00_JPRB ,&
&  0.77119E+00_JPRB, 0.72135E+00_JPRB, 0.71683E+00_JPRB, 0.70719E+00_JPRB, 0.71332E+00_JPRB, 0.76037E+00_JPRB, 0.82053E+00_JPRB ,&
&  0.88512E+00_JPRB, 0.96807E+00_JPRB, 0.10433E+01_JPRB, 0.11156E+01_JPRB, 0.12000E+01_JPRB, 0.13400E+01_JPRB, 0.14270E+01_JPRB ,&
&  0.12582E+01_JPRB, 0.11395E+01_JPRB, 0.11333E+01_JPRB, 0.11498E+01_JPRB, 0.11789E+01_JPRB, 0.12488E+01_JPRB, 0.13234E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.15268E+01_JPRB, 0.12914E+01_JPRB, 0.10800E+01_JPRB, 0.13761E+01_JPRB, 0.14800E+01_JPRB, 0.82785E+00_JPRB ,&
&  0.80522E+00_JPRB, 0.83054E+00_JPRB, 0.85676E+00_JPRB, 0.90609E+00_JPRB, 0.95577E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.79671E+00_JPRB, 0.82278E+00_JPRB, 0.91650E+00_JPRB, 0.95282E+00_JPRB, 0.10511E+01_JPRB, 0.11024E+01_JPRB, 0.11594E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11997E+01_JPRB, 0.12143E+01_JPRB, 0.12349E+01_JPRB, 0.12610E+01_JPRB, 0.12958E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.13701E+01_JPRB, 0.14198E+01_JPRB, 0.14786E+01_JPRB, 0.15418E+01_JPRB, 0.16240E+01_JPRB, 0.17211E+01_JPRB, 0.18548E+01_JPRB ,&
&  0.20246E+01_JPRB, 0.22246E+01_JPRB, 0.24319E+01_JPRB, 0.26410E+01_JPRB, 0.28583E+01_JPRB, 0.30232E+01_JPRB, 0.31043E+01_JPRB ,&
&  0.20559E+01_JPRB, 0.25422E+01_JPRB, 0.25110E+01_JPRB, 0.23651E+01_JPRB, 0.21741E+01_JPRB, 0.20479E+01_JPRB, 0.19790E+01_JPRB ,&
&  0.19099E+01_JPRB, 0.17458E+01_JPRB, 0.16665E+01_JPRB, 0.15434E+01_JPRB, 0.14878E+01_JPRB, 0.14352E+01_JPRB, 0.14671E+01_JPRB ,&
&  0.15004E+01_JPRB, 0.15602E+01_JPRB, 0.16367E+01_JPRB, 0.17092E+01_JPRB, 0.18383E+01_JPRB, 0.20120E+01_JPRB, 0.21982E+01_JPRB ,&
&  0.23941E+01_JPRB, 0.25835E+01_JPRB, 0.27023E+01_JPRB, 0.23942E+01_JPRB, 0.21212E+01_JPRB, 0.20240E+01_JPRB, 0.15849E+01_JPRB ,&
&  0.13571E+01_JPRB, 0.12358E+01_JPRB, 0.13297E+01_JPRB, 0.13507E+01_JPRB, 0.12036E+01_JPRB, 0.10817E+01_JPRB, 0.97446E+00_JPRB ,&
&  0.86085E+00_JPRB, 0.77205E+00_JPRB, 0.74732E+00_JPRB, 0.74646E+00_JPRB, 0.76011E+00_JPRB, 0.79637E+00_JPRB, 0.82785E+00_JPRB ,&
&  0.86501E+00_JPRB, 0.90870E+00_JPRB, 0.94779E+00_JPRB, 0.99195E+00_JPRB, 0.10147E+01_JPRB, 0.10364E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10792E+01_JPRB, 0.10727E+01_JPRB, 0.10827E+01_JPRB, 0.10658E+01_JPRB, 0.10728E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.11307E+01_JPRB, 0.11173E+01_JPRB, 0.10771E+01_JPRB, 0.10001E+01_JPRB, 0.91983E+00_JPRB, 0.76373E+00_JPRB ,&
&  0.74400E+00_JPRB, 0.72804E+00_JPRB, 0.74519E+00_JPRB, 0.75930E+00_JPRB, 0.78256E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.68292E+00_JPRB, 0.70708E+00_JPRB, 0.73224E+00_JPRB, 0.76160E+00_JPRB, 0.80739E+00_JPRB, 0.86237E+00_JPRB, 0.93395E+00_JPRB ,&
&  0.10076E+01_JPRB, 0.10363E+01_JPRB, 0.10457E+01_JPRB, 0.10525E+01_JPRB, 0.10670E+01_JPRB, 0.10976E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.12094E+01_JPRB, 0.12441E+01_JPRB, 0.12856E+01_JPRB, 0.13371E+01_JPRB, 0.14133E+01_JPRB, 0.15084E+01_JPRB ,&
&  0.16298E+01_JPRB, 0.18238E+01_JPRB, 0.20789E+01_JPRB, 0.23739E+01_JPRB, 0.26943E+01_JPRB, 0.29551E+01_JPRB, 0.30919E+01_JPRB ,&
&  0.16918E+01_JPRB, 0.21933E+01_JPRB, 0.26488E+01_JPRB, 0.24743E+01_JPRB, 0.22807E+01_JPRB, 0.21692E+01_JPRB, 0.21021E+01_JPRB ,&
&  0.19582E+01_JPRB, 0.17713E+01_JPRB, 0.16784E+01_JPRB, 0.16713E+01_JPRB, 0.14739E+01_JPRB, 0.14256E+01_JPRB, 0.14286E+01_JPRB ,&
&  0.14450E+01_JPRB, 0.14654E+01_JPRB, 0.15167E+01_JPRB, 0.15893E+01_JPRB, 0.17029E+01_JPRB, 0.18840E+01_JPRB, 0.20938E+01_JPRB ,&
&  0.23294E+01_JPRB, 0.25884E+01_JPRB, 0.28320E+01_JPRB, 0.25317E+01_JPRB, 0.24887E+01_JPRB, 0.25457E+01_JPRB, 0.23382E+01_JPRB ,&
&  0.17015E+01_JPRB, 0.14138E+01_JPRB, 0.14628E+01_JPRB, 0.14752E+01_JPRB, 0.13022E+01_JPRB, 0.11965E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.10024E+01_JPRB, 0.91176E+00_JPRB, 0.81920E+00_JPRB, 0.79980E+00_JPRB, 0.81097E+00_JPRB, 0.83850E+00_JPRB, 0.86653E+00_JPRB ,&
&  0.88280E+00_JPRB, 0.89010E+00_JPRB, 0.90613E+00_JPRB, 0.92808E+00_JPRB, 0.96718E+00_JPRB, 0.98632E+00_JPRB, 0.99002E+00_JPRB ,&
&  0.10152E+01_JPRB, 0.10264E+01_JPRB, 0.10434E+01_JPRB, 0.10655E+01_JPRB, 0.10872E+01_JPRB, 0.10979E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.10743E+01_JPRB, 0.10867E+01_JPRB, 0.11068E+01_JPRB, 0.10979E+01_JPRB, 0.10523E+01_JPRB, 0.95155E+00_JPRB, 0.86765E+00_JPRB ,&
&  0.78099E+00_JPRB, 0.76634E+00_JPRB, 0.73690E+00_JPRB, 0.71131E+00_JPRB, 0.68910E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.66526E+00_JPRB, 0.66232E+00_JPRB, 0.67226E+00_JPRB, 0.68082E+00_JPRB, 0.69803E+00_JPRB, 0.73117E+00_JPRB, 0.77195E+00_JPRB ,&
&  0.80722E+00_JPRB, 0.86557E+00_JPRB, 0.90737E+00_JPRB, 0.90924E+00_JPRB, 0.90343E+00_JPRB, 0.91739E+00_JPRB, 0.94816E+00_JPRB ,&
&  0.98906E+00_JPRB, 0.10255E+01_JPRB, 0.10520E+01_JPRB, 0.10763E+01_JPRB, 0.11079E+01_JPRB, 0.11424E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.13722E+01_JPRB, 0.15561E+01_JPRB, 0.18448E+01_JPRB, 0.22420E+01_JPRB, 0.27055E+01_JPRB, 0.30238E+01_JPRB ,&
&  0.21525E+01_JPRB, 0.10390E+01_JPRB, 0.23186E+01_JPRB, 0.27534E+01_JPRB, 0.24788E+01_JPRB, 0.23984E+01_JPRB, 0.22433E+01_JPRB ,&
&  0.20763E+01_JPRB, 0.19917E+01_JPRB, 0.18422E+01_JPRB, 0.17310E+01_JPRB, 0.15051E+01_JPRB, 0.14238E+01_JPRB, 0.13820E+01_JPRB ,&
&  0.13760E+01_JPRB, 0.13779E+01_JPRB, 0.14030E+01_JPRB, 0.14486E+01_JPRB, 0.15294E+01_JPRB, 0.16694E+01_JPRB, 0.18982E+01_JPRB ,&
&  0.21542E+01_JPRB, 0.24557E+01_JPRB, 0.28691E+01_JPRB, 0.27227E+01_JPRB, 0.28190E+01_JPRB, 0.26129E+01_JPRB, 0.22574E+01_JPRB ,&
&  0.18799E+01_JPRB, 0.16132E+01_JPRB, 0.16228E+01_JPRB, 0.16457E+01_JPRB, 0.14384E+01_JPRB, 0.12462E+01_JPRB, 0.12438E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.10519E+01_JPRB, 0.95108E+00_JPRB, 0.88348E+00_JPRB, 0.86870E+00_JPRB, 0.89072E+00_JPRB, 0.91344E+00_JPRB ,&
&  0.91834E+00_JPRB, 0.91660E+00_JPRB, 0.92152E+00_JPRB, 0.93566E+00_JPRB, 0.96098E+00_JPRB, 0.97953E+00_JPRB, 0.98719E+00_JPRB ,&
&  0.10128E+01_JPRB, 0.10403E+01_JPRB, 0.10668E+01_JPRB, 0.10914E+01_JPRB, 0.11080E+01_JPRB, 0.10888E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.11004E+01_JPRB, 0.10962E+01_JPRB, 0.11050E+01_JPRB, 0.10842E+01_JPRB, 0.10254E+01_JPRB, 0.10003E+01_JPRB ,&
&  0.92892E+00_JPRB, 0.85648E+00_JPRB, 0.76853E+00_JPRB, 0.71361E+00_JPRB, 0.69298E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.76145E+00_JPRB, 0.72633E+00_JPRB, 0.68714E+00_JPRB, 0.67819E+00_JPRB, 0.67878E+00_JPRB, 0.67737E+00_JPRB, 0.69755E+00_JPRB ,&
&  0.71675E+00_JPRB, 0.74529E+00_JPRB, 0.75584E+00_JPRB, 0.78170E+00_JPRB, 0.79226E+00_JPRB, 0.79578E+00_JPRB, 0.81299E+00_JPRB ,&
&  0.82921E+00_JPRB, 0.84913E+00_JPRB, 0.87099E+00_JPRB, 0.89675E+00_JPRB, 0.92370E+00_JPRB, 0.93585E+00_JPRB, 0.95246E+00_JPRB ,&
&  0.98994E+00_JPRB, 0.10401E+01_JPRB, 0.11189E+01_JPRB, 0.12838E+01_JPRB, 0.15771E+01_JPRB, 0.19969E+01_JPRB, 0.24963E+01_JPRB ,&
&  0.29875E+01_JPRB, 0.21137E+01_JPRB, 0.14469E+01_JPRB, 0.25715E+01_JPRB, 0.28296E+01_JPRB, 0.27847E+01_JPRB, 0.23681E+01_JPRB ,&
&  0.23850E+01_JPRB, 0.22626E+01_JPRB, 0.21075E+01_JPRB, 0.19107E+01_JPRB, 0.16232E+01_JPRB, 0.14744E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.13004E+01_JPRB, 0.12683E+01_JPRB, 0.12766E+01_JPRB, 0.13306E+01_JPRB, 0.13922E+01_JPRB, 0.14740E+01_JPRB, 0.16557E+01_JPRB ,&
&  0.19010E+01_JPRB, 0.22149E+01_JPRB, 0.27064E+01_JPRB, 0.28807E+01_JPRB, 0.32389E+01_JPRB, 0.28340E+01_JPRB, 0.23976E+01_JPRB ,&
&  0.19749E+01_JPRB, 0.19354E+01_JPRB, 0.20021E+01_JPRB, 0.18364E+01_JPRB, 0.15445E+01_JPRB, 0.14013E+01_JPRB, 0.14017E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.11716E+01_JPRB, 0.11039E+01_JPRB, 0.10164E+01_JPRB, 0.96645E+00_JPRB, 0.95836E+00_JPRB, 0.96373E+00_JPRB ,&
&  0.96331E+00_JPRB, 0.95068E+00_JPRB, 0.94243E+00_JPRB, 0.94841E+00_JPRB, 0.96292E+00_JPRB, 0.97397E+00_JPRB, 0.99055E+00_JPRB ,&
&  0.10102E+01_JPRB, 0.10363E+01_JPRB, 0.10685E+01_JPRB, 0.10910E+01_JPRB, 0.10857E+01_JPRB, 0.10653E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11146E+01_JPRB, 0.11003E+01_JPRB, 0.11070E+01_JPRB, 0.10938E+01_JPRB, 0.10871E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.10320E+01_JPRB, 0.94113E+00_JPRB, 0.82826E+00_JPRB, 0.79769E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.84138E+00_JPRB, 0.81315E+00_JPRB, 0.77855E+00_JPRB, 0.74416E+00_JPRB, 0.72632E+00_JPRB, 0.70837E+00_JPRB, 0.68566E+00_JPRB ,&
&  0.67388E+00_JPRB, 0.66928E+00_JPRB, 0.65939E+00_JPRB, 0.67155E+00_JPRB, 0.69884E+00_JPRB, 0.71244E+00_JPRB, 0.72359E+00_JPRB ,&
&  0.73697E+00_JPRB, 0.74958E+00_JPRB, 0.74546E+00_JPRB, 0.74826E+00_JPRB, 0.76497E+00_JPRB, 0.77817E+00_JPRB, 0.78187E+00_JPRB ,&
&  0.79338E+00_JPRB, 0.81927E+00_JPRB, 0.85586E+00_JPRB, 0.92575E+00_JPRB, 0.10703E+01_JPRB, 0.13060E+01_JPRB, 0.16447E+01_JPRB ,&
&  0.21766E+01_JPRB, 0.22271E+01_JPRB, 0.11824E+01_JPRB, 0.21764E+01_JPRB, 0.30176E+01_JPRB, 0.30584E+01_JPRB, 0.28106E+01_JPRB ,&
&  0.27877E+01_JPRB, 0.26928E+01_JPRB, 0.25369E+01_JPRB, 0.21370E+01_JPRB, 0.18538E+01_JPRB, 0.16362E+01_JPRB, 0.14558E+01_JPRB ,&
&  0.13104E+01_JPRB, 0.12091E+01_JPRB, 0.11623E+01_JPRB, 0.11860E+01_JPRB, 0.12436E+01_JPRB, 0.12891E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.15782E+01_JPRB, 0.18365E+01_JPRB, 0.22738E+01_JPRB, 0.26593E+01_JPRB, 0.33408E+01_JPRB, 0.33438E+01_JPRB, 0.27948E+01_JPRB ,&
&  0.22983E+01_JPRB, 0.23140E+01_JPRB, 0.23160E+01_JPRB, 0.20753E+01_JPRB, 0.16802E+01_JPRB, 0.15564E+01_JPRB, 0.15366E+01_JPRB ,&
&  0.13896E+01_JPRB, 0.12733E+01_JPRB, 0.12177E+01_JPRB, 0.11618E+01_JPRB, 0.10866E+01_JPRB, 0.10232E+01_JPRB, 0.99480E+00_JPRB ,&
&  0.98976E+00_JPRB, 0.98303E+00_JPRB, 0.97699E+00_JPRB, 0.97048E+00_JPRB, 0.97084E+00_JPRB, 0.97369E+00_JPRB, 0.98254E+00_JPRB ,&
&  0.10005E+01_JPRB, 0.10223E+01_JPRB, 0.10550E+01_JPRB, 0.10716E+01_JPRB, 0.10823E+01_JPRB, 0.10906E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.11105E+01_JPRB, 0.10614E+01_JPRB, 0.10353E+01_JPRB, 0.10735E+01_JPRB, 0.10675E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.11098E+01_JPRB, 0.11052E+01_JPRB, 0.10363E+01_JPRB, 0.90147E+00_JPRB, 0.87308E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.86172E+00_JPRB, 0.83869E+00_JPRB, 0.81526E+00_JPRB, 0.79677E+00_JPRB, 0.78339E+00_JPRB, 0.76952E+00_JPRB, 0.74627E+00_JPRB ,&
&  0.71611E+00_JPRB, 0.69740E+00_JPRB, 0.66697E+00_JPRB, 0.65769E+00_JPRB, 0.66320E+00_JPRB, 0.66808E+00_JPRB, 0.67886E+00_JPRB ,&
&  0.68645E+00_JPRB, 0.69386E+00_JPRB, 0.69750E+00_JPRB, 0.69715E+00_JPRB, 0.69837E+00_JPRB, 0.70099E+00_JPRB, 0.70120E+00_JPRB ,&
&  0.69767E+00_JPRB, 0.70480E+00_JPRB, 0.72214E+00_JPRB, 0.75013E+00_JPRB, 0.81157E+00_JPRB, 0.92709E+00_JPRB, 0.11016E+01_JPRB ,&
&  0.15237E+01_JPRB, 0.17971E+01_JPRB, 0.19818E+01_JPRB, 0.30241E+01_JPRB, 0.36314E+01_JPRB, 0.34948E+01_JPRB, 0.34798E+01_JPRB ,&
&  0.35601E+01_JPRB, 0.35076E+01_JPRB, 0.31873E+01_JPRB, 0.24199E+01_JPRB, 0.19382E+01_JPRB, 0.16913E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.13387E+01_JPRB, 0.12127E+01_JPRB, 0.11154E+01_JPRB, 0.10765E+01_JPRB, 0.10933E+01_JPRB, 0.11457E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.15103E+01_JPRB, 0.18073E+01_JPRB, 0.21451E+01_JPRB, 0.27979E+01_JPRB, 0.33060E+01_JPRB, 0.31802E+01_JPRB ,&
&  0.30285E+01_JPRB, 0.28433E+01_JPRB, 0.25754E+01_JPRB, 0.23345E+01_JPRB, 0.18436E+01_JPRB, 0.18035E+01_JPRB, 0.16319E+01_JPRB ,&
&  0.14632E+01_JPRB, 0.13411E+01_JPRB, 0.12711E+01_JPRB, 0.12311E+01_JPRB, 0.11766E+01_JPRB, 0.11033E+01_JPRB, 0.10436E+01_JPRB ,&
&  0.10122E+01_JPRB, 0.10039E+01_JPRB, 0.10023E+01_JPRB, 0.99411E+00_JPRB, 0.98995E+00_JPRB, 0.99182E+00_JPRB, 0.99457E+00_JPRB ,&
&  0.99377E+00_JPRB, 0.99726E+00_JPRB, 0.99280E+00_JPRB, 0.10213E+01_JPRB, 0.10432E+01_JPRB, 0.10498E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10702E+01_JPRB, 0.10609E+01_JPRB, 0.10546E+01_JPRB, 0.10442E+01_JPRB, 0.10313E+01_JPRB, 0.10017E+01_JPRB ,&
&  0.99732E+00_JPRB, 0.96647E+00_JPRB, 0.91495E+00_JPRB, 0.89480E+00_JPRB, 0.87901E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.86147E+00_JPRB, 0.84041E+00_JPRB, 0.82526E+00_JPRB, 0.80866E+00_JPRB, 0.79377E+00_JPRB, 0.78575E+00_JPRB, 0.77888E+00_JPRB ,&
&  0.76574E+00_JPRB, 0.73428E+00_JPRB, 0.70124E+00_JPRB, 0.68421E+00_JPRB, 0.67796E+00_JPRB, 0.66472E+00_JPRB, 0.66467E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.67227E+00_JPRB, 0.66926E+00_JPRB, 0.66631E+00_JPRB, 0.66458E+00_JPRB, 0.66506E+00_JPRB, 0.66386E+00_JPRB ,&
&  0.65698E+00_JPRB, 0.65418E+00_JPRB, 0.66074E+00_JPRB, 0.67292E+00_JPRB, 0.69871E+00_JPRB, 0.75513E+00_JPRB, 0.83175E+00_JPRB ,&
&  0.10859E+01_JPRB, 0.14371E+01_JPRB, 0.15392E+01_JPRB, 0.29706E+01_JPRB, 0.35282E+01_JPRB, 0.37821E+01_JPRB, 0.36107E+01_JPRB ,&
&  0.36932E+01_JPRB, 0.41261E+01_JPRB, 0.34497E+01_JPRB, 0.25025E+01_JPRB, 0.18697E+01_JPRB, 0.15762E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.12678E+01_JPRB, 0.11485E+01_JPRB, 0.10661E+01_JPRB, 0.10188E+01_JPRB, 0.10135E+01_JPRB, 0.10249E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.12589E+01_JPRB, 0.14744E+01_JPRB, 0.16348E+01_JPRB, 0.21151E+01_JPRB, 0.26977E+01_JPRB, 0.30279E+01_JPRB ,&
&  0.32027E+01_JPRB, 0.32286E+01_JPRB, 0.27445E+01_JPRB, 0.24301E+01_JPRB, 0.20539E+01_JPRB, 0.18616E+01_JPRB, 0.16566E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.13915E+01_JPRB, 0.13054E+01_JPRB, 0.12546E+01_JPRB, 0.12054E+01_JPRB, 0.11505E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.10129E+01_JPRB, 0.10067E+01_JPRB, 0.99939E+00_JPRB, 0.99824E+00_JPRB, 0.99632E+00_JPRB, 0.99353E+00_JPRB ,&
&  0.98824E+00_JPRB, 0.98187E+00_JPRB, 0.96320E+00_JPRB, 0.97658E+00_JPRB, 0.98485E+00_JPRB, 0.97108E+00_JPRB, 0.97706E+00_JPRB ,&
&  0.10210E+01_JPRB, 0.10553E+01_JPRB, 0.10513E+01_JPRB, 0.10251E+01_JPRB, 0.10450E+01_JPRB, 0.10640E+01_JPRB, 0.97992E+00_JPRB ,&
&  0.95765E+00_JPRB, 0.93581E+00_JPRB, 0.90649E+00_JPRB, 0.88809E+00_JPRB, 0.87642E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.85737E+00_JPRB, 0.82796E+00_JPRB, 0.80722E+00_JPRB, 0.79091E+00_JPRB, 0.77689E+00_JPRB, 0.76449E+00_JPRB, 0.76117E+00_JPRB ,&
&  0.75771E+00_JPRB, 0.74962E+00_JPRB, 0.72851E+00_JPRB, 0.70313E+00_JPRB, 0.69053E+00_JPRB, 0.68459E+00_JPRB, 0.66979E+00_JPRB ,&
&  0.66210E+00_JPRB, 0.66119E+00_JPRB, 0.65883E+00_JPRB, 0.65038E+00_JPRB, 0.64233E+00_JPRB, 0.63711E+00_JPRB, 0.63453E+00_JPRB ,&
&  0.62354E+00_JPRB, 0.61870E+00_JPRB, 0.61993E+00_JPRB, 0.62194E+00_JPRB, 0.63023E+00_JPRB, 0.67277E+00_JPRB, 0.75290E+00_JPRB ,&
&  0.90478E+00_JPRB, 0.16142E+01_JPRB, 0.15229E+01_JPRB, 0.24066E+01_JPRB, 0.27588E+01_JPRB, 0.31750E+01_JPRB, 0.34258E+01_JPRB ,&
&  0.37102E+01_JPRB, 0.40231E+01_JPRB, 0.32667E+01_JPRB, 0.24084E+01_JPRB, 0.17086E+01_JPRB, 0.13387E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.10301E+01_JPRB, 0.97778E+00_JPRB, 0.95811E+00_JPRB, 0.95747E+00_JPRB, 0.95969E+00_JPRB, 0.96805E+00_JPRB ,&
&  0.10061E+01_JPRB, 0.10940E+01_JPRB, 0.12989E+01_JPRB, 0.15446E+01_JPRB, 0.16435E+01_JPRB, 0.20417E+01_JPRB, 0.26221E+01_JPRB ,&
&  0.31304E+01_JPRB, 0.33864E+01_JPRB, 0.28743E+01_JPRB, 0.24531E+01_JPRB, 0.21372E+01_JPRB, 0.18972E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.15315E+01_JPRB, 0.14119E+01_JPRB, 0.13230E+01_JPRB, 0.12590E+01_JPRB, 0.12137E+01_JPRB, 0.11798E+01_JPRB, 0.11339E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.10490E+01_JPRB, 0.10307E+01_JPRB, 0.10207E+01_JPRB, 0.10144E+01_JPRB, 0.10013E+01_JPRB, 0.98022E+00_JPRB ,&
&  0.96367E+00_JPRB, 0.95923E+00_JPRB, 0.94371E+00_JPRB, 0.94948E+00_JPRB, 0.95077E+00_JPRB, 0.93030E+00_JPRB, 0.92824E+00_JPRB ,&
&  0.97759E+00_JPRB, 0.10085E+01_JPRB, 0.10202E+01_JPRB, 0.10289E+01_JPRB, 0.10695E+01_JPRB, 0.11620E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.95644E+00_JPRB, 0.92642E+00_JPRB, 0.90290E+00_JPRB, 0.88996E+00_JPRB, 0.87633E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.84966E+00_JPRB, 0.81916E+00_JPRB, 0.79427E+00_JPRB, 0.77026E+00_JPRB, 0.75277E+00_JPRB, 0.73474E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.71841E+00_JPRB, 0.71922E+00_JPRB, 0.71762E+00_JPRB, 0.70799E+00_JPRB, 0.69066E+00_JPRB, 0.67587E+00_JPRB, 0.66451E+00_JPRB ,&
&  0.65315E+00_JPRB, 0.64914E+00_JPRB, 0.64663E+00_JPRB, 0.63644E+00_JPRB, 0.62511E+00_JPRB, 0.61560E+00_JPRB, 0.60649E+00_JPRB ,&
&  0.59897E+00_JPRB, 0.59543E+00_JPRB, 0.59269E+00_JPRB, 0.59023E+00_JPRB, 0.58940E+00_JPRB, 0.61303E+00_JPRB, 0.72640E+00_JPRB ,&
&  0.97080E+00_JPRB, 0.85234E+00_JPRB, 0.19034E+01_JPRB, 0.21891E+01_JPRB, 0.22891E+01_JPRB, 0.24650E+01_JPRB, 0.29188E+01_JPRB ,&
&  0.33864E+01_JPRB, 0.33599E+01_JPRB, 0.29005E+01_JPRB, 0.22562E+01_JPRB, 0.16857E+01_JPRB, 0.13159E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.99537E+00_JPRB, 0.92261E+00_JPRB, 0.88783E+00_JPRB, 0.86910E+00_JPRB, 0.86791E+00_JPRB, 0.87789E+00_JPRB, 0.89302E+00_JPRB ,&
&  0.92218E+00_JPRB, 0.98268E+00_JPRB, 0.11345E+01_JPRB, 0.14108E+01_JPRB, 0.16304E+01_JPRB, 0.18107E+01_JPRB, 0.21905E+01_JPRB ,&
&  0.28941E+01_JPRB, 0.30533E+01_JPRB, 0.25815E+01_JPRB, 0.22359E+01_JPRB, 0.20089E+01_JPRB, 0.18456E+01_JPRB, 0.16852E+01_JPRB ,&
&  0.15358E+01_JPRB, 0.13940E+01_JPRB, 0.12914E+01_JPRB, 0.12201E+01_JPRB, 0.11790E+01_JPRB, 0.11621E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.11036E+01_JPRB, 0.10799E+01_JPRB, 0.10562E+01_JPRB, 0.10338E+01_JPRB, 0.10063E+01_JPRB, 0.96877E+00_JPRB ,&
&  0.93995E+00_JPRB, 0.92587E+00_JPRB, 0.91152E+00_JPRB, 0.90180E+00_JPRB, 0.90414E+00_JPRB, 0.90335E+00_JPRB, 0.91093E+00_JPRB ,&
&  0.94221E+00_JPRB, 0.95673E+00_JPRB, 0.96939E+00_JPRB, 0.99503E+00_JPRB, 0.10112E+01_JPRB, 0.10893E+01_JPRB, 0.10349E+01_JPRB ,&
&  0.99489E+00_JPRB, 0.94913E+00_JPRB, 0.90778E+00_JPRB, 0.88532E+00_JPRB, 0.87128E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.83674E+00_JPRB, 0.81284E+00_JPRB, 0.78058E+00_JPRB, 0.75204E+00_JPRB, 0.72931E+00_JPRB, 0.70615E+00_JPRB, 0.69003E+00_JPRB ,&
&  0.67832E+00_JPRB, 0.67937E+00_JPRB, 0.68470E+00_JPRB, 0.68760E+00_JPRB, 0.68172E+00_JPRB, 0.66531E+00_JPRB, 0.64845E+00_JPRB ,&
&  0.63611E+00_JPRB, 0.63154E+00_JPRB, 0.62952E+00_JPRB, 0.62365E+00_JPRB, 0.61486E+00_JPRB, 0.60402E+00_JPRB, 0.59452E+00_JPRB ,&
&  0.58623E+00_JPRB, 0.58036E+00_JPRB, 0.57431E+00_JPRB, 0.56968E+00_JPRB, 0.56427E+00_JPRB, 0.57667E+00_JPRB, 0.68496E+00_JPRB ,&
&  0.10220E+01_JPRB, 0.14484E+01_JPRB, 0.18614E+01_JPRB, 0.17468E+01_JPRB, 0.17392E+01_JPRB, 0.18335E+01_JPRB, 0.21687E+01_JPRB ,&
&  0.23614E+01_JPRB, 0.24442E+01_JPRB, 0.23101E+01_JPRB, 0.19844E+01_JPRB, 0.15954E+01_JPRB, 0.13220E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.96307E+00_JPRB, 0.86384E+00_JPRB, 0.80943E+00_JPRB, 0.79273E+00_JPRB, 0.79681E+00_JPRB, 0.80991E+00_JPRB, 0.82280E+00_JPRB ,&
&  0.83997E+00_JPRB, 0.87608E+00_JPRB, 0.99046E+00_JPRB, 0.12478E+01_JPRB, 0.16519E+01_JPRB, 0.16940E+01_JPRB, 0.21594E+01_JPRB ,&
&  0.27221E+01_JPRB, 0.24498E+01_JPRB, 0.22229E+01_JPRB, 0.20308E+01_JPRB, 0.18777E+01_JPRB, 0.17480E+01_JPRB, 0.15576E+01_JPRB ,&
&  0.14103E+01_JPRB, 0.12776E+01_JPRB, 0.11810E+01_JPRB, 0.11295E+01_JPRB, 0.10990E+01_JPRB, 0.10872E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.10795E+01_JPRB, 0.10718E+01_JPRB, 0.10659E+01_JPRB, 0.10485E+01_JPRB, 0.10208E+01_JPRB, 0.98782E+00_JPRB, 0.95121E+00_JPRB ,&
&  0.92035E+00_JPRB, 0.89201E+00_JPRB, 0.88123E+00_JPRB, 0.85653E+00_JPRB, 0.85714E+00_JPRB, 0.86652E+00_JPRB, 0.86917E+00_JPRB ,&
&  0.86884E+00_JPRB, 0.88650E+00_JPRB, 0.90983E+00_JPRB, 0.92267E+00_JPRB, 0.99258E+00_JPRB, 0.10619E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.10142E+01_JPRB, 0.95567E+00_JPRB, 0.90501E+00_JPRB, 0.86996E+00_JPRB, 0.85425E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.82678E+00_JPRB, 0.79507E+00_JPRB, 0.76129E+00_JPRB, 0.73110E+00_JPRB, 0.70372E+00_JPRB, 0.68281E+00_JPRB, 0.66986E+00_JPRB ,&
&  0.65854E+00_JPRB, 0.65185E+00_JPRB, 0.65419E+00_JPRB, 0.66008E+00_JPRB, 0.65950E+00_JPRB, 0.64947E+00_JPRB, 0.63485E+00_JPRB ,&
&  0.61969E+00_JPRB, 0.60787E+00_JPRB, 0.59918E+00_JPRB, 0.59522E+00_JPRB, 0.58943E+00_JPRB, 0.58343E+00_JPRB, 0.57411E+00_JPRB ,&
&  0.56701E+00_JPRB, 0.56122E+00_JPRB, 0.55243E+00_JPRB, 0.54303E+00_JPRB, 0.53613E+00_JPRB, 0.54132E+00_JPRB, 0.62273E+00_JPRB ,&
&  0.87862E+00_JPRB, 0.12453E+01_JPRB, 0.12823E+01_JPRB, 0.13215E+01_JPRB, 0.14455E+01_JPRB, 0.14820E+01_JPRB, 0.16113E+01_JPRB ,&
&  0.16778E+01_JPRB, 0.17194E+01_JPRB, 0.17381E+01_JPRB, 0.16341E+01_JPRB, 0.14238E+01_JPRB, 0.12570E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.97104E+00_JPRB, 0.86640E+00_JPRB, 0.80080E+00_JPRB, 0.77229E+00_JPRB, 0.76977E+00_JPRB, 0.77859E+00_JPRB, 0.78429E+00_JPRB ,&
&  0.78608E+00_JPRB, 0.78768E+00_JPRB, 0.82592E+00_JPRB, 0.93030E+00_JPRB, 0.11922E+01_JPRB, 0.13630E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.17245E+01_JPRB, 0.16909E+01_JPRB, 0.16303E+01_JPRB, 0.15597E+01_JPRB, 0.14832E+01_JPRB, 0.13856E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.11247E+01_JPRB, 0.10582E+01_JPRB, 0.10152E+01_JPRB, 0.99050E+00_JPRB, 0.97732E+00_JPRB, 0.97460E+00_JPRB ,&
&  0.96348E+00_JPRB, 0.95802E+00_JPRB, 0.94705E+00_JPRB, 0.94015E+00_JPRB, 0.93573E+00_JPRB, 0.91585E+00_JPRB, 0.88430E+00_JPRB ,&
&  0.84994E+00_JPRB, 0.81957E+00_JPRB, 0.80553E+00_JPRB, 0.80068E+00_JPRB, 0.79938E+00_JPRB, 0.79704E+00_JPRB, 0.79522E+00_JPRB ,&
&  0.80483E+00_JPRB, 0.83703E+00_JPRB, 0.85000E+00_JPRB, 0.85986E+00_JPRB, 0.90613E+00_JPRB, 0.95708E+00_JPRB, 0.95520E+00_JPRB ,&
&  0.94246E+00_JPRB, 0.91585E+00_JPRB, 0.88034E+00_JPRB, 0.84690E+00_JPRB, 0.84618E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.83150E+00_JPRB, 0.80226E+00_JPRB, 0.76553E+00_JPRB, 0.73023E+00_JPRB, 0.69809E+00_JPRB, 0.67292E+00_JPRB, 0.65762E+00_JPRB ,&
&  0.64720E+00_JPRB, 0.63995E+00_JPRB, 0.63597E+00_JPRB, 0.63333E+00_JPRB, 0.62947E+00_JPRB, 0.62523E+00_JPRB, 0.61850E+00_JPRB ,&
&  0.60980E+00_JPRB, 0.59898E+00_JPRB, 0.58901E+00_JPRB, 0.58187E+00_JPRB, 0.57619E+00_JPRB, 0.57276E+00_JPRB, 0.56796E+00_JPRB ,&
&  0.55876E+00_JPRB, 0.54797E+00_JPRB, 0.53450E+00_JPRB, 0.52085E+00_JPRB, 0.51201E+00_JPRB, 0.51371E+00_JPRB, 0.56562E+00_JPRB ,&
&  0.74854E+00_JPRB, 0.86029E+00_JPRB, 0.95663E+00_JPRB, 0.10268E+01_JPRB, 0.11274E+01_JPRB, 0.11905E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.12516E+01_JPRB, 0.12821E+01_JPRB, 0.13393E+01_JPRB, 0.13320E+01_JPRB, 0.12289E+01_JPRB, 0.11257E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.94073E+00_JPRB, 0.86535E+00_JPRB, 0.80430E+00_JPRB, 0.77532E+00_JPRB, 0.76521E+00_JPRB, 0.76627E+00_JPRB, 0.76557E+00_JPRB ,&
&  0.75911E+00_JPRB, 0.74256E+00_JPRB, 0.73367E+00_JPRB, 0.74479E+00_JPRB, 0.78551E+00_JPRB, 0.86467E+00_JPRB, 0.99937E+00_JPRB ,&
&  0.10645E+01_JPRB, 0.10661E+01_JPRB, 0.10770E+01_JPRB, 0.10839E+01_JPRB, 0.10561E+01_JPRB, 0.10229E+01_JPRB, 0.99342E+00_JPRB ,&
&  0.96596E+00_JPRB, 0.93985E+00_JPRB, 0.91911E+00_JPRB, 0.89394E+00_JPRB, 0.87295E+00_JPRB, 0.85911E+00_JPRB, 0.85026E+00_JPRB ,&
&  0.84269E+00_JPRB, 0.83821E+00_JPRB, 0.82858E+00_JPRB, 0.80969E+00_JPRB, 0.79511E+00_JPRB, 0.78066E+00_JPRB, 0.75971E+00_JPRB ,&
&  0.73322E+00_JPRB, 0.71765E+00_JPRB, 0.71245E+00_JPRB, 0.71529E+00_JPRB, 0.72226E+00_JPRB, 0.72985E+00_JPRB, 0.73376E+00_JPRB ,&
&  0.73874E+00_JPRB, 0.76087E+00_JPRB, 0.76527E+00_JPRB, 0.80541E+00_JPRB, 0.85886E+00_JPRB, 0.86633E+00_JPRB, 0.87452E+00_JPRB ,&
&  0.87553E+00_JPRB, 0.86204E+00_JPRB, 0.83497E+00_JPRB, 0.80127E+00_JPRB, 0.83534E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.72609E+00_JPRB, 0.72542E+00_JPRB, 0.70668E+00_JPRB, 0.68421E+00_JPRB, 0.66400E+00_JPRB, 0.64836E+00_JPRB, 0.63515E+00_JPRB ,&
&  0.62562E+00_JPRB, 0.61831E+00_JPRB, 0.61325E+00_JPRB, 0.60971E+00_JPRB, 0.60505E+00_JPRB, 0.59831E+00_JPRB, 0.59103E+00_JPRB ,&
&  0.58493E+00_JPRB, 0.57770E+00_JPRB, 0.56700E+00_JPRB, 0.55929E+00_JPRB, 0.55462E+00_JPRB, 0.54897E+00_JPRB, 0.54318E+00_JPRB ,&
&  0.53589E+00_JPRB, 0.52175E+00_JPRB, 0.50749E+00_JPRB, 0.49772E+00_JPRB, 0.49000E+00_JPRB, 0.48862E+00_JPRB, 0.52357E+00_JPRB ,&
&  0.63767E+00_JPRB, 0.64941E+00_JPRB, 0.73012E+00_JPRB, 0.81038E+00_JPRB, 0.91168E+00_JPRB, 0.96918E+00_JPRB, 0.99155E+00_JPRB ,&
&  0.10021E+01_JPRB, 0.10372E+01_JPRB, 0.10663E+01_JPRB, 0.11057E+01_JPRB, 0.10660E+01_JPRB, 0.99437E+00_JPRB, 0.93229E+00_JPRB ,&
&  0.87346E+00_JPRB, 0.82480E+00_JPRB, 0.78506E+00_JPRB, 0.75764E+00_JPRB, 0.74585E+00_JPRB, 0.74061E+00_JPRB, 0.73075E+00_JPRB ,&
&  0.71721E+00_JPRB, 0.69657E+00_JPRB, 0.67784E+00_JPRB, 0.67187E+00_JPRB, 0.66448E+00_JPRB, 0.68531E+00_JPRB, 0.71643E+00_JPRB ,&
&  0.74357E+00_JPRB, 0.78243E+00_JPRB, 0.80156E+00_JPRB, 0.81318E+00_JPRB, 0.80637E+00_JPRB, 0.79755E+00_JPRB, 0.78750E+00_JPRB ,&
&  0.77894E+00_JPRB, 0.77025E+00_JPRB, 0.76251E+00_JPRB, 0.75175E+00_JPRB, 0.73887E+00_JPRB, 0.72837E+00_JPRB, 0.71947E+00_JPRB ,&
&  0.71175E+00_JPRB, 0.70382E+00_JPRB, 0.69692E+00_JPRB, 0.68185E+00_JPRB, 0.66093E+00_JPRB, 0.64368E+00_JPRB, 0.63053E+00_JPRB ,&
&  0.62028E+00_JPRB, 0.61739E+00_JPRB, 0.61943E+00_JPRB, 0.62856E+00_JPRB, 0.63787E+00_JPRB, 0.64839E+00_JPRB, 0.65559E+00_JPRB ,&
&  0.66093E+00_JPRB, 0.67583E+00_JPRB, 0.69878E+00_JPRB, 0.74063E+00_JPRB, 0.76652E+00_JPRB, 0.78473E+00_JPRB, 0.79794E+00_JPRB ,&
&  0.80174E+00_JPRB, 0.79786E+00_JPRB, 0.77710E+00_JPRB, 0.75318E+00_JPRB, 0.71823E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.66675E+00_JPRB, 0.66584E+00_JPRB, 0.65465E+00_JPRB, 0.63825E+00_JPRB, 0.62414E+00_JPRB, 0.61184E+00_JPRB, 0.60206E+00_JPRB ,&
&  0.58876E+00_JPRB, 0.57725E+00_JPRB, 0.57239E+00_JPRB, 0.57102E+00_JPRB, 0.56922E+00_JPRB, 0.56411E+00_JPRB, 0.55751E+00_JPRB ,&
&  0.55023E+00_JPRB, 0.54204E+00_JPRB, 0.53254E+00_JPRB, 0.52295E+00_JPRB, 0.51491E+00_JPRB, 0.50600E+00_JPRB, 0.49894E+00_JPRB ,&
&  0.49217E+00_JPRB, 0.48599E+00_JPRB, 0.47651E+00_JPRB, 0.46517E+00_JPRB, 0.45991E+00_JPRB, 0.45528E+00_JPRB, 0.48492E+00_JPRB ,&
&  0.57883E+00_JPRB, 0.51095E+00_JPRB, 0.56507E+00_JPRB, 0.65127E+00_JPRB, 0.74597E+00_JPRB, 0.83651E+00_JPRB, 0.86592E+00_JPRB ,&
&  0.87797E+00_JPRB, 0.89209E+00_JPRB, 0.90204E+00_JPRB, 0.92008E+00_JPRB, 0.92599E+00_JPRB, 0.88827E+00_JPRB, 0.83543E+00_JPRB ,&
&  0.79298E+00_JPRB, 0.75595E+00_JPRB, 0.73248E+00_JPRB, 0.71045E+00_JPRB, 0.69846E+00_JPRB, 0.69193E+00_JPRB, 0.67740E+00_JPRB ,&
&  0.65632E+00_JPRB, 0.64090E+00_JPRB, 0.62835E+00_JPRB, 0.62499E+00_JPRB, 0.62593E+00_JPRB, 0.62841E+00_JPRB, 0.61845E+00_JPRB ,&
&  0.60866E+00_JPRB, 0.61842E+00_JPRB, 0.63792E+00_JPRB, 0.64804E+00_JPRB, 0.65102E+00_JPRB, 0.64693E+00_JPRB, 0.63870E+00_JPRB ,&
&  0.63259E+00_JPRB, 0.62758E+00_JPRB, 0.62606E+00_JPRB, 0.62098E+00_JPRB, 0.61361E+00_JPRB, 0.60359E+00_JPRB, 0.59334E+00_JPRB ,&
&  0.58451E+00_JPRB, 0.57725E+00_JPRB, 0.57099E+00_JPRB, 0.56481E+00_JPRB, 0.55725E+00_JPRB, 0.54709E+00_JPRB, 0.54135E+00_JPRB ,&
&  0.53860E+00_JPRB, 0.53922E+00_JPRB, 0.54320E+00_JPRB, 0.55063E+00_JPRB, 0.56132E+00_JPRB, 0.57147E+00_JPRB, 0.58358E+00_JPRB ,&
&  0.59490E+00_JPRB, 0.60802E+00_JPRB, 0.61992E+00_JPRB, 0.68078E+00_JPRB, 0.72077E+00_JPRB, 0.73373E+00_JPRB, 0.74375E+00_JPRB ,&
&  0.74600E+00_JPRB, 0.74281E+00_JPRB, 0.73346E+00_JPRB, 0.64435E+00_JPRB, 0.66555E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.61329E+00_JPRB, 0.60964E+00_JPRB, 0.60005E+00_JPRB, 0.58909E+00_JPRB, 0.57984E+00_JPRB, 0.57083E+00_JPRB, 0.56358E+00_JPRB ,&
&  0.55379E+00_JPRB, 0.54197E+00_JPRB, 0.53306E+00_JPRB, 0.52870E+00_JPRB, 0.52638E+00_JPRB, 0.52261E+00_JPRB, 0.51651E+00_JPRB ,&
&  0.50876E+00_JPRB, 0.50044E+00_JPRB, 0.49237E+00_JPRB, 0.48264E+00_JPRB, 0.47341E+00_JPRB, 0.46500E+00_JPRB, 0.45802E+00_JPRB ,&
&  0.45108E+00_JPRB, 0.44370E+00_JPRB, 0.43645E+00_JPRB, 0.43133E+00_JPRB, 0.42743E+00_JPRB, 0.42633E+00_JPRB, 0.45063E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.45031E+00_JPRB, 0.48817E+00_JPRB, 0.54065E+00_JPRB, 0.62398E+00_JPRB, 0.72581E+00_JPRB, 0.77172E+00_JPRB ,&
&  0.78915E+00_JPRB, 0.79787E+00_JPRB, 0.79661E+00_JPRB, 0.79481E+00_JPRB, 0.80280E+00_JPRB, 0.79698E+00_JPRB, 0.76885E+00_JPRB ,&
&  0.73587E+00_JPRB, 0.70499E+00_JPRB, 0.68335E+00_JPRB, 0.66737E+00_JPRB, 0.65183E+00_JPRB, 0.63773E+00_JPRB, 0.61948E+00_JPRB ,&
&  0.59468E+00_JPRB, 0.57805E+00_JPRB, 0.56719E+00_JPRB, 0.55958E+00_JPRB, 0.55646E+00_JPRB, 0.55103E+00_JPRB, 0.53603E+00_JPRB ,&
&  0.52166E+00_JPRB, 0.51164E+00_JPRB, 0.50753E+00_JPRB, 0.50896E+00_JPRB, 0.51195E+00_JPRB, 0.51309E+00_JPRB, 0.50987E+00_JPRB ,&
&  0.50536E+00_JPRB, 0.49889E+00_JPRB, 0.49498E+00_JPRB, 0.49245E+00_JPRB, 0.48992E+00_JPRB, 0.48910E+00_JPRB, 0.48744E+00_JPRB ,&
&  0.48445E+00_JPRB, 0.48333E+00_JPRB, 0.48480E+00_JPRB, 0.48641E+00_JPRB, 0.48601E+00_JPRB, 0.48383E+00_JPRB, 0.48106E+00_JPRB ,&
&  0.47761E+00_JPRB, 0.47534E+00_JPRB, 0.47612E+00_JPRB, 0.48134E+00_JPRB, 0.48943E+00_JPRB, 0.49894E+00_JPRB, 0.50959E+00_JPRB ,&
&  0.52149E+00_JPRB, 0.53441E+00_JPRB, 0.54586E+00_JPRB, 0.58240E+00_JPRB, 0.60720E+00_JPRB, 0.63039E+00_JPRB, 0.65129E+00_JPRB ,&
&  0.65877E+00_JPRB, 0.65760E+00_JPRB, 0.56072E+00_JPRB, 0.59995E+00_JPRB, 0.60844E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.57448E+00_JPRB, 0.56256E+00_JPRB, 0.54841E+00_JPRB, 0.53863E+00_JPRB, 0.53362E+00_JPRB, 0.52828E+00_JPRB, 0.52306E+00_JPRB ,&
&  0.51572E+00_JPRB, 0.50768E+00_JPRB, 0.49974E+00_JPRB, 0.49250E+00_JPRB, 0.48667E+00_JPRB, 0.48260E+00_JPRB, 0.47783E+00_JPRB ,&
&  0.47115E+00_JPRB, 0.46353E+00_JPRB, 0.45559E+00_JPRB, 0.44660E+00_JPRB, 0.43499E+00_JPRB, 0.42556E+00_JPRB, 0.41804E+00_JPRB ,&
&  0.41215E+00_JPRB, 0.40749E+00_JPRB, 0.40548E+00_JPRB, 0.40318E+00_JPRB, 0.40080E+00_JPRB, 0.40100E+00_JPRB, 0.41890E+00_JPRB ,&
&  0.43701E+00_JPRB, 0.41116E+00_JPRB, 0.43878E+00_JPRB, 0.47771E+00_JPRB, 0.54082E+00_JPRB, 0.63987E+00_JPRB, 0.70243E+00_JPRB ,&
&  0.73368E+00_JPRB, 0.74654E+00_JPRB, 0.74815E+00_JPRB, 0.73755E+00_JPRB, 0.72787E+00_JPRB, 0.72955E+00_JPRB, 0.72434E+00_JPRB ,&
&  0.70356E+00_JPRB, 0.67519E+00_JPRB, 0.64897E+00_JPRB, 0.62927E+00_JPRB, 0.60625E+00_JPRB, 0.58583E+00_JPRB, 0.56496E+00_JPRB ,&
&  0.54589E+00_JPRB, 0.53160E+00_JPRB, 0.52227E+00_JPRB, 0.51521E+00_JPRB, 0.50777E+00_JPRB, 0.49605E+00_JPRB, 0.48235E+00_JPRB ,&
&  0.47084E+00_JPRB, 0.46057E+00_JPRB, 0.45141E+00_JPRB, 0.44373E+00_JPRB, 0.43876E+00_JPRB, 0.43562E+00_JPRB, 0.43283E+00_JPRB ,&
&  0.43194E+00_JPRB, 0.43213E+00_JPRB, 0.43287E+00_JPRB, 0.43326E+00_JPRB, 0.43243E+00_JPRB, 0.43045E+00_JPRB, 0.42961E+00_JPRB ,&
&  0.42962E+00_JPRB, 0.42706E+00_JPRB, 0.42628E+00_JPRB, 0.42964E+00_JPRB, 0.43369E+00_JPRB, 0.43603E+00_JPRB, 0.43651E+00_JPRB ,&
&  0.43495E+00_JPRB, 0.43387E+00_JPRB, 0.43435E+00_JPRB, 0.43712E+00_JPRB, 0.44113E+00_JPRB, 0.44729E+00_JPRB, 0.45492E+00_JPRB ,&
&  0.46290E+00_JPRB, 0.47307E+00_JPRB, 0.48604E+00_JPRB, 0.50759E+00_JPRB, 0.52695E+00_JPRB, 0.54465E+00_JPRB, 0.56168E+00_JPRB ,&
&  0.56957E+00_JPRB, 0.57471E+00_JPRB, 0.58167E+00_JPRB, 0.58116E+00_JPRB, 0.57973E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.50545E+00_JPRB, 0.49894E+00_JPRB, 0.49070E+00_JPRB, 0.48342E+00_JPRB, 0.47944E+00_JPRB, 0.47583E+00_JPRB, 0.47020E+00_JPRB ,&
&  0.46522E+00_JPRB, 0.46074E+00_JPRB, 0.45623E+00_JPRB, 0.45216E+00_JPRB, 0.44851E+00_JPRB, 0.44563E+00_JPRB, 0.44138E+00_JPRB ,&
&  0.43502E+00_JPRB, 0.42719E+00_JPRB, 0.41851E+00_JPRB, 0.41120E+00_JPRB, 0.40237E+00_JPRB, 0.39317E+00_JPRB, 0.38619E+00_JPRB ,&
&  0.38034E+00_JPRB, 0.37626E+00_JPRB, 0.37454E+00_JPRB, 0.37235E+00_JPRB, 0.37289E+00_JPRB, 0.37584E+00_JPRB, 0.38729E+00_JPRB ,&
&  0.41481E+00_JPRB, 0.39948E+00_JPRB, 0.42661E+00_JPRB, 0.45234E+00_JPRB, 0.50409E+00_JPRB, 0.58317E+00_JPRB, 0.63468E+00_JPRB ,&
&  0.66539E+00_JPRB, 0.68975E+00_JPRB, 0.70628E+00_JPRB, 0.70667E+00_JPRB, 0.69350E+00_JPRB, 0.68724E+00_JPRB, 0.68429E+00_JPRB ,&
&  0.67239E+00_JPRB, 0.65048E+00_JPRB, 0.61969E+00_JPRB, 0.58660E+00_JPRB, 0.55724E+00_JPRB, 0.53517E+00_JPRB, 0.51864E+00_JPRB ,&
&  0.50606E+00_JPRB, 0.49541E+00_JPRB, 0.48644E+00_JPRB, 0.47438E+00_JPRB, 0.46041E+00_JPRB, 0.44611E+00_JPRB, 0.43231E+00_JPRB ,&
&  0.41847E+00_JPRB, 0.40525E+00_JPRB, 0.39396E+00_JPRB, 0.38485E+00_JPRB, 0.37943E+00_JPRB, 0.37643E+00_JPRB, 0.37483E+00_JPRB ,&
&  0.37459E+00_JPRB, 0.37636E+00_JPRB, 0.37882E+00_JPRB, 0.38093E+00_JPRB, 0.38220E+00_JPRB, 0.38157E+00_JPRB, 0.37865E+00_JPRB ,&
&  0.37740E+00_JPRB, 0.37673E+00_JPRB, 0.37679E+00_JPRB, 0.37839E+00_JPRB, 0.37913E+00_JPRB, 0.38035E+00_JPRB, 0.38345E+00_JPRB ,&
&  0.38626E+00_JPRB, 0.38772E+00_JPRB, 0.38893E+00_JPRB, 0.39052E+00_JPRB, 0.39265E+00_JPRB, 0.39621E+00_JPRB, 0.40151E+00_JPRB ,&
&  0.40762E+00_JPRB, 0.41458E+00_JPRB, 0.42475E+00_JPRB, 0.43808E+00_JPRB, 0.45156E+00_JPRB, 0.46450E+00_JPRB, 0.47684E+00_JPRB ,&
&  0.48413E+00_JPRB, 0.49099E+00_JPRB, 0.49996E+00_JPRB, 0.50450E+00_JPRB, 0.50756E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.44859E+00_JPRB, 0.44598E+00_JPRB, 0.44251E+00_JPRB, 0.43921E+00_JPRB, 0.43655E+00_JPRB, 0.43178E+00_JPRB, 0.42613E+00_JPRB ,&
&  0.42240E+00_JPRB, 0.41912E+00_JPRB, 0.41676E+00_JPRB, 0.41312E+00_JPRB, 0.41089E+00_JPRB, 0.40931E+00_JPRB, 0.40473E+00_JPRB ,&
&  0.39938E+00_JPRB, 0.39188E+00_JPRB, 0.38455E+00_JPRB, 0.37785E+00_JPRB, 0.37145E+00_JPRB, 0.36542E+00_JPRB, 0.35872E+00_JPRB ,&
&  0.35236E+00_JPRB, 0.34755E+00_JPRB, 0.34476E+00_JPRB, 0.34240E+00_JPRB, 0.34356E+00_JPRB, 0.34948E+00_JPRB, 0.35946E+00_JPRB ,&
&  0.37206E+00_JPRB, 0.39015E+00_JPRB, 0.40873E+00_JPRB, 0.42827E+00_JPRB, 0.46755E+00_JPRB, 0.52536E+00_JPRB, 0.56791E+00_JPRB ,&
&  0.58866E+00_JPRB, 0.61226E+00_JPRB, 0.64275E+00_JPRB, 0.66291E+00_JPRB, 0.66463E+00_JPRB, 0.65440E+00_JPRB, 0.64641E+00_JPRB ,&
&  0.63571E+00_JPRB, 0.61495E+00_JPRB, 0.59187E+00_JPRB, 0.56518E+00_JPRB, 0.53844E+00_JPRB, 0.51660E+00_JPRB, 0.49966E+00_JPRB ,&
&  0.48521E+00_JPRB, 0.47210E+00_JPRB, 0.45938E+00_JPRB, 0.44570E+00_JPRB, 0.43079E+00_JPRB, 0.41435E+00_JPRB, 0.39683E+00_JPRB ,&
&  0.38010E+00_JPRB, 0.36621E+00_JPRB, 0.35570E+00_JPRB, 0.34961E+00_JPRB, 0.34572E+00_JPRB, 0.34234E+00_JPRB, 0.33987E+00_JPRB ,&
&  0.33940E+00_JPRB, 0.34045E+00_JPRB, 0.34262E+00_JPRB, 0.34501E+00_JPRB, 0.34708E+00_JPRB, 0.34750E+00_JPRB, 0.34488E+00_JPRB ,&
&  0.34211E+00_JPRB, 0.34139E+00_JPRB, 0.34139E+00_JPRB, 0.33851E+00_JPRB, 0.33537E+00_JPRB, 0.33606E+00_JPRB, 0.33894E+00_JPRB ,&
&  0.34383E+00_JPRB, 0.34824E+00_JPRB, 0.35174E+00_JPRB, 0.35449E+00_JPRB, 0.35644E+00_JPRB, 0.35992E+00_JPRB, 0.36435E+00_JPRB ,&
&  0.36990E+00_JPRB, 0.37679E+00_JPRB, 0.38636E+00_JPRB, 0.39818E+00_JPRB, 0.40915E+00_JPRB, 0.41870E+00_JPRB, 0.42534E+00_JPRB ,&
&  0.42998E+00_JPRB, 0.43387E+00_JPRB, 0.43800E+00_JPRB, 0.44287E+00_JPRB, 0.44712E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.39644E+00_JPRB, 0.39667E+00_JPRB, 0.39737E+00_JPRB, 0.39727E+00_JPRB, 0.39384E+00_JPRB, 0.39003E+00_JPRB, 0.38582E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38218E+00_JPRB, 0.38158E+00_JPRB, 0.37834E+00_JPRB, 0.37580E+00_JPRB, 0.37410E+00_JPRB, 0.36994E+00_JPRB ,&
&  0.36581E+00_JPRB, 0.35741E+00_JPRB, 0.34863E+00_JPRB, 0.34225E+00_JPRB, 0.33824E+00_JPRB, 0.33533E+00_JPRB, 0.32960E+00_JPRB ,&
&  0.32529E+00_JPRB, 0.32182E+00_JPRB, 0.32001E+00_JPRB, 0.31996E+00_JPRB, 0.32214E+00_JPRB, 0.32820E+00_JPRB, 0.33805E+00_JPRB ,&
&  0.34617E+00_JPRB, 0.36380E+00_JPRB, 0.36756E+00_JPRB, 0.38824E+00_JPRB, 0.41309E+00_JPRB, 0.45084E+00_JPRB, 0.48815E+00_JPRB ,&
&  0.50894E+00_JPRB, 0.52192E+00_JPRB, 0.54321E+00_JPRB, 0.56919E+00_JPRB, 0.58188E+00_JPRB, 0.58462E+00_JPRB, 0.57990E+00_JPRB ,&
&  0.57222E+00_JPRB, 0.56366E+00_JPRB, 0.55386E+00_JPRB, 0.54066E+00_JPRB, 0.52236E+00_JPRB, 0.50218E+00_JPRB, 0.48289E+00_JPRB ,&
&  0.46548E+00_JPRB, 0.44866E+00_JPRB, 0.43188E+00_JPRB, 0.41351E+00_JPRB, 0.39456E+00_JPRB, 0.37643E+00_JPRB, 0.35882E+00_JPRB ,&
&  0.34458E+00_JPRB, 0.33373E+00_JPRB, 0.32575E+00_JPRB, 0.31978E+00_JPRB, 0.31514E+00_JPRB, 0.31081E+00_JPRB, 0.30840E+00_JPRB ,&
&  0.30782E+00_JPRB, 0.30916E+00_JPRB, 0.31168E+00_JPRB, 0.31381E+00_JPRB, 0.31643E+00_JPRB, 0.31677E+00_JPRB, 0.31320E+00_JPRB ,&
&  0.30978E+00_JPRB, 0.30862E+00_JPRB, 0.30373E+00_JPRB, 0.30075E+00_JPRB, 0.30001E+00_JPRB, 0.30060E+00_JPRB, 0.30293E+00_JPRB ,&
&  0.30743E+00_JPRB, 0.31347E+00_JPRB, 0.31895E+00_JPRB, 0.32366E+00_JPRB, 0.32777E+00_JPRB, 0.33192E+00_JPRB, 0.33666E+00_JPRB ,&
&  0.34167E+00_JPRB, 0.34652E+00_JPRB, 0.35394E+00_JPRB, 0.36220E+00_JPRB, 0.36934E+00_JPRB, 0.37714E+00_JPRB, 0.38227E+00_JPRB ,&
&  0.38463E+00_JPRB, 0.38586E+00_JPRB, 0.38864E+00_JPRB, 0.39265E+00_JPRB, 0.39556E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.35425E+00_JPRB, 0.35613E+00_JPRB, 0.35824E+00_JPRB, 0.35831E+00_JPRB, 0.35731E+00_JPRB, 0.35685E+00_JPRB, 0.35534E+00_JPRB ,&
&  0.35259E+00_JPRB, 0.35087E+00_JPRB, 0.35082E+00_JPRB, 0.34915E+00_JPRB, 0.34602E+00_JPRB, 0.34217E+00_JPRB, 0.33785E+00_JPRB ,&
&  0.33430E+00_JPRB, 0.33070E+00_JPRB, 0.32491E+00_JPRB, 0.31755E+00_JPRB, 0.31127E+00_JPRB, 0.30717E+00_JPRB, 0.30489E+00_JPRB ,&
&  0.30380E+00_JPRB, 0.30358E+00_JPRB, 0.30307E+00_JPRB, 0.30368E+00_JPRB, 0.30593E+00_JPRB, 0.31103E+00_JPRB, 0.31849E+00_JPRB ,&
&  0.32577E+00_JPRB, 0.33423E+00_JPRB, 0.34303E+00_JPRB, 0.35309E+00_JPRB, 0.36591E+00_JPRB, 0.38412E+00_JPRB, 0.40513E+00_JPRB ,&
&  0.42746E+00_JPRB, 0.43554E+00_JPRB, 0.43788E+00_JPRB, 0.44641E+00_JPRB, 0.46208E+00_JPRB, 0.47480E+00_JPRB, 0.48353E+00_JPRB ,&
&  0.48848E+00_JPRB, 0.48859E+00_JPRB, 0.48615E+00_JPRB, 0.47886E+00_JPRB, 0.46734E+00_JPRB, 0.45238E+00_JPRB, 0.43605E+00_JPRB ,&
&  0.42010E+00_JPRB, 0.40369E+00_JPRB, 0.38631E+00_JPRB, 0.36926E+00_JPRB, 0.35369E+00_JPRB, 0.33940E+00_JPRB, 0.32607E+00_JPRB ,&
&  0.31455E+00_JPRB, 0.30559E+00_JPRB, 0.29858E+00_JPRB, 0.29354E+00_JPRB, 0.28938E+00_JPRB, 0.28613E+00_JPRB, 0.28480E+00_JPRB ,&
&  0.28478E+00_JPRB, 0.28589E+00_JPRB, 0.28642E+00_JPRB, 0.28785E+00_JPRB, 0.29043E+00_JPRB, 0.29123E+00_JPRB, 0.28945E+00_JPRB ,&
&  0.28495E+00_JPRB, 0.28058E+00_JPRB, 0.27621E+00_JPRB, 0.27538E+00_JPRB, 0.27528E+00_JPRB, 0.27677E+00_JPRB, 0.27909E+00_JPRB ,&
&  0.28342E+00_JPRB, 0.28939E+00_JPRB, 0.29479E+00_JPRB, 0.29881E+00_JPRB, 0.30319E+00_JPRB, 0.30706E+00_JPRB, 0.31079E+00_JPRB ,&
&  0.31480E+00_JPRB, 0.31903E+00_JPRB, 0.32464E+00_JPRB, 0.33028E+00_JPRB, 0.33545E+00_JPRB, 0.33962E+00_JPRB, 0.34282E+00_JPRB ,&
&  0.34511E+00_JPRB, 0.34606E+00_JPRB, 0.34681E+00_JPRB, 0.34889E+00_JPRB, 0.35189E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.31801E+00_JPRB, 0.32035E+00_JPRB, 0.32324E+00_JPRB, 0.32539E+00_JPRB, 0.32601E+00_JPRB, 0.32532E+00_JPRB, 0.32437E+00_JPRB ,&
&  0.32389E+00_JPRB, 0.32327E+00_JPRB, 0.32140E+00_JPRB, 0.31855E+00_JPRB, 0.31571E+00_JPRB, 0.31276E+00_JPRB, 0.31055E+00_JPRB ,&
&  0.30843E+00_JPRB, 0.30609E+00_JPRB, 0.30195E+00_JPRB, 0.29585E+00_JPRB, 0.28959E+00_JPRB, 0.28605E+00_JPRB, 0.28451E+00_JPRB ,&
&  0.28464E+00_JPRB, 0.28542E+00_JPRB, 0.28807E+00_JPRB, 0.28989E+00_JPRB, 0.29237E+00_JPRB, 0.29655E+00_JPRB, 0.30240E+00_JPRB ,&
&  0.30774E+00_JPRB, 0.31294E+00_JPRB, 0.31866E+00_JPRB, 0.32394E+00_JPRB, 0.32931E+00_JPRB, 0.33683E+00_JPRB, 0.34649E+00_JPRB ,&
&  0.36024E+00_JPRB, 0.37120E+00_JPRB, 0.37157E+00_JPRB, 0.37178E+00_JPRB, 0.37710E+00_JPRB, 0.38501E+00_JPRB, 0.39197E+00_JPRB ,&
&  0.39763E+00_JPRB, 0.39861E+00_JPRB, 0.39736E+00_JPRB, 0.39340E+00_JPRB, 0.38810E+00_JPRB, 0.38141E+00_JPRB, 0.37224E+00_JPRB ,&
&  0.36119E+00_JPRB, 0.34905E+00_JPRB, 0.33709E+00_JPRB, 0.32527E+00_JPRB, 0.31364E+00_JPRB, 0.30308E+00_JPRB, 0.29393E+00_JPRB ,&
&  0.28585E+00_JPRB, 0.27957E+00_JPRB, 0.27375E+00_JPRB, 0.26995E+00_JPRB, 0.26739E+00_JPRB, 0.26474E+00_JPRB, 0.26403E+00_JPRB ,&
&  0.26283E+00_JPRB, 0.26195E+00_JPRB, 0.26160E+00_JPRB, 0.26295E+00_JPRB, 0.26585E+00_JPRB, 0.26756E+00_JPRB, 0.26605E+00_JPRB ,&
&  0.26258E+00_JPRB, 0.26043E+00_JPRB, 0.25908E+00_JPRB, 0.25814E+00_JPRB, 0.25586E+00_JPRB, 0.25583E+00_JPRB, 0.25825E+00_JPRB ,&
&  0.26314E+00_JPRB, 0.26840E+00_JPRB, 0.27276E+00_JPRB, 0.27546E+00_JPRB, 0.27830E+00_JPRB, 0.28079E+00_JPRB, 0.28297E+00_JPRB ,&
&  0.28513E+00_JPRB, 0.28663E+00_JPRB, 0.28772E+00_JPRB, 0.28919E+00_JPRB, 0.29166E+00_JPRB, 0.29408E+00_JPRB, 0.29652E+00_JPRB ,&
&  0.29937E+00_JPRB, 0.30379E+00_JPRB, 0.30827E+00_JPRB, 0.31166E+00_JPRB, 0.31560E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.29323E+00_JPRB, 0.29586E+00_JPRB, 0.29819E+00_JPRB, 0.29795E+00_JPRB, 0.29576E+00_JPRB, 0.29453E+00_JPRB, 0.29455E+00_JPRB ,&
&  0.29480E+00_JPRB, 0.29441E+00_JPRB, 0.29263E+00_JPRB, 0.29224E+00_JPRB, 0.29217E+00_JPRB, 0.29127E+00_JPRB, 0.29067E+00_JPRB ,&
&  0.28928E+00_JPRB, 0.28601E+00_JPRB, 0.28133E+00_JPRB, 0.27566E+00_JPRB, 0.27160E+00_JPRB, 0.26972E+00_JPRB, 0.26984E+00_JPRB ,&
&  0.27110E+00_JPRB, 0.27203E+00_JPRB, 0.27473E+00_JPRB, 0.27683E+00_JPRB, 0.27930E+00_JPRB, 0.28274E+00_JPRB, 0.28645E+00_JPRB ,&
&  0.28903E+00_JPRB, 0.29178E+00_JPRB, 0.29476E+00_JPRB, 0.29775E+00_JPRB, 0.30171E+00_JPRB, 0.30786E+00_JPRB, 0.31359E+00_JPRB ,&
&  0.32253E+00_JPRB, 0.32977E+00_JPRB, 0.32673E+00_JPRB, 0.32210E+00_JPRB, 0.32178E+00_JPRB, 0.32492E+00_JPRB, 0.32928E+00_JPRB ,&
&  0.33410E+00_JPRB, 0.33873E+00_JPRB, 0.34236E+00_JPRB, 0.34328E+00_JPRB, 0.34104E+00_JPRB, 0.33606E+00_JPRB, 0.32863E+00_JPRB ,&
&  0.31994E+00_JPRB, 0.31076E+00_JPRB, 0.30144E+00_JPRB, 0.29180E+00_JPRB, 0.28338E+00_JPRB, 0.27567E+00_JPRB, 0.26894E+00_JPRB ,&
&  0.26279E+00_JPRB, 0.25710E+00_JPRB, 0.25280E+00_JPRB, 0.24993E+00_JPRB, 0.24852E+00_JPRB, 0.24690E+00_JPRB, 0.24668E+00_JPRB ,&
&  0.24577E+00_JPRB, 0.24524E+00_JPRB, 0.24492E+00_JPRB, 0.24537E+00_JPRB, 0.24665E+00_JPRB, 0.24730E+00_JPRB, 0.24639E+00_JPRB ,&
&  0.24594E+00_JPRB, 0.24734E+00_JPRB, 0.24713E+00_JPRB, 0.24455E+00_JPRB, 0.24136E+00_JPRB, 0.24072E+00_JPRB, 0.24244E+00_JPRB ,&
&  0.24564E+00_JPRB, 0.24898E+00_JPRB, 0.25186E+00_JPRB, 0.25430E+00_JPRB, 0.25630E+00_JPRB, 0.25881E+00_JPRB, 0.26113E+00_JPRB ,&
&  0.26313E+00_JPRB, 0.26428E+00_JPRB, 0.26605E+00_JPRB, 0.26793E+00_JPRB, 0.26982E+00_JPRB, 0.27113E+00_JPRB, 0.27097E+00_JPRB ,&
&  0.27052E+00_JPRB, 0.27238E+00_JPRB, 0.27767E+00_JPRB, 0.28527E+00_JPRB, 0.29015E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.27188E+00_JPRB, 0.27410E+00_JPRB, 0.27509E+00_JPRB, 0.27449E+00_JPRB, 0.27344E+00_JPRB, 0.27383E+00_JPRB, 0.27423E+00_JPRB ,&
&  0.27490E+00_JPRB, 0.27481E+00_JPRB, 0.27310E+00_JPRB, 0.27159E+00_JPRB, 0.27310E+00_JPRB, 0.27276E+00_JPRB, 0.27232E+00_JPRB ,&
&  0.27066E+00_JPRB, 0.26849E+00_JPRB, 0.26550E+00_JPRB, 0.26219E+00_JPRB, 0.25840E+00_JPRB, 0.25652E+00_JPRB, 0.25748E+00_JPRB ,&
&  0.25954E+00_JPRB, 0.26036E+00_JPRB, 0.26113E+00_JPRB, 0.26195E+00_JPRB, 0.26380E+00_JPRB, 0.26624E+00_JPRB, 0.26803E+00_JPRB ,&
&  0.26935E+00_JPRB, 0.27231E+00_JPRB, 0.27416E+00_JPRB, 0.27367E+00_JPRB, 0.27790E+00_JPRB, 0.28264E+00_JPRB, 0.28769E+00_JPRB ,&
&  0.29322E+00_JPRB, 0.29674E+00_JPRB, 0.29680E+00_JPRB, 0.29518E+00_JPRB, 0.29398E+00_JPRB, 0.29522E+00_JPRB, 0.29628E+00_JPRB ,&
&  0.29785E+00_JPRB, 0.30212E+00_JPRB, 0.30722E+00_JPRB, 0.31009E+00_JPRB, 0.31002E+00_JPRB, 0.30629E+00_JPRB, 0.29903E+00_JPRB ,&
&  0.28875E+00_JPRB, 0.27740E+00_JPRB, 0.26775E+00_JPRB, 0.25991E+00_JPRB, 0.25396E+00_JPRB, 0.24943E+00_JPRB, 0.24578E+00_JPRB ,&
&  0.24181E+00_JPRB, 0.23888E+00_JPRB, 0.23854E+00_JPRB, 0.23895E+00_JPRB, 0.23853E+00_JPRB, 0.23606E+00_JPRB, 0.23435E+00_JPRB ,&
&  0.23280E+00_JPRB, 0.23324E+00_JPRB, 0.23437E+00_JPRB, 0.23481E+00_JPRB, 0.23499E+00_JPRB, 0.23486E+00_JPRB, 0.23466E+00_JPRB ,&
&  0.23558E+00_JPRB, 0.23524E+00_JPRB, 0.23389E+00_JPRB, 0.23173E+00_JPRB, 0.22982E+00_JPRB, 0.22953E+00_JPRB, 0.23014E+00_JPRB ,&
&  0.23119E+00_JPRB, 0.23301E+00_JPRB, 0.23528E+00_JPRB, 0.23755E+00_JPRB, 0.23931E+00_JPRB, 0.24017E+00_JPRB, 0.24066E+00_JPRB ,&
&  0.24108E+00_JPRB, 0.24131E+00_JPRB, 0.24294E+00_JPRB, 0.24695E+00_JPRB, 0.25072E+00_JPRB, 0.25343E+00_JPRB, 0.25468E+00_JPRB ,&
&  0.25398E+00_JPRB, 0.25452E+00_JPRB, 0.25776E+00_JPRB, 0.26385E+00_JPRB, 0.26833E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.25530E+00_JPRB, 0.25732E+00_JPRB, 0.25777E+00_JPRB, 0.25708E+00_JPRB, 0.25628E+00_JPRB, 0.25701E+00_JPRB, 0.25818E+00_JPRB ,&
&  0.25878E+00_JPRB, 0.25884E+00_JPRB, 0.25767E+00_JPRB, 0.25540E+00_JPRB, 0.25362E+00_JPRB, 0.25369E+00_JPRB, 0.25218E+00_JPRB ,&
&  0.25095E+00_JPRB, 0.25153E+00_JPRB, 0.25068E+00_JPRB, 0.24969E+00_JPRB, 0.24803E+00_JPRB, 0.24742E+00_JPRB, 0.24863E+00_JPRB ,&
&  0.25096E+00_JPRB, 0.25151E+00_JPRB, 0.25203E+00_JPRB, 0.25273E+00_JPRB, 0.25413E+00_JPRB, 0.25593E+00_JPRB, 0.25689E+00_JPRB ,&
&  0.25733E+00_JPRB, 0.25581E+00_JPRB, 0.25448E+00_JPRB, 0.25692E+00_JPRB, 0.26080E+00_JPRB, 0.26611E+00_JPRB, 0.27005E+00_JPRB ,&
&  0.27339E+00_JPRB, 0.27546E+00_JPRB, 0.27613E+00_JPRB, 0.27575E+00_JPRB, 0.27588E+00_JPRB, 0.27817E+00_JPRB, 0.28022E+00_JPRB ,&
&  0.28126E+00_JPRB, 0.28185E+00_JPRB, 0.28137E+00_JPRB, 0.28107E+00_JPRB, 0.27850E+00_JPRB, 0.27376E+00_JPRB, 0.26535E+00_JPRB ,&
&  0.25610E+00_JPRB, 0.24832E+00_JPRB, 0.24234E+00_JPRB, 0.23767E+00_JPRB, 0.23457E+00_JPRB, 0.23433E+00_JPRB, 0.23410E+00_JPRB ,&
&  0.23338E+00_JPRB, 0.23270E+00_JPRB, 0.23393E+00_JPRB, 0.23412E+00_JPRB, 0.22850E+00_JPRB, 0.22382E+00_JPRB, 0.21870E+00_JPRB ,&
&  0.21726E+00_JPRB, 0.21739E+00_JPRB, 0.21763E+00_JPRB, 0.22076E+00_JPRB, 0.22523E+00_JPRB, 0.22625E+00_JPRB, 0.22475E+00_JPRB ,&
&  0.21821E+00_JPRB, 0.21354E+00_JPRB, 0.21044E+00_JPRB, 0.20762E+00_JPRB, 0.20606E+00_JPRB, 0.20643E+00_JPRB, 0.20733E+00_JPRB ,&
&  0.20983E+00_JPRB, 0.21141E+00_JPRB, 0.21406E+00_JPRB, 0.21557E+00_JPRB, 0.21548E+00_JPRB, 0.21554E+00_JPRB, 0.21487E+00_JPRB ,&
&  0.21682E+00_JPRB, 0.22205E+00_JPRB, 0.22956E+00_JPRB, 0.23193E+00_JPRB, 0.23682E+00_JPRB, 0.24017E+00_JPRB, 0.24152E+00_JPRB ,&
&  0.24220E+00_JPRB, 0.24231E+00_JPRB, 0.24212E+00_JPRB, 0.24605E+00_JPRB, 0.25177E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.24181E+00_JPRB, 0.24325E+00_JPRB, 0.24368E+00_JPRB, 0.24342E+00_JPRB, 0.24300E+00_JPRB, 0.24321E+00_JPRB, 0.24353E+00_JPRB ,&
&  0.24331E+00_JPRB, 0.24241E+00_JPRB, 0.24108E+00_JPRB, 0.23879E+00_JPRB, 0.23596E+00_JPRB, 0.23429E+00_JPRB, 0.23539E+00_JPRB ,&
&  0.23699E+00_JPRB, 0.23796E+00_JPRB, 0.23908E+00_JPRB, 0.23977E+00_JPRB, 0.24016E+00_JPRB, 0.24102E+00_JPRB, 0.24251E+00_JPRB ,&
&  0.24377E+00_JPRB, 0.24437E+00_JPRB, 0.24564E+00_JPRB, 0.24661E+00_JPRB, 0.24865E+00_JPRB, 0.25044E+00_JPRB, 0.25068E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.23901E+00_JPRB, 0.23418E+00_JPRB, 0.23699E+00_JPRB, 0.24454E+00_JPRB, 0.25061E+00_JPRB, 0.25464E+00_JPRB ,&
&  0.25707E+00_JPRB, 0.25794E+00_JPRB, 0.25799E+00_JPRB, 0.25927E+00_JPRB, 0.26156E+00_JPRB, 0.26407E+00_JPRB, 0.26404E+00_JPRB ,&
&  0.26231E+00_JPRB, 0.26092E+00_JPRB, 0.25885E+00_JPRB, 0.25486E+00_JPRB, 0.24901E+00_JPRB, 0.24301E+00_JPRB, 0.23756E+00_JPRB ,&
&  0.23246E+00_JPRB, 0.22753E+00_JPRB, 0.22360E+00_JPRB, 0.22111E+00_JPRB, 0.22047E+00_JPRB, 0.22202E+00_JPRB, 0.22139E+00_JPRB ,&
&  0.22195E+00_JPRB, 0.21990E+00_JPRB, 0.21762E+00_JPRB, 0.21463E+00_JPRB, 0.21196E+00_JPRB, 0.20707E+00_JPRB, 0.20086E+00_JPRB ,&
&  0.19783E+00_JPRB, 0.20086E+00_JPRB, 0.20871E+00_JPRB, 0.21433E+00_JPRB, 0.21061E+00_JPRB, 0.20489E+00_JPRB, 0.20209E+00_JPRB ,&
&  0.19818E+00_JPRB, 0.19364E+00_JPRB, 0.19060E+00_JPRB, 0.18983E+00_JPRB, 0.18997E+00_JPRB, 0.19013E+00_JPRB, 0.19121E+00_JPRB ,&
&  0.19304E+00_JPRB, 0.19463E+00_JPRB, 0.19548E+00_JPRB, 0.19659E+00_JPRB, 0.19705E+00_JPRB, 0.19713E+00_JPRB, 0.19722E+00_JPRB ,&
&  0.19921E+00_JPRB, 0.20329E+00_JPRB, 0.20856E+00_JPRB, 0.21433E+00_JPRB, 0.21860E+00_JPRB, 0.22248E+00_JPRB, 0.22549E+00_JPRB ,&
&  0.22817E+00_JPRB, 0.22863E+00_JPRB, 0.23166E+00_JPRB, 0.23601E+00_JPRB, 0.23931E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.22682E+00_JPRB, 0.23065E+00_JPRB, 0.23194E+00_JPRB, 0.23174E+00_JPRB, 0.23046E+00_JPRB, 0.22926E+00_JPRB, 0.22880E+00_JPRB ,&
&  0.22840E+00_JPRB, 0.22710E+00_JPRB, 0.22616E+00_JPRB, 0.22553E+00_JPRB, 0.22485E+00_JPRB, 0.22398E+00_JPRB, 0.22441E+00_JPRB ,&
&  0.22610E+00_JPRB, 0.22844E+00_JPRB, 0.23041E+00_JPRB, 0.23175E+00_JPRB, 0.23180E+00_JPRB, 0.23243E+00_JPRB, 0.23339E+00_JPRB ,&
&  0.23343E+00_JPRB, 0.23335E+00_JPRB, 0.23655E+00_JPRB, 0.23656E+00_JPRB, 0.23739E+00_JPRB, 0.23717E+00_JPRB, 0.23712E+00_JPRB ,&
&  0.23158E+00_JPRB, 0.22597E+00_JPRB, 0.22393E+00_JPRB, 0.22649E+00_JPRB, 0.23461E+00_JPRB, 0.23606E+00_JPRB, 0.24005E+00_JPRB ,&
&  0.24217E+00_JPRB, 0.24305E+00_JPRB, 0.24481E+00_JPRB, 0.24623E+00_JPRB, 0.24789E+00_JPRB, 0.24737E+00_JPRB, 0.24532E+00_JPRB ,&
&  0.24230E+00_JPRB, 0.24145E+00_JPRB, 0.23829E+00_JPRB, 0.22334E+00_JPRB, 0.21456E+00_JPRB, 0.20954E+00_JPRB, 0.20415E+00_JPRB ,&
&  0.19915E+00_JPRB, 0.19673E+00_JPRB, 0.19636E+00_JPRB, 0.19474E+00_JPRB, 0.19571E+00_JPRB, 0.19824E+00_JPRB, 0.19922E+00_JPRB ,&
&  0.19882E+00_JPRB, 0.19928E+00_JPRB, 0.19894E+00_JPRB, 0.19769E+00_JPRB, 0.19531E+00_JPRB, 0.19377E+00_JPRB, 0.19287E+00_JPRB ,&
&  0.19343E+00_JPRB, 0.19297E+00_JPRB, 0.19730E+00_JPRB, 0.19827E+00_JPRB, 0.19194E+00_JPRB, 0.19144E+00_JPRB, 0.19033E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18132E+00_JPRB, 0.17948E+00_JPRB, 0.17874E+00_JPRB, 0.17989E+00_JPRB, 0.18147E+00_JPRB, 0.18212E+00_JPRB ,&
&  0.18222E+00_JPRB, 0.18343E+00_JPRB, 0.18400E+00_JPRB, 0.18464E+00_JPRB, 0.18505E+00_JPRB, 0.18604E+00_JPRB, 0.18750E+00_JPRB ,&
&  0.18933E+00_JPRB, 0.19270E+00_JPRB, 0.19596E+00_JPRB, 0.19957E+00_JPRB, 0.20162E+00_JPRB, 0.20314E+00_JPRB, 0.20691E+00_JPRB ,&
&  0.20861E+00_JPRB, 0.21625E+00_JPRB, 0.22712E+00_JPRB, 0.22823E+00_JPRB, 0.22179E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.21860E+00_JPRB, 0.22029E+00_JPRB, 0.22150E+00_JPRB, 0.22040E+00_JPRB, 0.22053E+00_JPRB, 0.22028E+00_JPRB, 0.22023E+00_JPRB ,&
&  0.22002E+00_JPRB, 0.21927E+00_JPRB, 0.21951E+00_JPRB, 0.21728E+00_JPRB, 0.21599E+00_JPRB, 0.21282E+00_JPRB, 0.21222E+00_JPRB ,&
&  0.21084E+00_JPRB, 0.21105E+00_JPRB, 0.21178E+00_JPRB, 0.21359E+00_JPRB, 0.21422E+00_JPRB, 0.21584E+00_JPRB, 0.21485E+00_JPRB ,&
&  0.21522E+00_JPRB, 0.21557E+00_JPRB, 0.21421E+00_JPRB, 0.21386E+00_JPRB, 0.21413E+00_JPRB, 0.21427E+00_JPRB, 0.21450E+00_JPRB ,&
&  0.21685E+00_JPRB, 0.21918E+00_JPRB, 0.22145E+00_JPRB, 0.22261E+00_JPRB, 0.22753E+00_JPRB, 0.22898E+00_JPRB, 0.23140E+00_JPRB ,&
&  0.23242E+00_JPRB, 0.23365E+00_JPRB, 0.23484E+00_JPRB, 0.23411E+00_JPRB, 0.23283E+00_JPRB, 0.22858E+00_JPRB, 0.22719E+00_JPRB ,&
&  0.21717E+00_JPRB, 0.21024E+00_JPRB, 0.20343E+00_JPRB, 0.19714E+00_JPRB, 0.19233E+00_JPRB, 0.18905E+00_JPRB, 0.18641E+00_JPRB ,&
&  0.18427E+00_JPRB, 0.18211E+00_JPRB, 0.17971E+00_JPRB, 0.17990E+00_JPRB, 0.18077E+00_JPRB, 0.18213E+00_JPRB, 0.18355E+00_JPRB ,&
&  0.18522E+00_JPRB, 0.18617E+00_JPRB, 0.18654E+00_JPRB, 0.18730E+00_JPRB, 0.18733E+00_JPRB, 0.18756E+00_JPRB, 0.18768E+00_JPRB ,&
&  0.18906E+00_JPRB, 0.18938E+00_JPRB, 0.18932E+00_JPRB, 0.19117E+00_JPRB, 0.18912E+00_JPRB, 0.18611E+00_JPRB, 0.18300E+00_JPRB ,&
&  0.17966E+00_JPRB, 0.17761E+00_JPRB, 0.17580E+00_JPRB, 0.17509E+00_JPRB, 0.17555E+00_JPRB, 0.17640E+00_JPRB, 0.17745E+00_JPRB ,&
&  0.17756E+00_JPRB, 0.17823E+00_JPRB, 0.17875E+00_JPRB, 0.17977E+00_JPRB, 0.18085E+00_JPRB, 0.18211E+00_JPRB, 0.18351E+00_JPRB ,&
&  0.18562E+00_JPRB, 0.18808E+00_JPRB, 0.19048E+00_JPRB, 0.19290E+00_JPRB, 0.19681E+00_JPRB, 0.20124E+00_JPRB, 0.20819E+00_JPRB ,&
&  0.21357E+00_JPRB, 0.21907E+00_JPRB, 0.21603E+00_JPRB, 0.21994E+00_JPRB, 0.22032E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.21302E+00_JPRB, 0.21372E+00_JPRB, 0.21147E+00_JPRB, 0.21163E+00_JPRB, 0.21225E+00_JPRB, 0.21253E+00_JPRB, 0.21298E+00_JPRB ,&
&  0.21033E+00_JPRB, 0.20966E+00_JPRB, 0.20735E+00_JPRB, 0.20557E+00_JPRB, 0.20318E+00_JPRB, 0.20082E+00_JPRB, 0.19910E+00_JPRB ,&
&  0.19719E+00_JPRB, 0.19670E+00_JPRB, 0.19588E+00_JPRB, 0.19712E+00_JPRB, 0.19731E+00_JPRB, 0.19935E+00_JPRB, 0.20094E+00_JPRB ,&
&  0.20169E+00_JPRB, 0.20287E+00_JPRB, 0.20125E+00_JPRB, 0.20064E+00_JPRB, 0.20232E+00_JPRB, 0.20372E+00_JPRB, 0.20634E+00_JPRB ,&
&  0.20900E+00_JPRB, 0.21236E+00_JPRB, 0.21795E+00_JPRB, 0.21990E+00_JPRB, 0.22248E+00_JPRB, 0.22401E+00_JPRB, 0.22382E+00_JPRB ,&
&  0.22346E+00_JPRB, 0.22296E+00_JPRB, 0.22377E+00_JPRB, 0.22356E+00_JPRB, 0.21641E+00_JPRB, 0.21109E+00_JPRB, 0.20536E+00_JPRB ,&
&  0.20024E+00_JPRB, 0.19523E+00_JPRB, 0.19068E+00_JPRB, 0.18763E+00_JPRB, 0.18417E+00_JPRB, 0.18180E+00_JPRB, 0.17992E+00_JPRB ,&
&  0.17808E+00_JPRB, 0.17709E+00_JPRB, 0.17569E+00_JPRB, 0.17525E+00_JPRB, 0.17563E+00_JPRB, 0.17654E+00_JPRB, 0.17789E+00_JPRB ,&
&  0.17948E+00_JPRB, 0.18055E+00_JPRB, 0.18188E+00_JPRB, 0.18308E+00_JPRB, 0.18393E+00_JPRB, 0.18437E+00_JPRB, 0.18392E+00_JPRB ,&
&  0.18407E+00_JPRB, 0.18435E+00_JPRB, 0.18384E+00_JPRB, 0.18388E+00_JPRB, 0.18261E+00_JPRB, 0.18113E+00_JPRB, 0.17963E+00_JPRB ,&
&  0.17750E+00_JPRB, 0.17588E+00_JPRB, 0.17532E+00_JPRB, 0.17501E+00_JPRB, 0.17584E+00_JPRB, 0.17708E+00_JPRB, 0.17786E+00_JPRB ,&
&  0.17819E+00_JPRB, 0.17801E+00_JPRB, 0.17796E+00_JPRB, 0.17930E+00_JPRB, 0.18071E+00_JPRB, 0.18251E+00_JPRB, 0.18439E+00_JPRB ,&
&  0.18496E+00_JPRB, 0.18648E+00_JPRB, 0.18777E+00_JPRB, 0.18944E+00_JPRB, 0.19517E+00_JPRB, 0.19939E+00_JPRB, 0.20748E+00_JPRB ,&
&  0.21043E+00_JPRB, 0.20886E+00_JPRB, 0.20634E+00_JPRB, 0.21174E+00_JPRB, 0.21266E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.21137E+00_JPRB, 0.21395E+00_JPRB, 0.21232E+00_JPRB, 0.21195E+00_JPRB, 0.21107E+00_JPRB, 0.21069E+00_JPRB, 0.20925E+00_JPRB ,&
&  0.20900E+00_JPRB, 0.20730E+00_JPRB, 0.20707E+00_JPRB, 0.20541E+00_JPRB, 0.20373E+00_JPRB, 0.20130E+00_JPRB, 0.19889E+00_JPRB ,&
&  0.19653E+00_JPRB, 0.19451E+00_JPRB, 0.19319E+00_JPRB, 0.19277E+00_JPRB, 0.19172E+00_JPRB, 0.19204E+00_JPRB, 0.19475E+00_JPRB ,&
&  0.19538E+00_JPRB, 0.19666E+00_JPRB, 0.19664E+00_JPRB, 0.19788E+00_JPRB, 0.19951E+00_JPRB, 0.19981E+00_JPRB, 0.20202E+00_JPRB ,&
&  0.20259E+00_JPRB, 0.20422E+00_JPRB, 0.21113E+00_JPRB, 0.21472E+00_JPRB, 0.21656E+00_JPRB, 0.21957E+00_JPRB, 0.22022E+00_JPRB ,&
&  0.21846E+00_JPRB, 0.21296E+00_JPRB, 0.21597E+00_JPRB, 0.21282E+00_JPRB, 0.20598E+00_JPRB, 0.20332E+00_JPRB, 0.19783E+00_JPRB ,&
&  0.19508E+00_JPRB, 0.19136E+00_JPRB, 0.18861E+00_JPRB, 0.18650E+00_JPRB, 0.18312E+00_JPRB, 0.18103E+00_JPRB, 0.17863E+00_JPRB ,&
&  0.17646E+00_JPRB, 0.17562E+00_JPRB, 0.17415E+00_JPRB, 0.17409E+00_JPRB, 0.17436E+00_JPRB, 0.17492E+00_JPRB, 0.17584E+00_JPRB ,&
&  0.17699E+00_JPRB, 0.17802E+00_JPRB, 0.17897E+00_JPRB, 0.17975E+00_JPRB, 0.18041E+00_JPRB, 0.18077E+00_JPRB, 0.18056E+00_JPRB ,&
&  0.18060E+00_JPRB, 0.18043E+00_JPRB, 0.17986E+00_JPRB, 0.17964E+00_JPRB, 0.17860E+00_JPRB, 0.17818E+00_JPRB, 0.17773E+00_JPRB ,&
&  0.17754E+00_JPRB, 0.17757E+00_JPRB, 0.17746E+00_JPRB, 0.17768E+00_JPRB, 0.17796E+00_JPRB, 0.17872E+00_JPRB, 0.17931E+00_JPRB ,&
&  0.17977E+00_JPRB, 0.18076E+00_JPRB, 0.18135E+00_JPRB, 0.18218E+00_JPRB, 0.18336E+00_JPRB, 0.18385E+00_JPRB, 0.18523E+00_JPRB ,&
&  0.18640E+00_JPRB, 0.18779E+00_JPRB, 0.19036E+00_JPRB, 0.19290E+00_JPRB, 0.19658E+00_JPRB, 0.20097E+00_JPRB, 0.20225E+00_JPRB ,&
&  0.20313E+00_JPRB, 0.20302E+00_JPRB, 0.20055E+00_JPRB, 0.20612E+00_JPRB, 0.20876E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.20428E+00_JPRB, 0.20627E+00_JPRB, 0.20756E+00_JPRB, 0.20844E+00_JPRB, 0.20993E+00_JPRB, 0.21062E+00_JPRB, 0.21123E+00_JPRB ,&
&  0.20925E+00_JPRB, 0.20875E+00_JPRB, 0.20775E+00_JPRB, 0.20647E+00_JPRB, 0.20562E+00_JPRB, 0.20302E+00_JPRB, 0.20053E+00_JPRB ,&
&  0.19783E+00_JPRB, 0.19546E+00_JPRB, 0.19418E+00_JPRB, 0.19294E+00_JPRB, 0.19164E+00_JPRB, 0.19056E+00_JPRB, 0.19311E+00_JPRB ,&
&  0.19471E+00_JPRB, 0.19565E+00_JPRB, 0.19712E+00_JPRB, 0.19667E+00_JPRB, 0.19802E+00_JPRB, 0.20019E+00_JPRB, 0.20313E+00_JPRB ,&
&  0.20473E+00_JPRB, 0.20775E+00_JPRB, 0.20949E+00_JPRB, 0.21037E+00_JPRB, 0.21165E+00_JPRB, 0.21260E+00_JPRB, 0.21233E+00_JPRB ,&
&  0.21091E+00_JPRB, 0.20903E+00_JPRB, 0.20676E+00_JPRB, 0.20378E+00_JPRB, 0.20065E+00_JPRB, 0.19768E+00_JPRB, 0.19471E+00_JPRB ,&
&  0.19262E+00_JPRB, 0.19071E+00_JPRB, 0.18862E+00_JPRB, 0.18672E+00_JPRB, 0.18490E+00_JPRB, 0.18326E+00_JPRB, 0.18159E+00_JPRB ,&
&  0.18030E+00_JPRB, 0.17926E+00_JPRB, 0.17796E+00_JPRB, 0.17733E+00_JPRB, 0.17670E+00_JPRB, 0.17647E+00_JPRB, 0.17637E+00_JPRB ,&
&  0.17689E+00_JPRB, 0.17749E+00_JPRB, 0.17808E+00_JPRB, 0.17839E+00_JPRB, 0.17871E+00_JPRB, 0.17885E+00_JPRB, 0.17914E+00_JPRB ,&
&  0.17941E+00_JPRB, 0.17973E+00_JPRB, 0.17924E+00_JPRB, 0.17881E+00_JPRB, 0.17823E+00_JPRB, 0.17759E+00_JPRB, 0.17739E+00_JPRB ,&
&  0.17772E+00_JPRB, 0.17798E+00_JPRB, 0.17776E+00_JPRB, 0.17806E+00_JPRB, 0.17862E+00_JPRB, 0.17919E+00_JPRB, 0.18066E+00_JPRB ,&
&  0.18145E+00_JPRB, 0.18221E+00_JPRB, 0.18292E+00_JPRB, 0.18349E+00_JPRB, 0.18436E+00_JPRB, 0.18506E+00_JPRB, 0.18594E+00_JPRB ,&
&  0.18700E+00_JPRB, 0.18802E+00_JPRB, 0.18940E+00_JPRB, 0.19119E+00_JPRB, 0.19558E+00_JPRB, 0.19826E+00_JPRB, 0.20043E+00_JPRB ,&
&  0.20110E+00_JPRB, 0.19632E+00_JPRB, 0.20346E+00_JPRB, 0.20207E+00_JPRB, 0.20155E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.19724E+00_JPRB, 0.19901E+00_JPRB, 0.20031E+00_JPRB, 0.20213E+00_JPRB, 0.19657E+00_JPRB, 0.20337E+00_JPRB, 0.20001E+00_JPRB ,&
&  0.19656E+00_JPRB, 0.19168E+00_JPRB, 0.19211E+00_JPRB, 0.19149E+00_JPRB, 0.19048E+00_JPRB, 0.19016E+00_JPRB, 0.19030E+00_JPRB ,&
&  0.19062E+00_JPRB, 0.19063E+00_JPRB, 0.19137E+00_JPRB, 0.19195E+00_JPRB, 0.19241E+00_JPRB, 0.19279E+00_JPRB, 0.19329E+00_JPRB ,&
&  0.19405E+00_JPRB, 0.19524E+00_JPRB, 0.19647E+00_JPRB, 0.19427E+00_JPRB, 0.19401E+00_JPRB, 0.19424E+00_JPRB, 0.19487E+00_JPRB ,&
&  0.19518E+00_JPRB, 0.19634E+00_JPRB, 0.19727E+00_JPRB, 0.19799E+00_JPRB, 0.20009E+00_JPRB, 0.20027E+00_JPRB, 0.20016E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.19886E+00_JPRB, 0.19759E+00_JPRB, 0.19640E+00_JPRB, 0.19517E+00_JPRB, 0.19355E+00_JPRB, 0.19199E+00_JPRB ,&
&  0.19072E+00_JPRB, 0.18969E+00_JPRB, 0.18834E+00_JPRB, 0.18703E+00_JPRB, 0.18584E+00_JPRB, 0.18472E+00_JPRB, 0.18375E+00_JPRB ,&
&  0.18305E+00_JPRB, 0.18246E+00_JPRB, 0.18183E+00_JPRB, 0.18135E+00_JPRB, 0.18083E+00_JPRB, 0.18034E+00_JPRB, 0.17986E+00_JPRB ,&
&  0.18003E+00_JPRB, 0.18014E+00_JPRB, 0.18025E+00_JPRB, 0.18037E+00_JPRB, 0.18085E+00_JPRB, 0.18094E+00_JPRB, 0.18106E+00_JPRB ,&
&  0.18113E+00_JPRB, 0.18101E+00_JPRB, 0.18086E+00_JPRB, 0.18060E+00_JPRB, 0.18031E+00_JPRB, 0.17995E+00_JPRB, 0.17979E+00_JPRB ,&
&  0.17977E+00_JPRB, 0.17990E+00_JPRB, 0.18029E+00_JPRB, 0.18055E+00_JPRB, 0.18081E+00_JPRB, 0.18104E+00_JPRB, 0.18156E+00_JPRB ,&
&  0.18209E+00_JPRB, 0.18260E+00_JPRB, 0.18309E+00_JPRB, 0.18294E+00_JPRB, 0.18320E+00_JPRB, 0.18345E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18382E+00_JPRB, 0.18433E+00_JPRB, 0.18515E+00_JPRB, 0.18631E+00_JPRB, 0.18847E+00_JPRB, 0.19025E+00_JPRB, 0.19190E+00_JPRB ,&
&  0.19365E+00_JPRB, 0.19727E+00_JPRB, 0.19853E+00_JPRB, 0.19880E+00_JPRB, 0.19856E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.18734E+00_JPRB, 0.18827E+00_JPRB, 0.18889E+00_JPRB, 0.18937E+00_JPRB, 0.18911E+00_JPRB, 0.18913E+00_JPRB, 0.18914E+00_JPRB ,&
&  0.18905E+00_JPRB, 0.18895E+00_JPRB, 0.18924E+00_JPRB, 0.18950E+00_JPRB, 0.18980E+00_JPRB, 0.19013E+00_JPRB, 0.19055E+00_JPRB ,&
&  0.19071E+00_JPRB, 0.19109E+00_JPRB, 0.18966E+00_JPRB, 0.18849E+00_JPRB, 0.18771E+00_JPRB, 0.18723E+00_JPRB, 0.18696E+00_JPRB ,&
&  0.18677E+00_JPRB, 0.18666E+00_JPRB, 0.18652E+00_JPRB, 0.18759E+00_JPRB, 0.18829E+00_JPRB, 0.18889E+00_JPRB, 0.18936E+00_JPRB ,&
&  0.18954E+00_JPRB, 0.18982E+00_JPRB, 0.19007E+00_JPRB, 0.19032E+00_JPRB, 0.19062E+00_JPRB, 0.19062E+00_JPRB, 0.19052E+00_JPRB ,&
&  0.19033E+00_JPRB, 0.18999E+00_JPRB, 0.18960E+00_JPRB, 0.18923E+00_JPRB, 0.18892E+00_JPRB, 0.18855E+00_JPRB, 0.18815E+00_JPRB ,&
&  0.18776E+00_JPRB, 0.18731E+00_JPRB, 0.18689E+00_JPRB, 0.18648E+00_JPRB, 0.18608E+00_JPRB, 0.18574E+00_JPRB, 0.18538E+00_JPRB ,&
&  0.18511E+00_JPRB, 0.18487E+00_JPRB, 0.18462E+00_JPRB, 0.18444E+00_JPRB, 0.18414E+00_JPRB, 0.18403E+00_JPRB, 0.18377E+00_JPRB ,&
&  0.18367E+00_JPRB, 0.18356E+00_JPRB, 0.18349E+00_JPRB, 0.18340E+00_JPRB, 0.18339E+00_JPRB, 0.18336E+00_JPRB, 0.18335E+00_JPRB ,&
&  0.18332E+00_JPRB, 0.18347E+00_JPRB, 0.18334E+00_JPRB, 0.18316E+00_JPRB, 0.18297E+00_JPRB, 0.18278E+00_JPRB, 0.18260E+00_JPRB ,&
&  0.18244E+00_JPRB, 0.18227E+00_JPRB, 0.18217E+00_JPRB, 0.18220E+00_JPRB, 0.18225E+00_JPRB, 0.18233E+00_JPRB, 0.18243E+00_JPRB ,&
&  0.18252E+00_JPRB, 0.18264E+00_JPRB, 0.18275E+00_JPRB, 0.18269E+00_JPRB, 0.18271E+00_JPRB, 0.18273E+00_JPRB, 0.18275E+00_JPRB ,&
&  0.18276E+00_JPRB, 0.18282E+00_JPRB, 0.18291E+00_JPRB, 0.18305E+00_JPRB, 0.18336E+00_JPRB, 0.18348E+00_JPRB, 0.18365E+00_JPRB ,&
&  0.18389E+00_JPRB, 0.18434E+00_JPRB, 0.18473E+00_JPRB, 0.18515E+00_JPRB, 0.18563E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB, 0.18451E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB, 0.88310E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.88458E+00_JPRB, 0.88337E+00_JPRB, 0.88229E+00_JPRB, 0.88125E+00_JPRB, 0.88024E+00_JPRB, 0.87948E+00_JPRB, 0.87887E+00_JPRB ,&
&  0.87822E+00_JPRB, 0.87759E+00_JPRB, 0.87687E+00_JPRB, 0.87627E+00_JPRB, 0.87581E+00_JPRB, 0.87527E+00_JPRB, 0.87480E+00_JPRB ,&
&  0.87415E+00_JPRB, 0.87354E+00_JPRB, 0.87309E+00_JPRB, 0.87217E+00_JPRB, 0.87137E+00_JPRB, 0.87061E+00_JPRB, 0.86921E+00_JPRB ,&
&  0.86733E+00_JPRB, 0.86548E+00_JPRB, 0.86339E+00_JPRB, 0.86153E+00_JPRB, 0.86004E+00_JPRB, 0.85805E+00_JPRB, 0.85576E+00_JPRB ,&
&  0.85372E+00_JPRB, 0.85224E+00_JPRB, 0.85114E+00_JPRB, 0.85032E+00_JPRB, 0.84974E+00_JPRB, 0.84934E+00_JPRB, 0.84889E+00_JPRB ,&
&  0.84861E+00_JPRB, 0.84881E+00_JPRB, 0.84951E+00_JPRB, 0.85058E+00_JPRB, 0.85195E+00_JPRB, 0.85368E+00_JPRB, 0.85634E+00_JPRB ,&
&  0.85922E+00_JPRB, 0.86220E+00_JPRB, 0.86531E+00_JPRB, 0.86814E+00_JPRB, 0.87072E+00_JPRB, 0.87300E+00_JPRB, 0.87478E+00_JPRB ,&
&  0.87674E+00_JPRB, 0.87872E+00_JPRB, 0.88079E+00_JPRB, 0.88238E+00_JPRB, 0.88374E+00_JPRB, 0.88498E+00_JPRB, 0.88633E+00_JPRB ,&
&  0.88708E+00_JPRB, 0.88761E+00_JPRB, 0.88829E+00_JPRB, 0.88919E+00_JPRB, 0.88985E+00_JPRB, 0.89037E+00_JPRB, 0.89096E+00_JPRB ,&
&  0.89179E+00_JPRB, 0.89249E+00_JPRB, 0.89293E+00_JPRB, 0.89339E+00_JPRB, 0.89408E+00_JPRB, 0.89503E+00_JPRB, 0.89608E+00_JPRB ,&
&  0.89703E+00_JPRB, 0.89798E+00_JPRB, 0.89874E+00_JPRB, 0.89923E+00_JPRB, 0.89934E+00_JPRB, 0.89902E+00_JPRB, 0.89789E+00_JPRB ,&
&  0.89649E+00_JPRB, 0.89493E+00_JPRB, 0.89370E+00_JPRB, 0.89306E+00_JPRB, 0.89236E+00_JPRB, 0.89185E+00_JPRB, 0.89166E+00_JPRB ,&
&  0.89191E+00_JPRB, 0.89221E+00_JPRB, 0.89254E+00_JPRB, 0.89304E+00_JPRB, 0.89333E+00_JPRB, 0.89320E+00_JPRB, 0.89285E+00_JPRB ,&
&  0.89224E+00_JPRB, 0.89118E+00_JPRB, 0.88970E+00_JPRB, 0.88798E+00_JPRB, 0.88616E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.87627E+00_JPRB, 0.87291E+00_JPRB, 0.87050E+00_JPRB, 0.86883E+00_JPRB, 0.86759E+00_JPRB, 0.86637E+00_JPRB, 0.86516E+00_JPRB ,&
&  0.86394E+00_JPRB, 0.86415E+00_JPRB, 0.86390E+00_JPRB, 0.86377E+00_JPRB, 0.86350E+00_JPRB, 0.86419E+00_JPRB, 0.86447E+00_JPRB ,&
&  0.86408E+00_JPRB, 0.86236E+00_JPRB, 0.86101E+00_JPRB, 0.85885E+00_JPRB, 0.85862E+00_JPRB, 0.85825E+00_JPRB, 0.85919E+00_JPRB ,&
&  0.85786E+00_JPRB, 0.85609E+00_JPRB, 0.85325E+00_JPRB, 0.84587E+00_JPRB, 0.83910E+00_JPRB, 0.83255E+00_JPRB, 0.82683E+00_JPRB ,&
&  0.82146E+00_JPRB, 0.81734E+00_JPRB, 0.81517E+00_JPRB, 0.81428E+00_JPRB, 0.81797E+00_JPRB, 0.81900E+00_JPRB, 0.81925E+00_JPRB ,&
&  0.81802E+00_JPRB, 0.81289E+00_JPRB, 0.81393E+00_JPRB, 0.81542E+00_JPRB, 0.81774E+00_JPRB, 0.82340E+00_JPRB, 0.82853E+00_JPRB ,&
&  0.83450E+00_JPRB, 0.84123E+00_JPRB, 0.85195E+00_JPRB, 0.85967E+00_JPRB, 0.86716E+00_JPRB, 0.87343E+00_JPRB, 0.87671E+00_JPRB ,&
&  0.87999E+00_JPRB, 0.88289E+00_JPRB, 0.88588E+00_JPRB, 0.88824E+00_JPRB, 0.89000E+00_JPRB, 0.89196E+00_JPRB, 0.89426E+00_JPRB ,&
&  0.89567E+00_JPRB, 0.89667E+00_JPRB, 0.89778E+00_JPRB, 0.89893E+00_JPRB, 0.89964E+00_JPRB, 0.90051E+00_JPRB, 0.90169E+00_JPRB ,&
&  0.90288E+00_JPRB, 0.90273E+00_JPRB, 0.90255E+00_JPRB, 0.90334E+00_JPRB, 0.90428E+00_JPRB, 0.90531E+00_JPRB, 0.90556E+00_JPRB ,&
&  0.90620E+00_JPRB, 0.90651E+00_JPRB, 0.90725E+00_JPRB, 0.90851E+00_JPRB, 0.90909E+00_JPRB, 0.90969E+00_JPRB, 0.90927E+00_JPRB ,&
&  0.90849E+00_JPRB, 0.90678E+00_JPRB, 0.90526E+00_JPRB, 0.90486E+00_JPRB, 0.90447E+00_JPRB, 0.90516E+00_JPRB, 0.90635E+00_JPRB ,&
&  0.90830E+00_JPRB, 0.90799E+00_JPRB, 0.90712E+00_JPRB, 0.90531E+00_JPRB, 0.90337E+00_JPRB, 0.90090E+00_JPRB, 0.89850E+00_JPRB ,&
&  0.89597E+00_JPRB, 0.89296E+00_JPRB, 0.88898E+00_JPRB, 0.88463E+00_JPRB, 0.88028E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.87906E+00_JPRB, 0.87340E+00_JPRB, 0.86923E+00_JPRB, 0.86639E+00_JPRB, 0.86373E+00_JPRB, 0.86139E+00_JPRB, 0.85951E+00_JPRB ,&
&  0.85774E+00_JPRB, 0.85484E+00_JPRB, 0.85337E+00_JPRB, 0.85236E+00_JPRB, 0.85083E+00_JPRB, 0.85152E+00_JPRB, 0.85177E+00_JPRB ,&
&  0.85278E+00_JPRB, 0.85250E+00_JPRB, 0.85561E+00_JPRB, 0.85449E+00_JPRB, 0.85361E+00_JPRB, 0.85235E+00_JPRB, 0.84745E+00_JPRB ,&
&  0.84534E+00_JPRB, 0.84261E+00_JPRB, 0.83572E+00_JPRB, 0.82766E+00_JPRB, 0.82233E+00_JPRB, 0.79876E+00_JPRB, 0.79903E+00_JPRB ,&
&  0.80806E+00_JPRB, 0.79255E+00_JPRB, 0.78419E+00_JPRB, 0.77594E+00_JPRB, 0.76469E+00_JPRB, 0.76413E+00_JPRB, 0.76906E+00_JPRB ,&
&  0.76159E+00_JPRB, 0.78238E+00_JPRB, 0.78514E+00_JPRB, 0.78687E+00_JPRB, 0.78934E+00_JPRB, 0.78617E+00_JPRB, 0.79280E+00_JPRB ,&
&  0.81738E+00_JPRB, 0.82233E+00_JPRB, 0.83387E+00_JPRB, 0.84400E+00_JPRB, 0.85632E+00_JPRB, 0.86734E+00_JPRB, 0.87924E+00_JPRB ,&
&  0.88815E+00_JPRB, 0.89450E+00_JPRB, 0.89903E+00_JPRB, 0.89988E+00_JPRB, 0.90066E+00_JPRB, 0.90060E+00_JPRB, 0.90209E+00_JPRB ,&
&  0.90230E+00_JPRB, 0.90376E+00_JPRB, 0.90636E+00_JPRB, 0.90910E+00_JPRB, 0.91280E+00_JPRB, 0.91478E+00_JPRB, 0.91690E+00_JPRB ,&
&  0.91822E+00_JPRB, 0.91826E+00_JPRB, 0.91779E+00_JPRB, 0.91757E+00_JPRB, 0.91618E+00_JPRB, 0.91542E+00_JPRB, 0.91519E+00_JPRB ,&
&  0.91623E+00_JPRB, 0.91673E+00_JPRB, 0.91830E+00_JPRB, 0.91988E+00_JPRB, 0.92248E+00_JPRB, 0.92653E+00_JPRB, 0.92975E+00_JPRB ,&
&  0.93443E+00_JPRB, 0.93699E+00_JPRB, 0.93862E+00_JPRB, 0.93947E+00_JPRB, 0.94006E+00_JPRB, 0.94180E+00_JPRB, 0.94306E+00_JPRB ,&
&  0.94524E+00_JPRB, 0.94162E+00_JPRB, 0.93672E+00_JPRB, 0.93120E+00_JPRB, 0.92639E+00_JPRB, 0.92202E+00_JPRB, 0.91738E+00_JPRB ,&
&  0.91321E+00_JPRB, 0.90755E+00_JPRB, 0.90102E+00_JPRB, 0.89332E+00_JPRB, 0.88567E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.89886E+00_JPRB, 0.89006E+00_JPRB, 0.88215E+00_JPRB, 0.87656E+00_JPRB, 0.87107E+00_JPRB, 0.86662E+00_JPRB, 0.86265E+00_JPRB ,&
&  0.85922E+00_JPRB, 0.85604E+00_JPRB, 0.85400E+00_JPRB, 0.85195E+00_JPRB, 0.85054E+00_JPRB, 0.84896E+00_JPRB, 0.84813E+00_JPRB ,&
&  0.85037E+00_JPRB, 0.85044E+00_JPRB, 0.85384E+00_JPRB, 0.85254E+00_JPRB, 0.85146E+00_JPRB, 0.84740E+00_JPRB, 0.83557E+00_JPRB ,&
&  0.82875E+00_JPRB, 0.82221E+00_JPRB, 0.80247E+00_JPRB, 0.81677E+00_JPRB, 0.80732E+00_JPRB, 0.80096E+00_JPRB, 0.79220E+00_JPRB ,&
&  0.77411E+00_JPRB, 0.76552E+00_JPRB, 0.75505E+00_JPRB, 0.75413E+00_JPRB, 0.76040E+00_JPRB, 0.75710E+00_JPRB, 0.76206E+00_JPRB ,&
&  0.75698E+00_JPRB, 0.75550E+00_JPRB, 0.75681E+00_JPRB, 0.75721E+00_JPRB, 0.76380E+00_JPRB, 0.76499E+00_JPRB, 0.77027E+00_JPRB ,&
&  0.78126E+00_JPRB, 0.79753E+00_JPRB, 0.82191E+00_JPRB, 0.82988E+00_JPRB, 0.84894E+00_JPRB, 0.86438E+00_JPRB, 0.87861E+00_JPRB ,&
&  0.88804E+00_JPRB, 0.89385E+00_JPRB, 0.89828E+00_JPRB, 0.90003E+00_JPRB, 0.89998E+00_JPRB, 0.89622E+00_JPRB, 0.89645E+00_JPRB ,&
&  0.89800E+00_JPRB, 0.90051E+00_JPRB, 0.90380E+00_JPRB, 0.90620E+00_JPRB, 0.90993E+00_JPRB, 0.91279E+00_JPRB, 0.91734E+00_JPRB ,&
&  0.92113E+00_JPRB, 0.92494E+00_JPRB, 0.92604E+00_JPRB, 0.92789E+00_JPRB, 0.92969E+00_JPRB, 0.93130E+00_JPRB, 0.93310E+00_JPRB ,&
&  0.93531E+00_JPRB, 0.93883E+00_JPRB, 0.94443E+00_JPRB, 0.94256E+00_JPRB, 0.94720E+00_JPRB, 0.96346E+00_JPRB, 0.96883E+00_JPRB ,&
&  0.97558E+00_JPRB, 0.97770E+00_JPRB, 0.97652E+00_JPRB, 0.97508E+00_JPRB, 0.97338E+00_JPRB, 0.97384E+00_JPRB, 0.97429E+00_JPRB ,&
&  0.97454E+00_JPRB, 0.96836E+00_JPRB, 0.96151E+00_JPRB, 0.95557E+00_JPRB, 0.94926E+00_JPRB, 0.94440E+00_JPRB, 0.93931E+00_JPRB ,&
&  0.93576E+00_JPRB, 0.93258E+00_JPRB, 0.92826E+00_JPRB, 0.92033E+00_JPRB, 0.91033E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.92226E+00_JPRB, 0.91139E+00_JPRB, 0.90265E+00_JPRB, 0.89531E+00_JPRB, 0.88940E+00_JPRB, 0.88441E+00_JPRB, 0.87965E+00_JPRB ,&
&  0.87543E+00_JPRB, 0.87052E+00_JPRB, 0.86568E+00_JPRB, 0.86194E+00_JPRB, 0.85885E+00_JPRB, 0.85619E+00_JPRB, 0.85325E+00_JPRB ,&
&  0.85216E+00_JPRB, 0.85055E+00_JPRB, 0.85257E+00_JPRB, 0.85032E+00_JPRB, 0.85272E+00_JPRB, 0.84880E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.82232E+00_JPRB, 0.80642E+00_JPRB, 0.79131E+00_JPRB, 0.80380E+00_JPRB, 0.79998E+00_JPRB, 0.79813E+00_JPRB, 0.77867E+00_JPRB ,&
&  0.76202E+00_JPRB, 0.74636E+00_JPRB, 0.77188E+00_JPRB, 0.76140E+00_JPRB, 0.75569E+00_JPRB, 0.74677E+00_JPRB, 0.72617E+00_JPRB ,&
&  0.72275E+00_JPRB, 0.70987E+00_JPRB, 0.70900E+00_JPRB, 0.71410E+00_JPRB, 0.72146E+00_JPRB, 0.73263E+00_JPRB, 0.74345E+00_JPRB ,&
&  0.74322E+00_JPRB, 0.78364E+00_JPRB, 0.81129E+00_JPRB, 0.83180E+00_JPRB, 0.85225E+00_JPRB, 0.86857E+00_JPRB, 0.87973E+00_JPRB ,&
&  0.89042E+00_JPRB, 0.89854E+00_JPRB, 0.90697E+00_JPRB, 0.90980E+00_JPRB, 0.92263E+00_JPRB, 0.92628E+00_JPRB, 0.93165E+00_JPRB ,&
&  0.93778E+00_JPRB, 0.93783E+00_JPRB, 0.93583E+00_JPRB, 0.93070E+00_JPRB, 0.92758E+00_JPRB, 0.92366E+00_JPRB, 0.92415E+00_JPRB ,&
&  0.92439E+00_JPRB, 0.92587E+00_JPRB, 0.93212E+00_JPRB, 0.93773E+00_JPRB, 0.94293E+00_JPRB, 0.94762E+00_JPRB, 0.95301E+00_JPRB ,&
&  0.95848E+00_JPRB, 0.96899E+00_JPRB, 0.97982E+00_JPRB, 0.99889E+00_JPRB, 0.10162E+01_JPRB, 0.10264E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10434E+01_JPRB, 0.10373E+01_JPRB, 0.10314E+01_JPRB, 0.10281E+01_JPRB, 0.10265E+01_JPRB, 0.10265E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10270E+01_JPRB, 0.10210E+01_JPRB, 0.10135E+01_JPRB, 0.10045E+01_JPRB, 0.99582E+00_JPRB, 0.98845E+00_JPRB, 0.98209E+00_JPRB ,&
&  0.97597E+00_JPRB, 0.96913E+00_JPRB, 0.95858E+00_JPRB, 0.94617E+00_JPRB, 0.93381E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.95732E+00_JPRB, 0.94459E+00_JPRB, 0.93465E+00_JPRB, 0.92625E+00_JPRB, 0.92018E+00_JPRB, 0.91515E+00_JPRB, 0.91102E+00_JPRB ,&
&  0.90775E+00_JPRB, 0.90180E+00_JPRB, 0.89491E+00_JPRB, 0.88639E+00_JPRB, 0.88015E+00_JPRB, 0.87451E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.85999E+00_JPRB, 0.85116E+00_JPRB, 0.84208E+00_JPRB, 0.83495E+00_JPRB, 0.83777E+00_JPRB, 0.83489E+00_JPRB, 0.83019E+00_JPRB ,&
&  0.81823E+00_JPRB, 0.80175E+00_JPRB, 0.78860E+00_JPRB, 0.79298E+00_JPRB, 0.79414E+00_JPRB, 0.78013E+00_JPRB, 0.77612E+00_JPRB ,&
&  0.78930E+00_JPRB, 0.79044E+00_JPRB, 0.78156E+00_JPRB, 0.77658E+00_JPRB, 0.74364E+00_JPRB, 0.69882E+00_JPRB, 0.72675E+00_JPRB ,&
&  0.72611E+00_JPRB, 0.70404E+00_JPRB, 0.70060E+00_JPRB, 0.69833E+00_JPRB, 0.70767E+00_JPRB, 0.72708E+00_JPRB, 0.74496E+00_JPRB ,&
&  0.74955E+00_JPRB, 0.81785E+00_JPRB, 0.84886E+00_JPRB, 0.87118E+00_JPRB, 0.88670E+00_JPRB, 0.89876E+00_JPRB, 0.90856E+00_JPRB ,&
&  0.91956E+00_JPRB, 0.93633E+00_JPRB, 0.95927E+00_JPRB, 0.98163E+00_JPRB, 0.10049E+01_JPRB, 0.10212E+01_JPRB, 0.10387E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.10597E+01_JPRB, 0.10405E+01_JPRB, 0.10197E+01_JPRB, 0.10002E+01_JPRB, 0.98268E+00_JPRB, 0.96731E+00_JPRB ,&
&  0.96079E+00_JPRB, 0.95588E+00_JPRB, 0.96238E+00_JPRB, 0.96319E+00_JPRB, 0.96695E+00_JPRB, 0.97213E+00_JPRB, 0.98738E+00_JPRB ,&
&  0.99793E+00_JPRB, 0.10147E+01_JPRB, 0.10348E+01_JPRB, 0.10622E+01_JPRB, 0.10858E+01_JPRB, 0.10996E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.11129E+01_JPRB, 0.11225E+01_JPRB, 0.11255E+01_JPRB, 0.11336E+01_JPRB, 0.11467E+01_JPRB, 0.11534E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.11635E+01_JPRB, 0.11568E+01_JPRB, 0.11444E+01_JPRB, 0.11267E+01_JPRB, 0.11064E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.10257E+01_JPRB, 0.10009E+01_JPRB, 0.98380E+00_JPRB, 0.96933E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.10241E+01_JPRB, 0.99945E+00_JPRB, 0.98143E+00_JPRB, 0.97005E+00_JPRB, 0.96102E+00_JPRB, 0.95428E+00_JPRB, 0.94803E+00_JPRB ,&
&  0.94176E+00_JPRB, 0.93545E+00_JPRB, 0.92677E+00_JPRB, 0.91794E+00_JPRB, 0.91075E+00_JPRB, 0.90333E+00_JPRB, 0.89725E+00_JPRB ,&
&  0.88720E+00_JPRB, 0.86424E+00_JPRB, 0.86020E+00_JPRB, 0.84898E+00_JPRB, 0.83012E+00_JPRB, 0.81940E+00_JPRB, 0.81118E+00_JPRB ,&
&  0.80148E+00_JPRB, 0.79773E+00_JPRB, 0.79865E+00_JPRB, 0.79235E+00_JPRB, 0.78878E+00_JPRB, 0.79964E+00_JPRB, 0.80032E+00_JPRB ,&
&  0.76855E+00_JPRB, 0.76835E+00_JPRB, 0.80980E+00_JPRB, 0.81004E+00_JPRB, 0.76257E+00_JPRB, 0.74928E+00_JPRB, 0.75715E+00_JPRB ,&
&  0.75348E+00_JPRB, 0.71924E+00_JPRB, 0.71370E+00_JPRB, 0.71190E+00_JPRB, 0.72644E+00_JPRB, 0.74111E+00_JPRB, 0.77716E+00_JPRB ,&
&  0.80581E+00_JPRB, 0.86913E+00_JPRB, 0.90153E+00_JPRB, 0.92581E+00_JPRB, 0.94398E+00_JPRB, 0.96202E+00_JPRB, 0.98485E+00_JPRB ,&
&  0.10189E+01_JPRB, 0.10669E+01_JPRB, 0.11265E+01_JPRB, 0.11790E+01_JPRB, 0.12287E+01_JPRB, 0.12532E+01_JPRB, 0.12578E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.12800E+01_JPRB, 0.12323E+01_JPRB, 0.11908E+01_JPRB, 0.11444E+01_JPRB, 0.10998E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.10413E+01_JPRB, 0.10485E+01_JPRB, 0.10353E+01_JPRB, 0.10237E+01_JPRB, 0.10182E+01_JPRB, 0.10186E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10780E+01_JPRB, 0.12141E+01_JPRB, 0.12541E+01_JPRB, 0.12762E+01_JPRB, 0.12857E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.13195E+01_JPRB, 0.13416E+01_JPRB, 0.13531E+01_JPRB, 0.13650E+01_JPRB, 0.13559E+01_JPRB, 0.13662E+01_JPRB ,&
&  0.13575E+01_JPRB, 0.13456E+01_JPRB, 0.13308E+01_JPRB, 0.13188E+01_JPRB, 0.13036E+01_JPRB, 0.12883E+01_JPRB, 0.12633E+01_JPRB ,&
&  0.12288E+01_JPRB, 0.11885E+01_JPRB, 0.11469E+01_JPRB, 0.11025E+01_JPRB, 0.10598E+01_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.11264E+01_JPRB, 0.10715E+01_JPRB, 0.10332E+01_JPRB, 0.10150E+01_JPRB, 0.10027E+01_JPRB, 0.98136E+00_JPRB, 0.96481E+00_JPRB ,&
&  0.95251E+00_JPRB, 0.94971E+00_JPRB, 0.93811E+00_JPRB, 0.92537E+00_JPRB, 0.91709E+00_JPRB, 0.90991E+00_JPRB, 0.90607E+00_JPRB ,&
&  0.90151E+00_JPRB, 0.89072E+00_JPRB, 0.86849E+00_JPRB, 0.84518E+00_JPRB, 0.82890E+00_JPRB, 0.81516E+00_JPRB, 0.80317E+00_JPRB ,&
&  0.79326E+00_JPRB, 0.79194E+00_JPRB, 0.80430E+00_JPRB, 0.80462E+00_JPRB, 0.80352E+00_JPRB, 0.79649E+00_JPRB, 0.78973E+00_JPRB ,&
&  0.79222E+00_JPRB, 0.80818E+00_JPRB, 0.78567E+00_JPRB, 0.79318E+00_JPRB, 0.81153E+00_JPRB, 0.78987E+00_JPRB, 0.77355E+00_JPRB ,&
&  0.75921E+00_JPRB, 0.74642E+00_JPRB, 0.73948E+00_JPRB, 0.73956E+00_JPRB, 0.76410E+00_JPRB, 0.79283E+00_JPRB, 0.84020E+00_JPRB ,&
&  0.89319E+00_JPRB, 0.93157E+00_JPRB, 0.95793E+00_JPRB, 0.98332E+00_JPRB, 0.10074E+01_JPRB, 0.10373E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.12851E+01_JPRB, 0.13777E+01_JPRB, 0.14436E+01_JPRB, 0.15080E+01_JPRB, 0.14713E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.16294E+01_JPRB, 0.16823E+01_JPRB, 0.15910E+01_JPRB, 0.15076E+01_JPRB, 0.14120E+01_JPRB, 0.13261E+01_JPRB, 0.12581E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.11833E+01_JPRB, 0.11578E+01_JPRB, 0.11239E+01_JPRB, 0.11016E+01_JPRB, 0.11079E+01_JPRB, 0.11434E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.14202E+01_JPRB, 0.18074E+01_JPRB, 0.15999E+01_JPRB, 0.15281E+01_JPRB, 0.15203E+01_JPRB, 0.15288E+01_JPRB ,&
&  0.15647E+01_JPRB, 0.15934E+01_JPRB, 0.16086E+01_JPRB, 0.15812E+01_JPRB, 0.15303E+01_JPRB, 0.14684E+01_JPRB, 0.14275E+01_JPRB ,&
&  0.13930E+01_JPRB, 0.13589E+01_JPRB, 0.13379E+01_JPRB, 0.13323E+01_JPRB, 0.13168E+01_JPRB, 0.12996E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.12587E+01_JPRB, 0.12425E+01_JPRB, 0.12332E+01_JPRB, 0.12145E+01_JPRB, 0.11843E+01_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.11611E+01_JPRB, 0.11631E+01_JPRB, 0.11409E+01_JPRB, 0.11132E+01_JPRB, 0.10836E+01_JPRB, 0.10269E+01_JPRB, 0.98264E+00_JPRB ,&
&  0.95847E+00_JPRB, 0.94888E+00_JPRB, 0.94611E+00_JPRB, 0.94041E+00_JPRB, 0.93476E+00_JPRB, 0.92706E+00_JPRB, 0.91452E+00_JPRB ,&
&  0.89847E+00_JPRB, 0.88714E+00_JPRB, 0.87799E+00_JPRB, 0.86305E+00_JPRB, 0.84276E+00_JPRB, 0.82073E+00_JPRB, 0.80260E+00_JPRB ,&
&  0.78949E+00_JPRB, 0.78446E+00_JPRB, 0.79523E+00_JPRB, 0.80700E+00_JPRB, 0.81794E+00_JPRB, 0.82054E+00_JPRB, 0.81508E+00_JPRB ,&
&  0.79648E+00_JPRB, 0.79266E+00_JPRB, 0.80598E+00_JPRB, 0.80814E+00_JPRB, 0.81708E+00_JPRB, 0.81784E+00_JPRB, 0.80245E+00_JPRB ,&
&  0.78914E+00_JPRB, 0.77936E+00_JPRB, 0.78358E+00_JPRB, 0.80114E+00_JPRB, 0.85493E+00_JPRB, 0.91305E+00_JPRB, 0.94261E+00_JPRB ,&
&  0.97200E+00_JPRB, 0.99261E+00_JPRB, 0.10151E+01_JPRB, 0.10453E+01_JPRB, 0.10950E+01_JPRB, 0.11685E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.16015E+01_JPRB, 0.17144E+01_JPRB, 0.16165E+01_JPRB, 0.15818E+01_JPRB, 0.16306E+01_JPRB, 0.17093E+01_JPRB ,&
&  0.17900E+01_JPRB, 0.18216E+01_JPRB, 0.18074E+01_JPRB, 0.18173E+01_JPRB, 0.17218E+01_JPRB, 0.16036E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.12736E+01_JPRB, 0.12436E+01_JPRB, 0.12178E+01_JPRB, 0.12307E+01_JPRB, 0.12743E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.14780E+01_JPRB, 0.16295E+01_JPRB, 0.17094E+01_JPRB, 0.16629E+01_JPRB, 0.16984E+01_JPRB, 0.17728E+01_JPRB, 0.18285E+01_JPRB ,&
&  0.18268E+01_JPRB, 0.18188E+01_JPRB, 0.18271E+01_JPRB, 0.18080E+01_JPRB, 0.16876E+01_JPRB, 0.15764E+01_JPRB, 0.14516E+01_JPRB ,&
&  0.13307E+01_JPRB, 0.12238E+01_JPRB, 0.11704E+01_JPRB, 0.11564E+01_JPRB, 0.11471E+01_JPRB, 0.11334E+01_JPRB, 0.11053E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10705E+01_JPRB, 0.10873E+01_JPRB, 0.11115E+01_JPRB, 0.11429E+01_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.12104E+01_JPRB, 0.12380E+01_JPRB, 0.12519E+01_JPRB, 0.12487E+01_JPRB, 0.11894E+01_JPRB, 0.11288E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.10319E+01_JPRB, 0.99582E+00_JPRB, 0.97114E+00_JPRB, 0.94436E+00_JPRB, 0.92138E+00_JPRB, 0.90919E+00_JPRB, 0.90581E+00_JPRB ,&
&  0.90499E+00_JPRB, 0.89336E+00_JPRB, 0.87873E+00_JPRB, 0.86066E+00_JPRB, 0.84628E+00_JPRB, 0.83383E+00_JPRB, 0.82163E+00_JPRB ,&
&  0.80929E+00_JPRB, 0.80713E+00_JPRB, 0.81255E+00_JPRB, 0.82662E+00_JPRB, 0.83623E+00_JPRB, 0.84072E+00_JPRB, 0.84155E+00_JPRB ,&
&  0.83310E+00_JPRB, 0.82021E+00_JPRB, 0.82481E+00_JPRB, 0.83774E+00_JPRB, 0.84753E+00_JPRB, 0.85561E+00_JPRB, 0.83407E+00_JPRB ,&
&  0.82346E+00_JPRB, 0.83019E+00_JPRB, 0.85868E+00_JPRB, 0.92929E+00_JPRB, 0.97138E+00_JPRB, 0.99646E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.10042E+01_JPRB, 0.10088E+01_JPRB, 0.10442E+01_JPRB, 0.11143E+01_JPRB, 0.12250E+01_JPRB, 0.14274E+01_JPRB, 0.16763E+01_JPRB ,&
&  0.18293E+01_JPRB, 0.19116E+01_JPRB, 0.19574E+01_JPRB, 0.18444E+01_JPRB, 0.18614E+01_JPRB, 0.21077E+01_JPRB, 0.21849E+01_JPRB ,&
&  0.22409E+01_JPRB, 0.22454E+01_JPRB, 0.22799E+01_JPRB, 0.21803E+01_JPRB, 0.19465E+01_JPRB, 0.17710E+01_JPRB, 0.16427E+01_JPRB ,&
&  0.15143E+01_JPRB, 0.14406E+01_JPRB, 0.14250E+01_JPRB, 0.14617E+01_JPRB, 0.15492E+01_JPRB, 0.17282E+01_JPRB, 0.19579E+01_JPRB ,&
&  0.21810E+01_JPRB, 0.23146E+01_JPRB, 0.23913E+01_JPRB, 0.23686E+01_JPRB, 0.23273E+01_JPRB, 0.22681E+01_JPRB, 0.22225E+01_JPRB ,&
&  0.21713E+01_JPRB, 0.21541E+01_JPRB, 0.21324E+01_JPRB, 0.21064E+01_JPRB, 0.19461E+01_JPRB, 0.16904E+01_JPRB, 0.14463E+01_JPRB ,&
&  0.12758E+01_JPRB, 0.11807E+01_JPRB, 0.11146E+01_JPRB, 0.11002E+01_JPRB, 0.11035E+01_JPRB, 0.11059E+01_JPRB, 0.10731E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.10698E+01_JPRB, 0.10943E+01_JPRB, 0.11200E+01_JPRB, 0.11630E+01_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.13248E+01_JPRB, 0.13553E+01_JPRB, 0.13669E+01_JPRB, 0.13493E+01_JPRB, 0.12984E+01_JPRB, 0.12186E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.10845E+01_JPRB, 0.10414E+01_JPRB, 0.10165E+01_JPRB, 0.98962E+00_JPRB, 0.96604E+00_JPRB, 0.93406E+00_JPRB, 0.90320E+00_JPRB ,&
&  0.89333E+00_JPRB, 0.89706E+00_JPRB, 0.89703E+00_JPRB, 0.88747E+00_JPRB, 0.87674E+00_JPRB, 0.86619E+00_JPRB, 0.85704E+00_JPRB ,&
&  0.84917E+00_JPRB, 0.85001E+00_JPRB, 0.86417E+00_JPRB, 0.88505E+00_JPRB, 0.89547E+00_JPRB, 0.89891E+00_JPRB, 0.90514E+00_JPRB ,&
&  0.90091E+00_JPRB, 0.88716E+00_JPRB, 0.88121E+00_JPRB, 0.88632E+00_JPRB, 0.89047E+00_JPRB, 0.91368E+00_JPRB, 0.91340E+00_JPRB ,&
&  0.87511E+00_JPRB, 0.90489E+00_JPRB, 0.97462E+00_JPRB, 0.10141E+01_JPRB, 0.10401E+01_JPRB, 0.10585E+01_JPRB, 0.10736E+01_JPRB ,&
&  0.10877E+01_JPRB, 0.11159E+01_JPRB, 0.11517E+01_JPRB, 0.11976E+01_JPRB, 0.14066E+01_JPRB, 0.18711E+01_JPRB, 0.20592E+01_JPRB ,&
&  0.21639E+01_JPRB, 0.19540E+01_JPRB, 0.18507E+01_JPRB, 0.18832E+01_JPRB, 0.21488E+01_JPRB, 0.23179E+01_JPRB, 0.24521E+01_JPRB ,&
&  0.26046E+01_JPRB, 0.25987E+01_JPRB, 0.25412E+01_JPRB, 0.24110E+01_JPRB, 0.21869E+01_JPRB, 0.20197E+01_JPRB, 0.19494E+01_JPRB ,&
&  0.18702E+01_JPRB, 0.18542E+01_JPRB, 0.19446E+01_JPRB, 0.21578E+01_JPRB, 0.24095E+01_JPRB, 0.26229E+01_JPRB, 0.27849E+01_JPRB ,&
&  0.29440E+01_JPRB, 0.30157E+01_JPRB, 0.29705E+01_JPRB, 0.28873E+01_JPRB, 0.27505E+01_JPRB, 0.26662E+01_JPRB, 0.26275E+01_JPRB ,&
&  0.25820E+01_JPRB, 0.25420E+01_JPRB, 0.24626E+01_JPRB, 0.23562E+01_JPRB, 0.21837E+01_JPRB, 0.20239E+01_JPRB, 0.18195E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.13302E+01_JPRB, 0.11859E+01_JPRB, 0.11454E+01_JPRB, 0.11468E+01_JPRB, 0.11646E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.11755E+01_JPRB, 0.11901E+01_JPRB, 0.11980E+01_JPRB, 0.12367E+01_JPRB, 0.12859E+01_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.14515E+01_JPRB, 0.14620E+01_JPRB, 0.14508E+01_JPRB, 0.14314E+01_JPRB, 0.13924E+01_JPRB, 0.13194E+01_JPRB, 0.12500E+01_JPRB ,&
&  0.12747E+01_JPRB, 0.12257E+01_JPRB, 0.12198E+01_JPRB, 0.10839E+01_JPRB, 0.10355E+01_JPRB, 0.10137E+01_JPRB, 0.97294E+00_JPRB ,&
&  0.94911E+00_JPRB, 0.93977E+00_JPRB, 0.92911E+00_JPRB, 0.92686E+00_JPRB, 0.92363E+00_JPRB, 0.91330E+00_JPRB, 0.90759E+00_JPRB ,&
&  0.91172E+00_JPRB, 0.92598E+00_JPRB, 0.95335E+00_JPRB, 0.97281E+00_JPRB, 0.99082E+00_JPRB, 0.10020E+01_JPRB, 0.10086E+01_JPRB ,&
&  0.97830E+00_JPRB, 0.96305E+00_JPRB, 0.96108E+00_JPRB, 0.94751E+00_JPRB, 0.95151E+00_JPRB, 0.97109E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10057E+01_JPRB, 0.10697E+01_JPRB, 0.11038E+01_JPRB, 0.11251E+01_JPRB, 0.11395E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.11898E+01_JPRB, 0.12314E+01_JPRB, 0.13014E+01_JPRB, 0.17014E+01_JPRB, 0.24448E+01_JPRB, 0.24707E+01_JPRB ,&
&  0.24220E+01_JPRB, 0.20523E+01_JPRB, 0.22545E+01_JPRB, 0.23474E+01_JPRB, 0.25253E+01_JPRB, 0.26977E+01_JPRB, 0.27517E+01_JPRB ,&
&  0.30449E+01_JPRB, 0.31587E+01_JPRB, 0.30406E+01_JPRB, 0.27753E+01_JPRB, 0.25413E+01_JPRB, 0.24246E+01_JPRB, 0.23758E+01_JPRB ,&
&  0.23898E+01_JPRB, 0.23263E+01_JPRB, 0.25958E+01_JPRB, 0.28274E+01_JPRB, 0.29590E+01_JPRB, 0.30934E+01_JPRB, 0.32487E+01_JPRB ,&
&  0.33442E+01_JPRB, 0.33354E+01_JPRB, 0.33007E+01_JPRB, 0.30984E+01_JPRB, 0.29888E+01_JPRB, 0.29026E+01_JPRB, 0.27596E+01_JPRB ,&
&  0.26664E+01_JPRB, 0.25918E+01_JPRB, 0.24902E+01_JPRB, 0.24953E+01_JPRB, 0.22651E+01_JPRB, 0.21060E+01_JPRB, 0.20134E+01_JPRB ,&
&  0.17696E+01_JPRB, 0.15182E+01_JPRB, 0.14102E+01_JPRB, 0.13706E+01_JPRB, 0.13358E+01_JPRB, 0.13298E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.12864E+01_JPRB, 0.13056E+01_JPRB, 0.13730E+01_JPRB, 0.14041E+01_JPRB, 0.14248E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.15606E+01_JPRB, 0.15677E+01_JPRB, 0.15548E+01_JPRB, 0.15359E+01_JPRB, 0.15154E+01_JPRB, 0.14785E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.13958E+01_JPRB, 0.13514E+01_JPRB, 0.13011E+01_JPRB, 0.12634E+01_JPRB, 0.12491E+01_JPRB, 0.11618E+01_JPRB, 0.10368E+01_JPRB ,&
&  0.98872E+00_JPRB, 0.98855E+00_JPRB, 0.99929E+00_JPRB, 0.10001E+01_JPRB, 0.10008E+01_JPRB, 0.99997E+00_JPRB, 0.10137E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10593E+01_JPRB, 0.10965E+01_JPRB, 0.11380E+01_JPRB, 0.11496E+01_JPRB, 0.11534E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.10732E+01_JPRB, 0.10443E+01_JPRB, 0.99864E+00_JPRB, 0.10451E+01_JPRB, 0.10837E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11961E+01_JPRB, 0.12284E+01_JPRB, 0.12477E+01_JPRB, 0.12512E+01_JPRB, 0.12562E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12767E+01_JPRB, 0.13103E+01_JPRB, 0.13636E+01_JPRB, 0.14536E+01_JPRB, 0.22447E+01_JPRB, 0.30785E+01_JPRB, 0.31977E+01_JPRB ,&
&  0.29793E+01_JPRB, 0.31538E+01_JPRB, 0.33050E+01_JPRB, 0.30937E+01_JPRB, 0.32170E+01_JPRB, 0.33080E+01_JPRB, 0.32193E+01_JPRB ,&
&  0.33663E+01_JPRB, 0.33824E+01_JPRB, 0.33261E+01_JPRB, 0.32403E+01_JPRB, 0.31729E+01_JPRB, 0.30723E+01_JPRB, 0.29974E+01_JPRB ,&
&  0.30288E+01_JPRB, 0.32155E+01_JPRB, 0.32680E+01_JPRB, 0.33522E+01_JPRB, 0.33758E+01_JPRB, 0.34430E+01_JPRB, 0.35044E+01_JPRB ,&
&  0.35190E+01_JPRB, 0.35368E+01_JPRB, 0.35071E+01_JPRB, 0.33811E+01_JPRB, 0.32276E+01_JPRB, 0.30674E+01_JPRB, 0.28804E+01_JPRB ,&
&  0.27255E+01_JPRB, 0.24475E+01_JPRB, 0.22362E+01_JPRB, 0.21672E+01_JPRB, 0.21028E+01_JPRB, 0.19355E+01_JPRB, 0.18885E+01_JPRB ,&
&  0.18469E+01_JPRB, 0.17717E+01_JPRB, 0.18869E+01_JPRB, 0.17781E+01_JPRB, 0.16636E+01_JPRB, 0.15237E+01_JPRB, 0.13715E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.14202E+01_JPRB, 0.15137E+01_JPRB, 0.15628E+01_JPRB, 0.15641E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.16358E+01_JPRB, 0.16476E+01_JPRB, 0.16496E+01_JPRB, 0.16478E+01_JPRB, 0.16300E+01_JPRB, 0.15932E+01_JPRB, 0.15433E+01_JPRB ,&
&  0.14956E+01_JPRB, 0.14393E+01_JPRB, 0.13894E+01_JPRB, 0.13495E+01_JPRB, 0.13302E+01_JPRB, 0.12995E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.10989E+01_JPRB, 0.10806E+01_JPRB, 0.10577E+01_JPRB, 0.10633E+01_JPRB, 0.11071E+01_JPRB, 0.11284E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.12721E+01_JPRB, 0.13070E+01_JPRB, 0.13433E+01_JPRB, 0.13684E+01_JPRB, 0.13979E+01_JPRB, 0.14311E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.12627E+01_JPRB, 0.11990E+01_JPRB, 0.11668E+01_JPRB, 0.12188E+01_JPRB, 0.12984E+01_JPRB, 0.13673E+01_JPRB ,&
&  0.14066E+01_JPRB, 0.14245E+01_JPRB, 0.14384E+01_JPRB, 0.14443E+01_JPRB, 0.14357E+01_JPRB, 0.14318E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.14631E+01_JPRB, 0.14906E+01_JPRB, 0.15268E+01_JPRB, 0.16857E+01_JPRB, 0.29292E+01_JPRB, 0.42410E+01_JPRB, 0.46418E+01_JPRB ,&
&  0.42937E+01_JPRB, 0.42400E+01_JPRB, 0.43461E+01_JPRB, 0.45242E+01_JPRB, 0.44418E+01_JPRB, 0.40859E+01_JPRB, 0.37921E+01_JPRB ,&
&  0.38029E+01_JPRB, 0.36436E+01_JPRB, 0.38854E+01_JPRB, 0.38486E+01_JPRB, 0.38713E+01_JPRB, 0.39294E+01_JPRB, 0.38546E+01_JPRB ,&
&  0.38773E+01_JPRB, 0.38349E+01_JPRB, 0.38014E+01_JPRB, 0.36869E+01_JPRB, 0.36511E+01_JPRB, 0.37098E+01_JPRB, 0.37172E+01_JPRB ,&
&  0.37666E+01_JPRB, 0.37935E+01_JPRB, 0.35756E+01_JPRB, 0.34094E+01_JPRB, 0.32178E+01_JPRB, 0.33541E+01_JPRB, 0.30905E+01_JPRB ,&
&  0.26654E+01_JPRB, 0.21575E+01_JPRB, 0.21604E+01_JPRB, 0.21608E+01_JPRB, 0.21904E+01_JPRB, 0.21444E+01_JPRB, 0.20106E+01_JPRB ,&
&  0.20103E+01_JPRB, 0.22335E+01_JPRB, 0.23276E+01_JPRB, 0.20725E+01_JPRB, 0.19110E+01_JPRB, 0.17242E+01_JPRB, 0.15025E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.16077E+01_JPRB, 0.16534E+01_JPRB, 0.16400E+01_JPRB, 0.16209E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.18020E+01_JPRB, 0.17536E+01_JPRB, 0.17318E+01_JPRB, 0.17333E+01_JPRB, 0.17123E+01_JPRB, 0.16805E+01_JPRB, 0.16394E+01_JPRB ,&
&  0.16026E+01_JPRB, 0.15641E+01_JPRB, 0.15220E+01_JPRB, 0.14831E+01_JPRB, 0.14559E+01_JPRB, 0.14175E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.12111E+01_JPRB, 0.11885E+01_JPRB, 0.12106E+01_JPRB, 0.13487E+01_JPRB, 0.13340E+01_JPRB, 0.13570E+01_JPRB ,&
&  0.13942E+01_JPRB, 0.14342E+01_JPRB, 0.14379E+01_JPRB, 0.14566E+01_JPRB, 0.15138E+01_JPRB, 0.16446E+01_JPRB, 0.18123E+01_JPRB ,&
&  0.17739E+01_JPRB, 0.16208E+01_JPRB, 0.15660E+01_JPRB, 0.15409E+01_JPRB, 0.16021E+01_JPRB, 0.16584E+01_JPRB, 0.17410E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.16923E+01_JPRB, 0.16600E+01_JPRB, 0.16374E+01_JPRB, 0.16228E+01_JPRB, 0.16300E+01_JPRB, 0.16630E+01_JPRB ,&
&  0.16998E+01_JPRB, 0.16946E+01_JPRB, 0.17197E+01_JPRB, 0.20495E+01_JPRB, 0.37182E+01_JPRB, 0.54145E+01_JPRB, 0.62589E+01_JPRB ,&
&  0.63009E+01_JPRB, 0.58727E+01_JPRB, 0.60644E+01_JPRB, 0.67571E+01_JPRB, 0.63165E+01_JPRB, 0.55318E+01_JPRB, 0.48912E+01_JPRB ,&
&  0.46186E+01_JPRB, 0.43856E+01_JPRB, 0.45149E+01_JPRB, 0.46292E+01_JPRB, 0.44492E+01_JPRB, 0.44004E+01_JPRB, 0.43027E+01_JPRB ,&
&  0.42206E+01_JPRB, 0.42340E+01_JPRB, 0.40938E+01_JPRB, 0.39393E+01_JPRB, 0.38245E+01_JPRB, 0.39122E+01_JPRB, 0.38863E+01_JPRB ,&
&  0.40013E+01_JPRB, 0.38841E+01_JPRB, 0.34379E+01_JPRB, 0.33976E+01_JPRB, 0.27570E+01_JPRB, 0.26287E+01_JPRB, 0.30155E+01_JPRB ,&
&  0.26024E+01_JPRB, 0.22970E+01_JPRB, 0.22753E+01_JPRB, 0.23918E+01_JPRB, 0.24778E+01_JPRB, 0.26876E+01_JPRB, 0.26404E+01_JPRB ,&
&  0.22757E+01_JPRB, 0.23871E+01_JPRB, 0.27711E+01_JPRB, 0.24631E+01_JPRB, 0.22196E+01_JPRB, 0.20612E+01_JPRB, 0.18591E+01_JPRB ,&
&  0.18851E+01_JPRB, 0.18728E+01_JPRB, 0.18509E+01_JPRB, 0.18164E+01_JPRB, 0.18057E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.19361E+01_JPRB, 0.17789E+01_JPRB, 0.17701E+01_JPRB, 0.17581E+01_JPRB, 0.17357E+01_JPRB, 0.17077E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.16895E+01_JPRB, 0.16759E+01_JPRB, 0.16446E+01_JPRB, 0.16045E+01_JPRB, 0.15698E+01_JPRB, 0.15252E+01_JPRB, 0.14740E+01_JPRB ,&
&  0.14398E+01_JPRB, 0.14670E+01_JPRB, 0.14634E+01_JPRB, 0.14033E+01_JPRB, 0.15043E+01_JPRB, 0.14987E+01_JPRB, 0.15337E+01_JPRB ,&
&  0.15857E+01_JPRB, 0.16722E+01_JPRB, 0.16938E+01_JPRB, 0.17341E+01_JPRB, 0.18277E+01_JPRB, 0.20473E+01_JPRB, 0.23258E+01_JPRB ,&
&  0.24240E+01_JPRB, 0.23193E+01_JPRB, 0.22787E+01_JPRB, 0.23518E+01_JPRB, 0.24639E+01_JPRB, 0.22104E+01_JPRB, 0.21462E+01_JPRB ,&
&  0.20145E+01_JPRB, 0.19045E+01_JPRB, 0.18286E+01_JPRB, 0.17899E+01_JPRB, 0.18028E+01_JPRB, 0.18449E+01_JPRB, 0.18827E+01_JPRB ,&
&  0.18829E+01_JPRB, 0.18148E+01_JPRB, 0.18781E+01_JPRB, 0.25333E+01_JPRB, 0.42708E+01_JPRB, 0.59222E+01_JPRB, 0.66661E+01_JPRB ,&
&  0.67905E+01_JPRB, 0.66883E+01_JPRB, 0.68637E+01_JPRB, 0.75711E+01_JPRB, 0.70919E+01_JPRB, 0.63236E+01_JPRB, 0.58828E+01_JPRB ,&
&  0.55922E+01_JPRB, 0.54187E+01_JPRB, 0.54395E+01_JPRB, 0.50720E+01_JPRB, 0.46001E+01_JPRB, 0.43108E+01_JPRB, 0.40477E+01_JPRB ,&
&  0.38340E+01_JPRB, 0.38307E+01_JPRB, 0.38385E+01_JPRB, 0.36385E+01_JPRB, 0.35272E+01_JPRB, 0.33678E+01_JPRB, 0.35800E+01_JPRB ,&
&  0.39601E+01_JPRB, 0.30640E+01_JPRB, 0.25730E+01_JPRB, 0.28456E+01_JPRB, 0.22203E+01_JPRB, 0.21609E+01_JPRB, 0.24301E+01_JPRB ,&
&  0.23059E+01_JPRB, 0.23735E+01_JPRB, 0.27297E+01_JPRB, 0.27304E+01_JPRB, 0.29079E+01_JPRB, 0.32646E+01_JPRB, 0.33090E+01_JPRB ,&
&  0.30569E+01_JPRB, 0.28554E+01_JPRB, 0.34522E+01_JPRB, 0.30639E+01_JPRB, 0.27075E+01_JPRB, 0.25430E+01_JPRB, 0.24020E+01_JPRB ,&
&  0.23127E+01_JPRB, 0.22282E+01_JPRB, 0.21278E+01_JPRB, 0.20536E+01_JPRB, 0.20242E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.19139E+01_JPRB, 0.17649E+01_JPRB, 0.18254E+01_JPRB, 0.18352E+01_JPRB, 0.18249E+01_JPRB, 0.18158E+01_JPRB, 0.18195E+01_JPRB ,&
&  0.18047E+01_JPRB, 0.17858E+01_JPRB, 0.17399E+01_JPRB, 0.16656E+01_JPRB, 0.16119E+01_JPRB, 0.15662E+01_JPRB, 0.15311E+01_JPRB ,&
&  0.14973E+01_JPRB, 0.16004E+01_JPRB, 0.15387E+01_JPRB, 0.15295E+01_JPRB, 0.14885E+01_JPRB, 0.15689E+01_JPRB, 0.16509E+01_JPRB ,&
&  0.17931E+01_JPRB, 0.19218E+01_JPRB, 0.21024E+01_JPRB, 0.23626E+01_JPRB, 0.26320E+01_JPRB, 0.28981E+01_JPRB, 0.33164E+01_JPRB ,&
&  0.34742E+01_JPRB, 0.33607E+01_JPRB, 0.31508E+01_JPRB, 0.31693E+01_JPRB, 0.30434E+01_JPRB, 0.26756E+01_JPRB, 0.25344E+01_JPRB ,&
&  0.23187E+01_JPRB, 0.21751E+01_JPRB, 0.20737E+01_JPRB, 0.20356E+01_JPRB, 0.19906E+01_JPRB, 0.18991E+01_JPRB, 0.18286E+01_JPRB ,&
&  0.17763E+01_JPRB, 0.18563E+01_JPRB, 0.22789E+01_JPRB, 0.32588E+01_JPRB, 0.48760E+01_JPRB, 0.63844E+01_JPRB, 0.67228E+01_JPRB ,&
&  0.70165E+01_JPRB, 0.66777E+01_JPRB, 0.54107E+01_JPRB, 0.61237E+01_JPRB, 0.76710E+01_JPRB, 0.72621E+01_JPRB, 0.64872E+01_JPRB ,&
&  0.63672E+01_JPRB, 0.58655E+01_JPRB, 0.55845E+01_JPRB, 0.49430E+01_JPRB, 0.43396E+01_JPRB, 0.39514E+01_JPRB, 0.36334E+01_JPRB ,&
&  0.33986E+01_JPRB, 0.33513E+01_JPRB, 0.33618E+01_JPRB, 0.33905E+01_JPRB, 0.32914E+01_JPRB, 0.34040E+01_JPRB, 0.37623E+01_JPRB ,&
&  0.37721E+01_JPRB, 0.35246E+01_JPRB, 0.27365E+01_JPRB, 0.26825E+01_JPRB, 0.20908E+01_JPRB, 0.20399E+01_JPRB, 0.22880E+01_JPRB ,&
&  0.23233E+01_JPRB, 0.28812E+01_JPRB, 0.36374E+01_JPRB, 0.37585E+01_JPRB, 0.43907E+01_JPRB, 0.45208E+01_JPRB, 0.44713E+01_JPRB ,&
&  0.42135E+01_JPRB, 0.37669E+01_JPRB, 0.43245E+01_JPRB, 0.36928E+01_JPRB, 0.31378E+01_JPRB, 0.29033E+01_JPRB, 0.27483E+01_JPRB ,&
&  0.26347E+01_JPRB, 0.24740E+01_JPRB, 0.22963E+01_JPRB, 0.21705E+01_JPRB, 0.20628E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.19107E+01_JPRB, 0.18182E+01_JPRB, 0.18783E+01_JPRB, 0.18569E+01_JPRB, 0.18812E+01_JPRB, 0.18649E+01_JPRB, 0.18845E+01_JPRB ,&
&  0.18694E+01_JPRB, 0.18090E+01_JPRB, 0.17280E+01_JPRB, 0.16568E+01_JPRB, 0.16050E+01_JPRB, 0.15584E+01_JPRB, 0.15350E+01_JPRB ,&
&  0.15171E+01_JPRB, 0.15590E+01_JPRB, 0.14462E+01_JPRB, 0.13819E+01_JPRB, 0.13603E+01_JPRB, 0.14884E+01_JPRB, 0.17117E+01_JPRB ,&
&  0.19734E+01_JPRB, 0.22263E+01_JPRB, 0.26577E+01_JPRB, 0.30459E+01_JPRB, 0.34352E+01_JPRB, 0.38140E+01_JPRB, 0.43239E+01_JPRB ,&
&  0.47598E+01_JPRB, 0.40591E+01_JPRB, 0.37772E+01_JPRB, 0.33411E+01_JPRB, 0.30251E+01_JPRB, 0.27554E+01_JPRB, 0.25546E+01_JPRB ,&
&  0.23998E+01_JPRB, 0.23030E+01_JPRB, 0.21796E+01_JPRB, 0.20192E+01_JPRB, 0.18659E+01_JPRB, 0.17858E+01_JPRB, 0.17769E+01_JPRB ,&
&  0.18622E+01_JPRB, 0.21939E+01_JPRB, 0.30897E+01_JPRB, 0.43806E+01_JPRB, 0.63464E+01_JPRB, 0.74944E+01_JPRB, 0.72949E+01_JPRB ,&
&  0.61702E+01_JPRB, 0.60639E+01_JPRB, 0.86019E+01_JPRB, 0.73787E+01_JPRB, 0.75251E+01_JPRB, 0.79890E+01_JPRB, 0.72958E+01_JPRB ,&
&  0.63044E+01_JPRB, 0.52117E+01_JPRB, 0.49639E+01_JPRB, 0.44931E+01_JPRB, 0.51728E+01_JPRB, 0.44724E+01_JPRB, 0.38942E+01_JPRB ,&
&  0.35563E+01_JPRB, 0.35312E+01_JPRB, 0.34256E+01_JPRB, 0.34445E+01_JPRB, 0.36649E+01_JPRB, 0.42306E+01_JPRB, 0.43276E+01_JPRB ,&
&  0.39515E+01_JPRB, 0.40771E+01_JPRB, 0.34277E+01_JPRB, 0.28435E+01_JPRB, 0.24799E+01_JPRB, 0.24995E+01_JPRB, 0.26174E+01_JPRB ,&
&  0.29872E+01_JPRB, 0.35400E+01_JPRB, 0.46765E+01_JPRB, 0.57539E+01_JPRB, 0.65820E+01_JPRB, 0.63586E+01_JPRB, 0.55765E+01_JPRB ,&
&  0.51314E+01_JPRB, 0.57091E+01_JPRB, 0.55682E+01_JPRB, 0.42900E+01_JPRB, 0.35061E+01_JPRB, 0.32236E+01_JPRB, 0.30136E+01_JPRB ,&
&  0.27347E+01_JPRB, 0.24430E+01_JPRB, 0.22312E+01_JPRB, 0.21199E+01_JPRB, 0.20551E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.20019E+01_JPRB, 0.19189E+01_JPRB, 0.18347E+01_JPRB, 0.18133E+01_JPRB, 0.18615E+01_JPRB, 0.18380E+01_JPRB, 0.18071E+01_JPRB ,&
&  0.17457E+01_JPRB, 0.16727E+01_JPRB, 0.16440E+01_JPRB, 0.16287E+01_JPRB, 0.16004E+01_JPRB, 0.15608E+01_JPRB, 0.15371E+01_JPRB ,&
&  0.15209E+01_JPRB, 0.14728E+01_JPRB, 0.13636E+01_JPRB, 0.13819E+01_JPRB, 0.13925E+01_JPRB, 0.14314E+01_JPRB, 0.17260E+01_JPRB ,&
&  0.21336E+01_JPRB, 0.26281E+01_JPRB, 0.31558E+01_JPRB, 0.36304E+01_JPRB, 0.39656E+01_JPRB, 0.43671E+01_JPRB, 0.48610E+01_JPRB ,&
&  0.45871E+01_JPRB, 0.44524E+01_JPRB, 0.39764E+01_JPRB, 0.34606E+01_JPRB, 0.30405E+01_JPRB, 0.27110E+01_JPRB, 0.25356E+01_JPRB ,&
&  0.24035E+01_JPRB, 0.23300E+01_JPRB, 0.21502E+01_JPRB, 0.19402E+01_JPRB, 0.18668E+01_JPRB, 0.18548E+01_JPRB, 0.18842E+01_JPRB ,&
&  0.20291E+01_JPRB, 0.27018E+01_JPRB, 0.38010E+01_JPRB, 0.51610E+01_JPRB, 0.51888E+01_JPRB, 0.60328E+01_JPRB, 0.61996E+01_JPRB ,&
&  0.79734E+01_JPRB, 0.81553E+01_JPRB, 0.79417E+01_JPRB, 0.85097E+01_JPRB, 0.70901E+01_JPRB, 0.72625E+01_JPRB, 0.70945E+01_JPRB ,&
&  0.62158E+01_JPRB, 0.57653E+01_JPRB, 0.61763E+01_JPRB, 0.52483E+01_JPRB, 0.49655E+01_JPRB, 0.56308E+01_JPRB, 0.48871E+01_JPRB ,&
&  0.45087E+01_JPRB, 0.42799E+01_JPRB, 0.39753E+01_JPRB, 0.41910E+01_JPRB, 0.40348E+01_JPRB, 0.41226E+01_JPRB, 0.35673E+01_JPRB ,&
&  0.35161E+01_JPRB, 0.30184E+01_JPRB, 0.33325E+01_JPRB, 0.30640E+01_JPRB, 0.30832E+01_JPRB, 0.30832E+01_JPRB, 0.32168E+01_JPRB ,&
&  0.39357E+01_JPRB, 0.46244E+01_JPRB, 0.62968E+01_JPRB, 0.79629E+01_JPRB, 0.91473E+01_JPRB, 0.75590E+01_JPRB, 0.68399E+01_JPRB ,&
&  0.70995E+01_JPRB, 0.71584E+01_JPRB, 0.58236E+01_JPRB, 0.45168E+01_JPRB, 0.37425E+01_JPRB, 0.33962E+01_JPRB, 0.31635E+01_JPRB ,&
&  0.29081E+01_JPRB, 0.26572E+01_JPRB, 0.24541E+01_JPRB, 0.23067E+01_JPRB, 0.21867E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.18546E+01_JPRB, 0.18141E+01_JPRB, 0.17040E+01_JPRB, 0.17189E+01_JPRB, 0.17906E+01_JPRB, 0.17618E+01_JPRB, 0.17181E+01_JPRB ,&
&  0.16498E+01_JPRB, 0.16018E+01_JPRB, 0.16138E+01_JPRB, 0.16137E+01_JPRB, 0.15910E+01_JPRB, 0.15627E+01_JPRB, 0.15205E+01_JPRB ,&
&  0.14980E+01_JPRB, 0.14707E+01_JPRB, 0.13755E+01_JPRB, 0.13586E+01_JPRB, 0.14460E+01_JPRB, 0.15078E+01_JPRB, 0.17215E+01_JPRB ,&
&  0.24390E+01_JPRB, 0.30807E+01_JPRB, 0.35069E+01_JPRB, 0.39107E+01_JPRB, 0.41827E+01_JPRB, 0.44525E+01_JPRB, 0.45501E+01_JPRB ,&
&  0.43570E+01_JPRB, 0.37511E+01_JPRB, 0.33834E+01_JPRB, 0.31161E+01_JPRB, 0.28792E+01_JPRB, 0.26817E+01_JPRB, 0.25083E+01_JPRB ,&
&  0.23453E+01_JPRB, 0.20754E+01_JPRB, 0.18070E+01_JPRB, 0.16609E+01_JPRB, 0.16768E+01_JPRB, 0.16845E+01_JPRB, 0.18117E+01_JPRB ,&
&  0.23101E+01_JPRB, 0.29932E+01_JPRB, 0.38164E+01_JPRB, 0.47220E+01_JPRB, 0.48565E+01_JPRB, 0.55292E+01_JPRB, 0.66582E+01_JPRB ,&
&  0.70054E+01_JPRB, 0.68705E+01_JPRB, 0.79057E+01_JPRB, 0.88429E+01_JPRB, 0.87889E+01_JPRB, 0.75632E+01_JPRB, 0.72240E+01_JPRB ,&
&  0.53895E+01_JPRB, 0.43514E+01_JPRB, 0.39784E+01_JPRB, 0.39890E+01_JPRB, 0.49898E+01_JPRB, 0.65300E+01_JPRB, 0.61635E+01_JPRB ,&
&  0.59208E+01_JPRB, 0.55789E+01_JPRB, 0.51384E+01_JPRB, 0.44858E+01_JPRB, 0.41806E+01_JPRB, 0.38757E+01_JPRB, 0.49049E+01_JPRB ,&
&  0.49115E+01_JPRB, 0.48958E+01_JPRB, 0.42069E+01_JPRB, 0.35081E+01_JPRB, 0.30927E+01_JPRB, 0.36884E+01_JPRB, 0.42309E+01_JPRB ,&
&  0.50626E+01_JPRB, 0.60948E+01_JPRB, 0.10846E+02_JPRB, 0.11832E+02_JPRB, 0.10706E+02_JPRB, 0.92767E+01_JPRB, 0.93846E+01_JPRB ,&
&  0.83119E+01_JPRB, 0.79278E+01_JPRB, 0.63870E+01_JPRB, 0.46287E+01_JPRB, 0.35630E+01_JPRB, 0.29859E+01_JPRB, 0.26802E+01_JPRB ,&
&  0.25798E+01_JPRB, 0.24871E+01_JPRB, 0.23365E+01_JPRB, 0.21495E+01_JPRB, 0.20088E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.17246E+01_JPRB, 0.16911E+01_JPRB, 0.15689E+01_JPRB, 0.15624E+01_JPRB, 0.16262E+01_JPRB, 0.16719E+01_JPRB, 0.16589E+01_JPRB ,&
&  0.16327E+01_JPRB, 0.16337E+01_JPRB, 0.16507E+01_JPRB, 0.16227E+01_JPRB, 0.15823E+01_JPRB, 0.15568E+01_JPRB, 0.15201E+01_JPRB ,&
&  0.15210E+01_JPRB, 0.16189E+01_JPRB, 0.14859E+01_JPRB, 0.15071E+01_JPRB, 0.16340E+01_JPRB, 0.17976E+01_JPRB, 0.20580E+01_JPRB ,&
&  0.29692E+01_JPRB, 0.33175E+01_JPRB, 0.33155E+01_JPRB, 0.36519E+01_JPRB, 0.42738E+01_JPRB, 0.39982E+01_JPRB, 0.37686E+01_JPRB ,&
&  0.33994E+01_JPRB, 0.29505E+01_JPRB, 0.27659E+01_JPRB, 0.26379E+01_JPRB, 0.26094E+01_JPRB, 0.25869E+01_JPRB, 0.24492E+01_JPRB ,&
&  0.21828E+01_JPRB, 0.18491E+01_JPRB, 0.16604E+01_JPRB, 0.16450E+01_JPRB, 0.15827E+01_JPRB, 0.16213E+01_JPRB, 0.17810E+01_JPRB ,&
&  0.23570E+01_JPRB, 0.30654E+01_JPRB, 0.35120E+01_JPRB, 0.39844E+01_JPRB, 0.42973E+01_JPRB, 0.50460E+01_JPRB, 0.72304E+01_JPRB ,&
&  0.70142E+01_JPRB, 0.72532E+01_JPRB, 0.77250E+01_JPRB, 0.91118E+01_JPRB, 0.82758E+01_JPRB, 0.71693E+01_JPRB, 0.66979E+01_JPRB ,&
&  0.49387E+01_JPRB, 0.41762E+01_JPRB, 0.44881E+01_JPRB, 0.48498E+01_JPRB, 0.45555E+01_JPRB, 0.50265E+01_JPRB, 0.66220E+01_JPRB ,&
&  0.56784E+01_JPRB, 0.57188E+01_JPRB, 0.56296E+01_JPRB, 0.54685E+01_JPRB, 0.33250E+01_JPRB, 0.27438E+01_JPRB, 0.40887E+01_JPRB ,&
&  0.44032E+01_JPRB, 0.28822E+01_JPRB, 0.19505E+01_JPRB, 0.25862E+01_JPRB, 0.20016E+01_JPRB, 0.25448E+01_JPRB, 0.43002E+01_JPRB ,&
&  0.62790E+01_JPRB, 0.88948E+01_JPRB, 0.15711E+02_JPRB, 0.13307E+02_JPRB, 0.11604E+02_JPRB, 0.11848E+02_JPRB, 0.10357E+02_JPRB ,&
&  0.10017E+02_JPRB, 0.71910E+01_JPRB, 0.56364E+01_JPRB, 0.38353E+01_JPRB, 0.26953E+01_JPRB, 0.22140E+01_JPRB, 0.20557E+01_JPRB ,&
&  0.19947E+01_JPRB, 0.20052E+01_JPRB, 0.20074E+01_JPRB, 0.19173E+01_JPRB, 0.18080E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.17793E+01_JPRB, 0.16503E+01_JPRB, 0.14573E+01_JPRB, 0.14314E+01_JPRB, 0.14770E+01_JPRB, 0.15426E+01_JPRB, 0.15522E+01_JPRB ,&
&  0.15778E+01_JPRB, 0.16304E+01_JPRB, 0.17114E+01_JPRB, 0.16899E+01_JPRB, 0.16187E+01_JPRB, 0.15883E+01_JPRB, 0.15420E+01_JPRB ,&
&  0.15479E+01_JPRB, 0.16625E+01_JPRB, 0.18385E+01_JPRB, 0.19981E+01_JPRB, 0.22135E+01_JPRB, 0.23494E+01_JPRB, 0.29351E+01_JPRB ,&
&  0.34240E+01_JPRB, 0.32595E+01_JPRB, 0.32121E+01_JPRB, 0.33992E+01_JPRB, 0.35451E+01_JPRB, 0.31167E+01_JPRB, 0.30716E+01_JPRB ,&
&  0.27477E+01_JPRB, 0.24554E+01_JPRB, 0.23455E+01_JPRB, 0.22948E+01_JPRB, 0.23730E+01_JPRB, 0.24237E+01_JPRB, 0.23415E+01_JPRB ,&
&  0.20734E+01_JPRB, 0.18302E+01_JPRB, 0.17002E+01_JPRB, 0.15846E+01_JPRB, 0.15167E+01_JPRB, 0.15623E+01_JPRB, 0.17873E+01_JPRB ,&
&  0.23076E+01_JPRB, 0.28948E+01_JPRB, 0.31503E+01_JPRB, 0.33321E+01_JPRB, 0.44341E+01_JPRB, 0.48888E+01_JPRB, 0.52053E+01_JPRB ,&
&  0.54566E+01_JPRB, 0.57412E+01_JPRB, 0.77471E+01_JPRB, 0.83392E+01_JPRB, 0.79719E+01_JPRB, 0.76375E+01_JPRB, 0.67173E+01_JPRB ,&
&  0.59598E+01_JPRB, 0.53960E+01_JPRB, 0.48742E+01_JPRB, 0.51565E+01_JPRB, 0.50869E+01_JPRB, 0.46905E+01_JPRB, 0.55540E+01_JPRB ,&
&  0.58602E+01_JPRB, 0.56178E+01_JPRB, 0.46208E+01_JPRB, 0.37212E+01_JPRB, 0.35119E+01_JPRB, 0.27460E+01_JPRB, 0.20148E+01_JPRB ,&
&  0.18348E+01_JPRB, 0.17881E+01_JPRB, 0.16247E+01_JPRB, 0.17023E+01_JPRB, 0.16210E+01_JPRB, 0.22756E+01_JPRB, 0.38774E+01_JPRB ,&
&  0.83808E+01_JPRB, 0.13401E+02_JPRB, 0.15803E+02_JPRB, 0.12693E+02_JPRB, 0.10908E+02_JPRB, 0.11547E+02_JPRB, 0.11939E+02_JPRB ,&
&  0.76188E+01_JPRB, 0.57060E+01_JPRB, 0.38232E+01_JPRB, 0.26348E+01_JPRB, 0.21082E+01_JPRB, 0.18481E+01_JPRB, 0.17671E+01_JPRB ,&
&  0.17457E+01_JPRB, 0.17155E+01_JPRB, 0.17365E+01_JPRB, 0.17717E+01_JPRB, 0.17520E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.17486E+01_JPRB, 0.15486E+01_JPRB, 0.13396E+01_JPRB, 0.12898E+01_JPRB, 0.13719E+01_JPRB, 0.14096E+01_JPRB, 0.14560E+01_JPRB ,&
&  0.15361E+01_JPRB, 0.16592E+01_JPRB, 0.17419E+01_JPRB, 0.17202E+01_JPRB, 0.16653E+01_JPRB, 0.16171E+01_JPRB, 0.15915E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.18634E+01_JPRB, 0.23068E+01_JPRB, 0.27516E+01_JPRB, 0.32266E+01_JPRB, 0.31774E+01_JPRB, 0.36872E+01_JPRB ,&
&  0.33861E+01_JPRB, 0.31087E+01_JPRB, 0.33209E+01_JPRB, 0.32871E+01_JPRB, 0.32493E+01_JPRB, 0.27410E+01_JPRB, 0.25178E+01_JPRB ,&
&  0.22503E+01_JPRB, 0.21871E+01_JPRB, 0.21470E+01_JPRB, 0.22190E+01_JPRB, 0.22953E+01_JPRB, 0.22901E+01_JPRB, 0.21684E+01_JPRB ,&
&  0.19304E+01_JPRB, 0.18286E+01_JPRB, 0.16545E+01_JPRB, 0.16366E+01_JPRB, 0.16236E+01_JPRB, 0.17218E+01_JPRB, 0.19488E+01_JPRB ,&
&  0.23565E+01_JPRB, 0.27633E+01_JPRB, 0.29558E+01_JPRB, 0.33956E+01_JPRB, 0.36085E+01_JPRB, 0.35728E+01_JPRB, 0.47217E+01_JPRB ,&
&  0.57912E+01_JPRB, 0.66031E+01_JPRB, 0.68330E+01_JPRB, 0.81357E+01_JPRB, 0.79281E+01_JPRB, 0.72009E+01_JPRB, 0.67091E+01_JPRB ,&
&  0.58558E+01_JPRB, 0.55400E+01_JPRB, 0.54415E+01_JPRB, 0.56864E+01_JPRB, 0.63672E+01_JPRB, 0.57052E+01_JPRB, 0.57340E+01_JPRB ,&
&  0.62143E+01_JPRB, 0.58584E+01_JPRB, 0.50178E+01_JPRB, 0.41974E+01_JPRB, 0.48179E+01_JPRB, 0.46700E+01_JPRB, 0.27471E+01_JPRB ,&
&  0.23592E+01_JPRB, 0.21195E+01_JPRB, 0.18080E+01_JPRB, 0.16103E+01_JPRB, 0.17153E+01_JPRB, 0.24742E+01_JPRB, 0.52837E+01_JPRB ,&
&  0.75318E+01_JPRB, 0.13999E+02_JPRB, 0.16481E+02_JPRB, 0.13534E+02_JPRB, 0.10646E+02_JPRB, 0.11256E+02_JPRB, 0.81400E+01_JPRB ,&
&  0.66479E+01_JPRB, 0.46872E+01_JPRB, 0.26652E+01_JPRB, 0.20100E+01_JPRB, 0.18804E+01_JPRB, 0.17808E+01_JPRB, 0.16791E+01_JPRB ,&
&  0.15698E+01_JPRB, 0.15403E+01_JPRB, 0.15797E+01_JPRB, 0.16208E+01_JPRB, 0.16358E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.15212E+01_JPRB, 0.13567E+01_JPRB, 0.12314E+01_JPRB, 0.11592E+01_JPRB, 0.12514E+01_JPRB, 0.13754E+01_JPRB, 0.14203E+01_JPRB ,&
&  0.15583E+01_JPRB, 0.17253E+01_JPRB, 0.18057E+01_JPRB, 0.17995E+01_JPRB, 0.17138E+01_JPRB, 0.16934E+01_JPRB, 0.17706E+01_JPRB ,&
&  0.19891E+01_JPRB, 0.23923E+01_JPRB, 0.30194E+01_JPRB, 0.37031E+01_JPRB, 0.40185E+01_JPRB, 0.41116E+01_JPRB, 0.39110E+01_JPRB ,&
&  0.31569E+01_JPRB, 0.29911E+01_JPRB, 0.33107E+01_JPRB, 0.33698E+01_JPRB, 0.27878E+01_JPRB, 0.25264E+01_JPRB, 0.20985E+01_JPRB ,&
&  0.20690E+01_JPRB, 0.21148E+01_JPRB, 0.21158E+01_JPRB, 0.21413E+01_JPRB, 0.21299E+01_JPRB, 0.20867E+01_JPRB, 0.20278E+01_JPRB ,&
&  0.18822E+01_JPRB, 0.17548E+01_JPRB, 0.16851E+01_JPRB, 0.16824E+01_JPRB, 0.18157E+01_JPRB, 0.20316E+01_JPRB, 0.22620E+01_JPRB ,&
&  0.25483E+01_JPRB, 0.27978E+01_JPRB, 0.30462E+01_JPRB, 0.35333E+01_JPRB, 0.30232E+01_JPRB, 0.39838E+01_JPRB, 0.47420E+01_JPRB ,&
&  0.52292E+01_JPRB, 0.57144E+01_JPRB, 0.57789E+01_JPRB, 0.67485E+01_JPRB, 0.72898E+01_JPRB, 0.68342E+01_JPRB, 0.61084E+01_JPRB ,&
&  0.57340E+01_JPRB, 0.53133E+01_JPRB, 0.61548E+01_JPRB, 0.67493E+01_JPRB, 0.72160E+01_JPRB, 0.86043E+01_JPRB, 0.64152E+01_JPRB ,&
&  0.56834E+01_JPRB, 0.62007E+01_JPRB, 0.55267E+01_JPRB, 0.48563E+01_JPRB, 0.52147E+01_JPRB, 0.63493E+01_JPRB, 0.73136E+01_JPRB ,&
&  0.34711E+01_JPRB, 0.22610E+01_JPRB, 0.22923E+01_JPRB, 0.20983E+01_JPRB, 0.18550E+01_JPRB, 0.27101E+01_JPRB, 0.92179E+01_JPRB ,&
&  0.77159E+01_JPRB, 0.12900E+02_JPRB, 0.14448E+02_JPRB, 0.12064E+02_JPRB, 0.10116E+02_JPRB, 0.87655E+01_JPRB, 0.70492E+01_JPRB ,&
&  0.49663E+01_JPRB, 0.33884E+01_JPRB, 0.22674E+01_JPRB, 0.18654E+01_JPRB, 0.17711E+01_JPRB, 0.17057E+01_JPRB, 0.16214E+01_JPRB ,&
&  0.15442E+01_JPRB, 0.14774E+01_JPRB, 0.14940E+01_JPRB, 0.14496E+01_JPRB, 0.15339E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.13756E+01_JPRB, 0.12785E+01_JPRB, 0.12680E+01_JPRB, 0.12691E+01_JPRB, 0.13359E+01_JPRB, 0.14013E+01_JPRB, 0.15067E+01_JPRB ,&
&  0.16853E+01_JPRB, 0.18685E+01_JPRB, 0.19998E+01_JPRB, 0.19999E+01_JPRB, 0.19355E+01_JPRB, 0.20366E+01_JPRB, 0.22931E+01_JPRB ,&
&  0.26685E+01_JPRB, 0.31598E+01_JPRB, 0.37526E+01_JPRB, 0.41722E+01_JPRB, 0.43862E+01_JPRB, 0.39982E+01_JPRB, 0.35189E+01_JPRB ,&
&  0.29158E+01_JPRB, 0.27115E+01_JPRB, 0.26091E+01_JPRB, 0.26456E+01_JPRB, 0.25951E+01_JPRB, 0.24174E+01_JPRB, 0.19437E+01_JPRB ,&
&  0.19162E+01_JPRB, 0.19015E+01_JPRB, 0.18671E+01_JPRB, 0.18823E+01_JPRB, 0.18999E+01_JPRB, 0.19471E+01_JPRB, 0.19168E+01_JPRB ,&
&  0.17833E+01_JPRB, 0.17478E+01_JPRB, 0.17474E+01_JPRB, 0.18480E+01_JPRB, 0.20936E+01_JPRB, 0.23336E+01_JPRB, 0.25674E+01_JPRB ,&
&  0.27960E+01_JPRB, 0.29616E+01_JPRB, 0.31644E+01_JPRB, 0.32612E+01_JPRB, 0.36130E+01_JPRB, 0.41027E+01_JPRB, 0.45099E+01_JPRB ,&
&  0.47732E+01_JPRB, 0.53100E+01_JPRB, 0.57765E+01_JPRB, 0.61167E+01_JPRB, 0.64703E+01_JPRB, 0.63249E+01_JPRB, 0.58119E+01_JPRB ,&
&  0.57327E+01_JPRB, 0.61475E+01_JPRB, 0.67446E+01_JPRB, 0.71098E+01_JPRB, 0.74664E+01_JPRB, 0.86328E+01_JPRB, 0.83975E+01_JPRB ,&
&  0.68948E+01_JPRB, 0.55426E+01_JPRB, 0.52252E+01_JPRB, 0.57827E+01_JPRB, 0.63550E+01_JPRB, 0.77141E+01_JPRB, 0.11928E+02_JPRB ,&
&  0.12220E+02_JPRB, 0.56058E+01_JPRB, 0.37334E+01_JPRB, 0.43101E+01_JPRB, 0.27846E+01_JPRB, 0.29182E+01_JPRB, 0.55506E+01_JPRB ,&
&  0.78025E+01_JPRB, 0.11350E+02_JPRB, 0.12759E+02_JPRB, 0.99156E+01_JPRB, 0.84166E+01_JPRB, 0.61618E+01_JPRB, 0.46862E+01_JPRB ,&
&  0.34979E+01_JPRB, 0.24954E+01_JPRB, 0.18850E+01_JPRB, 0.16974E+01_JPRB, 0.15654E+01_JPRB, 0.15650E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.15403E+01_JPRB, 0.14149E+01_JPRB, 0.14141E+01_JPRB, 0.14105E+01_JPRB, 0.13827E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.13922E+01_JPRB, 0.13663E+01_JPRB, 0.14099E+01_JPRB, 0.14689E+01_JPRB, 0.15434E+01_JPRB, 0.16341E+01_JPRB, 0.17371E+01_JPRB ,&
&  0.19339E+01_JPRB, 0.21490E+01_JPRB, 0.22869E+01_JPRB, 0.23721E+01_JPRB, 0.24773E+01_JPRB, 0.27036E+01_JPRB, 0.30106E+01_JPRB ,&
&  0.33773E+01_JPRB, 0.37604E+01_JPRB, 0.41468E+01_JPRB, 0.45557E+01_JPRB, 0.50661E+01_JPRB, 0.52742E+01_JPRB, 0.31236E+01_JPRB ,&
&  0.25964E+01_JPRB, 0.27433E+01_JPRB, 0.24009E+01_JPRB, 0.24779E+01_JPRB, 0.25033E+01_JPRB, 0.24628E+01_JPRB, 0.20045E+01_JPRB ,&
&  0.17732E+01_JPRB, 0.17273E+01_JPRB, 0.17163E+01_JPRB, 0.17463E+01_JPRB, 0.17770E+01_JPRB, 0.18925E+01_JPRB, 0.19235E+01_JPRB ,&
&  0.19364E+01_JPRB, 0.20150E+01_JPRB, 0.21119E+01_JPRB, 0.23336E+01_JPRB, 0.26000E+01_JPRB, 0.28021E+01_JPRB, 0.29293E+01_JPRB ,&
&  0.30185E+01_JPRB, 0.31215E+01_JPRB, 0.33004E+01_JPRB, 0.34446E+01_JPRB, 0.38761E+01_JPRB, 0.41559E+01_JPRB, 0.44753E+01_JPRB ,&
&  0.45719E+01_JPRB, 0.45463E+01_JPRB, 0.51577E+01_JPRB, 0.56383E+01_JPRB, 0.58515E+01_JPRB, 0.56324E+01_JPRB, 0.54534E+01_JPRB ,&
&  0.62098E+01_JPRB, 0.71053E+01_JPRB, 0.80981E+01_JPRB, 0.73181E+01_JPRB, 0.71152E+01_JPRB, 0.78113E+01_JPRB, 0.85029E+01_JPRB ,&
&  0.75616E+01_JPRB, 0.66336E+01_JPRB, 0.67007E+01_JPRB, 0.69824E+01_JPRB, 0.75331E+01_JPRB, 0.82300E+01_JPRB, 0.10686E+02_JPRB ,&
&  0.12587E+02_JPRB, 0.98612E+01_JPRB, 0.58513E+01_JPRB, 0.34733E+01_JPRB, 0.31810E+01_JPRB, 0.37405E+01_JPRB, 0.49748E+01_JPRB ,&
&  0.80960E+01_JPRB, 0.94674E+01_JPRB, 0.10212E+02_JPRB, 0.97489E+01_JPRB, 0.53765E+01_JPRB, 0.35995E+01_JPRB, 0.28486E+01_JPRB ,&
&  0.19937E+01_JPRB, 0.15729E+01_JPRB, 0.15199E+01_JPRB, 0.14545E+01_JPRB, 0.13807E+01_JPRB, 0.14527E+01_JPRB, 0.16125E+01_JPRB ,&
&  0.15349E+01_JPRB, 0.13524E+01_JPRB, 0.13607E+01_JPRB, 0.13531E+01_JPRB, 0.13596E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.14296E+01_JPRB, 0.14537E+01_JPRB, 0.15235E+01_JPRB, 0.16449E+01_JPRB, 0.18076E+01_JPRB, 0.19867E+01_JPRB, 0.20698E+01_JPRB ,&
&  0.21789E+01_JPRB, 0.23627E+01_JPRB, 0.25427E+01_JPRB, 0.27209E+01_JPRB, 0.29055E+01_JPRB, 0.31327E+01_JPRB, 0.33806E+01_JPRB ,&
&  0.36453E+01_JPRB, 0.39039E+01_JPRB, 0.41637E+01_JPRB, 0.44895E+01_JPRB, 0.48811E+01_JPRB, 0.50552E+01_JPRB, 0.37347E+01_JPRB ,&
&  0.25435E+01_JPRB, 0.35996E+01_JPRB, 0.25056E+01_JPRB, 0.26973E+01_JPRB, 0.28362E+01_JPRB, 0.30678E+01_JPRB, 0.24368E+01_JPRB ,&
&  0.18518E+01_JPRB, 0.17860E+01_JPRB, 0.17998E+01_JPRB, 0.17567E+01_JPRB, 0.16827E+01_JPRB, 0.18375E+01_JPRB, 0.20376E+01_JPRB ,&
&  0.21679E+01_JPRB, 0.23332E+01_JPRB, 0.25398E+01_JPRB, 0.27812E+01_JPRB, 0.29320E+01_JPRB, 0.30575E+01_JPRB, 0.31561E+01_JPRB ,&
&  0.32581E+01_JPRB, 0.33967E+01_JPRB, 0.34988E+01_JPRB, 0.37399E+01_JPRB, 0.39325E+01_JPRB, 0.42365E+01_JPRB, 0.45234E+01_JPRB ,&
&  0.45448E+01_JPRB, 0.44833E+01_JPRB, 0.47975E+01_JPRB, 0.48676E+01_JPRB, 0.50451E+01_JPRB, 0.56781E+01_JPRB, 0.57531E+01_JPRB ,&
&  0.68195E+01_JPRB, 0.71571E+01_JPRB, 0.83033E+01_JPRB, 0.68411E+01_JPRB, 0.64949E+01_JPRB, 0.69864E+01_JPRB, 0.69714E+01_JPRB ,&
&  0.65156E+01_JPRB, 0.70161E+01_JPRB, 0.72653E+01_JPRB, 0.76439E+01_JPRB, 0.81049E+01_JPRB, 0.85797E+01_JPRB, 0.95580E+01_JPRB ,&
&  0.83549E+01_JPRB, 0.46337E+01_JPRB, 0.31530E+01_JPRB, 0.26034E+01_JPRB, 0.26050E+01_JPRB, 0.34191E+01_JPRB, 0.56455E+01_JPRB ,&
&  0.85402E+01_JPRB, 0.86771E+01_JPRB, 0.86220E+01_JPRB, 0.58099E+01_JPRB, 0.29228E+01_JPRB, 0.20810E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.11731E+01_JPRB, 0.10881E+01_JPRB, 0.11934E+01_JPRB, 0.12360E+01_JPRB, 0.12079E+01_JPRB, 0.12753E+01_JPRB, 0.13605E+01_JPRB ,&
&  0.13351E+01_JPRB, 0.13096E+01_JPRB, 0.13868E+01_JPRB, 0.13728E+01_JPRB, 0.13935E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.14330E+01_JPRB, 0.14830E+01_JPRB, 0.16175E+01_JPRB, 0.18066E+01_JPRB, 0.20928E+01_JPRB, 0.24216E+01_JPRB, 0.26041E+01_JPRB ,&
&  0.24901E+01_JPRB, 0.23643E+01_JPRB, 0.26257E+01_JPRB, 0.28728E+01_JPRB, 0.30594E+01_JPRB, 0.32915E+01_JPRB, 0.35353E+01_JPRB ,&
&  0.38161E+01_JPRB, 0.41039E+01_JPRB, 0.44101E+01_JPRB, 0.47814E+01_JPRB, 0.52103E+01_JPRB, 0.56233E+01_JPRB, 0.54707E+01_JPRB ,&
&  0.39224E+01_JPRB, 0.29331E+01_JPRB, 0.29799E+01_JPRB, 0.27479E+01_JPRB, 0.26764E+01_JPRB, 0.29131E+01_JPRB, 0.28947E+01_JPRB ,&
&  0.23736E+01_JPRB, 0.20192E+01_JPRB, 0.20287E+01_JPRB, 0.19855E+01_JPRB, 0.14937E+01_JPRB, 0.16385E+01_JPRB, 0.19460E+01_JPRB ,&
&  0.21432E+01_JPRB, 0.23453E+01_JPRB, 0.25852E+01_JPRB, 0.28107E+01_JPRB, 0.29593E+01_JPRB, 0.31543E+01_JPRB, 0.33328E+01_JPRB ,&
&  0.34806E+01_JPRB, 0.36482E+01_JPRB, 0.37223E+01_JPRB, 0.37712E+01_JPRB, 0.39844E+01_JPRB, 0.42294E+01_JPRB, 0.43955E+01_JPRB ,&
&  0.44647E+01_JPRB, 0.47219E+01_JPRB, 0.49280E+01_JPRB, 0.50816E+01_JPRB, 0.50786E+01_JPRB, 0.58261E+01_JPRB, 0.61319E+01_JPRB ,&
&  0.66447E+01_JPRB, 0.66932E+01_JPRB, 0.73449E+01_JPRB, 0.60742E+01_JPRB, 0.59245E+01_JPRB, 0.60926E+01_JPRB, 0.62192E+01_JPRB ,&
&  0.61676E+01_JPRB, 0.66374E+01_JPRB, 0.68321E+01_JPRB, 0.74298E+01_JPRB, 0.81382E+01_JPRB, 0.80545E+01_JPRB, 0.84121E+01_JPRB ,&
&  0.52886E+01_JPRB, 0.32507E+01_JPRB, 0.21924E+01_JPRB, 0.22451E+01_JPRB, 0.24930E+01_JPRB, 0.33115E+01_JPRB, 0.48234E+01_JPRB ,&
&  0.64444E+01_JPRB, 0.48788E+01_JPRB, 0.41571E+01_JPRB, 0.34386E+01_JPRB, 0.18363E+01_JPRB, 0.12794E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.93459E+00_JPRB, 0.95600E+00_JPRB, 0.10382E+01_JPRB, 0.10568E+01_JPRB, 0.10351E+01_JPRB, 0.10963E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.12624E+01_JPRB, 0.13454E+01_JPRB, 0.13519E+01_JPRB, 0.14109E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.13274E+01_JPRB, 0.14354E+01_JPRB, 0.15305E+01_JPRB, 0.16497E+01_JPRB, 0.17814E+01_JPRB, 0.19632E+01_JPRB, 0.20538E+01_JPRB ,&
&  0.20510E+01_JPRB, 0.20587E+01_JPRB, 0.22365E+01_JPRB, 0.24521E+01_JPRB, 0.26414E+01_JPRB, 0.27693E+01_JPRB, 0.29082E+01_JPRB ,&
&  0.30434E+01_JPRB, 0.31664E+01_JPRB, 0.32938E+01_JPRB, 0.34113E+01_JPRB, 0.36155E+01_JPRB, 0.39078E+01_JPRB, 0.41657E+01_JPRB ,&
&  0.36768E+01_JPRB, 0.23905E+01_JPRB, 0.26668E+01_JPRB, 0.25904E+01_JPRB, 0.25703E+01_JPRB, 0.26957E+01_JPRB, 0.28492E+01_JPRB ,&
&  0.23425E+01_JPRB, 0.21014E+01_JPRB, 0.21057E+01_JPRB, 0.21263E+01_JPRB, 0.14031E+01_JPRB, 0.14625E+01_JPRB, 0.17158E+01_JPRB ,&
&  0.19644E+01_JPRB, 0.22098E+01_JPRB, 0.24434E+01_JPRB, 0.26208E+01_JPRB, 0.28004E+01_JPRB, 0.30475E+01_JPRB, 0.32906E+01_JPRB ,&
&  0.35098E+01_JPRB, 0.36690E+01_JPRB, 0.37657E+01_JPRB, 0.38630E+01_JPRB, 0.39932E+01_JPRB, 0.41859E+01_JPRB, 0.42832E+01_JPRB ,&
&  0.44326E+01_JPRB, 0.45781E+01_JPRB, 0.49548E+01_JPRB, 0.54115E+01_JPRB, 0.57403E+01_JPRB, 0.55293E+01_JPRB, 0.57379E+01_JPRB ,&
&  0.58005E+01_JPRB, 0.56613E+01_JPRB, 0.58788E+01_JPRB, 0.61188E+01_JPRB, 0.46812E+01_JPRB, 0.47958E+01_JPRB, 0.50519E+01_JPRB ,&
&  0.57278E+01_JPRB, 0.58001E+01_JPRB, 0.60528E+01_JPRB, 0.64580E+01_JPRB, 0.66924E+01_JPRB, 0.66779E+01_JPRB, 0.61949E+01_JPRB ,&
&  0.42330E+01_JPRB, 0.25334E+01_JPRB, 0.19552E+01_JPRB, 0.21265E+01_JPRB, 0.25839E+01_JPRB, 0.37194E+01_JPRB, 0.36715E+01_JPRB ,&
&  0.43400E+01_JPRB, 0.27044E+01_JPRB, 0.24419E+01_JPRB, 0.27953E+01_JPRB, 0.16771E+01_JPRB, 0.11587E+01_JPRB, 0.94063E+00_JPRB ,&
&  0.87167E+00_JPRB, 0.90978E+00_JPRB, 0.92155E+00_JPRB, 0.91418E+00_JPRB, 0.91284E+00_JPRB, 0.97401E+00_JPRB, 0.10538E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.11317E+01_JPRB, 0.11874E+01_JPRB, 0.12475E+01_JPRB, 0.13209E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.10456E+01_JPRB, 0.10681E+01_JPRB, 0.11295E+01_JPRB, 0.12415E+01_JPRB, 0.13455E+01_JPRB, 0.14474E+01_JPRB, 0.15204E+01_JPRB ,&
&  0.16397E+01_JPRB, 0.17423E+01_JPRB, 0.18586E+01_JPRB, 0.19463E+01_JPRB, 0.20209E+01_JPRB, 0.21806E+01_JPRB, 0.22813E+01_JPRB ,&
&  0.23016E+01_JPRB, 0.22596E+01_JPRB, 0.22650E+01_JPRB, 0.23157E+01_JPRB, 0.24943E+01_JPRB, 0.28113E+01_JPRB, 0.31784E+01_JPRB ,&
&  0.35615E+01_JPRB, 0.33652E+01_JPRB, 0.29920E+01_JPRB, 0.19369E+01_JPRB, 0.15988E+01_JPRB, 0.18715E+01_JPRB, 0.19100E+01_JPRB ,&
&  0.19998E+01_JPRB, 0.20240E+01_JPRB, 0.17561E+01_JPRB, 0.15731E+01_JPRB, 0.14100E+01_JPRB, 0.13642E+01_JPRB, 0.14926E+01_JPRB ,&
&  0.16764E+01_JPRB, 0.18926E+01_JPRB, 0.21218E+01_JPRB, 0.22833E+01_JPRB, 0.24592E+01_JPRB, 0.27096E+01_JPRB, 0.29961E+01_JPRB ,&
&  0.32450E+01_JPRB, 0.34271E+01_JPRB, 0.35456E+01_JPRB, 0.37551E+01_JPRB, 0.39095E+01_JPRB, 0.41301E+01_JPRB, 0.43082E+01_JPRB ,&
&  0.44970E+01_JPRB, 0.46226E+01_JPRB, 0.48965E+01_JPRB, 0.52009E+01_JPRB, 0.52285E+01_JPRB, 0.47956E+01_JPRB, 0.46255E+01_JPRB ,&
&  0.46317E+01_JPRB, 0.44492E+01_JPRB, 0.45625E+01_JPRB, 0.50378E+01_JPRB, 0.37367E+01_JPRB, 0.40561E+01_JPRB, 0.46650E+01_JPRB ,&
&  0.47217E+01_JPRB, 0.48861E+01_JPRB, 0.47525E+01_JPRB, 0.42245E+01_JPRB, 0.40875E+01_JPRB, 0.42692E+01_JPRB, 0.41840E+01_JPRB ,&
&  0.33443E+01_JPRB, 0.22797E+01_JPRB, 0.17864E+01_JPRB, 0.16335E+01_JPRB, 0.21053E+01_JPRB, 0.34221E+01_JPRB, 0.27713E+01_JPRB ,&
&  0.25262E+01_JPRB, 0.21853E+01_JPRB, 0.19399E+01_JPRB, 0.28321E+01_JPRB, 0.21210E+01_JPRB, 0.13388E+01_JPRB, 0.99761E+00_JPRB ,&
&  0.86537E+00_JPRB, 0.85884E+00_JPRB, 0.86649E+00_JPRB, 0.85099E+00_JPRB, 0.87412E+00_JPRB, 0.92497E+00_JPRB, 0.93487E+00_JPRB ,&
&  0.10024E+01_JPRB, 0.10178E+01_JPRB, 0.10534E+01_JPRB, 0.10840E+01_JPRB, 0.10475E+01_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.70930E+00_JPRB, 0.70025E+00_JPRB, 0.73038E+00_JPRB, 0.82930E+00_JPRB, 0.90319E+00_JPRB, 0.96262E+00_JPRB, 0.10518E+01_JPRB ,&
&  0.11637E+01_JPRB, 0.13267E+01_JPRB, 0.14595E+01_JPRB, 0.15044E+01_JPRB, 0.14567E+01_JPRB, 0.14063E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.13821E+01_JPRB, 0.12514E+01_JPRB, 0.11409E+01_JPRB, 0.11036E+01_JPRB, 0.11518E+01_JPRB, 0.12843E+01_JPRB, 0.14850E+01_JPRB ,&
&  0.18133E+01_JPRB, 0.20809E+01_JPRB, 0.23030E+01_JPRB, 0.29135E+01_JPRB, 0.28887E+01_JPRB, 0.17617E+01_JPRB, 0.20076E+01_JPRB ,&
&  0.27489E+01_JPRB, 0.27679E+01_JPRB, 0.18236E+01_JPRB, 0.15373E+01_JPRB, 0.13751E+01_JPRB, 0.12678E+01_JPRB, 0.12919E+01_JPRB ,&
&  0.14150E+01_JPRB, 0.15458E+01_JPRB, 0.17010E+01_JPRB, 0.18988E+01_JPRB, 0.21138E+01_JPRB, 0.23143E+01_JPRB, 0.25664E+01_JPRB ,&
&  0.27837E+01_JPRB, 0.28806E+01_JPRB, 0.29806E+01_JPRB, 0.31433E+01_JPRB, 0.34210E+01_JPRB, 0.37574E+01_JPRB, 0.40660E+01_JPRB ,&
&  0.43021E+01_JPRB, 0.44342E+01_JPRB, 0.47088E+01_JPRB, 0.48469E+01_JPRB, 0.44688E+01_JPRB, 0.41013E+01_JPRB, 0.37134E+01_JPRB ,&
&  0.34187E+01_JPRB, 0.30276E+01_JPRB, 0.28482E+01_JPRB, 0.37878E+01_JPRB, 0.40475E+01_JPRB, 0.37473E+01_JPRB, 0.37943E+01_JPRB ,&
&  0.38985E+01_JPRB, 0.38991E+01_JPRB, 0.35262E+01_JPRB, 0.28055E+01_JPRB, 0.23053E+01_JPRB, 0.26663E+01_JPRB, 0.28961E+01_JPRB ,&
&  0.25001E+01_JPRB, 0.18301E+01_JPRB, 0.15088E+01_JPRB, 0.14174E+01_JPRB, 0.14983E+01_JPRB, 0.23530E+01_JPRB, 0.19073E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.16243E+01_JPRB, 0.15429E+01_JPRB, 0.22356E+01_JPRB, 0.29802E+01_JPRB, 0.16354E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.83550E+00_JPRB, 0.82274E+00_JPRB, 0.82233E+00_JPRB, 0.82052E+00_JPRB, 0.82441E+00_JPRB, 0.84519E+00_JPRB, 0.84739E+00_JPRB ,&
&  0.86213E+00_JPRB, 0.84418E+00_JPRB, 0.84300E+00_JPRB, 0.79509E+00_JPRB, 0.72477E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.71855E+00_JPRB, 0.71403E+00_JPRB, 0.70112E+00_JPRB, 0.69817E+00_JPRB, 0.71110E+00_JPRB, 0.73153E+00_JPRB, 0.76199E+00_JPRB ,&
&  0.80897E+00_JPRB, 0.86279E+00_JPRB, 0.91302E+00_JPRB, 0.96717E+00_JPRB, 0.97288E+00_JPRB, 0.89930E+00_JPRB, 0.88436E+00_JPRB ,&
&  0.87341E+00_JPRB, 0.88020E+00_JPRB, 0.87258E+00_JPRB, 0.96254E+00_JPRB, 0.98370E+00_JPRB, 0.99204E+00_JPRB, 0.11119E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.12600E+01_JPRB, 0.13578E+01_JPRB, 0.14932E+01_JPRB, 0.23636E+01_JPRB, 0.22440E+01_JPRB, 0.17696E+01_JPRB ,&
&  0.25077E+01_JPRB, 0.25009E+01_JPRB, 0.18179E+01_JPRB, 0.13990E+01_JPRB, 0.12573E+01_JPRB, 0.11269E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.12223E+01_JPRB, 0.13864E+01_JPRB, 0.15509E+01_JPRB, 0.16815E+01_JPRB, 0.18081E+01_JPRB, 0.19625E+01_JPRB ,&
&  0.20637E+01_JPRB, 0.21598E+01_JPRB, 0.21602E+01_JPRB, 0.21863E+01_JPRB, 0.25127E+01_JPRB, 0.28154E+01_JPRB, 0.30420E+01_JPRB ,&
&  0.32216E+01_JPRB, 0.34167E+01_JPRB, 0.37031E+01_JPRB, 0.35832E+01_JPRB, 0.32795E+01_JPRB, 0.30689E+01_JPRB, 0.25978E+01_JPRB ,&
&  0.21565E+01_JPRB, 0.17757E+01_JPRB, 0.16872E+01_JPRB, 0.22755E+01_JPRB, 0.25166E+01_JPRB, 0.23390E+01_JPRB, 0.27782E+01_JPRB ,&
&  0.29237E+01_JPRB, 0.30113E+01_JPRB, 0.25575E+01_JPRB, 0.22461E+01_JPRB, 0.18757E+01_JPRB, 0.19130E+01_JPRB, 0.20710E+01_JPRB ,&
&  0.17380E+01_JPRB, 0.13492E+01_JPRB, 0.11891E+01_JPRB, 0.11349E+01_JPRB, 0.11566E+01_JPRB, 0.15478E+01_JPRB, 0.18124E+01_JPRB ,&
&  0.17153E+01_JPRB, 0.14746E+01_JPRB, 0.14662E+01_JPRB, 0.21697E+01_JPRB, 0.18295E+01_JPRB, 0.11325E+01_JPRB, 0.79769E+00_JPRB ,&
&  0.72533E+00_JPRB, 0.73045E+00_JPRB, 0.72224E+00_JPRB, 0.71756E+00_JPRB, 0.72817E+00_JPRB, 0.74763E+00_JPRB, 0.73249E+00_JPRB ,&
&  0.72800E+00_JPRB, 0.70947E+00_JPRB, 0.69659E+00_JPRB, 0.68636E+00_JPRB, 0.69103E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.86783E+00_JPRB, 0.87656E+00_JPRB, 0.88702E+00_JPRB, 0.89995E+00_JPRB, 0.92650E+00_JPRB, 0.95668E+00_JPRB, 0.98390E+00_JPRB ,&
&  0.10332E+01_JPRB, 0.10698E+01_JPRB, 0.11038E+01_JPRB, 0.11236E+01_JPRB, 0.11395E+01_JPRB, 0.11509E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.11809E+01_JPRB, 0.12129E+01_JPRB, 0.12120E+01_JPRB, 0.12189E+01_JPRB, 0.12450E+01_JPRB, 0.12620E+01_JPRB, 0.12427E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.11950E+01_JPRB, 0.12225E+01_JPRB, 0.12673E+01_JPRB, 0.14875E+01_JPRB, 0.16508E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.19008E+01_JPRB, 0.20137E+01_JPRB, 0.18993E+01_JPRB, 0.13676E+01_JPRB, 0.12277E+01_JPRB, 0.11447E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10884E+01_JPRB, 0.11280E+01_JPRB, 0.11732E+01_JPRB, 0.12313E+01_JPRB, 0.12560E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.13174E+01_JPRB, 0.13463E+01_JPRB, 0.13919E+01_JPRB, 0.14846E+01_JPRB, 0.16785E+01_JPRB, 0.19541E+01_JPRB, 0.19703E+01_JPRB ,&
&  0.19807E+01_JPRB, 0.21237E+01_JPRB, 0.19902E+01_JPRB, 0.19504E+01_JPRB, 0.18736E+01_JPRB, 0.16304E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.13320E+01_JPRB, 0.12688E+01_JPRB, 0.11200E+01_JPRB, 0.14658E+01_JPRB, 0.17508E+01_JPRB, 0.18649E+01_JPRB, 0.21381E+01_JPRB ,&
&  0.22861E+01_JPRB, 0.20247E+01_JPRB, 0.16605E+01_JPRB, 0.16508E+01_JPRB, 0.16363E+01_JPRB, 0.13972E+01_JPRB, 0.13963E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.11276E+01_JPRB, 0.10886E+01_JPRB, 0.11102E+01_JPRB, 0.13460E+01_JPRB, 0.21722E+01_JPRB, 0.23143E+01_JPRB ,&
&  0.18112E+01_JPRB, 0.15373E+01_JPRB, 0.18042E+01_JPRB, 0.28253E+01_JPRB, 0.15517E+01_JPRB, 0.10433E+01_JPRB, 0.74528E+00_JPRB ,&
&  0.72917E+00_JPRB, 0.73265E+00_JPRB, 0.73757E+00_JPRB, 0.74350E+00_JPRB, 0.75224E+00_JPRB, 0.77169E+00_JPRB, 0.76244E+00_JPRB ,&
&  0.75273E+00_JPRB, 0.74417E+00_JPRB, 0.75742E+00_JPRB, 0.78071E+00_JPRB, 0.82908E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.99457E+00_JPRB, 0.99855E+00_JPRB, 0.10180E+01_JPRB, 0.10426E+01_JPRB, 0.10668E+01_JPRB, 0.10955E+01_JPRB, 0.11390E+01_JPRB ,&
&  0.11885E+01_JPRB, 0.12270E+01_JPRB, 0.12628E+01_JPRB, 0.12890E+01_JPRB, 0.12968E+01_JPRB, 0.13116E+01_JPRB, 0.13286E+01_JPRB ,&
&  0.13470E+01_JPRB, 0.13401E+01_JPRB, 0.13621E+01_JPRB, 0.13985E+01_JPRB, 0.14449E+01_JPRB, 0.14637E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.15065E+01_JPRB, 0.15589E+01_JPRB, 0.16168E+01_JPRB, 0.16872E+01_JPRB, 0.17983E+01_JPRB, 0.19827E+01_JPRB, 0.21523E+01_JPRB ,&
&  0.18257E+01_JPRB, 0.21134E+01_JPRB, 0.17819E+01_JPRB, 0.13982E+01_JPRB, 0.12941E+01_JPRB, 0.12945E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.11885E+01_JPRB, 0.11919E+01_JPRB, 0.12012E+01_JPRB, 0.12115E+01_JPRB, 0.12297E+01_JPRB, 0.12687E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.12898E+01_JPRB, 0.13034E+01_JPRB, 0.13182E+01_JPRB, 0.13676E+01_JPRB, 0.13358E+01_JPRB, 0.15903E+01_JPRB, 0.15298E+01_JPRB ,&
&  0.14359E+01_JPRB, 0.13992E+01_JPRB, 0.11007E+01_JPRB, 0.11136E+01_JPRB, 0.11162E+01_JPRB, 0.98372E+00_JPRB, 0.90929E+00_JPRB ,&
&  0.11210E+01_JPRB, 0.12699E+01_JPRB, 0.11247E+01_JPRB, 0.13880E+01_JPRB, 0.15571E+01_JPRB, 0.16361E+01_JPRB, 0.16858E+01_JPRB ,&
&  0.16611E+01_JPRB, 0.14520E+01_JPRB, 0.12732E+01_JPRB, 0.11950E+01_JPRB, 0.11719E+01_JPRB, 0.11303E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10370E+01_JPRB, 0.10206E+01_JPRB, 0.10094E+01_JPRB, 0.10344E+01_JPRB, 0.15373E+01_JPRB, 0.20471E+01_JPRB, 0.30274E+01_JPRB ,&
&  0.16680E+01_JPRB, 0.14574E+01_JPRB, 0.11745E+01_JPRB, 0.17248E+01_JPRB, 0.12230E+01_JPRB, 0.79966E+00_JPRB, 0.79628E+00_JPRB ,&
&  0.85077E+00_JPRB, 0.88954E+00_JPRB, 0.88898E+00_JPRB, 0.87374E+00_JPRB, 0.87277E+00_JPRB, 0.87612E+00_JPRB, 0.85074E+00_JPRB ,&
&  0.83529E+00_JPRB, 0.85080E+00_JPRB, 0.88818E+00_JPRB, 0.90748E+00_JPRB, 0.95196E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.10629E+01_JPRB, 0.10935E+01_JPRB, 0.11261E+01_JPRB, 0.11615E+01_JPRB, 0.11955E+01_JPRB, 0.12275E+01_JPRB, 0.12622E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.13349E+01_JPRB, 0.13643E+01_JPRB, 0.13813E+01_JPRB, 0.13899E+01_JPRB, 0.14096E+01_JPRB, 0.14167E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.14250E+01_JPRB, 0.14493E+01_JPRB, 0.14665E+01_JPRB, 0.14794E+01_JPRB, 0.14839E+01_JPRB, 0.15132E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.15715E+01_JPRB, 0.16121E+01_JPRB, 0.16643E+01_JPRB, 0.17081E+01_JPRB, 0.17313E+01_JPRB, 0.18759E+01_JPRB ,&
&  0.16280E+01_JPRB, 0.23457E+01_JPRB, 0.21126E+01_JPRB, 0.16111E+01_JPRB, 0.15158E+01_JPRB, 0.13999E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.13323E+01_JPRB, 0.13448E+01_JPRB, 0.13788E+01_JPRB, 0.14121E+01_JPRB, 0.14518E+01_JPRB, 0.14679E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.15094E+01_JPRB, 0.15282E+01_JPRB, 0.15414E+01_JPRB, 0.15631E+01_JPRB, 0.15680E+01_JPRB, 0.15216E+01_JPRB ,&
&  0.14233E+01_JPRB, 0.13361E+01_JPRB, 0.10003E+01_JPRB, 0.94782E+00_JPRB, 0.98278E+00_JPRB, 0.93484E+00_JPRB, 0.84933E+00_JPRB ,&
&  0.10495E+01_JPRB, 0.12166E+01_JPRB, 0.12627E+01_JPRB, 0.13522E+01_JPRB, 0.13804E+01_JPRB, 0.13188E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.11593E+01_JPRB, 0.10190E+01_JPRB, 0.93638E+00_JPRB, 0.89086E+00_JPRB, 0.90854E+00_JPRB, 0.91260E+00_JPRB, 0.90180E+00_JPRB ,&
&  0.87402E+00_JPRB, 0.90219E+00_JPRB, 0.97691E+00_JPRB, 0.10922E+01_JPRB, 0.13702E+01_JPRB, 0.27327E+01_JPRB, 0.29679E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.12113E+01_JPRB, 0.82512E+00_JPRB, 0.96684E+00_JPRB, 0.97850E+00_JPRB, 0.92315E+00_JPRB, 0.10306E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.11572E+01_JPRB, 0.10853E+01_JPRB, 0.99465E+00_JPRB, 0.95687E+00_JPRB, 0.94306E+00_JPRB, 0.91586E+00_JPRB ,&
&  0.91236E+00_JPRB, 0.94308E+00_JPRB, 0.97381E+00_JPRB, 0.10010E+01_JPRB, 0.10314E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.11226E+01_JPRB, 0.11512E+01_JPRB, 0.11798E+01_JPRB, 0.12216E+01_JPRB, 0.12663E+01_JPRB, 0.13087E+01_JPRB, 0.13428E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.14039E+01_JPRB, 0.14343E+01_JPRB, 0.14585E+01_JPRB, 0.14770E+01_JPRB, 0.14919E+01_JPRB, 0.14988E+01_JPRB ,&
&  0.15036E+01_JPRB, 0.15151E+01_JPRB, 0.15404E+01_JPRB, 0.15660E+01_JPRB, 0.15850E+01_JPRB, 0.16077E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.16787E+01_JPRB, 0.17245E+01_JPRB, 0.17938E+01_JPRB, 0.18848E+01_JPRB, 0.21133E+01_JPRB, 0.20089E+01_JPRB ,&
&  0.22586E+01_JPRB, 0.24824E+01_JPRB, 0.23562E+01_JPRB, 0.18147E+01_JPRB, 0.16282E+01_JPRB, 0.15439E+01_JPRB, 0.15166E+01_JPRB ,&
&  0.15001E+01_JPRB, 0.14645E+01_JPRB, 0.14565E+01_JPRB, 0.14831E+01_JPRB, 0.15291E+01_JPRB, 0.15836E+01_JPRB, 0.16096E+01_JPRB ,&
&  0.16439E+01_JPRB, 0.16828E+01_JPRB, 0.17289E+01_JPRB, 0.17711E+01_JPRB, 0.18448E+01_JPRB, 0.18692E+01_JPRB, 0.18319E+01_JPRB ,&
&  0.17520E+01_JPRB, 0.16963E+01_JPRB, 0.14593E+01_JPRB, 0.13562E+01_JPRB, 0.13722E+01_JPRB, 0.12248E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.11794E+01_JPRB, 0.11171E+01_JPRB, 0.10692E+01_JPRB, 0.12486E+01_JPRB, 0.12258E+01_JPRB, 0.11040E+01_JPRB, 0.10508E+01_JPRB ,&
&  0.91577E+00_JPRB, 0.80129E+00_JPRB, 0.73066E+00_JPRB, 0.72226E+00_JPRB, 0.74209E+00_JPRB, 0.79087E+00_JPRB, 0.81301E+00_JPRB ,&
&  0.85793E+00_JPRB, 0.93255E+00_JPRB, 0.10196E+01_JPRB, 0.11628E+01_JPRB, 0.14928E+01_JPRB, 0.15192E+01_JPRB, 0.17579E+01_JPRB ,&
&  0.12536E+01_JPRB, 0.10397E+01_JPRB, 0.10790E+01_JPRB, 0.10579E+01_JPRB, 0.11092E+01_JPRB, 0.11356E+01_JPRB, 0.12876E+01_JPRB ,&
&  0.14705E+01_JPRB, 0.16245E+01_JPRB, 0.14922E+01_JPRB, 0.12942E+01_JPRB, 0.10961E+01_JPRB, 0.10124E+01_JPRB, 0.97588E+00_JPRB ,&
&  0.97729E+00_JPRB, 0.10147E+01_JPRB, 0.10452E+01_JPRB, 0.10661E+01_JPRB, 0.10932E+01_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.11247E+01_JPRB, 0.11540E+01_JPRB, 0.11780E+01_JPRB, 0.12091E+01_JPRB, 0.12469E+01_JPRB, 0.12899E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.13604E+01_JPRB, 0.13910E+01_JPRB, 0.14226E+01_JPRB, 0.14575E+01_JPRB, 0.14883E+01_JPRB, 0.15266E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.15875E+01_JPRB, 0.16010E+01_JPRB, 0.16287E+01_JPRB, 0.16652E+01_JPRB, 0.17071E+01_JPRB, 0.17554E+01_JPRB, 0.18106E+01_JPRB ,&
&  0.18449E+01_JPRB, 0.18554E+01_JPRB, 0.18735E+01_JPRB, 0.19013E+01_JPRB, 0.19129E+01_JPRB, 0.20776E+01_JPRB, 0.20351E+01_JPRB ,&
&  0.25817E+01_JPRB, 0.27995E+01_JPRB, 0.25872E+01_JPRB, 0.20465E+01_JPRB, 0.18532E+01_JPRB, 0.17495E+01_JPRB, 0.17044E+01_JPRB ,&
&  0.16595E+01_JPRB, 0.16218E+01_JPRB, 0.15657E+01_JPRB, 0.15632E+01_JPRB, 0.15982E+01_JPRB, 0.16737E+01_JPRB, 0.17440E+01_JPRB ,&
&  0.17894E+01_JPRB, 0.18498E+01_JPRB, 0.19321E+01_JPRB, 0.20278E+01_JPRB, 0.21542E+01_JPRB, 0.22179E+01_JPRB, 0.22225E+01_JPRB ,&
&  0.21998E+01_JPRB, 0.21738E+01_JPRB, 0.19889E+01_JPRB, 0.20074E+01_JPRB, 0.20337E+01_JPRB, 0.16776E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.11969E+01_JPRB, 0.11040E+01_JPRB, 0.11255E+01_JPRB, 0.11828E+01_JPRB, 0.10994E+01_JPRB, 0.10251E+01_JPRB, 0.95257E+00_JPRB ,&
&  0.81205E+00_JPRB, 0.73765E+00_JPRB, 0.70227E+00_JPRB, 0.67047E+00_JPRB, 0.67940E+00_JPRB, 0.71738E+00_JPRB, 0.80535E+00_JPRB ,&
&  0.88015E+00_JPRB, 0.96440E+00_JPRB, 0.10775E+01_JPRB, 0.11945E+01_JPRB, 0.13714E+01_JPRB, 0.13847E+01_JPRB, 0.15523E+01_JPRB ,&
&  0.13390E+01_JPRB, 0.12672E+01_JPRB, 0.12546E+01_JPRB, 0.95586E+00_JPRB, 0.11744E+01_JPRB, 0.12669E+01_JPRB, 0.14217E+01_JPRB ,&
&  0.14679E+01_JPRB, 0.19414E+01_JPRB, 0.22938E+01_JPRB, 0.19360E+01_JPRB, 0.14434E+01_JPRB, 0.10617E+01_JPRB, 0.97576E+00_JPRB ,&
&  0.98556E+00_JPRB, 0.10206E+01_JPRB, 0.10542E+01_JPRB, 0.10756E+01_JPRB, 0.10998E+01_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.10663E+01_JPRB, 0.10930E+01_JPRB, 0.11184E+01_JPRB, 0.11381E+01_JPRB, 0.11502E+01_JPRB, 0.11742E+01_JPRB, 0.12123E+01_JPRB ,&
&  0.12483E+01_JPRB, 0.12853E+01_JPRB, 0.13233E+01_JPRB, 0.13611E+01_JPRB, 0.14011E+01_JPRB, 0.14423E+01_JPRB, 0.14993E+01_JPRB ,&
&  0.15765E+01_JPRB, 0.16519E+01_JPRB, 0.17126E+01_JPRB, 0.18011E+01_JPRB, 0.18826E+01_JPRB, 0.19583E+01_JPRB, 0.20834E+01_JPRB ,&
&  0.21988E+01_JPRB, 0.23210E+01_JPRB, 0.24166E+01_JPRB, 0.24752E+01_JPRB, 0.24683E+01_JPRB, 0.26448E+01_JPRB, 0.24174E+01_JPRB ,&
&  0.28541E+01_JPRB, 0.30787E+01_JPRB, 0.28036E+01_JPRB, 0.22633E+01_JPRB, 0.20843E+01_JPRB, 0.19573E+01_JPRB, 0.18512E+01_JPRB ,&
&  0.17949E+01_JPRB, 0.17074E+01_JPRB, 0.15702E+01_JPRB, 0.15867E+01_JPRB, 0.16167E+01_JPRB, 0.16856E+01_JPRB, 0.17985E+01_JPRB ,&
&  0.18854E+01_JPRB, 0.19808E+01_JPRB, 0.20818E+01_JPRB, 0.21867E+01_JPRB, 0.23264E+01_JPRB, 0.24385E+01_JPRB, 0.25129E+01_JPRB ,&
&  0.25665E+01_JPRB, 0.26067E+01_JPRB, 0.26387E+01_JPRB, 0.24598E+01_JPRB, 0.24286E+01_JPRB, 0.21741E+01_JPRB, 0.17051E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.10832E+01_JPRB, 0.11181E+01_JPRB, 0.11400E+01_JPRB, 0.10790E+01_JPRB, 0.94730E+00_JPRB, 0.89493E+00_JPRB ,&
&  0.77238E+00_JPRB, 0.70173E+00_JPRB, 0.68850E+00_JPRB, 0.67515E+00_JPRB, 0.67360E+00_JPRB, 0.72726E+00_JPRB, 0.78183E+00_JPRB ,&
&  0.87134E+00_JPRB, 0.99016E+00_JPRB, 0.11065E+01_JPRB, 0.12323E+01_JPRB, 0.14502E+01_JPRB, 0.16460E+01_JPRB, 0.14698E+01_JPRB ,&
&  0.14164E+01_JPRB, 0.12598E+01_JPRB, 0.12470E+01_JPRB, 0.13575E+01_JPRB, 0.12751E+01_JPRB, 0.14828E+01_JPRB, 0.17074E+01_JPRB ,&
&  0.20337E+01_JPRB, 0.19438E+01_JPRB, 0.22183E+01_JPRB, 0.32355E+01_JPRB, 0.27526E+01_JPRB, 0.23542E+01_JPRB, 0.95947E+00_JPRB ,&
&  0.96676E+00_JPRB, 0.99576E+00_JPRB, 0.10358E+01_JPRB, 0.10499E+01_JPRB, 0.10633E+01_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.92646E+00_JPRB, 0.97267E+00_JPRB, 0.98958E+00_JPRB, 0.10333E+01_JPRB, 0.10425E+01_JPRB, 0.10432E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.11350E+01_JPRB, 0.11780E+01_JPRB, 0.12218E+01_JPRB, 0.12607E+01_JPRB, 0.12887E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.13987E+01_JPRB, 0.14813E+01_JPRB, 0.15848E+01_JPRB, 0.17204E+01_JPRB, 0.18293E+01_JPRB, 0.19429E+01_JPRB, 0.20912E+01_JPRB ,&
&  0.22246E+01_JPRB, 0.24229E+01_JPRB, 0.26173E+01_JPRB, 0.28023E+01_JPRB, 0.29250E+01_JPRB, 0.29634E+01_JPRB, 0.22810E+01_JPRB ,&
&  0.30779E+01_JPRB, 0.30292E+01_JPRB, 0.28558E+01_JPRB, 0.24411E+01_JPRB, 0.22589E+01_JPRB, 0.21153E+01_JPRB, 0.19929E+01_JPRB ,&
&  0.18954E+01_JPRB, 0.17621E+01_JPRB, 0.16140E+01_JPRB, 0.15111E+01_JPRB, 0.16171E+01_JPRB, 0.16338E+01_JPRB, 0.17374E+01_JPRB ,&
&  0.18469E+01_JPRB, 0.19562E+01_JPRB, 0.20593E+01_JPRB, 0.21640E+01_JPRB, 0.23067E+01_JPRB, 0.24698E+01_JPRB, 0.26169E+01_JPRB ,&
&  0.27409E+01_JPRB, 0.28061E+01_JPRB, 0.29218E+01_JPRB, 0.26049E+01_JPRB, 0.25209E+01_JPRB, 0.23927E+01_JPRB, 0.16610E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.11078E+01_JPRB, 0.11823E+01_JPRB, 0.11786E+01_JPRB, 0.10773E+01_JPRB, 0.93862E+00_JPRB, 0.84356E+00_JPRB ,&
&  0.74961E+00_JPRB, 0.72779E+00_JPRB, 0.71081E+00_JPRB, 0.69924E+00_JPRB, 0.69929E+00_JPRB, 0.74164E+00_JPRB, 0.78537E+00_JPRB ,&
&  0.87640E+00_JPRB, 0.95136E+00_JPRB, 0.10403E+01_JPRB, 0.11305E+01_JPRB, 0.12642E+01_JPRB, 0.14291E+01_JPRB, 0.15881E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.11739E+01_JPRB, 0.11470E+01_JPRB, 0.11890E+01_JPRB, 0.12120E+01_JPRB, 0.12963E+01_JPRB, 0.14020E+01_JPRB ,&
&  0.15436E+01_JPRB, 0.16434E+01_JPRB, 0.14910E+01_JPRB, 0.12416E+01_JPRB, 0.15526E+01_JPRB, 0.15883E+01_JPRB, 0.79601E+00_JPRB ,&
&  0.79807E+00_JPRB, 0.84115E+00_JPRB, 0.87167E+00_JPRB, 0.90424E+00_JPRB, 0.93416E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.75615E+00_JPRB, 0.76939E+00_JPRB, 0.77491E+00_JPRB, 0.82546E+00_JPRB, 0.84365E+00_JPRB, 0.87445E+00_JPRB, 0.90057E+00_JPRB ,&
&  0.93217E+00_JPRB, 0.97543E+00_JPRB, 0.10250E+01_JPRB, 0.10737E+01_JPRB, 0.11178E+01_JPRB, 0.11547E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.12284E+01_JPRB, 0.12738E+01_JPRB, 0.13337E+01_JPRB, 0.14153E+01_JPRB, 0.15404E+01_JPRB, 0.16855E+01_JPRB, 0.18456E+01_JPRB ,&
&  0.20134E+01_JPRB, 0.22277E+01_JPRB, 0.24679E+01_JPRB, 0.27301E+01_JPRB, 0.29989E+01_JPRB, 0.31896E+01_JPRB, 0.33271E+01_JPRB ,&
&  0.24612E+01_JPRB, 0.29140E+01_JPRB, 0.28276E+01_JPRB, 0.25632E+01_JPRB, 0.23894E+01_JPRB, 0.22957E+01_JPRB, 0.21727E+01_JPRB ,&
&  0.20567E+01_JPRB, 0.18602E+01_JPRB, 0.17361E+01_JPRB, 0.15975E+01_JPRB, 0.15807E+01_JPRB, 0.15830E+01_JPRB, 0.16482E+01_JPRB ,&
&  0.17537E+01_JPRB, 0.18370E+01_JPRB, 0.19369E+01_JPRB, 0.20319E+01_JPRB, 0.21567E+01_JPRB, 0.23566E+01_JPRB, 0.25842E+01_JPRB ,&
&  0.28259E+01_JPRB, 0.30111E+01_JPRB, 0.31490E+01_JPRB, 0.27757E+01_JPRB, 0.25565E+01_JPRB, 0.23667E+01_JPRB, 0.16152E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.10964E+01_JPRB, 0.11749E+01_JPRB, 0.12208E+01_JPRB, 0.10917E+01_JPRB, 0.96664E+00_JPRB, 0.85932E+00_JPRB ,&
&  0.77858E+00_JPRB, 0.73215E+00_JPRB, 0.72559E+00_JPRB, 0.71332E+00_JPRB, 0.72084E+00_JPRB, 0.75208E+00_JPRB, 0.78381E+00_JPRB ,&
&  0.84411E+00_JPRB, 0.87455E+00_JPRB, 0.93919E+00_JPRB, 0.99199E+00_JPRB, 0.10475E+01_JPRB, 0.10892E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.11345E+01_JPRB, 0.11416E+01_JPRB, 0.11389E+01_JPRB, 0.11603E+01_JPRB, 0.11598E+01_JPRB, 0.11698E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.12385E+01_JPRB, 0.12016E+01_JPRB, 0.11538E+01_JPRB, 0.10746E+01_JPRB, 0.10053E+01_JPRB, 0.78256E+00_JPRB ,&
&  0.74523E+00_JPRB, 0.70728E+00_JPRB, 0.73332E+00_JPRB, 0.76399E+00_JPRB, 0.75887E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.66940E+00_JPRB, 0.65871E+00_JPRB, 0.64665E+00_JPRB, 0.65753E+00_JPRB, 0.66942E+00_JPRB, 0.70118E+00_JPRB, 0.70715E+00_JPRB ,&
&  0.73057E+00_JPRB, 0.79391E+00_JPRB, 0.87575E+00_JPRB, 0.93249E+00_JPRB, 0.97909E+00_JPRB, 0.10247E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.11190E+01_JPRB, 0.11404E+01_JPRB, 0.11688E+01_JPRB, 0.12208E+01_JPRB, 0.13043E+01_JPRB, 0.14381E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.17516E+01_JPRB, 0.19635E+01_JPRB, 0.22236E+01_JPRB, 0.25062E+01_JPRB, 0.26893E+01_JPRB, 0.28370E+01_JPRB ,&
&  0.16330E+01_JPRB, 0.24634E+01_JPRB, 0.29617E+01_JPRB, 0.27636E+01_JPRB, 0.25442E+01_JPRB, 0.24229E+01_JPRB, 0.22729E+01_JPRB ,&
&  0.20497E+01_JPRB, 0.18240E+01_JPRB, 0.17067E+01_JPRB, 0.17451E+01_JPRB, 0.15713E+01_JPRB, 0.15529E+01_JPRB, 0.15583E+01_JPRB ,&
&  0.16327E+01_JPRB, 0.16855E+01_JPRB, 0.17075E+01_JPRB, 0.17461E+01_JPRB, 0.18122E+01_JPRB, 0.19778E+01_JPRB, 0.22622E+01_JPRB ,&
&  0.26535E+01_JPRB, 0.30623E+01_JPRB, 0.32765E+01_JPRB, 0.27932E+01_JPRB, 0.28524E+01_JPRB, 0.26336E+01_JPRB, 0.21983E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.11618E+01_JPRB, 0.11983E+01_JPRB, 0.12255E+01_JPRB, 0.11104E+01_JPRB, 0.10118E+01_JPRB, 0.92162E+00_JPRB ,&
&  0.82955E+00_JPRB, 0.77995E+00_JPRB, 0.75887E+00_JPRB, 0.74764E+00_JPRB, 0.75226E+00_JPRB, 0.76708E+00_JPRB, 0.79162E+00_JPRB ,&
&  0.81061E+00_JPRB, 0.83506E+00_JPRB, 0.87297E+00_JPRB, 0.91424E+00_JPRB, 0.96167E+00_JPRB, 0.10060E+01_JPRB, 0.10524E+01_JPRB ,&
&  0.10821E+01_JPRB, 0.11072E+01_JPRB, 0.11307E+01_JPRB, 0.11487E+01_JPRB, 0.11580E+01_JPRB, 0.11657E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.11488E+01_JPRB, 0.11515E+01_JPRB, 0.11328E+01_JPRB, 0.10767E+01_JPRB, 0.10222E+01_JPRB, 0.97366E+00_JPRB ,&
&  0.89409E+00_JPRB, 0.78857E+00_JPRB, 0.70148E+00_JPRB, 0.68301E+00_JPRB, 0.67292E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.63250E+00_JPRB, 0.60523E+00_JPRB, 0.59854E+00_JPRB, 0.60395E+00_JPRB, 0.62031E+00_JPRB, 0.63118E+00_JPRB, 0.63277E+00_JPRB ,&
&  0.63920E+00_JPRB, 0.65824E+00_JPRB, 0.68059E+00_JPRB, 0.74471E+00_JPRB, 0.82084E+00_JPRB, 0.88336E+00_JPRB, 0.92589E+00_JPRB ,&
&  0.95966E+00_JPRB, 0.97168E+00_JPRB, 0.96831E+00_JPRB, 0.97082E+00_JPRB, 0.97634E+00_JPRB, 0.99978E+00_JPRB, 0.10576E+01_JPRB ,&
&  0.11664E+01_JPRB, 0.12961E+01_JPRB, 0.14637E+01_JPRB, 0.16993E+01_JPRB, 0.20254E+01_JPRB, 0.23697E+01_JPRB, 0.26300E+01_JPRB ,&
&  0.19191E+01_JPRB, 0.92712E+00_JPRB, 0.26620E+01_JPRB, 0.32030E+01_JPRB, 0.28245E+01_JPRB, 0.26700E+01_JPRB, 0.23902E+01_JPRB ,&
&  0.20394E+01_JPRB, 0.18659E+01_JPRB, 0.17292E+01_JPRB, 0.17378E+01_JPRB, 0.15852E+01_JPRB, 0.15548E+01_JPRB, 0.15556E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.16139E+01_JPRB, 0.16104E+01_JPRB, 0.15831E+01_JPRB, 0.15700E+01_JPRB, 0.16186E+01_JPRB, 0.18038E+01_JPRB ,&
&  0.21915E+01_JPRB, 0.27633E+01_JPRB, 0.32954E+01_JPRB, 0.29168E+01_JPRB, 0.29916E+01_JPRB, 0.24251E+01_JPRB, 0.18495E+01_JPRB ,&
&  0.14726E+01_JPRB, 0.12550E+01_JPRB, 0.12274E+01_JPRB, 0.12418E+01_JPRB, 0.11508E+01_JPRB, 0.10326E+01_JPRB, 0.10096E+01_JPRB ,&
&  0.90455E+00_JPRB, 0.83613E+00_JPRB, 0.79445E+00_JPRB, 0.77690E+00_JPRB, 0.77566E+00_JPRB, 0.78687E+00_JPRB, 0.80204E+00_JPRB ,&
&  0.80870E+00_JPRB, 0.82273E+00_JPRB, 0.84283E+00_JPRB, 0.87984E+00_JPRB, 0.92474E+00_JPRB, 0.98232E+00_JPRB, 0.10284E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10739E+01_JPRB, 0.10955E+01_JPRB, 0.11087E+01_JPRB, 0.11206E+01_JPRB, 0.10990E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.11108E+01_JPRB, 0.11292E+01_JPRB, 0.11566E+01_JPRB, 0.11147E+01_JPRB, 0.10749E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.97331E+00_JPRB, 0.83517E+00_JPRB, 0.71032E+00_JPRB, 0.66247E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.69658E+00_JPRB, 0.66244E+00_JPRB, 0.63187E+00_JPRB, 0.62418E+00_JPRB, 0.63661E+00_JPRB, 0.63249E+00_JPRB, 0.63012E+00_JPRB ,&
&  0.62556E+00_JPRB, 0.62222E+00_JPRB, 0.60967E+00_JPRB, 0.62172E+00_JPRB, 0.66309E+00_JPRB, 0.73005E+00_JPRB, 0.78079E+00_JPRB ,&
&  0.82299E+00_JPRB, 0.83668E+00_JPRB, 0.83504E+00_JPRB, 0.83166E+00_JPRB, 0.82769E+00_JPRB, 0.82879E+00_JPRB, 0.84070E+00_JPRB ,&
&  0.87823E+00_JPRB, 0.94185E+00_JPRB, 0.10286E+01_JPRB, 0.11448E+01_JPRB, 0.13220E+01_JPRB, 0.16513E+01_JPRB, 0.21705E+01_JPRB ,&
&  0.30075E+01_JPRB, 0.21490E+01_JPRB, 0.12619E+01_JPRB, 0.28190E+01_JPRB, 0.31707E+01_JPRB, 0.31269E+01_JPRB, 0.25827E+01_JPRB ,&
&  0.23534E+01_JPRB, 0.20548E+01_JPRB, 0.18142E+01_JPRB, 0.18123E+01_JPRB, 0.16176E+01_JPRB, 0.15623E+01_JPRB, 0.15687E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.15663E+01_JPRB, 0.15620E+01_JPRB, 0.15248E+01_JPRB, 0.14657E+01_JPRB, 0.14272E+01_JPRB, 0.14762E+01_JPRB ,&
&  0.17159E+01_JPRB, 0.22113E+01_JPRB, 0.30109E+01_JPRB, 0.29907E+01_JPRB, 0.30495E+01_JPRB, 0.22495E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.14933E+01_JPRB, 0.14530E+01_JPRB, 0.14028E+01_JPRB, 0.12967E+01_JPRB, 0.11618E+01_JPRB, 0.10747E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.99483E+00_JPRB, 0.92706E+00_JPRB, 0.86829E+00_JPRB, 0.83402E+00_JPRB, 0.81791E+00_JPRB, 0.81414E+00_JPRB, 0.81410E+00_JPRB ,&
&  0.82629E+00_JPRB, 0.83317E+00_JPRB, 0.85163E+00_JPRB, 0.87579E+00_JPRB, 0.90359E+00_JPRB, 0.94069E+00_JPRB, 0.98827E+00_JPRB ,&
&  0.10239E+01_JPRB, 0.10443E+01_JPRB, 0.10558E+01_JPRB, 0.10582E+01_JPRB, 0.10376E+01_JPRB, 0.10223E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.10555E+01_JPRB, 0.10487E+01_JPRB, 0.10793E+01_JPRB, 0.10932E+01_JPRB, 0.10970E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11157E+01_JPRB, 0.99600E+00_JPRB, 0.78747E+00_JPRB, 0.72838E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.77548E+00_JPRB, 0.75097E+00_JPRB, 0.73495E+00_JPRB, 0.73333E+00_JPRB, 0.73247E+00_JPRB, 0.71574E+00_JPRB, 0.69276E+00_JPRB ,&
&  0.66279E+00_JPRB, 0.64045E+00_JPRB, 0.60984E+00_JPRB, 0.58060E+00_JPRB, 0.57676E+00_JPRB, 0.59934E+00_JPRB, 0.64107E+00_JPRB ,&
&  0.67841E+00_JPRB, 0.69331E+00_JPRB, 0.70346E+00_JPRB, 0.71112E+00_JPRB, 0.71867E+00_JPRB, 0.72222E+00_JPRB, 0.72298E+00_JPRB ,&
&  0.73305E+00_JPRB, 0.75490E+00_JPRB, 0.78800E+00_JPRB, 0.83152E+00_JPRB, 0.90038E+00_JPRB, 0.10321E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.18631E+01_JPRB, 0.21958E+01_JPRB, 0.10201E+01_JPRB, 0.21850E+01_JPRB, 0.31812E+01_JPRB, 0.32017E+01_JPRB, 0.30091E+01_JPRB ,&
&  0.27319E+01_JPRB, 0.22917E+01_JPRB, 0.21072E+01_JPRB, 0.18843E+01_JPRB, 0.16650E+01_JPRB, 0.15761E+01_JPRB, 0.15411E+01_JPRB ,&
&  0.15248E+01_JPRB, 0.14727E+01_JPRB, 0.14288E+01_JPRB, 0.14041E+01_JPRB, 0.13534E+01_JPRB, 0.12974E+01_JPRB, 0.12894E+01_JPRB ,&
&  0.13963E+01_JPRB, 0.17427E+01_JPRB, 0.25008E+01_JPRB, 0.28939E+01_JPRB, 0.30152E+01_JPRB, 0.24319E+01_JPRB, 0.19093E+01_JPRB ,&
&  0.17053E+01_JPRB, 0.17477E+01_JPRB, 0.16428E+01_JPRB, 0.14196E+01_JPRB, 0.12209E+01_JPRB, 0.11520E+01_JPRB, 0.11430E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10035E+01_JPRB, 0.95858E+00_JPRB, 0.92198E+00_JPRB, 0.89465E+00_JPRB, 0.87183E+00_JPRB, 0.85817E+00_JPRB ,&
&  0.86333E+00_JPRB, 0.86678E+00_JPRB, 0.87256E+00_JPRB, 0.88061E+00_JPRB, 0.88859E+00_JPRB, 0.90417E+00_JPRB, 0.93509E+00_JPRB ,&
&  0.97465E+00_JPRB, 0.10008E+01_JPRB, 0.10189E+01_JPRB, 0.10054E+01_JPRB, 0.99422E+00_JPRB, 0.99689E+00_JPRB, 0.10193E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10635E+01_JPRB, 0.10126E+01_JPRB, 0.10154E+01_JPRB, 0.10596E+01_JPRB, 0.10656E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.11399E+01_JPRB, 0.11945E+01_JPRB, 0.11673E+01_JPRB, 0.87683E+00_JPRB, 0.81881E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.83727E+00_JPRB, 0.80845E+00_JPRB, 0.80302E+00_JPRB, 0.80373E+00_JPRB, 0.80188E+00_JPRB, 0.79170E+00_JPRB, 0.77003E+00_JPRB ,&
&  0.74239E+00_JPRB, 0.72058E+00_JPRB, 0.69599E+00_JPRB, 0.66395E+00_JPRB, 0.61817E+00_JPRB, 0.57932E+00_JPRB, 0.57873E+00_JPRB ,&
&  0.60644E+00_JPRB, 0.61397E+00_JPRB, 0.61984E+00_JPRB, 0.62812E+00_JPRB, 0.63901E+00_JPRB, 0.65195E+00_JPRB, 0.66103E+00_JPRB ,&
&  0.66564E+00_JPRB, 0.67509E+00_JPRB, 0.68921E+00_JPRB, 0.70683E+00_JPRB, 0.73621E+00_JPRB, 0.79871E+00_JPRB, 0.92379E+00_JPRB ,&
&  0.12660E+01_JPRB, 0.17941E+01_JPRB, 0.19445E+01_JPRB, 0.32518E+01_JPRB, 0.37205E+01_JPRB, 0.35407E+01_JPRB, 0.35737E+01_JPRB ,&
&  0.35052E+01_JPRB, 0.30874E+01_JPRB, 0.26128E+01_JPRB, 0.20382E+01_JPRB, 0.17392E+01_JPRB, 0.15997E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.14622E+01_JPRB, 0.13971E+01_JPRB, 0.13054E+01_JPRB, 0.12494E+01_JPRB, 0.12041E+01_JPRB, 0.11659E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.14067E+01_JPRB, 0.19587E+01_JPRB, 0.27831E+01_JPRB, 0.30028E+01_JPRB, 0.27278E+01_JPRB, 0.23081E+01_JPRB ,&
&  0.23145E+01_JPRB, 0.22209E+01_JPRB, 0.20255E+01_JPRB, 0.16691E+01_JPRB, 0.13775E+01_JPRB, 0.13514E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.11131E+01_JPRB, 0.10530E+01_JPRB, 0.10191E+01_JPRB, 0.99448E+00_JPRB, 0.97445E+00_JPRB, 0.94805E+00_JPRB, 0.91641E+00_JPRB ,&
&  0.89504E+00_JPRB, 0.88884E+00_JPRB, 0.88178E+00_JPRB, 0.87917E+00_JPRB, 0.87613E+00_JPRB, 0.87712E+00_JPRB, 0.88758E+00_JPRB ,&
&  0.91899E+00_JPRB, 0.94824E+00_JPRB, 0.93611E+00_JPRB, 0.93805E+00_JPRB, 0.94341E+00_JPRB, 0.94615E+00_JPRB, 0.94889E+00_JPRB ,&
&  0.95747E+00_JPRB, 0.99203E+00_JPRB, 0.10090E+01_JPRB, 0.10147E+01_JPRB, 0.10300E+01_JPRB, 0.10430E+01_JPRB, 0.10404E+01_JPRB ,&
&  0.10471E+01_JPRB, 0.10308E+01_JPRB, 0.99119E+00_JPRB, 0.91420E+00_JPRB, 0.87683E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.86444E+00_JPRB, 0.83747E+00_JPRB, 0.82207E+00_JPRB, 0.81435E+00_JPRB, 0.80977E+00_JPRB, 0.80412E+00_JPRB, 0.78662E+00_JPRB ,&
&  0.76714E+00_JPRB, 0.75485E+00_JPRB, 0.73529E+00_JPRB, 0.71474E+00_JPRB, 0.67512E+00_JPRB, 0.62480E+00_JPRB, 0.60337E+00_JPRB ,&
&  0.59283E+00_JPRB, 0.59079E+00_JPRB, 0.59983E+00_JPRB, 0.60492E+00_JPRB, 0.60878E+00_JPRB, 0.61710E+00_JPRB, 0.62462E+00_JPRB ,&
&  0.63277E+00_JPRB, 0.63709E+00_JPRB, 0.64570E+00_JPRB, 0.65514E+00_JPRB, 0.66924E+00_JPRB, 0.71347E+00_JPRB, 0.79125E+00_JPRB ,&
&  0.10068E+01_JPRB, 0.16337E+01_JPRB, 0.14268E+01_JPRB, 0.31659E+01_JPRB, 0.35665E+01_JPRB, 0.35975E+01_JPRB, 0.34072E+01_JPRB ,&
&  0.34482E+01_JPRB, 0.34345E+01_JPRB, 0.26238E+01_JPRB, 0.21063E+01_JPRB, 0.17645E+01_JPRB, 0.15923E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.14178E+01_JPRB, 0.13404E+01_JPRB, 0.12385E+01_JPRB, 0.11498E+01_JPRB, 0.10989E+01_JPRB, 0.10636E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.11759E+01_JPRB, 0.15412E+01_JPRB, 0.22930E+01_JPRB, 0.29223E+01_JPRB, 0.29915E+01_JPRB, 0.27047E+01_JPRB ,&
&  0.27219E+01_JPRB, 0.27989E+01_JPRB, 0.25622E+01_JPRB, 0.20515E+01_JPRB, 0.16837E+01_JPRB, 0.15048E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.11246E+01_JPRB, 0.10836E+01_JPRB, 0.10486E+01_JPRB, 0.10176E+01_JPRB, 0.98630E+00_JPRB, 0.94802E+00_JPRB ,&
&  0.91345E+00_JPRB, 0.88557E+00_JPRB, 0.87385E+00_JPRB, 0.86570E+00_JPRB, 0.86521E+00_JPRB, 0.86541E+00_JPRB, 0.86740E+00_JPRB ,&
&  0.88145E+00_JPRB, 0.91018E+00_JPRB, 0.88759E+00_JPRB, 0.88817E+00_JPRB, 0.89548E+00_JPRB, 0.89963E+00_JPRB, 0.91558E+00_JPRB ,&
&  0.95095E+00_JPRB, 0.96333E+00_JPRB, 0.97889E+00_JPRB, 0.97776E+00_JPRB, 0.10062E+01_JPRB, 0.10476E+01_JPRB, 0.10209E+01_JPRB ,&
&  0.10129E+01_JPRB, 0.98544E+00_JPRB, 0.95438E+00_JPRB, 0.91830E+00_JPRB, 0.89086E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.86399E+00_JPRB, 0.84394E+00_JPRB, 0.82118E+00_JPRB, 0.80055E+00_JPRB, 0.79488E+00_JPRB, 0.79185E+00_JPRB, 0.77761E+00_JPRB ,&
&  0.76384E+00_JPRB, 0.74898E+00_JPRB, 0.72804E+00_JPRB, 0.70310E+00_JPRB, 0.67376E+00_JPRB, 0.63773E+00_JPRB, 0.60558E+00_JPRB ,&
&  0.59126E+00_JPRB, 0.57867E+00_JPRB, 0.58023E+00_JPRB, 0.58703E+00_JPRB, 0.59670E+00_JPRB, 0.60330E+00_JPRB, 0.61077E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.62211E+00_JPRB, 0.62749E+00_JPRB, 0.63256E+00_JPRB, 0.63847E+00_JPRB, 0.66598E+00_JPRB, 0.75027E+00_JPRB ,&
&  0.93025E+00_JPRB, 0.19386E+01_JPRB, 0.14481E+01_JPRB, 0.26003E+01_JPRB, 0.29776E+01_JPRB, 0.31832E+01_JPRB, 0.30305E+01_JPRB ,&
&  0.30805E+01_JPRB, 0.29011E+01_JPRB, 0.24068E+01_JPRB, 0.20315E+01_JPRB, 0.17593E+01_JPRB, 0.15852E+01_JPRB, 0.14663E+01_JPRB ,&
&  0.13817E+01_JPRB, 0.12874E+01_JPRB, 0.11845E+01_JPRB, 0.10860E+01_JPRB, 0.10258E+01_JPRB, 0.99134E+00_JPRB, 0.97493E+00_JPRB ,&
&  0.97250E+00_JPRB, 0.10277E+01_JPRB, 0.12406E+01_JPRB, 0.18859E+01_JPRB, 0.25676E+01_JPRB, 0.29846E+01_JPRB, 0.30561E+01_JPRB ,&
&  0.31129E+01_JPRB, 0.33340E+01_JPRB, 0.30750E+01_JPRB, 0.25007E+01_JPRB, 0.20493E+01_JPRB, 0.17215E+01_JPRB, 0.14740E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.12326E+01_JPRB, 0.11735E+01_JPRB, 0.11221E+01_JPRB, 0.10672E+01_JPRB, 0.10074E+01_JPRB, 0.94753E+00_JPRB ,&
&  0.90179E+00_JPRB, 0.86974E+00_JPRB, 0.85055E+00_JPRB, 0.84846E+00_JPRB, 0.84960E+00_JPRB, 0.85233E+00_JPRB, 0.85866E+00_JPRB ,&
&  0.86863E+00_JPRB, 0.88848E+00_JPRB, 0.87387E+00_JPRB, 0.87197E+00_JPRB, 0.87667E+00_JPRB, 0.88878E+00_JPRB, 0.90038E+00_JPRB ,&
&  0.94627E+00_JPRB, 0.95556E+00_JPRB, 0.94992E+00_JPRB, 0.96482E+00_JPRB, 0.10103E+01_JPRB, 0.10855E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.96994E+00_JPRB, 0.93318E+00_JPRB, 0.90918E+00_JPRB, 0.88809E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.85745E+00_JPRB, 0.83395E+00_JPRB, 0.81905E+00_JPRB, 0.80431E+00_JPRB, 0.79739E+00_JPRB, 0.79023E+00_JPRB, 0.77353E+00_JPRB ,&
&  0.75135E+00_JPRB, 0.72633E+00_JPRB, 0.70059E+00_JPRB, 0.67830E+00_JPRB, 0.66037E+00_JPRB, 0.63937E+00_JPRB, 0.62217E+00_JPRB ,&
&  0.60744E+00_JPRB, 0.59009E+00_JPRB, 0.57892E+00_JPRB, 0.57456E+00_JPRB, 0.57774E+00_JPRB, 0.58324E+00_JPRB, 0.59112E+00_JPRB ,&
&  0.60058E+00_JPRB, 0.60689E+00_JPRB, 0.60835E+00_JPRB, 0.61112E+00_JPRB, 0.61177E+00_JPRB, 0.63660E+00_JPRB, 0.72449E+00_JPRB ,&
&  0.93838E+00_JPRB, 0.90446E+00_JPRB, 0.16591E+01_JPRB, 0.20434E+01_JPRB, 0.23177E+01_JPRB, 0.25919E+01_JPRB, 0.27863E+01_JPRB ,&
&  0.28803E+01_JPRB, 0.25663E+01_JPRB, 0.21620E+01_JPRB, 0.18859E+01_JPRB, 0.16649E+01_JPRB, 0.15071E+01_JPRB, 0.13878E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.11860E+01_JPRB, 0.11159E+01_JPRB, 0.10287E+01_JPRB, 0.96716E+00_JPRB, 0.93704E+00_JPRB, 0.92258E+00_JPRB ,&
&  0.91070E+00_JPRB, 0.92831E+00_JPRB, 0.10378E+01_JPRB, 0.13960E+01_JPRB, 0.22111E+01_JPRB, 0.28393E+01_JPRB, 0.32805E+01_JPRB ,&
&  0.38564E+01_JPRB, 0.39931E+01_JPRB, 0.35798E+01_JPRB, 0.29427E+01_JPRB, 0.23282E+01_JPRB, 0.19166E+01_JPRB, 0.15844E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.12894E+01_JPRB, 0.12263E+01_JPRB, 0.11728E+01_JPRB, 0.11160E+01_JPRB, 0.10394E+01_JPRB, 0.95477E+00_JPRB ,&
&  0.88824E+00_JPRB, 0.84973E+00_JPRB, 0.82525E+00_JPRB, 0.81302E+00_JPRB, 0.80872E+00_JPRB, 0.81182E+00_JPRB, 0.82202E+00_JPRB ,&
&  0.83909E+00_JPRB, 0.86460E+00_JPRB, 0.86426E+00_JPRB, 0.85699E+00_JPRB, 0.86729E+00_JPRB, 0.88970E+00_JPRB, 0.91445E+00_JPRB ,&
&  0.93011E+00_JPRB, 0.91969E+00_JPRB, 0.90171E+00_JPRB, 0.92819E+00_JPRB, 0.98429E+00_JPRB, 0.10759E+01_JPRB, 0.10495E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.97814E+00_JPRB, 0.92575E+00_JPRB, 0.90086E+00_JPRB, 0.88637E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.84621E+00_JPRB, 0.82401E+00_JPRB, 0.81920E+00_JPRB, 0.82363E+00_JPRB, 0.82685E+00_JPRB, 0.81231E+00_JPRB, 0.77544E+00_JPRB ,&
&  0.74168E+00_JPRB, 0.71285E+00_JPRB, 0.68491E+00_JPRB, 0.66222E+00_JPRB, 0.64708E+00_JPRB, 0.63585E+00_JPRB, 0.62345E+00_JPRB ,&
&  0.61424E+00_JPRB, 0.60584E+00_JPRB, 0.58974E+00_JPRB, 0.57497E+00_JPRB, 0.56839E+00_JPRB, 0.56729E+00_JPRB, 0.57153E+00_JPRB ,&
&  0.57659E+00_JPRB, 0.58244E+00_JPRB, 0.58644E+00_JPRB, 0.58951E+00_JPRB, 0.59102E+00_JPRB, 0.61241E+00_JPRB, 0.68767E+00_JPRB ,&
&  0.93227E+00_JPRB, 0.14248E+01_JPRB, 0.17086E+01_JPRB, 0.17283E+01_JPRB, 0.18811E+01_JPRB, 0.21622E+01_JPRB, 0.25011E+01_JPRB ,&
&  0.24950E+01_JPRB, 0.22604E+01_JPRB, 0.19118E+01_JPRB, 0.16699E+01_JPRB, 0.14857E+01_JPRB, 0.13549E+01_JPRB, 0.12309E+01_JPRB ,&
&  0.11315E+01_JPRB, 0.10639E+01_JPRB, 0.99724E+00_JPRB, 0.93822E+00_JPRB, 0.91145E+00_JPRB, 0.90143E+00_JPRB, 0.89589E+00_JPRB ,&
&  0.88895E+00_JPRB, 0.88262E+00_JPRB, 0.92584E+00_JPRB, 0.11693E+01_JPRB, 0.18861E+01_JPRB, 0.23625E+01_JPRB, 0.32424E+01_JPRB ,&
&  0.40365E+01_JPRB, 0.39302E+01_JPRB, 0.35302E+01_JPRB, 0.29873E+01_JPRB, 0.24035E+01_JPRB, 0.19177E+01_JPRB, 0.16160E+01_JPRB ,&
&  0.14272E+01_JPRB, 0.12757E+01_JPRB, 0.11807E+01_JPRB, 0.11192E+01_JPRB, 0.10689E+01_JPRB, 0.10215E+01_JPRB, 0.96456E+00_JPRB ,&
&  0.90464E+00_JPRB, 0.85487E+00_JPRB, 0.82016E+00_JPRB, 0.79473E+00_JPRB, 0.78050E+00_JPRB, 0.77670E+00_JPRB, 0.77949E+00_JPRB ,&
&  0.78586E+00_JPRB, 0.80545E+00_JPRB, 0.83736E+00_JPRB, 0.82942E+00_JPRB, 0.84315E+00_JPRB, 0.86673E+00_JPRB, 0.88687E+00_JPRB ,&
&  0.87381E+00_JPRB, 0.87472E+00_JPRB, 0.86175E+00_JPRB, 0.88212E+00_JPRB, 0.10276E+01_JPRB, 0.11256E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.98071E+00_JPRB, 0.93634E+00_JPRB, 0.89458E+00_JPRB, 0.88134E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.84221E+00_JPRB, 0.81123E+00_JPRB, 0.79669E+00_JPRB, 0.80609E+00_JPRB, 0.82128E+00_JPRB, 0.81235E+00_JPRB, 0.77809E+00_JPRB ,&
&  0.73292E+00_JPRB, 0.69572E+00_JPRB, 0.66942E+00_JPRB, 0.65077E+00_JPRB, 0.63649E+00_JPRB, 0.62413E+00_JPRB, 0.61039E+00_JPRB ,&
&  0.59845E+00_JPRB, 0.59165E+00_JPRB, 0.58677E+00_JPRB, 0.58016E+00_JPRB, 0.56803E+00_JPRB, 0.55862E+00_JPRB, 0.55547E+00_JPRB ,&
&  0.55371E+00_JPRB, 0.55401E+00_JPRB, 0.55965E+00_JPRB, 0.56641E+00_JPRB, 0.57031E+00_JPRB, 0.58357E+00_JPRB, 0.63293E+00_JPRB ,&
&  0.80638E+00_JPRB, 0.12379E+01_JPRB, 0.12502E+01_JPRB, 0.13796E+01_JPRB, 0.16370E+01_JPRB, 0.18811E+01_JPRB, 0.21046E+01_JPRB ,&
&  0.21071E+01_JPRB, 0.19854E+01_JPRB, 0.17455E+01_JPRB, 0.15059E+01_JPRB, 0.13118E+01_JPRB, 0.11932E+01_JPRB, 0.11093E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.96719E+00_JPRB, 0.90637E+00_JPRB, 0.86102E+00_JPRB, 0.83005E+00_JPRB, 0.81636E+00_JPRB, 0.81593E+00_JPRB ,&
&  0.81656E+00_JPRB, 0.81411E+00_JPRB, 0.81952E+00_JPRB, 0.90413E+00_JPRB, 0.13031E+01_JPRB, 0.18276E+01_JPRB, 0.24139E+01_JPRB ,&
&  0.25805E+01_JPRB, 0.26486E+01_JPRB, 0.25982E+01_JPRB, 0.24372E+01_JPRB, 0.21726E+01_JPRB, 0.19148E+01_JPRB, 0.16378E+01_JPRB ,&
&  0.14348E+01_JPRB, 0.12939E+01_JPRB, 0.11907E+01_JPRB, 0.11039E+01_JPRB, 0.10259E+01_JPRB, 0.96375E+00_JPRB, 0.92073E+00_JPRB ,&
&  0.88786E+00_JPRB, 0.85832E+00_JPRB, 0.83008E+00_JPRB, 0.80374E+00_JPRB, 0.78359E+00_JPRB, 0.77184E+00_JPRB, 0.77102E+00_JPRB ,&
&  0.76941E+00_JPRB, 0.76340E+00_JPRB, 0.77165E+00_JPRB, 0.78166E+00_JPRB, 0.79453E+00_JPRB, 0.80124E+00_JPRB, 0.80496E+00_JPRB ,&
&  0.81348E+00_JPRB, 0.84806E+00_JPRB, 0.84435E+00_JPRB, 0.87736E+00_JPRB, 0.99558E+00_JPRB, 0.10728E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.99306E+00_JPRB, 0.94698E+00_JPRB, 0.91340E+00_JPRB, 0.88955E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.95655E+00_JPRB, 0.85874E+00_JPRB, 0.79919E+00_JPRB, 0.77586E+00_JPRB, 0.77318E+00_JPRB, 0.75759E+00_JPRB, 0.73321E+00_JPRB ,&
&  0.70755E+00_JPRB, 0.68624E+00_JPRB, 0.66921E+00_JPRB, 0.65043E+00_JPRB, 0.63136E+00_JPRB, 0.61500E+00_JPRB, 0.59869E+00_JPRB ,&
&  0.58248E+00_JPRB, 0.57279E+00_JPRB, 0.56579E+00_JPRB, 0.56078E+00_JPRB, 0.55692E+00_JPRB, 0.54978E+00_JPRB, 0.54221E+00_JPRB ,&
&  0.53646E+00_JPRB, 0.52869E+00_JPRB, 0.52662E+00_JPRB, 0.53380E+00_JPRB, 0.54193E+00_JPRB, 0.55332E+00_JPRB, 0.59023E+00_JPRB ,&
&  0.72385E+00_JPRB, 0.91182E+00_JPRB, 0.10461E+01_JPRB, 0.11679E+01_JPRB, 0.12939E+01_JPRB, 0.14412E+01_JPRB, 0.14908E+01_JPRB ,&
&  0.14211E+01_JPRB, 0.14294E+01_JPRB, 0.13917E+01_JPRB, 0.12697E+01_JPRB, 0.11329E+01_JPRB, 0.10405E+01_JPRB, 0.97467E+00_JPRB ,&
&  0.91945E+00_JPRB, 0.86363E+00_JPRB, 0.80861E+00_JPRB, 0.76503E+00_JPRB, 0.73880E+00_JPRB, 0.72836E+00_JPRB, 0.72731E+00_JPRB ,&
&  0.73042E+00_JPRB, 0.73677E+00_JPRB, 0.74071E+00_JPRB, 0.75366E+00_JPRB, 0.84396E+00_JPRB, 0.10824E+01_JPRB, 0.14586E+01_JPRB ,&
&  0.16349E+01_JPRB, 0.16824E+01_JPRB, 0.17145E+01_JPRB, 0.17092E+01_JPRB, 0.16086E+01_JPRB, 0.15008E+01_JPRB, 0.13868E+01_JPRB ,&
&  0.12724E+01_JPRB, 0.11512E+01_JPRB, 0.10479E+01_JPRB, 0.96630E+00_JPRB, 0.90272E+00_JPRB, 0.86019E+00_JPRB, 0.83775E+00_JPRB ,&
&  0.82736E+00_JPRB, 0.82098E+00_JPRB, 0.81501E+00_JPRB, 0.80326E+00_JPRB, 0.78748E+00_JPRB, 0.77603E+00_JPRB, 0.76821E+00_JPRB ,&
&  0.76007E+00_JPRB, 0.75410E+00_JPRB, 0.75113E+00_JPRB, 0.74974E+00_JPRB, 0.75258E+00_JPRB, 0.75382E+00_JPRB, 0.75837E+00_JPRB ,&
&  0.76611E+00_JPRB, 0.78677E+00_JPRB, 0.79072E+00_JPRB, 0.85969E+00_JPRB, 0.94747E+00_JPRB, 0.97364E+00_JPRB, 0.10061E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10171E+01_JPRB, 0.96408E+00_JPRB, 0.93954E+00_JPRB, 0.10298E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.87940E+00_JPRB, 0.83400E+00_JPRB, 0.77694E+00_JPRB, 0.74113E+00_JPRB, 0.71701E+00_JPRB, 0.68806E+00_JPRB, 0.66631E+00_JPRB ,&
&  0.65067E+00_JPRB, 0.63862E+00_JPRB, 0.62523E+00_JPRB, 0.60928E+00_JPRB, 0.59391E+00_JPRB, 0.58266E+00_JPRB, 0.57240E+00_JPRB ,&
&  0.56216E+00_JPRB, 0.55221E+00_JPRB, 0.54305E+00_JPRB, 0.53611E+00_JPRB, 0.52823E+00_JPRB, 0.52186E+00_JPRB, 0.51627E+00_JPRB ,&
&  0.50971E+00_JPRB, 0.50288E+00_JPRB, 0.49455E+00_JPRB, 0.49852E+00_JPRB, 0.50960E+00_JPRB, 0.52437E+00_JPRB, 0.54446E+00_JPRB ,&
&  0.62375E+00_JPRB, 0.71176E+00_JPRB, 0.83060E+00_JPRB, 0.96019E+00_JPRB, 0.10571E+01_JPRB, 0.11282E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10771E+01_JPRB, 0.10522E+01_JPRB, 0.99955E+00_JPRB, 0.96097E+00_JPRB, 0.88173E+00_JPRB, 0.80963E+00_JPRB ,&
&  0.75879E+00_JPRB, 0.72588E+00_JPRB, 0.69906E+00_JPRB, 0.67289E+00_JPRB, 0.65749E+00_JPRB, 0.65426E+00_JPRB, 0.65965E+00_JPRB ,&
&  0.66537E+00_JPRB, 0.67353E+00_JPRB, 0.68119E+00_JPRB, 0.69013E+00_JPRB, 0.70506E+00_JPRB, 0.77599E+00_JPRB, 0.95592E+00_JPRB ,&
&  0.10886E+01_JPRB, 0.11646E+01_JPRB, 0.12100E+01_JPRB, 0.12257E+01_JPRB, 0.11912E+01_JPRB, 0.11334E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.99532E+00_JPRB, 0.91915E+00_JPRB, 0.84851E+00_JPRB, 0.79271E+00_JPRB, 0.74874E+00_JPRB, 0.71903E+00_JPRB, 0.70252E+00_JPRB ,&
&  0.69903E+00_JPRB, 0.70547E+00_JPRB, 0.71712E+00_JPRB, 0.72881E+00_JPRB, 0.73543E+00_JPRB, 0.73535E+00_JPRB, 0.73177E+00_JPRB ,&
&  0.72564E+00_JPRB, 0.71798E+00_JPRB, 0.71119E+00_JPRB, 0.70685E+00_JPRB, 0.70436E+00_JPRB, 0.70383E+00_JPRB, 0.70539E+00_JPRB ,&
&  0.70879E+00_JPRB, 0.71777E+00_JPRB, 0.73118E+00_JPRB, 0.78799E+00_JPRB, 0.81366E+00_JPRB, 0.84278E+00_JPRB, 0.86760E+00_JPRB ,&
&  0.89590E+00_JPRB, 0.90607E+00_JPRB, 0.88933E+00_JPRB, 0.85329E+00_JPRB, 0.85121E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.77008E+00_JPRB, 0.75492E+00_JPRB, 0.72540E+00_JPRB, 0.69357E+00_JPRB, 0.66663E+00_JPRB, 0.64203E+00_JPRB, 0.62175E+00_JPRB ,&
&  0.60659E+00_JPRB, 0.59357E+00_JPRB, 0.58019E+00_JPRB, 0.56775E+00_JPRB, 0.55501E+00_JPRB, 0.54406E+00_JPRB, 0.53325E+00_JPRB ,&
&  0.52548E+00_JPRB, 0.51988E+00_JPRB, 0.51458E+00_JPRB, 0.50844E+00_JPRB, 0.50218E+00_JPRB, 0.49595E+00_JPRB, 0.48879E+00_JPRB ,&
&  0.48268E+00_JPRB, 0.47924E+00_JPRB, 0.47198E+00_JPRB, 0.47014E+00_JPRB, 0.47874E+00_JPRB, 0.49210E+00_JPRB, 0.50580E+00_JPRB ,&
&  0.55902E+00_JPRB, 0.56594E+00_JPRB, 0.59030E+00_JPRB, 0.70844E+00_JPRB, 0.80649E+00_JPRB, 0.91395E+00_JPRB, 0.96002E+00_JPRB ,&
&  0.96677E+00_JPRB, 0.97571E+00_JPRB, 0.94324E+00_JPRB, 0.89907E+00_JPRB, 0.83089E+00_JPRB, 0.77429E+00_JPRB, 0.73241E+00_JPRB ,&
&  0.69566E+00_JPRB, 0.66131E+00_JPRB, 0.63458E+00_JPRB, 0.61430E+00_JPRB, 0.59967E+00_JPRB, 0.59412E+00_JPRB, 0.59660E+00_JPRB ,&
&  0.60070E+00_JPRB, 0.60494E+00_JPRB, 0.61098E+00_JPRB, 0.62172E+00_JPRB, 0.63474E+00_JPRB, 0.65353E+00_JPRB, 0.71556E+00_JPRB ,&
&  0.81295E+00_JPRB, 0.88227E+00_JPRB, 0.93929E+00_JPRB, 0.96759E+00_JPRB, 0.95787E+00_JPRB, 0.91816E+00_JPRB, 0.85239E+00_JPRB ,&
&  0.78427E+00_JPRB, 0.73109E+00_JPRB, 0.69701E+00_JPRB, 0.67468E+00_JPRB, 0.66109E+00_JPRB, 0.65002E+00_JPRB, 0.63988E+00_JPRB ,&
&  0.63039E+00_JPRB, 0.62390E+00_JPRB, 0.62321E+00_JPRB, 0.62858E+00_JPRB, 0.63755E+00_JPRB, 0.64533E+00_JPRB, 0.64876E+00_JPRB ,&
&  0.64870E+00_JPRB, 0.64705E+00_JPRB, 0.64716E+00_JPRB, 0.64741E+00_JPRB, 0.64702E+00_JPRB, 0.64483E+00_JPRB, 0.64541E+00_JPRB ,&
&  0.64906E+00_JPRB, 0.65708E+00_JPRB, 0.66716E+00_JPRB, 0.72445E+00_JPRB, 0.75722E+00_JPRB, 0.76692E+00_JPRB, 0.77242E+00_JPRB ,&
&  0.78183E+00_JPRB, 0.78106E+00_JPRB, 0.77951E+00_JPRB, 0.68972E+00_JPRB, 0.75741E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.66294E+00_JPRB, 0.65850E+00_JPRB, 0.64034E+00_JPRB, 0.62078E+00_JPRB, 0.60082E+00_JPRB, 0.58159E+00_JPRB, 0.56725E+00_JPRB ,&
&  0.55649E+00_JPRB, 0.54316E+00_JPRB, 0.53164E+00_JPRB, 0.52367E+00_JPRB, 0.51519E+00_JPRB, 0.50717E+00_JPRB, 0.49810E+00_JPRB ,&
&  0.49078E+00_JPRB, 0.48612E+00_JPRB, 0.48272E+00_JPRB, 0.47895E+00_JPRB, 0.47474E+00_JPRB, 0.46974E+00_JPRB, 0.46484E+00_JPRB ,&
&  0.46012E+00_JPRB, 0.45661E+00_JPRB, 0.45181E+00_JPRB, 0.44916E+00_JPRB, 0.45331E+00_JPRB, 0.46321E+00_JPRB, 0.47271E+00_JPRB ,&
&  0.51869E+00_JPRB, 0.47479E+00_JPRB, 0.48841E+00_JPRB, 0.54216E+00_JPRB, 0.60806E+00_JPRB, 0.69440E+00_JPRB, 0.76631E+00_JPRB ,&
&  0.79748E+00_JPRB, 0.82606E+00_JPRB, 0.81065E+00_JPRB, 0.78137E+00_JPRB, 0.73206E+00_JPRB, 0.68437E+00_JPRB, 0.64857E+00_JPRB ,&
&  0.62763E+00_JPRB, 0.61181E+00_JPRB, 0.59377E+00_JPRB, 0.57399E+00_JPRB, 0.55604E+00_JPRB, 0.54386E+00_JPRB, 0.53879E+00_JPRB ,&
&  0.53996E+00_JPRB, 0.54221E+00_JPRB, 0.54279E+00_JPRB, 0.54650E+00_JPRB, 0.55574E+00_JPRB, 0.56920E+00_JPRB, 0.59019E+00_JPRB ,&
&  0.62734E+00_JPRB, 0.67786E+00_JPRB, 0.72006E+00_JPRB, 0.75767E+00_JPRB, 0.77765E+00_JPRB, 0.77671E+00_JPRB, 0.74268E+00_JPRB ,&
&  0.69885E+00_JPRB, 0.65950E+00_JPRB, 0.63254E+00_JPRB, 0.61677E+00_JPRB, 0.60597E+00_JPRB, 0.59736E+00_JPRB, 0.58628E+00_JPRB ,&
&  0.57251E+00_JPRB, 0.55798E+00_JPRB, 0.54610E+00_JPRB, 0.53644E+00_JPRB, 0.53045E+00_JPRB, 0.52969E+00_JPRB, 0.53265E+00_JPRB ,&
&  0.53660E+00_JPRB, 0.54007E+00_JPRB, 0.54376E+00_JPRB, 0.54907E+00_JPRB, 0.55455E+00_JPRB, 0.56029E+00_JPRB, 0.56494E+00_JPRB ,&
&  0.57116E+00_JPRB, 0.57920E+00_JPRB, 0.58709E+00_JPRB, 0.61711E+00_JPRB, 0.64289E+00_JPRB, 0.65661E+00_JPRB, 0.66509E+00_JPRB ,&
&  0.66855E+00_JPRB, 0.67668E+00_JPRB, 0.58793E+00_JPRB, 0.63275E+00_JPRB, 0.65395E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.57809E+00_JPRB, 0.57554E+00_JPRB, 0.56865E+00_JPRB, 0.55483E+00_JPRB, 0.53925E+00_JPRB, 0.52573E+00_JPRB, 0.51716E+00_JPRB ,&
&  0.51120E+00_JPRB, 0.50302E+00_JPRB, 0.49591E+00_JPRB, 0.49065E+00_JPRB, 0.48425E+00_JPRB, 0.47687E+00_JPRB, 0.46739E+00_JPRB ,&
&  0.45960E+00_JPRB, 0.45356E+00_JPRB, 0.44918E+00_JPRB, 0.44650E+00_JPRB, 0.44474E+00_JPRB, 0.44302E+00_JPRB, 0.44047E+00_JPRB ,&
&  0.43796E+00_JPRB, 0.43643E+00_JPRB, 0.43493E+00_JPRB, 0.43419E+00_JPRB, 0.43535E+00_JPRB, 0.44158E+00_JPRB, 0.44996E+00_JPRB ,&
&  0.44858E+00_JPRB, 0.42811E+00_JPRB, 0.44071E+00_JPRB, 0.46577E+00_JPRB, 0.50089E+00_JPRB, 0.56023E+00_JPRB, 0.62253E+00_JPRB ,&
&  0.66012E+00_JPRB, 0.68120E+00_JPRB, 0.66601E+00_JPRB, 0.64459E+00_JPRB, 0.62221E+00_JPRB, 0.59617E+00_JPRB, 0.57494E+00_JPRB ,&
&  0.55786E+00_JPRB, 0.54638E+00_JPRB, 0.53710E+00_JPRB, 0.52417E+00_JPRB, 0.50890E+00_JPRB, 0.49654E+00_JPRB, 0.48920E+00_JPRB ,&
&  0.48670E+00_JPRB, 0.48838E+00_JPRB, 0.49089E+00_JPRB, 0.49326E+00_JPRB, 0.49756E+00_JPRB, 0.50618E+00_JPRB, 0.51783E+00_JPRB ,&
&  0.52875E+00_JPRB, 0.54528E+00_JPRB, 0.56769E+00_JPRB, 0.58885E+00_JPRB, 0.60793E+00_JPRB, 0.61874E+00_JPRB, 0.62199E+00_JPRB ,&
&  0.61570E+00_JPRB, 0.60037E+00_JPRB, 0.58207E+00_JPRB, 0.56354E+00_JPRB, 0.54347E+00_JPRB, 0.52099E+00_JPRB, 0.49835E+00_JPRB ,&
&  0.47965E+00_JPRB, 0.46521E+00_JPRB, 0.45533E+00_JPRB, 0.45091E+00_JPRB, 0.44973E+00_JPRB, 0.45196E+00_JPRB, 0.45603E+00_JPRB ,&
&  0.46088E+00_JPRB, 0.46604E+00_JPRB, 0.46996E+00_JPRB, 0.47504E+00_JPRB, 0.48021E+00_JPRB, 0.48630E+00_JPRB, 0.49280E+00_JPRB ,&
&  0.50018E+00_JPRB, 0.50736E+00_JPRB, 0.51481E+00_JPRB, 0.53292E+00_JPRB, 0.55397E+00_JPRB, 0.57243E+00_JPRB, 0.58211E+00_JPRB ,&
&  0.58210E+00_JPRB, 0.58035E+00_JPRB, 0.57973E+00_JPRB, 0.57696E+00_JPRB, 0.57999E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.51816E+00_JPRB, 0.51507E+00_JPRB, 0.51170E+00_JPRB, 0.50733E+00_JPRB, 0.50100E+00_JPRB, 0.49254E+00_JPRB, 0.48599E+00_JPRB ,&
&  0.48139E+00_JPRB, 0.47607E+00_JPRB, 0.47108E+00_JPRB, 0.46497E+00_JPRB, 0.45777E+00_JPRB, 0.44939E+00_JPRB, 0.43911E+00_JPRB ,&
&  0.42998E+00_JPRB, 0.42182E+00_JPRB, 0.41645E+00_JPRB, 0.41381E+00_JPRB, 0.41354E+00_JPRB, 0.41339E+00_JPRB, 0.41145E+00_JPRB ,&
&  0.41027E+00_JPRB, 0.41013E+00_JPRB, 0.40999E+00_JPRB, 0.40975E+00_JPRB, 0.41021E+00_JPRB, 0.41480E+00_JPRB, 0.41977E+00_JPRB ,&
&  0.43499E+00_JPRB, 0.41627E+00_JPRB, 0.43127E+00_JPRB, 0.43740E+00_JPRB, 0.45151E+00_JPRB, 0.48400E+00_JPRB, 0.52813E+00_JPRB ,&
&  0.56396E+00_JPRB, 0.56648E+00_JPRB, 0.55078E+00_JPRB, 0.53308E+00_JPRB, 0.52437E+00_JPRB, 0.51397E+00_JPRB, 0.50202E+00_JPRB ,&
&  0.49072E+00_JPRB, 0.48275E+00_JPRB, 0.47619E+00_JPRB, 0.46996E+00_JPRB, 0.46175E+00_JPRB, 0.45151E+00_JPRB, 0.44203E+00_JPRB ,&
&  0.43567E+00_JPRB, 0.43366E+00_JPRB, 0.43523E+00_JPRB, 0.43860E+00_JPRB, 0.44236E+00_JPRB, 0.44867E+00_JPRB, 0.45684E+00_JPRB ,&
&  0.46473E+00_JPRB, 0.46807E+00_JPRB, 0.47051E+00_JPRB, 0.47602E+00_JPRB, 0.48224E+00_JPRB, 0.48609E+00_JPRB, 0.48700E+00_JPRB ,&
&  0.48531E+00_JPRB, 0.47952E+00_JPRB, 0.47136E+00_JPRB, 0.46125E+00_JPRB, 0.45047E+00_JPRB, 0.43974E+00_JPRB, 0.43040E+00_JPRB ,&
&  0.42288E+00_JPRB, 0.41622E+00_JPRB, 0.41029E+00_JPRB, 0.40603E+00_JPRB, 0.40431E+00_JPRB, 0.40446E+00_JPRB, 0.40612E+00_JPRB ,&
&  0.40993E+00_JPRB, 0.41446E+00_JPRB, 0.41815E+00_JPRB, 0.42175E+00_JPRB, 0.42507E+00_JPRB, 0.42808E+00_JPRB, 0.43247E+00_JPRB ,&
&  0.43871E+00_JPRB, 0.44367E+00_JPRB, 0.44815E+00_JPRB, 0.45604E+00_JPRB, 0.46990E+00_JPRB, 0.48741E+00_JPRB, 0.50218E+00_JPRB ,&
&  0.51030E+00_JPRB, 0.51162E+00_JPRB, 0.51404E+00_JPRB, 0.51545E+00_JPRB, 0.51907E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.47752E+00_JPRB, 0.47575E+00_JPRB, 0.47278E+00_JPRB, 0.47063E+00_JPRB, 0.46764E+00_JPRB, 0.46320E+00_JPRB, 0.45827E+00_JPRB ,&
&  0.45410E+00_JPRB, 0.44993E+00_JPRB, 0.44518E+00_JPRB, 0.43896E+00_JPRB, 0.43075E+00_JPRB, 0.42285E+00_JPRB, 0.41416E+00_JPRB ,&
&  0.40610E+00_JPRB, 0.39923E+00_JPRB, 0.39445E+00_JPRB, 0.39161E+00_JPRB, 0.38925E+00_JPRB, 0.38716E+00_JPRB, 0.38473E+00_JPRB ,&
&  0.38324E+00_JPRB, 0.38270E+00_JPRB, 0.38274E+00_JPRB, 0.38327E+00_JPRB, 0.38404E+00_JPRB, 0.38729E+00_JPRB, 0.38961E+00_JPRB ,&
&  0.39539E+00_JPRB, 0.41197E+00_JPRB, 0.42265E+00_JPRB, 0.42672E+00_JPRB, 0.43321E+00_JPRB, 0.44465E+00_JPRB, 0.46046E+00_JPRB ,&
&  0.47509E+00_JPRB, 0.48151E+00_JPRB, 0.47368E+00_JPRB, 0.46098E+00_JPRB, 0.45344E+00_JPRB, 0.45018E+00_JPRB, 0.44269E+00_JPRB ,&
&  0.43498E+00_JPRB, 0.42906E+00_JPRB, 0.42447E+00_JPRB, 0.42099E+00_JPRB, 0.41761E+00_JPRB, 0.41279E+00_JPRB, 0.40736E+00_JPRB ,&
&  0.40073E+00_JPRB, 0.39557E+00_JPRB, 0.39177E+00_JPRB, 0.39039E+00_JPRB, 0.39068E+00_JPRB, 0.39246E+00_JPRB, 0.39569E+00_JPRB ,&
&  0.40074E+00_JPRB, 0.40520E+00_JPRB, 0.40691E+00_JPRB, 0.40711E+00_JPRB, 0.40714E+00_JPRB, 0.40779E+00_JPRB, 0.40768E+00_JPRB ,&
&  0.40606E+00_JPRB, 0.40321E+00_JPRB, 0.39943E+00_JPRB, 0.39486E+00_JPRB, 0.38987E+00_JPRB, 0.38569E+00_JPRB, 0.38222E+00_JPRB ,&
&  0.37901E+00_JPRB, 0.37572E+00_JPRB, 0.37248E+00_JPRB, 0.37063E+00_JPRB, 0.36949E+00_JPRB, 0.36900E+00_JPRB, 0.36993E+00_JPRB ,&
&  0.37235E+00_JPRB, 0.37534E+00_JPRB, 0.37865E+00_JPRB, 0.38171E+00_JPRB, 0.38465E+00_JPRB, 0.38716E+00_JPRB, 0.39039E+00_JPRB ,&
&  0.39574E+00_JPRB, 0.40172E+00_JPRB, 0.40759E+00_JPRB, 0.41331E+00_JPRB, 0.42160E+00_JPRB, 0.43429E+00_JPRB, 0.44846E+00_JPRB ,&
&  0.45813E+00_JPRB, 0.46311E+00_JPRB, 0.46705E+00_JPRB, 0.47185E+00_JPRB, 0.47628E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.43059E+00_JPRB, 0.43247E+00_JPRB, 0.43296E+00_JPRB, 0.43389E+00_JPRB, 0.43380E+00_JPRB, 0.43206E+00_JPRB, 0.42815E+00_JPRB ,&
&  0.42364E+00_JPRB, 0.41852E+00_JPRB, 0.41316E+00_JPRB, 0.40637E+00_JPRB, 0.39936E+00_JPRB, 0.39357E+00_JPRB, 0.38741E+00_JPRB ,&
&  0.38123E+00_JPRB, 0.37557E+00_JPRB, 0.37152E+00_JPRB, 0.36828E+00_JPRB, 0.36508E+00_JPRB, 0.36189E+00_JPRB, 0.35789E+00_JPRB ,&
&  0.35443E+00_JPRB, 0.35135E+00_JPRB, 0.35158E+00_JPRB, 0.35424E+00_JPRB, 0.35653E+00_JPRB, 0.35824E+00_JPRB, 0.35990E+00_JPRB ,&
&  0.36104E+00_JPRB, 0.38025E+00_JPRB, 0.37594E+00_JPRB, 0.39023E+00_JPRB, 0.39531E+00_JPRB, 0.39615E+00_JPRB, 0.39739E+00_JPRB ,&
&  0.40169E+00_JPRB, 0.40723E+00_JPRB, 0.41130E+00_JPRB, 0.40638E+00_JPRB, 0.39879E+00_JPRB, 0.39301E+00_JPRB, 0.38713E+00_JPRB ,&
&  0.37933E+00_JPRB, 0.37577E+00_JPRB, 0.37526E+00_JPRB, 0.37527E+00_JPRB, 0.37528E+00_JPRB, 0.37474E+00_JPRB, 0.37325E+00_JPRB ,&
&  0.37080E+00_JPRB, 0.36729E+00_JPRB, 0.36325E+00_JPRB, 0.35932E+00_JPRB, 0.35672E+00_JPRB, 0.35418E+00_JPRB, 0.35225E+00_JPRB ,&
&  0.35206E+00_JPRB, 0.35361E+00_JPRB, 0.35492E+00_JPRB, 0.35514E+00_JPRB, 0.35418E+00_JPRB, 0.35328E+00_JPRB, 0.35271E+00_JPRB ,&
&  0.35159E+00_JPRB, 0.35004E+00_JPRB, 0.34805E+00_JPRB, 0.34556E+00_JPRB, 0.34293E+00_JPRB, 0.34119E+00_JPRB, 0.34043E+00_JPRB ,&
&  0.33955E+00_JPRB, 0.33763E+00_JPRB, 0.33495E+00_JPRB, 0.33429E+00_JPRB, 0.33422E+00_JPRB, 0.33455E+00_JPRB, 0.33509E+00_JPRB ,&
&  0.33629E+00_JPRB, 0.33857E+00_JPRB, 0.34121E+00_JPRB, 0.34388E+00_JPRB, 0.34664E+00_JPRB, 0.35058E+00_JPRB, 0.35516E+00_JPRB ,&
&  0.36025E+00_JPRB, 0.36573E+00_JPRB, 0.37104E+00_JPRB, 0.37595E+00_JPRB, 0.38076E+00_JPRB, 0.38754E+00_JPRB, 0.39719E+00_JPRB ,&
&  0.40584E+00_JPRB, 0.41231E+00_JPRB, 0.41661E+00_JPRB, 0.42130E+00_JPRB, 0.42652E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.39082E+00_JPRB, 0.39627E+00_JPRB, 0.39946E+00_JPRB, 0.40106E+00_JPRB, 0.40160E+00_JPRB, 0.39996E+00_JPRB, 0.39680E+00_JPRB ,&
&  0.39177E+00_JPRB, 0.38656E+00_JPRB, 0.38102E+00_JPRB, 0.37501E+00_JPRB, 0.36960E+00_JPRB, 0.36494E+00_JPRB, 0.36081E+00_JPRB ,&
&  0.35669E+00_JPRB, 0.35288E+00_JPRB, 0.34975E+00_JPRB, 0.34661E+00_JPRB, 0.34299E+00_JPRB, 0.33847E+00_JPRB, 0.33283E+00_JPRB ,&
&  0.32929E+00_JPRB, 0.32656E+00_JPRB, 0.32734E+00_JPRB, 0.33045E+00_JPRB, 0.33296E+00_JPRB, 0.33445E+00_JPRB, 0.33594E+00_JPRB ,&
&  0.33813E+00_JPRB, 0.34355E+00_JPRB, 0.34803E+00_JPRB, 0.35363E+00_JPRB, 0.35751E+00_JPRB, 0.35743E+00_JPRB, 0.35498E+00_JPRB ,&
&  0.35359E+00_JPRB, 0.35485E+00_JPRB, 0.35718E+00_JPRB, 0.35869E+00_JPRB, 0.35750E+00_JPRB, 0.35410E+00_JPRB, 0.34866E+00_JPRB ,&
&  0.33992E+00_JPRB, 0.33344E+00_JPRB, 0.33202E+00_JPRB, 0.33343E+00_JPRB, 0.33566E+00_JPRB, 0.33734E+00_JPRB, 0.33830E+00_JPRB ,&
&  0.33854E+00_JPRB, 0.33765E+00_JPRB, 0.33609E+00_JPRB, 0.33413E+00_JPRB, 0.33166E+00_JPRB, 0.32907E+00_JPRB, 0.32744E+00_JPRB ,&
&  0.32591E+00_JPRB, 0.32486E+00_JPRB, 0.32439E+00_JPRB, 0.32406E+00_JPRB, 0.32276E+00_JPRB, 0.32066E+00_JPRB, 0.31923E+00_JPRB ,&
&  0.31743E+00_JPRB, 0.31621E+00_JPRB, 0.31456E+00_JPRB, 0.31287E+00_JPRB, 0.31090E+00_JPRB, 0.30910E+00_JPRB, 0.30835E+00_JPRB ,&
&  0.30828E+00_JPRB, 0.30699E+00_JPRB, 0.30549E+00_JPRB, 0.30544E+00_JPRB, 0.30610E+00_JPRB, 0.30706E+00_JPRB, 0.30778E+00_JPRB ,&
&  0.30849E+00_JPRB, 0.30941E+00_JPRB, 0.31157E+00_JPRB, 0.31415E+00_JPRB, 0.31716E+00_JPRB, 0.32082E+00_JPRB, 0.32501E+00_JPRB ,&
&  0.32988E+00_JPRB, 0.33549E+00_JPRB, 0.34172E+00_JPRB, 0.34741E+00_JPRB, 0.35233E+00_JPRB, 0.35634E+00_JPRB, 0.36205E+00_JPRB ,&
&  0.36887E+00_JPRB, 0.37403E+00_JPRB, 0.37789E+00_JPRB, 0.38154E+00_JPRB, 0.38546E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.35201E+00_JPRB, 0.35757E+00_JPRB, 0.36189E+00_JPRB, 0.36436E+00_JPRB, 0.36459E+00_JPRB, 0.36326E+00_JPRB, 0.36017E+00_JPRB ,&
&  0.35560E+00_JPRB, 0.35089E+00_JPRB, 0.34623E+00_JPRB, 0.34243E+00_JPRB, 0.33987E+00_JPRB, 0.33765E+00_JPRB, 0.33566E+00_JPRB ,&
&  0.33313E+00_JPRB, 0.33024E+00_JPRB, 0.32689E+00_JPRB, 0.32395E+00_JPRB, 0.32039E+00_JPRB, 0.31563E+00_JPRB, 0.31198E+00_JPRB ,&
&  0.30936E+00_JPRB, 0.30703E+00_JPRB, 0.30635E+00_JPRB, 0.30777E+00_JPRB, 0.31037E+00_JPRB, 0.31282E+00_JPRB, 0.31569E+00_JPRB ,&
&  0.31793E+00_JPRB, 0.32024E+00_JPRB, 0.32242E+00_JPRB, 0.32283E+00_JPRB, 0.32277E+00_JPRB, 0.32246E+00_JPRB, 0.32183E+00_JPRB ,&
&  0.32119E+00_JPRB, 0.32067E+00_JPRB, 0.31972E+00_JPRB, 0.31978E+00_JPRB, 0.31974E+00_JPRB, 0.31847E+00_JPRB, 0.31610E+00_JPRB ,&
&  0.31208E+00_JPRB, 0.30655E+00_JPRB, 0.30230E+00_JPRB, 0.30035E+00_JPRB, 0.29989E+00_JPRB, 0.30030E+00_JPRB, 0.30125E+00_JPRB ,&
&  0.30242E+00_JPRB, 0.30413E+00_JPRB, 0.30578E+00_JPRB, 0.30659E+00_JPRB, 0.30635E+00_JPRB, 0.30603E+00_JPRB, 0.30452E+00_JPRB ,&
&  0.30308E+00_JPRB, 0.30209E+00_JPRB, 0.30150E+00_JPRB, 0.30083E+00_JPRB, 0.29844E+00_JPRB, 0.29542E+00_JPRB, 0.29443E+00_JPRB ,&
&  0.29249E+00_JPRB, 0.29119E+00_JPRB, 0.28942E+00_JPRB, 0.28833E+00_JPRB, 0.28714E+00_JPRB, 0.28557E+00_JPRB, 0.28396E+00_JPRB ,&
&  0.28275E+00_JPRB, 0.28209E+00_JPRB, 0.28173E+00_JPRB, 0.28090E+00_JPRB, 0.28124E+00_JPRB, 0.28184E+00_JPRB, 0.28330E+00_JPRB ,&
&  0.28461E+00_JPRB, 0.28584E+00_JPRB, 0.28756E+00_JPRB, 0.28939E+00_JPRB, 0.29211E+00_JPRB, 0.29530E+00_JPRB, 0.29898E+00_JPRB ,&
&  0.30220E+00_JPRB, 0.30652E+00_JPRB, 0.31139E+00_JPRB, 0.31599E+00_JPRB, 0.31963E+00_JPRB, 0.32248E+00_JPRB, 0.32664E+00_JPRB ,&
&  0.33313E+00_JPRB, 0.33851E+00_JPRB, 0.34150E+00_JPRB, 0.34460E+00_JPRB, 0.34742E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.32224E+00_JPRB, 0.32434E+00_JPRB, 0.32640E+00_JPRB, 0.32857E+00_JPRB, 0.33012E+00_JPRB, 0.33065E+00_JPRB, 0.32922E+00_JPRB ,&
&  0.32632E+00_JPRB, 0.32289E+00_JPRB, 0.31943E+00_JPRB, 0.31645E+00_JPRB, 0.31455E+00_JPRB, 0.31328E+00_JPRB, 0.31233E+00_JPRB ,&
&  0.31144E+00_JPRB, 0.31053E+00_JPRB, 0.30820E+00_JPRB, 0.30497E+00_JPRB, 0.30078E+00_JPRB, 0.29711E+00_JPRB, 0.29577E+00_JPRB ,&
&  0.29386E+00_JPRB, 0.29260E+00_JPRB, 0.29172E+00_JPRB, 0.29079E+00_JPRB, 0.29039E+00_JPRB, 0.29136E+00_JPRB, 0.29337E+00_JPRB ,&
&  0.29551E+00_JPRB, 0.29746E+00_JPRB, 0.29915E+00_JPRB, 0.29994E+00_JPRB, 0.29927E+00_JPRB, 0.29832E+00_JPRB, 0.29714E+00_JPRB ,&
&  0.29563E+00_JPRB, 0.29405E+00_JPRB, 0.29263E+00_JPRB, 0.29078E+00_JPRB, 0.28812E+00_JPRB, 0.28483E+00_JPRB, 0.28256E+00_JPRB ,&
&  0.28092E+00_JPRB, 0.27933E+00_JPRB, 0.27765E+00_JPRB, 0.27679E+00_JPRB, 0.27613E+00_JPRB, 0.27589E+00_JPRB, 0.27602E+00_JPRB ,&
&  0.27657E+00_JPRB, 0.27795E+00_JPRB, 0.27990E+00_JPRB, 0.28208E+00_JPRB, 0.28286E+00_JPRB, 0.28314E+00_JPRB, 0.28330E+00_JPRB ,&
&  0.28240E+00_JPRB, 0.28080E+00_JPRB, 0.27876E+00_JPRB, 0.27648E+00_JPRB, 0.27484E+00_JPRB, 0.27353E+00_JPRB, 0.27206E+00_JPRB ,&
&  0.26976E+00_JPRB, 0.26833E+00_JPRB, 0.26759E+00_JPRB, 0.26742E+00_JPRB, 0.26760E+00_JPRB, 0.26759E+00_JPRB, 0.26676E+00_JPRB ,&
&  0.26574E+00_JPRB, 0.26489E+00_JPRB, 0.26383E+00_JPRB, 0.26259E+00_JPRB, 0.26059E+00_JPRB, 0.26102E+00_JPRB, 0.26223E+00_JPRB ,&
&  0.26278E+00_JPRB, 0.26294E+00_JPRB, 0.26315E+00_JPRB, 0.26447E+00_JPRB, 0.26598E+00_JPRB, 0.26800E+00_JPRB, 0.27073E+00_JPRB ,&
&  0.27310E+00_JPRB, 0.27635E+00_JPRB, 0.28097E+00_JPRB, 0.28529E+00_JPRB, 0.28835E+00_JPRB, 0.28997E+00_JPRB, 0.29327E+00_JPRB ,&
&  0.30014E+00_JPRB, 0.30736E+00_JPRB, 0.31202E+00_JPRB, 0.31640E+00_JPRB, 0.32033E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.29534E+00_JPRB, 0.29655E+00_JPRB, 0.29762E+00_JPRB, 0.29890E+00_JPRB, 0.30055E+00_JPRB, 0.30153E+00_JPRB, 0.30226E+00_JPRB ,&
&  0.30153E+00_JPRB, 0.29951E+00_JPRB, 0.29714E+00_JPRB, 0.29469E+00_JPRB, 0.29352E+00_JPRB, 0.29321E+00_JPRB, 0.29316E+00_JPRB ,&
&  0.29252E+00_JPRB, 0.29216E+00_JPRB, 0.29014E+00_JPRB, 0.28683E+00_JPRB, 0.28289E+00_JPRB, 0.28011E+00_JPRB, 0.27922E+00_JPRB ,&
&  0.27852E+00_JPRB, 0.27752E+00_JPRB, 0.27696E+00_JPRB, 0.27609E+00_JPRB, 0.27340E+00_JPRB, 0.27209E+00_JPRB, 0.27254E+00_JPRB ,&
&  0.27347E+00_JPRB, 0.27511E+00_JPRB, 0.27530E+00_JPRB, 0.27307E+00_JPRB, 0.27531E+00_JPRB, 0.27522E+00_JPRB, 0.27338E+00_JPRB ,&
&  0.27096E+00_JPRB, 0.26845E+00_JPRB, 0.26618E+00_JPRB, 0.26382E+00_JPRB, 0.26135E+00_JPRB, 0.25948E+00_JPRB, 0.25831E+00_JPRB ,&
&  0.25808E+00_JPRB, 0.25846E+00_JPRB, 0.25911E+00_JPRB, 0.26016E+00_JPRB, 0.26086E+00_JPRB, 0.26129E+00_JPRB, 0.26142E+00_JPRB ,&
&  0.26166E+00_JPRB, 0.26198E+00_JPRB, 0.26251E+00_JPRB, 0.26286E+00_JPRB, 0.26323E+00_JPRB, 0.26347E+00_JPRB, 0.26319E+00_JPRB ,&
&  0.26180E+00_JPRB, 0.25961E+00_JPRB, 0.25686E+00_JPRB, 0.25570E+00_JPRB, 0.25548E+00_JPRB, 0.25351E+00_JPRB, 0.25069E+00_JPRB ,&
&  0.24819E+00_JPRB, 0.24785E+00_JPRB, 0.24935E+00_JPRB, 0.25078E+00_JPRB, 0.25194E+00_JPRB, 0.25240E+00_JPRB, 0.25217E+00_JPRB ,&
&  0.25115E+00_JPRB, 0.24848E+00_JPRB, 0.24538E+00_JPRB, 0.24279E+00_JPRB, 0.24114E+00_JPRB, 0.24096E+00_JPRB, 0.24203E+00_JPRB ,&
&  0.24231E+00_JPRB, 0.24254E+00_JPRB, 0.24374E+00_JPRB, 0.24483E+00_JPRB, 0.24569E+00_JPRB, 0.24665E+00_JPRB, 0.24825E+00_JPRB ,&
&  0.25039E+00_JPRB, 0.25399E+00_JPRB, 0.25852E+00_JPRB, 0.26367E+00_JPRB, 0.26712E+00_JPRB, 0.26638E+00_JPRB, 0.26749E+00_JPRB ,&
&  0.27300E+00_JPRB, 0.27938E+00_JPRB, 0.28453E+00_JPRB, 0.28860E+00_JPRB, 0.29238E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.27375E+00_JPRB, 0.27510E+00_JPRB, 0.27614E+00_JPRB, 0.27699E+00_JPRB, 0.27858E+00_JPRB, 0.27981E+00_JPRB, 0.28129E+00_JPRB ,&
&  0.28236E+00_JPRB, 0.28096E+00_JPRB, 0.27914E+00_JPRB, 0.27758E+00_JPRB, 0.27683E+00_JPRB, 0.27590E+00_JPRB, 0.27510E+00_JPRB ,&
&  0.27558E+00_JPRB, 0.27472E+00_JPRB, 0.27181E+00_JPRB, 0.26949E+00_JPRB, 0.26588E+00_JPRB, 0.26356E+00_JPRB, 0.26322E+00_JPRB ,&
&  0.26485E+00_JPRB, 0.26440E+00_JPRB, 0.26335E+00_JPRB, 0.26193E+00_JPRB, 0.26019E+00_JPRB, 0.25926E+00_JPRB, 0.25814E+00_JPRB ,&
&  0.25750E+00_JPRB, 0.25624E+00_JPRB, 0.25169E+00_JPRB, 0.25219E+00_JPRB, 0.25194E+00_JPRB, 0.25216E+00_JPRB, 0.25177E+00_JPRB ,&
&  0.25123E+00_JPRB, 0.25024E+00_JPRB, 0.24840E+00_JPRB, 0.24634E+00_JPRB, 0.24464E+00_JPRB, 0.24377E+00_JPRB, 0.24382E+00_JPRB ,&
&  0.24443E+00_JPRB, 0.24577E+00_JPRB, 0.24776E+00_JPRB, 0.25002E+00_JPRB, 0.25100E+00_JPRB, 0.25174E+00_JPRB, 0.25295E+00_JPRB ,&
&  0.25353E+00_JPRB, 0.25371E+00_JPRB, 0.25372E+00_JPRB, 0.25352E+00_JPRB, 0.25293E+00_JPRB, 0.25175E+00_JPRB, 0.25074E+00_JPRB ,&
&  0.24856E+00_JPRB, 0.24570E+00_JPRB, 0.24370E+00_JPRB, 0.24128E+00_JPRB, 0.23470E+00_JPRB, 0.22438E+00_JPRB, 0.21864E+00_JPRB ,&
&  0.21918E+00_JPRB, 0.22406E+00_JPRB, 0.22914E+00_JPRB, 0.23285E+00_JPRB, 0.23914E+00_JPRB, 0.23921E+00_JPRB, 0.23481E+00_JPRB ,&
&  0.22491E+00_JPRB, 0.22064E+00_JPRB, 0.21711E+00_JPRB, 0.21221E+00_JPRB, 0.20925E+00_JPRB, 0.20889E+00_JPRB, 0.20922E+00_JPRB ,&
&  0.21143E+00_JPRB, 0.21419E+00_JPRB, 0.21733E+00_JPRB, 0.21918E+00_JPRB, 0.21948E+00_JPRB, 0.21845E+00_JPRB, 0.21698E+00_JPRB ,&
&  0.21714E+00_JPRB, 0.22182E+00_JPRB, 0.23087E+00_JPRB, 0.23945E+00_JPRB, 0.24434E+00_JPRB, 0.24634E+00_JPRB, 0.24840E+00_JPRB ,&
&  0.25280E+00_JPRB, 0.25854E+00_JPRB, 0.26413E+00_JPRB, 0.26804E+00_JPRB, 0.27124E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.25253E+00_JPRB, 0.25365E+00_JPRB, 0.25473E+00_JPRB, 0.25588E+00_JPRB, 0.25716E+00_JPRB, 0.25855E+00_JPRB, 0.25972E+00_JPRB ,&
&  0.26112E+00_JPRB, 0.26202E+00_JPRB, 0.26215E+00_JPRB, 0.26097E+00_JPRB, 0.26005E+00_JPRB, 0.25973E+00_JPRB, 0.26005E+00_JPRB ,&
&  0.25778E+00_JPRB, 0.25465E+00_JPRB, 0.25322E+00_JPRB, 0.25287E+00_JPRB, 0.25213E+00_JPRB, 0.25185E+00_JPRB, 0.25274E+00_JPRB ,&
&  0.25366E+00_JPRB, 0.25304E+00_JPRB, 0.25250E+00_JPRB, 0.25098E+00_JPRB, 0.24979E+00_JPRB, 0.24890E+00_JPRB, 0.24774E+00_JPRB ,&
&  0.24358E+00_JPRB, 0.23336E+00_JPRB, 0.22732E+00_JPRB, 0.23174E+00_JPRB, 0.23807E+00_JPRB, 0.23722E+00_JPRB, 0.23744E+00_JPRB ,&
&  0.23716E+00_JPRB, 0.23589E+00_JPRB, 0.23445E+00_JPRB, 0.23317E+00_JPRB, 0.23282E+00_JPRB, 0.23344E+00_JPRB, 0.23474E+00_JPRB ,&
&  0.23627E+00_JPRB, 0.23917E+00_JPRB, 0.24214E+00_JPRB, 0.24331E+00_JPRB, 0.24453E+00_JPRB, 0.24523E+00_JPRB, 0.24544E+00_JPRB ,&
&  0.24473E+00_JPRB, 0.24332E+00_JPRB, 0.24210E+00_JPRB, 0.24087E+00_JPRB, 0.23960E+00_JPRB, 0.23785E+00_JPRB, 0.23304E+00_JPRB ,&
&  0.22773E+00_JPRB, 0.22207E+00_JPRB, 0.21526E+00_JPRB, 0.20841E+00_JPRB, 0.20253E+00_JPRB, 0.19825E+00_JPRB, 0.19448E+00_JPRB ,&
&  0.19417E+00_JPRB, 0.19881E+00_JPRB, 0.20771E+00_JPRB, 0.21774E+00_JPRB, 0.22092E+00_JPRB, 0.21191E+00_JPRB, 0.20609E+00_JPRB ,&
&  0.20006E+00_JPRB, 0.19496E+00_JPRB, 0.19176E+00_JPRB, 0.19001E+00_JPRB, 0.18868E+00_JPRB, 0.18851E+00_JPRB, 0.18915E+00_JPRB ,&
&  0.19075E+00_JPRB, 0.19247E+00_JPRB, 0.19403E+00_JPRB, 0.19514E+00_JPRB, 0.19559E+00_JPRB, 0.19547E+00_JPRB, 0.19508E+00_JPRB ,&
&  0.19584E+00_JPRB, 0.19869E+00_JPRB, 0.20342E+00_JPRB, 0.20887E+00_JPRB, 0.21350E+00_JPRB, 0.21792E+00_JPRB, 0.22384E+00_JPRB ,&
&  0.23129E+00_JPRB, 0.23665E+00_JPRB, 0.24181E+00_JPRB, 0.24747E+00_JPRB, 0.25064E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.23543E+00_JPRB, 0.23779E+00_JPRB, 0.23847E+00_JPRB, 0.23984E+00_JPRB, 0.24014E+00_JPRB, 0.24125E+00_JPRB, 0.24190E+00_JPRB ,&
&  0.24408E+00_JPRB, 0.24611E+00_JPRB, 0.24656E+00_JPRB, 0.24472E+00_JPRB, 0.24333E+00_JPRB, 0.23978E+00_JPRB, 0.23823E+00_JPRB ,&
&  0.23755E+00_JPRB, 0.23841E+00_JPRB, 0.23852E+00_JPRB, 0.23856E+00_JPRB, 0.23848E+00_JPRB, 0.23845E+00_JPRB, 0.23907E+00_JPRB ,&
&  0.23692E+00_JPRB, 0.23558E+00_JPRB, 0.23765E+00_JPRB, 0.23656E+00_JPRB, 0.23568E+00_JPRB, 0.23410E+00_JPRB, 0.23323E+00_JPRB ,&
&  0.22873E+00_JPRB, 0.22257E+00_JPRB, 0.21640E+00_JPRB, 0.22032E+00_JPRB, 0.22756E+00_JPRB, 0.22455E+00_JPRB, 0.22403E+00_JPRB ,&
&  0.22379E+00_JPRB, 0.22293E+00_JPRB, 0.22290E+00_JPRB, 0.22281E+00_JPRB, 0.22402E+00_JPRB, 0.22518E+00_JPRB, 0.22757E+00_JPRB ,&
&  0.23073E+00_JPRB, 0.23510E+00_JPRB, 0.23509E+00_JPRB, 0.22805E+00_JPRB, 0.22518E+00_JPRB, 0.22438E+00_JPRB, 0.22083E+00_JPRB ,&
&  0.21646E+00_JPRB, 0.21527E+00_JPRB, 0.21404E+00_JPRB, 0.21056E+00_JPRB, 0.20923E+00_JPRB, 0.20861E+00_JPRB, 0.20548E+00_JPRB ,&
&  0.20154E+00_JPRB, 0.19861E+00_JPRB, 0.19474E+00_JPRB, 0.19086E+00_JPRB, 0.18818E+00_JPRB, 0.18696E+00_JPRB, 0.18715E+00_JPRB ,&
&  0.18936E+00_JPRB, 0.19137E+00_JPRB, 0.19767E+00_JPRB, 0.20140E+00_JPRB, 0.19748E+00_JPRB, 0.19479E+00_JPRB, 0.19139E+00_JPRB ,&
&  0.18519E+00_JPRB, 0.18241E+00_JPRB, 0.18009E+00_JPRB, 0.17833E+00_JPRB, 0.17809E+00_JPRB, 0.17883E+00_JPRB, 0.17931E+00_JPRB ,&
&  0.18011E+00_JPRB, 0.18140E+00_JPRB, 0.18204E+00_JPRB, 0.18239E+00_JPRB, 0.18248E+00_JPRB, 0.18283E+00_JPRB, 0.18359E+00_JPRB ,&
&  0.18451E+00_JPRB, 0.18671E+00_JPRB, 0.18970E+00_JPRB, 0.19286E+00_JPRB, 0.19509E+00_JPRB, 0.19686E+00_JPRB, 0.20074E+00_JPRB ,&
&  0.20500E+00_JPRB, 0.21503E+00_JPRB, 0.22890E+00_JPRB, 0.23788E+00_JPRB, 0.23356E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.23199E+00_JPRB, 0.23350E+00_JPRB, 0.23310E+00_JPRB, 0.23321E+00_JPRB, 0.23261E+00_JPRB, 0.23324E+00_JPRB, 0.23193E+00_JPRB ,&
&  0.23140E+00_JPRB, 0.22976E+00_JPRB, 0.22832E+00_JPRB, 0.22498E+00_JPRB, 0.22316E+00_JPRB, 0.21692E+00_JPRB, 0.21501E+00_JPRB ,&
&  0.21320E+00_JPRB, 0.21384E+00_JPRB, 0.21396E+00_JPRB, 0.21442E+00_JPRB, 0.21515E+00_JPRB, 0.21673E+00_JPRB, 0.21646E+00_JPRB ,&
&  0.21517E+00_JPRB, 0.21313E+00_JPRB, 0.21072E+00_JPRB, 0.20854E+00_JPRB, 0.20899E+00_JPRB, 0.20855E+00_JPRB, 0.20745E+00_JPRB ,&
&  0.20651E+00_JPRB, 0.20723E+00_JPRB, 0.20773E+00_JPRB, 0.20920E+00_JPRB, 0.21421E+00_JPRB, 0.21225E+00_JPRB, 0.21134E+00_JPRB ,&
&  0.21167E+00_JPRB, 0.21213E+00_JPRB, 0.21332E+00_JPRB, 0.21436E+00_JPRB, 0.21735E+00_JPRB, 0.21692E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.21857E+00_JPRB, 0.21767E+00_JPRB, 0.21590E+00_JPRB, 0.21282E+00_JPRB, 0.20827E+00_JPRB, 0.20541E+00_JPRB, 0.20274E+00_JPRB ,&
&  0.19969E+00_JPRB, 0.19712E+00_JPRB, 0.19379E+00_JPRB, 0.19263E+00_JPRB, 0.19078E+00_JPRB, 0.18913E+00_JPRB, 0.18763E+00_JPRB ,&
&  0.18657E+00_JPRB, 0.18507E+00_JPRB, 0.18381E+00_JPRB, 0.18297E+00_JPRB, 0.18275E+00_JPRB, 0.18289E+00_JPRB, 0.18411E+00_JPRB ,&
&  0.18635E+00_JPRB, 0.18807E+00_JPRB, 0.18944E+00_JPRB, 0.19091E+00_JPRB, 0.18929E+00_JPRB, 0.18642E+00_JPRB, 0.18362E+00_JPRB ,&
&  0.17947E+00_JPRB, 0.17723E+00_JPRB, 0.17506E+00_JPRB, 0.17375E+00_JPRB, 0.17334E+00_JPRB, 0.17317E+00_JPRB, 0.17338E+00_JPRB ,&
&  0.17361E+00_JPRB, 0.17411E+00_JPRB, 0.17447E+00_JPRB, 0.17498E+00_JPRB, 0.17564E+00_JPRB, 0.17630E+00_JPRB, 0.17731E+00_JPRB ,&
&  0.17892E+00_JPRB, 0.18097E+00_JPRB, 0.18370E+00_JPRB, 0.18683E+00_JPRB, 0.19057E+00_JPRB, 0.19533E+00_JPRB, 0.20216E+00_JPRB ,&
&  0.20876E+00_JPRB, 0.21874E+00_JPRB, 0.21941E+00_JPRB, 0.22699E+00_JPRB, 0.23103E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.22930E+00_JPRB, 0.23142E+00_JPRB, 0.23193E+00_JPRB, 0.23260E+00_JPRB, 0.23170E+00_JPRB, 0.22872E+00_JPRB, 0.22561E+00_JPRB ,&
&  0.21940E+00_JPRB, 0.21453E+00_JPRB, 0.20867E+00_JPRB, 0.20329E+00_JPRB, 0.19814E+00_JPRB, 0.19494E+00_JPRB, 0.19294E+00_JPRB ,&
&  0.19262E+00_JPRB, 0.19389E+00_JPRB, 0.19479E+00_JPRB, 0.19533E+00_JPRB, 0.19555E+00_JPRB, 0.19645E+00_JPRB, 0.19632E+00_JPRB ,&
&  0.19574E+00_JPRB, 0.19470E+00_JPRB, 0.19396E+00_JPRB, 0.19477E+00_JPRB, 0.19691E+00_JPRB, 0.19908E+00_JPRB, 0.20090E+00_JPRB ,&
&  0.20239E+00_JPRB, 0.20147E+00_JPRB, 0.20147E+00_JPRB, 0.20069E+00_JPRB, 0.20049E+00_JPRB, 0.20027E+00_JPRB, 0.20062E+00_JPRB ,&
&  0.20213E+00_JPRB, 0.20333E+00_JPRB, 0.20594E+00_JPRB, 0.20725E+00_JPRB, 0.20630E+00_JPRB, 0.20928E+00_JPRB, 0.21025E+00_JPRB ,&
&  0.20997E+00_JPRB, 0.20671E+00_JPRB, 0.20300E+00_JPRB, 0.19996E+00_JPRB, 0.19668E+00_JPRB, 0.19399E+00_JPRB, 0.19179E+00_JPRB ,&
&  0.18978E+00_JPRB, 0.18878E+00_JPRB, 0.18657E+00_JPRB, 0.18469E+00_JPRB, 0.18347E+00_JPRB, 0.18217E+00_JPRB, 0.18137E+00_JPRB ,&
&  0.18088E+00_JPRB, 0.17998E+00_JPRB, 0.17982E+00_JPRB, 0.17930E+00_JPRB, 0.17937E+00_JPRB, 0.17947E+00_JPRB, 0.17979E+00_JPRB ,&
&  0.18064E+00_JPRB, 0.18178E+00_JPRB, 0.18168E+00_JPRB, 0.18159E+00_JPRB, 0.17986E+00_JPRB, 0.17798E+00_JPRB, 0.17640E+00_JPRB ,&
&  0.17423E+00_JPRB, 0.17266E+00_JPRB, 0.17168E+00_JPRB, 0.17070E+00_JPRB, 0.17051E+00_JPRB, 0.17049E+00_JPRB, 0.17036E+00_JPRB ,&
&  0.17050E+00_JPRB, 0.17081E+00_JPRB, 0.17132E+00_JPRB, 0.17249E+00_JPRB, 0.17348E+00_JPRB, 0.17494E+00_JPRB, 0.17654E+00_JPRB ,&
&  0.17778E+00_JPRB, 0.18012E+00_JPRB, 0.18259E+00_JPRB, 0.18550E+00_JPRB, 0.19130E+00_JPRB, 0.19612E+00_JPRB, 0.20469E+00_JPRB ,&
&  0.21076E+00_JPRB, 0.21944E+00_JPRB, 0.21693E+00_JPRB, 0.22249E+00_JPRB, 0.22696E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.22826E+00_JPRB, 0.22949E+00_JPRB, 0.22892E+00_JPRB, 0.22892E+00_JPRB, 0.22723E+00_JPRB, 0.22491E+00_JPRB, 0.22152E+00_JPRB ,&
&  0.21829E+00_JPRB, 0.21222E+00_JPRB, 0.20780E+00_JPRB, 0.20286E+00_JPRB, 0.19871E+00_JPRB, 0.19501E+00_JPRB, 0.19234E+00_JPRB ,&
&  0.19105E+00_JPRB, 0.19048E+00_JPRB, 0.19014E+00_JPRB, 0.18914E+00_JPRB, 0.18735E+00_JPRB, 0.18618E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18366E+00_JPRB, 0.18368E+00_JPRB, 0.18467E+00_JPRB, 0.18666E+00_JPRB, 0.19019E+00_JPRB, 0.19256E+00_JPRB, 0.19477E+00_JPRB ,&
&  0.19664E+00_JPRB, 0.19255E+00_JPRB, 0.19257E+00_JPRB, 0.19317E+00_JPRB, 0.19338E+00_JPRB, 0.19303E+00_JPRB, 0.19248E+00_JPRB ,&
&  0.19643E+00_JPRB, 0.19757E+00_JPRB, 0.20349E+00_JPRB, 0.20321E+00_JPRB, 0.20233E+00_JPRB, 0.20417E+00_JPRB, 0.20304E+00_JPRB ,&
&  0.20321E+00_JPRB, 0.19993E+00_JPRB, 0.19749E+00_JPRB, 0.19375E+00_JPRB, 0.18991E+00_JPRB, 0.18796E+00_JPRB, 0.18574E+00_JPRB ,&
&  0.18506E+00_JPRB, 0.18495E+00_JPRB, 0.18336E+00_JPRB, 0.18239E+00_JPRB, 0.18048E+00_JPRB, 0.17831E+00_JPRB, 0.17699E+00_JPRB ,&
&  0.17564E+00_JPRB, 0.17457E+00_JPRB, 0.17400E+00_JPRB, 0.17342E+00_JPRB, 0.17316E+00_JPRB, 0.17301E+00_JPRB, 0.17285E+00_JPRB ,&
&  0.17286E+00_JPRB, 0.17279E+00_JPRB, 0.17218E+00_JPRB, 0.17189E+00_JPRB, 0.17075E+00_JPRB, 0.16997E+00_JPRB, 0.16933E+00_JPRB ,&
&  0.16874E+00_JPRB, 0.16852E+00_JPRB, 0.16838E+00_JPRB, 0.16842E+00_JPRB, 0.16853E+00_JPRB, 0.16902E+00_JPRB, 0.16959E+00_JPRB ,&
&  0.17014E+00_JPRB, 0.17121E+00_JPRB, 0.17200E+00_JPRB, 0.17317E+00_JPRB, 0.17455E+00_JPRB, 0.17572E+00_JPRB, 0.17761E+00_JPRB ,&
&  0.17969E+00_JPRB, 0.18214E+00_JPRB, 0.18576E+00_JPRB, 0.18893E+00_JPRB, 0.19271E+00_JPRB, 0.19770E+00_JPRB, 0.20301E+00_JPRB ,&
&  0.20873E+00_JPRB, 0.21480E+00_JPRB, 0.21515E+00_JPRB, 0.22189E+00_JPRB, 0.22608E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.21888E+00_JPRB, 0.21957E+00_JPRB, 0.22023E+00_JPRB, 0.22042E+00_JPRB, 0.22040E+00_JPRB, 0.21937E+00_JPRB, 0.21806E+00_JPRB ,&
&  0.21375E+00_JPRB, 0.21067E+00_JPRB, 0.20718E+00_JPRB, 0.20370E+00_JPRB, 0.20061E+00_JPRB, 0.19729E+00_JPRB, 0.19439E+00_JPRB ,&
&  0.19175E+00_JPRB, 0.18974E+00_JPRB, 0.18799E+00_JPRB, 0.18571E+00_JPRB, 0.18285E+00_JPRB, 0.18024E+00_JPRB, 0.17870E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17877E+00_JPRB, 0.18038E+00_JPRB, 0.18197E+00_JPRB, 0.18366E+00_JPRB, 0.18493E+00_JPRB, 0.18624E+00_JPRB ,&
&  0.18571E+00_JPRB, 0.18542E+00_JPRB, 0.18514E+00_JPRB, 0.18457E+00_JPRB, 0.18543E+00_JPRB, 0.18656E+00_JPRB, 0.18733E+00_JPRB ,&
&  0.18823E+00_JPRB, 0.19029E+00_JPRB, 0.19187E+00_JPRB, 0.19330E+00_JPRB, 0.19346E+00_JPRB, 0.19288E+00_JPRB, 0.19139E+00_JPRB ,&
&  0.18944E+00_JPRB, 0.18738E+00_JPRB, 0.18550E+00_JPRB, 0.18386E+00_JPRB, 0.18259E+00_JPRB, 0.18111E+00_JPRB, 0.17984E+00_JPRB ,&
&  0.17922E+00_JPRB, 0.17865E+00_JPRB, 0.17784E+00_JPRB, 0.17705E+00_JPRB, 0.17582E+00_JPRB, 0.17473E+00_JPRB, 0.17361E+00_JPRB ,&
&  0.17260E+00_JPRB, 0.17165E+00_JPRB, 0.17077E+00_JPRB, 0.16986E+00_JPRB, 0.16924E+00_JPRB, 0.16868E+00_JPRB, 0.16813E+00_JPRB ,&
&  0.16756E+00_JPRB, 0.16738E+00_JPRB, 0.16697E+00_JPRB, 0.16683E+00_JPRB, 0.16657E+00_JPRB, 0.16640E+00_JPRB, 0.16661E+00_JPRB ,&
&  0.16719E+00_JPRB, 0.16778E+00_JPRB, 0.16805E+00_JPRB, 0.16841E+00_JPRB, 0.16878E+00_JPRB, 0.16926E+00_JPRB, 0.17006E+00_JPRB ,&
&  0.17066E+00_JPRB, 0.17139E+00_JPRB, 0.17197E+00_JPRB, 0.17269E+00_JPRB, 0.17412E+00_JPRB, 0.17558E+00_JPRB, 0.17724E+00_JPRB ,&
&  0.17935E+00_JPRB, 0.18155E+00_JPRB, 0.18415E+00_JPRB, 0.18667E+00_JPRB, 0.19000E+00_JPRB, 0.19175E+00_JPRB, 0.19522E+00_JPRB ,&
&  0.19901E+00_JPRB, 0.19977E+00_JPRB, 0.21285E+00_JPRB, 0.21409E+00_JPRB, 0.21588E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.19087E+00_JPRB, 0.19282E+00_JPRB, 0.19402E+00_JPRB, 0.19541E+00_JPRB, 0.19315E+00_JPRB, 0.20526E+00_JPRB, 0.20299E+00_JPRB ,&
&  0.20167E+00_JPRB, 0.19722E+00_JPRB, 0.19705E+00_JPRB, 0.19537E+00_JPRB, 0.19322E+00_JPRB, 0.19082E+00_JPRB, 0.18860E+00_JPRB ,&
&  0.18652E+00_JPRB, 0.18421E+00_JPRB, 0.18260E+00_JPRB, 0.18135E+00_JPRB, 0.18020E+00_JPRB, 0.17915E+00_JPRB, 0.17836E+00_JPRB ,&
&  0.17840E+00_JPRB, 0.17905E+00_JPRB, 0.18026E+00_JPRB, 0.18013E+00_JPRB, 0.17979E+00_JPRB, 0.17934E+00_JPRB, 0.17909E+00_JPRB ,&
&  0.17848E+00_JPRB, 0.17805E+00_JPRB, 0.17766E+00_JPRB, 0.17707E+00_JPRB, 0.17797E+00_JPRB, 0.17891E+00_JPRB, 0.17983E+00_JPRB ,&
&  0.18072E+00_JPRB, 0.18160E+00_JPRB, 0.18176E+00_JPRB, 0.18174E+00_JPRB, 0.18141E+00_JPRB, 0.18089E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.17916E+00_JPRB, 0.17825E+00_JPRB, 0.17722E+00_JPRB, 0.17653E+00_JPRB, 0.17593E+00_JPRB, 0.17536E+00_JPRB, 0.17486E+00_JPRB ,&
&  0.17460E+00_JPRB, 0.17443E+00_JPRB, 0.17421E+00_JPRB, 0.17387E+00_JPRB, 0.17346E+00_JPRB, 0.17304E+00_JPRB, 0.17267E+00_JPRB ,&
&  0.17239E+00_JPRB, 0.17183E+00_JPRB, 0.17134E+00_JPRB, 0.17081E+00_JPRB, 0.17042E+00_JPRB, 0.16989E+00_JPRB, 0.16945E+00_JPRB ,&
&  0.16902E+00_JPRB, 0.16870E+00_JPRB, 0.16859E+00_JPRB, 0.16851E+00_JPRB, 0.16845E+00_JPRB, 0.16849E+00_JPRB, 0.16861E+00_JPRB ,&
&  0.16879E+00_JPRB, 0.16899E+00_JPRB, 0.16929E+00_JPRB, 0.16950E+00_JPRB, 0.16969E+00_JPRB, 0.16985E+00_JPRB, 0.17028E+00_JPRB ,&
&  0.17079E+00_JPRB, 0.17133E+00_JPRB, 0.17182E+00_JPRB, 0.17199E+00_JPRB, 0.17287E+00_JPRB, 0.17380E+00_JPRB, 0.17483E+00_JPRB ,&
&  0.17593E+00_JPRB, 0.17736E+00_JPRB, 0.17891E+00_JPRB, 0.18059E+00_JPRB, 0.18255E+00_JPRB, 0.18375E+00_JPRB, 0.18513E+00_JPRB ,&
&  0.18704E+00_JPRB, 0.19022E+00_JPRB, 0.19117E+00_JPRB, 0.19141E+00_JPRB, 0.19137E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.17938E+00_JPRB, 0.17910E+00_JPRB, 0.17878E+00_JPRB, 0.17841E+00_JPRB, 0.17733E+00_JPRB, 0.17676E+00_JPRB, 0.17630E+00_JPRB ,&
&  0.17600E+00_JPRB, 0.17579E+00_JPRB, 0.17585E+00_JPRB, 0.17599E+00_JPRB, 0.17614E+00_JPRB, 0.17627E+00_JPRB, 0.17640E+00_JPRB ,&
&  0.17636E+00_JPRB, 0.17636E+00_JPRB, 0.17557E+00_JPRB, 0.17494E+00_JPRB, 0.17453E+00_JPRB, 0.17422E+00_JPRB, 0.17383E+00_JPRB ,&
&  0.17351E+00_JPRB, 0.17313E+00_JPRB, 0.17269E+00_JPRB, 0.17240E+00_JPRB, 0.17246E+00_JPRB, 0.17233E+00_JPRB, 0.17238E+00_JPRB ,&
&  0.17240E+00_JPRB, 0.17265E+00_JPRB, 0.17279E+00_JPRB, 0.17294E+00_JPRB, 0.17342E+00_JPRB, 0.17388E+00_JPRB, 0.17404E+00_JPRB ,&
&  0.17415E+00_JPRB, 0.17400E+00_JPRB, 0.17386E+00_JPRB, 0.17375E+00_JPRB, 0.17371E+00_JPRB, 0.17372E+00_JPRB, 0.17369E+00_JPRB ,&
&  0.17358E+00_JPRB, 0.17339E+00_JPRB, 0.17313E+00_JPRB, 0.17283E+00_JPRB, 0.17256E+00_JPRB, 0.17245E+00_JPRB, 0.17245E+00_JPRB ,&
&  0.17246E+00_JPRB, 0.17234E+00_JPRB, 0.17213E+00_JPRB, 0.17178E+00_JPRB, 0.17144E+00_JPRB, 0.17111E+00_JPRB, 0.17080E+00_JPRB ,&
&  0.17050E+00_JPRB, 0.17036E+00_JPRB, 0.17021E+00_JPRB, 0.17010E+00_JPRB, 0.17002E+00_JPRB, 0.16989E+00_JPRB, 0.16977E+00_JPRB ,&
&  0.16967E+00_JPRB, 0.16957E+00_JPRB, 0.16946E+00_JPRB, 0.16937E+00_JPRB, 0.16931E+00_JPRB, 0.16925E+00_JPRB, 0.16923E+00_JPRB ,&
&  0.16924E+00_JPRB, 0.16924E+00_JPRB, 0.16926E+00_JPRB, 0.16940E+00_JPRB, 0.16957E+00_JPRB, 0.16979E+00_JPRB, 0.17004E+00_JPRB ,&
&  0.17032E+00_JPRB, 0.17064E+00_JPRB, 0.17099E+00_JPRB, 0.17124E+00_JPRB, 0.17164E+00_JPRB, 0.17205E+00_JPRB, 0.17248E+00_JPRB ,&
&  0.17290E+00_JPRB, 0.17343E+00_JPRB, 0.17401E+00_JPRB, 0.17467E+00_JPRB, 0.17540E+00_JPRB, 0.17581E+00_JPRB, 0.17632E+00_JPRB ,&
&  0.17691E+00_JPRB, 0.17769E+00_JPRB, 0.17811E+00_JPRB, 0.17841E+00_JPRB, 0.17873E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB ,&
&  0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB, 0.17187E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB, 0.89161E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.90450E+00_JPRB, 0.90360E+00_JPRB, 0.90266E+00_JPRB, 0.90178E+00_JPRB, 0.90089E+00_JPRB, 0.90014E+00_JPRB, 0.89933E+00_JPRB ,&
&  0.89883E+00_JPRB, 0.89827E+00_JPRB, 0.89779E+00_JPRB, 0.89721E+00_JPRB, 0.89692E+00_JPRB, 0.89653E+00_JPRB, 0.89615E+00_JPRB ,&
&  0.89565E+00_JPRB, 0.89528E+00_JPRB, 0.89449E+00_JPRB, 0.89378E+00_JPRB, 0.89285E+00_JPRB, 0.89211E+00_JPRB, 0.89097E+00_JPRB ,&
&  0.89022E+00_JPRB, 0.88903E+00_JPRB, 0.88795E+00_JPRB, 0.88606E+00_JPRB, 0.88462E+00_JPRB, 0.88307E+00_JPRB, 0.88174E+00_JPRB ,&
&  0.88059E+00_JPRB, 0.87983E+00_JPRB, 0.87910E+00_JPRB, 0.87817E+00_JPRB, 0.87760E+00_JPRB, 0.87777E+00_JPRB, 0.87810E+00_JPRB ,&
&  0.87841E+00_JPRB, 0.87889E+00_JPRB, 0.87956E+00_JPRB, 0.87984E+00_JPRB, 0.87962E+00_JPRB, 0.87958E+00_JPRB, 0.87946E+00_JPRB ,&
&  0.87913E+00_JPRB, 0.87857E+00_JPRB, 0.87802E+00_JPRB, 0.87773E+00_JPRB, 0.87737E+00_JPRB, 0.87677E+00_JPRB, 0.87626E+00_JPRB ,&
&  0.87593E+00_JPRB, 0.87558E+00_JPRB, 0.87513E+00_JPRB, 0.87450E+00_JPRB, 0.87412E+00_JPRB, 0.87376E+00_JPRB, 0.87331E+00_JPRB ,&
&  0.87282E+00_JPRB, 0.87231E+00_JPRB, 0.87172E+00_JPRB, 0.87106E+00_JPRB, 0.87041E+00_JPRB, 0.87014E+00_JPRB, 0.87011E+00_JPRB ,&
&  0.87024E+00_JPRB, 0.87070E+00_JPRB, 0.87140E+00_JPRB, 0.87213E+00_JPRB, 0.87302E+00_JPRB, 0.87416E+00_JPRB, 0.87556E+00_JPRB ,&
&  0.87705E+00_JPRB, 0.87866E+00_JPRB, 0.88066E+00_JPRB, 0.88295E+00_JPRB, 0.88512E+00_JPRB, 0.88729E+00_JPRB, 0.88966E+00_JPRB ,&
&  0.89176E+00_JPRB, 0.89347E+00_JPRB, 0.89483E+00_JPRB, 0.89621E+00_JPRB, 0.89797E+00_JPRB, 0.89950E+00_JPRB, 0.90077E+00_JPRB ,&
&  0.90191E+00_JPRB, 0.90305E+00_JPRB, 0.90409E+00_JPRB, 0.90516E+00_JPRB, 0.90555E+00_JPRB, 0.90585E+00_JPRB, 0.90606E+00_JPRB ,&
&  0.90615E+00_JPRB, 0.90607E+00_JPRB, 0.90587E+00_JPRB, 0.90559E+00_JPRB, 0.90513E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.90667E+00_JPRB, 0.90432E+00_JPRB, 0.90233E+00_JPRB, 0.90084E+00_JPRB, 0.89944E+00_JPRB, 0.89854E+00_JPRB, 0.89783E+00_JPRB ,&
&  0.89738E+00_JPRB, 0.89654E+00_JPRB, 0.89547E+00_JPRB, 0.89508E+00_JPRB, 0.89553E+00_JPRB, 0.89623E+00_JPRB, 0.89584E+00_JPRB ,&
&  0.89555E+00_JPRB, 0.89491E+00_JPRB, 0.89455E+00_JPRB, 0.89376E+00_JPRB, 0.89232E+00_JPRB, 0.89037E+00_JPRB, 0.88790E+00_JPRB ,&
&  0.88600E+00_JPRB, 0.88367E+00_JPRB, 0.88119E+00_JPRB, 0.87650E+00_JPRB, 0.87255E+00_JPRB, 0.86871E+00_JPRB, 0.86499E+00_JPRB ,&
&  0.86014E+00_JPRB, 0.85596E+00_JPRB, 0.85265E+00_JPRB, 0.84984E+00_JPRB, 0.85350E+00_JPRB, 0.85587E+00_JPRB, 0.85725E+00_JPRB ,&
&  0.85721E+00_JPRB, 0.85312E+00_JPRB, 0.85194E+00_JPRB, 0.85138E+00_JPRB, 0.85115E+00_JPRB, 0.85493E+00_JPRB, 0.85760E+00_JPRB ,&
&  0.85973E+00_JPRB, 0.86132E+00_JPRB, 0.86490E+00_JPRB, 0.86666E+00_JPRB, 0.86777E+00_JPRB, 0.86800E+00_JPRB, 0.86750E+00_JPRB ,&
&  0.86797E+00_JPRB, 0.86847E+00_JPRB, 0.86875E+00_JPRB, 0.86824E+00_JPRB, 0.86784E+00_JPRB, 0.86750E+00_JPRB, 0.86656E+00_JPRB ,&
&  0.86493E+00_JPRB, 0.86328E+00_JPRB, 0.86164E+00_JPRB, 0.85993E+00_JPRB, 0.85803E+00_JPRB, 0.85688E+00_JPRB, 0.85597E+00_JPRB ,&
&  0.85525E+00_JPRB, 0.85495E+00_JPRB, 0.85514E+00_JPRB, 0.85559E+00_JPRB, 0.85641E+00_JPRB, 0.85818E+00_JPRB, 0.86115E+00_JPRB ,&
&  0.86506E+00_JPRB, 0.86978E+00_JPRB, 0.87497E+00_JPRB, 0.88008E+00_JPRB, 0.88431E+00_JPRB, 0.88801E+00_JPRB, 0.89227E+00_JPRB ,&
&  0.89678E+00_JPRB, 0.90045E+00_JPRB, 0.90372E+00_JPRB, 0.90681E+00_JPRB, 0.91025E+00_JPRB, 0.91247E+00_JPRB, 0.91383E+00_JPRB ,&
&  0.91486E+00_JPRB, 0.91640E+00_JPRB, 0.91794E+00_JPRB, 0.91969E+00_JPRB, 0.92057E+00_JPRB, 0.92042E+00_JPRB, 0.91957E+00_JPRB ,&
&  0.91832E+00_JPRB, 0.91702E+00_JPRB, 0.91509E+00_JPRB, 0.91241E+00_JPRB, 0.90957E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.90127E+00_JPRB, 0.90104E+00_JPRB, 0.90157E+00_JPRB, 0.90284E+00_JPRB, 0.90450E+00_JPRB, 0.90560E+00_JPRB, 0.90696E+00_JPRB ,&
&  0.90782E+00_JPRB, 0.90768E+00_JPRB, 0.90676E+00_JPRB, 0.90717E+00_JPRB, 0.90712E+00_JPRB, 0.90757E+00_JPRB, 0.90628E+00_JPRB ,&
&  0.90576E+00_JPRB, 0.90396E+00_JPRB, 0.90445E+00_JPRB, 0.90444E+00_JPRB, 0.90380E+00_JPRB, 0.90350E+00_JPRB, 0.89511E+00_JPRB ,&
&  0.88740E+00_JPRB, 0.88125E+00_JPRB, 0.87317E+00_JPRB, 0.86546E+00_JPRB, 0.86021E+00_JPRB, 0.82899E+00_JPRB, 0.81952E+00_JPRB ,&
&  0.83323E+00_JPRB, 0.84182E+00_JPRB, 0.84909E+00_JPRB, 0.85418E+00_JPRB, 0.83755E+00_JPRB, 0.81759E+00_JPRB, 0.80539E+00_JPRB ,&
&  0.77562E+00_JPRB, 0.78395E+00_JPRB, 0.79000E+00_JPRB, 0.79497E+00_JPRB, 0.80327E+00_JPRB, 0.80622E+00_JPRB, 0.81430E+00_JPRB ,&
&  0.83393E+00_JPRB, 0.83945E+00_JPRB, 0.84687E+00_JPRB, 0.85316E+00_JPRB, 0.85665E+00_JPRB, 0.86089E+00_JPRB, 0.86981E+00_JPRB ,&
&  0.87381E+00_JPRB, 0.87745E+00_JPRB, 0.87979E+00_JPRB, 0.87960E+00_JPRB, 0.88045E+00_JPRB, 0.88009E+00_JPRB, 0.87931E+00_JPRB ,&
&  0.87609E+00_JPRB, 0.87412E+00_JPRB, 0.87236E+00_JPRB, 0.87025E+00_JPRB, 0.86876E+00_JPRB, 0.86455E+00_JPRB, 0.86178E+00_JPRB ,&
&  0.86023E+00_JPRB, 0.86009E+00_JPRB, 0.86075E+00_JPRB, 0.86270E+00_JPRB, 0.86642E+00_JPRB, 0.87155E+00_JPRB, 0.87705E+00_JPRB ,&
&  0.88370E+00_JPRB, 0.88987E+00_JPRB, 0.89647E+00_JPRB, 0.90282E+00_JPRB, 0.90820E+00_JPRB, 0.91298E+00_JPRB, 0.91720E+00_JPRB ,&
&  0.92232E+00_JPRB, 0.92768E+00_JPRB, 0.93086E+00_JPRB, 0.93158E+00_JPRB, 0.93474E+00_JPRB, 0.93738E+00_JPRB, 0.93849E+00_JPRB ,&
&  0.93910E+00_JPRB, 0.93997E+00_JPRB, 0.93867E+00_JPRB, 0.93591E+00_JPRB, 0.93259E+00_JPRB, 0.92934E+00_JPRB, 0.92558E+00_JPRB ,&
&  0.92154E+00_JPRB, 0.91765E+00_JPRB, 0.91330E+00_JPRB, 0.90821E+00_JPRB, 0.90400E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.90336E+00_JPRB, 0.90653E+00_JPRB, 0.91112E+00_JPRB, 0.91347E+00_JPRB, 0.91700E+00_JPRB, 0.91792E+00_JPRB, 0.91993E+00_JPRB ,&
&  0.92006E+00_JPRB, 0.92095E+00_JPRB, 0.92144E+00_JPRB, 0.92133E+00_JPRB, 0.92082E+00_JPRB, 0.91932E+00_JPRB, 0.91796E+00_JPRB ,&
&  0.91715E+00_JPRB, 0.91586E+00_JPRB, 0.91618E+00_JPRB, 0.91599E+00_JPRB, 0.91477E+00_JPRB, 0.91366E+00_JPRB, 0.90393E+00_JPRB ,&
&  0.89335E+00_JPRB, 0.88290E+00_JPRB, 0.84413E+00_JPRB, 0.84313E+00_JPRB, 0.83837E+00_JPRB, 0.83589E+00_JPRB, 0.83871E+00_JPRB ,&
&  0.83339E+00_JPRB, 0.82724E+00_JPRB, 0.80564E+00_JPRB, 0.78500E+00_JPRB, 0.77038E+00_JPRB, 0.75440E+00_JPRB, 0.75394E+00_JPRB ,&
&  0.75246E+00_JPRB, 0.75812E+00_JPRB, 0.76369E+00_JPRB, 0.76735E+00_JPRB, 0.77839E+00_JPRB, 0.78439E+00_JPRB, 0.79651E+00_JPRB ,&
&  0.81152E+00_JPRB, 0.82185E+00_JPRB, 0.82094E+00_JPRB, 0.81820E+00_JPRB, 0.84091E+00_JPRB, 0.85650E+00_JPRB, 0.87053E+00_JPRB ,&
&  0.87920E+00_JPRB, 0.88086E+00_JPRB, 0.88522E+00_JPRB, 0.88556E+00_JPRB, 0.88791E+00_JPRB, 0.89294E+00_JPRB, 0.89508E+00_JPRB ,&
&  0.89780E+00_JPRB, 0.89682E+00_JPRB, 0.89355E+00_JPRB, 0.88885E+00_JPRB, 0.88111E+00_JPRB, 0.87610E+00_JPRB, 0.87362E+00_JPRB ,&
&  0.87456E+00_JPRB, 0.87929E+00_JPRB, 0.88604E+00_JPRB, 0.89476E+00_JPRB, 0.90376E+00_JPRB, 0.91134E+00_JPRB, 0.91699E+00_JPRB ,&
&  0.92160E+00_JPRB, 0.92488E+00_JPRB, 0.93302E+00_JPRB, 0.93462E+00_JPRB, 0.93941E+00_JPRB, 0.95079E+00_JPRB, 0.95162E+00_JPRB ,&
&  0.95500E+00_JPRB, 0.96190E+00_JPRB, 0.96648E+00_JPRB, 0.97038E+00_JPRB, 0.97315E+00_JPRB, 0.97453E+00_JPRB, 0.97580E+00_JPRB ,&
&  0.97580E+00_JPRB, 0.97328E+00_JPRB, 0.96815E+00_JPRB, 0.96133E+00_JPRB, 0.95441E+00_JPRB, 0.94733E+00_JPRB, 0.93817E+00_JPRB ,&
&  0.92887E+00_JPRB, 0.91924E+00_JPRB, 0.91089E+00_JPRB, 0.90370E+00_JPRB, 0.90243E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.91579E+00_JPRB, 0.91873E+00_JPRB, 0.92500E+00_JPRB, 0.92637E+00_JPRB, 0.92979E+00_JPRB, 0.93039E+00_JPRB, 0.93137E+00_JPRB ,&
&  0.93138E+00_JPRB, 0.93176E+00_JPRB, 0.93109E+00_JPRB, 0.93089E+00_JPRB, 0.92999E+00_JPRB, 0.92942E+00_JPRB, 0.92861E+00_JPRB ,&
&  0.92725E+00_JPRB, 0.92583E+00_JPRB, 0.92432E+00_JPRB, 0.92286E+00_JPRB, 0.92216E+00_JPRB, 0.92157E+00_JPRB, 0.91633E+00_JPRB ,&
&  0.90838E+00_JPRB, 0.89198E+00_JPRB, 0.86295E+00_JPRB, 0.86120E+00_JPRB, 0.85833E+00_JPRB, 0.86071E+00_JPRB, 0.85945E+00_JPRB ,&
&  0.84721E+00_JPRB, 0.79034E+00_JPRB, 0.79649E+00_JPRB, 0.78446E+00_JPRB, 0.77958E+00_JPRB, 0.77417E+00_JPRB, 0.75868E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.74709E+00_JPRB, 0.74588E+00_JPRB, 0.75353E+00_JPRB, 0.76082E+00_JPRB, 0.77228E+00_JPRB, 0.78605E+00_JPRB ,&
&  0.78365E+00_JPRB, 0.80459E+00_JPRB, 0.81761E+00_JPRB, 0.82562E+00_JPRB, 0.84843E+00_JPRB, 0.86732E+00_JPRB, 0.87201E+00_JPRB ,&
&  0.87793E+00_JPRB, 0.87544E+00_JPRB, 0.87954E+00_JPRB, 0.88921E+00_JPRB, 0.89956E+00_JPRB, 0.90019E+00_JPRB, 0.90156E+00_JPRB ,&
&  0.90130E+00_JPRB, 0.90087E+00_JPRB, 0.90022E+00_JPRB, 0.89800E+00_JPRB, 0.89793E+00_JPRB, 0.90225E+00_JPRB, 0.90841E+00_JPRB ,&
&  0.91880E+00_JPRB, 0.93052E+00_JPRB, 0.94400E+00_JPRB, 0.95686E+00_JPRB, 0.96754E+00_JPRB, 0.97772E+00_JPRB, 0.98668E+00_JPRB ,&
&  0.99558E+00_JPRB, 0.10021E+01_JPRB, 0.10050E+01_JPRB, 0.10071E+01_JPRB, 0.10065E+01_JPRB, 0.10023E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.10104E+01_JPRB, 0.10114E+01_JPRB, 0.10125E+01_JPRB, 0.10149E+01_JPRB, 0.10157E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.10105E+01_JPRB, 0.10045E+01_JPRB, 0.99259E+00_JPRB, 0.98018E+00_JPRB, 0.97177E+00_JPRB, 0.95808E+00_JPRB ,&
&  0.94344E+00_JPRB, 0.92672E+00_JPRB, 0.91644E+00_JPRB, 0.90913E+00_JPRB, 0.91134E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.93534E+00_JPRB, 0.94069E+00_JPRB, 0.94800E+00_JPRB, 0.95137E+00_JPRB, 0.95528E+00_JPRB, 0.95569E+00_JPRB, 0.95729E+00_JPRB ,&
&  0.95798E+00_JPRB, 0.95933E+00_JPRB, 0.95981E+00_JPRB, 0.95964E+00_JPRB, 0.95836E+00_JPRB, 0.95668E+00_JPRB, 0.95412E+00_JPRB ,&
&  0.95007E+00_JPRB, 0.94611E+00_JPRB, 0.94001E+00_JPRB, 0.93471E+00_JPRB, 0.93182E+00_JPRB, 0.93223E+00_JPRB, 0.92908E+00_JPRB ,&
&  0.92220E+00_JPRB, 0.90833E+00_JPRB, 0.88701E+00_JPRB, 0.88119E+00_JPRB, 0.88303E+00_JPRB, 0.87129E+00_JPRB, 0.86936E+00_JPRB ,&
&  0.86472E+00_JPRB, 0.86294E+00_JPRB, 0.85130E+00_JPRB, 0.84440E+00_JPRB, 0.82355E+00_JPRB, 0.74999E+00_JPRB, 0.74575E+00_JPRB ,&
&  0.74191E+00_JPRB, 0.73195E+00_JPRB, 0.73106E+00_JPRB, 0.73357E+00_JPRB, 0.74188E+00_JPRB, 0.76933E+00_JPRB, 0.79257E+00_JPRB ,&
&  0.78944E+00_JPRB, 0.81415E+00_JPRB, 0.82480E+00_JPRB, 0.83492E+00_JPRB, 0.85402E+00_JPRB, 0.87085E+00_JPRB, 0.87172E+00_JPRB ,&
&  0.87575E+00_JPRB, 0.87851E+00_JPRB, 0.88675E+00_JPRB, 0.89567E+00_JPRB, 0.90320E+00_JPRB, 0.90845E+00_JPRB, 0.91552E+00_JPRB ,&
&  0.92448E+00_JPRB, 0.93230E+00_JPRB, 0.94206E+00_JPRB, 0.94749E+00_JPRB, 0.95110E+00_JPRB, 0.95597E+00_JPRB, 0.96399E+00_JPRB ,&
&  0.97786E+00_JPRB, 0.99250E+00_JPRB, 0.10129E+01_JPRB, 0.10186E+01_JPRB, 0.10291E+01_JPRB, 0.10388E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.10822E+01_JPRB, 0.10930E+01_JPRB, 0.11008E+01_JPRB, 0.10887E+01_JPRB, 0.10759E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10633E+01_JPRB, 0.10643E+01_JPRB, 0.10538E+01_JPRB, 0.10496E+01_JPRB, 0.10476E+01_JPRB, 0.10473E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10388E+01_JPRB, 0.10293E+01_JPRB, 0.10148E+01_JPRB, 0.10019E+01_JPRB, 0.99009E+00_JPRB, 0.96849E+00_JPRB ,&
&  0.94820E+00_JPRB, 0.92729E+00_JPRB, 0.91960E+00_JPRB, 0.91778E+00_JPRB, 0.92519E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.96059E+00_JPRB, 0.97239E+00_JPRB, 0.98333E+00_JPRB, 0.99207E+00_JPRB, 0.99480E+00_JPRB, 0.99414E+00_JPRB, 0.99052E+00_JPRB ,&
&  0.98665E+00_JPRB, 0.98543E+00_JPRB, 0.98355E+00_JPRB, 0.98205E+00_JPRB, 0.97996E+00_JPRB, 0.97614E+00_JPRB, 0.97166E+00_JPRB ,&
&  0.96790E+00_JPRB, 0.95017E+00_JPRB, 0.94147E+00_JPRB, 0.94081E+00_JPRB, 0.93789E+00_JPRB, 0.93648E+00_JPRB, 0.93511E+00_JPRB ,&
&  0.92679E+00_JPRB, 0.91591E+00_JPRB, 0.90856E+00_JPRB, 0.89836E+00_JPRB, 0.88770E+00_JPRB, 0.89423E+00_JPRB, 0.89307E+00_JPRB ,&
&  0.86941E+00_JPRB, 0.85177E+00_JPRB, 0.85928E+00_JPRB, 0.86046E+00_JPRB, 0.82144E+00_JPRB, 0.78216E+00_JPRB, 0.75883E+00_JPRB ,&
&  0.76192E+00_JPRB, 0.74356E+00_JPRB, 0.74068E+00_JPRB, 0.74263E+00_JPRB, 0.75468E+00_JPRB, 0.76782E+00_JPRB, 0.79279E+00_JPRB ,&
&  0.79961E+00_JPRB, 0.80438E+00_JPRB, 0.82563E+00_JPRB, 0.84635E+00_JPRB, 0.85811E+00_JPRB, 0.86947E+00_JPRB, 0.87281E+00_JPRB ,&
&  0.88228E+00_JPRB, 0.89157E+00_JPRB, 0.90051E+00_JPRB, 0.90996E+00_JPRB, 0.92307E+00_JPRB, 0.93010E+00_JPRB, 0.94088E+00_JPRB ,&
&  0.99441E+00_JPRB, 0.10394E+01_JPRB, 0.10675E+01_JPRB, 0.10886E+01_JPRB, 0.10983E+01_JPRB, 0.10987E+01_JPRB, 0.10828E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10906E+01_JPRB, 0.11020E+01_JPRB, 0.11182E+01_JPRB, 0.11314E+01_JPRB, 0.11388E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.11146E+01_JPRB, 0.11139E+01_JPRB, 0.12093E+01_JPRB, 0.12340E+01_JPRB, 0.12192E+01_JPRB, 0.11852E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.10975E+01_JPRB, 0.10855E+01_JPRB, 0.10757E+01_JPRB, 0.10676E+01_JPRB, 0.10572E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10513E+01_JPRB, 0.10402E+01_JPRB, 0.10281E+01_JPRB, 0.10151E+01_JPRB, 0.10032E+01_JPRB, 0.99278E+00_JPRB, 0.97614E+00_JPRB ,&
&  0.95715E+00_JPRB, 0.93924E+00_JPRB, 0.93521E+00_JPRB, 0.93876E+00_JPRB, 0.94927E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.99444E+00_JPRB, 0.10107E+01_JPRB, 0.10256E+01_JPRB, 0.10356E+01_JPRB, 0.10326E+01_JPRB, 0.10111E+01_JPRB, 0.99233E+00_JPRB ,&
&  0.97981E+00_JPRB, 0.97613E+00_JPRB, 0.97280E+00_JPRB, 0.96769E+00_JPRB, 0.96413E+00_JPRB, 0.96075E+00_JPRB, 0.95847E+00_JPRB ,&
&  0.95754E+00_JPRB, 0.95314E+00_JPRB, 0.94313E+00_JPRB, 0.93212E+00_JPRB, 0.92707E+00_JPRB, 0.92730E+00_JPRB, 0.92901E+00_JPRB ,&
&  0.92571E+00_JPRB, 0.92167E+00_JPRB, 0.92368E+00_JPRB, 0.91068E+00_JPRB, 0.90037E+00_JPRB, 0.88125E+00_JPRB, 0.87071E+00_JPRB ,&
&  0.87308E+00_JPRB, 0.87584E+00_JPRB, 0.85486E+00_JPRB, 0.85358E+00_JPRB, 0.85751E+00_JPRB, 0.81697E+00_JPRB, 0.78736E+00_JPRB ,&
&  0.77017E+00_JPRB, 0.75748E+00_JPRB, 0.75191E+00_JPRB, 0.75448E+00_JPRB, 0.78447E+00_JPRB, 0.82594E+00_JPRB, 0.83654E+00_JPRB ,&
&  0.79368E+00_JPRB, 0.81526E+00_JPRB, 0.84347E+00_JPRB, 0.85762E+00_JPRB, 0.86801E+00_JPRB, 0.88263E+00_JPRB, 0.89689E+00_JPRB ,&
&  0.91006E+00_JPRB, 0.92196E+00_JPRB, 0.93084E+00_JPRB, 0.95319E+00_JPRB, 0.96540E+00_JPRB, 0.96390E+00_JPRB, 0.98978E+00_JPRB ,&
&  0.11268E+01_JPRB, 0.12346E+01_JPRB, 0.12897E+01_JPRB, 0.13096E+01_JPRB, 0.13088E+01_JPRB, 0.12868E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12591E+01_JPRB, 0.12511E+01_JPRB, 0.12478E+01_JPRB, 0.12462E+01_JPRB, 0.12432E+01_JPRB, 0.12291E+01_JPRB, 0.12135E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.11953E+01_JPRB, 0.14620E+01_JPRB, 0.14264E+01_JPRB, 0.13581E+01_JPRB, 0.13083E+01_JPRB, 0.12498E+01_JPRB ,&
&  0.12032E+01_JPRB, 0.11710E+01_JPRB, 0.11519E+01_JPRB, 0.11315E+01_JPRB, 0.10843E+01_JPRB, 0.10494E+01_JPRB, 0.10325E+01_JPRB ,&
&  0.10241E+01_JPRB, 0.10104E+01_JPRB, 0.99988E+00_JPRB, 0.99067E+00_JPRB, 0.98311E+00_JPRB, 0.97949E+00_JPRB, 0.97124E+00_JPRB ,&
&  0.96196E+00_JPRB, 0.96079E+00_JPRB, 0.96467E+00_JPRB, 0.97456E+00_JPRB, 0.98392E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.10181E+01_JPRB, 0.10550E+01_JPRB, 0.10794E+01_JPRB, 0.10849E+01_JPRB, 0.10735E+01_JPRB, 0.10198E+01_JPRB, 0.97236E+00_JPRB ,&
&  0.94451E+00_JPRB, 0.93504E+00_JPRB, 0.93519E+00_JPRB, 0.93511E+00_JPRB, 0.93623E+00_JPRB, 0.93789E+00_JPRB, 0.93693E+00_JPRB ,&
&  0.93352E+00_JPRB, 0.93301E+00_JPRB, 0.93337E+00_JPRB, 0.92913E+00_JPRB, 0.92211E+00_JPRB, 0.91774E+00_JPRB, 0.91820E+00_JPRB ,&
&  0.91844E+00_JPRB, 0.91688E+00_JPRB, 0.91710E+00_JPRB, 0.91249E+00_JPRB, 0.91140E+00_JPRB, 0.90379E+00_JPRB, 0.88997E+00_JPRB ,&
&  0.87032E+00_JPRB, 0.87099E+00_JPRB, 0.88245E+00_JPRB, 0.88005E+00_JPRB, 0.88518E+00_JPRB, 0.85771E+00_JPRB, 0.81945E+00_JPRB ,&
&  0.79457E+00_JPRB, 0.78539E+00_JPRB, 0.80233E+00_JPRB, 0.84890E+00_JPRB, 0.87743E+00_JPRB, 0.83315E+00_JPRB, 0.85110E+00_JPRB ,&
&  0.85221E+00_JPRB, 0.85364E+00_JPRB, 0.85936E+00_JPRB, 0.87799E+00_JPRB, 0.89815E+00_JPRB, 0.91599E+00_JPRB, 0.93005E+00_JPRB ,&
&  0.94556E+00_JPRB, 0.96202E+00_JPRB, 0.97892E+00_JPRB, 0.98846E+00_JPRB, 0.10010E+01_JPRB, 0.10690E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.12513E+01_JPRB, 0.13196E+01_JPRB, 0.13522E+01_JPRB, 0.14084E+01_JPRB, 0.14462E+01_JPRB, 0.14567E+01_JPRB, 0.14520E+01_JPRB ,&
&  0.14456E+01_JPRB, 0.14143E+01_JPRB, 0.13858E+01_JPRB, 0.13749E+01_JPRB, 0.13741E+01_JPRB, 0.13701E+01_JPRB, 0.13450E+01_JPRB ,&
&  0.13145E+01_JPRB, 0.12580E+01_JPRB, 0.12051E+01_JPRB, 0.11341E+01_JPRB, 0.11277E+01_JPRB, 0.11530E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.11595E+01_JPRB, 0.11525E+01_JPRB, 0.11381E+01_JPRB, 0.10672E+01_JPRB, 0.10181E+01_JPRB, 0.98816E+00_JPRB ,&
&  0.97172E+00_JPRB, 0.95136E+00_JPRB, 0.95254E+00_JPRB, 0.95034E+00_JPRB, 0.95747E+00_JPRB, 0.96627E+00_JPRB, 0.96015E+00_JPRB ,&
&  0.95512E+00_JPRB, 0.95390E+00_JPRB, 0.96194E+00_JPRB, 0.97258E+00_JPRB, 0.99277E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.11247E+01_JPRB, 0.11415E+01_JPRB, 0.11367E+01_JPRB, 0.11354E+01_JPRB, 0.11091E+01_JPRB, 0.10749E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10062E+01_JPRB, 0.97184E+00_JPRB, 0.93926E+00_JPRB, 0.91081E+00_JPRB, 0.89663E+00_JPRB, 0.89268E+00_JPRB, 0.89748E+00_JPRB ,&
&  0.90638E+00_JPRB, 0.91532E+00_JPRB, 0.92094E+00_JPRB, 0.91812E+00_JPRB, 0.91233E+00_JPRB, 0.91069E+00_JPRB, 0.91339E+00_JPRB ,&
&  0.91455E+00_JPRB, 0.91601E+00_JPRB, 0.91579E+00_JPRB, 0.91281E+00_JPRB, 0.90862E+00_JPRB, 0.90379E+00_JPRB, 0.90436E+00_JPRB ,&
&  0.90461E+00_JPRB, 0.90358E+00_JPRB, 0.91026E+00_JPRB, 0.91181E+00_JPRB, 0.91887E+00_JPRB, 0.89690E+00_JPRB, 0.86030E+00_JPRB ,&
&  0.83203E+00_JPRB, 0.83270E+00_JPRB, 0.89149E+00_JPRB, 0.86372E+00_JPRB, 0.88904E+00_JPRB, 0.90429E+00_JPRB, 0.90085E+00_JPRB ,&
&  0.89816E+00_JPRB, 0.89932E+00_JPRB, 0.91641E+00_JPRB, 0.93130E+00_JPRB, 0.94585E+00_JPRB, 0.96371E+00_JPRB, 0.98080E+00_JPRB ,&
&  0.10106E+01_JPRB, 0.10484E+01_JPRB, 0.10790E+01_JPRB, 0.10851E+01_JPRB, 0.11884E+01_JPRB, 0.14730E+01_JPRB, 0.15465E+01_JPRB ,&
&  0.15867E+01_JPRB, 0.16192E+01_JPRB, 0.16802E+01_JPRB, 0.17232E+01_JPRB, 0.16969E+01_JPRB, 0.16779E+01_JPRB, 0.16648E+01_JPRB ,&
&  0.16239E+01_JPRB, 0.16035E+01_JPRB, 0.15935E+01_JPRB, 0.16180E+01_JPRB, 0.16024E+01_JPRB, 0.15769E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.15343E+01_JPRB, 0.14806E+01_JPRB, 0.14249E+01_JPRB, 0.13460E+01_JPRB, 0.13013E+01_JPRB, 0.12720E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.12352E+01_JPRB, 0.12257E+01_JPRB, 0.12163E+01_JPRB, 0.11935E+01_JPRB, 0.11187E+01_JPRB, 0.10399E+01_JPRB, 0.98949E+00_JPRB ,&
&  0.95543E+00_JPRB, 0.92974E+00_JPRB, 0.92998E+00_JPRB, 0.94082E+00_JPRB, 0.95881E+00_JPRB, 0.97587E+00_JPRB, 0.96958E+00_JPRB ,&
&  0.97565E+00_JPRB, 0.98697E+00_JPRB, 0.10134E+01_JPRB, 0.10501E+01_JPRB, 0.10897E+01_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.12415E+01_JPRB, 0.12464E+01_JPRB, 0.12254E+01_JPRB, 0.12103E+01_JPRB, 0.11834E+01_JPRB, 0.11317E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.97480E+00_JPRB, 0.94387E+00_JPRB, 0.93089E+00_JPRB, 0.92183E+00_JPRB, 0.90690E+00_JPRB, 0.89055E+00_JPRB ,&
&  0.89044E+00_JPRB, 0.90567E+00_JPRB, 0.92175E+00_JPRB, 0.92864E+00_JPRB, 0.93274E+00_JPRB, 0.93171E+00_JPRB, 0.93340E+00_JPRB ,&
&  0.93469E+00_JPRB, 0.93603E+00_JPRB, 0.93916E+00_JPRB, 0.93158E+00_JPRB, 0.92613E+00_JPRB, 0.92650E+00_JPRB, 0.92738E+00_JPRB ,&
&  0.93287E+00_JPRB, 0.93188E+00_JPRB, 0.93854E+00_JPRB, 0.93657E+00_JPRB, 0.93183E+00_JPRB, 0.93264E+00_JPRB, 0.87988E+00_JPRB ,&
&  0.85879E+00_JPRB, 0.88971E+00_JPRB, 0.88076E+00_JPRB, 0.89108E+00_JPRB, 0.91351E+00_JPRB, 0.92633E+00_JPRB, 0.93550E+00_JPRB ,&
&  0.94089E+00_JPRB, 0.94779E+00_JPRB, 0.95913E+00_JPRB, 0.97653E+00_JPRB, 0.10011E+01_JPRB, 0.10272E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.11175E+01_JPRB, 0.11108E+01_JPRB, 0.11444E+01_JPRB, 0.12631E+01_JPRB, 0.15509E+01_JPRB, 0.17001E+01_JPRB, 0.17900E+01_JPRB ,&
&  0.18818E+01_JPRB, 0.19391E+01_JPRB, 0.19591E+01_JPRB, 0.19889E+01_JPRB, 0.19744E+01_JPRB, 0.19565E+01_JPRB, 0.19744E+01_JPRB ,&
&  0.19603E+01_JPRB, 0.19838E+01_JPRB, 0.19858E+01_JPRB, 0.20023E+01_JPRB, 0.19657E+01_JPRB, 0.19480E+01_JPRB, 0.19327E+01_JPRB ,&
&  0.18987E+01_JPRB, 0.18368E+01_JPRB, 0.17256E+01_JPRB, 0.16140E+01_JPRB, 0.15137E+01_JPRB, 0.14555E+01_JPRB, 0.14085E+01_JPRB ,&
&  0.13591E+01_JPRB, 0.13234E+01_JPRB, 0.12859E+01_JPRB, 0.12380E+01_JPRB, 0.11738E+01_JPRB, 0.11200E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.98115E+00_JPRB, 0.98734E+00_JPRB, 0.99791E+00_JPRB, 0.10161E+01_JPRB, 0.10464E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.10996E+01_JPRB, 0.11101E+01_JPRB, 0.11405E+01_JPRB, 0.11875E+01_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.13528E+01_JPRB, 0.13489E+01_JPRB, 0.13247E+01_JPRB, 0.13011E+01_JPRB, 0.12631E+01_JPRB, 0.11838E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.10806E+01_JPRB, 0.10566E+01_JPRB, 0.99624E+00_JPRB, 0.96476E+00_JPRB, 0.94109E+00_JPRB, 0.91900E+00_JPRB ,&
&  0.91522E+00_JPRB, 0.92785E+00_JPRB, 0.94171E+00_JPRB, 0.95587E+00_JPRB, 0.96347E+00_JPRB, 0.96294E+00_JPRB, 0.96100E+00_JPRB ,&
&  0.95905E+00_JPRB, 0.96344E+00_JPRB, 0.97125E+00_JPRB, 0.97611E+00_JPRB, 0.97905E+00_JPRB, 0.97726E+00_JPRB, 0.97147E+00_JPRB ,&
&  0.96220E+00_JPRB, 0.96080E+00_JPRB, 0.97201E+00_JPRB, 0.96487E+00_JPRB, 0.96133E+00_JPRB, 0.95576E+00_JPRB, 0.94412E+00_JPRB ,&
&  0.91204E+00_JPRB, 0.90740E+00_JPRB, 0.91542E+00_JPRB, 0.92813E+00_JPRB, 0.94228E+00_JPRB, 0.95286E+00_JPRB, 0.96201E+00_JPRB ,&
&  0.97430E+00_JPRB, 0.99029E+00_JPRB, 0.10107E+01_JPRB, 0.10343E+01_JPRB, 0.10645E+01_JPRB, 0.11179E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.12690E+01_JPRB, 0.14842E+01_JPRB, 0.16674E+01_JPRB, 0.19285E+01_JPRB, 0.21378E+01_JPRB, 0.22135E+01_JPRB ,&
&  0.23845E+01_JPRB, 0.24249E+01_JPRB, 0.24552E+01_JPRB, 0.24354E+01_JPRB, 0.24023E+01_JPRB, 0.23919E+01_JPRB, 0.24239E+01_JPRB ,&
&  0.24666E+01_JPRB, 0.24149E+01_JPRB, 0.25490E+01_JPRB, 0.25017E+01_JPRB, 0.24376E+01_JPRB, 0.24098E+01_JPRB, 0.23718E+01_JPRB ,&
&  0.23056E+01_JPRB, 0.22019E+01_JPRB, 0.20636E+01_JPRB, 0.18359E+01_JPRB, 0.17438E+01_JPRB, 0.16612E+01_JPRB, 0.15254E+01_JPRB ,&
&  0.14466E+01_JPRB, 0.13613E+01_JPRB, 0.12821E+01_JPRB, 0.12664E+01_JPRB, 0.11786E+01_JPRB, 0.11160E+01_JPRB, 0.10785E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.10288E+01_JPRB, 0.10754E+01_JPRB, 0.11225E+01_JPRB, 0.11510E+01_JPRB, 0.11550E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.12083E+01_JPRB, 0.12782E+01_JPRB, 0.13079E+01_JPRB, 0.13328E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.15161E+01_JPRB, 0.14982E+01_JPRB, 0.14653E+01_JPRB, 0.14237E+01_JPRB, 0.13439E+01_JPRB, 0.12446E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11648E+01_JPRB, 0.11531E+01_JPRB, 0.11327E+01_JPRB, 0.10997E+01_JPRB, 0.10613E+01_JPRB, 0.10087E+01_JPRB, 0.94437E+00_JPRB ,&
&  0.93377E+00_JPRB, 0.96270E+00_JPRB, 0.99935E+00_JPRB, 0.10199E+01_JPRB, 0.10137E+01_JPRB, 0.10084E+01_JPRB, 0.10151E+01_JPRB ,&
&  0.10226E+01_JPRB, 0.10291E+01_JPRB, 0.10401E+01_JPRB, 0.10479E+01_JPRB, 0.10431E+01_JPRB, 0.10412E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.10081E+01_JPRB, 0.10092E+01_JPRB, 0.99285E+00_JPRB, 0.10310E+01_JPRB, 0.10435E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10144E+01_JPRB, 0.10034E+01_JPRB, 0.10093E+01_JPRB, 0.10163E+01_JPRB, 0.10185E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10688E+01_JPRB, 0.11024E+01_JPRB, 0.11537E+01_JPRB, 0.12081E+01_JPRB, 0.12817E+01_JPRB, 0.13910E+01_JPRB ,&
&  0.15582E+01_JPRB, 0.17924E+01_JPRB, 0.20697E+01_JPRB, 0.21300E+01_JPRB, 0.23578E+01_JPRB, 0.25515E+01_JPRB, 0.26014E+01_JPRB ,&
&  0.27668E+01_JPRB, 0.28107E+01_JPRB, 0.29580E+01_JPRB, 0.30026E+01_JPRB, 0.30428E+01_JPRB, 0.30709E+01_JPRB, 0.30822E+01_JPRB ,&
&  0.30893E+01_JPRB, 0.31127E+01_JPRB, 0.31604E+01_JPRB, 0.31244E+01_JPRB, 0.30079E+01_JPRB, 0.28791E+01_JPRB, 0.27289E+01_JPRB ,&
&  0.25990E+01_JPRB, 0.24573E+01_JPRB, 0.23155E+01_JPRB, 0.21967E+01_JPRB, 0.20519E+01_JPRB, 0.18756E+01_JPRB, 0.16948E+01_JPRB ,&
&  0.15178E+01_JPRB, 0.13148E+01_JPRB, 0.12185E+01_JPRB, 0.12008E+01_JPRB, 0.11746E+01_JPRB, 0.10954E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.11267E+01_JPRB, 0.12188E+01_JPRB, 0.12755E+01_JPRB, 0.13660E+01_JPRB, 0.14185E+01_JPRB, 0.13253E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.13684E+01_JPRB, 0.14342E+01_JPRB, 0.14765E+01_JPRB, 0.15078E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.16618E+01_JPRB, 0.16328E+01_JPRB, 0.16006E+01_JPRB, 0.15580E+01_JPRB, 0.14710E+01_JPRB, 0.13544E+01_JPRB, 0.12713E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.12021E+01_JPRB, 0.11722E+01_JPRB, 0.11449E+01_JPRB, 0.11164E+01_JPRB, 0.10849E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.10398E+01_JPRB, 0.10881E+01_JPRB, 0.10769E+01_JPRB, 0.10935E+01_JPRB, 0.11113E+01_JPRB, 0.10908E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.11381E+01_JPRB, 0.11322E+01_JPRB, 0.11174E+01_JPRB, 0.11097E+01_JPRB, 0.11221E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.11178E+01_JPRB, 0.10908E+01_JPRB, 0.10588E+01_JPRB, 0.10478E+01_JPRB, 0.10841E+01_JPRB, 0.11372E+01_JPRB, 0.11568E+01_JPRB ,&
&  0.11562E+01_JPRB, 0.11522E+01_JPRB, 0.11407E+01_JPRB, 0.11325E+01_JPRB, 0.11214E+01_JPRB, 0.11106E+01_JPRB, 0.11060E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.11310E+01_JPRB, 0.11625E+01_JPRB, 0.12223E+01_JPRB, 0.12906E+01_JPRB, 0.14365E+01_JPRB, 0.16855E+01_JPRB ,&
&  0.19536E+01_JPRB, 0.22251E+01_JPRB, 0.24828E+01_JPRB, 0.27740E+01_JPRB, 0.29915E+01_JPRB, 0.30771E+01_JPRB, 0.31092E+01_JPRB ,&
&  0.32391E+01_JPRB, 0.32207E+01_JPRB, 0.34488E+01_JPRB, 0.34440E+01_JPRB, 0.34746E+01_JPRB, 0.36226E+01_JPRB, 0.36149E+01_JPRB ,&
&  0.35452E+01_JPRB, 0.34164E+01_JPRB, 0.35140E+01_JPRB, 0.34711E+01_JPRB, 0.33654E+01_JPRB, 0.32424E+01_JPRB, 0.30598E+01_JPRB ,&
&  0.28790E+01_JPRB, 0.26956E+01_JPRB, 0.23440E+01_JPRB, 0.20889E+01_JPRB, 0.17989E+01_JPRB, 0.17803E+01_JPRB, 0.16240E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.12463E+01_JPRB, 0.12229E+01_JPRB, 0.12095E+01_JPRB, 0.12260E+01_JPRB, 0.11908E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.11683E+01_JPRB, 0.12745E+01_JPRB, 0.13369E+01_JPRB, 0.14290E+01_JPRB, 0.15655E+01_JPRB, 0.16683E+01_JPRB, 0.14328E+01_JPRB ,&
&  0.14319E+01_JPRB, 0.15772E+01_JPRB, 0.16383E+01_JPRB, 0.16702E+01_JPRB, 0.16774E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.16990E+01_JPRB, 0.16517E+01_JPRB, 0.16059E+01_JPRB, 0.15631E+01_JPRB, 0.15087E+01_JPRB, 0.14398E+01_JPRB, 0.13538E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.12226E+01_JPRB, 0.11935E+01_JPRB, 0.11820E+01_JPRB, 0.11700E+01_JPRB, 0.11431E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.12000E+01_JPRB, 0.11238E+01_JPRB, 0.11659E+01_JPRB, 0.13075E+01_JPRB, 0.12172E+01_JPRB, 0.12110E+01_JPRB ,&
&  0.12161E+01_JPRB, 0.12430E+01_JPRB, 0.12259E+01_JPRB, 0.12017E+01_JPRB, 0.11933E+01_JPRB, 0.12133E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.12768E+01_JPRB, 0.11882E+01_JPRB, 0.11232E+01_JPRB, 0.10942E+01_JPRB, 0.11249E+01_JPRB, 0.11758E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.12222E+01_JPRB, 0.12208E+01_JPRB, 0.12191E+01_JPRB, 0.12141E+01_JPRB, 0.12030E+01_JPRB, 0.11959E+01_JPRB, 0.11950E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.12165E+01_JPRB, 0.12500E+01_JPRB, 0.13140E+01_JPRB, 0.13593E+01_JPRB, 0.15790E+01_JPRB, 0.20712E+01_JPRB ,&
&  0.27129E+01_JPRB, 0.28757E+01_JPRB, 0.31679E+01_JPRB, 0.37509E+01_JPRB, 0.39609E+01_JPRB, 0.40093E+01_JPRB, 0.40119E+01_JPRB ,&
&  0.40368E+01_JPRB, 0.39980E+01_JPRB, 0.39872E+01_JPRB, 0.41391E+01_JPRB, 0.41216E+01_JPRB, 0.41368E+01_JPRB, 0.40081E+01_JPRB ,&
&  0.37705E+01_JPRB, 0.36889E+01_JPRB, 0.36692E+01_JPRB, 0.35306E+01_JPRB, 0.33406E+01_JPRB, 0.32842E+01_JPRB, 0.31900E+01_JPRB ,&
&  0.30054E+01_JPRB, 0.26840E+01_JPRB, 0.20545E+01_JPRB, 0.18554E+01_JPRB, 0.14501E+01_JPRB, 0.14517E+01_JPRB, 0.16799E+01_JPRB ,&
&  0.16168E+01_JPRB, 0.14832E+01_JPRB, 0.14009E+01_JPRB, 0.13720E+01_JPRB, 0.13814E+01_JPRB, 0.14537E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.13674E+01_JPRB, 0.14362E+01_JPRB, 0.15497E+01_JPRB, 0.17076E+01_JPRB, 0.18827E+01_JPRB, 0.19382E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.17995E+01_JPRB, 0.18288E+01_JPRB, 0.18298E+01_JPRB, 0.18158E+01_JPRB, 0.17651E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.16108E+01_JPRB, 0.15663E+01_JPRB, 0.15151E+01_JPRB, 0.14982E+01_JPRB, 0.15033E+01_JPRB, 0.14835E+01_JPRB, 0.14160E+01_JPRB ,&
&  0.13194E+01_JPRB, 0.12480E+01_JPRB, 0.12126E+01_JPRB, 0.12137E+01_JPRB, 0.12242E+01_JPRB, 0.12092E+01_JPRB, 0.12215E+01_JPRB ,&
&  0.13155E+01_JPRB, 0.13920E+01_JPRB, 0.12646E+01_JPRB, 0.11616E+01_JPRB, 0.12645E+01_JPRB, 0.12358E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.12411E+01_JPRB, 0.13316E+01_JPRB, 0.13357E+01_JPRB, 0.13579E+01_JPRB, 0.13918E+01_JPRB, 0.14697E+01_JPRB, 0.16241E+01_JPRB ,&
&  0.16483E+01_JPRB, 0.14823E+01_JPRB, 0.13544E+01_JPRB, 0.13154E+01_JPRB, 0.13496E+01_JPRB, 0.12773E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.12892E+01_JPRB, 0.12921E+01_JPRB, 0.13028E+01_JPRB, 0.13102E+01_JPRB, 0.13153E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.13642E+01_JPRB, 0.13664E+01_JPRB, 0.14041E+01_JPRB, 0.15373E+01_JPRB, 0.18875E+01_JPRB, 0.24332E+01_JPRB ,&
&  0.28588E+01_JPRB, 0.30157E+01_JPRB, 0.32471E+01_JPRB, 0.38030E+01_JPRB, 0.41832E+01_JPRB, 0.43997E+01_JPRB, 0.46575E+01_JPRB ,&
&  0.47294E+01_JPRB, 0.47225E+01_JPRB, 0.46907E+01_JPRB, 0.43952E+01_JPRB, 0.39433E+01_JPRB, 0.36061E+01_JPRB, 0.34375E+01_JPRB ,&
&  0.32598E+01_JPRB, 0.32155E+01_JPRB, 0.33001E+01_JPRB, 0.32143E+01_JPRB, 0.31687E+01_JPRB, 0.29951E+01_JPRB, 0.31072E+01_JPRB ,&
&  0.30300E+01_JPRB, 0.19223E+01_JPRB, 0.15433E+01_JPRB, 0.16689E+01_JPRB, 0.14005E+01_JPRB, 0.14198E+01_JPRB, 0.15908E+01_JPRB ,&
&  0.15535E+01_JPRB, 0.15399E+01_JPRB, 0.16059E+01_JPRB, 0.16715E+01_JPRB, 0.17015E+01_JPRB, 0.18667E+01_JPRB, 0.19353E+01_JPRB ,&
&  0.18872E+01_JPRB, 0.17911E+01_JPRB, 0.19340E+01_JPRB, 0.21243E+01_JPRB, 0.22803E+01_JPRB, 0.22502E+01_JPRB, 0.21330E+01_JPRB ,&
&  0.20612E+01_JPRB, 0.20058E+01_JPRB, 0.19384E+01_JPRB, 0.18206E+01_JPRB, 0.16906E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.15941E+01_JPRB, 0.15654E+01_JPRB, 0.15081E+01_JPRB, 0.15144E+01_JPRB, 0.15489E+01_JPRB, 0.15368E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.12551E+01_JPRB, 0.12221E+01_JPRB, 0.12233E+01_JPRB, 0.12442E+01_JPRB, 0.12553E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.13529E+01_JPRB, 0.12359E+01_JPRB, 0.11830E+01_JPRB, 0.10920E+01_JPRB, 0.11661E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.13497E+01_JPRB, 0.14007E+01_JPRB, 0.15351E+01_JPRB, 0.17171E+01_JPRB, 0.18955E+01_JPRB, 0.20547E+01_JPRB, 0.23293E+01_JPRB ,&
&  0.22794E+01_JPRB, 0.20546E+01_JPRB, 0.17189E+01_JPRB, 0.16473E+01_JPRB, 0.15905E+01_JPRB, 0.14459E+01_JPRB, 0.14806E+01_JPRB ,&
&  0.14568E+01_JPRB, 0.14570E+01_JPRB, 0.14603E+01_JPRB, 0.14623E+01_JPRB, 0.14549E+01_JPRB, 0.14210E+01_JPRB, 0.14079E+01_JPRB ,&
&  0.14127E+01_JPRB, 0.14328E+01_JPRB, 0.14596E+01_JPRB, 0.15186E+01_JPRB, 0.18490E+01_JPRB, 0.22768E+01_JPRB, 0.26146E+01_JPRB ,&
&  0.29843E+01_JPRB, 0.26982E+01_JPRB, 0.25281E+01_JPRB, 0.33797E+01_JPRB, 0.48812E+01_JPRB, 0.53695E+01_JPRB, 0.51421E+01_JPRB ,&
&  0.55660E+01_JPRB, 0.51887E+01_JPRB, 0.45210E+01_JPRB, 0.39876E+01_JPRB, 0.36392E+01_JPRB, 0.33311E+01_JPRB, 0.31827E+01_JPRB ,&
&  0.29550E+01_JPRB, 0.29053E+01_JPRB, 0.30305E+01_JPRB, 0.31141E+01_JPRB, 0.30122E+01_JPRB, 0.28768E+01_JPRB, 0.28970E+01_JPRB ,&
&  0.27646E+01_JPRB, 0.23448E+01_JPRB, 0.17113E+01_JPRB, 0.16477E+01_JPRB, 0.13658E+01_JPRB, 0.13589E+01_JPRB, 0.15679E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.16919E+01_JPRB, 0.17924E+01_JPRB, 0.18231E+01_JPRB, 0.21151E+01_JPRB, 0.23210E+01_JPRB, 0.25833E+01_JPRB ,&
&  0.27088E+01_JPRB, 0.23668E+01_JPRB, 0.25383E+01_JPRB, 0.27060E+01_JPRB, 0.27103E+01_JPRB, 0.25054E+01_JPRB, 0.22923E+01_JPRB ,&
&  0.21613E+01_JPRB, 0.20601E+01_JPRB, 0.19323E+01_JPRB, 0.17638E+01_JPRB, 0.16470E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.16175E+01_JPRB, 0.16257E+01_JPRB, 0.15716E+01_JPRB, 0.15750E+01_JPRB, 0.15760E+01_JPRB, 0.15306E+01_JPRB, 0.14573E+01_JPRB ,&
&  0.13754E+01_JPRB, 0.13276E+01_JPRB, 0.12922E+01_JPRB, 0.12756E+01_JPRB, 0.12797E+01_JPRB, 0.12755E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.12473E+01_JPRB, 0.11334E+01_JPRB, 0.10560E+01_JPRB, 0.10123E+01_JPRB, 0.10684E+01_JPRB, 0.11864E+01_JPRB ,&
&  0.13781E+01_JPRB, 0.16371E+01_JPRB, 0.20353E+01_JPRB, 0.23744E+01_JPRB, 0.28364E+01_JPRB, 0.32096E+01_JPRB, 0.34204E+01_JPRB ,&
&  0.34444E+01_JPRB, 0.26398E+01_JPRB, 0.23674E+01_JPRB, 0.20785E+01_JPRB, 0.19319E+01_JPRB, 0.17817E+01_JPRB, 0.16939E+01_JPRB ,&
&  0.16221E+01_JPRB, 0.15896E+01_JPRB, 0.15619E+01_JPRB, 0.15208E+01_JPRB, 0.14945E+01_JPRB, 0.14815E+01_JPRB, 0.15089E+01_JPRB ,&
&  0.15328E+01_JPRB, 0.15236E+01_JPRB, 0.15770E+01_JPRB, 0.18357E+01_JPRB, 0.23621E+01_JPRB, 0.27996E+01_JPRB, 0.30430E+01_JPRB ,&
&  0.28793E+01_JPRB, 0.28025E+01_JPRB, 0.42783E+01_JPRB, 0.48240E+01_JPRB, 0.47731E+01_JPRB, 0.56212E+01_JPRB, 0.59551E+01_JPRB ,&
&  0.55536E+01_JPRB, 0.44816E+01_JPRB, 0.37884E+01_JPRB, 0.33110E+01_JPRB, 0.36995E+01_JPRB, 0.33903E+01_JPRB, 0.30657E+01_JPRB ,&
&  0.27994E+01_JPRB, 0.29387E+01_JPRB, 0.29235E+01_JPRB, 0.29613E+01_JPRB, 0.30069E+01_JPRB, 0.30287E+01_JPRB, 0.26825E+01_JPRB ,&
&  0.24209E+01_JPRB, 0.28042E+01_JPRB, 0.22941E+01_JPRB, 0.18149E+01_JPRB, 0.15698E+01_JPRB, 0.16038E+01_JPRB, 0.17343E+01_JPRB ,&
&  0.18460E+01_JPRB, 0.19768E+01_JPRB, 0.20451E+01_JPRB, 0.22354E+01_JPRB, 0.27189E+01_JPRB, 0.29960E+01_JPRB, 0.31262E+01_JPRB ,&
&  0.30677E+01_JPRB, 0.33190E+01_JPRB, 0.32911E+01_JPRB, 0.31589E+01_JPRB, 0.28222E+01_JPRB, 0.24333E+01_JPRB, 0.22058E+01_JPRB ,&
&  0.21000E+01_JPRB, 0.20067E+01_JPRB, 0.18416E+01_JPRB, 0.16821E+01_JPRB, 0.15996E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.16277E+01_JPRB, 0.16524E+01_JPRB, 0.16448E+01_JPRB, 0.16272E+01_JPRB, 0.15963E+01_JPRB, 0.15467E+01_JPRB, 0.15015E+01_JPRB ,&
&  0.14400E+01_JPRB, 0.13670E+01_JPRB, 0.13462E+01_JPRB, 0.13418E+01_JPRB, 0.13374E+01_JPRB, 0.13145E+01_JPRB, 0.12739E+01_JPRB ,&
&  0.12465E+01_JPRB, 0.11864E+01_JPRB, 0.10600E+01_JPRB, 0.10687E+01_JPRB, 0.10892E+01_JPRB, 0.10925E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.14648E+01_JPRB, 0.19788E+01_JPRB, 0.25618E+01_JPRB, 0.32448E+01_JPRB, 0.37742E+01_JPRB, 0.39520E+01_JPRB, 0.39215E+01_JPRB ,&
&  0.35129E+01_JPRB, 0.31628E+01_JPRB, 0.27924E+01_JPRB, 0.24889E+01_JPRB, 0.22565E+01_JPRB, 0.20899E+01_JPRB, 0.19225E+01_JPRB ,&
&  0.17807E+01_JPRB, 0.16795E+01_JPRB, 0.16196E+01_JPRB, 0.15493E+01_JPRB, 0.15294E+01_JPRB, 0.15442E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.15578E+01_JPRB, 0.15730E+01_JPRB, 0.17464E+01_JPRB, 0.21460E+01_JPRB, 0.23437E+01_JPRB, 0.26399E+01_JPRB, 0.28914E+01_JPRB ,&
&  0.38326E+01_JPRB, 0.41706E+01_JPRB, 0.41667E+01_JPRB, 0.53633E+01_JPRB, 0.38562E+01_JPRB, 0.46521E+01_JPRB, 0.55512E+01_JPRB ,&
&  0.50858E+01_JPRB, 0.45391E+01_JPRB, 0.44562E+01_JPRB, 0.36834E+01_JPRB, 0.30799E+01_JPRB, 0.39770E+01_JPRB, 0.33866E+01_JPRB ,&
&  0.29964E+01_JPRB, 0.30188E+01_JPRB, 0.30280E+01_JPRB, 0.32981E+01_JPRB, 0.27499E+01_JPRB, 0.24707E+01_JPRB, 0.20502E+01_JPRB ,&
&  0.21420E+01_JPRB, 0.20109E+01_JPRB, 0.22622E+01_JPRB, 0.20395E+01_JPRB, 0.20980E+01_JPRB, 0.20373E+01_JPRB, 0.20304E+01_JPRB ,&
&  0.22147E+01_JPRB, 0.22990E+01_JPRB, 0.23448E+01_JPRB, 0.28237E+01_JPRB, 0.36775E+01_JPRB, 0.33299E+01_JPRB, 0.36323E+01_JPRB ,&
&  0.37599E+01_JPRB, 0.36788E+01_JPRB, 0.33198E+01_JPRB, 0.30193E+01_JPRB, 0.25477E+01_JPRB, 0.22552E+01_JPRB, 0.21273E+01_JPRB ,&
&  0.20322E+01_JPRB, 0.18687E+01_JPRB, 0.16945E+01_JPRB, 0.16243E+01_JPRB, 0.15741E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.16082E+01_JPRB, 0.15994E+01_JPRB, 0.15981E+01_JPRB, 0.15812E+01_JPRB, 0.15819E+01_JPRB, 0.15463E+01_JPRB, 0.15052E+01_JPRB ,&
&  0.14421E+01_JPRB, 0.13849E+01_JPRB, 0.13711E+01_JPRB, 0.13613E+01_JPRB, 0.13525E+01_JPRB, 0.13431E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.12969E+01_JPRB, 0.12583E+01_JPRB, 0.11039E+01_JPRB, 0.10662E+01_JPRB, 0.11282E+01_JPRB, 0.11469E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.25150E+01_JPRB, 0.30861E+01_JPRB, 0.36970E+01_JPRB, 0.40367E+01_JPRB, 0.38901E+01_JPRB, 0.36298E+01_JPRB ,&
&  0.35612E+01_JPRB, 0.29902E+01_JPRB, 0.26575E+01_JPRB, 0.24690E+01_JPRB, 0.22630E+01_JPRB, 0.20834E+01_JPRB, 0.19351E+01_JPRB ,&
&  0.17856E+01_JPRB, 0.16481E+01_JPRB, 0.15598E+01_JPRB, 0.14786E+01_JPRB, 0.14643E+01_JPRB, 0.14893E+01_JPRB, 0.15278E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.16797E+01_JPRB, 0.19841E+01_JPRB, 0.23601E+01_JPRB, 0.26161E+01_JPRB, 0.28662E+01_JPRB, 0.35028E+01_JPRB ,&
&  0.38560E+01_JPRB, 0.38651E+01_JPRB, 0.43812E+01_JPRB, 0.50619E+01_JPRB, 0.52939E+01_JPRB, 0.47706E+01_JPRB, 0.51718E+01_JPRB ,&
&  0.38473E+01_JPRB, 0.32161E+01_JPRB, 0.29167E+01_JPRB, 0.26966E+01_JPRB, 0.31315E+01_JPRB, 0.45839E+01_JPRB, 0.41433E+01_JPRB ,&
&  0.37938E+01_JPRB, 0.34470E+01_JPRB, 0.34624E+01_JPRB, 0.32155E+01_JPRB, 0.29111E+01_JPRB, 0.22188E+01_JPRB, 0.28109E+01_JPRB ,&
&  0.30374E+01_JPRB, 0.31679E+01_JPRB, 0.28850E+01_JPRB, 0.24320E+01_JPRB, 0.21523E+01_JPRB, 0.25241E+01_JPRB, 0.26982E+01_JPRB ,&
&  0.28188E+01_JPRB, 0.28002E+01_JPRB, 0.36828E+01_JPRB, 0.42212E+01_JPRB, 0.43651E+01_JPRB, 0.40386E+01_JPRB, 0.43211E+01_JPRB ,&
&  0.38975E+01_JPRB, 0.37130E+01_JPRB, 0.33124E+01_JPRB, 0.27442E+01_JPRB, 0.23499E+01_JPRB, 0.21643E+01_JPRB, 0.20315E+01_JPRB ,&
&  0.18879E+01_JPRB, 0.17016E+01_JPRB, 0.16001E+01_JPRB, 0.15431E+01_JPRB, 0.15370E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.15559E+01_JPRB, 0.15318E+01_JPRB, 0.14957E+01_JPRB, 0.14908E+01_JPRB, 0.14907E+01_JPRB, 0.14571E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.14288E+01_JPRB, 0.13853E+01_JPRB, 0.13418E+01_JPRB, 0.13147E+01_JPRB, 0.13027E+01_JPRB, 0.13373E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.13751E+01_JPRB, 0.14151E+01_JPRB, 0.12159E+01_JPRB, 0.11692E+01_JPRB, 0.12034E+01_JPRB, 0.12716E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.21235E+01_JPRB, 0.28564E+01_JPRB, 0.31773E+01_JPRB, 0.35568E+01_JPRB, 0.38579E+01_JPRB, 0.36463E+01_JPRB, 0.34896E+01_JPRB ,&
&  0.33548E+01_JPRB, 0.29081E+01_JPRB, 0.25669E+01_JPRB, 0.23249E+01_JPRB, 0.21331E+01_JPRB, 0.19930E+01_JPRB, 0.18552E+01_JPRB ,&
&  0.16976E+01_JPRB, 0.15770E+01_JPRB, 0.14885E+01_JPRB, 0.14292E+01_JPRB, 0.13835E+01_JPRB, 0.14546E+01_JPRB, 0.15602E+01_JPRB ,&
&  0.16849E+01_JPRB, 0.19286E+01_JPRB, 0.22090E+01_JPRB, 0.25870E+01_JPRB, 0.29718E+01_JPRB, 0.31271E+01_JPRB, 0.41172E+01_JPRB ,&
&  0.40345E+01_JPRB, 0.40948E+01_JPRB, 0.44399E+01_JPRB, 0.57518E+01_JPRB, 0.49334E+01_JPRB, 0.44561E+01_JPRB, 0.46145E+01_JPRB ,&
&  0.33498E+01_JPRB, 0.29483E+01_JPRB, 0.31729E+01_JPRB, 0.34959E+01_JPRB, 0.33018E+01_JPRB, 0.36108E+01_JPRB, 0.49250E+01_JPRB ,&
&  0.41315E+01_JPRB, 0.38210E+01_JPRB, 0.37944E+01_JPRB, 0.39935E+01_JPRB, 0.21581E+01_JPRB, 0.17752E+01_JPRB, 0.27507E+01_JPRB ,&
&  0.29632E+01_JPRB, 0.20203E+01_JPRB, 0.15665E+01_JPRB, 0.19508E+01_JPRB, 0.15785E+01_JPRB, 0.20088E+01_JPRB, 0.30667E+01_JPRB ,&
&  0.34067E+01_JPRB, 0.37299E+01_JPRB, 0.58141E+01_JPRB, 0.55040E+01_JPRB, 0.49621E+01_JPRB, 0.54412E+01_JPRB, 0.47316E+01_JPRB ,&
&  0.41620E+01_JPRB, 0.32625E+01_JPRB, 0.29441E+01_JPRB, 0.24649E+01_JPRB, 0.21773E+01_JPRB, 0.19757E+01_JPRB, 0.18289E+01_JPRB ,&
&  0.16738E+01_JPRB, 0.15791E+01_JPRB, 0.15663E+01_JPRB, 0.15573E+01_JPRB, 0.15394E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.15199E+01_JPRB, 0.14599E+01_JPRB, 0.13983E+01_JPRB, 0.14006E+01_JPRB, 0.13855E+01_JPRB, 0.13535E+01_JPRB, 0.14157E+01_JPRB ,&
&  0.14534E+01_JPRB, 0.13599E+01_JPRB, 0.12812E+01_JPRB, 0.12593E+01_JPRB, 0.12744E+01_JPRB, 0.13895E+01_JPRB, 0.14603E+01_JPRB ,&
&  0.15037E+01_JPRB, 0.15073E+01_JPRB, 0.15005E+01_JPRB, 0.14711E+01_JPRB, 0.14936E+01_JPRB, 0.15709E+01_JPRB, 0.18695E+01_JPRB ,&
&  0.26829E+01_JPRB, 0.30462E+01_JPRB, 0.30730E+01_JPRB, 0.31439E+01_JPRB, 0.32821E+01_JPRB, 0.32192E+01_JPRB, 0.32920E+01_JPRB ,&
&  0.30355E+01_JPRB, 0.26571E+01_JPRB, 0.23591E+01_JPRB, 0.21283E+01_JPRB, 0.20147E+01_JPRB, 0.19044E+01_JPRB, 0.17790E+01_JPRB ,&
&  0.16119E+01_JPRB, 0.14917E+01_JPRB, 0.14383E+01_JPRB, 0.13819E+01_JPRB, 0.13931E+01_JPRB, 0.14888E+01_JPRB, 0.16296E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.21625E+01_JPRB, 0.24162E+01_JPRB, 0.28557E+01_JPRB, 0.33376E+01_JPRB, 0.34544E+01_JPRB, 0.32927E+01_JPRB ,&
&  0.31894E+01_JPRB, 0.32179E+01_JPRB, 0.43213E+01_JPRB, 0.49863E+01_JPRB, 0.47991E+01_JPRB, 0.46652E+01_JPRB, 0.44158E+01_JPRB ,&
&  0.38475E+01_JPRB, 0.35892E+01_JPRB, 0.33668E+01_JPRB, 0.38289E+01_JPRB, 0.38523E+01_JPRB, 0.34357E+01_JPRB, 0.41843E+01_JPRB ,&
&  0.43521E+01_JPRB, 0.39471E+01_JPRB, 0.31109E+01_JPRB, 0.25488E+01_JPRB, 0.23712E+01_JPRB, 0.21394E+01_JPRB, 0.15996E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.14547E+01_JPRB, 0.13731E+01_JPRB, 0.14094E+01_JPRB, 0.14234E+01_JPRB, 0.21185E+01_JPRB, 0.31151E+01_JPRB ,&
&  0.46030E+01_JPRB, 0.56034E+01_JPRB, 0.65628E+01_JPRB, 0.62659E+01_JPRB, 0.54809E+01_JPRB, 0.61258E+01_JPRB, 0.49871E+01_JPRB ,&
&  0.34952E+01_JPRB, 0.28808E+01_JPRB, 0.25573E+01_JPRB, 0.22308E+01_JPRB, 0.19927E+01_JPRB, 0.17795E+01_JPRB, 0.16168E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.15415E+01_JPRB, 0.15545E+01_JPRB, 0.15518E+01_JPRB, 0.15016E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.14857E+01_JPRB, 0.14286E+01_JPRB, 0.13700E+01_JPRB, 0.13286E+01_JPRB, 0.12796E+01_JPRB, 0.12970E+01_JPRB, 0.14288E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.13390E+01_JPRB, 0.12435E+01_JPRB, 0.12432E+01_JPRB, 0.13507E+01_JPRB, 0.14819E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.15986E+01_JPRB, 0.16141E+01_JPRB, 0.17758E+01_JPRB, 0.18751E+01_JPRB, 0.20996E+01_JPRB, 0.21777E+01_JPRB, 0.27183E+01_JPRB ,&
&  0.32128E+01_JPRB, 0.30715E+01_JPRB, 0.29651E+01_JPRB, 0.29284E+01_JPRB, 0.30704E+01_JPRB, 0.31021E+01_JPRB, 0.30409E+01_JPRB ,&
&  0.26909E+01_JPRB, 0.23890E+01_JPRB, 0.21443E+01_JPRB, 0.19806E+01_JPRB, 0.18471E+01_JPRB, 0.16798E+01_JPRB, 0.15741E+01_JPRB ,&
&  0.14715E+01_JPRB, 0.14382E+01_JPRB, 0.14083E+01_JPRB, 0.14259E+01_JPRB, 0.15194E+01_JPRB, 0.16419E+01_JPRB, 0.18182E+01_JPRB ,&
&  0.20569E+01_JPRB, 0.23657E+01_JPRB, 0.26538E+01_JPRB, 0.31154E+01_JPRB, 0.29240E+01_JPRB, 0.26128E+01_JPRB, 0.30099E+01_JPRB ,&
&  0.33834E+01_JPRB, 0.37397E+01_JPRB, 0.38174E+01_JPRB, 0.49105E+01_JPRB, 0.48922E+01_JPRB, 0.43813E+01_JPRB, 0.41699E+01_JPRB ,&
&  0.36790E+01_JPRB, 0.37862E+01_JPRB, 0.35320E+01_JPRB, 0.38177E+01_JPRB, 0.45771E+01_JPRB, 0.41125E+01_JPRB, 0.42116E+01_JPRB ,&
&  0.45226E+01_JPRB, 0.40357E+01_JPRB, 0.33501E+01_JPRB, 0.27609E+01_JPRB, 0.34602E+01_JPRB, 0.40504E+01_JPRB, 0.19071E+01_JPRB ,&
&  0.15852E+01_JPRB, 0.14821E+01_JPRB, 0.14575E+01_JPRB, 0.13787E+01_JPRB, 0.14565E+01_JPRB, 0.22130E+01_JPRB, 0.46376E+01_JPRB ,&
&  0.49273E+01_JPRB, 0.74112E+01_JPRB, 0.73934E+01_JPRB, 0.69882E+01_JPRB, 0.58688E+01_JPRB, 0.62602E+01_JPRB, 0.43891E+01_JPRB ,&
&  0.35048E+01_JPRB, 0.29429E+01_JPRB, 0.24792E+01_JPRB, 0.21996E+01_JPRB, 0.19170E+01_JPRB, 0.16673E+01_JPRB, 0.15643E+01_JPRB ,&
&  0.15190E+01_JPRB, 0.14859E+01_JPRB, 0.14743E+01_JPRB, 0.14669E+01_JPRB, 0.14628E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.14751E+01_JPRB, 0.14131E+01_JPRB, 0.13590E+01_JPRB, 0.13206E+01_JPRB, 0.12822E+01_JPRB, 0.13658E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.14768E+01_JPRB, 0.13728E+01_JPRB, 0.13037E+01_JPRB, 0.13406E+01_JPRB, 0.14647E+01_JPRB, 0.16107E+01_JPRB, 0.16958E+01_JPRB ,&
&  0.17900E+01_JPRB, 0.18962E+01_JPRB, 0.21107E+01_JPRB, 0.25762E+01_JPRB, 0.29249E+01_JPRB, 0.31165E+01_JPRB, 0.35987E+01_JPRB ,&
&  0.33655E+01_JPRB, 0.28545E+01_JPRB, 0.28715E+01_JPRB, 0.28999E+01_JPRB, 0.27754E+01_JPRB, 0.27164E+01_JPRB, 0.25240E+01_JPRB ,&
&  0.23398E+01_JPRB, 0.21567E+01_JPRB, 0.19684E+01_JPRB, 0.18232E+01_JPRB, 0.17148E+01_JPRB, 0.15842E+01_JPRB, 0.15280E+01_JPRB ,&
&  0.14747E+01_JPRB, 0.14533E+01_JPRB, 0.14960E+01_JPRB, 0.15437E+01_JPRB, 0.16764E+01_JPRB, 0.18012E+01_JPRB, 0.19973E+01_JPRB ,&
&  0.22590E+01_JPRB, 0.25236E+01_JPRB, 0.27970E+01_JPRB, 0.31529E+01_JPRB, 0.26913E+01_JPRB, 0.31622E+01_JPRB, 0.33371E+01_JPRB ,&
&  0.35471E+01_JPRB, 0.35940E+01_JPRB, 0.34574E+01_JPRB, 0.41821E+01_JPRB, 0.45277E+01_JPRB, 0.41073E+01_JPRB, 0.36964E+01_JPRB ,&
&  0.36119E+01_JPRB, 0.34980E+01_JPRB, 0.37609E+01_JPRB, 0.42159E+01_JPRB, 0.47114E+01_JPRB, 0.58744E+01_JPRB, 0.42820E+01_JPRB ,&
&  0.37868E+01_JPRB, 0.40192E+01_JPRB, 0.37400E+01_JPRB, 0.33806E+01_JPRB, 0.40673E+01_JPRB, 0.63648E+01_JPRB, 0.46933E+01_JPRB ,&
&  0.18356E+01_JPRB, 0.14417E+01_JPRB, 0.16247E+01_JPRB, 0.16598E+01_JPRB, 0.14794E+01_JPRB, 0.22053E+01_JPRB, 0.80850E+01_JPRB ,&
&  0.63900E+01_JPRB, 0.88460E+01_JPRB, 0.76811E+01_JPRB, 0.68116E+01_JPRB, 0.60676E+01_JPRB, 0.59516E+01_JPRB, 0.47083E+01_JPRB ,&
&  0.35366E+01_JPRB, 0.29218E+01_JPRB, 0.24597E+01_JPRB, 0.21144E+01_JPRB, 0.18031E+01_JPRB, 0.16130E+01_JPRB, 0.15274E+01_JPRB ,&
&  0.14761E+01_JPRB, 0.14586E+01_JPRB, 0.14509E+01_JPRB, 0.14403E+01_JPRB, 0.14572E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.14738E+01_JPRB, 0.13706E+01_JPRB, 0.13205E+01_JPRB, 0.12896E+01_JPRB, 0.13550E+01_JPRB, 0.14504E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.15526E+01_JPRB, 0.15636E+01_JPRB, 0.15602E+01_JPRB, 0.15604E+01_JPRB, 0.16663E+01_JPRB, 0.18602E+01_JPRB, 0.20147E+01_JPRB ,&
&  0.22127E+01_JPRB, 0.24393E+01_JPRB, 0.27567E+01_JPRB, 0.32618E+01_JPRB, 0.37801E+01_JPRB, 0.37506E+01_JPRB, 0.36655E+01_JPRB ,&
&  0.32344E+01_JPRB, 0.27636E+01_JPRB, 0.25280E+01_JPRB, 0.26726E+01_JPRB, 0.26547E+01_JPRB, 0.26737E+01_JPRB, 0.24628E+01_JPRB ,&
&  0.23806E+01_JPRB, 0.21765E+01_JPRB, 0.19322E+01_JPRB, 0.17854E+01_JPRB, 0.16085E+01_JPRB, 0.15538E+01_JPRB, 0.16013E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.15886E+01_JPRB, 0.16554E+01_JPRB, 0.17349E+01_JPRB, 0.18812E+01_JPRB, 0.20234E+01_JPRB, 0.22196E+01_JPRB ,&
&  0.24517E+01_JPRB, 0.26615E+01_JPRB, 0.28922E+01_JPRB, 0.29742E+01_JPRB, 0.30212E+01_JPRB, 0.31417E+01_JPRB, 0.34258E+01_JPRB ,&
&  0.35390E+01_JPRB, 0.35700E+01_JPRB, 0.35864E+01_JPRB, 0.37615E+01_JPRB, 0.39004E+01_JPRB, 0.37622E+01_JPRB, 0.34692E+01_JPRB ,&
&  0.36117E+01_JPRB, 0.39354E+01_JPRB, 0.41957E+01_JPRB, 0.46295E+01_JPRB, 0.50136E+01_JPRB, 0.59277E+01_JPRB, 0.55272E+01_JPRB ,&
&  0.45385E+01_JPRB, 0.38067E+01_JPRB, 0.38685E+01_JPRB, 0.46419E+01_JPRB, 0.52530E+01_JPRB, 0.63203E+01_JPRB, 0.75112E+01_JPRB ,&
&  0.74486E+01_JPRB, 0.39561E+01_JPRB, 0.29749E+01_JPRB, 0.44193E+01_JPRB, 0.30072E+01_JPRB, 0.25429E+01_JPRB, 0.44836E+01_JPRB ,&
&  0.67048E+01_JPRB, 0.82463E+01_JPRB, 0.75901E+01_JPRB, 0.63617E+01_JPRB, 0.59936E+01_JPRB, 0.53155E+01_JPRB, 0.41256E+01_JPRB ,&
&  0.31615E+01_JPRB, 0.25299E+01_JPRB, 0.21349E+01_JPRB, 0.18632E+01_JPRB, 0.16225E+01_JPRB, 0.15286E+01_JPRB, 0.15171E+01_JPRB ,&
&  0.14736E+01_JPRB, 0.14939E+01_JPRB, 0.14938E+01_JPRB, 0.14722E+01_JPRB, 0.14861E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.14121E+01_JPRB, 0.13283E+01_JPRB, 0.12843E+01_JPRB, 0.12987E+01_JPRB, 0.14205E+01_JPRB, 0.15535E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.16833E+01_JPRB, 0.17305E+01_JPRB, 0.17959E+01_JPRB, 0.18103E+01_JPRB, 0.19973E+01_JPRB, 0.23327E+01_JPRB, 0.26166E+01_JPRB ,&
&  0.28526E+01_JPRB, 0.31058E+01_JPRB, 0.34216E+01_JPRB, 0.39419E+01_JPRB, 0.46759E+01_JPRB, 0.55958E+01_JPRB, 0.33161E+01_JPRB ,&
&  0.28074E+01_JPRB, 0.27764E+01_JPRB, 0.25136E+01_JPRB, 0.26648E+01_JPRB, 0.25558E+01_JPRB, 0.25089E+01_JPRB, 0.23271E+01_JPRB ,&
&  0.23308E+01_JPRB, 0.21679E+01_JPRB, 0.20123E+01_JPRB, 0.19656E+01_JPRB, 0.18245E+01_JPRB, 0.18697E+01_JPRB, 0.18995E+01_JPRB ,&
&  0.18734E+01_JPRB, 0.19119E+01_JPRB, 0.20314E+01_JPRB, 0.21443E+01_JPRB, 0.22585E+01_JPRB, 0.23656E+01_JPRB, 0.24952E+01_JPRB ,&
&  0.26484E+01_JPRB, 0.28085E+01_JPRB, 0.30193E+01_JPRB, 0.29375E+01_JPRB, 0.29531E+01_JPRB, 0.30346E+01_JPRB, 0.32378E+01_JPRB ,&
&  0.31433E+01_JPRB, 0.29410E+01_JPRB, 0.31887E+01_JPRB, 0.34801E+01_JPRB, 0.35104E+01_JPRB, 0.33324E+01_JPRB, 0.34033E+01_JPRB ,&
&  0.40088E+01_JPRB, 0.46078E+01_JPRB, 0.53709E+01_JPRB, 0.49714E+01_JPRB, 0.51075E+01_JPRB, 0.55039E+01_JPRB, 0.55853E+01_JPRB ,&
&  0.53993E+01_JPRB, 0.49161E+01_JPRB, 0.49261E+01_JPRB, 0.53546E+01_JPRB, 0.54251E+01_JPRB, 0.56202E+01_JPRB, 0.58433E+01_JPRB ,&
&  0.72545E+01_JPRB, 0.91699E+01_JPRB, 0.80264E+01_JPRB, 0.53962E+01_JPRB, 0.43216E+01_JPRB, 0.36233E+01_JPRB, 0.44870E+01_JPRB ,&
&  0.71530E+01_JPRB, 0.67627E+01_JPRB, 0.60207E+01_JPRB, 0.62778E+01_JPRB, 0.49481E+01_JPRB, 0.40497E+01_JPRB, 0.31054E+01_JPRB ,&
&  0.23726E+01_JPRB, 0.18378E+01_JPRB, 0.16441E+01_JPRB, 0.14706E+01_JPRB, 0.14318E+01_JPRB, 0.14496E+01_JPRB, 0.14923E+01_JPRB ,&
&  0.15417E+01_JPRB, 0.15499E+01_JPRB, 0.15273E+01_JPRB, 0.14918E+01_JPRB, 0.14776E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.13993E+01_JPRB, 0.13416E+01_JPRB, 0.13046E+01_JPRB, 0.13549E+01_JPRB, 0.14926E+01_JPRB, 0.16119E+01_JPRB, 0.16631E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.17658E+01_JPRB, 0.19168E+01_JPRB, 0.20717E+01_JPRB, 0.24030E+01_JPRB, 0.27639E+01_JPRB, 0.30954E+01_JPRB ,&
&  0.33565E+01_JPRB, 0.35689E+01_JPRB, 0.38602E+01_JPRB, 0.42933E+01_JPRB, 0.47732E+01_JPRB, 0.51920E+01_JPRB, 0.39637E+01_JPRB ,&
&  0.24682E+01_JPRB, 0.34263E+01_JPRB, 0.26797E+01_JPRB, 0.28368E+01_JPRB, 0.28260E+01_JPRB, 0.29917E+01_JPRB, 0.27029E+01_JPRB ,&
&  0.25972E+01_JPRB, 0.23796E+01_JPRB, 0.23028E+01_JPRB, 0.23146E+01_JPRB, 0.21233E+01_JPRB, 0.21596E+01_JPRB, 0.22502E+01_JPRB ,&
&  0.23328E+01_JPRB, 0.24370E+01_JPRB, 0.25004E+01_JPRB, 0.25328E+01_JPRB, 0.25855E+01_JPRB, 0.26332E+01_JPRB, 0.27142E+01_JPRB ,&
&  0.28491E+01_JPRB, 0.29952E+01_JPRB, 0.30353E+01_JPRB, 0.30354E+01_JPRB, 0.29062E+01_JPRB, 0.29790E+01_JPRB, 0.30927E+01_JPRB ,&
&  0.29344E+01_JPRB, 0.27324E+01_JPRB, 0.29252E+01_JPRB, 0.30089E+01_JPRB, 0.30878E+01_JPRB, 0.35885E+01_JPRB, 0.38501E+01_JPRB ,&
&  0.46211E+01_JPRB, 0.48618E+01_JPRB, 0.59819E+01_JPRB, 0.47066E+01_JPRB, 0.46556E+01_JPRB, 0.49433E+01_JPRB, 0.46921E+01_JPRB ,&
&  0.43942E+01_JPRB, 0.46618E+01_JPRB, 0.48701E+01_JPRB, 0.55013E+01_JPRB, 0.56975E+01_JPRB, 0.54486E+01_JPRB, 0.49322E+01_JPRB ,&
&  0.53443E+01_JPRB, 0.58886E+01_JPRB, 0.54379E+01_JPRB, 0.40142E+01_JPRB, 0.38735E+01_JPRB, 0.41690E+01_JPRB, 0.57155E+01_JPRB ,&
&  0.64835E+01_JPRB, 0.59028E+01_JPRB, 0.53667E+01_JPRB, 0.47322E+01_JPRB, 0.33971E+01_JPRB, 0.26177E+01_JPRB, 0.18885E+01_JPRB ,&
&  0.13968E+01_JPRB, 0.12117E+01_JPRB, 0.13449E+01_JPRB, 0.13043E+01_JPRB, 0.13498E+01_JPRB, 0.14157E+01_JPRB, 0.14381E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.14862E+01_JPRB, 0.14912E+01_JPRB, 0.14651E+01_JPRB, 0.14442E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.12895E+01_JPRB, 0.12655E+01_JPRB, 0.12480E+01_JPRB, 0.13521E+01_JPRB, 0.15353E+01_JPRB, 0.16963E+01_JPRB, 0.17254E+01_JPRB ,&
&  0.17614E+01_JPRB, 0.16802E+01_JPRB, 0.19103E+01_JPRB, 0.21325E+01_JPRB, 0.24559E+01_JPRB, 0.28258E+01_JPRB, 0.32387E+01_JPRB ,&
&  0.36452E+01_JPRB, 0.40246E+01_JPRB, 0.43465E+01_JPRB, 0.47089E+01_JPRB, 0.51816E+01_JPRB, 0.56862E+01_JPRB, 0.50835E+01_JPRB ,&
&  0.36909E+01_JPRB, 0.28077E+01_JPRB, 0.29209E+01_JPRB, 0.28095E+01_JPRB, 0.27606E+01_JPRB, 0.29372E+01_JPRB, 0.29930E+01_JPRB ,&
&  0.29721E+01_JPRB, 0.25723E+01_JPRB, 0.24283E+01_JPRB, 0.24660E+01_JPRB, 0.22199E+01_JPRB, 0.23145E+01_JPRB, 0.24584E+01_JPRB ,&
&  0.25597E+01_JPRB, 0.26630E+01_JPRB, 0.27123E+01_JPRB, 0.27307E+01_JPRB, 0.27677E+01_JPRB, 0.28347E+01_JPRB, 0.29270E+01_JPRB ,&
&  0.30660E+01_JPRB, 0.32165E+01_JPRB, 0.32909E+01_JPRB, 0.31484E+01_JPRB, 0.30982E+01_JPRB, 0.31593E+01_JPRB, 0.31413E+01_JPRB ,&
&  0.30326E+01_JPRB, 0.30709E+01_JPRB, 0.32320E+01_JPRB, 0.34522E+01_JPRB, 0.35092E+01_JPRB, 0.41390E+01_JPRB, 0.43626E+01_JPRB ,&
&  0.47525E+01_JPRB, 0.49494E+01_JPRB, 0.55177E+01_JPRB, 0.42393E+01_JPRB, 0.43449E+01_JPRB, 0.44775E+01_JPRB, 0.44132E+01_JPRB ,&
&  0.44028E+01_JPRB, 0.47452E+01_JPRB, 0.50986E+01_JPRB, 0.53474E+01_JPRB, 0.54752E+01_JPRB, 0.47910E+01_JPRB, 0.46574E+01_JPRB ,&
&  0.45580E+01_JPRB, 0.51594E+01_JPRB, 0.44141E+01_JPRB, 0.41523E+01_JPRB, 0.38238E+01_JPRB, 0.41863E+01_JPRB, 0.46769E+01_JPRB ,&
&  0.51243E+01_JPRB, 0.45009E+01_JPRB, 0.37560E+01_JPRB, 0.28106E+01_JPRB, 0.18428E+01_JPRB, 0.13331E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.11214E+01_JPRB, 0.11901E+01_JPRB, 0.12180E+01_JPRB, 0.12574E+01_JPRB, 0.13026E+01_JPRB, 0.13141E+01_JPRB ,&
&  0.13186E+01_JPRB, 0.12992E+01_JPRB, 0.12712E+01_JPRB, 0.13108E+01_JPRB, 0.13233E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.10943E+01_JPRB, 0.10461E+01_JPRB, 0.10357E+01_JPRB, 0.11487E+01_JPRB, 0.12888E+01_JPRB, 0.13730E+01_JPRB, 0.13751E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.14101E+01_JPRB, 0.15815E+01_JPRB, 0.18029E+01_JPRB, 0.20635E+01_JPRB, 0.24028E+01_JPRB, 0.27667E+01_JPRB ,&
&  0.31521E+01_JPRB, 0.34484E+01_JPRB, 0.36592E+01_JPRB, 0.38914E+01_JPRB, 0.42021E+01_JPRB, 0.45618E+01_JPRB, 0.47219E+01_JPRB ,&
&  0.37832E+01_JPRB, 0.23310E+01_JPRB, 0.25581E+01_JPRB, 0.26504E+01_JPRB, 0.28282E+01_JPRB, 0.29905E+01_JPRB, 0.29546E+01_JPRB ,&
&  0.25230E+01_JPRB, 0.23605E+01_JPRB, 0.24020E+01_JPRB, 0.24424E+01_JPRB, 0.21918E+01_JPRB, 0.22864E+01_JPRB, 0.24206E+01_JPRB ,&
&  0.25817E+01_JPRB, 0.27092E+01_JPRB, 0.27922E+01_JPRB, 0.28682E+01_JPRB, 0.29252E+01_JPRB, 0.30069E+01_JPRB, 0.31529E+01_JPRB ,&
&  0.33263E+01_JPRB, 0.34864E+01_JPRB, 0.35932E+01_JPRB, 0.35734E+01_JPRB, 0.35649E+01_JPRB, 0.36449E+01_JPRB, 0.36095E+01_JPRB ,&
&  0.36088E+01_JPRB, 0.36042E+01_JPRB, 0.39186E+01_JPRB, 0.43307E+01_JPRB, 0.45558E+01_JPRB, 0.41442E+01_JPRB, 0.44472E+01_JPRB ,&
&  0.45648E+01_JPRB, 0.45495E+01_JPRB, 0.44074E+01_JPRB, 0.47266E+01_JPRB, 0.35840E+01_JPRB, 0.39143E+01_JPRB, 0.40944E+01_JPRB ,&
&  0.46633E+01_JPRB, 0.48182E+01_JPRB, 0.49741E+01_JPRB, 0.49016E+01_JPRB, 0.50942E+01_JPRB, 0.43297E+01_JPRB, 0.39845E+01_JPRB ,&
&  0.40214E+01_JPRB, 0.42572E+01_JPRB, 0.40686E+01_JPRB, 0.40824E+01_JPRB, 0.38537E+01_JPRB, 0.40765E+01_JPRB, 0.38632E+01_JPRB ,&
&  0.38798E+01_JPRB, 0.28319E+01_JPRB, 0.22671E+01_JPRB, 0.17094E+01_JPRB, 0.10592E+01_JPRB, 0.90741E+00_JPRB, 0.93861E+00_JPRB ,&
&  0.97900E+00_JPRB, 0.10308E+01_JPRB, 0.10430E+01_JPRB, 0.10911E+01_JPRB, 0.11310E+01_JPRB, 0.11218E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.11333E+01_JPRB, 0.11289E+01_JPRB, 0.11258E+01_JPRB, 0.11150E+01_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.85475E+00_JPRB, 0.85196E+00_JPRB, 0.86716E+00_JPRB, 0.92022E+00_JPRB, 0.10263E+01_JPRB, 0.11019E+01_JPRB, 0.11499E+01_JPRB ,&
&  0.11682E+01_JPRB, 0.12198E+01_JPRB, 0.13237E+01_JPRB, 0.14033E+01_JPRB, 0.15531E+01_JPRB, 0.17663E+01_JPRB, 0.19810E+01_JPRB ,&
&  0.22595E+01_JPRB, 0.25082E+01_JPRB, 0.26938E+01_JPRB, 0.29405E+01_JPRB, 0.31699E+01_JPRB, 0.34979E+01_JPRB, 0.37190E+01_JPRB ,&
&  0.36776E+01_JPRB, 0.30202E+01_JPRB, 0.27167E+01_JPRB, 0.21356E+01_JPRB, 0.19826E+01_JPRB, 0.23406E+01_JPRB, 0.21556E+01_JPRB ,&
&  0.20979E+01_JPRB, 0.20937E+01_JPRB, 0.20270E+01_JPRB, 0.20379E+01_JPRB, 0.20892E+01_JPRB, 0.21716E+01_JPRB, 0.23261E+01_JPRB ,&
&  0.25073E+01_JPRB, 0.26764E+01_JPRB, 0.28273E+01_JPRB, 0.29546E+01_JPRB, 0.30318E+01_JPRB, 0.31378E+01_JPRB, 0.33104E+01_JPRB ,&
&  0.35038E+01_JPRB, 0.36440E+01_JPRB, 0.37891E+01_JPRB, 0.40157E+01_JPRB, 0.40028E+01_JPRB, 0.41200E+01_JPRB, 0.41759E+01_JPRB ,&
&  0.42271E+01_JPRB, 0.41937E+01_JPRB, 0.42217E+01_JPRB, 0.43502E+01_JPRB, 0.43170E+01_JPRB, 0.38709E+01_JPRB, 0.38820E+01_JPRB ,&
&  0.40769E+01_JPRB, 0.39610E+01_JPRB, 0.35753E+01_JPRB, 0.41157E+01_JPRB, 0.30287E+01_JPRB, 0.36281E+01_JPRB, 0.42132E+01_JPRB ,&
&  0.42634E+01_JPRB, 0.42477E+01_JPRB, 0.41055E+01_JPRB, 0.40997E+01_JPRB, 0.41337E+01_JPRB, 0.38433E+01_JPRB, 0.37025E+01_JPRB ,&
&  0.36221E+01_JPRB, 0.31156E+01_JPRB, 0.30272E+01_JPRB, 0.33246E+01_JPRB, 0.36044E+01_JPRB, 0.32998E+01_JPRB, 0.27141E+01_JPRB ,&
&  0.22010E+01_JPRB, 0.20760E+01_JPRB, 0.18912E+01_JPRB, 0.20069E+01_JPRB, 0.13675E+01_JPRB, 0.86026E+00_JPRB, 0.86197E+00_JPRB ,&
&  0.88457E+00_JPRB, 0.91109E+00_JPRB, 0.90668E+00_JPRB, 0.95903E+00_JPRB, 0.97664E+00_JPRB, 0.95777E+00_JPRB, 0.96177E+00_JPRB ,&
&  0.98743E+00_JPRB, 0.98091E+00_JPRB, 0.96841E+00_JPRB, 0.93445E+00_JPRB, 0.90684E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.80480E+00_JPRB, 0.78873E+00_JPRB, 0.79243E+00_JPRB, 0.79249E+00_JPRB, 0.80903E+00_JPRB, 0.81635E+00_JPRB, 0.84241E+00_JPRB ,&
&  0.88434E+00_JPRB, 0.93109E+00_JPRB, 0.93825E+00_JPRB, 0.92999E+00_JPRB, 0.94385E+00_JPRB, 0.97586E+00_JPRB, 0.10645E+01_JPRB ,&
&  0.12487E+01_JPRB, 0.14545E+01_JPRB, 0.15804E+01_JPRB, 0.17102E+01_JPRB, 0.19738E+01_JPRB, 0.24666E+01_JPRB, 0.29380E+01_JPRB ,&
&  0.32870E+01_JPRB, 0.34901E+01_JPRB, 0.33641E+01_JPRB, 0.33889E+01_JPRB, 0.26490E+01_JPRB, 0.21939E+01_JPRB, 0.24319E+01_JPRB ,&
&  0.26388E+01_JPRB, 0.24877E+01_JPRB, 0.19759E+01_JPRB, 0.18821E+01_JPRB, 0.18521E+01_JPRB, 0.19224E+01_JPRB, 0.21233E+01_JPRB ,&
&  0.23263E+01_JPRB, 0.25537E+01_JPRB, 0.27351E+01_JPRB, 0.28791E+01_JPRB, 0.29898E+01_JPRB, 0.31221E+01_JPRB, 0.32504E+01_JPRB ,&
&  0.33914E+01_JPRB, 0.34798E+01_JPRB, 0.35727E+01_JPRB, 0.37743E+01_JPRB, 0.39874E+01_JPRB, 0.41966E+01_JPRB, 0.43224E+01_JPRB ,&
&  0.43680E+01_JPRB, 0.41669E+01_JPRB, 0.41896E+01_JPRB, 0.42203E+01_JPRB, 0.40055E+01_JPRB, 0.37134E+01_JPRB, 0.36607E+01_JPRB ,&
&  0.36280E+01_JPRB, 0.35510E+01_JPRB, 0.28509E+01_JPRB, 0.35240E+01_JPRB, 0.41147E+01_JPRB, 0.38901E+01_JPRB, 0.37960E+01_JPRB ,&
&  0.36068E+01_JPRB, 0.34721E+01_JPRB, 0.32974E+01_JPRB, 0.33839E+01_JPRB, 0.36748E+01_JPRB, 0.47205E+01_JPRB, 0.37554E+01_JPRB ,&
&  0.32188E+01_JPRB, 0.21932E+01_JPRB, 0.20237E+01_JPRB, 0.23157E+01_JPRB, 0.25709E+01_JPRB, 0.23833E+01_JPRB, 0.17761E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.16893E+01_JPRB, 0.18056E+01_JPRB, 0.23265E+01_JPRB, 0.24815E+01_JPRB, 0.89182E+00_JPRB, 0.78563E+00_JPRB ,&
&  0.78736E+00_JPRB, 0.78963E+00_JPRB, 0.78396E+00_JPRB, 0.80812E+00_JPRB, 0.80999E+00_JPRB, 0.81850E+00_JPRB, 0.84559E+00_JPRB ,&
&  0.86229E+00_JPRB, 0.85948E+00_JPRB, 0.84784E+00_JPRB, 0.81424E+00_JPRB, 0.81480E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.86472E+00_JPRB, 0.86263E+00_JPRB, 0.86927E+00_JPRB, 0.86140E+00_JPRB, 0.85313E+00_JPRB, 0.85378E+00_JPRB, 0.86018E+00_JPRB ,&
&  0.86431E+00_JPRB, 0.85348E+00_JPRB, 0.83951E+00_JPRB, 0.83144E+00_JPRB, 0.82509E+00_JPRB, 0.78753E+00_JPRB, 0.73564E+00_JPRB ,&
&  0.75478E+00_JPRB, 0.78301E+00_JPRB, 0.80192E+00_JPRB, 0.84328E+00_JPRB, 0.95460E+00_JPRB, 0.11287E+01_JPRB, 0.13762E+01_JPRB ,&
&  0.18427E+01_JPRB, 0.23515E+01_JPRB, 0.26966E+01_JPRB, 0.27466E+01_JPRB, 0.30398E+01_JPRB, 0.24401E+01_JPRB, 0.25392E+01_JPRB ,&
&  0.29341E+01_JPRB, 0.23293E+01_JPRB, 0.19517E+01_JPRB, 0.17208E+01_JPRB, 0.16705E+01_JPRB, 0.17162E+01_JPRB, 0.18391E+01_JPRB ,&
&  0.20403E+01_JPRB, 0.22552E+01_JPRB, 0.24359E+01_JPRB, 0.26150E+01_JPRB, 0.27826E+01_JPRB, 0.29560E+01_JPRB, 0.30850E+01_JPRB ,&
&  0.31425E+01_JPRB, 0.32217E+01_JPRB, 0.33616E+01_JPRB, 0.34210E+01_JPRB, 0.37384E+01_JPRB, 0.39354E+01_JPRB, 0.40246E+01_JPRB ,&
&  0.39520E+01_JPRB, 0.37983E+01_JPRB, 0.38206E+01_JPRB, 0.36784E+01_JPRB, 0.35293E+01_JPRB, 0.33922E+01_JPRB, 0.32511E+01_JPRB ,&
&  0.31201E+01_JPRB, 0.29123E+01_JPRB, 0.24309E+01_JPRB, 0.28144E+01_JPRB, 0.30933E+01_JPRB, 0.29086E+01_JPRB, 0.34068E+01_JPRB ,&
&  0.31940E+01_JPRB, 0.30777E+01_JPRB, 0.29759E+01_JPRB, 0.28747E+01_JPRB, 0.28008E+01_JPRB, 0.33137E+01_JPRB, 0.33273E+01_JPRB ,&
&  0.22973E+01_JPRB, 0.15653E+01_JPRB, 0.13468E+01_JPRB, 0.14291E+01_JPRB, 0.16322E+01_JPRB, 0.17506E+01_JPRB, 0.16499E+01_JPRB ,&
&  0.15316E+01_JPRB, 0.14388E+01_JPRB, 0.16080E+01_JPRB, 0.20539E+01_JPRB, 0.14071E+01_JPRB, 0.85576E+00_JPRB, 0.71655E+00_JPRB ,&
&  0.68617E+00_JPRB, 0.68304E+00_JPRB, 0.69166E+00_JPRB, 0.72371E+00_JPRB, 0.75222E+00_JPRB, 0.78116E+00_JPRB, 0.80599E+00_JPRB ,&
&  0.81694E+00_JPRB, 0.83433E+00_JPRB, 0.84712E+00_JPRB, 0.86258E+00_JPRB, 0.87064E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.96536E+00_JPRB, 0.98343E+00_JPRB, 0.97872E+00_JPRB, 0.98897E+00_JPRB, 0.10002E+01_JPRB, 0.10220E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10222E+01_JPRB, 0.99972E+00_JPRB, 0.10054E+01_JPRB, 0.10064E+01_JPRB, 0.10083E+01_JPRB, 0.99407E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.10241E+01_JPRB, 0.10297E+01_JPRB, 0.10270E+01_JPRB, 0.10084E+01_JPRB, 0.97582E+00_JPRB, 0.95187E+00_JPRB ,&
&  0.10106E+01_JPRB, 0.11453E+01_JPRB, 0.13125E+01_JPRB, 0.14267E+01_JPRB, 0.17107E+01_JPRB, 0.17802E+01_JPRB, 0.19991E+01_JPRB ,&
&  0.24825E+01_JPRB, 0.20393E+01_JPRB, 0.19224E+01_JPRB, 0.15847E+01_JPRB, 0.15278E+01_JPRB, 0.14687E+01_JPRB, 0.14394E+01_JPRB ,&
&  0.14656E+01_JPRB, 0.15430E+01_JPRB, 0.16399E+01_JPRB, 0.17524E+01_JPRB, 0.18464E+01_JPRB, 0.18773E+01_JPRB, 0.18891E+01_JPRB ,&
&  0.18855E+01_JPRB, 0.20254E+01_JPRB, 0.24364E+01_JPRB, 0.27429E+01_JPRB, 0.31042E+01_JPRB, 0.32824E+01_JPRB, 0.33051E+01_JPRB ,&
&  0.32613E+01_JPRB, 0.32553E+01_JPRB, 0.28667E+01_JPRB, 0.25833E+01_JPRB, 0.25408E+01_JPRB, 0.23463E+01_JPRB, 0.23148E+01_JPRB ,&
&  0.24434E+01_JPRB, 0.24272E+01_JPRB, 0.20210E+01_JPRB, 0.22551E+01_JPRB, 0.25303E+01_JPRB, 0.25118E+01_JPRB, 0.29139E+01_JPRB ,&
&  0.25465E+01_JPRB, 0.22898E+01_JPRB, 0.21794E+01_JPRB, 0.21209E+01_JPRB, 0.20424E+01_JPRB, 0.19609E+01_JPRB, 0.20128E+01_JPRB ,&
&  0.15875E+01_JPRB, 0.13185E+01_JPRB, 0.12267E+01_JPRB, 0.12731E+01_JPRB, 0.15738E+01_JPRB, 0.21122E+01_JPRB, 0.16733E+01_JPRB ,&
&  0.15742E+01_JPRB, 0.15178E+01_JPRB, 0.20382E+01_JPRB, 0.29138E+01_JPRB, 0.13534E+01_JPRB, 0.88679E+00_JPRB, 0.67553E+00_JPRB ,&
&  0.67037E+00_JPRB, 0.70398E+00_JPRB, 0.73559E+00_JPRB, 0.77677E+00_JPRB, 0.80203E+00_JPRB, 0.81439E+00_JPRB, 0.83336E+00_JPRB ,&
&  0.87267E+00_JPRB, 0.90739E+00_JPRB, 0.95569E+00_JPRB, 0.96059E+00_JPRB, 0.94701E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.10263E+01_JPRB, 0.10529E+01_JPRB, 0.10598E+01_JPRB, 0.10839E+01_JPRB, 0.11157E+01_JPRB, 0.11317E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.11340E+01_JPRB, 0.11322E+01_JPRB, 0.11512E+01_JPRB, 0.11564E+01_JPRB, 0.11560E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.11994E+01_JPRB, 0.12406E+01_JPRB, 0.12419E+01_JPRB, 0.12345E+01_JPRB, 0.12389E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.12960E+01_JPRB, 0.13211E+01_JPRB, 0.13609E+01_JPRB, 0.13969E+01_JPRB, 0.15332E+01_JPRB, 0.17128E+01_JPRB, 0.20240E+01_JPRB ,&
&  0.20958E+01_JPRB, 0.20862E+01_JPRB, 0.17622E+01_JPRB, 0.14736E+01_JPRB, 0.14346E+01_JPRB, 0.14006E+01_JPRB, 0.13077E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.11698E+01_JPRB, 0.11350E+01_JPRB, 0.11217E+01_JPRB, 0.11252E+01_JPRB, 0.11034E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10551E+01_JPRB, 0.12901E+01_JPRB, 0.16009E+01_JPRB, 0.18770E+01_JPRB, 0.21153E+01_JPRB, 0.19134E+01_JPRB ,&
&  0.18318E+01_JPRB, 0.19007E+01_JPRB, 0.17524E+01_JPRB, 0.16485E+01_JPRB, 0.15153E+01_JPRB, 0.13976E+01_JPRB, 0.13563E+01_JPRB ,&
&  0.15035E+01_JPRB, 0.17065E+01_JPRB, 0.17065E+01_JPRB, 0.20047E+01_JPRB, 0.21912E+01_JPRB, 0.22327E+01_JPRB, 0.20266E+01_JPRB ,&
&  0.17497E+01_JPRB, 0.16144E+01_JPRB, 0.15834E+01_JPRB, 0.15774E+01_JPRB, 0.15988E+01_JPRB, 0.15189E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.11846E+01_JPRB, 0.11648E+01_JPRB, 0.11363E+01_JPRB, 0.15123E+01_JPRB, 0.18449E+01_JPRB, 0.23712E+01_JPRB ,&
&  0.16743E+01_JPRB, 0.15429E+01_JPRB, 0.13482E+01_JPRB, 0.20871E+01_JPRB, 0.11444E+01_JPRB, 0.70124E+00_JPRB, 0.67541E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.79821E+00_JPRB, 0.82362E+00_JPRB, 0.86002E+00_JPRB, 0.87793E+00_JPRB, 0.88174E+00_JPRB, 0.89876E+00_JPRB ,&
&  0.94429E+00_JPRB, 0.95938E+00_JPRB, 0.97419E+00_JPRB, 0.97784E+00_JPRB, 0.10000E+01_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.10548E+01_JPRB, 0.10928E+01_JPRB, 0.11178E+01_JPRB, 0.11440E+01_JPRB, 0.11772E+01_JPRB, 0.11998E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.12265E+01_JPRB, 0.12265E+01_JPRB, 0.12400E+01_JPRB, 0.12583E+01_JPRB, 0.12661E+01_JPRB, 0.12607E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.13267E+01_JPRB, 0.13685E+01_JPRB, 0.13634E+01_JPRB, 0.13613E+01_JPRB, 0.13877E+01_JPRB, 0.14243E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.14127E+01_JPRB, 0.14228E+01_JPRB, 0.14429E+01_JPRB, 0.14804E+01_JPRB, 0.15008E+01_JPRB, 0.16301E+01_JPRB ,&
&  0.16791E+01_JPRB, 0.22323E+01_JPRB, 0.19440E+01_JPRB, 0.16369E+01_JPRB, 0.15895E+01_JPRB, 0.14366E+01_JPRB, 0.13859E+01_JPRB ,&
&  0.13437E+01_JPRB, 0.12706E+01_JPRB, 0.12183E+01_JPRB, 0.11745E+01_JPRB, 0.11660E+01_JPRB, 0.11732E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11730E+01_JPRB, 0.12462E+01_JPRB, 0.13422E+01_JPRB, 0.14215E+01_JPRB, 0.14268E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.11442E+01_JPRB, 0.94811E+00_JPRB, 0.93960E+00_JPRB, 0.91091E+00_JPRB, 0.77943E+00_JPRB, 0.73024E+00_JPRB ,&
&  0.95114E+00_JPRB, 0.11487E+01_JPRB, 0.15226E+01_JPRB, 0.17327E+01_JPRB, 0.17728E+01_JPRB, 0.18510E+01_JPRB, 0.15847E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.12824E+01_JPRB, 0.12808E+01_JPRB, 0.12439E+01_JPRB, 0.12661E+01_JPRB, 0.12531E+01_JPRB, 0.11615E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10820E+01_JPRB, 0.11131E+01_JPRB, 0.11966E+01_JPRB, 0.13520E+01_JPRB, 0.25052E+01_JPRB, 0.38477E+01_JPRB ,&
&  0.19373E+01_JPRB, 0.13853E+01_JPRB, 0.84146E+00_JPRB, 0.10500E+01_JPRB, 0.82040E+00_JPRB, 0.68883E+00_JPRB, 0.73166E+00_JPRB ,&
&  0.85589E+00_JPRB, 0.91419E+00_JPRB, 0.92839E+00_JPRB, 0.92598E+00_JPRB, 0.90861E+00_JPRB, 0.89443E+00_JPRB, 0.89015E+00_JPRB ,&
&  0.91431E+00_JPRB, 0.93869E+00_JPRB, 0.96714E+00_JPRB, 0.98772E+00_JPRB, 0.10163E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.10616E+01_JPRB, 0.10985E+01_JPRB, 0.11299E+01_JPRB, 0.11666E+01_JPRB, 0.12064E+01_JPRB, 0.12431E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.13296E+01_JPRB, 0.13532E+01_JPRB, 0.13774E+01_JPRB, 0.14036E+01_JPRB, 0.14238E+01_JPRB, 0.14500E+01_JPRB ,&
&  0.14902E+01_JPRB, 0.15259E+01_JPRB, 0.15702E+01_JPRB, 0.15764E+01_JPRB, 0.15828E+01_JPRB, 0.16260E+01_JPRB, 0.16643E+01_JPRB ,&
&  0.16730E+01_JPRB, 0.16928E+01_JPRB, 0.17269E+01_JPRB, 0.17744E+01_JPRB, 0.18404E+01_JPRB, 0.19966E+01_JPRB, 0.19649E+01_JPRB ,&
&  0.21885E+01_JPRB, 0.23393E+01_JPRB, 0.21276E+01_JPRB, 0.18465E+01_JPRB, 0.17228E+01_JPRB, 0.16539E+01_JPRB, 0.16258E+01_JPRB ,&
&  0.16077E+01_JPRB, 0.15341E+01_JPRB, 0.14858E+01_JPRB, 0.14325E+01_JPRB, 0.14049E+01_JPRB, 0.14130E+01_JPRB, 0.14265E+01_JPRB ,&
&  0.14557E+01_JPRB, 0.14909E+01_JPRB, 0.15456E+01_JPRB, 0.16194E+01_JPRB, 0.16935E+01_JPRB, 0.16620E+01_JPRB, 0.15947E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.13741E+01_JPRB, 0.10965E+01_JPRB, 0.10130E+01_JPRB, 0.94771E+00_JPRB, 0.83410E+00_JPRB, 0.73556E+00_JPRB ,&
&  0.88513E+00_JPRB, 0.10001E+01_JPRB, 0.12262E+01_JPRB, 0.15366E+01_JPRB, 0.15236E+01_JPRB, 0.14632E+01_JPRB, 0.13042E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.10798E+01_JPRB, 0.10582E+01_JPRB, 0.10234E+01_JPRB, 0.10289E+01_JPRB, 0.10497E+01_JPRB, 0.98465E+00_JPRB ,&
&  0.97230E+00_JPRB, 0.99948E+00_JPRB, 0.10451E+01_JPRB, 0.11358E+01_JPRB, 0.13561E+01_JPRB, 0.15471E+01_JPRB, 0.23884E+01_JPRB ,&
&  0.17228E+01_JPRB, 0.10282E+01_JPRB, 0.99637E+00_JPRB, 0.89802E+00_JPRB, 0.80570E+00_JPRB, 0.73276E+00_JPRB, 0.80059E+00_JPRB ,&
&  0.95725E+00_JPRB, 0.10853E+01_JPRB, 0.11167E+01_JPRB, 0.10662E+01_JPRB, 0.98263E+00_JPRB, 0.93847E+00_JPRB, 0.91717E+00_JPRB ,&
&  0.93774E+00_JPRB, 0.95702E+00_JPRB, 0.98034E+00_JPRB, 0.99610E+00_JPRB, 0.10222E+01_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.10413E+01_JPRB, 0.10822E+01_JPRB, 0.11187E+01_JPRB, 0.11562E+01_JPRB, 0.11947E+01_JPRB, 0.12337E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.13072E+01_JPRB, 0.13487E+01_JPRB, 0.13958E+01_JPRB, 0.14426E+01_JPRB, 0.14882E+01_JPRB, 0.15366E+01_JPRB, 0.15795E+01_JPRB ,&
&  0.16315E+01_JPRB, 0.16724E+01_JPRB, 0.17228E+01_JPRB, 0.17436E+01_JPRB, 0.17690E+01_JPRB, 0.18120E+01_JPRB, 0.18544E+01_JPRB ,&
&  0.18700E+01_JPRB, 0.18949E+01_JPRB, 0.19269E+01_JPRB, 0.19765E+01_JPRB, 0.20031E+01_JPRB, 0.21490E+01_JPRB, 0.20151E+01_JPRB ,&
&  0.24319E+01_JPRB, 0.26068E+01_JPRB, 0.23931E+01_JPRB, 0.21101E+01_JPRB, 0.19645E+01_JPRB, 0.19335E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.19222E+01_JPRB, 0.19638E+01_JPRB, 0.17994E+01_JPRB, 0.17110E+01_JPRB, 0.16698E+01_JPRB, 0.16760E+01_JPRB, 0.17260E+01_JPRB ,&
&  0.17879E+01_JPRB, 0.18549E+01_JPRB, 0.19246E+01_JPRB, 0.19921E+01_JPRB, 0.20466E+01_JPRB, 0.20676E+01_JPRB, 0.20436E+01_JPRB ,&
&  0.19734E+01_JPRB, 0.18904E+01_JPRB, 0.17038E+01_JPRB, 0.16577E+01_JPRB, 0.15643E+01_JPRB, 0.13127E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10821E+01_JPRB, 0.11516E+01_JPRB, 0.13537E+01_JPRB, 0.13357E+01_JPRB, 0.12518E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.93497E+00_JPRB, 0.93348E+00_JPRB, 0.90087E+00_JPRB, 0.87723E+00_JPRB, 0.86865E+00_JPRB, 0.85839E+00_JPRB ,&
&  0.88356E+00_JPRB, 0.92550E+00_JPRB, 0.98688E+00_JPRB, 0.10650E+01_JPRB, 0.11547E+01_JPRB, 0.11963E+01_JPRB, 0.19089E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.11730E+01_JPRB, 0.13564E+01_JPRB, 0.73107E+00_JPRB, 0.78329E+00_JPRB, 0.84421E+00_JPRB, 0.97742E+00_JPRB ,&
&  0.10640E+01_JPRB, 0.14469E+01_JPRB, 0.19207E+01_JPRB, 0.15457E+01_JPRB, 0.12755E+01_JPRB, 0.10116E+01_JPRB, 0.92892E+00_JPRB ,&
&  0.93609E+00_JPRB, 0.94737E+00_JPRB, 0.96580E+00_JPRB, 0.97939E+00_JPRB, 0.10043E+01_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.10096E+01_JPRB, 0.10435E+01_JPRB, 0.10830E+01_JPRB, 0.11197E+01_JPRB, 0.11494E+01_JPRB, 0.11781E+01_JPRB, 0.12119E+01_JPRB ,&
&  0.12370E+01_JPRB, 0.12566E+01_JPRB, 0.12902E+01_JPRB, 0.13355E+01_JPRB, 0.13867E+01_JPRB, 0.14458E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.16653E+01_JPRB, 0.17647E+01_JPRB, 0.18568E+01_JPRB, 0.19338E+01_JPRB, 0.20161E+01_JPRB, 0.21042E+01_JPRB ,&
&  0.22103E+01_JPRB, 0.22849E+01_JPRB, 0.23434E+01_JPRB, 0.24480E+01_JPRB, 0.25469E+01_JPRB, 0.26986E+01_JPRB, 0.22547E+01_JPRB ,&
&  0.27066E+01_JPRB, 0.30251E+01_JPRB, 0.27305E+01_JPRB, 0.23546E+01_JPRB, 0.21850E+01_JPRB, 0.21543E+01_JPRB, 0.21340E+01_JPRB ,&
&  0.21216E+01_JPRB, 0.20721E+01_JPRB, 0.18823E+01_JPRB, 0.18910E+01_JPRB, 0.19046E+01_JPRB, 0.18843E+01_JPRB, 0.19608E+01_JPRB ,&
&  0.20463E+01_JPRB, 0.21278E+01_JPRB, 0.22063E+01_JPRB, 0.22806E+01_JPRB, 0.23388E+01_JPRB, 0.24056E+01_JPRB, 0.24097E+01_JPRB ,&
&  0.24004E+01_JPRB, 0.23928E+01_JPRB, 0.23672E+01_JPRB, 0.21507E+01_JPRB, 0.20488E+01_JPRB, 0.19223E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.13225E+01_JPRB, 0.11523E+01_JPRB, 0.11025E+01_JPRB, 0.11674E+01_JPRB, 0.11197E+01_JPRB, 0.10216E+01_JPRB, 0.94071E+00_JPRB ,&
&  0.86799E+00_JPRB, 0.81842E+00_JPRB, 0.83611E+00_JPRB, 0.81998E+00_JPRB, 0.74809E+00_JPRB, 0.77289E+00_JPRB, 0.80198E+00_JPRB ,&
&  0.80795E+00_JPRB, 0.84486E+00_JPRB, 0.90254E+00_JPRB, 0.95099E+00_JPRB, 0.10691E+01_JPRB, 0.13165E+01_JPRB, 0.14806E+01_JPRB ,&
&  0.15310E+01_JPRB, 0.11083E+01_JPRB, 0.10309E+01_JPRB, 0.10147E+01_JPRB, 0.88148E+00_JPRB, 0.11057E+01_JPRB, 0.14686E+01_JPRB ,&
&  0.20907E+01_JPRB, 0.21989E+01_JPRB, 0.27832E+01_JPRB, 0.42749E+01_JPRB, 0.32629E+01_JPRB, 0.22164E+01_JPRB, 0.93512E+00_JPRB ,&
&  0.94603E+00_JPRB, 0.94797E+00_JPRB, 0.97392E+00_JPRB, 0.98200E+00_JPRB, 0.99880E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.85459E+00_JPRB, 0.86156E+00_JPRB, 0.89912E+00_JPRB, 0.94142E+00_JPRB, 0.98196E+00_JPRB, 0.10218E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.11467E+01_JPRB, 0.11710E+01_JPRB, 0.11858E+01_JPRB, 0.12135E+01_JPRB, 0.12446E+01_JPRB, 0.12954E+01_JPRB ,&
&  0.13671E+01_JPRB, 0.14472E+01_JPRB, 0.15581E+01_JPRB, 0.16495E+01_JPRB, 0.17517E+01_JPRB, 0.18872E+01_JPRB, 0.20227E+01_JPRB ,&
&  0.21704E+01_JPRB, 0.23529E+01_JPRB, 0.25000E+01_JPRB, 0.26402E+01_JPRB, 0.27957E+01_JPRB, 0.27985E+01_JPRB, 0.20684E+01_JPRB ,&
&  0.29160E+01_JPRB, 0.31814E+01_JPRB, 0.29840E+01_JPRB, 0.25825E+01_JPRB, 0.23674E+01_JPRB, 0.22933E+01_JPRB, 0.22903E+01_JPRB ,&
&  0.22510E+01_JPRB, 0.21101E+01_JPRB, 0.19330E+01_JPRB, 0.18295E+01_JPRB, 0.19997E+01_JPRB, 0.19413E+01_JPRB, 0.20216E+01_JPRB ,&
&  0.21064E+01_JPRB, 0.21865E+01_JPRB, 0.22496E+01_JPRB, 0.23019E+01_JPRB, 0.23454E+01_JPRB, 0.24028E+01_JPRB, 0.24645E+01_JPRB ,&
&  0.25215E+01_JPRB, 0.25540E+01_JPRB, 0.26245E+01_JPRB, 0.23832E+01_JPRB, 0.22736E+01_JPRB, 0.24068E+01_JPRB, 0.21761E+01_JPRB ,&
&  0.15366E+01_JPRB, 0.11751E+01_JPRB, 0.11804E+01_JPRB, 0.11406E+01_JPRB, 0.10426E+01_JPRB, 0.93469E+00_JPRB, 0.86569E+00_JPRB ,&
&  0.81567E+00_JPRB, 0.77657E+00_JPRB, 0.76447E+00_JPRB, 0.74623E+00_JPRB, 0.74531E+00_JPRB, 0.76299E+00_JPRB, 0.76688E+00_JPRB ,&
&  0.80775E+00_JPRB, 0.79920E+00_JPRB, 0.81058E+00_JPRB, 0.84705E+00_JPRB, 0.96770E+00_JPRB, 0.12318E+01_JPRB, 0.17013E+01_JPRB ,&
&  0.18313E+01_JPRB, 0.10771E+01_JPRB, 0.10024E+01_JPRB, 0.96898E+00_JPRB, 0.99782E+00_JPRB, 0.11182E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.16811E+01_JPRB, 0.21847E+01_JPRB, 0.20694E+01_JPRB, 0.16281E+01_JPRB, 0.19971E+01_JPRB, 0.17025E+01_JPRB, 0.82744E+00_JPRB ,&
&  0.83655E+00_JPRB, 0.81387E+00_JPRB, 0.82688E+00_JPRB, 0.84153E+00_JPRB, 0.85097E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.69548E+00_JPRB, 0.68661E+00_JPRB, 0.69956E+00_JPRB, 0.72575E+00_JPRB, 0.75350E+00_JPRB, 0.80284E+00_JPRB, 0.86379E+00_JPRB ,&
&  0.93865E+00_JPRB, 0.99647E+00_JPRB, 0.10247E+01_JPRB, 0.10409E+01_JPRB, 0.10636E+01_JPRB, 0.10907E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.11841E+01_JPRB, 0.12410E+01_JPRB, 0.13180E+01_JPRB, 0.13831E+01_JPRB, 0.14596E+01_JPRB, 0.15931E+01_JPRB, 0.17534E+01_JPRB ,&
&  0.19456E+01_JPRB, 0.21516E+01_JPRB, 0.23851E+01_JPRB, 0.26276E+01_JPRB, 0.28818E+01_JPRB, 0.29999E+01_JPRB, 0.30734E+01_JPRB ,&
&  0.22059E+01_JPRB, 0.30115E+01_JPRB, 0.30493E+01_JPRB, 0.27710E+01_JPRB, 0.25481E+01_JPRB, 0.25256E+01_JPRB, 0.24811E+01_JPRB ,&
&  0.24007E+01_JPRB, 0.21714E+01_JPRB, 0.20180E+01_JPRB, 0.18668E+01_JPRB, 0.18937E+01_JPRB, 0.19023E+01_JPRB, 0.19696E+01_JPRB ,&
&  0.20356E+01_JPRB, 0.20849E+01_JPRB, 0.21198E+01_JPRB, 0.21581E+01_JPRB, 0.22238E+01_JPRB, 0.23259E+01_JPRB, 0.24662E+01_JPRB ,&
&  0.25881E+01_JPRB, 0.26886E+01_JPRB, 0.27638E+01_JPRB, 0.25647E+01_JPRB, 0.23961E+01_JPRB, 0.26139E+01_JPRB, 0.25057E+01_JPRB ,&
&  0.17061E+01_JPRB, 0.12404E+01_JPRB, 0.12734E+01_JPRB, 0.12767E+01_JPRB, 0.11190E+01_JPRB, 0.10053E+01_JPRB, 0.92278E+00_JPRB ,&
&  0.82096E+00_JPRB, 0.74634E+00_JPRB, 0.75642E+00_JPRB, 0.75910E+00_JPRB, 0.76985E+00_JPRB, 0.77378E+00_JPRB, 0.79283E+00_JPRB ,&
&  0.83003E+00_JPRB, 0.81528E+00_JPRB, 0.81512E+00_JPRB, 0.82620E+00_JPRB, 0.87023E+00_JPRB, 0.92647E+00_JPRB, 0.97782E+00_JPRB ,&
&  0.10448E+01_JPRB, 0.97103E+00_JPRB, 0.95588E+00_JPRB, 0.97938E+00_JPRB, 0.99578E+00_JPRB, 0.10297E+01_JPRB, 0.10782E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.11573E+01_JPRB, 0.11889E+01_JPRB, 0.11762E+01_JPRB, 0.10670E+01_JPRB, 0.98343E+00_JPRB, 0.74295E+00_JPRB ,&
&  0.72987E+00_JPRB, 0.71169E+00_JPRB, 0.71292E+00_JPRB, 0.71174E+00_JPRB, 0.71297E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.60586E+00_JPRB, 0.60154E+00_JPRB, 0.61176E+00_JPRB, 0.62653E+00_JPRB, 0.65205E+00_JPRB, 0.69517E+00_JPRB, 0.75839E+00_JPRB ,&
&  0.83051E+00_JPRB, 0.87633E+00_JPRB, 0.91783E+00_JPRB, 0.91914E+00_JPRB, 0.92912E+00_JPRB, 0.95931E+00_JPRB, 0.98699E+00_JPRB ,&
&  0.10433E+01_JPRB, 0.11009E+01_JPRB, 0.11386E+01_JPRB, 0.11581E+01_JPRB, 0.11873E+01_JPRB, 0.12563E+01_JPRB, 0.13549E+01_JPRB ,&
&  0.14854E+01_JPRB, 0.16753E+01_JPRB, 0.19362E+01_JPRB, 0.22868E+01_JPRB, 0.26720E+01_JPRB, 0.28611E+01_JPRB, 0.29152E+01_JPRB ,&
&  0.13870E+01_JPRB, 0.22875E+01_JPRB, 0.30415E+01_JPRB, 0.30040E+01_JPRB, 0.28778E+01_JPRB, 0.28119E+01_JPRB, 0.27737E+01_JPRB ,&
&  0.24598E+01_JPRB, 0.21224E+01_JPRB, 0.19423E+01_JPRB, 0.20134E+01_JPRB, 0.18139E+01_JPRB, 0.18265E+01_JPRB, 0.18672E+01_JPRB ,&
&  0.19235E+01_JPRB, 0.19711E+01_JPRB, 0.19873E+01_JPRB, 0.19965E+01_JPRB, 0.20411E+01_JPRB, 0.21351E+01_JPRB, 0.22633E+01_JPRB ,&
&  0.24349E+01_JPRB, 0.26541E+01_JPRB, 0.28088E+01_JPRB, 0.26253E+01_JPRB, 0.25657E+01_JPRB, 0.27253E+01_JPRB, 0.32666E+01_JPRB ,&
&  0.21733E+01_JPRB, 0.14362E+01_JPRB, 0.14352E+01_JPRB, 0.14567E+01_JPRB, 0.12503E+01_JPRB, 0.11288E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.91796E+00_JPRB, 0.85256E+00_JPRB, 0.81636E+00_JPRB, 0.79621E+00_JPRB, 0.81180E+00_JPRB, 0.84040E+00_JPRB, 0.89012E+00_JPRB ,&
&  0.92101E+00_JPRB, 0.92224E+00_JPRB, 0.91467E+00_JPRB, 0.92087E+00_JPRB, 0.92755E+00_JPRB, 0.93972E+00_JPRB, 0.94151E+00_JPRB ,&
&  0.96154E+00_JPRB, 0.94556E+00_JPRB, 0.94636E+00_JPRB, 0.98112E+00_JPRB, 0.10276E+01_JPRB, 0.10404E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.99041E+00_JPRB, 0.99366E+00_JPRB, 0.98556E+00_JPRB, 0.96180E+00_JPRB, 0.92015E+00_JPRB, 0.85900E+00_JPRB, 0.75670E+00_JPRB ,&
&  0.70764E+00_JPRB, 0.65399E+00_JPRB, 0.62182E+00_JPRB, 0.60626E+00_JPRB, 0.61611E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.58204E+00_JPRB, 0.57632E+00_JPRB, 0.57891E+00_JPRB, 0.60532E+00_JPRB, 0.60377E+00_JPRB, 0.62463E+00_JPRB, 0.65580E+00_JPRB ,&
&  0.72516E+00_JPRB, 0.78926E+00_JPRB, 0.85116E+00_JPRB, 0.85404E+00_JPRB, 0.85008E+00_JPRB, 0.83455E+00_JPRB, 0.84507E+00_JPRB ,&
&  0.87283E+00_JPRB, 0.91302E+00_JPRB, 0.95628E+00_JPRB, 0.98240E+00_JPRB, 0.10052E+01_JPRB, 0.10304E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.11114E+01_JPRB, 0.11881E+01_JPRB, 0.13432E+01_JPRB, 0.16468E+01_JPRB, 0.21548E+01_JPRB, 0.27306E+01_JPRB, 0.31918E+01_JPRB ,&
&  0.21911E+01_JPRB, 0.85515E+00_JPRB, 0.24925E+01_JPRB, 0.32267E+01_JPRB, 0.31625E+01_JPRB, 0.30611E+01_JPRB, 0.29118E+01_JPRB ,&
&  0.25226E+01_JPRB, 0.22380E+01_JPRB, 0.19209E+01_JPRB, 0.20021E+01_JPRB, 0.17554E+01_JPRB, 0.16955E+01_JPRB, 0.17150E+01_JPRB ,&
&  0.17697E+01_JPRB, 0.18243E+01_JPRB, 0.18808E+01_JPRB, 0.18862E+01_JPRB, 0.18695E+01_JPRB, 0.18906E+01_JPRB, 0.19701E+01_JPRB ,&
&  0.21343E+01_JPRB, 0.24086E+01_JPRB, 0.27556E+01_JPRB, 0.28231E+01_JPRB, 0.32049E+01_JPRB, 0.30546E+01_JPRB, 0.32646E+01_JPRB ,&
&  0.22872E+01_JPRB, 0.16927E+01_JPRB, 0.16142E+01_JPRB, 0.16423E+01_JPRB, 0.14139E+01_JPRB, 0.11976E+01_JPRB, 0.11850E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.94564E+00_JPRB, 0.91704E+00_JPRB, 0.88622E+00_JPRB, 0.86265E+00_JPRB, 0.89371E+00_JPRB, 0.94217E+00_JPRB ,&
&  0.98887E+00_JPRB, 0.10094E+01_JPRB, 0.10075E+01_JPRB, 0.10089E+01_JPRB, 0.10074E+01_JPRB, 0.99305E+00_JPRB, 0.97786E+00_JPRB ,&
&  0.99293E+00_JPRB, 0.10118E+01_JPRB, 0.10336E+01_JPRB, 0.10565E+01_JPRB, 0.10813E+01_JPRB, 0.10705E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.10516E+01_JPRB, 0.99745E+00_JPRB, 0.98915E+00_JPRB, 0.95565E+00_JPRB, 0.90745E+00_JPRB, 0.86727E+00_JPRB ,&
&  0.80162E+00_JPRB, 0.70420E+00_JPRB, 0.63714E+00_JPRB, 0.61253E+00_JPRB, 0.59017E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.62527E+00_JPRB, 0.60420E+00_JPRB, 0.60591E+00_JPRB, 0.61954E+00_JPRB, 0.63104E+00_JPRB, 0.64099E+00_JPRB, 0.62688E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.70369E+00_JPRB, 0.74129E+00_JPRB, 0.77757E+00_JPRB, 0.77007E+00_JPRB, 0.75165E+00_JPRB, 0.74046E+00_JPRB ,&
&  0.74013E+00_JPRB, 0.74864E+00_JPRB, 0.77552E+00_JPRB, 0.79066E+00_JPRB, 0.80649E+00_JPRB, 0.82940E+00_JPRB, 0.84428E+00_JPRB ,&
&  0.85779E+00_JPRB, 0.88174E+00_JPRB, 0.93576E+00_JPRB, 0.10573E+01_JPRB, 0.13028E+01_JPRB, 0.17968E+01_JPRB, 0.25660E+01_JPRB ,&
&  0.40191E+01_JPRB, 0.29037E+01_JPRB, 0.12089E+01_JPRB, 0.27257E+01_JPRB, 0.34827E+01_JPRB, 0.35534E+01_JPRB, 0.30169E+01_JPRB ,&
&  0.28191E+01_JPRB, 0.23742E+01_JPRB, 0.20251E+01_JPRB, 0.20717E+01_JPRB, 0.17457E+01_JPRB, 0.15924E+01_JPRB, 0.15319E+01_JPRB ,&
&  0.15447E+01_JPRB, 0.16097E+01_JPRB, 0.16849E+01_JPRB, 0.17405E+01_JPRB, 0.17569E+01_JPRB, 0.17445E+01_JPRB, 0.17507E+01_JPRB ,&
&  0.18444E+01_JPRB, 0.20139E+01_JPRB, 0.23278E+01_JPRB, 0.27963E+01_JPRB, 0.38597E+01_JPRB, 0.36227E+01_JPRB, 0.36061E+01_JPRB ,&
&  0.24146E+01_JPRB, 0.21281E+01_JPRB, 0.20200E+01_JPRB, 0.18773E+01_JPRB, 0.15590E+01_JPRB, 0.13786E+01_JPRB, 0.13851E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.10409E+01_JPRB, 0.98940E+00_JPRB, 0.94438E+00_JPRB, 0.91716E+00_JPRB, 0.92707E+00_JPRB, 0.96548E+00_JPRB ,&
&  0.10131E+01_JPRB, 0.10219E+01_JPRB, 0.10187E+01_JPRB, 0.10174E+01_JPRB, 0.10175E+01_JPRB, 0.10163E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.10171E+01_JPRB, 0.10412E+01_JPRB, 0.10757E+01_JPRB, 0.10781E+01_JPRB, 0.10583E+01_JPRB, 0.10536E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11294E+01_JPRB, 0.10533E+01_JPRB, 0.10424E+01_JPRB, 0.99593E+00_JPRB, 0.96497E+00_JPRB, 0.93645E+00_JPRB ,&
&  0.89639E+00_JPRB, 0.81033E+00_JPRB, 0.71932E+00_JPRB, 0.63616E+00_JPRB, 0.62693E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.68935E+00_JPRB, 0.66934E+00_JPRB, 0.67307E+00_JPRB, 0.66747E+00_JPRB, 0.65687E+00_JPRB, 0.65390E+00_JPRB, 0.64760E+00_JPRB ,&
&  0.64032E+00_JPRB, 0.64073E+00_JPRB, 0.64226E+00_JPRB, 0.68253E+00_JPRB, 0.70266E+00_JPRB, 0.68753E+00_JPRB, 0.67152E+00_JPRB ,&
&  0.65789E+00_JPRB, 0.65809E+00_JPRB, 0.66793E+00_JPRB, 0.67595E+00_JPRB, 0.68306E+00_JPRB, 0.69890E+00_JPRB, 0.71158E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.71130E+00_JPRB, 0.72981E+00_JPRB, 0.77453E+00_JPRB, 0.86766E+00_JPRB, 0.10517E+01_JPRB, 0.14199E+01_JPRB ,&
&  0.23894E+01_JPRB, 0.25168E+01_JPRB, 0.10768E+01_JPRB, 0.21678E+01_JPRB, 0.36072E+01_JPRB, 0.38515E+01_JPRB, 0.36750E+01_JPRB ,&
&  0.33813E+01_JPRB, 0.26641E+01_JPRB, 0.22769E+01_JPRB, 0.21856E+01_JPRB, 0.18440E+01_JPRB, 0.15899E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.13558E+01_JPRB, 0.13546E+01_JPRB, 0.14174E+01_JPRB, 0.14895E+01_JPRB, 0.15586E+01_JPRB, 0.15859E+01_JPRB, 0.15877E+01_JPRB ,&
&  0.16012E+01_JPRB, 0.16769E+01_JPRB, 0.18696E+01_JPRB, 0.20808E+01_JPRB, 0.32255E+01_JPRB, 0.38497E+01_JPRB, 0.40326E+01_JPRB ,&
&  0.30835E+01_JPRB, 0.27637E+01_JPRB, 0.24783E+01_JPRB, 0.21665E+01_JPRB, 0.17668E+01_JPRB, 0.16330E+01_JPRB, 0.16186E+01_JPRB ,&
&  0.13876E+01_JPRB, 0.11889E+01_JPRB, 0.10988E+01_JPRB, 0.10312E+01_JPRB, 0.97854E+00_JPRB, 0.95686E+00_JPRB, 0.95342E+00_JPRB ,&
&  0.97484E+00_JPRB, 0.98949E+00_JPRB, 0.98401E+00_JPRB, 0.98223E+00_JPRB, 0.99096E+00_JPRB, 0.99991E+00_JPRB, 0.10132E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.10412E+01_JPRB, 0.10748E+01_JPRB, 0.10746E+01_JPRB, 0.10903E+01_JPRB, 0.11279E+01_JPRB, 0.11137E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.11704E+01_JPRB, 0.10852E+01_JPRB, 0.10231E+01_JPRB, 0.10399E+01_JPRB, 0.10185E+01_JPRB, 0.99039E+00_JPRB ,&
&  0.10244E+01_JPRB, 0.11267E+01_JPRB, 0.10694E+01_JPRB, 0.75733E+00_JPRB, 0.70437E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.73620E+00_JPRB, 0.70787E+00_JPRB, 0.70327E+00_JPRB, 0.69784E+00_JPRB, 0.68440E+00_JPRB, 0.67416E+00_JPRB, 0.66887E+00_JPRB ,&
&  0.65564E+00_JPRB, 0.63325E+00_JPRB, 0.61359E+00_JPRB, 0.61811E+00_JPRB, 0.63634E+00_JPRB, 0.64952E+00_JPRB, 0.63570E+00_JPRB ,&
&  0.61827E+00_JPRB, 0.61218E+00_JPRB, 0.62505E+00_JPRB, 0.63202E+00_JPRB, 0.63152E+00_JPRB, 0.63777E+00_JPRB, 0.63975E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.63335E+00_JPRB, 0.63636E+00_JPRB, 0.65564E+00_JPRB, 0.69663E+00_JPRB, 0.78288E+00_JPRB, 0.92735E+00_JPRB ,&
&  0.14481E+01_JPRB, 0.19169E+01_JPRB, 0.25055E+01_JPRB, 0.37809E+01_JPRB, 0.47696E+01_JPRB, 0.46278E+01_JPRB, 0.44740E+01_JPRB ,&
&  0.42965E+01_JPRB, 0.36111E+01_JPRB, 0.29033E+01_JPRB, 0.24075E+01_JPRB, 0.19685E+01_JPRB, 0.16483E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.12672E+01_JPRB, 0.12541E+01_JPRB, 0.12749E+01_JPRB, 0.13126E+01_JPRB, 0.13536E+01_JPRB, 0.13715E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.14368E+01_JPRB, 0.15657E+01_JPRB, 0.16922E+01_JPRB, 0.22891E+01_JPRB, 0.33734E+01_JPRB, 0.39327E+01_JPRB ,&
&  0.39338E+01_JPRB, 0.35652E+01_JPRB, 0.30282E+01_JPRB, 0.25249E+01_JPRB, 0.19595E+01_JPRB, 0.19910E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.15785E+01_JPRB, 0.13905E+01_JPRB, 0.12500E+01_JPRB, 0.11477E+01_JPRB, 0.10615E+01_JPRB, 0.99349E+00_JPRB, 0.95885E+00_JPRB ,&
&  0.95176E+00_JPRB, 0.95719E+00_JPRB, 0.94956E+00_JPRB, 0.93835E+00_JPRB, 0.95285E+00_JPRB, 0.96907E+00_JPRB, 0.98963E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.10250E+01_JPRB, 0.10119E+01_JPRB, 0.10410E+01_JPRB, 0.10808E+01_JPRB, 0.10955E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10769E+01_JPRB, 0.10769E+01_JPRB, 0.10551E+01_JPRB, 0.10433E+01_JPRB, 0.10385E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.98136E+00_JPRB, 0.90803E+00_JPRB, 0.81064E+00_JPRB, 0.76823E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.78547E+00_JPRB, 0.75621E+00_JPRB, 0.73723E+00_JPRB, 0.72283E+00_JPRB, 0.71589E+00_JPRB, 0.71529E+00_JPRB, 0.70890E+00_JPRB ,&
&  0.69784E+00_JPRB, 0.67473E+00_JPRB, 0.64487E+00_JPRB, 0.61875E+00_JPRB, 0.62114E+00_JPRB, 0.62025E+00_JPRB, 0.61792E+00_JPRB ,&
&  0.60376E+00_JPRB, 0.59846E+00_JPRB, 0.60309E+00_JPRB, 0.60435E+00_JPRB, 0.60208E+00_JPRB, 0.60006E+00_JPRB, 0.60023E+00_JPRB ,&
&  0.60565E+00_JPRB, 0.60902E+00_JPRB, 0.60858E+00_JPRB, 0.61170E+00_JPRB, 0.62742E+00_JPRB, 0.67815E+00_JPRB, 0.78381E+00_JPRB ,&
&  0.10843E+01_JPRB, 0.17551E+01_JPRB, 0.19938E+01_JPRB, 0.44243E+01_JPRB, 0.51750E+01_JPRB, 0.51333E+01_JPRB, 0.46292E+01_JPRB ,&
&  0.41035E+01_JPRB, 0.40003E+01_JPRB, 0.31774E+01_JPRB, 0.24703E+01_JPRB, 0.19317E+01_JPRB, 0.16329E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.11978E+01_JPRB, 0.11712E+01_JPRB, 0.11600E+01_JPRB, 0.11487E+01_JPRB, 0.11558E+01_JPRB, 0.11656E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.12523E+01_JPRB, 0.13933E+01_JPRB, 0.14938E+01_JPRB, 0.18375E+01_JPRB, 0.26783E+01_JPRB, 0.33984E+01_JPRB ,&
&  0.37432E+01_JPRB, 0.35771E+01_JPRB, 0.32043E+01_JPRB, 0.26950E+01_JPRB, 0.22533E+01_JPRB, 0.20877E+01_JPRB, 0.18976E+01_JPRB ,&
&  0.17019E+01_JPRB, 0.15632E+01_JPRB, 0.13761E+01_JPRB, 0.12442E+01_JPRB, 0.11785E+01_JPRB, 0.10879E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.96817E+00_JPRB, 0.94783E+00_JPRB, 0.93475E+00_JPRB, 0.92138E+00_JPRB, 0.92303E+00_JPRB, 0.93955E+00_JPRB, 0.95950E+00_JPRB ,&
&  0.97621E+00_JPRB, 0.99991E+00_JPRB, 0.97039E+00_JPRB, 0.97426E+00_JPRB, 0.10122E+01_JPRB, 0.10357E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.10210E+01_JPRB, 0.10329E+01_JPRB, 0.10445E+01_JPRB, 0.10279E+01_JPRB, 0.10009E+01_JPRB, 0.10670E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.92563E+00_JPRB, 0.88263E+00_JPRB, 0.83892E+00_JPRB, 0.81349E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.80838E+00_JPRB, 0.78912E+00_JPRB, 0.77347E+00_JPRB, 0.75602E+00_JPRB, 0.74453E+00_JPRB, 0.73831E+00_JPRB, 0.72998E+00_JPRB ,&
&  0.71469E+00_JPRB, 0.69313E+00_JPRB, 0.66824E+00_JPRB, 0.63432E+00_JPRB, 0.61491E+00_JPRB, 0.61199E+00_JPRB, 0.61851E+00_JPRB ,&
&  0.60800E+00_JPRB, 0.59110E+00_JPRB, 0.59107E+00_JPRB, 0.59477E+00_JPRB, 0.59409E+00_JPRB, 0.58701E+00_JPRB, 0.58011E+00_JPRB ,&
&  0.58544E+00_JPRB, 0.59283E+00_JPRB, 0.59759E+00_JPRB, 0.59935E+00_JPRB, 0.60320E+00_JPRB, 0.62296E+00_JPRB, 0.70469E+00_JPRB ,&
&  0.97387E+00_JPRB, 0.23778E+01_JPRB, 0.21385E+01_JPRB, 0.41425E+01_JPRB, 0.44790E+01_JPRB, 0.44408E+01_JPRB, 0.41057E+01_JPRB ,&
&  0.36796E+01_JPRB, 0.33505E+01_JPRB, 0.27258E+01_JPRB, 0.22578E+01_JPRB, 0.18407E+01_JPRB, 0.15315E+01_JPRB, 0.13320E+01_JPRB ,&
&  0.12070E+01_JPRB, 0.11176E+01_JPRB, 0.10656E+01_JPRB, 0.10477E+01_JPRB, 0.10402E+01_JPRB, 0.10340E+01_JPRB, 0.10377E+01_JPRB ,&
&  0.10531E+01_JPRB, 0.11072E+01_JPRB, 0.12477E+01_JPRB, 0.15429E+01_JPRB, 0.17330E+01_JPRB, 0.23205E+01_JPRB, 0.31971E+01_JPRB ,&
&  0.36705E+01_JPRB, 0.35652E+01_JPRB, 0.30398E+01_JPRB, 0.25804E+01_JPRB, 0.22384E+01_JPRB, 0.20067E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.16745E+01_JPRB, 0.15722E+01_JPRB, 0.14605E+01_JPRB, 0.13231E+01_JPRB, 0.12308E+01_JPRB, 0.11651E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.96494E+00_JPRB, 0.93170E+00_JPRB, 0.90621E+00_JPRB, 0.89443E+00_JPRB, 0.90516E+00_JPRB, 0.92246E+00_JPRB ,&
&  0.93325E+00_JPRB, 0.95102E+00_JPRB, 0.92948E+00_JPRB, 0.93058E+00_JPRB, 0.95938E+00_JPRB, 0.98376E+00_JPRB, 0.97597E+00_JPRB ,&
&  0.99313E+00_JPRB, 0.10112E+01_JPRB, 0.10290E+01_JPRB, 0.10579E+01_JPRB, 0.10299E+01_JPRB, 0.11231E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.99567E+00_JPRB, 0.92779E+00_JPRB, 0.87654E+00_JPRB, 0.85776E+00_JPRB, 0.83629E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.81708E+00_JPRB, 0.79928E+00_JPRB, 0.78590E+00_JPRB, 0.77723E+00_JPRB, 0.76938E+00_JPRB, 0.75508E+00_JPRB, 0.73990E+00_JPRB ,&
&  0.72555E+00_JPRB, 0.70992E+00_JPRB, 0.68477E+00_JPRB, 0.64766E+00_JPRB, 0.61810E+00_JPRB, 0.60483E+00_JPRB, 0.60831E+00_JPRB ,&
&  0.60878E+00_JPRB, 0.59875E+00_JPRB, 0.59066E+00_JPRB, 0.58996E+00_JPRB, 0.58978E+00_JPRB, 0.58303E+00_JPRB, 0.57382E+00_JPRB ,&
&  0.56792E+00_JPRB, 0.57062E+00_JPRB, 0.57540E+00_JPRB, 0.58069E+00_JPRB, 0.58471E+00_JPRB, 0.59237E+00_JPRB, 0.64654E+00_JPRB ,&
&  0.92327E+00_JPRB, 0.10129E+01_JPRB, 0.25801E+01_JPRB, 0.35328E+01_JPRB, 0.35738E+01_JPRB, 0.34465E+01_JPRB, 0.34547E+01_JPRB ,&
&  0.31792E+01_JPRB, 0.27619E+01_JPRB, 0.22639E+01_JPRB, 0.18601E+01_JPRB, 0.15618E+01_JPRB, 0.13658E+01_JPRB, 0.12210E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.10302E+01_JPRB, 0.97152E+00_JPRB, 0.93878E+00_JPRB, 0.94273E+00_JPRB, 0.95305E+00_JPRB, 0.96375E+00_JPRB ,&
&  0.97891E+00_JPRB, 0.10049E+01_JPRB, 0.10786E+01_JPRB, 0.13108E+01_JPRB, 0.17953E+01_JPRB, 0.22382E+01_JPRB, 0.29715E+01_JPRB ,&
&  0.36486E+01_JPRB, 0.32882E+01_JPRB, 0.26942E+01_JPRB, 0.22894E+01_JPRB, 0.20004E+01_JPRB, 0.17997E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.14501E+01_JPRB, 0.13822E+01_JPRB, 0.13010E+01_JPRB, 0.12357E+01_JPRB, 0.11882E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.10257E+01_JPRB, 0.96951E+00_JPRB, 0.91781E+00_JPRB, 0.87720E+00_JPRB, 0.87023E+00_JPRB, 0.87687E+00_JPRB ,&
&  0.88168E+00_JPRB, 0.89603E+00_JPRB, 0.89897E+00_JPRB, 0.90817E+00_JPRB, 0.94259E+00_JPRB, 0.95941E+00_JPRB, 0.97018E+00_JPRB ,&
&  0.96755E+00_JPRB, 0.98964E+00_JPRB, 0.99979E+00_JPRB, 0.10267E+01_JPRB, 0.98481E+00_JPRB, 0.10619E+01_JPRB, 0.98752E+00_JPRB ,&
&  0.96400E+00_JPRB, 0.93849E+00_JPRB, 0.88848E+00_JPRB, 0.86703E+00_JPRB, 0.84491E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.82167E+00_JPRB, 0.79811E+00_JPRB, 0.78554E+00_JPRB, 0.77717E+00_JPRB, 0.76981E+00_JPRB, 0.76236E+00_JPRB, 0.74616E+00_JPRB ,&
&  0.72644E+00_JPRB, 0.71515E+00_JPRB, 0.70072E+00_JPRB, 0.68046E+00_JPRB, 0.64973E+00_JPRB, 0.62647E+00_JPRB, 0.62048E+00_JPRB ,&
&  0.61697E+00_JPRB, 0.61025E+00_JPRB, 0.59728E+00_JPRB, 0.58947E+00_JPRB, 0.58167E+00_JPRB, 0.57287E+00_JPRB, 0.56759E+00_JPRB ,&
&  0.55720E+00_JPRB, 0.55323E+00_JPRB, 0.55650E+00_JPRB, 0.56335E+00_JPRB, 0.56920E+00_JPRB, 0.57644E+00_JPRB, 0.59438E+00_JPRB ,&
&  0.82535E+00_JPRB, 0.18595E+01_JPRB, 0.25198E+01_JPRB, 0.28901E+01_JPRB, 0.29016E+01_JPRB, 0.26799E+01_JPRB, 0.26742E+01_JPRB ,&
&  0.23855E+01_JPRB, 0.20869E+01_JPRB, 0.17836E+01_JPRB, 0.15048E+01_JPRB, 0.12834E+01_JPRB, 0.11635E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.96404E+00_JPRB, 0.91205E+00_JPRB, 0.86766E+00_JPRB, 0.85206E+00_JPRB, 0.86992E+00_JPRB, 0.89723E+00_JPRB ,&
&  0.91044E+00_JPRB, 0.92109E+00_JPRB, 0.95206E+00_JPRB, 0.10894E+01_JPRB, 0.15082E+01_JPRB, 0.17342E+01_JPRB, 0.24425E+01_JPRB ,&
&  0.28241E+01_JPRB, 0.25311E+01_JPRB, 0.21438E+01_JPRB, 0.18664E+01_JPRB, 0.16445E+01_JPRB, 0.14766E+01_JPRB, 0.13559E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.12386E+01_JPRB, 0.11986E+01_JPRB, 0.11586E+01_JPRB, 0.11223E+01_JPRB, 0.10960E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10240E+01_JPRB, 0.98614E+00_JPRB, 0.93738E+00_JPRB, 0.88677E+00_JPRB, 0.85723E+00_JPRB, 0.84272E+00_JPRB ,&
&  0.83256E+00_JPRB, 0.84457E+00_JPRB, 0.87041E+00_JPRB, 0.87644E+00_JPRB, 0.91222E+00_JPRB, 0.93646E+00_JPRB, 0.95242E+00_JPRB ,&
&  0.92994E+00_JPRB, 0.94051E+00_JPRB, 0.94174E+00_JPRB, 0.96240E+00_JPRB, 0.10387E+01_JPRB, 0.10282E+01_JPRB, 0.93922E+00_JPRB ,&
&  0.90487E+00_JPRB, 0.88763E+00_JPRB, 0.86833E+00_JPRB, 0.84841E+00_JPRB, 0.85180E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.83983E+00_JPRB, 0.80503E+00_JPRB, 0.78455E+00_JPRB, 0.77169E+00_JPRB, 0.76425E+00_JPRB, 0.75651E+00_JPRB, 0.74205E+00_JPRB ,&
&  0.72420E+00_JPRB, 0.70791E+00_JPRB, 0.69518E+00_JPRB, 0.68599E+00_JPRB, 0.66516E+00_JPRB, 0.63091E+00_JPRB, 0.60436E+00_JPRB ,&
&  0.59705E+00_JPRB, 0.59132E+00_JPRB, 0.58571E+00_JPRB, 0.57531E+00_JPRB, 0.56653E+00_JPRB, 0.56268E+00_JPRB, 0.55829E+00_JPRB ,&
&  0.55336E+00_JPRB, 0.54872E+00_JPRB, 0.55016E+00_JPRB, 0.55473E+00_JPRB, 0.55872E+00_JPRB, 0.56237E+00_JPRB, 0.57093E+00_JPRB ,&
&  0.68698E+00_JPRB, 0.13192E+01_JPRB, 0.16321E+01_JPRB, 0.21022E+01_JPRB, 0.22588E+01_JPRB, 0.20898E+01_JPRB, 0.19814E+01_JPRB ,&
&  0.18224E+01_JPRB, 0.16373E+01_JPRB, 0.14676E+01_JPRB, 0.13084E+01_JPRB, 0.11332E+01_JPRB, 0.10273E+01_JPRB, 0.97620E+00_JPRB ,&
&  0.94172E+00_JPRB, 0.89831E+00_JPRB, 0.85717E+00_JPRB, 0.81611E+00_JPRB, 0.78943E+00_JPRB, 0.79558E+00_JPRB, 0.81380E+00_JPRB ,&
&  0.81685E+00_JPRB, 0.81727E+00_JPRB, 0.82130E+00_JPRB, 0.87483E+00_JPRB, 0.10976E+01_JPRB, 0.13129E+01_JPRB, 0.16131E+01_JPRB ,&
&  0.16348E+01_JPRB, 0.16910E+01_JPRB, 0.15596E+01_JPRB, 0.14181E+01_JPRB, 0.12988E+01_JPRB, 0.12062E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.10559E+01_JPRB, 0.10287E+01_JPRB, 0.10058E+01_JPRB, 0.98662E+00_JPRB, 0.96540E+00_JPRB, 0.95618E+00_JPRB ,&
&  0.95408E+00_JPRB, 0.94168E+00_JPRB, 0.92418E+00_JPRB, 0.90128E+00_JPRB, 0.87096E+00_JPRB, 0.84440E+00_JPRB, 0.82316E+00_JPRB ,&
&  0.80306E+00_JPRB, 0.79104E+00_JPRB, 0.80430E+00_JPRB, 0.82718E+00_JPRB, 0.84462E+00_JPRB, 0.85940E+00_JPRB, 0.86550E+00_JPRB ,&
&  0.87005E+00_JPRB, 0.89691E+00_JPRB, 0.88213E+00_JPRB, 0.90380E+00_JPRB, 0.96985E+00_JPRB, 0.94807E+00_JPRB, 0.88489E+00_JPRB ,&
&  0.85221E+00_JPRB, 0.83142E+00_JPRB, 0.82202E+00_JPRB, 0.82101E+00_JPRB, 0.87230E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.87458E+00_JPRB, 0.83566E+00_JPRB, 0.79710E+00_JPRB, 0.77238E+00_JPRB, 0.75476E+00_JPRB, 0.74002E+00_JPRB, 0.72180E+00_JPRB ,&
&  0.70465E+00_JPRB, 0.68806E+00_JPRB, 0.67464E+00_JPRB, 0.66515E+00_JPRB, 0.65504E+00_JPRB, 0.63094E+00_JPRB, 0.59763E+00_JPRB ,&
&  0.57587E+00_JPRB, 0.56967E+00_JPRB, 0.56562E+00_JPRB, 0.55735E+00_JPRB, 0.54946E+00_JPRB, 0.54634E+00_JPRB, 0.54320E+00_JPRB ,&
&  0.54308E+00_JPRB, 0.54365E+00_JPRB, 0.54699E+00_JPRB, 0.54830E+00_JPRB, 0.54542E+00_JPRB, 0.55163E+00_JPRB, 0.55271E+00_JPRB ,&
&  0.63683E+00_JPRB, 0.87207E+00_JPRB, 0.11094E+01_JPRB, 0.13812E+01_JPRB, 0.15856E+01_JPRB, 0.16416E+01_JPRB, 0.15896E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.14348E+01_JPRB, 0.13217E+01_JPRB, 0.12150E+01_JPRB, 0.10956E+01_JPRB, 0.97056E+00_JPRB, 0.89837E+00_JPRB ,&
&  0.87144E+00_JPRB, 0.84191E+00_JPRB, 0.81515E+00_JPRB, 0.78721E+00_JPRB, 0.76340E+00_JPRB, 0.75420E+00_JPRB, 0.75466E+00_JPRB ,&
&  0.74981E+00_JPRB, 0.74156E+00_JPRB, 0.73178E+00_JPRB, 0.73425E+00_JPRB, 0.77589E+00_JPRB, 0.88967E+00_JPRB, 0.10360E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.11278E+01_JPRB, 0.11365E+01_JPRB, 0.11163E+01_JPRB, 0.10696E+01_JPRB, 0.10128E+01_JPRB, 0.95511E+00_JPRB ,&
&  0.91401E+00_JPRB, 0.88742E+00_JPRB, 0.87410E+00_JPRB, 0.86814E+00_JPRB, 0.86430E+00_JPRB, 0.85677E+00_JPRB, 0.84535E+00_JPRB ,&
&  0.83697E+00_JPRB, 0.83364E+00_JPRB, 0.83351E+00_JPRB, 0.82800E+00_JPRB, 0.81968E+00_JPRB, 0.80897E+00_JPRB, 0.79953E+00_JPRB ,&
&  0.79252E+00_JPRB, 0.78574E+00_JPRB, 0.78019E+00_JPRB, 0.77902E+00_JPRB, 0.78744E+00_JPRB, 0.79306E+00_JPRB, 0.79517E+00_JPRB ,&
&  0.79751E+00_JPRB, 0.81654E+00_JPRB, 0.81584E+00_JPRB, 0.88610E+00_JPRB, 0.93983E+00_JPRB, 0.88112E+00_JPRB, 0.83634E+00_JPRB ,&
&  0.79536E+00_JPRB, 0.77459E+00_JPRB, 0.77363E+00_JPRB, 0.79371E+00_JPRB, 0.86440E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.75051E+00_JPRB, 0.74329E+00_JPRB, 0.73335E+00_JPRB, 0.72405E+00_JPRB, 0.70983E+00_JPRB, 0.69638E+00_JPRB, 0.67966E+00_JPRB ,&
&  0.66410E+00_JPRB, 0.64910E+00_JPRB, 0.63659E+00_JPRB, 0.63231E+00_JPRB, 0.62493E+00_JPRB, 0.61342E+00_JPRB, 0.58945E+00_JPRB ,&
&  0.56950E+00_JPRB, 0.55468E+00_JPRB, 0.54580E+00_JPRB, 0.54154E+00_JPRB, 0.53824E+00_JPRB, 0.53413E+00_JPRB, 0.53229E+00_JPRB ,&
&  0.53287E+00_JPRB, 0.53230E+00_JPRB, 0.53419E+00_JPRB, 0.53293E+00_JPRB, 0.53489E+00_JPRB, 0.53036E+00_JPRB, 0.52639E+00_JPRB ,&
&  0.57057E+00_JPRB, 0.66968E+00_JPRB, 0.81146E+00_JPRB, 0.96566E+00_JPRB, 0.11390E+01_JPRB, 0.12603E+01_JPRB, 0.13380E+01_JPRB ,&
&  0.13702E+01_JPRB, 0.13082E+01_JPRB, 0.12140E+01_JPRB, 0.11079E+01_JPRB, 0.10255E+01_JPRB, 0.94113E+00_JPRB, 0.85760E+00_JPRB ,&
&  0.81533E+00_JPRB, 0.79096E+00_JPRB, 0.77103E+00_JPRB, 0.75513E+00_JPRB, 0.74186E+00_JPRB, 0.73345E+00_JPRB, 0.72690E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.69956E+00_JPRB, 0.68189E+00_JPRB, 0.67410E+00_JPRB, 0.67773E+00_JPRB, 0.71114E+00_JPRB, 0.78226E+00_JPRB ,&
&  0.81478E+00_JPRB, 0.83039E+00_JPRB, 0.84072E+00_JPRB, 0.85808E+00_JPRB, 0.86226E+00_JPRB, 0.85200E+00_JPRB, 0.81566E+00_JPRB ,&
&  0.78347E+00_JPRB, 0.76014E+00_JPRB, 0.74819E+00_JPRB, 0.74429E+00_JPRB, 0.74621E+00_JPRB, 0.74747E+00_JPRB, 0.74825E+00_JPRB ,&
&  0.74143E+00_JPRB, 0.73337E+00_JPRB, 0.73042E+00_JPRB, 0.73048E+00_JPRB, 0.73210E+00_JPRB, 0.73334E+00_JPRB, 0.73655E+00_JPRB ,&
&  0.74449E+00_JPRB, 0.75588E+00_JPRB, 0.76678E+00_JPRB, 0.76306E+00_JPRB, 0.74887E+00_JPRB, 0.73887E+00_JPRB, 0.73580E+00_JPRB ,&
&  0.73665E+00_JPRB, 0.74949E+00_JPRB, 0.76277E+00_JPRB, 0.81682E+00_JPRB, 0.83451E+00_JPRB, 0.81229E+00_JPRB, 0.77159E+00_JPRB ,&
&  0.73853E+00_JPRB, 0.72142E+00_JPRB, 0.72087E+00_JPRB, 0.73506E+00_JPRB, 0.72434E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.67552E+00_JPRB, 0.67231E+00_JPRB, 0.66718E+00_JPRB, 0.66333E+00_JPRB, 0.65474E+00_JPRB, 0.64527E+00_JPRB, 0.63536E+00_JPRB ,&
&  0.62285E+00_JPRB, 0.61031E+00_JPRB, 0.59965E+00_JPRB, 0.59587E+00_JPRB, 0.59355E+00_JPRB, 0.59162E+00_JPRB, 0.58377E+00_JPRB ,&
&  0.57115E+00_JPRB, 0.55925E+00_JPRB, 0.54883E+00_JPRB, 0.54193E+00_JPRB, 0.53908E+00_JPRB, 0.53567E+00_JPRB, 0.53130E+00_JPRB ,&
&  0.52794E+00_JPRB, 0.52728E+00_JPRB, 0.52543E+00_JPRB, 0.52334E+00_JPRB, 0.52047E+00_JPRB, 0.51404E+00_JPRB, 0.51254E+00_JPRB ,&
&  0.52880E+00_JPRB, 0.53396E+00_JPRB, 0.60984E+00_JPRB, 0.67366E+00_JPRB, 0.75074E+00_JPRB, 0.87313E+00_JPRB, 0.10138E+01_JPRB ,&
&  0.11256E+01_JPRB, 0.11320E+01_JPRB, 0.10896E+01_JPRB, 0.10140E+01_JPRB, 0.95933E+00_JPRB, 0.91998E+00_JPRB, 0.86489E+00_JPRB ,&
&  0.81054E+00_JPRB, 0.77314E+00_JPRB, 0.74394E+00_JPRB, 0.72088E+00_JPRB, 0.70403E+00_JPRB, 0.68955E+00_JPRB, 0.67585E+00_JPRB ,&
&  0.65894E+00_JPRB, 0.64232E+00_JPRB, 0.62886E+00_JPRB, 0.62283E+00_JPRB, 0.62561E+00_JPRB, 0.63443E+00_JPRB, 0.65830E+00_JPRB ,&
&  0.67597E+00_JPRB, 0.68335E+00_JPRB, 0.68121E+00_JPRB, 0.68747E+00_JPRB, 0.70027E+00_JPRB, 0.71075E+00_JPRB, 0.70777E+00_JPRB ,&
&  0.68986E+00_JPRB, 0.66676E+00_JPRB, 0.64829E+00_JPRB, 0.63567E+00_JPRB, 0.62966E+00_JPRB, 0.63050E+00_JPRB, 0.63632E+00_JPRB ,&
&  0.63792E+00_JPRB, 0.63578E+00_JPRB, 0.63421E+00_JPRB, 0.63608E+00_JPRB, 0.64013E+00_JPRB, 0.64818E+00_JPRB, 0.65910E+00_JPRB ,&
&  0.67211E+00_JPRB, 0.68693E+00_JPRB, 0.70414E+00_JPRB, 0.71828E+00_JPRB, 0.72268E+00_JPRB, 0.72204E+00_JPRB, 0.71754E+00_JPRB ,&
&  0.71303E+00_JPRB, 0.71512E+00_JPRB, 0.71782E+00_JPRB, 0.75725E+00_JPRB, 0.78523E+00_JPRB, 0.78092E+00_JPRB, 0.74840E+00_JPRB ,&
&  0.71884E+00_JPRB, 0.69142E+00_JPRB, 0.68109E+00_JPRB, 0.63101E+00_JPRB, 0.66844E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.61177E+00_JPRB, 0.60984E+00_JPRB, 0.60463E+00_JPRB, 0.59943E+00_JPRB, 0.59580E+00_JPRB, 0.59034E+00_JPRB, 0.58597E+00_JPRB ,&
&  0.57976E+00_JPRB, 0.57478E+00_JPRB, 0.56673E+00_JPRB, 0.56128E+00_JPRB, 0.55835E+00_JPRB, 0.55789E+00_JPRB, 0.55712E+00_JPRB ,&
&  0.55258E+00_JPRB, 0.54480E+00_JPRB, 0.53681E+00_JPRB, 0.53038E+00_JPRB, 0.52513E+00_JPRB, 0.51935E+00_JPRB, 0.51359E+00_JPRB ,&
&  0.50826E+00_JPRB, 0.50354E+00_JPRB, 0.49935E+00_JPRB, 0.49828E+00_JPRB, 0.49746E+00_JPRB, 0.49567E+00_JPRB, 0.49260E+00_JPRB ,&
&  0.49855E+00_JPRB, 0.45205E+00_JPRB, 0.47396E+00_JPRB, 0.49863E+00_JPRB, 0.53137E+00_JPRB, 0.61290E+00_JPRB, 0.73079E+00_JPRB ,&
&  0.84357E+00_JPRB, 0.91717E+00_JPRB, 0.93510E+00_JPRB, 0.90207E+00_JPRB, 0.86451E+00_JPRB, 0.84186E+00_JPRB, 0.82321E+00_JPRB ,&
&  0.78668E+00_JPRB, 0.75340E+00_JPRB, 0.71997E+00_JPRB, 0.68615E+00_JPRB, 0.65489E+00_JPRB, 0.62712E+00_JPRB, 0.60483E+00_JPRB ,&
&  0.59017E+00_JPRB, 0.58261E+00_JPRB, 0.57875E+00_JPRB, 0.57716E+00_JPRB, 0.57846E+00_JPRB, 0.58063E+00_JPRB, 0.58206E+00_JPRB ,&
&  0.58514E+00_JPRB, 0.59128E+00_JPRB, 0.59168E+00_JPRB, 0.59229E+00_JPRB, 0.59757E+00_JPRB, 0.60787E+00_JPRB, 0.61047E+00_JPRB ,&
&  0.60508E+00_JPRB, 0.59432E+00_JPRB, 0.58216E+00_JPRB, 0.57099E+00_JPRB, 0.56178E+00_JPRB, 0.55373E+00_JPRB, 0.55026E+00_JPRB ,&
&  0.54769E+00_JPRB, 0.54775E+00_JPRB, 0.54916E+00_JPRB, 0.55404E+00_JPRB, 0.56042E+00_JPRB, 0.56748E+00_JPRB, 0.57500E+00_JPRB ,&
&  0.58274E+00_JPRB, 0.59092E+00_JPRB, 0.60248E+00_JPRB, 0.61602E+00_JPRB, 0.62792E+00_JPRB, 0.63502E+00_JPRB, 0.63849E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.64362E+00_JPRB, 0.64387E+00_JPRB, 0.66333E+00_JPRB, 0.67502E+00_JPRB, 0.68549E+00_JPRB, 0.67971E+00_JPRB ,&
&  0.66638E+00_JPRB, 0.65120E+00_JPRB, 0.56958E+00_JPRB, 0.60146E+00_JPRB, 0.60828E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.57102E+00_JPRB, 0.56460E+00_JPRB, 0.55789E+00_JPRB, 0.55056E+00_JPRB, 0.54480E+00_JPRB, 0.54081E+00_JPRB, 0.53904E+00_JPRB ,&
&  0.53975E+00_JPRB, 0.54107E+00_JPRB, 0.53947E+00_JPRB, 0.53354E+00_JPRB, 0.52757E+00_JPRB, 0.52378E+00_JPRB, 0.52212E+00_JPRB ,&
&  0.51979E+00_JPRB, 0.51542E+00_JPRB, 0.50964E+00_JPRB, 0.50398E+00_JPRB, 0.49750E+00_JPRB, 0.49191E+00_JPRB, 0.48701E+00_JPRB ,&
&  0.48278E+00_JPRB, 0.48035E+00_JPRB, 0.47835E+00_JPRB, 0.47683E+00_JPRB, 0.47659E+00_JPRB, 0.47549E+00_JPRB, 0.47865E+00_JPRB ,&
&  0.45141E+00_JPRB, 0.42152E+00_JPRB, 0.43583E+00_JPRB, 0.44942E+00_JPRB, 0.46316E+00_JPRB, 0.49963E+00_JPRB, 0.56190E+00_JPRB ,&
&  0.64595E+00_JPRB, 0.72098E+00_JPRB, 0.76789E+00_JPRB, 0.76715E+00_JPRB, 0.74759E+00_JPRB, 0.73632E+00_JPRB, 0.73125E+00_JPRB ,&
&  0.71272E+00_JPRB, 0.69008E+00_JPRB, 0.66911E+00_JPRB, 0.64425E+00_JPRB, 0.61767E+00_JPRB, 0.59435E+00_JPRB, 0.57609E+00_JPRB ,&
&  0.56240E+00_JPRB, 0.55433E+00_JPRB, 0.55099E+00_JPRB, 0.54813E+00_JPRB, 0.54507E+00_JPRB, 0.54343E+00_JPRB, 0.54053E+00_JPRB ,&
&  0.53891E+00_JPRB, 0.53807E+00_JPRB, 0.53882E+00_JPRB, 0.53827E+00_JPRB, 0.53883E+00_JPRB, 0.53796E+00_JPRB, 0.53524E+00_JPRB ,&
&  0.53064E+00_JPRB, 0.52637E+00_JPRB, 0.52358E+00_JPRB, 0.51826E+00_JPRB, 0.50831E+00_JPRB, 0.49893E+00_JPRB, 0.49403E+00_JPRB ,&
&  0.49255E+00_JPRB, 0.49211E+00_JPRB, 0.49192E+00_JPRB, 0.49371E+00_JPRB, 0.49726E+00_JPRB, 0.50191E+00_JPRB, 0.50752E+00_JPRB ,&
&  0.51398E+00_JPRB, 0.52009E+00_JPRB, 0.52559E+00_JPRB, 0.53178E+00_JPRB, 0.54132E+00_JPRB, 0.55089E+00_JPRB, 0.55975E+00_JPRB ,&
&  0.56755E+00_JPRB, 0.57516E+00_JPRB, 0.58251E+00_JPRB, 0.59426E+00_JPRB, 0.60875E+00_JPRB, 0.61858E+00_JPRB, 0.61977E+00_JPRB ,&
&  0.61347E+00_JPRB, 0.59963E+00_JPRB, 0.59292E+00_JPRB, 0.58288E+00_JPRB, 0.57541E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.51518E+00_JPRB, 0.51094E+00_JPRB, 0.50540E+00_JPRB, 0.49932E+00_JPRB, 0.49328E+00_JPRB, 0.49017E+00_JPRB, 0.48906E+00_JPRB ,&
&  0.49086E+00_JPRB, 0.49142E+00_JPRB, 0.49340E+00_JPRB, 0.49154E+00_JPRB, 0.48861E+00_JPRB, 0.48606E+00_JPRB, 0.48342E+00_JPRB ,&
&  0.48045E+00_JPRB, 0.47656E+00_JPRB, 0.47263E+00_JPRB, 0.46940E+00_JPRB, 0.46575E+00_JPRB, 0.46085E+00_JPRB, 0.45624E+00_JPRB ,&
&  0.45258E+00_JPRB, 0.45056E+00_JPRB, 0.44917E+00_JPRB, 0.44801E+00_JPRB, 0.44684E+00_JPRB, 0.44708E+00_JPRB, 0.44657E+00_JPRB ,&
&  0.44937E+00_JPRB, 0.42116E+00_JPRB, 0.43354E+00_JPRB, 0.43815E+00_JPRB, 0.44612E+00_JPRB, 0.46363E+00_JPRB, 0.48737E+00_JPRB ,&
&  0.52252E+00_JPRB, 0.56842E+00_JPRB, 0.60874E+00_JPRB, 0.62554E+00_JPRB, 0.62189E+00_JPRB, 0.61985E+00_JPRB, 0.62498E+00_JPRB ,&
&  0.62371E+00_JPRB, 0.61659E+00_JPRB, 0.61071E+00_JPRB, 0.60167E+00_JPRB, 0.58744E+00_JPRB, 0.56875E+00_JPRB, 0.55198E+00_JPRB ,&
&  0.54157E+00_JPRB, 0.53535E+00_JPRB, 0.52963E+00_JPRB, 0.52325E+00_JPRB, 0.51439E+00_JPRB, 0.50536E+00_JPRB, 0.49719E+00_JPRB ,&
&  0.49002E+00_JPRB, 0.48200E+00_JPRB, 0.47695E+00_JPRB, 0.47380E+00_JPRB, 0.47001E+00_JPRB, 0.46751E+00_JPRB, 0.46476E+00_JPRB ,&
&  0.46111E+00_JPRB, 0.45695E+00_JPRB, 0.45435E+00_JPRB, 0.45319E+00_JPRB, 0.45085E+00_JPRB, 0.44863E+00_JPRB, 0.44621E+00_JPRB ,&
&  0.44556E+00_JPRB, 0.44528E+00_JPRB, 0.44424E+00_JPRB, 0.44380E+00_JPRB, 0.44474E+00_JPRB, 0.44783E+00_JPRB, 0.45191E+00_JPRB ,&
&  0.45673E+00_JPRB, 0.46227E+00_JPRB, 0.46746E+00_JPRB, 0.47211E+00_JPRB, 0.47846E+00_JPRB, 0.48462E+00_JPRB, 0.49250E+00_JPRB ,&
&  0.50141E+00_JPRB, 0.50996E+00_JPRB, 0.51830E+00_JPRB, 0.52754E+00_JPRB, 0.53669E+00_JPRB, 0.54076E+00_JPRB, 0.54312E+00_JPRB ,&
&  0.54371E+00_JPRB, 0.53730E+00_JPRB, 0.53132E+00_JPRB, 0.52423E+00_JPRB, 0.51805E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.47127E+00_JPRB, 0.46617E+00_JPRB, 0.46046E+00_JPRB, 0.45594E+00_JPRB, 0.45189E+00_JPRB, 0.44770E+00_JPRB, 0.44557E+00_JPRB ,&
&  0.44537E+00_JPRB, 0.44659E+00_JPRB, 0.44909E+00_JPRB, 0.45059E+00_JPRB, 0.45127E+00_JPRB, 0.45214E+00_JPRB, 0.45174E+00_JPRB ,&
&  0.45065E+00_JPRB, 0.44879E+00_JPRB, 0.44623E+00_JPRB, 0.44277E+00_JPRB, 0.43873E+00_JPRB, 0.43492E+00_JPRB, 0.43008E+00_JPRB ,&
&  0.42653E+00_JPRB, 0.42422E+00_JPRB, 0.42277E+00_JPRB, 0.42203E+00_JPRB, 0.42263E+00_JPRB, 0.42353E+00_JPRB, 0.42412E+00_JPRB ,&
&  0.42230E+00_JPRB, 0.42262E+00_JPRB, 0.42975E+00_JPRB, 0.43095E+00_JPRB, 0.43719E+00_JPRB, 0.44995E+00_JPRB, 0.46173E+00_JPRB ,&
&  0.47114E+00_JPRB, 0.48883E+00_JPRB, 0.51286E+00_JPRB, 0.53089E+00_JPRB, 0.53457E+00_JPRB, 0.53497E+00_JPRB, 0.54217E+00_JPRB ,&
&  0.54681E+00_JPRB, 0.54609E+00_JPRB, 0.54690E+00_JPRB, 0.54631E+00_JPRB, 0.54281E+00_JPRB, 0.53775E+00_JPRB, 0.52931E+00_JPRB ,&
&  0.51998E+00_JPRB, 0.51186E+00_JPRB, 0.50309E+00_JPRB, 0.49407E+00_JPRB, 0.48261E+00_JPRB, 0.47020E+00_JPRB, 0.45714E+00_JPRB ,&
&  0.44419E+00_JPRB, 0.43484E+00_JPRB, 0.42869E+00_JPRB, 0.42333E+00_JPRB, 0.41821E+00_JPRB, 0.41478E+00_JPRB, 0.41095E+00_JPRB ,&
&  0.40871E+00_JPRB, 0.40827E+00_JPRB, 0.40668E+00_JPRB, 0.40582E+00_JPRB, 0.40659E+00_JPRB, 0.40710E+00_JPRB, 0.40873E+00_JPRB ,&
&  0.40883E+00_JPRB, 0.40855E+00_JPRB, 0.40850E+00_JPRB, 0.40810E+00_JPRB, 0.40727E+00_JPRB, 0.40844E+00_JPRB, 0.41110E+00_JPRB ,&
&  0.41420E+00_JPRB, 0.41882E+00_JPRB, 0.42392E+00_JPRB, 0.42815E+00_JPRB, 0.43203E+00_JPRB, 0.43797E+00_JPRB, 0.44555E+00_JPRB ,&
&  0.45420E+00_JPRB, 0.46402E+00_JPRB, 0.47329E+00_JPRB, 0.48163E+00_JPRB, 0.48637E+00_JPRB, 0.48834E+00_JPRB, 0.48976E+00_JPRB ,&
&  0.48974E+00_JPRB, 0.48489E+00_JPRB, 0.48215E+00_JPRB, 0.47978E+00_JPRB, 0.47576E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.41721E+00_JPRB, 0.41477E+00_JPRB, 0.41267E+00_JPRB, 0.41151E+00_JPRB, 0.41063E+00_JPRB, 0.41007E+00_JPRB, 0.41027E+00_JPRB ,&
&  0.41144E+00_JPRB, 0.41240E+00_JPRB, 0.41604E+00_JPRB, 0.41805E+00_JPRB, 0.41897E+00_JPRB, 0.42047E+00_JPRB, 0.42139E+00_JPRB ,&
&  0.42024E+00_JPRB, 0.41917E+00_JPRB, 0.41779E+00_JPRB, 0.41515E+00_JPRB, 0.41094E+00_JPRB, 0.40832E+00_JPRB, 0.40581E+00_JPRB ,&
&  0.40219E+00_JPRB, 0.39798E+00_JPRB, 0.39423E+00_JPRB, 0.39203E+00_JPRB, 0.39158E+00_JPRB, 0.39065E+00_JPRB, 0.39070E+00_JPRB ,&
&  0.38976E+00_JPRB, 0.39680E+00_JPRB, 0.38638E+00_JPRB, 0.39611E+00_JPRB, 0.40464E+00_JPRB, 0.41311E+00_JPRB, 0.42214E+00_JPRB ,&
&  0.42841E+00_JPRB, 0.43581E+00_JPRB, 0.44900E+00_JPRB, 0.46065E+00_JPRB, 0.46545E+00_JPRB, 0.46431E+00_JPRB, 0.46637E+00_JPRB ,&
&  0.46808E+00_JPRB, 0.46518E+00_JPRB, 0.46486E+00_JPRB, 0.46800E+00_JPRB, 0.47122E+00_JPRB, 0.47186E+00_JPRB, 0.47111E+00_JPRB ,&
&  0.47033E+00_JPRB, 0.46537E+00_JPRB, 0.45639E+00_JPRB, 0.44650E+00_JPRB, 0.43708E+00_JPRB, 0.42694E+00_JPRB, 0.41491E+00_JPRB ,&
&  0.40196E+00_JPRB, 0.39150E+00_JPRB, 0.38288E+00_JPRB, 0.37695E+00_JPRB, 0.37339E+00_JPRB, 0.37001E+00_JPRB, 0.36692E+00_JPRB ,&
&  0.36551E+00_JPRB, 0.36522E+00_JPRB, 0.36481E+00_JPRB, 0.36425E+00_JPRB, 0.36501E+00_JPRB, 0.36620E+00_JPRB, 0.36991E+00_JPRB ,&
&  0.37409E+00_JPRB, 0.37634E+00_JPRB, 0.37745E+00_JPRB, 0.37837E+00_JPRB, 0.37738E+00_JPRB, 0.37815E+00_JPRB, 0.38008E+00_JPRB ,&
&  0.38222E+00_JPRB, 0.38507E+00_JPRB, 0.38718E+00_JPRB, 0.38927E+00_JPRB, 0.39200E+00_JPRB, 0.39603E+00_JPRB, 0.40221E+00_JPRB ,&
&  0.40837E+00_JPRB, 0.41523E+00_JPRB, 0.42293E+00_JPRB, 0.42986E+00_JPRB, 0.43447E+00_JPRB, 0.43658E+00_JPRB, 0.43804E+00_JPRB ,&
&  0.43740E+00_JPRB, 0.43313E+00_JPRB, 0.42808E+00_JPRB, 0.42407E+00_JPRB, 0.42045E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.37560E+00_JPRB, 0.37545E+00_JPRB, 0.37629E+00_JPRB, 0.37646E+00_JPRB, 0.37619E+00_JPRB, 0.37690E+00_JPRB, 0.37988E+00_JPRB ,&
&  0.38255E+00_JPRB, 0.38437E+00_JPRB, 0.38615E+00_JPRB, 0.38739E+00_JPRB, 0.38827E+00_JPRB, 0.39050E+00_JPRB, 0.39223E+00_JPRB ,&
&  0.39272E+00_JPRB, 0.39174E+00_JPRB, 0.39040E+00_JPRB, 0.38857E+00_JPRB, 0.38600E+00_JPRB, 0.38314E+00_JPRB, 0.38067E+00_JPRB ,&
&  0.37744E+00_JPRB, 0.37376E+00_JPRB, 0.37011E+00_JPRB, 0.36814E+00_JPRB, 0.36729E+00_JPRB, 0.36674E+00_JPRB, 0.36714E+00_JPRB ,&
&  0.36740E+00_JPRB, 0.37013E+00_JPRB, 0.37338E+00_JPRB, 0.37731E+00_JPRB, 0.38103E+00_JPRB, 0.38577E+00_JPRB, 0.39084E+00_JPRB ,&
&  0.39552E+00_JPRB, 0.39810E+00_JPRB, 0.40152E+00_JPRB, 0.40277E+00_JPRB, 0.40815E+00_JPRB, 0.41228E+00_JPRB, 0.41465E+00_JPRB ,&
&  0.41432E+00_JPRB, 0.40787E+00_JPRB, 0.40327E+00_JPRB, 0.40374E+00_JPRB, 0.40628E+00_JPRB, 0.40894E+00_JPRB, 0.41142E+00_JPRB ,&
&  0.41222E+00_JPRB, 0.41075E+00_JPRB, 0.40603E+00_JPRB, 0.39926E+00_JPRB, 0.39194E+00_JPRB, 0.38383E+00_JPRB, 0.37338E+00_JPRB ,&
&  0.36373E+00_JPRB, 0.35587E+00_JPRB, 0.34956E+00_JPRB, 0.34405E+00_JPRB, 0.34010E+00_JPRB, 0.33600E+00_JPRB, 0.33420E+00_JPRB ,&
&  0.33409E+00_JPRB, 0.33591E+00_JPRB, 0.33695E+00_JPRB, 0.33627E+00_JPRB, 0.33510E+00_JPRB, 0.33534E+00_JPRB, 0.33832E+00_JPRB ,&
&  0.34363E+00_JPRB, 0.34779E+00_JPRB, 0.34955E+00_JPRB, 0.34981E+00_JPRB, 0.34947E+00_JPRB, 0.35090E+00_JPRB, 0.35333E+00_JPRB ,&
&  0.35470E+00_JPRB, 0.35550E+00_JPRB, 0.35675E+00_JPRB, 0.35667E+00_JPRB, 0.35863E+00_JPRB, 0.36198E+00_JPRB, 0.36582E+00_JPRB ,&
&  0.36967E+00_JPRB, 0.37447E+00_JPRB, 0.37943E+00_JPRB, 0.38366E+00_JPRB, 0.38748E+00_JPRB, 0.38825E+00_JPRB, 0.38887E+00_JPRB ,&
&  0.38889E+00_JPRB, 0.38709E+00_JPRB, 0.38478E+00_JPRB, 0.38165E+00_JPRB, 0.37799E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.34125E+00_JPRB, 0.34406E+00_JPRB, 0.34765E+00_JPRB, 0.34830E+00_JPRB, 0.34849E+00_JPRB, 0.34992E+00_JPRB, 0.35314E+00_JPRB ,&
&  0.35551E+00_JPRB, 0.35640E+00_JPRB, 0.35672E+00_JPRB, 0.35695E+00_JPRB, 0.35789E+00_JPRB, 0.36015E+00_JPRB, 0.36138E+00_JPRB ,&
&  0.36102E+00_JPRB, 0.36045E+00_JPRB, 0.35957E+00_JPRB, 0.35875E+00_JPRB, 0.35697E+00_JPRB, 0.35435E+00_JPRB, 0.35202E+00_JPRB ,&
&  0.34880E+00_JPRB, 0.34573E+00_JPRB, 0.34326E+00_JPRB, 0.34115E+00_JPRB, 0.33969E+00_JPRB, 0.33919E+00_JPRB, 0.34038E+00_JPRB ,&
&  0.34196E+00_JPRB, 0.34418E+00_JPRB, 0.34724E+00_JPRB, 0.35096E+00_JPRB, 0.35460E+00_JPRB, 0.35865E+00_JPRB, 0.36224E+00_JPRB ,&
&  0.36503E+00_JPRB, 0.36753E+00_JPRB, 0.36963E+00_JPRB, 0.36818E+00_JPRB, 0.36679E+00_JPRB, 0.36958E+00_JPRB, 0.37309E+00_JPRB ,&
&  0.37179E+00_JPRB, 0.36703E+00_JPRB, 0.36114E+00_JPRB, 0.35819E+00_JPRB, 0.35759E+00_JPRB, 0.35668E+00_JPRB, 0.35538E+00_JPRB ,&
&  0.35449E+00_JPRB, 0.35345E+00_JPRB, 0.35140E+00_JPRB, 0.34806E+00_JPRB, 0.34329E+00_JPRB, 0.33731E+00_JPRB, 0.33157E+00_JPRB ,&
&  0.32791E+00_JPRB, 0.32527E+00_JPRB, 0.32469E+00_JPRB, 0.32193E+00_JPRB, 0.31686E+00_JPRB, 0.31190E+00_JPRB, 0.31050E+00_JPRB ,&
&  0.31015E+00_JPRB, 0.31092E+00_JPRB, 0.31130E+00_JPRB, 0.31069E+00_JPRB, 0.31118E+00_JPRB, 0.31186E+00_JPRB, 0.31357E+00_JPRB ,&
&  0.31884E+00_JPRB, 0.32270E+00_JPRB, 0.32297E+00_JPRB, 0.32213E+00_JPRB, 0.32027E+00_JPRB, 0.31998E+00_JPRB, 0.32104E+00_JPRB ,&
&  0.32222E+00_JPRB, 0.32298E+00_JPRB, 0.32351E+00_JPRB, 0.32306E+00_JPRB, 0.32316E+00_JPRB, 0.32466E+00_JPRB, 0.32662E+00_JPRB ,&
&  0.32908E+00_JPRB, 0.33210E+00_JPRB, 0.33477E+00_JPRB, 0.33758E+00_JPRB, 0.34028E+00_JPRB, 0.34186E+00_JPRB, 0.34326E+00_JPRB ,&
&  0.34448E+00_JPRB, 0.34457E+00_JPRB, 0.34393E+00_JPRB, 0.34322E+00_JPRB, 0.34230E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.31499E+00_JPRB, 0.31933E+00_JPRB, 0.32472E+00_JPRB, 0.32572E+00_JPRB, 0.32765E+00_JPRB, 0.33002E+00_JPRB, 0.33351E+00_JPRB ,&
&  0.33596E+00_JPRB, 0.33650E+00_JPRB, 0.33521E+00_JPRB, 0.33384E+00_JPRB, 0.33331E+00_JPRB, 0.33347E+00_JPRB, 0.33335E+00_JPRB ,&
&  0.33344E+00_JPRB, 0.33374E+00_JPRB, 0.33369E+00_JPRB, 0.33337E+00_JPRB, 0.33190E+00_JPRB, 0.32975E+00_JPRB, 0.32726E+00_JPRB ,&
&  0.32333E+00_JPRB, 0.32057E+00_JPRB, 0.31937E+00_JPRB, 0.31860E+00_JPRB, 0.31736E+00_JPRB, 0.31755E+00_JPRB, 0.31917E+00_JPRB ,&
&  0.32289E+00_JPRB, 0.32578E+00_JPRB, 0.32960E+00_JPRB, 0.33351E+00_JPRB, 0.33582E+00_JPRB, 0.33756E+00_JPRB, 0.33957E+00_JPRB ,&
&  0.34085E+00_JPRB, 0.34120E+00_JPRB, 0.34143E+00_JPRB, 0.34187E+00_JPRB, 0.34022E+00_JPRB, 0.34174E+00_JPRB, 0.34460E+00_JPRB ,&
&  0.34563E+00_JPRB, 0.34271E+00_JPRB, 0.33801E+00_JPRB, 0.33443E+00_JPRB, 0.33094E+00_JPRB, 0.32825E+00_JPRB, 0.32594E+00_JPRB ,&
&  0.32409E+00_JPRB, 0.32288E+00_JPRB, 0.32224E+00_JPRB, 0.31991E+00_JPRB, 0.31376E+00_JPRB, 0.30783E+00_JPRB, 0.30449E+00_JPRB ,&
&  0.30289E+00_JPRB, 0.30236E+00_JPRB, 0.30356E+00_JPRB, 0.30191E+00_JPRB, 0.29707E+00_JPRB, 0.29291E+00_JPRB, 0.29113E+00_JPRB ,&
&  0.28936E+00_JPRB, 0.28877E+00_JPRB, 0.28984E+00_JPRB, 0.29071E+00_JPRB, 0.29142E+00_JPRB, 0.29159E+00_JPRB, 0.29332E+00_JPRB ,&
&  0.29546E+00_JPRB, 0.29767E+00_JPRB, 0.29769E+00_JPRB, 0.29486E+00_JPRB, 0.29089E+00_JPRB, 0.28821E+00_JPRB, 0.28813E+00_JPRB ,&
&  0.29022E+00_JPRB, 0.29225E+00_JPRB, 0.29315E+00_JPRB, 0.29370E+00_JPRB, 0.29503E+00_JPRB, 0.29770E+00_JPRB, 0.30107E+00_JPRB ,&
&  0.30263E+00_JPRB, 0.30411E+00_JPRB, 0.30613E+00_JPRB, 0.30819E+00_JPRB, 0.30965E+00_JPRB, 0.31059E+00_JPRB, 0.31096E+00_JPRB ,&
&  0.31040E+00_JPRB, 0.31104E+00_JPRB, 0.31246E+00_JPRB, 0.31409E+00_JPRB, 0.31484E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.29677E+00_JPRB, 0.29975E+00_JPRB, 0.30249E+00_JPRB, 0.30383E+00_JPRB, 0.30541E+00_JPRB, 0.30734E+00_JPRB, 0.31084E+00_JPRB ,&
&  0.31506E+00_JPRB, 0.31664E+00_JPRB, 0.31490E+00_JPRB, 0.31191E+00_JPRB, 0.31013E+00_JPRB, 0.30955E+00_JPRB, 0.30965E+00_JPRB ,&
&  0.30976E+00_JPRB, 0.30984E+00_JPRB, 0.30913E+00_JPRB, 0.30724E+00_JPRB, 0.30446E+00_JPRB, 0.30155E+00_JPRB, 0.29918E+00_JPRB ,&
&  0.29846E+00_JPRB, 0.29958E+00_JPRB, 0.30150E+00_JPRB, 0.30131E+00_JPRB, 0.30126E+00_JPRB, 0.30258E+00_JPRB, 0.30488E+00_JPRB ,&
&  0.30902E+00_JPRB, 0.31088E+00_JPRB, 0.31248E+00_JPRB, 0.30882E+00_JPRB, 0.31286E+00_JPRB, 0.31440E+00_JPRB, 0.31480E+00_JPRB ,&
&  0.31597E+00_JPRB, 0.31695E+00_JPRB, 0.31814E+00_JPRB, 0.31778E+00_JPRB, 0.31414E+00_JPRB, 0.31401E+00_JPRB, 0.31735E+00_JPRB ,&
&  0.32001E+00_JPRB, 0.31852E+00_JPRB, 0.31501E+00_JPRB, 0.31185E+00_JPRB, 0.30919E+00_JPRB, 0.30713E+00_JPRB, 0.30534E+00_JPRB ,&
&  0.30436E+00_JPRB, 0.30313E+00_JPRB, 0.30052E+00_JPRB, 0.29451E+00_JPRB, 0.28792E+00_JPRB, 0.28343E+00_JPRB, 0.28010E+00_JPRB ,&
&  0.27970E+00_JPRB, 0.28145E+00_JPRB, 0.28317E+00_JPRB, 0.28302E+00_JPRB, 0.28103E+00_JPRB, 0.27763E+00_JPRB, 0.27333E+00_JPRB ,&
&  0.26812E+00_JPRB, 0.26638E+00_JPRB, 0.26635E+00_JPRB, 0.26584E+00_JPRB, 0.26650E+00_JPRB, 0.26862E+00_JPRB, 0.27109E+00_JPRB ,&
&  0.27409E+00_JPRB, 0.27407E+00_JPRB, 0.27240E+00_JPRB, 0.26974E+00_JPRB, 0.26699E+00_JPRB, 0.26678E+00_JPRB, 0.26775E+00_JPRB ,&
&  0.26875E+00_JPRB, 0.26978E+00_JPRB, 0.27125E+00_JPRB, 0.27308E+00_JPRB, 0.27525E+00_JPRB, 0.27800E+00_JPRB, 0.28015E+00_JPRB ,&
&  0.28011E+00_JPRB, 0.27896E+00_JPRB, 0.28020E+00_JPRB, 0.28244E+00_JPRB, 0.28369E+00_JPRB, 0.28443E+00_JPRB, 0.28491E+00_JPRB ,&
&  0.28435E+00_JPRB, 0.28399E+00_JPRB, 0.28567E+00_JPRB, 0.29016E+00_JPRB, 0.29415E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.28031E+00_JPRB, 0.28148E+00_JPRB, 0.28143E+00_JPRB, 0.28176E+00_JPRB, 0.28230E+00_JPRB, 0.28411E+00_JPRB, 0.28831E+00_JPRB ,&
&  0.29252E+00_JPRB, 0.29439E+00_JPRB, 0.29255E+00_JPRB, 0.28998E+00_JPRB, 0.28869E+00_JPRB, 0.28819E+00_JPRB, 0.28822E+00_JPRB ,&
&  0.28828E+00_JPRB, 0.28746E+00_JPRB, 0.28471E+00_JPRB, 0.28075E+00_JPRB, 0.27764E+00_JPRB, 0.27659E+00_JPRB, 0.27673E+00_JPRB ,&
&  0.28027E+00_JPRB, 0.28267E+00_JPRB, 0.28481E+00_JPRB, 0.28696E+00_JPRB, 0.28779E+00_JPRB, 0.28787E+00_JPRB, 0.28974E+00_JPRB ,&
&  0.29077E+00_JPRB, 0.28783E+00_JPRB, 0.28239E+00_JPRB, 0.28600E+00_JPRB, 0.28840E+00_JPRB, 0.28985E+00_JPRB, 0.28910E+00_JPRB ,&
&  0.28971E+00_JPRB, 0.29188E+00_JPRB, 0.29423E+00_JPRB, 0.29612E+00_JPRB, 0.29585E+00_JPRB, 0.29404E+00_JPRB, 0.29471E+00_JPRB ,&
&  0.29630E+00_JPRB, 0.29533E+00_JPRB, 0.29202E+00_JPRB, 0.28894E+00_JPRB, 0.28551E+00_JPRB, 0.28381E+00_JPRB, 0.28344E+00_JPRB ,&
&  0.28250E+00_JPRB, 0.28008E+00_JPRB, 0.27552E+00_JPRB, 0.27096E+00_JPRB, 0.26753E+00_JPRB, 0.26390E+00_JPRB, 0.26187E+00_JPRB ,&
&  0.26188E+00_JPRB, 0.26273E+00_JPRB, 0.26248E+00_JPRB, 0.26030E+00_JPRB, 0.25365E+00_JPRB, 0.24445E+00_JPRB, 0.23693E+00_JPRB ,&
&  0.23551E+00_JPRB, 0.23956E+00_JPRB, 0.24674E+00_JPRB, 0.24953E+00_JPRB, 0.25252E+00_JPRB, 0.25442E+00_JPRB, 0.25581E+00_JPRB ,&
&  0.25334E+00_JPRB, 0.24148E+00_JPRB, 0.23623E+00_JPRB, 0.23277E+00_JPRB, 0.22973E+00_JPRB, 0.22996E+00_JPRB, 0.23210E+00_JPRB ,&
&  0.23616E+00_JPRB, 0.24050E+00_JPRB, 0.24386E+00_JPRB, 0.24600E+00_JPRB, 0.24679E+00_JPRB, 0.24698E+00_JPRB, 0.24634E+00_JPRB ,&
&  0.24697E+00_JPRB, 0.25141E+00_JPRB, 0.25967E+00_JPRB, 0.26629E+00_JPRB, 0.26740E+00_JPRB, 0.26896E+00_JPRB, 0.26890E+00_JPRB ,&
&  0.26886E+00_JPRB, 0.27070E+00_JPRB, 0.27165E+00_JPRB, 0.27425E+00_JPRB, 0.27759E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.26684E+00_JPRB, 0.26786E+00_JPRB, 0.26774E+00_JPRB, 0.26651E+00_JPRB, 0.26502E+00_JPRB, 0.26497E+00_JPRB, 0.26624E+00_JPRB ,&
&  0.26910E+00_JPRB, 0.27111E+00_JPRB, 0.27146E+00_JPRB, 0.27143E+00_JPRB, 0.27059E+00_JPRB, 0.26875E+00_JPRB, 0.26734E+00_JPRB ,&
&  0.26628E+00_JPRB, 0.26576E+00_JPRB, 0.26534E+00_JPRB, 0.26531E+00_JPRB, 0.26533E+00_JPRB, 0.26614E+00_JPRB, 0.26678E+00_JPRB ,&
&  0.27016E+00_JPRB, 0.26987E+00_JPRB, 0.26919E+00_JPRB, 0.26906E+00_JPRB, 0.26905E+00_JPRB, 0.26934E+00_JPRB, 0.26934E+00_JPRB ,&
&  0.26593E+00_JPRB, 0.25842E+00_JPRB, 0.25242E+00_JPRB, 0.25905E+00_JPRB, 0.26965E+00_JPRB, 0.26991E+00_JPRB, 0.27032E+00_JPRB ,&
&  0.27179E+00_JPRB, 0.27463E+00_JPRB, 0.27695E+00_JPRB, 0.27805E+00_JPRB, 0.27724E+00_JPRB, 0.27628E+00_JPRB, 0.27653E+00_JPRB ,&
&  0.27734E+00_JPRB, 0.27551E+00_JPRB, 0.26958E+00_JPRB, 0.26260E+00_JPRB, 0.25559E+00_JPRB, 0.25293E+00_JPRB, 0.25139E+00_JPRB ,&
&  0.24863E+00_JPRB, 0.24691E+00_JPRB, 0.24634E+00_JPRB, 0.24587E+00_JPRB, 0.24629E+00_JPRB, 0.24370E+00_JPRB, 0.23953E+00_JPRB ,&
&  0.23676E+00_JPRB, 0.23226E+00_JPRB, 0.22728E+00_JPRB, 0.22199E+00_JPRB, 0.21757E+00_JPRB, 0.21228E+00_JPRB, 0.20860E+00_JPRB ,&
&  0.20804E+00_JPRB, 0.21273E+00_JPRB, 0.22323E+00_JPRB, 0.23295E+00_JPRB, 0.23713E+00_JPRB, 0.22889E+00_JPRB, 0.22314E+00_JPRB ,&
&  0.21581E+00_JPRB, 0.20918E+00_JPRB, 0.20544E+00_JPRB, 0.20484E+00_JPRB, 0.20522E+00_JPRB, 0.20624E+00_JPRB, 0.20834E+00_JPRB ,&
&  0.21108E+00_JPRB, 0.21437E+00_JPRB, 0.21676E+00_JPRB, 0.21868E+00_JPRB, 0.22006E+00_JPRB, 0.22042E+00_JPRB, 0.22011E+00_JPRB ,&
&  0.21983E+00_JPRB, 0.22126E+00_JPRB, 0.22369E+00_JPRB, 0.22831E+00_JPRB, 0.23102E+00_JPRB, 0.23479E+00_JPRB, 0.23945E+00_JPRB ,&
&  0.24655E+00_JPRB, 0.25442E+00_JPRB, 0.25850E+00_JPRB, 0.26137E+00_JPRB, 0.26447E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.25293E+00_JPRB, 0.25762E+00_JPRB, 0.25967E+00_JPRB, 0.25885E+00_JPRB, 0.25824E+00_JPRB, 0.25854E+00_JPRB, 0.25902E+00_JPRB ,&
&  0.25987E+00_JPRB, 0.25931E+00_JPRB, 0.25780E+00_JPRB, 0.25655E+00_JPRB, 0.25520E+00_JPRB, 0.25343E+00_JPRB, 0.25248E+00_JPRB ,&
&  0.25180E+00_JPRB, 0.25235E+00_JPRB, 0.25262E+00_JPRB, 0.25398E+00_JPRB, 0.25462E+00_JPRB, 0.25637E+00_JPRB, 0.25440E+00_JPRB ,&
&  0.25035E+00_JPRB, 0.24794E+00_JPRB, 0.25040E+00_JPRB, 0.25075E+00_JPRB, 0.25136E+00_JPRB, 0.25210E+00_JPRB, 0.25371E+00_JPRB ,&
&  0.25089E+00_JPRB, 0.24819E+00_JPRB, 0.24592E+00_JPRB, 0.25281E+00_JPRB, 0.25997E+00_JPRB, 0.25828E+00_JPRB, 0.25818E+00_JPRB ,&
&  0.26000E+00_JPRB, 0.26268E+00_JPRB, 0.26410E+00_JPRB, 0.26430E+00_JPRB, 0.26311E+00_JPRB, 0.26292E+00_JPRB, 0.26184E+00_JPRB ,&
&  0.25857E+00_JPRB, 0.25560E+00_JPRB, 0.24549E+00_JPRB, 0.22785E+00_JPRB, 0.22192E+00_JPRB, 0.21947E+00_JPRB, 0.21552E+00_JPRB ,&
&  0.21005E+00_JPRB, 0.20823E+00_JPRB, 0.20838E+00_JPRB, 0.20614E+00_JPRB, 0.20623E+00_JPRB, 0.20589E+00_JPRB, 0.20445E+00_JPRB ,&
&  0.20270E+00_JPRB, 0.20157E+00_JPRB, 0.19994E+00_JPRB, 0.19721E+00_JPRB, 0.19508E+00_JPRB, 0.19466E+00_JPRB, 0.19561E+00_JPRB ,&
&  0.19770E+00_JPRB, 0.20062E+00_JPRB, 0.20966E+00_JPRB, 0.21486E+00_JPRB, 0.21300E+00_JPRB, 0.20988E+00_JPRB, 0.20326E+00_JPRB ,&
&  0.19659E+00_JPRB, 0.19465E+00_JPRB, 0.19330E+00_JPRB, 0.19210E+00_JPRB, 0.19251E+00_JPRB, 0.19367E+00_JPRB, 0.19480E+00_JPRB ,&
&  0.19628E+00_JPRB, 0.19814E+00_JPRB, 0.19931E+00_JPRB, 0.20000E+00_JPRB, 0.20066E+00_JPRB, 0.20096E+00_JPRB, 0.20148E+00_JPRB ,&
&  0.20144E+00_JPRB, 0.20292E+00_JPRB, 0.20402E+00_JPRB, 0.20657E+00_JPRB, 0.20822E+00_JPRB, 0.21050E+00_JPRB, 0.21573E+00_JPRB ,&
&  0.21952E+00_JPRB, 0.23144E+00_JPRB, 0.24675E+00_JPRB, 0.25198E+00_JPRB, 0.24812E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.24355E+00_JPRB, 0.24576E+00_JPRB, 0.24741E+00_JPRB, 0.24571E+00_JPRB, 0.24573E+00_JPRB, 0.24589E+00_JPRB, 0.24705E+00_JPRB ,&
&  0.24752E+00_JPRB, 0.24698E+00_JPRB, 0.24670E+00_JPRB, 0.24485E+00_JPRB, 0.24328E+00_JPRB, 0.23695E+00_JPRB, 0.23463E+00_JPRB ,&
&  0.23272E+00_JPRB, 0.23375E+00_JPRB, 0.23444E+00_JPRB, 0.23662E+00_JPRB, 0.23644E+00_JPRB, 0.23487E+00_JPRB, 0.23109E+00_JPRB ,&
&  0.22850E+00_JPRB, 0.22732E+00_JPRB, 0.22589E+00_JPRB, 0.22652E+00_JPRB, 0.22899E+00_JPRB, 0.23139E+00_JPRB, 0.23305E+00_JPRB ,&
&  0.23434E+00_JPRB, 0.23695E+00_JPRB, 0.23936E+00_JPRB, 0.24252E+00_JPRB, 0.24886E+00_JPRB, 0.24901E+00_JPRB, 0.25029E+00_JPRB ,&
&  0.25130E+00_JPRB, 0.25232E+00_JPRB, 0.25284E+00_JPRB, 0.25227E+00_JPRB, 0.25041E+00_JPRB, 0.24440E+00_JPRB, 0.24273E+00_JPRB ,&
&  0.23056E+00_JPRB, 0.22329E+00_JPRB, 0.21689E+00_JPRB, 0.21238E+00_JPRB, 0.20611E+00_JPRB, 0.20279E+00_JPRB, 0.19904E+00_JPRB ,&
&  0.19585E+00_JPRB, 0.19236E+00_JPRB, 0.18984E+00_JPRB, 0.18886E+00_JPRB, 0.18856E+00_JPRB, 0.18823E+00_JPRB, 0.18736E+00_JPRB ,&
&  0.18747E+00_JPRB, 0.18695E+00_JPRB, 0.18656E+00_JPRB, 0.18615E+00_JPRB, 0.18586E+00_JPRB, 0.18592E+00_JPRB, 0.18677E+00_JPRB ,&
&  0.18894E+00_JPRB, 0.19083E+00_JPRB, 0.19289E+00_JPRB, 0.19511E+00_JPRB, 0.19477E+00_JPRB, 0.19288E+00_JPRB, 0.19000E+00_JPRB ,&
&  0.18736E+00_JPRB, 0.18563E+00_JPRB, 0.18379E+00_JPRB, 0.18327E+00_JPRB, 0.18393E+00_JPRB, 0.18475E+00_JPRB, 0.18627E+00_JPRB ,&
&  0.18761E+00_JPRB, 0.18875E+00_JPRB, 0.18953E+00_JPRB, 0.18998E+00_JPRB, 0.19074E+00_JPRB, 0.19141E+00_JPRB, 0.19257E+00_JPRB ,&
&  0.19387E+00_JPRB, 0.19546E+00_JPRB, 0.19683E+00_JPRB, 0.19876E+00_JPRB, 0.20179E+00_JPRB, 0.20613E+00_JPRB, 0.21375E+00_JPRB ,&
&  0.22054E+00_JPRB, 0.23831E+00_JPRB, 0.24050E+00_JPRB, 0.24220E+00_JPRB, 0.24399E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.23350E+00_JPRB, 0.23538E+00_JPRB, 0.23629E+00_JPRB, 0.23727E+00_JPRB, 0.23722E+00_JPRB, 0.23730E+00_JPRB, 0.23797E+00_JPRB ,&
&  0.23393E+00_JPRB, 0.23129E+00_JPRB, 0.22668E+00_JPRB, 0.22261E+00_JPRB, 0.21868E+00_JPRB, 0.21557E+00_JPRB, 0.21376E+00_JPRB ,&
&  0.21278E+00_JPRB, 0.21301E+00_JPRB, 0.21293E+00_JPRB, 0.21338E+00_JPRB, 0.21208E+00_JPRB, 0.21081E+00_JPRB, 0.20964E+00_JPRB ,&
&  0.20850E+00_JPRB, 0.20931E+00_JPRB, 0.21016E+00_JPRB, 0.21383E+00_JPRB, 0.21899E+00_JPRB, 0.22463E+00_JPRB, 0.22888E+00_JPRB ,&
&  0.23174E+00_JPRB, 0.23405E+00_JPRB, 0.23717E+00_JPRB, 0.23860E+00_JPRB, 0.24027E+00_JPRB, 0.24184E+00_JPRB, 0.24228E+00_JPRB ,&
&  0.24360E+00_JPRB, 0.24334E+00_JPRB, 0.24306E+00_JPRB, 0.24294E+00_JPRB, 0.23389E+00_JPRB, 0.22747E+00_JPRB, 0.22075E+00_JPRB ,&
&  0.21441E+00_JPRB, 0.20798E+00_JPRB, 0.20218E+00_JPRB, 0.19912E+00_JPRB, 0.19545E+00_JPRB, 0.19335E+00_JPRB, 0.19157E+00_JPRB ,&
&  0.19018E+00_JPRB, 0.18874E+00_JPRB, 0.18665E+00_JPRB, 0.18462E+00_JPRB, 0.18343E+00_JPRB, 0.18240E+00_JPRB, 0.18107E+00_JPRB ,&
&  0.18097E+00_JPRB, 0.18024E+00_JPRB, 0.18025E+00_JPRB, 0.17986E+00_JPRB, 0.17960E+00_JPRB, 0.17936E+00_JPRB, 0.17934E+00_JPRB ,&
&  0.17975E+00_JPRB, 0.18070E+00_JPRB, 0.18113E+00_JPRB, 0.18172E+00_JPRB, 0.18172E+00_JPRB, 0.18105E+00_JPRB, 0.18049E+00_JPRB ,&
&  0.17975E+00_JPRB, 0.17915E+00_JPRB, 0.17914E+00_JPRB, 0.17896E+00_JPRB, 0.17948E+00_JPRB, 0.18018E+00_JPRB, 0.18087E+00_JPRB ,&
&  0.18187E+00_JPRB, 0.18257E+00_JPRB, 0.18330E+00_JPRB, 0.18431E+00_JPRB, 0.18511E+00_JPRB, 0.18617E+00_JPRB, 0.18739E+00_JPRB ,&
&  0.18824E+00_JPRB, 0.19012E+00_JPRB, 0.19166E+00_JPRB, 0.19360E+00_JPRB, 0.19760E+00_JPRB, 0.20129E+00_JPRB, 0.21049E+00_JPRB ,&
&  0.21820E+00_JPRB, 0.23375E+00_JPRB, 0.23471E+00_JPRB, 0.23115E+00_JPRB, 0.23217E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.22781E+00_JPRB, 0.22921E+00_JPRB, 0.22961E+00_JPRB, 0.23042E+00_JPRB, 0.23058E+00_JPRB, 0.23198E+00_JPRB, 0.23185E+00_JPRB ,&
&  0.23198E+00_JPRB, 0.22836E+00_JPRB, 0.22620E+00_JPRB, 0.22350E+00_JPRB, 0.22116E+00_JPRB, 0.21825E+00_JPRB, 0.21545E+00_JPRB ,&
&  0.21327E+00_JPRB, 0.21132E+00_JPRB, 0.20917E+00_JPRB, 0.20629E+00_JPRB, 0.20334E+00_JPRB, 0.20129E+00_JPRB, 0.20000E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19997E+00_JPRB, 0.20198E+00_JPRB, 0.20573E+00_JPRB, 0.21057E+00_JPRB, 0.21439E+00_JPRB, 0.21961E+00_JPRB ,&
&  0.22553E+00_JPRB, 0.22746E+00_JPRB, 0.23063E+00_JPRB, 0.23198E+00_JPRB, 0.23175E+00_JPRB, 0.23257E+00_JPRB, 0.23076E+00_JPRB ,&
&  0.23082E+00_JPRB, 0.22713E+00_JPRB, 0.23166E+00_JPRB, 0.22769E+00_JPRB, 0.22028E+00_JPRB, 0.21729E+00_JPRB, 0.21085E+00_JPRB ,&
&  0.20664E+00_JPRB, 0.20189E+00_JPRB, 0.19886E+00_JPRB, 0.19725E+00_JPRB, 0.19367E+00_JPRB, 0.19179E+00_JPRB, 0.18879E+00_JPRB ,&
&  0.18601E+00_JPRB, 0.18522E+00_JPRB, 0.18359E+00_JPRB, 0.18304E+00_JPRB, 0.18222E+00_JPRB, 0.18097E+00_JPRB, 0.17962E+00_JPRB ,&
&  0.17872E+00_JPRB, 0.17795E+00_JPRB, 0.17759E+00_JPRB, 0.17703E+00_JPRB, 0.17623E+00_JPRB, 0.17542E+00_JPRB, 0.17446E+00_JPRB ,&
&  0.17385E+00_JPRB, 0.17360E+00_JPRB, 0.17345E+00_JPRB, 0.17354E+00_JPRB, 0.17362E+00_JPRB, 0.17346E+00_JPRB, 0.17367E+00_JPRB ,&
&  0.17399E+00_JPRB, 0.17467E+00_JPRB, 0.17517E+00_JPRB, 0.17586E+00_JPRB, 0.17642E+00_JPRB, 0.17717E+00_JPRB, 0.17766E+00_JPRB ,&
&  0.17827E+00_JPRB, 0.17924E+00_JPRB, 0.18008E+00_JPRB, 0.18111E+00_JPRB, 0.18221E+00_JPRB, 0.18311E+00_JPRB, 0.18463E+00_JPRB ,&
&  0.18617E+00_JPRB, 0.18769E+00_JPRB, 0.18993E+00_JPRB, 0.19211E+00_JPRB, 0.19493E+00_JPRB, 0.19950E+00_JPRB, 0.20462E+00_JPRB ,&
&  0.21106E+00_JPRB, 0.22039E+00_JPRB, 0.22063E+00_JPRB, 0.22444E+00_JPRB, 0.22633E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.22735E+00_JPRB, 0.22869E+00_JPRB, 0.22979E+00_JPRB, 0.23032E+00_JPRB, 0.23106E+00_JPRB, 0.23107E+00_JPRB, 0.23092E+00_JPRB ,&
&  0.22782E+00_JPRB, 0.22656E+00_JPRB, 0.22465E+00_JPRB, 0.22249E+00_JPRB, 0.22060E+00_JPRB, 0.21765E+00_JPRB, 0.21509E+00_JPRB ,&
&  0.21274E+00_JPRB, 0.21003E+00_JPRB, 0.20747E+00_JPRB, 0.20439E+00_JPRB, 0.20156E+00_JPRB, 0.19891E+00_JPRB, 0.19776E+00_JPRB ,&
&  0.19730E+00_JPRB, 0.19798E+00_JPRB, 0.19998E+00_JPRB, 0.20144E+00_JPRB, 0.20351E+00_JPRB, 0.20612E+00_JPRB, 0.21011E+00_JPRB ,&
&  0.21297E+00_JPRB, 0.21667E+00_JPRB, 0.21904E+00_JPRB, 0.21969E+00_JPRB, 0.21905E+00_JPRB, 0.21876E+00_JPRB, 0.21786E+00_JPRB ,&
&  0.21575E+00_JPRB, 0.21360E+00_JPRB, 0.21180E+00_JPRB, 0.20981E+00_JPRB, 0.20770E+00_JPRB, 0.20527E+00_JPRB, 0.20218E+00_JPRB ,&
&  0.19944E+00_JPRB, 0.19704E+00_JPRB, 0.19472E+00_JPRB, 0.19288E+00_JPRB, 0.19105E+00_JPRB, 0.18935E+00_JPRB, 0.18742E+00_JPRB ,&
&  0.18592E+00_JPRB, 0.18465E+00_JPRB, 0.18303E+00_JPRB, 0.18180E+00_JPRB, 0.18037E+00_JPRB, 0.17918E+00_JPRB, 0.17788E+00_JPRB ,&
&  0.17691E+00_JPRB, 0.17610E+00_JPRB, 0.17545E+00_JPRB, 0.17484E+00_JPRB, 0.17451E+00_JPRB, 0.17420E+00_JPRB, 0.17383E+00_JPRB ,&
&  0.17353E+00_JPRB, 0.17365E+00_JPRB, 0.17378E+00_JPRB, 0.17391E+00_JPRB, 0.17409E+00_JPRB, 0.17374E+00_JPRB, 0.17381E+00_JPRB ,&
&  0.17413E+00_JPRB, 0.17445E+00_JPRB, 0.17440E+00_JPRB, 0.17478E+00_JPRB, 0.17544E+00_JPRB, 0.17623E+00_JPRB, 0.17749E+00_JPRB ,&
&  0.17847E+00_JPRB, 0.17948E+00_JPRB, 0.18045E+00_JPRB, 0.18144E+00_JPRB, 0.18277E+00_JPRB, 0.18421E+00_JPRB, 0.18570E+00_JPRB ,&
&  0.18749E+00_JPRB, 0.18915E+00_JPRB, 0.19087E+00_JPRB, 0.19263E+00_JPRB, 0.19551E+00_JPRB, 0.19811E+00_JPRB, 0.20213E+00_JPRB ,&
&  0.20615E+00_JPRB, 0.20748E+00_JPRB, 0.22155E+00_JPRB, 0.22258E+00_JPRB, 0.22421E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.20612E+00_JPRB, 0.20831E+00_JPRB, 0.20959E+00_JPRB, 0.21088E+00_JPRB, 0.20735E+00_JPRB, 0.21915E+00_JPRB, 0.21685E+00_JPRB ,&
&  0.21640E+00_JPRB, 0.21177E+00_JPRB, 0.21259E+00_JPRB, 0.21212E+00_JPRB, 0.21080E+00_JPRB, 0.20929E+00_JPRB, 0.20751E+00_JPRB ,&
&  0.20573E+00_JPRB, 0.20341E+00_JPRB, 0.20158E+00_JPRB, 0.20005E+00_JPRB, 0.19864E+00_JPRB, 0.19744E+00_JPRB, 0.19623E+00_JPRB ,&
&  0.19620E+00_JPRB, 0.19702E+00_JPRB, 0.19824E+00_JPRB, 0.19731E+00_JPRB, 0.19765E+00_JPRB, 0.19851E+00_JPRB, 0.19974E+00_JPRB ,&
&  0.20074E+00_JPRB, 0.20225E+00_JPRB, 0.20325E+00_JPRB, 0.20360E+00_JPRB, 0.20437E+00_JPRB, 0.20369E+00_JPRB, 0.20321E+00_JPRB ,&
&  0.20244E+00_JPRB, 0.20135E+00_JPRB, 0.19997E+00_JPRB, 0.19878E+00_JPRB, 0.19746E+00_JPRB, 0.19579E+00_JPRB, 0.19406E+00_JPRB ,&
&  0.19252E+00_JPRB, 0.19118E+00_JPRB, 0.18968E+00_JPRB, 0.18846E+00_JPRB, 0.18729E+00_JPRB, 0.18612E+00_JPRB, 0.18502E+00_JPRB ,&
&  0.18415E+00_JPRB, 0.18337E+00_JPRB, 0.18253E+00_JPRB, 0.18168E+00_JPRB, 0.18076E+00_JPRB, 0.17987E+00_JPRB, 0.17902E+00_JPRB ,&
&  0.17868E+00_JPRB, 0.17839E+00_JPRB, 0.17807E+00_JPRB, 0.17774E+00_JPRB, 0.17770E+00_JPRB, 0.17764E+00_JPRB, 0.17752E+00_JPRB ,&
&  0.17738E+00_JPRB, 0.17712E+00_JPRB, 0.17704E+00_JPRB, 0.17695E+00_JPRB, 0.17686E+00_JPRB, 0.17679E+00_JPRB, 0.17690E+00_JPRB ,&
&  0.17707E+00_JPRB, 0.17733E+00_JPRB, 0.17789E+00_JPRB, 0.17847E+00_JPRB, 0.17908E+00_JPRB, 0.17963E+00_JPRB, 0.18045E+00_JPRB ,&
&  0.18137E+00_JPRB, 0.18223E+00_JPRB, 0.18295E+00_JPRB, 0.18355E+00_JPRB, 0.18460E+00_JPRB, 0.18560E+00_JPRB, 0.18659E+00_JPRB ,&
&  0.18766E+00_JPRB, 0.18905E+00_JPRB, 0.19049E+00_JPRB, 0.19197E+00_JPRB, 0.19373E+00_JPRB, 0.19514E+00_JPRB, 0.19657E+00_JPRB ,&
&  0.19827E+00_JPRB, 0.20150E+00_JPRB, 0.20353E+00_JPRB, 0.20480E+00_JPRB, 0.20608E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.19080E+00_JPRB, 0.19095E+00_JPRB, 0.19098E+00_JPRB, 0.19091E+00_JPRB, 0.19061E+00_JPRB, 0.19052E+00_JPRB, 0.19043E+00_JPRB ,&
&  0.19034E+00_JPRB, 0.19008E+00_JPRB, 0.19029E+00_JPRB, 0.19038E+00_JPRB, 0.19039E+00_JPRB, 0.19038E+00_JPRB, 0.19019E+00_JPRB ,&
&  0.18981E+00_JPRB, 0.18934E+00_JPRB, 0.18817E+00_JPRB, 0.18795E+00_JPRB, 0.18794E+00_JPRB, 0.18814E+00_JPRB, 0.18829E+00_JPRB ,&
&  0.18856E+00_JPRB, 0.18883E+00_JPRB, 0.18907E+00_JPRB, 0.18980E+00_JPRB, 0.19012E+00_JPRB, 0.19046E+00_JPRB, 0.19081E+00_JPRB ,&
&  0.19114E+00_JPRB, 0.19153E+00_JPRB, 0.19184E+00_JPRB, 0.19217E+00_JPRB, 0.19248E+00_JPRB, 0.19245E+00_JPRB, 0.19223E+00_JPRB ,&
&  0.19188E+00_JPRB, 0.19132E+00_JPRB, 0.19072E+00_JPRB, 0.19018E+00_JPRB, 0.18969E+00_JPRB, 0.18917E+00_JPRB, 0.18868E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18756E+00_JPRB, 0.18694E+00_JPRB, 0.18638E+00_JPRB, 0.18581E+00_JPRB, 0.18531E+00_JPRB, 0.18474E+00_JPRB ,&
&  0.18430E+00_JPRB, 0.18386E+00_JPRB, 0.18344E+00_JPRB, 0.18300E+00_JPRB, 0.18262E+00_JPRB, 0.18223E+00_JPRB, 0.18185E+00_JPRB ,&
&  0.18144E+00_JPRB, 0.18125E+00_JPRB, 0.18097E+00_JPRB, 0.18073E+00_JPRB, 0.18051E+00_JPRB, 0.18035E+00_JPRB, 0.18012E+00_JPRB ,&
&  0.17991E+00_JPRB, 0.17971E+00_JPRB, 0.17963E+00_JPRB, 0.17949E+00_JPRB, 0.17946E+00_JPRB, 0.17943E+00_JPRB, 0.17950E+00_JPRB ,&
&  0.17956E+00_JPRB, 0.17967E+00_JPRB, 0.17985E+00_JPRB, 0.18016E+00_JPRB, 0.18044E+00_JPRB, 0.18073E+00_JPRB, 0.18102E+00_JPRB ,&
&  0.18136E+00_JPRB, 0.18171E+00_JPRB, 0.18207E+00_JPRB, 0.18242E+00_JPRB, 0.18286E+00_JPRB, 0.18331E+00_JPRB, 0.18382E+00_JPRB ,&
&  0.18429E+00_JPRB, 0.18480E+00_JPRB, 0.18525E+00_JPRB, 0.18579E+00_JPRB, 0.18624E+00_JPRB, 0.18652E+00_JPRB, 0.18677E+00_JPRB ,&
&  0.18723E+00_JPRB, 0.18767E+00_JPRB, 0.18819E+00_JPRB, 0.18866E+00_JPRB, 0.18950E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB, 0.18321E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB ,&
&  0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB, 0.87859E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.86560E+00_JPRB, 0.86596E+00_JPRB, 0.86624E+00_JPRB, 0.86653E+00_JPRB, 0.86674E+00_JPRB, 0.86696E+00_JPRB, 0.86719E+00_JPRB ,&
&  0.86731E+00_JPRB, 0.86765E+00_JPRB, 0.86815E+00_JPRB, 0.86875E+00_JPRB, 0.86931E+00_JPRB, 0.86987E+00_JPRB, 0.87048E+00_JPRB ,&
&  0.87106E+00_JPRB, 0.87141E+00_JPRB, 0.87172E+00_JPRB, 0.87226E+00_JPRB, 0.87301E+00_JPRB, 0.87390E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.87566E+00_JPRB, 0.87527E+00_JPRB, 0.87486E+00_JPRB, 0.87541E+00_JPRB, 0.87518E+00_JPRB, 0.87506E+00_JPRB, 0.87500E+00_JPRB ,&
&  0.87488E+00_JPRB, 0.87474E+00_JPRB, 0.87445E+00_JPRB, 0.87417E+00_JPRB, 0.87424E+00_JPRB, 0.87456E+00_JPRB, 0.87494E+00_JPRB ,&
&  0.87542E+00_JPRB, 0.87586E+00_JPRB, 0.87685E+00_JPRB, 0.87787E+00_JPRB, 0.87902E+00_JPRB, 0.87988E+00_JPRB, 0.88089E+00_JPRB ,&
&  0.88156E+00_JPRB, 0.88227E+00_JPRB, 0.88340E+00_JPRB, 0.88493E+00_JPRB, 0.88654E+00_JPRB, 0.88824E+00_JPRB, 0.88950E+00_JPRB ,&
&  0.89140E+00_JPRB, 0.89383E+00_JPRB, 0.89665E+00_JPRB, 0.89962E+00_JPRB, 0.90194E+00_JPRB, 0.90428E+00_JPRB, 0.90687E+00_JPRB ,&
&  0.90915E+00_JPRB, 0.91051E+00_JPRB, 0.91207E+00_JPRB, 0.91373E+00_JPRB, 0.91495E+00_JPRB, 0.91428E+00_JPRB, 0.91379E+00_JPRB ,&
&  0.91341E+00_JPRB, 0.91266E+00_JPRB, 0.91011E+00_JPRB, 0.90800E+00_JPRB, 0.90606E+00_JPRB, 0.90399E+00_JPRB, 0.90163E+00_JPRB ,&
&  0.89944E+00_JPRB, 0.89744E+00_JPRB, 0.89516E+00_JPRB, 0.89270E+00_JPRB, 0.89053E+00_JPRB, 0.88845E+00_JPRB, 0.88638E+00_JPRB ,&
&  0.88438E+00_JPRB, 0.88268E+00_JPRB, 0.88103E+00_JPRB, 0.87948E+00_JPRB, 0.87779E+00_JPRB, 0.87618E+00_JPRB, 0.87453E+00_JPRB ,&
&  0.87301E+00_JPRB, 0.87145E+00_JPRB, 0.87001E+00_JPRB, 0.86857E+00_JPRB, 0.86756E+00_JPRB, 0.86687E+00_JPRB, 0.86628E+00_JPRB ,&
&  0.86581E+00_JPRB, 0.86555E+00_JPRB, 0.86554E+00_JPRB, 0.86546E+00_JPRB, 0.86544E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.85786E+00_JPRB, 0.85899E+00_JPRB, 0.85995E+00_JPRB, 0.86078E+00_JPRB, 0.86175E+00_JPRB, 0.86283E+00_JPRB, 0.86372E+00_JPRB ,&
&  0.86441E+00_JPRB, 0.86542E+00_JPRB, 0.86661E+00_JPRB, 0.86778E+00_JPRB, 0.86884E+00_JPRB, 0.87006E+00_JPRB, 0.87124E+00_JPRB ,&
&  0.87228E+00_JPRB, 0.87323E+00_JPRB, 0.87422E+00_JPRB, 0.87505E+00_JPRB, 0.87573E+00_JPRB, 0.87594E+00_JPRB, 0.87805E+00_JPRB ,&
&  0.87959E+00_JPRB, 0.87953E+00_JPRB, 0.87759E+00_JPRB, 0.86938E+00_JPRB, 0.86348E+00_JPRB, 0.85804E+00_JPRB, 0.85338E+00_JPRB ,&
&  0.84850E+00_JPRB, 0.84798E+00_JPRB, 0.84808E+00_JPRB, 0.84821E+00_JPRB, 0.85140E+00_JPRB, 0.85057E+00_JPRB, 0.84923E+00_JPRB ,&
&  0.84637E+00_JPRB, 0.84005E+00_JPRB, 0.84089E+00_JPRB, 0.84202E+00_JPRB, 0.84313E+00_JPRB, 0.84679E+00_JPRB, 0.84997E+00_JPRB ,&
&  0.85378E+00_JPRB, 0.85780E+00_JPRB, 0.86488E+00_JPRB, 0.87279E+00_JPRB, 0.88005E+00_JPRB, 0.88628E+00_JPRB, 0.89208E+00_JPRB ,&
&  0.89715E+00_JPRB, 0.90251E+00_JPRB, 0.90852E+00_JPRB, 0.91544E+00_JPRB, 0.92129E+00_JPRB, 0.92737E+00_JPRB, 0.93240E+00_JPRB ,&
&  0.93516E+00_JPRB, 0.93646E+00_JPRB, 0.93995E+00_JPRB, 0.94470E+00_JPRB, 0.94972E+00_JPRB, 0.95073E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.94803E+00_JPRB, 0.94581E+00_JPRB, 0.94153E+00_JPRB, 0.93763E+00_JPRB, 0.93331E+00_JPRB, 0.92869E+00_JPRB, 0.92342E+00_JPRB ,&
&  0.91832E+00_JPRB, 0.91349E+00_JPRB, 0.90934E+00_JPRB, 0.90507E+00_JPRB, 0.90113E+00_JPRB, 0.89738E+00_JPRB, 0.89485E+00_JPRB ,&
&  0.89192E+00_JPRB, 0.88942E+00_JPRB, 0.88670E+00_JPRB, 0.88404E+00_JPRB, 0.87979E+00_JPRB, 0.87545E+00_JPRB, 0.87064E+00_JPRB ,&
&  0.86663E+00_JPRB, 0.86259E+00_JPRB, 0.85924E+00_JPRB, 0.85641E+00_JPRB, 0.85508E+00_JPRB, 0.85442E+00_JPRB, 0.85415E+00_JPRB ,&
&  0.85401E+00_JPRB, 0.85452E+00_JPRB, 0.85536E+00_JPRB, 0.85610E+00_JPRB, 0.85691E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.86310E+00_JPRB, 0.86464E+00_JPRB, 0.86601E+00_JPRB, 0.86674E+00_JPRB, 0.86807E+00_JPRB, 0.86928E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.87155E+00_JPRB, 0.87214E+00_JPRB, 0.87267E+00_JPRB, 0.87335E+00_JPRB, 0.87401E+00_JPRB, 0.87562E+00_JPRB, 0.87699E+00_JPRB ,&
&  0.87801E+00_JPRB, 0.87837E+00_JPRB, 0.88024E+00_JPRB, 0.88178E+00_JPRB, 0.88212E+00_JPRB, 0.88218E+00_JPRB, 0.87531E+00_JPRB ,&
&  0.87086E+00_JPRB, 0.86739E+00_JPRB, 0.86285E+00_JPRB, 0.85424E+00_JPRB, 0.84986E+00_JPRB, 0.82734E+00_JPRB, 0.83278E+00_JPRB ,&
&  0.85717E+00_JPRB, 0.85456E+00_JPRB, 0.84832E+00_JPRB, 0.83834E+00_JPRB, 0.80437E+00_JPRB, 0.78878E+00_JPRB, 0.78536E+00_JPRB ,&
&  0.76378E+00_JPRB, 0.78221E+00_JPRB, 0.78736E+00_JPRB, 0.78967E+00_JPRB, 0.79429E+00_JPRB, 0.79143E+00_JPRB, 0.79255E+00_JPRB ,&
&  0.80722E+00_JPRB, 0.80869E+00_JPRB, 0.81771E+00_JPRB, 0.82897E+00_JPRB, 0.84140E+00_JPRB, 0.85325E+00_JPRB, 0.86618E+00_JPRB ,&
&  0.87631E+00_JPRB, 0.88354E+00_JPRB, 0.88858E+00_JPRB, 0.89067E+00_JPRB, 0.89342E+00_JPRB, 0.89701E+00_JPRB, 0.90385E+00_JPRB ,&
&  0.91479E+00_JPRB, 0.92653E+00_JPRB, 0.93292E+00_JPRB, 0.93243E+00_JPRB, 0.93565E+00_JPRB, 0.93851E+00_JPRB, 0.94157E+00_JPRB ,&
&  0.94264E+00_JPRB, 0.94213E+00_JPRB, 0.93884E+00_JPRB, 0.93838E+00_JPRB, 0.93813E+00_JPRB, 0.93598E+00_JPRB, 0.93230E+00_JPRB ,&
&  0.92776E+00_JPRB, 0.92198E+00_JPRB, 0.91717E+00_JPRB, 0.91269E+00_JPRB, 0.90931E+00_JPRB, 0.90616E+00_JPRB, 0.90549E+00_JPRB ,&
&  0.90298E+00_JPRB, 0.90106E+00_JPRB, 0.89663E+00_JPRB, 0.89053E+00_JPRB, 0.88274E+00_JPRB, 0.87553E+00_JPRB, 0.86771E+00_JPRB ,&
&  0.86251E+00_JPRB, 0.85902E+00_JPRB, 0.85695E+00_JPRB, 0.85463E+00_JPRB, 0.85422E+00_JPRB, 0.85388E+00_JPRB, 0.85456E+00_JPRB ,&
&  0.85516E+00_JPRB, 0.85674E+00_JPRB, 0.85805E+00_JPRB, 0.85996E+00_JPRB, 0.86118E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.86314E+00_JPRB, 0.86295E+00_JPRB, 0.86324E+00_JPRB, 0.86323E+00_JPRB, 0.86425E+00_JPRB, 0.86490E+00_JPRB, 0.86773E+00_JPRB ,&
&  0.87030E+00_JPRB, 0.87288E+00_JPRB, 0.87535E+00_JPRB, 0.87746E+00_JPRB, 0.87958E+00_JPRB, 0.88051E+00_JPRB, 0.88086E+00_JPRB ,&
&  0.88110E+00_JPRB, 0.88082E+00_JPRB, 0.88178E+00_JPRB, 0.88281E+00_JPRB, 0.88241E+00_JPRB, 0.88288E+00_JPRB, 0.87550E+00_JPRB ,&
&  0.86984E+00_JPRB, 0.86535E+00_JPRB, 0.84161E+00_JPRB, 0.85046E+00_JPRB, 0.84642E+00_JPRB, 0.84392E+00_JPRB, 0.83781E+00_JPRB ,&
&  0.81213E+00_JPRB, 0.80067E+00_JPRB, 0.77467E+00_JPRB, 0.75374E+00_JPRB, 0.75583E+00_JPRB, 0.74142E+00_JPRB, 0.74302E+00_JPRB ,&
&  0.73984E+00_JPRB, 0.73414E+00_JPRB, 0.73160E+00_JPRB, 0.73008E+00_JPRB, 0.73388E+00_JPRB, 0.73699E+00_JPRB, 0.74061E+00_JPRB ,&
&  0.74994E+00_JPRB, 0.75822E+00_JPRB, 0.77052E+00_JPRB, 0.77336E+00_JPRB, 0.78928E+00_JPRB, 0.80733E+00_JPRB, 0.82642E+00_JPRB ,&
&  0.83912E+00_JPRB, 0.83805E+00_JPRB, 0.84361E+00_JPRB, 0.85232E+00_JPRB, 0.85971E+00_JPRB, 0.86805E+00_JPRB, 0.88110E+00_JPRB ,&
&  0.89787E+00_JPRB, 0.91313E+00_JPRB, 0.91927E+00_JPRB, 0.91787E+00_JPRB, 0.91576E+00_JPRB, 0.91394E+00_JPRB, 0.91659E+00_JPRB ,&
&  0.91901E+00_JPRB, 0.92477E+00_JPRB, 0.92826E+00_JPRB, 0.93402E+00_JPRB, 0.93796E+00_JPRB, 0.94499E+00_JPRB, 0.94973E+00_JPRB ,&
&  0.95293E+00_JPRB, 0.95375E+00_JPRB, 0.95966E+00_JPRB, 0.95232E+00_JPRB, 0.95117E+00_JPRB, 0.94795E+00_JPRB, 0.93908E+00_JPRB ,&
&  0.92985E+00_JPRB, 0.92188E+00_JPRB, 0.91315E+00_JPRB, 0.90388E+00_JPRB, 0.89439E+00_JPRB, 0.88669E+00_JPRB, 0.88065E+00_JPRB ,&
&  0.87666E+00_JPRB, 0.87238E+00_JPRB, 0.86913E+00_JPRB, 0.86321E+00_JPRB, 0.86151E+00_JPRB, 0.85715E+00_JPRB, 0.85558E+00_JPRB ,&
&  0.85497E+00_JPRB, 0.85683E+00_JPRB, 0.85754E+00_JPRB, 0.86016E+00_JPRB, 0.86236E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.84697E+00_JPRB, 0.85085E+00_JPRB, 0.85788E+00_JPRB, 0.86371E+00_JPRB, 0.86906E+00_JPRB, 0.87335E+00_JPRB, 0.87525E+00_JPRB ,&
&  0.87684E+00_JPRB, 0.87710E+00_JPRB, 0.87760E+00_JPRB, 0.87829E+00_JPRB, 0.87990E+00_JPRB, 0.88114E+00_JPRB, 0.88296E+00_JPRB ,&
&  0.88353E+00_JPRB, 0.88379E+00_JPRB, 0.88360E+00_JPRB, 0.88318E+00_JPRB, 0.88316E+00_JPRB, 0.88453E+00_JPRB, 0.88247E+00_JPRB ,&
&  0.88195E+00_JPRB, 0.87475E+00_JPRB, 0.85731E+00_JPRB, 0.86091E+00_JPRB, 0.85991E+00_JPRB, 0.85679E+00_JPRB, 0.84470E+00_JPRB ,&
&  0.81742E+00_JPRB, 0.77423E+00_JPRB, 0.79810E+00_JPRB, 0.78284E+00_JPRB, 0.76622E+00_JPRB, 0.74913E+00_JPRB, 0.72449E+00_JPRB ,&
&  0.70927E+00_JPRB, 0.68915E+00_JPRB, 0.67384E+00_JPRB, 0.66793E+00_JPRB, 0.66671E+00_JPRB, 0.67404E+00_JPRB, 0.68449E+00_JPRB ,&
&  0.68982E+00_JPRB, 0.70500E+00_JPRB, 0.71395E+00_JPRB, 0.72625E+00_JPRB, 0.74130E+00_JPRB, 0.76090E+00_JPRB, 0.76941E+00_JPRB ,&
&  0.78401E+00_JPRB, 0.78966E+00_JPRB, 0.80468E+00_JPRB, 0.82522E+00_JPRB, 0.85110E+00_JPRB, 0.86888E+00_JPRB, 0.88145E+00_JPRB ,&
&  0.89740E+00_JPRB, 0.90697E+00_JPRB, 0.91332E+00_JPRB, 0.91689E+00_JPRB, 0.92390E+00_JPRB, 0.93058E+00_JPRB, 0.93785E+00_JPRB ,&
&  0.94389E+00_JPRB, 0.95416E+00_JPRB, 0.96521E+00_JPRB, 0.97796E+00_JPRB, 0.98586E+00_JPRB, 0.99899E+00_JPRB, 0.10137E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.10336E+01_JPRB, 0.10470E+01_JPRB, 0.10364E+01_JPRB, 0.10308E+01_JPRB, 0.10081E+01_JPRB, 0.99039E+00_JPRB ,&
&  0.97710E+00_JPRB, 0.96103E+00_JPRB, 0.94635E+00_JPRB, 0.93264E+00_JPRB, 0.92110E+00_JPRB, 0.91146E+00_JPRB, 0.90276E+00_JPRB ,&
&  0.89307E+00_JPRB, 0.88048E+00_JPRB, 0.87123E+00_JPRB, 0.86126E+00_JPRB, 0.85699E+00_JPRB, 0.85071E+00_JPRB, 0.84800E+00_JPRB ,&
&  0.84625E+00_JPRB, 0.84573E+00_JPRB, 0.84478E+00_JPRB, 0.84653E+00_JPRB, 0.84713E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.84506E+00_JPRB, 0.85104E+00_JPRB, 0.85732E+00_JPRB, 0.86207E+00_JPRB, 0.86310E+00_JPRB, 0.86455E+00_JPRB, 0.86167E+00_JPRB ,&
&  0.86027E+00_JPRB, 0.85823E+00_JPRB, 0.85676E+00_JPRB, 0.85772E+00_JPRB, 0.85966E+00_JPRB, 0.86339E+00_JPRB, 0.86749E+00_JPRB ,&
&  0.87130E+00_JPRB, 0.87585E+00_JPRB, 0.87789E+00_JPRB, 0.88038E+00_JPRB, 0.88299E+00_JPRB, 0.88833E+00_JPRB, 0.89021E+00_JPRB ,&
&  0.89254E+00_JPRB, 0.88519E+00_JPRB, 0.86946E+00_JPRB, 0.86436E+00_JPRB, 0.86989E+00_JPRB, 0.85135E+00_JPRB, 0.85007E+00_JPRB ,&
&  0.85842E+00_JPRB, 0.85916E+00_JPRB, 0.84407E+00_JPRB, 0.83428E+00_JPRB, 0.80470E+00_JPRB, 0.71797E+00_JPRB, 0.71737E+00_JPRB ,&
&  0.70032E+00_JPRB, 0.67097E+00_JPRB, 0.65527E+00_JPRB, 0.64172E+00_JPRB, 0.63561E+00_JPRB, 0.64624E+00_JPRB, 0.66320E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.68849E+00_JPRB, 0.69633E+00_JPRB, 0.70980E+00_JPRB, 0.72408E+00_JPRB, 0.74510E+00_JPRB, 0.75722E+00_JPRB ,&
&  0.77064E+00_JPRB, 0.78482E+00_JPRB, 0.80772E+00_JPRB, 0.83799E+00_JPRB, 0.86216E+00_JPRB, 0.87727E+00_JPRB, 0.87894E+00_JPRB ,&
&  0.89764E+00_JPRB, 0.89546E+00_JPRB, 0.90612E+00_JPRB, 0.91694E+00_JPRB, 0.92614E+00_JPRB, 0.93879E+00_JPRB, 0.94630E+00_JPRB ,&
&  0.96389E+00_JPRB, 0.99385E+00_JPRB, 0.10230E+01_JPRB, 0.10402E+01_JPRB, 0.10526E+01_JPRB, 0.10665E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.11042E+01_JPRB, 0.11248E+01_JPRB, 0.11392E+01_JPRB, 0.11159E+01_JPRB, 0.11032E+01_JPRB, 0.10736E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10246E+01_JPRB, 0.10070E+01_JPRB, 0.97782E+00_JPRB, 0.96050E+00_JPRB, 0.94251E+00_JPRB, 0.92901E+00_JPRB, 0.91343E+00_JPRB ,&
&  0.89834E+00_JPRB, 0.88328E+00_JPRB, 0.87309E+00_JPRB, 0.86678E+00_JPRB, 0.86237E+00_JPRB, 0.85480E+00_JPRB, 0.84970E+00_JPRB ,&
&  0.84247E+00_JPRB, 0.83811E+00_JPRB, 0.83424E+00_JPRB, 0.83550E+00_JPRB, 0.83829E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.82771E+00_JPRB, 0.83432E+00_JPRB, 0.83712E+00_JPRB, 0.84316E+00_JPRB, 0.84082E+00_JPRB, 0.84236E+00_JPRB, 0.83424E+00_JPRB ,&
&  0.82856E+00_JPRB, 0.82506E+00_JPRB, 0.82338E+00_JPRB, 0.82535E+00_JPRB, 0.82917E+00_JPRB, 0.83514E+00_JPRB, 0.84286E+00_JPRB ,&
&  0.85134E+00_JPRB, 0.85288E+00_JPRB, 0.85981E+00_JPRB, 0.87071E+00_JPRB, 0.87626E+00_JPRB, 0.88395E+00_JPRB, 0.88852E+00_JPRB ,&
&  0.88974E+00_JPRB, 0.88671E+00_JPRB, 0.88365E+00_JPRB, 0.87314E+00_JPRB, 0.86883E+00_JPRB, 0.87166E+00_JPRB, 0.87442E+00_JPRB ,&
&  0.85149E+00_JPRB, 0.84206E+00_JPRB, 0.86462E+00_JPRB, 0.87147E+00_JPRB, 0.81990E+00_JPRB, 0.77240E+00_JPRB, 0.74852E+00_JPRB ,&
&  0.71524E+00_JPRB, 0.67001E+00_JPRB, 0.65347E+00_JPRB, 0.64255E+00_JPRB, 0.64582E+00_JPRB, 0.65064E+00_JPRB, 0.66356E+00_JPRB ,&
&  0.67247E+00_JPRB, 0.68425E+00_JPRB, 0.70076E+00_JPRB, 0.72239E+00_JPRB, 0.73724E+00_JPRB, 0.75469E+00_JPRB, 0.76845E+00_JPRB ,&
&  0.78417E+00_JPRB, 0.80553E+00_JPRB, 0.83613E+00_JPRB, 0.87467E+00_JPRB, 0.90369E+00_JPRB, 0.92705E+00_JPRB, 0.93893E+00_JPRB ,&
&  0.98268E+00_JPRB, 0.97196E+00_JPRB, 0.98515E+00_JPRB, 0.99297E+00_JPRB, 0.99933E+00_JPRB, 0.10120E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10961E+01_JPRB, 0.11096E+01_JPRB, 0.11155E+01_JPRB, 0.11174E+01_JPRB, 0.11251E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.11653E+01_JPRB, 0.11996E+01_JPRB, 0.12879E+01_JPRB, 0.12408E+01_JPRB, 0.12026E+01_JPRB, 0.11610E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.10388E+01_JPRB, 0.10020E+01_JPRB, 0.96879E+00_JPRB, 0.94484E+00_JPRB, 0.92198E+00_JPRB, 0.91110E+00_JPRB ,&
&  0.90124E+00_JPRB, 0.89128E+00_JPRB, 0.88478E+00_JPRB, 0.87955E+00_JPRB, 0.86919E+00_JPRB, 0.85725E+00_JPRB, 0.84844E+00_JPRB ,&
&  0.83540E+00_JPRB, 0.82755E+00_JPRB, 0.82088E+00_JPRB, 0.82072E+00_JPRB, 0.82260E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.81112E+00_JPRB, 0.82512E+00_JPRB, 0.83467E+00_JPRB, 0.83908E+00_JPRB, 0.83231E+00_JPRB, 0.83306E+00_JPRB, 0.82451E+00_JPRB ,&
&  0.81944E+00_JPRB, 0.81965E+00_JPRB, 0.81700E+00_JPRB, 0.81775E+00_JPRB, 0.81919E+00_JPRB, 0.82044E+00_JPRB, 0.82349E+00_JPRB ,&
&  0.82885E+00_JPRB, 0.83638E+00_JPRB, 0.84397E+00_JPRB, 0.85338E+00_JPRB, 0.86445E+00_JPRB, 0.87669E+00_JPRB, 0.88670E+00_JPRB ,&
&  0.89066E+00_JPRB, 0.88877E+00_JPRB, 0.89348E+00_JPRB, 0.88926E+00_JPRB, 0.88399E+00_JPRB, 0.87654E+00_JPRB, 0.87175E+00_JPRB ,&
&  0.87979E+00_JPRB, 0.89241E+00_JPRB, 0.88149E+00_JPRB, 0.87464E+00_JPRB, 0.86850E+00_JPRB, 0.82391E+00_JPRB, 0.77953E+00_JPRB ,&
&  0.73241E+00_JPRB, 0.69037E+00_JPRB, 0.67124E+00_JPRB, 0.66213E+00_JPRB, 0.66964E+00_JPRB, 0.68643E+00_JPRB, 0.70214E+00_JPRB ,&
&  0.69855E+00_JPRB, 0.72244E+00_JPRB, 0.74755E+00_JPRB, 0.76420E+00_JPRB, 0.77762E+00_JPRB, 0.79158E+00_JPRB, 0.80609E+00_JPRB ,&
&  0.82905E+00_JPRB, 0.86690E+00_JPRB, 0.93555E+00_JPRB, 0.10165E+01_JPRB, 0.10823E+01_JPRB, 0.10992E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.12403E+01_JPRB, 0.12026E+01_JPRB, 0.11910E+01_JPRB, 0.11741E+01_JPRB, 0.11706E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12174E+01_JPRB, 0.12425E+01_JPRB, 0.12428E+01_JPRB, 0.12312E+01_JPRB, 0.12260E+01_JPRB, 0.12302E+01_JPRB, 0.12481E+01_JPRB ,&
&  0.12624E+01_JPRB, 0.13202E+01_JPRB, 0.14676E+01_JPRB, 0.12983E+01_JPRB, 0.12115E+01_JPRB, 0.11674E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.10831E+01_JPRB, 0.10497E+01_JPRB, 0.10149E+01_JPRB, 0.97435E+00_JPRB, 0.93961E+00_JPRB, 0.90583E+00_JPRB, 0.89089E+00_JPRB ,&
&  0.88391E+00_JPRB, 0.87770E+00_JPRB, 0.87708E+00_JPRB, 0.87125E+00_JPRB, 0.86123E+00_JPRB, 0.85412E+00_JPRB, 0.84445E+00_JPRB ,&
&  0.82863E+00_JPRB, 0.81385E+00_JPRB, 0.80382E+00_JPRB, 0.80179E+00_JPRB, 0.80222E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.82351E+00_JPRB, 0.83258E+00_JPRB, 0.84233E+00_JPRB, 0.86211E+00_JPRB, 0.86604E+00_JPRB, 0.85928E+00_JPRB, 0.83827E+00_JPRB ,&
&  0.82400E+00_JPRB, 0.81865E+00_JPRB, 0.82159E+00_JPRB, 0.82583E+00_JPRB, 0.82824E+00_JPRB, 0.82887E+00_JPRB, 0.82735E+00_JPRB ,&
&  0.82445E+00_JPRB, 0.82570E+00_JPRB, 0.83148E+00_JPRB, 0.84010E+00_JPRB, 0.85138E+00_JPRB, 0.86371E+00_JPRB, 0.87741E+00_JPRB ,&
&  0.88835E+00_JPRB, 0.89169E+00_JPRB, 0.89430E+00_JPRB, 0.89327E+00_JPRB, 0.89592E+00_JPRB, 0.90168E+00_JPRB, 0.89903E+00_JPRB ,&
&  0.88769E+00_JPRB, 0.89650E+00_JPRB, 0.90832E+00_JPRB, 0.89278E+00_JPRB, 0.88367E+00_JPRB, 0.85029E+00_JPRB, 0.80615E+00_JPRB ,&
&  0.76724E+00_JPRB, 0.72835E+00_JPRB, 0.71738E+00_JPRB, 0.71959E+00_JPRB, 0.73001E+00_JPRB, 0.73222E+00_JPRB, 0.75707E+00_JPRB ,&
&  0.77750E+00_JPRB, 0.79283E+00_JPRB, 0.80517E+00_JPRB, 0.81284E+00_JPRB, 0.82187E+00_JPRB, 0.84311E+00_JPRB, 0.88984E+00_JPRB ,&
&  0.97490E+00_JPRB, 0.10849E+01_JPRB, 0.12108E+01_JPRB, 0.11805E+01_JPRB, 0.12024E+01_JPRB, 0.12557E+01_JPRB, 0.13470E+01_JPRB ,&
&  0.14068E+01_JPRB, 0.13977E+01_JPRB, 0.14047E+01_JPRB, 0.14710E+01_JPRB, 0.15143E+01_JPRB, 0.15538E+01_JPRB, 0.15374E+01_JPRB ,&
&  0.15011E+01_JPRB, 0.14541E+01_JPRB, 0.14162E+01_JPRB, 0.14051E+01_JPRB, 0.14185E+01_JPRB, 0.14258E+01_JPRB, 0.14066E+01_JPRB ,&
&  0.13549E+01_JPRB, 0.12729E+01_JPRB, 0.11837E+01_JPRB, 0.11053E+01_JPRB, 0.10843E+01_JPRB, 0.10871E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.10401E+01_JPRB, 0.10088E+01_JPRB, 0.97108E+00_JPRB, 0.92746E+00_JPRB, 0.88264E+00_JPRB, 0.86251E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85016E+00_JPRB, 0.85413E+00_JPRB, 0.83898E+00_JPRB, 0.83464E+00_JPRB, 0.83142E+00_JPRB, 0.81305E+00_JPRB ,&
&  0.79863E+00_JPRB, 0.79182E+00_JPRB, 0.78224E+00_JPRB, 0.78693E+00_JPRB, 0.79964E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.85316E+00_JPRB, 0.88284E+00_JPRB, 0.91082E+00_JPRB, 0.92088E+00_JPRB, 0.92569E+00_JPRB, 0.92281E+00_JPRB, 0.90115E+00_JPRB ,&
&  0.89088E+00_JPRB, 0.86204E+00_JPRB, 0.83915E+00_JPRB, 0.81756E+00_JPRB, 0.80253E+00_JPRB, 0.79531E+00_JPRB, 0.79969E+00_JPRB ,&
&  0.80820E+00_JPRB, 0.81659E+00_JPRB, 0.82413E+00_JPRB, 0.83040E+00_JPRB, 0.83870E+00_JPRB, 0.85028E+00_JPRB, 0.86537E+00_JPRB ,&
&  0.88132E+00_JPRB, 0.89268E+00_JPRB, 0.90218E+00_JPRB, 0.90899E+00_JPRB, 0.91416E+00_JPRB, 0.91539E+00_JPRB, 0.90752E+00_JPRB ,&
&  0.90809E+00_JPRB, 0.90691E+00_JPRB, 0.91203E+00_JPRB, 0.90426E+00_JPRB, 0.89731E+00_JPRB, 0.87081E+00_JPRB, 0.82102E+00_JPRB ,&
&  0.79513E+00_JPRB, 0.78009E+00_JPRB, 0.78559E+00_JPRB, 0.78962E+00_JPRB, 0.80459E+00_JPRB, 0.82292E+00_JPRB, 0.83210E+00_JPRB ,&
&  0.83370E+00_JPRB, 0.83644E+00_JPRB, 0.85574E+00_JPRB, 0.87957E+00_JPRB, 0.90050E+00_JPRB, 0.93851E+00_JPRB, 0.98743E+00_JPRB ,&
&  0.10632E+01_JPRB, 0.11671E+01_JPRB, 0.13225E+01_JPRB, 0.13558E+01_JPRB, 0.14574E+01_JPRB, 0.17212E+01_JPRB, 0.16640E+01_JPRB ,&
&  0.16867E+01_JPRB, 0.17264E+01_JPRB, 0.18141E+01_JPRB, 0.19434E+01_JPRB, 0.19248E+01_JPRB, 0.18987E+01_JPRB, 0.18623E+01_JPRB ,&
&  0.17626E+01_JPRB, 0.17166E+01_JPRB, 0.16715E+01_JPRB, 0.16915E+01_JPRB, 0.16611E+01_JPRB, 0.16236E+01_JPRB, 0.15806E+01_JPRB ,&
&  0.14885E+01_JPRB, 0.13671E+01_JPRB, 0.12560E+01_JPRB, 0.11598E+01_JPRB, 0.11128E+01_JPRB, 0.10875E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.10368E+01_JPRB, 0.10203E+01_JPRB, 0.99586E+00_JPRB, 0.94697E+00_JPRB, 0.89142E+00_JPRB, 0.87196E+00_JPRB ,&
&  0.85827E+00_JPRB, 0.84712E+00_JPRB, 0.83833E+00_JPRB, 0.82939E+00_JPRB, 0.82721E+00_JPRB, 0.81670E+00_JPRB, 0.80136E+00_JPRB ,&
&  0.80145E+00_JPRB, 0.79375E+00_JPRB, 0.78652E+00_JPRB, 0.79282E+00_JPRB, 0.82325E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.91127E+00_JPRB, 0.94883E+00_JPRB, 0.96420E+00_JPRB, 0.97148E+00_JPRB, 0.97936E+00_JPRB, 0.97623E+00_JPRB, 0.93514E+00_JPRB ,&
&  0.91714E+00_JPRB, 0.85857E+00_JPRB, 0.82678E+00_JPRB, 0.80740E+00_JPRB, 0.79906E+00_JPRB, 0.78959E+00_JPRB, 0.77986E+00_JPRB ,&
&  0.78547E+00_JPRB, 0.80287E+00_JPRB, 0.82176E+00_JPRB, 0.83376E+00_JPRB, 0.84423E+00_JPRB, 0.85276E+00_JPRB, 0.86209E+00_JPRB ,&
&  0.87555E+00_JPRB, 0.89079E+00_JPRB, 0.90798E+00_JPRB, 0.91718E+00_JPRB, 0.92212E+00_JPRB, 0.92456E+00_JPRB, 0.92244E+00_JPRB ,&
&  0.91966E+00_JPRB, 0.90863E+00_JPRB, 0.91646E+00_JPRB, 0.91533E+00_JPRB, 0.90281E+00_JPRB, 0.89469E+00_JPRB, 0.86216E+00_JPRB ,&
&  0.82827E+00_JPRB, 0.84508E+00_JPRB, 0.85210E+00_JPRB, 0.86847E+00_JPRB, 0.87900E+00_JPRB, 0.88595E+00_JPRB, 0.89001E+00_JPRB ,&
&  0.89275E+00_JPRB, 0.89697E+00_JPRB, 0.90932E+00_JPRB, 0.92589E+00_JPRB, 0.94789E+00_JPRB, 0.97704E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.10129E+01_JPRB, 0.11134E+01_JPRB, 0.13474E+01_JPRB, 0.18108E+01_JPRB, 0.19247E+01_JPRB, 0.20116E+01_JPRB ,&
&  0.21324E+01_JPRB, 0.21431E+01_JPRB, 0.22000E+01_JPRB, 0.22409E+01_JPRB, 0.22268E+01_JPRB, 0.21948E+01_JPRB, 0.21857E+01_JPRB ,&
&  0.20958E+01_JPRB, 0.20921E+01_JPRB, 0.20344E+01_JPRB, 0.20030E+01_JPRB, 0.19271E+01_JPRB, 0.18687E+01_JPRB, 0.17853E+01_JPRB ,&
&  0.16776E+01_JPRB, 0.15696E+01_JPRB, 0.14391E+01_JPRB, 0.13078E+01_JPRB, 0.12060E+01_JPRB, 0.11573E+01_JPRB, 0.11149E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.10706E+01_JPRB, 0.10481E+01_JPRB, 0.10319E+01_JPRB, 0.99745E+00_JPRB, 0.96010E+00_JPRB, 0.95498E+00_JPRB ,&
&  0.91960E+00_JPRB, 0.89516E+00_JPRB, 0.87244E+00_JPRB, 0.86175E+00_JPRB, 0.86727E+00_JPRB, 0.86669E+00_JPRB, 0.86209E+00_JPRB ,&
&  0.87316E+00_JPRB, 0.86366E+00_JPRB, 0.86062E+00_JPRB, 0.86965E+00_JPRB, 0.88495E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.96661E+00_JPRB, 0.99468E+00_JPRB, 0.10062E+01_JPRB, 0.10186E+01_JPRB, 0.10355E+01_JPRB, 0.10295E+01_JPRB, 0.10137E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.97848E+00_JPRB, 0.10087E+01_JPRB, 0.87792E+00_JPRB, 0.82630E+00_JPRB, 0.81553E+00_JPRB, 0.80259E+00_JPRB ,&
&  0.80599E+00_JPRB, 0.81990E+00_JPRB, 0.83477E+00_JPRB, 0.84835E+00_JPRB, 0.86510E+00_JPRB, 0.86995E+00_JPRB, 0.87404E+00_JPRB ,&
&  0.88308E+00_JPRB, 0.89914E+00_JPRB, 0.92829E+00_JPRB, 0.94457E+00_JPRB, 0.96504E+00_JPRB, 0.98277E+00_JPRB, 0.97808E+00_JPRB ,&
&  0.95713E+00_JPRB, 0.95655E+00_JPRB, 0.95738E+00_JPRB, 0.93630E+00_JPRB, 0.91516E+00_JPRB, 0.90753E+00_JPRB, 0.92538E+00_JPRB ,&
&  0.92715E+00_JPRB, 0.91047E+00_JPRB, 0.93113E+00_JPRB, 0.93809E+00_JPRB, 0.93745E+00_JPRB, 0.93323E+00_JPRB, 0.92929E+00_JPRB ,&
&  0.92768E+00_JPRB, 0.92505E+00_JPRB, 0.93083E+00_JPRB, 0.94118E+00_JPRB, 0.96276E+00_JPRB, 0.99744E+00_JPRB, 0.10176E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10688E+01_JPRB, 0.12845E+01_JPRB, 0.15252E+01_JPRB, 0.19074E+01_JPRB, 0.21451E+01_JPRB, 0.23197E+01_JPRB ,&
&  0.26044E+01_JPRB, 0.25439E+01_JPRB, 0.26452E+01_JPRB, 0.26144E+01_JPRB, 0.25679E+01_JPRB, 0.25048E+01_JPRB, 0.24257E+01_JPRB ,&
&  0.23847E+01_JPRB, 0.23084E+01_JPRB, 0.23885E+01_JPRB, 0.23147E+01_JPRB, 0.21774E+01_JPRB, 0.20909E+01_JPRB, 0.20074E+01_JPRB ,&
&  0.19342E+01_JPRB, 0.18567E+01_JPRB, 0.17469E+01_JPRB, 0.15072E+01_JPRB, 0.13958E+01_JPRB, 0.13119E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.10863E+01_JPRB, 0.10388E+01_JPRB, 0.10322E+01_JPRB, 0.10025E+01_JPRB, 0.97652E+00_JPRB, 0.97559E+00_JPRB ,&
&  0.95308E+00_JPRB, 0.93313E+00_JPRB, 0.92680E+00_JPRB, 0.93409E+00_JPRB, 0.94067E+00_JPRB, 0.93910E+00_JPRB, 0.94328E+00_JPRB ,&
&  0.92507E+00_JPRB, 0.91106E+00_JPRB, 0.92348E+00_JPRB, 0.92502E+00_JPRB, 0.94093E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.10014E+01_JPRB, 0.10314E+01_JPRB, 0.10535E+01_JPRB, 0.10732E+01_JPRB, 0.10972E+01_JPRB, 0.11167E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.10961E+01_JPRB, 0.10929E+01_JPRB, 0.10727E+01_JPRB, 0.10357E+01_JPRB, 0.94614E+00_JPRB, 0.83809E+00_JPRB ,&
&  0.81247E+00_JPRB, 0.83329E+00_JPRB, 0.85997E+00_JPRB, 0.87343E+00_JPRB, 0.87930E+00_JPRB, 0.88728E+00_JPRB, 0.88833E+00_JPRB ,&
&  0.89809E+00_JPRB, 0.91430E+00_JPRB, 0.94734E+00_JPRB, 0.99997E+00_JPRB, 0.10513E+01_JPRB, 0.10572E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.10215E+01_JPRB, 0.99217E+00_JPRB, 0.95476E+00_JPRB, 0.95574E+00_JPRB, 0.96259E+00_JPRB, 0.99405E+00_JPRB ,&
&  0.10256E+01_JPRB, 0.10346E+01_JPRB, 0.10293E+01_JPRB, 0.10241E+01_JPRB, 0.10159E+01_JPRB, 0.10014E+01_JPRB, 0.98395E+00_JPRB ,&
&  0.96914E+00_JPRB, 0.95873E+00_JPRB, 0.95326E+00_JPRB, 0.95825E+00_JPRB, 0.96306E+00_JPRB, 0.10428E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.11870E+01_JPRB, 0.13492E+01_JPRB, 0.16168E+01_JPRB, 0.17262E+01_JPRB, 0.20600E+01_JPRB, 0.23003E+01_JPRB, 0.25569E+01_JPRB ,&
&  0.27519E+01_JPRB, 0.26975E+01_JPRB, 0.29162E+01_JPRB, 0.29693E+01_JPRB, 0.29760E+01_JPRB, 0.28912E+01_JPRB, 0.27209E+01_JPRB ,&
&  0.26224E+01_JPRB, 0.26707E+01_JPRB, 0.27179E+01_JPRB, 0.26605E+01_JPRB, 0.25184E+01_JPRB, 0.24304E+01_JPRB, 0.23396E+01_JPRB ,&
&  0.23071E+01_JPRB, 0.22782E+01_JPRB, 0.21631E+01_JPRB, 0.19749E+01_JPRB, 0.17415E+01_JPRB, 0.15328E+01_JPRB, 0.13617E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.10838E+01_JPRB, 0.10300E+01_JPRB, 0.10173E+01_JPRB, 0.10136E+01_JPRB, 0.98147E+00_JPRB, 0.97881E+00_JPRB ,&
&  0.98920E+00_JPRB, 0.98768E+00_JPRB, 0.99544E+00_JPRB, 0.10064E+01_JPRB, 0.10429E+01_JPRB, 0.10550E+01_JPRB, 0.99013E+00_JPRB ,&
&  0.94569E+00_JPRB, 0.94100E+00_JPRB, 0.94183E+00_JPRB, 0.94668E+00_JPRB, 0.97035E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.10535E+01_JPRB, 0.10901E+01_JPRB, 0.11147E+01_JPRB, 0.11316E+01_JPRB, 0.11540E+01_JPRB, 0.11879E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.11621E+01_JPRB, 0.11557E+01_JPRB, 0.11421E+01_JPRB, 0.11130E+01_JPRB, 0.10736E+01_JPRB, 0.98987E+00_JPRB ,&
&  0.88670E+00_JPRB, 0.93059E+00_JPRB, 0.90498E+00_JPRB, 0.92588E+00_JPRB, 0.93253E+00_JPRB, 0.91670E+00_JPRB, 0.89971E+00_JPRB ,&
&  0.91770E+00_JPRB, 0.93664E+00_JPRB, 0.98259E+00_JPRB, 0.10356E+01_JPRB, 0.10233E+01_JPRB, 0.10269E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.10511E+01_JPRB, 0.10122E+01_JPRB, 0.10041E+01_JPRB, 0.10312E+01_JPRB, 0.10795E+01_JPRB, 0.11254E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.11247E+01_JPRB, 0.11046E+01_JPRB, 0.10872E+01_JPRB, 0.10719E+01_JPRB, 0.10518E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.98377E+00_JPRB, 0.97656E+00_JPRB, 0.98758E+00_JPRB, 0.10283E+01_JPRB, 0.11750E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.14443E+01_JPRB, 0.16173E+01_JPRB, 0.18560E+01_JPRB, 0.21312E+01_JPRB, 0.24165E+01_JPRB, 0.26597E+01_JPRB, 0.28501E+01_JPRB ,&
&  0.30588E+01_JPRB, 0.30546E+01_JPRB, 0.33849E+01_JPRB, 0.33706E+01_JPRB, 0.32326E+01_JPRB, 0.31755E+01_JPRB, 0.29732E+01_JPRB ,&
&  0.27905E+01_JPRB, 0.26179E+01_JPRB, 0.27339E+01_JPRB, 0.27525E+01_JPRB, 0.27318E+01_JPRB, 0.27321E+01_JPRB, 0.26744E+01_JPRB ,&
&  0.26311E+01_JPRB, 0.25648E+01_JPRB, 0.22383E+01_JPRB, 0.19230E+01_JPRB, 0.15766E+01_JPRB, 0.15451E+01_JPRB, 0.14016E+01_JPRB ,&
&  0.12371E+01_JPRB, 0.10634E+01_JPRB, 0.10508E+01_JPRB, 0.10390E+01_JPRB, 0.10575E+01_JPRB, 0.10488E+01_JPRB, 0.10513E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.11042E+01_JPRB, 0.11076E+01_JPRB, 0.11406E+01_JPRB, 0.11884E+01_JPRB, 0.11811E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.10160E+01_JPRB, 0.98166E+00_JPRB, 0.98038E+00_JPRB, 0.10081E+01_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.11270E+01_JPRB, 0.11558E+01_JPRB, 0.11749E+01_JPRB, 0.11932E+01_JPRB, 0.12190E+01_JPRB, 0.12479E+01_JPRB, 0.12435E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.12133E+01_JPRB, 0.12025E+01_JPRB, 0.11833E+01_JPRB, 0.11586E+01_JPRB, 0.11160E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.96791E+00_JPRB, 0.10005E+01_JPRB, 0.91289E+00_JPRB, 0.93143E+00_JPRB, 0.10694E+01_JPRB, 0.10081E+01_JPRB, 0.98472E+00_JPRB ,&
&  0.95980E+00_JPRB, 0.98393E+00_JPRB, 0.10206E+01_JPRB, 0.10399E+01_JPRB, 0.99644E+00_JPRB, 0.10277E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.11273E+01_JPRB, 0.11189E+01_JPRB, 0.11292E+01_JPRB, 0.11379E+01_JPRB, 0.11649E+01_JPRB, 0.12320E+01_JPRB, 0.12717E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.12194E+01_JPRB, 0.11844E+01_JPRB, 0.11562E+01_JPRB, 0.11301E+01_JPRB, 0.11021E+01_JPRB, 0.10720E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10222E+01_JPRB, 0.10189E+01_JPRB, 0.10500E+01_JPRB, 0.10891E+01_JPRB, 0.12836E+01_JPRB, 0.16637E+01_JPRB ,&
&  0.20422E+01_JPRB, 0.21981E+01_JPRB, 0.24881E+01_JPRB, 0.29480E+01_JPRB, 0.31791E+01_JPRB, 0.33087E+01_JPRB, 0.33490E+01_JPRB ,&
&  0.34048E+01_JPRB, 0.34073E+01_JPRB, 0.33569E+01_JPRB, 0.33754E+01_JPRB, 0.32518E+01_JPRB, 0.32702E+01_JPRB, 0.31043E+01_JPRB ,&
&  0.28111E+01_JPRB, 0.27282E+01_JPRB, 0.27947E+01_JPRB, 0.27849E+01_JPRB, 0.27375E+01_JPRB, 0.27658E+01_JPRB, 0.27777E+01_JPRB ,&
&  0.26859E+01_JPRB, 0.23711E+01_JPRB, 0.18186E+01_JPRB, 0.16451E+01_JPRB, 0.12703E+01_JPRB, 0.12588E+01_JPRB, 0.14261E+01_JPRB ,&
&  0.14202E+01_JPRB, 0.12824E+01_JPRB, 0.11740E+01_JPRB, 0.11253E+01_JPRB, 0.11348E+01_JPRB, 0.13068E+01_JPRB, 0.13368E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.12627E+01_JPRB, 0.13156E+01_JPRB, 0.13837E+01_JPRB, 0.14235E+01_JPRB, 0.14045E+01_JPRB, 0.13034E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.11783E+01_JPRB, 0.11091E+01_JPRB, 0.10844E+01_JPRB, 0.10957E+01_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.12617E+01_JPRB, 0.12612E+01_JPRB, 0.12601E+01_JPRB, 0.12631E+01_JPRB, 0.12883E+01_JPRB, 0.13048E+01_JPRB, 0.12891E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.12565E+01_JPRB, 0.12397E+01_JPRB, 0.12170E+01_JPRB, 0.11770E+01_JPRB, 0.11183E+01_JPRB, 0.10900E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11617E+01_JPRB, 0.10567E+01_JPRB, 0.95375E+00_JPRB, 0.10386E+01_JPRB, 0.10131E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.10800E+01_JPRB, 0.11065E+01_JPRB, 0.10733E+01_JPRB, 0.10377E+01_JPRB, 0.11283E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.13541E+01_JPRB, 0.14130E+01_JPRB, 0.14141E+01_JPRB, 0.14226E+01_JPRB, 0.14698E+01_JPRB, 0.13762E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.13419E+01_JPRB, 0.12931E+01_JPRB, 0.12441E+01_JPRB, 0.12130E+01_JPRB, 0.11842E+01_JPRB, 0.11651E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11397E+01_JPRB, 0.11075E+01_JPRB, 0.11087E+01_JPRB, 0.11412E+01_JPRB, 0.13088E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.19549E+01_JPRB, 0.21480E+01_JPRB, 0.24057E+01_JPRB, 0.28793E+01_JPRB, 0.31389E+01_JPRB, 0.32517E+01_JPRB, 0.33878E+01_JPRB ,&
&  0.34556E+01_JPRB, 0.34685E+01_JPRB, 0.34634E+01_JPRB, 0.34228E+01_JPRB, 0.34289E+01_JPRB, 0.33971E+01_JPRB, 0.32754E+01_JPRB ,&
&  0.30107E+01_JPRB, 0.28717E+01_JPRB, 0.28738E+01_JPRB, 0.27159E+01_JPRB, 0.26049E+01_JPRB, 0.24379E+01_JPRB, 0.25426E+01_JPRB ,&
&  0.25332E+01_JPRB, 0.16336E+01_JPRB, 0.13385E+01_JPRB, 0.15182E+01_JPRB, 0.12900E+01_JPRB, 0.12559E+01_JPRB, 0.13644E+01_JPRB ,&
&  0.13312E+01_JPRB, 0.12813E+01_JPRB, 0.12771E+01_JPRB, 0.12571E+01_JPRB, 0.13084E+01_JPRB, 0.16717E+01_JPRB, 0.17143E+01_JPRB ,&
&  0.16529E+01_JPRB, 0.16288E+01_JPRB, 0.17214E+01_JPRB, 0.17135E+01_JPRB, 0.16750E+01_JPRB, 0.16624E+01_JPRB, 0.15880E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.13732E+01_JPRB, 0.12992E+01_JPRB, 0.12670E+01_JPRB, 0.12620E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.14383E+01_JPRB, 0.13939E+01_JPRB, 0.13559E+01_JPRB, 0.13462E+01_JPRB, 0.13638E+01_JPRB, 0.13697E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.13033E+01_JPRB, 0.12790E+01_JPRB, 0.12409E+01_JPRB, 0.11823E+01_JPRB, 0.11202E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.11700E+01_JPRB, 0.10303E+01_JPRB, 0.98716E+00_JPRB, 0.93219E+00_JPRB, 0.97794E+00_JPRB, 0.10344E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.11206E+01_JPRB, 0.11587E+01_JPRB, 0.11606E+01_JPRB, 0.11707E+01_JPRB, 0.13249E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.16974E+01_JPRB, 0.17792E+01_JPRB, 0.16716E+01_JPRB, 0.17451E+01_JPRB, 0.16517E+01_JPRB, 0.15073E+01_JPRB, 0.15193E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.13671E+01_JPRB, 0.12969E+01_JPRB, 0.12688E+01_JPRB, 0.12534E+01_JPRB, 0.12226E+01_JPRB, 0.11932E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11330E+01_JPRB, 0.11241E+01_JPRB, 0.11161E+01_JPRB, 0.11400E+01_JPRB, 0.12660E+01_JPRB, 0.13946E+01_JPRB ,&
&  0.16243E+01_JPRB, 0.15729E+01_JPRB, 0.15426E+01_JPRB, 0.20618E+01_JPRB, 0.28158E+01_JPRB, 0.30424E+01_JPRB, 0.29716E+01_JPRB ,&
&  0.32160E+01_JPRB, 0.33450E+01_JPRB, 0.34705E+01_JPRB, 0.35430E+01_JPRB, 0.35815E+01_JPRB, 0.34730E+01_JPRB, 0.32913E+01_JPRB ,&
&  0.30863E+01_JPRB, 0.29584E+01_JPRB, 0.28395E+01_JPRB, 0.27368E+01_JPRB, 0.26014E+01_JPRB, 0.25910E+01_JPRB, 0.26884E+01_JPRB ,&
&  0.25156E+01_JPRB, 0.22417E+01_JPRB, 0.16132E+01_JPRB, 0.16165E+01_JPRB, 0.13027E+01_JPRB, 0.12587E+01_JPRB, 0.13774E+01_JPRB ,&
&  0.13369E+01_JPRB, 0.13806E+01_JPRB, 0.13898E+01_JPRB, 0.13809E+01_JPRB, 0.16314E+01_JPRB, 0.20348E+01_JPRB, 0.21416E+01_JPRB ,&
&  0.22237E+01_JPRB, 0.20765E+01_JPRB, 0.22420E+01_JPRB, 0.22244E+01_JPRB, 0.21617E+01_JPRB, 0.20976E+01_JPRB, 0.19987E+01_JPRB ,&
&  0.18487E+01_JPRB, 0.16889E+01_JPRB, 0.15690E+01_JPRB, 0.15066E+01_JPRB, 0.14670E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.15359E+01_JPRB, 0.14604E+01_JPRB, 0.14090E+01_JPRB, 0.13918E+01_JPRB, 0.14033E+01_JPRB, 0.14038E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.13486E+01_JPRB, 0.13177E+01_JPRB, 0.12900E+01_JPRB, 0.12301E+01_JPRB, 0.11767E+01_JPRB, 0.11122E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10791E+01_JPRB, 0.10752E+01_JPRB, 0.95962E+00_JPRB, 0.91289E+00_JPRB, 0.87029E+00_JPRB, 0.90485E+00_JPRB, 0.98610E+00_JPRB ,&
&  0.10784E+01_JPRB, 0.11479E+01_JPRB, 0.12990E+01_JPRB, 0.13475E+01_JPRB, 0.14187E+01_JPRB, 0.16365E+01_JPRB, 0.19286E+01_JPRB ,&
&  0.22966E+01_JPRB, 0.19687E+01_JPRB, 0.19694E+01_JPRB, 0.18671E+01_JPRB, 0.17727E+01_JPRB, 0.16501E+01_JPRB, 0.15278E+01_JPRB ,&
&  0.14240E+01_JPRB, 0.13415E+01_JPRB, 0.13096E+01_JPRB, 0.12804E+01_JPRB, 0.12514E+01_JPRB, 0.12276E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.12029E+01_JPRB, 0.11822E+01_JPRB, 0.11470E+01_JPRB, 0.11111E+01_JPRB, 0.12101E+01_JPRB, 0.13455E+01_JPRB, 0.13867E+01_JPRB ,&
&  0.13371E+01_JPRB, 0.14730E+01_JPRB, 0.19699E+01_JPRB, 0.22994E+01_JPRB, 0.26097E+01_JPRB, 0.29463E+01_JPRB, 0.31915E+01_JPRB ,&
&  0.33856E+01_JPRB, 0.34459E+01_JPRB, 0.35027E+01_JPRB, 0.31302E+01_JPRB, 0.34438E+01_JPRB, 0.33824E+01_JPRB, 0.31731E+01_JPRB ,&
&  0.29307E+01_JPRB, 0.28462E+01_JPRB, 0.26536E+01_JPRB, 0.26412E+01_JPRB, 0.27396E+01_JPRB, 0.29514E+01_JPRB, 0.26273E+01_JPRB ,&
&  0.22721E+01_JPRB, 0.24778E+01_JPRB, 0.21798E+01_JPRB, 0.17991E+01_JPRB, 0.14912E+01_JPRB, 0.14746E+01_JPRB, 0.14837E+01_JPRB ,&
&  0.14697E+01_JPRB, 0.14705E+01_JPRB, 0.14679E+01_JPRB, 0.16414E+01_JPRB, 0.21420E+01_JPRB, 0.25326E+01_JPRB, 0.25319E+01_JPRB ,&
&  0.24603E+01_JPRB, 0.26682E+01_JPRB, 0.26824E+01_JPRB, 0.26317E+01_JPRB, 0.25879E+01_JPRB, 0.25047E+01_JPRB, 0.23603E+01_JPRB ,&
&  0.21722E+01_JPRB, 0.19962E+01_JPRB, 0.18542E+01_JPRB, 0.17282E+01_JPRB, 0.16224E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.15733E+01_JPRB, 0.14925E+01_JPRB, 0.14482E+01_JPRB, 0.14401E+01_JPRB, 0.14565E+01_JPRB, 0.14431E+01_JPRB, 0.14070E+01_JPRB ,&
&  0.13587E+01_JPRB, 0.13067E+01_JPRB, 0.12461E+01_JPRB, 0.11921E+01_JPRB, 0.11555E+01_JPRB, 0.11020E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10249E+01_JPRB, 0.94987E+00_JPRB, 0.98905E+00_JPRB, 0.99874E+00_JPRB, 0.93787E+00_JPRB, 0.98928E+00_JPRB ,&
&  0.10730E+01_JPRB, 0.12216E+01_JPRB, 0.14408E+01_JPRB, 0.16794E+01_JPRB, 0.18745E+01_JPRB, 0.21624E+01_JPRB, 0.24103E+01_JPRB ,&
&  0.23669E+01_JPRB, 0.22929E+01_JPRB, 0.21678E+01_JPRB, 0.20105E+01_JPRB, 0.17879E+01_JPRB, 0.16139E+01_JPRB, 0.14956E+01_JPRB ,&
&  0.14002E+01_JPRB, 0.13471E+01_JPRB, 0.13207E+01_JPRB, 0.12963E+01_JPRB, 0.12877E+01_JPRB, 0.12819E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.12410E+01_JPRB, 0.11935E+01_JPRB, 0.11382E+01_JPRB, 0.11530E+01_JPRB, 0.12286E+01_JPRB, 0.12969E+01_JPRB, 0.13128E+01_JPRB ,&
&  0.15109E+01_JPRB, 0.17558E+01_JPRB, 0.18761E+01_JPRB, 0.24194E+01_JPRB, 0.22923E+01_JPRB, 0.24643E+01_JPRB, 0.30234E+01_JPRB ,&
&  0.32896E+01_JPRB, 0.33749E+01_JPRB, 0.33869E+01_JPRB, 0.27780E+01_JPRB, 0.24314E+01_JPRB, 0.31885E+01_JPRB, 0.30779E+01_JPRB ,&
&  0.28172E+01_JPRB, 0.27160E+01_JPRB, 0.26329E+01_JPRB, 0.29322E+01_JPRB, 0.24828E+01_JPRB, 0.22623E+01_JPRB, 0.18488E+01_JPRB ,&
&  0.19209E+01_JPRB, 0.17167E+01_JPRB, 0.19482E+01_JPRB, 0.17653E+01_JPRB, 0.17474E+01_JPRB, 0.16670E+01_JPRB, 0.15596E+01_JPRB ,&
&  0.15434E+01_JPRB, 0.15681E+01_JPRB, 0.16867E+01_JPRB, 0.22335E+01_JPRB, 0.30257E+01_JPRB, 0.29268E+01_JPRB, 0.30805E+01_JPRB ,&
&  0.31914E+01_JPRB, 0.31961E+01_JPRB, 0.29616E+01_JPRB, 0.29419E+01_JPRB, 0.28331E+01_JPRB, 0.26953E+01_JPRB, 0.25339E+01_JPRB ,&
&  0.23620E+01_JPRB, 0.21954E+01_JPRB, 0.20191E+01_JPRB, 0.18434E+01_JPRB, 0.16901E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.16275E+01_JPRB, 0.15633E+01_JPRB, 0.15371E+01_JPRB, 0.15323E+01_JPRB, 0.15184E+01_JPRB, 0.14770E+01_JPRB, 0.14204E+01_JPRB ,&
&  0.13513E+01_JPRB, 0.12676E+01_JPRB, 0.11955E+01_JPRB, 0.11453E+01_JPRB, 0.11230E+01_JPRB, 0.10975E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.11076E+01_JPRB, 0.10330E+01_JPRB, 0.10078E+01_JPRB, 0.11245E+01_JPRB, 0.10468E+01_JPRB, 0.99537E+00_JPRB ,&
&  0.11331E+01_JPRB, 0.13252E+01_JPRB, 0.16304E+01_JPRB, 0.20203E+01_JPRB, 0.23225E+01_JPRB, 0.26121E+01_JPRB, 0.27453E+01_JPRB ,&
&  0.27352E+01_JPRB, 0.24639E+01_JPRB, 0.21862E+01_JPRB, 0.19335E+01_JPRB, 0.16779E+01_JPRB, 0.15044E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.13133E+01_JPRB, 0.12911E+01_JPRB, 0.12719E+01_JPRB, 0.12344E+01_JPRB, 0.12307E+01_JPRB, 0.11872E+01_JPRB, 0.11568E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.11508E+01_JPRB, 0.11360E+01_JPRB, 0.12033E+01_JPRB, 0.12258E+01_JPRB, 0.12802E+01_JPRB, 0.13503E+01_JPRB ,&
&  0.15027E+01_JPRB, 0.16953E+01_JPRB, 0.18311E+01_JPRB, 0.22105E+01_JPRB, 0.27566E+01_JPRB, 0.28606E+01_JPRB, 0.31652E+01_JPRB ,&
&  0.27647E+01_JPRB, 0.23818E+01_JPRB, 0.20756E+01_JPRB, 0.17686E+01_JPRB, 0.20693E+01_JPRB, 0.30076E+01_JPRB, 0.31483E+01_JPRB ,&
&  0.29446E+01_JPRB, 0.29079E+01_JPRB, 0.30594E+01_JPRB, 0.29092E+01_JPRB, 0.23338E+01_JPRB, 0.18151E+01_JPRB, 0.22922E+01_JPRB ,&
&  0.24070E+01_JPRB, 0.24251E+01_JPRB, 0.22320E+01_JPRB, 0.18763E+01_JPRB, 0.16483E+01_JPRB, 0.18688E+01_JPRB, 0.17872E+01_JPRB ,&
&  0.17291E+01_JPRB, 0.17664E+01_JPRB, 0.25222E+01_JPRB, 0.35661E+01_JPRB, 0.38923E+01_JPRB, 0.39444E+01_JPRB, 0.39692E+01_JPRB ,&
&  0.36595E+01_JPRB, 0.34693E+01_JPRB, 0.32483E+01_JPRB, 0.29990E+01_JPRB, 0.27801E+01_JPRB, 0.25638E+01_JPRB, 0.23896E+01_JPRB ,&
&  0.22425E+01_JPRB, 0.21086E+01_JPRB, 0.19673E+01_JPRB, 0.18317E+01_JPRB, 0.17156E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.16736E+01_JPRB, 0.16424E+01_JPRB, 0.16128E+01_JPRB, 0.15837E+01_JPRB, 0.15391E+01_JPRB, 0.14799E+01_JPRB, 0.14037E+01_JPRB ,&
&  0.13062E+01_JPRB, 0.12288E+01_JPRB, 0.11598E+01_JPRB, 0.11283E+01_JPRB, 0.11038E+01_JPRB, 0.10713E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.12623E+01_JPRB, 0.11344E+01_JPRB, 0.11014E+01_JPRB, 0.11516E+01_JPRB, 0.11765E+01_JPRB, 0.10185E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.14612E+01_JPRB, 0.17289E+01_JPRB, 0.21743E+01_JPRB, 0.28278E+01_JPRB, 0.27215E+01_JPRB, 0.27651E+01_JPRB ,&
&  0.27212E+01_JPRB, 0.24167E+01_JPRB, 0.21726E+01_JPRB, 0.18709E+01_JPRB, 0.16043E+01_JPRB, 0.14443E+01_JPRB, 0.13150E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.12724E+01_JPRB, 0.12636E+01_JPRB, 0.12327E+01_JPRB, 0.11982E+01_JPRB, 0.11676E+01_JPRB, 0.11375E+01_JPRB ,&
&  0.11341E+01_JPRB, 0.11349E+01_JPRB, 0.11474E+01_JPRB, 0.12739E+01_JPRB, 0.13436E+01_JPRB, 0.13098E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.19344E+01_JPRB, 0.19886E+01_JPRB, 0.23069E+01_JPRB, 0.26729E+01_JPRB, 0.28428E+01_JPRB, 0.29965E+01_JPRB ,&
&  0.24189E+01_JPRB, 0.21340E+01_JPRB, 0.22546E+01_JPRB, 0.23631E+01_JPRB, 0.21693E+01_JPRB, 0.23454E+01_JPRB, 0.35455E+01_JPRB ,&
&  0.26813E+01_JPRB, 0.29172E+01_JPRB, 0.33136E+01_JPRB, 0.34577E+01_JPRB, 0.18240E+01_JPRB, 0.13796E+01_JPRB, 0.21107E+01_JPRB ,&
&  0.21668E+01_JPRB, 0.14158E+01_JPRB, 0.11328E+01_JPRB, 0.13757E+01_JPRB, 0.11552E+01_JPRB, 0.13601E+01_JPRB, 0.17767E+01_JPRB ,&
&  0.20240E+01_JPRB, 0.22263E+01_JPRB, 0.34397E+01_JPRB, 0.40546E+01_JPRB, 0.41637E+01_JPRB, 0.52004E+01_JPRB, 0.46203E+01_JPRB ,&
&  0.41634E+01_JPRB, 0.34401E+01_JPRB, 0.32375E+01_JPRB, 0.28716E+01_JPRB, 0.26136E+01_JPRB, 0.23786E+01_JPRB, 0.21823E+01_JPRB ,&
&  0.20339E+01_JPRB, 0.19095E+01_JPRB, 0.18196E+01_JPRB, 0.17501E+01_JPRB, 0.17107E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.17294E+01_JPRB, 0.16906E+01_JPRB, 0.16524E+01_JPRB, 0.16060E+01_JPRB, 0.15453E+01_JPRB, 0.14623E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.12580E+01_JPRB, 0.11834E+01_JPRB, 0.11314E+01_JPRB, 0.11067E+01_JPRB, 0.10775E+01_JPRB, 0.10513E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.12668E+01_JPRB, 0.13183E+01_JPRB, 0.12671E+01_JPRB, 0.12760E+01_JPRB, 0.12904E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.14912E+01_JPRB, 0.16504E+01_JPRB, 0.18662E+01_JPRB, 0.22074E+01_JPRB, 0.25357E+01_JPRB, 0.25371E+01_JPRB, 0.26572E+01_JPRB ,&
&  0.25345E+01_JPRB, 0.23383E+01_JPRB, 0.20494E+01_JPRB, 0.17602E+01_JPRB, 0.15420E+01_JPRB, 0.13747E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.12600E+01_JPRB, 0.12816E+01_JPRB, 0.12762E+01_JPRB, 0.12307E+01_JPRB, 0.12046E+01_JPRB, 0.11749E+01_JPRB, 0.11520E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11675E+01_JPRB, 0.12078E+01_JPRB, 0.13296E+01_JPRB, 0.15443E+01_JPRB, 0.16048E+01_JPRB, 0.15278E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.16863E+01_JPRB, 0.20346E+01_JPRB, 0.21999E+01_JPRB, 0.24358E+01_JPRB, 0.26800E+01_JPRB, 0.26725E+01_JPRB ,&
&  0.26124E+01_JPRB, 0.26850E+01_JPRB, 0.24835E+01_JPRB, 0.26986E+01_JPRB, 0.27209E+01_JPRB, 0.22840E+01_JPRB, 0.27397E+01_JPRB ,&
&  0.29263E+01_JPRB, 0.29746E+01_JPRB, 0.26267E+01_JPRB, 0.21325E+01_JPRB, 0.23778E+01_JPRB, 0.19126E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.10262E+01_JPRB, 0.97740E+00_JPRB, 0.10055E+01_JPRB, 0.10263E+01_JPRB, 0.13537E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.26338E+01_JPRB, 0.32678E+01_JPRB, 0.38149E+01_JPRB, 0.41197E+01_JPRB, 0.39582E+01_JPRB, 0.48058E+01_JPRB, 0.43722E+01_JPRB ,&
&  0.35569E+01_JPRB, 0.32170E+01_JPRB, 0.29682E+01_JPRB, 0.26928E+01_JPRB, 0.24887E+01_JPRB, 0.23211E+01_JPRB, 0.21756E+01_JPRB ,&
&  0.20611E+01_JPRB, 0.19641E+01_JPRB, 0.18915E+01_JPRB, 0.18319E+01_JPRB, 0.17699E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.17298E+01_JPRB, 0.16822E+01_JPRB, 0.16363E+01_JPRB, 0.15673E+01_JPRB, 0.14866E+01_JPRB, 0.13829E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.11107E+01_JPRB, 0.10852E+01_JPRB, 0.10602E+01_JPRB, 0.10444E+01_JPRB, 0.10426E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.11619E+01_JPRB, 0.12716E+01_JPRB, 0.14343E+01_JPRB, 0.14673E+01_JPRB, 0.16332E+01_JPRB, 0.16234E+01_JPRB, 0.17027E+01_JPRB ,&
&  0.18822E+01_JPRB, 0.19669E+01_JPRB, 0.21696E+01_JPRB, 0.23372E+01_JPRB, 0.25578E+01_JPRB, 0.25555E+01_JPRB, 0.25248E+01_JPRB ,&
&  0.23877E+01_JPRB, 0.22282E+01_JPRB, 0.19402E+01_JPRB, 0.16720E+01_JPRB, 0.14627E+01_JPRB, 0.13346E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.12544E+01_JPRB, 0.12885E+01_JPRB, 0.12957E+01_JPRB, 0.12546E+01_JPRB, 0.12244E+01_JPRB, 0.12117E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.12512E+01_JPRB, 0.12987E+01_JPRB, 0.14188E+01_JPRB, 0.13522E+01_JPRB, 0.12480E+01_JPRB, 0.14375E+01_JPRB ,&
&  0.15877E+01_JPRB, 0.18724E+01_JPRB, 0.19351E+01_JPRB, 0.21766E+01_JPRB, 0.22914E+01_JPRB, 0.23044E+01_JPRB, 0.24657E+01_JPRB ,&
&  0.25535E+01_JPRB, 0.30828E+01_JPRB, 0.24474E+01_JPRB, 0.25046E+01_JPRB, 0.34474E+01_JPRB, 0.29590E+01_JPRB, 0.28322E+01_JPRB ,&
&  0.32626E+01_JPRB, 0.31308E+01_JPRB, 0.34494E+01_JPRB, 0.31914E+01_JPRB, 0.53088E+01_JPRB, 0.60751E+01_JPRB, 0.18552E+01_JPRB ,&
&  0.12932E+01_JPRB, 0.11336E+01_JPRB, 0.10472E+01_JPRB, 0.97558E+00_JPRB, 0.11028E+01_JPRB, 0.15883E+01_JPRB, 0.24956E+01_JPRB ,&
&  0.29900E+01_JPRB, 0.39192E+01_JPRB, 0.42317E+01_JPRB, 0.46156E+01_JPRB, 0.40120E+01_JPRB, 0.40861E+01_JPRB, 0.32955E+01_JPRB ,&
&  0.31290E+01_JPRB, 0.29760E+01_JPRB, 0.26936E+01_JPRB, 0.24504E+01_JPRB, 0.23018E+01_JPRB, 0.21959E+01_JPRB, 0.20973E+01_JPRB ,&
&  0.20114E+01_JPRB, 0.19508E+01_JPRB, 0.18931E+01_JPRB, 0.18363E+01_JPRB, 0.17812E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.16647E+01_JPRB, 0.16169E+01_JPRB, 0.15560E+01_JPRB, 0.14648E+01_JPRB, 0.13808E+01_JPRB, 0.12867E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.10980E+01_JPRB, 0.10656E+01_JPRB, 0.10348E+01_JPRB, 0.10337E+01_JPRB, 0.10521E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.11969E+01_JPRB, 0.13171E+01_JPRB, 0.14521E+01_JPRB, 0.17048E+01_JPRB, 0.19325E+01_JPRB, 0.19647E+01_JPRB, 0.24298E+01_JPRB ,&
&  0.24848E+01_JPRB, 0.24355E+01_JPRB, 0.25804E+01_JPRB, 0.26067E+01_JPRB, 0.25788E+01_JPRB, 0.25024E+01_JPRB, 0.23964E+01_JPRB ,&
&  0.22449E+01_JPRB, 0.20966E+01_JPRB, 0.18096E+01_JPRB, 0.15812E+01_JPRB, 0.14300E+01_JPRB, 0.13204E+01_JPRB, 0.12645E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.13228E+01_JPRB, 0.13355E+01_JPRB, 0.12997E+01_JPRB, 0.12714E+01_JPRB, 0.12582E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.13518E+01_JPRB, 0.14157E+01_JPRB, 0.15081E+01_JPRB, 0.13268E+01_JPRB, 0.15271E+01_JPRB, 0.16115E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.18514E+01_JPRB, 0.18189E+01_JPRB, 0.19459E+01_JPRB, 0.21350E+01_JPRB, 0.21818E+01_JPRB, 0.23067E+01_JPRB ,&
&  0.26403E+01_JPRB, 0.25400E+01_JPRB, 0.23098E+01_JPRB, 0.25617E+01_JPRB, 0.32516E+01_JPRB, 0.44847E+01_JPRB, 0.27583E+01_JPRB ,&
&  0.24222E+01_JPRB, 0.37039E+01_JPRB, 0.44170E+01_JPRB, 0.45456E+01_JPRB, 0.66791E+01_JPRB, 0.93201E+01_JPRB, 0.57698E+01_JPRB ,&
&  0.16324E+01_JPRB, 0.11638E+01_JPRB, 0.13387E+01_JPRB, 0.12015E+01_JPRB, 0.11612E+01_JPRB, 0.17071E+01_JPRB, 0.49132E+01_JPRB ,&
&  0.38911E+01_JPRB, 0.46774E+01_JPRB, 0.48819E+01_JPRB, 0.47040E+01_JPRB, 0.39408E+01_JPRB, 0.35184E+01_JPRB, 0.30243E+01_JPRB ,&
&  0.25976E+01_JPRB, 0.24115E+01_JPRB, 0.21754E+01_JPRB, 0.20382E+01_JPRB, 0.19891E+01_JPRB, 0.19584E+01_JPRB, 0.18897E+01_JPRB ,&
&  0.18422E+01_JPRB, 0.18097E+01_JPRB, 0.17766E+01_JPRB, 0.17474E+01_JPRB, 0.17129E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.15514E+01_JPRB, 0.15064E+01_JPRB, 0.14244E+01_JPRB, 0.13485E+01_JPRB, 0.13103E+01_JPRB, 0.12371E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11198E+01_JPRB, 0.10908E+01_JPRB, 0.10929E+01_JPRB, 0.10827E+01_JPRB, 0.11160E+01_JPRB, 0.11816E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.13896E+01_JPRB, 0.15066E+01_JPRB, 0.17874E+01_JPRB, 0.21718E+01_JPRB, 0.22561E+01_JPRB, 0.27893E+01_JPRB ,&
&  0.33678E+01_JPRB, 0.32252E+01_JPRB, 0.29243E+01_JPRB, 0.27459E+01_JPRB, 0.26481E+01_JPRB, 0.26391E+01_JPRB, 0.23618E+01_JPRB ,&
&  0.21386E+01_JPRB, 0.20029E+01_JPRB, 0.17339E+01_JPRB, 0.15310E+01_JPRB, 0.13892E+01_JPRB, 0.13172E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.13445E+01_JPRB, 0.13821E+01_JPRB, 0.13882E+01_JPRB, 0.13622E+01_JPRB, 0.13366E+01_JPRB, 0.13203E+01_JPRB, 0.13578E+01_JPRB ,&
&  0.14255E+01_JPRB, 0.14955E+01_JPRB, 0.15501E+01_JPRB, 0.15574E+01_JPRB, 0.16028E+01_JPRB, 0.16512E+01_JPRB, 0.17633E+01_JPRB ,&
&  0.18133E+01_JPRB, 0.18480E+01_JPRB, 0.18831E+01_JPRB, 0.19078E+01_JPRB, 0.20535E+01_JPRB, 0.22131E+01_JPRB, 0.23758E+01_JPRB ,&
&  0.26159E+01_JPRB, 0.26784E+01_JPRB, 0.25160E+01_JPRB, 0.27397E+01_JPRB, 0.32676E+01_JPRB, 0.40480E+01_JPRB, 0.39604E+01_JPRB ,&
&  0.36770E+01_JPRB, 0.37343E+01_JPRB, 0.45779E+01_JPRB, 0.60286E+01_JPRB, 0.68825E+01_JPRB, 0.72194E+01_JPRB, 0.87850E+01_JPRB ,&
&  0.73187E+01_JPRB, 0.32638E+01_JPRB, 0.25556E+01_JPRB, 0.33552E+01_JPRB, 0.24233E+01_JPRB, 0.25643E+01_JPRB, 0.38359E+01_JPRB ,&
&  0.42221E+01_JPRB, 0.50574E+01_JPRB, 0.52374E+01_JPRB, 0.41193E+01_JPRB, 0.35452E+01_JPRB, 0.31317E+01_JPRB, 0.26115E+01_JPRB ,&
&  0.22264E+01_JPRB, 0.19871E+01_JPRB, 0.18456E+01_JPRB, 0.17504E+01_JPRB, 0.16914E+01_JPRB, 0.16356E+01_JPRB, 0.16037E+01_JPRB ,&
&  0.15731E+01_JPRB, 0.15601E+01_JPRB, 0.15621E+01_JPRB, 0.15659E+01_JPRB, 0.15694E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.14467E+01_JPRB, 0.13858E+01_JPRB, 0.13539E+01_JPRB, 0.13461E+01_JPRB, 0.13203E+01_JPRB, 0.12895E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12038E+01_JPRB, 0.12101E+01_JPRB, 0.12493E+01_JPRB, 0.12516E+01_JPRB, 0.12216E+01_JPRB, 0.12409E+01_JPRB, 0.13033E+01_JPRB ,&
&  0.13914E+01_JPRB, 0.15497E+01_JPRB, 0.17510E+01_JPRB, 0.21969E+01_JPRB, 0.27260E+01_JPRB, 0.36800E+01_JPRB, 0.28567E+01_JPRB ,&
&  0.32270E+01_JPRB, 0.35808E+01_JPRB, 0.30562E+01_JPRB, 0.28672E+01_JPRB, 0.27193E+01_JPRB, 0.26203E+01_JPRB, 0.22450E+01_JPRB ,&
&  0.20858E+01_JPRB, 0.19240E+01_JPRB, 0.17014E+01_JPRB, 0.15154E+01_JPRB, 0.13904E+01_JPRB, 0.14030E+01_JPRB, 0.14125E+01_JPRB ,&
&  0.14403E+01_JPRB, 0.14843E+01_JPRB, 0.14626E+01_JPRB, 0.14528E+01_JPRB, 0.14338E+01_JPRB, 0.14236E+01_JPRB, 0.14818E+01_JPRB ,&
&  0.15628E+01_JPRB, 0.16283E+01_JPRB, 0.16809E+01_JPRB, 0.16427E+01_JPRB, 0.17189E+01_JPRB, 0.17870E+01_JPRB, 0.18909E+01_JPRB ,&
&  0.18474E+01_JPRB, 0.17121E+01_JPRB, 0.17962E+01_JPRB, 0.19623E+01_JPRB, 0.21354E+01_JPRB, 0.22013E+01_JPRB, 0.23370E+01_JPRB ,&
&  0.27617E+01_JPRB, 0.31868E+01_JPRB, 0.34885E+01_JPRB, 0.31384E+01_JPRB, 0.32811E+01_JPRB, 0.40381E+01_JPRB, 0.49999E+01_JPRB ,&
&  0.56309E+01_JPRB, 0.58557E+01_JPRB, 0.63824E+01_JPRB, 0.63410E+01_JPRB, 0.61656E+01_JPRB, 0.59713E+01_JPRB, 0.63973E+01_JPRB ,&
&  0.74418E+01_JPRB, 0.81712E+01_JPRB, 0.64709E+01_JPRB, 0.51438E+01_JPRB, 0.50430E+01_JPRB, 0.42599E+01_JPRB, 0.39950E+01_JPRB ,&
&  0.47580E+01_JPRB, 0.48560E+01_JPRB, 0.44981E+01_JPRB, 0.44142E+01_JPRB, 0.33323E+01_JPRB, 0.26452E+01_JPRB, 0.20074E+01_JPRB ,&
&  0.15985E+01_JPRB, 0.14481E+01_JPRB, 0.13899E+01_JPRB, 0.13702E+01_JPRB, 0.14102E+01_JPRB, 0.14179E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.13989E+01_JPRB, 0.14191E+01_JPRB, 0.14548E+01_JPRB, 0.14869E+01_JPRB, 0.14853E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.13654E+01_JPRB, 0.13483E+01_JPRB, 0.13610E+01_JPRB, 0.13760E+01_JPRB, 0.13724E+01_JPRB, 0.13940E+01_JPRB, 0.13876E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.13037E+01_JPRB, 0.13515E+01_JPRB, 0.13408E+01_JPRB, 0.13310E+01_JPRB, 0.13585E+01_JPRB, 0.14394E+01_JPRB ,&
&  0.15502E+01_JPRB, 0.17800E+01_JPRB, 0.21559E+01_JPRB, 0.26723E+01_JPRB, 0.30753E+01_JPRB, 0.35798E+01_JPRB, 0.38328E+01_JPRB ,&
&  0.28558E+01_JPRB, 0.36909E+01_JPRB, 0.28805E+01_JPRB, 0.27935E+01_JPRB, 0.27291E+01_JPRB, 0.29026E+01_JPRB, 0.25674E+01_JPRB ,&
&  0.21248E+01_JPRB, 0.18542E+01_JPRB, 0.17649E+01_JPRB, 0.16404E+01_JPRB, 0.15050E+01_JPRB, 0.14877E+01_JPRB, 0.15140E+01_JPRB ,&
&  0.15165E+01_JPRB, 0.15211E+01_JPRB, 0.15327E+01_JPRB, 0.15423E+01_JPRB, 0.15341E+01_JPRB, 0.15404E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.16658E+01_JPRB, 0.17412E+01_JPRB, 0.17604E+01_JPRB, 0.17936E+01_JPRB, 0.18198E+01_JPRB, 0.19486E+01_JPRB, 0.20569E+01_JPRB ,&
&  0.19877E+01_JPRB, 0.18316E+01_JPRB, 0.18844E+01_JPRB, 0.19779E+01_JPRB, 0.20600E+01_JPRB, 0.23064E+01_JPRB, 0.24480E+01_JPRB ,&
&  0.30586E+01_JPRB, 0.32787E+01_JPRB, 0.39929E+01_JPRB, 0.33223E+01_JPRB, 0.33792E+01_JPRB, 0.41274E+01_JPRB, 0.45091E+01_JPRB ,&
&  0.45959E+01_JPRB, 0.56421E+01_JPRB, 0.60386E+01_JPRB, 0.63112E+01_JPRB, 0.61966E+01_JPRB, 0.57161E+01_JPRB, 0.53745E+01_JPRB ,&
&  0.56362E+01_JPRB, 0.61168E+01_JPRB, 0.62957E+01_JPRB, 0.52537E+01_JPRB, 0.49375E+01_JPRB, 0.43781E+01_JPRB, 0.46698E+01_JPRB ,&
&  0.53822E+01_JPRB, 0.53347E+01_JPRB, 0.46929E+01_JPRB, 0.38434E+01_JPRB, 0.24169E+01_JPRB, 0.17688E+01_JPRB, 0.13779E+01_JPRB ,&
&  0.12259E+01_JPRB, 0.11771E+01_JPRB, 0.12018E+01_JPRB, 0.12698E+01_JPRB, 0.13310E+01_JPRB, 0.13583E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.13573E+01_JPRB, 0.13792E+01_JPRB, 0.13937E+01_JPRB, 0.13955E+01_JPRB, 0.13868E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.13616E+01_JPRB, 0.13786E+01_JPRB, 0.14030E+01_JPRB, 0.14478E+01_JPRB, 0.14815E+01_JPRB, 0.15717E+01_JPRB, 0.16599E+01_JPRB ,&
&  0.15513E+01_JPRB, 0.12321E+01_JPRB, 0.12794E+01_JPRB, 0.13261E+01_JPRB, 0.13757E+01_JPRB, 0.14395E+01_JPRB, 0.15454E+01_JPRB ,&
&  0.17555E+01_JPRB, 0.20306E+01_JPRB, 0.23738E+01_JPRB, 0.27213E+01_JPRB, 0.31418E+01_JPRB, 0.36049E+01_JPRB, 0.36939E+01_JPRB ,&
&  0.30157E+01_JPRB, 0.26585E+01_JPRB, 0.29446E+01_JPRB, 0.28722E+01_JPRB, 0.28081E+01_JPRB, 0.28006E+01_JPRB, 0.27389E+01_JPRB ,&
&  0.22534E+01_JPRB, 0.19769E+01_JPRB, 0.19331E+01_JPRB, 0.19277E+01_JPRB, 0.16130E+01_JPRB, 0.16383E+01_JPRB, 0.16669E+01_JPRB ,&
&  0.16675E+01_JPRB, 0.16561E+01_JPRB, 0.16726E+01_JPRB, 0.16969E+01_JPRB, 0.16779E+01_JPRB, 0.16823E+01_JPRB, 0.17309E+01_JPRB ,&
&  0.18385E+01_JPRB, 0.19182E+01_JPRB, 0.19584E+01_JPRB, 0.19506E+01_JPRB, 0.20265E+01_JPRB, 0.21578E+01_JPRB, 0.21858E+01_JPRB ,&
&  0.20971E+01_JPRB, 0.20762E+01_JPRB, 0.21223E+01_JPRB, 0.21967E+01_JPRB, 0.21672E+01_JPRB, 0.25329E+01_JPRB, 0.27717E+01_JPRB ,&
&  0.31773E+01_JPRB, 0.34701E+01_JPRB, 0.41136E+01_JPRB, 0.33124E+01_JPRB, 0.36753E+01_JPRB, 0.41454E+01_JPRB, 0.43933E+01_JPRB ,&
&  0.44328E+01_JPRB, 0.49349E+01_JPRB, 0.51121E+01_JPRB, 0.53675E+01_JPRB, 0.54685E+01_JPRB, 0.48843E+01_JPRB, 0.52675E+01_JPRB ,&
&  0.51482E+01_JPRB, 0.59145E+01_JPRB, 0.57370E+01_JPRB, 0.55688E+01_JPRB, 0.46812E+01_JPRB, 0.44975E+01_JPRB, 0.46743E+01_JPRB ,&
&  0.51479E+01_JPRB, 0.47235E+01_JPRB, 0.36472E+01_JPRB, 0.26334E+01_JPRB, 0.15588E+01_JPRB, 0.11908E+01_JPRB, 0.10982E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.11522E+01_JPRB, 0.12183E+01_JPRB, 0.12798E+01_JPRB, 0.13151E+01_JPRB, 0.13278E+01_JPRB, 0.13328E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.13573E+01_JPRB, 0.13543E+01_JPRB, 0.13557E+01_JPRB, 0.13574E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.13532E+01_JPRB, 0.13484E+01_JPRB, 0.13470E+01_JPRB, 0.13182E+01_JPRB, 0.12823E+01_JPRB, 0.12406E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.11167E+01_JPRB, 0.11093E+01_JPRB, 0.11819E+01_JPRB, 0.12724E+01_JPRB, 0.13599E+01_JPRB, 0.14627E+01_JPRB, 0.16233E+01_JPRB ,&
&  0.18170E+01_JPRB, 0.19688E+01_JPRB, 0.21504E+01_JPRB, 0.24458E+01_JPRB, 0.28073E+01_JPRB, 0.29704E+01_JPRB, 0.31019E+01_JPRB ,&
&  0.28380E+01_JPRB, 0.21317E+01_JPRB, 0.26694E+01_JPRB, 0.29488E+01_JPRB, 0.29755E+01_JPRB, 0.29479E+01_JPRB, 0.27150E+01_JPRB ,&
&  0.21883E+01_JPRB, 0.21365E+01_JPRB, 0.22243E+01_JPRB, 0.22202E+01_JPRB, 0.18126E+01_JPRB, 0.18217E+01_JPRB, 0.18322E+01_JPRB ,&
&  0.18492E+01_JPRB, 0.18494E+01_JPRB, 0.18930E+01_JPRB, 0.19204E+01_JPRB, 0.19181E+01_JPRB, 0.19605E+01_JPRB, 0.20279E+01_JPRB ,&
&  0.20891E+01_JPRB, 0.21359E+01_JPRB, 0.22080E+01_JPRB, 0.22541E+01_JPRB, 0.23055E+01_JPRB, 0.23764E+01_JPRB, 0.23507E+01_JPRB ,&
&  0.22829E+01_JPRB, 0.22211E+01_JPRB, 0.23553E+01_JPRB, 0.26436E+01_JPRB, 0.28931E+01_JPRB, 0.27197E+01_JPRB, 0.30762E+01_JPRB ,&
&  0.32920E+01_JPRB, 0.35463E+01_JPRB, 0.36396E+01_JPRB, 0.43301E+01_JPRB, 0.32193E+01_JPRB, 0.37261E+01_JPRB, 0.37011E+01_JPRB ,&
&  0.41327E+01_JPRB, 0.40830E+01_JPRB, 0.40509E+01_JPRB, 0.41663E+01_JPRB, 0.43345E+01_JPRB, 0.40165E+01_JPRB, 0.44029E+01_JPRB ,&
&  0.48463E+01_JPRB, 0.52333E+01_JPRB, 0.53759E+01_JPRB, 0.55280E+01_JPRB, 0.51465E+01_JPRB, 0.50852E+01_JPRB, 0.48998E+01_JPRB ,&
&  0.49934E+01_JPRB, 0.35578E+01_JPRB, 0.24835E+01_JPRB, 0.17131E+01_JPRB, 0.11945E+01_JPRB, 0.99574E+00_JPRB, 0.10380E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11680E+01_JPRB, 0.12177E+01_JPRB, 0.12557E+01_JPRB, 0.12809E+01_JPRB, 0.13002E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.13435E+01_JPRB, 0.13485E+01_JPRB, 0.13470E+01_JPRB, 0.13464E+01_JPRB, 0.13495E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.13060E+01_JPRB, 0.12770E+01_JPRB, 0.12232E+01_JPRB, 0.11424E+01_JPRB, 0.10568E+01_JPRB, 0.97545E+00_JPRB, 0.92986E+00_JPRB ,&
&  0.89937E+00_JPRB, 0.93691E+00_JPRB, 0.10275E+01_JPRB, 0.11474E+01_JPRB, 0.12825E+01_JPRB, 0.14169E+01_JPRB, 0.15385E+01_JPRB ,&
&  0.16141E+01_JPRB, 0.16635E+01_JPRB, 0.17743E+01_JPRB, 0.20336E+01_JPRB, 0.23314E+01_JPRB, 0.25702E+01_JPRB, 0.27377E+01_JPRB ,&
&  0.28878E+01_JPRB, 0.29217E+01_JPRB, 0.27486E+01_JPRB, 0.22195E+01_JPRB, 0.20793E+01_JPRB, 0.25286E+01_JPRB, 0.21823E+01_JPRB ,&
&  0.20886E+01_JPRB, 0.20094E+01_JPRB, 0.20076E+01_JPRB, 0.20162E+01_JPRB, 0.19612E+01_JPRB, 0.19255E+01_JPRB, 0.19570E+01_JPRB ,&
&  0.19835E+01_JPRB, 0.20308E+01_JPRB, 0.20797E+01_JPRB, 0.21074E+01_JPRB, 0.21575E+01_JPRB, 0.22294E+01_JPRB, 0.22911E+01_JPRB ,&
&  0.23337E+01_JPRB, 0.24083E+01_JPRB, 0.24909E+01_JPRB, 0.25837E+01_JPRB, 0.25660E+01_JPRB, 0.26095E+01_JPRB, 0.25928E+01_JPRB ,&
&  0.25725E+01_JPRB, 0.25658E+01_JPRB, 0.26380E+01_JPRB, 0.28687E+01_JPRB, 0.30109E+01_JPRB, 0.27664E+01_JPRB, 0.28916E+01_JPRB ,&
&  0.32805E+01_JPRB, 0.35066E+01_JPRB, 0.32370E+01_JPRB, 0.42216E+01_JPRB, 0.27371E+01_JPRB, 0.34264E+01_JPRB, 0.38546E+01_JPRB ,&
&  0.36328E+01_JPRB, 0.33697E+01_JPRB, 0.33525E+01_JPRB, 0.34417E+01_JPRB, 0.34604E+01_JPRB, 0.34425E+01_JPRB, 0.39764E+01_JPRB ,&
&  0.44277E+01_JPRB, 0.46413E+01_JPRB, 0.49525E+01_JPRB, 0.49995E+01_JPRB, 0.49748E+01_JPRB, 0.45668E+01_JPRB, 0.39732E+01_JPRB ,&
&  0.31886E+01_JPRB, 0.26662E+01_JPRB, 0.20988E+01_JPRB, 0.17972E+01_JPRB, 0.12401E+01_JPRB, 0.96156E+00_JPRB, 0.10276E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.11248E+01_JPRB, 0.11804E+01_JPRB, 0.12231E+01_JPRB, 0.12405E+01_JPRB, 0.12607E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.12970E+01_JPRB, 0.12960E+01_JPRB, 0.12999E+01_JPRB, 0.13014E+01_JPRB, 0.13087E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.10781E+01_JPRB, 0.10420E+01_JPRB, 0.98720E+00_JPRB, 0.93492E+00_JPRB, 0.88410E+00_JPRB, 0.82822E+00_JPRB, 0.81980E+00_JPRB ,&
&  0.80116E+00_JPRB, 0.81537E+00_JPRB, 0.85826E+00_JPRB, 0.91151E+00_JPRB, 0.99047E+00_JPRB, 0.10998E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.12604E+01_JPRB, 0.13279E+01_JPRB, 0.14319E+01_JPRB, 0.16409E+01_JPRB, 0.20290E+01_JPRB, 0.26676E+01_JPRB ,&
&  0.30315E+01_JPRB, 0.32122E+01_JPRB, 0.31182E+01_JPRB, 0.28629E+01_JPRB, 0.25013E+01_JPRB, 0.25696E+01_JPRB, 0.27414E+01_JPRB ,&
&  0.28381E+01_JPRB, 0.25040E+01_JPRB, 0.20939E+01_JPRB, 0.20403E+01_JPRB, 0.20219E+01_JPRB, 0.20132E+01_JPRB, 0.20469E+01_JPRB ,&
&  0.20960E+01_JPRB, 0.21342E+01_JPRB, 0.21714E+01_JPRB, 0.22347E+01_JPRB, 0.22881E+01_JPRB, 0.23440E+01_JPRB, 0.24177E+01_JPRB ,&
&  0.24882E+01_JPRB, 0.25445E+01_JPRB, 0.25967E+01_JPRB, 0.26497E+01_JPRB, 0.27643E+01_JPRB, 0.28875E+01_JPRB, 0.29504E+01_JPRB ,&
&  0.29823E+01_JPRB, 0.28956E+01_JPRB, 0.30326E+01_JPRB, 0.32488E+01_JPRB, 0.32424E+01_JPRB, 0.30778E+01_JPRB, 0.31178E+01_JPRB ,&
&  0.33699E+01_JPRB, 0.35608E+01_JPRB, 0.26624E+01_JPRB, 0.35259E+01_JPRB, 0.42027E+01_JPRB, 0.37775E+01_JPRB, 0.33680E+01_JPRB ,&
&  0.29837E+01_JPRB, 0.27730E+01_JPRB, 0.28854E+01_JPRB, 0.29601E+01_JPRB, 0.32271E+01_JPRB, 0.32637E+01_JPRB, 0.29451E+01_JPRB ,&
&  0.33188E+01_JPRB, 0.35270E+01_JPRB, 0.39697E+01_JPRB, 0.41476E+01_JPRB, 0.39706E+01_JPRB, 0.34841E+01_JPRB, 0.27908E+01_JPRB ,&
&  0.20983E+01_JPRB, 0.20899E+01_JPRB, 0.17992E+01_JPRB, 0.17604E+01_JPRB, 0.14870E+01_JPRB, 0.92862E+00_JPRB, 0.98945E+00_JPRB ,&
&  0.10230E+01_JPRB, 0.10582E+01_JPRB, 0.10991E+01_JPRB, 0.11244E+01_JPRB, 0.11512E+01_JPRB, 0.11696E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.11900E+01_JPRB, 0.11705E+01_JPRB, 0.11545E+01_JPRB, 0.11424E+01_JPRB, 0.11179E+01_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.93526E+00_JPRB, 0.91854E+00_JPRB, 0.90102E+00_JPRB, 0.89170E+00_JPRB, 0.89252E+00_JPRB, 0.88312E+00_JPRB, 0.86353E+00_JPRB ,&
&  0.83599E+00_JPRB, 0.82475E+00_JPRB, 0.80032E+00_JPRB, 0.77745E+00_JPRB, 0.78775E+00_JPRB, 0.85431E+00_JPRB, 0.92173E+00_JPRB ,&
&  0.96148E+00_JPRB, 0.97666E+00_JPRB, 0.96828E+00_JPRB, 0.95568E+00_JPRB, 0.98788E+00_JPRB, 0.10376E+01_JPRB, 0.13445E+01_JPRB ,&
&  0.18730E+01_JPRB, 0.25807E+01_JPRB, 0.30815E+01_JPRB, 0.33190E+01_JPRB, 0.32269E+01_JPRB, 0.28712E+01_JPRB, 0.34900E+01_JPRB ,&
&  0.30747E+01_JPRB, 0.22554E+01_JPRB, 0.19863E+01_JPRB, 0.18748E+01_JPRB, 0.18599E+01_JPRB, 0.18763E+01_JPRB, 0.19233E+01_JPRB ,&
&  0.19863E+01_JPRB, 0.20519E+01_JPRB, 0.21190E+01_JPRB, 0.22004E+01_JPRB, 0.22858E+01_JPRB, 0.23820E+01_JPRB, 0.24788E+01_JPRB ,&
&  0.25890E+01_JPRB, 0.26517E+01_JPRB, 0.27601E+01_JPRB, 0.26553E+01_JPRB, 0.28666E+01_JPRB, 0.30646E+01_JPRB, 0.32055E+01_JPRB ,&
&  0.32478E+01_JPRB, 0.32267E+01_JPRB, 0.34180E+01_JPRB, 0.35093E+01_JPRB, 0.35656E+01_JPRB, 0.35408E+01_JPRB, 0.34711E+01_JPRB ,&
&  0.35668E+01_JPRB, 0.33412E+01_JPRB, 0.25713E+01_JPRB, 0.28924E+01_JPRB, 0.32989E+01_JPRB, 0.27035E+01_JPRB, 0.29536E+01_JPRB ,&
&  0.25213E+01_JPRB, 0.22227E+01_JPRB, 0.20251E+01_JPRB, 0.20744E+01_JPRB, 0.23953E+01_JPRB, 0.27379E+01_JPRB, 0.24469E+01_JPRB ,&
&  0.22386E+01_JPRB, 0.23555E+01_JPRB, 0.26921E+01_JPRB, 0.28732E+01_JPRB, 0.27798E+01_JPRB, 0.24736E+01_JPRB, 0.20983E+01_JPRB ,&
&  0.18202E+01_JPRB, 0.16449E+01_JPRB, 0.14867E+01_JPRB, 0.13741E+01_JPRB, 0.10162E+01_JPRB, 0.85370E+00_JPRB, 0.88689E+00_JPRB ,&
&  0.90862E+00_JPRB, 0.91016E+00_JPRB, 0.92688E+00_JPRB, 0.94251E+00_JPRB, 0.96947E+00_JPRB, 0.97465E+00_JPRB, 0.99595E+00_JPRB ,&
&  0.10071E+01_JPRB, 0.10028E+01_JPRB, 0.99683E+00_JPRB, 0.99622E+00_JPRB, 0.96765E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.99999E+00_JPRB, 0.10037E+01_JPRB, 0.10170E+01_JPRB, 0.10317E+01_JPRB, 0.10526E+01_JPRB, 0.10371E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.97351E+00_JPRB, 0.96407E+00_JPRB, 0.95447E+00_JPRB, 0.95033E+00_JPRB, 0.96300E+00_JPRB, 0.99448E+00_JPRB, 0.10524E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10732E+01_JPRB, 0.10594E+01_JPRB, 0.10408E+01_JPRB, 0.99220E+00_JPRB, 0.94837E+00_JPRB, 0.92122E+00_JPRB ,&
&  0.96572E+00_JPRB, 0.12393E+01_JPRB, 0.15842E+01_JPRB, 0.19121E+01_JPRB, 0.23420E+01_JPRB, 0.22936E+01_JPRB, 0.27914E+01_JPRB ,&
&  0.32763E+01_JPRB, 0.20489E+01_JPRB, 0.19106E+01_JPRB, 0.16784E+01_JPRB, 0.16631E+01_JPRB, 0.16526E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.17726E+01_JPRB, 0.18208E+01_JPRB, 0.18570E+01_JPRB, 0.19037E+01_JPRB, 0.19726E+01_JPRB, 0.20435E+01_JPRB, 0.20811E+01_JPRB ,&
&  0.21061E+01_JPRB, 0.20411E+01_JPRB, 0.20761E+01_JPRB, 0.23988E+01_JPRB, 0.27137E+01_JPRB, 0.29048E+01_JPRB, 0.30625E+01_JPRB ,&
&  0.32697E+01_JPRB, 0.34232E+01_JPRB, 0.32490E+01_JPRB, 0.32826E+01_JPRB, 0.34817E+01_JPRB, 0.32686E+01_JPRB, 0.33648E+01_JPRB ,&
&  0.34755E+01_JPRB, 0.31055E+01_JPRB, 0.22564E+01_JPRB, 0.26840E+01_JPRB, 0.29355E+01_JPRB, 0.27154E+01_JPRB, 0.27035E+01_JPRB ,&
&  0.22642E+01_JPRB, 0.19402E+01_JPRB, 0.15965E+01_JPRB, 0.14861E+01_JPRB, 0.15635E+01_JPRB, 0.18194E+01_JPRB, 0.19007E+01_JPRB ,&
&  0.16710E+01_JPRB, 0.16392E+01_JPRB, 0.19142E+01_JPRB, 0.21947E+01_JPRB, 0.23032E+01_JPRB, 0.21103E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.16984E+01_JPRB, 0.17026E+01_JPRB, 0.20454E+01_JPRB, 0.19368E+01_JPRB, 0.10674E+01_JPRB, 0.87422E+00_JPRB, 0.84363E+00_JPRB ,&
&  0.86171E+00_JPRB, 0.86195E+00_JPRB, 0.85700E+00_JPRB, 0.87242E+00_JPRB, 0.88218E+00_JPRB, 0.88490E+00_JPRB, 0.92267E+00_JPRB ,&
&  0.96757E+00_JPRB, 0.97134E+00_JPRB, 0.97972E+00_JPRB, 0.98284E+00_JPRB, 0.98990E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.11037E+01_JPRB, 0.11225E+01_JPRB, 0.11489E+01_JPRB, 0.11558E+01_JPRB, 0.11628E+01_JPRB, 0.11543E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11263E+01_JPRB, 0.11201E+01_JPRB, 0.11236E+01_JPRB, 0.11292E+01_JPRB, 0.11463E+01_JPRB, 0.11646E+01_JPRB, 0.12033E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.12167E+01_JPRB, 0.12284E+01_JPRB, 0.12516E+01_JPRB, 0.12704E+01_JPRB, 0.12615E+01_JPRB, 0.12222E+01_JPRB ,&
&  0.11981E+01_JPRB, 0.11934E+01_JPRB, 0.12112E+01_JPRB, 0.12504E+01_JPRB, 0.14573E+01_JPRB, 0.18180E+01_JPRB, 0.24229E+01_JPRB ,&
&  0.26428E+01_JPRB, 0.20102E+01_JPRB, 0.16814E+01_JPRB, 0.14614E+01_JPRB, 0.14361E+01_JPRB, 0.14448E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.13379E+01_JPRB, 0.12894E+01_JPRB, 0.12923E+01_JPRB, 0.13301E+01_JPRB, 0.13644E+01_JPRB, 0.13485E+01_JPRB, 0.13250E+01_JPRB ,&
&  0.12292E+01_JPRB, 0.11972E+01_JPRB, 0.12640E+01_JPRB, 0.14492E+01_JPRB, 0.18653E+01_JPRB, 0.21412E+01_JPRB, 0.20511E+01_JPRB ,&
&  0.22783E+01_JPRB, 0.26645E+01_JPRB, 0.28246E+01_JPRB, 0.28743E+01_JPRB, 0.28216E+01_JPRB, 0.27164E+01_JPRB, 0.26490E+01_JPRB ,&
&  0.25971E+01_JPRB, 0.24063E+01_JPRB, 0.21349E+01_JPRB, 0.24642E+01_JPRB, 0.26036E+01_JPRB, 0.26427E+01_JPRB, 0.23412E+01_JPRB ,&
&  0.19661E+01_JPRB, 0.17160E+01_JPRB, 0.15533E+01_JPRB, 0.14077E+01_JPRB, 0.13287E+01_JPRB, 0.14054E+01_JPRB, 0.13869E+01_JPRB ,&
&  0.13152E+01_JPRB, 0.13725E+01_JPRB, 0.14816E+01_JPRB, 0.16481E+01_JPRB, 0.20713E+01_JPRB, 0.18968E+01_JPRB, 0.19947E+01_JPRB ,&
&  0.17335E+01_JPRB, 0.17465E+01_JPRB, 0.14143E+01_JPRB, 0.20509E+01_JPRB, 0.10584E+01_JPRB, 0.79853E+00_JPRB, 0.81089E+00_JPRB ,&
&  0.85557E+00_JPRB, 0.87820E+00_JPRB, 0.91195E+00_JPRB, 0.94546E+00_JPRB, 0.95575E+00_JPRB, 0.96681E+00_JPRB, 0.99815E+00_JPRB ,&
&  0.10331E+01_JPRB, 0.10552E+01_JPRB, 0.10757E+01_JPRB, 0.10789E+01_JPRB, 0.10909E+01_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.11641E+01_JPRB, 0.11837E+01_JPRB, 0.12020E+01_JPRB, 0.12172E+01_JPRB, 0.12243E+01_JPRB, 0.12288E+01_JPRB, 0.12325E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.12298E+01_JPRB, 0.12305E+01_JPRB, 0.12481E+01_JPRB, 0.12764E+01_JPRB, 0.12927E+01_JPRB, 0.13269E+01_JPRB ,&
&  0.13246E+01_JPRB, 0.13465E+01_JPRB, 0.13790E+01_JPRB, 0.14081E+01_JPRB, 0.14497E+01_JPRB, 0.14505E+01_JPRB, 0.14281E+01_JPRB ,&
&  0.14168E+01_JPRB, 0.14010E+01_JPRB, 0.14073E+01_JPRB, 0.14168E+01_JPRB, 0.14350E+01_JPRB, 0.14731E+01_JPRB, 0.17260E+01_JPRB ,&
&  0.19401E+01_JPRB, 0.20939E+01_JPRB, 0.18083E+01_JPRB, 0.16064E+01_JPRB, 0.15816E+01_JPRB, 0.14409E+01_JPRB, 0.13592E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.11584E+01_JPRB, 0.11200E+01_JPRB, 0.11028E+01_JPRB, 0.11453E+01_JPRB, 0.11967E+01_JPRB, 0.12300E+01_JPRB ,&
&  0.13040E+01_JPRB, 0.13596E+01_JPRB, 0.14249E+01_JPRB, 0.14712E+01_JPRB, 0.15096E+01_JPRB, 0.16393E+01_JPRB, 0.15669E+01_JPRB ,&
&  0.16069E+01_JPRB, 0.17801E+01_JPRB, 0.18552E+01_JPRB, 0.19419E+01_JPRB, 0.18552E+01_JPRB, 0.17450E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.13769E+01_JPRB, 0.13832E+01_JPRB, 0.16705E+01_JPRB, 0.19245E+01_JPRB, 0.21146E+01_JPRB, 0.22014E+01_JPRB, 0.19887E+01_JPRB ,&
&  0.17712E+01_JPRB, 0.16202E+01_JPRB, 0.14695E+01_JPRB, 0.13337E+01_JPRB, 0.12641E+01_JPRB, 0.12624E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.12234E+01_JPRB, 0.12846E+01_JPRB, 0.14016E+01_JPRB, 0.15676E+01_JPRB, 0.26307E+01_JPRB, 0.31951E+01_JPRB ,&
&  0.19954E+01_JPRB, 0.16118E+01_JPRB, 0.98934E+00_JPRB, 0.14351E+01_JPRB, 0.90789E+00_JPRB, 0.77030E+00_JPRB, 0.82628E+00_JPRB ,&
&  0.92953E+00_JPRB, 0.10112E+01_JPRB, 0.10547E+01_JPRB, 0.11068E+01_JPRB, 0.10297E+01_JPRB, 0.10248E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10818E+01_JPRB, 0.11053E+01_JPRB, 0.11231E+01_JPRB, 0.11443E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.11796E+01_JPRB, 0.11990E+01_JPRB, 0.12189E+01_JPRB, 0.12429E+01_JPRB, 0.12636E+01_JPRB, 0.12886E+01_JPRB, 0.13047E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.13287E+01_JPRB, 0.13337E+01_JPRB, 0.13502E+01_JPRB, 0.13682E+01_JPRB, 0.14015E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.14886E+01_JPRB, 0.15409E+01_JPRB, 0.16011E+01_JPRB, 0.16447E+01_JPRB, 0.16840E+01_JPRB, 0.17183E+01_JPRB, 0.17471E+01_JPRB ,&
&  0.17613E+01_JPRB, 0.17527E+01_JPRB, 0.17762E+01_JPRB, 0.18311E+01_JPRB, 0.18888E+01_JPRB, 0.20189E+01_JPRB, 0.18488E+01_JPRB ,&
&  0.21501E+01_JPRB, 0.21558E+01_JPRB, 0.20380E+01_JPRB, 0.18502E+01_JPRB, 0.17855E+01_JPRB, 0.17369E+01_JPRB, 0.17263E+01_JPRB ,&
&  0.17545E+01_JPRB, 0.16865E+01_JPRB, 0.16204E+01_JPRB, 0.15085E+01_JPRB, 0.14435E+01_JPRB, 0.14625E+01_JPRB, 0.15298E+01_JPRB ,&
&  0.15660E+01_JPRB, 0.16118E+01_JPRB, 0.16222E+01_JPRB, 0.16124E+01_JPRB, 0.16101E+01_JPRB, 0.15908E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.14319E+01_JPRB, 0.14092E+01_JPRB, 0.12327E+01_JPRB, 0.12233E+01_JPRB, 0.12195E+01_JPRB, 0.11241E+01_JPRB, 0.96835E+00_JPRB ,&
&  0.89597E+00_JPRB, 0.89030E+00_JPRB, 0.10232E+01_JPRB, 0.14606E+01_JPRB, 0.17147E+01_JPRB, 0.17767E+01_JPRB, 0.16922E+01_JPRB ,&
&  0.15366E+01_JPRB, 0.13825E+01_JPRB, 0.12092E+01_JPRB, 0.11111E+01_JPRB, 0.10945E+01_JPRB, 0.11424E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.11653E+01_JPRB, 0.11844E+01_JPRB, 0.12329E+01_JPRB, 0.14710E+01_JPRB, 0.19166E+01_JPRB, 0.29154E+01_JPRB ,&
&  0.20731E+01_JPRB, 0.12105E+01_JPRB, 0.11484E+01_JPRB, 0.11819E+01_JPRB, 0.86417E+00_JPRB, 0.72508E+00_JPRB, 0.81171E+00_JPRB ,&
&  0.10298E+01_JPRB, 0.11639E+01_JPRB, 0.11673E+01_JPRB, 0.11568E+01_JPRB, 0.11137E+01_JPRB, 0.11237E+01_JPRB, 0.11127E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.11192E+01_JPRB, 0.11311E+01_JPRB, 0.11429E+01_JPRB, 0.11605E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.11776E+01_JPRB, 0.12033E+01_JPRB, 0.12286E+01_JPRB, 0.12547E+01_JPRB, 0.12759E+01_JPRB, 0.13049E+01_JPRB, 0.13191E+01_JPRB ,&
&  0.13370E+01_JPRB, 0.13500E+01_JPRB, 0.13744E+01_JPRB, 0.14018E+01_JPRB, 0.14341E+01_JPRB, 0.14736E+01_JPRB, 0.15217E+01_JPRB ,&
&  0.15835E+01_JPRB, 0.16360E+01_JPRB, 0.16979E+01_JPRB, 0.17466E+01_JPRB, 0.17711E+01_JPRB, 0.18098E+01_JPRB, 0.18470E+01_JPRB ,&
&  0.18549E+01_JPRB, 0.18461E+01_JPRB, 0.18798E+01_JPRB, 0.19353E+01_JPRB, 0.19540E+01_JPRB, 0.20705E+01_JPRB, 0.17257E+01_JPRB ,&
&  0.22093E+01_JPRB, 0.23296E+01_JPRB, 0.23495E+01_JPRB, 0.21614E+01_JPRB, 0.20507E+01_JPRB, 0.20545E+01_JPRB, 0.20919E+01_JPRB ,&
&  0.21628E+01_JPRB, 0.22298E+01_JPRB, 0.21326E+01_JPRB, 0.19878E+01_JPRB, 0.18708E+01_JPRB, 0.18661E+01_JPRB, 0.18867E+01_JPRB ,&
&  0.19059E+01_JPRB, 0.19239E+01_JPRB, 0.19141E+01_JPRB, 0.18884E+01_JPRB, 0.18613E+01_JPRB, 0.18234E+01_JPRB, 0.17515E+01_JPRB ,&
&  0.16846E+01_JPRB, 0.16235E+01_JPRB, 0.14393E+01_JPRB, 0.15061E+01_JPRB, 0.14075E+01_JPRB, 0.11967E+01_JPRB, 0.99166E+00_JPRB ,&
&  0.88431E+00_JPRB, 0.90058E+00_JPRB, 0.10975E+01_JPRB, 0.13020E+01_JPRB, 0.13701E+01_JPRB, 0.13750E+01_JPRB, 0.13298E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.10838E+01_JPRB, 0.10116E+01_JPRB, 0.99559E+00_JPRB, 0.10157E+01_JPRB, 0.10581E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.11009E+01_JPRB, 0.10965E+01_JPRB, 0.10954E+01_JPRB, 0.12017E+01_JPRB, 0.14255E+01_JPRB, 0.23570E+01_JPRB ,&
&  0.18348E+01_JPRB, 0.13240E+01_JPRB, 0.17487E+01_JPRB, 0.71011E+00_JPRB, 0.74756E+00_JPRB, 0.66963E+00_JPRB, 0.74962E+00_JPRB ,&
&  0.89115E+00_JPRB, 0.12345E+01_JPRB, 0.16456E+01_JPRB, 0.14485E+01_JPRB, 0.13353E+01_JPRB, 0.12383E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11251E+01_JPRB, 0.11310E+01_JPRB, 0.11352E+01_JPRB, 0.11526E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.11946E+01_JPRB, 0.12252E+01_JPRB, 0.12386E+01_JPRB, 0.12669E+01_JPRB, 0.12571E+01_JPRB, 0.12835E+01_JPRB, 0.12933E+01_JPRB ,&
&  0.13222E+01_JPRB, 0.13464E+01_JPRB, 0.13809E+01_JPRB, 0.14153E+01_JPRB, 0.14616E+01_JPRB, 0.15055E+01_JPRB, 0.15527E+01_JPRB ,&
&  0.16096E+01_JPRB, 0.16734E+01_JPRB, 0.17292E+01_JPRB, 0.17999E+01_JPRB, 0.18334E+01_JPRB, 0.18978E+01_JPRB, 0.19584E+01_JPRB ,&
&  0.20505E+01_JPRB, 0.20594E+01_JPRB, 0.20849E+01_JPRB, 0.21230E+01_JPRB, 0.21257E+01_JPRB, 0.22553E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.22912E+01_JPRB, 0.24875E+01_JPRB, 0.26053E+01_JPRB, 0.24778E+01_JPRB, 0.23183E+01_JPRB, 0.22761E+01_JPRB, 0.22845E+01_JPRB ,&
&  0.23349E+01_JPRB, 0.23590E+01_JPRB, 0.22514E+01_JPRB, 0.22833E+01_JPRB, 0.22887E+01_JPRB, 0.22220E+01_JPRB, 0.22673E+01_JPRB ,&
&  0.22975E+01_JPRB, 0.23254E+01_JPRB, 0.23462E+01_JPRB, 0.23631E+01_JPRB, 0.23662E+01_JPRB, 0.24081E+01_JPRB, 0.24233E+01_JPRB ,&
&  0.23939E+01_JPRB, 0.23854E+01_JPRB, 0.23657E+01_JPRB, 0.22291E+01_JPRB, 0.20798E+01_JPRB, 0.17564E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.10342E+01_JPRB, 0.11201E+01_JPRB, 0.12326E+01_JPRB, 0.12107E+01_JPRB, 0.11639E+01_JPRB, 0.10846E+01_JPRB ,&
&  0.98437E+00_JPRB, 0.96270E+00_JPRB, 0.94875E+00_JPRB, 0.96541E+00_JPRB, 0.99546E+00_JPRB, 0.10205E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10549E+01_JPRB, 0.10219E+01_JPRB, 0.99912E+00_JPRB, 0.10793E+01_JPRB, 0.13276E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.12885E+01_JPRB, 0.11169E+01_JPRB, 0.10050E+01_JPRB, 0.75634E+00_JPRB, 0.70832E+00_JPRB, 0.86146E+00_JPRB ,&
&  0.13779E+01_JPRB, 0.17346E+01_JPRB, 0.24265E+01_JPRB, 0.40677E+01_JPRB, 0.28915E+01_JPRB, 0.21817E+01_JPRB, 0.11942E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.11283E+01_JPRB, 0.11270E+01_JPRB, 0.11267E+01_JPRB, 0.11577E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.10838E+01_JPRB, 0.11220E+01_JPRB, 0.11630E+01_JPRB, 0.12036E+01_JPRB, 0.12292E+01_JPRB, 0.12432E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.12842E+01_JPRB, 0.12784E+01_JPRB, 0.13135E+01_JPRB, 0.13073E+01_JPRB, 0.13426E+01_JPRB, 0.13543E+01_JPRB, 0.13929E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.15170E+01_JPRB, 0.15962E+01_JPRB, 0.16626E+01_JPRB, 0.17268E+01_JPRB, 0.18396E+01_JPRB, 0.19833E+01_JPRB ,&
&  0.21444E+01_JPRB, 0.22446E+01_JPRB, 0.23288E+01_JPRB, 0.23853E+01_JPRB, 0.23471E+01_JPRB, 0.23003E+01_JPRB, 0.16415E+01_JPRB ,&
&  0.22358E+01_JPRB, 0.25785E+01_JPRB, 0.28735E+01_JPRB, 0.28645E+01_JPRB, 0.26571E+01_JPRB, 0.24899E+01_JPRB, 0.24853E+01_JPRB ,&
&  0.24705E+01_JPRB, 0.23731E+01_JPRB, 0.22350E+01_JPRB, 0.21912E+01_JPRB, 0.24158E+01_JPRB, 0.23336E+01_JPRB, 0.23814E+01_JPRB ,&
&  0.24245E+01_JPRB, 0.24819E+01_JPRB, 0.25565E+01_JPRB, 0.26515E+01_JPRB, 0.27421E+01_JPRB, 0.28344E+01_JPRB, 0.29399E+01_JPRB ,&
&  0.30130E+01_JPRB, 0.30297E+01_JPRB, 0.30473E+01_JPRB, 0.27178E+01_JPRB, 0.24959E+01_JPRB, 0.24706E+01_JPRB, 0.19201E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.11345E+01_JPRB, 0.11982E+01_JPRB, 0.12405E+01_JPRB, 0.11982E+01_JPRB, 0.10845E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.96077E+00_JPRB, 0.94157E+00_JPRB, 0.93515E+00_JPRB, 0.97054E+00_JPRB, 0.99804E+00_JPRB, 0.10262E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10367E+01_JPRB, 0.10003E+01_JPRB, 0.99390E+00_JPRB, 0.10886E+01_JPRB, 0.13553E+01_JPRB, 0.17753E+01_JPRB ,&
&  0.19218E+01_JPRB, 0.11318E+01_JPRB, 0.10542E+01_JPRB, 0.90863E+00_JPRB, 0.83423E+00_JPRB, 0.85322E+00_JPRB, 0.10236E+01_JPRB ,&
&  0.14994E+01_JPRB, 0.22694E+01_JPRB, 0.24893E+01_JPRB, 0.21243E+01_JPRB, 0.25734E+01_JPRB, 0.18815E+01_JPRB, 0.11001E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.10658E+01_JPRB, 0.10345E+01_JPRB, 0.10295E+01_JPRB, 0.10453E+01_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.90258E+00_JPRB, 0.92928E+00_JPRB, 0.96002E+00_JPRB, 0.10048E+01_JPRB, 0.10556E+01_JPRB, 0.10934E+01_JPRB, 0.11319E+01_JPRB ,&
&  0.11488E+01_JPRB, 0.11796E+01_JPRB, 0.12101E+01_JPRB, 0.11924E+01_JPRB, 0.12223E+01_JPRB, 0.12321E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.13311E+01_JPRB, 0.13964E+01_JPRB, 0.14761E+01_JPRB, 0.15859E+01_JPRB, 0.17163E+01_JPRB, 0.18756E+01_JPRB ,&
&  0.20681E+01_JPRB, 0.22432E+01_JPRB, 0.24145E+01_JPRB, 0.25309E+01_JPRB, 0.26185E+01_JPRB, 0.26621E+01_JPRB, 0.26836E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.24245E+01_JPRB, 0.27276E+01_JPRB, 0.29408E+01_JPRB, 0.30617E+01_JPRB, 0.28937E+01_JPRB, 0.28754E+01_JPRB ,&
&  0.26617E+01_JPRB, 0.23694E+01_JPRB, 0.22441E+01_JPRB, 0.21571E+01_JPRB, 0.22554E+01_JPRB, 0.22291E+01_JPRB, 0.22386E+01_JPRB ,&
&  0.22542E+01_JPRB, 0.23152E+01_JPRB, 0.23994E+01_JPRB, 0.24964E+01_JPRB, 0.26233E+01_JPRB, 0.27779E+01_JPRB, 0.29846E+01_JPRB ,&
&  0.31962E+01_JPRB, 0.33844E+01_JPRB, 0.35452E+01_JPRB, 0.31580E+01_JPRB, 0.27975E+01_JPRB, 0.28266E+01_JPRB, 0.24106E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.12006E+01_JPRB, 0.12638E+01_JPRB, 0.13751E+01_JPRB, 0.12957E+01_JPRB, 0.11910E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.99006E+00_JPRB, 0.10260E+01_JPRB, 0.10100E+01_JPRB, 0.10550E+01_JPRB, 0.10622E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.10974E+01_JPRB, 0.10825E+01_JPRB, 0.10816E+01_JPRB, 0.10763E+01_JPRB, 0.10734E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10332E+01_JPRB, 0.98205E+00_JPRB, 0.97682E+00_JPRB, 0.96151E+00_JPRB, 0.95453E+00_JPRB, 0.10408E+01_JPRB ,&
&  0.11947E+01_JPRB, 0.13736E+01_JPRB, 0.16625E+01_JPRB, 0.17331E+01_JPRB, 0.16400E+01_JPRB, 0.12605E+01_JPRB, 0.95581E+00_JPRB ,&
&  0.95612E+00_JPRB, 0.91666E+00_JPRB, 0.88609E+00_JPRB, 0.87034E+00_JPRB, 0.87960E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.81550E+00_JPRB, 0.83667E+00_JPRB, 0.83344E+00_JPRB, 0.86475E+00_JPRB, 0.88888E+00_JPRB, 0.92348E+00_JPRB, 0.96242E+00_JPRB ,&
&  0.99518E+00_JPRB, 0.10249E+01_JPRB, 0.10695E+01_JPRB, 0.10685E+01_JPRB, 0.11018E+01_JPRB, 0.11197E+01_JPRB, 0.11231E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.11803E+01_JPRB, 0.12184E+01_JPRB, 0.12651E+01_JPRB, 0.13343E+01_JPRB, 0.14454E+01_JPRB, 0.15727E+01_JPRB ,&
&  0.17304E+01_JPRB, 0.19222E+01_JPRB, 0.21712E+01_JPRB, 0.24225E+01_JPRB, 0.26570E+01_JPRB, 0.27831E+01_JPRB, 0.27877E+01_JPRB ,&
&  0.13099E+01_JPRB, 0.15341E+01_JPRB, 0.23935E+01_JPRB, 0.27384E+01_JPRB, 0.29886E+01_JPRB, 0.30587E+01_JPRB, 0.29928E+01_JPRB ,&
&  0.26865E+01_JPRB, 0.22339E+01_JPRB, 0.20427E+01_JPRB, 0.21996E+01_JPRB, 0.20915E+01_JPRB, 0.20882E+01_JPRB, 0.20721E+01_JPRB ,&
&  0.20676E+01_JPRB, 0.20961E+01_JPRB, 0.21499E+01_JPRB, 0.22169E+01_JPRB, 0.23415E+01_JPRB, 0.25139E+01_JPRB, 0.27583E+01_JPRB ,&
&  0.30639E+01_JPRB, 0.33786E+01_JPRB, 0.36568E+01_JPRB, 0.32183E+01_JPRB, 0.31418E+01_JPRB, 0.30932E+01_JPRB, 0.31692E+01_JPRB ,&
&  0.19073E+01_JPRB, 0.13408E+01_JPRB, 0.14032E+01_JPRB, 0.15165E+01_JPRB, 0.14002E+01_JPRB, 0.13268E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.11414E+01_JPRB, 0.11308E+01_JPRB, 0.11109E+01_JPRB, 0.11109E+01_JPRB, 0.11332E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.11879E+01_JPRB, 0.12054E+01_JPRB, 0.12076E+01_JPRB, 0.12059E+01_JPRB, 0.11770E+01_JPRB, 0.11454E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.11061E+01_JPRB, 0.10995E+01_JPRB, 0.11268E+01_JPRB, 0.11489E+01_JPRB, 0.11496E+01_JPRB, 0.11485E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.11223E+01_JPRB, 0.12364E+01_JPRB, 0.12705E+01_JPRB, 0.12397E+01_JPRB, 0.10515E+01_JPRB, 0.88387E+00_JPRB ,&
&  0.86651E+00_JPRB, 0.83975E+00_JPRB, 0.81617E+00_JPRB, 0.81159E+00_JPRB, 0.80788E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.74974E+00_JPRB, 0.73437E+00_JPRB, 0.72011E+00_JPRB, 0.73080E+00_JPRB, 0.74406E+00_JPRB, 0.76375E+00_JPRB, 0.82022E+00_JPRB ,&
&  0.86961E+00_JPRB, 0.88848E+00_JPRB, 0.91564E+00_JPRB, 0.92721E+00_JPRB, 0.95239E+00_JPRB, 0.96022E+00_JPRB, 0.98520E+00_JPRB ,&
&  0.99678E+00_JPRB, 0.10224E+01_JPRB, 0.10620E+01_JPRB, 0.11021E+01_JPRB, 0.11368E+01_JPRB, 0.11845E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.13326E+01_JPRB, 0.14324E+01_JPRB, 0.15937E+01_JPRB, 0.18707E+01_JPRB, 0.23087E+01_JPRB, 0.28406E+01_JPRB, 0.32688E+01_JPRB ,&
&  0.25450E+01_JPRB, 0.10106E+01_JPRB, 0.20208E+01_JPRB, 0.26355E+01_JPRB, 0.27409E+01_JPRB, 0.28786E+01_JPRB, 0.27387E+01_JPRB ,&
&  0.24452E+01_JPRB, 0.21474E+01_JPRB, 0.19250E+01_JPRB, 0.20442E+01_JPRB, 0.19878E+01_JPRB, 0.19672E+01_JPRB, 0.19534E+01_JPRB ,&
&  0.19260E+01_JPRB, 0.19184E+01_JPRB, 0.19230E+01_JPRB, 0.19610E+01_JPRB, 0.20520E+01_JPRB, 0.22102E+01_JPRB, 0.24326E+01_JPRB ,&
&  0.27468E+01_JPRB, 0.31803E+01_JPRB, 0.35916E+01_JPRB, 0.32968E+01_JPRB, 0.33733E+01_JPRB, 0.30263E+01_JPRB, 0.26579E+01_JPRB ,&
&  0.18216E+01_JPRB, 0.14511E+01_JPRB, 0.14845E+01_JPRB, 0.16333E+01_JPRB, 0.15162E+01_JPRB, 0.13412E+01_JPRB, 0.14033E+01_JPRB ,&
&  0.13323E+01_JPRB, 0.12786E+01_JPRB, 0.12450E+01_JPRB, 0.12138E+01_JPRB, 0.12098E+01_JPRB, 0.12142E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.12486E+01_JPRB, 0.12655E+01_JPRB, 0.12601E+01_JPRB, 0.12350E+01_JPRB, 0.12232E+01_JPRB, 0.12343E+01_JPRB ,&
&  0.12507E+01_JPRB, 0.12504E+01_JPRB, 0.12693E+01_JPRB, 0.12873E+01_JPRB, 0.12953E+01_JPRB, 0.12551E+01_JPRB, 0.12390E+01_JPRB ,&
&  0.12082E+01_JPRB, 0.11865E+01_JPRB, 0.11570E+01_JPRB, 0.11141E+01_JPRB, 0.10782E+01_JPRB, 0.99352E+00_JPRB, 0.89620E+00_JPRB ,&
&  0.86349E+00_JPRB, 0.82766E+00_JPRB, 0.80341E+00_JPRB, 0.78708E+00_JPRB, 0.77019E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.80290E+00_JPRB, 0.78295E+00_JPRB, 0.75023E+00_JPRB, 0.70469E+00_JPRB, 0.70127E+00_JPRB, 0.69956E+00_JPRB, 0.71150E+00_JPRB ,&
&  0.75877E+00_JPRB, 0.81707E+00_JPRB, 0.83191E+00_JPRB, 0.83108E+00_JPRB, 0.81062E+00_JPRB, 0.81735E+00_JPRB, 0.83216E+00_JPRB ,&
&  0.85726E+00_JPRB, 0.88079E+00_JPRB, 0.91667E+00_JPRB, 0.96426E+00_JPRB, 0.10086E+01_JPRB, 0.10459E+01_JPRB, 0.10763E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.11207E+01_JPRB, 0.11584E+01_JPRB, 0.12298E+01_JPRB, 0.13937E+01_JPRB, 0.17648E+01_JPRB, 0.24267E+01_JPRB ,&
&  0.39263E+01_JPRB, 0.32995E+01_JPRB, 0.14415E+01_JPRB, 0.24586E+01_JPRB, 0.27173E+01_JPRB, 0.29197E+01_JPRB, 0.24603E+01_JPRB ,&
&  0.23245E+01_JPRB, 0.20218E+01_JPRB, 0.17790E+01_JPRB, 0.18666E+01_JPRB, 0.18058E+01_JPRB, 0.17737E+01_JPRB, 0.17547E+01_JPRB ,&
&  0.17078E+01_JPRB, 0.17048E+01_JPRB, 0.17094E+01_JPRB, 0.17248E+01_JPRB, 0.17626E+01_JPRB, 0.18708E+01_JPRB, 0.20916E+01_JPRB ,&
&  0.23829E+01_JPRB, 0.27743E+01_JPRB, 0.33448E+01_JPRB, 0.35421E+01_JPRB, 0.38152E+01_JPRB, 0.29514E+01_JPRB, 0.23842E+01_JPRB ,&
&  0.17313E+01_JPRB, 0.16486E+01_JPRB, 0.17385E+01_JPRB, 0.17524E+01_JPRB, 0.16200E+01_JPRB, 0.14877E+01_JPRB, 0.15548E+01_JPRB ,&
&  0.14644E+01_JPRB, 0.13983E+01_JPRB, 0.13627E+01_JPRB, 0.13198E+01_JPRB, 0.12912E+01_JPRB, 0.12795E+01_JPRB, 0.12735E+01_JPRB ,&
&  0.12684E+01_JPRB, 0.12821E+01_JPRB, 0.12956E+01_JPRB, 0.13034E+01_JPRB, 0.13049E+01_JPRB, 0.12962E+01_JPRB, 0.13111E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.13466E+01_JPRB, 0.13697E+01_JPRB, 0.13912E+01_JPRB, 0.13584E+01_JPRB, 0.12996E+01_JPRB, 0.12997E+01_JPRB ,&
&  0.12808E+01_JPRB, 0.12720E+01_JPRB, 0.12256E+01_JPRB, 0.12187E+01_JPRB, 0.11379E+01_JPRB, 0.10419E+01_JPRB, 0.94873E+00_JPRB ,&
&  0.93048E+00_JPRB, 0.91237E+00_JPRB, 0.86438E+00_JPRB, 0.81476E+00_JPRB, 0.80182E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.86816E+00_JPRB, 0.87590E+00_JPRB, 0.86769E+00_JPRB, 0.82162E+00_JPRB, 0.77964E+00_JPRB, 0.73966E+00_JPRB, 0.71157E+00_JPRB ,&
&  0.70699E+00_JPRB, 0.71342E+00_JPRB, 0.74372E+00_JPRB, 0.76902E+00_JPRB, 0.76107E+00_JPRB, 0.73704E+00_JPRB, 0.73075E+00_JPRB ,&
&  0.74197E+00_JPRB, 0.77029E+00_JPRB, 0.79824E+00_JPRB, 0.84259E+00_JPRB, 0.88433E+00_JPRB, 0.91734E+00_JPRB, 0.95279E+00_JPRB ,&
&  0.95093E+00_JPRB, 0.95331E+00_JPRB, 0.96537E+00_JPRB, 0.98479E+00_JPRB, 0.10246E+01_JPRB, 0.11297E+01_JPRB, 0.12992E+01_JPRB ,&
&  0.18550E+01_JPRB, 0.33740E+01_JPRB, 0.18590E+01_JPRB, 0.26116E+01_JPRB, 0.29990E+01_JPRB, 0.29953E+01_JPRB, 0.25898E+01_JPRB ,&
&  0.23182E+01_JPRB, 0.19946E+01_JPRB, 0.17958E+01_JPRB, 0.17576E+01_JPRB, 0.15947E+01_JPRB, 0.15532E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.15131E+01_JPRB, 0.15288E+01_JPRB, 0.15395E+01_JPRB, 0.15514E+01_JPRB, 0.15459E+01_JPRB, 0.15645E+01_JPRB, 0.16739E+01_JPRB ,&
&  0.19241E+01_JPRB, 0.22815E+01_JPRB, 0.28916E+01_JPRB, 0.33749E+01_JPRB, 0.40020E+01_JPRB, 0.32035E+01_JPRB, 0.24835E+01_JPRB ,&
&  0.19372E+01_JPRB, 0.19114E+01_JPRB, 0.19529E+01_JPRB, 0.19379E+01_JPRB, 0.17600E+01_JPRB, 0.17293E+01_JPRB, 0.17886E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.15380E+01_JPRB, 0.14800E+01_JPRB, 0.14332E+01_JPRB, 0.13781E+01_JPRB, 0.13369E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.12956E+01_JPRB, 0.12961E+01_JPRB, 0.13119E+01_JPRB, 0.13240E+01_JPRB, 0.13402E+01_JPRB, 0.13456E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.13605E+01_JPRB, 0.13788E+01_JPRB, 0.13827E+01_JPRB, 0.13636E+01_JPRB, 0.13302E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.13329E+01_JPRB, 0.13588E+01_JPRB, 0.12888E+01_JPRB, 0.12507E+01_JPRB, 0.12602E+01_JPRB, 0.11774E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.12140E+01_JPRB, 0.12450E+01_JPRB, 0.89129E+00_JPRB, 0.86874E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.90821E+00_JPRB, 0.90710E+00_JPRB, 0.89273E+00_JPRB, 0.88724E+00_JPRB, 0.85084E+00_JPRB, 0.80969E+00_JPRB, 0.77273E+00_JPRB ,&
&  0.74801E+00_JPRB, 0.73397E+00_JPRB, 0.73452E+00_JPRB, 0.73189E+00_JPRB, 0.73207E+00_JPRB, 0.72379E+00_JPRB, 0.71860E+00_JPRB ,&
&  0.70716E+00_JPRB, 0.71555E+00_JPRB, 0.73641E+00_JPRB, 0.77266E+00_JPRB, 0.81385E+00_JPRB, 0.82657E+00_JPRB, 0.83973E+00_JPRB ,&
&  0.84827E+00_JPRB, 0.84852E+00_JPRB, 0.85196E+00_JPRB, 0.86457E+00_JPRB, 0.89013E+00_JPRB, 0.93006E+00_JPRB, 0.10085E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.27747E+01_JPRB, 0.42566E+01_JPRB, 0.47609E+01_JPRB, 0.40847E+01_JPRB, 0.33987E+01_JPRB, 0.29755E+01_JPRB ,&
&  0.28763E+01_JPRB, 0.26126E+01_JPRB, 0.22582E+01_JPRB, 0.18371E+01_JPRB, 0.15494E+01_JPRB, 0.14384E+01_JPRB, 0.13998E+01_JPRB ,&
&  0.14367E+01_JPRB, 0.14564E+01_JPRB, 0.14597E+01_JPRB, 0.14527E+01_JPRB, 0.14225E+01_JPRB, 0.14007E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.14918E+01_JPRB, 0.17179E+01_JPRB, 0.21573E+01_JPRB, 0.29061E+01_JPRB, 0.37292E+01_JPRB, 0.33755E+01_JPRB, 0.28090E+01_JPRB ,&
&  0.26420E+01_JPRB, 0.24750E+01_JPRB, 0.23102E+01_JPRB, 0.22116E+01_JPRB, 0.18512E+01_JPRB, 0.21511E+01_JPRB, 0.20631E+01_JPRB ,&
&  0.19504E+01_JPRB, 0.18695E+01_JPRB, 0.17456E+01_JPRB, 0.16132E+01_JPRB, 0.15099E+01_JPRB, 0.14105E+01_JPRB, 0.13345E+01_JPRB ,&
&  0.13060E+01_JPRB, 0.13091E+01_JPRB, 0.13206E+01_JPRB, 0.13428E+01_JPRB, 0.13721E+01_JPRB, 0.13917E+01_JPRB, 0.13891E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.13374E+01_JPRB, 0.12824E+01_JPRB, 0.12825E+01_JPRB, 0.12807E+01_JPRB, 0.12493E+01_JPRB, 0.12275E+01_JPRB ,&
&  0.12511E+01_JPRB, 0.12992E+01_JPRB, 0.12956E+01_JPRB, 0.12800E+01_JPRB, 0.12597E+01_JPRB, 0.12432E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.11203E+01_JPRB, 0.10485E+01_JPRB, 0.96417E+00_JPRB, 0.93131E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.95128E+00_JPRB, 0.92029E+00_JPRB, 0.90104E+00_JPRB, 0.89892E+00_JPRB, 0.87577E+00_JPRB, 0.85897E+00_JPRB, 0.85039E+00_JPRB ,&
&  0.80715E+00_JPRB, 0.77450E+00_JPRB, 0.74655E+00_JPRB, 0.73528E+00_JPRB, 0.73446E+00_JPRB, 0.72859E+00_JPRB, 0.73250E+00_JPRB ,&
&  0.72532E+00_JPRB, 0.71448E+00_JPRB, 0.71700E+00_JPRB, 0.74142E+00_JPRB, 0.77207E+00_JPRB, 0.77866E+00_JPRB, 0.79564E+00_JPRB ,&
&  0.81498E+00_JPRB, 0.81488E+00_JPRB, 0.80674E+00_JPRB, 0.80694E+00_JPRB, 0.81441E+00_JPRB, 0.82072E+00_JPRB, 0.86563E+00_JPRB ,&
&  0.10804E+01_JPRB, 0.28197E+01_JPRB, 0.28723E+01_JPRB, 0.54313E+01_JPRB, 0.46764E+01_JPRB, 0.39672E+01_JPRB, 0.31091E+01_JPRB ,&
&  0.31447E+01_JPRB, 0.34500E+01_JPRB, 0.26127E+01_JPRB, 0.20695E+01_JPRB, 0.17069E+01_JPRB, 0.14692E+01_JPRB, 0.13818E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.14226E+01_JPRB, 0.14421E+01_JPRB, 0.14039E+01_JPRB, 0.13581E+01_JPRB, 0.13342E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.13304E+01_JPRB, 0.13811E+01_JPRB, 0.15859E+01_JPRB, 0.20102E+01_JPRB, 0.30335E+01_JPRB, 0.35262E+01_JPRB, 0.32297E+01_JPRB ,&
&  0.31175E+01_JPRB, 0.32555E+01_JPRB, 0.29924E+01_JPRB, 0.26974E+01_JPRB, 0.23238E+01_JPRB, 0.23233E+01_JPRB, 0.22298E+01_JPRB ,&
&  0.21348E+01_JPRB, 0.21093E+01_JPRB, 0.20485E+01_JPRB, 0.19234E+01_JPRB, 0.17752E+01_JPRB, 0.16170E+01_JPRB, 0.14682E+01_JPRB ,&
&  0.13648E+01_JPRB, 0.13155E+01_JPRB, 0.12916E+01_JPRB, 0.13106E+01_JPRB, 0.13578E+01_JPRB, 0.13957E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.13898E+01_JPRB, 0.13561E+01_JPRB, 0.12534E+01_JPRB, 0.11971E+01_JPRB, 0.12008E+01_JPRB, 0.11734E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.12061E+01_JPRB, 0.12481E+01_JPRB, 0.12546E+01_JPRB, 0.12437E+01_JPRB, 0.12432E+01_JPRB, 0.12775E+01_JPRB, 0.11728E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11184E+01_JPRB, 0.11112E+01_JPRB, 0.10406E+01_JPRB, 0.99729E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.10142E+01_JPRB, 0.96363E+00_JPRB, 0.92429E+00_JPRB, 0.90613E+00_JPRB, 0.89710E+00_JPRB, 0.88545E+00_JPRB, 0.87342E+00_JPRB ,&
&  0.84659E+00_JPRB, 0.81330E+00_JPRB, 0.78319E+00_JPRB, 0.75700E+00_JPRB, 0.75257E+00_JPRB, 0.74658E+00_JPRB, 0.74181E+00_JPRB ,&
&  0.73352E+00_JPRB, 0.72505E+00_JPRB, 0.71971E+00_JPRB, 0.72913E+00_JPRB, 0.74277E+00_JPRB, 0.75330E+00_JPRB, 0.77122E+00_JPRB ,&
&  0.78036E+00_JPRB, 0.79121E+00_JPRB, 0.79411E+00_JPRB, 0.79003E+00_JPRB, 0.78220E+00_JPRB, 0.77444E+00_JPRB, 0.78275E+00_JPRB ,&
&  0.10520E+01_JPRB, 0.35459E+01_JPRB, 0.26003E+01_JPRB, 0.49764E+01_JPRB, 0.43122E+01_JPRB, 0.38813E+01_JPRB, 0.31012E+01_JPRB ,&
&  0.29398E+01_JPRB, 0.32000E+01_JPRB, 0.27040E+01_JPRB, 0.22536E+01_JPRB, 0.19509E+01_JPRB, 0.17089E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.14662E+01_JPRB, 0.14356E+01_JPRB, 0.14003E+01_JPRB, 0.13540E+01_JPRB, 0.13303E+01_JPRB, 0.13137E+01_JPRB, 0.13133E+01_JPRB ,&
&  0.13088E+01_JPRB, 0.13109E+01_JPRB, 0.13737E+01_JPRB, 0.16516E+01_JPRB, 0.22311E+01_JPRB, 0.30884E+01_JPRB, 0.36601E+01_JPRB ,&
&  0.37699E+01_JPRB, 0.41216E+01_JPRB, 0.37166E+01_JPRB, 0.32348E+01_JPRB, 0.28750E+01_JPRB, 0.26547E+01_JPRB, 0.24633E+01_JPRB ,&
&  0.22724E+01_JPRB, 0.21595E+01_JPRB, 0.21248E+01_JPRB, 0.20953E+01_JPRB, 0.20330E+01_JPRB, 0.19132E+01_JPRB, 0.17557E+01_JPRB ,&
&  0.15938E+01_JPRB, 0.14790E+01_JPRB, 0.13960E+01_JPRB, 0.13557E+01_JPRB, 0.13501E+01_JPRB, 0.13684E+01_JPRB, 0.13838E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.13352E+01_JPRB, 0.12631E+01_JPRB, 0.11869E+01_JPRB, 0.11818E+01_JPRB, 0.11596E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.12156E+01_JPRB, 0.12103E+01_JPRB, 0.12260E+01_JPRB, 0.12152E+01_JPRB, 0.13027E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.11309E+01_JPRB, 0.11136E+01_JPRB, 0.11024E+01_JPRB, 0.10770E+01_JPRB, 0.10563E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.10638E+01_JPRB, 0.10260E+01_JPRB, 0.98561E+00_JPRB, 0.95156E+00_JPRB, 0.92478E+00_JPRB, 0.90928E+00_JPRB, 0.89492E+00_JPRB ,&
&  0.86840E+00_JPRB, 0.84007E+00_JPRB, 0.81899E+00_JPRB, 0.80036E+00_JPRB, 0.78650E+00_JPRB, 0.77849E+00_JPRB, 0.76387E+00_JPRB ,&
&  0.74823E+00_JPRB, 0.72613E+00_JPRB, 0.73002E+00_JPRB, 0.74181E+00_JPRB, 0.74875E+00_JPRB, 0.75534E+00_JPRB, 0.77691E+00_JPRB ,&
&  0.77405E+00_JPRB, 0.77519E+00_JPRB, 0.77903E+00_JPRB, 0.78064E+00_JPRB, 0.77026E+00_JPRB, 0.77059E+00_JPRB, 0.76464E+00_JPRB ,&
&  0.99986E+00_JPRB, 0.13707E+01_JPRB, 0.30927E+01_JPRB, 0.43096E+01_JPRB, 0.39072E+01_JPRB, 0.34913E+01_JPRB, 0.31539E+01_JPRB ,&
&  0.30463E+01_JPRB, 0.29178E+01_JPRB, 0.25414E+01_JPRB, 0.22590E+01_JPRB, 0.20182E+01_JPRB, 0.18368E+01_JPRB, 0.16998E+01_JPRB ,&
&  0.15925E+01_JPRB, 0.14911E+01_JPRB, 0.14138E+01_JPRB, 0.13440E+01_JPRB, 0.13006E+01_JPRB, 0.12799E+01_JPRB, 0.12784E+01_JPRB ,&
&  0.12769E+01_JPRB, 0.12696E+01_JPRB, 0.12823E+01_JPRB, 0.14236E+01_JPRB, 0.19166E+01_JPRB, 0.26355E+01_JPRB, 0.34730E+01_JPRB ,&
&  0.41885E+01_JPRB, 0.44468E+01_JPRB, 0.40674E+01_JPRB, 0.36212E+01_JPRB, 0.32058E+01_JPRB, 0.28926E+01_JPRB, 0.26519E+01_JPRB ,&
&  0.24303E+01_JPRB, 0.22431E+01_JPRB, 0.20979E+01_JPRB, 0.20193E+01_JPRB, 0.19677E+01_JPRB, 0.19231E+01_JPRB, 0.18652E+01_JPRB ,&
&  0.17836E+01_JPRB, 0.16742E+01_JPRB, 0.15609E+01_JPRB, 0.14665E+01_JPRB, 0.14006E+01_JPRB, 0.13756E+01_JPRB, 0.13609E+01_JPRB ,&
&  0.13173E+01_JPRB, 0.12852E+01_JPRB, 0.12441E+01_JPRB, 0.12003E+01_JPRB, 0.12121E+01_JPRB, 0.12063E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.12216E+01_JPRB, 0.12026E+01_JPRB, 0.12287E+01_JPRB, 0.12091E+01_JPRB, 0.12802E+01_JPRB, 0.12067E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.11305E+01_JPRB, 0.11055E+01_JPRB, 0.10893E+01_JPRB, 0.10832E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.10997E+01_JPRB, 0.10791E+01_JPRB, 0.10343E+01_JPRB, 0.98955E+00_JPRB, 0.95616E+00_JPRB, 0.93131E+00_JPRB, 0.91012E+00_JPRB ,&
&  0.88618E+00_JPRB, 0.85737E+00_JPRB, 0.83404E+00_JPRB, 0.82138E+00_JPRB, 0.81719E+00_JPRB, 0.81216E+00_JPRB, 0.79477E+00_JPRB ,&
&  0.77280E+00_JPRB, 0.75469E+00_JPRB, 0.75424E+00_JPRB, 0.76100E+00_JPRB, 0.76993E+00_JPRB, 0.76178E+00_JPRB, 0.77343E+00_JPRB ,&
&  0.78031E+00_JPRB, 0.77897E+00_JPRB, 0.77669E+00_JPRB, 0.77344E+00_JPRB, 0.76799E+00_JPRB, 0.76433E+00_JPRB, 0.75649E+00_JPRB ,&
&  0.91666E+00_JPRB, 0.19435E+01_JPRB, 0.30834E+01_JPRB, 0.37563E+01_JPRB, 0.36729E+01_JPRB, 0.31872E+01_JPRB, 0.28966E+01_JPRB ,&
&  0.26429E+01_JPRB, 0.25167E+01_JPRB, 0.22650E+01_JPRB, 0.20818E+01_JPRB, 0.19092E+01_JPRB, 0.17619E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.15442E+01_JPRB, 0.14686E+01_JPRB, 0.13973E+01_JPRB, 0.13370E+01_JPRB, 0.12933E+01_JPRB, 0.12589E+01_JPRB, 0.12421E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.12260E+01_JPRB, 0.12253E+01_JPRB, 0.12821E+01_JPRB, 0.16470E+01_JPRB, 0.19131E+01_JPRB, 0.29251E+01_JPRB ,&
&  0.37776E+01_JPRB, 0.35623E+01_JPRB, 0.33958E+01_JPRB, 0.31456E+01_JPRB, 0.28456E+01_JPRB, 0.26011E+01_JPRB, 0.23461E+01_JPRB ,&
&  0.21234E+01_JPRB, 0.19460E+01_JPRB, 0.18029E+01_JPRB, 0.17049E+01_JPRB, 0.16410E+01_JPRB, 0.16034E+01_JPRB, 0.15881E+01_JPRB ,&
&  0.15741E+01_JPRB, 0.15596E+01_JPRB, 0.15341E+01_JPRB, 0.14925E+01_JPRB, 0.14301E+01_JPRB, 0.13665E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.12232E+01_JPRB, 0.12081E+01_JPRB, 0.11810E+01_JPRB, 0.12155E+01_JPRB, 0.12533E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.12208E+01_JPRB, 0.12337E+01_JPRB, 0.12243E+01_JPRB, 0.12059E+01_JPRB, 0.12326E+01_JPRB, 0.13128E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.11569E+01_JPRB, 0.11311E+01_JPRB, 0.11032E+01_JPRB, 0.11084E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.11371E+01_JPRB, 0.10915E+01_JPRB, 0.10477E+01_JPRB, 0.10032E+01_JPRB, 0.97163E+00_JPRB, 0.93843E+00_JPRB, 0.90696E+00_JPRB ,&
&  0.88068E+00_JPRB, 0.86497E+00_JPRB, 0.85148E+00_JPRB, 0.83435E+00_JPRB, 0.83166E+00_JPRB, 0.82433E+00_JPRB, 0.80812E+00_JPRB ,&
&  0.78503E+00_JPRB, 0.76846E+00_JPRB, 0.76215E+00_JPRB, 0.76334E+00_JPRB, 0.77360E+00_JPRB, 0.77772E+00_JPRB, 0.78513E+00_JPRB ,&
&  0.79123E+00_JPRB, 0.77388E+00_JPRB, 0.76301E+00_JPRB, 0.76267E+00_JPRB, 0.76193E+00_JPRB, 0.76384E+00_JPRB, 0.76274E+00_JPRB ,&
&  0.87264E+00_JPRB, 0.14608E+01_JPRB, 0.21253E+01_JPRB, 0.29789E+01_JPRB, 0.31001E+01_JPRB, 0.28251E+01_JPRB, 0.25622E+01_JPRB ,&
&  0.23448E+01_JPRB, 0.21835E+01_JPRB, 0.19867E+01_JPRB, 0.17617E+01_JPRB, 0.15854E+01_JPRB, 0.14640E+01_JPRB, 0.13773E+01_JPRB ,&
&  0.13259E+01_JPRB, 0.12872E+01_JPRB, 0.12505E+01_JPRB, 0.12186E+01_JPRB, 0.11898E+01_JPRB, 0.11651E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.11346E+01_JPRB, 0.11319E+01_JPRB, 0.11486E+01_JPRB, 0.13329E+01_JPRB, 0.15166E+01_JPRB, 0.19569E+01_JPRB ,&
&  0.21414E+01_JPRB, 0.22029E+01_JPRB, 0.22605E+01_JPRB, 0.22252E+01_JPRB, 0.21127E+01_JPRB, 0.20020E+01_JPRB, 0.18871E+01_JPRB ,&
&  0.17588E+01_JPRB, 0.16250E+01_JPRB, 0.15342E+01_JPRB, 0.14692E+01_JPRB, 0.14087E+01_JPRB, 0.13627E+01_JPRB, 0.13328E+01_JPRB ,&
&  0.13261E+01_JPRB, 0.13434E+01_JPRB, 0.13606E+01_JPRB, 0.13686E+01_JPRB, 0.13501E+01_JPRB, 0.13169E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.11680E+01_JPRB, 0.11565E+01_JPRB, 0.11655E+01_JPRB, 0.11887E+01_JPRB, 0.11983E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.12256E+01_JPRB, 0.12577E+01_JPRB, 0.12414E+01_JPRB, 0.12398E+01_JPRB, 0.12647E+01_JPRB, 0.13320E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.11928E+01_JPRB, 0.11628E+01_JPRB, 0.11334E+01_JPRB, 0.11684E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.12168E+01_JPRB, 0.11410E+01_JPRB, 0.10804E+01_JPRB, 0.10352E+01_JPRB, 0.10041E+01_JPRB, 0.96666E+00_JPRB, 0.92394E+00_JPRB ,&
&  0.89254E+00_JPRB, 0.87999E+00_JPRB, 0.86711E+00_JPRB, 0.84504E+00_JPRB, 0.82812E+00_JPRB, 0.81137E+00_JPRB, 0.79712E+00_JPRB ,&
&  0.77785E+00_JPRB, 0.77028E+00_JPRB, 0.76397E+00_JPRB, 0.76566E+00_JPRB, 0.76475E+00_JPRB, 0.76761E+00_JPRB, 0.77778E+00_JPRB ,&
&  0.78135E+00_JPRB, 0.76311E+00_JPRB, 0.75964E+00_JPRB, 0.75816E+00_JPRB, 0.75565E+00_JPRB, 0.75441E+00_JPRB, 0.75254E+00_JPRB ,&
&  0.86729E+00_JPRB, 0.11052E+01_JPRB, 0.15013E+01_JPRB, 0.19369E+01_JPRB, 0.21402E+01_JPRB, 0.22152E+01_JPRB, 0.21286E+01_JPRB ,&
&  0.19905E+01_JPRB, 0.18337E+01_JPRB, 0.16668E+01_JPRB, 0.15098E+01_JPRB, 0.13628E+01_JPRB, 0.12565E+01_JPRB, 0.11879E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11193E+01_JPRB, 0.10995E+01_JPRB, 0.10837E+01_JPRB, 0.10681E+01_JPRB, 0.10491E+01_JPRB, 0.10393E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.10254E+01_JPRB, 0.10216E+01_JPRB, 0.10381E+01_JPRB, 0.10770E+01_JPRB, 0.11522E+01_JPRB, 0.13575E+01_JPRB ,&
&  0.14811E+01_JPRB, 0.15641E+01_JPRB, 0.16489E+01_JPRB, 0.16686E+01_JPRB, 0.16225E+01_JPRB, 0.15595E+01_JPRB, 0.14753E+01_JPRB ,&
&  0.14104E+01_JPRB, 0.13611E+01_JPRB, 0.13135E+01_JPRB, 0.12680E+01_JPRB, 0.12302E+01_JPRB, 0.12039E+01_JPRB, 0.11859E+01_JPRB ,&
&  0.11727E+01_JPRB, 0.11784E+01_JPRB, 0.11919E+01_JPRB, 0.12038E+01_JPRB, 0.12067E+01_JPRB, 0.12040E+01_JPRB, 0.11945E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.11234E+01_JPRB, 0.11026E+01_JPRB, 0.11113E+01_JPRB, 0.11432E+01_JPRB, 0.11598E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.11807E+01_JPRB, 0.12041E+01_JPRB, 0.11974E+01_JPRB, 0.12671E+01_JPRB, 0.13466E+01_JPRB, 0.13193E+01_JPRB, 0.12804E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.12062E+01_JPRB, 0.11719E+01_JPRB, 0.11380E+01_JPRB, 0.12088E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.10504E+01_JPRB, 0.10439E+01_JPRB, 0.10166E+01_JPRB, 0.99285E+00_JPRB, 0.97947E+00_JPRB, 0.96538E+00_JPRB, 0.94096E+00_JPRB ,&
&  0.91379E+00_JPRB, 0.88869E+00_JPRB, 0.87244E+00_JPRB, 0.85016E+00_JPRB, 0.82768E+00_JPRB, 0.81245E+00_JPRB, 0.79773E+00_JPRB ,&
&  0.77918E+00_JPRB, 0.77189E+00_JPRB, 0.76945E+00_JPRB, 0.76603E+00_JPRB, 0.76049E+00_JPRB, 0.75673E+00_JPRB, 0.76239E+00_JPRB ,&
&  0.76289E+00_JPRB, 0.76174E+00_JPRB, 0.76169E+00_JPRB, 0.75576E+00_JPRB, 0.74812E+00_JPRB, 0.74562E+00_JPRB, 0.73402E+00_JPRB ,&
&  0.77650E+00_JPRB, 0.84725E+00_JPRB, 0.10374E+01_JPRB, 0.12771E+01_JPRB, 0.14873E+01_JPRB, 0.16398E+01_JPRB, 0.17157E+01_JPRB ,&
&  0.16569E+01_JPRB, 0.15250E+01_JPRB, 0.13959E+01_JPRB, 0.13128E+01_JPRB, 0.12356E+01_JPRB, 0.11713E+01_JPRB, 0.11162E+01_JPRB ,&
&  0.10791E+01_JPRB, 0.10553E+01_JPRB, 0.10355E+01_JPRB, 0.10164E+01_JPRB, 0.99622E+00_JPRB, 0.97786E+00_JPRB, 0.96352E+00_JPRB ,&
&  0.95005E+00_JPRB, 0.94429E+00_JPRB, 0.94334E+00_JPRB, 0.93967E+00_JPRB, 0.94870E+00_JPRB, 0.99558E+00_JPRB, 0.10893E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.12057E+01_JPRB, 0.12857E+01_JPRB, 0.13204E+01_JPRB, 0.12953E+01_JPRB, 0.12595E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.11768E+01_JPRB, 0.11353E+01_JPRB, 0.11117E+01_JPRB, 0.10872E+01_JPRB, 0.10606E+01_JPRB, 0.10401E+01_JPRB, 0.10306E+01_JPRB ,&
&  0.10289E+01_JPRB, 0.10293E+01_JPRB, 0.10384E+01_JPRB, 0.10512E+01_JPRB, 0.10568E+01_JPRB, 0.10606E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.10671E+01_JPRB, 0.10493E+01_JPRB, 0.10568E+01_JPRB, 0.10830E+01_JPRB, 0.11028E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.11270E+01_JPRB, 0.11440E+01_JPRB, 0.11613E+01_JPRB, 0.12188E+01_JPRB, 0.12278E+01_JPRB, 0.12002E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.11535E+01_JPRB, 0.11299E+01_JPRB, 0.11068E+01_JPRB, 0.10822E+01_JPRB, 0.99972E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.98333E+00_JPRB, 0.97299E+00_JPRB, 0.95206E+00_JPRB, 0.93159E+00_JPRB, 0.92258E+00_JPRB, 0.91945E+00_JPRB, 0.91733E+00_JPRB ,&
&  0.90859E+00_JPRB, 0.89590E+00_JPRB, 0.88131E+00_JPRB, 0.86437E+00_JPRB, 0.84562E+00_JPRB, 0.82905E+00_JPRB, 0.81370E+00_JPRB ,&
&  0.79150E+00_JPRB, 0.77794E+00_JPRB, 0.77549E+00_JPRB, 0.77219E+00_JPRB, 0.76940E+00_JPRB, 0.76439E+00_JPRB, 0.76343E+00_JPRB ,&
&  0.76542E+00_JPRB, 0.76637E+00_JPRB, 0.76632E+00_JPRB, 0.75850E+00_JPRB, 0.75180E+00_JPRB, 0.75068E+00_JPRB, 0.74455E+00_JPRB ,&
&  0.75139E+00_JPRB, 0.69082E+00_JPRB, 0.83289E+00_JPRB, 0.98513E+00_JPRB, 0.10881E+01_JPRB, 0.12188E+01_JPRB, 0.13473E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.13235E+01_JPRB, 0.12432E+01_JPRB, 0.11765E+01_JPRB, 0.11173E+01_JPRB, 0.10736E+01_JPRB, 0.10398E+01_JPRB ,&
&  0.10168E+01_JPRB, 0.99947E+00_JPRB, 0.98047E+00_JPRB, 0.95659E+00_JPRB, 0.93108E+00_JPRB, 0.91295E+00_JPRB, 0.89903E+00_JPRB ,&
&  0.88908E+00_JPRB, 0.88460E+00_JPRB, 0.88026E+00_JPRB, 0.88292E+00_JPRB, 0.88516E+00_JPRB, 0.90056E+00_JPRB, 0.94507E+00_JPRB ,&
&  0.97251E+00_JPRB, 0.10064E+01_JPRB, 0.10590E+01_JPRB, 0.10762E+01_JPRB, 0.10635E+01_JPRB, 0.10474E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.99915E+00_JPRB, 0.96267E+00_JPRB, 0.94413E+00_JPRB, 0.92757E+00_JPRB, 0.91483E+00_JPRB, 0.90860E+00_JPRB, 0.91502E+00_JPRB ,&
&  0.92237E+00_JPRB, 0.92442E+00_JPRB, 0.92937E+00_JPRB, 0.93830E+00_JPRB, 0.94395E+00_JPRB, 0.94472E+00_JPRB, 0.95363E+00_JPRB ,&
&  0.97341E+00_JPRB, 0.99643E+00_JPRB, 0.10052E+01_JPRB, 0.10034E+01_JPRB, 0.10096E+01_JPRB, 0.10272E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10535E+01_JPRB, 0.10624E+01_JPRB, 0.10737E+01_JPRB, 0.10859E+01_JPRB, 0.11444E+01_JPRB, 0.11278E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10535E+01_JPRB, 0.10410E+01_JPRB, 0.90594E+00_JPRB, 0.95711E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.90419E+00_JPRB, 0.89594E+00_JPRB, 0.87954E+00_JPRB, 0.86220E+00_JPRB, 0.85001E+00_JPRB, 0.84533E+00_JPRB, 0.84378E+00_JPRB ,&
&  0.84765E+00_JPRB, 0.85061E+00_JPRB, 0.84995E+00_JPRB, 0.84518E+00_JPRB, 0.83773E+00_JPRB, 0.82616E+00_JPRB, 0.81565E+00_JPRB ,&
&  0.79978E+00_JPRB, 0.78341E+00_JPRB, 0.77690E+00_JPRB, 0.77627E+00_JPRB, 0.77399E+00_JPRB, 0.76627E+00_JPRB, 0.76066E+00_JPRB ,&
&  0.75742E+00_JPRB, 0.75832E+00_JPRB, 0.75080E+00_JPRB, 0.74551E+00_JPRB, 0.73942E+00_JPRB, 0.74066E+00_JPRB, 0.74046E+00_JPRB ,&
&  0.73972E+00_JPRB, 0.63127E+00_JPRB, 0.76220E+00_JPRB, 0.86042E+00_JPRB, 0.90341E+00_JPRB, 0.98792E+00_JPRB, 0.10676E+01_JPRB ,&
&  0.11188E+01_JPRB, 0.11372E+01_JPRB, 0.11213E+01_JPRB, 0.10894E+01_JPRB, 0.10450E+01_JPRB, 0.10088E+01_JPRB, 0.98096E+00_JPRB ,&
&  0.95418E+00_JPRB, 0.92877E+00_JPRB, 0.90134E+00_JPRB, 0.87349E+00_JPRB, 0.85207E+00_JPRB, 0.83064E+00_JPRB, 0.81222E+00_JPRB ,&
&  0.80289E+00_JPRB, 0.80189E+00_JPRB, 0.79727E+00_JPRB, 0.79696E+00_JPRB, 0.80115E+00_JPRB, 0.80493E+00_JPRB, 0.82291E+00_JPRB ,&
&  0.83969E+00_JPRB, 0.85234E+00_JPRB, 0.86632E+00_JPRB, 0.87421E+00_JPRB, 0.87159E+00_JPRB, 0.87147E+00_JPRB, 0.86581E+00_JPRB ,&
&  0.85876E+00_JPRB, 0.84531E+00_JPRB, 0.83159E+00_JPRB, 0.82059E+00_JPRB, 0.81289E+00_JPRB, 0.80722E+00_JPRB, 0.80927E+00_JPRB ,&
&  0.81651E+00_JPRB, 0.81490E+00_JPRB, 0.81346E+00_JPRB, 0.81672E+00_JPRB, 0.82127E+00_JPRB, 0.82741E+00_JPRB, 0.83897E+00_JPRB ,&
&  0.85849E+00_JPRB, 0.88479E+00_JPRB, 0.90744E+00_JPRB, 0.91759E+00_JPRB, 0.92114E+00_JPRB, 0.93023E+00_JPRB, 0.94227E+00_JPRB ,&
&  0.95480E+00_JPRB, 0.96177E+00_JPRB, 0.96715E+00_JPRB, 0.10085E+01_JPRB, 0.10206E+01_JPRB, 0.10147E+01_JPRB, 0.10009E+01_JPRB ,&
&  0.97461E+00_JPRB, 0.96290E+00_JPRB, 0.79802E+00_JPRB, 0.86782E+00_JPRB, 0.88675E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.84032E+00_JPRB, 0.82969E+00_JPRB, 0.81235E+00_JPRB, 0.79748E+00_JPRB, 0.78493E+00_JPRB, 0.77952E+00_JPRB, 0.77630E+00_JPRB ,&
&  0.78057E+00_JPRB, 0.78897E+00_JPRB, 0.79725E+00_JPRB, 0.80187E+00_JPRB, 0.80527E+00_JPRB, 0.80476E+00_JPRB, 0.79986E+00_JPRB ,&
&  0.79544E+00_JPRB, 0.78283E+00_JPRB, 0.77250E+00_JPRB, 0.77107E+00_JPRB, 0.76700E+00_JPRB, 0.76204E+00_JPRB, 0.75544E+00_JPRB ,&
&  0.74877E+00_JPRB, 0.74272E+00_JPRB, 0.73129E+00_JPRB, 0.72541E+00_JPRB, 0.72381E+00_JPRB, 0.72618E+00_JPRB, 0.73517E+00_JPRB ,&
&  0.65918E+00_JPRB, 0.58683E+00_JPRB, 0.67891E+00_JPRB, 0.78412E+00_JPRB, 0.83118E+00_JPRB, 0.87766E+00_JPRB, 0.91863E+00_JPRB ,&
&  0.97425E+00_JPRB, 0.10121E+01_JPRB, 0.10365E+01_JPRB, 0.10369E+01_JPRB, 0.10174E+01_JPRB, 0.98783E+00_JPRB, 0.95420E+00_JPRB ,&
&  0.91164E+00_JPRB, 0.86260E+00_JPRB, 0.82793E+00_JPRB, 0.80734E+00_JPRB, 0.79085E+00_JPRB, 0.77434E+00_JPRB, 0.76109E+00_JPRB ,&
&  0.75230E+00_JPRB, 0.74596E+00_JPRB, 0.74081E+00_JPRB, 0.73725E+00_JPRB, 0.73820E+00_JPRB, 0.74109E+00_JPRB, 0.74670E+00_JPRB ,&
&  0.75335E+00_JPRB, 0.75932E+00_JPRB, 0.76292E+00_JPRB, 0.76226E+00_JPRB, 0.76242E+00_JPRB, 0.76850E+00_JPRB, 0.77126E+00_JPRB ,&
&  0.77360E+00_JPRB, 0.77155E+00_JPRB, 0.76796E+00_JPRB, 0.76329E+00_JPRB, 0.75648E+00_JPRB, 0.74986E+00_JPRB, 0.74651E+00_JPRB ,&
&  0.74680E+00_JPRB, 0.74345E+00_JPRB, 0.74401E+00_JPRB, 0.74868E+00_JPRB, 0.75510E+00_JPRB, 0.76213E+00_JPRB, 0.76872E+00_JPRB ,&
&  0.77991E+00_JPRB, 0.79881E+00_JPRB, 0.81412E+00_JPRB, 0.82447E+00_JPRB, 0.83168E+00_JPRB, 0.83843E+00_JPRB, 0.84801E+00_JPRB ,&
&  0.85673E+00_JPRB, 0.86531E+00_JPRB, 0.87575E+00_JPRB, 0.89476E+00_JPRB, 0.90288E+00_JPRB, 0.90187E+00_JPRB, 0.89544E+00_JPRB ,&
&  0.88181E+00_JPRB, 0.86200E+00_JPRB, 0.85718E+00_JPRB, 0.84940E+00_JPRB, 0.84382E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.74282E+00_JPRB, 0.73562E+00_JPRB, 0.72351E+00_JPRB, 0.71581E+00_JPRB, 0.71201E+00_JPRB, 0.71398E+00_JPRB, 0.71736E+00_JPRB ,&
&  0.72139E+00_JPRB, 0.72276E+00_JPRB, 0.73038E+00_JPRB, 0.73876E+00_JPRB, 0.74426E+00_JPRB, 0.74822E+00_JPRB, 0.74750E+00_JPRB ,&
&  0.74701E+00_JPRB, 0.74637E+00_JPRB, 0.74106E+00_JPRB, 0.73598E+00_JPRB, 0.72946E+00_JPRB, 0.72920E+00_JPRB, 0.72792E+00_JPRB ,&
&  0.72492E+00_JPRB, 0.71978E+00_JPRB, 0.71143E+00_JPRB, 0.70121E+00_JPRB, 0.69626E+00_JPRB, 0.69480E+00_JPRB, 0.69188E+00_JPRB ,&
&  0.69887E+00_JPRB, 0.62106E+00_JPRB, 0.66694E+00_JPRB, 0.70673E+00_JPRB, 0.75763E+00_JPRB, 0.79892E+00_JPRB, 0.83602E+00_JPRB ,&
&  0.87317E+00_JPRB, 0.90498E+00_JPRB, 0.92093E+00_JPRB, 0.92553E+00_JPRB, 0.90778E+00_JPRB, 0.87192E+00_JPRB, 0.83122E+00_JPRB ,&
&  0.79730E+00_JPRB, 0.77217E+00_JPRB, 0.74732E+00_JPRB, 0.73178E+00_JPRB, 0.71908E+00_JPRB, 0.70315E+00_JPRB, 0.68823E+00_JPRB ,&
&  0.67863E+00_JPRB, 0.67185E+00_JPRB, 0.66895E+00_JPRB, 0.66918E+00_JPRB, 0.67122E+00_JPRB, 0.67316E+00_JPRB, 0.67337E+00_JPRB ,&
&  0.67412E+00_JPRB, 0.67535E+00_JPRB, 0.67507E+00_JPRB, 0.67649E+00_JPRB, 0.67928E+00_JPRB, 0.68443E+00_JPRB, 0.68336E+00_JPRB ,&
&  0.67760E+00_JPRB, 0.67870E+00_JPRB, 0.68078E+00_JPRB, 0.68078E+00_JPRB, 0.68100E+00_JPRB, 0.68106E+00_JPRB, 0.67877E+00_JPRB ,&
&  0.67693E+00_JPRB, 0.67929E+00_JPRB, 0.68233E+00_JPRB, 0.68880E+00_JPRB, 0.69386E+00_JPRB, 0.70395E+00_JPRB, 0.71331E+00_JPRB ,&
&  0.72322E+00_JPRB, 0.73815E+00_JPRB, 0.75319E+00_JPRB, 0.76475E+00_JPRB, 0.77211E+00_JPRB, 0.77804E+00_JPRB, 0.78384E+00_JPRB ,&
&  0.78845E+00_JPRB, 0.79280E+00_JPRB, 0.79708E+00_JPRB, 0.79979E+00_JPRB, 0.79739E+00_JPRB, 0.79327E+00_JPRB, 0.78969E+00_JPRB ,&
&  0.77948E+00_JPRB, 0.76474E+00_JPRB, 0.75781E+00_JPRB, 0.75161E+00_JPRB, 0.74587E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.67590E+00_JPRB, 0.67050E+00_JPRB, 0.66494E+00_JPRB, 0.66385E+00_JPRB, 0.66247E+00_JPRB, 0.66394E+00_JPRB, 0.66620E+00_JPRB ,&
&  0.67256E+00_JPRB, 0.67666E+00_JPRB, 0.68597E+00_JPRB, 0.69383E+00_JPRB, 0.69792E+00_JPRB, 0.70274E+00_JPRB, 0.70573E+00_JPRB ,&
&  0.70689E+00_JPRB, 0.70834E+00_JPRB, 0.70891E+00_JPRB, 0.71298E+00_JPRB, 0.71335E+00_JPRB, 0.71238E+00_JPRB, 0.71100E+00_JPRB ,&
&  0.70826E+00_JPRB, 0.70483E+00_JPRB, 0.69814E+00_JPRB, 0.68944E+00_JPRB, 0.67882E+00_JPRB, 0.67342E+00_JPRB, 0.66621E+00_JPRB ,&
&  0.66630E+00_JPRB, 0.64633E+00_JPRB, 0.66026E+00_JPRB, 0.65924E+00_JPRB, 0.68522E+00_JPRB, 0.72535E+00_JPRB, 0.76869E+00_JPRB ,&
&  0.79470E+00_JPRB, 0.80503E+00_JPRB, 0.80162E+00_JPRB, 0.79287E+00_JPRB, 0.77340E+00_JPRB, 0.75433E+00_JPRB, 0.73282E+00_JPRB ,&
&  0.70873E+00_JPRB, 0.69439E+00_JPRB, 0.67821E+00_JPRB, 0.66622E+00_JPRB, 0.66076E+00_JPRB, 0.65654E+00_JPRB, 0.64930E+00_JPRB ,&
&  0.63833E+00_JPRB, 0.62865E+00_JPRB, 0.62735E+00_JPRB, 0.63004E+00_JPRB, 0.63341E+00_JPRB, 0.63554E+00_JPRB, 0.63424E+00_JPRB ,&
&  0.62805E+00_JPRB, 0.62341E+00_JPRB, 0.62227E+00_JPRB, 0.62297E+00_JPRB, 0.61994E+00_JPRB, 0.61353E+00_JPRB, 0.60661E+00_JPRB ,&
&  0.60513E+00_JPRB, 0.60789E+00_JPRB, 0.61156E+00_JPRB, 0.61621E+00_JPRB, 0.61943E+00_JPRB, 0.62332E+00_JPRB, 0.62504E+00_JPRB ,&
&  0.62954E+00_JPRB, 0.63395E+00_JPRB, 0.64313E+00_JPRB, 0.65211E+00_JPRB, 0.65460E+00_JPRB, 0.66388E+00_JPRB, 0.67373E+00_JPRB ,&
&  0.68413E+00_JPRB, 0.69646E+00_JPRB, 0.70684E+00_JPRB, 0.71492E+00_JPRB, 0.72303E+00_JPRB, 0.72800E+00_JPRB, 0.73456E+00_JPRB ,&
&  0.74276E+00_JPRB, 0.74489E+00_JPRB, 0.75064E+00_JPRB, 0.75115E+00_JPRB, 0.74616E+00_JPRB, 0.73806E+00_JPRB, 0.72715E+00_JPRB ,&
&  0.71447E+00_JPRB, 0.69727E+00_JPRB, 0.68748E+00_JPRB, 0.68143E+00_JPRB, 0.67795E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.59903E+00_JPRB, 0.59672E+00_JPRB, 0.59686E+00_JPRB, 0.59994E+00_JPRB, 0.60342E+00_JPRB, 0.60609E+00_JPRB, 0.61168E+00_JPRB ,&
&  0.62153E+00_JPRB, 0.62933E+00_JPRB, 0.63902E+00_JPRB, 0.64907E+00_JPRB, 0.65684E+00_JPRB, 0.66209E+00_JPRB, 0.66491E+00_JPRB ,&
&  0.66329E+00_JPRB, 0.66080E+00_JPRB, 0.66401E+00_JPRB, 0.66885E+00_JPRB, 0.67027E+00_JPRB, 0.66975E+00_JPRB, 0.66958E+00_JPRB ,&
&  0.66866E+00_JPRB, 0.66568E+00_JPRB, 0.66165E+00_JPRB, 0.65589E+00_JPRB, 0.64925E+00_JPRB, 0.64535E+00_JPRB, 0.64188E+00_JPRB ,&
&  0.63367E+00_JPRB, 0.63757E+00_JPRB, 0.58746E+00_JPRB, 0.60153E+00_JPRB, 0.61347E+00_JPRB, 0.63729E+00_JPRB, 0.66212E+00_JPRB ,&
&  0.67851E+00_JPRB, 0.68647E+00_JPRB, 0.68930E+00_JPRB, 0.68548E+00_JPRB, 0.67129E+00_JPRB, 0.66371E+00_JPRB, 0.65604E+00_JPRB ,&
&  0.64531E+00_JPRB, 0.63753E+00_JPRB, 0.62332E+00_JPRB, 0.60776E+00_JPRB, 0.60237E+00_JPRB, 0.60280E+00_JPRB, 0.60234E+00_JPRB ,&
&  0.59682E+00_JPRB, 0.59118E+00_JPRB, 0.59111E+00_JPRB, 0.59150E+00_JPRB, 0.59114E+00_JPRB, 0.58581E+00_JPRB, 0.57920E+00_JPRB ,&
&  0.56879E+00_JPRB, 0.55856E+00_JPRB, 0.55149E+00_JPRB, 0.54641E+00_JPRB, 0.54205E+00_JPRB, 0.53712E+00_JPRB, 0.53236E+00_JPRB ,&
&  0.53663E+00_JPRB, 0.54885E+00_JPRB, 0.56031E+00_JPRB, 0.56955E+00_JPRB, 0.57537E+00_JPRB, 0.57910E+00_JPRB, 0.58420E+00_JPRB ,&
&  0.59029E+00_JPRB, 0.60075E+00_JPRB, 0.60842E+00_JPRB, 0.61548E+00_JPRB, 0.62084E+00_JPRB, 0.62589E+00_JPRB, 0.63169E+00_JPRB ,&
&  0.63682E+00_JPRB, 0.64514E+00_JPRB, 0.65199E+00_JPRB, 0.65987E+00_JPRB, 0.66500E+00_JPRB, 0.67118E+00_JPRB, 0.67784E+00_JPRB ,&
&  0.68500E+00_JPRB, 0.69112E+00_JPRB, 0.69348E+00_JPRB, 0.69186E+00_JPRB, 0.68424E+00_JPRB, 0.67431E+00_JPRB, 0.66435E+00_JPRB ,&
&  0.65151E+00_JPRB, 0.63608E+00_JPRB, 0.62045E+00_JPRB, 0.61034E+00_JPRB, 0.60191E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.54059E+00_JPRB, 0.54118E+00_JPRB, 0.54577E+00_JPRB, 0.55027E+00_JPRB, 0.55362E+00_JPRB, 0.55850E+00_JPRB, 0.56821E+00_JPRB ,&
&  0.57806E+00_JPRB, 0.58748E+00_JPRB, 0.59395E+00_JPRB, 0.60182E+00_JPRB, 0.61308E+00_JPRB, 0.62079E+00_JPRB, 0.62692E+00_JPRB ,&
&  0.62956E+00_JPRB, 0.63018E+00_JPRB, 0.63155E+00_JPRB, 0.63492E+00_JPRB, 0.63462E+00_JPRB, 0.63101E+00_JPRB, 0.62966E+00_JPRB ,&
&  0.62935E+00_JPRB, 0.62730E+00_JPRB, 0.62653E+00_JPRB, 0.62681E+00_JPRB, 0.62344E+00_JPRB, 0.61849E+00_JPRB, 0.61683E+00_JPRB ,&
&  0.61550E+00_JPRB, 0.60168E+00_JPRB, 0.59378E+00_JPRB, 0.58623E+00_JPRB, 0.58910E+00_JPRB, 0.59962E+00_JPRB, 0.61156E+00_JPRB ,&
&  0.61863E+00_JPRB, 0.62352E+00_JPRB, 0.62384E+00_JPRB, 0.62333E+00_JPRB, 0.61866E+00_JPRB, 0.60773E+00_JPRB, 0.59927E+00_JPRB ,&
&  0.59398E+00_JPRB, 0.58538E+00_JPRB, 0.56237E+00_JPRB, 0.54819E+00_JPRB, 0.54406E+00_JPRB, 0.54701E+00_JPRB, 0.55193E+00_JPRB ,&
&  0.55365E+00_JPRB, 0.55034E+00_JPRB, 0.54326E+00_JPRB, 0.53553E+00_JPRB, 0.52571E+00_JPRB, 0.51462E+00_JPRB, 0.50488E+00_JPRB ,&
&  0.49951E+00_JPRB, 0.49543E+00_JPRB, 0.49203E+00_JPRB, 0.49468E+00_JPRB, 0.49387E+00_JPRB, 0.48970E+00_JPRB, 0.49176E+00_JPRB ,&
&  0.50387E+00_JPRB, 0.51946E+00_JPRB, 0.53384E+00_JPRB, 0.54157E+00_JPRB, 0.54466E+00_JPRB, 0.54821E+00_JPRB, 0.55222E+00_JPRB ,&
&  0.55796E+00_JPRB, 0.56346E+00_JPRB, 0.56882E+00_JPRB, 0.57626E+00_JPRB, 0.58549E+00_JPRB, 0.59381E+00_JPRB, 0.59770E+00_JPRB ,&
&  0.60100E+00_JPRB, 0.60335E+00_JPRB, 0.60571E+00_JPRB, 0.61216E+00_JPRB, 0.62106E+00_JPRB, 0.62742E+00_JPRB, 0.63244E+00_JPRB ,&
&  0.63745E+00_JPRB, 0.63970E+00_JPRB, 0.63811E+00_JPRB, 0.63376E+00_JPRB, 0.62779E+00_JPRB, 0.61641E+00_JPRB, 0.60230E+00_JPRB ,&
&  0.58596E+00_JPRB, 0.57080E+00_JPRB, 0.55932E+00_JPRB, 0.54947E+00_JPRB, 0.54263E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.49595E+00_JPRB, 0.49544E+00_JPRB, 0.49811E+00_JPRB, 0.50431E+00_JPRB, 0.50850E+00_JPRB, 0.51234E+00_JPRB, 0.52021E+00_JPRB ,&
&  0.52759E+00_JPRB, 0.53423E+00_JPRB, 0.54192E+00_JPRB, 0.55013E+00_JPRB, 0.55647E+00_JPRB, 0.56569E+00_JPRB, 0.57458E+00_JPRB ,&
&  0.58046E+00_JPRB, 0.58302E+00_JPRB, 0.58470E+00_JPRB, 0.58613E+00_JPRB, 0.58626E+00_JPRB, 0.58390E+00_JPRB, 0.58348E+00_JPRB ,&
&  0.58155E+00_JPRB, 0.57929E+00_JPRB, 0.58114E+00_JPRB, 0.58110E+00_JPRB, 0.58213E+00_JPRB, 0.58401E+00_JPRB, 0.58304E+00_JPRB ,&
&  0.57963E+00_JPRB, 0.56555E+00_JPRB, 0.56010E+00_JPRB, 0.55655E+00_JPRB, 0.56032E+00_JPRB, 0.56084E+00_JPRB, 0.55884E+00_JPRB ,&
&  0.55174E+00_JPRB, 0.54725E+00_JPRB, 0.54900E+00_JPRB, 0.55142E+00_JPRB, 0.54568E+00_JPRB, 0.53285E+00_JPRB, 0.52351E+00_JPRB ,&
&  0.51678E+00_JPRB, 0.50543E+00_JPRB, 0.48751E+00_JPRB, 0.47616E+00_JPRB, 0.47120E+00_JPRB, 0.46649E+00_JPRB, 0.46270E+00_JPRB ,&
&  0.46087E+00_JPRB, 0.46005E+00_JPRB, 0.46105E+00_JPRB, 0.46010E+00_JPRB, 0.45651E+00_JPRB, 0.45241E+00_JPRB, 0.45107E+00_JPRB ,&
&  0.45401E+00_JPRB, 0.45652E+00_JPRB, 0.45994E+00_JPRB, 0.46269E+00_JPRB, 0.46224E+00_JPRB, 0.46238E+00_JPRB, 0.46604E+00_JPRB ,&
&  0.47225E+00_JPRB, 0.47831E+00_JPRB, 0.48416E+00_JPRB, 0.48649E+00_JPRB, 0.48998E+00_JPRB, 0.49720E+00_JPRB, 0.49798E+00_JPRB ,&
&  0.49634E+00_JPRB, 0.49797E+00_JPRB, 0.50216E+00_JPRB, 0.51068E+00_JPRB, 0.52382E+00_JPRB, 0.53284E+00_JPRB, 0.53487E+00_JPRB ,&
&  0.53562E+00_JPRB, 0.53845E+00_JPRB, 0.54289E+00_JPRB, 0.55320E+00_JPRB, 0.56438E+00_JPRB, 0.57238E+00_JPRB, 0.57571E+00_JPRB ,&
&  0.57259E+00_JPRB, 0.56564E+00_JPRB, 0.55318E+00_JPRB, 0.53874E+00_JPRB, 0.52586E+00_JPRB, 0.51571E+00_JPRB, 0.50732E+00_JPRB ,&
&  0.49893E+00_JPRB, 0.49491E+00_JPRB, 0.49404E+00_JPRB, 0.49550E+00_JPRB, 0.49617E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.46073E+00_JPRB, 0.46235E+00_JPRB, 0.46458E+00_JPRB, 0.46758E+00_JPRB, 0.47078E+00_JPRB, 0.47264E+00_JPRB, 0.47633E+00_JPRB ,&
&  0.47866E+00_JPRB, 0.48553E+00_JPRB, 0.49665E+00_JPRB, 0.50765E+00_JPRB, 0.51557E+00_JPRB, 0.52224E+00_JPRB, 0.52892E+00_JPRB ,&
&  0.53448E+00_JPRB, 0.54179E+00_JPRB, 0.54299E+00_JPRB, 0.54086E+00_JPRB, 0.53846E+00_JPRB, 0.53665E+00_JPRB, 0.53494E+00_JPRB ,&
&  0.53239E+00_JPRB, 0.53211E+00_JPRB, 0.53564E+00_JPRB, 0.53953E+00_JPRB, 0.54338E+00_JPRB, 0.54393E+00_JPRB, 0.54175E+00_JPRB ,&
&  0.54436E+00_JPRB, 0.53381E+00_JPRB, 0.52668E+00_JPRB, 0.51870E+00_JPRB, 0.51178E+00_JPRB, 0.49993E+00_JPRB, 0.49106E+00_JPRB ,&
&  0.48049E+00_JPRB, 0.47108E+00_JPRB, 0.46702E+00_JPRB, 0.46281E+00_JPRB, 0.45315E+00_JPRB, 0.44233E+00_JPRB, 0.43480E+00_JPRB ,&
&  0.43007E+00_JPRB, 0.42524E+00_JPRB, 0.41900E+00_JPRB, 0.41354E+00_JPRB, 0.40894E+00_JPRB, 0.40724E+00_JPRB, 0.40889E+00_JPRB ,&
&  0.41349E+00_JPRB, 0.41796E+00_JPRB, 0.42087E+00_JPRB, 0.42177E+00_JPRB, 0.42016E+00_JPRB, 0.41751E+00_JPRB, 0.41607E+00_JPRB ,&
&  0.41676E+00_JPRB, 0.41827E+00_JPRB, 0.42012E+00_JPRB, 0.42526E+00_JPRB, 0.42603E+00_JPRB, 0.42401E+00_JPRB, 0.41803E+00_JPRB ,&
&  0.41562E+00_JPRB, 0.41450E+00_JPRB, 0.41630E+00_JPRB, 0.42152E+00_JPRB, 0.43028E+00_JPRB, 0.43894E+00_JPRB, 0.44512E+00_JPRB ,&
&  0.44805E+00_JPRB, 0.44961E+00_JPRB, 0.45267E+00_JPRB, 0.45664E+00_JPRB, 0.45870E+00_JPRB, 0.45869E+00_JPRB, 0.45782E+00_JPRB ,&
&  0.46116E+00_JPRB, 0.46920E+00_JPRB, 0.47759E+00_JPRB, 0.48323E+00_JPRB, 0.48638E+00_JPRB, 0.48594E+00_JPRB, 0.47940E+00_JPRB ,&
&  0.46695E+00_JPRB, 0.45239E+00_JPRB, 0.43826E+00_JPRB, 0.42643E+00_JPRB, 0.41818E+00_JPRB, 0.41419E+00_JPRB, 0.41692E+00_JPRB ,&
&  0.42445E+00_JPRB, 0.43412E+00_JPRB, 0.44281E+00_JPRB, 0.44909E+00_JPRB, 0.45604E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.42761E+00_JPRB, 0.42748E+00_JPRB, 0.42913E+00_JPRB, 0.42826E+00_JPRB, 0.42563E+00_JPRB, 0.42337E+00_JPRB, 0.42652E+00_JPRB ,&
&  0.43319E+00_JPRB, 0.44507E+00_JPRB, 0.46102E+00_JPRB, 0.47506E+00_JPRB, 0.48349E+00_JPRB, 0.48856E+00_JPRB, 0.49625E+00_JPRB ,&
&  0.50352E+00_JPRB, 0.51164E+00_JPRB, 0.51206E+00_JPRB, 0.50346E+00_JPRB, 0.49623E+00_JPRB, 0.49247E+00_JPRB, 0.49209E+00_JPRB ,&
&  0.49141E+00_JPRB, 0.49301E+00_JPRB, 0.49591E+00_JPRB, 0.49890E+00_JPRB, 0.49774E+00_JPRB, 0.49532E+00_JPRB, 0.49818E+00_JPRB ,&
&  0.49532E+00_JPRB, 0.48463E+00_JPRB, 0.47360E+00_JPRB, 0.44815E+00_JPRB, 0.44679E+00_JPRB, 0.43586E+00_JPRB, 0.42765E+00_JPRB ,&
&  0.41664E+00_JPRB, 0.40292E+00_JPRB, 0.39290E+00_JPRB, 0.38657E+00_JPRB, 0.38365E+00_JPRB, 0.38128E+00_JPRB, 0.37959E+00_JPRB ,&
&  0.37877E+00_JPRB, 0.37933E+00_JPRB, 0.38155E+00_JPRB, 0.37954E+00_JPRB, 0.37851E+00_JPRB, 0.37949E+00_JPRB, 0.38153E+00_JPRB ,&
&  0.38213E+00_JPRB, 0.38274E+00_JPRB, 0.38334E+00_JPRB, 0.38294E+00_JPRB, 0.38132E+00_JPRB, 0.37871E+00_JPRB, 0.37633E+00_JPRB ,&
&  0.37544E+00_JPRB, 0.37667E+00_JPRB, 0.37726E+00_JPRB, 0.37334E+00_JPRB, 0.36552E+00_JPRB, 0.35534E+00_JPRB, 0.34863E+00_JPRB ,&
&  0.34585E+00_JPRB, 0.34815E+00_JPRB, 0.35641E+00_JPRB, 0.36786E+00_JPRB, 0.37835E+00_JPRB, 0.38480E+00_JPRB, 0.38905E+00_JPRB ,&
&  0.39372E+00_JPRB, 0.39145E+00_JPRB, 0.38749E+00_JPRB, 0.38431E+00_JPRB, 0.38146E+00_JPRB, 0.37732E+00_JPRB, 0.37731E+00_JPRB ,&
&  0.38040E+00_JPRB, 0.38366E+00_JPRB, 0.38489E+00_JPRB, 0.38394E+00_JPRB, 0.38188E+00_JPRB, 0.37763E+00_JPRB, 0.37172E+00_JPRB ,&
&  0.36424E+00_JPRB, 0.35550E+00_JPRB, 0.35378E+00_JPRB, 0.35485E+00_JPRB, 0.35880E+00_JPRB, 0.36664E+00_JPRB, 0.37737E+00_JPRB ,&
&  0.38855E+00_JPRB, 0.39750E+00_JPRB, 0.40716E+00_JPRB, 0.41608E+00_JPRB, 0.42414E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.40629E+00_JPRB, 0.40765E+00_JPRB, 0.40499E+00_JPRB, 0.40221E+00_JPRB, 0.40202E+00_JPRB, 0.40359E+00_JPRB, 0.40907E+00_JPRB ,&
&  0.41876E+00_JPRB, 0.43173E+00_JPRB, 0.44427E+00_JPRB, 0.45467E+00_JPRB, 0.46389E+00_JPRB, 0.46792E+00_JPRB, 0.47384E+00_JPRB ,&
&  0.48026E+00_JPRB, 0.48617E+00_JPRB, 0.48700E+00_JPRB, 0.47677E+00_JPRB, 0.46690E+00_JPRB, 0.46170E+00_JPRB, 0.46175E+00_JPRB ,&
&  0.46344E+00_JPRB, 0.46585E+00_JPRB, 0.47167E+00_JPRB, 0.47351E+00_JPRB, 0.46713E+00_JPRB, 0.46513E+00_JPRB, 0.46245E+00_JPRB ,&
&  0.45622E+00_JPRB, 0.44353E+00_JPRB, 0.41052E+00_JPRB, 0.40396E+00_JPRB, 0.39520E+00_JPRB, 0.38590E+00_JPRB, 0.37694E+00_JPRB ,&
&  0.36660E+00_JPRB, 0.35603E+00_JPRB, 0.34884E+00_JPRB, 0.34454E+00_JPRB, 0.34509E+00_JPRB, 0.34641E+00_JPRB, 0.34621E+00_JPRB ,&
&  0.34870E+00_JPRB, 0.35148E+00_JPRB, 0.35411E+00_JPRB, 0.35491E+00_JPRB, 0.35498E+00_JPRB, 0.35456E+00_JPRB, 0.35077E+00_JPRB ,&
&  0.34737E+00_JPRB, 0.34581E+00_JPRB, 0.34456E+00_JPRB, 0.34345E+00_JPRB, 0.34349E+00_JPRB, 0.34414E+00_JPRB, 0.34437E+00_JPRB ,&
&  0.34189E+00_JPRB, 0.33817E+00_JPRB, 0.33401E+00_JPRB, 0.32658E+00_JPRB, 0.31298E+00_JPRB, 0.29832E+00_JPRB, 0.28873E+00_JPRB ,&
&  0.28747E+00_JPRB, 0.29804E+00_JPRB, 0.31844E+00_JPRB, 0.32911E+00_JPRB, 0.33672E+00_JPRB, 0.34042E+00_JPRB, 0.34443E+00_JPRB ,&
&  0.33803E+00_JPRB, 0.31537E+00_JPRB, 0.30274E+00_JPRB, 0.29543E+00_JPRB, 0.29024E+00_JPRB, 0.28667E+00_JPRB, 0.28637E+00_JPRB ,&
&  0.28991E+00_JPRB, 0.29598E+00_JPRB, 0.30256E+00_JPRB, 0.30568E+00_JPRB, 0.30472E+00_JPRB, 0.30164E+00_JPRB, 0.29816E+00_JPRB ,&
&  0.29587E+00_JPRB, 0.30099E+00_JPRB, 0.31586E+00_JPRB, 0.33605E+00_JPRB, 0.34446E+00_JPRB, 0.35112E+00_JPRB, 0.35714E+00_JPRB ,&
&  0.36373E+00_JPRB, 0.37371E+00_JPRB, 0.38553E+00_JPRB, 0.39463E+00_JPRB, 0.40263E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.37712E+00_JPRB, 0.38089E+00_JPRB, 0.38316E+00_JPRB, 0.38639E+00_JPRB, 0.39402E+00_JPRB, 0.40464E+00_JPRB, 0.41671E+00_JPRB ,&
&  0.42823E+00_JPRB, 0.43794E+00_JPRB, 0.44292E+00_JPRB, 0.44827E+00_JPRB, 0.45326E+00_JPRB, 0.45556E+00_JPRB, 0.46054E+00_JPRB ,&
&  0.46118E+00_JPRB, 0.45760E+00_JPRB, 0.45142E+00_JPRB, 0.44338E+00_JPRB, 0.43914E+00_JPRB, 0.44035E+00_JPRB, 0.44306E+00_JPRB ,&
&  0.44465E+00_JPRB, 0.44581E+00_JPRB, 0.44557E+00_JPRB, 0.44175E+00_JPRB, 0.43819E+00_JPRB, 0.43603E+00_JPRB, 0.43152E+00_JPRB ,&
&  0.42068E+00_JPRB, 0.37042E+00_JPRB, 0.33311E+00_JPRB, 0.32967E+00_JPRB, 0.34481E+00_JPRB, 0.34677E+00_JPRB, 0.34215E+00_JPRB ,&
&  0.33674E+00_JPRB, 0.33179E+00_JPRB, 0.32889E+00_JPRB, 0.32663E+00_JPRB, 0.32390E+00_JPRB, 0.32144E+00_JPRB, 0.32210E+00_JPRB ,&
&  0.32347E+00_JPRB, 0.32142E+00_JPRB, 0.31863E+00_JPRB, 0.31238E+00_JPRB, 0.30796E+00_JPRB, 0.30676E+00_JPRB, 0.30581E+00_JPRB ,&
&  0.30176E+00_JPRB, 0.29755E+00_JPRB, 0.29767E+00_JPRB, 0.29915E+00_JPRB, 0.30142E+00_JPRB, 0.30295E+00_JPRB, 0.30000E+00_JPRB ,&
&  0.29523E+00_JPRB, 0.28821E+00_JPRB, 0.28010E+00_JPRB, 0.27018E+00_JPRB, 0.26094E+00_JPRB, 0.25405E+00_JPRB, 0.24830E+00_JPRB ,&
&  0.24775E+00_JPRB, 0.25455E+00_JPRB, 0.26882E+00_JPRB, 0.28396E+00_JPRB, 0.29827E+00_JPRB, 0.29002E+00_JPRB, 0.28023E+00_JPRB ,&
&  0.26929E+00_JPRB, 0.25987E+00_JPRB, 0.25311E+00_JPRB, 0.25022E+00_JPRB, 0.24914E+00_JPRB, 0.24914E+00_JPRB, 0.25049E+00_JPRB ,&
&  0.25293E+00_JPRB, 0.25587E+00_JPRB, 0.25825E+00_JPRB, 0.25936E+00_JPRB, 0.25977E+00_JPRB, 0.25999E+00_JPRB, 0.25945E+00_JPRB ,&
&  0.25871E+00_JPRB, 0.26130E+00_JPRB, 0.26831E+00_JPRB, 0.27716E+00_JPRB, 0.28546E+00_JPRB, 0.29346E+00_JPRB, 0.30647E+00_JPRB ,&
&  0.32241E+00_JPRB, 0.34153E+00_JPRB, 0.35476E+00_JPRB, 0.36505E+00_JPRB, 0.37355E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.36757E+00_JPRB, 0.37654E+00_JPRB, 0.38810E+00_JPRB, 0.39831E+00_JPRB, 0.40757E+00_JPRB, 0.41625E+00_JPRB, 0.42407E+00_JPRB ,&
&  0.42865E+00_JPRB, 0.43131E+00_JPRB, 0.43243E+00_JPRB, 0.43615E+00_JPRB, 0.43302E+00_JPRB, 0.42565E+00_JPRB, 0.41991E+00_JPRB ,&
&  0.41506E+00_JPRB, 0.41135E+00_JPRB, 0.40602E+00_JPRB, 0.40321E+00_JPRB, 0.40290E+00_JPRB, 0.40285E+00_JPRB, 0.39706E+00_JPRB ,&
&  0.38616E+00_JPRB, 0.38452E+00_JPRB, 0.38884E+00_JPRB, 0.38294E+00_JPRB, 0.37694E+00_JPRB, 0.37109E+00_JPRB, 0.36500E+00_JPRB ,&
&  0.34831E+00_JPRB, 0.32771E+00_JPRB, 0.30129E+00_JPRB, 0.29863E+00_JPRB, 0.31432E+00_JPRB, 0.30717E+00_JPRB, 0.31103E+00_JPRB ,&
&  0.31207E+00_JPRB, 0.31211E+00_JPRB, 0.31065E+00_JPRB, 0.30889E+00_JPRB, 0.30549E+00_JPRB, 0.30472E+00_JPRB, 0.30313E+00_JPRB ,&
&  0.29682E+00_JPRB, 0.29171E+00_JPRB, 0.28329E+00_JPRB, 0.26495E+00_JPRB, 0.26084E+00_JPRB, 0.26025E+00_JPRB, 0.25605E+00_JPRB ,&
&  0.24859E+00_JPRB, 0.24394E+00_JPRB, 0.24257E+00_JPRB, 0.23788E+00_JPRB, 0.24011E+00_JPRB, 0.24244E+00_JPRB, 0.24227E+00_JPRB ,&
&  0.24211E+00_JPRB, 0.24158E+00_JPRB, 0.23925E+00_JPRB, 0.23527E+00_JPRB, 0.23200E+00_JPRB, 0.23102E+00_JPRB, 0.23210E+00_JPRB ,&
&  0.23463E+00_JPRB, 0.23894E+00_JPRB, 0.25024E+00_JPRB, 0.25880E+00_JPRB, 0.25732E+00_JPRB, 0.25332E+00_JPRB, 0.24405E+00_JPRB ,&
&  0.23520E+00_JPRB, 0.23205E+00_JPRB, 0.22929E+00_JPRB, 0.22776E+00_JPRB, 0.22805E+00_JPRB, 0.22956E+00_JPRB, 0.23024E+00_JPRB ,&
&  0.23144E+00_JPRB, 0.23279E+00_JPRB, 0.23375E+00_JPRB, 0.23424E+00_JPRB, 0.23500E+00_JPRB, 0.23589E+00_JPRB, 0.23764E+00_JPRB ,&
&  0.24018E+00_JPRB, 0.24465E+00_JPRB, 0.25050E+00_JPRB, 0.25680E+00_JPRB, 0.26140E+00_JPRB, 0.26372E+00_JPRB, 0.26849E+00_JPRB ,&
&  0.27463E+00_JPRB, 0.29068E+00_JPRB, 0.31966E+00_JPRB, 0.36341E+00_JPRB, 0.36627E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.36558E+00_JPRB, 0.37777E+00_JPRB, 0.39290E+00_JPRB, 0.40248E+00_JPRB, 0.40827E+00_JPRB, 0.41127E+00_JPRB, 0.41049E+00_JPRB ,&
&  0.40716E+00_JPRB, 0.40645E+00_JPRB, 0.41733E+00_JPRB, 0.39866E+00_JPRB, 0.38123E+00_JPRB, 0.34859E+00_JPRB, 0.33202E+00_JPRB ,&
&  0.31921E+00_JPRB, 0.31689E+00_JPRB, 0.31634E+00_JPRB, 0.32087E+00_JPRB, 0.32440E+00_JPRB, 0.32710E+00_JPRB, 0.32289E+00_JPRB ,&
&  0.31715E+00_JPRB, 0.30778E+00_JPRB, 0.29993E+00_JPRB, 0.29272E+00_JPRB, 0.29306E+00_JPRB, 0.29191E+00_JPRB, 0.28931E+00_JPRB ,&
&  0.28651E+00_JPRB, 0.28488E+00_JPRB, 0.28503E+00_JPRB, 0.28591E+00_JPRB, 0.29748E+00_JPRB, 0.29624E+00_JPRB, 0.29694E+00_JPRB ,&
&  0.29683E+00_JPRB, 0.29678E+00_JPRB, 0.29574E+00_JPRB, 0.29403E+00_JPRB, 0.28926E+00_JPRB, 0.27402E+00_JPRB, 0.27271E+00_JPRB ,&
&  0.26442E+00_JPRB, 0.25416E+00_JPRB, 0.24400E+00_JPRB, 0.23695E+00_JPRB, 0.23268E+00_JPRB, 0.23123E+00_JPRB, 0.22663E+00_JPRB ,&
&  0.22177E+00_JPRB, 0.21581E+00_JPRB, 0.21324E+00_JPRB, 0.21330E+00_JPRB, 0.21404E+00_JPRB, 0.21465E+00_JPRB, 0.21514E+00_JPRB ,&
&  0.21655E+00_JPRB, 0.21722E+00_JPRB, 0.21762E+00_JPRB, 0.21841E+00_JPRB, 0.21917E+00_JPRB, 0.22026E+00_JPRB, 0.22217E+00_JPRB ,&
&  0.22554E+00_JPRB, 0.22910E+00_JPRB, 0.23248E+00_JPRB, 0.23571E+00_JPRB, 0.23470E+00_JPRB, 0.23078E+00_JPRB, 0.22576E+00_JPRB ,&
&  0.22202E+00_JPRB, 0.22071E+00_JPRB, 0.21946E+00_JPRB, 0.21925E+00_JPRB, 0.21931E+00_JPRB, 0.21967E+00_JPRB, 0.22026E+00_JPRB ,&
&  0.22115E+00_JPRB, 0.22189E+00_JPRB, 0.22279E+00_JPRB, 0.22397E+00_JPRB, 0.22558E+00_JPRB, 0.22706E+00_JPRB, 0.22909E+00_JPRB ,&
&  0.23215E+00_JPRB, 0.23549E+00_JPRB, 0.24013E+00_JPRB, 0.24427E+00_JPRB, 0.25039E+00_JPRB, 0.25873E+00_JPRB, 0.27250E+00_JPRB ,&
&  0.28697E+00_JPRB, 0.32005E+00_JPRB, 0.33470E+00_JPRB, 0.34743E+00_JPRB, 0.35548E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.36531E+00_JPRB, 0.37523E+00_JPRB, 0.38924E+00_JPRB, 0.39851E+00_JPRB, 0.40564E+00_JPRB, 0.40153E+00_JPRB, 0.39753E+00_JPRB ,&
&  0.39328E+00_JPRB, 0.37952E+00_JPRB, 0.36224E+00_JPRB, 0.33306E+00_JPRB, 0.30851E+00_JPRB, 0.28981E+00_JPRB, 0.27905E+00_JPRB ,&
&  0.27222E+00_JPRB, 0.26974E+00_JPRB, 0.26708E+00_JPRB, 0.26613E+00_JPRB, 0.26431E+00_JPRB, 0.26546E+00_JPRB, 0.26641E+00_JPRB ,&
&  0.26571E+00_JPRB, 0.26482E+00_JPRB, 0.26453E+00_JPRB, 0.26780E+00_JPRB, 0.27417E+00_JPRB, 0.27910E+00_JPRB, 0.28394E+00_JPRB ,&
&  0.28720E+00_JPRB, 0.28435E+00_JPRB, 0.28605E+00_JPRB, 0.28441E+00_JPRB, 0.28327E+00_JPRB, 0.28432E+00_JPRB, 0.28589E+00_JPRB ,&
&  0.28628E+00_JPRB, 0.28624E+00_JPRB, 0.28521E+00_JPRB, 0.28168E+00_JPRB, 0.26931E+00_JPRB, 0.26319E+00_JPRB, 0.25502E+00_JPRB ,&
&  0.24689E+00_JPRB, 0.23747E+00_JPRB, 0.22962E+00_JPRB, 0.22607E+00_JPRB, 0.22131E+00_JPRB, 0.21969E+00_JPRB, 0.21647E+00_JPRB ,&
&  0.21282E+00_JPRB, 0.20940E+00_JPRB, 0.20737E+00_JPRB, 0.20638E+00_JPRB, 0.20599E+00_JPRB, 0.20455E+00_JPRB, 0.20405E+00_JPRB ,&
&  0.20445E+00_JPRB, 0.20486E+00_JPRB, 0.20598E+00_JPRB, 0.20692E+00_JPRB, 0.20794E+00_JPRB, 0.20874E+00_JPRB, 0.20970E+00_JPRB ,&
&  0.21139E+00_JPRB, 0.21358E+00_JPRB, 0.21455E+00_JPRB, 0.21549E+00_JPRB, 0.21516E+00_JPRB, 0.21448E+00_JPRB, 0.21350E+00_JPRB ,&
&  0.21268E+00_JPRB, 0.21240E+00_JPRB, 0.21240E+00_JPRB, 0.21228E+00_JPRB, 0.21273E+00_JPRB, 0.21362E+00_JPRB, 0.21445E+00_JPRB ,&
&  0.21603E+00_JPRB, 0.21743E+00_JPRB, 0.21905E+00_JPRB, 0.22110E+00_JPRB, 0.22335E+00_JPRB, 0.22609E+00_JPRB, 0.22912E+00_JPRB ,&
&  0.23167E+00_JPRB, 0.23503E+00_JPRB, 0.23783E+00_JPRB, 0.24002E+00_JPRB, 0.24610E+00_JPRB, 0.25351E+00_JPRB, 0.26876E+00_JPRB ,&
&  0.28539E+00_JPRB, 0.32361E+00_JPRB, 0.36453E+00_JPRB, 0.35144E+00_JPRB, 0.35705E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.33798E+00_JPRB, 0.34918E+00_JPRB, 0.35780E+00_JPRB, 0.36325E+00_JPRB, 0.36854E+00_JPRB, 0.37043E+00_JPRB, 0.37788E+00_JPRB ,&
&  0.37236E+00_JPRB, 0.35460E+00_JPRB, 0.34345E+00_JPRB, 0.32679E+00_JPRB, 0.30642E+00_JPRB, 0.28958E+00_JPRB, 0.28084E+00_JPRB ,&
&  0.27557E+00_JPRB, 0.27120E+00_JPRB, 0.26607E+00_JPRB, 0.26050E+00_JPRB, 0.25496E+00_JPRB, 0.25079E+00_JPRB, 0.24827E+00_JPRB ,&
&  0.24595E+00_JPRB, 0.24700E+00_JPRB, 0.25015E+00_JPRB, 0.25498E+00_JPRB, 0.26190E+00_JPRB, 0.26617E+00_JPRB, 0.26966E+00_JPRB ,&
&  0.27097E+00_JPRB, 0.26099E+00_JPRB, 0.26383E+00_JPRB, 0.26670E+00_JPRB, 0.26569E+00_JPRB, 0.26985E+00_JPRB, 0.27691E+00_JPRB ,&
&  0.27379E+00_JPRB, 0.26462E+00_JPRB, 0.26674E+00_JPRB, 0.26025E+00_JPRB, 0.25163E+00_JPRB, 0.25060E+00_JPRB, 0.24481E+00_JPRB ,&
&  0.24297E+00_JPRB, 0.23593E+00_JPRB, 0.23085E+00_JPRB, 0.22756E+00_JPRB, 0.22104E+00_JPRB, 0.21847E+00_JPRB, 0.21469E+00_JPRB ,&
&  0.21096E+00_JPRB, 0.20861E+00_JPRB, 0.20570E+00_JPRB, 0.20452E+00_JPRB, 0.20343E+00_JPRB, 0.20202E+00_JPRB, 0.20085E+00_JPRB ,&
&  0.20016E+00_JPRB, 0.19953E+00_JPRB, 0.19954E+00_JPRB, 0.19952E+00_JPRB, 0.19942E+00_JPRB, 0.19971E+00_JPRB, 0.19980E+00_JPRB ,&
&  0.20045E+00_JPRB, 0.20131E+00_JPRB, 0.20200E+00_JPRB, 0.20317E+00_JPRB, 0.20361E+00_JPRB, 0.20422E+00_JPRB, 0.20463E+00_JPRB ,&
&  0.20502E+00_JPRB, 0.20566E+00_JPRB, 0.20585E+00_JPRB, 0.20645E+00_JPRB, 0.20711E+00_JPRB, 0.20860E+00_JPRB, 0.21006E+00_JPRB ,&
&  0.21186E+00_JPRB, 0.21407E+00_JPRB, 0.21638E+00_JPRB, 0.21883E+00_JPRB, 0.22188E+00_JPRB, 0.22459E+00_JPRB, 0.22824E+00_JPRB ,&
&  0.23223E+00_JPRB, 0.23605E+00_JPRB, 0.24081E+00_JPRB, 0.24661E+00_JPRB, 0.25450E+00_JPRB, 0.26444E+00_JPRB, 0.27303E+00_JPRB ,&
&  0.28274E+00_JPRB, 0.30771E+00_JPRB, 0.31400E+00_JPRB, 0.31993E+00_JPRB, 0.32905E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.32819E+00_JPRB, 0.33337E+00_JPRB, 0.33734E+00_JPRB, 0.33991E+00_JPRB, 0.34247E+00_JPRB, 0.34415E+00_JPRB, 0.34477E+00_JPRB ,&
&  0.33819E+00_JPRB, 0.33325E+00_JPRB, 0.32611E+00_JPRB, 0.31743E+00_JPRB, 0.30947E+00_JPRB, 0.30137E+00_JPRB, 0.29280E+00_JPRB ,&
&  0.28576E+00_JPRB, 0.27913E+00_JPRB, 0.27250E+00_JPRB, 0.26546E+00_JPRB, 0.25815E+00_JPRB, 0.25078E+00_JPRB, 0.24784E+00_JPRB ,&
&  0.24738E+00_JPRB, 0.24911E+00_JPRB, 0.25183E+00_JPRB, 0.25339E+00_JPRB, 0.25435E+00_JPRB, 0.25481E+00_JPRB, 0.25360E+00_JPRB ,&
&  0.24965E+00_JPRB, 0.25060E+00_JPRB, 0.25194E+00_JPRB, 0.25283E+00_JPRB, 0.25465E+00_JPRB, 0.25629E+00_JPRB, 0.25623E+00_JPRB ,&
&  0.25428E+00_JPRB, 0.25173E+00_JPRB, 0.24822E+00_JPRB, 0.24490E+00_JPRB, 0.24391E+00_JPRB, 0.24190E+00_JPRB, 0.23963E+00_JPRB ,&
&  0.23818E+00_JPRB, 0.23437E+00_JPRB, 0.23016E+00_JPRB, 0.22720E+00_JPRB, 0.22302E+00_JPRB, 0.22018E+00_JPRB, 0.21643E+00_JPRB ,&
&  0.21349E+00_JPRB, 0.21092E+00_JPRB, 0.20807E+00_JPRB, 0.20594E+00_JPRB, 0.20398E+00_JPRB, 0.20254E+00_JPRB, 0.20106E+00_JPRB ,&
&  0.20022E+00_JPRB, 0.19937E+00_JPRB, 0.19882E+00_JPRB, 0.19818E+00_JPRB, 0.19776E+00_JPRB, 0.19735E+00_JPRB, 0.19703E+00_JPRB ,&
&  0.19685E+00_JPRB, 0.19721E+00_JPRB, 0.19745E+00_JPRB, 0.19789E+00_JPRB, 0.19838E+00_JPRB, 0.19862E+00_JPRB, 0.19916E+00_JPRB ,&
&  0.20024E+00_JPRB, 0.20142E+00_JPRB, 0.20208E+00_JPRB, 0.20300E+00_JPRB, 0.20405E+00_JPRB, 0.20531E+00_JPRB, 0.20711E+00_JPRB ,&
&  0.20898E+00_JPRB, 0.21110E+00_JPRB, 0.21351E+00_JPRB, 0.21586E+00_JPRB, 0.21897E+00_JPRB, 0.22217E+00_JPRB, 0.22558E+00_JPRB ,&
&  0.22976E+00_JPRB, 0.23417E+00_JPRB, 0.23880E+00_JPRB, 0.24390E+00_JPRB, 0.25139E+00_JPRB, 0.25732E+00_JPRB, 0.26458E+00_JPRB ,&
&  0.27134E+00_JPRB, 0.27527E+00_JPRB, 0.30811E+00_JPRB, 0.31286E+00_JPRB, 0.31973E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.25365E+00_JPRB, 0.26002E+00_JPRB, 0.26614E+00_JPRB, 0.27493E+00_JPRB, 0.27913E+00_JPRB, 0.31504E+00_JPRB, 0.30995E+00_JPRB ,&
&  0.30700E+00_JPRB, 0.29466E+00_JPRB, 0.29370E+00_JPRB, 0.29036E+00_JPRB, 0.28597E+00_JPRB, 0.28127E+00_JPRB, 0.27640E+00_JPRB ,&
&  0.27186E+00_JPRB, 0.26645E+00_JPRB, 0.26201E+00_JPRB, 0.25847E+00_JPRB, 0.25532E+00_JPRB, 0.25219E+00_JPRB, 0.24951E+00_JPRB ,&
&  0.24845E+00_JPRB, 0.24844E+00_JPRB, 0.24772E+00_JPRB, 0.24304E+00_JPRB, 0.23938E+00_JPRB, 0.23719E+00_JPRB, 0.23661E+00_JPRB ,&
&  0.23661E+00_JPRB, 0.23868E+00_JPRB, 0.24015E+00_JPRB, 0.24145E+00_JPRB, 0.24335E+00_JPRB, 0.24252E+00_JPRB, 0.24167E+00_JPRB ,&
&  0.24040E+00_JPRB, 0.23903E+00_JPRB, 0.23764E+00_JPRB, 0.23640E+00_JPRB, 0.23516E+00_JPRB, 0.23328E+00_JPRB, 0.23176E+00_JPRB ,&
&  0.23033E+00_JPRB, 0.22835E+00_JPRB, 0.22542E+00_JPRB, 0.22349E+00_JPRB, 0.22118E+00_JPRB, 0.21924E+00_JPRB, 0.21722E+00_JPRB ,&
&  0.21570E+00_JPRB, 0.21435E+00_JPRB, 0.21281E+00_JPRB, 0.21094E+00_JPRB, 0.20911E+00_JPRB, 0.20751E+00_JPRB, 0.20600E+00_JPRB ,&
&  0.20522E+00_JPRB, 0.20438E+00_JPRB, 0.20356E+00_JPRB, 0.20268E+00_JPRB, 0.20226E+00_JPRB, 0.20180E+00_JPRB, 0.20136E+00_JPRB ,&
&  0.20101E+00_JPRB, 0.20077E+00_JPRB, 0.20075E+00_JPRB, 0.20081E+00_JPRB, 0.20089E+00_JPRB, 0.20125E+00_JPRB, 0.20170E+00_JPRB ,&
&  0.20232E+00_JPRB, 0.20308E+00_JPRB, 0.20417E+00_JPRB, 0.20531E+00_JPRB, 0.20631E+00_JPRB, 0.20708E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20963E+00_JPRB, 0.21144E+00_JPRB, 0.21343E+00_JPRB, 0.21488E+00_JPRB, 0.21709E+00_JPRB, 0.21912E+00_JPRB, 0.22116E+00_JPRB ,&
&  0.22318E+00_JPRB, 0.22569E+00_JPRB, 0.22858E+00_JPRB, 0.23178E+00_JPRB, 0.23561E+00_JPRB, 0.23856E+00_JPRB, 0.24135E+00_JPRB ,&
&  0.24419E+00_JPRB, 0.24848E+00_JPRB, 0.25039E+00_JPRB, 0.25127E+00_JPRB, 0.25232E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.23301E+00_JPRB, 0.23300E+00_JPRB, 0.23330E+00_JPRB, 0.23363E+00_JPRB, 0.23387E+00_JPRB, 0.23443E+00_JPRB, 0.23446E+00_JPRB ,&
&  0.23430E+00_JPRB, 0.23405E+00_JPRB, 0.23437E+00_JPRB, 0.23474E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23453E+00_JPRB ,&
&  0.23378E+00_JPRB, 0.23287E+00_JPRB, 0.23132E+00_JPRB, 0.23065E+00_JPRB, 0.23016E+00_JPRB, 0.22972E+00_JPRB, 0.22892E+00_JPRB ,&
&  0.22819E+00_JPRB, 0.22694E+00_JPRB, 0.22526E+00_JPRB, 0.22415E+00_JPRB, 0.22408E+00_JPRB, 0.22456E+00_JPRB, 0.22537E+00_JPRB ,&
&  0.22601E+00_JPRB, 0.22645E+00_JPRB, 0.22666E+00_JPRB, 0.22694E+00_JPRB, 0.22732E+00_JPRB, 0.22723E+00_JPRB, 0.22701E+00_JPRB ,&
&  0.22671E+00_JPRB, 0.22616E+00_JPRB, 0.22563E+00_JPRB, 0.22517E+00_JPRB, 0.22475E+00_JPRB, 0.22422E+00_JPRB, 0.22361E+00_JPRB ,&
&  0.22290E+00_JPRB, 0.22203E+00_JPRB, 0.22094E+00_JPRB, 0.22008E+00_JPRB, 0.21916E+00_JPRB, 0.21845E+00_JPRB, 0.21753E+00_JPRB ,&
&  0.21677E+00_JPRB, 0.21607E+00_JPRB, 0.21537E+00_JPRB, 0.21471E+00_JPRB, 0.21410E+00_JPRB, 0.21352E+00_JPRB, 0.21296E+00_JPRB ,&
&  0.21242E+00_JPRB, 0.21197E+00_JPRB, 0.21147E+00_JPRB, 0.21101E+00_JPRB, 0.21068E+00_JPRB, 0.21042E+00_JPRB, 0.21019E+00_JPRB ,&
&  0.21001E+00_JPRB, 0.20996E+00_JPRB, 0.20995E+00_JPRB, 0.20994E+00_JPRB, 0.20994E+00_JPRB, 0.21006E+00_JPRB, 0.21018E+00_JPRB ,&
&  0.21034E+00_JPRB, 0.21044E+00_JPRB, 0.21055E+00_JPRB, 0.21072E+00_JPRB, 0.21090E+00_JPRB, 0.21112E+00_JPRB, 0.21142E+00_JPRB ,&
&  0.21179E+00_JPRB, 0.21222E+00_JPRB, 0.21269E+00_JPRB, 0.21293E+00_JPRB, 0.21360E+00_JPRB, 0.21437E+00_JPRB, 0.21529E+00_JPRB ,&
&  0.21631E+00_JPRB, 0.21754E+00_JPRB, 0.21892E+00_JPRB, 0.22059E+00_JPRB, 0.22236E+00_JPRB, 0.22344E+00_JPRB, 0.22453E+00_JPRB ,&
&  0.22566E+00_JPRB, 0.22703E+00_JPRB, 0.22831E+00_JPRB, 0.22962E+00_JPRB, 0.23116E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB, 0.21653E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB, 0.77323E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.78990E+00_JPRB, 0.78924E+00_JPRB, 0.78861E+00_JPRB, 0.78788E+00_JPRB, 0.78695E+00_JPRB, 0.78596E+00_JPRB, 0.78514E+00_JPRB ,&
&  0.78425E+00_JPRB, 0.78368E+00_JPRB, 0.78256E+00_JPRB, 0.78140E+00_JPRB, 0.78017E+00_JPRB, 0.77908E+00_JPRB, 0.77789E+00_JPRB ,&
&  0.77661E+00_JPRB, 0.77532E+00_JPRB, 0.77423E+00_JPRB, 0.77299E+00_JPRB, 0.77162E+00_JPRB, 0.77032E+00_JPRB, 0.76906E+00_JPRB ,&
&  0.76796E+00_JPRB, 0.76671E+00_JPRB, 0.76546E+00_JPRB, 0.76397E+00_JPRB, 0.76300E+00_JPRB, 0.76162E+00_JPRB, 0.76050E+00_JPRB ,&
&  0.75921E+00_JPRB, 0.75832E+00_JPRB, 0.75730E+00_JPRB, 0.75640E+00_JPRB, 0.75532E+00_JPRB, 0.75509E+00_JPRB, 0.75447E+00_JPRB ,&
&  0.75413E+00_JPRB, 0.75380E+00_JPRB, 0.75379E+00_JPRB, 0.75368E+00_JPRB, 0.75365E+00_JPRB, 0.75378E+00_JPRB, 0.75442E+00_JPRB ,&
&  0.75496E+00_JPRB, 0.75560E+00_JPRB, 0.75660E+00_JPRB, 0.75755E+00_JPRB, 0.75842E+00_JPRB, 0.75927E+00_JPRB, 0.76022E+00_JPRB ,&
&  0.76160E+00_JPRB, 0.76303E+00_JPRB, 0.76442E+00_JPRB, 0.76590E+00_JPRB, 0.76736E+00_JPRB, 0.76850E+00_JPRB, 0.76942E+00_JPRB ,&
&  0.77004E+00_JPRB, 0.77081E+00_JPRB, 0.77149E+00_JPRB, 0.77216E+00_JPRB, 0.77231E+00_JPRB, 0.77276E+00_JPRB, 0.77311E+00_JPRB ,&
&  0.77339E+00_JPRB, 0.77302E+00_JPRB, 0.77314E+00_JPRB, 0.77309E+00_JPRB, 0.77325E+00_JPRB, 0.77324E+00_JPRB, 0.77379E+00_JPRB ,&
&  0.77427E+00_JPRB, 0.77512E+00_JPRB, 0.77605E+00_JPRB, 0.77729E+00_JPRB, 0.77822E+00_JPRB, 0.77924E+00_JPRB, 0.78015E+00_JPRB ,&
&  0.78119E+00_JPRB, 0.78231E+00_JPRB, 0.78348E+00_JPRB, 0.78436E+00_JPRB, 0.78522E+00_JPRB, 0.78620E+00_JPRB, 0.78717E+00_JPRB ,&
&  0.78833E+00_JPRB, 0.78905E+00_JPRB, 0.78978E+00_JPRB, 0.79042E+00_JPRB, 0.79073E+00_JPRB, 0.79094E+00_JPRB, 0.79101E+00_JPRB ,&
&  0.79095E+00_JPRB, 0.79082E+00_JPRB, 0.79060E+00_JPRB, 0.79047E+00_JPRB, 0.79033E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.80089E+00_JPRB, 0.80054E+00_JPRB, 0.79988E+00_JPRB, 0.79874E+00_JPRB, 0.79739E+00_JPRB, 0.79569E+00_JPRB, 0.79401E+00_JPRB ,&
&  0.79197E+00_JPRB, 0.79015E+00_JPRB, 0.78786E+00_JPRB, 0.78547E+00_JPRB, 0.78292E+00_JPRB, 0.78057E+00_JPRB, 0.77817E+00_JPRB ,&
&  0.77580E+00_JPRB, 0.77339E+00_JPRB, 0.77165E+00_JPRB, 0.76986E+00_JPRB, 0.76782E+00_JPRB, 0.76525E+00_JPRB, 0.76286E+00_JPRB ,&
&  0.76052E+00_JPRB, 0.75778E+00_JPRB, 0.75481E+00_JPRB, 0.75003E+00_JPRB, 0.74604E+00_JPRB, 0.74226E+00_JPRB, 0.73899E+00_JPRB ,&
&  0.73579E+00_JPRB, 0.73298E+00_JPRB, 0.73086E+00_JPRB, 0.72934E+00_JPRB, 0.73021E+00_JPRB, 0.73035E+00_JPRB, 0.72999E+00_JPRB ,&
&  0.72943E+00_JPRB, 0.72712E+00_JPRB, 0.72637E+00_JPRB, 0.72613E+00_JPRB, 0.72614E+00_JPRB, 0.72793E+00_JPRB, 0.72952E+00_JPRB ,&
&  0.73121E+00_JPRB, 0.73266E+00_JPRB, 0.73536E+00_JPRB, 0.73799E+00_JPRB, 0.74047E+00_JPRB, 0.74310E+00_JPRB, 0.74484E+00_JPRB ,&
&  0.74682E+00_JPRB, 0.74969E+00_JPRB, 0.75245E+00_JPRB, 0.75543E+00_JPRB, 0.75731E+00_JPRB, 0.75901E+00_JPRB, 0.76033E+00_JPRB ,&
&  0.76139E+00_JPRB, 0.76265E+00_JPRB, 0.76399E+00_JPRB, 0.76522E+00_JPRB, 0.76592E+00_JPRB, 0.76671E+00_JPRB, 0.76727E+00_JPRB ,&
&  0.76756E+00_JPRB, 0.76696E+00_JPRB, 0.76683E+00_JPRB, 0.76659E+00_JPRB, 0.76609E+00_JPRB, 0.76523E+00_JPRB, 0.76512E+00_JPRB ,&
&  0.76497E+00_JPRB, 0.76458E+00_JPRB, 0.76381E+00_JPRB, 0.76440E+00_JPRB, 0.76508E+00_JPRB, 0.76544E+00_JPRB, 0.76537E+00_JPRB ,&
&  0.76813E+00_JPRB, 0.77222E+00_JPRB, 0.77733E+00_JPRB, 0.78421E+00_JPRB, 0.78945E+00_JPRB, 0.79290E+00_JPRB, 0.79546E+00_JPRB ,&
&  0.79661E+00_JPRB, 0.79755E+00_JPRB, 0.79813E+00_JPRB, 0.79887E+00_JPRB, 0.79891E+00_JPRB, 0.79959E+00_JPRB, 0.80099E+00_JPRB ,&
&  0.80198E+00_JPRB, 0.80222E+00_JPRB, 0.80211E+00_JPRB, 0.80180E+00_JPRB, 0.80134E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.80590E+00_JPRB, 0.80389E+00_JPRB, 0.80181E+00_JPRB, 0.79879E+00_JPRB, 0.79692E+00_JPRB, 0.79417E+00_JPRB, 0.79188E+00_JPRB ,&
&  0.78912E+00_JPRB, 0.78617E+00_JPRB, 0.78385E+00_JPRB, 0.78189E+00_JPRB, 0.77981E+00_JPRB, 0.77773E+00_JPRB, 0.77565E+00_JPRB ,&
&  0.77345E+00_JPRB, 0.77099E+00_JPRB, 0.76975E+00_JPRB, 0.76816E+00_JPRB, 0.76634E+00_JPRB, 0.76421E+00_JPRB, 0.75951E+00_JPRB ,&
&  0.75441E+00_JPRB, 0.74808E+00_JPRB, 0.74135E+00_JPRB, 0.73527E+00_JPRB, 0.73191E+00_JPRB, 0.70756E+00_JPRB, 0.70340E+00_JPRB ,&
&  0.71061E+00_JPRB, 0.71497E+00_JPRB, 0.72072E+00_JPRB, 0.72657E+00_JPRB, 0.71775E+00_JPRB, 0.70419E+00_JPRB, 0.69594E+00_JPRB ,&
&  0.67454E+00_JPRB, 0.68121E+00_JPRB, 0.68401E+00_JPRB, 0.68710E+00_JPRB, 0.69255E+00_JPRB, 0.69304E+00_JPRB, 0.69718E+00_JPRB ,&
&  0.71179E+00_JPRB, 0.71238E+00_JPRB, 0.71461E+00_JPRB, 0.71683E+00_JPRB, 0.71899E+00_JPRB, 0.72120E+00_JPRB, 0.72513E+00_JPRB ,&
&  0.72843E+00_JPRB, 0.73238E+00_JPRB, 0.73680E+00_JPRB, 0.73929E+00_JPRB, 0.74170E+00_JPRB, 0.74478E+00_JPRB, 0.74760E+00_JPRB ,&
&  0.74916E+00_JPRB, 0.75088E+00_JPRB, 0.75276E+00_JPRB, 0.75478E+00_JPRB, 0.75692E+00_JPRB, 0.75855E+00_JPRB, 0.75994E+00_JPRB ,&
&  0.76114E+00_JPRB, 0.76095E+00_JPRB, 0.76099E+00_JPRB, 0.76137E+00_JPRB, 0.76136E+00_JPRB, 0.76111E+00_JPRB, 0.76157E+00_JPRB ,&
&  0.76268E+00_JPRB, 0.76367E+00_JPRB, 0.76515E+00_JPRB, 0.76737E+00_JPRB, 0.77242E+00_JPRB, 0.77929E+00_JPRB, 0.79246E+00_JPRB ,&
&  0.80319E+00_JPRB, 0.81258E+00_JPRB, 0.81622E+00_JPRB, 0.81982E+00_JPRB, 0.82321E+00_JPRB, 0.82459E+00_JPRB, 0.82541E+00_JPRB ,&
&  0.82446E+00_JPRB, 0.82238E+00_JPRB, 0.81962E+00_JPRB, 0.81659E+00_JPRB, 0.81416E+00_JPRB, 0.81197E+00_JPRB, 0.81083E+00_JPRB ,&
&  0.81060E+00_JPRB, 0.81144E+00_JPRB, 0.81149E+00_JPRB, 0.80973E+00_JPRB, 0.80757E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.81778E+00_JPRB, 0.81292E+00_JPRB, 0.81036E+00_JPRB, 0.80596E+00_JPRB, 0.80227E+00_JPRB, 0.79728E+00_JPRB, 0.79308E+00_JPRB ,&
&  0.78837E+00_JPRB, 0.78419E+00_JPRB, 0.78038E+00_JPRB, 0.77688E+00_JPRB, 0.77494E+00_JPRB, 0.77301E+00_JPRB, 0.77175E+00_JPRB ,&
&  0.77027E+00_JPRB, 0.76839E+00_JPRB, 0.76722E+00_JPRB, 0.76581E+00_JPRB, 0.76457E+00_JPRB, 0.76325E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75253E+00_JPRB, 0.74240E+00_JPRB, 0.72038E+00_JPRB, 0.72191E+00_JPRB, 0.71810E+00_JPRB, 0.71520E+00_JPRB, 0.71690E+00_JPRB ,&
&  0.70905E+00_JPRB, 0.70697E+00_JPRB, 0.69641E+00_JPRB, 0.68517E+00_JPRB, 0.67806E+00_JPRB, 0.66442E+00_JPRB, 0.66062E+00_JPRB ,&
&  0.65504E+00_JPRB, 0.65203E+00_JPRB, 0.65257E+00_JPRB, 0.65216E+00_JPRB, 0.65833E+00_JPRB, 0.66232E+00_JPRB, 0.66798E+00_JPRB ,&
&  0.67753E+00_JPRB, 0.68549E+00_JPRB, 0.69204E+00_JPRB, 0.68683E+00_JPRB, 0.69047E+00_JPRB, 0.69417E+00_JPRB, 0.69895E+00_JPRB ,&
&  0.70355E+00_JPRB, 0.70725E+00_JPRB, 0.71126E+00_JPRB, 0.71463E+00_JPRB, 0.71824E+00_JPRB, 0.72301E+00_JPRB, 0.72787E+00_JPRB ,&
&  0.73346E+00_JPRB, 0.73740E+00_JPRB, 0.73994E+00_JPRB, 0.74280E+00_JPRB, 0.74578E+00_JPRB, 0.74884E+00_JPRB, 0.75270E+00_JPRB ,&
&  0.75732E+00_JPRB, 0.76280E+00_JPRB, 0.76878E+00_JPRB, 0.77503E+00_JPRB, 0.78103E+00_JPRB, 0.78873E+00_JPRB, 0.79759E+00_JPRB ,&
&  0.81025E+00_JPRB, 0.82184E+00_JPRB, 0.83879E+00_JPRB, 0.84944E+00_JPRB, 0.87100E+00_JPRB, 0.89083E+00_JPRB, 0.90184E+00_JPRB ,&
&  0.89924E+00_JPRB, 0.89251E+00_JPRB, 0.88617E+00_JPRB, 0.87933E+00_JPRB, 0.87387E+00_JPRB, 0.86683E+00_JPRB, 0.86220E+00_JPRB ,&
&  0.85692E+00_JPRB, 0.85222E+00_JPRB, 0.84736E+00_JPRB, 0.84283E+00_JPRB, 0.83883E+00_JPRB, 0.83444E+00_JPRB, 0.83112E+00_JPRB ,&
&  0.82907E+00_JPRB, 0.82859E+00_JPRB, 0.82768E+00_JPRB, 0.82572E+00_JPRB, 0.82214E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.83272E+00_JPRB, 0.83044E+00_JPRB, 0.83014E+00_JPRB, 0.82671E+00_JPRB, 0.82192E+00_JPRB, 0.81687E+00_JPRB, 0.80788E+00_JPRB ,&
&  0.79895E+00_JPRB, 0.79006E+00_JPRB, 0.78202E+00_JPRB, 0.77579E+00_JPRB, 0.77056E+00_JPRB, 0.76665E+00_JPRB, 0.76502E+00_JPRB ,&
&  0.76277E+00_JPRB, 0.76236E+00_JPRB, 0.76213E+00_JPRB, 0.76244E+00_JPRB, 0.76370E+00_JPRB, 0.76447E+00_JPRB, 0.76512E+00_JPRB ,&
&  0.76100E+00_JPRB, 0.74966E+00_JPRB, 0.73411E+00_JPRB, 0.73610E+00_JPRB, 0.73744E+00_JPRB, 0.74177E+00_JPRB, 0.74624E+00_JPRB ,&
&  0.74904E+00_JPRB, 0.71721E+00_JPRB, 0.72352E+00_JPRB, 0.70534E+00_JPRB, 0.69323E+00_JPRB, 0.68054E+00_JPRB, 0.65800E+00_JPRB ,&
&  0.65007E+00_JPRB, 0.63801E+00_JPRB, 0.63131E+00_JPRB, 0.63153E+00_JPRB, 0.63476E+00_JPRB, 0.64338E+00_JPRB, 0.65410E+00_JPRB ,&
&  0.65696E+00_JPRB, 0.67481E+00_JPRB, 0.67670E+00_JPRB, 0.67627E+00_JPRB, 0.67937E+00_JPRB, 0.68306E+00_JPRB, 0.68033E+00_JPRB ,&
&  0.68159E+00_JPRB, 0.68122E+00_JPRB, 0.68185E+00_JPRB, 0.68789E+00_JPRB, 0.69789E+00_JPRB, 0.70367E+00_JPRB, 0.70915E+00_JPRB ,&
&  0.71658E+00_JPRB, 0.72096E+00_JPRB, 0.72366E+00_JPRB, 0.72708E+00_JPRB, 0.73182E+00_JPRB, 0.73697E+00_JPRB, 0.74510E+00_JPRB ,&
&  0.75479E+00_JPRB, 0.77183E+00_JPRB, 0.78760E+00_JPRB, 0.81078E+00_JPRB, 0.83214E+00_JPRB, 0.85994E+00_JPRB, 0.88374E+00_JPRB ,&
&  0.90801E+00_JPRB, 0.92696E+00_JPRB, 0.94722E+00_JPRB, 0.96569E+00_JPRB, 0.97947E+00_JPRB, 0.97227E+00_JPRB, 0.95161E+00_JPRB ,&
&  0.93699E+00_JPRB, 0.91564E+00_JPRB, 0.90201E+00_JPRB, 0.89101E+00_JPRB, 0.88290E+00_JPRB, 0.87865E+00_JPRB, 0.87437E+00_JPRB ,&
&  0.87153E+00_JPRB, 0.86749E+00_JPRB, 0.86303E+00_JPRB, 0.85725E+00_JPRB, 0.85273E+00_JPRB, 0.84825E+00_JPRB, 0.84547E+00_JPRB ,&
&  0.84183E+00_JPRB, 0.84083E+00_JPRB, 0.83825E+00_JPRB, 0.83561E+00_JPRB, 0.83376E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.84854E+00_JPRB, 0.85044E+00_JPRB, 0.84535E+00_JPRB, 0.84032E+00_JPRB, 0.82963E+00_JPRB, 0.82015E+00_JPRB, 0.80637E+00_JPRB ,&
&  0.79304E+00_JPRB, 0.77990E+00_JPRB, 0.76816E+00_JPRB, 0.75958E+00_JPRB, 0.75258E+00_JPRB, 0.74768E+00_JPRB, 0.74574E+00_JPRB ,&
&  0.74521E+00_JPRB, 0.74623E+00_JPRB, 0.74760E+00_JPRB, 0.74963E+00_JPRB, 0.75358E+00_JPRB, 0.75814E+00_JPRB, 0.76308E+00_JPRB ,&
&  0.76470E+00_JPRB, 0.76028E+00_JPRB, 0.74649E+00_JPRB, 0.74550E+00_JPRB, 0.75377E+00_JPRB, 0.75219E+00_JPRB, 0.77051E+00_JPRB ,&
&  0.78199E+00_JPRB, 0.79030E+00_JPRB, 0.78208E+00_JPRB, 0.77612E+00_JPRB, 0.76408E+00_JPRB, 0.68921E+00_JPRB, 0.68091E+00_JPRB ,&
&  0.66602E+00_JPRB, 0.64716E+00_JPRB, 0.63832E+00_JPRB, 0.63145E+00_JPRB, 0.62990E+00_JPRB, 0.63945E+00_JPRB, 0.65370E+00_JPRB ,&
&  0.65837E+00_JPRB, 0.68352E+00_JPRB, 0.68278E+00_JPRB, 0.68066E+00_JPRB, 0.68050E+00_JPRB, 0.68319E+00_JPRB, 0.67945E+00_JPRB ,&
&  0.68090E+00_JPRB, 0.68209E+00_JPRB, 0.68443E+00_JPRB, 0.69329E+00_JPRB, 0.70256E+00_JPRB, 0.71049E+00_JPRB, 0.72113E+00_JPRB ,&
&  0.74003E+00_JPRB, 0.75372E+00_JPRB, 0.76178E+00_JPRB, 0.76902E+00_JPRB, 0.78085E+00_JPRB, 0.79543E+00_JPRB, 0.81544E+00_JPRB ,&
&  0.82987E+00_JPRB, 0.84706E+00_JPRB, 0.86511E+00_JPRB, 0.88431E+00_JPRB, 0.90268E+00_JPRB, 0.92191E+00_JPRB, 0.94183E+00_JPRB ,&
&  0.95156E+00_JPRB, 0.96509E+00_JPRB, 0.97564E+00_JPRB, 0.99842E+00_JPRB, 0.99427E+00_JPRB, 0.97237E+00_JPRB, 0.95625E+00_JPRB ,&
&  0.94319E+00_JPRB, 0.94277E+00_JPRB, 0.93063E+00_JPRB, 0.92198E+00_JPRB, 0.91285E+00_JPRB, 0.90851E+00_JPRB, 0.90216E+00_JPRB ,&
&  0.89743E+00_JPRB, 0.89148E+00_JPRB, 0.88391E+00_JPRB, 0.87494E+00_JPRB, 0.86857E+00_JPRB, 0.86202E+00_JPRB, 0.85793E+00_JPRB ,&
&  0.85231E+00_JPRB, 0.84964E+00_JPRB, 0.84783E+00_JPRB, 0.84705E+00_JPRB, 0.84551E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.85790E+00_JPRB, 0.85538E+00_JPRB, 0.84289E+00_JPRB, 0.83566E+00_JPRB, 0.81971E+00_JPRB, 0.80274E+00_JPRB, 0.78230E+00_JPRB ,&
&  0.76618E+00_JPRB, 0.75383E+00_JPRB, 0.74284E+00_JPRB, 0.73676E+00_JPRB, 0.73365E+00_JPRB, 0.73215E+00_JPRB, 0.73387E+00_JPRB ,&
&  0.73651E+00_JPRB, 0.73437E+00_JPRB, 0.73772E+00_JPRB, 0.74403E+00_JPRB, 0.74978E+00_JPRB, 0.75507E+00_JPRB, 0.76088E+00_JPRB ,&
&  0.76175E+00_JPRB, 0.76109E+00_JPRB, 0.75917E+00_JPRB, 0.75374E+00_JPRB, 0.75282E+00_JPRB, 0.76488E+00_JPRB, 0.78357E+00_JPRB ,&
&  0.78444E+00_JPRB, 0.79049E+00_JPRB, 0.80471E+00_JPRB, 0.81507E+00_JPRB, 0.80257E+00_JPRB, 0.76501E+00_JPRB, 0.71421E+00_JPRB ,&
&  0.67923E+00_JPRB, 0.64415E+00_JPRB, 0.62797E+00_JPRB, 0.62019E+00_JPRB, 0.62559E+00_JPRB, 0.63811E+00_JPRB, 0.66100E+00_JPRB ,&
&  0.67588E+00_JPRB, 0.68255E+00_JPRB, 0.68793E+00_JPRB, 0.69238E+00_JPRB, 0.69205E+00_JPRB, 0.69487E+00_JPRB, 0.69584E+00_JPRB ,&
&  0.70311E+00_JPRB, 0.70930E+00_JPRB, 0.71717E+00_JPRB, 0.73117E+00_JPRB, 0.75137E+00_JPRB, 0.77386E+00_JPRB, 0.80658E+00_JPRB ,&
&  0.87419E+00_JPRB, 0.90349E+00_JPRB, 0.92803E+00_JPRB, 0.94185E+00_JPRB, 0.96073E+00_JPRB, 0.96866E+00_JPRB, 0.95867E+00_JPRB ,&
&  0.95827E+00_JPRB, 0.97740E+00_JPRB, 0.98122E+00_JPRB, 0.98471E+00_JPRB, 0.98848E+00_JPRB, 0.99300E+00_JPRB, 0.99821E+00_JPRB ,&
&  0.98470E+00_JPRB, 0.99624E+00_JPRB, 0.10740E+01_JPRB, 0.10945E+01_JPRB, 0.10736E+01_JPRB, 0.10479E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.99711E+00_JPRB, 0.97675E+00_JPRB, 0.95941E+00_JPRB, 0.94480E+00_JPRB, 0.93444E+00_JPRB, 0.92924E+00_JPRB, 0.92876E+00_JPRB ,&
&  0.92112E+00_JPRB, 0.91147E+00_JPRB, 0.89965E+00_JPRB, 0.89125E+00_JPRB, 0.88457E+00_JPRB, 0.87669E+00_JPRB, 0.87165E+00_JPRB ,&
&  0.86689E+00_JPRB, 0.86638E+00_JPRB, 0.86590E+00_JPRB, 0.86540E+00_JPRB, 0.86282E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.85714E+00_JPRB, 0.85534E+00_JPRB, 0.84704E+00_JPRB, 0.82868E+00_JPRB, 0.79781E+00_JPRB, 0.76834E+00_JPRB, 0.74739E+00_JPRB ,&
&  0.73732E+00_JPRB, 0.73161E+00_JPRB, 0.72422E+00_JPRB, 0.72074E+00_JPRB, 0.71961E+00_JPRB, 0.72060E+00_JPRB, 0.72373E+00_JPRB ,&
&  0.72886E+00_JPRB, 0.73417E+00_JPRB, 0.73780E+00_JPRB, 0.74000E+00_JPRB, 0.74521E+00_JPRB, 0.75385E+00_JPRB, 0.76378E+00_JPRB ,&
&  0.77164E+00_JPRB, 0.77413E+00_JPRB, 0.77744E+00_JPRB, 0.77965E+00_JPRB, 0.78140E+00_JPRB, 0.78584E+00_JPRB, 0.78757E+00_JPRB ,&
&  0.79621E+00_JPRB, 0.80952E+00_JPRB, 0.81874E+00_JPRB, 0.83631E+00_JPRB, 0.84962E+00_JPRB, 0.81273E+00_JPRB, 0.74795E+00_JPRB ,&
&  0.68578E+00_JPRB, 0.64197E+00_JPRB, 0.62312E+00_JPRB, 0.61853E+00_JPRB, 0.62883E+00_JPRB, 0.65264E+00_JPRB, 0.68239E+00_JPRB ,&
&  0.69103E+00_JPRB, 0.69768E+00_JPRB, 0.70796E+00_JPRB, 0.71331E+00_JPRB, 0.71333E+00_JPRB, 0.71213E+00_JPRB, 0.71403E+00_JPRB ,&
&  0.72265E+00_JPRB, 0.74425E+00_JPRB, 0.77971E+00_JPRB, 0.82949E+00_JPRB, 0.89865E+00_JPRB, 0.93093E+00_JPRB, 0.99114E+00_JPRB ,&
&  0.10862E+01_JPRB, 0.11016E+01_JPRB, 0.11236E+01_JPRB, 0.11289E+01_JPRB, 0.11306E+01_JPRB, 0.11234E+01_JPRB, 0.11223E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.11155E+01_JPRB, 0.10983E+01_JPRB, 0.10782E+01_JPRB, 0.10715E+01_JPRB, 0.10774E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.11807E+01_JPRB, 0.13234E+01_JPRB, 0.12036E+01_JPRB, 0.11488E+01_JPRB, 0.11283E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10418E+01_JPRB, 0.10285E+01_JPRB, 0.10138E+01_JPRB, 0.98275E+00_JPRB, 0.96128E+00_JPRB, 0.94527E+00_JPRB ,&
&  0.93388E+00_JPRB, 0.92161E+00_JPRB, 0.91219E+00_JPRB, 0.90551E+00_JPRB, 0.89817E+00_JPRB, 0.89202E+00_JPRB, 0.88995E+00_JPRB ,&
&  0.88610E+00_JPRB, 0.88220E+00_JPRB, 0.87774E+00_JPRB, 0.86981E+00_JPRB, 0.86286E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.85186E+00_JPRB, 0.84977E+00_JPRB, 0.83340E+00_JPRB, 0.81261E+00_JPRB, 0.77687E+00_JPRB, 0.74983E+00_JPRB, 0.73172E+00_JPRB ,&
&  0.72223E+00_JPRB, 0.71411E+00_JPRB, 0.70854E+00_JPRB, 0.70595E+00_JPRB, 0.70652E+00_JPRB, 0.71056E+00_JPRB, 0.71431E+00_JPRB ,&
&  0.71791E+00_JPRB, 0.72387E+00_JPRB, 0.73149E+00_JPRB, 0.73834E+00_JPRB, 0.74516E+00_JPRB, 0.75341E+00_JPRB, 0.76354E+00_JPRB ,&
&  0.77579E+00_JPRB, 0.78445E+00_JPRB, 0.79079E+00_JPRB, 0.79835E+00_JPRB, 0.81220E+00_JPRB, 0.83283E+00_JPRB, 0.84736E+00_JPRB ,&
&  0.84659E+00_JPRB, 0.84231E+00_JPRB, 0.85573E+00_JPRB, 0.87443E+00_JPRB, 0.88830E+00_JPRB, 0.86082E+00_JPRB, 0.77470E+00_JPRB ,&
&  0.69995E+00_JPRB, 0.65569E+00_JPRB, 0.64682E+00_JPRB, 0.65953E+00_JPRB, 0.68580E+00_JPRB, 0.70308E+00_JPRB, 0.71634E+00_JPRB ,&
&  0.72663E+00_JPRB, 0.72986E+00_JPRB, 0.73223E+00_JPRB, 0.73820E+00_JPRB, 0.74087E+00_JPRB, 0.74164E+00_JPRB, 0.74933E+00_JPRB ,&
&  0.77145E+00_JPRB, 0.80831E+00_JPRB, 0.84264E+00_JPRB, 0.85625E+00_JPRB, 0.91033E+00_JPRB, 0.10380E+01_JPRB, 0.11590E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.12159E+01_JPRB, 0.12399E+01_JPRB, 0.12890E+01_JPRB, 0.13085E+01_JPRB, 0.13211E+01_JPRB, 0.13160E+01_JPRB ,&
&  0.12941E+01_JPRB, 0.12564E+01_JPRB, 0.12209E+01_JPRB, 0.12114E+01_JPRB, 0.12368E+01_JPRB, 0.12887E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.13385E+01_JPRB, 0.12885E+01_JPRB, 0.11733E+01_JPRB, 0.10717E+01_JPRB, 0.10388E+01_JPRB, 0.10450E+01_JPRB, 0.10605E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.10455E+01_JPRB, 0.10345E+01_JPRB, 0.10280E+01_JPRB, 0.97924E+00_JPRB, 0.94854E+00_JPRB, 0.92377E+00_JPRB ,&
&  0.90836E+00_JPRB, 0.89718E+00_JPRB, 0.89821E+00_JPRB, 0.89600E+00_JPRB, 0.89303E+00_JPRB, 0.88780E+00_JPRB, 0.88033E+00_JPRB ,&
&  0.87650E+00_JPRB, 0.87052E+00_JPRB, 0.86154E+00_JPRB, 0.85377E+00_JPRB, 0.84987E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.86904E+00_JPRB, 0.85312E+00_JPRB, 0.83611E+00_JPRB, 0.81075E+00_JPRB, 0.79053E+00_JPRB, 0.77886E+00_JPRB, 0.76884E+00_JPRB ,&
&  0.76825E+00_JPRB, 0.74463E+00_JPRB, 0.71991E+00_JPRB, 0.70372E+00_JPRB, 0.69586E+00_JPRB, 0.69483E+00_JPRB, 0.70162E+00_JPRB ,&
&  0.71189E+00_JPRB, 0.72156E+00_JPRB, 0.73147E+00_JPRB, 0.74320E+00_JPRB, 0.75494E+00_JPRB, 0.76440E+00_JPRB, 0.77427E+00_JPRB ,&
&  0.78803E+00_JPRB, 0.80281E+00_JPRB, 0.81910E+00_JPRB, 0.84295E+00_JPRB, 0.86671E+00_JPRB, 0.89597E+00_JPRB, 0.90959E+00_JPRB ,&
&  0.90925E+00_JPRB, 0.89985E+00_JPRB, 0.89822E+00_JPRB, 0.91363E+00_JPRB, 0.92182E+00_JPRB, 0.89759E+00_JPRB, 0.79862E+00_JPRB ,&
&  0.71798E+00_JPRB, 0.69320E+00_JPRB, 0.71197E+00_JPRB, 0.74426E+00_JPRB, 0.75294E+00_JPRB, 0.76093E+00_JPRB, 0.76208E+00_JPRB ,&
&  0.75625E+00_JPRB, 0.75264E+00_JPRB, 0.76142E+00_JPRB, 0.77375E+00_JPRB, 0.77618E+00_JPRB, 0.78093E+00_JPRB, 0.79585E+00_JPRB ,&
&  0.83251E+00_JPRB, 0.87865E+00_JPRB, 0.92154E+00_JPRB, 0.91974E+00_JPRB, 0.10254E+01_JPRB, 0.13698E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.14823E+01_JPRB, 0.15037E+01_JPRB, 0.15440E+01_JPRB, 0.15674E+01_JPRB, 0.15363E+01_JPRB, 0.15201E+01_JPRB, 0.15120E+01_JPRB ,&
&  0.14679E+01_JPRB, 0.14433E+01_JPRB, 0.14317E+01_JPRB, 0.14742E+01_JPRB, 0.14886E+01_JPRB, 0.14822E+01_JPRB, 0.14770E+01_JPRB ,&
&  0.14453E+01_JPRB, 0.13573E+01_JPRB, 0.12576E+01_JPRB, 0.11665E+01_JPRB, 0.11219E+01_JPRB, 0.10940E+01_JPRB, 0.10777E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10523E+01_JPRB, 0.10527E+01_JPRB, 0.10522E+01_JPRB, 0.10139E+01_JPRB, 0.96071E+00_JPRB, 0.92736E+00_JPRB ,&
&  0.90682E+00_JPRB, 0.89024E+00_JPRB, 0.87259E+00_JPRB, 0.86631E+00_JPRB, 0.87603E+00_JPRB, 0.88526E+00_JPRB, 0.88129E+00_JPRB ,&
&  0.88308E+00_JPRB, 0.87633E+00_JPRB, 0.87295E+00_JPRB, 0.87211E+00_JPRB, 0.87648E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.86853E+00_JPRB, 0.85548E+00_JPRB, 0.83928E+00_JPRB, 0.82256E+00_JPRB, 0.81479E+00_JPRB, 0.80685E+00_JPRB, 0.78761E+00_JPRB ,&
&  0.79000E+00_JPRB, 0.74935E+00_JPRB, 0.72300E+00_JPRB, 0.71311E+00_JPRB, 0.71312E+00_JPRB, 0.70840E+00_JPRB, 0.70181E+00_JPRB ,&
&  0.70545E+00_JPRB, 0.71942E+00_JPRB, 0.73691E+00_JPRB, 0.75130E+00_JPRB, 0.76566E+00_JPRB, 0.77752E+00_JPRB, 0.78852E+00_JPRB ,&
&  0.80417E+00_JPRB, 0.82355E+00_JPRB, 0.84831E+00_JPRB, 0.87709E+00_JPRB, 0.90438E+00_JPRB, 0.93814E+00_JPRB, 0.95553E+00_JPRB ,&
&  0.96743E+00_JPRB, 0.96466E+00_JPRB, 0.94598E+00_JPRB, 0.94067E+00_JPRB, 0.93441E+00_JPRB, 0.92500E+00_JPRB, 0.88779E+00_JPRB ,&
&  0.77982E+00_JPRB, 0.76507E+00_JPRB, 0.80132E+00_JPRB, 0.80608E+00_JPRB, 0.80275E+00_JPRB, 0.79415E+00_JPRB, 0.79115E+00_JPRB ,&
&  0.79183E+00_JPRB, 0.79203E+00_JPRB, 0.79698E+00_JPRB, 0.79820E+00_JPRB, 0.80379E+00_JPRB, 0.81467E+00_JPRB, 0.84956E+00_JPRB ,&
&  0.92256E+00_JPRB, 0.90817E+00_JPRB, 0.93632E+00_JPRB, 0.10185E+01_JPRB, 0.12943E+01_JPRB, 0.14774E+01_JPRB, 0.16035E+01_JPRB ,&
&  0.17773E+01_JPRB, 0.18242E+01_JPRB, 0.18221E+01_JPRB, 0.18201E+01_JPRB, 0.18180E+01_JPRB, 0.18191E+01_JPRB, 0.18249E+01_JPRB ,&
&  0.17679E+01_JPRB, 0.17504E+01_JPRB, 0.17371E+01_JPRB, 0.17511E+01_JPRB, 0.17203E+01_JPRB, 0.16798E+01_JPRB, 0.16311E+01_JPRB ,&
&  0.15721E+01_JPRB, 0.14976E+01_JPRB, 0.13957E+01_JPRB, 0.12984E+01_JPRB, 0.12169E+01_JPRB, 0.11780E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.11193E+01_JPRB, 0.11064E+01_JPRB, 0.10919E+01_JPRB, 0.10746E+01_JPRB, 0.10533E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.99280E+00_JPRB, 0.95550E+00_JPRB, 0.91200E+00_JPRB, 0.89799E+00_JPRB, 0.90398E+00_JPRB, 0.91300E+00_JPRB, 0.91885E+00_JPRB ,&
&  0.92668E+00_JPRB, 0.91313E+00_JPRB, 0.90611E+00_JPRB, 0.89681E+00_JPRB, 0.88708E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.87038E+00_JPRB, 0.86529E+00_JPRB, 0.85279E+00_JPRB, 0.84253E+00_JPRB, 0.84358E+00_JPRB, 0.83783E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.86154E+00_JPRB, 0.83670E+00_JPRB, 0.84266E+00_JPRB, 0.77254E+00_JPRB, 0.74350E+00_JPRB, 0.72945E+00_JPRB, 0.71271E+00_JPRB ,&
&  0.71313E+00_JPRB, 0.72438E+00_JPRB, 0.73675E+00_JPRB, 0.75067E+00_JPRB, 0.76976E+00_JPRB, 0.78393E+00_JPRB, 0.79515E+00_JPRB ,&
&  0.80963E+00_JPRB, 0.82906E+00_JPRB, 0.85450E+00_JPRB, 0.87808E+00_JPRB, 0.90654E+00_JPRB, 0.94311E+00_JPRB, 0.96375E+00_JPRB ,&
&  0.94316E+00_JPRB, 0.93185E+00_JPRB, 0.93712E+00_JPRB, 0.93030E+00_JPRB, 0.93693E+00_JPRB, 0.93226E+00_JPRB, 0.93324E+00_JPRB ,&
&  0.91650E+00_JPRB, 0.86728E+00_JPRB, 0.87507E+00_JPRB, 0.85813E+00_JPRB, 0.84428E+00_JPRB, 0.83098E+00_JPRB, 0.81989E+00_JPRB ,&
&  0.81424E+00_JPRB, 0.81584E+00_JPRB, 0.82452E+00_JPRB, 0.83282E+00_JPRB, 0.84406E+00_JPRB, 0.87886E+00_JPRB, 0.94079E+00_JPRB ,&
&  0.10421E+01_JPRB, 0.98621E+00_JPRB, 0.11308E+01_JPRB, 0.12130E+01_JPRB, 0.14886E+01_JPRB, 0.17687E+01_JPRB, 0.19038E+01_JPRB ,&
&  0.21955E+01_JPRB, 0.22247E+01_JPRB, 0.21880E+01_JPRB, 0.21493E+01_JPRB, 0.21529E+01_JPRB, 0.21031E+01_JPRB, 0.20186E+01_JPRB ,&
&  0.19589E+01_JPRB, 0.18614E+01_JPRB, 0.19320E+01_JPRB, 0.19026E+01_JPRB, 0.18340E+01_JPRB, 0.17845E+01_JPRB, 0.17102E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.15914E+01_JPRB, 0.15351E+01_JPRB, 0.14062E+01_JPRB, 0.13633E+01_JPRB, 0.13205E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11457E+01_JPRB, 0.10973E+01_JPRB, 0.11139E+01_JPRB, 0.10819E+01_JPRB, 0.10662E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10266E+01_JPRB, 0.98203E+00_JPRB, 0.98054E+00_JPRB, 0.97991E+00_JPRB, 0.96678E+00_JPRB, 0.95565E+00_JPRB, 0.96899E+00_JPRB ,&
&  0.94372E+00_JPRB, 0.93370E+00_JPRB, 0.94103E+00_JPRB, 0.91581E+00_JPRB, 0.89189E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.89813E+00_JPRB, 0.89110E+00_JPRB, 0.87547E+00_JPRB, 0.87036E+00_JPRB, 0.87556E+00_JPRB, 0.87890E+00_JPRB, 0.88099E+00_JPRB ,&
&  0.90074E+00_JPRB, 0.90612E+00_JPRB, 0.90228E+00_JPRB, 0.88393E+00_JPRB, 0.85537E+00_JPRB, 0.79337E+00_JPRB, 0.72243E+00_JPRB ,&
&  0.71217E+00_JPRB, 0.73526E+00_JPRB, 0.76045E+00_JPRB, 0.77349E+00_JPRB, 0.79125E+00_JPRB, 0.80948E+00_JPRB, 0.82489E+00_JPRB ,&
&  0.83839E+00_JPRB, 0.84940E+00_JPRB, 0.86948E+00_JPRB, 0.89631E+00_JPRB, 0.92231E+00_JPRB, 0.95250E+00_JPRB, 0.98120E+00_JPRB ,&
&  0.94502E+00_JPRB, 0.92515E+00_JPRB, 0.92169E+00_JPRB, 0.90774E+00_JPRB, 0.96038E+00_JPRB, 0.97192E+00_JPRB, 0.97557E+00_JPRB ,&
&  0.97271E+00_JPRB, 0.95776E+00_JPRB, 0.93251E+00_JPRB, 0.91235E+00_JPRB, 0.89413E+00_JPRB, 0.88201E+00_JPRB, 0.86728E+00_JPRB ,&
&  0.85917E+00_JPRB, 0.86020E+00_JPRB, 0.87411E+00_JPRB, 0.89113E+00_JPRB, 0.91909E+00_JPRB, 0.10015E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.12460E+01_JPRB, 0.13538E+01_JPRB, 0.14228E+01_JPRB, 0.14504E+01_JPRB, 0.16853E+01_JPRB, 0.19285E+01_JPRB, 0.20588E+01_JPRB ,&
&  0.22488E+01_JPRB, 0.22200E+01_JPRB, 0.23112E+01_JPRB, 0.24026E+01_JPRB, 0.24828E+01_JPRB, 0.24223E+01_JPRB, 0.22465E+01_JPRB ,&
&  0.21568E+01_JPRB, 0.21850E+01_JPRB, 0.21616E+01_JPRB, 0.21347E+01_JPRB, 0.20431E+01_JPRB, 0.19804E+01_JPRB, 0.19009E+01_JPRB ,&
&  0.18849E+01_JPRB, 0.18755E+01_JPRB, 0.18108E+01_JPRB, 0.17285E+01_JPRB, 0.16251E+01_JPRB, 0.15045E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.11281E+01_JPRB, 0.10763E+01_JPRB, 0.10754E+01_JPRB, 0.10916E+01_JPRB, 0.10638E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10455E+01_JPRB, 0.10696E+01_JPRB, 0.10472E+01_JPRB, 0.10187E+01_JPRB, 0.99244E+00_JPRB, 0.97481E+00_JPRB ,&
&  0.93536E+00_JPRB, 0.94617E+00_JPRB, 0.94562E+00_JPRB, 0.93006E+00_JPRB, 0.90916E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.92296E+00_JPRB, 0.90190E+00_JPRB, 0.88755E+00_JPRB, 0.89046E+00_JPRB, 0.90010E+00_JPRB, 0.91277E+00_JPRB, 0.93292E+00_JPRB ,&
&  0.95432E+00_JPRB, 0.96195E+00_JPRB, 0.94448E+00_JPRB, 0.91570E+00_JPRB, 0.89067E+00_JPRB, 0.87254E+00_JPRB, 0.83116E+00_JPRB ,&
&  0.77899E+00_JPRB, 0.80676E+00_JPRB, 0.79806E+00_JPRB, 0.80969E+00_JPRB, 0.84541E+00_JPRB, 0.86114E+00_JPRB, 0.88084E+00_JPRB ,&
&  0.90911E+00_JPRB, 0.91815E+00_JPRB, 0.92972E+00_JPRB, 0.94450E+00_JPRB, 0.96766E+00_JPRB, 0.10242E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10054E+01_JPRB, 0.99998E+00_JPRB, 0.10232E+01_JPRB, 0.10617E+01_JPRB, 0.10860E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10424E+01_JPRB, 0.10169E+01_JPRB, 0.99409E+00_JPRB, 0.96666E+00_JPRB, 0.93862E+00_JPRB, 0.92036E+00_JPRB ,&
&  0.91007E+00_JPRB, 0.91252E+00_JPRB, 0.92964E+00_JPRB, 0.94595E+00_JPRB, 0.98036E+00_JPRB, 0.11398E+01_JPRB, 0.14330E+01_JPRB ,&
&  0.16620E+01_JPRB, 0.18635E+01_JPRB, 0.19522E+01_JPRB, 0.20635E+01_JPRB, 0.21763E+01_JPRB, 0.22918E+01_JPRB, 0.23856E+01_JPRB ,&
&  0.24738E+01_JPRB, 0.23777E+01_JPRB, 0.25892E+01_JPRB, 0.26740E+01_JPRB, 0.26623E+01_JPRB, 0.26676E+01_JPRB, 0.24793E+01_JPRB ,&
&  0.23320E+01_JPRB, 0.22053E+01_JPRB, 0.22614E+01_JPRB, 0.22577E+01_JPRB, 0.22063E+01_JPRB, 0.22268E+01_JPRB, 0.21640E+01_JPRB ,&
&  0.21260E+01_JPRB, 0.20659E+01_JPRB, 0.18326E+01_JPRB, 0.16471E+01_JPRB, 0.14349E+01_JPRB, 0.14906E+01_JPRB, 0.13954E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.11117E+01_JPRB, 0.11154E+01_JPRB, 0.11174E+01_JPRB, 0.11567E+01_JPRB, 0.11984E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.11722E+01_JPRB, 0.11949E+01_JPRB, 0.11581E+01_JPRB, 0.11301E+01_JPRB, 0.11004E+01_JPRB, 0.10561E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.10198E+01_JPRB, 0.10193E+01_JPRB, 0.99528E+00_JPRB, 0.96325E+00_JPRB, 0.94190E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.97156E+00_JPRB, 0.94954E+00_JPRB, 0.93554E+00_JPRB, 0.93627E+00_JPRB, 0.94442E+00_JPRB, 0.96284E+00_JPRB, 0.98501E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.99750E+00_JPRB, 0.96670E+00_JPRB, 0.93907E+00_JPRB, 0.92080E+00_JPRB, 0.90449E+00_JPRB, 0.90470E+00_JPRB ,&
&  0.83621E+00_JPRB, 0.86184E+00_JPRB, 0.81624E+00_JPRB, 0.83618E+00_JPRB, 0.89849E+00_JPRB, 0.92007E+00_JPRB, 0.94170E+00_JPRB ,&
&  0.95498E+00_JPRB, 0.98228E+00_JPRB, 0.99117E+00_JPRB, 0.10072E+01_JPRB, 0.10407E+01_JPRB, 0.11210E+01_JPRB, 0.12172E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.11493E+01_JPRB, 0.11794E+01_JPRB, 0.12254E+01_JPRB, 0.12212E+01_JPRB, 0.12573E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.12430E+01_JPRB, 0.11840E+01_JPRB, 0.11195E+01_JPRB, 0.10631E+01_JPRB, 0.10206E+01_JPRB, 0.98830E+00_JPRB, 0.97385E+00_JPRB ,&
&  0.97026E+00_JPRB, 0.97400E+00_JPRB, 0.98290E+00_JPRB, 0.10168E+01_JPRB, 0.10537E+01_JPRB, 0.12683E+01_JPRB, 0.16560E+01_JPRB ,&
&  0.21058E+01_JPRB, 0.22215E+01_JPRB, 0.24348E+01_JPRB, 0.28774E+01_JPRB, 0.29818E+01_JPRB, 0.29883E+01_JPRB, 0.29126E+01_JPRB ,&
&  0.28287E+01_JPRB, 0.26662E+01_JPRB, 0.26080E+01_JPRB, 0.26297E+01_JPRB, 0.25709E+01_JPRB, 0.26406E+01_JPRB, 0.25257E+01_JPRB ,&
&  0.23269E+01_JPRB, 0.22618E+01_JPRB, 0.22547E+01_JPRB, 0.22236E+01_JPRB, 0.21614E+01_JPRB, 0.21870E+01_JPRB, 0.21420E+01_JPRB ,&
&  0.20520E+01_JPRB, 0.18271E+01_JPRB, 0.14662E+01_JPRB, 0.14021E+01_JPRB, 0.11836E+01_JPRB, 0.12356E+01_JPRB, 0.14511E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.13812E+01_JPRB, 0.12929E+01_JPRB, 0.12684E+01_JPRB, 0.12950E+01_JPRB, 0.14453E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.13409E+01_JPRB, 0.13496E+01_JPRB, 0.13367E+01_JPRB, 0.13008E+01_JPRB, 0.12470E+01_JPRB, 0.11988E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11373E+01_JPRB, 0.11225E+01_JPRB, 0.10949E+01_JPRB, 0.10353E+01_JPRB, 0.99251E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.10567E+01_JPRB, 0.10306E+01_JPRB, 0.10067E+01_JPRB, 0.10031E+01_JPRB, 0.10107E+01_JPRB, 0.10263E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.10228E+01_JPRB, 0.98405E+00_JPRB, 0.96372E+00_JPRB, 0.94726E+00_JPRB, 0.92584E+00_JPRB, 0.92174E+00_JPRB ,&
&  0.94532E+00_JPRB, 0.97932E+00_JPRB, 0.92053E+00_JPRB, 0.85876E+00_JPRB, 0.88349E+00_JPRB, 0.89921E+00_JPRB, 0.94654E+00_JPRB ,&
&  0.97197E+00_JPRB, 0.10327E+01_JPRB, 0.10647E+01_JPRB, 0.10883E+01_JPRB, 0.11450E+01_JPRB, 0.12610E+01_JPRB, 0.13670E+01_JPRB ,&
&  0.13994E+01_JPRB, 0.14346E+01_JPRB, 0.15066E+01_JPRB, 0.15770E+01_JPRB, 0.15804E+01_JPRB, 0.14608E+01_JPRB, 0.14608E+01_JPRB ,&
&  0.13834E+01_JPRB, 0.12865E+01_JPRB, 0.11858E+01_JPRB, 0.11209E+01_JPRB, 0.10853E+01_JPRB, 0.10703E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10751E+01_JPRB, 0.10519E+01_JPRB, 0.10531E+01_JPRB, 0.10963E+01_JPRB, 0.13068E+01_JPRB, 0.16036E+01_JPRB ,&
&  0.20574E+01_JPRB, 0.21629E+01_JPRB, 0.22820E+01_JPRB, 0.26665E+01_JPRB, 0.29086E+01_JPRB, 0.30363E+01_JPRB, 0.30896E+01_JPRB ,&
&  0.30261E+01_JPRB, 0.29108E+01_JPRB, 0.27843E+01_JPRB, 0.26206E+01_JPRB, 0.25826E+01_JPRB, 0.25937E+01_JPRB, 0.25139E+01_JPRB ,&
&  0.23257E+01_JPRB, 0.22008E+01_JPRB, 0.21668E+01_JPRB, 0.20713E+01_JPRB, 0.19922E+01_JPRB, 0.18455E+01_JPRB, 0.18571E+01_JPRB ,&
&  0.18956E+01_JPRB, 0.12880E+01_JPRB, 0.11205E+01_JPRB, 0.12782E+01_JPRB, 0.11452E+01_JPRB, 0.11877E+01_JPRB, 0.13432E+01_JPRB ,&
&  0.13522E+01_JPRB, 0.13427E+01_JPRB, 0.13934E+01_JPRB, 0.13848E+01_JPRB, 0.14753E+01_JPRB, 0.16380E+01_JPRB, 0.16370E+01_JPRB ,&
&  0.15932E+01_JPRB, 0.15096E+01_JPRB, 0.15213E+01_JPRB, 0.14845E+01_JPRB, 0.14213E+01_JPRB, 0.13665E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.12236E+01_JPRB, 0.12027E+01_JPRB, 0.11420E+01_JPRB, 0.10869E+01_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.11734E+01_JPRB, 0.11402E+01_JPRB, 0.11159E+01_JPRB, 0.11059E+01_JPRB, 0.11037E+01_JPRB, 0.11104E+01_JPRB, 0.11068E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.10432E+01_JPRB, 0.99912E+00_JPRB, 0.98867E+00_JPRB, 0.96779E+00_JPRB, 0.94507E+00_JPRB, 0.94719E+00_JPRB ,&
&  0.97210E+00_JPRB, 0.10258E+01_JPRB, 0.91570E+00_JPRB, 0.86382E+00_JPRB, 0.82036E+00_JPRB, 0.85193E+00_JPRB, 0.89141E+00_JPRB ,&
&  0.97399E+00_JPRB, 0.10217E+01_JPRB, 0.10548E+01_JPRB, 0.11118E+01_JPRB, 0.12341E+01_JPRB, 0.13810E+01_JPRB, 0.15389E+01_JPRB ,&
&  0.16203E+01_JPRB, 0.17490E+01_JPRB, 0.17531E+01_JPRB, 0.19093E+01_JPRB, 0.18427E+01_JPRB, 0.16605E+01_JPRB, 0.16464E+01_JPRB ,&
&  0.15230E+01_JPRB, 0.14063E+01_JPRB, 0.13018E+01_JPRB, 0.12448E+01_JPRB, 0.12051E+01_JPRB, 0.11640E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.10992E+01_JPRB, 0.10922E+01_JPRB, 0.10850E+01_JPRB, 0.10756E+01_JPRB, 0.11367E+01_JPRB, 0.13122E+01_JPRB, 0.15039E+01_JPRB ,&
&  0.18499E+01_JPRB, 0.16742E+01_JPRB, 0.15148E+01_JPRB, 0.19738E+01_JPRB, 0.25268E+01_JPRB, 0.26181E+01_JPRB, 0.25564E+01_JPRB ,&
&  0.27613E+01_JPRB, 0.27800E+01_JPRB, 0.27620E+01_JPRB, 0.27855E+01_JPRB, 0.27376E+01_JPRB, 0.26567E+01_JPRB, 0.25272E+01_JPRB ,&
&  0.22816E+01_JPRB, 0.21392E+01_JPRB, 0.20445E+01_JPRB, 0.20437E+01_JPRB, 0.19375E+01_JPRB, 0.18698E+01_JPRB, 0.18340E+01_JPRB ,&
&  0.16731E+01_JPRB, 0.15728E+01_JPRB, 0.12447E+01_JPRB, 0.12801E+01_JPRB, 0.10967E+01_JPRB, 0.11105E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.13357E+01_JPRB, 0.13682E+01_JPRB, 0.13660E+01_JPRB, 0.16713E+01_JPRB, 0.18810E+01_JPRB, 0.19083E+01_JPRB ,&
&  0.18510E+01_JPRB, 0.16408E+01_JPRB, 0.16843E+01_JPRB, 0.17276E+01_JPRB, 0.17029E+01_JPRB, 0.16549E+01_JPRB, 0.15913E+01_JPRB ,&
&  0.15068E+01_JPRB, 0.14265E+01_JPRB, 0.13703E+01_JPRB, 0.12919E+01_JPRB, 0.12222E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.13051E+01_JPRB, 0.12516E+01_JPRB, 0.12153E+01_JPRB, 0.11963E+01_JPRB, 0.11872E+01_JPRB, 0.11775E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.10603E+01_JPRB, 0.10277E+01_JPRB, 0.10132E+01_JPRB, 0.98442E+00_JPRB, 0.96473E+00_JPRB, 0.97695E+00_JPRB ,&
&  0.10077E+01_JPRB, 0.99570E+00_JPRB, 0.87080E+00_JPRB, 0.79866E+00_JPRB, 0.76234E+00_JPRB, 0.79330E+00_JPRB, 0.84437E+00_JPRB ,&
&  0.91566E+00_JPRB, 0.97529E+00_JPRB, 0.10545E+01_JPRB, 0.11596E+01_JPRB, 0.12990E+01_JPRB, 0.14545E+01_JPRB, 0.17183E+01_JPRB ,&
&  0.21209E+01_JPRB, 0.19765E+01_JPRB, 0.20726E+01_JPRB, 0.20311E+01_JPRB, 0.19892E+01_JPRB, 0.18396E+01_JPRB, 0.16885E+01_JPRB ,&
&  0.15458E+01_JPRB, 0.14450E+01_JPRB, 0.13421E+01_JPRB, 0.12554E+01_JPRB, 0.11838E+01_JPRB, 0.11315E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.11389E+01_JPRB, 0.11446E+01_JPRB, 0.11810E+01_JPRB, 0.13272E+01_JPRB, 0.14522E+01_JPRB, 0.15606E+01_JPRB ,&
&  0.15572E+01_JPRB, 0.16062E+01_JPRB, 0.20790E+01_JPRB, 0.22384E+01_JPRB, 0.23715E+01_JPRB, 0.25097E+01_JPRB, 0.25696E+01_JPRB ,&
&  0.26809E+01_JPRB, 0.25678E+01_JPRB, 0.25377E+01_JPRB, 0.22738E+01_JPRB, 0.25390E+01_JPRB, 0.24806E+01_JPRB, 0.23533E+01_JPRB ,&
&  0.21868E+01_JPRB, 0.21303E+01_JPRB, 0.20054E+01_JPRB, 0.20051E+01_JPRB, 0.19923E+01_JPRB, 0.20196E+01_JPRB, 0.16589E+01_JPRB ,&
&  0.14352E+01_JPRB, 0.15936E+01_JPRB, 0.14643E+01_JPRB, 0.13230E+01_JPRB, 0.12012E+01_JPRB, 0.12352E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.13131E+01_JPRB, 0.13014E+01_JPRB, 0.13955E+01_JPRB, 0.17874E+01_JPRB, 0.21899E+01_JPRB, 0.22170E+01_JPRB ,&
&  0.19957E+01_JPRB, 0.20488E+01_JPRB, 0.20934E+01_JPRB, 0.20964E+01_JPRB, 0.21091E+01_JPRB, 0.20460E+01_JPRB, 0.19480E+01_JPRB ,&
&  0.18295E+01_JPRB, 0.17137E+01_JPRB, 0.16066E+01_JPRB, 0.14944E+01_JPRB, 0.13823E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.14467E+01_JPRB, 0.13884E+01_JPRB, 0.13372E+01_JPRB, 0.13007E+01_JPRB, 0.12684E+01_JPRB, 0.12435E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.11592E+01_JPRB, 0.10811E+01_JPRB, 0.10654E+01_JPRB, 0.10450E+01_JPRB, 0.10194E+01_JPRB, 0.10046E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.10522E+01_JPRB, 0.98638E+00_JPRB, 0.85684E+00_JPRB, 0.85526E+00_JPRB, 0.85634E+00_JPRB, 0.81369E+00_JPRB, 0.83864E+00_JPRB ,&
&  0.89058E+00_JPRB, 0.95805E+00_JPRB, 0.10352E+01_JPRB, 0.12157E+01_JPRB, 0.14376E+01_JPRB, 0.17080E+01_JPRB, 0.20036E+01_JPRB ,&
&  0.21005E+01_JPRB, 0.22279E+01_JPRB, 0.23298E+01_JPRB, 0.21909E+01_JPRB, 0.19896E+01_JPRB, 0.17806E+01_JPRB, 0.16172E+01_JPRB ,&
&  0.14995E+01_JPRB, 0.13877E+01_JPRB, 0.12774E+01_JPRB, 0.11944E+01_JPRB, 0.11428E+01_JPRB, 0.11126E+01_JPRB, 0.11021E+01_JPRB ,&
&  0.11114E+01_JPRB, 0.11210E+01_JPRB, 0.11320E+01_JPRB, 0.12455E+01_JPRB, 0.12653E+01_JPRB, 0.13381E+01_JPRB, 0.13924E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.19419E+01_JPRB, 0.20368E+01_JPRB, 0.23544E+01_JPRB, 0.19751E+01_JPRB, 0.21219E+01_JPRB, 0.25699E+01_JPRB ,&
&  0.26571E+01_JPRB, 0.25981E+01_JPRB, 0.26605E+01_JPRB, 0.21660E+01_JPRB, 0.19412E+01_JPRB, 0.25237E+01_JPRB, 0.24315E+01_JPRB ,&
&  0.22498E+01_JPRB, 0.21831E+01_JPRB, 0.21045E+01_JPRB, 0.22819E+01_JPRB, 0.17034E+01_JPRB, 0.14972E+01_JPRB, 0.12359E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.12442E+01_JPRB, 0.14147E+01_JPRB, 0.13671E+01_JPRB, 0.14072E+01_JPRB, 0.13879E+01_JPRB, 0.12987E+01_JPRB ,&
&  0.12922E+01_JPRB, 0.12984E+01_JPRB, 0.13500E+01_JPRB, 0.16776E+01_JPRB, 0.24261E+01_JPRB, 0.23916E+01_JPRB, 0.25889E+01_JPRB ,&
&  0.26063E+01_JPRB, 0.27469E+01_JPRB, 0.25690E+01_JPRB, 0.25762E+01_JPRB, 0.25065E+01_JPRB, 0.23823E+01_JPRB, 0.22031E+01_JPRB ,&
&  0.20606E+01_JPRB, 0.19263E+01_JPRB, 0.17787E+01_JPRB, 0.16435E+01_JPRB, 0.15319E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.15527E+01_JPRB, 0.14783E+01_JPRB, 0.14141E+01_JPRB, 0.13570E+01_JPRB, 0.13214E+01_JPRB, 0.12865E+01_JPRB, 0.12359E+01_JPRB ,&
&  0.11532E+01_JPRB, 0.10786E+01_JPRB, 0.10734E+01_JPRB, 0.10322E+01_JPRB, 0.10107E+01_JPRB, 0.10136E+01_JPRB, 0.10532E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.11067E+01_JPRB, 0.96273E+00_JPRB, 0.89149E+00_JPRB, 0.94209E+00_JPRB, 0.87845E+00_JPRB, 0.85826E+00_JPRB ,&
&  0.91973E+00_JPRB, 0.99601E+00_JPRB, 0.11452E+01_JPRB, 0.14626E+01_JPRB, 0.17855E+01_JPRB, 0.21165E+01_JPRB, 0.23883E+01_JPRB ,&
&  0.24901E+01_JPRB, 0.24673E+01_JPRB, 0.23346E+01_JPRB, 0.21457E+01_JPRB, 0.18582E+01_JPRB, 0.16295E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.12646E+01_JPRB, 0.12078E+01_JPRB, 0.11475E+01_JPRB, 0.11129E+01_JPRB, 0.10630E+01_JPRB, 0.10508E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.10980E+01_JPRB, 0.11426E+01_JPRB, 0.12752E+01_JPRB, 0.12909E+01_JPRB, 0.12962E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.16235E+01_JPRB, 0.17330E+01_JPRB, 0.18984E+01_JPRB, 0.21361E+01_JPRB, 0.24975E+01_JPRB, 0.25619E+01_JPRB, 0.27596E+01_JPRB ,&
&  0.22263E+01_JPRB, 0.19607E+01_JPRB, 0.18680E+01_JPRB, 0.16961E+01_JPRB, 0.20730E+01_JPRB, 0.28710E+01_JPRB, 0.28418E+01_JPRB ,&
&  0.26314E+01_JPRB, 0.25211E+01_JPRB, 0.24947E+01_JPRB, 0.20706E+01_JPRB, 0.16289E+01_JPRB, 0.13420E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.16846E+01_JPRB, 0.17929E+01_JPRB, 0.16621E+01_JPRB, 0.14646E+01_JPRB, 0.13991E+01_JPRB, 0.17026E+01_JPRB, 0.17063E+01_JPRB ,&
&  0.15654E+01_JPRB, 0.15124E+01_JPRB, 0.21052E+01_JPRB, 0.29190E+01_JPRB, 0.32881E+01_JPRB, 0.32244E+01_JPRB, 0.36491E+01_JPRB ,&
&  0.36766E+01_JPRB, 0.37284E+01_JPRB, 0.35515E+01_JPRB, 0.33004E+01_JPRB, 0.30032E+01_JPRB, 0.26842E+01_JPRB, 0.24351E+01_JPRB ,&
&  0.22416E+01_JPRB, 0.20598E+01_JPRB, 0.18906E+01_JPRB, 0.17451E+01_JPRB, 0.16379E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.16417E+01_JPRB, 0.15493E+01_JPRB, 0.14695E+01_JPRB, 0.14153E+01_JPRB, 0.13621E+01_JPRB, 0.12975E+01_JPRB, 0.12163E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.10793E+01_JPRB, 0.10656E+01_JPRB, 0.10232E+01_JPRB, 0.10143E+01_JPRB, 0.10293E+01_JPRB, 0.10693E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.12604E+01_JPRB, 0.10909E+01_JPRB, 0.10111E+01_JPRB, 0.10129E+01_JPRB, 0.99461E+00_JPRB, 0.90089E+00_JPRB ,&
&  0.97077E+00_JPRB, 0.10775E+01_JPRB, 0.12709E+01_JPRB, 0.16425E+01_JPRB, 0.21831E+01_JPRB, 0.23149E+01_JPRB, 0.24434E+01_JPRB ,&
&  0.25152E+01_JPRB, 0.23365E+01_JPRB, 0.21524E+01_JPRB, 0.19562E+01_JPRB, 0.16975E+01_JPRB, 0.15136E+01_JPRB, 0.13920E+01_JPRB ,&
&  0.13080E+01_JPRB, 0.12716E+01_JPRB, 0.12350E+01_JPRB, 0.12120E+01_JPRB, 0.11483E+01_JPRB, 0.10983E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.11028E+01_JPRB, 0.11459E+01_JPRB, 0.12723E+01_JPRB, 0.14049E+01_JPRB, 0.13785E+01_JPRB, 0.16089E+01_JPRB ,&
&  0.16715E+01_JPRB, 0.18072E+01_JPRB, 0.18695E+01_JPRB, 0.21564E+01_JPRB, 0.23790E+01_JPRB, 0.24212E+01_JPRB, 0.25599E+01_JPRB ,&
&  0.22808E+01_JPRB, 0.23436E+01_JPRB, 0.26906E+01_JPRB, 0.30396E+01_JPRB, 0.25915E+01_JPRB, 0.25862E+01_JPRB, 0.34620E+01_JPRB ,&
&  0.25660E+01_JPRB, 0.25885E+01_JPRB, 0.27117E+01_JPRB, 0.26968E+01_JPRB, 0.13117E+01_JPRB, 0.11347E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.17102E+01_JPRB, 0.12087E+01_JPRB, 0.10020E+01_JPRB, 0.11623E+01_JPRB, 0.10239E+01_JPRB, 0.12973E+01_JPRB, 0.19728E+01_JPRB ,&
&  0.21313E+01_JPRB, 0.22542E+01_JPRB, 0.34936E+01_JPRB, 0.40252E+01_JPRB, 0.38451E+01_JPRB, 0.45140E+01_JPRB, 0.45815E+01_JPRB ,&
&  0.45159E+01_JPRB, 0.40415E+01_JPRB, 0.40340E+01_JPRB, 0.36849E+01_JPRB, 0.32858E+01_JPRB, 0.29168E+01_JPRB, 0.26097E+01_JPRB ,&
&  0.23253E+01_JPRB, 0.21213E+01_JPRB, 0.19816E+01_JPRB, 0.18386E+01_JPRB, 0.17251E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.15901E+01_JPRB, 0.14938E+01_JPRB, 0.13913E+01_JPRB, 0.13237E+01_JPRB, 0.12823E+01_JPRB, 0.12285E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10877E+01_JPRB, 0.10505E+01_JPRB, 0.10213E+01_JPRB, 0.10046E+01_JPRB, 0.10211E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.11204E+01_JPRB, 0.12011E+01_JPRB, 0.12266E+01_JPRB, 0.11423E+01_JPRB, 0.11130E+01_JPRB, 0.10661E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.11108E+01_JPRB, 0.12654E+01_JPRB, 0.14999E+01_JPRB, 0.18346E+01_JPRB, 0.21160E+01_JPRB, 0.22230E+01_JPRB, 0.23982E+01_JPRB ,&
&  0.22846E+01_JPRB, 0.20906E+01_JPRB, 0.19517E+01_JPRB, 0.17704E+01_JPRB, 0.15614E+01_JPRB, 0.14294E+01_JPRB, 0.13346E+01_JPRB ,&
&  0.12756E+01_JPRB, 0.12507E+01_JPRB, 0.12347E+01_JPRB, 0.11621E+01_JPRB, 0.11005E+01_JPRB, 0.10727E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.11046E+01_JPRB, 0.11655E+01_JPRB, 0.12482E+01_JPRB, 0.13856E+01_JPRB, 0.14406E+01_JPRB, 0.13907E+01_JPRB ,&
&  0.13800E+01_JPRB, 0.14997E+01_JPRB, 0.18706E+01_JPRB, 0.19698E+01_JPRB, 0.19845E+01_JPRB, 0.20474E+01_JPRB, 0.20381E+01_JPRB ,&
&  0.21879E+01_JPRB, 0.25699E+01_JPRB, 0.24984E+01_JPRB, 0.29327E+01_JPRB, 0.30086E+01_JPRB, 0.23462E+01_JPRB, 0.25820E+01_JPRB ,&
&  0.26041E+01_JPRB, 0.23346E+01_JPRB, 0.19194E+01_JPRB, 0.15395E+01_JPRB, 0.17090E+01_JPRB, 0.16343E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.10273E+01_JPRB, 0.97887E+00_JPRB, 0.94464E+00_JPRB, 0.94717E+00_JPRB, 0.96097E+00_JPRB, 0.13194E+01_JPRB, 0.18945E+01_JPRB ,&
&  0.28783E+01_JPRB, 0.37640E+01_JPRB, 0.45734E+01_JPRB, 0.47011E+01_JPRB, 0.41236E+01_JPRB, 0.46812E+01_JPRB, 0.49528E+01_JPRB ,&
&  0.41295E+01_JPRB, 0.37902E+01_JPRB, 0.37081E+01_JPRB, 0.33689E+01_JPRB, 0.30366E+01_JPRB, 0.27533E+01_JPRB, 0.24801E+01_JPRB ,&
&  0.22448E+01_JPRB, 0.20765E+01_JPRB, 0.19471E+01_JPRB, 0.18193E+01_JPRB, 0.16971E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.14556E+01_JPRB, 0.13877E+01_JPRB, 0.13407E+01_JPRB, 0.13023E+01_JPRB, 0.12518E+01_JPRB, 0.12048E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.10846E+01_JPRB, 0.10836E+01_JPRB, 0.10311E+01_JPRB, 0.99557E+00_JPRB, 0.98022E+00_JPRB, 0.10021E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.11673E+01_JPRB, 0.13341E+01_JPRB, 0.13303E+01_JPRB, 0.14429E+01_JPRB, 0.13179E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.15915E+01_JPRB, 0.18886E+01_JPRB, 0.20569E+01_JPRB, 0.21916E+01_JPRB, 0.22970E+01_JPRB, 0.22545E+01_JPRB ,&
&  0.21014E+01_JPRB, 0.19759E+01_JPRB, 0.17772E+01_JPRB, 0.15652E+01_JPRB, 0.14173E+01_JPRB, 0.13094E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.12167E+01_JPRB, 0.11532E+01_JPRB, 0.11149E+01_JPRB, 0.11020E+01_JPRB, 0.10915E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.11435E+01_JPRB, 0.12080E+01_JPRB, 0.12686E+01_JPRB, 0.12163E+01_JPRB, 0.11291E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.16649E+01_JPRB, 0.17213E+01_JPRB, 0.19771E+01_JPRB, 0.19020E+01_JPRB, 0.18218E+01_JPRB, 0.19483E+01_JPRB ,&
&  0.21750E+01_JPRB, 0.26573E+01_JPRB, 0.23542E+01_JPRB, 0.26621E+01_JPRB, 0.33147E+01_JPRB, 0.25977E+01_JPRB, 0.22401E+01_JPRB ,&
&  0.24189E+01_JPRB, 0.21185E+01_JPRB, 0.19197E+01_JPRB, 0.18225E+01_JPRB, 0.26739E+01_JPRB, 0.34669E+01_JPRB, 0.15719E+01_JPRB ,&
&  0.12262E+01_JPRB, 0.11640E+01_JPRB, 0.11221E+01_JPRB, 0.10491E+01_JPRB, 0.10932E+01_JPRB, 0.15339E+01_JPRB, 0.26404E+01_JPRB ,&
&  0.33745E+01_JPRB, 0.48872E+01_JPRB, 0.54726E+01_JPRB, 0.53440E+01_JPRB, 0.42539E+01_JPRB, 0.43407E+01_JPRB, 0.39396E+01_JPRB ,&
&  0.40728E+01_JPRB, 0.38919E+01_JPRB, 0.35680E+01_JPRB, 0.32246E+01_JPRB, 0.28917E+01_JPRB, 0.25923E+01_JPRB, 0.23240E+01_JPRB ,&
&  0.20927E+01_JPRB, 0.19150E+01_JPRB, 0.17734E+01_JPRB, 0.16585E+01_JPRB, 0.15454E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.14345E+01_JPRB, 0.13740E+01_JPRB, 0.13218E+01_JPRB, 0.12662E+01_JPRB, 0.12282E+01_JPRB, 0.11751E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10674E+01_JPRB, 0.10134E+01_JPRB, 0.98448E+00_JPRB, 0.97981E+00_JPRB, 0.99695E+00_JPRB, 0.10340E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.11486E+01_JPRB, 0.13031E+01_JPRB, 0.16295E+01_JPRB, 0.18010E+01_JPRB, 0.16801E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.18290E+01_JPRB, 0.19926E+01_JPRB, 0.22501E+01_JPRB, 0.22722E+01_JPRB, 0.22479E+01_JPRB, 0.23148E+01_JPRB, 0.21251E+01_JPRB ,&
&  0.19454E+01_JPRB, 0.17838E+01_JPRB, 0.15666E+01_JPRB, 0.14126E+01_JPRB, 0.13052E+01_JPRB, 0.12668E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.11660E+01_JPRB, 0.11352E+01_JPRB, 0.11354E+01_JPRB, 0.11425E+01_JPRB, 0.11417E+01_JPRB, 0.11454E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.12178E+01_JPRB, 0.12809E+01_JPRB, 0.13259E+01_JPRB, 0.11578E+01_JPRB, 0.13101E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.14551E+01_JPRB, 0.15318E+01_JPRB, 0.15187E+01_JPRB, 0.16901E+01_JPRB, 0.18046E+01_JPRB, 0.17753E+01_JPRB, 0.18574E+01_JPRB ,&
&  0.22222E+01_JPRB, 0.22168E+01_JPRB, 0.22491E+01_JPRB, 0.26340E+01_JPRB, 0.29186E+01_JPRB, 0.34969E+01_JPRB, 0.20475E+01_JPRB ,&
&  0.17057E+01_JPRB, 0.20344E+01_JPRB, 0.21541E+01_JPRB, 0.24053E+01_JPRB, 0.31810E+01_JPRB, 0.51253E+01_JPRB, 0.44039E+01_JPRB ,&
&  0.15993E+01_JPRB, 0.12451E+01_JPRB, 0.14365E+01_JPRB, 0.14512E+01_JPRB, 0.13394E+01_JPRB, 0.19193E+01_JPRB, 0.53227E+01_JPRB ,&
&  0.49179E+01_JPRB, 0.58835E+01_JPRB, 0.56682E+01_JPRB, 0.47633E+01_JPRB, 0.39332E+01_JPRB, 0.38591E+01_JPRB, 0.39825E+01_JPRB ,&
&  0.37894E+01_JPRB, 0.34611E+01_JPRB, 0.31622E+01_JPRB, 0.29015E+01_JPRB, 0.25854E+01_JPRB, 0.22636E+01_JPRB, 0.19799E+01_JPRB ,&
&  0.17774E+01_JPRB, 0.16621E+01_JPRB, 0.15997E+01_JPRB, 0.15467E+01_JPRB, 0.14907E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.13552E+01_JPRB, 0.13010E+01_JPRB, 0.12609E+01_JPRB, 0.12297E+01_JPRB, 0.12158E+01_JPRB, 0.11660E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.10708E+01_JPRB, 0.10234E+01_JPRB, 0.98780E+00_JPRB, 0.99089E+00_JPRB, 0.10167E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.11454E+01_JPRB, 0.13313E+01_JPRB, 0.17484E+01_JPRB, 0.20268E+01_JPRB, 0.19322E+01_JPRB, 0.20640E+01_JPRB ,&
&  0.23508E+01_JPRB, 0.23993E+01_JPRB, 0.22470E+01_JPRB, 0.21726E+01_JPRB, 0.22835E+01_JPRB, 0.23342E+01_JPRB, 0.20246E+01_JPRB ,&
&  0.18010E+01_JPRB, 0.16306E+01_JPRB, 0.14640E+01_JPRB, 0.13584E+01_JPRB, 0.13163E+01_JPRB, 0.12842E+01_JPRB, 0.12566E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.11949E+01_JPRB, 0.11981E+01_JPRB, 0.12031E+01_JPRB, 0.12201E+01_JPRB, 0.12073E+01_JPRB, 0.12301E+01_JPRB ,&
&  0.12790E+01_JPRB, 0.13178E+01_JPRB, 0.13630E+01_JPRB, 0.13244E+01_JPRB, 0.13454E+01_JPRB, 0.13749E+01_JPRB, 0.14409E+01_JPRB ,&
&  0.14550E+01_JPRB, 0.15040E+01_JPRB, 0.15746E+01_JPRB, 0.16440E+01_JPRB, 0.17775E+01_JPRB, 0.18632E+01_JPRB, 0.19567E+01_JPRB ,&
&  0.21944E+01_JPRB, 0.23193E+01_JPRB, 0.22550E+01_JPRB, 0.24661E+01_JPRB, 0.27520E+01_JPRB, 0.31073E+01_JPRB, 0.25500E+01_JPRB ,&
&  0.21691E+01_JPRB, 0.20186E+01_JPRB, 0.25222E+01_JPRB, 0.35947E+01_JPRB, 0.43552E+01_JPRB, 0.59490E+01_JPRB, 0.80623E+01_JPRB ,&
&  0.88412E+01_JPRB, 0.45746E+01_JPRB, 0.35722E+01_JPRB, 0.45863E+01_JPRB, 0.35032E+01_JPRB, 0.31314E+01_JPRB, 0.46409E+01_JPRB ,&
&  0.54501E+01_JPRB, 0.58469E+01_JPRB, 0.53121E+01_JPRB, 0.40355E+01_JPRB, 0.36865E+01_JPRB, 0.38035E+01_JPRB, 0.36572E+01_JPRB ,&
&  0.32807E+01_JPRB, 0.29681E+01_JPRB, 0.26415E+01_JPRB, 0.22904E+01_JPRB, 0.19923E+01_JPRB, 0.17746E+01_JPRB, 0.16427E+01_JPRB ,&
&  0.15731E+01_JPRB, 0.15370E+01_JPRB, 0.15129E+01_JPRB, 0.14701E+01_JPRB, 0.14140E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.12704E+01_JPRB, 0.12479E+01_JPRB, 0.12474E+01_JPRB, 0.12435E+01_JPRB, 0.12222E+01_JPRB, 0.11702E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.11108E+01_JPRB, 0.10406E+01_JPRB, 0.10101E+01_JPRB, 0.10228E+01_JPRB, 0.10590E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.12244E+01_JPRB, 0.15330E+01_JPRB, 0.19247E+01_JPRB, 0.23139E+01_JPRB, 0.30028E+01_JPRB, 0.21177E+01_JPRB ,&
&  0.22638E+01_JPRB, 0.23917E+01_JPRB, 0.21529E+01_JPRB, 0.21731E+01_JPRB, 0.22472E+01_JPRB, 0.22236E+01_JPRB, 0.19349E+01_JPRB ,&
&  0.17042E+01_JPRB, 0.15750E+01_JPRB, 0.14885E+01_JPRB, 0.14155E+01_JPRB, 0.13733E+01_JPRB, 0.13572E+01_JPRB, 0.13144E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.13104E+01_JPRB, 0.12890E+01_JPRB, 0.13023E+01_JPRB, 0.13121E+01_JPRB, 0.13038E+01_JPRB, 0.13397E+01_JPRB ,&
&  0.13797E+01_JPRB, 0.14060E+01_JPRB, 0.14200E+01_JPRB, 0.13766E+01_JPRB, 0.14177E+01_JPRB, 0.14547E+01_JPRB, 0.15024E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.13720E+01_JPRB, 0.15313E+01_JPRB, 0.17249E+01_JPRB, 0.18624E+01_JPRB, 0.18742E+01_JPRB, 0.19955E+01_JPRB ,&
&  0.23272E+01_JPRB, 0.25689E+01_JPRB, 0.26973E+01_JPRB, 0.24180E+01_JPRB, 0.25737E+01_JPRB, 0.28879E+01_JPRB, 0.31999E+01_JPRB ,&
&  0.35114E+01_JPRB, 0.35343E+01_JPRB, 0.41018E+01_JPRB, 0.46332E+01_JPRB, 0.51178E+01_JPRB, 0.65244E+01_JPRB, 0.68365E+01_JPRB ,&
&  0.81918E+01_JPRB, 0.10043E+02_JPRB, 0.86603E+01_JPRB, 0.68331E+01_JPRB, 0.60131E+01_JPRB, 0.52877E+01_JPRB, 0.53552E+01_JPRB ,&
&  0.59629E+01_JPRB, 0.53497E+01_JPRB, 0.44361E+01_JPRB, 0.42782E+01_JPRB, 0.35680E+01_JPRB, 0.33786E+01_JPRB, 0.30134E+01_JPRB ,&
&  0.25016E+01_JPRB, 0.21207E+01_JPRB, 0.17683E+01_JPRB, 0.15486E+01_JPRB, 0.14345E+01_JPRB, 0.14054E+01_JPRB, 0.13990E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.13978E+01_JPRB, 0.13850E+01_JPRB, 0.13595E+01_JPRB, 0.13149E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.12506E+01_JPRB, 0.12477E+01_JPRB, 0.12672E+01_JPRB, 0.12737E+01_JPRB, 0.12517E+01_JPRB, 0.12342E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.11212E+01_JPRB, 0.10983E+01_JPRB, 0.11026E+01_JPRB, 0.11276E+01_JPRB, 0.11417E+01_JPRB, 0.11585E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.14323E+01_JPRB, 0.17719E+01_JPRB, 0.20909E+01_JPRB, 0.24089E+01_JPRB, 0.28409E+01_JPRB, 0.27720E+01_JPRB ,&
&  0.20974E+01_JPRB, 0.26032E+01_JPRB, 0.21552E+01_JPRB, 0.22737E+01_JPRB, 0.23300E+01_JPRB, 0.24225E+01_JPRB, 0.21024E+01_JPRB ,&
&  0.18000E+01_JPRB, 0.16879E+01_JPRB, 0.17091E+01_JPRB, 0.16189E+01_JPRB, 0.14842E+01_JPRB, 0.14146E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.14289E+01_JPRB, 0.14169E+01_JPRB, 0.14071E+01_JPRB, 0.14262E+01_JPRB, 0.14353E+01_JPRB, 0.14258E+01_JPRB, 0.14307E+01_JPRB ,&
&  0.14453E+01_JPRB, 0.14548E+01_JPRB, 0.14382E+01_JPRB, 0.14555E+01_JPRB, 0.14803E+01_JPRB, 0.15429E+01_JPRB, 0.15897E+01_JPRB ,&
&  0.15534E+01_JPRB, 0.14988E+01_JPRB, 0.16047E+01_JPRB, 0.17160E+01_JPRB, 0.18153E+01_JPRB, 0.20192E+01_JPRB, 0.21157E+01_JPRB ,&
&  0.24660E+01_JPRB, 0.25335E+01_JPRB, 0.30176E+01_JPRB, 0.24126E+01_JPRB, 0.25229E+01_JPRB, 0.28413E+01_JPRB, 0.30506E+01_JPRB ,&
&  0.29866E+01_JPRB, 0.36607E+01_JPRB, 0.42384E+01_JPRB, 0.50237E+01_JPRB, 0.56172E+01_JPRB, 0.62816E+01_JPRB, 0.59131E+01_JPRB ,&
&  0.60926E+01_JPRB, 0.65996E+01_JPRB, 0.66840E+01_JPRB, 0.56336E+01_JPRB, 0.53791E+01_JPRB, 0.48376E+01_JPRB, 0.54272E+01_JPRB ,&
&  0.57007E+01_JPRB, 0.50473E+01_JPRB, 0.46105E+01_JPRB, 0.41029E+01_JPRB, 0.30196E+01_JPRB, 0.25657E+01_JPRB, 0.21009E+01_JPRB ,&
&  0.16828E+01_JPRB, 0.14256E+01_JPRB, 0.12217E+01_JPRB, 0.11858E+01_JPRB, 0.12030E+01_JPRB, 0.12288E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.12854E+01_JPRB, 0.12887E+01_JPRB, 0.12935E+01_JPRB, 0.12858E+01_JPRB, 0.12663E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.12623E+01_JPRB, 0.12636E+01_JPRB, 0.12897E+01_JPRB, 0.13203E+01_JPRB, 0.13424E+01_JPRB, 0.13785E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.11710E+01_JPRB, 0.12033E+01_JPRB, 0.12238E+01_JPRB, 0.12442E+01_JPRB, 0.12384E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.14351E+01_JPRB, 0.16699E+01_JPRB, 0.19794E+01_JPRB, 0.23327E+01_JPRB, 0.26919E+01_JPRB, 0.31208E+01_JPRB, 0.31883E+01_JPRB ,&
&  0.24582E+01_JPRB, 0.22363E+01_JPRB, 0.23322E+01_JPRB, 0.24004E+01_JPRB, 0.23923E+01_JPRB, 0.25078E+01_JPRB, 0.24793E+01_JPRB ,&
&  0.22679E+01_JPRB, 0.20669E+01_JPRB, 0.21063E+01_JPRB, 0.18494E+01_JPRB, 0.15016E+01_JPRB, 0.15051E+01_JPRB, 0.15324E+01_JPRB ,&
&  0.15428E+01_JPRB, 0.15418E+01_JPRB, 0.15473E+01_JPRB, 0.15696E+01_JPRB, 0.15821E+01_JPRB, 0.15617E+01_JPRB, 0.15345E+01_JPRB ,&
&  0.15318E+01_JPRB, 0.15434E+01_JPRB, 0.15412E+01_JPRB, 0.15270E+01_JPRB, 0.15750E+01_JPRB, 0.16339E+01_JPRB, 0.16716E+01_JPRB ,&
&  0.16936E+01_JPRB, 0.17493E+01_JPRB, 0.18179E+01_JPRB, 0.19083E+01_JPRB, 0.19163E+01_JPRB, 0.21863E+01_JPRB, 0.23141E+01_JPRB ,&
&  0.25314E+01_JPRB, 0.27087E+01_JPRB, 0.30890E+01_JPRB, 0.23978E+01_JPRB, 0.26849E+01_JPRB, 0.29364E+01_JPRB, 0.32595E+01_JPRB ,&
&  0.34294E+01_JPRB, 0.39658E+01_JPRB, 0.43661E+01_JPRB, 0.48292E+01_JPRB, 0.52354E+01_JPRB, 0.50443E+01_JPRB, 0.52693E+01_JPRB ,&
&  0.49992E+01_JPRB, 0.57290E+01_JPRB, 0.55295E+01_JPRB, 0.52463E+01_JPRB, 0.45634E+01_JPRB, 0.43733E+01_JPRB, 0.46747E+01_JPRB ,&
&  0.49547E+01_JPRB, 0.43066E+01_JPRB, 0.37342E+01_JPRB, 0.29953E+01_JPRB, 0.21373E+01_JPRB, 0.17327E+01_JPRB, 0.13956E+01_JPRB ,&
&  0.12370E+01_JPRB, 0.11112E+01_JPRB, 0.10727E+01_JPRB, 0.11237E+01_JPRB, 0.11543E+01_JPRB, 0.12233E+01_JPRB, 0.12390E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.12496E+01_JPRB, 0.12737E+01_JPRB, 0.12848E+01_JPRB, 0.12782E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.12270E+01_JPRB, 0.12177E+01_JPRB, 0.12095E+01_JPRB, 0.12146E+01_JPRB, 0.12367E+01_JPRB, 0.12486E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.12544E+01_JPRB, 0.13136E+01_JPRB, 0.13437E+01_JPRB, 0.13593E+01_JPRB, 0.13817E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.16287E+01_JPRB, 0.18714E+01_JPRB, 0.21614E+01_JPRB, 0.23825E+01_JPRB, 0.25866E+01_JPRB, 0.28215E+01_JPRB, 0.30473E+01_JPRB ,&
&  0.26096E+01_JPRB, 0.19622E+01_JPRB, 0.23233E+01_JPRB, 0.24545E+01_JPRB, 0.24713E+01_JPRB, 0.27329E+01_JPRB, 0.30322E+01_JPRB ,&
&  0.23327E+01_JPRB, 0.20994E+01_JPRB, 0.21080E+01_JPRB, 0.19314E+01_JPRB, 0.15903E+01_JPRB, 0.16080E+01_JPRB, 0.16407E+01_JPRB ,&
&  0.16582E+01_JPRB, 0.16620E+01_JPRB, 0.16924E+01_JPRB, 0.17241E+01_JPRB, 0.17384E+01_JPRB, 0.17173E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.16918E+01_JPRB, 0.17106E+01_JPRB, 0.17359E+01_JPRB, 0.17359E+01_JPRB, 0.17636E+01_JPRB, 0.18217E+01_JPRB, 0.18725E+01_JPRB ,&
&  0.19194E+01_JPRB, 0.19321E+01_JPRB, 0.20343E+01_JPRB, 0.22341E+01_JPRB, 0.24278E+01_JPRB, 0.22478E+01_JPRB, 0.24729E+01_JPRB ,&
&  0.25661E+01_JPRB, 0.26990E+01_JPRB, 0.27106E+01_JPRB, 0.31544E+01_JPRB, 0.22634E+01_JPRB, 0.27203E+01_JPRB, 0.29674E+01_JPRB ,&
&  0.38079E+01_JPRB, 0.39572E+01_JPRB, 0.42209E+01_JPRB, 0.45224E+01_JPRB, 0.47741E+01_JPRB, 0.43946E+01_JPRB, 0.44272E+01_JPRB ,&
&  0.46572E+01_JPRB, 0.47885E+01_JPRB, 0.48005E+01_JPRB, 0.47958E+01_JPRB, 0.44099E+01_JPRB, 0.43663E+01_JPRB, 0.42370E+01_JPRB ,&
&  0.41490E+01_JPRB, 0.32717E+01_JPRB, 0.26038E+01_JPRB, 0.19719E+01_JPRB, 0.14372E+01_JPRB, 0.12306E+01_JPRB, 0.11481E+01_JPRB ,&
&  0.11267E+01_JPRB, 0.11505E+01_JPRB, 0.11404E+01_JPRB, 0.11459E+01_JPRB, 0.11764E+01_JPRB, 0.12295E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.12486E+01_JPRB, 0.12682E+01_JPRB, 0.12722E+01_JPRB, 0.12578E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.12083E+01_JPRB, 0.11928E+01_JPRB, 0.11656E+01_JPRB, 0.11746E+01_JPRB, 0.11826E+01_JPRB, 0.11867E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.12954E+01_JPRB, 0.13707E+01_JPRB, 0.14235E+01_JPRB, 0.14454E+01_JPRB, 0.14768E+01_JPRB, 0.15659E+01_JPRB ,&
&  0.16970E+01_JPRB, 0.18931E+01_JPRB, 0.21394E+01_JPRB, 0.23248E+01_JPRB, 0.24700E+01_JPRB, 0.25406E+01_JPRB, 0.26517E+01_JPRB ,&
&  0.26098E+01_JPRB, 0.25966E+01_JPRB, 0.24496E+01_JPRB, 0.20027E+01_JPRB, 0.19597E+01_JPRB, 0.24363E+01_JPRB, 0.22443E+01_JPRB ,&
&  0.19923E+01_JPRB, 0.18769E+01_JPRB, 0.18175E+01_JPRB, 0.17551E+01_JPRB, 0.16999E+01_JPRB, 0.17219E+01_JPRB, 0.17622E+01_JPRB ,&
&  0.17699E+01_JPRB, 0.18032E+01_JPRB, 0.18383E+01_JPRB, 0.18656E+01_JPRB, 0.18844E+01_JPRB, 0.19178E+01_JPRB, 0.19254E+01_JPRB ,&
&  0.19729E+01_JPRB, 0.20143E+01_JPRB, 0.20670E+01_JPRB, 0.20945E+01_JPRB, 0.20725E+01_JPRB, 0.21251E+01_JPRB, 0.21688E+01_JPRB ,&
&  0.22272E+01_JPRB, 0.22403E+01_JPRB, 0.22508E+01_JPRB, 0.23773E+01_JPRB, 0.24595E+01_JPRB, 0.22074E+01_JPRB, 0.22775E+01_JPRB ,&
&  0.25573E+01_JPRB, 0.27076E+01_JPRB, 0.25061E+01_JPRB, 0.31277E+01_JPRB, 0.19910E+01_JPRB, 0.27095E+01_JPRB, 0.34974E+01_JPRB ,&
&  0.36330E+01_JPRB, 0.37650E+01_JPRB, 0.39262E+01_JPRB, 0.40574E+01_JPRB, 0.40933E+01_JPRB, 0.38127E+01_JPRB, 0.40957E+01_JPRB ,&
&  0.43665E+01_JPRB, 0.43101E+01_JPRB, 0.42572E+01_JPRB, 0.42071E+01_JPRB, 0.40669E+01_JPRB, 0.36795E+01_JPRB, 0.31389E+01_JPRB ,&
&  0.24819E+01_JPRB, 0.24819E+01_JPRB, 0.19766E+01_JPRB, 0.16438E+01_JPRB, 0.12318E+01_JPRB, 0.10598E+01_JPRB, 0.10749E+01_JPRB ,&
&  0.11135E+01_JPRB, 0.11395E+01_JPRB, 0.11432E+01_JPRB, 0.11705E+01_JPRB, 0.11904E+01_JPRB, 0.12104E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.12350E+01_JPRB, 0.12458E+01_JPRB, 0.12403E+01_JPRB, 0.12262E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.11408E+01_JPRB, 0.11223E+01_JPRB, 0.11018E+01_JPRB, 0.11390E+01_JPRB, 0.11349E+01_JPRB, 0.11279E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.12113E+01_JPRB, 0.12682E+01_JPRB, 0.13311E+01_JPRB, 0.14053E+01_JPRB, 0.15026E+01_JPRB, 0.15893E+01_JPRB ,&
&  0.16893E+01_JPRB, 0.17881E+01_JPRB, 0.19548E+01_JPRB, 0.21661E+01_JPRB, 0.24342E+01_JPRB, 0.25786E+01_JPRB, 0.25993E+01_JPRB ,&
&  0.24719E+01_JPRB, 0.25242E+01_JPRB, 0.25922E+01_JPRB, 0.26313E+01_JPRB, 0.23032E+01_JPRB, 0.25154E+01_JPRB, 0.25974E+01_JPRB ,&
&  0.24826E+01_JPRB, 0.21299E+01_JPRB, 0.18570E+01_JPRB, 0.18065E+01_JPRB, 0.17847E+01_JPRB, 0.18167E+01_JPRB, 0.18674E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.19378E+01_JPRB, 0.19659E+01_JPRB, 0.19999E+01_JPRB, 0.20610E+01_JPRB, 0.21641E+01_JPRB, 0.22142E+01_JPRB ,&
&  0.22447E+01_JPRB, 0.23124E+01_JPRB, 0.23277E+01_JPRB, 0.22899E+01_JPRB, 0.23505E+01_JPRB, 0.24004E+01_JPRB, 0.24509E+01_JPRB ,&
&  0.25460E+01_JPRB, 0.24744E+01_JPRB, 0.25503E+01_JPRB, 0.26585E+01_JPRB, 0.26070E+01_JPRB, 0.24155E+01_JPRB, 0.24432E+01_JPRB ,&
&  0.26293E+01_JPRB, 0.27630E+01_JPRB, 0.21079E+01_JPRB, 0.29920E+01_JPRB, 0.36974E+01_JPRB, 0.33717E+01_JPRB, 0.34387E+01_JPRB ,&
&  0.34597E+01_JPRB, 0.35737E+01_JPRB, 0.37445E+01_JPRB, 0.39020E+01_JPRB, 0.39752E+01_JPRB, 0.40098E+01_JPRB, 0.37115E+01_JPRB ,&
&  0.42209E+01_JPRB, 0.40884E+01_JPRB, 0.38619E+01_JPRB, 0.36331E+01_JPRB, 0.33169E+01_JPRB, 0.27780E+01_JPRB, 0.22387E+01_JPRB ,&
&  0.18232E+01_JPRB, 0.20472E+01_JPRB, 0.17238E+01_JPRB, 0.16361E+01_JPRB, 0.14524E+01_JPRB, 0.10161E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.11230E+01_JPRB, 0.11351E+01_JPRB, 0.11274E+01_JPRB, 0.11392E+01_JPRB, 0.11535E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.11690E+01_JPRB, 0.11724E+01_JPRB, 0.11621E+01_JPRB, 0.11553E+01_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.11011E+01_JPRB, 0.10959E+01_JPRB, 0.10819E+01_JPRB, 0.10959E+01_JPRB, 0.10846E+01_JPRB, 0.11082E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10790E+01_JPRB, 0.10964E+01_JPRB, 0.11177E+01_JPRB, 0.11617E+01_JPRB, 0.12400E+01_JPRB, 0.13302E+01_JPRB ,&
&  0.14168E+01_JPRB, 0.14291E+01_JPRB, 0.14692E+01_JPRB, 0.16016E+01_JPRB, 0.19131E+01_JPRB, 0.22444E+01_JPRB, 0.23731E+01_JPRB ,&
&  0.25187E+01_JPRB, 0.27139E+01_JPRB, 0.28603E+01_JPRB, 0.30253E+01_JPRB, 0.30837E+01_JPRB, 0.31714E+01_JPRB, 0.33985E+01_JPRB ,&
&  0.27944E+01_JPRB, 0.21448E+01_JPRB, 0.19350E+01_JPRB, 0.18477E+01_JPRB, 0.18545E+01_JPRB, 0.18816E+01_JPRB, 0.19124E+01_JPRB ,&
&  0.19442E+01_JPRB, 0.19915E+01_JPRB, 0.20307E+01_JPRB, 0.21014E+01_JPRB, 0.22123E+01_JPRB, 0.23440E+01_JPRB, 0.24135E+01_JPRB ,&
&  0.24358E+01_JPRB, 0.24716E+01_JPRB, 0.25177E+01_JPRB, 0.24112E+01_JPRB, 0.26440E+01_JPRB, 0.27569E+01_JPRB, 0.28228E+01_JPRB ,&
&  0.28258E+01_JPRB, 0.27705E+01_JPRB, 0.28716E+01_JPRB, 0.29132E+01_JPRB, 0.29319E+01_JPRB, 0.27882E+01_JPRB, 0.27010E+01_JPRB ,&
&  0.27392E+01_JPRB, 0.24557E+01_JPRB, 0.19341E+01_JPRB, 0.22520E+01_JPRB, 0.26130E+01_JPRB, 0.24668E+01_JPRB, 0.33178E+01_JPRB ,&
&  0.33356E+01_JPRB, 0.33041E+01_JPRB, 0.33900E+01_JPRB, 0.35120E+01_JPRB, 0.37073E+01_JPRB, 0.36876E+01_JPRB, 0.35971E+01_JPRB ,&
&  0.37238E+01_JPRB, 0.35348E+01_JPRB, 0.32297E+01_JPRB, 0.28379E+01_JPRB, 0.24235E+01_JPRB, 0.20552E+01_JPRB, 0.18157E+01_JPRB ,&
&  0.16552E+01_JPRB, 0.16069E+01_JPRB, 0.14815E+01_JPRB, 0.13237E+01_JPRB, 0.10739E+01_JPRB, 0.94575E+00_JPRB, 0.97150E+00_JPRB ,&
&  0.10320E+01_JPRB, 0.10452E+01_JPRB, 0.10574E+01_JPRB, 0.10521E+01_JPRB, 0.10539E+01_JPRB, 0.10538E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.11083E+01_JPRB, 0.11122E+01_JPRB, 0.11123E+01_JPRB, 0.11117E+01_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.10415E+01_JPRB, 0.10363E+01_JPRB, 0.10339E+01_JPRB, 0.10491E+01_JPRB, 0.10602E+01_JPRB, 0.10431E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.10110E+01_JPRB, 0.10231E+01_JPRB, 0.10298E+01_JPRB, 0.10513E+01_JPRB, 0.10766E+01_JPRB, 0.10930E+01_JPRB ,&
&  0.10927E+01_JPRB, 0.10802E+01_JPRB, 0.10251E+01_JPRB, 0.10252E+01_JPRB, 0.11355E+01_JPRB, 0.13411E+01_JPRB, 0.17317E+01_JPRB ,&
&  0.21648E+01_JPRB, 0.23847E+01_JPRB, 0.25262E+01_JPRB, 0.25791E+01_JPRB, 0.25557E+01_JPRB, 0.24887E+01_JPRB, 0.33264E+01_JPRB ,&
&  0.34763E+01_JPRB, 0.20613E+01_JPRB, 0.19385E+01_JPRB, 0.17817E+01_JPRB, 0.17992E+01_JPRB, 0.18004E+01_JPRB, 0.18226E+01_JPRB ,&
&  0.18318E+01_JPRB, 0.18812E+01_JPRB, 0.19582E+01_JPRB, 0.20802E+01_JPRB, 0.22006E+01_JPRB, 0.23277E+01_JPRB, 0.24150E+01_JPRB ,&
&  0.24545E+01_JPRB, 0.24884E+01_JPRB, 0.25851E+01_JPRB, 0.25815E+01_JPRB, 0.26059E+01_JPRB, 0.27424E+01_JPRB, 0.28686E+01_JPRB ,&
&  0.29733E+01_JPRB, 0.31758E+01_JPRB, 0.29286E+01_JPRB, 0.29521E+01_JPRB, 0.30661E+01_JPRB, 0.27796E+01_JPRB, 0.27521E+01_JPRB ,&
&  0.27613E+01_JPRB, 0.23668E+01_JPRB, 0.17801E+01_JPRB, 0.19946E+01_JPRB, 0.22668E+01_JPRB, 0.25191E+01_JPRB, 0.28769E+01_JPRB ,&
&  0.28218E+01_JPRB, 0.26982E+01_JPRB, 0.27438E+01_JPRB, 0.29375E+01_JPRB, 0.30015E+01_JPRB, 0.29644E+01_JPRB, 0.28665E+01_JPRB ,&
&  0.27932E+01_JPRB, 0.25682E+01_JPRB, 0.24003E+01_JPRB, 0.21491E+01_JPRB, 0.19432E+01_JPRB, 0.17589E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.17436E+01_JPRB, 0.21108E+01_JPRB, 0.17745E+01_JPRB, 0.11008E+01_JPRB, 0.95170E+00_JPRB, 0.92118E+00_JPRB ,&
&  0.94143E+00_JPRB, 0.95469E+00_JPRB, 0.98444E+00_JPRB, 0.99055E+00_JPRB, 0.98274E+00_JPRB, 0.97355E+00_JPRB, 0.96847E+00_JPRB ,&
&  0.99516E+00_JPRB, 0.10333E+01_JPRB, 0.10520E+01_JPRB, 0.10459E+01_JPRB, 0.10387E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10827E+01_JPRB, 0.10881E+01_JPRB, 0.11133E+01_JPRB, 0.11211E+01_JPRB, 0.11186E+01_JPRB, 0.11243E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.11152E+01_JPRB, 0.11207E+01_JPRB, 0.11289E+01_JPRB, 0.11402E+01_JPRB, 0.11578E+01_JPRB, 0.11752E+01_JPRB, 0.11879E+01_JPRB ,&
&  0.11810E+01_JPRB, 0.11755E+01_JPRB, 0.12075E+01_JPRB, 0.12288E+01_JPRB, 0.12241E+01_JPRB, 0.12209E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.13453E+01_JPRB, 0.13973E+01_JPRB, 0.14093E+01_JPRB, 0.13916E+01_JPRB, 0.14484E+01_JPRB, 0.16831E+01_JPRB, 0.25594E+01_JPRB ,&
&  0.29250E+01_JPRB, 0.20296E+01_JPRB, 0.17224E+01_JPRB, 0.15634E+01_JPRB, 0.15786E+01_JPRB, 0.16299E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.15872E+01_JPRB, 0.16093E+01_JPRB, 0.16348E+01_JPRB, 0.16821E+01_JPRB, 0.17203E+01_JPRB, 0.17309E+01_JPRB ,&
&  0.16802E+01_JPRB, 0.16572E+01_JPRB, 0.16857E+01_JPRB, 0.17427E+01_JPRB, 0.19648E+01_JPRB, 0.23032E+01_JPRB, 0.22210E+01_JPRB ,&
&  0.22346E+01_JPRB, 0.24374E+01_JPRB, 0.26382E+01_JPRB, 0.27513E+01_JPRB, 0.26891E+01_JPRB, 0.25314E+01_JPRB, 0.24169E+01_JPRB ,&
&  0.23153E+01_JPRB, 0.21246E+01_JPRB, 0.19062E+01_JPRB, 0.22512E+01_JPRB, 0.24038E+01_JPRB, 0.25616E+01_JPRB, 0.24226E+01_JPRB ,&
&  0.22970E+01_JPRB, 0.20615E+01_JPRB, 0.20455E+01_JPRB, 0.21825E+01_JPRB, 0.22852E+01_JPRB, 0.22426E+01_JPRB, 0.21232E+01_JPRB ,&
&  0.19641E+01_JPRB, 0.18743E+01_JPRB, 0.19100E+01_JPRB, 0.18519E+01_JPRB, 0.18310E+01_JPRB, 0.15115E+01_JPRB, 0.14259E+01_JPRB ,&
&  0.14970E+01_JPRB, 0.16929E+01_JPRB, 0.14210E+01_JPRB, 0.17789E+01_JPRB, 0.10063E+01_JPRB, 0.85241E+00_JPRB, 0.85207E+00_JPRB ,&
&  0.87708E+00_JPRB, 0.91304E+00_JPRB, 0.92597E+00_JPRB, 0.10250E+01_JPRB, 0.10171E+01_JPRB, 0.10177E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.10308E+01_JPRB, 0.10390E+01_JPRB, 0.10595E+01_JPRB, 0.10728E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.11431E+01_JPRB, 0.11483E+01_JPRB, 0.11585E+01_JPRB, 0.11747E+01_JPRB, 0.11848E+01_JPRB, 0.11931E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.11884E+01_JPRB, 0.12021E+01_JPRB, 0.12223E+01_JPRB, 0.12435E+01_JPRB, 0.12620E+01_JPRB, 0.12856E+01_JPRB ,&
&  0.12896E+01_JPRB, 0.12998E+01_JPRB, 0.13322E+01_JPRB, 0.13653E+01_JPRB, 0.13809E+01_JPRB, 0.13809E+01_JPRB, 0.13872E+01_JPRB ,&
&  0.13788E+01_JPRB, 0.13906E+01_JPRB, 0.14299E+01_JPRB, 0.14567E+01_JPRB, 0.14956E+01_JPRB, 0.16022E+01_JPRB, 0.19473E+01_JPRB ,&
&  0.21634E+01_JPRB, 0.21736E+01_JPRB, 0.17692E+01_JPRB, 0.15748E+01_JPRB, 0.15548E+01_JPRB, 0.14813E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.13116E+01_JPRB, 0.12543E+01_JPRB, 0.12570E+01_JPRB, 0.12388E+01_JPRB, 0.12672E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.13006E+01_JPRB, 0.13473E+01_JPRB, 0.14390E+01_JPRB, 0.15180E+01_JPRB, 0.16295E+01_JPRB, 0.16851E+01_JPRB, 0.15488E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.16878E+01_JPRB, 0.18925E+01_JPRB, 0.20746E+01_JPRB, 0.20004E+01_JPRB, 0.18724E+01_JPRB, 0.17929E+01_JPRB ,&
&  0.17300E+01_JPRB, 0.17667E+01_JPRB, 0.21963E+01_JPRB, 0.23054E+01_JPRB, 0.23704E+01_JPRB, 0.22473E+01_JPRB, 0.20203E+01_JPRB ,&
&  0.17526E+01_JPRB, 0.14547E+01_JPRB, 0.14352E+01_JPRB, 0.14494E+01_JPRB, 0.14803E+01_JPRB, 0.14439E+01_JPRB, 0.14164E+01_JPRB ,&
&  0.13324E+01_JPRB, 0.12874E+01_JPRB, 0.14996E+01_JPRB, 0.17580E+01_JPRB, 0.19459E+01_JPRB, 0.24265E+01_JPRB, 0.21612E+01_JPRB ,&
&  0.14684E+01_JPRB, 0.13911E+01_JPRB, 0.86063E+00_JPRB, 0.11742E+01_JPRB, 0.82017E+00_JPRB, 0.78905E+00_JPRB, 0.86663E+00_JPRB ,&
&  0.98119E+00_JPRB, 0.10837E+01_JPRB, 0.11604E+01_JPRB, 0.11351E+01_JPRB, 0.10731E+01_JPRB, 0.10714E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.10946E+01_JPRB, 0.11062E+01_JPRB, 0.11146E+01_JPRB, 0.11231E+01_JPRB, 0.11276E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.11588E+01_JPRB, 0.11731E+01_JPRB, 0.11923E+01_JPRB, 0.12151E+01_JPRB, 0.12355E+01_JPRB, 0.12548E+01_JPRB, 0.12706E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.13036E+01_JPRB, 0.13366E+01_JPRB, 0.13740E+01_JPRB, 0.13944E+01_JPRB, 0.14171E+01_JPRB, 0.14278E+01_JPRB ,&
&  0.14630E+01_JPRB, 0.15061E+01_JPRB, 0.15328E+01_JPRB, 0.15644E+01_JPRB, 0.15838E+01_JPRB, 0.16047E+01_JPRB, 0.16329E+01_JPRB ,&
&  0.16509E+01_JPRB, 0.17020E+01_JPRB, 0.17868E+01_JPRB, 0.18510E+01_JPRB, 0.19525E+01_JPRB, 0.22094E+01_JPRB, 0.20262E+01_JPRB ,&
&  0.23410E+01_JPRB, 0.21585E+01_JPRB, 0.18967E+01_JPRB, 0.16525E+01_JPRB, 0.15793E+01_JPRB, 0.15164E+01_JPRB, 0.14619E+01_JPRB ,&
&  0.14234E+01_JPRB, 0.13477E+01_JPRB, 0.13616E+01_JPRB, 0.13457E+01_JPRB, 0.13277E+01_JPRB, 0.13875E+01_JPRB, 0.14200E+01_JPRB ,&
&  0.14447E+01_JPRB, 0.14736E+01_JPRB, 0.15134E+01_JPRB, 0.15330E+01_JPRB, 0.15728E+01_JPRB, 0.15788E+01_JPRB, 0.15066E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.13653E+01_JPRB, 0.13098E+01_JPRB, 0.13288E+01_JPRB, 0.13195E+01_JPRB, 0.11941E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.13778E+01_JPRB, 0.15978E+01_JPRB, 0.23690E+01_JPRB, 0.22889E+01_JPRB, 0.20557E+01_JPRB, 0.17034E+01_JPRB ,&
&  0.14261E+01_JPRB, 0.11092E+01_JPRB, 0.10320E+01_JPRB, 0.10591E+01_JPRB, 0.10335E+01_JPRB, 0.10210E+01_JPRB, 0.98985E+00_JPRB ,&
&  0.93594E+00_JPRB, 0.97510E+00_JPRB, 0.11916E+01_JPRB, 0.15143E+01_JPRB, 0.19094E+01_JPRB, 0.20644E+01_JPRB, 0.26665E+01_JPRB ,&
&  0.17106E+01_JPRB, 0.99290E+00_JPRB, 0.10738E+01_JPRB, 0.96545E+00_JPRB, 0.84767E+00_JPRB, 0.74833E+00_JPRB, 0.86604E+00_JPRB ,&
&  0.10799E+01_JPRB, 0.11506E+01_JPRB, 0.11790E+01_JPRB, 0.11388E+01_JPRB, 0.11368E+01_JPRB, 0.11466E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11476E+01_JPRB, 0.11525E+01_JPRB, 0.11495E+01_JPRB, 0.11500E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.11807E+01_JPRB, 0.12024E+01_JPRB, 0.12325E+01_JPRB, 0.12693E+01_JPRB, 0.12930E+01_JPRB, 0.13261E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.13674E+01_JPRB, 0.14103E+01_JPRB, 0.14526E+01_JPRB, 0.14985E+01_JPRB, 0.15445E+01_JPRB, 0.15871E+01_JPRB, 0.16297E+01_JPRB ,&
&  0.16669E+01_JPRB, 0.17086E+01_JPRB, 0.17360E+01_JPRB, 0.17582E+01_JPRB, 0.17593E+01_JPRB, 0.17632E+01_JPRB, 0.17537E+01_JPRB ,&
&  0.17408E+01_JPRB, 0.17509E+01_JPRB, 0.17802E+01_JPRB, 0.18059E+01_JPRB, 0.18660E+01_JPRB, 0.21196E+01_JPRB, 0.17049E+01_JPRB ,&
&  0.22172E+01_JPRB, 0.21222E+01_JPRB, 0.20062E+01_JPRB, 0.18181E+01_JPRB, 0.17354E+01_JPRB, 0.17390E+01_JPRB, 0.17779E+01_JPRB ,&
&  0.18496E+01_JPRB, 0.19401E+01_JPRB, 0.20025E+01_JPRB, 0.18607E+01_JPRB, 0.16961E+01_JPRB, 0.17062E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.17459E+01_JPRB, 0.17456E+01_JPRB, 0.17460E+01_JPRB, 0.17238E+01_JPRB, 0.17037E+01_JPRB, 0.16759E+01_JPRB, 0.16182E+01_JPRB ,&
&  0.15380E+01_JPRB, 0.14351E+01_JPRB, 0.12944E+01_JPRB, 0.13342E+01_JPRB, 0.13273E+01_JPRB, 0.11427E+01_JPRB, 0.86229E+00_JPRB ,&
&  0.79864E+00_JPRB, 0.11241E+01_JPRB, 0.15288E+01_JPRB, 0.22827E+01_JPRB, 0.20917E+01_JPRB, 0.18069E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.98207E+00_JPRB, 0.87473E+00_JPRB, 0.88717E+00_JPRB, 0.88082E+00_JPRB, 0.88069E+00_JPRB, 0.85871E+00_JPRB ,&
&  0.83813E+00_JPRB, 0.86621E+00_JPRB, 0.10067E+01_JPRB, 0.12049E+01_JPRB, 0.14954E+01_JPRB, 0.15884E+01_JPRB, 0.24966E+01_JPRB ,&
&  0.15831E+01_JPRB, 0.12570E+01_JPRB, 0.16847E+01_JPRB, 0.73854E+00_JPRB, 0.83660E+00_JPRB, 0.91197E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.13541E+01_JPRB, 0.16406E+01_JPRB, 0.14626E+01_JPRB, 0.13354E+01_JPRB, 0.12328E+01_JPRB, 0.11947E+01_JPRB ,&
&  0.11885E+01_JPRB, 0.11830E+01_JPRB, 0.11858E+01_JPRB, 0.11829E+01_JPRB, 0.11760E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.12413E+01_JPRB, 0.12528E+01_JPRB, 0.12542E+01_JPRB, 0.12952E+01_JPRB, 0.13038E+01_JPRB, 0.13300E+01_JPRB, 0.13411E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.14070E+01_JPRB, 0.14479E+01_JPRB, 0.15069E+01_JPRB, 0.15466E+01_JPRB, 0.15846E+01_JPRB, 0.16378E+01_JPRB ,&
&  0.16915E+01_JPRB, 0.17558E+01_JPRB, 0.17759E+01_JPRB, 0.17875E+01_JPRB, 0.17832E+01_JPRB, 0.17966E+01_JPRB, 0.18001E+01_JPRB ,&
&  0.18113E+01_JPRB, 0.18183E+01_JPRB, 0.18529E+01_JPRB, 0.19056E+01_JPRB, 0.19829E+01_JPRB, 0.21699E+01_JPRB, 0.15278E+01_JPRB ,&
&  0.21127E+01_JPRB, 0.22110E+01_JPRB, 0.20927E+01_JPRB, 0.20452E+01_JPRB, 0.19394E+01_JPRB, 0.19795E+01_JPRB, 0.20694E+01_JPRB ,&
&  0.21917E+01_JPRB, 0.23076E+01_JPRB, 0.22711E+01_JPRB, 0.21204E+01_JPRB, 0.20492E+01_JPRB, 0.20759E+01_JPRB, 0.21027E+01_JPRB ,&
&  0.21197E+01_JPRB, 0.21286E+01_JPRB, 0.21405E+01_JPRB, 0.21391E+01_JPRB, 0.21441E+01_JPRB, 0.21628E+01_JPRB, 0.21389E+01_JPRB ,&
&  0.20779E+01_JPRB, 0.20159E+01_JPRB, 0.20062E+01_JPRB, 0.20734E+01_JPRB, 0.20314E+01_JPRB, 0.17458E+01_JPRB, 0.13483E+01_JPRB ,&
&  0.89600E+00_JPRB, 0.87827E+00_JPRB, 0.13571E+01_JPRB, 0.20429E+01_JPRB, 0.18638E+01_JPRB, 0.16083E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.94756E+00_JPRB, 0.82189E+00_JPRB, 0.82993E+00_JPRB, 0.84864E+00_JPRB, 0.90226E+00_JPRB, 0.88680E+00_JPRB ,&
&  0.91724E+00_JPRB, 0.95296E+00_JPRB, 0.99757E+00_JPRB, 0.11164E+01_JPRB, 0.13659E+01_JPRB, 0.15631E+01_JPRB, 0.14813E+01_JPRB ,&
&  0.16945E+01_JPRB, 0.12522E+01_JPRB, 0.11573E+01_JPRB, 0.12344E+01_JPRB, 0.11152E+01_JPRB, 0.13022E+01_JPRB, 0.15989E+01_JPRB ,&
&  0.20658E+01_JPRB, 0.19755E+01_JPRB, 0.25672E+01_JPRB, 0.38749E+01_JPRB, 0.30519E+01_JPRB, 0.22193E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.12376E+01_JPRB, 0.12556E+01_JPRB, 0.12471E+01_JPRB, 0.12413E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.11681E+01_JPRB, 0.11583E+01_JPRB, 0.11662E+01_JPRB, 0.11726E+01_JPRB, 0.11741E+01_JPRB, 0.11900E+01_JPRB, 0.12196E+01_JPRB ,&
&  0.12404E+01_JPRB, 0.12895E+01_JPRB, 0.13674E+01_JPRB, 0.14304E+01_JPRB, 0.15127E+01_JPRB, 0.15517E+01_JPRB, 0.15532E+01_JPRB ,&
&  0.15772E+01_JPRB, 0.16473E+01_JPRB, 0.17140E+01_JPRB, 0.17601E+01_JPRB, 0.17540E+01_JPRB, 0.17800E+01_JPRB, 0.18643E+01_JPRB ,&
&  0.19801E+01_JPRB, 0.20673E+01_JPRB, 0.21467E+01_JPRB, 0.22530E+01_JPRB, 0.23079E+01_JPRB, 0.22425E+01_JPRB, 0.15754E+01_JPRB ,&
&  0.20430E+01_JPRB, 0.22211E+01_JPRB, 0.22653E+01_JPRB, 0.22371E+01_JPRB, 0.22339E+01_JPRB, 0.21603E+01_JPRB, 0.22464E+01_JPRB ,&
&  0.23819E+01_JPRB, 0.24746E+01_JPRB, 0.23326E+01_JPRB, 0.21096E+01_JPRB, 0.22858E+01_JPRB, 0.22988E+01_JPRB, 0.23608E+01_JPRB ,&
&  0.24042E+01_JPRB, 0.24506E+01_JPRB, 0.25023E+01_JPRB, 0.25685E+01_JPRB, 0.26143E+01_JPRB, 0.26615E+01_JPRB, 0.26918E+01_JPRB ,&
&  0.26731E+01_JPRB, 0.26357E+01_JPRB, 0.26631E+01_JPRB, 0.25302E+01_JPRB, 0.24510E+01_JPRB, 0.24879E+01_JPRB, 0.20977E+01_JPRB ,&
&  0.12451E+01_JPRB, 0.91836E+00_JPRB, 0.13379E+01_JPRB, 0.17734E+01_JPRB, 0.15719E+01_JPRB, 0.13752E+01_JPRB, 0.12240E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.90240E+00_JPRB, 0.86488E+00_JPRB, 0.92408E+00_JPRB, 0.10092E+01_JPRB, 0.10866E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10855E+01_JPRB, 0.10887E+01_JPRB, 0.11234E+01_JPRB, 0.12044E+01_JPRB, 0.13683E+01_JPRB, 0.16828E+01_JPRB, 0.21144E+01_JPRB ,&
&  0.19996E+01_JPRB, 0.13185E+01_JPRB, 0.13091E+01_JPRB, 0.13250E+01_JPRB, 0.13863E+01_JPRB, 0.15256E+01_JPRB, 0.18307E+01_JPRB ,&
&  0.23134E+01_JPRB, 0.28157E+01_JPRB, 0.27018E+01_JPRB, 0.22541E+01_JPRB, 0.27988E+01_JPRB, 0.19163E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11388E+01_JPRB, 0.11665E+01_JPRB, 0.11712E+01_JPRB, 0.11728E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.10040E+01_JPRB, 0.98852E+00_JPRB, 0.97965E+00_JPRB, 0.97211E+00_JPRB, 0.98685E+00_JPRB, 0.10283E+01_JPRB, 0.10786E+01_JPRB ,&
&  0.11362E+01_JPRB, 0.12028E+01_JPRB, 0.12888E+01_JPRB, 0.13439E+01_JPRB, 0.14133E+01_JPRB, 0.14423E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.15234E+01_JPRB, 0.15929E+01_JPRB, 0.16638E+01_JPRB, 0.17255E+01_JPRB, 0.18226E+01_JPRB, 0.19451E+01_JPRB ,&
&  0.20975E+01_JPRB, 0.22344E+01_JPRB, 0.23858E+01_JPRB, 0.26000E+01_JPRB, 0.27201E+01_JPRB, 0.28036E+01_JPRB, 0.29193E+01_JPRB ,&
&  0.15336E+01_JPRB, 0.20993E+01_JPRB, 0.22218E+01_JPRB, 0.22413E+01_JPRB, 0.24389E+01_JPRB, 0.25140E+01_JPRB, 0.25605E+01_JPRB ,&
&  0.26588E+01_JPRB, 0.25773E+01_JPRB, 0.24115E+01_JPRB, 0.22178E+01_JPRB, 0.23184E+01_JPRB, 0.23706E+01_JPRB, 0.24169E+01_JPRB ,&
&  0.24217E+01_JPRB, 0.24280E+01_JPRB, 0.24893E+01_JPRB, 0.25814E+01_JPRB, 0.26622E+01_JPRB, 0.27425E+01_JPRB, 0.28511E+01_JPRB ,&
&  0.29047E+01_JPRB, 0.29431E+01_JPRB, 0.30670E+01_JPRB, 0.27884E+01_JPRB, 0.25903E+01_JPRB, 0.27138E+01_JPRB, 0.24723E+01_JPRB ,&
&  0.17357E+01_JPRB, 0.10851E+01_JPRB, 0.12675E+01_JPRB, 0.16345E+01_JPRB, 0.14048E+01_JPRB, 0.12297E+01_JPRB, 0.11284E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.91687E+00_JPRB, 0.99218E+00_JPRB, 0.10813E+01_JPRB, 0.11710E+01_JPRB, 0.12469E+01_JPRB, 0.12631E+01_JPRB ,&
&  0.12003E+01_JPRB, 0.11736E+01_JPRB, 0.11378E+01_JPRB, 0.11464E+01_JPRB, 0.11834E+01_JPRB, 0.12251E+01_JPRB, 0.12435E+01_JPRB ,&
&  0.12495E+01_JPRB, 0.12189E+01_JPRB, 0.12318E+01_JPRB, 0.12659E+01_JPRB, 0.12845E+01_JPRB, 0.12797E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.16459E+01_JPRB, 0.18594E+01_JPRB, 0.21063E+01_JPRB, 0.20332E+01_JPRB, 0.18204E+01_JPRB, 0.13376E+01_JPRB, 0.93725E+00_JPRB ,&
&  0.96471E+00_JPRB, 0.99066E+00_JPRB, 0.99882E+00_JPRB, 0.10178E+01_JPRB, 0.10120E+01_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.89938E+00_JPRB, 0.89220E+00_JPRB, 0.88629E+00_JPRB, 0.89490E+00_JPRB, 0.90815E+00_JPRB, 0.96456E+00_JPRB, 0.10581E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.12204E+01_JPRB, 0.12805E+01_JPRB, 0.13243E+01_JPRB, 0.13519E+01_JPRB, 0.13458E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.13938E+01_JPRB, 0.14244E+01_JPRB, 0.14867E+01_JPRB, 0.15722E+01_JPRB, 0.16607E+01_JPRB, 0.17622E+01_JPRB, 0.18840E+01_JPRB ,&
&  0.20341E+01_JPRB, 0.22338E+01_JPRB, 0.24554E+01_JPRB, 0.27565E+01_JPRB, 0.30405E+01_JPRB, 0.32822E+01_JPRB, 0.33936E+01_JPRB ,&
&  0.15611E+01_JPRB, 0.16380E+01_JPRB, 0.22310E+01_JPRB, 0.21834E+01_JPRB, 0.23847E+01_JPRB, 0.26479E+01_JPRB, 0.29199E+01_JPRB ,&
&  0.27475E+01_JPRB, 0.23351E+01_JPRB, 0.21995E+01_JPRB, 0.23291E+01_JPRB, 0.22379E+01_JPRB, 0.22854E+01_JPRB, 0.23202E+01_JPRB ,&
&  0.23076E+01_JPRB, 0.22835E+01_JPRB, 0.22889E+01_JPRB, 0.23406E+01_JPRB, 0.24297E+01_JPRB, 0.25248E+01_JPRB, 0.26455E+01_JPRB ,&
&  0.27624E+01_JPRB, 0.29037E+01_JPRB, 0.30520E+01_JPRB, 0.25724E+01_JPRB, 0.26954E+01_JPRB, 0.29183E+01_JPRB, 0.32094E+01_JPRB ,&
&  0.25110E+01_JPRB, 0.15236E+01_JPRB, 0.14011E+01_JPRB, 0.16863E+01_JPRB, 0.14681E+01_JPRB, 0.13952E+01_JPRB, 0.12094E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.10950E+01_JPRB, 0.11374E+01_JPRB, 0.12191E+01_JPRB, 0.12994E+01_JPRB, 0.13610E+01_JPRB, 0.13645E+01_JPRB ,&
&  0.12996E+01_JPRB, 0.12611E+01_JPRB, 0.12439E+01_JPRB, 0.12542E+01_JPRB, 0.12786E+01_JPRB, 0.12961E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.12881E+01_JPRB, 0.13227E+01_JPRB, 0.13429E+01_JPRB, 0.13637E+01_JPRB, 0.13822E+01_JPRB, 0.14022E+01_JPRB ,&
&  0.13314E+01_JPRB, 0.12844E+01_JPRB, 0.13559E+01_JPRB, 0.12439E+01_JPRB, 0.10998E+01_JPRB, 0.95311E+00_JPRB, 0.82058E+00_JPRB ,&
&  0.84647E+00_JPRB, 0.86019E+00_JPRB, 0.86184E+00_JPRB, 0.87385E+00_JPRB, 0.89331E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.80148E+00_JPRB, 0.80772E+00_JPRB, 0.80194E+00_JPRB, 0.82369E+00_JPRB, 0.82896E+00_JPRB, 0.90718E+00_JPRB, 0.98794E+00_JPRB ,&
&  0.11151E+01_JPRB, 0.12129E+01_JPRB, 0.12793E+01_JPRB, 0.13080E+01_JPRB, 0.13003E+01_JPRB, 0.12687E+01_JPRB, 0.12444E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.12924E+01_JPRB, 0.13251E+01_JPRB, 0.13728E+01_JPRB, 0.14288E+01_JPRB, 0.15071E+01_JPRB, 0.15940E+01_JPRB ,&
&  0.17224E+01_JPRB, 0.19120E+01_JPRB, 0.21650E+01_JPRB, 0.25254E+01_JPRB, 0.30088E+01_JPRB, 0.36672E+01_JPRB, 0.43697E+01_JPRB ,&
&  0.35365E+01_JPRB, 0.17080E+01_JPRB, 0.24878E+01_JPRB, 0.22307E+01_JPRB, 0.20361E+01_JPRB, 0.24314E+01_JPRB, 0.28447E+01_JPRB ,&
&  0.26136E+01_JPRB, 0.22811E+01_JPRB, 0.20734E+01_JPRB, 0.22073E+01_JPRB, 0.21564E+01_JPRB, 0.21810E+01_JPRB, 0.21963E+01_JPRB ,&
&  0.21951E+01_JPRB, 0.21728E+01_JPRB, 0.21585E+01_JPRB, 0.22005E+01_JPRB, 0.22779E+01_JPRB, 0.23872E+01_JPRB, 0.25034E+01_JPRB ,&
&  0.26490E+01_JPRB, 0.28130E+01_JPRB, 0.30280E+01_JPRB, 0.25797E+01_JPRB, 0.28163E+01_JPRB, 0.28520E+01_JPRB, 0.25536E+01_JPRB ,&
&  0.24567E+01_JPRB, 0.17398E+01_JPRB, 0.15132E+01_JPRB, 0.17677E+01_JPRB, 0.15976E+01_JPRB, 0.12577E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12172E+01_JPRB, 0.12321E+01_JPRB, 0.12287E+01_JPRB, 0.13170E+01_JPRB, 0.13690E+01_JPRB, 0.14072E+01_JPRB, 0.13914E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.13377E+01_JPRB, 0.13270E+01_JPRB, 0.13386E+01_JPRB, 0.13511E+01_JPRB, 0.13851E+01_JPRB, 0.13941E+01_JPRB ,&
&  0.14002E+01_JPRB, 0.14282E+01_JPRB, 0.14870E+01_JPRB, 0.15430E+01_JPRB, 0.15619E+01_JPRB, 0.14796E+01_JPRB, 0.14062E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.12840E+01_JPRB, 0.11515E+01_JPRB, 0.10131E+01_JPRB, 0.95501E+00_JPRB, 0.87281E+00_JPRB, 0.80457E+00_JPRB ,&
&  0.81590E+00_JPRB, 0.81383E+00_JPRB, 0.80101E+00_JPRB, 0.80166E+00_JPRB, 0.80381E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.86067E+00_JPRB, 0.84465E+00_JPRB, 0.81276E+00_JPRB, 0.79576E+00_JPRB, 0.79440E+00_JPRB, 0.82769E+00_JPRB, 0.87657E+00_JPRB ,&
&  0.95316E+00_JPRB, 0.10561E+01_JPRB, 0.11414E+01_JPRB, 0.11855E+01_JPRB, 0.11813E+01_JPRB, 0.11932E+01_JPRB, 0.11708E+01_JPRB ,&
&  0.11572E+01_JPRB, 0.11750E+01_JPRB, 0.12000E+01_JPRB, 0.12149E+01_JPRB, 0.12406E+01_JPRB, 0.12771E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.13420E+01_JPRB, 0.13776E+01_JPRB, 0.14380E+01_JPRB, 0.16367E+01_JPRB, 0.19877E+01_JPRB, 0.25372E+01_JPRB, 0.35378E+01_JPRB ,&
&  0.55840E+01_JPRB, 0.57842E+01_JPRB, 0.27967E+01_JPRB, 0.29493E+01_JPRB, 0.24655E+01_JPRB, 0.26222E+01_JPRB, 0.24756E+01_JPRB ,&
&  0.25567E+01_JPRB, 0.22842E+01_JPRB, 0.20499E+01_JPRB, 0.20917E+01_JPRB, 0.20318E+01_JPRB, 0.20429E+01_JPRB, 0.20482E+01_JPRB ,&
&  0.20841E+01_JPRB, 0.20873E+01_JPRB, 0.20613E+01_JPRB, 0.20552E+01_JPRB, 0.21097E+01_JPRB, 0.22060E+01_JPRB, 0.23224E+01_JPRB ,&
&  0.24888E+01_JPRB, 0.26334E+01_JPRB, 0.28532E+01_JPRB, 0.26042E+01_JPRB, 0.28159E+01_JPRB, 0.26250E+01_JPRB, 0.21607E+01_JPRB ,&
&  0.18711E+01_JPRB, 0.17485E+01_JPRB, 0.17922E+01_JPRB, 0.18646E+01_JPRB, 0.16991E+01_JPRB, 0.13472E+01_JPRB, 0.13556E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.12820E+01_JPRB, 0.12851E+01_JPRB, 0.13203E+01_JPRB, 0.13754E+01_JPRB, 0.13949E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.13664E+01_JPRB, 0.13614E+01_JPRB, 0.13653E+01_JPRB, 0.13767E+01_JPRB, 0.14145E+01_JPRB, 0.14384E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.15121E+01_JPRB, 0.15758E+01_JPRB, 0.16377E+01_JPRB, 0.16169E+01_JPRB, 0.15320E+01_JPRB, 0.14297E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.13819E+01_JPRB, 0.12684E+01_JPRB, 0.11920E+01_JPRB, 0.10999E+01_JPRB, 0.10655E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.10145E+01_JPRB, 0.10064E+01_JPRB, 0.96355E+00_JPRB, 0.85716E+00_JPRB, 0.85824E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.95980E+00_JPRB, 0.91327E+00_JPRB, 0.90892E+00_JPRB, 0.90574E+00_JPRB, 0.89473E+00_JPRB, 0.89624E+00_JPRB, 0.85548E+00_JPRB ,&
&  0.85259E+00_JPRB, 0.90505E+00_JPRB, 0.97478E+00_JPRB, 0.10134E+01_JPRB, 0.10380E+01_JPRB, 0.10426E+01_JPRB, 0.10303E+01_JPRB ,&
&  0.10264E+01_JPRB, 0.10599E+01_JPRB, 0.10899E+01_JPRB, 0.11110E+01_JPRB, 0.11392E+01_JPRB, 0.11629E+01_JPRB, 0.11927E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.11910E+01_JPRB, 0.11475E+01_JPRB, 0.11483E+01_JPRB, 0.12780E+01_JPRB, 0.15024E+01_JPRB, 0.18560E+01_JPRB ,&
&  0.31925E+01_JPRB, 0.71622E+01_JPRB, 0.33456E+01_JPRB, 0.32794E+01_JPRB, 0.31195E+01_JPRB, 0.29148E+01_JPRB, 0.26842E+01_JPRB ,&
&  0.26849E+01_JPRB, 0.22818E+01_JPRB, 0.19713E+01_JPRB, 0.19547E+01_JPRB, 0.18714E+01_JPRB, 0.18354E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.18765E+01_JPRB, 0.19020E+01_JPRB, 0.19192E+01_JPRB, 0.18884E+01_JPRB, 0.19211E+01_JPRB, 0.19806E+01_JPRB, 0.20797E+01_JPRB ,&
&  0.22098E+01_JPRB, 0.23894E+01_JPRB, 0.26358E+01_JPRB, 0.25487E+01_JPRB, 0.26883E+01_JPRB, 0.25471E+01_JPRB, 0.21379E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.19318E+01_JPRB, 0.19069E+01_JPRB, 0.18906E+01_JPRB, 0.17449E+01_JPRB, 0.15017E+01_JPRB, 0.15243E+01_JPRB ,&
&  0.13887E+01_JPRB, 0.13487E+01_JPRB, 0.13531E+01_JPRB, 0.13559E+01_JPRB, 0.13809E+01_JPRB, 0.13859E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13605E+01_JPRB, 0.13547E+01_JPRB, 0.13617E+01_JPRB, 0.13677E+01_JPRB, 0.13817E+01_JPRB, 0.14177E+01_JPRB ,&
&  0.14532E+01_JPRB, 0.15101E+01_JPRB, 0.15871E+01_JPRB, 0.16095E+01_JPRB, 0.15886E+01_JPRB, 0.15022E+01_JPRB, 0.14128E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.14400E+01_JPRB, 0.13256E+01_JPRB, 0.12613E+01_JPRB, 0.12351E+01_JPRB, 0.11852E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.12759E+01_JPRB, 0.13750E+01_JPRB, 0.14441E+01_JPRB, 0.99357E+00_JPRB, 0.98925E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.10188E+01_JPRB, 0.97214E+00_JPRB, 0.97248E+00_JPRB, 0.10013E+01_JPRB, 0.10021E+01_JPRB, 0.97996E+00_JPRB, 0.93508E+00_JPRB ,&
&  0.87878E+00_JPRB, 0.87684E+00_JPRB, 0.89625E+00_JPRB, 0.91721E+00_JPRB, 0.93496E+00_JPRB, 0.95308E+00_JPRB, 0.94965E+00_JPRB ,&
&  0.93511E+00_JPRB, 0.95236E+00_JPRB, 0.97491E+00_JPRB, 0.10109E+01_JPRB, 0.10439E+01_JPRB, 0.10763E+01_JPRB, 0.11129E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.11147E+01_JPRB, 0.10906E+01_JPRB, 0.10818E+01_JPRB, 0.11012E+01_JPRB, 0.11723E+01_JPRB, 0.13371E+01_JPRB ,&
&  0.20397E+01_JPRB, 0.60411E+01_JPRB, 0.55433E+01_JPRB, 0.41636E+01_JPRB, 0.32976E+01_JPRB, 0.28629E+01_JPRB, 0.26837E+01_JPRB ,&
&  0.29566E+01_JPRB, 0.29905E+01_JPRB, 0.25549E+01_JPRB, 0.20380E+01_JPRB, 0.18105E+01_JPRB, 0.17343E+01_JPRB, 0.17193E+01_JPRB ,&
&  0.16917E+01_JPRB, 0.17013E+01_JPRB, 0.17124E+01_JPRB, 0.17064E+01_JPRB, 0.17005E+01_JPRB, 0.17532E+01_JPRB, 0.18238E+01_JPRB ,&
&  0.18962E+01_JPRB, 0.20573E+01_JPRB, 0.23620E+01_JPRB, 0.26296E+01_JPRB, 0.25816E+01_JPRB, 0.26310E+01_JPRB, 0.25147E+01_JPRB ,&
&  0.26097E+01_JPRB, 0.24360E+01_JPRB, 0.21184E+01_JPRB, 0.18639E+01_JPRB, 0.16548E+01_JPRB, 0.17109E+01_JPRB, 0.16322E+01_JPRB ,&
&  0.14769E+01_JPRB, 0.13943E+01_JPRB, 0.13677E+01_JPRB, 0.13706E+01_JPRB, 0.13787E+01_JPRB, 0.13797E+01_JPRB, 0.13521E+01_JPRB ,&
&  0.13323E+01_JPRB, 0.13282E+01_JPRB, 0.13325E+01_JPRB, 0.13452E+01_JPRB, 0.13504E+01_JPRB, 0.13550E+01_JPRB, 0.13833E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.14430E+01_JPRB, 0.14642E+01_JPRB, 0.15267E+01_JPRB, 0.15102E+01_JPRB, 0.14236E+01_JPRB, 0.13430E+01_JPRB ,&
&  0.13383E+01_JPRB, 0.13834E+01_JPRB, 0.13603E+01_JPRB, 0.13316E+01_JPRB, 0.12917E+01_JPRB, 0.12383E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.11925E+01_JPRB, 0.11237E+01_JPRB, 0.10558E+01_JPRB, 0.10616E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.10831E+01_JPRB, 0.10534E+01_JPRB, 0.10346E+01_JPRB, 0.10446E+01_JPRB, 0.10522E+01_JPRB, 0.10351E+01_JPRB, 0.10066E+01_JPRB ,&
&  0.97615E+00_JPRB, 0.94348E+00_JPRB, 0.94660E+00_JPRB, 0.93408E+00_JPRB, 0.92620E+00_JPRB, 0.92265E+00_JPRB, 0.92429E+00_JPRB ,&
&  0.93233E+00_JPRB, 0.94520E+00_JPRB, 0.93655E+00_JPRB, 0.94183E+00_JPRB, 0.96857E+00_JPRB, 0.10019E+01_JPRB, 0.10210E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10469E+01_JPRB, 0.10416E+01_JPRB, 0.10416E+01_JPRB, 0.10386E+01_JPRB, 0.10703E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.16138E+01_JPRB, 0.47866E+01_JPRB, 0.31724E+01_JPRB, 0.42573E+01_JPRB, 0.32507E+01_JPRB, 0.28106E+01_JPRB, 0.23560E+01_JPRB ,&
&  0.25605E+01_JPRB, 0.30963E+01_JPRB, 0.24312E+01_JPRB, 0.20160E+01_JPRB, 0.17900E+01_JPRB, 0.16924E+01_JPRB, 0.16300E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.15860E+01_JPRB, 0.15822E+01_JPRB, 0.15963E+01_JPRB, 0.15812E+01_JPRB, 0.16039E+01_JPRB, 0.16374E+01_JPRB ,&
&  0.16630E+01_JPRB, 0.17300E+01_JPRB, 0.19256E+01_JPRB, 0.21556E+01_JPRB, 0.26515E+01_JPRB, 0.29296E+01_JPRB, 0.30445E+01_JPRB ,&
&  0.31356E+01_JPRB, 0.28704E+01_JPRB, 0.24369E+01_JPRB, 0.20276E+01_JPRB, 0.17961E+01_JPRB, 0.17392E+01_JPRB, 0.17181E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.14806E+01_JPRB, 0.14186E+01_JPRB, 0.13898E+01_JPRB, 0.13972E+01_JPRB, 0.13956E+01_JPRB, 0.13477E+01_JPRB ,&
&  0.13113E+01_JPRB, 0.13143E+01_JPRB, 0.13312E+01_JPRB, 0.13482E+01_JPRB, 0.13428E+01_JPRB, 0.13243E+01_JPRB, 0.13303E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.13743E+01_JPRB, 0.13901E+01_JPRB, 0.14772E+01_JPRB, 0.14523E+01_JPRB, 0.13958E+01_JPRB, 0.13312E+01_JPRB ,&
&  0.13372E+01_JPRB, 0.13500E+01_JPRB, 0.13397E+01_JPRB, 0.13314E+01_JPRB, 0.13460E+01_JPRB, 0.13377E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.11625E+01_JPRB, 0.11399E+01_JPRB, 0.11008E+01_JPRB, 0.11062E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.10973E+01_JPRB, 0.10778E+01_JPRB, 0.10875E+01_JPRB, 0.10927E+01_JPRB, 0.10848E+01_JPRB, 0.10561E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10402E+01_JPRB, 0.10261E+01_JPRB, 0.10112E+01_JPRB, 0.98263E+00_JPRB, 0.96584E+00_JPRB, 0.94713E+00_JPRB, 0.93930E+00_JPRB ,&
&  0.94552E+00_JPRB, 0.95001E+00_JPRB, 0.94151E+00_JPRB, 0.94098E+00_JPRB, 0.93651E+00_JPRB, 0.95018E+00_JPRB, 0.96777E+00_JPRB ,&
&  0.98586E+00_JPRB, 0.99748E+00_JPRB, 0.10111E+01_JPRB, 0.10162E+01_JPRB, 0.10134E+01_JPRB, 0.10165E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.14597E+01_JPRB, 0.50411E+01_JPRB, 0.28152E+01_JPRB, 0.42679E+01_JPRB, 0.31870E+01_JPRB, 0.27887E+01_JPRB, 0.22760E+01_JPRB ,&
&  0.24126E+01_JPRB, 0.28381E+01_JPRB, 0.23497E+01_JPRB, 0.19364E+01_JPRB, 0.17855E+01_JPRB, 0.16862E+01_JPRB, 0.15808E+01_JPRB ,&
&  0.15296E+01_JPRB, 0.15287E+01_JPRB, 0.15153E+01_JPRB, 0.15260E+01_JPRB, 0.15340E+01_JPRB, 0.15255E+01_JPRB, 0.15308E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.15277E+01_JPRB, 0.16317E+01_JPRB, 0.20797E+01_JPRB, 0.24346E+01_JPRB, 0.31044E+01_JPRB, 0.38077E+01_JPRB ,&
&  0.37160E+01_JPRB, 0.33359E+01_JPRB, 0.25730E+01_JPRB, 0.22041E+01_JPRB, 0.19630E+01_JPRB, 0.18410E+01_JPRB, 0.17695E+01_JPRB ,&
&  0.16876E+01_JPRB, 0.16232E+01_JPRB, 0.15150E+01_JPRB, 0.14625E+01_JPRB, 0.14502E+01_JPRB, 0.14373E+01_JPRB, 0.14017E+01_JPRB ,&
&  0.13622E+01_JPRB, 0.13600E+01_JPRB, 0.13727E+01_JPRB, 0.13756E+01_JPRB, 0.13655E+01_JPRB, 0.13328E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.13567E+01_JPRB, 0.13867E+01_JPRB, 0.15050E+01_JPRB, 0.14742E+01_JPRB, 0.13924E+01_JPRB, 0.13422E+01_JPRB ,&
&  0.13594E+01_JPRB, 0.13406E+01_JPRB, 0.13371E+01_JPRB, 0.13765E+01_JPRB, 0.14186E+01_JPRB, 0.14861E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.11688E+01_JPRB, 0.11357E+01_JPRB, 0.11146E+01_JPRB, 0.11088E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.11438E+01_JPRB, 0.11302E+01_JPRB, 0.11366E+01_JPRB, 0.11548E+01_JPRB, 0.11355E+01_JPRB, 0.11008E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10778E+01_JPRB, 0.10708E+01_JPRB, 0.10569E+01_JPRB, 0.10204E+01_JPRB, 0.10022E+01_JPRB, 0.98701E+00_JPRB, 0.98146E+00_JPRB ,&
&  0.98521E+00_JPRB, 0.97473E+00_JPRB, 0.95568E+00_JPRB, 0.94535E+00_JPRB, 0.94419E+00_JPRB, 0.94299E+00_JPRB, 0.95851E+00_JPRB ,&
&  0.95302E+00_JPRB, 0.96195E+00_JPRB, 0.98562E+00_JPRB, 0.99309E+00_JPRB, 0.10014E+01_JPRB, 0.99076E+00_JPRB, 0.10285E+01_JPRB ,&
&  0.12826E+01_JPRB, 0.18142E+01_JPRB, 0.33209E+01_JPRB, 0.38975E+01_JPRB, 0.31611E+01_JPRB, 0.28037E+01_JPRB, 0.25662E+01_JPRB ,&
&  0.25390E+01_JPRB, 0.25398E+01_JPRB, 0.22327E+01_JPRB, 0.19646E+01_JPRB, 0.18125E+01_JPRB, 0.16737E+01_JPRB, 0.15527E+01_JPRB ,&
&  0.14634E+01_JPRB, 0.14266E+01_JPRB, 0.14313E+01_JPRB, 0.14602E+01_JPRB, 0.14752E+01_JPRB, 0.14729E+01_JPRB, 0.14408E+01_JPRB ,&
&  0.14075E+01_JPRB, 0.13833E+01_JPRB, 0.14146E+01_JPRB, 0.17039E+01_JPRB, 0.23735E+01_JPRB, 0.31950E+01_JPRB, 0.37860E+01_JPRB ,&
&  0.38984E+01_JPRB, 0.33606E+01_JPRB, 0.26003E+01_JPRB, 0.22933E+01_JPRB, 0.20733E+01_JPRB, 0.19112E+01_JPRB, 0.17675E+01_JPRB ,&
&  0.16843E+01_JPRB, 0.16649E+01_JPRB, 0.16187E+01_JPRB, 0.15477E+01_JPRB, 0.14804E+01_JPRB, 0.14446E+01_JPRB, 0.14144E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.14030E+01_JPRB, 0.14149E+01_JPRB, 0.13940E+01_JPRB, 0.13689E+01_JPRB, 0.13363E+01_JPRB, 0.13237E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.13759E+01_JPRB, 0.14309E+01_JPRB, 0.15010E+01_JPRB, 0.14804E+01_JPRB, 0.14338E+01_JPRB, 0.13885E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.13283E+01_JPRB, 0.13186E+01_JPRB, 0.14125E+01_JPRB, 0.14584E+01_JPRB, 0.15467E+01_JPRB, 0.14342E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.12312E+01_JPRB, 0.11884E+01_JPRB, 0.11669E+01_JPRB, 0.11621E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.12094E+01_JPRB, 0.12008E+01_JPRB, 0.12139E+01_JPRB, 0.12267E+01_JPRB, 0.12104E+01_JPRB, 0.11705E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10844E+01_JPRB, 0.10587E+01_JPRB, 0.10246E+01_JPRB, 0.10112E+01_JPRB, 0.10143E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.10087E+01_JPRB, 0.98637E+00_JPRB, 0.96343E+00_JPRB, 0.95818E+00_JPRB, 0.95857E+00_JPRB, 0.96835E+00_JPRB ,&
&  0.96249E+00_JPRB, 0.96087E+00_JPRB, 0.96801E+00_JPRB, 0.97584E+00_JPRB, 0.98444E+00_JPRB, 0.97786E+00_JPRB, 0.98277E+00_JPRB ,&
&  0.11472E+01_JPRB, 0.24702E+01_JPRB, 0.30043E+01_JPRB, 0.31510E+01_JPRB, 0.29894E+01_JPRB, 0.27045E+01_JPRB, 0.25676E+01_JPRB ,&
&  0.23392E+01_JPRB, 0.22579E+01_JPRB, 0.21543E+01_JPRB, 0.20219E+01_JPRB, 0.18676E+01_JPRB, 0.17119E+01_JPRB, 0.15793E+01_JPRB ,&
&  0.14792E+01_JPRB, 0.14239E+01_JPRB, 0.13949E+01_JPRB, 0.13950E+01_JPRB, 0.14026E+01_JPRB, 0.14129E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.13443E+01_JPRB, 0.13091E+01_JPRB, 0.12928E+01_JPRB, 0.14308E+01_JPRB, 0.19773E+01_JPRB, 0.23038E+01_JPRB, 0.31628E+01_JPRB ,&
&  0.35167E+01_JPRB, 0.27599E+01_JPRB, 0.23026E+01_JPRB, 0.20913E+01_JPRB, 0.19409E+01_JPRB, 0.17894E+01_JPRB, 0.16773E+01_JPRB ,&
&  0.16264E+01_JPRB, 0.16002E+01_JPRB, 0.16027E+01_JPRB, 0.15704E+01_JPRB, 0.14900E+01_JPRB, 0.14056E+01_JPRB, 0.13661E+01_JPRB ,&
&  0.13452E+01_JPRB, 0.13268E+01_JPRB, 0.13223E+01_JPRB, 0.13256E+01_JPRB, 0.13195E+01_JPRB, 0.12857E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.13205E+01_JPRB, 0.14062E+01_JPRB, 0.14256E+01_JPRB, 0.14486E+01_JPRB, 0.14630E+01_JPRB, 0.14046E+01_JPRB ,&
&  0.13238E+01_JPRB, 0.13099E+01_JPRB, 0.13075E+01_JPRB, 0.14037E+01_JPRB, 0.15701E+01_JPRB, 0.16538E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.13977E+01_JPRB, 0.12968E+01_JPRB, 0.12451E+01_JPRB, 0.12162E+01_JPRB, 0.12283E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.12603E+01_JPRB, 0.12437E+01_JPRB, 0.12489E+01_JPRB, 0.12532E+01_JPRB, 0.12392E+01_JPRB, 0.12002E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.11005E+01_JPRB, 0.10584E+01_JPRB, 0.10285E+01_JPRB, 0.10285E+01_JPRB, 0.10352E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10219E+01_JPRB, 0.10039E+01_JPRB, 0.98706E+00_JPRB, 0.97760E+00_JPRB, 0.96947E+00_JPRB, 0.95391E+00_JPRB ,&
&  0.95331E+00_JPRB, 0.95189E+00_JPRB, 0.95644E+00_JPRB, 0.96589E+00_JPRB, 0.96437E+00_JPRB, 0.96876E+00_JPRB, 0.97361E+00_JPRB ,&
&  0.10384E+01_JPRB, 0.16258E+01_JPRB, 0.19006E+01_JPRB, 0.23962E+01_JPRB, 0.26000E+01_JPRB, 0.24927E+01_JPRB, 0.23579E+01_JPRB ,&
&  0.21803E+01_JPRB, 0.20613E+01_JPRB, 0.20019E+01_JPRB, 0.19260E+01_JPRB, 0.18049E+01_JPRB, 0.16755E+01_JPRB, 0.15588E+01_JPRB ,&
&  0.14753E+01_JPRB, 0.14261E+01_JPRB, 0.14094E+01_JPRB, 0.13859E+01_JPRB, 0.13593E+01_JPRB, 0.13524E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.12967E+01_JPRB, 0.12558E+01_JPRB, 0.12277E+01_JPRB, 0.12433E+01_JPRB, 0.15099E+01_JPRB, 0.18314E+01_JPRB, 0.22860E+01_JPRB ,&
&  0.21895E+01_JPRB, 0.19938E+01_JPRB, 0.18605E+01_JPRB, 0.17578E+01_JPRB, 0.16905E+01_JPRB, 0.15902E+01_JPRB, 0.15097E+01_JPRB ,&
&  0.14350E+01_JPRB, 0.14059E+01_JPRB, 0.13791E+01_JPRB, 0.13653E+01_JPRB, 0.13425E+01_JPRB, 0.13254E+01_JPRB, 0.12774E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.12276E+01_JPRB, 0.12212E+01_JPRB, 0.12300E+01_JPRB, 0.12497E+01_JPRB, 0.12487E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.12731E+01_JPRB, 0.13283E+01_JPRB, 0.13956E+01_JPRB, 0.13878E+01_JPRB, 0.13567E+01_JPRB, 0.13368E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.13223E+01_JPRB, 0.12856E+01_JPRB, 0.14143E+01_JPRB, 0.15295E+01_JPRB, 0.16161E+01_JPRB, 0.15430E+01_JPRB ,&
&  0.14481E+01_JPRB, 0.13413E+01_JPRB, 0.12831E+01_JPRB, 0.12520E+01_JPRB, 0.12669E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.12987E+01_JPRB, 0.12952E+01_JPRB, 0.12835E+01_JPRB, 0.12748E+01_JPRB, 0.12587E+01_JPRB, 0.12250E+01_JPRB, 0.11864E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.11221E+01_JPRB, 0.10756E+01_JPRB, 0.10488E+01_JPRB, 0.10389E+01_JPRB, 0.10358E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.10242E+01_JPRB, 0.10154E+01_JPRB, 0.10125E+01_JPRB, 0.10019E+01_JPRB, 0.99283E+00_JPRB, 0.96872E+00_JPRB ,&
&  0.95165E+00_JPRB, 0.93997E+00_JPRB, 0.93682E+00_JPRB, 0.94423E+00_JPRB, 0.95133E+00_JPRB, 0.95636E+00_JPRB, 0.96847E+00_JPRB ,&
&  0.10335E+01_JPRB, 0.10967E+01_JPRB, 0.12879E+01_JPRB, 0.16366E+01_JPRB, 0.18986E+01_JPRB, 0.20441E+01_JPRB, 0.19941E+01_JPRB ,&
&  0.18977E+01_JPRB, 0.18039E+01_JPRB, 0.17497E+01_JPRB, 0.16801E+01_JPRB, 0.15937E+01_JPRB, 0.15032E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.14022E+01_JPRB, 0.13722E+01_JPRB, 0.13469E+01_JPRB, 0.13264E+01_JPRB, 0.13091E+01_JPRB, 0.12948E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.12372E+01_JPRB, 0.11940E+01_JPRB, 0.11668E+01_JPRB, 0.12069E+01_JPRB, 0.13243E+01_JPRB, 0.15363E+01_JPRB ,&
&  0.16512E+01_JPRB, 0.15878E+01_JPRB, 0.15261E+01_JPRB, 0.14714E+01_JPRB, 0.14281E+01_JPRB, 0.14063E+01_JPRB, 0.13813E+01_JPRB ,&
&  0.13646E+01_JPRB, 0.13484E+01_JPRB, 0.13375E+01_JPRB, 0.13219E+01_JPRB, 0.13055E+01_JPRB, 0.12899E+01_JPRB, 0.12677E+01_JPRB ,&
&  0.12473E+01_JPRB, 0.12370E+01_JPRB, 0.12235E+01_JPRB, 0.12268E+01_JPRB, 0.12295E+01_JPRB, 0.12280E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.12744E+01_JPRB, 0.13016E+01_JPRB, 0.13317E+01_JPRB, 0.13289E+01_JPRB, 0.13076E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.12828E+01_JPRB, 0.12706E+01_JPRB, 0.12302E+01_JPRB, 0.13462E+01_JPRB, 0.14728E+01_JPRB, 0.14656E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.14344E+01_JPRB, 0.13954E+01_JPRB, 0.13227E+01_JPRB, 0.12510E+01_JPRB, 0.12668E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.11910E+01_JPRB, 0.12192E+01_JPRB, 0.12115E+01_JPRB, 0.12033E+01_JPRB, 0.11953E+01_JPRB, 0.11780E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.11134E+01_JPRB, 0.10918E+01_JPRB, 0.10641E+01_JPRB, 0.10444E+01_JPRB, 0.10311E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.10117E+01_JPRB, 0.10044E+01_JPRB, 0.99829E+00_JPRB, 0.99118E+00_JPRB, 0.97865E+00_JPRB, 0.96753E+00_JPRB, 0.95107E+00_JPRB ,&
&  0.93929E+00_JPRB, 0.93626E+00_JPRB, 0.93546E+00_JPRB, 0.93920E+00_JPRB, 0.94716E+00_JPRB, 0.95661E+00_JPRB, 0.96366E+00_JPRB ,&
&  0.99750E+00_JPRB, 0.94812E+00_JPRB, 0.10136E+01_JPRB, 0.11957E+01_JPRB, 0.14288E+01_JPRB, 0.16282E+01_JPRB, 0.16861E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.16012E+01_JPRB, 0.15365E+01_JPRB, 0.14744E+01_JPRB, 0.14213E+01_JPRB, 0.13813E+01_JPRB, 0.13239E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.12655E+01_JPRB, 0.12588E+01_JPRB, 0.12473E+01_JPRB, 0.12323E+01_JPRB, 0.12196E+01_JPRB, 0.12162E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.11807E+01_JPRB, 0.11493E+01_JPRB, 0.11290E+01_JPRB, 0.11346E+01_JPRB, 0.11964E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.13727E+01_JPRB, 0.13586E+01_JPRB, 0.13174E+01_JPRB, 0.12785E+01_JPRB, 0.12591E+01_JPRB, 0.12558E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.12577E+01_JPRB, 0.12526E+01_JPRB, 0.12640E+01_JPRB, 0.12672E+01_JPRB, 0.12704E+01_JPRB, 0.12676E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.12491E+01_JPRB, 0.12312E+01_JPRB, 0.12209E+01_JPRB, 0.12157E+01_JPRB, 0.12195E+01_JPRB, 0.12179E+01_JPRB, 0.12083E+01_JPRB ,&
&  0.11966E+01_JPRB, 0.12100E+01_JPRB, 0.12510E+01_JPRB, 0.12859E+01_JPRB, 0.12809E+01_JPRB, 0.12658E+01_JPRB, 0.12528E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.12427E+01_JPRB, 0.12354E+01_JPRB, 0.13087E+01_JPRB, 0.13197E+01_JPRB, 0.13212E+01_JPRB, 0.13223E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.13069E+01_JPRB, 0.12847E+01_JPRB, 0.12403E+01_JPRB, 0.11334E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.10884E+01_JPRB, 0.11185E+01_JPRB, 0.11307E+01_JPRB, 0.11295E+01_JPRB, 0.11326E+01_JPRB, 0.11339E+01_JPRB, 0.11306E+01_JPRB ,&
&  0.11281E+01_JPRB, 0.11219E+01_JPRB, 0.11068E+01_JPRB, 0.10828E+01_JPRB, 0.10582E+01_JPRB, 0.10314E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.10001E+01_JPRB, 0.99284E+00_JPRB, 0.97586E+00_JPRB, 0.96035E+00_JPRB, 0.94621E+00_JPRB, 0.93483E+00_JPRB, 0.92950E+00_JPRB ,&
&  0.92325E+00_JPRB, 0.92013E+00_JPRB, 0.92463E+00_JPRB, 0.93118E+00_JPRB, 0.94047E+00_JPRB, 0.95113E+00_JPRB, 0.96516E+00_JPRB ,&
&  0.97807E+00_JPRB, 0.82132E+00_JPRB, 0.86446E+00_JPRB, 0.98900E+00_JPRB, 0.11218E+01_JPRB, 0.12689E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.14284E+01_JPRB, 0.14408E+01_JPRB, 0.14069E+01_JPRB, 0.13660E+01_JPRB, 0.13415E+01_JPRB, 0.13026E+01_JPRB, 0.12407E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.11793E+01_JPRB, 0.11845E+01_JPRB, 0.11847E+01_JPRB, 0.11731E+01_JPRB, 0.11590E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.11444E+01_JPRB, 0.11399E+01_JPRB, 0.11342E+01_JPRB, 0.11455E+01_JPRB, 0.11727E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.12056E+01_JPRB, 0.11895E+01_JPRB, 0.11692E+01_JPRB, 0.11591E+01_JPRB, 0.11675E+01_JPRB, 0.11763E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.11766E+01_JPRB, 0.11837E+01_JPRB, 0.11883E+01_JPRB, 0.11994E+01_JPRB, 0.12109E+01_JPRB, 0.12294E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.12348E+01_JPRB, 0.12260E+01_JPRB, 0.12098E+01_JPRB, 0.11863E+01_JPRB, 0.11711E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.11611E+01_JPRB, 0.11678E+01_JPRB, 0.11896E+01_JPRB, 0.12144E+01_JPRB, 0.12206E+01_JPRB, 0.12212E+01_JPRB, 0.12224E+01_JPRB ,&
&  0.12282E+01_JPRB, 0.12316E+01_JPRB, 0.12334E+01_JPRB, 0.12179E+01_JPRB, 0.12670E+01_JPRB, 0.12645E+01_JPRB, 0.12428E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.11804E+01_JPRB, 0.11674E+01_JPRB, 0.98732E+00_JPRB, 0.10628E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.10113E+01_JPRB, 0.10308E+01_JPRB, 0.10336E+01_JPRB, 0.10458E+01_JPRB, 0.10538E+01_JPRB, 0.10596E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.10704E+01_JPRB, 0.10618E+01_JPRB, 0.10452E+01_JPRB, 0.10189E+01_JPRB, 0.99091E+00_JPRB, 0.96775E+00_JPRB ,&
&  0.95032E+00_JPRB, 0.94516E+00_JPRB, 0.93494E+00_JPRB, 0.92260E+00_JPRB, 0.91492E+00_JPRB, 0.91050E+00_JPRB, 0.91164E+00_JPRB ,&
&  0.91866E+00_JPRB, 0.92216E+00_JPRB, 0.91652E+00_JPRB, 0.91463E+00_JPRB, 0.92285E+00_JPRB, 0.93826E+00_JPRB, 0.94492E+00_JPRB ,&
&  0.94732E+00_JPRB, 0.75775E+00_JPRB, 0.81303E+00_JPRB, 0.88418E+00_JPRB, 0.96341E+00_JPRB, 0.10712E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.12436E+01_JPRB, 0.12797E+01_JPRB, 0.12719E+01_JPRB, 0.12491E+01_JPRB, 0.12217E+01_JPRB, 0.11971E+01_JPRB, 0.11647E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.11142E+01_JPRB, 0.11109E+01_JPRB, 0.11116E+01_JPRB, 0.11111E+01_JPRB, 0.11137E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.11042E+01_JPRB, 0.11011E+01_JPRB, 0.10997E+01_JPRB, 0.11001E+01_JPRB, 0.10979E+01_JPRB, 0.10947E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.10709E+01_JPRB, 0.10629E+01_JPRB, 0.10530E+01_JPRB, 0.10525E+01_JPRB, 0.10610E+01_JPRB, 0.10765E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.10996E+01_JPRB, 0.11066E+01_JPRB, 0.11159E+01_JPRB, 0.11306E+01_JPRB, 0.11470E+01_JPRB, 0.11665E+01_JPRB ,&
&  0.11773E+01_JPRB, 0.11751E+01_JPRB, 0.11723E+01_JPRB, 0.11617E+01_JPRB, 0.11582E+01_JPRB, 0.11390E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.11123E+01_JPRB, 0.11124E+01_JPRB, 0.11149E+01_JPRB, 0.11277E+01_JPRB, 0.11351E+01_JPRB, 0.11342E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11559E+01_JPRB, 0.11481E+01_JPRB, 0.11493E+01_JPRB, 0.11404E+01_JPRB, 0.11349E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10687E+01_JPRB, 0.86047E+00_JPRB, 0.95748E+00_JPRB, 0.98242E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.98537E+00_JPRB, 0.99113E+00_JPRB, 0.99415E+00_JPRB, 0.99860E+00_JPRB, 0.99914E+00_JPRB, 0.99745E+00_JPRB, 0.10015E+01_JPRB ,&
&  0.10062E+01_JPRB, 0.10010E+01_JPRB, 0.99202E+00_JPRB, 0.98819E+00_JPRB, 0.97737E+00_JPRB, 0.96174E+00_JPRB, 0.94852E+00_JPRB ,&
&  0.93248E+00_JPRB, 0.92532E+00_JPRB, 0.92078E+00_JPRB, 0.91608E+00_JPRB, 0.90847E+00_JPRB, 0.89931E+00_JPRB, 0.90228E+00_JPRB ,&
&  0.90886E+00_JPRB, 0.91602E+00_JPRB, 0.91768E+00_JPRB, 0.92111E+00_JPRB, 0.92654E+00_JPRB, 0.93267E+00_JPRB, 0.94364E+00_JPRB ,&
&  0.83085E+00_JPRB, 0.70715E+00_JPRB, 0.76418E+00_JPRB, 0.82567E+00_JPRB, 0.88853E+00_JPRB, 0.98684E+00_JPRB, 0.10660E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11662E+01_JPRB, 0.11743E+01_JPRB, 0.11470E+01_JPRB, 0.11184E+01_JPRB, 0.11116E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.10994E+01_JPRB, 0.10966E+01_JPRB, 0.10964E+01_JPRB, 0.10990E+01_JPRB, 0.11045E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.10907E+01_JPRB, 0.10740E+01_JPRB, 0.10493E+01_JPRB, 0.10259E+01_JPRB, 0.10133E+01_JPRB, 0.10006E+01_JPRB ,&
&  0.98290E+00_JPRB, 0.97409E+00_JPRB, 0.97286E+00_JPRB, 0.97477E+00_JPRB, 0.98344E+00_JPRB, 0.99529E+00_JPRB, 0.10143E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.10547E+01_JPRB, 0.10683E+01_JPRB, 0.10830E+01_JPRB, 0.10907E+01_JPRB, 0.11006E+01_JPRB, 0.11189E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11245E+01_JPRB, 0.11156E+01_JPRB, 0.11068E+01_JPRB, 0.11048E+01_JPRB, 0.11037E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.10849E+01_JPRB, 0.10785E+01_JPRB, 0.10754E+01_JPRB, 0.10801E+01_JPRB, 0.10779E+01_JPRB, 0.10706E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.10499E+01_JPRB, 0.10354E+01_JPRB, 0.10272E+01_JPRB, 0.10172E+01_JPRB, 0.10121E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.98577E+00_JPRB, 0.99068E+00_JPRB, 0.98417E+00_JPRB, 0.98057E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.94843E+00_JPRB, 0.95230E+00_JPRB, 0.95611E+00_JPRB, 0.95707E+00_JPRB, 0.95373E+00_JPRB, 0.95152E+00_JPRB, 0.95333E+00_JPRB ,&
&  0.95457E+00_JPRB, 0.94668E+00_JPRB, 0.93879E+00_JPRB, 0.93261E+00_JPRB, 0.92968E+00_JPRB, 0.91987E+00_JPRB, 0.90515E+00_JPRB ,&
&  0.89449E+00_JPRB, 0.89264E+00_JPRB, 0.89739E+00_JPRB, 0.89970E+00_JPRB, 0.89043E+00_JPRB, 0.88069E+00_JPRB, 0.87711E+00_JPRB ,&
&  0.87793E+00_JPRB, 0.88381E+00_JPRB, 0.89376E+00_JPRB, 0.89908E+00_JPRB, 0.90716E+00_JPRB, 0.92082E+00_JPRB, 0.92088E+00_JPRB ,&
&  0.91904E+00_JPRB, 0.78268E+00_JPRB, 0.82327E+00_JPRB, 0.84015E+00_JPRB, 0.87473E+00_JPRB, 0.93014E+00_JPRB, 0.98156E+00_JPRB ,&
&  0.10331E+01_JPRB, 0.10624E+01_JPRB, 0.10758E+01_JPRB, 0.10687E+01_JPRB, 0.10502E+01_JPRB, 0.10332E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.10446E+01_JPRB, 0.10582E+01_JPRB, 0.10664E+01_JPRB, 0.10685E+01_JPRB, 0.10689E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.10502E+01_JPRB, 0.10229E+01_JPRB, 0.98867E+00_JPRB, 0.95960E+00_JPRB, 0.94248E+00_JPRB, 0.92422E+00_JPRB ,&
&  0.90214E+00_JPRB, 0.88916E+00_JPRB, 0.88705E+00_JPRB, 0.89078E+00_JPRB, 0.90138E+00_JPRB, 0.91378E+00_JPRB, 0.92387E+00_JPRB ,&
&  0.93481E+00_JPRB, 0.95192E+00_JPRB, 0.97083E+00_JPRB, 0.98932E+00_JPRB, 0.10070E+01_JPRB, 0.10225E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10396E+01_JPRB, 0.10285E+01_JPRB, 0.10204E+01_JPRB, 0.10182E+01_JPRB, 0.10243E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.10126E+01_JPRB, 0.10088E+01_JPRB, 0.99718E+00_JPRB, 0.98399E+00_JPRB, 0.97113E+00_JPRB, 0.95744E+00_JPRB ,&
&  0.94667E+00_JPRB, 0.94137E+00_JPRB, 0.93652E+00_JPRB, 0.92931E+00_JPRB, 0.91941E+00_JPRB, 0.91474E+00_JPRB, 0.91097E+00_JPRB ,&
&  0.90778E+00_JPRB, 0.90435E+00_JPRB, 0.90727E+00_JPRB, 0.91958E+00_JPRB, 0.93687E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.89772E+00_JPRB, 0.90451E+00_JPRB, 0.91451E+00_JPRB, 0.92586E+00_JPRB, 0.93019E+00_JPRB, 0.93022E+00_JPRB, 0.92794E+00_JPRB ,&
&  0.92866E+00_JPRB, 0.92044E+00_JPRB, 0.91433E+00_JPRB, 0.91040E+00_JPRB, 0.90922E+00_JPRB, 0.90506E+00_JPRB, 0.89490E+00_JPRB ,&
&  0.88632E+00_JPRB, 0.88266E+00_JPRB, 0.88651E+00_JPRB, 0.88727E+00_JPRB, 0.87124E+00_JPRB, 0.86563E+00_JPRB, 0.85700E+00_JPRB ,&
&  0.85522E+00_JPRB, 0.85454E+00_JPRB, 0.85668E+00_JPRB, 0.86192E+00_JPRB, 0.87741E+00_JPRB, 0.88832E+00_JPRB, 0.89377E+00_JPRB ,&
&  0.89093E+00_JPRB, 0.82139E+00_JPRB, 0.85046E+00_JPRB, 0.86036E+00_JPRB, 0.88154E+00_JPRB, 0.90506E+00_JPRB, 0.92967E+00_JPRB ,&
&  0.96321E+00_JPRB, 0.98360E+00_JPRB, 0.99448E+00_JPRB, 0.99886E+00_JPRB, 0.10012E+01_JPRB, 0.99955E+00_JPRB, 0.99086E+00_JPRB ,&
&  0.98711E+00_JPRB, 0.99661E+00_JPRB, 0.10168E+01_JPRB, 0.10279E+01_JPRB, 0.10340E+01_JPRB, 0.10374E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10117E+01_JPRB, 0.98403E+00_JPRB, 0.94989E+00_JPRB, 0.91099E+00_JPRB, 0.88225E+00_JPRB, 0.86479E+00_JPRB ,&
&  0.85395E+00_JPRB, 0.85257E+00_JPRB, 0.85332E+00_JPRB, 0.85531E+00_JPRB, 0.85949E+00_JPRB, 0.86897E+00_JPRB, 0.87333E+00_JPRB ,&
&  0.88096E+00_JPRB, 0.88811E+00_JPRB, 0.90009E+00_JPRB, 0.92015E+00_JPRB, 0.94038E+00_JPRB, 0.95205E+00_JPRB, 0.95357E+00_JPRB ,&
&  0.96045E+00_JPRB, 0.96453E+00_JPRB, 0.96180E+00_JPRB, 0.96237E+00_JPRB, 0.95924E+00_JPRB, 0.96140E+00_JPRB, 0.95974E+00_JPRB ,&
&  0.95157E+00_JPRB, 0.94531E+00_JPRB, 0.93719E+00_JPRB, 0.92123E+00_JPRB, 0.90674E+00_JPRB, 0.89549E+00_JPRB, 0.88583E+00_JPRB ,&
&  0.88348E+00_JPRB, 0.88644E+00_JPRB, 0.89124E+00_JPRB, 0.88567E+00_JPRB, 0.87121E+00_JPRB, 0.86221E+00_JPRB, 0.85158E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.84733E+00_JPRB, 0.84840E+00_JPRB, 0.85760E+00_JPRB, 0.87903E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.84429E+00_JPRB, 0.85803E+00_JPRB, 0.87525E+00_JPRB, 0.89279E+00_JPRB, 0.90416E+00_JPRB, 0.91124E+00_JPRB, 0.90950E+00_JPRB ,&
&  0.89866E+00_JPRB, 0.88792E+00_JPRB, 0.88961E+00_JPRB, 0.88944E+00_JPRB, 0.88816E+00_JPRB, 0.88594E+00_JPRB, 0.87877E+00_JPRB ,&
&  0.86858E+00_JPRB, 0.85973E+00_JPRB, 0.85571E+00_JPRB, 0.85222E+00_JPRB, 0.84168E+00_JPRB, 0.83334E+00_JPRB, 0.82424E+00_JPRB ,&
&  0.82315E+00_JPRB, 0.82486E+00_JPRB, 0.82683E+00_JPRB, 0.82608E+00_JPRB, 0.83305E+00_JPRB, 0.84222E+00_JPRB, 0.84607E+00_JPRB ,&
&  0.83982E+00_JPRB, 0.85253E+00_JPRB, 0.76704E+00_JPRB, 0.83590E+00_JPRB, 0.85941E+00_JPRB, 0.87349E+00_JPRB, 0.89071E+00_JPRB ,&
&  0.90421E+00_JPRB, 0.91838E+00_JPRB, 0.92911E+00_JPRB, 0.93710E+00_JPRB, 0.95800E+00_JPRB, 0.97254E+00_JPRB, 0.97419E+00_JPRB ,&
&  0.97553E+00_JPRB, 0.97412E+00_JPRB, 0.97599E+00_JPRB, 0.97793E+00_JPRB, 0.98457E+00_JPRB, 0.98682E+00_JPRB, 0.98258E+00_JPRB ,&
&  0.97536E+00_JPRB, 0.96320E+00_JPRB, 0.94408E+00_JPRB, 0.92124E+00_JPRB, 0.88555E+00_JPRB, 0.85443E+00_JPRB, 0.83418E+00_JPRB ,&
&  0.81796E+00_JPRB, 0.81267E+00_JPRB, 0.80674E+00_JPRB, 0.80092E+00_JPRB, 0.79881E+00_JPRB, 0.80063E+00_JPRB, 0.79924E+00_JPRB ,&
&  0.80542E+00_JPRB, 0.81390E+00_JPRB, 0.81809E+00_JPRB, 0.82596E+00_JPRB, 0.83648E+00_JPRB, 0.85154E+00_JPRB, 0.86601E+00_JPRB ,&
&  0.87981E+00_JPRB, 0.88976E+00_JPRB, 0.89231E+00_JPRB, 0.89331E+00_JPRB, 0.89165E+00_JPRB, 0.89105E+00_JPRB, 0.88435E+00_JPRB ,&
&  0.87900E+00_JPRB, 0.87626E+00_JPRB, 0.86983E+00_JPRB, 0.85744E+00_JPRB, 0.84931E+00_JPRB, 0.84220E+00_JPRB, 0.83989E+00_JPRB ,&
&  0.84004E+00_JPRB, 0.84096E+00_JPRB, 0.83832E+00_JPRB, 0.83900E+00_JPRB, 0.83259E+00_JPRB, 0.81210E+00_JPRB, 0.79607E+00_JPRB ,&
&  0.79529E+00_JPRB, 0.80066E+00_JPRB, 0.80566E+00_JPRB, 0.81504E+00_JPRB, 0.83099E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.85635E+00_JPRB, 0.86326E+00_JPRB, 0.87364E+00_JPRB, 0.88853E+00_JPRB, 0.90394E+00_JPRB, 0.91413E+00_JPRB, 0.91437E+00_JPRB ,&
&  0.90362E+00_JPRB, 0.89029E+00_JPRB, 0.88583E+00_JPRB, 0.88510E+00_JPRB, 0.88202E+00_JPRB, 0.88046E+00_JPRB, 0.87559E+00_JPRB ,&
&  0.86388E+00_JPRB, 0.84615E+00_JPRB, 0.83747E+00_JPRB, 0.82302E+00_JPRB, 0.80927E+00_JPRB, 0.79775E+00_JPRB, 0.79701E+00_JPRB ,&
&  0.80504E+00_JPRB, 0.81460E+00_JPRB, 0.82646E+00_JPRB, 0.83968E+00_JPRB, 0.84607E+00_JPRB, 0.84353E+00_JPRB, 0.83940E+00_JPRB ,&
&  0.83583E+00_JPRB, 0.83310E+00_JPRB, 0.84298E+00_JPRB, 0.85124E+00_JPRB, 0.85652E+00_JPRB, 0.86718E+00_JPRB, 0.87688E+00_JPRB ,&
&  0.88611E+00_JPRB, 0.89880E+00_JPRB, 0.91898E+00_JPRB, 0.93443E+00_JPRB, 0.94667E+00_JPRB, 0.95885E+00_JPRB, 0.96122E+00_JPRB ,&
&  0.96611E+00_JPRB, 0.97688E+00_JPRB, 0.97926E+00_JPRB, 0.97057E+00_JPRB, 0.95696E+00_JPRB, 0.94913E+00_JPRB, 0.94571E+00_JPRB ,&
&  0.93788E+00_JPRB, 0.91907E+00_JPRB, 0.90435E+00_JPRB, 0.89193E+00_JPRB, 0.87142E+00_JPRB, 0.84954E+00_JPRB, 0.82436E+00_JPRB ,&
&  0.79592E+00_JPRB, 0.77470E+00_JPRB, 0.75797E+00_JPRB, 0.74283E+00_JPRB, 0.73840E+00_JPRB, 0.72884E+00_JPRB, 0.72186E+00_JPRB ,&
&  0.72628E+00_JPRB, 0.74493E+00_JPRB, 0.75914E+00_JPRB, 0.76453E+00_JPRB, 0.76781E+00_JPRB, 0.78446E+00_JPRB, 0.80470E+00_JPRB ,&
&  0.82081E+00_JPRB, 0.82829E+00_JPRB, 0.82544E+00_JPRB, 0.82876E+00_JPRB, 0.83469E+00_JPRB, 0.83940E+00_JPRB, 0.83354E+00_JPRB ,&
&  0.82507E+00_JPRB, 0.81967E+00_JPRB, 0.81849E+00_JPRB, 0.81469E+00_JPRB, 0.81593E+00_JPRB, 0.81557E+00_JPRB, 0.81252E+00_JPRB ,&
&  0.80666E+00_JPRB, 0.79739E+00_JPRB, 0.78539E+00_JPRB, 0.77785E+00_JPRB, 0.76696E+00_JPRB, 0.75033E+00_JPRB, 0.75067E+00_JPRB ,&
&  0.76194E+00_JPRB, 0.78021E+00_JPRB, 0.79787E+00_JPRB, 0.81915E+00_JPRB, 0.84425E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.87915E+00_JPRB, 0.88782E+00_JPRB, 0.89409E+00_JPRB, 0.89884E+00_JPRB, 0.90847E+00_JPRB, 0.92046E+00_JPRB, 0.92100E+00_JPRB ,&
&  0.90664E+00_JPRB, 0.89289E+00_JPRB, 0.88314E+00_JPRB, 0.88036E+00_JPRB, 0.88196E+00_JPRB, 0.88414E+00_JPRB, 0.87802E+00_JPRB ,&
&  0.85600E+00_JPRB, 0.83242E+00_JPRB, 0.81766E+00_JPRB, 0.80310E+00_JPRB, 0.79378E+00_JPRB, 0.78761E+00_JPRB, 0.79382E+00_JPRB ,&
&  0.80327E+00_JPRB, 0.81860E+00_JPRB, 0.83091E+00_JPRB, 0.83960E+00_JPRB, 0.84266E+00_JPRB, 0.83042E+00_JPRB, 0.83097E+00_JPRB ,&
&  0.83714E+00_JPRB, 0.83861E+00_JPRB, 0.85298E+00_JPRB, 0.85189E+00_JPRB, 0.83782E+00_JPRB, 0.82623E+00_JPRB, 0.81507E+00_JPRB ,&
&  0.81229E+00_JPRB, 0.82386E+00_JPRB, 0.84987E+00_JPRB, 0.88747E+00_JPRB, 0.91039E+00_JPRB, 0.91943E+00_JPRB, 0.91546E+00_JPRB ,&
&  0.91688E+00_JPRB, 0.92831E+00_JPRB, 0.93503E+00_JPRB, 0.92705E+00_JPRB, 0.90934E+00_JPRB, 0.89509E+00_JPRB, 0.88291E+00_JPRB ,&
&  0.87473E+00_JPRB, 0.86450E+00_JPRB, 0.85723E+00_JPRB, 0.84752E+00_JPRB, 0.82492E+00_JPRB, 0.79802E+00_JPRB, 0.77441E+00_JPRB ,&
&  0.75665E+00_JPRB, 0.73918E+00_JPRB, 0.71802E+00_JPRB, 0.71072E+00_JPRB, 0.71388E+00_JPRB, 0.69056E+00_JPRB, 0.66568E+00_JPRB ,&
&  0.66918E+00_JPRB, 0.70066E+00_JPRB, 0.72550E+00_JPRB, 0.73591E+00_JPRB, 0.73734E+00_JPRB, 0.74227E+00_JPRB, 0.74645E+00_JPRB ,&
&  0.75396E+00_JPRB, 0.76365E+00_JPRB, 0.77126E+00_JPRB, 0.78320E+00_JPRB, 0.79537E+00_JPRB, 0.79399E+00_JPRB, 0.77972E+00_JPRB ,&
&  0.76432E+00_JPRB, 0.75574E+00_JPRB, 0.75242E+00_JPRB, 0.75495E+00_JPRB, 0.75840E+00_JPRB, 0.75634E+00_JPRB, 0.75058E+00_JPRB ,&
&  0.73888E+00_JPRB, 0.72053E+00_JPRB, 0.70480E+00_JPRB, 0.69560E+00_JPRB, 0.69472E+00_JPRB, 0.70089E+00_JPRB, 0.71708E+00_JPRB ,&
&  0.74848E+00_JPRB, 0.79634E+00_JPRB, 0.83246E+00_JPRB, 0.85371E+00_JPRB, 0.86068E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.90464E+00_JPRB, 0.91355E+00_JPRB, 0.92375E+00_JPRB, 0.93747E+00_JPRB, 0.95021E+00_JPRB, 0.94767E+00_JPRB, 0.93534E+00_JPRB ,&
&  0.91176E+00_JPRB, 0.90262E+00_JPRB, 0.90700E+00_JPRB, 0.90919E+00_JPRB, 0.91070E+00_JPRB, 0.90827E+00_JPRB, 0.89162E+00_JPRB ,&
&  0.85302E+00_JPRB, 0.82196E+00_JPRB, 0.80674E+00_JPRB, 0.79855E+00_JPRB, 0.79552E+00_JPRB, 0.80173E+00_JPRB, 0.80907E+00_JPRB ,&
&  0.82472E+00_JPRB, 0.83376E+00_JPRB, 0.84527E+00_JPRB, 0.85446E+00_JPRB, 0.85696E+00_JPRB, 0.86048E+00_JPRB, 0.86095E+00_JPRB ,&
&  0.84599E+00_JPRB, 0.84563E+00_JPRB, 0.84613E+00_JPRB, 0.83564E+00_JPRB, 0.81288E+00_JPRB, 0.79002E+00_JPRB, 0.76239E+00_JPRB ,&
&  0.74390E+00_JPRB, 0.73481E+00_JPRB, 0.76050E+00_JPRB, 0.82302E+00_JPRB, 0.86757E+00_JPRB, 0.87588E+00_JPRB, 0.86698E+00_JPRB ,&
&  0.86788E+00_JPRB, 0.87498E+00_JPRB, 0.87870E+00_JPRB, 0.86753E+00_JPRB, 0.84808E+00_JPRB, 0.83112E+00_JPRB, 0.81531E+00_JPRB ,&
&  0.81091E+00_JPRB, 0.81143E+00_JPRB, 0.79530E+00_JPRB, 0.76371E+00_JPRB, 0.73568E+00_JPRB, 0.71562E+00_JPRB, 0.70483E+00_JPRB ,&
&  0.69795E+00_JPRB, 0.68769E+00_JPRB, 0.68089E+00_JPRB, 0.68490E+00_JPRB, 0.66447E+00_JPRB, 0.64635E+00_JPRB, 0.65002E+00_JPRB ,&
&  0.66634E+00_JPRB, 0.67212E+00_JPRB, 0.68171E+00_JPRB, 0.68534E+00_JPRB, 0.68293E+00_JPRB, 0.67982E+00_JPRB, 0.68169E+00_JPRB ,&
&  0.69122E+00_JPRB, 0.70206E+00_JPRB, 0.71019E+00_JPRB, 0.71561E+00_JPRB, 0.72099E+00_JPRB, 0.70924E+00_JPRB, 0.68819E+00_JPRB ,&
&  0.67110E+00_JPRB, 0.66840E+00_JPRB, 0.67342E+00_JPRB, 0.67318E+00_JPRB, 0.67183E+00_JPRB, 0.67217E+00_JPRB, 0.67117E+00_JPRB ,&
&  0.66875E+00_JPRB, 0.66175E+00_JPRB, 0.65310E+00_JPRB, 0.65300E+00_JPRB, 0.66406E+00_JPRB, 0.68710E+00_JPRB, 0.72284E+00_JPRB ,&
&  0.77534E+00_JPRB, 0.81484E+00_JPRB, 0.85367E+00_JPRB, 0.87063E+00_JPRB, 0.87583E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.93680E+00_JPRB, 0.95960E+00_JPRB, 0.97899E+00_JPRB, 0.99226E+00_JPRB, 0.99447E+00_JPRB, 0.98781E+00_JPRB, 0.97174E+00_JPRB ,&
&  0.94155E+00_JPRB, 0.92428E+00_JPRB, 0.93306E+00_JPRB, 0.94755E+00_JPRB, 0.95613E+00_JPRB, 0.95575E+00_JPRB, 0.92353E+00_JPRB ,&
&  0.86781E+00_JPRB, 0.84623E+00_JPRB, 0.84242E+00_JPRB, 0.85130E+00_JPRB, 0.86868E+00_JPRB, 0.87457E+00_JPRB, 0.86134E+00_JPRB ,&
&  0.84815E+00_JPRB, 0.84905E+00_JPRB, 0.85134E+00_JPRB, 0.85803E+00_JPRB, 0.84745E+00_JPRB, 0.82614E+00_JPRB, 0.81636E+00_JPRB ,&
&  0.81297E+00_JPRB, 0.80413E+00_JPRB, 0.79476E+00_JPRB, 0.74560E+00_JPRB, 0.76645E+00_JPRB, 0.75388E+00_JPRB, 0.73431E+00_JPRB ,&
&  0.71613E+00_JPRB, 0.71816E+00_JPRB, 0.76035E+00_JPRB, 0.80762E+00_JPRB, 0.81949E+00_JPRB, 0.82186E+00_JPRB, 0.81585E+00_JPRB ,&
&  0.81167E+00_JPRB, 0.81196E+00_JPRB, 0.81344E+00_JPRB, 0.80540E+00_JPRB, 0.78526E+00_JPRB, 0.75923E+00_JPRB, 0.73091E+00_JPRB ,&
&  0.71426E+00_JPRB, 0.69036E+00_JPRB, 0.64754E+00_JPRB, 0.62775E+00_JPRB, 0.62636E+00_JPRB, 0.63463E+00_JPRB, 0.63847E+00_JPRB ,&
&  0.63525E+00_JPRB, 0.62729E+00_JPRB, 0.62977E+00_JPRB, 0.62560E+00_JPRB, 0.60334E+00_JPRB, 0.58946E+00_JPRB, 0.58306E+00_JPRB ,&
&  0.57183E+00_JPRB, 0.57152E+00_JPRB, 0.57726E+00_JPRB, 0.58963E+00_JPRB, 0.60045E+00_JPRB, 0.61530E+00_JPRB, 0.62203E+00_JPRB ,&
&  0.62469E+00_JPRB, 0.62084E+00_JPRB, 0.61459E+00_JPRB, 0.60252E+00_JPRB, 0.58399E+00_JPRB, 0.57691E+00_JPRB, 0.57880E+00_JPRB ,&
&  0.58846E+00_JPRB, 0.59541E+00_JPRB, 0.59573E+00_JPRB, 0.59521E+00_JPRB, 0.58400E+00_JPRB, 0.57153E+00_JPRB, 0.56228E+00_JPRB ,&
&  0.55825E+00_JPRB, 0.56181E+00_JPRB, 0.57832E+00_JPRB, 0.60443E+00_JPRB, 0.63733E+00_JPRB, 0.67714E+00_JPRB, 0.72256E+00_JPRB ,&
&  0.75442E+00_JPRB, 0.77288E+00_JPRB, 0.79759E+00_JPRB, 0.84645E+00_JPRB, 0.89428E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.93593E+00_JPRB, 0.96383E+00_JPRB, 0.99378E+00_JPRB, 0.10110E+01_JPRB, 0.10121E+01_JPRB, 0.10092E+01_JPRB, 0.99422E+00_JPRB ,&
&  0.96345E+00_JPRB, 0.93829E+00_JPRB, 0.93415E+00_JPRB, 0.93793E+00_JPRB, 0.94930E+00_JPRB, 0.96739E+00_JPRB, 0.96787E+00_JPRB ,&
&  0.93984E+00_JPRB, 0.91932E+00_JPRB, 0.90881E+00_JPRB, 0.90088E+00_JPRB, 0.88299E+00_JPRB, 0.85754E+00_JPRB, 0.84075E+00_JPRB ,&
&  0.83563E+00_JPRB, 0.83493E+00_JPRB, 0.82561E+00_JPRB, 0.81158E+00_JPRB, 0.77991E+00_JPRB, 0.74075E+00_JPRB, 0.70940E+00_JPRB ,&
&  0.69762E+00_JPRB, 0.68624E+00_JPRB, 0.64594E+00_JPRB, 0.65930E+00_JPRB, 0.66304E+00_JPRB, 0.66287E+00_JPRB, 0.65914E+00_JPRB ,&
&  0.66996E+00_JPRB, 0.70322E+00_JPRB, 0.73667E+00_JPRB, 0.74636E+00_JPRB, 0.75533E+00_JPRB, 0.75669E+00_JPRB, 0.75764E+00_JPRB ,&
&  0.76185E+00_JPRB, 0.75391E+00_JPRB, 0.72442E+00_JPRB, 0.68054E+00_JPRB, 0.63732E+00_JPRB, 0.60378E+00_JPRB, 0.57543E+00_JPRB ,&
&  0.55628E+00_JPRB, 0.54456E+00_JPRB, 0.53528E+00_JPRB, 0.53222E+00_JPRB, 0.54027E+00_JPRB, 0.55551E+00_JPRB, 0.55013E+00_JPRB ,&
&  0.53096E+00_JPRB, 0.51286E+00_JPRB, 0.48961E+00_JPRB, 0.45839E+00_JPRB, 0.42590E+00_JPRB, 0.38855E+00_JPRB, 0.36897E+00_JPRB ,&
&  0.37404E+00_JPRB, 0.41363E+00_JPRB, 0.46526E+00_JPRB, 0.48634E+00_JPRB, 0.51456E+00_JPRB, 0.54221E+00_JPRB, 0.53154E+00_JPRB ,&
&  0.48182E+00_JPRB, 0.41595E+00_JPRB, 0.39344E+00_JPRB, 0.38001E+00_JPRB, 0.37317E+00_JPRB, 0.36886E+00_JPRB, 0.36771E+00_JPRB ,&
&  0.37260E+00_JPRB, 0.38393E+00_JPRB, 0.39431E+00_JPRB, 0.39204E+00_JPRB, 0.38234E+00_JPRB, 0.37746E+00_JPRB, 0.37916E+00_JPRB ,&
&  0.39069E+00_JPRB, 0.42037E+00_JPRB, 0.48346E+00_JPRB, 0.59611E+00_JPRB, 0.63492E+00_JPRB, 0.66956E+00_JPRB, 0.70893E+00_JPRB ,&
&  0.74699E+00_JPRB, 0.78178E+00_JPRB, 0.81116E+00_JPRB, 0.84700E+00_JPRB, 0.89497E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.87101E+00_JPRB, 0.89083E+00_JPRB, 0.91330E+00_JPRB, 0.92524E+00_JPRB, 0.92674E+00_JPRB, 0.92409E+00_JPRB, 0.91562E+00_JPRB ,&
&  0.90082E+00_JPRB, 0.88290E+00_JPRB, 0.86955E+00_JPRB, 0.86020E+00_JPRB, 0.85550E+00_JPRB, 0.85756E+00_JPRB, 0.86181E+00_JPRB ,&
&  0.85976E+00_JPRB, 0.85472E+00_JPRB, 0.84171E+00_JPRB, 0.82502E+00_JPRB, 0.80740E+00_JPRB, 0.79378E+00_JPRB, 0.78337E+00_JPRB ,&
&  0.77322E+00_JPRB, 0.75826E+00_JPRB, 0.73521E+00_JPRB, 0.69961E+00_JPRB, 0.65757E+00_JPRB, 0.61926E+00_JPRB, 0.58833E+00_JPRB ,&
&  0.55966E+00_JPRB, 0.49389E+00_JPRB, 0.45757E+00_JPRB, 0.46651E+00_JPRB, 0.55363E+00_JPRB, 0.58706E+00_JPRB, 0.61208E+00_JPRB ,&
&  0.64771E+00_JPRB, 0.68593E+00_JPRB, 0.70426E+00_JPRB, 0.70429E+00_JPRB, 0.69842E+00_JPRB, 0.68549E+00_JPRB, 0.66549E+00_JPRB ,&
&  0.64023E+00_JPRB, 0.60959E+00_JPRB, 0.58341E+00_JPRB, 0.55372E+00_JPRB, 0.52348E+00_JPRB, 0.50144E+00_JPRB, 0.48622E+00_JPRB ,&
&  0.47580E+00_JPRB, 0.46593E+00_JPRB, 0.45791E+00_JPRB, 0.44788E+00_JPRB, 0.43859E+00_JPRB, 0.41719E+00_JPRB, 0.39385E+00_JPRB ,&
&  0.37179E+00_JPRB, 0.35319E+00_JPRB, 0.33449E+00_JPRB, 0.31597E+00_JPRB, 0.30088E+00_JPRB, 0.29088E+00_JPRB, 0.28330E+00_JPRB ,&
&  0.28148E+00_JPRB, 0.28904E+00_JPRB, 0.30959E+00_JPRB, 0.34240E+00_JPRB, 0.37629E+00_JPRB, 0.35324E+00_JPRB, 0.33438E+00_JPRB ,&
&  0.31226E+00_JPRB, 0.29941E+00_JPRB, 0.29467E+00_JPRB, 0.29414E+00_JPRB, 0.29539E+00_JPRB, 0.29768E+00_JPRB, 0.30028E+00_JPRB ,&
&  0.30419E+00_JPRB, 0.30801E+00_JPRB, 0.31188E+00_JPRB, 0.31495E+00_JPRB, 0.31703E+00_JPRB, 0.32025E+00_JPRB, 0.32541E+00_JPRB ,&
&  0.33372E+00_JPRB, 0.34701E+00_JPRB, 0.36765E+00_JPRB, 0.39432E+00_JPRB, 0.42258E+00_JPRB, 0.44812E+00_JPRB, 0.50141E+00_JPRB ,&
&  0.58891E+00_JPRB, 0.69961E+00_JPRB, 0.75967E+00_JPRB, 0.81851E+00_JPRB, 0.84792E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.80249E+00_JPRB, 0.81656E+00_JPRB, 0.82798E+00_JPRB, 0.83115E+00_JPRB, 0.83077E+00_JPRB, 0.82593E+00_JPRB, 0.81384E+00_JPRB ,&
&  0.79418E+00_JPRB, 0.76785E+00_JPRB, 0.74211E+00_JPRB, 0.72160E+00_JPRB, 0.70852E+00_JPRB, 0.71070E+00_JPRB, 0.71858E+00_JPRB ,&
&  0.73505E+00_JPRB, 0.75972E+00_JPRB, 0.76423E+00_JPRB, 0.75679E+00_JPRB, 0.72052E+00_JPRB, 0.68264E+00_JPRB, 0.64798E+00_JPRB ,&
&  0.61690E+00_JPRB, 0.58688E+00_JPRB, 0.57566E+00_JPRB, 0.55627E+00_JPRB, 0.54276E+00_JPRB, 0.52588E+00_JPRB, 0.50107E+00_JPRB ,&
&  0.46040E+00_JPRB, 0.43026E+00_JPRB, 0.41541E+00_JPRB, 0.42477E+00_JPRB, 0.50895E+00_JPRB, 0.57002E+00_JPRB, 0.58207E+00_JPRB ,&
&  0.61581E+00_JPRB, 0.63920E+00_JPRB, 0.63361E+00_JPRB, 0.61301E+00_JPRB, 0.59096E+00_JPRB, 0.56734E+00_JPRB, 0.54722E+00_JPRB ,&
&  0.52727E+00_JPRB, 0.50085E+00_JPRB, 0.45051E+00_JPRB, 0.37904E+00_JPRB, 0.34392E+00_JPRB, 0.33066E+00_JPRB, 0.31622E+00_JPRB ,&
&  0.30544E+00_JPRB, 0.29763E+00_JPRB, 0.29439E+00_JPRB, 0.29020E+00_JPRB, 0.28963E+00_JPRB, 0.28575E+00_JPRB, 0.28207E+00_JPRB ,&
&  0.27674E+00_JPRB, 0.27275E+00_JPRB, 0.26754E+00_JPRB, 0.26246E+00_JPRB, 0.25830E+00_JPRB, 0.25765E+00_JPRB, 0.25760E+00_JPRB ,&
&  0.25887E+00_JPRB, 0.26326E+00_JPRB, 0.27624E+00_JPRB, 0.28327E+00_JPRB, 0.28393E+00_JPRB, 0.28048E+00_JPRB, 0.27330E+00_JPRB ,&
&  0.26592E+00_JPRB, 0.26444E+00_JPRB, 0.26319E+00_JPRB, 0.26404E+00_JPRB, 0.26733E+00_JPRB, 0.27141E+00_JPRB, 0.27453E+00_JPRB ,&
&  0.27709E+00_JPRB, 0.28006E+00_JPRB, 0.28278E+00_JPRB, 0.28571E+00_JPRB, 0.28933E+00_JPRB, 0.29351E+00_JPRB, 0.29914E+00_JPRB ,&
&  0.30489E+00_JPRB, 0.31301E+00_JPRB, 0.32396E+00_JPRB, 0.33985E+00_JPRB, 0.35445E+00_JPRB, 0.36423E+00_JPRB, 0.37825E+00_JPRB ,&
&  0.39514E+00_JPRB, 0.44398E+00_JPRB, 0.57995E+00_JPRB, 0.80171E+00_JPRB, 0.83387E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.72133E+00_JPRB, 0.73453E+00_JPRB, 0.74952E+00_JPRB, 0.75359E+00_JPRB, 0.76416E+00_JPRB, 0.76306E+00_JPRB, 0.74551E+00_JPRB ,&
&  0.71749E+00_JPRB, 0.69057E+00_JPRB, 0.67599E+00_JPRB, 0.65612E+00_JPRB, 0.62747E+00_JPRB, 0.56798E+00_JPRB, 0.54534E+00_JPRB ,&
&  0.52634E+00_JPRB, 0.52561E+00_JPRB, 0.51577E+00_JPRB, 0.51542E+00_JPRB, 0.50380E+00_JPRB, 0.49201E+00_JPRB, 0.47254E+00_JPRB ,&
&  0.45477E+00_JPRB, 0.43801E+00_JPRB, 0.42617E+00_JPRB, 0.42114E+00_JPRB, 0.41874E+00_JPRB, 0.41337E+00_JPRB, 0.39987E+00_JPRB ,&
&  0.39040E+00_JPRB, 0.38825E+00_JPRB, 0.39025E+00_JPRB, 0.39710E+00_JPRB, 0.43794E+00_JPRB, 0.49519E+00_JPRB, 0.49665E+00_JPRB ,&
&  0.51824E+00_JPRB, 0.53065E+00_JPRB, 0.54293E+00_JPRB, 0.53110E+00_JPRB, 0.49756E+00_JPRB, 0.45653E+00_JPRB, 0.40350E+00_JPRB ,&
&  0.35311E+00_JPRB, 0.32358E+00_JPRB, 0.29951E+00_JPRB, 0.28223E+00_JPRB, 0.26999E+00_JPRB, 0.26527E+00_JPRB, 0.25915E+00_JPRB ,&
&  0.25530E+00_JPRB, 0.25230E+00_JPRB, 0.25115E+00_JPRB, 0.25073E+00_JPRB, 0.24990E+00_JPRB, 0.24757E+00_JPRB, 0.24691E+00_JPRB ,&
&  0.24758E+00_JPRB, 0.24763E+00_JPRB, 0.24722E+00_JPRB, 0.24813E+00_JPRB, 0.24915E+00_JPRB, 0.25076E+00_JPRB, 0.25234E+00_JPRB ,&
&  0.25478E+00_JPRB, 0.25617E+00_JPRB, 0.25830E+00_JPRB, 0.26073E+00_JPRB, 0.25974E+00_JPRB, 0.25874E+00_JPRB, 0.25606E+00_JPRB ,&
&  0.25285E+00_JPRB, 0.25290E+00_JPRB, 0.25263E+00_JPRB, 0.25349E+00_JPRB, 0.25556E+00_JPRB, 0.25818E+00_JPRB, 0.26151E+00_JPRB ,&
&  0.26464E+00_JPRB, 0.26789E+00_JPRB, 0.27043E+00_JPRB, 0.27295E+00_JPRB, 0.27713E+00_JPRB, 0.28146E+00_JPRB, 0.28695E+00_JPRB ,&
&  0.29263E+00_JPRB, 0.29913E+00_JPRB, 0.30790E+00_JPRB, 0.31922E+00_JPRB, 0.33596E+00_JPRB, 0.35423E+00_JPRB, 0.38279E+00_JPRB ,&
&  0.40684E+00_JPRB, 0.50351E+00_JPRB, 0.73779E+00_JPRB, 0.67027E+00_JPRB, 0.69203E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.68260E+00_JPRB, 0.70051E+00_JPRB, 0.72520E+00_JPRB, 0.71848E+00_JPRB, 0.73849E+00_JPRB, 0.72526E+00_JPRB, 0.70181E+00_JPRB ,&
&  0.65459E+00_JPRB, 0.59892E+00_JPRB, 0.55203E+00_JPRB, 0.50009E+00_JPRB, 0.45885E+00_JPRB, 0.42921E+00_JPRB, 0.41434E+00_JPRB ,&
&  0.40626E+00_JPRB, 0.40350E+00_JPRB, 0.40094E+00_JPRB, 0.39948E+00_JPRB, 0.39356E+00_JPRB, 0.38950E+00_JPRB, 0.38310E+00_JPRB ,&
&  0.37712E+00_JPRB, 0.37362E+00_JPRB, 0.37044E+00_JPRB, 0.37539E+00_JPRB, 0.38127E+00_JPRB, 0.38716E+00_JPRB, 0.38973E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.39292E+00_JPRB, 0.39590E+00_JPRB, 0.39365E+00_JPRB, 0.40513E+00_JPRB, 0.42594E+00_JPRB, 0.44275E+00_JPRB ,&
&  0.45915E+00_JPRB, 0.47242E+00_JPRB, 0.46761E+00_JPRB, 0.44296E+00_JPRB, 0.37636E+00_JPRB, 0.34222E+00_JPRB, 0.31135E+00_JPRB ,&
&  0.29142E+00_JPRB, 0.27659E+00_JPRB, 0.26640E+00_JPRB, 0.26068E+00_JPRB, 0.25576E+00_JPRB, 0.25335E+00_JPRB, 0.25078E+00_JPRB ,&
&  0.24836E+00_JPRB, 0.24710E+00_JPRB, 0.24542E+00_JPRB, 0.24453E+00_JPRB, 0.24335E+00_JPRB, 0.24179E+00_JPRB, 0.24081E+00_JPRB ,&
&  0.24120E+00_JPRB, 0.24111E+00_JPRB, 0.24247E+00_JPRB, 0.24354E+00_JPRB, 0.24559E+00_JPRB, 0.24724E+00_JPRB, 0.24967E+00_JPRB ,&
&  0.25186E+00_JPRB, 0.25374E+00_JPRB, 0.25370E+00_JPRB, 0.25344E+00_JPRB, 0.25264E+00_JPRB, 0.25230E+00_JPRB, 0.25161E+00_JPRB ,&
&  0.25053E+00_JPRB, 0.25094E+00_JPRB, 0.25155E+00_JPRB, 0.25219E+00_JPRB, 0.25373E+00_JPRB, 0.25573E+00_JPRB, 0.25854E+00_JPRB ,&
&  0.26224E+00_JPRB, 0.26603E+00_JPRB, 0.26930E+00_JPRB, 0.27287E+00_JPRB, 0.27727E+00_JPRB, 0.28213E+00_JPRB, 0.28803E+00_JPRB ,&
&  0.29284E+00_JPRB, 0.30170E+00_JPRB, 0.31425E+00_JPRB, 0.32751E+00_JPRB, 0.34950E+00_JPRB, 0.36850E+00_JPRB, 0.40025E+00_JPRB ,&
&  0.44701E+00_JPRB, 0.61195E+00_JPRB, 0.89350E+00_JPRB, 0.67340E+00_JPRB, 0.66599E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.60025E+00_JPRB, 0.61021E+00_JPRB, 0.62451E+00_JPRB, 0.63622E+00_JPRB, 0.65997E+00_JPRB, 0.67342E+00_JPRB, 0.71790E+00_JPRB ,&
&  0.67240E+00_JPRB, 0.59428E+00_JPRB, 0.56216E+00_JPRB, 0.52674E+00_JPRB, 0.48682E+00_JPRB, 0.45777E+00_JPRB, 0.43739E+00_JPRB ,&
&  0.42488E+00_JPRB, 0.41411E+00_JPRB, 0.40358E+00_JPRB, 0.39068E+00_JPRB, 0.37755E+00_JPRB, 0.36332E+00_JPRB, 0.35297E+00_JPRB ,&
&  0.34695E+00_JPRB, 0.34579E+00_JPRB, 0.34525E+00_JPRB, 0.34907E+00_JPRB, 0.35423E+00_JPRB, 0.36082E+00_JPRB, 0.36878E+00_JPRB ,&
&  0.38812E+00_JPRB, 0.39627E+00_JPRB, 0.39875E+00_JPRB, 0.39830E+00_JPRB, 0.39890E+00_JPRB, 0.40178E+00_JPRB, 0.39242E+00_JPRB ,&
&  0.39765E+00_JPRB, 0.41596E+00_JPRB, 0.42122E+00_JPRB, 0.40040E+00_JPRB, 0.34565E+00_JPRB, 0.31354E+00_JPRB, 0.29053E+00_JPRB ,&
&  0.28259E+00_JPRB, 0.27207E+00_JPRB, 0.26552E+00_JPRB, 0.25906E+00_JPRB, 0.25444E+00_JPRB, 0.25253E+00_JPRB, 0.25053E+00_JPRB ,&
&  0.24929E+00_JPRB, 0.24869E+00_JPRB, 0.24666E+00_JPRB, 0.24585E+00_JPRB, 0.24451E+00_JPRB, 0.24371E+00_JPRB, 0.24334E+00_JPRB ,&
&  0.24313E+00_JPRB, 0.24281E+00_JPRB, 0.24306E+00_JPRB, 0.24361E+00_JPRB, 0.24455E+00_JPRB, 0.24531E+00_JPRB, 0.24603E+00_JPRB ,&
&  0.24707E+00_JPRB, 0.24806E+00_JPRB, 0.24839E+00_JPRB, 0.24911E+00_JPRB, 0.24970E+00_JPRB, 0.25115E+00_JPRB, 0.25189E+00_JPRB ,&
&  0.25263E+00_JPRB, 0.25332E+00_JPRB, 0.25346E+00_JPRB, 0.25433E+00_JPRB, 0.25548E+00_JPRB, 0.25767E+00_JPRB, 0.26010E+00_JPRB ,&
&  0.26320E+00_JPRB, 0.26692E+00_JPRB, 0.27089E+00_JPRB, 0.27486E+00_JPRB, 0.27985E+00_JPRB, 0.28501E+00_JPRB, 0.29196E+00_JPRB ,&
&  0.29943E+00_JPRB, 0.30948E+00_JPRB, 0.32357E+00_JPRB, 0.34024E+00_JPRB, 0.36092E+00_JPRB, 0.37946E+00_JPRB, 0.39872E+00_JPRB ,&
&  0.43187E+00_JPRB, 0.52785E+00_JPRB, 0.58773E+00_JPRB, 0.57996E+00_JPRB, 0.59272E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.57830E+00_JPRB, 0.58145E+00_JPRB, 0.58475E+00_JPRB, 0.58948E+00_JPRB, 0.59460E+00_JPRB, 0.59933E+00_JPRB, 0.60308E+00_JPRB ,&
&  0.58452E+00_JPRB, 0.56140E+00_JPRB, 0.54227E+00_JPRB, 0.51942E+00_JPRB, 0.49852E+00_JPRB, 0.48056E+00_JPRB, 0.46027E+00_JPRB ,&
&  0.43987E+00_JPRB, 0.41959E+00_JPRB, 0.39682E+00_JPRB, 0.37807E+00_JPRB, 0.35967E+00_JPRB, 0.34481E+00_JPRB, 0.33616E+00_JPRB ,&
&  0.33114E+00_JPRB, 0.32962E+00_JPRB, 0.33083E+00_JPRB, 0.33300E+00_JPRB, 0.33524E+00_JPRB, 0.33955E+00_JPRB, 0.34581E+00_JPRB ,&
&  0.35450E+00_JPRB, 0.37010E+00_JPRB, 0.38396E+00_JPRB, 0.38866E+00_JPRB, 0.38445E+00_JPRB, 0.37538E+00_JPRB, 0.36746E+00_JPRB ,&
&  0.35809E+00_JPRB, 0.34607E+00_JPRB, 0.33091E+00_JPRB, 0.31578E+00_JPRB, 0.30169E+00_JPRB, 0.29144E+00_JPRB, 0.28257E+00_JPRB ,&
&  0.27494E+00_JPRB, 0.26815E+00_JPRB, 0.26282E+00_JPRB, 0.25929E+00_JPRB, 0.25624E+00_JPRB, 0.25388E+00_JPRB, 0.25153E+00_JPRB ,&
&  0.25049E+00_JPRB, 0.24942E+00_JPRB, 0.24877E+00_JPRB, 0.24841E+00_JPRB, 0.24782E+00_JPRB, 0.24758E+00_JPRB, 0.24742E+00_JPRB ,&
&  0.24701E+00_JPRB, 0.24631E+00_JPRB, 0.24593E+00_JPRB, 0.24569E+00_JPRB, 0.24622E+00_JPRB, 0.24669E+00_JPRB, 0.24739E+00_JPRB ,&
&  0.24841E+00_JPRB, 0.25043E+00_JPRB, 0.25165E+00_JPRB, 0.25283E+00_JPRB, 0.25345E+00_JPRB, 0.25404E+00_JPRB, 0.25482E+00_JPRB ,&
&  0.25606E+00_JPRB, 0.25751E+00_JPRB, 0.25878E+00_JPRB, 0.26048E+00_JPRB, 0.26222E+00_JPRB, 0.26461E+00_JPRB, 0.26722E+00_JPRB ,&
&  0.27011E+00_JPRB, 0.27371E+00_JPRB, 0.27805E+00_JPRB, 0.28191E+00_JPRB, 0.28809E+00_JPRB, 0.29433E+00_JPRB, 0.30139E+00_JPRB ,&
&  0.30883E+00_JPRB, 0.31691E+00_JPRB, 0.32770E+00_JPRB, 0.34054E+00_JPRB, 0.35694E+00_JPRB, 0.36469E+00_JPRB, 0.37486E+00_JPRB ,&
&  0.39066E+00_JPRB, 0.42364E+00_JPRB, 0.56008E+00_JPRB, 0.57359E+00_JPRB, 0.57870E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.35803E+00_JPRB, 0.36376E+00_JPRB, 0.36760E+00_JPRB, 0.37650E+00_JPRB, 0.38737E+00_JPRB, 0.48614E+00_JPRB, 0.48123E+00_JPRB ,&
&  0.48064E+00_JPRB, 0.43748E+00_JPRB, 0.42982E+00_JPRB, 0.41780E+00_JPRB, 0.40253E+00_JPRB, 0.38368E+00_JPRB, 0.37205E+00_JPRB ,&
&  0.36239E+00_JPRB, 0.35100E+00_JPRB, 0.34045E+00_JPRB, 0.33415E+00_JPRB, 0.32916E+00_JPRB, 0.32464E+00_JPRB, 0.32185E+00_JPRB ,&
&  0.32045E+00_JPRB, 0.32049E+00_JPRB, 0.32084E+00_JPRB, 0.32109E+00_JPRB, 0.32416E+00_JPRB, 0.32763E+00_JPRB, 0.33117E+00_JPRB ,&
&  0.33256E+00_JPRB, 0.33713E+00_JPRB, 0.33824E+00_JPRB, 0.33520E+00_JPRB, 0.33040E+00_JPRB, 0.32298E+00_JPRB, 0.31551E+00_JPRB ,&
&  0.30784E+00_JPRB, 0.30086E+00_JPRB, 0.29502E+00_JPRB, 0.28974E+00_JPRB, 0.28363E+00_JPRB, 0.27768E+00_JPRB, 0.27374E+00_JPRB ,&
&  0.27008E+00_JPRB, 0.26630E+00_JPRB, 0.26327E+00_JPRB, 0.26177E+00_JPRB, 0.26008E+00_JPRB, 0.25894E+00_JPRB, 0.25805E+00_JPRB ,&
&  0.25780E+00_JPRB, 0.25764E+00_JPRB, 0.25769E+00_JPRB, 0.25795E+00_JPRB, 0.25790E+00_JPRB, 0.25781E+00_JPRB, 0.25775E+00_JPRB ,&
&  0.25836E+00_JPRB, 0.25842E+00_JPRB, 0.25837E+00_JPRB, 0.25820E+00_JPRB, 0.25901E+00_JPRB, 0.25933E+00_JPRB, 0.25981E+00_JPRB ,&
&  0.26048E+00_JPRB, 0.26139E+00_JPRB, 0.26206E+00_JPRB, 0.26282E+00_JPRB, 0.26355E+00_JPRB, 0.26448E+00_JPRB, 0.26556E+00_JPRB ,&
&  0.26667E+00_JPRB, 0.26829E+00_JPRB, 0.26996E+00_JPRB, 0.27142E+00_JPRB, 0.27273E+00_JPRB, 0.27449E+00_JPRB, 0.27650E+00_JPRB ,&
&  0.27941E+00_JPRB, 0.28223E+00_JPRB, 0.28510E+00_JPRB, 0.28666E+00_JPRB, 0.28935E+00_JPRB, 0.29165E+00_JPRB, 0.29416E+00_JPRB ,&
&  0.29723E+00_JPRB, 0.30231E+00_JPRB, 0.30984E+00_JPRB, 0.31820E+00_JPRB, 0.32773E+00_JPRB, 0.33493E+00_JPRB, 0.33957E+00_JPRB ,&
&  0.34340E+00_JPRB, 0.34760E+00_JPRB, 0.35253E+00_JPRB, 0.35505E+00_JPRB, 0.35698E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.32356E+00_JPRB, 0.32348E+00_JPRB, 0.32354E+00_JPRB, 0.32316E+00_JPRB, 0.32148E+00_JPRB, 0.32011E+00_JPRB, 0.31884E+00_JPRB ,&
&  0.31807E+00_JPRB, 0.31704E+00_JPRB, 0.31653E+00_JPRB, 0.31623E+00_JPRB, 0.31570E+00_JPRB, 0.31500E+00_JPRB, 0.31408E+00_JPRB ,&
&  0.31287E+00_JPRB, 0.31131E+00_JPRB, 0.30956E+00_JPRB, 0.30905E+00_JPRB, 0.30890E+00_JPRB, 0.30876E+00_JPRB, 0.30853E+00_JPRB ,&
&  0.30817E+00_JPRB, 0.30786E+00_JPRB, 0.30716E+00_JPRB, 0.30726E+00_JPRB, 0.30721E+00_JPRB, 0.30717E+00_JPRB, 0.30723E+00_JPRB ,&
&  0.30738E+00_JPRB, 0.30764E+00_JPRB, 0.30749E+00_JPRB, 0.30706E+00_JPRB, 0.30623E+00_JPRB, 0.30454E+00_JPRB, 0.30184E+00_JPRB ,&
&  0.29878E+00_JPRB, 0.29572E+00_JPRB, 0.29327E+00_JPRB, 0.29101E+00_JPRB, 0.28894E+00_JPRB, 0.28706E+00_JPRB, 0.28551E+00_JPRB ,&
&  0.28361E+00_JPRB, 0.28174E+00_JPRB, 0.27999E+00_JPRB, 0.27871E+00_JPRB, 0.27728E+00_JPRB, 0.27603E+00_JPRB, 0.27483E+00_JPRB ,&
&  0.27399E+00_JPRB, 0.27326E+00_JPRB, 0.27264E+00_JPRB, 0.27231E+00_JPRB, 0.27205E+00_JPRB, 0.27182E+00_JPRB, 0.27156E+00_JPRB ,&
&  0.27133E+00_JPRB, 0.27120E+00_JPRB, 0.27111E+00_JPRB, 0.27111E+00_JPRB, 0.27125E+00_JPRB, 0.27118E+00_JPRB, 0.27108E+00_JPRB ,&
&  0.27106E+00_JPRB, 0.27140E+00_JPRB, 0.27162E+00_JPRB, 0.27194E+00_JPRB, 0.27242E+00_JPRB, 0.27311E+00_JPRB, 0.27385E+00_JPRB ,&
&  0.27460E+00_JPRB, 0.27547E+00_JPRB, 0.27640E+00_JPRB, 0.27756E+00_JPRB, 0.27881E+00_JPRB, 0.28006E+00_JPRB, 0.28125E+00_JPRB ,&
&  0.28254E+00_JPRB, 0.28373E+00_JPRB, 0.28485E+00_JPRB, 0.28563E+00_JPRB, 0.28679E+00_JPRB, 0.28800E+00_JPRB, 0.28943E+00_JPRB ,&
&  0.29108E+00_JPRB, 0.29328E+00_JPRB, 0.29640E+00_JPRB, 0.30013E+00_JPRB, 0.30492E+00_JPRB, 0.30837E+00_JPRB, 0.31179E+00_JPRB ,&
&  0.31419E+00_JPRB, 0.31610E+00_JPRB, 0.31757E+00_JPRB, 0.31940E+00_JPRB, 0.32140E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB, 0.28966E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2010:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2010
