!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2020 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2020* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB, 0.58285E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.60532E+00_JPRB, 0.60432E+00_JPRB, 0.60341E+00_JPRB, 0.60261E+00_JPRB, 0.60182E+00_JPRB, 0.60123E+00_JPRB, 0.60073E+00_JPRB ,&
&  0.60003E+00_JPRB, 0.59930E+00_JPRB, 0.59844E+00_JPRB, 0.59765E+00_JPRB, 0.59690E+00_JPRB, 0.59605E+00_JPRB, 0.59503E+00_JPRB ,&
&  0.59384E+00_JPRB, 0.59275E+00_JPRB, 0.59167E+00_JPRB, 0.59040E+00_JPRB, 0.58908E+00_JPRB, 0.58757E+00_JPRB, 0.58586E+00_JPRB ,&
&  0.58391E+00_JPRB, 0.58192E+00_JPRB, 0.57983E+00_JPRB, 0.57766E+00_JPRB, 0.57524E+00_JPRB, 0.57244E+00_JPRB, 0.56995E+00_JPRB ,&
&  0.56811E+00_JPRB, 0.56620E+00_JPRB, 0.56437E+00_JPRB, 0.56249E+00_JPRB, 0.56092E+00_JPRB, 0.55976E+00_JPRB, 0.55855E+00_JPRB ,&
&  0.55753E+00_JPRB, 0.55668E+00_JPRB, 0.55661E+00_JPRB, 0.55683E+00_JPRB, 0.55723E+00_JPRB, 0.55835E+00_JPRB, 0.56019E+00_JPRB ,&
&  0.56176E+00_JPRB, 0.56313E+00_JPRB, 0.56481E+00_JPRB, 0.56668E+00_JPRB, 0.56864E+00_JPRB, 0.57061E+00_JPRB, 0.57246E+00_JPRB ,&
&  0.57390E+00_JPRB, 0.57507E+00_JPRB, 0.57632E+00_JPRB, 0.57738E+00_JPRB, 0.57798E+00_JPRB, 0.57850E+00_JPRB, 0.57906E+00_JPRB ,&
&  0.57950E+00_JPRB, 0.57983E+00_JPRB, 0.58028E+00_JPRB, 0.58108E+00_JPRB, 0.58184E+00_JPRB, 0.58268E+00_JPRB, 0.58381E+00_JPRB ,&
&  0.58532E+00_JPRB, 0.58645E+00_JPRB, 0.58784E+00_JPRB, 0.58956E+00_JPRB, 0.59174E+00_JPRB, 0.59433E+00_JPRB, 0.59648E+00_JPRB ,&
&  0.59875E+00_JPRB, 0.60117E+00_JPRB, 0.60375E+00_JPRB, 0.60568E+00_JPRB, 0.60779E+00_JPRB, 0.60978E+00_JPRB, 0.61169E+00_JPRB ,&
&  0.61240E+00_JPRB, 0.61345E+00_JPRB, 0.61478E+00_JPRB, 0.61614E+00_JPRB, 0.61579E+00_JPRB, 0.61558E+00_JPRB, 0.61542E+00_JPRB ,&
&  0.61616E+00_JPRB, 0.61493E+00_JPRB, 0.61439E+00_JPRB, 0.61359E+00_JPRB, 0.61323E+00_JPRB, 0.61194E+00_JPRB, 0.61099E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.60914E+00_JPRB, 0.60812E+00_JPRB, 0.60708E+00_JPRB, 0.60610E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.62612E+00_JPRB, 0.62425E+00_JPRB, 0.62224E+00_JPRB, 0.62015E+00_JPRB, 0.61785E+00_JPRB, 0.61607E+00_JPRB, 0.61431E+00_JPRB ,&
&  0.61245E+00_JPRB, 0.61083E+00_JPRB, 0.60923E+00_JPRB, 0.60788E+00_JPRB, 0.60661E+00_JPRB, 0.60600E+00_JPRB, 0.60498E+00_JPRB ,&
&  0.60369E+00_JPRB, 0.60186E+00_JPRB, 0.60027E+00_JPRB, 0.59775E+00_JPRB, 0.59491E+00_JPRB, 0.59113E+00_JPRB, 0.58761E+00_JPRB ,&
&  0.58363E+00_JPRB, 0.57891E+00_JPRB, 0.57469E+00_JPRB, 0.57073E+00_JPRB, 0.56680E+00_JPRB, 0.56244E+00_JPRB, 0.55930E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.55474E+00_JPRB, 0.55235E+00_JPRB, 0.55031E+00_JPRB, 0.54794E+00_JPRB, 0.54609E+00_JPRB, 0.54430E+00_JPRB ,&
&  0.54130E+00_JPRB, 0.53827E+00_JPRB, 0.53829E+00_JPRB, 0.53878E+00_JPRB, 0.54016E+00_JPRB, 0.54315E+00_JPRB, 0.54868E+00_JPRB ,&
&  0.55597E+00_JPRB, 0.56282E+00_JPRB, 0.57024E+00_JPRB, 0.57688E+00_JPRB, 0.58212E+00_JPRB, 0.58628E+00_JPRB, 0.58969E+00_JPRB ,&
&  0.59194E+00_JPRB, 0.59359E+00_JPRB, 0.59438E+00_JPRB, 0.59423E+00_JPRB, 0.59319E+00_JPRB, 0.59273E+00_JPRB, 0.59270E+00_JPRB ,&
&  0.59295E+00_JPRB, 0.59097E+00_JPRB, 0.59032E+00_JPRB, 0.59083E+00_JPRB, 0.59230E+00_JPRB, 0.59385E+00_JPRB, 0.59678E+00_JPRB ,&
&  0.60084E+00_JPRB, 0.60571E+00_JPRB, 0.61047E+00_JPRB, 0.61669E+00_JPRB, 0.62388E+00_JPRB, 0.63196E+00_JPRB, 0.63715E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.65031E+00_JPRB, 0.65740E+00_JPRB, 0.65946E+00_JPRB, 0.66197E+00_JPRB, 0.66416E+00_JPRB, 0.66684E+00_JPRB ,&
&  0.66494E+00_JPRB, 0.66486E+00_JPRB, 0.66517E+00_JPRB, 0.66717E+00_JPRB, 0.66468E+00_JPRB, 0.66382E+00_JPRB, 0.66284E+00_JPRB ,&
&  0.66420E+00_JPRB, 0.65930E+00_JPRB, 0.65655E+00_JPRB, 0.65285E+00_JPRB, 0.65060E+00_JPRB, 0.64530E+00_JPRB, 0.64114E+00_JPRB ,&
&  0.63762E+00_JPRB, 0.63542E+00_JPRB, 0.63259E+00_JPRB, 0.63018E+00_JPRB, 0.62823E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.65020E+00_JPRB, 0.64599E+00_JPRB, 0.63974E+00_JPRB, 0.63188E+00_JPRB, 0.62448E+00_JPRB, 0.61893E+00_JPRB, 0.61299E+00_JPRB ,&
&  0.60814E+00_JPRB, 0.60387E+00_JPRB, 0.60083E+00_JPRB, 0.59853E+00_JPRB, 0.59632E+00_JPRB, 0.59677E+00_JPRB, 0.59718E+00_JPRB ,&
&  0.59768E+00_JPRB, 0.59720E+00_JPRB, 0.59879E+00_JPRB, 0.59804E+00_JPRB, 0.59612E+00_JPRB, 0.59184E+00_JPRB, 0.58600E+00_JPRB ,&
&  0.57926E+00_JPRB, 0.57158E+00_JPRB, 0.56289E+00_JPRB, 0.55307E+00_JPRB, 0.54373E+00_JPRB, 0.51054E+00_JPRB, 0.50230E+00_JPRB ,&
&  0.50355E+00_JPRB, 0.49873E+00_JPRB, 0.49506E+00_JPRB, 0.49317E+00_JPRB, 0.49003E+00_JPRB, 0.48547E+00_JPRB, 0.48091E+00_JPRB ,&
&  0.45563E+00_JPRB, 0.45405E+00_JPRB, 0.45139E+00_JPRB, 0.44990E+00_JPRB, 0.45405E+00_JPRB, 0.46811E+00_JPRB, 0.48979E+00_JPRB ,&
&  0.52884E+00_JPRB, 0.54538E+00_JPRB, 0.55941E+00_JPRB, 0.57572E+00_JPRB, 0.58875E+00_JPRB, 0.59863E+00_JPRB, 0.60842E+00_JPRB ,&
&  0.61658E+00_JPRB, 0.62453E+00_JPRB, 0.63035E+00_JPRB, 0.63329E+00_JPRB, 0.63361E+00_JPRB, 0.63349E+00_JPRB, 0.63299E+00_JPRB ,&
&  0.63230E+00_JPRB, 0.63130E+00_JPRB, 0.63111E+00_JPRB, 0.63073E+00_JPRB, 0.62979E+00_JPRB, 0.63032E+00_JPRB, 0.63445E+00_JPRB ,&
&  0.64100E+00_JPRB, 0.65076E+00_JPRB, 0.65965E+00_JPRB, 0.67247E+00_JPRB, 0.68468E+00_JPRB, 0.70018E+00_JPRB, 0.70698E+00_JPRB ,&
&  0.71536E+00_JPRB, 0.72100E+00_JPRB, 0.72839E+00_JPRB, 0.72904E+00_JPRB, 0.72976E+00_JPRB, 0.72933E+00_JPRB, 0.73055E+00_JPRB ,&
&  0.72740E+00_JPRB, 0.72546E+00_JPRB, 0.72265E+00_JPRB, 0.72296E+00_JPRB, 0.71902E+00_JPRB, 0.71885E+00_JPRB, 0.71677E+00_JPRB ,&
&  0.71909E+00_JPRB, 0.71377E+00_JPRB, 0.71175E+00_JPRB, 0.70489E+00_JPRB, 0.69925E+00_JPRB, 0.68897E+00_JPRB, 0.67998E+00_JPRB ,&
&  0.67229E+00_JPRB, 0.66652E+00_JPRB, 0.66164E+00_JPRB, 0.65751E+00_JPRB, 0.65362E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.66783E+00_JPRB, 0.65616E+00_JPRB, 0.64333E+00_JPRB, 0.63253E+00_JPRB, 0.62245E+00_JPRB, 0.61497E+00_JPRB, 0.60770E+00_JPRB ,&
&  0.60214E+00_JPRB, 0.59677E+00_JPRB, 0.59277E+00_JPRB, 0.58912E+00_JPRB, 0.58634E+00_JPRB, 0.58464E+00_JPRB, 0.58305E+00_JPRB ,&
&  0.58256E+00_JPRB, 0.58182E+00_JPRB, 0.58246E+00_JPRB, 0.58246E+00_JPRB, 0.58295E+00_JPRB, 0.58027E+00_JPRB, 0.57451E+00_JPRB ,&
&  0.56716E+00_JPRB, 0.55892E+00_JPRB, 0.53150E+00_JPRB, 0.52100E+00_JPRB, 0.49807E+00_JPRB, 0.48102E+00_JPRB, 0.47147E+00_JPRB ,&
&  0.45542E+00_JPRB, 0.44664E+00_JPRB, 0.43007E+00_JPRB, 0.41985E+00_JPRB, 0.41687E+00_JPRB, 0.40562E+00_JPRB, 0.40061E+00_JPRB ,&
&  0.39255E+00_JPRB, 0.37893E+00_JPRB, 0.37357E+00_JPRB, 0.37153E+00_JPRB, 0.37602E+00_JPRB, 0.38856E+00_JPRB, 0.41081E+00_JPRB ,&
&  0.45259E+00_JPRB, 0.49909E+00_JPRB, 0.56262E+00_JPRB, 0.58105E+00_JPRB, 0.58440E+00_JPRB, 0.59929E+00_JPRB, 0.61144E+00_JPRB ,&
&  0.61894E+00_JPRB, 0.63572E+00_JPRB, 0.65335E+00_JPRB, 0.66105E+00_JPRB, 0.66609E+00_JPRB, 0.66806E+00_JPRB, 0.66977E+00_JPRB ,&
&  0.67135E+00_JPRB, 0.67796E+00_JPRB, 0.68157E+00_JPRB, 0.68474E+00_JPRB, 0.68772E+00_JPRB, 0.69124E+00_JPRB, 0.69991E+00_JPRB ,&
&  0.70750E+00_JPRB, 0.72525E+00_JPRB, 0.74075E+00_JPRB, 0.76578E+00_JPRB, 0.77723E+00_JPRB, 0.79108E+00_JPRB, 0.79536E+00_JPRB ,&
&  0.80198E+00_JPRB, 0.80584E+00_JPRB, 0.80882E+00_JPRB, 0.80224E+00_JPRB, 0.80143E+00_JPRB, 0.80699E+00_JPRB, 0.80593E+00_JPRB ,&
&  0.80371E+00_JPRB, 0.80241E+00_JPRB, 0.80026E+00_JPRB, 0.79868E+00_JPRB, 0.79771E+00_JPRB, 0.79979E+00_JPRB, 0.79976E+00_JPRB ,&
&  0.80478E+00_JPRB, 0.80177E+00_JPRB, 0.79898E+00_JPRB, 0.79096E+00_JPRB, 0.78045E+00_JPRB, 0.76516E+00_JPRB, 0.74537E+00_JPRB ,&
&  0.72815E+00_JPRB, 0.71476E+00_JPRB, 0.70068E+00_JPRB, 0.68994E+00_JPRB, 0.67861E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.63760E+00_JPRB, 0.62275E+00_JPRB, 0.61012E+00_JPRB, 0.60094E+00_JPRB, 0.59293E+00_JPRB, 0.58764E+00_JPRB, 0.58269E+00_JPRB ,&
&  0.58001E+00_JPRB, 0.57694E+00_JPRB, 0.57563E+00_JPRB, 0.57401E+00_JPRB, 0.57373E+00_JPRB, 0.57405E+00_JPRB, 0.57519E+00_JPRB ,&
&  0.57680E+00_JPRB, 0.57744E+00_JPRB, 0.57847E+00_JPRB, 0.57638E+00_JPRB, 0.57473E+00_JPRB, 0.57152E+00_JPRB, 0.56834E+00_JPRB ,&
&  0.56024E+00_JPRB, 0.55043E+00_JPRB, 0.53064E+00_JPRB, 0.51900E+00_JPRB, 0.49763E+00_JPRB, 0.48255E+00_JPRB, 0.46851E+00_JPRB ,&
&  0.45847E+00_JPRB, 0.43330E+00_JPRB, 0.44010E+00_JPRB, 0.42331E+00_JPRB, 0.40735E+00_JPRB, 0.39114E+00_JPRB, 0.37236E+00_JPRB ,&
&  0.36418E+00_JPRB, 0.34583E+00_JPRB, 0.33875E+00_JPRB, 0.33874E+00_JPRB, 0.34137E+00_JPRB, 0.35421E+00_JPRB, 0.37427E+00_JPRB ,&
&  0.41487E+00_JPRB, 0.51668E+00_JPRB, 0.55279E+00_JPRB, 0.56904E+00_JPRB, 0.56545E+00_JPRB, 0.56737E+00_JPRB, 0.56231E+00_JPRB ,&
&  0.58069E+00_JPRB, 0.60528E+00_JPRB, 0.63050E+00_JPRB, 0.64769E+00_JPRB, 0.66844E+00_JPRB, 0.68038E+00_JPRB, 0.69108E+00_JPRB ,&
&  0.70354E+00_JPRB, 0.72220E+00_JPRB, 0.73849E+00_JPRB, 0.75377E+00_JPRB, 0.77182E+00_JPRB, 0.78959E+00_JPRB, 0.81071E+00_JPRB ,&
&  0.82373E+00_JPRB, 0.85364E+00_JPRB, 0.86502E+00_JPRB, 0.88111E+00_JPRB, 0.88472E+00_JPRB, 0.88426E+00_JPRB, 0.88625E+00_JPRB ,&
&  0.89106E+00_JPRB, 0.89708E+00_JPRB, 0.90222E+00_JPRB, 0.90129E+00_JPRB, 0.90118E+00_JPRB, 0.88978E+00_JPRB, 0.88582E+00_JPRB ,&
&  0.88636E+00_JPRB, 0.88536E+00_JPRB, 0.88284E+00_JPRB, 0.87699E+00_JPRB, 0.86902E+00_JPRB, 0.86041E+00_JPRB, 0.85590E+00_JPRB ,&
&  0.85012E+00_JPRB, 0.83954E+00_JPRB, 0.82811E+00_JPRB, 0.81528E+00_JPRB, 0.80200E+00_JPRB, 0.78795E+00_JPRB, 0.76492E+00_JPRB ,&
&  0.73680E+00_JPRB, 0.71249E+00_JPRB, 0.68967E+00_JPRB, 0.67223E+00_JPRB, 0.65386E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.61406E+00_JPRB, 0.59949E+00_JPRB, 0.58681E+00_JPRB, 0.57939E+00_JPRB, 0.57234E+00_JPRB, 0.56911E+00_JPRB, 0.56506E+00_JPRB ,&
&  0.56296E+00_JPRB, 0.56109E+00_JPRB, 0.56046E+00_JPRB, 0.56009E+00_JPRB, 0.56061E+00_JPRB, 0.56150E+00_JPRB, 0.56336E+00_JPRB ,&
&  0.56558E+00_JPRB, 0.56879E+00_JPRB, 0.57230E+00_JPRB, 0.57248E+00_JPRB, 0.57240E+00_JPRB, 0.57074E+00_JPRB, 0.56843E+00_JPRB ,&
&  0.56185E+00_JPRB, 0.55299E+00_JPRB, 0.53428E+00_JPRB, 0.52071E+00_JPRB, 0.50837E+00_JPRB, 0.49161E+00_JPRB, 0.48823E+00_JPRB ,&
&  0.48403E+00_JPRB, 0.47211E+00_JPRB, 0.45487E+00_JPRB, 0.44654E+00_JPRB, 0.43393E+00_JPRB, 0.39067E+00_JPRB, 0.38920E+00_JPRB ,&
&  0.37553E+00_JPRB, 0.34836E+00_JPRB, 0.33999E+00_JPRB, 0.33434E+00_JPRB, 0.33601E+00_JPRB, 0.34737E+00_JPRB, 0.36516E+00_JPRB ,&
&  0.40349E+00_JPRB, 0.52041E+00_JPRB, 0.54952E+00_JPRB, 0.55939E+00_JPRB, 0.54751E+00_JPRB, 0.55165E+00_JPRB, 0.55227E+00_JPRB ,&
&  0.56528E+00_JPRB, 0.57814E+00_JPRB, 0.59393E+00_JPRB, 0.60783E+00_JPRB, 0.62212E+00_JPRB, 0.63018E+00_JPRB, 0.64604E+00_JPRB ,&
&  0.67520E+00_JPRB, 0.71109E+00_JPRB, 0.74307E+00_JPRB, 0.78143E+00_JPRB, 0.82707E+00_JPRB, 0.90848E+00_JPRB, 0.99222E+00_JPRB ,&
&  0.99892E+00_JPRB, 0.98840E+00_JPRB, 0.98594E+00_JPRB, 0.97922E+00_JPRB, 0.97874E+00_JPRB, 0.97824E+00_JPRB, 0.98372E+00_JPRB ,&
&  0.99684E+00_JPRB, 0.10086E+01_JPRB, 0.99173E+00_JPRB, 0.95956E+00_JPRB, 0.94637E+00_JPRB, 0.92341E+00_JPRB, 0.92171E+00_JPRB ,&
&  0.91782E+00_JPRB, 0.93274E+00_JPRB, 0.92645E+00_JPRB, 0.91351E+00_JPRB, 0.90147E+00_JPRB, 0.88823E+00_JPRB, 0.88087E+00_JPRB ,&
&  0.87179E+00_JPRB, 0.85633E+00_JPRB, 0.83906E+00_JPRB, 0.82444E+00_JPRB, 0.81083E+00_JPRB, 0.79949E+00_JPRB, 0.78234E+00_JPRB ,&
&  0.75092E+00_JPRB, 0.71184E+00_JPRB, 0.68174E+00_JPRB, 0.65403E+00_JPRB, 0.63148E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.59962E+00_JPRB, 0.58949E+00_JPRB, 0.58311E+00_JPRB, 0.58159E+00_JPRB, 0.57952E+00_JPRB, 0.57876E+00_JPRB, 0.57466E+00_JPRB ,&
&  0.57051E+00_JPRB, 0.56630E+00_JPRB, 0.56250E+00_JPRB, 0.55923E+00_JPRB, 0.55729E+00_JPRB, 0.55554E+00_JPRB, 0.55473E+00_JPRB ,&
&  0.55439E+00_JPRB, 0.54841E+00_JPRB, 0.54721E+00_JPRB, 0.55075E+00_JPRB, 0.55525E+00_JPRB, 0.55879E+00_JPRB, 0.56340E+00_JPRB ,&
&  0.56058E+00_JPRB, 0.55275E+00_JPRB, 0.54254E+00_JPRB, 0.52888E+00_JPRB, 0.51809E+00_JPRB, 0.51390E+00_JPRB, 0.50621E+00_JPRB ,&
&  0.49099E+00_JPRB, 0.47731E+00_JPRB, 0.48310E+00_JPRB, 0.47882E+00_JPRB, 0.45230E+00_JPRB, 0.43837E+00_JPRB, 0.41443E+00_JPRB ,&
&  0.38715E+00_JPRB, 0.35350E+00_JPRB, 0.34529E+00_JPRB, 0.34172E+00_JPRB, 0.34833E+00_JPRB, 0.35992E+00_JPRB, 0.38593E+00_JPRB ,&
&  0.43262E+00_JPRB, 0.52460E+00_JPRB, 0.54884E+00_JPRB, 0.56095E+00_JPRB, 0.56655E+00_JPRB, 0.56564E+00_JPRB, 0.56531E+00_JPRB ,&
&  0.57426E+00_JPRB, 0.58324E+00_JPRB, 0.59753E+00_JPRB, 0.61341E+00_JPRB, 0.63334E+00_JPRB, 0.65037E+00_JPRB, 0.66389E+00_JPRB ,&
&  0.70344E+00_JPRB, 0.72773E+00_JPRB, 0.75143E+00_JPRB, 0.82044E+00_JPRB, 0.94528E+00_JPRB, 0.10737E+01_JPRB, 0.11236E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.10987E+01_JPRB, 0.10589E+01_JPRB, 0.10346E+01_JPRB, 0.10237E+01_JPRB, 0.10217E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10599E+01_JPRB, 0.11259E+01_JPRB, 0.10755E+01_JPRB, 0.10456E+01_JPRB, 0.10146E+01_JPRB, 0.97835E+00_JPRB ,&
&  0.94987E+00_JPRB, 0.93373E+00_JPRB, 0.92319E+00_JPRB, 0.90940E+00_JPRB, 0.89821E+00_JPRB, 0.88994E+00_JPRB, 0.89269E+00_JPRB ,&
&  0.88829E+00_JPRB, 0.87639E+00_JPRB, 0.85227E+00_JPRB, 0.83798E+00_JPRB, 0.82365E+00_JPRB, 0.80865E+00_JPRB, 0.79247E+00_JPRB ,&
&  0.75837E+00_JPRB, 0.71317E+00_JPRB, 0.67720E+00_JPRB, 0.64535E+00_JPRB, 0.61897E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.60416E+00_JPRB, 0.59897E+00_JPRB, 0.60037E+00_JPRB, 0.60698E+00_JPRB, 0.61114E+00_JPRB, 0.60271E+00_JPRB, 0.59238E+00_JPRB ,&
&  0.58481E+00_JPRB, 0.58192E+00_JPRB, 0.57925E+00_JPRB, 0.57400E+00_JPRB, 0.56991E+00_JPRB, 0.56603E+00_JPRB, 0.56236E+00_JPRB ,&
&  0.55880E+00_JPRB, 0.55500E+00_JPRB, 0.55066E+00_JPRB, 0.54409E+00_JPRB, 0.54097E+00_JPRB, 0.54372E+00_JPRB, 0.55076E+00_JPRB ,&
&  0.55755E+00_JPRB, 0.55777E+00_JPRB, 0.55464E+00_JPRB, 0.54407E+00_JPRB, 0.53156E+00_JPRB, 0.52141E+00_JPRB, 0.51933E+00_JPRB ,&
&  0.51047E+00_JPRB, 0.49641E+00_JPRB, 0.48283E+00_JPRB, 0.49016E+00_JPRB, 0.49386E+00_JPRB, 0.46692E+00_JPRB, 0.43320E+00_JPRB ,&
&  0.40239E+00_JPRB, 0.37683E+00_JPRB, 0.36490E+00_JPRB, 0.36338E+00_JPRB, 0.37513E+00_JPRB, 0.39759E+00_JPRB, 0.42880E+00_JPRB ,&
&  0.47529E+00_JPRB, 0.51969E+00_JPRB, 0.54035E+00_JPRB, 0.55665E+00_JPRB, 0.55937E+00_JPRB, 0.55678E+00_JPRB, 0.55429E+00_JPRB ,&
&  0.55737E+00_JPRB, 0.56439E+00_JPRB, 0.57607E+00_JPRB, 0.58653E+00_JPRB, 0.60459E+00_JPRB, 0.63912E+00_JPRB, 0.71021E+00_JPRB ,&
&  0.88061E+00_JPRB, 0.89795E+00_JPRB, 0.83116E+00_JPRB, 0.91282E+00_JPRB, 0.10446E+01_JPRB, 0.11566E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.12307E+01_JPRB, 0.11673E+01_JPRB, 0.11165E+01_JPRB, 0.10874E+01_JPRB, 0.10683E+01_JPRB, 0.10553E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.10852E+01_JPRB, 0.11784E+01_JPRB, 0.10419E+01_JPRB, 0.97435E+00_JPRB, 0.95335E+00_JPRB, 0.92740E+00_JPRB ,&
&  0.90260E+00_JPRB, 0.88942E+00_JPRB, 0.89261E+00_JPRB, 0.90442E+00_JPRB, 0.87238E+00_JPRB, 0.84376E+00_JPRB, 0.82579E+00_JPRB ,&
&  0.82635E+00_JPRB, 0.82011E+00_JPRB, 0.81621E+00_JPRB, 0.82332E+00_JPRB, 0.82085E+00_JPRB, 0.79644E+00_JPRB, 0.77995E+00_JPRB ,&
&  0.75432E+00_JPRB, 0.71229E+00_JPRB, 0.68228E+00_JPRB, 0.65346E+00_JPRB, 0.62335E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.60169E+00_JPRB, 0.61519E+00_JPRB, 0.63374E+00_JPRB, 0.64509E+00_JPRB, 0.64701E+00_JPRB, 0.61851E+00_JPRB, 0.58957E+00_JPRB ,&
&  0.57258E+00_JPRB, 0.56653E+00_JPRB, 0.56763E+00_JPRB, 0.56809E+00_JPRB, 0.56738E+00_JPRB, 0.56711E+00_JPRB, 0.56559E+00_JPRB ,&
&  0.55957E+00_JPRB, 0.55185E+00_JPRB, 0.54509E+00_JPRB, 0.53921E+00_JPRB, 0.53528E+00_JPRB, 0.53547E+00_JPRB, 0.53943E+00_JPRB ,&
&  0.54610E+00_JPRB, 0.55050E+00_JPRB, 0.55183E+00_JPRB, 0.54715E+00_JPRB, 0.54365E+00_JPRB, 0.54034E+00_JPRB, 0.52987E+00_JPRB ,&
&  0.51193E+00_JPRB, 0.50910E+00_JPRB, 0.50587E+00_JPRB, 0.50330E+00_JPRB, 0.50949E+00_JPRB, 0.49291E+00_JPRB, 0.45730E+00_JPRB ,&
&  0.42389E+00_JPRB, 0.40110E+00_JPRB, 0.39566E+00_JPRB, 0.40526E+00_JPRB, 0.43055E+00_JPRB, 0.46371E+00_JPRB, 0.48274E+00_JPRB ,&
&  0.50051E+00_JPRB, 0.51605E+00_JPRB, 0.52737E+00_JPRB, 0.53745E+00_JPRB, 0.54481E+00_JPRB, 0.54794E+00_JPRB, 0.54831E+00_JPRB ,&
&  0.54715E+00_JPRB, 0.54793E+00_JPRB, 0.55140E+00_JPRB, 0.53982E+00_JPRB, 0.55808E+00_JPRB, 0.65689E+00_JPRB, 0.76052E+00_JPRB ,&
&  0.87209E+00_JPRB, 0.90676E+00_JPRB, 0.94397E+00_JPRB, 0.10429E+01_JPRB, 0.11271E+01_JPRB, 0.11923E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.12433E+01_JPRB, 0.11772E+01_JPRB, 0.11197E+01_JPRB, 0.11011E+01_JPRB, 0.10945E+01_JPRB, 0.10884E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10250E+01_JPRB, 0.97845E+00_JPRB, 0.91415E+00_JPRB, 0.85771E+00_JPRB, 0.84911E+00_JPRB, 0.87182E+00_JPRB, 0.90782E+00_JPRB ,&
&  0.91281E+00_JPRB, 0.91088E+00_JPRB, 0.91115E+00_JPRB, 0.92605E+00_JPRB, 0.85070E+00_JPRB, 0.80265E+00_JPRB, 0.77283E+00_JPRB ,&
&  0.76794E+00_JPRB, 0.76819E+00_JPRB, 0.76847E+00_JPRB, 0.78799E+00_JPRB, 0.79222E+00_JPRB, 0.78234E+00_JPRB, 0.74778E+00_JPRB ,&
&  0.70081E+00_JPRB, 0.66235E+00_JPRB, 0.63390E+00_JPRB, 0.61316E+00_JPRB, 0.60424E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.66988E+00_JPRB, 0.67921E+00_JPRB, 0.68242E+00_JPRB, 0.68373E+00_JPRB, 0.67355E+00_JPRB, 0.65761E+00_JPRB, 0.64135E+00_JPRB ,&
&  0.63536E+00_JPRB, 0.62163E+00_JPRB, 0.60685E+00_JPRB, 0.58732E+00_JPRB, 0.57046E+00_JPRB, 0.55606E+00_JPRB, 0.55297E+00_JPRB ,&
&  0.55549E+00_JPRB, 0.55798E+00_JPRB, 0.55463E+00_JPRB, 0.54617E+00_JPRB, 0.53858E+00_JPRB, 0.53525E+00_JPRB, 0.53713E+00_JPRB ,&
&  0.54199E+00_JPRB, 0.54834E+00_JPRB, 0.55228E+00_JPRB, 0.55299E+00_JPRB, 0.55262E+00_JPRB, 0.55087E+00_JPRB, 0.54576E+00_JPRB ,&
&  0.53349E+00_JPRB, 0.52618E+00_JPRB, 0.52401E+00_JPRB, 0.52501E+00_JPRB, 0.53282E+00_JPRB, 0.51726E+00_JPRB, 0.48295E+00_JPRB ,&
&  0.44768E+00_JPRB, 0.43614E+00_JPRB, 0.44732E+00_JPRB, 0.48205E+00_JPRB, 0.50532E+00_JPRB, 0.52470E+00_JPRB, 0.53783E+00_JPRB ,&
&  0.53612E+00_JPRB, 0.53200E+00_JPRB, 0.53855E+00_JPRB, 0.55630E+00_JPRB, 0.55892E+00_JPRB, 0.55963E+00_JPRB, 0.55692E+00_JPRB ,&
&  0.55310E+00_JPRB, 0.55375E+00_JPRB, 0.55601E+00_JPRB, 0.54900E+00_JPRB, 0.63392E+00_JPRB, 0.79536E+00_JPRB, 0.85547E+00_JPRB ,&
&  0.95347E+00_JPRB, 0.10263E+01_JPRB, 0.11285E+01_JPRB, 0.11546E+01_JPRB, 0.11398E+01_JPRB, 0.11654E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12402E+01_JPRB, 0.12106E+01_JPRB, 0.11760E+01_JPRB, 0.11660E+01_JPRB, 0.11242E+01_JPRB, 0.10776E+01_JPRB, 0.10324E+01_JPRB ,&
&  0.10234E+01_JPRB, 0.10022E+01_JPRB, 0.95539E+00_JPRB, 0.89979E+00_JPRB, 0.87968E+00_JPRB, 0.87709E+00_JPRB, 0.89337E+00_JPRB ,&
&  0.90068E+00_JPRB, 0.91567E+00_JPRB, 0.92586E+00_JPRB, 0.93905E+00_JPRB, 0.87413E+00_JPRB, 0.79529E+00_JPRB, 0.75092E+00_JPRB ,&
&  0.73690E+00_JPRB, 0.73311E+00_JPRB, 0.72215E+00_JPRB, 0.73632E+00_JPRB, 0.75238E+00_JPRB, 0.74530E+00_JPRB, 0.70133E+00_JPRB ,&
&  0.66385E+00_JPRB, 0.64286E+00_JPRB, 0.63310E+00_JPRB, 0.64619E+00_JPRB, 0.66121E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.71763E+00_JPRB, 0.72858E+00_JPRB, 0.72417E+00_JPRB, 0.71632E+00_JPRB, 0.71281E+00_JPRB, 0.69925E+00_JPRB, 0.67238E+00_JPRB ,&
&  0.65845E+00_JPRB, 0.62835E+00_JPRB, 0.61074E+00_JPRB, 0.59792E+00_JPRB, 0.58798E+00_JPRB, 0.57355E+00_JPRB, 0.55214E+00_JPRB ,&
&  0.54437E+00_JPRB, 0.54834E+00_JPRB, 0.55732E+00_JPRB, 0.55733E+00_JPRB, 0.55297E+00_JPRB, 0.54761E+00_JPRB, 0.54560E+00_JPRB ,&
&  0.54773E+00_JPRB, 0.55340E+00_JPRB, 0.56129E+00_JPRB, 0.56192E+00_JPRB, 0.55837E+00_JPRB, 0.55553E+00_JPRB, 0.55713E+00_JPRB ,&
&  0.55122E+00_JPRB, 0.54260E+00_JPRB, 0.54553E+00_JPRB, 0.54683E+00_JPRB, 0.55616E+00_JPRB, 0.56159E+00_JPRB, 0.53994E+00_JPRB ,&
&  0.49519E+00_JPRB, 0.50040E+00_JPRB, 0.53973E+00_JPRB, 0.56230E+00_JPRB, 0.58102E+00_JPRB, 0.58480E+00_JPRB, 0.58681E+00_JPRB ,&
&  0.58768E+00_JPRB, 0.58984E+00_JPRB, 0.58932E+00_JPRB, 0.59008E+00_JPRB, 0.58937E+00_JPRB, 0.58744E+00_JPRB, 0.58432E+00_JPRB ,&
&  0.58460E+00_JPRB, 0.55277E+00_JPRB, 0.55112E+00_JPRB, 0.58312E+00_JPRB, 0.75276E+00_JPRB, 0.82647E+00_JPRB, 0.88996E+00_JPRB ,&
&  0.99628E+00_JPRB, 0.10717E+01_JPRB, 0.11649E+01_JPRB, 0.12217E+01_JPRB, 0.12301E+01_JPRB, 0.12311E+01_JPRB, 0.12871E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.13080E+01_JPRB, 0.12855E+01_JPRB, 0.12403E+01_JPRB, 0.11615E+01_JPRB, 0.11283E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.11161E+01_JPRB, 0.11149E+01_JPRB, 0.10665E+01_JPRB, 0.10047E+01_JPRB, 0.96087E+00_JPRB, 0.96267E+00_JPRB, 0.97123E+00_JPRB ,&
&  0.97453E+00_JPRB, 0.98976E+00_JPRB, 0.98346E+00_JPRB, 0.96725E+00_JPRB, 0.91915E+00_JPRB, 0.87896E+00_JPRB, 0.86833E+00_JPRB ,&
&  0.84760E+00_JPRB, 0.80815E+00_JPRB, 0.74626E+00_JPRB, 0.73220E+00_JPRB, 0.73937E+00_JPRB, 0.72793E+00_JPRB, 0.69856E+00_JPRB ,&
&  0.68537E+00_JPRB, 0.67439E+00_JPRB, 0.66877E+00_JPRB, 0.68557E+00_JPRB, 0.69859E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.72958E+00_JPRB, 0.73708E+00_JPRB, 0.73135E+00_JPRB, 0.72203E+00_JPRB, 0.73003E+00_JPRB, 0.71926E+00_JPRB, 0.71630E+00_JPRB ,&
&  0.72661E+00_JPRB, 0.71342E+00_JPRB, 0.73041E+00_JPRB, 0.65606E+00_JPRB, 0.60518E+00_JPRB, 0.58630E+00_JPRB, 0.56500E+00_JPRB ,&
&  0.55797E+00_JPRB, 0.56408E+00_JPRB, 0.56766E+00_JPRB, 0.56968E+00_JPRB, 0.57027E+00_JPRB, 0.56100E+00_JPRB, 0.55699E+00_JPRB ,&
&  0.55887E+00_JPRB, 0.56578E+00_JPRB, 0.57507E+00_JPRB, 0.57513E+00_JPRB, 0.57302E+00_JPRB, 0.57116E+00_JPRB, 0.56892E+00_JPRB ,&
&  0.56182E+00_JPRB, 0.56643E+00_JPRB, 0.57646E+00_JPRB, 0.57256E+00_JPRB, 0.57762E+00_JPRB, 0.59210E+00_JPRB, 0.59240E+00_JPRB ,&
&  0.60114E+00_JPRB, 0.58646E+00_JPRB, 0.61902E+00_JPRB, 0.63655E+00_JPRB, 0.64266E+00_JPRB, 0.63721E+00_JPRB, 0.63160E+00_JPRB ,&
&  0.63170E+00_JPRB, 0.63648E+00_JPRB, 0.64315E+00_JPRB, 0.64909E+00_JPRB, 0.64774E+00_JPRB, 0.65114E+00_JPRB, 0.65443E+00_JPRB ,&
&  0.66180E+00_JPRB, 0.60967E+00_JPRB, 0.68309E+00_JPRB, 0.72534E+00_JPRB, 0.84863E+00_JPRB, 0.99326E+00_JPRB, 0.10388E+01_JPRB ,&
&  0.12566E+01_JPRB, 0.12426E+01_JPRB, 0.12980E+01_JPRB, 0.13056E+01_JPRB, 0.13119E+01_JPRB, 0.13450E+01_JPRB, 0.14034E+01_JPRB ,&
&  0.14973E+01_JPRB, 0.14269E+01_JPRB, 0.14485E+01_JPRB, 0.13454E+01_JPRB, 0.12728E+01_JPRB, 0.12650E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.12611E+01_JPRB, 0.12406E+01_JPRB, 0.11154E+01_JPRB, 0.10868E+01_JPRB, 0.10777E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.10337E+01_JPRB, 0.98862E+00_JPRB, 0.10361E+01_JPRB, 0.95396E+00_JPRB, 0.90776E+00_JPRB, 0.91628E+00_JPRB ,&
&  0.96942E+00_JPRB, 0.88441E+00_JPRB, 0.81860E+00_JPRB, 0.78458E+00_JPRB, 0.75521E+00_JPRB, 0.72952E+00_JPRB, 0.72933E+00_JPRB ,&
&  0.70388E+00_JPRB, 0.70599E+00_JPRB, 0.73029E+00_JPRB, 0.72645E+00_JPRB, 0.72073E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.74211E+00_JPRB, 0.73627E+00_JPRB, 0.72835E+00_JPRB, 0.72405E+00_JPRB, 0.74941E+00_JPRB, 0.76035E+00_JPRB, 0.76134E+00_JPRB ,&
&  0.77851E+00_JPRB, 0.77182E+00_JPRB, 0.76697E+00_JPRB, 0.74947E+00_JPRB, 0.72805E+00_JPRB, 0.67363E+00_JPRB, 0.59166E+00_JPRB ,&
&  0.56444E+00_JPRB, 0.57015E+00_JPRB, 0.58556E+00_JPRB, 0.59121E+00_JPRB, 0.58348E+00_JPRB, 0.57295E+00_JPRB, 0.56838E+00_JPRB ,&
&  0.57355E+00_JPRB, 0.57945E+00_JPRB, 0.58882E+00_JPRB, 0.59368E+00_JPRB, 0.59307E+00_JPRB, 0.59371E+00_JPRB, 0.59492E+00_JPRB ,&
&  0.58798E+00_JPRB, 0.59690E+00_JPRB, 0.61213E+00_JPRB, 0.59525E+00_JPRB, 0.62375E+00_JPRB, 0.65298E+00_JPRB, 0.66025E+00_JPRB ,&
&  0.68586E+00_JPRB, 0.70156E+00_JPRB, 0.70238E+00_JPRB, 0.70744E+00_JPRB, 0.69680E+00_JPRB, 0.68644E+00_JPRB, 0.68605E+00_JPRB ,&
&  0.69378E+00_JPRB, 0.70212E+00_JPRB, 0.71092E+00_JPRB, 0.71802E+00_JPRB, 0.72081E+00_JPRB, 0.74256E+00_JPRB, 0.78802E+00_JPRB ,&
&  0.81191E+00_JPRB, 0.85755E+00_JPRB, 0.94797E+00_JPRB, 0.95780E+00_JPRB, 0.10551E+01_JPRB, 0.11488E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.12681E+01_JPRB, 0.14109E+01_JPRB, 0.15156E+01_JPRB, 0.16093E+01_JPRB, 0.16524E+01_JPRB, 0.17002E+01_JPRB ,&
&  0.18243E+01_JPRB, 0.18935E+01_JPRB, 0.16917E+01_JPRB, 0.15809E+01_JPRB, 0.14858E+01_JPRB, 0.14771E+01_JPRB, 0.14188E+01_JPRB ,&
&  0.14249E+01_JPRB, 0.14592E+01_JPRB, 0.14428E+01_JPRB, 0.14083E+01_JPRB, 0.13301E+01_JPRB, 0.12406E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.10050E+01_JPRB, 0.95543E+00_JPRB, 0.97860E+00_JPRB, 0.98380E+00_JPRB, 0.89797E+00_JPRB, 0.94369E+00_JPRB ,&
&  0.10434E+01_JPRB, 0.98644E+00_JPRB, 0.90685E+00_JPRB, 0.83687E+00_JPRB, 0.80634E+00_JPRB, 0.78041E+00_JPRB, 0.73330E+00_JPRB ,&
&  0.69783E+00_JPRB, 0.73475E+00_JPRB, 0.75407E+00_JPRB, 0.74928E+00_JPRB, 0.74322E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.75804E+00_JPRB, 0.74260E+00_JPRB, 0.72596E+00_JPRB, 0.73052E+00_JPRB, 0.76120E+00_JPRB, 0.77503E+00_JPRB, 0.79502E+00_JPRB ,&
&  0.80588E+00_JPRB, 0.80797E+00_JPRB, 0.79954E+00_JPRB, 0.76925E+00_JPRB, 0.75400E+00_JPRB, 0.75007E+00_JPRB, 0.70996E+00_JPRB ,&
&  0.64597E+00_JPRB, 0.63344E+00_JPRB, 0.61678E+00_JPRB, 0.61650E+00_JPRB, 0.61624E+00_JPRB, 0.60095E+00_JPRB, 0.59327E+00_JPRB ,&
&  0.60193E+00_JPRB, 0.60003E+00_JPRB, 0.60167E+00_JPRB, 0.60535E+00_JPRB, 0.60971E+00_JPRB, 0.61773E+00_JPRB, 0.62379E+00_JPRB ,&
&  0.61874E+00_JPRB, 0.63913E+00_JPRB, 0.66300E+00_JPRB, 0.65330E+00_JPRB, 0.70141E+00_JPRB, 0.75630E+00_JPRB, 0.78509E+00_JPRB ,&
&  0.80365E+00_JPRB, 0.79821E+00_JPRB, 0.78122E+00_JPRB, 0.76666E+00_JPRB, 0.74884E+00_JPRB, 0.74702E+00_JPRB, 0.75686E+00_JPRB ,&
&  0.77186E+00_JPRB, 0.78136E+00_JPRB, 0.78869E+00_JPRB, 0.80153E+00_JPRB, 0.80262E+00_JPRB, 0.86899E+00_JPRB, 0.98984E+00_JPRB ,&
&  0.11062E+01_JPRB, 0.12040E+01_JPRB, 0.12797E+01_JPRB, 0.13433E+01_JPRB, 0.14308E+01_JPRB, 0.14570E+01_JPRB, 0.14038E+01_JPRB ,&
&  0.14535E+01_JPRB, 0.14196E+01_JPRB, 0.18087E+01_JPRB, 0.18701E+01_JPRB, 0.18678E+01_JPRB, 0.19468E+01_JPRB, 0.19205E+01_JPRB ,&
&  0.19238E+01_JPRB, 0.18242E+01_JPRB, 0.17496E+01_JPRB, 0.16912E+01_JPRB, 0.16453E+01_JPRB, 0.17023E+01_JPRB, 0.16274E+01_JPRB ,&
&  0.16458E+01_JPRB, 0.16439E+01_JPRB, 0.14536E+01_JPRB, 0.12968E+01_JPRB, 0.11268E+01_JPRB, 0.11765E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.93081E+00_JPRB, 0.97935E+00_JPRB, 0.10297E+01_JPRB, 0.10556E+01_JPRB, 0.11121E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.12063E+01_JPRB, 0.10691E+01_JPRB, 0.95832E+00_JPRB, 0.89881E+00_JPRB, 0.85411E+00_JPRB, 0.76770E+00_JPRB ,&
&  0.74723E+00_JPRB, 0.77967E+00_JPRB, 0.79757E+00_JPRB, 0.79284E+00_JPRB, 0.77468E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.79109E+00_JPRB, 0.77180E+00_JPRB, 0.74597E+00_JPRB, 0.75501E+00_JPRB, 0.77911E+00_JPRB, 0.78776E+00_JPRB, 0.81053E+00_JPRB ,&
&  0.83475E+00_JPRB, 0.83551E+00_JPRB, 0.80309E+00_JPRB, 0.77728E+00_JPRB, 0.76548E+00_JPRB, 0.77046E+00_JPRB, 0.78905E+00_JPRB ,&
&  0.72093E+00_JPRB, 0.69026E+00_JPRB, 0.64019E+00_JPRB, 0.64246E+00_JPRB, 0.69361E+00_JPRB, 0.66023E+00_JPRB, 0.65473E+00_JPRB ,&
&  0.64308E+00_JPRB, 0.64167E+00_JPRB, 0.62984E+00_JPRB, 0.62329E+00_JPRB, 0.62372E+00_JPRB, 0.63567E+00_JPRB, 0.65902E+00_JPRB ,&
&  0.67321E+00_JPRB, 0.72691E+00_JPRB, 0.78115E+00_JPRB, 0.77953E+00_JPRB, 0.83337E+00_JPRB, 0.87873E+00_JPRB, 0.91815E+00_JPRB ,&
&  0.91657E+00_JPRB, 0.87599E+00_JPRB, 0.83509E+00_JPRB, 0.81892E+00_JPRB, 0.81440E+00_JPRB, 0.82843E+00_JPRB, 0.84885E+00_JPRB ,&
&  0.87279E+00_JPRB, 0.89434E+00_JPRB, 0.90811E+00_JPRB, 0.92414E+00_JPRB, 0.92809E+00_JPRB, 0.10224E+01_JPRB, 0.12187E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.16229E+01_JPRB, 0.17388E+01_JPRB, 0.20197E+01_JPRB, 0.21205E+01_JPRB, 0.20579E+01_JPRB, 0.18595E+01_JPRB ,&
&  0.17392E+01_JPRB, 0.16349E+01_JPRB, 0.17704E+01_JPRB, 0.19042E+01_JPRB, 0.18849E+01_JPRB, 0.20426E+01_JPRB, 0.19882E+01_JPRB ,&
&  0.18186E+01_JPRB, 0.16974E+01_JPRB, 0.16183E+01_JPRB, 0.16491E+01_JPRB, 0.16193E+01_JPRB, 0.16995E+01_JPRB, 0.16903E+01_JPRB ,&
&  0.17023E+01_JPRB, 0.15047E+01_JPRB, 0.11616E+01_JPRB, 0.10832E+01_JPRB, 0.87866E+00_JPRB, 0.89294E+00_JPRB, 0.10819E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10864E+01_JPRB, 0.11000E+01_JPRB, 0.11744E+01_JPRB, 0.12143E+01_JPRB, 0.14640E+01_JPRB, 0.15763E+01_JPRB ,&
&  0.14085E+01_JPRB, 0.14016E+01_JPRB, 0.13410E+01_JPRB, 0.12447E+01_JPRB, 0.11415E+01_JPRB, 0.10100E+01_JPRB, 0.90824E+00_JPRB ,&
&  0.87717E+00_JPRB, 0.85895E+00_JPRB, 0.85907E+00_JPRB, 0.83827E+00_JPRB, 0.80722E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.84638E+00_JPRB, 0.82851E+00_JPRB, 0.79010E+00_JPRB, 0.79221E+00_JPRB, 0.79975E+00_JPRB, 0.80943E+00_JPRB, 0.83671E+00_JPRB ,&
&  0.86236E+00_JPRB, 0.84250E+00_JPRB, 0.80159E+00_JPRB, 0.78603E+00_JPRB, 0.78461E+00_JPRB, 0.78956E+00_JPRB, 0.81307E+00_JPRB ,&
&  0.85548E+00_JPRB, 0.83239E+00_JPRB, 0.74028E+00_JPRB, 0.66381E+00_JPRB, 0.69264E+00_JPRB, 0.68826E+00_JPRB, 0.71536E+00_JPRB ,&
&  0.70223E+00_JPRB, 0.73204E+00_JPRB, 0.70715E+00_JPRB, 0.69562E+00_JPRB, 0.69017E+00_JPRB, 0.71265E+00_JPRB, 0.76379E+00_JPRB ,&
&  0.83506E+00_JPRB, 0.95453E+00_JPRB, 0.10633E+01_JPRB, 0.10673E+01_JPRB, 0.11040E+01_JPRB, 0.10346E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.10087E+01_JPRB, 0.95414E+00_JPRB, 0.90859E+00_JPRB, 0.89613E+00_JPRB, 0.90628E+00_JPRB, 0.92795E+00_JPRB, 0.96667E+00_JPRB ,&
&  0.10145E+01_JPRB, 0.10354E+01_JPRB, 0.10168E+01_JPRB, 0.10076E+01_JPRB, 0.10485E+01_JPRB, 0.11420E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.15076E+01_JPRB, 0.16700E+01_JPRB, 0.17280E+01_JPRB, 0.20891E+01_JPRB, 0.23298E+01_JPRB, 0.23639E+01_JPRB, 0.22530E+01_JPRB ,&
&  0.21195E+01_JPRB, 0.19886E+01_JPRB, 0.20581E+01_JPRB, 0.20954E+01_JPRB, 0.21090E+01_JPRB, 0.21176E+01_JPRB, 0.20048E+01_JPRB ,&
&  0.17755E+01_JPRB, 0.16427E+01_JPRB, 0.16134E+01_JPRB, 0.15737E+01_JPRB, 0.15665E+01_JPRB, 0.15007E+01_JPRB, 0.15391E+01_JPRB ,&
&  0.16216E+01_JPRB, 0.10066E+01_JPRB, 0.84350E+00_JPRB, 0.96541E+00_JPRB, 0.84094E+00_JPRB, 0.86072E+00_JPRB, 0.97007E+00_JPRB ,&
&  0.96461E+00_JPRB, 0.97770E+00_JPRB, 0.11267E+01_JPRB, 0.13044E+01_JPRB, 0.15074E+01_JPRB, 0.18713E+01_JPRB, 0.19001E+01_JPRB ,&
&  0.17701E+01_JPRB, 0.16090E+01_JPRB, 0.16384E+01_JPRB, 0.15956E+01_JPRB, 0.14606E+01_JPRB, 0.12528E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.10000E+01_JPRB, 0.96308E+00_JPRB, 0.94319E+00_JPRB, 0.90839E+00_JPRB, 0.86288E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.92623E+00_JPRB, 0.89936E+00_JPRB, 0.83267E+00_JPRB, 0.81774E+00_JPRB, 0.83082E+00_JPRB, 0.85887E+00_JPRB, 0.88768E+00_JPRB ,&
&  0.89056E+00_JPRB, 0.84980E+00_JPRB, 0.81920E+00_JPRB, 0.81067E+00_JPRB, 0.80650E+00_JPRB, 0.80109E+00_JPRB, 0.80460E+00_JPRB ,&
&  0.81741E+00_JPRB, 0.84103E+00_JPRB, 0.75210E+00_JPRB, 0.71743E+00_JPRB, 0.66931E+00_JPRB, 0.69283E+00_JPRB, 0.73043E+00_JPRB ,&
&  0.77760E+00_JPRB, 0.77444E+00_JPRB, 0.76115E+00_JPRB, 0.77101E+00_JPRB, 0.78365E+00_JPRB, 0.82473E+00_JPRB, 0.95645E+00_JPRB ,&
&  0.11154E+01_JPRB, 0.13066E+01_JPRB, 0.13460E+01_JPRB, 0.14064E+01_JPRB, 0.13046E+01_JPRB, 0.11620E+01_JPRB, 0.11615E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10299E+01_JPRB, 0.97678E+00_JPRB, 0.96744E+00_JPRB, 0.98043E+00_JPRB, 0.98973E+00_JPRB, 0.10260E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10676E+01_JPRB, 0.10577E+01_JPRB, 0.10400E+01_JPRB, 0.11022E+01_JPRB, 0.11741E+01_JPRB, 0.11669E+01_JPRB ,&
&  0.13533E+01_JPRB, 0.13178E+01_JPRB, 0.11622E+01_JPRB, 0.16459E+01_JPRB, 0.24258E+01_JPRB, 0.24783E+01_JPRB, 0.21571E+01_JPRB ,&
&  0.22266E+01_JPRB, 0.21964E+01_JPRB, 0.22958E+01_JPRB, 0.23999E+01_JPRB, 0.23622E+01_JPRB, 0.22192E+01_JPRB, 0.20372E+01_JPRB ,&
&  0.18247E+01_JPRB, 0.17115E+01_JPRB, 0.16800E+01_JPRB, 0.18329E+01_JPRB, 0.18207E+01_JPRB, 0.17824E+01_JPRB, 0.17126E+01_JPRB ,&
&  0.14738E+01_JPRB, 0.13062E+01_JPRB, 0.97436E+00_JPRB, 0.99307E+00_JPRB, 0.81208E+00_JPRB, 0.81896E+00_JPRB, 0.92897E+00_JPRB ,&
&  0.91402E+00_JPRB, 0.98422E+00_JPRB, 0.11540E+01_JPRB, 0.13640E+01_JPRB, 0.19163E+01_JPRB, 0.23240E+01_JPRB, 0.24493E+01_JPRB ,&
&  0.23307E+01_JPRB, 0.20189E+01_JPRB, 0.20556E+01_JPRB, 0.18969E+01_JPRB, 0.17680E+01_JPRB, 0.15648E+01_JPRB, 0.13380E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.11198E+01_JPRB, 0.10755E+01_JPRB, 0.10169E+01_JPRB, 0.95214E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.97508E+00_JPRB, 0.92030E+00_JPRB, 0.82926E+00_JPRB, 0.81644E+00_JPRB, 0.84041E+00_JPRB, 0.89919E+00_JPRB, 0.93120E+00_JPRB ,&
&  0.90833E+00_JPRB, 0.86218E+00_JPRB, 0.83488E+00_JPRB, 0.82411E+00_JPRB, 0.81009E+00_JPRB, 0.79153E+00_JPRB, 0.78523E+00_JPRB ,&
&  0.78448E+00_JPRB, 0.77917E+00_JPRB, 0.71649E+00_JPRB, 0.67808E+00_JPRB, 0.64453E+00_JPRB, 0.65866E+00_JPRB, 0.70184E+00_JPRB ,&
&  0.74348E+00_JPRB, 0.77926E+00_JPRB, 0.82838E+00_JPRB, 0.86724E+00_JPRB, 0.88680E+00_JPRB, 0.98827E+00_JPRB, 0.12082E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.15534E+01_JPRB, 0.16044E+01_JPRB, 0.15032E+01_JPRB, 0.13788E+01_JPRB, 0.12652E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.98885E+00_JPRB, 0.94884E+00_JPRB, 0.94053E+00_JPRB, 0.98118E+00_JPRB, 0.10277E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.11470E+01_JPRB, 0.11266E+01_JPRB, 0.10804E+01_JPRB, 0.10920E+01_JPRB, 0.11790E+01_JPRB, 0.12163E+01_JPRB, 0.12208E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.10950E+01_JPRB, 0.13776E+01_JPRB, 0.16619E+01_JPRB, 0.21542E+01_JPRB, 0.24813E+01_JPRB, 0.23910E+01_JPRB ,&
&  0.24147E+01_JPRB, 0.23588E+01_JPRB, 0.23718E+01_JPRB, 0.19793E+01_JPRB, 0.21974E+01_JPRB, 0.20996E+01_JPRB, 0.19437E+01_JPRB ,&
&  0.18060E+01_JPRB, 0.17536E+01_JPRB, 0.18042E+01_JPRB, 0.19545E+01_JPRB, 0.20716E+01_JPRB, 0.21447E+01_JPRB, 0.16580E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.14602E+01_JPRB, 0.13376E+01_JPRB, 0.11441E+01_JPRB, 0.99109E+00_JPRB, 0.10108E+01_JPRB, 0.10168E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10693E+01_JPRB, 0.12764E+01_JPRB, 0.17211E+01_JPRB, 0.24893E+01_JPRB, 0.30200E+01_JPRB, 0.30197E+01_JPRB ,&
&  0.26663E+01_JPRB, 0.27184E+01_JPRB, 0.25065E+01_JPRB, 0.22009E+01_JPRB, 0.19774E+01_JPRB, 0.17905E+01_JPRB, 0.16028E+01_JPRB ,&
&  0.14252E+01_JPRB, 0.13134E+01_JPRB, 0.12294E+01_JPRB, 0.11319E+01_JPRB, 0.10286E+01_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.10149E+01_JPRB, 0.95506E+00_JPRB, 0.86655E+00_JPRB, 0.85478E+00_JPRB, 0.90030E+00_JPRB, 0.96013E+00_JPRB, 0.96805E+00_JPRB ,&
&  0.90169E+00_JPRB, 0.85853E+00_JPRB, 0.83115E+00_JPRB, 0.82891E+00_JPRB, 0.81937E+00_JPRB, 0.79525E+00_JPRB, 0.77578E+00_JPRB ,&
&  0.78571E+00_JPRB, 0.74993E+00_JPRB, 0.69016E+00_JPRB, 0.69929E+00_JPRB, 0.69811E+00_JPRB, 0.65264E+00_JPRB, 0.67767E+00_JPRB ,&
&  0.72326E+00_JPRB, 0.78407E+00_JPRB, 0.84614E+00_JPRB, 0.94504E+00_JPRB, 0.10804E+01_JPRB, 0.12863E+01_JPRB, 0.15516E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.16930E+01_JPRB, 0.17047E+01_JPRB, 0.15718E+01_JPRB, 0.13449E+01_JPRB, 0.11860E+01_JPRB, 0.11004E+01_JPRB ,&
&  0.10160E+01_JPRB, 0.94870E+00_JPRB, 0.94439E+00_JPRB, 0.99129E+00_JPRB, 0.10759E+01_JPRB, 0.11399E+01_JPRB, 0.11956E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11615E+01_JPRB, 0.11664E+01_JPRB, 0.12730E+01_JPRB, 0.10510E+01_JPRB, 0.10400E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.12759E+01_JPRB, 0.13109E+01_JPRB, 0.13675E+01_JPRB, 0.17192E+01_JPRB, 0.16529E+01_JPRB, 0.19312E+01_JPRB, 0.23299E+01_JPRB ,&
&  0.25353E+01_JPRB, 0.25592E+01_JPRB, 0.25557E+01_JPRB, 0.16590E+01_JPRB, 0.14753E+01_JPRB, 0.19050E+01_JPRB, 0.19159E+01_JPRB ,&
&  0.18413E+01_JPRB, 0.19258E+01_JPRB, 0.20138E+01_JPRB, 0.22560E+01_JPRB, 0.17733E+01_JPRB, 0.15404E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.11432E+01_JPRB, 0.13348E+01_JPRB, 0.12372E+01_JPRB, 0.12906E+01_JPRB, 0.12433E+01_JPRB, 0.11715E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.14048E+01_JPRB, 0.17558E+01_JPRB, 0.24986E+01_JPRB, 0.34076E+01_JPRB, 0.32806E+01_JPRB, 0.36028E+01_JPRB ,&
&  0.35541E+01_JPRB, 0.32040E+01_JPRB, 0.26509E+01_JPRB, 0.24055E+01_JPRB, 0.21976E+01_JPRB, 0.20237E+01_JPRB, 0.18372E+01_JPRB ,&
&  0.16444E+01_JPRB, 0.14909E+01_JPRB, 0.13501E+01_JPRB, 0.12102E+01_JPRB, 0.10874E+01_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.10995E+01_JPRB, 0.10363E+01_JPRB, 0.92881E+00_JPRB, 0.92497E+00_JPRB, 0.97870E+00_JPRB, 0.10040E+01_JPRB, 0.95504E+00_JPRB ,&
&  0.89996E+00_JPRB, 0.85297E+00_JPRB, 0.83696E+00_JPRB, 0.82871E+00_JPRB, 0.81512E+00_JPRB, 0.80808E+00_JPRB, 0.79276E+00_JPRB ,&
&  0.82110E+00_JPRB, 0.81709E+00_JPRB, 0.73976E+00_JPRB, 0.69805E+00_JPRB, 0.72412E+00_JPRB, 0.67747E+00_JPRB, 0.65319E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.83493E+00_JPRB, 0.95659E+00_JPRB, 0.11291E+01_JPRB, 0.13337E+01_JPRB, 0.16955E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.18613E+01_JPRB, 0.18249E+01_JPRB, 0.16061E+01_JPRB, 0.13781E+01_JPRB, 0.11803E+01_JPRB, 0.10679E+01_JPRB, 0.99332E+00_JPRB ,&
&  0.94210E+00_JPRB, 0.92071E+00_JPRB, 0.96100E+00_JPRB, 0.10393E+01_JPRB, 0.11243E+01_JPRB, 0.11609E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.12068E+01_JPRB, 0.12905E+01_JPRB, 0.13184E+01_JPRB, 0.12717E+01_JPRB, 0.11926E+01_JPRB, 0.10589E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.12589E+01_JPRB, 0.12229E+01_JPRB, 0.13296E+01_JPRB, 0.16249E+01_JPRB, 0.18834E+01_JPRB, 0.19399E+01_JPRB, 0.23423E+01_JPRB ,&
&  0.19687E+01_JPRB, 0.16425E+01_JPRB, 0.13909E+01_JPRB, 0.11019E+01_JPRB, 0.14066E+01_JPRB, 0.19621E+01_JPRB, 0.20785E+01_JPRB ,&
&  0.20854E+01_JPRB, 0.21740E+01_JPRB, 0.23082E+01_JPRB, 0.20830E+01_JPRB, 0.15984E+01_JPRB, 0.13452E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.17361E+01_JPRB, 0.18448E+01_JPRB, 0.17955E+01_JPRB, 0.15653E+01_JPRB, 0.14033E+01_JPRB, 0.15989E+01_JPRB, 0.16179E+01_JPRB ,&
&  0.18013E+01_JPRB, 0.20127E+01_JPRB, 0.30537E+01_JPRB, 0.40605E+01_JPRB, 0.43123E+01_JPRB, 0.44041E+01_JPRB, 0.46106E+01_JPRB ,&
&  0.40476E+01_JPRB, 0.36285E+01_JPRB, 0.30941E+01_JPRB, 0.26558E+01_JPRB, 0.23609E+01_JPRB, 0.21603E+01_JPRB, 0.20028E+01_JPRB ,&
&  0.18161E+01_JPRB, 0.16761E+01_JPRB, 0.15166E+01_JPRB, 0.13219E+01_JPRB, 0.11765E+01_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.12088E+01_JPRB, 0.11167E+01_JPRB, 0.99340E+00_JPRB, 0.98609E+00_JPRB, 0.10367E+01_JPRB, 0.10127E+01_JPRB, 0.93669E+00_JPRB ,&
&  0.90727E+00_JPRB, 0.86411E+00_JPRB, 0.84909E+00_JPRB, 0.83248E+00_JPRB, 0.81410E+00_JPRB, 0.80356E+00_JPRB, 0.81513E+00_JPRB ,&
&  0.87278E+00_JPRB, 0.95889E+00_JPRB, 0.86583E+00_JPRB, 0.79238E+00_JPRB, 0.76496E+00_JPRB, 0.74031E+00_JPRB, 0.66619E+00_JPRB ,&
&  0.79180E+00_JPRB, 0.96243E+00_JPRB, 0.11232E+01_JPRB, 0.13495E+01_JPRB, 0.17017E+01_JPRB, 0.18066E+01_JPRB, 0.18513E+01_JPRB ,&
&  0.18144E+01_JPRB, 0.16584E+01_JPRB, 0.14236E+01_JPRB, 0.12173E+01_JPRB, 0.10829E+01_JPRB, 0.10135E+01_JPRB, 0.95786E+00_JPRB ,&
&  0.93546E+00_JPRB, 0.97017E+00_JPRB, 0.10259E+01_JPRB, 0.10907E+01_JPRB, 0.11812E+01_JPRB, 0.12419E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.14214E+01_JPRB, 0.14396E+01_JPRB, 0.14058E+01_JPRB, 0.13902E+01_JPRB, 0.12519E+01_JPRB, 0.15269E+01_JPRB ,&
&  0.14661E+01_JPRB, 0.15023E+01_JPRB, 0.14842E+01_JPRB, 0.18502E+01_JPRB, 0.18794E+01_JPRB, 0.18693E+01_JPRB, 0.21025E+01_JPRB ,&
&  0.16436E+01_JPRB, 0.15049E+01_JPRB, 0.16156E+01_JPRB, 0.16934E+01_JPRB, 0.16299E+01_JPRB, 0.17778E+01_JPRB, 0.24941E+01_JPRB ,&
&  0.20728E+01_JPRB, 0.23127E+01_JPRB, 0.24895E+01_JPRB, 0.24558E+01_JPRB, 0.12120E+01_JPRB, 0.99922E+00_JPRB, 0.15987E+01_JPRB ,&
&  0.17902E+01_JPRB, 0.12588E+01_JPRB, 0.98323E+00_JPRB, 0.12119E+01_JPRB, 0.99017E+00_JPRB, 0.12267E+01_JPRB, 0.18653E+01_JPRB ,&
&  0.23641E+01_JPRB, 0.30593E+01_JPRB, 0.49858E+01_JPRB, 0.52585E+01_JPRB, 0.48358E+01_JPRB, 0.54844E+01_JPRB, 0.46860E+01_JPRB ,&
&  0.41761E+01_JPRB, 0.32578E+01_JPRB, 0.30640E+01_JPRB, 0.27186E+01_JPRB, 0.24568E+01_JPRB, 0.22949E+01_JPRB, 0.21465E+01_JPRB ,&
&  0.19719E+01_JPRB, 0.18094E+01_JPRB, 0.16372E+01_JPRB, 0.14580E+01_JPRB, 0.13174E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.12807E+01_JPRB, 0.11736E+01_JPRB, 0.10634E+01_JPRB, 0.10399E+01_JPRB, 0.10652E+01_JPRB, 0.98838E+00_JPRB, 0.93272E+00_JPRB ,&
&  0.88903E+00_JPRB, 0.86663E+00_JPRB, 0.84645E+00_JPRB, 0.82148E+00_JPRB, 0.80139E+00_JPRB, 0.80988E+00_JPRB, 0.83956E+00_JPRB ,&
&  0.89472E+00_JPRB, 0.10063E+01_JPRB, 0.11084E+01_JPRB, 0.10322E+01_JPRB, 0.97927E+00_JPRB, 0.91051E+00_JPRB, 0.84936E+00_JPRB ,&
&  0.97605E+00_JPRB, 0.12169E+01_JPRB, 0.13688E+01_JPRB, 0.14804E+01_JPRB, 0.15984E+01_JPRB, 0.15942E+01_JPRB, 0.16681E+01_JPRB ,&
&  0.16351E+01_JPRB, 0.14328E+01_JPRB, 0.12706E+01_JPRB, 0.11352E+01_JPRB, 0.10542E+01_JPRB, 0.99725E+00_JPRB, 0.98180E+00_JPRB ,&
&  0.99777E+00_JPRB, 0.10248E+01_JPRB, 0.10506E+01_JPRB, 0.11472E+01_JPRB, 0.12764E+01_JPRB, 0.13449E+01_JPRB, 0.14074E+01_JPRB ,&
&  0.14633E+01_JPRB, 0.14862E+01_JPRB, 0.14920E+01_JPRB, 0.15081E+01_JPRB, 0.15992E+01_JPRB, 0.14834E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.12579E+01_JPRB, 0.14021E+01_JPRB, 0.19119E+01_JPRB, 0.19862E+01_JPRB, 0.18312E+01_JPRB, 0.19040E+01_JPRB, 0.18966E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.19980E+01_JPRB, 0.19698E+01_JPRB, 0.21160E+01_JPRB, 0.21160E+01_JPRB, 0.17071E+01_JPRB, 0.22312E+01_JPRB ,&
&  0.23807E+01_JPRB, 0.23264E+01_JPRB, 0.18187E+01_JPRB, 0.13381E+01_JPRB, 0.14122E+01_JPRB, 0.11768E+01_JPRB, 0.88137E+00_JPRB ,&
&  0.84259E+00_JPRB, 0.81190E+00_JPRB, 0.75555E+00_JPRB, 0.79134E+00_JPRB, 0.81370E+00_JPRB, 0.12090E+01_JPRB, 0.19686E+01_JPRB ,&
&  0.33829E+01_JPRB, 0.48773E+01_JPRB, 0.58628E+01_JPRB, 0.57118E+01_JPRB, 0.51222E+01_JPRB, 0.55138E+01_JPRB, 0.47750E+01_JPRB ,&
&  0.34769E+01_JPRB, 0.29485E+01_JPRB, 0.27781E+01_JPRB, 0.26628E+01_JPRB, 0.25653E+01_JPRB, 0.24618E+01_JPRB, 0.22917E+01_JPRB ,&
&  0.20819E+01_JPRB, 0.19181E+01_JPRB, 0.17595E+01_JPRB, 0.15645E+01_JPRB, 0.14129E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.13334E+01_JPRB, 0.12265E+01_JPRB, 0.11614E+01_JPRB, 0.11132E+01_JPRB, 0.10814E+01_JPRB, 0.97269E+00_JPRB, 0.92833E+00_JPRB ,&
&  0.87107E+00_JPRB, 0.85489E+00_JPRB, 0.82111E+00_JPRB, 0.80328E+00_JPRB, 0.79435E+00_JPRB, 0.82337E+00_JPRB, 0.87918E+00_JPRB ,&
&  0.95162E+00_JPRB, 0.11183E+01_JPRB, 0.13560E+01_JPRB, 0.14344E+01_JPRB, 0.15275E+01_JPRB, 0.14692E+01_JPRB, 0.13885E+01_JPRB ,&
&  0.13761E+01_JPRB, 0.16343E+01_JPRB, 0.16350E+01_JPRB, 0.16005E+01_JPRB, 0.16231E+01_JPRB, 0.15911E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.14458E+01_JPRB, 0.12901E+01_JPRB, 0.11615E+01_JPRB, 0.10787E+01_JPRB, 0.10327E+01_JPRB, 0.10091E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.10460E+01_JPRB, 0.10419E+01_JPRB, 0.10900E+01_JPRB, 0.12242E+01_JPRB, 0.13467E+01_JPRB, 0.14054E+01_JPRB, 0.14837E+01_JPRB ,&
&  0.15513E+01_JPRB, 0.15548E+01_JPRB, 0.15405E+01_JPRB, 0.15097E+01_JPRB, 0.13162E+01_JPRB, 0.11511E+01_JPRB, 0.13654E+01_JPRB ,&
&  0.15540E+01_JPRB, 0.18677E+01_JPRB, 0.20644E+01_JPRB, 0.22928E+01_JPRB, 0.19342E+01_JPRB, 0.18497E+01_JPRB, 0.18624E+01_JPRB ,&
&  0.19312E+01_JPRB, 0.25522E+01_JPRB, 0.22036E+01_JPRB, 0.20563E+01_JPRB, 0.23860E+01_JPRB, 0.19676E+01_JPRB, 0.20070E+01_JPRB ,&
&  0.23708E+01_JPRB, 0.22679E+01_JPRB, 0.18597E+01_JPRB, 0.15111E+01_JPRB, 0.20234E+01_JPRB, 0.22575E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.93065E+00_JPRB, 0.87372E+00_JPRB, 0.83097E+00_JPRB, 0.80064E+00_JPRB, 0.84506E+00_JPRB, 0.13412E+01_JPRB, 0.27656E+01_JPRB ,&
&  0.41349E+01_JPRB, 0.61036E+01_JPRB, 0.63753E+01_JPRB, 0.60429E+01_JPRB, 0.52460E+01_JPRB, 0.50332E+01_JPRB, 0.40362E+01_JPRB ,&
&  0.34383E+01_JPRB, 0.30397E+01_JPRB, 0.28149E+01_JPRB, 0.27008E+01_JPRB, 0.25938E+01_JPRB, 0.24854E+01_JPRB, 0.23371E+01_JPRB ,&
&  0.21494E+01_JPRB, 0.19861E+01_JPRB, 0.18251E+01_JPRB, 0.16227E+01_JPRB, 0.14592E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.13584E+01_JPRB, 0.12804E+01_JPRB, 0.12448E+01_JPRB, 0.12015E+01_JPRB, 0.11089E+01_JPRB, 0.99635E+00_JPRB, 0.90627E+00_JPRB ,&
&  0.86006E+00_JPRB, 0.83284E+00_JPRB, 0.81086E+00_JPRB, 0.80427E+00_JPRB, 0.81293E+00_JPRB, 0.85340E+00_JPRB, 0.91511E+00_JPRB ,&
&  0.10392E+01_JPRB, 0.12434E+01_JPRB, 0.14634E+01_JPRB, 0.18568E+01_JPRB, 0.19978E+01_JPRB, 0.18545E+01_JPRB, 0.20496E+01_JPRB ,&
&  0.20678E+01_JPRB, 0.21213E+01_JPRB, 0.19724E+01_JPRB, 0.17821E+01_JPRB, 0.16569E+01_JPRB, 0.15797E+01_JPRB, 0.14742E+01_JPRB ,&
&  0.13334E+01_JPRB, 0.12125E+01_JPRB, 0.11106E+01_JPRB, 0.10806E+01_JPRB, 0.10597E+01_JPRB, 0.10421E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10345E+01_JPRB, 0.11330E+01_JPRB, 0.12717E+01_JPRB, 0.13762E+01_JPRB, 0.14669E+01_JPRB, 0.15716E+01_JPRB ,&
&  0.16098E+01_JPRB, 0.16079E+01_JPRB, 0.15769E+01_JPRB, 0.15327E+01_JPRB, 0.12855E+01_JPRB, 0.14513E+01_JPRB, 0.15141E+01_JPRB ,&
&  0.16608E+01_JPRB, 0.18437E+01_JPRB, 0.19118E+01_JPRB, 0.21191E+01_JPRB, 0.19614E+01_JPRB, 0.18409E+01_JPRB, 0.17864E+01_JPRB ,&
&  0.20904E+01_JPRB, 0.21139E+01_JPRB, 0.18791E+01_JPRB, 0.19810E+01_JPRB, 0.23236E+01_JPRB, 0.27432E+01_JPRB, 0.18688E+01_JPRB ,&
&  0.16988E+01_JPRB, 0.21692E+01_JPRB, 0.19365E+01_JPRB, 0.17829E+01_JPRB, 0.22766E+01_JPRB, 0.32256E+01_JPRB, 0.23706E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.94449E+00_JPRB, 0.98986E+00_JPRB, 0.10770E+01_JPRB, 0.10961E+01_JPRB, 0.17767E+01_JPRB, 0.55813E+01_JPRB ,&
&  0.59394E+01_JPRB, 0.71558E+01_JPRB, 0.65305E+01_JPRB, 0.57002E+01_JPRB, 0.51733E+01_JPRB, 0.46987E+01_JPRB, 0.39697E+01_JPRB ,&
&  0.34218E+01_JPRB, 0.30887E+01_JPRB, 0.28301E+01_JPRB, 0.26429E+01_JPRB, 0.24579E+01_JPRB, 0.22744E+01_JPRB, 0.21135E+01_JPRB ,&
&  0.20028E+01_JPRB, 0.18834E+01_JPRB, 0.17674E+01_JPRB, 0.16068E+01_JPRB, 0.14632E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.13424E+01_JPRB, 0.12854E+01_JPRB, 0.12603E+01_JPRB, 0.12119E+01_JPRB, 0.11285E+01_JPRB, 0.10514E+01_JPRB, 0.94337E+00_JPRB ,&
&  0.86945E+00_JPRB, 0.82767E+00_JPRB, 0.81140E+00_JPRB, 0.81805E+00_JPRB, 0.84759E+00_JPRB, 0.89693E+00_JPRB, 0.95307E+00_JPRB ,&
&  0.11327E+01_JPRB, 0.13083E+01_JPRB, 0.15899E+01_JPRB, 0.19827E+01_JPRB, 0.22047E+01_JPRB, 0.20384E+01_JPRB, 0.22432E+01_JPRB ,&
&  0.27612E+01_JPRB, 0.25336E+01_JPRB, 0.21546E+01_JPRB, 0.19575E+01_JPRB, 0.18116E+01_JPRB, 0.17324E+01_JPRB, 0.15073E+01_JPRB ,&
&  0.13628E+01_JPRB, 0.12907E+01_JPRB, 0.11987E+01_JPRB, 0.11663E+01_JPRB, 0.11191E+01_JPRB, 0.11198E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.10767E+01_JPRB, 0.11849E+01_JPRB, 0.12988E+01_JPRB, 0.14050E+01_JPRB, 0.15071E+01_JPRB, 0.16277E+01_JPRB ,&
&  0.16864E+01_JPRB, 0.16705E+01_JPRB, 0.16140E+01_JPRB, 0.15027E+01_JPRB, 0.14874E+01_JPRB, 0.15171E+01_JPRB, 0.16252E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.18000E+01_JPRB, 0.18649E+01_JPRB, 0.19097E+01_JPRB, 0.18478E+01_JPRB, 0.17992E+01_JPRB, 0.17526E+01_JPRB ,&
&  0.19348E+01_JPRB, 0.19062E+01_JPRB, 0.16986E+01_JPRB, 0.19857E+01_JPRB, 0.24010E+01_JPRB, 0.27002E+01_JPRB, 0.25300E+01_JPRB ,&
&  0.22375E+01_JPRB, 0.19436E+01_JPRB, 0.19256E+01_JPRB, 0.22840E+01_JPRB, 0.26699E+01_JPRB, 0.33165E+01_JPRB, 0.44481E+01_JPRB ,&
&  0.47392E+01_JPRB, 0.27315E+01_JPRB, 0.22737E+01_JPRB, 0.31116E+01_JPRB, 0.27117E+01_JPRB, 0.28800E+01_JPRB, 0.46448E+01_JPRB ,&
&  0.60770E+01_JPRB, 0.69175E+01_JPRB, 0.64614E+01_JPRB, 0.55111E+01_JPRB, 0.50168E+01_JPRB, 0.43236E+01_JPRB, 0.37680E+01_JPRB ,&
&  0.32284E+01_JPRB, 0.27965E+01_JPRB, 0.24915E+01_JPRB, 0.22584E+01_JPRB, 0.20448E+01_JPRB, 0.18837E+01_JPRB, 0.18145E+01_JPRB ,&
&  0.17686E+01_JPRB, 0.17061E+01_JPRB, 0.16389E+01_JPRB, 0.15197E+01_JPRB, 0.14113E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.12989E+01_JPRB, 0.12614E+01_JPRB, 0.12343E+01_JPRB, 0.11874E+01_JPRB, 0.11686E+01_JPRB, 0.11403E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.95794E+00_JPRB, 0.84287E+00_JPRB, 0.83351E+00_JPRB, 0.85805E+00_JPRB, 0.89877E+00_JPRB, 0.95405E+00_JPRB, 0.10613E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.14059E+01_JPRB, 0.17597E+01_JPRB, 0.21936E+01_JPRB, 0.26102E+01_JPRB, 0.33167E+01_JPRB, 0.23248E+01_JPRB ,&
&  0.23746E+01_JPRB, 0.23777E+01_JPRB, 0.21091E+01_JPRB, 0.20392E+01_JPRB, 0.19244E+01_JPRB, 0.18812E+01_JPRB, 0.15932E+01_JPRB ,&
&  0.14389E+01_JPRB, 0.13727E+01_JPRB, 0.12857E+01_JPRB, 0.12033E+01_JPRB, 0.11568E+01_JPRB, 0.11305E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.11364E+01_JPRB, 0.12315E+01_JPRB, 0.13300E+01_JPRB, 0.14374E+01_JPRB, 0.15565E+01_JPRB, 0.16427E+01_JPRB ,&
&  0.17286E+01_JPRB, 0.17367E+01_JPRB, 0.16824E+01_JPRB, 0.15506E+01_JPRB, 0.15561E+01_JPRB, 0.15730E+01_JPRB, 0.15961E+01_JPRB ,&
&  0.15202E+01_JPRB, 0.14702E+01_JPRB, 0.16585E+01_JPRB, 0.17859E+01_JPRB, 0.17506E+01_JPRB, 0.16445E+01_JPRB, 0.16036E+01_JPRB ,&
&  0.18277E+01_JPRB, 0.19779E+01_JPRB, 0.21438E+01_JPRB, 0.22297E+01_JPRB, 0.25183E+01_JPRB, 0.27320E+01_JPRB, 0.28702E+01_JPRB ,&
&  0.29556E+01_JPRB, 0.27444E+01_JPRB, 0.27626E+01_JPRB, 0.30074E+01_JPRB, 0.31617E+01_JPRB, 0.36425E+01_JPRB, 0.43035E+01_JPRB ,&
&  0.52207E+01_JPRB, 0.65032E+01_JPRB, 0.67313E+01_JPRB, 0.54995E+01_JPRB, 0.49680E+01_JPRB, 0.44128E+01_JPRB, 0.50266E+01_JPRB ,&
&  0.60948E+01_JPRB, 0.58886E+01_JPRB, 0.55377E+01_JPRB, 0.55918E+01_JPRB, 0.42429E+01_JPRB, 0.35933E+01_JPRB, 0.31536E+01_JPRB ,&
&  0.28314E+01_JPRB, 0.24462E+01_JPRB, 0.21392E+01_JPRB, 0.19024E+01_JPRB, 0.16835E+01_JPRB, 0.15868E+01_JPRB, 0.15847E+01_JPRB ,&
&  0.15670E+01_JPRB, 0.15442E+01_JPRB, 0.15028E+01_JPRB, 0.14230E+01_JPRB, 0.13549E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.12551E+01_JPRB, 0.12126E+01_JPRB, 0.11871E+01_JPRB, 0.11613E+01_JPRB, 0.11955E+01_JPRB, 0.11974E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.89966E+00_JPRB, 0.88106E+00_JPRB, 0.90791E+00_JPRB, 0.96032E+00_JPRB, 0.10353E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.13017E+01_JPRB, 0.15009E+01_JPRB, 0.18903E+01_JPRB, 0.23781E+01_JPRB, 0.28121E+01_JPRB, 0.31527E+01_JPRB, 0.31673E+01_JPRB ,&
&  0.20036E+01_JPRB, 0.24293E+01_JPRB, 0.20932E+01_JPRB, 0.21541E+01_JPRB, 0.21513E+01_JPRB, 0.20752E+01_JPRB, 0.17183E+01_JPRB ,&
&  0.14930E+01_JPRB, 0.14140E+01_JPRB, 0.13561E+01_JPRB, 0.13251E+01_JPRB, 0.12388E+01_JPRB, 0.11517E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.12434E+01_JPRB, 0.13409E+01_JPRB, 0.14290E+01_JPRB, 0.15387E+01_JPRB, 0.16422E+01_JPRB, 0.17118E+01_JPRB ,&
&  0.17903E+01_JPRB, 0.18037E+01_JPRB, 0.17020E+01_JPRB, 0.16504E+01_JPRB, 0.16264E+01_JPRB, 0.16421E+01_JPRB, 0.16324E+01_JPRB ,&
&  0.15425E+01_JPRB, 0.14723E+01_JPRB, 0.15812E+01_JPRB, 0.15895E+01_JPRB, 0.15310E+01_JPRB, 0.15910E+01_JPRB, 0.16189E+01_JPRB ,&
&  0.19336E+01_JPRB, 0.20727E+01_JPRB, 0.26561E+01_JPRB, 0.24414E+01_JPRB, 0.24919E+01_JPRB, 0.27545E+01_JPRB, 0.27424E+01_JPRB ,&
&  0.25693E+01_JPRB, 0.28001E+01_JPRB, 0.29762E+01_JPRB, 0.34884E+01_JPRB, 0.39442E+01_JPRB, 0.44451E+01_JPRB, 0.46756E+01_JPRB ,&
&  0.51622E+01_JPRB, 0.57755E+01_JPRB, 0.64872E+01_JPRB, 0.52933E+01_JPRB, 0.48699E+01_JPRB, 0.47055E+01_JPRB, 0.52528E+01_JPRB ,&
&  0.56921E+01_JPRB, 0.53566E+01_JPRB, 0.52628E+01_JPRB, 0.46645E+01_JPRB, 0.35039E+01_JPRB, 0.30354E+01_JPRB, 0.26554E+01_JPRB ,&
&  0.23645E+01_JPRB, 0.20578E+01_JPRB, 0.18084E+01_JPRB, 0.16031E+01_JPRB, 0.14784E+01_JPRB, 0.14537E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.14611E+01_JPRB, 0.14480E+01_JPRB, 0.14234E+01_JPRB, 0.13611E+01_JPRB, 0.13028E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.12305E+01_JPRB, 0.12076E+01_JPRB, 0.11940E+01_JPRB, 0.11525E+01_JPRB, 0.11909E+01_JPRB, 0.12915E+01_JPRB, 0.13644E+01_JPRB ,&
&  0.13136E+01_JPRB, 0.94054E+00_JPRB, 0.93098E+00_JPRB, 0.95973E+00_JPRB, 0.10100E+01_JPRB, 0.11103E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.16204E+01_JPRB, 0.20434E+01_JPRB, 0.25424E+01_JPRB, 0.29540E+01_JPRB, 0.34410E+01_JPRB, 0.35685E+01_JPRB ,&
&  0.24942E+01_JPRB, 0.20063E+01_JPRB, 0.20457E+01_JPRB, 0.21523E+01_JPRB, 0.20780E+01_JPRB, 0.21552E+01_JPRB, 0.21195E+01_JPRB ,&
&  0.18928E+01_JPRB, 0.16385E+01_JPRB, 0.16214E+01_JPRB, 0.15748E+01_JPRB, 0.12743E+01_JPRB, 0.12432E+01_JPRB, 0.12507E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.13818E+01_JPRB, 0.14701E+01_JPRB, 0.15512E+01_JPRB, 0.16586E+01_JPRB, 0.17375E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.18534E+01_JPRB, 0.18799E+01_JPRB, 0.18077E+01_JPRB, 0.17143E+01_JPRB, 0.16893E+01_JPRB, 0.16682E+01_JPRB, 0.16204E+01_JPRB ,&
&  0.15797E+01_JPRB, 0.16111E+01_JPRB, 0.16299E+01_JPRB, 0.15806E+01_JPRB, 0.15287E+01_JPRB, 0.17186E+01_JPRB, 0.18651E+01_JPRB ,&
&  0.20965E+01_JPRB, 0.23867E+01_JPRB, 0.30422E+01_JPRB, 0.24551E+01_JPRB, 0.24757E+01_JPRB, 0.26548E+01_JPRB, 0.27099E+01_JPRB ,&
&  0.27044E+01_JPRB, 0.29942E+01_JPRB, 0.33945E+01_JPRB, 0.38933E+01_JPRB, 0.44285E+01_JPRB, 0.45420E+01_JPRB, 0.56315E+01_JPRB ,&
&  0.53327E+01_JPRB, 0.61455E+01_JPRB, 0.55825E+01_JPRB, 0.53426E+01_JPRB, 0.45546E+01_JPRB, 0.45822E+01_JPRB, 0.45863E+01_JPRB ,&
&  0.47870E+01_JPRB, 0.44753E+01_JPRB, 0.40107E+01_JPRB, 0.35236E+01_JPRB, 0.27735E+01_JPRB, 0.24556E+01_JPRB, 0.21389E+01_JPRB ,&
&  0.18561E+01_JPRB, 0.16582E+01_JPRB, 0.15649E+01_JPRB, 0.14935E+01_JPRB, 0.14659E+01_JPRB, 0.14560E+01_JPRB, 0.14497E+01_JPRB ,&
&  0.14295E+01_JPRB, 0.14064E+01_JPRB, 0.13768E+01_JPRB, 0.13170E+01_JPRB, 0.12671E+01_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.12077E+01_JPRB, 0.11818E+01_JPRB, 0.11475E+01_JPRB, 0.10663E+01_JPRB, 0.10183E+01_JPRB, 0.10201E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.99090E+00_JPRB, 0.96337E+00_JPRB, 0.97982E+00_JPRB, 0.10256E+01_JPRB, 0.10874E+01_JPRB, 0.11913E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.17193E+01_JPRB, 0.20423E+01_JPRB, 0.23963E+01_JPRB, 0.27042E+01_JPRB, 0.29493E+01_JPRB, 0.30852E+01_JPRB ,&
&  0.24940E+01_JPRB, 0.17699E+01_JPRB, 0.21527E+01_JPRB, 0.23125E+01_JPRB, 0.22889E+01_JPRB, 0.24497E+01_JPRB, 0.23848E+01_JPRB ,&
&  0.19897E+01_JPRB, 0.18557E+01_JPRB, 0.18810E+01_JPRB, 0.17056E+01_JPRB, 0.13539E+01_JPRB, 0.13617E+01_JPRB, 0.14047E+01_JPRB ,&
&  0.14715E+01_JPRB, 0.15270E+01_JPRB, 0.15993E+01_JPRB, 0.16810E+01_JPRB, 0.17602E+01_JPRB, 0.18662E+01_JPRB, 0.19396E+01_JPRB ,&
&  0.19641E+01_JPRB, 0.19668E+01_JPRB, 0.19391E+01_JPRB, 0.18330E+01_JPRB, 0.17536E+01_JPRB, 0.17084E+01_JPRB, 0.16576E+01_JPRB ,&
&  0.16475E+01_JPRB, 0.16358E+01_JPRB, 0.16952E+01_JPRB, 0.18140E+01_JPRB, 0.20054E+01_JPRB, 0.19643E+01_JPRB, 0.21818E+01_JPRB ,&
&  0.23498E+01_JPRB, 0.27287E+01_JPRB, 0.28506E+01_JPRB, 0.31867E+01_JPRB, 0.21127E+01_JPRB, 0.23992E+01_JPRB, 0.23647E+01_JPRB ,&
&  0.30218E+01_JPRB, 0.32779E+01_JPRB, 0.37799E+01_JPRB, 0.43696E+01_JPRB, 0.51647E+01_JPRB, 0.52632E+01_JPRB, 0.56379E+01_JPRB ,&
&  0.54806E+01_JPRB, 0.52079E+01_JPRB, 0.46814E+01_JPRB, 0.46564E+01_JPRB, 0.44250E+01_JPRB, 0.47690E+01_JPRB, 0.41009E+01_JPRB ,&
&  0.40130E+01_JPRB, 0.36408E+01_JPRB, 0.32779E+01_JPRB, 0.29510E+01_JPRB, 0.23137E+01_JPRB, 0.19095E+01_JPRB, 0.16602E+01_JPRB ,&
&  0.15438E+01_JPRB, 0.14929E+01_JPRB, 0.14540E+01_JPRB, 0.14329E+01_JPRB, 0.14359E+01_JPRB, 0.14390E+01_JPRB, 0.14345E+01_JPRB ,&
&  0.14152E+01_JPRB, 0.13846E+01_JPRB, 0.13470E+01_JPRB, 0.12936E+01_JPRB, 0.12491E+01_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.11705E+01_JPRB, 0.11366E+01_JPRB, 0.10869E+01_JPRB, 0.10253E+01_JPRB, 0.99488E+00_JPRB, 0.97635E+00_JPRB, 0.97546E+00_JPRB ,&
&  0.97995E+00_JPRB, 0.10048E+01_JPRB, 0.10266E+01_JPRB, 0.10642E+01_JPRB, 0.11145E+01_JPRB, 0.12047E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.15618E+01_JPRB, 0.18470E+01_JPRB, 0.22017E+01_JPRB, 0.24982E+01_JPRB, 0.26635E+01_JPRB, 0.25389E+01_JPRB ,&
&  0.24249E+01_JPRB, 0.23817E+01_JPRB, 0.22215E+01_JPRB, 0.18095E+01_JPRB, 0.17998E+01_JPRB, 0.22568E+01_JPRB, 0.20320E+01_JPRB ,&
&  0.17708E+01_JPRB, 0.16970E+01_JPRB, 0.16247E+01_JPRB, 0.15439E+01_JPRB, 0.14685E+01_JPRB, 0.14791E+01_JPRB, 0.15231E+01_JPRB ,&
&  0.15781E+01_JPRB, 0.16361E+01_JPRB, 0.17182E+01_JPRB, 0.18023E+01_JPRB, 0.18922E+01_JPRB, 0.20220E+01_JPRB, 0.20964E+01_JPRB ,&
&  0.20975E+01_JPRB, 0.20847E+01_JPRB, 0.20422E+01_JPRB, 0.19499E+01_JPRB, 0.18303E+01_JPRB, 0.18095E+01_JPRB, 0.18257E+01_JPRB ,&
&  0.18530E+01_JPRB, 0.18918E+01_JPRB, 0.19517E+01_JPRB, 0.21315E+01_JPRB, 0.22961E+01_JPRB, 0.21695E+01_JPRB, 0.22143E+01_JPRB ,&
&  0.25300E+01_JPRB, 0.29154E+01_JPRB, 0.26715E+01_JPRB, 0.31426E+01_JPRB, 0.19093E+01_JPRB, 0.23202E+01_JPRB, 0.28879E+01_JPRB ,&
&  0.31227E+01_JPRB, 0.35652E+01_JPRB, 0.41381E+01_JPRB, 0.48191E+01_JPRB, 0.55495E+01_JPRB, 0.52750E+01_JPRB, 0.52528E+01_JPRB ,&
&  0.47491E+01_JPRB, 0.40854E+01_JPRB, 0.39299E+01_JPRB, 0.38416E+01_JPRB, 0.40399E+01_JPRB, 0.41078E+01_JPRB, 0.31971E+01_JPRB ,&
&  0.26995E+01_JPRB, 0.29623E+01_JPRB, 0.29382E+01_JPRB, 0.27109E+01_JPRB, 0.19646E+01_JPRB, 0.15336E+01_JPRB, 0.14366E+01_JPRB ,&
&  0.14282E+01_JPRB, 0.13986E+01_JPRB, 0.13907E+01_JPRB, 0.13987E+01_JPRB, 0.14155E+01_JPRB, 0.14199E+01_JPRB, 0.14153E+01_JPRB ,&
&  0.14002E+01_JPRB, 0.13625E+01_JPRB, 0.13200E+01_JPRB, 0.12680E+01_JPRB, 0.12169E+01_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.11215E+01_JPRB, 0.10847E+01_JPRB, 0.10303E+01_JPRB, 0.98020E+00_JPRB, 0.92048E+00_JPRB, 0.89179E+00_JPRB, 0.89078E+00_JPRB ,&
&  0.91562E+00_JPRB, 0.95147E+00_JPRB, 0.97714E+00_JPRB, 0.98621E+00_JPRB, 0.10188E+01_JPRB, 0.10396E+01_JPRB, 0.10736E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.12739E+01_JPRB, 0.15120E+01_JPRB, 0.18904E+01_JPRB, 0.22616E+01_JPRB, 0.26117E+01_JPRB, 0.25041E+01_JPRB ,&
&  0.23807E+01_JPRB, 0.22943E+01_JPRB, 0.22496E+01_JPRB, 0.22312E+01_JPRB, 0.19923E+01_JPRB, 0.21873E+01_JPRB, 0.21816E+01_JPRB ,&
&  0.21691E+01_JPRB, 0.19954E+01_JPRB, 0.17009E+01_JPRB, 0.16220E+01_JPRB, 0.15806E+01_JPRB, 0.15874E+01_JPRB, 0.16302E+01_JPRB ,&
&  0.16839E+01_JPRB, 0.17400E+01_JPRB, 0.18330E+01_JPRB, 0.19491E+01_JPRB, 0.20884E+01_JPRB, 0.22183E+01_JPRB, 0.22727E+01_JPRB ,&
&  0.22611E+01_JPRB, 0.22446E+01_JPRB, 0.22108E+01_JPRB, 0.21059E+01_JPRB, 0.20661E+01_JPRB, 0.20982E+01_JPRB, 0.21591E+01_JPRB ,&
&  0.22423E+01_JPRB, 0.22045E+01_JPRB, 0.23013E+01_JPRB, 0.25127E+01_JPRB, 0.25582E+01_JPRB, 0.24897E+01_JPRB, 0.25188E+01_JPRB ,&
&  0.27398E+01_JPRB, 0.30656E+01_JPRB, 0.22418E+01_JPRB, 0.27176E+01_JPRB, 0.32846E+01_JPRB, 0.29306E+01_JPRB, 0.30022E+01_JPRB ,&
&  0.34180E+01_JPRB, 0.38244E+01_JPRB, 0.43548E+01_JPRB, 0.49380E+01_JPRB, 0.56053E+01_JPRB, 0.61439E+01_JPRB, 0.43556E+01_JPRB ,&
&  0.38699E+01_JPRB, 0.32850E+01_JPRB, 0.31718E+01_JPRB, 0.32205E+01_JPRB, 0.33484E+01_JPRB, 0.31975E+01_JPRB, 0.24512E+01_JPRB ,&
&  0.22242E+01_JPRB, 0.24160E+01_JPRB, 0.23193E+01_JPRB, 0.22676E+01_JPRB, 0.20339E+01_JPRB, 0.13400E+01_JPRB, 0.13591E+01_JPRB ,&
&  0.13616E+01_JPRB, 0.13465E+01_JPRB, 0.13487E+01_JPRB, 0.13592E+01_JPRB, 0.13729E+01_JPRB, 0.13766E+01_JPRB, 0.13727E+01_JPRB ,&
&  0.13481E+01_JPRB, 0.13046E+01_JPRB, 0.12574E+01_JPRB, 0.12081E+01_JPRB, 0.11621E+01_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.10795E+01_JPRB, 0.10331E+01_JPRB, 0.97252E+00_JPRB, 0.90670E+00_JPRB, 0.84736E+00_JPRB, 0.82466E+00_JPRB, 0.81632E+00_JPRB ,&
&  0.82609E+00_JPRB, 0.85731E+00_JPRB, 0.87690E+00_JPRB, 0.87557E+00_JPRB, 0.87144E+00_JPRB, 0.88668E+00_JPRB, 0.93865E+00_JPRB ,&
&  0.10051E+01_JPRB, 0.10888E+01_JPRB, 0.12581E+01_JPRB, 0.16206E+01_JPRB, 0.20878E+01_JPRB, 0.24828E+01_JPRB, 0.24840E+01_JPRB ,&
&  0.23623E+01_JPRB, 0.23751E+01_JPRB, 0.23916E+01_JPRB, 0.24514E+01_JPRB, 0.25100E+01_JPRB, 0.26325E+01_JPRB, 0.27763E+01_JPRB ,&
&  0.25119E+01_JPRB, 0.20125E+01_JPRB, 0.17932E+01_JPRB, 0.16585E+01_JPRB, 0.16451E+01_JPRB, 0.16775E+01_JPRB, 0.17209E+01_JPRB ,&
&  0.17741E+01_JPRB, 0.18571E+01_JPRB, 0.19742E+01_JPRB, 0.21518E+01_JPRB, 0.23355E+01_JPRB, 0.24423E+01_JPRB, 0.24899E+01_JPRB ,&
&  0.25119E+01_JPRB, 0.25254E+01_JPRB, 0.25760E+01_JPRB, 0.23586E+01_JPRB, 0.24473E+01_JPRB, 0.25273E+01_JPRB, 0.25970E+01_JPRB ,&
&  0.25872E+01_JPRB, 0.25152E+01_JPRB, 0.26951E+01_JPRB, 0.28189E+01_JPRB, 0.29430E+01_JPRB, 0.29470E+01_JPRB, 0.29132E+01_JPRB ,&
&  0.30450E+01_JPRB, 0.28471E+01_JPRB, 0.20374E+01_JPRB, 0.20799E+01_JPRB, 0.23210E+01_JPRB, 0.22950E+01_JPRB, 0.30894E+01_JPRB ,&
&  0.33450E+01_JPRB, 0.36080E+01_JPRB, 0.39066E+01_JPRB, 0.42028E+01_JPRB, 0.45208E+01_JPRB, 0.46980E+01_JPRB, 0.36491E+01_JPRB ,&
&  0.29898E+01_JPRB, 0.26626E+01_JPRB, 0.25830E+01_JPRB, 0.25188E+01_JPRB, 0.24542E+01_JPRB, 0.22546E+01_JPRB, 0.21293E+01_JPRB ,&
&  0.20709E+01_JPRB, 0.20144E+01_JPRB, 0.18943E+01_JPRB, 0.16837E+01_JPRB, 0.14581E+01_JPRB, 0.12773E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.13126E+01_JPRB, 0.12928E+01_JPRB, 0.12954E+01_JPRB, 0.12968E+01_JPRB, 0.13030E+01_JPRB, 0.12989E+01_JPRB, 0.12838E+01_JPRB ,&
&  0.12536E+01_JPRB, 0.12192E+01_JPRB, 0.11848E+01_JPRB, 0.11467E+01_JPRB, 0.11096E+01_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.10373E+01_JPRB, 0.98020E+00_JPRB, 0.90958E+00_JPRB, 0.85108E+00_JPRB, 0.84180E+00_JPRB, 0.84534E+00_JPRB, 0.82978E+00_JPRB ,&
&  0.82400E+00_JPRB, 0.82095E+00_JPRB, 0.84284E+00_JPRB, 0.84808E+00_JPRB, 0.85362E+00_JPRB, 0.87229E+00_JPRB, 0.89712E+00_JPRB ,&
&  0.91707E+00_JPRB, 0.93355E+00_JPRB, 0.10167E+01_JPRB, 0.12002E+01_JPRB, 0.15514E+01_JPRB, 0.20235E+01_JPRB, 0.23874E+01_JPRB ,&
&  0.26112E+01_JPRB, 0.28010E+01_JPRB, 0.29849E+01_JPRB, 0.31563E+01_JPRB, 0.30459E+01_JPRB, 0.32104E+01_JPRB, 0.36201E+01_JPRB ,&
&  0.26951E+01_JPRB, 0.18310E+01_JPRB, 0.18250E+01_JPRB, 0.16408E+01_JPRB, 0.16480E+01_JPRB, 0.17241E+01_JPRB, 0.17865E+01_JPRB ,&
&  0.18697E+01_JPRB, 0.19733E+01_JPRB, 0.21007E+01_JPRB, 0.22757E+01_JPRB, 0.24502E+01_JPRB, 0.25713E+01_JPRB, 0.26589E+01_JPRB ,&
&  0.27315E+01_JPRB, 0.28174E+01_JPRB, 0.29486E+01_JPRB, 0.29261E+01_JPRB, 0.28844E+01_JPRB, 0.29996E+01_JPRB, 0.30581E+01_JPRB ,&
&  0.31288E+01_JPRB, 0.31430E+01_JPRB, 0.28145E+01_JPRB, 0.29027E+01_JPRB, 0.32321E+01_JPRB, 0.30915E+01_JPRB, 0.30875E+01_JPRB ,&
&  0.32995E+01_JPRB, 0.28810E+01_JPRB, 0.19143E+01_JPRB, 0.19091E+01_JPRB, 0.21893E+01_JPRB, 0.25678E+01_JPRB, 0.29927E+01_JPRB ,&
&  0.32538E+01_JPRB, 0.34282E+01_JPRB, 0.34592E+01_JPRB, 0.34960E+01_JPRB, 0.33376E+01_JPRB, 0.31165E+01_JPRB, 0.28533E+01_JPRB ,&
&  0.24255E+01_JPRB, 0.22869E+01_JPRB, 0.21669E+01_JPRB, 0.21123E+01_JPRB, 0.21236E+01_JPRB, 0.20278E+01_JPRB, 0.19126E+01_JPRB ,&
&  0.20583E+01_JPRB, 0.21597E+01_JPRB, 0.23761E+01_JPRB, 0.18337E+01_JPRB, 0.13707E+01_JPRB, 0.12580E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.12743E+01_JPRB, 0.12672E+01_JPRB, 0.12585E+01_JPRB, 0.12524E+01_JPRB, 0.12304E+01_JPRB, 0.11957E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.11339E+01_JPRB, 0.11187E+01_JPRB, 0.10971E+01_JPRB, 0.10652E+01_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.96650E+00_JPRB, 0.88746E+00_JPRB, 0.84372E+00_JPRB, 0.88568E+00_JPRB, 0.85877E+00_JPRB, 0.88647E+00_JPRB, 0.87858E+00_JPRB ,&
&  0.88273E+00_JPRB, 0.88904E+00_JPRB, 0.91352E+00_JPRB, 0.94687E+00_JPRB, 0.98260E+00_JPRB, 0.10112E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.10418E+01_JPRB, 0.10550E+01_JPRB, 0.10747E+01_JPRB, 0.11160E+01_JPRB, 0.12007E+01_JPRB, 0.13505E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.19431E+01_JPRB, 0.21271E+01_JPRB, 0.22179E+01_JPRB, 0.22633E+01_JPRB, 0.25532E+01_JPRB, 0.31917E+01_JPRB, 0.37126E+01_JPRB ,&
&  0.22256E+01_JPRB, 0.19113E+01_JPRB, 0.16525E+01_JPRB, 0.14903E+01_JPRB, 0.15377E+01_JPRB, 0.16891E+01_JPRB, 0.17653E+01_JPRB ,&
&  0.18163E+01_JPRB, 0.19101E+01_JPRB, 0.20299E+01_JPRB, 0.21750E+01_JPRB, 0.22980E+01_JPRB, 0.24142E+01_JPRB, 0.25410E+01_JPRB ,&
&  0.26839E+01_JPRB, 0.28762E+01_JPRB, 0.30787E+01_JPRB, 0.32888E+01_JPRB, 0.32892E+01_JPRB, 0.36551E+01_JPRB, 0.37234E+01_JPRB ,&
&  0.37635E+01_JPRB, 0.35994E+01_JPRB, 0.31735E+01_JPRB, 0.32361E+01_JPRB, 0.34366E+01_JPRB, 0.34191E+01_JPRB, 0.32288E+01_JPRB ,&
&  0.32794E+01_JPRB, 0.27655E+01_JPRB, 0.21253E+01_JPRB, 0.22013E+01_JPRB, 0.23380E+01_JPRB, 0.25865E+01_JPRB, 0.26114E+01_JPRB ,&
&  0.28132E+01_JPRB, 0.28435E+01_JPRB, 0.27751E+01_JPRB, 0.26228E+01_JPRB, 0.24422E+01_JPRB, 0.23467E+01_JPRB, 0.23152E+01_JPRB ,&
&  0.21145E+01_JPRB, 0.20391E+01_JPRB, 0.19787E+01_JPRB, 0.20408E+01_JPRB, 0.21550E+01_JPRB, 0.18198E+01_JPRB, 0.18326E+01_JPRB ,&
&  0.19476E+01_JPRB, 0.20859E+01_JPRB, 0.18204E+01_JPRB, 0.19914E+01_JPRB, 0.12873E+01_JPRB, 0.11592E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.12195E+01_JPRB, 0.11920E+01_JPRB, 0.11903E+01_JPRB, 0.11801E+01_JPRB, 0.11809E+01_JPRB, 0.11638E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.10478E+01_JPRB, 0.10466E+01_JPRB, 0.10361E+01_JPRB, 0.10185E+01_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.87749E+00_JPRB, 0.83092E+00_JPRB, 0.88437E+00_JPRB, 0.85746E+00_JPRB, 0.86052E+00_JPRB, 0.88231E+00_JPRB, 0.90481E+00_JPRB ,&
&  0.91920E+00_JPRB, 0.94437E+00_JPRB, 0.98527E+00_JPRB, 0.10428E+01_JPRB, 0.10958E+01_JPRB, 0.11371E+01_JPRB, 0.11838E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.12431E+01_JPRB, 0.12576E+01_JPRB, 0.12711E+01_JPRB, 0.13079E+01_JPRB, 0.13523E+01_JPRB, 0.14271E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.15722E+01_JPRB, 0.15884E+01_JPRB, 0.16535E+01_JPRB, 0.17319E+01_JPRB, 0.19420E+01_JPRB, 0.24423E+01_JPRB ,&
&  0.18990E+01_JPRB, 0.18555E+01_JPRB, 0.16814E+01_JPRB, 0.15738E+01_JPRB, 0.16013E+01_JPRB, 0.15537E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.16914E+01_JPRB, 0.17127E+01_JPRB, 0.17720E+01_JPRB, 0.18194E+01_JPRB, 0.18870E+01_JPRB, 0.19855E+01_JPRB, 0.20897E+01_JPRB ,&
&  0.22336E+01_JPRB, 0.24104E+01_JPRB, 0.25514E+01_JPRB, 0.27789E+01_JPRB, 0.30051E+01_JPRB, 0.32381E+01_JPRB, 0.33441E+01_JPRB ,&
&  0.34103E+01_JPRB, 0.34410E+01_JPRB, 0.32096E+01_JPRB, 0.32357E+01_JPRB, 0.32375E+01_JPRB, 0.30946E+01_JPRB, 0.27698E+01_JPRB ,&
&  0.24796E+01_JPRB, 0.24007E+01_JPRB, 0.23941E+01_JPRB, 0.23254E+01_JPRB, 0.23383E+01_JPRB, 0.23112E+01_JPRB, 0.23190E+01_JPRB ,&
&  0.23732E+01_JPRB, 0.22653E+01_JPRB, 0.21311E+01_JPRB, 0.19485E+01_JPRB, 0.18635E+01_JPRB, 0.18463E+01_JPRB, 0.18786E+01_JPRB ,&
&  0.18305E+01_JPRB, 0.17911E+01_JPRB, 0.18612E+01_JPRB, 0.21053E+01_JPRB, 0.24275E+01_JPRB, 0.31564E+01_JPRB, 0.23718E+01_JPRB ,&
&  0.17433E+01_JPRB, 0.16077E+01_JPRB, 0.10990E+01_JPRB, 0.13928E+01_JPRB, 0.10748E+01_JPRB, 0.10198E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.10832E+01_JPRB, 0.11311E+01_JPRB, 0.10983E+01_JPRB, 0.10675E+01_JPRB, 0.10524E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.95292E+00_JPRB, 0.89403E+00_JPRB, 0.90617E+00_JPRB, 0.85564E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.74468E+00_JPRB, 0.77039E+00_JPRB, 0.87689E+00_JPRB, 0.82388E+00_JPRB, 0.83026E+00_JPRB, 0.85681E+00_JPRB, 0.89444E+00_JPRB ,&
&  0.93535E+00_JPRB, 0.98589E+00_JPRB, 0.10396E+01_JPRB, 0.11106E+01_JPRB, 0.11740E+01_JPRB, 0.12290E+01_JPRB, 0.12814E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.13719E+01_JPRB, 0.14132E+01_JPRB, 0.14387E+01_JPRB, 0.14538E+01_JPRB, 0.14939E+01_JPRB, 0.15447E+01_JPRB ,&
&  0.16468E+01_JPRB, 0.17018E+01_JPRB, 0.17668E+01_JPRB, 0.18703E+01_JPRB, 0.19564E+01_JPRB, 0.20969E+01_JPRB, 0.19335E+01_JPRB ,&
&  0.21698E+01_JPRB, 0.18245E+01_JPRB, 0.16969E+01_JPRB, 0.16517E+01_JPRB, 0.15867E+01_JPRB, 0.15305E+01_JPRB, 0.15174E+01_JPRB ,&
&  0.15614E+01_JPRB, 0.15022E+01_JPRB, 0.15098E+01_JPRB, 0.14868E+01_JPRB, 0.15185E+01_JPRB, 0.16091E+01_JPRB, 0.16675E+01_JPRB ,&
&  0.17954E+01_JPRB, 0.19553E+01_JPRB, 0.21230E+01_JPRB, 0.22485E+01_JPRB, 0.23634E+01_JPRB, 0.23668E+01_JPRB, 0.23609E+01_JPRB ,&
&  0.24132E+01_JPRB, 0.25360E+01_JPRB, 0.24918E+01_JPRB, 0.25482E+01_JPRB, 0.25327E+01_JPRB, 0.23337E+01_JPRB, 0.18804E+01_JPRB ,&
&  0.16948E+01_JPRB, 0.19660E+01_JPRB, 0.20231E+01_JPRB, 0.24657E+01_JPRB, 0.22620E+01_JPRB, 0.20544E+01_JPRB, 0.20555E+01_JPRB ,&
&  0.19739E+01_JPRB, 0.18066E+01_JPRB, 0.16746E+01_JPRB, 0.14816E+01_JPRB, 0.14500E+01_JPRB, 0.14627E+01_JPRB, 0.15092E+01_JPRB ,&
&  0.14765E+01_JPRB, 0.14623E+01_JPRB, 0.15150E+01_JPRB, 0.17884E+01_JPRB, 0.25922E+01_JPRB, 0.26857E+01_JPRB, 0.28896E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.99969E+00_JPRB, 0.10597E+01_JPRB, 0.10111E+01_JPRB, 0.91458E+00_JPRB, 0.85202E+00_JPRB, 0.91074E+00_JPRB ,&
&  0.99064E+00_JPRB, 0.99335E+00_JPRB, 0.97145E+00_JPRB, 0.97931E+00_JPRB, 0.97785E+00_JPRB, 0.98391E+00_JPRB, 0.90727E+00_JPRB ,&
&  0.88793E+00_JPRB, 0.82907E+00_JPRB, 0.79338E+00_JPRB, 0.77652E+00_JPRB, 0.73511E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.68739E+00_JPRB, 0.76371E+00_JPRB, 0.84406E+00_JPRB, 0.79223E+00_JPRB, 0.81539E+00_JPRB, 0.85322E+00_JPRB, 0.90303E+00_JPRB ,&
&  0.95795E+00_JPRB, 0.10200E+01_JPRB, 0.10839E+01_JPRB, 0.11573E+01_JPRB, 0.12177E+01_JPRB, 0.12749E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.14312E+01_JPRB, 0.14860E+01_JPRB, 0.15156E+01_JPRB, 0.15539E+01_JPRB, 0.15817E+01_JPRB, 0.16187E+01_JPRB ,&
&  0.17283E+01_JPRB, 0.18116E+01_JPRB, 0.19081E+01_JPRB, 0.20106E+01_JPRB, 0.20801E+01_JPRB, 0.23311E+01_JPRB, 0.19613E+01_JPRB ,&
&  0.21855E+01_JPRB, 0.18219E+01_JPRB, 0.16800E+01_JPRB, 0.16771E+01_JPRB, 0.16217E+01_JPRB, 0.15935E+01_JPRB, 0.15807E+01_JPRB ,&
&  0.15740E+01_JPRB, 0.15823E+01_JPRB, 0.15665E+01_JPRB, 0.15290E+01_JPRB, 0.13769E+01_JPRB, 0.14271E+01_JPRB, 0.14874E+01_JPRB ,&
&  0.15706E+01_JPRB, 0.16779E+01_JPRB, 0.17588E+01_JPRB, 0.18104E+01_JPRB, 0.18384E+01_JPRB, 0.18046E+01_JPRB, 0.17673E+01_JPRB ,&
&  0.17243E+01_JPRB, 0.17407E+01_JPRB, 0.16463E+01_JPRB, 0.17244E+01_JPRB, 0.18719E+01_JPRB, 0.16057E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.14931E+01_JPRB, 0.19826E+01_JPRB, 0.23925E+01_JPRB, 0.20830E+01_JPRB, 0.19037E+01_JPRB, 0.18086E+01_JPRB ,&
&  0.16555E+01_JPRB, 0.14753E+01_JPRB, 0.13249E+01_JPRB, 0.11731E+01_JPRB, 0.11189E+01_JPRB, 0.11077E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.11234E+01_JPRB, 0.11734E+01_JPRB, 0.12889E+01_JPRB, 0.16730E+01_JPRB, 0.19628E+01_JPRB, 0.34717E+01_JPRB ,&
&  0.17666E+01_JPRB, 0.12097E+01_JPRB, 0.18318E+01_JPRB, 0.64048E+00_JPRB, 0.70828E+00_JPRB, 0.73536E+00_JPRB, 0.77109E+00_JPRB ,&
&  0.72052E+00_JPRB, 0.91085E+00_JPRB, 0.12562E+01_JPRB, 0.12684E+01_JPRB, 0.13212E+01_JPRB, 0.13952E+01_JPRB, 0.87885E+00_JPRB ,&
&  0.73919E+00_JPRB, 0.73617E+00_JPRB, 0.74371E+00_JPRB, 0.69132E+00_JPRB, 0.68082E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.64043E+00_JPRB, 0.70307E+00_JPRB, 0.78451E+00_JPRB, 0.84951E+00_JPRB, 0.84992E+00_JPRB, 0.86803E+00_JPRB, 0.90263E+00_JPRB ,&
&  0.94121E+00_JPRB, 0.98593E+00_JPRB, 0.10296E+01_JPRB, 0.10846E+01_JPRB, 0.11251E+01_JPRB, 0.11770E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.14022E+01_JPRB, 0.14573E+01_JPRB, 0.15200E+01_JPRB, 0.15793E+01_JPRB, 0.16365E+01_JPRB, 0.17001E+01_JPRB ,&
&  0.17472E+01_JPRB, 0.18257E+01_JPRB, 0.19025E+01_JPRB, 0.20673E+01_JPRB, 0.22229E+01_JPRB, 0.26496E+01_JPRB, 0.19612E+01_JPRB ,&
&  0.20969E+01_JPRB, 0.17930E+01_JPRB, 0.16736E+01_JPRB, 0.16909E+01_JPRB, 0.16616E+01_JPRB, 0.16292E+01_JPRB, 0.16317E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.17533E+01_JPRB, 0.17185E+01_JPRB, 0.17204E+01_JPRB, 0.15757E+01_JPRB, 0.15340E+01_JPRB, 0.16643E+01_JPRB ,&
&  0.17846E+01_JPRB, 0.18896E+01_JPRB, 0.19267E+01_JPRB, 0.19871E+01_JPRB, 0.19918E+01_JPRB, 0.19522E+01_JPRB, 0.19005E+01_JPRB ,&
&  0.18362E+01_JPRB, 0.18136E+01_JPRB, 0.18355E+01_JPRB, 0.19282E+01_JPRB, 0.19670E+01_JPRB, 0.17423E+01_JPRB, 0.14317E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10737E+01_JPRB, 0.16655E+01_JPRB, 0.21184E+01_JPRB, 0.18693E+01_JPRB, 0.17284E+01_JPRB, 0.15414E+01_JPRB ,&
&  0.14379E+01_JPRB, 0.12759E+01_JPRB, 0.11503E+01_JPRB, 0.10181E+01_JPRB, 0.96395E+00_JPRB, 0.94453E+00_JPRB, 0.99044E+00_JPRB ,&
&  0.10167E+01_JPRB, 0.99225E+00_JPRB, 0.10040E+01_JPRB, 0.10600E+01_JPRB, 0.14072E+01_JPRB, 0.19736E+01_JPRB, 0.20664E+01_JPRB ,&
&  0.20483E+01_JPRB, 0.13477E+01_JPRB, 0.11037E+01_JPRB, 0.10848E+01_JPRB, 0.69372E+00_JPRB, 0.71044E+00_JPRB, 0.74962E+00_JPRB ,&
&  0.93977E+00_JPRB, 0.10150E+01_JPRB, 0.13439E+01_JPRB, 0.35654E+01_JPRB, 0.40474E+01_JPRB, 0.50326E+01_JPRB, 0.20554E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.69614E+00_JPRB, 0.63473E+00_JPRB, 0.63962E+00_JPRB, 0.63004E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.60397E+00_JPRB, 0.65456E+00_JPRB, 0.70007E+00_JPRB, 0.76739E+00_JPRB, 0.79970E+00_JPRB, 0.81780E+00_JPRB, 0.82862E+00_JPRB ,&
&  0.82566E+00_JPRB, 0.83662E+00_JPRB, 0.86672E+00_JPRB, 0.93754E+00_JPRB, 0.10153E+01_JPRB, 0.10970E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.13122E+01_JPRB, 0.14251E+01_JPRB, 0.14525E+01_JPRB, 0.15039E+01_JPRB, 0.15835E+01_JPRB, 0.16572E+01_JPRB, 0.17073E+01_JPRB ,&
&  0.17621E+01_JPRB, 0.18617E+01_JPRB, 0.19805E+01_JPRB, 0.21384E+01_JPRB, 0.23416E+01_JPRB, 0.25680E+01_JPRB, 0.18981E+01_JPRB ,&
&  0.20652E+01_JPRB, 0.18063E+01_JPRB, 0.17802E+01_JPRB, 0.17982E+01_JPRB, 0.17138E+01_JPRB, 0.16409E+01_JPRB, 0.16771E+01_JPRB ,&
&  0.18004E+01_JPRB, 0.19458E+01_JPRB, 0.19599E+01_JPRB, 0.18617E+01_JPRB, 0.19335E+01_JPRB, 0.18130E+01_JPRB, 0.20489E+01_JPRB ,&
&  0.22417E+01_JPRB, 0.23548E+01_JPRB, 0.24089E+01_JPRB, 0.24594E+01_JPRB, 0.25199E+01_JPRB, 0.26073E+01_JPRB, 0.27117E+01_JPRB ,&
&  0.27659E+01_JPRB, 0.27816E+01_JPRB, 0.28024E+01_JPRB, 0.28183E+01_JPRB, 0.25127E+01_JPRB, 0.26233E+01_JPRB, 0.21052E+01_JPRB ,&
&  0.14388E+01_JPRB, 0.12028E+01_JPRB, 0.15734E+01_JPRB, 0.18527E+01_JPRB, 0.16959E+01_JPRB, 0.15433E+01_JPRB, 0.13967E+01_JPRB ,&
&  0.12491E+01_JPRB, 0.11378E+01_JPRB, 0.10397E+01_JPRB, 0.94257E+00_JPRB, 0.86953E+00_JPRB, 0.83470E+00_JPRB, 0.83282E+00_JPRB ,&
&  0.84146E+00_JPRB, 0.84140E+00_JPRB, 0.85331E+00_JPRB, 0.92911E+00_JPRB, 0.11661E+01_JPRB, 0.17243E+01_JPRB, 0.25396E+01_JPRB ,&
&  0.29131E+01_JPRB, 0.13969E+01_JPRB, 0.10344E+01_JPRB, 0.89574E+00_JPRB, 0.82949E+00_JPRB, 0.84794E+00_JPRB, 0.88240E+00_JPRB ,&
&  0.10241E+01_JPRB, 0.13334E+01_JPRB, 0.12277E+01_JPRB, 0.13885E+01_JPRB, 0.26419E+01_JPRB, 0.28359E+01_JPRB, 0.16615E+01_JPRB ,&
&  0.10779E+01_JPRB, 0.77968E+00_JPRB, 0.64329E+00_JPRB, 0.60429E+00_JPRB, 0.59686E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.60381E+00_JPRB, 0.62113E+00_JPRB, 0.64653E+00_JPRB, 0.67835E+00_JPRB, 0.68200E+00_JPRB, 0.68983E+00_JPRB, 0.68235E+00_JPRB ,&
&  0.68536E+00_JPRB, 0.71160E+00_JPRB, 0.77317E+00_JPRB, 0.87684E+00_JPRB, 0.95908E+00_JPRB, 0.10707E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.13439E+01_JPRB, 0.13755E+01_JPRB, 0.14270E+01_JPRB, 0.14882E+01_JPRB, 0.15621E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.19002E+01_JPRB, 0.20632E+01_JPRB, 0.22613E+01_JPRB, 0.24561E+01_JPRB, 0.26244E+01_JPRB, 0.29944E+01_JPRB ,&
&  0.16043E+01_JPRB, 0.19800E+01_JPRB, 0.20042E+01_JPRB, 0.19617E+01_JPRB, 0.16721E+01_JPRB, 0.15632E+01_JPRB, 0.17002E+01_JPRB ,&
&  0.20540E+01_JPRB, 0.23264E+01_JPRB, 0.25816E+01_JPRB, 0.19984E+01_JPRB, 0.20941E+01_JPRB, 0.21522E+01_JPRB, 0.24271E+01_JPRB ,&
&  0.25314E+01_JPRB, 0.26274E+01_JPRB, 0.27538E+01_JPRB, 0.28967E+01_JPRB, 0.30448E+01_JPRB, 0.32291E+01_JPRB, 0.34005E+01_JPRB ,&
&  0.35229E+01_JPRB, 0.35567E+01_JPRB, 0.36538E+01_JPRB, 0.32693E+01_JPRB, 0.28440E+01_JPRB, 0.29588E+01_JPRB, 0.28499E+01_JPRB ,&
&  0.23728E+01_JPRB, 0.16562E+01_JPRB, 0.16317E+01_JPRB, 0.18002E+01_JPRB, 0.16583E+01_JPRB, 0.15093E+01_JPRB, 0.13531E+01_JPRB ,&
&  0.11659E+01_JPRB, 0.99717E+00_JPRB, 0.92616E+00_JPRB, 0.85409E+00_JPRB, 0.79847E+00_JPRB, 0.78207E+00_JPRB, 0.80004E+00_JPRB ,&
&  0.81633E+00_JPRB, 0.81950E+00_JPRB, 0.81434E+00_JPRB, 0.83591E+00_JPRB, 0.92158E+00_JPRB, 0.10063E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10138E+01_JPRB, 0.96860E+00_JPRB, 0.97083E+00_JPRB, 0.96290E+00_JPRB, 0.93607E+00_JPRB, 0.95414E+00_JPRB ,&
&  0.95493E+00_JPRB, 0.94447E+00_JPRB, 0.10430E+01_JPRB, 0.12691E+01_JPRB, 0.12958E+01_JPRB, 0.13502E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.91192E+00_JPRB, 0.78155E+00_JPRB, 0.69339E+00_JPRB, 0.63625E+00_JPRB, 0.59161E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.59277E+00_JPRB, 0.59303E+00_JPRB, 0.59409E+00_JPRB, 0.61149E+00_JPRB, 0.61607E+00_JPRB, 0.62206E+00_JPRB, 0.62238E+00_JPRB ,&
&  0.65547E+00_JPRB, 0.71799E+00_JPRB, 0.81410E+00_JPRB, 0.92596E+00_JPRB, 0.10163E+01_JPRB, 0.11254E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.12449E+01_JPRB, 0.12709E+01_JPRB, 0.13309E+01_JPRB, 0.13975E+01_JPRB, 0.14881E+01_JPRB, 0.15873E+01_JPRB ,&
&  0.16999E+01_JPRB, 0.18494E+01_JPRB, 0.20522E+01_JPRB, 0.22943E+01_JPRB, 0.25557E+01_JPRB, 0.27943E+01_JPRB, 0.31343E+01_JPRB ,&
&  0.12533E+01_JPRB, 0.15910E+01_JPRB, 0.21336E+01_JPRB, 0.22458E+01_JPRB, 0.17311E+01_JPRB, 0.15582E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.17968E+01_JPRB, 0.21369E+01_JPRB, 0.22000E+01_JPRB, 0.22119E+01_JPRB, 0.21610E+01_JPRB, 0.23675E+01_JPRB, 0.25378E+01_JPRB ,&
&  0.25953E+01_JPRB, 0.26695E+01_JPRB, 0.28131E+01_JPRB, 0.29911E+01_JPRB, 0.31628E+01_JPRB, 0.33270E+01_JPRB, 0.34990E+01_JPRB ,&
&  0.36744E+01_JPRB, 0.37694E+01_JPRB, 0.38882E+01_JPRB, 0.32860E+01_JPRB, 0.33941E+01_JPRB, 0.35832E+01_JPRB, 0.40419E+01_JPRB ,&
&  0.33581E+01_JPRB, 0.20863E+01_JPRB, 0.16414E+01_JPRB, 0.17720E+01_JPRB, 0.16481E+01_JPRB, 0.15299E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.95415E+00_JPRB, 0.89466E+00_JPRB, 0.87664E+00_JPRB, 0.85566E+00_JPRB, 0.85491E+00_JPRB, 0.86744E+00_JPRB ,&
&  0.87443E+00_JPRB, 0.88066E+00_JPRB, 0.86645E+00_JPRB, 0.86522E+00_JPRB, 0.92181E+00_JPRB, 0.97802E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.10239E+01_JPRB, 0.10227E+01_JPRB, 0.10381E+01_JPRB, 0.10633E+01_JPRB, 0.10933E+01_JPRB, 0.10966E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.97714E+00_JPRB, 0.91348E+00_JPRB, 0.89193E+00_JPRB, 0.87455E+00_JPRB, 0.85924E+00_JPRB, 0.88239E+00_JPRB, 0.83012E+00_JPRB ,&
&  0.79909E+00_JPRB, 0.78720E+00_JPRB, 0.75332E+00_JPRB, 0.68073E+00_JPRB, 0.60826E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.61621E+00_JPRB, 0.61244E+00_JPRB, 0.60515E+00_JPRB, 0.61091E+00_JPRB, 0.60556E+00_JPRB, 0.61670E+00_JPRB, 0.63549E+00_JPRB ,&
&  0.67261E+00_JPRB, 0.74036E+00_JPRB, 0.82849E+00_JPRB, 0.91104E+00_JPRB, 0.10077E+01_JPRB, 0.10906E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11509E+01_JPRB, 0.11760E+01_JPRB, 0.12375E+01_JPRB, 0.13036E+01_JPRB, 0.13946E+01_JPRB, 0.15167E+01_JPRB ,&
&  0.16657E+01_JPRB, 0.18591E+01_JPRB, 0.21048E+01_JPRB, 0.24024E+01_JPRB, 0.27932E+01_JPRB, 0.32551E+01_JPRB, 0.38114E+01_JPRB ,&
&  0.28591E+01_JPRB, 0.15302E+01_JPRB, 0.25941E+01_JPRB, 0.27050E+01_JPRB, 0.20545E+01_JPRB, 0.18378E+01_JPRB, 0.15676E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.19379E+01_JPRB, 0.19875E+01_JPRB, 0.20491E+01_JPRB, 0.21178E+01_JPRB, 0.23147E+01_JPRB, 0.24647E+01_JPRB ,&
&  0.25476E+01_JPRB, 0.26261E+01_JPRB, 0.27203E+01_JPRB, 0.28272E+01_JPRB, 0.29674E+01_JPRB, 0.31165E+01_JPRB, 0.32978E+01_JPRB ,&
&  0.35088E+01_JPRB, 0.36486E+01_JPRB, 0.37490E+01_JPRB, 0.32127E+01_JPRB, 0.34284E+01_JPRB, 0.32789E+01_JPRB, 0.31335E+01_JPRB ,&
&  0.28210E+01_JPRB, 0.17879E+01_JPRB, 0.13972E+01_JPRB, 0.16896E+01_JPRB, 0.16028E+01_JPRB, 0.12473E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.99029E+00_JPRB, 0.97246E+00_JPRB, 0.99180E+00_JPRB, 0.94891E+00_JPRB, 0.92196E+00_JPRB, 0.90861E+00_JPRB ,&
&  0.94782E+00_JPRB, 0.94565E+00_JPRB, 0.92736E+00_JPRB, 0.94172E+00_JPRB, 0.97582E+00_JPRB, 0.10277E+01_JPRB, 0.10684E+01_JPRB ,&
&  0.11042E+01_JPRB, 0.11276E+01_JPRB, 0.11492E+01_JPRB, 0.11869E+01_JPRB, 0.12146E+01_JPRB, 0.11523E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.10308E+01_JPRB, 0.99967E+00_JPRB, 0.96853E+00_JPRB, 0.94173E+00_JPRB, 0.89561E+00_JPRB, 0.89547E+00_JPRB, 0.92005E+00_JPRB ,&
&  0.94766E+00_JPRB, 0.94842E+00_JPRB, 0.86939E+00_JPRB, 0.72913E+00_JPRB, 0.64362E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.63371E+00_JPRB, 0.60741E+00_JPRB, 0.60118E+00_JPRB, 0.62011E+00_JPRB, 0.61853E+00_JPRB, 0.62730E+00_JPRB, 0.65059E+00_JPRB ,&
&  0.68686E+00_JPRB, 0.73780E+00_JPRB, 0.78378E+00_JPRB, 0.84509E+00_JPRB, 0.92694E+00_JPRB, 0.10109E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10587E+01_JPRB, 0.10866E+01_JPRB, 0.11276E+01_JPRB, 0.11998E+01_JPRB, 0.12818E+01_JPRB, 0.13791E+01_JPRB ,&
&  0.15260E+01_JPRB, 0.17311E+01_JPRB, 0.19563E+01_JPRB, 0.22044E+01_JPRB, 0.25492E+01_JPRB, 0.30996E+01_JPRB, 0.39547E+01_JPRB ,&
&  0.62960E+01_JPRB, 0.68008E+01_JPRB, 0.22761E+01_JPRB, 0.28376E+01_JPRB, 0.25367E+01_JPRB, 0.22017E+01_JPRB, 0.17404E+01_JPRB ,&
&  0.16368E+01_JPRB, 0.18656E+01_JPRB, 0.18678E+01_JPRB, 0.19451E+01_JPRB, 0.19219E+01_JPRB, 0.20840E+01_JPRB, 0.22286E+01_JPRB ,&
&  0.24006E+01_JPRB, 0.25005E+01_JPRB, 0.25443E+01_JPRB, 0.25626E+01_JPRB, 0.26231E+01_JPRB, 0.27292E+01_JPRB, 0.29129E+01_JPRB ,&
&  0.31845E+01_JPRB, 0.34708E+01_JPRB, 0.36822E+01_JPRB, 0.33772E+01_JPRB, 0.35154E+01_JPRB, 0.30935E+01_JPRB, 0.27546E+01_JPRB ,&
&  0.21553E+01_JPRB, 0.18366E+01_JPRB, 0.17396E+01_JPRB, 0.17406E+01_JPRB, 0.15204E+01_JPRB, 0.13546E+01_JPRB, 0.14354E+01_JPRB ,&
&  0.13252E+01_JPRB, 0.11754E+01_JPRB, 0.10907E+01_JPRB, 0.10487E+01_JPRB, 0.10252E+01_JPRB, 0.98739E+00_JPRB, 0.98904E+00_JPRB ,&
&  0.10202E+01_JPRB, 0.10280E+01_JPRB, 0.10054E+01_JPRB, 0.10322E+01_JPRB, 0.10607E+01_JPRB, 0.11002E+01_JPRB, 0.11355E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.12114E+01_JPRB, 0.12370E+01_JPRB, 0.12491E+01_JPRB, 0.11905E+01_JPRB, 0.11276E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10793E+01_JPRB, 0.10815E+01_JPRB, 0.10788E+01_JPRB, 0.10419E+01_JPRB, 0.10517E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.11014E+01_JPRB, 0.99553E+00_JPRB, 0.77054E+00_JPRB, 0.67456E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.68130E+00_JPRB, 0.65693E+00_JPRB, 0.65442E+00_JPRB, 0.66431E+00_JPRB, 0.66833E+00_JPRB, 0.65495E+00_JPRB, 0.65719E+00_JPRB ,&
&  0.67187E+00_JPRB, 0.69377E+00_JPRB, 0.69810E+00_JPRB, 0.73986E+00_JPRB, 0.83083E+00_JPRB, 0.92648E+00_JPRB, 0.95462E+00_JPRB ,&
&  0.95664E+00_JPRB, 0.95333E+00_JPRB, 0.95541E+00_JPRB, 0.98195E+00_JPRB, 0.10253E+01_JPRB, 0.11171E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.13465E+01_JPRB, 0.14903E+01_JPRB, 0.16049E+01_JPRB, 0.16880E+01_JPRB, 0.18708E+01_JPRB, 0.22358E+01_JPRB, 0.32089E+01_JPRB ,&
&  0.50185E+01_JPRB, 0.10018E+02_JPRB, 0.26438E+01_JPRB, 0.27623E+01_JPRB, 0.28072E+01_JPRB, 0.25608E+01_JPRB, 0.22495E+01_JPRB ,&
&  0.22025E+01_JPRB, 0.19816E+01_JPRB, 0.18954E+01_JPRB, 0.18513E+01_JPRB, 0.17251E+01_JPRB, 0.17801E+01_JPRB, 0.18794E+01_JPRB ,&
&  0.21054E+01_JPRB, 0.22539E+01_JPRB, 0.23381E+01_JPRB, 0.23195E+01_JPRB, 0.22873E+01_JPRB, 0.22713E+01_JPRB, 0.23207E+01_JPRB ,&
&  0.25384E+01_JPRB, 0.29541E+01_JPRB, 0.33969E+01_JPRB, 0.34278E+01_JPRB, 0.36564E+01_JPRB, 0.32396E+01_JPRB, 0.29547E+01_JPRB ,&
&  0.24564E+01_JPRB, 0.22531E+01_JPRB, 0.22369E+01_JPRB, 0.20672E+01_JPRB, 0.17202E+01_JPRB, 0.15600E+01_JPRB, 0.15761E+01_JPRB ,&
&  0.14982E+01_JPRB, 0.13563E+01_JPRB, 0.12033E+01_JPRB, 0.10965E+01_JPRB, 0.10798E+01_JPRB, 0.10926E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.11494E+01_JPRB, 0.11281E+01_JPRB, 0.11707E+01_JPRB, 0.11917E+01_JPRB, 0.12023E+01_JPRB, 0.12108E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.12599E+01_JPRB, 0.12797E+01_JPRB, 0.12373E+01_JPRB, 0.11448E+01_JPRB, 0.10902E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.11730E+01_JPRB, 0.11405E+01_JPRB, 0.10984E+01_JPRB, 0.11245E+01_JPRB, 0.11130E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.11276E+01_JPRB, 0.11785E+01_JPRB, 0.11939E+01_JPRB, 0.87766E+00_JPRB, 0.74771E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.77293E+00_JPRB, 0.74146E+00_JPRB, 0.71670E+00_JPRB, 0.69562E+00_JPRB, 0.68167E+00_JPRB, 0.68463E+00_JPRB, 0.68975E+00_JPRB ,&
&  0.68780E+00_JPRB, 0.69322E+00_JPRB, 0.69123E+00_JPRB, 0.69935E+00_JPRB, 0.72609E+00_JPRB, 0.78030E+00_JPRB, 0.84396E+00_JPRB ,&
&  0.85819E+00_JPRB, 0.83661E+00_JPRB, 0.82886E+00_JPRB, 0.82271E+00_JPRB, 0.84756E+00_JPRB, 0.89451E+00_JPRB, 0.99127E+00_JPRB ,&
&  0.11366E+01_JPRB, 0.12759E+01_JPRB, 0.13349E+01_JPRB, 0.13324E+01_JPRB, 0.13733E+01_JPRB, 0.15001E+01_JPRB, 0.23127E+01_JPRB ,&
&  0.40789E+01_JPRB, 0.89875E+01_JPRB, 0.54971E+01_JPRB, 0.42520E+01_JPRB, 0.31241E+01_JPRB, 0.26970E+01_JPRB, 0.26057E+01_JPRB ,&
&  0.29343E+01_JPRB, 0.27631E+01_JPRB, 0.24003E+01_JPRB, 0.18293E+01_JPRB, 0.15916E+01_JPRB, 0.15053E+01_JPRB, 0.15683E+01_JPRB ,&
&  0.17572E+01_JPRB, 0.19294E+01_JPRB, 0.20435E+01_JPRB, 0.20677E+01_JPRB, 0.20207E+01_JPRB, 0.19773E+01_JPRB, 0.19483E+01_JPRB ,&
&  0.19313E+01_JPRB, 0.20916E+01_JPRB, 0.27284E+01_JPRB, 0.33920E+01_JPRB, 0.37615E+01_JPRB, 0.34981E+01_JPRB, 0.33159E+01_JPRB ,&
&  0.32340E+01_JPRB, 0.29062E+01_JPRB, 0.27009E+01_JPRB, 0.23482E+01_JPRB, 0.18998E+01_JPRB, 0.19296E+01_JPRB, 0.16822E+01_JPRB ,&
&  0.15749E+01_JPRB, 0.14682E+01_JPRB, 0.13345E+01_JPRB, 0.11876E+01_JPRB, 0.11819E+01_JPRB, 0.11770E+01_JPRB, 0.11418E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.12227E+01_JPRB, 0.12065E+01_JPRB, 0.12196E+01_JPRB, 0.12278E+01_JPRB, 0.12446E+01_JPRB, 0.12533E+01_JPRB ,&
&  0.12573E+01_JPRB, 0.12920E+01_JPRB, 0.12238E+01_JPRB, 0.11494E+01_JPRB, 0.11102E+01_JPRB, 0.10655E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.11023E+01_JPRB, 0.11880E+01_JPRB, 0.11782E+01_JPRB, 0.11802E+01_JPRB, 0.11470E+01_JPRB, 0.11453E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.10598E+01_JPRB, 0.10454E+01_JPRB, 0.95793E+00_JPRB, 0.84335E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.83700E+00_JPRB, 0.80553E+00_JPRB, 0.74152E+00_JPRB, 0.68912E+00_JPRB, 0.67724E+00_JPRB, 0.68424E+00_JPRB, 0.69940E+00_JPRB ,&
&  0.70800E+00_JPRB, 0.71210E+00_JPRB, 0.71096E+00_JPRB, 0.70503E+00_JPRB, 0.71025E+00_JPRB, 0.69118E+00_JPRB, 0.71610E+00_JPRB ,&
&  0.75473E+00_JPRB, 0.76954E+00_JPRB, 0.78074E+00_JPRB, 0.74803E+00_JPRB, 0.74213E+00_JPRB, 0.73997E+00_JPRB, 0.78610E+00_JPRB ,&
&  0.89578E+00_JPRB, 0.10546E+01_JPRB, 0.11562E+01_JPRB, 0.11791E+01_JPRB, 0.11938E+01_JPRB, 0.13019E+01_JPRB, 0.18248E+01_JPRB ,&
&  0.33089E+01_JPRB, 0.75895E+01_JPRB, 0.32188E+01_JPRB, 0.53847E+01_JPRB, 0.37515E+01_JPRB, 0.29106E+01_JPRB, 0.22960E+01_JPRB ,&
&  0.24447E+01_JPRB, 0.29496E+01_JPRB, 0.22696E+01_JPRB, 0.17577E+01_JPRB, 0.14982E+01_JPRB, 0.14187E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.14566E+01_JPRB, 0.16036E+01_JPRB, 0.17379E+01_JPRB, 0.18003E+01_JPRB, 0.17677E+01_JPRB, 0.16951E+01_JPRB, 0.16736E+01_JPRB ,&
&  0.16562E+01_JPRB, 0.16608E+01_JPRB, 0.19313E+01_JPRB, 0.23375E+01_JPRB, 0.33620E+01_JPRB, 0.37352E+01_JPRB, 0.38330E+01_JPRB ,&
&  0.39174E+01_JPRB, 0.36540E+01_JPRB, 0.31306E+01_JPRB, 0.26515E+01_JPRB, 0.22220E+01_JPRB, 0.21220E+01_JPRB, 0.18379E+01_JPRB ,&
&  0.16563E+01_JPRB, 0.15285E+01_JPRB, 0.14078E+01_JPRB, 0.12889E+01_JPRB, 0.12330E+01_JPRB, 0.12127E+01_JPRB, 0.11804E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.12730E+01_JPRB, 0.12604E+01_JPRB, 0.12358E+01_JPRB, 0.12301E+01_JPRB, 0.12399E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.13028E+01_JPRB, 0.12298E+01_JPRB, 0.11065E+01_JPRB, 0.10810E+01_JPRB, 0.10856E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.12286E+01_JPRB, 0.12232E+01_JPRB, 0.11804E+01_JPRB, 0.11735E+01_JPRB, 0.12114E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10294E+01_JPRB, 0.99528E+00_JPRB, 0.95851E+00_JPRB, 0.90343E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.84578E+00_JPRB, 0.78909E+00_JPRB, 0.72344E+00_JPRB, 0.69137E+00_JPRB, 0.69252E+00_JPRB, 0.68137E+00_JPRB, 0.69350E+00_JPRB ,&
&  0.72007E+00_JPRB, 0.72430E+00_JPRB, 0.72457E+00_JPRB, 0.71383E+00_JPRB, 0.70445E+00_JPRB, 0.68207E+00_JPRB, 0.65877E+00_JPRB ,&
&  0.68114E+00_JPRB, 0.74673E+00_JPRB, 0.77498E+00_JPRB, 0.75783E+00_JPRB, 0.73500E+00_JPRB, 0.71177E+00_JPRB, 0.71074E+00_JPRB ,&
&  0.75042E+00_JPRB, 0.84590E+00_JPRB, 0.95727E+00_JPRB, 0.10207E+01_JPRB, 0.10486E+01_JPRB, 0.10964E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.24974E+01_JPRB, 0.78862E+01_JPRB, 0.31305E+01_JPRB, 0.58751E+01_JPRB, 0.42762E+01_JPRB, 0.33098E+01_JPRB, 0.23989E+01_JPRB ,&
&  0.22269E+01_JPRB, 0.23966E+01_JPRB, 0.20809E+01_JPRB, 0.17573E+01_JPRB, 0.15290E+01_JPRB, 0.14285E+01_JPRB, 0.14006E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.13826E+01_JPRB, 0.14288E+01_JPRB, 0.14769E+01_JPRB, 0.14809E+01_JPRB, 0.14475E+01_JPRB, 0.14181E+01_JPRB ,&
&  0.14086E+01_JPRB, 0.14171E+01_JPRB, 0.15564E+01_JPRB, 0.18863E+01_JPRB, 0.25736E+01_JPRB, 0.35278E+01_JPRB, 0.45516E+01_JPRB ,&
&  0.49316E+01_JPRB, 0.45726E+01_JPRB, 0.37475E+01_JPRB, 0.31400E+01_JPRB, 0.26945E+01_JPRB, 0.24932E+01_JPRB, 0.21608E+01_JPRB ,&
&  0.18924E+01_JPRB, 0.17084E+01_JPRB, 0.15540E+01_JPRB, 0.14466E+01_JPRB, 0.13471E+01_JPRB, 0.13135E+01_JPRB, 0.12795E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.13502E+01_JPRB, 0.13455E+01_JPRB, 0.13015E+01_JPRB, 0.12433E+01_JPRB, 0.12267E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.12653E+01_JPRB, 0.12420E+01_JPRB, 0.11421E+01_JPRB, 0.10963E+01_JPRB, 0.11021E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.12024E+01_JPRB, 0.12734E+01_JPRB, 0.12712E+01_JPRB, 0.12550E+01_JPRB, 0.12282E+01_JPRB, 0.12821E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.10397E+01_JPRB, 0.99734E+00_JPRB, 0.93772E+00_JPRB, 0.90804E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.86420E+00_JPRB, 0.79091E+00_JPRB, 0.74093E+00_JPRB, 0.70673E+00_JPRB, 0.70429E+00_JPRB, 0.69739E+00_JPRB, 0.69670E+00_JPRB ,&
&  0.71749E+00_JPRB, 0.72968E+00_JPRB, 0.74203E+00_JPRB, 0.73048E+00_JPRB, 0.70931E+00_JPRB, 0.70533E+00_JPRB, 0.68644E+00_JPRB ,&
&  0.67609E+00_JPRB, 0.71897E+00_JPRB, 0.76339E+00_JPRB, 0.77306E+00_JPRB, 0.75465E+00_JPRB, 0.71689E+00_JPRB, 0.69969E+00_JPRB ,&
&  0.70785E+00_JPRB, 0.73605E+00_JPRB, 0.79291E+00_JPRB, 0.85768E+00_JPRB, 0.90294E+00_JPRB, 0.93648E+00_JPRB, 0.10301E+01_JPRB ,&
&  0.15291E+01_JPRB, 0.25494E+01_JPRB, 0.45708E+01_JPRB, 0.57472E+01_JPRB, 0.45560E+01_JPRB, 0.34753E+01_JPRB, 0.26866E+01_JPRB ,&
&  0.23297E+01_JPRB, 0.21637E+01_JPRB, 0.19719E+01_JPRB, 0.17414E+01_JPRB, 0.15579E+01_JPRB, 0.14203E+01_JPRB, 0.13687E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.12706E+01_JPRB, 0.12357E+01_JPRB, 0.12232E+01_JPRB, 0.12142E+01_JPRB, 0.12054E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.12070E+01_JPRB, 0.12259E+01_JPRB, 0.12908E+01_JPRB, 0.15357E+01_JPRB, 0.22402E+01_JPRB, 0.31885E+01_JPRB, 0.46236E+01_JPRB ,&
&  0.55956E+01_JPRB, 0.50689E+01_JPRB, 0.42515E+01_JPRB, 0.35891E+01_JPRB, 0.31130E+01_JPRB, 0.28464E+01_JPRB, 0.25680E+01_JPRB ,&
&  0.23104E+01_JPRB, 0.20479E+01_JPRB, 0.18268E+01_JPRB, 0.16800E+01_JPRB, 0.15152E+01_JPRB, 0.14331E+01_JPRB, 0.14282E+01_JPRB ,&
&  0.13926E+01_JPRB, 0.13882E+01_JPRB, 0.13929E+01_JPRB, 0.14333E+01_JPRB, 0.13700E+01_JPRB, 0.12981E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.12181E+01_JPRB, 0.12153E+01_JPRB, 0.11740E+01_JPRB, 0.11586E+01_JPRB, 0.11602E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.12680E+01_JPRB, 0.12938E+01_JPRB, 0.13082E+01_JPRB, 0.12294E+01_JPRB, 0.12972E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.11136E+01_JPRB, 0.10597E+01_JPRB, 0.10234E+01_JPRB, 0.97674E+00_JPRB, 0.94316E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.95808E+00_JPRB, 0.87953E+00_JPRB, 0.80645E+00_JPRB, 0.76123E+00_JPRB, 0.72738E+00_JPRB, 0.71250E+00_JPRB, 0.70443E+00_JPRB ,&
&  0.71268E+00_JPRB, 0.72169E+00_JPRB, 0.73689E+00_JPRB, 0.73437E+00_JPRB, 0.71560E+00_JPRB, 0.71999E+00_JPRB, 0.73602E+00_JPRB ,&
&  0.72939E+00_JPRB, 0.73825E+00_JPRB, 0.75649E+00_JPRB, 0.74745E+00_JPRB, 0.72890E+00_JPRB, 0.70798E+00_JPRB, 0.69000E+00_JPRB ,&
&  0.69270E+00_JPRB, 0.69917E+00_JPRB, 0.71688E+00_JPRB, 0.75261E+00_JPRB, 0.80260E+00_JPRB, 0.84414E+00_JPRB, 0.87963E+00_JPRB ,&
&  0.10621E+01_JPRB, 0.28573E+01_JPRB, 0.42242E+01_JPRB, 0.49692E+01_JPRB, 0.44807E+01_JPRB, 0.34454E+01_JPRB, 0.28433E+01_JPRB ,&
&  0.22834E+01_JPRB, 0.20136E+01_JPRB, 0.18359E+01_JPRB, 0.16770E+01_JPRB, 0.15107E+01_JPRB, 0.14057E+01_JPRB, 0.13506E+01_JPRB ,&
&  0.12977E+01_JPRB, 0.12242E+01_JPRB, 0.11430E+01_JPRB, 0.10748E+01_JPRB, 0.10467E+01_JPRB, 0.10391E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10717E+01_JPRB, 0.11140E+01_JPRB, 0.12165E+01_JPRB, 0.17626E+01_JPRB, 0.20935E+01_JPRB, 0.35998E+01_JPRB ,&
&  0.43890E+01_JPRB, 0.38192E+01_JPRB, 0.34492E+01_JPRB, 0.31851E+01_JPRB, 0.29060E+01_JPRB, 0.26556E+01_JPRB, 0.25014E+01_JPRB ,&
&  0.23454E+01_JPRB, 0.21649E+01_JPRB, 0.20106E+01_JPRB, 0.18957E+01_JPRB, 0.17764E+01_JPRB, 0.16814E+01_JPRB, 0.15656E+01_JPRB ,&
&  0.15183E+01_JPRB, 0.14607E+01_JPRB, 0.14366E+01_JPRB, 0.14648E+01_JPRB, 0.14850E+01_JPRB, 0.14201E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.12162E+01_JPRB, 0.12077E+01_JPRB, 0.11813E+01_JPRB, 0.11893E+01_JPRB, 0.12098E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.12352E+01_JPRB, 0.12878E+01_JPRB, 0.13020E+01_JPRB, 0.12667E+01_JPRB, 0.13297E+01_JPRB, 0.12401E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.10702E+01_JPRB, 0.10200E+01_JPRB, 0.97770E+00_JPRB, 0.99526E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.10069E+01_JPRB, 0.95596E+00_JPRB, 0.89396E+00_JPRB, 0.84590E+00_JPRB, 0.80466E+00_JPRB, 0.77580E+00_JPRB, 0.76759E+00_JPRB ,&
&  0.76064E+00_JPRB, 0.74956E+00_JPRB, 0.75438E+00_JPRB, 0.75224E+00_JPRB, 0.74913E+00_JPRB, 0.74590E+00_JPRB, 0.76078E+00_JPRB ,&
&  0.77175E+00_JPRB, 0.77024E+00_JPRB, 0.76468E+00_JPRB, 0.75223E+00_JPRB, 0.73670E+00_JPRB, 0.71774E+00_JPRB, 0.70510E+00_JPRB ,&
&  0.69748E+00_JPRB, 0.69789E+00_JPRB, 0.70487E+00_JPRB, 0.72038E+00_JPRB, 0.75369E+00_JPRB, 0.79871E+00_JPRB, 0.84170E+00_JPRB ,&
&  0.89492E+00_JPRB, 0.16878E+01_JPRB, 0.24293E+01_JPRB, 0.37581E+01_JPRB, 0.41267E+01_JPRB, 0.34318E+01_JPRB, 0.28042E+01_JPRB ,&
&  0.23059E+01_JPRB, 0.19712E+01_JPRB, 0.17541E+01_JPRB, 0.15972E+01_JPRB, 0.14400E+01_JPRB, 0.13473E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.11564E+01_JPRB, 0.10798E+01_JPRB, 0.10183E+01_JPRB, 0.99040E+00_JPRB, 0.98516E+00_JPRB, 0.97626E+00_JPRB ,&
&  0.96740E+00_JPRB, 0.97404E+00_JPRB, 0.98361E+00_JPRB, 0.10112E+01_JPRB, 0.12575E+01_JPRB, 0.15306E+01_JPRB, 0.21565E+01_JPRB ,&
&  0.21797E+01_JPRB, 0.20690E+01_JPRB, 0.20480E+01_JPRB, 0.20232E+01_JPRB, 0.20438E+01_JPRB, 0.20539E+01_JPRB, 0.20851E+01_JPRB ,&
&  0.21086E+01_JPRB, 0.20870E+01_JPRB, 0.20294E+01_JPRB, 0.19820E+01_JPRB, 0.19288E+01_JPRB, 0.18721E+01_JPRB, 0.17654E+01_JPRB ,&
&  0.16790E+01_JPRB, 0.16075E+01_JPRB, 0.15601E+01_JPRB, 0.15119E+01_JPRB, 0.15014E+01_JPRB, 0.14675E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.12390E+01_JPRB, 0.12216E+01_JPRB, 0.12155E+01_JPRB, 0.12045E+01_JPRB, 0.11888E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.12145E+01_JPRB, 0.12488E+01_JPRB, 0.12630E+01_JPRB, 0.12955E+01_JPRB, 0.12680E+01_JPRB, 0.13093E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.10963E+01_JPRB, 0.10230E+01_JPRB, 0.98286E+00_JPRB, 0.10254E+01_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.11004E+01_JPRB, 0.10468E+01_JPRB, 0.99139E+00_JPRB, 0.92106E+00_JPRB, 0.86421E+00_JPRB, 0.83738E+00_JPRB, 0.81930E+00_JPRB ,&
&  0.79723E+00_JPRB, 0.77692E+00_JPRB, 0.77356E+00_JPRB, 0.77851E+00_JPRB, 0.78247E+00_JPRB, 0.78385E+00_JPRB, 0.77773E+00_JPRB ,&
&  0.77603E+00_JPRB, 0.77914E+00_JPRB, 0.77081E+00_JPRB, 0.75710E+00_JPRB, 0.74215E+00_JPRB, 0.72951E+00_JPRB, 0.71989E+00_JPRB ,&
&  0.71424E+00_JPRB, 0.71687E+00_JPRB, 0.71905E+00_JPRB, 0.73042E+00_JPRB, 0.75224E+00_JPRB, 0.78282E+00_JPRB, 0.82534E+00_JPRB ,&
&  0.87434E+00_JPRB, 0.98027E+00_JPRB, 0.14835E+01_JPRB, 0.22077E+01_JPRB, 0.28269E+01_JPRB, 0.28709E+01_JPRB, 0.25607E+01_JPRB ,&
&  0.22690E+01_JPRB, 0.19801E+01_JPRB, 0.17561E+01_JPRB, 0.15701E+01_JPRB, 0.14173E+01_JPRB, 0.13011E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.10872E+01_JPRB, 0.10343E+01_JPRB, 0.98660E+00_JPRB, 0.96194E+00_JPRB, 0.94349E+00_JPRB, 0.92617E+00_JPRB ,&
&  0.91109E+00_JPRB, 0.90226E+00_JPRB, 0.89807E+00_JPRB, 0.89950E+00_JPRB, 0.92939E+00_JPRB, 0.10137E+01_JPRB, 0.11892E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.13303E+01_JPRB, 0.13405E+01_JPRB, 0.13365E+01_JPRB, 0.13586E+01_JPRB, 0.14152E+01_JPRB, 0.15080E+01_JPRB ,&
&  0.16221E+01_JPRB, 0.17215E+01_JPRB, 0.17974E+01_JPRB, 0.18455E+01_JPRB, 0.18491E+01_JPRB, 0.18182E+01_JPRB, 0.17649E+01_JPRB ,&
&  0.17081E+01_JPRB, 0.16257E+01_JPRB, 0.15474E+01_JPRB, 0.14850E+01_JPRB, 0.14471E+01_JPRB, 0.14207E+01_JPRB, 0.13797E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.12542E+01_JPRB, 0.12149E+01_JPRB, 0.12001E+01_JPRB, 0.11921E+01_JPRB, 0.11757E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.11915E+01_JPRB, 0.11750E+01_JPRB, 0.12510E+01_JPRB, 0.12882E+01_JPRB, 0.12431E+01_JPRB, 0.12010E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.10931E+01_JPRB, 0.10419E+01_JPRB, 0.10024E+01_JPRB, 0.10954E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.10224E+01_JPRB, 0.10118E+01_JPRB, 0.97155E+00_JPRB, 0.91826E+00_JPRB, 0.86719E+00_JPRB, 0.82504E+00_JPRB, 0.80169E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.77302E+00_JPRB, 0.77741E+00_JPRB, 0.79041E+00_JPRB, 0.80589E+00_JPRB, 0.81040E+00_JPRB, 0.80525E+00_JPRB ,&
&  0.79284E+00_JPRB, 0.77328E+00_JPRB, 0.75967E+00_JPRB, 0.75086E+00_JPRB, 0.73957E+00_JPRB, 0.73408E+00_JPRB, 0.73146E+00_JPRB ,&
&  0.73417E+00_JPRB, 0.73926E+00_JPRB, 0.74234E+00_JPRB, 0.75199E+00_JPRB, 0.76844E+00_JPRB, 0.79478E+00_JPRB, 0.82642E+00_JPRB ,&
&  0.84926E+00_JPRB, 0.78510E+00_JPRB, 0.98139E+00_JPRB, 0.14095E+01_JPRB, 0.19008E+01_JPRB, 0.21578E+01_JPRB, 0.21469E+01_JPRB ,&
&  0.20007E+01_JPRB, 0.18526E+01_JPRB, 0.16965E+01_JPRB, 0.15221E+01_JPRB, 0.13698E+01_JPRB, 0.12484E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10078E+01_JPRB, 0.97348E+00_JPRB, 0.94827E+00_JPRB, 0.92460E+00_JPRB, 0.88961E+00_JPRB, 0.86461E+00_JPRB ,&
&  0.85373E+00_JPRB, 0.84682E+00_JPRB, 0.84414E+00_JPRB, 0.84350E+00_JPRB, 0.85749E+00_JPRB, 0.88404E+00_JPRB, 0.91508E+00_JPRB ,&
&  0.94497E+00_JPRB, 0.97611E+00_JPRB, 0.99231E+00_JPRB, 0.99620E+00_JPRB, 0.99781E+00_JPRB, 0.10445E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.11584E+01_JPRB, 0.12653E+01_JPRB, 0.13923E+01_JPRB, 0.14729E+01_JPRB, 0.14898E+01_JPRB, 0.14582E+01_JPRB, 0.14005E+01_JPRB ,&
&  0.13580E+01_JPRB, 0.13117E+01_JPRB, 0.12653E+01_JPRB, 0.12277E+01_JPRB, 0.12069E+01_JPRB, 0.12065E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.11876E+01_JPRB, 0.11716E+01_JPRB, 0.11757E+01_JPRB, 0.11840E+01_JPRB, 0.11779E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.11273E+01_JPRB, 0.11191E+01_JPRB, 0.11640E+01_JPRB, 0.11437E+01_JPRB, 0.11306E+01_JPRB, 0.11023E+01_JPRB ,&
&  0.10761E+01_JPRB, 0.10420E+01_JPRB, 0.99935E+00_JPRB, 0.96273E+00_JPRB, 0.95444E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.90366E+00_JPRB, 0.93802E+00_JPRB, 0.93986E+00_JPRB, 0.90739E+00_JPRB, 0.86965E+00_JPRB, 0.84281E+00_JPRB, 0.82417E+00_JPRB ,&
&  0.81697E+00_JPRB, 0.80959E+00_JPRB, 0.81197E+00_JPRB, 0.82361E+00_JPRB, 0.83637E+00_JPRB, 0.83894E+00_JPRB, 0.82115E+00_JPRB ,&
&  0.79715E+00_JPRB, 0.77694E+00_JPRB, 0.76514E+00_JPRB, 0.75910E+00_JPRB, 0.75262E+00_JPRB, 0.74931E+00_JPRB, 0.75725E+00_JPRB ,&
&  0.76712E+00_JPRB, 0.77935E+00_JPRB, 0.79306E+00_JPRB, 0.80506E+00_JPRB, 0.81500E+00_JPRB, 0.82502E+00_JPRB, 0.84833E+00_JPRB ,&
&  0.86826E+00_JPRB, 0.71060E+00_JPRB, 0.77466E+00_JPRB, 0.10053E+01_JPRB, 0.12970E+01_JPRB, 0.16285E+01_JPRB, 0.17660E+01_JPRB ,&
&  0.17256E+01_JPRB, 0.16288E+01_JPRB, 0.15128E+01_JPRB, 0.13885E+01_JPRB, 0.12669E+01_JPRB, 0.11651E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.96455E+00_JPRB, 0.93145E+00_JPRB, 0.90023E+00_JPRB, 0.87738E+00_JPRB, 0.85689E+00_JPRB, 0.83906E+00_JPRB ,&
&  0.82355E+00_JPRB, 0.80880E+00_JPRB, 0.80287E+00_JPRB, 0.80297E+00_JPRB, 0.79850E+00_JPRB, 0.80263E+00_JPRB, 0.80048E+00_JPRB ,&
&  0.81655E+00_JPRB, 0.83927E+00_JPRB, 0.84838E+00_JPRB, 0.84907E+00_JPRB, 0.85377E+00_JPRB, 0.86902E+00_JPRB, 0.89186E+00_JPRB ,&
&  0.91902E+00_JPRB, 0.97560E+00_JPRB, 0.10377E+01_JPRB, 0.10975E+01_JPRB, 0.11345E+01_JPRB, 0.11421E+01_JPRB, 0.11313E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.11235E+01_JPRB, 0.11072E+01_JPRB, 0.10847E+01_JPRB, 0.10575E+01_JPRB, 0.10428E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.10835E+01_JPRB, 0.10985E+01_JPRB, 0.11202E+01_JPRB, 0.11255E+01_JPRB, 0.11122E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.10806E+01_JPRB, 0.10581E+01_JPRB, 0.10370E+01_JPRB, 0.10095E+01_JPRB, 0.10662E+01_JPRB, 0.10641E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.97966E+00_JPRB, 0.95295E+00_JPRB, 0.78509E+00_JPRB, 0.86393E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.85370E+00_JPRB, 0.88485E+00_JPRB, 0.89452E+00_JPRB, 0.88458E+00_JPRB, 0.86245E+00_JPRB, 0.84892E+00_JPRB, 0.83942E+00_JPRB ,&
&  0.83286E+00_JPRB, 0.82766E+00_JPRB, 0.83605E+00_JPRB, 0.84205E+00_JPRB, 0.84308E+00_JPRB, 0.83343E+00_JPRB, 0.82271E+00_JPRB ,&
&  0.80668E+00_JPRB, 0.78739E+00_JPRB, 0.77287E+00_JPRB, 0.76023E+00_JPRB, 0.75772E+00_JPRB, 0.76488E+00_JPRB, 0.78036E+00_JPRB ,&
&  0.80087E+00_JPRB, 0.81559E+00_JPRB, 0.82815E+00_JPRB, 0.84192E+00_JPRB, 0.84606E+00_JPRB, 0.84826E+00_JPRB, 0.85374E+00_JPRB ,&
&  0.86369E+00_JPRB, 0.68374E+00_JPRB, 0.74155E+00_JPRB, 0.83331E+00_JPRB, 0.94961E+00_JPRB, 0.11319E+01_JPRB, 0.12820E+01_JPRB ,&
&  0.13351E+01_JPRB, 0.12739E+01_JPRB, 0.12135E+01_JPRB, 0.11476E+01_JPRB, 0.10826E+01_JPRB, 0.10261E+01_JPRB, 0.97027E+00_JPRB ,&
&  0.92581E+00_JPRB, 0.89823E+00_JPRB, 0.88391E+00_JPRB, 0.86944E+00_JPRB, 0.85284E+00_JPRB, 0.83296E+00_JPRB, 0.81117E+00_JPRB ,&
&  0.78916E+00_JPRB, 0.77422E+00_JPRB, 0.76239E+00_JPRB, 0.75370E+00_JPRB, 0.75199E+00_JPRB, 0.75425E+00_JPRB, 0.75583E+00_JPRB ,&
&  0.76395E+00_JPRB, 0.77116E+00_JPRB, 0.77426E+00_JPRB, 0.77236E+00_JPRB, 0.76993E+00_JPRB, 0.77097E+00_JPRB, 0.78750E+00_JPRB ,&
&  0.80131E+00_JPRB, 0.82580E+00_JPRB, 0.85802E+00_JPRB, 0.90505E+00_JPRB, 0.94487E+00_JPRB, 0.96360E+00_JPRB, 0.97242E+00_JPRB ,&
&  0.97591E+00_JPRB, 0.96766E+00_JPRB, 0.94866E+00_JPRB, 0.92880E+00_JPRB, 0.91328E+00_JPRB, 0.90746E+00_JPRB, 0.90311E+00_JPRB ,&
&  0.90813E+00_JPRB, 0.92033E+00_JPRB, 0.93145E+00_JPRB, 0.94980E+00_JPRB, 0.97113E+00_JPRB, 0.97643E+00_JPRB, 0.95759E+00_JPRB ,&
&  0.94165E+00_JPRB, 0.93463E+00_JPRB, 0.92792E+00_JPRB, 0.95558E+00_JPRB, 0.96344E+00_JPRB, 0.96040E+00_JPRB, 0.94192E+00_JPRB ,&
&  0.92631E+00_JPRB, 0.91167E+00_JPRB, 0.70620E+00_JPRB, 0.79921E+00_JPRB, 0.82072E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.84415E+00_JPRB, 0.83968E+00_JPRB, 0.83310E+00_JPRB, 0.83643E+00_JPRB, 0.83999E+00_JPRB, 0.83851E+00_JPRB, 0.83709E+00_JPRB ,&
&  0.83388E+00_JPRB, 0.82945E+00_JPRB, 0.83684E+00_JPRB, 0.84375E+00_JPRB, 0.84417E+00_JPRB, 0.83698E+00_JPRB, 0.82931E+00_JPRB ,&
&  0.82331E+00_JPRB, 0.81393E+00_JPRB, 0.80444E+00_JPRB, 0.80002E+00_JPRB, 0.80181E+00_JPRB, 0.81503E+00_JPRB, 0.83553E+00_JPRB ,&
&  0.85084E+00_JPRB, 0.84987E+00_JPRB, 0.85357E+00_JPRB, 0.86524E+00_JPRB, 0.86882E+00_JPRB, 0.87175E+00_JPRB, 0.87968E+00_JPRB ,&
&  0.76164E+00_JPRB, 0.64199E+00_JPRB, 0.71361E+00_JPRB, 0.78326E+00_JPRB, 0.84555E+00_JPRB, 0.92358E+00_JPRB, 0.10265E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10533E+01_JPRB, 0.10293E+01_JPRB, 0.99556E+00_JPRB, 0.95750E+00_JPRB, 0.92958E+00_JPRB, 0.90719E+00_JPRB ,&
&  0.89968E+00_JPRB, 0.88660E+00_JPRB, 0.86660E+00_JPRB, 0.84984E+00_JPRB, 0.83354E+00_JPRB, 0.81243E+00_JPRB, 0.79197E+00_JPRB ,&
&  0.77106E+00_JPRB, 0.75291E+00_JPRB, 0.74483E+00_JPRB, 0.73913E+00_JPRB, 0.73871E+00_JPRB, 0.73660E+00_JPRB, 0.73010E+00_JPRB ,&
&  0.72554E+00_JPRB, 0.72571E+00_JPRB, 0.73371E+00_JPRB, 0.74104E+00_JPRB, 0.74267E+00_JPRB, 0.74035E+00_JPRB, 0.74880E+00_JPRB ,&
&  0.75618E+00_JPRB, 0.77070E+00_JPRB, 0.78970E+00_JPRB, 0.82004E+00_JPRB, 0.84465E+00_JPRB, 0.85587E+00_JPRB, 0.86329E+00_JPRB ,&
&  0.87262E+00_JPRB, 0.86861E+00_JPRB, 0.85268E+00_JPRB, 0.84037E+00_JPRB, 0.83370E+00_JPRB, 0.82507E+00_JPRB, 0.82070E+00_JPRB ,&
&  0.82529E+00_JPRB, 0.82958E+00_JPRB, 0.82806E+00_JPRB, 0.83407E+00_JPRB, 0.84574E+00_JPRB, 0.85853E+00_JPRB, 0.86811E+00_JPRB ,&
&  0.85468E+00_JPRB, 0.85919E+00_JPRB, 0.85923E+00_JPRB, 0.87298E+00_JPRB, 0.88904E+00_JPRB, 0.88095E+00_JPRB, 0.87090E+00_JPRB ,&
&  0.86006E+00_JPRB, 0.83541E+00_JPRB, 0.83352E+00_JPRB, 0.83278E+00_JPRB, 0.83503E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.79386E+00_JPRB, 0.78547E+00_JPRB, 0.77996E+00_JPRB, 0.78901E+00_JPRB, 0.80482E+00_JPRB, 0.81676E+00_JPRB, 0.81736E+00_JPRB ,&
&  0.81748E+00_JPRB, 0.82210E+00_JPRB, 0.83028E+00_JPRB, 0.83234E+00_JPRB, 0.82514E+00_JPRB, 0.80813E+00_JPRB, 0.79919E+00_JPRB ,&
&  0.79659E+00_JPRB, 0.79702E+00_JPRB, 0.79643E+00_JPRB, 0.80025E+00_JPRB, 0.81918E+00_JPRB, 0.84915E+00_JPRB, 0.87754E+00_JPRB ,&
&  0.89141E+00_JPRB, 0.88579E+00_JPRB, 0.87955E+00_JPRB, 0.87468E+00_JPRB, 0.87384E+00_JPRB, 0.87922E+00_JPRB, 0.88036E+00_JPRB ,&
&  0.87401E+00_JPRB, 0.73101E+00_JPRB, 0.79200E+00_JPRB, 0.81787E+00_JPRB, 0.84923E+00_JPRB, 0.88665E+00_JPRB, 0.94591E+00_JPRB ,&
&  0.96922E+00_JPRB, 0.95640E+00_JPRB, 0.92752E+00_JPRB, 0.90592E+00_JPRB, 0.88686E+00_JPRB, 0.87730E+00_JPRB, 0.86997E+00_JPRB ,&
&  0.86654E+00_JPRB, 0.86265E+00_JPRB, 0.83984E+00_JPRB, 0.80363E+00_JPRB, 0.78208E+00_JPRB, 0.76615E+00_JPRB, 0.75168E+00_JPRB ,&
&  0.73796E+00_JPRB, 0.72150E+00_JPRB, 0.70585E+00_JPRB, 0.69308E+00_JPRB, 0.69247E+00_JPRB, 0.68881E+00_JPRB, 0.67467E+00_JPRB ,&
&  0.66814E+00_JPRB, 0.66491E+00_JPRB, 0.67168E+00_JPRB, 0.68170E+00_JPRB, 0.68568E+00_JPRB, 0.68091E+00_JPRB, 0.69051E+00_JPRB ,&
&  0.71266E+00_JPRB, 0.72736E+00_JPRB, 0.74041E+00_JPRB, 0.75726E+00_JPRB, 0.76698E+00_JPRB, 0.76892E+00_JPRB, 0.77262E+00_JPRB ,&
&  0.78150E+00_JPRB, 0.78168E+00_JPRB, 0.77432E+00_JPRB, 0.76853E+00_JPRB, 0.76996E+00_JPRB, 0.77087E+00_JPRB, 0.78022E+00_JPRB ,&
&  0.78906E+00_JPRB, 0.79200E+00_JPRB, 0.78781E+00_JPRB, 0.79012E+00_JPRB, 0.79597E+00_JPRB, 0.80643E+00_JPRB, 0.81351E+00_JPRB ,&
&  0.82097E+00_JPRB, 0.81747E+00_JPRB, 0.80496E+00_JPRB, 0.80739E+00_JPRB, 0.81576E+00_JPRB, 0.81489E+00_JPRB, 0.80264E+00_JPRB ,&
&  0.79120E+00_JPRB, 0.78175E+00_JPRB, 0.78358E+00_JPRB, 0.78448E+00_JPRB, 0.78973E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.77811E+00_JPRB, 0.77432E+00_JPRB, 0.77323E+00_JPRB, 0.78448E+00_JPRB, 0.80220E+00_JPRB, 0.80654E+00_JPRB, 0.80828E+00_JPRB ,&
&  0.82097E+00_JPRB, 0.83066E+00_JPRB, 0.83052E+00_JPRB, 0.82905E+00_JPRB, 0.82308E+00_JPRB, 0.80904E+00_JPRB, 0.79815E+00_JPRB ,&
&  0.80308E+00_JPRB, 0.80827E+00_JPRB, 0.80987E+00_JPRB, 0.83324E+00_JPRB, 0.86432E+00_JPRB, 0.90143E+00_JPRB, 0.92653E+00_JPRB ,&
&  0.93530E+00_JPRB, 0.92782E+00_JPRB, 0.92415E+00_JPRB, 0.91908E+00_JPRB, 0.91390E+00_JPRB, 0.91584E+00_JPRB, 0.91033E+00_JPRB ,&
&  0.90028E+00_JPRB, 0.81895E+00_JPRB, 0.85205E+00_JPRB, 0.86241E+00_JPRB, 0.87971E+00_JPRB, 0.90735E+00_JPRB, 0.94818E+00_JPRB ,&
&  0.95696E+00_JPRB, 0.94971E+00_JPRB, 0.93255E+00_JPRB, 0.90403E+00_JPRB, 0.88508E+00_JPRB, 0.87498E+00_JPRB, 0.85692E+00_JPRB ,&
&  0.82883E+00_JPRB, 0.79151E+00_JPRB, 0.75814E+00_JPRB, 0.73524E+00_JPRB, 0.72417E+00_JPRB, 0.71978E+00_JPRB, 0.71265E+00_JPRB ,&
&  0.70800E+00_JPRB, 0.70391E+00_JPRB, 0.69112E+00_JPRB, 0.67954E+00_JPRB, 0.66732E+00_JPRB, 0.65835E+00_JPRB, 0.64373E+00_JPRB ,&
&  0.63497E+00_JPRB, 0.62338E+00_JPRB, 0.62195E+00_JPRB, 0.64130E+00_JPRB, 0.66212E+00_JPRB, 0.66703E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.69931E+00_JPRB, 0.70782E+00_JPRB, 0.71009E+00_JPRB, 0.70779E+00_JPRB, 0.70098E+00_JPRB, 0.70094E+00_JPRB, 0.70948E+00_JPRB ,&
&  0.71731E+00_JPRB, 0.72796E+00_JPRB, 0.73217E+00_JPRB, 0.73731E+00_JPRB, 0.74425E+00_JPRB, 0.74951E+00_JPRB, 0.76037E+00_JPRB ,&
&  0.76598E+00_JPRB, 0.77001E+00_JPRB, 0.78130E+00_JPRB, 0.79271E+00_JPRB, 0.79776E+00_JPRB, 0.80055E+00_JPRB, 0.81033E+00_JPRB ,&
&  0.81450E+00_JPRB, 0.81304E+00_JPRB, 0.80458E+00_JPRB, 0.79267E+00_JPRB, 0.78492E+00_JPRB, 0.77269E+00_JPRB, 0.75323E+00_JPRB ,&
&  0.73525E+00_JPRB, 0.73279E+00_JPRB, 0.74190E+00_JPRB, 0.75184E+00_JPRB, 0.76891E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.76156E+00_JPRB, 0.77313E+00_JPRB, 0.78637E+00_JPRB, 0.79920E+00_JPRB, 0.81242E+00_JPRB, 0.81198E+00_JPRB, 0.81370E+00_JPRB ,&
&  0.83386E+00_JPRB, 0.83777E+00_JPRB, 0.83777E+00_JPRB, 0.82952E+00_JPRB, 0.82343E+00_JPRB, 0.82324E+00_JPRB, 0.81407E+00_JPRB ,&
&  0.80928E+00_JPRB, 0.81515E+00_JPRB, 0.83277E+00_JPRB, 0.86007E+00_JPRB, 0.88672E+00_JPRB, 0.92216E+00_JPRB, 0.94837E+00_JPRB ,&
&  0.95222E+00_JPRB, 0.94114E+00_JPRB, 0.94143E+00_JPRB, 0.94493E+00_JPRB, 0.94515E+00_JPRB, 0.92834E+00_JPRB, 0.92436E+00_JPRB ,&
&  0.90388E+00_JPRB, 0.91933E+00_JPRB, 0.78901E+00_JPRB, 0.86017E+00_JPRB, 0.90524E+00_JPRB, 0.94331E+00_JPRB, 0.96373E+00_JPRB ,&
&  0.95879E+00_JPRB, 0.94533E+00_JPRB, 0.91626E+00_JPRB, 0.88423E+00_JPRB, 0.87328E+00_JPRB, 0.85363E+00_JPRB, 0.83032E+00_JPRB ,&
&  0.78114E+00_JPRB, 0.71806E+00_JPRB, 0.68585E+00_JPRB, 0.67381E+00_JPRB, 0.66753E+00_JPRB, 0.66752E+00_JPRB, 0.67118E+00_JPRB ,&
&  0.67332E+00_JPRB, 0.67241E+00_JPRB, 0.65985E+00_JPRB, 0.64172E+00_JPRB, 0.61931E+00_JPRB, 0.60702E+00_JPRB, 0.60406E+00_JPRB ,&
&  0.60054E+00_JPRB, 0.58692E+00_JPRB, 0.58979E+00_JPRB, 0.61935E+00_JPRB, 0.64385E+00_JPRB, 0.66208E+00_JPRB, 0.67109E+00_JPRB ,&
&  0.67814E+00_JPRB, 0.67326E+00_JPRB, 0.67038E+00_JPRB, 0.66165E+00_JPRB, 0.63823E+00_JPRB, 0.63196E+00_JPRB, 0.65308E+00_JPRB ,&
&  0.67690E+00_JPRB, 0.69058E+00_JPRB, 0.69896E+00_JPRB, 0.71152E+00_JPRB, 0.72497E+00_JPRB, 0.73276E+00_JPRB, 0.73914E+00_JPRB ,&
&  0.74356E+00_JPRB, 0.74805E+00_JPRB, 0.76158E+00_JPRB, 0.77678E+00_JPRB, 0.78419E+00_JPRB, 0.78584E+00_JPRB, 0.78609E+00_JPRB ,&
&  0.78352E+00_JPRB, 0.77607E+00_JPRB, 0.76734E+00_JPRB, 0.75921E+00_JPRB, 0.73964E+00_JPRB, 0.71571E+00_JPRB, 0.69748E+00_JPRB ,&
&  0.68789E+00_JPRB, 0.69758E+00_JPRB, 0.70519E+00_JPRB, 0.71774E+00_JPRB, 0.74451E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.79736E+00_JPRB, 0.81127E+00_JPRB, 0.82275E+00_JPRB, 0.84348E+00_JPRB, 0.86213E+00_JPRB, 0.85926E+00_JPRB, 0.85262E+00_JPRB ,&
&  0.84843E+00_JPRB, 0.84465E+00_JPRB, 0.83815E+00_JPRB, 0.82904E+00_JPRB, 0.82292E+00_JPRB, 0.82249E+00_JPRB, 0.82294E+00_JPRB ,&
&  0.82417E+00_JPRB, 0.84386E+00_JPRB, 0.87628E+00_JPRB, 0.90154E+00_JPRB, 0.92914E+00_JPRB, 0.97175E+00_JPRB, 0.99154E+00_JPRB ,&
&  0.97411E+00_JPRB, 0.97265E+00_JPRB, 0.96183E+00_JPRB, 0.94755E+00_JPRB, 0.94976E+00_JPRB, 0.94111E+00_JPRB, 0.91381E+00_JPRB ,&
&  0.90841E+00_JPRB, 0.90565E+00_JPRB, 0.90183E+00_JPRB, 0.93176E+00_JPRB, 0.95703E+00_JPRB, 0.96126E+00_JPRB, 0.96300E+00_JPRB ,&
&  0.94043E+00_JPRB, 0.90335E+00_JPRB, 0.86384E+00_JPRB, 0.84277E+00_JPRB, 0.83984E+00_JPRB, 0.82015E+00_JPRB, 0.79237E+00_JPRB ,&
&  0.73474E+00_JPRB, 0.68700E+00_JPRB, 0.66023E+00_JPRB, 0.64393E+00_JPRB, 0.63817E+00_JPRB, 0.63467E+00_JPRB, 0.62924E+00_JPRB ,&
&  0.63362E+00_JPRB, 0.62721E+00_JPRB, 0.59751E+00_JPRB, 0.56589E+00_JPRB, 0.55932E+00_JPRB, 0.56781E+00_JPRB, 0.58794E+00_JPRB ,&
&  0.60284E+00_JPRB, 0.59754E+00_JPRB, 0.60384E+00_JPRB, 0.61135E+00_JPRB, 0.62882E+00_JPRB, 0.64726E+00_JPRB, 0.64983E+00_JPRB ,&
&  0.65138E+00_JPRB, 0.64731E+00_JPRB, 0.63501E+00_JPRB, 0.62647E+00_JPRB, 0.61513E+00_JPRB, 0.61040E+00_JPRB, 0.62156E+00_JPRB ,&
&  0.63886E+00_JPRB, 0.64838E+00_JPRB, 0.65890E+00_JPRB, 0.67255E+00_JPRB, 0.68429E+00_JPRB, 0.69327E+00_JPRB, 0.70017E+00_JPRB ,&
&  0.71052E+00_JPRB, 0.72544E+00_JPRB, 0.74052E+00_JPRB, 0.75379E+00_JPRB, 0.75408E+00_JPRB, 0.73993E+00_JPRB, 0.71451E+00_JPRB ,&
&  0.69751E+00_JPRB, 0.68429E+00_JPRB, 0.68141E+00_JPRB, 0.68742E+00_JPRB, 0.68259E+00_JPRB, 0.66302E+00_JPRB, 0.65320E+00_JPRB ,&
&  0.66393E+00_JPRB, 0.68785E+00_JPRB, 0.71061E+00_JPRB, 0.74149E+00_JPRB, 0.77390E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.83251E+00_JPRB, 0.86383E+00_JPRB, 0.88750E+00_JPRB, 0.89541E+00_JPRB, 0.89281E+00_JPRB, 0.88411E+00_JPRB, 0.86195E+00_JPRB ,&
&  0.83777E+00_JPRB, 0.82093E+00_JPRB, 0.80303E+00_JPRB, 0.77680E+00_JPRB, 0.75503E+00_JPRB, 0.75062E+00_JPRB, 0.76323E+00_JPRB ,&
&  0.80946E+00_JPRB, 0.85038E+00_JPRB, 0.86671E+00_JPRB, 0.87994E+00_JPRB, 0.92916E+00_JPRB, 0.97036E+00_JPRB, 0.97877E+00_JPRB ,&
&  0.97262E+00_JPRB, 0.97359E+00_JPRB, 0.93703E+00_JPRB, 0.90411E+00_JPRB, 0.90489E+00_JPRB, 0.91128E+00_JPRB, 0.89456E+00_JPRB ,&
&  0.92159E+00_JPRB, 0.94090E+00_JPRB, 0.93721E+00_JPRB, 0.91722E+00_JPRB, 0.91311E+00_JPRB, 0.90316E+00_JPRB, 0.89068E+00_JPRB ,&
&  0.86835E+00_JPRB, 0.81999E+00_JPRB, 0.78589E+00_JPRB, 0.78122E+00_JPRB, 0.77396E+00_JPRB, 0.76743E+00_JPRB, 0.75461E+00_JPRB ,&
&  0.71446E+00_JPRB, 0.67801E+00_JPRB, 0.65010E+00_JPRB, 0.63081E+00_JPRB, 0.61843E+00_JPRB, 0.60843E+00_JPRB, 0.59753E+00_JPRB ,&
&  0.58780E+00_JPRB, 0.55830E+00_JPRB, 0.52814E+00_JPRB, 0.52030E+00_JPRB, 0.52496E+00_JPRB, 0.53202E+00_JPRB, 0.55963E+00_JPRB ,&
&  0.58457E+00_JPRB, 0.58287E+00_JPRB, 0.57912E+00_JPRB, 0.58196E+00_JPRB, 0.59164E+00_JPRB, 0.59339E+00_JPRB, 0.58458E+00_JPRB ,&
&  0.58071E+00_JPRB, 0.57918E+00_JPRB, 0.57009E+00_JPRB, 0.56334E+00_JPRB, 0.56302E+00_JPRB, 0.56702E+00_JPRB, 0.58115E+00_JPRB ,&
&  0.58964E+00_JPRB, 0.59046E+00_JPRB, 0.60363E+00_JPRB, 0.62069E+00_JPRB, 0.63152E+00_JPRB, 0.64030E+00_JPRB, 0.64675E+00_JPRB ,&
&  0.66084E+00_JPRB, 0.68080E+00_JPRB, 0.69591E+00_JPRB, 0.69809E+00_JPRB, 0.67538E+00_JPRB, 0.65130E+00_JPRB, 0.63250E+00_JPRB ,&
&  0.62428E+00_JPRB, 0.62881E+00_JPRB, 0.63657E+00_JPRB, 0.64409E+00_JPRB, 0.65087E+00_JPRB, 0.65547E+00_JPRB, 0.65503E+00_JPRB ,&
&  0.67484E+00_JPRB, 0.70606E+00_JPRB, 0.73113E+00_JPRB, 0.76100E+00_JPRB, 0.79232E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.89712E+00_JPRB, 0.91447E+00_JPRB, 0.92584E+00_JPRB, 0.92602E+00_JPRB, 0.91007E+00_JPRB, 0.88319E+00_JPRB, 0.86034E+00_JPRB ,&
&  0.83437E+00_JPRB, 0.80911E+00_JPRB, 0.78514E+00_JPRB, 0.75219E+00_JPRB, 0.73438E+00_JPRB, 0.73756E+00_JPRB, 0.78625E+00_JPRB ,&
&  0.85155E+00_JPRB, 0.85646E+00_JPRB, 0.85030E+00_JPRB, 0.87511E+00_JPRB, 0.92684E+00_JPRB, 0.95805E+00_JPRB, 0.97842E+00_JPRB ,&
&  0.97994E+00_JPRB, 0.94608E+00_JPRB, 0.91320E+00_JPRB, 0.90072E+00_JPRB, 0.92536E+00_JPRB, 0.94448E+00_JPRB, 0.95062E+00_JPRB ,&
&  0.94720E+00_JPRB, 0.92773E+00_JPRB, 0.91113E+00_JPRB, 0.89736E+00_JPRB, 0.90287E+00_JPRB, 0.87812E+00_JPRB, 0.82534E+00_JPRB ,&
&  0.75894E+00_JPRB, 0.73193E+00_JPRB, 0.71802E+00_JPRB, 0.70244E+00_JPRB, 0.69162E+00_JPRB, 0.69217E+00_JPRB, 0.68355E+00_JPRB ,&
&  0.66992E+00_JPRB, 0.65335E+00_JPRB, 0.63700E+00_JPRB, 0.61806E+00_JPRB, 0.60297E+00_JPRB, 0.58182E+00_JPRB, 0.56203E+00_JPRB ,&
&  0.54377E+00_JPRB, 0.52111E+00_JPRB, 0.49998E+00_JPRB, 0.48470E+00_JPRB, 0.47741E+00_JPRB, 0.48779E+00_JPRB, 0.52819E+00_JPRB ,&
&  0.55983E+00_JPRB, 0.55694E+00_JPRB, 0.55493E+00_JPRB, 0.55990E+00_JPRB, 0.55415E+00_JPRB, 0.54520E+00_JPRB, 0.54060E+00_JPRB ,&
&  0.53630E+00_JPRB, 0.53302E+00_JPRB, 0.53218E+00_JPRB, 0.53487E+00_JPRB, 0.53753E+00_JPRB, 0.53569E+00_JPRB, 0.53736E+00_JPRB ,&
&  0.54098E+00_JPRB, 0.54565E+00_JPRB, 0.56254E+00_JPRB, 0.58695E+00_JPRB, 0.60821E+00_JPRB, 0.62394E+00_JPRB, 0.63853E+00_JPRB ,&
&  0.66042E+00_JPRB, 0.67380E+00_JPRB, 0.66141E+00_JPRB, 0.64980E+00_JPRB, 0.64416E+00_JPRB, 0.64977E+00_JPRB, 0.66045E+00_JPRB ,&
&  0.68092E+00_JPRB, 0.69349E+00_JPRB, 0.69968E+00_JPRB, 0.70103E+00_JPRB, 0.70404E+00_JPRB, 0.70483E+00_JPRB, 0.71760E+00_JPRB ,&
&  0.75176E+00_JPRB, 0.78110E+00_JPRB, 0.78824E+00_JPRB, 0.82064E+00_JPRB, 0.86225E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.93467E+00_JPRB, 0.93857E+00_JPRB, 0.92842E+00_JPRB, 0.92378E+00_JPRB, 0.91295E+00_JPRB, 0.89906E+00_JPRB, 0.87669E+00_JPRB ,&
&  0.85052E+00_JPRB, 0.82239E+00_JPRB, 0.79547E+00_JPRB, 0.77325E+00_JPRB, 0.77659E+00_JPRB, 0.80855E+00_JPRB, 0.83568E+00_JPRB ,&
&  0.82465E+00_JPRB, 0.82049E+00_JPRB, 0.83709E+00_JPRB, 0.86719E+00_JPRB, 0.89986E+00_JPRB, 0.92998E+00_JPRB, 0.95121E+00_JPRB ,&
&  0.96814E+00_JPRB, 0.94981E+00_JPRB, 0.95450E+00_JPRB, 0.95772E+00_JPRB, 0.95273E+00_JPRB, 0.93562E+00_JPRB, 0.90753E+00_JPRB ,&
&  0.87947E+00_JPRB, 0.85222E+00_JPRB, 0.81871E+00_JPRB, 0.76135E+00_JPRB, 0.83531E+00_JPRB, 0.76159E+00_JPRB, 0.73390E+00_JPRB ,&
&  0.70654E+00_JPRB, 0.68777E+00_JPRB, 0.66593E+00_JPRB, 0.64307E+00_JPRB, 0.63521E+00_JPRB, 0.63051E+00_JPRB, 0.62224E+00_JPRB ,&
&  0.61508E+00_JPRB, 0.60682E+00_JPRB, 0.59451E+00_JPRB, 0.57955E+00_JPRB, 0.56579E+00_JPRB, 0.55146E+00_JPRB, 0.53833E+00_JPRB ,&
&  0.51896E+00_JPRB, 0.49063E+00_JPRB, 0.46083E+00_JPRB, 0.43868E+00_JPRB, 0.42691E+00_JPRB, 0.43632E+00_JPRB, 0.47006E+00_JPRB ,&
&  0.50463E+00_JPRB, 0.51081E+00_JPRB, 0.52122E+00_JPRB, 0.52524E+00_JPRB, 0.51763E+00_JPRB, 0.50769E+00_JPRB, 0.49036E+00_JPRB ,&
&  0.47776E+00_JPRB, 0.47672E+00_JPRB, 0.47892E+00_JPRB, 0.48396E+00_JPRB, 0.48892E+00_JPRB, 0.49563E+00_JPRB, 0.50488E+00_JPRB ,&
&  0.51322E+00_JPRB, 0.51601E+00_JPRB, 0.52819E+00_JPRB, 0.55491E+00_JPRB, 0.58635E+00_JPRB, 0.61859E+00_JPRB, 0.64660E+00_JPRB ,&
&  0.66607E+00_JPRB, 0.67995E+00_JPRB, 0.68863E+00_JPRB, 0.70386E+00_JPRB, 0.71969E+00_JPRB, 0.73526E+00_JPRB, 0.74308E+00_JPRB ,&
&  0.74024E+00_JPRB, 0.74417E+00_JPRB, 0.76217E+00_JPRB, 0.79099E+00_JPRB, 0.81092E+00_JPRB, 0.84483E+00_JPRB, 0.86610E+00_JPRB ,&
&  0.88848E+00_JPRB, 0.89412E+00_JPRB, 0.88865E+00_JPRB, 0.90108E+00_JPRB, 0.91762E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.91812E+00_JPRB, 0.92854E+00_JPRB, 0.92622E+00_JPRB, 0.92487E+00_JPRB, 0.92618E+00_JPRB, 0.91874E+00_JPRB, 0.90471E+00_JPRB ,&
&  0.89057E+00_JPRB, 0.86381E+00_JPRB, 0.83274E+00_JPRB, 0.81845E+00_JPRB, 0.82516E+00_JPRB, 0.82440E+00_JPRB, 0.80558E+00_JPRB ,&
&  0.80367E+00_JPRB, 0.81697E+00_JPRB, 0.83949E+00_JPRB, 0.86458E+00_JPRB, 0.89835E+00_JPRB, 0.93347E+00_JPRB, 0.95953E+00_JPRB ,&
&  0.97000E+00_JPRB, 0.96862E+00_JPRB, 0.96149E+00_JPRB, 0.93504E+00_JPRB, 0.89947E+00_JPRB, 0.87263E+00_JPRB, 0.85210E+00_JPRB ,&
&  0.82622E+00_JPRB, 0.77699E+00_JPRB, 0.70876E+00_JPRB, 0.70348E+00_JPRB, 0.64224E+00_JPRB, 0.62418E+00_JPRB, 0.64702E+00_JPRB ,&
&  0.66253E+00_JPRB, 0.64463E+00_JPRB, 0.62001E+00_JPRB, 0.59952E+00_JPRB, 0.59500E+00_JPRB, 0.59295E+00_JPRB, 0.58716E+00_JPRB ,&
&  0.58260E+00_JPRB, 0.57525E+00_JPRB, 0.56055E+00_JPRB, 0.54717E+00_JPRB, 0.53449E+00_JPRB, 0.52186E+00_JPRB, 0.51286E+00_JPRB ,&
&  0.48350E+00_JPRB, 0.43915E+00_JPRB, 0.40604E+00_JPRB, 0.39120E+00_JPRB, 0.38299E+00_JPRB, 0.38706E+00_JPRB, 0.40086E+00_JPRB ,&
&  0.41256E+00_JPRB, 0.41434E+00_JPRB, 0.40952E+00_JPRB, 0.39660E+00_JPRB, 0.37502E+00_JPRB, 0.33275E+00_JPRB, 0.30786E+00_JPRB ,&
&  0.32538E+00_JPRB, 0.35716E+00_JPRB, 0.39052E+00_JPRB, 0.40375E+00_JPRB, 0.41278E+00_JPRB, 0.41393E+00_JPRB, 0.39796E+00_JPRB ,&
&  0.36797E+00_JPRB, 0.34129E+00_JPRB, 0.33703E+00_JPRB, 0.33920E+00_JPRB, 0.34522E+00_JPRB, 0.35584E+00_JPRB, 0.37257E+00_JPRB ,&
&  0.39757E+00_JPRB, 0.43474E+00_JPRB, 0.45732E+00_JPRB, 0.45999E+00_JPRB, 0.45261E+00_JPRB, 0.44159E+00_JPRB, 0.43906E+00_JPRB ,&
&  0.45377E+00_JPRB, 0.51365E+00_JPRB, 0.63422E+00_JPRB, 0.77858E+00_JPRB, 0.78116E+00_JPRB, 0.82598E+00_JPRB, 0.86064E+00_JPRB ,&
&  0.89068E+00_JPRB, 0.90437E+00_JPRB, 0.89134E+00_JPRB, 0.89214E+00_JPRB, 0.89835E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.87560E+00_JPRB, 0.89540E+00_JPRB, 0.90606E+00_JPRB, 0.91975E+00_JPRB, 0.91972E+00_JPRB, 0.88936E+00_JPRB, 0.87479E+00_JPRB ,&
&  0.87920E+00_JPRB, 0.87379E+00_JPRB, 0.86213E+00_JPRB, 0.85455E+00_JPRB, 0.84522E+00_JPRB, 0.83526E+00_JPRB, 0.82707E+00_JPRB ,&
&  0.81868E+00_JPRB, 0.81327E+00_JPRB, 0.81980E+00_JPRB, 0.83330E+00_JPRB, 0.84429E+00_JPRB, 0.85020E+00_JPRB, 0.85518E+00_JPRB ,&
&  0.85482E+00_JPRB, 0.82204E+00_JPRB, 0.77669E+00_JPRB, 0.74354E+00_JPRB, 0.72743E+00_JPRB, 0.71394E+00_JPRB, 0.68039E+00_JPRB ,&
&  0.61039E+00_JPRB, 0.48914E+00_JPRB, 0.43476E+00_JPRB, 0.47738E+00_JPRB, 0.59275E+00_JPRB, 0.56021E+00_JPRB, 0.58157E+00_JPRB ,&
&  0.61266E+00_JPRB, 0.62886E+00_JPRB, 0.62454E+00_JPRB, 0.61372E+00_JPRB, 0.60973E+00_JPRB, 0.61300E+00_JPRB, 0.60963E+00_JPRB ,&
&  0.60612E+00_JPRB, 0.58996E+00_JPRB, 0.58007E+00_JPRB, 0.56800E+00_JPRB, 0.54941E+00_JPRB, 0.52308E+00_JPRB, 0.50670E+00_JPRB ,&
&  0.46720E+00_JPRB, 0.42846E+00_JPRB, 0.41670E+00_JPRB, 0.41649E+00_JPRB, 0.38548E+00_JPRB, 0.37308E+00_JPRB, 0.35778E+00_JPRB ,&
&  0.31996E+00_JPRB, 0.29922E+00_JPRB, 0.26844E+00_JPRB, 0.24948E+00_JPRB, 0.23640E+00_JPRB, 0.22182E+00_JPRB, 0.21975E+00_JPRB ,&
&  0.22559E+00_JPRB, 0.23943E+00_JPRB, 0.26024E+00_JPRB, 0.29118E+00_JPRB, 0.31838E+00_JPRB, 0.28146E+00_JPRB, 0.26549E+00_JPRB ,&
&  0.25067E+00_JPRB, 0.24722E+00_JPRB, 0.25393E+00_JPRB, 0.26156E+00_JPRB, 0.26913E+00_JPRB, 0.27799E+00_JPRB, 0.28695E+00_JPRB ,&
&  0.29353E+00_JPRB, 0.29665E+00_JPRB, 0.29709E+00_JPRB, 0.29622E+00_JPRB, 0.29454E+00_JPRB, 0.29489E+00_JPRB, 0.29822E+00_JPRB ,&
&  0.30515E+00_JPRB, 0.31803E+00_JPRB, 0.34210E+00_JPRB, 0.37763E+00_JPRB, 0.42342E+00_JPRB, 0.48395E+00_JPRB, 0.57558E+00_JPRB ,&
&  0.69990E+00_JPRB, 0.86299E+00_JPRB, 0.90971E+00_JPRB, 0.91906E+00_JPRB, 0.88745E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.87899E+00_JPRB, 0.87596E+00_JPRB, 0.88281E+00_JPRB, 0.87232E+00_JPRB, 0.85322E+00_JPRB, 0.84422E+00_JPRB, 0.86114E+00_JPRB ,&
&  0.88074E+00_JPRB, 0.85088E+00_JPRB, 0.82881E+00_JPRB, 0.80941E+00_JPRB, 0.79517E+00_JPRB, 0.78097E+00_JPRB, 0.75903E+00_JPRB ,&
&  0.73886E+00_JPRB, 0.73125E+00_JPRB, 0.72897E+00_JPRB, 0.72366E+00_JPRB, 0.71327E+00_JPRB, 0.70313E+00_JPRB, 0.68545E+00_JPRB ,&
&  0.65198E+00_JPRB, 0.61664E+00_JPRB, 0.59843E+00_JPRB, 0.57461E+00_JPRB, 0.55780E+00_JPRB, 0.53834E+00_JPRB, 0.51073E+00_JPRB ,&
&  0.46481E+00_JPRB, 0.40685E+00_JPRB, 0.37334E+00_JPRB, 0.41147E+00_JPRB, 0.50416E+00_JPRB, 0.49711E+00_JPRB, 0.51764E+00_JPRB ,&
&  0.55294E+00_JPRB, 0.57431E+00_JPRB, 0.58494E+00_JPRB, 0.58825E+00_JPRB, 0.59737E+00_JPRB, 0.60343E+00_JPRB, 0.59965E+00_JPRB ,&
&  0.58395E+00_JPRB, 0.53130E+00_JPRB, 0.48724E+00_JPRB, 0.38265E+00_JPRB, 0.31243E+00_JPRB, 0.27516E+00_JPRB, 0.24490E+00_JPRB ,&
&  0.22487E+00_JPRB, 0.21360E+00_JPRB, 0.21121E+00_JPRB, 0.20972E+00_JPRB, 0.21550E+00_JPRB, 0.22297E+00_JPRB, 0.22351E+00_JPRB ,&
&  0.21930E+00_JPRB, 0.21086E+00_JPRB, 0.19950E+00_JPRB, 0.19333E+00_JPRB, 0.18996E+00_JPRB, 0.19033E+00_JPRB, 0.19265E+00_JPRB ,&
&  0.19388E+00_JPRB, 0.19718E+00_JPRB, 0.20984E+00_JPRB, 0.22035E+00_JPRB, 0.23390E+00_JPRB, 0.21783E+00_JPRB, 0.20391E+00_JPRB ,&
&  0.19833E+00_JPRB, 0.20367E+00_JPRB, 0.21154E+00_JPRB, 0.21736E+00_JPRB, 0.22341E+00_JPRB, 0.22829E+00_JPRB, 0.23310E+00_JPRB ,&
&  0.23826E+00_JPRB, 0.24202E+00_JPRB, 0.24365E+00_JPRB, 0.24543E+00_JPRB, 0.24700E+00_JPRB, 0.24943E+00_JPRB, 0.25230E+00_JPRB ,&
&  0.25720E+00_JPRB, 0.26696E+00_JPRB, 0.27955E+00_JPRB, 0.30046E+00_JPRB, 0.32006E+00_JPRB, 0.33947E+00_JPRB, 0.36246E+00_JPRB ,&
&  0.38277E+00_JPRB, 0.46399E+00_JPRB, 0.65670E+00_JPRB, 0.97477E+00_JPRB, 0.96825E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.82463E+00_JPRB, 0.83664E+00_JPRB, 0.85316E+00_JPRB, 0.82961E+00_JPRB, 0.81440E+00_JPRB, 0.81757E+00_JPRB, 0.81821E+00_JPRB ,&
&  0.79469E+00_JPRB, 0.75864E+00_JPRB, 0.70869E+00_JPRB, 0.66670E+00_JPRB, 0.60247E+00_JPRB, 0.51964E+00_JPRB, 0.48168E+00_JPRB ,&
&  0.44657E+00_JPRB, 0.44257E+00_JPRB, 0.44431E+00_JPRB, 0.46027E+00_JPRB, 0.46873E+00_JPRB, 0.46515E+00_JPRB, 0.44271E+00_JPRB ,&
&  0.41529E+00_JPRB, 0.38844E+00_JPRB, 0.37074E+00_JPRB, 0.35716E+00_JPRB, 0.35711E+00_JPRB, 0.35513E+00_JPRB, 0.34861E+00_JPRB ,&
&  0.34079E+00_JPRB, 0.34604E+00_JPRB, 0.36401E+00_JPRB, 0.39136E+00_JPRB, 0.43897E+00_JPRB, 0.41879E+00_JPRB, 0.42696E+00_JPRB ,&
&  0.45757E+00_JPRB, 0.47965E+00_JPRB, 0.48838E+00_JPRB, 0.49154E+00_JPRB, 0.48400E+00_JPRB, 0.50653E+00_JPRB, 0.44969E+00_JPRB ,&
&  0.34326E+00_JPRB, 0.28404E+00_JPRB, 0.24157E+00_JPRB, 0.21344E+00_JPRB, 0.19392E+00_JPRB, 0.18232E+00_JPRB, 0.16925E+00_JPRB ,&
&  0.16308E+00_JPRB, 0.15836E+00_JPRB, 0.15750E+00_JPRB, 0.15986E+00_JPRB, 0.16315E+00_JPRB, 0.16733E+00_JPRB, 0.16897E+00_JPRB ,&
&  0.17146E+00_JPRB, 0.17173E+00_JPRB, 0.17118E+00_JPRB, 0.17136E+00_JPRB, 0.17033E+00_JPRB, 0.17120E+00_JPRB, 0.17121E+00_JPRB ,&
&  0.17095E+00_JPRB, 0.17054E+00_JPRB, 0.17231E+00_JPRB, 0.17344E+00_JPRB, 0.17441E+00_JPRB, 0.17140E+00_JPRB, 0.17015E+00_JPRB ,&
&  0.17083E+00_JPRB, 0.17338E+00_JPRB, 0.17723E+00_JPRB, 0.18150E+00_JPRB, 0.18738E+00_JPRB, 0.19185E+00_JPRB, 0.19709E+00_JPRB ,&
&  0.20160E+00_JPRB, 0.20611E+00_JPRB, 0.21049E+00_JPRB, 0.21496E+00_JPRB, 0.21988E+00_JPRB, 0.22477E+00_JPRB, 0.22978E+00_JPRB ,&
&  0.23506E+00_JPRB, 0.24381E+00_JPRB, 0.25442E+00_JPRB, 0.27288E+00_JPRB, 0.29766E+00_JPRB, 0.32347E+00_JPRB, 0.36939E+00_JPRB ,&
&  0.42621E+00_JPRB, 0.65330E+00_JPRB, 0.95662E+00_JPRB, 0.79017E+00_JPRB, 0.80739E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.81368E+00_JPRB, 0.83181E+00_JPRB, 0.84747E+00_JPRB, 0.84467E+00_JPRB, 0.82719E+00_JPRB, 0.76952E+00_JPRB, 0.70600E+00_JPRB ,&
&  0.62729E+00_JPRB, 0.56163E+00_JPRB, 0.49205E+00_JPRB, 0.43219E+00_JPRB, 0.38159E+00_JPRB, 0.34250E+00_JPRB, 0.31807E+00_JPRB ,&
&  0.30046E+00_JPRB, 0.29265E+00_JPRB, 0.28903E+00_JPRB, 0.29344E+00_JPRB, 0.29536E+00_JPRB, 0.30072E+00_JPRB, 0.29495E+00_JPRB ,&
&  0.28784E+00_JPRB, 0.28015E+00_JPRB, 0.27803E+00_JPRB, 0.28662E+00_JPRB, 0.30013E+00_JPRB, 0.31319E+00_JPRB, 0.33028E+00_JPRB ,&
&  0.34861E+00_JPRB, 0.35450E+00_JPRB, 0.36920E+00_JPRB, 0.37275E+00_JPRB, 0.35844E+00_JPRB, 0.36048E+00_JPRB, 0.36648E+00_JPRB ,&
&  0.37982E+00_JPRB, 0.39837E+00_JPRB, 0.40891E+00_JPRB, 0.42842E+00_JPRB, 0.37392E+00_JPRB, 0.31388E+00_JPRB, 0.26142E+00_JPRB ,&
&  0.22223E+00_JPRB, 0.19966E+00_JPRB, 0.18321E+00_JPRB, 0.17489E+00_JPRB, 0.16641E+00_JPRB, 0.16181E+00_JPRB, 0.15708E+00_JPRB ,&
&  0.15504E+00_JPRB, 0.15525E+00_JPRB, 0.15504E+00_JPRB, 0.15510E+00_JPRB, 0.15600E+00_JPRB, 0.15727E+00_JPRB, 0.15703E+00_JPRB ,&
&  0.15662E+00_JPRB, 0.15658E+00_JPRB, 0.15627E+00_JPRB, 0.15597E+00_JPRB, 0.15512E+00_JPRB, 0.15508E+00_JPRB, 0.15330E+00_JPRB ,&
&  0.15242E+00_JPRB, 0.15091E+00_JPRB, 0.15009E+00_JPRB, 0.15016E+00_JPRB, 0.15075E+00_JPRB, 0.15130E+00_JPRB, 0.15218E+00_JPRB ,&
&  0.15373E+00_JPRB, 0.15511E+00_JPRB, 0.15739E+00_JPRB, 0.15943E+00_JPRB, 0.16277E+00_JPRB, 0.16559E+00_JPRB, 0.16954E+00_JPRB ,&
&  0.17375E+00_JPRB, 0.17884E+00_JPRB, 0.18482E+00_JPRB, 0.19161E+00_JPRB, 0.19719E+00_JPRB, 0.20260E+00_JPRB, 0.20840E+00_JPRB ,&
&  0.21287E+00_JPRB, 0.22073E+00_JPRB, 0.22947E+00_JPRB, 0.24411E+00_JPRB, 0.27481E+00_JPRB, 0.29792E+00_JPRB, 0.34591E+00_JPRB ,&
&  0.42411E+00_JPRB, 0.67540E+00_JPRB, 0.90882E+00_JPRB, 0.76391E+00_JPRB, 0.79540E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.69093E+00_JPRB, 0.72196E+00_JPRB, 0.74556E+00_JPRB, 0.74718E+00_JPRB, 0.74537E+00_JPRB, 0.70772E+00_JPRB, 0.64912E+00_JPRB ,&
&  0.54305E+00_JPRB, 0.47366E+00_JPRB, 0.42919E+00_JPRB, 0.38714E+00_JPRB, 0.34922E+00_JPRB, 0.32320E+00_JPRB, 0.30561E+00_JPRB ,&
&  0.29235E+00_JPRB, 0.28499E+00_JPRB, 0.27943E+00_JPRB, 0.27478E+00_JPRB, 0.26965E+00_JPRB, 0.26473E+00_JPRB, 0.25606E+00_JPRB ,&
&  0.25068E+00_JPRB, 0.24748E+00_JPRB, 0.24586E+00_JPRB, 0.24826E+00_JPRB, 0.25516E+00_JPRB, 0.26161E+00_JPRB, 0.27201E+00_JPRB ,&
&  0.28378E+00_JPRB, 0.29807E+00_JPRB, 0.30738E+00_JPRB, 0.31119E+00_JPRB, 0.31438E+00_JPRB, 0.31302E+00_JPRB, 0.29119E+00_JPRB ,&
&  0.29601E+00_JPRB, 0.31736E+00_JPRB, 0.33549E+00_JPRB, 0.34382E+00_JPRB, 0.29798E+00_JPRB, 0.25664E+00_JPRB, 0.21589E+00_JPRB ,&
&  0.19702E+00_JPRB, 0.18114E+00_JPRB, 0.17267E+00_JPRB, 0.16609E+00_JPRB, 0.16087E+00_JPRB, 0.15824E+00_JPRB, 0.15597E+00_JPRB ,&
&  0.15413E+00_JPRB, 0.15369E+00_JPRB, 0.15256E+00_JPRB, 0.15235E+00_JPRB, 0.15168E+00_JPRB, 0.15184E+00_JPRB, 0.15077E+00_JPRB ,&
&  0.14964E+00_JPRB, 0.14872E+00_JPRB, 0.14815E+00_JPRB, 0.14787E+00_JPRB, 0.14694E+00_JPRB, 0.14604E+00_JPRB, 0.14389E+00_JPRB ,&
&  0.14245E+00_JPRB, 0.14106E+00_JPRB, 0.14007E+00_JPRB, 0.14034E+00_JPRB, 0.14114E+00_JPRB, 0.14348E+00_JPRB, 0.14479E+00_JPRB ,&
&  0.14506E+00_JPRB, 0.14519E+00_JPRB, 0.14517E+00_JPRB, 0.14601E+00_JPRB, 0.14724E+00_JPRB, 0.14935E+00_JPRB, 0.15232E+00_JPRB ,&
&  0.15590E+00_JPRB, 0.16144E+00_JPRB, 0.16688E+00_JPRB, 0.17283E+00_JPRB, 0.17826E+00_JPRB, 0.18333E+00_JPRB, 0.19029E+00_JPRB ,&
&  0.19731E+00_JPRB, 0.20582E+00_JPRB, 0.21865E+00_JPRB, 0.23202E+00_JPRB, 0.25630E+00_JPRB, 0.27567E+00_JPRB, 0.30563E+00_JPRB ,&
&  0.37446E+00_JPRB, 0.51948E+00_JPRB, 0.60564E+00_JPRB, 0.61931E+00_JPRB, 0.65287E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.50723E+00_JPRB, 0.51891E+00_JPRB, 0.53403E+00_JPRB, 0.54317E+00_JPRB, 0.55374E+00_JPRB, 0.52125E+00_JPRB, 0.49556E+00_JPRB ,&
&  0.44274E+00_JPRB, 0.41234E+00_JPRB, 0.38700E+00_JPRB, 0.36373E+00_JPRB, 0.34296E+00_JPRB, 0.32397E+00_JPRB, 0.30653E+00_JPRB ,&
&  0.29538E+00_JPRB, 0.28877E+00_JPRB, 0.27989E+00_JPRB, 0.26973E+00_JPRB, 0.25881E+00_JPRB, 0.24925E+00_JPRB, 0.24457E+00_JPRB ,&
&  0.23973E+00_JPRB, 0.23629E+00_JPRB, 0.23485E+00_JPRB, 0.23462E+00_JPRB, 0.23635E+00_JPRB, 0.23630E+00_JPRB, 0.23673E+00_JPRB ,&
&  0.23301E+00_JPRB, 0.24033E+00_JPRB, 0.25007E+00_JPRB, 0.25455E+00_JPRB, 0.26304E+00_JPRB, 0.26191E+00_JPRB, 0.25731E+00_JPRB ,&
&  0.24985E+00_JPRB, 0.24137E+00_JPRB, 0.23151E+00_JPRB, 0.21930E+00_JPRB, 0.20891E+00_JPRB, 0.20139E+00_JPRB, 0.19391E+00_JPRB ,&
&  0.18955E+00_JPRB, 0.18388E+00_JPRB, 0.17686E+00_JPRB, 0.17072E+00_JPRB, 0.16539E+00_JPRB, 0.16113E+00_JPRB, 0.15719E+00_JPRB ,&
&  0.15449E+00_JPRB, 0.15302E+00_JPRB, 0.15164E+00_JPRB, 0.15157E+00_JPRB, 0.15062E+00_JPRB, 0.14958E+00_JPRB, 0.14852E+00_JPRB ,&
&  0.14800E+00_JPRB, 0.14674E+00_JPRB, 0.14570E+00_JPRB, 0.14479E+00_JPRB, 0.14331E+00_JPRB, 0.14203E+00_JPRB, 0.14056E+00_JPRB ,&
&  0.13934E+00_JPRB, 0.13877E+00_JPRB, 0.13847E+00_JPRB, 0.13848E+00_JPRB, 0.13890E+00_JPRB, 0.13892E+00_JPRB, 0.13928E+00_JPRB ,&
&  0.13970E+00_JPRB, 0.14028E+00_JPRB, 0.14029E+00_JPRB, 0.14070E+00_JPRB, 0.14170E+00_JPRB, 0.14307E+00_JPRB, 0.14637E+00_JPRB ,&
&  0.15011E+00_JPRB, 0.15487E+00_JPRB, 0.15998E+00_JPRB, 0.16560E+00_JPRB, 0.17117E+00_JPRB, 0.17656E+00_JPRB, 0.18256E+00_JPRB ,&
&  0.19034E+00_JPRB, 0.19741E+00_JPRB, 0.20585E+00_JPRB, 0.21880E+00_JPRB, 0.24792E+00_JPRB, 0.26316E+00_JPRB, 0.27731E+00_JPRB ,&
&  0.30237E+00_JPRB, 0.33499E+00_JPRB, 0.43864E+00_JPRB, 0.47081E+00_JPRB, 0.48815E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.25162E+00_JPRB, 0.25132E+00_JPRB, 0.25159E+00_JPRB, 0.25744E+00_JPRB, 0.26642E+00_JPRB, 0.34387E+00_JPRB, 0.33426E+00_JPRB ,&
&  0.32648E+00_JPRB, 0.29872E+00_JPRB, 0.29557E+00_JPRB, 0.28875E+00_JPRB, 0.27939E+00_JPRB, 0.27212E+00_JPRB, 0.26796E+00_JPRB ,&
&  0.26332E+00_JPRB, 0.25810E+00_JPRB, 0.25543E+00_JPRB, 0.25235E+00_JPRB, 0.24927E+00_JPRB, 0.24563E+00_JPRB, 0.24279E+00_JPRB ,&
&  0.24001E+00_JPRB, 0.23804E+00_JPRB, 0.23593E+00_JPRB, 0.23156E+00_JPRB, 0.22970E+00_JPRB, 0.22618E+00_JPRB, 0.22100E+00_JPRB ,&
&  0.21420E+00_JPRB, 0.21071E+00_JPRB, 0.20686E+00_JPRB, 0.20347E+00_JPRB, 0.20695E+00_JPRB, 0.20626E+00_JPRB, 0.20425E+00_JPRB ,&
&  0.20063E+00_JPRB, 0.19689E+00_JPRB, 0.19196E+00_JPRB, 0.18720E+00_JPRB, 0.18331E+00_JPRB, 0.18014E+00_JPRB, 0.17744E+00_JPRB ,&
&  0.17532E+00_JPRB, 0.17356E+00_JPRB, 0.17096E+00_JPRB, 0.16861E+00_JPRB, 0.16612E+00_JPRB, 0.16394E+00_JPRB, 0.16162E+00_JPRB ,&
&  0.15983E+00_JPRB, 0.15815E+00_JPRB, 0.15639E+00_JPRB, 0.15460E+00_JPRB, 0.15320E+00_JPRB, 0.15134E+00_JPRB, 0.14996E+00_JPRB ,&
&  0.14878E+00_JPRB, 0.14722E+00_JPRB, 0.14607E+00_JPRB, 0.14467E+00_JPRB, 0.14364E+00_JPRB, 0.14278E+00_JPRB, 0.14195E+00_JPRB ,&
&  0.14084E+00_JPRB, 0.14009E+00_JPRB, 0.13983E+00_JPRB, 0.13965E+00_JPRB, 0.13936E+00_JPRB, 0.13945E+00_JPRB, 0.13985E+00_JPRB ,&
&  0.14040E+00_JPRB, 0.14107E+00_JPRB, 0.14209E+00_JPRB, 0.14327E+00_JPRB, 0.14501E+00_JPRB, 0.14734E+00_JPRB, 0.15100E+00_JPRB ,&
&  0.15494E+00_JPRB, 0.15928E+00_JPRB, 0.16362E+00_JPRB, 0.16819E+00_JPRB, 0.17289E+00_JPRB, 0.17759E+00_JPRB, 0.18303E+00_JPRB ,&
&  0.18889E+00_JPRB, 0.19349E+00_JPRB, 0.19789E+00_JPRB, 0.20258E+00_JPRB, 0.21289E+00_JPRB, 0.22052E+00_JPRB, 0.22620E+00_JPRB ,&
&  0.23263E+00_JPRB, 0.24153E+00_JPRB, 0.24612E+00_JPRB, 0.24909E+00_JPRB, 0.25055E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.22268E+00_JPRB, 0.22554E+00_JPRB, 0.22747E+00_JPRB, 0.22869E+00_JPRB, 0.22838E+00_JPRB, 0.22928E+00_JPRB, 0.23021E+00_JPRB ,&
&  0.23146E+00_JPRB, 0.23306E+00_JPRB, 0.23424E+00_JPRB, 0.23528E+00_JPRB, 0.23574E+00_JPRB, 0.23574E+00_JPRB, 0.23574E+00_JPRB ,&
&  0.23550E+00_JPRB, 0.23523E+00_JPRB, 0.23121E+00_JPRB, 0.22760E+00_JPRB, 0.22495E+00_JPRB, 0.22337E+00_JPRB, 0.22202E+00_JPRB ,&
&  0.22094E+00_JPRB, 0.21961E+00_JPRB, 0.21813E+00_JPRB, 0.21601E+00_JPRB, 0.21265E+00_JPRB, 0.20937E+00_JPRB, 0.20598E+00_JPRB ,&
&  0.20293E+00_JPRB, 0.20051E+00_JPRB, 0.19828E+00_JPRB, 0.19606E+00_JPRB, 0.19482E+00_JPRB, 0.19333E+00_JPRB, 0.19176E+00_JPRB ,&
&  0.18971E+00_JPRB, 0.18746E+00_JPRB, 0.18532E+00_JPRB, 0.18314E+00_JPRB, 0.18103E+00_JPRB, 0.17896E+00_JPRB, 0.17744E+00_JPRB ,&
&  0.17587E+00_JPRB, 0.17470E+00_JPRB, 0.17340E+00_JPRB, 0.17223E+00_JPRB, 0.17098E+00_JPRB, 0.16980E+00_JPRB, 0.16838E+00_JPRB ,&
&  0.16741E+00_JPRB, 0.16640E+00_JPRB, 0.16537E+00_JPRB, 0.16442E+00_JPRB, 0.16347E+00_JPRB, 0.16255E+00_JPRB, 0.16151E+00_JPRB ,&
&  0.16054E+00_JPRB, 0.15993E+00_JPRB, 0.15928E+00_JPRB, 0.15856E+00_JPRB, 0.15791E+00_JPRB, 0.15746E+00_JPRB, 0.15707E+00_JPRB ,&
&  0.15673E+00_JPRB, 0.15672E+00_JPRB, 0.15690E+00_JPRB, 0.15709E+00_JPRB, 0.15729E+00_JPRB, 0.15772E+00_JPRB, 0.15835E+00_JPRB ,&
&  0.15909E+00_JPRB, 0.15990E+00_JPRB, 0.16091E+00_JPRB, 0.16236E+00_JPRB, 0.16396E+00_JPRB, 0.16579E+00_JPRB, 0.16778E+00_JPRB ,&
&  0.16994E+00_JPRB, 0.17222E+00_JPRB, 0.17461E+00_JPRB, 0.17686E+00_JPRB, 0.17935E+00_JPRB, 0.18180E+00_JPRB, 0.18419E+00_JPRB ,&
&  0.18669E+00_JPRB, 0.18919E+00_JPRB, 0.19179E+00_JPRB, 0.19420E+00_JPRB, 0.19803E+00_JPRB, 0.20053E+00_JPRB, 0.20310E+00_JPRB ,&
&  0.20543E+00_JPRB, 0.20815E+00_JPRB, 0.21089E+00_JPRB, 0.21425E+00_JPRB, 0.21761E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB, 0.18550E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB, 0.62335E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.67093E+00_JPRB, 0.66854E+00_JPRB, 0.66586E+00_JPRB, 0.66345E+00_JPRB, 0.66051E+00_JPRB, 0.65800E+00_JPRB, 0.65539E+00_JPRB ,&
&  0.65304E+00_JPRB, 0.65052E+00_JPRB, 0.64793E+00_JPRB, 0.64506E+00_JPRB, 0.64227E+00_JPRB, 0.63956E+00_JPRB, 0.63695E+00_JPRB ,&
&  0.63433E+00_JPRB, 0.63186E+00_JPRB, 0.62931E+00_JPRB, 0.62650E+00_JPRB, 0.62409E+00_JPRB, 0.62128E+00_JPRB, 0.61773E+00_JPRB ,&
&  0.61252E+00_JPRB, 0.60705E+00_JPRB, 0.60214E+00_JPRB, 0.59867E+00_JPRB, 0.59605E+00_JPRB, 0.59389E+00_JPRB, 0.59196E+00_JPRB ,&
&  0.59065E+00_JPRB, 0.58991E+00_JPRB, 0.58942E+00_JPRB, 0.58862E+00_JPRB, 0.58834E+00_JPRB, 0.58926E+00_JPRB, 0.58969E+00_JPRB ,&
&  0.58970E+00_JPRB, 0.58981E+00_JPRB, 0.59074E+00_JPRB, 0.59148E+00_JPRB, 0.59214E+00_JPRB, 0.59314E+00_JPRB, 0.59516E+00_JPRB ,&
&  0.59718E+00_JPRB, 0.59899E+00_JPRB, 0.60110E+00_JPRB, 0.60341E+00_JPRB, 0.60577E+00_JPRB, 0.60813E+00_JPRB, 0.61134E+00_JPRB ,&
&  0.61489E+00_JPRB, 0.61804E+00_JPRB, 0.62088E+00_JPRB, 0.62329E+00_JPRB, 0.62531E+00_JPRB, 0.62643E+00_JPRB, 0.62719E+00_JPRB ,&
&  0.62725E+00_JPRB, 0.62828E+00_JPRB, 0.62856E+00_JPRB, 0.62894E+00_JPRB, 0.62866E+00_JPRB, 0.62963E+00_JPRB, 0.63023E+00_JPRB ,&
&  0.63188E+00_JPRB, 0.63297E+00_JPRB, 0.63492E+00_JPRB, 0.63682E+00_JPRB, 0.63974E+00_JPRB, 0.64299E+00_JPRB, 0.64652E+00_JPRB ,&
&  0.65042E+00_JPRB, 0.65487E+00_JPRB, 0.65993E+00_JPRB, 0.66305E+00_JPRB, 0.66616E+00_JPRB, 0.66925E+00_JPRB, 0.67256E+00_JPRB ,&
&  0.67509E+00_JPRB, 0.67745E+00_JPRB, 0.67987E+00_JPRB, 0.68184E+00_JPRB, 0.68304E+00_JPRB, 0.68402E+00_JPRB, 0.68498E+00_JPRB ,&
&  0.68550E+00_JPRB, 0.68593E+00_JPRB, 0.68615E+00_JPRB, 0.68643E+00_JPRB, 0.68567E+00_JPRB, 0.68434E+00_JPRB, 0.68273E+00_JPRB ,&
&  0.68140E+00_JPRB, 0.67944E+00_JPRB, 0.67752E+00_JPRB, 0.67546E+00_JPRB, 0.67346E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.75908E+00_JPRB, 0.75499E+00_JPRB, 0.75066E+00_JPRB, 0.74679E+00_JPRB, 0.74108E+00_JPRB, 0.73530E+00_JPRB, 0.72927E+00_JPRB ,&
&  0.72282E+00_JPRB, 0.71461E+00_JPRB, 0.70558E+00_JPRB, 0.69594E+00_JPRB, 0.68624E+00_JPRB, 0.67785E+00_JPRB, 0.67110E+00_JPRB ,&
&  0.66462E+00_JPRB, 0.65793E+00_JPRB, 0.65189E+00_JPRB, 0.64424E+00_JPRB, 0.63655E+00_JPRB, 0.62960E+00_JPRB, 0.62445E+00_JPRB ,&
&  0.61696E+00_JPRB, 0.60865E+00_JPRB, 0.60000E+00_JPRB, 0.59083E+00_JPRB, 0.58223E+00_JPRB, 0.57369E+00_JPRB, 0.56528E+00_JPRB ,&
&  0.55746E+00_JPRB, 0.55159E+00_JPRB, 0.54631E+00_JPRB, 0.54086E+00_JPRB, 0.53535E+00_JPRB, 0.53370E+00_JPRB, 0.53192E+00_JPRB ,&
&  0.53030E+00_JPRB, 0.52989E+00_JPRB, 0.53345E+00_JPRB, 0.53661E+00_JPRB, 0.53992E+00_JPRB, 0.54339E+00_JPRB, 0.54941E+00_JPRB ,&
&  0.55505E+00_JPRB, 0.56169E+00_JPRB, 0.57014E+00_JPRB, 0.57859E+00_JPRB, 0.58550E+00_JPRB, 0.59060E+00_JPRB, 0.59746E+00_JPRB ,&
&  0.60233E+00_JPRB, 0.60891E+00_JPRB, 0.61354E+00_JPRB, 0.61582E+00_JPRB, 0.61556E+00_JPRB, 0.61552E+00_JPRB, 0.61663E+00_JPRB ,&
&  0.61696E+00_JPRB, 0.62065E+00_JPRB, 0.62432E+00_JPRB, 0.62818E+00_JPRB, 0.62989E+00_JPRB, 0.63483E+00_JPRB, 0.63855E+00_JPRB ,&
&  0.64518E+00_JPRB, 0.65475E+00_JPRB, 0.66449E+00_JPRB, 0.67507E+00_JPRB, 0.68797E+00_JPRB, 0.70286E+00_JPRB, 0.71503E+00_JPRB ,&
&  0.72654E+00_JPRB, 0.73662E+00_JPRB, 0.74508E+00_JPRB, 0.75009E+00_JPRB, 0.75476E+00_JPRB, 0.75927E+00_JPRB, 0.76391E+00_JPRB ,&
&  0.76597E+00_JPRB, 0.76841E+00_JPRB, 0.77110E+00_JPRB, 0.77326E+00_JPRB, 0.77302E+00_JPRB, 0.77259E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.77329E+00_JPRB, 0.77334E+00_JPRB, 0.77375E+00_JPRB, 0.77493E+00_JPRB, 0.77538E+00_JPRB, 0.77488E+00_JPRB, 0.77419E+00_JPRB ,&
&  0.77364E+00_JPRB, 0.77159E+00_JPRB, 0.76867E+00_JPRB, 0.76571E+00_JPRB, 0.76313E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.79855E+00_JPRB, 0.79703E+00_JPRB, 0.79542E+00_JPRB, 0.79368E+00_JPRB, 0.78996E+00_JPRB, 0.78583E+00_JPRB, 0.78044E+00_JPRB ,&
&  0.77415E+00_JPRB, 0.76506E+00_JPRB, 0.75528E+00_JPRB, 0.74442E+00_JPRB, 0.73385E+00_JPRB, 0.72427E+00_JPRB, 0.71502E+00_JPRB ,&
&  0.70570E+00_JPRB, 0.69409E+00_JPRB, 0.68039E+00_JPRB, 0.66252E+00_JPRB, 0.64652E+00_JPRB, 0.63335E+00_JPRB, 0.61939E+00_JPRB ,&
&  0.59945E+00_JPRB, 0.57802E+00_JPRB, 0.55902E+00_JPRB, 0.53991E+00_JPRB, 0.52612E+00_JPRB, 0.49152E+00_JPRB, 0.48042E+00_JPRB ,&
&  0.47263E+00_JPRB, 0.46429E+00_JPRB, 0.46010E+00_JPRB, 0.45653E+00_JPRB, 0.45601E+00_JPRB, 0.44969E+00_JPRB, 0.44621E+00_JPRB ,&
&  0.43087E+00_JPRB, 0.43398E+00_JPRB, 0.43820E+00_JPRB, 0.44332E+00_JPRB, 0.45352E+00_JPRB, 0.46469E+00_JPRB, 0.48233E+00_JPRB ,&
&  0.51425E+00_JPRB, 0.52816E+00_JPRB, 0.54926E+00_JPRB, 0.56562E+00_JPRB, 0.57589E+00_JPRB, 0.58158E+00_JPRB, 0.58858E+00_JPRB ,&
&  0.59595E+00_JPRB, 0.60739E+00_JPRB, 0.61408E+00_JPRB, 0.61852E+00_JPRB, 0.61841E+00_JPRB, 0.61694E+00_JPRB, 0.61686E+00_JPRB ,&
&  0.61610E+00_JPRB, 0.61981E+00_JPRB, 0.62648E+00_JPRB, 0.63983E+00_JPRB, 0.65866E+00_JPRB, 0.68235E+00_JPRB, 0.70629E+00_JPRB ,&
&  0.72909E+00_JPRB, 0.75285E+00_JPRB, 0.77222E+00_JPRB, 0.79068E+00_JPRB, 0.81067E+00_JPRB, 0.82709E+00_JPRB, 0.84102E+00_JPRB ,&
&  0.85034E+00_JPRB, 0.85446E+00_JPRB, 0.85229E+00_JPRB, 0.84897E+00_JPRB, 0.84349E+00_JPRB, 0.83916E+00_JPRB, 0.83835E+00_JPRB ,&
&  0.83787E+00_JPRB, 0.83900E+00_JPRB, 0.84087E+00_JPRB, 0.83926E+00_JPRB, 0.83698E+00_JPRB, 0.83437E+00_JPRB, 0.83231E+00_JPRB ,&
&  0.82969E+00_JPRB, 0.82750E+00_JPRB, 0.82551E+00_JPRB, 0.82334E+00_JPRB, 0.82117E+00_JPRB, 0.81714E+00_JPRB, 0.81354E+00_JPRB ,&
&  0.80967E+00_JPRB, 0.80794E+00_JPRB, 0.80490E+00_JPRB, 0.80244E+00_JPRB, 0.80006E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.74905E+00_JPRB, 0.74865E+00_JPRB, 0.74994E+00_JPRB, 0.75028E+00_JPRB, 0.75240E+00_JPRB, 0.75362E+00_JPRB, 0.75559E+00_JPRB ,&
&  0.75662E+00_JPRB, 0.75761E+00_JPRB, 0.75679E+00_JPRB, 0.75478E+00_JPRB, 0.75088E+00_JPRB, 0.74660E+00_JPRB, 0.74135E+00_JPRB ,&
&  0.73408E+00_JPRB, 0.72056E+00_JPRB, 0.70135E+00_JPRB, 0.67967E+00_JPRB, 0.65488E+00_JPRB, 0.62784E+00_JPRB, 0.59864E+00_JPRB ,&
&  0.56940E+00_JPRB, 0.54328E+00_JPRB, 0.51023E+00_JPRB, 0.49554E+00_JPRB, 0.47489E+00_JPRB, 0.45910E+00_JPRB, 0.44759E+00_JPRB ,&
&  0.43361E+00_JPRB, 0.42334E+00_JPRB, 0.41241E+00_JPRB, 0.40165E+00_JPRB, 0.39567E+00_JPRB, 0.38551E+00_JPRB, 0.38097E+00_JPRB ,&
&  0.37783E+00_JPRB, 0.37639E+00_JPRB, 0.37691E+00_JPRB, 0.37978E+00_JPRB, 0.38630E+00_JPRB, 0.39481E+00_JPRB, 0.41036E+00_JPRB ,&
&  0.44423E+00_JPRB, 0.49148E+00_JPRB, 0.56357E+00_JPRB, 0.57809E+00_JPRB, 0.57830E+00_JPRB, 0.58940E+00_JPRB, 0.60198E+00_JPRB ,&
&  0.61446E+00_JPRB, 0.62664E+00_JPRB, 0.63660E+00_JPRB, 0.64070E+00_JPRB, 0.64608E+00_JPRB, 0.65091E+00_JPRB, 0.65663E+00_JPRB ,&
&  0.66290E+00_JPRB, 0.67651E+00_JPRB, 0.69607E+00_JPRB, 0.71937E+00_JPRB, 0.74449E+00_JPRB, 0.76753E+00_JPRB, 0.78697E+00_JPRB ,&
&  0.80246E+00_JPRB, 0.81778E+00_JPRB, 0.83940E+00_JPRB, 0.86219E+00_JPRB, 0.88606E+00_JPRB, 0.89988E+00_JPRB, 0.91148E+00_JPRB ,&
&  0.91888E+00_JPRB, 0.91993E+00_JPRB, 0.91260E+00_JPRB, 0.90300E+00_JPRB, 0.89584E+00_JPRB, 0.90041E+00_JPRB, 0.89856E+00_JPRB ,&
&  0.89766E+00_JPRB, 0.89031E+00_JPRB, 0.87753E+00_JPRB, 0.86448E+00_JPRB, 0.85593E+00_JPRB, 0.84851E+00_JPRB, 0.84363E+00_JPRB ,&
&  0.83786E+00_JPRB, 0.83087E+00_JPRB, 0.82227E+00_JPRB, 0.81100E+00_JPRB, 0.79932E+00_JPRB, 0.78640E+00_JPRB, 0.77611E+00_JPRB ,&
&  0.76653E+00_JPRB, 0.76063E+00_JPRB, 0.75530E+00_JPRB, 0.75236E+00_JPRB, 0.74952E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.71528E+00_JPRB, 0.72160E+00_JPRB, 0.72852E+00_JPRB, 0.73302E+00_JPRB, 0.73648E+00_JPRB, 0.73882E+00_JPRB, 0.73973E+00_JPRB ,&
&  0.73996E+00_JPRB, 0.74024E+00_JPRB, 0.74088E+00_JPRB, 0.74139E+00_JPRB, 0.74201E+00_JPRB, 0.74287E+00_JPRB, 0.74286E+00_JPRB ,&
&  0.74206E+00_JPRB, 0.73243E+00_JPRB, 0.71353E+00_JPRB, 0.69201E+00_JPRB, 0.66818E+00_JPRB, 0.63709E+00_JPRB, 0.60023E+00_JPRB ,&
&  0.56819E+00_JPRB, 0.54500E+00_JPRB, 0.51967E+00_JPRB, 0.50250E+00_JPRB, 0.48481E+00_JPRB, 0.47470E+00_JPRB, 0.46053E+00_JPRB ,&
&  0.45511E+00_JPRB, 0.42260E+00_JPRB, 0.41827E+00_JPRB, 0.39732E+00_JPRB, 0.38234E+00_JPRB, 0.37014E+00_JPRB, 0.36125E+00_JPRB ,&
&  0.35907E+00_JPRB, 0.35562E+00_JPRB, 0.35565E+00_JPRB, 0.35996E+00_JPRB, 0.36545E+00_JPRB, 0.37728E+00_JPRB, 0.39077E+00_JPRB ,&
&  0.42090E+00_JPRB, 0.51913E+00_JPRB, 0.55622E+00_JPRB, 0.56752E+00_JPRB, 0.57419E+00_JPRB, 0.59133E+00_JPRB, 0.60859E+00_JPRB ,&
&  0.63297E+00_JPRB, 0.65005E+00_JPRB, 0.66744E+00_JPRB, 0.67786E+00_JPRB, 0.70164E+00_JPRB, 0.72259E+00_JPRB, 0.74067E+00_JPRB ,&
&  0.75910E+00_JPRB, 0.77612E+00_JPRB, 0.79457E+00_JPRB, 0.80699E+00_JPRB, 0.80882E+00_JPRB, 0.81094E+00_JPRB, 0.81219E+00_JPRB ,&
&  0.81941E+00_JPRB, 0.82750E+00_JPRB, 0.85481E+00_JPRB, 0.89074E+00_JPRB, 0.93063E+00_JPRB, 0.95756E+00_JPRB, 0.97193E+00_JPRB ,&
&  0.98302E+00_JPRB, 0.98962E+00_JPRB, 0.99891E+00_JPRB, 0.10114E+01_JPRB, 0.10017E+01_JPRB, 0.98184E+00_JPRB, 0.97112E+00_JPRB ,&
&  0.95940E+00_JPRB, 0.93274E+00_JPRB, 0.91252E+00_JPRB, 0.89783E+00_JPRB, 0.88517E+00_JPRB, 0.87473E+00_JPRB, 0.86172E+00_JPRB ,&
&  0.84426E+00_JPRB, 0.82438E+00_JPRB, 0.80270E+00_JPRB, 0.77719E+00_JPRB, 0.75534E+00_JPRB, 0.73294E+00_JPRB, 0.71939E+00_JPRB ,&
&  0.70819E+00_JPRB, 0.70501E+00_JPRB, 0.70198E+00_JPRB, 0.70532E+00_JPRB, 0.70778E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.68978E+00_JPRB, 0.70413E+00_JPRB, 0.71418E+00_JPRB, 0.71830E+00_JPRB, 0.72137E+00_JPRB, 0.72559E+00_JPRB, 0.72709E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.72838E+00_JPRB, 0.72957E+00_JPRB, 0.72993E+00_JPRB, 0.73188E+00_JPRB, 0.73234E+00_JPRB, 0.73185E+00_JPRB ,&
&  0.73054E+00_JPRB, 0.72724E+00_JPRB, 0.71755E+00_JPRB, 0.69985E+00_JPRB, 0.67678E+00_JPRB, 0.65063E+00_JPRB, 0.61410E+00_JPRB ,&
&  0.57723E+00_JPRB, 0.55210E+00_JPRB, 0.52944E+00_JPRB, 0.51219E+00_JPRB, 0.50225E+00_JPRB, 0.48949E+00_JPRB, 0.48673E+00_JPRB ,&
&  0.48407E+00_JPRB, 0.47219E+00_JPRB, 0.45627E+00_JPRB, 0.43826E+00_JPRB, 0.42011E+00_JPRB, 0.37800E+00_JPRB, 0.37353E+00_JPRB ,&
&  0.36338E+00_JPRB, 0.35369E+00_JPRB, 0.35126E+00_JPRB, 0.35275E+00_JPRB, 0.35708E+00_JPRB, 0.37039E+00_JPRB, 0.38767E+00_JPRB ,&
&  0.41144E+00_JPRB, 0.52677E+00_JPRB, 0.55465E+00_JPRB, 0.56866E+00_JPRB, 0.59196E+00_JPRB, 0.61609E+00_JPRB, 0.63559E+00_JPRB ,&
&  0.65574E+00_JPRB, 0.67480E+00_JPRB, 0.70174E+00_JPRB, 0.72782E+00_JPRB, 0.75541E+00_JPRB, 0.77926E+00_JPRB, 0.79444E+00_JPRB ,&
&  0.80855E+00_JPRB, 0.81298E+00_JPRB, 0.84365E+00_JPRB, 0.85441E+00_JPRB, 0.84839E+00_JPRB, 0.87438E+00_JPRB, 0.89077E+00_JPRB ,&
&  0.88914E+00_JPRB, 0.87959E+00_JPRB, 0.89966E+00_JPRB, 0.94834E+00_JPRB, 0.10178E+01_JPRB, 0.10720E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.10876E+01_JPRB, 0.10763E+01_JPRB, 0.10723E+01_JPRB, 0.10426E+01_JPRB, 0.10032E+01_JPRB, 0.97672E+00_JPRB ,&
&  0.95354E+00_JPRB, 0.95311E+00_JPRB, 0.94451E+00_JPRB, 0.93642E+00_JPRB, 0.92009E+00_JPRB, 0.90350E+00_JPRB, 0.87657E+00_JPRB ,&
&  0.83634E+00_JPRB, 0.79105E+00_JPRB, 0.75491E+00_JPRB, 0.72494E+00_JPRB, 0.70677E+00_JPRB, 0.68761E+00_JPRB, 0.67757E+00_JPRB ,&
&  0.66797E+00_JPRB, 0.66552E+00_JPRB, 0.66336E+00_JPRB, 0.66826E+00_JPRB, 0.67593E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.68144E+00_JPRB, 0.69962E+00_JPRB, 0.71639E+00_JPRB, 0.72713E+00_JPRB, 0.72950E+00_JPRB, 0.73081E+00_JPRB, 0.72982E+00_JPRB ,&
&  0.73024E+00_JPRB, 0.73077E+00_JPRB, 0.73043E+00_JPRB, 0.72700E+00_JPRB, 0.72339E+00_JPRB, 0.71837E+00_JPRB, 0.71581E+00_JPRB ,&
&  0.71228E+00_JPRB, 0.70145E+00_JPRB, 0.69396E+00_JPRB, 0.68580E+00_JPRB, 0.67132E+00_JPRB, 0.65143E+00_JPRB, 0.62285E+00_JPRB ,&
&  0.58802E+00_JPRB, 0.55944E+00_JPRB, 0.53903E+00_JPRB, 0.52135E+00_JPRB, 0.50837E+00_JPRB, 0.50071E+00_JPRB, 0.49656E+00_JPRB ,&
&  0.49578E+00_JPRB, 0.48894E+00_JPRB, 0.48433E+00_JPRB, 0.46085E+00_JPRB, 0.43127E+00_JPRB, 0.42125E+00_JPRB, 0.39809E+00_JPRB ,&
&  0.37542E+00_JPRB, 0.35790E+00_JPRB, 0.35380E+00_JPRB, 0.35424E+00_JPRB, 0.36124E+00_JPRB, 0.37659E+00_JPRB, 0.41038E+00_JPRB ,&
&  0.46596E+00_JPRB, 0.56083E+00_JPRB, 0.58331E+00_JPRB, 0.60817E+00_JPRB, 0.63319E+00_JPRB, 0.65725E+00_JPRB, 0.67383E+00_JPRB ,&
&  0.68974E+00_JPRB, 0.70599E+00_JPRB, 0.73713E+00_JPRB, 0.76976E+00_JPRB, 0.81719E+00_JPRB, 0.85927E+00_JPRB, 0.89388E+00_JPRB ,&
&  0.92562E+00_JPRB, 0.89432E+00_JPRB, 0.90369E+00_JPRB, 0.91367E+00_JPRB, 0.95139E+00_JPRB, 0.98666E+00_JPRB, 0.97981E+00_JPRB ,&
&  0.97885E+00_JPRB, 0.10181E+01_JPRB, 0.10234E+01_JPRB, 0.10507E+01_JPRB, 0.11118E+01_JPRB, 0.11789E+01_JPRB, 0.12046E+01_JPRB ,&
&  0.11795E+01_JPRB, 0.11754E+01_JPRB, 0.12241E+01_JPRB, 0.11708E+01_JPRB, 0.11362E+01_JPRB, 0.10837E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.98622E+00_JPRB, 0.96941E+00_JPRB, 0.96599E+00_JPRB, 0.96328E+00_JPRB, 0.94616E+00_JPRB, 0.91885E+00_JPRB, 0.86840E+00_JPRB ,&
&  0.80123E+00_JPRB, 0.75089E+00_JPRB, 0.71592E+00_JPRB, 0.69547E+00_JPRB, 0.68023E+00_JPRB, 0.66391E+00_JPRB, 0.64921E+00_JPRB ,&
&  0.63612E+00_JPRB, 0.63208E+00_JPRB, 0.63789E+00_JPRB, 0.65348E+00_JPRB, 0.66898E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.69988E+00_JPRB, 0.72050E+00_JPRB, 0.73297E+00_JPRB, 0.73054E+00_JPRB, 0.71617E+00_JPRB, 0.69534E+00_JPRB, 0.68061E+00_JPRB ,&
&  0.67610E+00_JPRB, 0.67816E+00_JPRB, 0.68190E+00_JPRB, 0.67991E+00_JPRB, 0.67936E+00_JPRB, 0.67986E+00_JPRB, 0.68239E+00_JPRB ,&
&  0.68404E+00_JPRB, 0.68556E+00_JPRB, 0.68626E+00_JPRB, 0.67817E+00_JPRB, 0.66508E+00_JPRB, 0.64703E+00_JPRB, 0.62472E+00_JPRB ,&
&  0.59822E+00_JPRB, 0.57462E+00_JPRB, 0.55422E+00_JPRB, 0.53893E+00_JPRB, 0.52471E+00_JPRB, 0.51801E+00_JPRB, 0.51414E+00_JPRB ,&
&  0.51122E+00_JPRB, 0.51068E+00_JPRB, 0.50445E+00_JPRB, 0.49156E+00_JPRB, 0.46650E+00_JPRB, 0.43877E+00_JPRB, 0.41188E+00_JPRB ,&
&  0.39027E+00_JPRB, 0.37731E+00_JPRB, 0.37034E+00_JPRB, 0.36942E+00_JPRB, 0.38028E+00_JPRB, 0.40400E+00_JPRB, 0.45580E+00_JPRB ,&
&  0.55593E+00_JPRB, 0.58972E+00_JPRB, 0.60812E+00_JPRB, 0.62782E+00_JPRB, 0.64676E+00_JPRB, 0.66369E+00_JPRB, 0.68103E+00_JPRB ,&
&  0.69969E+00_JPRB, 0.71790E+00_JPRB, 0.74467E+00_JPRB, 0.76675E+00_JPRB, 0.80150E+00_JPRB, 0.82252E+00_JPRB, 0.90769E+00_JPRB ,&
&  0.10752E+01_JPRB, 0.10865E+01_JPRB, 0.10114E+01_JPRB, 0.10258E+01_JPRB, 0.10706E+01_JPRB, 0.11148E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.11925E+01_JPRB, 0.11954E+01_JPRB, 0.11995E+01_JPRB, 0.12227E+01_JPRB, 0.12470E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.12381E+01_JPRB, 0.12591E+01_JPRB, 0.13120E+01_JPRB, 0.11199E+01_JPRB, 0.10341E+01_JPRB, 0.10111E+01_JPRB, 0.97592E+00_JPRB ,&
&  0.95358E+00_JPRB, 0.95190E+00_JPRB, 0.97127E+00_JPRB, 0.98288E+00_JPRB, 0.93294E+00_JPRB, 0.86247E+00_JPRB, 0.78532E+00_JPRB ,&
&  0.73992E+00_JPRB, 0.71173E+00_JPRB, 0.68736E+00_JPRB, 0.67698E+00_JPRB, 0.66797E+00_JPRB, 0.65264E+00_JPRB, 0.62730E+00_JPRB ,&
&  0.62192E+00_JPRB, 0.63139E+00_JPRB, 0.64486E+00_JPRB, 0.67087E+00_JPRB, 0.68763E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.69051E+00_JPRB, 0.72511E+00_JPRB, 0.73032E+00_JPRB, 0.70832E+00_JPRB, 0.68488E+00_JPRB, 0.64462E+00_JPRB, 0.60981E+00_JPRB ,&
&  0.59463E+00_JPRB, 0.58843E+00_JPRB, 0.58883E+00_JPRB, 0.59083E+00_JPRB, 0.59637E+00_JPRB, 0.60603E+00_JPRB, 0.61624E+00_JPRB ,&
&  0.62179E+00_JPRB, 0.62563E+00_JPRB, 0.63039E+00_JPRB, 0.63684E+00_JPRB, 0.63684E+00_JPRB, 0.62516E+00_JPRB, 0.60979E+00_JPRB ,&
&  0.58722E+00_JPRB, 0.56918E+00_JPRB, 0.55513E+00_JPRB, 0.54491E+00_JPRB, 0.53972E+00_JPRB, 0.53991E+00_JPRB, 0.54261E+00_JPRB ,&
&  0.54078E+00_JPRB, 0.53783E+00_JPRB, 0.53555E+00_JPRB, 0.52578E+00_JPRB, 0.50160E+00_JPRB, 0.46996E+00_JPRB, 0.43435E+00_JPRB ,&
&  0.41055E+00_JPRB, 0.40187E+00_JPRB, 0.40525E+00_JPRB, 0.41976E+00_JPRB, 0.46051E+00_JPRB, 0.53542E+00_JPRB, 0.56736E+00_JPRB ,&
&  0.59509E+00_JPRB, 0.61269E+00_JPRB, 0.62869E+00_JPRB, 0.64808E+00_JPRB, 0.66432E+00_JPRB, 0.68133E+00_JPRB, 0.69979E+00_JPRB ,&
&  0.72916E+00_JPRB, 0.76030E+00_JPRB, 0.80115E+00_JPRB, 0.74989E+00_JPRB, 0.76848E+00_JPRB, 0.89412E+00_JPRB, 0.98764E+00_JPRB ,&
&  0.10902E+01_JPRB, 0.11183E+01_JPRB, 0.11343E+01_JPRB, 0.12258E+01_JPRB, 0.12691E+01_JPRB, 0.13273E+01_JPRB, 0.13334E+01_JPRB ,&
&  0.13144E+01_JPRB, 0.12810E+01_JPRB, 0.12494E+01_JPRB, 0.12349E+01_JPRB, 0.12561E+01_JPRB, 0.12956E+01_JPRB, 0.12770E+01_JPRB ,&
&  0.12544E+01_JPRB, 0.12308E+01_JPRB, 0.11326E+01_JPRB, 0.10012E+01_JPRB, 0.95242E+00_JPRB, 0.94252E+00_JPRB, 0.95145E+00_JPRB ,&
&  0.94594E+00_JPRB, 0.95229E+00_JPRB, 0.96540E+00_JPRB, 0.97458E+00_JPRB, 0.89144E+00_JPRB, 0.80009E+00_JPRB, 0.71985E+00_JPRB ,&
&  0.67855E+00_JPRB, 0.66530E+00_JPRB, 0.65306E+00_JPRB, 0.65924E+00_JPRB, 0.65242E+00_JPRB, 0.63690E+00_JPRB, 0.62006E+00_JPRB ,&
&  0.61653E+00_JPRB, 0.61945E+00_JPRB, 0.63901E+00_JPRB, 0.66140E+00_JPRB, 0.67470E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.73512E+00_JPRB, 0.74367E+00_JPRB, 0.71601E+00_JPRB, 0.69553E+00_JPRB, 0.67263E+00_JPRB, 0.64811E+00_JPRB, 0.62136E+00_JPRB ,&
&  0.61343E+00_JPRB, 0.58769E+00_JPRB, 0.56674E+00_JPRB, 0.54922E+00_JPRB, 0.54085E+00_JPRB, 0.53831E+00_JPRB, 0.54830E+00_JPRB ,&
&  0.56893E+00_JPRB, 0.59445E+00_JPRB, 0.61032E+00_JPRB, 0.61112E+00_JPRB, 0.60748E+00_JPRB, 0.59493E+00_JPRB, 0.58597E+00_JPRB ,&
&  0.56972E+00_JPRB, 0.56043E+00_JPRB, 0.56172E+00_JPRB, 0.56984E+00_JPRB, 0.57901E+00_JPRB, 0.57943E+00_JPRB, 0.57021E+00_JPRB ,&
&  0.55765E+00_JPRB, 0.55057E+00_JPRB, 0.55160E+00_JPRB, 0.55099E+00_JPRB, 0.53434E+00_JPRB, 0.49585E+00_JPRB, 0.45800E+00_JPRB ,&
&  0.43330E+00_JPRB, 0.43114E+00_JPRB, 0.46352E+00_JPRB, 0.53759E+00_JPRB, 0.56968E+00_JPRB, 0.59104E+00_JPRB, 0.60506E+00_JPRB ,&
&  0.59992E+00_JPRB, 0.60645E+00_JPRB, 0.64022E+00_JPRB, 0.70186E+00_JPRB, 0.71561E+00_JPRB, 0.73545E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.79694E+00_JPRB, 0.80291E+00_JPRB, 0.83264E+00_JPRB, 0.82309E+00_JPRB, 0.96253E+00_JPRB, 0.12709E+01_JPRB, 0.12301E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.13103E+01_JPRB, 0.14082E+01_JPRB, 0.14445E+01_JPRB, 0.14438E+01_JPRB, 0.14497E+01_JPRB, 0.14247E+01_JPRB ,&
&  0.13551E+01_JPRB, 0.13158E+01_JPRB, 0.12726E+01_JPRB, 0.12796E+01_JPRB, 0.12820E+01_JPRB, 0.12930E+01_JPRB, 0.12880E+01_JPRB ,&
&  0.12745E+01_JPRB, 0.12467E+01_JPRB, 0.11629E+01_JPRB, 0.10559E+01_JPRB, 0.99955E+00_JPRB, 0.95088E+00_JPRB, 0.91959E+00_JPRB ,&
&  0.90053E+00_JPRB, 0.91627E+00_JPRB, 0.94922E+00_JPRB, 0.97580E+00_JPRB, 0.91354E+00_JPRB, 0.80211E+00_JPRB, 0.70334E+00_JPRB ,&
&  0.65027E+00_JPRB, 0.63114E+00_JPRB, 0.61012E+00_JPRB, 0.60602E+00_JPRB, 0.61087E+00_JPRB, 0.61303E+00_JPRB, 0.61415E+00_JPRB ,&
&  0.61748E+00_JPRB, 0.62811E+00_JPRB, 0.66239E+00_JPRB, 0.69890E+00_JPRB, 0.71905E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.73372E+00_JPRB, 0.74417E+00_JPRB, 0.72804E+00_JPRB, 0.71908E+00_JPRB, 0.70792E+00_JPRB, 0.68016E+00_JPRB, 0.63876E+00_JPRB ,&
&  0.63172E+00_JPRB, 0.58046E+00_JPRB, 0.55882E+00_JPRB, 0.54913E+00_JPRB, 0.54895E+00_JPRB, 0.54343E+00_JPRB, 0.53378E+00_JPRB ,&
&  0.53964E+00_JPRB, 0.56038E+00_JPRB, 0.59323E+00_JPRB, 0.60820E+00_JPRB, 0.61054E+00_JPRB, 0.59997E+00_JPRB, 0.58111E+00_JPRB ,&
&  0.56198E+00_JPRB, 0.55753E+00_JPRB, 0.57155E+00_JPRB, 0.59668E+00_JPRB, 0.60968E+00_JPRB, 0.60693E+00_JPRB, 0.59731E+00_JPRB ,&
&  0.58471E+00_JPRB, 0.57396E+00_JPRB, 0.56809E+00_JPRB, 0.56534E+00_JPRB, 0.55716E+00_JPRB, 0.54310E+00_JPRB, 0.50449E+00_JPRB ,&
&  0.47739E+00_JPRB, 0.48536E+00_JPRB, 0.55497E+00_JPRB, 0.59638E+00_JPRB, 0.61908E+00_JPRB, 0.62832E+00_JPRB, 0.63981E+00_JPRB ,&
&  0.65896E+00_JPRB, 0.68426E+00_JPRB, 0.71639E+00_JPRB, 0.75646E+00_JPRB, 0.78748E+00_JPRB, 0.81998E+00_JPRB, 0.85615E+00_JPRB ,&
&  0.93010E+00_JPRB, 0.80642E+00_JPRB, 0.78457E+00_JPRB, 0.85226E+00_JPRB, 0.11848E+01_JPRB, 0.13382E+01_JPRB, 0.14011E+01_JPRB ,&
&  0.14808E+01_JPRB, 0.14466E+01_JPRB, 0.14879E+01_JPRB, 0.15616E+01_JPRB, 0.15998E+01_JPRB, 0.16010E+01_JPRB, 0.15740E+01_JPRB ,&
&  0.15278E+01_JPRB, 0.14770E+01_JPRB, 0.14028E+01_JPRB, 0.13622E+01_JPRB, 0.13252E+01_JPRB, 0.13355E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.12605E+01_JPRB, 0.11813E+01_JPRB, 0.10935E+01_JPRB, 0.10257E+01_JPRB, 0.98229E+00_JPRB, 0.94812E+00_JPRB ,&
&  0.92476E+00_JPRB, 0.92869E+00_JPRB, 0.95052E+00_JPRB, 0.99726E+00_JPRB, 0.97886E+00_JPRB, 0.88492E+00_JPRB, 0.79275E+00_JPRB ,&
&  0.70956E+00_JPRB, 0.66447E+00_JPRB, 0.62305E+00_JPRB, 0.61331E+00_JPRB, 0.62245E+00_JPRB, 0.63556E+00_JPRB, 0.65203E+00_JPRB ,&
&  0.67255E+00_JPRB, 0.68657E+00_JPRB, 0.70050E+00_JPRB, 0.72168E+00_JPRB, 0.72571E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.73639E+00_JPRB, 0.75496E+00_JPRB, 0.74719E+00_JPRB, 0.74652E+00_JPRB, 0.74114E+00_JPRB, 0.70166E+00_JPRB, 0.67397E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.65264E+00_JPRB, 0.67648E+00_JPRB, 0.60426E+00_JPRB, 0.57523E+00_JPRB, 0.56778E+00_JPRB, 0.55237E+00_JPRB ,&
&  0.55245E+00_JPRB, 0.56869E+00_JPRB, 0.58873E+00_JPRB, 0.60527E+00_JPRB, 0.61750E+00_JPRB, 0.60249E+00_JPRB, 0.58553E+00_JPRB ,&
&  0.56526E+00_JPRB, 0.56221E+00_JPRB, 0.58043E+00_JPRB, 0.59891E+00_JPRB, 0.61539E+00_JPRB, 0.62132E+00_JPRB, 0.61688E+00_JPRB ,&
&  0.59766E+00_JPRB, 0.59165E+00_JPRB, 0.59151E+00_JPRB, 0.58488E+00_JPRB, 0.57825E+00_JPRB, 0.57000E+00_JPRB, 0.55871E+00_JPRB ,&
&  0.54937E+00_JPRB, 0.55635E+00_JPRB, 0.60247E+00_JPRB, 0.63451E+00_JPRB, 0.65134E+00_JPRB, 0.65598E+00_JPRB, 0.66884E+00_JPRB ,&
&  0.68370E+00_JPRB, 0.70658E+00_JPRB, 0.74730E+00_JPRB, 0.80533E+00_JPRB, 0.85938E+00_JPRB, 0.91676E+00_JPRB, 0.98340E+00_JPRB ,&
&  0.10711E+01_JPRB, 0.88641E+00_JPRB, 0.10177E+01_JPRB, 0.10584E+01_JPRB, 0.12514E+01_JPRB, 0.14727E+01_JPRB, 0.14943E+01_JPRB ,&
&  0.16892E+01_JPRB, 0.16505E+01_JPRB, 0.16912E+01_JPRB, 0.17123E+01_JPRB, 0.17474E+01_JPRB, 0.17648E+01_JPRB, 0.17562E+01_JPRB ,&
&  0.17852E+01_JPRB, 0.16004E+01_JPRB, 0.15914E+01_JPRB, 0.15073E+01_JPRB, 0.14405E+01_JPRB, 0.14264E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.12883E+01_JPRB, 0.12382E+01_JPRB, 0.11019E+01_JPRB, 0.10603E+01_JPRB, 0.10263E+01_JPRB, 0.94906E+00_JPRB ,&
&  0.92207E+00_JPRB, 0.89232E+00_JPRB, 0.87097E+00_JPRB, 0.96962E+00_JPRB, 0.98389E+00_JPRB, 0.89879E+00_JPRB, 0.79507E+00_JPRB ,&
&  0.72780E+00_JPRB, 0.68088E+00_JPRB, 0.69843E+00_JPRB, 0.69918E+00_JPRB, 0.68261E+00_JPRB, 0.68301E+00_JPRB, 0.72761E+00_JPRB ,&
&  0.71682E+00_JPRB, 0.72678E+00_JPRB, 0.75602E+00_JPRB, 0.75514E+00_JPRB, 0.73983E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.75490E+00_JPRB, 0.78087E+00_JPRB, 0.76613E+00_JPRB, 0.77051E+00_JPRB, 0.77535E+00_JPRB, 0.73921E+00_JPRB, 0.69984E+00_JPRB ,&
&  0.70475E+00_JPRB, 0.70381E+00_JPRB, 0.70653E+00_JPRB, 0.70697E+00_JPRB, 0.70984E+00_JPRB, 0.65671E+00_JPRB, 0.56662E+00_JPRB ,&
&  0.54056E+00_JPRB, 0.55644E+00_JPRB, 0.58955E+00_JPRB, 0.61324E+00_JPRB, 0.61527E+00_JPRB, 0.60452E+00_JPRB, 0.59708E+00_JPRB ,&
&  0.59115E+00_JPRB, 0.58854E+00_JPRB, 0.60152E+00_JPRB, 0.61119E+00_JPRB, 0.61701E+00_JPRB, 0.62851E+00_JPRB, 0.63533E+00_JPRB ,&
&  0.62182E+00_JPRB, 0.62273E+00_JPRB, 0.61909E+00_JPRB, 0.59753E+00_JPRB, 0.60740E+00_JPRB, 0.59640E+00_JPRB, 0.58448E+00_JPRB ,&
&  0.59192E+00_JPRB, 0.61479E+00_JPRB, 0.63833E+00_JPRB, 0.66687E+00_JPRB, 0.68601E+00_JPRB, 0.69416E+00_JPRB, 0.70467E+00_JPRB ,&
&  0.72575E+00_JPRB, 0.76184E+00_JPRB, 0.80846E+00_JPRB, 0.86976E+00_JPRB, 0.94832E+00_JPRB, 0.99947E+00_JPRB, 0.11443E+01_JPRB ,&
&  0.11688E+01_JPRB, 0.12112E+01_JPRB, 0.12585E+01_JPRB, 0.11953E+01_JPRB, 0.13748E+01_JPRB, 0.15784E+01_JPRB, 0.16060E+01_JPRB ,&
&  0.17069E+01_JPRB, 0.16437E+01_JPRB, 0.17748E+01_JPRB, 0.19291E+01_JPRB, 0.20494E+01_JPRB, 0.20253E+01_JPRB, 0.19742E+01_JPRB ,&
&  0.20215E+01_JPRB, 0.19760E+01_JPRB, 0.17617E+01_JPRB, 0.16719E+01_JPRB, 0.15792E+01_JPRB, 0.15454E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.13655E+01_JPRB, 0.13134E+01_JPRB, 0.12489E+01_JPRB, 0.11713E+01_JPRB, 0.11002E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.95781E+00_JPRB, 0.85804E+00_JPRB, 0.82869E+00_JPRB, 0.89666E+00_JPRB, 0.96654E+00_JPRB, 0.86730E+00_JPRB, 0.75539E+00_JPRB ,&
&  0.71425E+00_JPRB, 0.72082E+00_JPRB, 0.78341E+00_JPRB, 0.78557E+00_JPRB, 0.77072E+00_JPRB, 0.76200E+00_JPRB, 0.73454E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.75465E+00_JPRB, 0.77015E+00_JPRB, 0.76169E+00_JPRB, 0.74778E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.79051E+00_JPRB, 0.82990E+00_JPRB, 0.81033E+00_JPRB, 0.80940E+00_JPRB, 0.81913E+00_JPRB, 0.77137E+00_JPRB, 0.75126E+00_JPRB ,&
&  0.74858E+00_JPRB, 0.74372E+00_JPRB, 0.73415E+00_JPRB, 0.73053E+00_JPRB, 0.72955E+00_JPRB, 0.71846E+00_JPRB, 0.65613E+00_JPRB ,&
&  0.58755E+00_JPRB, 0.60270E+00_JPRB, 0.60039E+00_JPRB, 0.62558E+00_JPRB, 0.65232E+00_JPRB, 0.63391E+00_JPRB, 0.62174E+00_JPRB ,&
&  0.62583E+00_JPRB, 0.61872E+00_JPRB, 0.61729E+00_JPRB, 0.62541E+00_JPRB, 0.64127E+00_JPRB, 0.66761E+00_JPRB, 0.68023E+00_JPRB ,&
&  0.66059E+00_JPRB, 0.65304E+00_JPRB, 0.63977E+00_JPRB, 0.63109E+00_JPRB, 0.63584E+00_JPRB, 0.63316E+00_JPRB, 0.62769E+00_JPRB ,&
&  0.63492E+00_JPRB, 0.65420E+00_JPRB, 0.67459E+00_JPRB, 0.70632E+00_JPRB, 0.73409E+00_JPRB, 0.75381E+00_JPRB, 0.77720E+00_JPRB ,&
&  0.82906E+00_JPRB, 0.88709E+00_JPRB, 0.92756E+00_JPRB, 0.98860E+00_JPRB, 0.10565E+01_JPRB, 0.11964E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.13721E+01_JPRB, 0.13889E+01_JPRB, 0.13975E+01_JPRB, 0.14453E+01_JPRB, 0.16217E+01_JPRB, 0.17433E+01_JPRB, 0.18122E+01_JPRB ,&
&  0.19316E+01_JPRB, 0.18652E+01_JPRB, 0.22184E+01_JPRB, 0.23384E+01_JPRB, 0.22868E+01_JPRB, 0.22941E+01_JPRB, 0.21633E+01_JPRB ,&
&  0.21143E+01_JPRB, 0.19724E+01_JPRB, 0.18764E+01_JPRB, 0.17891E+01_JPRB, 0.16998E+01_JPRB, 0.16899E+01_JPRB, 0.15829E+01_JPRB ,&
&  0.15542E+01_JPRB, 0.15053E+01_JPRB, 0.13129E+01_JPRB, 0.11727E+01_JPRB, 0.10230E+01_JPRB, 0.10733E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.95700E+00_JPRB, 0.82585E+00_JPRB, 0.88422E+00_JPRB, 0.97863E+00_JPRB, 0.10746E+01_JPRB, 0.10760E+01_JPRB, 0.90086E+00_JPRB ,&
&  0.80415E+00_JPRB, 0.81838E+00_JPRB, 0.83030E+00_JPRB, 0.84837E+00_JPRB, 0.85681E+00_JPRB, 0.86199E+00_JPRB, 0.78875E+00_JPRB ,&
&  0.73974E+00_JPRB, 0.77589E+00_JPRB, 0.78857E+00_JPRB, 0.78039E+00_JPRB, 0.77639E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.83874E+00_JPRB, 0.88757E+00_JPRB, 0.88767E+00_JPRB, 0.86505E+00_JPRB, 0.86363E+00_JPRB, 0.81222E+00_JPRB, 0.79086E+00_JPRB ,&
&  0.79397E+00_JPRB, 0.78800E+00_JPRB, 0.76370E+00_JPRB, 0.75744E+00_JPRB, 0.75382E+00_JPRB, 0.74228E+00_JPRB, 0.72313E+00_JPRB ,&
&  0.62783E+00_JPRB, 0.64110E+00_JPRB, 0.60584E+00_JPRB, 0.62645E+00_JPRB, 0.71680E+00_JPRB, 0.69412E+00_JPRB, 0.68131E+00_JPRB ,&
&  0.66860E+00_JPRB, 0.66234E+00_JPRB, 0.64645E+00_JPRB, 0.64832E+00_JPRB, 0.66734E+00_JPRB, 0.71180E+00_JPRB, 0.75546E+00_JPRB ,&
&  0.74496E+00_JPRB, 0.72067E+00_JPRB, 0.69767E+00_JPRB, 0.68398E+00_JPRB, 0.67617E+00_JPRB, 0.67926E+00_JPRB, 0.69521E+00_JPRB ,&
&  0.70772E+00_JPRB, 0.71640E+00_JPRB, 0.74093E+00_JPRB, 0.78153E+00_JPRB, 0.81533E+00_JPRB, 0.84280E+00_JPRB, 0.89997E+00_JPRB ,&
&  0.98699E+00_JPRB, 0.10555E+01_JPRB, 0.10939E+01_JPRB, 0.11483E+01_JPRB, 0.11945E+01_JPRB, 0.13262E+01_JPRB, 0.14426E+01_JPRB ,&
&  0.16375E+01_JPRB, 0.16433E+01_JPRB, 0.17184E+01_JPRB, 0.19524E+01_JPRB, 0.21136E+01_JPRB, 0.21523E+01_JPRB, 0.21347E+01_JPRB ,&
&  0.21796E+01_JPRB, 0.21299E+01_JPRB, 0.22582E+01_JPRB, 0.24045E+01_JPRB, 0.23413E+01_JPRB, 0.24228E+01_JPRB, 0.22607E+01_JPRB ,&
&  0.20142E+01_JPRB, 0.18750E+01_JPRB, 0.18120E+01_JPRB, 0.17744E+01_JPRB, 0.16838E+01_JPRB, 0.17307E+01_JPRB, 0.16694E+01_JPRB ,&
&  0.16697E+01_JPRB, 0.14869E+01_JPRB, 0.11360E+01_JPRB, 0.10506E+01_JPRB, 0.85182E+00_JPRB, 0.87397E+00_JPRB, 0.10731E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.98549E+00_JPRB, 0.10326E+01_JPRB, 0.11510E+01_JPRB, 0.12970E+01_JPRB, 0.15004E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.99573E+00_JPRB, 0.98749E+00_JPRB, 0.99009E+00_JPRB, 0.98886E+00_JPRB, 0.10090E+01_JPRB, 0.97510E+00_JPRB ,&
&  0.92603E+00_JPRB, 0.88018E+00_JPRB, 0.85900E+00_JPRB, 0.83825E+00_JPRB, 0.83352E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.89080E+00_JPRB, 0.93180E+00_JPRB, 0.94754E+00_JPRB, 0.91991E+00_JPRB, 0.90590E+00_JPRB, 0.86561E+00_JPRB, 0.83331E+00_JPRB ,&
&  0.83225E+00_JPRB, 0.82279E+00_JPRB, 0.79451E+00_JPRB, 0.78734E+00_JPRB, 0.77892E+00_JPRB, 0.75943E+00_JPRB, 0.74704E+00_JPRB ,&
&  0.76490E+00_JPRB, 0.75344E+00_JPRB, 0.67985E+00_JPRB, 0.63493E+00_JPRB, 0.70627E+00_JPRB, 0.70136E+00_JPRB, 0.72013E+00_JPRB ,&
&  0.70438E+00_JPRB, 0.72679E+00_JPRB, 0.72280E+00_JPRB, 0.72817E+00_JPRB, 0.76908E+00_JPRB, 0.82775E+00_JPRB, 0.88059E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.89314E+00_JPRB, 0.86088E+00_JPRB, 0.84038E+00_JPRB, 0.85144E+00_JPRB, 0.81820E+00_JPRB, 0.83545E+00_JPRB ,&
&  0.83255E+00_JPRB, 0.82205E+00_JPRB, 0.82825E+00_JPRB, 0.86471E+00_JPRB, 0.90281E+00_JPRB, 0.94664E+00_JPRB, 0.10276E+01_JPRB ,&
&  0.11096E+01_JPRB, 0.11448E+01_JPRB, 0.11465E+01_JPRB, 0.11676E+01_JPRB, 0.12424E+01_JPRB, 0.13570E+01_JPRB, 0.14504E+01_JPRB ,&
&  0.15846E+01_JPRB, 0.16226E+01_JPRB, 0.16466E+01_JPRB, 0.19787E+01_JPRB, 0.22102E+01_JPRB, 0.22201E+01_JPRB, 0.22159E+01_JPRB ,&
&  0.22897E+01_JPRB, 0.23317E+01_JPRB, 0.24747E+01_JPRB, 0.25422E+01_JPRB, 0.25547E+01_JPRB, 0.25118E+01_JPRB, 0.22793E+01_JPRB ,&
&  0.19767E+01_JPRB, 0.18087E+01_JPRB, 0.17746E+01_JPRB, 0.17149E+01_JPRB, 0.16679E+01_JPRB, 0.15627E+01_JPRB, 0.16326E+01_JPRB ,&
&  0.17565E+01_JPRB, 0.10261E+01_JPRB, 0.83750E+00_JPRB, 0.97267E+00_JPRB, 0.85322E+00_JPRB, 0.87585E+00_JPRB, 0.99370E+00_JPRB ,&
&  0.96537E+00_JPRB, 0.96091E+00_JPRB, 0.11418E+01_JPRB, 0.12895E+01_JPRB, 0.14957E+01_JPRB, 0.17848E+01_JPRB, 0.16005E+01_JPRB ,&
&  0.13931E+01_JPRB, 0.12827E+01_JPRB, 0.12840E+01_JPRB, 0.12486E+01_JPRB, 0.12658E+01_JPRB, 0.12437E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.11254E+01_JPRB, 0.10663E+01_JPRB, 0.10015E+01_JPRB, 0.94830E+00_JPRB, 0.90966E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.99012E+00_JPRB, 0.10101E+01_JPRB, 0.10073E+01_JPRB, 0.97919E+00_JPRB, 0.95165E+00_JPRB, 0.93254E+00_JPRB, 0.89233E+00_JPRB ,&
&  0.87097E+00_JPRB, 0.85271E+00_JPRB, 0.82498E+00_JPRB, 0.81541E+00_JPRB, 0.79399E+00_JPRB, 0.77114E+00_JPRB, 0.76030E+00_JPRB ,&
&  0.77383E+00_JPRB, 0.78898E+00_JPRB, 0.68238E+00_JPRB, 0.65032E+00_JPRB, 0.62666E+00_JPRB, 0.66645E+00_JPRB, 0.70430E+00_JPRB ,&
&  0.75133E+00_JPRB, 0.76668E+00_JPRB, 0.79215E+00_JPRB, 0.84254E+00_JPRB, 0.92889E+00_JPRB, 0.98425E+00_JPRB, 0.10646E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.11265E+01_JPRB, 0.10502E+01_JPRB, 0.10572E+01_JPRB, 0.10712E+01_JPRB, 0.10152E+01_JPRB, 0.10362E+01_JPRB ,&
&  0.99147E+00_JPRB, 0.97562E+00_JPRB, 0.97203E+00_JPRB, 0.97291E+00_JPRB, 0.99005E+00_JPRB, 0.10068E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.11093E+01_JPRB, 0.11276E+01_JPRB, 0.11160E+01_JPRB, 0.11186E+01_JPRB, 0.11678E+01_JPRB, 0.12493E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.14360E+01_JPRB, 0.13387E+01_JPRB, 0.11218E+01_JPRB, 0.16334E+01_JPRB, 0.22337E+01_JPRB, 0.21482E+01_JPRB, 0.19328E+01_JPRB ,&
&  0.21148E+01_JPRB, 0.22720E+01_JPRB, 0.24959E+01_JPRB, 0.25839E+01_JPRB, 0.25849E+01_JPRB, 0.24814E+01_JPRB, 0.22864E+01_JPRB ,&
&  0.20491E+01_JPRB, 0.19692E+01_JPRB, 0.19945E+01_JPRB, 0.21293E+01_JPRB, 0.20741E+01_JPRB, 0.19975E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.14299E+01_JPRB, 0.10289E+01_JPRB, 0.10662E+01_JPRB, 0.86604E+00_JPRB, 0.87295E+00_JPRB, 0.98380E+00_JPRB ,&
&  0.95777E+00_JPRB, 0.10308E+01_JPRB, 0.12341E+01_JPRB, 0.14341E+01_JPRB, 0.18929E+01_JPRB, 0.20660E+01_JPRB, 0.20001E+01_JPRB ,&
&  0.17488E+01_JPRB, 0.15204E+01_JPRB, 0.15635E+01_JPRB, 0.15493E+01_JPRB, 0.15617E+01_JPRB, 0.15025E+01_JPRB, 0.14231E+01_JPRB ,&
&  0.13245E+01_JPRB, 0.12352E+01_JPRB, 0.11738E+01_JPRB, 0.10913E+01_JPRB, 0.10131E+01_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.11137E+01_JPRB, 0.10964E+01_JPRB, 0.10559E+01_JPRB, 0.10026E+01_JPRB, 0.96146E+00_JPRB, 0.94524E+00_JPRB, 0.92505E+00_JPRB ,&
&  0.90893E+00_JPRB, 0.87977E+00_JPRB, 0.85572E+00_JPRB, 0.83344E+00_JPRB, 0.81176E+00_JPRB, 0.78880E+00_JPRB, 0.78217E+00_JPRB ,&
&  0.79157E+00_JPRB, 0.74246E+00_JPRB, 0.63225E+00_JPRB, 0.58900E+00_JPRB, 0.56873E+00_JPRB, 0.60701E+00_JPRB, 0.66317E+00_JPRB ,&
&  0.73075E+00_JPRB, 0.77597E+00_JPRB, 0.86255E+00_JPRB, 0.98208E+00_JPRB, 0.11117E+01_JPRB, 0.12148E+01_JPRB, 0.12873E+01_JPRB ,&
&  0.15155E+01_JPRB, 0.13854E+01_JPRB, 0.13877E+01_JPRB, 0.13186E+01_JPRB, 0.12982E+01_JPRB, 0.12180E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.10401E+01_JPRB, 0.10036E+01_JPRB, 0.97346E+00_JPRB, 0.96916E+00_JPRB, 0.99915E+00_JPRB, 0.10764E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.11318E+01_JPRB, 0.10932E+01_JPRB, 0.11196E+01_JPRB, 0.11993E+01_JPRB, 0.12656E+01_JPRB, 0.12976E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.13007E+01_JPRB, 0.17161E+01_JPRB, 0.19079E+01_JPRB, 0.21673E+01_JPRB, 0.22092E+01_JPRB, 0.21144E+01_JPRB ,&
&  0.22464E+01_JPRB, 0.23132E+01_JPRB, 0.24364E+01_JPRB, 0.20269E+01_JPRB, 0.23096E+01_JPRB, 0.22880E+01_JPRB, 0.21656E+01_JPRB ,&
&  0.20646E+01_JPRB, 0.21156E+01_JPRB, 0.21424E+01_JPRB, 0.22923E+01_JPRB, 0.24210E+01_JPRB, 0.25401E+01_JPRB, 0.19722E+01_JPRB ,&
&  0.15778E+01_JPRB, 0.16892E+01_JPRB, 0.15278E+01_JPRB, 0.12810E+01_JPRB, 0.10901E+01_JPRB, 0.11099E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.12483E+01_JPRB, 0.14650E+01_JPRB, 0.19169E+01_JPRB, 0.24391E+01_JPRB, 0.26429E+01_JPRB, 0.24478E+01_JPRB ,&
&  0.20477E+01_JPRB, 0.20850E+01_JPRB, 0.20022E+01_JPRB, 0.19164E+01_JPRB, 0.18777E+01_JPRB, 0.17856E+01_JPRB, 0.16463E+01_JPRB ,&
&  0.14974E+01_JPRB, 0.13874E+01_JPRB, 0.12972E+01_JPRB, 0.12060E+01_JPRB, 0.11385E+01_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.11932E+01_JPRB, 0.11615E+01_JPRB, 0.11081E+01_JPRB, 0.10507E+01_JPRB, 0.10018E+01_JPRB, 0.97805E+00_JPRB, 0.95921E+00_JPRB ,&
&  0.95523E+00_JPRB, 0.89841E+00_JPRB, 0.87384E+00_JPRB, 0.84939E+00_JPRB, 0.82946E+00_JPRB, 0.81457E+00_JPRB, 0.81426E+00_JPRB ,&
&  0.81808E+00_JPRB, 0.71597E+00_JPRB, 0.60236E+00_JPRB, 0.60167E+00_JPRB, 0.61825E+00_JPRB, 0.61298E+00_JPRB, 0.66026E+00_JPRB ,&
&  0.72446E+00_JPRB, 0.80694E+00_JPRB, 0.92820E+00_JPRB, 0.11483E+01_JPRB, 0.13504E+01_JPRB, 0.15255E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.17129E+01_JPRB, 0.17390E+01_JPRB, 0.16872E+01_JPRB, 0.15286E+01_JPRB, 0.13562E+01_JPRB, 0.12352E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.10555E+01_JPRB, 0.10225E+01_JPRB, 0.10112E+01_JPRB, 0.10221E+01_JPRB, 0.10690E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.11422E+01_JPRB, 0.11214E+01_JPRB, 0.11878E+01_JPRB, 0.11508E+01_JPRB, 0.11436E+01_JPRB, 0.11205E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.16524E+01_JPRB, 0.17389E+01_JPRB, 0.19392E+01_JPRB, 0.17644E+01_JPRB, 0.18837E+01_JPRB, 0.21936E+01_JPRB ,&
&  0.23758E+01_JPRB, 0.23984E+01_JPRB, 0.24807E+01_JPRB, 0.17715E+01_JPRB, 0.15974E+01_JPRB, 0.21522E+01_JPRB, 0.21713E+01_JPRB ,&
&  0.20933E+01_JPRB, 0.22605E+01_JPRB, 0.24393E+01_JPRB, 0.27489E+01_JPRB, 0.21393E+01_JPRB, 0.19004E+01_JPRB, 0.14593E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.13188E+01_JPRB, 0.15003E+01_JPRB, 0.14022E+01_JPRB, 0.14144E+01_JPRB, 0.13914E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.13727E+01_JPRB, 0.15076E+01_JPRB, 0.19226E+01_JPRB, 0.26803E+01_JPRB, 0.33777E+01_JPRB, 0.31008E+01_JPRB, 0.30640E+01_JPRB ,&
&  0.29278E+01_JPRB, 0.27510E+01_JPRB, 0.24463E+01_JPRB, 0.23763E+01_JPRB, 0.22282E+01_JPRB, 0.20172E+01_JPRB, 0.17757E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.14676E+01_JPRB, 0.13638E+01_JPRB, 0.12781E+01_JPRB, 0.12235E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.12700E+01_JPRB, 0.12241E+01_JPRB, 0.11709E+01_JPRB, 0.11122E+01_JPRB, 0.10634E+01_JPRB, 0.10299E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.93799E+00_JPRB, 0.88199E+00_JPRB, 0.85608E+00_JPRB, 0.84589E+00_JPRB, 0.84279E+00_JPRB, 0.84781E+00_JPRB ,&
&  0.86008E+00_JPRB, 0.81147E+00_JPRB, 0.65312E+00_JPRB, 0.61069E+00_JPRB, 0.66692E+00_JPRB, 0.65313E+00_JPRB, 0.66531E+00_JPRB ,&
&  0.75533E+00_JPRB, 0.89017E+00_JPRB, 0.10967E+01_JPRB, 0.14256E+01_JPRB, 0.16912E+01_JPRB, 0.19488E+01_JPRB, 0.20844E+01_JPRB ,&
&  0.21263E+01_JPRB, 0.19514E+01_JPRB, 0.16854E+01_JPRB, 0.14375E+01_JPRB, 0.12692E+01_JPRB, 0.11667E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.10524E+01_JPRB, 0.10427E+01_JPRB, 0.10330E+01_JPRB, 0.10680E+01_JPRB, 0.10954E+01_JPRB, 0.11481E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.11561E+01_JPRB, 0.11788E+01_JPRB, 0.12279E+01_JPRB, 0.12263E+01_JPRB, 0.11777E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.13241E+01_JPRB, 0.15047E+01_JPRB, 0.16837E+01_JPRB, 0.18550E+01_JPRB, 0.20244E+01_JPRB, 0.20962E+01_JPRB, 0.23646E+01_JPRB ,&
&  0.20281E+01_JPRB, 0.16695E+01_JPRB, 0.14189E+01_JPRB, 0.11963E+01_JPRB, 0.15426E+01_JPRB, 0.22023E+01_JPRB, 0.23790E+01_JPRB ,&
&  0.24936E+01_JPRB, 0.26793E+01_JPRB, 0.28603E+01_JPRB, 0.25438E+01_JPRB, 0.20416E+01_JPRB, 0.16419E+01_JPRB, 0.19525E+01_JPRB ,&
&  0.20407E+01_JPRB, 0.20689E+01_JPRB, 0.19381E+01_JPRB, 0.17085E+01_JPRB, 0.14523E+01_JPRB, 0.16840E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.17015E+01_JPRB, 0.18596E+01_JPRB, 0.30043E+01_JPRB, 0.40064E+01_JPRB, 0.41262E+01_JPRB, 0.38836E+01_JPRB, 0.39229E+01_JPRB ,&
&  0.35525E+01_JPRB, 0.33377E+01_JPRB, 0.29807E+01_JPRB, 0.26138E+01_JPRB, 0.22864E+01_JPRB, 0.20109E+01_JPRB, 0.17894E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.15298E+01_JPRB, 0.14156E+01_JPRB, 0.13527E+01_JPRB, 0.13070E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.13649E+01_JPRB, 0.13099E+01_JPRB, 0.12577E+01_JPRB, 0.12023E+01_JPRB, 0.11525E+01_JPRB, 0.10927E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.97678E+00_JPRB, 0.91045E+00_JPRB, 0.88367E+00_JPRB, 0.89211E+00_JPRB, 0.90001E+00_JPRB, 0.90175E+00_JPRB ,&
&  0.90842E+00_JPRB, 0.97568E+00_JPRB, 0.77579E+00_JPRB, 0.68532E+00_JPRB, 0.69996E+00_JPRB, 0.70152E+00_JPRB, 0.67296E+00_JPRB ,&
&  0.82640E+00_JPRB, 0.10593E+01_JPRB, 0.13288E+01_JPRB, 0.16895E+01_JPRB, 0.21138E+01_JPRB, 0.21118E+01_JPRB, 0.21599E+01_JPRB ,&
&  0.21423E+01_JPRB, 0.19061E+01_JPRB, 0.16454E+01_JPRB, 0.14365E+01_JPRB, 0.12912E+01_JPRB, 0.12034E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.11350E+01_JPRB, 0.11199E+01_JPRB, 0.11247E+01_JPRB, 0.11207E+01_JPRB, 0.11346E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.11616E+01_JPRB, 0.11831E+01_JPRB, 0.12640E+01_JPRB, 0.13503E+01_JPRB, 0.12513E+01_JPRB, 0.15026E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.15666E+01_JPRB, 0.17170E+01_JPRB, 0.21033E+01_JPRB, 0.22096E+01_JPRB, 0.21474E+01_JPRB, 0.22684E+01_JPRB ,&
&  0.18479E+01_JPRB, 0.16586E+01_JPRB, 0.17779E+01_JPRB, 0.18804E+01_JPRB, 0.18289E+01_JPRB, 0.19455E+01_JPRB, 0.29106E+01_JPRB ,&
&  0.24611E+01_JPRB, 0.27361E+01_JPRB, 0.29159E+01_JPRB, 0.28827E+01_JPRB, 0.14838E+01_JPRB, 0.12078E+01_JPRB, 0.18011E+01_JPRB ,&
&  0.19986E+01_JPRB, 0.14007E+01_JPRB, 0.10396E+01_JPRB, 0.12390E+01_JPRB, 0.98043E+00_JPRB, 0.12249E+01_JPRB, 0.17398E+01_JPRB ,&
&  0.20343E+01_JPRB, 0.24539E+01_JPRB, 0.40385E+01_JPRB, 0.46974E+01_JPRB, 0.46711E+01_JPRB, 0.50021E+01_JPRB, 0.43314E+01_JPRB ,&
&  0.39626E+01_JPRB, 0.32889E+01_JPRB, 0.31405E+01_JPRB, 0.27343E+01_JPRB, 0.23788E+01_JPRB, 0.21266E+01_JPRB, 0.19245E+01_JPRB ,&
&  0.17672E+01_JPRB, 0.16328E+01_JPRB, 0.15299E+01_JPRB, 0.14671E+01_JPRB, 0.14179E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.15060E+01_JPRB, 0.14309E+01_JPRB, 0.13607E+01_JPRB, 0.12932E+01_JPRB, 0.12229E+01_JPRB, 0.11673E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10134E+01_JPRB, 0.95834E+00_JPRB, 0.91721E+00_JPRB, 0.91764E+00_JPRB, 0.92731E+00_JPRB, 0.91722E+00_JPRB ,&
&  0.90356E+00_JPRB, 0.91774E+00_JPRB, 0.89995E+00_JPRB, 0.83522E+00_JPRB, 0.79493E+00_JPRB, 0.78147E+00_JPRB, 0.81799E+00_JPRB ,&
&  0.10656E+01_JPRB, 0.14450E+01_JPRB, 0.17198E+01_JPRB, 0.19768E+01_JPRB, 0.21674E+01_JPRB, 0.21198E+01_JPRB, 0.22484E+01_JPRB ,&
&  0.21908E+01_JPRB, 0.19974E+01_JPRB, 0.17716E+01_JPRB, 0.15517E+01_JPRB, 0.14038E+01_JPRB, 0.13049E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.12048E+01_JPRB, 0.11855E+01_JPRB, 0.11544E+01_JPRB, 0.11328E+01_JPRB, 0.11429E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11568E+01_JPRB, 0.11877E+01_JPRB, 0.12403E+01_JPRB, 0.13874E+01_JPRB, 0.14161E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.12733E+01_JPRB, 0.17397E+01_JPRB, 0.20041E+01_JPRB, 0.20442E+01_JPRB, 0.20706E+01_JPRB, 0.19943E+01_JPRB ,&
&  0.19528E+01_JPRB, 0.21449E+01_JPRB, 0.21604E+01_JPRB, 0.23658E+01_JPRB, 0.23627E+01_JPRB, 0.18785E+01_JPRB, 0.24423E+01_JPRB ,&
&  0.26421E+01_JPRB, 0.26295E+01_JPRB, 0.21665E+01_JPRB, 0.15827E+01_JPRB, 0.17025E+01_JPRB, 0.14055E+01_JPRB, 0.10104E+01_JPRB ,&
&  0.95413E+00_JPRB, 0.91015E+00_JPRB, 0.83072E+00_JPRB, 0.83143E+00_JPRB, 0.82219E+00_JPRB, 0.11940E+01_JPRB, 0.18051E+01_JPRB ,&
&  0.27136E+01_JPRB, 0.37779E+01_JPRB, 0.48570E+01_JPRB, 0.53271E+01_JPRB, 0.49381E+01_JPRB, 0.51175E+01_JPRB, 0.45980E+01_JPRB ,&
&  0.36404E+01_JPRB, 0.31912E+01_JPRB, 0.29861E+01_JPRB, 0.26602E+01_JPRB, 0.23979E+01_JPRB, 0.22167E+01_JPRB, 0.20656E+01_JPRB ,&
&  0.19132E+01_JPRB, 0.17779E+01_JPRB, 0.16832E+01_JPRB, 0.16246E+01_JPRB, 0.15729E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.15982E+01_JPRB, 0.15192E+01_JPRB, 0.14465E+01_JPRB, 0.13823E+01_JPRB, 0.13178E+01_JPRB, 0.12608E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.10430E+01_JPRB, 0.98412E+00_JPRB, 0.94807E+00_JPRB, 0.93913E+00_JPRB, 0.94423E+00_JPRB, 0.93998E+00_JPRB ,&
&  0.92781E+00_JPRB, 0.92710E+00_JPRB, 0.95898E+00_JPRB, 0.97477E+00_JPRB, 0.10117E+01_JPRB, 0.10730E+01_JPRB, 0.12208E+01_JPRB ,&
&  0.15851E+01_JPRB, 0.20635E+01_JPRB, 0.22687E+01_JPRB, 0.24143E+01_JPRB, 0.25445E+01_JPRB, 0.25329E+01_JPRB, 0.24554E+01_JPRB ,&
&  0.22873E+01_JPRB, 0.20992E+01_JPRB, 0.18778E+01_JPRB, 0.16395E+01_JPRB, 0.14661E+01_JPRB, 0.13658E+01_JPRB, 0.13109E+01_JPRB ,&
&  0.12748E+01_JPRB, 0.12746E+01_JPRB, 0.12449E+01_JPRB, 0.11925E+01_JPRB, 0.11429E+01_JPRB, 0.11576E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.12018E+01_JPRB, 0.12522E+01_JPRB, 0.12718E+01_JPRB, 0.12002E+01_JPRB, 0.10525E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.12882E+01_JPRB, 0.15572E+01_JPRB, 0.16500E+01_JPRB, 0.20706E+01_JPRB, 0.21217E+01_JPRB, 0.19995E+01_JPRB, 0.20203E+01_JPRB ,&
&  0.20851E+01_JPRB, 0.25370E+01_JPRB, 0.21706E+01_JPRB, 0.21845E+01_JPRB, 0.25685E+01_JPRB, 0.20968E+01_JPRB, 0.21849E+01_JPRB ,&
&  0.25459E+01_JPRB, 0.24009E+01_JPRB, 0.20582E+01_JPRB, 0.16097E+01_JPRB, 0.21521E+01_JPRB, 0.23643E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.95718E+00_JPRB, 0.90500E+00_JPRB, 0.86667E+00_JPRB, 0.89263E+00_JPRB, 0.14188E+01_JPRB, 0.27154E+01_JPRB ,&
&  0.33610E+01_JPRB, 0.50202E+01_JPRB, 0.58086E+01_JPRB, 0.59566E+01_JPRB, 0.52208E+01_JPRB, 0.45094E+01_JPRB, 0.36971E+01_JPRB ,&
&  0.34277E+01_JPRB, 0.33256E+01_JPRB, 0.30426E+01_JPRB, 0.28247E+01_JPRB, 0.26026E+01_JPRB, 0.24024E+01_JPRB, 0.22196E+01_JPRB ,&
&  0.20366E+01_JPRB, 0.19024E+01_JPRB, 0.18027E+01_JPRB, 0.17564E+01_JPRB, 0.16811E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.16721E+01_JPRB, 0.16071E+01_JPRB, 0.15116E+01_JPRB, 0.14393E+01_JPRB, 0.13700E+01_JPRB, 0.12959E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.11778E+01_JPRB, 0.10852E+01_JPRB, 0.10191E+01_JPRB, 0.99162E+00_JPRB, 0.97665E+00_JPRB, 0.96098E+00_JPRB, 0.95386E+00_JPRB ,&
&  0.94298E+00_JPRB, 0.94650E+00_JPRB, 0.10047E+01_JPRB, 0.12145E+01_JPRB, 0.13462E+01_JPRB, 0.14497E+01_JPRB, 0.16537E+01_JPRB ,&
&  0.22877E+01_JPRB, 0.25717E+01_JPRB, 0.25670E+01_JPRB, 0.25919E+01_JPRB, 0.24918E+01_JPRB, 0.23969E+01_JPRB, 0.22328E+01_JPRB ,&
&  0.20645E+01_JPRB, 0.19145E+01_JPRB, 0.17580E+01_JPRB, 0.15918E+01_JPRB, 0.14618E+01_JPRB, 0.13954E+01_JPRB, 0.13570E+01_JPRB ,&
&  0.13545E+01_JPRB, 0.13277E+01_JPRB, 0.12749E+01_JPRB, 0.11866E+01_JPRB, 0.11378E+01_JPRB, 0.11671E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.12091E+01_JPRB, 0.12729E+01_JPRB, 0.13130E+01_JPRB, 0.12896E+01_JPRB, 0.10957E+01_JPRB, 0.12106E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.13348E+01_JPRB, 0.14182E+01_JPRB, 0.14279E+01_JPRB, 0.17499E+01_JPRB, 0.19923E+01_JPRB, 0.19308E+01_JPRB, 0.18888E+01_JPRB ,&
&  0.21993E+01_JPRB, 0.21612E+01_JPRB, 0.19650E+01_JPRB, 0.20517E+01_JPRB, 0.22527E+01_JPRB, 0.26836E+01_JPRB, 0.18663E+01_JPRB ,&
&  0.17342E+01_JPRB, 0.20769E+01_JPRB, 0.18740E+01_JPRB, 0.15522E+01_JPRB, 0.20352E+01_JPRB, 0.30265E+01_JPRB, 0.25060E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.97710E+00_JPRB, 0.10601E+01_JPRB, 0.11864E+01_JPRB, 0.11090E+01_JPRB, 0.17956E+01_JPRB, 0.52208E+01_JPRB ,&
&  0.49807E+01_JPRB, 0.62982E+01_JPRB, 0.62312E+01_JPRB, 0.57697E+01_JPRB, 0.50596E+01_JPRB, 0.42287E+01_JPRB, 0.37691E+01_JPRB ,&
&  0.34697E+01_JPRB, 0.33236E+01_JPRB, 0.31879E+01_JPRB, 0.30326E+01_JPRB, 0.28272E+01_JPRB, 0.25500E+01_JPRB, 0.23018E+01_JPRB ,&
&  0.21106E+01_JPRB, 0.19754E+01_JPRB, 0.18649E+01_JPRB, 0.17897E+01_JPRB, 0.17215E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.17584E+01_JPRB, 0.16481E+01_JPRB, 0.15348E+01_JPRB, 0.14611E+01_JPRB, 0.13788E+01_JPRB, 0.13008E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.12331E+01_JPRB, 0.11431E+01_JPRB, 0.10643E+01_JPRB, 0.10293E+01_JPRB, 0.10056E+01_JPRB, 0.98223E+00_JPRB, 0.97524E+00_JPRB ,&
&  0.97210E+00_JPRB, 0.98851E+00_JPRB, 0.10336E+01_JPRB, 0.12178E+01_JPRB, 0.14021E+01_JPRB, 0.13821E+01_JPRB, 0.16907E+01_JPRB ,&
&  0.25722E+01_JPRB, 0.27659E+01_JPRB, 0.26183E+01_JPRB, 0.25329E+01_JPRB, 0.23979E+01_JPRB, 0.23391E+01_JPRB, 0.21143E+01_JPRB ,&
&  0.18518E+01_JPRB, 0.16803E+01_JPRB, 0.15478E+01_JPRB, 0.14949E+01_JPRB, 0.14560E+01_JPRB, 0.14169E+01_JPRB, 0.13912E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.12908E+01_JPRB, 0.12187E+01_JPRB, 0.11342E+01_JPRB, 0.11334E+01_JPRB, 0.11844E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.12797E+01_JPRB, 0.13488E+01_JPRB, 0.13552E+01_JPRB, 0.12631E+01_JPRB, 0.12330E+01_JPRB, 0.12385E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.12937E+01_JPRB, 0.13731E+01_JPRB, 0.14903E+01_JPRB, 0.16339E+01_JPRB, 0.18084E+01_JPRB, 0.18496E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.19901E+01_JPRB, 0.19404E+01_JPRB, 0.17637E+01_JPRB, 0.19000E+01_JPRB, 0.20875E+01_JPRB, 0.24874E+01_JPRB, 0.24328E+01_JPRB ,&
&  0.20599E+01_JPRB, 0.17055E+01_JPRB, 0.16148E+01_JPRB, 0.17085E+01_JPRB, 0.20241E+01_JPRB, 0.28517E+01_JPRB, 0.43648E+01_JPRB ,&
&  0.44469E+01_JPRB, 0.29496E+01_JPRB, 0.24461E+01_JPRB, 0.33024E+01_JPRB, 0.28834E+01_JPRB, 0.28456E+01_JPRB, 0.41880E+01_JPRB ,&
&  0.54497E+01_JPRB, 0.64302E+01_JPRB, 0.62189E+01_JPRB, 0.55781E+01_JPRB, 0.49872E+01_JPRB, 0.42002E+01_JPRB, 0.38586E+01_JPRB ,&
&  0.36259E+01_JPRB, 0.34118E+01_JPRB, 0.33473E+01_JPRB, 0.31976E+01_JPRB, 0.29229E+01_JPRB, 0.25816E+01_JPRB, 0.22791E+01_JPRB ,&
&  0.20958E+01_JPRB, 0.19866E+01_JPRB, 0.19107E+01_JPRB, 0.18630E+01_JPRB, 0.18190E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.17594E+01_JPRB, 0.16164E+01_JPRB, 0.15135E+01_JPRB, 0.14312E+01_JPRB, 0.13426E+01_JPRB, 0.12909E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.11856E+01_JPRB, 0.11127E+01_JPRB, 0.10535E+01_JPRB, 0.10135E+01_JPRB, 0.99965E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.99823E+00_JPRB, 0.10162E+01_JPRB, 0.10554E+01_JPRB, 0.12481E+01_JPRB, 0.15860E+01_JPRB, 0.21236E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.25069E+01_JPRB, 0.30768E+01_JPRB, 0.27987E+01_JPRB, 0.25027E+01_JPRB, 0.22576E+01_JPRB, 0.22555E+01_JPRB, 0.19989E+01_JPRB ,&
&  0.16832E+01_JPRB, 0.15084E+01_JPRB, 0.14181E+01_JPRB, 0.13835E+01_JPRB, 0.13795E+01_JPRB, 0.13528E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.12507E+01_JPRB, 0.12051E+01_JPRB, 0.11511E+01_JPRB, 0.11258E+01_JPRB, 0.11498E+01_JPRB, 0.12076E+01_JPRB, 0.12839E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.14149E+01_JPRB, 0.13961E+01_JPRB, 0.12909E+01_JPRB, 0.12836E+01_JPRB, 0.12806E+01_JPRB, 0.12913E+01_JPRB ,&
&  0.12473E+01_JPRB, 0.12146E+01_JPRB, 0.14003E+01_JPRB, 0.16056E+01_JPRB, 0.17137E+01_JPRB, 0.16661E+01_JPRB, 0.16257E+01_JPRB ,&
&  0.18437E+01_JPRB, 0.20051E+01_JPRB, 0.21100E+01_JPRB, 0.20214E+01_JPRB, 0.22198E+01_JPRB, 0.24632E+01_JPRB, 0.26503E+01_JPRB ,&
&  0.25734E+01_JPRB, 0.21090E+01_JPRB, 0.19378E+01_JPRB, 0.19787E+01_JPRB, 0.20632E+01_JPRB, 0.27808E+01_JPRB, 0.39949E+01_JPRB ,&
&  0.53139E+01_JPRB, 0.72490E+01_JPRB, 0.72548E+01_JPRB, 0.56955E+01_JPRB, 0.49659E+01_JPRB, 0.46306E+01_JPRB, 0.45228E+01_JPRB ,&
&  0.60456E+01_JPRB, 0.63851E+01_JPRB, 0.60137E+01_JPRB, 0.65190E+01_JPRB, 0.50907E+01_JPRB, 0.42514E+01_JPRB, 0.38311E+01_JPRB ,&
&  0.36861E+01_JPRB, 0.34875E+01_JPRB, 0.32905E+01_JPRB, 0.30638E+01_JPRB, 0.27202E+01_JPRB, 0.24011E+01_JPRB, 0.21856E+01_JPRB ,&
&  0.20566E+01_JPRB, 0.19823E+01_JPRB, 0.19277E+01_JPRB, 0.18956E+01_JPRB, 0.18577E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.16670E+01_JPRB, 0.15356E+01_JPRB, 0.14044E+01_JPRB, 0.13386E+01_JPRB, 0.12840E+01_JPRB, 0.12751E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.12171E+01_JPRB, 0.11152E+01_JPRB, 0.10400E+01_JPRB, 0.10108E+01_JPRB, 0.10141E+01_JPRB, 0.10213E+01_JPRB ,&
&  0.10306E+01_JPRB, 0.10637E+01_JPRB, 0.11607E+01_JPRB, 0.13075E+01_JPRB, 0.16533E+01_JPRB, 0.20733E+01_JPRB, 0.22035E+01_JPRB ,&
&  0.27267E+01_JPRB, 0.39675E+01_JPRB, 0.29997E+01_JPRB, 0.24607E+01_JPRB, 0.23060E+01_JPRB, 0.25674E+01_JPRB, 0.21389E+01_JPRB ,&
&  0.16549E+01_JPRB, 0.15178E+01_JPRB, 0.14766E+01_JPRB, 0.13623E+01_JPRB, 0.13232E+01_JPRB, 0.12790E+01_JPRB, 0.12474E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.11782E+01_JPRB, 0.11410E+01_JPRB, 0.11484E+01_JPRB, 0.11944E+01_JPRB, 0.12670E+01_JPRB, 0.13777E+01_JPRB ,&
&  0.14816E+01_JPRB, 0.14578E+01_JPRB, 0.13734E+01_JPRB, 0.13337E+01_JPRB, 0.13203E+01_JPRB, 0.13317E+01_JPRB, 0.13377E+01_JPRB ,&
&  0.13052E+01_JPRB, 0.12662E+01_JPRB, 0.13759E+01_JPRB, 0.14309E+01_JPRB, 0.14535E+01_JPRB, 0.15481E+01_JPRB, 0.15299E+01_JPRB ,&
&  0.18026E+01_JPRB, 0.18542E+01_JPRB, 0.22609E+01_JPRB, 0.20726E+01_JPRB, 0.23060E+01_JPRB, 0.25097E+01_JPRB, 0.24013E+01_JPRB ,&
&  0.20851E+01_JPRB, 0.19858E+01_JPRB, 0.18656E+01_JPRB, 0.20007E+01_JPRB, 0.24514E+01_JPRB, 0.33859E+01_JPRB, 0.43427E+01_JPRB ,&
&  0.53915E+01_JPRB, 0.64343E+01_JPRB, 0.72211E+01_JPRB, 0.57823E+01_JPRB, 0.52200E+01_JPRB, 0.50302E+01_JPRB, 0.52874E+01_JPRB ,&
&  0.63667E+01_JPRB, 0.65126E+01_JPRB, 0.65348E+01_JPRB, 0.60094E+01_JPRB, 0.46503E+01_JPRB, 0.39253E+01_JPRB, 0.35721E+01_JPRB ,&
&  0.33675E+01_JPRB, 0.31737E+01_JPRB, 0.28389E+01_JPRB, 0.25690E+01_JPRB, 0.23728E+01_JPRB, 0.22115E+01_JPRB, 0.20512E+01_JPRB ,&
&  0.19551E+01_JPRB, 0.18800E+01_JPRB, 0.18450E+01_JPRB, 0.18293E+01_JPRB, 0.17740E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.14975E+01_JPRB, 0.14015E+01_JPRB, 0.13359E+01_JPRB, 0.13123E+01_JPRB, 0.12781E+01_JPRB, 0.12990E+01_JPRB, 0.13537E+01_JPRB ,&
&  0.14898E+01_JPRB, 0.11785E+01_JPRB, 0.10735E+01_JPRB, 0.10355E+01_JPRB, 0.10258E+01_JPRB, 0.10230E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.11467E+01_JPRB, 0.12911E+01_JPRB, 0.14717E+01_JPRB, 0.18348E+01_JPRB, 0.23730E+01_JPRB, 0.29286E+01_JPRB ,&
&  0.34651E+01_JPRB, 0.33843E+01_JPRB, 0.29010E+01_JPRB, 0.23853E+01_JPRB, 0.22121E+01_JPRB, 0.24013E+01_JPRB, 0.23069E+01_JPRB ,&
&  0.19458E+01_JPRB, 0.16398E+01_JPRB, 0.15105E+01_JPRB, 0.13735E+01_JPRB, 0.12803E+01_JPRB, 0.12553E+01_JPRB, 0.12224E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.11427E+01_JPRB, 0.11320E+01_JPRB, 0.11756E+01_JPRB, 0.12466E+01_JPRB, 0.13561E+01_JPRB, 0.15175E+01_JPRB ,&
&  0.15570E+01_JPRB, 0.14841E+01_JPRB, 0.14199E+01_JPRB, 0.13486E+01_JPRB, 0.13508E+01_JPRB, 0.13603E+01_JPRB, 0.13592E+01_JPRB ,&
&  0.13452E+01_JPRB, 0.13743E+01_JPRB, 0.14066E+01_JPRB, 0.14063E+01_JPRB, 0.13664E+01_JPRB, 0.15184E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.17630E+01_JPRB, 0.19494E+01_JPRB, 0.24400E+01_JPRB, 0.20455E+01_JPRB, 0.23757E+01_JPRB, 0.24402E+01_JPRB, 0.22566E+01_JPRB ,&
&  0.20007E+01_JPRB, 0.19265E+01_JPRB, 0.19947E+01_JPRB, 0.22531E+01_JPRB, 0.30475E+01_JPRB, 0.40524E+01_JPRB, 0.58476E+01_JPRB ,&
&  0.59006E+01_JPRB, 0.66315E+01_JPRB, 0.62247E+01_JPRB, 0.61075E+01_JPRB, 0.54376E+01_JPRB, 0.52774E+01_JPRB, 0.53681E+01_JPRB ,&
&  0.58932E+01_JPRB, 0.58165E+01_JPRB, 0.53436E+01_JPRB, 0.47373E+01_JPRB, 0.38547E+01_JPRB, 0.32854E+01_JPRB, 0.29966E+01_JPRB ,&
&  0.27172E+01_JPRB, 0.24569E+01_JPRB, 0.22540E+01_JPRB, 0.21692E+01_JPRB, 0.20713E+01_JPRB, 0.19606E+01_JPRB, 0.18712E+01_JPRB ,&
&  0.18185E+01_JPRB, 0.17706E+01_JPRB, 0.17308E+01_JPRB, 0.16801E+01_JPRB, 0.16083E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.13534E+01_JPRB, 0.12998E+01_JPRB, 0.12704E+01_JPRB, 0.12298E+01_JPRB, 0.11907E+01_JPRB, 0.11843E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.10737E+01_JPRB, 0.10569E+01_JPRB, 0.10555E+01_JPRB, 0.10437E+01_JPRB, 0.10321E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.12289E+01_JPRB, 0.13890E+01_JPRB, 0.16507E+01_JPRB, 0.19953E+01_JPRB, 0.25137E+01_JPRB, 0.30871E+01_JPRB ,&
&  0.31699E+01_JPRB, 0.22473E+01_JPRB, 0.24697E+01_JPRB, 0.23518E+01_JPRB, 0.21184E+01_JPRB, 0.22056E+01_JPRB, 0.21603E+01_JPRB ,&
&  0.17394E+01_JPRB, 0.15997E+01_JPRB, 0.15699E+01_JPRB, 0.14825E+01_JPRB, 0.12579E+01_JPRB, 0.12288E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11407E+01_JPRB, 0.11614E+01_JPRB, 0.12510E+01_JPRB, 0.13700E+01_JPRB, 0.15372E+01_JPRB, 0.16341E+01_JPRB ,&
&  0.15953E+01_JPRB, 0.15505E+01_JPRB, 0.15256E+01_JPRB, 0.14513E+01_JPRB, 0.14426E+01_JPRB, 0.14521E+01_JPRB, 0.14475E+01_JPRB ,&
&  0.14357E+01_JPRB, 0.14219E+01_JPRB, 0.14641E+01_JPRB, 0.15417E+01_JPRB, 0.16070E+01_JPRB, 0.15354E+01_JPRB, 0.16877E+01_JPRB ,&
&  0.18477E+01_JPRB, 0.21651E+01_JPRB, 0.24158E+01_JPRB, 0.27672E+01_JPRB, 0.20610E+01_JPRB, 0.22181E+01_JPRB, 0.19364E+01_JPRB ,&
&  0.20467E+01_JPRB, 0.21862E+01_JPRB, 0.25517E+01_JPRB, 0.30753E+01_JPRB, 0.41947E+01_JPRB, 0.52442E+01_JPRB, 0.68760E+01_JPRB ,&
&  0.60540E+01_JPRB, 0.57581E+01_JPRB, 0.55082E+01_JPRB, 0.59524E+01_JPRB, 0.56275E+01_JPRB, 0.57974E+01_JPRB, 0.53255E+01_JPRB ,&
&  0.52787E+01_JPRB, 0.47038E+01_JPRB, 0.41429E+01_JPRB, 0.37623E+01_JPRB, 0.30731E+01_JPRB, 0.24781E+01_JPRB, 0.21424E+01_JPRB ,&
&  0.19539E+01_JPRB, 0.18965E+01_JPRB, 0.18610E+01_JPRB, 0.18333E+01_JPRB, 0.17803E+01_JPRB, 0.17302E+01_JPRB, 0.16912E+01_JPRB ,&
&  0.16536E+01_JPRB, 0.16037E+01_JPRB, 0.15553E+01_JPRB, 0.15030E+01_JPRB, 0.14367E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.12572E+01_JPRB, 0.12159E+01_JPRB, 0.11917E+01_JPRB, 0.11577E+01_JPRB, 0.11328E+01_JPRB, 0.11241E+01_JPRB, 0.11162E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10811E+01_JPRB, 0.10784E+01_JPRB, 0.10638E+01_JPRB, 0.10384E+01_JPRB, 0.10394E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.13755E+01_JPRB, 0.15753E+01_JPRB, 0.17899E+01_JPRB, 0.21230E+01_JPRB, 0.24010E+01_JPRB, 0.25308E+01_JPRB ,&
&  0.27150E+01_JPRB, 0.27659E+01_JPRB, 0.23376E+01_JPRB, 0.16836E+01_JPRB, 0.15592E+01_JPRB, 0.18718E+01_JPRB, 0.16684E+01_JPRB ,&
&  0.14912E+01_JPRB, 0.14159E+01_JPRB, 0.13756E+01_JPRB, 0.13276E+01_JPRB, 0.12490E+01_JPRB, 0.12116E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.12010E+01_JPRB, 0.12827E+01_JPRB, 0.14444E+01_JPRB, 0.16051E+01_JPRB, 0.17108E+01_JPRB, 0.17163E+01_JPRB ,&
&  0.17093E+01_JPRB, 0.17226E+01_JPRB, 0.17120E+01_JPRB, 0.16481E+01_JPRB, 0.15950E+01_JPRB, 0.16058E+01_JPRB, 0.16113E+01_JPRB ,&
&  0.16104E+01_JPRB, 0.16033E+01_JPRB, 0.16032E+01_JPRB, 0.16655E+01_JPRB, 0.17075E+01_JPRB, 0.16090E+01_JPRB, 0.16983E+01_JPRB ,&
&  0.20074E+01_JPRB, 0.24720E+01_JPRB, 0.25136E+01_JPRB, 0.29812E+01_JPRB, 0.18628E+01_JPRB, 0.20622E+01_JPRB, 0.22665E+01_JPRB ,&
&  0.23407E+01_JPRB, 0.28268E+01_JPRB, 0.33287E+01_JPRB, 0.40092E+01_JPRB, 0.50579E+01_JPRB, 0.57248E+01_JPRB, 0.63590E+01_JPRB ,&
&  0.53575E+01_JPRB, 0.48114E+01_JPRB, 0.48097E+01_JPRB, 0.51602E+01_JPRB, 0.53111E+01_JPRB, 0.52811E+01_JPRB, 0.44313E+01_JPRB ,&
&  0.37089E+01_JPRB, 0.37517E+01_JPRB, 0.32662E+01_JPRB, 0.30880E+01_JPRB, 0.22830E+01_JPRB, 0.17988E+01_JPRB, 0.17181E+01_JPRB ,&
&  0.16482E+01_JPRB, 0.15971E+01_JPRB, 0.15874E+01_JPRB, 0.15707E+01_JPRB, 0.15502E+01_JPRB, 0.15314E+01_JPRB, 0.14981E+01_JPRB ,&
&  0.14670E+01_JPRB, 0.14400E+01_JPRB, 0.14067E+01_JPRB, 0.13670E+01_JPRB, 0.13166E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.11853E+01_JPRB, 0.11569E+01_JPRB, 0.11462E+01_JPRB, 0.11054E+01_JPRB, 0.10708E+01_JPRB, 0.10647E+01_JPRB, 0.10760E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.10609E+01_JPRB, 0.10457E+01_JPRB, 0.10349E+01_JPRB, 0.10511E+01_JPRB, 0.11074E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.15007E+01_JPRB, 0.17053E+01_JPRB, 0.19094E+01_JPRB, 0.21781E+01_JPRB, 0.22905E+01_JPRB, 0.22791E+01_JPRB ,&
&  0.23509E+01_JPRB, 0.24481E+01_JPRB, 0.23998E+01_JPRB, 0.23538E+01_JPRB, 0.19546E+01_JPRB, 0.21116E+01_JPRB, 0.21564E+01_JPRB ,&
&  0.20445E+01_JPRB, 0.16430E+01_JPRB, 0.13765E+01_JPRB, 0.13169E+01_JPRB, 0.12619E+01_JPRB, 0.12436E+01_JPRB, 0.12618E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.13972E+01_JPRB, 0.15374E+01_JPRB, 0.17004E+01_JPRB, 0.18095E+01_JPRB, 0.18531E+01_JPRB, 0.18701E+01_JPRB ,&
&  0.18967E+01_JPRB, 0.19239E+01_JPRB, 0.19524E+01_JPRB, 0.19035E+01_JPRB, 0.18912E+01_JPRB, 0.18908E+01_JPRB, 0.19281E+01_JPRB ,&
&  0.19661E+01_JPRB, 0.18826E+01_JPRB, 0.18894E+01_JPRB, 0.19824E+01_JPRB, 0.19795E+01_JPRB, 0.19470E+01_JPRB, 0.20314E+01_JPRB ,&
&  0.23545E+01_JPRB, 0.28977E+01_JPRB, 0.23874E+01_JPRB, 0.25619E+01_JPRB, 0.30150E+01_JPRB, 0.25726E+01_JPRB, 0.24406E+01_JPRB ,&
&  0.27875E+01_JPRB, 0.32267E+01_JPRB, 0.36942E+01_JPRB, 0.43129E+01_JPRB, 0.52432E+01_JPRB, 0.64237E+01_JPRB, 0.53477E+01_JPRB ,&
&  0.45173E+01_JPRB, 0.40341E+01_JPRB, 0.41221E+01_JPRB, 0.42767E+01_JPRB, 0.42989E+01_JPRB, 0.41746E+01_JPRB, 0.35336E+01_JPRB ,&
&  0.30627E+01_JPRB, 0.30842E+01_JPRB, 0.28008E+01_JPRB, 0.24893E+01_JPRB, 0.22441E+01_JPRB, 0.16233E+01_JPRB, 0.15277E+01_JPRB ,&
&  0.14250E+01_JPRB, 0.14168E+01_JPRB, 0.14245E+01_JPRB, 0.14214E+01_JPRB, 0.14076E+01_JPRB, 0.13800E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.13111E+01_JPRB, 0.12913E+01_JPRB, 0.12687E+01_JPRB, 0.12265E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.11328E+01_JPRB, 0.11159E+01_JPRB, 0.11005E+01_JPRB, 0.10667E+01_JPRB, 0.10309E+01_JPRB, 0.10242E+01_JPRB, 0.10209E+01_JPRB ,&
&  0.99955E+00_JPRB, 0.97628E+00_JPRB, 0.94968E+00_JPRB, 0.96821E+00_JPRB, 0.10121E+01_JPRB, 0.10950E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.14507E+01_JPRB, 0.16621E+01_JPRB, 0.18379E+01_JPRB, 0.20857E+01_JPRB, 0.22248E+01_JPRB, 0.23106E+01_JPRB ,&
&  0.24142E+01_JPRB, 0.25323E+01_JPRB, 0.26227E+01_JPRB, 0.26855E+01_JPRB, 0.26967E+01_JPRB, 0.27683E+01_JPRB, 0.29471E+01_JPRB ,&
&  0.24646E+01_JPRB, 0.17698E+01_JPRB, 0.15297E+01_JPRB, 0.13880E+01_JPRB, 0.13532E+01_JPRB, 0.13664E+01_JPRB, 0.14242E+01_JPRB ,&
&  0.15221E+01_JPRB, 0.16319E+01_JPRB, 0.17723E+01_JPRB, 0.18835E+01_JPRB, 0.19769E+01_JPRB, 0.20414E+01_JPRB, 0.20822E+01_JPRB ,&
&  0.21547E+01_JPRB, 0.22304E+01_JPRB, 0.23477E+01_JPRB, 0.22473E+01_JPRB, 0.24180E+01_JPRB, 0.24221E+01_JPRB, 0.24362E+01_JPRB ,&
&  0.23795E+01_JPRB, 0.22320E+01_JPRB, 0.23027E+01_JPRB, 0.23915E+01_JPRB, 0.24857E+01_JPRB, 0.25108E+01_JPRB, 0.25285E+01_JPRB ,&
&  0.28529E+01_JPRB, 0.29671E+01_JPRB, 0.23171E+01_JPRB, 0.21400E+01_JPRB, 0.22727E+01_JPRB, 0.20717E+01_JPRB, 0.26088E+01_JPRB ,&
&  0.28506E+01_JPRB, 0.31892E+01_JPRB, 0.35398E+01_JPRB, 0.40790E+01_JPRB, 0.47051E+01_JPRB, 0.51782E+01_JPRB, 0.45162E+01_JPRB ,&
&  0.35365E+01_JPRB, 0.32040E+01_JPRB, 0.32162E+01_JPRB, 0.31999E+01_JPRB, 0.31916E+01_JPRB, 0.31617E+01_JPRB, 0.29826E+01_JPRB ,&
&  0.27480E+01_JPRB, 0.24924E+01_JPRB, 0.22791E+01_JPRB, 0.18203E+01_JPRB, 0.15624E+01_JPRB, 0.14069E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.13107E+01_JPRB, 0.13239E+01_JPRB, 0.13225E+01_JPRB, 0.13124E+01_JPRB, 0.12818E+01_JPRB, 0.12612E+01_JPRB ,&
&  0.12502E+01_JPRB, 0.12314E+01_JPRB, 0.12127E+01_JPRB, 0.11968E+01_JPRB, 0.11626E+01_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.10510E+01_JPRB, 0.10231E+01_JPRB, 0.10032E+01_JPRB, 0.98231E+00_JPRB, 0.94813E+00_JPRB, 0.92453E+00_JPRB, 0.91631E+00_JPRB ,&
&  0.87615E+00_JPRB, 0.87614E+00_JPRB, 0.85968E+00_JPRB, 0.87529E+00_JPRB, 0.92041E+00_JPRB, 0.96491E+00_JPRB, 0.10277E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.11516E+01_JPRB, 0.12746E+01_JPRB, 0.15354E+01_JPRB, 0.18347E+01_JPRB, 0.20710E+01_JPRB, 0.23519E+01_JPRB ,&
&  0.24915E+01_JPRB, 0.26201E+01_JPRB, 0.28439E+01_JPRB, 0.31558E+01_JPRB, 0.33995E+01_JPRB, 0.36229E+01_JPRB, 0.36088E+01_JPRB ,&
&  0.26993E+01_JPRB, 0.16449E+01_JPRB, 0.15548E+01_JPRB, 0.14147E+01_JPRB, 0.14381E+01_JPRB, 0.14936E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.16597E+01_JPRB, 0.17639E+01_JPRB, 0.18742E+01_JPRB, 0.19663E+01_JPRB, 0.20806E+01_JPRB, 0.22047E+01_JPRB, 0.22681E+01_JPRB ,&
&  0.23250E+01_JPRB, 0.23861E+01_JPRB, 0.24411E+01_JPRB, 0.23888E+01_JPRB, 0.24983E+01_JPRB, 0.26616E+01_JPRB, 0.27880E+01_JPRB ,&
&  0.29101E+01_JPRB, 0.29147E+01_JPRB, 0.24975E+01_JPRB, 0.25719E+01_JPRB, 0.29035E+01_JPRB, 0.27572E+01_JPRB, 0.28684E+01_JPRB ,&
&  0.33349E+01_JPRB, 0.34596E+01_JPRB, 0.21901E+01_JPRB, 0.19962E+01_JPRB, 0.21322E+01_JPRB, 0.22728E+01_JPRB, 0.24926E+01_JPRB ,&
&  0.26620E+01_JPRB, 0.29535E+01_JPRB, 0.32685E+01_JPRB, 0.35425E+01_JPRB, 0.36940E+01_JPRB, 0.35224E+01_JPRB, 0.32984E+01_JPRB ,&
&  0.27232E+01_JPRB, 0.24924E+01_JPRB, 0.25085E+01_JPRB, 0.25553E+01_JPRB, 0.26576E+01_JPRB, 0.27738E+01_JPRB, 0.24952E+01_JPRB ,&
&  0.23868E+01_JPRB, 0.23384E+01_JPRB, 0.25342E+01_JPRB, 0.20462E+01_JPRB, 0.14393E+01_JPRB, 0.12529E+01_JPRB, 0.11581E+01_JPRB ,&
&  0.12096E+01_JPRB, 0.12469E+01_JPRB, 0.12642E+01_JPRB, 0.12744E+01_JPRB, 0.12685E+01_JPRB, 0.12521E+01_JPRB, 0.12318E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.11780E+01_JPRB, 0.11468E+01_JPRB, 0.11147E+01_JPRB, 0.10850E+01_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.91253E+00_JPRB, 0.88536E+00_JPRB, 0.87407E+00_JPRB, 0.82619E+00_JPRB, 0.83988E+00_JPRB, 0.80605E+00_JPRB, 0.84480E+00_JPRB ,&
&  0.81992E+00_JPRB, 0.82043E+00_JPRB, 0.82480E+00_JPRB, 0.84550E+00_JPRB, 0.86602E+00_JPRB, 0.88991E+00_JPRB, 0.91018E+00_JPRB ,&
&  0.93502E+00_JPRB, 0.99104E+00_JPRB, 0.10752E+01_JPRB, 0.11784E+01_JPRB, 0.13148E+01_JPRB, 0.14569E+01_JPRB, 0.15892E+01_JPRB ,&
&  0.17641E+01_JPRB, 0.19229E+01_JPRB, 0.20347E+01_JPRB, 0.22829E+01_JPRB, 0.28848E+01_JPRB, 0.36216E+01_JPRB, 0.37424E+01_JPRB ,&
&  0.23888E+01_JPRB, 0.16737E+01_JPRB, 0.14434E+01_JPRB, 0.13269E+01_JPRB, 0.13884E+01_JPRB, 0.15309E+01_JPRB, 0.15790E+01_JPRB ,&
&  0.16340E+01_JPRB, 0.17033E+01_JPRB, 0.17967E+01_JPRB, 0.18972E+01_JPRB, 0.20246E+01_JPRB, 0.21561E+01_JPRB, 0.22173E+01_JPRB ,&
&  0.22172E+01_JPRB, 0.22580E+01_JPRB, 0.22692E+01_JPRB, 0.24109E+01_JPRB, 0.26890E+01_JPRB, 0.30197E+01_JPRB, 0.32419E+01_JPRB ,&
&  0.33629E+01_JPRB, 0.33154E+01_JPRB, 0.28515E+01_JPRB, 0.28813E+01_JPRB, 0.30390E+01_JPRB, 0.30251E+01_JPRB, 0.29644E+01_JPRB ,&
&  0.32795E+01_JPRB, 0.32170E+01_JPRB, 0.23462E+01_JPRB, 0.22430E+01_JPRB, 0.22467E+01_JPRB, 0.22939E+01_JPRB, 0.22331E+01_JPRB ,&
&  0.24485E+01_JPRB, 0.26742E+01_JPRB, 0.28306E+01_JPRB, 0.28604E+01_JPRB, 0.27614E+01_JPRB, 0.25633E+01_JPRB, 0.25333E+01_JPRB ,&
&  0.23234E+01_JPRB, 0.22553E+01_JPRB, 0.22800E+01_JPRB, 0.23510E+01_JPRB, 0.27228E+01_JPRB, 0.24590E+01_JPRB, 0.23914E+01_JPRB ,&
&  0.20721E+01_JPRB, 0.20346E+01_JPRB, 0.16955E+01_JPRB, 0.19571E+01_JPRB, 0.13096E+01_JPRB, 0.10470E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10767E+01_JPRB, 0.11199E+01_JPRB, 0.11208E+01_JPRB, 0.11370E+01_JPRB, 0.11419E+01_JPRB, 0.11266E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.10771E+01_JPRB, 0.10334E+01_JPRB, 0.99770E+00_JPRB, 0.95950E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.86851E+00_JPRB, 0.86478E+00_JPRB, 0.83810E+00_JPRB, 0.79594E+00_JPRB, 0.76928E+00_JPRB, 0.74933E+00_JPRB, 0.75927E+00_JPRB ,&
&  0.77078E+00_JPRB, 0.79765E+00_JPRB, 0.82217E+00_JPRB, 0.84625E+00_JPRB, 0.85716E+00_JPRB, 0.86461E+00_JPRB, 0.88550E+00_JPRB ,&
&  0.91371E+00_JPRB, 0.94065E+00_JPRB, 0.97789E+00_JPRB, 0.10285E+01_JPRB, 0.10818E+01_JPRB, 0.11504E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.12014E+01_JPRB, 0.12511E+01_JPRB, 0.13005E+01_JPRB, 0.13925E+01_JPRB, 0.17261E+01_JPRB, 0.20982E+01_JPRB, 0.26223E+01_JPRB ,&
&  0.19865E+01_JPRB, 0.17911E+01_JPRB, 0.15086E+01_JPRB, 0.13799E+01_JPRB, 0.14083E+01_JPRB, 0.13858E+01_JPRB, 0.14002E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.14477E+01_JPRB, 0.14686E+01_JPRB, 0.14944E+01_JPRB, 0.15278E+01_JPRB, 0.16042E+01_JPRB, 0.15612E+01_JPRB ,&
&  0.15435E+01_JPRB, 0.16038E+01_JPRB, 0.16830E+01_JPRB, 0.19167E+01_JPRB, 0.22564E+01_JPRB, 0.25748E+01_JPRB, 0.26033E+01_JPRB ,&
&  0.26289E+01_JPRB, 0.27988E+01_JPRB, 0.27424E+01_JPRB, 0.27588E+01_JPRB, 0.27329E+01_JPRB, 0.26768E+01_JPRB, 0.27264E+01_JPRB ,&
&  0.26454E+01_JPRB, 0.25696E+01_JPRB, 0.25320E+01_JPRB, 0.23011E+01_JPRB, 0.21626E+01_JPRB, 0.20405E+01_JPRB, 0.20348E+01_JPRB ,&
&  0.21229E+01_JPRB, 0.22514E+01_JPRB, 0.23174E+01_JPRB, 0.22300E+01_JPRB, 0.21331E+01_JPRB, 0.20081E+01_JPRB, 0.20460E+01_JPRB ,&
&  0.20458E+01_JPRB, 0.19291E+01_JPRB, 0.19464E+01_JPRB, 0.21345E+01_JPRB, 0.23706E+01_JPRB, 0.39070E+01_JPRB, 0.34007E+01_JPRB ,&
&  0.19465E+01_JPRB, 0.15773E+01_JPRB, 0.10474E+01_JPRB, 0.15045E+01_JPRB, 0.10001E+01_JPRB, 0.90755E+00_JPRB, 0.93168E+00_JPRB ,&
&  0.94821E+00_JPRB, 0.96544E+00_JPRB, 0.97627E+00_JPRB, 0.97971E+00_JPRB, 0.98045E+00_JPRB, 0.98247E+00_JPRB, 0.96801E+00_JPRB ,&
&  0.95606E+00_JPRB, 0.93742E+00_JPRB, 0.93265E+00_JPRB, 0.89590E+00_JPRB, 0.87864E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.82351E+00_JPRB, 0.82107E+00_JPRB, 0.78567E+00_JPRB, 0.73584E+00_JPRB, 0.71371E+00_JPRB, 0.71788E+00_JPRB, 0.73601E+00_JPRB ,&
&  0.75485E+00_JPRB, 0.77938E+00_JPRB, 0.80848E+00_JPRB, 0.83742E+00_JPRB, 0.86401E+00_JPRB, 0.89297E+00_JPRB, 0.92752E+00_JPRB ,&
&  0.96035E+00_JPRB, 0.10003E+01_JPRB, 0.10421E+01_JPRB, 0.10868E+01_JPRB, 0.11290E+01_JPRB, 0.12001E+01_JPRB, 0.12560E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.14152E+01_JPRB, 0.15157E+01_JPRB, 0.16240E+01_JPRB, 0.17670E+01_JPRB, 0.19095E+01_JPRB, 0.18081E+01_JPRB ,&
&  0.21014E+01_JPRB, 0.17967E+01_JPRB, 0.15449E+01_JPRB, 0.14061E+01_JPRB, 0.13278E+01_JPRB, 0.12672E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.11243E+01_JPRB, 0.10778E+01_JPRB, 0.10392E+01_JPRB, 0.10159E+01_JPRB, 0.10790E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.13199E+01_JPRB, 0.14515E+01_JPRB, 0.16672E+01_JPRB, 0.18482E+01_JPRB, 0.19739E+01_JPRB, 0.19028E+01_JPRB ,&
&  0.19216E+01_JPRB, 0.20817E+01_JPRB, 0.20521E+01_JPRB, 0.20989E+01_JPRB, 0.20905E+01_JPRB, 0.20214E+01_JPRB, 0.19958E+01_JPRB ,&
&  0.18263E+01_JPRB, 0.19146E+01_JPRB, 0.21364E+01_JPRB, 0.23776E+01_JPRB, 0.20894E+01_JPRB, 0.19031E+01_JPRB, 0.19137E+01_JPRB ,&
&  0.18708E+01_JPRB, 0.19037E+01_JPRB, 0.19162E+01_JPRB, 0.18131E+01_JPRB, 0.17259E+01_JPRB, 0.16540E+01_JPRB, 0.17049E+01_JPRB ,&
&  0.17437E+01_JPRB, 0.16263E+01_JPRB, 0.17129E+01_JPRB, 0.20106E+01_JPRB, 0.24505E+01_JPRB, 0.26810E+01_JPRB, 0.32416E+01_JPRB ,&
&  0.18665E+01_JPRB, 0.10736E+01_JPRB, 0.10624E+01_JPRB, 0.10915E+01_JPRB, 0.86905E+00_JPRB, 0.76376E+00_JPRB, 0.81289E+00_JPRB ,&
&  0.84937E+00_JPRB, 0.87303E+00_JPRB, 0.85721E+00_JPRB, 0.88160E+00_JPRB, 0.93381E+00_JPRB, 0.93016E+00_JPRB, 0.88044E+00_JPRB ,&
&  0.89086E+00_JPRB, 0.85451E+00_JPRB, 0.83827E+00_JPRB, 0.83495E+00_JPRB, 0.83165E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.76964E+00_JPRB, 0.75596E+00_JPRB, 0.71678E+00_JPRB, 0.67166E+00_JPRB, 0.65395E+00_JPRB, 0.66970E+00_JPRB, 0.69700E+00_JPRB ,&
&  0.72244E+00_JPRB, 0.75258E+00_JPRB, 0.78358E+00_JPRB, 0.81834E+00_JPRB, 0.85478E+00_JPRB, 0.89035E+00_JPRB, 0.91865E+00_JPRB ,&
&  0.94177E+00_JPRB, 0.97202E+00_JPRB, 0.10011E+01_JPRB, 0.10301E+01_JPRB, 0.10538E+01_JPRB, 0.10688E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.11553E+01_JPRB, 0.12301E+01_JPRB, 0.13244E+01_JPRB, 0.14300E+01_JPRB, 0.15293E+01_JPRB, 0.16709E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.18382E+01_JPRB, 0.17234E+01_JPRB, 0.15578E+01_JPRB, 0.14161E+01_JPRB, 0.13085E+01_JPRB, 0.12285E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.11267E+01_JPRB, 0.11652E+01_JPRB, 0.11080E+01_JPRB, 0.11194E+01_JPRB, 0.95654E+00_JPRB, 0.10028E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.13691E+01_JPRB, 0.15054E+01_JPRB, 0.16261E+01_JPRB, 0.16809E+01_JPRB, 0.17222E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.16448E+01_JPRB, 0.16396E+01_JPRB, 0.15329E+01_JPRB, 0.15607E+01_JPRB, 0.16110E+01_JPRB, 0.14247E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.14714E+01_JPRB, 0.20425E+01_JPRB, 0.23684E+01_JPRB, 0.20380E+01_JPRB, 0.19262E+01_JPRB, 0.18209E+01_JPRB ,&
&  0.16897E+01_JPRB, 0.16681E+01_JPRB, 0.16397E+01_JPRB, 0.15620E+01_JPRB, 0.14301E+01_JPRB, 0.13542E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.14448E+01_JPRB, 0.13537E+01_JPRB, 0.13563E+01_JPRB, 0.16350E+01_JPRB, 0.18570E+01_JPRB, 0.17979E+01_JPRB, 0.30815E+01_JPRB ,&
&  0.18364E+01_JPRB, 0.13046E+01_JPRB, 0.17920E+01_JPRB, 0.70797E+00_JPRB, 0.73790E+00_JPRB, 0.66578E+00_JPRB, 0.67783E+00_JPRB ,&
&  0.57638E+00_JPRB, 0.67946E+00_JPRB, 0.10531E+01_JPRB, 0.13398E+01_JPRB, 0.15295E+01_JPRB, 0.15305E+01_JPRB, 0.93345E+00_JPRB ,&
&  0.84040E+00_JPRB, 0.82170E+00_JPRB, 0.80934E+00_JPRB, 0.79111E+00_JPRB, 0.77061E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.70453E+00_JPRB, 0.66563E+00_JPRB, 0.64387E+00_JPRB, 0.62627E+00_JPRB, 0.61526E+00_JPRB, 0.63600E+00_JPRB, 0.65506E+00_JPRB ,&
&  0.68423E+00_JPRB, 0.71866E+00_JPRB, 0.75963E+00_JPRB, 0.79700E+00_JPRB, 0.84532E+00_JPRB, 0.89479E+00_JPRB, 0.93970E+00_JPRB ,&
&  0.98156E+00_JPRB, 0.10216E+01_JPRB, 0.10632E+01_JPRB, 0.10926E+01_JPRB, 0.11218E+01_JPRB, 0.11490E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.11986E+01_JPRB, 0.12722E+01_JPRB, 0.13863E+01_JPRB, 0.15144E+01_JPRB, 0.16284E+01_JPRB, 0.18130E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.15826E+01_JPRB, 0.16024E+01_JPRB, 0.15556E+01_JPRB, 0.14468E+01_JPRB, 0.13473E+01_JPRB, 0.12842E+01_JPRB, 0.12595E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.13311E+01_JPRB, 0.12862E+01_JPRB, 0.13137E+01_JPRB, 0.12433E+01_JPRB, 0.11772E+01_JPRB, 0.13008E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.15529E+01_JPRB, 0.16526E+01_JPRB, 0.17269E+01_JPRB, 0.17802E+01_JPRB, 0.18261E+01_JPRB, 0.18419E+01_JPRB ,&
&  0.18461E+01_JPRB, 0.17938E+01_JPRB, 0.17204E+01_JPRB, 0.17512E+01_JPRB, 0.17713E+01_JPRB, 0.15323E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.10937E+01_JPRB, 0.17459E+01_JPRB, 0.22565E+01_JPRB, 0.20585E+01_JPRB, 0.19424E+01_JPRB, 0.17408E+01_JPRB ,&
&  0.15516E+01_JPRB, 0.14540E+01_JPRB, 0.13972E+01_JPRB, 0.13242E+01_JPRB, 0.12233E+01_JPRB, 0.11585E+01_JPRB, 0.11593E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.10533E+01_JPRB, 0.10306E+01_JPRB, 0.11714E+01_JPRB, 0.12911E+01_JPRB, 0.15737E+01_JPRB, 0.16846E+01_JPRB ,&
&  0.19328E+01_JPRB, 0.17191E+01_JPRB, 0.12715E+01_JPRB, 0.11206E+01_JPRB, 0.74590E+00_JPRB, 0.62933E+00_JPRB, 0.59265E+00_JPRB ,&
&  0.60661E+00_JPRB, 0.54788E+00_JPRB, 0.10486E+01_JPRB, 0.38596E+01_JPRB, 0.37803E+01_JPRB, 0.47670E+01_JPRB, 0.25337E+01_JPRB ,&
&  0.12485E+01_JPRB, 0.81961E+00_JPRB, 0.73520E+00_JPRB, 0.73177E+00_JPRB, 0.71711E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.61554E+00_JPRB, 0.59122E+00_JPRB, 0.56493E+00_JPRB, 0.57073E+00_JPRB, 0.57791E+00_JPRB, 0.58779E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.68477E+00_JPRB, 0.70359E+00_JPRB, 0.73638E+00_JPRB, 0.79064E+00_JPRB, 0.87059E+00_JPRB, 0.94462E+00_JPRB, 0.10172E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.11597E+01_JPRB, 0.12042E+01_JPRB, 0.12353E+01_JPRB, 0.12806E+01_JPRB, 0.13312E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.14360E+01_JPRB, 0.15142E+01_JPRB, 0.16460E+01_JPRB, 0.17864E+01_JPRB, 0.19186E+01_JPRB, 0.19709E+01_JPRB, 0.12541E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.14614E+01_JPRB, 0.14995E+01_JPRB, 0.14748E+01_JPRB, 0.13891E+01_JPRB, 0.13256E+01_JPRB, 0.13702E+01_JPRB ,&
&  0.14728E+01_JPRB, 0.15578E+01_JPRB, 0.15097E+01_JPRB, 0.14338E+01_JPRB, 0.15661E+01_JPRB, 0.14666E+01_JPRB, 0.16878E+01_JPRB ,&
&  0.17956E+01_JPRB, 0.18836E+01_JPRB, 0.19981E+01_JPRB, 0.20641E+01_JPRB, 0.21486E+01_JPRB, 0.22515E+01_JPRB, 0.23462E+01_JPRB ,&
&  0.23843E+01_JPRB, 0.23673E+01_JPRB, 0.24189E+01_JPRB, 0.26082E+01_JPRB, 0.24023E+01_JPRB, 0.26503E+01_JPRB, 0.21680E+01_JPRB ,&
&  0.14898E+01_JPRB, 0.12899E+01_JPRB, 0.16560E+01_JPRB, 0.20568E+01_JPRB, 0.19905E+01_JPRB, 0.19145E+01_JPRB, 0.17356E+01_JPRB ,&
&  0.14816E+01_JPRB, 0.13212E+01_JPRB, 0.12566E+01_JPRB, 0.11939E+01_JPRB, 0.11160E+01_JPRB, 0.10487E+01_JPRB, 0.10081E+01_JPRB ,&
&  0.94717E+00_JPRB, 0.88075E+00_JPRB, 0.91190E+00_JPRB, 0.93984E+00_JPRB, 0.10221E+01_JPRB, 0.13647E+01_JPRB, 0.20881E+01_JPRB ,&
&  0.32939E+01_JPRB, 0.18089E+01_JPRB, 0.13232E+01_JPRB, 0.10615E+01_JPRB, 0.80435E+00_JPRB, 0.68109E+00_JPRB, 0.61464E+00_JPRB ,&
&  0.59350E+00_JPRB, 0.71933E+00_JPRB, 0.85461E+00_JPRB, 0.15302E+01_JPRB, 0.26475E+01_JPRB, 0.26888E+01_JPRB, 0.18351E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.81640E+00_JPRB, 0.66435E+00_JPRB, 0.63767E+00_JPRB, 0.61778E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.56198E+00_JPRB, 0.54905E+00_JPRB, 0.54636E+00_JPRB, 0.54297E+00_JPRB, 0.58097E+00_JPRB, 0.60722E+00_JPRB, 0.63296E+00_JPRB ,&
&  0.67402E+00_JPRB, 0.70282E+00_JPRB, 0.74308E+00_JPRB, 0.81932E+00_JPRB, 0.90841E+00_JPRB, 0.99083E+00_JPRB, 0.10479E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.11286E+01_JPRB, 0.11643E+01_JPRB, 0.12008E+01_JPRB, 0.12743E+01_JPRB, 0.13446E+01_JPRB, 0.14487E+01_JPRB ,&
&  0.15747E+01_JPRB, 0.17109E+01_JPRB, 0.18701E+01_JPRB, 0.20166E+01_JPRB, 0.21818E+01_JPRB, 0.23355E+01_JPRB, 0.24662E+01_JPRB ,&
&  0.10709E+01_JPRB, 0.13967E+01_JPRB, 0.14493E+01_JPRB, 0.14457E+01_JPRB, 0.13748E+01_JPRB, 0.13953E+01_JPRB, 0.15737E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.18107E+01_JPRB, 0.19625E+01_JPRB, 0.16109E+01_JPRB, 0.17771E+01_JPRB, 0.18088E+01_JPRB, 0.19917E+01_JPRB ,&
&  0.21184E+01_JPRB, 0.22116E+01_JPRB, 0.23363E+01_JPRB, 0.24696E+01_JPRB, 0.26179E+01_JPRB, 0.27803E+01_JPRB, 0.29815E+01_JPRB ,&
&  0.31720E+01_JPRB, 0.34106E+01_JPRB, 0.37275E+01_JPRB, 0.36985E+01_JPRB, 0.34366E+01_JPRB, 0.36724E+01_JPRB, 0.34842E+01_JPRB ,&
&  0.29387E+01_JPRB, 0.21221E+01_JPRB, 0.20586E+01_JPRB, 0.20570E+01_JPRB, 0.19459E+01_JPRB, 0.18737E+01_JPRB, 0.17054E+01_JPRB ,&
&  0.14602E+01_JPRB, 0.13128E+01_JPRB, 0.12052E+01_JPRB, 0.11345E+01_JPRB, 0.10543E+01_JPRB, 0.99387E+00_JPRB, 0.95361E+00_JPRB ,&
&  0.91619E+00_JPRB, 0.92412E+00_JPRB, 0.94026E+00_JPRB, 0.87335E+00_JPRB, 0.84993E+00_JPRB, 0.96398E+00_JPRB, 0.11875E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.12375E+01_JPRB, 0.10450E+01_JPRB, 0.10265E+01_JPRB, 0.93021E+00_JPRB, 0.82677E+00_JPRB, 0.77888E+00_JPRB ,&
&  0.76608E+00_JPRB, 0.74564E+00_JPRB, 0.85284E+00_JPRB, 0.12785E+01_JPRB, 0.16010E+01_JPRB, 0.18157E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.82660E+00_JPRB, 0.66940E+00_JPRB, 0.62942E+00_JPRB, 0.60870E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.59966E+00_JPRB, 0.58791E+00_JPRB, 0.58901E+00_JPRB, 0.58599E+00_JPRB, 0.62196E+00_JPRB, 0.66331E+00_JPRB, 0.67678E+00_JPRB ,&
&  0.67707E+00_JPRB, 0.70224E+00_JPRB, 0.72849E+00_JPRB, 0.77344E+00_JPRB, 0.85772E+00_JPRB, 0.91239E+00_JPRB, 0.97540E+00_JPRB ,&
&  0.10384E+01_JPRB, 0.10643E+01_JPRB, 0.10719E+01_JPRB, 0.11107E+01_JPRB, 0.12005E+01_JPRB, 0.13016E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.15934E+01_JPRB, 0.17955E+01_JPRB, 0.20461E+01_JPRB, 0.23232E+01_JPRB, 0.26328E+01_JPRB, 0.29389E+01_JPRB, 0.31124E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12417E+01_JPRB, 0.14311E+01_JPRB, 0.14106E+01_JPRB, 0.13709E+01_JPRB, 0.14604E+01_JPRB, 0.16092E+01_JPRB ,&
&  0.16748E+01_JPRB, 0.15167E+01_JPRB, 0.15895E+01_JPRB, 0.19312E+01_JPRB, 0.19197E+01_JPRB, 0.20394E+01_JPRB, 0.21858E+01_JPRB ,&
&  0.23044E+01_JPRB, 0.24347E+01_JPRB, 0.25466E+01_JPRB, 0.26446E+01_JPRB, 0.27717E+01_JPRB, 0.29369E+01_JPRB, 0.31612E+01_JPRB ,&
&  0.34493E+01_JPRB, 0.38759E+01_JPRB, 0.42951E+01_JPRB, 0.39499E+01_JPRB, 0.40635E+01_JPRB, 0.43103E+01_JPRB, 0.48976E+01_JPRB ,&
&  0.43966E+01_JPRB, 0.30740E+01_JPRB, 0.23467E+01_JPRB, 0.20692E+01_JPRB, 0.17608E+01_JPRB, 0.16613E+01_JPRB, 0.15266E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.12891E+01_JPRB, 0.11900E+01_JPRB, 0.11110E+01_JPRB, 0.10161E+01_JPRB, 0.97218E+00_JPRB, 0.94739E+00_JPRB ,&
&  0.98034E+00_JPRB, 0.10099E+01_JPRB, 0.96887E+00_JPRB, 0.90791E+00_JPRB, 0.88941E+00_JPRB, 0.94236E+00_JPRB, 0.10368E+01_JPRB ,&
&  0.10821E+01_JPRB, 0.10646E+01_JPRB, 0.10651E+01_JPRB, 0.10751E+01_JPRB, 0.10730E+01_JPRB, 0.10650E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.96629E+00_JPRB, 0.90981E+00_JPRB, 0.90304E+00_JPRB, 0.88658E+00_JPRB, 0.10391E+01_JPRB, 0.13299E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.11685E+01_JPRB, 0.91117E+00_JPRB, 0.74620E+00_JPRB, 0.69534E+00_JPRB, 0.66957E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.71584E+00_JPRB, 0.67424E+00_JPRB, 0.66512E+00_JPRB, 0.64956E+00_JPRB, 0.66592E+00_JPRB, 0.69636E+00_JPRB, 0.71825E+00_JPRB ,&
&  0.72671E+00_JPRB, 0.72799E+00_JPRB, 0.73403E+00_JPRB, 0.76260E+00_JPRB, 0.79915E+00_JPRB, 0.83897E+00_JPRB, 0.88857E+00_JPRB ,&
&  0.92481E+00_JPRB, 0.94913E+00_JPRB, 0.94112E+00_JPRB, 0.93520E+00_JPRB, 0.99081E+00_JPRB, 0.10739E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.15011E+01_JPRB, 0.17583E+01_JPRB, 0.21546E+01_JPRB, 0.27472E+01_JPRB, 0.34674E+01_JPRB, 0.42220E+01_JPRB ,&
&  0.34247E+01_JPRB, 0.15403E+01_JPRB, 0.16618E+01_JPRB, 0.17070E+01_JPRB, 0.15346E+01_JPRB, 0.16313E+01_JPRB, 0.16787E+01_JPRB ,&
&  0.16365E+01_JPRB, 0.15792E+01_JPRB, 0.14520E+01_JPRB, 0.15416E+01_JPRB, 0.18056E+01_JPRB, 0.19937E+01_JPRB, 0.21616E+01_JPRB ,&
&  0.22908E+01_JPRB, 0.23873E+01_JPRB, 0.24393E+01_JPRB, 0.24929E+01_JPRB, 0.25992E+01_JPRB, 0.27637E+01_JPRB, 0.29880E+01_JPRB ,&
&  0.32976E+01_JPRB, 0.36918E+01_JPRB, 0.41768E+01_JPRB, 0.38216E+01_JPRB, 0.43072E+01_JPRB, 0.43243E+01_JPRB, 0.43877E+01_JPRB ,&
&  0.40436E+01_JPRB, 0.27843E+01_JPRB, 0.21302E+01_JPRB, 0.19751E+01_JPRB, 0.16023E+01_JPRB, 0.12301E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.12420E+01_JPRB, 0.12006E+01_JPRB, 0.11019E+01_JPRB, 0.10140E+01_JPRB, 0.98161E+00_JPRB, 0.10270E+01_JPRB ,&
&  0.10927E+01_JPRB, 0.11298E+01_JPRB, 0.10599E+01_JPRB, 0.10429E+01_JPRB, 0.10177E+01_JPRB, 0.10516E+01_JPRB, 0.11007E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.11746E+01_JPRB, 0.11977E+01_JPRB, 0.12301E+01_JPRB, 0.12649E+01_JPRB, 0.12015E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.10755E+01_JPRB, 0.10151E+01_JPRB, 0.97322E+00_JPRB, 0.93513E+00_JPRB, 0.95945E+00_JPRB, 0.10596E+01_JPRB ,&
&  0.10408E+01_JPRB, 0.92238E+00_JPRB, 0.80357E+00_JPRB, 0.77438E+00_JPRB, 0.78040E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.83297E+00_JPRB, 0.73730E+00_JPRB, 0.68777E+00_JPRB, 0.68344E+00_JPRB, 0.67059E+00_JPRB, 0.67839E+00_JPRB, 0.68982E+00_JPRB ,&
&  0.68798E+00_JPRB, 0.70909E+00_JPRB, 0.70618E+00_JPRB, 0.72516E+00_JPRB, 0.74779E+00_JPRB, 0.75949E+00_JPRB, 0.79284E+00_JPRB ,&
&  0.82408E+00_JPRB, 0.85714E+00_JPRB, 0.89027E+00_JPRB, 0.86933E+00_JPRB, 0.88992E+00_JPRB, 0.92782E+00_JPRB, 0.99547E+00_JPRB ,&
&  0.11140E+01_JPRB, 0.12711E+01_JPRB, 0.14549E+01_JPRB, 0.16771E+01_JPRB, 0.20002E+01_JPRB, 0.25804E+01_JPRB, 0.36424E+01_JPRB ,&
&  0.62456E+01_JPRB, 0.53282E+01_JPRB, 0.15707E+01_JPRB, 0.20040E+01_JPRB, 0.19309E+01_JPRB, 0.19420E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.18114E+01_JPRB, 0.19007E+01_JPRB, 0.15685E+01_JPRB, 0.13590E+01_JPRB, 0.14305E+01_JPRB, 0.17071E+01_JPRB, 0.19323E+01_JPRB ,&
&  0.20673E+01_JPRB, 0.21660E+01_JPRB, 0.21998E+01_JPRB, 0.22325E+01_JPRB, 0.22874E+01_JPRB, 0.24016E+01_JPRB, 0.25857E+01_JPRB ,&
&  0.28628E+01_JPRB, 0.32555E+01_JPRB, 0.37796E+01_JPRB, 0.37076E+01_JPRB, 0.45288E+01_JPRB, 0.43591E+01_JPRB, 0.40747E+01_JPRB ,&
&  0.33814E+01_JPRB, 0.27579E+01_JPRB, 0.23468E+01_JPRB, 0.19161E+01_JPRB, 0.14042E+01_JPRB, 0.11518E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.10804E+01_JPRB, 0.10468E+01_JPRB, 0.10368E+01_JPRB, 0.99139E+00_JPRB, 0.10609E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.12136E+01_JPRB, 0.12095E+01_JPRB, 0.11699E+01_JPRB, 0.11574E+01_JPRB, 0.11500E+01_JPRB, 0.11608E+01_JPRB, 0.11969E+01_JPRB ,&
&  0.12200E+01_JPRB, 0.12488E+01_JPRB, 0.12601E+01_JPRB, 0.12924E+01_JPRB, 0.13240E+01_JPRB, 0.12522E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11503E+01_JPRB, 0.11155E+01_JPRB, 0.10850E+01_JPRB, 0.10200E+01_JPRB, 0.97171E+00_JPRB, 0.95557E+00_JPRB ,&
&  0.10327E+01_JPRB, 0.96601E+00_JPRB, 0.90619E+00_JPRB, 0.84022E+00_JPRB, 0.86369E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.88256E+00_JPRB, 0.82654E+00_JPRB, 0.73650E+00_JPRB, 0.70541E+00_JPRB, 0.69834E+00_JPRB, 0.68361E+00_JPRB, 0.66362E+00_JPRB ,&
&  0.63986E+00_JPRB, 0.63119E+00_JPRB, 0.65987E+00_JPRB, 0.69374E+00_JPRB, 0.72043E+00_JPRB, 0.72763E+00_JPRB, 0.74497E+00_JPRB ,&
&  0.75601E+00_JPRB, 0.78232E+00_JPRB, 0.83131E+00_JPRB, 0.85230E+00_JPRB, 0.85627E+00_JPRB, 0.86862E+00_JPRB, 0.89111E+00_JPRB ,&
&  0.93125E+00_JPRB, 0.10024E+01_JPRB, 0.11262E+01_JPRB, 0.12677E+01_JPRB, 0.14539E+01_JPRB, 0.17842E+01_JPRB, 0.24744E+01_JPRB ,&
&  0.38609E+01_JPRB, 0.71580E+01_JPRB, 0.16901E+01_JPRB, 0.23750E+01_JPRB, 0.23510E+01_JPRB, 0.21935E+01_JPRB, 0.20433E+01_JPRB ,&
&  0.20537E+01_JPRB, 0.20226E+01_JPRB, 0.18116E+01_JPRB, 0.14550E+01_JPRB, 0.11654E+01_JPRB, 0.12832E+01_JPRB, 0.15399E+01_JPRB ,&
&  0.17657E+01_JPRB, 0.18693E+01_JPRB, 0.19043E+01_JPRB, 0.18877E+01_JPRB, 0.19024E+01_JPRB, 0.19450E+01_JPRB, 0.20459E+01_JPRB ,&
&  0.22401E+01_JPRB, 0.25590E+01_JPRB, 0.30219E+01_JPRB, 0.31869E+01_JPRB, 0.41691E+01_JPRB, 0.44374E+01_JPRB, 0.42534E+01_JPRB ,&
&  0.34588E+01_JPRB, 0.31371E+01_JPRB, 0.27857E+01_JPRB, 0.21007E+01_JPRB, 0.15332E+01_JPRB, 0.12423E+01_JPRB, 0.12085E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.10687E+01_JPRB, 0.10472E+01_JPRB, 0.10268E+01_JPRB, 0.10414E+01_JPRB, 0.11204E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.12304E+01_JPRB, 0.11987E+01_JPRB, 0.11930E+01_JPRB, 0.11982E+01_JPRB, 0.12081E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.12667E+01_JPRB, 0.12874E+01_JPRB, 0.12935E+01_JPRB, 0.12669E+01_JPRB, 0.12818E+01_JPRB, 0.12420E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.12360E+01_JPRB, 0.11833E+01_JPRB, 0.11287E+01_JPRB, 0.11362E+01_JPRB, 0.10962E+01_JPRB, 0.10533E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10906E+01_JPRB, 0.10561E+01_JPRB, 0.92655E+00_JPRB, 0.93104E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.93370E+00_JPRB, 0.88327E+00_JPRB, 0.81912E+00_JPRB, 0.74928E+00_JPRB, 0.74629E+00_JPRB, 0.75110E+00_JPRB, 0.71469E+00_JPRB ,&
&  0.65873E+00_JPRB, 0.65247E+00_JPRB, 0.66061E+00_JPRB, 0.68484E+00_JPRB, 0.70287E+00_JPRB, 0.71225E+00_JPRB, 0.74580E+00_JPRB ,&
&  0.74267E+00_JPRB, 0.73990E+00_JPRB, 0.76480E+00_JPRB, 0.80494E+00_JPRB, 0.82182E+00_JPRB, 0.83481E+00_JPRB, 0.85995E+00_JPRB ,&
&  0.87239E+00_JPRB, 0.86870E+00_JPRB, 0.89864E+00_JPRB, 0.96561E+00_JPRB, 0.10888E+01_JPRB, 0.13097E+01_JPRB, 0.17459E+01_JPRB ,&
&  0.30242E+01_JPRB, 0.74662E+01_JPRB, 0.43782E+01_JPRB, 0.42108E+01_JPRB, 0.31049E+01_JPRB, 0.25310E+01_JPRB, 0.23147E+01_JPRB ,&
&  0.25402E+01_JPRB, 0.23667E+01_JPRB, 0.21192E+01_JPRB, 0.15675E+01_JPRB, 0.12434E+01_JPRB, 0.11058E+01_JPRB, 0.12238E+01_JPRB ,&
&  0.13693E+01_JPRB, 0.14885E+01_JPRB, 0.15731E+01_JPRB, 0.15776E+01_JPRB, 0.15822E+01_JPRB, 0.16143E+01_JPRB, 0.16541E+01_JPRB ,&
&  0.17501E+01_JPRB, 0.19605E+01_JPRB, 0.23207E+01_JPRB, 0.27675E+01_JPRB, 0.33254E+01_JPRB, 0.39452E+01_JPRB, 0.45586E+01_JPRB ,&
&  0.43545E+01_JPRB, 0.38580E+01_JPRB, 0.31908E+01_JPRB, 0.22954E+01_JPRB, 0.16242E+01_JPRB, 0.15055E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.10991E+01_JPRB, 0.10873E+01_JPRB, 0.11058E+01_JPRB, 0.11373E+01_JPRB, 0.11938E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.12408E+01_JPRB, 0.12270E+01_JPRB, 0.12206E+01_JPRB, 0.12185E+01_JPRB, 0.12202E+01_JPRB, 0.12324E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.12735E+01_JPRB, 0.12326E+01_JPRB, 0.12235E+01_JPRB, 0.12283E+01_JPRB, 0.12127E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.11749E+01_JPRB, 0.12395E+01_JPRB, 0.12424E+01_JPRB, 0.12185E+01_JPRB, 0.12047E+01_JPRB, 0.12080E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.11670E+01_JPRB, 0.11468E+01_JPRB, 0.10603E+01_JPRB, 0.10108E+01_JPRB, 0.99660E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.99164E+00_JPRB, 0.95029E+00_JPRB, 0.89953E+00_JPRB, 0.83646E+00_JPRB, 0.80174E+00_JPRB, 0.80188E+00_JPRB, 0.80915E+00_JPRB ,&
&  0.77531E+00_JPRB, 0.74788E+00_JPRB, 0.73597E+00_JPRB, 0.74999E+00_JPRB, 0.77503E+00_JPRB, 0.78570E+00_JPRB, 0.79323E+00_JPRB ,&
&  0.78548E+00_JPRB, 0.78146E+00_JPRB, 0.76764E+00_JPRB, 0.77268E+00_JPRB, 0.82646E+00_JPRB, 0.84021E+00_JPRB, 0.83801E+00_JPRB ,&
&  0.85830E+00_JPRB, 0.84899E+00_JPRB, 0.83421E+00_JPRB, 0.84901E+00_JPRB, 0.90539E+00_JPRB, 0.10553E+01_JPRB, 0.13991E+01_JPRB ,&
&  0.27536E+01_JPRB, 0.68111E+01_JPRB, 0.29320E+01_JPRB, 0.49163E+01_JPRB, 0.34428E+01_JPRB, 0.28086E+01_JPRB, 0.22473E+01_JPRB ,&
&  0.22740E+01_JPRB, 0.26255E+01_JPRB, 0.20507E+01_JPRB, 0.16626E+01_JPRB, 0.13625E+01_JPRB, 0.12022E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.13236E+01_JPRB, 0.13390E+01_JPRB, 0.13553E+01_JPRB, 0.13352E+01_JPRB, 0.13394E+01_JPRB, 0.13713E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.15166E+01_JPRB, 0.17452E+01_JPRB, 0.20852E+01_JPRB, 0.28953E+01_JPRB, 0.38274E+01_JPRB, 0.48436E+01_JPRB ,&
&  0.48780E+01_JPRB, 0.48323E+01_JPRB, 0.36754E+01_JPRB, 0.25299E+01_JPRB, 0.18277E+01_JPRB, 0.15885E+01_JPRB, 0.13687E+01_JPRB ,&
&  0.12566E+01_JPRB, 0.12026E+01_JPRB, 0.11464E+01_JPRB, 0.11430E+01_JPRB, 0.12164E+01_JPRB, 0.12788E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.12765E+01_JPRB, 0.12552E+01_JPRB, 0.12396E+01_JPRB, 0.12229E+01_JPRB, 0.12321E+01_JPRB, 0.12331E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.12360E+01_JPRB, 0.12098E+01_JPRB, 0.12131E+01_JPRB, 0.12166E+01_JPRB, 0.12064E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.12202E+01_JPRB, 0.12290E+01_JPRB, 0.12174E+01_JPRB, 0.12107E+01_JPRB, 0.12610E+01_JPRB, 0.12197E+01_JPRB ,&
&  0.12123E+01_JPRB, 0.11810E+01_JPRB, 0.11125E+01_JPRB, 0.10755E+01_JPRB, 0.10536E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.10533E+01_JPRB, 0.99894E+00_JPRB, 0.95716E+00_JPRB, 0.90731E+00_JPRB, 0.86963E+00_JPRB, 0.84857E+00_JPRB, 0.84763E+00_JPRB ,&
&  0.83420E+00_JPRB, 0.80169E+00_JPRB, 0.78027E+00_JPRB, 0.78023E+00_JPRB, 0.78115E+00_JPRB, 0.79115E+00_JPRB, 0.79918E+00_JPRB ,&
&  0.79652E+00_JPRB, 0.78679E+00_JPRB, 0.78315E+00_JPRB, 0.78421E+00_JPRB, 0.81146E+00_JPRB, 0.85753E+00_JPRB, 0.87679E+00_JPRB ,&
&  0.86728E+00_JPRB, 0.85501E+00_JPRB, 0.83300E+00_JPRB, 0.81905E+00_JPRB, 0.83509E+00_JPRB, 0.93409E+00_JPRB, 0.12317E+01_JPRB ,&
&  0.26249E+01_JPRB, 0.80261E+01_JPRB, 0.31302E+01_JPRB, 0.54167E+01_JPRB, 0.37133E+01_JPRB, 0.29501E+01_JPRB, 0.23093E+01_JPRB ,&
&  0.22658E+01_JPRB, 0.24594E+01_JPRB, 0.20951E+01_JPRB, 0.17869E+01_JPRB, 0.15456E+01_JPRB, 0.13770E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.13364E+01_JPRB, 0.13217E+01_JPRB, 0.12881E+01_JPRB, 0.12721E+01_JPRB, 0.12399E+01_JPRB, 0.12270E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.12346E+01_JPRB, 0.12458E+01_JPRB, 0.13494E+01_JPRB, 0.17861E+01_JPRB, 0.23959E+01_JPRB, 0.36712E+01_JPRB, 0.55466E+01_JPRB ,&
&  0.54729E+01_JPRB, 0.56611E+01_JPRB, 0.41025E+01_JPRB, 0.28047E+01_JPRB, 0.19886E+01_JPRB, 0.16632E+01_JPRB, 0.15211E+01_JPRB ,&
&  0.13907E+01_JPRB, 0.12966E+01_JPRB, 0.12255E+01_JPRB, 0.12051E+01_JPRB, 0.12487E+01_JPRB, 0.12709E+01_JPRB, 0.12929E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.12404E+01_JPRB, 0.12132E+01_JPRB, 0.12032E+01_JPRB, 0.12179E+01_JPRB, 0.12244E+01_JPRB, 0.12172E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.12071E+01_JPRB, 0.11957E+01_JPRB, 0.12078E+01_JPRB, 0.12213E+01_JPRB, 0.12143E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.12111E+01_JPRB, 0.12061E+01_JPRB, 0.12314E+01_JPRB, 0.12396E+01_JPRB, 0.13190E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.11908E+01_JPRB, 0.11411E+01_JPRB, 0.11148E+01_JPRB, 0.11010E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.11019E+01_JPRB, 0.10599E+01_JPRB, 0.10259E+01_JPRB, 0.98417E+00_JPRB, 0.93149E+00_JPRB, 0.90323E+00_JPRB, 0.88351E+00_JPRB ,&
&  0.87073E+00_JPRB, 0.83910E+00_JPRB, 0.81506E+00_JPRB, 0.81059E+00_JPRB, 0.81417E+00_JPRB, 0.80881E+00_JPRB, 0.80946E+00_JPRB ,&
&  0.81794E+00_JPRB, 0.82241E+00_JPRB, 0.81348E+00_JPRB, 0.80264E+00_JPRB, 0.80048E+00_JPRB, 0.83734E+00_JPRB, 0.86100E+00_JPRB ,&
&  0.87159E+00_JPRB, 0.85750E+00_JPRB, 0.83126E+00_JPRB, 0.80242E+00_JPRB, 0.80001E+00_JPRB, 0.85028E+00_JPRB, 0.10811E+01_JPRB ,&
&  0.22396E+01_JPRB, 0.31131E+01_JPRB, 0.56224E+01_JPRB, 0.57720E+01_JPRB, 0.39542E+01_JPRB, 0.31087E+01_JPRB, 0.26698E+01_JPRB ,&
&  0.25587E+01_JPRB, 0.23993E+01_JPRB, 0.20980E+01_JPRB, 0.18695E+01_JPRB, 0.16786E+01_JPRB, 0.15189E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.13248E+01_JPRB, 0.12944E+01_JPRB, 0.12791E+01_JPRB, 0.12649E+01_JPRB, 0.12499E+01_JPRB, 0.12413E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.12408E+01_JPRB, 0.12316E+01_JPRB, 0.12432E+01_JPRB, 0.14141E+01_JPRB, 0.21336E+01_JPRB, 0.32982E+01_JPRB, 0.50204E+01_JPRB ,&
&  0.58884E+01_JPRB, 0.54936E+01_JPRB, 0.39140E+01_JPRB, 0.28707E+01_JPRB, 0.20912E+01_JPRB, 0.16805E+01_JPRB, 0.15511E+01_JPRB ,&
&  0.14844E+01_JPRB, 0.14200E+01_JPRB, 0.13257E+01_JPRB, 0.12810E+01_JPRB, 0.12699E+01_JPRB, 0.12600E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.12489E+01_JPRB, 0.12319E+01_JPRB, 0.12157E+01_JPRB, 0.12020E+01_JPRB, 0.11858E+01_JPRB, 0.11761E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.11607E+01_JPRB, 0.11662E+01_JPRB, 0.11885E+01_JPRB, 0.12439E+01_JPRB, 0.12549E+01_JPRB, 0.12390E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.11986E+01_JPRB, 0.11937E+01_JPRB, 0.12369E+01_JPRB, 0.12292E+01_JPRB, 0.13425E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.12119E+01_JPRB, 0.11723E+01_JPRB, 0.11347E+01_JPRB, 0.11137E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.11133E+01_JPRB, 0.10675E+01_JPRB, 0.10341E+01_JPRB, 0.10099E+01_JPRB, 0.97452E+00_JPRB, 0.93902E+00_JPRB, 0.92769E+00_JPRB ,&
&  0.92177E+00_JPRB, 0.89882E+00_JPRB, 0.87106E+00_JPRB, 0.83651E+00_JPRB, 0.82519E+00_JPRB, 0.82903E+00_JPRB, 0.83540E+00_JPRB ,&
&  0.83641E+00_JPRB, 0.83913E+00_JPRB, 0.83868E+00_JPRB, 0.83598E+00_JPRB, 0.83237E+00_JPRB, 0.85349E+00_JPRB, 0.86515E+00_JPRB ,&
&  0.86037E+00_JPRB, 0.86108E+00_JPRB, 0.83827E+00_JPRB, 0.81439E+00_JPRB, 0.80450E+00_JPRB, 0.80163E+00_JPRB, 0.85681E+00_JPRB ,&
&  0.16496E+01_JPRB, 0.38943E+01_JPRB, 0.56144E+01_JPRB, 0.50967E+01_JPRB, 0.38556E+01_JPRB, 0.29743E+01_JPRB, 0.27242E+01_JPRB ,&
&  0.25188E+01_JPRB, 0.23320E+01_JPRB, 0.20850E+01_JPRB, 0.18914E+01_JPRB, 0.17383E+01_JPRB, 0.16068E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.13845E+01_JPRB, 0.13432E+01_JPRB, 0.13103E+01_JPRB, 0.12956E+01_JPRB, 0.12822E+01_JPRB, 0.12862E+01_JPRB, 0.12897E+01_JPRB ,&
&  0.12929E+01_JPRB, 0.12880E+01_JPRB, 0.12800E+01_JPRB, 0.13290E+01_JPRB, 0.17374E+01_JPRB, 0.20271E+01_JPRB, 0.34654E+01_JPRB ,&
&  0.48264E+01_JPRB, 0.39408E+01_JPRB, 0.30461E+01_JPRB, 0.25280E+01_JPRB, 0.20884E+01_JPRB, 0.17072E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.14512E+01_JPRB, 0.14016E+01_JPRB, 0.13332E+01_JPRB, 0.12873E+01_JPRB, 0.12496E+01_JPRB, 0.12330E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.12017E+01_JPRB, 0.12013E+01_JPRB, 0.11984E+01_JPRB, 0.11960E+01_JPRB, 0.11790E+01_JPRB, 0.11632E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.11167E+01_JPRB, 0.11234E+01_JPRB, 0.11422E+01_JPRB, 0.12243E+01_JPRB, 0.12879E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.12178E+01_JPRB, 0.12182E+01_JPRB, 0.12453E+01_JPRB, 0.13049E+01_JPRB, 0.14306E+01_JPRB, 0.13253E+01_JPRB ,&
&  0.12700E+01_JPRB, 0.12445E+01_JPRB, 0.12102E+01_JPRB, 0.11693E+01_JPRB, 0.11468E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.11574E+01_JPRB, 0.11050E+01_JPRB, 0.10604E+01_JPRB, 0.10193E+01_JPRB, 0.10027E+01_JPRB, 0.97685E+00_JPRB, 0.94770E+00_JPRB ,&
&  0.93174E+00_JPRB, 0.92886E+00_JPRB, 0.91915E+00_JPRB, 0.89816E+00_JPRB, 0.87923E+00_JPRB, 0.87126E+00_JPRB, 0.87009E+00_JPRB ,&
&  0.86885E+00_JPRB, 0.86872E+00_JPRB, 0.86701E+00_JPRB, 0.85885E+00_JPRB, 0.86061E+00_JPRB, 0.86903E+00_JPRB, 0.87842E+00_JPRB ,&
&  0.86168E+00_JPRB, 0.84694E+00_JPRB, 0.83521E+00_JPRB, 0.82255E+00_JPRB, 0.81872E+00_JPRB, 0.80226E+00_JPRB, 0.79262E+00_JPRB ,&
&  0.11426E+01_JPRB, 0.24767E+01_JPRB, 0.35691E+01_JPRB, 0.44346E+01_JPRB, 0.37864E+01_JPRB, 0.30671E+01_JPRB, 0.27447E+01_JPRB ,&
&  0.24906E+01_JPRB, 0.22191E+01_JPRB, 0.19888E+01_JPRB, 0.18405E+01_JPRB, 0.17457E+01_JPRB, 0.16349E+01_JPRB, 0.14997E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.13353E+01_JPRB, 0.13169E+01_JPRB, 0.13184E+01_JPRB, 0.13151E+01_JPRB, 0.13230E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.13163E+01_JPRB, 0.13000E+01_JPRB, 0.12743E+01_JPRB, 0.12582E+01_JPRB, 0.14165E+01_JPRB, 0.15576E+01_JPRB, 0.20230E+01_JPRB ,&
&  0.22408E+01_JPRB, 0.23274E+01_JPRB, 0.22147E+01_JPRB, 0.21150E+01_JPRB, 0.19602E+01_JPRB, 0.17013E+01_JPRB, 0.15007E+01_JPRB ,&
&  0.13868E+01_JPRB, 0.13368E+01_JPRB, 0.12843E+01_JPRB, 0.12576E+01_JPRB, 0.12303E+01_JPRB, 0.12072E+01_JPRB, 0.11974E+01_JPRB ,&
&  0.11872E+01_JPRB, 0.11769E+01_JPRB, 0.11657E+01_JPRB, 0.11676E+01_JPRB, 0.11530E+01_JPRB, 0.11230E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.10679E+01_JPRB, 0.10777E+01_JPRB, 0.11168E+01_JPRB, 0.11591E+01_JPRB, 0.11890E+01_JPRB, 0.12212E+01_JPRB ,&
&  0.12507E+01_JPRB, 0.12716E+01_JPRB, 0.12470E+01_JPRB, 0.12910E+01_JPRB, 0.13615E+01_JPRB, 0.14673E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.12734E+01_JPRB, 0.12373E+01_JPRB, 0.11817E+01_JPRB, 0.11998E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.11598E+01_JPRB, 0.11297E+01_JPRB, 0.10941E+01_JPRB, 0.10473E+01_JPRB, 0.10252E+01_JPRB, 0.10051E+01_JPRB, 0.98414E+00_JPRB ,&
&  0.97308E+00_JPRB, 0.96253E+00_JPRB, 0.95433E+00_JPRB, 0.93743E+00_JPRB, 0.92005E+00_JPRB, 0.90891E+00_JPRB, 0.89639E+00_JPRB ,&
&  0.89635E+00_JPRB, 0.89657E+00_JPRB, 0.88886E+00_JPRB, 0.88036E+00_JPRB, 0.87583E+00_JPRB, 0.87194E+00_JPRB, 0.87552E+00_JPRB ,&
&  0.86793E+00_JPRB, 0.85724E+00_JPRB, 0.85646E+00_JPRB, 0.84791E+00_JPRB, 0.83348E+00_JPRB, 0.82164E+00_JPRB, 0.79207E+00_JPRB ,&
&  0.87969E+00_JPRB, 0.13457E+01_JPRB, 0.21571E+01_JPRB, 0.30714E+01_JPRB, 0.30058E+01_JPRB, 0.28326E+01_JPRB, 0.25372E+01_JPRB ,&
&  0.22636E+01_JPRB, 0.20542E+01_JPRB, 0.18223E+01_JPRB, 0.16620E+01_JPRB, 0.15994E+01_JPRB, 0.15381E+01_JPRB, 0.14733E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.13441E+01_JPRB, 0.13120E+01_JPRB, 0.13144E+01_JPRB, 0.13052E+01_JPRB, 0.12965E+01_JPRB, 0.12856E+01_JPRB ,&
&  0.12623E+01_JPRB, 0.12378E+01_JPRB, 0.12020E+01_JPRB, 0.11679E+01_JPRB, 0.11405E+01_JPRB, 0.11385E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.14108E+01_JPRB, 0.14844E+01_JPRB, 0.15240E+01_JPRB, 0.15757E+01_JPRB, 0.16045E+01_JPRB, 0.15664E+01_JPRB, 0.14606E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.13169E+01_JPRB, 0.12558E+01_JPRB, 0.12528E+01_JPRB, 0.12302E+01_JPRB, 0.12007E+01_JPRB, 0.11731E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.11342E+01_JPRB, 0.11129E+01_JPRB, 0.10990E+01_JPRB, 0.10851E+01_JPRB, 0.10639E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10177E+01_JPRB, 0.10244E+01_JPRB, 0.10546E+01_JPRB, 0.10953E+01_JPRB, 0.11247E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.11837E+01_JPRB, 0.12233E+01_JPRB, 0.12054E+01_JPRB, 0.12680E+01_JPRB, 0.14121E+01_JPRB, 0.14181E+01_JPRB, 0.13592E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.12636E+01_JPRB, 0.12334E+01_JPRB, 0.11680E+01_JPRB, 0.11470E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.10361E+01_JPRB, 0.10534E+01_JPRB, 0.10376E+01_JPRB, 0.10031E+01_JPRB, 0.98370E+00_JPRB, 0.97552E+00_JPRB, 0.97346E+00_JPRB ,&
&  0.96885E+00_JPRB, 0.95752E+00_JPRB, 0.96118E+00_JPRB, 0.96585E+00_JPRB, 0.96083E+00_JPRB, 0.94635E+00_JPRB, 0.93558E+00_JPRB ,&
&  0.93167E+00_JPRB, 0.93444E+00_JPRB, 0.93887E+00_JPRB, 0.92825E+00_JPRB, 0.90613E+00_JPRB, 0.89072E+00_JPRB, 0.87761E+00_JPRB ,&
&  0.87649E+00_JPRB, 0.87692E+00_JPRB, 0.87478E+00_JPRB, 0.87549E+00_JPRB, 0.85251E+00_JPRB, 0.82637E+00_JPRB, 0.80495E+00_JPRB ,&
&  0.80669E+00_JPRB, 0.85527E+00_JPRB, 0.12562E+01_JPRB, 0.18087E+01_JPRB, 0.20672E+01_JPRB, 0.20266E+01_JPRB, 0.18111E+01_JPRB ,&
&  0.16720E+01_JPRB, 0.15869E+01_JPRB, 0.15192E+01_JPRB, 0.14349E+01_JPRB, 0.13771E+01_JPRB, 0.13384E+01_JPRB, 0.13063E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.12440E+01_JPRB, 0.12169E+01_JPRB, 0.11846E+01_JPRB, 0.11580E+01_JPRB, 0.11457E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.11254E+01_JPRB, 0.11067E+01_JPRB, 0.10928E+01_JPRB, 0.10774E+01_JPRB, 0.10396E+01_JPRB, 0.10207E+01_JPRB, 0.10216E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.11034E+01_JPRB, 0.11274E+01_JPRB, 0.11976E+01_JPRB, 0.12579E+01_JPRB, 0.13097E+01_JPRB, 0.13086E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.12584E+01_JPRB, 0.12308E+01_JPRB, 0.12195E+01_JPRB, 0.12001E+01_JPRB, 0.11595E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.10757E+01_JPRB, 0.10502E+01_JPRB, 0.10309E+01_JPRB, 0.10100E+01_JPRB, 0.98250E+00_JPRB, 0.95827E+00_JPRB ,&
&  0.94118E+00_JPRB, 0.93631E+00_JPRB, 0.93992E+00_JPRB, 0.95910E+00_JPRB, 0.98919E+00_JPRB, 0.10263E+01_JPRB, 0.10642E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.11294E+01_JPRB, 0.11659E+01_JPRB, 0.12492E+01_JPRB, 0.12701E+01_JPRB, 0.12765E+01_JPRB, 0.12875E+01_JPRB ,&
&  0.12679E+01_JPRB, 0.12184E+01_JPRB, 0.11668E+01_JPRB, 0.10876E+01_JPRB, 0.98885E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.98449E+00_JPRB, 0.99322E+00_JPRB, 0.98629E+00_JPRB, 0.97164E+00_JPRB, 0.96039E+00_JPRB, 0.95316E+00_JPRB, 0.95543E+00_JPRB ,&
&  0.95337E+00_JPRB, 0.94733E+00_JPRB, 0.94940E+00_JPRB, 0.95031E+00_JPRB, 0.95063E+00_JPRB, 0.94588E+00_JPRB, 0.94877E+00_JPRB ,&
&  0.94806E+00_JPRB, 0.94863E+00_JPRB, 0.95116E+00_JPRB, 0.94424E+00_JPRB, 0.92370E+00_JPRB, 0.90884E+00_JPRB, 0.89417E+00_JPRB ,&
&  0.89288E+00_JPRB, 0.89242E+00_JPRB, 0.89280E+00_JPRB, 0.88584E+00_JPRB, 0.86399E+00_JPRB, 0.84435E+00_JPRB, 0.83150E+00_JPRB ,&
&  0.83221E+00_JPRB, 0.72875E+00_JPRB, 0.90182E+00_JPRB, 0.12982E+01_JPRB, 0.14976E+01_JPRB, 0.13942E+01_JPRB, 0.13388E+01_JPRB ,&
&  0.13212E+01_JPRB, 0.13105E+01_JPRB, 0.12906E+01_JPRB, 0.12491E+01_JPRB, 0.11779E+01_JPRB, 0.11440E+01_JPRB, 0.11060E+01_JPRB ,&
&  0.10808E+01_JPRB, 0.10699E+01_JPRB, 0.10569E+01_JPRB, 0.10487E+01_JPRB, 0.10284E+01_JPRB, 0.10081E+01_JPRB, 0.10013E+01_JPRB ,&
&  0.10028E+01_JPRB, 0.99696E+00_JPRB, 0.99518E+00_JPRB, 0.99169E+00_JPRB, 0.97680E+00_JPRB, 0.98247E+00_JPRB, 0.98343E+00_JPRB ,&
&  0.98980E+00_JPRB, 0.99453E+00_JPRB, 0.99798E+00_JPRB, 0.10112E+01_JPRB, 0.10435E+01_JPRB, 0.11078E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11494E+01_JPRB, 0.11385E+01_JPRB, 0.11421E+01_JPRB, 0.11428E+01_JPRB, 0.11307E+01_JPRB, 0.11011E+01_JPRB, 0.10670E+01_JPRB ,&
&  0.10427E+01_JPRB, 0.10199E+01_JPRB, 0.99790E+00_JPRB, 0.97351E+00_JPRB, 0.94782E+00_JPRB, 0.92381E+00_JPRB, 0.90432E+00_JPRB ,&
&  0.89153E+00_JPRB, 0.88632E+00_JPRB, 0.89389E+00_JPRB, 0.91402E+00_JPRB, 0.94134E+00_JPRB, 0.97339E+00_JPRB, 0.10078E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10890E+01_JPRB, 0.11276E+01_JPRB, 0.11244E+01_JPRB, 0.12024E+01_JPRB, 0.12206E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.11650E+01_JPRB, 0.10909E+01_JPRB, 0.87148E+00_JPRB, 0.95170E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.95843E+00_JPRB, 0.95964E+00_JPRB, 0.95915E+00_JPRB, 0.94470E+00_JPRB, 0.92706E+00_JPRB, 0.91958E+00_JPRB, 0.92507E+00_JPRB ,&
&  0.91958E+00_JPRB, 0.90552E+00_JPRB, 0.90962E+00_JPRB, 0.91206E+00_JPRB, 0.91778E+00_JPRB, 0.91960E+00_JPRB, 0.92175E+00_JPRB ,&
&  0.92417E+00_JPRB, 0.92695E+00_JPRB, 0.92461E+00_JPRB, 0.93261E+00_JPRB, 0.92498E+00_JPRB, 0.91919E+00_JPRB, 0.91601E+00_JPRB ,&
&  0.90732E+00_JPRB, 0.89845E+00_JPRB, 0.89075E+00_JPRB, 0.87979E+00_JPRB, 0.87060E+00_JPRB, 0.85931E+00_JPRB, 0.84222E+00_JPRB ,&
&  0.85295E+00_JPRB, 0.72016E+00_JPRB, 0.85964E+00_JPRB, 0.11108E+01_JPRB, 0.11095E+01_JPRB, 0.10690E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11138E+01_JPRB, 0.11070E+01_JPRB, 0.10758E+01_JPRB, 0.10083E+01_JPRB, 0.97748E+00_JPRB, 0.96208E+00_JPRB ,&
&  0.94493E+00_JPRB, 0.93154E+00_JPRB, 0.92232E+00_JPRB, 0.92288E+00_JPRB, 0.92325E+00_JPRB, 0.91986E+00_JPRB, 0.91399E+00_JPRB ,&
&  0.91437E+00_JPRB, 0.92016E+00_JPRB, 0.92002E+00_JPRB, 0.91962E+00_JPRB, 0.91913E+00_JPRB, 0.90840E+00_JPRB, 0.90011E+00_JPRB ,&
&  0.89280E+00_JPRB, 0.88977E+00_JPRB, 0.88680E+00_JPRB, 0.89182E+00_JPRB, 0.90613E+00_JPRB, 0.94860E+00_JPRB, 0.98831E+00_JPRB ,&
&  0.10036E+01_JPRB, 0.10084E+01_JPRB, 0.10133E+01_JPRB, 0.10182E+01_JPRB, 0.10237E+01_JPRB, 0.10211E+01_JPRB, 0.10053E+01_JPRB ,&
&  0.98584E+00_JPRB, 0.96163E+00_JPRB, 0.94408E+00_JPRB, 0.92972E+00_JPRB, 0.91283E+00_JPRB, 0.90023E+00_JPRB, 0.89499E+00_JPRB ,&
&  0.88484E+00_JPRB, 0.87120E+00_JPRB, 0.87321E+00_JPRB, 0.88853E+00_JPRB, 0.90403E+00_JPRB, 0.91665E+00_JPRB, 0.93996E+00_JPRB ,&
&  0.97941E+00_JPRB, 0.10213E+01_JPRB, 0.10540E+01_JPRB, 0.11002E+01_JPRB, 0.11252E+01_JPRB, 0.11381E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.10947E+01_JPRB, 0.81990E+00_JPRB, 0.92776E+00_JPRB, 0.95073E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.95226E+00_JPRB, 0.94104E+00_JPRB, 0.92741E+00_JPRB, 0.90894E+00_JPRB, 0.89083E+00_JPRB, 0.87961E+00_JPRB, 0.88402E+00_JPRB ,&
&  0.88167E+00_JPRB, 0.86879E+00_JPRB, 0.87853E+00_JPRB, 0.89107E+00_JPRB, 0.90082E+00_JPRB, 0.90948E+00_JPRB, 0.91158E+00_JPRB ,&
&  0.91405E+00_JPRB, 0.91024E+00_JPRB, 0.91310E+00_JPRB, 0.92530E+00_JPRB, 0.94420E+00_JPRB, 0.94601E+00_JPRB, 0.94295E+00_JPRB ,&
&  0.92052E+00_JPRB, 0.90908E+00_JPRB, 0.89845E+00_JPRB, 0.88539E+00_JPRB, 0.87306E+00_JPRB, 0.86749E+00_JPRB, 0.86814E+00_JPRB ,&
&  0.75413E+00_JPRB, 0.67208E+00_JPRB, 0.82854E+00_JPRB, 0.94675E+00_JPRB, 0.93019E+00_JPRB, 0.90990E+00_JPRB, 0.92780E+00_JPRB ,&
&  0.93469E+00_JPRB, 0.92892E+00_JPRB, 0.93128E+00_JPRB, 0.93157E+00_JPRB, 0.90347E+00_JPRB, 0.86726E+00_JPRB, 0.85129E+00_JPRB ,&
&  0.84261E+00_JPRB, 0.84849E+00_JPRB, 0.85160E+00_JPRB, 0.85992E+00_JPRB, 0.86037E+00_JPRB, 0.85266E+00_JPRB, 0.84918E+00_JPRB ,&
&  0.84694E+00_JPRB, 0.85364E+00_JPRB, 0.85895E+00_JPRB, 0.86113E+00_JPRB, 0.85948E+00_JPRB, 0.85656E+00_JPRB, 0.84935E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.83742E+00_JPRB, 0.83213E+00_JPRB, 0.83449E+00_JPRB, 0.84357E+00_JPRB, 0.86425E+00_JPRB, 0.89585E+00_JPRB ,&
&  0.91619E+00_JPRB, 0.92472E+00_JPRB, 0.93009E+00_JPRB, 0.93120E+00_JPRB, 0.94191E+00_JPRB, 0.94746E+00_JPRB, 0.93680E+00_JPRB ,&
&  0.92151E+00_JPRB, 0.90871E+00_JPRB, 0.90570E+00_JPRB, 0.90135E+00_JPRB, 0.89897E+00_JPRB, 0.89437E+00_JPRB, 0.88498E+00_JPRB ,&
&  0.87394E+00_JPRB, 0.85895E+00_JPRB, 0.84796E+00_JPRB, 0.84826E+00_JPRB, 0.86033E+00_JPRB, 0.87093E+00_JPRB, 0.89259E+00_JPRB ,&
&  0.92923E+00_JPRB, 0.97284E+00_JPRB, 0.10084E+01_JPRB, 0.10512E+01_JPRB, 0.10779E+01_JPRB, 0.10909E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10265E+01_JPRB, 0.10097E+01_JPRB, 0.98181E+00_JPRB, 0.96557E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.90284E+00_JPRB, 0.89516E+00_JPRB, 0.87995E+00_JPRB, 0.85273E+00_JPRB, 0.81797E+00_JPRB, 0.82527E+00_JPRB, 0.83568E+00_JPRB ,&
&  0.82833E+00_JPRB, 0.81830E+00_JPRB, 0.82877E+00_JPRB, 0.85388E+00_JPRB, 0.87804E+00_JPRB, 0.89145E+00_JPRB, 0.89282E+00_JPRB ,&
&  0.88683E+00_JPRB, 0.88800E+00_JPRB, 0.89597E+00_JPRB, 0.91331E+00_JPRB, 0.91280E+00_JPRB, 0.89512E+00_JPRB, 0.88423E+00_JPRB ,&
&  0.88134E+00_JPRB, 0.87942E+00_JPRB, 0.87669E+00_JPRB, 0.85960E+00_JPRB, 0.84819E+00_JPRB, 0.84472E+00_JPRB, 0.84448E+00_JPRB ,&
&  0.84426E+00_JPRB, 0.71182E+00_JPRB, 0.84808E+00_JPRB, 0.85143E+00_JPRB, 0.84979E+00_JPRB, 0.82796E+00_JPRB, 0.82377E+00_JPRB ,&
&  0.82638E+00_JPRB, 0.84154E+00_JPRB, 0.83946E+00_JPRB, 0.82374E+00_JPRB, 0.80563E+00_JPRB, 0.78122E+00_JPRB, 0.76478E+00_JPRB ,&
&  0.75651E+00_JPRB, 0.76091E+00_JPRB, 0.77214E+00_JPRB, 0.78458E+00_JPRB, 0.79718E+00_JPRB, 0.79922E+00_JPRB, 0.80235E+00_JPRB ,&
&  0.80314E+00_JPRB, 0.80565E+00_JPRB, 0.80503E+00_JPRB, 0.80672E+00_JPRB, 0.80533E+00_JPRB, 0.79986E+00_JPRB, 0.79033E+00_JPRB ,&
&  0.77701E+00_JPRB, 0.76846E+00_JPRB, 0.76646E+00_JPRB, 0.76888E+00_JPRB, 0.77902E+00_JPRB, 0.79889E+00_JPRB, 0.82211E+00_JPRB ,&
&  0.84649E+00_JPRB, 0.86287E+00_JPRB, 0.87258E+00_JPRB, 0.87628E+00_JPRB, 0.88547E+00_JPRB, 0.89443E+00_JPRB, 0.89134E+00_JPRB ,&
&  0.87954E+00_JPRB, 0.86611E+00_JPRB, 0.85780E+00_JPRB, 0.84858E+00_JPRB, 0.84375E+00_JPRB, 0.83970E+00_JPRB, 0.83217E+00_JPRB ,&
&  0.82250E+00_JPRB, 0.82037E+00_JPRB, 0.82176E+00_JPRB, 0.81168E+00_JPRB, 0.81489E+00_JPRB, 0.82690E+00_JPRB, 0.84833E+00_JPRB ,&
&  0.87532E+00_JPRB, 0.91033E+00_JPRB, 0.94417E+00_JPRB, 0.97293E+00_JPRB, 0.99113E+00_JPRB, 0.99827E+00_JPRB, 0.99412E+00_JPRB ,&
&  0.97714E+00_JPRB, 0.95266E+00_JPRB, 0.93298E+00_JPRB, 0.92105E+00_JPRB, 0.91931E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.85285E+00_JPRB, 0.85281E+00_JPRB, 0.84197E+00_JPRB, 0.80285E+00_JPRB, 0.77286E+00_JPRB, 0.78124E+00_JPRB, 0.78894E+00_JPRB ,&
&  0.78834E+00_JPRB, 0.80366E+00_JPRB, 0.82882E+00_JPRB, 0.85410E+00_JPRB, 0.87093E+00_JPRB, 0.87039E+00_JPRB, 0.87374E+00_JPRB ,&
&  0.87041E+00_JPRB, 0.86778E+00_JPRB, 0.87748E+00_JPRB, 0.88246E+00_JPRB, 0.87185E+00_JPRB, 0.85110E+00_JPRB, 0.84917E+00_JPRB ,&
&  0.85552E+00_JPRB, 0.85111E+00_JPRB, 0.83276E+00_JPRB, 0.82024E+00_JPRB, 0.81006E+00_JPRB, 0.81570E+00_JPRB, 0.82416E+00_JPRB ,&
&  0.82136E+00_JPRB, 0.79238E+00_JPRB, 0.83365E+00_JPRB, 0.83930E+00_JPRB, 0.83590E+00_JPRB, 0.81161E+00_JPRB, 0.81303E+00_JPRB ,&
&  0.82225E+00_JPRB, 0.82788E+00_JPRB, 0.81108E+00_JPRB, 0.79151E+00_JPRB, 0.77539E+00_JPRB, 0.76071E+00_JPRB, 0.73519E+00_JPRB ,&
&  0.72659E+00_JPRB, 0.72462E+00_JPRB, 0.73901E+00_JPRB, 0.75439E+00_JPRB, 0.77220E+00_JPRB, 0.77992E+00_JPRB, 0.79044E+00_JPRB ,&
&  0.79162E+00_JPRB, 0.78443E+00_JPRB, 0.77693E+00_JPRB, 0.76981E+00_JPRB, 0.76647E+00_JPRB, 0.75915E+00_JPRB, 0.74851E+00_JPRB ,&
&  0.73897E+00_JPRB, 0.73627E+00_JPRB, 0.74026E+00_JPRB, 0.74932E+00_JPRB, 0.76229E+00_JPRB, 0.78473E+00_JPRB, 0.80425E+00_JPRB ,&
&  0.82759E+00_JPRB, 0.83701E+00_JPRB, 0.84490E+00_JPRB, 0.85927E+00_JPRB, 0.86798E+00_JPRB, 0.86716E+00_JPRB, 0.86380E+00_JPRB ,&
&  0.85779E+00_JPRB, 0.84811E+00_JPRB, 0.84886E+00_JPRB, 0.84404E+00_JPRB, 0.84092E+00_JPRB, 0.82932E+00_JPRB, 0.81172E+00_JPRB ,&
&  0.80454E+00_JPRB, 0.79954E+00_JPRB, 0.79111E+00_JPRB, 0.78832E+00_JPRB, 0.78606E+00_JPRB, 0.80493E+00_JPRB, 0.83811E+00_JPRB ,&
&  0.85733E+00_JPRB, 0.88298E+00_JPRB, 0.91833E+00_JPRB, 0.94118E+00_JPRB, 0.94185E+00_JPRB, 0.94173E+00_JPRB, 0.93065E+00_JPRB ,&
&  0.91225E+00_JPRB, 0.89891E+00_JPRB, 0.88550E+00_JPRB, 0.87963E+00_JPRB, 0.87441E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.80299E+00_JPRB, 0.82458E+00_JPRB, 0.79176E+00_JPRB, 0.77332E+00_JPRB, 0.75183E+00_JPRB, 0.74501E+00_JPRB, 0.73526E+00_JPRB ,&
&  0.74659E+00_JPRB, 0.76445E+00_JPRB, 0.78875E+00_JPRB, 0.79687E+00_JPRB, 0.80364E+00_JPRB, 0.80620E+00_JPRB, 0.82244E+00_JPRB ,&
&  0.81703E+00_JPRB, 0.80835E+00_JPRB, 0.81121E+00_JPRB, 0.80069E+00_JPRB, 0.79192E+00_JPRB, 0.79727E+00_JPRB, 0.80701E+00_JPRB ,&
&  0.81758E+00_JPRB, 0.80879E+00_JPRB, 0.79112E+00_JPRB, 0.76439E+00_JPRB, 0.75166E+00_JPRB, 0.74886E+00_JPRB, 0.75708E+00_JPRB ,&
&  0.76462E+00_JPRB, 0.80231E+00_JPRB, 0.73427E+00_JPRB, 0.79308E+00_JPRB, 0.79498E+00_JPRB, 0.78555E+00_JPRB, 0.78120E+00_JPRB ,&
&  0.78442E+00_JPRB, 0.77385E+00_JPRB, 0.76699E+00_JPRB, 0.76096E+00_JPRB, 0.73883E+00_JPRB, 0.72134E+00_JPRB, 0.70351E+00_JPRB ,&
&  0.68801E+00_JPRB, 0.67843E+00_JPRB, 0.68459E+00_JPRB, 0.70340E+00_JPRB, 0.72235E+00_JPRB, 0.73285E+00_JPRB, 0.73911E+00_JPRB ,&
&  0.74234E+00_JPRB, 0.73473E+00_JPRB, 0.72365E+00_JPRB, 0.72215E+00_JPRB, 0.72665E+00_JPRB, 0.72992E+00_JPRB, 0.73113E+00_JPRB ,&
&  0.72947E+00_JPRB, 0.70614E+00_JPRB, 0.69234E+00_JPRB, 0.70852E+00_JPRB, 0.72754E+00_JPRB, 0.74970E+00_JPRB, 0.77852E+00_JPRB ,&
&  0.80121E+00_JPRB, 0.81065E+00_JPRB, 0.81134E+00_JPRB, 0.81858E+00_JPRB, 0.82544E+00_JPRB, 0.83455E+00_JPRB, 0.84114E+00_JPRB ,&
&  0.82843E+00_JPRB, 0.82403E+00_JPRB, 0.82353E+00_JPRB, 0.82193E+00_JPRB, 0.81082E+00_JPRB, 0.78573E+00_JPRB, 0.75184E+00_JPRB ,&
&  0.73586E+00_JPRB, 0.72024E+00_JPRB, 0.70904E+00_JPRB, 0.70493E+00_JPRB, 0.70571E+00_JPRB, 0.73264E+00_JPRB, 0.77182E+00_JPRB ,&
&  0.81112E+00_JPRB, 0.84315E+00_JPRB, 0.87075E+00_JPRB, 0.89578E+00_JPRB, 0.89515E+00_JPRB, 0.87418E+00_JPRB, 0.86522E+00_JPRB ,&
&  0.85049E+00_JPRB, 0.83393E+00_JPRB, 0.82183E+00_JPRB, 0.81214E+00_JPRB, 0.80202E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.80127E+00_JPRB, 0.80268E+00_JPRB, 0.78765E+00_JPRB, 0.77677E+00_JPRB, 0.75877E+00_JPRB, 0.73551E+00_JPRB, 0.71563E+00_JPRB ,&
&  0.71785E+00_JPRB, 0.73429E+00_JPRB, 0.74993E+00_JPRB, 0.76104E+00_JPRB, 0.77658E+00_JPRB, 0.77962E+00_JPRB, 0.78556E+00_JPRB ,&
&  0.78048E+00_JPRB, 0.74294E+00_JPRB, 0.73684E+00_JPRB, 0.74423E+00_JPRB, 0.74599E+00_JPRB, 0.75241E+00_JPRB, 0.76387E+00_JPRB ,&
&  0.77430E+00_JPRB, 0.76978E+00_JPRB, 0.76139E+00_JPRB, 0.73140E+00_JPRB, 0.72446E+00_JPRB, 0.72935E+00_JPRB, 0.73466E+00_JPRB ,&
&  0.74569E+00_JPRB, 0.76693E+00_JPRB, 0.76863E+00_JPRB, 0.77934E+00_JPRB, 0.78511E+00_JPRB, 0.78609E+00_JPRB, 0.78106E+00_JPRB ,&
&  0.75421E+00_JPRB, 0.72789E+00_JPRB, 0.71767E+00_JPRB, 0.70159E+00_JPRB, 0.67817E+00_JPRB, 0.66199E+00_JPRB, 0.65522E+00_JPRB ,&
&  0.65263E+00_JPRB, 0.65882E+00_JPRB, 0.66551E+00_JPRB, 0.67976E+00_JPRB, 0.68513E+00_JPRB, 0.68953E+00_JPRB, 0.69545E+00_JPRB ,&
&  0.69582E+00_JPRB, 0.69252E+00_JPRB, 0.69274E+00_JPRB, 0.70221E+00_JPRB, 0.70691E+00_JPRB, 0.70309E+00_JPRB, 0.69721E+00_JPRB ,&
&  0.68251E+00_JPRB, 0.66793E+00_JPRB, 0.67460E+00_JPRB, 0.69030E+00_JPRB, 0.69875E+00_JPRB, 0.72640E+00_JPRB, 0.76157E+00_JPRB ,&
&  0.79144E+00_JPRB, 0.81069E+00_JPRB, 0.81087E+00_JPRB, 0.80657E+00_JPRB, 0.80454E+00_JPRB, 0.81181E+00_JPRB, 0.82355E+00_JPRB ,&
&  0.80699E+00_JPRB, 0.79726E+00_JPRB, 0.79448E+00_JPRB, 0.77405E+00_JPRB, 0.73633E+00_JPRB, 0.70351E+00_JPRB, 0.67943E+00_JPRB ,&
&  0.66034E+00_JPRB, 0.64977E+00_JPRB, 0.65360E+00_JPRB, 0.65625E+00_JPRB, 0.66809E+00_JPRB, 0.70385E+00_JPRB, 0.74188E+00_JPRB ,&
&  0.78364E+00_JPRB, 0.80562E+00_JPRB, 0.81336E+00_JPRB, 0.82258E+00_JPRB, 0.82959E+00_JPRB, 0.81942E+00_JPRB, 0.81063E+00_JPRB ,&
&  0.80807E+00_JPRB, 0.79933E+00_JPRB, 0.78980E+00_JPRB, 0.78017E+00_JPRB, 0.78788E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.79340E+00_JPRB, 0.78573E+00_JPRB, 0.77154E+00_JPRB, 0.74369E+00_JPRB, 0.72512E+00_JPRB, 0.69793E+00_JPRB, 0.68290E+00_JPRB ,&
&  0.68574E+00_JPRB, 0.68788E+00_JPRB, 0.69358E+00_JPRB, 0.70291E+00_JPRB, 0.70078E+00_JPRB, 0.69599E+00_JPRB, 0.69091E+00_JPRB ,&
&  0.69328E+00_JPRB, 0.69347E+00_JPRB, 0.70192E+00_JPRB, 0.71721E+00_JPRB, 0.73328E+00_JPRB, 0.74273E+00_JPRB, 0.75316E+00_JPRB ,&
&  0.76116E+00_JPRB, 0.75518E+00_JPRB, 0.75045E+00_JPRB, 0.71914E+00_JPRB, 0.70088E+00_JPRB, 0.70583E+00_JPRB, 0.71360E+00_JPRB ,&
&  0.72420E+00_JPRB, 0.73623E+00_JPRB, 0.74477E+00_JPRB, 0.75294E+00_JPRB, 0.73914E+00_JPRB, 0.73435E+00_JPRB, 0.72275E+00_JPRB ,&
&  0.67857E+00_JPRB, 0.63564E+00_JPRB, 0.61255E+00_JPRB, 0.60399E+00_JPRB, 0.59677E+00_JPRB, 0.60218E+00_JPRB, 0.61068E+00_JPRB ,&
&  0.62542E+00_JPRB, 0.63867E+00_JPRB, 0.64190E+00_JPRB, 0.64469E+00_JPRB, 0.64624E+00_JPRB, 0.64595E+00_JPRB, 0.64759E+00_JPRB ,&
&  0.64629E+00_JPRB, 0.64622E+00_JPRB, 0.65380E+00_JPRB, 0.65386E+00_JPRB, 0.64392E+00_JPRB, 0.63978E+00_JPRB, 0.63608E+00_JPRB ,&
&  0.63243E+00_JPRB, 0.62921E+00_JPRB, 0.62563E+00_JPRB, 0.63623E+00_JPRB, 0.66301E+00_JPRB, 0.69346E+00_JPRB, 0.71924E+00_JPRB ,&
&  0.73441E+00_JPRB, 0.74703E+00_JPRB, 0.75048E+00_JPRB, 0.75232E+00_JPRB, 0.75189E+00_JPRB, 0.74298E+00_JPRB, 0.73933E+00_JPRB ,&
&  0.73134E+00_JPRB, 0.71160E+00_JPRB, 0.68521E+00_JPRB, 0.64468E+00_JPRB, 0.60542E+00_JPRB, 0.58286E+00_JPRB, 0.58229E+00_JPRB ,&
&  0.59440E+00_JPRB, 0.60695E+00_JPRB, 0.61312E+00_JPRB, 0.62074E+00_JPRB, 0.63480E+00_JPRB, 0.65178E+00_JPRB, 0.67013E+00_JPRB ,&
&  0.68179E+00_JPRB, 0.68225E+00_JPRB, 0.68063E+00_JPRB, 0.68976E+00_JPRB, 0.70187E+00_JPRB, 0.72096E+00_JPRB, 0.74162E+00_JPRB ,&
&  0.75953E+00_JPRB, 0.75969E+00_JPRB, 0.76049E+00_JPRB, 0.76225E+00_JPRB, 0.78257E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.78522E+00_JPRB, 0.78304E+00_JPRB, 0.75327E+00_JPRB, 0.72061E+00_JPRB, 0.70184E+00_JPRB, 0.68164E+00_JPRB, 0.66562E+00_JPRB ,&
&  0.66434E+00_JPRB, 0.66166E+00_JPRB, 0.65931E+00_JPRB, 0.65624E+00_JPRB, 0.65426E+00_JPRB, 0.65128E+00_JPRB, 0.63933E+00_JPRB ,&
&  0.63471E+00_JPRB, 0.64097E+00_JPRB, 0.66313E+00_JPRB, 0.68787E+00_JPRB, 0.70767E+00_JPRB, 0.73381E+00_JPRB, 0.74537E+00_JPRB ,&
&  0.75780E+00_JPRB, 0.77063E+00_JPRB, 0.77247E+00_JPRB, 0.73847E+00_JPRB, 0.71053E+00_JPRB, 0.71143E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.72385E+00_JPRB, 0.72775E+00_JPRB, 0.73023E+00_JPRB, 0.72840E+00_JPRB, 0.70841E+00_JPRB, 0.67042E+00_JPRB, 0.63964E+00_JPRB ,&
&  0.61344E+00_JPRB, 0.59296E+00_JPRB, 0.58562E+00_JPRB, 0.57222E+00_JPRB, 0.56873E+00_JPRB, 0.57856E+00_JPRB, 0.59323E+00_JPRB ,&
&  0.60876E+00_JPRB, 0.61628E+00_JPRB, 0.62347E+00_JPRB, 0.61942E+00_JPRB, 0.61528E+00_JPRB, 0.61878E+00_JPRB, 0.61942E+00_JPRB ,&
&  0.61992E+00_JPRB, 0.61133E+00_JPRB, 0.60808E+00_JPRB, 0.60036E+00_JPRB, 0.59574E+00_JPRB, 0.59483E+00_JPRB, 0.59649E+00_JPRB ,&
&  0.59690E+00_JPRB, 0.60205E+00_JPRB, 0.60364E+00_JPRB, 0.60623E+00_JPRB, 0.61610E+00_JPRB, 0.63316E+00_JPRB, 0.65008E+00_JPRB ,&
&  0.66265E+00_JPRB, 0.67245E+00_JPRB, 0.67991E+00_JPRB, 0.68585E+00_JPRB, 0.68589E+00_JPRB, 0.68104E+00_JPRB, 0.66687E+00_JPRB ,&
&  0.63619E+00_JPRB, 0.60342E+00_JPRB, 0.57849E+00_JPRB, 0.55868E+00_JPRB, 0.55552E+00_JPRB, 0.56720E+00_JPRB, 0.58563E+00_JPRB ,&
&  0.59944E+00_JPRB, 0.60294E+00_JPRB, 0.59612E+00_JPRB, 0.59337E+00_JPRB, 0.59701E+00_JPRB, 0.60559E+00_JPRB, 0.61229E+00_JPRB ,&
&  0.61102E+00_JPRB, 0.60797E+00_JPRB, 0.61259E+00_JPRB, 0.62113E+00_JPRB, 0.63690E+00_JPRB, 0.65635E+00_JPRB, 0.67702E+00_JPRB ,&
&  0.69734E+00_JPRB, 0.71293E+00_JPRB, 0.73265E+00_JPRB, 0.75245E+00_JPRB, 0.77757E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.76518E+00_JPRB, 0.73632E+00_JPRB, 0.70607E+00_JPRB, 0.68060E+00_JPRB, 0.66883E+00_JPRB, 0.65170E+00_JPRB, 0.63731E+00_JPRB ,&
&  0.63660E+00_JPRB, 0.64439E+00_JPRB, 0.63952E+00_JPRB, 0.63327E+00_JPRB, 0.62407E+00_JPRB, 0.61674E+00_JPRB, 0.60807E+00_JPRB ,&
&  0.60412E+00_JPRB, 0.61515E+00_JPRB, 0.64045E+00_JPRB, 0.65275E+00_JPRB, 0.65857E+00_JPRB, 0.68881E+00_JPRB, 0.72034E+00_JPRB ,&
&  0.73004E+00_JPRB, 0.74433E+00_JPRB, 0.74808E+00_JPRB, 0.72174E+00_JPRB, 0.70446E+00_JPRB, 0.69777E+00_JPRB, 0.70165E+00_JPRB ,&
&  0.69747E+00_JPRB, 0.70212E+00_JPRB, 0.69348E+00_JPRB, 0.62544E+00_JPRB, 0.62471E+00_JPRB, 0.59453E+00_JPRB, 0.58374E+00_JPRB ,&
&  0.57776E+00_JPRB, 0.58210E+00_JPRB, 0.57689E+00_JPRB, 0.56596E+00_JPRB, 0.56140E+00_JPRB, 0.56470E+00_JPRB, 0.56707E+00_JPRB ,&
&  0.56394E+00_JPRB, 0.55404E+00_JPRB, 0.55877E+00_JPRB, 0.56516E+00_JPRB, 0.56881E+00_JPRB, 0.57626E+00_JPRB, 0.58729E+00_JPRB ,&
&  0.57754E+00_JPRB, 0.56795E+00_JPRB, 0.55915E+00_JPRB, 0.55955E+00_JPRB, 0.56604E+00_JPRB, 0.57942E+00_JPRB, 0.58301E+00_JPRB ,&
&  0.58021E+00_JPRB, 0.57880E+00_JPRB, 0.58011E+00_JPRB, 0.58694E+00_JPRB, 0.60161E+00_JPRB, 0.61592E+00_JPRB, 0.61701E+00_JPRB ,&
&  0.61827E+00_JPRB, 0.62308E+00_JPRB, 0.62877E+00_JPRB, 0.62996E+00_JPRB, 0.62538E+00_JPRB, 0.61000E+00_JPRB, 0.59242E+00_JPRB ,&
&  0.57585E+00_JPRB, 0.55782E+00_JPRB, 0.54324E+00_JPRB, 0.53939E+00_JPRB, 0.55385E+00_JPRB, 0.58431E+00_JPRB, 0.60162E+00_JPRB ,&
&  0.60615E+00_JPRB, 0.60835E+00_JPRB, 0.61120E+00_JPRB, 0.61137E+00_JPRB, 0.61251E+00_JPRB, 0.60966E+00_JPRB, 0.60198E+00_JPRB ,&
&  0.58990E+00_JPRB, 0.57901E+00_JPRB, 0.57772E+00_JPRB, 0.58425E+00_JPRB, 0.59159E+00_JPRB, 0.60603E+00_JPRB, 0.62446E+00_JPRB ,&
&  0.64777E+00_JPRB, 0.68658E+00_JPRB, 0.71893E+00_JPRB, 0.74365E+00_JPRB, 0.76067E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.74170E+00_JPRB, 0.71409E+00_JPRB, 0.69546E+00_JPRB, 0.68119E+00_JPRB, 0.66271E+00_JPRB, 0.64866E+00_JPRB, 0.64304E+00_JPRB ,&
&  0.65125E+00_JPRB, 0.64983E+00_JPRB, 0.64021E+00_JPRB, 0.63233E+00_JPRB, 0.61685E+00_JPRB, 0.60300E+00_JPRB, 0.59747E+00_JPRB ,&
&  0.60592E+00_JPRB, 0.62188E+00_JPRB, 0.64153E+00_JPRB, 0.64899E+00_JPRB, 0.64779E+00_JPRB, 0.65613E+00_JPRB, 0.67545E+00_JPRB ,&
&  0.70417E+00_JPRB, 0.72422E+00_JPRB, 0.73071E+00_JPRB, 0.71659E+00_JPRB, 0.69829E+00_JPRB, 0.68985E+00_JPRB, 0.67582E+00_JPRB ,&
&  0.65936E+00_JPRB, 0.66209E+00_JPRB, 0.59549E+00_JPRB, 0.58783E+00_JPRB, 0.56518E+00_JPRB, 0.55052E+00_JPRB, 0.54653E+00_JPRB ,&
&  0.55950E+00_JPRB, 0.56276E+00_JPRB, 0.55338E+00_JPRB, 0.54016E+00_JPRB, 0.53154E+00_JPRB, 0.52049E+00_JPRB, 0.51065E+00_JPRB ,&
&  0.50441E+00_JPRB, 0.50745E+00_JPRB, 0.51652E+00_JPRB, 0.52600E+00_JPRB, 0.53695E+00_JPRB, 0.54766E+00_JPRB, 0.54452E+00_JPRB ,&
&  0.53293E+00_JPRB, 0.53109E+00_JPRB, 0.53874E+00_JPRB, 0.55418E+00_JPRB, 0.57081E+00_JPRB, 0.57817E+00_JPRB, 0.58062E+00_JPRB ,&
&  0.57105E+00_JPRB, 0.55324E+00_JPRB, 0.53829E+00_JPRB, 0.52613E+00_JPRB, 0.49822E+00_JPRB, 0.43608E+00_JPRB, 0.41394E+00_JPRB ,&
&  0.42581E+00_JPRB, 0.48340E+00_JPRB, 0.55095E+00_JPRB, 0.54368E+00_JPRB, 0.54150E+00_JPRB, 0.52711E+00_JPRB, 0.48934E+00_JPRB ,&
&  0.44376E+00_JPRB, 0.38324E+00_JPRB, 0.36390E+00_JPRB, 0.36119E+00_JPRB, 0.36131E+00_JPRB, 0.36900E+00_JPRB, 0.38216E+00_JPRB ,&
&  0.39930E+00_JPRB, 0.42445E+00_JPRB, 0.44090E+00_JPRB, 0.44677E+00_JPRB, 0.44442E+00_JPRB, 0.42819E+00_JPRB, 0.41381E+00_JPRB ,&
&  0.40494E+00_JPRB, 0.43110E+00_JPRB, 0.49169E+00_JPRB, 0.57385E+00_JPRB, 0.57588E+00_JPRB, 0.60353E+00_JPRB, 0.63009E+00_JPRB ,&
&  0.65522E+00_JPRB, 0.69620E+00_JPRB, 0.72218E+00_JPRB, 0.72726E+00_JPRB, 0.74478E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.70129E+00_JPRB, 0.68776E+00_JPRB, 0.67832E+00_JPRB, 0.67170E+00_JPRB, 0.66425E+00_JPRB, 0.66252E+00_JPRB, 0.66158E+00_JPRB ,&
&  0.65576E+00_JPRB, 0.64759E+00_JPRB, 0.64908E+00_JPRB, 0.63263E+00_JPRB, 0.60869E+00_JPRB, 0.59270E+00_JPRB, 0.59067E+00_JPRB ,&
&  0.59458E+00_JPRB, 0.60180E+00_JPRB, 0.61075E+00_JPRB, 0.61875E+00_JPRB, 0.62337E+00_JPRB, 0.63495E+00_JPRB, 0.65561E+00_JPRB ,&
&  0.67958E+00_JPRB, 0.69298E+00_JPRB, 0.69677E+00_JPRB, 0.68819E+00_JPRB, 0.67555E+00_JPRB, 0.66273E+00_JPRB, 0.65141E+00_JPRB ,&
&  0.64032E+00_JPRB, 0.53849E+00_JPRB, 0.47233E+00_JPRB, 0.48015E+00_JPRB, 0.54580E+00_JPRB, 0.52947E+00_JPRB, 0.52888E+00_JPRB ,&
&  0.54151E+00_JPRB, 0.54055E+00_JPRB, 0.52662E+00_JPRB, 0.50910E+00_JPRB, 0.49288E+00_JPRB, 0.48126E+00_JPRB, 0.47339E+00_JPRB ,&
&  0.47020E+00_JPRB, 0.47749E+00_JPRB, 0.48527E+00_JPRB, 0.49723E+00_JPRB, 0.49980E+00_JPRB, 0.48655E+00_JPRB, 0.47049E+00_JPRB ,&
&  0.45523E+00_JPRB, 0.44963E+00_JPRB, 0.45234E+00_JPRB, 0.45983E+00_JPRB, 0.46962E+00_JPRB, 0.47718E+00_JPRB, 0.47434E+00_JPRB ,&
&  0.44644E+00_JPRB, 0.42294E+00_JPRB, 0.39465E+00_JPRB, 0.36792E+00_JPRB, 0.34493E+00_JPRB, 0.32965E+00_JPRB, 0.32144E+00_JPRB ,&
&  0.31969E+00_JPRB, 0.32825E+00_JPRB, 0.35028E+00_JPRB, 0.39913E+00_JPRB, 0.46111E+00_JPRB, 0.38177E+00_JPRB, 0.34355E+00_JPRB ,&
&  0.32296E+00_JPRB, 0.30974E+00_JPRB, 0.30469E+00_JPRB, 0.30593E+00_JPRB, 0.31086E+00_JPRB, 0.31770E+00_JPRB, 0.32309E+00_JPRB ,&
&  0.32750E+00_JPRB, 0.33087E+00_JPRB, 0.33195E+00_JPRB, 0.33228E+00_JPRB, 0.33347E+00_JPRB, 0.33457E+00_JPRB, 0.33336E+00_JPRB ,&
&  0.33231E+00_JPRB, 0.33981E+00_JPRB, 0.35903E+00_JPRB, 0.38883E+00_JPRB, 0.42275E+00_JPRB, 0.46007E+00_JPRB, 0.51329E+00_JPRB ,&
&  0.57890E+00_JPRB, 0.65121E+00_JPRB, 0.67107E+00_JPRB, 0.68559E+00_JPRB, 0.69605E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.64107E+00_JPRB, 0.64392E+00_JPRB, 0.64706E+00_JPRB, 0.64857E+00_JPRB, 0.64272E+00_JPRB, 0.63506E+00_JPRB, 0.63046E+00_JPRB ,&
&  0.63587E+00_JPRB, 0.63649E+00_JPRB, 0.61378E+00_JPRB, 0.59637E+00_JPRB, 0.58658E+00_JPRB, 0.58314E+00_JPRB, 0.58054E+00_JPRB ,&
&  0.57442E+00_JPRB, 0.57225E+00_JPRB, 0.57316E+00_JPRB, 0.58392E+00_JPRB, 0.59283E+00_JPRB, 0.60054E+00_JPRB, 0.60766E+00_JPRB ,&
&  0.60219E+00_JPRB, 0.61372E+00_JPRB, 0.63484E+00_JPRB, 0.63338E+00_JPRB, 0.63012E+00_JPRB, 0.62244E+00_JPRB, 0.60747E+00_JPRB ,&
&  0.55788E+00_JPRB, 0.49336E+00_JPRB, 0.43433E+00_JPRB, 0.44743E+00_JPRB, 0.52166E+00_JPRB, 0.52855E+00_JPRB, 0.51522E+00_JPRB ,&
&  0.52614E+00_JPRB, 0.52090E+00_JPRB, 0.50267E+00_JPRB, 0.48886E+00_JPRB, 0.47309E+00_JPRB, 0.46288E+00_JPRB, 0.45542E+00_JPRB ,&
&  0.45233E+00_JPRB, 0.45196E+00_JPRB, 0.43578E+00_JPRB, 0.38562E+00_JPRB, 0.35804E+00_JPRB, 0.34065E+00_JPRB, 0.32248E+00_JPRB ,&
&  0.30786E+00_JPRB, 0.29947E+00_JPRB, 0.29871E+00_JPRB, 0.29915E+00_JPRB, 0.30479E+00_JPRB, 0.30805E+00_JPRB, 0.31095E+00_JPRB ,&
&  0.31143E+00_JPRB, 0.31044E+00_JPRB, 0.30652E+00_JPRB, 0.30209E+00_JPRB, 0.29895E+00_JPRB, 0.29873E+00_JPRB, 0.29832E+00_JPRB ,&
&  0.30111E+00_JPRB, 0.30482E+00_JPRB, 0.32178E+00_JPRB, 0.33945E+00_JPRB, 0.33562E+00_JPRB, 0.32056E+00_JPRB, 0.30682E+00_JPRB ,&
&  0.29621E+00_JPRB, 0.29085E+00_JPRB, 0.28740E+00_JPRB, 0.28687E+00_JPRB, 0.29026E+00_JPRB, 0.29517E+00_JPRB, 0.29632E+00_JPRB ,&
&  0.29593E+00_JPRB, 0.29648E+00_JPRB, 0.29641E+00_JPRB, 0.29673E+00_JPRB, 0.29725E+00_JPRB, 0.30006E+00_JPRB, 0.30540E+00_JPRB ,&
&  0.31171E+00_JPRB, 0.32309E+00_JPRB, 0.33573E+00_JPRB, 0.35251E+00_JPRB, 0.36485E+00_JPRB, 0.37439E+00_JPRB, 0.38802E+00_JPRB ,&
&  0.40675E+00_JPRB, 0.44703E+00_JPRB, 0.51618E+00_JPRB, 0.63946E+00_JPRB, 0.63250E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.60781E+00_JPRB, 0.61852E+00_JPRB, 0.63013E+00_JPRB, 0.62139E+00_JPRB, 0.60795E+00_JPRB, 0.60036E+00_JPRB, 0.60041E+00_JPRB ,&
&  0.60065E+00_JPRB, 0.59818E+00_JPRB, 0.58659E+00_JPRB, 0.57294E+00_JPRB, 0.55474E+00_JPRB, 0.50735E+00_JPRB, 0.49050E+00_JPRB ,&
&  0.47326E+00_JPRB, 0.47605E+00_JPRB, 0.47998E+00_JPRB, 0.48587E+00_JPRB, 0.49901E+00_JPRB, 0.50932E+00_JPRB, 0.51049E+00_JPRB ,&
&  0.49970E+00_JPRB, 0.48098E+00_JPRB, 0.46273E+00_JPRB, 0.44881E+00_JPRB, 0.44826E+00_JPRB, 0.44775E+00_JPRB, 0.44020E+00_JPRB ,&
&  0.42664E+00_JPRB, 0.42092E+00_JPRB, 0.42082E+00_JPRB, 0.44063E+00_JPRB, 0.49121E+00_JPRB, 0.50958E+00_JPRB, 0.48589E+00_JPRB ,&
&  0.49033E+00_JPRB, 0.48476E+00_JPRB, 0.48041E+00_JPRB, 0.48115E+00_JPRB, 0.47031E+00_JPRB, 0.44825E+00_JPRB, 0.41422E+00_JPRB ,&
&  0.37246E+00_JPRB, 0.35087E+00_JPRB, 0.32988E+00_JPRB, 0.30918E+00_JPRB, 0.29230E+00_JPRB, 0.28269E+00_JPRB, 0.27465E+00_JPRB ,&
&  0.26950E+00_JPRB, 0.26503E+00_JPRB, 0.26222E+00_JPRB, 0.26253E+00_JPRB, 0.26470E+00_JPRB, 0.26585E+00_JPRB, 0.26717E+00_JPRB ,&
&  0.26907E+00_JPRB, 0.27031E+00_JPRB, 0.27108E+00_JPRB, 0.27281E+00_JPRB, 0.27460E+00_JPRB, 0.27772E+00_JPRB, 0.28172E+00_JPRB ,&
&  0.28670E+00_JPRB, 0.29041E+00_JPRB, 0.29386E+00_JPRB, 0.30063E+00_JPRB, 0.30127E+00_JPRB, 0.29982E+00_JPRB, 0.29155E+00_JPRB ,&
&  0.28541E+00_JPRB, 0.28276E+00_JPRB, 0.27997E+00_JPRB, 0.27924E+00_JPRB, 0.28054E+00_JPRB, 0.28275E+00_JPRB, 0.28500E+00_JPRB ,&
&  0.28657E+00_JPRB, 0.28655E+00_JPRB, 0.28552E+00_JPRB, 0.28526E+00_JPRB, 0.28575E+00_JPRB, 0.28584E+00_JPRB, 0.28979E+00_JPRB ,&
&  0.29869E+00_JPRB, 0.31163E+00_JPRB, 0.32584E+00_JPRB, 0.34485E+00_JPRB, 0.36303E+00_JPRB, 0.38003E+00_JPRB, 0.40832E+00_JPRB ,&
&  0.44680E+00_JPRB, 0.55204E+00_JPRB, 0.61967E+00_JPRB, 0.56255E+00_JPRB, 0.58773E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.58116E+00_JPRB, 0.58846E+00_JPRB, 0.59879E+00_JPRB, 0.60303E+00_JPRB, 0.61075E+00_JPRB, 0.59897E+00_JPRB, 0.59158E+00_JPRB ,&
&  0.56137E+00_JPRB, 0.54044E+00_JPRB, 0.50996E+00_JPRB, 0.47265E+00_JPRB, 0.44750E+00_JPRB, 0.42838E+00_JPRB, 0.42114E+00_JPRB ,&
&  0.41092E+00_JPRB, 0.40484E+00_JPRB, 0.39797E+00_JPRB, 0.39498E+00_JPRB, 0.39230E+00_JPRB, 0.39261E+00_JPRB, 0.39158E+00_JPRB ,&
&  0.38742E+00_JPRB, 0.38162E+00_JPRB, 0.37467E+00_JPRB, 0.37689E+00_JPRB, 0.38954E+00_JPRB, 0.40403E+00_JPRB, 0.42141E+00_JPRB ,&
&  0.44173E+00_JPRB, 0.45200E+00_JPRB, 0.46237E+00_JPRB, 0.46644E+00_JPRB, 0.45102E+00_JPRB, 0.45380E+00_JPRB, 0.45767E+00_JPRB ,&
&  0.46550E+00_JPRB, 0.47082E+00_JPRB, 0.46710E+00_JPRB, 0.46125E+00_JPRB, 0.40938E+00_JPRB, 0.37722E+00_JPRB, 0.34854E+00_JPRB ,&
&  0.32731E+00_JPRB, 0.31394E+00_JPRB, 0.30179E+00_JPRB, 0.29133E+00_JPRB, 0.27981E+00_JPRB, 0.27263E+00_JPRB, 0.26729E+00_JPRB ,&
&  0.26313E+00_JPRB, 0.26155E+00_JPRB, 0.26014E+00_JPRB, 0.25990E+00_JPRB, 0.26069E+00_JPRB, 0.26016E+00_JPRB, 0.26032E+00_JPRB ,&
&  0.26041E+00_JPRB, 0.26002E+00_JPRB, 0.26051E+00_JPRB, 0.26224E+00_JPRB, 0.26459E+00_JPRB, 0.26824E+00_JPRB, 0.26915E+00_JPRB ,&
&  0.27028E+00_JPRB, 0.27123E+00_JPRB, 0.27137E+00_JPRB, 0.27366E+00_JPRB, 0.27494E+00_JPRB, 0.27509E+00_JPRB, 0.27349E+00_JPRB ,&
&  0.27175E+00_JPRB, 0.27037E+00_JPRB, 0.27052E+00_JPRB, 0.27124E+00_JPRB, 0.27308E+00_JPRB, 0.27574E+00_JPRB, 0.27620E+00_JPRB ,&
&  0.27708E+00_JPRB, 0.27643E+00_JPRB, 0.27574E+00_JPRB, 0.27816E+00_JPRB, 0.28092E+00_JPRB, 0.28386E+00_JPRB, 0.28766E+00_JPRB ,&
&  0.28927E+00_JPRB, 0.29704E+00_JPRB, 0.31147E+00_JPRB, 0.33391E+00_JPRB, 0.37099E+00_JPRB, 0.38817E+00_JPRB, 0.42831E+00_JPRB ,&
&  0.48434E+00_JPRB, 0.62046E+00_JPRB, 0.68437E+00_JPRB, 0.54832E+00_JPRB, 0.56546E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.52270E+00_JPRB, 0.53495E+00_JPRB, 0.55125E+00_JPRB, 0.56140E+00_JPRB, 0.57468E+00_JPRB, 0.57686E+00_JPRB, 0.56986E+00_JPRB ,&
&  0.55209E+00_JPRB, 0.52442E+00_JPRB, 0.50822E+00_JPRB, 0.48692E+00_JPRB, 0.46577E+00_JPRB, 0.44396E+00_JPRB, 0.43417E+00_JPRB ,&
&  0.42239E+00_JPRB, 0.40847E+00_JPRB, 0.39440E+00_JPRB, 0.38201E+00_JPRB, 0.36999E+00_JPRB, 0.35905E+00_JPRB, 0.35287E+00_JPRB ,&
&  0.34654E+00_JPRB, 0.34625E+00_JPRB, 0.34563E+00_JPRB, 0.34953E+00_JPRB, 0.36099E+00_JPRB, 0.37941E+00_JPRB, 0.40526E+00_JPRB ,&
&  0.44364E+00_JPRB, 0.44737E+00_JPRB, 0.43296E+00_JPRB, 0.42686E+00_JPRB, 0.41933E+00_JPRB, 0.41319E+00_JPRB, 0.40245E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.41918E+00_JPRB, 0.43411E+00_JPRB, 0.42035E+00_JPRB, 0.37394E+00_JPRB, 0.35464E+00_JPRB, 0.33096E+00_JPRB ,&
&  0.32016E+00_JPRB, 0.31045E+00_JPRB, 0.30255E+00_JPRB, 0.29442E+00_JPRB, 0.28450E+00_JPRB, 0.27881E+00_JPRB, 0.27252E+00_JPRB ,&
&  0.26851E+00_JPRB, 0.26732E+00_JPRB, 0.26502E+00_JPRB, 0.26555E+00_JPRB, 0.26574E+00_JPRB, 0.26514E+00_JPRB, 0.26473E+00_JPRB ,&
&  0.26376E+00_JPRB, 0.26272E+00_JPRB, 0.26242E+00_JPRB, 0.26322E+00_JPRB, 0.26409E+00_JPRB, 0.26489E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26114E+00_JPRB, 0.25993E+00_JPRB, 0.25993E+00_JPRB, 0.26181E+00_JPRB, 0.26251E+00_JPRB, 0.26379E+00_JPRB, 0.26418E+00_JPRB ,&
&  0.26449E+00_JPRB, 0.26526E+00_JPRB, 0.26486E+00_JPRB, 0.26570E+00_JPRB, 0.26594E+00_JPRB, 0.26714E+00_JPRB, 0.26785E+00_JPRB ,&
&  0.26860E+00_JPRB, 0.27094E+00_JPRB, 0.27335E+00_JPRB, 0.27703E+00_JPRB, 0.28193E+00_JPRB, 0.28472E+00_JPRB, 0.28942E+00_JPRB ,&
&  0.29474E+00_JPRB, 0.30437E+00_JPRB, 0.32365E+00_JPRB, 0.34513E+00_JPRB, 0.37227E+00_JPRB, 0.38318E+00_JPRB, 0.38933E+00_JPRB ,&
&  0.39931E+00_JPRB, 0.42896E+00_JPRB, 0.47942E+00_JPRB, 0.50602E+00_JPRB, 0.51409E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.51340E+00_JPRB, 0.51604E+00_JPRB, 0.51798E+00_JPRB, 0.51549E+00_JPRB, 0.51550E+00_JPRB, 0.51579E+00_JPRB, 0.51319E+00_JPRB ,&
&  0.50511E+00_JPRB, 0.50319E+00_JPRB, 0.49805E+00_JPRB, 0.49012E+00_JPRB, 0.48211E+00_JPRB, 0.46873E+00_JPRB, 0.45208E+00_JPRB ,&
&  0.42994E+00_JPRB, 0.41010E+00_JPRB, 0.39553E+00_JPRB, 0.38150E+00_JPRB, 0.36782E+00_JPRB, 0.35449E+00_JPRB, 0.34993E+00_JPRB ,&
&  0.34559E+00_JPRB, 0.34391E+00_JPRB, 0.34361E+00_JPRB, 0.34673E+00_JPRB, 0.35409E+00_JPRB, 0.36336E+00_JPRB, 0.37365E+00_JPRB ,&
&  0.38479E+00_JPRB, 0.39532E+00_JPRB, 0.39629E+00_JPRB, 0.38837E+00_JPRB, 0.38135E+00_JPRB, 0.37721E+00_JPRB, 0.37386E+00_JPRB ,&
&  0.36995E+00_JPRB, 0.36191E+00_JPRB, 0.35224E+00_JPRB, 0.34188E+00_JPRB, 0.33148E+00_JPRB, 0.32392E+00_JPRB, 0.31508E+00_JPRB ,&
&  0.30858E+00_JPRB, 0.30253E+00_JPRB, 0.29579E+00_JPRB, 0.28997E+00_JPRB, 0.28599E+00_JPRB, 0.28277E+00_JPRB, 0.27937E+00_JPRB ,&
&  0.27793E+00_JPRB, 0.27648E+00_JPRB, 0.27488E+00_JPRB, 0.27344E+00_JPRB, 0.27150E+00_JPRB, 0.27014E+00_JPRB, 0.26892E+00_JPRB ,&
&  0.26804E+00_JPRB, 0.26723E+00_JPRB, 0.26677E+00_JPRB, 0.26624E+00_JPRB, 0.26579E+00_JPRB, 0.26466E+00_JPRB, 0.26337E+00_JPRB ,&
&  0.26270E+00_JPRB, 0.26278E+00_JPRB, 0.26204E+00_JPRB, 0.26174E+00_JPRB, 0.26146E+00_JPRB, 0.26061E+00_JPRB, 0.26125E+00_JPRB ,&
&  0.26271E+00_JPRB, 0.26432E+00_JPRB, 0.26442E+00_JPRB, 0.26549E+00_JPRB, 0.26675E+00_JPRB, 0.26838E+00_JPRB, 0.27119E+00_JPRB ,&
&  0.27328E+00_JPRB, 0.27548E+00_JPRB, 0.27831E+00_JPRB, 0.28060E+00_JPRB, 0.28434E+00_JPRB, 0.28898E+00_JPRB, 0.29474E+00_JPRB ,&
&  0.30266E+00_JPRB, 0.31226E+00_JPRB, 0.32610E+00_JPRB, 0.34313E+00_JPRB, 0.36838E+00_JPRB, 0.37615E+00_JPRB, 0.38299E+00_JPRB ,&
&  0.38760E+00_JPRB, 0.39590E+00_JPRB, 0.47840E+00_JPRB, 0.49254E+00_JPRB, 0.50740E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.39119E+00_JPRB, 0.40079E+00_JPRB, 0.40841E+00_JPRB, 0.41536E+00_JPRB, 0.41105E+00_JPRB, 0.47545E+00_JPRB, 0.45424E+00_JPRB ,&
&  0.44402E+00_JPRB, 0.41778E+00_JPRB, 0.42022E+00_JPRB, 0.41684E+00_JPRB, 0.41102E+00_JPRB, 0.40501E+00_JPRB, 0.39979E+00_JPRB ,&
&  0.39439E+00_JPRB, 0.38701E+00_JPRB, 0.38287E+00_JPRB, 0.37736E+00_JPRB, 0.37196E+00_JPRB, 0.36640E+00_JPRB, 0.36301E+00_JPRB ,&
&  0.36049E+00_JPRB, 0.35901E+00_JPRB, 0.35788E+00_JPRB, 0.35550E+00_JPRB, 0.35715E+00_JPRB, 0.35771E+00_JPRB, 0.35525E+00_JPRB ,&
&  0.35067E+00_JPRB, 0.34658E+00_JPRB, 0.34176E+00_JPRB, 0.33686E+00_JPRB, 0.33463E+00_JPRB, 0.33246E+00_JPRB, 0.33011E+00_JPRB ,&
&  0.32671E+00_JPRB, 0.32286E+00_JPRB, 0.31871E+00_JPRB, 0.31525E+00_JPRB, 0.31184E+00_JPRB, 0.30821E+00_JPRB, 0.30473E+00_JPRB ,&
&  0.30097E+00_JPRB, 0.29720E+00_JPRB, 0.29341E+00_JPRB, 0.29105E+00_JPRB, 0.28907E+00_JPRB, 0.28726E+00_JPRB, 0.28560E+00_JPRB ,&
&  0.28501E+00_JPRB, 0.28427E+00_JPRB, 0.28333E+00_JPRB, 0.28212E+00_JPRB, 0.28107E+00_JPRB, 0.27994E+00_JPRB, 0.27907E+00_JPRB ,&
&  0.27906E+00_JPRB, 0.27848E+00_JPRB, 0.27788E+00_JPRB, 0.27708E+00_JPRB, 0.27675E+00_JPRB, 0.27585E+00_JPRB, 0.27473E+00_JPRB ,&
&  0.27373E+00_JPRB, 0.27282E+00_JPRB, 0.27270E+00_JPRB, 0.27254E+00_JPRB, 0.27235E+00_JPRB, 0.27218E+00_JPRB, 0.27267E+00_JPRB ,&
&  0.27329E+00_JPRB, 0.27408E+00_JPRB, 0.27532E+00_JPRB, 0.27672E+00_JPRB, 0.27796E+00_JPRB, 0.27919E+00_JPRB, 0.28095E+00_JPRB ,&
&  0.28311E+00_JPRB, 0.28528E+00_JPRB, 0.28773E+00_JPRB, 0.28974E+00_JPRB, 0.29349E+00_JPRB, 0.29755E+00_JPRB, 0.30208E+00_JPRB ,&
&  0.30745E+00_JPRB, 0.31495E+00_JPRB, 0.32385E+00_JPRB, 0.33197E+00_JPRB, 0.34129E+00_JPRB, 0.35009E+00_JPRB, 0.35730E+00_JPRB ,&
&  0.36350E+00_JPRB, 0.36996E+00_JPRB, 0.37456E+00_JPRB, 0.37989E+00_JPRB, 0.38558E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.34963E+00_JPRB, 0.35284E+00_JPRB, 0.35532E+00_JPRB, 0.35690E+00_JPRB, 0.35645E+00_JPRB, 0.35763E+00_JPRB, 0.35748E+00_JPRB ,&
&  0.35707E+00_JPRB, 0.35646E+00_JPRB, 0.35713E+00_JPRB, 0.35789E+00_JPRB, 0.35834E+00_JPRB, 0.35864E+00_JPRB, 0.35869E+00_JPRB ,&
&  0.35806E+00_JPRB, 0.35696E+00_JPRB, 0.35202E+00_JPRB, 0.34924E+00_JPRB, 0.34748E+00_JPRB, 0.34647E+00_JPRB, 0.34585E+00_JPRB ,&
&  0.34469E+00_JPRB, 0.34341E+00_JPRB, 0.34151E+00_JPRB, 0.34135E+00_JPRB, 0.34056E+00_JPRB, 0.33949E+00_JPRB, 0.33792E+00_JPRB ,&
&  0.33634E+00_JPRB, 0.33510E+00_JPRB, 0.33361E+00_JPRB, 0.33208E+00_JPRB, 0.33032E+00_JPRB, 0.32814E+00_JPRB, 0.32525E+00_JPRB ,&
&  0.32230E+00_JPRB, 0.31935E+00_JPRB, 0.31691E+00_JPRB, 0.31458E+00_JPRB, 0.31257E+00_JPRB, 0.31091E+00_JPRB, 0.30964E+00_JPRB ,&
&  0.30823E+00_JPRB, 0.30660E+00_JPRB, 0.30493E+00_JPRB, 0.30353E+00_JPRB, 0.30215E+00_JPRB, 0.30095E+00_JPRB, 0.30029E+00_JPRB ,&
&  0.29992E+00_JPRB, 0.29947E+00_JPRB, 0.29892E+00_JPRB, 0.29839E+00_JPRB, 0.29775E+00_JPRB, 0.29716E+00_JPRB, 0.29657E+00_JPRB ,&
&  0.29605E+00_JPRB, 0.29581E+00_JPRB, 0.29541E+00_JPRB, 0.29500E+00_JPRB, 0.29475E+00_JPRB, 0.29432E+00_JPRB, 0.29384E+00_JPRB ,&
&  0.29331E+00_JPRB, 0.29309E+00_JPRB, 0.29281E+00_JPRB, 0.29245E+00_JPRB, 0.29209E+00_JPRB, 0.29185E+00_JPRB, 0.29190E+00_JPRB ,&
&  0.29189E+00_JPRB, 0.29181E+00_JPRB, 0.29190E+00_JPRB, 0.29276E+00_JPRB, 0.29368E+00_JPRB, 0.29478E+00_JPRB, 0.29601E+00_JPRB ,&
&  0.29758E+00_JPRB, 0.29923E+00_JPRB, 0.30111E+00_JPRB, 0.30290E+00_JPRB, 0.30533E+00_JPRB, 0.30774E+00_JPRB, 0.31016E+00_JPRB ,&
&  0.31248E+00_JPRB, 0.31489E+00_JPRB, 0.31736E+00_JPRB, 0.31993E+00_JPRB, 0.32344E+00_JPRB, 0.32617E+00_JPRB, 0.32926E+00_JPRB ,&
&  0.33222E+00_JPRB, 0.33563E+00_JPRB, 0.33898E+00_JPRB, 0.34197E+00_JPRB, 0.34487E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB ,&
&  0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB, 0.31754E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB, 0.62151E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.63178E+00_JPRB, 0.63163E+00_JPRB, 0.63155E+00_JPRB, 0.63166E+00_JPRB, 0.63137E+00_JPRB, 0.63078E+00_JPRB, 0.62984E+00_JPRB ,&
&  0.62877E+00_JPRB, 0.62757E+00_JPRB, 0.62589E+00_JPRB, 0.62417E+00_JPRB, 0.62248E+00_JPRB, 0.62079E+00_JPRB, 0.61866E+00_JPRB ,&
&  0.61636E+00_JPRB, 0.61376E+00_JPRB, 0.61105E+00_JPRB, 0.60832E+00_JPRB, 0.60628E+00_JPRB, 0.60459E+00_JPRB, 0.60347E+00_JPRB ,&
&  0.60235E+00_JPRB, 0.60137E+00_JPRB, 0.60042E+00_JPRB, 0.59975E+00_JPRB, 0.59950E+00_JPRB, 0.59907E+00_JPRB, 0.59874E+00_JPRB ,&
&  0.59849E+00_JPRB, 0.59849E+00_JPRB, 0.59827E+00_JPRB, 0.59804E+00_JPRB, 0.59780E+00_JPRB, 0.59811E+00_JPRB, 0.59826E+00_JPRB ,&
&  0.59861E+00_JPRB, 0.59883E+00_JPRB, 0.59898E+00_JPRB, 0.59899E+00_JPRB, 0.59918E+00_JPRB, 0.59941E+00_JPRB, 0.59958E+00_JPRB ,&
&  0.59987E+00_JPRB, 0.60050E+00_JPRB, 0.60185E+00_JPRB, 0.60361E+00_JPRB, 0.60533E+00_JPRB, 0.60723E+00_JPRB, 0.60921E+00_JPRB ,&
&  0.61126E+00_JPRB, 0.61309E+00_JPRB, 0.61522E+00_JPRB, 0.61734E+00_JPRB, 0.61978E+00_JPRB, 0.62208E+00_JPRB, 0.62451E+00_JPRB ,&
&  0.62679E+00_JPRB, 0.62923E+00_JPRB, 0.63201E+00_JPRB, 0.63488E+00_JPRB, 0.63759E+00_JPRB, 0.63997E+00_JPRB, 0.64253E+00_JPRB ,&
&  0.64523E+00_JPRB, 0.64784E+00_JPRB, 0.64982E+00_JPRB, 0.65197E+00_JPRB, 0.65418E+00_JPRB, 0.65636E+00_JPRB, 0.65793E+00_JPRB ,&
&  0.65956E+00_JPRB, 0.66137E+00_JPRB, 0.66322E+00_JPRB, 0.66380E+00_JPRB, 0.66420E+00_JPRB, 0.66438E+00_JPRB, 0.66410E+00_JPRB ,&
&  0.66299E+00_JPRB, 0.66172E+00_JPRB, 0.66041E+00_JPRB, 0.65887E+00_JPRB, 0.65683E+00_JPRB, 0.65454E+00_JPRB, 0.65194E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.64597E+00_JPRB, 0.64286E+00_JPRB, 0.64049E+00_JPRB, 0.63891E+00_JPRB, 0.63715E+00_JPRB, 0.63534E+00_JPRB ,&
&  0.63363E+00_JPRB, 0.63302E+00_JPRB, 0.63275E+00_JPRB, 0.63243E+00_JPRB, 0.63218E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.63366E+00_JPRB, 0.63407E+00_JPRB, 0.63471E+00_JPRB, 0.63484E+00_JPRB, 0.63346E+00_JPRB, 0.63211E+00_JPRB, 0.63060E+00_JPRB ,&
&  0.62945E+00_JPRB, 0.62843E+00_JPRB, 0.62655E+00_JPRB, 0.62408E+00_JPRB, 0.62160E+00_JPRB, 0.61928E+00_JPRB, 0.61688E+00_JPRB ,&
&  0.61462E+00_JPRB, 0.61222E+00_JPRB, 0.60965E+00_JPRB, 0.60507E+00_JPRB, 0.60062E+00_JPRB, 0.59684E+00_JPRB, 0.59491E+00_JPRB ,&
&  0.59289E+00_JPRB, 0.59100E+00_JPRB, 0.58849E+00_JPRB, 0.58565E+00_JPRB, 0.58290E+00_JPRB, 0.58042E+00_JPRB, 0.57786E+00_JPRB ,&
&  0.57580E+00_JPRB, 0.57411E+00_JPRB, 0.57278E+00_JPRB, 0.57118E+00_JPRB, 0.57192E+00_JPRB, 0.57280E+00_JPRB, 0.57357E+00_JPRB ,&
&  0.57425E+00_JPRB, 0.57447E+00_JPRB, 0.57606E+00_JPRB, 0.57764E+00_JPRB, 0.57911E+00_JPRB, 0.58152E+00_JPRB, 0.58300E+00_JPRB ,&
&  0.58343E+00_JPRB, 0.58414E+00_JPRB, 0.58738E+00_JPRB, 0.59007E+00_JPRB, 0.59247E+00_JPRB, 0.59467E+00_JPRB, 0.59796E+00_JPRB ,&
&  0.60126E+00_JPRB, 0.60461E+00_JPRB, 0.60828E+00_JPRB, 0.61287E+00_JPRB, 0.61741E+00_JPRB, 0.62259E+00_JPRB, 0.62784E+00_JPRB ,&
&  0.63278E+00_JPRB, 0.63786E+00_JPRB, 0.64305E+00_JPRB, 0.64818E+00_JPRB, 0.65288E+00_JPRB, 0.65694E+00_JPRB, 0.66156E+00_JPRB ,&
&  0.66652E+00_JPRB, 0.67233E+00_JPRB, 0.67607E+00_JPRB, 0.68009E+00_JPRB, 0.68446E+00_JPRB, 0.69049E+00_JPRB, 0.69296E+00_JPRB ,&
&  0.69517E+00_JPRB, 0.69718E+00_JPRB, 0.69938E+00_JPRB, 0.70041E+00_JPRB, 0.70143E+00_JPRB, 0.70238E+00_JPRB, 0.70288E+00_JPRB ,&
&  0.70257E+00_JPRB, 0.70126E+00_JPRB, 0.69891E+00_JPRB, 0.69533E+00_JPRB, 0.69116E+00_JPRB, 0.68681E+00_JPRB, 0.68156E+00_JPRB ,&
&  0.67674E+00_JPRB, 0.67072E+00_JPRB, 0.66480E+00_JPRB, 0.66026E+00_JPRB, 0.65707E+00_JPRB, 0.65220E+00_JPRB, 0.64710E+00_JPRB ,&
&  0.64245E+00_JPRB, 0.63887E+00_JPRB, 0.63694E+00_JPRB, 0.63530E+00_JPRB, 0.63442E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.63720E+00_JPRB, 0.63746E+00_JPRB, 0.63742E+00_JPRB, 0.63668E+00_JPRB, 0.63430E+00_JPRB, 0.63214E+00_JPRB, 0.62982E+00_JPRB ,&
&  0.62787E+00_JPRB, 0.62556E+00_JPRB, 0.62363E+00_JPRB, 0.62189E+00_JPRB, 0.62027E+00_JPRB, 0.61917E+00_JPRB, 0.61802E+00_JPRB ,&
&  0.61722E+00_JPRB, 0.61600E+00_JPRB, 0.61498E+00_JPRB, 0.61275E+00_JPRB, 0.60874E+00_JPRB, 0.60249E+00_JPRB, 0.59566E+00_JPRB ,&
&  0.58914E+00_JPRB, 0.58334E+00_JPRB, 0.57722E+00_JPRB, 0.57137E+00_JPRB, 0.56651E+00_JPRB, 0.53491E+00_JPRB, 0.52592E+00_JPRB ,&
&  0.52733E+00_JPRB, 0.52894E+00_JPRB, 0.53337E+00_JPRB, 0.53908E+00_JPRB, 0.54253E+00_JPRB, 0.53478E+00_JPRB, 0.53100E+00_JPRB ,&
&  0.50864E+00_JPRB, 0.51109E+00_JPRB, 0.51357E+00_JPRB, 0.51672E+00_JPRB, 0.52336E+00_JPRB, 0.52934E+00_JPRB, 0.53956E+00_JPRB ,&
&  0.56207E+00_JPRB, 0.56638E+00_JPRB, 0.57156E+00_JPRB, 0.57761E+00_JPRB, 0.58277E+00_JPRB, 0.58736E+00_JPRB, 0.59282E+00_JPRB ,&
&  0.59741E+00_JPRB, 0.60212E+00_JPRB, 0.60689E+00_JPRB, 0.61227E+00_JPRB, 0.61851E+00_JPRB, 0.62514E+00_JPRB, 0.63210E+00_JPRB ,&
&  0.63876E+00_JPRB, 0.64650E+00_JPRB, 0.65445E+00_JPRB, 0.66251E+00_JPRB, 0.67045E+00_JPRB, 0.67849E+00_JPRB, 0.68754E+00_JPRB ,&
&  0.69603E+00_JPRB, 0.70496E+00_JPRB, 0.71106E+00_JPRB, 0.71758E+00_JPRB, 0.72198E+00_JPRB, 0.72721E+00_JPRB, 0.72851E+00_JPRB ,&
&  0.73031E+00_JPRB, 0.72826E+00_JPRB, 0.72800E+00_JPRB, 0.72645E+00_JPRB, 0.72522E+00_JPRB, 0.72468E+00_JPRB, 0.72343E+00_JPRB ,&
&  0.72300E+00_JPRB, 0.72134E+00_JPRB, 0.71974E+00_JPRB, 0.71636E+00_JPRB, 0.71141E+00_JPRB, 0.70498E+00_JPRB, 0.69819E+00_JPRB ,&
&  0.69310E+00_JPRB, 0.68914E+00_JPRB, 0.68534E+00_JPRB, 0.68196E+00_JPRB, 0.67674E+00_JPRB, 0.66993E+00_JPRB, 0.66246E+00_JPRB ,&
&  0.65595E+00_JPRB, 0.64986E+00_JPRB, 0.64482E+00_JPRB, 0.64076E+00_JPRB, 0.63846E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.64318E+00_JPRB, 0.64408E+00_JPRB, 0.64423E+00_JPRB, 0.64402E+00_JPRB, 0.64178E+00_JPRB, 0.63971E+00_JPRB, 0.63746E+00_JPRB ,&
&  0.63508E+00_JPRB, 0.63284E+00_JPRB, 0.63047E+00_JPRB, 0.62864E+00_JPRB, 0.62650E+00_JPRB, 0.62489E+00_JPRB, 0.62245E+00_JPRB ,&
&  0.62136E+00_JPRB, 0.61927E+00_JPRB, 0.61708E+00_JPRB, 0.61257E+00_JPRB, 0.60797E+00_JPRB, 0.60258E+00_JPRB, 0.59282E+00_JPRB ,&
&  0.58177E+00_JPRB, 0.57158E+00_JPRB, 0.54544E+00_JPRB, 0.53879E+00_JPRB, 0.52579E+00_JPRB, 0.51854E+00_JPRB, 0.51556E+00_JPRB ,&
&  0.51033E+00_JPRB, 0.50657E+00_JPRB, 0.49637E+00_JPRB, 0.48704E+00_JPRB, 0.47989E+00_JPRB, 0.46795E+00_JPRB, 0.46174E+00_JPRB ,&
&  0.45422E+00_JPRB, 0.45076E+00_JPRB, 0.45226E+00_JPRB, 0.45677E+00_JPRB, 0.46820E+00_JPRB, 0.48154E+00_JPRB, 0.49933E+00_JPRB ,&
&  0.52412E+00_JPRB, 0.54653E+00_JPRB, 0.56833E+00_JPRB, 0.57161E+00_JPRB, 0.57965E+00_JPRB, 0.58694E+00_JPRB, 0.59247E+00_JPRB ,&
&  0.58994E+00_JPRB, 0.59098E+00_JPRB, 0.60376E+00_JPRB, 0.60921E+00_JPRB, 0.61547E+00_JPRB, 0.62245E+00_JPRB, 0.62966E+00_JPRB ,&
&  0.63732E+00_JPRB, 0.64584E+00_JPRB, 0.65481E+00_JPRB, 0.66572E+00_JPRB, 0.67988E+00_JPRB, 0.69466E+00_JPRB, 0.70713E+00_JPRB ,&
&  0.71816E+00_JPRB, 0.73143E+00_JPRB, 0.74101E+00_JPRB, 0.74997E+00_JPRB, 0.75823E+00_JPRB, 0.76977E+00_JPRB, 0.77869E+00_JPRB ,&
&  0.78376E+00_JPRB, 0.78047E+00_JPRB, 0.77269E+00_JPRB, 0.75735E+00_JPRB, 0.74971E+00_JPRB, 0.75570E+00_JPRB, 0.75271E+00_JPRB ,&
&  0.75150E+00_JPRB, 0.74752E+00_JPRB, 0.74444E+00_JPRB, 0.73936E+00_JPRB, 0.73395E+00_JPRB, 0.72625E+00_JPRB, 0.71656E+00_JPRB ,&
&  0.70844E+00_JPRB, 0.70404E+00_JPRB, 0.69975E+00_JPRB, 0.69522E+00_JPRB, 0.68708E+00_JPRB, 0.67852E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.66293E+00_JPRB, 0.65639E+00_JPRB, 0.65027E+00_JPRB, 0.64631E+00_JPRB, 0.64407E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.65728E+00_JPRB, 0.65812E+00_JPRB, 0.65666E+00_JPRB, 0.65540E+00_JPRB, 0.65250E+00_JPRB, 0.64993E+00_JPRB, 0.64641E+00_JPRB ,&
&  0.64308E+00_JPRB, 0.63977E+00_JPRB, 0.63663E+00_JPRB, 0.63389E+00_JPRB, 0.63119E+00_JPRB, 0.62868E+00_JPRB, 0.62696E+00_JPRB ,&
&  0.62531E+00_JPRB, 0.62263E+00_JPRB, 0.61844E+00_JPRB, 0.61322E+00_JPRB, 0.60868E+00_JPRB, 0.60375E+00_JPRB, 0.59732E+00_JPRB ,&
&  0.59045E+00_JPRB, 0.58013E+00_JPRB, 0.56301E+00_JPRB, 0.55542E+00_JPRB, 0.54505E+00_JPRB, 0.54146E+00_JPRB, 0.53519E+00_JPRB ,&
&  0.53307E+00_JPRB, 0.49616E+00_JPRB, 0.49502E+00_JPRB, 0.48066E+00_JPRB, 0.47032E+00_JPRB, 0.46231E+00_JPRB, 0.44652E+00_JPRB ,&
&  0.43950E+00_JPRB, 0.43053E+00_JPRB, 0.42706E+00_JPRB, 0.43115E+00_JPRB, 0.43890E+00_JPRB, 0.45496E+00_JPRB, 0.47565E+00_JPRB ,&
&  0.50049E+00_JPRB, 0.55200E+00_JPRB, 0.56520E+00_JPRB, 0.57502E+00_JPRB, 0.58456E+00_JPRB, 0.59192E+00_JPRB, 0.58335E+00_JPRB ,&
&  0.58243E+00_JPRB, 0.58171E+00_JPRB, 0.58741E+00_JPRB, 0.59766E+00_JPRB, 0.60929E+00_JPRB, 0.61400E+00_JPRB, 0.62051E+00_JPRB ,&
&  0.62645E+00_JPRB, 0.63591E+00_JPRB, 0.64606E+00_JPRB, 0.66158E+00_JPRB, 0.68530E+00_JPRB, 0.71006E+00_JPRB, 0.73122E+00_JPRB ,&
&  0.74858E+00_JPRB, 0.75892E+00_JPRB, 0.76421E+00_JPRB, 0.77450E+00_JPRB, 0.78896E+00_JPRB, 0.80968E+00_JPRB, 0.82244E+00_JPRB ,&
&  0.83022E+00_JPRB, 0.83652E+00_JPRB, 0.83419E+00_JPRB, 0.82714E+00_JPRB, 0.81115E+00_JPRB, 0.79568E+00_JPRB, 0.78787E+00_JPRB ,&
&  0.79098E+00_JPRB, 0.78553E+00_JPRB, 0.77669E+00_JPRB, 0.76312E+00_JPRB, 0.75398E+00_JPRB, 0.74256E+00_JPRB, 0.73069E+00_JPRB ,&
&  0.72053E+00_JPRB, 0.71435E+00_JPRB, 0.70664E+00_JPRB, 0.69996E+00_JPRB, 0.68970E+00_JPRB, 0.68117E+00_JPRB, 0.67466E+00_JPRB ,&
&  0.66821E+00_JPRB, 0.66354E+00_JPRB, 0.65865E+00_JPRB, 0.65678E+00_JPRB, 0.65710E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.67115E+00_JPRB, 0.67049E+00_JPRB, 0.66874E+00_JPRB, 0.66704E+00_JPRB, 0.66474E+00_JPRB, 0.66338E+00_JPRB, 0.66253E+00_JPRB ,&
&  0.66077E+00_JPRB, 0.65902E+00_JPRB, 0.65710E+00_JPRB, 0.65505E+00_JPRB, 0.65283E+00_JPRB, 0.65090E+00_JPRB, 0.64892E+00_JPRB ,&
&  0.64517E+00_JPRB, 0.64023E+00_JPRB, 0.63333E+00_JPRB, 0.62682E+00_JPRB, 0.62100E+00_JPRB, 0.61788E+00_JPRB, 0.61264E+00_JPRB ,&
&  0.60739E+00_JPRB, 0.59902E+00_JPRB, 0.58567E+00_JPRB, 0.57613E+00_JPRB, 0.57353E+00_JPRB, 0.56560E+00_JPRB, 0.56542E+00_JPRB ,&
&  0.56180E+00_JPRB, 0.55556E+00_JPRB, 0.54018E+00_JPRB, 0.53474E+00_JPRB, 0.52522E+00_JPRB, 0.47337E+00_JPRB, 0.46153E+00_JPRB ,&
&  0.44552E+00_JPRB, 0.42754E+00_JPRB, 0.42063E+00_JPRB, 0.41846E+00_JPRB, 0.42224E+00_JPRB, 0.43922E+00_JPRB, 0.46179E+00_JPRB ,&
&  0.48500E+00_JPRB, 0.54863E+00_JPRB, 0.56511E+00_JPRB, 0.57883E+00_JPRB, 0.58810E+00_JPRB, 0.59288E+00_JPRB, 0.58839E+00_JPRB ,&
&  0.58608E+00_JPRB, 0.58611E+00_JPRB, 0.59377E+00_JPRB, 0.60652E+00_JPRB, 0.61625E+00_JPRB, 0.61566E+00_JPRB, 0.61562E+00_JPRB ,&
&  0.61157E+00_JPRB, 0.61867E+00_JPRB, 0.63292E+00_JPRB, 0.65323E+00_JPRB, 0.68154E+00_JPRB, 0.70319E+00_JPRB, 0.73207E+00_JPRB ,&
&  0.75722E+00_JPRB, 0.77432E+00_JPRB, 0.80116E+00_JPRB, 0.82848E+00_JPRB, 0.84760E+00_JPRB, 0.86022E+00_JPRB, 0.86253E+00_JPRB ,&
&  0.85918E+00_JPRB, 0.87768E+00_JPRB, 0.88658E+00_JPRB, 0.88326E+00_JPRB, 0.87299E+00_JPRB, 0.85299E+00_JPRB, 0.83169E+00_JPRB ,&
&  0.82175E+00_JPRB, 0.82565E+00_JPRB, 0.81686E+00_JPRB, 0.79198E+00_JPRB, 0.77147E+00_JPRB, 0.75477E+00_JPRB, 0.74138E+00_JPRB ,&
&  0.73155E+00_JPRB, 0.72535E+00_JPRB, 0.71621E+00_JPRB, 0.70797E+00_JPRB, 0.69861E+00_JPRB, 0.68994E+00_JPRB, 0.68468E+00_JPRB ,&
&  0.68028E+00_JPRB, 0.67890E+00_JPRB, 0.67673E+00_JPRB, 0.67467E+00_JPRB, 0.67395E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.68522E+00_JPRB, 0.68565E+00_JPRB, 0.68453E+00_JPRB, 0.68286E+00_JPRB, 0.68224E+00_JPRB, 0.68361E+00_JPRB, 0.68201E+00_JPRB ,&
&  0.67878E+00_JPRB, 0.67604E+00_JPRB, 0.67153E+00_JPRB, 0.66991E+00_JPRB, 0.66768E+00_JPRB, 0.66497E+00_JPRB, 0.66257E+00_JPRB ,&
&  0.65862E+00_JPRB, 0.64594E+00_JPRB, 0.63803E+00_JPRB, 0.63544E+00_JPRB, 0.63284E+00_JPRB, 0.63196E+00_JPRB, 0.63104E+00_JPRB ,&
&  0.62564E+00_JPRB, 0.61703E+00_JPRB, 0.60823E+00_JPRB, 0.59356E+00_JPRB, 0.58279E+00_JPRB, 0.58333E+00_JPRB, 0.58868E+00_JPRB ,&
&  0.58321E+00_JPRB, 0.56663E+00_JPRB, 0.56899E+00_JPRB, 0.56694E+00_JPRB, 0.54523E+00_JPRB, 0.53667E+00_JPRB, 0.49337E+00_JPRB ,&
&  0.46354E+00_JPRB, 0.43734E+00_JPRB, 0.42773E+00_JPRB, 0.42165E+00_JPRB, 0.42727E+00_JPRB, 0.44187E+00_JPRB, 0.47532E+00_JPRB ,&
&  0.51019E+00_JPRB, 0.54686E+00_JPRB, 0.55982E+00_JPRB, 0.57384E+00_JPRB, 0.58147E+00_JPRB, 0.59448E+00_JPRB, 0.60475E+00_JPRB ,&
&  0.61108E+00_JPRB, 0.61663E+00_JPRB, 0.62674E+00_JPRB, 0.63613E+00_JPRB, 0.65528E+00_JPRB, 0.67155E+00_JPRB, 0.68004E+00_JPRB ,&
&  0.69229E+00_JPRB, 0.69672E+00_JPRB, 0.70945E+00_JPRB, 0.72503E+00_JPRB, 0.74559E+00_JPRB, 0.75679E+00_JPRB, 0.76903E+00_JPRB ,&
&  0.78343E+00_JPRB, 0.81756E+00_JPRB, 0.86658E+00_JPRB, 0.92577E+00_JPRB, 0.94302E+00_JPRB, 0.93916E+00_JPRB, 0.91343E+00_JPRB ,&
&  0.88238E+00_JPRB, 0.90496E+00_JPRB, 0.10178E+01_JPRB, 0.10040E+01_JPRB, 0.99191E+00_JPRB, 0.97352E+00_JPRB, 0.92796E+00_JPRB ,&
&  0.87421E+00_JPRB, 0.85243E+00_JPRB, 0.83166E+00_JPRB, 0.79573E+00_JPRB, 0.76644E+00_JPRB, 0.74694E+00_JPRB, 0.73659E+00_JPRB ,&
&  0.73042E+00_JPRB, 0.72914E+00_JPRB, 0.72257E+00_JPRB, 0.71750E+00_JPRB, 0.70881E+00_JPRB, 0.69941E+00_JPRB, 0.69459E+00_JPRB ,&
&  0.69027E+00_JPRB, 0.68937E+00_JPRB, 0.68972E+00_JPRB, 0.68917E+00_JPRB, 0.68784E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.69849E+00_JPRB, 0.70323E+00_JPRB, 0.70648E+00_JPRB, 0.71419E+00_JPRB, 0.72179E+00_JPRB, 0.72310E+00_JPRB, 0.71803E+00_JPRB ,&
&  0.70531E+00_JPRB, 0.68988E+00_JPRB, 0.67760E+00_JPRB, 0.66889E+00_JPRB, 0.66351E+00_JPRB, 0.66028E+00_JPRB, 0.65721E+00_JPRB ,&
&  0.65505E+00_JPRB, 0.65283E+00_JPRB, 0.64691E+00_JPRB, 0.63843E+00_JPRB, 0.63442E+00_JPRB, 0.63494E+00_JPRB, 0.63614E+00_JPRB ,&
&  0.63497E+00_JPRB, 0.62939E+00_JPRB, 0.62144E+00_JPRB, 0.60792E+00_JPRB, 0.59523E+00_JPRB, 0.59816E+00_JPRB, 0.59193E+00_JPRB ,&
&  0.58643E+00_JPRB, 0.58446E+00_JPRB, 0.58351E+00_JPRB, 0.58041E+00_JPRB, 0.57530E+00_JPRB, 0.55731E+00_JPRB, 0.52077E+00_JPRB ,&
&  0.48337E+00_JPRB, 0.45349E+00_JPRB, 0.43593E+00_JPRB, 0.43069E+00_JPRB, 0.44004E+00_JPRB, 0.46499E+00_JPRB, 0.50045E+00_JPRB ,&
&  0.54113E+00_JPRB, 0.55852E+00_JPRB, 0.57452E+00_JPRB, 0.59171E+00_JPRB, 0.60605E+00_JPRB, 0.62511E+00_JPRB, 0.64359E+00_JPRB ,&
&  0.66410E+00_JPRB, 0.69242E+00_JPRB, 0.73074E+00_JPRB, 0.78145E+00_JPRB, 0.83962E+00_JPRB, 0.83082E+00_JPRB, 0.87832E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.10119E+01_JPRB, 0.97416E+00_JPRB, 0.92897E+00_JPRB, 0.90181E+00_JPRB, 0.89296E+00_JPRB, 0.89817E+00_JPRB ,&
&  0.90861E+00_JPRB, 0.93052E+00_JPRB, 0.96754E+00_JPRB, 0.10107E+01_JPRB, 0.10252E+01_JPRB, 0.10203E+01_JPRB, 0.98259E+00_JPRB ,&
&  0.94894E+00_JPRB, 0.97230E+00_JPRB, 0.11979E+01_JPRB, 0.10992E+01_JPRB, 0.10127E+01_JPRB, 0.97181E+00_JPRB, 0.92033E+00_JPRB ,&
&  0.88309E+00_JPRB, 0.85259E+00_JPRB, 0.81867E+00_JPRB, 0.79044E+00_JPRB, 0.75795E+00_JPRB, 0.72820E+00_JPRB, 0.70242E+00_JPRB ,&
&  0.69416E+00_JPRB, 0.69240E+00_JPRB, 0.69198E+00_JPRB, 0.69231E+00_JPRB, 0.68802E+00_JPRB, 0.68744E+00_JPRB, 0.68860E+00_JPRB ,&
&  0.68328E+00_JPRB, 0.68190E+00_JPRB, 0.68608E+00_JPRB, 0.69065E+00_JPRB, 0.69443E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.67582E+00_JPRB, 0.69491E+00_JPRB, 0.72633E+00_JPRB, 0.74689E+00_JPRB, 0.75469E+00_JPRB, 0.73810E+00_JPRB, 0.71703E+00_JPRB ,&
&  0.69404E+00_JPRB, 0.66636E+00_JPRB, 0.64963E+00_JPRB, 0.64016E+00_JPRB, 0.63487E+00_JPRB, 0.63110E+00_JPRB, 0.62820E+00_JPRB ,&
&  0.62636E+00_JPRB, 0.62480E+00_JPRB, 0.62614E+00_JPRB, 0.63154E+00_JPRB, 0.63567E+00_JPRB, 0.63473E+00_JPRB, 0.63262E+00_JPRB ,&
&  0.63177E+00_JPRB, 0.62919E+00_JPRB, 0.62016E+00_JPRB, 0.60609E+00_JPRB, 0.60092E+00_JPRB, 0.61090E+00_JPRB, 0.61067E+00_JPRB ,&
&  0.60529E+00_JPRB, 0.59687E+00_JPRB, 0.59668E+00_JPRB, 0.60088E+00_JPRB, 0.59759E+00_JPRB, 0.58380E+00_JPRB, 0.54955E+00_JPRB ,&
&  0.50709E+00_JPRB, 0.47267E+00_JPRB, 0.46159E+00_JPRB, 0.47596E+00_JPRB, 0.50618E+00_JPRB, 0.54859E+00_JPRB, 0.56619E+00_JPRB ,&
&  0.57940E+00_JPRB, 0.59082E+00_JPRB, 0.60295E+00_JPRB, 0.62449E+00_JPRB, 0.65123E+00_JPRB, 0.67618E+00_JPRB, 0.70132E+00_JPRB ,&
&  0.73894E+00_JPRB, 0.78055E+00_JPRB, 0.83240E+00_JPRB, 0.80637E+00_JPRB, 0.83515E+00_JPRB, 0.90834E+00_JPRB, 0.94705E+00_JPRB ,&
&  0.96953E+00_JPRB, 0.10121E+01_JPRB, 0.10420E+01_JPRB, 0.10807E+01_JPRB, 0.10686E+01_JPRB, 0.10489E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10206E+01_JPRB, 0.10297E+01_JPRB, 0.10739E+01_JPRB, 0.11178E+01_JPRB, 0.11192E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.10367E+01_JPRB, 0.96051E+00_JPRB, 0.88868E+00_JPRB, 0.84128E+00_JPRB, 0.83840E+00_JPRB, 0.85621E+00_JPRB, 0.86574E+00_JPRB ,&
&  0.86269E+00_JPRB, 0.85487E+00_JPRB, 0.83340E+00_JPRB, 0.81488E+00_JPRB, 0.75644E+00_JPRB, 0.71415E+00_JPRB, 0.68109E+00_JPRB ,&
&  0.66855E+00_JPRB, 0.66119E+00_JPRB, 0.65822E+00_JPRB, 0.66638E+00_JPRB, 0.66274E+00_JPRB, 0.66506E+00_JPRB, 0.66761E+00_JPRB ,&
&  0.66509E+00_JPRB, 0.66006E+00_JPRB, 0.66052E+00_JPRB, 0.66268E+00_JPRB, 0.66725E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.74655E+00_JPRB, 0.76226E+00_JPRB, 0.77559E+00_JPRB, 0.78115E+00_JPRB, 0.76961E+00_JPRB, 0.75929E+00_JPRB, 0.74931E+00_JPRB ,&
&  0.74366E+00_JPRB, 0.69273E+00_JPRB, 0.65698E+00_JPRB, 0.63151E+00_JPRB, 0.61285E+00_JPRB, 0.59566E+00_JPRB, 0.59064E+00_JPRB ,&
&  0.59259E+00_JPRB, 0.59734E+00_JPRB, 0.60354E+00_JPRB, 0.61068E+00_JPRB, 0.61724E+00_JPRB, 0.61952E+00_JPRB, 0.62394E+00_JPRB ,&
&  0.63145E+00_JPRB, 0.63605E+00_JPRB, 0.63180E+00_JPRB, 0.62874E+00_JPRB, 0.62930E+00_JPRB, 0.63587E+00_JPRB, 0.63451E+00_JPRB ,&
&  0.63089E+00_JPRB, 0.62486E+00_JPRB, 0.62163E+00_JPRB, 0.62676E+00_JPRB, 0.62795E+00_JPRB, 0.61712E+00_JPRB, 0.57608E+00_JPRB ,&
&  0.52027E+00_JPRB, 0.49953E+00_JPRB, 0.51702E+00_JPRB, 0.56709E+00_JPRB, 0.58616E+00_JPRB, 0.59969E+00_JPRB, 0.61396E+00_JPRB ,&
&  0.60768E+00_JPRB, 0.60551E+00_JPRB, 0.62442E+00_JPRB, 0.67162E+00_JPRB, 0.69400E+00_JPRB, 0.71627E+00_JPRB, 0.74490E+00_JPRB ,&
&  0.79580E+00_JPRB, 0.83640E+00_JPRB, 0.88367E+00_JPRB, 0.85165E+00_JPRB, 0.94138E+00_JPRB, 0.11345E+01_JPRB, 0.11058E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.11135E+01_JPRB, 0.11686E+01_JPRB, 0.12030E+01_JPRB, 0.12024E+01_JPRB, 0.11841E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.11561E+01_JPRB, 0.11497E+01_JPRB, 0.11624E+01_JPRB, 0.12219E+01_JPRB, 0.12210E+01_JPRB, 0.11986E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.10308E+01_JPRB, 0.95628E+00_JPRB, 0.90106E+00_JPRB, 0.88614E+00_JPRB, 0.88950E+00_JPRB, 0.88476E+00_JPRB ,&
&  0.86663E+00_JPRB, 0.86265E+00_JPRB, 0.86293E+00_JPRB, 0.85843E+00_JPRB, 0.79859E+00_JPRB, 0.73066E+00_JPRB, 0.68050E+00_JPRB ,&
&  0.64506E+00_JPRB, 0.62358E+00_JPRB, 0.60754E+00_JPRB, 0.61270E+00_JPRB, 0.62861E+00_JPRB, 0.64871E+00_JPRB, 0.64883E+00_JPRB ,&
&  0.65166E+00_JPRB, 0.65253E+00_JPRB, 0.66423E+00_JPRB, 0.69319E+00_JPRB, 0.72241E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.78814E+00_JPRB, 0.80472E+00_JPRB, 0.81657E+00_JPRB, 0.80682E+00_JPRB, 0.78913E+00_JPRB, 0.77628E+00_JPRB, 0.76245E+00_JPRB ,&
&  0.75500E+00_JPRB, 0.67778E+00_JPRB, 0.64171E+00_JPRB, 0.62325E+00_JPRB, 0.61006E+00_JPRB, 0.58567E+00_JPRB, 0.56676E+00_JPRB ,&
&  0.56538E+00_JPRB, 0.57804E+00_JPRB, 0.59871E+00_JPRB, 0.61425E+00_JPRB, 0.62863E+00_JPRB, 0.63935E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.65521E+00_JPRB, 0.65631E+00_JPRB, 0.65739E+00_JPRB, 0.66093E+00_JPRB, 0.67188E+00_JPRB, 0.68650E+00_JPRB ,&
&  0.69478E+00_JPRB, 0.69544E+00_JPRB, 0.67872E+00_JPRB, 0.67687E+00_JPRB, 0.67793E+00_JPRB, 0.68002E+00_JPRB, 0.67577E+00_JPRB ,&
&  0.56045E+00_JPRB, 0.55450E+00_JPRB, 0.60629E+00_JPRB, 0.61052E+00_JPRB, 0.62301E+00_JPRB, 0.62669E+00_JPRB, 0.62831E+00_JPRB ,&
&  0.63493E+00_JPRB, 0.65136E+00_JPRB, 0.67842E+00_JPRB, 0.71149E+00_JPRB, 0.73747E+00_JPRB, 0.75983E+00_JPRB, 0.78769E+00_JPRB ,&
&  0.85706E+00_JPRB, 0.80770E+00_JPRB, 0.80028E+00_JPRB, 0.87723E+00_JPRB, 0.10442E+01_JPRB, 0.11632E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.13076E+01_JPRB, 0.13078E+01_JPRB, 0.13329E+01_JPRB, 0.13394E+01_JPRB, 0.13335E+01_JPRB, 0.13569E+01_JPRB ,&
&  0.13544E+01_JPRB, 0.13711E+01_JPRB, 0.13968E+01_JPRB, 0.14184E+01_JPRB, 0.13658E+01_JPRB, 0.13237E+01_JPRB, 0.12746E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.11451E+01_JPRB, 0.10640E+01_JPRB, 0.99927E+00_JPRB, 0.95388E+00_JPRB, 0.94967E+00_JPRB, 0.94261E+00_JPRB ,&
&  0.92235E+00_JPRB, 0.91000E+00_JPRB, 0.89269E+00_JPRB, 0.87542E+00_JPRB, 0.83412E+00_JPRB, 0.79341E+00_JPRB, 0.77292E+00_JPRB ,&
&  0.71424E+00_JPRB, 0.67144E+00_JPRB, 0.62280E+00_JPRB, 0.62307E+00_JPRB, 0.65775E+00_JPRB, 0.67685E+00_JPRB, 0.68304E+00_JPRB ,&
&  0.70749E+00_JPRB, 0.72319E+00_JPRB, 0.73029E+00_JPRB, 0.74700E+00_JPRB, 0.76822E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.81228E+00_JPRB, 0.81597E+00_JPRB, 0.81448E+00_JPRB, 0.80582E+00_JPRB, 0.80612E+00_JPRB, 0.79990E+00_JPRB, 0.80254E+00_JPRB ,&
&  0.82682E+00_JPRB, 0.78007E+00_JPRB, 0.81862E+00_JPRB, 0.69890E+00_JPRB, 0.63575E+00_JPRB, 0.60276E+00_JPRB, 0.58075E+00_JPRB ,&
&  0.58052E+00_JPRB, 0.59696E+00_JPRB, 0.61282E+00_JPRB, 0.62737E+00_JPRB, 0.65379E+00_JPRB, 0.65909E+00_JPRB, 0.66016E+00_JPRB ,&
&  0.66609E+00_JPRB, 0.67304E+00_JPRB, 0.69003E+00_JPRB, 0.68904E+00_JPRB, 0.70268E+00_JPRB, 0.72357E+00_JPRB, 0.76921E+00_JPRB ,&
&  0.74113E+00_JPRB, 0.73850E+00_JPRB, 0.76593E+00_JPRB, 0.77113E+00_JPRB, 0.77346E+00_JPRB, 0.76506E+00_JPRB, 0.75218E+00_JPRB ,&
&  0.74463E+00_JPRB, 0.67343E+00_JPRB, 0.69249E+00_JPRB, 0.69138E+00_JPRB, 0.68880E+00_JPRB, 0.68681E+00_JPRB, 0.67901E+00_JPRB ,&
&  0.67850E+00_JPRB, 0.68996E+00_JPRB, 0.71288E+00_JPRB, 0.74058E+00_JPRB, 0.77599E+00_JPRB, 0.81302E+00_JPRB, 0.84686E+00_JPRB ,&
&  0.92963E+00_JPRB, 0.85748E+00_JPRB, 0.10034E+01_JPRB, 0.10815E+01_JPRB, 0.12176E+01_JPRB, 0.14011E+01_JPRB, 0.15095E+01_JPRB ,&
&  0.16951E+01_JPRB, 0.16563E+01_JPRB, 0.16450E+01_JPRB, 0.16148E+01_JPRB, 0.15716E+01_JPRB, 0.15503E+01_JPRB, 0.15539E+01_JPRB ,&
&  0.16024E+01_JPRB, 0.16126E+01_JPRB, 0.17227E+01_JPRB, 0.16793E+01_JPRB, 0.15608E+01_JPRB, 0.14823E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.12812E+01_JPRB, 0.12322E+01_JPRB, 0.11167E+01_JPRB, 0.10867E+01_JPRB, 0.10676E+01_JPRB, 0.10005E+01_JPRB ,&
&  0.96788E+00_JPRB, 0.91734E+00_JPRB, 0.85734E+00_JPRB, 0.86385E+00_JPRB, 0.81940E+00_JPRB, 0.78592E+00_JPRB, 0.78062E+00_JPRB ,&
&  0.75551E+00_JPRB, 0.69671E+00_JPRB, 0.69771E+00_JPRB, 0.72260E+00_JPRB, 0.71746E+00_JPRB, 0.70974E+00_JPRB, 0.73849E+00_JPRB ,&
&  0.75134E+00_JPRB, 0.76691E+00_JPRB, 0.80459E+00_JPRB, 0.80963E+00_JPRB, 0.81702E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.82613E+00_JPRB, 0.80988E+00_JPRB, 0.80025E+00_JPRB, 0.80075E+00_JPRB, 0.81793E+00_JPRB, 0.84836E+00_JPRB, 0.83900E+00_JPRB ,&
&  0.86084E+00_JPRB, 0.86866E+00_JPRB, 0.85693E+00_JPRB, 0.83374E+00_JPRB, 0.79706E+00_JPRB, 0.70567E+00_JPRB, 0.60365E+00_JPRB ,&
&  0.57943E+00_JPRB, 0.60317E+00_JPRB, 0.63055E+00_JPRB, 0.64238E+00_JPRB, 0.66398E+00_JPRB, 0.66627E+00_JPRB, 0.66888E+00_JPRB ,&
&  0.67547E+00_JPRB, 0.68610E+00_JPRB, 0.70432E+00_JPRB, 0.72809E+00_JPRB, 0.74198E+00_JPRB, 0.78554E+00_JPRB, 0.84370E+00_JPRB ,&
&  0.79268E+00_JPRB, 0.75991E+00_JPRB, 0.75832E+00_JPRB, 0.74102E+00_JPRB, 0.78903E+00_JPRB, 0.80609E+00_JPRB, 0.80959E+00_JPRB ,&
&  0.80464E+00_JPRB, 0.79328E+00_JPRB, 0.77125E+00_JPRB, 0.75810E+00_JPRB, 0.74807E+00_JPRB, 0.74614E+00_JPRB, 0.75012E+00_JPRB ,&
&  0.76606E+00_JPRB, 0.78644E+00_JPRB, 0.81591E+00_JPRB, 0.86006E+00_JPRB, 0.91226E+00_JPRB, 0.94403E+00_JPRB, 0.10196E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.12311E+01_JPRB, 0.13943E+01_JPRB, 0.13700E+01_JPRB, 0.15167E+01_JPRB, 0.16591E+01_JPRB, 0.16967E+01_JPRB ,&
&  0.18066E+01_JPRB, 0.17788E+01_JPRB, 0.18481E+01_JPRB, 0.19017E+01_JPRB, 0.19417E+01_JPRB, 0.19313E+01_JPRB, 0.18664E+01_JPRB ,&
&  0.18471E+01_JPRB, 0.20083E+01_JPRB, 0.20368E+01_JPRB, 0.19424E+01_JPRB, 0.17730E+01_JPRB, 0.16608E+01_JPRB, 0.15341E+01_JPRB ,&
&  0.14876E+01_JPRB, 0.14882E+01_JPRB, 0.14362E+01_JPRB, 0.13800E+01_JPRB, 0.12834E+01_JPRB, 0.11778E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.88188E+00_JPRB, 0.82146E+00_JPRB, 0.80703E+00_JPRB, 0.79958E+00_JPRB, 0.76069E+00_JPRB, 0.76265E+00_JPRB ,&
&  0.76936E+00_JPRB, 0.75881E+00_JPRB, 0.79624E+00_JPRB, 0.81828E+00_JPRB, 0.80489E+00_JPRB, 0.78407E+00_JPRB, 0.75365E+00_JPRB ,&
&  0.73520E+00_JPRB, 0.77825E+00_JPRB, 0.82004E+00_JPRB, 0.83944E+00_JPRB, 0.83896E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.84251E+00_JPRB, 0.82222E+00_JPRB, 0.79622E+00_JPRB, 0.79833E+00_JPRB, 0.83358E+00_JPRB, 0.86774E+00_JPRB, 0.88847E+00_JPRB ,&
&  0.88353E+00_JPRB, 0.88328E+00_JPRB, 0.87151E+00_JPRB, 0.85620E+00_JPRB, 0.82902E+00_JPRB, 0.79600E+00_JPRB, 0.72171E+00_JPRB ,&
&  0.63569E+00_JPRB, 0.66659E+00_JPRB, 0.64104E+00_JPRB, 0.65081E+00_JPRB, 0.69460E+00_JPRB, 0.68903E+00_JPRB, 0.68883E+00_JPRB ,&
&  0.69984E+00_JPRB, 0.71108E+00_JPRB, 0.73391E+00_JPRB, 0.75045E+00_JPRB, 0.77506E+00_JPRB, 0.87026E+00_JPRB, 0.86793E+00_JPRB ,&
&  0.78767E+00_JPRB, 0.74255E+00_JPRB, 0.74518E+00_JPRB, 0.76707E+00_JPRB, 0.79968E+00_JPRB, 0.83239E+00_JPRB, 0.83337E+00_JPRB ,&
&  0.82347E+00_JPRB, 0.80677E+00_JPRB, 0.77519E+00_JPRB, 0.76136E+00_JPRB, 0.76181E+00_JPRB, 0.77423E+00_JPRB, 0.78868E+00_JPRB ,&
&  0.80204E+00_JPRB, 0.82606E+00_JPRB, 0.86696E+00_JPRB, 0.93206E+00_JPRB, 0.99660E+00_JPRB, 0.11281E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.13493E+01_JPRB, 0.14550E+01_JPRB, 0.15798E+01_JPRB, 0.17254E+01_JPRB, 0.19079E+01_JPRB, 0.19575E+01_JPRB, 0.19785E+01_JPRB ,&
&  0.20304E+01_JPRB, 0.19905E+01_JPRB, 0.22579E+01_JPRB, 0.23030E+01_JPRB, 0.22755E+01_JPRB, 0.22885E+01_JPRB, 0.21680E+01_JPRB ,&
&  0.20709E+01_JPRB, 0.20076E+01_JPRB, 0.19784E+01_JPRB, 0.19179E+01_JPRB, 0.18453E+01_JPRB, 0.18402E+01_JPRB, 0.17224E+01_JPRB ,&
&  0.17123E+01_JPRB, 0.16702E+01_JPRB, 0.14380E+01_JPRB, 0.12621E+01_JPRB, 0.10933E+01_JPRB, 0.11093E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.98969E+00_JPRB, 0.84836E+00_JPRB, 0.83738E+00_JPRB, 0.82077E+00_JPRB, 0.82835E+00_JPRB, 0.86206E+00_JPRB, 0.86897E+00_JPRB ,&
&  0.88070E+00_JPRB, 0.91145E+00_JPRB, 0.92055E+00_JPRB, 0.97003E+00_JPRB, 0.94357E+00_JPRB, 0.89274E+00_JPRB, 0.78047E+00_JPRB ,&
&  0.74819E+00_JPRB, 0.78199E+00_JPRB, 0.81828E+00_JPRB, 0.84373E+00_JPRB, 0.85142E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.87813E+00_JPRB, 0.85987E+00_JPRB, 0.83475E+00_JPRB, 0.83398E+00_JPRB, 0.86608E+00_JPRB, 0.90084E+00_JPRB, 0.92022E+00_JPRB ,&
&  0.91275E+00_JPRB, 0.91127E+00_JPRB, 0.89071E+00_JPRB, 0.87355E+00_JPRB, 0.85359E+00_JPRB, 0.82031E+00_JPRB, 0.81348E+00_JPRB ,&
&  0.69005E+00_JPRB, 0.71940E+00_JPRB, 0.64080E+00_JPRB, 0.63945E+00_JPRB, 0.74019E+00_JPRB, 0.73004E+00_JPRB, 0.73756E+00_JPRB ,&
&  0.73086E+00_JPRB, 0.75175E+00_JPRB, 0.75907E+00_JPRB, 0.77506E+00_JPRB, 0.85069E+00_JPRB, 0.92193E+00_JPRB, 0.88782E+00_JPRB ,&
&  0.82925E+00_JPRB, 0.79114E+00_JPRB, 0.79806E+00_JPRB, 0.80536E+00_JPRB, 0.81211E+00_JPRB, 0.83157E+00_JPRB, 0.83754E+00_JPRB ,&
&  0.82122E+00_JPRB, 0.82168E+00_JPRB, 0.81397E+00_JPRB, 0.82232E+00_JPRB, 0.84010E+00_JPRB, 0.86879E+00_JPRB, 0.89062E+00_JPRB ,&
&  0.91736E+00_JPRB, 0.95617E+00_JPRB, 0.10021E+01_JPRB, 0.10789E+01_JPRB, 0.11430E+01_JPRB, 0.13153E+01_JPRB, 0.15493E+01_JPRB ,&
&  0.18492E+01_JPRB, 0.19494E+01_JPRB, 0.20974E+01_JPRB, 0.24488E+01_JPRB, 0.25386E+01_JPRB, 0.24835E+01_JPRB, 0.23278E+01_JPRB ,&
&  0.22717E+01_JPRB, 0.22178E+01_JPRB, 0.22545E+01_JPRB, 0.23366E+01_JPRB, 0.23318E+01_JPRB, 0.24055E+01_JPRB, 0.23020E+01_JPRB ,&
&  0.20696E+01_JPRB, 0.19610E+01_JPRB, 0.19058E+01_JPRB, 0.18943E+01_JPRB, 0.18055E+01_JPRB, 0.18144E+01_JPRB, 0.17284E+01_JPRB ,&
&  0.17345E+01_JPRB, 0.15257E+01_JPRB, 0.11710E+01_JPRB, 0.10788E+01_JPRB, 0.89495E+00_JPRB, 0.91817E+00_JPRB, 0.11196E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.98729E+00_JPRB, 0.91301E+00_JPRB, 0.87865E+00_JPRB, 0.89591E+00_JPRB, 0.11211E+01_JPRB, 0.11971E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.11190E+01_JPRB, 0.11751E+01_JPRB, 0.12181E+01_JPRB, 0.11770E+01_JPRB, 0.11085E+01_JPRB, 0.98508E+00_JPRB ,&
&  0.94493E+00_JPRB, 0.92601E+00_JPRB, 0.92363E+00_JPRB, 0.91313E+00_JPRB, 0.89618E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.96494E+00_JPRB, 0.93636E+00_JPRB, 0.90917E+00_JPRB, 0.90576E+00_JPRB, 0.92613E+00_JPRB, 0.95566E+00_JPRB, 0.97242E+00_JPRB ,&
&  0.95964E+00_JPRB, 0.94984E+00_JPRB, 0.92305E+00_JPRB, 0.89955E+00_JPRB, 0.87720E+00_JPRB, 0.85030E+00_JPRB, 0.83386E+00_JPRB ,&
&  0.84583E+00_JPRB, 0.84104E+00_JPRB, 0.71219E+00_JPRB, 0.63983E+00_JPRB, 0.69635E+00_JPRB, 0.71607E+00_JPRB, 0.75653E+00_JPRB ,&
&  0.74860E+00_JPRB, 0.79929E+00_JPRB, 0.80548E+00_JPRB, 0.83752E+00_JPRB, 0.94694E+00_JPRB, 0.97366E+00_JPRB, 0.96060E+00_JPRB ,&
&  0.95728E+00_JPRB, 0.91661E+00_JPRB, 0.90372E+00_JPRB, 0.91420E+00_JPRB, 0.92528E+00_JPRB, 0.86407E+00_JPRB, 0.87179E+00_JPRB ,&
&  0.87553E+00_JPRB, 0.88696E+00_JPRB, 0.89189E+00_JPRB, 0.91798E+00_JPRB, 0.95839E+00_JPRB, 0.99485E+00_JPRB, 0.10370E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.11118E+01_JPRB, 0.11297E+01_JPRB, 0.11864E+01_JPRB, 0.12778E+01_JPRB, 0.14396E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.19568E+01_JPRB, 0.20758E+01_JPRB, 0.24885E+01_JPRB, 0.26848E+01_JPRB, 0.26536E+01_JPRB, 0.25762E+01_JPRB ,&
&  0.25308E+01_JPRB, 0.24698E+01_JPRB, 0.25450E+01_JPRB, 0.25737E+01_JPRB, 0.25929E+01_JPRB, 0.25649E+01_JPRB, 0.23781E+01_JPRB ,&
&  0.21301E+01_JPRB, 0.19632E+01_JPRB, 0.18720E+01_JPRB, 0.17644E+01_JPRB, 0.16664E+01_JPRB, 0.15709E+01_JPRB, 0.15902E+01_JPRB ,&
&  0.17207E+01_JPRB, 0.10540E+01_JPRB, 0.87138E+00_JPRB, 0.98668E+00_JPRB, 0.86269E+00_JPRB, 0.89327E+00_JPRB, 0.10022E+01_JPRB ,&
&  0.99216E+00_JPRB, 0.98196E+00_JPRB, 0.10214E+01_JPRB, 0.10438E+01_JPRB, 0.11530E+01_JPRB, 0.15135E+01_JPRB, 0.16125E+01_JPRB ,&
&  0.15175E+01_JPRB, 0.14078E+01_JPRB, 0.14782E+01_JPRB, 0.14620E+01_JPRB, 0.14272E+01_JPRB, 0.13824E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.11536E+01_JPRB, 0.10948E+01_JPRB, 0.10387E+01_JPRB, 0.99711E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.11077E+01_JPRB, 0.10742E+01_JPRB, 0.10484E+01_JPRB, 0.10323E+01_JPRB, 0.10334E+01_JPRB, 0.10529E+01_JPRB, 0.10577E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.10140E+01_JPRB, 0.97489E+00_JPRB, 0.94075E+00_JPRB, 0.91231E+00_JPRB, 0.89425E+00_JPRB, 0.87651E+00_JPRB ,&
&  0.85472E+00_JPRB, 0.86813E+00_JPRB, 0.71646E+00_JPRB, 0.65921E+00_JPRB, 0.62647E+00_JPRB, 0.68241E+00_JPRB, 0.73816E+00_JPRB ,&
&  0.76816E+00_JPRB, 0.80669E+00_JPRB, 0.84878E+00_JPRB, 0.99511E+00_JPRB, 0.11451E+01_JPRB, 0.11242E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.11655E+01_JPRB, 0.10567E+01_JPRB, 0.10658E+01_JPRB, 0.98975E+00_JPRB, 0.92299E+00_JPRB, 0.99265E+00_JPRB ,&
&  0.10029E+01_JPRB, 0.10215E+01_JPRB, 0.10334E+01_JPRB, 0.10454E+01_JPRB, 0.10705E+01_JPRB, 0.10784E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.11877E+01_JPRB, 0.12392E+01_JPRB, 0.12781E+01_JPRB, 0.13542E+01_JPRB, 0.14584E+01_JPRB, 0.14846E+01_JPRB ,&
&  0.16301E+01_JPRB, 0.15972E+01_JPRB, 0.13956E+01_JPRB, 0.18554E+01_JPRB, 0.26445E+01_JPRB, 0.27134E+01_JPRB, 0.25342E+01_JPRB ,&
&  0.26994E+01_JPRB, 0.27281E+01_JPRB, 0.28598E+01_JPRB, 0.29934E+01_JPRB, 0.29360E+01_JPRB, 0.27866E+01_JPRB, 0.25644E+01_JPRB ,&
&  0.22748E+01_JPRB, 0.20790E+01_JPRB, 0.19551E+01_JPRB, 0.19987E+01_JPRB, 0.18763E+01_JPRB, 0.18246E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.15909E+01_JPRB, 0.13861E+01_JPRB, 0.10310E+01_JPRB, 0.10568E+01_JPRB, 0.87564E+00_JPRB, 0.89847E+00_JPRB, 0.10123E+01_JPRB ,&
&  0.99981E+00_JPRB, 0.10939E+01_JPRB, 0.11952E+01_JPRB, 0.12580E+01_JPRB, 0.15946E+01_JPRB, 0.19217E+01_JPRB, 0.22128E+01_JPRB ,&
&  0.21714E+01_JPRB, 0.18295E+01_JPRB, 0.18907E+01_JPRB, 0.18198E+01_JPRB, 0.18006E+01_JPRB, 0.17452E+01_JPRB, 0.16594E+01_JPRB ,&
&  0.15467E+01_JPRB, 0.14195E+01_JPRB, 0.13061E+01_JPRB, 0.12139E+01_JPRB, 0.11510E+01_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.12164E+01_JPRB, 0.11687E+01_JPRB, 0.11385E+01_JPRB, 0.11227E+01_JPRB, 0.11236E+01_JPRB, 0.11390E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11034E+01_JPRB, 0.10653E+01_JPRB, 0.10180E+01_JPRB, 0.97589E+00_JPRB, 0.94456E+00_JPRB, 0.92708E+00_JPRB, 0.91073E+00_JPRB ,&
&  0.87522E+00_JPRB, 0.80425E+00_JPRB, 0.66928E+00_JPRB, 0.60829E+00_JPRB, 0.58803E+00_JPRB, 0.62611E+00_JPRB, 0.71058E+00_JPRB ,&
&  0.78702E+00_JPRB, 0.85299E+00_JPRB, 0.98702E+00_JPRB, 0.11956E+01_JPRB, 0.13494E+01_JPRB, 0.14029E+01_JPRB, 0.14688E+01_JPRB ,&
&  0.16800E+01_JPRB, 0.13577E+01_JPRB, 0.12951E+01_JPRB, 0.12016E+01_JPRB, 0.11669E+01_JPRB, 0.11543E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10814E+01_JPRB, 0.10747E+01_JPRB, 0.10916E+01_JPRB, 0.11195E+01_JPRB, 0.11662E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.13493E+01_JPRB, 0.13848E+01_JPRB, 0.13456E+01_JPRB, 0.13332E+01_JPRB, 0.14315E+01_JPRB, 0.15107E+01_JPRB, 0.15135E+01_JPRB ,&
&  0.14252E+01_JPRB, 0.14202E+01_JPRB, 0.17612E+01_JPRB, 0.21136E+01_JPRB, 0.25027E+01_JPRB, 0.27789E+01_JPRB, 0.28509E+01_JPRB ,&
&  0.29598E+01_JPRB, 0.29058E+01_JPRB, 0.29563E+01_JPRB, 0.26884E+01_JPRB, 0.29335E+01_JPRB, 0.28499E+01_JPRB, 0.26814E+01_JPRB ,&
&  0.25213E+01_JPRB, 0.24380E+01_JPRB, 0.22692E+01_JPRB, 0.22483E+01_JPRB, 0.23245E+01_JPRB, 0.23757E+01_JPRB, 0.18930E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.16343E+01_JPRB, 0.13852E+01_JPRB, 0.12209E+01_JPRB, 0.11025E+01_JPRB, 0.11359E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.13230E+01_JPRB, 0.13646E+01_JPRB, 0.15678E+01_JPRB, 0.19489E+01_JPRB, 0.23144E+01_JPRB, 0.25518E+01_JPRB ,&
&  0.23318E+01_JPRB, 0.24399E+01_JPRB, 0.23612E+01_JPRB, 0.22651E+01_JPRB, 0.22510E+01_JPRB, 0.21784E+01_JPRB, 0.20436E+01_JPRB ,&
&  0.18549E+01_JPRB, 0.16748E+01_JPRB, 0.15241E+01_JPRB, 0.13951E+01_JPRB, 0.12909E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.14402E+01_JPRB, 0.13738E+01_JPRB, 0.13262E+01_JPRB, 0.12971E+01_JPRB, 0.12915E+01_JPRB, 0.12956E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.11366E+01_JPRB, 0.10829E+01_JPRB, 0.10355E+01_JPRB, 0.99048E+00_JPRB, 0.95980E+00_JPRB, 0.93733E+00_JPRB ,&
&  0.91591E+00_JPRB, 0.78536E+00_JPRB, 0.63989E+00_JPRB, 0.62831E+00_JPRB, 0.63958E+00_JPRB, 0.61956E+00_JPRB, 0.68391E+00_JPRB ,&
&  0.78066E+00_JPRB, 0.93169E+00_JPRB, 0.11099E+01_JPRB, 0.14202E+01_JPRB, 0.16009E+01_JPRB, 0.17284E+01_JPRB, 0.17912E+01_JPRB ,&
&  0.17246E+01_JPRB, 0.16300E+01_JPRB, 0.15134E+01_JPRB, 0.13818E+01_JPRB, 0.12405E+01_JPRB, 0.11999E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11160E+01_JPRB, 0.11313E+01_JPRB, 0.11895E+01_JPRB, 0.12635E+01_JPRB, 0.13531E+01_JPRB, 0.14725E+01_JPRB ,&
&  0.14968E+01_JPRB, 0.14552E+01_JPRB, 0.13417E+01_JPRB, 0.13595E+01_JPRB, 0.13094E+01_JPRB, 0.13370E+01_JPRB, 0.13363E+01_JPRB ,&
&  0.16526E+01_JPRB, 0.17908E+01_JPRB, 0.18057E+01_JPRB, 0.21906E+01_JPRB, 0.21180E+01_JPRB, 0.23288E+01_JPRB, 0.29302E+01_JPRB ,&
&  0.30658E+01_JPRB, 0.30547E+01_JPRB, 0.31343E+01_JPRB, 0.24967E+01_JPRB, 0.21346E+01_JPRB, 0.29053E+01_JPRB, 0.29252E+01_JPRB ,&
&  0.28918E+01_JPRB, 0.29398E+01_JPRB, 0.28522E+01_JPRB, 0.29665E+01_JPRB, 0.24682E+01_JPRB, 0.20685E+01_JPRB, 0.15564E+01_JPRB ,&
&  0.14906E+01_JPRB, 0.14223E+01_JPRB, 0.15576E+01_JPRB, 0.14394E+01_JPRB, 0.14927E+01_JPRB, 0.13778E+01_JPRB, 0.13388E+01_JPRB ,&
&  0.14722E+01_JPRB, 0.15673E+01_JPRB, 0.17334E+01_JPRB, 0.20631E+01_JPRB, 0.25144E+01_JPRB, 0.24668E+01_JPRB, 0.27976E+01_JPRB ,&
&  0.28961E+01_JPRB, 0.28346E+01_JPRB, 0.25026E+01_JPRB, 0.25272E+01_JPRB, 0.24304E+01_JPRB, 0.23285E+01_JPRB, 0.21992E+01_JPRB ,&
&  0.20358E+01_JPRB, 0.18801E+01_JPRB, 0.17487E+01_JPRB, 0.16292E+01_JPRB, 0.15266E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.16441E+01_JPRB, 0.15816E+01_JPRB, 0.15305E+01_JPRB, 0.14974E+01_JPRB, 0.14757E+01_JPRB, 0.14359E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.11851E+01_JPRB, 0.11183E+01_JPRB, 0.10630E+01_JPRB, 0.10049E+01_JPRB, 0.97761E+00_JPRB, 0.96892E+00_JPRB ,&
&  0.95329E+00_JPRB, 0.88581E+00_JPRB, 0.70192E+00_JPRB, 0.65080E+00_JPRB, 0.68799E+00_JPRB, 0.66119E+00_JPRB, 0.67385E+00_JPRB ,&
&  0.82950E+00_JPRB, 0.10508E+01_JPRB, 0.13049E+01_JPRB, 0.17069E+01_JPRB, 0.18881E+01_JPRB, 0.20829E+01_JPRB, 0.21233E+01_JPRB ,&
&  0.21610E+01_JPRB, 0.18450E+01_JPRB, 0.15812E+01_JPRB, 0.13829E+01_JPRB, 0.12417E+01_JPRB, 0.11649E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11606E+01_JPRB, 0.12351E+01_JPRB, 0.12735E+01_JPRB, 0.13486E+01_JPRB, 0.14119E+01_JPRB, 0.14828E+01_JPRB ,&
&  0.14951E+01_JPRB, 0.14515E+01_JPRB, 0.14180E+01_JPRB, 0.14895E+01_JPRB, 0.14903E+01_JPRB, 0.14278E+01_JPRB, 0.14708E+01_JPRB ,&
&  0.16253E+01_JPRB, 0.16860E+01_JPRB, 0.17935E+01_JPRB, 0.20557E+01_JPRB, 0.24034E+01_JPRB, 0.24722E+01_JPRB, 0.28649E+01_JPRB ,&
&  0.23691E+01_JPRB, 0.21070E+01_JPRB, 0.18468E+01_JPRB, 0.15971E+01_JPRB, 0.19662E+01_JPRB, 0.31574E+01_JPRB, 0.32483E+01_JPRB ,&
&  0.31716E+01_JPRB, 0.33145E+01_JPRB, 0.33439E+01_JPRB, 0.29268E+01_JPRB, 0.24120E+01_JPRB, 0.19612E+01_JPRB, 0.23611E+01_JPRB ,&
&  0.24175E+01_JPRB, 0.26790E+01_JPRB, 0.24037E+01_JPRB, 0.19071E+01_JPRB, 0.16853E+01_JPRB, 0.20019E+01_JPRB, 0.20280E+01_JPRB ,&
&  0.20889E+01_JPRB, 0.22041E+01_JPRB, 0.30026E+01_JPRB, 0.31022E+01_JPRB, 0.30418E+01_JPRB, 0.30927E+01_JPRB, 0.33737E+01_JPRB ,&
&  0.31231E+01_JPRB, 0.30359E+01_JPRB, 0.28893E+01_JPRB, 0.26900E+01_JPRB, 0.25269E+01_JPRB, 0.23719E+01_JPRB, 0.22278E+01_JPRB ,&
&  0.21103E+01_JPRB, 0.19984E+01_JPRB, 0.18786E+01_JPRB, 0.17879E+01_JPRB, 0.17094E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.17747E+01_JPRB, 0.17178E+01_JPRB, 0.16730E+01_JPRB, 0.16235E+01_JPRB, 0.15644E+01_JPRB, 0.14805E+01_JPRB, 0.13795E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.12131E+01_JPRB, 0.11491E+01_JPRB, 0.10954E+01_JPRB, 0.10383E+01_JPRB, 0.10051E+01_JPRB, 0.98981E+00_JPRB ,&
&  0.98375E+00_JPRB, 0.10588E+01_JPRB, 0.82887E+00_JPRB, 0.71678E+00_JPRB, 0.72534E+00_JPRB, 0.73408E+00_JPRB, 0.71234E+00_JPRB ,&
&  0.92612E+00_JPRB, 0.12140E+01_JPRB, 0.14338E+01_JPRB, 0.18682E+01_JPRB, 0.23238E+01_JPRB, 0.22178E+01_JPRB, 0.21615E+01_JPRB ,&
&  0.20929E+01_JPRB, 0.18008E+01_JPRB, 0.15732E+01_JPRB, 0.14205E+01_JPRB, 0.13017E+01_JPRB, 0.12488E+01_JPRB, 0.12326E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.13271E+01_JPRB, 0.13343E+01_JPRB, 0.13593E+01_JPRB, 0.13962E+01_JPRB, 0.14827E+01_JPRB, 0.15587E+01_JPRB ,&
&  0.15207E+01_JPRB, 0.14797E+01_JPRB, 0.15079E+01_JPRB, 0.16285E+01_JPRB, 0.16579E+01_JPRB, 0.15742E+01_JPRB, 0.17625E+01_JPRB ,&
&  0.17364E+01_JPRB, 0.18697E+01_JPRB, 0.18802E+01_JPRB, 0.21503E+01_JPRB, 0.23495E+01_JPRB, 0.23859E+01_JPRB, 0.25806E+01_JPRB ,&
&  0.20222E+01_JPRB, 0.18358E+01_JPRB, 0.19076E+01_JPRB, 0.20011E+01_JPRB, 0.20450E+01_JPRB, 0.24609E+01_JPRB, 0.36490E+01_JPRB ,&
&  0.30112E+01_JPRB, 0.32315E+01_JPRB, 0.32681E+01_JPRB, 0.32140E+01_JPRB, 0.18080E+01_JPRB, 0.15154E+01_JPRB, 0.25528E+01_JPRB ,&
&  0.28781E+01_JPRB, 0.18624E+01_JPRB, 0.13212E+01_JPRB, 0.15319E+01_JPRB, 0.12267E+01_JPRB, 0.16511E+01_JPRB, 0.24734E+01_JPRB ,&
&  0.28620E+01_JPRB, 0.31449E+01_JPRB, 0.42890E+01_JPRB, 0.38588E+01_JPRB, 0.35870E+01_JPRB, 0.42663E+01_JPRB, 0.37331E+01_JPRB ,&
&  0.34397E+01_JPRB, 0.29470E+01_JPRB, 0.29128E+01_JPRB, 0.26043E+01_JPRB, 0.24039E+01_JPRB, 0.22782E+01_JPRB, 0.22069E+01_JPRB ,&
&  0.21625E+01_JPRB, 0.21068E+01_JPRB, 0.20294E+01_JPRB, 0.19379E+01_JPRB, 0.18436E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.18183E+01_JPRB, 0.17752E+01_JPRB, 0.17386E+01_JPRB, 0.16802E+01_JPRB, 0.15989E+01_JPRB, 0.14985E+01_JPRB, 0.13993E+01_JPRB ,&
&  0.13300E+01_JPRB, 0.12528E+01_JPRB, 0.11885E+01_JPRB, 0.11345E+01_JPRB, 0.10794E+01_JPRB, 0.10367E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.98755E+00_JPRB, 0.10003E+01_JPRB, 0.10054E+01_JPRB, 0.90365E+00_JPRB, 0.87958E+00_JPRB, 0.91228E+00_JPRB, 0.93475E+00_JPRB ,&
&  0.11776E+01_JPRB, 0.14413E+01_JPRB, 0.16893E+01_JPRB, 0.20033E+01_JPRB, 0.22303E+01_JPRB, 0.21239E+01_JPRB, 0.21670E+01_JPRB ,&
&  0.20236E+01_JPRB, 0.17988E+01_JPRB, 0.16102E+01_JPRB, 0.14954E+01_JPRB, 0.14291E+01_JPRB, 0.14000E+01_JPRB, 0.13877E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.13986E+01_JPRB, 0.13669E+01_JPRB, 0.13466E+01_JPRB, 0.14184E+01_JPRB, 0.15414E+01_JPRB, 0.15960E+01_JPRB ,&
&  0.15484E+01_JPRB, 0.15440E+01_JPRB, 0.15755E+01_JPRB, 0.16774E+01_JPRB, 0.18254E+01_JPRB, 0.18111E+01_JPRB, 0.16008E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.15957E+01_JPRB, 0.20007E+01_JPRB, 0.20568E+01_JPRB, 0.21465E+01_JPRB, 0.22816E+01_JPRB, 0.23767E+01_JPRB ,&
&  0.23087E+01_JPRB, 0.23041E+01_JPRB, 0.21736E+01_JPRB, 0.23909E+01_JPRB, 0.25261E+01_JPRB, 0.24487E+01_JPRB, 0.33194E+01_JPRB ,&
&  0.33567E+01_JPRB, 0.31187E+01_JPRB, 0.24994E+01_JPRB, 0.19901E+01_JPRB, 0.20633E+01_JPRB, 0.17474E+01_JPRB, 0.12586E+01_JPRB ,&
&  0.12266E+01_JPRB, 0.11704E+01_JPRB, 0.10415E+01_JPRB, 0.10211E+01_JPRB, 0.97656E+00_JPRB, 0.15023E+01_JPRB, 0.25677E+01_JPRB ,&
&  0.39046E+01_JPRB, 0.46460E+01_JPRB, 0.50606E+01_JPRB, 0.46253E+01_JPRB, 0.40609E+01_JPRB, 0.46341E+01_JPRB, 0.40686E+01_JPRB ,&
&  0.31093E+01_JPRB, 0.27817E+01_JPRB, 0.27010E+01_JPRB, 0.24963E+01_JPRB, 0.23921E+01_JPRB, 0.23010E+01_JPRB, 0.22213E+01_JPRB ,&
&  0.21438E+01_JPRB, 0.20823E+01_JPRB, 0.20175E+01_JPRB, 0.19426E+01_JPRB, 0.18769E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.18232E+01_JPRB, 0.17861E+01_JPRB, 0.17379E+01_JPRB, 0.16686E+01_JPRB, 0.15809E+01_JPRB, 0.14940E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.13862E+01_JPRB, 0.13245E+01_JPRB, 0.12552E+01_JPRB, 0.11910E+01_JPRB, 0.11274E+01_JPRB, 0.10725E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10083E+01_JPRB, 0.10880E+01_JPRB, 0.10754E+01_JPRB, 0.11739E+01_JPRB, 0.13246E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.16665E+01_JPRB, 0.18856E+01_JPRB, 0.21775E+01_JPRB, 0.22980E+01_JPRB, 0.24306E+01_JPRB, 0.24298E+01_JPRB, 0.23152E+01_JPRB ,&
&  0.20944E+01_JPRB, 0.19432E+01_JPRB, 0.18047E+01_JPRB, 0.17050E+01_JPRB, 0.16037E+01_JPRB, 0.15233E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.14381E+01_JPRB, 0.13750E+01_JPRB, 0.13750E+01_JPRB, 0.14593E+01_JPRB, 0.15567E+01_JPRB, 0.16131E+01_JPRB ,&
&  0.16292E+01_JPRB, 0.16349E+01_JPRB, 0.16813E+01_JPRB, 0.17740E+01_JPRB, 0.16442E+01_JPRB, 0.14362E+01_JPRB, 0.15762E+01_JPRB ,&
&  0.17071E+01_JPRB, 0.19383E+01_JPRB, 0.19317E+01_JPRB, 0.22412E+01_JPRB, 0.22385E+01_JPRB, 0.21839E+01_JPRB, 0.23804E+01_JPRB ,&
&  0.24692E+01_JPRB, 0.28029E+01_JPRB, 0.23293E+01_JPRB, 0.23549E+01_JPRB, 0.31062E+01_JPRB, 0.28460E+01_JPRB, 0.30053E+01_JPRB ,&
&  0.32956E+01_JPRB, 0.28576E+01_JPRB, 0.25077E+01_JPRB, 0.23341E+01_JPRB, 0.34015E+01_JPRB, 0.35478E+01_JPRB, 0.17126E+01_JPRB ,&
&  0.13985E+01_JPRB, 0.12337E+01_JPRB, 0.11211E+01_JPRB, 0.10083E+01_JPRB, 0.10008E+01_JPRB, 0.16389E+01_JPRB, 0.37405E+01_JPRB ,&
&  0.42719E+01_JPRB, 0.61637E+01_JPRB, 0.62319E+01_JPRB, 0.55965E+01_JPRB, 0.45781E+01_JPRB, 0.45071E+01_JPRB, 0.35535E+01_JPRB ,&
&  0.30909E+01_JPRB, 0.28693E+01_JPRB, 0.26430E+01_JPRB, 0.24520E+01_JPRB, 0.22570E+01_JPRB, 0.21403E+01_JPRB, 0.20840E+01_JPRB ,&
&  0.20346E+01_JPRB, 0.19875E+01_JPRB, 0.19475E+01_JPRB, 0.19041E+01_JPRB, 0.18650E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.17340E+01_JPRB, 0.16854E+01_JPRB, 0.16342E+01_JPRB, 0.15694E+01_JPRB, 0.15105E+01_JPRB, 0.14693E+01_JPRB, 0.14386E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.13756E+01_JPRB, 0.13070E+01_JPRB, 0.12293E+01_JPRB, 0.11562E+01_JPRB, 0.10963E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10407E+01_JPRB, 0.10918E+01_JPRB, 0.12497E+01_JPRB, 0.14732E+01_JPRB, 0.16171E+01_JPRB, 0.19338E+01_JPRB ,&
&  0.23435E+01_JPRB, 0.26185E+01_JPRB, 0.29280E+01_JPRB, 0.29593E+01_JPRB, 0.28992E+01_JPRB, 0.27494E+01_JPRB, 0.25318E+01_JPRB ,&
&  0.23355E+01_JPRB, 0.22086E+01_JPRB, 0.20467E+01_JPRB, 0.18843E+01_JPRB, 0.17360E+01_JPRB, 0.16251E+01_JPRB, 0.15450E+01_JPRB ,&
&  0.15336E+01_JPRB, 0.14623E+01_JPRB, 0.13960E+01_JPRB, 0.14041E+01_JPRB, 0.14820E+01_JPRB, 0.15945E+01_JPRB, 0.16711E+01_JPRB ,&
&  0.16997E+01_JPRB, 0.16958E+01_JPRB, 0.17317E+01_JPRB, 0.18026E+01_JPRB, 0.15170E+01_JPRB, 0.16969E+01_JPRB, 0.17175E+01_JPRB ,&
&  0.17612E+01_JPRB, 0.18489E+01_JPRB, 0.18365E+01_JPRB, 0.21051E+01_JPRB, 0.22114E+01_JPRB, 0.21586E+01_JPRB, 0.22420E+01_JPRB ,&
&  0.25687E+01_JPRB, 0.23819E+01_JPRB, 0.22166E+01_JPRB, 0.25200E+01_JPRB, 0.30313E+01_JPRB, 0.40232E+01_JPRB, 0.27666E+01_JPRB ,&
&  0.24508E+01_JPRB, 0.28588E+01_JPRB, 0.27674E+01_JPRB, 0.26750E+01_JPRB, 0.34923E+01_JPRB, 0.43966E+01_JPRB, 0.37822E+01_JPRB ,&
&  0.16390E+01_JPRB, 0.12026E+01_JPRB, 0.12701E+01_JPRB, 0.13199E+01_JPRB, 0.12827E+01_JPRB, 0.19573E+01_JPRB, 0.61023E+01_JPRB ,&
&  0.54292E+01_JPRB, 0.70144E+01_JPRB, 0.65153E+01_JPRB, 0.55347E+01_JPRB, 0.46385E+01_JPRB, 0.42045E+01_JPRB, 0.36179E+01_JPRB ,&
&  0.30225E+01_JPRB, 0.26696E+01_JPRB, 0.23806E+01_JPRB, 0.21797E+01_JPRB, 0.20648E+01_JPRB, 0.19800E+01_JPRB, 0.19254E+01_JPRB ,&
&  0.18773E+01_JPRB, 0.18391E+01_JPRB, 0.18156E+01_JPRB, 0.18003E+01_JPRB, 0.17764E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.16298E+01_JPRB, 0.15708E+01_JPRB, 0.15127E+01_JPRB, 0.14807E+01_JPRB, 0.14626E+01_JPRB, 0.14382E+01_JPRB, 0.14216E+01_JPRB ,&
&  0.14078E+01_JPRB, 0.13535E+01_JPRB, 0.12947E+01_JPRB, 0.12224E+01_JPRB, 0.11529E+01_JPRB, 0.11023E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10708E+01_JPRB, 0.11074E+01_JPRB, 0.12945E+01_JPRB, 0.16451E+01_JPRB, 0.17351E+01_JPRB, 0.21058E+01_JPRB ,&
&  0.33609E+01_JPRB, 0.38427E+01_JPRB, 0.36219E+01_JPRB, 0.35042E+01_JPRB, 0.34001E+01_JPRB, 0.33246E+01_JPRB, 0.29974E+01_JPRB ,&
&  0.26637E+01_JPRB, 0.23681E+01_JPRB, 0.21031E+01_JPRB, 0.19045E+01_JPRB, 0.17482E+01_JPRB, 0.16405E+01_JPRB, 0.15850E+01_JPRB ,&
&  0.15295E+01_JPRB, 0.14612E+01_JPRB, 0.14069E+01_JPRB, 0.14181E+01_JPRB, 0.15179E+01_JPRB, 0.16256E+01_JPRB, 0.17101E+01_JPRB ,&
&  0.17282E+01_JPRB, 0.17222E+01_JPRB, 0.17252E+01_JPRB, 0.16830E+01_JPRB, 0.16940E+01_JPRB, 0.17243E+01_JPRB, 0.17823E+01_JPRB ,&
&  0.17841E+01_JPRB, 0.18804E+01_JPRB, 0.19906E+01_JPRB, 0.20360E+01_JPRB, 0.20990E+01_JPRB, 0.21564E+01_JPRB, 0.22212E+01_JPRB ,&
&  0.24053E+01_JPRB, 0.23322E+01_JPRB, 0.23397E+01_JPRB, 0.28278E+01_JPRB, 0.31893E+01_JPRB, 0.36222E+01_JPRB, 0.35195E+01_JPRB ,&
&  0.30219E+01_JPRB, 0.24546E+01_JPRB, 0.24668E+01_JPRB, 0.29175E+01_JPRB, 0.33613E+01_JPRB, 0.38177E+01_JPRB, 0.57626E+01_JPRB ,&
&  0.57480E+01_JPRB, 0.38283E+01_JPRB, 0.31941E+01_JPRB, 0.41872E+01_JPRB, 0.35695E+01_JPRB, 0.33289E+01_JPRB, 0.45770E+01_JPRB ,&
&  0.57878E+01_JPRB, 0.70573E+01_JPRB, 0.63858E+01_JPRB, 0.50834E+01_JPRB, 0.45612E+01_JPRB, 0.39904E+01_JPRB, 0.33261E+01_JPRB ,&
&  0.27524E+01_JPRB, 0.24504E+01_JPRB, 0.22436E+01_JPRB, 0.20437E+01_JPRB, 0.18843E+01_JPRB, 0.17989E+01_JPRB, 0.17590E+01_JPRB ,&
&  0.17255E+01_JPRB, 0.17091E+01_JPRB, 0.17040E+01_JPRB, 0.17020E+01_JPRB, 0.16740E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.14972E+01_JPRB, 0.14560E+01_JPRB, 0.14380E+01_JPRB, 0.14042E+01_JPRB, 0.13684E+01_JPRB, 0.13408E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.12800E+01_JPRB, 0.12314E+01_JPRB, 0.11761E+01_JPRB, 0.11163E+01_JPRB, 0.10821E+01_JPRB, 0.10690E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10924E+01_JPRB, 0.11801E+01_JPRB, 0.14198E+01_JPRB, 0.17989E+01_JPRB, 0.24083E+01_JPRB, 0.22335E+01_JPRB ,&
&  0.34112E+01_JPRB, 0.46289E+01_JPRB, 0.43835E+01_JPRB, 0.41409E+01_JPRB, 0.38855E+01_JPRB, 0.37136E+01_JPRB, 0.32032E+01_JPRB ,&
&  0.27322E+01_JPRB, 0.23952E+01_JPRB, 0.20816E+01_JPRB, 0.18626E+01_JPRB, 0.17267E+01_JPRB, 0.16407E+01_JPRB, 0.15592E+01_JPRB ,&
&  0.14996E+01_JPRB, 0.14344E+01_JPRB, 0.13953E+01_JPRB, 0.14209E+01_JPRB, 0.14990E+01_JPRB, 0.16125E+01_JPRB, 0.16869E+01_JPRB ,&
&  0.17162E+01_JPRB, 0.17142E+01_JPRB, 0.17012E+01_JPRB, 0.16126E+01_JPRB, 0.16563E+01_JPRB, 0.17026E+01_JPRB, 0.17583E+01_JPRB ,&
&  0.17065E+01_JPRB, 0.16407E+01_JPRB, 0.18396E+01_JPRB, 0.20219E+01_JPRB, 0.20898E+01_JPRB, 0.20439E+01_JPRB, 0.20673E+01_JPRB ,&
&  0.23566E+01_JPRB, 0.25885E+01_JPRB, 0.30524E+01_JPRB, 0.31768E+01_JPRB, 0.33464E+01_JPRB, 0.35137E+01_JPRB, 0.36334E+01_JPRB ,&
&  0.37031E+01_JPRB, 0.32022E+01_JPRB, 0.30535E+01_JPRB, 0.31316E+01_JPRB, 0.31515E+01_JPRB, 0.35558E+01_JPRB, 0.50496E+01_JPRB ,&
&  0.65605E+01_JPRB, 0.75621E+01_JPRB, 0.78118E+01_JPRB, 0.65713E+01_JPRB, 0.55720E+01_JPRB, 0.48804E+01_JPRB, 0.48362E+01_JPRB ,&
&  0.62909E+01_JPRB, 0.65580E+01_JPRB, 0.57030E+01_JPRB, 0.56684E+01_JPRB, 0.42597E+01_JPRB, 0.37642E+01_JPRB, 0.31562E+01_JPRB ,&
&  0.27078E+01_JPRB, 0.23695E+01_JPRB, 0.21044E+01_JPRB, 0.19093E+01_JPRB, 0.17741E+01_JPRB, 0.16653E+01_JPRB, 0.16007E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.15777E+01_JPRB, 0.15912E+01_JPRB, 0.15851E+01_JPRB, 0.15442E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.14195E+01_JPRB, 0.14011E+01_JPRB, 0.13782E+01_JPRB, 0.13326E+01_JPRB, 0.12987E+01_JPRB, 0.12771E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.12032E+01_JPRB, 0.11621E+01_JPRB, 0.11146E+01_JPRB, 0.10765E+01_JPRB, 0.10521E+01_JPRB, 0.10560E+01_JPRB ,&
&  0.11096E+01_JPRB, 0.12253E+01_JPRB, 0.14361E+01_JPRB, 0.16828E+01_JPRB, 0.20922E+01_JPRB, 0.27168E+01_JPRB, 0.31403E+01_JPRB ,&
&  0.36076E+01_JPRB, 0.56057E+01_JPRB, 0.46750E+01_JPRB, 0.42666E+01_JPRB, 0.39801E+01_JPRB, 0.36961E+01_JPRB, 0.30976E+01_JPRB ,&
&  0.24489E+01_JPRB, 0.21656E+01_JPRB, 0.19801E+01_JPRB, 0.18225E+01_JPRB, 0.17132E+01_JPRB, 0.16000E+01_JPRB, 0.15357E+01_JPRB ,&
&  0.14765E+01_JPRB, 0.14387E+01_JPRB, 0.14081E+01_JPRB, 0.14438E+01_JPRB, 0.15288E+01_JPRB, 0.16101E+01_JPRB, 0.16843E+01_JPRB ,&
&  0.17082E+01_JPRB, 0.16941E+01_JPRB, 0.16316E+01_JPRB, 0.15954E+01_JPRB, 0.15969E+01_JPRB, 0.16519E+01_JPRB, 0.17180E+01_JPRB ,&
&  0.17167E+01_JPRB, 0.16747E+01_JPRB, 0.17930E+01_JPRB, 0.18383E+01_JPRB, 0.18512E+01_JPRB, 0.19874E+01_JPRB, 0.20093E+01_JPRB ,&
&  0.23594E+01_JPRB, 0.23731E+01_JPRB, 0.32197E+01_JPRB, 0.29735E+01_JPRB, 0.30240E+01_JPRB, 0.32358E+01_JPRB, 0.31519E+01_JPRB ,&
&  0.28325E+01_JPRB, 0.28393E+01_JPRB, 0.27812E+01_JPRB, 0.29257E+01_JPRB, 0.32957E+01_JPRB, 0.39746E+01_JPRB, 0.53004E+01_JPRB ,&
&  0.58702E+01_JPRB, 0.57598E+01_JPRB, 0.58340E+01_JPRB, 0.48694E+01_JPRB, 0.46747E+01_JPRB, 0.44594E+01_JPRB, 0.46236E+01_JPRB ,&
&  0.59756E+01_JPRB, 0.59606E+01_JPRB, 0.54518E+01_JPRB, 0.48348E+01_JPRB, 0.36772E+01_JPRB, 0.31759E+01_JPRB, 0.26611E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.20369E+01_JPRB, 0.18755E+01_JPRB, 0.17452E+01_JPRB, 0.16466E+01_JPRB, 0.15479E+01_JPRB, 0.15032E+01_JPRB ,&
&  0.14852E+01_JPRB, 0.14858E+01_JPRB, 0.14896E+01_JPRB, 0.14725E+01_JPRB, 0.14438E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.14056E+01_JPRB, 0.14095E+01_JPRB, 0.14070E+01_JPRB, 0.14260E+01_JPRB, 0.14690E+01_JPRB, 0.15343E+01_JPRB, 0.15664E+01_JPRB ,&
&  0.14205E+01_JPRB, 0.11338E+01_JPRB, 0.11085E+01_JPRB, 0.10787E+01_JPRB, 0.10475E+01_JPRB, 0.10512E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.12444E+01_JPRB, 0.14058E+01_JPRB, 0.15968E+01_JPRB, 0.19208E+01_JPRB, 0.23998E+01_JPRB, 0.31947E+01_JPRB, 0.40933E+01_JPRB ,&
&  0.43751E+01_JPRB, 0.47687E+01_JPRB, 0.46732E+01_JPRB, 0.39665E+01_JPRB, 0.35719E+01_JPRB, 0.33668E+01_JPRB, 0.30949E+01_JPRB ,&
&  0.24793E+01_JPRB, 0.21459E+01_JPRB, 0.20629E+01_JPRB, 0.20344E+01_JPRB, 0.16099E+01_JPRB, 0.15390E+01_JPRB, 0.15032E+01_JPRB ,&
&  0.14843E+01_JPRB, 0.14820E+01_JPRB, 0.14850E+01_JPRB, 0.15360E+01_JPRB, 0.16040E+01_JPRB, 0.16685E+01_JPRB, 0.17171E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.17126E+01_JPRB, 0.16693E+01_JPRB, 0.15722E+01_JPRB, 0.15722E+01_JPRB, 0.16112E+01_JPRB, 0.16617E+01_JPRB ,&
&  0.16965E+01_JPRB, 0.17582E+01_JPRB, 0.17982E+01_JPRB, 0.17964E+01_JPRB, 0.17508E+01_JPRB, 0.19882E+01_JPRB, 0.20935E+01_JPRB ,&
&  0.23220E+01_JPRB, 0.25862E+01_JPRB, 0.34430E+01_JPRB, 0.26066E+01_JPRB, 0.27519E+01_JPRB, 0.29108E+01_JPRB, 0.28472E+01_JPRB ,&
&  0.26114E+01_JPRB, 0.25988E+01_JPRB, 0.25431E+01_JPRB, 0.27088E+01_JPRB, 0.34100E+01_JPRB, 0.42860E+01_JPRB, 0.58833E+01_JPRB ,&
&  0.49963E+01_JPRB, 0.50519E+01_JPRB, 0.43248E+01_JPRB, 0.43654E+01_JPRB, 0.42931E+01_JPRB, 0.41122E+01_JPRB, 0.39888E+01_JPRB ,&
&  0.49127E+01_JPRB, 0.44997E+01_JPRB, 0.39150E+01_JPRB, 0.33339E+01_JPRB, 0.25581E+01_JPRB, 0.21831E+01_JPRB, 0.18466E+01_JPRB ,&
&  0.16915E+01_JPRB, 0.16117E+01_JPRB, 0.15979E+01_JPRB, 0.15439E+01_JPRB, 0.15070E+01_JPRB, 0.14709E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.14442E+01_JPRB, 0.14344E+01_JPRB, 0.14223E+01_JPRB, 0.14107E+01_JPRB, 0.14017E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.14340E+01_JPRB, 0.14183E+01_JPRB, 0.14030E+01_JPRB, 0.13930E+01_JPRB, 0.13837E+01_JPRB, 0.13522E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.11219E+01_JPRB, 0.11018E+01_JPRB, 0.10866E+01_JPRB, 0.10901E+01_JPRB, 0.11361E+01_JPRB, 0.12247E+01_JPRB ,&
&  0.13300E+01_JPRB, 0.15142E+01_JPRB, 0.17675E+01_JPRB, 0.21155E+01_JPRB, 0.25759E+01_JPRB, 0.33157E+01_JPRB, 0.42247E+01_JPRB ,&
&  0.44444E+01_JPRB, 0.33231E+01_JPRB, 0.37271E+01_JPRB, 0.35711E+01_JPRB, 0.33377E+01_JPRB, 0.30782E+01_JPRB, 0.25951E+01_JPRB ,&
&  0.21145E+01_JPRB, 0.19710E+01_JPRB, 0.19111E+01_JPRB, 0.17798E+01_JPRB, 0.15318E+01_JPRB, 0.15358E+01_JPRB, 0.15553E+01_JPRB ,&
&  0.15814E+01_JPRB, 0.16026E+01_JPRB, 0.16045E+01_JPRB, 0.16540E+01_JPRB, 0.16942E+01_JPRB, 0.17518E+01_JPRB, 0.18181E+01_JPRB ,&
&  0.18373E+01_JPRB, 0.18305E+01_JPRB, 0.18041E+01_JPRB, 0.17038E+01_JPRB, 0.16795E+01_JPRB, 0.17222E+01_JPRB, 0.17593E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.17672E+01_JPRB, 0.18328E+01_JPRB, 0.19829E+01_JPRB, 0.21664E+01_JPRB, 0.20543E+01_JPRB, 0.23309E+01_JPRB ,&
&  0.25232E+01_JPRB, 0.28733E+01_JPRB, 0.28171E+01_JPRB, 0.31887E+01_JPRB, 0.22121E+01_JPRB, 0.24421E+01_JPRB, 0.23088E+01_JPRB ,&
&  0.24957E+01_JPRB, 0.24214E+01_JPRB, 0.25641E+01_JPRB, 0.30360E+01_JPRB, 0.39592E+01_JPRB, 0.50741E+01_JPRB, 0.54016E+01_JPRB ,&
&  0.38646E+01_JPRB, 0.35102E+01_JPRB, 0.37331E+01_JPRB, 0.42376E+01_JPRB, 0.42889E+01_JPRB, 0.41467E+01_JPRB, 0.35509E+01_JPRB ,&
&  0.36125E+01_JPRB, 0.31134E+01_JPRB, 0.27441E+01_JPRB, 0.23193E+01_JPRB, 0.16909E+01_JPRB, 0.14802E+01_JPRB, 0.14272E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.14668E+01_JPRB, 0.14763E+01_JPRB, 0.14834E+01_JPRB, 0.14642E+01_JPRB, 0.14412E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.14189E+01_JPRB, 0.14191E+01_JPRB, 0.14173E+01_JPRB, 0.14254E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.13736E+01_JPRB, 0.13436E+01_JPRB, 0.13162E+01_JPRB, 0.12916E+01_JPRB, 0.12753E+01_JPRB, 0.12321E+01_JPRB, 0.11750E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11201E+01_JPRB, 0.11191E+01_JPRB, 0.11321E+01_JPRB, 0.11699E+01_JPRB, 0.12136E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.14295E+01_JPRB, 0.16276E+01_JPRB, 0.18936E+01_JPRB, 0.22195E+01_JPRB, 0.26364E+01_JPRB, 0.31399E+01_JPRB, 0.35750E+01_JPRB ,&
&  0.40241E+01_JPRB, 0.41745E+01_JPRB, 0.34686E+01_JPRB, 0.24884E+01_JPRB, 0.23577E+01_JPRB, 0.23340E+01_JPRB, 0.19568E+01_JPRB ,&
&  0.18553E+01_JPRB, 0.17623E+01_JPRB, 0.16853E+01_JPRB, 0.16346E+01_JPRB, 0.16191E+01_JPRB, 0.16426E+01_JPRB, 0.16801E+01_JPRB ,&
&  0.17079E+01_JPRB, 0.17281E+01_JPRB, 0.17580E+01_JPRB, 0.18247E+01_JPRB, 0.18782E+01_JPRB, 0.19388E+01_JPRB, 0.19879E+01_JPRB ,&
&  0.20127E+01_JPRB, 0.20507E+01_JPRB, 0.20304E+01_JPRB, 0.19781E+01_JPRB, 0.19189E+01_JPRB, 0.19566E+01_JPRB, 0.20086E+01_JPRB ,&
&  0.20554E+01_JPRB, 0.20643E+01_JPRB, 0.21027E+01_JPRB, 0.22993E+01_JPRB, 0.24680E+01_JPRB, 0.22959E+01_JPRB, 0.24024E+01_JPRB ,&
&  0.27656E+01_JPRB, 0.30287E+01_JPRB, 0.24054E+01_JPRB, 0.28128E+01_JPRB, 0.17809E+01_JPRB, 0.22203E+01_JPRB, 0.24802E+01_JPRB ,&
&  0.24639E+01_JPRB, 0.26192E+01_JPRB, 0.28614E+01_JPRB, 0.34770E+01_JPRB, 0.43415E+01_JPRB, 0.49681E+01_JPRB, 0.43820E+01_JPRB ,&
&  0.31079E+01_JPRB, 0.30303E+01_JPRB, 0.33024E+01_JPRB, 0.35864E+01_JPRB, 0.36581E+01_JPRB, 0.33945E+01_JPRB, 0.25257E+01_JPRB ,&
&  0.19444E+01_JPRB, 0.22296E+01_JPRB, 0.21859E+01_JPRB, 0.20538E+01_JPRB, 0.14563E+01_JPRB, 0.12644E+01_JPRB, 0.13254E+01_JPRB ,&
&  0.14033E+01_JPRB, 0.14604E+01_JPRB, 0.14885E+01_JPRB, 0.14834E+01_JPRB, 0.14581E+01_JPRB, 0.14350E+01_JPRB, 0.14291E+01_JPRB ,&
&  0.14269E+01_JPRB, 0.14158E+01_JPRB, 0.14116E+01_JPRB, 0.14119E+01_JPRB, 0.14005E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.12545E+01_JPRB, 0.12142E+01_JPRB, 0.11903E+01_JPRB, 0.11743E+01_JPRB, 0.11712E+01_JPRB, 0.11448E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.10703E+01_JPRB, 0.10887E+01_JPRB, 0.11333E+01_JPRB, 0.11936E+01_JPRB, 0.12542E+01_JPRB, 0.13681E+01_JPRB ,&
&  0.15162E+01_JPRB, 0.17273E+01_JPRB, 0.19961E+01_JPRB, 0.23893E+01_JPRB, 0.27229E+01_JPRB, 0.30485E+01_JPRB, 0.31640E+01_JPRB ,&
&  0.33303E+01_JPRB, 0.33727E+01_JPRB, 0.31508E+01_JPRB, 0.29222E+01_JPRB, 0.23065E+01_JPRB, 0.22760E+01_JPRB, 0.23845E+01_JPRB ,&
&  0.24785E+01_JPRB, 0.20561E+01_JPRB, 0.17588E+01_JPRB, 0.17377E+01_JPRB, 0.17326E+01_JPRB, 0.17745E+01_JPRB, 0.18244E+01_JPRB ,&
&  0.18667E+01_JPRB, 0.19074E+01_JPRB, 0.19459E+01_JPRB, 0.20093E+01_JPRB, 0.20770E+01_JPRB, 0.21092E+01_JPRB, 0.21476E+01_JPRB ,&
&  0.21603E+01_JPRB, 0.21861E+01_JPRB, 0.22014E+01_JPRB, 0.21882E+01_JPRB, 0.22278E+01_JPRB, 0.22971E+01_JPRB, 0.24183E+01_JPRB ,&
&  0.25433E+01_JPRB, 0.25094E+01_JPRB, 0.26689E+01_JPRB, 0.29179E+01_JPRB, 0.29515E+01_JPRB, 0.28670E+01_JPRB, 0.28879E+01_JPRB ,&
&  0.31649E+01_JPRB, 0.34356E+01_JPRB, 0.20797E+01_JPRB, 0.23272E+01_JPRB, 0.27473E+01_JPRB, 0.24790E+01_JPRB, 0.24045E+01_JPRB ,&
&  0.25813E+01_JPRB, 0.28306E+01_JPRB, 0.32043E+01_JPRB, 0.36553E+01_JPRB, 0.43756E+01_JPRB, 0.56360E+01_JPRB, 0.38786E+01_JPRB ,&
&  0.25806E+01_JPRB, 0.24808E+01_JPRB, 0.26525E+01_JPRB, 0.27936E+01_JPRB, 0.28022E+01_JPRB, 0.24250E+01_JPRB, 0.17095E+01_JPRB ,&
&  0.14845E+01_JPRB, 0.19857E+01_JPRB, 0.20591E+01_JPRB, 0.20590E+01_JPRB, 0.17494E+01_JPRB, 0.11996E+01_JPRB, 0.13254E+01_JPRB ,&
&  0.13928E+01_JPRB, 0.14362E+01_JPRB, 0.14258E+01_JPRB, 0.14237E+01_JPRB, 0.14209E+01_JPRB, 0.14114E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.14058E+01_JPRB, 0.14016E+01_JPRB, 0.13985E+01_JPRB, 0.13631E+01_JPRB, 0.13017E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.11216E+01_JPRB, 0.10910E+01_JPRB, 0.10748E+01_JPRB, 0.10512E+01_JPRB, 0.10359E+01_JPRB, 0.10147E+01_JPRB, 0.10043E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.10133E+01_JPRB, 0.10422E+01_JPRB, 0.10864E+01_JPRB, 0.11492E+01_JPRB, 0.12357E+01_JPRB, 0.13868E+01_JPRB ,&
&  0.15416E+01_JPRB, 0.17793E+01_JPRB, 0.20723E+01_JPRB, 0.24174E+01_JPRB, 0.26959E+01_JPRB, 0.28143E+01_JPRB, 0.29753E+01_JPRB ,&
&  0.30872E+01_JPRB, 0.31221E+01_JPRB, 0.30684E+01_JPRB, 0.29884E+01_JPRB, 0.28758E+01_JPRB, 0.28954E+01_JPRB, 0.30822E+01_JPRB ,&
&  0.28424E+01_JPRB, 0.21402E+01_JPRB, 0.19300E+01_JPRB, 0.18405E+01_JPRB, 0.18574E+01_JPRB, 0.18942E+01_JPRB, 0.19550E+01_JPRB ,&
&  0.20028E+01_JPRB, 0.20362E+01_JPRB, 0.20703E+01_JPRB, 0.21140E+01_JPRB, 0.22006E+01_JPRB, 0.22493E+01_JPRB, 0.23110E+01_JPRB ,&
&  0.23653E+01_JPRB, 0.24252E+01_JPRB, 0.25329E+01_JPRB, 0.24097E+01_JPRB, 0.26083E+01_JPRB, 0.27620E+01_JPRB, 0.29051E+01_JPRB ,&
&  0.29513E+01_JPRB, 0.29286E+01_JPRB, 0.31534E+01_JPRB, 0.33178E+01_JPRB, 0.34560E+01_JPRB, 0.35164E+01_JPRB, 0.34637E+01_JPRB ,&
&  0.36156E+01_JPRB, 0.34125E+01_JPRB, 0.20548E+01_JPRB, 0.19085E+01_JPRB, 0.21213E+01_JPRB, 0.20132E+01_JPRB, 0.24586E+01_JPRB ,&
&  0.26393E+01_JPRB, 0.29189E+01_JPRB, 0.32284E+01_JPRB, 0.36339E+01_JPRB, 0.41334E+01_JPRB, 0.51539E+01_JPRB, 0.34718E+01_JPRB ,&
&  0.19741E+01_JPRB, 0.18679E+01_JPRB, 0.19199E+01_JPRB, 0.19812E+01_JPRB, 0.20514E+01_JPRB, 0.18723E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.16066E+01_JPRB, 0.17042E+01_JPRB, 0.17588E+01_JPRB, 0.15686E+01_JPRB, 0.12520E+01_JPRB, 0.11256E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.12966E+01_JPRB, 0.13002E+01_JPRB, 0.13147E+01_JPRB, 0.13124E+01_JPRB, 0.13075E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.12815E+01_JPRB, 0.12552E+01_JPRB, 0.12008E+01_JPRB, 0.11535E+01_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.94561E+00_JPRB, 0.94592E+00_JPRB, 0.92966E+00_JPRB, 0.90878E+00_JPRB, 0.89783E+00_JPRB, 0.87726E+00_JPRB, 0.90340E+00_JPRB ,&
&  0.90566E+00_JPRB, 0.93612E+00_JPRB, 0.96855E+00_JPRB, 0.10054E+01_JPRB, 0.10682E+01_JPRB, 0.11363E+01_JPRB, 0.12338E+01_JPRB ,&
&  0.13983E+01_JPRB, 0.16555E+01_JPRB, 0.19887E+01_JPRB, 0.24055E+01_JPRB, 0.25821E+01_JPRB, 0.27082E+01_JPRB, 0.28814E+01_JPRB ,&
&  0.30867E+01_JPRB, 0.32591E+01_JPRB, 0.33642E+01_JPRB, 0.34993E+01_JPRB, 0.36499E+01_JPRB, 0.36305E+01_JPRB, 0.35621E+01_JPRB ,&
&  0.29020E+01_JPRB, 0.19726E+01_JPRB, 0.19974E+01_JPRB, 0.18400E+01_JPRB, 0.19106E+01_JPRB, 0.19909E+01_JPRB, 0.20448E+01_JPRB ,&
&  0.20972E+01_JPRB, 0.21310E+01_JPRB, 0.21822E+01_JPRB, 0.22481E+01_JPRB, 0.23447E+01_JPRB, 0.24397E+01_JPRB, 0.25405E+01_JPRB ,&
&  0.26189E+01_JPRB, 0.26983E+01_JPRB, 0.27551E+01_JPRB, 0.27018E+01_JPRB, 0.28445E+01_JPRB, 0.30696E+01_JPRB, 0.32685E+01_JPRB ,&
&  0.34287E+01_JPRB, 0.34184E+01_JPRB, 0.30977E+01_JPRB, 0.32262E+01_JPRB, 0.34703E+01_JPRB, 0.33601E+01_JPRB, 0.35186E+01_JPRB ,&
&  0.39018E+01_JPRB, 0.35523E+01_JPRB, 0.19532E+01_JPRB, 0.17944E+01_JPRB, 0.20094E+01_JPRB, 0.21943E+01_JPRB, 0.23717E+01_JPRB ,&
&  0.25296E+01_JPRB, 0.26789E+01_JPRB, 0.28817E+01_JPRB, 0.31183E+01_JPRB, 0.32473E+01_JPRB, 0.33922E+01_JPRB, 0.27562E+01_JPRB ,&
&  0.16112E+01_JPRB, 0.15437E+01_JPRB, 0.16448E+01_JPRB, 0.17674E+01_JPRB, 0.20599E+01_JPRB, 0.20105E+01_JPRB, 0.15889E+01_JPRB ,&
&  0.16951E+01_JPRB, 0.20040E+01_JPRB, 0.24128E+01_JPRB, 0.19921E+01_JPRB, 0.12138E+01_JPRB, 0.10309E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10811E+01_JPRB, 0.11067E+01_JPRB, 0.11256E+01_JPRB, 0.11495E+01_JPRB, 0.11356E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.10694E+01_JPRB, 0.10388E+01_JPRB, 0.99569E+00_JPRB, 0.96354E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.79277E+00_JPRB, 0.75467E+00_JPRB, 0.71128E+00_JPRB, 0.70483E+00_JPRB, 0.70753E+00_JPRB, 0.69253E+00_JPRB, 0.73343E+00_JPRB ,&
&  0.71021E+00_JPRB, 0.74613E+00_JPRB, 0.76788E+00_JPRB, 0.77791E+00_JPRB, 0.83742E+00_JPRB, 0.89702E+00_JPRB, 0.97221E+00_JPRB ,&
&  0.10746E+01_JPRB, 0.12332E+01_JPRB, 0.14366E+01_JPRB, 0.16946E+01_JPRB, 0.20115E+01_JPRB, 0.21827E+01_JPRB, 0.23518E+01_JPRB ,&
&  0.25820E+01_JPRB, 0.28831E+01_JPRB, 0.33038E+01_JPRB, 0.37889E+01_JPRB, 0.43446E+01_JPRB, 0.41854E+01_JPRB, 0.36065E+01_JPRB ,&
&  0.26040E+01_JPRB, 0.19303E+01_JPRB, 0.17708E+01_JPRB, 0.16451E+01_JPRB, 0.17421E+01_JPRB, 0.19410E+01_JPRB, 0.20281E+01_JPRB ,&
&  0.20909E+01_JPRB, 0.21614E+01_JPRB, 0.22378E+01_JPRB, 0.23262E+01_JPRB, 0.24306E+01_JPRB, 0.25502E+01_JPRB, 0.26209E+01_JPRB ,&
&  0.26657E+01_JPRB, 0.27132E+01_JPRB, 0.27548E+01_JPRB, 0.28988E+01_JPRB, 0.31090E+01_JPRB, 0.33726E+01_JPRB, 0.34271E+01_JPRB ,&
&  0.34387E+01_JPRB, 0.33514E+01_JPRB, 0.29373E+01_JPRB, 0.29150E+01_JPRB, 0.30237E+01_JPRB, 0.30890E+01_JPRB, 0.31954E+01_JPRB ,&
&  0.35391E+01_JPRB, 0.29646E+01_JPRB, 0.20016E+01_JPRB, 0.20537E+01_JPRB, 0.21774E+01_JPRB, 0.23403E+01_JPRB, 0.21828E+01_JPRB ,&
&  0.22322E+01_JPRB, 0.22882E+01_JPRB, 0.23969E+01_JPRB, 0.24192E+01_JPRB, 0.24419E+01_JPRB, 0.23747E+01_JPRB, 0.20816E+01_JPRB ,&
&  0.16286E+01_JPRB, 0.15189E+01_JPRB, 0.14992E+01_JPRB, 0.15839E+01_JPRB, 0.20325E+01_JPRB, 0.20240E+01_JPRB, 0.18207E+01_JPRB ,&
&  0.16414E+01_JPRB, 0.18426E+01_JPRB, 0.16154E+01_JPRB, 0.19178E+01_JPRB, 0.10507E+01_JPRB, 0.88326E+00_JPRB, 0.85989E+00_JPRB ,&
&  0.88965E+00_JPRB, 0.89840E+00_JPRB, 0.91759E+00_JPRB, 0.90670E+00_JPRB, 0.92150E+00_JPRB, 0.95765E+00_JPRB, 0.96593E+00_JPRB ,&
&  0.91659E+00_JPRB, 0.90053E+00_JPRB, 0.86217E+00_JPRB, 0.81943E+00_JPRB, 0.80772E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.65815E+00_JPRB, 0.64655E+00_JPRB, 0.64066E+00_JPRB, 0.63450E+00_JPRB, 0.64505E+00_JPRB, 0.65094E+00_JPRB, 0.65837E+00_JPRB ,&
&  0.66461E+00_JPRB, 0.68598E+00_JPRB, 0.70480E+00_JPRB, 0.72950E+00_JPRB, 0.75526E+00_JPRB, 0.80263E+00_JPRB, 0.85561E+00_JPRB ,&
&  0.91857E+00_JPRB, 0.98894E+00_JPRB, 0.10682E+01_JPRB, 0.11837E+01_JPRB, 0.13479E+01_JPRB, 0.14729E+01_JPRB, 0.16100E+01_JPRB ,&
&  0.17903E+01_JPRB, 0.20587E+01_JPRB, 0.23692E+01_JPRB, 0.27869E+01_JPRB, 0.32715E+01_JPRB, 0.34300E+01_JPRB, 0.32728E+01_JPRB ,&
&  0.23379E+01_JPRB, 0.20304E+01_JPRB, 0.18212E+01_JPRB, 0.16589E+01_JPRB, 0.16835E+01_JPRB, 0.16412E+01_JPRB, 0.16702E+01_JPRB ,&
&  0.17281E+01_JPRB, 0.17402E+01_JPRB, 0.17366E+01_JPRB, 0.17644E+01_JPRB, 0.17872E+01_JPRB, 0.18102E+01_JPRB, 0.17979E+01_JPRB ,&
&  0.17961E+01_JPRB, 0.18672E+01_JPRB, 0.21580E+01_JPRB, 0.25323E+01_JPRB, 0.28047E+01_JPRB, 0.29252E+01_JPRB, 0.27803E+01_JPRB ,&
&  0.25749E+01_JPRB, 0.24349E+01_JPRB, 0.22946E+01_JPRB, 0.22750E+01_JPRB, 0.23728E+01_JPRB, 0.24418E+01_JPRB, 0.26572E+01_JPRB ,&
&  0.25712E+01_JPRB, 0.22573E+01_JPRB, 0.21990E+01_JPRB, 0.21923E+01_JPRB, 0.22241E+01_JPRB, 0.22027E+01_JPRB, 0.20667E+01_JPRB ,&
&  0.21028E+01_JPRB, 0.20333E+01_JPRB, 0.20283E+01_JPRB, 0.19271E+01_JPRB, 0.17775E+01_JPRB, 0.16262E+01_JPRB, 0.15465E+01_JPRB ,&
&  0.14226E+01_JPRB, 0.13420E+01_JPRB, 0.13762E+01_JPRB, 0.15996E+01_JPRB, 0.20076E+01_JPRB, 0.32893E+01_JPRB, 0.28505E+01_JPRB ,&
&  0.16177E+01_JPRB, 0.13702E+01_JPRB, 0.91880E+00_JPRB, 0.11936E+01_JPRB, 0.83805E+00_JPRB, 0.75808E+00_JPRB, 0.75012E+00_JPRB ,&
&  0.75535E+00_JPRB, 0.74967E+00_JPRB, 0.72954E+00_JPRB, 0.72556E+00_JPRB, 0.73738E+00_JPRB, 0.74894E+00_JPRB, 0.74104E+00_JPRB ,&
&  0.70281E+00_JPRB, 0.68637E+00_JPRB, 0.71771E+00_JPRB, 0.67049E+00_JPRB, 0.65939E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.60048E+00_JPRB, 0.60096E+00_JPRB, 0.61769E+00_JPRB, 0.63369E+00_JPRB, 0.64478E+00_JPRB, 0.65656E+00_JPRB, 0.67364E+00_JPRB ,&
&  0.68766E+00_JPRB, 0.70436E+00_JPRB, 0.72190E+00_JPRB, 0.74864E+00_JPRB, 0.78458E+00_JPRB, 0.83752E+00_JPRB, 0.89835E+00_JPRB ,&
&  0.97353E+00_JPRB, 0.10550E+01_JPRB, 0.11476E+01_JPRB, 0.12489E+01_JPRB, 0.13740E+01_JPRB, 0.15464E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.19243E+01_JPRB, 0.21051E+01_JPRB, 0.23341E+01_JPRB, 0.25486E+01_JPRB, 0.26138E+01_JPRB, 0.25955E+01_JPRB, 0.21917E+01_JPRB ,&
&  0.24738E+01_JPRB, 0.20401E+01_JPRB, 0.17792E+01_JPRB, 0.15988E+01_JPRB, 0.14827E+01_JPRB, 0.13841E+01_JPRB, 0.13091E+01_JPRB ,&
&  0.12836E+01_JPRB, 0.12158E+01_JPRB, 0.11489E+01_JPRB, 0.11065E+01_JPRB, 0.10711E+01_JPRB, 0.10851E+01_JPRB, 0.10715E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.13999E+01_JPRB, 0.16982E+01_JPRB, 0.20687E+01_JPRB, 0.22783E+01_JPRB, 0.22492E+01_JPRB, 0.18959E+01_JPRB ,&
&  0.16865E+01_JPRB, 0.15946E+01_JPRB, 0.15762E+01_JPRB, 0.16748E+01_JPRB, 0.17217E+01_JPRB, 0.18315E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.19069E+01_JPRB, 0.17873E+01_JPRB, 0.18731E+01_JPRB, 0.23267E+01_JPRB, 0.22833E+01_JPRB, 0.21125E+01_JPRB, 0.20402E+01_JPRB ,&
&  0.19376E+01_JPRB, 0.18702E+01_JPRB, 0.18322E+01_JPRB, 0.17236E+01_JPRB, 0.15642E+01_JPRB, 0.13970E+01_JPRB, 0.13470E+01_JPRB ,&
&  0.12764E+01_JPRB, 0.11945E+01_JPRB, 0.11822E+01_JPRB, 0.13376E+01_JPRB, 0.17566E+01_JPRB, 0.22484E+01_JPRB, 0.31835E+01_JPRB ,&
&  0.16948E+01_JPRB, 0.98232E+00_JPRB, 0.95523E+00_JPRB, 0.89927E+00_JPRB, 0.77710E+00_JPRB, 0.68809E+00_JPRB, 0.73179E+00_JPRB ,&
&  0.79500E+00_JPRB, 0.86182E+00_JPRB, 0.85211E+00_JPRB, 0.74167E+00_JPRB, 0.69940E+00_JPRB, 0.68172E+00_JPRB, 0.65821E+00_JPRB ,&
&  0.63833E+00_JPRB, 0.60105E+00_JPRB, 0.61723E+00_JPRB, 0.60072E+00_JPRB, 0.60109E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.55955E+00_JPRB, 0.55876E+00_JPRB, 0.56886E+00_JPRB, 0.57908E+00_JPRB, 0.58363E+00_JPRB, 0.58916E+00_JPRB, 0.59520E+00_JPRB ,&
&  0.60214E+00_JPRB, 0.60558E+00_JPRB, 0.61085E+00_JPRB, 0.62115E+00_JPRB, 0.64163E+00_JPRB, 0.68573E+00_JPRB, 0.73916E+00_JPRB ,&
&  0.78932E+00_JPRB, 0.83990E+00_JPRB, 0.89377E+00_JPRB, 0.94218E+00_JPRB, 0.99296E+00_JPRB, 0.10296E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.13538E+01_JPRB, 0.14918E+01_JPRB, 0.16051E+01_JPRB, 0.17268E+01_JPRB, 0.17834E+01_JPRB, 0.15521E+01_JPRB ,&
&  0.20586E+01_JPRB, 0.18729E+01_JPRB, 0.16720E+01_JPRB, 0.14989E+01_JPRB, 0.13812E+01_JPRB, 0.12758E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11846E+01_JPRB, 0.11286E+01_JPRB, 0.12044E+01_JPRB, 0.97314E+00_JPRB, 0.99467E+00_JPRB, 0.11669E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.14588E+01_JPRB, 0.16400E+01_JPRB, 0.18280E+01_JPRB, 0.19371E+01_JPRB, 0.19014E+01_JPRB, 0.17088E+01_JPRB ,&
&  0.15956E+01_JPRB, 0.15552E+01_JPRB, 0.15015E+01_JPRB, 0.16277E+01_JPRB, 0.16037E+01_JPRB, 0.15465E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.14281E+01_JPRB, 0.14916E+01_JPRB, 0.17418E+01_JPRB, 0.22362E+01_JPRB, 0.21730E+01_JPRB, 0.20914E+01_JPRB, 0.20023E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.17981E+01_JPRB, 0.17263E+01_JPRB, 0.16061E+01_JPRB, 0.14608E+01_JPRB, 0.13160E+01_JPRB, 0.12275E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.10965E+01_JPRB, 0.10550E+01_JPRB, 0.10841E+01_JPRB, 0.12460E+01_JPRB, 0.15123E+01_JPRB, 0.26516E+01_JPRB ,&
&  0.16545E+01_JPRB, 0.12873E+01_JPRB, 0.13882E+01_JPRB, 0.56577E+00_JPRB, 0.63980E+00_JPRB, 0.64474E+00_JPRB, 0.74876E+00_JPRB ,&
&  0.90138E+00_JPRB, 0.14089E+01_JPRB, 0.21034E+01_JPRB, 0.18772E+01_JPRB, 0.14433E+01_JPRB, 0.94800E+00_JPRB, 0.59433E+00_JPRB ,&
&  0.57490E+00_JPRB, 0.57005E+00_JPRB, 0.57538E+00_JPRB, 0.56927E+00_JPRB, 0.56713E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.50358E+00_JPRB, 0.49486E+00_JPRB, 0.50489E+00_JPRB, 0.51657E+00_JPRB, 0.51422E+00_JPRB, 0.51544E+00_JPRB, 0.51116E+00_JPRB ,&
&  0.50255E+00_JPRB, 0.49635E+00_JPRB, 0.52105E+00_JPRB, 0.56785E+00_JPRB, 0.63501E+00_JPRB, 0.72388E+00_JPRB, 0.81646E+00_JPRB ,&
&  0.88905E+00_JPRB, 0.97865E+00_JPRB, 0.10651E+01_JPRB, 0.11390E+01_JPRB, 0.11689E+01_JPRB, 0.11939E+01_JPRB, 0.11963E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.12570E+01_JPRB, 0.13625E+01_JPRB, 0.14784E+01_JPRB, 0.16616E+01_JPRB, 0.19684E+01_JPRB, 0.12383E+01_JPRB ,&
&  0.16755E+01_JPRB, 0.16397E+01_JPRB, 0.15674E+01_JPRB, 0.14422E+01_JPRB, 0.13814E+01_JPRB, 0.12868E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.12685E+01_JPRB, 0.13659E+01_JPRB, 0.13471E+01_JPRB, 0.13947E+01_JPRB, 0.14166E+01_JPRB, 0.14091E+01_JPRB, 0.16086E+01_JPRB ,&
&  0.17681E+01_JPRB, 0.19776E+01_JPRB, 0.21961E+01_JPRB, 0.23471E+01_JPRB, 0.24727E+01_JPRB, 0.25220E+01_JPRB, 0.25053E+01_JPRB ,&
&  0.23455E+01_JPRB, 0.22351E+01_JPRB, 0.22943E+01_JPRB, 0.24628E+01_JPRB, 0.21949E+01_JPRB, 0.20765E+01_JPRB, 0.17897E+01_JPRB ,&
&  0.14001E+01_JPRB, 0.13298E+01_JPRB, 0.15406E+01_JPRB, 0.19715E+01_JPRB, 0.20442E+01_JPRB, 0.19629E+01_JPRB, 0.17987E+01_JPRB ,&
&  0.16822E+01_JPRB, 0.15714E+01_JPRB, 0.14852E+01_JPRB, 0.13847E+01_JPRB, 0.12806E+01_JPRB, 0.11796E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.98596E+00_JPRB, 0.93246E+00_JPRB, 0.96485E+00_JPRB, 0.10899E+01_JPRB, 0.14391E+01_JPRB, 0.16069E+01_JPRB ,&
&  0.17477E+01_JPRB, 0.12005E+01_JPRB, 0.98253E+00_JPRB, 0.90200E+00_JPRB, 0.63683E+00_JPRB, 0.63611E+00_JPRB, 0.79453E+00_JPRB ,&
&  0.12138E+01_JPRB, 0.17391E+01_JPRB, 0.28692E+01_JPRB, 0.50916E+01_JPRB, 0.28289E+01_JPRB, 0.21937E+01_JPRB, 0.57191E+00_JPRB ,&
&  0.55071E+00_JPRB, 0.55067E+00_JPRB, 0.54790E+00_JPRB, 0.53206E+00_JPRB, 0.52724E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.48323E+00_JPRB, 0.47554E+00_JPRB, 0.48588E+00_JPRB, 0.48471E+00_JPRB, 0.48898E+00_JPRB, 0.49708E+00_JPRB, 0.50185E+00_JPRB ,&
&  0.50690E+00_JPRB, 0.52526E+00_JPRB, 0.62366E+00_JPRB, 0.76297E+00_JPRB, 0.90073E+00_JPRB, 0.10109E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.12071E+01_JPRB, 0.12457E+01_JPRB, 0.12793E+01_JPRB, 0.13491E+01_JPRB, 0.14458E+01_JPRB, 0.15297E+01_JPRB ,&
&  0.16144E+01_JPRB, 0.17029E+01_JPRB, 0.18063E+01_JPRB, 0.19630E+01_JPRB, 0.21136E+01_JPRB, 0.21065E+01_JPRB, 0.12679E+01_JPRB ,&
&  0.15228E+01_JPRB, 0.15786E+01_JPRB, 0.16056E+01_JPRB, 0.15140E+01_JPRB, 0.14164E+01_JPRB, 0.13814E+01_JPRB, 0.14947E+01_JPRB ,&
&  0.16097E+01_JPRB, 0.16584E+01_JPRB, 0.16050E+01_JPRB, 0.16143E+01_JPRB, 0.20708E+01_JPRB, 0.20447E+01_JPRB, 0.23910E+01_JPRB ,&
&  0.26279E+01_JPRB, 0.28833E+01_JPRB, 0.30560E+01_JPRB, 0.32508E+01_JPRB, 0.34146E+01_JPRB, 0.35287E+01_JPRB, 0.36923E+01_JPRB ,&
&  0.37449E+01_JPRB, 0.38175E+01_JPRB, 0.38164E+01_JPRB, 0.37092E+01_JPRB, 0.30591E+01_JPRB, 0.32398E+01_JPRB, 0.22698E+01_JPRB ,&
&  0.16682E+01_JPRB, 0.14796E+01_JPRB, 0.16474E+01_JPRB, 0.17590E+01_JPRB, 0.17273E+01_JPRB, 0.15677E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12679E+01_JPRB, 0.12534E+01_JPRB, 0.11939E+01_JPRB, 0.11531E+01_JPRB, 0.10759E+01_JPRB, 0.10160E+01_JPRB ,&
&  0.98307E+00_JPRB, 0.93308E+00_JPRB, 0.87926E+00_JPRB, 0.92084E+00_JPRB, 0.10900E+01_JPRB, 0.15468E+01_JPRB, 0.22931E+01_JPRB ,&
&  0.24485E+01_JPRB, 0.12856E+01_JPRB, 0.10380E+01_JPRB, 0.85994E+00_JPRB, 0.76639E+00_JPRB, 0.76108E+00_JPRB, 0.82785E+00_JPRB ,&
&  0.10497E+01_JPRB, 0.18484E+01_JPRB, 0.27092E+01_JPRB, 0.20073E+01_JPRB, 0.22578E+01_JPRB, 0.15645E+01_JPRB, 0.66764E+00_JPRB ,&
&  0.53534E+00_JPRB, 0.51664E+00_JPRB, 0.52075E+00_JPRB, 0.51203E+00_JPRB, 0.49928E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.53067E+00_JPRB, 0.51929E+00_JPRB, 0.51816E+00_JPRB, 0.52904E+00_JPRB, 0.53611E+00_JPRB, 0.56229E+00_JPRB, 0.59450E+00_JPRB ,&
&  0.61330E+00_JPRB, 0.69063E+00_JPRB, 0.86402E+00_JPRB, 0.10166E+01_JPRB, 0.10923E+01_JPRB, 0.11508E+01_JPRB, 0.11357E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.11666E+01_JPRB, 0.11634E+01_JPRB, 0.12103E+01_JPRB, 0.12911E+01_JPRB, 0.14028E+01_JPRB, 0.15483E+01_JPRB ,&
&  0.17126E+01_JPRB, 0.18755E+01_JPRB, 0.20271E+01_JPRB, 0.21677E+01_JPRB, 0.23591E+01_JPRB, 0.23956E+01_JPRB, 0.23542E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.16258E+01_JPRB, 0.16468E+01_JPRB, 0.15765E+01_JPRB, 0.14781E+01_JPRB, 0.15075E+01_JPRB, 0.17432E+01_JPRB ,&
&  0.19374E+01_JPRB, 0.19496E+01_JPRB, 0.22578E+01_JPRB, 0.20255E+01_JPRB, 0.24183E+01_JPRB, 0.26422E+01_JPRB, 0.29038E+01_JPRB ,&
&  0.31496E+01_JPRB, 0.33268E+01_JPRB, 0.35157E+01_JPRB, 0.36982E+01_JPRB, 0.38802E+01_JPRB, 0.40401E+01_JPRB, 0.42344E+01_JPRB ,&
&  0.44613E+01_JPRB, 0.47734E+01_JPRB, 0.50935E+01_JPRB, 0.48959E+01_JPRB, 0.41794E+01_JPRB, 0.39247E+01_JPRB, 0.31560E+01_JPRB ,&
&  0.21841E+01_JPRB, 0.14587E+01_JPRB, 0.14802E+01_JPRB, 0.15435E+01_JPRB, 0.14400E+01_JPRB, 0.13140E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.10894E+01_JPRB, 0.10548E+01_JPRB, 0.10840E+01_JPRB, 0.11068E+01_JPRB, 0.10740E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.96875E+00_JPRB, 0.92470E+00_JPRB, 0.91605E+00_JPRB, 0.99217E+00_JPRB, 0.11562E+01_JPRB, 0.12980E+01_JPRB ,&
&  0.13245E+01_JPRB, 0.11390E+01_JPRB, 0.99618E+00_JPRB, 0.95909E+00_JPRB, 0.90732E+00_JPRB, 0.87175E+00_JPRB, 0.84612E+00_JPRB ,&
&  0.88608E+00_JPRB, 0.10572E+01_JPRB, 0.14636E+01_JPRB, 0.18533E+01_JPRB, 0.14557E+01_JPRB, 0.11530E+01_JPRB, 0.67797E+00_JPRB ,&
&  0.55270E+00_JPRB, 0.53360E+00_JPRB, 0.55049E+00_JPRB, 0.55401E+00_JPRB, 0.55408E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.64048E+00_JPRB, 0.62631E+00_JPRB, 0.60709E+00_JPRB, 0.62327E+00_JPRB, 0.65756E+00_JPRB, 0.70208E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.75151E+00_JPRB, 0.86295E+00_JPRB, 0.10094E+01_JPRB, 0.11405E+01_JPRB, 0.11721E+01_JPRB, 0.11514E+01_JPRB, 0.10743E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10709E+01_JPRB, 0.10845E+01_JPRB, 0.11077E+01_JPRB, 0.11680E+01_JPRB, 0.12881E+01_JPRB, 0.14669E+01_JPRB ,&
&  0.17001E+01_JPRB, 0.19595E+01_JPRB, 0.22515E+01_JPRB, 0.25727E+01_JPRB, 0.28794E+01_JPRB, 0.30635E+01_JPRB, 0.29577E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.13168E+01_JPRB, 0.17707E+01_JPRB, 0.17644E+01_JPRB, 0.16817E+01_JPRB, 0.16699E+01_JPRB, 0.17930E+01_JPRB ,&
&  0.19153E+01_JPRB, 0.19147E+01_JPRB, 0.21178E+01_JPRB, 0.24107E+01_JPRB, 0.25222E+01_JPRB, 0.28298E+01_JPRB, 0.29925E+01_JPRB ,&
&  0.31563E+01_JPRB, 0.33326E+01_JPRB, 0.35311E+01_JPRB, 0.37306E+01_JPRB, 0.39258E+01_JPRB, 0.41092E+01_JPRB, 0.43550E+01_JPRB ,&
&  0.46940E+01_JPRB, 0.50603E+01_JPRB, 0.53662E+01_JPRB, 0.48462E+01_JPRB, 0.51397E+01_JPRB, 0.45215E+01_JPRB, 0.39032E+01_JPRB ,&
&  0.23717E+01_JPRB, 0.15119E+01_JPRB, 0.15049E+01_JPRB, 0.15767E+01_JPRB, 0.13938E+01_JPRB, 0.12279E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.98776E+00_JPRB, 0.92378E+00_JPRB, 0.97625E+00_JPRB, 0.10436E+01_JPRB, 0.10381E+01_JPRB, 0.10822E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.10345E+01_JPRB, 0.10396E+01_JPRB, 0.10165E+01_JPRB, 0.10195E+01_JPRB, 0.10467E+01_JPRB, 0.10900E+01_JPRB, 0.10998E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.10661E+01_JPRB, 0.10460E+01_JPRB, 0.10329E+01_JPRB, 0.10062E+01_JPRB, 0.97416E+00_JPRB, 0.91760E+00_JPRB ,&
&  0.85644E+00_JPRB, 0.86004E+00_JPRB, 0.95862E+00_JPRB, 0.10879E+01_JPRB, 0.10193E+01_JPRB, 0.88259E+00_JPRB, 0.70819E+00_JPRB ,&
&  0.69433E+00_JPRB, 0.71494E+00_JPRB, 0.71443E+00_JPRB, 0.69433E+00_JPRB, 0.67087E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.76200E+00_JPRB, 0.73221E+00_JPRB, 0.69254E+00_JPRB, 0.68259E+00_JPRB, 0.73559E+00_JPRB, 0.78597E+00_JPRB, 0.78997E+00_JPRB ,&
&  0.80284E+00_JPRB, 0.87448E+00_JPRB, 0.10016E+01_JPRB, 0.11014E+01_JPRB, 0.11361E+01_JPRB, 0.11318E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10305E+01_JPRB, 0.10243E+01_JPRB, 0.10133E+01_JPRB, 0.10227E+01_JPRB, 0.10489E+01_JPRB, 0.11035E+01_JPRB, 0.12017E+01_JPRB ,&
&  0.13567E+01_JPRB, 0.16076E+01_JPRB, 0.19684E+01_JPRB, 0.24873E+01_JPRB, 0.32219E+01_JPRB, 0.40924E+01_JPRB, 0.48567E+01_JPRB ,&
&  0.36748E+01_JPRB, 0.14126E+01_JPRB, 0.16697E+01_JPRB, 0.21052E+01_JPRB, 0.19420E+01_JPRB, 0.19494E+01_JPRB, 0.18946E+01_JPRB ,&
&  0.19443E+01_JPRB, 0.19794E+01_JPRB, 0.20265E+01_JPRB, 0.22772E+01_JPRB, 0.25025E+01_JPRB, 0.27646E+01_JPRB, 0.28755E+01_JPRB ,&
&  0.29745E+01_JPRB, 0.31053E+01_JPRB, 0.32407E+01_JPRB, 0.33824E+01_JPRB, 0.35429E+01_JPRB, 0.37205E+01_JPRB, 0.39992E+01_JPRB ,&
&  0.43572E+01_JPRB, 0.47027E+01_JPRB, 0.49450E+01_JPRB, 0.42590E+01_JPRB, 0.44946E+01_JPRB, 0.36068E+01_JPRB, 0.27632E+01_JPRB ,&
&  0.21927E+01_JPRB, 0.17873E+01_JPRB, 0.16342E+01_JPRB, 0.16521E+01_JPRB, 0.14184E+01_JPRB, 0.11007E+01_JPRB, 0.10555E+01_JPRB ,&
&  0.95513E+00_JPRB, 0.93156E+00_JPRB, 0.96024E+00_JPRB, 0.93002E+00_JPRB, 0.99675E+00_JPRB, 0.10328E+01_JPRB, 0.10642E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.10821E+01_JPRB, 0.11039E+01_JPRB, 0.11440E+01_JPRB, 0.11407E+01_JPRB, 0.11504E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.11422E+01_JPRB, 0.11465E+01_JPRB, 0.11538E+01_JPRB, 0.11478E+01_JPRB, 0.10546E+01_JPRB, 0.99226E+00_JPRB ,&
&  0.95246E+00_JPRB, 0.93001E+00_JPRB, 0.88580E+00_JPRB, 0.92064E+00_JPRB, 0.91466E+00_JPRB, 0.91685E+00_JPRB, 0.94682E+00_JPRB ,&
&  0.95633E+00_JPRB, 0.95678E+00_JPRB, 0.90078E+00_JPRB, 0.84189E+00_JPRB, 0.80502E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.84200E+00_JPRB, 0.78159E+00_JPRB, 0.74736E+00_JPRB, 0.75652E+00_JPRB, 0.80757E+00_JPRB, 0.85986E+00_JPRB, 0.85705E+00_JPRB ,&
&  0.84249E+00_JPRB, 0.86640E+00_JPRB, 0.94207E+00_JPRB, 0.10080E+01_JPRB, 0.10187E+01_JPRB, 0.10261E+01_JPRB, 0.10533E+01_JPRB ,&
&  0.10135E+01_JPRB, 0.99068E+00_JPRB, 0.98361E+00_JPRB, 0.96871E+00_JPRB, 0.97661E+00_JPRB, 0.98396E+00_JPRB, 0.10150E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.12319E+01_JPRB, 0.14828E+01_JPRB, 0.18614E+01_JPRB, 0.24186E+01_JPRB, 0.34459E+01_JPRB, 0.50427E+01_JPRB ,&
&  0.74946E+01_JPRB, 0.49070E+01_JPRB, 0.16251E+01_JPRB, 0.21380E+01_JPRB, 0.22567E+01_JPRB, 0.22818E+01_JPRB, 0.19630E+01_JPRB ,&
&  0.21022E+01_JPRB, 0.21725E+01_JPRB, 0.19558E+01_JPRB, 0.20998E+01_JPRB, 0.23221E+01_JPRB, 0.25093E+01_JPRB, 0.26241E+01_JPRB ,&
&  0.26685E+01_JPRB, 0.27479E+01_JPRB, 0.28137E+01_JPRB, 0.28666E+01_JPRB, 0.29336E+01_JPRB, 0.30482E+01_JPRB, 0.32493E+01_JPRB ,&
&  0.36360E+01_JPRB, 0.41119E+01_JPRB, 0.45161E+01_JPRB, 0.39334E+01_JPRB, 0.39510E+01_JPRB, 0.31066E+01_JPRB, 0.26189E+01_JPRB ,&
&  0.22902E+01_JPRB, 0.22496E+01_JPRB, 0.21364E+01_JPRB, 0.17927E+01_JPRB, 0.14544E+01_JPRB, 0.12414E+01_JPRB, 0.12404E+01_JPRB ,&
&  0.10622E+01_JPRB, 0.97193E+00_JPRB, 0.95625E+00_JPRB, 0.99399E+00_JPRB, 0.10150E+01_JPRB, 0.10354E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.11194E+01_JPRB, 0.11712E+01_JPRB, 0.12019E+01_JPRB, 0.12279E+01_JPRB, 0.12731E+01_JPRB, 0.12713E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.12542E+01_JPRB, 0.12574E+01_JPRB, 0.12637E+01_JPRB, 0.11992E+01_JPRB, 0.11049E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.10613E+01_JPRB, 0.10062E+01_JPRB, 0.10063E+01_JPRB, 0.10056E+01_JPRB, 0.10611E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.11209E+01_JPRB, 0.10421E+01_JPRB, 0.90919E+00_JPRB, 0.89365E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.90825E+00_JPRB, 0.88556E+00_JPRB, 0.85342E+00_JPRB, 0.83177E+00_JPRB, 0.85642E+00_JPRB, 0.89311E+00_JPRB, 0.89257E+00_JPRB ,&
&  0.86558E+00_JPRB, 0.84897E+00_JPRB, 0.88734E+00_JPRB, 0.92794E+00_JPRB, 0.92593E+00_JPRB, 0.94765E+00_JPRB, 0.96039E+00_JPRB ,&
&  0.99840E+00_JPRB, 0.99694E+00_JPRB, 0.98044E+00_JPRB, 0.95753E+00_JPRB, 0.95655E+00_JPRB, 0.94444E+00_JPRB, 0.94899E+00_JPRB ,&
&  0.97704E+00_JPRB, 0.10415E+01_JPRB, 0.11766E+01_JPRB, 0.13869E+01_JPRB, 0.16519E+01_JPRB, 0.21404E+01_JPRB, 0.31235E+01_JPRB ,&
&  0.42289E+01_JPRB, 0.54238E+01_JPRB, 0.17481E+01_JPRB, 0.23621E+01_JPRB, 0.28875E+01_JPRB, 0.27656E+01_JPRB, 0.24634E+01_JPRB ,&
&  0.24586E+01_JPRB, 0.24260E+01_JPRB, 0.22279E+01_JPRB, 0.19642E+01_JPRB, 0.19943E+01_JPRB, 0.21206E+01_JPRB, 0.21221E+01_JPRB ,&
&  0.21835E+01_JPRB, 0.22914E+01_JPRB, 0.24523E+01_JPRB, 0.24859E+01_JPRB, 0.24766E+01_JPRB, 0.24530E+01_JPRB, 0.25237E+01_JPRB ,&
&  0.27944E+01_JPRB, 0.32589E+01_JPRB, 0.39528E+01_JPRB, 0.36712E+01_JPRB, 0.36633E+01_JPRB, 0.32401E+01_JPRB, 0.28864E+01_JPRB ,&
&  0.26036E+01_JPRB, 0.26079E+01_JPRB, 0.24762E+01_JPRB, 0.20528E+01_JPRB, 0.16213E+01_JPRB, 0.14227E+01_JPRB, 0.13834E+01_JPRB ,&
&  0.12032E+01_JPRB, 0.10860E+01_JPRB, 0.10688E+01_JPRB, 0.10756E+01_JPRB, 0.11211E+01_JPRB, 0.11162E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.11428E+01_JPRB, 0.11366E+01_JPRB, 0.11729E+01_JPRB, 0.12195E+01_JPRB, 0.12672E+01_JPRB, 0.13293E+01_JPRB, 0.13574E+01_JPRB ,&
&  0.13553E+01_JPRB, 0.13590E+01_JPRB, 0.13480E+01_JPRB, 0.12896E+01_JPRB, 0.12096E+01_JPRB, 0.11658E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.11683E+01_JPRB, 0.11765E+01_JPRB, 0.11037E+01_JPRB, 0.10495E+01_JPRB, 0.10793E+01_JPRB, 0.10860E+01_JPRB, 0.11718E+01_JPRB ,&
&  0.12753E+01_JPRB, 0.13344E+01_JPRB, 0.12710E+01_JPRB, 0.99131E+00_JPRB, 0.97128E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.10234E+01_JPRB, 0.97577E+00_JPRB, 0.94667E+00_JPRB, 0.91914E+00_JPRB, 0.89689E+00_JPRB, 0.93038E+00_JPRB, 0.92366E+00_JPRB ,&
&  0.90000E+00_JPRB, 0.86459E+00_JPRB, 0.88063E+00_JPRB, 0.93081E+00_JPRB, 0.93465E+00_JPRB, 0.91821E+00_JPRB, 0.90757E+00_JPRB ,&
&  0.90846E+00_JPRB, 0.92581E+00_JPRB, 0.93015E+00_JPRB, 0.94582E+00_JPRB, 0.94937E+00_JPRB, 0.94060E+00_JPRB, 0.93640E+00_JPRB ,&
&  0.95645E+00_JPRB, 0.98707E+00_JPRB, 0.10441E+01_JPRB, 0.11174E+01_JPRB, 0.12187E+01_JPRB, 0.13730E+01_JPRB, 0.18704E+01_JPRB ,&
&  0.25310E+01_JPRB, 0.42662E+01_JPRB, 0.34136E+01_JPRB, 0.40786E+01_JPRB, 0.38355E+01_JPRB, 0.35033E+01_JPRB, 0.31252E+01_JPRB ,&
&  0.33637E+01_JPRB, 0.30774E+01_JPRB, 0.28695E+01_JPRB, 0.21257E+01_JPRB, 0.17686E+01_JPRB, 0.17087E+01_JPRB, 0.16454E+01_JPRB ,&
&  0.16143E+01_JPRB, 0.16262E+01_JPRB, 0.18553E+01_JPRB, 0.19606E+01_JPRB, 0.20002E+01_JPRB, 0.20074E+01_JPRB, 0.20228E+01_JPRB ,&
&  0.21637E+01_JPRB, 0.24078E+01_JPRB, 0.30598E+01_JPRB, 0.35285E+01_JPRB, 0.34211E+01_JPRB, 0.33750E+01_JPRB, 0.33067E+01_JPRB ,&
&  0.33411E+01_JPRB, 0.30553E+01_JPRB, 0.27524E+01_JPRB, 0.22953E+01_JPRB, 0.18252E+01_JPRB, 0.17489E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.13161E+01_JPRB, 0.11933E+01_JPRB, 0.11707E+01_JPRB, 0.11757E+01_JPRB, 0.12044E+01_JPRB, 0.11873E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11524E+01_JPRB, 0.11728E+01_JPRB, 0.11677E+01_JPRB, 0.12051E+01_JPRB, 0.12381E+01_JPRB, 0.12790E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.13531E+01_JPRB, 0.13833E+01_JPRB, 0.13386E+01_JPRB, 0.12887E+01_JPRB, 0.12242E+01_JPRB, 0.11842E+01_JPRB, 0.11455E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.11850E+01_JPRB, 0.11730E+01_JPRB, 0.11280E+01_JPRB, 0.10981E+01_JPRB, 0.10802E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.11564E+01_JPRB, 0.11293E+01_JPRB, 0.10734E+01_JPRB, 0.10556E+01_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.10985E+01_JPRB, 0.10716E+01_JPRB, 0.10180E+01_JPRB, 0.10027E+01_JPRB, 0.96417E+00_JPRB, 0.98620E+00_JPRB, 0.99083E+00_JPRB ,&
&  0.95758E+00_JPRB, 0.92376E+00_JPRB, 0.89753E+00_JPRB, 0.91764E+00_JPRB, 0.94023E+00_JPRB, 0.93032E+00_JPRB, 0.89461E+00_JPRB ,&
&  0.88275E+00_JPRB, 0.87969E+00_JPRB, 0.87844E+00_JPRB, 0.88366E+00_JPRB, 0.90195E+00_JPRB, 0.92854E+00_JPRB, 0.92876E+00_JPRB ,&
&  0.92679E+00_JPRB, 0.96932E+00_JPRB, 0.10227E+01_JPRB, 0.10439E+01_JPRB, 0.10738E+01_JPRB, 0.11348E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.17443E+01_JPRB, 0.42132E+01_JPRB, 0.26416E+01_JPRB, 0.52445E+01_JPRB, 0.41648E+01_JPRB, 0.37181E+01_JPRB, 0.30424E+01_JPRB ,&
&  0.28264E+01_JPRB, 0.29628E+01_JPRB, 0.24061E+01_JPRB, 0.19952E+01_JPRB, 0.16609E+01_JPRB, 0.14909E+01_JPRB, 0.13557E+01_JPRB ,&
&  0.12262E+01_JPRB, 0.12471E+01_JPRB, 0.13615E+01_JPRB, 0.14919E+01_JPRB, 0.15681E+01_JPRB, 0.16005E+01_JPRB, 0.16506E+01_JPRB ,&
&  0.17428E+01_JPRB, 0.18599E+01_JPRB, 0.22884E+01_JPRB, 0.28288E+01_JPRB, 0.33246E+01_JPRB, 0.35193E+01_JPRB, 0.37823E+01_JPRB ,&
&  0.38747E+01_JPRB, 0.36273E+01_JPRB, 0.30673E+01_JPRB, 0.25189E+01_JPRB, 0.21175E+01_JPRB, 0.19681E+01_JPRB, 0.17629E+01_JPRB ,&
&  0.15639E+01_JPRB, 0.13817E+01_JPRB, 0.13324E+01_JPRB, 0.12925E+01_JPRB, 0.13121E+01_JPRB, 0.12925E+01_JPRB, 0.12243E+01_JPRB ,&
&  0.11732E+01_JPRB, 0.11710E+01_JPRB, 0.11947E+01_JPRB, 0.12236E+01_JPRB, 0.12571E+01_JPRB, 0.12623E+01_JPRB, 0.12635E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.13463E+01_JPRB, 0.13252E+01_JPRB, 0.13168E+01_JPRB, 0.12652E+01_JPRB, 0.12274E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.12333E+01_JPRB, 0.11878E+01_JPRB, 0.11363E+01_JPRB, 0.11121E+01_JPRB, 0.11153E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.10955E+01_JPRB, 0.11376E+01_JPRB, 0.11031E+01_JPRB, 0.10992E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.11293E+01_JPRB, 0.11220E+01_JPRB, 0.10545E+01_JPRB, 0.10322E+01_JPRB, 0.99852E+00_JPRB, 0.98519E+00_JPRB, 0.99309E+00_JPRB ,&
&  0.98320E+00_JPRB, 0.95747E+00_JPRB, 0.92495E+00_JPRB, 0.90970E+00_JPRB, 0.92194E+00_JPRB, 0.93619E+00_JPRB, 0.90497E+00_JPRB ,&
&  0.86875E+00_JPRB, 0.85483E+00_JPRB, 0.84606E+00_JPRB, 0.84856E+00_JPRB, 0.86166E+00_JPRB, 0.87914E+00_JPRB, 0.89072E+00_JPRB ,&
&  0.88782E+00_JPRB, 0.93394E+00_JPRB, 0.99937E+00_JPRB, 0.10381E+01_JPRB, 0.10486E+01_JPRB, 0.11036E+01_JPRB, 0.12846E+01_JPRB ,&
&  0.17437E+01_JPRB, 0.59117E+01_JPRB, 0.30224E+01_JPRB, 0.54946E+01_JPRB, 0.43161E+01_JPRB, 0.37110E+01_JPRB, 0.29460E+01_JPRB ,&
&  0.26756E+01_JPRB, 0.25076E+01_JPRB, 0.21661E+01_JPRB, 0.18899E+01_JPRB, 0.16765E+01_JPRB, 0.14940E+01_JPRB, 0.13451E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.12064E+01_JPRB, 0.12310E+01_JPRB, 0.13152E+01_JPRB, 0.13622E+01_JPRB, 0.13748E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.15352E+01_JPRB, 0.17931E+01_JPRB, 0.25014E+01_JPRB, 0.30228E+01_JPRB, 0.35357E+01_JPRB, 0.49908E+01_JPRB ,&
&  0.48168E+01_JPRB, 0.44792E+01_JPRB, 0.34066E+01_JPRB, 0.27754E+01_JPRB, 0.23591E+01_JPRB, 0.21122E+01_JPRB, 0.19795E+01_JPRB ,&
&  0.18324E+01_JPRB, 0.16442E+01_JPRB, 0.15532E+01_JPRB, 0.14894E+01_JPRB, 0.14338E+01_JPRB, 0.13839E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.12686E+01_JPRB, 0.12326E+01_JPRB, 0.12487E+01_JPRB, 0.12676E+01_JPRB, 0.12866E+01_JPRB, 0.12770E+01_JPRB, 0.12656E+01_JPRB ,&
&  0.12745E+01_JPRB, 0.13091E+01_JPRB, 0.13008E+01_JPRB, 0.13088E+01_JPRB, 0.12944E+01_JPRB, 0.12663E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.13001E+01_JPRB, 0.12994E+01_JPRB, 0.12270E+01_JPRB, 0.11941E+01_JPRB, 0.11579E+01_JPRB, 0.11885E+01_JPRB, 0.10925E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.10503E+01_JPRB, 0.11103E+01_JPRB, 0.11178E+01_JPRB, 0.11345E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.11460E+01_JPRB, 0.11274E+01_JPRB, 0.10918E+01_JPRB, 0.10388E+01_JPRB, 0.10174E+01_JPRB, 0.99361E+00_JPRB, 0.97901E+00_JPRB ,&
&  0.97281E+00_JPRB, 0.95750E+00_JPRB, 0.94179E+00_JPRB, 0.91918E+00_JPRB, 0.90954E+00_JPRB, 0.92125E+00_JPRB, 0.91609E+00_JPRB ,&
&  0.89814E+00_JPRB, 0.87427E+00_JPRB, 0.86520E+00_JPRB, 0.85553E+00_JPRB, 0.84482E+00_JPRB, 0.84489E+00_JPRB, 0.85630E+00_JPRB ,&
&  0.86394E+00_JPRB, 0.88529E+00_JPRB, 0.92586E+00_JPRB, 0.96491E+00_JPRB, 0.97771E+00_JPRB, 0.10161E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.18827E+01_JPRB, 0.20404E+01_JPRB, 0.45244E+01_JPRB, 0.53715E+01_JPRB, 0.44260E+01_JPRB, 0.38363E+01_JPRB, 0.32162E+01_JPRB ,&
&  0.28109E+01_JPRB, 0.23352E+01_JPRB, 0.19478E+01_JPRB, 0.17008E+01_JPRB, 0.15611E+01_JPRB, 0.14033E+01_JPRB, 0.13117E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.12344E+01_JPRB, 0.12325E+01_JPRB, 0.12496E+01_JPRB, 0.12474E+01_JPRB, 0.12417E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.12819E+01_JPRB, 0.13486E+01_JPRB, 0.14966E+01_JPRB, 0.19486E+01_JPRB, 0.27932E+01_JPRB, 0.36605E+01_JPRB, 0.49748E+01_JPRB ,&
&  0.56168E+01_JPRB, 0.51718E+01_JPRB, 0.37530E+01_JPRB, 0.29780E+01_JPRB, 0.25567E+01_JPRB, 0.22666E+01_JPRB, 0.21175E+01_JPRB ,&
&  0.19962E+01_JPRB, 0.18493E+01_JPRB, 0.17323E+01_JPRB, 0.17108E+01_JPRB, 0.16049E+01_JPRB, 0.15047E+01_JPRB, 0.14147E+01_JPRB ,&
&  0.13632E+01_JPRB, 0.13248E+01_JPRB, 0.12830E+01_JPRB, 0.12563E+01_JPRB, 0.12675E+01_JPRB, 0.12686E+01_JPRB, 0.12570E+01_JPRB ,&
&  0.12536E+01_JPRB, 0.12678E+01_JPRB, 0.12726E+01_JPRB, 0.12678E+01_JPRB, 0.12941E+01_JPRB, 0.13019E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.13091E+01_JPRB, 0.13195E+01_JPRB, 0.12717E+01_JPRB, 0.12569E+01_JPRB, 0.11955E+01_JPRB, 0.12375E+01_JPRB, 0.11474E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.10509E+01_JPRB, 0.10555E+01_JPRB, 0.11055E+01_JPRB, 0.11455E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.11296E+01_JPRB, 0.11146E+01_JPRB, 0.10919E+01_JPRB, 0.10616E+01_JPRB, 0.10350E+01_JPRB, 0.10188E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.99859E+00_JPRB, 0.99683E+00_JPRB, 0.98474E+00_JPRB, 0.95590E+00_JPRB, 0.93137E+00_JPRB, 0.92501E+00_JPRB, 0.92843E+00_JPRB ,&
&  0.92128E+00_JPRB, 0.90968E+00_JPRB, 0.91210E+00_JPRB, 0.89426E+00_JPRB, 0.87227E+00_JPRB, 0.85821E+00_JPRB, 0.85416E+00_JPRB ,&
&  0.86023E+00_JPRB, 0.87222E+00_JPRB, 0.90238E+00_JPRB, 0.93350E+00_JPRB, 0.94144E+00_JPRB, 0.94446E+00_JPRB, 0.99772E+00_JPRB ,&
&  0.16379E+01_JPRB, 0.34570E+01_JPRB, 0.52761E+01_JPRB, 0.49300E+01_JPRB, 0.42207E+01_JPRB, 0.35353E+01_JPRB, 0.31663E+01_JPRB ,&
&  0.26148E+01_JPRB, 0.22716E+01_JPRB, 0.19289E+01_JPRB, 0.16622E+01_JPRB, 0.14772E+01_JPRB, 0.13625E+01_JPRB, 0.12916E+01_JPRB ,&
&  0.12365E+01_JPRB, 0.12084E+01_JPRB, 0.12030E+01_JPRB, 0.11973E+01_JPRB, 0.11856E+01_JPRB, 0.11658E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.11944E+01_JPRB, 0.12535E+01_JPRB, 0.13426E+01_JPRB, 0.15863E+01_JPRB, 0.24256E+01_JPRB, 0.28946E+01_JPRB, 0.43341E+01_JPRB ,&
&  0.58090E+01_JPRB, 0.48903E+01_JPRB, 0.38265E+01_JPRB, 0.31263E+01_JPRB, 0.26908E+01_JPRB, 0.23963E+01_JPRB, 0.22189E+01_JPRB ,&
&  0.20425E+01_JPRB, 0.19237E+01_JPRB, 0.18193E+01_JPRB, 0.17176E+01_JPRB, 0.16500E+01_JPRB, 0.15536E+01_JPRB, 0.14679E+01_JPRB ,&
&  0.14335E+01_JPRB, 0.13839E+01_JPRB, 0.13293E+01_JPRB, 0.12677E+01_JPRB, 0.12491E+01_JPRB, 0.12433E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.12373E+01_JPRB, 0.12481E+01_JPRB, 0.12426E+01_JPRB, 0.12812E+01_JPRB, 0.13190E+01_JPRB, 0.13162E+01_JPRB ,&
&  0.12709E+01_JPRB, 0.12798E+01_JPRB, 0.12979E+01_JPRB, 0.12730E+01_JPRB, 0.12781E+01_JPRB, 0.13466E+01_JPRB, 0.12288E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.10851E+01_JPRB, 0.10531E+01_JPRB, 0.10589E+01_JPRB, 0.11245E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.11370E+01_JPRB, 0.11268E+01_JPRB, 0.11028E+01_JPRB, 0.10811E+01_JPRB, 0.10703E+01_JPRB, 0.10516E+01_JPRB, 0.10368E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.10139E+01_JPRB, 0.10030E+01_JPRB, 0.98693E+00_JPRB, 0.96568E+00_JPRB, 0.94672E+00_JPRB, 0.93854E+00_JPRB ,&
&  0.93629E+00_JPRB, 0.92560E+00_JPRB, 0.92866E+00_JPRB, 0.92920E+00_JPRB, 0.92533E+00_JPRB, 0.91338E+00_JPRB, 0.90171E+00_JPRB ,&
&  0.89433E+00_JPRB, 0.88028E+00_JPRB, 0.87862E+00_JPRB, 0.90416E+00_JPRB, 0.92685E+00_JPRB, 0.93534E+00_JPRB, 0.95009E+00_JPRB ,&
&  0.12624E+01_JPRB, 0.26618E+01_JPRB, 0.38231E+01_JPRB, 0.40992E+01_JPRB, 0.38310E+01_JPRB, 0.33260E+01_JPRB, 0.28367E+01_JPRB ,&
&  0.24061E+01_JPRB, 0.20709E+01_JPRB, 0.18520E+01_JPRB, 0.16808E+01_JPRB, 0.15287E+01_JPRB, 0.14035E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.11982E+01_JPRB, 0.11725E+01_JPRB, 0.11584E+01_JPRB, 0.11469E+01_JPRB, 0.11374E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.11955E+01_JPRB, 0.12464E+01_JPRB, 0.13316E+01_JPRB, 0.17323E+01_JPRB, 0.24065E+01_JPRB, 0.33343E+01_JPRB ,&
&  0.35920E+01_JPRB, 0.36448E+01_JPRB, 0.33307E+01_JPRB, 0.28901E+01_JPRB, 0.25327E+01_JPRB, 0.22665E+01_JPRB, 0.20941E+01_JPRB ,&
&  0.19302E+01_JPRB, 0.18066E+01_JPRB, 0.17125E+01_JPRB, 0.16039E+01_JPRB, 0.14971E+01_JPRB, 0.14369E+01_JPRB, 0.14351E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.14196E+01_JPRB, 0.13586E+01_JPRB, 0.12861E+01_JPRB, 0.12120E+01_JPRB, 0.11797E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.11741E+01_JPRB, 0.12056E+01_JPRB, 0.12384E+01_JPRB, 0.12595E+01_JPRB, 0.12685E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.12762E+01_JPRB, 0.12907E+01_JPRB, 0.12720E+01_JPRB, 0.12995E+01_JPRB, 0.13117E+01_JPRB, 0.13718E+01_JPRB, 0.12970E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.11278E+01_JPRB, 0.10915E+01_JPRB, 0.10731E+01_JPRB, 0.11200E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11514E+01_JPRB, 0.11396E+01_JPRB, 0.11166E+01_JPRB, 0.10964E+01_JPRB, 0.10707E+01_JPRB, 0.10485E+01_JPRB, 0.10298E+01_JPRB ,&
&  0.10192E+01_JPRB, 0.10001E+01_JPRB, 0.98992E+00_JPRB, 0.98048E+00_JPRB, 0.97515E+00_JPRB, 0.96353E+00_JPRB, 0.94651E+00_JPRB ,&
&  0.92889E+00_JPRB, 0.91831E+00_JPRB, 0.91418E+00_JPRB, 0.91714E+00_JPRB, 0.92626E+00_JPRB, 0.93246E+00_JPRB, 0.92759E+00_JPRB ,&
&  0.91812E+00_JPRB, 0.90265E+00_JPRB, 0.88493E+00_JPRB, 0.90309E+00_JPRB, 0.92368E+00_JPRB, 0.93259E+00_JPRB, 0.93221E+00_JPRB ,&
&  0.11203E+01_JPRB, 0.17518E+01_JPRB, 0.25150E+01_JPRB, 0.27287E+01_JPRB, 0.27268E+01_JPRB, 0.27246E+01_JPRB, 0.24742E+01_JPRB ,&
&  0.22076E+01_JPRB, 0.19288E+01_JPRB, 0.16847E+01_JPRB, 0.15477E+01_JPRB, 0.14591E+01_JPRB, 0.13697E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.12172E+01_JPRB, 0.11702E+01_JPRB, 0.11393E+01_JPRB, 0.11267E+01_JPRB, 0.11214E+01_JPRB, 0.11216E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.11354E+01_JPRB, 0.11539E+01_JPRB, 0.11955E+01_JPRB, 0.12895E+01_JPRB, 0.15771E+01_JPRB, 0.20218E+01_JPRB ,&
&  0.23206E+01_JPRB, 0.25354E+01_JPRB, 0.26806E+01_JPRB, 0.26484E+01_JPRB, 0.24342E+01_JPRB, 0.21862E+01_JPRB, 0.20034E+01_JPRB ,&
&  0.18526E+01_JPRB, 0.17001E+01_JPRB, 0.15652E+01_JPRB, 0.14553E+01_JPRB, 0.13607E+01_JPRB, 0.13196E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.13515E+01_JPRB, 0.13003E+01_JPRB, 0.12396E+01_JPRB, 0.11673E+01_JPRB, 0.11225E+01_JPRB, 0.11107E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.11300E+01_JPRB, 0.11558E+01_JPRB, 0.11870E+01_JPRB, 0.12131E+01_JPRB, 0.12269E+01_JPRB, 0.12297E+01_JPRB ,&
&  0.12363E+01_JPRB, 0.12439E+01_JPRB, 0.11899E+01_JPRB, 0.12298E+01_JPRB, 0.13399E+01_JPRB, 0.13209E+01_JPRB, 0.12903E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.11542E+01_JPRB, 0.10983E+01_JPRB, 0.10630E+01_JPRB, 0.11097E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.10051E+01_JPRB, 0.10285E+01_JPRB, 0.10344E+01_JPRB, 0.10324E+01_JPRB, 0.10173E+01_JPRB, 0.99419E+00_JPRB, 0.97584E+00_JPRB ,&
&  0.96719E+00_JPRB, 0.95096E+00_JPRB, 0.94328E+00_JPRB, 0.93976E+00_JPRB, 0.94220E+00_JPRB, 0.94797E+00_JPRB, 0.94611E+00_JPRB ,&
&  0.93448E+00_JPRB, 0.91808E+00_JPRB, 0.90672E+00_JPRB, 0.90502E+00_JPRB, 0.91408E+00_JPRB, 0.92452E+00_JPRB, 0.93131E+00_JPRB ,&
&  0.93372E+00_JPRB, 0.92619E+00_JPRB, 0.92179E+00_JPRB, 0.93384E+00_JPRB, 0.94591E+00_JPRB, 0.94088E+00_JPRB, 0.92709E+00_JPRB ,&
&  0.10069E+01_JPRB, 0.11764E+01_JPRB, 0.14654E+01_JPRB, 0.17178E+01_JPRB, 0.19415E+01_JPRB, 0.20227E+01_JPRB, 0.19601E+01_JPRB ,&
&  0.18199E+01_JPRB, 0.16921E+01_JPRB, 0.15442E+01_JPRB, 0.14110E+01_JPRB, 0.13132E+01_JPRB, 0.12588E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.11311E+01_JPRB, 0.10912E+01_JPRB, 0.10682E+01_JPRB, 0.10591E+01_JPRB, 0.10510E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.10227E+01_JPRB, 0.10300E+01_JPRB, 0.10547E+01_JPRB, 0.10994E+01_JPRB, 0.12142E+01_JPRB, 0.14192E+01_JPRB ,&
&  0.16006E+01_JPRB, 0.17904E+01_JPRB, 0.19746E+01_JPRB, 0.21168E+01_JPRB, 0.21068E+01_JPRB, 0.20076E+01_JPRB, 0.18727E+01_JPRB ,&
&  0.17193E+01_JPRB, 0.15356E+01_JPRB, 0.13700E+01_JPRB, 0.12703E+01_JPRB, 0.12358E+01_JPRB, 0.12381E+01_JPRB, 0.12693E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.12619E+01_JPRB, 0.12072E+01_JPRB, 0.11515E+01_JPRB, 0.11117E+01_JPRB, 0.10750E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10686E+01_JPRB, 0.10892E+01_JPRB, 0.11112E+01_JPRB, 0.11405E+01_JPRB, 0.11602E+01_JPRB, 0.11641E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.11577E+01_JPRB, 0.11527E+01_JPRB, 0.11973E+01_JPRB, 0.12022E+01_JPRB, 0.12042E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.11636E+01_JPRB, 0.11127E+01_JPRB, 0.10561E+01_JPRB, 0.10231E+01_JPRB, 0.94947E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.94015E+00_JPRB, 0.96069E+00_JPRB, 0.96727E+00_JPRB, 0.96614E+00_JPRB, 0.96154E+00_JPRB, 0.94593E+00_JPRB, 0.92715E+00_JPRB ,&
&  0.90684E+00_JPRB, 0.89841E+00_JPRB, 0.89531E+00_JPRB, 0.89924E+00_JPRB, 0.90440E+00_JPRB, 0.91232E+00_JPRB, 0.92358E+00_JPRB ,&
&  0.93404E+00_JPRB, 0.93810E+00_JPRB, 0.94086E+00_JPRB, 0.93777E+00_JPRB, 0.93503E+00_JPRB, 0.93654E+00_JPRB, 0.94016E+00_JPRB ,&
&  0.94836E+00_JPRB, 0.95032E+00_JPRB, 0.94546E+00_JPRB, 0.95490E+00_JPRB, 0.97202E+00_JPRB, 0.96385E+00_JPRB, 0.94837E+00_JPRB ,&
&  0.97366E+00_JPRB, 0.92880E+00_JPRB, 0.99696E+00_JPRB, 0.12155E+01_JPRB, 0.14322E+01_JPRB, 0.16003E+01_JPRB, 0.16278E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.14705E+01_JPRB, 0.13787E+01_JPRB, 0.12876E+01_JPRB, 0.12214E+01_JPRB, 0.11625E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.10677E+01_JPRB, 0.10286E+01_JPRB, 0.99359E+00_JPRB, 0.96384E+00_JPRB, 0.93573E+00_JPRB, 0.91806E+00_JPRB ,&
&  0.91149E+00_JPRB, 0.91846E+00_JPRB, 0.93131E+00_JPRB, 0.95261E+00_JPRB, 0.99353E+00_JPRB, 0.10502E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.13622E+01_JPRB, 0.14734E+01_JPRB, 0.15697E+01_JPRB, 0.16476E+01_JPRB, 0.16535E+01_JPRB, 0.15766E+01_JPRB ,&
&  0.14449E+01_JPRB, 0.13386E+01_JPRB, 0.12578E+01_JPRB, 0.12029E+01_JPRB, 0.11906E+01_JPRB, 0.12025E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.11917E+01_JPRB, 0.11521E+01_JPRB, 0.11005E+01_JPRB, 0.10527E+01_JPRB, 0.10247E+01_JPRB, 0.10067E+01_JPRB, 0.98885E+00_JPRB ,&
&  0.97494E+00_JPRB, 0.97160E+00_JPRB, 0.97997E+00_JPRB, 0.10019E+01_JPRB, 0.10312E+01_JPRB, 0.10626E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.10937E+01_JPRB, 0.11042E+01_JPRB, 0.10780E+01_JPRB, 0.11243E+01_JPRB, 0.11376E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.10828E+01_JPRB, 0.10373E+01_JPRB, 0.86492E+00_JPRB, 0.91753E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.90796E+00_JPRB, 0.91052E+00_JPRB, 0.91272E+00_JPRB, 0.90688E+00_JPRB, 0.90149E+00_JPRB, 0.89230E+00_JPRB, 0.87445E+00_JPRB ,&
&  0.85781E+00_JPRB, 0.85153E+00_JPRB, 0.85164E+00_JPRB, 0.85298E+00_JPRB, 0.85577E+00_JPRB, 0.86159E+00_JPRB, 0.87241E+00_JPRB ,&
&  0.88545E+00_JPRB, 0.89883E+00_JPRB, 0.91452E+00_JPRB, 0.92855E+00_JPRB, 0.93038E+00_JPRB, 0.92562E+00_JPRB, 0.92428E+00_JPRB ,&
&  0.92912E+00_JPRB, 0.92709E+00_JPRB, 0.91879E+00_JPRB, 0.92029E+00_JPRB, 0.93203E+00_JPRB, 0.92625E+00_JPRB, 0.91010E+00_JPRB ,&
&  0.91453E+00_JPRB, 0.80796E+00_JPRB, 0.87004E+00_JPRB, 0.97637E+00_JPRB, 0.11304E+01_JPRB, 0.12584E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.13021E+01_JPRB, 0.12350E+01_JPRB, 0.11743E+01_JPRB, 0.11060E+01_JPRB, 0.10652E+01_JPRB, 0.10395E+01_JPRB, 0.10143E+01_JPRB ,&
&  0.97819E+00_JPRB, 0.93995E+00_JPRB, 0.89699E+00_JPRB, 0.85695E+00_JPRB, 0.82542E+00_JPRB, 0.80735E+00_JPRB, 0.79864E+00_JPRB ,&
&  0.79528E+00_JPRB, 0.80479E+00_JPRB, 0.81173E+00_JPRB, 0.82534E+00_JPRB, 0.84916E+00_JPRB, 0.88662E+00_JPRB, 0.93747E+00_JPRB ,&
&  0.10055E+01_JPRB, 0.10850E+01_JPRB, 0.11621E+01_JPRB, 0.12160E+01_JPRB, 0.12493E+01_JPRB, 0.12718E+01_JPRB, 0.12622E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.11651E+01_JPRB, 0.11542E+01_JPRB, 0.11498E+01_JPRB, 0.11332E+01_JPRB, 0.11157E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10090E+01_JPRB, 0.97061E+00_JPRB, 0.93478E+00_JPRB, 0.91139E+00_JPRB, 0.89847E+00_JPRB, 0.87728E+00_JPRB ,&
&  0.86367E+00_JPRB, 0.86196E+00_JPRB, 0.87161E+00_JPRB, 0.89463E+00_JPRB, 0.92601E+00_JPRB, 0.95625E+00_JPRB, 0.97443E+00_JPRB ,&
&  0.98188E+00_JPRB, 0.98908E+00_JPRB, 0.99302E+00_JPRB, 0.10236E+01_JPRB, 0.10348E+01_JPRB, 0.10416E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10439E+01_JPRB, 0.10391E+01_JPRB, 0.81507E+00_JPRB, 0.88302E+00_JPRB, 0.89405E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.91771E+00_JPRB, 0.89481E+00_JPRB, 0.88156E+00_JPRB, 0.87321E+00_JPRB, 0.86689E+00_JPRB, 0.85895E+00_JPRB, 0.84432E+00_JPRB ,&
&  0.82907E+00_JPRB, 0.82098E+00_JPRB, 0.81417E+00_JPRB, 0.80579E+00_JPRB, 0.80758E+00_JPRB, 0.81393E+00_JPRB, 0.82534E+00_JPRB ,&
&  0.83756E+00_JPRB, 0.85284E+00_JPRB, 0.86712E+00_JPRB, 0.88483E+00_JPRB, 0.89555E+00_JPRB, 0.90232E+00_JPRB, 0.90512E+00_JPRB ,&
&  0.90672E+00_JPRB, 0.90612E+00_JPRB, 0.89714E+00_JPRB, 0.88572E+00_JPRB, 0.88400E+00_JPRB, 0.88938E+00_JPRB, 0.88547E+00_JPRB ,&
&  0.78928E+00_JPRB, 0.71886E+00_JPRB, 0.79105E+00_JPRB, 0.87654E+00_JPRB, 0.97091E+00_JPRB, 0.10544E+01_JPRB, 0.10941E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.10324E+01_JPRB, 0.99158E+00_JPRB, 0.94462E+00_JPRB, 0.92064E+00_JPRB, 0.90843E+00_JPRB, 0.89499E+00_JPRB ,&
&  0.88321E+00_JPRB, 0.86836E+00_JPRB, 0.84671E+00_JPRB, 0.81274E+00_JPRB, 0.78669E+00_JPRB, 0.76750E+00_JPRB, 0.76081E+00_JPRB ,&
&  0.76029E+00_JPRB, 0.76265E+00_JPRB, 0.76587E+00_JPRB, 0.76935E+00_JPRB, 0.77817E+00_JPRB, 0.79342E+00_JPRB, 0.81389E+00_JPRB ,&
&  0.84539E+00_JPRB, 0.88353E+00_JPRB, 0.93205E+00_JPRB, 0.96568E+00_JPRB, 0.98539E+00_JPRB, 0.10019E+01_JPRB, 0.10228E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10263E+01_JPRB, 0.99938E+00_JPRB, 0.98559E+00_JPRB, 0.98172E+00_JPRB, 0.97496E+00_JPRB, 0.95798E+00_JPRB ,&
&  0.92873E+00_JPRB, 0.89286E+00_JPRB, 0.85568E+00_JPRB, 0.83270E+00_JPRB, 0.82656E+00_JPRB, 0.82332E+00_JPRB, 0.81619E+00_JPRB ,&
&  0.81034E+00_JPRB, 0.80958E+00_JPRB, 0.81581E+00_JPRB, 0.82842E+00_JPRB, 0.84555E+00_JPRB, 0.86625E+00_JPRB, 0.88303E+00_JPRB ,&
&  0.89860E+00_JPRB, 0.91318E+00_JPRB, 0.92336E+00_JPRB, 0.94353E+00_JPRB, 0.96377E+00_JPRB, 0.97425E+00_JPRB, 0.97278E+00_JPRB ,&
&  0.97503E+00_JPRB, 0.96607E+00_JPRB, 0.95546E+00_JPRB, 0.94882E+00_JPRB, 0.93706E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.87330E+00_JPRB, 0.85205E+00_JPRB, 0.83909E+00_JPRB, 0.83294E+00_JPRB, 0.82280E+00_JPRB, 0.80272E+00_JPRB, 0.78617E+00_JPRB ,&
&  0.77056E+00_JPRB, 0.76137E+00_JPRB, 0.75691E+00_JPRB, 0.75568E+00_JPRB, 0.76169E+00_JPRB, 0.76771E+00_JPRB, 0.77527E+00_JPRB ,&
&  0.78730E+00_JPRB, 0.80621E+00_JPRB, 0.82194E+00_JPRB, 0.83565E+00_JPRB, 0.84738E+00_JPRB, 0.85867E+00_JPRB, 0.86056E+00_JPRB ,&
&  0.86183E+00_JPRB, 0.86824E+00_JPRB, 0.86848E+00_JPRB, 0.85418E+00_JPRB, 0.84000E+00_JPRB, 0.84520E+00_JPRB, 0.84401E+00_JPRB ,&
&  0.83614E+00_JPRB, 0.72420E+00_JPRB, 0.78925E+00_JPRB, 0.82700E+00_JPRB, 0.87112E+00_JPRB, 0.91838E+00_JPRB, 0.94392E+00_JPRB ,&
&  0.93722E+00_JPRB, 0.90941E+00_JPRB, 0.88061E+00_JPRB, 0.84867E+00_JPRB, 0.83864E+00_JPRB, 0.82933E+00_JPRB, 0.81625E+00_JPRB ,&
&  0.80734E+00_JPRB, 0.79901E+00_JPRB, 0.78706E+00_JPRB, 0.77166E+00_JPRB, 0.75296E+00_JPRB, 0.73873E+00_JPRB, 0.72587E+00_JPRB ,&
&  0.72063E+00_JPRB, 0.71115E+00_JPRB, 0.70321E+00_JPRB, 0.70384E+00_JPRB, 0.71639E+00_JPRB, 0.72387E+00_JPRB, 0.72754E+00_JPRB ,&
&  0.73357E+00_JPRB, 0.74381E+00_JPRB, 0.76188E+00_JPRB, 0.78512E+00_JPRB, 0.80401E+00_JPRB, 0.81822E+00_JPRB, 0.83694E+00_JPRB ,&
&  0.86482E+00_JPRB, 0.88359E+00_JPRB, 0.88291E+00_JPRB, 0.86620E+00_JPRB, 0.85386E+00_JPRB, 0.84949E+00_JPRB, 0.83394E+00_JPRB ,&
&  0.80890E+00_JPRB, 0.78982E+00_JPRB, 0.78116E+00_JPRB, 0.77687E+00_JPRB, 0.77122E+00_JPRB, 0.76456E+00_JPRB, 0.76351E+00_JPRB ,&
&  0.76463E+00_JPRB, 0.76453E+00_JPRB, 0.76350E+00_JPRB, 0.76650E+00_JPRB, 0.77315E+00_JPRB, 0.78186E+00_JPRB, 0.79372E+00_JPRB ,&
&  0.81053E+00_JPRB, 0.82580E+00_JPRB, 0.83376E+00_JPRB, 0.84568E+00_JPRB, 0.86005E+00_JPRB, 0.86717E+00_JPRB, 0.88280E+00_JPRB ,&
&  0.88453E+00_JPRB, 0.88217E+00_JPRB, 0.88145E+00_JPRB, 0.88659E+00_JPRB, 0.88743E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.82835E+00_JPRB, 0.80710E+00_JPRB, 0.79381E+00_JPRB, 0.78137E+00_JPRB, 0.77258E+00_JPRB, 0.75401E+00_JPRB, 0.74327E+00_JPRB ,&
&  0.73598E+00_JPRB, 0.73105E+00_JPRB, 0.72918E+00_JPRB, 0.72891E+00_JPRB, 0.73931E+00_JPRB, 0.74631E+00_JPRB, 0.74933E+00_JPRB ,&
&  0.75746E+00_JPRB, 0.77568E+00_JPRB, 0.79262E+00_JPRB, 0.80646E+00_JPRB, 0.81683E+00_JPRB, 0.82372E+00_JPRB, 0.82246E+00_JPRB ,&
&  0.82508E+00_JPRB, 0.83214E+00_JPRB, 0.83517E+00_JPRB, 0.82365E+00_JPRB, 0.80946E+00_JPRB, 0.80212E+00_JPRB, 0.80388E+00_JPRB ,&
&  0.79307E+00_JPRB, 0.76270E+00_JPRB, 0.78226E+00_JPRB, 0.79665E+00_JPRB, 0.81436E+00_JPRB, 0.83115E+00_JPRB, 0.84835E+00_JPRB ,&
&  0.85470E+00_JPRB, 0.83524E+00_JPRB, 0.80783E+00_JPRB, 0.78872E+00_JPRB, 0.79263E+00_JPRB, 0.78861E+00_JPRB, 0.77531E+00_JPRB ,&
&  0.76637E+00_JPRB, 0.75828E+00_JPRB, 0.74785E+00_JPRB, 0.73298E+00_JPRB, 0.71995E+00_JPRB, 0.71037E+00_JPRB, 0.70312E+00_JPRB ,&
&  0.68961E+00_JPRB, 0.67847E+00_JPRB, 0.67248E+00_JPRB, 0.66617E+00_JPRB, 0.66151E+00_JPRB, 0.66660E+00_JPRB, 0.67187E+00_JPRB ,&
&  0.67413E+00_JPRB, 0.67865E+00_JPRB, 0.69005E+00_JPRB, 0.70807E+00_JPRB, 0.72468E+00_JPRB, 0.74271E+00_JPRB, 0.76270E+00_JPRB ,&
&  0.78284E+00_JPRB, 0.80104E+00_JPRB, 0.80787E+00_JPRB, 0.81158E+00_JPRB, 0.80417E+00_JPRB, 0.79263E+00_JPRB, 0.77046E+00_JPRB ,&
&  0.74587E+00_JPRB, 0.73878E+00_JPRB, 0.73794E+00_JPRB, 0.74260E+00_JPRB, 0.74015E+00_JPRB, 0.73713E+00_JPRB, 0.73560E+00_JPRB ,&
&  0.73166E+00_JPRB, 0.72794E+00_JPRB, 0.72266E+00_JPRB, 0.71780E+00_JPRB, 0.72110E+00_JPRB, 0.72715E+00_JPRB, 0.74093E+00_JPRB ,&
&  0.75958E+00_JPRB, 0.77934E+00_JPRB, 0.79908E+00_JPRB, 0.81233E+00_JPRB, 0.81677E+00_JPRB, 0.81947E+00_JPRB, 0.82322E+00_JPRB ,&
&  0.82465E+00_JPRB, 0.82331E+00_JPRB, 0.82975E+00_JPRB, 0.83442E+00_JPRB, 0.83906E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.76634E+00_JPRB, 0.74106E+00_JPRB, 0.72239E+00_JPRB, 0.71540E+00_JPRB, 0.71157E+00_JPRB, 0.69892E+00_JPRB, 0.69281E+00_JPRB ,&
&  0.69816E+00_JPRB, 0.69954E+00_JPRB, 0.70618E+00_JPRB, 0.70541E+00_JPRB, 0.70457E+00_JPRB, 0.70725E+00_JPRB, 0.69965E+00_JPRB ,&
&  0.69980E+00_JPRB, 0.70919E+00_JPRB, 0.72030E+00_JPRB, 0.73528E+00_JPRB, 0.74782E+00_JPRB, 0.75719E+00_JPRB, 0.76479E+00_JPRB ,&
&  0.76754E+00_JPRB, 0.77366E+00_JPRB, 0.77684E+00_JPRB, 0.77198E+00_JPRB, 0.75992E+00_JPRB, 0.75029E+00_JPRB, 0.74829E+00_JPRB ,&
&  0.74632E+00_JPRB, 0.74789E+00_JPRB, 0.70433E+00_JPRB, 0.73057E+00_JPRB, 0.73922E+00_JPRB, 0.74712E+00_JPRB, 0.75479E+00_JPRB ,&
&  0.75433E+00_JPRB, 0.74236E+00_JPRB, 0.73508E+00_JPRB, 0.72007E+00_JPRB, 0.71783E+00_JPRB, 0.72358E+00_JPRB, 0.71885E+00_JPRB ,&
&  0.71015E+00_JPRB, 0.69558E+00_JPRB, 0.67877E+00_JPRB, 0.66669E+00_JPRB, 0.66339E+00_JPRB, 0.66008E+00_JPRB, 0.65016E+00_JPRB ,&
&  0.64387E+00_JPRB, 0.64285E+00_JPRB, 0.64101E+00_JPRB, 0.63463E+00_JPRB, 0.61920E+00_JPRB, 0.60557E+00_JPRB, 0.60369E+00_JPRB ,&
&  0.60862E+00_JPRB, 0.61763E+00_JPRB, 0.63424E+00_JPRB, 0.65520E+00_JPRB, 0.67707E+00_JPRB, 0.69745E+00_JPRB, 0.71547E+00_JPRB ,&
&  0.72295E+00_JPRB, 0.72690E+00_JPRB, 0.73097E+00_JPRB, 0.73668E+00_JPRB, 0.73775E+00_JPRB, 0.72892E+00_JPRB, 0.70641E+00_JPRB ,&
&  0.68284E+00_JPRB, 0.67597E+00_JPRB, 0.67954E+00_JPRB, 0.68846E+00_JPRB, 0.69271E+00_JPRB, 0.69220E+00_JPRB, 0.68582E+00_JPRB ,&
&  0.68174E+00_JPRB, 0.67327E+00_JPRB, 0.66396E+00_JPRB, 0.65924E+00_JPRB, 0.65864E+00_JPRB, 0.66470E+00_JPRB, 0.68270E+00_JPRB ,&
&  0.70457E+00_JPRB, 0.72872E+00_JPRB, 0.74945E+00_JPRB, 0.75673E+00_JPRB, 0.76077E+00_JPRB, 0.75931E+00_JPRB, 0.74975E+00_JPRB ,&
&  0.75404E+00_JPRB, 0.77338E+00_JPRB, 0.78337E+00_JPRB, 0.78485E+00_JPRB, 0.78213E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.70989E+00_JPRB, 0.68122E+00_JPRB, 0.66970E+00_JPRB, 0.67169E+00_JPRB, 0.67502E+00_JPRB, 0.67036E+00_JPRB, 0.67716E+00_JPRB ,&
&  0.68799E+00_JPRB, 0.69632E+00_JPRB, 0.70079E+00_JPRB, 0.69666E+00_JPRB, 0.69386E+00_JPRB, 0.69061E+00_JPRB, 0.67547E+00_JPRB ,&
&  0.67027E+00_JPRB, 0.66845E+00_JPRB, 0.67287E+00_JPRB, 0.68056E+00_JPRB, 0.68911E+00_JPRB, 0.70205E+00_JPRB, 0.71161E+00_JPRB ,&
&  0.71772E+00_JPRB, 0.72137E+00_JPRB, 0.72543E+00_JPRB, 0.72543E+00_JPRB, 0.72147E+00_JPRB, 0.71160E+00_JPRB, 0.70400E+00_JPRB ,&
&  0.69936E+00_JPRB, 0.68969E+00_JPRB, 0.67905E+00_JPRB, 0.67807E+00_JPRB, 0.68176E+00_JPRB, 0.68569E+00_JPRB, 0.68759E+00_JPRB ,&
&  0.67828E+00_JPRB, 0.66414E+00_JPRB, 0.66422E+00_JPRB, 0.66342E+00_JPRB, 0.66750E+00_JPRB, 0.67924E+00_JPRB, 0.68180E+00_JPRB ,&
&  0.67651E+00_JPRB, 0.65997E+00_JPRB, 0.63877E+00_JPRB, 0.63412E+00_JPRB, 0.63571E+00_JPRB, 0.62670E+00_JPRB, 0.61830E+00_JPRB ,&
&  0.61682E+00_JPRB, 0.61389E+00_JPRB, 0.60796E+00_JPRB, 0.59972E+00_JPRB, 0.58651E+00_JPRB, 0.56950E+00_JPRB, 0.56104E+00_JPRB ,&
&  0.56113E+00_JPRB, 0.57819E+00_JPRB, 0.60842E+00_JPRB, 0.64175E+00_JPRB, 0.67249E+00_JPRB, 0.68705E+00_JPRB, 0.68605E+00_JPRB ,&
&  0.67297E+00_JPRB, 0.66921E+00_JPRB, 0.67426E+00_JPRB, 0.67914E+00_JPRB, 0.68064E+00_JPRB, 0.66956E+00_JPRB, 0.65082E+00_JPRB ,&
&  0.63689E+00_JPRB, 0.62711E+00_JPRB, 0.62630E+00_JPRB, 0.63156E+00_JPRB, 0.63639E+00_JPRB, 0.63756E+00_JPRB, 0.63180E+00_JPRB ,&
&  0.62602E+00_JPRB, 0.61970E+00_JPRB, 0.61547E+00_JPRB, 0.61725E+00_JPRB, 0.62130E+00_JPRB, 0.62589E+00_JPRB, 0.63986E+00_JPRB ,&
&  0.66014E+00_JPRB, 0.67982E+00_JPRB, 0.68400E+00_JPRB, 0.67951E+00_JPRB, 0.67881E+00_JPRB, 0.67581E+00_JPRB, 0.67289E+00_JPRB ,&
&  0.68287E+00_JPRB, 0.70560E+00_JPRB, 0.72196E+00_JPRB, 0.73075E+00_JPRB, 0.73060E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.63005E+00_JPRB, 0.61325E+00_JPRB, 0.61412E+00_JPRB, 0.62202E+00_JPRB, 0.62650E+00_JPRB, 0.63438E+00_JPRB, 0.65368E+00_JPRB ,&
&  0.66446E+00_JPRB, 0.67121E+00_JPRB, 0.67371E+00_JPRB, 0.66840E+00_JPRB, 0.66736E+00_JPRB, 0.66000E+00_JPRB, 0.64699E+00_JPRB ,&
&  0.64148E+00_JPRB, 0.64037E+00_JPRB, 0.64208E+00_JPRB, 0.64617E+00_JPRB, 0.64878E+00_JPRB, 0.65223E+00_JPRB, 0.65609E+00_JPRB ,&
&  0.65823E+00_JPRB, 0.65829E+00_JPRB, 0.65732E+00_JPRB, 0.65739E+00_JPRB, 0.65620E+00_JPRB, 0.64815E+00_JPRB, 0.63893E+00_JPRB ,&
&  0.62922E+00_JPRB, 0.62054E+00_JPRB, 0.61768E+00_JPRB, 0.62162E+00_JPRB, 0.62633E+00_JPRB, 0.62906E+00_JPRB, 0.62684E+00_JPRB ,&
&  0.62330E+00_JPRB, 0.62339E+00_JPRB, 0.62816E+00_JPRB, 0.63532E+00_JPRB, 0.64344E+00_JPRB, 0.65281E+00_JPRB, 0.65773E+00_JPRB ,&
&  0.65141E+00_JPRB, 0.63254E+00_JPRB, 0.62079E+00_JPRB, 0.61976E+00_JPRB, 0.61454E+00_JPRB, 0.59734E+00_JPRB, 0.58577E+00_JPRB ,&
&  0.58072E+00_JPRB, 0.57151E+00_JPRB, 0.56186E+00_JPRB, 0.54914E+00_JPRB, 0.53773E+00_JPRB, 0.53337E+00_JPRB, 0.53223E+00_JPRB ,&
&  0.53538E+00_JPRB, 0.55529E+00_JPRB, 0.59171E+00_JPRB, 0.62632E+00_JPRB, 0.64091E+00_JPRB, 0.63390E+00_JPRB, 0.61594E+00_JPRB ,&
&  0.60705E+00_JPRB, 0.60289E+00_JPRB, 0.60415E+00_JPRB, 0.60440E+00_JPRB, 0.60139E+00_JPRB, 0.59230E+00_JPRB, 0.58436E+00_JPRB ,&
&  0.57333E+00_JPRB, 0.56144E+00_JPRB, 0.55686E+00_JPRB, 0.55439E+00_JPRB, 0.55169E+00_JPRB, 0.55219E+00_JPRB, 0.55191E+00_JPRB ,&
&  0.55311E+00_JPRB, 0.55721E+00_JPRB, 0.56844E+00_JPRB, 0.57356E+00_JPRB, 0.57288E+00_JPRB, 0.56223E+00_JPRB, 0.56384E+00_JPRB ,&
&  0.56747E+00_JPRB, 0.56555E+00_JPRB, 0.56088E+00_JPRB, 0.56069E+00_JPRB, 0.56080E+00_JPRB, 0.56309E+00_JPRB, 0.57567E+00_JPRB ,&
&  0.60323E+00_JPRB, 0.62683E+00_JPRB, 0.64608E+00_JPRB, 0.65138E+00_JPRB, 0.64735E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.57102E+00_JPRB, 0.57654E+00_JPRB, 0.59021E+00_JPRB, 0.60149E+00_JPRB, 0.60524E+00_JPRB, 0.61511E+00_JPRB, 0.62615E+00_JPRB ,&
&  0.63450E+00_JPRB, 0.63561E+00_JPRB, 0.63153E+00_JPRB, 0.62145E+00_JPRB, 0.60965E+00_JPRB, 0.59987E+00_JPRB, 0.59613E+00_JPRB ,&
&  0.59836E+00_JPRB, 0.60136E+00_JPRB, 0.60693E+00_JPRB, 0.61014E+00_JPRB, 0.61414E+00_JPRB, 0.61332E+00_JPRB, 0.61425E+00_JPRB ,&
&  0.61654E+00_JPRB, 0.61787E+00_JPRB, 0.61842E+00_JPRB, 0.61452E+00_JPRB, 0.61129E+00_JPRB, 0.60837E+00_JPRB, 0.59916E+00_JPRB ,&
&  0.58914E+00_JPRB, 0.58285E+00_JPRB, 0.58679E+00_JPRB, 0.59777E+00_JPRB, 0.60452E+00_JPRB, 0.60100E+00_JPRB, 0.59909E+00_JPRB ,&
&  0.59941E+00_JPRB, 0.60549E+00_JPRB, 0.61526E+00_JPRB, 0.62858E+00_JPRB, 0.63676E+00_JPRB, 0.64728E+00_JPRB, 0.64783E+00_JPRB ,&
&  0.64136E+00_JPRB, 0.63182E+00_JPRB, 0.62859E+00_JPRB, 0.62677E+00_JPRB, 0.59552E+00_JPRB, 0.56113E+00_JPRB, 0.54437E+00_JPRB ,&
&  0.53531E+00_JPRB, 0.52435E+00_JPRB, 0.50844E+00_JPRB, 0.49754E+00_JPRB, 0.49677E+00_JPRB, 0.50212E+00_JPRB, 0.51383E+00_JPRB ,&
&  0.52225E+00_JPRB, 0.53508E+00_JPRB, 0.55133E+00_JPRB, 0.56223E+00_JPRB, 0.56268E+00_JPRB, 0.55005E+00_JPRB, 0.53798E+00_JPRB ,&
&  0.53061E+00_JPRB, 0.52826E+00_JPRB, 0.52652E+00_JPRB, 0.52454E+00_JPRB, 0.52275E+00_JPRB, 0.52216E+00_JPRB, 0.52127E+00_JPRB ,&
&  0.51712E+00_JPRB, 0.50989E+00_JPRB, 0.50120E+00_JPRB, 0.49662E+00_JPRB, 0.49659E+00_JPRB, 0.49755E+00_JPRB, 0.49988E+00_JPRB ,&
&  0.50879E+00_JPRB, 0.52006E+00_JPRB, 0.52869E+00_JPRB, 0.53158E+00_JPRB, 0.52735E+00_JPRB, 0.51845E+00_JPRB, 0.51314E+00_JPRB ,&
&  0.50939E+00_JPRB, 0.50679E+00_JPRB, 0.50332E+00_JPRB, 0.50203E+00_JPRB, 0.50237E+00_JPRB, 0.51085E+00_JPRB, 0.52805E+00_JPRB ,&
&  0.54857E+00_JPRB, 0.57454E+00_JPRB, 0.58600E+00_JPRB, 0.58426E+00_JPRB, 0.57716E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.53897E+00_JPRB, 0.55238E+00_JPRB, 0.56910E+00_JPRB, 0.58068E+00_JPRB, 0.58548E+00_JPRB, 0.59059E+00_JPRB, 0.59271E+00_JPRB ,&
&  0.58670E+00_JPRB, 0.58392E+00_JPRB, 0.57826E+00_JPRB, 0.57050E+00_JPRB, 0.56379E+00_JPRB, 0.56058E+00_JPRB, 0.56117E+00_JPRB ,&
&  0.56339E+00_JPRB, 0.56531E+00_JPRB, 0.56206E+00_JPRB, 0.56056E+00_JPRB, 0.56492E+00_JPRB, 0.57079E+00_JPRB, 0.57476E+00_JPRB ,&
&  0.57675E+00_JPRB, 0.57872E+00_JPRB, 0.58386E+00_JPRB, 0.58704E+00_JPRB, 0.58709E+00_JPRB, 0.58020E+00_JPRB, 0.57486E+00_JPRB ,&
&  0.56947E+00_JPRB, 0.56752E+00_JPRB, 0.57317E+00_JPRB, 0.56564E+00_JPRB, 0.57977E+00_JPRB, 0.58090E+00_JPRB, 0.58569E+00_JPRB ,&
&  0.59334E+00_JPRB, 0.60717E+00_JPRB, 0.61871E+00_JPRB, 0.62790E+00_JPRB, 0.63054E+00_JPRB, 0.63337E+00_JPRB, 0.62312E+00_JPRB ,&
&  0.60919E+00_JPRB, 0.59058E+00_JPRB, 0.57167E+00_JPRB, 0.55029E+00_JPRB, 0.52850E+00_JPRB, 0.51168E+00_JPRB, 0.50390E+00_JPRB ,&
&  0.49441E+00_JPRB, 0.48026E+00_JPRB, 0.46694E+00_JPRB, 0.46199E+00_JPRB, 0.46682E+00_JPRB, 0.47701E+00_JPRB, 0.49236E+00_JPRB ,&
&  0.50188E+00_JPRB, 0.50419E+00_JPRB, 0.50113E+00_JPRB, 0.49737E+00_JPRB, 0.49340E+00_JPRB, 0.48550E+00_JPRB, 0.48079E+00_JPRB ,&
&  0.47713E+00_JPRB, 0.47173E+00_JPRB, 0.46915E+00_JPRB, 0.47053E+00_JPRB, 0.47331E+00_JPRB, 0.47536E+00_JPRB, 0.47451E+00_JPRB ,&
&  0.47235E+00_JPRB, 0.46759E+00_JPRB, 0.45959E+00_JPRB, 0.45598E+00_JPRB, 0.45729E+00_JPRB, 0.46232E+00_JPRB, 0.46987E+00_JPRB ,&
&  0.47222E+00_JPRB, 0.47860E+00_JPRB, 0.48445E+00_JPRB, 0.48611E+00_JPRB, 0.48818E+00_JPRB, 0.48949E+00_JPRB, 0.48903E+00_JPRB ,&
&  0.48676E+00_JPRB, 0.48140E+00_JPRB, 0.47458E+00_JPRB, 0.47253E+00_JPRB, 0.47464E+00_JPRB, 0.48254E+00_JPRB, 0.49249E+00_JPRB ,&
&  0.50201E+00_JPRB, 0.51684E+00_JPRB, 0.52795E+00_JPRB, 0.53034E+00_JPRB, 0.53251E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.51620E+00_JPRB, 0.52825E+00_JPRB, 0.54221E+00_JPRB, 0.55159E+00_JPRB, 0.55028E+00_JPRB, 0.54986E+00_JPRB, 0.55547E+00_JPRB ,&
&  0.54826E+00_JPRB, 0.54033E+00_JPRB, 0.53423E+00_JPRB, 0.52838E+00_JPRB, 0.52674E+00_JPRB, 0.52597E+00_JPRB, 0.52946E+00_JPRB ,&
&  0.53207E+00_JPRB, 0.53137E+00_JPRB, 0.53017E+00_JPRB, 0.53145E+00_JPRB, 0.53867E+00_JPRB, 0.54756E+00_JPRB, 0.55504E+00_JPRB ,&
&  0.55694E+00_JPRB, 0.56109E+00_JPRB, 0.56747E+00_JPRB, 0.57290E+00_JPRB, 0.57554E+00_JPRB, 0.57693E+00_JPRB, 0.57538E+00_JPRB ,&
&  0.57234E+00_JPRB, 0.56845E+00_JPRB, 0.55123E+00_JPRB, 0.56312E+00_JPRB, 0.56765E+00_JPRB, 0.56802E+00_JPRB, 0.57210E+00_JPRB ,&
&  0.57936E+00_JPRB, 0.58863E+00_JPRB, 0.59337E+00_JPRB, 0.59517E+00_JPRB, 0.59036E+00_JPRB, 0.58059E+00_JPRB, 0.55563E+00_JPRB ,&
&  0.53471E+00_JPRB, 0.51328E+00_JPRB, 0.49981E+00_JPRB, 0.49119E+00_JPRB, 0.48402E+00_JPRB, 0.47807E+00_JPRB, 0.47312E+00_JPRB ,&
&  0.46621E+00_JPRB, 0.46014E+00_JPRB, 0.45641E+00_JPRB, 0.45612E+00_JPRB, 0.45987E+00_JPRB, 0.47040E+00_JPRB, 0.48337E+00_JPRB ,&
&  0.48868E+00_JPRB, 0.48005E+00_JPRB, 0.47155E+00_JPRB, 0.46195E+00_JPRB, 0.44525E+00_JPRB, 0.41927E+00_JPRB, 0.40278E+00_JPRB ,&
&  0.39939E+00_JPRB, 0.41196E+00_JPRB, 0.43207E+00_JPRB, 0.43867E+00_JPRB, 0.44491E+00_JPRB, 0.44235E+00_JPRB, 0.43668E+00_JPRB ,&
&  0.42608E+00_JPRB, 0.40410E+00_JPRB, 0.39362E+00_JPRB, 0.39109E+00_JPRB, 0.38958E+00_JPRB, 0.39109E+00_JPRB, 0.39419E+00_JPRB ,&
&  0.39916E+00_JPRB, 0.40571E+00_JPRB, 0.41022E+00_JPRB, 0.41186E+00_JPRB, 0.41447E+00_JPRB, 0.41472E+00_JPRB, 0.41350E+00_JPRB ,&
&  0.41027E+00_JPRB, 0.41797E+00_JPRB, 0.43751E+00_JPRB, 0.46455E+00_JPRB, 0.47133E+00_JPRB, 0.47981E+00_JPRB, 0.48870E+00_JPRB ,&
&  0.49430E+00_JPRB, 0.50031E+00_JPRB, 0.50143E+00_JPRB, 0.50210E+00_JPRB, 0.50717E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.48922E+00_JPRB, 0.49636E+00_JPRB, 0.50524E+00_JPRB, 0.51658E+00_JPRB, 0.51885E+00_JPRB, 0.52032E+00_JPRB, 0.52912E+00_JPRB ,&
&  0.52886E+00_JPRB, 0.52545E+00_JPRB, 0.52232E+00_JPRB, 0.52309E+00_JPRB, 0.52590E+00_JPRB, 0.52899E+00_JPRB, 0.52940E+00_JPRB ,&
&  0.52083E+00_JPRB, 0.51609E+00_JPRB, 0.51944E+00_JPRB, 0.52308E+00_JPRB, 0.52811E+00_JPRB, 0.53466E+00_JPRB, 0.54061E+00_JPRB ,&
&  0.54413E+00_JPRB, 0.54830E+00_JPRB, 0.55155E+00_JPRB, 0.55831E+00_JPRB, 0.56818E+00_JPRB, 0.57516E+00_JPRB, 0.58072E+00_JPRB ,&
&  0.56344E+00_JPRB, 0.51509E+00_JPRB, 0.48536E+00_JPRB, 0.49095E+00_JPRB, 0.53642E+00_JPRB, 0.54739E+00_JPRB, 0.54813E+00_JPRB ,&
&  0.55081E+00_JPRB, 0.55810E+00_JPRB, 0.55775E+00_JPRB, 0.55226E+00_JPRB, 0.54013E+00_JPRB, 0.52053E+00_JPRB, 0.49684E+00_JPRB ,&
&  0.48128E+00_JPRB, 0.47143E+00_JPRB, 0.46454E+00_JPRB, 0.45718E+00_JPRB, 0.45342E+00_JPRB, 0.44849E+00_JPRB, 0.44061E+00_JPRB ,&
&  0.43732E+00_JPRB, 0.43648E+00_JPRB, 0.43783E+00_JPRB, 0.43873E+00_JPRB, 0.44000E+00_JPRB, 0.44028E+00_JPRB, 0.43432E+00_JPRB ,&
&  0.42242E+00_JPRB, 0.40590E+00_JPRB, 0.39039E+00_JPRB, 0.37345E+00_JPRB, 0.35938E+00_JPRB, 0.35114E+00_JPRB, 0.34419E+00_JPRB ,&
&  0.34183E+00_JPRB, 0.34921E+00_JPRB, 0.36898E+00_JPRB, 0.39057E+00_JPRB, 0.41815E+00_JPRB, 0.39189E+00_JPRB, 0.36869E+00_JPRB ,&
&  0.35853E+00_JPRB, 0.35347E+00_JPRB, 0.35071E+00_JPRB, 0.35124E+00_JPRB, 0.35332E+00_JPRB, 0.35391E+00_JPRB, 0.35414E+00_JPRB ,&
&  0.35598E+00_JPRB, 0.35904E+00_JPRB, 0.36101E+00_JPRB, 0.36278E+00_JPRB, 0.36547E+00_JPRB, 0.36849E+00_JPRB, 0.37001E+00_JPRB ,&
&  0.37098E+00_JPRB, 0.37661E+00_JPRB, 0.38718E+00_JPRB, 0.39955E+00_JPRB, 0.41146E+00_JPRB, 0.42382E+00_JPRB, 0.44100E+00_JPRB ,&
&  0.46146E+00_JPRB, 0.48429E+00_JPRB, 0.48494E+00_JPRB, 0.48671E+00_JPRB, 0.48683E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.47328E+00_JPRB, 0.48489E+00_JPRB, 0.49048E+00_JPRB, 0.49612E+00_JPRB, 0.49864E+00_JPRB, 0.50014E+00_JPRB, 0.50818E+00_JPRB ,&
&  0.51309E+00_JPRB, 0.51715E+00_JPRB, 0.51986E+00_JPRB, 0.52195E+00_JPRB, 0.51885E+00_JPRB, 0.51287E+00_JPRB, 0.51177E+00_JPRB ,&
&  0.51087E+00_JPRB, 0.51145E+00_JPRB, 0.51072E+00_JPRB, 0.51082E+00_JPRB, 0.51139E+00_JPRB, 0.51614E+00_JPRB, 0.52342E+00_JPRB ,&
&  0.52520E+00_JPRB, 0.52936E+00_JPRB, 0.54295E+00_JPRB, 0.55071E+00_JPRB, 0.55563E+00_JPRB, 0.55450E+00_JPRB, 0.54793E+00_JPRB ,&
&  0.51778E+00_JPRB, 0.48892E+00_JPRB, 0.46317E+00_JPRB, 0.46426E+00_JPRB, 0.51447E+00_JPRB, 0.51967E+00_JPRB, 0.51984E+00_JPRB ,&
&  0.51841E+00_JPRB, 0.52229E+00_JPRB, 0.52250E+00_JPRB, 0.50763E+00_JPRB, 0.49687E+00_JPRB, 0.48268E+00_JPRB, 0.46988E+00_JPRB ,&
&  0.45797E+00_JPRB, 0.45061E+00_JPRB, 0.43661E+00_JPRB, 0.40803E+00_JPRB, 0.39171E+00_JPRB, 0.38337E+00_JPRB, 0.36954E+00_JPRB ,&
&  0.35881E+00_JPRB, 0.35623E+00_JPRB, 0.35555E+00_JPRB, 0.35371E+00_JPRB, 0.35745E+00_JPRB, 0.35960E+00_JPRB, 0.35738E+00_JPRB ,&
&  0.35258E+00_JPRB, 0.34868E+00_JPRB, 0.34431E+00_JPRB, 0.33871E+00_JPRB, 0.33589E+00_JPRB, 0.33536E+00_JPRB, 0.33642E+00_JPRB ,&
&  0.34151E+00_JPRB, 0.34852E+00_JPRB, 0.36353E+00_JPRB, 0.37375E+00_JPRB, 0.36749E+00_JPRB, 0.35497E+00_JPRB, 0.34499E+00_JPRB ,&
&  0.33777E+00_JPRB, 0.33498E+00_JPRB, 0.33281E+00_JPRB, 0.33043E+00_JPRB, 0.33118E+00_JPRB, 0.33364E+00_JPRB, 0.33507E+00_JPRB ,&
&  0.33518E+00_JPRB, 0.33743E+00_JPRB, 0.33889E+00_JPRB, 0.34042E+00_JPRB, 0.34171E+00_JPRB, 0.34381E+00_JPRB, 0.34668E+00_JPRB ,&
&  0.34914E+00_JPRB, 0.35482E+00_JPRB, 0.36090E+00_JPRB, 0.36794E+00_JPRB, 0.37245E+00_JPRB, 0.37589E+00_JPRB, 0.38346E+00_JPRB ,&
&  0.38968E+00_JPRB, 0.40540E+00_JPRB, 0.43788E+00_JPRB, 0.47187E+00_JPRB, 0.46239E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.46060E+00_JPRB, 0.46962E+00_JPRB, 0.47741E+00_JPRB, 0.48351E+00_JPRB, 0.48769E+00_JPRB, 0.48879E+00_JPRB, 0.49184E+00_JPRB ,&
&  0.49979E+00_JPRB, 0.50460E+00_JPRB, 0.50869E+00_JPRB, 0.50758E+00_JPRB, 0.50703E+00_JPRB, 0.48941E+00_JPRB, 0.48610E+00_JPRB ,&
&  0.47973E+00_JPRB, 0.47524E+00_JPRB, 0.46862E+00_JPRB, 0.46829E+00_JPRB, 0.46793E+00_JPRB, 0.47032E+00_JPRB, 0.47129E+00_JPRB ,&
&  0.47241E+00_JPRB, 0.47178E+00_JPRB, 0.46911E+00_JPRB, 0.46754E+00_JPRB, 0.46859E+00_JPRB, 0.46947E+00_JPRB, 0.46611E+00_JPRB ,&
&  0.46139E+00_JPRB, 0.45998E+00_JPRB, 0.46283E+00_JPRB, 0.46887E+00_JPRB, 0.50742E+00_JPRB, 0.50105E+00_JPRB, 0.49953E+00_JPRB ,&
&  0.49617E+00_JPRB, 0.49175E+00_JPRB, 0.49083E+00_JPRB, 0.48376E+00_JPRB, 0.47378E+00_JPRB, 0.44981E+00_JPRB, 0.43664E+00_JPRB ,&
&  0.40294E+00_JPRB, 0.38496E+00_JPRB, 0.36930E+00_JPRB, 0.35919E+00_JPRB, 0.34794E+00_JPRB, 0.34137E+00_JPRB, 0.33577E+00_JPRB ,&
&  0.33143E+00_JPRB, 0.32781E+00_JPRB, 0.32457E+00_JPRB, 0.32539E+00_JPRB, 0.32740E+00_JPRB, 0.32823E+00_JPRB, 0.32920E+00_JPRB ,&
&  0.32973E+00_JPRB, 0.33026E+00_JPRB, 0.33086E+00_JPRB, 0.33246E+00_JPRB, 0.33463E+00_JPRB, 0.33752E+00_JPRB, 0.34124E+00_JPRB ,&
&  0.34714E+00_JPRB, 0.35157E+00_JPRB, 0.35292E+00_JPRB, 0.35605E+00_JPRB, 0.35571E+00_JPRB, 0.34886E+00_JPRB, 0.34150E+00_JPRB ,&
&  0.33364E+00_JPRB, 0.32955E+00_JPRB, 0.32544E+00_JPRB, 0.32248E+00_JPRB, 0.32172E+00_JPRB, 0.32157E+00_JPRB, 0.32372E+00_JPRB ,&
&  0.32613E+00_JPRB, 0.32838E+00_JPRB, 0.32999E+00_JPRB, 0.33166E+00_JPRB, 0.33382E+00_JPRB, 0.33532E+00_JPRB, 0.33729E+00_JPRB ,&
&  0.33988E+00_JPRB, 0.34336E+00_JPRB, 0.34853E+00_JPRB, 0.35460E+00_JPRB, 0.36294E+00_JPRB, 0.37466E+00_JPRB, 0.39434E+00_JPRB ,&
&  0.41886E+00_JPRB, 0.47204E+00_JPRB, 0.47468E+00_JPRB, 0.44722E+00_JPRB, 0.45399E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.43664E+00_JPRB, 0.44417E+00_JPRB, 0.45124E+00_JPRB, 0.45891E+00_JPRB, 0.46854E+00_JPRB, 0.47288E+00_JPRB, 0.47485E+00_JPRB ,&
&  0.47044E+00_JPRB, 0.46652E+00_JPRB, 0.45619E+00_JPRB, 0.44772E+00_JPRB, 0.43866E+00_JPRB, 0.43160E+00_JPRB, 0.42551E+00_JPRB ,&
&  0.42125E+00_JPRB, 0.41805E+00_JPRB, 0.41261E+00_JPRB, 0.40896E+00_JPRB, 0.40545E+00_JPRB, 0.40586E+00_JPRB, 0.40672E+00_JPRB ,&
&  0.40681E+00_JPRB, 0.40771E+00_JPRB, 0.40613E+00_JPRB, 0.41131E+00_JPRB, 0.42077E+00_JPRB, 0.43189E+00_JPRB, 0.44643E+00_JPRB ,&
&  0.46668E+00_JPRB, 0.47728E+00_JPRB, 0.49271E+00_JPRB, 0.49238E+00_JPRB, 0.49059E+00_JPRB, 0.48692E+00_JPRB, 0.48377E+00_JPRB ,&
&  0.47891E+00_JPRB, 0.47189E+00_JPRB, 0.46646E+00_JPRB, 0.45932E+00_JPRB, 0.42846E+00_JPRB, 0.40516E+00_JPRB, 0.38708E+00_JPRB ,&
&  0.37226E+00_JPRB, 0.35821E+00_JPRB, 0.34542E+00_JPRB, 0.33911E+00_JPRB, 0.33345E+00_JPRB, 0.33008E+00_JPRB, 0.32651E+00_JPRB ,&
&  0.32264E+00_JPRB, 0.32166E+00_JPRB, 0.31978E+00_JPRB, 0.31900E+00_JPRB, 0.32025E+00_JPRB, 0.32000E+00_JPRB, 0.32079E+00_JPRB ,&
&  0.32160E+00_JPRB, 0.32311E+00_JPRB, 0.32649E+00_JPRB, 0.32900E+00_JPRB, 0.33152E+00_JPRB, 0.33382E+00_JPRB, 0.33478E+00_JPRB ,&
&  0.33691E+00_JPRB, 0.33883E+00_JPRB, 0.33872E+00_JPRB, 0.33782E+00_JPRB, 0.33572E+00_JPRB, 0.33074E+00_JPRB, 0.32589E+00_JPRB ,&
&  0.32063E+00_JPRB, 0.31682E+00_JPRB, 0.31560E+00_JPRB, 0.31444E+00_JPRB, 0.31547E+00_JPRB, 0.31624E+00_JPRB, 0.31695E+00_JPRB ,&
&  0.31896E+00_JPRB, 0.32090E+00_JPRB, 0.32331E+00_JPRB, 0.32695E+00_JPRB, 0.32955E+00_JPRB, 0.33169E+00_JPRB, 0.33430E+00_JPRB ,&
&  0.33384E+00_JPRB, 0.33844E+00_JPRB, 0.34429E+00_JPRB, 0.35278E+00_JPRB, 0.36889E+00_JPRB, 0.37974E+00_JPRB, 0.40364E+00_JPRB ,&
&  0.43611E+00_JPRB, 0.49609E+00_JPRB, 0.48757E+00_JPRB, 0.42365E+00_JPRB, 0.42957E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.42304E+00_JPRB, 0.42697E+00_JPRB, 0.43034E+00_JPRB, 0.43463E+00_JPRB, 0.43979E+00_JPRB, 0.44465E+00_JPRB, 0.44837E+00_JPRB ,&
&  0.45099E+00_JPRB, 0.44929E+00_JPRB, 0.45082E+00_JPRB, 0.44556E+00_JPRB, 0.44033E+00_JPRB, 0.43337E+00_JPRB, 0.42688E+00_JPRB ,&
&  0.42060E+00_JPRB, 0.41141E+00_JPRB, 0.40170E+00_JPRB, 0.39323E+00_JPRB, 0.38596E+00_JPRB, 0.38063E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.37390E+00_JPRB, 0.37498E+00_JPRB, 0.37708E+00_JPRB, 0.38323E+00_JPRB, 0.39348E+00_JPRB, 0.40567E+00_JPRB, 0.42462E+00_JPRB ,&
&  0.45079E+00_JPRB, 0.45286E+00_JPRB, 0.45953E+00_JPRB, 0.45874E+00_JPRB, 0.45459E+00_JPRB, 0.45313E+00_JPRB, 0.45046E+00_JPRB ,&
&  0.45127E+00_JPRB, 0.44141E+00_JPRB, 0.45090E+00_JPRB, 0.43201E+00_JPRB, 0.40274E+00_JPRB, 0.38736E+00_JPRB, 0.36865E+00_JPRB ,&
&  0.35602E+00_JPRB, 0.34517E+00_JPRB, 0.33716E+00_JPRB, 0.33440E+00_JPRB, 0.33004E+00_JPRB, 0.32807E+00_JPRB, 0.32364E+00_JPRB ,&
&  0.31927E+00_JPRB, 0.31753E+00_JPRB, 0.31420E+00_JPRB, 0.31432E+00_JPRB, 0.31463E+00_JPRB, 0.31397E+00_JPRB, 0.31410E+00_JPRB ,&
&  0.31460E+00_JPRB, 0.31605E+00_JPRB, 0.31867E+00_JPRB, 0.32101E+00_JPRB, 0.32242E+00_JPRB, 0.32321E+00_JPRB, 0.32297E+00_JPRB ,&
&  0.32355E+00_JPRB, 0.32317E+00_JPRB, 0.32215E+00_JPRB, 0.32065E+00_JPRB, 0.31822E+00_JPRB, 0.31566E+00_JPRB, 0.31314E+00_JPRB ,&
&  0.31102E+00_JPRB, 0.31007E+00_JPRB, 0.30916E+00_JPRB, 0.30916E+00_JPRB, 0.30958E+00_JPRB, 0.31145E+00_JPRB, 0.31319E+00_JPRB ,&
&  0.31523E+00_JPRB, 0.31828E+00_JPRB, 0.32089E+00_JPRB, 0.32412E+00_JPRB, 0.32695E+00_JPRB, 0.32822E+00_JPRB, 0.33262E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.34660E+00_JPRB, 0.35633E+00_JPRB, 0.36530E+00_JPRB, 0.37660E+00_JPRB, 0.39200E+00_JPRB, 0.41220E+00_JPRB ,&
&  0.43193E+00_JPRB, 0.45041E+00_JPRB, 0.44222E+00_JPRB, 0.41775E+00_JPRB, 0.41864E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.41877E+00_JPRB, 0.41806E+00_JPRB, 0.41945E+00_JPRB, 0.42141E+00_JPRB, 0.42661E+00_JPRB, 0.43310E+00_JPRB, 0.44003E+00_JPRB ,&
&  0.43999E+00_JPRB, 0.44363E+00_JPRB, 0.44231E+00_JPRB, 0.43874E+00_JPRB, 0.43309E+00_JPRB, 0.42682E+00_JPRB, 0.42203E+00_JPRB ,&
&  0.41538E+00_JPRB, 0.40681E+00_JPRB, 0.39854E+00_JPRB, 0.38991E+00_JPRB, 0.38189E+00_JPRB, 0.37359E+00_JPRB, 0.36951E+00_JPRB ,&
&  0.36606E+00_JPRB, 0.36527E+00_JPRB, 0.36734E+00_JPRB, 0.36968E+00_JPRB, 0.37494E+00_JPRB, 0.38220E+00_JPRB, 0.39191E+00_JPRB ,&
&  0.40105E+00_JPRB, 0.41216E+00_JPRB, 0.41812E+00_JPRB, 0.41795E+00_JPRB, 0.41788E+00_JPRB, 0.41594E+00_JPRB, 0.41257E+00_JPRB ,&
&  0.40560E+00_JPRB, 0.39800E+00_JPRB, 0.38893E+00_JPRB, 0.37934E+00_JPRB, 0.36951E+00_JPRB, 0.36082E+00_JPRB, 0.35272E+00_JPRB ,&
&  0.34618E+00_JPRB, 0.34062E+00_JPRB, 0.33536E+00_JPRB, 0.33090E+00_JPRB, 0.32747E+00_JPRB, 0.32498E+00_JPRB, 0.32163E+00_JPRB ,&
&  0.31896E+00_JPRB, 0.31656E+00_JPRB, 0.31365E+00_JPRB, 0.31173E+00_JPRB, 0.30997E+00_JPRB, 0.30885E+00_JPRB, 0.30790E+00_JPRB ,&
&  0.30803E+00_JPRB, 0.30819E+00_JPRB, 0.30925E+00_JPRB, 0.31031E+00_JPRB, 0.31146E+00_JPRB, 0.31161E+00_JPRB, 0.31150E+00_JPRB ,&
&  0.31111E+00_JPRB, 0.31069E+00_JPRB, 0.30890E+00_JPRB, 0.30748E+00_JPRB, 0.30644E+00_JPRB, 0.30530E+00_JPRB, 0.30509E+00_JPRB ,&
&  0.30585E+00_JPRB, 0.30664E+00_JPRB, 0.30718E+00_JPRB, 0.30891E+00_JPRB, 0.31061E+00_JPRB, 0.31317E+00_JPRB, 0.31633E+00_JPRB ,&
&  0.31857E+00_JPRB, 0.32123E+00_JPRB, 0.32417E+00_JPRB, 0.32758E+00_JPRB, 0.33175E+00_JPRB, 0.33600E+00_JPRB, 0.34110E+00_JPRB ,&
&  0.34689E+00_JPRB, 0.35271E+00_JPRB, 0.35927E+00_JPRB, 0.36657E+00_JPRB, 0.37542E+00_JPRB, 0.38068E+00_JPRB, 0.38932E+00_JPRB ,&
&  0.39531E+00_JPRB, 0.38915E+00_JPRB, 0.42926E+00_JPRB, 0.42273E+00_JPRB, 0.41900E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.36343E+00_JPRB, 0.36983E+00_JPRB, 0.37378E+00_JPRB, 0.37736E+00_JPRB, 0.36698E+00_JPRB, 0.40410E+00_JPRB, 0.40200E+00_JPRB ,&
&  0.40327E+00_JPRB, 0.39318E+00_JPRB, 0.39589E+00_JPRB, 0.39586E+00_JPRB, 0.39388E+00_JPRB, 0.39067E+00_JPRB, 0.38757E+00_JPRB ,&
&  0.38479E+00_JPRB, 0.38050E+00_JPRB, 0.37522E+00_JPRB, 0.37058E+00_JPRB, 0.36702E+00_JPRB, 0.36400E+00_JPRB, 0.36135E+00_JPRB ,&
&  0.36072E+00_JPRB, 0.36208E+00_JPRB, 0.36462E+00_JPRB, 0.36230E+00_JPRB, 0.36256E+00_JPRB, 0.36387E+00_JPRB, 0.36576E+00_JPRB ,&
&  0.36741E+00_JPRB, 0.36904E+00_JPRB, 0.36899E+00_JPRB, 0.36708E+00_JPRB, 0.36663E+00_JPRB, 0.36470E+00_JPRB, 0.36280E+00_JPRB ,&
&  0.36026E+00_JPRB, 0.35749E+00_JPRB, 0.35414E+00_JPRB, 0.35148E+00_JPRB, 0.34848E+00_JPRB, 0.34481E+00_JPRB, 0.34129E+00_JPRB ,&
&  0.33828E+00_JPRB, 0.33555E+00_JPRB, 0.33232E+00_JPRB, 0.32933E+00_JPRB, 0.32646E+00_JPRB, 0.32354E+00_JPRB, 0.32084E+00_JPRB ,&
&  0.31877E+00_JPRB, 0.31703E+00_JPRB, 0.31532E+00_JPRB, 0.31351E+00_JPRB, 0.31180E+00_JPRB, 0.31014E+00_JPRB, 0.30886E+00_JPRB ,&
&  0.30886E+00_JPRB, 0.30818E+00_JPRB, 0.30753E+00_JPRB, 0.30730E+00_JPRB, 0.30757E+00_JPRB, 0.30709E+00_JPRB, 0.30693E+00_JPRB ,&
&  0.30691E+00_JPRB, 0.30697E+00_JPRB, 0.30721E+00_JPRB, 0.30752E+00_JPRB, 0.30797E+00_JPRB, 0.30856E+00_JPRB, 0.30953E+00_JPRB ,&
&  0.31068E+00_JPRB, 0.31209E+00_JPRB, 0.31362E+00_JPRB, 0.31517E+00_JPRB, 0.31675E+00_JPRB, 0.31843E+00_JPRB, 0.32033E+00_JPRB ,&
&  0.32221E+00_JPRB, 0.32403E+00_JPRB, 0.32577E+00_JPRB, 0.32749E+00_JPRB, 0.33053E+00_JPRB, 0.33341E+00_JPRB, 0.33648E+00_JPRB ,&
&  0.33981E+00_JPRB, 0.34375E+00_JPRB, 0.34792E+00_JPRB, 0.35249E+00_JPRB, 0.35774E+00_JPRB, 0.35850E+00_JPRB, 0.35900E+00_JPRB ,&
&  0.35899E+00_JPRB, 0.36210E+00_JPRB, 0.36297E+00_JPRB, 0.36165E+00_JPRB, 0.36058E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.33770E+00_JPRB, 0.33761E+00_JPRB, 0.33743E+00_JPRB, 0.33715E+00_JPRB, 0.33500E+00_JPRB, 0.33511E+00_JPRB, 0.33535E+00_JPRB ,&
&  0.33605E+00_JPRB, 0.33791E+00_JPRB, 0.34002E+00_JPRB, 0.34158E+00_JPRB, 0.34266E+00_JPRB, 0.34337E+00_JPRB, 0.34394E+00_JPRB ,&
&  0.34409E+00_JPRB, 0.34450E+00_JPRB, 0.34035E+00_JPRB, 0.33882E+00_JPRB, 0.33823E+00_JPRB, 0.33812E+00_JPRB, 0.33829E+00_JPRB ,&
&  0.33835E+00_JPRB, 0.33825E+00_JPRB, 0.33752E+00_JPRB, 0.33788E+00_JPRB, 0.33828E+00_JPRB, 0.33852E+00_JPRB, 0.33878E+00_JPRB ,&
&  0.33904E+00_JPRB, 0.33929E+00_JPRB, 0.33924E+00_JPRB, 0.33899E+00_JPRB, 0.33893E+00_JPRB, 0.33851E+00_JPRB, 0.33769E+00_JPRB ,&
&  0.33665E+00_JPRB, 0.33532E+00_JPRB, 0.33400E+00_JPRB, 0.33279E+00_JPRB, 0.33175E+00_JPRB, 0.33087E+00_JPRB, 0.33001E+00_JPRB ,&
&  0.32892E+00_JPRB, 0.32746E+00_JPRB, 0.32591E+00_JPRB, 0.32438E+00_JPRB, 0.32291E+00_JPRB, 0.32157E+00_JPRB, 0.32013E+00_JPRB ,&
&  0.31906E+00_JPRB, 0.31815E+00_JPRB, 0.31727E+00_JPRB, 0.31651E+00_JPRB, 0.31575E+00_JPRB, 0.31502E+00_JPRB, 0.31431E+00_JPRB ,&
&  0.31331E+00_JPRB, 0.31282E+00_JPRB, 0.31226E+00_JPRB, 0.31205E+00_JPRB, 0.31220E+00_JPRB, 0.31232E+00_JPRB, 0.31255E+00_JPRB ,&
&  0.31286E+00_JPRB, 0.31321E+00_JPRB, 0.31338E+00_JPRB, 0.31364E+00_JPRB, 0.31401E+00_JPRB, 0.31449E+00_JPRB, 0.31506E+00_JPRB ,&
&  0.31567E+00_JPRB, 0.31634E+00_JPRB, 0.31707E+00_JPRB, 0.31789E+00_JPRB, 0.31874E+00_JPRB, 0.31967E+00_JPRB, 0.32056E+00_JPRB ,&
&  0.32146E+00_JPRB, 0.32230E+00_JPRB, 0.32315E+00_JPRB, 0.32380E+00_JPRB, 0.32480E+00_JPRB, 0.32575E+00_JPRB, 0.32689E+00_JPRB ,&
&  0.32803E+00_JPRB, 0.32926E+00_JPRB, 0.33034E+00_JPRB, 0.33152E+00_JPRB, 0.33284E+00_JPRB, 0.33248E+00_JPRB, 0.33228E+00_JPRB ,&
&  0.33213E+00_JPRB, 0.33238E+00_JPRB, 0.33216E+00_JPRB, 0.33243E+00_JPRB, 0.33336E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB, 0.32010E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB, 0.73290E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.74656E+00_JPRB, 0.74679E+00_JPRB, 0.74697E+00_JPRB, 0.74691E+00_JPRB, 0.74677E+00_JPRB, 0.74681E+00_JPRB, 0.74679E+00_JPRB ,&
&  0.74673E+00_JPRB, 0.74662E+00_JPRB, 0.74683E+00_JPRB, 0.74660E+00_JPRB, 0.74637E+00_JPRB, 0.74588E+00_JPRB, 0.74541E+00_JPRB ,&
&  0.74487E+00_JPRB, 0.74438E+00_JPRB, 0.74382E+00_JPRB, 0.74342E+00_JPRB, 0.74313E+00_JPRB, 0.74307E+00_JPRB, 0.74297E+00_JPRB ,&
&  0.74267E+00_JPRB, 0.74202E+00_JPRB, 0.74154E+00_JPRB, 0.74068E+00_JPRB, 0.73964E+00_JPRB, 0.73806E+00_JPRB, 0.73621E+00_JPRB ,&
&  0.73387E+00_JPRB, 0.73170E+00_JPRB, 0.72993E+00_JPRB, 0.72839E+00_JPRB, 0.72717E+00_JPRB, 0.72626E+00_JPRB, 0.72555E+00_JPRB ,&
&  0.72469E+00_JPRB, 0.72412E+00_JPRB, 0.72360E+00_JPRB, 0.72313E+00_JPRB, 0.72228E+00_JPRB, 0.72149E+00_JPRB, 0.72064E+00_JPRB ,&
&  0.71974E+00_JPRB, 0.71861E+00_JPRB, 0.71781E+00_JPRB, 0.71703E+00_JPRB, 0.71610E+00_JPRB, 0.71445E+00_JPRB, 0.71292E+00_JPRB ,&
&  0.71150E+00_JPRB, 0.71000E+00_JPRB, 0.70836E+00_JPRB, 0.70684E+00_JPRB, 0.70573E+00_JPRB, 0.70470E+00_JPRB, 0.70358E+00_JPRB ,&
&  0.70334E+00_JPRB, 0.70399E+00_JPRB, 0.70463E+00_JPRB, 0.70502E+00_JPRB, 0.70589E+00_JPRB, 0.70691E+00_JPRB, 0.70803E+00_JPRB ,&
&  0.70911E+00_JPRB, 0.71049E+00_JPRB, 0.71242E+00_JPRB, 0.71420E+00_JPRB, 0.71616E+00_JPRB, 0.71815E+00_JPRB, 0.72040E+00_JPRB ,&
&  0.72248E+00_JPRB, 0.72457E+00_JPRB, 0.72643E+00_JPRB, 0.72850E+00_JPRB, 0.72989E+00_JPRB, 0.73126E+00_JPRB, 0.73223E+00_JPRB ,&
&  0.73327E+00_JPRB, 0.73408E+00_JPRB, 0.73499E+00_JPRB, 0.73575E+00_JPRB, 0.73677E+00_JPRB, 0.73762E+00_JPRB, 0.73850E+00_JPRB ,&
&  0.73935E+00_JPRB, 0.74032E+00_JPRB, 0.74111E+00_JPRB, 0.74184E+00_JPRB, 0.74240E+00_JPRB, 0.74321E+00_JPRB, 0.74384E+00_JPRB ,&
&  0.74441E+00_JPRB, 0.74501E+00_JPRB, 0.74559E+00_JPRB, 0.74613E+00_JPRB, 0.74645E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.74472E+00_JPRB, 0.74702E+00_JPRB, 0.74825E+00_JPRB, 0.74941E+00_JPRB, 0.75027E+00_JPRB, 0.75178E+00_JPRB, 0.75283E+00_JPRB ,&
&  0.75382E+00_JPRB, 0.75519E+00_JPRB, 0.75737E+00_JPRB, 0.75834E+00_JPRB, 0.75892E+00_JPRB, 0.75886E+00_JPRB, 0.75870E+00_JPRB ,&
&  0.75801E+00_JPRB, 0.75691E+00_JPRB, 0.75574E+00_JPRB, 0.75474E+00_JPRB, 0.75387E+00_JPRB, 0.75324E+00_JPRB, 0.75219E+00_JPRB ,&
&  0.75025E+00_JPRB, 0.74782E+00_JPRB, 0.74511E+00_JPRB, 0.74073E+00_JPRB, 0.73582E+00_JPRB, 0.73066E+00_JPRB, 0.72521E+00_JPRB ,&
&  0.71919E+00_JPRB, 0.71456E+00_JPRB, 0.71028E+00_JPRB, 0.70629E+00_JPRB, 0.70495E+00_JPRB, 0.70348E+00_JPRB, 0.70196E+00_JPRB ,&
&  0.70045E+00_JPRB, 0.69784E+00_JPRB, 0.69699E+00_JPRB, 0.69647E+00_JPRB, 0.69641E+00_JPRB, 0.69749E+00_JPRB, 0.69824E+00_JPRB ,&
&  0.69873E+00_JPRB, 0.69845E+00_JPRB, 0.69801E+00_JPRB, 0.69834E+00_JPRB, 0.69921E+00_JPRB, 0.69925E+00_JPRB, 0.69798E+00_JPRB ,&
&  0.69697E+00_JPRB, 0.69529E+00_JPRB, 0.69332E+00_JPRB, 0.69101E+00_JPRB, 0.68990E+00_JPRB, 0.68835E+00_JPRB, 0.68658E+00_JPRB ,&
&  0.68552E+00_JPRB, 0.68543E+00_JPRB, 0.68562E+00_JPRB, 0.68600E+00_JPRB, 0.68721E+00_JPRB, 0.68875E+00_JPRB, 0.69029E+00_JPRB ,&
&  0.69194E+00_JPRB, 0.69473E+00_JPRB, 0.69785E+00_JPRB, 0.70076E+00_JPRB, 0.70386E+00_JPRB, 0.70711E+00_JPRB, 0.71063E+00_JPRB ,&
&  0.71351E+00_JPRB, 0.71575E+00_JPRB, 0.71689E+00_JPRB, 0.71899E+00_JPRB, 0.72092E+00_JPRB, 0.72346E+00_JPRB, 0.72551E+00_JPRB ,&
&  0.72817E+00_JPRB, 0.73002E+00_JPRB, 0.73126E+00_JPRB, 0.73250E+00_JPRB, 0.73371E+00_JPRB, 0.73461E+00_JPRB, 0.73512E+00_JPRB ,&
&  0.73539E+00_JPRB, 0.73574E+00_JPRB, 0.73594E+00_JPRB, 0.73597E+00_JPRB, 0.73610E+00_JPRB, 0.73636E+00_JPRB, 0.73669E+00_JPRB ,&
&  0.73735E+00_JPRB, 0.73872E+00_JPRB, 0.74054E+00_JPRB, 0.74181E+00_JPRB, 0.74312E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.73539E+00_JPRB, 0.73727E+00_JPRB, 0.73872E+00_JPRB, 0.74110E+00_JPRB, 0.74531E+00_JPRB, 0.74959E+00_JPRB, 0.75192E+00_JPRB ,&
&  0.75517E+00_JPRB, 0.75899E+00_JPRB, 0.76342E+00_JPRB, 0.76593E+00_JPRB, 0.76779E+00_JPRB, 0.76848E+00_JPRB, 0.76806E+00_JPRB ,&
&  0.76650E+00_JPRB, 0.76429E+00_JPRB, 0.76284E+00_JPRB, 0.76119E+00_JPRB, 0.75925E+00_JPRB, 0.75706E+00_JPRB, 0.75126E+00_JPRB ,&
&  0.74617E+00_JPRB, 0.74269E+00_JPRB, 0.73557E+00_JPRB, 0.72730E+00_JPRB, 0.72073E+00_JPRB, 0.67462E+00_JPRB, 0.65944E+00_JPRB ,&
&  0.65837E+00_JPRB, 0.65437E+00_JPRB, 0.65550E+00_JPRB, 0.65742E+00_JPRB, 0.66319E+00_JPRB, 0.65566E+00_JPRB, 0.65061E+00_JPRB ,&
&  0.62246E+00_JPRB, 0.62333E+00_JPRB, 0.61886E+00_JPRB, 0.61653E+00_JPRB, 0.61948E+00_JPRB, 0.62147E+00_JPRB, 0.63076E+00_JPRB ,&
&  0.65610E+00_JPRB, 0.66310E+00_JPRB, 0.66642E+00_JPRB, 0.66941E+00_JPRB, 0.67347E+00_JPRB, 0.67612E+00_JPRB, 0.67819E+00_JPRB ,&
&  0.67906E+00_JPRB, 0.67893E+00_JPRB, 0.67911E+00_JPRB, 0.67843E+00_JPRB, 0.67780E+00_JPRB, 0.67638E+00_JPRB, 0.67549E+00_JPRB ,&
&  0.67493E+00_JPRB, 0.67596E+00_JPRB, 0.67778E+00_JPRB, 0.67969E+00_JPRB, 0.68239E+00_JPRB, 0.68426E+00_JPRB, 0.68603E+00_JPRB ,&
&  0.68787E+00_JPRB, 0.68962E+00_JPRB, 0.69159E+00_JPRB, 0.69370E+00_JPRB, 0.69633E+00_JPRB, 0.69806E+00_JPRB, 0.70046E+00_JPRB ,&
&  0.70246E+00_JPRB, 0.70501E+00_JPRB, 0.70722E+00_JPRB, 0.71198E+00_JPRB, 0.71583E+00_JPRB, 0.72113E+00_JPRB, 0.72580E+00_JPRB ,&
&  0.73104E+00_JPRB, 0.73455E+00_JPRB, 0.73762E+00_JPRB, 0.73980E+00_JPRB, 0.74219E+00_JPRB, 0.74400E+00_JPRB, 0.74479E+00_JPRB ,&
&  0.74492E+00_JPRB, 0.74384E+00_JPRB, 0.74248E+00_JPRB, 0.74042E+00_JPRB, 0.73855E+00_JPRB, 0.73609E+00_JPRB, 0.73398E+00_JPRB ,&
&  0.73138E+00_JPRB, 0.73064E+00_JPRB, 0.73104E+00_JPRB, 0.73248E+00_JPRB, 0.73383E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.74326E+00_JPRB, 0.74292E+00_JPRB, 0.74251E+00_JPRB, 0.74522E+00_JPRB, 0.74868E+00_JPRB, 0.75472E+00_JPRB, 0.75613E+00_JPRB ,&
&  0.76130E+00_JPRB, 0.76659E+00_JPRB, 0.77263E+00_JPRB, 0.77529E+00_JPRB, 0.77756E+00_JPRB, 0.77701E+00_JPRB, 0.77483E+00_JPRB ,&
&  0.77246E+00_JPRB, 0.76833E+00_JPRB, 0.76569E+00_JPRB, 0.76161E+00_JPRB, 0.75599E+00_JPRB, 0.75194E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.73976E+00_JPRB, 0.73376E+00_JPRB, 0.69002E+00_JPRB, 0.67761E+00_JPRB, 0.65623E+00_JPRB, 0.64359E+00_JPRB, 0.63712E+00_JPRB ,&
&  0.62817E+00_JPRB, 0.61940E+00_JPRB, 0.60730E+00_JPRB, 0.59291E+00_JPRB, 0.57768E+00_JPRB, 0.55568E+00_JPRB, 0.54615E+00_JPRB ,&
&  0.53684E+00_JPRB, 0.53500E+00_JPRB, 0.53703E+00_JPRB, 0.54260E+00_JPRB, 0.55487E+00_JPRB, 0.56725E+00_JPRB, 0.58304E+00_JPRB ,&
&  0.60512E+00_JPRB, 0.62720E+00_JPRB, 0.63829E+00_JPRB, 0.63703E+00_JPRB, 0.65568E+00_JPRB, 0.66515E+00_JPRB, 0.67063E+00_JPRB ,&
&  0.67301E+00_JPRB, 0.67123E+00_JPRB, 0.67752E+00_JPRB, 0.67768E+00_JPRB, 0.67773E+00_JPRB, 0.67825E+00_JPRB, 0.67960E+00_JPRB ,&
&  0.68279E+00_JPRB, 0.68616E+00_JPRB, 0.68882E+00_JPRB, 0.69122E+00_JPRB, 0.69273E+00_JPRB, 0.69324E+00_JPRB, 0.69367E+00_JPRB ,&
&  0.69437E+00_JPRB, 0.69520E+00_JPRB, 0.69624E+00_JPRB, 0.69735E+00_JPRB, 0.69930E+00_JPRB, 0.70054E+00_JPRB, 0.70139E+00_JPRB ,&
&  0.70351E+00_JPRB, 0.70677E+00_JPRB, 0.71047E+00_JPRB, 0.71018E+00_JPRB, 0.71475E+00_JPRB, 0.72578E+00_JPRB, 0.73199E+00_JPRB ,&
&  0.74138E+00_JPRB, 0.74831E+00_JPRB, 0.75615E+00_JPRB, 0.76230E+00_JPRB, 0.76765E+00_JPRB, 0.77012E+00_JPRB, 0.77015E+00_JPRB ,&
&  0.76943E+00_JPRB, 0.76739E+00_JPRB, 0.76591E+00_JPRB, 0.76246E+00_JPRB, 0.75940E+00_JPRB, 0.75480E+00_JPRB, 0.75124E+00_JPRB ,&
&  0.74653E+00_JPRB, 0.74428E+00_JPRB, 0.74008E+00_JPRB, 0.74089E+00_JPRB, 0.74151E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.75580E+00_JPRB, 0.75422E+00_JPRB, 0.75532E+00_JPRB, 0.76119E+00_JPRB, 0.77194E+00_JPRB, 0.78432E+00_JPRB, 0.78447E+00_JPRB ,&
&  0.78532E+00_JPRB, 0.78419E+00_JPRB, 0.78731E+00_JPRB, 0.78710E+00_JPRB, 0.78639E+00_JPRB, 0.78323E+00_JPRB, 0.78062E+00_JPRB ,&
&  0.77807E+00_JPRB, 0.77094E+00_JPRB, 0.76169E+00_JPRB, 0.75653E+00_JPRB, 0.75420E+00_JPRB, 0.75289E+00_JPRB, 0.74643E+00_JPRB ,&
&  0.73842E+00_JPRB, 0.72147E+00_JPRB, 0.68891E+00_JPRB, 0.66905E+00_JPRB, 0.64632E+00_JPRB, 0.63794E+00_JPRB, 0.62365E+00_JPRB ,&
&  0.62559E+00_JPRB, 0.57663E+00_JPRB, 0.57903E+00_JPRB, 0.55852E+00_JPRB, 0.54462E+00_JPRB, 0.52885E+00_JPRB, 0.51055E+00_JPRB ,&
&  0.50518E+00_JPRB, 0.49824E+00_JPRB, 0.49705E+00_JPRB, 0.50602E+00_JPRB, 0.51730E+00_JPRB, 0.53661E+00_JPRB, 0.55699E+00_JPRB ,&
&  0.56983E+00_JPRB, 0.62117E+00_JPRB, 0.63666E+00_JPRB, 0.64303E+00_JPRB, 0.65983E+00_JPRB, 0.67131E+00_JPRB, 0.67287E+00_JPRB ,&
&  0.67425E+00_JPRB, 0.67428E+00_JPRB, 0.68239E+00_JPRB, 0.68755E+00_JPRB, 0.69523E+00_JPRB, 0.69592E+00_JPRB, 0.69696E+00_JPRB ,&
&  0.70095E+00_JPRB, 0.70580E+00_JPRB, 0.71110E+00_JPRB, 0.71597E+00_JPRB, 0.72016E+00_JPRB, 0.72284E+00_JPRB, 0.72393E+00_JPRB ,&
&  0.72409E+00_JPRB, 0.72313E+00_JPRB, 0.72288E+00_JPRB, 0.72386E+00_JPRB, 0.72491E+00_JPRB, 0.72792E+00_JPRB, 0.72980E+00_JPRB ,&
&  0.73194E+00_JPRB, 0.73254E+00_JPRB, 0.73167E+00_JPRB, 0.73289E+00_JPRB, 0.73780E+00_JPRB, 0.74277E+00_JPRB, 0.75187E+00_JPRB ,&
&  0.76466E+00_JPRB, 0.77253E+00_JPRB, 0.78098E+00_JPRB, 0.78837E+00_JPRB, 0.79476E+00_JPRB, 0.79605E+00_JPRB, 0.79479E+00_JPRB ,&
&  0.79325E+00_JPRB, 0.79124E+00_JPRB, 0.78883E+00_JPRB, 0.78428E+00_JPRB, 0.77942E+00_JPRB, 0.77229E+00_JPRB, 0.76590E+00_JPRB ,&
&  0.75891E+00_JPRB, 0.75623E+00_JPRB, 0.75080E+00_JPRB, 0.75143E+00_JPRB, 0.75335E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.77675E+00_JPRB, 0.77744E+00_JPRB, 0.78221E+00_JPRB, 0.78969E+00_JPRB, 0.79785E+00_JPRB, 0.80788E+00_JPRB, 0.81220E+00_JPRB ,&
&  0.81562E+00_JPRB, 0.81492E+00_JPRB, 0.81198E+00_JPRB, 0.79976E+00_JPRB, 0.79140E+00_JPRB, 0.78701E+00_JPRB, 0.78209E+00_JPRB ,&
&  0.77505E+00_JPRB, 0.76972E+00_JPRB, 0.76569E+00_JPRB, 0.76319E+00_JPRB, 0.75883E+00_JPRB, 0.75743E+00_JPRB, 0.74687E+00_JPRB ,&
&  0.72555E+00_JPRB, 0.70608E+00_JPRB, 0.67753E+00_JPRB, 0.65501E+00_JPRB, 0.63954E+00_JPRB, 0.62454E+00_JPRB, 0.62359E+00_JPRB ,&
&  0.61922E+00_JPRB, 0.61552E+00_JPRB, 0.61064E+00_JPRB, 0.60698E+00_JPRB, 0.60752E+00_JPRB, 0.52611E+00_JPRB, 0.50682E+00_JPRB ,&
&  0.49398E+00_JPRB, 0.48323E+00_JPRB, 0.48050E+00_JPRB, 0.48050E+00_JPRB, 0.48804E+00_JPRB, 0.51268E+00_JPRB, 0.54547E+00_JPRB ,&
&  0.56497E+00_JPRB, 0.63372E+00_JPRB, 0.64698E+00_JPRB, 0.65892E+00_JPRB, 0.68026E+00_JPRB, 0.69376E+00_JPRB, 0.69699E+00_JPRB ,&
&  0.70575E+00_JPRB, 0.72216E+00_JPRB, 0.74272E+00_JPRB, 0.76316E+00_JPRB, 0.76668E+00_JPRB, 0.77475E+00_JPRB, 0.77656E+00_JPRB ,&
&  0.77410E+00_JPRB, 0.76645E+00_JPRB, 0.76424E+00_JPRB, 0.76376E+00_JPRB, 0.76174E+00_JPRB, 0.76222E+00_JPRB, 0.75848E+00_JPRB ,&
&  0.76073E+00_JPRB, 0.75859E+00_JPRB, 0.76554E+00_JPRB, 0.76872E+00_JPRB, 0.77503E+00_JPRB, 0.78131E+00_JPRB, 0.78381E+00_JPRB ,&
&  0.78317E+00_JPRB, 0.78322E+00_JPRB, 0.77761E+00_JPRB, 0.76785E+00_JPRB, 0.76451E+00_JPRB, 0.76064E+00_JPRB, 0.76408E+00_JPRB ,&
&  0.77079E+00_JPRB, 0.78349E+00_JPRB, 0.79064E+00_JPRB, 0.79831E+00_JPRB, 0.80502E+00_JPRB, 0.80562E+00_JPRB, 0.80824E+00_JPRB ,&
&  0.80935E+00_JPRB, 0.81101E+00_JPRB, 0.80911E+00_JPRB, 0.80465E+00_JPRB, 0.79701E+00_JPRB, 0.78607E+00_JPRB, 0.77431E+00_JPRB ,&
&  0.76761E+00_JPRB, 0.76694E+00_JPRB, 0.76487E+00_JPRB, 0.76722E+00_JPRB, 0.77193E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.79581E+00_JPRB, 0.79656E+00_JPRB, 0.80490E+00_JPRB, 0.81370E+00_JPRB, 0.82481E+00_JPRB, 0.83527E+00_JPRB, 0.83903E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84672E+00_JPRB, 0.83232E+00_JPRB, 0.81462E+00_JPRB, 0.79488E+00_JPRB, 0.78089E+00_JPRB, 0.77459E+00_JPRB ,&
&  0.77399E+00_JPRB, 0.76491E+00_JPRB, 0.75904E+00_JPRB, 0.75466E+00_JPRB, 0.74923E+00_JPRB, 0.74577E+00_JPRB, 0.73340E+00_JPRB ,&
&  0.70900E+00_JPRB, 0.69380E+00_JPRB, 0.67746E+00_JPRB, 0.65437E+00_JPRB, 0.63446E+00_JPRB, 0.62298E+00_JPRB, 0.62477E+00_JPRB ,&
&  0.62358E+00_JPRB, 0.61366E+00_JPRB, 0.61388E+00_JPRB, 0.60883E+00_JPRB, 0.59499E+00_JPRB, 0.58869E+00_JPRB, 0.52795E+00_JPRB ,&
&  0.50713E+00_JPRB, 0.48427E+00_JPRB, 0.47598E+00_JPRB, 0.47514E+00_JPRB, 0.49088E+00_JPRB, 0.51677E+00_JPRB, 0.56364E+00_JPRB ,&
&  0.60485E+00_JPRB, 0.64255E+00_JPRB, 0.67119E+00_JPRB, 0.69154E+00_JPRB, 0.71244E+00_JPRB, 0.73197E+00_JPRB, 0.74548E+00_JPRB ,&
&  0.77017E+00_JPRB, 0.80372E+00_JPRB, 0.83775E+00_JPRB, 0.87060E+00_JPRB, 0.89307E+00_JPRB, 0.90115E+00_JPRB, 0.90816E+00_JPRB ,&
&  0.90325E+00_JPRB, 0.83259E+00_JPRB, 0.80538E+00_JPRB, 0.78921E+00_JPRB, 0.77795E+00_JPRB, 0.77835E+00_JPRB, 0.77131E+00_JPRB ,&
&  0.77299E+00_JPRB, 0.77881E+00_JPRB, 0.78174E+00_JPRB, 0.78590E+00_JPRB, 0.79188E+00_JPRB, 0.79415E+00_JPRB, 0.80021E+00_JPRB ,&
&  0.81146E+00_JPRB, 0.82807E+00_JPRB, 0.85904E+00_JPRB, 0.83545E+00_JPRB, 0.80639E+00_JPRB, 0.77866E+00_JPRB, 0.76447E+00_JPRB ,&
&  0.76280E+00_JPRB, 0.76888E+00_JPRB, 0.78007E+00_JPRB, 0.78552E+00_JPRB, 0.79193E+00_JPRB, 0.79748E+00_JPRB, 0.81298E+00_JPRB ,&
&  0.81914E+00_JPRB, 0.82447E+00_JPRB, 0.82570E+00_JPRB, 0.82316E+00_JPRB, 0.81395E+00_JPRB, 0.80089E+00_JPRB, 0.78789E+00_JPRB ,&
&  0.78331E+00_JPRB, 0.78494E+00_JPRB, 0.78628E+00_JPRB, 0.78987E+00_JPRB, 0.79346E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.80220E+00_JPRB, 0.81153E+00_JPRB, 0.83095E+00_JPRB, 0.84939E+00_JPRB, 0.86300E+00_JPRB, 0.85235E+00_JPRB, 0.83845E+00_JPRB ,&
&  0.83534E+00_JPRB, 0.83451E+00_JPRB, 0.82146E+00_JPRB, 0.80741E+00_JPRB, 0.78613E+00_JPRB, 0.76934E+00_JPRB, 0.76410E+00_JPRB ,&
&  0.76249E+00_JPRB, 0.75463E+00_JPRB, 0.74292E+00_JPRB, 0.72754E+00_JPRB, 0.71837E+00_JPRB, 0.71585E+00_JPRB, 0.70715E+00_JPRB ,&
&  0.69422E+00_JPRB, 0.68688E+00_JPRB, 0.67795E+00_JPRB, 0.66162E+00_JPRB, 0.64526E+00_JPRB, 0.63568E+00_JPRB, 0.63109E+00_JPRB ,&
&  0.62781E+00_JPRB, 0.62645E+00_JPRB, 0.62791E+00_JPRB, 0.62326E+00_JPRB, 0.62065E+00_JPRB, 0.61495E+00_JPRB, 0.57072E+00_JPRB ,&
&  0.52494E+00_JPRB, 0.49406E+00_JPRB, 0.47651E+00_JPRB, 0.47540E+00_JPRB, 0.49740E+00_JPRB, 0.54251E+00_JPRB, 0.59262E+00_JPRB ,&
&  0.63632E+00_JPRB, 0.68309E+00_JPRB, 0.71987E+00_JPRB, 0.74157E+00_JPRB, 0.75965E+00_JPRB, 0.78301E+00_JPRB, 0.81149E+00_JPRB ,&
&  0.85200E+00_JPRB, 0.90308E+00_JPRB, 0.94995E+00_JPRB, 0.99638E+00_JPRB, 0.10294E+01_JPRB, 0.99726E+00_JPRB, 0.10640E+01_JPRB ,&
&  0.12892E+01_JPRB, 0.10715E+01_JPRB, 0.89904E+00_JPRB, 0.83538E+00_JPRB, 0.82248E+00_JPRB, 0.82741E+00_JPRB, 0.83457E+00_JPRB ,&
&  0.84609E+00_JPRB, 0.84872E+00_JPRB, 0.84330E+00_JPRB, 0.83842E+00_JPRB, 0.84236E+00_JPRB, 0.85108E+00_JPRB, 0.87224E+00_JPRB ,&
&  0.90858E+00_JPRB, 0.10774E+01_JPRB, 0.13371E+01_JPRB, 0.10300E+01_JPRB, 0.91877E+00_JPRB, 0.83708E+00_JPRB, 0.76829E+00_JPRB ,&
&  0.74095E+00_JPRB, 0.73509E+00_JPRB, 0.74334E+00_JPRB, 0.75180E+00_JPRB, 0.75537E+00_JPRB, 0.76277E+00_JPRB, 0.77482E+00_JPRB ,&
&  0.79216E+00_JPRB, 0.80944E+00_JPRB, 0.82066E+00_JPRB, 0.82671E+00_JPRB, 0.82322E+00_JPRB, 0.80959E+00_JPRB, 0.78964E+00_JPRB ,&
&  0.78791E+00_JPRB, 0.79716E+00_JPRB, 0.80386E+00_JPRB, 0.80760E+00_JPRB, 0.80462E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.80267E+00_JPRB, 0.83742E+00_JPRB, 0.88336E+00_JPRB, 0.89476E+00_JPRB, 0.88836E+00_JPRB, 0.84307E+00_JPRB, 0.79723E+00_JPRB ,&
&  0.77149E+00_JPRB, 0.76302E+00_JPRB, 0.75915E+00_JPRB, 0.74697E+00_JPRB, 0.73436E+00_JPRB, 0.72689E+00_JPRB, 0.72108E+00_JPRB ,&
&  0.71599E+00_JPRB, 0.71116E+00_JPRB, 0.70506E+00_JPRB, 0.69346E+00_JPRB, 0.68900E+00_JPRB, 0.69098E+00_JPRB, 0.68940E+00_JPRB ,&
&  0.68501E+00_JPRB, 0.68392E+00_JPRB, 0.67233E+00_JPRB, 0.65571E+00_JPRB, 0.64828E+00_JPRB, 0.64858E+00_JPRB, 0.65205E+00_JPRB ,&
&  0.64277E+00_JPRB, 0.63347E+00_JPRB, 0.63930E+00_JPRB, 0.64901E+00_JPRB, 0.64262E+00_JPRB, 0.63385E+00_JPRB, 0.58592E+00_JPRB ,&
&  0.53512E+00_JPRB, 0.50308E+00_JPRB, 0.49399E+00_JPRB, 0.52331E+00_JPRB, 0.57707E+00_JPRB, 0.64489E+00_JPRB, 0.68907E+00_JPRB ,&
&  0.71456E+00_JPRB, 0.74943E+00_JPRB, 0.78252E+00_JPRB, 0.81921E+00_JPRB, 0.86620E+00_JPRB, 0.91921E+00_JPRB, 0.98024E+00_JPRB ,&
&  0.10290E+01_JPRB, 0.10591E+01_JPRB, 0.10602E+01_JPRB, 0.96648E+00_JPRB, 0.94566E+00_JPRB, 0.94799E+00_JPRB, 0.10329E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.10429E+01_JPRB, 0.98172E+00_JPRB, 0.98022E+00_JPRB, 0.97216E+00_JPRB, 0.98680E+00_JPRB, 0.99346E+00_JPRB ,&
&  0.97820E+00_JPRB, 0.94812E+00_JPRB, 0.91462E+00_JPRB, 0.88796E+00_JPRB, 0.88749E+00_JPRB, 0.90925E+00_JPRB, 0.93204E+00_JPRB ,&
&  0.98280E+00_JPRB, 0.10936E+01_JPRB, 0.10629E+01_JPRB, 0.83688E+00_JPRB, 0.80185E+00_JPRB, 0.78518E+00_JPRB, 0.76678E+00_JPRB ,&
&  0.73795E+00_JPRB, 0.72109E+00_JPRB, 0.72090E+00_JPRB, 0.72497E+00_JPRB, 0.72173E+00_JPRB, 0.71884E+00_JPRB, 0.72202E+00_JPRB ,&
&  0.73040E+00_JPRB, 0.74560E+00_JPRB, 0.76553E+00_JPRB, 0.79588E+00_JPRB, 0.80917E+00_JPRB, 0.80447E+00_JPRB, 0.78555E+00_JPRB ,&
&  0.77710E+00_JPRB, 0.78222E+00_JPRB, 0.78575E+00_JPRB, 0.78550E+00_JPRB, 0.78916E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.91871E+00_JPRB, 0.92938E+00_JPRB, 0.93045E+00_JPRB, 0.93084E+00_JPRB, 0.88622E+00_JPRB, 0.86105E+00_JPRB, 0.81998E+00_JPRB ,&
&  0.80891E+00_JPRB, 0.78509E+00_JPRB, 0.74632E+00_JPRB, 0.70178E+00_JPRB, 0.67609E+00_JPRB, 0.66434E+00_JPRB, 0.66442E+00_JPRB ,&
&  0.67163E+00_JPRB, 0.68067E+00_JPRB, 0.68286E+00_JPRB, 0.67296E+00_JPRB, 0.67559E+00_JPRB, 0.67904E+00_JPRB, 0.67765E+00_JPRB ,&
&  0.67745E+00_JPRB, 0.68142E+00_JPRB, 0.67783E+00_JPRB, 0.66964E+00_JPRB, 0.66593E+00_JPRB, 0.66547E+00_JPRB, 0.66064E+00_JPRB ,&
&  0.64812E+00_JPRB, 0.63955E+00_JPRB, 0.64747E+00_JPRB, 0.67068E+00_JPRB, 0.65305E+00_JPRB, 0.64437E+00_JPRB, 0.58682E+00_JPRB ,&
&  0.54009E+00_JPRB, 0.53042E+00_JPRB, 0.56333E+00_JPRB, 0.63463E+00_JPRB, 0.66923E+00_JPRB, 0.70483E+00_JPRB, 0.72783E+00_JPRB ,&
&  0.72893E+00_JPRB, 0.74821E+00_JPRB, 0.80310E+00_JPRB, 0.90676E+00_JPRB, 0.95832E+00_JPRB, 0.10262E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.11470E+01_JPRB, 0.11872E+01_JPRB, 0.12116E+01_JPRB, 0.11554E+01_JPRB, 0.11820E+01_JPRB, 0.12808E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.10946E+01_JPRB, 0.11042E+01_JPRB, 0.11675E+01_JPRB, 0.11210E+01_JPRB, 0.11441E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.10445E+01_JPRB, 0.99781E+00_JPRB, 0.96257E+00_JPRB, 0.94948E+00_JPRB, 0.94556E+00_JPRB, 0.95808E+00_JPRB, 0.98618E+00_JPRB ,&
&  0.10365E+01_JPRB, 0.10545E+01_JPRB, 0.97972E+00_JPRB, 0.84051E+00_JPRB, 0.78728E+00_JPRB, 0.76456E+00_JPRB, 0.74831E+00_JPRB ,&
&  0.73045E+00_JPRB, 0.71916E+00_JPRB, 0.72181E+00_JPRB, 0.72432E+00_JPRB, 0.72205E+00_JPRB, 0.71148E+00_JPRB, 0.70154E+00_JPRB ,&
&  0.69757E+00_JPRB, 0.70242E+00_JPRB, 0.70628E+00_JPRB, 0.73701E+00_JPRB, 0.76854E+00_JPRB, 0.78922E+00_JPRB, 0.78434E+00_JPRB ,&
&  0.78097E+00_JPRB, 0.79466E+00_JPRB, 0.81600E+00_JPRB, 0.85688E+00_JPRB, 0.88480E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.95658E+00_JPRB, 0.97232E+00_JPRB, 0.97970E+00_JPRB, 0.96723E+00_JPRB, 0.92945E+00_JPRB, 0.88362E+00_JPRB, 0.79368E+00_JPRB ,&
&  0.78739E+00_JPRB, 0.74223E+00_JPRB, 0.70215E+00_JPRB, 0.67268E+00_JPRB, 0.66209E+00_JPRB, 0.64741E+00_JPRB, 0.64154E+00_JPRB ,&
&  0.64175E+00_JPRB, 0.65857E+00_JPRB, 0.67155E+00_JPRB, 0.66650E+00_JPRB, 0.67149E+00_JPRB, 0.67593E+00_JPRB, 0.67312E+00_JPRB ,&
&  0.67940E+00_JPRB, 0.68976E+00_JPRB, 0.69779E+00_JPRB, 0.69971E+00_JPRB, 0.68720E+00_JPRB, 0.67784E+00_JPRB, 0.66845E+00_JPRB ,&
&  0.66228E+00_JPRB, 0.65942E+00_JPRB, 0.66400E+00_JPRB, 0.67989E+00_JPRB, 0.66593E+00_JPRB, 0.64953E+00_JPRB, 0.62425E+00_JPRB ,&
&  0.56790E+00_JPRB, 0.58319E+00_JPRB, 0.64762E+00_JPRB, 0.67955E+00_JPRB, 0.71223E+00_JPRB, 0.74632E+00_JPRB, 0.79733E+00_JPRB ,&
&  0.84148E+00_JPRB, 0.88859E+00_JPRB, 0.94581E+00_JPRB, 0.10179E+01_JPRB, 0.11089E+01_JPRB, 0.12039E+01_JPRB, 0.12832E+01_JPRB ,&
&  0.13645E+01_JPRB, 0.11433E+01_JPRB, 0.11223E+01_JPRB, 0.12995E+01_JPRB, 0.15877E+01_JPRB, 0.14032E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.12588E+01_JPRB, 0.11968E+01_JPRB, 0.11733E+01_JPRB, 0.12445E+01_JPRB, 0.12615E+01_JPRB, 0.13166E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.11633E+01_JPRB, 0.10934E+01_JPRB, 0.10696E+01_JPRB, 0.10638E+01_JPRB, 0.10665E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.10314E+01_JPRB, 0.97447E+00_JPRB, 0.89577E+00_JPRB, 0.83930E+00_JPRB, 0.80613E+00_JPRB, 0.78209E+00_JPRB ,&
&  0.76271E+00_JPRB, 0.74262E+00_JPRB, 0.72851E+00_JPRB, 0.72383E+00_JPRB, 0.72624E+00_JPRB, 0.73898E+00_JPRB, 0.77020E+00_JPRB ,&
&  0.75469E+00_JPRB, 0.74575E+00_JPRB, 0.73028E+00_JPRB, 0.75126E+00_JPRB, 0.79071E+00_JPRB, 0.82530E+00_JPRB, 0.83347E+00_JPRB ,&
&  0.85296E+00_JPRB, 0.86259E+00_JPRB, 0.85857E+00_JPRB, 0.88652E+00_JPRB, 0.91131E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.99038E+00_JPRB, 0.10121E+01_JPRB, 0.10183E+01_JPRB, 0.10191E+01_JPRB, 0.97369E+00_JPRB, 0.89904E+00_JPRB, 0.82946E+00_JPRB ,&
&  0.88856E+00_JPRB, 0.81608E+00_JPRB, 0.80556E+00_JPRB, 0.70111E+00_JPRB, 0.66815E+00_JPRB, 0.65349E+00_JPRB, 0.63326E+00_JPRB ,&
&  0.61196E+00_JPRB, 0.62788E+00_JPRB, 0.64995E+00_JPRB, 0.65893E+00_JPRB, 0.66817E+00_JPRB, 0.67676E+00_JPRB, 0.67320E+00_JPRB ,&
&  0.68166E+00_JPRB, 0.69949E+00_JPRB, 0.72634E+00_JPRB, 0.72395E+00_JPRB, 0.70630E+00_JPRB, 0.70215E+00_JPRB, 0.69764E+00_JPRB ,&
&  0.68048E+00_JPRB, 0.67408E+00_JPRB, 0.68641E+00_JPRB, 0.69712E+00_JPRB, 0.69523E+00_JPRB, 0.66612E+00_JPRB, 0.64637E+00_JPRB ,&
&  0.64563E+00_JPRB, 0.64715E+00_JPRB, 0.69464E+00_JPRB, 0.72581E+00_JPRB, 0.76522E+00_JPRB, 0.81220E+00_JPRB, 0.86040E+00_JPRB ,&
&  0.91948E+00_JPRB, 0.99061E+00_JPRB, 0.10763E+01_JPRB, 0.11882E+01_JPRB, 0.13045E+01_JPRB, 0.14386E+01_JPRB, 0.15445E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.13048E+01_JPRB, 0.15736E+01_JPRB, 0.16861E+01_JPRB, 0.19046E+01_JPRB, 0.18830E+01_JPRB, 0.17930E+01_JPRB ,&
&  0.18004E+01_JPRB, 0.15905E+01_JPRB, 0.14586E+01_JPRB, 0.13650E+01_JPRB, 0.13647E+01_JPRB, 0.14565E+01_JPRB, 0.15342E+01_JPRB ,&
&  0.16237E+01_JPRB, 0.15612E+01_JPRB, 0.14444E+01_JPRB, 0.13742E+01_JPRB, 0.13275E+01_JPRB, 0.12941E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.11250E+01_JPRB, 0.10594E+01_JPRB, 0.97132E+00_JPRB, 0.92486E+00_JPRB, 0.88320E+00_JPRB, 0.83919E+00_JPRB ,&
&  0.81526E+00_JPRB, 0.77346E+00_JPRB, 0.73318E+00_JPRB, 0.71076E+00_JPRB, 0.69674E+00_JPRB, 0.69991E+00_JPRB, 0.72778E+00_JPRB ,&
&  0.74481E+00_JPRB, 0.74833E+00_JPRB, 0.80386E+00_JPRB, 0.85323E+00_JPRB, 0.88187E+00_JPRB, 0.90924E+00_JPRB, 0.95569E+00_JPRB ,&
&  0.89928E+00_JPRB, 0.90749E+00_JPRB, 0.95811E+00_JPRB, 0.96974E+00_JPRB, 0.97760E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.10546E+01_JPRB, 0.10821E+01_JPRB, 0.10765E+01_JPRB, 0.10734E+01_JPRB, 0.99865E+00_JPRB, 0.96621E+00_JPRB, 0.88443E+00_JPRB ,&
&  0.90421E+00_JPRB, 0.87063E+00_JPRB, 0.79829E+00_JPRB, 0.77378E+00_JPRB, 0.77857E+00_JPRB, 0.70059E+00_JPRB, 0.63813E+00_JPRB ,&
&  0.60540E+00_JPRB, 0.60684E+00_JPRB, 0.64414E+00_JPRB, 0.66563E+00_JPRB, 0.67127E+00_JPRB, 0.67909E+00_JPRB, 0.67287E+00_JPRB ,&
&  0.70991E+00_JPRB, 0.72206E+00_JPRB, 0.74779E+00_JPRB, 0.75820E+00_JPRB, 0.75124E+00_JPRB, 0.75324E+00_JPRB, 0.76113E+00_JPRB ,&
&  0.73263E+00_JPRB, 0.72357E+00_JPRB, 0.71159E+00_JPRB, 0.69093E+00_JPRB, 0.73761E+00_JPRB, 0.71921E+00_JPRB, 0.69439E+00_JPRB ,&
&  0.69353E+00_JPRB, 0.71495E+00_JPRB, 0.74859E+00_JPRB, 0.79068E+00_JPRB, 0.83065E+00_JPRB, 0.87558E+00_JPRB, 0.93165E+00_JPRB ,&
&  0.99290E+00_JPRB, 0.10824E+01_JPRB, 0.11688E+01_JPRB, 0.13193E+01_JPRB, 0.14537E+01_JPRB, 0.14546E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.18281E+01_JPRB, 0.20067E+01_JPRB, 0.22010E+01_JPRB, 0.20495E+01_JPRB, 0.22708E+01_JPRB, 0.22121E+01_JPRB, 0.20134E+01_JPRB ,&
&  0.20254E+01_JPRB, 0.18609E+01_JPRB, 0.18106E+01_JPRB, 0.18043E+01_JPRB, 0.18980E+01_JPRB, 0.18457E+01_JPRB, 0.18204E+01_JPRB ,&
&  0.18937E+01_JPRB, 0.21165E+01_JPRB, 0.19788E+01_JPRB, 0.19510E+01_JPRB, 0.18408E+01_JPRB, 0.17459E+01_JPRB, 0.16068E+01_JPRB ,&
&  0.15073E+01_JPRB, 0.13927E+01_JPRB, 0.12841E+01_JPRB, 0.11960E+01_JPRB, 0.10815E+01_JPRB, 0.99328E+00_JPRB, 0.92212E+00_JPRB ,&
&  0.84799E+00_JPRB, 0.76064E+00_JPRB, 0.72949E+00_JPRB, 0.70945E+00_JPRB, 0.70120E+00_JPRB, 0.67860E+00_JPRB, 0.69096E+00_JPRB ,&
&  0.73162E+00_JPRB, 0.80265E+00_JPRB, 0.90905E+00_JPRB, 0.97561E+00_JPRB, 0.10604E+01_JPRB, 0.11414E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.96416E+00_JPRB, 0.10374E+01_JPRB, 0.10521E+01_JPRB, 0.10526E+01_JPRB, 0.10504E+01_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.12037E+01_JPRB, 0.12127E+01_JPRB, 0.11883E+01_JPRB, 0.11450E+01_JPRB, 0.10415E+01_JPRB, 0.99202E+00_JPRB, 0.92783E+00_JPRB ,&
&  0.92818E+00_JPRB, 0.88821E+00_JPRB, 0.81829E+00_JPRB, 0.79159E+00_JPRB, 0.79027E+00_JPRB, 0.77226E+00_JPRB, 0.70113E+00_JPRB ,&
&  0.65097E+00_JPRB, 0.66037E+00_JPRB, 0.65344E+00_JPRB, 0.69449E+00_JPRB, 0.73306E+00_JPRB, 0.71683E+00_JPRB, 0.73280E+00_JPRB ,&
&  0.77619E+00_JPRB, 0.78327E+00_JPRB, 0.78259E+00_JPRB, 0.78355E+00_JPRB, 0.78831E+00_JPRB, 0.79913E+00_JPRB, 0.79807E+00_JPRB ,&
&  0.76238E+00_JPRB, 0.73741E+00_JPRB, 0.71331E+00_JPRB, 0.71027E+00_JPRB, 0.76146E+00_JPRB, 0.78206E+00_JPRB, 0.76873E+00_JPRB ,&
&  0.75914E+00_JPRB, 0.76085E+00_JPRB, 0.77750E+00_JPRB, 0.82624E+00_JPRB, 0.87571E+00_JPRB, 0.92356E+00_JPRB, 0.98910E+00_JPRB ,&
&  0.10669E+01_JPRB, 0.11602E+01_JPRB, 0.12454E+01_JPRB, 0.13566E+01_JPRB, 0.13863E+01_JPRB, 0.15590E+01_JPRB, 0.18616E+01_JPRB ,&
&  0.20081E+01_JPRB, 0.21869E+01_JPRB, 0.23542E+01_JPRB, 0.25843E+01_JPRB, 0.27563E+01_JPRB, 0.25851E+01_JPRB, 0.23971E+01_JPRB ,&
&  0.23293E+01_JPRB, 0.22314E+01_JPRB, 0.27260E+01_JPRB, 0.25374E+01_JPRB, 0.23448E+01_JPRB, 0.22946E+01_JPRB, 0.22125E+01_JPRB ,&
&  0.22312E+01_JPRB, 0.23002E+01_JPRB, 0.23050E+01_JPRB, 0.23035E+01_JPRB, 0.22725E+01_JPRB, 0.22624E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.20368E+01_JPRB, 0.18679E+01_JPRB, 0.15190E+01_JPRB, 0.12541E+01_JPRB, 0.10419E+01_JPRB, 0.10385E+01_JPRB, 0.96454E+00_JPRB ,&
&  0.86415E+00_JPRB, 0.76151E+00_JPRB, 0.74296E+00_JPRB, 0.72294E+00_JPRB, 0.72334E+00_JPRB, 0.71373E+00_JPRB, 0.70527E+00_JPRB ,&
&  0.74691E+00_JPRB, 0.86170E+00_JPRB, 0.94880E+00_JPRB, 0.10886E+01_JPRB, 0.12424E+01_JPRB, 0.13663E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.12624E+01_JPRB, 0.12814E+01_JPRB, 0.12710E+01_JPRB, 0.12351E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.14753E+01_JPRB, 0.14172E+01_JPRB, 0.13424E+01_JPRB, 0.12335E+01_JPRB, 0.10920E+01_JPRB, 0.10075E+01_JPRB, 0.93884E+00_JPRB ,&
&  0.92766E+00_JPRB, 0.89067E+00_JPRB, 0.83195E+00_JPRB, 0.80710E+00_JPRB, 0.79995E+00_JPRB, 0.79233E+00_JPRB, 0.78903E+00_JPRB ,&
&  0.71099E+00_JPRB, 0.75089E+00_JPRB, 0.71582E+00_JPRB, 0.76982E+00_JPRB, 0.91509E+00_JPRB, 0.81036E+00_JPRB, 0.81948E+00_JPRB ,&
&  0.81865E+00_JPRB, 0.83945E+00_JPRB, 0.81975E+00_JPRB, 0.81761E+00_JPRB, 0.84746E+00_JPRB, 0.87080E+00_JPRB, 0.86296E+00_JPRB ,&
&  0.82674E+00_JPRB, 0.78275E+00_JPRB, 0.76402E+00_JPRB, 0.76440E+00_JPRB, 0.81409E+00_JPRB, 0.85210E+00_JPRB, 0.86761E+00_JPRB ,&
&  0.86188E+00_JPRB, 0.85492E+00_JPRB, 0.85065E+00_JPRB, 0.87646E+00_JPRB, 0.91924E+00_JPRB, 0.96985E+00_JPRB, 0.10505E+01_JPRB ,&
&  0.11429E+01_JPRB, 0.12296E+01_JPRB, 0.13241E+01_JPRB, 0.14309E+01_JPRB, 0.14507E+01_JPRB, 0.16909E+01_JPRB, 0.20529E+01_JPRB ,&
&  0.23406E+01_JPRB, 0.23343E+01_JPRB, 0.24704E+01_JPRB, 0.29980E+01_JPRB, 0.32891E+01_JPRB, 0.32052E+01_JPRB, 0.29771E+01_JPRB ,&
&  0.27473E+01_JPRB, 0.25774E+01_JPRB, 0.26805E+01_JPRB, 0.26557E+01_JPRB, 0.24719E+01_JPRB, 0.25394E+01_JPRB, 0.24988E+01_JPRB ,&
&  0.24477E+01_JPRB, 0.24725E+01_JPRB, 0.25588E+01_JPRB, 0.26217E+01_JPRB, 0.25146E+01_JPRB, 0.25488E+01_JPRB, 0.24575E+01_JPRB ,&
&  0.23715E+01_JPRB, 0.20318E+01_JPRB, 0.14836E+01_JPRB, 0.12098E+01_JPRB, 0.93234E+00_JPRB, 0.91123E+00_JPRB, 0.10554E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.96125E+00_JPRB, 0.88188E+00_JPRB, 0.83418E+00_JPRB, 0.80691E+00_JPRB, 0.84162E+00_JPRB, 0.87008E+00_JPRB ,&
&  0.89909E+00_JPRB, 0.10434E+01_JPRB, 0.12695E+01_JPRB, 0.15113E+01_JPRB, 0.16517E+01_JPRB, 0.16998E+01_JPRB, 0.15512E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.16120E+01_JPRB, 0.16069E+01_JPRB, 0.16073E+01_JPRB, 0.15607E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.17434E+01_JPRB, 0.16457E+01_JPRB, 0.15002E+01_JPRB, 0.13213E+01_JPRB, 0.11226E+01_JPRB, 0.10045E+01_JPRB, 0.95611E+00_JPRB ,&
&  0.94265E+00_JPRB, 0.90951E+00_JPRB, 0.85456E+00_JPRB, 0.82377E+00_JPRB, 0.81226E+00_JPRB, 0.80535E+00_JPRB, 0.82262E+00_JPRB ,&
&  0.88075E+00_JPRB, 0.95049E+00_JPRB, 0.83987E+00_JPRB, 0.75956E+00_JPRB, 0.82337E+00_JPRB, 0.81180E+00_JPRB, 0.83503E+00_JPRB ,&
&  0.83811E+00_JPRB, 0.89707E+00_JPRB, 0.85426E+00_JPRB, 0.87372E+00_JPRB, 0.96079E+00_JPRB, 0.10091E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.10248E+01_JPRB, 0.99339E+00_JPRB, 0.95823E+00_JPRB, 0.97466E+00_JPRB, 0.10246E+01_JPRB, 0.98389E+00_JPRB, 0.10040E+01_JPRB ,&
&  0.98300E+00_JPRB, 0.97158E+00_JPRB, 0.94674E+00_JPRB, 0.96481E+00_JPRB, 0.10189E+01_JPRB, 0.11097E+01_JPRB, 0.11752E+01_JPRB ,&
&  0.12424E+01_JPRB, 0.13014E+01_JPRB, 0.13354E+01_JPRB, 0.14334E+01_JPRB, 0.15803E+01_JPRB, 0.17774E+01_JPRB, 0.18590E+01_JPRB ,&
&  0.20508E+01_JPRB, 0.21165E+01_JPRB, 0.21919E+01_JPRB, 0.26776E+01_JPRB, 0.30800E+01_JPRB, 0.31815E+01_JPRB, 0.31762E+01_JPRB ,&
&  0.31029E+01_JPRB, 0.30103E+01_JPRB, 0.30825E+01_JPRB, 0.29627E+01_JPRB, 0.28701E+01_JPRB, 0.28891E+01_JPRB, 0.28863E+01_JPRB ,&
&  0.27376E+01_JPRB, 0.27259E+01_JPRB, 0.28124E+01_JPRB, 0.27657E+01_JPRB, 0.26951E+01_JPRB, 0.25735E+01_JPRB, 0.25764E+01_JPRB ,&
&  0.25656E+01_JPRB, 0.15796E+01_JPRB, 0.12366E+01_JPRB, 0.13011E+01_JPRB, 0.10163E+01_JPRB, 0.90904E+00_JPRB, 0.99070E+00_JPRB ,&
&  0.96355E+00_JPRB, 0.95158E+00_JPRB, 0.98682E+00_JPRB, 0.10120E+01_JPRB, 0.10302E+01_JPRB, 0.11632E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.14851E+01_JPRB, 0.18373E+01_JPRB, 0.20627E+01_JPRB, 0.21198E+01_JPRB, 0.21084E+01_JPRB, 0.20618E+01_JPRB ,&
&  0.20023E+01_JPRB, 0.19358E+01_JPRB, 0.18910E+01_JPRB, 0.18674E+01_JPRB, 0.18248E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.19932E+01_JPRB, 0.18816E+01_JPRB, 0.16890E+01_JPRB, 0.14534E+01_JPRB, 0.11971E+01_JPRB, 0.10483E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.10081E+01_JPRB, 0.98174E+00_JPRB, 0.93008E+00_JPRB, 0.88689E+00_JPRB, 0.86806E+00_JPRB, 0.85478E+00_JPRB, 0.86076E+00_JPRB ,&
&  0.86756E+00_JPRB, 0.96526E+00_JPRB, 0.82674E+00_JPRB, 0.72503E+00_JPRB, 0.66974E+00_JPRB, 0.71990E+00_JPRB, 0.76777E+00_JPRB ,&
&  0.83400E+00_JPRB, 0.90534E+00_JPRB, 0.93192E+00_JPRB, 0.10737E+01_JPRB, 0.12027E+01_JPRB, 0.12665E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.13767E+01_JPRB, 0.13593E+01_JPRB, 0.12546E+01_JPRB, 0.12786E+01_JPRB, 0.12464E+01_JPRB, 0.11513E+01_JPRB, 0.11929E+01_JPRB ,&
&  0.11578E+01_JPRB, 0.11364E+01_JPRB, 0.11185E+01_JPRB, 0.11425E+01_JPRB, 0.11914E+01_JPRB, 0.12144E+01_JPRB, 0.11947E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.12709E+01_JPRB, 0.13576E+01_JPRB, 0.14435E+01_JPRB, 0.15610E+01_JPRB, 0.17029E+01_JPRB, 0.17152E+01_JPRB ,&
&  0.18143E+01_JPRB, 0.16912E+01_JPRB, 0.15566E+01_JPRB, 0.20098E+01_JPRB, 0.28405E+01_JPRB, 0.31742E+01_JPRB, 0.30585E+01_JPRB ,&
&  0.33093E+01_JPRB, 0.33890E+01_JPRB, 0.34615E+01_JPRB, 0.33696E+01_JPRB, 0.32418E+01_JPRB, 0.32043E+01_JPRB, 0.32344E+01_JPRB ,&
&  0.31335E+01_JPRB, 0.31464E+01_JPRB, 0.31484E+01_JPRB, 0.31175E+01_JPRB, 0.30138E+01_JPRB, 0.29999E+01_JPRB, 0.30110E+01_JPRB ,&
&  0.27623E+01_JPRB, 0.23234E+01_JPRB, 0.16306E+01_JPRB, 0.15707E+01_JPRB, 0.11487E+01_JPRB, 0.97670E+00_JPRB, 0.10271E+01_JPRB ,&
&  0.96632E+00_JPRB, 0.10013E+01_JPRB, 0.11068E+01_JPRB, 0.11992E+01_JPRB, 0.14863E+01_JPRB, 0.17784E+01_JPRB, 0.20775E+01_JPRB ,&
&  0.22389E+01_JPRB, 0.21588E+01_JPRB, 0.25214E+01_JPRB, 0.25906E+01_JPRB, 0.25865E+01_JPRB, 0.25903E+01_JPRB, 0.25767E+01_JPRB ,&
&  0.24951E+01_JPRB, 0.23762E+01_JPRB, 0.22737E+01_JPRB, 0.21729E+01_JPRB, 0.20845E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.21265E+01_JPRB, 0.19848E+01_JPRB, 0.17599E+01_JPRB, 0.15090E+01_JPRB, 0.12534E+01_JPRB, 0.10834E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.11021E+01_JPRB, 0.10705E+01_JPRB, 0.10089E+01_JPRB, 0.96786E+00_JPRB, 0.94366E+00_JPRB, 0.91979E+00_JPRB ,&
&  0.89458E+00_JPRB, 0.86080E+00_JPRB, 0.72634E+00_JPRB, 0.64015E+00_JPRB, 0.61911E+00_JPRB, 0.63423E+00_JPRB, 0.71123E+00_JPRB ,&
&  0.80847E+00_JPRB, 0.98483E+00_JPRB, 0.11720E+01_JPRB, 0.13818E+01_JPRB, 0.16276E+01_JPRB, 0.18635E+01_JPRB, 0.19407E+01_JPRB ,&
&  0.19995E+01_JPRB, 0.17122E+01_JPRB, 0.16947E+01_JPRB, 0.15919E+01_JPRB, 0.15306E+01_JPRB, 0.14581E+01_JPRB, 0.14054E+01_JPRB ,&
&  0.13289E+01_JPRB, 0.12922E+01_JPRB, 0.12575E+01_JPRB, 0.12424E+01_JPRB, 0.12356E+01_JPRB, 0.12316E+01_JPRB, 0.12313E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.13644E+01_JPRB, 0.14325E+01_JPRB, 0.14875E+01_JPRB, 0.16020E+01_JPRB, 0.17500E+01_JPRB, 0.17897E+01_JPRB ,&
&  0.17003E+01_JPRB, 0.16942E+01_JPRB, 0.20238E+01_JPRB, 0.22705E+01_JPRB, 0.25967E+01_JPRB, 0.30339E+01_JPRB, 0.32998E+01_JPRB ,&
&  0.34698E+01_JPRB, 0.33652E+01_JPRB, 0.33769E+01_JPRB, 0.31395E+01_JPRB, 0.35029E+01_JPRB, 0.34836E+01_JPRB, 0.34492E+01_JPRB ,&
&  0.33983E+01_JPRB, 0.35072E+01_JPRB, 0.33560E+01_JPRB, 0.32488E+01_JPRB, 0.32944E+01_JPRB, 0.34971E+01_JPRB, 0.31409E+01_JPRB ,&
&  0.26734E+01_JPRB, 0.29177E+01_JPRB, 0.25613E+01_JPRB, 0.21201E+01_JPRB, 0.16530E+01_JPRB, 0.14447E+01_JPRB, 0.12960E+01_JPRB ,&
&  0.11920E+01_JPRB, 0.11632E+01_JPRB, 0.12838E+01_JPRB, 0.16478E+01_JPRB, 0.22371E+01_JPRB, 0.27005E+01_JPRB, 0.29349E+01_JPRB ,&
&  0.29570E+01_JPRB, 0.35259E+01_JPRB, 0.34670E+01_JPRB, 0.33040E+01_JPRB, 0.32211E+01_JPRB, 0.31875E+01_JPRB, 0.30884E+01_JPRB ,&
&  0.28655E+01_JPRB, 0.26805E+01_JPRB, 0.25237E+01_JPRB, 0.23836E+01_JPRB, 0.22425E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.21765E+01_JPRB, 0.20482E+01_JPRB, 0.18290E+01_JPRB, 0.15625E+01_JPRB, 0.13186E+01_JPRB, 0.11558E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.11761E+01_JPRB, 0.11874E+01_JPRB, 0.11797E+01_JPRB, 0.11506E+01_JPRB, 0.10986E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.99476E+00_JPRB, 0.83997E+00_JPRB, 0.69182E+00_JPRB, 0.64829E+00_JPRB, 0.63318E+00_JPRB, 0.63088E+00_JPRB, 0.68458E+00_JPRB ,&
&  0.82345E+00_JPRB, 0.11595E+01_JPRB, 0.15295E+01_JPRB, 0.19504E+01_JPRB, 0.23217E+01_JPRB, 0.25721E+01_JPRB, 0.25281E+01_JPRB ,&
&  0.23355E+01_JPRB, 0.22835E+01_JPRB, 0.21366E+01_JPRB, 0.19512E+01_JPRB, 0.18120E+01_JPRB, 0.16810E+01_JPRB, 0.15673E+01_JPRB ,&
&  0.14644E+01_JPRB, 0.13818E+01_JPRB, 0.13613E+01_JPRB, 0.13541E+01_JPRB, 0.13540E+01_JPRB, 0.13603E+01_JPRB, 0.13761E+01_JPRB ,&
&  0.14054E+01_JPRB, 0.14549E+01_JPRB, 0.14953E+01_JPRB, 0.15861E+01_JPRB, 0.14829E+01_JPRB, 0.15699E+01_JPRB, 0.16918E+01_JPRB ,&
&  0.19550E+01_JPRB, 0.19708E+01_JPRB, 0.19638E+01_JPRB, 0.21973E+01_JPRB, 0.21341E+01_JPRB, 0.24109E+01_JPRB, 0.30515E+01_JPRB ,&
&  0.32685E+01_JPRB, 0.32646E+01_JPRB, 0.34154E+01_JPRB, 0.29043E+01_JPRB, 0.27255E+01_JPRB, 0.37914E+01_JPRB, 0.38450E+01_JPRB ,&
&  0.37615E+01_JPRB, 0.37200E+01_JPRB, 0.35323E+01_JPRB, 0.35857E+01_JPRB, 0.31623E+01_JPRB, 0.28370E+01_JPRB, 0.21899E+01_JPRB ,&
&  0.23651E+01_JPRB, 0.23127E+01_JPRB, 0.25518E+01_JPRB, 0.23397E+01_JPRB, 0.22242E+01_JPRB, 0.19931E+01_JPRB, 0.16711E+01_JPRB ,&
&  0.14990E+01_JPRB, 0.14643E+01_JPRB, 0.17158E+01_JPRB, 0.25672E+01_JPRB, 0.35938E+01_JPRB, 0.35487E+01_JPRB, 0.41817E+01_JPRB ,&
&  0.45218E+01_JPRB, 0.45781E+01_JPRB, 0.39709E+01_JPRB, 0.38591E+01_JPRB, 0.36198E+01_JPRB, 0.33899E+01_JPRB, 0.31187E+01_JPRB ,&
&  0.28961E+01_JPRB, 0.27346E+01_JPRB, 0.25887E+01_JPRB, 0.24372E+01_JPRB, 0.22879E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.22191E+01_JPRB, 0.20997E+01_JPRB, 0.18858E+01_JPRB, 0.16205E+01_JPRB, 0.13836E+01_JPRB, 0.12349E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.12539E+01_JPRB, 0.12650E+01_JPRB, 0.12559E+01_JPRB, 0.12354E+01_JPRB, 0.12102E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.94952E+00_JPRB, 0.74527E+00_JPRB, 0.66214E+00_JPRB, 0.69411E+00_JPRB, 0.67635E+00_JPRB, 0.70403E+00_JPRB ,&
&  0.96493E+00_JPRB, 0.14521E+01_JPRB, 0.20680E+01_JPRB, 0.26174E+01_JPRB, 0.30464E+01_JPRB, 0.31052E+01_JPRB, 0.29304E+01_JPRB ,&
&  0.28736E+01_JPRB, 0.26228E+01_JPRB, 0.22849E+01_JPRB, 0.20077E+01_JPRB, 0.18234E+01_JPRB, 0.17007E+01_JPRB, 0.15765E+01_JPRB ,&
&  0.14581E+01_JPRB, 0.13880E+01_JPRB, 0.13614E+01_JPRB, 0.13471E+01_JPRB, 0.13557E+01_JPRB, 0.13519E+01_JPRB, 0.13501E+01_JPRB ,&
&  0.13919E+01_JPRB, 0.14599E+01_JPRB, 0.15394E+01_JPRB, 0.16453E+01_JPRB, 0.15727E+01_JPRB, 0.16094E+01_JPRB, 0.18049E+01_JPRB ,&
&  0.19480E+01_JPRB, 0.19610E+01_JPRB, 0.19771E+01_JPRB, 0.20670E+01_JPRB, 0.22719E+01_JPRB, 0.23751E+01_JPRB, 0.28565E+01_JPRB ,&
&  0.27380E+01_JPRB, 0.25733E+01_JPRB, 0.24863E+01_JPRB, 0.22834E+01_JPRB, 0.27543E+01_JPRB, 0.38664E+01_JPRB, 0.41945E+01_JPRB ,&
&  0.40667E+01_JPRB, 0.39474E+01_JPRB, 0.38466E+01_JPRB, 0.35057E+01_JPRB, 0.29365E+01_JPRB, 0.23187E+01_JPRB, 0.27754E+01_JPRB ,&
&  0.30153E+01_JPRB, 0.31797E+01_JPRB, 0.30162E+01_JPRB, 0.25014E+01_JPRB, 0.22462E+01_JPRB, 0.25873E+01_JPRB, 0.23843E+01_JPRB ,&
&  0.20399E+01_JPRB, 0.20179E+01_JPRB, 0.30173E+01_JPRB, 0.42956E+01_JPRB, 0.48420E+01_JPRB, 0.47306E+01_JPRB, 0.53125E+01_JPRB ,&
&  0.50849E+01_JPRB, 0.48535E+01_JPRB, 0.44524E+01_JPRB, 0.39752E+01_JPRB, 0.35722E+01_JPRB, 0.32395E+01_JPRB, 0.29693E+01_JPRB ,&
&  0.28164E+01_JPRB, 0.27262E+01_JPRB, 0.26120E+01_JPRB, 0.24928E+01_JPRB, 0.23428E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.22545E+01_JPRB, 0.21228E+01_JPRB, 0.18996E+01_JPRB, 0.16317E+01_JPRB, 0.14401E+01_JPRB, 0.13176E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12705E+01_JPRB, 0.12756E+01_JPRB, 0.12767E+01_JPRB, 0.12652E+01_JPRB, 0.12508E+01_JPRB, 0.12350E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11597E+01_JPRB, 0.88962E+00_JPRB, 0.76369E+00_JPRB, 0.75961E+00_JPRB, 0.79046E+00_JPRB, 0.81346E+00_JPRB ,&
&  0.12427E+01_JPRB, 0.20280E+01_JPRB, 0.26845E+01_JPRB, 0.31810E+01_JPRB, 0.35247E+01_JPRB, 0.30996E+01_JPRB, 0.29488E+01_JPRB ,&
&  0.28611E+01_JPRB, 0.26291E+01_JPRB, 0.23380E+01_JPRB, 0.20583E+01_JPRB, 0.18148E+01_JPRB, 0.16735E+01_JPRB, 0.15818E+01_JPRB ,&
&  0.14908E+01_JPRB, 0.14245E+01_JPRB, 0.13928E+01_JPRB, 0.13959E+01_JPRB, 0.13946E+01_JPRB, 0.13948E+01_JPRB, 0.14162E+01_JPRB ,&
&  0.14481E+01_JPRB, 0.14922E+01_JPRB, 0.15451E+01_JPRB, 0.16891E+01_JPRB, 0.17287E+01_JPRB, 0.17807E+01_JPRB, 0.20940E+01_JPRB ,&
&  0.20906E+01_JPRB, 0.21244E+01_JPRB, 0.20749E+01_JPRB, 0.21405E+01_JPRB, 0.22311E+01_JPRB, 0.23471E+01_JPRB, 0.27293E+01_JPRB ,&
&  0.26529E+01_JPRB, 0.26476E+01_JPRB, 0.28190E+01_JPRB, 0.28988E+01_JPRB, 0.28974E+01_JPRB, 0.33345E+01_JPRB, 0.45438E+01_JPRB ,&
&  0.38796E+01_JPRB, 0.38904E+01_JPRB, 0.39234E+01_JPRB, 0.38517E+01_JPRB, 0.22130E+01_JPRB, 0.17735E+01_JPRB, 0.27271E+01_JPRB ,&
&  0.28869E+01_JPRB, 0.18366E+01_JPRB, 0.13612E+01_JPRB, 0.18019E+01_JPRB, 0.14540E+01_JPRB, 0.18107E+01_JPRB, 0.26808E+01_JPRB ,&
&  0.29323E+01_JPRB, 0.27784E+01_JPRB, 0.40205E+01_JPRB, 0.47934E+01_JPRB, 0.49861E+01_JPRB, 0.57140E+01_JPRB, 0.57448E+01_JPRB ,&
&  0.54755E+01_JPRB, 0.44925E+01_JPRB, 0.43800E+01_JPRB, 0.38662E+01_JPRB, 0.34575E+01_JPRB, 0.32045E+01_JPRB, 0.30018E+01_JPRB ,&
&  0.27802E+01_JPRB, 0.25861E+01_JPRB, 0.24781E+01_JPRB, 0.24421E+01_JPRB, 0.23632E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.22356E+01_JPRB, 0.21011E+01_JPRB, 0.18702E+01_JPRB, 0.16115E+01_JPRB, 0.14581E+01_JPRB, 0.13515E+01_JPRB, 0.12922E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.12795E+01_JPRB, 0.12731E+01_JPRB, 0.12611E+01_JPRB, 0.12494E+01_JPRB, 0.12484E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.11506E+01_JPRB, 0.10484E+01_JPRB, 0.93992E+00_JPRB, 0.92462E+00_JPRB, 0.94329E+00_JPRB, 0.10745E+01_JPRB ,&
&  0.15532E+01_JPRB, 0.25228E+01_JPRB, 0.32314E+01_JPRB, 0.34344E+01_JPRB, 0.31721E+01_JPRB, 0.28159E+01_JPRB, 0.28967E+01_JPRB ,&
&  0.27602E+01_JPRB, 0.25269E+01_JPRB, 0.22549E+01_JPRB, 0.20495E+01_JPRB, 0.18695E+01_JPRB, 0.17459E+01_JPRB, 0.16671E+01_JPRB ,&
&  0.15687E+01_JPRB, 0.14957E+01_JPRB, 0.14749E+01_JPRB, 0.14601E+01_JPRB, 0.14496E+01_JPRB, 0.14513E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.14888E+01_JPRB, 0.15299E+01_JPRB, 0.15977E+01_JPRB, 0.17200E+01_JPRB, 0.19801E+01_JPRB, 0.20870E+01_JPRB, 0.20112E+01_JPRB ,&
&  0.19291E+01_JPRB, 0.19766E+01_JPRB, 0.22039E+01_JPRB, 0.21424E+01_JPRB, 0.21494E+01_JPRB, 0.22729E+01_JPRB, 0.25631E+01_JPRB ,&
&  0.29435E+01_JPRB, 0.31868E+01_JPRB, 0.29407E+01_JPRB, 0.30581E+01_JPRB, 0.31086E+01_JPRB, 0.28170E+01_JPRB, 0.36012E+01_JPRB ,&
&  0.39406E+01_JPRB, 0.34989E+01_JPRB, 0.28949E+01_JPRB, 0.24280E+01_JPRB, 0.26664E+01_JPRB, 0.21915E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.11144E+01_JPRB, 0.10126E+01_JPRB, 0.10929E+01_JPRB, 0.11015E+01_JPRB, 0.16196E+01_JPRB, 0.26543E+01_JPRB ,&
&  0.37086E+01_JPRB, 0.39938E+01_JPRB, 0.47125E+01_JPRB, 0.49777E+01_JPRB, 0.49152E+01_JPRB, 0.56978E+01_JPRB, 0.58430E+01_JPRB ,&
&  0.46807E+01_JPRB, 0.41174E+01_JPRB, 0.40089E+01_JPRB, 0.35515E+01_JPRB, 0.33062E+01_JPRB, 0.31197E+01_JPRB, 0.28804E+01_JPRB ,&
&  0.25870E+01_JPRB, 0.23316E+01_JPRB, 0.22388E+01_JPRB, 0.22734E+01_JPRB, 0.23153E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.21971E+01_JPRB, 0.20668E+01_JPRB, 0.18623E+01_JPRB, 0.16378E+01_JPRB, 0.15112E+01_JPRB, 0.14247E+01_JPRB, 0.13662E+01_JPRB ,&
&  0.13378E+01_JPRB, 0.13150E+01_JPRB, 0.12999E+01_JPRB, 0.12906E+01_JPRB, 0.12885E+01_JPRB, 0.12936E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.12578E+01_JPRB, 0.11795E+01_JPRB, 0.11490E+01_JPRB, 0.10936E+01_JPRB, 0.11265E+01_JPRB, 0.11439E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.19859E+01_JPRB, 0.30336E+01_JPRB, 0.36439E+01_JPRB, 0.35600E+01_JPRB, 0.32588E+01_JPRB, 0.29922E+01_JPRB, 0.27706E+01_JPRB ,&
&  0.25206E+01_JPRB, 0.23541E+01_JPRB, 0.21829E+01_JPRB, 0.20431E+01_JPRB, 0.19272E+01_JPRB, 0.18348E+01_JPRB, 0.17419E+01_JPRB ,&
&  0.16492E+01_JPRB, 0.16081E+01_JPRB, 0.15596E+01_JPRB, 0.15317E+01_JPRB, 0.15067E+01_JPRB, 0.14905E+01_JPRB, 0.14896E+01_JPRB ,&
&  0.15219E+01_JPRB, 0.15921E+01_JPRB, 0.17133E+01_JPRB, 0.19245E+01_JPRB, 0.19259E+01_JPRB, 0.17628E+01_JPRB, 0.18910E+01_JPRB ,&
&  0.19908E+01_JPRB, 0.21352E+01_JPRB, 0.21008E+01_JPRB, 0.22139E+01_JPRB, 0.21680E+01_JPRB, 0.21715E+01_JPRB, 0.24730E+01_JPRB ,&
&  0.28922E+01_JPRB, 0.33562E+01_JPRB, 0.28756E+01_JPRB, 0.28174E+01_JPRB, 0.33671E+01_JPRB, 0.30612E+01_JPRB, 0.32434E+01_JPRB ,&
&  0.36719E+01_JPRB, 0.32344E+01_JPRB, 0.29623E+01_JPRB, 0.28068E+01_JPRB, 0.39903E+01_JPRB, 0.42733E+01_JPRB, 0.21375E+01_JPRB ,&
&  0.15087E+01_JPRB, 0.12356E+01_JPRB, 0.10940E+01_JPRB, 0.10304E+01_JPRB, 0.10898E+01_JPRB, 0.17254E+01_JPRB, 0.33371E+01_JPRB ,&
&  0.39206E+01_JPRB, 0.49220E+01_JPRB, 0.54743E+01_JPRB, 0.55139E+01_JPRB, 0.52556E+01_JPRB, 0.52230E+01_JPRB, 0.46124E+01_JPRB ,&
&  0.45323E+01_JPRB, 0.41253E+01_JPRB, 0.36415E+01_JPRB, 0.32321E+01_JPRB, 0.29072E+01_JPRB, 0.25598E+01_JPRB, 0.22773E+01_JPRB ,&
&  0.20868E+01_JPRB, 0.20218E+01_JPRB, 0.20203E+01_JPRB, 0.21062E+01_JPRB, 0.22475E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.21580E+01_JPRB, 0.20865E+01_JPRB, 0.19072E+01_JPRB, 0.17059E+01_JPRB, 0.15843E+01_JPRB, 0.15017E+01_JPRB, 0.14479E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.13520E+01_JPRB, 0.13141E+01_JPRB, 0.12956E+01_JPRB, 0.13014E+01_JPRB, 0.13016E+01_JPRB, 0.12922E+01_JPRB ,&
&  0.12657E+01_JPRB, 0.12006E+01_JPRB, 0.11568E+01_JPRB, 0.12332E+01_JPRB, 0.12947E+01_JPRB, 0.14005E+01_JPRB, 0.17214E+01_JPRB ,&
&  0.25942E+01_JPRB, 0.37182E+01_JPRB, 0.39394E+01_JPRB, 0.37376E+01_JPRB, 0.33428E+01_JPRB, 0.30189E+01_JPRB, 0.24357E+01_JPRB ,&
&  0.21532E+01_JPRB, 0.21260E+01_JPRB, 0.20383E+01_JPRB, 0.20372E+01_JPRB, 0.20037E+01_JPRB, 0.19171E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.16474E+01_JPRB, 0.15968E+01_JPRB, 0.15600E+01_JPRB, 0.15285E+01_JPRB, 0.15043E+01_JPRB, 0.15286E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.16912E+01_JPRB, 0.18563E+01_JPRB, 0.20151E+01_JPRB, 0.17069E+01_JPRB, 0.18426E+01_JPRB, 0.18993E+01_JPRB ,&
&  0.19350E+01_JPRB, 0.19804E+01_JPRB, 0.19079E+01_JPRB, 0.19825E+01_JPRB, 0.21039E+01_JPRB, 0.21573E+01_JPRB, 0.23625E+01_JPRB ,&
&  0.29453E+01_JPRB, 0.27858E+01_JPRB, 0.25663E+01_JPRB, 0.26458E+01_JPRB, 0.31008E+01_JPRB, 0.41984E+01_JPRB, 0.30150E+01_JPRB ,&
&  0.28150E+01_JPRB, 0.32727E+01_JPRB, 0.32201E+01_JPRB, 0.34292E+01_JPRB, 0.45645E+01_JPRB, 0.59244E+01_JPRB, 0.56021E+01_JPRB ,&
&  0.22286E+01_JPRB, 0.14451E+01_JPRB, 0.14162E+01_JPRB, 0.13797E+01_JPRB, 0.13145E+01_JPRB, 0.18240E+01_JPRB, 0.57634E+01_JPRB ,&
&  0.49506E+01_JPRB, 0.58899E+01_JPRB, 0.59582E+01_JPRB, 0.54621E+01_JPRB, 0.52512E+01_JPRB, 0.49825E+01_JPRB, 0.44808E+01_JPRB ,&
&  0.39426E+01_JPRB, 0.34497E+01_JPRB, 0.29628E+01_JPRB, 0.24897E+01_JPRB, 0.21211E+01_JPRB, 0.18855E+01_JPRB, 0.17906E+01_JPRB ,&
&  0.17768E+01_JPRB, 0.18122E+01_JPRB, 0.18740E+01_JPRB, 0.19789E+01_JPRB, 0.21278E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.21242E+01_JPRB, 0.20966E+01_JPRB, 0.19247E+01_JPRB, 0.17479E+01_JPRB, 0.16312E+01_JPRB, 0.15646E+01_JPRB, 0.14837E+01_JPRB ,&
&  0.14532E+01_JPRB, 0.13775E+01_JPRB, 0.13050E+01_JPRB, 0.12654E+01_JPRB, 0.12670E+01_JPRB, 0.12624E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.12075E+01_JPRB, 0.11772E+01_JPRB, 0.12114E+01_JPRB, 0.13752E+01_JPRB, 0.13187E+01_JPRB, 0.16357E+01_JPRB ,&
&  0.30198E+01_JPRB, 0.45554E+01_JPRB, 0.40180E+01_JPRB, 0.36654E+01_JPRB, 0.32560E+01_JPRB, 0.30057E+01_JPRB, 0.21860E+01_JPRB ,&
&  0.19850E+01_JPRB, 0.19980E+01_JPRB, 0.19872E+01_JPRB, 0.20204E+01_JPRB, 0.19902E+01_JPRB, 0.19155E+01_JPRB, 0.18236E+01_JPRB ,&
&  0.17104E+01_JPRB, 0.16348E+01_JPRB, 0.16136E+01_JPRB, 0.15876E+01_JPRB, 0.15533E+01_JPRB, 0.15554E+01_JPRB, 0.16085E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.17760E+01_JPRB, 0.19571E+01_JPRB, 0.19277E+01_JPRB, 0.18489E+01_JPRB, 0.18532E+01_JPRB, 0.19077E+01_JPRB ,&
&  0.18914E+01_JPRB, 0.19124E+01_JPRB, 0.19167E+01_JPRB, 0.19037E+01_JPRB, 0.20101E+01_JPRB, 0.21764E+01_JPRB, 0.23418E+01_JPRB ,&
&  0.26255E+01_JPRB, 0.24931E+01_JPRB, 0.23002E+01_JPRB, 0.24770E+01_JPRB, 0.29654E+01_JPRB, 0.38586E+01_JPRB, 0.38822E+01_JPRB ,&
&  0.35422E+01_JPRB, 0.29050E+01_JPRB, 0.31580E+01_JPRB, 0.42801E+01_JPRB, 0.50972E+01_JPRB, 0.58643E+01_JPRB, 0.82307E+01_JPRB ,&
&  0.81774E+01_JPRB, 0.43826E+01_JPRB, 0.34348E+01_JPRB, 0.43912E+01_JPRB, 0.29743E+01_JPRB, 0.22628E+01_JPRB, 0.38236E+01_JPRB ,&
&  0.48587E+01_JPRB, 0.56943E+01_JPRB, 0.58479E+01_JPRB, 0.53951E+01_JPRB, 0.52518E+01_JPRB, 0.47316E+01_JPRB, 0.38315E+01_JPRB ,&
&  0.31254E+01_JPRB, 0.25120E+01_JPRB, 0.20909E+01_JPRB, 0.18673E+01_JPRB, 0.17767E+01_JPRB, 0.17298E+01_JPRB, 0.17085E+01_JPRB ,&
&  0.17159E+01_JPRB, 0.17550E+01_JPRB, 0.18297E+01_JPRB, 0.19612E+01_JPRB, 0.20897E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.20796E+01_JPRB, 0.20311E+01_JPRB, 0.18821E+01_JPRB, 0.17433E+01_JPRB, 0.16498E+01_JPRB, 0.15741E+01_JPRB, 0.14780E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.13995E+01_JPRB, 0.12804E+01_JPRB, 0.12315E+01_JPRB, 0.12113E+01_JPRB, 0.12128E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.12050E+01_JPRB, 0.12061E+01_JPRB, 0.12508E+01_JPRB, 0.14027E+01_JPRB, 0.18680E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.31301E+01_JPRB, 0.52676E+01_JPRB, 0.41393E+01_JPRB, 0.36382E+01_JPRB, 0.32546E+01_JPRB, 0.30676E+01_JPRB, 0.22009E+01_JPRB ,&
&  0.19567E+01_JPRB, 0.19289E+01_JPRB, 0.19202E+01_JPRB, 0.19215E+01_JPRB, 0.19173E+01_JPRB, 0.18663E+01_JPRB, 0.17368E+01_JPRB ,&
&  0.16460E+01_JPRB, 0.16031E+01_JPRB, 0.16055E+01_JPRB, 0.15991E+01_JPRB, 0.15950E+01_JPRB, 0.16274E+01_JPRB, 0.16594E+01_JPRB ,&
&  0.17283E+01_JPRB, 0.19163E+01_JPRB, 0.20492E+01_JPRB, 0.18916E+01_JPRB, 0.18724E+01_JPRB, 0.19013E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.18343E+01_JPRB, 0.17240E+01_JPRB, 0.18303E+01_JPRB, 0.19330E+01_JPRB, 0.20194E+01_JPRB, 0.20896E+01_JPRB, 0.21523E+01_JPRB ,&
&  0.24049E+01_JPRB, 0.25012E+01_JPRB, 0.26690E+01_JPRB, 0.26267E+01_JPRB, 0.29512E+01_JPRB, 0.36178E+01_JPRB, 0.39336E+01_JPRB ,&
&  0.42372E+01_JPRB, 0.39769E+01_JPRB, 0.45181E+01_JPRB, 0.52448E+01_JPRB, 0.55193E+01_JPRB, 0.56416E+01_JPRB, 0.65071E+01_JPRB ,&
&  0.75054E+01_JPRB, 0.89653E+01_JPRB, 0.79427E+01_JPRB, 0.61308E+01_JPRB, 0.45655E+01_JPRB, 0.33088E+01_JPRB, 0.39336E+01_JPRB ,&
&  0.51054E+01_JPRB, 0.52569E+01_JPRB, 0.52399E+01_JPRB, 0.64680E+01_JPRB, 0.48285E+01_JPRB, 0.35932E+01_JPRB, 0.26263E+01_JPRB ,&
&  0.20681E+01_JPRB, 0.18622E+01_JPRB, 0.18147E+01_JPRB, 0.17587E+01_JPRB, 0.17333E+01_JPRB, 0.17100E+01_JPRB, 0.17184E+01_JPRB ,&
&  0.17620E+01_JPRB, 0.18302E+01_JPRB, 0.19106E+01_JPRB, 0.20082E+01_JPRB, 0.20821E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.19709E+01_JPRB, 0.18900E+01_JPRB, 0.17846E+01_JPRB, 0.16832E+01_JPRB, 0.15898E+01_JPRB, 0.15051E+01_JPRB, 0.14538E+01_JPRB ,&
&  0.16451E+01_JPRB, 0.13428E+01_JPRB, 0.12245E+01_JPRB, 0.11883E+01_JPRB, 0.11778E+01_JPRB, 0.11743E+01_JPRB, 0.11894E+01_JPRB ,&
&  0.12081E+01_JPRB, 0.12295E+01_JPRB, 0.12830E+01_JPRB, 0.13931E+01_JPRB, 0.16673E+01_JPRB, 0.23298E+01_JPRB, 0.29357E+01_JPRB ,&
&  0.35068E+01_JPRB, 0.62215E+01_JPRB, 0.42187E+01_JPRB, 0.35873E+01_JPRB, 0.32768E+01_JPRB, 0.31506E+01_JPRB, 0.24465E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.18919E+01_JPRB, 0.18577E+01_JPRB, 0.18174E+01_JPRB, 0.17864E+01_JPRB, 0.17154E+01_JPRB, 0.16582E+01_JPRB ,&
&  0.16203E+01_JPRB, 0.16078E+01_JPRB, 0.16079E+01_JPRB, 0.16102E+01_JPRB, 0.16132E+01_JPRB, 0.16645E+01_JPRB, 0.17423E+01_JPRB ,&
&  0.19012E+01_JPRB, 0.20893E+01_JPRB, 0.20054E+01_JPRB, 0.19067E+01_JPRB, 0.18670E+01_JPRB, 0.19109E+01_JPRB, 0.19380E+01_JPRB ,&
&  0.18584E+01_JPRB, 0.17626E+01_JPRB, 0.18461E+01_JPRB, 0.18758E+01_JPRB, 0.18858E+01_JPRB, 0.20260E+01_JPRB, 0.20523E+01_JPRB ,&
&  0.23696E+01_JPRB, 0.24979E+01_JPRB, 0.31952E+01_JPRB, 0.30526E+01_JPRB, 0.29130E+01_JPRB, 0.34692E+01_JPRB, 0.35571E+01_JPRB ,&
&  0.33692E+01_JPRB, 0.39615E+01_JPRB, 0.47869E+01_JPRB, 0.56616E+01_JPRB, 0.59669E+01_JPRB, 0.61329E+01_JPRB, 0.60349E+01_JPRB ,&
&  0.54949E+01_JPRB, 0.57452E+01_JPRB, 0.47771E+01_JPRB, 0.39520E+01_JPRB, 0.40504E+01_JPRB, 0.36033E+01_JPRB, 0.44243E+01_JPRB ,&
&  0.54641E+01_JPRB, 0.54881E+01_JPRB, 0.56937E+01_JPRB, 0.53406E+01_JPRB, 0.33837E+01_JPRB, 0.23662E+01_JPRB, 0.18556E+01_JPRB ,&
&  0.17354E+01_JPRB, 0.17567E+01_JPRB, 0.17794E+01_JPRB, 0.17574E+01_JPRB, 0.17479E+01_JPRB, 0.17526E+01_JPRB, 0.17928E+01_JPRB ,&
&  0.18669E+01_JPRB, 0.19482E+01_JPRB, 0.19902E+01_JPRB, 0.20078E+01_JPRB, 0.20194E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.18583E+01_JPRB, 0.17886E+01_JPRB, 0.17120E+01_JPRB, 0.16451E+01_JPRB, 0.16054E+01_JPRB, 0.15264E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.17072E+01_JPRB, 0.12736E+01_JPRB, 0.11691E+01_JPRB, 0.11464E+01_JPRB, 0.11515E+01_JPRB, 0.11753E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.13552E+01_JPRB, 0.14809E+01_JPRB, 0.17459E+01_JPRB, 0.22846E+01_JPRB, 0.32660E+01_JPRB, 0.42068E+01_JPRB ,&
&  0.53106E+01_JPRB, 0.52113E+01_JPRB, 0.42213E+01_JPRB, 0.32339E+01_JPRB, 0.30228E+01_JPRB, 0.29868E+01_JPRB, 0.28693E+01_JPRB ,&
&  0.22565E+01_JPRB, 0.20046E+01_JPRB, 0.19093E+01_JPRB, 0.18051E+01_JPRB, 0.16738E+01_JPRB, 0.16513E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.16385E+01_JPRB, 0.16405E+01_JPRB, 0.16556E+01_JPRB, 0.17101E+01_JPRB, 0.18297E+01_JPRB, 0.19659E+01_JPRB ,&
&  0.21114E+01_JPRB, 0.21707E+01_JPRB, 0.20542E+01_JPRB, 0.19347E+01_JPRB, 0.18905E+01_JPRB, 0.19292E+01_JPRB, 0.19375E+01_JPRB ,&
&  0.19019E+01_JPRB, 0.18971E+01_JPRB, 0.19212E+01_JPRB, 0.19160E+01_JPRB, 0.18454E+01_JPRB, 0.20376E+01_JPRB, 0.21285E+01_JPRB ,&
&  0.24293E+01_JPRB, 0.27375E+01_JPRB, 0.35305E+01_JPRB, 0.29624E+01_JPRB, 0.31619E+01_JPRB, 0.33339E+01_JPRB, 0.34520E+01_JPRB ,&
&  0.35429E+01_JPRB, 0.43695E+01_JPRB, 0.52609E+01_JPRB, 0.57447E+01_JPRB, 0.63676E+01_JPRB, 0.60978E+01_JPRB, 0.63864E+01_JPRB ,&
&  0.47472E+01_JPRB, 0.44835E+01_JPRB, 0.33409E+01_JPRB, 0.35846E+01_JPRB, 0.37431E+01_JPRB, 0.36963E+01_JPRB, 0.41034E+01_JPRB ,&
&  0.49073E+01_JPRB, 0.47322E+01_JPRB, 0.41693E+01_JPRB, 0.32492E+01_JPRB, 0.20583E+01_JPRB, 0.16911E+01_JPRB, 0.15870E+01_JPRB ,&
&  0.16113E+01_JPRB, 0.17366E+01_JPRB, 0.17751E+01_JPRB, 0.17970E+01_JPRB, 0.17962E+01_JPRB, 0.18137E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.19176E+01_JPRB, 0.19602E+01_JPRB, 0.19720E+01_JPRB, 0.19467E+01_JPRB, 0.19145E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.17317E+01_JPRB, 0.16711E+01_JPRB, 0.16180E+01_JPRB, 0.15418E+01_JPRB, 0.14110E+01_JPRB, 0.13159E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.11777E+01_JPRB, 0.11872E+01_JPRB, 0.12008E+01_JPRB, 0.12307E+01_JPRB, 0.12878E+01_JPRB, 0.13702E+01_JPRB ,&
&  0.14677E+01_JPRB, 0.16265E+01_JPRB, 0.19199E+01_JPRB, 0.23598E+01_JPRB, 0.29098E+01_JPRB, 0.37391E+01_JPRB, 0.49105E+01_JPRB ,&
&  0.51860E+01_JPRB, 0.33093E+01_JPRB, 0.34252E+01_JPRB, 0.30386E+01_JPRB, 0.28973E+01_JPRB, 0.26285E+01_JPRB, 0.24232E+01_JPRB ,&
&  0.22405E+01_JPRB, 0.21308E+01_JPRB, 0.20794E+01_JPRB, 0.19447E+01_JPRB, 0.17312E+01_JPRB, 0.17420E+01_JPRB, 0.17559E+01_JPRB ,&
&  0.17593E+01_JPRB, 0.17804E+01_JPRB, 0.18191E+01_JPRB, 0.18876E+01_JPRB, 0.20022E+01_JPRB, 0.21289E+01_JPRB, 0.22268E+01_JPRB ,&
&  0.22704E+01_JPRB, 0.22973E+01_JPRB, 0.22402E+01_JPRB, 0.21529E+01_JPRB, 0.20642E+01_JPRB, 0.20556E+01_JPRB, 0.20114E+01_JPRB ,&
&  0.19771E+01_JPRB, 0.19375E+01_JPRB, 0.19909E+01_JPRB, 0.21052E+01_JPRB, 0.21836E+01_JPRB, 0.21004E+01_JPRB, 0.23420E+01_JPRB ,&
&  0.26053E+01_JPRB, 0.29815E+01_JPRB, 0.32177E+01_JPRB, 0.39569E+01_JPRB, 0.28353E+01_JPRB, 0.30794E+01_JPRB, 0.31269E+01_JPRB ,&
&  0.42003E+01_JPRB, 0.47289E+01_JPRB, 0.52332E+01_JPRB, 0.56729E+01_JPRB, 0.65289E+01_JPRB, 0.60025E+01_JPRB, 0.55461E+01_JPRB ,&
&  0.43537E+01_JPRB, 0.33817E+01_JPRB, 0.28421E+01_JPRB, 0.31766E+01_JPRB, 0.35696E+01_JPRB, 0.40995E+01_JPRB, 0.39377E+01_JPRB ,&
&  0.40068E+01_JPRB, 0.33807E+01_JPRB, 0.26831E+01_JPRB, 0.23736E+01_JPRB, 0.17317E+01_JPRB, 0.15734E+01_JPRB, 0.15932E+01_JPRB ,&
&  0.16544E+01_JPRB, 0.17596E+01_JPRB, 0.17895E+01_JPRB, 0.17898E+01_JPRB, 0.17916E+01_JPRB, 0.18001E+01_JPRB, 0.18212E+01_JPRB ,&
&  0.18418E+01_JPRB, 0.18629E+01_JPRB, 0.18630E+01_JPRB, 0.18287E+01_JPRB, 0.17807E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.16136E+01_JPRB, 0.15494E+01_JPRB, 0.14414E+01_JPRB, 0.13107E+01_JPRB, 0.12137E+01_JPRB, 0.11606E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11611E+01_JPRB, 0.11965E+01_JPRB, 0.12389E+01_JPRB, 0.12955E+01_JPRB, 0.13524E+01_JPRB, 0.14173E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.16830E+01_JPRB, 0.19166E+01_JPRB, 0.22643E+01_JPRB, 0.26582E+01_JPRB, 0.30184E+01_JPRB, 0.33888E+01_JPRB, 0.37555E+01_JPRB ,&
&  0.41575E+01_JPRB, 0.41402E+01_JPRB, 0.35207E+01_JPRB, 0.24041E+01_JPRB, 0.21317E+01_JPRB, 0.22349E+01_JPRB, 0.21558E+01_JPRB ,&
&  0.21095E+01_JPRB, 0.20375E+01_JPRB, 0.19800E+01_JPRB, 0.19434E+01_JPRB, 0.19270E+01_JPRB, 0.19499E+01_JPRB, 0.19698E+01_JPRB ,&
&  0.20107E+01_JPRB, 0.20620E+01_JPRB, 0.21237E+01_JPRB, 0.21978E+01_JPRB, 0.22601E+01_JPRB, 0.23285E+01_JPRB, 0.24057E+01_JPRB ,&
&  0.24620E+01_JPRB, 0.25496E+01_JPRB, 0.26019E+01_JPRB, 0.25815E+01_JPRB, 0.24473E+01_JPRB, 0.23841E+01_JPRB, 0.22945E+01_JPRB ,&
&  0.22527E+01_JPRB, 0.22139E+01_JPRB, 0.22359E+01_JPRB, 0.23690E+01_JPRB, 0.24627E+01_JPRB, 0.22886E+01_JPRB, 0.23960E+01_JPRB ,&
&  0.28540E+01_JPRB, 0.33464E+01_JPRB, 0.31292E+01_JPRB, 0.38893E+01_JPRB, 0.24605E+01_JPRB, 0.30397E+01_JPRB, 0.37940E+01_JPRB ,&
&  0.40489E+01_JPRB, 0.44607E+01_JPRB, 0.49849E+01_JPRB, 0.54125E+01_JPRB, 0.55538E+01_JPRB, 0.51466E+01_JPRB, 0.47366E+01_JPRB ,&
&  0.38672E+01_JPRB, 0.29439E+01_JPRB, 0.25558E+01_JPRB, 0.26700E+01_JPRB, 0.31200E+01_JPRB, 0.36347E+01_JPRB, 0.29088E+01_JPRB ,&
&  0.22425E+01_JPRB, 0.23800E+01_JPRB, 0.22221E+01_JPRB, 0.22999E+01_JPRB, 0.18063E+01_JPRB, 0.15361E+01_JPRB, 0.16195E+01_JPRB ,&
&  0.17206E+01_JPRB, 0.17404E+01_JPRB, 0.17647E+01_JPRB, 0.17487E+01_JPRB, 0.17501E+01_JPRB, 0.17475E+01_JPRB, 0.17441E+01_JPRB ,&
&  0.17515E+01_JPRB, 0.17702E+01_JPRB, 0.17681E+01_JPRB, 0.17197E+01_JPRB, 0.16609E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.14476E+01_JPRB, 0.13248E+01_JPRB, 0.11924E+01_JPRB, 0.10994E+01_JPRB, 0.10581E+01_JPRB, 0.10544E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11706E+01_JPRB, 0.12364E+01_JPRB, 0.13198E+01_JPRB, 0.13996E+01_JPRB, 0.14892E+01_JPRB, 0.16148E+01_JPRB ,&
&  0.17629E+01_JPRB, 0.19681E+01_JPRB, 0.22416E+01_JPRB, 0.25434E+01_JPRB, 0.27863E+01_JPRB, 0.30132E+01_JPRB, 0.32250E+01_JPRB ,&
&  0.34271E+01_JPRB, 0.34876E+01_JPRB, 0.33511E+01_JPRB, 0.31387E+01_JPRB, 0.25178E+01_JPRB, 0.26204E+01_JPRB, 0.27693E+01_JPRB ,&
&  0.28265E+01_JPRB, 0.23271E+01_JPRB, 0.21395E+01_JPRB, 0.21055E+01_JPRB, 0.21198E+01_JPRB, 0.21586E+01_JPRB, 0.21908E+01_JPRB ,&
&  0.22329E+01_JPRB, 0.22751E+01_JPRB, 0.23147E+01_JPRB, 0.23490E+01_JPRB, 0.23860E+01_JPRB, 0.24665E+01_JPRB, 0.26164E+01_JPRB ,&
&  0.26808E+01_JPRB, 0.27447E+01_JPRB, 0.27903E+01_JPRB, 0.27890E+01_JPRB, 0.29147E+01_JPRB, 0.30075E+01_JPRB, 0.29410E+01_JPRB ,&
&  0.28681E+01_JPRB, 0.26939E+01_JPRB, 0.27670E+01_JPRB, 0.29627E+01_JPRB, 0.29830E+01_JPRB, 0.28353E+01_JPRB, 0.28951E+01_JPRB ,&
&  0.33247E+01_JPRB, 0.38258E+01_JPRB, 0.28437E+01_JPRB, 0.32835E+01_JPRB, 0.39044E+01_JPRB, 0.34361E+01_JPRB, 0.32589E+01_JPRB ,&
&  0.31027E+01_JPRB, 0.33672E+01_JPRB, 0.41154E+01_JPRB, 0.44585E+01_JPRB, 0.44168E+01_JPRB, 0.51208E+01_JPRB, 0.41661E+01_JPRB ,&
&  0.33543E+01_JPRB, 0.25195E+01_JPRB, 0.22175E+01_JPRB, 0.21912E+01_JPRB, 0.24191E+01_JPRB, 0.24951E+01_JPRB, 0.17613E+01_JPRB ,&
&  0.15471E+01_JPRB, 0.19432E+01_JPRB, 0.19727E+01_JPRB, 0.21731E+01_JPRB, 0.20718E+01_JPRB, 0.14645E+01_JPRB, 0.15464E+01_JPRB ,&
&  0.16191E+01_JPRB, 0.16696E+01_JPRB, 0.16800E+01_JPRB, 0.16757E+01_JPRB, 0.16696E+01_JPRB, 0.16646E+01_JPRB, 0.16662E+01_JPRB ,&
&  0.16712E+01_JPRB, 0.16716E+01_JPRB, 0.16584E+01_JPRB, 0.16078E+01_JPRB, 0.15346E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.11682E+01_JPRB, 0.10563E+01_JPRB, 0.97864E+00_JPRB, 0.95506E+00_JPRB, 0.96651E+00_JPRB, 0.98630E+00_JPRB, 0.99948E+00_JPRB ,&
&  0.10311E+01_JPRB, 0.10779E+01_JPRB, 0.11443E+01_JPRB, 0.12048E+01_JPRB, 0.12778E+01_JPRB, 0.13519E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.15858E+01_JPRB, 0.17161E+01_JPRB, 0.19128E+01_JPRB, 0.21916E+01_JPRB, 0.24055E+01_JPRB, 0.25949E+01_JPRB, 0.28614E+01_JPRB ,&
&  0.31404E+01_JPRB, 0.33174E+01_JPRB, 0.35061E+01_JPRB, 0.36238E+01_JPRB, 0.35382E+01_JPRB, 0.34175E+01_JPRB, 0.35501E+01_JPRB ,&
&  0.29399E+01_JPRB, 0.22953E+01_JPRB, 0.22207E+01_JPRB, 0.22005E+01_JPRB, 0.22520E+01_JPRB, 0.22735E+01_JPRB, 0.22941E+01_JPRB ,&
&  0.23275E+01_JPRB, 0.23493E+01_JPRB, 0.23689E+01_JPRB, 0.23991E+01_JPRB, 0.24623E+01_JPRB, 0.25810E+01_JPRB, 0.27447E+01_JPRB ,&
&  0.28097E+01_JPRB, 0.28933E+01_JPRB, 0.30157E+01_JPRB, 0.28003E+01_JPRB, 0.30289E+01_JPRB, 0.33588E+01_JPRB, 0.35497E+01_JPRB ,&
&  0.35285E+01_JPRB, 0.33268E+01_JPRB, 0.34098E+01_JPRB, 0.35161E+01_JPRB, 0.35742E+01_JPRB, 0.35400E+01_JPRB, 0.35596E+01_JPRB ,&
&  0.38046E+01_JPRB, 0.36904E+01_JPRB, 0.27828E+01_JPRB, 0.26869E+01_JPRB, 0.28482E+01_JPRB, 0.24398E+01_JPRB, 0.25670E+01_JPRB ,&
&  0.22843E+01_JPRB, 0.24550E+01_JPRB, 0.31081E+01_JPRB, 0.36360E+01_JPRB, 0.39114E+01_JPRB, 0.45053E+01_JPRB, 0.38160E+01_JPRB ,&
&  0.26119E+01_JPRB, 0.19546E+01_JPRB, 0.16467E+01_JPRB, 0.15251E+01_JPRB, 0.16424E+01_JPRB, 0.16888E+01_JPRB, 0.15188E+01_JPRB ,&
&  0.15352E+01_JPRB, 0.15668E+01_JPRB, 0.16157E+01_JPRB, 0.15343E+01_JPRB, 0.13682E+01_JPRB, 0.13199E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.14325E+01_JPRB, 0.14714E+01_JPRB, 0.14931E+01_JPRB, 0.14948E+01_JPRB, 0.15012E+01_JPRB, 0.15027E+01_JPRB, 0.15077E+01_JPRB ,&
&  0.15129E+01_JPRB, 0.15020E+01_JPRB, 0.14639E+01_JPRB, 0.14032E+01_JPRB, 0.12985E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.92348E+00_JPRB, 0.85814E+00_JPRB, 0.83991E+00_JPRB, 0.85018E+00_JPRB, 0.86371E+00_JPRB, 0.87828E+00_JPRB, 0.89488E+00_JPRB ,&
&  0.90984E+00_JPRB, 0.92292E+00_JPRB, 0.94332E+00_JPRB, 0.96044E+00_JPRB, 0.10182E+01_JPRB, 0.10868E+01_JPRB, 0.11732E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.14115E+01_JPRB, 0.15104E+01_JPRB, 0.16650E+01_JPRB, 0.17754E+01_JPRB, 0.18920E+01_JPRB, 0.20741E+01_JPRB ,&
&  0.23099E+01_JPRB, 0.25936E+01_JPRB, 0.28716E+01_JPRB, 0.31857E+01_JPRB, 0.36112E+01_JPRB, 0.40147E+01_JPRB, 0.41160E+01_JPRB ,&
&  0.30942E+01_JPRB, 0.21162E+01_JPRB, 0.22178E+01_JPRB, 0.20822E+01_JPRB, 0.21578E+01_JPRB, 0.22216E+01_JPRB, 0.22410E+01_JPRB ,&
&  0.22785E+01_JPRB, 0.23036E+01_JPRB, 0.23263E+01_JPRB, 0.23536E+01_JPRB, 0.24324E+01_JPRB, 0.25399E+01_JPRB, 0.26815E+01_JPRB ,&
&  0.27711E+01_JPRB, 0.28445E+01_JPRB, 0.28681E+01_JPRB, 0.28645E+01_JPRB, 0.30749E+01_JPRB, 0.34175E+01_JPRB, 0.36667E+01_JPRB ,&
&  0.38087E+01_JPRB, 0.39455E+01_JPRB, 0.36346E+01_JPRB, 0.36840E+01_JPRB, 0.36727E+01_JPRB, 0.33769E+01_JPRB, 0.34762E+01_JPRB ,&
&  0.37036E+01_JPRB, 0.34465E+01_JPRB, 0.23313E+01_JPRB, 0.22988E+01_JPRB, 0.23304E+01_JPRB, 0.22174E+01_JPRB, 0.21615E+01_JPRB ,&
&  0.17767E+01_JPRB, 0.18389E+01_JPRB, 0.21393E+01_JPRB, 0.28184E+01_JPRB, 0.31853E+01_JPRB, 0.33761E+01_JPRB, 0.29789E+01_JPRB ,&
&  0.20288E+01_JPRB, 0.15721E+01_JPRB, 0.13406E+01_JPRB, 0.13609E+01_JPRB, 0.16240E+01_JPRB, 0.19321E+01_JPRB, 0.15601E+01_JPRB ,&
&  0.15994E+01_JPRB, 0.18967E+01_JPRB, 0.23907E+01_JPRB, 0.18188E+01_JPRB, 0.12346E+01_JPRB, 0.11516E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.11932E+01_JPRB, 0.12187E+01_JPRB, 0.12325E+01_JPRB, 0.12395E+01_JPRB, 0.12490E+01_JPRB, 0.12697E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.12298E+01_JPRB, 0.11958E+01_JPRB, 0.11034E+01_JPRB, 0.10051E+01_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.76255E+00_JPRB, 0.75384E+00_JPRB, 0.75677E+00_JPRB, 0.77369E+00_JPRB, 0.77543E+00_JPRB, 0.77779E+00_JPRB, 0.81356E+00_JPRB ,&
&  0.80497E+00_JPRB, 0.79822E+00_JPRB, 0.80754E+00_JPRB, 0.79058E+00_JPRB, 0.82794E+00_JPRB, 0.86734E+00_JPRB, 0.92940E+00_JPRB ,&
&  0.99255E+00_JPRB, 0.10351E+01_JPRB, 0.10690E+01_JPRB, 0.11170E+01_JPRB, 0.12092E+01_JPRB, 0.12825E+01_JPRB, 0.13466E+01_JPRB ,&
&  0.14618E+01_JPRB, 0.16344E+01_JPRB, 0.18213E+01_JPRB, 0.22299E+01_JPRB, 0.31594E+01_JPRB, 0.39115E+01_JPRB, 0.39253E+01_JPRB ,&
&  0.29245E+01_JPRB, 0.20422E+01_JPRB, 0.18933E+01_JPRB, 0.17244E+01_JPRB, 0.18026E+01_JPRB, 0.19738E+01_JPRB, 0.20192E+01_JPRB ,&
&  0.20023E+01_JPRB, 0.20181E+01_JPRB, 0.20230E+01_JPRB, 0.20356E+01_JPRB, 0.20597E+01_JPRB, 0.20745E+01_JPRB, 0.20481E+01_JPRB ,&
&  0.19722E+01_JPRB, 0.18759E+01_JPRB, 0.19320E+01_JPRB, 0.23079E+01_JPRB, 0.26872E+01_JPRB, 0.32400E+01_JPRB, 0.31771E+01_JPRB ,&
&  0.30425E+01_JPRB, 0.29897E+01_JPRB, 0.29999E+01_JPRB, 0.29798E+01_JPRB, 0.29012E+01_JPRB, 0.27963E+01_JPRB, 0.28736E+01_JPRB ,&
&  0.30458E+01_JPRB, 0.28534E+01_JPRB, 0.19919E+01_JPRB, 0.20608E+01_JPRB, 0.20414E+01_JPRB, 0.20718E+01_JPRB, 0.18149E+01_JPRB ,&
&  0.14747E+01_JPRB, 0.14237E+01_JPRB, 0.15422E+01_JPRB, 0.18182E+01_JPRB, 0.22907E+01_JPRB, 0.24538E+01_JPRB, 0.21634E+01_JPRB ,&
&  0.16595E+01_JPRB, 0.13388E+01_JPRB, 0.11853E+01_JPRB, 0.12384E+01_JPRB, 0.16556E+01_JPRB, 0.19888E+01_JPRB, 0.21596E+01_JPRB ,&
&  0.14888E+01_JPRB, 0.16892E+01_JPRB, 0.14979E+01_JPRB, 0.18428E+01_JPRB, 0.10261E+01_JPRB, 0.89992E+00_JPRB, 0.89083E+00_JPRB ,&
&  0.90560E+00_JPRB, 0.89907E+00_JPRB, 0.89092E+00_JPRB, 0.87192E+00_JPRB, 0.85429E+00_JPRB, 0.92082E+00_JPRB, 0.98336E+00_JPRB ,&
&  0.92971E+00_JPRB, 0.96035E+00_JPRB, 0.91632E+00_JPRB, 0.84353E+00_JPRB, 0.80674E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.71842E+00_JPRB, 0.72930E+00_JPRB, 0.73363E+00_JPRB, 0.74543E+00_JPRB, 0.76863E+00_JPRB, 0.78822E+00_JPRB, 0.81380E+00_JPRB ,&
&  0.84268E+00_JPRB, 0.86077E+00_JPRB, 0.87992E+00_JPRB, 0.90432E+00_JPRB, 0.92487E+00_JPRB, 0.96126E+00_JPRB, 0.99568E+00_JPRB ,&
&  0.10385E+01_JPRB, 0.10732E+01_JPRB, 0.11092E+01_JPRB, 0.11646E+01_JPRB, 0.12331E+01_JPRB, 0.13346E+01_JPRB, 0.14448E+01_JPRB ,&
&  0.15918E+01_JPRB, 0.17663E+01_JPRB, 0.19759E+01_JPRB, 0.21538E+01_JPRB, 0.22526E+01_JPRB, 0.23266E+01_JPRB, 0.26830E+01_JPRB ,&
&  0.22588E+01_JPRB, 0.21068E+01_JPRB, 0.19613E+01_JPRB, 0.17500E+01_JPRB, 0.17259E+01_JPRB, 0.15933E+01_JPRB, 0.15390E+01_JPRB ,&
&  0.15310E+01_JPRB, 0.14732E+01_JPRB, 0.14391E+01_JPRB, 0.14096E+01_JPRB, 0.13706E+01_JPRB, 0.12949E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.11895E+01_JPRB, 0.12609E+01_JPRB, 0.14285E+01_JPRB, 0.17804E+01_JPRB, 0.21328E+01_JPRB, 0.22659E+01_JPRB, 0.21106E+01_JPRB ,&
&  0.20262E+01_JPRB, 0.20273E+01_JPRB, 0.19608E+01_JPRB, 0.18907E+01_JPRB, 0.19817E+01_JPRB, 0.20549E+01_JPRB, 0.21250E+01_JPRB ,&
&  0.20459E+01_JPRB, 0.18536E+01_JPRB, 0.17686E+01_JPRB, 0.17401E+01_JPRB, 0.17551E+01_JPRB, 0.17407E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.13651E+01_JPRB, 0.12198E+01_JPRB, 0.11784E+01_JPRB, 0.12620E+01_JPRB, 0.15347E+01_JPRB, 0.15793E+01_JPRB, 0.14492E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.10856E+01_JPRB, 0.10670E+01_JPRB, 0.11564E+01_JPRB, 0.14914E+01_JPRB, 0.33050E+01_JPRB, 0.43406E+01_JPRB ,&
&  0.16153E+01_JPRB, 0.12982E+01_JPRB, 0.79567E+00_JPRB, 0.11180E+01_JPRB, 0.77882E+00_JPRB, 0.72822E+00_JPRB, 0.72629E+00_JPRB ,&
&  0.71955E+00_JPRB, 0.71825E+00_JPRB, 0.71856E+00_JPRB, 0.70596E+00_JPRB, 0.70045E+00_JPRB, 0.71344E+00_JPRB, 0.76048E+00_JPRB ,&
&  0.76083E+00_JPRB, 0.76772E+00_JPRB, 0.74574E+00_JPRB, 0.74206E+00_JPRB, 0.72697E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.71968E+00_JPRB, 0.73859E+00_JPRB, 0.76373E+00_JPRB, 0.79512E+00_JPRB, 0.82014E+00_JPRB, 0.85089E+00_JPRB, 0.87836E+00_JPRB ,&
&  0.91214E+00_JPRB, 0.94176E+00_JPRB, 0.96909E+00_JPRB, 0.99894E+00_JPRB, 0.10334E+01_JPRB, 0.10716E+01_JPRB, 0.11188E+01_JPRB ,&
&  0.11697E+01_JPRB, 0.12213E+01_JPRB, 0.12779E+01_JPRB, 0.13419E+01_JPRB, 0.14168E+01_JPRB, 0.15085E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.16850E+01_JPRB, 0.17779E+01_JPRB, 0.18812E+01_JPRB, 0.19644E+01_JPRB, 0.20021E+01_JPRB, 0.21091E+01_JPRB, 0.19578E+01_JPRB ,&
&  0.22222E+01_JPRB, 0.19923E+01_JPRB, 0.18381E+01_JPRB, 0.16671E+01_JPRB, 0.15193E+01_JPRB, 0.13872E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.12244E+01_JPRB, 0.11969E+01_JPRB, 0.11867E+01_JPRB, 0.10748E+01_JPRB, 0.10575E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.12012E+01_JPRB, 0.13505E+01_JPRB, 0.14452E+01_JPRB, 0.15592E+01_JPRB, 0.17051E+01_JPRB, 0.16443E+01_JPRB, 0.15597E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.13820E+01_JPRB, 0.11373E+01_JPRB, 0.12767E+01_JPRB, 0.13393E+01_JPRB, 0.13977E+01_JPRB, 0.14827E+01_JPRB ,&
&  0.13896E+01_JPRB, 0.11845E+01_JPRB, 0.12533E+01_JPRB, 0.15346E+01_JPRB, 0.15191E+01_JPRB, 0.14537E+01_JPRB, 0.13912E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.11888E+01_JPRB, 0.11013E+01_JPRB, 0.10665E+01_JPRB, 0.11352E+01_JPRB, 0.11328E+01_JPRB, 0.10570E+01_JPRB ,&
&  0.97034E+00_JPRB, 0.96460E+00_JPRB, 0.10016E+01_JPRB, 0.10929E+01_JPRB, 0.14304E+01_JPRB, 0.18391E+01_JPRB, 0.29898E+01_JPRB ,&
&  0.16893E+01_JPRB, 0.89463E+00_JPRB, 0.93816E+00_JPRB, 0.89270E+00_JPRB, 0.76689E+00_JPRB, 0.68973E+00_JPRB, 0.70904E+00_JPRB ,&
&  0.73609E+00_JPRB, 0.74706E+00_JPRB, 0.70441E+00_JPRB, 0.70445E+00_JPRB, 0.70445E+00_JPRB, 0.70620E+00_JPRB, 0.71321E+00_JPRB ,&
&  0.71302E+00_JPRB, 0.71332E+00_JPRB, 0.72050E+00_JPRB, 0.71469E+00_JPRB, 0.71262E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.63204E+00_JPRB, 0.64480E+00_JPRB, 0.66380E+00_JPRB, 0.68122E+00_JPRB, 0.69392E+00_JPRB, 0.71276E+00_JPRB, 0.72840E+00_JPRB ,&
&  0.74597E+00_JPRB, 0.75688E+00_JPRB, 0.77220E+00_JPRB, 0.78722E+00_JPRB, 0.80385E+00_JPRB, 0.82609E+00_JPRB, 0.85016E+00_JPRB ,&
&  0.87253E+00_JPRB, 0.90772E+00_JPRB, 0.94420E+00_JPRB, 0.97286E+00_JPRB, 0.10313E+01_JPRB, 0.11378E+01_JPRB, 0.12628E+01_JPRB ,&
&  0.13726E+01_JPRB, 0.15221E+01_JPRB, 0.16633E+01_JPRB, 0.17540E+01_JPRB, 0.17926E+01_JPRB, 0.19504E+01_JPRB, 0.16826E+01_JPRB ,&
&  0.19820E+01_JPRB, 0.18917E+01_JPRB, 0.17176E+01_JPRB, 0.16171E+01_JPRB, 0.15232E+01_JPRB, 0.14381E+01_JPRB, 0.14006E+01_JPRB ,&
&  0.14474E+01_JPRB, 0.17287E+01_JPRB, 0.16538E+01_JPRB, 0.16803E+01_JPRB, 0.14000E+01_JPRB, 0.13344E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.16259E+01_JPRB, 0.16815E+01_JPRB, 0.16867E+01_JPRB, 0.17659E+01_JPRB, 0.18556E+01_JPRB, 0.18955E+01_JPRB, 0.19038E+01_JPRB ,&
&  0.18340E+01_JPRB, 0.17676E+01_JPRB, 0.16563E+01_JPRB, 0.19261E+01_JPRB, 0.17412E+01_JPRB, 0.15530E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.10810E+01_JPRB, 0.11494E+01_JPRB, 0.12522E+01_JPRB, 0.12500E+01_JPRB, 0.12588E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.11658E+01_JPRB, 0.11003E+01_JPRB, 0.10784E+01_JPRB, 0.10837E+01_JPRB, 0.10417E+01_JPRB, 0.95586E+00_JPRB ,&
&  0.91643E+00_JPRB, 0.94847E+00_JPRB, 0.96326E+00_JPRB, 0.10121E+01_JPRB, 0.11914E+01_JPRB, 0.12725E+01_JPRB, 0.19481E+01_JPRB ,&
&  0.13715E+01_JPRB, 0.11797E+01_JPRB, 0.15530E+01_JPRB, 0.57975E+00_JPRB, 0.66320E+00_JPRB, 0.75843E+00_JPRB, 0.95157E+00_JPRB ,&
&  0.98735E+00_JPRB, 0.12566E+01_JPRB, 0.15829E+01_JPRB, 0.12603E+01_JPRB, 0.98011E+00_JPRB, 0.69515E+00_JPRB, 0.66500E+00_JPRB ,&
&  0.65719E+00_JPRB, 0.64694E+00_JPRB, 0.64803E+00_JPRB, 0.63874E+00_JPRB, 0.63400E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.54103E+00_JPRB, 0.54694E+00_JPRB, 0.55982E+00_JPRB, 0.56949E+00_JPRB, 0.57924E+00_JPRB, 0.58824E+00_JPRB, 0.59579E+00_JPRB ,&
&  0.61418E+00_JPRB, 0.62705E+00_JPRB, 0.65818E+00_JPRB, 0.68822E+00_JPRB, 0.71745E+00_JPRB, 0.77016E+00_JPRB, 0.82084E+00_JPRB ,&
&  0.89583E+00_JPRB, 0.99173E+00_JPRB, 0.10466E+01_JPRB, 0.11092E+01_JPRB, 0.11505E+01_JPRB, 0.12016E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.16617E+01_JPRB, 0.18558E+01_JPRB, 0.20619E+01_JPRB, 0.20902E+01_JPRB, 0.22921E+01_JPRB, 0.17087E+01_JPRB ,&
&  0.20093E+01_JPRB, 0.20335E+01_JPRB, 0.19177E+01_JPRB, 0.18599E+01_JPRB, 0.18311E+01_JPRB, 0.18175E+01_JPRB, 0.18143E+01_JPRB ,&
&  0.18971E+01_JPRB, 0.19944E+01_JPRB, 0.19045E+01_JPRB, 0.19102E+01_JPRB, 0.19133E+01_JPRB, 0.18415E+01_JPRB, 0.20414E+01_JPRB ,&
&  0.22123E+01_JPRB, 0.23493E+01_JPRB, 0.25198E+01_JPRB, 0.27299E+01_JPRB, 0.28769E+01_JPRB, 0.29662E+01_JPRB, 0.29976E+01_JPRB ,&
&  0.30058E+01_JPRB, 0.30905E+01_JPRB, 0.33327E+01_JPRB, 0.33155E+01_JPRB, 0.26787E+01_JPRB, 0.22266E+01_JPRB, 0.17538E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.10363E+01_JPRB, 0.10236E+01_JPRB, 0.10718E+01_JPRB, 0.10539E+01_JPRB, 0.10508E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10516E+01_JPRB, 0.10750E+01_JPRB, 0.10761E+01_JPRB, 0.10725E+01_JPRB, 0.10399E+01_JPRB, 0.95725E+00_JPRB, 0.87488E+00_JPRB ,&
&  0.87225E+00_JPRB, 0.88196E+00_JPRB, 0.90989E+00_JPRB, 0.97919E+00_JPRB, 0.11224E+01_JPRB, 0.13937E+01_JPRB, 0.14051E+01_JPRB ,&
&  0.14722E+01_JPRB, 0.94765E+00_JPRB, 0.87224E+00_JPRB, 0.89503E+00_JPRB, 0.70794E+00_JPRB, 0.95700E+00_JPRB, 0.13726E+01_JPRB ,&
&  0.19988E+01_JPRB, 0.22168E+01_JPRB, 0.28724E+01_JPRB, 0.41778E+01_JPRB, 0.26423E+01_JPRB, 0.17200E+01_JPRB, 0.61552E+00_JPRB ,&
&  0.60607E+00_JPRB, 0.58199E+00_JPRB, 0.57426E+00_JPRB, 0.55719E+00_JPRB, 0.54852E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.44241E+00_JPRB, 0.43627E+00_JPRB, 0.44926E+00_JPRB, 0.46907E+00_JPRB, 0.48496E+00_JPRB, 0.52015E+00_JPRB, 0.56327E+00_JPRB ,&
&  0.61141E+00_JPRB, 0.67450E+00_JPRB, 0.73591E+00_JPRB, 0.79013E+00_JPRB, 0.87514E+00_JPRB, 0.98689E+00_JPRB, 0.10600E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.12700E+01_JPRB, 0.13415E+01_JPRB, 0.13983E+01_JPRB, 0.14713E+01_JPRB, 0.15668E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.18256E+01_JPRB, 0.19734E+01_JPRB, 0.21674E+01_JPRB, 0.22962E+01_JPRB, 0.23664E+01_JPRB, 0.23909E+01_JPRB, 0.18289E+01_JPRB ,&
&  0.21594E+01_JPRB, 0.23297E+01_JPRB, 0.23604E+01_JPRB, 0.22891E+01_JPRB, 0.21970E+01_JPRB, 0.21458E+01_JPRB, 0.21753E+01_JPRB ,&
&  0.22227E+01_JPRB, 0.21971E+01_JPRB, 0.20926E+01_JPRB, 0.20447E+01_JPRB, 0.24464E+01_JPRB, 0.24791E+01_JPRB, 0.27115E+01_JPRB ,&
&  0.29195E+01_JPRB, 0.30962E+01_JPRB, 0.33286E+01_JPRB, 0.35715E+01_JPRB, 0.38242E+01_JPRB, 0.40401E+01_JPRB, 0.42317E+01_JPRB ,&
&  0.44126E+01_JPRB, 0.46139E+01_JPRB, 0.48081E+01_JPRB, 0.41585E+01_JPRB, 0.33363E+01_JPRB, 0.27630E+01_JPRB, 0.18002E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.10503E+01_JPRB, 0.10578E+01_JPRB, 0.10335E+01_JPRB, 0.95020E+00_JPRB, 0.91194E+00_JPRB, 0.88880E+00_JPRB ,&
&  0.90930E+00_JPRB, 0.95529E+00_JPRB, 0.98372E+00_JPRB, 0.98387E+00_JPRB, 0.93762E+00_JPRB, 0.86624E+00_JPRB, 0.80823E+00_JPRB ,&
&  0.83855E+00_JPRB, 0.85474E+00_JPRB, 0.87669E+00_JPRB, 0.98711E+00_JPRB, 0.11614E+01_JPRB, 0.15286E+01_JPRB, 0.19413E+01_JPRB ,&
&  0.17370E+01_JPRB, 0.94870E+00_JPRB, 0.81756E+00_JPRB, 0.76883E+00_JPRB, 0.73653E+00_JPRB, 0.80296E+00_JPRB, 0.96419E+00_JPRB ,&
&  0.13446E+01_JPRB, 0.21611E+01_JPRB, 0.23732E+01_JPRB, 0.15621E+01_JPRB, 0.20574E+01_JPRB, 0.13677E+01_JPRB, 0.59309E+00_JPRB ,&
&  0.56761E+00_JPRB, 0.54651E+00_JPRB, 0.51793E+00_JPRB, 0.48497E+00_JPRB, 0.45991E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.44253E+00_JPRB, 0.42347E+00_JPRB, 0.43612E+00_JPRB, 0.46260E+00_JPRB, 0.49361E+00_JPRB, 0.54277E+00_JPRB, 0.60609E+00_JPRB ,&
&  0.66766E+00_JPRB, 0.73420E+00_JPRB, 0.84475E+00_JPRB, 0.88892E+00_JPRB, 0.93987E+00_JPRB, 0.10236E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.13009E+01_JPRB, 0.14017E+01_JPRB, 0.15010E+01_JPRB, 0.16189E+01_JPRB, 0.17431E+01_JPRB, 0.18519E+01_JPRB ,&
&  0.19969E+01_JPRB, 0.21440E+01_JPRB, 0.23377E+01_JPRB, 0.25415E+01_JPRB, 0.27291E+01_JPRB, 0.27659E+01_JPRB, 0.28754E+01_JPRB ,&
&  0.18354E+01_JPRB, 0.25406E+01_JPRB, 0.25715E+01_JPRB, 0.25177E+01_JPRB, 0.23922E+01_JPRB, 0.22883E+01_JPRB, 0.23059E+01_JPRB ,&
&  0.23372E+01_JPRB, 0.22266E+01_JPRB, 0.22823E+01_JPRB, 0.22960E+01_JPRB, 0.27499E+01_JPRB, 0.28735E+01_JPRB, 0.30519E+01_JPRB ,&
&  0.32432E+01_JPRB, 0.34190E+01_JPRB, 0.36178E+01_JPRB, 0.38570E+01_JPRB, 0.41014E+01_JPRB, 0.43314E+01_JPRB, 0.45619E+01_JPRB ,&
&  0.47663E+01_JPRB, 0.49594E+01_JPRB, 0.51755E+01_JPRB, 0.43210E+01_JPRB, 0.36091E+01_JPRB, 0.29380E+01_JPRB, 0.19774E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.10493E+01_JPRB, 0.10609E+01_JPRB, 0.10763E+01_JPRB, 0.95985E+00_JPRB, 0.89263E+00_JPRB, 0.80943E+00_JPRB ,&
&  0.80770E+00_JPRB, 0.82982E+00_JPRB, 0.84120E+00_JPRB, 0.83913E+00_JPRB, 0.84165E+00_JPRB, 0.78394E+00_JPRB, 0.75716E+00_JPRB ,&
&  0.78823E+00_JPRB, 0.79475E+00_JPRB, 0.82264E+00_JPRB, 0.88180E+00_JPRB, 0.97573E+00_JPRB, 0.10420E+01_JPRB, 0.10375E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.89753E+00_JPRB, 0.83838E+00_JPRB, 0.84250E+00_JPRB, 0.79998E+00_JPRB, 0.75970E+00_JPRB, 0.77218E+00_JPRB ,&
&  0.85504E+00_JPRB, 0.96719E+00_JPRB, 0.10570E+01_JPRB, 0.10300E+01_JPRB, 0.10451E+01_JPRB, 0.83897E+00_JPRB, 0.64553E+00_JPRB ,&
&  0.60600E+00_JPRB, 0.59609E+00_JPRB, 0.54873E+00_JPRB, 0.51423E+00_JPRB, 0.47584E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.54749E+00_JPRB, 0.48217E+00_JPRB, 0.47840E+00_JPRB, 0.52375E+00_JPRB, 0.56968E+00_JPRB, 0.61896E+00_JPRB, 0.66849E+00_JPRB ,&
&  0.72570E+00_JPRB, 0.76372E+00_JPRB, 0.83113E+00_JPRB, 0.87604E+00_JPRB, 0.91731E+00_JPRB, 0.95602E+00_JPRB, 0.10089E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.11968E+01_JPRB, 0.12993E+01_JPRB, 0.14310E+01_JPRB, 0.15732E+01_JPRB, 0.17070E+01_JPRB, 0.18551E+01_JPRB ,&
&  0.20037E+01_JPRB, 0.21699E+01_JPRB, 0.23426E+01_JPRB, 0.25403E+01_JPRB, 0.27229E+01_JPRB, 0.27944E+01_JPRB, 0.27526E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.21572E+01_JPRB, 0.27101E+01_JPRB, 0.25990E+01_JPRB, 0.24790E+01_JPRB, 0.23237E+01_JPRB, 0.22815E+01_JPRB ,&
&  0.21343E+01_JPRB, 0.19320E+01_JPRB, 0.20207E+01_JPRB, 0.25977E+01_JPRB, 0.28364E+01_JPRB, 0.30044E+01_JPRB, 0.31116E+01_JPRB ,&
&  0.32452E+01_JPRB, 0.33463E+01_JPRB, 0.34646E+01_JPRB, 0.35920E+01_JPRB, 0.37385E+01_JPRB, 0.38791E+01_JPRB, 0.40035E+01_JPRB ,&
&  0.41055E+01_JPRB, 0.41606E+01_JPRB, 0.41392E+01_JPRB, 0.32838E+01_JPRB, 0.32204E+01_JPRB, 0.27502E+01_JPRB, 0.23590E+01_JPRB ,&
&  0.15244E+01_JPRB, 0.11196E+01_JPRB, 0.10914E+01_JPRB, 0.11066E+01_JPRB, 0.98331E+00_JPRB, 0.90018E+00_JPRB, 0.79247E+00_JPRB ,&
&  0.78335E+00_JPRB, 0.77940E+00_JPRB, 0.78730E+00_JPRB, 0.78092E+00_JPRB, 0.78154E+00_JPRB, 0.72726E+00_JPRB, 0.74613E+00_JPRB ,&
&  0.77421E+00_JPRB, 0.78408E+00_JPRB, 0.81142E+00_JPRB, 0.85747E+00_JPRB, 0.91630E+00_JPRB, 0.96709E+00_JPRB, 0.98044E+00_JPRB ,&
&  0.97586E+00_JPRB, 0.94604E+00_JPRB, 0.93174E+00_JPRB, 0.91970E+00_JPRB, 0.90216E+00_JPRB, 0.88288E+00_JPRB, 0.85777E+00_JPRB ,&
&  0.82765E+00_JPRB, 0.83025E+00_JPRB, 0.85639E+00_JPRB, 0.78592E+00_JPRB, 0.82431E+00_JPRB, 0.75145E+00_JPRB, 0.73692E+00_JPRB ,&
&  0.71831E+00_JPRB, 0.70183E+00_JPRB, 0.65534E+00_JPRB, 0.63093E+00_JPRB, 0.59418E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.67004E+00_JPRB, 0.58714E+00_JPRB, 0.52486E+00_JPRB, 0.54056E+00_JPRB, 0.60973E+00_JPRB, 0.66874E+00_JPRB, 0.70380E+00_JPRB ,&
&  0.74678E+00_JPRB, 0.76514E+00_JPRB, 0.77957E+00_JPRB, 0.79794E+00_JPRB, 0.83677E+00_JPRB, 0.86850E+00_JPRB, 0.93061E+00_JPRB ,&
&  0.98855E+00_JPRB, 0.10460E+01_JPRB, 0.11091E+01_JPRB, 0.12129E+01_JPRB, 0.13470E+01_JPRB, 0.14816E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.17668E+01_JPRB, 0.19595E+01_JPRB, 0.22186E+01_JPRB, 0.25256E+01_JPRB, 0.29521E+01_JPRB, 0.34220E+01_JPRB, 0.37785E+01_JPRB ,&
&  0.26194E+01_JPRB, 0.11350E+01_JPRB, 0.23510E+01_JPRB, 0.27357E+01_JPRB, 0.25039E+01_JPRB, 0.23696E+01_JPRB, 0.21868E+01_JPRB ,&
&  0.20213E+01_JPRB, 0.18568E+01_JPRB, 0.18142E+01_JPRB, 0.22206E+01_JPRB, 0.27219E+01_JPRB, 0.29078E+01_JPRB, 0.29787E+01_JPRB ,&
&  0.30442E+01_JPRB, 0.30721E+01_JPRB, 0.31182E+01_JPRB, 0.31874E+01_JPRB, 0.32730E+01_JPRB, 0.33296E+01_JPRB, 0.33650E+01_JPRB ,&
&  0.33866E+01_JPRB, 0.33630E+01_JPRB, 0.32378E+01_JPRB, 0.23757E+01_JPRB, 0.21630E+01_JPRB, 0.19130E+01_JPRB, 0.17504E+01_JPRB ,&
&  0.15587E+01_JPRB, 0.12501E+01_JPRB, 0.11528E+01_JPRB, 0.11308E+01_JPRB, 0.10252E+01_JPRB, 0.88997E+00_JPRB, 0.82495E+00_JPRB ,&
&  0.80979E+00_JPRB, 0.78626E+00_JPRB, 0.76132E+00_JPRB, 0.74000E+00_JPRB, 0.77336E+00_JPRB, 0.77403E+00_JPRB, 0.77134E+00_JPRB ,&
&  0.77175E+00_JPRB, 0.81952E+00_JPRB, 0.86817E+00_JPRB, 0.91045E+00_JPRB, 0.95467E+00_JPRB, 0.10051E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.10248E+01_JPRB, 0.10152E+01_JPRB, 0.98903E+00_JPRB, 0.96577E+00_JPRB, 0.92662E+00_JPRB, 0.91722E+00_JPRB ,&
&  0.91204E+00_JPRB, 0.91217E+00_JPRB, 0.89479E+00_JPRB, 0.87229E+00_JPRB, 0.88791E+00_JPRB, 0.87308E+00_JPRB, 0.87513E+00_JPRB ,&
&  0.86713E+00_JPRB, 0.84459E+00_JPRB, 0.78869E+00_JPRB, 0.74149E+00_JPRB, 0.71856E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.74902E+00_JPRB, 0.68958E+00_JPRB, 0.61675E+00_JPRB, 0.56607E+00_JPRB, 0.58496E+00_JPRB, 0.65605E+00_JPRB, 0.71513E+00_JPRB ,&
&  0.73938E+00_JPRB, 0.74329E+00_JPRB, 0.73682E+00_JPRB, 0.73948E+00_JPRB, 0.75979E+00_JPRB, 0.76642E+00_JPRB, 0.80037E+00_JPRB ,&
&  0.85441E+00_JPRB, 0.90034E+00_JPRB, 0.93719E+00_JPRB, 0.99866E+00_JPRB, 0.10712E+01_JPRB, 0.11911E+01_JPRB, 0.13040E+01_JPRB ,&
&  0.13996E+01_JPRB, 0.15047E+01_JPRB, 0.16725E+01_JPRB, 0.19110E+01_JPRB, 0.22481E+01_JPRB, 0.28363E+01_JPRB, 0.38563E+01_JPRB ,&
&  0.57007E+01_JPRB, 0.37929E+01_JPRB, 0.15490E+01_JPRB, 0.26561E+01_JPRB, 0.27571E+01_JPRB, 0.25845E+01_JPRB, 0.21990E+01_JPRB ,&
&  0.21598E+01_JPRB, 0.18760E+01_JPRB, 0.16652E+01_JPRB, 0.18048E+01_JPRB, 0.20751E+01_JPRB, 0.24357E+01_JPRB, 0.26546E+01_JPRB ,&
&  0.27095E+01_JPRB, 0.27366E+01_JPRB, 0.27603E+01_JPRB, 0.28135E+01_JPRB, 0.28943E+01_JPRB, 0.29710E+01_JPRB, 0.29881E+01_JPRB ,&
&  0.30065E+01_JPRB, 0.30459E+01_JPRB, 0.29651E+01_JPRB, 0.22414E+01_JPRB, 0.20943E+01_JPRB, 0.20207E+01_JPRB, 0.19206E+01_JPRB ,&
&  0.16981E+01_JPRB, 0.15470E+01_JPRB, 0.14151E+01_JPRB, 0.12249E+01_JPRB, 0.10747E+01_JPRB, 0.98685E+00_JPRB, 0.96188E+00_JPRB ,&
&  0.92166E+00_JPRB, 0.85810E+00_JPRB, 0.82670E+00_JPRB, 0.79941E+00_JPRB, 0.81762E+00_JPRB, 0.85248E+00_JPRB, 0.86362E+00_JPRB ,&
&  0.89320E+00_JPRB, 0.93403E+00_JPRB, 0.96194E+00_JPRB, 0.98730E+00_JPRB, 0.10240E+01_JPRB, 0.10578E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.11015E+01_JPRB, 0.10906E+01_JPRB, 0.10703E+01_JPRB, 0.10233E+01_JPRB, 0.98014E+00_JPRB, 0.98867E+00_JPRB ,&
&  0.97680E+00_JPRB, 0.97435E+00_JPRB, 0.95047E+00_JPRB, 0.95500E+00_JPRB, 0.94564E+00_JPRB, 0.95990E+00_JPRB, 0.98512E+00_JPRB ,&
&  0.10067E+01_JPRB, 0.98079E+00_JPRB, 0.89241E+00_JPRB, 0.79930E+00_JPRB, 0.77316E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.79029E+00_JPRB, 0.76933E+00_JPRB, 0.72376E+00_JPRB, 0.63668E+00_JPRB, 0.59937E+00_JPRB, 0.61262E+00_JPRB, 0.68049E+00_JPRB ,&
&  0.71224E+00_JPRB, 0.72401E+00_JPRB, 0.71055E+00_JPRB, 0.69709E+00_JPRB, 0.69313E+00_JPRB, 0.70240E+00_JPRB, 0.73264E+00_JPRB ,&
&  0.77264E+00_JPRB, 0.77841E+00_JPRB, 0.77254E+00_JPRB, 0.78202E+00_JPRB, 0.82931E+00_JPRB, 0.89854E+00_JPRB, 0.10145E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.11693E+01_JPRB, 0.12360E+01_JPRB, 0.13890E+01_JPRB, 0.15746E+01_JPRB, 0.16769E+01_JPRB, 0.19690E+01_JPRB ,&
&  0.31949E+01_JPRB, 0.49778E+01_JPRB, 0.18541E+01_JPRB, 0.27894E+01_JPRB, 0.31999E+01_JPRB, 0.28545E+01_JPRB, 0.25822E+01_JPRB ,&
&  0.24874E+01_JPRB, 0.20350E+01_JPRB, 0.15843E+01_JPRB, 0.15217E+01_JPRB, 0.16374E+01_JPRB, 0.17676E+01_JPRB, 0.19741E+01_JPRB ,&
&  0.21677E+01_JPRB, 0.22932E+01_JPRB, 0.23964E+01_JPRB, 0.24810E+01_JPRB, 0.25880E+01_JPRB, 0.27322E+01_JPRB, 0.28594E+01_JPRB ,&
&  0.29502E+01_JPRB, 0.30818E+01_JPRB, 0.31555E+01_JPRB, 0.25360E+01_JPRB, 0.24287E+01_JPRB, 0.25834E+01_JPRB, 0.24475E+01_JPRB ,&
&  0.21395E+01_JPRB, 0.19337E+01_JPRB, 0.16793E+01_JPRB, 0.13940E+01_JPRB, 0.11902E+01_JPRB, 0.11081E+01_JPRB, 0.10733E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.94642E+00_JPRB, 0.93177E+00_JPRB, 0.92708E+00_JPRB, 0.92840E+00_JPRB, 0.93985E+00_JPRB, 0.96885E+00_JPRB ,&
&  0.10169E+01_JPRB, 0.10586E+01_JPRB, 0.10854E+01_JPRB, 0.10982E+01_JPRB, 0.11107E+01_JPRB, 0.11266E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11316E+01_JPRB, 0.11309E+01_JPRB, 0.11256E+01_JPRB, 0.11080E+01_JPRB, 0.10806E+01_JPRB, 0.10638E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10323E+01_JPRB, 0.97461E+00_JPRB, 0.95307E+00_JPRB, 0.97860E+00_JPRB, 0.95009E+00_JPRB, 0.99331E+00_JPRB ,&
&  0.10595E+01_JPRB, 0.10927E+01_JPRB, 0.10288E+01_JPRB, 0.82825E+00_JPRB, 0.80060E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.82745E+00_JPRB, 0.81546E+00_JPRB, 0.79776E+00_JPRB, 0.73588E+00_JPRB, 0.67331E+00_JPRB, 0.64149E+00_JPRB, 0.65563E+00_JPRB ,&
&  0.69490E+00_JPRB, 0.72264E+00_JPRB, 0.72448E+00_JPRB, 0.71496E+00_JPRB, 0.71474E+00_JPRB, 0.72791E+00_JPRB, 0.74769E+00_JPRB ,&
&  0.76216E+00_JPRB, 0.75894E+00_JPRB, 0.74219E+00_JPRB, 0.74388E+00_JPRB, 0.74973E+00_JPRB, 0.76826E+00_JPRB, 0.82578E+00_JPRB ,&
&  0.90684E+00_JPRB, 0.95512E+00_JPRB, 0.97201E+00_JPRB, 0.10249E+01_JPRB, 0.11668E+01_JPRB, 0.12239E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.17781E+01_JPRB, 0.37871E+01_JPRB, 0.40191E+01_JPRB, 0.45646E+01_JPRB, 0.40218E+01_JPRB, 0.33221E+01_JPRB, 0.31599E+01_JPRB ,&
&  0.31756E+01_JPRB, 0.25794E+01_JPRB, 0.20233E+01_JPRB, 0.15757E+01_JPRB, 0.15113E+01_JPRB, 0.15513E+01_JPRB, 0.15621E+01_JPRB ,&
&  0.16438E+01_JPRB, 0.18442E+01_JPRB, 0.19706E+01_JPRB, 0.20898E+01_JPRB, 0.22041E+01_JPRB, 0.23336E+01_JPRB, 0.25220E+01_JPRB ,&
&  0.27414E+01_JPRB, 0.30195E+01_JPRB, 0.33544E+01_JPRB, 0.31263E+01_JPRB, 0.27611E+01_JPRB, 0.31462E+01_JPRB, 0.32568E+01_JPRB ,&
&  0.31164E+01_JPRB, 0.24953E+01_JPRB, 0.20023E+01_JPRB, 0.16280E+01_JPRB, 0.13571E+01_JPRB, 0.13687E+01_JPRB, 0.12187E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.10543E+01_JPRB, 0.10327E+01_JPRB, 0.10383E+01_JPRB, 0.10449E+01_JPRB, 0.10351E+01_JPRB, 0.10510E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.11201E+01_JPRB, 0.11392E+01_JPRB, 0.11530E+01_JPRB, 0.11698E+01_JPRB, 0.11811E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.11327E+01_JPRB, 0.10826E+01_JPRB, 0.10837E+01_JPRB, 0.10850E+01_JPRB, 0.10868E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.10566E+01_JPRB, 0.10200E+01_JPRB, 0.99465E+00_JPRB, 0.10004E+01_JPRB, 0.98205E+00_JPRB, 0.94159E+00_JPRB ,&
&  0.94625E+00_JPRB, 0.92743E+00_JPRB, 0.89357E+00_JPRB, 0.86351E+00_JPRB, 0.84216E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.87958E+00_JPRB, 0.86729E+00_JPRB, 0.85977E+00_JPRB, 0.82794E+00_JPRB, 0.78682E+00_JPRB, 0.74912E+00_JPRB, 0.72790E+00_JPRB ,&
&  0.72854E+00_JPRB, 0.74263E+00_JPRB, 0.75501E+00_JPRB, 0.74505E+00_JPRB, 0.74215E+00_JPRB, 0.75884E+00_JPRB, 0.77100E+00_JPRB ,&
&  0.78122E+00_JPRB, 0.78169E+00_JPRB, 0.77543E+00_JPRB, 0.77417E+00_JPRB, 0.76979E+00_JPRB, 0.75630E+00_JPRB, 0.75790E+00_JPRB ,&
&  0.79111E+00_JPRB, 0.82483E+00_JPRB, 0.84129E+00_JPRB, 0.85374E+00_JPRB, 0.93217E+00_JPRB, 0.99675E+00_JPRB, 0.10814E+01_JPRB ,&
&  0.13171E+01_JPRB, 0.32257E+01_JPRB, 0.27549E+01_JPRB, 0.52889E+01_JPRB, 0.44560E+01_JPRB, 0.37037E+01_JPRB, 0.31415E+01_JPRB ,&
&  0.30201E+01_JPRB, 0.31094E+01_JPRB, 0.24172E+01_JPRB, 0.19321E+01_JPRB, 0.16486E+01_JPRB, 0.15550E+01_JPRB, 0.14810E+01_JPRB ,&
&  0.14198E+01_JPRB, 0.14496E+01_JPRB, 0.15196E+01_JPRB, 0.16266E+01_JPRB, 0.17628E+01_JPRB, 0.18830E+01_JPRB, 0.20311E+01_JPRB ,&
&  0.22041E+01_JPRB, 0.24871E+01_JPRB, 0.29317E+01_JPRB, 0.29923E+01_JPRB, 0.30277E+01_JPRB, 0.34950E+01_JPRB, 0.39831E+01_JPRB ,&
&  0.39457E+01_JPRB, 0.32715E+01_JPRB, 0.24243E+01_JPRB, 0.18732E+01_JPRB, 0.15990E+01_JPRB, 0.15208E+01_JPRB, 0.13605E+01_JPRB ,&
&  0.12257E+01_JPRB, 0.11597E+01_JPRB, 0.11347E+01_JPRB, 0.11219E+01_JPRB, 0.11245E+01_JPRB, 0.11231E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.11676E+01_JPRB, 0.11723E+01_JPRB, 0.11692E+01_JPRB, 0.11722E+01_JPRB, 0.11744E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.11504E+01_JPRB, 0.10882E+01_JPRB, 0.10621E+01_JPRB, 0.10689E+01_JPRB, 0.10687E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.10971E+01_JPRB, 0.10611E+01_JPRB, 0.10101E+01_JPRB, 0.10153E+01_JPRB, 0.10266E+01_JPRB, 0.94881E+00_JPRB ,&
&  0.93178E+00_JPRB, 0.91940E+00_JPRB, 0.91364E+00_JPRB, 0.90655E+00_JPRB, 0.89308E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.89400E+00_JPRB, 0.88082E+00_JPRB, 0.87658E+00_JPRB, 0.86112E+00_JPRB, 0.83242E+00_JPRB, 0.80223E+00_JPRB, 0.78393E+00_JPRB ,&
&  0.77030E+00_JPRB, 0.77066E+00_JPRB, 0.78044E+00_JPRB, 0.77218E+00_JPRB, 0.77539E+00_JPRB, 0.78809E+00_JPRB, 0.79597E+00_JPRB ,&
&  0.79383E+00_JPRB, 0.79200E+00_JPRB, 0.78367E+00_JPRB, 0.77888E+00_JPRB, 0.77604E+00_JPRB, 0.76308E+00_JPRB, 0.75505E+00_JPRB ,&
&  0.75840E+00_JPRB, 0.77381E+00_JPRB, 0.78796E+00_JPRB, 0.80118E+00_JPRB, 0.84190E+00_JPRB, 0.88696E+00_JPRB, 0.95239E+00_JPRB ,&
&  0.11925E+01_JPRB, 0.36107E+01_JPRB, 0.25795E+01_JPRB, 0.49402E+01_JPRB, 0.42968E+01_JPRB, 0.34960E+01_JPRB, 0.28771E+01_JPRB ,&
&  0.28029E+01_JPRB, 0.27668E+01_JPRB, 0.24407E+01_JPRB, 0.21871E+01_JPRB, 0.19511E+01_JPRB, 0.17174E+01_JPRB, 0.15447E+01_JPRB ,&
&  0.14051E+01_JPRB, 0.13348E+01_JPRB, 0.13262E+01_JPRB, 0.13534E+01_JPRB, 0.14284E+01_JPRB, 0.15218E+01_JPRB, 0.16138E+01_JPRB ,&
&  0.17188E+01_JPRB, 0.18831E+01_JPRB, 0.21946E+01_JPRB, 0.27043E+01_JPRB, 0.28123E+01_JPRB, 0.31989E+01_JPRB, 0.46010E+01_JPRB ,&
&  0.47571E+01_JPRB, 0.41935E+01_JPRB, 0.29851E+01_JPRB, 0.21317E+01_JPRB, 0.17887E+01_JPRB, 0.16460E+01_JPRB, 0.15085E+01_JPRB ,&
&  0.13741E+01_JPRB, 0.12705E+01_JPRB, 0.12392E+01_JPRB, 0.11907E+01_JPRB, 0.11599E+01_JPRB, 0.11743E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.12016E+01_JPRB, 0.12029E+01_JPRB, 0.11803E+01_JPRB, 0.11582E+01_JPRB, 0.11352E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.11322E+01_JPRB, 0.11463E+01_JPRB, 0.11130E+01_JPRB, 0.10746E+01_JPRB, 0.10707E+01_JPRB, 0.10773E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11235E+01_JPRB, 0.10888E+01_JPRB, 0.10617E+01_JPRB, 0.10342E+01_JPRB, 0.10493E+01_JPRB, 0.94619E+00_JPRB ,&
&  0.91806E+00_JPRB, 0.90772E+00_JPRB, 0.90914E+00_JPRB, 0.91640E+00_JPRB, 0.91325E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.91015E+00_JPRB, 0.89142E+00_JPRB, 0.88617E+00_JPRB, 0.87958E+00_JPRB, 0.86436E+00_JPRB, 0.84595E+00_JPRB, 0.83951E+00_JPRB ,&
&  0.83283E+00_JPRB, 0.82756E+00_JPRB, 0.81670E+00_JPRB, 0.79929E+00_JPRB, 0.79803E+00_JPRB, 0.79855E+00_JPRB, 0.79765E+00_JPRB ,&
&  0.79479E+00_JPRB, 0.79328E+00_JPRB, 0.78103E+00_JPRB, 0.77484E+00_JPRB, 0.77631E+00_JPRB, 0.77151E+00_JPRB, 0.75968E+00_JPRB ,&
&  0.74858E+00_JPRB, 0.74806E+00_JPRB, 0.76121E+00_JPRB, 0.77642E+00_JPRB, 0.80766E+00_JPRB, 0.82851E+00_JPRB, 0.88280E+00_JPRB ,&
&  0.11813E+01_JPRB, 0.12115E+01_JPRB, 0.31379E+01_JPRB, 0.42519E+01_JPRB, 0.41109E+01_JPRB, 0.34717E+01_JPRB, 0.30325E+01_JPRB ,&
&  0.28141E+01_JPRB, 0.25846E+01_JPRB, 0.24070E+01_JPRB, 0.21789E+01_JPRB, 0.19313E+01_JPRB, 0.16614E+01_JPRB, 0.15079E+01_JPRB ,&
&  0.14138E+01_JPRB, 0.13305E+01_JPRB, 0.12991E+01_JPRB, 0.13005E+01_JPRB, 0.13188E+01_JPRB, 0.13393E+01_JPRB, 0.13818E+01_JPRB ,&
&  0.14293E+01_JPRB, 0.14906E+01_JPRB, 0.16203E+01_JPRB, 0.20032E+01_JPRB, 0.25794E+01_JPRB, 0.30624E+01_JPRB, 0.41078E+01_JPRB ,&
&  0.51042E+01_JPRB, 0.48037E+01_JPRB, 0.34874E+01_JPRB, 0.24426E+01_JPRB, 0.19662E+01_JPRB, 0.17713E+01_JPRB, 0.16226E+01_JPRB ,&
&  0.15030E+01_JPRB, 0.13812E+01_JPRB, 0.13011E+01_JPRB, 0.12396E+01_JPRB, 0.11927E+01_JPRB, 0.11718E+01_JPRB, 0.12113E+01_JPRB ,&
&  0.12478E+01_JPRB, 0.12506E+01_JPRB, 0.12179E+01_JPRB, 0.11850E+01_JPRB, 0.11573E+01_JPRB, 0.11299E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.11397E+01_JPRB, 0.11262E+01_JPRB, 0.10975E+01_JPRB, 0.10931E+01_JPRB, 0.11108E+01_JPRB, 0.11265E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.11273E+01_JPRB, 0.10887E+01_JPRB, 0.10770E+01_JPRB, 0.10240E+01_JPRB, 0.10380E+01_JPRB, 0.95056E+00_JPRB ,&
&  0.91302E+00_JPRB, 0.90459E+00_JPRB, 0.91364E+00_JPRB, 0.92168E+00_JPRB, 0.93535E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.92480E+00_JPRB, 0.89577E+00_JPRB, 0.88574E+00_JPRB, 0.88817E+00_JPRB, 0.88382E+00_JPRB, 0.88667E+00_JPRB, 0.88747E+00_JPRB ,&
&  0.87776E+00_JPRB, 0.85484E+00_JPRB, 0.83616E+00_JPRB, 0.81684E+00_JPRB, 0.81014E+00_JPRB, 0.80317E+00_JPRB, 0.80239E+00_JPRB ,&
&  0.79459E+00_JPRB, 0.78893E+00_JPRB, 0.77877E+00_JPRB, 0.77581E+00_JPRB, 0.77542E+00_JPRB, 0.76879E+00_JPRB, 0.74547E+00_JPRB ,&
&  0.73701E+00_JPRB, 0.73834E+00_JPRB, 0.75357E+00_JPRB, 0.77219E+00_JPRB, 0.79507E+00_JPRB, 0.80642E+00_JPRB, 0.83816E+00_JPRB ,&
&  0.11195E+01_JPRB, 0.20524E+01_JPRB, 0.32715E+01_JPRB, 0.34447E+01_JPRB, 0.34136E+01_JPRB, 0.30744E+01_JPRB, 0.27795E+01_JPRB ,&
&  0.25009E+01_JPRB, 0.23796E+01_JPRB, 0.23594E+01_JPRB, 0.21241E+01_JPRB, 0.17680E+01_JPRB, 0.14861E+01_JPRB, 0.13721E+01_JPRB ,&
&  0.13075E+01_JPRB, 0.12560E+01_JPRB, 0.12306E+01_JPRB, 0.12294E+01_JPRB, 0.12316E+01_JPRB, 0.12363E+01_JPRB, 0.12521E+01_JPRB ,&
&  0.12678E+01_JPRB, 0.12910E+01_JPRB, 0.13337E+01_JPRB, 0.15197E+01_JPRB, 0.22153E+01_JPRB, 0.24471E+01_JPRB, 0.36114E+01_JPRB ,&
&  0.51840E+01_JPRB, 0.46655E+01_JPRB, 0.36555E+01_JPRB, 0.27471E+01_JPRB, 0.22240E+01_JPRB, 0.19609E+01_JPRB, 0.17607E+01_JPRB ,&
&  0.16060E+01_JPRB, 0.14766E+01_JPRB, 0.13666E+01_JPRB, 0.12989E+01_JPRB, 0.12381E+01_JPRB, 0.11924E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.12328E+01_JPRB, 0.12132E+01_JPRB, 0.11787E+01_JPRB, 0.11502E+01_JPRB, 0.11418E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11390E+01_JPRB, 0.11369E+01_JPRB, 0.10915E+01_JPRB, 0.10891E+01_JPRB, 0.11161E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.10986E+01_JPRB, 0.10799E+01_JPRB, 0.10906E+01_JPRB, 0.10671E+01_JPRB, 0.10772E+01_JPRB, 0.10734E+01_JPRB, 0.97093E+00_JPRB ,&
&  0.91331E+00_JPRB, 0.89788E+00_JPRB, 0.91660E+00_JPRB, 0.94054E+00_JPRB, 0.97246E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.95312E+00_JPRB, 0.90637E+00_JPRB, 0.88122E+00_JPRB, 0.88044E+00_JPRB, 0.88763E+00_JPRB, 0.88559E+00_JPRB, 0.88894E+00_JPRB ,&
&  0.87898E+00_JPRB, 0.85884E+00_JPRB, 0.83670E+00_JPRB, 0.81801E+00_JPRB, 0.80838E+00_JPRB, 0.80188E+00_JPRB, 0.79694E+00_JPRB ,&
&  0.79225E+00_JPRB, 0.78672E+00_JPRB, 0.77804E+00_JPRB, 0.78033E+00_JPRB, 0.78220E+00_JPRB, 0.77937E+00_JPRB, 0.75413E+00_JPRB ,&
&  0.73958E+00_JPRB, 0.74185E+00_JPRB, 0.75352E+00_JPRB, 0.76381E+00_JPRB, 0.77881E+00_JPRB, 0.79345E+00_JPRB, 0.81994E+00_JPRB ,&
&  0.97620E+00_JPRB, 0.18006E+01_JPRB, 0.24912E+01_JPRB, 0.29347E+01_JPRB, 0.30658E+01_JPRB, 0.29113E+01_JPRB, 0.26119E+01_JPRB ,&
&  0.23834E+01_JPRB, 0.22543E+01_JPRB, 0.21351E+01_JPRB, 0.18442E+01_JPRB, 0.15348E+01_JPRB, 0.13631E+01_JPRB, 0.12731E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.12132E+01_JPRB, 0.11949E+01_JPRB, 0.11807E+01_JPRB, 0.11678E+01_JPRB, 0.11685E+01_JPRB, 0.11772E+01_JPRB ,&
&  0.11870E+01_JPRB, 0.11927E+01_JPRB, 0.12087E+01_JPRB, 0.12315E+01_JPRB, 0.15699E+01_JPRB, 0.20554E+01_JPRB, 0.28284E+01_JPRB ,&
&  0.31965E+01_JPRB, 0.32654E+01_JPRB, 0.30027E+01_JPRB, 0.26499E+01_JPRB, 0.23333E+01_JPRB, 0.20743E+01_JPRB, 0.18518E+01_JPRB ,&
&  0.16607E+01_JPRB, 0.14927E+01_JPRB, 0.13549E+01_JPRB, 0.12668E+01_JPRB, 0.12132E+01_JPRB, 0.11661E+01_JPRB, 0.11432E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.11624E+01_JPRB, 0.11629E+01_JPRB, 0.11539E+01_JPRB, 0.11326E+01_JPRB, 0.11129E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.10996E+01_JPRB, 0.11118E+01_JPRB, 0.10945E+01_JPRB, 0.10752E+01_JPRB, 0.10640E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10696E+01_JPRB, 0.10539E+01_JPRB, 0.10597E+01_JPRB, 0.10618E+01_JPRB, 0.10673E+01_JPRB, 0.97955E+00_JPRB ,&
&  0.92389E+00_JPRB, 0.90305E+00_JPRB, 0.90292E+00_JPRB, 0.92091E+00_JPRB, 0.96217E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.97165E+00_JPRB, 0.93678E+00_JPRB, 0.90292E+00_JPRB, 0.88368E+00_JPRB, 0.87674E+00_JPRB, 0.87416E+00_JPRB, 0.87249E+00_JPRB ,&
&  0.86776E+00_JPRB, 0.85870E+00_JPRB, 0.83619E+00_JPRB, 0.81841E+00_JPRB, 0.80563E+00_JPRB, 0.80062E+00_JPRB, 0.79630E+00_JPRB ,&
&  0.79374E+00_JPRB, 0.79012E+00_JPRB, 0.78292E+00_JPRB, 0.78086E+00_JPRB, 0.78209E+00_JPRB, 0.77593E+00_JPRB, 0.75594E+00_JPRB ,&
&  0.73455E+00_JPRB, 0.73472E+00_JPRB, 0.74460E+00_JPRB, 0.75455E+00_JPRB, 0.76910E+00_JPRB, 0.78191E+00_JPRB, 0.80846E+00_JPRB ,&
&  0.94475E+00_JPRB, 0.14022E+01_JPRB, 0.20788E+01_JPRB, 0.24512E+01_JPRB, 0.26069E+01_JPRB, 0.26220E+01_JPRB, 0.24388E+01_JPRB ,&
&  0.22042E+01_JPRB, 0.19644E+01_JPRB, 0.17701E+01_JPRB, 0.15650E+01_JPRB, 0.14175E+01_JPRB, 0.13047E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11707E+01_JPRB, 0.11551E+01_JPRB, 0.11357E+01_JPRB, 0.11207E+01_JPRB, 0.11115E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.11254E+01_JPRB, 0.11413E+01_JPRB, 0.11542E+01_JPRB, 0.11958E+01_JPRB, 0.13775E+01_JPRB, 0.17887E+01_JPRB ,&
&  0.21633E+01_JPRB, 0.23777E+01_JPRB, 0.23403E+01_JPRB, 0.22590E+01_JPRB, 0.21796E+01_JPRB, 0.20147E+01_JPRB, 0.18428E+01_JPRB ,&
&  0.16731E+01_JPRB, 0.15166E+01_JPRB, 0.13568E+01_JPRB, 0.12280E+01_JPRB, 0.11449E+01_JPRB, 0.10966E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.10844E+01_JPRB, 0.10893E+01_JPRB, 0.10952E+01_JPRB, 0.10866E+01_JPRB, 0.10682E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10100E+01_JPRB, 0.10092E+01_JPRB, 0.10026E+01_JPRB, 0.99249E+00_JPRB, 0.99064E+00_JPRB, 0.99177E+00_JPRB ,&
&  0.99556E+00_JPRB, 0.10143E+01_JPRB, 0.98725E+00_JPRB, 0.10218E+01_JPRB, 0.10921E+01_JPRB, 0.10414E+01_JPRB, 0.98600E+00_JPRB ,&
&  0.94060E+00_JPRB, 0.91405E+00_JPRB, 0.90428E+00_JPRB, 0.91949E+00_JPRB, 0.96182E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.87174E+00_JPRB, 0.86781E+00_JPRB, 0.86339E+00_JPRB, 0.86431E+00_JPRB, 0.86406E+00_JPRB, 0.85476E+00_JPRB, 0.84684E+00_JPRB ,&
&  0.84415E+00_JPRB, 0.83864E+00_JPRB, 0.83099E+00_JPRB, 0.81941E+00_JPRB, 0.80452E+00_JPRB, 0.79295E+00_JPRB, 0.78889E+00_JPRB ,&
&  0.78272E+00_JPRB, 0.77957E+00_JPRB, 0.77786E+00_JPRB, 0.77234E+00_JPRB, 0.76582E+00_JPRB, 0.75601E+00_JPRB, 0.73898E+00_JPRB ,&
&  0.71655E+00_JPRB, 0.70837E+00_JPRB, 0.71864E+00_JPRB, 0.72992E+00_JPRB, 0.73970E+00_JPRB, 0.74539E+00_JPRB, 0.76791E+00_JPRB ,&
&  0.90624E+00_JPRB, 0.11927E+01_JPRB, 0.14933E+01_JPRB, 0.18451E+01_JPRB, 0.20528E+01_JPRB, 0.19988E+01_JPRB, 0.19429E+01_JPRB ,&
&  0.18309E+01_JPRB, 0.17114E+01_JPRB, 0.15937E+01_JPRB, 0.14651E+01_JPRB, 0.13530E+01_JPRB, 0.12636E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.11676E+01_JPRB, 0.11337E+01_JPRB, 0.11068E+01_JPRB, 0.10778E+01_JPRB, 0.10556E+01_JPRB, 0.10401E+01_JPRB, 0.10372E+01_JPRB ,&
&  0.10405E+01_JPRB, 0.10491E+01_JPRB, 0.10621E+01_JPRB, 0.10755E+01_JPRB, 0.10949E+01_JPRB, 0.11495E+01_JPRB, 0.13290E+01_JPRB ,&
&  0.15673E+01_JPRB, 0.17931E+01_JPRB, 0.18937E+01_JPRB, 0.18606E+01_JPRB, 0.18154E+01_JPRB, 0.17554E+01_JPRB, 0.16269E+01_JPRB ,&
&  0.14687E+01_JPRB, 0.13609E+01_JPRB, 0.12764E+01_JPRB, 0.12026E+01_JPRB, 0.11476E+01_JPRB, 0.11037E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10429E+01_JPRB, 0.10381E+01_JPRB, 0.10333E+01_JPRB, 0.10228E+01_JPRB, 0.99839E+00_JPRB, 0.96384E+00_JPRB ,&
&  0.92557E+00_JPRB, 0.90881E+00_JPRB, 0.91074E+00_JPRB, 0.90562E+00_JPRB, 0.90011E+00_JPRB, 0.90572E+00_JPRB, 0.92048E+00_JPRB ,&
&  0.92856E+00_JPRB, 0.94397E+00_JPRB, 0.95360E+00_JPRB, 0.10066E+01_JPRB, 0.10122E+01_JPRB, 0.98963E+00_JPRB, 0.97540E+00_JPRB ,&
&  0.94816E+00_JPRB, 0.91567E+00_JPRB, 0.88968E+00_JPRB, 0.88496E+00_JPRB, 0.83542E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.85267E+00_JPRB, 0.84744E+00_JPRB, 0.83969E+00_JPRB, 0.83861E+00_JPRB, 0.84051E+00_JPRB, 0.82776E+00_JPRB, 0.81554E+00_JPRB ,&
&  0.80826E+00_JPRB, 0.80783E+00_JPRB, 0.80973E+00_JPRB, 0.80888E+00_JPRB, 0.80047E+00_JPRB, 0.78613E+00_JPRB, 0.77682E+00_JPRB ,&
&  0.76510E+00_JPRB, 0.75540E+00_JPRB, 0.75238E+00_JPRB, 0.75071E+00_JPRB, 0.74389E+00_JPRB, 0.73573E+00_JPRB, 0.72848E+00_JPRB ,&
&  0.71242E+00_JPRB, 0.70052E+00_JPRB, 0.69797E+00_JPRB, 0.70260E+00_JPRB, 0.70892E+00_JPRB, 0.71074E+00_JPRB, 0.74306E+00_JPRB ,&
&  0.89745E+00_JPRB, 0.97460E+00_JPRB, 0.10035E+01_JPRB, 0.12576E+01_JPRB, 0.14768E+01_JPRB, 0.15109E+01_JPRB, 0.15390E+01_JPRB ,&
&  0.15851E+01_JPRB, 0.15764E+01_JPRB, 0.15203E+01_JPRB, 0.14036E+01_JPRB, 0.13032E+01_JPRB, 0.12215E+01_JPRB, 0.11455E+01_JPRB ,&
&  0.10785E+01_JPRB, 0.10201E+01_JPRB, 0.98173E+00_JPRB, 0.95226E+00_JPRB, 0.93635E+00_JPRB, 0.91684E+00_JPRB, 0.90772E+00_JPRB ,&
&  0.90899E+00_JPRB, 0.92069E+00_JPRB, 0.93592E+00_JPRB, 0.95048E+00_JPRB, 0.97456E+00_JPRB, 0.10131E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.12303E+01_JPRB, 0.14123E+01_JPRB, 0.15367E+01_JPRB, 0.15688E+01_JPRB, 0.15383E+01_JPRB, 0.14743E+01_JPRB, 0.13821E+01_JPRB ,&
&  0.12807E+01_JPRB, 0.11944E+01_JPRB, 0.11309E+01_JPRB, 0.10943E+01_JPRB, 0.10669E+01_JPRB, 0.10449E+01_JPRB, 0.10235E+01_JPRB ,&
&  0.10050E+01_JPRB, 0.99271E+00_JPRB, 0.98387E+00_JPRB, 0.97219E+00_JPRB, 0.95628E+00_JPRB, 0.92511E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.84757E+00_JPRB, 0.82718E+00_JPRB, 0.82309E+00_JPRB, 0.82050E+00_JPRB, 0.82120E+00_JPRB, 0.83630E+00_JPRB, 0.85633E+00_JPRB ,&
&  0.86941E+00_JPRB, 0.88540E+00_JPRB, 0.90776E+00_JPRB, 0.91460E+00_JPRB, 0.95884E+00_JPRB, 0.95534E+00_JPRB, 0.95540E+00_JPRB ,&
&  0.94390E+00_JPRB, 0.92245E+00_JPRB, 0.89915E+00_JPRB, 0.77527E+00_JPRB, 0.83657E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.83749E+00_JPRB, 0.83173E+00_JPRB, 0.81667E+00_JPRB, 0.80675E+00_JPRB, 0.80003E+00_JPRB, 0.79178E+00_JPRB, 0.77870E+00_JPRB ,&
&  0.77042E+00_JPRB, 0.76793E+00_JPRB, 0.76635E+00_JPRB, 0.76621E+00_JPRB, 0.76520E+00_JPRB, 0.75767E+00_JPRB, 0.74661E+00_JPRB ,&
&  0.73592E+00_JPRB, 0.72692E+00_JPRB, 0.72804E+00_JPRB, 0.72661E+00_JPRB, 0.72032E+00_JPRB, 0.71509E+00_JPRB, 0.70864E+00_JPRB ,&
&  0.69325E+00_JPRB, 0.67663E+00_JPRB, 0.66388E+00_JPRB, 0.66718E+00_JPRB, 0.67456E+00_JPRB, 0.67123E+00_JPRB, 0.69987E+00_JPRB ,&
&  0.84377E+00_JPRB, 0.82749E+00_JPRB, 0.87088E+00_JPRB, 0.10158E+01_JPRB, 0.11101E+01_JPRB, 0.11965E+01_JPRB, 0.12600E+01_JPRB ,&
&  0.13128E+01_JPRB, 0.13503E+01_JPRB, 0.12705E+01_JPRB, 0.11689E+01_JPRB, 0.10678E+01_JPRB, 0.98057E+00_JPRB, 0.91671E+00_JPRB ,&
&  0.87336E+00_JPRB, 0.84866E+00_JPRB, 0.83153E+00_JPRB, 0.81279E+00_JPRB, 0.79803E+00_JPRB, 0.78502E+00_JPRB, 0.77794E+00_JPRB ,&
&  0.77530E+00_JPRB, 0.77949E+00_JPRB, 0.79021E+00_JPRB, 0.81235E+00_JPRB, 0.83708E+00_JPRB, 0.86508E+00_JPRB, 0.90454E+00_JPRB ,&
&  0.96880E+00_JPRB, 0.10711E+01_JPRB, 0.11780E+01_JPRB, 0.12315E+01_JPRB, 0.12267E+01_JPRB, 0.11858E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.10078E+01_JPRB, 0.97298E+00_JPRB, 0.94869E+00_JPRB, 0.92610E+00_JPRB, 0.90712E+00_JPRB, 0.89197E+00_JPRB ,&
&  0.87992E+00_JPRB, 0.86645E+00_JPRB, 0.85917E+00_JPRB, 0.84909E+00_JPRB, 0.83287E+00_JPRB, 0.80901E+00_JPRB, 0.78545E+00_JPRB ,&
&  0.76385E+00_JPRB, 0.74720E+00_JPRB, 0.73908E+00_JPRB, 0.74109E+00_JPRB, 0.74699E+00_JPRB, 0.76217E+00_JPRB, 0.78400E+00_JPRB ,&
&  0.80148E+00_JPRB, 0.81700E+00_JPRB, 0.83082E+00_JPRB, 0.86916E+00_JPRB, 0.89403E+00_JPRB, 0.91244E+00_JPRB, 0.92230E+00_JPRB ,&
&  0.92392E+00_JPRB, 0.92320E+00_JPRB, 0.74522E+00_JPRB, 0.81427E+00_JPRB, 0.83017E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.83418E+00_JPRB, 0.81339E+00_JPRB, 0.78846E+00_JPRB, 0.76990E+00_JPRB, 0.75904E+00_JPRB, 0.75350E+00_JPRB, 0.74744E+00_JPRB ,&
&  0.73991E+00_JPRB, 0.73879E+00_JPRB, 0.73930E+00_JPRB, 0.73963E+00_JPRB, 0.73912E+00_JPRB, 0.73424E+00_JPRB, 0.72435E+00_JPRB ,&
&  0.71331E+00_JPRB, 0.70044E+00_JPRB, 0.69808E+00_JPRB, 0.69687E+00_JPRB, 0.68998E+00_JPRB, 0.68493E+00_JPRB, 0.68379E+00_JPRB ,&
&  0.67680E+00_JPRB, 0.66569E+00_JPRB, 0.64608E+00_JPRB, 0.64021E+00_JPRB, 0.64351E+00_JPRB, 0.64432E+00_JPRB, 0.65723E+00_JPRB ,&
&  0.68246E+00_JPRB, 0.70366E+00_JPRB, 0.78292E+00_JPRB, 0.84147E+00_JPRB, 0.88123E+00_JPRB, 0.92171E+00_JPRB, 0.96495E+00_JPRB ,&
&  0.95995E+00_JPRB, 0.96082E+00_JPRB, 0.92129E+00_JPRB, 0.85958E+00_JPRB, 0.81429E+00_JPRB, 0.78151E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.74873E+00_JPRB, 0.74107E+00_JPRB, 0.73262E+00_JPRB, 0.72469E+00_JPRB, 0.72017E+00_JPRB, 0.71283E+00_JPRB, 0.70872E+00_JPRB ,&
&  0.70998E+00_JPRB, 0.71488E+00_JPRB, 0.71847E+00_JPRB, 0.72567E+00_JPRB, 0.73978E+00_JPRB, 0.76129E+00_JPRB, 0.78698E+00_JPRB ,&
&  0.81510E+00_JPRB, 0.85306E+00_JPRB, 0.90985E+00_JPRB, 0.95467E+00_JPRB, 0.96656E+00_JPRB, 0.94533E+00_JPRB, 0.90945E+00_JPRB ,&
&  0.87618E+00_JPRB, 0.85209E+00_JPRB, 0.83367E+00_JPRB, 0.81770E+00_JPRB, 0.79887E+00_JPRB, 0.78172E+00_JPRB, 0.76591E+00_JPRB ,&
&  0.75689E+00_JPRB, 0.74568E+00_JPRB, 0.73548E+00_JPRB, 0.72470E+00_JPRB, 0.71590E+00_JPRB, 0.71113E+00_JPRB, 0.70881E+00_JPRB ,&
&  0.70581E+00_JPRB, 0.69885E+00_JPRB, 0.69488E+00_JPRB, 0.69792E+00_JPRB, 0.70199E+00_JPRB, 0.71067E+00_JPRB, 0.72436E+00_JPRB ,&
&  0.74150E+00_JPRB, 0.75820E+00_JPRB, 0.77480E+00_JPRB, 0.80885E+00_JPRB, 0.84342E+00_JPRB, 0.87170E+00_JPRB, 0.88782E+00_JPRB ,&
&  0.89726E+00_JPRB, 0.89027E+00_JPRB, 0.88533E+00_JPRB, 0.87007E+00_JPRB, 0.85082E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.76962E+00_JPRB, 0.74443E+00_JPRB, 0.72356E+00_JPRB, 0.70878E+00_JPRB, 0.69965E+00_JPRB, 0.69658E+00_JPRB, 0.69239E+00_JPRB ,&
&  0.68924E+00_JPRB, 0.68484E+00_JPRB, 0.68217E+00_JPRB, 0.68469E+00_JPRB, 0.68596E+00_JPRB, 0.68623E+00_JPRB, 0.68660E+00_JPRB ,&
&  0.67982E+00_JPRB, 0.66558E+00_JPRB, 0.65430E+00_JPRB, 0.65078E+00_JPRB, 0.65108E+00_JPRB, 0.64648E+00_JPRB, 0.64462E+00_JPRB ,&
&  0.64761E+00_JPRB, 0.63937E+00_JPRB, 0.62537E+00_JPRB, 0.61621E+00_JPRB, 0.61698E+00_JPRB, 0.61487E+00_JPRB, 0.61599E+00_JPRB ,&
&  0.66406E+00_JPRB, 0.62569E+00_JPRB, 0.69008E+00_JPRB, 0.70363E+00_JPRB, 0.71738E+00_JPRB, 0.72887E+00_JPRB, 0.73537E+00_JPRB ,&
&  0.73302E+00_JPRB, 0.72337E+00_JPRB, 0.71175E+00_JPRB, 0.69757E+00_JPRB, 0.68426E+00_JPRB, 0.67053E+00_JPRB, 0.65976E+00_JPRB ,&
&  0.65500E+00_JPRB, 0.65004E+00_JPRB, 0.64936E+00_JPRB, 0.65183E+00_JPRB, 0.65801E+00_JPRB, 0.65974E+00_JPRB, 0.66144E+00_JPRB ,&
&  0.66400E+00_JPRB, 0.66582E+00_JPRB, 0.66589E+00_JPRB, 0.66522E+00_JPRB, 0.66430E+00_JPRB, 0.66958E+00_JPRB, 0.67842E+00_JPRB ,&
&  0.68900E+00_JPRB, 0.70713E+00_JPRB, 0.73350E+00_JPRB, 0.75597E+00_JPRB, 0.76493E+00_JPRB, 0.75967E+00_JPRB, 0.74564E+00_JPRB ,&
&  0.73644E+00_JPRB, 0.73437E+00_JPRB, 0.73365E+00_JPRB, 0.72652E+00_JPRB, 0.71632E+00_JPRB, 0.70667E+00_JPRB, 0.69310E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.66221E+00_JPRB, 0.65305E+00_JPRB, 0.64792E+00_JPRB, 0.64619E+00_JPRB, 0.64387E+00_JPRB, 0.64177E+00_JPRB ,&
&  0.64238E+00_JPRB, 0.64186E+00_JPRB, 0.64328E+00_JPRB, 0.64872E+00_JPRB, 0.65379E+00_JPRB, 0.66053E+00_JPRB, 0.66945E+00_JPRB ,&
&  0.68183E+00_JPRB, 0.69601E+00_JPRB, 0.70939E+00_JPRB, 0.73148E+00_JPRB, 0.76305E+00_JPRB, 0.78710E+00_JPRB, 0.80163E+00_JPRB ,&
&  0.81437E+00_JPRB, 0.81829E+00_JPRB, 0.81269E+00_JPRB, 0.80356E+00_JPRB, 0.78856E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.71121E+00_JPRB, 0.69571E+00_JPRB, 0.68348E+00_JPRB, 0.67393E+00_JPRB, 0.66733E+00_JPRB, 0.66507E+00_JPRB, 0.66213E+00_JPRB ,&
&  0.65923E+00_JPRB, 0.65648E+00_JPRB, 0.64893E+00_JPRB, 0.64190E+00_JPRB, 0.64158E+00_JPRB, 0.64772E+00_JPRB, 0.65232E+00_JPRB ,&
&  0.64889E+00_JPRB, 0.63739E+00_JPRB, 0.62656E+00_JPRB, 0.62384E+00_JPRB, 0.62618E+00_JPRB, 0.62820E+00_JPRB, 0.62753E+00_JPRB ,&
&  0.62914E+00_JPRB, 0.63032E+00_JPRB, 0.62197E+00_JPRB, 0.60427E+00_JPRB, 0.59628E+00_JPRB, 0.59199E+00_JPRB, 0.59954E+00_JPRB ,&
&  0.62097E+00_JPRB, 0.63948E+00_JPRB, 0.64754E+00_JPRB, 0.64217E+00_JPRB, 0.64174E+00_JPRB, 0.63996E+00_JPRB, 0.63453E+00_JPRB ,&
&  0.62911E+00_JPRB, 0.62211E+00_JPRB, 0.61614E+00_JPRB, 0.61165E+00_JPRB, 0.60875E+00_JPRB, 0.60697E+00_JPRB, 0.60751E+00_JPRB ,&
&  0.60451E+00_JPRB, 0.59966E+00_JPRB, 0.60058E+00_JPRB, 0.60684E+00_JPRB, 0.61549E+00_JPRB, 0.62083E+00_JPRB, 0.62582E+00_JPRB ,&
&  0.62999E+00_JPRB, 0.63228E+00_JPRB, 0.63521E+00_JPRB, 0.63581E+00_JPRB, 0.63188E+00_JPRB, 0.63117E+00_JPRB, 0.63133E+00_JPRB ,&
&  0.63430E+00_JPRB, 0.64068E+00_JPRB, 0.65294E+00_JPRB, 0.66771E+00_JPRB, 0.67406E+00_JPRB, 0.66961E+00_JPRB, 0.66314E+00_JPRB ,&
&  0.66246E+00_JPRB, 0.66738E+00_JPRB, 0.67304E+00_JPRB, 0.67612E+00_JPRB, 0.67489E+00_JPRB, 0.66625E+00_JPRB, 0.65544E+00_JPRB ,&
&  0.64385E+00_JPRB, 0.63210E+00_JPRB, 0.62160E+00_JPRB, 0.61473E+00_JPRB, 0.60929E+00_JPRB, 0.60534E+00_JPRB, 0.60296E+00_JPRB ,&
&  0.60221E+00_JPRB, 0.60266E+00_JPRB, 0.60655E+00_JPRB, 0.61385E+00_JPRB, 0.61860E+00_JPRB, 0.62793E+00_JPRB, 0.63923E+00_JPRB ,&
&  0.65082E+00_JPRB, 0.66222E+00_JPRB, 0.67573E+00_JPRB, 0.69463E+00_JPRB, 0.71641E+00_JPRB, 0.73374E+00_JPRB, 0.74721E+00_JPRB ,&
&  0.75588E+00_JPRB, 0.75539E+00_JPRB, 0.74738E+00_JPRB, 0.73768E+00_JPRB, 0.72600E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.65783E+00_JPRB, 0.64936E+00_JPRB, 0.64242E+00_JPRB, 0.63634E+00_JPRB, 0.63108E+00_JPRB, 0.62883E+00_JPRB, 0.62669E+00_JPRB ,&
&  0.62470E+00_JPRB, 0.62175E+00_JPRB, 0.61963E+00_JPRB, 0.61533E+00_JPRB, 0.61204E+00_JPRB, 0.61509E+00_JPRB, 0.62096E+00_JPRB ,&
&  0.61950E+00_JPRB, 0.61221E+00_JPRB, 0.60640E+00_JPRB, 0.60602E+00_JPRB, 0.60917E+00_JPRB, 0.61143E+00_JPRB, 0.61395E+00_JPRB ,&
&  0.61343E+00_JPRB, 0.61244E+00_JPRB, 0.61020E+00_JPRB, 0.59362E+00_JPRB, 0.57832E+00_JPRB, 0.57087E+00_JPRB, 0.56747E+00_JPRB ,&
&  0.57130E+00_JPRB, 0.59111E+00_JPRB, 0.57009E+00_JPRB, 0.57999E+00_JPRB, 0.57583E+00_JPRB, 0.56910E+00_JPRB, 0.56467E+00_JPRB ,&
&  0.56226E+00_JPRB, 0.55986E+00_JPRB, 0.55760E+00_JPRB, 0.55840E+00_JPRB, 0.55965E+00_JPRB, 0.55912E+00_JPRB, 0.55956E+00_JPRB ,&
&  0.55833E+00_JPRB, 0.56020E+00_JPRB, 0.56232E+00_JPRB, 0.56757E+00_JPRB, 0.57141E+00_JPRB, 0.57392E+00_JPRB, 0.57733E+00_JPRB ,&
&  0.58226E+00_JPRB, 0.58686E+00_JPRB, 0.59115E+00_JPRB, 0.59480E+00_JPRB, 0.59634E+00_JPRB, 0.59613E+00_JPRB, 0.59194E+00_JPRB ,&
&  0.58862E+00_JPRB, 0.58870E+00_JPRB, 0.59320E+00_JPRB, 0.60325E+00_JPRB, 0.61215E+00_JPRB, 0.61362E+00_JPRB, 0.61238E+00_JPRB ,&
&  0.61284E+00_JPRB, 0.61677E+00_JPRB, 0.61802E+00_JPRB, 0.61708E+00_JPRB, 0.61512E+00_JPRB, 0.61234E+00_JPRB, 0.60743E+00_JPRB ,&
&  0.60068E+00_JPRB, 0.59477E+00_JPRB, 0.58858E+00_JPRB, 0.57645E+00_JPRB, 0.56744E+00_JPRB, 0.56664E+00_JPRB, 0.56852E+00_JPRB ,&
&  0.56828E+00_JPRB, 0.56690E+00_JPRB, 0.56839E+00_JPRB, 0.57442E+00_JPRB, 0.57536E+00_JPRB, 0.58354E+00_JPRB, 0.59868E+00_JPRB ,&
&  0.61376E+00_JPRB, 0.62688E+00_JPRB, 0.63772E+00_JPRB, 0.64909E+00_JPRB, 0.66049E+00_JPRB, 0.66957E+00_JPRB, 0.67733E+00_JPRB ,&
&  0.68080E+00_JPRB, 0.67880E+00_JPRB, 0.67268E+00_JPRB, 0.66902E+00_JPRB, 0.66583E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.62627E+00_JPRB, 0.62463E+00_JPRB, 0.62272E+00_JPRB, 0.62016E+00_JPRB, 0.61652E+00_JPRB, 0.61323E+00_JPRB, 0.60826E+00_JPRB ,&
&  0.60330E+00_JPRB, 0.59943E+00_JPRB, 0.59775E+00_JPRB, 0.59702E+00_JPRB, 0.59816E+00_JPRB, 0.60240E+00_JPRB, 0.60948E+00_JPRB ,&
&  0.60942E+00_JPRB, 0.60210E+00_JPRB, 0.59848E+00_JPRB, 0.59830E+00_JPRB, 0.60052E+00_JPRB, 0.60205E+00_JPRB, 0.60443E+00_JPRB ,&
&  0.60581E+00_JPRB, 0.60597E+00_JPRB, 0.60445E+00_JPRB, 0.59615E+00_JPRB, 0.57838E+00_JPRB, 0.56789E+00_JPRB, 0.55923E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.55401E+00_JPRB, 0.54984E+00_JPRB, 0.54873E+00_JPRB, 0.54747E+00_JPRB, 0.54553E+00_JPRB, 0.54541E+00_JPRB ,&
&  0.54363E+00_JPRB, 0.54124E+00_JPRB, 0.54014E+00_JPRB, 0.54079E+00_JPRB, 0.54262E+00_JPRB, 0.54416E+00_JPRB, 0.54381E+00_JPRB ,&
&  0.54447E+00_JPRB, 0.54786E+00_JPRB, 0.54846E+00_JPRB, 0.54820E+00_JPRB, 0.54766E+00_JPRB, 0.54689E+00_JPRB, 0.54679E+00_JPRB ,&
&  0.54921E+00_JPRB, 0.55469E+00_JPRB, 0.55963E+00_JPRB, 0.56483E+00_JPRB, 0.56741E+00_JPRB, 0.56869E+00_JPRB, 0.56877E+00_JPRB ,&
&  0.56544E+00_JPRB, 0.56112E+00_JPRB, 0.55953E+00_JPRB, 0.56247E+00_JPRB, 0.56862E+00_JPRB, 0.57490E+00_JPRB, 0.57800E+00_JPRB ,&
&  0.58019E+00_JPRB, 0.58110E+00_JPRB, 0.57894E+00_JPRB, 0.57790E+00_JPRB, 0.57535E+00_JPRB, 0.57090E+00_JPRB, 0.56841E+00_JPRB ,&
&  0.56596E+00_JPRB, 0.56353E+00_JPRB, 0.56099E+00_JPRB, 0.55157E+00_JPRB, 0.54387E+00_JPRB, 0.53965E+00_JPRB, 0.53716E+00_JPRB ,&
&  0.53650E+00_JPRB, 0.53581E+00_JPRB, 0.53571E+00_JPRB, 0.53693E+00_JPRB, 0.53932E+00_JPRB, 0.54706E+00_JPRB, 0.56179E+00_JPRB ,&
&  0.58292E+00_JPRB, 0.60224E+00_JPRB, 0.61265E+00_JPRB, 0.61991E+00_JPRB, 0.62462E+00_JPRB, 0.62700E+00_JPRB, 0.62936E+00_JPRB ,&
&  0.63292E+00_JPRB, 0.63419E+00_JPRB, 0.63216E+00_JPRB, 0.62955E+00_JPRB, 0.62831E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.59481E+00_JPRB, 0.59521E+00_JPRB, 0.59382E+00_JPRB, 0.59235E+00_JPRB, 0.59067E+00_JPRB, 0.58910E+00_JPRB, 0.58692E+00_JPRB ,&
&  0.58199E+00_JPRB, 0.57557E+00_JPRB, 0.57186E+00_JPRB, 0.57047E+00_JPRB, 0.57647E+00_JPRB, 0.58500E+00_JPRB, 0.59252E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.58142E+00_JPRB, 0.57822E+00_JPRB, 0.58001E+00_JPRB, 0.58247E+00_JPRB, 0.58514E+00_JPRB, 0.58871E+00_JPRB ,&
&  0.59266E+00_JPRB, 0.59659E+00_JPRB, 0.59820E+00_JPRB, 0.59486E+00_JPRB, 0.58727E+00_JPRB, 0.57277E+00_JPRB, 0.56065E+00_JPRB ,&
&  0.55286E+00_JPRB, 0.54716E+00_JPRB, 0.54266E+00_JPRB, 0.54144E+00_JPRB, 0.53908E+00_JPRB, 0.53533E+00_JPRB, 0.53026E+00_JPRB ,&
&  0.52617E+00_JPRB, 0.52454E+00_JPRB, 0.52750E+00_JPRB, 0.52991E+00_JPRB, 0.53109E+00_JPRB, 0.53058E+00_JPRB, 0.52715E+00_JPRB ,&
&  0.52487E+00_JPRB, 0.52365E+00_JPRB, 0.52256E+00_JPRB, 0.52325E+00_JPRB, 0.52004E+00_JPRB, 0.51745E+00_JPRB, 0.51909E+00_JPRB ,&
&  0.52348E+00_JPRB, 0.52764E+00_JPRB, 0.53170E+00_JPRB, 0.53529E+00_JPRB, 0.53730E+00_JPRB, 0.54002E+00_JPRB, 0.54148E+00_JPRB ,&
&  0.54024E+00_JPRB, 0.53608E+00_JPRB, 0.53223E+00_JPRB, 0.53245E+00_JPRB, 0.53460E+00_JPRB, 0.53719E+00_JPRB, 0.54061E+00_JPRB ,&
&  0.54100E+00_JPRB, 0.53575E+00_JPRB, 0.53060E+00_JPRB, 0.52607E+00_JPRB, 0.52192E+00_JPRB, 0.51881E+00_JPRB, 0.51737E+00_JPRB ,&
&  0.51710E+00_JPRB, 0.51391E+00_JPRB, 0.50776E+00_JPRB, 0.50277E+00_JPRB, 0.50245E+00_JPRB, 0.50227E+00_JPRB, 0.49897E+00_JPRB ,&
&  0.50004E+00_JPRB, 0.50313E+00_JPRB, 0.50222E+00_JPRB, 0.50036E+00_JPRB, 0.50237E+00_JPRB, 0.50824E+00_JPRB, 0.52046E+00_JPRB ,&
&  0.53675E+00_JPRB, 0.55520E+00_JPRB, 0.57211E+00_JPRB, 0.58637E+00_JPRB, 0.59467E+00_JPRB, 0.59661E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.59433E+00_JPRB, 0.59537E+00_JPRB, 0.59472E+00_JPRB, 0.59329E+00_JPRB, 0.59367E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.56387E+00_JPRB, 0.56397E+00_JPRB, 0.56428E+00_JPRB, 0.56550E+00_JPRB, 0.56631E+00_JPRB, 0.56525E+00_JPRB, 0.56483E+00_JPRB ,&
&  0.56152E+00_JPRB, 0.55679E+00_JPRB, 0.55416E+00_JPRB, 0.55625E+00_JPRB, 0.56614E+00_JPRB, 0.57818E+00_JPRB, 0.58506E+00_JPRB ,&
&  0.57987E+00_JPRB, 0.56658E+00_JPRB, 0.56320E+00_JPRB, 0.56542E+00_JPRB, 0.56769E+00_JPRB, 0.57180E+00_JPRB, 0.57780E+00_JPRB ,&
&  0.58330E+00_JPRB, 0.58699E+00_JPRB, 0.59105E+00_JPRB, 0.59256E+00_JPRB, 0.59226E+00_JPRB, 0.58454E+00_JPRB, 0.57309E+00_JPRB ,&
&  0.56215E+00_JPRB, 0.55412E+00_JPRB, 0.54819E+00_JPRB, 0.54435E+00_JPRB, 0.53923E+00_JPRB, 0.53440E+00_JPRB, 0.52819E+00_JPRB ,&
&  0.52561E+00_JPRB, 0.52595E+00_JPRB, 0.52746E+00_JPRB, 0.52583E+00_JPRB, 0.52314E+00_JPRB, 0.51733E+00_JPRB, 0.51213E+00_JPRB ,&
&  0.50954E+00_JPRB, 0.50984E+00_JPRB, 0.51005E+00_JPRB, 0.50804E+00_JPRB, 0.50353E+00_JPRB, 0.50013E+00_JPRB, 0.50085E+00_JPRB ,&
&  0.50639E+00_JPRB, 0.50603E+00_JPRB, 0.50503E+00_JPRB, 0.50638E+00_JPRB, 0.50921E+00_JPRB, 0.51159E+00_JPRB, 0.51096E+00_JPRB ,&
&  0.51003E+00_JPRB, 0.50948E+00_JPRB, 0.50923E+00_JPRB, 0.51066E+00_JPRB, 0.51051E+00_JPRB, 0.50816E+00_JPRB, 0.50234E+00_JPRB ,&
&  0.49361E+00_JPRB, 0.48281E+00_JPRB, 0.47848E+00_JPRB, 0.47532E+00_JPRB, 0.47223E+00_JPRB, 0.47175E+00_JPRB, 0.47341E+00_JPRB ,&
&  0.47218E+00_JPRB, 0.46603E+00_JPRB, 0.46341E+00_JPRB, 0.46440E+00_JPRB, 0.46428E+00_JPRB, 0.46742E+00_JPRB, 0.47231E+00_JPRB ,&
&  0.48385E+00_JPRB, 0.49056E+00_JPRB, 0.49256E+00_JPRB, 0.49030E+00_JPRB, 0.49038E+00_JPRB, 0.49209E+00_JPRB, 0.49668E+00_JPRB ,&
&  0.50362E+00_JPRB, 0.51130E+00_JPRB, 0.51941E+00_JPRB, 0.52837E+00_JPRB, 0.53922E+00_JPRB, 0.54951E+00_JPRB, 0.55363E+00_JPRB ,&
&  0.55421E+00_JPRB, 0.55507E+00_JPRB, 0.55689E+00_JPRB, 0.55892E+00_JPRB, 0.56230E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.52679E+00_JPRB, 0.52930E+00_JPRB, 0.53184E+00_JPRB, 0.53368E+00_JPRB, 0.53664E+00_JPRB, 0.54179E+00_JPRB, 0.54387E+00_JPRB ,&
&  0.54179E+00_JPRB, 0.53940E+00_JPRB, 0.53927E+00_JPRB, 0.54315E+00_JPRB, 0.55070E+00_JPRB, 0.55831E+00_JPRB, 0.56614E+00_JPRB ,&
&  0.56132E+00_JPRB, 0.54773E+00_JPRB, 0.54283E+00_JPRB, 0.54490E+00_JPRB, 0.54989E+00_JPRB, 0.55537E+00_JPRB, 0.56200E+00_JPRB ,&
&  0.56828E+00_JPRB, 0.57228E+00_JPRB, 0.57649E+00_JPRB, 0.58036E+00_JPRB, 0.58278E+00_JPRB, 0.58065E+00_JPRB, 0.57604E+00_JPRB ,&
&  0.57048E+00_JPRB, 0.56441E+00_JPRB, 0.55848E+00_JPRB, 0.54295E+00_JPRB, 0.54055E+00_JPRB, 0.53184E+00_JPRB, 0.52483E+00_JPRB ,&
&  0.52024E+00_JPRB, 0.51636E+00_JPRB, 0.51379E+00_JPRB, 0.51045E+00_JPRB, 0.50551E+00_JPRB, 0.49719E+00_JPRB, 0.49404E+00_JPRB ,&
&  0.49198E+00_JPRB, 0.49196E+00_JPRB, 0.48615E+00_JPRB, 0.48264E+00_JPRB, 0.47876E+00_JPRB, 0.47556E+00_JPRB, 0.47894E+00_JPRB ,&
&  0.48354E+00_JPRB, 0.48054E+00_JPRB, 0.47628E+00_JPRB, 0.47617E+00_JPRB, 0.48076E+00_JPRB, 0.48253E+00_JPRB, 0.48338E+00_JPRB ,&
&  0.48489E+00_JPRB, 0.48489E+00_JPRB, 0.47955E+00_JPRB, 0.47391E+00_JPRB, 0.46544E+00_JPRB, 0.45384E+00_JPRB, 0.45014E+00_JPRB ,&
&  0.44681E+00_JPRB, 0.44139E+00_JPRB, 0.43883E+00_JPRB, 0.43799E+00_JPRB, 0.43778E+00_JPRB, 0.43903E+00_JPRB, 0.43960E+00_JPRB ,&
&  0.44013E+00_JPRB, 0.43670E+00_JPRB, 0.43175E+00_JPRB, 0.42778E+00_JPRB, 0.42229E+00_JPRB, 0.42596E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.45049E+00_JPRB, 0.45888E+00_JPRB, 0.46435E+00_JPRB, 0.46780E+00_JPRB, 0.47085E+00_JPRB, 0.47204E+00_JPRB, 0.47429E+00_JPRB ,&
&  0.47727E+00_JPRB, 0.47858E+00_JPRB, 0.47740E+00_JPRB, 0.47859E+00_JPRB, 0.48282E+00_JPRB, 0.48885E+00_JPRB, 0.49590E+00_JPRB ,&
&  0.50273E+00_JPRB, 0.50849E+00_JPRB, 0.51330E+00_JPRB, 0.51919E+00_JPRB, 0.52320E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.49616E+00_JPRB, 0.50081E+00_JPRB, 0.50539E+00_JPRB, 0.51035E+00_JPRB, 0.51719E+00_JPRB, 0.52095E+00_JPRB, 0.52229E+00_JPRB ,&
&  0.52362E+00_JPRB, 0.52397E+00_JPRB, 0.52518E+00_JPRB, 0.52944E+00_JPRB, 0.53608E+00_JPRB, 0.54125E+00_JPRB, 0.54757E+00_JPRB ,&
&  0.54752E+00_JPRB, 0.53790E+00_JPRB, 0.52955E+00_JPRB, 0.52927E+00_JPRB, 0.53503E+00_JPRB, 0.54098E+00_JPRB, 0.54827E+00_JPRB ,&
&  0.55539E+00_JPRB, 0.56038E+00_JPRB, 0.56306E+00_JPRB, 0.56434E+00_JPRB, 0.56604E+00_JPRB, 0.56524E+00_JPRB, 0.56202E+00_JPRB ,&
&  0.55736E+00_JPRB, 0.54725E+00_JPRB, 0.52765E+00_JPRB, 0.52523E+00_JPRB, 0.51652E+00_JPRB, 0.50909E+00_JPRB, 0.50401E+00_JPRB ,&
&  0.49998E+00_JPRB, 0.49645E+00_JPRB, 0.49222E+00_JPRB, 0.48812E+00_JPRB, 0.48330E+00_JPRB, 0.47514E+00_JPRB, 0.47045E+00_JPRB ,&
&  0.46607E+00_JPRB, 0.46254E+00_JPRB, 0.45884E+00_JPRB, 0.45807E+00_JPRB, 0.45748E+00_JPRB, 0.45940E+00_JPRB, 0.46009E+00_JPRB ,&
&  0.45624E+00_JPRB, 0.45185E+00_JPRB, 0.44858E+00_JPRB, 0.45014E+00_JPRB, 0.45266E+00_JPRB, 0.45437E+00_JPRB, 0.45497E+00_JPRB ,&
&  0.45191E+00_JPRB, 0.44630E+00_JPRB, 0.44041E+00_JPRB, 0.43391E+00_JPRB, 0.42614E+00_JPRB, 0.40053E+00_JPRB, 0.38991E+00_JPRB ,&
&  0.38810E+00_JPRB, 0.39783E+00_JPRB, 0.40904E+00_JPRB, 0.40883E+00_JPRB, 0.41292E+00_JPRB, 0.41368E+00_JPRB, 0.41108E+00_JPRB ,&
&  0.40079E+00_JPRB, 0.38768E+00_JPRB, 0.38003E+00_JPRB, 0.37693E+00_JPRB, 0.37469E+00_JPRB, 0.37745E+00_JPRB, 0.38478E+00_JPRB ,&
&  0.39632E+00_JPRB, 0.40835E+00_JPRB, 0.41727E+00_JPRB, 0.42122E+00_JPRB, 0.41973E+00_JPRB, 0.41665E+00_JPRB, 0.41255E+00_JPRB ,&
&  0.40874E+00_JPRB, 0.41575E+00_JPRB, 0.43144E+00_JPRB, 0.45085E+00_JPRB, 0.45357E+00_JPRB, 0.45957E+00_JPRB, 0.46376E+00_JPRB ,&
&  0.46970E+00_JPRB, 0.47719E+00_JPRB, 0.48342E+00_JPRB, 0.48959E+00_JPRB, 0.49324E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.47199E+00_JPRB, 0.47730E+00_JPRB, 0.48443E+00_JPRB, 0.49233E+00_JPRB, 0.49810E+00_JPRB, 0.50107E+00_JPRB, 0.50424E+00_JPRB ,&
&  0.50800E+00_JPRB, 0.50964E+00_JPRB, 0.51067E+00_JPRB, 0.51331E+00_JPRB, 0.51896E+00_JPRB, 0.52244E+00_JPRB, 0.52784E+00_JPRB ,&
&  0.53015E+00_JPRB, 0.52767E+00_JPRB, 0.52044E+00_JPRB, 0.51851E+00_JPRB, 0.52238E+00_JPRB, 0.52739E+00_JPRB, 0.53257E+00_JPRB ,&
&  0.53773E+00_JPRB, 0.54239E+00_JPRB, 0.54445E+00_JPRB, 0.54471E+00_JPRB, 0.54454E+00_JPRB, 0.54408E+00_JPRB, 0.54120E+00_JPRB ,&
&  0.53438E+00_JPRB, 0.49324E+00_JPRB, 0.46708E+00_JPRB, 0.47340E+00_JPRB, 0.48409E+00_JPRB, 0.47699E+00_JPRB, 0.47545E+00_JPRB ,&
&  0.47398E+00_JPRB, 0.46997E+00_JPRB, 0.46407E+00_JPRB, 0.45901E+00_JPRB, 0.45302E+00_JPRB, 0.44509E+00_JPRB, 0.43888E+00_JPRB ,&
&  0.43689E+00_JPRB, 0.44231E+00_JPRB, 0.44345E+00_JPRB, 0.44426E+00_JPRB, 0.44129E+00_JPRB, 0.44175E+00_JPRB, 0.43914E+00_JPRB ,&
&  0.43224E+00_JPRB, 0.42456E+00_JPRB, 0.42148E+00_JPRB, 0.42062E+00_JPRB, 0.42179E+00_JPRB, 0.42458E+00_JPRB, 0.42337E+00_JPRB ,&
&  0.41079E+00_JPRB, 0.39893E+00_JPRB, 0.37983E+00_JPRB, 0.36128E+00_JPRB, 0.34803E+00_JPRB, 0.34221E+00_JPRB, 0.33804E+00_JPRB ,&
&  0.33778E+00_JPRB, 0.34518E+00_JPRB, 0.36483E+00_JPRB, 0.38778E+00_JPRB, 0.39801E+00_JPRB, 0.37922E+00_JPRB, 0.36467E+00_JPRB ,&
&  0.35614E+00_JPRB, 0.35140E+00_JPRB, 0.34724E+00_JPRB, 0.34621E+00_JPRB, 0.34723E+00_JPRB, 0.35001E+00_JPRB, 0.35382E+00_JPRB ,&
&  0.35929E+00_JPRB, 0.36438E+00_JPRB, 0.36721E+00_JPRB, 0.36856E+00_JPRB, 0.36939E+00_JPRB, 0.37035E+00_JPRB, 0.36979E+00_JPRB ,&
&  0.37003E+00_JPRB, 0.37453E+00_JPRB, 0.38260E+00_JPRB, 0.39234E+00_JPRB, 0.40095E+00_JPRB, 0.41091E+00_JPRB, 0.42481E+00_JPRB ,&
&  0.43971E+00_JPRB, 0.45032E+00_JPRB, 0.45446E+00_JPRB, 0.46134E+00_JPRB, 0.46705E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.46011E+00_JPRB, 0.46778E+00_JPRB, 0.47263E+00_JPRB, 0.48248E+00_JPRB, 0.48566E+00_JPRB, 0.48925E+00_JPRB, 0.49257E+00_JPRB ,&
&  0.49964E+00_JPRB, 0.50349E+00_JPRB, 0.50152E+00_JPRB, 0.50114E+00_JPRB, 0.50026E+00_JPRB, 0.49843E+00_JPRB, 0.49971E+00_JPRB ,&
&  0.50018E+00_JPRB, 0.50257E+00_JPRB, 0.49993E+00_JPRB, 0.49726E+00_JPRB, 0.49933E+00_JPRB, 0.50439E+00_JPRB, 0.50770E+00_JPRB ,&
&  0.50203E+00_JPRB, 0.49990E+00_JPRB, 0.50813E+00_JPRB, 0.50956E+00_JPRB, 0.51013E+00_JPRB, 0.51164E+00_JPRB, 0.51100E+00_JPRB ,&
&  0.49619E+00_JPRB, 0.46923E+00_JPRB, 0.44175E+00_JPRB, 0.44570E+00_JPRB, 0.46062E+00_JPRB, 0.45243E+00_JPRB, 0.45051E+00_JPRB ,&
&  0.44770E+00_JPRB, 0.44292E+00_JPRB, 0.43851E+00_JPRB, 0.43604E+00_JPRB, 0.43325E+00_JPRB, 0.42797E+00_JPRB, 0.42643E+00_JPRB ,&
&  0.42475E+00_JPRB, 0.43476E+00_JPRB, 0.42802E+00_JPRB, 0.40302E+00_JPRB, 0.38584E+00_JPRB, 0.37620E+00_JPRB, 0.36351E+00_JPRB ,&
&  0.34839E+00_JPRB, 0.34055E+00_JPRB, 0.33985E+00_JPRB, 0.33722E+00_JPRB, 0.33806E+00_JPRB, 0.33923E+00_JPRB, 0.34033E+00_JPRB ,&
&  0.34071E+00_JPRB, 0.34002E+00_JPRB, 0.33496E+00_JPRB, 0.32849E+00_JPRB, 0.32502E+00_JPRB, 0.32515E+00_JPRB, 0.32860E+00_JPRB ,&
&  0.33574E+00_JPRB, 0.34266E+00_JPRB, 0.35849E+00_JPRB, 0.36640E+00_JPRB, 0.35644E+00_JPRB, 0.35255E+00_JPRB, 0.34822E+00_JPRB ,&
&  0.33620E+00_JPRB, 0.33187E+00_JPRB, 0.32831E+00_JPRB, 0.32644E+00_JPRB, 0.32755E+00_JPRB, 0.33147E+00_JPRB, 0.33300E+00_JPRB ,&
&  0.33573E+00_JPRB, 0.33908E+00_JPRB, 0.34047E+00_JPRB, 0.34152E+00_JPRB, 0.34244E+00_JPRB, 0.34483E+00_JPRB, 0.34727E+00_JPRB ,&
&  0.34911E+00_JPRB, 0.35252E+00_JPRB, 0.35658E+00_JPRB, 0.36184E+00_JPRB, 0.36648E+00_JPRB, 0.36912E+00_JPRB, 0.37796E+00_JPRB ,&
&  0.38772E+00_JPRB, 0.40741E+00_JPRB, 0.43885E+00_JPRB, 0.46223E+00_JPRB, 0.45449E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.46049E+00_JPRB, 0.46472E+00_JPRB, 0.46489E+00_JPRB, 0.46818E+00_JPRB, 0.47392E+00_JPRB, 0.48041E+00_JPRB, 0.48607E+00_JPRB ,&
&  0.49165E+00_JPRB, 0.49131E+00_JPRB, 0.48630E+00_JPRB, 0.47806E+00_JPRB, 0.47128E+00_JPRB, 0.45449E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.43909E+00_JPRB, 0.43937E+00_JPRB, 0.43970E+00_JPRB, 0.44155E+00_JPRB, 0.44238E+00_JPRB, 0.44750E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44551E+00_JPRB, 0.43986E+00_JPRB, 0.43400E+00_JPRB, 0.43277E+00_JPRB, 0.43542E+00_JPRB, 0.43709E+00_JPRB, 0.43362E+00_JPRB ,&
&  0.42986E+00_JPRB, 0.42978E+00_JPRB, 0.43025E+00_JPRB, 0.43168E+00_JPRB, 0.44103E+00_JPRB, 0.43435E+00_JPRB, 0.42956E+00_JPRB ,&
&  0.42253E+00_JPRB, 0.41666E+00_JPRB, 0.41284E+00_JPRB, 0.41337E+00_JPRB, 0.41725E+00_JPRB, 0.41509E+00_JPRB, 0.41804E+00_JPRB ,&
&  0.40779E+00_JPRB, 0.39930E+00_JPRB, 0.38376E+00_JPRB, 0.37099E+00_JPRB, 0.35633E+00_JPRB, 0.34634E+00_JPRB, 0.33730E+00_JPRB ,&
&  0.32968E+00_JPRB, 0.32298E+00_JPRB, 0.31680E+00_JPRB, 0.31502E+00_JPRB, 0.31610E+00_JPRB, 0.31584E+00_JPRB, 0.31489E+00_JPRB ,&
&  0.31518E+00_JPRB, 0.31605E+00_JPRB, 0.31671E+00_JPRB, 0.31782E+00_JPRB, 0.31981E+00_JPRB, 0.32294E+00_JPRB, 0.32727E+00_JPRB ,&
&  0.33393E+00_JPRB, 0.33903E+00_JPRB, 0.34171E+00_JPRB, 0.34353E+00_JPRB, 0.34035E+00_JPRB, 0.33447E+00_JPRB, 0.32890E+00_JPRB ,&
&  0.32283E+00_JPRB, 0.32006E+00_JPRB, 0.31738E+00_JPRB, 0.31613E+00_JPRB, 0.31579E+00_JPRB, 0.31668E+00_JPRB, 0.31873E+00_JPRB ,&
&  0.32076E+00_JPRB, 0.32355E+00_JPRB, 0.32537E+00_JPRB, 0.32755E+00_JPRB, 0.33011E+00_JPRB, 0.33232E+00_JPRB, 0.33459E+00_JPRB ,&
&  0.33718E+00_JPRB, 0.33921E+00_JPRB, 0.34279E+00_JPRB, 0.34726E+00_JPRB, 0.35599E+00_JPRB, 0.36552E+00_JPRB, 0.38222E+00_JPRB ,&
&  0.40100E+00_JPRB, 0.43154E+00_JPRB, 0.43883E+00_JPRB, 0.45035E+00_JPRB, 0.45937E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.45506E+00_JPRB, 0.45772E+00_JPRB, 0.46133E+00_JPRB, 0.46710E+00_JPRB, 0.47449E+00_JPRB, 0.48153E+00_JPRB, 0.48341E+00_JPRB ,&
&  0.47334E+00_JPRB, 0.46048E+00_JPRB, 0.44183E+00_JPRB, 0.42742E+00_JPRB, 0.41297E+00_JPRB, 0.40235E+00_JPRB, 0.39395E+00_JPRB ,&
&  0.38805E+00_JPRB, 0.38787E+00_JPRB, 0.38786E+00_JPRB, 0.38902E+00_JPRB, 0.38746E+00_JPRB, 0.38819E+00_JPRB, 0.38803E+00_JPRB ,&
&  0.38628E+00_JPRB, 0.38602E+00_JPRB, 0.38594E+00_JPRB, 0.38927E+00_JPRB, 0.39616E+00_JPRB, 0.40232E+00_JPRB, 0.40840E+00_JPRB ,&
&  0.41565E+00_JPRB, 0.41705E+00_JPRB, 0.42098E+00_JPRB, 0.41919E+00_JPRB, 0.41573E+00_JPRB, 0.41133E+00_JPRB, 0.40504E+00_JPRB ,&
&  0.40125E+00_JPRB, 0.39771E+00_JPRB, 0.39696E+00_JPRB, 0.39730E+00_JPRB, 0.39725E+00_JPRB, 0.40158E+00_JPRB, 0.39689E+00_JPRB ,&
&  0.39012E+00_JPRB, 0.37915E+00_JPRB, 0.36541E+00_JPRB, 0.35492E+00_JPRB, 0.34424E+00_JPRB, 0.33661E+00_JPRB, 0.32980E+00_JPRB ,&
&  0.32358E+00_JPRB, 0.31946E+00_JPRB, 0.31510E+00_JPRB, 0.31182E+00_JPRB, 0.31076E+00_JPRB, 0.30841E+00_JPRB, 0.30702E+00_JPRB ,&
&  0.30641E+00_JPRB, 0.30681E+00_JPRB, 0.30892E+00_JPRB, 0.31077E+00_JPRB, 0.31338E+00_JPRB, 0.31619E+00_JPRB, 0.31848E+00_JPRB ,&
&  0.32162E+00_JPRB, 0.32421E+00_JPRB, 0.32456E+00_JPRB, 0.32401E+00_JPRB, 0.32167E+00_JPRB, 0.31846E+00_JPRB, 0.31582E+00_JPRB ,&
&  0.31290E+00_JPRB, 0.31091E+00_JPRB, 0.31028E+00_JPRB, 0.30925E+00_JPRB, 0.30980E+00_JPRB, 0.31110E+00_JPRB, 0.31194E+00_JPRB ,&
&  0.31388E+00_JPRB, 0.31546E+00_JPRB, 0.31739E+00_JPRB, 0.32094E+00_JPRB, 0.32405E+00_JPRB, 0.32681E+00_JPRB, 0.32952E+00_JPRB ,&
&  0.32936E+00_JPRB, 0.33121E+00_JPRB, 0.33389E+00_JPRB, 0.33914E+00_JPRB, 0.35495E+00_JPRB, 0.36485E+00_JPRB, 0.38778E+00_JPRB ,&
&  0.40503E+00_JPRB, 0.43714E+00_JPRB, 0.43409E+00_JPRB, 0.44291E+00_JPRB, 0.45088E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.44899E+00_JPRB, 0.45620E+00_JPRB, 0.46277E+00_JPRB, 0.46817E+00_JPRB, 0.47167E+00_JPRB, 0.47504E+00_JPRB, 0.47220E+00_JPRB ,&
&  0.46612E+00_JPRB, 0.45198E+00_JPRB, 0.44087E+00_JPRB, 0.42928E+00_JPRB, 0.41976E+00_JPRB, 0.41040E+00_JPRB, 0.40178E+00_JPRB ,&
&  0.39332E+00_JPRB, 0.38618E+00_JPRB, 0.38004E+00_JPRB, 0.37456E+00_JPRB, 0.36720E+00_JPRB, 0.36064E+00_JPRB, 0.35535E+00_JPRB ,&
&  0.35097E+00_JPRB, 0.35088E+00_JPRB, 0.35350E+00_JPRB, 0.35819E+00_JPRB, 0.36705E+00_JPRB, 0.37545E+00_JPRB, 0.38477E+00_JPRB ,&
&  0.39877E+00_JPRB, 0.40065E+00_JPRB, 0.39944E+00_JPRB, 0.39887E+00_JPRB, 0.39384E+00_JPRB, 0.38741E+00_JPRB, 0.37858E+00_JPRB ,&
&  0.38158E+00_JPRB, 0.37959E+00_JPRB, 0.38817E+00_JPRB, 0.38800E+00_JPRB, 0.38662E+00_JPRB, 0.38825E+00_JPRB, 0.38124E+00_JPRB ,&
&  0.37711E+00_JPRB, 0.36745E+00_JPRB, 0.35991E+00_JPRB, 0.35123E+00_JPRB, 0.34191E+00_JPRB, 0.33647E+00_JPRB, 0.32923E+00_JPRB ,&
&  0.32407E+00_JPRB, 0.32047E+00_JPRB, 0.31561E+00_JPRB, 0.31274E+00_JPRB, 0.31010E+00_JPRB, 0.30727E+00_JPRB, 0.30579E+00_JPRB ,&
&  0.30466E+00_JPRB, 0.30427E+00_JPRB, 0.30452E+00_JPRB, 0.30480E+00_JPRB, 0.30543E+00_JPRB, 0.30651E+00_JPRB, 0.30728E+00_JPRB ,&
&  0.30817E+00_JPRB, 0.30855E+00_JPRB, 0.30819E+00_JPRB, 0.30784E+00_JPRB, 0.30652E+00_JPRB, 0.30570E+00_JPRB, 0.30452E+00_JPRB ,&
&  0.30375E+00_JPRB, 0.30408E+00_JPRB, 0.30434E+00_JPRB, 0.30508E+00_JPRB, 0.30579E+00_JPRB, 0.30752E+00_JPRB, 0.30910E+00_JPRB ,&
&  0.31082E+00_JPRB, 0.31307E+00_JPRB, 0.31483E+00_JPRB, 0.31721E+00_JPRB, 0.31996E+00_JPRB, 0.32162E+00_JPRB, 0.32484E+00_JPRB ,&
&  0.32790E+00_JPRB, 0.33185E+00_JPRB, 0.33906E+00_JPRB, 0.34679E+00_JPRB, 0.35720E+00_JPRB, 0.36888E+00_JPRB, 0.38316E+00_JPRB ,&
&  0.40121E+00_JPRB, 0.42510E+00_JPRB, 0.42572E+00_JPRB, 0.43638E+00_JPRB, 0.44282E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.43523E+00_JPRB, 0.44106E+00_JPRB, 0.44594E+00_JPRB, 0.45030E+00_JPRB, 0.45437E+00_JPRB, 0.45581E+00_JPRB, 0.45491E+00_JPRB ,&
&  0.44550E+00_JPRB, 0.43881E+00_JPRB, 0.43040E+00_JPRB, 0.42255E+00_JPRB, 0.41528E+00_JPRB, 0.40567E+00_JPRB, 0.39805E+00_JPRB ,&
&  0.39027E+00_JPRB, 0.38242E+00_JPRB, 0.37521E+00_JPRB, 0.36665E+00_JPRB, 0.35780E+00_JPRB, 0.34927E+00_JPRB, 0.34434E+00_JPRB ,&
&  0.34159E+00_JPRB, 0.34166E+00_JPRB, 0.34515E+00_JPRB, 0.34919E+00_JPRB, 0.35434E+00_JPRB, 0.36090E+00_JPRB, 0.36800E+00_JPRB ,&
&  0.37241E+00_JPRB, 0.37469E+00_JPRB, 0.37394E+00_JPRB, 0.37178E+00_JPRB, 0.36938E+00_JPRB, 0.36763E+00_JPRB, 0.36485E+00_JPRB ,&
&  0.36128E+00_JPRB, 0.35937E+00_JPRB, 0.35969E+00_JPRB, 0.36219E+00_JPRB, 0.36309E+00_JPRB, 0.36194E+00_JPRB, 0.35801E+00_JPRB ,&
&  0.35321E+00_JPRB, 0.34825E+00_JPRB, 0.34289E+00_JPRB, 0.33782E+00_JPRB, 0.33367E+00_JPRB, 0.32917E+00_JPRB, 0.32419E+00_JPRB ,&
&  0.32113E+00_JPRB, 0.31795E+00_JPRB, 0.31444E+00_JPRB, 0.31119E+00_JPRB, 0.30846E+00_JPRB, 0.30637E+00_JPRB, 0.30459E+00_JPRB ,&
&  0.30332E+00_JPRB, 0.30236E+00_JPRB, 0.30164E+00_JPRB, 0.30111E+00_JPRB, 0.30061E+00_JPRB, 0.29970E+00_JPRB, 0.29880E+00_JPRB ,&
&  0.29790E+00_JPRB, 0.29763E+00_JPRB, 0.29705E+00_JPRB, 0.29708E+00_JPRB, 0.29739E+00_JPRB, 0.29754E+00_JPRB, 0.29810E+00_JPRB ,&
&  0.29934E+00_JPRB, 0.30063E+00_JPRB, 0.30114E+00_JPRB, 0.30233E+00_JPRB, 0.30349E+00_JPRB, 0.30504E+00_JPRB, 0.30716E+00_JPRB ,&
&  0.30840E+00_JPRB, 0.30977E+00_JPRB, 0.31138E+00_JPRB, 0.31310E+00_JPRB, 0.31622E+00_JPRB, 0.31960E+00_JPRB, 0.32335E+00_JPRB ,&
&  0.32821E+00_JPRB, 0.33283E+00_JPRB, 0.33818E+00_JPRB, 0.34457E+00_JPRB, 0.35662E+00_JPRB, 0.36352E+00_JPRB, 0.37179E+00_JPRB ,&
&  0.37939E+00_JPRB, 0.38080E+00_JPRB, 0.41723E+00_JPRB, 0.42044E+00_JPRB, 0.42669E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.35955E+00_JPRB, 0.36642E+00_JPRB, 0.37047E+00_JPRB, 0.37441E+00_JPRB, 0.36799E+00_JPRB, 0.40771E+00_JPRB, 0.40680E+00_JPRB ,&
&  0.40628E+00_JPRB, 0.39379E+00_JPRB, 0.39282E+00_JPRB, 0.38861E+00_JPRB, 0.38324E+00_JPRB, 0.37710E+00_JPRB, 0.37176E+00_JPRB ,&
&  0.36679E+00_JPRB, 0.36111E+00_JPRB, 0.35659E+00_JPRB, 0.35240E+00_JPRB, 0.34897E+00_JPRB, 0.34596E+00_JPRB, 0.34385E+00_JPRB ,&
&  0.34341E+00_JPRB, 0.34516E+00_JPRB, 0.34877E+00_JPRB, 0.35035E+00_JPRB, 0.35069E+00_JPRB, 0.35097E+00_JPRB, 0.35131E+00_JPRB ,&
&  0.35103E+00_JPRB, 0.35058E+00_JPRB, 0.34891E+00_JPRB, 0.34597E+00_JPRB, 0.34392E+00_JPRB, 0.34266E+00_JPRB, 0.34179E+00_JPRB ,&
&  0.34103E+00_JPRB, 0.34044E+00_JPRB, 0.33979E+00_JPRB, 0.33956E+00_JPRB, 0.33886E+00_JPRB, 0.33759E+00_JPRB, 0.33541E+00_JPRB ,&
&  0.33312E+00_JPRB, 0.33066E+00_JPRB, 0.32755E+00_JPRB, 0.32506E+00_JPRB, 0.32262E+00_JPRB, 0.32015E+00_JPRB, 0.31765E+00_JPRB ,&
&  0.31625E+00_JPRB, 0.31488E+00_JPRB, 0.31313E+00_JPRB, 0.31093E+00_JPRB, 0.30930E+00_JPRB, 0.30759E+00_JPRB, 0.30611E+00_JPRB ,&
&  0.30519E+00_JPRB, 0.30418E+00_JPRB, 0.30316E+00_JPRB, 0.30211E+00_JPRB, 0.30137E+00_JPRB, 0.30047E+00_JPRB, 0.29961E+00_JPRB ,&
&  0.29884E+00_JPRB, 0.29792E+00_JPRB, 0.29768E+00_JPRB, 0.29757E+00_JPRB, 0.29764E+00_JPRB, 0.29765E+00_JPRB, 0.29798E+00_JPRB ,&
&  0.29850E+00_JPRB, 0.29930E+00_JPRB, 0.30030E+00_JPRB, 0.30121E+00_JPRB, 0.30208E+00_JPRB, 0.30298E+00_JPRB, 0.30435E+00_JPRB ,&
&  0.30579E+00_JPRB, 0.30728E+00_JPRB, 0.30882E+00_JPRB, 0.30998E+00_JPRB, 0.31280E+00_JPRB, 0.31544E+00_JPRB, 0.31823E+00_JPRB ,&
&  0.32144E+00_JPRB, 0.32530E+00_JPRB, 0.32953E+00_JPRB, 0.33413E+00_JPRB, 0.34036E+00_JPRB, 0.34384E+00_JPRB, 0.34626E+00_JPRB ,&
&  0.34892E+00_JPRB, 0.35369E+00_JPRB, 0.35471E+00_JPRB, 0.35572E+00_JPRB, 0.35737E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.33222E+00_JPRB, 0.33247E+00_JPRB, 0.33242E+00_JPRB, 0.33209E+00_JPRB, 0.32886E+00_JPRB, 0.32860E+00_JPRB, 0.32823E+00_JPRB ,&
&  0.32816E+00_JPRB, 0.32826E+00_JPRB, 0.32901E+00_JPRB, 0.32992E+00_JPRB, 0.33086E+00_JPRB, 0.33177E+00_JPRB, 0.33263E+00_JPRB ,&
&  0.33318E+00_JPRB, 0.33386E+00_JPRB, 0.33228E+00_JPRB, 0.33123E+00_JPRB, 0.33104E+00_JPRB, 0.33106E+00_JPRB, 0.33097E+00_JPRB ,&
&  0.33079E+00_JPRB, 0.33058E+00_JPRB, 0.32994E+00_JPRB, 0.32951E+00_JPRB, 0.32933E+00_JPRB, 0.32907E+00_JPRB, 0.32869E+00_JPRB ,&
&  0.32826E+00_JPRB, 0.32792E+00_JPRB, 0.32752E+00_JPRB, 0.32703E+00_JPRB, 0.32698E+00_JPRB, 0.32696E+00_JPRB, 0.32643E+00_JPRB ,&
&  0.32580E+00_JPRB, 0.32478E+00_JPRB, 0.32386E+00_JPRB, 0.32300E+00_JPRB, 0.32231E+00_JPRB, 0.32202E+00_JPRB, 0.32169E+00_JPRB ,&
&  0.32120E+00_JPRB, 0.32052E+00_JPRB, 0.31987E+00_JPRB, 0.31901E+00_JPRB, 0.31824E+00_JPRB, 0.31752E+00_JPRB, 0.31667E+00_JPRB ,&
&  0.31574E+00_JPRB, 0.31478E+00_JPRB, 0.31378E+00_JPRB, 0.31265E+00_JPRB, 0.31152E+00_JPRB, 0.31037E+00_JPRB, 0.30911E+00_JPRB ,&
&  0.30786E+00_JPRB, 0.30735E+00_JPRB, 0.30676E+00_JPRB, 0.30627E+00_JPRB, 0.30589E+00_JPRB, 0.30561E+00_JPRB, 0.30532E+00_JPRB ,&
&  0.30509E+00_JPRB, 0.30485E+00_JPRB, 0.30474E+00_JPRB, 0.30463E+00_JPRB, 0.30463E+00_JPRB, 0.30470E+00_JPRB, 0.30491E+00_JPRB ,&
&  0.30515E+00_JPRB, 0.30545E+00_JPRB, 0.30586E+00_JPRB, 0.30656E+00_JPRB, 0.30726E+00_JPRB, 0.30807E+00_JPRB, 0.30887E+00_JPRB ,&
&  0.30975E+00_JPRB, 0.31062E+00_JPRB, 0.31152E+00_JPRB, 0.31237E+00_JPRB, 0.31355E+00_JPRB, 0.31468E+00_JPRB, 0.31593E+00_JPRB ,&
&  0.31717E+00_JPRB, 0.31851E+00_JPRB, 0.31984E+00_JPRB, 0.32127E+00_JPRB, 0.32291E+00_JPRB, 0.32336E+00_JPRB, 0.32384E+00_JPRB ,&
&  0.32442E+00_JPRB, 0.32529E+00_JPRB, 0.32605E+00_JPRB, 0.32711E+00_JPRB, 0.32892E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB, 0.31598E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB, 0.74442E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.74310E+00_JPRB, 0.74394E+00_JPRB, 0.74467E+00_JPRB, 0.74508E+00_JPRB, 0.74532E+00_JPRB, 0.74548E+00_JPRB, 0.74554E+00_JPRB ,&
&  0.74538E+00_JPRB, 0.74559E+00_JPRB, 0.74601E+00_JPRB, 0.74644E+00_JPRB, 0.74680E+00_JPRB, 0.74703E+00_JPRB, 0.74697E+00_JPRB ,&
&  0.74739E+00_JPRB, 0.74977E+00_JPRB, 0.75119E+00_JPRB, 0.75180E+00_JPRB, 0.75231E+00_JPRB, 0.75269E+00_JPRB, 0.75286E+00_JPRB ,&
&  0.75311E+00_JPRB, 0.75336E+00_JPRB, 0.75361E+00_JPRB, 0.75377E+00_JPRB, 0.75397E+00_JPRB, 0.75422E+00_JPRB, 0.75452E+00_JPRB ,&
&  0.75506E+00_JPRB, 0.75630E+00_JPRB, 0.75800E+00_JPRB, 0.75988E+00_JPRB, 0.76215E+00_JPRB, 0.76518E+00_JPRB, 0.76769E+00_JPRB ,&
&  0.76947E+00_JPRB, 0.77036E+00_JPRB, 0.76929E+00_JPRB, 0.76668E+00_JPRB, 0.76318E+00_JPRB, 0.75948E+00_JPRB, 0.75589E+00_JPRB ,&
&  0.75181E+00_JPRB, 0.74809E+00_JPRB, 0.74478E+00_JPRB, 0.74190E+00_JPRB, 0.73921E+00_JPRB, 0.73688E+00_JPRB, 0.73471E+00_JPRB ,&
&  0.73307E+00_JPRB, 0.73144E+00_JPRB, 0.73004E+00_JPRB, 0.72867E+00_JPRB, 0.72726E+00_JPRB, 0.72543E+00_JPRB, 0.72350E+00_JPRB ,&
&  0.72212E+00_JPRB, 0.72132E+00_JPRB, 0.72081E+00_JPRB, 0.72024E+00_JPRB, 0.72000E+00_JPRB, 0.72058E+00_JPRB, 0.72150E+00_JPRB ,&
&  0.72234E+00_JPRB, 0.72348E+00_JPRB, 0.72483E+00_JPRB, 0.72572E+00_JPRB, 0.72630E+00_JPRB, 0.72644E+00_JPRB, 0.72632E+00_JPRB ,&
&  0.72608E+00_JPRB, 0.72637E+00_JPRB, 0.72680E+00_JPRB, 0.72772E+00_JPRB, 0.72866E+00_JPRB, 0.72983E+00_JPRB, 0.73116E+00_JPRB ,&
&  0.73219E+00_JPRB, 0.73334E+00_JPRB, 0.73431E+00_JPRB, 0.73556E+00_JPRB, 0.73666E+00_JPRB, 0.73747E+00_JPRB, 0.73800E+00_JPRB ,&
&  0.73841E+00_JPRB, 0.73862E+00_JPRB, 0.73886E+00_JPRB, 0.73893E+00_JPRB, 0.73917E+00_JPRB, 0.73946E+00_JPRB, 0.73975E+00_JPRB ,&
&  0.74002E+00_JPRB, 0.74050E+00_JPRB, 0.74088E+00_JPRB, 0.74167E+00_JPRB, 0.74236E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.74835E+00_JPRB, 0.74967E+00_JPRB, 0.75076E+00_JPRB, 0.75128E+00_JPRB, 0.75134E+00_JPRB, 0.75070E+00_JPRB, 0.74945E+00_JPRB ,&
&  0.74740E+00_JPRB, 0.74586E+00_JPRB, 0.74507E+00_JPRB, 0.74410E+00_JPRB, 0.74254E+00_JPRB, 0.74362E+00_JPRB, 0.74554E+00_JPRB ,&
&  0.74441E+00_JPRB, 0.74267E+00_JPRB, 0.74047E+00_JPRB, 0.73850E+00_JPRB, 0.73637E+00_JPRB, 0.73400E+00_JPRB, 0.73254E+00_JPRB ,&
&  0.73187E+00_JPRB, 0.73063E+00_JPRB, 0.72861E+00_JPRB, 0.72571E+00_JPRB, 0.72485E+00_JPRB, 0.72405E+00_JPRB, 0.72310E+00_JPRB ,&
&  0.72166E+00_JPRB, 0.71809E+00_JPRB, 0.71485E+00_JPRB, 0.71283E+00_JPRB, 0.71701E+00_JPRB, 0.72149E+00_JPRB, 0.72640E+00_JPRB ,&
&  0.73157E+00_JPRB, 0.73481E+00_JPRB, 0.73918E+00_JPRB, 0.74183E+00_JPRB, 0.74157E+00_JPRB, 0.74405E+00_JPRB, 0.74571E+00_JPRB ,&
&  0.74498E+00_JPRB, 0.74187E+00_JPRB, 0.73960E+00_JPRB, 0.73915E+00_JPRB, 0.73901E+00_JPRB, 0.73846E+00_JPRB, 0.73776E+00_JPRB ,&
&  0.73770E+00_JPRB, 0.73724E+00_JPRB, 0.73611E+00_JPRB, 0.73441E+00_JPRB, 0.73192E+00_JPRB, 0.72891E+00_JPRB, 0.72542E+00_JPRB ,&
&  0.72267E+00_JPRB, 0.71943E+00_JPRB, 0.71625E+00_JPRB, 0.71310E+00_JPRB, 0.71077E+00_JPRB, 0.70776E+00_JPRB, 0.70469E+00_JPRB ,&
&  0.70326E+00_JPRB, 0.70459E+00_JPRB, 0.70693E+00_JPRB, 0.70751E+00_JPRB, 0.70813E+00_JPRB, 0.70804E+00_JPRB, 0.70941E+00_JPRB ,&
&  0.71097E+00_JPRB, 0.71442E+00_JPRB, 0.71812E+00_JPRB, 0.72325E+00_JPRB, 0.72811E+00_JPRB, 0.73285E+00_JPRB, 0.73653E+00_JPRB ,&
&  0.73988E+00_JPRB, 0.74204E+00_JPRB, 0.74332E+00_JPRB, 0.74386E+00_JPRB, 0.74467E+00_JPRB, 0.74504E+00_JPRB, 0.74475E+00_JPRB ,&
&  0.74414E+00_JPRB, 0.74387E+00_JPRB, 0.74335E+00_JPRB, 0.74253E+00_JPRB, 0.74218E+00_JPRB, 0.74156E+00_JPRB, 0.74140E+00_JPRB ,&
&  0.74132E+00_JPRB, 0.74207E+00_JPRB, 0.74294E+00_JPRB, 0.74456E+00_JPRB, 0.74649E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.73842E+00_JPRB, 0.74156E+00_JPRB, 0.74402E+00_JPRB, 0.74482E+00_JPRB, 0.74513E+00_JPRB, 0.74548E+00_JPRB, 0.74543E+00_JPRB ,&
&  0.74526E+00_JPRB, 0.74475E+00_JPRB, 0.74503E+00_JPRB, 0.74488E+00_JPRB, 0.74454E+00_JPRB, 0.74869E+00_JPRB, 0.74946E+00_JPRB ,&
&  0.74826E+00_JPRB, 0.74572E+00_JPRB, 0.74324E+00_JPRB, 0.73957E+00_JPRB, 0.73476E+00_JPRB, 0.72846E+00_JPRB, 0.71710E+00_JPRB ,&
&  0.71020E+00_JPRB, 0.70315E+00_JPRB, 0.69518E+00_JPRB, 0.69350E+00_JPRB, 0.69156E+00_JPRB, 0.66526E+00_JPRB, 0.66521E+00_JPRB ,&
&  0.69117E+00_JPRB, 0.70173E+00_JPRB, 0.70463E+00_JPRB, 0.70871E+00_JPRB, 0.67794E+00_JPRB, 0.64376E+00_JPRB, 0.62146E+00_JPRB ,&
&  0.58591E+00_JPRB, 0.61650E+00_JPRB, 0.63470E+00_JPRB, 0.63926E+00_JPRB, 0.63621E+00_JPRB, 0.63020E+00_JPRB, 0.65140E+00_JPRB ,&
&  0.70318E+00_JPRB, 0.71250E+00_JPRB, 0.72529E+00_JPRB, 0.73928E+00_JPRB, 0.75322E+00_JPRB, 0.76269E+00_JPRB, 0.77365E+00_JPRB ,&
&  0.77947E+00_JPRB, 0.78334E+00_JPRB, 0.78183E+00_JPRB, 0.77806E+00_JPRB, 0.76861E+00_JPRB, 0.76096E+00_JPRB, 0.75344E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74534E+00_JPRB, 0.74167E+00_JPRB, 0.73571E+00_JPRB, 0.72885E+00_JPRB, 0.72121E+00_JPRB, 0.71353E+00_JPRB ,&
&  0.70600E+00_JPRB, 0.69782E+00_JPRB, 0.69283E+00_JPRB, 0.68916E+00_JPRB, 0.69087E+00_JPRB, 0.69108E+00_JPRB, 0.69482E+00_JPRB ,&
&  0.69825E+00_JPRB, 0.70358E+00_JPRB, 0.70942E+00_JPRB, 0.71733E+00_JPRB, 0.72360E+00_JPRB, 0.72851E+00_JPRB, 0.73130E+00_JPRB ,&
&  0.73356E+00_JPRB, 0.73446E+00_JPRB, 0.73468E+00_JPRB, 0.73393E+00_JPRB, 0.73378E+00_JPRB, 0.73386E+00_JPRB, 0.73344E+00_JPRB ,&
&  0.73355E+00_JPRB, 0.73374E+00_JPRB, 0.73424E+00_JPRB, 0.73432E+00_JPRB, 0.73406E+00_JPRB, 0.73222E+00_JPRB, 0.73009E+00_JPRB ,&
&  0.72792E+00_JPRB, 0.72790E+00_JPRB, 0.72875E+00_JPRB, 0.73135E+00_JPRB, 0.73421E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.74937E+00_JPRB, 0.74942E+00_JPRB, 0.75025E+00_JPRB, 0.75075E+00_JPRB, 0.75100E+00_JPRB, 0.75090E+00_JPRB, 0.75055E+00_JPRB ,&
&  0.75068E+00_JPRB, 0.75126E+00_JPRB, 0.75123E+00_JPRB, 0.75018E+00_JPRB, 0.75050E+00_JPRB, 0.75143E+00_JPRB, 0.75149E+00_JPRB ,&
&  0.74869E+00_JPRB, 0.74488E+00_JPRB, 0.74153E+00_JPRB, 0.73667E+00_JPRB, 0.73196E+00_JPRB, 0.72765E+00_JPRB, 0.71775E+00_JPRB ,&
&  0.71129E+00_JPRB, 0.70131E+00_JPRB, 0.66466E+00_JPRB, 0.66872E+00_JPRB, 0.66190E+00_JPRB, 0.66088E+00_JPRB, 0.66501E+00_JPRB ,&
&  0.65388E+00_JPRB, 0.65226E+00_JPRB, 0.63782E+00_JPRB, 0.61607E+00_JPRB, 0.60465E+00_JPRB, 0.58519E+00_JPRB, 0.58577E+00_JPRB ,&
&  0.58202E+00_JPRB, 0.58805E+00_JPRB, 0.60155E+00_JPRB, 0.60550E+00_JPRB, 0.60150E+00_JPRB, 0.59865E+00_JPRB, 0.60899E+00_JPRB ,&
&  0.63760E+00_JPRB, 0.68012E+00_JPRB, 0.73582E+00_JPRB, 0.76546E+00_JPRB, 0.79482E+00_JPRB, 0.81916E+00_JPRB, 0.84186E+00_JPRB ,&
&  0.85878E+00_JPRB, 0.87224E+00_JPRB, 0.87410E+00_JPRB, 0.87312E+00_JPRB, 0.86200E+00_JPRB, 0.84407E+00_JPRB, 0.82110E+00_JPRB ,&
&  0.79870E+00_JPRB, 0.78244E+00_JPRB, 0.77047E+00_JPRB, 0.76439E+00_JPRB, 0.76200E+00_JPRB, 0.76002E+00_JPRB, 0.75927E+00_JPRB ,&
&  0.75449E+00_JPRB, 0.73903E+00_JPRB, 0.72646E+00_JPRB, 0.71298E+00_JPRB, 0.70943E+00_JPRB, 0.70791E+00_JPRB, 0.71248E+00_JPRB ,&
&  0.71791E+00_JPRB, 0.72893E+00_JPRB, 0.74153E+00_JPRB, 0.74553E+00_JPRB, 0.75199E+00_JPRB, 0.76001E+00_JPRB, 0.75892E+00_JPRB ,&
&  0.75792E+00_JPRB, 0.75525E+00_JPRB, 0.75052E+00_JPRB, 0.74497E+00_JPRB, 0.73896E+00_JPRB, 0.73349E+00_JPRB, 0.72925E+00_JPRB ,&
&  0.72795E+00_JPRB, 0.72920E+00_JPRB, 0.73476E+00_JPRB, 0.74103E+00_JPRB, 0.74832E+00_JPRB, 0.75252E+00_JPRB, 0.75256E+00_JPRB ,&
&  0.74860E+00_JPRB, 0.74644E+00_JPRB, 0.74492E+00_JPRB, 0.74691E+00_JPRB, 0.74860E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.76938E+00_JPRB, 0.77072E+00_JPRB, 0.77510E+00_JPRB, 0.78056E+00_JPRB, 0.78525E+00_JPRB, 0.78645E+00_JPRB, 0.77872E+00_JPRB ,&
&  0.77184E+00_JPRB, 0.76512E+00_JPRB, 0.75983E+00_JPRB, 0.75618E+00_JPRB, 0.75474E+00_JPRB, 0.75136E+00_JPRB, 0.74744E+00_JPRB ,&
&  0.74415E+00_JPRB, 0.74287E+00_JPRB, 0.74307E+00_JPRB, 0.74291E+00_JPRB, 0.74480E+00_JPRB, 0.74410E+00_JPRB, 0.74100E+00_JPRB ,&
&  0.73794E+00_JPRB, 0.72353E+00_JPRB, 0.69455E+00_JPRB, 0.68553E+00_JPRB, 0.67502E+00_JPRB, 0.67595E+00_JPRB, 0.67615E+00_JPRB ,&
&  0.67112E+00_JPRB, 0.60409E+00_JPRB, 0.61417E+00_JPRB, 0.59759E+00_JPRB, 0.58786E+00_JPRB, 0.58795E+00_JPRB, 0.57279E+00_JPRB ,&
&  0.56309E+00_JPRB, 0.55043E+00_JPRB, 0.55006E+00_JPRB, 0.55704E+00_JPRB, 0.55526E+00_JPRB, 0.56581E+00_JPRB, 0.58061E+00_JPRB ,&
&  0.60601E+00_JPRB, 0.72213E+00_JPRB, 0.77519E+00_JPRB, 0.80664E+00_JPRB, 0.83326E+00_JPRB, 0.86374E+00_JPRB, 0.89400E+00_JPRB ,&
&  0.91805E+00_JPRB, 0.94731E+00_JPRB, 0.96111E+00_JPRB, 0.95776E+00_JPRB, 0.96806E+00_JPRB, 0.96053E+00_JPRB, 0.93838E+00_JPRB ,&
&  0.90351E+00_JPRB, 0.85625E+00_JPRB, 0.82358E+00_JPRB, 0.80376E+00_JPRB, 0.78921E+00_JPRB, 0.77850E+00_JPRB, 0.77219E+00_JPRB ,&
&  0.77735E+00_JPRB, 0.77263E+00_JPRB, 0.76500E+00_JPRB, 0.74921E+00_JPRB, 0.74076E+00_JPRB, 0.73610E+00_JPRB, 0.73815E+00_JPRB ,&
&  0.74323E+00_JPRB, 0.75498E+00_JPRB, 0.76942E+00_JPRB, 0.78049E+00_JPRB, 0.78605E+00_JPRB, 0.78509E+00_JPRB, 0.78922E+00_JPRB ,&
&  0.78778E+00_JPRB, 0.78302E+00_JPRB, 0.77775E+00_JPRB, 0.77236E+00_JPRB, 0.76722E+00_JPRB, 0.76423E+00_JPRB, 0.76715E+00_JPRB ,&
&  0.77111E+00_JPRB, 0.77342E+00_JPRB, 0.77389E+00_JPRB, 0.77620E+00_JPRB, 0.78074E+00_JPRB, 0.78469E+00_JPRB, 0.78602E+00_JPRB ,&
&  0.77995E+00_JPRB, 0.77365E+00_JPRB, 0.76875E+00_JPRB, 0.76917E+00_JPRB, 0.76942E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.76284E+00_JPRB, 0.77251E+00_JPRB, 0.79068E+00_JPRB, 0.80584E+00_JPRB, 0.81563E+00_JPRB, 0.81469E+00_JPRB, 0.80245E+00_JPRB ,&
&  0.79080E+00_JPRB, 0.78226E+00_JPRB, 0.77347E+00_JPRB, 0.76539E+00_JPRB, 0.75631E+00_JPRB, 0.74927E+00_JPRB, 0.74436E+00_JPRB ,&
&  0.74174E+00_JPRB, 0.74239E+00_JPRB, 0.74327E+00_JPRB, 0.74402E+00_JPRB, 0.74612E+00_JPRB, 0.74902E+00_JPRB, 0.74902E+00_JPRB ,&
&  0.74821E+00_JPRB, 0.73652E+00_JPRB, 0.71131E+00_JPRB, 0.69298E+00_JPRB, 0.68689E+00_JPRB, 0.67555E+00_JPRB, 0.68082E+00_JPRB ,&
&  0.68083E+00_JPRB, 0.67939E+00_JPRB, 0.67185E+00_JPRB, 0.66787E+00_JPRB, 0.64242E+00_JPRB, 0.55391E+00_JPRB, 0.55864E+00_JPRB ,&
&  0.54480E+00_JPRB, 0.52586E+00_JPRB, 0.52217E+00_JPRB, 0.52131E+00_JPRB, 0.52260E+00_JPRB, 0.54822E+00_JPRB, 0.57629E+00_JPRB ,&
&  0.60239E+00_JPRB, 0.73226E+00_JPRB, 0.76851E+00_JPRB, 0.80778E+00_JPRB, 0.84304E+00_JPRB, 0.88585E+00_JPRB, 0.93090E+00_JPRB ,&
&  0.96049E+00_JPRB, 0.99241E+00_JPRB, 0.10185E+01_JPRB, 0.10388E+01_JPRB, 0.10483E+01_JPRB, 0.10554E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.96691E+00_JPRB, 0.91262E+00_JPRB, 0.88082E+00_JPRB, 0.85050E+00_JPRB, 0.83019E+00_JPRB, 0.81458E+00_JPRB ,&
&  0.81102E+00_JPRB, 0.79470E+00_JPRB, 0.79353E+00_JPRB, 0.78353E+00_JPRB, 0.77797E+00_JPRB, 0.77358E+00_JPRB, 0.76814E+00_JPRB ,&
&  0.75983E+00_JPRB, 0.76352E+00_JPRB, 0.78096E+00_JPRB, 0.78967E+00_JPRB, 0.79633E+00_JPRB, 0.79113E+00_JPRB, 0.78140E+00_JPRB ,&
&  0.76502E+00_JPRB, 0.76219E+00_JPRB, 0.75479E+00_JPRB, 0.75305E+00_JPRB, 0.75456E+00_JPRB, 0.76612E+00_JPRB, 0.78109E+00_JPRB ,&
&  0.79055E+00_JPRB, 0.79300E+00_JPRB, 0.79204E+00_JPRB, 0.79220E+00_JPRB, 0.79047E+00_JPRB, 0.78677E+00_JPRB, 0.78010E+00_JPRB ,&
&  0.77369E+00_JPRB, 0.77056E+00_JPRB, 0.76413E+00_JPRB, 0.75996E+00_JPRB, 0.75962E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.75308E+00_JPRB, 0.77738E+00_JPRB, 0.80625E+00_JPRB, 0.82524E+00_JPRB, 0.83280E+00_JPRB, 0.81966E+00_JPRB, 0.80091E+00_JPRB ,&
&  0.78624E+00_JPRB, 0.77567E+00_JPRB, 0.76693E+00_JPRB, 0.76097E+00_JPRB, 0.75432E+00_JPRB, 0.74778E+00_JPRB, 0.74257E+00_JPRB ,&
&  0.73912E+00_JPRB, 0.72748E+00_JPRB, 0.72444E+00_JPRB, 0.72894E+00_JPRB, 0.73185E+00_JPRB, 0.73804E+00_JPRB, 0.74340E+00_JPRB ,&
&  0.74083E+00_JPRB, 0.73182E+00_JPRB, 0.72378E+00_JPRB, 0.70448E+00_JPRB, 0.68570E+00_JPRB, 0.68064E+00_JPRB, 0.68832E+00_JPRB ,&
&  0.67300E+00_JPRB, 0.64722E+00_JPRB, 0.67148E+00_JPRB, 0.68913E+00_JPRB, 0.65131E+00_JPRB, 0.61339E+00_JPRB, 0.56544E+00_JPRB ,&
&  0.55027E+00_JPRB, 0.52070E+00_JPRB, 0.51313E+00_JPRB, 0.51158E+00_JPRB, 0.52103E+00_JPRB, 0.54332E+00_JPRB, 0.58822E+00_JPRB ,&
&  0.64046E+00_JPRB, 0.73361E+00_JPRB, 0.76690E+00_JPRB, 0.81598E+00_JPRB, 0.85758E+00_JPRB, 0.89406E+00_JPRB, 0.92072E+00_JPRB ,&
&  0.96331E+00_JPRB, 0.10241E+01_JPRB, 0.10673E+01_JPRB, 0.10860E+01_JPRB, 0.11100E+01_JPRB, 0.11426E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12829E+01_JPRB, 0.11223E+01_JPRB, 0.10283E+01_JPRB, 0.96752E+00_JPRB, 0.92822E+00_JPRB, 0.91529E+00_JPRB, 0.86788E+00_JPRB ,&
&  0.85248E+00_JPRB, 0.84826E+00_JPRB, 0.83669E+00_JPRB, 0.83190E+00_JPRB, 0.82697E+00_JPRB, 0.82145E+00_JPRB, 0.81165E+00_JPRB ,&
&  0.79882E+00_JPRB, 0.80540E+00_JPRB, 0.88026E+00_JPRB, 0.89738E+00_JPRB, 0.88187E+00_JPRB, 0.83191E+00_JPRB, 0.77842E+00_JPRB ,&
&  0.74150E+00_JPRB, 0.72699E+00_JPRB, 0.72171E+00_JPRB, 0.72001E+00_JPRB, 0.72874E+00_JPRB, 0.74103E+00_JPRB, 0.75999E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.77000E+00_JPRB, 0.78038E+00_JPRB, 0.78608E+00_JPRB, 0.78551E+00_JPRB, 0.77890E+00_JPRB, 0.77399E+00_JPRB ,&
&  0.76985E+00_JPRB, 0.76529E+00_JPRB, 0.75727E+00_JPRB, 0.74807E+00_JPRB, 0.74571E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.73200E+00_JPRB, 0.77252E+00_JPRB, 0.82051E+00_JPRB, 0.85056E+00_JPRB, 0.85045E+00_JPRB, 0.82313E+00_JPRB, 0.79333E+00_JPRB ,&
&  0.77345E+00_JPRB, 0.76418E+00_JPRB, 0.75683E+00_JPRB, 0.75118E+00_JPRB, 0.74637E+00_JPRB, 0.74133E+00_JPRB, 0.73626E+00_JPRB ,&
&  0.73239E+00_JPRB, 0.72867E+00_JPRB, 0.72209E+00_JPRB, 0.71332E+00_JPRB, 0.70921E+00_JPRB, 0.71251E+00_JPRB, 0.71995E+00_JPRB ,&
&  0.72716E+00_JPRB, 0.72661E+00_JPRB, 0.72587E+00_JPRB, 0.71498E+00_JPRB, 0.70037E+00_JPRB, 0.68345E+00_JPRB, 0.67133E+00_JPRB ,&
&  0.66659E+00_JPRB, 0.67047E+00_JPRB, 0.66388E+00_JPRB, 0.68640E+00_JPRB, 0.67832E+00_JPRB, 0.63499E+00_JPRB, 0.59009E+00_JPRB ,&
&  0.56220E+00_JPRB, 0.52921E+00_JPRB, 0.51607E+00_JPRB, 0.52117E+00_JPRB, 0.55417E+00_JPRB, 0.59359E+00_JPRB, 0.65040E+00_JPRB ,&
&  0.74902E+00_JPRB, 0.81691E+00_JPRB, 0.86450E+00_JPRB, 0.90484E+00_JPRB, 0.93643E+00_JPRB, 0.97197E+00_JPRB, 0.10146E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.11263E+01_JPRB, 0.11650E+01_JPRB, 0.11528E+01_JPRB, 0.11580E+01_JPRB, 0.11086E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.16327E+01_JPRB, 0.13914E+01_JPRB, 0.12187E+01_JPRB, 0.11263E+01_JPRB, 0.10720E+01_JPRB, 0.10251E+01_JPRB, 0.98135E+00_JPRB ,&
&  0.93801E+00_JPRB, 0.91259E+00_JPRB, 0.89956E+00_JPRB, 0.85926E+00_JPRB, 0.81969E+00_JPRB, 0.81740E+00_JPRB, 0.86498E+00_JPRB ,&
&  0.98439E+00_JPRB, 0.11573E+01_JPRB, 0.13673E+01_JPRB, 0.98828E+00_JPRB, 0.86986E+00_JPRB, 0.78467E+00_JPRB, 0.74367E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73009E+00_JPRB, 0.73252E+00_JPRB, 0.72954E+00_JPRB, 0.72705E+00_JPRB, 0.72941E+00_JPRB, 0.72973E+00_JPRB ,&
&  0.72684E+00_JPRB, 0.73709E+00_JPRB, 0.75213E+00_JPRB, 0.76413E+00_JPRB, 0.77590E+00_JPRB, 0.77788E+00_JPRB, 0.77663E+00_JPRB ,&
&  0.77381E+00_JPRB, 0.76420E+00_JPRB, 0.75193E+00_JPRB, 0.73108E+00_JPRB, 0.72121E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.72597E+00_JPRB, 0.77232E+00_JPRB, 0.83557E+00_JPRB, 0.87205E+00_JPRB, 0.86903E+00_JPRB, 0.82628E+00_JPRB, 0.77197E+00_JPRB ,&
&  0.73410E+00_JPRB, 0.71108E+00_JPRB, 0.70347E+00_JPRB, 0.70033E+00_JPRB, 0.70035E+00_JPRB, 0.70103E+00_JPRB, 0.70014E+00_JPRB ,&
&  0.69792E+00_JPRB, 0.69637E+00_JPRB, 0.69417E+00_JPRB, 0.69285E+00_JPRB, 0.69061E+00_JPRB, 0.69021E+00_JPRB, 0.69357E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.70175E+00_JPRB, 0.70176E+00_JPRB, 0.69991E+00_JPRB, 0.70149E+00_JPRB, 0.70685E+00_JPRB, 0.70571E+00_JPRB ,&
&  0.68683E+00_JPRB, 0.67076E+00_JPRB, 0.67727E+00_JPRB, 0.69468E+00_JPRB, 0.69115E+00_JPRB, 0.65639E+00_JPRB, 0.60569E+00_JPRB ,&
&  0.57089E+00_JPRB, 0.54733E+00_JPRB, 0.56512E+00_JPRB, 0.61170E+00_JPRB, 0.67078E+00_JPRB, 0.79509E+00_JPRB, 0.88491E+00_JPRB ,&
&  0.95978E+00_JPRB, 0.10019E+01_JPRB, 0.10284E+01_JPRB, 0.10570E+01_JPRB, 0.11129E+01_JPRB, 0.11728E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.13135E+01_JPRB, 0.13563E+01_JPRB, 0.10982E+01_JPRB, 0.10858E+01_JPRB, 0.12039E+01_JPRB, 0.13382E+01_JPRB ,&
&  0.14099E+01_JPRB, 0.13304E+01_JPRB, 0.12835E+01_JPRB, 0.13005E+01_JPRB, 0.12183E+01_JPRB, 0.11541E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10489E+01_JPRB, 0.97671E+00_JPRB, 0.89916E+00_JPRB, 0.88989E+00_JPRB, 0.89449E+00_JPRB, 0.91206E+00_JPRB ,&
&  0.95536E+00_JPRB, 0.98524E+00_JPRB, 0.90931E+00_JPRB, 0.74325E+00_JPRB, 0.70120E+00_JPRB, 0.68615E+00_JPRB, 0.68986E+00_JPRB ,&
&  0.69784E+00_JPRB, 0.70685E+00_JPRB, 0.71842E+00_JPRB, 0.72252E+00_JPRB, 0.72334E+00_JPRB, 0.71565E+00_JPRB, 0.70591E+00_JPRB ,&
&  0.69573E+00_JPRB, 0.69673E+00_JPRB, 0.70714E+00_JPRB, 0.70865E+00_JPRB, 0.72608E+00_JPRB, 0.74217E+00_JPRB, 0.75793E+00_JPRB ,&
&  0.75551E+00_JPRB, 0.74799E+00_JPRB, 0.73893E+00_JPRB, 0.71863E+00_JPRB, 0.70594E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.76862E+00_JPRB, 0.81479E+00_JPRB, 0.85517E+00_JPRB, 0.87698E+00_JPRB, 0.86935E+00_JPRB, 0.83779E+00_JPRB, 0.76831E+00_JPRB ,&
&  0.74112E+00_JPRB, 0.70276E+00_JPRB, 0.68732E+00_JPRB, 0.67485E+00_JPRB, 0.66488E+00_JPRB, 0.65713E+00_JPRB, 0.65570E+00_JPRB ,&
&  0.66002E+00_JPRB, 0.66458E+00_JPRB, 0.66793E+00_JPRB, 0.67232E+00_JPRB, 0.67412E+00_JPRB, 0.67764E+00_JPRB, 0.68227E+00_JPRB ,&
&  0.68525E+00_JPRB, 0.68744E+00_JPRB, 0.69129E+00_JPRB, 0.69547E+00_JPRB, 0.70103E+00_JPRB, 0.70606E+00_JPRB, 0.70344E+00_JPRB ,&
&  0.68987E+00_JPRB, 0.67592E+00_JPRB, 0.68471E+00_JPRB, 0.70161E+00_JPRB, 0.69889E+00_JPRB, 0.67620E+00_JPRB, 0.62248E+00_JPRB ,&
&  0.58672E+00_JPRB, 0.59866E+00_JPRB, 0.68860E+00_JPRB, 0.83585E+00_JPRB, 0.90188E+00_JPRB, 0.96660E+00_JPRB, 0.10039E+01_JPRB ,&
&  0.95903E+00_JPRB, 0.96640E+00_JPRB, 0.10308E+01_JPRB, 0.12142E+01_JPRB, 0.12551E+01_JPRB, 0.13136E+01_JPRB, 0.13899E+01_JPRB ,&
&  0.15117E+01_JPRB, 0.15535E+01_JPRB, 0.15532E+01_JPRB, 0.13547E+01_JPRB, 0.15605E+01_JPRB, 0.19017E+01_JPRB, 0.17120E+01_JPRB ,&
&  0.16594E+01_JPRB, 0.16070E+01_JPRB, 0.16354E+01_JPRB, 0.15270E+01_JPRB, 0.13684E+01_JPRB, 0.12725E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.11964E+01_JPRB, 0.11592E+01_JPRB, 0.10307E+01_JPRB, 0.98580E+00_JPRB, 0.99168E+00_JPRB, 0.96661E+00_JPRB, 0.94301E+00_JPRB ,&
&  0.91429E+00_JPRB, 0.85562E+00_JPRB, 0.80635E+00_JPRB, 0.75299E+00_JPRB, 0.71775E+00_JPRB, 0.70995E+00_JPRB, 0.71295E+00_JPRB ,&
&  0.71679E+00_JPRB, 0.72077E+00_JPRB, 0.73048E+00_JPRB, 0.74246E+00_JPRB, 0.74368E+00_JPRB, 0.72254E+00_JPRB, 0.71720E+00_JPRB ,&
&  0.70939E+00_JPRB, 0.69493E+00_JPRB, 0.68377E+00_JPRB, 0.69051E+00_JPRB, 0.69798E+00_JPRB, 0.71346E+00_JPRB, 0.73166E+00_JPRB ,&
&  0.74602E+00_JPRB, 0.75386E+00_JPRB, 0.74981E+00_JPRB, 0.75219E+00_JPRB, 0.74848E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.81521E+00_JPRB, 0.83493E+00_JPRB, 0.86272E+00_JPRB, 0.88338E+00_JPRB, 0.89369E+00_JPRB, 0.83589E+00_JPRB, 0.78049E+00_JPRB ,&
&  0.79516E+00_JPRB, 0.72412E+00_JPRB, 0.68631E+00_JPRB, 0.67449E+00_JPRB, 0.67087E+00_JPRB, 0.65278E+00_JPRB, 0.64054E+00_JPRB ,&
&  0.63715E+00_JPRB, 0.64615E+00_JPRB, 0.65952E+00_JPRB, 0.66880E+00_JPRB, 0.67695E+00_JPRB, 0.68397E+00_JPRB, 0.68739E+00_JPRB ,&
&  0.69046E+00_JPRB, 0.69402E+00_JPRB, 0.69758E+00_JPRB, 0.70008E+00_JPRB, 0.70014E+00_JPRB, 0.70229E+00_JPRB, 0.69993E+00_JPRB ,&
&  0.68873E+00_JPRB, 0.68097E+00_JPRB, 0.68985E+00_JPRB, 0.70981E+00_JPRB, 0.71759E+00_JPRB, 0.71591E+00_JPRB, 0.68977E+00_JPRB ,&
&  0.62911E+00_JPRB, 0.68477E+00_JPRB, 0.88645E+00_JPRB, 0.95568E+00_JPRB, 0.10029E+01_JPRB, 0.10634E+01_JPRB, 0.11158E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.11939E+01_JPRB, 0.12264E+01_JPRB, 0.12729E+01_JPRB, 0.13349E+01_JPRB, 0.14121E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.18308E+01_JPRB, 0.14945E+01_JPRB, 0.14026E+01_JPRB, 0.14915E+01_JPRB, 0.20700E+01_JPRB, 0.21939E+01_JPRB, 0.19680E+01_JPRB ,&
&  0.18870E+01_JPRB, 0.17970E+01_JPRB, 0.17186E+01_JPRB, 0.15806E+01_JPRB, 0.14753E+01_JPRB, 0.14374E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.13166E+01_JPRB, 0.12688E+01_JPRB, 0.11241E+01_JPRB, 0.10924E+01_JPRB, 0.10892E+01_JPRB, 0.10380E+01_JPRB, 0.96371E+00_JPRB ,&
&  0.90017E+00_JPRB, 0.84954E+00_JPRB, 0.81271E+00_JPRB, 0.78526E+00_JPRB, 0.76358E+00_JPRB, 0.75088E+00_JPRB, 0.74200E+00_JPRB ,&
&  0.73811E+00_JPRB, 0.73873E+00_JPRB, 0.73469E+00_JPRB, 0.74685E+00_JPRB, 0.75372E+00_JPRB, 0.73956E+00_JPRB, 0.74863E+00_JPRB ,&
&  0.75096E+00_JPRB, 0.74127E+00_JPRB, 0.72031E+00_JPRB, 0.72739E+00_JPRB, 0.74655E+00_JPRB, 0.77993E+00_JPRB, 0.81020E+00_JPRB ,&
&  0.85050E+00_JPRB, 0.84358E+00_JPRB, 0.80788E+00_JPRB, 0.79915E+00_JPRB, 0.79803E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.85226E+00_JPRB, 0.86628E+00_JPRB, 0.87703E+00_JPRB, 0.89510E+00_JPRB, 0.90987E+00_JPRB, 0.86189E+00_JPRB, 0.82522E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.79449E+00_JPRB, 0.83983E+00_JPRB, 0.74943E+00_JPRB, 0.70382E+00_JPRB, 0.67896E+00_JPRB, 0.66115E+00_JPRB ,&
&  0.64710E+00_JPRB, 0.65989E+00_JPRB, 0.67755E+00_JPRB, 0.69402E+00_JPRB, 0.72063E+00_JPRB, 0.71413E+00_JPRB, 0.70727E+00_JPRB ,&
&  0.70181E+00_JPRB, 0.70002E+00_JPRB, 0.70946E+00_JPRB, 0.70810E+00_JPRB, 0.70515E+00_JPRB, 0.70175E+00_JPRB, 0.68961E+00_JPRB ,&
&  0.66493E+00_JPRB, 0.66329E+00_JPRB, 0.68588E+00_JPRB, 0.70795E+00_JPRB, 0.73051E+00_JPRB, 0.74088E+00_JPRB, 0.75386E+00_JPRB ,&
&  0.77136E+00_JPRB, 0.79708E+00_JPRB, 0.93171E+00_JPRB, 0.97481E+00_JPRB, 0.10224E+01_JPRB, 0.10572E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.11833E+01_JPRB, 0.12086E+01_JPRB, 0.12581E+01_JPRB, 0.13524E+01_JPRB, 0.14860E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.20624E+01_JPRB, 0.15873E+01_JPRB, 0.18800E+01_JPRB, 0.18830E+01_JPRB, 0.21509E+01_JPRB, 0.23525E+01_JPRB, 0.23550E+01_JPRB ,&
&  0.25402E+01_JPRB, 0.22271E+01_JPRB, 0.19999E+01_JPRB, 0.17670E+01_JPRB, 0.15875E+01_JPRB, 0.15481E+01_JPRB, 0.15165E+01_JPRB ,&
&  0.15448E+01_JPRB, 0.14985E+01_JPRB, 0.12889E+01_JPRB, 0.11873E+01_JPRB, 0.11524E+01_JPRB, 0.11305E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10082E+01_JPRB, 0.97682E+00_JPRB, 0.93151E+00_JPRB, 0.88191E+00_JPRB, 0.84547E+00_JPRB, 0.82587E+00_JPRB, 0.79900E+00_JPRB ,&
&  0.79428E+00_JPRB, 0.78160E+00_JPRB, 0.74124E+00_JPRB, 0.74213E+00_JPRB, 0.74404E+00_JPRB, 0.71683E+00_JPRB, 0.72768E+00_JPRB ,&
&  0.75451E+00_JPRB, 0.75872E+00_JPRB, 0.80774E+00_JPRB, 0.87466E+00_JPRB, 0.89583E+00_JPRB, 0.90145E+00_JPRB, 0.98784E+00_JPRB ,&
&  0.91720E+00_JPRB, 0.91573E+00_JPRB, 0.91009E+00_JPRB, 0.87103E+00_JPRB, 0.85100E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.90702E+00_JPRB, 0.89913E+00_JPRB, 0.89454E+00_JPRB, 0.90771E+00_JPRB, 0.92371E+00_JPRB, 0.92977E+00_JPRB, 0.89784E+00_JPRB ,&
&  0.94984E+00_JPRB, 0.91755E+00_JPRB, 0.89934E+00_JPRB, 0.89052E+00_JPRB, 0.87233E+00_JPRB, 0.77757E+00_JPRB, 0.67973E+00_JPRB ,&
&  0.65802E+00_JPRB, 0.66498E+00_JPRB, 0.70042E+00_JPRB, 0.71681E+00_JPRB, 0.73919E+00_JPRB, 0.73800E+00_JPRB, 0.72877E+00_JPRB ,&
&  0.72331E+00_JPRB, 0.71615E+00_JPRB, 0.72453E+00_JPRB, 0.72501E+00_JPRB, 0.71618E+00_JPRB, 0.71625E+00_JPRB, 0.71423E+00_JPRB ,&
&  0.67803E+00_JPRB, 0.65944E+00_JPRB, 0.66508E+00_JPRB, 0.66817E+00_JPRB, 0.73030E+00_JPRB, 0.75850E+00_JPRB, 0.78255E+00_JPRB ,&
&  0.80939E+00_JPRB, 0.84797E+00_JPRB, 0.91248E+00_JPRB, 0.96538E+00_JPRB, 0.10012E+01_JPRB, 0.10276E+01_JPRB, 0.10678E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.11454E+01_JPRB, 0.12014E+01_JPRB, 0.12972E+01_JPRB, 0.14436E+01_JPRB, 0.14718E+01_JPRB, 0.18635E+01_JPRB ,&
&  0.20176E+01_JPRB, 0.22444E+01_JPRB, 0.24906E+01_JPRB, 0.23337E+01_JPRB, 0.25447E+01_JPRB, 0.26686E+01_JPRB, 0.25637E+01_JPRB ,&
&  0.24846E+01_JPRB, 0.22005E+01_JPRB, 0.21383E+01_JPRB, 0.20948E+01_JPRB, 0.20898E+01_JPRB, 0.19485E+01_JPRB, 0.18044E+01_JPRB ,&
&  0.18920E+01_JPRB, 0.21300E+01_JPRB, 0.16942E+01_JPRB, 0.15327E+01_JPRB, 0.14017E+01_JPRB, 0.13613E+01_JPRB, 0.12891E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.12681E+01_JPRB, 0.12200E+01_JPRB, 0.11515E+01_JPRB, 0.10241E+01_JPRB, 0.96721E+00_JPRB, 0.90196E+00_JPRB ,&
&  0.87000E+00_JPRB, 0.82619E+00_JPRB, 0.76417E+00_JPRB, 0.75139E+00_JPRB, 0.73172E+00_JPRB, 0.71423E+00_JPRB, 0.73496E+00_JPRB ,&
&  0.81037E+00_JPRB, 0.89008E+00_JPRB, 0.10633E+01_JPRB, 0.11250E+01_JPRB, 0.11624E+01_JPRB, 0.12055E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.94362E+00_JPRB, 0.10223E+01_JPRB, 0.98989E+00_JPRB, 0.95043E+00_JPRB, 0.92095E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.10041E+01_JPRB, 0.97791E+00_JPRB, 0.95202E+00_JPRB, 0.94823E+00_JPRB, 0.95910E+00_JPRB, 0.95790E+00_JPRB, 0.96445E+00_JPRB ,&
&  0.98396E+00_JPRB, 0.97466E+00_JPRB, 0.94546E+00_JPRB, 0.92898E+00_JPRB, 0.90878E+00_JPRB, 0.90580E+00_JPRB, 0.80553E+00_JPRB ,&
&  0.72453E+00_JPRB, 0.72863E+00_JPRB, 0.71100E+00_JPRB, 0.73574E+00_JPRB, 0.78752E+00_JPRB, 0.77457E+00_JPRB, 0.77072E+00_JPRB ,&
&  0.76031E+00_JPRB, 0.74556E+00_JPRB, 0.74638E+00_JPRB, 0.75111E+00_JPRB, 0.76203E+00_JPRB, 0.79523E+00_JPRB, 0.81180E+00_JPRB ,&
&  0.74967E+00_JPRB, 0.67729E+00_JPRB, 0.64773E+00_JPRB, 0.65876E+00_JPRB, 0.71604E+00_JPRB, 0.77311E+00_JPRB, 0.80868E+00_JPRB ,&
&  0.83695E+00_JPRB, 0.86739E+00_JPRB, 0.90882E+00_JPRB, 0.96362E+00_JPRB, 0.99421E+00_JPRB, 0.10242E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.11023E+01_JPRB, 0.11658E+01_JPRB, 0.12504E+01_JPRB, 0.13833E+01_JPRB, 0.14047E+01_JPRB, 0.15804E+01_JPRB, 0.19213E+01_JPRB ,&
&  0.21251E+01_JPRB, 0.23266E+01_JPRB, 0.25918E+01_JPRB, 0.29912E+01_JPRB, 0.31929E+01_JPRB, 0.30269E+01_JPRB, 0.27939E+01_JPRB ,&
&  0.26384E+01_JPRB, 0.23456E+01_JPRB, 0.26275E+01_JPRB, 0.24323E+01_JPRB, 0.22767E+01_JPRB, 0.22711E+01_JPRB, 0.21612E+01_JPRB ,&
&  0.21326E+01_JPRB, 0.22041E+01_JPRB, 0.21180E+01_JPRB, 0.20592E+01_JPRB, 0.18993E+01_JPRB, 0.18649E+01_JPRB, 0.17374E+01_JPRB ,&
&  0.17510E+01_JPRB, 0.17319E+01_JPRB, 0.14904E+01_JPRB, 0.12770E+01_JPRB, 0.10814E+01_JPRB, 0.10424E+01_JPRB, 0.96396E+00_JPRB ,&
&  0.90711E+00_JPRB, 0.84861E+00_JPRB, 0.83077E+00_JPRB, 0.80244E+00_JPRB, 0.79217E+00_JPRB, 0.82920E+00_JPRB, 0.84962E+00_JPRB ,&
&  0.94324E+00_JPRB, 0.11385E+01_JPRB, 0.12420E+01_JPRB, 0.13051E+01_JPRB, 0.13603E+01_JPRB, 0.14090E+01_JPRB, 0.10923E+01_JPRB ,&
&  0.10345E+01_JPRB, 0.10959E+01_JPRB, 0.10793E+01_JPRB, 0.10487E+01_JPRB, 0.10227E+01_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.11459E+01_JPRB, 0.10963E+01_JPRB, 0.10511E+01_JPRB, 0.10237E+01_JPRB, 0.10073E+01_JPRB, 0.99886E+00_JPRB, 0.10146E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.10162E+01_JPRB, 0.99256E+00_JPRB, 0.97318E+00_JPRB, 0.95240E+00_JPRB, 0.95356E+00_JPRB, 0.96884E+00_JPRB ,&
&  0.83328E+00_JPRB, 0.83264E+00_JPRB, 0.73879E+00_JPRB, 0.75132E+00_JPRB, 0.91424E+00_JPRB, 0.83926E+00_JPRB, 0.81509E+00_JPRB ,&
&  0.78240E+00_JPRB, 0.78955E+00_JPRB, 0.79278E+00_JPRB, 0.84655E+00_JPRB, 0.89869E+00_JPRB, 0.96366E+00_JPRB, 0.97234E+00_JPRB ,&
&  0.87912E+00_JPRB, 0.76201E+00_JPRB, 0.70936E+00_JPRB, 0.69754E+00_JPRB, 0.73817E+00_JPRB, 0.77814E+00_JPRB, 0.85313E+00_JPRB ,&
&  0.89508E+00_JPRB, 0.92735E+00_JPRB, 0.95917E+00_JPRB, 0.10056E+01_JPRB, 0.10383E+01_JPRB, 0.10784E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.12803E+01_JPRB, 0.13661E+01_JPRB, 0.14888E+01_JPRB, 0.15391E+01_JPRB, 0.17042E+01_JPRB, 0.20289E+01_JPRB ,&
&  0.22554E+01_JPRB, 0.24158E+01_JPRB, 0.28252E+01_JPRB, 0.37261E+01_JPRB, 0.41443E+01_JPRB, 0.40541E+01_JPRB, 0.35986E+01_JPRB ,&
&  0.31296E+01_JPRB, 0.26823E+01_JPRB, 0.26162E+01_JPRB, 0.24680E+01_JPRB, 0.23008E+01_JPRB, 0.23955E+01_JPRB, 0.23219E+01_JPRB ,&
&  0.22151E+01_JPRB, 0.22132E+01_JPRB, 0.21925E+01_JPRB, 0.22402E+01_JPRB, 0.21352E+01_JPRB, 0.21462E+01_JPRB, 0.20828E+01_JPRB ,&
&  0.21098E+01_JPRB, 0.19405E+01_JPRB, 0.14828E+01_JPRB, 0.12613E+01_JPRB, 0.10558E+01_JPRB, 0.99843E+00_JPRB, 0.11589E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.10263E+01_JPRB, 0.10433E+01_JPRB, 0.10389E+01_JPRB, 0.10956E+01_JPRB, 0.12949E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.12802E+01_JPRB, 0.15231E+01_JPRB, 0.17073E+01_JPRB, 0.17940E+01_JPRB, 0.17387E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.12909E+01_JPRB, 0.12574E+01_JPRB, 0.12088E+01_JPRB, 0.11749E+01_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.13662E+01_JPRB, 0.13050E+01_JPRB, 0.12361E+01_JPRB, 0.11553E+01_JPRB, 0.11087E+01_JPRB, 0.10969E+01_JPRB, 0.11074E+01_JPRB ,&
&  0.11028E+01_JPRB, 0.10672E+01_JPRB, 0.10423E+01_JPRB, 0.10224E+01_JPRB, 0.10151E+01_JPRB, 0.10238E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10620E+01_JPRB, 0.89198E+00_JPRB, 0.78084E+00_JPRB, 0.84048E+00_JPRB, 0.80871E+00_JPRB, 0.82294E+00_JPRB ,&
&  0.81618E+00_JPRB, 0.90132E+00_JPRB, 0.93433E+00_JPRB, 0.10337E+01_JPRB, 0.11522E+01_JPRB, 0.12822E+01_JPRB, 0.12946E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.10034E+01_JPRB, 0.88333E+00_JPRB, 0.85194E+00_JPRB, 0.88964E+00_JPRB, 0.85500E+00_JPRB, 0.92001E+00_JPRB ,&
&  0.95655E+00_JPRB, 0.99489E+00_JPRB, 0.10366E+01_JPRB, 0.10711E+01_JPRB, 0.11233E+01_JPRB, 0.11916E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.14173E+01_JPRB, 0.14495E+01_JPRB, 0.15050E+01_JPRB, 0.16390E+01_JPRB, 0.18055E+01_JPRB, 0.18527E+01_JPRB ,&
&  0.20838E+01_JPRB, 0.23091E+01_JPRB, 0.27194E+01_JPRB, 0.35838E+01_JPRB, 0.41498E+01_JPRB, 0.42977E+01_JPRB, 0.41225E+01_JPRB ,&
&  0.37600E+01_JPRB, 0.33774E+01_JPRB, 0.32802E+01_JPRB, 0.29598E+01_JPRB, 0.27662E+01_JPRB, 0.27288E+01_JPRB, 0.26514E+01_JPRB ,&
&  0.25035E+01_JPRB, 0.24265E+01_JPRB, 0.24308E+01_JPRB, 0.23594E+01_JPRB, 0.23338E+01_JPRB, 0.22849E+01_JPRB, 0.23333E+01_JPRB ,&
&  0.25732E+01_JPRB, 0.17681E+01_JPRB, 0.14080E+01_JPRB, 0.14641E+01_JPRB, 0.12174E+01_JPRB, 0.10850E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.11293E+01_JPRB, 0.13534E+01_JPRB, 0.13830E+01_JPRB, 0.16463E+01_JPRB, 0.19868E+01_JPRB, 0.17913E+01_JPRB ,&
&  0.17335E+01_JPRB, 0.17142E+01_JPRB, 0.20980E+01_JPRB, 0.23976E+01_JPRB, 0.24376E+01_JPRB, 0.22269E+01_JPRB, 0.19336E+01_JPRB ,&
&  0.17006E+01_JPRB, 0.15795E+01_JPRB, 0.15203E+01_JPRB, 0.14676E+01_JPRB, 0.14269E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.16196E+01_JPRB, 0.15383E+01_JPRB, 0.14682E+01_JPRB, 0.13930E+01_JPRB, 0.13375E+01_JPRB, 0.12948E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11299E+01_JPRB, 0.11002E+01_JPRB, 0.10891E+01_JPRB, 0.10897E+01_JPRB, 0.11028E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.11309E+01_JPRB, 0.11609E+01_JPRB, 0.96137E+00_JPRB, 0.82785E+00_JPRB, 0.73695E+00_JPRB, 0.76004E+00_JPRB, 0.79464E+00_JPRB ,&
&  0.87246E+00_JPRB, 0.97700E+00_JPRB, 0.11758E+01_JPRB, 0.14457E+01_JPRB, 0.16210E+01_JPRB, 0.17158E+01_JPRB, 0.17439E+01_JPRB ,&
&  0.15986E+01_JPRB, 0.14478E+01_JPRB, 0.11993E+01_JPRB, 0.11478E+01_JPRB, 0.10786E+01_JPRB, 0.10045E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.11100E+01_JPRB, 0.11419E+01_JPRB, 0.11979E+01_JPRB, 0.12488E+01_JPRB, 0.12905E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.13729E+01_JPRB, 0.14237E+01_JPRB, 0.14692E+01_JPRB, 0.15126E+01_JPRB, 0.16119E+01_JPRB, 0.17662E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.20844E+01_JPRB, 0.21247E+01_JPRB, 0.22328E+01_JPRB, 0.33013E+01_JPRB, 0.47180E+01_JPRB, 0.47693E+01_JPRB, 0.43688E+01_JPRB ,&
&  0.42814E+01_JPRB, 0.42125E+01_JPRB, 0.42981E+01_JPRB, 0.41799E+01_JPRB, 0.38067E+01_JPRB, 0.36202E+01_JPRB, 0.34037E+01_JPRB ,&
&  0.31561E+01_JPRB, 0.29904E+01_JPRB, 0.28833E+01_JPRB, 0.28952E+01_JPRB, 0.28526E+01_JPRB, 0.28841E+01_JPRB, 0.29509E+01_JPRB ,&
&  0.29249E+01_JPRB, 0.25803E+01_JPRB, 0.18256E+01_JPRB, 0.17537E+01_JPRB, 0.14127E+01_JPRB, 0.12723E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.13873E+01_JPRB, 0.16534E+01_JPRB, 0.17144E+01_JPRB, 0.24281E+01_JPRB, 0.27800E+01_JPRB, 0.27615E+01_JPRB ,&
&  0.26544E+01_JPRB, 0.25208E+01_JPRB, 0.29625E+01_JPRB, 0.31312E+01_JPRB, 0.30879E+01_JPRB, 0.28648E+01_JPRB, 0.25414E+01_JPRB ,&
&  0.22014E+01_JPRB, 0.19908E+01_JPRB, 0.18827E+01_JPRB, 0.18048E+01_JPRB, 0.17090E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.18804E+01_JPRB, 0.17729E+01_JPRB, 0.16723E+01_JPRB, 0.15952E+01_JPRB, 0.15289E+01_JPRB, 0.14542E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.12651E+01_JPRB, 0.12185E+01_JPRB, 0.11986E+01_JPRB, 0.11762E+01_JPRB, 0.11626E+01_JPRB, 0.11640E+01_JPRB, 0.11718E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.11325E+01_JPRB, 0.92719E+00_JPRB, 0.78283E+00_JPRB, 0.69302E+00_JPRB, 0.69014E+00_JPRB, 0.75422E+00_JPRB ,&
&  0.87306E+00_JPRB, 0.11790E+01_JPRB, 0.16411E+01_JPRB, 0.19595E+01_JPRB, 0.21985E+01_JPRB, 0.23590E+01_JPRB, 0.23478E+01_JPRB ,&
&  0.24321E+01_JPRB, 0.19553E+01_JPRB, 0.17845E+01_JPRB, 0.15259E+01_JPRB, 0.13863E+01_JPRB, 0.13231E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.11884E+01_JPRB, 0.11961E+01_JPRB, 0.12372E+01_JPRB, 0.12680E+01_JPRB, 0.13175E+01_JPRB, 0.14017E+01_JPRB ,&
&  0.15026E+01_JPRB, 0.15616E+01_JPRB, 0.15418E+01_JPRB, 0.15367E+01_JPRB, 0.17869E+01_JPRB, 0.20986E+01_JPRB, 0.22647E+01_JPRB ,&
&  0.22769E+01_JPRB, 0.24783E+01_JPRB, 0.38251E+01_JPRB, 0.45084E+01_JPRB, 0.48477E+01_JPRB, 0.53339E+01_JPRB, 0.53339E+01_JPRB ,&
&  0.49082E+01_JPRB, 0.44925E+01_JPRB, 0.42920E+01_JPRB, 0.38166E+01_JPRB, 0.42681E+01_JPRB, 0.42908E+01_JPRB, 0.41449E+01_JPRB ,&
&  0.39078E+01_JPRB, 0.37009E+01_JPRB, 0.35100E+01_JPRB, 0.34485E+01_JPRB, 0.34818E+01_JPRB, 0.35794E+01_JPRB, 0.34121E+01_JPRB ,&
&  0.30442E+01_JPRB, 0.32169E+01_JPRB, 0.27509E+01_JPRB, 0.22443E+01_JPRB, 0.18554E+01_JPRB, 0.17331E+01_JPRB, 0.16357E+01_JPRB ,&
&  0.15314E+01_JPRB, 0.16208E+01_JPRB, 0.20635E+01_JPRB, 0.26040E+01_JPRB, 0.34019E+01_JPRB, 0.39115E+01_JPRB, 0.38378E+01_JPRB ,&
&  0.35509E+01_JPRB, 0.38672E+01_JPRB, 0.37428E+01_JPRB, 0.34978E+01_JPRB, 0.33204E+01_JPRB, 0.30487E+01_JPRB, 0.28015E+01_JPRB ,&
&  0.25370E+01_JPRB, 0.23608E+01_JPRB, 0.22371E+01_JPRB, 0.21244E+01_JPRB, 0.20024E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.20103E+01_JPRB, 0.18990E+01_JPRB, 0.17894E+01_JPRB, 0.16917E+01_JPRB, 0.15907E+01_JPRB, 0.15017E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.13411E+01_JPRB, 0.13057E+01_JPRB, 0.12646E+01_JPRB, 0.12293E+01_JPRB, 0.12088E+01_JPRB, 0.11914E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.11885E+01_JPRB, 0.10675E+01_JPRB, 0.89080E+00_JPRB, 0.82294E+00_JPRB, 0.77107E+00_JPRB, 0.71856E+00_JPRB, 0.76118E+00_JPRB ,&
&  0.96890E+00_JPRB, 0.15203E+01_JPRB, 0.20782E+01_JPRB, 0.25208E+01_JPRB, 0.28509E+01_JPRB, 0.30353E+01_JPRB, 0.29798E+01_JPRB ,&
&  0.26577E+01_JPRB, 0.26132E+01_JPRB, 0.23733E+01_JPRB, 0.21166E+01_JPRB, 0.18479E+01_JPRB, 0.16644E+01_JPRB, 0.15285E+01_JPRB ,&
&  0.14197E+01_JPRB, 0.13760E+01_JPRB, 0.13761E+01_JPRB, 0.13982E+01_JPRB, 0.14546E+01_JPRB, 0.15312E+01_JPRB, 0.16257E+01_JPRB ,&
&  0.16665E+01_JPRB, 0.16852E+01_JPRB, 0.16372E+01_JPRB, 0.16755E+01_JPRB, 0.18049E+01_JPRB, 0.23301E+01_JPRB, 0.25548E+01_JPRB ,&
&  0.31044E+01_JPRB, 0.35870E+01_JPRB, 0.37560E+01_JPRB, 0.45680E+01_JPRB, 0.40333E+01_JPRB, 0.47540E+01_JPRB, 0.55437E+01_JPRB ,&
&  0.50200E+01_JPRB, 0.45023E+01_JPRB, 0.43530E+01_JPRB, 0.36757E+01_JPRB, 0.34019E+01_JPRB, 0.47475E+01_JPRB, 0.47126E+01_JPRB ,&
&  0.45475E+01_JPRB, 0.46135E+01_JPRB, 0.43095E+01_JPRB, 0.41051E+01_JPRB, 0.38016E+01_JPRB, 0.34977E+01_JPRB, 0.26980E+01_JPRB ,&
&  0.27666E+01_JPRB, 0.25760E+01_JPRB, 0.26821E+01_JPRB, 0.23891E+01_JPRB, 0.22315E+01_JPRB, 0.20450E+01_JPRB, 0.18624E+01_JPRB ,&
&  0.17797E+01_JPRB, 0.18956E+01_JPRB, 0.30997E+01_JPRB, 0.46795E+01_JPRB, 0.55233E+01_JPRB, 0.48182E+01_JPRB, 0.53495E+01_JPRB ,&
&  0.51265E+01_JPRB, 0.43751E+01_JPRB, 0.35439E+01_JPRB, 0.32259E+01_JPRB, 0.29605E+01_JPRB, 0.27412E+01_JPRB, 0.25778E+01_JPRB ,&
&  0.24304E+01_JPRB, 0.22997E+01_JPRB, 0.22048E+01_JPRB, 0.21549E+01_JPRB, 0.20936E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.20443E+01_JPRB, 0.19377E+01_JPRB, 0.18213E+01_JPRB, 0.17226E+01_JPRB, 0.16265E+01_JPRB, 0.15386E+01_JPRB, 0.14686E+01_JPRB ,&
&  0.14223E+01_JPRB, 0.13682E+01_JPRB, 0.12909E+01_JPRB, 0.12378E+01_JPRB, 0.12119E+01_JPRB, 0.12104E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.11872E+01_JPRB, 0.11063E+01_JPRB, 0.93871E+00_JPRB, 0.86684E+00_JPRB, 0.88933E+00_JPRB, 0.83813E+00_JPRB, 0.85312E+00_JPRB ,&
&  0.12727E+01_JPRB, 0.21509E+01_JPRB, 0.26851E+01_JPRB, 0.29941E+01_JPRB, 0.33878E+01_JPRB, 0.35626E+01_JPRB, 0.33667E+01_JPRB ,&
&  0.33034E+01_JPRB, 0.29389E+01_JPRB, 0.26653E+01_JPRB, 0.24168E+01_JPRB, 0.21709E+01_JPRB, 0.19186E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.15980E+01_JPRB, 0.15358E+01_JPRB, 0.15231E+01_JPRB, 0.15037E+01_JPRB, 0.15682E+01_JPRB, 0.16358E+01_JPRB, 0.16916E+01_JPRB ,&
&  0.17168E+01_JPRB, 0.17319E+01_JPRB, 0.17091E+01_JPRB, 0.17372E+01_JPRB, 0.20452E+01_JPRB, 0.25769E+01_JPRB, 0.30264E+01_JPRB ,&
&  0.33187E+01_JPRB, 0.36074E+01_JPRB, 0.39326E+01_JPRB, 0.43974E+01_JPRB, 0.52290E+01_JPRB, 0.53052E+01_JPRB, 0.56775E+01_JPRB ,&
&  0.44957E+01_JPRB, 0.37361E+01_JPRB, 0.31179E+01_JPRB, 0.25238E+01_JPRB, 0.34929E+01_JPRB, 0.55165E+01_JPRB, 0.58450E+01_JPRB ,&
&  0.56633E+01_JPRB, 0.57360E+01_JPRB, 0.54572E+01_JPRB, 0.45644E+01_JPRB, 0.38601E+01_JPRB, 0.31673E+01_JPRB, 0.36369E+01_JPRB ,&
&  0.35782E+01_JPRB, 0.34940E+01_JPRB, 0.31015E+01_JPRB, 0.25321E+01_JPRB, 0.21920E+01_JPRB, 0.23647E+01_JPRB, 0.21957E+01_JPRB ,&
&  0.21252E+01_JPRB, 0.25954E+01_JPRB, 0.55034E+01_JPRB, 0.70620E+01_JPRB, 0.67745E+01_JPRB, 0.58750E+01_JPRB, 0.64187E+01_JPRB ,&
&  0.51223E+01_JPRB, 0.40968E+01_JPRB, 0.33923E+01_JPRB, 0.28854E+01_JPRB, 0.26116E+01_JPRB, 0.24574E+01_JPRB, 0.23869E+01_JPRB ,&
&  0.23456E+01_JPRB, 0.23159E+01_JPRB, 0.22450E+01_JPRB, 0.21686E+01_JPRB, 0.21056E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.20871E+01_JPRB, 0.19788E+01_JPRB, 0.18810E+01_JPRB, 0.17757E+01_JPRB, 0.16701E+01_JPRB, 0.15930E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.13986E+01_JPRB, 0.13325E+01_JPRB, 0.12889E+01_JPRB, 0.12696E+01_JPRB, 0.12574E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.12151E+01_JPRB, 0.10426E+01_JPRB, 0.96874E+00_JPRB, 0.97857E+00_JPRB, 0.98411E+00_JPRB, 0.10877E+01_JPRB ,&
&  0.19162E+01_JPRB, 0.29171E+01_JPRB, 0.30761E+01_JPRB, 0.31326E+01_JPRB, 0.36219E+01_JPRB, 0.32291E+01_JPRB, 0.31118E+01_JPRB ,&
&  0.31778E+01_JPRB, 0.28564E+01_JPRB, 0.26403E+01_JPRB, 0.24272E+01_JPRB, 0.22367E+01_JPRB, 0.20665E+01_JPRB, 0.19061E+01_JPRB ,&
&  0.17794E+01_JPRB, 0.17011E+01_JPRB, 0.16865E+01_JPRB, 0.17341E+01_JPRB, 0.17592E+01_JPRB, 0.17695E+01_JPRB, 0.17716E+01_JPRB ,&
&  0.17571E+01_JPRB, 0.17319E+01_JPRB, 0.17091E+01_JPRB, 0.18761E+01_JPRB, 0.22778E+01_JPRB, 0.28293E+01_JPRB, 0.38655E+01_JPRB ,&
&  0.38543E+01_JPRB, 0.41894E+01_JPRB, 0.41850E+01_JPRB, 0.53111E+01_JPRB, 0.59345E+01_JPRB, 0.57813E+01_JPRB, 0.56902E+01_JPRB ,&
&  0.43227E+01_JPRB, 0.36235E+01_JPRB, 0.33586E+01_JPRB, 0.33852E+01_JPRB, 0.38617E+01_JPRB, 0.47490E+01_JPRB, 0.61642E+01_JPRB ,&
&  0.52489E+01_JPRB, 0.52757E+01_JPRB, 0.52580E+01_JPRB, 0.48550E+01_JPRB, 0.29414E+01_JPRB, 0.23224E+01_JPRB, 0.34264E+01_JPRB ,&
&  0.34856E+01_JPRB, 0.21303E+01_JPRB, 0.15397E+01_JPRB, 0.18470E+01_JPRB, 0.14500E+01_JPRB, 0.17454E+01_JPRB, 0.25347E+01_JPRB ,&
&  0.29450E+01_JPRB, 0.40233E+01_JPRB, 0.70152E+01_JPRB, 0.66644E+01_JPRB, 0.64158E+01_JPRB, 0.69594E+01_JPRB, 0.58580E+01_JPRB ,&
&  0.46349E+01_JPRB, 0.35360E+01_JPRB, 0.32838E+01_JPRB, 0.28353E+01_JPRB, 0.25923E+01_JPRB, 0.24560E+01_JPRB, 0.24517E+01_JPRB ,&
&  0.25249E+01_JPRB, 0.25335E+01_JPRB, 0.24587E+01_JPRB, 0.23222E+01_JPRB, 0.21897E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.21550E+01_JPRB, 0.20553E+01_JPRB, 0.19559E+01_JPRB, 0.18580E+01_JPRB, 0.17676E+01_JPRB, 0.16918E+01_JPRB, 0.15925E+01_JPRB ,&
&  0.14698E+01_JPRB, 0.13957E+01_JPRB, 0.13666E+01_JPRB, 0.13359E+01_JPRB, 0.13173E+01_JPRB, 0.13157E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.12620E+01_JPRB, 0.12426E+01_JPRB, 0.12146E+01_JPRB, 0.11431E+01_JPRB, 0.11150E+01_JPRB, 0.10892E+01_JPRB, 0.14807E+01_JPRB ,&
&  0.27987E+01_JPRB, 0.37066E+01_JPRB, 0.34885E+01_JPRB, 0.31506E+01_JPRB, 0.30613E+01_JPRB, 0.28353E+01_JPRB, 0.30862E+01_JPRB ,&
&  0.29748E+01_JPRB, 0.27620E+01_JPRB, 0.25449E+01_JPRB, 0.23600E+01_JPRB, 0.22098E+01_JPRB, 0.20773E+01_JPRB, 0.19897E+01_JPRB ,&
&  0.18958E+01_JPRB, 0.18133E+01_JPRB, 0.18147E+01_JPRB, 0.17811E+01_JPRB, 0.17601E+01_JPRB, 0.17369E+01_JPRB, 0.17732E+01_JPRB ,&
&  0.17599E+01_JPRB, 0.17499E+01_JPRB, 0.17892E+01_JPRB, 0.20400E+01_JPRB, 0.26992E+01_JPRB, 0.34345E+01_JPRB, 0.36473E+01_JPRB ,&
&  0.37253E+01_JPRB, 0.39676E+01_JPRB, 0.47906E+01_JPRB, 0.53531E+01_JPRB, 0.57617E+01_JPRB, 0.58531E+01_JPRB, 0.53111E+01_JPRB ,&
&  0.48523E+01_JPRB, 0.44323E+01_JPRB, 0.36918E+01_JPRB, 0.37728E+01_JPRB, 0.44198E+01_JPRB, 0.43840E+01_JPRB, 0.51545E+01_JPRB ,&
&  0.52451E+01_JPRB, 0.47286E+01_JPRB, 0.37987E+01_JPRB, 0.31062E+01_JPRB, 0.34173E+01_JPRB, 0.25923E+01_JPRB, 0.17092E+01_JPRB ,&
&  0.15140E+01_JPRB, 0.13449E+01_JPRB, 0.11891E+01_JPRB, 0.12073E+01_JPRB, 0.11913E+01_JPRB, 0.16790E+01_JPRB, 0.27703E+01_JPRB ,&
&  0.41991E+01_JPRB, 0.54464E+01_JPRB, 0.65250E+01_JPRB, 0.59294E+01_JPRB, 0.53999E+01_JPRB, 0.61217E+01_JPRB, 0.56236E+01_JPRB ,&
&  0.37008E+01_JPRB, 0.30836E+01_JPRB, 0.30084E+01_JPRB, 0.27328E+01_JPRB, 0.25441E+01_JPRB, 0.25166E+01_JPRB, 0.26365E+01_JPRB ,&
&  0.27657E+01_JPRB, 0.27608E+01_JPRB, 0.26464E+01_JPRB, 0.24463E+01_JPRB, 0.22725E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.21432E+01_JPRB, 0.20163E+01_JPRB, 0.19320E+01_JPRB, 0.18791E+01_JPRB, 0.18241E+01_JPRB, 0.17247E+01_JPRB, 0.15548E+01_JPRB ,&
&  0.14291E+01_JPRB, 0.13802E+01_JPRB, 0.13713E+01_JPRB, 0.13513E+01_JPRB, 0.13304E+01_JPRB, 0.13203E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.12568E+01_JPRB, 0.12427E+01_JPRB, 0.12856E+01_JPRB, 0.12867E+01_JPRB, 0.13195E+01_JPRB, 0.13350E+01_JPRB, 0.20229E+01_JPRB ,&
&  0.37344E+01_JPRB, 0.43325E+01_JPRB, 0.36908E+01_JPRB, 0.30057E+01_JPRB, 0.29413E+01_JPRB, 0.29933E+01_JPRB, 0.30164E+01_JPRB ,&
&  0.27257E+01_JPRB, 0.24552E+01_JPRB, 0.21602E+01_JPRB, 0.20022E+01_JPRB, 0.19754E+01_JPRB, 0.19543E+01_JPRB, 0.19269E+01_JPRB ,&
&  0.18491E+01_JPRB, 0.18068E+01_JPRB, 0.17565E+01_JPRB, 0.17153E+01_JPRB, 0.17097E+01_JPRB, 0.17392E+01_JPRB, 0.17922E+01_JPRB ,&
&  0.18240E+01_JPRB, 0.18555E+01_JPRB, 0.19964E+01_JPRB, 0.24244E+01_JPRB, 0.29601E+01_JPRB, 0.31762E+01_JPRB, 0.37062E+01_JPRB ,&
&  0.42800E+01_JPRB, 0.47544E+01_JPRB, 0.46496E+01_JPRB, 0.56321E+01_JPRB, 0.57030E+01_JPRB, 0.53405E+01_JPRB, 0.50375E+01_JPRB ,&
&  0.47208E+01_JPRB, 0.49010E+01_JPRB, 0.38836E+01_JPRB, 0.35576E+01_JPRB, 0.47478E+01_JPRB, 0.47929E+01_JPRB, 0.49739E+01_JPRB ,&
&  0.54107E+01_JPRB, 0.47088E+01_JPRB, 0.40030E+01_JPRB, 0.38568E+01_JPRB, 0.55922E+01_JPRB, 0.54637E+01_JPRB, 0.21785E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.13901E+01_JPRB, 0.11844E+01_JPRB, 0.11053E+01_JPRB, 0.12017E+01_JPRB, 0.18044E+01_JPRB, 0.36659E+01_JPRB ,&
&  0.43138E+01_JPRB, 0.60744E+01_JPRB, 0.62408E+01_JPRB, 0.58174E+01_JPRB, 0.49922E+01_JPRB, 0.47373E+01_JPRB, 0.41071E+01_JPRB ,&
&  0.36425E+01_JPRB, 0.31597E+01_JPRB, 0.26938E+01_JPRB, 0.24120E+01_JPRB, 0.23103E+01_JPRB, 0.23951E+01_JPRB, 0.26086E+01_JPRB ,&
&  0.27234E+01_JPRB, 0.27343E+01_JPRB, 0.26385E+01_JPRB, 0.24675E+01_JPRB, 0.22828E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.20470E+01_JPRB, 0.19892E+01_JPRB, 0.19545E+01_JPRB, 0.18862E+01_JPRB, 0.17957E+01_JPRB, 0.16714E+01_JPRB, 0.15147E+01_JPRB ,&
&  0.14337E+01_JPRB, 0.13997E+01_JPRB, 0.13937E+01_JPRB, 0.13593E+01_JPRB, 0.13172E+01_JPRB, 0.12919E+01_JPRB, 0.12798E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.12234E+01_JPRB, 0.12430E+01_JPRB, 0.13796E+01_JPRB, 0.15551E+01_JPRB, 0.18864E+01_JPRB, 0.27552E+01_JPRB ,&
&  0.45029E+01_JPRB, 0.47069E+01_JPRB, 0.38624E+01_JPRB, 0.32697E+01_JPRB, 0.29850E+01_JPRB, 0.29632E+01_JPRB, 0.26338E+01_JPRB ,&
&  0.23011E+01_JPRB, 0.20723E+01_JPRB, 0.18694E+01_JPRB, 0.18625E+01_JPRB, 0.19011E+01_JPRB, 0.19293E+01_JPRB, 0.19010E+01_JPRB ,&
&  0.18779E+01_JPRB, 0.17928E+01_JPRB, 0.17561E+01_JPRB, 0.17261E+01_JPRB, 0.17451E+01_JPRB, 0.18119E+01_JPRB, 0.18804E+01_JPRB ,&
&  0.19510E+01_JPRB, 0.20508E+01_JPRB, 0.23074E+01_JPRB, 0.27388E+01_JPRB, 0.27740E+01_JPRB, 0.34392E+01_JPRB, 0.38469E+01_JPRB ,&
&  0.42057E+01_JPRB, 0.43806E+01_JPRB, 0.43237E+01_JPRB, 0.49746E+01_JPRB, 0.54044E+01_JPRB, 0.51397E+01_JPRB, 0.47321E+01_JPRB ,&
&  0.48207E+01_JPRB, 0.42051E+01_JPRB, 0.37279E+01_JPRB, 0.34357E+01_JPRB, 0.40671E+01_JPRB, 0.59017E+01_JPRB, 0.46118E+01_JPRB ,&
&  0.44946E+01_JPRB, 0.51547E+01_JPRB, 0.47412E+01_JPRB, 0.49084E+01_JPRB, 0.59509E+01_JPRB, 0.73593E+01_JPRB, 0.57376E+01_JPRB ,&
&  0.20784E+01_JPRB, 0.14222E+01_JPRB, 0.14492E+01_JPRB, 0.14068E+01_JPRB, 0.12680E+01_JPRB, 0.19420E+01_JPRB, 0.66517E+01_JPRB ,&
&  0.54610E+01_JPRB, 0.71359E+01_JPRB, 0.67728E+01_JPRB, 0.58742E+01_JPRB, 0.51965E+01_JPRB, 0.42752E+01_JPRB, 0.37238E+01_JPRB ,&
&  0.30191E+01_JPRB, 0.25802E+01_JPRB, 0.23530E+01_JPRB, 0.22353E+01_JPRB, 0.22282E+01_JPRB, 0.22810E+01_JPRB, 0.24302E+01_JPRB ,&
&  0.25284E+01_JPRB, 0.25456E+01_JPRB, 0.24501E+01_JPRB, 0.23006E+01_JPRB, 0.21547E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.19794E+01_JPRB, 0.19519E+01_JPRB, 0.19036E+01_JPRB, 0.18075E+01_JPRB, 0.17106E+01_JPRB, 0.15766E+01_JPRB, 0.14700E+01_JPRB ,&
&  0.14452E+01_JPRB, 0.14184E+01_JPRB, 0.13926E+01_JPRB, 0.13547E+01_JPRB, 0.13070E+01_JPRB, 0.12705E+01_JPRB, 0.12566E+01_JPRB ,&
&  0.12473E+01_JPRB, 0.12129E+01_JPRB, 0.12137E+01_JPRB, 0.13657E+01_JPRB, 0.18404E+01_JPRB, 0.23978E+01_JPRB, 0.30187E+01_JPRB ,&
&  0.51453E+01_JPRB, 0.50781E+01_JPRB, 0.34489E+01_JPRB, 0.30420E+01_JPRB, 0.29401E+01_JPRB, 0.29261E+01_JPRB, 0.24032E+01_JPRB ,&
&  0.21166E+01_JPRB, 0.19633E+01_JPRB, 0.18193E+01_JPRB, 0.18083E+01_JPRB, 0.18422E+01_JPRB, 0.19025E+01_JPRB, 0.19203E+01_JPRB ,&
&  0.18470E+01_JPRB, 0.18221E+01_JPRB, 0.18430E+01_JPRB, 0.18427E+01_JPRB, 0.18595E+01_JPRB, 0.19409E+01_JPRB, 0.20212E+01_JPRB ,&
&  0.21316E+01_JPRB, 0.23031E+01_JPRB, 0.26083E+01_JPRB, 0.28203E+01_JPRB, 0.31356E+01_JPRB, 0.34579E+01_JPRB, 0.38571E+01_JPRB ,&
&  0.39102E+01_JPRB, 0.40814E+01_JPRB, 0.43145E+01_JPRB, 0.45638E+01_JPRB, 0.47827E+01_JPRB, 0.48132E+01_JPRB, 0.45983E+01_JPRB ,&
&  0.45947E+01_JPRB, 0.39780E+01_JPRB, 0.34743E+01_JPRB, 0.33345E+01_JPRB, 0.38118E+01_JPRB, 0.53875E+01_JPRB, 0.57952E+01_JPRB ,&
&  0.58272E+01_JPRB, 0.44966E+01_JPRB, 0.45847E+01_JPRB, 0.57558E+01_JPRB, 0.58348E+01_JPRB, 0.66038E+01_JPRB, 0.90407E+01_JPRB ,&
&  0.80375E+01_JPRB, 0.39930E+01_JPRB, 0.31677E+01_JPRB, 0.41241E+01_JPRB, 0.24718E+01_JPRB, 0.21408E+01_JPRB, 0.40504E+01_JPRB ,&
&  0.53842E+01_JPRB, 0.65172E+01_JPRB, 0.62975E+01_JPRB, 0.55502E+01_JPRB, 0.50076E+01_JPRB, 0.37874E+01_JPRB, 0.30047E+01_JPRB ,&
&  0.24081E+01_JPRB, 0.21946E+01_JPRB, 0.20915E+01_JPRB, 0.19835E+01_JPRB, 0.19475E+01_JPRB, 0.20293E+01_JPRB, 0.21522E+01_JPRB ,&
&  0.22546E+01_JPRB, 0.22817E+01_JPRB, 0.22455E+01_JPRB, 0.21534E+01_JPRB, 0.20453E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.18904E+01_JPRB, 0.18732E+01_JPRB, 0.18404E+01_JPRB, 0.17380E+01_JPRB, 0.16294E+01_JPRB, 0.15185E+01_JPRB, 0.14816E+01_JPRB ,&
&  0.14463E+01_JPRB, 0.14145E+01_JPRB, 0.13792E+01_JPRB, 0.13280E+01_JPRB, 0.13016E+01_JPRB, 0.12702E+01_JPRB, 0.12732E+01_JPRB ,&
&  0.12864E+01_JPRB, 0.12649E+01_JPRB, 0.13303E+01_JPRB, 0.17032E+01_JPRB, 0.24601E+01_JPRB, 0.40441E+01_JPRB, 0.34882E+01_JPRB ,&
&  0.51582E+01_JPRB, 0.54694E+01_JPRB, 0.34721E+01_JPRB, 0.31668E+01_JPRB, 0.30710E+01_JPRB, 0.30702E+01_JPRB, 0.25289E+01_JPRB ,&
&  0.21739E+01_JPRB, 0.20239E+01_JPRB, 0.19330E+01_JPRB, 0.18479E+01_JPRB, 0.18538E+01_JPRB, 0.18918E+01_JPRB, 0.18463E+01_JPRB ,&
&  0.18583E+01_JPRB, 0.19122E+01_JPRB, 0.19348E+01_JPRB, 0.19426E+01_JPRB, 0.19927E+01_JPRB, 0.20752E+01_JPRB, 0.21787E+01_JPRB ,&
&  0.23487E+01_JPRB, 0.25620E+01_JPRB, 0.28789E+01_JPRB, 0.29670E+01_JPRB, 0.31832E+01_JPRB, 0.35259E+01_JPRB, 0.37068E+01_JPRB ,&
&  0.36046E+01_JPRB, 0.34987E+01_JPRB, 0.39198E+01_JPRB, 0.42927E+01_JPRB, 0.43902E+01_JPRB, 0.43641E+01_JPRB, 0.43198E+01_JPRB ,&
&  0.45170E+01_JPRB, 0.43149E+01_JPRB, 0.43296E+01_JPRB, 0.38040E+01_JPRB, 0.42782E+01_JPRB, 0.53912E+01_JPRB, 0.55303E+01_JPRB ,&
&  0.66068E+01_JPRB, 0.61412E+01_JPRB, 0.62528E+01_JPRB, 0.61660E+01_JPRB, 0.58046E+01_JPRB, 0.61199E+01_JPRB, 0.69154E+01_JPRB ,&
&  0.74322E+01_JPRB, 0.82687E+01_JPRB, 0.78162E+01_JPRB, 0.47157E+01_JPRB, 0.30944E+01_JPRB, 0.26879E+01_JPRB, 0.33263E+01_JPRB ,&
&  0.47882E+01_JPRB, 0.50818E+01_JPRB, 0.48351E+01_JPRB, 0.62956E+01_JPRB, 0.40504E+01_JPRB, 0.28725E+01_JPRB, 0.22364E+01_JPRB ,&
&  0.19383E+01_JPRB, 0.18566E+01_JPRB, 0.17916E+01_JPRB, 0.16815E+01_JPRB, 0.16613E+01_JPRB, 0.17281E+01_JPRB, 0.18360E+01_JPRB ,&
&  0.19527E+01_JPRB, 0.20273E+01_JPRB, 0.20612E+01_JPRB, 0.20072E+01_JPRB, 0.19352E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.17538E+01_JPRB, 0.17533E+01_JPRB, 0.16970E+01_JPRB, 0.16152E+01_JPRB, 0.15478E+01_JPRB, 0.14908E+01_JPRB, 0.14826E+01_JPRB ,&
&  0.14025E+01_JPRB, 0.13621E+01_JPRB, 0.13313E+01_JPRB, 0.13027E+01_JPRB, 0.13171E+01_JPRB, 0.13403E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.15700E+01_JPRB, 0.18869E+01_JPRB, 0.26346E+01_JPRB, 0.37050E+01_JPRB, 0.52173E+01_JPRB, 0.53857E+01_JPRB ,&
&  0.50655E+01_JPRB, 0.59690E+01_JPRB, 0.34197E+01_JPRB, 0.34100E+01_JPRB, 0.34782E+01_JPRB, 0.39245E+01_JPRB, 0.32652E+01_JPRB ,&
&  0.24829E+01_JPRB, 0.22634E+01_JPRB, 0.21527E+01_JPRB, 0.19579E+01_JPRB, 0.18928E+01_JPRB, 0.18650E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.19415E+01_JPRB, 0.20096E+01_JPRB, 0.20470E+01_JPRB, 0.20930E+01_JPRB, 0.21687E+01_JPRB, 0.22532E+01_JPRB, 0.24093E+01_JPRB ,&
&  0.26360E+01_JPRB, 0.28816E+01_JPRB, 0.29964E+01_JPRB, 0.30802E+01_JPRB, 0.31738E+01_JPRB, 0.34262E+01_JPRB, 0.35874E+01_JPRB ,&
&  0.35622E+01_JPRB, 0.34680E+01_JPRB, 0.37445E+01_JPRB, 0.38287E+01_JPRB, 0.38471E+01_JPRB, 0.41655E+01_JPRB, 0.41037E+01_JPRB ,&
&  0.44742E+01_JPRB, 0.42044E+01_JPRB, 0.47853E+01_JPRB, 0.43752E+01_JPRB, 0.45540E+01_JPRB, 0.50665E+01_JPRB, 0.46576E+01_JPRB ,&
&  0.48437E+01_JPRB, 0.56762E+01_JPRB, 0.59819E+01_JPRB, 0.61115E+01_JPRB, 0.60890E+01_JPRB, 0.60615E+01_JPRB, 0.58691E+01_JPRB ,&
&  0.52131E+01_JPRB, 0.51416E+01_JPRB, 0.45587E+01_JPRB, 0.29935E+01_JPRB, 0.23785E+01_JPRB, 0.24439E+01_JPRB, 0.28529E+01_JPRB ,&
&  0.37926E+01_JPRB, 0.40952E+01_JPRB, 0.45612E+01_JPRB, 0.41338E+01_JPRB, 0.26305E+01_JPRB, 0.19636E+01_JPRB, 0.17656E+01_JPRB ,&
&  0.16844E+01_JPRB, 0.16171E+01_JPRB, 0.15184E+01_JPRB, 0.14276E+01_JPRB, 0.14330E+01_JPRB, 0.15135E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.17213E+01_JPRB, 0.17903E+01_JPRB, 0.17935E+01_JPRB, 0.17623E+01_JPRB, 0.17574E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.17083E+01_JPRB, 0.17071E+01_JPRB, 0.16959E+01_JPRB, 0.17077E+01_JPRB, 0.17319E+01_JPRB, 0.17739E+01_JPRB, 0.17488E+01_JPRB ,&
&  0.15737E+01_JPRB, 0.13109E+01_JPRB, 0.13287E+01_JPRB, 0.13802E+01_JPRB, 0.14912E+01_JPRB, 0.16023E+01_JPRB, 0.17573E+01_JPRB ,&
&  0.20011E+01_JPRB, 0.23448E+01_JPRB, 0.29413E+01_JPRB, 0.38460E+01_JPRB, 0.49897E+01_JPRB, 0.63854E+01_JPRB, 0.67971E+01_JPRB ,&
&  0.65286E+01_JPRB, 0.44498E+01_JPRB, 0.36432E+01_JPRB, 0.35203E+01_JPRB, 0.33295E+01_JPRB, 0.37427E+01_JPRB, 0.35445E+01_JPRB ,&
&  0.28143E+01_JPRB, 0.24916E+01_JPRB, 0.23697E+01_JPRB, 0.21804E+01_JPRB, 0.20130E+01_JPRB, 0.20408E+01_JPRB, 0.20981E+01_JPRB ,&
&  0.21784E+01_JPRB, 0.22286E+01_JPRB, 0.22463E+01_JPRB, 0.22818E+01_JPRB, 0.23705E+01_JPRB, 0.24726E+01_JPRB, 0.26746E+01_JPRB ,&
&  0.29042E+01_JPRB, 0.30857E+01_JPRB, 0.31475E+01_JPRB, 0.30769E+01_JPRB, 0.31587E+01_JPRB, 0.33042E+01_JPRB, 0.34018E+01_JPRB ,&
&  0.34595E+01_JPRB, 0.35770E+01_JPRB, 0.36736E+01_JPRB, 0.37405E+01_JPRB, 0.35919E+01_JPRB, 0.40088E+01_JPRB, 0.39723E+01_JPRB ,&
&  0.41604E+01_JPRB, 0.40416E+01_JPRB, 0.46359E+01_JPRB, 0.41048E+01_JPRB, 0.45146E+01_JPRB, 0.45700E+01_JPRB, 0.44537E+01_JPRB ,&
&  0.47124E+01_JPRB, 0.55837E+01_JPRB, 0.59235E+01_JPRB, 0.59569E+01_JPRB, 0.60339E+01_JPRB, 0.53803E+01_JPRB, 0.56836E+01_JPRB ,&
&  0.44487E+01_JPRB, 0.44733E+01_JPRB, 0.29938E+01_JPRB, 0.22364E+01_JPRB, 0.18366E+01_JPRB, 0.22149E+01_JPRB, 0.27627E+01_JPRB ,&
&  0.34470E+01_JPRB, 0.33615E+01_JPRB, 0.28343E+01_JPRB, 0.25883E+01_JPRB, 0.20355E+01_JPRB, 0.16828E+01_JPRB, 0.15991E+01_JPRB ,&
&  0.15603E+01_JPRB, 0.14683E+01_JPRB, 0.13853E+01_JPRB, 0.13636E+01_JPRB, 0.13817E+01_JPRB, 0.14824E+01_JPRB, 0.15558E+01_JPRB ,&
&  0.16362E+01_JPRB, 0.16607E+01_JPRB, 0.16622E+01_JPRB, 0.16740E+01_JPRB, 0.16858E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.16703E+01_JPRB, 0.16372E+01_JPRB, 0.16226E+01_JPRB, 0.16055E+01_JPRB, 0.15798E+01_JPRB, 0.15037E+01_JPRB, 0.14469E+01_JPRB ,&
&  0.13874E+01_JPRB, 0.13541E+01_JPRB, 0.14729E+01_JPRB, 0.16394E+01_JPRB, 0.18509E+01_JPRB, 0.20841E+01_JPRB, 0.23651E+01_JPRB ,&
&  0.27300E+01_JPRB, 0.31975E+01_JPRB, 0.37896E+01_JPRB, 0.43334E+01_JPRB, 0.49120E+01_JPRB, 0.56662E+01_JPRB, 0.61774E+01_JPRB ,&
&  0.54861E+01_JPRB, 0.33553E+01_JPRB, 0.35199E+01_JPRB, 0.35299E+01_JPRB, 0.33757E+01_JPRB, 0.33722E+01_JPRB, 0.30962E+01_JPRB ,&
&  0.27092E+01_JPRB, 0.25744E+01_JPRB, 0.25238E+01_JPRB, 0.24284E+01_JPRB, 0.22375E+01_JPRB, 0.22809E+01_JPRB, 0.23291E+01_JPRB ,&
&  0.23483E+01_JPRB, 0.23304E+01_JPRB, 0.23292E+01_JPRB, 0.24324E+01_JPRB, 0.26221E+01_JPRB, 0.27299E+01_JPRB, 0.28955E+01_JPRB ,&
&  0.30362E+01_JPRB, 0.32172E+01_JPRB, 0.32785E+01_JPRB, 0.32557E+01_JPRB, 0.33388E+01_JPRB, 0.34120E+01_JPRB, 0.34454E+01_JPRB ,&
&  0.34729E+01_JPRB, 0.34860E+01_JPRB, 0.36149E+01_JPRB, 0.39450E+01_JPRB, 0.41317E+01_JPRB, 0.37618E+01_JPRB, 0.39060E+01_JPRB ,&
&  0.39962E+01_JPRB, 0.41012E+01_JPRB, 0.40592E+01_JPRB, 0.51484E+01_JPRB, 0.39889E+01_JPRB, 0.41367E+01_JPRB, 0.38953E+01_JPRB ,&
&  0.46198E+01_JPRB, 0.51266E+01_JPRB, 0.55497E+01_JPRB, 0.56806E+01_JPRB, 0.55476E+01_JPRB, 0.50895E+01_JPRB, 0.54521E+01_JPRB ,&
&  0.47161E+01_JPRB, 0.39800E+01_JPRB, 0.27801E+01_JPRB, 0.20245E+01_JPRB, 0.20440E+01_JPRB, 0.34824E+01_JPRB, 0.35632E+01_JPRB ,&
&  0.32885E+01_JPRB, 0.25435E+01_JPRB, 0.23389E+01_JPRB, 0.26462E+01_JPRB, 0.20205E+01_JPRB, 0.15754E+01_JPRB, 0.14523E+01_JPRB ,&
&  0.14600E+01_JPRB, 0.14315E+01_JPRB, 0.13726E+01_JPRB, 0.13724E+01_JPRB, 0.14349E+01_JPRB, 0.15304E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.16280E+01_JPRB, 0.16136E+01_JPRB, 0.16216E+01_JPRB, 0.16377E+01_JPRB, 0.16809E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.15507E+01_JPRB, 0.14894E+01_JPRB, 0.14410E+01_JPRB, 0.13936E+01_JPRB, 0.13472E+01_JPRB, 0.13220E+01_JPRB, 0.13378E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.14757E+01_JPRB, 0.16198E+01_JPRB, 0.17980E+01_JPRB, 0.20265E+01_JPRB, 0.23078E+01_JPRB, 0.26176E+01_JPRB ,&
&  0.30146E+01_JPRB, 0.33887E+01_JPRB, 0.35899E+01_JPRB, 0.37291E+01_JPRB, 0.38975E+01_JPRB, 0.41467E+01_JPRB, 0.43579E+01_JPRB ,&
&  0.45347E+01_JPRB, 0.43203E+01_JPRB, 0.37592E+01_JPRB, 0.27497E+01_JPRB, 0.24611E+01_JPRB, 0.27764E+01_JPRB, 0.26978E+01_JPRB ,&
&  0.26178E+01_JPRB, 0.25153E+01_JPRB, 0.24397E+01_JPRB, 0.23713E+01_JPRB, 0.23513E+01_JPRB, 0.23859E+01_JPRB, 0.24159E+01_JPRB ,&
&  0.24099E+01_JPRB, 0.24210E+01_JPRB, 0.24811E+01_JPRB, 0.26159E+01_JPRB, 0.28145E+01_JPRB, 0.29346E+01_JPRB, 0.30566E+01_JPRB ,&
&  0.31523E+01_JPRB, 0.33197E+01_JPRB, 0.33943E+01_JPRB, 0.34600E+01_JPRB, 0.34519E+01_JPRB, 0.35169E+01_JPRB, 0.36673E+01_JPRB ,&
&  0.37602E+01_JPRB, 0.38689E+01_JPRB, 0.40531E+01_JPRB, 0.43287E+01_JPRB, 0.44479E+01_JPRB, 0.39467E+01_JPRB, 0.38282E+01_JPRB ,&
&  0.40688E+01_JPRB, 0.43378E+01_JPRB, 0.41162E+01_JPRB, 0.53132E+01_JPRB, 0.35114E+01_JPRB, 0.39293E+01_JPRB, 0.40772E+01_JPRB ,&
&  0.40940E+01_JPRB, 0.44384E+01_JPRB, 0.47853E+01_JPRB, 0.50314E+01_JPRB, 0.47597E+01_JPRB, 0.43851E+01_JPRB, 0.52459E+01_JPRB ,&
&  0.47945E+01_JPRB, 0.34955E+01_JPRB, 0.25396E+01_JPRB, 0.17701E+01_JPRB, 0.22804E+01_JPRB, 0.40248E+01_JPRB, 0.31081E+01_JPRB ,&
&  0.21318E+01_JPRB, 0.21407E+01_JPRB, 0.23519E+01_JPRB, 0.26116E+01_JPRB, 0.20399E+01_JPRB, 0.14309E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.14311E+01_JPRB, 0.14373E+01_JPRB, 0.14621E+01_JPRB, 0.14894E+01_JPRB, 0.15346E+01_JPRB, 0.15934E+01_JPRB, 0.16444E+01_JPRB ,&
&  0.16457E+01_JPRB, 0.16353E+01_JPRB, 0.16624E+01_JPRB, 0.16557E+01_JPRB, 0.16145E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.12971E+01_JPRB, 0.12274E+01_JPRB, 0.11762E+01_JPRB, 0.11610E+01_JPRB, 0.11713E+01_JPRB, 0.12009E+01_JPRB, 0.12550E+01_JPRB ,&
&  0.13528E+01_JPRB, 0.14717E+01_JPRB, 0.15932E+01_JPRB, 0.17203E+01_JPRB, 0.18816E+01_JPRB, 0.20544E+01_JPRB, 0.22558E+01_JPRB ,&
&  0.24742E+01_JPRB, 0.27006E+01_JPRB, 0.27914E+01_JPRB, 0.27777E+01_JPRB, 0.27939E+01_JPRB, 0.28223E+01_JPRB, 0.30164E+01_JPRB ,&
&  0.34532E+01_JPRB, 0.38182E+01_JPRB, 0.39696E+01_JPRB, 0.36967E+01_JPRB, 0.29304E+01_JPRB, 0.29353E+01_JPRB, 0.31990E+01_JPRB ,&
&  0.32062E+01_JPRB, 0.27836E+01_JPRB, 0.24563E+01_JPRB, 0.24245E+01_JPRB, 0.24243E+01_JPRB, 0.24613E+01_JPRB, 0.24900E+01_JPRB ,&
&  0.24938E+01_JPRB, 0.25241E+01_JPRB, 0.26073E+01_JPRB, 0.27036E+01_JPRB, 0.28479E+01_JPRB, 0.29957E+01_JPRB, 0.31512E+01_JPRB ,&
&  0.32282E+01_JPRB, 0.33146E+01_JPRB, 0.32831E+01_JPRB, 0.31826E+01_JPRB, 0.32199E+01_JPRB, 0.34637E+01_JPRB, 0.38336E+01_JPRB ,&
&  0.40955E+01_JPRB, 0.41164E+01_JPRB, 0.44137E+01_JPRB, 0.47153E+01_JPRB, 0.46459E+01_JPRB, 0.43282E+01_JPRB, 0.42855E+01_JPRB ,&
&  0.44305E+01_JPRB, 0.48844E+01_JPRB, 0.37056E+01_JPRB, 0.44792E+01_JPRB, 0.46464E+01_JPRB, 0.41399E+01_JPRB, 0.36862E+01_JPRB ,&
&  0.34337E+01_JPRB, 0.32017E+01_JPRB, 0.28371E+01_JPRB, 0.30545E+01_JPRB, 0.32575E+01_JPRB, 0.36183E+01_JPRB, 0.43788E+01_JPRB ,&
&  0.41458E+01_JPRB, 0.30182E+01_JPRB, 0.20854E+01_JPRB, 0.16413E+01_JPRB, 0.20049E+01_JPRB, 0.28591E+01_JPRB, 0.21736E+01_JPRB ,&
&  0.17983E+01_JPRB, 0.19807E+01_JPRB, 0.20830E+01_JPRB, 0.24043E+01_JPRB, 0.22530E+01_JPRB, 0.13284E+01_JPRB, 0.13395E+01_JPRB ,&
&  0.13840E+01_JPRB, 0.14392E+01_JPRB, 0.14579E+01_JPRB, 0.14770E+01_JPRB, 0.14903E+01_JPRB, 0.15254E+01_JPRB, 0.15547E+01_JPRB ,&
&  0.15574E+01_JPRB, 0.15468E+01_JPRB, 0.15128E+01_JPRB, 0.14529E+01_JPRB, 0.13689E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.10142E+01_JPRB, 0.10054E+01_JPRB, 0.99621E+00_JPRB, 0.10234E+01_JPRB, 0.10541E+01_JPRB, 0.10693E+01_JPRB, 0.11039E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.11922E+01_JPRB, 0.12377E+01_JPRB, 0.12811E+01_JPRB, 0.14056E+01_JPRB, 0.15379E+01_JPRB, 0.16886E+01_JPRB ,&
&  0.17681E+01_JPRB, 0.18294E+01_JPRB, 0.18123E+01_JPRB, 0.17355E+01_JPRB, 0.16630E+01_JPRB, 0.15120E+01_JPRB, 0.13706E+01_JPRB ,&
&  0.14362E+01_JPRB, 0.18541E+01_JPRB, 0.25370E+01_JPRB, 0.32351E+01_JPRB, 0.38991E+01_JPRB, 0.35097E+01_JPRB, 0.39123E+01_JPRB ,&
&  0.32948E+01_JPRB, 0.24708E+01_JPRB, 0.23488E+01_JPRB, 0.23866E+01_JPRB, 0.24741E+01_JPRB, 0.24710E+01_JPRB, 0.25099E+01_JPRB ,&
&  0.25341E+01_JPRB, 0.25675E+01_JPRB, 0.26261E+01_JPRB, 0.27417E+01_JPRB, 0.28365E+01_JPRB, 0.28999E+01_JPRB, 0.29820E+01_JPRB ,&
&  0.30506E+01_JPRB, 0.30661E+01_JPRB, 0.30948E+01_JPRB, 0.28884E+01_JPRB, 0.30929E+01_JPRB, 0.35103E+01_JPRB, 0.38718E+01_JPRB ,&
&  0.40622E+01_JPRB, 0.41676E+01_JPRB, 0.45460E+01_JPRB, 0.47161E+01_JPRB, 0.48093E+01_JPRB, 0.47757E+01_JPRB, 0.48232E+01_JPRB ,&
&  0.48631E+01_JPRB, 0.44857E+01_JPRB, 0.33565E+01_JPRB, 0.37358E+01_JPRB, 0.34380E+01_JPRB, 0.28999E+01_JPRB, 0.30181E+01_JPRB ,&
&  0.27437E+01_JPRB, 0.24041E+01_JPRB, 0.19611E+01_JPRB, 0.19684E+01_JPRB, 0.22682E+01_JPRB, 0.26457E+01_JPRB, 0.35026E+01_JPRB ,&
&  0.29026E+01_JPRB, 0.22175E+01_JPRB, 0.16286E+01_JPRB, 0.15358E+01_JPRB, 0.16880E+01_JPRB, 0.19331E+01_JPRB, 0.19357E+01_JPRB ,&
&  0.19036E+01_JPRB, 0.18284E+01_JPRB, 0.19646E+01_JPRB, 0.22441E+01_JPRB, 0.14721E+01_JPRB, 0.11584E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.12374E+01_JPRB, 0.12412E+01_JPRB, 0.12326E+01_JPRB, 0.12462E+01_JPRB, 0.12746E+01_JPRB, 0.12761E+01_JPRB ,&
&  0.12451E+01_JPRB, 0.11732E+01_JPRB, 0.11047E+01_JPRB, 0.10590E+01_JPRB, 0.10262E+01_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.77037E+00_JPRB, 0.77498E+00_JPRB, 0.78370E+00_JPRB, 0.79956E+00_JPRB, 0.80723E+00_JPRB, 0.81197E+00_JPRB, 0.80365E+00_JPRB ,&
&  0.79890E+00_JPRB, 0.78930E+00_JPRB, 0.79061E+00_JPRB, 0.79931E+00_JPRB, 0.83907E+00_JPRB, 0.90834E+00_JPRB, 0.99507E+00_JPRB ,&
&  0.10191E+01_JPRB, 0.10119E+01_JPRB, 0.95607E+00_JPRB, 0.90464E+00_JPRB, 0.90851E+00_JPRB, 0.88811E+00_JPRB, 0.84506E+00_JPRB ,&
&  0.82104E+00_JPRB, 0.91116E+00_JPRB, 0.11581E+01_JPRB, 0.15549E+01_JPRB, 0.22706E+01_JPRB, 0.27762E+01_JPRB, 0.37169E+01_JPRB ,&
&  0.37056E+01_JPRB, 0.22053E+01_JPRB, 0.21654E+01_JPRB, 0.20423E+01_JPRB, 0.21508E+01_JPRB, 0.22123E+01_JPRB, 0.22294E+01_JPRB ,&
&  0.22849E+01_JPRB, 0.23432E+01_JPRB, 0.24012E+01_JPRB, 0.24608E+01_JPRB, 0.24974E+01_JPRB, 0.24828E+01_JPRB, 0.24745E+01_JPRB ,&
&  0.24707E+01_JPRB, 0.24369E+01_JPRB, 0.24794E+01_JPRB, 0.26320E+01_JPRB, 0.30373E+01_JPRB, 0.34937E+01_JPRB, 0.35656E+01_JPRB ,&
&  0.36195E+01_JPRB, 0.38858E+01_JPRB, 0.38423E+01_JPRB, 0.41242E+01_JPRB, 0.42445E+01_JPRB, 0.41460E+01_JPRB, 0.44409E+01_JPRB ,&
&  0.46200E+01_JPRB, 0.39192E+01_JPRB, 0.27004E+01_JPRB, 0.29371E+01_JPRB, 0.27964E+01_JPRB, 0.26134E+01_JPRB, 0.27143E+01_JPRB ,&
&  0.25130E+01_JPRB, 0.22700E+01_JPRB, 0.20226E+01_JPRB, 0.18870E+01_JPRB, 0.19284E+01_JPRB, 0.18736E+01_JPRB, 0.19940E+01_JPRB ,&
&  0.18293E+01_JPRB, 0.17802E+01_JPRB, 0.16701E+01_JPRB, 0.16479E+01_JPRB, 0.19413E+01_JPRB, 0.23889E+01_JPRB, 0.20352E+01_JPRB ,&
&  0.19701E+01_JPRB, 0.19904E+01_JPRB, 0.23328E+01_JPRB, 0.24904E+01_JPRB, 0.13129E+01_JPRB, 0.94842E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.89236E+00_JPRB, 0.88547E+00_JPRB, 0.87505E+00_JPRB, 0.85769E+00_JPRB, 0.89475E+00_JPRB, 0.96000E+00_JPRB, 0.93258E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.84905E+00_JPRB, 0.81623E+00_JPRB, 0.80385E+00_JPRB, 0.79432E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.70135E+00_JPRB, 0.69407E+00_JPRB, 0.68740E+00_JPRB, 0.68033E+00_JPRB, 0.69330E+00_JPRB, 0.70242E+00_JPRB, 0.72195E+00_JPRB ,&
&  0.74390E+00_JPRB, 0.75438E+00_JPRB, 0.77091E+00_JPRB, 0.78016E+00_JPRB, 0.80907E+00_JPRB, 0.83406E+00_JPRB, 0.87527E+00_JPRB ,&
&  0.91754E+00_JPRB, 0.94274E+00_JPRB, 0.96926E+00_JPRB, 0.99666E+00_JPRB, 0.10109E+01_JPRB, 0.10294E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.99943E+00_JPRB, 0.10010E+01_JPRB, 0.10425E+01_JPRB, 0.12180E+01_JPRB, 0.15506E+01_JPRB, 0.19787E+01_JPRB, 0.28587E+01_JPRB ,&
&  0.30680E+01_JPRB, 0.20079E+01_JPRB, 0.16664E+01_JPRB, 0.14696E+01_JPRB, 0.15220E+01_JPRB, 0.16090E+01_JPRB, 0.15903E+01_JPRB ,&
&  0.15912E+01_JPRB, 0.15896E+01_JPRB, 0.15755E+01_JPRB, 0.15388E+01_JPRB, 0.14515E+01_JPRB, 0.13474E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.12715E+01_JPRB, 0.12520E+01_JPRB, 0.14254E+01_JPRB, 0.18719E+01_JPRB, 0.22484E+01_JPRB, 0.28575E+01_JPRB, 0.27133E+01_JPRB ,&
&  0.27525E+01_JPRB, 0.28787E+01_JPRB, 0.28519E+01_JPRB, 0.30645E+01_JPRB, 0.31644E+01_JPRB, 0.32930E+01_JPRB, 0.33184E+01_JPRB ,&
&  0.35415E+01_JPRB, 0.31454E+01_JPRB, 0.23045E+01_JPRB, 0.25904E+01_JPRB, 0.25248E+01_JPRB, 0.25704E+01_JPRB, 0.24664E+01_JPRB ,&
&  0.23163E+01_JPRB, 0.21364E+01_JPRB, 0.19853E+01_JPRB, 0.18784E+01_JPRB, 0.18859E+01_JPRB, 0.18432E+01_JPRB, 0.16980E+01_JPRB ,&
&  0.16874E+01_JPRB, 0.17177E+01_JPRB, 0.16940E+01_JPRB, 0.16657E+01_JPRB, 0.20898E+01_JPRB, 0.23632E+01_JPRB, 0.30040E+01_JPRB ,&
&  0.20169E+01_JPRB, 0.18390E+01_JPRB, 0.13986E+01_JPRB, 0.17738E+01_JPRB, 0.10220E+01_JPRB, 0.73917E+00_JPRB, 0.72377E+00_JPRB ,&
&  0.70411E+00_JPRB, 0.67955E+00_JPRB, 0.65887E+00_JPRB, 0.65322E+00_JPRB, 0.66675E+00_JPRB, 0.70558E+00_JPRB, 0.69710E+00_JPRB ,&
&  0.69788E+00_JPRB, 0.69855E+00_JPRB, 0.70945E+00_JPRB, 0.72770E+00_JPRB, 0.71946E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.74342E+00_JPRB, 0.74919E+00_JPRB, 0.76487E+00_JPRB, 0.78947E+00_JPRB, 0.80610E+00_JPRB, 0.82162E+00_JPRB, 0.83658E+00_JPRB ,&
&  0.86757E+00_JPRB, 0.90180E+00_JPRB, 0.93292E+00_JPRB, 0.96334E+00_JPRB, 0.99504E+00_JPRB, 0.10207E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.11503E+01_JPRB, 0.11791E+01_JPRB, 0.12127E+01_JPRB, 0.12429E+01_JPRB, 0.12338E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.12688E+01_JPRB, 0.13179E+01_JPRB, 0.13607E+01_JPRB, 0.14436E+01_JPRB, 0.15512E+01_JPRB, 0.16070E+01_JPRB, 0.19320E+01_JPRB ,&
&  0.19772E+01_JPRB, 0.19518E+01_JPRB, 0.16081E+01_JPRB, 0.13731E+01_JPRB, 0.13026E+01_JPRB, 0.11644E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.10745E+01_JPRB, 0.10470E+01_JPRB, 0.99779E+00_JPRB, 0.93521E+00_JPRB, 0.98342E+00_JPRB, 0.10838E+01_JPRB ,&
&  0.12831E+01_JPRB, 0.13980E+01_JPRB, 0.15605E+01_JPRB, 0.17068E+01_JPRB, 0.18907E+01_JPRB, 0.20018E+01_JPRB, 0.19768E+01_JPRB ,&
&  0.19842E+01_JPRB, 0.19625E+01_JPRB, 0.18797E+01_JPRB, 0.21628E+01_JPRB, 0.23809E+01_JPRB, 0.24879E+01_JPRB, 0.25272E+01_JPRB ,&
&  0.25450E+01_JPRB, 0.23282E+01_JPRB, 0.22986E+01_JPRB, 0.22978E+01_JPRB, 0.22306E+01_JPRB, 0.22778E+01_JPRB, 0.21790E+01_JPRB ,&
&  0.20333E+01_JPRB, 0.19265E+01_JPRB, 0.18948E+01_JPRB, 0.19073E+01_JPRB, 0.19650E+01_JPRB, 0.19860E+01_JPRB, 0.18528E+01_JPRB ,&
&  0.17535E+01_JPRB, 0.17036E+01_JPRB, 0.16700E+01_JPRB, 0.16760E+01_JPRB, 0.18993E+01_JPRB, 0.40855E+01_JPRB, 0.51303E+01_JPRB ,&
&  0.19456E+01_JPRB, 0.13540E+01_JPRB, 0.77092E+00_JPRB, 0.93740E+00_JPRB, 0.76122E+00_JPRB, 0.70344E+00_JPRB, 0.77110E+00_JPRB ,&
&  0.78317E+00_JPRB, 0.76649E+00_JPRB, 0.69780E+00_JPRB, 0.63264E+00_JPRB, 0.65628E+00_JPRB, 0.67170E+00_JPRB, 0.67129E+00_JPRB ,&
&  0.68918E+00_JPRB, 0.69655E+00_JPRB, 0.70987E+00_JPRB, 0.73531E+00_JPRB, 0.73328E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.79298E+00_JPRB, 0.81465E+00_JPRB, 0.84171E+00_JPRB, 0.87687E+00_JPRB, 0.91167E+00_JPRB, 0.94074E+00_JPRB, 0.96282E+00_JPRB ,&
&  0.99596E+00_JPRB, 0.10393E+01_JPRB, 0.10689E+01_JPRB, 0.10855E+01_JPRB, 0.11190E+01_JPRB, 0.11474E+01_JPRB, 0.11958E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.12984E+01_JPRB, 0.13582E+01_JPRB, 0.14138E+01_JPRB, 0.14616E+01_JPRB, 0.15133E+01_JPRB, 0.15619E+01_JPRB ,&
&  0.16003E+01_JPRB, 0.16791E+01_JPRB, 0.17366E+01_JPRB, 0.17677E+01_JPRB, 0.18442E+01_JPRB, 0.20144E+01_JPRB, 0.17818E+01_JPRB ,&
&  0.21402E+01_JPRB, 0.18434E+01_JPRB, 0.15826E+01_JPRB, 0.14135E+01_JPRB, 0.12983E+01_JPRB, 0.12269E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.12177E+01_JPRB, 0.12851E+01_JPRB, 0.13436E+01_JPRB, 0.13147E+01_JPRB, 0.12269E+01_JPRB, 0.13456E+01_JPRB, 0.16113E+01_JPRB ,&
&  0.17606E+01_JPRB, 0.18439E+01_JPRB, 0.19086E+01_JPRB, 0.19282E+01_JPRB, 0.20107E+01_JPRB, 0.21292E+01_JPRB, 0.21866E+01_JPRB ,&
&  0.21826E+01_JPRB, 0.21153E+01_JPRB, 0.19008E+01_JPRB, 0.22858E+01_JPRB, 0.22844E+01_JPRB, 0.21955E+01_JPRB, 0.20145E+01_JPRB ,&
&  0.18379E+01_JPRB, 0.16907E+01_JPRB, 0.17963E+01_JPRB, 0.19792E+01_JPRB, 0.19420E+01_JPRB, 0.19123E+01_JPRB, 0.17940E+01_JPRB ,&
&  0.16688E+01_JPRB, 0.16453E+01_JPRB, 0.16848E+01_JPRB, 0.17746E+01_JPRB, 0.18565E+01_JPRB, 0.18669E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.16314E+01_JPRB, 0.15184E+01_JPRB, 0.14597E+01_JPRB, 0.14565E+01_JPRB, 0.16470E+01_JPRB, 0.20827E+01_JPRB, 0.30865E+01_JPRB ,&
&  0.16198E+01_JPRB, 0.90671E+00_JPRB, 0.89078E+00_JPRB, 0.78100E+00_JPRB, 0.78066E+00_JPRB, 0.71683E+00_JPRB, 0.86662E+00_JPRB ,&
&  0.10749E+01_JPRB, 0.12216E+01_JPRB, 0.11950E+01_JPRB, 0.93052E+00_JPRB, 0.77303E+00_JPRB, 0.72896E+00_JPRB, 0.72322E+00_JPRB ,&
&  0.73755E+00_JPRB, 0.74033E+00_JPRB, 0.74846E+00_JPRB, 0.76036E+00_JPRB, 0.77666E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.77130E+00_JPRB, 0.78751E+00_JPRB, 0.80759E+00_JPRB, 0.82922E+00_JPRB, 0.85547E+00_JPRB, 0.88281E+00_JPRB, 0.90326E+00_JPRB ,&
&  0.92740E+00_JPRB, 0.95622E+00_JPRB, 0.98390E+00_JPRB, 0.10047E+01_JPRB, 0.10299E+01_JPRB, 0.10726E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.11568E+01_JPRB, 0.12370E+01_JPRB, 0.12932E+01_JPRB, 0.13794E+01_JPRB, 0.14662E+01_JPRB, 0.15627E+01_JPRB, 0.16393E+01_JPRB ,&
&  0.17230E+01_JPRB, 0.18473E+01_JPRB, 0.19148E+01_JPRB, 0.19187E+01_JPRB, 0.19803E+01_JPRB, 0.22421E+01_JPRB, 0.19218E+01_JPRB ,&
&  0.21177E+01_JPRB, 0.19369E+01_JPRB, 0.17212E+01_JPRB, 0.16297E+01_JPRB, 0.15614E+01_JPRB, 0.15601E+01_JPRB, 0.15989E+01_JPRB ,&
&  0.16818E+01_JPRB, 0.18309E+01_JPRB, 0.17461E+01_JPRB, 0.17541E+01_JPRB, 0.17672E+01_JPRB, 0.19719E+01_JPRB, 0.21890E+01_JPRB ,&
&  0.22677E+01_JPRB, 0.24234E+01_JPRB, 0.24988E+01_JPRB, 0.25740E+01_JPRB, 0.26996E+01_JPRB, 0.28286E+01_JPRB, 0.29182E+01_JPRB ,&
&  0.28856E+01_JPRB, 0.28288E+01_JPRB, 0.26247E+01_JPRB, 0.28014E+01_JPRB, 0.26601E+01_JPRB, 0.22061E+01_JPRB, 0.17815E+01_JPRB ,&
&  0.13766E+01_JPRB, 0.14399E+01_JPRB, 0.15404E+01_JPRB, 0.16678E+01_JPRB, 0.15862E+01_JPRB, 0.15088E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.12431E+01_JPRB, 0.13234E+01_JPRB, 0.14412E+01_JPRB, 0.15134E+01_JPRB, 0.15069E+01_JPRB, 0.14123E+01_JPRB ,&
&  0.13226E+01_JPRB, 0.12229E+01_JPRB, 0.11999E+01_JPRB, 0.12436E+01_JPRB, 0.13028E+01_JPRB, 0.13346E+01_JPRB, 0.18377E+01_JPRB ,&
&  0.12737E+01_JPRB, 0.11018E+01_JPRB, 0.12648E+01_JPRB, 0.57042E+00_JPRB, 0.72943E+00_JPRB, 0.91305E+00_JPRB, 0.11846E+01_JPRB ,&
&  0.13798E+01_JPRB, 0.19004E+01_JPRB, 0.23208E+01_JPRB, 0.17677E+01_JPRB, 0.12558E+01_JPRB, 0.77955E+00_JPRB, 0.68907E+00_JPRB ,&
&  0.70506E+00_JPRB, 0.71348E+00_JPRB, 0.72653E+00_JPRB, 0.73141E+00_JPRB, 0.75056E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.69987E+00_JPRB, 0.69718E+00_JPRB, 0.69937E+00_JPRB, 0.69088E+00_JPRB, 0.68820E+00_JPRB, 0.69052E+00_JPRB, 0.68130E+00_JPRB ,&
&  0.68909E+00_JPRB, 0.70959E+00_JPRB, 0.71869E+00_JPRB, 0.74746E+00_JPRB, 0.77002E+00_JPRB, 0.83276E+00_JPRB, 0.92717E+00_JPRB ,&
&  0.10089E+01_JPRB, 0.11647E+01_JPRB, 0.12700E+01_JPRB, 0.13889E+01_JPRB, 0.14958E+01_JPRB, 0.16119E+01_JPRB, 0.17118E+01_JPRB ,&
&  0.18407E+01_JPRB, 0.20058E+01_JPRB, 0.21728E+01_JPRB, 0.22985E+01_JPRB, 0.23509E+01_JPRB, 0.27208E+01_JPRB, 0.21183E+01_JPRB ,&
&  0.22273E+01_JPRB, 0.21748E+01_JPRB, 0.20271E+01_JPRB, 0.19443E+01_JPRB, 0.18617E+01_JPRB, 0.18153E+01_JPRB, 0.17852E+01_JPRB ,&
&  0.18178E+01_JPRB, 0.18401E+01_JPRB, 0.18406E+01_JPRB, 0.19931E+01_JPRB, 0.21701E+01_JPRB, 0.23129E+01_JPRB, 0.24926E+01_JPRB ,&
&  0.26531E+01_JPRB, 0.28097E+01_JPRB, 0.30083E+01_JPRB, 0.32177E+01_JPRB, 0.34344E+01_JPRB, 0.35719E+01_JPRB, 0.37318E+01_JPRB ,&
&  0.38094E+01_JPRB, 0.38643E+01_JPRB, 0.36220E+01_JPRB, 0.31896E+01_JPRB, 0.27238E+01_JPRB, 0.22041E+01_JPRB, 0.16956E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.12768E+01_JPRB, 0.12880E+01_JPRB, 0.13030E+01_JPRB, 0.11744E+01_JPRB, 0.10876E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.98036E+00_JPRB, 0.91208E+00_JPRB, 0.96211E+00_JPRB, 0.10473E+01_JPRB, 0.10838E+01_JPRB, 0.10757E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.96910E+00_JPRB, 0.99772E+00_JPRB, 0.10549E+01_JPRB, 0.11663E+01_JPRB, 0.13431E+01_JPRB, 0.12010E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.87357E+00_JPRB, 0.82208E+00_JPRB, 0.80558E+00_JPRB, 0.76617E+00_JPRB, 0.97195E+00_JPRB, 0.13221E+01_JPRB ,&
&  0.19482E+01_JPRB, 0.21563E+01_JPRB, 0.23639E+01_JPRB, 0.29018E+01_JPRB, 0.25230E+01_JPRB, 0.18920E+01_JPRB, 0.62521E+00_JPRB ,&
&  0.62814E+00_JPRB, 0.63587E+00_JPRB, 0.66794E+00_JPRB, 0.67964E+00_JPRB, 0.68657E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.58750E+00_JPRB, 0.56943E+00_JPRB, 0.56120E+00_JPRB, 0.55791E+00_JPRB, 0.54220E+00_JPRB, 0.54483E+00_JPRB, 0.54443E+00_JPRB ,&
&  0.56580E+00_JPRB, 0.59648E+00_JPRB, 0.64892E+00_JPRB, 0.71678E+00_JPRB, 0.81138E+00_JPRB, 0.89637E+00_JPRB, 0.10055E+01_JPRB ,&
&  0.10884E+01_JPRB, 0.11909E+01_JPRB, 0.12927E+01_JPRB, 0.13985E+01_JPRB, 0.15157E+01_JPRB, 0.16224E+01_JPRB, 0.17627E+01_JPRB ,&
&  0.18928E+01_JPRB, 0.20462E+01_JPRB, 0.22347E+01_JPRB, 0.24016E+01_JPRB, 0.25306E+01_JPRB, 0.26271E+01_JPRB, 0.20157E+01_JPRB ,&
&  0.23438E+01_JPRB, 0.23929E+01_JPRB, 0.23895E+01_JPRB, 0.22845E+01_JPRB, 0.21243E+01_JPRB, 0.19767E+01_JPRB, 0.19357E+01_JPRB ,&
&  0.19241E+01_JPRB, 0.18957E+01_JPRB, 0.18541E+01_JPRB, 0.18758E+01_JPRB, 0.22457E+01_JPRB, 0.23778E+01_JPRB, 0.25520E+01_JPRB ,&
&  0.27352E+01_JPRB, 0.29412E+01_JPRB, 0.31537E+01_JPRB, 0.33741E+01_JPRB, 0.35274E+01_JPRB, 0.36985E+01_JPRB, 0.38784E+01_JPRB ,&
&  0.40131E+01_JPRB, 0.41281E+01_JPRB, 0.39636E+01_JPRB, 0.31290E+01_JPRB, 0.26124E+01_JPRB, 0.21695E+01_JPRB, 0.15108E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.11337E+01_JPRB, 0.11435E+01_JPRB, 0.11225E+01_JPRB, 0.10140E+01_JPRB, 0.92713E+00_JPRB, 0.86191E+00_JPRB ,&
&  0.76461E+00_JPRB, 0.73987E+00_JPRB, 0.73442E+00_JPRB, 0.79400E+00_JPRB, 0.82714E+00_JPRB, 0.79070E+00_JPRB, 0.77752E+00_JPRB ,&
&  0.74839E+00_JPRB, 0.72909E+00_JPRB, 0.79696E+00_JPRB, 0.91197E+00_JPRB, 0.11568E+01_JPRB, 0.14297E+01_JPRB, 0.16287E+01_JPRB ,&
&  0.16093E+01_JPRB, 0.88375E+00_JPRB, 0.78129E+00_JPRB, 0.74450E+00_JPRB, 0.74834E+00_JPRB, 0.80085E+00_JPRB, 0.89579E+00_JPRB ,&
&  0.11172E+01_JPRB, 0.15073E+01_JPRB, 0.13091E+01_JPRB, 0.10053E+01_JPRB, 0.10418E+01_JPRB, 0.96526E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.57582E+00_JPRB, 0.57183E+00_JPRB, 0.59025E+00_JPRB, 0.59695E+00_JPRB, 0.60094E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.51805E+00_JPRB, 0.49593E+00_JPRB, 0.47430E+00_JPRB, 0.46549E+00_JPRB, 0.45949E+00_JPRB, 0.45325E+00_JPRB, 0.46948E+00_JPRB ,&
&  0.50852E+00_JPRB, 0.54693E+00_JPRB, 0.61112E+00_JPRB, 0.71028E+00_JPRB, 0.81858E+00_JPRB, 0.89809E+00_JPRB, 0.98138E+00_JPRB ,&
&  0.10585E+01_JPRB, 0.11378E+01_JPRB, 0.12046E+01_JPRB, 0.12731E+01_JPRB, 0.13634E+01_JPRB, 0.14800E+01_JPRB, 0.16403E+01_JPRB ,&
&  0.18287E+01_JPRB, 0.20141E+01_JPRB, 0.22388E+01_JPRB, 0.25037E+01_JPRB, 0.27595E+01_JPRB, 0.28701E+01_JPRB, 0.30808E+01_JPRB ,&
&  0.20524E+01_JPRB, 0.25655E+01_JPRB, 0.25538E+01_JPRB, 0.24616E+01_JPRB, 0.23412E+01_JPRB, 0.22271E+01_JPRB, 0.21497E+01_JPRB ,&
&  0.20924E+01_JPRB, 0.19123E+01_JPRB, 0.18521E+01_JPRB, 0.18321E+01_JPRB, 0.20404E+01_JPRB, 0.22084E+01_JPRB, 0.23774E+01_JPRB ,&
&  0.25531E+01_JPRB, 0.27550E+01_JPRB, 0.29668E+01_JPRB, 0.31722E+01_JPRB, 0.33377E+01_JPRB, 0.35246E+01_JPRB, 0.36634E+01_JPRB ,&
&  0.38317E+01_JPRB, 0.39728E+01_JPRB, 0.40190E+01_JPRB, 0.33022E+01_JPRB, 0.26608E+01_JPRB, 0.22999E+01_JPRB, 0.17146E+01_JPRB ,&
&  0.12269E+01_JPRB, 0.10306E+01_JPRB, 0.10544E+01_JPRB, 0.10805E+01_JPRB, 0.10039E+01_JPRB, 0.93243E+00_JPRB, 0.84224E+00_JPRB ,&
&  0.77004E+00_JPRB, 0.68580E+00_JPRB, 0.65013E+00_JPRB, 0.66878E+00_JPRB, 0.72019E+00_JPRB, 0.68182E+00_JPRB, 0.63500E+00_JPRB ,&
&  0.60633E+00_JPRB, 0.60084E+00_JPRB, 0.67399E+00_JPRB, 0.76286E+00_JPRB, 0.88484E+00_JPRB, 0.95368E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.83369E+00_JPRB, 0.79106E+00_JPRB, 0.78938E+00_JPRB, 0.77536E+00_JPRB, 0.77591E+00_JPRB, 0.79200E+00_JPRB ,&
&  0.82780E+00_JPRB, 0.89681E+00_JPRB, 0.91154E+00_JPRB, 0.84845E+00_JPRB, 0.75870E+00_JPRB, 0.70207E+00_JPRB, 0.65398E+00_JPRB ,&
&  0.65627E+00_JPRB, 0.64212E+00_JPRB, 0.62613E+00_JPRB, 0.59580E+00_JPRB, 0.55885E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.53293E+00_JPRB, 0.49567E+00_JPRB, 0.44818E+00_JPRB, 0.41002E+00_JPRB, 0.43840E+00_JPRB, 0.44432E+00_JPRB, 0.46750E+00_JPRB ,&
&  0.48067E+00_JPRB, 0.51769E+00_JPRB, 0.56012E+00_JPRB, 0.62232E+00_JPRB, 0.71039E+00_JPRB, 0.80199E+00_JPRB, 0.88291E+00_JPRB ,&
&  0.95725E+00_JPRB, 0.10274E+01_JPRB, 0.10801E+01_JPRB, 0.11074E+01_JPRB, 0.11519E+01_JPRB, 0.12241E+01_JPRB, 0.13190E+01_JPRB ,&
&  0.14524E+01_JPRB, 0.16239E+01_JPRB, 0.18514E+01_JPRB, 0.21370E+01_JPRB, 0.24635E+01_JPRB, 0.26587E+01_JPRB, 0.28133E+01_JPRB ,&
&  0.14880E+01_JPRB, 0.23227E+01_JPRB, 0.27681E+01_JPRB, 0.25662E+01_JPRB, 0.24789E+01_JPRB, 0.23933E+01_JPRB, 0.23063E+01_JPRB ,&
&  0.21096E+01_JPRB, 0.18620E+01_JPRB, 0.17586E+01_JPRB, 0.19150E+01_JPRB, 0.18049E+01_JPRB, 0.19647E+01_JPRB, 0.20858E+01_JPRB ,&
&  0.22150E+01_JPRB, 0.23524E+01_JPRB, 0.25149E+01_JPRB, 0.27263E+01_JPRB, 0.29470E+01_JPRB, 0.31332E+01_JPRB, 0.32934E+01_JPRB ,&
&  0.35093E+01_JPRB, 0.37045E+01_JPRB, 0.37669E+01_JPRB, 0.29329E+01_JPRB, 0.26691E+01_JPRB, 0.25028E+01_JPRB, 0.22349E+01_JPRB ,&
&  0.14170E+01_JPRB, 0.10518E+01_JPRB, 0.10597E+01_JPRB, 0.11009E+01_JPRB, 0.10454E+01_JPRB, 0.98698E+00_JPRB, 0.91973E+00_JPRB ,&
&  0.82560E+00_JPRB, 0.75219E+00_JPRB, 0.69490E+00_JPRB, 0.67010E+00_JPRB, 0.69349E+00_JPRB, 0.69086E+00_JPRB, 0.65971E+00_JPRB ,&
&  0.62598E+00_JPRB, 0.63110E+00_JPRB, 0.68284E+00_JPRB, 0.72900E+00_JPRB, 0.79867E+00_JPRB, 0.83799E+00_JPRB, 0.89176E+00_JPRB ,&
&  0.90115E+00_JPRB, 0.81259E+00_JPRB, 0.80175E+00_JPRB, 0.81300E+00_JPRB, 0.81762E+00_JPRB, 0.82194E+00_JPRB, 0.82123E+00_JPRB ,&
&  0.81737E+00_JPRB, 0.83663E+00_JPRB, 0.84359E+00_JPRB, 0.84851E+00_JPRB, 0.82149E+00_JPRB, 0.79107E+00_JPRB, 0.77019E+00_JPRB ,&
&  0.74232E+00_JPRB, 0.70842E+00_JPRB, 0.67979E+00_JPRB, 0.62981E+00_JPRB, 0.57846E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.56179E+00_JPRB, 0.51723E+00_JPRB, 0.47086E+00_JPRB, 0.45429E+00_JPRB, 0.48541E+00_JPRB, 0.51470E+00_JPRB, 0.51418E+00_JPRB ,&
&  0.51083E+00_JPRB, 0.50258E+00_JPRB, 0.50345E+00_JPRB, 0.52400E+00_JPRB, 0.58390E+00_JPRB, 0.67559E+00_JPRB, 0.75559E+00_JPRB ,&
&  0.82515E+00_JPRB, 0.87679E+00_JPRB, 0.91620E+00_JPRB, 0.92814E+00_JPRB, 0.91803E+00_JPRB, 0.94196E+00_JPRB, 0.98951E+00_JPRB ,&
&  0.10747E+01_JPRB, 0.11899E+01_JPRB, 0.13453E+01_JPRB, 0.15793E+01_JPRB, 0.19281E+01_JPRB, 0.23566E+01_JPRB, 0.27964E+01_JPRB ,&
&  0.19588E+01_JPRB, 0.83287E+00_JPRB, 0.25344E+01_JPRB, 0.29997E+01_JPRB, 0.28016E+01_JPRB, 0.27562E+01_JPRB, 0.24934E+01_JPRB ,&
&  0.22399E+01_JPRB, 0.21596E+01_JPRB, 0.19440E+01_JPRB, 0.19684E+01_JPRB, 0.17763E+01_JPRB, 0.17779E+01_JPRB, 0.17919E+01_JPRB ,&
&  0.18357E+01_JPRB, 0.19006E+01_JPRB, 0.20092E+01_JPRB, 0.21469E+01_JPRB, 0.23106E+01_JPRB, 0.25760E+01_JPRB, 0.27690E+01_JPRB ,&
&  0.29442E+01_JPRB, 0.31099E+01_JPRB, 0.31507E+01_JPRB, 0.23272E+01_JPRB, 0.19243E+01_JPRB, 0.17195E+01_JPRB, 0.15668E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.11336E+01_JPRB, 0.11041E+01_JPRB, 0.11399E+01_JPRB, 0.11069E+01_JPRB, 0.10242E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.91054E+00_JPRB, 0.83436E+00_JPRB, 0.77049E+00_JPRB, 0.72021E+00_JPRB, 0.72104E+00_JPRB, 0.72772E+00_JPRB, 0.73105E+00_JPRB ,&
&  0.71141E+00_JPRB, 0.70720E+00_JPRB, 0.72958E+00_JPRB, 0.73338E+00_JPRB, 0.74147E+00_JPRB, 0.75087E+00_JPRB, 0.76431E+00_JPRB ,&
&  0.81191E+00_JPRB, 0.80349E+00_JPRB, 0.81062E+00_JPRB, 0.83160E+00_JPRB, 0.84353E+00_JPRB, 0.83016E+00_JPRB, 0.83722E+00_JPRB ,&
&  0.85158E+00_JPRB, 0.86711E+00_JPRB, 0.85905E+00_JPRB, 0.87117E+00_JPRB, 0.85789E+00_JPRB, 0.84700E+00_JPRB, 0.84529E+00_JPRB ,&
&  0.84232E+00_JPRB, 0.82292E+00_JPRB, 0.77502E+00_JPRB, 0.67962E+00_JPRB, 0.61835E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.61184E+00_JPRB, 0.53889E+00_JPRB, 0.49703E+00_JPRB, 0.50147E+00_JPRB, 0.53101E+00_JPRB, 0.54350E+00_JPRB, 0.54778E+00_JPRB ,&
&  0.53927E+00_JPRB, 0.52642E+00_JPRB, 0.50608E+00_JPRB, 0.49837E+00_JPRB, 0.51029E+00_JPRB, 0.53699E+00_JPRB, 0.60011E+00_JPRB ,&
&  0.66246E+00_JPRB, 0.72164E+00_JPRB, 0.76668E+00_JPRB, 0.76402E+00_JPRB, 0.72427E+00_JPRB, 0.71606E+00_JPRB, 0.73627E+00_JPRB ,&
&  0.77197E+00_JPRB, 0.83889E+00_JPRB, 0.93364E+00_JPRB, 0.10672E+01_JPRB, 0.12841E+01_JPRB, 0.16627E+01_JPRB, 0.22663E+01_JPRB ,&
&  0.33905E+01_JPRB, 0.22847E+01_JPRB, 0.13751E+01_JPRB, 0.27681E+01_JPRB, 0.33318E+01_JPRB, 0.34862E+01_JPRB, 0.28572E+01_JPRB ,&
&  0.27370E+01_JPRB, 0.25254E+01_JPRB, 0.22697E+01_JPRB, 0.21875E+01_JPRB, 0.18932E+01_JPRB, 0.17805E+01_JPRB, 0.16790E+01_JPRB ,&
&  0.16015E+01_JPRB, 0.16228E+01_JPRB, 0.16871E+01_JPRB, 0.17500E+01_JPRB, 0.18177E+01_JPRB, 0.19886E+01_JPRB, 0.22141E+01_JPRB ,&
&  0.24444E+01_JPRB, 0.25862E+01_JPRB, 0.26326E+01_JPRB, 0.20161E+01_JPRB, 0.17205E+01_JPRB, 0.15800E+01_JPRB, 0.15045E+01_JPRB ,&
&  0.13553E+01_JPRB, 0.12748E+01_JPRB, 0.12550E+01_JPRB, 0.12094E+01_JPRB, 0.11506E+01_JPRB, 0.10929E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.93422E+00_JPRB, 0.87564E+00_JPRB, 0.83833E+00_JPRB, 0.82394E+00_JPRB, 0.81933E+00_JPRB, 0.80719E+00_JPRB ,&
&  0.81036E+00_JPRB, 0.80930E+00_JPRB, 0.81028E+00_JPRB, 0.78819E+00_JPRB, 0.76426E+00_JPRB, 0.75030E+00_JPRB, 0.73236E+00_JPRB ,&
&  0.76046E+00_JPRB, 0.79373E+00_JPRB, 0.81258E+00_JPRB, 0.84112E+00_JPRB, 0.84462E+00_JPRB, 0.83957E+00_JPRB, 0.86255E+00_JPRB ,&
&  0.86891E+00_JPRB, 0.88691E+00_JPRB, 0.87521E+00_JPRB, 0.88469E+00_JPRB, 0.86824E+00_JPRB, 0.85334E+00_JPRB, 0.85414E+00_JPRB ,&
&  0.88817E+00_JPRB, 0.92434E+00_JPRB, 0.89183E+00_JPRB, 0.72086E+00_JPRB, 0.65277E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.59613E+00_JPRB, 0.56278E+00_JPRB, 0.52057E+00_JPRB, 0.52843E+00_JPRB, 0.53605E+00_JPRB, 0.54838E+00_JPRB, 0.55445E+00_JPRB ,&
&  0.55887E+00_JPRB, 0.56124E+00_JPRB, 0.56422E+00_JPRB, 0.55232E+00_JPRB, 0.52194E+00_JPRB, 0.51021E+00_JPRB, 0.51366E+00_JPRB ,&
&  0.53332E+00_JPRB, 0.57632E+00_JPRB, 0.62469E+00_JPRB, 0.63349E+00_JPRB, 0.62400E+00_JPRB, 0.61219E+00_JPRB, 0.60746E+00_JPRB ,&
&  0.61597E+00_JPRB, 0.63605E+00_JPRB, 0.66835E+00_JPRB, 0.73507E+00_JPRB, 0.85479E+00_JPRB, 0.10483E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.23238E+01_JPRB, 0.25346E+01_JPRB, 0.10710E+01_JPRB, 0.22452E+01_JPRB, 0.34654E+01_JPRB, 0.37329E+01_JPRB, 0.35651E+01_JPRB ,&
&  0.33659E+01_JPRB, 0.29821E+01_JPRB, 0.26532E+01_JPRB, 0.23126E+01_JPRB, 0.19416E+01_JPRB, 0.17426E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.14518E+01_JPRB, 0.14349E+01_JPRB, 0.14905E+01_JPRB, 0.15341E+01_JPRB, 0.15403E+01_JPRB, 0.15883E+01_JPRB, 0.17476E+01_JPRB ,&
&  0.19955E+01_JPRB, 0.22548E+01_JPRB, 0.23706E+01_JPRB, 0.19169E+01_JPRB, 0.17102E+01_JPRB, 0.17171E+01_JPRB, 0.16592E+01_JPRB ,&
&  0.15493E+01_JPRB, 0.14295E+01_JPRB, 0.13425E+01_JPRB, 0.12766E+01_JPRB, 0.12008E+01_JPRB, 0.11720E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.11223E+01_JPRB, 0.10345E+01_JPRB, 0.97284E+00_JPRB, 0.93281E+00_JPRB, 0.90608E+00_JPRB, 0.89047E+00_JPRB, 0.87510E+00_JPRB ,&
&  0.86603E+00_JPRB, 0.87320E+00_JPRB, 0.88423E+00_JPRB, 0.87017E+00_JPRB, 0.83809E+00_JPRB, 0.80162E+00_JPRB, 0.76724E+00_JPRB ,&
&  0.76407E+00_JPRB, 0.77842E+00_JPRB, 0.81251E+00_JPRB, 0.82999E+00_JPRB, 0.84289E+00_JPRB, 0.85927E+00_JPRB, 0.87098E+00_JPRB ,&
&  0.88657E+00_JPRB, 0.90459E+00_JPRB, 0.87396E+00_JPRB, 0.85929E+00_JPRB, 0.87459E+00_JPRB, 0.84950E+00_JPRB, 0.82586E+00_JPRB ,&
&  0.85734E+00_JPRB, 0.92242E+00_JPRB, 0.10078E+01_JPRB, 0.72559E+00_JPRB, 0.65001E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.59311E+00_JPRB, 0.55318E+00_JPRB, 0.54626E+00_JPRB, 0.54707E+00_JPRB, 0.55406E+00_JPRB, 0.56746E+00_JPRB, 0.58284E+00_JPRB ,&
&  0.58862E+00_JPRB, 0.59016E+00_JPRB, 0.59019E+00_JPRB, 0.58495E+00_JPRB, 0.56858E+00_JPRB, 0.55149E+00_JPRB, 0.53414E+00_JPRB ,&
&  0.52797E+00_JPRB, 0.53342E+00_JPRB, 0.55374E+00_JPRB, 0.56364E+00_JPRB, 0.56378E+00_JPRB, 0.55975E+00_JPRB, 0.56061E+00_JPRB ,&
&  0.56619E+00_JPRB, 0.56884E+00_JPRB, 0.57175E+00_JPRB, 0.59698E+00_JPRB, 0.65205E+00_JPRB, 0.76148E+00_JPRB, 0.97786E+00_JPRB ,&
&  0.16096E+01_JPRB, 0.21608E+01_JPRB, 0.20843E+01_JPRB, 0.34402E+01_JPRB, 0.42451E+01_JPRB, 0.42063E+01_JPRB, 0.41465E+01_JPRB ,&
&  0.39034E+01_JPRB, 0.34646E+01_JPRB, 0.32105E+01_JPRB, 0.25279E+01_JPRB, 0.20633E+01_JPRB, 0.17363E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.12289E+01_JPRB, 0.12508E+01_JPRB, 0.13182E+01_JPRB, 0.13643E+01_JPRB, 0.13808E+01_JPRB, 0.14451E+01_JPRB ,&
&  0.16162E+01_JPRB, 0.19367E+01_JPRB, 0.21993E+01_JPRB, 0.20393E+01_JPRB, 0.17029E+01_JPRB, 0.18505E+01_JPRB, 0.20092E+01_JPRB ,&
&  0.22353E+01_JPRB, 0.17355E+01_JPRB, 0.14519E+01_JPRB, 0.13401E+01_JPRB, 0.12432E+01_JPRB, 0.13210E+01_JPRB, 0.12747E+01_JPRB ,&
&  0.12037E+01_JPRB, 0.11253E+01_JPRB, 0.10537E+01_JPRB, 0.10058E+01_JPRB, 0.96840E+00_JPRB, 0.94095E+00_JPRB, 0.92190E+00_JPRB ,&
&  0.91128E+00_JPRB, 0.90649E+00_JPRB, 0.91576E+00_JPRB, 0.91620E+00_JPRB, 0.90220E+00_JPRB, 0.87129E+00_JPRB, 0.82901E+00_JPRB ,&
&  0.79575E+00_JPRB, 0.78934E+00_JPRB, 0.78106E+00_JPRB, 0.79935E+00_JPRB, 0.82457E+00_JPRB, 0.83761E+00_JPRB, 0.83272E+00_JPRB ,&
&  0.84483E+00_JPRB, 0.86828E+00_JPRB, 0.86997E+00_JPRB, 0.86342E+00_JPRB, 0.86106E+00_JPRB, 0.85468E+00_JPRB, 0.80713E+00_JPRB ,&
&  0.78284E+00_JPRB, 0.76769E+00_JPRB, 0.72475E+00_JPRB, 0.68672E+00_JPRB, 0.63505E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.60170E+00_JPRB, 0.57730E+00_JPRB, 0.57088E+00_JPRB, 0.57564E+00_JPRB, 0.58200E+00_JPRB, 0.59045E+00_JPRB, 0.60492E+00_JPRB ,&
&  0.61259E+00_JPRB, 0.61425E+00_JPRB, 0.61285E+00_JPRB, 0.60966E+00_JPRB, 0.59837E+00_JPRB, 0.58465E+00_JPRB, 0.56921E+00_JPRB ,&
&  0.55507E+00_JPRB, 0.53025E+00_JPRB, 0.51807E+00_JPRB, 0.52493E+00_JPRB, 0.52779E+00_JPRB, 0.52865E+00_JPRB, 0.53105E+00_JPRB ,&
&  0.53197E+00_JPRB, 0.53782E+00_JPRB, 0.53818E+00_JPRB, 0.54871E+00_JPRB, 0.57494E+00_JPRB, 0.64574E+00_JPRB, 0.77929E+00_JPRB ,&
&  0.11933E+01_JPRB, 0.19541E+01_JPRB, 0.16965E+01_JPRB, 0.39630E+01_JPRB, 0.44886E+01_JPRB, 0.44365E+01_JPRB, 0.40119E+01_JPRB ,&
&  0.37049E+01_JPRB, 0.38232E+01_JPRB, 0.29356E+01_JPRB, 0.23536E+01_JPRB, 0.20284E+01_JPRB, 0.17459E+01_JPRB, 0.14519E+01_JPRB ,&
&  0.12238E+01_JPRB, 0.11011E+01_JPRB, 0.10557E+01_JPRB, 0.10814E+01_JPRB, 0.11534E+01_JPRB, 0.12191E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.16068E+01_JPRB, 0.19455E+01_JPRB, 0.19046E+01_JPRB, 0.17921E+01_JPRB, 0.20037E+01_JPRB, 0.24209E+01_JPRB ,&
&  0.28701E+01_JPRB, 0.23636E+01_JPRB, 0.16904E+01_JPRB, 0.14346E+01_JPRB, 0.13630E+01_JPRB, 0.14063E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.12997E+01_JPRB, 0.12244E+01_JPRB, 0.11354E+01_JPRB, 0.10737E+01_JPRB, 0.10294E+01_JPRB, 0.97804E+00_JPRB, 0.94371E+00_JPRB ,&
&  0.92762E+00_JPRB, 0.91963E+00_JPRB, 0.91562E+00_JPRB, 0.90688E+00_JPRB, 0.90312E+00_JPRB, 0.89729E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.85112E+00_JPRB, 0.81357E+00_JPRB, 0.77297E+00_JPRB, 0.76715E+00_JPRB, 0.78879E+00_JPRB, 0.80049E+00_JPRB, 0.80370E+00_JPRB ,&
&  0.83576E+00_JPRB, 0.85206E+00_JPRB, 0.84630E+00_JPRB, 0.82640E+00_JPRB, 0.83814E+00_JPRB, 0.87592E+00_JPRB, 0.81415E+00_JPRB ,&
&  0.79058E+00_JPRB, 0.76569E+00_JPRB, 0.73584E+00_JPRB, 0.70267E+00_JPRB, 0.65252E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.64126E+00_JPRB, 0.62240E+00_JPRB, 0.61106E+00_JPRB, 0.60822E+00_JPRB, 0.60326E+00_JPRB, 0.60470E+00_JPRB, 0.61481E+00_JPRB ,&
&  0.63088E+00_JPRB, 0.63625E+00_JPRB, 0.63435E+00_JPRB, 0.63133E+00_JPRB, 0.62381E+00_JPRB, 0.61097E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.58224E+00_JPRB, 0.56312E+00_JPRB, 0.53727E+00_JPRB, 0.51686E+00_JPRB, 0.51802E+00_JPRB, 0.52296E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.52618E+00_JPRB, 0.53368E+00_JPRB, 0.53690E+00_JPRB, 0.54766E+00_JPRB, 0.55769E+00_JPRB, 0.59794E+00_JPRB, 0.72255E+00_JPRB ,&
&  0.10454E+01_JPRB, 0.23638E+01_JPRB, 0.18281E+01_JPRB, 0.36920E+01_JPRB, 0.42113E+01_JPRB, 0.40985E+01_JPRB, 0.36395E+01_JPRB ,&
&  0.33630E+01_JPRB, 0.31507E+01_JPRB, 0.25863E+01_JPRB, 0.21102E+01_JPRB, 0.18089E+01_JPRB, 0.15805E+01_JPRB, 0.13873E+01_JPRB ,&
&  0.12377E+01_JPRB, 0.11401E+01_JPRB, 0.10788E+01_JPRB, 0.10387E+01_JPRB, 0.10427E+01_JPRB, 0.10796E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.13357E+01_JPRB, 0.16264E+01_JPRB, 0.19094E+01_JPRB, 0.17865E+01_JPRB, 0.20108E+01_JPRB, 0.28218E+01_JPRB ,&
&  0.32684E+01_JPRB, 0.32251E+01_JPRB, 0.22066E+01_JPRB, 0.16549E+01_JPRB, 0.14824E+01_JPRB, 0.15012E+01_JPRB, 0.14928E+01_JPRB ,&
&  0.14163E+01_JPRB, 0.13519E+01_JPRB, 0.12533E+01_JPRB, 0.11483E+01_JPRB, 0.10707E+01_JPRB, 0.10134E+01_JPRB, 0.95663E+00_JPRB ,&
&  0.91705E+00_JPRB, 0.90077E+00_JPRB, 0.88825E+00_JPRB, 0.87317E+00_JPRB, 0.86476E+00_JPRB, 0.86661E+00_JPRB, 0.87047E+00_JPRB ,&
&  0.86940E+00_JPRB, 0.84958E+00_JPRB, 0.79943E+00_JPRB, 0.76421E+00_JPRB, 0.76038E+00_JPRB, 0.76868E+00_JPRB, 0.77289E+00_JPRB ,&
&  0.81518E+00_JPRB, 0.83051E+00_JPRB, 0.81943E+00_JPRB, 0.82117E+00_JPRB, 0.82551E+00_JPRB, 0.90426E+00_JPRB, 0.82739E+00_JPRB ,&
&  0.79261E+00_JPRB, 0.76679E+00_JPRB, 0.75331E+00_JPRB, 0.73037E+00_JPRB, 0.68543E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.67496E+00_JPRB, 0.65236E+00_JPRB, 0.64093E+00_JPRB, 0.62655E+00_JPRB, 0.61515E+00_JPRB, 0.61174E+00_JPRB, 0.61910E+00_JPRB ,&
&  0.63684E+00_JPRB, 0.65093E+00_JPRB, 0.64490E+00_JPRB, 0.64130E+00_JPRB, 0.63837E+00_JPRB, 0.62802E+00_JPRB, 0.61378E+00_JPRB ,&
&  0.59953E+00_JPRB, 0.58640E+00_JPRB, 0.56896E+00_JPRB, 0.54785E+00_JPRB, 0.53303E+00_JPRB, 0.53190E+00_JPRB, 0.52981E+00_JPRB ,&
&  0.52879E+00_JPRB, 0.53387E+00_JPRB, 0.53824E+00_JPRB, 0.54588E+00_JPRB, 0.55437E+00_JPRB, 0.57788E+00_JPRB, 0.69259E+00_JPRB ,&
&  0.10752E+01_JPRB, 0.82482E+00_JPRB, 0.23099E+01_JPRB, 0.32371E+01_JPRB, 0.34086E+01_JPRB, 0.31847E+01_JPRB, 0.30642E+01_JPRB ,&
&  0.28955E+01_JPRB, 0.25856E+01_JPRB, 0.22938E+01_JPRB, 0.20332E+01_JPRB, 0.17075E+01_JPRB, 0.14915E+01_JPRB, 0.13610E+01_JPRB ,&
&  0.12644E+01_JPRB, 0.11812E+01_JPRB, 0.11108E+01_JPRB, 0.10399E+01_JPRB, 0.98790E+00_JPRB, 0.97435E+00_JPRB, 0.10049E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.11316E+01_JPRB, 0.13409E+01_JPRB, 0.16248E+01_JPRB, 0.17731E+01_JPRB, 0.19616E+01_JPRB, 0.23718E+01_JPRB ,&
&  0.30338E+01_JPRB, 0.33351E+01_JPRB, 0.26127E+01_JPRB, 0.20178E+01_JPRB, 0.16901E+01_JPRB, 0.16118E+01_JPRB, 0.16101E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.14521E+01_JPRB, 0.13412E+01_JPRB, 0.12168E+01_JPRB, 0.11106E+01_JPRB, 0.10394E+01_JPRB, 0.98625E+00_JPRB ,&
&  0.93632E+00_JPRB, 0.89519E+00_JPRB, 0.86804E+00_JPRB, 0.84901E+00_JPRB, 0.83682E+00_JPRB, 0.83204E+00_JPRB, 0.83661E+00_JPRB ,&
&  0.85047E+00_JPRB, 0.85358E+00_JPRB, 0.83205E+00_JPRB, 0.79690E+00_JPRB, 0.78397E+00_JPRB, 0.78756E+00_JPRB, 0.79521E+00_JPRB ,&
&  0.82264E+00_JPRB, 0.82807E+00_JPRB, 0.81813E+00_JPRB, 0.82411E+00_JPRB, 0.80170E+00_JPRB, 0.84170E+00_JPRB, 0.80998E+00_JPRB ,&
&  0.77958E+00_JPRB, 0.76384E+00_JPRB, 0.76128E+00_JPRB, 0.74606E+00_JPRB, 0.71196E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.69938E+00_JPRB, 0.67671E+00_JPRB, 0.66066E+00_JPRB, 0.63731E+00_JPRB, 0.62489E+00_JPRB, 0.62182E+00_JPRB, 0.62890E+00_JPRB ,&
&  0.63634E+00_JPRB, 0.64232E+00_JPRB, 0.63678E+00_JPRB, 0.63092E+00_JPRB, 0.62295E+00_JPRB, 0.61653E+00_JPRB, 0.60719E+00_JPRB ,&
&  0.60159E+00_JPRB, 0.59682E+00_JPRB, 0.58802E+00_JPRB, 0.57520E+00_JPRB, 0.55885E+00_JPRB, 0.55029E+00_JPRB, 0.54186E+00_JPRB ,&
&  0.53461E+00_JPRB, 0.53134E+00_JPRB, 0.53208E+00_JPRB, 0.54090E+00_JPRB, 0.55337E+00_JPRB, 0.57381E+00_JPRB, 0.64896E+00_JPRB ,&
&  0.10984E+01_JPRB, 0.16723E+01_JPRB, 0.27080E+01_JPRB, 0.28430E+01_JPRB, 0.25700E+01_JPRB, 0.22289E+01_JPRB, 0.21838E+01_JPRB ,&
&  0.20992E+01_JPRB, 0.19853E+01_JPRB, 0.18386E+01_JPRB, 0.17347E+01_JPRB, 0.16225E+01_JPRB, 0.15458E+01_JPRB, 0.14274E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.12039E+01_JPRB, 0.10822E+01_JPRB, 0.98243E+00_JPRB, 0.91511E+00_JPRB, 0.88357E+00_JPRB, 0.88838E+00_JPRB ,&
&  0.91035E+00_JPRB, 0.94430E+00_JPRB, 0.10688E+01_JPRB, 0.13245E+01_JPRB, 0.16990E+01_JPRB, 0.17667E+01_JPRB, 0.22827E+01_JPRB ,&
&  0.29920E+01_JPRB, 0.29495E+01_JPRB, 0.26512E+01_JPRB, 0.23653E+01_JPRB, 0.20261E+01_JPRB, 0.18882E+01_JPRB, 0.18491E+01_JPRB ,&
&  0.17332E+01_JPRB, 0.15497E+01_JPRB, 0.13622E+01_JPRB, 0.12198E+01_JPRB, 0.11213E+01_JPRB, 0.10613E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.99565E+00_JPRB, 0.95268E+00_JPRB, 0.90709E+00_JPRB, 0.86984E+00_JPRB, 0.84708E+00_JPRB, 0.83714E+00_JPRB, 0.83500E+00_JPRB ,&
&  0.83849E+00_JPRB, 0.84130E+00_JPRB, 0.84701E+00_JPRB, 0.82478E+00_JPRB, 0.83354E+00_JPRB, 0.85818E+00_JPRB, 0.86630E+00_JPRB ,&
&  0.85261E+00_JPRB, 0.84737E+00_JPRB, 0.83508E+00_JPRB, 0.82700E+00_JPRB, 0.83980E+00_JPRB, 0.85559E+00_JPRB, 0.81096E+00_JPRB ,&
&  0.76499E+00_JPRB, 0.75019E+00_JPRB, 0.75986E+00_JPRB, 0.75484E+00_JPRB, 0.73900E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.72179E+00_JPRB, 0.69621E+00_JPRB, 0.67300E+00_JPRB, 0.65121E+00_JPRB, 0.63395E+00_JPRB, 0.62731E+00_JPRB, 0.62812E+00_JPRB ,&
&  0.63049E+00_JPRB, 0.62782E+00_JPRB, 0.62139E+00_JPRB, 0.61214E+00_JPRB, 0.60538E+00_JPRB, 0.59910E+00_JPRB, 0.59362E+00_JPRB ,&
&  0.58585E+00_JPRB, 0.58709E+00_JPRB, 0.58347E+00_JPRB, 0.57532E+00_JPRB, 0.56563E+00_JPRB, 0.55380E+00_JPRB, 0.54252E+00_JPRB ,&
&  0.53481E+00_JPRB, 0.53134E+00_JPRB, 0.53080E+00_JPRB, 0.53520E+00_JPRB, 0.54902E+00_JPRB, 0.56704E+00_JPRB, 0.60682E+00_JPRB ,&
&  0.93707E+00_JPRB, 0.15975E+01_JPRB, 0.20464E+01_JPRB, 0.23546E+01_JPRB, 0.20973E+01_JPRB, 0.17776E+01_JPRB, 0.15911E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.15015E+01_JPRB, 0.14330E+01_JPRB, 0.13443E+01_JPRB, 0.12369E+01_JPRB, 0.11816E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.99044E+00_JPRB, 0.93467E+00_JPRB, 0.90119E+00_JPRB, 0.87120E+00_JPRB, 0.83864E+00_JPRB, 0.81802E+00_JPRB ,&
&  0.81641E+00_JPRB, 0.82539E+00_JPRB, 0.86957E+00_JPRB, 0.10298E+01_JPRB, 0.13692E+01_JPRB, 0.16346E+01_JPRB, 0.19684E+01_JPRB ,&
&  0.20557E+01_JPRB, 0.21362E+01_JPRB, 0.21954E+01_JPRB, 0.21659E+01_JPRB, 0.20303E+01_JPRB, 0.19393E+01_JPRB, 0.19217E+01_JPRB ,&
&  0.18190E+01_JPRB, 0.15716E+01_JPRB, 0.13322E+01_JPRB, 0.11837E+01_JPRB, 0.11090E+01_JPRB, 0.10731E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10028E+01_JPRB, 0.96638E+00_JPRB, 0.92331E+00_JPRB, 0.88394E+00_JPRB, 0.85264E+00_JPRB, 0.83480E+00_JPRB ,&
&  0.82184E+00_JPRB, 0.81662E+00_JPRB, 0.82904E+00_JPRB, 0.84456E+00_JPRB, 0.85875E+00_JPRB, 0.87234E+00_JPRB, 0.88213E+00_JPRB ,&
&  0.89055E+00_JPRB, 0.89397E+00_JPRB, 0.84969E+00_JPRB, 0.83627E+00_JPRB, 0.83482E+00_JPRB, 0.82954E+00_JPRB, 0.78335E+00_JPRB ,&
&  0.74361E+00_JPRB, 0.72746E+00_JPRB, 0.73523E+00_JPRB, 0.74187E+00_JPRB, 0.74379E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.79900E+00_JPRB, 0.74560E+00_JPRB, 0.69983E+00_JPRB, 0.66407E+00_JPRB, 0.64323E+00_JPRB, 0.63098E+00_JPRB, 0.62189E+00_JPRB ,&
&  0.61636E+00_JPRB, 0.60965E+00_JPRB, 0.59784E+00_JPRB, 0.58875E+00_JPRB, 0.58223E+00_JPRB, 0.57869E+00_JPRB, 0.57513E+00_JPRB ,&
&  0.56460E+00_JPRB, 0.56294E+00_JPRB, 0.56336E+00_JPRB, 0.55870E+00_JPRB, 0.55200E+00_JPRB, 0.54412E+00_JPRB, 0.53371E+00_JPRB ,&
&  0.52610E+00_JPRB, 0.52333E+00_JPRB, 0.52467E+00_JPRB, 0.52890E+00_JPRB, 0.53395E+00_JPRB, 0.54809E+00_JPRB, 0.59039E+00_JPRB ,&
&  0.87038E+00_JPRB, 0.12427E+01_JPRB, 0.15503E+01_JPRB, 0.16936E+01_JPRB, 0.14449E+01_JPRB, 0.13044E+01_JPRB, 0.12276E+01_JPRB ,&
&  0.12166E+01_JPRB, 0.12317E+01_JPRB, 0.11630E+01_JPRB, 0.10488E+01_JPRB, 0.94287E+00_JPRB, 0.88487E+00_JPRB, 0.84586E+00_JPRB ,&
&  0.81102E+00_JPRB, 0.78271E+00_JPRB, 0.75752E+00_JPRB, 0.75266E+00_JPRB, 0.74945E+00_JPRB, 0.74111E+00_JPRB, 0.73288E+00_JPRB ,&
&  0.72945E+00_JPRB, 0.73289E+00_JPRB, 0.74658E+00_JPRB, 0.81128E+00_JPRB, 0.98366E+00_JPRB, 0.11722E+01_JPRB, 0.13556E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.15270E+01_JPRB, 0.16332E+01_JPRB, 0.17114E+01_JPRB, 0.17153E+01_JPRB, 0.17038E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.16446E+01_JPRB, 0.14958E+01_JPRB, 0.12992E+01_JPRB, 0.11536E+01_JPRB, 0.10742E+01_JPRB, 0.10339E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.98655E+00_JPRB, 0.96516E+00_JPRB, 0.93481E+00_JPRB, 0.89437E+00_JPRB, 0.85133E+00_JPRB, 0.82091E+00_JPRB, 0.79864E+00_JPRB ,&
&  0.78536E+00_JPRB, 0.78774E+00_JPRB, 0.80520E+00_JPRB, 0.83461E+00_JPRB, 0.85544E+00_JPRB, 0.86634E+00_JPRB, 0.86626E+00_JPRB ,&
&  0.86344E+00_JPRB, 0.87683E+00_JPRB, 0.83045E+00_JPRB, 0.83674E+00_JPRB, 0.87635E+00_JPRB, 0.82016E+00_JPRB, 0.76283E+00_JPRB ,&
&  0.72703E+00_JPRB, 0.70264E+00_JPRB, 0.69611E+00_JPRB, 0.71459E+00_JPRB, 0.82551E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.73881E+00_JPRB, 0.72016E+00_JPRB, 0.68975E+00_JPRB, 0.66072E+00_JPRB, 0.64138E+00_JPRB, 0.62912E+00_JPRB, 0.62128E+00_JPRB ,&
&  0.61279E+00_JPRB, 0.60218E+00_JPRB, 0.58953E+00_JPRB, 0.57412E+00_JPRB, 0.56416E+00_JPRB, 0.55922E+00_JPRB, 0.55577E+00_JPRB ,&
&  0.55006E+00_JPRB, 0.53994E+00_JPRB, 0.53684E+00_JPRB, 0.53401E+00_JPRB, 0.53264E+00_JPRB, 0.53044E+00_JPRB, 0.52463E+00_JPRB ,&
&  0.51744E+00_JPRB, 0.51383E+00_JPRB, 0.51656E+00_JPRB, 0.51798E+00_JPRB, 0.52025E+00_JPRB, 0.52811E+00_JPRB, 0.55720E+00_JPRB ,&
&  0.73644E+00_JPRB, 0.93759E+00_JPRB, 0.11524E+01_JPRB, 0.13275E+01_JPRB, 0.12240E+01_JPRB, 0.11270E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.99960E+00_JPRB, 0.95589E+00_JPRB, 0.88674E+00_JPRB, 0.80852E+00_JPRB, 0.74274E+00_JPRB, 0.71600E+00_JPRB, 0.70039E+00_JPRB ,&
&  0.68758E+00_JPRB, 0.67659E+00_JPRB, 0.66784E+00_JPRB, 0.66340E+00_JPRB, 0.66344E+00_JPRB, 0.66128E+00_JPRB, 0.65881E+00_JPRB ,&
&  0.65944E+00_JPRB, 0.66496E+00_JPRB, 0.67278E+00_JPRB, 0.69231E+00_JPRB, 0.76066E+00_JPRB, 0.88098E+00_JPRB, 0.10090E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.11551E+01_JPRB, 0.12531E+01_JPRB, 0.13326E+01_JPRB, 0.13793E+01_JPRB, 0.13926E+01_JPRB, 0.13790E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.13032E+01_JPRB, 0.11903E+01_JPRB, 0.10699E+01_JPRB, 0.99062E+00_JPRB, 0.93959E+00_JPRB, 0.90240E+00_JPRB ,&
&  0.87391E+00_JPRB, 0.84041E+00_JPRB, 0.80745E+00_JPRB, 0.77989E+00_JPRB, 0.76159E+00_JPRB, 0.74495E+00_JPRB, 0.73175E+00_JPRB ,&
&  0.73531E+00_JPRB, 0.75500E+00_JPRB, 0.78367E+00_JPRB, 0.80279E+00_JPRB, 0.81035E+00_JPRB, 0.81484E+00_JPRB, 0.81145E+00_JPRB ,&
&  0.80865E+00_JPRB, 0.81677E+00_JPRB, 0.81393E+00_JPRB, 0.83348E+00_JPRB, 0.81647E+00_JPRB, 0.78733E+00_JPRB, 0.74057E+00_JPRB ,&
&  0.70668E+00_JPRB, 0.68404E+00_JPRB, 0.66542E+00_JPRB, 0.67602E+00_JPRB, 0.72620E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.67041E+00_JPRB, 0.66745E+00_JPRB, 0.66051E+00_JPRB, 0.64856E+00_JPRB, 0.64053E+00_JPRB, 0.63241E+00_JPRB, 0.62095E+00_JPRB ,&
&  0.60846E+00_JPRB, 0.59738E+00_JPRB, 0.58641E+00_JPRB, 0.57271E+00_JPRB, 0.55813E+00_JPRB, 0.55244E+00_JPRB, 0.55069E+00_JPRB ,&
&  0.54608E+00_JPRB, 0.53739E+00_JPRB, 0.52929E+00_JPRB, 0.52725E+00_JPRB, 0.52625E+00_JPRB, 0.52533E+00_JPRB, 0.52141E+00_JPRB ,&
&  0.51532E+00_JPRB, 0.51397E+00_JPRB, 0.51673E+00_JPRB, 0.51675E+00_JPRB, 0.51229E+00_JPRB, 0.51444E+00_JPRB, 0.53948E+00_JPRB ,&
&  0.66648E+00_JPRB, 0.68990E+00_JPRB, 0.82488E+00_JPRB, 0.10364E+01_JPRB, 0.95916E+00_JPRB, 0.91805E+00_JPRB, 0.84274E+00_JPRB ,&
&  0.76704E+00_JPRB, 0.72397E+00_JPRB, 0.69486E+00_JPRB, 0.66432E+00_JPRB, 0.63572E+00_JPRB, 0.62033E+00_JPRB, 0.62020E+00_JPRB ,&
&  0.62841E+00_JPRB, 0.63193E+00_JPRB, 0.63148E+00_JPRB, 0.62292E+00_JPRB, 0.61658E+00_JPRB, 0.61114E+00_JPRB, 0.60751E+00_JPRB ,&
&  0.60509E+00_JPRB, 0.60832E+00_JPRB, 0.61467E+00_JPRB, 0.62141E+00_JPRB, 0.64305E+00_JPRB, 0.70487E+00_JPRB, 0.80284E+00_JPRB ,&
&  0.89279E+00_JPRB, 0.96965E+00_JPRB, 0.10366E+01_JPRB, 0.11017E+01_JPRB, 0.11467E+01_JPRB, 0.11842E+01_JPRB, 0.11987E+01_JPRB ,&
&  0.11738E+01_JPRB, 0.11196E+01_JPRB, 0.10519E+01_JPRB, 0.97511E+00_JPRB, 0.90489E+00_JPRB, 0.84868E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.77132E+00_JPRB, 0.74849E+00_JPRB, 0.73078E+00_JPRB, 0.71238E+00_JPRB, 0.70201E+00_JPRB, 0.70088E+00_JPRB, 0.70683E+00_JPRB ,&
&  0.71949E+00_JPRB, 0.73388E+00_JPRB, 0.74305E+00_JPRB, 0.74522E+00_JPRB, 0.74113E+00_JPRB, 0.73813E+00_JPRB, 0.73572E+00_JPRB ,&
&  0.74175E+00_JPRB, 0.75645E+00_JPRB, 0.76763E+00_JPRB, 0.78739E+00_JPRB, 0.80810E+00_JPRB, 0.79281E+00_JPRB, 0.75558E+00_JPRB ,&
&  0.71615E+00_JPRB, 0.68313E+00_JPRB, 0.66401E+00_JPRB, 0.59620E+00_JPRB, 0.67042E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.63170E+00_JPRB, 0.63185E+00_JPRB, 0.63008E+00_JPRB, 0.62732E+00_JPRB, 0.62366E+00_JPRB, 0.61613E+00_JPRB, 0.60425E+00_JPRB ,&
&  0.59206E+00_JPRB, 0.58046E+00_JPRB, 0.57175E+00_JPRB, 0.56290E+00_JPRB, 0.55006E+00_JPRB, 0.54157E+00_JPRB, 0.53975E+00_JPRB ,&
&  0.53765E+00_JPRB, 0.53171E+00_JPRB, 0.52479E+00_JPRB, 0.52337E+00_JPRB, 0.52089E+00_JPRB, 0.51730E+00_JPRB, 0.51226E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.50530E+00_JPRB, 0.50519E+00_JPRB, 0.50554E+00_JPRB, 0.50185E+00_JPRB, 0.50420E+00_JPRB, 0.53047E+00_JPRB ,&
&  0.63232E+00_JPRB, 0.58693E+00_JPRB, 0.69710E+00_JPRB, 0.81365E+00_JPRB, 0.74342E+00_JPRB, 0.68915E+00_JPRB, 0.64497E+00_JPRB ,&
&  0.61048E+00_JPRB, 0.59007E+00_JPRB, 0.57866E+00_JPRB, 0.56819E+00_JPRB, 0.56019E+00_JPRB, 0.55336E+00_JPRB, 0.55385E+00_JPRB ,&
&  0.55833E+00_JPRB, 0.56968E+00_JPRB, 0.57697E+00_JPRB, 0.57727E+00_JPRB, 0.57155E+00_JPRB, 0.56477E+00_JPRB, 0.55855E+00_JPRB ,&
&  0.55455E+00_JPRB, 0.55434E+00_JPRB, 0.55704E+00_JPRB, 0.56306E+00_JPRB, 0.57630E+00_JPRB, 0.60623E+00_JPRB, 0.66278E+00_JPRB ,&
&  0.73561E+00_JPRB, 0.79730E+00_JPRB, 0.85299E+00_JPRB, 0.91680E+00_JPRB, 0.97604E+00_JPRB, 0.10154E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.95128E+00_JPRB, 0.90736E+00_JPRB, 0.86363E+00_JPRB, 0.82160E+00_JPRB, 0.78654E+00_JPRB, 0.76197E+00_JPRB ,&
&  0.74575E+00_JPRB, 0.73473E+00_JPRB, 0.72809E+00_JPRB, 0.72306E+00_JPRB, 0.72268E+00_JPRB, 0.72380E+00_JPRB, 0.72670E+00_JPRB ,&
&  0.72373E+00_JPRB, 0.71698E+00_JPRB, 0.70004E+00_JPRB, 0.67746E+00_JPRB, 0.66280E+00_JPRB, 0.65641E+00_JPRB, 0.65548E+00_JPRB ,&
&  0.66096E+00_JPRB, 0.67398E+00_JPRB, 0.68911E+00_JPRB, 0.72804E+00_JPRB, 0.74313E+00_JPRB, 0.75070E+00_JPRB, 0.74191E+00_JPRB ,&
&  0.71567E+00_JPRB, 0.68275E+00_JPRB, 0.56694E+00_JPRB, 0.62273E+00_JPRB, 0.63562E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.63098E+00_JPRB, 0.62351E+00_JPRB, 0.61671E+00_JPRB, 0.60987E+00_JPRB, 0.60149E+00_JPRB, 0.59151E+00_JPRB, 0.58017E+00_JPRB ,&
&  0.56959E+00_JPRB, 0.55991E+00_JPRB, 0.55040E+00_JPRB, 0.54232E+00_JPRB, 0.53473E+00_JPRB, 0.53097E+00_JPRB, 0.52992E+00_JPRB ,&
&  0.53040E+00_JPRB, 0.52950E+00_JPRB, 0.52724E+00_JPRB, 0.52672E+00_JPRB, 0.52593E+00_JPRB, 0.52277E+00_JPRB, 0.51827E+00_JPRB ,&
&  0.51135E+00_JPRB, 0.50530E+00_JPRB, 0.50256E+00_JPRB, 0.50416E+00_JPRB, 0.50258E+00_JPRB, 0.50639E+00_JPRB, 0.52289E+00_JPRB ,&
&  0.52756E+00_JPRB, 0.52662E+00_JPRB, 0.59356E+00_JPRB, 0.67769E+00_JPRB, 0.62779E+00_JPRB, 0.57951E+00_JPRB, 0.55929E+00_JPRB ,&
&  0.54301E+00_JPRB, 0.53187E+00_JPRB, 0.52412E+00_JPRB, 0.51966E+00_JPRB, 0.51939E+00_JPRB, 0.51931E+00_JPRB, 0.51988E+00_JPRB ,&
&  0.52207E+00_JPRB, 0.52782E+00_JPRB, 0.53394E+00_JPRB, 0.53795E+00_JPRB, 0.53715E+00_JPRB, 0.53355E+00_JPRB, 0.52754E+00_JPRB ,&
&  0.52197E+00_JPRB, 0.51823E+00_JPRB, 0.51754E+00_JPRB, 0.52165E+00_JPRB, 0.53020E+00_JPRB, 0.54621E+00_JPRB, 0.57609E+00_JPRB ,&
&  0.62074E+00_JPRB, 0.66728E+00_JPRB, 0.70956E+00_JPRB, 0.75587E+00_JPRB, 0.81172E+00_JPRB, 0.86317E+00_JPRB, 0.89367E+00_JPRB ,&
&  0.88521E+00_JPRB, 0.84849E+00_JPRB, 0.80460E+00_JPRB, 0.76787E+00_JPRB, 0.74021E+00_JPRB, 0.72223E+00_JPRB, 0.71140E+00_JPRB ,&
&  0.70720E+00_JPRB, 0.70475E+00_JPRB, 0.70323E+00_JPRB, 0.70249E+00_JPRB, 0.70133E+00_JPRB, 0.69660E+00_JPRB, 0.68862E+00_JPRB ,&
&  0.67523E+00_JPRB, 0.65480E+00_JPRB, 0.63179E+00_JPRB, 0.61324E+00_JPRB, 0.60079E+00_JPRB, 0.59571E+00_JPRB, 0.59637E+00_JPRB ,&
&  0.60301E+00_JPRB, 0.61801E+00_JPRB, 0.63664E+00_JPRB, 0.67367E+00_JPRB, 0.69968E+00_JPRB, 0.71716E+00_JPRB, 0.72444E+00_JPRB ,&
&  0.71341E+00_JPRB, 0.68833E+00_JPRB, 0.66310E+00_JPRB, 0.64892E+00_JPRB, 0.64205E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.62044E+00_JPRB, 0.60503E+00_JPRB, 0.58953E+00_JPRB, 0.57866E+00_JPRB, 0.57019E+00_JPRB, 0.56197E+00_JPRB, 0.55305E+00_JPRB ,&
&  0.54285E+00_JPRB, 0.53293E+00_JPRB, 0.52383E+00_JPRB, 0.51548E+00_JPRB, 0.51121E+00_JPRB, 0.50913E+00_JPRB, 0.50956E+00_JPRB ,&
&  0.51239E+00_JPRB, 0.51730E+00_JPRB, 0.51887E+00_JPRB, 0.51799E+00_JPRB, 0.51844E+00_JPRB, 0.51788E+00_JPRB, 0.51535E+00_JPRB ,&
&  0.51004E+00_JPRB, 0.50376E+00_JPRB, 0.49988E+00_JPRB, 0.49976E+00_JPRB, 0.49962E+00_JPRB, 0.50199E+00_JPRB, 0.50664E+00_JPRB ,&
&  0.52331E+00_JPRB, 0.49532E+00_JPRB, 0.54126E+00_JPRB, 0.59591E+00_JPRB, 0.57947E+00_JPRB, 0.55519E+00_JPRB, 0.53499E+00_JPRB ,&
&  0.51889E+00_JPRB, 0.50976E+00_JPRB, 0.50466E+00_JPRB, 0.50477E+00_JPRB, 0.50785E+00_JPRB, 0.51015E+00_JPRB, 0.51184E+00_JPRB ,&
&  0.51042E+00_JPRB, 0.51118E+00_JPRB, 0.50974E+00_JPRB, 0.50855E+00_JPRB, 0.50850E+00_JPRB, 0.50933E+00_JPRB, 0.50907E+00_JPRB ,&
&  0.50645E+00_JPRB, 0.50199E+00_JPRB, 0.49762E+00_JPRB, 0.49652E+00_JPRB, 0.49795E+00_JPRB, 0.50285E+00_JPRB, 0.51383E+00_JPRB ,&
&  0.53294E+00_JPRB, 0.55714E+00_JPRB, 0.58548E+00_JPRB, 0.62054E+00_JPRB, 0.66817E+00_JPRB, 0.71626E+00_JPRB, 0.75374E+00_JPRB ,&
&  0.76507E+00_JPRB, 0.75568E+00_JPRB, 0.73354E+00_JPRB, 0.70646E+00_JPRB, 0.68159E+00_JPRB, 0.65810E+00_JPRB, 0.63745E+00_JPRB ,&
&  0.62224E+00_JPRB, 0.61040E+00_JPRB, 0.60200E+00_JPRB, 0.59712E+00_JPRB, 0.59245E+00_JPRB, 0.58569E+00_JPRB, 0.57904E+00_JPRB ,&
&  0.57185E+00_JPRB, 0.56517E+00_JPRB, 0.55983E+00_JPRB, 0.55480E+00_JPRB, 0.54954E+00_JPRB, 0.54540E+00_JPRB, 0.54511E+00_JPRB ,&
&  0.55004E+00_JPRB, 0.56034E+00_JPRB, 0.57802E+00_JPRB, 0.60370E+00_JPRB, 0.63060E+00_JPRB, 0.65205E+00_JPRB, 0.66516E+00_JPRB ,&
&  0.66760E+00_JPRB, 0.66409E+00_JPRB, 0.65373E+00_JPRB, 0.64294E+00_JPRB, 0.63360E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.59126E+00_JPRB, 0.57926E+00_JPRB, 0.56695E+00_JPRB, 0.55703E+00_JPRB, 0.54904E+00_JPRB, 0.54053E+00_JPRB, 0.53114E+00_JPRB ,&
&  0.52350E+00_JPRB, 0.51522E+00_JPRB, 0.50906E+00_JPRB, 0.50377E+00_JPRB, 0.49986E+00_JPRB, 0.49858E+00_JPRB, 0.49935E+00_JPRB ,&
&  0.50214E+00_JPRB, 0.50803E+00_JPRB, 0.51073E+00_JPRB, 0.50876E+00_JPRB, 0.51026E+00_JPRB, 0.51231E+00_JPRB, 0.51232E+00_JPRB ,&
&  0.50905E+00_JPRB, 0.50488E+00_JPRB, 0.50038E+00_JPRB, 0.49748E+00_JPRB, 0.49720E+00_JPRB, 0.49947E+00_JPRB, 0.49869E+00_JPRB ,&
&  0.49973E+00_JPRB, 0.50838E+00_JPRB, 0.53072E+00_JPRB, 0.55085E+00_JPRB, 0.55089E+00_JPRB, 0.53792E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.50617E+00_JPRB, 0.49738E+00_JPRB, 0.49591E+00_JPRB, 0.49768E+00_JPRB, 0.50086E+00_JPRB, 0.50309E+00_JPRB, 0.50610E+00_JPRB ,&
&  0.50387E+00_JPRB, 0.50227E+00_JPRB, 0.49869E+00_JPRB, 0.49508E+00_JPRB, 0.49451E+00_JPRB, 0.49536E+00_JPRB, 0.49642E+00_JPRB ,&
&  0.49594E+00_JPRB, 0.49665E+00_JPRB, 0.49689E+00_JPRB, 0.49529E+00_JPRB, 0.49618E+00_JPRB, 0.49829E+00_JPRB, 0.50192E+00_JPRB ,&
&  0.50844E+00_JPRB, 0.51845E+00_JPRB, 0.53080E+00_JPRB, 0.54916E+00_JPRB, 0.57877E+00_JPRB, 0.61229E+00_JPRB, 0.63928E+00_JPRB ,&
&  0.65354E+00_JPRB, 0.65512E+00_JPRB, 0.64786E+00_JPRB, 0.63409E+00_JPRB, 0.62050E+00_JPRB, 0.60725E+00_JPRB, 0.59202E+00_JPRB ,&
&  0.57566E+00_JPRB, 0.56334E+00_JPRB, 0.55382E+00_JPRB, 0.54677E+00_JPRB, 0.54039E+00_JPRB, 0.53536E+00_JPRB, 0.53276E+00_JPRB ,&
&  0.53248E+00_JPRB, 0.53266E+00_JPRB, 0.53127E+00_JPRB, 0.52914E+00_JPRB, 0.52773E+00_JPRB, 0.52454E+00_JPRB, 0.52273E+00_JPRB ,&
&  0.52469E+00_JPRB, 0.53057E+00_JPRB, 0.54137E+00_JPRB, 0.55997E+00_JPRB, 0.57755E+00_JPRB, 0.59226E+00_JPRB, 0.60375E+00_JPRB ,&
&  0.61175E+00_JPRB, 0.61553E+00_JPRB, 0.61432E+00_JPRB, 0.60994E+00_JPRB, 0.60240E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.54438E+00_JPRB, 0.53864E+00_JPRB, 0.53165E+00_JPRB, 0.52461E+00_JPRB, 0.51795E+00_JPRB, 0.51138E+00_JPRB, 0.50529E+00_JPRB ,&
&  0.49999E+00_JPRB, 0.49669E+00_JPRB, 0.49464E+00_JPRB, 0.49307E+00_JPRB, 0.48964E+00_JPRB, 0.48811E+00_JPRB, 0.48885E+00_JPRB ,&
&  0.49143E+00_JPRB, 0.49408E+00_JPRB, 0.49559E+00_JPRB, 0.49463E+00_JPRB, 0.49664E+00_JPRB, 0.49843E+00_JPRB, 0.49821E+00_JPRB ,&
&  0.49677E+00_JPRB, 0.49511E+00_JPRB, 0.49207E+00_JPRB, 0.48871E+00_JPRB, 0.48854E+00_JPRB, 0.48972E+00_JPRB, 0.48740E+00_JPRB ,&
&  0.48213E+00_JPRB, 0.49483E+00_JPRB, 0.48836E+00_JPRB, 0.51337E+00_JPRB, 0.52123E+00_JPRB, 0.51416E+00_JPRB, 0.50307E+00_JPRB ,&
&  0.49529E+00_JPRB, 0.49212E+00_JPRB, 0.49379E+00_JPRB, 0.49603E+00_JPRB, 0.49762E+00_JPRB, 0.49871E+00_JPRB, 0.49872E+00_JPRB ,&
&  0.49003E+00_JPRB, 0.48360E+00_JPRB, 0.48108E+00_JPRB, 0.48038E+00_JPRB, 0.47960E+00_JPRB, 0.47986E+00_JPRB, 0.48178E+00_JPRB ,&
&  0.48356E+00_JPRB, 0.48486E+00_JPRB, 0.48609E+00_JPRB, 0.48844E+00_JPRB, 0.48982E+00_JPRB, 0.49036E+00_JPRB, 0.49216E+00_JPRB ,&
&  0.49539E+00_JPRB, 0.50010E+00_JPRB, 0.50528E+00_JPRB, 0.51128E+00_JPRB, 0.52420E+00_JPRB, 0.54331E+00_JPRB, 0.56022E+00_JPRB ,&
&  0.56950E+00_JPRB, 0.57250E+00_JPRB, 0.57096E+00_JPRB, 0.56412E+00_JPRB, 0.55376E+00_JPRB, 0.54468E+00_JPRB, 0.53664E+00_JPRB ,&
&  0.52653E+00_JPRB, 0.51984E+00_JPRB, 0.51423E+00_JPRB, 0.50763E+00_JPRB, 0.50191E+00_JPRB, 0.49821E+00_JPRB, 0.49641E+00_JPRB ,&
&  0.49672E+00_JPRB, 0.49902E+00_JPRB, 0.50009E+00_JPRB, 0.50162E+00_JPRB, 0.50459E+00_JPRB, 0.50764E+00_JPRB, 0.50706E+00_JPRB ,&
&  0.50521E+00_JPRB, 0.50471E+00_JPRB, 0.50698E+00_JPRB, 0.51319E+00_JPRB, 0.52089E+00_JPRB, 0.52927E+00_JPRB, 0.53767E+00_JPRB ,&
&  0.54493E+00_JPRB, 0.54985E+00_JPRB, 0.55168E+00_JPRB, 0.55187E+00_JPRB, 0.54913E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.50165E+00_JPRB, 0.49995E+00_JPRB, 0.49765E+00_JPRB, 0.49399E+00_JPRB, 0.48995E+00_JPRB, 0.48630E+00_JPRB, 0.48327E+00_JPRB ,&
&  0.48130E+00_JPRB, 0.48079E+00_JPRB, 0.48136E+00_JPRB, 0.48161E+00_JPRB, 0.48152E+00_JPRB, 0.48238E+00_JPRB, 0.48276E+00_JPRB ,&
&  0.48437E+00_JPRB, 0.48455E+00_JPRB, 0.48172E+00_JPRB, 0.48083E+00_JPRB, 0.48319E+00_JPRB, 0.48783E+00_JPRB, 0.48926E+00_JPRB ,&
&  0.48654E+00_JPRB, 0.48574E+00_JPRB, 0.48487E+00_JPRB, 0.48213E+00_JPRB, 0.48039E+00_JPRB, 0.47999E+00_JPRB, 0.47705E+00_JPRB ,&
&  0.47251E+00_JPRB, 0.47735E+00_JPRB, 0.47958E+00_JPRB, 0.48939E+00_JPRB, 0.49442E+00_JPRB, 0.49650E+00_JPRB, 0.49737E+00_JPRB ,&
&  0.49469E+00_JPRB, 0.49264E+00_JPRB, 0.49101E+00_JPRB, 0.49048E+00_JPRB, 0.48781E+00_JPRB, 0.48377E+00_JPRB, 0.47642E+00_JPRB ,&
&  0.46372E+00_JPRB, 0.45791E+00_JPRB, 0.45884E+00_JPRB, 0.46292E+00_JPRB, 0.46200E+00_JPRB, 0.46225E+00_JPRB, 0.46527E+00_JPRB ,&
&  0.46937E+00_JPRB, 0.47253E+00_JPRB, 0.47449E+00_JPRB, 0.47566E+00_JPRB, 0.47716E+00_JPRB, 0.47858E+00_JPRB, 0.47930E+00_JPRB ,&
&  0.48033E+00_JPRB, 0.48241E+00_JPRB, 0.48536E+00_JPRB, 0.48906E+00_JPRB, 0.49361E+00_JPRB, 0.50105E+00_JPRB, 0.50867E+00_JPRB ,&
&  0.51303E+00_JPRB, 0.51304E+00_JPRB, 0.51110E+00_JPRB, 0.50620E+00_JPRB, 0.49928E+00_JPRB, 0.49327E+00_JPRB, 0.48829E+00_JPRB ,&
&  0.48114E+00_JPRB, 0.47494E+00_JPRB, 0.47021E+00_JPRB, 0.46814E+00_JPRB, 0.46747E+00_JPRB, 0.46800E+00_JPRB, 0.46956E+00_JPRB ,&
&  0.47098E+00_JPRB, 0.47406E+00_JPRB, 0.48072E+00_JPRB, 0.48493E+00_JPRB, 0.48692E+00_JPRB, 0.49003E+00_JPRB, 0.49135E+00_JPRB ,&
&  0.48928E+00_JPRB, 0.48740E+00_JPRB, 0.48620E+00_JPRB, 0.48842E+00_JPRB, 0.49098E+00_JPRB, 0.49196E+00_JPRB, 0.49326E+00_JPRB ,&
&  0.49687E+00_JPRB, 0.50064E+00_JPRB, 0.50178E+00_JPRB, 0.50223E+00_JPRB, 0.50275E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.46341E+00_JPRB, 0.46377E+00_JPRB, 0.46327E+00_JPRB, 0.46218E+00_JPRB, 0.46104E+00_JPRB, 0.46013E+00_JPRB, 0.46028E+00_JPRB ,&
&  0.46058E+00_JPRB, 0.46141E+00_JPRB, 0.46257E+00_JPRB, 0.46353E+00_JPRB, 0.46541E+00_JPRB, 0.46760E+00_JPRB, 0.46945E+00_JPRB ,&
&  0.47133E+00_JPRB, 0.47053E+00_JPRB, 0.46698E+00_JPRB, 0.46373E+00_JPRB, 0.46444E+00_JPRB, 0.47005E+00_JPRB, 0.47612E+00_JPRB ,&
&  0.47738E+00_JPRB, 0.47810E+00_JPRB, 0.47809E+00_JPRB, 0.47645E+00_JPRB, 0.47465E+00_JPRB, 0.47462E+00_JPRB, 0.47164E+00_JPRB ,&
&  0.46725E+00_JPRB, 0.46766E+00_JPRB, 0.46719E+00_JPRB, 0.46762E+00_JPRB, 0.47100E+00_JPRB, 0.47753E+00_JPRB, 0.48009E+00_JPRB ,&
&  0.48086E+00_JPRB, 0.47887E+00_JPRB, 0.47451E+00_JPRB, 0.46974E+00_JPRB, 0.46382E+00_JPRB, 0.45417E+00_JPRB, 0.44329E+00_JPRB ,&
&  0.43454E+00_JPRB, 0.43347E+00_JPRB, 0.43889E+00_JPRB, 0.44450E+00_JPRB, 0.44281E+00_JPRB, 0.44100E+00_JPRB, 0.44228E+00_JPRB ,&
&  0.44597E+00_JPRB, 0.45091E+00_JPRB, 0.45413E+00_JPRB, 0.45348E+00_JPRB, 0.45231E+00_JPRB, 0.45422E+00_JPRB, 0.45697E+00_JPRB ,&
&  0.45819E+00_JPRB, 0.45952E+00_JPRB, 0.46051E+00_JPRB, 0.46215E+00_JPRB, 0.46333E+00_JPRB, 0.46361E+00_JPRB, 0.46534E+00_JPRB ,&
&  0.46544E+00_JPRB, 0.46375E+00_JPRB, 0.45975E+00_JPRB, 0.45395E+00_JPRB, 0.44891E+00_JPRB, 0.44506E+00_JPRB, 0.44383E+00_JPRB ,&
&  0.44211E+00_JPRB, 0.44272E+00_JPRB, 0.44259E+00_JPRB, 0.44024E+00_JPRB, 0.43734E+00_JPRB, 0.43747E+00_JPRB, 0.44121E+00_JPRB ,&
&  0.44619E+00_JPRB, 0.45030E+00_JPRB, 0.45657E+00_JPRB, 0.45954E+00_JPRB, 0.46156E+00_JPRB, 0.46416E+00_JPRB, 0.46489E+00_JPRB ,&
&  0.46212E+00_JPRB, 0.45974E+00_JPRB, 0.45906E+00_JPRB, 0.46184E+00_JPRB, 0.46284E+00_JPRB, 0.46209E+00_JPRB, 0.45934E+00_JPRB ,&
&  0.45848E+00_JPRB, 0.46038E+00_JPRB, 0.46111E+00_JPRB, 0.46090E+00_JPRB, 0.46176E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.43577E+00_JPRB, 0.43821E+00_JPRB, 0.44077E+00_JPRB, 0.44306E+00_JPRB, 0.44450E+00_JPRB, 0.44548E+00_JPRB, 0.44462E+00_JPRB ,&
&  0.44451E+00_JPRB, 0.44630E+00_JPRB, 0.44862E+00_JPRB, 0.45060E+00_JPRB, 0.45190E+00_JPRB, 0.45420E+00_JPRB, 0.45657E+00_JPRB ,&
&  0.45816E+00_JPRB, 0.45705E+00_JPRB, 0.45272E+00_JPRB, 0.45031E+00_JPRB, 0.44907E+00_JPRB, 0.45449E+00_JPRB, 0.46349E+00_JPRB ,&
&  0.46750E+00_JPRB, 0.46981E+00_JPRB, 0.47255E+00_JPRB, 0.47290E+00_JPRB, 0.47310E+00_JPRB, 0.47330E+00_JPRB, 0.46914E+00_JPRB ,&
&  0.46417E+00_JPRB, 0.46035E+00_JPRB, 0.45700E+00_JPRB, 0.45424E+00_JPRB, 0.45589E+00_JPRB, 0.45595E+00_JPRB, 0.45637E+00_JPRB ,&
&  0.45658E+00_JPRB, 0.45422E+00_JPRB, 0.44963E+00_JPRB, 0.44226E+00_JPRB, 0.43338E+00_JPRB, 0.42202E+00_JPRB, 0.41382E+00_JPRB ,&
&  0.41018E+00_JPRB, 0.41384E+00_JPRB, 0.42278E+00_JPRB, 0.42642E+00_JPRB, 0.42233E+00_JPRB, 0.42071E+00_JPRB, 0.42365E+00_JPRB ,&
&  0.42863E+00_JPRB, 0.43441E+00_JPRB, 0.43680E+00_JPRB, 0.43604E+00_JPRB, 0.43394E+00_JPRB, 0.43562E+00_JPRB, 0.44018E+00_JPRB ,&
&  0.44404E+00_JPRB, 0.44590E+00_JPRB, 0.44487E+00_JPRB, 0.44370E+00_JPRB, 0.44103E+00_JPRB, 0.43559E+00_JPRB, 0.42848E+00_JPRB ,&
&  0.42197E+00_JPRB, 0.41792E+00_JPRB, 0.41475E+00_JPRB, 0.41157E+00_JPRB, 0.41029E+00_JPRB, 0.41061E+00_JPRB, 0.41287E+00_JPRB ,&
&  0.41377E+00_JPRB, 0.41564E+00_JPRB, 0.41585E+00_JPRB, 0.41173E+00_JPRB, 0.40849E+00_JPRB, 0.41013E+00_JPRB, 0.41188E+00_JPRB ,&
&  0.41471E+00_JPRB, 0.41766E+00_JPRB, 0.42054E+00_JPRB, 0.42159E+00_JPRB, 0.42318E+00_JPRB, 0.42613E+00_JPRB, 0.42799E+00_JPRB ,&
&  0.42896E+00_JPRB, 0.42964E+00_JPRB, 0.43160E+00_JPRB, 0.43487E+00_JPRB, 0.43802E+00_JPRB, 0.44057E+00_JPRB, 0.44008E+00_JPRB ,&
&  0.43879E+00_JPRB, 0.43713E+00_JPRB, 0.43735E+00_JPRB, 0.43739E+00_JPRB, 0.43571E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.42863E+00_JPRB, 0.42839E+00_JPRB, 0.43176E+00_JPRB, 0.43535E+00_JPRB, 0.43601E+00_JPRB, 0.43541E+00_JPRB, 0.43257E+00_JPRB ,&
&  0.43067E+00_JPRB, 0.43184E+00_JPRB, 0.43374E+00_JPRB, 0.43524E+00_JPRB, 0.43781E+00_JPRB, 0.44125E+00_JPRB, 0.44392E+00_JPRB ,&
&  0.44407E+00_JPRB, 0.44194E+00_JPRB, 0.43827E+00_JPRB, 0.43813E+00_JPRB, 0.43783E+00_JPRB, 0.44021E+00_JPRB, 0.44824E+00_JPRB ,&
&  0.45408E+00_JPRB, 0.45801E+00_JPRB, 0.46128E+00_JPRB, 0.46169E+00_JPRB, 0.46120E+00_JPRB, 0.46317E+00_JPRB, 0.45978E+00_JPRB ,&
&  0.45344E+00_JPRB, 0.44806E+00_JPRB, 0.44201E+00_JPRB, 0.43480E+00_JPRB, 0.43802E+00_JPRB, 0.43627E+00_JPRB, 0.43382E+00_JPRB ,&
&  0.42962E+00_JPRB, 0.42358E+00_JPRB, 0.41614E+00_JPRB, 0.40898E+00_JPRB, 0.40348E+00_JPRB, 0.39834E+00_JPRB, 0.39212E+00_JPRB ,&
&  0.38991E+00_JPRB, 0.39247E+00_JPRB, 0.39842E+00_JPRB, 0.40027E+00_JPRB, 0.39678E+00_JPRB, 0.39706E+00_JPRB, 0.40369E+00_JPRB ,&
&  0.41104E+00_JPRB, 0.41475E+00_JPRB, 0.41476E+00_JPRB, 0.41529E+00_JPRB, 0.41594E+00_JPRB, 0.41748E+00_JPRB, 0.41825E+00_JPRB ,&
&  0.41711E+00_JPRB, 0.41524E+00_JPRB, 0.41239E+00_JPRB, 0.40625E+00_JPRB, 0.39908E+00_JPRB, 0.39248E+00_JPRB, 0.38995E+00_JPRB ,&
&  0.39264E+00_JPRB, 0.39310E+00_JPRB, 0.39438E+00_JPRB, 0.39668E+00_JPRB, 0.39745E+00_JPRB, 0.39691E+00_JPRB, 0.39612E+00_JPRB ,&
&  0.39777E+00_JPRB, 0.39627E+00_JPRB, 0.39699E+00_JPRB, 0.39409E+00_JPRB, 0.38951E+00_JPRB, 0.38467E+00_JPRB, 0.38001E+00_JPRB ,&
&  0.37745E+00_JPRB, 0.37693E+00_JPRB, 0.37905E+00_JPRB, 0.37995E+00_JPRB, 0.38213E+00_JPRB, 0.38470E+00_JPRB, 0.38691E+00_JPRB ,&
&  0.38829E+00_JPRB, 0.39020E+00_JPRB, 0.39374E+00_JPRB, 0.40028E+00_JPRB, 0.40703E+00_JPRB, 0.41286E+00_JPRB, 0.41854E+00_JPRB ,&
&  0.42357E+00_JPRB, 0.42699E+00_JPRB, 0.43278E+00_JPRB, 0.43363E+00_JPRB, 0.43231E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.42672E+00_JPRB, 0.42364E+00_JPRB, 0.42331E+00_JPRB, 0.42380E+00_JPRB, 0.42268E+00_JPRB, 0.42118E+00_JPRB, 0.41802E+00_JPRB ,&
&  0.41539E+00_JPRB, 0.41574E+00_JPRB, 0.41719E+00_JPRB, 0.42000E+00_JPRB, 0.42486E+00_JPRB, 0.43000E+00_JPRB, 0.42964E+00_JPRB ,&
&  0.42659E+00_JPRB, 0.42538E+00_JPRB, 0.42762E+00_JPRB, 0.42929E+00_JPRB, 0.42926E+00_JPRB, 0.42999E+00_JPRB, 0.43399E+00_JPRB ,&
&  0.43890E+00_JPRB, 0.44122E+00_JPRB, 0.44433E+00_JPRB, 0.44397E+00_JPRB, 0.44021E+00_JPRB, 0.44087E+00_JPRB, 0.43909E+00_JPRB ,&
&  0.43411E+00_JPRB, 0.42775E+00_JPRB, 0.41801E+00_JPRB, 0.41859E+00_JPRB, 0.41488E+00_JPRB, 0.41175E+00_JPRB, 0.40688E+00_JPRB ,&
&  0.40139E+00_JPRB, 0.39648E+00_JPRB, 0.39109E+00_JPRB, 0.38678E+00_JPRB, 0.38474E+00_JPRB, 0.38456E+00_JPRB, 0.38220E+00_JPRB ,&
&  0.38093E+00_JPRB, 0.38346E+00_JPRB, 0.38479E+00_JPRB, 0.38047E+00_JPRB, 0.37918E+00_JPRB, 0.38510E+00_JPRB, 0.39044E+00_JPRB ,&
&  0.39299E+00_JPRB, 0.39321E+00_JPRB, 0.39032E+00_JPRB, 0.38948E+00_JPRB, 0.38847E+00_JPRB, 0.38649E+00_JPRB, 0.38287E+00_JPRB ,&
&  0.37714E+00_JPRB, 0.37421E+00_JPRB, 0.37216E+00_JPRB, 0.36817E+00_JPRB, 0.36056E+00_JPRB, 0.34464E+00_JPRB, 0.33592E+00_JPRB ,&
&  0.33734E+00_JPRB, 0.34671E+00_JPRB, 0.36053E+00_JPRB, 0.36864E+00_JPRB, 0.37667E+00_JPRB, 0.38233E+00_JPRB, 0.38252E+00_JPRB ,&
&  0.38150E+00_JPRB, 0.36207E+00_JPRB, 0.35125E+00_JPRB, 0.34149E+00_JPRB, 0.33274E+00_JPRB, 0.32756E+00_JPRB, 0.32552E+00_JPRB ,&
&  0.32838E+00_JPRB, 0.33395E+00_JPRB, 0.33973E+00_JPRB, 0.34297E+00_JPRB, 0.34316E+00_JPRB, 0.34089E+00_JPRB, 0.33792E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.34711E+00_JPRB, 0.36208E+00_JPRB, 0.37479E+00_JPRB, 0.38496E+00_JPRB, 0.39595E+00_JPRB, 0.40490E+00_JPRB ,&
&  0.41115E+00_JPRB, 0.41703E+00_JPRB, 0.42387E+00_JPRB, 0.42621E+00_JPRB, 0.42695E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.41220E+00_JPRB, 0.40777E+00_JPRB, 0.40740E+00_JPRB, 0.40744E+00_JPRB, 0.40582E+00_JPRB, 0.40516E+00_JPRB, 0.40392E+00_JPRB ,&
&  0.40317E+00_JPRB, 0.40112E+00_JPRB, 0.39938E+00_JPRB, 0.40011E+00_JPRB, 0.40252E+00_JPRB, 0.40517E+00_JPRB, 0.40595E+00_JPRB ,&
&  0.40597E+00_JPRB, 0.40764E+00_JPRB, 0.41133E+00_JPRB, 0.41484E+00_JPRB, 0.41529E+00_JPRB, 0.41553E+00_JPRB, 0.41746E+00_JPRB ,&
&  0.41985E+00_JPRB, 0.42074E+00_JPRB, 0.42371E+00_JPRB, 0.42142E+00_JPRB, 0.41686E+00_JPRB, 0.41549E+00_JPRB, 0.41419E+00_JPRB ,&
&  0.40681E+00_JPRB, 0.38213E+00_JPRB, 0.36668E+00_JPRB, 0.37251E+00_JPRB, 0.38745E+00_JPRB, 0.38597E+00_JPRB, 0.38352E+00_JPRB ,&
&  0.38097E+00_JPRB, 0.37744E+00_JPRB, 0.37452E+00_JPRB, 0.37346E+00_JPRB, 0.37391E+00_JPRB, 0.37416E+00_JPRB, 0.37302E+00_JPRB ,&
&  0.37389E+00_JPRB, 0.37991E+00_JPRB, 0.38617E+00_JPRB, 0.38157E+00_JPRB, 0.37550E+00_JPRB, 0.36738E+00_JPRB, 0.36223E+00_JPRB ,&
&  0.35746E+00_JPRB, 0.35288E+00_JPRB, 0.34970E+00_JPRB, 0.34877E+00_JPRB, 0.34851E+00_JPRB, 0.34611E+00_JPRB, 0.34200E+00_JPRB ,&
&  0.33570E+00_JPRB, 0.32588E+00_JPRB, 0.31629E+00_JPRB, 0.30683E+00_JPRB, 0.30060E+00_JPRB, 0.29595E+00_JPRB, 0.29204E+00_JPRB ,&
&  0.29461E+00_JPRB, 0.30846E+00_JPRB, 0.32943E+00_JPRB, 0.35030E+00_JPRB, 0.36523E+00_JPRB, 0.34446E+00_JPRB, 0.33484E+00_JPRB ,&
&  0.32281E+00_JPRB, 0.31214E+00_JPRB, 0.30455E+00_JPRB, 0.29910E+00_JPRB, 0.29498E+00_JPRB, 0.29247E+00_JPRB, 0.29202E+00_JPRB ,&
&  0.29530E+00_JPRB, 0.29982E+00_JPRB, 0.30401E+00_JPRB, 0.30558E+00_JPRB, 0.30542E+00_JPRB, 0.30376E+00_JPRB, 0.30287E+00_JPRB ,&
&  0.30469E+00_JPRB, 0.31149E+00_JPRB, 0.32213E+00_JPRB, 0.33767E+00_JPRB, 0.35113E+00_JPRB, 0.36481E+00_JPRB, 0.37992E+00_JPRB ,&
&  0.39530E+00_JPRB, 0.40791E+00_JPRB, 0.41146E+00_JPRB, 0.41505E+00_JPRB, 0.41543E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.40204E+00_JPRB, 0.39948E+00_JPRB, 0.39734E+00_JPRB, 0.39750E+00_JPRB, 0.39745E+00_JPRB, 0.39951E+00_JPRB, 0.39802E+00_JPRB ,&
&  0.39330E+00_JPRB, 0.38864E+00_JPRB, 0.38743E+00_JPRB, 0.38770E+00_JPRB, 0.38669E+00_JPRB, 0.38511E+00_JPRB, 0.38605E+00_JPRB ,&
&  0.38449E+00_JPRB, 0.38690E+00_JPRB, 0.39028E+00_JPRB, 0.39210E+00_JPRB, 0.39275E+00_JPRB, 0.39285E+00_JPRB, 0.39322E+00_JPRB ,&
&  0.38691E+00_JPRB, 0.38474E+00_JPRB, 0.39059E+00_JPRB, 0.38753E+00_JPRB, 0.38333E+00_JPRB, 0.37977E+00_JPRB, 0.37860E+00_JPRB ,&
&  0.36925E+00_JPRB, 0.35633E+00_JPRB, 0.34389E+00_JPRB, 0.35156E+00_JPRB, 0.36849E+00_JPRB, 0.36439E+00_JPRB, 0.36206E+00_JPRB ,&
&  0.36194E+00_JPRB, 0.36128E+00_JPRB, 0.36232E+00_JPRB, 0.36302E+00_JPRB, 0.36449E+00_JPRB, 0.36500E+00_JPRB, 0.36688E+00_JPRB ,&
&  0.36876E+00_JPRB, 0.37619E+00_JPRB, 0.37919E+00_JPRB, 0.35720E+00_JPRB, 0.33939E+00_JPRB, 0.32585E+00_JPRB, 0.31263E+00_JPRB ,&
&  0.30083E+00_JPRB, 0.29965E+00_JPRB, 0.29561E+00_JPRB, 0.28734E+00_JPRB, 0.28762E+00_JPRB, 0.28853E+00_JPRB, 0.28748E+00_JPRB ,&
&  0.28481E+00_JPRB, 0.28323E+00_JPRB, 0.28178E+00_JPRB, 0.27929E+00_JPRB, 0.27850E+00_JPRB, 0.28012E+00_JPRB, 0.28457E+00_JPRB ,&
&  0.29130E+00_JPRB, 0.29822E+00_JPRB, 0.31404E+00_JPRB, 0.32706E+00_JPRB, 0.32172E+00_JPRB, 0.31575E+00_JPRB, 0.30907E+00_JPRB ,&
&  0.29638E+00_JPRB, 0.29032E+00_JPRB, 0.28418E+00_JPRB, 0.27916E+00_JPRB, 0.27679E+00_JPRB, 0.27680E+00_JPRB, 0.27736E+00_JPRB ,&
&  0.27846E+00_JPRB, 0.28078E+00_JPRB, 0.28168E+00_JPRB, 0.28139E+00_JPRB, 0.28066E+00_JPRB, 0.28156E+00_JPRB, 0.28375E+00_JPRB ,&
&  0.28750E+00_JPRB, 0.29382E+00_JPRB, 0.30128E+00_JPRB, 0.31058E+00_JPRB, 0.31879E+00_JPRB, 0.32648E+00_JPRB, 0.33872E+00_JPRB ,&
&  0.34723E+00_JPRB, 0.36785E+00_JPRB, 0.39465E+00_JPRB, 0.41558E+00_JPRB, 0.40694E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.38035E+00_JPRB, 0.37807E+00_JPRB, 0.37534E+00_JPRB, 0.37485E+00_JPRB, 0.37566E+00_JPRB, 0.37799E+00_JPRB, 0.37526E+00_JPRB ,&
&  0.37025E+00_JPRB, 0.36823E+00_JPRB, 0.36942E+00_JPRB, 0.36859E+00_JPRB, 0.36849E+00_JPRB, 0.35897E+00_JPRB, 0.35356E+00_JPRB ,&
&  0.34652E+00_JPRB, 0.34468E+00_JPRB, 0.34308E+00_JPRB, 0.34419E+00_JPRB, 0.34619E+00_JPRB, 0.34736E+00_JPRB, 0.34285E+00_JPRB ,&
&  0.33827E+00_JPRB, 0.33417E+00_JPRB, 0.32962E+00_JPRB, 0.32767E+00_JPRB, 0.32901E+00_JPRB, 0.33015E+00_JPRB, 0.33007E+00_JPRB ,&
&  0.33086E+00_JPRB, 0.33403E+00_JPRB, 0.33712E+00_JPRB, 0.34114E+00_JPRB, 0.35400E+00_JPRB, 0.35118E+00_JPRB, 0.34994E+00_JPRB ,&
&  0.35063E+00_JPRB, 0.35147E+00_JPRB, 0.35301E+00_JPRB, 0.35422E+00_JPRB, 0.35942E+00_JPRB, 0.36195E+00_JPRB, 0.36402E+00_JPRB ,&
&  0.35084E+00_JPRB, 0.34453E+00_JPRB, 0.33743E+00_JPRB, 0.32792E+00_JPRB, 0.31482E+00_JPRB, 0.30462E+00_JPRB, 0.29535E+00_JPRB ,&
&  0.28760E+00_JPRB, 0.28026E+00_JPRB, 0.27325E+00_JPRB, 0.27056E+00_JPRB, 0.26952E+00_JPRB, 0.26783E+00_JPRB, 0.26754E+00_JPRB ,&
&  0.26810E+00_JPRB, 0.26895E+00_JPRB, 0.26944E+00_JPRB, 0.27117E+00_JPRB, 0.27403E+00_JPRB, 0.27857E+00_JPRB, 0.28341E+00_JPRB ,&
&  0.29049E+00_JPRB, 0.29526E+00_JPRB, 0.30106E+00_JPRB, 0.30653E+00_JPRB, 0.30507E+00_JPRB, 0.29940E+00_JPRB, 0.29357E+00_JPRB ,&
&  0.28580E+00_JPRB, 0.28134E+00_JPRB, 0.27671E+00_JPRB, 0.27330E+00_JPRB, 0.27129E+00_JPRB, 0.26999E+00_JPRB, 0.27002E+00_JPRB ,&
&  0.26981E+00_JPRB, 0.27015E+00_JPRB, 0.26941E+00_JPRB, 0.26969E+00_JPRB, 0.27067E+00_JPRB, 0.27290E+00_JPRB, 0.27584E+00_JPRB ,&
&  0.28043E+00_JPRB, 0.28571E+00_JPRB, 0.29183E+00_JPRB, 0.29842E+00_JPRB, 0.30836E+00_JPRB, 0.32086E+00_JPRB, 0.33807E+00_JPRB ,&
&  0.35412E+00_JPRB, 0.38055E+00_JPRB, 0.39065E+00_JPRB, 0.38632E+00_JPRB, 0.38433E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.36292E+00_JPRB, 0.36443E+00_JPRB, 0.36527E+00_JPRB, 0.36744E+00_JPRB, 0.37003E+00_JPRB, 0.37078E+00_JPRB, 0.37088E+00_JPRB ,&
&  0.36135E+00_JPRB, 0.35417E+00_JPRB, 0.34733E+00_JPRB, 0.33724E+00_JPRB, 0.32923E+00_JPRB, 0.32223E+00_JPRB, 0.31722E+00_JPRB ,&
&  0.31162E+00_JPRB, 0.30910E+00_JPRB, 0.30576E+00_JPRB, 0.30479E+00_JPRB, 0.30406E+00_JPRB, 0.30440E+00_JPRB, 0.30256E+00_JPRB ,&
&  0.30071E+00_JPRB, 0.30220E+00_JPRB, 0.30305E+00_JPRB, 0.30753E+00_JPRB, 0.31432E+00_JPRB, 0.32190E+00_JPRB, 0.32936E+00_JPRB ,&
&  0.33890E+00_JPRB, 0.34274E+00_JPRB, 0.34601E+00_JPRB, 0.34683E+00_JPRB, 0.34592E+00_JPRB, 0.34587E+00_JPRB, 0.34421E+00_JPRB ,&
&  0.34457E+00_JPRB, 0.34465E+00_JPRB, 0.34675E+00_JPRB, 0.34818E+00_JPRB, 0.34647E+00_JPRB, 0.34493E+00_JPRB, 0.34021E+00_JPRB ,&
&  0.33248E+00_JPRB, 0.32641E+00_JPRB, 0.31973E+00_JPRB, 0.31439E+00_JPRB, 0.30604E+00_JPRB, 0.29882E+00_JPRB, 0.29183E+00_JPRB ,&
&  0.28593E+00_JPRB, 0.28164E+00_JPRB, 0.27636E+00_JPRB, 0.27213E+00_JPRB, 0.27012E+00_JPRB, 0.26758E+00_JPRB, 0.26648E+00_JPRB ,&
&  0.26645E+00_JPRB, 0.26695E+00_JPRB, 0.26957E+00_JPRB, 0.27163E+00_JPRB, 0.27484E+00_JPRB, 0.27860E+00_JPRB, 0.28071E+00_JPRB ,&
&  0.28466E+00_JPRB, 0.28808E+00_JPRB, 0.28997E+00_JPRB, 0.29149E+00_JPRB, 0.29053E+00_JPRB, 0.28702E+00_JPRB, 0.28362E+00_JPRB ,&
&  0.27892E+00_JPRB, 0.27569E+00_JPRB, 0.27362E+00_JPRB, 0.27129E+00_JPRB, 0.27000E+00_JPRB, 0.26882E+00_JPRB, 0.26745E+00_JPRB ,&
&  0.26639E+00_JPRB, 0.26585E+00_JPRB, 0.26536E+00_JPRB, 0.26664E+00_JPRB, 0.26796E+00_JPRB, 0.27045E+00_JPRB, 0.27344E+00_JPRB ,&
&  0.27611E+00_JPRB, 0.28028E+00_JPRB, 0.28479E+00_JPRB, 0.29101E+00_JPRB, 0.30277E+00_JPRB, 0.31159E+00_JPRB, 0.33107E+00_JPRB ,&
&  0.34196E+00_JPRB, 0.36196E+00_JPRB, 0.37085E+00_JPRB, 0.36262E+00_JPRB, 0.36256E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.35130E+00_JPRB, 0.35364E+00_JPRB, 0.35416E+00_JPRB, 0.35603E+00_JPRB, 0.35758E+00_JPRB, 0.35802E+00_JPRB, 0.35744E+00_JPRB ,&
&  0.35535E+00_JPRB, 0.35015E+00_JPRB, 0.34574E+00_JPRB, 0.33904E+00_JPRB, 0.33298E+00_JPRB, 0.32616E+00_JPRB, 0.32028E+00_JPRB ,&
&  0.31374E+00_JPRB, 0.30772E+00_JPRB, 0.30145E+00_JPRB, 0.29611E+00_JPRB, 0.29208E+00_JPRB, 0.28875E+00_JPRB, 0.28549E+00_JPRB ,&
&  0.28269E+00_JPRB, 0.28428E+00_JPRB, 0.28736E+00_JPRB, 0.29298E+00_JPRB, 0.30157E+00_JPRB, 0.30910E+00_JPRB, 0.31742E+00_JPRB ,&
&  0.33112E+00_JPRB, 0.33358E+00_JPRB, 0.33380E+00_JPRB, 0.33634E+00_JPRB, 0.33607E+00_JPRB, 0.33675E+00_JPRB, 0.32732E+00_JPRB ,&
&  0.33152E+00_JPRB, 0.33391E+00_JPRB, 0.34279E+00_JPRB, 0.34559E+00_JPRB, 0.33958E+00_JPRB, 0.33570E+00_JPRB, 0.32798E+00_JPRB ,&
&  0.32249E+00_JPRB, 0.31787E+00_JPRB, 0.31479E+00_JPRB, 0.31305E+00_JPRB, 0.30729E+00_JPRB, 0.30303E+00_JPRB, 0.29588E+00_JPRB ,&
&  0.29017E+00_JPRB, 0.28663E+00_JPRB, 0.28208E+00_JPRB, 0.27938E+00_JPRB, 0.27711E+00_JPRB, 0.27482E+00_JPRB, 0.27306E+00_JPRB ,&
&  0.27208E+00_JPRB, 0.27126E+00_JPRB, 0.27207E+00_JPRB, 0.27297E+00_JPRB, 0.27472E+00_JPRB, 0.27613E+00_JPRB, 0.27632E+00_JPRB ,&
&  0.27659E+00_JPRB, 0.27715E+00_JPRB, 0.27743E+00_JPRB, 0.27838E+00_JPRB, 0.27784E+00_JPRB, 0.27672E+00_JPRB, 0.27536E+00_JPRB ,&
&  0.27378E+00_JPRB, 0.27309E+00_JPRB, 0.27188E+00_JPRB, 0.27109E+00_JPRB, 0.26875E+00_JPRB, 0.26716E+00_JPRB, 0.26630E+00_JPRB ,&
&  0.26550E+00_JPRB, 0.26602E+00_JPRB, 0.26615E+00_JPRB, 0.26714E+00_JPRB, 0.26866E+00_JPRB, 0.26992E+00_JPRB, 0.27228E+00_JPRB ,&
&  0.27521E+00_JPRB, 0.27861E+00_JPRB, 0.28470E+00_JPRB, 0.29081E+00_JPRB, 0.29758E+00_JPRB, 0.30521E+00_JPRB, 0.31085E+00_JPRB ,&
&  0.31925E+00_JPRB, 0.33242E+00_JPRB, 0.33426E+00_JPRB, 0.34127E+00_JPRB, 0.34743E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.33343E+00_JPRB, 0.33828E+00_JPRB, 0.34141E+00_JPRB, 0.34345E+00_JPRB, 0.34556E+00_JPRB, 0.34481E+00_JPRB, 0.34303E+00_JPRB ,&
&  0.33852E+00_JPRB, 0.33701E+00_JPRB, 0.33410E+00_JPRB, 0.33077E+00_JPRB, 0.32698E+00_JPRB, 0.32199E+00_JPRB, 0.31792E+00_JPRB ,&
&  0.31325E+00_JPRB, 0.30808E+00_JPRB, 0.30212E+00_JPRB, 0.29531E+00_JPRB, 0.28909E+00_JPRB, 0.28321E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.27908E+00_JPRB, 0.28026E+00_JPRB, 0.28372E+00_JPRB, 0.28764E+00_JPRB, 0.29234E+00_JPRB, 0.29762E+00_JPRB, 0.30417E+00_JPRB ,&
&  0.30961E+00_JPRB, 0.31602E+00_JPRB, 0.31987E+00_JPRB, 0.32140E+00_JPRB, 0.32001E+00_JPRB, 0.32013E+00_JPRB, 0.31933E+00_JPRB ,&
&  0.31827E+00_JPRB, 0.31683E+00_JPRB, 0.31674E+00_JPRB, 0.31759E+00_JPRB, 0.31744E+00_JPRB, 0.31613E+00_JPRB, 0.31379E+00_JPRB ,&
&  0.31138E+00_JPRB, 0.30931E+00_JPRB, 0.30632E+00_JPRB, 0.30375E+00_JPRB, 0.30186E+00_JPRB, 0.29933E+00_JPRB, 0.29579E+00_JPRB ,&
&  0.29269E+00_JPRB, 0.28965E+00_JPRB, 0.28609E+00_JPRB, 0.28321E+00_JPRB, 0.28035E+00_JPRB, 0.27829E+00_JPRB, 0.27643E+00_JPRB ,&
&  0.27571E+00_JPRB, 0.27490E+00_JPRB, 0.27471E+00_JPRB, 0.27462E+00_JPRB, 0.27516E+00_JPRB, 0.27478E+00_JPRB, 0.27422E+00_JPRB ,&
&  0.27357E+00_JPRB, 0.27340E+00_JPRB, 0.27273E+00_JPRB, 0.27248E+00_JPRB, 0.27222E+00_JPRB, 0.27128E+00_JPRB, 0.27076E+00_JPRB ,&
&  0.27064E+00_JPRB, 0.27035E+00_JPRB, 0.26948E+00_JPRB, 0.26926E+00_JPRB, 0.26868E+00_JPRB, 0.26790E+00_JPRB, 0.26818E+00_JPRB ,&
&  0.26792E+00_JPRB, 0.26812E+00_JPRB, 0.26844E+00_JPRB, 0.26893E+00_JPRB, 0.27015E+00_JPRB, 0.27166E+00_JPRB, 0.27371E+00_JPRB ,&
&  0.27697E+00_JPRB, 0.27986E+00_JPRB, 0.28271E+00_JPRB, 0.28598E+00_JPRB, 0.29266E+00_JPRB, 0.29484E+00_JPRB, 0.29906E+00_JPRB ,&
&  0.30249E+00_JPRB, 0.29751E+00_JPRB, 0.32123E+00_JPRB, 0.32260E+00_JPRB, 0.32591E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.28877E+00_JPRB, 0.29394E+00_JPRB, 0.29729E+00_JPRB, 0.29966E+00_JPRB, 0.29347E+00_JPRB, 0.31407E+00_JPRB, 0.31232E+00_JPRB ,&
&  0.31303E+00_JPRB, 0.30895E+00_JPRB, 0.31134E+00_JPRB, 0.31065E+00_JPRB, 0.30854E+00_JPRB, 0.30561E+00_JPRB, 0.30257E+00_JPRB ,&
&  0.29934E+00_JPRB, 0.29538E+00_JPRB, 0.29185E+00_JPRB, 0.28837E+00_JPRB, 0.28549E+00_JPRB, 0.28291E+00_JPRB, 0.28159E+00_JPRB ,&
&  0.28178E+00_JPRB, 0.28353E+00_JPRB, 0.28651E+00_JPRB, 0.28785E+00_JPRB, 0.28933E+00_JPRB, 0.29092E+00_JPRB, 0.29266E+00_JPRB ,&
&  0.29397E+00_JPRB, 0.29590E+00_JPRB, 0.29687E+00_JPRB, 0.29636E+00_JPRB, 0.29564E+00_JPRB, 0.29573E+00_JPRB, 0.29612E+00_JPRB ,&
&  0.29724E+00_JPRB, 0.29803E+00_JPRB, 0.29860E+00_JPRB, 0.29926E+00_JPRB, 0.29991E+00_JPRB, 0.29966E+00_JPRB, 0.29849E+00_JPRB ,&
&  0.29749E+00_JPRB, 0.29666E+00_JPRB, 0.29473E+00_JPRB, 0.29291E+00_JPRB, 0.29118E+00_JPRB, 0.28938E+00_JPRB, 0.28745E+00_JPRB ,&
&  0.28610E+00_JPRB, 0.28474E+00_JPRB, 0.28304E+00_JPRB, 0.28108E+00_JPRB, 0.27928E+00_JPRB, 0.27771E+00_JPRB, 0.27651E+00_JPRB ,&
&  0.27634E+00_JPRB, 0.27569E+00_JPRB, 0.27512E+00_JPRB, 0.27455E+00_JPRB, 0.27443E+00_JPRB, 0.27387E+00_JPRB, 0.27345E+00_JPRB ,&
&  0.27312E+00_JPRB, 0.27268E+00_JPRB, 0.27218E+00_JPRB, 0.27168E+00_JPRB, 0.27133E+00_JPRB, 0.27089E+00_JPRB, 0.27042E+00_JPRB ,&
&  0.27010E+00_JPRB, 0.26985E+00_JPRB, 0.26958E+00_JPRB, 0.26904E+00_JPRB, 0.26850E+00_JPRB, 0.26786E+00_JPRB, 0.26780E+00_JPRB ,&
&  0.26790E+00_JPRB, 0.26816E+00_JPRB, 0.26865E+00_JPRB, 0.26876E+00_JPRB, 0.26972E+00_JPRB, 0.27063E+00_JPRB, 0.27174E+00_JPRB ,&
&  0.27286E+00_JPRB, 0.27467E+00_JPRB, 0.27668E+00_JPRB, 0.27891E+00_JPRB, 0.28263E+00_JPRB, 0.28353E+00_JPRB, 0.28425E+00_JPRB ,&
&  0.28518E+00_JPRB, 0.28940E+00_JPRB, 0.28999E+00_JPRB, 0.28929E+00_JPRB, 0.28947E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.27891E+00_JPRB, 0.27917E+00_JPRB, 0.27907E+00_JPRB, 0.27876E+00_JPRB, 0.27613E+00_JPRB, 0.27495E+00_JPRB, 0.27395E+00_JPRB ,&
&  0.27319E+00_JPRB, 0.27282E+00_JPRB, 0.27331E+00_JPRB, 0.27396E+00_JPRB, 0.27464E+00_JPRB, 0.27536E+00_JPRB, 0.27602E+00_JPRB ,&
&  0.27637E+00_JPRB, 0.27678E+00_JPRB, 0.27550E+00_JPRB, 0.27486E+00_JPRB, 0.27465E+00_JPRB, 0.27468E+00_JPRB, 0.27465E+00_JPRB ,&
&  0.27474E+00_JPRB, 0.27480E+00_JPRB, 0.27476E+00_JPRB, 0.27519E+00_JPRB, 0.27592E+00_JPRB, 0.27651E+00_JPRB, 0.27713E+00_JPRB ,&
&  0.27774E+00_JPRB, 0.27863E+00_JPRB, 0.27945E+00_JPRB, 0.28025E+00_JPRB, 0.28135E+00_JPRB, 0.28247E+00_JPRB, 0.28319E+00_JPRB ,&
&  0.28378E+00_JPRB, 0.28406E+00_JPRB, 0.28440E+00_JPRB, 0.28468E+00_JPRB, 0.28508E+00_JPRB, 0.28555E+00_JPRB, 0.28570E+00_JPRB ,&
&  0.28568E+00_JPRB, 0.28556E+00_JPRB, 0.28534E+00_JPRB, 0.28472E+00_JPRB, 0.28405E+00_JPRB, 0.28340E+00_JPRB, 0.28274E+00_JPRB ,&
&  0.28217E+00_JPRB, 0.28152E+00_JPRB, 0.28080E+00_JPRB, 0.28004E+00_JPRB, 0.27924E+00_JPRB, 0.27846E+00_JPRB, 0.27767E+00_JPRB ,&
&  0.27686E+00_JPRB, 0.27635E+00_JPRB, 0.27578E+00_JPRB, 0.27529E+00_JPRB, 0.27489E+00_JPRB, 0.27447E+00_JPRB, 0.27407E+00_JPRB ,&
&  0.27368E+00_JPRB, 0.27321E+00_JPRB, 0.27279E+00_JPRB, 0.27231E+00_JPRB, 0.27189E+00_JPRB, 0.27141E+00_JPRB, 0.27102E+00_JPRB ,&
&  0.27064E+00_JPRB, 0.27030E+00_JPRB, 0.27002E+00_JPRB, 0.26988E+00_JPRB, 0.26978E+00_JPRB, 0.26979E+00_JPRB, 0.26981E+00_JPRB ,&
&  0.26985E+00_JPRB, 0.26986E+00_JPRB, 0.26985E+00_JPRB, 0.26965E+00_JPRB, 0.26988E+00_JPRB, 0.27011E+00_JPRB, 0.27046E+00_JPRB ,&
&  0.27078E+00_JPRB, 0.27134E+00_JPRB, 0.27193E+00_JPRB, 0.27260E+00_JPRB, 0.27368E+00_JPRB, 0.27406E+00_JPRB, 0.27438E+00_JPRB ,&
&  0.27471E+00_JPRB, 0.27545E+00_JPRB, 0.27605E+00_JPRB, 0.27635E+00_JPRB, 0.27703E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB, 0.27417E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB, 0.92803E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.90150E+00_JPRB, 0.89748E+00_JPRB, 0.89319E+00_JPRB, 0.88871E+00_JPRB, 0.88381E+00_JPRB, 0.87826E+00_JPRB, 0.87197E+00_JPRB ,&
&  0.86615E+00_JPRB, 0.86069E+00_JPRB, 0.85582E+00_JPRB, 0.85154E+00_JPRB, 0.84783E+00_JPRB, 0.84497E+00_JPRB, 0.84400E+00_JPRB ,&
&  0.84400E+00_JPRB, 0.84408E+00_JPRB, 0.84454E+00_JPRB, 0.84483E+00_JPRB, 0.84470E+00_JPRB, 0.84436E+00_JPRB, 0.84386E+00_JPRB ,&
&  0.84361E+00_JPRB, 0.84353E+00_JPRB, 0.84350E+00_JPRB, 0.84396E+00_JPRB, 0.84459E+00_JPRB, 0.84529E+00_JPRB, 0.84589E+00_JPRB ,&
&  0.84708E+00_JPRB, 0.84852E+00_JPRB, 0.85051E+00_JPRB, 0.85293E+00_JPRB, 0.85786E+00_JPRB, 0.86306E+00_JPRB, 0.86849E+00_JPRB ,&
&  0.87359E+00_JPRB, 0.87872E+00_JPRB, 0.88327E+00_JPRB, 0.88729E+00_JPRB, 0.89102E+00_JPRB, 0.89558E+00_JPRB, 0.90095E+00_JPRB ,&
&  0.90680E+00_JPRB, 0.91321E+00_JPRB, 0.92041E+00_JPRB, 0.92761E+00_JPRB, 0.93424E+00_JPRB, 0.94102E+00_JPRB, 0.94821E+00_JPRB ,&
&  0.95529E+00_JPRB, 0.96232E+00_JPRB, 0.96932E+00_JPRB, 0.97642E+00_JPRB, 0.98343E+00_JPRB, 0.99151E+00_JPRB, 0.10009E+01_JPRB ,&
&  0.10113E+01_JPRB, 0.10195E+01_JPRB, 0.10289E+01_JPRB, 0.10390E+01_JPRB, 0.10531E+01_JPRB, 0.10601E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10826E+01_JPRB, 0.10885E+01_JPRB, 0.10945E+01_JPRB, 0.11001E+01_JPRB, 0.11049E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.11103E+01_JPRB, 0.11126E+01_JPRB, 0.11126E+01_JPRB, 0.11114E+01_JPRB, 0.11089E+01_JPRB, 0.11059E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.10875E+01_JPRB, 0.10823E+01_JPRB, 0.10748E+01_JPRB, 0.10673E+01_JPRB, 0.10596E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10315E+01_JPRB, 0.10206E+01_JPRB, 0.10089E+01_JPRB, 0.99753E+00_JPRB, 0.98570E+00_JPRB, 0.97289E+00_JPRB ,&
&  0.95871E+00_JPRB, 0.94543E+00_JPRB, 0.93068E+00_JPRB, 0.91612E+00_JPRB, 0.90556E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.87954E+00_JPRB, 0.86740E+00_JPRB, 0.86212E+00_JPRB, 0.85860E+00_JPRB, 0.85550E+00_JPRB, 0.85139E+00_JPRB, 0.84676E+00_JPRB ,&
&  0.84327E+00_JPRB, 0.83892E+00_JPRB, 0.83544E+00_JPRB, 0.83469E+00_JPRB, 0.83281E+00_JPRB, 0.83182E+00_JPRB, 0.83074E+00_JPRB ,&
&  0.83007E+00_JPRB, 0.82922E+00_JPRB, 0.82898E+00_JPRB, 0.82728E+00_JPRB, 0.82465E+00_JPRB, 0.82108E+00_JPRB, 0.81819E+00_JPRB ,&
&  0.81579E+00_JPRB, 0.81409E+00_JPRB, 0.81159E+00_JPRB, 0.80948E+00_JPRB, 0.81034E+00_JPRB, 0.81056E+00_JPRB, 0.81004E+00_JPRB ,&
&  0.81339E+00_JPRB, 0.81554E+00_JPRB, 0.81507E+00_JPRB, 0.81224E+00_JPRB, 0.80873E+00_JPRB, 0.80687E+00_JPRB, 0.80924E+00_JPRB ,&
&  0.81355E+00_JPRB, 0.82649E+00_JPRB, 0.83715E+00_JPRB, 0.84721E+00_JPRB, 0.85663E+00_JPRB, 0.86809E+00_JPRB, 0.88160E+00_JPRB ,&
&  0.89581E+00_JPRB, 0.90740E+00_JPRB, 0.92030E+00_JPRB, 0.93588E+00_JPRB, 0.95263E+00_JPRB, 0.97028E+00_JPRB, 0.98956E+00_JPRB ,&
&  0.10063E+01_JPRB, 0.10236E+01_JPRB, 0.10435E+01_JPRB, 0.10652E+01_JPRB, 0.10840E+01_JPRB, 0.11080E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11715E+01_JPRB, 0.11900E+01_JPRB, 0.12107E+01_JPRB, 0.12314E+01_JPRB, 0.12573E+01_JPRB, 0.12728E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.13051E+01_JPRB, 0.13024E+01_JPRB, 0.12973E+01_JPRB, 0.12937E+01_JPRB, 0.12882E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.12748E+01_JPRB, 0.12668E+01_JPRB, 0.12579E+01_JPRB, 0.12460E+01_JPRB, 0.12338E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.11921E+01_JPRB, 0.11767E+01_JPRB, 0.11662E+01_JPRB, 0.11530E+01_JPRB, 0.11401E+01_JPRB, 0.11281E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.10920E+01_JPRB, 0.10779E+01_JPRB, 0.10628E+01_JPRB, 0.10503E+01_JPRB, 0.10328E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.99006E+00_JPRB, 0.97038E+00_JPRB, 0.94757E+00_JPRB, 0.92101E+00_JPRB, 0.89520E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.89721E+00_JPRB, 0.86885E+00_JPRB, 0.85356E+00_JPRB, 0.84899E+00_JPRB, 0.84639E+00_JPRB, 0.84378E+00_JPRB, 0.84110E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.83298E+00_JPRB, 0.83367E+00_JPRB, 0.83499E+00_JPRB, 0.83475E+00_JPRB, 0.83404E+00_JPRB, 0.83176E+00_JPRB ,&
&  0.82891E+00_JPRB, 0.82415E+00_JPRB, 0.81910E+00_JPRB, 0.81109E+00_JPRB, 0.80251E+00_JPRB, 0.79203E+00_JPRB, 0.77736E+00_JPRB ,&
&  0.76595E+00_JPRB, 0.75593E+00_JPRB, 0.74284E+00_JPRB, 0.73919E+00_JPRB, 0.73828E+00_JPRB, 0.70862E+00_JPRB, 0.70307E+00_JPRB ,&
&  0.71488E+00_JPRB, 0.71817E+00_JPRB, 0.71948E+00_JPRB, 0.72043E+00_JPRB, 0.71396E+00_JPRB, 0.69850E+00_JPRB, 0.69265E+00_JPRB ,&
&  0.66886E+00_JPRB, 0.68983E+00_JPRB, 0.70443E+00_JPRB, 0.71700E+00_JPRB, 0.72902E+00_JPRB, 0.74492E+00_JPRB, 0.77869E+00_JPRB ,&
&  0.84243E+00_JPRB, 0.86906E+00_JPRB, 0.90037E+00_JPRB, 0.93066E+00_JPRB, 0.96936E+00_JPRB, 0.10094E+01_JPRB, 0.10463E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.11157E+01_JPRB, 0.11576E+01_JPRB, 0.12105E+01_JPRB, 0.12546E+01_JPRB, 0.12950E+01_JPRB, 0.13249E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.13817E+01_JPRB, 0.13992E+01_JPRB, 0.14098E+01_JPRB, 0.14218E+01_JPRB, 0.14279E+01_JPRB, 0.14350E+01_JPRB ,&
&  0.14389E+01_JPRB, 0.14392E+01_JPRB, 0.14288E+01_JPRB, 0.14091E+01_JPRB, 0.13960E+01_JPRB, 0.13907E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.13707E+01_JPRB, 0.13584E+01_JPRB, 0.13425E+01_JPRB, 0.13231E+01_JPRB, 0.12976E+01_JPRB, 0.12752E+01_JPRB, 0.12478E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.12058E+01_JPRB, 0.11903E+01_JPRB, 0.11720E+01_JPRB, 0.11530E+01_JPRB, 0.11342E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.10951E+01_JPRB, 0.10789E+01_JPRB, 0.10662E+01_JPRB, 0.10528E+01_JPRB, 0.10433E+01_JPRB, 0.10303E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.99375E+00_JPRB, 0.97552E+00_JPRB, 0.95191E+00_JPRB, 0.92191E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.93271E+00_JPRB, 0.90268E+00_JPRB, 0.86996E+00_JPRB, 0.85126E+00_JPRB, 0.84604E+00_JPRB, 0.84111E+00_JPRB, 0.83669E+00_JPRB ,&
&  0.83398E+00_JPRB, 0.83593E+00_JPRB, 0.83840E+00_JPRB, 0.83893E+00_JPRB, 0.83698E+00_JPRB, 0.83288E+00_JPRB, 0.82715E+00_JPRB ,&
&  0.81934E+00_JPRB, 0.80960E+00_JPRB, 0.80153E+00_JPRB, 0.79329E+00_JPRB, 0.78417E+00_JPRB, 0.77482E+00_JPRB, 0.75991E+00_JPRB ,&
&  0.74605E+00_JPRB, 0.73340E+00_JPRB, 0.68935E+00_JPRB, 0.68892E+00_JPRB, 0.68155E+00_JPRB, 0.68032E+00_JPRB, 0.68526E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.66877E+00_JPRB, 0.65347E+00_JPRB, 0.63336E+00_JPRB, 0.61902E+00_JPRB, 0.60333E+00_JPRB, 0.59679E+00_JPRB ,&
&  0.59533E+00_JPRB, 0.60036E+00_JPRB, 0.61299E+00_JPRB, 0.62535E+00_JPRB, 0.63864E+00_JPRB, 0.65016E+00_JPRB, 0.67103E+00_JPRB ,&
&  0.72343E+00_JPRB, 0.79496E+00_JPRB, 0.88287E+00_JPRB, 0.93142E+00_JPRB, 0.99089E+00_JPRB, 0.10475E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.11798E+01_JPRB, 0.12296E+01_JPRB, 0.12837E+01_JPRB, 0.13378E+01_JPRB, 0.13839E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.14879E+01_JPRB, 0.15290E+01_JPRB, 0.15494E+01_JPRB, 0.15608E+01_JPRB, 0.15624E+01_JPRB, 0.15501E+01_JPRB, 0.15364E+01_JPRB ,&
&  0.15261E+01_JPRB, 0.15130E+01_JPRB, 0.14952E+01_JPRB, 0.14558E+01_JPRB, 0.14360E+01_JPRB, 0.14287E+01_JPRB, 0.14068E+01_JPRB ,&
&  0.13796E+01_JPRB, 0.13487E+01_JPRB, 0.13180E+01_JPRB, 0.12750E+01_JPRB, 0.12378E+01_JPRB, 0.12241E+01_JPRB, 0.12123E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11901E+01_JPRB, 0.11780E+01_JPRB, 0.11579E+01_JPRB, 0.11271E+01_JPRB, 0.10972E+01_JPRB, 0.10723E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10304E+01_JPRB, 0.10125E+01_JPRB, 0.99433E+00_JPRB, 0.98309E+00_JPRB, 0.97410E+00_JPRB, 0.97128E+00_JPRB ,&
&  0.97195E+00_JPRB, 0.97366E+00_JPRB, 0.97278E+00_JPRB, 0.96814E+00_JPRB, 0.95390E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.10226E+01_JPRB, 0.10096E+01_JPRB, 0.98164E+00_JPRB, 0.94427E+00_JPRB, 0.91425E+00_JPRB, 0.89217E+00_JPRB, 0.88051E+00_JPRB ,&
&  0.87276E+00_JPRB, 0.86889E+00_JPRB, 0.86411E+00_JPRB, 0.85911E+00_JPRB, 0.85137E+00_JPRB, 0.84240E+00_JPRB, 0.83279E+00_JPRB ,&
&  0.81767E+00_JPRB, 0.80294E+00_JPRB, 0.78849E+00_JPRB, 0.77906E+00_JPRB, 0.77128E+00_JPRB, 0.76527E+00_JPRB, 0.75967E+00_JPRB ,&
&  0.75379E+00_JPRB, 0.74644E+00_JPRB, 0.72251E+00_JPRB, 0.71447E+00_JPRB, 0.70501E+00_JPRB, 0.70469E+00_JPRB, 0.69978E+00_JPRB ,&
&  0.71189E+00_JPRB, 0.64329E+00_JPRB, 0.63769E+00_JPRB, 0.62062E+00_JPRB, 0.60826E+00_JPRB, 0.59369E+00_JPRB, 0.57824E+00_JPRB ,&
&  0.57051E+00_JPRB, 0.56059E+00_JPRB, 0.55846E+00_JPRB, 0.57007E+00_JPRB, 0.57982E+00_JPRB, 0.59888E+00_JPRB, 0.61857E+00_JPRB ,&
&  0.65118E+00_JPRB, 0.79915E+00_JPRB, 0.87426E+00_JPRB, 0.92840E+00_JPRB, 0.98479E+00_JPRB, 0.10349E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.11375E+01_JPRB, 0.11756E+01_JPRB, 0.11953E+01_JPRB, 0.12923E+01_JPRB, 0.13630E+01_JPRB, 0.14412E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.15858E+01_JPRB, 0.16490E+01_JPRB, 0.16982E+01_JPRB, 0.17362E+01_JPRB, 0.17387E+01_JPRB, 0.17178E+01_JPRB ,&
&  0.17024E+01_JPRB, 0.16883E+01_JPRB, 0.16793E+01_JPRB, 0.16263E+01_JPRB, 0.15928E+01_JPRB, 0.15689E+01_JPRB, 0.15135E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.13968E+01_JPRB, 0.13407E+01_JPRB, 0.12814E+01_JPRB, 0.12267E+01_JPRB, 0.11951E+01_JPRB, 0.11804E+01_JPRB ,&
&  0.11757E+01_JPRB, 0.11644E+01_JPRB, 0.11431E+01_JPRB, 0.11108E+01_JPRB, 0.10769E+01_JPRB, 0.10544E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.97865E+00_JPRB, 0.96188E+00_JPRB, 0.95050E+00_JPRB, 0.94744E+00_JPRB, 0.95037E+00_JPRB, 0.95887E+00_JPRB ,&
&  0.96866E+00_JPRB, 0.98116E+00_JPRB, 0.99562E+00_JPRB, 0.10098E+01_JPRB, 0.10202E+01_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.10214E+01_JPRB, 0.10291E+01_JPRB, 0.10295E+01_JPRB, 0.10211E+01_JPRB, 0.10045E+01_JPRB, 0.98309E+00_JPRB, 0.96182E+00_JPRB ,&
&  0.94167E+00_JPRB, 0.92580E+00_JPRB, 0.91003E+00_JPRB, 0.89722E+00_JPRB, 0.88191E+00_JPRB, 0.87051E+00_JPRB, 0.85757E+00_JPRB ,&
&  0.84900E+00_JPRB, 0.84430E+00_JPRB, 0.83381E+00_JPRB, 0.82195E+00_JPRB, 0.80294E+00_JPRB, 0.78644E+00_JPRB, 0.77351E+00_JPRB ,&
&  0.76181E+00_JPRB, 0.75182E+00_JPRB, 0.72901E+00_JPRB, 0.71298E+00_JPRB, 0.70442E+00_JPRB, 0.69445E+00_JPRB, 0.69991E+00_JPRB ,&
&  0.69593E+00_JPRB, 0.69964E+00_JPRB, 0.70322E+00_JPRB, 0.69605E+00_JPRB, 0.68683E+00_JPRB, 0.57250E+00_JPRB, 0.54444E+00_JPRB ,&
&  0.53733E+00_JPRB, 0.52970E+00_JPRB, 0.52915E+00_JPRB, 0.53701E+00_JPRB, 0.54996E+00_JPRB, 0.58548E+00_JPRB, 0.61110E+00_JPRB ,&
&  0.62587E+00_JPRB, 0.78082E+00_JPRB, 0.84565E+00_JPRB, 0.90223E+00_JPRB, 0.94573E+00_JPRB, 0.97842E+00_JPRB, 0.99817E+00_JPRB ,&
&  0.10192E+01_JPRB, 0.10494E+01_JPRB, 0.10688E+01_JPRB, 0.11094E+01_JPRB, 0.11870E+01_JPRB, 0.13145E+01_JPRB, 0.14354E+01_JPRB ,&
&  0.15709E+01_JPRB, 0.16515E+01_JPRB, 0.17263E+01_JPRB, 0.17725E+01_JPRB, 0.18342E+01_JPRB, 0.18736E+01_JPRB, 0.18863E+01_JPRB ,&
&  0.19038E+01_JPRB, 0.18792E+01_JPRB, 0.18901E+01_JPRB, 0.18567E+01_JPRB, 0.18267E+01_JPRB, 0.17907E+01_JPRB, 0.16865E+01_JPRB ,&
&  0.15632E+01_JPRB, 0.14802E+01_JPRB, 0.14034E+01_JPRB, 0.13056E+01_JPRB, 0.12334E+01_JPRB, 0.11815E+01_JPRB, 0.11608E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11416E+01_JPRB, 0.11105E+01_JPRB, 0.10747E+01_JPRB, 0.10504E+01_JPRB, 0.10345E+01_JPRB, 0.10081E+01_JPRB ,&
&  0.98521E+00_JPRB, 0.96304E+00_JPRB, 0.95010E+00_JPRB, 0.93813E+00_JPRB, 0.93017E+00_JPRB, 0.92828E+00_JPRB, 0.93167E+00_JPRB ,&
&  0.94006E+00_JPRB, 0.95250E+00_JPRB, 0.97009E+00_JPRB, 0.99097E+00_JPRB, 0.10135E+01_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.10070E+01_JPRB, 0.10212E+01_JPRB, 0.10272E+01_JPRB, 0.10235E+01_JPRB, 0.10073E+01_JPRB, 0.98659E+00_JPRB, 0.96800E+00_JPRB ,&
&  0.94991E+00_JPRB, 0.93285E+00_JPRB, 0.91654E+00_JPRB, 0.90380E+00_JPRB, 0.89229E+00_JPRB, 0.88327E+00_JPRB, 0.87604E+00_JPRB ,&
&  0.87041E+00_JPRB, 0.84598E+00_JPRB, 0.83502E+00_JPRB, 0.83259E+00_JPRB, 0.83234E+00_JPRB, 0.82814E+00_JPRB, 0.82109E+00_JPRB ,&
&  0.79723E+00_JPRB, 0.76450E+00_JPRB, 0.73373E+00_JPRB, 0.70699E+00_JPRB, 0.68493E+00_JPRB, 0.67449E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.68025E+00_JPRB, 0.67206E+00_JPRB, 0.67331E+00_JPRB, 0.69226E+00_JPRB, 0.69437E+00_JPRB, 0.62844E+00_JPRB, 0.54468E+00_JPRB ,&
&  0.53459E+00_JPRB, 0.51477E+00_JPRB, 0.51468E+00_JPRB, 0.52233E+00_JPRB, 0.53836E+00_JPRB, 0.56312E+00_JPRB, 0.60419E+00_JPRB ,&
&  0.65141E+00_JPRB, 0.76065E+00_JPRB, 0.82562E+00_JPRB, 0.87228E+00_JPRB, 0.91305E+00_JPRB, 0.94688E+00_JPRB, 0.96234E+00_JPRB ,&
&  0.97222E+00_JPRB, 0.99533E+00_JPRB, 0.10353E+01_JPRB, 0.10841E+01_JPRB, 0.11690E+01_JPRB, 0.13428E+01_JPRB, 0.15013E+01_JPRB ,&
&  0.17174E+01_JPRB, 0.18464E+01_JPRB, 0.18575E+01_JPRB, 0.18085E+01_JPRB, 0.18664E+01_JPRB, 0.19697E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.21776E+01_JPRB, 0.21532E+01_JPRB, 0.20519E+01_JPRB, 0.19874E+01_JPRB, 0.19472E+01_JPRB, 0.18013E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.15774E+01_JPRB, 0.15845E+01_JPRB, 0.14237E+01_JPRB, 0.13130E+01_JPRB, 0.12093E+01_JPRB, 0.11491E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.10777E+01_JPRB, 0.10364E+01_JPRB, 0.10050E+01_JPRB, 0.98864E+00_JPRB, 0.97815E+00_JPRB, 0.96679E+00_JPRB ,&
&  0.95340E+00_JPRB, 0.94119E+00_JPRB, 0.92901E+00_JPRB, 0.92083E+00_JPRB, 0.91209E+00_JPRB, 0.90831E+00_JPRB, 0.90847E+00_JPRB ,&
&  0.91395E+00_JPRB, 0.92593E+00_JPRB, 0.94335E+00_JPRB, 0.96435E+00_JPRB, 0.98930E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.10190E+01_JPRB, 0.10293E+01_JPRB, 0.10190E+01_JPRB, 0.99305E+00_JPRB, 0.95607E+00_JPRB, 0.90981E+00_JPRB, 0.87582E+00_JPRB ,&
&  0.85586E+00_JPRB, 0.84663E+00_JPRB, 0.83793E+00_JPRB, 0.82902E+00_JPRB, 0.82511E+00_JPRB, 0.82385E+00_JPRB, 0.82491E+00_JPRB ,&
&  0.82798E+00_JPRB, 0.83079E+00_JPRB, 0.82909E+00_JPRB, 0.81700E+00_JPRB, 0.81289E+00_JPRB, 0.81265E+00_JPRB, 0.81526E+00_JPRB ,&
&  0.80687E+00_JPRB, 0.77843E+00_JPRB, 0.74416E+00_JPRB, 0.71835E+00_JPRB, 0.69314E+00_JPRB, 0.68290E+00_JPRB, 0.66852E+00_JPRB ,&
&  0.64652E+00_JPRB, 0.64113E+00_JPRB, 0.65907E+00_JPRB, 0.68969E+00_JPRB, 0.68309E+00_JPRB, 0.69829E+00_JPRB, 0.58023E+00_JPRB ,&
&  0.54242E+00_JPRB, 0.52103E+00_JPRB, 0.51625E+00_JPRB, 0.52975E+00_JPRB, 0.56626E+00_JPRB, 0.60956E+00_JPRB, 0.64656E+00_JPRB ,&
&  0.68509E+00_JPRB, 0.75518E+00_JPRB, 0.79563E+00_JPRB, 0.82874E+00_JPRB, 0.86831E+00_JPRB, 0.90878E+00_JPRB, 0.93866E+00_JPRB ,&
&  0.95938E+00_JPRB, 0.99706E+00_JPRB, 0.10590E+01_JPRB, 0.11573E+01_JPRB, 0.13646E+01_JPRB, 0.13659E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.18866E+01_JPRB, 0.21193E+01_JPRB, 0.20075E+01_JPRB, 0.19115E+01_JPRB, 0.18923E+01_JPRB, 0.19624E+01_JPRB, 0.20765E+01_JPRB ,&
&  0.22760E+01_JPRB, 0.24426E+01_JPRB, 0.24320E+01_JPRB, 0.22057E+01_JPRB, 0.20912E+01_JPRB, 0.20234E+01_JPRB, 0.18266E+01_JPRB ,&
&  0.16088E+01_JPRB, 0.16422E+01_JPRB, 0.17144E+01_JPRB, 0.14542E+01_JPRB, 0.12745E+01_JPRB, 0.11593E+01_JPRB, 0.11087E+01_JPRB ,&
&  0.10767E+01_JPRB, 0.10501E+01_JPRB, 0.10204E+01_JPRB, 0.99339E+00_JPRB, 0.95983E+00_JPRB, 0.92908E+00_JPRB, 0.90234E+00_JPRB ,&
&  0.88314E+00_JPRB, 0.86965E+00_JPRB, 0.86793E+00_JPRB, 0.87257E+00_JPRB, 0.87515E+00_JPRB, 0.87831E+00_JPRB, 0.88919E+00_JPRB ,&
&  0.90501E+00_JPRB, 0.91310E+00_JPRB, 0.92989E+00_JPRB, 0.95409E+00_JPRB, 0.98763E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.98308E+00_JPRB, 0.98486E+00_JPRB, 0.96916E+00_JPRB, 0.93892E+00_JPRB, 0.89923E+00_JPRB, 0.84474E+00_JPRB, 0.80058E+00_JPRB ,&
&  0.77437E+00_JPRB, 0.75895E+00_JPRB, 0.74928E+00_JPRB, 0.74419E+00_JPRB, 0.74415E+00_JPRB, 0.74917E+00_JPRB, 0.74927E+00_JPRB ,&
&  0.75163E+00_JPRB, 0.75411E+00_JPRB, 0.75899E+00_JPRB, 0.76636E+00_JPRB, 0.77010E+00_JPRB, 0.77231E+00_JPRB, 0.77539E+00_JPRB ,&
&  0.77814E+00_JPRB, 0.76495E+00_JPRB, 0.73595E+00_JPRB, 0.71438E+00_JPRB, 0.71519E+00_JPRB, 0.72975E+00_JPRB, 0.70901E+00_JPRB ,&
&  0.66355E+00_JPRB, 0.64226E+00_JPRB, 0.64114E+00_JPRB, 0.69741E+00_JPRB, 0.73694E+00_JPRB, 0.74803E+00_JPRB, 0.62943E+00_JPRB ,&
&  0.56512E+00_JPRB, 0.53982E+00_JPRB, 0.54445E+00_JPRB, 0.57863E+00_JPRB, 0.63530E+00_JPRB, 0.69254E+00_JPRB, 0.76667E+00_JPRB ,&
&  0.81431E+00_JPRB, 0.85515E+00_JPRB, 0.86749E+00_JPRB, 0.89231E+00_JPRB, 0.93408E+00_JPRB, 0.97298E+00_JPRB, 0.10156E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.10922E+01_JPRB, 0.12165E+01_JPRB, 0.11168E+01_JPRB, 0.11804E+01_JPRB, 0.12649E+01_JPRB, 0.14208E+01_JPRB ,&
&  0.17138E+01_JPRB, 0.18395E+01_JPRB, 0.18877E+01_JPRB, 0.20415E+01_JPRB, 0.19677E+01_JPRB, 0.19244E+01_JPRB, 0.19892E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.25768E+01_JPRB, 0.25201E+01_JPRB, 0.22952E+01_JPRB, 0.22113E+01_JPRB, 0.21573E+01_JPRB, 0.19161E+01_JPRB ,&
&  0.16155E+01_JPRB, 0.14474E+01_JPRB, 0.13037E+01_JPRB, 0.11357E+01_JPRB, 0.10678E+01_JPRB, 0.10278E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.98843E+00_JPRB, 0.97163E+00_JPRB, 0.96024E+00_JPRB, 0.90351E+00_JPRB, 0.85110E+00_JPRB, 0.79453E+00_JPRB, 0.75672E+00_JPRB ,&
&  0.73335E+00_JPRB, 0.73277E+00_JPRB, 0.74784E+00_JPRB, 0.77427E+00_JPRB, 0.82464E+00_JPRB, 0.87113E+00_JPRB, 0.90412E+00_JPRB ,&
&  0.91963E+00_JPRB, 0.90580E+00_JPRB, 0.90334E+00_JPRB, 0.91626E+00_JPRB, 0.95716E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.93383E+00_JPRB, 0.89692E+00_JPRB, 0.87302E+00_JPRB, 0.87641E+00_JPRB, 0.82785E+00_JPRB, 0.77754E+00_JPRB, 0.74724E+00_JPRB ,&
&  0.74638E+00_JPRB, 0.74166E+00_JPRB, 0.73956E+00_JPRB, 0.70998E+00_JPRB, 0.68406E+00_JPRB, 0.67065E+00_JPRB, 0.66574E+00_JPRB ,&
&  0.67786E+00_JPRB, 0.69364E+00_JPRB, 0.70730E+00_JPRB, 0.70753E+00_JPRB, 0.70594E+00_JPRB, 0.70923E+00_JPRB, 0.72022E+00_JPRB ,&
&  0.73439E+00_JPRB, 0.73909E+00_JPRB, 0.72940E+00_JPRB, 0.72129E+00_JPRB, 0.74012E+00_JPRB, 0.76219E+00_JPRB, 0.73194E+00_JPRB ,&
&  0.69799E+00_JPRB, 0.66925E+00_JPRB, 0.66235E+00_JPRB, 0.70323E+00_JPRB, 0.77348E+00_JPRB, 0.79721E+00_JPRB, 0.67971E+00_JPRB ,&
&  0.59752E+00_JPRB, 0.58391E+00_JPRB, 0.63149E+00_JPRB, 0.74951E+00_JPRB, 0.80309E+00_JPRB, 0.85875E+00_JPRB, 0.92793E+00_JPRB ,&
&  0.88270E+00_JPRB, 0.88505E+00_JPRB, 0.91407E+00_JPRB, 0.10550E+01_JPRB, 0.10972E+01_JPRB, 0.11183E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.11998E+01_JPRB, 0.13581E+01_JPRB, 0.12081E+01_JPRB, 0.13185E+01_JPRB, 0.15808E+01_JPRB, 0.16074E+01_JPRB ,&
&  0.17450E+01_JPRB, 0.18191E+01_JPRB, 0.20574E+01_JPRB, 0.20475E+01_JPRB, 0.19326E+01_JPRB, 0.19155E+01_JPRB, 0.20810E+01_JPRB ,&
&  0.24762E+01_JPRB, 0.27949E+01_JPRB, 0.27138E+01_JPRB, 0.25883E+01_JPRB, 0.25004E+01_JPRB, 0.23699E+01_JPRB, 0.21000E+01_JPRB ,&
&  0.17277E+01_JPRB, 0.14496E+01_JPRB, 0.12403E+01_JPRB, 0.10606E+01_JPRB, 0.97533E+00_JPRB, 0.94353E+00_JPRB, 0.91042E+00_JPRB ,&
&  0.90263E+00_JPRB, 0.89522E+00_JPRB, 0.88722E+00_JPRB, 0.82616E+00_JPRB, 0.77330E+00_JPRB, 0.73392E+00_JPRB, 0.70463E+00_JPRB ,&
&  0.69623E+00_JPRB, 0.68512E+00_JPRB, 0.67940E+00_JPRB, 0.70426E+00_JPRB, 0.77421E+00_JPRB, 0.85682E+00_JPRB, 0.89824E+00_JPRB ,&
&  0.88610E+00_JPRB, 0.85624E+00_JPRB, 0.87746E+00_JPRB, 0.92270E+00_JPRB, 0.94767E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.94815E+00_JPRB, 0.94078E+00_JPRB, 0.93380E+00_JPRB, 0.92111E+00_JPRB, 0.89522E+00_JPRB, 0.83375E+00_JPRB, 0.79510E+00_JPRB ,&
&  0.80377E+00_JPRB, 0.77717E+00_JPRB, 0.76212E+00_JPRB, 0.72419E+00_JPRB, 0.68874E+00_JPRB, 0.65318E+00_JPRB, 0.63674E+00_JPRB ,&
&  0.62237E+00_JPRB, 0.63742E+00_JPRB, 0.66953E+00_JPRB, 0.68053E+00_JPRB, 0.68260E+00_JPRB, 0.68304E+00_JPRB, 0.69759E+00_JPRB ,&
&  0.71410E+00_JPRB, 0.73202E+00_JPRB, 0.74105E+00_JPRB, 0.74905E+00_JPRB, 0.76546E+00_JPRB, 0.78623E+00_JPRB, 0.77936E+00_JPRB ,&
&  0.76826E+00_JPRB, 0.73974E+00_JPRB, 0.71706E+00_JPRB, 0.74149E+00_JPRB, 0.80486E+00_JPRB, 0.86427E+00_JPRB, 0.82726E+00_JPRB ,&
&  0.67073E+00_JPRB, 0.67869E+00_JPRB, 0.84856E+00_JPRB, 0.91466E+00_JPRB, 0.97638E+00_JPRB, 0.10153E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.11537E+01_JPRB, 0.11863E+01_JPRB, 0.12122E+01_JPRB, 0.12324E+01_JPRB, 0.12599E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.13441E+01_JPRB, 0.11353E+01_JPRB, 0.11026E+01_JPRB, 0.12044E+01_JPRB, 0.15799E+01_JPRB, 0.17264E+01_JPRB, 0.17275E+01_JPRB ,&
&  0.18258E+01_JPRB, 0.18474E+01_JPRB, 0.19644E+01_JPRB, 0.21359E+01_JPRB, 0.22010E+01_JPRB, 0.23364E+01_JPRB, 0.25281E+01_JPRB ,&
&  0.28199E+01_JPRB, 0.31285E+01_JPRB, 0.31342E+01_JPRB, 0.30398E+01_JPRB, 0.28602E+01_JPRB, 0.26102E+01_JPRB, 0.22282E+01_JPRB ,&
&  0.17740E+01_JPRB, 0.14557E+01_JPRB, 0.11946E+01_JPRB, 0.10155E+01_JPRB, 0.91359E+00_JPRB, 0.86859E+00_JPRB, 0.83692E+00_JPRB ,&
&  0.83622E+00_JPRB, 0.83380E+00_JPRB, 0.82333E+00_JPRB, 0.77801E+00_JPRB, 0.74460E+00_JPRB, 0.75344E+00_JPRB, 0.76516E+00_JPRB ,&
&  0.77625E+00_JPRB, 0.77198E+00_JPRB, 0.73235E+00_JPRB, 0.75370E+00_JPRB, 0.82235E+00_JPRB, 0.89246E+00_JPRB, 0.91828E+00_JPRB ,&
&  0.91242E+00_JPRB, 0.88150E+00_JPRB, 0.89509E+00_JPRB, 0.92771E+00_JPRB, 0.93918E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.97725E+00_JPRB, 0.98152E+00_JPRB, 0.96549E+00_JPRB, 0.95320E+00_JPRB, 0.93278E+00_JPRB, 0.85731E+00_JPRB, 0.83373E+00_JPRB ,&
&  0.90511E+00_JPRB, 0.84595E+00_JPRB, 0.89549E+00_JPRB, 0.78520E+00_JPRB, 0.75542E+00_JPRB, 0.71694E+00_JPRB, 0.68932E+00_JPRB ,&
&  0.65350E+00_JPRB, 0.64351E+00_JPRB, 0.66292E+00_JPRB, 0.68464E+00_JPRB, 0.68697E+00_JPRB, 0.67974E+00_JPRB, 0.68459E+00_JPRB ,&
&  0.69467E+00_JPRB, 0.72256E+00_JPRB, 0.76068E+00_JPRB, 0.77933E+00_JPRB, 0.79280E+00_JPRB, 0.81589E+00_JPRB, 0.82534E+00_JPRB ,&
&  0.79438E+00_JPRB, 0.78782E+00_JPRB, 0.78423E+00_JPRB, 0.79872E+00_JPRB, 0.83814E+00_JPRB, 0.90005E+00_JPRB, 0.94002E+00_JPRB ,&
&  0.95179E+00_JPRB, 0.84241E+00_JPRB, 0.95646E+00_JPRB, 0.99105E+00_JPRB, 0.10571E+01_JPRB, 0.10963E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.11608E+01_JPRB, 0.12013E+01_JPRB, 0.12462E+01_JPRB, 0.12876E+01_JPRB, 0.13315E+01_JPRB, 0.13838E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.15194E+01_JPRB, 0.11644E+01_JPRB, 0.12596E+01_JPRB, 0.13603E+01_JPRB, 0.17988E+01_JPRB, 0.20135E+01_JPRB, 0.20206E+01_JPRB ,&
&  0.23459E+01_JPRB, 0.22638E+01_JPRB, 0.24484E+01_JPRB, 0.26147E+01_JPRB, 0.28295E+01_JPRB, 0.31090E+01_JPRB, 0.32675E+01_JPRB ,&
&  0.33079E+01_JPRB, 0.34111E+01_JPRB, 0.35616E+01_JPRB, 0.32819E+01_JPRB, 0.29003E+01_JPRB, 0.25021E+01_JPRB, 0.20311E+01_JPRB ,&
&  0.17154E+01_JPRB, 0.14384E+01_JPRB, 0.12028E+01_JPRB, 0.10407E+01_JPRB, 0.91903E+00_JPRB, 0.84217E+00_JPRB, 0.80325E+00_JPRB ,&
&  0.78380E+00_JPRB, 0.76703E+00_JPRB, 0.76597E+00_JPRB, 0.74859E+00_JPRB, 0.73337E+00_JPRB, 0.74896E+00_JPRB, 0.79209E+00_JPRB ,&
&  0.80573E+00_JPRB, 0.81943E+00_JPRB, 0.87284E+00_JPRB, 0.97943E+00_JPRB, 0.10035E+01_JPRB, 0.10310E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.10049E+01_JPRB, 0.10484E+01_JPRB, 0.10212E+01_JPRB, 0.98305E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.10499E+01_JPRB, 0.10407E+01_JPRB, 0.10065E+01_JPRB, 0.98593E+00_JPRB, 0.98108E+00_JPRB, 0.93914E+00_JPRB, 0.90096E+00_JPRB ,&
&  0.96322E+00_JPRB, 0.95177E+00_JPRB, 0.95060E+00_JPRB, 0.93466E+00_JPRB, 0.91783E+00_JPRB, 0.84434E+00_JPRB, 0.79057E+00_JPRB ,&
&  0.74221E+00_JPRB, 0.72352E+00_JPRB, 0.74436E+00_JPRB, 0.75036E+00_JPRB, 0.74659E+00_JPRB, 0.72708E+00_JPRB, 0.71683E+00_JPRB ,&
&  0.70804E+00_JPRB, 0.72115E+00_JPRB, 0.76099E+00_JPRB, 0.79561E+00_JPRB, 0.80010E+00_JPRB, 0.80661E+00_JPRB, 0.82984E+00_JPRB ,&
&  0.80426E+00_JPRB, 0.80766E+00_JPRB, 0.81455E+00_JPRB, 0.77691E+00_JPRB, 0.87167E+00_JPRB, 0.91781E+00_JPRB, 0.94494E+00_JPRB ,&
&  0.99297E+00_JPRB, 0.10351E+01_JPRB, 0.10579E+01_JPRB, 0.10687E+01_JPRB, 0.10947E+01_JPRB, 0.11118E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.12055E+01_JPRB, 0.12556E+01_JPRB, 0.13180E+01_JPRB, 0.14349E+01_JPRB, 0.14063E+01_JPRB, 0.15640E+01_JPRB ,&
&  0.16625E+01_JPRB, 0.17082E+01_JPRB, 0.18423E+01_JPRB, 0.17179E+01_JPRB, 0.21602E+01_JPRB, 0.24254E+01_JPRB, 0.23008E+01_JPRB ,&
&  0.24002E+01_JPRB, 0.23789E+01_JPRB, 0.28647E+01_JPRB, 0.32588E+01_JPRB, 0.35998E+01_JPRB, 0.39014E+01_JPRB, 0.40008E+01_JPRB ,&
&  0.39346E+01_JPRB, 0.41141E+01_JPRB, 0.40098E+01_JPRB, 0.35870E+01_JPRB, 0.30532E+01_JPRB, 0.26556E+01_JPRB, 0.21495E+01_JPRB ,&
&  0.17334E+01_JPRB, 0.13992E+01_JPRB, 0.11929E+01_JPRB, 0.10900E+01_JPRB, 0.91658E+00_JPRB, 0.86087E+00_JPRB, 0.80324E+00_JPRB ,&
&  0.74796E+00_JPRB, 0.71787E+00_JPRB, 0.73605E+00_JPRB, 0.75962E+00_JPRB, 0.74608E+00_JPRB, 0.76124E+00_JPRB, 0.77744E+00_JPRB ,&
&  0.85088E+00_JPRB, 0.10061E+01_JPRB, 0.12970E+01_JPRB, 0.13565E+01_JPRB, 0.13374E+01_JPRB, 0.13245E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.12065E+01_JPRB, 0.11557E+01_JPRB, 0.10916E+01_JPRB, 0.10474E+01_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.11073E+01_JPRB, 0.10874E+01_JPRB, 0.10633E+01_JPRB, 0.10424E+01_JPRB, 0.10293E+01_JPRB, 0.10036E+01_JPRB, 0.98330E+00_JPRB ,&
&  0.99225E+00_JPRB, 0.98190E+00_JPRB, 0.97757E+00_JPRB, 0.96845E+00_JPRB, 0.94812E+00_JPRB, 0.95844E+00_JPRB, 0.88019E+00_JPRB ,&
&  0.81100E+00_JPRB, 0.80090E+00_JPRB, 0.78292E+00_JPRB, 0.80752E+00_JPRB, 0.84691E+00_JPRB, 0.83008E+00_JPRB, 0.81144E+00_JPRB ,&
&  0.78789E+00_JPRB, 0.76441E+00_JPRB, 0.76869E+00_JPRB, 0.78900E+00_JPRB, 0.80086E+00_JPRB, 0.81388E+00_JPRB, 0.83219E+00_JPRB ,&
&  0.79373E+00_JPRB, 0.75774E+00_JPRB, 0.74230E+00_JPRB, 0.75144E+00_JPRB, 0.82198E+00_JPRB, 0.92542E+00_JPRB, 0.98617E+00_JPRB ,&
&  0.10088E+01_JPRB, 0.10276E+01_JPRB, 0.10721E+01_JPRB, 0.10960E+01_JPRB, 0.11122E+01_JPRB, 0.11415E+01_JPRB, 0.11750E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.12473E+01_JPRB, 0.12994E+01_JPRB, 0.13861E+01_JPRB, 0.14030E+01_JPRB, 0.14967E+01_JPRB, 0.17659E+01_JPRB ,&
&  0.19214E+01_JPRB, 0.20303E+01_JPRB, 0.21146E+01_JPRB, 0.22971E+01_JPRB, 0.25923E+01_JPRB, 0.25931E+01_JPRB, 0.25959E+01_JPRB ,&
&  0.28013E+01_JPRB, 0.28969E+01_JPRB, 0.36243E+01_JPRB, 0.39559E+01_JPRB, 0.41202E+01_JPRB, 0.43996E+01_JPRB, 0.43674E+01_JPRB ,&
&  0.43416E+01_JPRB, 0.42558E+01_JPRB, 0.41565E+01_JPRB, 0.37527E+01_JPRB, 0.31681E+01_JPRB, 0.27498E+01_JPRB, 0.21542E+01_JPRB ,&
&  0.17667E+01_JPRB, 0.14240E+01_JPRB, 0.10926E+01_JPRB, 0.95718E+00_JPRB, 0.83457E+00_JPRB, 0.81119E+00_JPRB, 0.77415E+00_JPRB ,&
&  0.72668E+00_JPRB, 0.73699E+00_JPRB, 0.78311E+00_JPRB, 0.88177E+00_JPRB, 0.89098E+00_JPRB, 0.81739E+00_JPRB, 0.87471E+00_JPRB ,&
&  0.99554E+00_JPRB, 0.13543E+01_JPRB, 0.14999E+01_JPRB, 0.16442E+01_JPRB, 0.15711E+01_JPRB, 0.15493E+01_JPRB, 0.12851E+01_JPRB ,&
&  0.12910E+01_JPRB, 0.13790E+01_JPRB, 0.12712E+01_JPRB, 0.11857E+01_JPRB, 0.11380E+01_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.11536E+01_JPRB, 0.11233E+01_JPRB, 0.11107E+01_JPRB, 0.11137E+01_JPRB, 0.11129E+01_JPRB, 0.10744E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10573E+01_JPRB, 0.10307E+01_JPRB, 0.10189E+01_JPRB, 0.10122E+01_JPRB, 0.99387E+00_JPRB, 0.10015E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.94085E+00_JPRB, 0.94596E+00_JPRB, 0.85134E+00_JPRB, 0.84923E+00_JPRB, 0.10005E+01_JPRB, 0.94340E+00_JPRB, 0.92967E+00_JPRB ,&
&  0.88157E+00_JPRB, 0.84943E+00_JPRB, 0.80286E+00_JPRB, 0.79696E+00_JPRB, 0.81236E+00_JPRB, 0.83941E+00_JPRB, 0.85062E+00_JPRB ,&
&  0.79713E+00_JPRB, 0.74613E+00_JPRB, 0.73423E+00_JPRB, 0.74614E+00_JPRB, 0.79959E+00_JPRB, 0.88436E+00_JPRB, 0.99175E+00_JPRB ,&
&  0.10337E+01_JPRB, 0.10543E+01_JPRB, 0.10902E+01_JPRB, 0.11329E+01_JPRB, 0.11809E+01_JPRB, 0.12436E+01_JPRB, 0.12832E+01_JPRB ,&
&  0.13102E+01_JPRB, 0.13348E+01_JPRB, 0.13808E+01_JPRB, 0.14705E+01_JPRB, 0.14389E+01_JPRB, 0.15842E+01_JPRB, 0.20919E+01_JPRB ,&
&  0.24507E+01_JPRB, 0.23239E+01_JPRB, 0.25174E+01_JPRB, 0.33151E+01_JPRB, 0.36728E+01_JPRB, 0.36805E+01_JPRB, 0.35667E+01_JPRB ,&
&  0.36629E+01_JPRB, 0.38228E+01_JPRB, 0.40506E+01_JPRB, 0.43697E+01_JPRB, 0.45602E+01_JPRB, 0.47194E+01_JPRB, 0.45953E+01_JPRB ,&
&  0.44156E+01_JPRB, 0.43065E+01_JPRB, 0.41067E+01_JPRB, 0.37413E+01_JPRB, 0.30670E+01_JPRB, 0.25982E+01_JPRB, 0.20412E+01_JPRB ,&
&  0.17124E+01_JPRB, 0.13136E+01_JPRB, 0.92201E+00_JPRB, 0.77882E+00_JPRB, 0.72545E+00_JPRB, 0.78879E+00_JPRB, 0.10276E+01_JPRB ,&
&  0.80035E+00_JPRB, 0.88820E+00_JPRB, 0.10122E+01_JPRB, 0.11870E+01_JPRB, 0.12960E+01_JPRB, 0.12884E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.13718E+01_JPRB, 0.18219E+01_JPRB, 0.19824E+01_JPRB, 0.18151E+01_JPRB, 0.17919E+01_JPRB, 0.16672E+01_JPRB ,&
&  0.17338E+01_JPRB, 0.15569E+01_JPRB, 0.13944E+01_JPRB, 0.12644E+01_JPRB, 0.12124E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.11714E+01_JPRB, 0.11660E+01_JPRB, 0.11786E+01_JPRB, 0.12059E+01_JPRB, 0.12235E+01_JPRB, 0.11652E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.10942E+01_JPRB, 0.10695E+01_JPRB, 0.10598E+01_JPRB, 0.10450E+01_JPRB, 0.10368E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11401E+01_JPRB, 0.10209E+01_JPRB, 0.87014E+00_JPRB, 0.92521E+00_JPRB, 0.91309E+00_JPRB, 0.93129E+00_JPRB ,&
&  0.91347E+00_JPRB, 0.92531E+00_JPRB, 0.89043E+00_JPRB, 0.86152E+00_JPRB, 0.89089E+00_JPRB, 0.93718E+00_JPRB, 0.95138E+00_JPRB ,&
&  0.91581E+00_JPRB, 0.85084E+00_JPRB, 0.81928E+00_JPRB, 0.84079E+00_JPRB, 0.93196E+00_JPRB, 0.91390E+00_JPRB, 0.10290E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.11379E+01_JPRB, 0.11857E+01_JPRB, 0.12515E+01_JPRB, 0.13162E+01_JPRB, 0.13910E+01_JPRB, 0.14356E+01_JPRB ,&
&  0.14636E+01_JPRB, 0.14623E+01_JPRB, 0.14276E+01_JPRB, 0.15003E+01_JPRB, 0.15915E+01_JPRB, 0.17335E+01_JPRB, 0.20683E+01_JPRB ,&
&  0.23659E+01_JPRB, 0.24596E+01_JPRB, 0.29302E+01_JPRB, 0.38358E+01_JPRB, 0.41703E+01_JPRB, 0.41342E+01_JPRB, 0.43171E+01_JPRB ,&
&  0.45837E+01_JPRB, 0.48509E+01_JPRB, 0.51273E+01_JPRB, 0.51876E+01_JPRB, 0.51748E+01_JPRB, 0.48770E+01_JPRB, 0.45951E+01_JPRB ,&
&  0.42976E+01_JPRB, 0.42194E+01_JPRB, 0.41129E+01_JPRB, 0.35618E+01_JPRB, 0.28949E+01_JPRB, 0.23705E+01_JPRB, 0.20924E+01_JPRB ,&
&  0.20104E+01_JPRB, 0.10639E+01_JPRB, 0.75912E+00_JPRB, 0.63725E+00_JPRB, 0.62346E+00_JPRB, 0.66924E+00_JPRB, 0.83699E+00_JPRB ,&
&  0.78328E+00_JPRB, 0.10134E+01_JPRB, 0.14516E+01_JPRB, 0.16894E+01_JPRB, 0.20508E+01_JPRB, 0.24729E+01_JPRB, 0.18972E+01_JPRB ,&
&  0.15571E+01_JPRB, 0.16738E+01_JPRB, 0.22642E+01_JPRB, 0.23827E+01_JPRB, 0.21231E+01_JPRB, 0.20659E+01_JPRB, 0.20831E+01_JPRB ,&
&  0.19217E+01_JPRB, 0.17995E+01_JPRB, 0.15460E+01_JPRB, 0.13121E+01_JPRB, 0.12159E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.12856E+01_JPRB, 0.12762E+01_JPRB, 0.13072E+01_JPRB, 0.13376E+01_JPRB, 0.13274E+01_JPRB, 0.12708E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.12024E+01_JPRB, 0.11568E+01_JPRB, 0.11198E+01_JPRB, 0.11009E+01_JPRB, 0.10856E+01_JPRB, 0.10775E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.11404E+01_JPRB, 0.99544E+00_JPRB, 0.88331E+00_JPRB, 0.78063E+00_JPRB, 0.81583E+00_JPRB, 0.86964E+00_JPRB ,&
&  0.95361E+00_JPRB, 0.99531E+00_JPRB, 0.10528E+01_JPRB, 0.11177E+01_JPRB, 0.11896E+01_JPRB, 0.11578E+01_JPRB, 0.12105E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.11042E+01_JPRB, 0.98604E+00_JPRB, 0.10668E+01_JPRB, 0.10972E+01_JPRB, 0.10525E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.12897E+01_JPRB, 0.13423E+01_JPRB, 0.13865E+01_JPRB, 0.14346E+01_JPRB, 0.14772E+01_JPRB, 0.14695E+01_JPRB, 0.14338E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.13765E+01_JPRB, 0.14287E+01_JPRB, 0.14833E+01_JPRB, 0.16935E+01_JPRB, 0.18715E+01_JPRB, 0.21889E+01_JPRB ,&
&  0.25640E+01_JPRB, 0.25772E+01_JPRB, 0.27525E+01_JPRB, 0.38262E+01_JPRB, 0.50191E+01_JPRB, 0.50360E+01_JPRB, 0.51310E+01_JPRB ,&
&  0.54537E+01_JPRB, 0.55292E+01_JPRB, 0.54311E+01_JPRB, 0.51577E+01_JPRB, 0.49762E+01_JPRB, 0.47805E+01_JPRB, 0.45515E+01_JPRB ,&
&  0.42283E+01_JPRB, 0.41126E+01_JPRB, 0.39756E+01_JPRB, 0.35662E+01_JPRB, 0.29433E+01_JPRB, 0.26081E+01_JPRB, 0.23142E+01_JPRB ,&
&  0.20414E+01_JPRB, 0.13422E+01_JPRB, 0.88400E+00_JPRB, 0.73717E+00_JPRB, 0.70476E+00_JPRB, 0.73131E+00_JPRB, 0.90141E+00_JPRB ,&
&  0.95656E+00_JPRB, 0.13167E+01_JPRB, 0.19284E+01_JPRB, 0.22341E+01_JPRB, 0.34311E+01_JPRB, 0.41500E+01_JPRB, 0.37752E+01_JPRB ,&
&  0.29430E+01_JPRB, 0.24328E+01_JPRB, 0.28473E+01_JPRB, 0.28297E+01_JPRB, 0.25688E+01_JPRB, 0.23279E+01_JPRB, 0.22652E+01_JPRB ,&
&  0.21083E+01_JPRB, 0.19817E+01_JPRB, 0.17935E+01_JPRB, 0.15102E+01_JPRB, 0.13461E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.15284E+01_JPRB, 0.14618E+01_JPRB, 0.14317E+01_JPRB, 0.14149E+01_JPRB, 0.13387E+01_JPRB, 0.12852E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.12730E+01_JPRB, 0.12176E+01_JPRB, 0.11769E+01_JPRB, 0.11649E+01_JPRB, 0.11564E+01_JPRB, 0.11532E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.10510E+01_JPRB, 0.10530E+01_JPRB, 0.90388E+00_JPRB, 0.79265E+00_JPRB, 0.74977E+00_JPRB, 0.81600E+00_JPRB, 0.93248E+00_JPRB ,&
&  0.11013E+01_JPRB, 0.12825E+01_JPRB, 0.14874E+01_JPRB, 0.15920E+01_JPRB, 0.16745E+01_JPRB, 0.16835E+01_JPRB, 0.17123E+01_JPRB ,&
&  0.18294E+01_JPRB, 0.14765E+01_JPRB, 0.14144E+01_JPRB, 0.13892E+01_JPRB, 0.14167E+01_JPRB, 0.14232E+01_JPRB, 0.14462E+01_JPRB ,&
&  0.14285E+01_JPRB, 0.14347E+01_JPRB, 0.14422E+01_JPRB, 0.14457E+01_JPRB, 0.14456E+01_JPRB, 0.14344E+01_JPRB, 0.14186E+01_JPRB ,&
&  0.14490E+01_JPRB, 0.14290E+01_JPRB, 0.14392E+01_JPRB, 0.17294E+01_JPRB, 0.23473E+01_JPRB, 0.27164E+01_JPRB, 0.28683E+01_JPRB ,&
&  0.27707E+01_JPRB, 0.29375E+01_JPRB, 0.44603E+01_JPRB, 0.50391E+01_JPRB, 0.54101E+01_JPRB, 0.61339E+01_JPRB, 0.63755E+01_JPRB ,&
&  0.60947E+01_JPRB, 0.54945E+01_JPRB, 0.49433E+01_JPRB, 0.44018E+01_JPRB, 0.49534E+01_JPRB, 0.48695E+01_JPRB, 0.45298E+01_JPRB ,&
&  0.40750E+01_JPRB, 0.40031E+01_JPRB, 0.36473E+01_JPRB, 0.32874E+01_JPRB, 0.30222E+01_JPRB, 0.30281E+01_JPRB, 0.24409E+01_JPRB ,&
&  0.19541E+01_JPRB, 0.18748E+01_JPRB, 0.14076E+01_JPRB, 0.11743E+01_JPRB, 0.11003E+01_JPRB, 0.11178E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.14609E+01_JPRB, 0.18870E+01_JPRB, 0.25419E+01_JPRB, 0.32144E+01_JPRB, 0.46190E+01_JPRB, 0.55398E+01_JPRB, 0.53855E+01_JPRB ,&
&  0.42424E+01_JPRB, 0.39743E+01_JPRB, 0.35209E+01_JPRB, 0.32882E+01_JPRB, 0.30027E+01_JPRB, 0.26542E+01_JPRB, 0.24579E+01_JPRB ,&
&  0.22391E+01_JPRB, 0.20842E+01_JPRB, 0.19608E+01_JPRB, 0.17705E+01_JPRB, 0.16461E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.18467E+01_JPRB, 0.17371E+01_JPRB, 0.15816E+01_JPRB, 0.14690E+01_JPRB, 0.13635E+01_JPRB, 0.12987E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.12545E+01_JPRB, 0.12253E+01_JPRB, 0.12311E+01_JPRB, 0.12465E+01_JPRB, 0.12415E+01_JPRB, 0.12155E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.95714E+00_JPRB, 0.84326E+00_JPRB, 0.83589E+00_JPRB, 0.92304E+00_JPRB, 0.10126E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.13861E+01_JPRB, 0.17810E+01_JPRB, 0.20376E+01_JPRB, 0.23566E+01_JPRB, 0.25162E+01_JPRB, 0.25313E+01_JPRB, 0.24538E+01_JPRB ,&
&  0.22286E+01_JPRB, 0.21644E+01_JPRB, 0.20728E+01_JPRB, 0.19197E+01_JPRB, 0.17702E+01_JPRB, 0.16693E+01_JPRB, 0.15900E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.14612E+01_JPRB, 0.14516E+01_JPRB, 0.14424E+01_JPRB, 0.14819E+01_JPRB, 0.15103E+01_JPRB, 0.15516E+01_JPRB ,&
&  0.15267E+01_JPRB, 0.15217E+01_JPRB, 0.17385E+01_JPRB, 0.24901E+01_JPRB, 0.31267E+01_JPRB, 0.35808E+01_JPRB, 0.34327E+01_JPRB ,&
&  0.38223E+01_JPRB, 0.42225E+01_JPRB, 0.44796E+01_JPRB, 0.51529E+01_JPRB, 0.44761E+01_JPRB, 0.51644E+01_JPRB, 0.60541E+01_JPRB ,&
&  0.58735E+01_JPRB, 0.53015E+01_JPRB, 0.49658E+01_JPRB, 0.46343E+01_JPRB, 0.45501E+01_JPRB, 0.56065E+01_JPRB, 0.49827E+01_JPRB ,&
&  0.42781E+01_JPRB, 0.39625E+01_JPRB, 0.35362E+01_JPRB, 0.34619E+01_JPRB, 0.29640E+01_JPRB, 0.26262E+01_JPRB, 0.19047E+01_JPRB ,&
&  0.19441E+01_JPRB, 0.17611E+01_JPRB, 0.17818E+01_JPRB, 0.16373E+01_JPRB, 0.16978E+01_JPRB, 0.16591E+01_JPRB, 0.17535E+01_JPRB ,&
&  0.21790E+01_JPRB, 0.26966E+01_JPRB, 0.40242E+01_JPRB, 0.50167E+01_JPRB, 0.71914E+01_JPRB, 0.66813E+01_JPRB, 0.65457E+01_JPRB ,&
&  0.56342E+01_JPRB, 0.51113E+01_JPRB, 0.41482E+01_JPRB, 0.39610E+01_JPRB, 0.35033E+01_JPRB, 0.30881E+01_JPRB, 0.27141E+01_JPRB ,&
&  0.23771E+01_JPRB, 0.21550E+01_JPRB, 0.20051E+01_JPRB, 0.19025E+01_JPRB, 0.18915E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.17890E+01_JPRB, 0.16917E+01_JPRB, 0.15829E+01_JPRB, 0.14682E+01_JPRB, 0.13441E+01_JPRB, 0.12747E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.12290E+01_JPRB, 0.12454E+01_JPRB, 0.12513E+01_JPRB, 0.12523E+01_JPRB, 0.12456E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.96659E+00_JPRB, 0.84737E+00_JPRB, 0.84083E+00_JPRB, 0.10674E+01_JPRB, 0.12769E+01_JPRB, 0.12997E+01_JPRB ,&
&  0.18879E+01_JPRB, 0.24318E+01_JPRB, 0.26405E+01_JPRB, 0.30205E+01_JPRB, 0.34536E+01_JPRB, 0.36863E+01_JPRB, 0.33875E+01_JPRB ,&
&  0.31997E+01_JPRB, 0.28928E+01_JPRB, 0.24763E+01_JPRB, 0.21474E+01_JPRB, 0.18719E+01_JPRB, 0.16703E+01_JPRB, 0.15153E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.13712E+01_JPRB, 0.13450E+01_JPRB, 0.13538E+01_JPRB, 0.14147E+01_JPRB, 0.14093E+01_JPRB, 0.14927E+01_JPRB ,&
&  0.15916E+01_JPRB, 0.17135E+01_JPRB, 0.21696E+01_JPRB, 0.30402E+01_JPRB, 0.39657E+01_JPRB, 0.44288E+01_JPRB, 0.44365E+01_JPRB ,&
&  0.42819E+01_JPRB, 0.43382E+01_JPRB, 0.46129E+01_JPRB, 0.47510E+01_JPRB, 0.49704E+01_JPRB, 0.53347E+01_JPRB, 0.59642E+01_JPRB ,&
&  0.47569E+01_JPRB, 0.39586E+01_JPRB, 0.34184E+01_JPRB, 0.36331E+01_JPRB, 0.48984E+01_JPRB, 0.61288E+01_JPRB, 0.59630E+01_JPRB ,&
&  0.51495E+01_JPRB, 0.45128E+01_JPRB, 0.39600E+01_JPRB, 0.34850E+01_JPRB, 0.34874E+01_JPRB, 0.23910E+01_JPRB, 0.25906E+01_JPRB ,&
&  0.26558E+01_JPRB, 0.26704E+01_JPRB, 0.24452E+01_JPRB, 0.20919E+01_JPRB, 0.20517E+01_JPRB, 0.24173E+01_JPRB, 0.28687E+01_JPRB ,&
&  0.36244E+01_JPRB, 0.43758E+01_JPRB, 0.73646E+01_JPRB, 0.83076E+01_JPRB, 0.89670E+01_JPRB, 0.75706E+01_JPRB, 0.68079E+01_JPRB ,&
&  0.54679E+01_JPRB, 0.50653E+01_JPRB, 0.48610E+01_JPRB, 0.44179E+01_JPRB, 0.38915E+01_JPRB, 0.34808E+01_JPRB, 0.29837E+01_JPRB ,&
&  0.25891E+01_JPRB, 0.22760E+01_JPRB, 0.20429E+01_JPRB, 0.18942E+01_JPRB, 0.18449E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.17512E+01_JPRB, 0.16730E+01_JPRB, 0.15560E+01_JPRB, 0.13839E+01_JPRB, 0.12845E+01_JPRB, 0.12487E+01_JPRB, 0.12533E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.12782E+01_JPRB, 0.12673E+01_JPRB, 0.12519E+01_JPRB, 0.12591E+01_JPRB, 0.12377E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10433E+01_JPRB, 0.90692E+00_JPRB, 0.95783E+00_JPRB, 0.12756E+01_JPRB, 0.15881E+01_JPRB, 0.16818E+01_JPRB ,&
&  0.26803E+01_JPRB, 0.28993E+01_JPRB, 0.27202E+01_JPRB, 0.30311E+01_JPRB, 0.42073E+01_JPRB, 0.40572E+01_JPRB, 0.37014E+01_JPRB ,&
&  0.34302E+01_JPRB, 0.29109E+01_JPRB, 0.24078E+01_JPRB, 0.20253E+01_JPRB, 0.17530E+01_JPRB, 0.15788E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.13746E+01_JPRB, 0.13336E+01_JPRB, 0.13433E+01_JPRB, 0.13954E+01_JPRB, 0.13988E+01_JPRB, 0.14153E+01_JPRB, 0.15532E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.18772E+01_JPRB, 0.23427E+01_JPRB, 0.31171E+01_JPRB, 0.41047E+01_JPRB, 0.48290E+01_JPRB, 0.58174E+01_JPRB ,&
&  0.52127E+01_JPRB, 0.52978E+01_JPRB, 0.49911E+01_JPRB, 0.57506E+01_JPRB, 0.54537E+01_JPRB, 0.56280E+01_JPRB, 0.58331E+01_JPRB ,&
&  0.43087E+01_JPRB, 0.36622E+01_JPRB, 0.38705E+01_JPRB, 0.49371E+01_JPRB, 0.48341E+01_JPRB, 0.53090E+01_JPRB, 0.65592E+01_JPRB ,&
&  0.52565E+01_JPRB, 0.46552E+01_JPRB, 0.43525E+01_JPRB, 0.43887E+01_JPRB, 0.28210E+01_JPRB, 0.21985E+01_JPRB, 0.33370E+01_JPRB ,&
&  0.31546E+01_JPRB, 0.18886E+01_JPRB, 0.14559E+01_JPRB, 0.17521E+01_JPRB, 0.14687E+01_JPRB, 0.19782E+01_JPRB, 0.35782E+01_JPRB ,&
&  0.46505E+01_JPRB, 0.57669E+01_JPRB, 0.93630E+01_JPRB, 0.82313E+01_JPRB, 0.81196E+01_JPRB, 0.79027E+01_JPRB, 0.64123E+01_JPRB ,&
&  0.56693E+01_JPRB, 0.44951E+01_JPRB, 0.45104E+01_JPRB, 0.40439E+01_JPRB, 0.37437E+01_JPRB, 0.35263E+01_JPRB, 0.31318E+01_JPRB ,&
&  0.27077E+01_JPRB, 0.23427E+01_JPRB, 0.20619E+01_JPRB, 0.18888E+01_JPRB, 0.18125E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.16380E+01_JPRB, 0.15431E+01_JPRB, 0.14891E+01_JPRB, 0.13909E+01_JPRB, 0.13145E+01_JPRB, 0.12921E+01_JPRB, 0.13283E+01_JPRB ,&
&  0.13213E+01_JPRB, 0.12706E+01_JPRB, 0.12188E+01_JPRB, 0.12079E+01_JPRB, 0.12238E+01_JPRB, 0.12140E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.10520E+01_JPRB, 0.10836E+01_JPRB, 0.12321E+01_JPRB, 0.17696E+01_JPRB, 0.19300E+01_JPRB, 0.25524E+01_JPRB ,&
&  0.33742E+01_JPRB, 0.30457E+01_JPRB, 0.27822E+01_JPRB, 0.30320E+01_JPRB, 0.32731E+01_JPRB, 0.32683E+01_JPRB, 0.33162E+01_JPRB ,&
&  0.28859E+01_JPRB, 0.24605E+01_JPRB, 0.21766E+01_JPRB, 0.19619E+01_JPRB, 0.17229E+01_JPRB, 0.15335E+01_JPRB, 0.14173E+01_JPRB ,&
&  0.13756E+01_JPRB, 0.13757E+01_JPRB, 0.14458E+01_JPRB, 0.14060E+01_JPRB, 0.14243E+01_JPRB, 0.14780E+01_JPRB, 0.16347E+01_JPRB ,&
&  0.18149E+01_JPRB, 0.20265E+01_JPRB, 0.24107E+01_JPRB, 0.31992E+01_JPRB, 0.47595E+01_JPRB, 0.58601E+01_JPRB, 0.57561E+01_JPRB ,&
&  0.52683E+01_JPRB, 0.51248E+01_JPRB, 0.55685E+01_JPRB, 0.58855E+01_JPRB, 0.53557E+01_JPRB, 0.56359E+01_JPRB, 0.55307E+01_JPRB ,&
&  0.48745E+01_JPRB, 0.44578E+01_JPRB, 0.41833E+01_JPRB, 0.51677E+01_JPRB, 0.55259E+01_JPRB, 0.53172E+01_JPRB, 0.59831E+01_JPRB ,&
&  0.56273E+01_JPRB, 0.47645E+01_JPRB, 0.38110E+01_JPRB, 0.31995E+01_JPRB, 0.32289E+01_JPRB, 0.30409E+01_JPRB, 0.22796E+01_JPRB ,&
&  0.19206E+01_JPRB, 0.16067E+01_JPRB, 0.13852E+01_JPRB, 0.14106E+01_JPRB, 0.12911E+01_JPRB, 0.19083E+01_JPRB, 0.32975E+01_JPRB ,&
&  0.54997E+01_JPRB, 0.66786E+01_JPRB, 0.83031E+01_JPRB, 0.76855E+01_JPRB, 0.74022E+01_JPRB, 0.81816E+01_JPRB, 0.76725E+01_JPRB ,&
&  0.49430E+01_JPRB, 0.41178E+01_JPRB, 0.41107E+01_JPRB, 0.38514E+01_JPRB, 0.36126E+01_JPRB, 0.33795E+01_JPRB, 0.30031E+01_JPRB ,&
&  0.25623E+01_JPRB, 0.21632E+01_JPRB, 0.19549E+01_JPRB, 0.18730E+01_JPRB, 0.17793E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.13758E+01_JPRB, 0.13537E+01_JPRB, 0.13737E+01_JPRB, 0.13533E+01_JPRB, 0.13196E+01_JPRB, 0.13302E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.11822E+01_JPRB, 0.11641E+01_JPRB, 0.11870E+01_JPRB, 0.11946E+01_JPRB, 0.11926E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.10805E+01_JPRB, 0.11770E+01_JPRB, 0.15248E+01_JPRB, 0.24702E+01_JPRB, 0.25257E+01_JPRB, 0.35517E+01_JPRB ,&
&  0.38145E+01_JPRB, 0.29708E+01_JPRB, 0.27677E+01_JPRB, 0.26668E+01_JPRB, 0.28651E+01_JPRB, 0.26557E+01_JPRB, 0.23842E+01_JPRB ,&
&  0.19549E+01_JPRB, 0.19123E+01_JPRB, 0.18099E+01_JPRB, 0.16911E+01_JPRB, 0.15535E+01_JPRB, 0.14588E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.14697E+01_JPRB, 0.15106E+01_JPRB, 0.14641E+01_JPRB, 0.14551E+01_JPRB, 0.15183E+01_JPRB, 0.15913E+01_JPRB, 0.17438E+01_JPRB ,&
&  0.19392E+01_JPRB, 0.21634E+01_JPRB, 0.25460E+01_JPRB, 0.37551E+01_JPRB, 0.47162E+01_JPRB, 0.49924E+01_JPRB, 0.55626E+01_JPRB ,&
&  0.58078E+01_JPRB, 0.59954E+01_JPRB, 0.53879E+01_JPRB, 0.60845E+01_JPRB, 0.53413E+01_JPRB, 0.52577E+01_JPRB, 0.52289E+01_JPRB ,&
&  0.47441E+01_JPRB, 0.48223E+01_JPRB, 0.43854E+01_JPRB, 0.45655E+01_JPRB, 0.56369E+01_JPRB, 0.59787E+01_JPRB, 0.57946E+01_JPRB ,&
&  0.57515E+01_JPRB, 0.49527E+01_JPRB, 0.41909E+01_JPRB, 0.39207E+01_JPRB, 0.51770E+01_JPRB, 0.61340E+01_JPRB, 0.29097E+01_JPRB ,&
&  0.19980E+01_JPRB, 0.18466E+01_JPRB, 0.14622E+01_JPRB, 0.12808E+01_JPRB, 0.13714E+01_JPRB, 0.19956E+01_JPRB, 0.41227E+01_JPRB ,&
&  0.48299E+01_JPRB, 0.79092E+01_JPRB, 0.84575E+01_JPRB, 0.86898E+01_JPRB, 0.80206E+01_JPRB, 0.74783E+01_JPRB, 0.57700E+01_JPRB ,&
&  0.56905E+01_JPRB, 0.49953E+01_JPRB, 0.42313E+01_JPRB, 0.38043E+01_JPRB, 0.33647E+01_JPRB, 0.29426E+01_JPRB, 0.26258E+01_JPRB ,&
&  0.23701E+01_JPRB, 0.20673E+01_JPRB, 0.18765E+01_JPRB, 0.17360E+01_JPRB, 0.15285E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.11483E+01_JPRB, 0.11915E+01_JPRB, 0.12634E+01_JPRB, 0.12821E+01_JPRB, 0.12675E+01_JPRB, 0.12867E+01_JPRB, 0.12372E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.11456E+01_JPRB, 0.11585E+01_JPRB, 0.11931E+01_JPRB, 0.12395E+01_JPRB, 0.12714E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.11569E+01_JPRB, 0.13012E+01_JPRB, 0.21346E+01_JPRB, 0.30504E+01_JPRB, 0.39340E+01_JPRB, 0.44886E+01_JPRB ,&
&  0.40830E+01_JPRB, 0.30364E+01_JPRB, 0.27186E+01_JPRB, 0.26775E+01_JPRB, 0.25298E+01_JPRB, 0.24450E+01_JPRB, 0.19069E+01_JPRB ,&
&  0.16407E+01_JPRB, 0.16085E+01_JPRB, 0.15153E+01_JPRB, 0.14906E+01_JPRB, 0.14608E+01_JPRB, 0.14475E+01_JPRB, 0.15011E+01_JPRB ,&
&  0.15615E+01_JPRB, 0.15354E+01_JPRB, 0.15534E+01_JPRB, 0.16036E+01_JPRB, 0.16846E+01_JPRB, 0.17495E+01_JPRB, 0.18771E+01_JPRB ,&
&  0.20852E+01_JPRB, 0.23750E+01_JPRB, 0.29300E+01_JPRB, 0.42256E+01_JPRB, 0.42407E+01_JPRB, 0.54078E+01_JPRB, 0.56283E+01_JPRB ,&
&  0.56654E+01_JPRB, 0.56358E+01_JPRB, 0.51664E+01_JPRB, 0.54475E+01_JPRB, 0.51787E+01_JPRB, 0.50917E+01_JPRB, 0.48522E+01_JPRB ,&
&  0.48563E+01_JPRB, 0.44945E+01_JPRB, 0.49751E+01_JPRB, 0.48562E+01_JPRB, 0.52693E+01_JPRB, 0.75229E+01_JPRB, 0.58783E+01_JPRB ,&
&  0.50892E+01_JPRB, 0.54003E+01_JPRB, 0.47841E+01_JPRB, 0.48208E+01_JPRB, 0.61468E+01_JPRB, 0.78835E+01_JPRB, 0.68363E+01_JPRB ,&
&  0.26282E+01_JPRB, 0.18086E+01_JPRB, 0.16659E+01_JPRB, 0.14898E+01_JPRB, 0.12999E+01_JPRB, 0.19484E+01_JPRB, 0.83383E+01_JPRB ,&
&  0.59094E+01_JPRB, 0.82270E+01_JPRB, 0.83574E+01_JPRB, 0.81129E+01_JPRB, 0.79284E+01_JPRB, 0.68001E+01_JPRB, 0.58239E+01_JPRB ,&
&  0.47337E+01_JPRB, 0.38548E+01_JPRB, 0.32076E+01_JPRB, 0.27233E+01_JPRB, 0.24108E+01_JPRB, 0.22469E+01_JPRB, 0.21934E+01_JPRB ,&
&  0.20984E+01_JPRB, 0.18752E+01_JPRB, 0.16643E+01_JPRB, 0.14021E+01_JPRB, 0.11869E+01_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.11652E+01_JPRB, 0.11906E+01_JPRB, 0.12330E+01_JPRB, 0.12932E+01_JPRB, 0.13324E+01_JPRB, 0.13171E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.12999E+01_JPRB, 0.13774E+01_JPRB, 0.14263E+01_JPRB, 0.14819E+01_JPRB, 0.14651E+01_JPRB, 0.13962E+01_JPRB ,&
&  0.13168E+01_JPRB, 0.13550E+01_JPRB, 0.17976E+01_JPRB, 0.28066E+01_JPRB, 0.34984E+01_JPRB, 0.41297E+01_JPRB, 0.49404E+01_JPRB ,&
&  0.47975E+01_JPRB, 0.31049E+01_JPRB, 0.23626E+01_JPRB, 0.24378E+01_JPRB, 0.24790E+01_JPRB, 0.26020E+01_JPRB, 0.19058E+01_JPRB ,&
&  0.16282E+01_JPRB, 0.15394E+01_JPRB, 0.14582E+01_JPRB, 0.14632E+01_JPRB, 0.14826E+01_JPRB, 0.15063E+01_JPRB, 0.15574E+01_JPRB ,&
&  0.15938E+01_JPRB, 0.16686E+01_JPRB, 0.17593E+01_JPRB, 0.18384E+01_JPRB, 0.18977E+01_JPRB, 0.19443E+01_JPRB, 0.20665E+01_JPRB ,&
&  0.23071E+01_JPRB, 0.26714E+01_JPRB, 0.32604E+01_JPRB, 0.40598E+01_JPRB, 0.47270E+01_JPRB, 0.50163E+01_JPRB, 0.53909E+01_JPRB ,&
&  0.54368E+01_JPRB, 0.54626E+01_JPRB, 0.52702E+01_JPRB, 0.50047E+01_JPRB, 0.47763E+01_JPRB, 0.49430E+01_JPRB, 0.47040E+01_JPRB ,&
&  0.48370E+01_JPRB, 0.49943E+01_JPRB, 0.53777E+01_JPRB, 0.52604E+01_JPRB, 0.54959E+01_JPRB, 0.72057E+01_JPRB, 0.74258E+01_JPRB ,&
&  0.62305E+01_JPRB, 0.49869E+01_JPRB, 0.47938E+01_JPRB, 0.54785E+01_JPRB, 0.62822E+01_JPRB, 0.72583E+01_JPRB, 0.10818E+02_JPRB ,&
&  0.89294E+01_JPRB, 0.39245E+01_JPRB, 0.22422E+01_JPRB, 0.23225E+01_JPRB, 0.13713E+01_JPRB, 0.14003E+01_JPRB, 0.33370E+01_JPRB ,&
&  0.46627E+01_JPRB, 0.58430E+01_JPRB, 0.60663E+01_JPRB, 0.59976E+01_JPRB, 0.58542E+01_JPRB, 0.47166E+01_JPRB, 0.35097E+01_JPRB ,&
&  0.25733E+01_JPRB, 0.20555E+01_JPRB, 0.17833E+01_JPRB, 0.16669E+01_JPRB, 0.17184E+01_JPRB, 0.18189E+01_JPRB, 0.18912E+01_JPRB ,&
&  0.18516E+01_JPRB, 0.16533E+01_JPRB, 0.13691E+01_JPRB, 0.11584E+01_JPRB, 0.11177E+01_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.13548E+01_JPRB, 0.13497E+01_JPRB, 0.13758E+01_JPRB, 0.14382E+01_JPRB, 0.14679E+01_JPRB, 0.14892E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.17126E+01_JPRB, 0.17788E+01_JPRB, 0.18258E+01_JPRB, 0.18589E+01_JPRB, 0.18369E+01_JPRB, 0.17261E+01_JPRB, 0.16382E+01_JPRB ,&
&  0.16603E+01_JPRB, 0.20163E+01_JPRB, 0.27855E+01_JPRB, 0.38518E+01_JPRB, 0.43974E+01_JPRB, 0.57644E+01_JPRB, 0.47142E+01_JPRB ,&
&  0.45999E+01_JPRB, 0.38193E+01_JPRB, 0.25444E+01_JPRB, 0.26507E+01_JPRB, 0.26665E+01_JPRB, 0.27802E+01_JPRB, 0.20019E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.16141E+01_JPRB, 0.15289E+01_JPRB, 0.15338E+01_JPRB, 0.15650E+01_JPRB, 0.16229E+01_JPRB, 0.16804E+01_JPRB ,&
&  0.17853E+01_JPRB, 0.19270E+01_JPRB, 0.20444E+01_JPRB, 0.21000E+01_JPRB, 0.21310E+01_JPRB, 0.21725E+01_JPRB, 0.23456E+01_JPRB ,&
&  0.26048E+01_JPRB, 0.29841E+01_JPRB, 0.35776E+01_JPRB, 0.41939E+01_JPRB, 0.46130E+01_JPRB, 0.46575E+01_JPRB, 0.49609E+01_JPRB ,&
&  0.48273E+01_JPRB, 0.45852E+01_JPRB, 0.47334E+01_JPRB, 0.47295E+01_JPRB, 0.46046E+01_JPRB, 0.45811E+01_JPRB, 0.43541E+01_JPRB ,&
&  0.49386E+01_JPRB, 0.58937E+01_JPRB, 0.70291E+01_JPRB, 0.62447E+01_JPRB, 0.61864E+01_JPRB, 0.72693E+01_JPRB, 0.76210E+01_JPRB ,&
&  0.69701E+01_JPRB, 0.58134E+01_JPRB, 0.56245E+01_JPRB, 0.53602E+01_JPRB, 0.56009E+01_JPRB, 0.62496E+01_JPRB, 0.79821E+01_JPRB ,&
&  0.87227E+01_JPRB, 0.89176E+01_JPRB, 0.46201E+01_JPRB, 0.19863E+01_JPRB, 0.14525E+01_JPRB, 0.17665E+01_JPRB, 0.24053E+01_JPRB ,&
&  0.32152E+01_JPRB, 0.35945E+01_JPRB, 0.37682E+01_JPRB, 0.46020E+01_JPRB, 0.40207E+01_JPRB, 0.30935E+01_JPRB, 0.23582E+01_JPRB ,&
&  0.18757E+01_JPRB, 0.15265E+01_JPRB, 0.13568E+01_JPRB, 0.13608E+01_JPRB, 0.15090E+01_JPRB, 0.16495E+01_JPRB, 0.17064E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.13414E+01_JPRB, 0.12398E+01_JPRB, 0.12367E+01_JPRB, 0.12954E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.15875E+01_JPRB, 0.16350E+01_JPRB, 0.17170E+01_JPRB, 0.18324E+01_JPRB, 0.19432E+01_JPRB, 0.20814E+01_JPRB, 0.22426E+01_JPRB ,&
&  0.23444E+01_JPRB, 0.23565E+01_JPRB, 0.23397E+01_JPRB, 0.22756E+01_JPRB, 0.22406E+01_JPRB, 0.22340E+01_JPRB, 0.23902E+01_JPRB ,&
&  0.28559E+01_JPRB, 0.35904E+01_JPRB, 0.43538E+01_JPRB, 0.48996E+01_JPRB, 0.51591E+01_JPRB, 0.59013E+01_JPRB, 0.59545E+01_JPRB ,&
&  0.43032E+01_JPRB, 0.50058E+01_JPRB, 0.28542E+01_JPRB, 0.30200E+01_JPRB, 0.31955E+01_JPRB, 0.35628E+01_JPRB, 0.25548E+01_JPRB ,&
&  0.19223E+01_JPRB, 0.18106E+01_JPRB, 0.17589E+01_JPRB, 0.17625E+01_JPRB, 0.17577E+01_JPRB, 0.18435E+01_JPRB, 0.19756E+01_JPRB ,&
&  0.21120E+01_JPRB, 0.22102E+01_JPRB, 0.22810E+01_JPRB, 0.23167E+01_JPRB, 0.23685E+01_JPRB, 0.24499E+01_JPRB, 0.26937E+01_JPRB ,&
&  0.29862E+01_JPRB, 0.34070E+01_JPRB, 0.38884E+01_JPRB, 0.43898E+01_JPRB, 0.43575E+01_JPRB, 0.43505E+01_JPRB, 0.45370E+01_JPRB ,&
&  0.44731E+01_JPRB, 0.42712E+01_JPRB, 0.43940E+01_JPRB, 0.42705E+01_JPRB, 0.41222E+01_JPRB, 0.43677E+01_JPRB, 0.42717E+01_JPRB ,&
&  0.54081E+01_JPRB, 0.60657E+01_JPRB, 0.77959E+01_JPRB, 0.71363E+01_JPRB, 0.64490E+01_JPRB, 0.68098E+01_JPRB, 0.62913E+01_JPRB ,&
&  0.54260E+01_JPRB, 0.52792E+01_JPRB, 0.51137E+01_JPRB, 0.48369E+01_JPRB, 0.49389E+01_JPRB, 0.54074E+01_JPRB, 0.62763E+01_JPRB ,&
&  0.67041E+01_JPRB, 0.63845E+01_JPRB, 0.43803E+01_JPRB, 0.20789E+01_JPRB, 0.16009E+01_JPRB, 0.16766E+01_JPRB, 0.20860E+01_JPRB ,&
&  0.27629E+01_JPRB, 0.29670E+01_JPRB, 0.29605E+01_JPRB, 0.27885E+01_JPRB, 0.22314E+01_JPRB, 0.17758E+01_JPRB, 0.14702E+01_JPRB ,&
&  0.13709E+01_JPRB, 0.13189E+01_JPRB, 0.13053E+01_JPRB, 0.14176E+01_JPRB, 0.15278E+01_JPRB, 0.15533E+01_JPRB, 0.13971E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.12827E+01_JPRB, 0.13714E+01_JPRB, 0.14605E+01_JPRB, 0.15360E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.18357E+01_JPRB, 0.19489E+01_JPRB, 0.21004E+01_JPRB, 0.23157E+01_JPRB, 0.26020E+01_JPRB, 0.29306E+01_JPRB, 0.29852E+01_JPRB ,&
&  0.29720E+01_JPRB, 0.29128E+01_JPRB, 0.30325E+01_JPRB, 0.31609E+01_JPRB, 0.33162E+01_JPRB, 0.35561E+01_JPRB, 0.39161E+01_JPRB ,&
&  0.43697E+01_JPRB, 0.48222E+01_JPRB, 0.52636E+01_JPRB, 0.57111E+01_JPRB, 0.61741E+01_JPRB, 0.70608E+01_JPRB, 0.74121E+01_JPRB ,&
&  0.68031E+01_JPRB, 0.40285E+01_JPRB, 0.34493E+01_JPRB, 0.30586E+01_JPRB, 0.31463E+01_JPRB, 0.35286E+01_JPRB, 0.32691E+01_JPRB ,&
&  0.25576E+01_JPRB, 0.22646E+01_JPRB, 0.21604E+01_JPRB, 0.21061E+01_JPRB, 0.20122E+01_JPRB, 0.21306E+01_JPRB, 0.22478E+01_JPRB ,&
&  0.23291E+01_JPRB, 0.23795E+01_JPRB, 0.24396E+01_JPRB, 0.24863E+01_JPRB, 0.25613E+01_JPRB, 0.27162E+01_JPRB, 0.30147E+01_JPRB ,&
&  0.33014E+01_JPRB, 0.36451E+01_JPRB, 0.39894E+01_JPRB, 0.41465E+01_JPRB, 0.40783E+01_JPRB, 0.40473E+01_JPRB, 0.41078E+01_JPRB ,&
&  0.41484E+01_JPRB, 0.42361E+01_JPRB, 0.42219E+01_JPRB, 0.41195E+01_JPRB, 0.39256E+01_JPRB, 0.44215E+01_JPRB, 0.47752E+01_JPRB ,&
&  0.57569E+01_JPRB, 0.63350E+01_JPRB, 0.78218E+01_JPRB, 0.68292E+01_JPRB, 0.61328E+01_JPRB, 0.57676E+01_JPRB, 0.52767E+01_JPRB ,&
&  0.45821E+01_JPRB, 0.45130E+01_JPRB, 0.42249E+01_JPRB, 0.36654E+01_JPRB, 0.35050E+01_JPRB, 0.36008E+01_JPRB, 0.46828E+01_JPRB ,&
&  0.51930E+01_JPRB, 0.51214E+01_JPRB, 0.30746E+01_JPRB, 0.19961E+01_JPRB, 0.16705E+01_JPRB, 0.16512E+01_JPRB, 0.20349E+01_JPRB ,&
&  0.26203E+01_JPRB, 0.25246E+01_JPRB, 0.21849E+01_JPRB, 0.20655E+01_JPRB, 0.17188E+01_JPRB, 0.14010E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.12003E+01_JPRB, 0.12310E+01_JPRB, 0.13181E+01_JPRB, 0.13481E+01_JPRB, 0.13989E+01_JPRB, 0.13783E+01_JPRB ,&
&  0.14422E+01_JPRB, 0.15210E+01_JPRB, 0.15980E+01_JPRB, 0.16741E+01_JPRB, 0.17411E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.19176E+01_JPRB, 0.20541E+01_JPRB, 0.22187E+01_JPRB, 0.24124E+01_JPRB, 0.26254E+01_JPRB, 0.28113E+01_JPRB, 0.29499E+01_JPRB ,&
&  0.31496E+01_JPRB, 0.33718E+01_JPRB, 0.36039E+01_JPRB, 0.37986E+01_JPRB, 0.39498E+01_JPRB, 0.41403E+01_JPRB, 0.43564E+01_JPRB ,&
&  0.45606E+01_JPRB, 0.47753E+01_JPRB, 0.50434E+01_JPRB, 0.53728E+01_JPRB, 0.58096E+01_JPRB, 0.63886E+01_JPRB, 0.67063E+01_JPRB ,&
&  0.54543E+01_JPRB, 0.32618E+01_JPRB, 0.35486E+01_JPRB, 0.32156E+01_JPRB, 0.31559E+01_JPRB, 0.32219E+01_JPRB, 0.29910E+01_JPRB ,&
&  0.26671E+01_JPRB, 0.25555E+01_JPRB, 0.25575E+01_JPRB, 0.24507E+01_JPRB, 0.22657E+01_JPRB, 0.23308E+01_JPRB, 0.23934E+01_JPRB ,&
&  0.24485E+01_JPRB, 0.25007E+01_JPRB, 0.25481E+01_JPRB, 0.25802E+01_JPRB, 0.26754E+01_JPRB, 0.28503E+01_JPRB, 0.31228E+01_JPRB ,&
&  0.34144E+01_JPRB, 0.36422E+01_JPRB, 0.38409E+01_JPRB, 0.39628E+01_JPRB, 0.38977E+01_JPRB, 0.38455E+01_JPRB, 0.38907E+01_JPRB ,&
&  0.39919E+01_JPRB, 0.40387E+01_JPRB, 0.42037E+01_JPRB, 0.44490E+01_JPRB, 0.47512E+01_JPRB, 0.47153E+01_JPRB, 0.54502E+01_JPRB ,&
&  0.61143E+01_JPRB, 0.63411E+01_JPRB, 0.67717E+01_JPRB, 0.78422E+01_JPRB, 0.52524E+01_JPRB, 0.48464E+01_JPRB, 0.42273E+01_JPRB ,&
&  0.43393E+01_JPRB, 0.40751E+01_JPRB, 0.35004E+01_JPRB, 0.27065E+01_JPRB, 0.24155E+01_JPRB, 0.23330E+01_JPRB, 0.30447E+01_JPRB ,&
&  0.39419E+01_JPRB, 0.38501E+01_JPRB, 0.26573E+01_JPRB, 0.20970E+01_JPRB, 0.19016E+01_JPRB, 0.21593E+01_JPRB, 0.25136E+01_JPRB ,&
&  0.26131E+01_JPRB, 0.21651E+01_JPRB, 0.19691E+01_JPRB, 0.21685E+01_JPRB, 0.16505E+01_JPRB, 0.11459E+01_JPRB, 0.93725E+00_JPRB ,&
&  0.93564E+00_JPRB, 0.98414E+00_JPRB, 0.10687E+01_JPRB, 0.11846E+01_JPRB, 0.12393E+01_JPRB, 0.13063E+01_JPRB, 0.13486E+01_JPRB ,&
&  0.14841E+01_JPRB, 0.15821E+01_JPRB, 0.16658E+01_JPRB, 0.17218E+01_JPRB, 0.18142E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.17722E+01_JPRB, 0.19011E+01_JPRB, 0.20140E+01_JPRB, 0.21351E+01_JPRB, 0.22453E+01_JPRB, 0.23502E+01_JPRB, 0.24991E+01_JPRB ,&
&  0.26750E+01_JPRB, 0.28787E+01_JPRB, 0.30729E+01_JPRB, 0.32256E+01_JPRB, 0.33898E+01_JPRB, 0.35015E+01_JPRB, 0.36002E+01_JPRB ,&
&  0.36423E+01_JPRB, 0.37051E+01_JPRB, 0.37530E+01_JPRB, 0.38438E+01_JPRB, 0.39899E+01_JPRB, 0.41585E+01_JPRB, 0.43218E+01_JPRB ,&
&  0.43915E+01_JPRB, 0.41291E+01_JPRB, 0.34165E+01_JPRB, 0.24374E+01_JPRB, 0.21965E+01_JPRB, 0.25091E+01_JPRB, 0.25095E+01_JPRB ,&
&  0.24781E+01_JPRB, 0.24050E+01_JPRB, 0.23306E+01_JPRB, 0.23558E+01_JPRB, 0.23602E+01_JPRB, 0.23834E+01_JPRB, 0.23991E+01_JPRB ,&
&  0.24294E+01_JPRB, 0.24811E+01_JPRB, 0.25038E+01_JPRB, 0.25337E+01_JPRB, 0.26145E+01_JPRB, 0.27746E+01_JPRB, 0.30070E+01_JPRB ,&
&  0.32883E+01_JPRB, 0.35024E+01_JPRB, 0.36367E+01_JPRB, 0.37433E+01_JPRB, 0.36901E+01_JPRB, 0.37097E+01_JPRB, 0.38421E+01_JPRB ,&
&  0.40581E+01_JPRB, 0.42171E+01_JPRB, 0.45097E+01_JPRB, 0.49355E+01_JPRB, 0.52079E+01_JPRB, 0.50375E+01_JPRB, 0.54447E+01_JPRB ,&
&  0.59212E+01_JPRB, 0.60512E+01_JPRB, 0.60043E+01_JPRB, 0.75513E+01_JPRB, 0.49213E+01_JPRB, 0.48898E+01_JPRB, 0.46239E+01_JPRB ,&
&  0.42953E+01_JPRB, 0.41537E+01_JPRB, 0.30488E+01_JPRB, 0.21589E+01_JPRB, 0.18637E+01_JPRB, 0.18105E+01_JPRB, 0.24169E+01_JPRB ,&
&  0.31459E+01_JPRB, 0.27351E+01_JPRB, 0.21195E+01_JPRB, 0.18934E+01_JPRB, 0.19278E+01_JPRB, 0.25483E+01_JPRB, 0.28501E+01_JPRB ,&
&  0.22813E+01_JPRB, 0.19400E+01_JPRB, 0.18788E+01_JPRB, 0.24002E+01_JPRB, 0.19389E+01_JPRB, 0.10247E+01_JPRB, 0.81426E+00_JPRB ,&
&  0.82012E+00_JPRB, 0.87984E+00_JPRB, 0.94970E+00_JPRB, 0.10143E+01_JPRB, 0.10683E+01_JPRB, 0.11342E+01_JPRB, 0.12080E+01_JPRB ,&
&  0.12976E+01_JPRB, 0.14161E+01_JPRB, 0.15083E+01_JPRB, 0.15969E+01_JPRB, 0.16761E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.13199E+01_JPRB, 0.14320E+01_JPRB, 0.15381E+01_JPRB, 0.16116E+01_JPRB, 0.16493E+01_JPRB, 0.16789E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.17071E+01_JPRB, 0.17078E+01_JPRB, 0.17677E+01_JPRB, 0.19073E+01_JPRB, 0.21476E+01_JPRB, 0.22936E+01_JPRB, 0.24509E+01_JPRB ,&
&  0.24486E+01_JPRB, 0.24505E+01_JPRB, 0.24473E+01_JPRB, 0.24847E+01_JPRB, 0.25642E+01_JPRB, 0.26814E+01_JPRB, 0.29701E+01_JPRB ,&
&  0.35518E+01_JPRB, 0.39158E+01_JPRB, 0.37970E+01_JPRB, 0.33539E+01_JPRB, 0.26965E+01_JPRB, 0.25881E+01_JPRB, 0.29743E+01_JPRB ,&
&  0.29636E+01_JPRB, 0.25838E+01_JPRB, 0.23325E+01_JPRB, 0.22857E+01_JPRB, 0.22753E+01_JPRB, 0.22782E+01_JPRB, 0.22855E+01_JPRB ,&
&  0.23124E+01_JPRB, 0.23467E+01_JPRB, 0.23630E+01_JPRB, 0.23939E+01_JPRB, 0.24679E+01_JPRB, 0.25795E+01_JPRB, 0.27199E+01_JPRB ,&
&  0.29174E+01_JPRB, 0.30710E+01_JPRB, 0.31551E+01_JPRB, 0.31955E+01_JPRB, 0.32798E+01_JPRB, 0.34951E+01_JPRB, 0.38282E+01_JPRB ,&
&  0.41444E+01_JPRB, 0.43044E+01_JPRB, 0.47109E+01_JPRB, 0.50957E+01_JPRB, 0.51080E+01_JPRB, 0.50468E+01_JPRB, 0.52247E+01_JPRB ,&
&  0.53429E+01_JPRB, 0.53551E+01_JPRB, 0.47169E+01_JPRB, 0.65007E+01_JPRB, 0.66811E+01_JPRB, 0.57111E+01_JPRB, 0.47108E+01_JPRB ,&
&  0.42554E+01_JPRB, 0.38936E+01_JPRB, 0.31514E+01_JPRB, 0.23480E+01_JPRB, 0.19933E+01_JPRB, 0.22216E+01_JPRB, 0.26373E+01_JPRB ,&
&  0.27774E+01_JPRB, 0.21725E+01_JPRB, 0.19088E+01_JPRB, 0.18738E+01_JPRB, 0.18344E+01_JPRB, 0.25234E+01_JPRB, 0.23777E+01_JPRB ,&
&  0.17726E+01_JPRB, 0.16234E+01_JPRB, 0.17123E+01_JPRB, 0.22635E+01_JPRB, 0.29192E+01_JPRB, 0.94563E+00_JPRB, 0.70955E+00_JPRB ,&
&  0.72179E+00_JPRB, 0.75698E+00_JPRB, 0.80204E+00_JPRB, 0.82320E+00_JPRB, 0.82583E+00_JPRB, 0.86790E+00_JPRB, 0.93521E+00_JPRB ,&
&  0.97223E+00_JPRB, 0.10040E+01_JPRB, 0.10446E+01_JPRB, 0.11484E+01_JPRB, 0.12254E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.82138E+00_JPRB, 0.87689E+00_JPRB, 0.91774E+00_JPRB, 0.95680E+00_JPRB, 0.97375E+00_JPRB, 0.96370E+00_JPRB, 0.91529E+00_JPRB ,&
&  0.87896E+00_JPRB, 0.84882E+00_JPRB, 0.85150E+00_JPRB, 0.84411E+00_JPRB, 0.89631E+00_JPRB, 0.99207E+00_JPRB, 0.10447E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.11092E+01_JPRB, 0.10918E+01_JPRB, 0.11133E+01_JPRB, 0.11079E+01_JPRB, 0.11536E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.16250E+01_JPRB, 0.19518E+01_JPRB, 0.24275E+01_JPRB, 0.32370E+01_JPRB, 0.35682E+01_JPRB, 0.30287E+01_JPRB, 0.34257E+01_JPRB ,&
&  0.30126E+01_JPRB, 0.23619E+01_JPRB, 0.21849E+01_JPRB, 0.20366E+01_JPRB, 0.20360E+01_JPRB, 0.20398E+01_JPRB, 0.20842E+01_JPRB ,&
&  0.21363E+01_JPRB, 0.21875E+01_JPRB, 0.22243E+01_JPRB, 0.22587E+01_JPRB, 0.23069E+01_JPRB, 0.23642E+01_JPRB, 0.24486E+01_JPRB ,&
&  0.25085E+01_JPRB, 0.25449E+01_JPRB, 0.26291E+01_JPRB, 0.26633E+01_JPRB, 0.29393E+01_JPRB, 0.33154E+01_JPRB, 0.36826E+01_JPRB ,&
&  0.39309E+01_JPRB, 0.41110E+01_JPRB, 0.44685E+01_JPRB, 0.45806E+01_JPRB, 0.46178E+01_JPRB, 0.46372E+01_JPRB, 0.45432E+01_JPRB ,&
&  0.43009E+01_JPRB, 0.35518E+01_JPRB, 0.33036E+01_JPRB, 0.51106E+01_JPRB, 0.50213E+01_JPRB, 0.40907E+01_JPRB, 0.41828E+01_JPRB ,&
&  0.38762E+01_JPRB, 0.36463E+01_JPRB, 0.31506E+01_JPRB, 0.25767E+01_JPRB, 0.21737E+01_JPRB, 0.21000E+01_JPRB, 0.24968E+01_JPRB ,&
&  0.21555E+01_JPRB, 0.17417E+01_JPRB, 0.17879E+01_JPRB, 0.19167E+01_JPRB, 0.19275E+01_JPRB, 0.21892E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.17112E+01_JPRB, 0.14602E+01_JPRB, 0.17302E+01_JPRB, 0.23243E+01_JPRB, 0.15913E+01_JPRB, 0.77783E+00_JPRB, 0.61540E+00_JPRB ,&
&  0.62230E+00_JPRB, 0.63472E+00_JPRB, 0.64591E+00_JPRB, 0.65186E+00_JPRB, 0.65708E+00_JPRB, 0.67622E+00_JPRB, 0.68851E+00_JPRB ,&
&  0.69979E+00_JPRB, 0.73024E+00_JPRB, 0.74391E+00_JPRB, 0.76475E+00_JPRB, 0.79808E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.68787E+00_JPRB, 0.69210E+00_JPRB, 0.68820E+00_JPRB, 0.69257E+00_JPRB, 0.70130E+00_JPRB, 0.72241E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.74067E+00_JPRB, 0.75200E+00_JPRB, 0.73588E+00_JPRB, 0.74454E+00_JPRB, 0.77825E+00_JPRB, 0.80812E+00_JPRB, 0.83952E+00_JPRB ,&
&  0.91292E+00_JPRB, 0.92898E+00_JPRB, 0.92723E+00_JPRB, 0.93679E+00_JPRB, 0.92636E+00_JPRB, 0.99564E+00_JPRB, 0.99732E+00_JPRB ,&
&  0.96168E+00_JPRB, 0.98484E+00_JPRB, 0.11605E+01_JPRB, 0.16321E+01_JPRB, 0.21209E+01_JPRB, 0.22496E+01_JPRB, 0.27537E+01_JPRB ,&
&  0.28230E+01_JPRB, 0.18859E+01_JPRB, 0.18131E+01_JPRB, 0.15291E+01_JPRB, 0.15182E+01_JPRB, 0.15768E+01_JPRB, 0.15943E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.16113E+01_JPRB, 0.16121E+01_JPRB, 0.16338E+01_JPRB, 0.16402E+01_JPRB, 0.16484E+01_JPRB, 0.16577E+01_JPRB ,&
&  0.15815E+01_JPRB, 0.16221E+01_JPRB, 0.17940E+01_JPRB, 0.21223E+01_JPRB, 0.25018E+01_JPRB, 0.29789E+01_JPRB, 0.31824E+01_JPRB ,&
&  0.33303E+01_JPRB, 0.35020E+01_JPRB, 0.33451E+01_JPRB, 0.34550E+01_JPRB, 0.37163E+01_JPRB, 0.34774E+01_JPRB, 0.33064E+01_JPRB ,&
&  0.31216E+01_JPRB, 0.24839E+01_JPRB, 0.24570E+01_JPRB, 0.35870E+01_JPRB, 0.38052E+01_JPRB, 0.36210E+01_JPRB, 0.36559E+01_JPRB ,&
&  0.34737E+01_JPRB, 0.31923E+01_JPRB, 0.27230E+01_JPRB, 0.23699E+01_JPRB, 0.20701E+01_JPRB, 0.18103E+01_JPRB, 0.18977E+01_JPRB ,&
&  0.15881E+01_JPRB, 0.15884E+01_JPRB, 0.18595E+01_JPRB, 0.20869E+01_JPRB, 0.23028E+01_JPRB, 0.27343E+01_JPRB, 0.20906E+01_JPRB ,&
&  0.15446E+01_JPRB, 0.15167E+01_JPRB, 0.22208E+01_JPRB, 0.29312E+01_JPRB, 0.13482E+01_JPRB, 0.78150E+00_JPRB, 0.53525E+00_JPRB ,&
&  0.55091E+00_JPRB, 0.56942E+00_JPRB, 0.57438E+00_JPRB, 0.57968E+00_JPRB, 0.59413E+00_JPRB, 0.60576E+00_JPRB, 0.62781E+00_JPRB ,&
&  0.64004E+00_JPRB, 0.64973E+00_JPRB, 0.66050E+00_JPRB, 0.67962E+00_JPRB, 0.67920E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.74928E+00_JPRB, 0.76417E+00_JPRB, 0.77611E+00_JPRB, 0.80153E+00_JPRB, 0.83801E+00_JPRB, 0.85633E+00_JPRB, 0.88450E+00_JPRB ,&
&  0.90545E+00_JPRB, 0.93594E+00_JPRB, 0.96663E+00_JPRB, 0.10030E+01_JPRB, 0.10356E+01_JPRB, 0.10567E+01_JPRB, 0.10984E+01_JPRB ,&
&  0.11401E+01_JPRB, 0.11419E+01_JPRB, 0.11656E+01_JPRB, 0.12110E+01_JPRB, 0.12202E+01_JPRB, 0.12725E+01_JPRB, 0.12583E+01_JPRB ,&
&  0.12864E+01_JPRB, 0.12627E+01_JPRB, 0.12807E+01_JPRB, 0.13883E+01_JPRB, 0.15197E+01_JPRB, 0.17626E+01_JPRB, 0.22753E+01_JPRB ,&
&  0.21746E+01_JPRB, 0.15967E+01_JPRB, 0.13552E+01_JPRB, 0.11417E+01_JPRB, 0.11177E+01_JPRB, 0.11222E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10180E+01_JPRB, 0.99485E+00_JPRB, 0.98809E+00_JPRB, 0.10339E+01_JPRB, 0.11514E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.14268E+01_JPRB, 0.16494E+01_JPRB, 0.19500E+01_JPRB, 0.21746E+01_JPRB, 0.27157E+01_JPRB, 0.27424E+01_JPRB ,&
&  0.26576E+01_JPRB, 0.25929E+01_JPRB, 0.22655E+01_JPRB, 0.23823E+01_JPRB, 0.26149E+01_JPRB, 0.25172E+01_JPRB, 0.22463E+01_JPRB ,&
&  0.22240E+01_JPRB, 0.21174E+01_JPRB, 0.22421E+01_JPRB, 0.29910E+01_JPRB, 0.32512E+01_JPRB, 0.33185E+01_JPRB, 0.31589E+01_JPRB ,&
&  0.29295E+01_JPRB, 0.26002E+01_JPRB, 0.23042E+01_JPRB, 0.21505E+01_JPRB, 0.20633E+01_JPRB, 0.18457E+01_JPRB, 0.16568E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.16698E+01_JPRB, 0.19232E+01_JPRB, 0.21476E+01_JPRB, 0.27376E+01_JPRB, 0.25262E+01_JPRB, 0.27909E+01_JPRB ,&
&  0.14870E+01_JPRB, 0.14646E+01_JPRB, 0.11335E+01_JPRB, 0.15265E+01_JPRB, 0.92160E+00_JPRB, 0.60786E+00_JPRB, 0.56669E+00_JPRB ,&
&  0.58135E+00_JPRB, 0.59089E+00_JPRB, 0.60582E+00_JPRB, 0.61689E+00_JPRB, 0.63076E+00_JPRB, 0.65589E+00_JPRB, 0.68259E+00_JPRB ,&
&  0.70989E+00_JPRB, 0.70454E+00_JPRB, 0.70914E+00_JPRB, 0.71980E+00_JPRB, 0.74004E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.81183E+00_JPRB, 0.84267E+00_JPRB, 0.87389E+00_JPRB, 0.90568E+00_JPRB, 0.93496E+00_JPRB, 0.96348E+00_JPRB, 0.99270E+00_JPRB ,&
&  0.10215E+01_JPRB, 0.10542E+01_JPRB, 0.10889E+01_JPRB, 0.11278E+01_JPRB, 0.11643E+01_JPRB, 0.11901E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12816E+01_JPRB, 0.12880E+01_JPRB, 0.13411E+01_JPRB, 0.13916E+01_JPRB, 0.14210E+01_JPRB, 0.14645E+01_JPRB ,&
&  0.15047E+01_JPRB, 0.14619E+01_JPRB, 0.14426E+01_JPRB, 0.14822E+01_JPRB, 0.15219E+01_JPRB, 0.14789E+01_JPRB, 0.17246E+01_JPRB ,&
&  0.14229E+01_JPRB, 0.14356E+01_JPRB, 0.12694E+01_JPRB, 0.10978E+01_JPRB, 0.10417E+01_JPRB, 0.91044E+00_JPRB, 0.83628E+00_JPRB ,&
&  0.87938E+00_JPRB, 0.88630E+00_JPRB, 0.89598E+00_JPRB, 0.93324E+00_JPRB, 0.10424E+01_JPRB, 0.12463E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.16005E+01_JPRB, 0.17980E+01_JPRB, 0.20282E+01_JPRB, 0.22795E+01_JPRB, 0.25531E+01_JPRB, 0.27054E+01_JPRB, 0.26477E+01_JPRB ,&
&  0.25097E+01_JPRB, 0.23582E+01_JPRB, 0.18272E+01_JPRB, 0.20524E+01_JPRB, 0.23956E+01_JPRB, 0.23365E+01_JPRB, 0.20088E+01_JPRB ,&
&  0.18295E+01_JPRB, 0.18462E+01_JPRB, 0.21077E+01_JPRB, 0.24197E+01_JPRB, 0.26109E+01_JPRB, 0.26813E+01_JPRB, 0.25510E+01_JPRB ,&
&  0.23872E+01_JPRB, 0.20912E+01_JPRB, 0.19596E+01_JPRB, 0.19967E+01_JPRB, 0.19718E+01_JPRB, 0.18529E+01_JPRB, 0.16171E+01_JPRB ,&
&  0.15623E+01_JPRB, 0.16887E+01_JPRB, 0.18716E+01_JPRB, 0.20956E+01_JPRB, 0.24430E+01_JPRB, 0.37601E+01_JPRB, 0.37034E+01_JPRB ,&
&  0.15138E+01_JPRB, 0.11899E+01_JPRB, 0.66838E+00_JPRB, 0.75910E+00_JPRB, 0.77236E+00_JPRB, 0.68284E+00_JPRB, 0.72449E+00_JPRB ,&
&  0.76104E+00_JPRB, 0.73133E+00_JPRB, 0.72332E+00_JPRB, 0.70138E+00_JPRB, 0.70501E+00_JPRB, 0.70426E+00_JPRB, 0.72192E+00_JPRB ,&
&  0.73399E+00_JPRB, 0.73992E+00_JPRB, 0.75457E+00_JPRB, 0.77319E+00_JPRB, 0.78898E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.84652E+00_JPRB, 0.88192E+00_JPRB, 0.91759E+00_JPRB, 0.95079E+00_JPRB, 0.98371E+00_JPRB, 0.10158E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.11178E+01_JPRB, 0.11532E+01_JPRB, 0.11965E+01_JPRB, 0.12382E+01_JPRB, 0.12659E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.14201E+01_JPRB, 0.14692E+01_JPRB, 0.15472E+01_JPRB, 0.16136E+01_JPRB, 0.16627E+01_JPRB, 0.17337E+01_JPRB ,&
&  0.17635E+01_JPRB, 0.17682E+01_JPRB, 0.17746E+01_JPRB, 0.18284E+01_JPRB, 0.18699E+01_JPRB, 0.20779E+01_JPRB, 0.17949E+01_JPRB ,&
&  0.17492E+01_JPRB, 0.15214E+01_JPRB, 0.13778E+01_JPRB, 0.12741E+01_JPRB, 0.11882E+01_JPRB, 0.11146E+01_JPRB, 0.10684E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.10628E+01_JPRB, 0.10804E+01_JPRB, 0.11186E+01_JPRB, 0.12178E+01_JPRB, 0.14345E+01_JPRB, 0.16128E+01_JPRB ,&
&  0.18106E+01_JPRB, 0.20404E+01_JPRB, 0.22528E+01_JPRB, 0.24706E+01_JPRB, 0.27124E+01_JPRB, 0.29552E+01_JPRB, 0.31436E+01_JPRB ,&
&  0.31538E+01_JPRB, 0.30836E+01_JPRB, 0.26367E+01_JPRB, 0.28997E+01_JPRB, 0.30911E+01_JPRB, 0.27935E+01_JPRB, 0.21662E+01_JPRB ,&
&  0.17484E+01_JPRB, 0.15260E+01_JPRB, 0.16140E+01_JPRB, 0.19044E+01_JPRB, 0.20212E+01_JPRB, 0.20396E+01_JPRB, 0.19477E+01_JPRB ,&
&  0.18260E+01_JPRB, 0.16554E+01_JPRB, 0.15967E+01_JPRB, 0.16336E+01_JPRB, 0.17207E+01_JPRB, 0.17700E+01_JPRB, 0.16680E+01_JPRB ,&
&  0.16412E+01_JPRB, 0.16299E+01_JPRB, 0.16433E+01_JPRB, 0.16878E+01_JPRB, 0.19037E+01_JPRB, 0.18789E+01_JPRB, 0.20693E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.88832E+00_JPRB, 0.86086E+00_JPRB, 0.73996E+00_JPRB, 0.81136E+00_JPRB, 0.81311E+00_JPRB, 0.97896E+00_JPRB ,&
&  0.11126E+01_JPRB, 0.11655E+01_JPRB, 0.10961E+01_JPRB, 0.91789E+00_JPRB, 0.78195E+00_JPRB, 0.73041E+00_JPRB, 0.73560E+00_JPRB ,&
&  0.74330E+00_JPRB, 0.74842E+00_JPRB, 0.76603E+00_JPRB, 0.79198E+00_JPRB, 0.81633E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.81245E+00_JPRB, 0.84662E+00_JPRB, 0.88401E+00_JPRB, 0.91618E+00_JPRB, 0.95517E+00_JPRB, 0.10089E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.11490E+01_JPRB, 0.11823E+01_JPRB, 0.12261E+01_JPRB, 0.12817E+01_JPRB, 0.13597E+01_JPRB, 0.14579E+01_JPRB ,&
&  0.15238E+01_JPRB, 0.16154E+01_JPRB, 0.16905E+01_JPRB, 0.18071E+01_JPRB, 0.18885E+01_JPRB, 0.20174E+01_JPRB, 0.21228E+01_JPRB ,&
&  0.21786E+01_JPRB, 0.22071E+01_JPRB, 0.22439E+01_JPRB, 0.23137E+01_JPRB, 0.23170E+01_JPRB, 0.24480E+01_JPRB, 0.19808E+01_JPRB ,&
&  0.19652E+01_JPRB, 0.18465E+01_JPRB, 0.16812E+01_JPRB, 0.16006E+01_JPRB, 0.15294E+01_JPRB, 0.14834E+01_JPRB, 0.14667E+01_JPRB ,&
&  0.14626E+01_JPRB, 0.13926E+01_JPRB, 0.12706E+01_JPRB, 0.12490E+01_JPRB, 0.13141E+01_JPRB, 0.15039E+01_JPRB, 0.16956E+01_JPRB ,&
&  0.19030E+01_JPRB, 0.21124E+01_JPRB, 0.23224E+01_JPRB, 0.25927E+01_JPRB, 0.29473E+01_JPRB, 0.33221E+01_JPRB, 0.36898E+01_JPRB ,&
&  0.38227E+01_JPRB, 0.38823E+01_JPRB, 0.35618E+01_JPRB, 0.35898E+01_JPRB, 0.36717E+01_JPRB, 0.30452E+01_JPRB, 0.21932E+01_JPRB ,&
&  0.14489E+01_JPRB, 0.13655E+01_JPRB, 0.14906E+01_JPRB, 0.16163E+01_JPRB, 0.15311E+01_JPRB, 0.14516E+01_JPRB, 0.13403E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.11446E+01_JPRB, 0.11592E+01_JPRB, 0.12500E+01_JPRB, 0.13688E+01_JPRB, 0.15153E+01_JPRB, 0.15594E+01_JPRB ,&
&  0.15407E+01_JPRB, 0.14602E+01_JPRB, 0.14059E+01_JPRB, 0.14157E+01_JPRB, 0.14260E+01_JPRB, 0.13215E+01_JPRB, 0.14611E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.99318E+00_JPRB, 0.97114E+00_JPRB, 0.64909E+00_JPRB, 0.81269E+00_JPRB, 0.10086E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.13931E+01_JPRB, 0.18404E+01_JPRB, 0.22253E+01_JPRB, 0.15953E+01_JPRB, 0.12006E+01_JPRB, 0.78646E+00_JPRB, 0.72117E+00_JPRB ,&
&  0.72985E+00_JPRB, 0.72737E+00_JPRB, 0.73835E+00_JPRB, 0.76021E+00_JPRB, 0.78320E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.74154E+00_JPRB, 0.76743E+00_JPRB, 0.80049E+00_JPRB, 0.81403E+00_JPRB, 0.83045E+00_JPRB, 0.85610E+00_JPRB, 0.88372E+00_JPRB ,&
&  0.90943E+00_JPRB, 0.93649E+00_JPRB, 0.95907E+00_JPRB, 0.99145E+00_JPRB, 0.10338E+01_JPRB, 0.11311E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.14035E+01_JPRB, 0.15381E+01_JPRB, 0.17023E+01_JPRB, 0.18085E+01_JPRB, 0.19573E+01_JPRB, 0.21209E+01_JPRB, 0.22456E+01_JPRB ,&
&  0.23803E+01_JPRB, 0.24507E+01_JPRB, 0.25164E+01_JPRB, 0.26720E+01_JPRB, 0.26267E+01_JPRB, 0.26962E+01_JPRB, 0.20511E+01_JPRB ,&
&  0.20949E+01_JPRB, 0.20635E+01_JPRB, 0.19590E+01_JPRB, 0.19042E+01_JPRB, 0.18237E+01_JPRB, 0.17092E+01_JPRB, 0.16273E+01_JPRB ,&
&  0.15683E+01_JPRB, 0.14225E+01_JPRB, 0.12584E+01_JPRB, 0.12560E+01_JPRB, 0.12888E+01_JPRB, 0.13891E+01_JPRB, 0.16212E+01_JPRB ,&
&  0.18446E+01_JPRB, 0.20533E+01_JPRB, 0.22620E+01_JPRB, 0.25640E+01_JPRB, 0.29667E+01_JPRB, 0.33955E+01_JPRB, 0.38462E+01_JPRB ,&
&  0.41511E+01_JPRB, 0.43799E+01_JPRB, 0.44373E+01_JPRB, 0.37890E+01_JPRB, 0.36019E+01_JPRB, 0.28084E+01_JPRB, 0.19310E+01_JPRB ,&
&  0.14768E+01_JPRB, 0.13455E+01_JPRB, 0.13665E+01_JPRB, 0.13788E+01_JPRB, 0.12232E+01_JPRB, 0.11178E+01_JPRB, 0.99941E+00_JPRB ,&
&  0.91003E+00_JPRB, 0.88868E+00_JPRB, 0.92293E+00_JPRB, 0.10280E+01_JPRB, 0.11386E+01_JPRB, 0.12777E+01_JPRB, 0.13247E+01_JPRB ,&
&  0.12918E+01_JPRB, 0.12112E+01_JPRB, 0.11868E+01_JPRB, 0.12066E+01_JPRB, 0.12653E+01_JPRB, 0.13468E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.10289E+01_JPRB, 0.82435E+00_JPRB, 0.78319E+00_JPRB, 0.83428E+00_JPRB, 0.85864E+00_JPRB, 0.11080E+01_JPRB, 0.15402E+01_JPRB ,&
&  0.21690E+01_JPRB, 0.22665E+01_JPRB, 0.27013E+01_JPRB, 0.35559E+01_JPRB, 0.26124E+01_JPRB, 0.18743E+01_JPRB, 0.64450E+00_JPRB ,&
&  0.66615E+00_JPRB, 0.66964E+00_JPRB, 0.67808E+00_JPRB, 0.68753E+00_JPRB, 0.70826E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.61081E+00_JPRB, 0.64432E+00_JPRB, 0.63956E+00_JPRB, 0.64701E+00_JPRB, 0.65735E+00_JPRB, 0.67435E+00_JPRB, 0.70799E+00_JPRB ,&
&  0.72319E+00_JPRB, 0.77874E+00_JPRB, 0.84235E+00_JPRB, 0.90011E+00_JPRB, 0.97137E+00_JPRB, 0.10702E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.13523E+01_JPRB, 0.14505E+01_JPRB, 0.15549E+01_JPRB, 0.16450E+01_JPRB, 0.17820E+01_JPRB, 0.19512E+01_JPRB, 0.20789E+01_JPRB ,&
&  0.22609E+01_JPRB, 0.23989E+01_JPRB, 0.25498E+01_JPRB, 0.27577E+01_JPRB, 0.26484E+01_JPRB, 0.26024E+01_JPRB, 0.18167E+01_JPRB ,&
&  0.20859E+01_JPRB, 0.21333E+01_JPRB, 0.20988E+01_JPRB, 0.20491E+01_JPRB, 0.19403E+01_JPRB, 0.17663E+01_JPRB, 0.16509E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.14167E+01_JPRB, 0.12689E+01_JPRB, 0.11818E+01_JPRB, 0.12635E+01_JPRB, 0.12606E+01_JPRB, 0.14539E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.19365E+01_JPRB, 0.21795E+01_JPRB, 0.24715E+01_JPRB, 0.28557E+01_JPRB, 0.32969E+01_JPRB, 0.37471E+01_JPRB ,&
&  0.40847E+01_JPRB, 0.43661E+01_JPRB, 0.43235E+01_JPRB, 0.36147E+01_JPRB, 0.32254E+01_JPRB, 0.24893E+01_JPRB, 0.17240E+01_JPRB ,&
&  0.14217E+01_JPRB, 0.13296E+01_JPRB, 0.13422E+01_JPRB, 0.12822E+01_JPRB, 0.11045E+01_JPRB, 0.96909E+00_JPRB, 0.85069E+00_JPRB ,&
&  0.75218E+00_JPRB, 0.74465E+00_JPRB, 0.81764E+00_JPRB, 0.92176E+00_JPRB, 0.10095E+01_JPRB, 0.10993E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.98811E+00_JPRB, 0.97947E+00_JPRB, 0.10693E+01_JPRB, 0.12710E+01_JPRB, 0.14756E+01_JPRB, 0.15894E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.82578E+00_JPRB, 0.78535E+00_JPRB, 0.78929E+00_JPRB, 0.82130E+00_JPRB, 0.89538E+00_JPRB, 0.10185E+01_JPRB ,&
&  0.12037E+01_JPRB, 0.16018E+01_JPRB, 0.15252E+01_JPRB, 0.12339E+01_JPRB, 0.17136E+01_JPRB, 0.14430E+01_JPRB, 0.58519E+00_JPRB ,&
&  0.56815E+00_JPRB, 0.55718E+00_JPRB, 0.57017E+00_JPRB, 0.57991E+00_JPRB, 0.60077E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.56730E+00_JPRB, 0.55716E+00_JPRB, 0.53984E+00_JPRB, 0.55321E+00_JPRB, 0.55838E+00_JPRB, 0.58383E+00_JPRB, 0.61455E+00_JPRB ,&
&  0.65797E+00_JPRB, 0.73799E+00_JPRB, 0.82555E+00_JPRB, 0.90471E+00_JPRB, 0.96706E+00_JPRB, 0.10382E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.12203E+01_JPRB, 0.12982E+01_JPRB, 0.13979E+01_JPRB, 0.15270E+01_JPRB, 0.16787E+01_JPRB, 0.18619E+01_JPRB ,&
&  0.20669E+01_JPRB, 0.22818E+01_JPRB, 0.25473E+01_JPRB, 0.26860E+01_JPRB, 0.27782E+01_JPRB, 0.28624E+01_JPRB, 0.28657E+01_JPRB ,&
&  0.18605E+01_JPRB, 0.22983E+01_JPRB, 0.22058E+01_JPRB, 0.20901E+01_JPRB, 0.19907E+01_JPRB, 0.18298E+01_JPRB, 0.16876E+01_JPRB ,&
&  0.15829E+01_JPRB, 0.14198E+01_JPRB, 0.13380E+01_JPRB, 0.11644E+01_JPRB, 0.11644E+01_JPRB, 0.11932E+01_JPRB, 0.12893E+01_JPRB ,&
&  0.14606E+01_JPRB, 0.17243E+01_JPRB, 0.20485E+01_JPRB, 0.23574E+01_JPRB, 0.26620E+01_JPRB, 0.30002E+01_JPRB, 0.34726E+01_JPRB ,&
&  0.39877E+01_JPRB, 0.43388E+01_JPRB, 0.45509E+01_JPRB, 0.39503E+01_JPRB, 0.31799E+01_JPRB, 0.27741E+01_JPRB, 0.21082E+01_JPRB ,&
&  0.14670E+01_JPRB, 0.12435E+01_JPRB, 0.12731E+01_JPRB, 0.12581E+01_JPRB, 0.10991E+01_JPRB, 0.97321E+00_JPRB, 0.85136E+00_JPRB ,&
&  0.73673E+00_JPRB, 0.67508E+00_JPRB, 0.69761E+00_JPRB, 0.81073E+00_JPRB, 0.86908E+00_JPRB, 0.90137E+00_JPRB, 0.88802E+00_JPRB ,&
&  0.86947E+00_JPRB, 0.81933E+00_JPRB, 0.77622E+00_JPRB, 0.79260E+00_JPRB, 0.85039E+00_JPRB, 0.89616E+00_JPRB, 0.88625E+00_JPRB ,&
&  0.85131E+00_JPRB, 0.83365E+00_JPRB, 0.82643E+00_JPRB, 0.83541E+00_JPRB, 0.83576E+00_JPRB, 0.84588E+00_JPRB, 0.86222E+00_JPRB ,&
&  0.88245E+00_JPRB, 0.91765E+00_JPRB, 0.88945E+00_JPRB, 0.85378E+00_JPRB, 0.90277E+00_JPRB, 0.94953E+00_JPRB, 0.63942E+00_JPRB ,&
&  0.55776E+00_JPRB, 0.52613E+00_JPRB, 0.53444E+00_JPRB, 0.53091E+00_JPRB, 0.57713E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.57890E+00_JPRB, 0.55862E+00_JPRB, 0.54812E+00_JPRB, 0.54063E+00_JPRB, 0.52634E+00_JPRB, 0.55883E+00_JPRB, 0.57322E+00_JPRB ,&
&  0.61106E+00_JPRB, 0.67025E+00_JPRB, 0.74511E+00_JPRB, 0.80156E+00_JPRB, 0.83228E+00_JPRB, 0.88393E+00_JPRB, 0.93002E+00_JPRB ,&
&  0.96620E+00_JPRB, 0.99579E+00_JPRB, 0.10360E+01_JPRB, 0.10873E+01_JPRB, 0.11720E+01_JPRB, 0.12996E+01_JPRB, 0.14893E+01_JPRB ,&
&  0.17131E+01_JPRB, 0.19562E+01_JPRB, 0.22227E+01_JPRB, 0.24781E+01_JPRB, 0.26730E+01_JPRB, 0.28252E+01_JPRB, 0.27772E+01_JPRB ,&
&  0.14942E+01_JPRB, 0.21483E+01_JPRB, 0.24810E+01_JPRB, 0.22592E+01_JPRB, 0.20704E+01_JPRB, 0.18830E+01_JPRB, 0.16974E+01_JPRB ,&
&  0.15329E+01_JPRB, 0.13757E+01_JPRB, 0.13016E+01_JPRB, 0.12696E+01_JPRB, 0.11174E+01_JPRB, 0.11281E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.14517E+01_JPRB, 0.17997E+01_JPRB, 0.21822E+01_JPRB, 0.23955E+01_JPRB, 0.26076E+01_JPRB, 0.30008E+01_JPRB ,&
&  0.35993E+01_JPRB, 0.40596E+01_JPRB, 0.43896E+01_JPRB, 0.33514E+01_JPRB, 0.28837E+01_JPRB, 0.27999E+01_JPRB, 0.25423E+01_JPRB ,&
&  0.16563E+01_JPRB, 0.12653E+01_JPRB, 0.12840E+01_JPRB, 0.12897E+01_JPRB, 0.11408E+01_JPRB, 0.10450E+01_JPRB, 0.93479E+00_JPRB ,&
&  0.80001E+00_JPRB, 0.70780E+00_JPRB, 0.64024E+00_JPRB, 0.67500E+00_JPRB, 0.71099E+00_JPRB, 0.73385E+00_JPRB, 0.70047E+00_JPRB ,&
&  0.74004E+00_JPRB, 0.72842E+00_JPRB, 0.72267E+00_JPRB, 0.73108E+00_JPRB, 0.80466E+00_JPRB, 0.83998E+00_JPRB, 0.85634E+00_JPRB ,&
&  0.87743E+00_JPRB, 0.89354E+00_JPRB, 0.89292E+00_JPRB, 0.89641E+00_JPRB, 0.89759E+00_JPRB, 0.88867E+00_JPRB, 0.87614E+00_JPRB ,&
&  0.85208E+00_JPRB, 0.83589E+00_JPRB, 0.79820E+00_JPRB, 0.75655E+00_JPRB, 0.69897E+00_JPRB, 0.69925E+00_JPRB, 0.63435E+00_JPRB ,&
&  0.59130E+00_JPRB, 0.56329E+00_JPRB, 0.54620E+00_JPRB, 0.54345E+00_JPRB, 0.55526E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.60081E+00_JPRB, 0.59645E+00_JPRB, 0.59064E+00_JPRB, 0.57875E+00_JPRB, 0.56313E+00_JPRB, 0.56390E+00_JPRB, 0.55309E+00_JPRB ,&
&  0.55702E+00_JPRB, 0.58961E+00_JPRB, 0.63846E+00_JPRB, 0.67790E+00_JPRB, 0.71553E+00_JPRB, 0.74551E+00_JPRB, 0.78270E+00_JPRB ,&
&  0.79988E+00_JPRB, 0.80505E+00_JPRB, 0.80473E+00_JPRB, 0.80872E+00_JPRB, 0.83131E+00_JPRB, 0.90085E+00_JPRB, 0.10192E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.14187E+01_JPRB, 0.17089E+01_JPRB, 0.20363E+01_JPRB, 0.23830E+01_JPRB, 0.27326E+01_JPRB, 0.29305E+01_JPRB ,&
&  0.18502E+01_JPRB, 0.87382E+00_JPRB, 0.23281E+01_JPRB, 0.25666E+01_JPRB, 0.23092E+01_JPRB, 0.21399E+01_JPRB, 0.18561E+01_JPRB ,&
&  0.15995E+01_JPRB, 0.14936E+01_JPRB, 0.13518E+01_JPRB, 0.12490E+01_JPRB, 0.11144E+01_JPRB, 0.10966E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.11283E+01_JPRB, 0.12394E+01_JPRB, 0.14811E+01_JPRB, 0.18938E+01_JPRB, 0.21722E+01_JPRB, 0.22822E+01_JPRB, 0.25376E+01_JPRB ,&
&  0.30490E+01_JPRB, 0.34853E+01_JPRB, 0.36624E+01_JPRB, 0.25386E+01_JPRB, 0.18561E+01_JPRB, 0.16901E+01_JPRB, 0.15881E+01_JPRB ,&
&  0.15281E+01_JPRB, 0.13190E+01_JPRB, 0.13029E+01_JPRB, 0.13254E+01_JPRB, 0.12219E+01_JPRB, 0.10926E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.95383E+00_JPRB, 0.85015E+00_JPRB, 0.75422E+00_JPRB, 0.70645E+00_JPRB, 0.67638E+00_JPRB, 0.68293E+00_JPRB, 0.67844E+00_JPRB ,&
&  0.68074E+00_JPRB, 0.70460E+00_JPRB, 0.71628E+00_JPRB, 0.73450E+00_JPRB, 0.78572E+00_JPRB, 0.83742E+00_JPRB, 0.87112E+00_JPRB ,&
&  0.89805E+00_JPRB, 0.92628E+00_JPRB, 0.94167E+00_JPRB, 0.94497E+00_JPRB, 0.94165E+00_JPRB, 0.91835E+00_JPRB, 0.91789E+00_JPRB ,&
&  0.91698E+00_JPRB, 0.90762E+00_JPRB, 0.86703E+00_JPRB, 0.83656E+00_JPRB, 0.76753E+00_JPRB, 0.71691E+00_JPRB, 0.64613E+00_JPRB ,&
&  0.61724E+00_JPRB, 0.57598E+00_JPRB, 0.54909E+00_JPRB, 0.54850E+00_JPRB, 0.56510E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.61575E+00_JPRB, 0.61556E+00_JPRB, 0.61860E+00_JPRB, 0.62215E+00_JPRB, 0.61842E+00_JPRB, 0.61973E+00_JPRB, 0.59888E+00_JPRB ,&
&  0.56817E+00_JPRB, 0.55936E+00_JPRB, 0.58468E+00_JPRB, 0.58963E+00_JPRB, 0.59212E+00_JPRB, 0.62047E+00_JPRB, 0.65294E+00_JPRB ,&
&  0.67309E+00_JPRB, 0.68265E+00_JPRB, 0.68237E+00_JPRB, 0.68211E+00_JPRB, 0.67062E+00_JPRB, 0.67538E+00_JPRB, 0.71600E+00_JPRB ,&
&  0.79151E+00_JPRB, 0.92921E+00_JPRB, 0.11377E+01_JPRB, 0.14043E+01_JPRB, 0.17280E+01_JPRB, 0.21974E+01_JPRB, 0.26614E+01_JPRB ,&
&  0.35145E+01_JPRB, 0.23323E+01_JPRB, 0.12851E+01_JPRB, 0.23479E+01_JPRB, 0.26308E+01_JPRB, 0.25835E+01_JPRB, 0.21096E+01_JPRB ,&
&  0.19484E+01_JPRB, 0.18266E+01_JPRB, 0.16808E+01_JPRB, 0.14924E+01_JPRB, 0.12383E+01_JPRB, 0.11396E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.10529E+01_JPRB, 0.11857E+01_JPRB, 0.14719E+01_JPRB, 0.18427E+01_JPRB, 0.20336E+01_JPRB, 0.21479E+01_JPRB ,&
&  0.24986E+01_JPRB, 0.29428E+01_JPRB, 0.30445E+01_JPRB, 0.21024E+01_JPRB, 0.16258E+01_JPRB, 0.15085E+01_JPRB, 0.14839E+01_JPRB ,&
&  0.14311E+01_JPRB, 0.14465E+01_JPRB, 0.14735E+01_JPRB, 0.13774E+01_JPRB, 0.12532E+01_JPRB, 0.11618E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.10702E+01_JPRB, 0.98275E+00_JPRB, 0.89831E+00_JPRB, 0.80344E+00_JPRB, 0.71955E+00_JPRB, 0.69828E+00_JPRB, 0.68859E+00_JPRB ,&
&  0.69704E+00_JPRB, 0.72281E+00_JPRB, 0.72886E+00_JPRB, 0.75499E+00_JPRB, 0.78402E+00_JPRB, 0.81493E+00_JPRB, 0.84552E+00_JPRB ,&
&  0.87289E+00_JPRB, 0.90305E+00_JPRB, 0.93041E+00_JPRB, 0.93589E+00_JPRB, 0.92626E+00_JPRB, 0.91092E+00_JPRB, 0.92752E+00_JPRB ,&
&  0.93498E+00_JPRB, 0.94583E+00_JPRB, 0.92302E+00_JPRB, 0.91620E+00_JPRB, 0.85797E+00_JPRB, 0.80227E+00_JPRB, 0.73925E+00_JPRB ,&
&  0.72009E+00_JPRB, 0.67310E+00_JPRB, 0.63805E+00_JPRB, 0.59806E+00_JPRB, 0.60443E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.64006E+00_JPRB, 0.63232E+00_JPRB, 0.63355E+00_JPRB, 0.63738E+00_JPRB, 0.63776E+00_JPRB, 0.64100E+00_JPRB, 0.64195E+00_JPRB ,&
&  0.60947E+00_JPRB, 0.57915E+00_JPRB, 0.56205E+00_JPRB, 0.55016E+00_JPRB, 0.54414E+00_JPRB, 0.56041E+00_JPRB, 0.57094E+00_JPRB ,&
&  0.57249E+00_JPRB, 0.57406E+00_JPRB, 0.60026E+00_JPRB, 0.60945E+00_JPRB, 0.60798E+00_JPRB, 0.59954E+00_JPRB, 0.59934E+00_JPRB ,&
&  0.62012E+00_JPRB, 0.67250E+00_JPRB, 0.77868E+00_JPRB, 0.92601E+00_JPRB, 0.10959E+01_JPRB, 0.14021E+01_JPRB, 0.18038E+01_JPRB ,&
&  0.24906E+01_JPRB, 0.25782E+01_JPRB, 0.11164E+01_JPRB, 0.20164E+01_JPRB, 0.27752E+01_JPRB, 0.28443E+01_JPRB, 0.26047E+01_JPRB ,&
&  0.23884E+01_JPRB, 0.21944E+01_JPRB, 0.22237E+01_JPRB, 0.19922E+01_JPRB, 0.15325E+01_JPRB, 0.12580E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.96928E+00_JPRB, 0.93431E+00_JPRB, 0.97292E+00_JPRB, 0.11150E+01_JPRB, 0.14099E+01_JPRB, 0.16862E+01_JPRB, 0.18231E+01_JPRB ,&
&  0.19916E+01_JPRB, 0.24135E+01_JPRB, 0.26988E+01_JPRB, 0.19409E+01_JPRB, 0.16161E+01_JPRB, 0.16519E+01_JPRB, 0.16403E+01_JPRB ,&
&  0.15774E+01_JPRB, 0.15661E+01_JPRB, 0.15430E+01_JPRB, 0.14366E+01_JPRB, 0.12999E+01_JPRB, 0.12421E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.10804E+01_JPRB, 0.10049E+01_JPRB, 0.92011E+00_JPRB, 0.82136E+00_JPRB, 0.74249E+00_JPRB, 0.73256E+00_JPRB ,&
&  0.73438E+00_JPRB, 0.75703E+00_JPRB, 0.76021E+00_JPRB, 0.77724E+00_JPRB, 0.79479E+00_JPRB, 0.81321E+00_JPRB, 0.82927E+00_JPRB ,&
&  0.84911E+00_JPRB, 0.87272E+00_JPRB, 0.89546E+00_JPRB, 0.89597E+00_JPRB, 0.89422E+00_JPRB, 0.89337E+00_JPRB, 0.90000E+00_JPRB ,&
&  0.91570E+00_JPRB, 0.93213E+00_JPRB, 0.91772E+00_JPRB, 0.91887E+00_JPRB, 0.92763E+00_JPRB, 0.89660E+00_JPRB, 0.85508E+00_JPRB ,&
&  0.88188E+00_JPRB, 0.90857E+00_JPRB, 0.83940E+00_JPRB, 0.69101E+00_JPRB, 0.65807E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.68678E+00_JPRB, 0.67216E+00_JPRB, 0.66222E+00_JPRB, 0.66042E+00_JPRB, 0.65719E+00_JPRB, 0.65656E+00_JPRB, 0.66498E+00_JPRB ,&
&  0.65299E+00_JPRB, 0.63082E+00_JPRB, 0.60442E+00_JPRB, 0.57550E+00_JPRB, 0.55337E+00_JPRB, 0.55549E+00_JPRB, 0.56342E+00_JPRB ,&
&  0.56615E+00_JPRB, 0.56136E+00_JPRB, 0.56091E+00_JPRB, 0.56939E+00_JPRB, 0.56882E+00_JPRB, 0.56320E+00_JPRB, 0.55708E+00_JPRB ,&
&  0.55685E+00_JPRB, 0.56928E+00_JPRB, 0.60864E+00_JPRB, 0.67876E+00_JPRB, 0.76872E+00_JPRB, 0.92320E+00_JPRB, 0.11547E+01_JPRB ,&
&  0.16855E+01_JPRB, 0.20444E+01_JPRB, 0.21950E+01_JPRB, 0.33139E+01_JPRB, 0.34789E+01_JPRB, 0.32720E+01_JPRB, 0.32120E+01_JPRB ,&
&  0.30806E+01_JPRB, 0.28531E+01_JPRB, 0.30052E+01_JPRB, 0.23589E+01_JPRB, 0.18445E+01_JPRB, 0.14246E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.91141E+00_JPRB, 0.88400E+00_JPRB, 0.91419E+00_JPRB, 0.10583E+01_JPRB, 0.13147E+01_JPRB, 0.15215E+01_JPRB ,&
&  0.16734E+01_JPRB, 0.19603E+01_JPRB, 0.23770E+01_JPRB, 0.20332E+01_JPRB, 0.16935E+01_JPRB, 0.19081E+01_JPRB, 0.21150E+01_JPRB ,&
&  0.23544E+01_JPRB, 0.18540E+01_JPRB, 0.16570E+01_JPRB, 0.15213E+01_JPRB, 0.13518E+01_JPRB, 0.14386E+01_JPRB, 0.13174E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.11563E+01_JPRB, 0.10845E+01_JPRB, 0.10143E+01_JPRB, 0.93568E+00_JPRB, 0.85391E+00_JPRB, 0.80906E+00_JPRB ,&
&  0.80221E+00_JPRB, 0.80986E+00_JPRB, 0.80788E+00_JPRB, 0.81536E+00_JPRB, 0.81649E+00_JPRB, 0.82807E+00_JPRB, 0.83996E+00_JPRB ,&
&  0.85554E+00_JPRB, 0.86469E+00_JPRB, 0.84803E+00_JPRB, 0.84806E+00_JPRB, 0.85030E+00_JPRB, 0.84835E+00_JPRB, 0.84283E+00_JPRB ,&
&  0.85360E+00_JPRB, 0.87860E+00_JPRB, 0.89319E+00_JPRB, 0.90451E+00_JPRB, 0.91697E+00_JPRB, 0.89954E+00_JPRB, 0.85669E+00_JPRB ,&
&  0.83704E+00_JPRB, 0.83495E+00_JPRB, 0.81579E+00_JPRB, 0.74671E+00_JPRB, 0.71252E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.71169E+00_JPRB, 0.69780E+00_JPRB, 0.68843E+00_JPRB, 0.67818E+00_JPRB, 0.66598E+00_JPRB, 0.66282E+00_JPRB, 0.66944E+00_JPRB ,&
&  0.66771E+00_JPRB, 0.65397E+00_JPRB, 0.63446E+00_JPRB, 0.61463E+00_JPRB, 0.58887E+00_JPRB, 0.57488E+00_JPRB, 0.56672E+00_JPRB ,&
&  0.56128E+00_JPRB, 0.55877E+00_JPRB, 0.55212E+00_JPRB, 0.55034E+00_JPRB, 0.54897E+00_JPRB, 0.53776E+00_JPRB, 0.52928E+00_JPRB ,&
&  0.52507E+00_JPRB, 0.53028E+00_JPRB, 0.54557E+00_JPRB, 0.58100E+00_JPRB, 0.63664E+00_JPRB, 0.73458E+00_JPRB, 0.86384E+00_JPRB ,&
&  0.11953E+01_JPRB, 0.17143E+01_JPRB, 0.15682E+01_JPRB, 0.34652E+01_JPRB, 0.34360E+01_JPRB, 0.33727E+01_JPRB, 0.31840E+01_JPRB ,&
&  0.31639E+01_JPRB, 0.35109E+01_JPRB, 0.30470E+01_JPRB, 0.25013E+01_JPRB, 0.19433E+01_JPRB, 0.15414E+01_JPRB, 0.12434E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.94754E+00_JPRB, 0.89444E+00_JPRB, 0.86870E+00_JPRB, 0.87483E+00_JPRB, 0.10110E+01_JPRB, 0.12368E+01_JPRB ,&
&  0.14111E+01_JPRB, 0.16233E+01_JPRB, 0.20114E+01_JPRB, 0.19225E+01_JPRB, 0.18045E+01_JPRB, 0.20889E+01_JPRB, 0.26261E+01_JPRB ,&
&  0.31191E+01_JPRB, 0.24745E+01_JPRB, 0.19125E+01_JPRB, 0.16719E+01_JPRB, 0.15315E+01_JPRB, 0.15032E+01_JPRB, 0.13897E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.11982E+01_JPRB, 0.11408E+01_JPRB, 0.10863E+01_JPRB, 0.10231E+01_JPRB, 0.95989E+00_JPRB, 0.89649E+00_JPRB ,&
&  0.86720E+00_JPRB, 0.85862E+00_JPRB, 0.85567E+00_JPRB, 0.86326E+00_JPRB, 0.86467E+00_JPRB, 0.87289E+00_JPRB, 0.87611E+00_JPRB ,&
&  0.88501E+00_JPRB, 0.88056E+00_JPRB, 0.83725E+00_JPRB, 0.81903E+00_JPRB, 0.81654E+00_JPRB, 0.81045E+00_JPRB, 0.80288E+00_JPRB ,&
&  0.81926E+00_JPRB, 0.84107E+00_JPRB, 0.85172E+00_JPRB, 0.85782E+00_JPRB, 0.89264E+00_JPRB, 0.90267E+00_JPRB, 0.86218E+00_JPRB ,&
&  0.82877E+00_JPRB, 0.80051E+00_JPRB, 0.77479E+00_JPRB, 0.74378E+00_JPRB, 0.72934E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.71895E+00_JPRB, 0.70837E+00_JPRB, 0.69850E+00_JPRB, 0.68551E+00_JPRB, 0.66763E+00_JPRB, 0.65846E+00_JPRB, 0.66143E+00_JPRB ,&
&  0.66643E+00_JPRB, 0.65812E+00_JPRB, 0.64472E+00_JPRB, 0.62607E+00_JPRB, 0.60153E+00_JPRB, 0.58746E+00_JPRB, 0.57764E+00_JPRB ,&
&  0.56345E+00_JPRB, 0.55139E+00_JPRB, 0.53533E+00_JPRB, 0.52885E+00_JPRB, 0.52862E+00_JPRB, 0.52074E+00_JPRB, 0.50462E+00_JPRB ,&
&  0.50560E+00_JPRB, 0.51511E+00_JPRB, 0.52741E+00_JPRB, 0.54516E+00_JPRB, 0.58416E+00_JPRB, 0.65662E+00_JPRB, 0.77104E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.20278E+01_JPRB, 0.14400E+01_JPRB, 0.26688E+01_JPRB, 0.27591E+01_JPRB, 0.29543E+01_JPRB, 0.30131E+01_JPRB ,&
&  0.30980E+01_JPRB, 0.31504E+01_JPRB, 0.28948E+01_JPRB, 0.23595E+01_JPRB, 0.18346E+01_JPRB, 0.14935E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.10215E+01_JPRB, 0.94298E+00_JPRB, 0.89054E+00_JPRB, 0.85612E+00_JPRB, 0.86156E+00_JPRB, 0.97126E+00_JPRB ,&
&  0.11669E+01_JPRB, 0.13493E+01_JPRB, 0.16447E+01_JPRB, 0.19515E+01_JPRB, 0.18050E+01_JPRB, 0.19995E+01_JPRB, 0.27895E+01_JPRB ,&
&  0.33139E+01_JPRB, 0.33642E+01_JPRB, 0.23689E+01_JPRB, 0.19253E+01_JPRB, 0.17388E+01_JPRB, 0.16216E+01_JPRB, 0.14445E+01_JPRB ,&
&  0.12810E+01_JPRB, 0.11985E+01_JPRB, 0.11526E+01_JPRB, 0.11158E+01_JPRB, 0.10712E+01_JPRB, 0.10080E+01_JPRB, 0.94322E+00_JPRB ,&
&  0.88905E+00_JPRB, 0.87505E+00_JPRB, 0.87419E+00_JPRB, 0.87871E+00_JPRB, 0.88993E+00_JPRB, 0.90232E+00_JPRB, 0.90993E+00_JPRB ,&
&  0.90971E+00_JPRB, 0.90463E+00_JPRB, 0.85429E+00_JPRB, 0.82251E+00_JPRB, 0.81001E+00_JPRB, 0.79904E+00_JPRB, 0.78801E+00_JPRB ,&
&  0.79885E+00_JPRB, 0.81128E+00_JPRB, 0.82046E+00_JPRB, 0.84279E+00_JPRB, 0.87647E+00_JPRB, 0.91379E+00_JPRB, 0.83571E+00_JPRB ,&
&  0.82852E+00_JPRB, 0.80482E+00_JPRB, 0.77820E+00_JPRB, 0.74899E+00_JPRB, 0.73425E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.71511E+00_JPRB, 0.70558E+00_JPRB, 0.69550E+00_JPRB, 0.68402E+00_JPRB, 0.67048E+00_JPRB, 0.65958E+00_JPRB, 0.65485E+00_JPRB ,&
&  0.65335E+00_JPRB, 0.65332E+00_JPRB, 0.64539E+00_JPRB, 0.62568E+00_JPRB, 0.60425E+00_JPRB, 0.58482E+00_JPRB, 0.57345E+00_JPRB ,&
&  0.56343E+00_JPRB, 0.55023E+00_JPRB, 0.53425E+00_JPRB, 0.51957E+00_JPRB, 0.51511E+00_JPRB, 0.51508E+00_JPRB, 0.50605E+00_JPRB ,&
&  0.51154E+00_JPRB, 0.52038E+00_JPRB, 0.52833E+00_JPRB, 0.53716E+00_JPRB, 0.56697E+00_JPRB, 0.62763E+00_JPRB, 0.73244E+00_JPRB ,&
&  0.93913E+00_JPRB, 0.81416E+00_JPRB, 0.16196E+01_JPRB, 0.19939E+01_JPRB, 0.21933E+01_JPRB, 0.23659E+01_JPRB, 0.27313E+01_JPRB ,&
&  0.30065E+01_JPRB, 0.29055E+01_JPRB, 0.25891E+01_JPRB, 0.21567E+01_JPRB, 0.17627E+01_JPRB, 0.14739E+01_JPRB, 0.12812E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.10652E+01_JPRB, 0.98197E+00_JPRB, 0.91045E+00_JPRB, 0.86298E+00_JPRB, 0.83641E+00_JPRB, 0.84416E+00_JPRB ,&
&  0.91786E+00_JPRB, 0.10618E+01_JPRB, 0.12630E+01_JPRB, 0.16181E+01_JPRB, 0.17877E+01_JPRB, 0.19065E+01_JPRB, 0.22040E+01_JPRB ,&
&  0.28791E+01_JPRB, 0.33061E+01_JPRB, 0.26938E+01_JPRB, 0.21891E+01_JPRB, 0.19229E+01_JPRB, 0.16792E+01_JPRB, 0.14622E+01_JPRB ,&
&  0.12923E+01_JPRB, 0.11896E+01_JPRB, 0.11260E+01_JPRB, 0.10880E+01_JPRB, 0.10658E+01_JPRB, 0.10342E+01_JPRB, 0.98841E+00_JPRB ,&
&  0.94616E+00_JPRB, 0.91852E+00_JPRB, 0.90795E+00_JPRB, 0.90633E+00_JPRB, 0.90799E+00_JPRB, 0.91781E+00_JPRB, 0.91930E+00_JPRB ,&
&  0.90990E+00_JPRB, 0.89466E+00_JPRB, 0.86596E+00_JPRB, 0.83273E+00_JPRB, 0.81792E+00_JPRB, 0.80154E+00_JPRB, 0.78445E+00_JPRB ,&
&  0.78392E+00_JPRB, 0.78813E+00_JPRB, 0.79820E+00_JPRB, 0.82898E+00_JPRB, 0.84707E+00_JPRB, 0.87662E+00_JPRB, 0.81724E+00_JPRB ,&
&  0.80334E+00_JPRB, 0.78073E+00_JPRB, 0.75895E+00_JPRB, 0.73780E+00_JPRB, 0.72905E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.71259E+00_JPRB, 0.70269E+00_JPRB, 0.69041E+00_JPRB, 0.67772E+00_JPRB, 0.66355E+00_JPRB, 0.65132E+00_JPRB, 0.64396E+00_JPRB ,&
&  0.63957E+00_JPRB, 0.63829E+00_JPRB, 0.63792E+00_JPRB, 0.62390E+00_JPRB, 0.60570E+00_JPRB, 0.58845E+00_JPRB, 0.57758E+00_JPRB ,&
&  0.56956E+00_JPRB, 0.56025E+00_JPRB, 0.54867E+00_JPRB, 0.53457E+00_JPRB, 0.52730E+00_JPRB, 0.52252E+00_JPRB, 0.51878E+00_JPRB ,&
&  0.52401E+00_JPRB, 0.52837E+00_JPRB, 0.53417E+00_JPRB, 0.54323E+00_JPRB, 0.56745E+00_JPRB, 0.62202E+00_JPRB, 0.71760E+00_JPRB ,&
&  0.93543E+00_JPRB, 0.14422E+01_JPRB, 0.18759E+01_JPRB, 0.17657E+01_JPRB, 0.17782E+01_JPRB, 0.18362E+01_JPRB, 0.21673E+01_JPRB ,&
&  0.23882E+01_JPRB, 0.24538E+01_JPRB, 0.22358E+01_JPRB, 0.19321E+01_JPRB, 0.16597E+01_JPRB, 0.14480E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.11500E+01_JPRB, 0.10228E+01_JPRB, 0.92970E+00_JPRB, 0.86451E+00_JPRB, 0.82264E+00_JPRB, 0.79600E+00_JPRB, 0.78529E+00_JPRB ,&
&  0.79657E+00_JPRB, 0.84785E+00_JPRB, 0.96718E+00_JPRB, 0.12093E+01_JPRB, 0.17176E+01_JPRB, 0.17162E+01_JPRB, 0.20754E+01_JPRB ,&
&  0.27540E+01_JPRB, 0.27039E+01_JPRB, 0.24681E+01_JPRB, 0.21647E+01_JPRB, 0.18990E+01_JPRB, 0.16688E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.12280E+01_JPRB, 0.11384E+01_JPRB, 0.10721E+01_JPRB, 0.10301E+01_JPRB, 0.10130E+01_JPRB, 0.99891E+00_JPRB ,&
&  0.97969E+00_JPRB, 0.95714E+00_JPRB, 0.94086E+00_JPRB, 0.93298E+00_JPRB, 0.92472E+00_JPRB, 0.91878E+00_JPRB, 0.89758E+00_JPRB ,&
&  0.87403E+00_JPRB, 0.85249E+00_JPRB, 0.84807E+00_JPRB, 0.81799E+00_JPRB, 0.80754E+00_JPRB, 0.79547E+00_JPRB, 0.77477E+00_JPRB ,&
&  0.75661E+00_JPRB, 0.76313E+00_JPRB, 0.78082E+00_JPRB, 0.80665E+00_JPRB, 0.85834E+00_JPRB, 0.88145E+00_JPRB, 0.81707E+00_JPRB ,&
&  0.79145E+00_JPRB, 0.75921E+00_JPRB, 0.73228E+00_JPRB, 0.72041E+00_JPRB, 0.72401E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.70161E+00_JPRB, 0.68823E+00_JPRB, 0.67526E+00_JPRB, 0.66254E+00_JPRB, 0.64747E+00_JPRB, 0.63555E+00_JPRB, 0.62495E+00_JPRB ,&
&  0.61749E+00_JPRB, 0.61357E+00_JPRB, 0.61436E+00_JPRB, 0.61524E+00_JPRB, 0.61192E+00_JPRB, 0.60197E+00_JPRB, 0.58526E+00_JPRB ,&
&  0.57413E+00_JPRB, 0.56410E+00_JPRB, 0.55192E+00_JPRB, 0.53713E+00_JPRB, 0.52281E+00_JPRB, 0.51587E+00_JPRB, 0.51456E+00_JPRB ,&
&  0.51871E+00_JPRB, 0.52363E+00_JPRB, 0.52740E+00_JPRB, 0.53926E+00_JPRB, 0.56224E+00_JPRB, 0.61392E+00_JPRB, 0.69231E+00_JPRB ,&
&  0.84888E+00_JPRB, 0.13492E+01_JPRB, 0.14870E+01_JPRB, 0.15029E+01_JPRB, 0.15504E+01_JPRB, 0.15632E+01_JPRB, 0.17192E+01_JPRB ,&
&  0.18685E+01_JPRB, 0.19617E+01_JPRB, 0.19032E+01_JPRB, 0.16880E+01_JPRB, 0.14969E+01_JPRB, 0.13649E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.10007E+01_JPRB, 0.91690E+00_JPRB, 0.86014E+00_JPRB, 0.81643E+00_JPRB, 0.78320E+00_JPRB, 0.76329E+00_JPRB ,&
&  0.75706E+00_JPRB, 0.76400E+00_JPRB, 0.80019E+00_JPRB, 0.88746E+00_JPRB, 0.12343E+01_JPRB, 0.15416E+01_JPRB, 0.18117E+01_JPRB ,&
&  0.19161E+01_JPRB, 0.19698E+01_JPRB, 0.19872E+01_JPRB, 0.18756E+01_JPRB, 0.17184E+01_JPRB, 0.15364E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.12136E+01_JPRB, 0.11238E+01_JPRB, 0.10409E+01_JPRB, 0.98201E+00_JPRB, 0.95029E+00_JPRB, 0.93256E+00_JPRB ,&
&  0.91527E+00_JPRB, 0.90390E+00_JPRB, 0.89419E+00_JPRB, 0.88433E+00_JPRB, 0.87204E+00_JPRB, 0.85554E+00_JPRB, 0.83400E+00_JPRB ,&
&  0.80903E+00_JPRB, 0.78480E+00_JPRB, 0.77582E+00_JPRB, 0.76687E+00_JPRB, 0.75937E+00_JPRB, 0.74749E+00_JPRB, 0.73270E+00_JPRB ,&
&  0.73021E+00_JPRB, 0.75745E+00_JPRB, 0.76752E+00_JPRB, 0.79612E+00_JPRB, 0.82731E+00_JPRB, 0.83853E+00_JPRB, 0.79824E+00_JPRB ,&
&  0.76374E+00_JPRB, 0.73143E+00_JPRB, 0.70681E+00_JPRB, 0.69533E+00_JPRB, 0.70966E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.69404E+00_JPRB, 0.67551E+00_JPRB, 0.65647E+00_JPRB, 0.63530E+00_JPRB, 0.61755E+00_JPRB, 0.60613E+00_JPRB, 0.59800E+00_JPRB ,&
&  0.59325E+00_JPRB, 0.59114E+00_JPRB, 0.58957E+00_JPRB, 0.58765E+00_JPRB, 0.58471E+00_JPRB, 0.58078E+00_JPRB, 0.57247E+00_JPRB ,&
&  0.56193E+00_JPRB, 0.54898E+00_JPRB, 0.53828E+00_JPRB, 0.52937E+00_JPRB, 0.51341E+00_JPRB, 0.50306E+00_JPRB, 0.50357E+00_JPRB ,&
&  0.50810E+00_JPRB, 0.51322E+00_JPRB, 0.51803E+00_JPRB, 0.52835E+00_JPRB, 0.55468E+00_JPRB, 0.60714E+00_JPRB, 0.66891E+00_JPRB ,&
&  0.80084E+00_JPRB, 0.97795E+00_JPRB, 0.11371E+01_JPRB, 0.11732E+01_JPRB, 0.12008E+01_JPRB, 0.12662E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.14143E+01_JPRB, 0.15363E+01_JPRB, 0.15148E+01_JPRB, 0.13759E+01_JPRB, 0.12258E+01_JPRB, 0.11158E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.99122E+00_JPRB, 0.94161E+00_JPRB, 0.89925E+00_JPRB, 0.86788E+00_JPRB, 0.83789E+00_JPRB, 0.80736E+00_JPRB, 0.77767E+00_JPRB ,&
&  0.75706E+00_JPRB, 0.74375E+00_JPRB, 0.74122E+00_JPRB, 0.76167E+00_JPRB, 0.84724E+00_JPRB, 0.10043E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.13899E+01_JPRB, 0.14890E+01_JPRB, 0.15707E+01_JPRB, 0.15660E+01_JPRB, 0.15009E+01_JPRB, 0.14226E+01_JPRB, 0.13383E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.11001E+01_JPRB, 0.10038E+01_JPRB, 0.93511E+00_JPRB, 0.88811E+00_JPRB, 0.85884E+00_JPRB, 0.84174E+00_JPRB ,&
&  0.82886E+00_JPRB, 0.82594E+00_JPRB, 0.84063E+00_JPRB, 0.85559E+00_JPRB, 0.85341E+00_JPRB, 0.82823E+00_JPRB, 0.79265E+00_JPRB ,&
&  0.76054E+00_JPRB, 0.73491E+00_JPRB, 0.71770E+00_JPRB, 0.70434E+00_JPRB, 0.69397E+00_JPRB, 0.68657E+00_JPRB, 0.67757E+00_JPRB ,&
&  0.67289E+00_JPRB, 0.69564E+00_JPRB, 0.71266E+00_JPRB, 0.77295E+00_JPRB, 0.81666E+00_JPRB, 0.78566E+00_JPRB, 0.74588E+00_JPRB ,&
&  0.71540E+00_JPRB, 0.69539E+00_JPRB, 0.67595E+00_JPRB, 0.66441E+00_JPRB, 0.71211E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.61270E+00_JPRB, 0.60806E+00_JPRB, 0.59650E+00_JPRB, 0.58455E+00_JPRB, 0.57467E+00_JPRB, 0.56942E+00_JPRB, 0.56488E+00_JPRB ,&
&  0.56205E+00_JPRB, 0.56036E+00_JPRB, 0.55932E+00_JPRB, 0.55665E+00_JPRB, 0.55175E+00_JPRB, 0.54432E+00_JPRB, 0.53499E+00_JPRB ,&
&  0.52628E+00_JPRB, 0.51762E+00_JPRB, 0.50994E+00_JPRB, 0.50184E+00_JPRB, 0.49151E+00_JPRB, 0.48548E+00_JPRB, 0.48767E+00_JPRB ,&
&  0.49384E+00_JPRB, 0.50111E+00_JPRB, 0.50828E+00_JPRB, 0.51766E+00_JPRB, 0.54966E+00_JPRB, 0.59756E+00_JPRB, 0.64177E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.80524E+00_JPRB, 0.87916E+00_JPRB, 0.95425E+00_JPRB, 0.10367E+01_JPRB, 0.10655E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.11669E+01_JPRB, 0.11679E+01_JPRB, 0.11147E+01_JPRB, 0.10073E+01_JPRB, 0.90152E+00_JPRB, 0.83569E+00_JPRB ,&
&  0.80286E+00_JPRB, 0.78706E+00_JPRB, 0.77257E+00_JPRB, 0.75867E+00_JPRB, 0.74679E+00_JPRB, 0.73491E+00_JPRB, 0.72281E+00_JPRB ,&
&  0.71126E+00_JPRB, 0.70212E+00_JPRB, 0.69657E+00_JPRB, 0.69528E+00_JPRB, 0.70839E+00_JPRB, 0.75808E+00_JPRB, 0.86623E+00_JPRB ,&
&  0.10179E+01_JPRB, 0.11604E+01_JPRB, 0.12365E+01_JPRB, 0.12818E+01_JPRB, 0.13119E+01_JPRB, 0.12637E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.92079E+00_JPRB, 0.84623E+00_JPRB, 0.79295E+00_JPRB, 0.75666E+00_JPRB, 0.73436E+00_JPRB, 0.72262E+00_JPRB ,&
&  0.71772E+00_JPRB, 0.71677E+00_JPRB, 0.72129E+00_JPRB, 0.72542E+00_JPRB, 0.72769E+00_JPRB, 0.72217E+00_JPRB, 0.70931E+00_JPRB ,&
&  0.69088E+00_JPRB, 0.67716E+00_JPRB, 0.66499E+00_JPRB, 0.65193E+00_JPRB, 0.64012E+00_JPRB, 0.62854E+00_JPRB, 0.61923E+00_JPRB ,&
&  0.61381E+00_JPRB, 0.62352E+00_JPRB, 0.64325E+00_JPRB, 0.69288E+00_JPRB, 0.68459E+00_JPRB, 0.66879E+00_JPRB, 0.65053E+00_JPRB ,&
&  0.63981E+00_JPRB, 0.63420E+00_JPRB, 0.62168E+00_JPRB, 0.61292E+00_JPRB, 0.61217E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.57018E+00_JPRB, 0.56501E+00_JPRB, 0.55497E+00_JPRB, 0.54525E+00_JPRB, 0.54044E+00_JPRB, 0.53882E+00_JPRB, 0.53848E+00_JPRB ,&
&  0.53634E+00_JPRB, 0.53234E+00_JPRB, 0.52736E+00_JPRB, 0.52394E+00_JPRB, 0.51933E+00_JPRB, 0.51290E+00_JPRB, 0.50492E+00_JPRB ,&
&  0.49728E+00_JPRB, 0.49021E+00_JPRB, 0.48460E+00_JPRB, 0.47860E+00_JPRB, 0.47402E+00_JPRB, 0.47361E+00_JPRB, 0.47526E+00_JPRB ,&
&  0.48172E+00_JPRB, 0.49040E+00_JPRB, 0.49872E+00_JPRB, 0.51665E+00_JPRB, 0.54062E+00_JPRB, 0.57830E+00_JPRB, 0.62053E+00_JPRB ,&
&  0.71453E+00_JPRB, 0.66932E+00_JPRB, 0.72373E+00_JPRB, 0.83368E+00_JPRB, 0.90832E+00_JPRB, 0.93371E+00_JPRB, 0.90551E+00_JPRB ,&
&  0.89012E+00_JPRB, 0.89608E+00_JPRB, 0.89948E+00_JPRB, 0.89490E+00_JPRB, 0.86165E+00_JPRB, 0.78829E+00_JPRB, 0.71367E+00_JPRB ,&
&  0.66216E+00_JPRB, 0.64298E+00_JPRB, 0.63393E+00_JPRB, 0.62153E+00_JPRB, 0.60892E+00_JPRB, 0.60161E+00_JPRB, 0.59982E+00_JPRB ,&
&  0.59846E+00_JPRB, 0.59651E+00_JPRB, 0.59324E+00_JPRB, 0.58949E+00_JPRB, 0.58861E+00_JPRB, 0.60140E+00_JPRB, 0.64601E+00_JPRB ,&
&  0.73437E+00_JPRB, 0.83536E+00_JPRB, 0.92826E+00_JPRB, 0.10012E+01_JPRB, 0.10169E+01_JPRB, 0.99002E+00_JPRB, 0.93709E+00_JPRB ,&
&  0.87342E+00_JPRB, 0.81278E+00_JPRB, 0.76522E+00_JPRB, 0.72715E+00_JPRB, 0.69631E+00_JPRB, 0.67196E+00_JPRB, 0.65395E+00_JPRB ,&
&  0.64350E+00_JPRB, 0.63611E+00_JPRB, 0.63048E+00_JPRB, 0.62352E+00_JPRB, 0.61723E+00_JPRB, 0.61264E+00_JPRB, 0.61038E+00_JPRB ,&
&  0.60668E+00_JPRB, 0.60225E+00_JPRB, 0.59905E+00_JPRB, 0.59629E+00_JPRB, 0.59281E+00_JPRB, 0.58838E+00_JPRB, 0.58414E+00_JPRB ,&
&  0.58201E+00_JPRB, 0.58387E+00_JPRB, 0.58641E+00_JPRB, 0.61574E+00_JPRB, 0.62766E+00_JPRB, 0.61378E+00_JPRB, 0.60121E+00_JPRB ,&
&  0.59479E+00_JPRB, 0.58592E+00_JPRB, 0.58107E+00_JPRB, 0.54189E+00_JPRB, 0.56878E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.52765E+00_JPRB, 0.52832E+00_JPRB, 0.52497E+00_JPRB, 0.51714E+00_JPRB, 0.51143E+00_JPRB, 0.50774E+00_JPRB, 0.50608E+00_JPRB ,&
&  0.50552E+00_JPRB, 0.50430E+00_JPRB, 0.50131E+00_JPRB, 0.49572E+00_JPRB, 0.49080E+00_JPRB, 0.48480E+00_JPRB, 0.47855E+00_JPRB ,&
&  0.47316E+00_JPRB, 0.46831E+00_JPRB, 0.46539E+00_JPRB, 0.46183E+00_JPRB, 0.46051E+00_JPRB, 0.46199E+00_JPRB, 0.46403E+00_JPRB ,&
&  0.46730E+00_JPRB, 0.47064E+00_JPRB, 0.48172E+00_JPRB, 0.49944E+00_JPRB, 0.53000E+00_JPRB, 0.55074E+00_JPRB, 0.58930E+00_JPRB ,&
&  0.67615E+00_JPRB, 0.59236E+00_JPRB, 0.65842E+00_JPRB, 0.74208E+00_JPRB, 0.79800E+00_JPRB, 0.82558E+00_JPRB, 0.77653E+00_JPRB ,&
&  0.73750E+00_JPRB, 0.71615E+00_JPRB, 0.70832E+00_JPRB, 0.70644E+00_JPRB, 0.70454E+00_JPRB, 0.68432E+00_JPRB, 0.65780E+00_JPRB ,&
&  0.62902E+00_JPRB, 0.60519E+00_JPRB, 0.58457E+00_JPRB, 0.56291E+00_JPRB, 0.54341E+00_JPRB, 0.53063E+00_JPRB, 0.52424E+00_JPRB ,&
&  0.52237E+00_JPRB, 0.52218E+00_JPRB, 0.52172E+00_JPRB, 0.52215E+00_JPRB, 0.52425E+00_JPRB, 0.52864E+00_JPRB, 0.53786E+00_JPRB ,&
&  0.56549E+00_JPRB, 0.61409E+00_JPRB, 0.66823E+00_JPRB, 0.71939E+00_JPRB, 0.75620E+00_JPRB, 0.77112E+00_JPRB, 0.77364E+00_JPRB ,&
&  0.75906E+00_JPRB, 0.73278E+00_JPRB, 0.70239E+00_JPRB, 0.67307E+00_JPRB, 0.64722E+00_JPRB, 0.62540E+00_JPRB, 0.60574E+00_JPRB ,&
&  0.58850E+00_JPRB, 0.57221E+00_JPRB, 0.55915E+00_JPRB, 0.54977E+00_JPRB, 0.54127E+00_JPRB, 0.53388E+00_JPRB, 0.52967E+00_JPRB ,&
&  0.52755E+00_JPRB, 0.52679E+00_JPRB, 0.52663E+00_JPRB, 0.52882E+00_JPRB, 0.53186E+00_JPRB, 0.53437E+00_JPRB, 0.53502E+00_JPRB ,&
&  0.53322E+00_JPRB, 0.53441E+00_JPRB, 0.53455E+00_JPRB, 0.55065E+00_JPRB, 0.55447E+00_JPRB, 0.55344E+00_JPRB, 0.55045E+00_JPRB ,&
&  0.54828E+00_JPRB, 0.54022E+00_JPRB, 0.48632E+00_JPRB, 0.51188E+00_JPRB, 0.52180E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.49728E+00_JPRB, 0.49600E+00_JPRB, 0.49611E+00_JPRB, 0.49481E+00_JPRB, 0.49288E+00_JPRB, 0.49049E+00_JPRB, 0.48716E+00_JPRB ,&
&  0.48276E+00_JPRB, 0.47987E+00_JPRB, 0.47723E+00_JPRB, 0.47359E+00_JPRB, 0.46965E+00_JPRB, 0.46638E+00_JPRB, 0.46340E+00_JPRB ,&
&  0.46018E+00_JPRB, 0.45653E+00_JPRB, 0.45386E+00_JPRB, 0.45195E+00_JPRB, 0.44993E+00_JPRB, 0.44918E+00_JPRB, 0.45019E+00_JPRB ,&
&  0.45012E+00_JPRB, 0.45337E+00_JPRB, 0.46676E+00_JPRB, 0.48573E+00_JPRB, 0.51144E+00_JPRB, 0.53506E+00_JPRB, 0.55872E+00_JPRB ,&
&  0.55907E+00_JPRB, 0.50896E+00_JPRB, 0.55736E+00_JPRB, 0.62090E+00_JPRB, 0.68498E+00_JPRB, 0.72705E+00_JPRB, 0.69525E+00_JPRB ,&
&  0.64894E+00_JPRB, 0.61078E+00_JPRB, 0.58919E+00_JPRB, 0.57874E+00_JPRB, 0.57984E+00_JPRB, 0.58011E+00_JPRB, 0.57991E+00_JPRB ,&
&  0.57379E+00_JPRB, 0.55943E+00_JPRB, 0.54234E+00_JPRB, 0.52570E+00_JPRB, 0.51312E+00_JPRB, 0.50325E+00_JPRB, 0.49511E+00_JPRB ,&
&  0.49049E+00_JPRB, 0.48870E+00_JPRB, 0.48793E+00_JPRB, 0.48781E+00_JPRB, 0.48865E+00_JPRB, 0.49173E+00_JPRB, 0.49545E+00_JPRB ,&
&  0.49998E+00_JPRB, 0.51085E+00_JPRB, 0.53385E+00_JPRB, 0.56229E+00_JPRB, 0.59083E+00_JPRB, 0.61205E+00_JPRB, 0.62558E+00_JPRB ,&
&  0.63352E+00_JPRB, 0.63437E+00_JPRB, 0.62651E+00_JPRB, 0.61072E+00_JPRB, 0.59117E+00_JPRB, 0.57251E+00_JPRB, 0.55488E+00_JPRB ,&
&  0.53790E+00_JPRB, 0.52001E+00_JPRB, 0.50334E+00_JPRB, 0.49301E+00_JPRB, 0.48741E+00_JPRB, 0.48222E+00_JPRB, 0.47795E+00_JPRB ,&
&  0.47600E+00_JPRB, 0.47575E+00_JPRB, 0.47640E+00_JPRB, 0.47759E+00_JPRB, 0.47949E+00_JPRB, 0.48256E+00_JPRB, 0.48541E+00_JPRB ,&
&  0.48742E+00_JPRB, 0.49193E+00_JPRB, 0.49808E+00_JPRB, 0.51282E+00_JPRB, 0.52240E+00_JPRB, 0.52554E+00_JPRB, 0.52409E+00_JPRB ,&
&  0.51822E+00_JPRB, 0.51032E+00_JPRB, 0.50219E+00_JPRB, 0.49666E+00_JPRB, 0.49632E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.47362E+00_JPRB, 0.47251E+00_JPRB, 0.47175E+00_JPRB, 0.47004E+00_JPRB, 0.46853E+00_JPRB, 0.46726E+00_JPRB, 0.46394E+00_JPRB ,&
&  0.46282E+00_JPRB, 0.45914E+00_JPRB, 0.45557E+00_JPRB, 0.45385E+00_JPRB, 0.45168E+00_JPRB, 0.44894E+00_JPRB, 0.44693E+00_JPRB ,&
&  0.44466E+00_JPRB, 0.44185E+00_JPRB, 0.43964E+00_JPRB, 0.43678E+00_JPRB, 0.43269E+00_JPRB, 0.42977E+00_JPRB, 0.43010E+00_JPRB ,&
&  0.43069E+00_JPRB, 0.43364E+00_JPRB, 0.44481E+00_JPRB, 0.45675E+00_JPRB, 0.47242E+00_JPRB, 0.48946E+00_JPRB, 0.50408E+00_JPRB ,&
&  0.53070E+00_JPRB, 0.47093E+00_JPRB, 0.51641E+00_JPRB, 0.55425E+00_JPRB, 0.60873E+00_JPRB, 0.64947E+00_JPRB, 0.63514E+00_JPRB ,&
&  0.59526E+00_JPRB, 0.55278E+00_JPRB, 0.52680E+00_JPRB, 0.51488E+00_JPRB, 0.51974E+00_JPRB, 0.52600E+00_JPRB, 0.52498E+00_JPRB ,&
&  0.51805E+00_JPRB, 0.50331E+00_JPRB, 0.48645E+00_JPRB, 0.47579E+00_JPRB, 0.46986E+00_JPRB, 0.46501E+00_JPRB, 0.46025E+00_JPRB ,&
&  0.45668E+00_JPRB, 0.45492E+00_JPRB, 0.45393E+00_JPRB, 0.45338E+00_JPRB, 0.45388E+00_JPRB, 0.45562E+00_JPRB, 0.45809E+00_JPRB ,&
&  0.45983E+00_JPRB, 0.46013E+00_JPRB, 0.46249E+00_JPRB, 0.47014E+00_JPRB, 0.48202E+00_JPRB, 0.49413E+00_JPRB, 0.50361E+00_JPRB ,&
&  0.51027E+00_JPRB, 0.51543E+00_JPRB, 0.51797E+00_JPRB, 0.51564E+00_JPRB, 0.50843E+00_JPRB, 0.49818E+00_JPRB, 0.48758E+00_JPRB ,&
&  0.47696E+00_JPRB, 0.46645E+00_JPRB, 0.45457E+00_JPRB, 0.44770E+00_JPRB, 0.44486E+00_JPRB, 0.44338E+00_JPRB, 0.44244E+00_JPRB ,&
&  0.44190E+00_JPRB, 0.44139E+00_JPRB, 0.44149E+00_JPRB, 0.44177E+00_JPRB, 0.44202E+00_JPRB, 0.44273E+00_JPRB, 0.44470E+00_JPRB ,&
&  0.44839E+00_JPRB, 0.45496E+00_JPRB, 0.46346E+00_JPRB, 0.47671E+00_JPRB, 0.48827E+00_JPRB, 0.49465E+00_JPRB, 0.49700E+00_JPRB ,&
&  0.49429E+00_JPRB, 0.48874E+00_JPRB, 0.48193E+00_JPRB, 0.47577E+00_JPRB, 0.47464E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.45621E+00_JPRB, 0.45241E+00_JPRB, 0.44984E+00_JPRB, 0.44593E+00_JPRB, 0.44307E+00_JPRB, 0.44104E+00_JPRB, 0.43887E+00_JPRB ,&
&  0.43745E+00_JPRB, 0.43802E+00_JPRB, 0.43695E+00_JPRB, 0.43623E+00_JPRB, 0.43494E+00_JPRB, 0.43175E+00_JPRB, 0.42798E+00_JPRB ,&
&  0.42587E+00_JPRB, 0.42398E+00_JPRB, 0.42224E+00_JPRB, 0.42016E+00_JPRB, 0.41786E+00_JPRB, 0.41485E+00_JPRB, 0.41449E+00_JPRB ,&
&  0.41498E+00_JPRB, 0.41786E+00_JPRB, 0.42299E+00_JPRB, 0.42903E+00_JPRB, 0.43944E+00_JPRB, 0.45421E+00_JPRB, 0.46859E+00_JPRB ,&
&  0.48701E+00_JPRB, 0.47478E+00_JPRB, 0.50437E+00_JPRB, 0.52229E+00_JPRB, 0.55337E+00_JPRB, 0.58004E+00_JPRB, 0.58170E+00_JPRB ,&
&  0.56655E+00_JPRB, 0.53866E+00_JPRB, 0.51663E+00_JPRB, 0.50528E+00_JPRB, 0.50509E+00_JPRB, 0.50318E+00_JPRB, 0.49256E+00_JPRB ,&
&  0.47650E+00_JPRB, 0.45806E+00_JPRB, 0.44430E+00_JPRB, 0.43677E+00_JPRB, 0.43340E+00_JPRB, 0.43149E+00_JPRB, 0.42847E+00_JPRB ,&
&  0.42525E+00_JPRB, 0.42228E+00_JPRB, 0.42062E+00_JPRB, 0.41975E+00_JPRB, 0.41901E+00_JPRB, 0.41898E+00_JPRB, 0.41966E+00_JPRB ,&
&  0.42037E+00_JPRB, 0.42062E+00_JPRB, 0.42043E+00_JPRB, 0.42116E+00_JPRB, 0.42369E+00_JPRB, 0.42771E+00_JPRB, 0.43173E+00_JPRB ,&
&  0.43408E+00_JPRB, 0.43629E+00_JPRB, 0.43823E+00_JPRB, 0.43946E+00_JPRB, 0.43792E+00_JPRB, 0.43330E+00_JPRB, 0.42736E+00_JPRB ,&
&  0.42232E+00_JPRB, 0.41809E+00_JPRB, 0.41425E+00_JPRB, 0.41139E+00_JPRB, 0.41010E+00_JPRB, 0.41041E+00_JPRB, 0.41131E+00_JPRB ,&
&  0.41224E+00_JPRB, 0.41323E+00_JPRB, 0.41417E+00_JPRB, 0.41517E+00_JPRB, 0.41541E+00_JPRB, 0.41595E+00_JPRB, 0.41716E+00_JPRB ,&
&  0.42100E+00_JPRB, 0.42607E+00_JPRB, 0.43338E+00_JPRB, 0.44482E+00_JPRB, 0.45592E+00_JPRB, 0.46369E+00_JPRB, 0.46930E+00_JPRB ,&
&  0.47107E+00_JPRB, 0.46971E+00_JPRB, 0.46678E+00_JPRB, 0.46388E+00_JPRB, 0.46076E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.42897E+00_JPRB, 0.42380E+00_JPRB, 0.42017E+00_JPRB, 0.41759E+00_JPRB, 0.41750E+00_JPRB, 0.41614E+00_JPRB, 0.41289E+00_JPRB ,&
&  0.41050E+00_JPRB, 0.41208E+00_JPRB, 0.41523E+00_JPRB, 0.41662E+00_JPRB, 0.41582E+00_JPRB, 0.41398E+00_JPRB, 0.41057E+00_JPRB ,&
&  0.40612E+00_JPRB, 0.40383E+00_JPRB, 0.40354E+00_JPRB, 0.40442E+00_JPRB, 0.40393E+00_JPRB, 0.40153E+00_JPRB, 0.39945E+00_JPRB ,&
&  0.39960E+00_JPRB, 0.40121E+00_JPRB, 0.40272E+00_JPRB, 0.40577E+00_JPRB, 0.41340E+00_JPRB, 0.42749E+00_JPRB, 0.44013E+00_JPRB ,&
&  0.44961E+00_JPRB, 0.45468E+00_JPRB, 0.45000E+00_JPRB, 0.47129E+00_JPRB, 0.48888E+00_JPRB, 0.51152E+00_JPRB, 0.52789E+00_JPRB ,&
&  0.53271E+00_JPRB, 0.52571E+00_JPRB, 0.50978E+00_JPRB, 0.49630E+00_JPRB, 0.48632E+00_JPRB, 0.47264E+00_JPRB, 0.45234E+00_JPRB ,&
&  0.43170E+00_JPRB, 0.41535E+00_JPRB, 0.40544E+00_JPRB, 0.40051E+00_JPRB, 0.39927E+00_JPRB, 0.39889E+00_JPRB, 0.39620E+00_JPRB ,&
&  0.39309E+00_JPRB, 0.39021E+00_JPRB, 0.38890E+00_JPRB, 0.38781E+00_JPRB, 0.38652E+00_JPRB, 0.38547E+00_JPRB, 0.38505E+00_JPRB ,&
&  0.38503E+00_JPRB, 0.38498E+00_JPRB, 0.38519E+00_JPRB, 0.38571E+00_JPRB, 0.38588E+00_JPRB, 0.38576E+00_JPRB, 0.38617E+00_JPRB ,&
&  0.38667E+00_JPRB, 0.38722E+00_JPRB, 0.38768E+00_JPRB, 0.38781E+00_JPRB, 0.38756E+00_JPRB, 0.38672E+00_JPRB, 0.38455E+00_JPRB ,&
&  0.38180E+00_JPRB, 0.38064E+00_JPRB, 0.38034E+00_JPRB, 0.37941E+00_JPRB, 0.37844E+00_JPRB, 0.37886E+00_JPRB, 0.38061E+00_JPRB ,&
&  0.38248E+00_JPRB, 0.38483E+00_JPRB, 0.38682E+00_JPRB, 0.38832E+00_JPRB, 0.38855E+00_JPRB, 0.38919E+00_JPRB, 0.39015E+00_JPRB ,&
&  0.39218E+00_JPRB, 0.39548E+00_JPRB, 0.40026E+00_JPRB, 0.40680E+00_JPRB, 0.41420E+00_JPRB, 0.42025E+00_JPRB, 0.42618E+00_JPRB ,&
&  0.43136E+00_JPRB, 0.43417E+00_JPRB, 0.43479E+00_JPRB, 0.43510E+00_JPRB, 0.43347E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.39806E+00_JPRB, 0.39440E+00_JPRB, 0.39182E+00_JPRB, 0.39143E+00_JPRB, 0.39319E+00_JPRB, 0.39311E+00_JPRB, 0.39068E+00_JPRB ,&
&  0.38855E+00_JPRB, 0.38998E+00_JPRB, 0.39381E+00_JPRB, 0.39628E+00_JPRB, 0.39746E+00_JPRB, 0.39831E+00_JPRB, 0.39670E+00_JPRB ,&
&  0.39239E+00_JPRB, 0.38836E+00_JPRB, 0.38875E+00_JPRB, 0.39064E+00_JPRB, 0.39144E+00_JPRB, 0.38931E+00_JPRB, 0.38685E+00_JPRB ,&
&  0.38666E+00_JPRB, 0.38676E+00_JPRB, 0.38919E+00_JPRB, 0.39175E+00_JPRB, 0.39806E+00_JPRB, 0.40972E+00_JPRB, 0.42004E+00_JPRB ,&
&  0.42700E+00_JPRB, 0.42661E+00_JPRB, 0.42590E+00_JPRB, 0.43177E+00_JPRB, 0.43921E+00_JPRB, 0.45540E+00_JPRB, 0.47359E+00_JPRB ,&
&  0.48383E+00_JPRB, 0.48351E+00_JPRB, 0.47034E+00_JPRB, 0.45496E+00_JPRB, 0.44078E+00_JPRB, 0.42498E+00_JPRB, 0.40936E+00_JPRB ,&
&  0.39511E+00_JPRB, 0.38516E+00_JPRB, 0.37936E+00_JPRB, 0.37655E+00_JPRB, 0.37523E+00_JPRB, 0.37375E+00_JPRB, 0.37052E+00_JPRB ,&
&  0.36659E+00_JPRB, 0.36348E+00_JPRB, 0.36124E+00_JPRB, 0.35971E+00_JPRB, 0.35837E+00_JPRB, 0.35773E+00_JPRB, 0.35692E+00_JPRB ,&
&  0.35653E+00_JPRB, 0.35687E+00_JPRB, 0.35758E+00_JPRB, 0.35733E+00_JPRB, 0.35678E+00_JPRB, 0.35569E+00_JPRB, 0.35510E+00_JPRB ,&
&  0.35492E+00_JPRB, 0.35371E+00_JPRB, 0.35288E+00_JPRB, 0.35345E+00_JPRB, 0.35393E+00_JPRB, 0.35462E+00_JPRB, 0.35468E+00_JPRB ,&
&  0.35405E+00_JPRB, 0.35389E+00_JPRB, 0.35422E+00_JPRB, 0.35438E+00_JPRB, 0.35440E+00_JPRB, 0.35457E+00_JPRB, 0.35522E+00_JPRB ,&
&  0.35718E+00_JPRB, 0.35954E+00_JPRB, 0.36139E+00_JPRB, 0.36307E+00_JPRB, 0.36453E+00_JPRB, 0.36649E+00_JPRB, 0.36694E+00_JPRB ,&
&  0.36720E+00_JPRB, 0.36855E+00_JPRB, 0.37181E+00_JPRB, 0.37633E+00_JPRB, 0.38096E+00_JPRB, 0.38460E+00_JPRB, 0.38845E+00_JPRB ,&
&  0.39208E+00_JPRB, 0.39516E+00_JPRB, 0.39720E+00_JPRB, 0.39799E+00_JPRB, 0.39860E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.36910E+00_JPRB, 0.36786E+00_JPRB, 0.36737E+00_JPRB, 0.36795E+00_JPRB, 0.36995E+00_JPRB, 0.36994E+00_JPRB, 0.36939E+00_JPRB ,&
&  0.36900E+00_JPRB, 0.37129E+00_JPRB, 0.37552E+00_JPRB, 0.37830E+00_JPRB, 0.37918E+00_JPRB, 0.37940E+00_JPRB, 0.37877E+00_JPRB ,&
&  0.37788E+00_JPRB, 0.37717E+00_JPRB, 0.37606E+00_JPRB, 0.37673E+00_JPRB, 0.37635E+00_JPRB, 0.37471E+00_JPRB, 0.37501E+00_JPRB ,&
&  0.37464E+00_JPRB, 0.37507E+00_JPRB, 0.37794E+00_JPRB, 0.38076E+00_JPRB, 0.38670E+00_JPRB, 0.39744E+00_JPRB, 0.40653E+00_JPRB ,&
&  0.41089E+00_JPRB, 0.40614E+00_JPRB, 0.40083E+00_JPRB, 0.40137E+00_JPRB, 0.40494E+00_JPRB, 0.41276E+00_JPRB, 0.42258E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.42414E+00_JPRB, 0.41252E+00_JPRB, 0.40076E+00_JPRB, 0.39053E+00_JPRB, 0.38178E+00_JPRB, 0.37421E+00_JPRB ,&
&  0.36748E+00_JPRB, 0.36281E+00_JPRB, 0.36048E+00_JPRB, 0.35967E+00_JPRB, 0.35860E+00_JPRB, 0.35487E+00_JPRB, 0.35045E+00_JPRB ,&
&  0.34514E+00_JPRB, 0.34133E+00_JPRB, 0.33828E+00_JPRB, 0.33713E+00_JPRB, 0.33707E+00_JPRB, 0.33658E+00_JPRB, 0.33527E+00_JPRB ,&
&  0.33371E+00_JPRB, 0.33310E+00_JPRB, 0.33251E+00_JPRB, 0.33101E+00_JPRB, 0.32912E+00_JPRB, 0.32825E+00_JPRB, 0.32706E+00_JPRB ,&
&  0.32525E+00_JPRB, 0.32480E+00_JPRB, 0.32575E+00_JPRB, 0.32750E+00_JPRB, 0.32773E+00_JPRB, 0.32881E+00_JPRB, 0.33057E+00_JPRB ,&
&  0.32970E+00_JPRB, 0.32694E+00_JPRB, 0.32524E+00_JPRB, 0.32544E+00_JPRB, 0.32719E+00_JPRB, 0.32979E+00_JPRB, 0.33110E+00_JPRB ,&
&  0.33230E+00_JPRB, 0.33259E+00_JPRB, 0.33384E+00_JPRB, 0.33565E+00_JPRB, 0.33802E+00_JPRB, 0.33994E+00_JPRB, 0.34090E+00_JPRB ,&
&  0.34145E+00_JPRB, 0.34205E+00_JPRB, 0.34441E+00_JPRB, 0.34808E+00_JPRB, 0.35139E+00_JPRB, 0.35510E+00_JPRB, 0.35807E+00_JPRB ,&
&  0.36032E+00_JPRB, 0.36272E+00_JPRB, 0.36453E+00_JPRB, 0.36583E+00_JPRB, 0.36750E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.35009E+00_JPRB, 0.35102E+00_JPRB, 0.35239E+00_JPRB, 0.35290E+00_JPRB, 0.35410E+00_JPRB, 0.35470E+00_JPRB, 0.35597E+00_JPRB ,&
&  0.35668E+00_JPRB, 0.35861E+00_JPRB, 0.36123E+00_JPRB, 0.36142E+00_JPRB, 0.36120E+00_JPRB, 0.36113E+00_JPRB, 0.36188E+00_JPRB ,&
&  0.36423E+00_JPRB, 0.36515E+00_JPRB, 0.36466E+00_JPRB, 0.36482E+00_JPRB, 0.36341E+00_JPRB, 0.36200E+00_JPRB, 0.36356E+00_JPRB ,&
&  0.36370E+00_JPRB, 0.36435E+00_JPRB, 0.36691E+00_JPRB, 0.36961E+00_JPRB, 0.37527E+00_JPRB, 0.38292E+00_JPRB, 0.39122E+00_JPRB ,&
&  0.39628E+00_JPRB, 0.39338E+00_JPRB, 0.38546E+00_JPRB, 0.37986E+00_JPRB, 0.37877E+00_JPRB, 0.38013E+00_JPRB, 0.38335E+00_JPRB ,&
&  0.38323E+00_JPRB, 0.37725E+00_JPRB, 0.36870E+00_JPRB, 0.36337E+00_JPRB, 0.36046E+00_JPRB, 0.35624E+00_JPRB, 0.35045E+00_JPRB ,&
&  0.34586E+00_JPRB, 0.34395E+00_JPRB, 0.34372E+00_JPRB, 0.34255E+00_JPRB, 0.34010E+00_JPRB, 0.33607E+00_JPRB, 0.33109E+00_JPRB ,&
&  0.32661E+00_JPRB, 0.32224E+00_JPRB, 0.31937E+00_JPRB, 0.31751E+00_JPRB, 0.31722E+00_JPRB, 0.31711E+00_JPRB, 0.31588E+00_JPRB ,&
&  0.31349E+00_JPRB, 0.31183E+00_JPRB, 0.30972E+00_JPRB, 0.30611E+00_JPRB, 0.30352E+00_JPRB, 0.30264E+00_JPRB, 0.30149E+00_JPRB ,&
&  0.29991E+00_JPRB, 0.29964E+00_JPRB, 0.29978E+00_JPRB, 0.29956E+00_JPRB, 0.30073E+00_JPRB, 0.30269E+00_JPRB, 0.30030E+00_JPRB ,&
&  0.29795E+00_JPRB, 0.29635E+00_JPRB, 0.29649E+00_JPRB, 0.29841E+00_JPRB, 0.30153E+00_JPRB, 0.30485E+00_JPRB, 0.30751E+00_JPRB ,&
&  0.30862E+00_JPRB, 0.30893E+00_JPRB, 0.30931E+00_JPRB, 0.30972E+00_JPRB, 0.31143E+00_JPRB, 0.31464E+00_JPRB, 0.31816E+00_JPRB ,&
&  0.31978E+00_JPRB, 0.32242E+00_JPRB, 0.32548E+00_JPRB, 0.32936E+00_JPRB, 0.33175E+00_JPRB, 0.33414E+00_JPRB, 0.33643E+00_JPRB ,&
&  0.33821E+00_JPRB, 0.34038E+00_JPRB, 0.34328E+00_JPRB, 0.34560E+00_JPRB, 0.34797E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.33559E+00_JPRB, 0.33747E+00_JPRB, 0.33856E+00_JPRB, 0.34026E+00_JPRB, 0.34329E+00_JPRB, 0.34492E+00_JPRB, 0.34483E+00_JPRB ,&
&  0.34504E+00_JPRB, 0.34672E+00_JPRB, 0.34809E+00_JPRB, 0.34875E+00_JPRB, 0.34648E+00_JPRB, 0.34617E+00_JPRB, 0.34809E+00_JPRB ,&
&  0.35060E+00_JPRB, 0.35263E+00_JPRB, 0.35283E+00_JPRB, 0.35291E+00_JPRB, 0.35176E+00_JPRB, 0.35007E+00_JPRB, 0.35133E+00_JPRB ,&
&  0.35235E+00_JPRB, 0.35278E+00_JPRB, 0.35453E+00_JPRB, 0.35676E+00_JPRB, 0.36121E+00_JPRB, 0.36653E+00_JPRB, 0.37163E+00_JPRB ,&
&  0.37618E+00_JPRB, 0.37593E+00_JPRB, 0.36791E+00_JPRB, 0.35907E+00_JPRB, 0.35885E+00_JPRB, 0.35950E+00_JPRB, 0.35916E+00_JPRB ,&
&  0.35690E+00_JPRB, 0.35372E+00_JPRB, 0.35158E+00_JPRB, 0.34973E+00_JPRB, 0.34710E+00_JPRB, 0.34199E+00_JPRB, 0.33627E+00_JPRB ,&
&  0.33145E+00_JPRB, 0.32958E+00_JPRB, 0.32795E+00_JPRB, 0.32504E+00_JPRB, 0.32074E+00_JPRB, 0.31706E+00_JPRB, 0.31322E+00_JPRB ,&
&  0.30699E+00_JPRB, 0.30109E+00_JPRB, 0.29775E+00_JPRB, 0.29658E+00_JPRB, 0.29657E+00_JPRB, 0.29579E+00_JPRB, 0.29353E+00_JPRB ,&
&  0.29095E+00_JPRB, 0.28793E+00_JPRB, 0.28605E+00_JPRB, 0.28547E+00_JPRB, 0.28667E+00_JPRB, 0.28477E+00_JPRB, 0.27965E+00_JPRB ,&
&  0.27684E+00_JPRB, 0.27551E+00_JPRB, 0.27620E+00_JPRB, 0.27523E+00_JPRB, 0.27437E+00_JPRB, 0.27321E+00_JPRB, 0.27376E+00_JPRB ,&
&  0.27505E+00_JPRB, 0.27513E+00_JPRB, 0.27801E+00_JPRB, 0.28323E+00_JPRB, 0.28789E+00_JPRB, 0.29060E+00_JPRB, 0.29226E+00_JPRB ,&
&  0.29285E+00_JPRB, 0.29244E+00_JPRB, 0.29205E+00_JPRB, 0.29296E+00_JPRB, 0.29565E+00_JPRB, 0.29928E+00_JPRB, 0.30264E+00_JPRB ,&
&  0.30341E+00_JPRB, 0.30476E+00_JPRB, 0.30809E+00_JPRB, 0.31309E+00_JPRB, 0.31598E+00_JPRB, 0.31817E+00_JPRB, 0.31997E+00_JPRB ,&
&  0.32085E+00_JPRB, 0.32260E+00_JPRB, 0.32665E+00_JPRB, 0.32941E+00_JPRB, 0.33222E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.32049E+00_JPRB, 0.32168E+00_JPRB, 0.32389E+00_JPRB, 0.32647E+00_JPRB, 0.32782E+00_JPRB, 0.32739E+00_JPRB, 0.32640E+00_JPRB ,&
&  0.32686E+00_JPRB, 0.32942E+00_JPRB, 0.33215E+00_JPRB, 0.33313E+00_JPRB, 0.33368E+00_JPRB, 0.33427E+00_JPRB, 0.33626E+00_JPRB ,&
&  0.33903E+00_JPRB, 0.34226E+00_JPRB, 0.34232E+00_JPRB, 0.34216E+00_JPRB, 0.34133E+00_JPRB, 0.33996E+00_JPRB, 0.34105E+00_JPRB ,&
&  0.34407E+00_JPRB, 0.34409E+00_JPRB, 0.34393E+00_JPRB, 0.34351E+00_JPRB, 0.34604E+00_JPRB, 0.35054E+00_JPRB, 0.35403E+00_JPRB ,&
&  0.35493E+00_JPRB, 0.34911E+00_JPRB, 0.34041E+00_JPRB, 0.34032E+00_JPRB, 0.34202E+00_JPRB, 0.34359E+00_JPRB, 0.34356E+00_JPRB ,&
&  0.34301E+00_JPRB, 0.34192E+00_JPRB, 0.34052E+00_JPRB, 0.33887E+00_JPRB, 0.33644E+00_JPRB, 0.33088E+00_JPRB, 0.32458E+00_JPRB ,&
&  0.31953E+00_JPRB, 0.31545E+00_JPRB, 0.31163E+00_JPRB, 0.30630E+00_JPRB, 0.30092E+00_JPRB, 0.29700E+00_JPRB, 0.29234E+00_JPRB ,&
&  0.28680E+00_JPRB, 0.28308E+00_JPRB, 0.28182E+00_JPRB, 0.28147E+00_JPRB, 0.28126E+00_JPRB, 0.28014E+00_JPRB, 0.27869E+00_JPRB ,&
&  0.27627E+00_JPRB, 0.27440E+00_JPRB, 0.27258E+00_JPRB, 0.27130E+00_JPRB, 0.26431E+00_JPRB, 0.25656E+00_JPRB, 0.25070E+00_JPRB ,&
&  0.25038E+00_JPRB, 0.25281E+00_JPRB, 0.25539E+00_JPRB, 0.25663E+00_JPRB, 0.25954E+00_JPRB, 0.26059E+00_JPRB, 0.25989E+00_JPRB ,&
&  0.25607E+00_JPRB, 0.25370E+00_JPRB, 0.25531E+00_JPRB, 0.25691E+00_JPRB, 0.25775E+00_JPRB, 0.25957E+00_JPRB, 0.26118E+00_JPRB ,&
&  0.26434E+00_JPRB, 0.26637E+00_JPRB, 0.26879E+00_JPRB, 0.27157E+00_JPRB, 0.27360E+00_JPRB, 0.27416E+00_JPRB, 0.27217E+00_JPRB ,&
&  0.27258E+00_JPRB, 0.27902E+00_JPRB, 0.28973E+00_JPRB, 0.29920E+00_JPRB, 0.30225E+00_JPRB, 0.30495E+00_JPRB, 0.30720E+00_JPRB ,&
&  0.30856E+00_JPRB, 0.31262E+00_JPRB, 0.31606E+00_JPRB, 0.31799E+00_JPRB, 0.31915E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.30948E+00_JPRB, 0.30895E+00_JPRB, 0.30764E+00_JPRB, 0.30655E+00_JPRB, 0.30495E+00_JPRB, 0.30345E+00_JPRB, 0.30425E+00_JPRB ,&
&  0.30727E+00_JPRB, 0.31122E+00_JPRB, 0.31502E+00_JPRB, 0.31793E+00_JPRB, 0.32013E+00_JPRB, 0.32127E+00_JPRB, 0.32295E+00_JPRB ,&
&  0.32487E+00_JPRB, 0.32831E+00_JPRB, 0.32992E+00_JPRB, 0.33125E+00_JPRB, 0.33035E+00_JPRB, 0.32939E+00_JPRB, 0.33039E+00_JPRB ,&
&  0.33363E+00_JPRB, 0.33496E+00_JPRB, 0.33405E+00_JPRB, 0.33190E+00_JPRB, 0.33278E+00_JPRB, 0.33567E+00_JPRB, 0.33627E+00_JPRB ,&
&  0.32991E+00_JPRB, 0.31530E+00_JPRB, 0.30573E+00_JPRB, 0.31022E+00_JPRB, 0.31979E+00_JPRB, 0.32406E+00_JPRB, 0.32736E+00_JPRB ,&
&  0.32935E+00_JPRB, 0.32931E+00_JPRB, 0.32788E+00_JPRB, 0.32536E+00_JPRB, 0.32158E+00_JPRB, 0.31548E+00_JPRB, 0.30858E+00_JPRB ,&
&  0.30264E+00_JPRB, 0.29983E+00_JPRB, 0.29615E+00_JPRB, 0.28959E+00_JPRB, 0.28344E+00_JPRB, 0.27975E+00_JPRB, 0.27489E+00_JPRB ,&
&  0.26980E+00_JPRB, 0.26626E+00_JPRB, 0.26548E+00_JPRB, 0.26478E+00_JPRB, 0.26429E+00_JPRB, 0.26308E+00_JPRB, 0.25984E+00_JPRB ,&
&  0.25682E+00_JPRB, 0.25203E+00_JPRB, 0.24622E+00_JPRB, 0.24083E+00_JPRB, 0.23734E+00_JPRB, 0.23347E+00_JPRB, 0.22846E+00_JPRB ,&
&  0.22599E+00_JPRB, 0.22901E+00_JPRB, 0.23927E+00_JPRB, 0.24985E+00_JPRB, 0.25133E+00_JPRB, 0.23975E+00_JPRB, 0.23676E+00_JPRB ,&
&  0.23694E+00_JPRB, 0.23701E+00_JPRB, 0.23753E+00_JPRB, 0.23932E+00_JPRB, 0.24015E+00_JPRB, 0.23966E+00_JPRB, 0.24026E+00_JPRB ,&
&  0.24279E+00_JPRB, 0.24533E+00_JPRB, 0.24659E+00_JPRB, 0.24820E+00_JPRB, 0.24926E+00_JPRB, 0.24913E+00_JPRB, 0.24874E+00_JPRB ,&
&  0.24959E+00_JPRB, 0.25328E+00_JPRB, 0.25905E+00_JPRB, 0.26712E+00_JPRB, 0.27249E+00_JPRB, 0.27741E+00_JPRB, 0.28338E+00_JPRB ,&
&  0.29112E+00_JPRB, 0.29782E+00_JPRB, 0.30284E+00_JPRB, 0.30731E+00_JPRB, 0.30896E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.29894E+00_JPRB, 0.29989E+00_JPRB, 0.29557E+00_JPRB, 0.29195E+00_JPRB, 0.29009E+00_JPRB, 0.28965E+00_JPRB, 0.29132E+00_JPRB ,&
&  0.29510E+00_JPRB, 0.29819E+00_JPRB, 0.30120E+00_JPRB, 0.30387E+00_JPRB, 0.30504E+00_JPRB, 0.30477E+00_JPRB, 0.30591E+00_JPRB ,&
&  0.30709E+00_JPRB, 0.31022E+00_JPRB, 0.31254E+00_JPRB, 0.31601E+00_JPRB, 0.31629E+00_JPRB, 0.31701E+00_JPRB, 0.31831E+00_JPRB ,&
&  0.31777E+00_JPRB, 0.31787E+00_JPRB, 0.31921E+00_JPRB, 0.31663E+00_JPRB, 0.31514E+00_JPRB, 0.31292E+00_JPRB, 0.31156E+00_JPRB ,&
&  0.30471E+00_JPRB, 0.29715E+00_JPRB, 0.29061E+00_JPRB, 0.29550E+00_JPRB, 0.30353E+00_JPRB, 0.30489E+00_JPRB, 0.30973E+00_JPRB ,&
&  0.31153E+00_JPRB, 0.30944E+00_JPRB, 0.30738E+00_JPRB, 0.30345E+00_JPRB, 0.29983E+00_JPRB, 0.29566E+00_JPRB, 0.29227E+00_JPRB ,&
&  0.28838E+00_JPRB, 0.28901E+00_JPRB, 0.28384E+00_JPRB, 0.26873E+00_JPRB, 0.26166E+00_JPRB, 0.25720E+00_JPRB, 0.24937E+00_JPRB ,&
&  0.24113E+00_JPRB, 0.23940E+00_JPRB, 0.23915E+00_JPRB, 0.23598E+00_JPRB, 0.23551E+00_JPRB, 0.23581E+00_JPRB, 0.23561E+00_JPRB ,&
&  0.23390E+00_JPRB, 0.23357E+00_JPRB, 0.23213E+00_JPRB, 0.22994E+00_JPRB, 0.22770E+00_JPRB, 0.22675E+00_JPRB, 0.22720E+00_JPRB ,&
&  0.22884E+00_JPRB, 0.23028E+00_JPRB, 0.23629E+00_JPRB, 0.23965E+00_JPRB, 0.23325E+00_JPRB, 0.23133E+00_JPRB, 0.23305E+00_JPRB ,&
&  0.22840E+00_JPRB, 0.22763E+00_JPRB, 0.22767E+00_JPRB, 0.22763E+00_JPRB, 0.22805E+00_JPRB, 0.22900E+00_JPRB, 0.22865E+00_JPRB ,&
&  0.22936E+00_JPRB, 0.23110E+00_JPRB, 0.23163E+00_JPRB, 0.23172E+00_JPRB, 0.23139E+00_JPRB, 0.23194E+00_JPRB, 0.23342E+00_JPRB ,&
&  0.23505E+00_JPRB, 0.23823E+00_JPRB, 0.24152E+00_JPRB, 0.24643E+00_JPRB, 0.25029E+00_JPRB, 0.25340E+00_JPRB, 0.25925E+00_JPRB ,&
&  0.26391E+00_JPRB, 0.27457E+00_JPRB, 0.29203E+00_JPRB, 0.30024E+00_JPRB, 0.29393E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.28781E+00_JPRB, 0.28641E+00_JPRB, 0.28448E+00_JPRB, 0.28285E+00_JPRB, 0.28392E+00_JPRB, 0.28527E+00_JPRB, 0.28725E+00_JPRB ,&
&  0.28944E+00_JPRB, 0.29018E+00_JPRB, 0.29030E+00_JPRB, 0.28764E+00_JPRB, 0.28636E+00_JPRB, 0.28088E+00_JPRB, 0.27909E+00_JPRB ,&
&  0.27778E+00_JPRB, 0.28036E+00_JPRB, 0.28359E+00_JPRB, 0.28842E+00_JPRB, 0.29021E+00_JPRB, 0.29212E+00_JPRB, 0.29090E+00_JPRB ,&
&  0.28847E+00_JPRB, 0.28565E+00_JPRB, 0.28279E+00_JPRB, 0.28057E+00_JPRB, 0.27991E+00_JPRB, 0.27781E+00_JPRB, 0.27573E+00_JPRB ,&
&  0.27586E+00_JPRB, 0.27760E+00_JPRB, 0.27830E+00_JPRB, 0.27906E+00_JPRB, 0.28486E+00_JPRB, 0.28495E+00_JPRB, 0.28750E+00_JPRB ,&
&  0.28845E+00_JPRB, 0.28951E+00_JPRB, 0.28923E+00_JPRB, 0.28795E+00_JPRB, 0.28674E+00_JPRB, 0.27996E+00_JPRB, 0.27725E+00_JPRB ,&
&  0.27024E+00_JPRB, 0.26627E+00_JPRB, 0.25903E+00_JPRB, 0.25221E+00_JPRB, 0.24415E+00_JPRB, 0.23810E+00_JPRB, 0.23300E+00_JPRB ,&
&  0.22894E+00_JPRB, 0.22587E+00_JPRB, 0.22342E+00_JPRB, 0.22312E+00_JPRB, 0.22315E+00_JPRB, 0.22258E+00_JPRB, 0.22277E+00_JPRB ,&
&  0.22389E+00_JPRB, 0.22509E+00_JPRB, 0.22497E+00_JPRB, 0.22563E+00_JPRB, 0.22617E+00_JPRB, 0.22696E+00_JPRB, 0.22858E+00_JPRB ,&
&  0.23105E+00_JPRB, 0.23295E+00_JPRB, 0.23331E+00_JPRB, 0.23563E+00_JPRB, 0.23489E+00_JPRB, 0.23193E+00_JPRB, 0.22915E+00_JPRB ,&
&  0.22575E+00_JPRB, 0.22429E+00_JPRB, 0.22254E+00_JPRB, 0.22150E+00_JPRB, 0.22128E+00_JPRB, 0.22139E+00_JPRB, 0.22225E+00_JPRB ,&
&  0.22320E+00_JPRB, 0.22422E+00_JPRB, 0.22459E+00_JPRB, 0.22507E+00_JPRB, 0.22545E+00_JPRB, 0.22641E+00_JPRB, 0.22834E+00_JPRB ,&
&  0.23103E+00_JPRB, 0.23411E+00_JPRB, 0.23712E+00_JPRB, 0.24076E+00_JPRB, 0.24672E+00_JPRB, 0.25442E+00_JPRB, 0.26597E+00_JPRB ,&
&  0.27429E+00_JPRB, 0.28496E+00_JPRB, 0.28289E+00_JPRB, 0.28711E+00_JPRB, 0.28861E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.27937E+00_JPRB, 0.27928E+00_JPRB, 0.27818E+00_JPRB, 0.27762E+00_JPRB, 0.27815E+00_JPRB, 0.27907E+00_JPRB, 0.28046E+00_JPRB ,&
&  0.27711E+00_JPRB, 0.27502E+00_JPRB, 0.27076E+00_JPRB, 0.26605E+00_JPRB, 0.26178E+00_JPRB, 0.25882E+00_JPRB, 0.25719E+00_JPRB ,&
&  0.25708E+00_JPRB, 0.25967E+00_JPRB, 0.26139E+00_JPRB, 0.26379E+00_JPRB, 0.26383E+00_JPRB, 0.26398E+00_JPRB, 0.26220E+00_JPRB ,&
&  0.26032E+00_JPRB, 0.25961E+00_JPRB, 0.25986E+00_JPRB, 0.26125E+00_JPRB, 0.26386E+00_JPRB, 0.26532E+00_JPRB, 0.26724E+00_JPRB ,&
&  0.26991E+00_JPRB, 0.26868E+00_JPRB, 0.26925E+00_JPRB, 0.26849E+00_JPRB, 0.26916E+00_JPRB, 0.27061E+00_JPRB, 0.27145E+00_JPRB ,&
&  0.27311E+00_JPRB, 0.27467E+00_JPRB, 0.27496E+00_JPRB, 0.27418E+00_JPRB, 0.26956E+00_JPRB, 0.26751E+00_JPRB, 0.26286E+00_JPRB ,&
&  0.25709E+00_JPRB, 0.25179E+00_JPRB, 0.24521E+00_JPRB, 0.24047E+00_JPRB, 0.23459E+00_JPRB, 0.23116E+00_JPRB, 0.22850E+00_JPRB ,&
&  0.22603E+00_JPRB, 0.22518E+00_JPRB, 0.22392E+00_JPRB, 0.22247E+00_JPRB, 0.22211E+00_JPRB, 0.22058E+00_JPRB, 0.22052E+00_JPRB ,&
&  0.22100E+00_JPRB, 0.22141E+00_JPRB, 0.22267E+00_JPRB, 0.22316E+00_JPRB, 0.22400E+00_JPRB, 0.22472E+00_JPRB, 0.22509E+00_JPRB ,&
&  0.22622E+00_JPRB, 0.22788E+00_JPRB, 0.22774E+00_JPRB, 0.22820E+00_JPRB, 0.22727E+00_JPRB, 0.22511E+00_JPRB, 0.22284E+00_JPRB ,&
&  0.22030E+00_JPRB, 0.21834E+00_JPRB, 0.21778E+00_JPRB, 0.21690E+00_JPRB, 0.21703E+00_JPRB, 0.21744E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21821E+00_JPRB, 0.21883E+00_JPRB, 0.21981E+00_JPRB, 0.22212E+00_JPRB, 0.22364E+00_JPRB, 0.22566E+00_JPRB, 0.22782E+00_JPRB ,&
&  0.22947E+00_JPRB, 0.23262E+00_JPRB, 0.23536E+00_JPRB, 0.23915E+00_JPRB, 0.24806E+00_JPRB, 0.25526E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.27218E+00_JPRB, 0.27994E+00_JPRB, 0.27615E+00_JPRB, 0.27843E+00_JPRB, 0.27985E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.27373E+00_JPRB, 0.27468E+00_JPRB, 0.27440E+00_JPRB, 0.27543E+00_JPRB, 0.27616E+00_JPRB, 0.27731E+00_JPRB, 0.27709E+00_JPRB ,&
&  0.27680E+00_JPRB, 0.27319E+00_JPRB, 0.27127E+00_JPRB, 0.26781E+00_JPRB, 0.26468E+00_JPRB, 0.26123E+00_JPRB, 0.25845E+00_JPRB ,&
&  0.25671E+00_JPRB, 0.25533E+00_JPRB, 0.25408E+00_JPRB, 0.25231E+00_JPRB, 0.25003E+00_JPRB, 0.24769E+00_JPRB, 0.24558E+00_JPRB ,&
&  0.24362E+00_JPRB, 0.24381E+00_JPRB, 0.24533E+00_JPRB, 0.24773E+00_JPRB, 0.25218E+00_JPRB, 0.25515E+00_JPRB, 0.25900E+00_JPRB ,&
&  0.26453E+00_JPRB, 0.25991E+00_JPRB, 0.26014E+00_JPRB, 0.26074E+00_JPRB, 0.26070E+00_JPRB, 0.26076E+00_JPRB, 0.25880E+00_JPRB ,&
&  0.26419E+00_JPRB, 0.26455E+00_JPRB, 0.26865E+00_JPRB, 0.26720E+00_JPRB, 0.26189E+00_JPRB, 0.25842E+00_JPRB, 0.25316E+00_JPRB ,&
&  0.24984E+00_JPRB, 0.24493E+00_JPRB, 0.24163E+00_JPRB, 0.23840E+00_JPRB, 0.23420E+00_JPRB, 0.23250E+00_JPRB, 0.22958E+00_JPRB ,&
&  0.22789E+00_JPRB, 0.22759E+00_JPRB, 0.22646E+00_JPRB, 0.22651E+00_JPRB, 0.22551E+00_JPRB, 0.22407E+00_JPRB, 0.22320E+00_JPRB ,&
&  0.22244E+00_JPRB, 0.22160E+00_JPRB, 0.22134E+00_JPRB, 0.22101E+00_JPRB, 0.22113E+00_JPRB, 0.22104E+00_JPRB, 0.22096E+00_JPRB ,&
&  0.22107E+00_JPRB, 0.22117E+00_JPRB, 0.22059E+00_JPRB, 0.22047E+00_JPRB, 0.21938E+00_JPRB, 0.21842E+00_JPRB, 0.21688E+00_JPRB ,&
&  0.21590E+00_JPRB, 0.21526E+00_JPRB, 0.21422E+00_JPRB, 0.21355E+00_JPRB, 0.21328E+00_JPRB, 0.21379E+00_JPRB, 0.21447E+00_JPRB ,&
&  0.21528E+00_JPRB, 0.21689E+00_JPRB, 0.21819E+00_JPRB, 0.21995E+00_JPRB, 0.22188E+00_JPRB, 0.22334E+00_JPRB, 0.22551E+00_JPRB ,&
&  0.22762E+00_JPRB, 0.22982E+00_JPRB, 0.23368E+00_JPRB, 0.23784E+00_JPRB, 0.24331E+00_JPRB, 0.24864E+00_JPRB, 0.25062E+00_JPRB ,&
&  0.25261E+00_JPRB, 0.25812E+00_JPRB, 0.26088E+00_JPRB, 0.27108E+00_JPRB, 0.27275E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.26216E+00_JPRB, 0.26404E+00_JPRB, 0.26587E+00_JPRB, 0.26763E+00_JPRB, 0.26983E+00_JPRB, 0.27078E+00_JPRB, 0.27089E+00_JPRB ,&
&  0.26849E+00_JPRB, 0.26816E+00_JPRB, 0.26670E+00_JPRB, 0.26466E+00_JPRB, 0.26301E+00_JPRB, 0.25917E+00_JPRB, 0.25598E+00_JPRB ,&
&  0.25287E+00_JPRB, 0.24990E+00_JPRB, 0.24740E+00_JPRB, 0.24388E+00_JPRB, 0.24058E+00_JPRB, 0.23722E+00_JPRB, 0.23660E+00_JPRB ,&
&  0.23649E+00_JPRB, 0.23734E+00_JPRB, 0.23955E+00_JPRB, 0.24109E+00_JPRB, 0.24336E+00_JPRB, 0.24573E+00_JPRB, 0.24926E+00_JPRB ,&
&  0.25241E+00_JPRB, 0.25465E+00_JPRB, 0.25480E+00_JPRB, 0.25337E+00_JPRB, 0.25248E+00_JPRB, 0.25245E+00_JPRB, 0.25230E+00_JPRB ,&
&  0.25182E+00_JPRB, 0.25186E+00_JPRB, 0.25155E+00_JPRB, 0.25137E+00_JPRB, 0.25006E+00_JPRB, 0.24739E+00_JPRB, 0.24458E+00_JPRB ,&
&  0.24193E+00_JPRB, 0.23955E+00_JPRB, 0.23719E+00_JPRB, 0.23510E+00_JPRB, 0.23389E+00_JPRB, 0.23261E+00_JPRB, 0.23092E+00_JPRB ,&
&  0.22997E+00_JPRB, 0.22895E+00_JPRB, 0.22783E+00_JPRB, 0.22703E+00_JPRB, 0.22617E+00_JPRB, 0.22558E+00_JPRB, 0.22489E+00_JPRB ,&
&  0.22442E+00_JPRB, 0.22358E+00_JPRB, 0.22287E+00_JPRB, 0.22207E+00_JPRB, 0.22173E+00_JPRB, 0.22076E+00_JPRB, 0.21983E+00_JPRB ,&
&  0.21865E+00_JPRB, 0.21792E+00_JPRB, 0.21654E+00_JPRB, 0.21571E+00_JPRB, 0.21473E+00_JPRB, 0.21381E+00_JPRB, 0.21314E+00_JPRB ,&
&  0.21307E+00_JPRB, 0.21289E+00_JPRB, 0.21210E+00_JPRB, 0.21181E+00_JPRB, 0.21170E+00_JPRB, 0.21213E+00_JPRB, 0.21371E+00_JPRB ,&
&  0.21472E+00_JPRB, 0.21581E+00_JPRB, 0.21685E+00_JPRB, 0.21761E+00_JPRB, 0.21899E+00_JPRB, 0.22030E+00_JPRB, 0.22164E+00_JPRB ,&
&  0.22325E+00_JPRB, 0.22459E+00_JPRB, 0.22616E+00_JPRB, 0.22796E+00_JPRB, 0.23274E+00_JPRB, 0.23558E+00_JPRB, 0.23837E+00_JPRB ,&
&  0.23992E+00_JPRB, 0.23697E+00_JPRB, 0.25233E+00_JPRB, 0.25522E+00_JPRB, 0.25748E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.23668E+00_JPRB, 0.23972E+00_JPRB, 0.24130E+00_JPRB, 0.24224E+00_JPRB, 0.23293E+00_JPRB, 0.24458E+00_JPRB, 0.24108E+00_JPRB ,&
&  0.24067E+00_JPRB, 0.24073E+00_JPRB, 0.24334E+00_JPRB, 0.24397E+00_JPRB, 0.24373E+00_JPRB, 0.24314E+00_JPRB, 0.24219E+00_JPRB ,&
&  0.24123E+00_JPRB, 0.23988E+00_JPRB, 0.23915E+00_JPRB, 0.23830E+00_JPRB, 0.23764E+00_JPRB, 0.23694E+00_JPRB, 0.23645E+00_JPRB ,&
&  0.23659E+00_JPRB, 0.23764E+00_JPRB, 0.23945E+00_JPRB, 0.23862E+00_JPRB, 0.23846E+00_JPRB, 0.23875E+00_JPRB, 0.23933E+00_JPRB ,&
&  0.23949E+00_JPRB, 0.23945E+00_JPRB, 0.23901E+00_JPRB, 0.23765E+00_JPRB, 0.23791E+00_JPRB, 0.23826E+00_JPRB, 0.23868E+00_JPRB ,&
&  0.23886E+00_JPRB, 0.23875E+00_JPRB, 0.23790E+00_JPRB, 0.23709E+00_JPRB, 0.23626E+00_JPRB, 0.23521E+00_JPRB, 0.23387E+00_JPRB ,&
&  0.23278E+00_JPRB, 0.23194E+00_JPRB, 0.23087E+00_JPRB, 0.22980E+00_JPRB, 0.22887E+00_JPRB, 0.22796E+00_JPRB, 0.22707E+00_JPRB ,&
&  0.22651E+00_JPRB, 0.22610E+00_JPRB, 0.22564E+00_JPRB, 0.22497E+00_JPRB, 0.22424E+00_JPRB, 0.22354E+00_JPRB, 0.22292E+00_JPRB ,&
&  0.22253E+00_JPRB, 0.22183E+00_JPRB, 0.22114E+00_JPRB, 0.22037E+00_JPRB, 0.21979E+00_JPRB, 0.21881E+00_JPRB, 0.21795E+00_JPRB ,&
&  0.21710E+00_JPRB, 0.21632E+00_JPRB, 0.21572E+00_JPRB, 0.21526E+00_JPRB, 0.21484E+00_JPRB, 0.21452E+00_JPRB, 0.21426E+00_JPRB ,&
&  0.21405E+00_JPRB, 0.21385E+00_JPRB, 0.21376E+00_JPRB, 0.21358E+00_JPRB, 0.21336E+00_JPRB, 0.21325E+00_JPRB, 0.21354E+00_JPRB ,&
&  0.21376E+00_JPRB, 0.21389E+00_JPRB, 0.21394E+00_JPRB, 0.21347E+00_JPRB, 0.21393E+00_JPRB, 0.21432E+00_JPRB, 0.21479E+00_JPRB ,&
&  0.21519E+00_JPRB, 0.21611E+00_JPRB, 0.21723E+00_JPRB, 0.21863E+00_JPRB, 0.22139E+00_JPRB, 0.22374E+00_JPRB, 0.22628E+00_JPRB ,&
&  0.22913E+00_JPRB, 0.23332E+00_JPRB, 0.23486E+00_JPRB, 0.23577E+00_JPRB, 0.23634E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.22508E+00_JPRB, 0.22491E+00_JPRB, 0.22456E+00_JPRB, 0.22344E+00_JPRB, 0.22031E+00_JPRB, 0.21968E+00_JPRB, 0.21958E+00_JPRB ,&
&  0.21975E+00_JPRB, 0.21985E+00_JPRB, 0.22098E+00_JPRB, 0.22219E+00_JPRB, 0.22342E+00_JPRB, 0.22436E+00_JPRB, 0.22520E+00_JPRB ,&
&  0.22566E+00_JPRB, 0.22613E+00_JPRB, 0.22467E+00_JPRB, 0.22395E+00_JPRB, 0.22370E+00_JPRB, 0.22367E+00_JPRB, 0.22365E+00_JPRB ,&
&  0.22355E+00_JPRB, 0.22352E+00_JPRB, 0.22337E+00_JPRB, 0.22364E+00_JPRB, 0.22405E+00_JPRB, 0.22438E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22508E+00_JPRB, 0.22535E+00_JPRB, 0.22553E+00_JPRB, 0.22568E+00_JPRB, 0.22614E+00_JPRB, 0.22641E+00_JPRB, 0.22639E+00_JPRB ,&
&  0.22625E+00_JPRB, 0.22589E+00_JPRB, 0.22548E+00_JPRB, 0.22504E+00_JPRB, 0.22474E+00_JPRB, 0.22442E+00_JPRB, 0.22411E+00_JPRB ,&
&  0.22379E+00_JPRB, 0.22346E+00_JPRB, 0.22310E+00_JPRB, 0.22268E+00_JPRB, 0.22228E+00_JPRB, 0.22199E+00_JPRB, 0.22167E+00_JPRB ,&
&  0.22140E+00_JPRB, 0.22115E+00_JPRB, 0.22086E+00_JPRB, 0.22049E+00_JPRB, 0.22009E+00_JPRB, 0.21968E+00_JPRB, 0.21923E+00_JPRB ,&
&  0.21866E+00_JPRB, 0.21825E+00_JPRB, 0.21785E+00_JPRB, 0.21749E+00_JPRB, 0.21719E+00_JPRB, 0.21686E+00_JPRB, 0.21658E+00_JPRB ,&
&  0.21628E+00_JPRB, 0.21600E+00_JPRB, 0.21565E+00_JPRB, 0.21533E+00_JPRB, 0.21503E+00_JPRB, 0.21477E+00_JPRB, 0.21453E+00_JPRB ,&
&  0.21431E+00_JPRB, 0.21409E+00_JPRB, 0.21388E+00_JPRB, 0.21386E+00_JPRB, 0.21379E+00_JPRB, 0.21379E+00_JPRB, 0.21379E+00_JPRB ,&
&  0.21379E+00_JPRB, 0.21378E+00_JPRB, 0.21376E+00_JPRB, 0.21359E+00_JPRB, 0.21375E+00_JPRB, 0.21399E+00_JPRB, 0.21438E+00_JPRB ,&
&  0.21482E+00_JPRB, 0.21542E+00_JPRB, 0.21605E+00_JPRB, 0.21672E+00_JPRB, 0.21752E+00_JPRB, 0.21819E+00_JPRB, 0.21893E+00_JPRB ,&
&  0.21976E+00_JPRB, 0.22062E+00_JPRB, 0.22122E+00_JPRB, 0.22194E+00_JPRB, 0.22304E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB, 0.21765E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB, 0.95015E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.96364E+00_JPRB, 0.96215E+00_JPRB, 0.96087E+00_JPRB, 0.96015E+00_JPRB, 0.95974E+00_JPRB, 0.95935E+00_JPRB, 0.95772E+00_JPRB ,&
&  0.95567E+00_JPRB, 0.95310E+00_JPRB, 0.95045E+00_JPRB, 0.94725E+00_JPRB, 0.94325E+00_JPRB, 0.93846E+00_JPRB, 0.93413E+00_JPRB ,&
&  0.93009E+00_JPRB, 0.92675E+00_JPRB, 0.92324E+00_JPRB, 0.92080E+00_JPRB, 0.91812E+00_JPRB, 0.91538E+00_JPRB, 0.91251E+00_JPRB ,&
&  0.91023E+00_JPRB, 0.90808E+00_JPRB, 0.90601E+00_JPRB, 0.90421E+00_JPRB, 0.90349E+00_JPRB, 0.90252E+00_JPRB, 0.90147E+00_JPRB ,&
&  0.90059E+00_JPRB, 0.90012E+00_JPRB, 0.89978E+00_JPRB, 0.89941E+00_JPRB, 0.89921E+00_JPRB, 0.89957E+00_JPRB, 0.89992E+00_JPRB ,&
&  0.90052E+00_JPRB, 0.90194E+00_JPRB, 0.90402E+00_JPRB, 0.90593E+00_JPRB, 0.90805E+00_JPRB, 0.91100E+00_JPRB, 0.91475E+00_JPRB ,&
&  0.91810E+00_JPRB, 0.92134E+00_JPRB, 0.92499E+00_JPRB, 0.92917E+00_JPRB, 0.93318E+00_JPRB, 0.93745E+00_JPRB, 0.94151E+00_JPRB ,&
&  0.94557E+00_JPRB, 0.94890E+00_JPRB, 0.95216E+00_JPRB, 0.95381E+00_JPRB, 0.95616E+00_JPRB, 0.95812E+00_JPRB, 0.96036E+00_JPRB ,&
&  0.96095E+00_JPRB, 0.96127E+00_JPRB, 0.96177E+00_JPRB, 0.96266E+00_JPRB, 0.96322E+00_JPRB, 0.96394E+00_JPRB, 0.96479E+00_JPRB ,&
&  0.96606E+00_JPRB, 0.96677E+00_JPRB, 0.96787E+00_JPRB, 0.96863E+00_JPRB, 0.96960E+00_JPRB, 0.96980E+00_JPRB, 0.97044E+00_JPRB ,&
&  0.97107E+00_JPRB, 0.97210E+00_JPRB, 0.97215E+00_JPRB, 0.97239E+00_JPRB, 0.97245E+00_JPRB, 0.97266E+00_JPRB, 0.97283E+00_JPRB ,&
&  0.97312E+00_JPRB, 0.97325E+00_JPRB, 0.97332E+00_JPRB, 0.97272E+00_JPRB, 0.97169E+00_JPRB, 0.97072E+00_JPRB, 0.96994E+00_JPRB ,&
&  0.96948E+00_JPRB, 0.96899E+00_JPRB, 0.96895E+00_JPRB, 0.96925E+00_JPRB, 0.97012E+00_JPRB, 0.97009E+00_JPRB, 0.97004E+00_JPRB ,&
&  0.96981E+00_JPRB, 0.96945E+00_JPRB, 0.96828E+00_JPRB, 0.96684E+00_JPRB, 0.96523E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.96329E+00_JPRB, 0.96128E+00_JPRB, 0.96005E+00_JPRB, 0.95983E+00_JPRB, 0.96009E+00_JPRB, 0.96000E+00_JPRB, 0.95952E+00_JPRB ,&
&  0.95706E+00_JPRB, 0.95140E+00_JPRB, 0.94481E+00_JPRB, 0.93754E+00_JPRB, 0.92865E+00_JPRB, 0.92047E+00_JPRB, 0.91518E+00_JPRB ,&
&  0.90974E+00_JPRB, 0.90426E+00_JPRB, 0.89890E+00_JPRB, 0.89456E+00_JPRB, 0.88962E+00_JPRB, 0.88436E+00_JPRB, 0.87929E+00_JPRB ,&
&  0.87540E+00_JPRB, 0.87101E+00_JPRB, 0.86609E+00_JPRB, 0.85974E+00_JPRB, 0.85560E+00_JPRB, 0.85118E+00_JPRB, 0.84720E+00_JPRB ,&
&  0.84189E+00_JPRB, 0.83941E+00_JPRB, 0.83825E+00_JPRB, 0.83854E+00_JPRB, 0.84344E+00_JPRB, 0.84709E+00_JPRB, 0.84787E+00_JPRB ,&
&  0.84691E+00_JPRB, 0.84169E+00_JPRB, 0.84224E+00_JPRB, 0.84365E+00_JPRB, 0.84670E+00_JPRB, 0.85315E+00_JPRB, 0.86150E+00_JPRB ,&
&  0.87359E+00_JPRB, 0.88831E+00_JPRB, 0.90396E+00_JPRB, 0.91635E+00_JPRB, 0.92848E+00_JPRB, 0.93936E+00_JPRB, 0.94735E+00_JPRB ,&
&  0.95784E+00_JPRB, 0.96655E+00_JPRB, 0.97549E+00_JPRB, 0.98354E+00_JPRB, 0.99301E+00_JPRB, 0.10004E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.10064E+01_JPRB, 0.10056E+01_JPRB, 0.10051E+01_JPRB, 0.10028E+01_JPRB, 0.10015E+01_JPRB, 0.10013E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.10019E+01_JPRB, 0.10037E+01_JPRB, 0.10058E+01_JPRB, 0.10081E+01_JPRB, 0.10078E+01_JPRB, 0.10083E+01_JPRB ,&
&  0.10082E+01_JPRB, 0.10084E+01_JPRB, 0.10065E+01_JPRB, 0.10029E+01_JPRB, 0.99990E+00_JPRB, 0.99826E+00_JPRB, 0.99569E+00_JPRB ,&
&  0.99313E+00_JPRB, 0.99111E+00_JPRB, 0.99004E+00_JPRB, 0.98791E+00_JPRB, 0.98482E+00_JPRB, 0.98199E+00_JPRB, 0.98041E+00_JPRB ,&
&  0.97878E+00_JPRB, 0.97667E+00_JPRB, 0.97596E+00_JPRB, 0.97553E+00_JPRB, 0.97782E+00_JPRB, 0.97784E+00_JPRB, 0.97763E+00_JPRB ,&
&  0.97625E+00_JPRB, 0.97559E+00_JPRB, 0.97346E+00_JPRB, 0.97015E+00_JPRB, 0.96609E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.97366E+00_JPRB, 0.97403E+00_JPRB, 0.97703E+00_JPRB, 0.97708E+00_JPRB, 0.97580E+00_JPRB, 0.97203E+00_JPRB, 0.96957E+00_JPRB ,&
&  0.96392E+00_JPRB, 0.95433E+00_JPRB, 0.93965E+00_JPRB, 0.92708E+00_JPRB, 0.91477E+00_JPRB, 0.90650E+00_JPRB, 0.89905E+00_JPRB ,&
&  0.89154E+00_JPRB, 0.88538E+00_JPRB, 0.87864E+00_JPRB, 0.87041E+00_JPRB, 0.86009E+00_JPRB, 0.84825E+00_JPRB, 0.83261E+00_JPRB ,&
&  0.82150E+00_JPRB, 0.81092E+00_JPRB, 0.80225E+00_JPRB, 0.79682E+00_JPRB, 0.79642E+00_JPRB, 0.76978E+00_JPRB, 0.76611E+00_JPRB ,&
&  0.78141E+00_JPRB, 0.76645E+00_JPRB, 0.75126E+00_JPRB, 0.72630E+00_JPRB, 0.71678E+00_JPRB, 0.72934E+00_JPRB, 0.73938E+00_JPRB ,&
&  0.73070E+00_JPRB, 0.77265E+00_JPRB, 0.77698E+00_JPRB, 0.77855E+00_JPRB, 0.77527E+00_JPRB, 0.76692E+00_JPRB, 0.77904E+00_JPRB ,&
&  0.80986E+00_JPRB, 0.82281E+00_JPRB, 0.85068E+00_JPRB, 0.88226E+00_JPRB, 0.90561E+00_JPRB, 0.92944E+00_JPRB, 0.95335E+00_JPRB ,&
&  0.97689E+00_JPRB, 0.99392E+00_JPRB, 0.10078E+01_JPRB, 0.10205E+01_JPRB, 0.10400E+01_JPRB, 0.10588E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10932E+01_JPRB, 0.11010E+01_JPRB, 0.11087E+01_JPRB, 0.11196E+01_JPRB, 0.11290E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.11520E+01_JPRB, 0.11555E+01_JPRB, 0.11566E+01_JPRB, 0.11561E+01_JPRB, 0.11515E+01_JPRB, 0.11503E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.11335E+01_JPRB, 0.11215E+01_JPRB, 0.11060E+01_JPRB, 0.10938E+01_JPRB, 0.10841E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.10638E+01_JPRB, 0.10537E+01_JPRB, 0.10447E+01_JPRB, 0.10337E+01_JPRB, 0.10233E+01_JPRB, 0.10127E+01_JPRB, 0.10054E+01_JPRB ,&
&  0.99995E+00_JPRB, 0.99558E+00_JPRB, 0.99357E+00_JPRB, 0.99089E+00_JPRB, 0.99090E+00_JPRB, 0.98973E+00_JPRB, 0.99126E+00_JPRB ,&
&  0.99075E+00_JPRB, 0.98653E+00_JPRB, 0.98004E+00_JPRB, 0.97581E+00_JPRB, 0.97284E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.97434E+00_JPRB, 0.97824E+00_JPRB, 0.98734E+00_JPRB, 0.98682E+00_JPRB, 0.98449E+00_JPRB, 0.97554E+00_JPRB, 0.97178E+00_JPRB ,&
&  0.96205E+00_JPRB, 0.95152E+00_JPRB, 0.92909E+00_JPRB, 0.91433E+00_JPRB, 0.90098E+00_JPRB, 0.89256E+00_JPRB, 0.88601E+00_JPRB ,&
&  0.88059E+00_JPRB, 0.87362E+00_JPRB, 0.86495E+00_JPRB, 0.85219E+00_JPRB, 0.83610E+00_JPRB, 0.81782E+00_JPRB, 0.79195E+00_JPRB ,&
&  0.77057E+00_JPRB, 0.75551E+00_JPRB, 0.73609E+00_JPRB, 0.74472E+00_JPRB, 0.73513E+00_JPRB, 0.72769E+00_JPRB, 0.72256E+00_JPRB ,&
&  0.70760E+00_JPRB, 0.69087E+00_JPRB, 0.66605E+00_JPRB, 0.65400E+00_JPRB, 0.64615E+00_JPRB, 0.63627E+00_JPRB, 0.65481E+00_JPRB ,&
&  0.66907E+00_JPRB, 0.70622E+00_JPRB, 0.72203E+00_JPRB, 0.72603E+00_JPRB, 0.73143E+00_JPRB, 0.72902E+00_JPRB, 0.73836E+00_JPRB ,&
&  0.75018E+00_JPRB, 0.76560E+00_JPRB, 0.81777E+00_JPRB, 0.85296E+00_JPRB, 0.87914E+00_JPRB, 0.90307E+00_JPRB, 0.92109E+00_JPRB ,&
&  0.94385E+00_JPRB, 0.95810E+00_JPRB, 0.97262E+00_JPRB, 0.99061E+00_JPRB, 0.10189E+01_JPRB, 0.10517E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.11225E+01_JPRB, 0.11431E+01_JPRB, 0.11669E+01_JPRB, 0.11991E+01_JPRB, 0.12351E+01_JPRB, 0.12732E+01_JPRB ,&
&  0.13122E+01_JPRB, 0.13505E+01_JPRB, 0.13818E+01_JPRB, 0.14094E+01_JPRB, 0.14237E+01_JPRB, 0.14258E+01_JPRB, 0.14221E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.13755E+01_JPRB, 0.13411E+01_JPRB, 0.12897E+01_JPRB, 0.12504E+01_JPRB, 0.12292E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.11305E+01_JPRB, 0.11035E+01_JPRB, 0.10747E+01_JPRB, 0.10490E+01_JPRB, 0.10251E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.99653E+00_JPRB, 0.98880E+00_JPRB, 0.98431E+00_JPRB, 0.98146E+00_JPRB, 0.98205E+00_JPRB, 0.98110E+00_JPRB, 0.98514E+00_JPRB ,&
&  0.98563E+00_JPRB, 0.98242E+00_JPRB, 0.97480E+00_JPRB, 0.97168E+00_JPRB, 0.97029E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.97906E+00_JPRB, 0.98083E+00_JPRB, 0.97945E+00_JPRB, 0.96756E+00_JPRB, 0.96124E+00_JPRB, 0.95441E+00_JPRB, 0.95519E+00_JPRB ,&
&  0.94743E+00_JPRB, 0.93930E+00_JPRB, 0.91811E+00_JPRB, 0.90428E+00_JPRB, 0.88898E+00_JPRB, 0.87996E+00_JPRB, 0.87152E+00_JPRB ,&
&  0.86506E+00_JPRB, 0.85704E+00_JPRB, 0.85110E+00_JPRB, 0.83827E+00_JPRB, 0.82567E+00_JPRB, 0.80635E+00_JPRB, 0.78499E+00_JPRB ,&
&  0.76261E+00_JPRB, 0.73984E+00_JPRB, 0.72209E+00_JPRB, 0.72528E+00_JPRB, 0.71580E+00_JPRB, 0.70884E+00_JPRB, 0.69967E+00_JPRB ,&
&  0.69021E+00_JPRB, 0.64879E+00_JPRB, 0.66472E+00_JPRB, 0.64881E+00_JPRB, 0.63158E+00_JPRB, 0.60933E+00_JPRB, 0.59056E+00_JPRB ,&
&  0.59523E+00_JPRB, 0.59890E+00_JPRB, 0.61432E+00_JPRB, 0.63830E+00_JPRB, 0.64945E+00_JPRB, 0.66923E+00_JPRB, 0.68189E+00_JPRB ,&
&  0.67454E+00_JPRB, 0.76478E+00_JPRB, 0.81203E+00_JPRB, 0.84048E+00_JPRB, 0.86849E+00_JPRB, 0.89039E+00_JPRB, 0.90414E+00_JPRB ,&
&  0.93028E+00_JPRB, 0.95820E+00_JPRB, 0.99104E+00_JPRB, 0.10253E+01_JPRB, 0.10749E+01_JPRB, 0.11099E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.11841E+01_JPRB, 0.12069E+01_JPRB, 0.12419E+01_JPRB, 0.12865E+01_JPRB, 0.13360E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.14513E+01_JPRB, 0.15088E+01_JPRB, 0.15693E+01_JPRB, 0.16220E+01_JPRB, 0.16637E+01_JPRB, 0.16892E+01_JPRB, 0.17139E+01_JPRB ,&
&  0.16938E+01_JPRB, 0.16647E+01_JPRB, 0.16176E+01_JPRB, 0.15524E+01_JPRB, 0.14716E+01_JPRB, 0.13829E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.12475E+01_JPRB, 0.11964E+01_JPRB, 0.11504E+01_JPRB, 0.11062E+01_JPRB, 0.10682E+01_JPRB, 0.10373E+01_JPRB, 0.10199E+01_JPRB ,&
&  0.10044E+01_JPRB, 0.99484E+00_JPRB, 0.98851E+00_JPRB, 0.98321E+00_JPRB, 0.98044E+00_JPRB, 0.97415E+00_JPRB, 0.97214E+00_JPRB ,&
&  0.97167E+00_JPRB, 0.97396E+00_JPRB, 0.97201E+00_JPRB, 0.97257E+00_JPRB, 0.97475E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.99042E+00_JPRB, 0.99292E+00_JPRB, 0.98565E+00_JPRB, 0.97686E+00_JPRB, 0.97215E+00_JPRB, 0.95973E+00_JPRB, 0.95506E+00_JPRB ,&
&  0.95221E+00_JPRB, 0.94936E+00_JPRB, 0.93735E+00_JPRB, 0.92310E+00_JPRB, 0.90390E+00_JPRB, 0.88959E+00_JPRB, 0.87381E+00_JPRB ,&
&  0.86026E+00_JPRB, 0.84417E+00_JPRB, 0.82600E+00_JPRB, 0.80899E+00_JPRB, 0.79588E+00_JPRB, 0.77948E+00_JPRB, 0.76746E+00_JPRB ,&
&  0.75397E+00_JPRB, 0.73686E+00_JPRB, 0.72469E+00_JPRB, 0.72458E+00_JPRB, 0.71385E+00_JPRB, 0.70245E+00_JPRB, 0.70298E+00_JPRB ,&
&  0.70723E+00_JPRB, 0.70023E+00_JPRB, 0.69137E+00_JPRB, 0.67429E+00_JPRB, 0.63943E+00_JPRB, 0.57378E+00_JPRB, 0.59005E+00_JPRB ,&
&  0.58414E+00_JPRB, 0.57245E+00_JPRB, 0.57718E+00_JPRB, 0.58937E+00_JPRB, 0.60216E+00_JPRB, 0.63717E+00_JPRB, 0.64056E+00_JPRB ,&
&  0.62623E+00_JPRB, 0.75525E+00_JPRB, 0.78326E+00_JPRB, 0.81702E+00_JPRB, 0.84860E+00_JPRB, 0.87139E+00_JPRB, 0.89257E+00_JPRB ,&
&  0.91488E+00_JPRB, 0.94671E+00_JPRB, 0.97800E+00_JPRB, 0.10244E+01_JPRB, 0.10737E+01_JPRB, 0.11349E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.12435E+01_JPRB, 0.12731E+01_JPRB, 0.12973E+01_JPRB, 0.13257E+01_JPRB, 0.13751E+01_JPRB, 0.14396E+01_JPRB ,&
&  0.15289E+01_JPRB, 0.15839E+01_JPRB, 0.17012E+01_JPRB, 0.17922E+01_JPRB, 0.18397E+01_JPRB, 0.19042E+01_JPRB, 0.19695E+01_JPRB ,&
&  0.19499E+01_JPRB, 0.19403E+01_JPRB, 0.19045E+01_JPRB, 0.17672E+01_JPRB, 0.16181E+01_JPRB, 0.15161E+01_JPRB, 0.14247E+01_JPRB ,&
&  0.13532E+01_JPRB, 0.12816E+01_JPRB, 0.12092E+01_JPRB, 0.11537E+01_JPRB, 0.11099E+01_JPRB, 0.10785E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.10432E+01_JPRB, 0.10277E+01_JPRB, 0.10122E+01_JPRB, 0.10053E+01_JPRB, 0.99031E+00_JPRB, 0.97758E+00_JPRB, 0.96391E+00_JPRB ,&
&  0.96043E+00_JPRB, 0.95997E+00_JPRB, 0.96714E+00_JPRB, 0.97413E+00_JPRB, 0.97839E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.10080E+01_JPRB, 0.10264E+01_JPRB, 0.10302E+01_JPRB, 0.10301E+01_JPRB, 0.10290E+01_JPRB, 0.10087E+01_JPRB, 0.99127E+00_JPRB ,&
&  0.97940E+00_JPRB, 0.97058E+00_JPRB, 0.95971E+00_JPRB, 0.94677E+00_JPRB, 0.93090E+00_JPRB, 0.91767E+00_JPRB, 0.90528E+00_JPRB ,&
&  0.89380E+00_JPRB, 0.84443E+00_JPRB, 0.81987E+00_JPRB, 0.79983E+00_JPRB, 0.78011E+00_JPRB, 0.76960E+00_JPRB, 0.76424E+00_JPRB ,&
&  0.75407E+00_JPRB, 0.74821E+00_JPRB, 0.74389E+00_JPRB, 0.73523E+00_JPRB, 0.72344E+00_JPRB, 0.70875E+00_JPRB, 0.70279E+00_JPRB ,&
&  0.70727E+00_JPRB, 0.69245E+00_JPRB, 0.68060E+00_JPRB, 0.66095E+00_JPRB, 0.62230E+00_JPRB, 0.60254E+00_JPRB, 0.60276E+00_JPRB ,&
&  0.59677E+00_JPRB, 0.56883E+00_JPRB, 0.57094E+00_JPRB, 0.57491E+00_JPRB, 0.57730E+00_JPRB, 0.59325E+00_JPRB, 0.62400E+00_JPRB ,&
&  0.65262E+00_JPRB, 0.75131E+00_JPRB, 0.79294E+00_JPRB, 0.83954E+00_JPRB, 0.87418E+00_JPRB, 0.90316E+00_JPRB, 0.92841E+00_JPRB ,&
&  0.95480E+00_JPRB, 0.99276E+00_JPRB, 0.10347E+01_JPRB, 0.10902E+01_JPRB, 0.11494E+01_JPRB, 0.11983E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.13153E+01_JPRB, 0.13034E+01_JPRB, 0.13283E+01_JPRB, 0.13357E+01_JPRB, 0.13604E+01_JPRB, 0.14247E+01_JPRB, 0.14493E+01_JPRB ,&
&  0.15245E+01_JPRB, 0.16414E+01_JPRB, 0.18307E+01_JPRB, 0.20175E+01_JPRB, 0.21052E+01_JPRB, 0.21987E+01_JPRB, 0.22323E+01_JPRB ,&
&  0.21300E+01_JPRB, 0.20715E+01_JPRB, 0.21256E+01_JPRB, 0.19606E+01_JPRB, 0.17807E+01_JPRB, 0.16288E+01_JPRB, 0.15243E+01_JPRB ,&
&  0.14218E+01_JPRB, 0.13218E+01_JPRB, 0.12586E+01_JPRB, 0.12089E+01_JPRB, 0.11583E+01_JPRB, 0.11142E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.10938E+01_JPRB, 0.10790E+01_JPRB, 0.10615E+01_JPRB, 0.10282E+01_JPRB, 0.99406E+00_JPRB, 0.96525E+00_JPRB ,&
&  0.95743E+00_JPRB, 0.95455E+00_JPRB, 0.96377E+00_JPRB, 0.97207E+00_JPRB, 0.98672E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.10191E+01_JPRB, 0.10348E+01_JPRB, 0.10377E+01_JPRB, 0.10425E+01_JPRB, 0.10513E+01_JPRB, 0.99819E+00_JPRB, 0.95882E+00_JPRB ,&
&  0.93607E+00_JPRB, 0.94102E+00_JPRB, 0.94251E+00_JPRB, 0.93739E+00_JPRB, 0.92541E+00_JPRB, 0.91162E+00_JPRB, 0.90046E+00_JPRB ,&
&  0.89060E+00_JPRB, 0.86696E+00_JPRB, 0.82536E+00_JPRB, 0.76932E+00_JPRB, 0.75333E+00_JPRB, 0.74796E+00_JPRB, 0.74977E+00_JPRB ,&
&  0.74907E+00_JPRB, 0.74398E+00_JPRB, 0.74594E+00_JPRB, 0.73272E+00_JPRB, 0.72373E+00_JPRB, 0.69815E+00_JPRB, 0.67838E+00_JPRB ,&
&  0.67921E+00_JPRB, 0.67377E+00_JPRB, 0.68106E+00_JPRB, 0.66916E+00_JPRB, 0.65485E+00_JPRB, 0.64409E+00_JPRB, 0.58724E+00_JPRB ,&
&  0.57801E+00_JPRB, 0.56319E+00_JPRB, 0.55837E+00_JPRB, 0.55850E+00_JPRB, 0.58081E+00_JPRB, 0.59984E+00_JPRB, 0.62419E+00_JPRB ,&
&  0.64078E+00_JPRB, 0.75585E+00_JPRB, 0.79349E+00_JPRB, 0.82410E+00_JPRB, 0.85303E+00_JPRB, 0.89058E+00_JPRB, 0.94177E+00_JPRB ,&
&  0.99314E+00_JPRB, 0.10446E+01_JPRB, 0.10864E+01_JPRB, 0.11474E+01_JPRB, 0.12081E+01_JPRB, 0.12003E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.16642E+01_JPRB, 0.15741E+01_JPRB, 0.14961E+01_JPRB, 0.14603E+01_JPRB, 0.14729E+01_JPRB, 0.15006E+01_JPRB, 0.15499E+01_JPRB ,&
&  0.16821E+01_JPRB, 0.19001E+01_JPRB, 0.22253E+01_JPRB, 0.24519E+01_JPRB, 0.24617E+01_JPRB, 0.24916E+01_JPRB, 0.24572E+01_JPRB ,&
&  0.22928E+01_JPRB, 0.21849E+01_JPRB, 0.22808E+01_JPRB, 0.18948E+01_JPRB, 0.16235E+01_JPRB, 0.14797E+01_JPRB, 0.14174E+01_JPRB ,&
&  0.13532E+01_JPRB, 0.12863E+01_JPRB, 0.12483E+01_JPRB, 0.12000E+01_JPRB, 0.11372E+01_JPRB, 0.11006E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.10805E+01_JPRB, 0.10735E+01_JPRB, 0.10611E+01_JPRB, 0.10368E+01_JPRB, 0.10097E+01_JPRB, 0.99908E+00_JPRB ,&
&  0.99597E+00_JPRB, 0.99008E+00_JPRB, 0.98553E+00_JPRB, 0.98651E+00_JPRB, 0.99710E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.10198E+01_JPRB, 0.10329E+01_JPRB, 0.10346E+01_JPRB, 0.10215E+01_JPRB, 0.10035E+01_JPRB, 0.92913E+00_JPRB, 0.84830E+00_JPRB ,&
&  0.78934E+00_JPRB, 0.79231E+00_JPRB, 0.81775E+00_JPRB, 0.83876E+00_JPRB, 0.83910E+00_JPRB, 0.82148E+00_JPRB, 0.81287E+00_JPRB ,&
&  0.81899E+00_JPRB, 0.81737E+00_JPRB, 0.78952E+00_JPRB, 0.76984E+00_JPRB, 0.73720E+00_JPRB, 0.73028E+00_JPRB, 0.73203E+00_JPRB ,&
&  0.73487E+00_JPRB, 0.73269E+00_JPRB, 0.73304E+00_JPRB, 0.73075E+00_JPRB, 0.73059E+00_JPRB, 0.72240E+00_JPRB, 0.70641E+00_JPRB ,&
&  0.68515E+00_JPRB, 0.68141E+00_JPRB, 0.68514E+00_JPRB, 0.69210E+00_JPRB, 0.68604E+00_JPRB, 0.67461E+00_JPRB, 0.60237E+00_JPRB ,&
&  0.57472E+00_JPRB, 0.55600E+00_JPRB, 0.55806E+00_JPRB, 0.57311E+00_JPRB, 0.60854E+00_JPRB, 0.61325E+00_JPRB, 0.69837E+00_JPRB ,&
&  0.76094E+00_JPRB, 0.79891E+00_JPRB, 0.81462E+00_JPRB, 0.82843E+00_JPRB, 0.85823E+00_JPRB, 0.92275E+00_JPRB, 0.10053E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.11054E+01_JPRB, 0.11061E+01_JPRB, 0.98316E+00_JPRB, 0.99200E+00_JPRB, 0.10889E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.14370E+01_JPRB, 0.14470E+01_JPRB, 0.15186E+01_JPRB, 0.15333E+01_JPRB, 0.15445E+01_JPRB, 0.15524E+01_JPRB ,&
&  0.17390E+01_JPRB, 0.20159E+01_JPRB, 0.24432E+01_JPRB, 0.26645E+01_JPRB, 0.26032E+01_JPRB, 0.26253E+01_JPRB, 0.25234E+01_JPRB ,&
&  0.24322E+01_JPRB, 0.21774E+01_JPRB, 0.19528E+01_JPRB, 0.16849E+01_JPRB, 0.14720E+01_JPRB, 0.13504E+01_JPRB, 0.12801E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.11726E+01_JPRB, 0.11718E+01_JPRB, 0.11451E+01_JPRB, 0.10851E+01_JPRB, 0.10282E+01_JPRB, 0.98502E+00_JPRB ,&
&  0.95480E+00_JPRB, 0.93984E+00_JPRB, 0.93918E+00_JPRB, 0.91938E+00_JPRB, 0.92107E+00_JPRB, 0.93611E+00_JPRB, 0.94268E+00_JPRB ,&
&  0.94175E+00_JPRB, 0.94064E+00_JPRB, 0.95331E+00_JPRB, 0.97159E+00_JPRB, 0.99017E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.10658E+01_JPRB, 0.10377E+01_JPRB, 0.99921E+00_JPRB, 0.99204E+00_JPRB, 0.96533E+00_JPRB, 0.94427E+00_JPRB, 0.85880E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.72025E+00_JPRB, 0.74623E+00_JPRB, 0.75624E+00_JPRB, 0.73896E+00_JPRB, 0.73339E+00_JPRB, 0.73982E+00_JPRB ,&
&  0.74766E+00_JPRB, 0.77459E+00_JPRB, 0.77229E+00_JPRB, 0.75410E+00_JPRB, 0.74200E+00_JPRB, 0.73549E+00_JPRB, 0.73807E+00_JPRB ,&
&  0.73819E+00_JPRB, 0.73713E+00_JPRB, 0.72902E+00_JPRB, 0.71552E+00_JPRB, 0.71304E+00_JPRB, 0.71090E+00_JPRB, 0.70733E+00_JPRB ,&
&  0.70610E+00_JPRB, 0.70060E+00_JPRB, 0.70774E+00_JPRB, 0.71119E+00_JPRB, 0.70272E+00_JPRB, 0.67464E+00_JPRB, 0.60930E+00_JPRB ,&
&  0.57632E+00_JPRB, 0.56256E+00_JPRB, 0.59299E+00_JPRB, 0.66314E+00_JPRB, 0.71763E+00_JPRB, 0.76669E+00_JPRB, 0.80627E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.78888E+00_JPRB, 0.82924E+00_JPRB, 0.94596E+00_JPRB, 0.10016E+01_JPRB, 0.10732E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.12051E+01_JPRB, 0.12538E+01_JPRB, 0.12629E+01_JPRB, 0.11072E+01_JPRB, 0.11664E+01_JPRB, 0.14338E+01_JPRB, 0.13912E+01_JPRB ,&
&  0.14895E+01_JPRB, 0.15917E+01_JPRB, 0.16822E+01_JPRB, 0.16621E+01_JPRB, 0.15706E+01_JPRB, 0.15894E+01_JPRB, 0.17154E+01_JPRB ,&
&  0.19137E+01_JPRB, 0.22172E+01_JPRB, 0.27890E+01_JPRB, 0.31033E+01_JPRB, 0.30498E+01_JPRB, 0.29979E+01_JPRB, 0.29288E+01_JPRB ,&
&  0.28758E+01_JPRB, 0.25976E+01_JPRB, 0.22393E+01_JPRB, 0.19102E+01_JPRB, 0.16266E+01_JPRB, 0.14051E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.11963E+01_JPRB, 0.12038E+01_JPRB, 0.11821E+01_JPRB, 0.11112E+01_JPRB, 0.10247E+01_JPRB, 0.95298E+00_JPRB ,&
&  0.90748E+00_JPRB, 0.86584E+00_JPRB, 0.82388E+00_JPRB, 0.81105E+00_JPRB, 0.85695E+00_JPRB, 0.91804E+00_JPRB, 0.93987E+00_JPRB ,&
&  0.92430E+00_JPRB, 0.92037E+00_JPRB, 0.96588E+00_JPRB, 0.10133E+01_JPRB, 0.10311E+01_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.11147E+01_JPRB, 0.10732E+01_JPRB, 0.10364E+01_JPRB, 0.10035E+01_JPRB, 0.97349E+00_JPRB, 0.91862E+00_JPRB, 0.82504E+00_JPRB ,&
&  0.73423E+00_JPRB, 0.69886E+00_JPRB, 0.70717E+00_JPRB, 0.69054E+00_JPRB, 0.66652E+00_JPRB, 0.65245E+00_JPRB, 0.65554E+00_JPRB ,&
&  0.66876E+00_JPRB, 0.70332E+00_JPRB, 0.72850E+00_JPRB, 0.73616E+00_JPRB, 0.73299E+00_JPRB, 0.72815E+00_JPRB, 0.73566E+00_JPRB ,&
&  0.74674E+00_JPRB, 0.75818E+00_JPRB, 0.76311E+00_JPRB, 0.75561E+00_JPRB, 0.74614E+00_JPRB, 0.74188E+00_JPRB, 0.73903E+00_JPRB ,&
&  0.73991E+00_JPRB, 0.73773E+00_JPRB, 0.72307E+00_JPRB, 0.72251E+00_JPRB, 0.70125E+00_JPRB, 0.69661E+00_JPRB, 0.65852E+00_JPRB ,&
&  0.60908E+00_JPRB, 0.61775E+00_JPRB, 0.74003E+00_JPRB, 0.79102E+00_JPRB, 0.80807E+00_JPRB, 0.84648E+00_JPRB, 0.89993E+00_JPRB ,&
&  0.94809E+00_JPRB, 0.99208E+00_JPRB, 0.10341E+01_JPRB, 0.10824E+01_JPRB, 0.11348E+01_JPRB, 0.12211E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.14530E+01_JPRB, 0.12571E+01_JPRB, 0.12013E+01_JPRB, 0.13022E+01_JPRB, 0.16948E+01_JPRB, 0.18523E+01_JPRB, 0.18815E+01_JPRB ,&
&  0.19319E+01_JPRB, 0.18926E+01_JPRB, 0.18554E+01_JPRB, 0.18682E+01_JPRB, 0.19529E+01_JPRB, 0.21035E+01_JPRB, 0.23466E+01_JPRB ,&
&  0.25589E+01_JPRB, 0.29520E+01_JPRB, 0.35313E+01_JPRB, 0.36675E+01_JPRB, 0.36014E+01_JPRB, 0.35517E+01_JPRB, 0.34259E+01_JPRB ,&
&  0.32762E+01_JPRB, 0.29821E+01_JPRB, 0.25068E+01_JPRB, 0.21080E+01_JPRB, 0.17740E+01_JPRB, 0.15207E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.12233E+01_JPRB, 0.12077E+01_JPRB, 0.12119E+01_JPRB, 0.11970E+01_JPRB, 0.11535E+01_JPRB, 0.10830E+01_JPRB, 0.10014E+01_JPRB ,&
&  0.90795E+00_JPRB, 0.86052E+00_JPRB, 0.81337E+00_JPRB, 0.84168E+00_JPRB, 0.91312E+00_JPRB, 0.98363E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.10064E+01_JPRB, 0.10130E+01_JPRB, 0.10496E+01_JPRB, 0.10940E+01_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.11355E+01_JPRB, 0.10867E+01_JPRB, 0.10289E+01_JPRB, 0.96296E+00_JPRB, 0.90486E+00_JPRB, 0.85785E+00_JPRB, 0.80191E+00_JPRB ,&
&  0.83589E+00_JPRB, 0.75525E+00_JPRB, 0.82444E+00_JPRB, 0.71558E+00_JPRB, 0.68570E+00_JPRB, 0.67661E+00_JPRB, 0.65767E+00_JPRB ,&
&  0.64728E+00_JPRB, 0.65366E+00_JPRB, 0.66703E+00_JPRB, 0.68296E+00_JPRB, 0.68453E+00_JPRB, 0.68506E+00_JPRB, 0.69623E+00_JPRB ,&
&  0.71611E+00_JPRB, 0.74503E+00_JPRB, 0.77649E+00_JPRB, 0.79186E+00_JPRB, 0.79612E+00_JPRB, 0.79082E+00_JPRB, 0.79086E+00_JPRB ,&
&  0.75147E+00_JPRB, 0.73846E+00_JPRB, 0.73964E+00_JPRB, 0.73593E+00_JPRB, 0.73063E+00_JPRB, 0.71538E+00_JPRB, 0.72737E+00_JPRB ,&
&  0.71259E+00_JPRB, 0.69283E+00_JPRB, 0.80425E+00_JPRB, 0.82715E+00_JPRB, 0.86142E+00_JPRB, 0.90015E+00_JPRB, 0.94184E+00_JPRB ,&
&  0.98439E+00_JPRB, 0.10324E+01_JPRB, 0.10865E+01_JPRB, 0.11322E+01_JPRB, 0.12184E+01_JPRB, 0.13617E+01_JPRB, 0.14968E+01_JPRB ,&
&  0.16371E+01_JPRB, 0.13529E+01_JPRB, 0.14989E+01_JPRB, 0.16163E+01_JPRB, 0.19404E+01_JPRB, 0.22880E+01_JPRB, 0.23762E+01_JPRB ,&
&  0.26103E+01_JPRB, 0.25710E+01_JPRB, 0.25119E+01_JPRB, 0.24568E+01_JPRB, 0.26121E+01_JPRB, 0.28360E+01_JPRB, 0.30881E+01_JPRB ,&
&  0.33952E+01_JPRB, 0.37962E+01_JPRB, 0.44017E+01_JPRB, 0.43846E+01_JPRB, 0.42697E+01_JPRB, 0.41308E+01_JPRB, 0.38944E+01_JPRB ,&
&  0.35878E+01_JPRB, 0.32195E+01_JPRB, 0.27578E+01_JPRB, 0.22491E+01_JPRB, 0.19016E+01_JPRB, 0.16090E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.12117E+01_JPRB, 0.11339E+01_JPRB, 0.11383E+01_JPRB, 0.11272E+01_JPRB, 0.11142E+01_JPRB, 0.10631E+01_JPRB, 0.98208E+00_JPRB ,&
&  0.89609E+00_JPRB, 0.92270E+00_JPRB, 0.98391E+00_JPRB, 0.11070E+01_JPRB, 0.11581E+01_JPRB, 0.11623E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11396E+01_JPRB, 0.12074E+01_JPRB, 0.11869E+01_JPRB, 0.11688E+01_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.11821E+01_JPRB, 0.10990E+01_JPRB, 0.97602E+00_JPRB, 0.90528E+00_JPRB, 0.89355E+00_JPRB, 0.88024E+00_JPRB, 0.84541E+00_JPRB ,&
&  0.87058E+00_JPRB, 0.85395E+00_JPRB, 0.87751E+00_JPRB, 0.91174E+00_JPRB, 0.94181E+00_JPRB, 0.84869E+00_JPRB, 0.72022E+00_JPRB ,&
&  0.68152E+00_JPRB, 0.69412E+00_JPRB, 0.70804E+00_JPRB, 0.71141E+00_JPRB, 0.71984E+00_JPRB, 0.71763E+00_JPRB, 0.72034E+00_JPRB ,&
&  0.72706E+00_JPRB, 0.74158E+00_JPRB, 0.77982E+00_JPRB, 0.79488E+00_JPRB, 0.79779E+00_JPRB, 0.79912E+00_JPRB, 0.80754E+00_JPRB ,&
&  0.76854E+00_JPRB, 0.75238E+00_JPRB, 0.74121E+00_JPRB, 0.71508E+00_JPRB, 0.75215E+00_JPRB, 0.76547E+00_JPRB, 0.77471E+00_JPRB ,&
&  0.78615E+00_JPRB, 0.80869E+00_JPRB, 0.84276E+00_JPRB, 0.87800E+00_JPRB, 0.91414E+00_JPRB, 0.95156E+00_JPRB, 0.98767E+00_JPRB ,&
&  0.10229E+01_JPRB, 0.10649E+01_JPRB, 0.11130E+01_JPRB, 0.11848E+01_JPRB, 0.13669E+01_JPRB, 0.14572E+01_JPRB, 0.16556E+01_JPRB ,&
&  0.17571E+01_JPRB, 0.19373E+01_JPRB, 0.21635E+01_JPRB, 0.21191E+01_JPRB, 0.22850E+01_JPRB, 0.25319E+01_JPRB, 0.25050E+01_JPRB ,&
&  0.25665E+01_JPRB, 0.25461E+01_JPRB, 0.27031E+01_JPRB, 0.29378E+01_JPRB, 0.32612E+01_JPRB, 0.35364E+01_JPRB, 0.37738E+01_JPRB ,&
&  0.41585E+01_JPRB, 0.48704E+01_JPRB, 0.51281E+01_JPRB, 0.49262E+01_JPRB, 0.46967E+01_JPRB, 0.44645E+01_JPRB, 0.41136E+01_JPRB ,&
&  0.37475E+01_JPRB, 0.33289E+01_JPRB, 0.28435E+01_JPRB, 0.23475E+01_JPRB, 0.19098E+01_JPRB, 0.16705E+01_JPRB, 0.14285E+01_JPRB ,&
&  0.12036E+01_JPRB, 0.10599E+01_JPRB, 0.10769E+01_JPRB, 0.11076E+01_JPRB, 0.11022E+01_JPRB, 0.10286E+01_JPRB, 0.96276E+00_JPRB ,&
&  0.99713E+00_JPRB, 0.11269E+01_JPRB, 0.14778E+01_JPRB, 0.15944E+01_JPRB, 0.16066E+01_JPRB, 0.15176E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.12471E+01_JPRB, 0.12379E+01_JPRB, 0.12444E+01_JPRB, 0.12280E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.12427E+01_JPRB, 0.11140E+01_JPRB, 0.94472E+00_JPRB, 0.88800E+00_JPRB, 0.90335E+00_JPRB, 0.89794E+00_JPRB, 0.88930E+00_JPRB ,&
&  0.90033E+00_JPRB, 0.90598E+00_JPRB, 0.91539E+00_JPRB, 0.93160E+00_JPRB, 0.94550E+00_JPRB, 0.97903E+00_JPRB, 0.89497E+00_JPRB ,&
&  0.79860E+00_JPRB, 0.78343E+00_JPRB, 0.76050E+00_JPRB, 0.77214E+00_JPRB, 0.83939E+00_JPRB, 0.84695E+00_JPRB, 0.84793E+00_JPRB ,&
&  0.86201E+00_JPRB, 0.85026E+00_JPRB, 0.85778E+00_JPRB, 0.84333E+00_JPRB, 0.82235E+00_JPRB, 0.82371E+00_JPRB, 0.82679E+00_JPRB ,&
&  0.77282E+00_JPRB, 0.73394E+00_JPRB, 0.71075E+00_JPRB, 0.70043E+00_JPRB, 0.73572E+00_JPRB, 0.78771E+00_JPRB, 0.81575E+00_JPRB ,&
&  0.83190E+00_JPRB, 0.85389E+00_JPRB, 0.88487E+00_JPRB, 0.92403E+00_JPRB, 0.96591E+00_JPRB, 0.10047E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10588E+01_JPRB, 0.10915E+01_JPRB, 0.11377E+01_JPRB, 0.12344E+01_JPRB, 0.13390E+01_JPRB, 0.15390E+01_JPRB, 0.18630E+01_JPRB ,&
&  0.20513E+01_JPRB, 0.22181E+01_JPRB, 0.24660E+01_JPRB, 0.28252E+01_JPRB, 0.30638E+01_JPRB, 0.29923E+01_JPRB, 0.28990E+01_JPRB ,&
&  0.29356E+01_JPRB, 0.29614E+01_JPRB, 0.34225E+01_JPRB, 0.37580E+01_JPRB, 0.39836E+01_JPRB, 0.42688E+01_JPRB, 0.44505E+01_JPRB ,&
&  0.47946E+01_JPRB, 0.51589E+01_JPRB, 0.51973E+01_JPRB, 0.49887E+01_JPRB, 0.47606E+01_JPRB, 0.45530E+01_JPRB, 0.41740E+01_JPRB ,&
&  0.37780E+01_JPRB, 0.32848E+01_JPRB, 0.26886E+01_JPRB, 0.21964E+01_JPRB, 0.17520E+01_JPRB, 0.16328E+01_JPRB, 0.14938E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.11083E+01_JPRB, 0.11449E+01_JPRB, 0.12590E+01_JPRB, 0.12572E+01_JPRB, 0.11509E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.13841E+01_JPRB, 0.16424E+01_JPRB, 0.18260E+01_JPRB, 0.18333E+01_JPRB, 0.16759E+01_JPRB, 0.12343E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.13061E+01_JPRB, 0.13125E+01_JPRB, 0.13083E+01_JPRB, 0.12923E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.13301E+01_JPRB, 0.11632E+01_JPRB, 0.10072E+01_JPRB, 0.91910E+00_JPRB, 0.92550E+00_JPRB, 0.93278E+00_JPRB, 0.93459E+00_JPRB ,&
&  0.95207E+00_JPRB, 0.96761E+00_JPRB, 0.98170E+00_JPRB, 0.99361E+00_JPRB, 0.99840E+00_JPRB, 0.99810E+00_JPRB, 0.10389E+01_JPRB ,&
&  0.89420E+00_JPRB, 0.86656E+00_JPRB, 0.82656E+00_JPRB, 0.86044E+00_JPRB, 0.10657E+01_JPRB, 0.10825E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11046E+01_JPRB, 0.10726E+01_JPRB, 0.10239E+01_JPRB, 0.97882E+00_JPRB, 0.96045E+00_JPRB, 0.91936E+00_JPRB ,&
&  0.83155E+00_JPRB, 0.77462E+00_JPRB, 0.75918E+00_JPRB, 0.76033E+00_JPRB, 0.79711E+00_JPRB, 0.82645E+00_JPRB, 0.88519E+00_JPRB ,&
&  0.91297E+00_JPRB, 0.94693E+00_JPRB, 0.98527E+00_JPRB, 0.10235E+01_JPRB, 0.10519E+01_JPRB, 0.10811E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.11953E+01_JPRB, 0.12674E+01_JPRB, 0.13817E+01_JPRB, 0.14193E+01_JPRB, 0.16627E+01_JPRB, 0.22066E+01_JPRB ,&
&  0.26161E+01_JPRB, 0.27198E+01_JPRB, 0.32867E+01_JPRB, 0.44817E+01_JPRB, 0.46657E+01_JPRB, 0.44241E+01_JPRB, 0.41349E+01_JPRB ,&
&  0.39989E+01_JPRB, 0.39129E+01_JPRB, 0.40810E+01_JPRB, 0.43288E+01_JPRB, 0.45247E+01_JPRB, 0.48501E+01_JPRB, 0.50371E+01_JPRB ,&
&  0.50628E+01_JPRB, 0.50427E+01_JPRB, 0.49221E+01_JPRB, 0.47420E+01_JPRB, 0.44316E+01_JPRB, 0.42174E+01_JPRB, 0.39528E+01_JPRB ,&
&  0.36156E+01_JPRB, 0.31672E+01_JPRB, 0.24777E+01_JPRB, 0.20493E+01_JPRB, 0.15566E+01_JPRB, 0.14343E+01_JPRB, 0.17232E+01_JPRB ,&
&  0.13453E+01_JPRB, 0.12359E+01_JPRB, 0.13424E+01_JPRB, 0.15368E+01_JPRB, 0.15888E+01_JPRB, 0.15633E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.13710E+01_JPRB, 0.19221E+01_JPRB, 0.20812E+01_JPRB, 0.20402E+01_JPRB, 0.18458E+01_JPRB, 0.14688E+01_JPRB ,&
&  0.14935E+01_JPRB, 0.14280E+01_JPRB, 0.14187E+01_JPRB, 0.14135E+01_JPRB, 0.13968E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.14003E+01_JPRB, 0.12254E+01_JPRB, 0.10838E+01_JPRB, 0.98249E+00_JPRB, 0.97276E+00_JPRB, 0.99281E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.10434E+01_JPRB, 0.10539E+01_JPRB, 0.10642E+01_JPRB, 0.10646E+01_JPRB, 0.10361E+01_JPRB, 0.10126E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.95765E+00_JPRB, 0.92941E+00_JPRB, 0.91850E+00_JPRB, 0.10865E+01_JPRB, 0.12025E+01_JPRB, 0.13446E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.15540E+01_JPRB, 0.15203E+01_JPRB, 0.14295E+01_JPRB, 0.13368E+01_JPRB, 0.12621E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.99936E+00_JPRB, 0.93986E+00_JPRB, 0.91772E+00_JPRB, 0.95771E+00_JPRB, 0.10234E+01_JPRB, 0.94699E+00_JPRB, 0.10240E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.11102E+01_JPRB, 0.11357E+01_JPRB, 0.11604E+01_JPRB, 0.12175E+01_JPRB, 0.12874E+01_JPRB, 0.13545E+01_JPRB ,&
&  0.14191E+01_JPRB, 0.14386E+01_JPRB, 0.14337E+01_JPRB, 0.15180E+01_JPRB, 0.16160E+01_JPRB, 0.17785E+01_JPRB, 0.21970E+01_JPRB ,&
&  0.27904E+01_JPRB, 0.30923E+01_JPRB, 0.38771E+01_JPRB, 0.53527E+01_JPRB, 0.55728E+01_JPRB, 0.53162E+01_JPRB, 0.52663E+01_JPRB ,&
&  0.52065E+01_JPRB, 0.50561E+01_JPRB, 0.49652E+01_JPRB, 0.48596E+01_JPRB, 0.50483E+01_JPRB, 0.53405E+01_JPRB, 0.54274E+01_JPRB ,&
&  0.51210E+01_JPRB, 0.47267E+01_JPRB, 0.45318E+01_JPRB, 0.42272E+01_JPRB, 0.39466E+01_JPRB, 0.36098E+01_JPRB, 0.36351E+01_JPRB ,&
&  0.36732E+01_JPRB, 0.26233E+01_JPRB, 0.21437E+01_JPRB, 0.20203E+01_JPRB, 0.15336E+01_JPRB, 0.13536E+01_JPRB, 0.15061E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.12114E+01_JPRB, 0.14632E+01_JPRB, 0.17291E+01_JPRB, 0.20328E+01_JPRB, 0.21893E+01_JPRB, 0.21120E+01_JPRB ,&
&  0.18660E+01_JPRB, 0.19052E+01_JPRB, 0.25216E+01_JPRB, 0.25689E+01_JPRB, 0.23369E+01_JPRB, 0.20593E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.16710E+01_JPRB, 0.16507E+01_JPRB, 0.16111E+01_JPRB, 0.15749E+01_JPRB, 0.15331E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.15482E+01_JPRB, 0.13694E+01_JPRB, 0.12015E+01_JPRB, 0.11010E+01_JPRB, 0.10899E+01_JPRB, 0.11057E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.10958E+01_JPRB, 0.11096E+01_JPRB, 0.11133E+01_JPRB, 0.11065E+01_JPRB, 0.10720E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.98748E+00_JPRB, 0.10228E+01_JPRB, 0.96690E+00_JPRB, 0.99526E+00_JPRB, 0.10690E+01_JPRB, 0.13044E+01_JPRB, 0.15246E+01_JPRB ,&
&  0.17605E+01_JPRB, 0.19241E+01_JPRB, 0.19929E+01_JPRB, 0.20321E+01_JPRB, 0.19763E+01_JPRB, 0.18285E+01_JPRB, 0.16715E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.14783E+01_JPRB, 0.13311E+01_JPRB, 0.14162E+01_JPRB, 0.14018E+01_JPRB, 0.12791E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.14410E+01_JPRB, 0.14501E+01_JPRB, 0.14975E+01_JPRB, 0.15500E+01_JPRB, 0.15644E+01_JPRB, 0.15512E+01_JPRB ,&
&  0.15378E+01_JPRB, 0.15482E+01_JPRB, 0.15904E+01_JPRB, 0.15868E+01_JPRB, 0.16727E+01_JPRB, 0.19594E+01_JPRB, 0.24705E+01_JPRB ,&
&  0.31426E+01_JPRB, 0.32325E+01_JPRB, 0.34447E+01_JPRB, 0.50310E+01_JPRB, 0.63314E+01_JPRB, 0.62258E+01_JPRB, 0.60395E+01_JPRB ,&
&  0.60534E+01_JPRB, 0.56886E+01_JPRB, 0.53153E+01_JPRB, 0.51587E+01_JPRB, 0.56065E+01_JPRB, 0.57695E+01_JPRB, 0.56446E+01_JPRB ,&
&  0.52061E+01_JPRB, 0.47067E+01_JPRB, 0.44619E+01_JPRB, 0.41820E+01_JPRB, 0.37375E+01_JPRB, 0.35641E+01_JPRB, 0.36326E+01_JPRB ,&
&  0.35836E+01_JPRB, 0.31629E+01_JPRB, 0.24369E+01_JPRB, 0.21959E+01_JPRB, 0.18030E+01_JPRB, 0.16426E+01_JPRB, 0.17942E+01_JPRB ,&
&  0.15994E+01_JPRB, 0.16326E+01_JPRB, 0.17834E+01_JPRB, 0.19156E+01_JPRB, 0.30975E+01_JPRB, 0.34415E+01_JPRB, 0.37231E+01_JPRB ,&
&  0.34952E+01_JPRB, 0.31661E+01_JPRB, 0.37501E+01_JPRB, 0.35331E+01_JPRB, 0.29729E+01_JPRB, 0.24279E+01_JPRB, 0.20344E+01_JPRB ,&
&  0.18920E+01_JPRB, 0.18876E+01_JPRB, 0.18224E+01_JPRB, 0.17578E+01_JPRB, 0.17059E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.16845E+01_JPRB, 0.14945E+01_JPRB, 0.13247E+01_JPRB, 0.12123E+01_JPRB, 0.11704E+01_JPRB, 0.11757E+01_JPRB, 0.11731E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.11342E+01_JPRB, 0.11339E+01_JPRB, 0.11142E+01_JPRB, 0.10877E+01_JPRB, 0.10555E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.98085E+00_JPRB, 0.10130E+01_JPRB, 0.97785E+00_JPRB, 0.10141E+01_JPRB, 0.11439E+01_JPRB, 0.14787E+01_JPRB, 0.18797E+01_JPRB ,&
&  0.22363E+01_JPRB, 0.24344E+01_JPRB, 0.25650E+01_JPRB, 0.26816E+01_JPRB, 0.27975E+01_JPRB, 0.27774E+01_JPRB, 0.26473E+01_JPRB ,&
&  0.27477E+01_JPRB, 0.23072E+01_JPRB, 0.23287E+01_JPRB, 0.21554E+01_JPRB, 0.20948E+01_JPRB, 0.20034E+01_JPRB, 0.19113E+01_JPRB ,&
&  0.18362E+01_JPRB, 0.17854E+01_JPRB, 0.17288E+01_JPRB, 0.17069E+01_JPRB, 0.17062E+01_JPRB, 0.17074E+01_JPRB, 0.17267E+01_JPRB ,&
&  0.17554E+01_JPRB, 0.17155E+01_JPRB, 0.16505E+01_JPRB, 0.16522E+01_JPRB, 0.20768E+01_JPRB, 0.28330E+01_JPRB, 0.34883E+01_JPRB ,&
&  0.34832E+01_JPRB, 0.33630E+01_JPRB, 0.54617E+01_JPRB, 0.62248E+01_JPRB, 0.61346E+01_JPRB, 0.67362E+01_JPRB, 0.67699E+01_JPRB ,&
&  0.61928E+01_JPRB, 0.54044E+01_JPRB, 0.47758E+01_JPRB, 0.48122E+01_JPRB, 0.61499E+01_JPRB, 0.62381E+01_JPRB, 0.60039E+01_JPRB ,&
&  0.54255E+01_JPRB, 0.49238E+01_JPRB, 0.44136E+01_JPRB, 0.40442E+01_JPRB, 0.38390E+01_JPRB, 0.40046E+01_JPRB, 0.37792E+01_JPRB ,&
&  0.35850E+01_JPRB, 0.38876E+01_JPRB, 0.32322E+01_JPRB, 0.25648E+01_JPRB, 0.22589E+01_JPRB, 0.22077E+01_JPRB, 0.22211E+01_JPRB ,&
&  0.21578E+01_JPRB, 0.22291E+01_JPRB, 0.23562E+01_JPRB, 0.30368E+01_JPRB, 0.47970E+01_JPRB, 0.49748E+01_JPRB, 0.48342E+01_JPRB ,&
&  0.47226E+01_JPRB, 0.55035E+01_JPRB, 0.48819E+01_JPRB, 0.41739E+01_JPRB, 0.34312E+01_JPRB, 0.28111E+01_JPRB, 0.23617E+01_JPRB ,&
&  0.21163E+01_JPRB, 0.20809E+01_JPRB, 0.20287E+01_JPRB, 0.19367E+01_JPRB, 0.18622E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.18657E+01_JPRB, 0.16356E+01_JPRB, 0.15126E+01_JPRB, 0.13920E+01_JPRB, 0.13264E+01_JPRB, 0.12754E+01_JPRB, 0.11888E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11300E+01_JPRB, 0.11099E+01_JPRB, 0.10900E+01_JPRB, 0.10679E+01_JPRB, 0.10295E+01_JPRB, 0.99170E+00_JPRB ,&
&  0.98740E+00_JPRB, 0.10043E+01_JPRB, 0.10267E+01_JPRB, 0.11702E+01_JPRB, 0.13578E+01_JPRB, 0.16455E+01_JPRB, 0.21159E+01_JPRB ,&
&  0.25449E+01_JPRB, 0.28334E+01_JPRB, 0.29571E+01_JPRB, 0.33347E+01_JPRB, 0.37216E+01_JPRB, 0.40437E+01_JPRB, 0.40109E+01_JPRB ,&
&  0.36331E+01_JPRB, 0.35589E+01_JPRB, 0.34272E+01_JPRB, 0.31359E+01_JPRB, 0.29004E+01_JPRB, 0.27045E+01_JPRB, 0.24895E+01_JPRB ,&
&  0.22638E+01_JPRB, 0.20574E+01_JPRB, 0.19188E+01_JPRB, 0.18523E+01_JPRB, 0.18238E+01_JPRB, 0.18609E+01_JPRB, 0.18501E+01_JPRB ,&
&  0.17844E+01_JPRB, 0.17583E+01_JPRB, 0.17212E+01_JPRB, 0.20489E+01_JPRB, 0.28642E+01_JPRB, 0.40660E+01_JPRB, 0.43814E+01_JPRB ,&
&  0.47322E+01_JPRB, 0.45599E+01_JPRB, 0.52431E+01_JPRB, 0.61075E+01_JPRB, 0.51787E+01_JPRB, 0.58361E+01_JPRB, 0.61524E+01_JPRB ,&
&  0.55178E+01_JPRB, 0.51478E+01_JPRB, 0.55866E+01_JPRB, 0.60546E+01_JPRB, 0.64703E+01_JPRB, 0.76368E+01_JPRB, 0.72569E+01_JPRB ,&
&  0.61528E+01_JPRB, 0.52933E+01_JPRB, 0.44374E+01_JPRB, 0.42389E+01_JPRB, 0.37626E+01_JPRB, 0.37248E+01_JPRB, 0.31064E+01_JPRB ,&
&  0.33055E+01_JPRB, 0.28843E+01_JPRB, 0.30645E+01_JPRB, 0.27542E+01_JPRB, 0.27500E+01_JPRB, 0.26151E+01_JPRB, 0.25258E+01_JPRB ,&
&  0.26343E+01_JPRB, 0.28439E+01_JPRB, 0.33684E+01_JPRB, 0.49486E+01_JPRB, 0.68840E+01_JPRB, 0.57725E+01_JPRB, 0.66074E+01_JPRB ,&
&  0.80360E+01_JPRB, 0.76206E+01_JPRB, 0.54168E+01_JPRB, 0.45318E+01_JPRB, 0.36838E+01_JPRB, 0.30891E+01_JPRB, 0.27534E+01_JPRB ,&
&  0.24690E+01_JPRB, 0.23341E+01_JPRB, 0.22525E+01_JPRB, 0.21629E+01_JPRB, 0.20642E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.19614E+01_JPRB, 0.17372E+01_JPRB, 0.16333E+01_JPRB, 0.15808E+01_JPRB, 0.14814E+01_JPRB, 0.13070E+01_JPRB, 0.11986E+01_JPRB ,&
&  0.11973E+01_JPRB, 0.11611E+01_JPRB, 0.10894E+01_JPRB, 0.10425E+01_JPRB, 0.10299E+01_JPRB, 0.10274E+01_JPRB, 0.10021E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.11398E+01_JPRB, 0.12434E+01_JPRB, 0.13828E+01_JPRB, 0.16250E+01_JPRB, 0.18795E+01_JPRB, 0.21240E+01_JPRB ,&
&  0.30600E+01_JPRB, 0.33127E+01_JPRB, 0.32364E+01_JPRB, 0.36589E+01_JPRB, 0.43199E+01_JPRB, 0.49778E+01_JPRB, 0.51234E+01_JPRB ,&
&  0.52592E+01_JPRB, 0.50005E+01_JPRB, 0.40840E+01_JPRB, 0.34289E+01_JPRB, 0.29883E+01_JPRB, 0.27289E+01_JPRB, 0.24555E+01_JPRB ,&
&  0.21359E+01_JPRB, 0.18746E+01_JPRB, 0.17624E+01_JPRB, 0.17028E+01_JPRB, 0.17008E+01_JPRB, 0.16195E+01_JPRB, 0.16188E+01_JPRB ,&
&  0.16689E+01_JPRB, 0.17643E+01_JPRB, 0.18947E+01_JPRB, 0.24625E+01_JPRB, 0.36969E+01_JPRB, 0.49782E+01_JPRB, 0.53357E+01_JPRB ,&
&  0.49143E+01_JPRB, 0.45403E+01_JPRB, 0.52370E+01_JPRB, 0.58150E+01_JPRB, 0.58871E+01_JPRB, 0.56537E+01_JPRB, 0.52655E+01_JPRB ,&
&  0.43109E+01_JPRB, 0.42551E+01_JPRB, 0.49008E+01_JPRB, 0.58570E+01_JPRB, 0.72171E+01_JPRB, 0.89575E+01_JPRB, 0.86625E+01_JPRB ,&
&  0.71088E+01_JPRB, 0.59052E+01_JPRB, 0.49924E+01_JPRB, 0.41711E+01_JPRB, 0.42401E+01_JPRB, 0.33165E+01_JPRB, 0.38704E+01_JPRB ,&
&  0.40240E+01_JPRB, 0.38974E+01_JPRB, 0.32525E+01_JPRB, 0.27327E+01_JPRB, 0.24350E+01_JPRB, 0.28562E+01_JPRB, 0.29395E+01_JPRB ,&
&  0.33894E+01_JPRB, 0.38384E+01_JPRB, 0.67715E+01_JPRB, 0.80400E+01_JPRB, 0.82423E+01_JPRB, 0.75438E+01_JPRB, 0.94416E+01_JPRB ,&
&  0.87161E+01_JPRB, 0.74648E+01_JPRB, 0.58756E+01_JPRB, 0.47809E+01_JPRB, 0.39165E+01_JPRB, 0.34102E+01_JPRB, 0.30936E+01_JPRB ,&
&  0.27672E+01_JPRB, 0.25949E+01_JPRB, 0.24678E+01_JPRB, 0.23416E+01_JPRB, 0.21904E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.19098E+01_JPRB, 0.17876E+01_JPRB, 0.17461E+01_JPRB, 0.16070E+01_JPRB, 0.13910E+01_JPRB, 0.12463E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.11394E+01_JPRB, 0.10717E+01_JPRB, 0.10267E+01_JPRB, 0.10328E+01_JPRB, 0.10469E+01_JPRB, 0.10694E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.14306E+01_JPRB, 0.15902E+01_JPRB, 0.17793E+01_JPRB, 0.19632E+01_JPRB, 0.21443E+01_JPRB, 0.24716E+01_JPRB ,&
&  0.35744E+01_JPRB, 0.33413E+01_JPRB, 0.28542E+01_JPRB, 0.32088E+01_JPRB, 0.44466E+01_JPRB, 0.48803E+01_JPRB, 0.50897E+01_JPRB ,&
&  0.49812E+01_JPRB, 0.43100E+01_JPRB, 0.35576E+01_JPRB, 0.29282E+01_JPRB, 0.25261E+01_JPRB, 0.22683E+01_JPRB, 0.19454E+01_JPRB ,&
&  0.17322E+01_JPRB, 0.16503E+01_JPRB, 0.16553E+01_JPRB, 0.16606E+01_JPRB, 0.16281E+01_JPRB, 0.16126E+01_JPRB, 0.16538E+01_JPRB ,&
&  0.17186E+01_JPRB, 0.18271E+01_JPRB, 0.20430E+01_JPRB, 0.27892E+01_JPRB, 0.40680E+01_JPRB, 0.53374E+01_JPRB, 0.61536E+01_JPRB ,&
&  0.51834E+01_JPRB, 0.50565E+01_JPRB, 0.55625E+01_JPRB, 0.69189E+01_JPRB, 0.56926E+01_JPRB, 0.53761E+01_JPRB, 0.50287E+01_JPRB ,&
&  0.44655E+01_JPRB, 0.49142E+01_JPRB, 0.61705E+01_JPRB, 0.72349E+01_JPRB, 0.70683E+01_JPRB, 0.81061E+01_JPRB, 0.95597E+01_JPRB ,&
&  0.71710E+01_JPRB, 0.59446E+01_JPRB, 0.53112E+01_JPRB, 0.52076E+01_JPRB, 0.35145E+01_JPRB, 0.25726E+01_JPRB, 0.38847E+01_JPRB ,&
&  0.39200E+01_JPRB, 0.22631E+01_JPRB, 0.16910E+01_JPRB, 0.19905E+01_JPRB, 0.15390E+01_JPRB, 0.19167E+01_JPRB, 0.31742E+01_JPRB ,&
&  0.42415E+01_JPRB, 0.52697E+01_JPRB, 0.10845E+02_JPRB, 0.98735E+01_JPRB, 0.99973E+01_JPRB, 0.10381E+02_JPRB, 0.10846E+02_JPRB ,&
&  0.95136E+01_JPRB, 0.60373E+01_JPRB, 0.54046E+01_JPRB, 0.46939E+01_JPRB, 0.38039E+01_JPRB, 0.31629E+01_JPRB, 0.27151E+01_JPRB ,&
&  0.23709E+01_JPRB, 0.21757E+01_JPRB, 0.20687E+01_JPRB, 0.20231E+01_JPRB, 0.20235E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.16311E+01_JPRB, 0.15869E+01_JPRB, 0.15177E+01_JPRB, 0.13559E+01_JPRB, 0.12360E+01_JPRB, 0.11930E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.11511E+01_JPRB, 0.10765E+01_JPRB, 0.10363E+01_JPRB, 0.10473E+01_JPRB, 0.10687E+01_JPRB, 0.10957E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.14922E+01_JPRB, 0.20056E+01_JPRB, 0.23944E+01_JPRB, 0.26269E+01_JPRB, 0.23090E+01_JPRB, 0.29838E+01_JPRB ,&
&  0.36520E+01_JPRB, 0.29398E+01_JPRB, 0.25647E+01_JPRB, 0.30279E+01_JPRB, 0.37198E+01_JPRB, 0.39755E+01_JPRB, 0.40362E+01_JPRB ,&
&  0.37244E+01_JPRB, 0.32612E+01_JPRB, 0.28810E+01_JPRB, 0.26388E+01_JPRB, 0.22926E+01_JPRB, 0.18932E+01_JPRB, 0.17203E+01_JPRB ,&
&  0.16470E+01_JPRB, 0.16952E+01_JPRB, 0.17578E+01_JPRB, 0.17206E+01_JPRB, 0.17308E+01_JPRB, 0.17356E+01_JPRB, 0.17854E+01_JPRB ,&
&  0.18510E+01_JPRB, 0.19816E+01_JPRB, 0.23246E+01_JPRB, 0.33575E+01_JPRB, 0.51218E+01_JPRB, 0.67955E+01_JPRB, 0.57768E+01_JPRB ,&
&  0.46051E+01_JPRB, 0.44007E+01_JPRB, 0.59374E+01_JPRB, 0.64568E+01_JPRB, 0.53613E+01_JPRB, 0.53904E+01_JPRB, 0.50728E+01_JPRB ,&
&  0.54544E+01_JPRB, 0.60913E+01_JPRB, 0.61753E+01_JPRB, 0.71063E+01_JPRB, 0.80270E+01_JPRB, 0.79646E+01_JPRB, 0.82769E+01_JPRB ,&
&  0.71697E+01_JPRB, 0.57584E+01_JPRB, 0.44877E+01_JPRB, 0.38395E+01_JPRB, 0.45017E+01_JPRB, 0.34559E+01_JPRB, 0.22744E+01_JPRB ,&
&  0.18164E+01_JPRB, 0.14507E+01_JPRB, 0.11803E+01_JPRB, 0.11788E+01_JPRB, 0.11098E+01_JPRB, 0.17272E+01_JPRB, 0.30997E+01_JPRB ,&
&  0.62867E+01_JPRB, 0.88392E+01_JPRB, 0.11789E+02_JPRB, 0.11906E+02_JPRB, 0.10716E+02_JPRB, 0.11126E+02_JPRB, 0.11927E+02_JPRB ,&
&  0.73547E+01_JPRB, 0.52493E+01_JPRB, 0.48966E+01_JPRB, 0.39359E+01_JPRB, 0.29147E+01_JPRB, 0.23352E+01_JPRB, 0.19983E+01_JPRB ,&
&  0.18202E+01_JPRB, 0.17154E+01_JPRB, 0.16487E+01_JPRB, 0.16492E+01_JPRB, 0.16756E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.13839E+01_JPRB, 0.13517E+01_JPRB, 0.13220E+01_JPRB, 0.12366E+01_JPRB, 0.11589E+01_JPRB, 0.11619E+01_JPRB, 0.11639E+01_JPRB ,&
&  0.11193E+01_JPRB, 0.10662E+01_JPRB, 0.10653E+01_JPRB, 0.10985E+01_JPRB, 0.11389E+01_JPRB, 0.12113E+01_JPRB, 0.12978E+01_JPRB ,&
&  0.14163E+01_JPRB, 0.17336E+01_JPRB, 0.24290E+01_JPRB, 0.31256E+01_JPRB, 0.35699E+01_JPRB, 0.28752E+01_JPRB, 0.34693E+01_JPRB ,&
&  0.34835E+01_JPRB, 0.27424E+01_JPRB, 0.26108E+01_JPRB, 0.29997E+01_JPRB, 0.34466E+01_JPRB, 0.32144E+01_JPRB, 0.29903E+01_JPRB ,&
&  0.27918E+01_JPRB, 0.25948E+01_JPRB, 0.24147E+01_JPRB, 0.22663E+01_JPRB, 0.19720E+01_JPRB, 0.18978E+01_JPRB, 0.19269E+01_JPRB ,&
&  0.19357E+01_JPRB, 0.19914E+01_JPRB, 0.19556E+01_JPRB, 0.19218E+01_JPRB, 0.19189E+01_JPRB, 0.19365E+01_JPRB, 0.19916E+01_JPRB ,&
&  0.20895E+01_JPRB, 0.23374E+01_JPRB, 0.30277E+01_JPRB, 0.46213E+01_JPRB, 0.58102E+01_JPRB, 0.58083E+01_JPRB, 0.54318E+01_JPRB ,&
&  0.51278E+01_JPRB, 0.54180E+01_JPRB, 0.57345E+01_JPRB, 0.66150E+01_JPRB, 0.54086E+01_JPRB, 0.52120E+01_JPRB, 0.50718E+01_JPRB ,&
&  0.54163E+01_JPRB, 0.65091E+01_JPRB, 0.65448E+01_JPRB, 0.72298E+01_JPRB, 0.96377E+01_JPRB, 0.95677E+01_JPRB, 0.77015E+01_JPRB ,&
&  0.69899E+01_JPRB, 0.56264E+01_JPRB, 0.48885E+01_JPRB, 0.45395E+01_JPRB, 0.68229E+01_JPRB, 0.72728E+01_JPRB, 0.25698E+01_JPRB ,&
&  0.15610E+01_JPRB, 0.12395E+01_JPRB, 0.93481E+00_JPRB, 0.91588E+00_JPRB, 0.10868E+01_JPRB, 0.17915E+01_JPRB, 0.49372E+01_JPRB ,&
&  0.58697E+01_JPRB, 0.95939E+01_JPRB, 0.10834E+02_JPRB, 0.11578E+02_JPRB, 0.10594E+02_JPRB, 0.10341E+02_JPRB, 0.77659E+01_JPRB ,&
&  0.61508E+01_JPRB, 0.47289E+01_JPRB, 0.35772E+01_JPRB, 0.27564E+01_JPRB, 0.21558E+01_JPRB, 0.18330E+01_JPRB, 0.16762E+01_JPRB ,&
&  0.16074E+01_JPRB, 0.15697E+01_JPRB, 0.15144E+01_JPRB, 0.14590E+01_JPRB, 0.13945E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.11694E+01_JPRB, 0.11949E+01_JPRB, 0.12270E+01_JPRB, 0.12206E+01_JPRB, 0.12128E+01_JPRB, 0.12423E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.11428E+01_JPRB, 0.11613E+01_JPRB, 0.12047E+01_JPRB, 0.12738E+01_JPRB, 0.13760E+01_JPRB, 0.14961E+01_JPRB ,&
&  0.17279E+01_JPRB, 0.22510E+01_JPRB, 0.30406E+01_JPRB, 0.40570E+01_JPRB, 0.43865E+01_JPRB, 0.39671E+01_JPRB, 0.40030E+01_JPRB ,&
&  0.35067E+01_JPRB, 0.28273E+01_JPRB, 0.26055E+01_JPRB, 0.29019E+01_JPRB, 0.29164E+01_JPRB, 0.26795E+01_JPRB, 0.23414E+01_JPRB ,&
&  0.23400E+01_JPRB, 0.23329E+01_JPRB, 0.22709E+01_JPRB, 0.22519E+01_JPRB, 0.22812E+01_JPRB, 0.23346E+01_JPRB, 0.23866E+01_JPRB ,&
&  0.24082E+01_JPRB, 0.23432E+01_JPRB, 0.22494E+01_JPRB, 0.21910E+01_JPRB, 0.21814E+01_JPRB, 0.22291E+01_JPRB, 0.23335E+01_JPRB ,&
&  0.25429E+01_JPRB, 0.30602E+01_JPRB, 0.42074E+01_JPRB, 0.56383E+01_JPRB, 0.54172E+01_JPRB, 0.59140E+01_JPRB, 0.55401E+01_JPRB ,&
&  0.52329E+01_JPRB, 0.53923E+01_JPRB, 0.53044E+01_JPRB, 0.58647E+01_JPRB, 0.55522E+01_JPRB, 0.54365E+01_JPRB, 0.52193E+01_JPRB ,&
&  0.62463E+01_JPRB, 0.68808E+01_JPRB, 0.78199E+01_JPRB, 0.83239E+01_JPRB, 0.98206E+01_JPRB, 0.11957E+02_JPRB, 0.78138E+01_JPRB ,&
&  0.61130E+01_JPRB, 0.59974E+01_JPRB, 0.53765E+01_JPRB, 0.50801E+01_JPRB, 0.63340E+01_JPRB, 0.87811E+01_JPRB, 0.66140E+01_JPRB ,&
&  0.21204E+01_JPRB, 0.13448E+01_JPRB, 0.12015E+01_JPRB, 0.12026E+01_JPRB, 0.10419E+01_JPRB, 0.18639E+01_JPRB, 0.73253E+01_JPRB ,&
&  0.62350E+01_JPRB, 0.83249E+01_JPRB, 0.93681E+01_JPRB, 0.99017E+01_JPRB, 0.96208E+01_JPRB, 0.70604E+01_JPRB, 0.48605E+01_JPRB ,&
&  0.31282E+01_JPRB, 0.22781E+01_JPRB, 0.19543E+01_JPRB, 0.17178E+01_JPRB, 0.14471E+01_JPRB, 0.13137E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.12256E+01_JPRB, 0.11951E+01_JPRB, 0.11549E+01_JPRB, 0.11536E+01_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.11188E+01_JPRB, 0.12011E+01_JPRB, 0.12745E+01_JPRB, 0.13433E+01_JPRB, 0.13847E+01_JPRB, 0.13800E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.13347E+01_JPRB, 0.13199E+01_JPRB, 0.13470E+01_JPRB, 0.13996E+01_JPRB, 0.15172E+01_JPRB, 0.16966E+01_JPRB, 0.19783E+01_JPRB ,&
&  0.24315E+01_JPRB, 0.30838E+01_JPRB, 0.38911E+01_JPRB, 0.46729E+01_JPRB, 0.47826E+01_JPRB, 0.41349E+01_JPRB, 0.40266E+01_JPRB ,&
&  0.36474E+01_JPRB, 0.30340E+01_JPRB, 0.25806E+01_JPRB, 0.28103E+01_JPRB, 0.29524E+01_JPRB, 0.28848E+01_JPRB, 0.25759E+01_JPRB ,&
&  0.25814E+01_JPRB, 0.26014E+01_JPRB, 0.26242E+01_JPRB, 0.26836E+01_JPRB, 0.27410E+01_JPRB, 0.27867E+01_JPRB, 0.27816E+01_JPRB ,&
&  0.26841E+01_JPRB, 0.25698E+01_JPRB, 0.25159E+01_JPRB, 0.25275E+01_JPRB, 0.25912E+01_JPRB, 0.26824E+01_JPRB, 0.28824E+01_JPRB ,&
&  0.33123E+01_JPRB, 0.40939E+01_JPRB, 0.51335E+01_JPRB, 0.55725E+01_JPRB, 0.55135E+01_JPRB, 0.54252E+01_JPRB, 0.52407E+01_JPRB ,&
&  0.51237E+01_JPRB, 0.53239E+01_JPRB, 0.53948E+01_JPRB, 0.54608E+01_JPRB, 0.54470E+01_JPRB, 0.56821E+01_JPRB, 0.57494E+01_JPRB ,&
&  0.70685E+01_JPRB, 0.81072E+01_JPRB, 0.81845E+01_JPRB, 0.82078E+01_JPRB, 0.86251E+01_JPRB, 0.97180E+01_JPRB, 0.89524E+01_JPRB ,&
&  0.70395E+01_JPRB, 0.55656E+01_JPRB, 0.51280E+01_JPRB, 0.54675E+01_JPRB, 0.57526E+01_JPRB, 0.67226E+01_JPRB, 0.82842E+01_JPRB ,&
&  0.74213E+01_JPRB, 0.28870E+01_JPRB, 0.18657E+01_JPRB, 0.26656E+01_JPRB, 0.16597E+01_JPRB, 0.19404E+01_JPRB, 0.35982E+01_JPRB ,&
&  0.45854E+01_JPRB, 0.55939E+01_JPRB, 0.64072E+01_JPRB, 0.71234E+01_JPRB, 0.59962E+01_JPRB, 0.41897E+01_JPRB, 0.27711E+01_JPRB ,&
&  0.19016E+01_JPRB, 0.15699E+01_JPRB, 0.14884E+01_JPRB, 0.12910E+01_JPRB, 0.11549E+01_JPRB, 0.11226E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10385E+01_JPRB, 0.10593E+01_JPRB, 0.10610E+01_JPRB, 0.10593E+01_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.12161E+01_JPRB, 0.13112E+01_JPRB, 0.14409E+01_JPRB, 0.15188E+01_JPRB, 0.15401E+01_JPRB, 0.15468E+01_JPRB, 0.15787E+01_JPRB ,&
&  0.16157E+01_JPRB, 0.16629E+01_JPRB, 0.17515E+01_JPRB, 0.18945E+01_JPRB, 0.21410E+01_JPRB, 0.24523E+01_JPRB, 0.28593E+01_JPRB ,&
&  0.33696E+01_JPRB, 0.39428E+01_JPRB, 0.45521E+01_JPRB, 0.51480E+01_JPRB, 0.55389E+01_JPRB, 0.58506E+01_JPRB, 0.38369E+01_JPRB ,&
&  0.31794E+01_JPRB, 0.34613E+01_JPRB, 0.27861E+01_JPRB, 0.30542E+01_JPRB, 0.31889E+01_JPRB, 0.32017E+01_JPRB, 0.29088E+01_JPRB ,&
&  0.28604E+01_JPRB, 0.28785E+01_JPRB, 0.29157E+01_JPRB, 0.29308E+01_JPRB, 0.29384E+01_JPRB, 0.29372E+01_JPRB, 0.28865E+01_JPRB ,&
&  0.28238E+01_JPRB, 0.28068E+01_JPRB, 0.28580E+01_JPRB, 0.29484E+01_JPRB, 0.30782E+01_JPRB, 0.32783E+01_JPRB, 0.36148E+01_JPRB ,&
&  0.41028E+01_JPRB, 0.46989E+01_JPRB, 0.52443E+01_JPRB, 0.51304E+01_JPRB, 0.50795E+01_JPRB, 0.49990E+01_JPRB, 0.50895E+01_JPRB ,&
&  0.49646E+01_JPRB, 0.47133E+01_JPRB, 0.51042E+01_JPRB, 0.55122E+01_JPRB, 0.55968E+01_JPRB, 0.57462E+01_JPRB, 0.61107E+01_JPRB ,&
&  0.75837E+01_JPRB, 0.83904E+01_JPRB, 0.86648E+01_JPRB, 0.75618E+01_JPRB, 0.72449E+01_JPRB, 0.79047E+01_JPRB, 0.81523E+01_JPRB ,&
&  0.71825E+01_JPRB, 0.60276E+01_JPRB, 0.54501E+01_JPRB, 0.51300E+01_JPRB, 0.48828E+01_JPRB, 0.52666E+01_JPRB, 0.58384E+01_JPRB ,&
&  0.62601E+01_JPRB, 0.70166E+01_JPRB, 0.56250E+01_JPRB, 0.33333E+01_JPRB, 0.23864E+01_JPRB, 0.25940E+01_JPRB, 0.32476E+01_JPRB ,&
&  0.36267E+01_JPRB, 0.38340E+01_JPRB, 0.37980E+01_JPRB, 0.52085E+01_JPRB, 0.30789E+01_JPRB, 0.21389E+01_JPRB, 0.15394E+01_JPRB ,&
&  0.13095E+01_JPRB, 0.12729E+01_JPRB, 0.12632E+01_JPRB, 0.11581E+01_JPRB, 0.10734E+01_JPRB, 0.10954E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.11008E+01_JPRB, 0.11150E+01_JPRB, 0.11297E+01_JPRB, 0.11716E+01_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.14301E+01_JPRB, 0.15154E+01_JPRB, 0.16243E+01_JPRB, 0.16941E+01_JPRB, 0.17784E+01_JPRB, 0.18528E+01_JPRB, 0.19696E+01_JPRB ,&
&  0.21043E+01_JPRB, 0.22647E+01_JPRB, 0.24466E+01_JPRB, 0.26820E+01_JPRB, 0.29937E+01_JPRB, 0.33358E+01_JPRB, 0.37209E+01_JPRB ,&
&  0.40921E+01_JPRB, 0.44288E+01_JPRB, 0.47573E+01_JPRB, 0.51070E+01_JPRB, 0.53829E+01_JPRB, 0.57172E+01_JPRB, 0.49275E+01_JPRB ,&
&  0.29415E+01_JPRB, 0.41292E+01_JPRB, 0.29604E+01_JPRB, 0.33231E+01_JPRB, 0.34054E+01_JPRB, 0.35538E+01_JPRB, 0.31630E+01_JPRB ,&
&  0.29510E+01_JPRB, 0.29269E+01_JPRB, 0.29112E+01_JPRB, 0.29329E+01_JPRB, 0.29481E+01_JPRB, 0.29595E+01_JPRB, 0.29643E+01_JPRB ,&
&  0.30013E+01_JPRB, 0.30922E+01_JPRB, 0.32243E+01_JPRB, 0.33679E+01_JPRB, 0.35802E+01_JPRB, 0.38647E+01_JPRB, 0.41451E+01_JPRB ,&
&  0.44456E+01_JPRB, 0.47553E+01_JPRB, 0.48609E+01_JPRB, 0.47474E+01_JPRB, 0.47053E+01_JPRB, 0.48435E+01_JPRB, 0.51357E+01_JPRB ,&
&  0.50742E+01_JPRB, 0.49175E+01_JPRB, 0.51988E+01_JPRB, 0.53371E+01_JPRB, 0.53480E+01_JPRB, 0.59269E+01_JPRB, 0.62433E+01_JPRB ,&
&  0.74058E+01_JPRB, 0.75239E+01_JPRB, 0.84590E+01_JPRB, 0.70972E+01_JPRB, 0.62270E+01_JPRB, 0.65104E+01_JPRB, 0.61219E+01_JPRB ,&
&  0.53039E+01_JPRB, 0.50790E+01_JPRB, 0.45907E+01_JPRB, 0.39758E+01_JPRB, 0.40121E+01_JPRB, 0.42058E+01_JPRB, 0.46869E+01_JPRB ,&
&  0.51422E+01_JPRB, 0.52668E+01_JPRB, 0.39533E+01_JPRB, 0.25381E+01_JPRB, 0.22890E+01_JPRB, 0.25460E+01_JPRB, 0.29178E+01_JPRB ,&
&  0.29426E+01_JPRB, 0.29510E+01_JPRB, 0.31007E+01_JPRB, 0.28081E+01_JPRB, 0.16969E+01_JPRB, 0.12976E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.11813E+01_JPRB, 0.11820E+01_JPRB, 0.11495E+01_JPRB, 0.10712E+01_JPRB, 0.11198E+01_JPRB, 0.11274E+01_JPRB ,&
&  0.11998E+01_JPRB, 0.12339E+01_JPRB, 0.12857E+01_JPRB, 0.13343E+01_JPRB, 0.13744E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.16809E+01_JPRB, 0.18018E+01_JPRB, 0.19894E+01_JPRB, 0.21588E+01_JPRB, 0.22997E+01_JPRB, 0.24179E+01_JPRB, 0.25150E+01_JPRB ,&
&  0.26046E+01_JPRB, 0.27253E+01_JPRB, 0.29567E+01_JPRB, 0.31979E+01_JPRB, 0.34987E+01_JPRB, 0.38351E+01_JPRB, 0.41571E+01_JPRB ,&
&  0.44337E+01_JPRB, 0.46943E+01_JPRB, 0.49566E+01_JPRB, 0.53130E+01_JPRB, 0.57364E+01_JPRB, 0.64541E+01_JPRB, 0.61895E+01_JPRB ,&
&  0.43938E+01_JPRB, 0.32730E+01_JPRB, 0.31404E+01_JPRB, 0.32201E+01_JPRB, 0.31487E+01_JPRB, 0.34402E+01_JPRB, 0.35024E+01_JPRB ,&
&  0.31416E+01_JPRB, 0.29976E+01_JPRB, 0.29626E+01_JPRB, 0.29620E+01_JPRB, 0.29095E+01_JPRB, 0.29673E+01_JPRB, 0.30359E+01_JPRB ,&
&  0.31631E+01_JPRB, 0.33241E+01_JPRB, 0.34890E+01_JPRB, 0.36857E+01_JPRB, 0.38994E+01_JPRB, 0.41341E+01_JPRB, 0.43389E+01_JPRB ,&
&  0.45065E+01_JPRB, 0.46578E+01_JPRB, 0.46638E+01_JPRB, 0.45233E+01_JPRB, 0.46038E+01_JPRB, 0.48008E+01_JPRB, 0.49231E+01_JPRB ,&
&  0.49863E+01_JPRB, 0.51932E+01_JPRB, 0.54493E+01_JPRB, 0.55253E+01_JPRB, 0.53815E+01_JPRB, 0.59870E+01_JPRB, 0.62191E+01_JPRB ,&
&  0.67380E+01_JPRB, 0.67709E+01_JPRB, 0.77124E+01_JPRB, 0.65290E+01_JPRB, 0.57719E+01_JPRB, 0.54892E+01_JPRB, 0.49532E+01_JPRB ,&
&  0.43942E+01_JPRB, 0.42056E+01_JPRB, 0.38390E+01_JPRB, 0.29960E+01_JPRB, 0.32430E+01_JPRB, 0.31750E+01_JPRB, 0.41442E+01_JPRB ,&
&  0.47915E+01_JPRB, 0.52684E+01_JPRB, 0.32508E+01_JPRB, 0.25056E+01_JPRB, 0.22401E+01_JPRB, 0.24684E+01_JPRB, 0.25372E+01_JPRB ,&
&  0.24050E+01_JPRB, 0.21403E+01_JPRB, 0.19800E+01_JPRB, 0.18273E+01_JPRB, 0.12145E+01_JPRB, 0.10512E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.11357E+01_JPRB, 0.11470E+01_JPRB, 0.11309E+01_JPRB, 0.11443E+01_JPRB, 0.12294E+01_JPRB, 0.12846E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.14212E+01_JPRB, 0.14872E+01_JPRB, 0.15266E+01_JPRB, 0.15809E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.17284E+01_JPRB, 0.18529E+01_JPRB, 0.20488E+01_JPRB, 0.22338E+01_JPRB, 0.23873E+01_JPRB, 0.24839E+01_JPRB, 0.25374E+01_JPRB ,&
&  0.26302E+01_JPRB, 0.27692E+01_JPRB, 0.29704E+01_JPRB, 0.31602E+01_JPRB, 0.33852E+01_JPRB, 0.36169E+01_JPRB, 0.38468E+01_JPRB ,&
&  0.40594E+01_JPRB, 0.42525E+01_JPRB, 0.44916E+01_JPRB, 0.47921E+01_JPRB, 0.51429E+01_JPRB, 0.56879E+01_JPRB, 0.57093E+01_JPRB ,&
&  0.44445E+01_JPRB, 0.26398E+01_JPRB, 0.28742E+01_JPRB, 0.29007E+01_JPRB, 0.29841E+01_JPRB, 0.33370E+01_JPRB, 0.34094E+01_JPRB ,&
&  0.29315E+01_JPRB, 0.28575E+01_JPRB, 0.29404E+01_JPRB, 0.29229E+01_JPRB, 0.28363E+01_JPRB, 0.29840E+01_JPRB, 0.31120E+01_JPRB ,&
&  0.32681E+01_JPRB, 0.34107E+01_JPRB, 0.35481E+01_JPRB, 0.37019E+01_JPRB, 0.38468E+01_JPRB, 0.40094E+01_JPRB, 0.41942E+01_JPRB ,&
&  0.43877E+01_JPRB, 0.45315E+01_JPRB, 0.45119E+01_JPRB, 0.44799E+01_JPRB, 0.45271E+01_JPRB, 0.46794E+01_JPRB, 0.48242E+01_JPRB ,&
&  0.49572E+01_JPRB, 0.50450E+01_JPRB, 0.54323E+01_JPRB, 0.58462E+01_JPRB, 0.60299E+01_JPRB, 0.57993E+01_JPRB, 0.60286E+01_JPRB ,&
&  0.61612E+01_JPRB, 0.59892E+01_JPRB, 0.63086E+01_JPRB, 0.68932E+01_JPRB, 0.49191E+01_JPRB, 0.47867E+01_JPRB, 0.39978E+01_JPRB ,&
&  0.40805E+01_JPRB, 0.37786E+01_JPRB, 0.33562E+01_JPRB, 0.28731E+01_JPRB, 0.26354E+01_JPRB, 0.26244E+01_JPRB, 0.33871E+01_JPRB ,&
&  0.42886E+01_JPRB, 0.37892E+01_JPRB, 0.26568E+01_JPRB, 0.22629E+01_JPRB, 0.22553E+01_JPRB, 0.27803E+01_JPRB, 0.27427E+01_JPRB ,&
&  0.23091E+01_JPRB, 0.19632E+01_JPRB, 0.18459E+01_JPRB, 0.22932E+01_JPRB, 0.14243E+01_JPRB, 0.10039E+01_JPRB, 0.92626E+00_JPRB ,&
&  0.10019E+01_JPRB, 0.10776E+01_JPRB, 0.11451E+01_JPRB, 0.11952E+01_JPRB, 0.12420E+01_JPRB, 0.13125E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.14585E+01_JPRB, 0.15427E+01_JPRB, 0.15906E+01_JPRB, 0.16521E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.17036E+01_JPRB, 0.18002E+01_JPRB, 0.19033E+01_JPRB, 0.20222E+01_JPRB, 0.21507E+01_JPRB, 0.22524E+01_JPRB, 0.23191E+01_JPRB ,&
&  0.24015E+01_JPRB, 0.25127E+01_JPRB, 0.26457E+01_JPRB, 0.27594E+01_JPRB, 0.29276E+01_JPRB, 0.30855E+01_JPRB, 0.32433E+01_JPRB ,&
&  0.33990E+01_JPRB, 0.35725E+01_JPRB, 0.37907E+01_JPRB, 0.39748E+01_JPRB, 0.40735E+01_JPRB, 0.41577E+01_JPRB, 0.42494E+01_JPRB ,&
&  0.43247E+01_JPRB, 0.36402E+01_JPRB, 0.31525E+01_JPRB, 0.24202E+01_JPRB, 0.19946E+01_JPRB, 0.25060E+01_JPRB, 0.24566E+01_JPRB ,&
&  0.24316E+01_JPRB, 0.23817E+01_JPRB, 0.23987E+01_JPRB, 0.24834E+01_JPRB, 0.26264E+01_JPRB, 0.27823E+01_JPRB, 0.29174E+01_JPRB ,&
&  0.30332E+01_JPRB, 0.31259E+01_JPRB, 0.32044E+01_JPRB, 0.32975E+01_JPRB, 0.34096E+01_JPRB, 0.35164E+01_JPRB, 0.36262E+01_JPRB ,&
&  0.37751E+01_JPRB, 0.39436E+01_JPRB, 0.40578E+01_JPRB, 0.41917E+01_JPRB, 0.42571E+01_JPRB, 0.44745E+01_JPRB, 0.47083E+01_JPRB ,&
&  0.48722E+01_JPRB, 0.49854E+01_JPRB, 0.53030E+01_JPRB, 0.56504E+01_JPRB, 0.57006E+01_JPRB, 0.51695E+01_JPRB, 0.52682E+01_JPRB ,&
&  0.54120E+01_JPRB, 0.51712E+01_JPRB, 0.50609E+01_JPRB, 0.64318E+01_JPRB, 0.40863E+01_JPRB, 0.43666E+01_JPRB, 0.42539E+01_JPRB ,&
&  0.37008E+01_JPRB, 0.34797E+01_JPRB, 0.30296E+01_JPRB, 0.23941E+01_JPRB, 0.21402E+01_JPRB, 0.21895E+01_JPRB, 0.28303E+01_JPRB ,&
&  0.32848E+01_JPRB, 0.26940E+01_JPRB, 0.19891E+01_JPRB, 0.17587E+01_JPRB, 0.19714E+01_JPRB, 0.27527E+01_JPRB, 0.28446E+01_JPRB ,&
&  0.21752E+01_JPRB, 0.19565E+01_JPRB, 0.17659E+01_JPRB, 0.27857E+01_JPRB, 0.20542E+01_JPRB, 0.98476E+00_JPRB, 0.80778E+00_JPRB ,&
&  0.84110E+00_JPRB, 0.91806E+00_JPRB, 0.10334E+01_JPRB, 0.11157E+01_JPRB, 0.11572E+01_JPRB, 0.12094E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.13104E+01_JPRB, 0.13938E+01_JPRB, 0.14909E+01_JPRB, 0.15614E+01_JPRB, 0.16308E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.13301E+01_JPRB, 0.14055E+01_JPRB, 0.14657E+01_JPRB, 0.15006E+01_JPRB, 0.15231E+01_JPRB, 0.15247E+01_JPRB, 0.14938E+01_JPRB ,&
&  0.14582E+01_JPRB, 0.15020E+01_JPRB, 0.16261E+01_JPRB, 0.18997E+01_JPRB, 0.21376E+01_JPRB, 0.22930E+01_JPRB, 0.24382E+01_JPRB ,&
&  0.25780E+01_JPRB, 0.26271E+01_JPRB, 0.25488E+01_JPRB, 0.25113E+01_JPRB, 0.24000E+01_JPRB, 0.22679E+01_JPRB, 0.24601E+01_JPRB ,&
&  0.28347E+01_JPRB, 0.32490E+01_JPRB, 0.34704E+01_JPRB, 0.36818E+01_JPRB, 0.29471E+01_JPRB, 0.24675E+01_JPRB, 0.27505E+01_JPRB ,&
&  0.30590E+01_JPRB, 0.26107E+01_JPRB, 0.20681E+01_JPRB, 0.20546E+01_JPRB, 0.20868E+01_JPRB, 0.21839E+01_JPRB, 0.23467E+01_JPRB ,&
&  0.24578E+01_JPRB, 0.25343E+01_JPRB, 0.26062E+01_JPRB, 0.26668E+01_JPRB, 0.27281E+01_JPRB, 0.27866E+01_JPRB, 0.28586E+01_JPRB ,&
&  0.29091E+01_JPRB, 0.29461E+01_JPRB, 0.30872E+01_JPRB, 0.33212E+01_JPRB, 0.35693E+01_JPRB, 0.38874E+01_JPRB, 0.41517E+01_JPRB ,&
&  0.43119E+01_JPRB, 0.44509E+01_JPRB, 0.49449E+01_JPRB, 0.52602E+01_JPRB, 0.49884E+01_JPRB, 0.45576E+01_JPRB, 0.46602E+01_JPRB ,&
&  0.46135E+01_JPRB, 0.42703E+01_JPRB, 0.34501E+01_JPRB, 0.49752E+01_JPRB, 0.50262E+01_JPRB, 0.43502E+01_JPRB, 0.36343E+01_JPRB ,&
&  0.32364E+01_JPRB, 0.30600E+01_JPRB, 0.27331E+01_JPRB, 0.23154E+01_JPRB, 0.19635E+01_JPRB, 0.24227E+01_JPRB, 0.27196E+01_JPRB ,&
&  0.27752E+01_JPRB, 0.21740E+01_JPRB, 0.15933E+01_JPRB, 0.14281E+01_JPRB, 0.15211E+01_JPRB, 0.23425E+01_JPRB, 0.23298E+01_JPRB ,&
&  0.18086E+01_JPRB, 0.16308E+01_JPRB, 0.15069E+01_JPRB, 0.22235E+01_JPRB, 0.33773E+01_JPRB, 0.10305E+01_JPRB, 0.70469E+00_JPRB ,&
&  0.70823E+00_JPRB, 0.74529E+00_JPRB, 0.78597E+00_JPRB, 0.79999E+00_JPRB, 0.80530E+00_JPRB, 0.84756E+00_JPRB, 0.91639E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.10787E+01_JPRB, 0.11322E+01_JPRB, 0.12055E+01_JPRB, 0.12666E+01_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.90808E+00_JPRB, 0.94459E+00_JPRB, 0.95284E+00_JPRB, 0.96813E+00_JPRB, 0.96460E+00_JPRB, 0.96437E+00_JPRB, 0.93081E+00_JPRB ,&
&  0.91098E+00_JPRB, 0.94175E+00_JPRB, 0.10305E+01_JPRB, 0.11934E+01_JPRB, 0.12549E+01_JPRB, 0.13030E+01_JPRB, 0.13393E+01_JPRB ,&
&  0.13854E+01_JPRB, 0.13109E+01_JPRB, 0.12543E+01_JPRB, 0.11584E+01_JPRB, 0.11147E+01_JPRB, 0.11312E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.14189E+01_JPRB, 0.16592E+01_JPRB, 0.19271E+01_JPRB, 0.22506E+01_JPRB, 0.30244E+01_JPRB, 0.27808E+01_JPRB, 0.28235E+01_JPRB ,&
&  0.33486E+01_JPRB, 0.24424E+01_JPRB, 0.19612E+01_JPRB, 0.16835E+01_JPRB, 0.15607E+01_JPRB, 0.15067E+01_JPRB, 0.15662E+01_JPRB ,&
&  0.15997E+01_JPRB, 0.16582E+01_JPRB, 0.17286E+01_JPRB, 0.17559E+01_JPRB, 0.17789E+01_JPRB, 0.17497E+01_JPRB, 0.17032E+01_JPRB ,&
&  0.16352E+01_JPRB, 0.17173E+01_JPRB, 0.18875E+01_JPRB, 0.20569E+01_JPRB, 0.24323E+01_JPRB, 0.27249E+01_JPRB, 0.29640E+01_JPRB ,&
&  0.30899E+01_JPRB, 0.32956E+01_JPRB, 0.37220E+01_JPRB, 0.38529E+01_JPRB, 0.39253E+01_JPRB, 0.37353E+01_JPRB, 0.35915E+01_JPRB ,&
&  0.33432E+01_JPRB, 0.27168E+01_JPRB, 0.23058E+01_JPRB, 0.34654E+01_JPRB, 0.36166E+01_JPRB, 0.29314E+01_JPRB, 0.30520E+01_JPRB ,&
&  0.28201E+01_JPRB, 0.26595E+01_JPRB, 0.25472E+01_JPRB, 0.21876E+01_JPRB, 0.17855E+01_JPRB, 0.20113E+01_JPRB, 0.25223E+01_JPRB ,&
&  0.22219E+01_JPRB, 0.16149E+01_JPRB, 0.12541E+01_JPRB, 0.12109E+01_JPRB, 0.13369E+01_JPRB, 0.18861E+01_JPRB, 0.20558E+01_JPRB ,&
&  0.18799E+01_JPRB, 0.14070E+01_JPRB, 0.16127E+01_JPRB, 0.25420E+01_JPRB, 0.18323E+01_JPRB, 0.90739E+00_JPRB, 0.59901E+00_JPRB ,&
&  0.57583E+00_JPRB, 0.59005E+00_JPRB, 0.61060E+00_JPRB, 0.61968E+00_JPRB, 0.62484E+00_JPRB, 0.63811E+00_JPRB, 0.68386E+00_JPRB ,&
&  0.72536E+00_JPRB, 0.76866E+00_JPRB, 0.80469E+00_JPRB, 0.85336E+00_JPRB, 0.88465E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.84280E+00_JPRB, 0.86517E+00_JPRB, 0.89252E+00_JPRB, 0.91048E+00_JPRB, 0.93417E+00_JPRB, 0.95613E+00_JPRB, 0.10059E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10988E+01_JPRB, 0.11566E+01_JPRB, 0.12183E+01_JPRB, 0.12377E+01_JPRB, 0.12555E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.12956E+01_JPRB, 0.13096E+01_JPRB, 0.12899E+01_JPRB, 0.12814E+01_JPRB, 0.12703E+01_JPRB, 0.13541E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.13200E+01_JPRB, 0.13381E+01_JPRB, 0.13490E+01_JPRB, 0.14228E+01_JPRB, 0.16581E+01_JPRB, 0.19083E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.26021E+01_JPRB, 0.20124E+01_JPRB, 0.17256E+01_JPRB, 0.12743E+01_JPRB, 0.11782E+01_JPRB, 0.11769E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.11349E+01_JPRB, 0.11952E+01_JPRB, 0.11907E+01_JPRB, 0.12356E+01_JPRB, 0.12626E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.13095E+01_JPRB, 0.13685E+01_JPRB, 0.14419E+01_JPRB, 0.15890E+01_JPRB, 0.17107E+01_JPRB, 0.19811E+01_JPRB, 0.20937E+01_JPRB ,&
&  0.21298E+01_JPRB, 0.22225E+01_JPRB, 0.21031E+01_JPRB, 0.21602E+01_JPRB, 0.24326E+01_JPRB, 0.24415E+01_JPRB, 0.23623E+01_JPRB ,&
&  0.23940E+01_JPRB, 0.21693E+01_JPRB, 0.16967E+01_JPRB, 0.25144E+01_JPRB, 0.28148E+01_JPRB, 0.26149E+01_JPRB, 0.25735E+01_JPRB ,&
&  0.24033E+01_JPRB, 0.21585E+01_JPRB, 0.19783E+01_JPRB, 0.18881E+01_JPRB, 0.16524E+01_JPRB, 0.15658E+01_JPRB, 0.17622E+01_JPRB ,&
&  0.15602E+01_JPRB, 0.13022E+01_JPRB, 0.11804E+01_JPRB, 0.12758E+01_JPRB, 0.17076E+01_JPRB, 0.28216E+01_JPRB, 0.25240E+01_JPRB ,&
&  0.17269E+01_JPRB, 0.13887E+01_JPRB, 0.20545E+01_JPRB, 0.30616E+01_JPRB, 0.16157E+01_JPRB, 0.95313E+00_JPRB, 0.58398E+00_JPRB ,&
&  0.53383E+00_JPRB, 0.55676E+00_JPRB, 0.58499E+00_JPRB, 0.60895E+00_JPRB, 0.63858E+00_JPRB, 0.66606E+00_JPRB, 0.68920E+00_JPRB ,&
&  0.71165E+00_JPRB, 0.74056E+00_JPRB, 0.76712E+00_JPRB, 0.79996E+00_JPRB, 0.82947E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.99675E+00_JPRB, 0.10502E+01_JPRB, 0.10910E+01_JPRB, 0.11253E+01_JPRB, 0.11564E+01_JPRB, 0.11887E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.12997E+01_JPRB, 0.13323E+01_JPRB, 0.13734E+01_JPRB, 0.14247E+01_JPRB, 0.14717E+01_JPRB, 0.14916E+01_JPRB ,&
&  0.14965E+01_JPRB, 0.15454E+01_JPRB, 0.15673E+01_JPRB, 0.16307E+01_JPRB, 0.16512E+01_JPRB, 0.16942E+01_JPRB, 0.16956E+01_JPRB ,&
&  0.16810E+01_JPRB, 0.16732E+01_JPRB, 0.16763E+01_JPRB, 0.16911E+01_JPRB, 0.16990E+01_JPRB, 0.18261E+01_JPRB, 0.20207E+01_JPRB ,&
&  0.20453E+01_JPRB, 0.19764E+01_JPRB, 0.14073E+01_JPRB, 0.11361E+01_JPRB, 0.11264E+01_JPRB, 0.11397E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.11191E+01_JPRB, 0.11444E+01_JPRB, 0.11463E+01_JPRB, 0.12141E+01_JPRB, 0.13428E+01_JPRB, 0.13843E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.15248E+01_JPRB, 0.15599E+01_JPRB, 0.17016E+01_JPRB, 0.17102E+01_JPRB, 0.20151E+01_JPRB, 0.21638E+01_JPRB ,&
&  0.21958E+01_JPRB, 0.20861E+01_JPRB, 0.16465E+01_JPRB, 0.16574E+01_JPRB, 0.18162E+01_JPRB, 0.17774E+01_JPRB, 0.17261E+01_JPRB ,&
&  0.19947E+01_JPRB, 0.20104E+01_JPRB, 0.18119E+01_JPRB, 0.22873E+01_JPRB, 0.24333E+01_JPRB, 0.23782E+01_JPRB, 0.21120E+01_JPRB ,&
&  0.18603E+01_JPRB, 0.14783E+01_JPRB, 0.14638E+01_JPRB, 0.14813E+01_JPRB, 0.13884E+01_JPRB, 0.13019E+01_JPRB, 0.12688E+01_JPRB ,&
&  0.12097E+01_JPRB, 0.11579E+01_JPRB, 0.12020E+01_JPRB, 0.14070E+01_JPRB, 0.22618E+01_JPRB, 0.27286E+01_JPRB, 0.32308E+01_JPRB ,&
&  0.14390E+01_JPRB, 0.13974E+01_JPRB, 0.11034E+01_JPRB, 0.14263E+01_JPRB, 0.11236E+01_JPRB, 0.70165E+00_JPRB, 0.57157E+00_JPRB ,&
&  0.59772E+00_JPRB, 0.64656E+00_JPRB, 0.68900E+00_JPRB, 0.72238E+00_JPRB, 0.75412E+00_JPRB, 0.78782E+00_JPRB, 0.80798E+00_JPRB ,&
&  0.83268E+00_JPRB, 0.86100E+00_JPRB, 0.88862E+00_JPRB, 0.92850E+00_JPRB, 0.95756E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.10876E+01_JPRB, 0.11322E+01_JPRB, 0.11719E+01_JPRB, 0.12099E+01_JPRB, 0.12519E+01_JPRB, 0.12977E+01_JPRB, 0.13400E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14140E+01_JPRB, 0.14471E+01_JPRB, 0.14792E+01_JPRB, 0.15218E+01_JPRB, 0.15640E+01_JPRB, 0.15915E+01_JPRB ,&
&  0.16199E+01_JPRB, 0.16684E+01_JPRB, 0.17176E+01_JPRB, 0.17658E+01_JPRB, 0.17909E+01_JPRB, 0.18160E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.17946E+01_JPRB, 0.17859E+01_JPRB, 0.18031E+01_JPRB, 0.18332E+01_JPRB, 0.18805E+01_JPRB, 0.18849E+01_JPRB, 0.20457E+01_JPRB ,&
&  0.18677E+01_JPRB, 0.20231E+01_JPRB, 0.15541E+01_JPRB, 0.13472E+01_JPRB, 0.13548E+01_JPRB, 0.12672E+01_JPRB, 0.12516E+01_JPRB ,&
&  0.12746E+01_JPRB, 0.12568E+01_JPRB, 0.12611E+01_JPRB, 0.12659E+01_JPRB, 0.13208E+01_JPRB, 0.14188E+01_JPRB, 0.14859E+01_JPRB ,&
&  0.15952E+01_JPRB, 0.17243E+01_JPRB, 0.18240E+01_JPRB, 0.19445E+01_JPRB, 0.22009E+01_JPRB, 0.24140E+01_JPRB, 0.25158E+01_JPRB ,&
&  0.24856E+01_JPRB, 0.23726E+01_JPRB, 0.18325E+01_JPRB, 0.18935E+01_JPRB, 0.20059E+01_JPRB, 0.18826E+01_JPRB, 0.17351E+01_JPRB ,&
&  0.18178E+01_JPRB, 0.16120E+01_JPRB, 0.18532E+01_JPRB, 0.19248E+01_JPRB, 0.18531E+01_JPRB, 0.16574E+01_JPRB, 0.13877E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.96398E+00_JPRB, 0.98898E+00_JPRB, 0.11308E+01_JPRB, 0.10959E+01_JPRB, 0.94515E+00_JPRB, 0.95033E+00_JPRB ,&
&  0.94235E+00_JPRB, 0.92776E+00_JPRB, 0.10043E+01_JPRB, 0.13061E+01_JPRB, 0.19285E+01_JPRB, 0.33488E+01_JPRB, 0.30508E+01_JPRB ,&
&  0.13461E+01_JPRB, 0.12010E+01_JPRB, 0.69829E+00_JPRB, 0.73239E+00_JPRB, 0.75137E+00_JPRB, 0.62086E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.72749E+00_JPRB, 0.83417E+00_JPRB, 0.85932E+00_JPRB, 0.81594E+00_JPRB, 0.81098E+00_JPRB, 0.83331E+00_JPRB, 0.86584E+00_JPRB ,&
&  0.89944E+00_JPRB, 0.93192E+00_JPRB, 0.96595E+00_JPRB, 0.10037E+01_JPRB, 0.10434E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.11277E+01_JPRB, 0.11729E+01_JPRB, 0.12135E+01_JPRB, 0.12507E+01_JPRB, 0.12956E+01_JPRB, 0.13449E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.14363E+01_JPRB, 0.14769E+01_JPRB, 0.15213E+01_JPRB, 0.15643E+01_JPRB, 0.16073E+01_JPRB, 0.16535E+01_JPRB, 0.17089E+01_JPRB ,&
&  0.17608E+01_JPRB, 0.18003E+01_JPRB, 0.18628E+01_JPRB, 0.19359E+01_JPRB, 0.19866E+01_JPRB, 0.20202E+01_JPRB, 0.20372E+01_JPRB ,&
&  0.20560E+01_JPRB, 0.21081E+01_JPRB, 0.21409E+01_JPRB, 0.21663E+01_JPRB, 0.22143E+01_JPRB, 0.24147E+01_JPRB, 0.21808E+01_JPRB ,&
&  0.23710E+01_JPRB, 0.20080E+01_JPRB, 0.16609E+01_JPRB, 0.15485E+01_JPRB, 0.15167E+01_JPRB, 0.14968E+01_JPRB, 0.14840E+01_JPRB ,&
&  0.14936E+01_JPRB, 0.14074E+01_JPRB, 0.13752E+01_JPRB, 0.13609E+01_JPRB, 0.14002E+01_JPRB, 0.14810E+01_JPRB, 0.15726E+01_JPRB ,&
&  0.17211E+01_JPRB, 0.19132E+01_JPRB, 0.21362E+01_JPRB, 0.23758E+01_JPRB, 0.26632E+01_JPRB, 0.28999E+01_JPRB, 0.30452E+01_JPRB ,&
&  0.30691E+01_JPRB, 0.30284E+01_JPRB, 0.25881E+01_JPRB, 0.27180E+01_JPRB, 0.26999E+01_JPRB, 0.22844E+01_JPRB, 0.19179E+01_JPRB ,&
&  0.16324E+01_JPRB, 0.13372E+01_JPRB, 0.14053E+01_JPRB, 0.14672E+01_JPRB, 0.13452E+01_JPRB, 0.11476E+01_JPRB, 0.96382E+00_JPRB ,&
&  0.75719E+00_JPRB, 0.68827E+00_JPRB, 0.69040E+00_JPRB, 0.81037E+00_JPRB, 0.81310E+00_JPRB, 0.73558E+00_JPRB, 0.76226E+00_JPRB ,&
&  0.78029E+00_JPRB, 0.79307E+00_JPRB, 0.89078E+00_JPRB, 0.11600E+01_JPRB, 0.15708E+01_JPRB, 0.15862E+01_JPRB, 0.16318E+01_JPRB ,&
&  0.11028E+01_JPRB, 0.93422E+00_JPRB, 0.86198E+00_JPRB, 0.72003E+00_JPRB, 0.74984E+00_JPRB, 0.74568E+00_JPRB, 0.84151E+00_JPRB ,&
&  0.10030E+01_JPRB, 0.12207E+01_JPRB, 0.11953E+01_JPRB, 0.99828E+00_JPRB, 0.85797E+00_JPRB, 0.84129E+00_JPRB, 0.87413E+00_JPRB ,&
&  0.91078E+00_JPRB, 0.94503E+00_JPRB, 0.98682E+00_JPRB, 0.10322E+01_JPRB, 0.10790E+01_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.11332E+01_JPRB, 0.11763E+01_JPRB, 0.12135E+01_JPRB, 0.12467E+01_JPRB, 0.12917E+01_JPRB, 0.13392E+01_JPRB, 0.13887E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.14857E+01_JPRB, 0.15381E+01_JPRB, 0.15892E+01_JPRB, 0.16466E+01_JPRB, 0.17067E+01_JPRB, 0.17720E+01_JPRB ,&
&  0.18342E+01_JPRB, 0.18946E+01_JPRB, 0.19506E+01_JPRB, 0.20520E+01_JPRB, 0.21658E+01_JPRB, 0.22504E+01_JPRB, 0.23105E+01_JPRB ,&
&  0.23405E+01_JPRB, 0.24059E+01_JPRB, 0.24846E+01_JPRB, 0.25488E+01_JPRB, 0.25676E+01_JPRB, 0.27371E+01_JPRB, 0.24739E+01_JPRB ,&
&  0.23883E+01_JPRB, 0.20894E+01_JPRB, 0.18271E+01_JPRB, 0.17647E+01_JPRB, 0.17170E+01_JPRB, 0.16978E+01_JPRB, 0.16716E+01_JPRB ,&
&  0.16327E+01_JPRB, 0.15843E+01_JPRB, 0.15090E+01_JPRB, 0.14636E+01_JPRB, 0.14645E+01_JPRB, 0.15284E+01_JPRB, 0.16397E+01_JPRB ,&
&  0.17947E+01_JPRB, 0.20078E+01_JPRB, 0.22608E+01_JPRB, 0.25355E+01_JPRB, 0.28491E+01_JPRB, 0.31278E+01_JPRB, 0.33836E+01_JPRB ,&
&  0.35761E+01_JPRB, 0.36885E+01_JPRB, 0.33953E+01_JPRB, 0.33548E+01_JPRB, 0.31898E+01_JPRB, 0.22949E+01_JPRB, 0.17341E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12513E+01_JPRB, 0.12889E+01_JPRB, 0.12503E+01_JPRB, 0.10209E+01_JPRB, 0.88404E+00_JPRB, 0.72887E+00_JPRB ,&
&  0.60793E+00_JPRB, 0.56377E+00_JPRB, 0.57369E+00_JPRB, 0.64091E+00_JPRB, 0.63380E+00_JPRB, 0.60550E+00_JPRB, 0.64916E+00_JPRB ,&
&  0.67644E+00_JPRB, 0.74808E+00_JPRB, 0.86427E+00_JPRB, 0.10485E+01_JPRB, 0.12317E+01_JPRB, 0.12362E+01_JPRB, 0.12688E+01_JPRB ,&
&  0.10641E+01_JPRB, 0.10354E+01_JPRB, 0.90786E+00_JPRB, 0.66860E+00_JPRB, 0.82915E+00_JPRB, 0.94135E+00_JPRB, 0.10772E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.16120E+01_JPRB, 0.18307E+01_JPRB, 0.14462E+01_JPRB, 0.11540E+01_JPRB, 0.88855E+00_JPRB, 0.87227E+00_JPRB ,&
&  0.91307E+00_JPRB, 0.93599E+00_JPRB, 0.97785E+00_JPRB, 0.10284E+01_JPRB, 0.10825E+01_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.10789E+01_JPRB, 0.11260E+01_JPRB, 0.11582E+01_JPRB, 0.11735E+01_JPRB, 0.12115E+01_JPRB, 0.12697E+01_JPRB, 0.13148E+01_JPRB ,&
&  0.13634E+01_JPRB, 0.14061E+01_JPRB, 0.14613E+01_JPRB, 0.15136E+01_JPRB, 0.15697E+01_JPRB, 0.16281E+01_JPRB, 0.16954E+01_JPRB ,&
&  0.17588E+01_JPRB, 0.18571E+01_JPRB, 0.19276E+01_JPRB, 0.20437E+01_JPRB, 0.21700E+01_JPRB, 0.23223E+01_JPRB, 0.24162E+01_JPRB ,&
&  0.24494E+01_JPRB, 0.25301E+01_JPRB, 0.26263E+01_JPRB, 0.27351E+01_JPRB, 0.27387E+01_JPRB, 0.27679E+01_JPRB, 0.21244E+01_JPRB ,&
&  0.22586E+01_JPRB, 0.21431E+01_JPRB, 0.20211E+01_JPRB, 0.19596E+01_JPRB, 0.18822E+01_JPRB, 0.18121E+01_JPRB, 0.17395E+01_JPRB ,&
&  0.17051E+01_JPRB, 0.16341E+01_JPRB, 0.15036E+01_JPRB, 0.15133E+01_JPRB, 0.15116E+01_JPRB, 0.15356E+01_JPRB, 0.16527E+01_JPRB ,&
&  0.17770E+01_JPRB, 0.19639E+01_JPRB, 0.22041E+01_JPRB, 0.24898E+01_JPRB, 0.28005E+01_JPRB, 0.31068E+01_JPRB, 0.34075E+01_JPRB ,&
&  0.36882E+01_JPRB, 0.39121E+01_JPRB, 0.39495E+01_JPRB, 0.32765E+01_JPRB, 0.27858E+01_JPRB, 0.20547E+01_JPRB, 0.16233E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.12618E+01_JPRB, 0.12266E+01_JPRB, 0.11547E+01_JPRB, 0.90955E+00_JPRB, 0.76356E+00_JPRB, 0.62881E+00_JPRB ,&
&  0.57677E+00_JPRB, 0.54071E+00_JPRB, 0.54283E+00_JPRB, 0.54845E+00_JPRB, 0.55367E+00_JPRB, 0.55615E+00_JPRB, 0.58988E+00_JPRB ,&
&  0.62703E+00_JPRB, 0.70550E+00_JPRB, 0.84802E+00_JPRB, 0.10515E+01_JPRB, 0.12417E+01_JPRB, 0.13327E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.91374E+00_JPRB, 0.85898E+00_JPRB, 0.88131E+00_JPRB, 0.91268E+00_JPRB, 0.10935E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.15655E+01_JPRB, 0.15851E+01_JPRB, 0.18617E+01_JPRB, 0.24148E+01_JPRB, 0.21766E+01_JPRB, 0.18944E+01_JPRB, 0.79089E+00_JPRB ,&
&  0.84972E+00_JPRB, 0.87589E+00_JPRB, 0.92733E+00_JPRB, 0.97967E+00_JPRB, 0.10439E+01_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.86069E+00_JPRB, 0.90434E+00_JPRB, 0.94392E+00_JPRB, 0.96367E+00_JPRB, 0.10178E+01_JPRB, 0.10780E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.11676E+01_JPRB, 0.12276E+01_JPRB, 0.12791E+01_JPRB, 0.13432E+01_JPRB, 0.13988E+01_JPRB, 0.14540E+01_JPRB, 0.15128E+01_JPRB ,&
&  0.15709E+01_JPRB, 0.16555E+01_JPRB, 0.17426E+01_JPRB, 0.18636E+01_JPRB, 0.19914E+01_JPRB, 0.21488E+01_JPRB, 0.22943E+01_JPRB ,&
&  0.24064E+01_JPRB, 0.25186E+01_JPRB, 0.26505E+01_JPRB, 0.27405E+01_JPRB, 0.27128E+01_JPRB, 0.26519E+01_JPRB, 0.19222E+01_JPRB ,&
&  0.21726E+01_JPRB, 0.22080E+01_JPRB, 0.21902E+01_JPRB, 0.21266E+01_JPRB, 0.20285E+01_JPRB, 0.19138E+01_JPRB, 0.18354E+01_JPRB ,&
&  0.17628E+01_JPRB, 0.16523E+01_JPRB, 0.15237E+01_JPRB, 0.14232E+01_JPRB, 0.15201E+01_JPRB, 0.15331E+01_JPRB, 0.16231E+01_JPRB ,&
&  0.17488E+01_JPRB, 0.19090E+01_JPRB, 0.21056E+01_JPRB, 0.23845E+01_JPRB, 0.26936E+01_JPRB, 0.29983E+01_JPRB, 0.33100E+01_JPRB ,&
&  0.35479E+01_JPRB, 0.36678E+01_JPRB, 0.37034E+01_JPRB, 0.29894E+01_JPRB, 0.25138E+01_JPRB, 0.20345E+01_JPRB, 0.15469E+01_JPRB ,&
&  0.13254E+01_JPRB, 0.12405E+01_JPRB, 0.12205E+01_JPRB, 0.11056E+01_JPRB, 0.89316E+00_JPRB, 0.72329E+00_JPRB, 0.60418E+00_JPRB ,&
&  0.55810E+00_JPRB, 0.52020E+00_JPRB, 0.52069E+00_JPRB, 0.52994E+00_JPRB, 0.54774E+00_JPRB, 0.56379E+00_JPRB, 0.59318E+00_JPRB ,&
&  0.63488E+00_JPRB, 0.69814E+00_JPRB, 0.80174E+00_JPRB, 0.97147E+00_JPRB, 0.12104E+01_JPRB, 0.14720E+01_JPRB, 0.14160E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.92050E+00_JPRB, 0.84008E+00_JPRB, 0.80023E+00_JPRB, 0.80844E+00_JPRB, 0.86303E+00_JPRB, 0.93472E+00_JPRB ,&
&  0.10675E+01_JPRB, 0.11419E+01_JPRB, 0.10224E+01_JPRB, 0.85384E+00_JPRB, 0.11065E+01_JPRB, 0.11040E+01_JPRB, 0.63864E+00_JPRB ,&
&  0.70118E+00_JPRB, 0.69131E+00_JPRB, 0.72907E+00_JPRB, 0.77474E+00_JPRB, 0.83013E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.65150E+00_JPRB, 0.71735E+00_JPRB, 0.75159E+00_JPRB, 0.79395E+00_JPRB, 0.83155E+00_JPRB, 0.88571E+00_JPRB, 0.93893E+00_JPRB ,&
&  0.99368E+00_JPRB, 0.10549E+01_JPRB, 0.11047E+01_JPRB, 0.11664E+01_JPRB, 0.12220E+01_JPRB, 0.12733E+01_JPRB, 0.13246E+01_JPRB ,&
&  0.13735E+01_JPRB, 0.14455E+01_JPRB, 0.15196E+01_JPRB, 0.16051E+01_JPRB, 0.16977E+01_JPRB, 0.18192E+01_JPRB, 0.19878E+01_JPRB ,&
&  0.21917E+01_JPRB, 0.23910E+01_JPRB, 0.25721E+01_JPRB, 0.27246E+01_JPRB, 0.28088E+01_JPRB, 0.29235E+01_JPRB, 0.29555E+01_JPRB ,&
&  0.19535E+01_JPRB, 0.24033E+01_JPRB, 0.23116E+01_JPRB, 0.21851E+01_JPRB, 0.20833E+01_JPRB, 0.19830E+01_JPRB, 0.19244E+01_JPRB ,&
&  0.18460E+01_JPRB, 0.16577E+01_JPRB, 0.15679E+01_JPRB, 0.14482E+01_JPRB, 0.14594E+01_JPRB, 0.14919E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.16875E+01_JPRB, 0.18490E+01_JPRB, 0.20252E+01_JPRB, 0.22152E+01_JPRB, 0.24780E+01_JPRB, 0.28039E+01_JPRB, 0.30976E+01_JPRB ,&
&  0.34034E+01_JPRB, 0.37145E+01_JPRB, 0.38986E+01_JPRB, 0.33040E+01_JPRB, 0.27530E+01_JPRB, 0.24763E+01_JPRB, 0.18801E+01_JPRB ,&
&  0.13517E+01_JPRB, 0.11476E+01_JPRB, 0.11479E+01_JPRB, 0.10961E+01_JPRB, 0.89757E+00_JPRB, 0.74656E+00_JPRB, 0.64155E+00_JPRB ,&
&  0.59425E+00_JPRB, 0.55084E+00_JPRB, 0.53973E+00_JPRB, 0.54882E+00_JPRB, 0.56492E+00_JPRB, 0.58613E+00_JPRB, 0.62984E+00_JPRB ,&
&  0.66393E+00_JPRB, 0.69631E+00_JPRB, 0.73556E+00_JPRB, 0.77557E+00_JPRB, 0.85131E+00_JPRB, 0.97326E+00_JPRB, 0.98750E+00_JPRB ,&
&  0.93804E+00_JPRB, 0.89262E+00_JPRB, 0.84372E+00_JPRB, 0.82414E+00_JPRB, 0.78704E+00_JPRB, 0.76868E+00_JPRB, 0.77720E+00_JPRB ,&
&  0.79525E+00_JPRB, 0.80544E+00_JPRB, 0.75772E+00_JPRB, 0.71851E+00_JPRB, 0.70105E+00_JPRB, 0.66837E+00_JPRB, 0.61525E+00_JPRB ,&
&  0.59659E+00_JPRB, 0.58044E+00_JPRB, 0.56293E+00_JPRB, 0.57570E+00_JPRB, 0.60911E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.55072E+00_JPRB, 0.57590E+00_JPRB, 0.60726E+00_JPRB, 0.64040E+00_JPRB, 0.67187E+00_JPRB, 0.70390E+00_JPRB, 0.75199E+00_JPRB ,&
&  0.80857E+00_JPRB, 0.85452E+00_JPRB, 0.88329E+00_JPRB, 0.92648E+00_JPRB, 0.97818E+00_JPRB, 0.10422E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.12509E+01_JPRB, 0.13180E+01_JPRB, 0.13870E+01_JPRB, 0.14584E+01_JPRB, 0.15382E+01_JPRB, 0.16581E+01_JPRB ,&
&  0.18135E+01_JPRB, 0.20179E+01_JPRB, 0.22741E+01_JPRB, 0.25264E+01_JPRB, 0.27591E+01_JPRB, 0.29862E+01_JPRB, 0.30693E+01_JPRB ,&
&  0.17486E+01_JPRB, 0.23185E+01_JPRB, 0.26632E+01_JPRB, 0.23582E+01_JPRB, 0.21209E+01_JPRB, 0.19736E+01_JPRB, 0.18859E+01_JPRB ,&
&  0.17511E+01_JPRB, 0.15720E+01_JPRB, 0.14883E+01_JPRB, 0.15268E+01_JPRB, 0.14075E+01_JPRB, 0.14241E+01_JPRB, 0.14613E+01_JPRB ,&
&  0.15422E+01_JPRB, 0.16596E+01_JPRB, 0.17772E+01_JPRB, 0.19189E+01_JPRB, 0.21279E+01_JPRB, 0.23882E+01_JPRB, 0.27113E+01_JPRB ,&
&  0.31199E+01_JPRB, 0.35851E+01_JPRB, 0.40169E+01_JPRB, 0.32510E+01_JPRB, 0.29728E+01_JPRB, 0.28077E+01_JPRB, 0.24287E+01_JPRB ,&
&  0.15591E+01_JPRB, 0.11623E+01_JPRB, 0.11466E+01_JPRB, 0.11071E+01_JPRB, 0.95522E+00_JPRB, 0.83453E+00_JPRB, 0.74879E+00_JPRB ,&
&  0.68867E+00_JPRB, 0.64200E+00_JPRB, 0.61862E+00_JPRB, 0.61386E+00_JPRB, 0.62000E+00_JPRB, 0.64675E+00_JPRB, 0.68234E+00_JPRB ,&
&  0.71459E+00_JPRB, 0.72296E+00_JPRB, 0.75913E+00_JPRB, 0.77650E+00_JPRB, 0.82616E+00_JPRB, 0.87004E+00_JPRB, 0.89952E+00_JPRB ,&
&  0.91064E+00_JPRB, 0.90034E+00_JPRB, 0.89008E+00_JPRB, 0.87924E+00_JPRB, 0.86974E+00_JPRB, 0.85780E+00_JPRB, 0.84394E+00_JPRB ,&
&  0.82479E+00_JPRB, 0.82588E+00_JPRB, 0.80228E+00_JPRB, 0.78248E+00_JPRB, 0.77658E+00_JPRB, 0.73551E+00_JPRB, 0.71352E+00_JPRB ,&
&  0.69252E+00_JPRB, 0.63961E+00_JPRB, 0.60409E+00_JPRB, 0.57699E+00_JPRB, 0.55549E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.58373E+00_JPRB, 0.54956E+00_JPRB, 0.55493E+00_JPRB, 0.56403E+00_JPRB, 0.58823E+00_JPRB, 0.61360E+00_JPRB, 0.61109E+00_JPRB ,&
&  0.62522E+00_JPRB, 0.64790E+00_JPRB, 0.67947E+00_JPRB, 0.70961E+00_JPRB, 0.74499E+00_JPRB, 0.78515E+00_JPRB, 0.83090E+00_JPRB ,&
&  0.89198E+00_JPRB, 0.96691E+00_JPRB, 0.10482E+01_JPRB, 0.11234E+01_JPRB, 0.11979E+01_JPRB, 0.12593E+01_JPRB, 0.13223E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.15458E+01_JPRB, 0.17511E+01_JPRB, 0.20121E+01_JPRB, 0.23523E+01_JPRB, 0.27596E+01_JPRB, 0.30499E+01_JPRB ,&
&  0.21081E+01_JPRB, 0.10186E+01_JPRB, 0.24819E+01_JPRB, 0.27437E+01_JPRB, 0.23307E+01_JPRB, 0.21374E+01_JPRB, 0.19065E+01_JPRB ,&
&  0.17534E+01_JPRB, 0.16982E+01_JPRB, 0.15429E+01_JPRB, 0.14993E+01_JPRB, 0.13897E+01_JPRB, 0.13765E+01_JPRB, 0.13730E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.14256E+01_JPRB, 0.15409E+01_JPRB, 0.16143E+01_JPRB, 0.17241E+01_JPRB, 0.19181E+01_JPRB, 0.22097E+01_JPRB ,&
&  0.26238E+01_JPRB, 0.31284E+01_JPRB, 0.36360E+01_JPRB, 0.28658E+01_JPRB, 0.23005E+01_JPRB, 0.18879E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.11900E+01_JPRB, 0.11370E+01_JPRB, 0.11087E+01_JPRB, 0.10164E+01_JPRB, 0.88702E+00_JPRB, 0.85885E+00_JPRB ,&
&  0.79997E+00_JPRB, 0.75819E+00_JPRB, 0.73279E+00_JPRB, 0.71584E+00_JPRB, 0.71317E+00_JPRB, 0.72997E+00_JPRB, 0.75020E+00_JPRB ,&
&  0.77686E+00_JPRB, 0.79607E+00_JPRB, 0.82485E+00_JPRB, 0.85813E+00_JPRB, 0.87092E+00_JPRB, 0.87861E+00_JPRB, 0.89856E+00_JPRB ,&
&  0.90225E+00_JPRB, 0.91489E+00_JPRB, 0.92850E+00_JPRB, 0.93306E+00_JPRB, 0.93426E+00_JPRB, 0.91896E+00_JPRB, 0.92240E+00_JPRB ,&
&  0.91642E+00_JPRB, 0.90796E+00_JPRB, 0.88460E+00_JPRB, 0.88333E+00_JPRB, 0.88039E+00_JPRB, 0.86835E+00_JPRB, 0.87560E+00_JPRB ,&
&  0.86065E+00_JPRB, 0.82259E+00_JPRB, 0.75225E+00_JPRB, 0.67718E+00_JPRB, 0.63456E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.71221E+00_JPRB, 0.65602E+00_JPRB, 0.61458E+00_JPRB, 0.59300E+00_JPRB, 0.58014E+00_JPRB, 0.57684E+00_JPRB, 0.57686E+00_JPRB ,&
&  0.58283E+00_JPRB, 0.58919E+00_JPRB, 0.58076E+00_JPRB, 0.59024E+00_JPRB, 0.61022E+00_JPRB, 0.63673E+00_JPRB, 0.67072E+00_JPRB ,&
&  0.69821E+00_JPRB, 0.72969E+00_JPRB, 0.76549E+00_JPRB, 0.81366E+00_JPRB, 0.87371E+00_JPRB, 0.94077E+00_JPRB, 0.10068E+01_JPRB ,&
&  0.10764E+01_JPRB, 0.11574E+01_JPRB, 0.12759E+01_JPRB, 0.14478E+01_JPRB, 0.16903E+01_JPRB, 0.20558E+01_JPRB, 0.24830E+01_JPRB ,&
&  0.30991E+01_JPRB, 0.22052E+01_JPRB, 0.14536E+01_JPRB, 0.25013E+01_JPRB, 0.26192E+01_JPRB, 0.24568E+01_JPRB, 0.20269E+01_JPRB ,&
&  0.20020E+01_JPRB, 0.19927E+01_JPRB, 0.17527E+01_JPRB, 0.15874E+01_JPRB, 0.13920E+01_JPRB, 0.13483E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12484E+01_JPRB, 0.13039E+01_JPRB, 0.13547E+01_JPRB, 0.13981E+01_JPRB, 0.15192E+01_JPRB, 0.17739E+01_JPRB ,&
&  0.21274E+01_JPRB, 0.25965E+01_JPRB, 0.31275E+01_JPRB, 0.26523E+01_JPRB, 0.20625E+01_JPRB, 0.16229E+01_JPRB, 0.14533E+01_JPRB ,&
&  0.13519E+01_JPRB, 0.12931E+01_JPRB, 0.12074E+01_JPRB, 0.11239E+01_JPRB, 0.10288E+01_JPRB, 0.93970E+00_JPRB, 0.92466E+00_JPRB ,&
&  0.89338E+00_JPRB, 0.84680E+00_JPRB, 0.81196E+00_JPRB, 0.78057E+00_JPRB, 0.77470E+00_JPRB, 0.78510E+00_JPRB, 0.79897E+00_JPRB ,&
&  0.81568E+00_JPRB, 0.83685E+00_JPRB, 0.86769E+00_JPRB, 0.90286E+00_JPRB, 0.91438E+00_JPRB, 0.91076E+00_JPRB, 0.91243E+00_JPRB ,&
&  0.91373E+00_JPRB, 0.91755E+00_JPRB, 0.93805E+00_JPRB, 0.94921E+00_JPRB, 0.94797E+00_JPRB, 0.93124E+00_JPRB, 0.94563E+00_JPRB ,&
&  0.94545E+00_JPRB, 0.95126E+00_JPRB, 0.93563E+00_JPRB, 0.94354E+00_JPRB, 0.94048E+00_JPRB, 0.95770E+00_JPRB, 0.97587E+00_JPRB ,&
&  0.98900E+00_JPRB, 0.97382E+00_JPRB, 0.92527E+00_JPRB, 0.80583E+00_JPRB, 0.75573E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.76856E+00_JPRB, 0.73293E+00_JPRB, 0.70597E+00_JPRB, 0.68159E+00_JPRB, 0.65750E+00_JPRB, 0.63461E+00_JPRB, 0.61664E+00_JPRB ,&
&  0.59559E+00_JPRB, 0.59309E+00_JPRB, 0.59046E+00_JPRB, 0.58458E+00_JPRB, 0.57750E+00_JPRB, 0.58153E+00_JPRB, 0.59423E+00_JPRB ,&
&  0.60466E+00_JPRB, 0.61416E+00_JPRB, 0.62880E+00_JPRB, 0.63832E+00_JPRB, 0.65125E+00_JPRB, 0.67680E+00_JPRB, 0.71620E+00_JPRB ,&
&  0.77073E+00_JPRB, 0.83425E+00_JPRB, 0.91911E+00_JPRB, 0.10271E+01_JPRB, 0.11394E+01_JPRB, 0.13167E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.20819E+01_JPRB, 0.21587E+01_JPRB, 0.11968E+01_JPRB, 0.21764E+01_JPRB, 0.27759E+01_JPRB, 0.26228E+01_JPRB, 0.24173E+01_JPRB ,&
&  0.24160E+01_JPRB, 0.22641E+01_JPRB, 0.20121E+01_JPRB, 0.16869E+01_JPRB, 0.14091E+01_JPRB, 0.13246E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.12038E+01_JPRB, 0.11538E+01_JPRB, 0.11476E+01_JPRB, 0.11523E+01_JPRB, 0.11629E+01_JPRB, 0.12109E+01_JPRB, 0.13880E+01_JPRB ,&
&  0.16695E+01_JPRB, 0.21093E+01_JPRB, 0.26082E+01_JPRB, 0.24577E+01_JPRB, 0.19187E+01_JPRB, 0.16718E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.15057E+01_JPRB, 0.14370E+01_JPRB, 0.12526E+01_JPRB, 0.11300E+01_JPRB, 0.10295E+01_JPRB, 0.95932E+00_JPRB, 0.96027E+00_JPRB ,&
&  0.93926E+00_JPRB, 0.90155E+00_JPRB, 0.87382E+00_JPRB, 0.83695E+00_JPRB, 0.81725E+00_JPRB, 0.80975E+00_JPRB, 0.81617E+00_JPRB ,&
&  0.82840E+00_JPRB, 0.84309E+00_JPRB, 0.86232E+00_JPRB, 0.88600E+00_JPRB, 0.90090E+00_JPRB, 0.90960E+00_JPRB, 0.91082E+00_JPRB ,&
&  0.91445E+00_JPRB, 0.92462E+00_JPRB, 0.93856E+00_JPRB, 0.94597E+00_JPRB, 0.94865E+00_JPRB, 0.94739E+00_JPRB, 0.94207E+00_JPRB ,&
&  0.94112E+00_JPRB, 0.94062E+00_JPRB, 0.91253E+00_JPRB, 0.90463E+00_JPRB, 0.93459E+00_JPRB, 0.93998E+00_JPRB, 0.94278E+00_JPRB ,&
&  0.97703E+00_JPRB, 0.10014E+01_JPRB, 0.98224E+00_JPRB, 0.86188E+00_JPRB, 0.80839E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.78993E+00_JPRB, 0.76162E+00_JPRB, 0.73948E+00_JPRB, 0.72254E+00_JPRB, 0.70242E+00_JPRB, 0.68232E+00_JPRB, 0.66546E+00_JPRB ,&
&  0.64335E+00_JPRB, 0.63008E+00_JPRB, 0.62144E+00_JPRB, 0.61183E+00_JPRB, 0.60190E+00_JPRB, 0.59264E+00_JPRB, 0.58547E+00_JPRB ,&
&  0.57898E+00_JPRB, 0.57381E+00_JPRB, 0.57111E+00_JPRB, 0.56528E+00_JPRB, 0.56058E+00_JPRB, 0.56196E+00_JPRB, 0.57215E+00_JPRB ,&
&  0.60087E+00_JPRB, 0.64518E+00_JPRB, 0.70228E+00_JPRB, 0.76793E+00_JPRB, 0.82610E+00_JPRB, 0.90475E+00_JPRB, 0.10011E+01_JPRB ,&
&  0.14117E+01_JPRB, 0.16760E+01_JPRB, 0.19699E+01_JPRB, 0.32312E+01_JPRB, 0.33063E+01_JPRB, 0.29516E+01_JPRB, 0.29242E+01_JPRB ,&
&  0.29973E+01_JPRB, 0.27915E+01_JPRB, 0.25279E+01_JPRB, 0.19560E+01_JPRB, 0.15533E+01_JPRB, 0.13396E+01_JPRB, 0.12163E+01_JPRB ,&
&  0.11144E+01_JPRB, 0.10555E+01_JPRB, 0.10407E+01_JPRB, 0.10279E+01_JPRB, 0.10274E+01_JPRB, 0.10889E+01_JPRB, 0.11785E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.16707E+01_JPRB, 0.21228E+01_JPRB, 0.23935E+01_JPRB, 0.19171E+01_JPRB, 0.18026E+01_JPRB, 0.18233E+01_JPRB ,&
&  0.21310E+01_JPRB, 0.17557E+01_JPRB, 0.13755E+01_JPRB, 0.11613E+01_JPRB, 0.10303E+01_JPRB, 0.98463E+00_JPRB, 0.95141E+00_JPRB ,&
&  0.93194E+00_JPRB, 0.91045E+00_JPRB, 0.89144E+00_JPRB, 0.86937E+00_JPRB, 0.85329E+00_JPRB, 0.83751E+00_JPRB, 0.83018E+00_JPRB ,&
&  0.83064E+00_JPRB, 0.83633E+00_JPRB, 0.84316E+00_JPRB, 0.85521E+00_JPRB, 0.87192E+00_JPRB, 0.88624E+00_JPRB, 0.89519E+00_JPRB ,&
&  0.90370E+00_JPRB, 0.91795E+00_JPRB, 0.90336E+00_JPRB, 0.90583E+00_JPRB, 0.91991E+00_JPRB, 0.92083E+00_JPRB, 0.89327E+00_JPRB ,&
&  0.88096E+00_JPRB, 0.89605E+00_JPRB, 0.89480E+00_JPRB, 0.88728E+00_JPRB, 0.89656E+00_JPRB, 0.90305E+00_JPRB, 0.87184E+00_JPRB ,&
&  0.87003E+00_JPRB, 0.87606E+00_JPRB, 0.86712E+00_JPRB, 0.84412E+00_JPRB, 0.81955E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.79940E+00_JPRB, 0.77717E+00_JPRB, 0.75456E+00_JPRB, 0.73294E+00_JPRB, 0.70891E+00_JPRB, 0.69003E+00_JPRB, 0.67942E+00_JPRB ,&
&  0.66339E+00_JPRB, 0.64954E+00_JPRB, 0.63971E+00_JPRB, 0.63486E+00_JPRB, 0.63071E+00_JPRB, 0.61925E+00_JPRB, 0.59849E+00_JPRB ,&
&  0.57793E+00_JPRB, 0.56388E+00_JPRB, 0.55527E+00_JPRB, 0.54664E+00_JPRB, 0.53813E+00_JPRB, 0.53406E+00_JPRB, 0.53771E+00_JPRB ,&
&  0.54899E+00_JPRB, 0.56810E+00_JPRB, 0.60484E+00_JPRB, 0.64849E+00_JPRB, 0.68866E+00_JPRB, 0.72857E+00_JPRB, 0.75943E+00_JPRB ,&
&  0.98765E+00_JPRB, 0.14144E+01_JPRB, 0.14111E+01_JPRB, 0.27365E+01_JPRB, 0.29388E+01_JPRB, 0.28862E+01_JPRB, 0.29110E+01_JPRB ,&
&  0.29437E+01_JPRB, 0.32546E+01_JPRB, 0.27621E+01_JPRB, 0.22322E+01_JPRB, 0.17792E+01_JPRB, 0.14824E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.10322E+01_JPRB, 0.98432E+00_JPRB, 0.96895E+00_JPRB, 0.96565E+00_JPRB, 0.99615E+00_JPRB, 0.10810E+01_JPRB ,&
&  0.12026E+01_JPRB, 0.14112E+01_JPRB, 0.17370E+01_JPRB, 0.19469E+01_JPRB, 0.18613E+01_JPRB, 0.18863E+01_JPRB, 0.20736E+01_JPRB ,&
&  0.25703E+01_JPRB, 0.22822E+01_JPRB, 0.16011E+01_JPRB, 0.12326E+01_JPRB, 0.10574E+01_JPRB, 0.98917E+00_JPRB, 0.93836E+00_JPRB ,&
&  0.91621E+00_JPRB, 0.89523E+00_JPRB, 0.87875E+00_JPRB, 0.86751E+00_JPRB, 0.85711E+00_JPRB, 0.85143E+00_JPRB, 0.84532E+00_JPRB ,&
&  0.83748E+00_JPRB, 0.83486E+00_JPRB, 0.83415E+00_JPRB, 0.83516E+00_JPRB, 0.83897E+00_JPRB, 0.85004E+00_JPRB, 0.86524E+00_JPRB ,&
&  0.88048E+00_JPRB, 0.89671E+00_JPRB, 0.88488E+00_JPRB, 0.87191E+00_JPRB, 0.87795E+00_JPRB, 0.87580E+00_JPRB, 0.85455E+00_JPRB ,&
&  0.85913E+00_JPRB, 0.87456E+00_JPRB, 0.87378E+00_JPRB, 0.85565E+00_JPRB, 0.86763E+00_JPRB, 0.89146E+00_JPRB, 0.84560E+00_JPRB ,&
&  0.83071E+00_JPRB, 0.82900E+00_JPRB, 0.83195E+00_JPRB, 0.82271E+00_JPRB, 0.81208E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.78105E+00_JPRB, 0.76926E+00_JPRB, 0.75108E+00_JPRB, 0.73006E+00_JPRB, 0.70815E+00_JPRB, 0.68946E+00_JPRB, 0.67913E+00_JPRB ,&
&  0.66687E+00_JPRB, 0.65429E+00_JPRB, 0.64300E+00_JPRB, 0.63622E+00_JPRB, 0.63279E+00_JPRB, 0.62165E+00_JPRB, 0.60229E+00_JPRB ,&
&  0.58194E+00_JPRB, 0.56380E+00_JPRB, 0.54841E+00_JPRB, 0.53865E+00_JPRB, 0.52857E+00_JPRB, 0.52417E+00_JPRB, 0.52453E+00_JPRB ,&
&  0.52785E+00_JPRB, 0.53828E+00_JPRB, 0.56056E+00_JPRB, 0.59433E+00_JPRB, 0.62875E+00_JPRB, 0.65850E+00_JPRB, 0.67862E+00_JPRB ,&
&  0.80896E+00_JPRB, 0.16785E+01_JPRB, 0.13098E+01_JPRB, 0.20962E+01_JPRB, 0.22943E+01_JPRB, 0.24417E+01_JPRB, 0.26403E+01_JPRB ,&
&  0.27262E+01_JPRB, 0.28564E+01_JPRB, 0.26733E+01_JPRB, 0.23318E+01_JPRB, 0.19400E+01_JPRB, 0.16634E+01_JPRB, 0.14457E+01_JPRB ,&
&  0.12809E+01_JPRB, 0.11652E+01_JPRB, 0.10827E+01_JPRB, 0.10219E+01_JPRB, 0.98420E+00_JPRB, 0.98604E+00_JPRB, 0.10562E+01_JPRB ,&
&  0.11941E+01_JPRB, 0.13497E+01_JPRB, 0.15384E+01_JPRB, 0.17640E+01_JPRB, 0.16802E+01_JPRB, 0.18571E+01_JPRB, 0.23838E+01_JPRB ,&
&  0.27952E+01_JPRB, 0.28226E+01_JPRB, 0.19711E+01_JPRB, 0.13857E+01_JPRB, 0.11228E+01_JPRB, 0.99623E+00_JPRB, 0.92076E+00_JPRB ,&
&  0.87955E+00_JPRB, 0.85810E+00_JPRB, 0.84559E+00_JPRB, 0.84228E+00_JPRB, 0.84383E+00_JPRB, 0.84914E+00_JPRB, 0.84834E+00_JPRB ,&
&  0.84576E+00_JPRB, 0.84289E+00_JPRB, 0.83941E+00_JPRB, 0.83423E+00_JPRB, 0.82660E+00_JPRB, 0.82397E+00_JPRB, 0.82915E+00_JPRB ,&
&  0.84261E+00_JPRB, 0.85999E+00_JPRB, 0.85814E+00_JPRB, 0.84957E+00_JPRB, 0.84798E+00_JPRB, 0.84449E+00_JPRB, 0.83262E+00_JPRB ,&
&  0.84140E+00_JPRB, 0.84663E+00_JPRB, 0.84523E+00_JPRB, 0.84775E+00_JPRB, 0.84688E+00_JPRB, 0.89072E+00_JPRB, 0.83385E+00_JPRB ,&
&  0.81432E+00_JPRB, 0.80777E+00_JPRB, 0.80421E+00_JPRB, 0.79255E+00_JPRB, 0.78600E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.74603E+00_JPRB, 0.73659E+00_JPRB, 0.72039E+00_JPRB, 0.70257E+00_JPRB, 0.69000E+00_JPRB, 0.67607E+00_JPRB, 0.66504E+00_JPRB ,&
&  0.65521E+00_JPRB, 0.64617E+00_JPRB, 0.63868E+00_JPRB, 0.63027E+00_JPRB, 0.61960E+00_JPRB, 0.60535E+00_JPRB, 0.59158E+00_JPRB ,&
&  0.57730E+00_JPRB, 0.56355E+00_JPRB, 0.54939E+00_JPRB, 0.53461E+00_JPRB, 0.52256E+00_JPRB, 0.51465E+00_JPRB, 0.50963E+00_JPRB ,&
&  0.50958E+00_JPRB, 0.51415E+00_JPRB, 0.52759E+00_JPRB, 0.55317E+00_JPRB, 0.58620E+00_JPRB, 0.61845E+00_JPRB, 0.65070E+00_JPRB ,&
&  0.78514E+00_JPRB, 0.73298E+00_JPRB, 0.14888E+01_JPRB, 0.17868E+01_JPRB, 0.17628E+01_JPRB, 0.18178E+01_JPRB, 0.21879E+01_JPRB ,&
&  0.24152E+01_JPRB, 0.25079E+01_JPRB, 0.25082E+01_JPRB, 0.23568E+01_JPRB, 0.20608E+01_JPRB, 0.18401E+01_JPRB, 0.16675E+01_JPRB ,&
&  0.15370E+01_JPRB, 0.14289E+01_JPRB, 0.13051E+01_JPRB, 0.11854E+01_JPRB, 0.10984E+01_JPRB, 0.10785E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.13658E+01_JPRB, 0.14451E+01_JPRB, 0.15701E+01_JPRB, 0.15784E+01_JPRB, 0.17888E+01_JPRB, 0.20702E+01_JPRB ,&
&  0.27493E+01_JPRB, 0.28660E+01_JPRB, 0.21953E+01_JPRB, 0.16250E+01_JPRB, 0.12673E+01_JPRB, 0.10766E+01_JPRB, 0.94503E+00_JPRB ,&
&  0.87351E+00_JPRB, 0.83481E+00_JPRB, 0.81166E+00_JPRB, 0.80824E+00_JPRB, 0.82130E+00_JPRB, 0.84184E+00_JPRB, 0.85765E+00_JPRB ,&
&  0.87413E+00_JPRB, 0.88398E+00_JPRB, 0.88062E+00_JPRB, 0.87052E+00_JPRB, 0.84709E+00_JPRB, 0.81762E+00_JPRB, 0.79983E+00_JPRB ,&
&  0.79948E+00_JPRB, 0.81260E+00_JPRB, 0.82104E+00_JPRB, 0.81551E+00_JPRB, 0.81628E+00_JPRB, 0.81673E+00_JPRB, 0.81444E+00_JPRB ,&
&  0.81895E+00_JPRB, 0.82143E+00_JPRB, 0.82029E+00_JPRB, 0.83095E+00_JPRB, 0.82016E+00_JPRB, 0.85929E+00_JPRB, 0.82531E+00_JPRB ,&
&  0.80210E+00_JPRB, 0.78709E+00_JPRB, 0.77543E+00_JPRB, 0.76069E+00_JPRB, 0.74929E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.71049E+00_JPRB, 0.69993E+00_JPRB, 0.68277E+00_JPRB, 0.66569E+00_JPRB, 0.65546E+00_JPRB, 0.64820E+00_JPRB, 0.64298E+00_JPRB ,&
&  0.63921E+00_JPRB, 0.62988E+00_JPRB, 0.62020E+00_JPRB, 0.60709E+00_JPRB, 0.59062E+00_JPRB, 0.57674E+00_JPRB, 0.56792E+00_JPRB ,&
&  0.55729E+00_JPRB, 0.55164E+00_JPRB, 0.54451E+00_JPRB, 0.53132E+00_JPRB, 0.51675E+00_JPRB, 0.50485E+00_JPRB, 0.49741E+00_JPRB ,&
&  0.49410E+00_JPRB, 0.49723E+00_JPRB, 0.50600E+00_JPRB, 0.52496E+00_JPRB, 0.54554E+00_JPRB, 0.57018E+00_JPRB, 0.61443E+00_JPRB ,&
&  0.78644E+00_JPRB, 0.14042E+01_JPRB, 0.18190E+01_JPRB, 0.16260E+01_JPRB, 0.14931E+01_JPRB, 0.13919E+01_JPRB, 0.16236E+01_JPRB ,&
&  0.18215E+01_JPRB, 0.20263E+01_JPRB, 0.21478E+01_JPRB, 0.21875E+01_JPRB, 0.20393E+01_JPRB, 0.19295E+01_JPRB, 0.18194E+01_JPRB ,&
&  0.17065E+01_JPRB, 0.15766E+01_JPRB, 0.14007E+01_JPRB, 0.12311E+01_JPRB, 0.11351E+01_JPRB, 0.11213E+01_JPRB, 0.11735E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.13617E+01_JPRB, 0.13859E+01_JPRB, 0.14795E+01_JPRB, 0.16444E+01_JPRB, 0.16432E+01_JPRB, 0.20675E+01_JPRB ,&
&  0.26166E+01_JPRB, 0.23866E+01_JPRB, 0.20260E+01_JPRB, 0.16504E+01_JPRB, 0.13572E+01_JPRB, 0.11239E+01_JPRB, 0.98982E+00_JPRB ,&
&  0.89770E+00_JPRB, 0.83853E+00_JPRB, 0.79951E+00_JPRB, 0.78249E+00_JPRB, 0.79817E+00_JPRB, 0.83603E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.91104E+00_JPRB, 0.95242E+00_JPRB, 0.97290E+00_JPRB, 0.96573E+00_JPRB, 0.92994E+00_JPRB, 0.87774E+00_JPRB, 0.82749E+00_JPRB ,&
&  0.79687E+00_JPRB, 0.78332E+00_JPRB, 0.79346E+00_JPRB, 0.78286E+00_JPRB, 0.78673E+00_JPRB, 0.79880E+00_JPRB, 0.79717E+00_JPRB ,&
&  0.79325E+00_JPRB, 0.80118E+00_JPRB, 0.81505E+00_JPRB, 0.81162E+00_JPRB, 0.82253E+00_JPRB, 0.86270E+00_JPRB, 0.82858E+00_JPRB ,&
&  0.79684E+00_JPRB, 0.76947E+00_JPRB, 0.75089E+00_JPRB, 0.72560E+00_JPRB, 0.71377E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.69247E+00_JPRB, 0.67544E+00_JPRB, 0.64962E+00_JPRB, 0.62773E+00_JPRB, 0.61516E+00_JPRB, 0.60796E+00_JPRB, 0.60575E+00_JPRB ,&
&  0.60494E+00_JPRB, 0.59676E+00_JPRB, 0.58644E+00_JPRB, 0.57123E+00_JPRB, 0.55780E+00_JPRB, 0.54577E+00_JPRB, 0.53529E+00_JPRB ,&
&  0.52653E+00_JPRB, 0.52163E+00_JPRB, 0.51674E+00_JPRB, 0.51125E+00_JPRB, 0.50322E+00_JPRB, 0.49195E+00_JPRB, 0.48477E+00_JPRB ,&
&  0.48061E+00_JPRB, 0.48081E+00_JPRB, 0.48578E+00_JPRB, 0.49841E+00_JPRB, 0.51442E+00_JPRB, 0.53509E+00_JPRB, 0.56738E+00_JPRB ,&
&  0.67961E+00_JPRB, 0.11733E+01_JPRB, 0.12705E+01_JPRB, 0.12391E+01_JPRB, 0.12642E+01_JPRB, 0.12346E+01_JPRB, 0.13089E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.15950E+01_JPRB, 0.17211E+01_JPRB, 0.17705E+01_JPRB, 0.17010E+01_JPRB, 0.16323E+01_JPRB, 0.15401E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.13259E+01_JPRB, 0.12101E+01_JPRB, 0.11024E+01_JPRB, 0.10202E+01_JPRB, 0.10048E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.11604E+01_JPRB, 0.11927E+01_JPRB, 0.12787E+01_JPRB, 0.14809E+01_JPRB, 0.16095E+01_JPRB, 0.17901E+01_JPRB ,&
&  0.18290E+01_JPRB, 0.17674E+01_JPRB, 0.16623E+01_JPRB, 0.15243E+01_JPRB, 0.13376E+01_JPRB, 0.11565E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.90799E+00_JPRB, 0.83863E+00_JPRB, 0.79662E+00_JPRB, 0.78008E+00_JPRB, 0.79108E+00_JPRB, 0.82691E+00_JPRB, 0.87420E+00_JPRB ,&
&  0.93081E+00_JPRB, 0.98633E+00_JPRB, 0.10205E+01_JPRB, 0.10327E+01_JPRB, 0.10171E+01_JPRB, 0.98041E+00_JPRB, 0.92458E+00_JPRB ,&
&  0.87208E+00_JPRB, 0.82886E+00_JPRB, 0.80613E+00_JPRB, 0.79730E+00_JPRB, 0.79701E+00_JPRB, 0.79851E+00_JPRB, 0.79923E+00_JPRB ,&
&  0.80490E+00_JPRB, 0.82050E+00_JPRB, 0.81825E+00_JPRB, 0.80940E+00_JPRB, 0.80915E+00_JPRB, 0.83012E+00_JPRB, 0.80338E+00_JPRB ,&
&  0.77600E+00_JPRB, 0.74651E+00_JPRB, 0.72285E+00_JPRB, 0.69627E+00_JPRB, 0.69777E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.72014E+00_JPRB, 0.68560E+00_JPRB, 0.64317E+00_JPRB, 0.61122E+00_JPRB, 0.58978E+00_JPRB, 0.57540E+00_JPRB, 0.56853E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.55652E+00_JPRB, 0.53938E+00_JPRB, 0.52453E+00_JPRB, 0.51652E+00_JPRB, 0.51035E+00_JPRB, 0.50241E+00_JPRB ,&
&  0.49603E+00_JPRB, 0.49029E+00_JPRB, 0.48578E+00_JPRB, 0.48399E+00_JPRB, 0.48311E+00_JPRB, 0.48104E+00_JPRB, 0.47539E+00_JPRB ,&
&  0.46722E+00_JPRB, 0.46425E+00_JPRB, 0.46542E+00_JPRB, 0.47406E+00_JPRB, 0.48778E+00_JPRB, 0.50553E+00_JPRB, 0.53373E+00_JPRB ,&
&  0.64639E+00_JPRB, 0.75780E+00_JPRB, 0.84910E+00_JPRB, 0.86963E+00_JPRB, 0.90945E+00_JPRB, 0.99661E+00_JPRB, 0.10724E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.12427E+01_JPRB, 0.13704E+01_JPRB, 0.14331E+01_JPRB, 0.13865E+01_JPRB, 0.12983E+01_JPRB, 0.12197E+01_JPRB ,&
&  0.11455E+01_JPRB, 0.10924E+01_JPRB, 0.10393E+01_JPRB, 0.98793E+00_JPRB, 0.95410E+00_JPRB, 0.94916E+00_JPRB, 0.96354E+00_JPRB ,&
&  0.97503E+00_JPRB, 0.96041E+00_JPRB, 0.10043E+01_JPRB, 0.10759E+01_JPRB, 0.11924E+01_JPRB, 0.12763E+01_JPRB, 0.13569E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.14403E+01_JPRB, 0.14445E+01_JPRB, 0.14123E+01_JPRB, 0.13211E+01_JPRB, 0.12081E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.97628E+00_JPRB, 0.90081E+00_JPRB, 0.86097E+00_JPRB, 0.84266E+00_JPRB, 0.84111E+00_JPRB, 0.85044E+00_JPRB, 0.87214E+00_JPRB ,&
&  0.90471E+00_JPRB, 0.93564E+00_JPRB, 0.96083E+00_JPRB, 0.97200E+00_JPRB, 0.96797E+00_JPRB, 0.96059E+00_JPRB, 0.94929E+00_JPRB ,&
&  0.93411E+00_JPRB, 0.91283E+00_JPRB, 0.88719E+00_JPRB, 0.86025E+00_JPRB, 0.83684E+00_JPRB, 0.82086E+00_JPRB, 0.80852E+00_JPRB ,&
&  0.79632E+00_JPRB, 0.79566E+00_JPRB, 0.77677E+00_JPRB, 0.79551E+00_JPRB, 0.81818E+00_JPRB, 0.79855E+00_JPRB, 0.77190E+00_JPRB ,&
&  0.73910E+00_JPRB, 0.70996E+00_JPRB, 0.68392E+00_JPRB, 0.65888E+00_JPRB, 0.70881E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.62026E+00_JPRB, 0.61379E+00_JPRB, 0.59582E+00_JPRB, 0.57321E+00_JPRB, 0.55700E+00_JPRB, 0.54308E+00_JPRB, 0.53204E+00_JPRB ,&
&  0.52505E+00_JPRB, 0.51549E+00_JPRB, 0.50227E+00_JPRB, 0.49216E+00_JPRB, 0.49030E+00_JPRB, 0.48726E+00_JPRB, 0.48201E+00_JPRB ,&
&  0.47297E+00_JPRB, 0.46422E+00_JPRB, 0.45819E+00_JPRB, 0.45622E+00_JPRB, 0.45728E+00_JPRB, 0.45964E+00_JPRB, 0.46035E+00_JPRB ,&
&  0.45710E+00_JPRB, 0.45155E+00_JPRB, 0.45042E+00_JPRB, 0.45280E+00_JPRB, 0.46004E+00_JPRB, 0.47253E+00_JPRB, 0.49828E+00_JPRB ,&
&  0.57917E+00_JPRB, 0.59339E+00_JPRB, 0.63480E+00_JPRB, 0.65979E+00_JPRB, 0.70340E+00_JPRB, 0.78264E+00_JPRB, 0.87273E+00_JPRB ,&
&  0.92439E+00_JPRB, 0.97780E+00_JPRB, 0.10644E+01_JPRB, 0.11388E+01_JPRB, 0.11422E+01_JPRB, 0.10980E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.96362E+00_JPRB, 0.91287E+00_JPRB, 0.86994E+00_JPRB, 0.83438E+00_JPRB, 0.81076E+00_JPRB, 0.79945E+00_JPRB, 0.79344E+00_JPRB ,&
&  0.77815E+00_JPRB, 0.78013E+00_JPRB, 0.81690E+00_JPRB, 0.87549E+00_JPRB, 0.94604E+00_JPRB, 0.10173E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.11611E+01_JPRB, 0.12024E+01_JPRB, 0.12324E+01_JPRB, 0.12537E+01_JPRB, 0.12380E+01_JPRB, 0.12100E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.98050E+00_JPRB, 0.93326E+00_JPRB, 0.89682E+00_JPRB, 0.87075E+00_JPRB, 0.85441E+00_JPRB, 0.84619E+00_JPRB ,&
&  0.84613E+00_JPRB, 0.85074E+00_JPRB, 0.85198E+00_JPRB, 0.84891E+00_JPRB, 0.84430E+00_JPRB, 0.84123E+00_JPRB, 0.83820E+00_JPRB ,&
&  0.83590E+00_JPRB, 0.83332E+00_JPRB, 0.82611E+00_JPRB, 0.81083E+00_JPRB, 0.78948E+00_JPRB, 0.76823E+00_JPRB, 0.74852E+00_JPRB ,&
&  0.73204E+00_JPRB, 0.72830E+00_JPRB, 0.72032E+00_JPRB, 0.73236E+00_JPRB, 0.72592E+00_JPRB, 0.71468E+00_JPRB, 0.69929E+00_JPRB ,&
&  0.67880E+00_JPRB, 0.65803E+00_JPRB, 0.63080E+00_JPRB, 0.60567E+00_JPRB, 0.60090E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.55681E+00_JPRB, 0.55941E+00_JPRB, 0.55586E+00_JPRB, 0.54084E+00_JPRB, 0.52510E+00_JPRB, 0.51600E+00_JPRB, 0.50607E+00_JPRB ,&
&  0.49537E+00_JPRB, 0.48669E+00_JPRB, 0.47460E+00_JPRB, 0.46848E+00_JPRB, 0.46587E+00_JPRB, 0.46506E+00_JPRB, 0.46248E+00_JPRB ,&
&  0.45610E+00_JPRB, 0.44665E+00_JPRB, 0.43951E+00_JPRB, 0.43664E+00_JPRB, 0.43771E+00_JPRB, 0.44017E+00_JPRB, 0.44208E+00_JPRB ,&
&  0.44030E+00_JPRB, 0.43855E+00_JPRB, 0.43649E+00_JPRB, 0.43528E+00_JPRB, 0.43659E+00_JPRB, 0.44625E+00_JPRB, 0.46554E+00_JPRB ,&
&  0.52537E+00_JPRB, 0.49757E+00_JPRB, 0.51774E+00_JPRB, 0.55596E+00_JPRB, 0.59802E+00_JPRB, 0.66331E+00_JPRB, 0.73651E+00_JPRB ,&
&  0.78522E+00_JPRB, 0.79995E+00_JPRB, 0.83375E+00_JPRB, 0.89090E+00_JPRB, 0.90171E+00_JPRB, 0.89708E+00_JPRB, 0.87210E+00_JPRB ,&
&  0.82885E+00_JPRB, 0.77432E+00_JPRB, 0.72128E+00_JPRB, 0.68301E+00_JPRB, 0.66500E+00_JPRB, 0.65646E+00_JPRB, 0.64850E+00_JPRB ,&
&  0.63072E+00_JPRB, 0.62548E+00_JPRB, 0.64356E+00_JPRB, 0.66982E+00_JPRB, 0.70158E+00_JPRB, 0.75380E+00_JPRB, 0.82378E+00_JPRB ,&
&  0.88286E+00_JPRB, 0.94459E+00_JPRB, 0.10054E+01_JPRB, 0.10520E+01_JPRB, 0.10901E+01_JPRB, 0.10976E+01_JPRB, 0.10653E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.96189E+00_JPRB, 0.91004E+00_JPRB, 0.86221E+00_JPRB, 0.82322E+00_JPRB, 0.79903E+00_JPRB, 0.78424E+00_JPRB ,&
&  0.77412E+00_JPRB, 0.76607E+00_JPRB, 0.75278E+00_JPRB, 0.73402E+00_JPRB, 0.71485E+00_JPRB, 0.69853E+00_JPRB, 0.68407E+00_JPRB ,&
&  0.67139E+00_JPRB, 0.65853E+00_JPRB, 0.64922E+00_JPRB, 0.64434E+00_JPRB, 0.63908E+00_JPRB, 0.63525E+00_JPRB, 0.63268E+00_JPRB ,&
&  0.62864E+00_JPRB, 0.63109E+00_JPRB, 0.62416E+00_JPRB, 0.64864E+00_JPRB, 0.65775E+00_JPRB, 0.64784E+00_JPRB, 0.63719E+00_JPRB ,&
&  0.62073E+00_JPRB, 0.60331E+00_JPRB, 0.58499E+00_JPRB, 0.53660E+00_JPRB, 0.55511E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.50968E+00_JPRB, 0.51262E+00_JPRB, 0.51039E+00_JPRB, 0.50273E+00_JPRB, 0.49223E+00_JPRB, 0.48503E+00_JPRB, 0.47505E+00_JPRB ,&
&  0.46428E+00_JPRB, 0.45871E+00_JPRB, 0.45192E+00_JPRB, 0.44839E+00_JPRB, 0.44827E+00_JPRB, 0.44419E+00_JPRB, 0.44046E+00_JPRB ,&
&  0.43696E+00_JPRB, 0.43064E+00_JPRB, 0.42648E+00_JPRB, 0.42321E+00_JPRB, 0.42311E+00_JPRB, 0.42373E+00_JPRB, 0.42352E+00_JPRB ,&
&  0.42086E+00_JPRB, 0.41681E+00_JPRB, 0.41335E+00_JPRB, 0.41099E+00_JPRB, 0.41235E+00_JPRB, 0.41964E+00_JPRB, 0.43427E+00_JPRB ,&
&  0.47605E+00_JPRB, 0.43524E+00_JPRB, 0.45871E+00_JPRB, 0.48484E+00_JPRB, 0.52438E+00_JPRB, 0.56467E+00_JPRB, 0.59176E+00_JPRB ,&
&  0.62321E+00_JPRB, 0.65174E+00_JPRB, 0.68079E+00_JPRB, 0.71175E+00_JPRB, 0.71744E+00_JPRB, 0.70969E+00_JPRB, 0.69938E+00_JPRB ,&
&  0.68071E+00_JPRB, 0.65528E+00_JPRB, 0.62861E+00_JPRB, 0.60104E+00_JPRB, 0.58314E+00_JPRB, 0.57286E+00_JPRB, 0.56038E+00_JPRB ,&
&  0.54314E+00_JPRB, 0.52826E+00_JPRB, 0.51942E+00_JPRB, 0.51996E+00_JPRB, 0.53192E+00_JPRB, 0.55622E+00_JPRB, 0.60199E+00_JPRB ,&
&  0.65222E+00_JPRB, 0.70875E+00_JPRB, 0.76994E+00_JPRB, 0.81971E+00_JPRB, 0.85686E+00_JPRB, 0.87327E+00_JPRB, 0.87510E+00_JPRB ,&
&  0.86042E+00_JPRB, 0.83870E+00_JPRB, 0.81708E+00_JPRB, 0.79033E+00_JPRB, 0.75646E+00_JPRB, 0.72498E+00_JPRB, 0.69998E+00_JPRB ,&
&  0.68066E+00_JPRB, 0.66072E+00_JPRB, 0.63872E+00_JPRB, 0.61473E+00_JPRB, 0.58965E+00_JPRB, 0.56698E+00_JPRB, 0.54810E+00_JPRB ,&
&  0.53226E+00_JPRB, 0.51872E+00_JPRB, 0.51071E+00_JPRB, 0.50902E+00_JPRB, 0.51335E+00_JPRB, 0.51956E+00_JPRB, 0.52421E+00_JPRB ,&
&  0.52916E+00_JPRB, 0.53726E+00_JPRB, 0.54120E+00_JPRB, 0.55747E+00_JPRB, 0.56225E+00_JPRB, 0.56208E+00_JPRB, 0.56150E+00_JPRB ,&
&  0.55673E+00_JPRB, 0.54540E+00_JPRB, 0.48404E+00_JPRB, 0.50662E+00_JPRB, 0.50831E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.47378E+00_JPRB, 0.46796E+00_JPRB, 0.46273E+00_JPRB, 0.46329E+00_JPRB, 0.46139E+00_JPRB, 0.45692E+00_JPRB, 0.45002E+00_JPRB ,&
&  0.43922E+00_JPRB, 0.43209E+00_JPRB, 0.42712E+00_JPRB, 0.42584E+00_JPRB, 0.42855E+00_JPRB, 0.42639E+00_JPRB, 0.42129E+00_JPRB ,&
&  0.41744E+00_JPRB, 0.41306E+00_JPRB, 0.40988E+00_JPRB, 0.40621E+00_JPRB, 0.40314E+00_JPRB, 0.40162E+00_JPRB, 0.40060E+00_JPRB ,&
&  0.39836E+00_JPRB, 0.39440E+00_JPRB, 0.39075E+00_JPRB, 0.38854E+00_JPRB, 0.39040E+00_JPRB, 0.39652E+00_JPRB, 0.40606E+00_JPRB ,&
&  0.40784E+00_JPRB, 0.39364E+00_JPRB, 0.41637E+00_JPRB, 0.44545E+00_JPRB, 0.46728E+00_JPRB, 0.49133E+00_JPRB, 0.51319E+00_JPRB ,&
&  0.53421E+00_JPRB, 0.54218E+00_JPRB, 0.55223E+00_JPRB, 0.56393E+00_JPRB, 0.57281E+00_JPRB, 0.57240E+00_JPRB, 0.56918E+00_JPRB ,&
&  0.56501E+00_JPRB, 0.55952E+00_JPRB, 0.55266E+00_JPRB, 0.54135E+00_JPRB, 0.52886E+00_JPRB, 0.51540E+00_JPRB, 0.50181E+00_JPRB ,&
&  0.48932E+00_JPRB, 0.47797E+00_JPRB, 0.47009E+00_JPRB, 0.46690E+00_JPRB, 0.46916E+00_JPRB, 0.47476E+00_JPRB, 0.48579E+00_JPRB ,&
&  0.49956E+00_JPRB, 0.52220E+00_JPRB, 0.56265E+00_JPRB, 0.60440E+00_JPRB, 0.63781E+00_JPRB, 0.66036E+00_JPRB, 0.67762E+00_JPRB ,&
&  0.68127E+00_JPRB, 0.67761E+00_JPRB, 0.67177E+00_JPRB, 0.66163E+00_JPRB, 0.64649E+00_JPRB, 0.62614E+00_JPRB, 0.60221E+00_JPRB ,&
&  0.58072E+00_JPRB, 0.56203E+00_JPRB, 0.54399E+00_JPRB, 0.52653E+00_JPRB, 0.50930E+00_JPRB, 0.49323E+00_JPRB, 0.47939E+00_JPRB ,&
&  0.46734E+00_JPRB, 0.45642E+00_JPRB, 0.44847E+00_JPRB, 0.44500E+00_JPRB, 0.44698E+00_JPRB, 0.45222E+00_JPRB, 0.45907E+00_JPRB ,&
&  0.46517E+00_JPRB, 0.47420E+00_JPRB, 0.48201E+00_JPRB, 0.49263E+00_JPRB, 0.50193E+00_JPRB, 0.50355E+00_JPRB, 0.50445E+00_JPRB ,&
&  0.50392E+00_JPRB, 0.49750E+00_JPRB, 0.49198E+00_JPRB, 0.48412E+00_JPRB, 0.47700E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.43560E+00_JPRB, 0.42822E+00_JPRB, 0.42671E+00_JPRB, 0.42987E+00_JPRB, 0.43373E+00_JPRB, 0.43308E+00_JPRB, 0.42864E+00_JPRB ,&
&  0.41853E+00_JPRB, 0.40957E+00_JPRB, 0.40273E+00_JPRB, 0.39997E+00_JPRB, 0.40222E+00_JPRB, 0.40272E+00_JPRB, 0.39978E+00_JPRB ,&
&  0.39643E+00_JPRB, 0.39029E+00_JPRB, 0.38489E+00_JPRB, 0.37948E+00_JPRB, 0.37548E+00_JPRB, 0.37359E+00_JPRB, 0.37273E+00_JPRB ,&
&  0.37110E+00_JPRB, 0.36893E+00_JPRB, 0.36796E+00_JPRB, 0.36736E+00_JPRB, 0.36876E+00_JPRB, 0.37305E+00_JPRB, 0.37933E+00_JPRB ,&
&  0.39225E+00_JPRB, 0.38059E+00_JPRB, 0.40367E+00_JPRB, 0.40959E+00_JPRB, 0.41916E+00_JPRB, 0.43259E+00_JPRB, 0.45107E+00_JPRB ,&
&  0.46781E+00_JPRB, 0.47417E+00_JPRB, 0.47716E+00_JPRB, 0.47951E+00_JPRB, 0.48095E+00_JPRB, 0.48482E+00_JPRB, 0.48414E+00_JPRB ,&
&  0.48017E+00_JPRB, 0.47797E+00_JPRB, 0.47681E+00_JPRB, 0.47436E+00_JPRB, 0.47183E+00_JPRB, 0.46684E+00_JPRB, 0.46025E+00_JPRB ,&
&  0.45341E+00_JPRB, 0.44647E+00_JPRB, 0.44034E+00_JPRB, 0.43430E+00_JPRB, 0.42930E+00_JPRB, 0.42657E+00_JPRB, 0.42647E+00_JPRB ,&
&  0.42767E+00_JPRB, 0.43227E+00_JPRB, 0.44485E+00_JPRB, 0.46364E+00_JPRB, 0.48464E+00_JPRB, 0.50393E+00_JPRB, 0.51957E+00_JPRB ,&
&  0.52883E+00_JPRB, 0.53301E+00_JPRB, 0.53235E+00_JPRB, 0.52900E+00_JPRB, 0.52226E+00_JPRB, 0.51385E+00_JPRB, 0.50250E+00_JPRB ,&
&  0.49159E+00_JPRB, 0.48147E+00_JPRB, 0.47098E+00_JPRB, 0.46078E+00_JPRB, 0.44974E+00_JPRB, 0.43817E+00_JPRB, 0.42780E+00_JPRB ,&
&  0.41966E+00_JPRB, 0.41382E+00_JPRB, 0.40960E+00_JPRB, 0.40720E+00_JPRB, 0.40627E+00_JPRB, 0.40800E+00_JPRB, 0.41086E+00_JPRB ,&
&  0.41477E+00_JPRB, 0.42046E+00_JPRB, 0.42724E+00_JPRB, 0.43422E+00_JPRB, 0.44254E+00_JPRB, 0.44733E+00_JPRB, 0.45105E+00_JPRB ,&
&  0.45231E+00_JPRB, 0.45001E+00_JPRB, 0.44832E+00_JPRB, 0.44515E+00_JPRB, 0.44149E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.39917E+00_JPRB, 0.39565E+00_JPRB, 0.39798E+00_JPRB, 0.40258E+00_JPRB, 0.40941E+00_JPRB, 0.40887E+00_JPRB, 0.40667E+00_JPRB ,&
&  0.40072E+00_JPRB, 0.39280E+00_JPRB, 0.38516E+00_JPRB, 0.37824E+00_JPRB, 0.37557E+00_JPRB, 0.37634E+00_JPRB, 0.37622E+00_JPRB ,&
&  0.37469E+00_JPRB, 0.37004E+00_JPRB, 0.36372E+00_JPRB, 0.35771E+00_JPRB, 0.35338E+00_JPRB, 0.35096E+00_JPRB, 0.34909E+00_JPRB ,&
&  0.34808E+00_JPRB, 0.34637E+00_JPRB, 0.34714E+00_JPRB, 0.34708E+00_JPRB, 0.34769E+00_JPRB, 0.35137E+00_JPRB, 0.35570E+00_JPRB ,&
&  0.36010E+00_JPRB, 0.38074E+00_JPRB, 0.37586E+00_JPRB, 0.37389E+00_JPRB, 0.37841E+00_JPRB, 0.38820E+00_JPRB, 0.40371E+00_JPRB ,&
&  0.41916E+00_JPRB, 0.42665E+00_JPRB, 0.42895E+00_JPRB, 0.42601E+00_JPRB, 0.42280E+00_JPRB, 0.42203E+00_JPRB, 0.42371E+00_JPRB ,&
&  0.42414E+00_JPRB, 0.42069E+00_JPRB, 0.41564E+00_JPRB, 0.41120E+00_JPRB, 0.40952E+00_JPRB, 0.40839E+00_JPRB, 0.40648E+00_JPRB ,&
&  0.40367E+00_JPRB, 0.40001E+00_JPRB, 0.39439E+00_JPRB, 0.38786E+00_JPRB, 0.38163E+00_JPRB, 0.37773E+00_JPRB, 0.37669E+00_JPRB ,&
&  0.37806E+00_JPRB, 0.38139E+00_JPRB, 0.38721E+00_JPRB, 0.39516E+00_JPRB, 0.40353E+00_JPRB, 0.41337E+00_JPRB, 0.42191E+00_JPRB ,&
&  0.42791E+00_JPRB, 0.43069E+00_JPRB, 0.43083E+00_JPRB, 0.42946E+00_JPRB, 0.42696E+00_JPRB, 0.42343E+00_JPRB, 0.41963E+00_JPRB ,&
&  0.41595E+00_JPRB, 0.41200E+00_JPRB, 0.40769E+00_JPRB, 0.40290E+00_JPRB, 0.39656E+00_JPRB, 0.38989E+00_JPRB, 0.38467E+00_JPRB ,&
&  0.38136E+00_JPRB, 0.37922E+00_JPRB, 0.37772E+00_JPRB, 0.37627E+00_JPRB, 0.37549E+00_JPRB, 0.37591E+00_JPRB, 0.37769E+00_JPRB ,&
&  0.38060E+00_JPRB, 0.38453E+00_JPRB, 0.38938E+00_JPRB, 0.39432E+00_JPRB, 0.39861E+00_JPRB, 0.40281E+00_JPRB, 0.40710E+00_JPRB ,&
&  0.40941E+00_JPRB, 0.40851E+00_JPRB, 0.40777E+00_JPRB, 0.40712E+00_JPRB, 0.40411E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.36829E+00_JPRB, 0.36819E+00_JPRB, 0.37312E+00_JPRB, 0.37828E+00_JPRB, 0.38407E+00_JPRB, 0.38620E+00_JPRB, 0.38394E+00_JPRB ,&
&  0.38087E+00_JPRB, 0.37672E+00_JPRB, 0.37043E+00_JPRB, 0.36264E+00_JPRB, 0.35565E+00_JPRB, 0.35384E+00_JPRB, 0.35407E+00_JPRB ,&
&  0.35428E+00_JPRB, 0.35181E+00_JPRB, 0.34742E+00_JPRB, 0.34266E+00_JPRB, 0.33737E+00_JPRB, 0.33273E+00_JPRB, 0.32924E+00_JPRB ,&
&  0.32817E+00_JPRB, 0.32764E+00_JPRB, 0.32840E+00_JPRB, 0.32921E+00_JPRB, 0.33021E+00_JPRB, 0.33281E+00_JPRB, 0.33948E+00_JPRB ,&
&  0.33950E+00_JPRB, 0.34852E+00_JPRB, 0.33804E+00_JPRB, 0.34124E+00_JPRB, 0.34488E+00_JPRB, 0.35051E+00_JPRB, 0.35900E+00_JPRB ,&
&  0.36950E+00_JPRB, 0.37677E+00_JPRB, 0.38103E+00_JPRB, 0.38069E+00_JPRB, 0.37826E+00_JPRB, 0.37844E+00_JPRB, 0.37939E+00_JPRB ,&
&  0.37993E+00_JPRB, 0.37975E+00_JPRB, 0.37750E+00_JPRB, 0.37246E+00_JPRB, 0.36560E+00_JPRB, 0.35976E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.35447E+00_JPRB, 0.35354E+00_JPRB, 0.35118E+00_JPRB, 0.34833E+00_JPRB, 0.34528E+00_JPRB, 0.34325E+00_JPRB, 0.34330E+00_JPRB ,&
&  0.34457E+00_JPRB, 0.34580E+00_JPRB, 0.34687E+00_JPRB, 0.34970E+00_JPRB, 0.35433E+00_JPRB, 0.35902E+00_JPRB, 0.36339E+00_JPRB ,&
&  0.36666E+00_JPRB, 0.36736E+00_JPRB, 0.36664E+00_JPRB, 0.36509E+00_JPRB, 0.36395E+00_JPRB, 0.36301E+00_JPRB, 0.36282E+00_JPRB ,&
&  0.36241E+00_JPRB, 0.36134E+00_JPRB, 0.35924E+00_JPRB, 0.35620E+00_JPRB, 0.35327E+00_JPRB, 0.35066E+00_JPRB, 0.34909E+00_JPRB ,&
&  0.34852E+00_JPRB, 0.34831E+00_JPRB, 0.34772E+00_JPRB, 0.34696E+00_JPRB, 0.34718E+00_JPRB, 0.34779E+00_JPRB, 0.34914E+00_JPRB ,&
&  0.35109E+00_JPRB, 0.35361E+00_JPRB, 0.35693E+00_JPRB, 0.36014E+00_JPRB, 0.36043E+00_JPRB, 0.36125E+00_JPRB, 0.36434E+00_JPRB ,&
&  0.36749E+00_JPRB, 0.36895E+00_JPRB, 0.36933E+00_JPRB, 0.36951E+00_JPRB, 0.36976E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.34347E+00_JPRB, 0.34568E+00_JPRB, 0.34988E+00_JPRB, 0.35616E+00_JPRB, 0.35982E+00_JPRB, 0.36403E+00_JPRB, 0.36410E+00_JPRB ,&
&  0.36142E+00_JPRB, 0.35769E+00_JPRB, 0.35317E+00_JPRB, 0.34736E+00_JPRB, 0.34117E+00_JPRB, 0.33655E+00_JPRB, 0.33495E+00_JPRB ,&
&  0.33476E+00_JPRB, 0.33329E+00_JPRB, 0.32999E+00_JPRB, 0.32570E+00_JPRB, 0.32098E+00_JPRB, 0.31709E+00_JPRB, 0.31411E+00_JPRB ,&
&  0.31330E+00_JPRB, 0.31390E+00_JPRB, 0.31528E+00_JPRB, 0.31619E+00_JPRB, 0.31641E+00_JPRB, 0.31891E+00_JPRB, 0.32398E+00_JPRB ,&
&  0.32309E+00_JPRB, 0.32192E+00_JPRB, 0.31410E+00_JPRB, 0.31347E+00_JPRB, 0.31556E+00_JPRB, 0.31849E+00_JPRB, 0.32253E+00_JPRB ,&
&  0.32745E+00_JPRB, 0.33203E+00_JPRB, 0.33669E+00_JPRB, 0.34036E+00_JPRB, 0.34275E+00_JPRB, 0.34463E+00_JPRB, 0.34588E+00_JPRB ,&
&  0.34658E+00_JPRB, 0.34585E+00_JPRB, 0.34353E+00_JPRB, 0.33928E+00_JPRB, 0.33390E+00_JPRB, 0.33010E+00_JPRB, 0.32837E+00_JPRB ,&
&  0.32676E+00_JPRB, 0.32638E+00_JPRB, 0.32585E+00_JPRB, 0.32446E+00_JPRB, 0.32260E+00_JPRB, 0.32173E+00_JPRB, 0.32198E+00_JPRB ,&
&  0.32221E+00_JPRB, 0.32149E+00_JPRB, 0.32006E+00_JPRB, 0.31983E+00_JPRB, 0.32136E+00_JPRB, 0.32389E+00_JPRB, 0.32603E+00_JPRB ,&
&  0.32756E+00_JPRB, 0.32733E+00_JPRB, 0.32621E+00_JPRB, 0.32398E+00_JPRB, 0.32235E+00_JPRB, 0.32239E+00_JPRB, 0.32378E+00_JPRB ,&
&  0.32482E+00_JPRB, 0.32484E+00_JPRB, 0.32480E+00_JPRB, 0.32320E+00_JPRB, 0.32149E+00_JPRB, 0.32100E+00_JPRB, 0.32099E+00_JPRB ,&
&  0.32060E+00_JPRB, 0.32001E+00_JPRB, 0.31948E+00_JPRB, 0.31938E+00_JPRB, 0.31997E+00_JPRB, 0.32008E+00_JPRB, 0.32098E+00_JPRB ,&
&  0.32361E+00_JPRB, 0.32675E+00_JPRB, 0.32952E+00_JPRB, 0.33113E+00_JPRB, 0.33165E+00_JPRB, 0.33143E+00_JPRB, 0.33371E+00_JPRB ,&
&  0.33734E+00_JPRB, 0.34014E+00_JPRB, 0.34038E+00_JPRB, 0.34002E+00_JPRB, 0.34152E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.32249E+00_JPRB, 0.32525E+00_JPRB, 0.32974E+00_JPRB, 0.33484E+00_JPRB, 0.33811E+00_JPRB, 0.34293E+00_JPRB, 0.34390E+00_JPRB ,&
&  0.34323E+00_JPRB, 0.34138E+00_JPRB, 0.33701E+00_JPRB, 0.33163E+00_JPRB, 0.32634E+00_JPRB, 0.32185E+00_JPRB, 0.31872E+00_JPRB ,&
&  0.31693E+00_JPRB, 0.31532E+00_JPRB, 0.31339E+00_JPRB, 0.31050E+00_JPRB, 0.30711E+00_JPRB, 0.30406E+00_JPRB, 0.30175E+00_JPRB ,&
&  0.30110E+00_JPRB, 0.30131E+00_JPRB, 0.30191E+00_JPRB, 0.30206E+00_JPRB, 0.30180E+00_JPRB, 0.30318E+00_JPRB, 0.30536E+00_JPRB ,&
&  0.30458E+00_JPRB, 0.30036E+00_JPRB, 0.29686E+00_JPRB, 0.29651E+00_JPRB, 0.29658E+00_JPRB, 0.29703E+00_JPRB, 0.29822E+00_JPRB ,&
&  0.29949E+00_JPRB, 0.30058E+00_JPRB, 0.30295E+00_JPRB, 0.30585E+00_JPRB, 0.30865E+00_JPRB, 0.31227E+00_JPRB, 0.31420E+00_JPRB ,&
&  0.31461E+00_JPRB, 0.31380E+00_JPRB, 0.31214E+00_JPRB, 0.30949E+00_JPRB, 0.30769E+00_JPRB, 0.30724E+00_JPRB, 0.30647E+00_JPRB ,&
&  0.30712E+00_JPRB, 0.30773E+00_JPRB, 0.30815E+00_JPRB, 0.30656E+00_JPRB, 0.30400E+00_JPRB, 0.30332E+00_JPRB, 0.30358E+00_JPRB ,&
&  0.30344E+00_JPRB, 0.30208E+00_JPRB, 0.29971E+00_JPRB, 0.29817E+00_JPRB, 0.29710E+00_JPRB, 0.29705E+00_JPRB, 0.29698E+00_JPRB ,&
&  0.29656E+00_JPRB, 0.29519E+00_JPRB, 0.29424E+00_JPRB, 0.29383E+00_JPRB, 0.29371E+00_JPRB, 0.29417E+00_JPRB, 0.29401E+00_JPRB ,&
&  0.29461E+00_JPRB, 0.29515E+00_JPRB, 0.29416E+00_JPRB, 0.29345E+00_JPRB, 0.29287E+00_JPRB, 0.29286E+00_JPRB, 0.29271E+00_JPRB ,&
&  0.29214E+00_JPRB, 0.29150E+00_JPRB, 0.29123E+00_JPRB, 0.29150E+00_JPRB, 0.29099E+00_JPRB, 0.28990E+00_JPRB, 0.29070E+00_JPRB ,&
&  0.29417E+00_JPRB, 0.29793E+00_JPRB, 0.30109E+00_JPRB, 0.30300E+00_JPRB, 0.30355E+00_JPRB, 0.30490E+00_JPRB, 0.30780E+00_JPRB ,&
&  0.31036E+00_JPRB, 0.31172E+00_JPRB, 0.31440E+00_JPRB, 0.31660E+00_JPRB, 0.31888E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.30419E+00_JPRB, 0.30806E+00_JPRB, 0.31084E+00_JPRB, 0.31442E+00_JPRB, 0.31745E+00_JPRB, 0.32260E+00_JPRB, 0.32472E+00_JPRB ,&
&  0.32468E+00_JPRB, 0.32449E+00_JPRB, 0.32303E+00_JPRB, 0.32032E+00_JPRB, 0.31605E+00_JPRB, 0.31156E+00_JPRB, 0.30728E+00_JPRB ,&
&  0.30393E+00_JPRB, 0.30121E+00_JPRB, 0.29907E+00_JPRB, 0.29717E+00_JPRB, 0.29459E+00_JPRB, 0.29220E+00_JPRB, 0.29042E+00_JPRB ,&
&  0.28977E+00_JPRB, 0.28936E+00_JPRB, 0.28927E+00_JPRB, 0.28942E+00_JPRB, 0.28914E+00_JPRB, 0.28870E+00_JPRB, 0.28872E+00_JPRB ,&
&  0.28741E+00_JPRB, 0.28357E+00_JPRB, 0.28123E+00_JPRB, 0.28042E+00_JPRB, 0.27964E+00_JPRB, 0.28013E+00_JPRB, 0.27992E+00_JPRB ,&
&  0.27993E+00_JPRB, 0.28021E+00_JPRB, 0.28212E+00_JPRB, 0.28478E+00_JPRB, 0.28580E+00_JPRB, 0.28736E+00_JPRB, 0.28957E+00_JPRB ,&
&  0.29059E+00_JPRB, 0.28980E+00_JPRB, 0.28779E+00_JPRB, 0.28741E+00_JPRB, 0.28854E+00_JPRB, 0.28878E+00_JPRB, 0.28919E+00_JPRB ,&
&  0.28979E+00_JPRB, 0.29084E+00_JPRB, 0.29124E+00_JPRB, 0.28900E+00_JPRB, 0.28675E+00_JPRB, 0.28590E+00_JPRB, 0.28575E+00_JPRB ,&
&  0.28533E+00_JPRB, 0.28408E+00_JPRB, 0.28153E+00_JPRB, 0.27904E+00_JPRB, 0.27567E+00_JPRB, 0.27296E+00_JPRB, 0.27152E+00_JPRB ,&
&  0.27183E+00_JPRB, 0.27227E+00_JPRB, 0.27365E+00_JPRB, 0.27483E+00_JPRB, 0.27554E+00_JPRB, 0.27555E+00_JPRB, 0.27397E+00_JPRB ,&
&  0.27202E+00_JPRB, 0.27117E+00_JPRB, 0.26996E+00_JPRB, 0.26865E+00_JPRB, 0.26884E+00_JPRB, 0.26871E+00_JPRB, 0.26830E+00_JPRB ,&
&  0.26795E+00_JPRB, 0.26865E+00_JPRB, 0.26930E+00_JPRB, 0.26882E+00_JPRB, 0.26776E+00_JPRB, 0.26660E+00_JPRB, 0.26718E+00_JPRB ,&
&  0.26889E+00_JPRB, 0.27141E+00_JPRB, 0.27425E+00_JPRB, 0.27722E+00_JPRB, 0.28017E+00_JPRB, 0.28280E+00_JPRB, 0.28554E+00_JPRB ,&
&  0.28889E+00_JPRB, 0.29005E+00_JPRB, 0.29245E+00_JPRB, 0.29603E+00_JPRB, 0.29944E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.28315E+00_JPRB, 0.28859E+00_JPRB, 0.29153E+00_JPRB, 0.29394E+00_JPRB, 0.29664E+00_JPRB, 0.30151E+00_JPRB, 0.30493E+00_JPRB ,&
&  0.30575E+00_JPRB, 0.30597E+00_JPRB, 0.30576E+00_JPRB, 0.30479E+00_JPRB, 0.30304E+00_JPRB, 0.30128E+00_JPRB, 0.29829E+00_JPRB ,&
&  0.29449E+00_JPRB, 0.29097E+00_JPRB, 0.28782E+00_JPRB, 0.28489E+00_JPRB, 0.28219E+00_JPRB, 0.27976E+00_JPRB, 0.27799E+00_JPRB ,&
&  0.27772E+00_JPRB, 0.27792E+00_JPRB, 0.27732E+00_JPRB, 0.27737E+00_JPRB, 0.27737E+00_JPRB, 0.27633E+00_JPRB, 0.27458E+00_JPRB ,&
&  0.27232E+00_JPRB, 0.26869E+00_JPRB, 0.26496E+00_JPRB, 0.26412E+00_JPRB, 0.26388E+00_JPRB, 0.26299E+00_JPRB, 0.26172E+00_JPRB ,&
&  0.26304E+00_JPRB, 0.26522E+00_JPRB, 0.26793E+00_JPRB, 0.26952E+00_JPRB, 0.27065E+00_JPRB, 0.27257E+00_JPRB, 0.27496E+00_JPRB ,&
&  0.27563E+00_JPRB, 0.27288E+00_JPRB, 0.27027E+00_JPRB, 0.27002E+00_JPRB, 0.27019E+00_JPRB, 0.26991E+00_JPRB, 0.27040E+00_JPRB ,&
&  0.27113E+00_JPRB, 0.27274E+00_JPRB, 0.27369E+00_JPRB, 0.27235E+00_JPRB, 0.26999E+00_JPRB, 0.26828E+00_JPRB, 0.26718E+00_JPRB ,&
&  0.26658E+00_JPRB, 0.26591E+00_JPRB, 0.26478E+00_JPRB, 0.26294E+00_JPRB, 0.26082E+00_JPRB, 0.25862E+00_JPRB, 0.25631E+00_JPRB ,&
&  0.25430E+00_JPRB, 0.25434E+00_JPRB, 0.25711E+00_JPRB, 0.25871E+00_JPRB, 0.25815E+00_JPRB, 0.25668E+00_JPRB, 0.25509E+00_JPRB ,&
&  0.25411E+00_JPRB, 0.25125E+00_JPRB, 0.24875E+00_JPRB, 0.24683E+00_JPRB, 0.24563E+00_JPRB, 0.24400E+00_JPRB, 0.24353E+00_JPRB ,&
&  0.24426E+00_JPRB, 0.24649E+00_JPRB, 0.24712E+00_JPRB, 0.24695E+00_JPRB, 0.24635E+00_JPRB, 0.24588E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.24845E+00_JPRB, 0.25098E+00_JPRB, 0.25383E+00_JPRB, 0.25762E+00_JPRB, 0.26083E+00_JPRB, 0.26462E+00_JPRB, 0.26763E+00_JPRB ,&
&  0.26946E+00_JPRB, 0.26996E+00_JPRB, 0.27128E+00_JPRB, 0.27353E+00_JPRB, 0.27724E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.26121E+00_JPRB, 0.26727E+00_JPRB, 0.27237E+00_JPRB, 0.27501E+00_JPRB, 0.27660E+00_JPRB, 0.27914E+00_JPRB, 0.28182E+00_JPRB ,&
&  0.28388E+00_JPRB, 0.28522E+00_JPRB, 0.28600E+00_JPRB, 0.28612E+00_JPRB, 0.28538E+00_JPRB, 0.28396E+00_JPRB, 0.28220E+00_JPRB ,&
&  0.28016E+00_JPRB, 0.27720E+00_JPRB, 0.27442E+00_JPRB, 0.27235E+00_JPRB, 0.27069E+00_JPRB, 0.26882E+00_JPRB, 0.26739E+00_JPRB ,&
&  0.26603E+00_JPRB, 0.26588E+00_JPRB, 0.26545E+00_JPRB, 0.26473E+00_JPRB, 0.26389E+00_JPRB, 0.26262E+00_JPRB, 0.26162E+00_JPRB ,&
&  0.25864E+00_JPRB, 0.25381E+00_JPRB, 0.25217E+00_JPRB, 0.25076E+00_JPRB, 0.25095E+00_JPRB, 0.25026E+00_JPRB, 0.24926E+00_JPRB ,&
&  0.25094E+00_JPRB, 0.25413E+00_JPRB, 0.25668E+00_JPRB, 0.25804E+00_JPRB, 0.25902E+00_JPRB, 0.26066E+00_JPRB, 0.26247E+00_JPRB ,&
&  0.26032E+00_JPRB, 0.25732E+00_JPRB, 0.25507E+00_JPRB, 0.25412E+00_JPRB, 0.25416E+00_JPRB, 0.25521E+00_JPRB, 0.25722E+00_JPRB ,&
&  0.25897E+00_JPRB, 0.25996E+00_JPRB, 0.25952E+00_JPRB, 0.25783E+00_JPRB, 0.25575E+00_JPRB, 0.25478E+00_JPRB, 0.25428E+00_JPRB ,&
&  0.25398E+00_JPRB, 0.25341E+00_JPRB, 0.25271E+00_JPRB, 0.25113E+00_JPRB, 0.24385E+00_JPRB, 0.24072E+00_JPRB, 0.23804E+00_JPRB ,&
&  0.23890E+00_JPRB, 0.23772E+00_JPRB, 0.23396E+00_JPRB, 0.24106E+00_JPRB, 0.24845E+00_JPRB, 0.24251E+00_JPRB, 0.23585E+00_JPRB ,&
&  0.22664E+00_JPRB, 0.22958E+00_JPRB, 0.22685E+00_JPRB, 0.22120E+00_JPRB, 0.21997E+00_JPRB, 0.22050E+00_JPRB, 0.22011E+00_JPRB ,&
&  0.22109E+00_JPRB, 0.22088E+00_JPRB, 0.22210E+00_JPRB, 0.22263E+00_JPRB, 0.22298E+00_JPRB, 0.22426E+00_JPRB, 0.22546E+00_JPRB ,&
&  0.22844E+00_JPRB, 0.23312E+00_JPRB, 0.23799E+00_JPRB, 0.23842E+00_JPRB, 0.24198E+00_JPRB, 0.24485E+00_JPRB, 0.24654E+00_JPRB ,&
&  0.24788E+00_JPRB, 0.24997E+00_JPRB, 0.25188E+00_JPRB, 0.25355E+00_JPRB, 0.25637E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.24620E+00_JPRB, 0.24887E+00_JPRB, 0.25208E+00_JPRB, 0.25524E+00_JPRB, 0.25748E+00_JPRB, 0.25974E+00_JPRB, 0.26177E+00_JPRB ,&
&  0.26357E+00_JPRB, 0.26484E+00_JPRB, 0.26574E+00_JPRB, 0.26633E+00_JPRB, 0.26676E+00_JPRB, 0.26662E+00_JPRB, 0.26604E+00_JPRB ,&
&  0.26543E+00_JPRB, 0.26426E+00_JPRB, 0.26259E+00_JPRB, 0.26123E+00_JPRB, 0.26034E+00_JPRB, 0.25909E+00_JPRB, 0.25731E+00_JPRB ,&
&  0.25547E+00_JPRB, 0.25377E+00_JPRB, 0.25233E+00_JPRB, 0.25077E+00_JPRB, 0.25054E+00_JPRB, 0.25116E+00_JPRB, 0.25174E+00_JPRB ,&
&  0.24585E+00_JPRB, 0.24032E+00_JPRB, 0.24255E+00_JPRB, 0.24364E+00_JPRB, 0.23962E+00_JPRB, 0.24104E+00_JPRB, 0.24185E+00_JPRB ,&
&  0.24291E+00_JPRB, 0.24448E+00_JPRB, 0.24613E+00_JPRB, 0.24690E+00_JPRB, 0.24748E+00_JPRB, 0.24845E+00_JPRB, 0.24812E+00_JPRB ,&
&  0.24609E+00_JPRB, 0.24428E+00_JPRB, 0.24260E+00_JPRB, 0.24084E+00_JPRB, 0.24041E+00_JPRB, 0.24246E+00_JPRB, 0.24375E+00_JPRB ,&
&  0.24263E+00_JPRB, 0.24085E+00_JPRB, 0.23912E+00_JPRB, 0.23954E+00_JPRB, 0.24049E+00_JPRB, 0.24095E+00_JPRB, 0.23831E+00_JPRB ,&
&  0.23757E+00_JPRB, 0.23444E+00_JPRB, 0.23168E+00_JPRB, 0.22890E+00_JPRB, 0.22785E+00_JPRB, 0.22382E+00_JPRB, 0.21935E+00_JPRB ,&
&  0.21858E+00_JPRB, 0.22348E+00_JPRB, 0.23037E+00_JPRB, 0.23141E+00_JPRB, 0.21518E+00_JPRB, 0.21865E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.21541E+00_JPRB, 0.20927E+00_JPRB, 0.20472E+00_JPRB, 0.20278E+00_JPRB, 0.20209E+00_JPRB, 0.20149E+00_JPRB, 0.20215E+00_JPRB ,&
&  0.20335E+00_JPRB, 0.20388E+00_JPRB, 0.20452E+00_JPRB, 0.20601E+00_JPRB, 0.20749E+00_JPRB, 0.20902E+00_JPRB, 0.21044E+00_JPRB ,&
&  0.21358E+00_JPRB, 0.21729E+00_JPRB, 0.21993E+00_JPRB, 0.22326E+00_JPRB, 0.22678E+00_JPRB, 0.23018E+00_JPRB, 0.23317E+00_JPRB ,&
&  0.23639E+00_JPRB, 0.23755E+00_JPRB, 0.23940E+00_JPRB, 0.24288E+00_JPRB, 0.24456E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.23897E+00_JPRB, 0.24216E+00_JPRB, 0.24420E+00_JPRB, 0.24619E+00_JPRB, 0.24774E+00_JPRB, 0.24936E+00_JPRB, 0.25015E+00_JPRB ,&
&  0.25091E+00_JPRB, 0.25098E+00_JPRB, 0.25155E+00_JPRB, 0.25163E+00_JPRB, 0.25195E+00_JPRB, 0.25075E+00_JPRB, 0.25033E+00_JPRB ,&
&  0.24994E+00_JPRB, 0.25054E+00_JPRB, 0.25069E+00_JPRB, 0.25064E+00_JPRB, 0.24980E+00_JPRB, 0.24865E+00_JPRB, 0.24684E+00_JPRB ,&
&  0.24319E+00_JPRB, 0.24036E+00_JPRB, 0.24039E+00_JPRB, 0.23964E+00_JPRB, 0.24004E+00_JPRB, 0.23991E+00_JPRB, 0.24041E+00_JPRB ,&
&  0.23520E+00_JPRB, 0.23380E+00_JPRB, 0.23501E+00_JPRB, 0.23361E+00_JPRB, 0.23473E+00_JPRB, 0.23537E+00_JPRB, 0.23744E+00_JPRB ,&
&  0.23789E+00_JPRB, 0.23817E+00_JPRB, 0.23916E+00_JPRB, 0.23859E+00_JPRB, 0.23899E+00_JPRB, 0.23980E+00_JPRB, 0.23873E+00_JPRB ,&
&  0.23509E+00_JPRB, 0.23241E+00_JPRB, 0.22844E+00_JPRB, 0.21935E+00_JPRB, 0.21804E+00_JPRB, 0.21909E+00_JPRB, 0.21793E+00_JPRB ,&
&  0.21490E+00_JPRB, 0.21596E+00_JPRB, 0.21544E+00_JPRB, 0.21335E+00_JPRB, 0.21569E+00_JPRB, 0.21916E+00_JPRB, 0.21926E+00_JPRB ,&
&  0.21758E+00_JPRB, 0.21711E+00_JPRB, 0.21631E+00_JPRB, 0.21477E+00_JPRB, 0.21289E+00_JPRB, 0.21216E+00_JPRB, 0.21149E+00_JPRB ,&
&  0.21110E+00_JPRB, 0.20916E+00_JPRB, 0.21272E+00_JPRB, 0.21292E+00_JPRB, 0.20669E+00_JPRB, 0.21053E+00_JPRB, 0.20831E+00_JPRB ,&
&  0.19980E+00_JPRB, 0.19610E+00_JPRB, 0.19394E+00_JPRB, 0.19208E+00_JPRB, 0.19170E+00_JPRB, 0.19171E+00_JPRB, 0.19141E+00_JPRB ,&
&  0.19245E+00_JPRB, 0.19382E+00_JPRB, 0.19443E+00_JPRB, 0.19530E+00_JPRB, 0.19653E+00_JPRB, 0.19837E+00_JPRB, 0.20080E+00_JPRB ,&
&  0.20294E+00_JPRB, 0.20584E+00_JPRB, 0.20719E+00_JPRB, 0.20944E+00_JPRB, 0.21108E+00_JPRB, 0.21301E+00_JPRB, 0.21777E+00_JPRB ,&
&  0.22139E+00_JPRB, 0.22849E+00_JPRB, 0.23783E+00_JPRB, 0.23964E+00_JPRB, 0.23579E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.23372E+00_JPRB, 0.23611E+00_JPRB, 0.23822E+00_JPRB, 0.23998E+00_JPRB, 0.24185E+00_JPRB, 0.24340E+00_JPRB, 0.24420E+00_JPRB ,&
&  0.24437E+00_JPRB, 0.24360E+00_JPRB, 0.24362E+00_JPRB, 0.24182E+00_JPRB, 0.24012E+00_JPRB, 0.23577E+00_JPRB, 0.23566E+00_JPRB ,&
&  0.23478E+00_JPRB, 0.23574E+00_JPRB, 0.23524E+00_JPRB, 0.23555E+00_JPRB, 0.23455E+00_JPRB, 0.23392E+00_JPRB, 0.23166E+00_JPRB ,&
&  0.22846E+00_JPRB, 0.22624E+00_JPRB, 0.22416E+00_JPRB, 0.22371E+00_JPRB, 0.22470E+00_JPRB, 0.22533E+00_JPRB, 0.22486E+00_JPRB ,&
&  0.22494E+00_JPRB, 0.22517E+00_JPRB, 0.22552E+00_JPRB, 0.22526E+00_JPRB, 0.23092E+00_JPRB, 0.23348E+00_JPRB, 0.23480E+00_JPRB ,&
&  0.23450E+00_JPRB, 0.23467E+00_JPRB, 0.23475E+00_JPRB, 0.23360E+00_JPRB, 0.23249E+00_JPRB, 0.22725E+00_JPRB, 0.22409E+00_JPRB ,&
&  0.21588E+00_JPRB, 0.20984E+00_JPRB, 0.20534E+00_JPRB, 0.20254E+00_JPRB, 0.20138E+00_JPRB, 0.20147E+00_JPRB, 0.20109E+00_JPRB ,&
&  0.19930E+00_JPRB, 0.19759E+00_JPRB, 0.19649E+00_JPRB, 0.19736E+00_JPRB, 0.19882E+00_JPRB, 0.20040E+00_JPRB, 0.20225E+00_JPRB ,&
&  0.20380E+00_JPRB, 0.20419E+00_JPRB, 0.20409E+00_JPRB, 0.20463E+00_JPRB, 0.20503E+00_JPRB, 0.20567E+00_JPRB, 0.20607E+00_JPRB ,&
&  0.20723E+00_JPRB, 0.20604E+00_JPRB, 0.20677E+00_JPRB, 0.20840E+00_JPRB, 0.20546E+00_JPRB, 0.20086E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19205E+00_JPRB, 0.18962E+00_JPRB, 0.18759E+00_JPRB, 0.18625E+00_JPRB, 0.18592E+00_JPRB, 0.18594E+00_JPRB, 0.18653E+00_JPRB ,&
&  0.18733E+00_JPRB, 0.18850E+00_JPRB, 0.18922E+00_JPRB, 0.19025E+00_JPRB, 0.19153E+00_JPRB, 0.19298E+00_JPRB, 0.19470E+00_JPRB ,&
&  0.19686E+00_JPRB, 0.19887E+00_JPRB, 0.20028E+00_JPRB, 0.20223E+00_JPRB, 0.20593E+00_JPRB, 0.21068E+00_JPRB, 0.21778E+00_JPRB ,&
&  0.22534E+00_JPRB, 0.23023E+00_JPRB, 0.22570E+00_JPRB, 0.22954E+00_JPRB, 0.23244E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.23362E+00_JPRB, 0.23568E+00_JPRB, 0.23730E+00_JPRB, 0.23899E+00_JPRB, 0.24034E+00_JPRB, 0.24158E+00_JPRB, 0.24238E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.23780E+00_JPRB, 0.23485E+00_JPRB, 0.23249E+00_JPRB, 0.22953E+00_JPRB, 0.22611E+00_JPRB, 0.22514E+00_JPRB ,&
&  0.22324E+00_JPRB, 0.22290E+00_JPRB, 0.22119E+00_JPRB, 0.22043E+00_JPRB, 0.21893E+00_JPRB, 0.21791E+00_JPRB, 0.21623E+00_JPRB ,&
&  0.21328E+00_JPRB, 0.21186E+00_JPRB, 0.21050E+00_JPRB, 0.21075E+00_JPRB, 0.21240E+00_JPRB, 0.21410E+00_JPRB, 0.21597E+00_JPRB ,&
&  0.21870E+00_JPRB, 0.21969E+00_JPRB, 0.22233E+00_JPRB, 0.22436E+00_JPRB, 0.22818E+00_JPRB, 0.23143E+00_JPRB, 0.23226E+00_JPRB ,&
&  0.23229E+00_JPRB, 0.22962E+00_JPRB, 0.22861E+00_JPRB, 0.22573E+00_JPRB, 0.21890E+00_JPRB, 0.21544E+00_JPRB, 0.21015E+00_JPRB ,&
&  0.20423E+00_JPRB, 0.19980E+00_JPRB, 0.19668E+00_JPRB, 0.19574E+00_JPRB, 0.19463E+00_JPRB, 0.19431E+00_JPRB, 0.19423E+00_JPRB ,&
&  0.19336E+00_JPRB, 0.19235E+00_JPRB, 0.19110E+00_JPRB, 0.19041E+00_JPRB, 0.19094E+00_JPRB, 0.19194E+00_JPRB, 0.19412E+00_JPRB ,&
&  0.19628E+00_JPRB, 0.19751E+00_JPRB, 0.19884E+00_JPRB, 0.19928E+00_JPRB, 0.19941E+00_JPRB, 0.19969E+00_JPRB, 0.19971E+00_JPRB ,&
&  0.20055E+00_JPRB, 0.20071E+00_JPRB, 0.20007E+00_JPRB, 0.19904E+00_JPRB, 0.19629E+00_JPRB, 0.19264E+00_JPRB, 0.18986E+00_JPRB ,&
&  0.18745E+00_JPRB, 0.18589E+00_JPRB, 0.18537E+00_JPRB, 0.18479E+00_JPRB, 0.18509E+00_JPRB, 0.18555E+00_JPRB, 0.18593E+00_JPRB ,&
&  0.18662E+00_JPRB, 0.18723E+00_JPRB, 0.18783E+00_JPRB, 0.18930E+00_JPRB, 0.19050E+00_JPRB, 0.19231E+00_JPRB, 0.19421E+00_JPRB ,&
&  0.19552E+00_JPRB, 0.19750E+00_JPRB, 0.19919E+00_JPRB, 0.20104E+00_JPRB, 0.20676E+00_JPRB, 0.21110E+00_JPRB, 0.21999E+00_JPRB ,&
&  0.22519E+00_JPRB, 0.22534E+00_JPRB, 0.22386E+00_JPRB, 0.23102E+00_JPRB, 0.23239E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.23358E+00_JPRB, 0.23452E+00_JPRB, 0.23459E+00_JPRB, 0.23571E+00_JPRB, 0.23641E+00_JPRB, 0.23717E+00_JPRB, 0.23590E+00_JPRB ,&
&  0.23485E+00_JPRB, 0.23354E+00_JPRB, 0.23249E+00_JPRB, 0.22913E+00_JPRB, 0.22652E+00_JPRB, 0.22322E+00_JPRB, 0.22036E+00_JPRB ,&
&  0.21780E+00_JPRB, 0.21533E+00_JPRB, 0.21300E+00_JPRB, 0.21099E+00_JPRB, 0.20907E+00_JPRB, 0.20732E+00_JPRB, 0.20555E+00_JPRB ,&
&  0.20291E+00_JPRB, 0.20216E+00_JPRB, 0.20208E+00_JPRB, 0.20350E+00_JPRB, 0.20655E+00_JPRB, 0.20831E+00_JPRB, 0.21051E+00_JPRB ,&
&  0.21260E+00_JPRB, 0.21139E+00_JPRB, 0.21509E+00_JPRB, 0.21861E+00_JPRB, 0.22153E+00_JPRB, 0.22407E+00_JPRB, 0.22638E+00_JPRB ,&
&  0.22705E+00_JPRB, 0.22035E+00_JPRB, 0.22132E+00_JPRB, 0.21572E+00_JPRB, 0.21040E+00_JPRB, 0.20848E+00_JPRB, 0.20420E+00_JPRB ,&
&  0.20278E+00_JPRB, 0.19937E+00_JPRB, 0.19717E+00_JPRB, 0.19516E+00_JPRB, 0.19259E+00_JPRB, 0.19202E+00_JPRB, 0.19107E+00_JPRB ,&
&  0.19060E+00_JPRB, 0.19086E+00_JPRB, 0.19001E+00_JPRB, 0.18981E+00_JPRB, 0.18933E+00_JPRB, 0.18891E+00_JPRB, 0.18953E+00_JPRB ,&
&  0.19048E+00_JPRB, 0.19153E+00_JPRB, 0.19251E+00_JPRB, 0.19266E+00_JPRB, 0.19256E+00_JPRB, 0.19253E+00_JPRB, 0.19240E+00_JPRB ,&
&  0.19276E+00_JPRB, 0.19280E+00_JPRB, 0.19232E+00_JPRB, 0.19175E+00_JPRB, 0.19042E+00_JPRB, 0.18908E+00_JPRB, 0.18776E+00_JPRB ,&
&  0.18658E+00_JPRB, 0.18594E+00_JPRB, 0.18538E+00_JPRB, 0.18521E+00_JPRB, 0.18521E+00_JPRB, 0.18559E+00_JPRB, 0.18597E+00_JPRB ,&
&  0.18633E+00_JPRB, 0.18723E+00_JPRB, 0.18789E+00_JPRB, 0.18899E+00_JPRB, 0.19031E+00_JPRB, 0.19148E+00_JPRB, 0.19328E+00_JPRB ,&
&  0.19523E+00_JPRB, 0.19735E+00_JPRB, 0.20044E+00_JPRB, 0.20304E+00_JPRB, 0.20636E+00_JPRB, 0.21052E+00_JPRB, 0.21341E+00_JPRB ,&
&  0.21650E+00_JPRB, 0.21896E+00_JPRB, 0.21854E+00_JPRB, 0.22705E+00_JPRB, 0.23179E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.22378E+00_JPRB, 0.22482E+00_JPRB, 0.22536E+00_JPRB, 0.22579E+00_JPRB, 0.22703E+00_JPRB, 0.22700E+00_JPRB, 0.22674E+00_JPRB ,&
&  0.22428E+00_JPRB, 0.22345E+00_JPRB, 0.22202E+00_JPRB, 0.21992E+00_JPRB, 0.21774E+00_JPRB, 0.21391E+00_JPRB, 0.21169E+00_JPRB ,&
&  0.20958E+00_JPRB, 0.20734E+00_JPRB, 0.20596E+00_JPRB, 0.20404E+00_JPRB, 0.20237E+00_JPRB, 0.20041E+00_JPRB, 0.19980E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19863E+00_JPRB, 0.19955E+00_JPRB, 0.20019E+00_JPRB, 0.20151E+00_JPRB, 0.20289E+00_JPRB, 0.20471E+00_JPRB ,&
&  0.20493E+00_JPRB, 0.20565E+00_JPRB, 0.20644E+00_JPRB, 0.20701E+00_JPRB, 0.20922E+00_JPRB, 0.21077E+00_JPRB, 0.21137E+00_JPRB ,&
&  0.21091E+00_JPRB, 0.20986E+00_JPRB, 0.20784E+00_JPRB, 0.20548E+00_JPRB, 0.20288E+00_JPRB, 0.20008E+00_JPRB, 0.19740E+00_JPRB ,&
&  0.19538E+00_JPRB, 0.19347E+00_JPRB, 0.19202E+00_JPRB, 0.19083E+00_JPRB, 0.19013E+00_JPRB, 0.18957E+00_JPRB, 0.18900E+00_JPRB ,&
&  0.18872E+00_JPRB, 0.18850E+00_JPRB, 0.18813E+00_JPRB, 0.18822E+00_JPRB, 0.18818E+00_JPRB, 0.18832E+00_JPRB, 0.18848E+00_JPRB ,&
&  0.18878E+00_JPRB, 0.18887E+00_JPRB, 0.18885E+00_JPRB, 0.18856E+00_JPRB, 0.18827E+00_JPRB, 0.18791E+00_JPRB, 0.18754E+00_JPRB ,&
&  0.18714E+00_JPRB, 0.18713E+00_JPRB, 0.18678E+00_JPRB, 0.18653E+00_JPRB, 0.18619E+00_JPRB, 0.18558E+00_JPRB, 0.18538E+00_JPRB ,&
&  0.18543E+00_JPRB, 0.18555E+00_JPRB, 0.18531E+00_JPRB, 0.18542E+00_JPRB, 0.18563E+00_JPRB, 0.18595E+00_JPRB, 0.18694E+00_JPRB ,&
&  0.18753E+00_JPRB, 0.18815E+00_JPRB, 0.18870E+00_JPRB, 0.18951E+00_JPRB, 0.19095E+00_JPRB, 0.19239E+00_JPRB, 0.19399E+00_JPRB ,&
&  0.19592E+00_JPRB, 0.19772E+00_JPRB, 0.19952E+00_JPRB, 0.20128E+00_JPRB, 0.20424E+00_JPRB, 0.20622E+00_JPRB, 0.20880E+00_JPRB ,&
&  0.21096E+00_JPRB, 0.20893E+00_JPRB, 0.21803E+00_JPRB, 0.21862E+00_JPRB, 0.21987E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.20651E+00_JPRB, 0.20633E+00_JPRB, 0.20507E+00_JPRB, 0.20366E+00_JPRB, 0.19509E+00_JPRB, 0.20182E+00_JPRB, 0.20118E+00_JPRB ,&
&  0.20197E+00_JPRB, 0.20280E+00_JPRB, 0.20471E+00_JPRB, 0.20520E+00_JPRB, 0.20509E+00_JPRB, 0.20518E+00_JPRB, 0.20506E+00_JPRB ,&
&  0.20484E+00_JPRB, 0.20405E+00_JPRB, 0.20374E+00_JPRB, 0.20314E+00_JPRB, 0.20252E+00_JPRB, 0.20139E+00_JPRB, 0.20016E+00_JPRB ,&
&  0.19959E+00_JPRB, 0.19965E+00_JPRB, 0.20008E+00_JPRB, 0.19794E+00_JPRB, 0.19691E+00_JPRB, 0.19621E+00_JPRB, 0.19583E+00_JPRB ,&
&  0.19525E+00_JPRB, 0.19506E+00_JPRB, 0.19506E+00_JPRB, 0.19501E+00_JPRB, 0.19670E+00_JPRB, 0.19725E+00_JPRB, 0.19768E+00_JPRB ,&
&  0.19779E+00_JPRB, 0.19766E+00_JPRB, 0.19692E+00_JPRB, 0.19622E+00_JPRB, 0.19552E+00_JPRB, 0.19423E+00_JPRB, 0.19289E+00_JPRB ,&
&  0.19174E+00_JPRB, 0.19088E+00_JPRB, 0.18988E+00_JPRB, 0.18911E+00_JPRB, 0.18849E+00_JPRB, 0.18790E+00_JPRB, 0.18752E+00_JPRB ,&
&  0.18744E+00_JPRB, 0.18746E+00_JPRB, 0.18747E+00_JPRB, 0.18748E+00_JPRB, 0.18747E+00_JPRB, 0.18749E+00_JPRB, 0.18753E+00_JPRB ,&
&  0.18801E+00_JPRB, 0.18802E+00_JPRB, 0.18785E+00_JPRB, 0.18753E+00_JPRB, 0.18744E+00_JPRB, 0.18709E+00_JPRB, 0.18684E+00_JPRB ,&
&  0.18664E+00_JPRB, 0.18639E+00_JPRB, 0.18627E+00_JPRB, 0.18624E+00_JPRB, 0.18625E+00_JPRB, 0.18623E+00_JPRB, 0.18634E+00_JPRB ,&
&  0.18651E+00_JPRB, 0.18673E+00_JPRB, 0.18717E+00_JPRB, 0.18752E+00_JPRB, 0.18784E+00_JPRB, 0.18811E+00_JPRB, 0.18871E+00_JPRB ,&
&  0.18932E+00_JPRB, 0.18993E+00_JPRB, 0.19052E+00_JPRB, 0.19090E+00_JPRB, 0.19179E+00_JPRB, 0.19268E+00_JPRB, 0.19366E+00_JPRB ,&
&  0.19472E+00_JPRB, 0.19606E+00_JPRB, 0.19746E+00_JPRB, 0.19886E+00_JPRB, 0.20073E+00_JPRB, 0.20220E+00_JPRB, 0.20364E+00_JPRB ,&
&  0.20524E+00_JPRB, 0.20825E+00_JPRB, 0.20892E+00_JPRB, 0.20891E+00_JPRB, 0.20804E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.20165E+00_JPRB, 0.20157E+00_JPRB, 0.20137E+00_JPRB, 0.20085E+00_JPRB, 0.19991E+00_JPRB, 0.19968E+00_JPRB, 0.19938E+00_JPRB ,&
&  0.19896E+00_JPRB, 0.19812E+00_JPRB, 0.19810E+00_JPRB, 0.19799E+00_JPRB, 0.19785E+00_JPRB, 0.19770E+00_JPRB, 0.19738E+00_JPRB ,&
&  0.19674E+00_JPRB, 0.19577E+00_JPRB, 0.19292E+00_JPRB, 0.19155E+00_JPRB, 0.19066E+00_JPRB, 0.19016E+00_JPRB, 0.18975E+00_JPRB ,&
&  0.18950E+00_JPRB, 0.18937E+00_JPRB, 0.18923E+00_JPRB, 0.18976E+00_JPRB, 0.19001E+00_JPRB, 0.19009E+00_JPRB, 0.19016E+00_JPRB ,&
&  0.19016E+00_JPRB, 0.19028E+00_JPRB, 0.19029E+00_JPRB, 0.19028E+00_JPRB, 0.19044E+00_JPRB, 0.19046E+00_JPRB, 0.19024E+00_JPRB ,&
&  0.19000E+00_JPRB, 0.18971E+00_JPRB, 0.18943E+00_JPRB, 0.18908E+00_JPRB, 0.18883E+00_JPRB, 0.18862E+00_JPRB, 0.18855E+00_JPRB ,&
&  0.18838E+00_JPRB, 0.18825E+00_JPRB, 0.18811E+00_JPRB, 0.18806E+00_JPRB, 0.18795E+00_JPRB, 0.18791E+00_JPRB, 0.18777E+00_JPRB ,&
&  0.18789E+00_JPRB, 0.18794E+00_JPRB, 0.18803E+00_JPRB, 0.18809E+00_JPRB, 0.18813E+00_JPRB, 0.18812E+00_JPRB, 0.18805E+00_JPRB ,&
&  0.18786E+00_JPRB, 0.18775E+00_JPRB, 0.18761E+00_JPRB, 0.18751E+00_JPRB, 0.18749E+00_JPRB, 0.18747E+00_JPRB, 0.18748E+00_JPRB ,&
&  0.18749E+00_JPRB, 0.18762E+00_JPRB, 0.18767E+00_JPRB, 0.18774E+00_JPRB, 0.18782E+00_JPRB, 0.18791E+00_JPRB, 0.18797E+00_JPRB ,&
&  0.18808E+00_JPRB, 0.18818E+00_JPRB, 0.18831E+00_JPRB, 0.18848E+00_JPRB, 0.18869E+00_JPRB, 0.18898E+00_JPRB, 0.18933E+00_JPRB ,&
&  0.18969E+00_JPRB, 0.19006E+00_JPRB, 0.19042E+00_JPRB, 0.19078E+00_JPRB, 0.19135E+00_JPRB, 0.19192E+00_JPRB, 0.19262E+00_JPRB ,&
&  0.19330E+00_JPRB, 0.19398E+00_JPRB, 0.19459E+00_JPRB, 0.19523E+00_JPRB, 0.19614E+00_JPRB, 0.19658E+00_JPRB, 0.19694E+00_JPRB ,&
&  0.19755E+00_JPRB, 0.19818E+00_JPRB, 0.19879E+00_JPRB, 0.19932E+00_JPRB, 0.20017E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB, 0.18999E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB, 0.87418E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.87859E+00_JPRB, 0.87592E+00_JPRB, 0.87350E+00_JPRB, 0.87095E+00_JPRB, 0.86831E+00_JPRB, 0.86617E+00_JPRB, 0.86414E+00_JPRB ,&
&  0.86234E+00_JPRB, 0.86136E+00_JPRB, 0.86084E+00_JPRB, 0.86058E+00_JPRB, 0.86065E+00_JPRB, 0.86060E+00_JPRB, 0.86066E+00_JPRB ,&
&  0.86062E+00_JPRB, 0.86075E+00_JPRB, 0.86074E+00_JPRB, 0.86034E+00_JPRB, 0.85959E+00_JPRB, 0.85858E+00_JPRB, 0.85728E+00_JPRB ,&
&  0.85543E+00_JPRB, 0.85335E+00_JPRB, 0.85134E+00_JPRB, 0.84918E+00_JPRB, 0.84716E+00_JPRB, 0.84486E+00_JPRB, 0.84242E+00_JPRB ,&
&  0.84022E+00_JPRB, 0.83891E+00_JPRB, 0.83782E+00_JPRB, 0.83670E+00_JPRB, 0.83609E+00_JPRB, 0.83694E+00_JPRB, 0.83765E+00_JPRB ,&
&  0.83843E+00_JPRB, 0.83938E+00_JPRB, 0.84159E+00_JPRB, 0.84377E+00_JPRB, 0.84583E+00_JPRB, 0.84801E+00_JPRB, 0.85033E+00_JPRB ,&
&  0.85157E+00_JPRB, 0.85230E+00_JPRB, 0.85268E+00_JPRB, 0.85313E+00_JPRB, 0.85309E+00_JPRB, 0.85330E+00_JPRB, 0.85346E+00_JPRB ,&
&  0.85345E+00_JPRB, 0.85347E+00_JPRB, 0.85315E+00_JPRB, 0.85199E+00_JPRB, 0.84891E+00_JPRB, 0.84668E+00_JPRB, 0.84573E+00_JPRB ,&
&  0.84586E+00_JPRB, 0.84626E+00_JPRB, 0.84714E+00_JPRB, 0.84849E+00_JPRB, 0.85074E+00_JPRB, 0.85325E+00_JPRB, 0.85623E+00_JPRB ,&
&  0.85946E+00_JPRB, 0.86336E+00_JPRB, 0.86827E+00_JPRB, 0.87335E+00_JPRB, 0.87892E+00_JPRB, 0.88545E+00_JPRB, 0.89145E+00_JPRB ,&
&  0.89767E+00_JPRB, 0.90422E+00_JPRB, 0.90974E+00_JPRB, 0.91502E+00_JPRB, 0.92012E+00_JPRB, 0.92530E+00_JPRB, 0.92912E+00_JPRB ,&
&  0.93091E+00_JPRB, 0.93217E+00_JPRB, 0.93306E+00_JPRB, 0.93221E+00_JPRB, 0.93018E+00_JPRB, 0.92850E+00_JPRB, 0.92674E+00_JPRB ,&
&  0.92452E+00_JPRB, 0.92175E+00_JPRB, 0.91908E+00_JPRB, 0.91611E+00_JPRB, 0.91202E+00_JPRB, 0.90754E+00_JPRB, 0.90328E+00_JPRB ,&
&  0.89908E+00_JPRB, 0.89446E+00_JPRB, 0.89021E+00_JPRB, 0.88611E+00_JPRB, 0.88229E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.90730E+00_JPRB, 0.89750E+00_JPRB, 0.88856E+00_JPRB, 0.88036E+00_JPRB, 0.87362E+00_JPRB, 0.86840E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.85997E+00_JPRB, 0.85802E+00_JPRB, 0.85611E+00_JPRB, 0.85434E+00_JPRB, 0.85267E+00_JPRB, 0.85063E+00_JPRB, 0.84817E+00_JPRB ,&
&  0.84590E+00_JPRB, 0.84437E+00_JPRB, 0.84286E+00_JPRB, 0.84085E+00_JPRB, 0.83713E+00_JPRB, 0.83148E+00_JPRB, 0.82486E+00_JPRB ,&
&  0.81719E+00_JPRB, 0.80796E+00_JPRB, 0.79737E+00_JPRB, 0.78619E+00_JPRB, 0.78033E+00_JPRB, 0.77355E+00_JPRB, 0.76706E+00_JPRB ,&
&  0.76133E+00_JPRB, 0.75948E+00_JPRB, 0.75804E+00_JPRB, 0.75709E+00_JPRB, 0.75714E+00_JPRB, 0.75897E+00_JPRB, 0.75995E+00_JPRB ,&
&  0.76059E+00_JPRB, 0.76278E+00_JPRB, 0.76975E+00_JPRB, 0.77727E+00_JPRB, 0.78604E+00_JPRB, 0.79677E+00_JPRB, 0.80897E+00_JPRB ,&
&  0.81828E+00_JPRB, 0.82414E+00_JPRB, 0.83027E+00_JPRB, 0.83599E+00_JPRB, 0.83995E+00_JPRB, 0.84284E+00_JPRB, 0.84491E+00_JPRB ,&
&  0.84853E+00_JPRB, 0.85092E+00_JPRB, 0.85311E+00_JPRB, 0.85451E+00_JPRB, 0.85369E+00_JPRB, 0.85258E+00_JPRB, 0.85091E+00_JPRB ,&
&  0.84823E+00_JPRB, 0.84789E+00_JPRB, 0.84974E+00_JPRB, 0.85194E+00_JPRB, 0.85648E+00_JPRB, 0.86316E+00_JPRB, 0.87085E+00_JPRB ,&
&  0.88122E+00_JPRB, 0.89535E+00_JPRB, 0.91260E+00_JPRB, 0.93171E+00_JPRB, 0.95383E+00_JPRB, 0.98148E+00_JPRB, 0.10040E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.10418E+01_JPRB, 0.10571E+01_JPRB, 0.10680E+01_JPRB, 0.10756E+01_JPRB, 0.10826E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10771E+01_JPRB, 0.10747E+01_JPRB, 0.10690E+01_JPRB, 0.10621E+01_JPRB, 0.10549E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.10378E+01_JPRB, 0.10275E+01_JPRB, 0.10173E+01_JPRB, 0.10074E+01_JPRB, 0.99611E+00_JPRB, 0.98521E+00_JPRB, 0.97440E+00_JPRB ,&
&  0.96350E+00_JPRB, 0.95181E+00_JPRB, 0.94060E+00_JPRB, 0.92965E+00_JPRB, 0.91860E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.94252E+00_JPRB, 0.92618E+00_JPRB, 0.91129E+00_JPRB, 0.89809E+00_JPRB, 0.88657E+00_JPRB, 0.87732E+00_JPRB, 0.86967E+00_JPRB ,&
&  0.86320E+00_JPRB, 0.85926E+00_JPRB, 0.85538E+00_JPRB, 0.85120E+00_JPRB, 0.84699E+00_JPRB, 0.84625E+00_JPRB, 0.84344E+00_JPRB ,&
&  0.83917E+00_JPRB, 0.83368E+00_JPRB, 0.82843E+00_JPRB, 0.82060E+00_JPRB, 0.80916E+00_JPRB, 0.79543E+00_JPRB, 0.77844E+00_JPRB ,&
&  0.76768E+00_JPRB, 0.75391E+00_JPRB, 0.73935E+00_JPRB, 0.72448E+00_JPRB, 0.71407E+00_JPRB, 0.69073E+00_JPRB, 0.68256E+00_JPRB ,&
&  0.67725E+00_JPRB, 0.66069E+00_JPRB, 0.65182E+00_JPRB, 0.64763E+00_JPRB, 0.66240E+00_JPRB, 0.68245E+00_JPRB, 0.69364E+00_JPRB ,&
&  0.68223E+00_JPRB, 0.69189E+00_JPRB, 0.68903E+00_JPRB, 0.68621E+00_JPRB, 0.68766E+00_JPRB, 0.69265E+00_JPRB, 0.71807E+00_JPRB ,&
&  0.76379E+00_JPRB, 0.78199E+00_JPRB, 0.79333E+00_JPRB, 0.80457E+00_JPRB, 0.81257E+00_JPRB, 0.82029E+00_JPRB, 0.82650E+00_JPRB ,&
&  0.83334E+00_JPRB, 0.83932E+00_JPRB, 0.84660E+00_JPRB, 0.84986E+00_JPRB, 0.85519E+00_JPRB, 0.86040E+00_JPRB, 0.86485E+00_JPRB ,&
&  0.86131E+00_JPRB, 0.86017E+00_JPRB, 0.86210E+00_JPRB, 0.86755E+00_JPRB, 0.87675E+00_JPRB, 0.89266E+00_JPRB, 0.91052E+00_JPRB ,&
&  0.92994E+00_JPRB, 0.94536E+00_JPRB, 0.97039E+00_JPRB, 0.10043E+01_JPRB, 0.10454E+01_JPRB, 0.10962E+01_JPRB, 0.11472E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.12389E+01_JPRB, 0.12783E+01_JPRB, 0.12976E+01_JPRB, 0.13074E+01_JPRB, 0.13139E+01_JPRB, 0.13132E+01_JPRB ,&
&  0.13105E+01_JPRB, 0.13030E+01_JPRB, 0.12951E+01_JPRB, 0.12840E+01_JPRB, 0.12689E+01_JPRB, 0.12491E+01_JPRB, 0.12266E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.11744E+01_JPRB, 0.11490E+01_JPRB, 0.11241E+01_JPRB, 0.10992E+01_JPRB, 0.10763E+01_JPRB, 0.10558E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.10163E+01_JPRB, 0.99762E+00_JPRB, 0.97946E+00_JPRB, 0.96113E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.97657E+00_JPRB, 0.95857E+00_JPRB, 0.93953E+00_JPRB, 0.92222E+00_JPRB, 0.90770E+00_JPRB, 0.89657E+00_JPRB, 0.88851E+00_JPRB ,&
&  0.88133E+00_JPRB, 0.87250E+00_JPRB, 0.86305E+00_JPRB, 0.85292E+00_JPRB, 0.84396E+00_JPRB, 0.83969E+00_JPRB, 0.83623E+00_JPRB ,&
&  0.83359E+00_JPRB, 0.82631E+00_JPRB, 0.81679E+00_JPRB, 0.80216E+00_JPRB, 0.78621E+00_JPRB, 0.77150E+00_JPRB, 0.75359E+00_JPRB ,&
&  0.74001E+00_JPRB, 0.72597E+00_JPRB, 0.70518E+00_JPRB, 0.69336E+00_JPRB, 0.66491E+00_JPRB, 0.64571E+00_JPRB, 0.63344E+00_JPRB ,&
&  0.62437E+00_JPRB, 0.62573E+00_JPRB, 0.63066E+00_JPRB, 0.64000E+00_JPRB, 0.64429E+00_JPRB, 0.64144E+00_JPRB, 0.64190E+00_JPRB ,&
&  0.63482E+00_JPRB, 0.62996E+00_JPRB, 0.62655E+00_JPRB, 0.62596E+00_JPRB, 0.62927E+00_JPRB, 0.63752E+00_JPRB, 0.65483E+00_JPRB ,&
&  0.69134E+00_JPRB, 0.73849E+00_JPRB, 0.78120E+00_JPRB, 0.80320E+00_JPRB, 0.80542E+00_JPRB, 0.81346E+00_JPRB, 0.81507E+00_JPRB ,&
&  0.81997E+00_JPRB, 0.82221E+00_JPRB, 0.82935E+00_JPRB, 0.83897E+00_JPRB, 0.84987E+00_JPRB, 0.85982E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87531E+00_JPRB, 0.87965E+00_JPRB, 0.87877E+00_JPRB, 0.88755E+00_JPRB, 0.90611E+00_JPRB, 0.92623E+00_JPRB, 0.93580E+00_JPRB ,&
&  0.95232E+00_JPRB, 0.97451E+00_JPRB, 0.10160E+01_JPRB, 0.10788E+01_JPRB, 0.11470E+01_JPRB, 0.12103E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.13970E+01_JPRB, 0.14629E+01_JPRB, 0.14892E+01_JPRB, 0.14881E+01_JPRB, 0.14938E+01_JPRB, 0.14822E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.14606E+01_JPRB, 0.14537E+01_JPRB, 0.14467E+01_JPRB, 0.14260E+01_JPRB, 0.13941E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.12830E+01_JPRB, 0.12447E+01_JPRB, 0.12058E+01_JPRB, 0.11687E+01_JPRB, 0.11326E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.10723E+01_JPRB, 0.10491E+01_JPRB, 0.10275E+01_JPRB, 0.10100E+01_JPRB, 0.99342E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.10110E+01_JPRB, 0.99893E+00_JPRB, 0.98676E+00_JPRB, 0.97077E+00_JPRB, 0.95435E+00_JPRB, 0.93935E+00_JPRB, 0.92736E+00_JPRB ,&
&  0.91711E+00_JPRB, 0.90578E+00_JPRB, 0.89438E+00_JPRB, 0.88052E+00_JPRB, 0.86732E+00_JPRB, 0.85530E+00_JPRB, 0.84548E+00_JPRB ,&
&  0.83720E+00_JPRB, 0.82455E+00_JPRB, 0.81234E+00_JPRB, 0.79705E+00_JPRB, 0.78247E+00_JPRB, 0.76594E+00_JPRB, 0.74667E+00_JPRB ,&
&  0.73218E+00_JPRB, 0.71111E+00_JPRB, 0.69315E+00_JPRB, 0.68293E+00_JPRB, 0.65623E+00_JPRB, 0.64240E+00_JPRB, 0.63460E+00_JPRB ,&
&  0.64968E+00_JPRB, 0.64660E+00_JPRB, 0.66193E+00_JPRB, 0.64448E+00_JPRB, 0.63211E+00_JPRB, 0.62075E+00_JPRB, 0.60847E+00_JPRB ,&
&  0.60043E+00_JPRB, 0.59156E+00_JPRB, 0.59269E+00_JPRB, 0.59834E+00_JPRB, 0.60222E+00_JPRB, 0.61485E+00_JPRB, 0.63158E+00_JPRB ,&
&  0.66890E+00_JPRB, 0.77054E+00_JPRB, 0.79150E+00_JPRB, 0.81379E+00_JPRB, 0.81822E+00_JPRB, 0.82463E+00_JPRB, 0.82888E+00_JPRB ,&
&  0.83379E+00_JPRB, 0.83610E+00_JPRB, 0.83831E+00_JPRB, 0.84233E+00_JPRB, 0.86074E+00_JPRB, 0.87368E+00_JPRB, 0.88956E+00_JPRB ,&
&  0.89583E+00_JPRB, 0.90378E+00_JPRB, 0.90787E+00_JPRB, 0.91939E+00_JPRB, 0.94550E+00_JPRB, 0.95858E+00_JPRB, 0.96186E+00_JPRB ,&
&  0.98806E+00_JPRB, 0.10243E+01_JPRB, 0.10810E+01_JPRB, 0.11501E+01_JPRB, 0.12023E+01_JPRB, 0.12467E+01_JPRB, 0.13133E+01_JPRB ,&
&  0.13595E+01_JPRB, 0.14360E+01_JPRB, 0.15166E+01_JPRB, 0.15534E+01_JPRB, 0.15585E+01_JPRB, 0.15471E+01_JPRB, 0.15320E+01_JPRB ,&
&  0.15277E+01_JPRB, 0.15044E+01_JPRB, 0.14906E+01_JPRB, 0.14832E+01_JPRB, 0.14578E+01_JPRB, 0.14267E+01_JPRB, 0.13942E+01_JPRB ,&
&  0.13531E+01_JPRB, 0.13164E+01_JPRB, 0.12857E+01_JPRB, 0.12503E+01_JPRB, 0.12126E+01_JPRB, 0.11759E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11058E+01_JPRB, 0.10780E+01_JPRB, 0.10521E+01_JPRB, 0.10353E+01_JPRB, 0.10203E+01_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.10388E+01_JPRB, 0.10269E+01_JPRB, 0.10158E+01_JPRB, 0.10051E+01_JPRB, 0.99290E+00_JPRB, 0.98062E+00_JPRB, 0.96779E+00_JPRB ,&
&  0.95656E+00_JPRB, 0.94580E+00_JPRB, 0.93410E+00_JPRB, 0.91896E+00_JPRB, 0.90303E+00_JPRB, 0.88512E+00_JPRB, 0.87043E+00_JPRB ,&
&  0.85500E+00_JPRB, 0.83861E+00_JPRB, 0.81588E+00_JPRB, 0.79476E+00_JPRB, 0.78003E+00_JPRB, 0.76597E+00_JPRB, 0.74863E+00_JPRB ,&
&  0.73344E+00_JPRB, 0.71319E+00_JPRB, 0.68992E+00_JPRB, 0.67075E+00_JPRB, 0.64606E+00_JPRB, 0.64090E+00_JPRB, 0.67492E+00_JPRB ,&
&  0.69146E+00_JPRB, 0.69146E+00_JPRB, 0.68613E+00_JPRB, 0.67571E+00_JPRB, 0.66526E+00_JPRB, 0.61757E+00_JPRB, 0.62517E+00_JPRB ,&
&  0.60561E+00_JPRB, 0.58901E+00_JPRB, 0.58794E+00_JPRB, 0.58976E+00_JPRB, 0.59770E+00_JPRB, 0.61596E+00_JPRB, 0.63586E+00_JPRB ,&
&  0.67249E+00_JPRB, 0.78918E+00_JPRB, 0.81194E+00_JPRB, 0.83339E+00_JPRB, 0.84023E+00_JPRB, 0.84649E+00_JPRB, 0.85264E+00_JPRB ,&
&  0.85646E+00_JPRB, 0.86382E+00_JPRB, 0.87162E+00_JPRB, 0.88325E+00_JPRB, 0.89719E+00_JPRB, 0.90456E+00_JPRB, 0.92397E+00_JPRB ,&
&  0.93812E+00_JPRB, 0.95125E+00_JPRB, 0.95973E+00_JPRB, 0.96060E+00_JPRB, 0.97349E+00_JPRB, 0.96875E+00_JPRB, 0.96949E+00_JPRB ,&
&  0.10207E+01_JPRB, 0.10956E+01_JPRB, 0.11606E+01_JPRB, 0.12204E+01_JPRB, 0.12728E+01_JPRB, 0.13314E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.14538E+01_JPRB, 0.15269E+01_JPRB, 0.16125E+01_JPRB, 0.16413E+01_JPRB, 0.16393E+01_JPRB, 0.16331E+01_JPRB, 0.16358E+01_JPRB ,&
&  0.16393E+01_JPRB, 0.16465E+01_JPRB, 0.16009E+01_JPRB, 0.15733E+01_JPRB, 0.15153E+01_JPRB, 0.14414E+01_JPRB, 0.13815E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.12445E+01_JPRB, 0.12049E+01_JPRB, 0.11759E+01_JPRB, 0.11546E+01_JPRB, 0.11404E+01_JPRB, 0.11277E+01_JPRB ,&
&  0.11159E+01_JPRB, 0.11003E+01_JPRB, 0.10831E+01_JPRB, 0.10659E+01_JPRB, 0.10510E+01_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.10457E+01_JPRB, 0.10371E+01_JPRB, 0.10249E+01_JPRB, 0.10143E+01_JPRB, 0.10008E+01_JPRB, 0.99105E+00_JPRB, 0.98124E+00_JPRB ,&
&  0.97529E+00_JPRB, 0.97174E+00_JPRB, 0.96717E+00_JPRB, 0.96236E+00_JPRB, 0.95387E+00_JPRB, 0.94281E+00_JPRB, 0.92797E+00_JPRB ,&
&  0.90654E+00_JPRB, 0.85833E+00_JPRB, 0.82337E+00_JPRB, 0.80644E+00_JPRB, 0.78957E+00_JPRB, 0.77553E+00_JPRB, 0.76156E+00_JPRB ,&
&  0.74160E+00_JPRB, 0.72399E+00_JPRB, 0.70307E+00_JPRB, 0.67530E+00_JPRB, 0.66144E+00_JPRB, 0.67141E+00_JPRB, 0.67697E+00_JPRB ,&
&  0.68452E+00_JPRB, 0.70269E+00_JPRB, 0.70013E+00_JPRB, 0.68830E+00_JPRB, 0.68832E+00_JPRB, 0.69003E+00_JPRB, 0.64902E+00_JPRB ,&
&  0.61697E+00_JPRB, 0.59188E+00_JPRB, 0.58816E+00_JPRB, 0.58872E+00_JPRB, 0.59771E+00_JPRB, 0.61727E+00_JPRB, 0.66569E+00_JPRB ,&
&  0.74757E+00_JPRB, 0.84759E+00_JPRB, 0.84829E+00_JPRB, 0.85212E+00_JPRB, 0.86158E+00_JPRB, 0.87032E+00_JPRB, 0.87997E+00_JPRB ,&
&  0.89038E+00_JPRB, 0.90690E+00_JPRB, 0.92777E+00_JPRB, 0.94905E+00_JPRB, 0.97395E+00_JPRB, 0.99039E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10316E+01_JPRB, 0.10196E+01_JPRB, 0.10196E+01_JPRB, 0.10106E+01_JPRB, 0.10025E+01_JPRB, 0.98433E+00_JPRB, 0.10122E+01_JPRB ,&
&  0.10785E+01_JPRB, 0.11456E+01_JPRB, 0.12153E+01_JPRB, 0.12716E+01_JPRB, 0.13321E+01_JPRB, 0.14140E+01_JPRB, 0.15056E+01_JPRB ,&
&  0.15886E+01_JPRB, 0.16979E+01_JPRB, 0.18488E+01_JPRB, 0.18298E+01_JPRB, 0.17993E+01_JPRB, 0.17812E+01_JPRB, 0.17857E+01_JPRB ,&
&  0.17841E+01_JPRB, 0.18017E+01_JPRB, 0.17195E+01_JPRB, 0.15982E+01_JPRB, 0.14907E+01_JPRB, 0.13683E+01_JPRB, 0.12966E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.11697E+01_JPRB, 0.11318E+01_JPRB, 0.11104E+01_JPRB, 0.10968E+01_JPRB, 0.10814E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10609E+01_JPRB, 0.10609E+01_JPRB, 0.10567E+01_JPRB, 0.10519E+01_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.10258E+01_JPRB, 0.10287E+01_JPRB, 0.10260E+01_JPRB, 0.10165E+01_JPRB, 0.99025E+00_JPRB, 0.95309E+00_JPRB, 0.92829E+00_JPRB ,&
&  0.91250E+00_JPRB, 0.90945E+00_JPRB, 0.90450E+00_JPRB, 0.90084E+00_JPRB, 0.90323E+00_JPRB, 0.91137E+00_JPRB, 0.91886E+00_JPRB ,&
&  0.91927E+00_JPRB, 0.90602E+00_JPRB, 0.88117E+00_JPRB, 0.84341E+00_JPRB, 0.81310E+00_JPRB, 0.79081E+00_JPRB, 0.77756E+00_JPRB ,&
&  0.76726E+00_JPRB, 0.75302E+00_JPRB, 0.73555E+00_JPRB, 0.70672E+00_JPRB, 0.68698E+00_JPRB, 0.67195E+00_JPRB, 0.67943E+00_JPRB ,&
&  0.69998E+00_JPRB, 0.70644E+00_JPRB, 0.70100E+00_JPRB, 0.71483E+00_JPRB, 0.71772E+00_JPRB, 0.72542E+00_JPRB, 0.67305E+00_JPRB ,&
&  0.63466E+00_JPRB, 0.61766E+00_JPRB, 0.60530E+00_JPRB, 0.60258E+00_JPRB, 0.61645E+00_JPRB, 0.64489E+00_JPRB, 0.70262E+00_JPRB ,&
&  0.82142E+00_JPRB, 0.85542E+00_JPRB, 0.86837E+00_JPRB, 0.88569E+00_JPRB, 0.89942E+00_JPRB, 0.90633E+00_JPRB, 0.92103E+00_JPRB ,&
&  0.95039E+00_JPRB, 0.99593E+00_JPRB, 0.10415E+01_JPRB, 0.10504E+01_JPRB, 0.10519E+01_JPRB, 0.10558E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.13608E+01_JPRB, 0.11966E+01_JPRB, 0.10982E+01_JPRB, 0.10504E+01_JPRB, 0.10311E+01_JPRB, 0.10693E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11989E+01_JPRB, 0.12881E+01_JPRB, 0.13573E+01_JPRB, 0.14316E+01_JPRB, 0.15335E+01_JPRB, 0.16767E+01_JPRB ,&
&  0.18229E+01_JPRB, 0.20117E+01_JPRB, 0.24043E+01_JPRB, 0.21451E+01_JPRB, 0.19931E+01_JPRB, 0.19382E+01_JPRB, 0.19174E+01_JPRB ,&
&  0.18545E+01_JPRB, 0.18509E+01_JPRB, 0.17216E+01_JPRB, 0.15488E+01_JPRB, 0.14599E+01_JPRB, 0.13621E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.11567E+01_JPRB, 0.11214E+01_JPRB, 0.10947E+01_JPRB, 0.10865E+01_JPRB, 0.10771E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.10496E+01_JPRB, 0.10364E+01_JPRB, 0.10273E+01_JPRB, 0.10209E+01_JPRB, 0.10197E+01_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.99117E+00_JPRB, 0.10129E+01_JPRB, 0.10293E+01_JPRB, 0.10077E+01_JPRB, 0.97106E+00_JPRB, 0.90559E+00_JPRB, 0.85422E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.80508E+00_JPRB, 0.79968E+00_JPRB, 0.80101E+00_JPRB, 0.80938E+00_JPRB, 0.82439E+00_JPRB, 0.83486E+00_JPRB ,&
&  0.84612E+00_JPRB, 0.84658E+00_JPRB, 0.84587E+00_JPRB, 0.83379E+00_JPRB, 0.81207E+00_JPRB, 0.78692E+00_JPRB, 0.76959E+00_JPRB ,&
&  0.75915E+00_JPRB, 0.75217E+00_JPRB, 0.74231E+00_JPRB, 0.72661E+00_JPRB, 0.71547E+00_JPRB, 0.70034E+00_JPRB, 0.70225E+00_JPRB ,&
&  0.69350E+00_JPRB, 0.69923E+00_JPRB, 0.71672E+00_JPRB, 0.72440E+00_JPRB, 0.73919E+00_JPRB, 0.74970E+00_JPRB, 0.70975E+00_JPRB ,&
&  0.66603E+00_JPRB, 0.64545E+00_JPRB, 0.64009E+00_JPRB, 0.64386E+00_JPRB, 0.69160E+00_JPRB, 0.80167E+00_JPRB, 0.85580E+00_JPRB ,&
&  0.88578E+00_JPRB, 0.90407E+00_JPRB, 0.92457E+00_JPRB, 0.94948E+00_JPRB, 0.95936E+00_JPRB, 0.97039E+00_JPRB, 0.10018E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.11422E+01_JPRB, 0.12065E+01_JPRB, 0.10854E+01_JPRB, 0.10313E+01_JPRB, 0.10752E+01_JPRB, 0.11418E+01_JPRB ,&
&  0.12286E+01_JPRB, 0.12066E+01_JPRB, 0.11742E+01_JPRB, 0.12010E+01_JPRB, 0.12075E+01_JPRB, 0.12150E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.12408E+01_JPRB, 0.13069E+01_JPRB, 0.14492E+01_JPRB, 0.15255E+01_JPRB, 0.16448E+01_JPRB, 0.18188E+01_JPRB, 0.19727E+01_JPRB ,&
&  0.21495E+01_JPRB, 0.22775E+01_JPRB, 0.23351E+01_JPRB, 0.22477E+01_JPRB, 0.21822E+01_JPRB, 0.21768E+01_JPRB, 0.21311E+01_JPRB ,&
&  0.19701E+01_JPRB, 0.18531E+01_JPRB, 0.17549E+01_JPRB, 0.16564E+01_JPRB, 0.15212E+01_JPRB, 0.14080E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.11796E+01_JPRB, 0.10972E+01_JPRB, 0.10556E+01_JPRB, 0.10274E+01_JPRB, 0.10186E+01_JPRB, 0.10155E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.98239E+00_JPRB, 0.96072E+00_JPRB, 0.95029E+00_JPRB, 0.95712E+00_JPRB, 0.98157E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.10392E+01_JPRB, 0.10331E+01_JPRB, 0.99568E+00_JPRB, 0.99795E+00_JPRB, 0.96451E+00_JPRB, 0.92945E+00_JPRB, 0.87850E+00_JPRB ,&
&  0.84284E+00_JPRB, 0.80289E+00_JPRB, 0.77402E+00_JPRB, 0.73749E+00_JPRB, 0.71608E+00_JPRB, 0.70740E+00_JPRB, 0.72707E+00_JPRB ,&
&  0.75480E+00_JPRB, 0.77501E+00_JPRB, 0.78760E+00_JPRB, 0.79054E+00_JPRB, 0.78696E+00_JPRB, 0.77537E+00_JPRB, 0.76281E+00_JPRB ,&
&  0.75176E+00_JPRB, 0.74962E+00_JPRB, 0.74283E+00_JPRB, 0.73479E+00_JPRB, 0.72567E+00_JPRB, 0.71643E+00_JPRB, 0.71212E+00_JPRB ,&
&  0.70121E+00_JPRB, 0.69703E+00_JPRB, 0.70503E+00_JPRB, 0.72505E+00_JPRB, 0.74862E+00_JPRB, 0.76236E+00_JPRB, 0.72950E+00_JPRB ,&
&  0.69467E+00_JPRB, 0.68755E+00_JPRB, 0.70236E+00_JPRB, 0.80128E+00_JPRB, 0.84098E+00_JPRB, 0.86268E+00_JPRB, 0.89472E+00_JPRB ,&
&  0.88166E+00_JPRB, 0.88174E+00_JPRB, 0.90487E+00_JPRB, 0.98074E+00_JPRB, 0.10077E+01_JPRB, 0.10296E+01_JPRB, 0.10734E+01_JPRB ,&
&  0.11454E+01_JPRB, 0.12493E+01_JPRB, 0.13370E+01_JPRB, 0.12280E+01_JPRB, 0.13051E+01_JPRB, 0.15278E+01_JPRB, 0.14049E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.13055E+01_JPRB, 0.13382E+01_JPRB, 0.13746E+01_JPRB, 0.13715E+01_JPRB, 0.14110E+01_JPRB, 0.14483E+01_JPRB ,&
&  0.14335E+01_JPRB, 0.15113E+01_JPRB, 0.16835E+01_JPRB, 0.18510E+01_JPRB, 0.20088E+01_JPRB, 0.22146E+01_JPRB, 0.24272E+01_JPRB ,&
&  0.26858E+01_JPRB, 0.29118E+01_JPRB, 0.30650E+01_JPRB, 0.30262E+01_JPRB, 0.29427E+01_JPRB, 0.27935E+01_JPRB, 0.25798E+01_JPRB ,&
&  0.22895E+01_JPRB, 0.20825E+01_JPRB, 0.19456E+01_JPRB, 0.18360E+01_JPRB, 0.17004E+01_JPRB, 0.15223E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.10063E+01_JPRB, 0.96172E+00_JPRB, 0.94982E+00_JPRB, 0.95584E+00_JPRB, 0.96155E+00_JPRB, 0.95512E+00_JPRB ,&
&  0.94409E+00_JPRB, 0.93612E+00_JPRB, 0.93138E+00_JPRB, 0.94519E+00_JPRB, 0.10084E+01_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.10787E+01_JPRB, 0.10551E+01_JPRB, 0.10296E+01_JPRB, 0.10219E+01_JPRB, 0.10082E+01_JPRB, 0.95570E+00_JPRB, 0.87381E+00_JPRB ,&
&  0.87808E+00_JPRB, 0.83533E+00_JPRB, 0.82342E+00_JPRB, 0.80810E+00_JPRB, 0.78443E+00_JPRB, 0.73886E+00_JPRB, 0.70408E+00_JPRB ,&
&  0.70354E+00_JPRB, 0.72445E+00_JPRB, 0.74751E+00_JPRB, 0.76641E+00_JPRB, 0.77345E+00_JPRB, 0.76889E+00_JPRB, 0.76058E+00_JPRB ,&
&  0.75318E+00_JPRB, 0.75082E+00_JPRB, 0.75189E+00_JPRB, 0.75325E+00_JPRB, 0.74956E+00_JPRB, 0.74447E+00_JPRB, 0.74389E+00_JPRB ,&
&  0.74026E+00_JPRB, 0.73475E+00_JPRB, 0.72902E+00_JPRB, 0.73910E+00_JPRB, 0.74998E+00_JPRB, 0.77397E+00_JPRB, 0.79112E+00_JPRB ,&
&  0.73157E+00_JPRB, 0.74658E+00_JPRB, 0.83924E+00_JPRB, 0.86358E+00_JPRB, 0.88962E+00_JPRB, 0.91255E+00_JPRB, 0.93198E+00_JPRB ,&
&  0.95338E+00_JPRB, 0.98346E+00_JPRB, 0.10190E+01_JPRB, 0.10525E+01_JPRB, 0.10777E+01_JPRB, 0.11179E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.12835E+01_JPRB, 0.12229E+01_JPRB, 0.11915E+01_JPRB, 0.12835E+01_JPRB, 0.16833E+01_JPRB, 0.17443E+01_JPRB, 0.17225E+01_JPRB ,&
&  0.17165E+01_JPRB, 0.16397E+01_JPRB, 0.16172E+01_JPRB, 0.16607E+01_JPRB, 0.16908E+01_JPRB, 0.17813E+01_JPRB, 0.19116E+01_JPRB ,&
&  0.19210E+01_JPRB, 0.20506E+01_JPRB, 0.22128E+01_JPRB, 0.23509E+01_JPRB, 0.25055E+01_JPRB, 0.27419E+01_JPRB, 0.30141E+01_JPRB ,&
&  0.32997E+01_JPRB, 0.35392E+01_JPRB, 0.36350E+01_JPRB, 0.35640E+01_JPRB, 0.33922E+01_JPRB, 0.31784E+01_JPRB, 0.29062E+01_JPRB ,&
&  0.25539E+01_JPRB, 0.23104E+01_JPRB, 0.21512E+01_JPRB, 0.19976E+01_JPRB, 0.18570E+01_JPRB, 0.16697E+01_JPRB, 0.13928E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.97981E+00_JPRB, 0.95546E+00_JPRB, 0.95277E+00_JPRB, 0.96195E+00_JPRB, 0.98018E+00_JPRB, 0.99553E+00_JPRB ,&
&  0.10029E+01_JPRB, 0.98489E+00_JPRB, 0.98181E+00_JPRB, 0.10165E+01_JPRB, 0.10590E+01_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.10455E+01_JPRB, 0.10359E+01_JPRB, 0.10402E+01_JPRB, 0.10555E+01_JPRB, 0.10358E+01_JPRB, 0.98247E+00_JPRB, 0.92875E+00_JPRB ,&
&  0.10435E+01_JPRB, 0.10357E+01_JPRB, 0.10884E+01_JPRB, 0.92845E+00_JPRB, 0.87255E+00_JPRB, 0.84338E+00_JPRB, 0.79947E+00_JPRB ,&
&  0.76505E+00_JPRB, 0.74487E+00_JPRB, 0.73769E+00_JPRB, 0.75154E+00_JPRB, 0.75903E+00_JPRB, 0.75557E+00_JPRB, 0.75135E+00_JPRB ,&
&  0.74769E+00_JPRB, 0.74962E+00_JPRB, 0.75857E+00_JPRB, 0.76114E+00_JPRB, 0.76208E+00_JPRB, 0.76410E+00_JPRB, 0.76946E+00_JPRB ,&
&  0.75000E+00_JPRB, 0.74488E+00_JPRB, 0.74891E+00_JPRB, 0.75056E+00_JPRB, 0.76444E+00_JPRB, 0.77618E+00_JPRB, 0.80184E+00_JPRB ,&
&  0.85130E+00_JPRB, 0.82932E+00_JPRB, 0.92406E+00_JPRB, 0.94394E+00_JPRB, 0.96656E+00_JPRB, 0.98703E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10198E+01_JPRB, 0.10451E+01_JPRB, 0.10777E+01_JPRB, 0.11052E+01_JPRB, 0.11386E+01_JPRB, 0.12057E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.12925E+01_JPRB, 0.14011E+01_JPRB, 0.14679E+01_JPRB, 0.17401E+01_JPRB, 0.20310E+01_JPRB, 0.20231E+01_JPRB ,&
&  0.23117E+01_JPRB, 0.21756E+01_JPRB, 0.20640E+01_JPRB, 0.19701E+01_JPRB, 0.20328E+01_JPRB, 0.21601E+01_JPRB, 0.23071E+01_JPRB ,&
&  0.24373E+01_JPRB, 0.25528E+01_JPRB, 0.27554E+01_JPRB, 0.29173E+01_JPRB, 0.30925E+01_JPRB, 0.33088E+01_JPRB, 0.35154E+01_JPRB ,&
&  0.37009E+01_JPRB, 0.38169E+01_JPRB, 0.38780E+01_JPRB, 0.36580E+01_JPRB, 0.34690E+01_JPRB, 0.32705E+01_JPRB, 0.28706E+01_JPRB ,&
&  0.25586E+01_JPRB, 0.23310E+01_JPRB, 0.21455E+01_JPRB, 0.20097E+01_JPRB, 0.17697E+01_JPRB, 0.15708E+01_JPRB, 0.13770E+01_JPRB ,&
&  0.11201E+01_JPRB, 0.10282E+01_JPRB, 0.10352E+01_JPRB, 0.10527E+01_JPRB, 0.10540E+01_JPRB, 0.10637E+01_JPRB, 0.10690E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10423E+01_JPRB, 0.10748E+01_JPRB, 0.10716E+01_JPRB, 0.10591E+01_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.11120E+01_JPRB, 0.11275E+01_JPRB, 0.11332E+01_JPRB, 0.11187E+01_JPRB, 0.10908E+01_JPRB, 0.11030E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.11885E+01_JPRB, 0.11856E+01_JPRB, 0.11869E+01_JPRB, 0.11976E+01_JPRB, 0.11022E+01_JPRB, 0.93757E+00_JPRB ,&
&  0.85734E+00_JPRB, 0.84331E+00_JPRB, 0.84086E+00_JPRB, 0.83730E+00_JPRB, 0.83703E+00_JPRB, 0.83201E+00_JPRB, 0.83132E+00_JPRB ,&
&  0.82958E+00_JPRB, 0.82153E+00_JPRB, 0.82284E+00_JPRB, 0.82061E+00_JPRB, 0.81292E+00_JPRB, 0.80898E+00_JPRB, 0.81222E+00_JPRB ,&
&  0.78151E+00_JPRB, 0.76355E+00_JPRB, 0.75213E+00_JPRB, 0.73883E+00_JPRB, 0.76675E+00_JPRB, 0.79261E+00_JPRB, 0.83288E+00_JPRB ,&
&  0.88943E+00_JPRB, 0.94695E+00_JPRB, 0.98502E+00_JPRB, 0.10194E+01_JPRB, 0.10474E+01_JPRB, 0.10729E+01_JPRB, 0.10970E+01_JPRB ,&
&  0.11205E+01_JPRB, 0.11462E+01_JPRB, 0.11781E+01_JPRB, 0.12026E+01_JPRB, 0.12622E+01_JPRB, 0.13520E+01_JPRB, 0.15798E+01_JPRB ,&
&  0.16861E+01_JPRB, 0.18454E+01_JPRB, 0.19622E+01_JPRB, 0.17952E+01_JPRB, 0.19857E+01_JPRB, 0.22413E+01_JPRB, 0.22585E+01_JPRB ,&
&  0.23388E+01_JPRB, 0.23360E+01_JPRB, 0.24125E+01_JPRB, 0.25283E+01_JPRB, 0.28260E+01_JPRB, 0.30438E+01_JPRB, 0.31407E+01_JPRB ,&
&  0.32203E+01_JPRB, 0.35103E+01_JPRB, 0.37111E+01_JPRB, 0.38097E+01_JPRB, 0.38363E+01_JPRB, 0.38777E+01_JPRB, 0.39546E+01_JPRB ,&
&  0.40148E+01_JPRB, 0.40758E+01_JPRB, 0.40555E+01_JPRB, 0.39191E+01_JPRB, 0.37082E+01_JPRB, 0.34262E+01_JPRB, 0.30408E+01_JPRB ,&
&  0.27024E+01_JPRB, 0.22929E+01_JPRB, 0.19607E+01_JPRB, 0.17626E+01_JPRB, 0.15802E+01_JPRB, 0.14009E+01_JPRB, 0.12781E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.11437E+01_JPRB, 0.12376E+01_JPRB, 0.12477E+01_JPRB, 0.12487E+01_JPRB, 0.12191E+01_JPRB, 0.11255E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.11308E+01_JPRB, 0.11481E+01_JPRB, 0.11350E+01_JPRB, 0.11167E+01_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.12941E+01_JPRB, 0.12825E+01_JPRB, 0.12630E+01_JPRB, 0.12364E+01_JPRB, 0.12305E+01_JPRB, 0.12362E+01_JPRB, 0.12394E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.12850E+01_JPRB, 0.12839E+01_JPRB, 0.12589E+01_JPRB, 0.12282E+01_JPRB, 0.12073E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.96872E+00_JPRB, 0.93944E+00_JPRB, 0.93223E+00_JPRB, 0.96225E+00_JPRB, 0.10037E+01_JPRB, 0.10126E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.10235E+01_JPRB, 0.99693E+00_JPRB, 0.99223E+00_JPRB, 0.98446E+00_JPRB, 0.96588E+00_JPRB, 0.94747E+00_JPRB, 0.92862E+00_JPRB ,&
&  0.86395E+00_JPRB, 0.82198E+00_JPRB, 0.80172E+00_JPRB, 0.79126E+00_JPRB, 0.81625E+00_JPRB, 0.86250E+00_JPRB, 0.92054E+00_JPRB ,&
&  0.98562E+00_JPRB, 0.10388E+01_JPRB, 0.10889E+01_JPRB, 0.11343E+01_JPRB, 0.11656E+01_JPRB, 0.11955E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.12824E+01_JPRB, 0.13010E+01_JPRB, 0.13335E+01_JPRB, 0.13780E+01_JPRB, 0.17075E+01_JPRB, 0.21444E+01_JPRB ,&
&  0.21398E+01_JPRB, 0.22078E+01_JPRB, 0.23526E+01_JPRB, 0.25876E+01_JPRB, 0.27063E+01_JPRB, 0.26510E+01_JPRB, 0.27016E+01_JPRB ,&
&  0.29387E+01_JPRB, 0.32158E+01_JPRB, 0.37791E+01_JPRB, 0.38196E+01_JPRB, 0.39066E+01_JPRB, 0.40261E+01_JPRB, 0.39825E+01_JPRB ,&
&  0.40664E+01_JPRB, 0.41341E+01_JPRB, 0.42872E+01_JPRB, 0.42533E+01_JPRB, 0.41662E+01_JPRB, 0.41883E+01_JPRB, 0.41739E+01_JPRB ,&
&  0.42227E+01_JPRB, 0.42496E+01_JPRB, 0.40388E+01_JPRB, 0.38526E+01_JPRB, 0.35061E+01_JPRB, 0.35055E+01_JPRB, 0.32442E+01_JPRB ,&
&  0.27307E+01_JPRB, 0.20497E+01_JPRB, 0.18707E+01_JPRB, 0.16771E+01_JPRB, 0.15524E+01_JPRB, 0.14229E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.12875E+01_JPRB, 0.13857E+01_JPRB, 0.14079E+01_JPRB, 0.14220E+01_JPRB, 0.14360E+01_JPRB, 0.14004E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.12347E+01_JPRB, 0.12918E+01_JPRB, 0.13123E+01_JPRB, 0.13074E+01_JPRB, 0.13002E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.15496E+01_JPRB, 0.14995E+01_JPRB, 0.14579E+01_JPRB, 0.14330E+01_JPRB, 0.14074E+01_JPRB, 0.13723E+01_JPRB, 0.13348E+01_JPRB ,&
&  0.13456E+01_JPRB, 0.13743E+01_JPRB, 0.13759E+01_JPRB, 0.13296E+01_JPRB, 0.12763E+01_JPRB, 0.12380E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.11059E+01_JPRB, 0.11275E+01_JPRB, 0.12178E+01_JPRB, 0.14258E+01_JPRB, 0.14258E+01_JPRB, 0.14667E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.13901E+01_JPRB, 0.13613E+01_JPRB, 0.13305E+01_JPRB, 0.12764E+01_JPRB, 0.12344E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.10542E+01_JPRB, 0.96553E+00_JPRB, 0.93425E+00_JPRB, 0.92039E+00_JPRB, 0.95555E+00_JPRB, 0.10008E+01_JPRB, 0.10761E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.12049E+01_JPRB, 0.12636E+01_JPRB, 0.13200E+01_JPRB, 0.13589E+01_JPRB, 0.13945E+01_JPRB, 0.14209E+01_JPRB ,&
&  0.14454E+01_JPRB, 0.14625E+01_JPRB, 0.14882E+01_JPRB, 0.15087E+01_JPRB, 0.16465E+01_JPRB, 0.22874E+01_JPRB, 0.29872E+01_JPRB ,&
&  0.29933E+01_JPRB, 0.30815E+01_JPRB, 0.39466E+01_JPRB, 0.48673E+01_JPRB, 0.44535E+01_JPRB, 0.39799E+01_JPRB, 0.38283E+01_JPRB ,&
&  0.40859E+01_JPRB, 0.44737E+01_JPRB, 0.48847E+01_JPRB, 0.49946E+01_JPRB, 0.49081E+01_JPRB, 0.49557E+01_JPRB, 0.49156E+01_JPRB ,&
&  0.48072E+01_JPRB, 0.47581E+01_JPRB, 0.46351E+01_JPRB, 0.44908E+01_JPRB, 0.43617E+01_JPRB, 0.44411E+01_JPRB, 0.44164E+01_JPRB ,&
&  0.43421E+01_JPRB, 0.41511E+01_JPRB, 0.38301E+01_JPRB, 0.36530E+01_JPRB, 0.30659E+01_JPRB, 0.29787E+01_JPRB, 0.33401E+01_JPRB ,&
&  0.27764E+01_JPRB, 0.22133E+01_JPRB, 0.19149E+01_JPRB, 0.17587E+01_JPRB, 0.17237E+01_JPRB, 0.18282E+01_JPRB, 0.18315E+01_JPRB ,&
&  0.16201E+01_JPRB, 0.16667E+01_JPRB, 0.18111E+01_JPRB, 0.18688E+01_JPRB, 0.18731E+01_JPRB, 0.18401E+01_JPRB, 0.16726E+01_JPRB ,&
&  0.16678E+01_JPRB, 0.16167E+01_JPRB, 0.16187E+01_JPRB, 0.16141E+01_JPRB, 0.15926E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.18138E+01_JPRB, 0.17163E+01_JPRB, 0.16592E+01_JPRB, 0.15880E+01_JPRB, 0.15072E+01_JPRB, 0.14328E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.14222E+01_JPRB, 0.14398E+01_JPRB, 0.14195E+01_JPRB, 0.13622E+01_JPRB, 0.13152E+01_JPRB, 0.12839E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.12987E+01_JPRB, 0.13078E+01_JPRB, 0.13289E+01_JPRB, 0.16164E+01_JPRB, 0.18500E+01_JPRB, 0.20535E+01_JPRB ,&
&  0.21137E+01_JPRB, 0.21430E+01_JPRB, 0.21108E+01_JPRB, 0.20095E+01_JPRB, 0.18912E+01_JPRB, 0.17774E+01_JPRB, 0.16430E+01_JPRB ,&
&  0.14897E+01_JPRB, 0.13358E+01_JPRB, 0.12517E+01_JPRB, 0.12362E+01_JPRB, 0.12844E+01_JPRB, 0.12166E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.14511E+01_JPRB, 0.15025E+01_JPRB, 0.15501E+01_JPRB, 0.15725E+01_JPRB, 0.15906E+01_JPRB, 0.15961E+01_JPRB ,&
&  0.16254E+01_JPRB, 0.16093E+01_JPRB, 0.15211E+01_JPRB, 0.16162E+01_JPRB, 0.19510E+01_JPRB, 0.28671E+01_JPRB, 0.37980E+01_JPRB ,&
&  0.42361E+01_JPRB, 0.51187E+01_JPRB, 0.63578E+01_JPRB, 0.68499E+01_JPRB, 0.59889E+01_JPRB, 0.53848E+01_JPRB, 0.53634E+01_JPRB ,&
&  0.56810E+01_JPRB, 0.60613E+01_JPRB, 0.62028E+01_JPRB, 0.58831E+01_JPRB, 0.58661E+01_JPRB, 0.58292E+01_JPRB, 0.57160E+01_JPRB ,&
&  0.54820E+01_JPRB, 0.51609E+01_JPRB, 0.49086E+01_JPRB, 0.46910E+01_JPRB, 0.45363E+01_JPRB, 0.42010E+01_JPRB, 0.41046E+01_JPRB ,&
&  0.41911E+01_JPRB, 0.35504E+01_JPRB, 0.32147E+01_JPRB, 0.33692E+01_JPRB, 0.27947E+01_JPRB, 0.27414E+01_JPRB, 0.29548E+01_JPRB ,&
&  0.25361E+01_JPRB, 0.21497E+01_JPRB, 0.19844E+01_JPRB, 0.18727E+01_JPRB, 0.19390E+01_JPRB, 0.23662E+01_JPRB, 0.26789E+01_JPRB ,&
&  0.25214E+01_JPRB, 0.23626E+01_JPRB, 0.26724E+01_JPRB, 0.28438E+01_JPRB, 0.29281E+01_JPRB, 0.27872E+01_JPRB, 0.25587E+01_JPRB ,&
&  0.23857E+01_JPRB, 0.22703E+01_JPRB, 0.21753E+01_JPRB, 0.20668E+01_JPRB, 0.19506E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.19964E+01_JPRB, 0.19417E+01_JPRB, 0.18481E+01_JPRB, 0.17197E+01_JPRB, 0.16089E+01_JPRB, 0.15265E+01_JPRB, 0.14576E+01_JPRB ,&
&  0.14298E+01_JPRB, 0.13951E+01_JPRB, 0.13675E+01_JPRB, 0.13252E+01_JPRB, 0.13078E+01_JPRB, 0.13090E+01_JPRB, 0.13177E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.13807E+01_JPRB, 0.13337E+01_JPRB, 0.13669E+01_JPRB, 0.16187E+01_JPRB, 0.23603E+01_JPRB, 0.29052E+01_JPRB ,&
&  0.31197E+01_JPRB, 0.31237E+01_JPRB, 0.30613E+01_JPRB, 0.30491E+01_JPRB, 0.29355E+01_JPRB, 0.26894E+01_JPRB, 0.25380E+01_JPRB ,&
&  0.23795E+01_JPRB, 0.21598E+01_JPRB, 0.18385E+01_JPRB, 0.18030E+01_JPRB, 0.17361E+01_JPRB, 0.15990E+01_JPRB, 0.17255E+01_JPRB ,&
&  0.17714E+01_JPRB, 0.17759E+01_JPRB, 0.17506E+01_JPRB, 0.17255E+01_JPRB, 0.17028E+01_JPRB, 0.16147E+01_JPRB, 0.15381E+01_JPRB ,&
&  0.15231E+01_JPRB, 0.15715E+01_JPRB, 0.16616E+01_JPRB, 0.19861E+01_JPRB, 0.28040E+01_JPRB, 0.40572E+01_JPRB, 0.50568E+01_JPRB ,&
&  0.58827E+01_JPRB, 0.63324E+01_JPRB, 0.60342E+01_JPRB, 0.65751E+01_JPRB, 0.73848E+01_JPRB, 0.70521E+01_JPRB, 0.66488E+01_JPRB ,&
&  0.66241E+01_JPRB, 0.65541E+01_JPRB, 0.63638E+01_JPRB, 0.62655E+01_JPRB, 0.64737E+01_JPRB, 0.64454E+01_JPRB, 0.61794E+01_JPRB ,&
&  0.53709E+01_JPRB, 0.50740E+01_JPRB, 0.49737E+01_JPRB, 0.48386E+01_JPRB, 0.43293E+01_JPRB, 0.41180E+01_JPRB, 0.41845E+01_JPRB ,&
&  0.40956E+01_JPRB, 0.40482E+01_JPRB, 0.32577E+01_JPRB, 0.30986E+01_JPRB, 0.24254E+01_JPRB, 0.23535E+01_JPRB, 0.26622E+01_JPRB ,&
&  0.24498E+01_JPRB, 0.22795E+01_JPRB, 0.20870E+01_JPRB, 0.21299E+01_JPRB, 0.28667E+01_JPRB, 0.36777E+01_JPRB, 0.45784E+01_JPRB ,&
&  0.43892E+01_JPRB, 0.36454E+01_JPRB, 0.38673E+01_JPRB, 0.39798E+01_JPRB, 0.41137E+01_JPRB, 0.38605E+01_JPRB, 0.34199E+01_JPRB ,&
&  0.30618E+01_JPRB, 0.27804E+01_JPRB, 0.25039E+01_JPRB, 0.22507E+01_JPRB, 0.20878E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.20598E+01_JPRB, 0.19997E+01_JPRB, 0.18846E+01_JPRB, 0.17412E+01_JPRB, 0.16318E+01_JPRB, 0.15471E+01_JPRB, 0.14776E+01_JPRB ,&
&  0.14072E+01_JPRB, 0.13484E+01_JPRB, 0.13128E+01_JPRB, 0.12922E+01_JPRB, 0.12879E+01_JPRB, 0.12928E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.13925E+01_JPRB, 0.13669E+01_JPRB, 0.14110E+01_JPRB, 0.18099E+01_JPRB, 0.27121E+01_JPRB, 0.35814E+01_JPRB ,&
&  0.39558E+01_JPRB, 0.38797E+01_JPRB, 0.38372E+01_JPRB, 0.38548E+01_JPRB, 0.39419E+01_JPRB, 0.39156E+01_JPRB, 0.38214E+01_JPRB ,&
&  0.36191E+01_JPRB, 0.28580E+01_JPRB, 0.25696E+01_JPRB, 0.23402E+01_JPRB, 0.22674E+01_JPRB, 0.22103E+01_JPRB, 0.21234E+01_JPRB ,&
&  0.19949E+01_JPRB, 0.18481E+01_JPRB, 0.17385E+01_JPRB, 0.16422E+01_JPRB, 0.15890E+01_JPRB, 0.15164E+01_JPRB, 0.15491E+01_JPRB ,&
&  0.16830E+01_JPRB, 0.18377E+01_JPRB, 0.23666E+01_JPRB, 0.32542E+01_JPRB, 0.47534E+01_JPRB, 0.62475E+01_JPRB, 0.64423E+01_JPRB ,&
&  0.55216E+01_JPRB, 0.56479E+01_JPRB, 0.80670E+01_JPRB, 0.74873E+01_JPRB, 0.75835E+01_JPRB, 0.82997E+01_JPRB, 0.77888E+01_JPRB ,&
&  0.69960E+01_JPRB, 0.63947E+01_JPRB, 0.60789E+01_JPRB, 0.61231E+01_JPRB, 0.74420E+01_JPRB, 0.74136E+01_JPRB, 0.67083E+01_JPRB ,&
&  0.57364E+01_JPRB, 0.55096E+01_JPRB, 0.51360E+01_JPRB, 0.47246E+01_JPRB, 0.44454E+01_JPRB, 0.46570E+01_JPRB, 0.44030E+01_JPRB ,&
&  0.41461E+01_JPRB, 0.45746E+01_JPRB, 0.36944E+01_JPRB, 0.28781E+01_JPRB, 0.24226E+01_JPRB, 0.25041E+01_JPRB, 0.26876E+01_JPRB ,&
&  0.27023E+01_JPRB, 0.26716E+01_JPRB, 0.27678E+01_JPRB, 0.36363E+01_JPRB, 0.54422E+01_JPRB, 0.67959E+01_JPRB, 0.64309E+01_JPRB ,&
&  0.59052E+01_JPRB, 0.61339E+01_JPRB, 0.57445E+01_JPRB, 0.56567E+01_JPRB, 0.55091E+01_JPRB, 0.47810E+01_JPRB, 0.38630E+01_JPRB ,&
&  0.30887E+01_JPRB, 0.26000E+01_JPRB, 0.23397E+01_JPRB, 0.21924E+01_JPRB, 0.21087E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.21618E+01_JPRB, 0.20363E+01_JPRB, 0.19002E+01_JPRB, 0.17639E+01_JPRB, 0.16383E+01_JPRB, 0.15344E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.13622E+01_JPRB, 0.13015E+01_JPRB, 0.12839E+01_JPRB, 0.12910E+01_JPRB, 0.13065E+01_JPRB, 0.12956E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.14226E+01_JPRB, 0.14075E+01_JPRB, 0.15665E+01_JPRB, 0.19768E+01_JPRB, 0.26287E+01_JPRB, 0.35617E+01_JPRB ,&
&  0.41542E+01_JPRB, 0.39535E+01_JPRB, 0.37900E+01_JPRB, 0.42468E+01_JPRB, 0.49236E+01_JPRB, 0.53172E+01_JPRB, 0.53273E+01_JPRB ,&
&  0.46364E+01_JPRB, 0.40366E+01_JPRB, 0.35332E+01_JPRB, 0.30754E+01_JPRB, 0.27738E+01_JPRB, 0.24910E+01_JPRB, 0.21530E+01_JPRB ,&
&  0.18606E+01_JPRB, 0.16814E+01_JPRB, 0.15968E+01_JPRB, 0.15651E+01_JPRB, 0.15951E+01_JPRB, 0.16541E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.20554E+01_JPRB, 0.25403E+01_JPRB, 0.34181E+01_JPRB, 0.45876E+01_JPRB, 0.51541E+01_JPRB, 0.68408E+01_JPRB, 0.66069E+01_JPRB ,&
&  0.67617E+01_JPRB, 0.70648E+01_JPRB, 0.70587E+01_JPRB, 0.76004E+01_JPRB, 0.66250E+01_JPRB, 0.72028E+01_JPRB, 0.74773E+01_JPRB ,&
&  0.70984E+01_JPRB, 0.67788E+01_JPRB, 0.69495E+01_JPRB, 0.66383E+01_JPRB, 0.71066E+01_JPRB, 0.86893E+01_JPRB, 0.77733E+01_JPRB ,&
&  0.65529E+01_JPRB, 0.59596E+01_JPRB, 0.52700E+01_JPRB, 0.49697E+01_JPRB, 0.42215E+01_JPRB, 0.42257E+01_JPRB, 0.37563E+01_JPRB ,&
&  0.36020E+01_JPRB, 0.30104E+01_JPRB, 0.32992E+01_JPRB, 0.28274E+01_JPRB, 0.27067E+01_JPRB, 0.27022E+01_JPRB, 0.28057E+01_JPRB ,&
&  0.31191E+01_JPRB, 0.35326E+01_JPRB, 0.46336E+01_JPRB, 0.66612E+01_JPRB, 0.91441E+01_JPRB, 0.79300E+01_JPRB, 0.79288E+01_JPRB ,&
&  0.76417E+01_JPRB, 0.74207E+01_JPRB, 0.62466E+01_JPRB, 0.57945E+01_JPRB, 0.51222E+01_JPRB, 0.45567E+01_JPRB, 0.39645E+01_JPRB ,&
&  0.33581E+01_JPRB, 0.28542E+01_JPRB, 0.25749E+01_JPRB, 0.24083E+01_JPRB, 0.22808E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.20958E+01_JPRB, 0.19385E+01_JPRB, 0.18117E+01_JPRB, 0.16907E+01_JPRB, 0.15770E+01_JPRB, 0.14605E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12199E+01_JPRB, 0.12325E+01_JPRB, 0.12587E+01_JPRB, 0.12860E+01_JPRB, 0.13243E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.15051E+01_JPRB, 0.15031E+01_JPRB, 0.16114E+01_JPRB, 0.20070E+01_JPRB, 0.25518E+01_JPRB, 0.31226E+01_JPRB ,&
&  0.40963E+01_JPRB, 0.34961E+01_JPRB, 0.32848E+01_JPRB, 0.40155E+01_JPRB, 0.57509E+01_JPRB, 0.69432E+01_JPRB, 0.68494E+01_JPRB ,&
&  0.66639E+01_JPRB, 0.55730E+01_JPRB, 0.43146E+01_JPRB, 0.33757E+01_JPRB, 0.26279E+01_JPRB, 0.20938E+01_JPRB, 0.17080E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.14375E+01_JPRB, 0.14355E+01_JPRB, 0.14645E+01_JPRB, 0.15544E+01_JPRB, 0.16378E+01_JPRB, 0.20041E+01_JPRB ,&
&  0.24283E+01_JPRB, 0.30011E+01_JPRB, 0.37219E+01_JPRB, 0.42796E+01_JPRB, 0.47082E+01_JPRB, 0.64826E+01_JPRB, 0.75591E+01_JPRB ,&
&  0.66077E+01_JPRB, 0.65491E+01_JPRB, 0.71365E+01_JPRB, 0.74469E+01_JPRB, 0.77331E+01_JPRB, 0.71078E+01_JPRB, 0.68955E+01_JPRB ,&
&  0.53898E+01_JPRB, 0.46974E+01_JPRB, 0.45942E+01_JPRB, 0.52136E+01_JPRB, 0.74515E+01_JPRB, 0.97183E+01_JPRB, 0.90132E+01_JPRB ,&
&  0.76763E+01_JPRB, 0.67597E+01_JPRB, 0.59094E+01_JPRB, 0.46767E+01_JPRB, 0.43190E+01_JPRB, 0.37327E+01_JPRB, 0.44294E+01_JPRB ,&
&  0.43664E+01_JPRB, 0.40923E+01_JPRB, 0.34584E+01_JPRB, 0.27254E+01_JPRB, 0.23311E+01_JPRB, 0.28990E+01_JPRB, 0.32632E+01_JPRB ,&
&  0.39277E+01_JPRB, 0.48004E+01_JPRB, 0.84945E+01_JPRB, 0.98402E+01_JPRB, 0.10123E+02_JPRB, 0.82798E+01_JPRB, 0.90509E+01_JPRB ,&
&  0.80768E+01_JPRB, 0.77639E+01_JPRB, 0.65256E+01_JPRB, 0.49845E+01_JPRB, 0.39898E+01_JPRB, 0.34366E+01_JPRB, 0.31541E+01_JPRB ,&
&  0.30431E+01_JPRB, 0.29264E+01_JPRB, 0.27250E+01_JPRB, 0.25034E+01_JPRB, 0.23099E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.17831E+01_JPRB, 0.16325E+01_JPRB, 0.15327E+01_JPRB, 0.14513E+01_JPRB, 0.13612E+01_JPRB, 0.12751E+01_JPRB, 0.11986E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.11534E+01_JPRB, 0.11853E+01_JPRB, 0.12446E+01_JPRB, 0.13054E+01_JPRB, 0.13680E+01_JPRB, 0.14145E+01_JPRB ,&
&  0.14881E+01_JPRB, 0.16387E+01_JPRB, 0.16770E+01_JPRB, 0.18167E+01_JPRB, 0.21863E+01_JPRB, 0.25130E+01_JPRB, 0.30699E+01_JPRB ,&
&  0.38765E+01_JPRB, 0.31412E+01_JPRB, 0.30777E+01_JPRB, 0.38913E+01_JPRB, 0.64309E+01_JPRB, 0.72332E+01_JPRB, 0.67446E+01_JPRB ,&
&  0.65984E+01_JPRB, 0.51032E+01_JPRB, 0.34784E+01_JPRB, 0.26036E+01_JPRB, 0.20504E+01_JPRB, 0.16540E+01_JPRB, 0.14738E+01_JPRB ,&
&  0.14294E+01_JPRB, 0.14378E+01_JPRB, 0.14769E+01_JPRB, 0.15343E+01_JPRB, 0.16171E+01_JPRB, 0.18474E+01_JPRB, 0.23007E+01_JPRB ,&
&  0.27683E+01_JPRB, 0.32403E+01_JPRB, 0.35812E+01_JPRB, 0.38118E+01_JPRB, 0.43270E+01_JPRB, 0.61048E+01_JPRB, 0.85031E+01_JPRB ,&
&  0.74490E+01_JPRB, 0.74649E+01_JPRB, 0.76076E+01_JPRB, 0.80324E+01_JPRB, 0.73049E+01_JPRB, 0.67622E+01_JPRB, 0.63418E+01_JPRB ,&
&  0.49110E+01_JPRB, 0.46095E+01_JPRB, 0.54276E+01_JPRB, 0.66729E+01_JPRB, 0.74037E+01_JPRB, 0.86837E+01_JPRB, 0.10353E+02_JPRB ,&
&  0.78712E+01_JPRB, 0.68758E+01_JPRB, 0.62447E+01_JPRB, 0.56938E+01_JPRB, 0.34206E+01_JPRB, 0.25516E+01_JPRB, 0.36603E+01_JPRB ,&
&  0.34929E+01_JPRB, 0.20829E+01_JPRB, 0.16879E+01_JPRB, 0.21933E+01_JPRB, 0.16435E+01_JPRB, 0.21249E+01_JPRB, 0.37654E+01_JPRB ,&
&  0.49434E+01_JPRB, 0.64078E+01_JPRB, 0.12050E+02_JPRB, 0.10543E+02_JPRB, 0.10365E+02_JPRB, 0.10952E+02_JPRB, 0.10513E+02_JPRB ,&
&  0.95331E+01_JPRB, 0.60018E+01_JPRB, 0.42718E+01_JPRB, 0.32562E+01_JPRB, 0.27209E+01_JPRB, 0.26441E+01_JPRB, 0.25839E+01_JPRB ,&
&  0.25329E+01_JPRB, 0.24064E+01_JPRB, 0.22814E+01_JPRB, 0.20990E+01_JPRB, 0.19540E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.13821E+01_JPRB, 0.12825E+01_JPRB, 0.12533E+01_JPRB, 0.12256E+01_JPRB, 0.11628E+01_JPRB, 0.11208E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.10946E+01_JPRB, 0.11385E+01_JPRB, 0.12017E+01_JPRB, 0.12879E+01_JPRB, 0.13910E+01_JPRB, 0.14816E+01_JPRB, 0.15186E+01_JPRB ,&
&  0.15706E+01_JPRB, 0.17329E+01_JPRB, 0.20043E+01_JPRB, 0.23354E+01_JPRB, 0.28164E+01_JPRB, 0.26786E+01_JPRB, 0.36916E+01_JPRB ,&
&  0.37115E+01_JPRB, 0.29145E+01_JPRB, 0.31250E+01_JPRB, 0.40833E+01_JPRB, 0.54186E+01_JPRB, 0.56133E+01_JPRB, 0.51238E+01_JPRB ,&
&  0.43043E+01_JPRB, 0.32004E+01_JPRB, 0.24533E+01_JPRB, 0.19882E+01_JPRB, 0.16956E+01_JPRB, 0.15401E+01_JPRB, 0.15515E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.15858E+01_JPRB, 0.16557E+01_JPRB, 0.16454E+01_JPRB, 0.18150E+01_JPRB, 0.20921E+01_JPRB, 0.25210E+01_JPRB ,&
&  0.29083E+01_JPRB, 0.32232E+01_JPRB, 0.34328E+01_JPRB, 0.36801E+01_JPRB, 0.50857E+01_JPRB, 0.68220E+01_JPRB, 0.71678E+01_JPRB ,&
&  0.68293E+01_JPRB, 0.67673E+01_JPRB, 0.80907E+01_JPRB, 0.78647E+01_JPRB, 0.71024E+01_JPRB, 0.71558E+01_JPRB, 0.64454E+01_JPRB ,&
&  0.62034E+01_JPRB, 0.64247E+01_JPRB, 0.65287E+01_JPRB, 0.75568E+01_JPRB, 0.88254E+01_JPRB, 0.87622E+01_JPRB, 0.90215E+01_JPRB ,&
&  0.83348E+01_JPRB, 0.67554E+01_JPRB, 0.49869E+01_JPRB, 0.38061E+01_JPRB, 0.37511E+01_JPRB, 0.32383E+01_JPRB, 0.22331E+01_JPRB ,&
&  0.18691E+01_JPRB, 0.16251E+01_JPRB, 0.12594E+01_JPRB, 0.14106E+01_JPRB, 0.13388E+01_JPRB, 0.19239E+01_JPRB, 0.37425E+01_JPRB ,&
&  0.70627E+01_JPRB, 0.94602E+01_JPRB, 0.12152E+02_JPRB, 0.10523E+02_JPRB, 0.10078E+02_JPRB, 0.11393E+02_JPRB, 0.11153E+02_JPRB ,&
&  0.52720E+01_JPRB, 0.33489E+01_JPRB, 0.26742E+01_JPRB, 0.24225E+01_JPRB, 0.23216E+01_JPRB, 0.22451E+01_JPRB, 0.21629E+01_JPRB ,&
&  0.20274E+01_JPRB, 0.19057E+01_JPRB, 0.18462E+01_JPRB, 0.17496E+01_JPRB, 0.15627E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.11862E+01_JPRB, 0.11640E+01_JPRB, 0.11680E+01_JPRB, 0.11404E+01_JPRB, 0.10782E+01_JPRB, 0.10871E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.12195E+01_JPRB, 0.13120E+01_JPRB, 0.14435E+01_JPRB, 0.16201E+01_JPRB, 0.17051E+01_JPRB, 0.17160E+01_JPRB ,&
&  0.17676E+01_JPRB, 0.20027E+01_JPRB, 0.24537E+01_JPRB, 0.30994E+01_JPRB, 0.36826E+01_JPRB, 0.31711E+01_JPRB, 0.43378E+01_JPRB ,&
&  0.34876E+01_JPRB, 0.28266E+01_JPRB, 0.30702E+01_JPRB, 0.37081E+01_JPRB, 0.43029E+01_JPRB, 0.40218E+01_JPRB, 0.32463E+01_JPRB ,&
&  0.25467E+01_JPRB, 0.22096E+01_JPRB, 0.19641E+01_JPRB, 0.17652E+01_JPRB, 0.16041E+01_JPRB, 0.15824E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.17253E+01_JPRB, 0.17478E+01_JPRB, 0.18947E+01_JPRB, 0.21849E+01_JPRB, 0.24855E+01_JPRB, 0.28125E+01_JPRB ,&
&  0.30318E+01_JPRB, 0.32619E+01_JPRB, 0.35759E+01_JPRB, 0.45041E+01_JPRB, 0.52897E+01_JPRB, 0.56022E+01_JPRB, 0.65445E+01_JPRB ,&
&  0.74563E+01_JPRB, 0.79942E+01_JPRB, 0.74232E+01_JPRB, 0.82906E+01_JPRB, 0.75890E+01_JPRB, 0.69628E+01_JPRB, 0.66485E+01_JPRB ,&
&  0.66633E+01_JPRB, 0.77051E+01_JPRB, 0.76993E+01_JPRB, 0.81850E+01_JPRB, 0.10750E+02_JPRB, 0.11080E+02_JPRB, 0.90538E+01_JPRB ,&
&  0.84638E+01_JPRB, 0.64897E+01_JPRB, 0.50052E+01_JPRB, 0.40209E+01_JPRB, 0.51911E+01_JPRB, 0.59590E+01_JPRB, 0.30837E+01_JPRB ,&
&  0.21036E+01_JPRB, 0.16633E+01_JPRB, 0.12830E+01_JPRB, 0.12755E+01_JPRB, 0.13817E+01_JPRB, 0.19853E+01_JPRB, 0.53098E+01_JPRB ,&
&  0.66308E+01_JPRB, 0.11071E+02_JPRB, 0.12789E+02_JPRB, 0.11693E+02_JPRB, 0.10513E+02_JPRB, 0.91140E+01_JPRB, 0.53135E+01_JPRB ,&
&  0.30133E+01_JPRB, 0.20601E+01_JPRB, 0.19907E+01_JPRB, 0.21688E+01_JPRB, 0.22127E+01_JPRB, 0.20782E+01_JPRB, 0.18890E+01_JPRB ,&
&  0.17627E+01_JPRB, 0.17129E+01_JPRB, 0.16174E+01_JPRB, 0.15202E+01_JPRB, 0.13481E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.11816E+01_JPRB, 0.12109E+01_JPRB, 0.12444E+01_JPRB, 0.11720E+01_JPRB, 0.11567E+01_JPRB, 0.11807E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.13750E+01_JPRB, 0.15042E+01_JPRB, 0.17284E+01_JPRB, 0.19506E+01_JPRB, 0.19854E+01_JPRB, 0.19844E+01_JPRB ,&
&  0.20986E+01_JPRB, 0.24899E+01_JPRB, 0.31594E+01_JPRB, 0.40320E+01_JPRB, 0.42557E+01_JPRB, 0.40089E+01_JPRB, 0.45949E+01_JPRB ,&
&  0.34934E+01_JPRB, 0.28936E+01_JPRB, 0.28665E+01_JPRB, 0.32651E+01_JPRB, 0.33343E+01_JPRB, 0.31229E+01_JPRB, 0.24041E+01_JPRB ,&
&  0.20312E+01_JPRB, 0.19971E+01_JPRB, 0.17668E+01_JPRB, 0.18102E+01_JPRB, 0.16942E+01_JPRB, 0.17259E+01_JPRB, 0.17377E+01_JPRB ,&
&  0.18095E+01_JPRB, 0.18940E+01_JPRB, 0.19984E+01_JPRB, 0.22257E+01_JPRB, 0.25513E+01_JPRB, 0.28488E+01_JPRB, 0.31205E+01_JPRB ,&
&  0.32578E+01_JPRB, 0.36069E+01_JPRB, 0.42251E+01_JPRB, 0.51701E+01_JPRB, 0.47471E+01_JPRB, 0.61392E+01_JPRB, 0.65854E+01_JPRB ,&
&  0.72135E+01_JPRB, 0.74477E+01_JPRB, 0.69009E+01_JPRB, 0.73530E+01_JPRB, 0.75606E+01_JPRB, 0.71676E+01_JPRB, 0.69052E+01_JPRB ,&
&  0.79075E+01_JPRB, 0.83531E+01_JPRB, 0.89817E+01_JPRB, 0.95242E+01_JPRB, 0.11709E+02_JPRB, 0.15071E+02_JPRB, 0.92899E+01_JPRB ,&
&  0.69937E+01_JPRB, 0.66146E+01_JPRB, 0.52664E+01_JPRB, 0.46045E+01_JPRB, 0.59029E+01_JPRB, 0.72485E+01_JPRB, 0.70841E+01_JPRB ,&
&  0.29971E+01_JPRB, 0.17797E+01_JPRB, 0.16595E+01_JPRB, 0.16565E+01_JPRB, 0.14291E+01_JPRB, 0.21826E+01_JPRB, 0.90145E+01_JPRB ,&
&  0.71131E+01_JPRB, 0.11296E+02_JPRB, 0.11963E+02_JPRB, 0.10844E+02_JPRB, 0.10197E+02_JPRB, 0.66931E+01_JPRB, 0.36183E+01_JPRB ,&
&  0.17733E+01_JPRB, 0.17246E+01_JPRB, 0.19390E+01_JPRB, 0.18778E+01_JPRB, 0.18418E+01_JPRB, 0.17239E+01_JPRB, 0.15210E+01_JPRB ,&
&  0.14913E+01_JPRB, 0.14666E+01_JPRB, 0.13628E+01_JPRB, 0.12673E+01_JPRB, 0.12058E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.12457E+01_JPRB, 0.12902E+01_JPRB, 0.12937E+01_JPRB, 0.12920E+01_JPRB, 0.13121E+01_JPRB, 0.13345E+01_JPRB, 0.13389E+01_JPRB ,&
&  0.14421E+01_JPRB, 0.16243E+01_JPRB, 0.18761E+01_JPRB, 0.22056E+01_JPRB, 0.24032E+01_JPRB, 0.23979E+01_JPRB, 0.24387E+01_JPRB ,&
&  0.27202E+01_JPRB, 0.32738E+01_JPRB, 0.39904E+01_JPRB, 0.45419E+01_JPRB, 0.44404E+01_JPRB, 0.40467E+01_JPRB, 0.43195E+01_JPRB ,&
&  0.37497E+01_JPRB, 0.30110E+01_JPRB, 0.24876E+01_JPRB, 0.27247E+01_JPRB, 0.29657E+01_JPRB, 0.30523E+01_JPRB, 0.24461E+01_JPRB ,&
&  0.22459E+01_JPRB, 0.21857E+01_JPRB, 0.20560E+01_JPRB, 0.20573E+01_JPRB, 0.20382E+01_JPRB, 0.20786E+01_JPRB, 0.21028E+01_JPRB ,&
&  0.20849E+01_JPRB, 0.21926E+01_JPRB, 0.23885E+01_JPRB, 0.26523E+01_JPRB, 0.29629E+01_JPRB, 0.32341E+01_JPRB, 0.35098E+01_JPRB ,&
&  0.38237E+01_JPRB, 0.43175E+01_JPRB, 0.49228E+01_JPRB, 0.50237E+01_JPRB, 0.52583E+01_JPRB, 0.57920E+01_JPRB, 0.63698E+01_JPRB ,&
&  0.68697E+01_JPRB, 0.72062E+01_JPRB, 0.71582E+01_JPRB, 0.70586E+01_JPRB, 0.72300E+01_JPRB, 0.74337E+01_JPRB, 0.78140E+01_JPRB ,&
&  0.90321E+01_JPRB, 0.97508E+01_JPRB, 0.96692E+01_JPRB, 0.10043E+02_JPRB, 0.10843E+02_JPRB, 0.11485E+02_JPRB, 0.97043E+01_JPRB ,&
&  0.75493E+01_JPRB, 0.58566E+01_JPRB, 0.51692E+01_JPRB, 0.57674E+01_JPRB, 0.72792E+01_JPRB, 0.77662E+01_JPRB, 0.10472E+02_JPRB ,&
&  0.10550E+02_JPRB, 0.42093E+01_JPRB, 0.28781E+01_JPRB, 0.38825E+01_JPRB, 0.20993E+01_JPRB, 0.21401E+01_JPRB, 0.44175E+01_JPRB ,&
&  0.73138E+01_JPRB, 0.98756E+01_JPRB, 0.98383E+01_JPRB, 0.86078E+01_JPRB, 0.81757E+01_JPRB, 0.45297E+01_JPRB, 0.21843E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.16618E+01_JPRB, 0.16644E+01_JPRB, 0.14896E+01_JPRB, 0.14276E+01_JPRB, 0.14781E+01_JPRB, 0.13142E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.13205E+01_JPRB, 0.13279E+01_JPRB, 0.12878E+01_JPRB, 0.12330E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.13910E+01_JPRB, 0.14308E+01_JPRB, 0.15046E+01_JPRB, 0.15564E+01_JPRB, 0.15726E+01_JPRB, 0.16198E+01_JPRB, 0.16966E+01_JPRB ,&
&  0.18882E+01_JPRB, 0.21633E+01_JPRB, 0.24804E+01_JPRB, 0.27886E+01_JPRB, 0.28884E+01_JPRB, 0.29143E+01_JPRB, 0.31081E+01_JPRB ,&
&  0.34966E+01_JPRB, 0.40395E+01_JPRB, 0.45844E+01_JPRB, 0.50072E+01_JPRB, 0.50646E+01_JPRB, 0.51958E+01_JPRB, 0.42496E+01_JPRB ,&
&  0.35464E+01_JPRB, 0.33638E+01_JPRB, 0.26341E+01_JPRB, 0.28477E+01_JPRB, 0.30370E+01_JPRB, 0.31035E+01_JPRB, 0.27775E+01_JPRB ,&
&  0.25953E+01_JPRB, 0.25242E+01_JPRB, 0.24190E+01_JPRB, 0.23847E+01_JPRB, 0.24319E+01_JPRB, 0.24878E+01_JPRB, 0.25453E+01_JPRB ,&
&  0.26433E+01_JPRB, 0.27387E+01_JPRB, 0.29139E+01_JPRB, 0.31577E+01_JPRB, 0.34159E+01_JPRB, 0.37016E+01_JPRB, 0.40689E+01_JPRB ,&
&  0.44259E+01_JPRB, 0.47209E+01_JPRB, 0.50130E+01_JPRB, 0.48866E+01_JPRB, 0.51526E+01_JPRB, 0.55998E+01_JPRB, 0.60548E+01_JPRB ,&
&  0.60902E+01_JPRB, 0.60366E+01_JPRB, 0.66601E+01_JPRB, 0.73979E+01_JPRB, 0.77962E+01_JPRB, 0.79003E+01_JPRB, 0.82976E+01_JPRB ,&
&  0.95168E+01_JPRB, 0.10183E+02_JPRB, 0.10224E+02_JPRB, 0.90560E+01_JPRB, 0.84711E+01_JPRB, 0.82192E+01_JPRB, 0.82596E+01_JPRB ,&
&  0.73714E+01_JPRB, 0.63842E+01_JPRB, 0.61738E+01_JPRB, 0.69379E+01_JPRB, 0.78383E+01_JPRB, 0.81462E+01_JPRB, 0.95482E+01_JPRB ,&
&  0.10963E+02_JPRB, 0.98412E+01_JPRB, 0.77368E+01_JPRB, 0.48943E+01_JPRB, 0.33235E+01_JPRB, 0.31302E+01_JPRB, 0.42471E+01_JPRB ,&
&  0.66816E+01_JPRB, 0.71625E+01_JPRB, 0.68226E+01_JPRB, 0.79608E+01_JPRB, 0.54957E+01_JPRB, 0.28964E+01_JPRB, 0.16494E+01_JPRB ,&
&  0.13789E+01_JPRB, 0.14128E+01_JPRB, 0.14081E+01_JPRB, 0.13261E+01_JPRB, 0.13449E+01_JPRB, 0.13816E+01_JPRB, 0.13199E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.14124E+01_JPRB, 0.13663E+01_JPRB, 0.13268E+01_JPRB, 0.13538E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.16656E+01_JPRB, 0.17524E+01_JPRB, 0.18309E+01_JPRB, 0.18804E+01_JPRB, 0.19750E+01_JPRB, 0.20394E+01_JPRB, 0.21935E+01_JPRB ,&
&  0.24597E+01_JPRB, 0.27480E+01_JPRB, 0.30193E+01_JPRB, 0.31826E+01_JPRB, 0.32527E+01_JPRB, 0.33815E+01_JPRB, 0.36554E+01_JPRB ,&
&  0.39709E+01_JPRB, 0.43520E+01_JPRB, 0.46949E+01_JPRB, 0.50649E+01_JPRB, 0.53225E+01_JPRB, 0.59321E+01_JPRB, 0.52873E+01_JPRB ,&
&  0.33685E+01_JPRB, 0.42939E+01_JPRB, 0.29361E+01_JPRB, 0.32213E+01_JPRB, 0.33450E+01_JPRB, 0.35004E+01_JPRB, 0.31002E+01_JPRB ,&
&  0.28348E+01_JPRB, 0.28027E+01_JPRB, 0.27591E+01_JPRB, 0.26785E+01_JPRB, 0.26478E+01_JPRB, 0.27811E+01_JPRB, 0.29401E+01_JPRB ,&
&  0.30787E+01_JPRB, 0.31788E+01_JPRB, 0.33227E+01_JPRB, 0.35239E+01_JPRB, 0.37520E+01_JPRB, 0.40225E+01_JPRB, 0.43302E+01_JPRB ,&
&  0.45421E+01_JPRB, 0.47505E+01_JPRB, 0.47692E+01_JPRB, 0.47629E+01_JPRB, 0.48802E+01_JPRB, 0.51741E+01_JPRB, 0.55350E+01_JPRB ,&
&  0.57217E+01_JPRB, 0.58732E+01_JPRB, 0.65316E+01_JPRB, 0.69645E+01_JPRB, 0.71376E+01_JPRB, 0.76217E+01_JPRB, 0.77370E+01_JPRB ,&
&  0.88132E+01_JPRB, 0.86794E+01_JPRB, 0.87719E+01_JPRB, 0.74511E+01_JPRB, 0.63952E+01_JPRB, 0.60773E+01_JPRB, 0.60626E+01_JPRB ,&
&  0.57218E+01_JPRB, 0.60232E+01_JPRB, 0.63231E+01_JPRB, 0.68661E+01_JPRB, 0.76386E+01_JPRB, 0.83632E+01_JPRB, 0.82614E+01_JPRB ,&
&  0.81238E+01_JPRB, 0.58618E+01_JPRB, 0.43961E+01_JPRB, 0.38382E+01_JPRB, 0.30632E+01_JPRB, 0.29938E+01_JPRB, 0.42228E+01_JPRB ,&
&  0.57359E+01_JPRB, 0.58434E+01_JPRB, 0.60026E+01_JPRB, 0.54766E+01_JPRB, 0.31268E+01_JPRB, 0.18463E+01_JPRB, 0.13344E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.12796E+01_JPRB, 0.12802E+01_JPRB, 0.12725E+01_JPRB, 0.13288E+01_JPRB, 0.12735E+01_JPRB, 0.13411E+01_JPRB ,&
&  0.13708E+01_JPRB, 0.14276E+01_JPRB, 0.14693E+01_JPRB, 0.14848E+01_JPRB, 0.15559E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.19725E+01_JPRB, 0.21648E+01_JPRB, 0.23653E+01_JPRB, 0.25199E+01_JPRB, 0.25389E+01_JPRB, 0.25738E+01_JPRB, 0.26792E+01_JPRB ,&
&  0.28167E+01_JPRB, 0.29922E+01_JPRB, 0.31762E+01_JPRB, 0.33360E+01_JPRB, 0.34709E+01_JPRB, 0.36904E+01_JPRB, 0.40088E+01_JPRB ,&
&  0.43684E+01_JPRB, 0.47468E+01_JPRB, 0.51225E+01_JPRB, 0.55791E+01_JPRB, 0.61526E+01_JPRB, 0.70516E+01_JPRB, 0.69131E+01_JPRB ,&
&  0.48333E+01_JPRB, 0.36213E+01_JPRB, 0.34894E+01_JPRB, 0.33183E+01_JPRB, 0.32091E+01_JPRB, 0.34729E+01_JPRB, 0.34598E+01_JPRB ,&
&  0.31561E+01_JPRB, 0.29949E+01_JPRB, 0.29452E+01_JPRB, 0.28567E+01_JPRB, 0.27529E+01_JPRB, 0.29277E+01_JPRB, 0.31005E+01_JPRB ,&
&  0.32592E+01_JPRB, 0.33632E+01_JPRB, 0.34786E+01_JPRB, 0.36354E+01_JPRB, 0.38163E+01_JPRB, 0.40058E+01_JPRB, 0.42298E+01_JPRB ,&
&  0.43337E+01_JPRB, 0.44576E+01_JPRB, 0.45046E+01_JPRB, 0.45031E+01_JPRB, 0.45952E+01_JPRB, 0.46883E+01_JPRB, 0.48173E+01_JPRB ,&
&  0.50978E+01_JPRB, 0.55408E+01_JPRB, 0.59822E+01_JPRB, 0.62347E+01_JPRB, 0.62271E+01_JPRB, 0.67322E+01_JPRB, 0.68773E+01_JPRB ,&
&  0.71205E+01_JPRB, 0.67110E+01_JPRB, 0.68422E+01_JPRB, 0.59693E+01_JPRB, 0.54358E+01_JPRB, 0.50129E+01_JPRB, 0.49163E+01_JPRB ,&
&  0.48039E+01_JPRB, 0.48476E+01_JPRB, 0.47667E+01_JPRB, 0.51224E+01_JPRB, 0.61125E+01_JPRB, 0.67255E+01_JPRB, 0.69693E+01_JPRB ,&
&  0.58818E+01_JPRB, 0.44628E+01_JPRB, 0.32116E+01_JPRB, 0.33092E+01_JPRB, 0.28226E+01_JPRB, 0.31950E+01_JPRB, 0.38964E+01_JPRB ,&
&  0.44782E+01_JPRB, 0.37640E+01_JPRB, 0.38359E+01_JPRB, 0.34604E+01_JPRB, 0.20439E+01_JPRB, 0.13574E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.10114E+01_JPRB, 0.10640E+01_JPRB, 0.10917E+01_JPRB, 0.10909E+01_JPRB, 0.11650E+01_JPRB, 0.12265E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.13690E+01_JPRB, 0.14466E+01_JPRB, 0.15381E+01_JPRB, 0.16316E+01_JPRB, 0.17820E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.19824E+01_JPRB, 0.21397E+01_JPRB, 0.23104E+01_JPRB, 0.23609E+01_JPRB, 0.24161E+01_JPRB, 0.24840E+01_JPRB, 0.25822E+01_JPRB ,&
&  0.27666E+01_JPRB, 0.29390E+01_JPRB, 0.31206E+01_JPRB, 0.32780E+01_JPRB, 0.34155E+01_JPRB, 0.36343E+01_JPRB, 0.39014E+01_JPRB ,&
&  0.41338E+01_JPRB, 0.43545E+01_JPRB, 0.45796E+01_JPRB, 0.48827E+01_JPRB, 0.52080E+01_JPRB, 0.56607E+01_JPRB, 0.58744E+01_JPRB ,&
&  0.45825E+01_JPRB, 0.27395E+01_JPRB, 0.29327E+01_JPRB, 0.29811E+01_JPRB, 0.30929E+01_JPRB, 0.34086E+01_JPRB, 0.34748E+01_JPRB ,&
&  0.30803E+01_JPRB, 0.29590E+01_JPRB, 0.29828E+01_JPRB, 0.28590E+01_JPRB, 0.26177E+01_JPRB, 0.28021E+01_JPRB, 0.29872E+01_JPRB ,&
&  0.31623E+01_JPRB, 0.33115E+01_JPRB, 0.34342E+01_JPRB, 0.35673E+01_JPRB, 0.36809E+01_JPRB, 0.37854E+01_JPRB, 0.38850E+01_JPRB ,&
&  0.39754E+01_JPRB, 0.39897E+01_JPRB, 0.40416E+01_JPRB, 0.41703E+01_JPRB, 0.41949E+01_JPRB, 0.41703E+01_JPRB, 0.41429E+01_JPRB ,&
&  0.42985E+01_JPRB, 0.45591E+01_JPRB, 0.49826E+01_JPRB, 0.53409E+01_JPRB, 0.56057E+01_JPRB, 0.55322E+01_JPRB, 0.55666E+01_JPRB ,&
&  0.53069E+01_JPRB, 0.48853E+01_JPRB, 0.51215E+01_JPRB, 0.54029E+01_JPRB, 0.42122E+01_JPRB, 0.40433E+01_JPRB, 0.37632E+01_JPRB ,&
&  0.38082E+01_JPRB, 0.34430E+01_JPRB, 0.32034E+01_JPRB, 0.34471E+01_JPRB, 0.45393E+01_JPRB, 0.48738E+01_JPRB, 0.50825E+01_JPRB ,&
&  0.43325E+01_JPRB, 0.28783E+01_JPRB, 0.26189E+01_JPRB, 0.31056E+01_JPRB, 0.31886E+01_JPRB, 0.44631E+01_JPRB, 0.38564E+01_JPRB ,&
&  0.28423E+01_JPRB, 0.23298E+01_JPRB, 0.26217E+01_JPRB, 0.26423E+01_JPRB, 0.16598E+01_JPRB, 0.11630E+01_JPRB, 0.99120E+00_JPRB ,&
&  0.92463E+00_JPRB, 0.90890E+00_JPRB, 0.95706E+00_JPRB, 0.94453E+00_JPRB, 0.10120E+01_JPRB, 0.10717E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.13177E+01_JPRB, 0.14244E+01_JPRB, 0.15183E+01_JPRB, 0.16460E+01_JPRB, 0.17941E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.13982E+01_JPRB, 0.14952E+01_JPRB, 0.15364E+01_JPRB, 0.17273E+01_JPRB, 0.19487E+01_JPRB, 0.21272E+01_JPRB, 0.22900E+01_JPRB ,&
&  0.24985E+01_JPRB, 0.27559E+01_JPRB, 0.29594E+01_JPRB, 0.31056E+01_JPRB, 0.32573E+01_JPRB, 0.34543E+01_JPRB, 0.35125E+01_JPRB ,&
&  0.35077E+01_JPRB, 0.35009E+01_JPRB, 0.35143E+01_JPRB, 0.35108E+01_JPRB, 0.35318E+01_JPRB, 0.35445E+01_JPRB, 0.38163E+01_JPRB ,&
&  0.37379E+01_JPRB, 0.33939E+01_JPRB, 0.30407E+01_JPRB, 0.22559E+01_JPRB, 0.18559E+01_JPRB, 0.23243E+01_JPRB, 0.23078E+01_JPRB ,&
&  0.25260E+01_JPRB, 0.25086E+01_JPRB, 0.23743E+01_JPRB, 0.24244E+01_JPRB, 0.24706E+01_JPRB, 0.25871E+01_JPRB, 0.27574E+01_JPRB ,&
&  0.28838E+01_JPRB, 0.30205E+01_JPRB, 0.31470E+01_JPRB, 0.32547E+01_JPRB, 0.33381E+01_JPRB, 0.34060E+01_JPRB, 0.34199E+01_JPRB ,&
&  0.34321E+01_JPRB, 0.34477E+01_JPRB, 0.34986E+01_JPRB, 0.36919E+01_JPRB, 0.37327E+01_JPRB, 0.37653E+01_JPRB, 0.36637E+01_JPRB ,&
&  0.36845E+01_JPRB, 0.37838E+01_JPRB, 0.40546E+01_JPRB, 0.42598E+01_JPRB, 0.42780E+01_JPRB, 0.40487E+01_JPRB, 0.39384E+01_JPRB ,&
&  0.37789E+01_JPRB, 0.35749E+01_JPRB, 0.37053E+01_JPRB, 0.41430E+01_JPRB, 0.30891E+01_JPRB, 0.32545E+01_JPRB, 0.33102E+01_JPRB ,&
&  0.28681E+01_JPRB, 0.25437E+01_JPRB, 0.23432E+01_JPRB, 0.22330E+01_JPRB, 0.28731E+01_JPRB, 0.34924E+01_JPRB, 0.41307E+01_JPRB ,&
&  0.33326E+01_JPRB, 0.22464E+01_JPRB, 0.21920E+01_JPRB, 0.28266E+01_JPRB, 0.33233E+01_JPRB, 0.44031E+01_JPRB, 0.31812E+01_JPRB ,&
&  0.20076E+01_JPRB, 0.20486E+01_JPRB, 0.23873E+01_JPRB, 0.27763E+01_JPRB, 0.18844E+01_JPRB, 0.10905E+01_JPRB, 0.89664E+00_JPRB ,&
&  0.85374E+00_JPRB, 0.85158E+00_JPRB, 0.85957E+00_JPRB, 0.85529E+00_JPRB, 0.89878E+00_JPRB, 0.94442E+00_JPRB, 0.96124E+00_JPRB ,&
&  0.98580E+00_JPRB, 0.10049E+01_JPRB, 0.10589E+01_JPRB, 0.11500E+01_JPRB, 0.12877E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.90047E+00_JPRB, 0.91400E+00_JPRB, 0.90510E+00_JPRB, 0.94934E+00_JPRB, 0.10341E+01_JPRB, 0.12042E+01_JPRB, 0.13760E+01_JPRB ,&
&  0.15709E+01_JPRB, 0.18065E+01_JPRB, 0.21415E+01_JPRB, 0.24597E+01_JPRB, 0.25992E+01_JPRB, 0.25751E+01_JPRB, 0.23319E+01_JPRB ,&
&  0.20003E+01_JPRB, 0.16431E+01_JPRB, 0.14166E+01_JPRB, 0.13152E+01_JPRB, 0.13659E+01_JPRB, 0.13837E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.16261E+01_JPRB, 0.19013E+01_JPRB, 0.21261E+01_JPRB, 0.26689E+01_JPRB, 0.28100E+01_JPRB, 0.22201E+01_JPRB, 0.22604E+01_JPRB ,&
&  0.29454E+01_JPRB, 0.29093E+01_JPRB, 0.19315E+01_JPRB, 0.18229E+01_JPRB, 0.19629E+01_JPRB, 0.20757E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.23551E+01_JPRB, 0.24520E+01_JPRB, 0.25298E+01_JPRB, 0.25701E+01_JPRB, 0.25857E+01_JPRB, 0.25455E+01_JPRB, 0.24982E+01_JPRB ,&
&  0.24566E+01_JPRB, 0.24159E+01_JPRB, 0.25270E+01_JPRB, 0.26984E+01_JPRB, 0.28920E+01_JPRB, 0.30171E+01_JPRB, 0.30191E+01_JPRB ,&
&  0.30162E+01_JPRB, 0.30485E+01_JPRB, 0.33185E+01_JPRB, 0.33596E+01_JPRB, 0.31430E+01_JPRB, 0.29662E+01_JPRB, 0.27274E+01_JPRB ,&
&  0.24758E+01_JPRB, 0.21203E+01_JPRB, 0.17605E+01_JPRB, 0.27456E+01_JPRB, 0.31544E+01_JPRB, 0.27770E+01_JPRB, 0.23767E+01_JPRB ,&
&  0.21513E+01_JPRB, 0.19678E+01_JPRB, 0.17979E+01_JPRB, 0.17710E+01_JPRB, 0.20817E+01_JPRB, 0.31932E+01_JPRB, 0.33421E+01_JPRB ,&
&  0.27834E+01_JPRB, 0.17875E+01_JPRB, 0.15808E+01_JPRB, 0.18970E+01_JPRB, 0.22937E+01_JPRB, 0.29174E+01_JPRB, 0.20657E+01_JPRB ,&
&  0.15676E+01_JPRB, 0.18966E+01_JPRB, 0.21706E+01_JPRB, 0.26549E+01_JPRB, 0.27034E+01_JPRB, 0.88607E+00_JPRB, 0.81288E+00_JPRB ,&
&  0.78794E+00_JPRB, 0.77552E+00_JPRB, 0.79254E+00_JPRB, 0.78291E+00_JPRB, 0.79027E+00_JPRB, 0.77508E+00_JPRB, 0.75514E+00_JPRB ,&
&  0.75482E+00_JPRB, 0.77523E+00_JPRB, 0.80329E+00_JPRB, 0.82448E+00_JPRB, 0.86752E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.81225E+00_JPRB, 0.81383E+00_JPRB, 0.81934E+00_JPRB, 0.82964E+00_JPRB, 0.83895E+00_JPRB, 0.84864E+00_JPRB, 0.88632E+00_JPRB ,&
&  0.94946E+00_JPRB, 0.10429E+01_JPRB, 0.11751E+01_JPRB, 0.12729E+01_JPRB, 0.12541E+01_JPRB, 0.11471E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.10398E+01_JPRB, 0.10145E+01_JPRB, 0.10471E+01_JPRB, 0.11044E+01_JPRB, 0.11749E+01_JPRB, 0.11891E+01_JPRB, 0.12566E+01_JPRB ,&
&  0.12501E+01_JPRB, 0.13050E+01_JPRB, 0.14264E+01_JPRB, 0.15297E+01_JPRB, 0.22753E+01_JPRB, 0.22115E+01_JPRB, 0.19504E+01_JPRB ,&
&  0.24205E+01_JPRB, 0.24509E+01_JPRB, 0.20464E+01_JPRB, 0.15719E+01_JPRB, 0.14275E+01_JPRB, 0.13900E+01_JPRB, 0.14216E+01_JPRB ,&
&  0.14092E+01_JPRB, 0.15024E+01_JPRB, 0.15803E+01_JPRB, 0.16676E+01_JPRB, 0.17260E+01_JPRB, 0.17396E+01_JPRB, 0.17902E+01_JPRB ,&
&  0.18313E+01_JPRB, 0.18767E+01_JPRB, 0.19340E+01_JPRB, 0.17705E+01_JPRB, 0.19211E+01_JPRB, 0.20811E+01_JPRB, 0.21826E+01_JPRB ,&
&  0.22136E+01_JPRB, 0.22299E+01_JPRB, 0.23919E+01_JPRB, 0.23097E+01_JPRB, 0.21954E+01_JPRB, 0.20032E+01_JPRB, 0.18390E+01_JPRB ,&
&  0.17028E+01_JPRB, 0.13011E+01_JPRB, 0.95187E+00_JPRB, 0.14156E+01_JPRB, 0.17600E+01_JPRB, 0.15690E+01_JPRB, 0.16697E+01_JPRB ,&
&  0.15905E+01_JPRB, 0.15048E+01_JPRB, 0.14209E+01_JPRB, 0.14324E+01_JPRB, 0.14749E+01_JPRB, 0.20153E+01_JPRB, 0.24422E+01_JPRB ,&
&  0.20151E+01_JPRB, 0.14027E+01_JPRB, 0.12024E+01_JPRB, 0.12804E+01_JPRB, 0.15015E+01_JPRB, 0.18162E+01_JPRB, 0.19728E+01_JPRB ,&
&  0.18277E+01_JPRB, 0.15390E+01_JPRB, 0.19230E+01_JPRB, 0.24144E+01_JPRB, 0.13359E+01_JPRB, 0.70363E+00_JPRB, 0.65040E+00_JPRB ,&
&  0.64576E+00_JPRB, 0.65495E+00_JPRB, 0.67777E+00_JPRB, 0.69332E+00_JPRB, 0.70398E+00_JPRB, 0.71068E+00_JPRB, 0.72906E+00_JPRB ,&
&  0.74496E+00_JPRB, 0.76355E+00_JPRB, 0.77440E+00_JPRB, 0.78820E+00_JPRB, 0.80333E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.10313E+01_JPRB, 0.10540E+01_JPRB, 0.10869E+01_JPRB, 0.11210E+01_JPRB, 0.11466E+01_JPRB, 0.11819E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.12740E+01_JPRB, 0.13432E+01_JPRB, 0.13927E+01_JPRB, 0.14118E+01_JPRB, 0.14248E+01_JPRB, 0.14402E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.14682E+01_JPRB, 0.14869E+01_JPRB, 0.15687E+01_JPRB, 0.16033E+01_JPRB, 0.16450E+01_JPRB, 0.16959E+01_JPRB, 0.16993E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.16628E+01_JPRB, 0.16719E+01_JPRB, 0.16576E+01_JPRB, 0.17385E+01_JPRB, 0.17611E+01_JPRB, 0.17111E+01_JPRB ,&
&  0.18441E+01_JPRB, 0.18340E+01_JPRB, 0.19099E+01_JPRB, 0.14119E+01_JPRB, 0.13334E+01_JPRB, 0.13336E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.11986E+01_JPRB, 0.12503E+01_JPRB, 0.13157E+01_JPRB, 0.13803E+01_JPRB, 0.14731E+01_JPRB, 0.15142E+01_JPRB, 0.15391E+01_JPRB ,&
&  0.15725E+01_JPRB, 0.16193E+01_JPRB, 0.16494E+01_JPRB, 0.16078E+01_JPRB, 0.15944E+01_JPRB, 0.18151E+01_JPRB, 0.18987E+01_JPRB ,&
&  0.19167E+01_JPRB, 0.19362E+01_JPRB, 0.16350E+01_JPRB, 0.15266E+01_JPRB, 0.15834E+01_JPRB, 0.14227E+01_JPRB, 0.13612E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.12873E+01_JPRB, 0.79747E+00_JPRB, 0.10452E+01_JPRB, 0.12431E+01_JPRB, 0.13160E+01_JPRB, 0.13452E+01_JPRB ,&
&  0.12761E+01_JPRB, 0.11415E+01_JPRB, 0.10934E+01_JPRB, 0.11666E+01_JPRB, 0.12374E+01_JPRB, 0.12496E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.11384E+01_JPRB, 0.10657E+01_JPRB, 0.12591E+01_JPRB, 0.18770E+01_JPRB, 0.30305E+01_JPRB, 0.26754E+01_JPRB ,&
&  0.18571E+01_JPRB, 0.16780E+01_JPRB, 0.27192E+01_JPRB, 0.36493E+01_JPRB, 0.13849E+01_JPRB, 0.84661E+00_JPRB, 0.62345E+00_JPRB ,&
&  0.65612E+00_JPRB, 0.67233E+00_JPRB, 0.68390E+00_JPRB, 0.71209E+00_JPRB, 0.75615E+00_JPRB, 0.78085E+00_JPRB, 0.79276E+00_JPRB ,&
&  0.82509E+00_JPRB, 0.87124E+00_JPRB, 0.90687E+00_JPRB, 0.95061E+00_JPRB, 0.99041E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.12426E+01_JPRB, 0.12789E+01_JPRB, 0.13080E+01_JPRB, 0.13400E+01_JPRB, 0.13790E+01_JPRB, 0.14173E+01_JPRB, 0.14454E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.15077E+01_JPRB, 0.15206E+01_JPRB, 0.15438E+01_JPRB, 0.15513E+01_JPRB, 0.16229E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.17234E+01_JPRB, 0.17679E+01_JPRB, 0.17866E+01_JPRB, 0.17936E+01_JPRB, 0.18136E+01_JPRB, 0.17989E+01_JPRB, 0.17831E+01_JPRB ,&
&  0.18125E+01_JPRB, 0.18754E+01_JPRB, 0.19324E+01_JPRB, 0.19841E+01_JPRB, 0.19967E+01_JPRB, 0.20753E+01_JPRB, 0.21886E+01_JPRB ,&
&  0.18125E+01_JPRB, 0.20338E+01_JPRB, 0.18587E+01_JPRB, 0.15469E+01_JPRB, 0.15256E+01_JPRB, 0.15333E+01_JPRB, 0.14629E+01_JPRB ,&
&  0.14068E+01_JPRB, 0.14270E+01_JPRB, 0.14448E+01_JPRB, 0.14593E+01_JPRB, 0.14999E+01_JPRB, 0.15478E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.16276E+01_JPRB, 0.16699E+01_JPRB, 0.17105E+01_JPRB, 0.17783E+01_JPRB, 0.17270E+01_JPRB, 0.19697E+01_JPRB, 0.20314E+01_JPRB ,&
&  0.20289E+01_JPRB, 0.19266E+01_JPRB, 0.14950E+01_JPRB, 0.14232E+01_JPRB, 0.14653E+01_JPRB, 0.14000E+01_JPRB, 0.13279E+01_JPRB ,&
&  0.14357E+01_JPRB, 0.13906E+01_JPRB, 0.10576E+01_JPRB, 0.11643E+01_JPRB, 0.12117E+01_JPRB, 0.12297E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.93393E+00_JPRB, 0.89675E+00_JPRB, 0.92730E+00_JPRB, 0.94841E+00_JPRB, 0.94665E+00_JPRB, 0.99760E+00_JPRB ,&
&  0.10276E+01_JPRB, 0.10304E+01_JPRB, 0.10746E+01_JPRB, 0.11895E+01_JPRB, 0.19964E+01_JPRB, 0.25854E+01_JPRB, 0.35373E+01_JPRB ,&
&  0.15666E+01_JPRB, 0.15766E+01_JPRB, 0.13916E+01_JPRB, 0.19148E+01_JPRB, 0.11235E+01_JPRB, 0.69459E+00_JPRB, 0.65836E+00_JPRB ,&
&  0.73014E+00_JPRB, 0.76567E+00_JPRB, 0.77783E+00_JPRB, 0.80355E+00_JPRB, 0.86034E+00_JPRB, 0.89887E+00_JPRB, 0.93723E+00_JPRB ,&
&  0.96117E+00_JPRB, 0.10214E+01_JPRB, 0.10990E+01_JPRB, 0.11591E+01_JPRB, 0.11937E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.13444E+01_JPRB, 0.13917E+01_JPRB, 0.14346E+01_JPRB, 0.14780E+01_JPRB, 0.15204E+01_JPRB, 0.15627E+01_JPRB, 0.15987E+01_JPRB ,&
&  0.16282E+01_JPRB, 0.16488E+01_JPRB, 0.16601E+01_JPRB, 0.16766E+01_JPRB, 0.16866E+01_JPRB, 0.17472E+01_JPRB, 0.17675E+01_JPRB ,&
&  0.18363E+01_JPRB, 0.18676E+01_JPRB, 0.18636E+01_JPRB, 0.19073E+01_JPRB, 0.18863E+01_JPRB, 0.18508E+01_JPRB, 0.18446E+01_JPRB ,&
&  0.18617E+01_JPRB, 0.18662E+01_JPRB, 0.18708E+01_JPRB, 0.18880E+01_JPRB, 0.19000E+01_JPRB, 0.18697E+01_JPRB, 0.20396E+01_JPRB ,&
&  0.16474E+01_JPRB, 0.22445E+01_JPRB, 0.21724E+01_JPRB, 0.18866E+01_JPRB, 0.17653E+01_JPRB, 0.16185E+01_JPRB, 0.15889E+01_JPRB ,&
&  0.15840E+01_JPRB, 0.15396E+01_JPRB, 0.15251E+01_JPRB, 0.15372E+01_JPRB, 0.15723E+01_JPRB, 0.16264E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.17287E+01_JPRB, 0.17779E+01_JPRB, 0.18201E+01_JPRB, 0.18620E+01_JPRB, 0.19512E+01_JPRB, 0.20012E+01_JPRB, 0.20090E+01_JPRB ,&
&  0.19959E+01_JPRB, 0.19920E+01_JPRB, 0.15826E+01_JPRB, 0.15421E+01_JPRB, 0.16456E+01_JPRB, 0.15807E+01_JPRB, 0.14218E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.13708E+01_JPRB, 0.13082E+01_JPRB, 0.12302E+01_JPRB, 0.11790E+01_JPRB, 0.11193E+01_JPRB, 0.10324E+01_JPRB ,&
&  0.87304E+00_JPRB, 0.75584E+00_JPRB, 0.72981E+00_JPRB, 0.77381E+00_JPRB, 0.79770E+00_JPRB, 0.81217E+00_JPRB, 0.80983E+00_JPRB ,&
&  0.84387E+00_JPRB, 0.90620E+00_JPRB, 0.10226E+01_JPRB, 0.12803E+01_JPRB, 0.16877E+01_JPRB, 0.32568E+01_JPRB, 0.33467E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.13943E+01_JPRB, 0.83848E+00_JPRB, 0.94837E+00_JPRB, 0.95160E+00_JPRB, 0.78236E+00_JPRB, 0.78963E+00_JPRB ,&
&  0.90553E+00_JPRB, 0.97496E+00_JPRB, 0.10056E+01_JPRB, 0.95694E+00_JPRB, 0.98899E+00_JPRB, 0.98964E+00_JPRB, 0.10095E+01_JPRB ,&
&  0.10494E+01_JPRB, 0.11062E+01_JPRB, 0.11735E+01_JPRB, 0.12369E+01_JPRB, 0.12923E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.13851E+01_JPRB, 0.14356E+01_JPRB, 0.14887E+01_JPRB, 0.15474E+01_JPRB, 0.16077E+01_JPRB, 0.16562E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.17467E+01_JPRB, 0.17790E+01_JPRB, 0.17911E+01_JPRB, 0.18001E+01_JPRB, 0.18087E+01_JPRB, 0.18656E+01_JPRB, 0.19116E+01_JPRB ,&
&  0.19149E+01_JPRB, 0.19474E+01_JPRB, 0.19685E+01_JPRB, 0.19986E+01_JPRB, 0.19905E+01_JPRB, 0.19469E+01_JPRB, 0.19667E+01_JPRB ,&
&  0.19659E+01_JPRB, 0.19779E+01_JPRB, 0.19914E+01_JPRB, 0.20439E+01_JPRB, 0.21156E+01_JPRB, 0.23144E+01_JPRB, 0.20604E+01_JPRB ,&
&  0.22751E+01_JPRB, 0.24053E+01_JPRB, 0.22966E+01_JPRB, 0.20726E+01_JPRB, 0.18542E+01_JPRB, 0.17978E+01_JPRB, 0.17666E+01_JPRB ,&
&  0.17607E+01_JPRB, 0.16860E+01_JPRB, 0.16462E+01_JPRB, 0.16170E+01_JPRB, 0.16146E+01_JPRB, 0.16587E+01_JPRB, 0.16920E+01_JPRB ,&
&  0.17462E+01_JPRB, 0.18343E+01_JPRB, 0.19235E+01_JPRB, 0.20093E+01_JPRB, 0.21214E+01_JPRB, 0.22313E+01_JPRB, 0.22929E+01_JPRB ,&
&  0.23217E+01_JPRB, 0.23462E+01_JPRB, 0.21321E+01_JPRB, 0.21335E+01_JPRB, 0.22219E+01_JPRB, 0.20216E+01_JPRB, 0.17156E+01_JPRB ,&
&  0.15828E+01_JPRB, 0.13276E+01_JPRB, 0.11795E+01_JPRB, 0.12944E+01_JPRB, 0.11821E+01_JPRB, 0.10583E+01_JPRB, 0.91527E+00_JPRB ,&
&  0.75562E+00_JPRB, 0.67584E+00_JPRB, 0.67815E+00_JPRB, 0.72650E+00_JPRB, 0.72925E+00_JPRB, 0.75189E+00_JPRB, 0.80440E+00_JPRB ,&
&  0.88507E+00_JPRB, 0.97916E+00_JPRB, 0.11171E+01_JPRB, 0.13557E+01_JPRB, 0.16756E+01_JPRB, 0.17203E+01_JPRB, 0.18745E+01_JPRB ,&
&  0.13263E+01_JPRB, 0.10909E+01_JPRB, 0.10904E+01_JPRB, 0.10179E+01_JPRB, 0.10359E+01_JPRB, 0.98268E+00_JPRB, 0.10802E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.13772E+01_JPRB, 0.13938E+01_JPRB, 0.11916E+01_JPRB, 0.11012E+01_JPRB, 0.10686E+01_JPRB, 0.10802E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.11495E+01_JPRB, 0.12092E+01_JPRB, 0.12761E+01_JPRB, 0.13345E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.13769E+01_JPRB, 0.14300E+01_JPRB, 0.14861E+01_JPRB, 0.15423E+01_JPRB, 0.15907E+01_JPRB, 0.16321E+01_JPRB, 0.16669E+01_JPRB ,&
&  0.17110E+01_JPRB, 0.17548E+01_JPRB, 0.17892E+01_JPRB, 0.18113E+01_JPRB, 0.18349E+01_JPRB, 0.18782E+01_JPRB, 0.19293E+01_JPRB ,&
&  0.19896E+01_JPRB, 0.20393E+01_JPRB, 0.21174E+01_JPRB, 0.21367E+01_JPRB, 0.21562E+01_JPRB, 0.21910E+01_JPRB, 0.22686E+01_JPRB ,&
&  0.23122E+01_JPRB, 0.23190E+01_JPRB, 0.23380E+01_JPRB, 0.24153E+01_JPRB, 0.24921E+01_JPRB, 0.26453E+01_JPRB, 0.24059E+01_JPRB ,&
&  0.25302E+01_JPRB, 0.26182E+01_JPRB, 0.24795E+01_JPRB, 0.23231E+01_JPRB, 0.20932E+01_JPRB, 0.20090E+01_JPRB, 0.19629E+01_JPRB ,&
&  0.19139E+01_JPRB, 0.18583E+01_JPRB, 0.17789E+01_JPRB, 0.17236E+01_JPRB, 0.16885E+01_JPRB, 0.16784E+01_JPRB, 0.17192E+01_JPRB ,&
&  0.17625E+01_JPRB, 0.18295E+01_JPRB, 0.19430E+01_JPRB, 0.20633E+01_JPRB, 0.22308E+01_JPRB, 0.23935E+01_JPRB, 0.25076E+01_JPRB ,&
&  0.26291E+01_JPRB, 0.27460E+01_JPRB, 0.25806E+01_JPRB, 0.26255E+01_JPRB, 0.27643E+01_JPRB, 0.24697E+01_JPRB, 0.19491E+01_JPRB ,&
&  0.14314E+01_JPRB, 0.12753E+01_JPRB, 0.12815E+01_JPRB, 0.13368E+01_JPRB, 0.11711E+01_JPRB, 0.10226E+01_JPRB, 0.91886E+00_JPRB ,&
&  0.79059E+00_JPRB, 0.69673E+00_JPRB, 0.71256E+00_JPRB, 0.71211E+00_JPRB, 0.71111E+00_JPRB, 0.75944E+00_JPRB, 0.82913E+00_JPRB ,&
&  0.92187E+00_JPRB, 0.10670E+01_JPRB, 0.12299E+01_JPRB, 0.14151E+01_JPRB, 0.15479E+01_JPRB, 0.15048E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.13552E+01_JPRB, 0.13040E+01_JPRB, 0.12663E+01_JPRB, 0.95643E+00_JPRB, 0.12014E+01_JPRB, 0.12758E+01_JPRB, 0.14002E+01_JPRB ,&
&  0.13611E+01_JPRB, 0.17862E+01_JPRB, 0.21418E+01_JPRB, 0.16967E+01_JPRB, 0.13978E+01_JPRB, 0.11118E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.11414E+01_JPRB, 0.11946E+01_JPRB, 0.12587E+01_JPRB, 0.13194E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.13202E+01_JPRB, 0.13780E+01_JPRB, 0.14390E+01_JPRB, 0.14903E+01_JPRB, 0.15297E+01_JPRB, 0.15368E+01_JPRB, 0.15423E+01_JPRB ,&
&  0.15620E+01_JPRB, 0.15937E+01_JPRB, 0.16348E+01_JPRB, 0.16798E+01_JPRB, 0.17177E+01_JPRB, 0.17579E+01_JPRB, 0.18110E+01_JPRB ,&
&  0.18745E+01_JPRB, 0.19498E+01_JPRB, 0.20495E+01_JPRB, 0.21457E+01_JPRB, 0.22613E+01_JPRB, 0.23736E+01_JPRB, 0.24975E+01_JPRB ,&
&  0.26098E+01_JPRB, 0.27261E+01_JPRB, 0.27798E+01_JPRB, 0.28604E+01_JPRB, 0.29086E+01_JPRB, 0.30279E+01_JPRB, 0.24316E+01_JPRB ,&
&  0.26538E+01_JPRB, 0.27135E+01_JPRB, 0.26651E+01_JPRB, 0.24861E+01_JPRB, 0.22468E+01_JPRB, 0.21239E+01_JPRB, 0.20308E+01_JPRB ,&
&  0.19747E+01_JPRB, 0.18823E+01_JPRB, 0.17283E+01_JPRB, 0.17381E+01_JPRB, 0.17445E+01_JPRB, 0.16616E+01_JPRB, 0.17142E+01_JPRB ,&
&  0.17655E+01_JPRB, 0.18392E+01_JPRB, 0.19335E+01_JPRB, 0.20842E+01_JPRB, 0.22798E+01_JPRB, 0.24599E+01_JPRB, 0.26409E+01_JPRB ,&
&  0.28300E+01_JPRB, 0.30283E+01_JPRB, 0.31780E+01_JPRB, 0.28914E+01_JPRB, 0.28813E+01_JPRB, 0.25818E+01_JPRB, 0.19841E+01_JPRB ,&
&  0.14143E+01_JPRB, 0.12589E+01_JPRB, 0.12959E+01_JPRB, 0.13544E+01_JPRB, 0.11885E+01_JPRB, 0.10088E+01_JPRB, 0.90412E+00_JPRB ,&
&  0.79841E+00_JPRB, 0.74497E+00_JPRB, 0.73951E+00_JPRB, 0.71901E+00_JPRB, 0.72065E+00_JPRB, 0.77062E+00_JPRB, 0.84070E+00_JPRB ,&
&  0.92928E+00_JPRB, 0.10551E+01_JPRB, 0.12147E+01_JPRB, 0.13918E+01_JPRB, 0.15672E+01_JPRB, 0.16278E+01_JPRB, 0.14483E+01_JPRB ,&
&  0.13872E+01_JPRB, 0.12546E+01_JPRB, 0.12775E+01_JPRB, 0.13677E+01_JPRB, 0.13713E+01_JPRB, 0.16033E+01_JPRB, 0.18421E+01_JPRB ,&
&  0.20669E+01_JPRB, 0.19014E+01_JPRB, 0.22013E+01_JPRB, 0.30184E+01_JPRB, 0.27753E+01_JPRB, 0.23115E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10372E+01_JPRB, 0.10957E+01_JPRB, 0.11714E+01_JPRB, 0.12565E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.10508E+01_JPRB, 0.11260E+01_JPRB, 0.12039E+01_JPRB, 0.13078E+01_JPRB, 0.14022E+01_JPRB, 0.14304E+01_JPRB, 0.14238E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.14355E+01_JPRB, 0.14672E+01_JPRB, 0.15114E+01_JPRB, 0.15536E+01_JPRB, 0.15913E+01_JPRB, 0.16366E+01_JPRB ,&
&  0.16964E+01_JPRB, 0.17654E+01_JPRB, 0.18481E+01_JPRB, 0.19387E+01_JPRB, 0.20342E+01_JPRB, 0.21685E+01_JPRB, 0.23309E+01_JPRB ,&
&  0.25081E+01_JPRB, 0.26823E+01_JPRB, 0.28173E+01_JPRB, 0.29491E+01_JPRB, 0.30620E+01_JPRB, 0.30491E+01_JPRB, 0.21763E+01_JPRB ,&
&  0.26720E+01_JPRB, 0.27219E+01_JPRB, 0.27734E+01_JPRB, 0.26079E+01_JPRB, 0.23287E+01_JPRB, 0.21657E+01_JPRB, 0.20859E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.18784E+01_JPRB, 0.17384E+01_JPRB, 0.16333E+01_JPRB, 0.17227E+01_JPRB, 0.16263E+01_JPRB, 0.16883E+01_JPRB ,&
&  0.17509E+01_JPRB, 0.18266E+01_JPRB, 0.19108E+01_JPRB, 0.20498E+01_JPRB, 0.22507E+01_JPRB, 0.24546E+01_JPRB, 0.26809E+01_JPRB ,&
&  0.29030E+01_JPRB, 0.30954E+01_JPRB, 0.32023E+01_JPRB, 0.28092E+01_JPRB, 0.26726E+01_JPRB, 0.24895E+01_JPRB, 0.17675E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.13692E+01_JPRB, 0.14418E+01_JPRB, 0.14072E+01_JPRB, 0.12433E+01_JPRB, 0.10858E+01_JPRB, 0.95623E+00_JPRB ,&
&  0.81422E+00_JPRB, 0.75152E+00_JPRB, 0.74548E+00_JPRB, 0.73851E+00_JPRB, 0.74804E+00_JPRB, 0.80118E+00_JPRB, 0.86826E+00_JPRB ,&
&  0.93962E+00_JPRB, 0.10303E+01_JPRB, 0.11142E+01_JPRB, 0.11963E+01_JPRB, 0.12902E+01_JPRB, 0.14457E+01_JPRB, 0.15429E+01_JPRB ,&
&  0.13526E+01_JPRB, 0.12122E+01_JPRB, 0.11901E+01_JPRB, 0.11976E+01_JPRB, 0.12207E+01_JPRB, 0.12879E+01_JPRB, 0.13605E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.15649E+01_JPRB, 0.13217E+01_JPRB, 0.11037E+01_JPRB, 0.13934E+01_JPRB, 0.15018E+01_JPRB, 0.85283E+00_JPRB ,&
&  0.83239E+00_JPRB, 0.85915E+00_JPRB, 0.88905E+00_JPRB, 0.94199E+00_JPRB, 0.99492E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.82833E+00_JPRB, 0.85655E+00_JPRB, 0.95598E+00_JPRB, 0.99580E+00_JPRB, 0.11014E+01_JPRB, 0.11587E+01_JPRB, 0.12230E+01_JPRB ,&
&  0.12522E+01_JPRB, 0.12719E+01_JPRB, 0.12898E+01_JPRB, 0.13144E+01_JPRB, 0.13456E+01_JPRB, 0.13863E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.14727E+01_JPRB, 0.15296E+01_JPRB, 0.15966E+01_JPRB, 0.16686E+01_JPRB, 0.17621E+01_JPRB, 0.18723E+01_JPRB, 0.20250E+01_JPRB ,&
&  0.22193E+01_JPRB, 0.24483E+01_JPRB, 0.26850E+01_JPRB, 0.29218E+01_JPRB, 0.31670E+01_JPRB, 0.33549E+01_JPRB, 0.34517E+01_JPRB ,&
&  0.22704E+01_JPRB, 0.28148E+01_JPRB, 0.27792E+01_JPRB, 0.26150E+01_JPRB, 0.24012E+01_JPRB, 0.22602E+01_JPRB, 0.21816E+01_JPRB ,&
&  0.21017E+01_JPRB, 0.19154E+01_JPRB, 0.18246E+01_JPRB, 0.16828E+01_JPRB, 0.16269E+01_JPRB, 0.15813E+01_JPRB, 0.16309E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.17626E+01_JPRB, 0.18714E+01_JPRB, 0.19789E+01_JPRB, 0.21605E+01_JPRB, 0.24051E+01_JPRB, 0.26644E+01_JPRB ,&
&  0.29379E+01_JPRB, 0.32127E+01_JPRB, 0.33795E+01_JPRB, 0.29949E+01_JPRB, 0.26396E+01_JPRB, 0.25065E+01_JPRB, 0.19056E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.14230E+01_JPRB, 0.15244E+01_JPRB, 0.15322E+01_JPRB, 0.13472E+01_JPRB, 0.12088E+01_JPRB, 0.10790E+01_JPRB ,&
&  0.93355E+00_JPRB, 0.81739E+00_JPRB, 0.78456E+00_JPRB, 0.78627E+00_JPRB, 0.80445E+00_JPRB, 0.84578E+00_JPRB, 0.88147E+00_JPRB ,&
&  0.92253E+00_JPRB, 0.96959E+00_JPRB, 0.10119E+01_JPRB, 0.10592E+01_JPRB, 0.10825E+01_JPRB, 0.11041E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.11417E+01_JPRB, 0.11325E+01_JPRB, 0.11383E+01_JPRB, 0.11151E+01_JPRB, 0.11164E+01_JPRB, 0.11377E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.11660E+01_JPRB, 0.11485E+01_JPRB, 0.11050E+01_JPRB, 0.10229E+01_JPRB, 0.94170E+00_JPRB, 0.78600E+00_JPRB ,&
&  0.76760E+00_JPRB, 0.75219E+00_JPRB, 0.77198E+00_JPRB, 0.78821E+00_JPRB, 0.81291E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.71049E+00_JPRB, 0.73660E+00_JPRB, 0.76359E+00_JPRB, 0.79467E+00_JPRB, 0.84380E+00_JPRB, 0.90326E+00_JPRB, 0.98129E+00_JPRB ,&
&  0.10618E+01_JPRB, 0.10943E+01_JPRB, 0.11054E+01_JPRB, 0.11137E+01_JPRB, 0.11313E+01_JPRB, 0.11672E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.12553E+01_JPRB, 0.12964E+01_JPRB, 0.13363E+01_JPRB, 0.13840E+01_JPRB, 0.14430E+01_JPRB, 0.15299E+01_JPRB, 0.16375E+01_JPRB ,&
&  0.17763E+01_JPRB, 0.19965E+01_JPRB, 0.22856E+01_JPRB, 0.26211E+01_JPRB, 0.29855E+01_JPRB, 0.32834E+01_JPRB, 0.34480E+01_JPRB ,&
&  0.18678E+01_JPRB, 0.24300E+01_JPRB, 0.29397E+01_JPRB, 0.27395E+01_JPRB, 0.25224E+01_JPRB, 0.23967E+01_JPRB, 0.23184E+01_JPRB ,&
&  0.21522E+01_JPRB, 0.19390E+01_JPRB, 0.18315E+01_JPRB, 0.18198E+01_JPRB, 0.16050E+01_JPRB, 0.15605E+01_JPRB, 0.15744E+01_JPRB ,&
&  0.16045E+01_JPRB, 0.16391E+01_JPRB, 0.17133E+01_JPRB, 0.18197E+01_JPRB, 0.19795E+01_JPRB, 0.22330E+01_JPRB, 0.25260E+01_JPRB ,&
&  0.28515E+01_JPRB, 0.32205E+01_JPRB, 0.35598E+01_JPRB, 0.32021E+01_JPRB, 0.31547E+01_JPRB, 0.32376E+01_JPRB, 0.29556E+01_JPRB ,&
&  0.20660E+01_JPRB, 0.16536E+01_JPRB, 0.17075E+01_JPRB, 0.17101E+01_JPRB, 0.14855E+01_JPRB, 0.13587E+01_JPRB, 0.12386E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.99781E+00_JPRB, 0.87601E+00_JPRB, 0.85024E+00_JPRB, 0.86446E+00_JPRB, 0.89641E+00_JPRB, 0.92724E+00_JPRB ,&
&  0.94469E+00_JPRB, 0.95242E+00_JPRB, 0.96990E+00_JPRB, 0.99347E+00_JPRB, 0.10341E+01_JPRB, 0.10528E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.10873E+01_JPRB, 0.11022E+01_JPRB, 0.11239E+01_JPRB, 0.11451E+01_JPRB, 0.11541E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.11314E+01_JPRB, 0.11481E+01_JPRB, 0.11355E+01_JPRB, 0.10864E+01_JPRB, 0.98137E+00_JPRB, 0.89515E+00_JPRB ,&
&  0.80726E+00_JPRB, 0.79415E+00_JPRB, 0.76560E+00_JPRB, 0.73992E+00_JPRB, 0.71670E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.69449E+00_JPRB, 0.69169E+00_JPRB, 0.70251E+00_JPRB, 0.71140E+00_JPRB, 0.72948E+00_JPRB, 0.76436E+00_JPRB, 0.80829E+00_JPRB ,&
&  0.84659E+00_JPRB, 0.90974E+00_JPRB, 0.95522E+00_JPRB, 0.95724E+00_JPRB, 0.95139E+00_JPRB, 0.96812E+00_JPRB, 0.10042E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10937E+01_JPRB, 0.11248E+01_JPRB, 0.11533E+01_JPRB, 0.11895E+01_JPRB, 0.12286E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.13562E+01_JPRB, 0.14862E+01_JPRB, 0.16938E+01_JPRB, 0.20214E+01_JPRB, 0.24732E+01_JPRB, 0.30033E+01_JPRB, 0.33732E+01_JPRB ,&
&  0.23918E+01_JPRB, 0.11331E+01_JPRB, 0.25762E+01_JPRB, 0.30584E+01_JPRB, 0.27467E+01_JPRB, 0.26573E+01_JPRB, 0.24795E+01_JPRB ,&
&  0.22880E+01_JPRB, 0.21867E+01_JPRB, 0.20136E+01_JPRB, 0.18836E+01_JPRB, 0.16356E+01_JPRB, 0.15513E+01_JPRB, 0.15102E+01_JPRB ,&
&  0.15098E+01_JPRB, 0.15203E+01_JPRB, 0.15612E+01_JPRB, 0.16312E+01_JPRB, 0.17457E+01_JPRB, 0.19422E+01_JPRB, 0.22608E+01_JPRB ,&
&  0.26149E+01_JPRB, 0.30385E+01_JPRB, 0.36085E+01_JPRB, 0.34663E+01_JPRB, 0.36058E+01_JPRB, 0.33241E+01_JPRB, 0.28339E+01_JPRB ,&
&  0.22986E+01_JPRB, 0.19207E+01_JPRB, 0.19269E+01_JPRB, 0.19455E+01_JPRB, 0.16769E+01_JPRB, 0.14386E+01_JPRB, 0.14301E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.11827E+01_JPRB, 0.10477E+01_JPRB, 0.95732E+00_JPRB, 0.93616E+00_JPRB, 0.96062E+00_JPRB, 0.98532E+00_JPRB ,&
&  0.98967E+00_JPRB, 0.98630E+00_JPRB, 0.99029E+00_JPRB, 0.10049E+01_JPRB, 0.10311E+01_JPRB, 0.10496E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.11036E+01_JPRB, 0.11279E+01_JPRB, 0.11512E+01_JPRB, 0.11672E+01_JPRB, 0.11474E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.11560E+01_JPRB, 0.11482E+01_JPRB, 0.11537E+01_JPRB, 0.11292E+01_JPRB, 0.10662E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.96412E+00_JPRB, 0.89068E+00_JPRB, 0.80151E+00_JPRB, 0.74535E+00_JPRB, 0.72402E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.80095E+00_JPRB, 0.76181E+00_JPRB, 0.72009E+00_JPRB, 0.71023E+00_JPRB, 0.71073E+00_JPRB, 0.70889E+00_JPRB, 0.73016E+00_JPRB ,&
&  0.75019E+00_JPRB, 0.78027E+00_JPRB, 0.79260E+00_JPRB, 0.82011E+00_JPRB, 0.83066E+00_JPRB, 0.83537E+00_JPRB, 0.85525E+00_JPRB ,&
&  0.87465E+00_JPRB, 0.89858E+00_JPRB, 0.92477E+00_JPRB, 0.95495E+00_JPRB, 0.98591E+00_JPRB, 0.10000E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.11161E+01_JPRB, 0.12050E+01_JPRB, 0.13898E+01_JPRB, 0.17206E+01_JPRB, 0.21989E+01_JPRB, 0.27715E+01_JPRB ,&
&  0.33441E+01_JPRB, 0.23579E+01_JPRB, 0.15994E+01_JPRB, 0.28644E+01_JPRB, 0.31472E+01_JPRB, 0.30982E+01_JPRB, 0.26245E+01_JPRB ,&
&  0.26426E+01_JPRB, 0.24975E+01_JPRB, 0.23139E+01_JPRB, 0.20847E+01_JPRB, 0.17644E+01_JPRB, 0.16011E+01_JPRB, 0.14813E+01_JPRB ,&
&  0.14148E+01_JPRB, 0.13823E+01_JPRB, 0.13988E+01_JPRB, 0.14740E+01_JPRB, 0.15605E+01_JPRB, 0.16799E+01_JPRB, 0.19330E+01_JPRB ,&
&  0.22726E+01_JPRB, 0.27047E+01_JPRB, 0.33847E+01_JPRB, 0.36741E+01_JPRB, 0.41709E+01_JPRB, 0.36164E+01_JPRB, 0.30154E+01_JPRB ,&
&  0.24314E+01_JPRB, 0.23533E+01_JPRB, 0.24286E+01_JPRB, 0.22054E+01_JPRB, 0.18289E+01_JPRB, 0.16486E+01_JPRB, 0.16430E+01_JPRB ,&
&  0.14474E+01_JPRB, 0.13418E+01_JPRB, 0.12526E+01_JPRB, 0.11339E+01_JPRB, 0.10635E+01_JPRB, 0.10475E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10311E+01_JPRB, 0.10203E+01_JPRB, 0.10251E+01_JPRB, 0.10384E+01_JPRB, 0.10485E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11034E+01_JPRB, 0.11313E+01_JPRB, 0.11493E+01_JPRB, 0.11412E+01_JPRB, 0.11204E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.11734E+01_JPRB, 0.11593E+01_JPRB, 0.11630E+01_JPRB, 0.11476E+01_JPRB, 0.11381E+01_JPRB, 0.11264E+01_JPRB ,&
&  0.11138E+01_JPRB, 0.10777E+01_JPRB, 0.98515E+00_JPRB, 0.87123E+00_JPRB, 0.83962E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.89202E+00_JPRB, 0.86085E+00_JPRB, 0.82287E+00_JPRB, 0.78511E+00_JPRB, 0.76545E+00_JPRB, 0.74564E+00_JPRB, 0.72059E+00_JPRB ,&
&  0.70716E+00_JPRB, 0.70163E+00_JPRB, 0.69100E+00_JPRB, 0.70334E+00_JPRB, 0.73196E+00_JPRB, 0.74653E+00_JPRB, 0.75935E+00_JPRB ,&
&  0.77467E+00_JPRB, 0.78952E+00_JPRB, 0.78723E+00_JPRB, 0.79163E+00_JPRB, 0.81085E+00_JPRB, 0.82594E+00_JPRB, 0.82987E+00_JPRB ,&
&  0.84248E+00_JPRB, 0.87111E+00_JPRB, 0.91206E+00_JPRB, 0.99035E+00_JPRB, 0.11519E+01_JPRB, 0.14175E+01_JPRB, 0.18023E+01_JPRB ,&
&  0.24117E+01_JPRB, 0.24777E+01_JPRB, 0.12999E+01_JPRB, 0.24221E+01_JPRB, 0.33648E+01_JPRB, 0.34130E+01_JPRB, 0.31282E+01_JPRB ,&
&  0.31019E+01_JPRB, 0.29916E+01_JPRB, 0.28062E+01_JPRB, 0.23443E+01_JPRB, 0.20260E+01_JPRB, 0.17832E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.14217E+01_JPRB, 0.13095E+01_JPRB, 0.12589E+01_JPRB, 0.12922E+01_JPRB, 0.13703E+01_JPRB, 0.14367E+01_JPRB, 0.15776E+01_JPRB ,&
&  0.18403E+01_JPRB, 0.21946E+01_JPRB, 0.27914E+01_JPRB, 0.33575E+01_JPRB, 0.43035E+01_JPRB, 0.43100E+01_JPRB, 0.35575E+01_JPRB ,&
&  0.28708E+01_JPRB, 0.28627E+01_JPRB, 0.28527E+01_JPRB, 0.25348E+01_JPRB, 0.20161E+01_JPRB, 0.18554E+01_JPRB, 0.18259E+01_JPRB ,&
&  0.16320E+01_JPRB, 0.14792E+01_JPRB, 0.14065E+01_JPRB, 0.13304E+01_JPRB, 0.12264E+01_JPRB, 0.11390E+01_JPRB, 0.10997E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.10781E+01_JPRB, 0.10687E+01_JPRB, 0.10588E+01_JPRB, 0.10556E+01_JPRB, 0.10558E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10964E+01_JPRB, 0.11217E+01_JPRB, 0.11301E+01_JPRB, 0.11351E+01_JPRB, 0.11386E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.11499E+01_JPRB, 0.11517E+01_JPRB, 0.11209E+01_JPRB, 0.10921E+01_JPRB, 0.11313E+01_JPRB, 0.11262E+01_JPRB, 0.11434E+01_JPRB ,&
&  0.11688E+01_JPRB, 0.11623E+01_JPRB, 0.10906E+01_JPRB, 0.95521E+00_JPRB, 0.92529E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.92003E+00_JPRB, 0.89514E+00_JPRB, 0.87000E+00_JPRB, 0.84960E+00_JPRB, 0.83389E+00_JPRB, 0.81772E+00_JPRB, 0.79131E+00_JPRB ,&
&  0.75746E+00_JPRB, 0.73580E+00_JPRB, 0.70230E+00_JPRB, 0.69107E+00_JPRB, 0.69604E+00_JPRB, 0.70081E+00_JPRB, 0.71228E+00_JPRB ,&
&  0.72107E+00_JPRB, 0.73021E+00_JPRB, 0.73545E+00_JPRB, 0.73627E+00_JPRB, 0.73837E+00_JPRB, 0.74206E+00_JPRB, 0.74213E+00_JPRB ,&
&  0.73782E+00_JPRB, 0.74533E+00_JPRB, 0.76410E+00_JPRB, 0.79516E+00_JPRB, 0.86307E+00_JPRB, 0.99209E+00_JPRB, 0.11894E+01_JPRB ,&
&  0.16704E+01_JPRB, 0.19898E+01_JPRB, 0.21871E+01_JPRB, 0.33647E+01_JPRB, 0.40470E+01_JPRB, 0.39002E+01_JPRB, 0.38868E+01_JPRB ,&
&  0.39777E+01_JPRB, 0.39121E+01_JPRB, 0.35479E+01_JPRB, 0.26713E+01_JPRB, 0.21284E+01_JPRB, 0.18510E+01_JPRB, 0.16407E+01_JPRB ,&
&  0.14556E+01_JPRB, 0.13143E+01_JPRB, 0.12053E+01_JPRB, 0.11629E+01_JPRB, 0.11863E+01_JPRB, 0.12542E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.14830E+01_JPRB, 0.17496E+01_JPRB, 0.21528E+01_JPRB, 0.26391E+01_JPRB, 0.35561E+01_JPRB, 0.42529E+01_JPRB, 0.40784E+01_JPRB ,&
&  0.38546E+01_JPRB, 0.35771E+01_JPRB, 0.32082E+01_JPRB, 0.28887E+01_JPRB, 0.22368E+01_JPRB, 0.21762E+01_JPRB, 0.19538E+01_JPRB ,&
&  0.17330E+01_JPRB, 0.15735E+01_JPRB, 0.14820E+01_JPRB, 0.14290E+01_JPRB, 0.13544E+01_JPRB, 0.12527E+01_JPRB, 0.11695E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.11102E+01_JPRB, 0.11059E+01_JPRB, 0.10953E+01_JPRB, 0.10877E+01_JPRB, 0.10858E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.10809E+01_JPRB, 0.10691E+01_JPRB, 0.10889E+01_JPRB, 0.11000E+01_JPRB, 0.10992E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.11184E+01_JPRB, 0.11141E+01_JPRB, 0.11062E+01_JPRB, 0.10967E+01_JPRB, 0.10867E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.10303E+01_JPRB, 0.97444E+00_JPRB, 0.95416E+00_JPRB, 0.93835E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.92165E+00_JPRB, 0.89956E+00_JPRB, 0.88368E+00_JPRB, 0.86587E+00_JPRB, 0.84921E+00_JPRB, 0.83947E+00_JPRB, 0.83099E+00_JPRB ,&
&  0.81554E+00_JPRB, 0.78004E+00_JPRB, 0.74279E+00_JPRB, 0.72272E+00_JPRB, 0.71468E+00_JPRB, 0.69958E+00_JPRB, 0.69883E+00_JPRB ,&
&  0.70055E+00_JPRB, 0.70756E+00_JPRB, 0.70553E+00_JPRB, 0.70362E+00_JPRB, 0.70244E+00_JPRB, 0.70350E+00_JPRB, 0.70226E+00_JPRB ,&
&  0.69420E+00_JPRB, 0.69062E+00_JPRB, 0.69706E+00_JPRB, 0.71014E+00_JPRB, 0.73843E+00_JPRB, 0.80099E+00_JPRB, 0.88706E+00_JPRB ,&
&  0.11750E+01_JPRB, 0.15817E+01_JPRB, 0.16793E+01_JPRB, 0.32960E+01_JPRB, 0.39203E+01_JPRB, 0.42104E+01_JPRB, 0.40283E+01_JPRB ,&
&  0.41260E+01_JPRB, 0.46129E+01_JPRB, 0.38499E+01_JPRB, 0.27739E+01_JPRB, 0.20577E+01_JPRB, 0.17254E+01_JPRB, 0.15345E+01_JPRB ,&
&  0.13792E+01_JPRB, 0.12457E+01_JPRB, 0.11532E+01_JPRB, 0.10999E+01_JPRB, 0.10939E+01_JPRB, 0.11098E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.12328E+01_JPRB, 0.14168E+01_JPRB, 0.17026E+01_JPRB, 0.19397E+01_JPRB, 0.26104E+01_JPRB, 0.34086E+01_JPRB, 0.38647E+01_JPRB ,&
&  0.40938E+01_JPRB, 0.40990E+01_JPRB, 0.34429E+01_JPRB, 0.30243E+01_JPRB, 0.25205E+01_JPRB, 0.22587E+01_JPRB, 0.19873E+01_JPRB ,&
&  0.17941E+01_JPRB, 0.16410E+01_JPRB, 0.15293E+01_JPRB, 0.14631E+01_JPRB, 0.13985E+01_JPRB, 0.13239E+01_JPRB, 0.12347E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.11275E+01_JPRB, 0.11155E+01_JPRB, 0.11064E+01_JPRB, 0.11052E+01_JPRB, 0.11018E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.10862E+01_JPRB, 0.10753E+01_JPRB, 0.10491E+01_JPRB, 0.10550E+01_JPRB, 0.10512E+01_JPRB, 0.10278E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.10698E+01_JPRB, 0.11029E+01_JPRB, 0.11008E+01_JPRB, 0.10738E+01_JPRB, 0.10885E+01_JPRB, 0.11052E+01_JPRB, 0.10364E+01_JPRB ,&
&  0.10155E+01_JPRB, 0.99447E+00_JPRB, 0.96589E+00_JPRB, 0.94859E+00_JPRB, 0.93720E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.91407E+00_JPRB, 0.88409E+00_JPRB, 0.86293E+00_JPRB, 0.84576E+00_JPRB, 0.83047E+00_JPRB, 0.81671E+00_JPRB, 0.81268E+00_JPRB ,&
&  0.80816E+00_JPRB, 0.79847E+00_JPRB, 0.77443E+00_JPRB, 0.74568E+00_JPRB, 0.73068E+00_JPRB, 0.72306E+00_JPRB, 0.70655E+00_JPRB ,&
&  0.69790E+00_JPRB, 0.69692E+00_JPRB, 0.69510E+00_JPRB, 0.68709E+00_JPRB, 0.67906E+00_JPRB, 0.67349E+00_JPRB, 0.67022E+00_JPRB ,&
&  0.65802E+00_JPRB, 0.65256E+00_JPRB, 0.65346E+00_JPRB, 0.65535E+00_JPRB, 0.66436E+00_JPRB, 0.71077E+00_JPRB, 0.80038E+00_JPRB ,&
&  0.97125E+00_JPRB, 0.17892E+01_JPRB, 0.16618E+01_JPRB, 0.26620E+01_JPRB, 0.30560E+01_JPRB, 0.35216E+01_JPRB, 0.38102E+01_JPRB ,&
&  0.41327E+01_JPRB, 0.44885E+01_JPRB, 0.36441E+01_JPRB, 0.26728E+01_JPRB, 0.18801E+01_JPRB, 0.14610E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.11152E+01_JPRB, 0.10567E+01_JPRB, 0.10342E+01_JPRB, 0.10329E+01_JPRB, 0.10360E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.12064E+01_JPRB, 0.14705E+01_JPRB, 0.18040E+01_JPRB, 0.19668E+01_JPRB, 0.25105E+01_JPRB, 0.33082E+01_JPRB ,&
&  0.39392E+01_JPRB, 0.42944E+01_JPRB, 0.36168E+01_JPRB, 0.30579E+01_JPRB, 0.26354E+01_JPRB, 0.23147E+01_JPRB, 0.20410E+01_JPRB ,&
&  0.18286E+01_JPRB, 0.16710E+01_JPRB, 0.15541E+01_JPRB, 0.14705E+01_JPRB, 0.14110E+01_JPRB, 0.13653E+01_JPRB, 0.13031E+01_JPRB ,&
&  0.12331E+01_JPRB, 0.11834E+01_JPRB, 0.11539E+01_JPRB, 0.11378E+01_JPRB, 0.11306E+01_JPRB, 0.11158E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10677E+01_JPRB, 0.10593E+01_JPRB, 0.10368E+01_JPRB, 0.10361E+01_JPRB, 0.10292E+01_JPRB, 0.99939E+00_JPRB, 0.99041E+00_JPRB ,&
&  0.10360E+01_JPRB, 0.10625E+01_JPRB, 0.10717E+01_JPRB, 0.10763E+01_JPRB, 0.11006E+01_JPRB, 0.11731E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.10078E+01_JPRB, 0.97970E+00_JPRB, 0.95756E+00_JPRB, 0.94621E+00_JPRB, 0.93311E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.89906E+00_JPRB, 0.86823E+00_JPRB, 0.84311E+00_JPRB, 0.81875E+00_JPRB, 0.80075E+00_JPRB, 0.78196E+00_JPRB, 0.76964E+00_JPRB ,&
&  0.76458E+00_JPRB, 0.76510E+00_JPRB, 0.76298E+00_JPRB, 0.75191E+00_JPRB, 0.73242E+00_JPRB, 0.71585E+00_JPRB, 0.70321E+00_JPRB ,&
&  0.69062E+00_JPRB, 0.68601E+00_JPRB, 0.68316E+00_JPRB, 0.67246E+00_JPRB, 0.66060E+00_JPRB, 0.65016E+00_JPRB, 0.63996E+00_JPRB ,&
&  0.63125E+00_JPRB, 0.62712E+00_JPRB, 0.62410E+00_JPRB, 0.62141E+00_JPRB, 0.62053E+00_JPRB, 0.64651E+00_JPRB, 0.77204E+00_JPRB ,&
&  0.10433E+01_JPRB, 0.91997E+00_JPRB, 0.20855E+01_JPRB, 0.24110E+01_JPRB, 0.25258E+01_JPRB, 0.27243E+01_JPRB, 0.32340E+01_JPRB ,&
&  0.37566E+01_JPRB, 0.37321E+01_JPRB, 0.32253E+01_JPRB, 0.25012E+01_JPRB, 0.18565E+01_JPRB, 0.14390E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.99657E+00_JPRB, 0.95742E+00_JPRB, 0.93655E+00_JPRB, 0.93484E+00_JPRB, 0.94604E+00_JPRB, 0.96300E+00_JPRB ,&
&  0.99640E+00_JPRB, 0.10723E+01_JPRB, 0.12653E+01_JPRB, 0.16241E+01_JPRB, 0.19427E+01_JPRB, 0.21944E+01_JPRB, 0.27084E+01_JPRB ,&
&  0.35772E+01_JPRB, 0.38158E+01_JPRB, 0.32107E+01_JPRB, 0.27608E+01_JPRB, 0.24653E+01_JPRB, 0.22525E+01_JPRB, 0.20424E+01_JPRB ,&
&  0.18439E+01_JPRB, 0.16547E+01_JPRB, 0.15174E+01_JPRB, 0.14217E+01_JPRB, 0.13671E+01_JPRB, 0.13456E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.13005E+01_JPRB, 0.12640E+01_JPRB, 0.12283E+01_JPRB, 0.11940E+01_JPRB, 0.11643E+01_JPRB, 0.11315E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.10294E+01_JPRB, 0.10070E+01_JPRB, 0.99118E+00_JPRB, 0.98880E+00_JPRB, 0.98279E+00_JPRB, 0.98455E+00_JPRB ,&
&  0.10074E+01_JPRB, 0.10174E+01_JPRB, 0.10255E+01_JPRB, 0.10439E+01_JPRB, 0.10462E+01_JPRB, 0.11100E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.99404E+00_JPRB, 0.95591E+00_JPRB, 0.93493E+00_JPRB, 0.92108E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.87887E+00_JPRB, 0.85475E+00_JPRB, 0.82208E+00_JPRB, 0.79335E+00_JPRB, 0.77067E+00_JPRB, 0.74730E+00_JPRB, 0.73084E+00_JPRB ,&
&  0.71903E+00_JPRB, 0.72048E+00_JPRB, 0.72622E+00_JPRB, 0.72917E+00_JPRB, 0.72248E+00_JPRB, 0.70462E+00_JPRB, 0.68634E+00_JPRB ,&
&  0.67276E+00_JPRB, 0.66738E+00_JPRB, 0.66488E+00_JPRB, 0.65848E+00_JPRB, 0.64902E+00_JPRB, 0.63726E+00_JPRB, 0.62681E+00_JPRB ,&
&  0.61742E+00_JPRB, 0.61095E+00_JPRB, 0.60443E+00_JPRB, 0.59936E+00_JPRB, 0.59361E+00_JPRB, 0.60723E+00_JPRB, 0.72616E+00_JPRB ,&
&  0.10993E+01_JPRB, 0.15797E+01_JPRB, 0.20432E+01_JPRB, 0.19228E+01_JPRB, 0.19150E+01_JPRB, 0.20188E+01_JPRB, 0.23950E+01_JPRB ,&
&  0.26083E+01_JPRB, 0.27026E+01_JPRB, 0.25568E+01_JPRB, 0.21936E+01_JPRB, 0.17553E+01_JPRB, 0.14477E+01_JPRB, 0.12244E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.93208E+00_JPRB, 0.87115E+00_JPRB, 0.85233E+00_JPRB, 0.85668E+00_JPRB, 0.87128E+00_JPRB, 0.88583E+00_JPRB ,&
&  0.90499E+00_JPRB, 0.94925E+00_JPRB, 0.10938E+01_JPRB, 0.14247E+01_JPRB, 0.19583E+01_JPRB, 0.20185E+01_JPRB, 0.26190E+01_JPRB ,&
&  0.33137E+01_JPRB, 0.29969E+01_JPRB, 0.27209E+01_JPRB, 0.24874E+01_JPRB, 0.22996E+01_JPRB, 0.21356E+01_JPRB, 0.18876E+01_JPRB ,&
&  0.16923E+01_JPRB, 0.15152E+01_JPRB, 0.13853E+01_JPRB, 0.13143E+01_JPRB, 0.12719E+01_JPRB, 0.12553E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.12455E+01_JPRB, 0.12339E+01_JPRB, 0.12235E+01_JPRB, 0.11978E+01_JPRB, 0.11599E+01_JPRB, 0.11184E+01_JPRB, 0.10734E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.99702E+00_JPRB, 0.97188E+00_JPRB, 0.94151E+00_JPRB, 0.93967E+00_JPRB, 0.94697E+00_JPRB, 0.94647E+00_JPRB ,&
&  0.94061E+00_JPRB, 0.94824E+00_JPRB, 0.96832E+00_JPRB, 0.97497E+00_JPRB, 0.10152E+01_JPRB, 0.10764E+01_JPRB, 0.10621E+01_JPRB ,&
&  0.10367E+01_JPRB, 0.98725E+00_JPRB, 0.94401E+00_JPRB, 0.91236E+00_JPRB, 0.89671E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.86241E+00_JPRB, 0.83146E+00_JPRB, 0.79791E+00_JPRB, 0.76769E+00_JPRB, 0.74024E+00_JPRB, 0.71924E+00_JPRB, 0.70619E+00_JPRB ,&
&  0.69479E+00_JPRB, 0.68834E+00_JPRB, 0.69130E+00_JPRB, 0.69776E+00_JPRB, 0.69715E+00_JPRB, 0.68645E+00_JPRB, 0.67074E+00_JPRB ,&
&  0.65440E+00_JPRB, 0.64159E+00_JPRB, 0.63219E+00_JPRB, 0.62788E+00_JPRB, 0.62182E+00_JPRB, 0.61533E+00_JPRB, 0.60510E+00_JPRB ,&
&  0.59706E+00_JPRB, 0.59059E+00_JPRB, 0.58102E+00_JPRB, 0.57088E+00_JPRB, 0.56336E+00_JPRB, 0.56902E+00_JPRB, 0.65789E+00_JPRB ,&
&  0.93897E+00_JPRB, 0.13498E+01_JPRB, 0.13990E+01_JPRB, 0.14457E+01_JPRB, 0.15876E+01_JPRB, 0.16262E+01_JPRB, 0.17709E+01_JPRB ,&
&  0.18429E+01_JPRB, 0.18891E+01_JPRB, 0.19121E+01_JPRB, 0.17979E+01_JPRB, 0.15625E+01_JPRB, 0.13749E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.93757E+00_JPRB, 0.86419E+00_JPRB, 0.83209E+00_JPRB, 0.82877E+00_JPRB, 0.83833E+00_JPRB, 0.84444E+00_JPRB ,&
&  0.84661E+00_JPRB, 0.84918E+00_JPRB, 0.89747E+00_JPRB, 0.10294E+01_JPRB, 0.13613E+01_JPRB, 0.15881E+01_JPRB, 0.19472E+01_JPRB ,&
&  0.20418E+01_JPRB, 0.20112E+01_JPRB, 0.19488E+01_JPRB, 0.18755E+01_JPRB, 0.17892E+01_JPRB, 0.16695E+01_JPRB, 0.15651E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.13280E+01_JPRB, 0.12378E+01_JPRB, 0.11787E+01_JPRB, 0.11432E+01_JPRB, 0.11234E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.10957E+01_JPRB, 0.10809E+01_JPRB, 0.10697E+01_JPRB, 0.10616E+01_JPRB, 0.10362E+01_JPRB, 0.99649E+00_JPRB ,&
&  0.95332E+00_JPRB, 0.91530E+00_JPRB, 0.89184E+00_JPRB, 0.88024E+00_JPRB, 0.87500E+00_JPRB, 0.87019E+00_JPRB, 0.86604E+00_JPRB ,&
&  0.87333E+00_JPRB, 0.89362E+00_JPRB, 0.90296E+00_JPRB, 0.90672E+00_JPRB, 0.93048E+00_JPRB, 0.97633E+00_JPRB, 0.97204E+00_JPRB ,&
&  0.96098E+00_JPRB, 0.94046E+00_JPRB, 0.91184E+00_JPRB, 0.88209E+00_JPRB, 0.88127E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.86030E+00_JPRB, 0.83230E+00_JPRB, 0.79643E+00_JPRB, 0.76164E+00_JPRB, 0.72977E+00_JPRB, 0.70482E+00_JPRB, 0.68980E+00_JPRB ,&
&  0.67940E+00_JPRB, 0.67236E+00_JPRB, 0.66877E+00_JPRB, 0.66648E+00_JPRB, 0.66290E+00_JPRB, 0.65869E+00_JPRB, 0.65171E+00_JPRB ,&
&  0.64251E+00_JPRB, 0.63100E+00_JPRB, 0.62043E+00_JPRB, 0.61290E+00_JPRB, 0.60704E+00_JPRB, 0.60332E+00_JPRB, 0.59789E+00_JPRB ,&
&  0.58785E+00_JPRB, 0.57617E+00_JPRB, 0.56172E+00_JPRB, 0.54719E+00_JPRB, 0.53761E+00_JPRB, 0.53944E+00_JPRB, 0.59590E+00_JPRB ,&
&  0.79662E+00_JPRB, 0.92571E+00_JPRB, 0.10378E+01_JPRB, 0.11171E+01_JPRB, 0.12313E+01_JPRB, 0.13005E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.13668E+01_JPRB, 0.14005E+01_JPRB, 0.14638E+01_JPRB, 0.14561E+01_JPRB, 0.13427E+01_JPRB, 0.12277E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.93748E+00_JPRB, 0.86955E+00_JPRB, 0.83685E+00_JPRB, 0.82526E+00_JPRB, 0.82624E+00_JPRB, 0.82540E+00_JPRB ,&
&  0.81856E+00_JPRB, 0.80042E+00_JPRB, 0.79090E+00_JPRB, 0.80434E+00_JPRB, 0.85544E+00_JPRB, 0.95841E+00_JPRB, 0.11307E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.12274E+01_JPRB, 0.12456E+01_JPRB, 0.12604E+01_JPRB, 0.12321E+01_JPRB, 0.11949E+01_JPRB, 0.11605E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.10926E+01_JPRB, 0.10641E+01_JPRB, 0.10292E+01_JPRB, 0.99932E+00_JPRB, 0.97868E+00_JPRB, 0.96470E+00_JPRB ,&
&  0.95330E+00_JPRB, 0.94699E+00_JPRB, 0.93470E+00_JPRB, 0.91128E+00_JPRB, 0.89319E+00_JPRB, 0.87480E+00_JPRB, 0.84869E+00_JPRB ,&
&  0.81452E+00_JPRB, 0.79364E+00_JPRB, 0.78337E+00_JPRB, 0.78354E+00_JPRB, 0.78853E+00_JPRB, 0.79465E+00_JPRB, 0.79648E+00_JPRB ,&
&  0.80039E+00_JPRB, 0.81886E+00_JPRB, 0.81764E+00_JPRB, 0.83873E+00_JPRB, 0.88502E+00_JPRB, 0.88879E+00_JPRB, 0.89319E+00_JPRB ,&
&  0.89228E+00_JPRB, 0.88085E+00_JPRB, 0.85881E+00_JPRB, 0.82878E+00_JPRB, 0.86284E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.74979E+00_JPRB, 0.74966E+00_JPRB, 0.73153E+00_JPRB, 0.70991E+00_JPRB, 0.69039E+00_JPRB, 0.67542E+00_JPRB, 0.66289E+00_JPRB ,&
&  0.65383E+00_JPRB, 0.64682E+00_JPRB, 0.64213E+00_JPRB, 0.63891E+00_JPRB, 0.63447E+00_JPRB, 0.62792E+00_JPRB, 0.62078E+00_JPRB ,&
&  0.61461E+00_JPRB, 0.60717E+00_JPRB, 0.59615E+00_JPRB, 0.58827E+00_JPRB, 0.58355E+00_JPRB, 0.57761E+00_JPRB, 0.57131E+00_JPRB ,&
&  0.56339E+00_JPRB, 0.54833E+00_JPRB, 0.53312E+00_JPRB, 0.52257E+00_JPRB, 0.51415E+00_JPRB, 0.51276E+00_JPRB, 0.55071E+00_JPRB ,&
&  0.67513E+00_JPRB, 0.69314E+00_JPRB, 0.78519E+00_JPRB, 0.87499E+00_JPRB, 0.98881E+00_JPRB, 0.10530E+01_JPRB, 0.10763E+01_JPRB ,&
&  0.10868E+01_JPRB, 0.11264E+01_JPRB, 0.11585E+01_JPRB, 0.12022E+01_JPRB, 0.11590E+01_JPRB, 0.10800E+01_JPRB, 0.10113E+01_JPRB ,&
&  0.94601E+00_JPRB, 0.89227E+00_JPRB, 0.84828E+00_JPRB, 0.81796E+00_JPRB, 0.80476E+00_JPRB, 0.79894E+00_JPRB, 0.78797E+00_JPRB ,&
&  0.77349E+00_JPRB, 0.75113E+00_JPRB, 0.73129E+00_JPRB, 0.72510E+00_JPRB, 0.71620E+00_JPRB, 0.74156E+00_JPRB, 0.78150E+00_JPRB ,&
&  0.81710E+00_JPRB, 0.86889E+00_JPRB, 0.89556E+00_JPRB, 0.91290E+00_JPRB, 0.90800E+00_JPRB, 0.90053E+00_JPRB, 0.89126E+00_JPRB ,&
&  0.88373E+00_JPRB, 0.87464E+00_JPRB, 0.86480E+00_JPRB, 0.85050E+00_JPRB, 0.83319E+00_JPRB, 0.81819E+00_JPRB, 0.80531E+00_JPRB ,&
&  0.79414E+00_JPRB, 0.78349E+00_JPRB, 0.77469E+00_JPRB, 0.75627E+00_JPRB, 0.73028E+00_JPRB, 0.70850E+00_JPRB, 0.69188E+00_JPRB ,&
&  0.67835E+00_JPRB, 0.67354E+00_JPRB, 0.67437E+00_JPRB, 0.68306E+00_JPRB, 0.69190E+00_JPRB, 0.70156E+00_JPRB, 0.70774E+00_JPRB ,&
&  0.71241E+00_JPRB, 0.72584E+00_JPRB, 0.74684E+00_JPRB, 0.78072E+00_JPRB, 0.80029E+00_JPRB, 0.81323E+00_JPRB, 0.82131E+00_JPRB ,&
&  0.82174E+00_JPRB, 0.81657E+00_JPRB, 0.79741E+00_JPRB, 0.77485E+00_JPRB, 0.74080E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.68546E+00_JPRB, 0.68525E+00_JPRB, 0.67507E+00_JPRB, 0.65975E+00_JPRB, 0.64666E+00_JPRB, 0.63525E+00_JPRB, 0.62624E+00_JPRB ,&
&  0.61346E+00_JPRB, 0.60231E+00_JPRB, 0.59782E+00_JPRB, 0.59688E+00_JPRB, 0.59539E+00_JPRB, 0.59044E+00_JPRB, 0.58397E+00_JPRB ,&
&  0.57669E+00_JPRB, 0.56847E+00_JPRB, 0.55886E+00_JPRB, 0.54918E+00_JPRB, 0.54112E+00_JPRB, 0.53199E+00_JPRB, 0.52452E+00_JPRB ,&
&  0.51725E+00_JPRB, 0.51046E+00_JPRB, 0.50024E+00_JPRB, 0.48816E+00_JPRB, 0.48243E+00_JPRB, 0.47774E+00_JPRB, 0.50995E+00_JPRB ,&
&  0.61140E+00_JPRB, 0.54239E+00_JPRB, 0.60294E+00_JPRB, 0.69836E+00_JPRB, 0.80436E+00_JPRB, 0.90455E+00_JPRB, 0.93550E+00_JPRB ,&
&  0.94705E+00_JPRB, 0.96281E+00_JPRB, 0.97452E+00_JPRB, 0.99432E+00_JPRB, 0.10016E+01_JPRB, 0.96049E+00_JPRB, 0.90246E+00_JPRB ,&
&  0.85535E+00_JPRB, 0.81479E+00_JPRB, 0.78913E+00_JPRB, 0.76532E+00_JPRB, 0.75222E+00_JPRB, 0.74486E+00_JPRB, 0.72878E+00_JPRB ,&
&  0.70644E+00_JPRB, 0.69068E+00_JPRB, 0.67869E+00_JPRB, 0.67625E+00_JPRB, 0.67794E+00_JPRB, 0.68063E+00_JPRB, 0.66920E+00_JPRB ,&
&  0.65802E+00_JPRB, 0.67100E+00_JPRB, 0.69730E+00_JPRB, 0.71201E+00_JPRB, 0.71861E+00_JPRB, 0.71550E+00_JPRB, 0.70708E+00_JPRB ,&
&  0.70158E+00_JPRB, 0.69717E+00_JPRB, 0.69640E+00_JPRB, 0.69071E+00_JPRB, 0.68179E+00_JPRB, 0.66883E+00_JPRB, 0.65494E+00_JPRB ,&
&  0.64252E+00_JPRB, 0.63199E+00_JPRB, 0.62319E+00_JPRB, 0.61482E+00_JPRB, 0.60491E+00_JPRB, 0.59205E+00_JPRB, 0.58454E+00_JPRB ,&
&  0.58069E+00_JPRB, 0.58080E+00_JPRB, 0.58462E+00_JPRB, 0.59205E+00_JPRB, 0.60285E+00_JPRB, 0.61301E+00_JPRB, 0.62529E+00_JPRB ,&
&  0.63642E+00_JPRB, 0.64890E+00_JPRB, 0.66019E+00_JPRB, 0.70037E+00_JPRB, 0.73786E+00_JPRB, 0.74928E+00_JPRB, 0.75853E+00_JPRB ,&
&  0.76021E+00_JPRB, 0.75753E+00_JPRB, 0.74903E+00_JPRB, 0.66225E+00_JPRB, 0.68376E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.62894E+00_JPRB, 0.62596E+00_JPRB, 0.61740E+00_JPRB, 0.60756E+00_JPRB, 0.59934E+00_JPRB, 0.59124E+00_JPRB, 0.58470E+00_JPRB ,&
&  0.57559E+00_JPRB, 0.56440E+00_JPRB, 0.55605E+00_JPRB, 0.55221E+00_JPRB, 0.55032E+00_JPRB, 0.54678E+00_JPRB, 0.54074E+00_JPRB ,&
&  0.53302E+00_JPRB, 0.52460E+00_JPRB, 0.51641E+00_JPRB, 0.50648E+00_JPRB, 0.49712E+00_JPRB, 0.48846E+00_JPRB, 0.48108E+00_JPRB ,&
&  0.47372E+00_JPRB, 0.46589E+00_JPRB, 0.45822E+00_JPRB, 0.45267E+00_JPRB, 0.44852E+00_JPRB, 0.44743E+00_JPRB, 0.47405E+00_JPRB ,&
&  0.55428E+00_JPRB, 0.47689E+00_JPRB, 0.51875E+00_JPRB, 0.57658E+00_JPRB, 0.66912E+00_JPRB, 0.78179E+00_JPRB, 0.83184E+00_JPRB ,&
&  0.84985E+00_JPRB, 0.85894E+00_JPRB, 0.85762E+00_JPRB, 0.85531E+00_JPRB, 0.86445E+00_JPRB, 0.85880E+00_JPRB, 0.82812E+00_JPRB ,&
&  0.79180E+00_JPRB, 0.75800E+00_JPRB, 0.73475E+00_JPRB, 0.71767E+00_JPRB, 0.70066E+00_JPRB, 0.68476E+00_JPRB, 0.66437E+00_JPRB ,&
&  0.63789E+00_JPRB, 0.62075E+00_JPRB, 0.61005E+00_JPRB, 0.60263E+00_JPRB, 0.59974E+00_JPRB, 0.59367E+00_JPRB, 0.57621E+00_JPRB ,&
&  0.55954E+00_JPRB, 0.54771E+00_JPRB, 0.54328E+00_JPRB, 0.54618E+00_JPRB, 0.55098E+00_JPRB, 0.55368E+00_JPRB, 0.55078E+00_JPRB ,&
&  0.54606E+00_JPRB, 0.53899E+00_JPRB, 0.53493E+00_JPRB, 0.53268E+00_JPRB, 0.53029E+00_JPRB, 0.52902E+00_JPRB, 0.52608E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.51968E+00_JPRB, 0.52073E+00_JPRB, 0.52219E+00_JPRB, 0.52160E+00_JPRB, 0.51901E+00_JPRB, 0.51580E+00_JPRB ,&
&  0.51168E+00_JPRB, 0.50885E+00_JPRB, 0.50947E+00_JPRB, 0.51481E+00_JPRB, 0.52317E+00_JPRB, 0.53285E+00_JPRB, 0.54363E+00_JPRB ,&
&  0.55559E+00_JPRB, 0.56837E+00_JPRB, 0.58005E+00_JPRB, 0.61091E+00_JPRB, 0.63061E+00_JPRB, 0.65016E+00_JPRB, 0.66811E+00_JPRB ,&
&  0.67364E+00_JPRB, 0.67139E+00_JPRB, 0.57732E+00_JPRB, 0.61555E+00_JPRB, 0.62385E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.58920E+00_JPRB, 0.57782E+00_JPRB, 0.56438E+00_JPRB, 0.55527E+00_JPRB, 0.55097E+00_JPRB, 0.54641E+00_JPRB, 0.54199E+00_JPRB ,&
&  0.53545E+00_JPRB, 0.52817E+00_JPRB, 0.52092E+00_JPRB, 0.51430E+00_JPRB, 0.50890E+00_JPRB, 0.50513E+00_JPRB, 0.50050E+00_JPRB ,&
&  0.49382E+00_JPRB, 0.48606E+00_JPRB, 0.47788E+00_JPRB, 0.46856E+00_JPRB, 0.45651E+00_JPRB, 0.44669E+00_JPRB, 0.43885E+00_JPRB ,&
&  0.43271E+00_JPRB, 0.42785E+00_JPRB, 0.42566E+00_JPRB, 0.42320E+00_JPRB, 0.42069E+00_JPRB, 0.42091E+00_JPRB, 0.44023E+00_JPRB ,&
&  0.46240E+00_JPRB, 0.43652E+00_JPRB, 0.46615E+00_JPRB, 0.50821E+00_JPRB, 0.57733E+00_JPRB, 0.68588E+00_JPRB, 0.75463E+00_JPRB ,&
&  0.78865E+00_JPRB, 0.80246E+00_JPRB, 0.80431E+00_JPRB, 0.79241E+00_JPRB, 0.78173E+00_JPRB, 0.78403E+00_JPRB, 0.77885E+00_JPRB ,&
&  0.75625E+00_JPRB, 0.72517E+00_JPRB, 0.69671E+00_JPRB, 0.67529E+00_JPRB, 0.65002E+00_JPRB, 0.62724E+00_JPRB, 0.60390E+00_JPRB ,&
&  0.58288E+00_JPRB, 0.56787E+00_JPRB, 0.55833E+00_JPRB, 0.55110E+00_JPRB, 0.54355E+00_JPRB, 0.53093E+00_JPRB, 0.51547E+00_JPRB ,&
&  0.50239E+00_JPRB, 0.49056E+00_JPRB, 0.48009E+00_JPRB, 0.47145E+00_JPRB, 0.46576E+00_JPRB, 0.46215E+00_JPRB, 0.45917E+00_JPRB ,&
&  0.45833E+00_JPRB, 0.45885E+00_JPRB, 0.46014E+00_JPRB, 0.46125E+00_JPRB, 0.46093E+00_JPRB, 0.45912E+00_JPRB, 0.45800E+00_JPRB ,&
&  0.45753E+00_JPRB, 0.45447E+00_JPRB, 0.45374E+00_JPRB, 0.45775E+00_JPRB, 0.46269E+00_JPRB, 0.46557E+00_JPRB, 0.46625E+00_JPRB ,&
&  0.46450E+00_JPRB, 0.46300E+00_JPRB, 0.46316E+00_JPRB, 0.46574E+00_JPRB, 0.46968E+00_JPRB, 0.47584E+00_JPRB, 0.48346E+00_JPRB ,&
&  0.49137E+00_JPRB, 0.50147E+00_JPRB, 0.51452E+00_JPRB, 0.53493E+00_JPRB, 0.55237E+00_JPRB, 0.56764E+00_JPRB, 0.58195E+00_JPRB ,&
&  0.58774E+00_JPRB, 0.59109E+00_JPRB, 0.59683E+00_JPRB, 0.59604E+00_JPRB, 0.59443E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.52209E+00_JPRB, 0.51568E+00_JPRB, 0.50767E+00_JPRB, 0.50061E+00_JPRB, 0.49677E+00_JPRB, 0.49347E+00_JPRB, 0.48835E+00_JPRB ,&
&  0.48397E+00_JPRB, 0.48009E+00_JPRB, 0.47611E+00_JPRB, 0.47252E+00_JPRB, 0.46924E+00_JPRB, 0.46662E+00_JPRB, 0.46256E+00_JPRB ,&
&  0.45617E+00_JPRB, 0.44818E+00_JPRB, 0.43924E+00_JPRB, 0.43162E+00_JPRB, 0.42239E+00_JPRB, 0.41277E+00_JPRB, 0.40544E+00_JPRB ,&
&  0.39933E+00_JPRB, 0.39508E+00_JPRB, 0.39327E+00_JPRB, 0.39100E+00_JPRB, 0.39153E+00_JPRB, 0.39447E+00_JPRB, 0.40643E+00_JPRB ,&
&  0.43723E+00_JPRB, 0.42270E+00_JPRB, 0.45203E+00_JPRB, 0.47988E+00_JPRB, 0.53634E+00_JPRB, 0.62278E+00_JPRB, 0.68002E+00_JPRB ,&
&  0.71380E+00_JPRB, 0.74072E+00_JPRB, 0.75911E+00_JPRB, 0.75929E+00_JPRB, 0.74483E+00_JPRB, 0.73831E+00_JPRB, 0.73551E+00_JPRB ,&
&  0.72269E+00_JPRB, 0.69849E+00_JPRB, 0.66446E+00_JPRB, 0.62779E+00_JPRB, 0.59538E+00_JPRB, 0.57095E+00_JPRB, 0.55275E+00_JPRB ,&
&  0.53901E+00_JPRB, 0.52755E+00_JPRB, 0.51806E+00_JPRB, 0.50506E+00_JPRB, 0.48994E+00_JPRB, 0.47448E+00_JPRB, 0.45937E+00_JPRB ,&
&  0.44416E+00_JPRB, 0.42964E+00_JPRB, 0.41729E+00_JPRB, 0.40735E+00_JPRB, 0.40138E+00_JPRB, 0.39799E+00_JPRB, 0.39611E+00_JPRB ,&
&  0.39567E+00_JPRB, 0.39740E+00_JPRB, 0.40008E+00_JPRB, 0.40275E+00_JPRB, 0.40456E+00_JPRB, 0.40427E+00_JPRB, 0.40144E+00_JPRB ,&
&  0.40001E+00_JPRB, 0.39919E+00_JPRB, 0.39940E+00_JPRB, 0.40148E+00_JPRB, 0.40268E+00_JPRB, 0.40438E+00_JPRB, 0.40790E+00_JPRB ,&
&  0.41097E+00_JPRB, 0.41245E+00_JPRB, 0.41352E+00_JPRB, 0.41497E+00_JPRB, 0.41693E+00_JPRB, 0.42036E+00_JPRB, 0.42562E+00_JPRB ,&
&  0.43176E+00_JPRB, 0.43874E+00_JPRB, 0.44898E+00_JPRB, 0.46240E+00_JPRB, 0.47559E+00_JPRB, 0.48770E+00_JPRB, 0.49883E+00_JPRB ,&
&  0.50491E+00_JPRB, 0.51031E+00_JPRB, 0.51789E+00_JPRB, 0.52173E+00_JPRB, 0.52439E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.46669E+00_JPRB, 0.46392E+00_JPRB, 0.46036E+00_JPRB, 0.45698E+00_JPRB, 0.45419E+00_JPRB, 0.44939E+00_JPRB, 0.44387E+00_JPRB ,&
&  0.44036E+00_JPRB, 0.43739E+00_JPRB, 0.43533E+00_JPRB, 0.43198E+00_JPRB, 0.43004E+00_JPRB, 0.42872E+00_JPRB, 0.42423E+00_JPRB ,&
&  0.41885E+00_JPRB, 0.41117E+00_JPRB, 0.40359E+00_JPRB, 0.39662E+00_JPRB, 0.38992E+00_JPRB, 0.38359E+00_JPRB, 0.37658E+00_JPRB ,&
&  0.36996E+00_JPRB, 0.36496E+00_JPRB, 0.36203E+00_JPRB, 0.35958E+00_JPRB, 0.36074E+00_JPRB, 0.36672E+00_JPRB, 0.37676E+00_JPRB ,&
&  0.39075E+00_JPRB, 0.41194E+00_JPRB, 0.43217E+00_JPRB, 0.45355E+00_JPRB, 0.49596E+00_JPRB, 0.55933E+00_JPRB, 0.60655E+00_JPRB ,&
&  0.62942E+00_JPRB, 0.65507E+00_JPRB, 0.68887E+00_JPRB, 0.71153E+00_JPRB, 0.71358E+00_JPRB, 0.70249E+00_JPRB, 0.69390E+00_JPRB ,&
&  0.68231E+00_JPRB, 0.65935E+00_JPRB, 0.63362E+00_JPRB, 0.60408E+00_JPRB, 0.57470E+00_JPRB, 0.55072E+00_JPRB, 0.53227E+00_JPRB ,&
&  0.51667E+00_JPRB, 0.50269E+00_JPRB, 0.48908E+00_JPRB, 0.47432E+00_JPRB, 0.45820E+00_JPRB, 0.44029E+00_JPRB, 0.42116E+00_JPRB ,&
&  0.40295E+00_JPRB, 0.38789E+00_JPRB, 0.37658E+00_JPRB, 0.36994E+00_JPRB, 0.36565E+00_JPRB, 0.36189E+00_JPRB, 0.35910E+00_JPRB ,&
&  0.35842E+00_JPRB, 0.35938E+00_JPRB, 0.36156E+00_JPRB, 0.36420E+00_JPRB, 0.36656E+00_JPRB, 0.36722E+00_JPRB, 0.36454E+00_JPRB ,&
&  0.36167E+00_JPRB, 0.36109E+00_JPRB, 0.36129E+00_JPRB, 0.35844E+00_JPRB, 0.35524E+00_JPRB, 0.35604E+00_JPRB, 0.35919E+00_JPRB ,&
&  0.36445E+00_JPRB, 0.36913E+00_JPRB, 0.37276E+00_JPRB, 0.37557E+00_JPRB, 0.37752E+00_JPRB, 0.38100E+00_JPRB, 0.38546E+00_JPRB ,&
&  0.39109E+00_JPRB, 0.39811E+00_JPRB, 0.40784E+00_JPRB, 0.41989E+00_JPRB, 0.43095E+00_JPRB, 0.44043E+00_JPRB, 0.44669E+00_JPRB ,&
&  0.45092E+00_JPRB, 0.45420E+00_JPRB, 0.45758E+00_JPRB, 0.46181E+00_JPRB, 0.46559E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.41452E+00_JPRB, 0.41462E+00_JPRB, 0.41523E+00_JPRB, 0.41499E+00_JPRB, 0.41137E+00_JPRB, 0.40748E+00_JPRB, 0.40322E+00_JPRB ,&
&  0.40106E+00_JPRB, 0.39957E+00_JPRB, 0.39906E+00_JPRB, 0.39594E+00_JPRB, 0.39356E+00_JPRB, 0.39206E+00_JPRB, 0.38798E+00_JPRB ,&
&  0.38375E+00_JPRB, 0.37510E+00_JPRB, 0.36603E+00_JPRB, 0.35947E+00_JPRB, 0.35533E+00_JPRB, 0.35229E+00_JPRB, 0.34635E+00_JPRB ,&
&  0.34181E+00_JPRB, 0.33816E+00_JPRB, 0.33623E+00_JPRB, 0.33615E+00_JPRB, 0.33839E+00_JPRB, 0.34457E+00_JPRB, 0.35449E+00_JPRB ,&
&  0.36290E+00_JPRB, 0.38226E+00_JPRB, 0.38677E+00_JPRB, 0.40904E+00_JPRB, 0.43610E+00_JPRB, 0.47741E+00_JPRB, 0.51920E+00_JPRB ,&
&  0.54232E+00_JPRB, 0.55615E+00_JPRB, 0.57967E+00_JPRB, 0.60875E+00_JPRB, 0.62308E+00_JPRB, 0.62613E+00_JPRB, 0.62065E+00_JPRB ,&
&  0.61200E+00_JPRB, 0.60276E+00_JPRB, 0.59222E+00_JPRB, 0.57781E+00_JPRB, 0.55775E+00_JPRB, 0.53574E+00_JPRB, 0.51487E+00_JPRB ,&
&  0.49607E+00_JPRB, 0.47795E+00_JPRB, 0.45984E+00_JPRB, 0.44001E+00_JPRB, 0.41943E+00_JPRB, 0.39964E+00_JPRB, 0.38050E+00_JPRB ,&
&  0.36511E+00_JPRB, 0.35345E+00_JPRB, 0.34489E+00_JPRB, 0.33846E+00_JPRB, 0.33341E+00_JPRB, 0.32870E+00_JPRB, 0.32600E+00_JPRB ,&
&  0.32523E+00_JPRB, 0.32648E+00_JPRB, 0.32905E+00_JPRB, 0.33131E+00_JPRB, 0.33407E+00_JPRB, 0.33443E+00_JPRB, 0.33065E+00_JPRB ,&
&  0.32710E+00_JPRB, 0.32611E+00_JPRB, 0.32124E+00_JPRB, 0.31828E+00_JPRB, 0.31755E+00_JPRB, 0.31818E+00_JPRB, 0.32063E+00_JPRB ,&
&  0.32533E+00_JPRB, 0.33159E+00_JPRB, 0.33730E+00_JPRB, 0.34221E+00_JPRB, 0.34655E+00_JPRB, 0.35085E+00_JPRB, 0.35572E+00_JPRB ,&
&  0.36084E+00_JPRB, 0.36579E+00_JPRB, 0.37332E+00_JPRB, 0.38172E+00_JPRB, 0.38898E+00_JPRB, 0.39688E+00_JPRB, 0.40195E+00_JPRB ,&
&  0.40417E+00_JPRB, 0.40522E+00_JPRB, 0.40771E+00_JPRB, 0.41140E+00_JPRB, 0.41393E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.37135E+00_JPRB, 0.37312E+00_JPRB, 0.37517E+00_JPRB, 0.37519E+00_JPRB, 0.37414E+00_JPRB, 0.37361E+00_JPRB, 0.37205E+00_JPRB ,&
&  0.36921E+00_JPRB, 0.36741E+00_JPRB, 0.36733E+00_JPRB, 0.36565E+00_JPRB, 0.36253E+00_JPRB, 0.35872E+00_JPRB, 0.35446E+00_JPRB ,&
&  0.35090E+00_JPRB, 0.34722E+00_JPRB, 0.34125E+00_JPRB, 0.33369E+00_JPRB, 0.32725E+00_JPRB, 0.32303E+00_JPRB, 0.32064E+00_JPRB ,&
&  0.31947E+00_JPRB, 0.31918E+00_JPRB, 0.31865E+00_JPRB, 0.31926E+00_JPRB, 0.32155E+00_JPRB, 0.32675E+00_JPRB, 0.33433E+00_JPRB ,&
&  0.34169E+00_JPRB, 0.35051E+00_JPRB, 0.35971E+00_JPRB, 0.37037E+00_JPRB, 0.38406E+00_JPRB, 0.40386E+00_JPRB, 0.42730E+00_JPRB ,&
&  0.45268E+00_JPRB, 0.46192E+00_JPRB, 0.46447E+00_JPRB, 0.47370E+00_JPRB, 0.49084E+00_JPRB, 0.50487E+00_JPRB, 0.51463E+00_JPRB ,&
&  0.52027E+00_JPRB, 0.52067E+00_JPRB, 0.51822E+00_JPRB, 0.51046E+00_JPRB, 0.49804E+00_JPRB, 0.48194E+00_JPRB, 0.46431E+00_JPRB ,&
&  0.44708E+00_JPRB, 0.42939E+00_JPRB, 0.41069E+00_JPRB, 0.39228E+00_JPRB, 0.37535E+00_JPRB, 0.35976E+00_JPRB, 0.34534E+00_JPRB ,&
&  0.33302E+00_JPRB, 0.32346E+00_JPRB, 0.31601E+00_JPRB, 0.31059E+00_JPRB, 0.30610E+00_JPRB, 0.30258E+00_JPRB, 0.30105E+00_JPRB ,&
&  0.30091E+00_JPRB, 0.30194E+00_JPRB, 0.30242E+00_JPRB, 0.30388E+00_JPRB, 0.30651E+00_JPRB, 0.30727E+00_JPRB, 0.30532E+00_JPRB ,&
&  0.30067E+00_JPRB, 0.29626E+00_JPRB, 0.29183E+00_JPRB, 0.29105E+00_JPRB, 0.29104E+00_JPRB, 0.29267E+00_JPRB, 0.29516E+00_JPRB ,&
&  0.29968E+00_JPRB, 0.30577E+00_JPRB, 0.31134E+00_JPRB, 0.31556E+00_JPRB, 0.32019E+00_JPRB, 0.32428E+00_JPRB, 0.32818E+00_JPRB ,&
&  0.33232E+00_JPRB, 0.33664E+00_JPRB, 0.34230E+00_JPRB, 0.34801E+00_JPRB, 0.35326E+00_JPRB, 0.35748E+00_JPRB, 0.36064E+00_JPRB ,&
&  0.36284E+00_JPRB, 0.36370E+00_JPRB, 0.36437E+00_JPRB, 0.36637E+00_JPRB, 0.36919E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.33424E+00_JPRB, 0.33652E+00_JPRB, 0.33934E+00_JPRB, 0.34137E+00_JPRB, 0.34190E+00_JPRB, 0.34116E+00_JPRB, 0.34018E+00_JPRB ,&
&  0.33959E+00_JPRB, 0.33881E+00_JPRB, 0.33681E+00_JPRB, 0.33389E+00_JPRB, 0.33103E+00_JPRB, 0.32808E+00_JPRB, 0.32593E+00_JPRB ,&
&  0.32387E+00_JPRB, 0.32153E+00_JPRB, 0.31730E+00_JPRB, 0.31104E+00_JPRB, 0.30460E+00_JPRB, 0.30100E+00_JPRB, 0.29947E+00_JPRB ,&
&  0.29968E+00_JPRB, 0.30054E+00_JPRB, 0.30322E+00_JPRB, 0.30503E+00_JPRB, 0.30749E+00_JPRB, 0.31167E+00_JPRB, 0.31756E+00_JPRB ,&
&  0.32291E+00_JPRB, 0.32815E+00_JPRB, 0.33396E+00_JPRB, 0.33944E+00_JPRB, 0.34503E+00_JPRB, 0.35304E+00_JPRB, 0.36356E+00_JPRB ,&
&  0.37894E+00_JPRB, 0.39150E+00_JPRB, 0.39184E+00_JPRB, 0.39186E+00_JPRB, 0.39769E+00_JPRB, 0.40655E+00_JPRB, 0.41451E+00_JPRB ,&
&  0.42101E+00_JPRB, 0.42255E+00_JPRB, 0.42158E+00_JPRB, 0.41761E+00_JPRB, 0.41209E+00_JPRB, 0.40499E+00_JPRB, 0.39520E+00_JPRB ,&
&  0.38341E+00_JPRB, 0.37041E+00_JPRB, 0.35757E+00_JPRB, 0.34482E+00_JPRB, 0.33226E+00_JPRB, 0.32088E+00_JPRB, 0.31108E+00_JPRB ,&
&  0.30254E+00_JPRB, 0.29587E+00_JPRB, 0.28974E+00_JPRB, 0.28566E+00_JPRB, 0.28283E+00_JPRB, 0.27993E+00_JPRB, 0.27908E+00_JPRB ,&
&  0.27775E+00_JPRB, 0.27673E+00_JPRB, 0.27634E+00_JPRB, 0.27761E+00_JPRB, 0.28048E+00_JPRB, 0.28217E+00_JPRB, 0.28058E+00_JPRB ,&
&  0.27705E+00_JPRB, 0.27490E+00_JPRB, 0.27355E+00_JPRB, 0.27265E+00_JPRB, 0.27037E+00_JPRB, 0.27042E+00_JPRB, 0.27300E+00_JPRB ,&
&  0.27817E+00_JPRB, 0.28364E+00_JPRB, 0.28810E+00_JPRB, 0.29088E+00_JPRB, 0.29387E+00_JPRB, 0.29658E+00_JPRB, 0.29893E+00_JPRB ,&
&  0.30123E+00_JPRB, 0.30281E+00_JPRB, 0.30394E+00_JPRB, 0.30544E+00_JPRB, 0.30794E+00_JPRB, 0.31037E+00_JPRB, 0.31278E+00_JPRB ,&
&  0.31562E+00_JPRB, 0.32012E+00_JPRB, 0.32466E+00_JPRB, 0.32807E+00_JPRB, 0.33192E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.30869E+00_JPRB, 0.31132E+00_JPRB, 0.31350E+00_JPRB, 0.31309E+00_JPRB, 0.31071E+00_JPRB, 0.30936E+00_JPRB, 0.30934E+00_JPRB ,&
&  0.30952E+00_JPRB, 0.30895E+00_JPRB, 0.30700E+00_JPRB, 0.30654E+00_JPRB, 0.30643E+00_JPRB, 0.30555E+00_JPRB, 0.30504E+00_JPRB ,&
&  0.30370E+00_JPRB, 0.30041E+00_JPRB, 0.29562E+00_JPRB, 0.28985E+00_JPRB, 0.28574E+00_JPRB, 0.28392E+00_JPRB, 0.28420E+00_JPRB ,&
&  0.28565E+00_JPRB, 0.28669E+00_JPRB, 0.28943E+00_JPRB, 0.29149E+00_JPRB, 0.29384E+00_JPRB, 0.29718E+00_JPRB, 0.30083E+00_JPRB ,&
&  0.30331E+00_JPRB, 0.30600E+00_JPRB, 0.30897E+00_JPRB, 0.31201E+00_JPRB, 0.31606E+00_JPRB, 0.32244E+00_JPRB, 0.32853E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.34683E+00_JPRB, 0.34372E+00_JPRB, 0.33873E+00_JPRB, 0.33835E+00_JPRB, 0.34179E+00_JPRB, 0.34672E+00_JPRB ,&
&  0.35226E+00_JPRB, 0.35757E+00_JPRB, 0.36173E+00_JPRB, 0.36293E+00_JPRB, 0.36080E+00_JPRB, 0.35576E+00_JPRB, 0.34807E+00_JPRB ,&
&  0.33894E+00_JPRB, 0.32922E+00_JPRB, 0.31928E+00_JPRB, 0.30895E+00_JPRB, 0.29990E+00_JPRB, 0.29169E+00_JPRB, 0.28455E+00_JPRB ,&
&  0.27808E+00_JPRB, 0.27210E+00_JPRB, 0.26759E+00_JPRB, 0.26452E+00_JPRB, 0.26286E+00_JPRB, 0.26103E+00_JPRB, 0.26071E+00_JPRB ,&
&  0.25964E+00_JPRB, 0.25899E+00_JPRB, 0.25857E+00_JPRB, 0.25890E+00_JPRB, 0.26012E+00_JPRB, 0.26088E+00_JPRB, 0.26002E+00_JPRB ,&
&  0.25961E+00_JPRB, 0.26110E+00_JPRB, 0.26091E+00_JPRB, 0.25826E+00_JPRB, 0.25496E+00_JPRB, 0.25433E+00_JPRB, 0.25615E+00_JPRB ,&
&  0.25952E+00_JPRB, 0.26309E+00_JPRB, 0.26619E+00_JPRB, 0.26876E+00_JPRB, 0.27090E+00_JPRB, 0.27357E+00_JPRB, 0.27603E+00_JPRB ,&
&  0.27816E+00_JPRB, 0.27943E+00_JPRB, 0.28132E+00_JPRB, 0.28332E+00_JPRB, 0.28531E+00_JPRB, 0.28666E+00_JPRB, 0.28647E+00_JPRB ,&
&  0.28596E+00_JPRB, 0.28785E+00_JPRB, 0.29322E+00_JPRB, 0.30082E+00_JPRB, 0.30564E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.28639E+00_JPRB, 0.28851E+00_JPRB, 0.28939E+00_JPRB, 0.28863E+00_JPRB, 0.28743E+00_JPRB, 0.28772E+00_JPRB, 0.28808E+00_JPRB ,&
&  0.28875E+00_JPRB, 0.28857E+00_JPRB, 0.28669E+00_JPRB, 0.28501E+00_JPRB, 0.28646E+00_JPRB, 0.28613E+00_JPRB, 0.28573E+00_JPRB ,&
&  0.28411E+00_JPRB, 0.28198E+00_JPRB, 0.27901E+00_JPRB, 0.27574E+00_JPRB, 0.27205E+00_JPRB, 0.27037E+00_JPRB, 0.27157E+00_JPRB ,&
&  0.27380E+00_JPRB, 0.27470E+00_JPRB, 0.27546E+00_JPRB, 0.27616E+00_JPRB, 0.27787E+00_JPRB, 0.28013E+00_JPRB, 0.28177E+00_JPRB ,&
&  0.28293E+00_JPRB, 0.28580E+00_JPRB, 0.28759E+00_JPRB, 0.28728E+00_JPRB, 0.29152E+00_JPRB, 0.29646E+00_JPRB, 0.30193E+00_JPRB ,&
&  0.30798E+00_JPRB, 0.31200E+00_JPRB, 0.31215E+00_JPRB, 0.31040E+00_JPRB, 0.30910E+00_JPRB, 0.31037E+00_JPRB, 0.31155E+00_JPRB ,&
&  0.31337E+00_JPRB, 0.31812E+00_JPRB, 0.32377E+00_JPRB, 0.32709E+00_JPRB, 0.32734E+00_JPRB, 0.32370E+00_JPRB, 0.31620E+00_JPRB ,&
&  0.30543E+00_JPRB, 0.29347E+00_JPRB, 0.28324E+00_JPRB, 0.27492E+00_JPRB, 0.26862E+00_JPRB, 0.26388E+00_JPRB, 0.26009E+00_JPRB ,&
&  0.25596E+00_JPRB, 0.25287E+00_JPRB, 0.25246E+00_JPRB, 0.25282E+00_JPRB, 0.25222E+00_JPRB, 0.24943E+00_JPRB, 0.24752E+00_JPRB ,&
&  0.24579E+00_JPRB, 0.24620E+00_JPRB, 0.24741E+00_JPRB, 0.24787E+00_JPRB, 0.24806E+00_JPRB, 0.24799E+00_JPRB, 0.24786E+00_JPRB ,&
&  0.24884E+00_JPRB, 0.24839E+00_JPRB, 0.24692E+00_JPRB, 0.24464E+00_JPRB, 0.24263E+00_JPRB, 0.24234E+00_JPRB, 0.24305E+00_JPRB ,&
&  0.24425E+00_JPRB, 0.24628E+00_JPRB, 0.24874E+00_JPRB, 0.25118E+00_JPRB, 0.25309E+00_JPRB, 0.25403E+00_JPRB, 0.25460E+00_JPRB ,&
&  0.25507E+00_JPRB, 0.25541E+00_JPRB, 0.25727E+00_JPRB, 0.26161E+00_JPRB, 0.26558E+00_JPRB, 0.26838E+00_JPRB, 0.26961E+00_JPRB ,&
&  0.26879E+00_JPRB, 0.26923E+00_JPRB, 0.27242E+00_JPRB, 0.27845E+00_JPRB, 0.28287E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.26888E+00_JPRB, 0.27084E+00_JPRB, 0.27117E+00_JPRB, 0.27037E+00_JPRB, 0.26948E+00_JPRB, 0.27013E+00_JPRB, 0.27126E+00_JPRB ,&
&  0.27184E+00_JPRB, 0.27187E+00_JPRB, 0.27059E+00_JPRB, 0.26814E+00_JPRB, 0.26616E+00_JPRB, 0.26612E+00_JPRB, 0.26462E+00_JPRB ,&
&  0.26351E+00_JPRB, 0.26435E+00_JPRB, 0.26377E+00_JPRB, 0.26302E+00_JPRB, 0.26159E+00_JPRB, 0.26121E+00_JPRB, 0.26261E+00_JPRB ,&
&  0.26502E+00_JPRB, 0.26554E+00_JPRB, 0.26600E+00_JPRB, 0.26661E+00_JPRB, 0.26787E+00_JPRB, 0.26955E+00_JPRB, 0.27034E+00_JPRB ,&
&  0.27056E+00_JPRB, 0.26878E+00_JPRB, 0.26751E+00_JPRB, 0.26985E+00_JPRB, 0.27393E+00_JPRB, 0.27960E+00_JPRB, 0.28386E+00_JPRB ,&
&  0.28745E+00_JPRB, 0.28965E+00_JPRB, 0.29035E+00_JPRB, 0.28992E+00_JPRB, 0.28997E+00_JPRB, 0.29237E+00_JPRB, 0.29458E+00_JPRB ,&
&  0.29577E+00_JPRB, 0.29658E+00_JPRB, 0.29629E+00_JPRB, 0.29621E+00_JPRB, 0.29376E+00_JPRB, 0.28904E+00_JPRB, 0.28038E+00_JPRB ,&
&  0.27074E+00_JPRB, 0.26259E+00_JPRB, 0.25633E+00_JPRB, 0.25146E+00_JPRB, 0.24829E+00_JPRB, 0.24809E+00_JPRB, 0.24785E+00_JPRB ,&
&  0.24708E+00_JPRB, 0.24634E+00_JPRB, 0.24754E+00_JPRB, 0.24759E+00_JPRB, 0.24146E+00_JPRB, 0.23692E+00_JPRB, 0.23162E+00_JPRB ,&
&  0.23008E+00_JPRB, 0.22994E+00_JPRB, 0.22972E+00_JPRB, 0.23314E+00_JPRB, 0.23793E+00_JPRB, 0.23902E+00_JPRB, 0.23740E+00_JPRB ,&
&  0.23050E+00_JPRB, 0.22602E+00_JPRB, 0.22294E+00_JPRB, 0.22000E+00_JPRB, 0.21843E+00_JPRB, 0.21897E+00_JPRB, 0.21994E+00_JPRB ,&
&  0.22258E+00_JPRB, 0.22407E+00_JPRB, 0.22684E+00_JPRB, 0.22847E+00_JPRB, 0.22844E+00_JPRB, 0.22864E+00_JPRB, 0.22800E+00_JPRB ,&
&  0.23017E+00_JPRB, 0.23572E+00_JPRB, 0.24355E+00_JPRB, 0.24559E+00_JPRB, 0.25080E+00_JPRB, 0.25439E+00_JPRB, 0.25575E+00_JPRB ,&
&  0.25638E+00_JPRB, 0.25635E+00_JPRB, 0.25589E+00_JPRB, 0.25968E+00_JPRB, 0.26538E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.25469E+00_JPRB, 0.25602E+00_JPRB, 0.25638E+00_JPRB, 0.25603E+00_JPRB, 0.25552E+00_JPRB, 0.25565E+00_JPRB, 0.25592E+00_JPRB ,&
&  0.25563E+00_JPRB, 0.25467E+00_JPRB, 0.25327E+00_JPRB, 0.25086E+00_JPRB, 0.24786E+00_JPRB, 0.24607E+00_JPRB, 0.24719E+00_JPRB ,&
&  0.24901E+00_JPRB, 0.25036E+00_JPRB, 0.25187E+00_JPRB, 0.25285E+00_JPRB, 0.25348E+00_JPRB, 0.25454E+00_JPRB, 0.25613E+00_JPRB ,&
&  0.25738E+00_JPRB, 0.25789E+00_JPRB, 0.25913E+00_JPRB, 0.26004E+00_JPRB, 0.26206E+00_JPRB, 0.26382E+00_JPRB, 0.26391E+00_JPRB ,&
&  0.25972E+00_JPRB, 0.25175E+00_JPRB, 0.24705E+00_JPRB, 0.24994E+00_JPRB, 0.25726E+00_JPRB, 0.26361E+00_JPRB, 0.26790E+00_JPRB ,&
&  0.27048E+00_JPRB, 0.27133E+00_JPRB, 0.27133E+00_JPRB, 0.27262E+00_JPRB, 0.27500E+00_JPRB, 0.27766E+00_JPRB, 0.27768E+00_JPRB ,&
&  0.27598E+00_JPRB, 0.27464E+00_JPRB, 0.27265E+00_JPRB, 0.26870E+00_JPRB, 0.26277E+00_JPRB, 0.25666E+00_JPRB, 0.25104E+00_JPRB ,&
&  0.24572E+00_JPRB, 0.24061E+00_JPRB, 0.23659E+00_JPRB, 0.23405E+00_JPRB, 0.23345E+00_JPRB, 0.23510E+00_JPRB, 0.23439E+00_JPRB ,&
&  0.23516E+00_JPRB, 0.23305E+00_JPRB, 0.23074E+00_JPRB, 0.22765E+00_JPRB, 0.22488E+00_JPRB, 0.21973E+00_JPRB, 0.21316E+00_JPRB ,&
&  0.20998E+00_JPRB, 0.21325E+00_JPRB, 0.22149E+00_JPRB, 0.22721E+00_JPRB, 0.22233E+00_JPRB, 0.21697E+00_JPRB, 0.21443E+00_JPRB ,&
&  0.21052E+00_JPRB, 0.20581E+00_JPRB, 0.20261E+00_JPRB, 0.20178E+00_JPRB, 0.20193E+00_JPRB, 0.20209E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.20517E+00_JPRB, 0.20684E+00_JPRB, 0.20772E+00_JPRB, 0.20892E+00_JPRB, 0.20941E+00_JPRB, 0.20949E+00_JPRB, 0.20958E+00_JPRB ,&
&  0.21172E+00_JPRB, 0.21609E+00_JPRB, 0.22164E+00_JPRB, 0.22768E+00_JPRB, 0.23210E+00_JPRB, 0.23612E+00_JPRB, 0.23910E+00_JPRB ,&
&  0.24161E+00_JPRB, 0.24159E+00_JPRB, 0.24461E+00_JPRB, 0.24906E+00_JPRB, 0.25233E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.23893E+00_JPRB, 0.24282E+00_JPRB, 0.24404E+00_JPRB, 0.24379E+00_JPRB, 0.24244E+00_JPRB, 0.24114E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24012E+00_JPRB, 0.23877E+00_JPRB, 0.23779E+00_JPRB, 0.23716E+00_JPRB, 0.23649E+00_JPRB, 0.23559E+00_JPRB, 0.23605E+00_JPRB ,&
&  0.23787E+00_JPRB, 0.24050E+00_JPRB, 0.24277E+00_JPRB, 0.24440E+00_JPRB, 0.24461E+00_JPRB, 0.24536E+00_JPRB, 0.24639E+00_JPRB ,&
&  0.24645E+00_JPRB, 0.24626E+00_JPRB, 0.24944E+00_JPRB, 0.24932E+00_JPRB, 0.25011E+00_JPRB, 0.24978E+00_JPRB, 0.24968E+00_JPRB ,&
&  0.24385E+00_JPRB, 0.23808E+00_JPRB, 0.23629E+00_JPRB, 0.23899E+00_JPRB, 0.24706E+00_JPRB, 0.24846E+00_JPRB, 0.25266E+00_JPRB ,&
&  0.25484E+00_JPRB, 0.25565E+00_JPRB, 0.25752E+00_JPRB, 0.25900E+00_JPRB, 0.26071E+00_JPRB, 0.26027E+00_JPRB, 0.25821E+00_JPRB ,&
&  0.25508E+00_JPRB, 0.25426E+00_JPRB, 0.25109E+00_JPRB, 0.23579E+00_JPRB, 0.22691E+00_JPRB, 0.22182E+00_JPRB, 0.21632E+00_JPRB ,&
&  0.21118E+00_JPRB, 0.20880E+00_JPRB, 0.20854E+00_JPRB, 0.20681E+00_JPRB, 0.20782E+00_JPRB, 0.21053E+00_JPRB, 0.21158E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21167E+00_JPRB, 0.21132E+00_JPRB, 0.21003E+00_JPRB, 0.20746E+00_JPRB, 0.20582E+00_JPRB, 0.20486E+00_JPRB ,&
&  0.20547E+00_JPRB, 0.20495E+00_JPRB, 0.20942E+00_JPRB, 0.21030E+00_JPRB, 0.20348E+00_JPRB, 0.20325E+00_JPRB, 0.20234E+00_JPRB ,&
&  0.19622E+00_JPRB, 0.19288E+00_JPRB, 0.19092E+00_JPRB, 0.19013E+00_JPRB, 0.19134E+00_JPRB, 0.19300E+00_JPRB, 0.19364E+00_JPRB ,&
&  0.19373E+00_JPRB, 0.19501E+00_JPRB, 0.19560E+00_JPRB, 0.19627E+00_JPRB, 0.19668E+00_JPRB, 0.19772E+00_JPRB, 0.19926E+00_JPRB ,&
&  0.20119E+00_JPRB, 0.20476E+00_JPRB, 0.20819E+00_JPRB, 0.21198E+00_JPRB, 0.21410E+00_JPRB, 0.21565E+00_JPRB, 0.21957E+00_JPRB ,&
&  0.22119E+00_JPRB, 0.22906E+00_JPRB, 0.24018E+00_JPRB, 0.24068E+00_JPRB, 0.23372E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.23043E+00_JPRB, 0.23206E+00_JPRB, 0.23314E+00_JPRB, 0.23200E+00_JPRB, 0.23211E+00_JPRB, 0.23180E+00_JPRB, 0.23170E+00_JPRB ,&
&  0.23144E+00_JPRB, 0.23065E+00_JPRB, 0.23090E+00_JPRB, 0.22852E+00_JPRB, 0.22716E+00_JPRB, 0.22412E+00_JPRB, 0.22375E+00_JPRB ,&
&  0.22250E+00_JPRB, 0.22291E+00_JPRB, 0.22376E+00_JPRB, 0.22575E+00_JPRB, 0.22642E+00_JPRB, 0.22815E+00_JPRB, 0.22710E+00_JPRB ,&
&  0.22754E+00_JPRB, 0.22800E+00_JPRB, 0.22659E+00_JPRB, 0.22622E+00_JPRB, 0.22643E+00_JPRB, 0.22648E+00_JPRB, 0.22667E+00_JPRB ,&
&  0.22915E+00_JPRB, 0.23150E+00_JPRB, 0.23381E+00_JPRB, 0.23490E+00_JPRB, 0.23975E+00_JPRB, 0.24118E+00_JPRB, 0.24375E+00_JPRB ,&
&  0.24477E+00_JPRB, 0.24594E+00_JPRB, 0.24715E+00_JPRB, 0.24637E+00_JPRB, 0.24505E+00_JPRB, 0.24058E+00_JPRB, 0.23914E+00_JPRB ,&
&  0.22912E+00_JPRB, 0.22213E+00_JPRB, 0.21528E+00_JPRB, 0.20887E+00_JPRB, 0.20391E+00_JPRB, 0.20053E+00_JPRB, 0.19783E+00_JPRB ,&
&  0.19564E+00_JPRB, 0.19342E+00_JPRB, 0.19092E+00_JPRB, 0.19117E+00_JPRB, 0.19210E+00_JPRB, 0.19353E+00_JPRB, 0.19505E+00_JPRB ,&
&  0.19685E+00_JPRB, 0.19786E+00_JPRB, 0.19825E+00_JPRB, 0.19904E+00_JPRB, 0.19906E+00_JPRB, 0.19928E+00_JPRB, 0.19939E+00_JPRB ,&
&  0.20085E+00_JPRB, 0.20117E+00_JPRB, 0.20109E+00_JPRB, 0.20310E+00_JPRB, 0.20092E+00_JPRB, 0.19777E+00_JPRB, 0.19455E+00_JPRB ,&
&  0.19104E+00_JPRB, 0.18888E+00_JPRB, 0.18696E+00_JPRB, 0.18621E+00_JPRB, 0.18670E+00_JPRB, 0.18760E+00_JPRB, 0.18869E+00_JPRB ,&
&  0.18878E+00_JPRB, 0.18948E+00_JPRB, 0.19001E+00_JPRB, 0.19107E+00_JPRB, 0.19219E+00_JPRB, 0.19350E+00_JPRB, 0.19498E+00_JPRB ,&
&  0.19721E+00_JPRB, 0.19981E+00_JPRB, 0.20233E+00_JPRB, 0.20488E+00_JPRB, 0.20900E+00_JPRB, 0.21361E+00_JPRB, 0.22081E+00_JPRB ,&
&  0.22614E+00_JPRB, 0.23108E+00_JPRB, 0.22750E+00_JPRB, 0.23188E+00_JPRB, 0.23231E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.22461E+00_JPRB, 0.22527E+00_JPRB, 0.22288E+00_JPRB, 0.22295E+00_JPRB, 0.22353E+00_JPRB, 0.22377E+00_JPRB, 0.22422E+00_JPRB ,&
&  0.22149E+00_JPRB, 0.22089E+00_JPRB, 0.21860E+00_JPRB, 0.21691E+00_JPRB, 0.21457E+00_JPRB, 0.21231E+00_JPRB, 0.21068E+00_JPRB ,&
&  0.20877E+00_JPRB, 0.20837E+00_JPRB, 0.20755E+00_JPRB, 0.20896E+00_JPRB, 0.20919E+00_JPRB, 0.21134E+00_JPRB, 0.21300E+00_JPRB ,&
&  0.21377E+00_JPRB, 0.21498E+00_JPRB, 0.21324E+00_JPRB, 0.21249E+00_JPRB, 0.21415E+00_JPRB, 0.21547E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.22072E+00_JPRB, 0.22416E+00_JPRB, 0.22993E+00_JPRB, 0.23190E+00_JPRB, 0.23458E+00_JPRB, 0.23615E+00_JPRB, 0.23592E+00_JPRB ,&
&  0.23554E+00_JPRB, 0.23488E+00_JPRB, 0.23565E+00_JPRB, 0.23542E+00_JPRB, 0.22811E+00_JPRB, 0.22273E+00_JPRB, 0.21691E+00_JPRB ,&
&  0.21176E+00_JPRB, 0.20668E+00_JPRB, 0.20205E+00_JPRB, 0.19895E+00_JPRB, 0.19534E+00_JPRB, 0.19289E+00_JPRB, 0.19095E+00_JPRB ,&
&  0.18906E+00_JPRB, 0.18807E+00_JPRB, 0.18663E+00_JPRB, 0.18619E+00_JPRB, 0.18660E+00_JPRB, 0.18759E+00_JPRB, 0.18904E+00_JPRB ,&
&  0.19075E+00_JPRB, 0.19190E+00_JPRB, 0.19333E+00_JPRB, 0.19461E+00_JPRB, 0.19551E+00_JPRB, 0.19597E+00_JPRB, 0.19547E+00_JPRB ,&
&  0.19562E+00_JPRB, 0.19592E+00_JPRB, 0.19540E+00_JPRB, 0.19546E+00_JPRB, 0.19415E+00_JPRB, 0.19260E+00_JPRB, 0.19102E+00_JPRB ,&
&  0.18877E+00_JPRB, 0.18705E+00_JPRB, 0.18647E+00_JPRB, 0.18615E+00_JPRB, 0.18703E+00_JPRB, 0.18834E+00_JPRB, 0.18915E+00_JPRB ,&
&  0.18947E+00_JPRB, 0.18927E+00_JPRB, 0.18920E+00_JPRB, 0.19061E+00_JPRB, 0.19208E+00_JPRB, 0.19397E+00_JPRB, 0.19596E+00_JPRB ,&
&  0.19654E+00_JPRB, 0.19813E+00_JPRB, 0.19947E+00_JPRB, 0.20122E+00_JPRB, 0.20733E+00_JPRB, 0.21173E+00_JPRB, 0.22019E+00_JPRB ,&
&  0.22297E+00_JPRB, 0.22046E+00_JPRB, 0.21726E+00_JPRB, 0.22311E+00_JPRB, 0.22425E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.22304E+00_JPRB, 0.22564E+00_JPRB, 0.22389E+00_JPRB, 0.22345E+00_JPRB, 0.22246E+00_JPRB, 0.22200E+00_JPRB, 0.22041E+00_JPRB ,&
&  0.22016E+00_JPRB, 0.21854E+00_JPRB, 0.21838E+00_JPRB, 0.21673E+00_JPRB, 0.21508E+00_JPRB, 0.21266E+00_JPRB, 0.21024E+00_JPRB ,&
&  0.20792E+00_JPRB, 0.20592E+00_JPRB, 0.20465E+00_JPRB, 0.20433E+00_JPRB, 0.20334E+00_JPRB, 0.20378E+00_JPRB, 0.20677E+00_JPRB ,&
&  0.20747E+00_JPRB, 0.20883E+00_JPRB, 0.20876E+00_JPRB, 0.21000E+00_JPRB, 0.21164E+00_JPRB, 0.21180E+00_JPRB, 0.21398E+00_JPRB ,&
&  0.21428E+00_JPRB, 0.21594E+00_JPRB, 0.22324E+00_JPRB, 0.22697E+00_JPRB, 0.22881E+00_JPRB, 0.23192E+00_JPRB, 0.23276E+00_JPRB ,&
&  0.23088E+00_JPRB, 0.22484E+00_JPRB, 0.22780E+00_JPRB, 0.22432E+00_JPRB, 0.21729E+00_JPRB, 0.21469E+00_JPRB, 0.20910E+00_JPRB ,&
&  0.20645E+00_JPRB, 0.20270E+00_JPRB, 0.19994E+00_JPRB, 0.19781E+00_JPRB, 0.19425E+00_JPRB, 0.19208E+00_JPRB, 0.18957E+00_JPRB ,&
&  0.18732E+00_JPRB, 0.18648E+00_JPRB, 0.18496E+00_JPRB, 0.18495E+00_JPRB, 0.18527E+00_JPRB, 0.18589E+00_JPRB, 0.18690E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18927E+00_JPRB, 0.19029E+00_JPRB, 0.19113E+00_JPRB, 0.19184E+00_JPRB, 0.19222E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19202E+00_JPRB, 0.19185E+00_JPRB, 0.19125E+00_JPRB, 0.19104E+00_JPRB, 0.18995E+00_JPRB, 0.18953E+00_JPRB, 0.18907E+00_JPRB ,&
&  0.18888E+00_JPRB, 0.18891E+00_JPRB, 0.18879E+00_JPRB, 0.18901E+00_JPRB, 0.18930E+00_JPRB, 0.19009E+00_JPRB, 0.19068E+00_JPRB ,&
&  0.19115E+00_JPRB, 0.19219E+00_JPRB, 0.19281E+00_JPRB, 0.19369E+00_JPRB, 0.19493E+00_JPRB, 0.19542E+00_JPRB, 0.19687E+00_JPRB ,&
&  0.19808E+00_JPRB, 0.19953E+00_JPRB, 0.20224E+00_JPRB, 0.20490E+00_JPRB, 0.20878E+00_JPRB, 0.21334E+00_JPRB, 0.21450E+00_JPRB ,&
&  0.21518E+00_JPRB, 0.21462E+00_JPRB, 0.21175E+00_JPRB, 0.21759E+00_JPRB, 0.22035E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.21563E+00_JPRB, 0.21766E+00_JPRB, 0.21897E+00_JPRB, 0.21988E+00_JPRB, 0.22146E+00_JPRB, 0.22220E+00_JPRB, 0.22283E+00_JPRB ,&
&  0.22081E+00_JPRB, 0.22033E+00_JPRB, 0.21934E+00_JPRB, 0.21807E+00_JPRB, 0.21725E+00_JPRB, 0.21454E+00_JPRB, 0.21199E+00_JPRB ,&
&  0.20924E+00_JPRB, 0.20685E+00_JPRB, 0.20561E+00_JPRB, 0.20442E+00_JPRB, 0.20314E+00_JPRB, 0.20208E+00_JPRB, 0.20491E+00_JPRB ,&
&  0.20667E+00_JPRB, 0.20771E+00_JPRB, 0.20927E+00_JPRB, 0.20871E+00_JPRB, 0.21009E+00_JPRB, 0.21230E+00_JPRB, 0.21531E+00_JPRB ,&
&  0.21684E+00_JPRB, 0.21991E+00_JPRB, 0.22166E+00_JPRB, 0.22254E+00_JPRB, 0.22386E+00_JPRB, 0.22486E+00_JPRB, 0.22461E+00_JPRB ,&
&  0.22314E+00_JPRB, 0.22119E+00_JPRB, 0.21882E+00_JPRB, 0.21569E+00_JPRB, 0.21246E+00_JPRB, 0.20940E+00_JPRB, 0.20633E+00_JPRB ,&
&  0.20420E+00_JPRB, 0.20224E+00_JPRB, 0.20009E+00_JPRB, 0.19812E+00_JPRB, 0.19623E+00_JPRB, 0.19453E+00_JPRB, 0.19279E+00_JPRB ,&
&  0.19145E+00_JPRB, 0.19037E+00_JPRB, 0.18901E+00_JPRB, 0.18838E+00_JPRB, 0.18775E+00_JPRB, 0.18753E+00_JPRB, 0.18745E+00_JPRB ,&
&  0.18803E+00_JPRB, 0.18870E+00_JPRB, 0.18934E+00_JPRB, 0.18969E+00_JPRB, 0.19004E+00_JPRB, 0.19019E+00_JPRB, 0.19051E+00_JPRB ,&
&  0.19080E+00_JPRB, 0.19115E+00_JPRB, 0.19063E+00_JPRB, 0.19017E+00_JPRB, 0.18957E+00_JPRB, 0.18888E+00_JPRB, 0.18867E+00_JPRB ,&
&  0.18901E+00_JPRB, 0.18928E+00_JPRB, 0.18902E+00_JPRB, 0.18934E+00_JPRB, 0.18992E+00_JPRB, 0.19051E+00_JPRB, 0.19207E+00_JPRB ,&
&  0.19291E+00_JPRB, 0.19370E+00_JPRB, 0.19445E+00_JPRB, 0.19504E+00_JPRB, 0.19597E+00_JPRB, 0.19670E+00_JPRB, 0.19763E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19980E+00_JPRB, 0.20124E+00_JPRB, 0.20312E+00_JPRB, 0.20776E+00_JPRB, 0.21053E+00_JPRB, 0.21269E+00_JPRB ,&
&  0.21323E+00_JPRB, 0.20788E+00_JPRB, 0.21502E+00_JPRB, 0.21339E+00_JPRB, 0.21277E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.20920E+00_JPRB, 0.21096E+00_JPRB, 0.21221E+00_JPRB, 0.21399E+00_JPRB, 0.20787E+00_JPRB, 0.21462E+00_JPRB, 0.21103E+00_JPRB ,&
&  0.20735E+00_JPRB, 0.20242E+00_JPRB, 0.20297E+00_JPRB, 0.20242E+00_JPRB, 0.20146E+00_JPRB, 0.20123E+00_JPRB, 0.20149E+00_JPRB ,&
&  0.20190E+00_JPRB, 0.20199E+00_JPRB, 0.20286E+00_JPRB, 0.20353E+00_JPRB, 0.20407E+00_JPRB, 0.20451E+00_JPRB, 0.20509E+00_JPRB ,&
&  0.20591E+00_JPRB, 0.20720E+00_JPRB, 0.20851E+00_JPRB, 0.20614E+00_JPRB, 0.20583E+00_JPRB, 0.20603E+00_JPRB, 0.20665E+00_JPRB ,&
&  0.20690E+00_JPRB, 0.20809E+00_JPRB, 0.20906E+00_JPRB, 0.20981E+00_JPRB, 0.21209E+00_JPRB, 0.21232E+00_JPRB, 0.21222E+00_JPRB ,&
&  0.21181E+00_JPRB, 0.21087E+00_JPRB, 0.20956E+00_JPRB, 0.20832E+00_JPRB, 0.20704E+00_JPRB, 0.20535E+00_JPRB, 0.20373E+00_JPRB ,&
&  0.20241E+00_JPRB, 0.20134E+00_JPRB, 0.19993E+00_JPRB, 0.19856E+00_JPRB, 0.19732E+00_JPRB, 0.19615E+00_JPRB, 0.19515E+00_JPRB ,&
&  0.19443E+00_JPRB, 0.19381E+00_JPRB, 0.19315E+00_JPRB, 0.19266E+00_JPRB, 0.19211E+00_JPRB, 0.19160E+00_JPRB, 0.19110E+00_JPRB ,&
&  0.19130E+00_JPRB, 0.19142E+00_JPRB, 0.19154E+00_JPRB, 0.19168E+00_JPRB, 0.19221E+00_JPRB, 0.19231E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.19253E+00_JPRB, 0.19242E+00_JPRB, 0.19226E+00_JPRB, 0.19198E+00_JPRB, 0.19167E+00_JPRB, 0.19129E+00_JPRB, 0.19112E+00_JPRB ,&
&  0.19110E+00_JPRB, 0.19123E+00_JPRB, 0.19164E+00_JPRB, 0.19191E+00_JPRB, 0.19219E+00_JPRB, 0.19243E+00_JPRB, 0.19300E+00_JPRB ,&
&  0.19355E+00_JPRB, 0.19410E+00_JPRB, 0.19461E+00_JPRB, 0.19444E+00_JPRB, 0.19472E+00_JPRB, 0.19498E+00_JPRB, 0.19523E+00_JPRB ,&
&  0.19536E+00_JPRB, 0.19589E+00_JPRB, 0.19676E+00_JPRB, 0.19799E+00_JPRB, 0.20030E+00_JPRB, 0.20217E+00_JPRB, 0.20391E+00_JPRB ,&
&  0.20575E+00_JPRB, 0.20958E+00_JPRB, 0.21086E+00_JPRB, 0.21107E+00_JPRB, 0.21072E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.19908E+00_JPRB, 0.20005E+00_JPRB, 0.20071E+00_JPRB, 0.20121E+00_JPRB, 0.20091E+00_JPRB, 0.20092E+00_JPRB, 0.20091E+00_JPRB ,&
&  0.20079E+00_JPRB, 0.20067E+00_JPRB, 0.20097E+00_JPRB, 0.20123E+00_JPRB, 0.20155E+00_JPRB, 0.20189E+00_JPRB, 0.20235E+00_JPRB ,&
&  0.20252E+00_JPRB, 0.20293E+00_JPRB, 0.20139E+00_JPRB, 0.20012E+00_JPRB, 0.19926E+00_JPRB, 0.19872E+00_JPRB, 0.19842E+00_JPRB ,&
&  0.19820E+00_JPRB, 0.19808E+00_JPRB, 0.19793E+00_JPRB, 0.19908E+00_JPRB, 0.19982E+00_JPRB, 0.20046E+00_JPRB, 0.20096E+00_JPRB ,&
&  0.20115E+00_JPRB, 0.20145E+00_JPRB, 0.20171E+00_JPRB, 0.20198E+00_JPRB, 0.20230E+00_JPRB, 0.20231E+00_JPRB, 0.20221E+00_JPRB ,&
&  0.20202E+00_JPRB, 0.20167E+00_JPRB, 0.20127E+00_JPRB, 0.20088E+00_JPRB, 0.20056E+00_JPRB, 0.20017E+00_JPRB, 0.19975E+00_JPRB ,&
&  0.19935E+00_JPRB, 0.19888E+00_JPRB, 0.19844E+00_JPRB, 0.19801E+00_JPRB, 0.19760E+00_JPRB, 0.19724E+00_JPRB, 0.19686E+00_JPRB ,&
&  0.19658E+00_JPRB, 0.19634E+00_JPRB, 0.19608E+00_JPRB, 0.19589E+00_JPRB, 0.19557E+00_JPRB, 0.19545E+00_JPRB, 0.19518E+00_JPRB ,&
&  0.19508E+00_JPRB, 0.19497E+00_JPRB, 0.19490E+00_JPRB, 0.19481E+00_JPRB, 0.19480E+00_JPRB, 0.19478E+00_JPRB, 0.19477E+00_JPRB ,&
&  0.19475E+00_JPRB, 0.19492E+00_JPRB, 0.19478E+00_JPRB, 0.19460E+00_JPRB, 0.19440E+00_JPRB, 0.19421E+00_JPRB, 0.19403E+00_JPRB ,&
&  0.19386E+00_JPRB, 0.19369E+00_JPRB, 0.19358E+00_JPRB, 0.19362E+00_JPRB, 0.19368E+00_JPRB, 0.19377E+00_JPRB, 0.19387E+00_JPRB ,&
&  0.19397E+00_JPRB, 0.19410E+00_JPRB, 0.19422E+00_JPRB, 0.19416E+00_JPRB, 0.19417E+00_JPRB, 0.19419E+00_JPRB, 0.19421E+00_JPRB ,&
&  0.19423E+00_JPRB, 0.19428E+00_JPRB, 0.19439E+00_JPRB, 0.19453E+00_JPRB, 0.19487E+00_JPRB, 0.19499E+00_JPRB, 0.19518E+00_JPRB ,&
&  0.19543E+00_JPRB, 0.19590E+00_JPRB, 0.19631E+00_JPRB, 0.19675E+00_JPRB, 0.19726E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB, 0.19601E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB, 0.85784E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.85639E+00_JPRB, 0.85527E+00_JPRB, 0.85427E+00_JPRB, 0.85331E+00_JPRB, 0.85233E+00_JPRB, 0.85162E+00_JPRB, 0.85108E+00_JPRB ,&
&  0.85050E+00_JPRB, 0.84998E+00_JPRB, 0.84938E+00_JPRB, 0.84892E+00_JPRB, 0.84861E+00_JPRB, 0.84819E+00_JPRB, 0.84778E+00_JPRB ,&
&  0.84723E+00_JPRB, 0.84671E+00_JPRB, 0.84635E+00_JPRB, 0.84555E+00_JPRB, 0.84483E+00_JPRB, 0.84416E+00_JPRB, 0.84295E+00_JPRB ,&
&  0.84131E+00_JPRB, 0.83969E+00_JPRB, 0.83790E+00_JPRB, 0.83634E+00_JPRB, 0.83513E+00_JPRB, 0.83344E+00_JPRB, 0.83147E+00_JPRB ,&
&  0.82975E+00_JPRB, 0.82855E+00_JPRB, 0.82771E+00_JPRB, 0.82711E+00_JPRB, 0.82671E+00_JPRB, 0.82649E+00_JPRB, 0.82622E+00_JPRB ,&
&  0.82611E+00_JPRB, 0.82642E+00_JPRB, 0.82721E+00_JPRB, 0.82834E+00_JPRB, 0.82976E+00_JPRB, 0.83150E+00_JPRB, 0.83407E+00_JPRB ,&
&  0.83685E+00_JPRB, 0.83972E+00_JPRB, 0.84272E+00_JPRB, 0.84544E+00_JPRB, 0.84792E+00_JPRB, 0.85010E+00_JPRB, 0.85179E+00_JPRB ,&
&  0.85363E+00_JPRB, 0.85552E+00_JPRB, 0.85747E+00_JPRB, 0.85897E+00_JPRB, 0.86026E+00_JPRB, 0.86146E+00_JPRB, 0.86278E+00_JPRB ,&
&  0.86352E+00_JPRB, 0.86405E+00_JPRB, 0.86471E+00_JPRB, 0.86556E+00_JPRB, 0.86618E+00_JPRB, 0.86664E+00_JPRB, 0.86715E+00_JPRB ,&
&  0.86788E+00_JPRB, 0.86846E+00_JPRB, 0.86879E+00_JPRB, 0.86914E+00_JPRB, 0.86972E+00_JPRB, 0.87053E+00_JPRB, 0.87139E+00_JPRB ,&
&  0.87216E+00_JPRB, 0.87290E+00_JPRB, 0.87345E+00_JPRB, 0.87375E+00_JPRB, 0.87373E+00_JPRB, 0.87331E+00_JPRB, 0.87216E+00_JPRB ,&
&  0.87078E+00_JPRB, 0.86923E+00_JPRB, 0.86795E+00_JPRB, 0.86718E+00_JPRB, 0.86636E+00_JPRB, 0.86569E+00_JPRB, 0.86528E+00_JPRB ,&
&  0.86525E+00_JPRB, 0.86530E+00_JPRB, 0.86533E+00_JPRB, 0.86552E+00_JPRB, 0.86554E+00_JPRB, 0.86522E+00_JPRB, 0.86470E+00_JPRB ,&
&  0.86393E+00_JPRB, 0.86281E+00_JPRB, 0.86134E+00_JPRB, 0.85967E+00_JPRB, 0.85793E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.84695E+00_JPRB, 0.84404E+00_JPRB, 0.84194E+00_JPRB, 0.84050E+00_JPRB, 0.83934E+00_JPRB, 0.83822E+00_JPRB, 0.83707E+00_JPRB ,&
&  0.83595E+00_JPRB, 0.83608E+00_JPRB, 0.83576E+00_JPRB, 0.83559E+00_JPRB, 0.83534E+00_JPRB, 0.83590E+00_JPRB, 0.83609E+00_JPRB ,&
&  0.83577E+00_JPRB, 0.83427E+00_JPRB, 0.83314E+00_JPRB, 0.83118E+00_JPRB, 0.83099E+00_JPRB, 0.83073E+00_JPRB, 0.83192E+00_JPRB ,&
&  0.83092E+00_JPRB, 0.82952E+00_JPRB, 0.82706E+00_JPRB, 0.82039E+00_JPRB, 0.81455E+00_JPRB, 0.80884E+00_JPRB, 0.80388E+00_JPRB ,&
&  0.79918E+00_JPRB, 0.79559E+00_JPRB, 0.79390E+00_JPRB, 0.79340E+00_JPRB, 0.79716E+00_JPRB, 0.79826E+00_JPRB, 0.79863E+00_JPRB ,&
&  0.79755E+00_JPRB, 0.79264E+00_JPRB, 0.79378E+00_JPRB, 0.79531E+00_JPRB, 0.79762E+00_JPRB, 0.80305E+00_JPRB, 0.80789E+00_JPRB ,&
&  0.81350E+00_JPRB, 0.81983E+00_JPRB, 0.82994E+00_JPRB, 0.83719E+00_JPRB, 0.84430E+00_JPRB, 0.85032E+00_JPRB, 0.85345E+00_JPRB ,&
&  0.85661E+00_JPRB, 0.85942E+00_JPRB, 0.86229E+00_JPRB, 0.86455E+00_JPRB, 0.86626E+00_JPRB, 0.86816E+00_JPRB, 0.87040E+00_JPRB ,&
&  0.87185E+00_JPRB, 0.87299E+00_JPRB, 0.87425E+00_JPRB, 0.87554E+00_JPRB, 0.87636E+00_JPRB, 0.87725E+00_JPRB, 0.87838E+00_JPRB ,&
&  0.87955E+00_JPRB, 0.87942E+00_JPRB, 0.87929E+00_JPRB, 0.88007E+00_JPRB, 0.88094E+00_JPRB, 0.88184E+00_JPRB, 0.88189E+00_JPRB ,&
&  0.88223E+00_JPRB, 0.88218E+00_JPRB, 0.88251E+00_JPRB, 0.88313E+00_JPRB, 0.88323E+00_JPRB, 0.88332E+00_JPRB, 0.88256E+00_JPRB ,&
&  0.88139E+00_JPRB, 0.87942E+00_JPRB, 0.87768E+00_JPRB, 0.87695E+00_JPRB, 0.87616E+00_JPRB, 0.87631E+00_JPRB, 0.87688E+00_JPRB ,&
&  0.87807E+00_JPRB, 0.87728E+00_JPRB, 0.87602E+00_JPRB, 0.87402E+00_JPRB, 0.87193E+00_JPRB, 0.86946E+00_JPRB, 0.86707E+00_JPRB ,&
&  0.86462E+00_JPRB, 0.86176E+00_JPRB, 0.85817E+00_JPRB, 0.85431E+00_JPRB, 0.85051E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.84943E+00_JPRB, 0.84437E+00_JPRB, 0.84068E+00_JPRB, 0.83822E+00_JPRB, 0.83594E+00_JPRB, 0.83382E+00_JPRB, 0.83224E+00_JPRB ,&
&  0.83063E+00_JPRB, 0.82793E+00_JPRB, 0.82649E+00_JPRB, 0.82548E+00_JPRB, 0.82400E+00_JPRB, 0.82462E+00_JPRB, 0.82481E+00_JPRB ,&
&  0.82570E+00_JPRB, 0.82530E+00_JPRB, 0.82801E+00_JPRB, 0.82676E+00_JPRB, 0.82572E+00_JPRB, 0.82426E+00_JPRB, 0.81968E+00_JPRB ,&
&  0.81809E+00_JPRB, 0.81613E+00_JPRB, 0.81042E+00_JPRB, 0.80346E+00_JPRB, 0.79918E+00_JPRB, 0.77835E+00_JPRB, 0.77930E+00_JPRB ,&
&  0.78841E+00_JPRB, 0.77390E+00_JPRB, 0.76609E+00_JPRB, 0.75821E+00_JPRB, 0.74685E+00_JPRB, 0.74662E+00_JPRB, 0.75166E+00_JPRB ,&
&  0.74533E+00_JPRB, 0.76550E+00_JPRB, 0.76838E+00_JPRB, 0.77020E+00_JPRB, 0.77250E+00_JPRB, 0.76927E+00_JPRB, 0.77519E+00_JPRB ,&
&  0.79790E+00_JPRB, 0.80221E+00_JPRB, 0.81261E+00_JPRB, 0.82183E+00_JPRB, 0.83331E+00_JPRB, 0.84376E+00_JPRB, 0.85524E+00_JPRB ,&
&  0.86383E+00_JPRB, 0.86995E+00_JPRB, 0.87434E+00_JPRB, 0.87525E+00_JPRB, 0.87617E+00_JPRB, 0.87628E+00_JPRB, 0.87778E+00_JPRB ,&
&  0.87792E+00_JPRB, 0.87941E+00_JPRB, 0.88196E+00_JPRB, 0.88485E+00_JPRB, 0.88871E+00_JPRB, 0.89095E+00_JPRB, 0.89324E+00_JPRB ,&
&  0.89470E+00_JPRB, 0.89491E+00_JPRB, 0.89454E+00_JPRB, 0.89439E+00_JPRB, 0.89306E+00_JPRB, 0.89234E+00_JPRB, 0.89181E+00_JPRB ,&
&  0.89247E+00_JPRB, 0.89250E+00_JPRB, 0.89349E+00_JPRB, 0.89438E+00_JPRB, 0.89616E+00_JPRB, 0.89917E+00_JPRB, 0.90151E+00_JPRB ,&
&  0.90494E+00_JPRB, 0.90666E+00_JPRB, 0.90762E+00_JPRB, 0.90779E+00_JPRB, 0.90770E+00_JPRB, 0.90866E+00_JPRB, 0.90930E+00_JPRB ,&
&  0.91067E+00_JPRB, 0.90704E+00_JPRB, 0.90238E+00_JPRB, 0.89725E+00_JPRB, 0.89276E+00_JPRB, 0.88870E+00_JPRB, 0.88439E+00_JPRB ,&
&  0.88048E+00_JPRB, 0.87518E+00_JPRB, 0.86916E+00_JPRB, 0.86223E+00_JPRB, 0.85536E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.86818E+00_JPRB, 0.86034E+00_JPRB, 0.85326E+00_JPRB, 0.84819E+00_JPRB, 0.84330E+00_JPRB, 0.83926E+00_JPRB, 0.83568E+00_JPRB ,&
&  0.83254E+00_JPRB, 0.82976E+00_JPRB, 0.82795E+00_JPRB, 0.82607E+00_JPRB, 0.82475E+00_JPRB, 0.82315E+00_JPRB, 0.82235E+00_JPRB ,&
&  0.82442E+00_JPRB, 0.82440E+00_JPRB, 0.82732E+00_JPRB, 0.82564E+00_JPRB, 0.82423E+00_JPRB, 0.82001E+00_JPRB, 0.80873E+00_JPRB ,&
&  0.80289E+00_JPRB, 0.79749E+00_JPRB, 0.78036E+00_JPRB, 0.79449E+00_JPRB, 0.78676E+00_JPRB, 0.78154E+00_JPRB, 0.77373E+00_JPRB ,&
&  0.75689E+00_JPRB, 0.74877E+00_JPRB, 0.73875E+00_JPRB, 0.73816E+00_JPRB, 0.74462E+00_JPRB, 0.74193E+00_JPRB, 0.74706E+00_JPRB ,&
&  0.74241E+00_JPRB, 0.74094E+00_JPRB, 0.74218E+00_JPRB, 0.74241E+00_JPRB, 0.74844E+00_JPRB, 0.74933E+00_JPRB, 0.75408E+00_JPRB ,&
&  0.76443E+00_JPRB, 0.77918E+00_JPRB, 0.80079E+00_JPRB, 0.80821E+00_JPRB, 0.82619E+00_JPRB, 0.84081E+00_JPRB, 0.85434E+00_JPRB ,&
&  0.86329E+00_JPRB, 0.86877E+00_JPRB, 0.87285E+00_JPRB, 0.87458E+00_JPRB, 0.87461E+00_JPRB, 0.87116E+00_JPRB, 0.87135E+00_JPRB ,&
&  0.87259E+00_JPRB, 0.87499E+00_JPRB, 0.87806E+00_JPRB, 0.88067E+00_JPRB, 0.88454E+00_JPRB, 0.88782E+00_JPRB, 0.89248E+00_JPRB ,&
&  0.89646E+00_JPRB, 0.90050E+00_JPRB, 0.90155E+00_JPRB, 0.90337E+00_JPRB, 0.90473E+00_JPRB, 0.90590E+00_JPRB, 0.90721E+00_JPRB ,&
&  0.90882E+00_JPRB, 0.91139E+00_JPRB, 0.91595E+00_JPRB, 0.91362E+00_JPRB, 0.91704E+00_JPRB, 0.93100E+00_JPRB, 0.93500E+00_JPRB ,&
&  0.94022E+00_JPRB, 0.94142E+00_JPRB, 0.93971E+00_JPRB, 0.93780E+00_JPRB, 0.93581E+00_JPRB, 0.93566E+00_JPRB, 0.93568E+00_JPRB ,&
&  0.93549E+00_JPRB, 0.92976E+00_JPRB, 0.92362E+00_JPRB, 0.91825E+00_JPRB, 0.91252E+00_JPRB, 0.90818E+00_JPRB, 0.90370E+00_JPRB ,&
&  0.90057E+00_JPRB, 0.89778E+00_JPRB, 0.89404E+00_JPRB, 0.88719E+00_JPRB, 0.87837E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.89068E+00_JPRB, 0.88125E+00_JPRB, 0.87361E+00_JPRB, 0.86715E+00_JPRB, 0.86202E+00_JPRB, 0.85764E+00_JPRB, 0.85341E+00_JPRB ,&
&  0.84951E+00_JPRB, 0.84491E+00_JPRB, 0.84027E+00_JPRB, 0.83672E+00_JPRB, 0.83383E+00_JPRB, 0.83123E+00_JPRB, 0.82843E+00_JPRB ,&
&  0.82723E+00_JPRB, 0.82561E+00_JPRB, 0.82727E+00_JPRB, 0.82480E+00_JPRB, 0.82665E+00_JPRB, 0.82232E+00_JPRB, 0.80976E+00_JPRB ,&
&  0.79722E+00_JPRB, 0.78320E+00_JPRB, 0.77011E+00_JPRB, 0.78272E+00_JPRB, 0.78053E+00_JPRB, 0.77972E+00_JPRB, 0.76169E+00_JPRB ,&
&  0.74521E+00_JPRB, 0.73082E+00_JPRB, 0.75575E+00_JPRB, 0.74658E+00_JPRB, 0.74172E+00_JPRB, 0.73316E+00_JPRB, 0.71347E+00_JPRB ,&
&  0.71030E+00_JPRB, 0.69833E+00_JPRB, 0.69756E+00_JPRB, 0.70265E+00_JPRB, 0.70965E+00_JPRB, 0.71996E+00_JPRB, 0.73001E+00_JPRB ,&
&  0.72935E+00_JPRB, 0.76596E+00_JPRB, 0.79127E+00_JPRB, 0.81020E+00_JPRB, 0.82897E+00_JPRB, 0.84426E+00_JPRB, 0.85472E+00_JPRB ,&
&  0.86453E+00_JPRB, 0.87187E+00_JPRB, 0.87913E+00_JPRB, 0.88162E+00_JPRB, 0.89308E+00_JPRB, 0.89609E+00_JPRB, 0.90046E+00_JPRB ,&
&  0.90552E+00_JPRB, 0.90561E+00_JPRB, 0.90414E+00_JPRB, 0.90007E+00_JPRB, 0.89801E+00_JPRB, 0.89506E+00_JPRB, 0.89612E+00_JPRB ,&
&  0.89659E+00_JPRB, 0.89818E+00_JPRB, 0.90376E+00_JPRB, 0.90881E+00_JPRB, 0.91348E+00_JPRB, 0.91779E+00_JPRB, 0.92270E+00_JPRB ,&
&  0.92750E+00_JPRB, 0.93663E+00_JPRB, 0.94571E+00_JPRB, 0.96189E+00_JPRB, 0.97631E+00_JPRB, 0.98426E+00_JPRB, 0.99044E+00_JPRB ,&
&  0.99743E+00_JPRB, 0.99133E+00_JPRB, 0.98538E+00_JPRB, 0.98167E+00_JPRB, 0.97971E+00_JPRB, 0.97912E+00_JPRB, 0.97982E+00_JPRB ,&
&  0.97891E+00_JPRB, 0.97347E+00_JPRB, 0.96684E+00_JPRB, 0.95904E+00_JPRB, 0.95139E+00_JPRB, 0.94520E+00_JPRB, 0.93994E+00_JPRB ,&
&  0.93506E+00_JPRB, 0.92971E+00_JPRB, 0.92120E+00_JPRB, 0.91091E+00_JPRB, 0.90051E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.92246E+00_JPRB, 0.91208E+00_JPRB, 0.90394E+00_JPRB, 0.89689E+00_JPRB, 0.89172E+00_JPRB, 0.88737E+00_JPRB, 0.88367E+00_JPRB ,&
&  0.88073E+00_JPRB, 0.87509E+00_JPRB, 0.86848E+00_JPRB, 0.86035E+00_JPRB, 0.85450E+00_JPRB, 0.84922E+00_JPRB, 0.84342E+00_JPRB ,&
&  0.83556E+00_JPRB, 0.82718E+00_JPRB, 0.81849E+00_JPRB, 0.81163E+00_JPRB, 0.81384E+00_JPRB, 0.81045E+00_JPRB, 0.80531E+00_JPRB ,&
&  0.79379E+00_JPRB, 0.77903E+00_JPRB, 0.76770E+00_JPRB, 0.77230E+00_JPRB, 0.77469E+00_JPRB, 0.76250E+00_JPRB, 0.75924E+00_JPRB ,&
&  0.77281E+00_JPRB, 0.77419E+00_JPRB, 0.76551E+00_JPRB, 0.76114E+00_JPRB, 0.72942E+00_JPRB, 0.68683E+00_JPRB, 0.71385E+00_JPRB ,&
&  0.71389E+00_JPRB, 0.69334E+00_JPRB, 0.69002E+00_JPRB, 0.68779E+00_JPRB, 0.69643E+00_JPRB, 0.71481E+00_JPRB, 0.73168E+00_JPRB ,&
&  0.73549E+00_JPRB, 0.79795E+00_JPRB, 0.82596E+00_JPRB, 0.84608E+00_JPRB, 0.86000E+00_JPRB, 0.87105E+00_JPRB, 0.87985E+00_JPRB ,&
&  0.88912E+00_JPRB, 0.90331E+00_JPRB, 0.92242E+00_JPRB, 0.94130E+00_JPRB, 0.96031E+00_JPRB, 0.97382E+00_JPRB, 0.98804E+00_JPRB ,&
&  0.10050E+01_JPRB, 0.10068E+01_JPRB, 0.99089E+00_JPRB, 0.97343E+00_JPRB, 0.95740E+00_JPRB, 0.94280E+00_JPRB, 0.92996E+00_JPRB ,&
&  0.92460E+00_JPRB, 0.92076E+00_JPRB, 0.92713E+00_JPRB, 0.92821E+00_JPRB, 0.93200E+00_JPRB, 0.93730E+00_JPRB, 0.95142E+00_JPRB ,&
&  0.96098E+00_JPRB, 0.97557E+00_JPRB, 0.99301E+00_JPRB, 0.10164E+01_JPRB, 0.10357E+01_JPRB, 0.10460E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10617E+01_JPRB, 0.10630E+01_JPRB, 0.10691E+01_JPRB, 0.10798E+01_JPRB, 0.10851E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.10939E+01_JPRB, 0.10932E+01_JPRB, 0.10874E+01_JPRB, 0.10768E+01_JPRB, 0.10618E+01_JPRB, 0.10447E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.97794E+00_JPRB, 0.95748E+00_JPRB, 0.94369E+00_JPRB, 0.93206E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.98081E+00_JPRB, 0.96071E+00_JPRB, 0.94608E+00_JPRB, 0.93702E+00_JPRB, 0.92969E+00_JPRB, 0.92391E+00_JPRB, 0.91839E+00_JPRB ,&
&  0.91281E+00_JPRB, 0.90702E+00_JPRB, 0.89911E+00_JPRB, 0.89074E+00_JPRB, 0.88382E+00_JPRB, 0.87685E+00_JPRB, 0.87114E+00_JPRB ,&
&  0.86162E+00_JPRB, 0.84003E+00_JPRB, 0.83636E+00_JPRB, 0.82531E+00_JPRB, 0.80709E+00_JPRB, 0.79661E+00_JPRB, 0.78853E+00_JPRB ,&
&  0.77910E+00_JPRB, 0.77561E+00_JPRB, 0.77681E+00_JPRB, 0.77107E+00_JPRB, 0.76846E+00_JPRB, 0.78051E+00_JPRB, 0.78265E+00_JPRB ,&
&  0.75236E+00_JPRB, 0.75199E+00_JPRB, 0.79124E+00_JPRB, 0.79201E+00_JPRB, 0.74684E+00_JPRB, 0.73348E+00_JPRB, 0.74124E+00_JPRB ,&
&  0.73893E+00_JPRB, 0.70738E+00_JPRB, 0.70231E+00_JPRB, 0.70081E+00_JPRB, 0.71455E+00_JPRB, 0.72829E+00_JPRB, 0.76184E+00_JPRB ,&
&  0.78786E+00_JPRB, 0.84575E+00_JPRB, 0.87469E+00_JPRB, 0.89579E+00_JPRB, 0.91177E+00_JPRB, 0.92754E+00_JPRB, 0.94659E+00_JPRB ,&
&  0.97439E+00_JPRB, 0.10124E+01_JPRB, 0.10603E+01_JPRB, 0.11027E+01_JPRB, 0.11430E+01_JPRB, 0.11625E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.11894E+01_JPRB, 0.11872E+01_JPRB, 0.11465E+01_JPRB, 0.11117E+01_JPRB, 0.10733E+01_JPRB, 0.10370E+01_JPRB, 0.10001E+01_JPRB ,&
&  0.98940E+00_JPRB, 0.99663E+00_JPRB, 0.98609E+00_JPRB, 0.97752E+00_JPRB, 0.97438E+00_JPRB, 0.97555E+00_JPRB, 0.98308E+00_JPRB ,&
&  0.98859E+00_JPRB, 0.10283E+01_JPRB, 0.11489E+01_JPRB, 0.11824E+01_JPRB, 0.11981E+01_JPRB, 0.12029E+01_JPRB, 0.12041E+01_JPRB ,&
&  0.12122E+01_JPRB, 0.12268E+01_JPRB, 0.12448E+01_JPRB, 0.12540E+01_JPRB, 0.12640E+01_JPRB, 0.12557E+01_JPRB, 0.12642E+01_JPRB ,&
&  0.12564E+01_JPRB, 0.12460E+01_JPRB, 0.12332E+01_JPRB, 0.12232E+01_JPRB, 0.12104E+01_JPRB, 0.11980E+01_JPRB, 0.11777E+01_JPRB ,&
&  0.11495E+01_JPRB, 0.11163E+01_JPRB, 0.10821E+01_JPRB, 0.10456E+01_JPRB, 0.10102E+01_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.10680E+01_JPRB, 0.10227E+01_JPRB, 0.99142E+00_JPRB, 0.97701E+00_JPRB, 0.96740E+00_JPRB, 0.94799E+00_JPRB, 0.93294E+00_JPRB ,&
&  0.92177E+00_JPRB, 0.91842E+00_JPRB, 0.90870E+00_JPRB, 0.89749E+00_JPRB, 0.89019E+00_JPRB, 0.88374E+00_JPRB, 0.88027E+00_JPRB ,&
&  0.87599E+00_JPRB, 0.86572E+00_JPRB, 0.84444E+00_JPRB, 0.82186E+00_JPRB, 0.80612E+00_JPRB, 0.79300E+00_JPRB, 0.78187E+00_JPRB ,&
&  0.77236E+00_JPRB, 0.77084E+00_JPRB, 0.78215E+00_JPRB, 0.78219E+00_JPRB, 0.78156E+00_JPRB, 0.77589E+00_JPRB, 0.77092E+00_JPRB ,&
&  0.77454E+00_JPRB, 0.78993E+00_JPRB, 0.76810E+00_JPRB, 0.77426E+00_JPRB, 0.79168E+00_JPRB, 0.77253E+00_JPRB, 0.75715E+00_JPRB ,&
&  0.74379E+00_JPRB, 0.73209E+00_JPRB, 0.72632E+00_JPRB, 0.72677E+00_JPRB, 0.74965E+00_JPRB, 0.77662E+00_JPRB, 0.82045E+00_JPRB ,&
&  0.86871E+00_JPRB, 0.90368E+00_JPRB, 0.92657E+00_JPRB, 0.94784E+00_JPRB, 0.96831E+00_JPRB, 0.99277E+00_JPRB, 0.10372E+01_JPRB ,&
&  0.11102E+01_JPRB, 0.11883E+01_JPRB, 0.12628E+01_JPRB, 0.13146E+01_JPRB, 0.13659E+01_JPRB, 0.13346E+01_JPRB, 0.13374E+01_JPRB ,&
&  0.14758E+01_JPRB, 0.15225E+01_JPRB, 0.14428E+01_JPRB, 0.13714E+01_JPRB, 0.12918E+01_JPRB, 0.12209E+01_JPRB, 0.11641E+01_JPRB ,&
&  0.11256E+01_JPRB, 0.11019E+01_JPRB, 0.10824E+01_JPRB, 0.10568E+01_JPRB, 0.10409E+01_JPRB, 0.10478E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.13246E+01_JPRB, 0.16774E+01_JPRB, 0.14809E+01_JPRB, 0.14102E+01_JPRB, 0.13992E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.14587E+01_JPRB, 0.14709E+01_JPRB, 0.14463E+01_JPRB, 0.14023E+01_JPRB, 0.13489E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.12554E+01_JPRB, 0.12375E+01_JPRB, 0.12329E+01_JPRB, 0.12203E+01_JPRB, 0.12066E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.11742E+01_JPRB, 0.11618E+01_JPRB, 0.11553E+01_JPRB, 0.11403E+01_JPRB, 0.11159E+01_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.10961E+01_JPRB, 0.11002E+01_JPRB, 0.10841E+01_JPRB, 0.10626E+01_JPRB, 0.10386E+01_JPRB, 0.98737E+00_JPRB, 0.94733E+00_JPRB ,&
&  0.92579E+00_JPRB, 0.91732E+00_JPRB, 0.91520E+00_JPRB, 0.91031E+00_JPRB, 0.90562E+00_JPRB, 0.89890E+00_JPRB, 0.88763E+00_JPRB ,&
&  0.87288E+00_JPRB, 0.86225E+00_JPRB, 0.85327E+00_JPRB, 0.83840E+00_JPRB, 0.81864E+00_JPRB, 0.79760E+00_JPRB, 0.78070E+00_JPRB ,&
&  0.76866E+00_JPRB, 0.76407E+00_JPRB, 0.77377E+00_JPRB, 0.78417E+00_JPRB, 0.79450E+00_JPRB, 0.79778E+00_JPRB, 0.79366E+00_JPRB ,&
&  0.77706E+00_JPRB, 0.77378E+00_JPRB, 0.78659E+00_JPRB, 0.78789E+00_JPRB, 0.79584E+00_JPRB, 0.79746E+00_JPRB, 0.78364E+00_JPRB ,&
&  0.77135E+00_JPRB, 0.76236E+00_JPRB, 0.76714E+00_JPRB, 0.78414E+00_JPRB, 0.83420E+00_JPRB, 0.88873E+00_JPRB, 0.91576E+00_JPRB ,&
&  0.94224E+00_JPRB, 0.95995E+00_JPRB, 0.97849E+00_JPRB, 0.10046E+01_JPRB, 0.10451E+01_JPRB, 0.11019E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.13467E+01_JPRB, 0.14442E+01_JPRB, 0.15346E+01_JPRB, 0.14464E+01_JPRB, 0.14140E+01_JPRB, 0.14538E+01_JPRB, 0.15171E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.16199E+01_JPRB, 0.16085E+01_JPRB, 0.16165E+01_JPRB, 0.15388E+01_JPRB, 0.14426E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.11711E+01_JPRB, 0.11479E+01_JPRB, 0.11293E+01_JPRB, 0.11438E+01_JPRB, 0.11826E+01_JPRB, 0.12505E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.14895E+01_JPRB, 0.15595E+01_JPRB, 0.15169E+01_JPRB, 0.15472E+01_JPRB, 0.16125E+01_JPRB, 0.16609E+01_JPRB ,&
&  0.16592E+01_JPRB, 0.16518E+01_JPRB, 0.16585E+01_JPRB, 0.16418E+01_JPRB, 0.15388E+01_JPRB, 0.14438E+01_JPRB, 0.13377E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.11434E+01_JPRB, 0.10983E+01_JPRB, 0.10866E+01_JPRB, 0.10789E+01_JPRB, 0.10680E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10197E+01_JPRB, 0.10153E+01_JPRB, 0.10300E+01_JPRB, 0.10511E+01_JPRB, 0.10791E+01_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.11439E+01_JPRB, 0.11684E+01_JPRB, 0.11816E+01_JPRB, 0.11813E+01_JPRB, 0.11302E+01_JPRB, 0.10777E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.99427E+00_JPRB, 0.96217E+00_JPRB, 0.94033E+00_JPRB, 0.91586E+00_JPRB, 0.89453E+00_JPRB, 0.88308E+00_JPRB, 0.87989E+00_JPRB ,&
&  0.87879E+00_JPRB, 0.86788E+00_JPRB, 0.85394E+00_JPRB, 0.83619E+00_JPRB, 0.82162E+00_JPRB, 0.80867E+00_JPRB, 0.79643E+00_JPRB ,&
&  0.78522E+00_JPRB, 0.78339E+00_JPRB, 0.78831E+00_JPRB, 0.80064E+00_JPRB, 0.80951E+00_JPRB, 0.81385E+00_JPRB, 0.81486E+00_JPRB ,&
&  0.80778E+00_JPRB, 0.79667E+00_JPRB, 0.80126E+00_JPRB, 0.81294E+00_JPRB, 0.82185E+00_JPRB, 0.83021E+00_JPRB, 0.81137E+00_JPRB ,&
&  0.80229E+00_JPRB, 0.80913E+00_JPRB, 0.83643E+00_JPRB, 0.90244E+00_JPRB, 0.94061E+00_JPRB, 0.96345E+00_JPRB, 0.97819E+00_JPRB ,&
&  0.96919E+00_JPRB, 0.97223E+00_JPRB, 0.10021E+01_JPRB, 0.10630E+01_JPRB, 0.11494E+01_JPRB, 0.13061E+01_JPRB, 0.14992E+01_JPRB ,&
&  0.16233E+01_JPRB, 0.16867E+01_JPRB, 0.17217E+01_JPRB, 0.16236E+01_JPRB, 0.16338E+01_JPRB, 0.18396E+01_JPRB, 0.18988E+01_JPRB ,&
&  0.19477E+01_JPRB, 0.19558E+01_JPRB, 0.19905E+01_JPRB, 0.19149E+01_JPRB, 0.17222E+01_JPRB, 0.15774E+01_JPRB, 0.14721E+01_JPRB ,&
&  0.13683E+01_JPRB, 0.13093E+01_JPRB, 0.12994E+01_JPRB, 0.13334E+01_JPRB, 0.14120E+01_JPRB, 0.15691E+01_JPRB, 0.17684E+01_JPRB ,&
&  0.19609E+01_JPRB, 0.20755E+01_JPRB, 0.21429E+01_JPRB, 0.21248E+01_JPRB, 0.20911E+01_JPRB, 0.20424E+01_JPRB, 0.20050E+01_JPRB ,&
&  0.19617E+01_JPRB, 0.19474E+01_JPRB, 0.19287E+01_JPRB, 0.19075E+01_JPRB, 0.17727E+01_JPRB, 0.15541E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.11108E+01_JPRB, 0.10532E+01_JPRB, 0.10409E+01_JPRB, 0.10451E+01_JPRB, 0.10485E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.10195E+01_JPRB, 0.10419E+01_JPRB, 0.10646E+01_JPRB, 0.11021E+01_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.12505E+01_JPRB, 0.12763E+01_JPRB, 0.12861E+01_JPRB, 0.12711E+01_JPRB, 0.12283E+01_JPRB, 0.11585E+01_JPRB, 0.10761E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10009E+01_JPRB, 0.97928E+00_JPRB, 0.95524E+00_JPRB, 0.93378E+00_JPRB, 0.90455E+00_JPRB, 0.87616E+00_JPRB ,&
&  0.86712E+00_JPRB, 0.87033E+00_JPRB, 0.86923E+00_JPRB, 0.85945E+00_JPRB, 0.84789E+00_JPRB, 0.83587E+00_JPRB, 0.82555E+00_JPRB ,&
&  0.81764E+00_JPRB, 0.81797E+00_JPRB, 0.83017E+00_JPRB, 0.84724E+00_JPRB, 0.85575E+00_JPRB, 0.85910E+00_JPRB, 0.86526E+00_JPRB ,&
&  0.86288E+00_JPRB, 0.85148E+00_JPRB, 0.84759E+00_JPRB, 0.85279E+00_JPRB, 0.85714E+00_JPRB, 0.87968E+00_JPRB, 0.87962E+00_JPRB ,&
&  0.84776E+00_JPRB, 0.87600E+00_JPRB, 0.93883E+00_JPRB, 0.97474E+00_JPRB, 0.99819E+00_JPRB, 0.10150E+01_JPRB, 0.10289E+01_JPRB ,&
&  0.10416E+01_JPRB, 0.10654E+01_JPRB, 0.10949E+01_JPRB, 0.11308E+01_JPRB, 0.12890E+01_JPRB, 0.16484E+01_JPRB, 0.17921E+01_JPRB ,&
&  0.18838E+01_JPRB, 0.17090E+01_JPRB, 0.16228E+01_JPRB, 0.16472E+01_JPRB, 0.18735E+01_JPRB, 0.20142E+01_JPRB, 0.21202E+01_JPRB ,&
&  0.22470E+01_JPRB, 0.22461E+01_JPRB, 0.22031E+01_JPRB, 0.20966E+01_JPRB, 0.19177E+01_JPRB, 0.17851E+01_JPRB, 0.17356E+01_JPRB ,&
&  0.16748E+01_JPRB, 0.16674E+01_JPRB, 0.17515E+01_JPRB, 0.19398E+01_JPRB, 0.21605E+01_JPRB, 0.23467E+01_JPRB, 0.24864E+01_JPRB ,&
&  0.26213E+01_JPRB, 0.26814E+01_JPRB, 0.26443E+01_JPRB, 0.25756E+01_JPRB, 0.24609E+01_JPRB, 0.23912E+01_JPRB, 0.23598E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.22903E+01_JPRB, 0.22257E+01_JPRB, 0.21383E+01_JPRB, 0.19949E+01_JPRB, 0.18595E+01_JPRB, 0.16796E+01_JPRB ,&
&  0.13984E+01_JPRB, 0.12452E+01_JPRB, 0.11183E+01_JPRB, 0.10840E+01_JPRB, 0.10875E+01_JPRB, 0.11047E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.11167E+01_JPRB, 0.11300E+01_JPRB, 0.11370E+01_JPRB, 0.11712E+01_JPRB, 0.12149E+01_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.13674E+01_JPRB, 0.13762E+01_JPRB, 0.13657E+01_JPRB, 0.13490E+01_JPRB, 0.13158E+01_JPRB, 0.12512E+01_JPRB, 0.11899E+01_JPRB ,&
&  0.12201E+01_JPRB, 0.11729E+01_JPRB, 0.11708E+01_JPRB, 0.10438E+01_JPRB, 0.99775E+00_JPRB, 0.97764E+00_JPRB, 0.93980E+00_JPRB ,&
&  0.91744E+00_JPRB, 0.90828E+00_JPRB, 0.89719E+00_JPRB, 0.89371E+00_JPRB, 0.88825E+00_JPRB, 0.87572E+00_JPRB, 0.86742E+00_JPRB ,&
&  0.86782E+00_JPRB, 0.87811E+00_JPRB, 0.89849E+00_JPRB, 0.91395E+00_JPRB, 0.92841E+00_JPRB, 0.93841E+00_JPRB, 0.94423E+00_JPRB ,&
&  0.92134E+00_JPRB, 0.91046E+00_JPRB, 0.90911E+00_JPRB, 0.89971E+00_JPRB, 0.90466E+00_JPRB, 0.92266E+00_JPRB, 0.95221E+00_JPRB ,&
&  0.97212E+00_JPRB, 0.95955E+00_JPRB, 0.10164E+01_JPRB, 0.10464E+01_JPRB, 0.10661E+01_JPRB, 0.10800E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.11001E+01_JPRB, 0.11235E+01_JPRB, 0.11569E+01_JPRB, 0.12090E+01_JPRB, 0.15084E+01_JPRB, 0.20830E+01_JPRB, 0.21070E+01_JPRB ,&
&  0.20717E+01_JPRB, 0.17742E+01_JPRB, 0.19447E+01_JPRB, 0.20247E+01_JPRB, 0.21849E+01_JPRB, 0.23402E+01_JPRB, 0.23766E+01_JPRB ,&
&  0.26324E+01_JPRB, 0.27385E+01_JPRB, 0.26416E+01_JPRB, 0.24245E+01_JPRB, 0.22328E+01_JPRB, 0.21395E+01_JPRB, 0.21027E+01_JPRB ,&
&  0.21238E+01_JPRB, 0.20799E+01_JPRB, 0.23205E+01_JPRB, 0.25237E+01_JPRB, 0.26351E+01_JPRB, 0.27479E+01_JPRB, 0.28789E+01_JPRB ,&
&  0.29613E+01_JPRB, 0.29578E+01_JPRB, 0.29336E+01_JPRB, 0.27655E+01_JPRB, 0.26755E+01_JPRB, 0.26046E+01_JPRB, 0.24844E+01_JPRB ,&
&  0.24069E+01_JPRB, 0.23452E+01_JPRB, 0.22611E+01_JPRB, 0.22707E+01_JPRB, 0.20761E+01_JPRB, 0.19384E+01_JPRB, 0.18574E+01_JPRB ,&
&  0.16396E+01_JPRB, 0.14144E+01_JPRB, 0.13211E+01_JPRB, 0.12877E+01_JPRB, 0.12580E+01_JPRB, 0.12540E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.12189E+01_JPRB, 0.12374E+01_JPRB, 0.12987E+01_JPRB, 0.13260E+01_JPRB, 0.13436E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.14689E+01_JPRB, 0.14758E+01_JPRB, 0.14650E+01_JPRB, 0.14483E+01_JPRB, 0.14298E+01_JPRB, 0.13970E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.13265E+01_JPRB, 0.12865E+01_JPRB, 0.12401E+01_JPRB, 0.12071E+01_JPRB, 0.11963E+01_JPRB, 0.11168E+01_JPRB, 0.99905E+00_JPRB ,&
&  0.95338E+00_JPRB, 0.95163E+00_JPRB, 0.95924E+00_JPRB, 0.95773E+00_JPRB, 0.95513E+00_JPRB, 0.95066E+00_JPRB, 0.95580E+00_JPRB ,&
&  0.96814E+00_JPRB, 0.98351E+00_JPRB, 0.10105E+01_JPRB, 0.10414E+01_JPRB, 0.10499E+01_JPRB, 0.10548E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.99157E+00_JPRB, 0.96915E+00_JPRB, 0.93704E+00_JPRB, 0.97569E+00_JPRB, 0.10073E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11083E+01_JPRB, 0.11387E+01_JPRB, 0.11586E+01_JPRB, 0.11650E+01_JPRB, 0.11715E+01_JPRB, 0.11779E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.12180E+01_JPRB, 0.12601E+01_JPRB, 0.13251E+01_JPRB, 0.19135E+01_JPRB, 0.25512E+01_JPRB, 0.26550E+01_JPRB ,&
&  0.25039E+01_JPRB, 0.26659E+01_JPRB, 0.27969E+01_JPRB, 0.26422E+01_JPRB, 0.27642E+01_JPRB, 0.28499E+01_JPRB, 0.27800E+01_JPRB ,&
&  0.29120E+01_JPRB, 0.29295E+01_JPRB, 0.28847E+01_JPRB, 0.28190E+01_JPRB, 0.27727E+01_JPRB, 0.26949E+01_JPRB, 0.26383E+01_JPRB ,&
&  0.26770E+01_JPRB, 0.28628E+01_JPRB, 0.29074E+01_JPRB, 0.29762E+01_JPRB, 0.29937E+01_JPRB, 0.30482E+01_JPRB, 0.30992E+01_JPRB ,&
&  0.31175E+01_JPRB, 0.31433E+01_JPRB, 0.31283E+01_JPRB, 0.30276E+01_JPRB, 0.28996E+01_JPRB, 0.27654E+01_JPRB, 0.26062E+01_JPRB ,&
&  0.24774E+01_JPRB, 0.22372E+01_JPRB, 0.20535E+01_JPRB, 0.19969E+01_JPRB, 0.19450E+01_JPRB, 0.17958E+01_JPRB, 0.17529E+01_JPRB ,&
&  0.17132E+01_JPRB, 0.16433E+01_JPRB, 0.17459E+01_JPRB, 0.16502E+01_JPRB, 0.15510E+01_JPRB, 0.14308E+01_JPRB, 0.12956E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.13415E+01_JPRB, 0.14238E+01_JPRB, 0.14674E+01_JPRB, 0.14701E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.15428E+01_JPRB, 0.15546E+01_JPRB, 0.15575E+01_JPRB, 0.15546E+01_JPRB, 0.15347E+01_JPRB, 0.14989E+01_JPRB, 0.14532E+01_JPRB ,&
&  0.14107E+01_JPRB, 0.13606E+01_JPRB, 0.13163E+01_JPRB, 0.12814E+01_JPRB, 0.12653E+01_JPRB, 0.12402E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10307E+01_JPRB, 0.10070E+01_JPRB, 0.10081E+01_JPRB, 0.10381E+01_JPRB, 0.10530E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.11549E+01_JPRB, 0.11794E+01_JPRB, 0.12042E+01_JPRB, 0.12211E+01_JPRB, 0.12442E+01_JPRB, 0.12689E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.11291E+01_JPRB, 0.10809E+01_JPRB, 0.10597E+01_JPRB, 0.11025E+01_JPRB, 0.11648E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.12590E+01_JPRB, 0.12811E+01_JPRB, 0.12997E+01_JPRB, 0.13107E+01_JPRB, 0.13078E+01_JPRB, 0.13075E+01_JPRB, 0.13137E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.13617E+01_JPRB, 0.13891E+01_JPRB, 0.15001E+01_JPRB, 0.24245E+01_JPRB, 0.34303E+01_JPRB, 0.37730E+01_JPRB ,&
&  0.35551E+01_JPRB, 0.35383E+01_JPRB, 0.36285E+01_JPRB, 0.38109E+01_JPRB, 0.37644E+01_JPRB, 0.34941E+01_JPRB, 0.32661E+01_JPRB ,&
&  0.32885E+01_JPRB, 0.31575E+01_JPRB, 0.33746E+01_JPRB, 0.33514E+01_JPRB, 0.33806E+01_JPRB, 0.34385E+01_JPRB, 0.33851E+01_JPRB ,&
&  0.34214E+01_JPRB, 0.34100E+01_JPRB, 0.33766E+01_JPRB, 0.32729E+01_JPRB, 0.32350E+01_JPRB, 0.32823E+01_JPRB, 0.32947E+01_JPRB ,&
&  0.33554E+01_JPRB, 0.33961E+01_JPRB, 0.32208E+01_JPRB, 0.30849E+01_JPRB, 0.29235E+01_JPRB, 0.30465E+01_JPRB, 0.28139E+01_JPRB ,&
&  0.24375E+01_JPRB, 0.19873E+01_JPRB, 0.19886E+01_JPRB, 0.19941E+01_JPRB, 0.20240E+01_JPRB, 0.19780E+01_JPRB, 0.18582E+01_JPRB ,&
&  0.18582E+01_JPRB, 0.20529E+01_JPRB, 0.21380E+01_JPRB, 0.19162E+01_JPRB, 0.17764E+01_JPRB, 0.16135E+01_JPRB, 0.14128E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.15104E+01_JPRB, 0.15506E+01_JPRB, 0.15405E+01_JPRB, 0.15268E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.16955E+01_JPRB, 0.16535E+01_JPRB, 0.16349E+01_JPRB, 0.16324E+01_JPRB, 0.16087E+01_JPRB, 0.15767E+01_JPRB, 0.15370E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.14680E+01_JPRB, 0.14318E+01_JPRB, 0.13990E+01_JPRB, 0.13770E+01_JPRB, 0.13441E+01_JPRB, 0.13060E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.11423E+01_JPRB, 0.11159E+01_JPRB, 0.11274E+01_JPRB, 0.12280E+01_JPRB, 0.12181E+01_JPRB, 0.12356E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.12934E+01_JPRB, 0.12936E+01_JPRB, 0.13058E+01_JPRB, 0.13480E+01_JPRB, 0.14450E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.14036E+01_JPRB, 0.13525E+01_JPRB, 0.13333E+01_JPRB, 0.13849E+01_JPRB, 0.14316E+01_JPRB, 0.15066E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.14964E+01_JPRB, 0.14800E+01_JPRB, 0.14679E+01_JPRB, 0.14619E+01_JPRB, 0.14738E+01_JPRB, 0.15055E+01_JPRB ,&
&  0.15383E+01_JPRB, 0.15310E+01_JPRB, 0.15476E+01_JPRB, 0.17829E+01_JPRB, 0.30518E+01_JPRB, 0.43594E+01_JPRB, 0.50495E+01_JPRB ,&
&  0.51515E+01_JPRB, 0.48551E+01_JPRB, 0.49864E+01_JPRB, 0.56037E+01_JPRB, 0.53064E+01_JPRB, 0.46967E+01_JPRB, 0.41834E+01_JPRB ,&
&  0.39671E+01_JPRB, 0.37785E+01_JPRB, 0.39083E+01_JPRB, 0.40227E+01_JPRB, 0.38861E+01_JPRB, 0.38567E+01_JPRB, 0.37908E+01_JPRB ,&
&  0.37410E+01_JPRB, 0.37591E+01_JPRB, 0.36313E+01_JPRB, 0.34923E+01_JPRB, 0.33864E+01_JPRB, 0.34697E+01_JPRB, 0.34713E+01_JPRB ,&
&  0.36004E+01_JPRB, 0.35176E+01_JPRB, 0.31333E+01_JPRB, 0.31049E+01_JPRB, 0.25379E+01_JPRB, 0.24193E+01_JPRB, 0.27543E+01_JPRB ,&
&  0.23836E+01_JPRB, 0.21093E+01_JPRB, 0.20839E+01_JPRB, 0.21881E+01_JPRB, 0.22679E+01_JPRB, 0.24440E+01_JPRB, 0.23980E+01_JPRB ,&
&  0.20826E+01_JPRB, 0.21803E+01_JPRB, 0.25355E+01_JPRB, 0.22692E+01_JPRB, 0.20582E+01_JPRB, 0.19186E+01_JPRB, 0.17347E+01_JPRB ,&
&  0.17606E+01_JPRB, 0.17492E+01_JPRB, 0.17302E+01_JPRB, 0.17017E+01_JPRB, 0.16959E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.18178E+01_JPRB, 0.16789E+01_JPRB, 0.16716E+01_JPRB, 0.16514E+01_JPRB, 0.16268E+01_JPRB, 0.15992E+01_JPRB, 0.15852E+01_JPRB ,&
&  0.15807E+01_JPRB, 0.15686E+01_JPRB, 0.15415E+01_JPRB, 0.15074E+01_JPRB, 0.14789E+01_JPRB, 0.14408E+01_JPRB, 0.13970E+01_JPRB ,&
&  0.13643E+01_JPRB, 0.13506E+01_JPRB, 0.13385E+01_JPRB, 0.12861E+01_JPRB, 0.13630E+01_JPRB, 0.13606E+01_JPRB, 0.13885E+01_JPRB ,&
&  0.14303E+01_JPRB, 0.14948E+01_JPRB, 0.15084E+01_JPRB, 0.15361E+01_JPRB, 0.16049E+01_JPRB, 0.17704E+01_JPRB, 0.19783E+01_JPRB ,&
&  0.20496E+01_JPRB, 0.19537E+01_JPRB, 0.19052E+01_JPRB, 0.19441E+01_JPRB, 0.20320E+01_JPRB, 0.18501E+01_JPRB, 0.18224E+01_JPRB ,&
&  0.17420E+01_JPRB, 0.16704E+01_JPRB, 0.16216E+01_JPRB, 0.16010E+01_JPRB, 0.16241E+01_JPRB, 0.16712E+01_JPRB, 0.17035E+01_JPRB ,&
&  0.17044E+01_JPRB, 0.16380E+01_JPRB, 0.16694E+01_JPRB, 0.21972E+01_JPRB, 0.35750E+01_JPRB, 0.49482E+01_JPRB, 0.55728E+01_JPRB ,&
&  0.56987E+01_JPRB, 0.56204E+01_JPRB, 0.57349E+01_JPRB, 0.63452E+01_JPRB, 0.59959E+01_JPRB, 0.53729E+01_JPRB, 0.50198E+01_JPRB ,&
&  0.47900E+01_JPRB, 0.46542E+01_JPRB, 0.46928E+01_JPRB, 0.44014E+01_JPRB, 0.40355E+01_JPRB, 0.38092E+01_JPRB, 0.36077E+01_JPRB ,&
&  0.34327E+01_JPRB, 0.34253E+01_JPRB, 0.34229E+01_JPRB, 0.32448E+01_JPRB, 0.31522E+01_JPRB, 0.30376E+01_JPRB, 0.32568E+01_JPRB ,&
&  0.36145E+01_JPRB, 0.28252E+01_JPRB, 0.23890E+01_JPRB, 0.26278E+01_JPRB, 0.20660E+01_JPRB, 0.20071E+01_JPRB, 0.22386E+01_JPRB ,&
&  0.21227E+01_JPRB, 0.21740E+01_JPRB, 0.24729E+01_JPRB, 0.24770E+01_JPRB, 0.26437E+01_JPRB, 0.29664E+01_JPRB, 0.29852E+01_JPRB ,&
&  0.27616E+01_JPRB, 0.25906E+01_JPRB, 0.31444E+01_JPRB, 0.28111E+01_JPRB, 0.24975E+01_JPRB, 0.23505E+01_JPRB, 0.22230E+01_JPRB ,&
&  0.21436E+01_JPRB, 0.20679E+01_JPRB, 0.19781E+01_JPRB, 0.19139E+01_JPRB, 0.18917E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.18107E+01_JPRB, 0.16744E+01_JPRB, 0.17267E+01_JPRB, 0.17269E+01_JPRB, 0.17148E+01_JPRB, 0.17052E+01_JPRB, 0.17063E+01_JPRB ,&
&  0.16901E+01_JPRB, 0.16737E+01_JPRB, 0.16347E+01_JPRB, 0.15696E+01_JPRB, 0.15226E+01_JPRB, 0.14826E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.14204E+01_JPRB, 0.14830E+01_JPRB, 0.14233E+01_JPRB, 0.14103E+01_JPRB, 0.13736E+01_JPRB, 0.14368E+01_JPRB, 0.15019E+01_JPRB ,&
&  0.16173E+01_JPRB, 0.17304E+01_JPRB, 0.18791E+01_JPRB, 0.20753E+01_JPRB, 0.22673E+01_JPRB, 0.24481E+01_JPRB, 0.27460E+01_JPRB ,&
&  0.28654E+01_JPRB, 0.27463E+01_JPRB, 0.25673E+01_JPRB, 0.25796E+01_JPRB, 0.24965E+01_JPRB, 0.22285E+01_JPRB, 0.21362E+01_JPRB ,&
&  0.19882E+01_JPRB, 0.18921E+01_JPRB, 0.18282E+01_JPRB, 0.18184E+01_JPRB, 0.17940E+01_JPRB, 0.17140E+01_JPRB, 0.16442E+01_JPRB ,&
&  0.15992E+01_JPRB, 0.16616E+01_JPRB, 0.20179E+01_JPRB, 0.28468E+01_JPRB, 0.42128E+01_JPRB, 0.55773E+01_JPRB, 0.59158E+01_JPRB ,&
&  0.61077E+01_JPRB, 0.57331E+01_JPRB, 0.46349E+01_JPRB, 0.52215E+01_JPRB, 0.64940E+01_JPRB, 0.61541E+01_JPRB, 0.55280E+01_JPRB ,&
&  0.54475E+01_JPRB, 0.50454E+01_JPRB, 0.48374E+01_JPRB, 0.43289E+01_JPRB, 0.38649E+01_JPRB, 0.35360E+01_JPRB, 0.32648E+01_JPRB ,&
&  0.30620E+01_JPRB, 0.30160E+01_JPRB, 0.30212E+01_JPRB, 0.30515E+01_JPRB, 0.29873E+01_JPRB, 0.31239E+01_JPRB, 0.34716E+01_JPRB ,&
&  0.34867E+01_JPRB, 0.32597E+01_JPRB, 0.25535E+01_JPRB, 0.24966E+01_JPRB, 0.19582E+01_JPRB, 0.19070E+01_JPRB, 0.21187E+01_JPRB ,&
&  0.21386E+01_JPRB, 0.26070E+01_JPRB, 0.32731E+01_JPRB, 0.34029E+01_JPRB, 0.39812E+01_JPRB, 0.41218E+01_JPRB, 0.40433E+01_JPRB ,&
&  0.38040E+01_JPRB, 0.34089E+01_JPRB, 0.39431E+01_JPRB, 0.33908E+01_JPRB, 0.28941E+01_JPRB, 0.26845E+01_JPRB, 0.25436E+01_JPRB ,&
&  0.24420E+01_JPRB, 0.22976E+01_JPRB, 0.21396E+01_JPRB, 0.20310E+01_JPRB, 0.19411E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.18169E+01_JPRB, 0.17296E+01_JPRB, 0.17810E+01_JPRB, 0.17538E+01_JPRB, 0.17742E+01_JPRB, 0.17534E+01_JPRB, 0.17657E+01_JPRB ,&
&  0.17517E+01_JPRB, 0.17000E+01_JPRB, 0.16289E+01_JPRB, 0.15659E+01_JPRB, 0.15206E+01_JPRB, 0.14801E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.14430E+01_JPRB, 0.14577E+01_JPRB, 0.13555E+01_JPRB, 0.13000E+01_JPRB, 0.12804E+01_JPRB, 0.13813E+01_JPRB, 0.15641E+01_JPRB ,&
&  0.17947E+01_JPRB, 0.20239E+01_JPRB, 0.23825E+01_JPRB, 0.26713E+01_JPRB, 0.29374E+01_JPRB, 0.31858E+01_JPRB, 0.35317E+01_JPRB ,&
&  0.38411E+01_JPRB, 0.32824E+01_JPRB, 0.30607E+01_JPRB, 0.27365E+01_JPRB, 0.25201E+01_JPRB, 0.23338E+01_JPRB, 0.21824E+01_JPRB ,&
&  0.20669E+01_JPRB, 0.19982E+01_JPRB, 0.19160E+01_JPRB, 0.17976E+01_JPRB, 0.16812E+01_JPRB, 0.16162E+01_JPRB, 0.16083E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.19537E+01_JPRB, 0.27313E+01_JPRB, 0.38501E+01_JPRB, 0.55604E+01_JPRB, 0.67107E+01_JPRB, 0.65963E+01_JPRB ,&
&  0.54604E+01_JPRB, 0.52169E+01_JPRB, 0.72109E+01_JPRB, 0.62509E+01_JPRB, 0.63686E+01_JPRB, 0.67686E+01_JPRB, 0.62211E+01_JPRB ,&
&  0.54075E+01_JPRB, 0.45134E+01_JPRB, 0.43368E+01_JPRB, 0.39822E+01_JPRB, 0.46840E+01_JPRB, 0.40474E+01_JPRB, 0.35369E+01_JPRB ,&
&  0.32396E+01_JPRB, 0.32112E+01_JPRB, 0.31263E+01_JPRB, 0.31648E+01_JPRB, 0.33988E+01_JPRB, 0.39422E+01_JPRB, 0.40567E+01_JPRB ,&
&  0.37054E+01_JPRB, 0.37912E+01_JPRB, 0.31945E+01_JPRB, 0.26553E+01_JPRB, 0.23160E+01_JPRB, 0.23197E+01_JPRB, 0.24106E+01_JPRB ,&
&  0.27124E+01_JPRB, 0.31703E+01_JPRB, 0.41987E+01_JPRB, 0.51956E+01_JPRB, 0.59622E+01_JPRB, 0.58089E+01_JPRB, 0.50497E+01_JPRB ,&
&  0.46839E+01_JPRB, 0.52466E+01_JPRB, 0.51359E+01_JPRB, 0.39614E+01_JPRB, 0.32431E+01_JPRB, 0.29917E+01_JPRB, 0.28041E+01_JPRB ,&
&  0.25539E+01_JPRB, 0.22944E+01_JPRB, 0.21066E+01_JPRB, 0.20061E+01_JPRB, 0.19489E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.19157E+01_JPRB, 0.18338E+01_JPRB, 0.17475E+01_JPRB, 0.17213E+01_JPRB, 0.17596E+01_JPRB, 0.17284E+01_JPRB, 0.16959E+01_JPRB ,&
&  0.16412E+01_JPRB, 0.15767E+01_JPRB, 0.15546E+01_JPRB, 0.15445E+01_JPRB, 0.15213E+01_JPRB, 0.14867E+01_JPRB, 0.14666E+01_JPRB ,&
&  0.14489E+01_JPRB, 0.13890E+01_JPRB, 0.12932E+01_JPRB, 0.13074E+01_JPRB, 0.13149E+01_JPRB, 0.13468E+01_JPRB, 0.15966E+01_JPRB ,&
&  0.19734E+01_JPRB, 0.24337E+01_JPRB, 0.28667E+01_JPRB, 0.31871E+01_JPRB, 0.33767E+01_JPRB, 0.36134E+01_JPRB, 0.39269E+01_JPRB ,&
&  0.36923E+01_JPRB, 0.35722E+01_JPRB, 0.32555E+01_JPRB, 0.28783E+01_JPRB, 0.25719E+01_JPRB, 0.23233E+01_JPRB, 0.21874E+01_JPRB ,&
&  0.20879E+01_JPRB, 0.20377E+01_JPRB, 0.19044E+01_JPRB, 0.17485E+01_JPRB, 0.17069E+01_JPRB, 0.17028E+01_JPRB, 0.17220E+01_JPRB ,&
&  0.18277E+01_JPRB, 0.23958E+01_JPRB, 0.33615E+01_JPRB, 0.45661E+01_JPRB, 0.47122E+01_JPRB, 0.56488E+01_JPRB, 0.57648E+01_JPRB ,&
&  0.70874E+01_JPRB, 0.71008E+01_JPRB, 0.68445E+01_JPRB, 0.73641E+01_JPRB, 0.61434E+01_JPRB, 0.62629E+01_JPRB, 0.61050E+01_JPRB ,&
&  0.53736E+01_JPRB, 0.50225E+01_JPRB, 0.54065E+01_JPRB, 0.46973E+01_JPRB, 0.46214E+01_JPRB, 0.51968E+01_JPRB, 0.45505E+01_JPRB ,&
&  0.42332E+01_JPRB, 0.40211E+01_JPRB, 0.37498E+01_JPRB, 0.39484E+01_JPRB, 0.38556E+01_JPRB, 0.39583E+01_JPRB, 0.34345E+01_JPRB ,&
&  0.33571E+01_JPRB, 0.28809E+01_JPRB, 0.31476E+01_JPRB, 0.28836E+01_JPRB, 0.28788E+01_JPRB, 0.28536E+01_JPRB, 0.29463E+01_JPRB ,&
&  0.35471E+01_JPRB, 0.41267E+01_JPRB, 0.56340E+01_JPRB, 0.71476E+01_JPRB, 0.83086E+01_JPRB, 0.69845E+01_JPRB, 0.62631E+01_JPRB ,&
&  0.65448E+01_JPRB, 0.66642E+01_JPRB, 0.54450E+01_JPRB, 0.42159E+01_JPRB, 0.35107E+01_JPRB, 0.32092E+01_JPRB, 0.29992E+01_JPRB ,&
&  0.27614E+01_JPRB, 0.25272E+01_JPRB, 0.23386E+01_JPRB, 0.22002E+01_JPRB, 0.20894E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.17833E+01_JPRB, 0.17380E+01_JPRB, 0.16302E+01_JPRB, 0.16368E+01_JPRB, 0.16966E+01_JPRB, 0.16650E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.15618E+01_JPRB, 0.15185E+01_JPRB, 0.15304E+01_JPRB, 0.15347E+01_JPRB, 0.15165E+01_JPRB, 0.14938E+01_JPRB, 0.14577E+01_JPRB ,&
&  0.14339E+01_JPRB, 0.13970E+01_JPRB, 0.13139E+01_JPRB, 0.13062E+01_JPRB, 0.13832E+01_JPRB, 0.14454E+01_JPRB, 0.16466E+01_JPRB ,&
&  0.23455E+01_JPRB, 0.29346E+01_JPRB, 0.32313E+01_JPRB, 0.34278E+01_JPRB, 0.35293E+01_JPRB, 0.36584E+01_JPRB, 0.36777E+01_JPRB ,&
&  0.35094E+01_JPRB, 0.30844E+01_JPRB, 0.28241E+01_JPRB, 0.26330E+01_JPRB, 0.24588E+01_JPRB, 0.23067E+01_JPRB, 0.21726E+01_JPRB ,&
&  0.20484E+01_JPRB, 0.18396E+01_JPRB, 0.16383E+01_JPRB, 0.15254E+01_JPRB, 0.15456E+01_JPRB, 0.15460E+01_JPRB, 0.16511E+01_JPRB ,&
&  0.20574E+01_JPRB, 0.26542E+01_JPRB, 0.33871E+01_JPRB, 0.42266E+01_JPRB, 0.45254E+01_JPRB, 0.53766E+01_JPRB, 0.63270E+01_JPRB ,&
&  0.64539E+01_JPRB, 0.61650E+01_JPRB, 0.70043E+01_JPRB, 0.78087E+01_JPRB, 0.76685E+01_JPRB, 0.66119E+01_JPRB, 0.62825E+01_JPRB ,&
&  0.47315E+01_JPRB, 0.38548E+01_JPRB, 0.35735E+01_JPRB, 0.37299E+01_JPRB, 0.49348E+01_JPRB, 0.64158E+01_JPRB, 0.60726E+01_JPRB ,&
&  0.58662E+01_JPRB, 0.55193E+01_JPRB, 0.50841E+01_JPRB, 0.44403E+01_JPRB, 0.41178E+01_JPRB, 0.38290E+01_JPRB, 0.48091E+01_JPRB ,&
&  0.47752E+01_JPRB, 0.47582E+01_JPRB, 0.40709E+01_JPRB, 0.33695E+01_JPRB, 0.29369E+01_JPRB, 0.34212E+01_JPRB, 0.38435E+01_JPRB ,&
&  0.45518E+01_JPRB, 0.54561E+01_JPRB, 0.96216E+01_JPRB, 0.10758E+02_JPRB, 0.99680E+01_JPRB, 0.87493E+01_JPRB, 0.87435E+01_JPRB ,&
&  0.78259E+01_JPRB, 0.74955E+01_JPRB, 0.60344E+01_JPRB, 0.43937E+01_JPRB, 0.34112E+01_JPRB, 0.28749E+01_JPRB, 0.25837E+01_JPRB ,&
&  0.24850E+01_JPRB, 0.23930E+01_JPRB, 0.22469E+01_JPRB, 0.20669E+01_JPRB, 0.19344E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.16610E+01_JPRB, 0.16222E+01_JPRB, 0.15066E+01_JPRB, 0.14976E+01_JPRB, 0.15537E+01_JPRB, 0.15942E+01_JPRB, 0.15775E+01_JPRB ,&
&  0.15509E+01_JPRB, 0.15556E+01_JPRB, 0.15745E+01_JPRB, 0.15514E+01_JPRB, 0.15165E+01_JPRB, 0.14973E+01_JPRB, 0.14670E+01_JPRB ,&
&  0.14687E+01_JPRB, 0.15395E+01_JPRB, 0.14380E+01_JPRB, 0.14788E+01_JPRB, 0.16076E+01_JPRB, 0.17723E+01_JPRB, 0.20338E+01_JPRB ,&
&  0.29413E+01_JPRB, 0.32180E+01_JPRB, 0.30871E+01_JPRB, 0.32142E+01_JPRB, 0.35597E+01_JPRB, 0.32747E+01_JPRB, 0.30964E+01_JPRB ,&
&  0.28729E+01_JPRB, 0.25520E+01_JPRB, 0.24049E+01_JPRB, 0.23003E+01_JPRB, 0.22703E+01_JPRB, 0.22444E+01_JPRB, 0.21278E+01_JPRB ,&
&  0.19209E+01_JPRB, 0.16692E+01_JPRB, 0.15310E+01_JPRB, 0.15189E+01_JPRB, 0.14576E+01_JPRB, 0.14860E+01_JPRB, 0.16196E+01_JPRB ,&
&  0.21039E+01_JPRB, 0.27172E+01_JPRB, 0.31220E+01_JPRB, 0.36060E+01_JPRB, 0.40491E+01_JPRB, 0.50026E+01_JPRB, 0.70542E+01_JPRB ,&
&  0.67182E+01_JPRB, 0.68156E+01_JPRB, 0.70660E+01_JPRB, 0.81827E+01_JPRB, 0.73344E+01_JPRB, 0.63632E+01_JPRB, 0.59077E+01_JPRB ,&
&  0.43934E+01_JPRB, 0.37466E+01_JPRB, 0.40932E+01_JPRB, 0.46718E+01_JPRB, 0.46935E+01_JPRB, 0.52934E+01_JPRB, 0.68041E+01_JPRB ,&
&  0.59551E+01_JPRB, 0.58954E+01_JPRB, 0.57417E+01_JPRB, 0.55272E+01_JPRB, 0.33881E+01_JPRB, 0.28296E+01_JPRB, 0.41743E+01_JPRB ,&
&  0.44481E+01_JPRB, 0.29371E+01_JPRB, 0.19973E+01_JPRB, 0.25705E+01_JPRB, 0.19797E+01_JPRB, 0.24327E+01_JPRB, 0.39239E+01_JPRB ,&
&  0.56484E+01_JPRB, 0.79651E+01_JPRB, 0.14022E+02_JPRB, 0.12222E+02_JPRB, 0.10963E+02_JPRB, 0.11724E+02_JPRB, 0.99528E+01_JPRB ,&
&  0.96482E+01_JPRB, 0.68786E+01_JPRB, 0.53961E+01_JPRB, 0.36942E+01_JPRB, 0.25975E+01_JPRB, 0.21329E+01_JPRB, 0.19813E+01_JPRB ,&
&  0.19246E+01_JPRB, 0.19338E+01_JPRB, 0.19346E+01_JPRB, 0.18484E+01_JPRB, 0.17450E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.17131E+01_JPRB, 0.15862E+01_JPRB, 0.14060E+01_JPRB, 0.13831E+01_JPRB, 0.14237E+01_JPRB, 0.14839E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.15138E+01_JPRB, 0.15673E+01_JPRB, 0.16523E+01_JPRB, 0.16346E+01_JPRB, 0.15669E+01_JPRB, 0.15404E+01_JPRB, 0.15005E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.16221E+01_JPRB, 0.18025E+01_JPRB, 0.19954E+01_JPRB, 0.22328E+01_JPRB, 0.23764E+01_JPRB, 0.29699E+01_JPRB ,&
&  0.34370E+01_JPRB, 0.31465E+01_JPRB, 0.29513E+01_JPRB, 0.30082E+01_JPRB, 0.30112E+01_JPRB, 0.26325E+01_JPRB, 0.26019E+01_JPRB ,&
&  0.24076E+01_JPRB, 0.22076E+01_JPRB, 0.21165E+01_JPRB, 0.20619E+01_JPRB, 0.21042E+01_JPRB, 0.21312E+01_JPRB, 0.20555E+01_JPRB ,&
&  0.18419E+01_JPRB, 0.16694E+01_JPRB, 0.15705E+01_JPRB, 0.14624E+01_JPRB, 0.14014E+01_JPRB, 0.14401E+01_JPRB, 0.16296E+01_JPRB ,&
&  0.20715E+01_JPRB, 0.25770E+01_JPRB, 0.28193E+01_JPRB, 0.30712E+01_JPRB, 0.42602E+01_JPRB, 0.49640E+01_JPRB, 0.52213E+01_JPRB ,&
&  0.53271E+01_JPRB, 0.55204E+01_JPRB, 0.72786E+01_JPRB, 0.76214E+01_JPRB, 0.72138E+01_JPRB, 0.69154E+01_JPRB, 0.60274E+01_JPRB ,&
&  0.53395E+01_JPRB, 0.48576E+01_JPRB, 0.46732E+01_JPRB, 0.51490E+01_JPRB, 0.54448E+01_JPRB, 0.52176E+01_JPRB, 0.62414E+01_JPRB ,&
&  0.64811E+01_JPRB, 0.61721E+01_JPRB, 0.50026E+01_JPRB, 0.39616E+01_JPRB, 0.36936E+01_JPRB, 0.29196E+01_JPRB, 0.21364E+01_JPRB ,&
&  0.19356E+01_JPRB, 0.18856E+01_JPRB, 0.17190E+01_JPRB, 0.17755E+01_JPRB, 0.16656E+01_JPRB, 0.22082E+01_JPRB, 0.35908E+01_JPRB ,&
&  0.75520E+01_JPRB, 0.11946E+02_JPRB, 0.14125E+02_JPRB, 0.11588E+02_JPRB, 0.10228E+02_JPRB, 0.11582E+02_JPRB, 0.11786E+02_JPRB ,&
&  0.73268E+01_JPRB, 0.53864E+01_JPRB, 0.36170E+01_JPRB, 0.25041E+01_JPRB, 0.20152E+01_JPRB, 0.17755E+01_JPRB, 0.17050E+01_JPRB ,&
&  0.16890E+01_JPRB, 0.16644E+01_JPRB, 0.16846E+01_JPRB, 0.17169E+01_JPRB, 0.16953E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.16848E+01_JPRB, 0.14956E+01_JPRB, 0.13045E+01_JPRB, 0.12593E+01_JPRB, 0.13324E+01_JPRB, 0.13695E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.16200E+01_JPRB, 0.17106E+01_JPRB, 0.16903E+01_JPRB, 0.16340E+01_JPRB, 0.15865E+01_JPRB, 0.15697E+01_JPRB ,&
&  0.16492E+01_JPRB, 0.18855E+01_JPRB, 0.23497E+01_JPRB, 0.28310E+01_JPRB, 0.33204E+01_JPRB, 0.32625E+01_JPRB, 0.37671E+01_JPRB ,&
&  0.33943E+01_JPRB, 0.29553E+01_JPRB, 0.30036E+01_JPRB, 0.29085E+01_JPRB, 0.27875E+01_JPRB, 0.23894E+01_JPRB, 0.22396E+01_JPRB ,&
&  0.20573E+01_JPRB, 0.20262E+01_JPRB, 0.19776E+01_JPRB, 0.20208E+01_JPRB, 0.20624E+01_JPRB, 0.20416E+01_JPRB, 0.19287E+01_JPRB ,&
&  0.17468E+01_JPRB, 0.16814E+01_JPRB, 0.15237E+01_JPRB, 0.15076E+01_JPRB, 0.14986E+01_JPRB, 0.15839E+01_JPRB, 0.17797E+01_JPRB ,&
&  0.21336E+01_JPRB, 0.24965E+01_JPRB, 0.27245E+01_JPRB, 0.32820E+01_JPRB, 0.36816E+01_JPRB, 0.36404E+01_JPRB, 0.47087E+01_JPRB ,&
&  0.56672E+01_JPRB, 0.63671E+01_JPRB, 0.65119E+01_JPRB, 0.75923E+01_JPRB, 0.73610E+01_JPRB, 0.66762E+01_JPRB, 0.61737E+01_JPRB ,&
&  0.54047E+01_JPRB, 0.53391E+01_JPRB, 0.57761E+01_JPRB, 0.61787E+01_JPRB, 0.71818E+01_JPRB, 0.66311E+01_JPRB, 0.68010E+01_JPRB ,&
&  0.72140E+01_JPRB, 0.67173E+01_JPRB, 0.56256E+01_JPRB, 0.46538E+01_JPRB, 0.52247E+01_JPRB, 0.51379E+01_JPRB, 0.30158E+01_JPRB ,&
&  0.25780E+01_JPRB, 0.23019E+01_JPRB, 0.19580E+01_JPRB, 0.17321E+01_JPRB, 0.17842E+01_JPRB, 0.24270E+01_JPRB, 0.48579E+01_JPRB ,&
&  0.68018E+01_JPRB, 0.12531E+02_JPRB, 0.14754E+02_JPRB, 0.12289E+02_JPRB, 0.99298E+01_JPRB, 0.10967E+02_JPRB, 0.78237E+01_JPRB ,&
&  0.63929E+01_JPRB, 0.44407E+01_JPRB, 0.25409E+01_JPRB, 0.19325E+01_JPRB, 0.18091E+01_JPRB, 0.17187E+01_JPRB, 0.16258E+01_JPRB ,&
&  0.15235E+01_JPRB, 0.15013E+01_JPRB, 0.15410E+01_JPRB, 0.15790E+01_JPRB, 0.15881E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.14738E+01_JPRB, 0.13213E+01_JPRB, 0.12106E+01_JPRB, 0.11450E+01_JPRB, 0.12327E+01_JPRB, 0.13556E+01_JPRB, 0.14024E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.17139E+01_JPRB, 0.18026E+01_JPRB, 0.17961E+01_JPRB, 0.17053E+01_JPRB, 0.16894E+01_JPRB, 0.17879E+01_JPRB ,&
&  0.20439E+01_JPRB, 0.24948E+01_JPRB, 0.31614E+01_JPRB, 0.38797E+01_JPRB, 0.42038E+01_JPRB, 0.43255E+01_JPRB, 0.40582E+01_JPRB ,&
&  0.31784E+01_JPRB, 0.28494E+01_JPRB, 0.29975E+01_JPRB, 0.29814E+01_JPRB, 0.25202E+01_JPRB, 0.23045E+01_JPRB, 0.19768E+01_JPRB ,&
&  0.19471E+01_JPRB, 0.19833E+01_JPRB, 0.19748E+01_JPRB, 0.19902E+01_JPRB, 0.19664E+01_JPRB, 0.19152E+01_JPRB, 0.18561E+01_JPRB ,&
&  0.17416E+01_JPRB, 0.16161E+01_JPRB, 0.15492E+01_JPRB, 0.15513E+01_JPRB, 0.16793E+01_JPRB, 0.18866E+01_JPRB, 0.21049E+01_JPRB ,&
&  0.23783E+01_JPRB, 0.26455E+01_JPRB, 0.29602E+01_JPRB, 0.35624E+01_JPRB, 0.31401E+01_JPRB, 0.40487E+01_JPRB, 0.47543E+01_JPRB ,&
&  0.51684E+01_JPRB, 0.55628E+01_JPRB, 0.55623E+01_JPRB, 0.64090E+01_JPRB, 0.68820E+01_JPRB, 0.64266E+01_JPRB, 0.57624E+01_JPRB ,&
&  0.56089E+01_JPRB, 0.55587E+01_JPRB, 0.70011E+01_JPRB, 0.78469E+01_JPRB, 0.85059E+01_JPRB, 0.10413E+02_JPRB, 0.77746E+01_JPRB ,&
&  0.67734E+01_JPRB, 0.72291E+01_JPRB, 0.63229E+01_JPRB, 0.54770E+01_JPRB, 0.57426E+01_JPRB, 0.71322E+01_JPRB, 0.84454E+01_JPRB ,&
&  0.39161E+01_JPRB, 0.24956E+01_JPRB, 0.25220E+01_JPRB, 0.23006E+01_JPRB, 0.19966E+01_JPRB, 0.26755E+01_JPRB, 0.82850E+01_JPRB ,&
&  0.69620E+01_JPRB, 0.11607E+02_JPRB, 0.13079E+02_JPRB, 0.11015E+02_JPRB, 0.94723E+01_JPRB, 0.84775E+01_JPRB, 0.67651E+01_JPRB ,&
&  0.47900E+01_JPRB, 0.32345E+01_JPRB, 0.21764E+01_JPRB, 0.17959E+01_JPRB, 0.17100E+01_JPRB, 0.16543E+01_JPRB, 0.15748E+01_JPRB ,&
&  0.15011E+01_JPRB, 0.14435E+01_JPRB, 0.14627E+01_JPRB, 0.14190E+01_JPRB, 0.14916E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.13417E+01_JPRB, 0.12586E+01_JPRB, 0.12584E+01_JPRB, 0.12662E+01_JPRB, 0.13332E+01_JPRB, 0.14021E+01_JPRB, 0.15102E+01_JPRB ,&
&  0.16918E+01_JPRB, 0.18904E+01_JPRB, 0.20369E+01_JPRB, 0.20364E+01_JPRB, 0.19699E+01_JPRB, 0.20925E+01_JPRB, 0.23918E+01_JPRB ,&
&  0.28155E+01_JPRB, 0.33502E+01_JPRB, 0.39753E+01_JPRB, 0.44058E+01_JPRB, 0.46310E+01_JPRB, 0.42288E+01_JPRB, 0.37066E+01_JPRB ,&
&  0.29949E+01_JPRB, 0.26644E+01_JPRB, 0.25139E+01_JPRB, 0.25300E+01_JPRB, 0.24700E+01_JPRB, 0.22756E+01_JPRB, 0.18895E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.18405E+01_JPRB, 0.18079E+01_JPRB, 0.18169E+01_JPRB, 0.18295E+01_JPRB, 0.18619E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.16882E+01_JPRB, 0.16421E+01_JPRB, 0.16337E+01_JPRB, 0.17369E+01_JPRB, 0.19904E+01_JPRB, 0.22498E+01_JPRB, 0.24988E+01_JPRB ,&
&  0.27484E+01_JPRB, 0.29612E+01_JPRB, 0.32335E+01_JPRB, 0.33919E+01_JPRB, 0.37718E+01_JPRB, 0.42132E+01_JPRB, 0.45637E+01_JPRB ,&
&  0.47399E+01_JPRB, 0.51862E+01_JPRB, 0.55873E+01_JPRB, 0.58884E+01_JPRB, 0.61939E+01_JPRB, 0.60481E+01_JPRB, 0.56741E+01_JPRB ,&
&  0.59626E+01_JPRB, 0.68700E+01_JPRB, 0.80080E+01_JPRB, 0.85492E+01_JPRB, 0.89418E+01_JPRB, 0.10502E+02_JPRB, 0.10291E+02_JPRB ,&
&  0.83076E+01_JPRB, 0.64867E+01_JPRB, 0.59878E+01_JPRB, 0.65665E+01_JPRB, 0.71667E+01_JPRB, 0.88533E+01_JPRB, 0.13994E+02_JPRB ,&
&  0.14356E+02_JPRB, 0.64635E+01_JPRB, 0.42363E+01_JPRB, 0.48489E+01_JPRB, 0.29172E+01_JPRB, 0.27930E+01_JPRB, 0.50510E+01_JPRB ,&
&  0.70381E+01_JPRB, 0.10292E+02_JPRB, 0.11733E+02_JPRB, 0.91826E+01_JPRB, 0.79705E+01_JPRB, 0.59475E+01_JPRB, 0.44870E+01_JPRB ,&
&  0.33701E+01_JPRB, 0.23944E+01_JPRB, 0.18165E+01_JPRB, 0.16437E+01_JPRB, 0.15264E+01_JPRB, 0.15303E+01_JPRB, 0.15733E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.13849E+01_JPRB, 0.13868E+01_JPRB, 0.13825E+01_JPRB, 0.13505E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.13683E+01_JPRB, 0.13552E+01_JPRB, 0.14081E+01_JPRB, 0.14767E+01_JPRB, 0.15558E+01_JPRB, 0.16537E+01_JPRB, 0.17666E+01_JPRB ,&
&  0.19775E+01_JPRB, 0.22194E+01_JPRB, 0.23750E+01_JPRB, 0.24745E+01_JPRB, 0.25981E+01_JPRB, 0.28552E+01_JPRB, 0.32001E+01_JPRB ,&
&  0.36058E+01_JPRB, 0.40243E+01_JPRB, 0.44364E+01_JPRB, 0.48745E+01_JPRB, 0.54466E+01_JPRB, 0.57255E+01_JPRB, 0.33743E+01_JPRB ,&
&  0.27674E+01_JPRB, 0.28295E+01_JPRB, 0.24169E+01_JPRB, 0.25221E+01_JPRB, 0.25654E+01_JPRB, 0.25128E+01_JPRB, 0.20219E+01_JPRB ,&
&  0.17696E+01_JPRB, 0.17282E+01_JPRB, 0.17130E+01_JPRB, 0.17403E+01_JPRB, 0.17687E+01_JPRB, 0.18838E+01_JPRB, 0.19090E+01_JPRB ,&
&  0.19129E+01_JPRB, 0.19776E+01_JPRB, 0.20669E+01_JPRB, 0.23032E+01_JPRB, 0.26050E+01_JPRB, 0.28365E+01_JPRB, 0.29879E+01_JPRB ,&
&  0.31027E+01_JPRB, 0.32420E+01_JPRB, 0.34664E+01_JPRB, 0.36383E+01_JPRB, 0.40652E+01_JPRB, 0.43183E+01_JPRB, 0.45664E+01_JPRB ,&
&  0.45771E+01_JPRB, 0.44886E+01_JPRB, 0.50380E+01_JPRB, 0.54688E+01_JPRB, 0.56635E+01_JPRB, 0.55645E+01_JPRB, 0.56606E+01_JPRB ,&
&  0.68805E+01_JPRB, 0.82905E+01_JPRB, 0.98288E+01_JPRB, 0.88710E+01_JPRB, 0.85715E+01_JPRB, 0.94391E+01_JPRB, 0.10295E+02_JPRB ,&
&  0.90041E+01_JPRB, 0.77020E+01_JPRB, 0.77165E+01_JPRB, 0.80238E+01_JPRB, 0.86650E+01_JPRB, 0.95168E+01_JPRB, 0.12503E+02_JPRB ,&
&  0.14814E+02_JPRB, 0.11535E+02_JPRB, 0.67546E+01_JPRB, 0.38226E+01_JPRB, 0.30979E+01_JPRB, 0.34555E+01_JPRB, 0.45241E+01_JPRB ,&
&  0.73353E+01_JPRB, 0.87682E+01_JPRB, 0.97964E+01_JPRB, 0.95945E+01_JPRB, 0.52587E+01_JPRB, 0.34880E+01_JPRB, 0.27396E+01_JPRB ,&
&  0.19325E+01_JPRB, 0.15331E+01_JPRB, 0.14892E+01_JPRB, 0.14319E+01_JPRB, 0.13607E+01_JPRB, 0.14293E+01_JPRB, 0.15775E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.13313E+01_JPRB, 0.13394E+01_JPRB, 0.13320E+01_JPRB, 0.13354E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.14171E+01_JPRB, 0.14513E+01_JPRB, 0.15322E+01_JPRB, 0.16651E+01_JPRB, 0.18386E+01_JPRB, 0.20292E+01_JPRB, 0.21290E+01_JPRB ,&
&  0.22624E+01_JPRB, 0.24767E+01_JPRB, 0.26817E+01_JPRB, 0.28852E+01_JPRB, 0.30957E+01_JPRB, 0.33493E+01_JPRB, 0.36250E+01_JPRB ,&
&  0.39223E+01_JPRB, 0.42081E+01_JPRB, 0.44970E+01_JPRB, 0.48691E+01_JPRB, 0.53394E+01_JPRB, 0.55794E+01_JPRB, 0.41222E+01_JPRB ,&
&  0.27554E+01_JPRB, 0.38386E+01_JPRB, 0.26050E+01_JPRB, 0.28300E+01_JPRB, 0.29990E+01_JPRB, 0.32602E+01_JPRB, 0.25503E+01_JPRB ,&
&  0.18970E+01_JPRB, 0.18239E+01_JPRB, 0.18313E+01_JPRB, 0.17896E+01_JPRB, 0.17012E+01_JPRB, 0.18741E+01_JPRB, 0.20860E+01_JPRB ,&
&  0.22113E+01_JPRB, 0.23717E+01_JPRB, 0.25831E+01_JPRB, 0.28508E+01_JPRB, 0.30335E+01_JPRB, 0.31831E+01_JPRB, 0.33014E+01_JPRB ,&
&  0.34212E+01_JPRB, 0.35898E+01_JPRB, 0.37263E+01_JPRB, 0.39792E+01_JPRB, 0.41509E+01_JPRB, 0.44294E+01_JPRB, 0.46556E+01_JPRB ,&
&  0.46092E+01_JPRB, 0.45057E+01_JPRB, 0.47831E+01_JPRB, 0.48584E+01_JPRB, 0.51289E+01_JPRB, 0.59564E+01_JPRB, 0.63656E+01_JPRB ,&
&  0.78445E+01_JPRB, 0.85195E+01_JPRB, 0.10079E+02_JPRB, 0.82661E+01_JPRB, 0.77572E+01_JPRB, 0.83230E+01_JPRB, 0.82257E+01_JPRB ,&
&  0.75851E+01_JPRB, 0.81176E+01_JPRB, 0.83899E+01_JPRB, 0.88401E+01_JPRB, 0.93871E+01_JPRB, 0.99740E+01_JPRB, 0.11184E+02_JPRB ,&
&  0.97701E+01_JPRB, 0.53013E+01_JPRB, 0.34953E+01_JPRB, 0.27848E+01_JPRB, 0.25888E+01_JPRB, 0.31905E+01_JPRB, 0.51603E+01_JPRB ,&
&  0.79038E+01_JPRB, 0.83193E+01_JPRB, 0.86746E+01_JPRB, 0.60018E+01_JPRB, 0.29012E+01_JPRB, 0.20213E+01_JPRB, 0.15333E+01_JPRB ,&
&  0.11622E+01_JPRB, 0.10880E+01_JPRB, 0.11924E+01_JPRB, 0.12316E+01_JPRB, 0.12000E+01_JPRB, 0.12633E+01_JPRB, 0.13417E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.12958E+01_JPRB, 0.13708E+01_JPRB, 0.13580E+01_JPRB, 0.13777E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.14351E+01_JPRB, 0.14952E+01_JPRB, 0.16397E+01_JPRB, 0.18405E+01_JPRB, 0.21398E+01_JPRB, 0.24862E+01_JPRB, 0.26887E+01_JPRB ,&
&  0.25998E+01_JPRB, 0.25017E+01_JPRB, 0.27985E+01_JPRB, 0.30779E+01_JPRB, 0.32873E+01_JPRB, 0.35455E+01_JPRB, 0.38159E+01_JPRB ,&
&  0.41299E+01_JPRB, 0.44550E+01_JPRB, 0.48109E+01_JPRB, 0.52460E+01_JPRB, 0.57543E+01_JPRB, 0.62417E+01_JPRB, 0.60908E+01_JPRB ,&
&  0.43092E+01_JPRB, 0.31733E+01_JPRB, 0.31646E+01_JPRB, 0.29038E+01_JPRB, 0.28327E+01_JPRB, 0.31078E+01_JPRB, 0.30872E+01_JPRB ,&
&  0.24971E+01_JPRB, 0.21028E+01_JPRB, 0.21076E+01_JPRB, 0.20731E+01_JPRB, 0.15222E+01_JPRB, 0.16865E+01_JPRB, 0.20176E+01_JPRB ,&
&  0.22255E+01_JPRB, 0.24322E+01_JPRB, 0.26866E+01_JPRB, 0.29399E+01_JPRB, 0.31117E+01_JPRB, 0.33315E+01_JPRB, 0.35329E+01_JPRB ,&
&  0.36976E+01_JPRB, 0.38885E+01_JPRB, 0.39796E+01_JPRB, 0.40266E+01_JPRB, 0.42308E+01_JPRB, 0.44624E+01_JPRB, 0.45961E+01_JPRB ,&
&  0.46486E+01_JPRB, 0.48927E+01_JPRB, 0.51334E+01_JPRB, 0.53690E+01_JPRB, 0.54980E+01_JPRB, 0.64671E+01_JPRB, 0.70506E+01_JPRB ,&
&  0.78079E+01_JPRB, 0.80037E+01_JPRB, 0.88454E+01_JPRB, 0.72553E+01_JPRB, 0.70070E+01_JPRB, 0.71469E+01_JPRB, 0.72409E+01_JPRB ,&
&  0.71320E+01_JPRB, 0.76838E+01_JPRB, 0.79149E+01_JPRB, 0.86447E+01_JPRB, 0.94705E+01_JPRB, 0.94068E+01_JPRB, 0.98747E+01_JPRB ,&
&  0.61441E+01_JPRB, 0.36714E+01_JPRB, 0.23633E+01_JPRB, 0.23995E+01_JPRB, 0.25845E+01_JPRB, 0.32673E+01_JPRB, 0.45612E+01_JPRB ,&
&  0.61233E+01_JPRB, 0.48600E+01_JPRB, 0.43336E+01_JPRB, 0.36644E+01_JPRB, 0.18968E+01_JPRB, 0.12888E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.94363E+00_JPRB, 0.96576E+00_JPRB, 0.10443E+01_JPRB, 0.10599E+01_JPRB, 0.10348E+01_JPRB, 0.10924E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.12239E+01_JPRB, 0.12562E+01_JPRB, 0.13382E+01_JPRB, 0.13464E+01_JPRB, 0.14069E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.13393E+01_JPRB, 0.14559E+01_JPRB, 0.15595E+01_JPRB, 0.16871E+01_JPRB, 0.18324E+01_JPRB, 0.20324E+01_JPRB, 0.21395E+01_JPRB ,&
&  0.21537E+01_JPRB, 0.21809E+01_JPRB, 0.23821E+01_JPRB, 0.26232E+01_JPRB, 0.28330E+01_JPRB, 0.29667E+01_JPRB, 0.31171E+01_JPRB ,&
&  0.32676E+01_JPRB, 0.34077E+01_JPRB, 0.35542E+01_JPRB, 0.36927E+01_JPRB, 0.39253E+01_JPRB, 0.42587E+01_JPRB, 0.45650E+01_JPRB ,&
&  0.40440E+01_JPRB, 0.25783E+01_JPRB, 0.28630E+01_JPRB, 0.27552E+01_JPRB, 0.27626E+01_JPRB, 0.28931E+01_JPRB, 0.30559E+01_JPRB ,&
&  0.24851E+01_JPRB, 0.22214E+01_JPRB, 0.22275E+01_JPRB, 0.22569E+01_JPRB, 0.14404E+01_JPRB, 0.15053E+01_JPRB, 0.17831E+01_JPRB ,&
&  0.20491E+01_JPRB, 0.23137E+01_JPRB, 0.25725E+01_JPRB, 0.27776E+01_JPRB, 0.29840E+01_JPRB, 0.32593E+01_JPRB, 0.35245E+01_JPRB ,&
&  0.37558E+01_JPRB, 0.39290E+01_JPRB, 0.40450E+01_JPRB, 0.41635E+01_JPRB, 0.43037E+01_JPRB, 0.45161E+01_JPRB, 0.46173E+01_JPRB ,&
&  0.47852E+01_JPRB, 0.49353E+01_JPRB, 0.54092E+01_JPRB, 0.59930E+01_JPRB, 0.64464E+01_JPRB, 0.63342E+01_JPRB, 0.67130E+01_JPRB ,&
&  0.68695E+01_JPRB, 0.67660E+01_JPRB, 0.70290E+01_JPRB, 0.72834E+01_JPRB, 0.54877E+01_JPRB, 0.55799E+01_JPRB, 0.58524E+01_JPRB ,&
&  0.66325E+01_JPRB, 0.67249E+01_JPRB, 0.70426E+01_JPRB, 0.75284E+01_JPRB, 0.77983E+01_JPRB, 0.78086E+01_JPRB, 0.72627E+01_JPRB ,&
&  0.49076E+01_JPRB, 0.28331E+01_JPRB, 0.21089E+01_JPRB, 0.23063E+01_JPRB, 0.28317E+01_JPRB, 0.40289E+01_JPRB, 0.36739E+01_JPRB ,&
&  0.42429E+01_JPRB, 0.27957E+01_JPRB, 0.26145E+01_JPRB, 0.30623E+01_JPRB, 0.17944E+01_JPRB, 0.12066E+01_JPRB, 0.96682E+00_JPRB ,&
&  0.89058E+00_JPRB, 0.92200E+00_JPRB, 0.92942E+00_JPRB, 0.91990E+00_JPRB, 0.91640E+00_JPRB, 0.97574E+00_JPRB, 0.10537E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11339E+01_JPRB, 0.11903E+01_JPRB, 0.12510E+01_JPRB, 0.13264E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.10626E+01_JPRB, 0.10903E+01_JPRB, 0.11570E+01_JPRB, 0.12736E+01_JPRB, 0.13854E+01_JPRB, 0.14978E+01_JPRB, 0.15845E+01_JPRB ,&
&  0.17216E+01_JPRB, 0.18406E+01_JPRB, 0.19698E+01_JPRB, 0.20676E+01_JPRB, 0.21497E+01_JPRB, 0.23244E+01_JPRB, 0.24352E+01_JPRB ,&
&  0.24592E+01_JPRB, 0.24138E+01_JPRB, 0.24159E+01_JPRB, 0.24664E+01_JPRB, 0.26535E+01_JPRB, 0.29893E+01_JPRB, 0.33890E+01_JPRB ,&
&  0.38047E+01_JPRB, 0.35732E+01_JPRB, 0.31435E+01_JPRB, 0.20174E+01_JPRB, 0.16926E+01_JPRB, 0.19929E+01_JPRB, 0.20147E+01_JPRB ,&
&  0.21063E+01_JPRB, 0.21351E+01_JPRB, 0.18379E+01_JPRB, 0.16402E+01_JPRB, 0.14579E+01_JPRB, 0.14058E+01_JPRB, 0.15484E+01_JPRB ,&
&  0.17472E+01_JPRB, 0.19820E+01_JPRB, 0.22385E+01_JPRB, 0.24270E+01_JPRB, 0.26313E+01_JPRB, 0.29135E+01_JPRB, 0.32271E+01_JPRB ,&
&  0.34968E+01_JPRB, 0.37101E+01_JPRB, 0.38680E+01_JPRB, 0.41314E+01_JPRB, 0.43219E+01_JPRB, 0.45877E+01_JPRB, 0.47959E+01_JPRB ,&
&  0.50268E+01_JPRB, 0.51726E+01_JPRB, 0.55509E+01_JPRB, 0.59704E+01_JPRB, 0.60476E+01_JPRB, 0.55743E+01_JPRB, 0.54553E+01_JPRB ,&
&  0.55210E+01_JPRB, 0.53364E+01_JPRB, 0.54389E+01_JPRB, 0.59823E+01_JPRB, 0.43703E+01_JPRB, 0.47143E+01_JPRB, 0.53944E+01_JPRB ,&
&  0.54493E+01_JPRB, 0.56599E+01_JPRB, 0.55197E+01_JPRB, 0.48873E+01_JPRB, 0.47181E+01_JPRB, 0.49568E+01_JPRB, 0.48639E+01_JPRB ,&
&  0.38342E+01_JPRB, 0.25296E+01_JPRB, 0.19245E+01_JPRB, 0.17538E+01_JPRB, 0.23335E+01_JPRB, 0.38870E+01_JPRB, 0.30105E+01_JPRB ,&
&  0.25588E+01_JPRB, 0.22887E+01_JPRB, 0.20917E+01_JPRB, 0.31633E+01_JPRB, 0.23028E+01_JPRB, 0.14190E+01_JPRB, 0.10365E+01_JPRB ,&
&  0.88386E+00_JPRB, 0.86763E+00_JPRB, 0.87298E+00_JPRB, 0.85663E+00_JPRB, 0.87808E+00_JPRB, 0.92830E+00_JPRB, 0.93944E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10249E+01_JPRB, 0.10620E+01_JPRB, 0.10934E+01_JPRB, 0.10600E+01_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.72302E+00_JPRB, 0.71574E+00_JPRB, 0.74827E+00_JPRB, 0.85126E+00_JPRB, 0.92990E+00_JPRB, 0.99467E+00_JPRB, 0.10917E+01_JPRB ,&
&  0.12138E+01_JPRB, 0.13895E+01_JPRB, 0.15327E+01_JPRB, 0.15815E+01_JPRB, 0.15315E+01_JPRB, 0.14791E+01_JPRB, 0.14670E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.13190E+01_JPRB, 0.11988E+01_JPRB, 0.11552E+01_JPRB, 0.12027E+01_JPRB, 0.13402E+01_JPRB, 0.15527E+01_JPRB ,&
&  0.19085E+01_JPRB, 0.21907E+01_JPRB, 0.24090E+01_JPRB, 0.30307E+01_JPRB, 0.29688E+01_JPRB, 0.18568E+01_JPRB, 0.21475E+01_JPRB ,&
&  0.29745E+01_JPRB, 0.29898E+01_JPRB, 0.19250E+01_JPRB, 0.16131E+01_JPRB, 0.14340E+01_JPRB, 0.13167E+01_JPRB, 0.13461E+01_JPRB ,&
&  0.14788E+01_JPRB, 0.16223E+01_JPRB, 0.18001E+01_JPRB, 0.20260E+01_JPRB, 0.22696E+01_JPRB, 0.24957E+01_JPRB, 0.27770E+01_JPRB ,&
&  0.30200E+01_JPRB, 0.31470E+01_JPRB, 0.32942E+01_JPRB, 0.35198E+01_JPRB, 0.38649E+01_JPRB, 0.42821E+01_JPRB, 0.46658E+01_JPRB ,&
&  0.49714E+01_JPRB, 0.51460E+01_JPRB, 0.55092E+01_JPRB, 0.57192E+01_JPRB, 0.52936E+01_JPRB, 0.48564E+01_JPRB, 0.44462E+01_JPRB ,&
&  0.41236E+01_JPRB, 0.36540E+01_JPRB, 0.33999E+01_JPRB, 0.44803E+01_JPRB, 0.47539E+01_JPRB, 0.43542E+01_JPRB, 0.43844E+01_JPRB ,&
&  0.44851E+01_JPRB, 0.44885E+01_JPRB, 0.40428E+01_JPRB, 0.31872E+01_JPRB, 0.25799E+01_JPRB, 0.29950E+01_JPRB, 0.32752E+01_JPRB ,&
&  0.28031E+01_JPRB, 0.20005E+01_JPRB, 0.16149E+01_JPRB, 0.15154E+01_JPRB, 0.16295E+01_JPRB, 0.26731E+01_JPRB, 0.21009E+01_JPRB ,&
&  0.15928E+01_JPRB, 0.17390E+01_JPRB, 0.16804E+01_JPRB, 0.24642E+01_JPRB, 0.31659E+01_JPRB, 0.17275E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.84863E+00_JPRB, 0.83055E+00_JPRB, 0.83049E+00_JPRB, 0.82887E+00_JPRB, 0.83238E+00_JPRB, 0.85299E+00_JPRB, 0.85518E+00_JPRB ,&
&  0.87053E+00_JPRB, 0.85390E+00_JPRB, 0.85397E+00_JPRB, 0.80699E+00_JPRB, 0.73746E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.73563E+00_JPRB, 0.73197E+00_JPRB, 0.71940E+00_JPRB, 0.71698E+00_JPRB, 0.73113E+00_JPRB, 0.75355E+00_JPRB, 0.78625E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.89391E+00_JPRB, 0.94709E+00_JPRB, 0.10045E+01_JPRB, 0.10103E+01_JPRB, 0.93273E+00_JPRB, 0.91599E+00_JPRB ,&
&  0.90455E+00_JPRB, 0.91047E+00_JPRB, 0.90225E+00_JPRB, 0.99402E+00_JPRB, 0.10134E+01_JPRB, 0.10206E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.13041E+01_JPRB, 0.14059E+01_JPRB, 0.15432E+01_JPRB, 0.24396E+01_JPRB, 0.23291E+01_JPRB, 0.18686E+01_JPRB ,&
&  0.27446E+01_JPRB, 0.27368E+01_JPRB, 0.19543E+01_JPRB, 0.14825E+01_JPRB, 0.13230E+01_JPRB, 0.11877E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.12972E+01_JPRB, 0.14811E+01_JPRB, 0.16679E+01_JPRB, 0.18192E+01_JPRB, 0.19646E+01_JPRB, 0.21366E+01_JPRB ,&
&  0.22557E+01_JPRB, 0.23754E+01_JPRB, 0.24068E+01_JPRB, 0.24812E+01_JPRB, 0.28881E+01_JPRB, 0.32755E+01_JPRB, 0.35760E+01_JPRB ,&
&  0.38174E+01_JPRB, 0.40633E+01_JPRB, 0.44408E+01_JPRB, 0.43129E+01_JPRB, 0.39593E+01_JPRB, 0.37200E+01_JPRB, 0.31659E+01_JPRB ,&
&  0.26245E+01_JPRB, 0.21319E+01_JPRB, 0.20012E+01_JPRB, 0.26733E+01_JPRB, 0.29162E+01_JPRB, 0.26746E+01_JPRB, 0.31706E+01_JPRB ,&
&  0.33333E+01_JPRB, 0.34329E+01_JPRB, 0.28760E+01_JPRB, 0.25033E+01_JPRB, 0.20613E+01_JPRB, 0.20922E+01_JPRB, 0.22981E+01_JPRB ,&
&  0.19058E+01_JPRB, 0.14464E+01_JPRB, 0.12625E+01_JPRB, 0.12099E+01_JPRB, 0.12514E+01_JPRB, 0.17351E+01_JPRB, 0.20087E+01_JPRB ,&
&  0.18824E+01_JPRB, 0.16239E+01_JPRB, 0.16285E+01_JPRB, 0.24299E+01_JPRB, 0.20026E+01_JPRB, 0.12195E+01_JPRB, 0.82472E+00_JPRB ,&
&  0.73558E+00_JPRB, 0.73987E+00_JPRB, 0.73257E+00_JPRB, 0.72797E+00_JPRB, 0.73880E+00_JPRB, 0.75856E+00_JPRB, 0.74353E+00_JPRB ,&
&  0.73972E+00_JPRB, 0.72224E+00_JPRB, 0.70980E+00_JPRB, 0.70057E+00_JPRB, 0.70641E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.89109E+00_JPRB, 0.90101E+00_JPRB, 0.91247E+00_JPRB, 0.92637E+00_JPRB, 0.95441E+00_JPRB, 0.98640E+00_JPRB, 0.10155E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.11075E+01_JPRB, 0.11437E+01_JPRB, 0.11659E+01_JPRB, 0.11827E+01_JPRB, 0.11948E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.12601E+01_JPRB, 0.12586E+01_JPRB, 0.12649E+01_JPRB, 0.12908E+01_JPRB, 0.13072E+01_JPRB, 0.12869E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.12386E+01_JPRB, 0.12673E+01_JPRB, 0.13136E+01_JPRB, 0.15396E+01_JPRB, 0.17119E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.20496E+01_JPRB, 0.22019E+01_JPRB, 0.20528E+01_JPRB, 0.14629E+01_JPRB, 0.13122E+01_JPRB, 0.12268E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.11767E+01_JPRB, 0.12252E+01_JPRB, 0.12811E+01_JPRB, 0.13512E+01_JPRB, 0.13796E+01_JPRB, 0.14402E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.14914E+01_JPRB, 0.15547E+01_JPRB, 0.16911E+01_JPRB, 0.19608E+01_JPRB, 0.23176E+01_JPRB, 0.23554E+01_JPRB ,&
&  0.23805E+01_JPRB, 0.25766E+01_JPRB, 0.24237E+01_JPRB, 0.23718E+01_JPRB, 0.22886E+01_JPRB, 0.19881E+01_JPRB, 0.16888E+01_JPRB ,&
&  0.16147E+01_JPRB, 0.15136E+01_JPRB, 0.12952E+01_JPRB, 0.16740E+01_JPRB, 0.19765E+01_JPRB, 0.20870E+01_JPRB, 0.24012E+01_JPRB ,&
&  0.25792E+01_JPRB, 0.22670E+01_JPRB, 0.18262E+01_JPRB, 0.18051E+01_JPRB, 0.17767E+01_JPRB, 0.14957E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.12014E+01_JPRB, 0.11654E+01_JPRB, 0.11964E+01_JPRB, 0.14928E+01_JPRB, 0.24970E+01_JPRB, 0.26667E+01_JPRB ,&
&  0.20514E+01_JPRB, 0.17388E+01_JPRB, 0.20564E+01_JPRB, 0.32118E+01_JPRB, 0.17478E+01_JPRB, 0.11395E+01_JPRB, 0.76345E+00_JPRB ,&
&  0.73965E+00_JPRB, 0.74410E+00_JPRB, 0.75001E+00_JPRB, 0.75641E+00_JPRB, 0.76572E+00_JPRB, 0.78594E+00_JPRB, 0.77726E+00_JPRB ,&
&  0.76828E+00_JPRB, 0.76118E+00_JPRB, 0.77553E+00_JPRB, 0.80017E+00_JPRB, 0.85060E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.10258E+01_JPRB, 0.10316E+01_JPRB, 0.10529E+01_JPRB, 0.10793E+01_JPRB, 0.11052E+01_JPRB, 0.11363E+01_JPRB, 0.11826E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.12762E+01_JPRB, 0.13145E+01_JPRB, 0.13431E+01_JPRB, 0.13518E+01_JPRB, 0.13675E+01_JPRB, 0.13857E+01_JPRB ,&
&  0.14052E+01_JPRB, 0.13978E+01_JPRB, 0.14213E+01_JPRB, 0.14600E+01_JPRB, 0.15097E+01_JPRB, 0.15303E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.15766E+01_JPRB, 0.16320E+01_JPRB, 0.16926E+01_JPRB, 0.17663E+01_JPRB, 0.18809E+01_JPRB, 0.20698E+01_JPRB, 0.22405E+01_JPRB ,&
&  0.19455E+01_JPRB, 0.23160E+01_JPRB, 0.19314E+01_JPRB, 0.15124E+01_JPRB, 0.14037E+01_JPRB, 0.14097E+01_JPRB, 0.13521E+01_JPRB ,&
&  0.13052E+01_JPRB, 0.13149E+01_JPRB, 0.13318E+01_JPRB, 0.13490E+01_JPRB, 0.13716E+01_JPRB, 0.14147E+01_JPRB, 0.14167E+01_JPRB ,&
&  0.14437E+01_JPRB, 0.14626E+01_JPRB, 0.14843E+01_JPRB, 0.15483E+01_JPRB, 0.15344E+01_JPRB, 0.18632E+01_JPRB, 0.17976E+01_JPRB ,&
&  0.16834E+01_JPRB, 0.16693E+01_JPRB, 0.13167E+01_JPRB, 0.13418E+01_JPRB, 0.13478E+01_JPRB, 0.11799E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.15058E+01_JPRB, 0.12800E+01_JPRB, 0.15504E+01_JPRB, 0.17305E+01_JPRB, 0.18164E+01_JPRB, 0.18734E+01_JPRB ,&
&  0.18455E+01_JPRB, 0.15965E+01_JPRB, 0.13824E+01_JPRB, 0.12886E+01_JPRB, 0.12578E+01_JPRB, 0.12027E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.10866E+01_JPRB, 0.10860E+01_JPRB, 0.11273E+01_JPRB, 0.17227E+01_JPRB, 0.23714E+01_JPRB, 0.34911E+01_JPRB ,&
&  0.18930E+01_JPRB, 0.16575E+01_JPRB, 0.13169E+01_JPRB, 0.19478E+01_JPRB, 0.13460E+01_JPRB, 0.83350E+00_JPRB, 0.80780E+00_JPRB ,&
&  0.86314E+00_JPRB, 0.90361E+00_JPRB, 0.90493E+00_JPRB, 0.89107E+00_JPRB, 0.89150E+00_JPRB, 0.89612E+00_JPRB, 0.87125E+00_JPRB ,&
&  0.85701E+00_JPRB, 0.87451E+00_JPRB, 0.91343E+00_JPRB, 0.93447E+00_JPRB, 0.98121E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.11010E+01_JPRB, 0.11342E+01_JPRB, 0.11697E+01_JPRB, 0.12080E+01_JPRB, 0.12447E+01_JPRB, 0.12796E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.13631E+01_JPRB, 0.13958E+01_JPRB, 0.14277E+01_JPRB, 0.14463E+01_JPRB, 0.14559E+01_JPRB, 0.14772E+01_JPRB, 0.14851E+01_JPRB ,&
&  0.14898E+01_JPRB, 0.14952E+01_JPRB, 0.15217E+01_JPRB, 0.15408E+01_JPRB, 0.15570E+01_JPRB, 0.15635E+01_JPRB, 0.15965E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.16609E+01_JPRB, 0.17049E+01_JPRB, 0.17617E+01_JPRB, 0.18100E+01_JPRB, 0.18376E+01_JPRB, 0.19978E+01_JPRB ,&
&  0.17705E+01_JPRB, 0.25710E+01_JPRB, 0.23059E+01_JPRB, 0.17603E+01_JPRB, 0.16636E+01_JPRB, 0.15390E+01_JPRB, 0.15044E+01_JPRB ,&
&  0.14941E+01_JPRB, 0.14878E+01_JPRB, 0.15102E+01_JPRB, 0.15569E+01_JPRB, 0.16009E+01_JPRB, 0.16495E+01_JPRB, 0.16713E+01_JPRB ,&
&  0.17030E+01_JPRB, 0.17312E+01_JPRB, 0.17603E+01_JPRB, 0.17799E+01_JPRB, 0.18145E+01_JPRB, 0.18348E+01_JPRB, 0.17837E+01_JPRB ,&
&  0.16617E+01_JPRB, 0.15493E+01_JPRB, 0.11512E+01_JPRB, 0.11036E+01_JPRB, 0.11573E+01_JPRB, 0.11012E+01_JPRB, 0.99720E+00_JPRB ,&
&  0.12561E+01_JPRB, 0.14249E+01_JPRB, 0.14253E+01_JPRB, 0.14957E+01_JPRB, 0.15175E+01_JPRB, 0.14500E+01_JPRB, 0.13973E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.11018E+01_JPRB, 0.10031E+01_JPRB, 0.94712E+00_JPRB, 0.96327E+00_JPRB, 0.96583E+00_JPRB, 0.95436E+00_JPRB ,&
&  0.92774E+00_JPRB, 0.96739E+00_JPRB, 0.10628E+01_JPRB, 0.12119E+01_JPRB, 0.15734E+01_JPRB, 0.31861E+01_JPRB, 0.33842E+01_JPRB ,&
&  0.15138E+01_JPRB, 0.13605E+01_JPRB, 0.88629E+00_JPRB, 0.10180E+01_JPRB, 0.10237E+01_JPRB, 0.94490E+00_JPRB, 0.10471E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.11757E+01_JPRB, 0.11057E+01_JPRB, 0.10166E+01_JPRB, 0.97986E+00_JPRB, 0.96735E+00_JPRB, 0.94122E+00_JPRB ,&
&  0.93966E+00_JPRB, 0.97288E+00_JPRB, 0.10056E+01_JPRB, 0.10347E+01_JPRB, 0.10672E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.11667E+01_JPRB, 0.11982E+01_JPRB, 0.12301E+01_JPRB, 0.12755E+01_JPRB, 0.13238E+01_JPRB, 0.13696E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.14748E+01_JPRB, 0.15082E+01_JPRB, 0.15345E+01_JPRB, 0.15550E+01_JPRB, 0.15724E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.15870E+01_JPRB, 0.16008E+01_JPRB, 0.16292E+01_JPRB, 0.16584E+01_JPRB, 0.16826E+01_JPRB, 0.17086E+01_JPRB, 0.17460E+01_JPRB ,&
&  0.17648E+01_JPRB, 0.17865E+01_JPRB, 0.18364E+01_JPRB, 0.19121E+01_JPRB, 0.20112E+01_JPRB, 0.22645E+01_JPRB, 0.21704E+01_JPRB ,&
&  0.24676E+01_JPRB, 0.27230E+01_JPRB, 0.25847E+01_JPRB, 0.19980E+01_JPRB, 0.17978E+01_JPRB, 0.17085E+01_JPRB, 0.16867E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.16453E+01_JPRB, 0.16443E+01_JPRB, 0.16834E+01_JPRB, 0.17471E+01_JPRB, 0.18199E+01_JPRB, 0.18576E+01_JPRB ,&
&  0.19068E+01_JPRB, 0.19639E+01_JPRB, 0.20281E+01_JPRB, 0.20874E+01_JPRB, 0.21848E+01_JPRB, 0.22284E+01_JPRB, 0.21891E+01_JPRB ,&
&  0.20887E+01_JPRB, 0.20148E+01_JPRB, 0.17243E+01_JPRB, 0.16181E+01_JPRB, 0.16511E+01_JPRB, 0.14669E+01_JPRB, 0.13125E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.12925E+01_JPRB, 0.12000E+01_JPRB, 0.13715E+01_JPRB, 0.13347E+01_JPRB, 0.12027E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.98316E+00_JPRB, 0.85292E+00_JPRB, 0.76759E+00_JPRB, 0.75474E+00_JPRB, 0.77623E+00_JPRB, 0.82727E+00_JPRB, 0.85338E+00_JPRB ,&
&  0.90771E+00_JPRB, 0.99826E+00_JPRB, 0.11069E+01_JPRB, 0.12872E+01_JPRB, 0.16858E+01_JPRB, 0.17285E+01_JPRB, 0.19839E+01_JPRB ,&
&  0.13823E+01_JPRB, 0.11379E+01_JPRB, 0.11649E+01_JPRB, 0.11007E+01_JPRB, 0.11484E+01_JPRB, 0.11617E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.14940E+01_JPRB, 0.16487E+01_JPRB, 0.15164E+01_JPRB, 0.13187E+01_JPRB, 0.11222E+01_JPRB, 0.10401E+01_JPRB, 0.10052E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.10493E+01_JPRB, 0.10822E+01_JPRB, 0.11051E+01_JPRB, 0.11345E+01_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.11716E+01_JPRB, 0.12041E+01_JPRB, 0.12314E+01_JPRB, 0.12660E+01_JPRB, 0.13077E+01_JPRB, 0.13545E+01_JPRB, 0.13920E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.14666E+01_JPRB, 0.15023E+01_JPRB, 0.15416E+01_JPRB, 0.15770E+01_JPRB, 0.16203E+01_JPRB, 0.16496E+01_JPRB ,&
&  0.16908E+01_JPRB, 0.17088E+01_JPRB, 0.17410E+01_JPRB, 0.17840E+01_JPRB, 0.18340E+01_JPRB, 0.18917E+01_JPRB, 0.19561E+01_JPRB ,&
&  0.19964E+01_JPRB, 0.20104E+01_JPRB, 0.20311E+01_JPRB, 0.20618E+01_JPRB, 0.20756E+01_JPRB, 0.22620E+01_JPRB, 0.22409E+01_JPRB ,&
&  0.28365E+01_JPRB, 0.30803E+01_JPRB, 0.28511E+01_JPRB, 0.22659E+01_JPRB, 0.20587E+01_JPRB, 0.19477E+01_JPRB, 0.19054E+01_JPRB ,&
&  0.18639E+01_JPRB, 0.18289E+01_JPRB, 0.17739E+01_JPRB, 0.17782E+01_JPRB, 0.18297E+01_JPRB, 0.19337E+01_JPRB, 0.20286E+01_JPRB ,&
&  0.20943E+01_JPRB, 0.21812E+01_JPRB, 0.22960E+01_JPRB, 0.24286E+01_JPRB, 0.25958E+01_JPRB, 0.26848E+01_JPRB, 0.27032E+01_JPRB ,&
&  0.26814E+01_JPRB, 0.26462E+01_JPRB, 0.24132E+01_JPRB, 0.24654E+01_JPRB, 0.25086E+01_JPRB, 0.20476E+01_JPRB, 0.16795E+01_JPRB ,&
&  0.14140E+01_JPRB, 0.12600E+01_JPRB, 0.12647E+01_JPRB, 0.12980E+01_JPRB, 0.11930E+01_JPRB, 0.11099E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.85812E+00_JPRB, 0.77137E+00_JPRB, 0.72759E+00_JPRB, 0.69276E+00_JPRB, 0.70360E+00_JPRB, 0.74542E+00_JPRB, 0.84273E+00_JPRB ,&
&  0.93066E+00_JPRB, 0.10306E+01_JPRB, 0.11657E+01_JPRB, 0.13089E+01_JPRB, 0.15193E+01_JPRB, 0.15430E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.14551E+01_JPRB, 0.13674E+01_JPRB, 0.13377E+01_JPRB, 0.99652E+00_JPRB, 0.12059E+01_JPRB, 0.13013E+01_JPRB, 0.14575E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.19775E+01_JPRB, 0.23225E+01_JPRB, 0.19600E+01_JPRB, 0.14654E+01_JPRB, 0.10890E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.10564E+01_JPRB, 0.10930E+01_JPRB, 0.11168E+01_JPRB, 0.11436E+01_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.11103E+01_JPRB, 0.11402E+01_JPRB, 0.11689E+01_JPRB, 0.11914E+01_JPRB, 0.12066E+01_JPRB, 0.12339E+01_JPRB, 0.12760E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.13570E+01_JPRB, 0.13995E+01_JPRB, 0.14427E+01_JPRB, 0.14882E+01_JPRB, 0.15368E+01_JPRB, 0.16014E+01_JPRB ,&
&  0.16884E+01_JPRB, 0.17742E+01_JPRB, 0.18456E+01_JPRB, 0.19471E+01_JPRB, 0.20435E+01_JPRB, 0.21349E+01_JPRB, 0.22791E+01_JPRB ,&
&  0.24110E+01_JPRB, 0.25485E+01_JPRB, 0.26572E+01_JPRB, 0.27238E+01_JPRB, 0.27171E+01_JPRB, 0.29153E+01_JPRB, 0.26778E+01_JPRB ,&
&  0.31534E+01_JPRB, 0.34022E+01_JPRB, 0.31036E+01_JPRB, 0.25158E+01_JPRB, 0.23249E+01_JPRB, 0.21863E+01_JPRB, 0.20721E+01_JPRB ,&
&  0.20166E+01_JPRB, 0.19245E+01_JPRB, 0.17730E+01_JPRB, 0.18029E+01_JPRB, 0.18500E+01_JPRB, 0.19515E+01_JPRB, 0.21016E+01_JPRB ,&
&  0.22199E+01_JPRB, 0.23511E+01_JPRB, 0.24905E+01_JPRB, 0.26391E+01_JPRB, 0.28305E+01_JPRB, 0.29845E+01_JPRB, 0.30959E+01_JPRB ,&
&  0.31788E+01_JPRB, 0.32367E+01_JPRB, 0.32802E+01_JPRB, 0.30734E+01_JPRB, 0.30390E+01_JPRB, 0.26948E+01_JPRB, 0.20647E+01_JPRB ,&
&  0.14130E+01_JPRB, 0.12289E+01_JPRB, 0.12527E+01_JPRB, 0.12510E+01_JPRB, 0.11675E+01_JPRB, 0.10180E+01_JPRB, 0.95268E+00_JPRB ,&
&  0.81162E+00_JPRB, 0.73064E+00_JPRB, 0.71147E+00_JPRB, 0.69570E+00_JPRB, 0.69605E+00_JPRB, 0.75524E+00_JPRB, 0.81694E+00_JPRB ,&
&  0.91920E+00_JPRB, 0.10539E+01_JPRB, 0.11895E+01_JPRB, 0.13404E+01_JPRB, 0.15932E+01_JPRB, 0.18193E+01_JPRB, 0.16097E+01_JPRB ,&
&  0.15315E+01_JPRB, 0.13315E+01_JPRB, 0.13079E+01_JPRB, 0.14167E+01_JPRB, 0.13172E+01_JPRB, 0.15264E+01_JPRB, 0.17527E+01_JPRB ,&
&  0.20853E+01_JPRB, 0.19854E+01_JPRB, 0.22495E+01_JPRB, 0.32639E+01_JPRB, 0.27702E+01_JPRB, 0.23768E+01_JPRB, 0.98415E+00_JPRB ,&
&  0.99531E+00_JPRB, 0.10277E+01_JPRB, 0.10716E+01_JPRB, 0.10886E+01_JPRB, 0.11049E+01_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.95908E+00_JPRB, 0.10087E+01_JPRB, 0.10288E+01_JPRB, 0.10772E+01_JPRB, 0.10895E+01_JPRB, 0.10939E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.11645E+01_JPRB, 0.11988E+01_JPRB, 0.12465E+01_JPRB, 0.12952E+01_JPRB, 0.13390E+01_JPRB, 0.13720E+01_JPRB, 0.14222E+01_JPRB ,&
&  0.14974E+01_JPRB, 0.15905E+01_JPRB, 0.17085E+01_JPRB, 0.18634E+01_JPRB, 0.19889E+01_JPRB, 0.21228E+01_JPRB, 0.22938E+01_JPRB ,&
&  0.24488E+01_JPRB, 0.26744E+01_JPRB, 0.28936E+01_JPRB, 0.31018E+01_JPRB, 0.32409E+01_JPRB, 0.32830E+01_JPRB, 0.25209E+01_JPRB ,&
&  0.34157E+01_JPRB, 0.33594E+01_JPRB, 0.31741E+01_JPRB, 0.27221E+01_JPRB, 0.25253E+01_JPRB, 0.23667E+01_JPRB, 0.22326E+01_JPRB ,&
&  0.21267E+01_JPRB, 0.19804E+01_JPRB, 0.18158E+01_JPRB, 0.17034E+01_JPRB, 0.18483E+01_JPRB, 0.18887E+01_JPRB, 0.20294E+01_JPRB ,&
&  0.21768E+01_JPRB, 0.23225E+01_JPRB, 0.24630E+01_JPRB, 0.26095E+01_JPRB, 0.28057E+01_JPRB, 0.30294E+01_JPRB, 0.32359E+01_JPRB ,&
&  0.34166E+01_JPRB, 0.35156E+01_JPRB, 0.36720E+01_JPRB, 0.32802E+01_JPRB, 0.31766E+01_JPRB, 0.29999E+01_JPRB, 0.20020E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.12563E+01_JPRB, 0.13266E+01_JPRB, 0.12998E+01_JPRB, 0.11694E+01_JPRB, 0.10082E+01_JPRB, 0.89525E+00_JPRB ,&
&  0.78642E+00_JPRB, 0.75765E+00_JPRB, 0.73563E+00_JPRB, 0.72323E+00_JPRB, 0.72532E+00_JPRB, 0.77207E+00_JPRB, 0.82165E+00_JPRB ,&
&  0.92330E+00_JPRB, 0.10086E+01_JPRB, 0.11082E+01_JPRB, 0.12113E+01_JPRB, 0.13627E+01_JPRB, 0.15476E+01_JPRB, 0.17242E+01_JPRB ,&
&  0.14366E+01_JPRB, 0.12552E+01_JPRB, 0.12118E+01_JPRB, 0.12495E+01_JPRB, 0.12676E+01_JPRB, 0.13501E+01_JPRB, 0.14552E+01_JPRB ,&
&  0.15997E+01_JPRB, 0.16991E+01_JPRB, 0.15375E+01_JPRB, 0.12764E+01_JPRB, 0.15732E+01_JPRB, 0.16065E+01_JPRB, 0.81603E+00_JPRB ,&
&  0.82059E+00_JPRB, 0.86558E+00_JPRB, 0.89870E+00_JPRB, 0.93380E+00_JPRB, 0.96589E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.78224E+00_JPRB, 0.79705E+00_JPRB, 0.80392E+00_JPRB, 0.85823E+00_JPRB, 0.87946E+00_JPRB, 0.91449E+00_JPRB, 0.94379E+00_JPRB ,&
&  0.98025E+00_JPRB, 0.10286E+01_JPRB, 0.10840E+01_JPRB, 0.11375E+01_JPRB, 0.11869E+01_JPRB, 0.12284E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.13635E+01_JPRB, 0.14310E+01_JPRB, 0.15242E+01_JPRB, 0.16675E+01_JPRB, 0.18340E+01_JPRB, 0.20178E+01_JPRB ,&
&  0.22115E+01_JPRB, 0.24575E+01_JPRB, 0.27315E+01_JPRB, 0.30278E+01_JPRB, 0.33324E+01_JPRB, 0.35499E+01_JPRB, 0.37093E+01_JPRB ,&
&  0.27328E+01_JPRB, 0.32461E+01_JPRB, 0.31539E+01_JPRB, 0.28629E+01_JPRB, 0.26740E+01_JPRB, 0.25710E+01_JPRB, 0.24338E+01_JPRB ,&
&  0.23043E+01_JPRB, 0.20817E+01_JPRB, 0.19469E+01_JPRB, 0.17992E+01_JPRB, 0.18005E+01_JPRB, 0.18199E+01_JPRB, 0.19129E+01_JPRB ,&
&  0.20554E+01_JPRB, 0.21678E+01_JPRB, 0.23031E+01_JPRB, 0.24361E+01_JPRB, 0.26084E+01_JPRB, 0.28812E+01_JPRB, 0.31937E+01_JPRB ,&
&  0.35307E+01_JPRB, 0.37945E+01_JPRB, 0.39889E+01_JPRB, 0.35178E+01_JPRB, 0.32345E+01_JPRB, 0.29736E+01_JPRB, 0.19574E+01_JPRB ,&
&  0.14070E+01_JPRB, 0.12410E+01_JPRB, 0.13217E+01_JPRB, 0.13627E+01_JPRB, 0.12000E+01_JPRB, 0.10528E+01_JPRB, 0.92174E+00_JPRB ,&
&  0.82405E+00_JPRB, 0.77039E+00_JPRB, 0.75786E+00_JPRB, 0.74306E+00_JPRB, 0.75168E+00_JPRB, 0.78675E+00_JPRB, 0.82252E+00_JPRB ,&
&  0.88907E+00_JPRB, 0.92338E+00_JPRB, 0.99559E+00_JPRB, 0.10551E+01_JPRB, 0.11170E+01_JPRB, 0.11611E+01_JPRB, 0.11940E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.12117E+01_JPRB, 0.12070E+01_JPRB, 0.12271E+01_JPRB, 0.12233E+01_JPRB, 0.12313E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.12956E+01_JPRB, 0.12523E+01_JPRB, 0.11977E+01_JPRB, 0.11081E+01_JPRB, 0.10314E+01_JPRB, 0.80491E+00_JPRB ,&
&  0.76607E+00_JPRB, 0.72724E+00_JPRB, 0.75537E+00_JPRB, 0.78842E+00_JPRB, 0.78409E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.69246E+00_JPRB, 0.68246E+00_JPRB, 0.67113E+00_JPRB, 0.68400E+00_JPRB, 0.69808E+00_JPRB, 0.73303E+00_JPRB, 0.73999E+00_JPRB ,&
&  0.76564E+00_JPRB, 0.83371E+00_JPRB, 0.92232E+00_JPRB, 0.98570E+00_JPRB, 0.10375E+01_JPRB, 0.10884E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11946E+01_JPRB, 0.12178E+01_JPRB, 0.12492E+01_JPRB, 0.13082E+01_JPRB, 0.14036E+01_JPRB, 0.15564E+01_JPRB ,&
&  0.17234E+01_JPRB, 0.19162E+01_JPRB, 0.21596E+01_JPRB, 0.24570E+01_JPRB, 0.27798E+01_JPRB, 0.29928E+01_JPRB, 0.31676E+01_JPRB ,&
&  0.18081E+01_JPRB, 0.27450E+01_JPRB, 0.33109E+01_JPRB, 0.30899E+01_JPRB, 0.28483E+01_JPRB, 0.27136E+01_JPRB, 0.25423E+01_JPRB ,&
&  0.22871E+01_JPRB, 0.20291E+01_JPRB, 0.18990E+01_JPRB, 0.19578E+01_JPRB, 0.17781E+01_JPRB, 0.17690E+01_JPRB, 0.17850E+01_JPRB ,&
&  0.18863E+01_JPRB, 0.19601E+01_JPRB, 0.19954E+01_JPRB, 0.20511E+01_JPRB, 0.21447E+01_JPRB, 0.23702E+01_JPRB, 0.27598E+01_JPRB ,&
&  0.32958E+01_JPRB, 0.38637E+01_JPRB, 0.41697E+01_JPRB, 0.35632E+01_JPRB, 0.36490E+01_JPRB, 0.33568E+01_JPRB, 0.27714E+01_JPRB ,&
&  0.17274E+01_JPRB, 0.13265E+01_JPRB, 0.13631E+01_JPRB, 0.13887E+01_JPRB, 0.12447E+01_JPRB, 0.11278E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.89971E+00_JPRB, 0.83884E+00_JPRB, 0.80775E+00_JPRB, 0.79027E+00_JPRB, 0.79118E+00_JPRB, 0.80681E+00_JPRB, 0.83414E+00_JPRB ,&
&  0.85551E+00_JPRB, 0.88126E+00_JPRB, 0.92243E+00_JPRB, 0.96819E+00_JPRB, 0.10198E+01_JPRB, 0.10660E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.11447E+01_JPRB, 0.11696E+01_JPRB, 0.11927E+01_JPRB, 0.12101E+01_JPRB, 0.12189E+01_JPRB, 0.12266E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.12108E+01_JPRB, 0.12113E+01_JPRB, 0.11898E+01_JPRB, 0.11278E+01_JPRB, 0.10677E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.92619E+00_JPRB, 0.81400E+00_JPRB, 0.72369E+00_JPRB, 0.70539E+00_JPRB, 0.69568E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.65626E+00_JPRB, 0.62827E+00_JPRB, 0.62255E+00_JPRB, 0.62997E+00_JPRB, 0.64880E+00_JPRB, 0.66123E+00_JPRB, 0.66300E+00_JPRB ,&
&  0.67018E+00_JPRB, 0.69072E+00_JPRB, 0.71487E+00_JPRB, 0.78365E+00_JPRB, 0.86643E+00_JPRB, 0.93529E+00_JPRB, 0.98272E+00_JPRB ,&
&  0.10200E+01_JPRB, 0.10331E+01_JPRB, 0.10286E+01_JPRB, 0.10304E+01_JPRB, 0.10362E+01_JPRB, 0.10625E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.12509E+01_JPRB, 0.13986E+01_JPRB, 0.15901E+01_JPRB, 0.18605E+01_JPRB, 0.22355E+01_JPRB, 0.26330E+01_JPRB, 0.29351E+01_JPRB ,&
&  0.21342E+01_JPRB, 0.10095E+01_JPRB, 0.29790E+01_JPRB, 0.35887E+01_JPRB, 0.31637E+01_JPRB, 0.29925E+01_JPRB, 0.26720E+01_JPRB ,&
&  0.22690E+01_JPRB, 0.20706E+01_JPRB, 0.19149E+01_JPRB, 0.19299E+01_JPRB, 0.17768E+01_JPRB, 0.17547E+01_JPRB, 0.17636E+01_JPRB ,&
&  0.17943E+01_JPRB, 0.18587E+01_JPRB, 0.18594E+01_JPRB, 0.18286E+01_JPRB, 0.18170E+01_JPRB, 0.18871E+01_JPRB, 0.21405E+01_JPRB ,&
&  0.26701E+01_JPRB, 0.34582E+01_JPRB, 0.41953E+01_JPRB, 0.37313E+01_JPRB, 0.38399E+01_JPRB, 0.30716E+01_JPRB, 0.22835E+01_JPRB ,&
&  0.17590E+01_JPRB, 0.14561E+01_JPRB, 0.14142E+01_JPRB, 0.14262E+01_JPRB, 0.13134E+01_JPRB, 0.11708E+01_JPRB, 0.11390E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.92008E+00_JPRB, 0.86420E+00_JPRB, 0.83700E+00_JPRB, 0.82912E+00_JPRB, 0.83690E+00_JPRB, 0.85139E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.87060E+00_JPRB, 0.89076E+00_JPRB, 0.92962E+00_JPRB, 0.97736E+00_JPRB, 0.10378E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.11293E+01_JPRB, 0.11502E+01_JPRB, 0.11617E+01_JPRB, 0.11722E+01_JPRB, 0.11508E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.11714E+01_JPRB, 0.11915E+01_JPRB, 0.12203E+01_JPRB, 0.11770E+01_JPRB, 0.11347E+01_JPRB, 0.11383E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10147E+01_JPRB, 0.86741E+00_JPRB, 0.73821E+00_JPRB, 0.68792E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.73058E+00_JPRB, 0.69344E+00_JPRB, 0.66191E+00_JPRB, 0.65491E+00_JPRB, 0.66919E+00_JPRB, 0.66520E+00_JPRB, 0.66313E+00_JPRB ,&
&  0.65830E+00_JPRB, 0.65437E+00_JPRB, 0.64148E+00_JPRB, 0.65419E+00_JPRB, 0.69839E+00_JPRB, 0.77106E+00_JPRB, 0.82522E+00_JPRB ,&
&  0.87106E+00_JPRB, 0.88525E+00_JPRB, 0.88263E+00_JPRB, 0.87820E+00_JPRB, 0.87303E+00_JPRB, 0.87396E+00_JPRB, 0.88777E+00_JPRB ,&
&  0.92991E+00_JPRB, 0.10009E+01_JPRB, 0.10983E+01_JPRB, 0.12306E+01_JPRB, 0.14341E+01_JPRB, 0.18143E+01_JPRB, 0.24122E+01_JPRB ,&
&  0.33787E+01_JPRB, 0.24077E+01_JPRB, 0.13934E+01_JPRB, 0.31566E+01_JPRB, 0.35547E+01_JPRB, 0.35100E+01_JPRB, 0.28898E+01_JPRB ,&
&  0.26249E+01_JPRB, 0.22829E+01_JPRB, 0.20069E+01_JPRB, 0.20058E+01_JPRB, 0.17938E+01_JPRB, 0.17409E+01_JPRB, 0.17605E+01_JPRB ,&
&  0.17698E+01_JPRB, 0.17798E+01_JPRB, 0.17861E+01_JPRB, 0.17459E+01_JPRB, 0.16754E+01_JPRB, 0.16318E+01_JPRB, 0.17042E+01_JPRB ,&
&  0.20305E+01_JPRB, 0.27077E+01_JPRB, 0.38061E+01_JPRB, 0.38258E+01_JPRB, 0.39181E+01_JPRB, 0.28323E+01_JPRB, 0.21379E+01_JPRB ,&
&  0.17926E+01_JPRB, 0.17270E+01_JPRB, 0.16506E+01_JPRB, 0.15094E+01_JPRB, 0.13408E+01_JPRB, 0.12351E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.10399E+01_JPRB, 0.96394E+00_JPRB, 0.91828E+00_JPRB, 0.89202E+00_JPRB, 0.88053E+00_JPRB, 0.87577E+00_JPRB ,&
&  0.88515E+00_JPRB, 0.88967E+00_JPRB, 0.90602E+00_JPRB, 0.92878E+00_JPRB, 0.95633E+00_JPRB, 0.99474E+00_JPRB, 0.10429E+01_JPRB ,&
&  0.10779E+01_JPRB, 0.10950E+01_JPRB, 0.11035E+01_JPRB, 0.11034E+01_JPRB, 0.10810E+01_JPRB, 0.10659E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.11061E+01_JPRB, 0.11064E+01_JPRB, 0.11401E+01_JPRB, 0.11574E+01_JPRB, 0.11629E+01_JPRB, 0.11826E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.11719E+01_JPRB, 0.10423E+01_JPRB, 0.82780E+00_JPRB, 0.76470E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.81854E+00_JPRB, 0.79166E+00_JPRB, 0.77546E+00_JPRB, 0.77470E+00_JPRB, 0.77489E+00_JPRB, 0.75751E+00_JPRB, 0.73332E+00_JPRB ,&
&  0.70134E+00_JPRB, 0.67739E+00_JPRB, 0.64443E+00_JPRB, 0.61251E+00_JPRB, 0.60786E+00_JPRB, 0.63213E+00_JPRB, 0.67696E+00_JPRB ,&
&  0.71669E+00_JPRB, 0.73178E+00_JPRB, 0.74171E+00_JPRB, 0.74892E+00_JPRB, 0.75616E+00_JPRB, 0.75953E+00_JPRB, 0.76056E+00_JPRB ,&
&  0.77209E+00_JPRB, 0.79644E+00_JPRB, 0.83307E+00_JPRB, 0.88196E+00_JPRB, 0.96007E+00_JPRB, 0.11106E+01_JPRB, 0.14153E+01_JPRB ,&
&  0.20645E+01_JPRB, 0.24495E+01_JPRB, 0.11169E+01_JPRB, 0.24346E+01_JPRB, 0.35614E+01_JPRB, 0.35904E+01_JPRB, 0.33723E+01_JPRB ,&
&  0.30554E+01_JPRB, 0.25549E+01_JPRB, 0.23410E+01_JPRB, 0.20849E+01_JPRB, 0.18359E+01_JPRB, 0.17391E+01_JPRB, 0.17061E+01_JPRB ,&
&  0.16937E+01_JPRB, 0.16379E+01_JPRB, 0.15966E+01_JPRB, 0.15817E+01_JPRB, 0.15296E+01_JPRB, 0.14635E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.16080E+01_JPRB, 0.20785E+01_JPRB, 0.31141E+01_JPRB, 0.36885E+01_JPRB, 0.38724E+01_JPRB, 0.30766E+01_JPRB, 0.23634E+01_JPRB ,&
&  0.20823E+01_JPRB, 0.21244E+01_JPRB, 0.19768E+01_JPRB, 0.16822E+01_JPRB, 0.14268E+01_JPRB, 0.13411E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.11411E+01_JPRB, 0.10820E+01_JPRB, 0.10330E+01_JPRB, 0.99492E+00_JPRB, 0.96014E+00_JPRB, 0.93665E+00_JPRB ,&
&  0.93555E+00_JPRB, 0.93493E+00_JPRB, 0.93820E+00_JPRB, 0.94355E+00_JPRB, 0.94852E+00_JPRB, 0.96205E+00_JPRB, 0.99141E+00_JPRB ,&
&  0.10284E+01_JPRB, 0.10508E+01_JPRB, 0.10634E+01_JPRB, 0.10462E+01_JPRB, 0.10330E+01_JPRB, 0.10351E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10911E+01_JPRB, 0.10613E+01_JPRB, 0.10671E+01_JPRB, 0.11170E+01_JPRB, 0.11289E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.12081E+01_JPRB, 0.12601E+01_JPRB, 0.12231E+01_JPRB, 0.92740E+00_JPRB, 0.86549E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.88914E+00_JPRB, 0.85826E+00_JPRB, 0.85344E+00_JPRB, 0.85476E+00_JPRB, 0.85339E+00_JPRB, 0.84311E+00_JPRB, 0.82030E+00_JPRB ,&
&  0.79096E+00_JPRB, 0.76717E+00_JPRB, 0.74020E+00_JPRB, 0.70503E+00_JPRB, 0.65483E+00_JPRB, 0.61238E+00_JPRB, 0.61118E+00_JPRB ,&
&  0.64095E+00_JPRB, 0.64801E+00_JPRB, 0.65332E+00_JPRB, 0.66141E+00_JPRB, 0.67265E+00_JPRB, 0.68600E+00_JPRB, 0.69556E+00_JPRB ,&
&  0.70085E+00_JPRB, 0.71150E+00_JPRB, 0.72687E+00_JPRB, 0.74635E+00_JPRB, 0.77948E+00_JPRB, 0.85003E+00_JPRB, 0.99278E+00_JPRB ,&
&  0.13827E+01_JPRB, 0.19896E+01_JPRB, 0.21596E+01_JPRB, 0.36256E+01_JPRB, 0.41531E+01_JPRB, 0.39598E+01_JPRB, 0.40058E+01_JPRB ,&
&  0.39317E+01_JPRB, 0.34503E+01_JPRB, 0.29138E+01_JPRB, 0.22587E+01_JPRB, 0.19173E+01_JPRB, 0.17588E+01_JPRB, 0.16642E+01_JPRB ,&
&  0.16066E+01_JPRB, 0.15348E+01_JPRB, 0.14349E+01_JPRB, 0.13792E+01_JPRB, 0.13360E+01_JPRB, 0.12955E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.13517E+01_JPRB, 0.16304E+01_JPRB, 0.23789E+01_JPRB, 0.35247E+01_JPRB, 0.38540E+01_JPRB, 0.34791E+01_JPRB, 0.29067E+01_JPRB ,&
&  0.29056E+01_JPRB, 0.27628E+01_JPRB, 0.24957E+01_JPRB, 0.20206E+01_JPRB, 0.16382E+01_JPRB, 0.16004E+01_JPRB, 0.14098E+01_JPRB ,&
&  0.12898E+01_JPRB, 0.12100E+01_JPRB, 0.11634E+01_JPRB, 0.11296E+01_JPRB, 0.11004E+01_JPRB, 0.10617E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.98317E+00_JPRB, 0.97056E+00_JPRB, 0.96103E+00_JPRB, 0.95630E+00_JPRB, 0.94990E+00_JPRB, 0.94673E+00_JPRB, 0.95332E+00_JPRB ,&
&  0.97977E+00_JPRB, 0.10030E+01_JPRB, 0.98296E+00_JPRB, 0.98132E+00_JPRB, 0.98777E+00_JPRB, 0.99079E+00_JPRB, 0.99277E+00_JPRB ,&
&  0.99820E+00_JPRB, 0.10293E+01_JPRB, 0.10501E+01_JPRB, 0.10572E+01_JPRB, 0.10770E+01_JPRB, 0.10978E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.11131E+01_JPRB, 0.10991E+01_JPRB, 0.10533E+01_JPRB, 0.97195E+00_JPRB, 0.93124E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.92174E+00_JPRB, 0.89348E+00_JPRB, 0.87819E+00_JPRB, 0.86997E+00_JPRB, 0.86558E+00_JPRB, 0.86041E+00_JPRB, 0.84223E+00_JPRB ,&
&  0.82158E+00_JPRB, 0.80829E+00_JPRB, 0.78688E+00_JPRB, 0.76427E+00_JPRB, 0.72075E+00_JPRB, 0.66498E+00_JPRB, 0.64060E+00_JPRB ,&
&  0.62877E+00_JPRB, 0.62563E+00_JPRB, 0.63429E+00_JPRB, 0.63935E+00_JPRB, 0.64301E+00_JPRB, 0.65113E+00_JPRB, 0.65871E+00_JPRB ,&
&  0.66746E+00_JPRB, 0.67255E+00_JPRB, 0.68181E+00_JPRB, 0.69206E+00_JPRB, 0.70838E+00_JPRB, 0.75841E+00_JPRB, 0.84663E+00_JPRB ,&
&  0.10898E+01_JPRB, 0.18011E+01_JPRB, 0.15663E+01_JPRB, 0.35272E+01_JPRB, 0.39731E+01_JPRB, 0.40164E+01_JPRB, 0.38131E+01_JPRB ,&
&  0.38644E+01_JPRB, 0.38471E+01_JPRB, 0.29265E+01_JPRB, 0.23360E+01_JPRB, 0.19467E+01_JPRB, 0.17508E+01_JPRB, 0.16359E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.14668E+01_JPRB, 0.13547E+01_JPRB, 0.12587E+01_JPRB, 0.12056E+01_JPRB, 0.11687E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.11782E+01_JPRB, 0.13272E+01_JPRB, 0.18176E+01_JPRB, 0.28498E+01_JPRB, 0.37346E+01_JPRB, 0.38332E+01_JPRB, 0.34498E+01_JPRB ,&
&  0.34716E+01_JPRB, 0.35472E+01_JPRB, 0.32262E+01_JPRB, 0.25419E+01_JPRB, 0.20517E+01_JPRB, 0.18129E+01_JPRB, 0.15660E+01_JPRB ,&
&  0.14065E+01_JPRB, 0.13088E+01_JPRB, 0.12524E+01_JPRB, 0.12045E+01_JPRB, 0.11628E+01_JPRB, 0.11191E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.97857E+00_JPRB, 0.96209E+00_JPRB, 0.95198E+00_JPRB, 0.94940E+00_JPRB, 0.94677E+00_JPRB, 0.94547E+00_JPRB ,&
&  0.95403E+00_JPRB, 0.97562E+00_JPRB, 0.94266E+00_JPRB, 0.93626E+00_JPRB, 0.94365E+00_JPRB, 0.94815E+00_JPRB, 0.96175E+00_JPRB ,&
&  0.99323E+00_JPRB, 0.10054E+01_JPRB, 0.10185E+01_JPRB, 0.10164E+01_JPRB, 0.10430E+01_JPRB, 0.10889E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10456E+01_JPRB, 0.10152E+01_JPRB, 0.97823E+00_JPRB, 0.94927E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.92409E+00_JPRB, 0.90359E+00_JPRB, 0.87991E+00_JPRB, 0.85864E+00_JPRB, 0.85400E+00_JPRB, 0.85179E+00_JPRB, 0.83725E+00_JPRB ,&
&  0.82288E+00_JPRB, 0.80601E+00_JPRB, 0.78141E+00_JPRB, 0.75289E+00_JPRB, 0.72001E+00_JPRB, 0.67977E+00_JPRB, 0.64392E+00_JPRB ,&
&  0.62808E+00_JPRB, 0.61425E+00_JPRB, 0.61546E+00_JPRB, 0.62234E+00_JPRB, 0.63204E+00_JPRB, 0.63846E+00_JPRB, 0.64600E+00_JPRB ,&
&  0.65324E+00_JPRB, 0.65804E+00_JPRB, 0.66374E+00_JPRB, 0.66911E+00_JPRB, 0.67618E+00_JPRB, 0.70778E+00_JPRB, 0.80265E+00_JPRB ,&
&  0.10062E+01_JPRB, 0.21584E+01_JPRB, 0.15929E+01_JPRB, 0.28920E+01_JPRB, 0.33137E+01_JPRB, 0.35490E+01_JPRB, 0.33849E+01_JPRB ,&
&  0.34474E+01_JPRB, 0.32443E+01_JPRB, 0.26826E+01_JPRB, 0.22540E+01_JPRB, 0.19441E+01_JPRB, 0.17460E+01_JPRB, 0.16106E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.14085E+01_JPRB, 0.12934E+01_JPRB, 0.11848E+01_JPRB, 0.11200E+01_JPRB, 0.10833E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.11369E+01_JPRB, 0.14170E+01_JPRB, 0.22830E+01_JPRB, 0.32434E+01_JPRB, 0.38214E+01_JPRB, 0.39269E+01_JPRB ,&
&  0.40014E+01_JPRB, 0.42812E+01_JPRB, 0.39299E+01_JPRB, 0.31593E+01_JPRB, 0.25532E+01_JPRB, 0.21163E+01_JPRB, 0.17839E+01_JPRB ,&
&  0.15804E+01_JPRB, 0.14595E+01_JPRB, 0.13782E+01_JPRB, 0.13084E+01_JPRB, 0.12364E+01_JPRB, 0.11571E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.97160E+00_JPRB, 0.94599E+00_JPRB, 0.94111E+00_JPRB, 0.94171E+00_JPRB, 0.94328E+00_JPRB, 0.94733E+00_JPRB ,&
&  0.95355E+00_JPRB, 0.96660E+00_JPRB, 0.93958E+00_JPRB, 0.92920E+00_JPRB, 0.93044E+00_JPRB, 0.94032E+00_JPRB, 0.94870E+00_JPRB ,&
&  0.99009E+00_JPRB, 0.99898E+00_JPRB, 0.99163E+00_JPRB, 0.99969E+00_JPRB, 0.10345E+01_JPRB, 0.11067E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10238E+01_JPRB, 0.99115E+00_JPRB, 0.96977E+00_JPRB, 0.94851E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.91920E+00_JPRB, 0.89614E+00_JPRB, 0.88154E+00_JPRB, 0.86816E+00_JPRB, 0.86385E+00_JPRB, 0.85744E+00_JPRB, 0.83910E+00_JPRB ,&
&  0.81311E+00_JPRB, 0.78316E+00_JPRB, 0.75230E+00_JPRB, 0.72627E+00_JPRB, 0.70560E+00_JPRB, 0.68204E+00_JPRB, 0.66254E+00_JPRB ,&
&  0.64617E+00_JPRB, 0.62728E+00_JPRB, 0.61515E+00_JPRB, 0.61002E+00_JPRB, 0.61279E+00_JPRB, 0.61832E+00_JPRB, 0.62650E+00_JPRB ,&
&  0.63643E+00_JPRB, 0.64303E+00_JPRB, 0.64460E+00_JPRB, 0.64739E+00_JPRB, 0.64841E+00_JPRB, 0.67706E+00_JPRB, 0.77559E+00_JPRB ,&
&  0.10174E+01_JPRB, 0.98218E+00_JPRB, 0.18294E+01_JPRB, 0.22652E+01_JPRB, 0.25729E+01_JPRB, 0.28818E+01_JPRB, 0.31079E+01_JPRB ,&
&  0.32178E+01_JPRB, 0.28635E+01_JPRB, 0.24069E+01_JPRB, 0.20921E+01_JPRB, 0.18399E+01_JPRB, 0.16604E+01_JPRB, 0.15252E+01_JPRB ,&
&  0.13944E+01_JPRB, 0.12970E+01_JPRB, 0.12176E+01_JPRB, 0.11198E+01_JPRB, 0.10524E+01_JPRB, 0.10202E+01_JPRB, 0.10053E+01_JPRB ,&
&  0.99247E+00_JPRB, 0.10153E+01_JPRB, 0.11558E+01_JPRB, 0.16282E+01_JPRB, 0.27414E+01_JPRB, 0.36019E+01_JPRB, 0.41909E+01_JPRB ,&
&  0.49268E+01_JPRB, 0.51611E+01_JPRB, 0.46178E+01_JPRB, 0.37658E+01_JPRB, 0.29404E+01_JPRB, 0.23887E+01_JPRB, 0.19417E+01_JPRB ,&
&  0.16854E+01_JPRB, 0.15422E+01_JPRB, 0.14570E+01_JPRB, 0.13855E+01_JPRB, 0.13107E+01_JPRB, 0.12089E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.10079E+01_JPRB, 0.95630E+00_JPRB, 0.92400E+00_JPRB, 0.90692E+00_JPRB, 0.90031E+00_JPRB, 0.90347E+00_JPRB, 0.91429E+00_JPRB ,&
&  0.93067E+00_JPRB, 0.95210E+00_JPRB, 0.93657E+00_JPRB, 0.92142E+00_JPRB, 0.92597E+00_JPRB, 0.94477E+00_JPRB, 0.96448E+00_JPRB ,&
&  0.97417E+00_JPRB, 0.96241E+00_JPRB, 0.94533E+00_JPRB, 0.96252E+00_JPRB, 0.10023E+01_JPRB, 0.10923E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.10253E+01_JPRB, 0.97975E+00_JPRB, 0.95974E+00_JPRB, 0.94712E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.90947E+00_JPRB, 0.88869E+00_JPRB, 0.88563E+00_JPRB, 0.89550E+00_JPRB, 0.90416E+00_JPRB, 0.88961E+00_JPRB, 0.84585E+00_JPRB ,&
&  0.80452E+00_JPRB, 0.76917E+00_JPRB, 0.73588E+00_JPRB, 0.70946E+00_JPRB, 0.69184E+00_JPRB, 0.67876E+00_JPRB, 0.66487E+00_JPRB ,&
&  0.65439E+00_JPRB, 0.64470E+00_JPRB, 0.62678E+00_JPRB, 0.61045E+00_JPRB, 0.60307E+00_JPRB, 0.60184E+00_JPRB, 0.60622E+00_JPRB ,&
&  0.61154E+00_JPRB, 0.61761E+00_JPRB, 0.62167E+00_JPRB, 0.62471E+00_JPRB, 0.62651E+00_JPRB, 0.65089E+00_JPRB, 0.73540E+00_JPRB ,&
&  0.10120E+01_JPRB, 0.15662E+01_JPRB, 0.18874E+01_JPRB, 0.19147E+01_JPRB, 0.20870E+01_JPRB, 0.23988E+01_JPRB, 0.27840E+01_JPRB ,&
&  0.27810E+01_JPRB, 0.25173E+01_JPRB, 0.21231E+01_JPRB, 0.18484E+01_JPRB, 0.16390E+01_JPRB, 0.14900E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.11609E+01_JPRB, 0.10856E+01_JPRB, 0.10190E+01_JPRB, 0.98912E+00_JPRB, 0.97894E+00_JPRB, 0.97393E+00_JPRB ,&
&  0.96662E+00_JPRB, 0.96017E+00_JPRB, 0.10156E+01_JPRB, 0.13302E+01_JPRB, 0.22832E+01_JPRB, 0.29180E+01_JPRB, 0.40639E+01_JPRB ,&
&  0.51583E+01_JPRB, 0.50703E+01_JPRB, 0.45578E+01_JPRB, 0.38355E+01_JPRB, 0.30513E+01_JPRB, 0.23971E+01_JPRB, 0.19888E+01_JPRB ,&
&  0.17330E+01_JPRB, 0.15301E+01_JPRB, 0.14040E+01_JPRB, 0.13227E+01_JPRB, 0.12560E+01_JPRB, 0.11930E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.97128E+00_JPRB, 0.92490E+00_JPRB, 0.89196E+00_JPRB, 0.87364E+00_JPRB, 0.86866E+00_JPRB, 0.87116E+00_JPRB ,&
&  0.87470E+00_JPRB, 0.89128E+00_JPRB, 0.91174E+00_JPRB, 0.90128E+00_JPRB, 0.91103E+00_JPRB, 0.93059E+00_JPRB, 0.94587E+00_JPRB ,&
&  0.92710E+00_JPRB, 0.91830E+00_JPRB, 0.90897E+00_JPRB, 0.92035E+00_JPRB, 0.10217E+01_JPRB, 0.11185E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10207E+01_JPRB, 0.98529E+00_JPRB, 0.95054E+00_JPRB, 0.94149E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.90089E+00_JPRB, 0.87346E+00_JPRB, 0.86173E+00_JPRB, 0.87782E+00_JPRB, 0.89939E+00_JPRB, 0.89148E+00_JPRB, 0.85102E+00_JPRB ,&
&  0.79637E+00_JPRB, 0.75184E+00_JPRB, 0.72045E+00_JPRB, 0.69830E+00_JPRB, 0.68134E+00_JPRB, 0.66668E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.63691E+00_JPRB, 0.62904E+00_JPRB, 0.62336E+00_JPRB, 0.61593E+00_JPRB, 0.60288E+00_JPRB, 0.59283E+00_JPRB, 0.58943E+00_JPRB ,&
&  0.58760E+00_JPRB, 0.58769E+00_JPRB, 0.59333E+00_JPRB, 0.60024E+00_JPRB, 0.60439E+00_JPRB, 0.61960E+00_JPRB, 0.67508E+00_JPRB ,&
&  0.87005E+00_JPRB, 0.13550E+01_JPRB, 0.13734E+01_JPRB, 0.15230E+01_JPRB, 0.18150E+01_JPRB, 0.20846E+01_JPRB, 0.23354E+01_JPRB ,&
&  0.23397E+01_JPRB, 0.22034E+01_JPRB, 0.19332E+01_JPRB, 0.16630E+01_JPRB, 0.14434E+01_JPRB, 0.13086E+01_JPRB, 0.12133E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.10531E+01_JPRB, 0.98446E+00_JPRB, 0.93325E+00_JPRB, 0.89834E+00_JPRB, 0.88311E+00_JPRB, 0.88335E+00_JPRB ,&
&  0.88446E+00_JPRB, 0.88190E+00_JPRB, 0.88926E+00_JPRB, 0.99837E+00_JPRB, 0.15123E+01_JPRB, 0.22099E+01_JPRB, 0.29898E+01_JPRB ,&
&  0.32336E+01_JPRB, 0.33439E+01_JPRB, 0.32914E+01_JPRB, 0.30851E+01_JPRB, 0.27340E+01_JPRB, 0.23903E+01_JPRB, 0.20201E+01_JPRB ,&
&  0.17477E+01_JPRB, 0.15607E+01_JPRB, 0.14240E+01_JPRB, 0.13087E+01_JPRB, 0.12053E+01_JPRB, 0.11229E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.98293E+00_JPRB, 0.94585E+00_JPRB, 0.91138E+00_JPRB, 0.88537E+00_JPRB, 0.87042E+00_JPRB, 0.86863E+00_JPRB ,&
&  0.86473E+00_JPRB, 0.85433E+00_JPRB, 0.85720E+00_JPRB, 0.86295E+00_JPRB, 0.87272E+00_JPRB, 0.87626E+00_JPRB, 0.87640E+00_JPRB ,&
&  0.88180E+00_JPRB, 0.90243E+00_JPRB, 0.89854E+00_JPRB, 0.91677E+00_JPRB, 0.99854E+00_JPRB, 0.10731E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.10223E+01_JPRB, 0.98764E+00_JPRB, 0.96181E+00_JPRB, 0.94287E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.99958E+00_JPRB, 0.90749E+00_JPRB, 0.85299E+00_JPRB, 0.83474E+00_JPRB, 0.83633E+00_JPRB, 0.82256E+00_JPRB, 0.79631E+00_JPRB ,&
&  0.76806E+00_JPRB, 0.74318E+00_JPRB, 0.72206E+00_JPRB, 0.69890E+00_JPRB, 0.67589E+00_JPRB, 0.65621E+00_JPRB, 0.63705E+00_JPRB ,&
&  0.61837E+00_JPRB, 0.60722E+00_JPRB, 0.59942E+00_JPRB, 0.59395E+00_JPRB, 0.58983E+00_JPRB, 0.58229E+00_JPRB, 0.57438E+00_JPRB ,&
&  0.56835E+00_JPRB, 0.55989E+00_JPRB, 0.55735E+00_JPRB, 0.56465E+00_JPRB, 0.57333E+00_JPRB, 0.58624E+00_JPRB, 0.62751E+00_JPRB ,&
&  0.77704E+00_JPRB, 0.99259E+00_JPRB, 0.11453E+01_JPRB, 0.12841E+01_JPRB, 0.14274E+01_JPRB, 0.15937E+01_JPRB, 0.16478E+01_JPRB ,&
&  0.15658E+01_JPRB, 0.15742E+01_JPRB, 0.15320E+01_JPRB, 0.13954E+01_JPRB, 0.12415E+01_JPRB, 0.11374E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.10006E+01_JPRB, 0.93780E+00_JPRB, 0.87595E+00_JPRB, 0.82693E+00_JPRB, 0.79732E+00_JPRB, 0.78538E+00_JPRB, 0.78438E+00_JPRB ,&
&  0.78820E+00_JPRB, 0.79521E+00_JPRB, 0.79936E+00_JPRB, 0.81519E+00_JPRB, 0.93143E+00_JPRB, 0.12436E+01_JPRB, 0.17361E+01_JPRB ,&
&  0.19779E+01_JPRB, 0.20535E+01_JPRB, 0.21072E+01_JPRB, 0.21078E+01_JPRB, 0.19790E+01_JPRB, 0.18395E+01_JPRB, 0.16893E+01_JPRB ,&
&  0.15384E+01_JPRB, 0.13790E+01_JPRB, 0.12426E+01_JPRB, 0.11352E+01_JPRB, 0.10516E+01_JPRB, 0.99506E+00_JPRB, 0.96453E+00_JPRB ,&
&  0.95029E+00_JPRB, 0.94192E+00_JPRB, 0.93392E+00_JPRB, 0.91836E+00_JPRB, 0.89780E+00_JPRB, 0.88251E+00_JPRB, 0.87202E+00_JPRB ,&
&  0.86040E+00_JPRB, 0.85069E+00_JPRB, 0.84343E+00_JPRB, 0.83811E+00_JPRB, 0.83730E+00_JPRB, 0.83548E+00_JPRB, 0.83706E+00_JPRB ,&
&  0.84215E+00_JPRB, 0.85780E+00_JPRB, 0.85473E+00_JPRB, 0.89621E+00_JPRB, 0.97163E+00_JPRB, 0.99630E+00_JPRB, 0.10246E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10336E+01_JPRB, 0.99030E+00_JPRB, 0.97234E+00_JPRB, 0.10666E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.91198E+00_JPRB, 0.87015E+00_JPRB, 0.81648E+00_JPRB, 0.78446E+00_JPRB, 0.76286E+00_JPRB, 0.73549E+00_JPRB, 0.71457E+00_JPRB ,&
&  0.69859E+00_JPRB, 0.68464E+00_JPRB, 0.66831E+00_JPRB, 0.64914E+00_JPRB, 0.63081E+00_JPRB, 0.61755E+00_JPRB, 0.60601E+00_JPRB ,&
&  0.59471E+00_JPRB, 0.58374E+00_JPRB, 0.57366E+00_JPRB, 0.56607E+00_JPRB, 0.55776E+00_JPRB, 0.55120E+00_JPRB, 0.54557E+00_JPRB ,&
&  0.53895E+00_JPRB, 0.53163E+00_JPRB, 0.52242E+00_JPRB, 0.52630E+00_JPRB, 0.53794E+00_JPRB, 0.55433E+00_JPRB, 0.57660E+00_JPRB ,&
&  0.66505E+00_JPRB, 0.76768E+00_JPRB, 0.90288E+00_JPRB, 0.10512E+01_JPRB, 0.11615E+01_JPRB, 0.12423E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.11687E+01_JPRB, 0.11775E+01_JPRB, 0.11489E+01_JPRB, 0.10899E+01_JPRB, 0.10471E+01_JPRB, 0.95850E+00_JPRB, 0.87761E+00_JPRB ,&
&  0.82041E+00_JPRB, 0.78334E+00_JPRB, 0.75328E+00_JPRB, 0.72417E+00_JPRB, 0.70707E+00_JPRB, 0.70342E+00_JPRB, 0.70954E+00_JPRB ,&
&  0.71610E+00_JPRB, 0.72508E+00_JPRB, 0.73327E+00_JPRB, 0.74299E+00_JPRB, 0.76138E+00_JPRB, 0.85331E+00_JPRB, 0.10909E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.13755E+01_JPRB, 0.14430E+01_JPRB, 0.14689E+01_JPRB, 0.14278E+01_JPRB, 0.13539E+01_JPRB, 0.12661E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.10734E+01_JPRB, 0.98133E+00_JPRB, 0.90884E+00_JPRB, 0.85213E+00_JPRB, 0.81408E+00_JPRB, 0.79329E+00_JPRB ,&
&  0.78949E+00_JPRB, 0.79845E+00_JPRB, 0.81372E+00_JPRB, 0.82872E+00_JPRB, 0.83690E+00_JPRB, 0.83640E+00_JPRB, 0.83146E+00_JPRB ,&
&  0.82285E+00_JPRB, 0.81185E+00_JPRB, 0.80061E+00_JPRB, 0.79200E+00_JPRB, 0.78539E+00_JPRB, 0.78134E+00_JPRB, 0.77997E+00_JPRB ,&
&  0.78083E+00_JPRB, 0.78658E+00_JPRB, 0.79699E+00_JPRB, 0.83891E+00_JPRB, 0.85841E+00_JPRB, 0.88302E+00_JPRB, 0.90210E+00_JPRB ,&
&  0.92338E+00_JPRB, 0.92987E+00_JPRB, 0.91407E+00_JPRB, 0.87956E+00_JPRB, 0.88022E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.79768E+00_JPRB, 0.78436E+00_JPRB, 0.75649E+00_JPRB, 0.72659E+00_JPRB, 0.70135E+00_JPRB, 0.67794E+00_JPRB, 0.65791E+00_JPRB ,&
&  0.64187E+00_JPRB, 0.62742E+00_JPRB, 0.61232E+00_JPRB, 0.59835E+00_JPRB, 0.58452E+00_JPRB, 0.57299E+00_JPRB, 0.56191E+00_JPRB ,&
&  0.55393E+00_JPRB, 0.54803E+00_JPRB, 0.54222E+00_JPRB, 0.53557E+00_JPRB, 0.52891E+00_JPRB, 0.52238E+00_JPRB, 0.51516E+00_JPRB ,&
&  0.50914E+00_JPRB, 0.50572E+00_JPRB, 0.49769E+00_JPRB, 0.49554E+00_JPRB, 0.50456E+00_JPRB, 0.51910E+00_JPRB, 0.53385E+00_JPRB ,&
&  0.59232E+00_JPRB, 0.60536E+00_JPRB, 0.63448E+00_JPRB, 0.76667E+00_JPRB, 0.87812E+00_JPRB, 0.99932E+00_JPRB, 0.10496E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.10638E+01_JPRB, 0.10265E+01_JPRB, 0.97680E+00_JPRB, 0.90050E+00_JPRB, 0.83735E+00_JPRB, 0.79082E+00_JPRB ,&
&  0.74991E+00_JPRB, 0.71132E+00_JPRB, 0.68140E+00_JPRB, 0.65878E+00_JPRB, 0.64272E+00_JPRB, 0.63666E+00_JPRB, 0.63949E+00_JPRB ,&
&  0.64416E+00_JPRB, 0.64885E+00_JPRB, 0.65530E+00_JPRB, 0.66692E+00_JPRB, 0.68144E+00_JPRB, 0.70462E+00_JPRB, 0.78511E+00_JPRB ,&
&  0.91282E+00_JPRB, 0.10067E+01_JPRB, 0.10863E+01_JPRB, 0.11288E+01_JPRB, 0.11204E+01_JPRB, 0.10717E+01_JPRB, 0.98725E+00_JPRB ,&
&  0.89876E+00_JPRB, 0.82980E+00_JPRB, 0.78602E+00_JPRB, 0.75777E+00_JPRB, 0.74098E+00_JPRB, 0.72791E+00_JPRB, 0.71602E+00_JPRB ,&
&  0.70512E+00_JPRB, 0.69809E+00_JPRB, 0.69814E+00_JPRB, 0.70524E+00_JPRB, 0.71680E+00_JPRB, 0.72660E+00_JPRB, 0.73056E+00_JPRB ,&
&  0.72988E+00_JPRB, 0.72680E+00_JPRB, 0.72484E+00_JPRB, 0.72234E+00_JPRB, 0.71861E+00_JPRB, 0.71320E+00_JPRB, 0.71115E+00_JPRB ,&
&  0.71289E+00_JPRB, 0.71911E+00_JPRB, 0.72728E+00_JPRB, 0.76144E+00_JPRB, 0.79205E+00_JPRB, 0.80096E+00_JPRB, 0.80600E+00_JPRB ,&
&  0.81228E+00_JPRB, 0.81016E+00_JPRB, 0.80554E+00_JPRB, 0.71765E+00_JPRB, 0.78336E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.68728E+00_JPRB, 0.68388E+00_JPRB, 0.66631E+00_JPRB, 0.64699E+00_JPRB, 0.62738E+00_JPRB, 0.60840E+00_JPRB, 0.59391E+00_JPRB ,&
&  0.58271E+00_JPRB, 0.56905E+00_JPRB, 0.55722E+00_JPRB, 0.54910E+00_JPRB, 0.54057E+00_JPRB, 0.53263E+00_JPRB, 0.52354E+00_JPRB ,&
&  0.51615E+00_JPRB, 0.51134E+00_JPRB, 0.50775E+00_JPRB, 0.50375E+00_JPRB, 0.49932E+00_JPRB, 0.49415E+00_JPRB, 0.48914E+00_JPRB ,&
&  0.48446E+00_JPRB, 0.48116E+00_JPRB, 0.47606E+00_JPRB, 0.47302E+00_JPRB, 0.47736E+00_JPRB, 0.48805E+00_JPRB, 0.49832E+00_JPRB ,&
&  0.54856E+00_JPRB, 0.50552E+00_JPRB, 0.52038E+00_JPRB, 0.58003E+00_JPRB, 0.65390E+00_JPRB, 0.75050E+00_JPRB, 0.83093E+00_JPRB ,&
&  0.86478E+00_JPRB, 0.89549E+00_JPRB, 0.87786E+00_JPRB, 0.84486E+00_JPRB, 0.78970E+00_JPRB, 0.73645E+00_JPRB, 0.69682E+00_JPRB ,&
&  0.67377E+00_JPRB, 0.65625E+00_JPRB, 0.63629E+00_JPRB, 0.61450E+00_JPRB, 0.59483E+00_JPRB, 0.58147E+00_JPRB, 0.57589E+00_JPRB ,&
&  0.57719E+00_JPRB, 0.57977E+00_JPRB, 0.58042E+00_JPRB, 0.58439E+00_JPRB, 0.59453E+00_JPRB, 0.60981E+00_JPRB, 0.63549E+00_JPRB ,&
&  0.68406E+00_JPRB, 0.75020E+00_JPRB, 0.80614E+00_JPRB, 0.85734E+00_JPRB, 0.88619E+00_JPRB, 0.88850E+00_JPRB, 0.84714E+00_JPRB ,&
&  0.79260E+00_JPRB, 0.74324E+00_JPRB, 0.70976E+00_JPRB, 0.69053E+00_JPRB, 0.67783E+00_JPRB, 0.66795E+00_JPRB, 0.65478E+00_JPRB ,&
&  0.63843E+00_JPRB, 0.62079E+00_JPRB, 0.60602E+00_JPRB, 0.59325E+00_JPRB, 0.58507E+00_JPRB, 0.58361E+00_JPRB, 0.58700E+00_JPRB ,&
&  0.59169E+00_JPRB, 0.59550E+00_JPRB, 0.59925E+00_JPRB, 0.60432E+00_JPRB, 0.60906E+00_JPRB, 0.61387E+00_JPRB, 0.61766E+00_JPRB ,&
&  0.62334E+00_JPRB, 0.63084E+00_JPRB, 0.63807E+00_JPRB, 0.66144E+00_JPRB, 0.68063E+00_JPRB, 0.69083E+00_JPRB, 0.69734E+00_JPRB ,&
&  0.69946E+00_JPRB, 0.70571E+00_JPRB, 0.61625E+00_JPRB, 0.65879E+00_JPRB, 0.67814E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.60137E+00_JPRB, 0.59936E+00_JPRB, 0.59301E+00_JPRB, 0.57925E+00_JPRB, 0.56349E+00_JPRB, 0.54969E+00_JPRB, 0.54074E+00_JPRB ,&
&  0.53457E+00_JPRB, 0.52630E+00_JPRB, 0.51918E+00_JPRB, 0.51406E+00_JPRB, 0.50786E+00_JPRB, 0.50061E+00_JPRB, 0.49096E+00_JPRB ,&
&  0.48283E+00_JPRB, 0.47645E+00_JPRB, 0.47190E+00_JPRB, 0.46915E+00_JPRB, 0.46744E+00_JPRB, 0.46568E+00_JPRB, 0.46303E+00_JPRB ,&
&  0.46045E+00_JPRB, 0.45908E+00_JPRB, 0.45771E+00_JPRB, 0.45675E+00_JPRB, 0.45793E+00_JPRB, 0.46473E+00_JPRB, 0.47361E+00_JPRB ,&
&  0.47431E+00_JPRB, 0.45525E+00_JPRB, 0.46841E+00_JPRB, 0.49532E+00_JPRB, 0.53380E+00_JPRB, 0.59962E+00_JPRB, 0.66893E+00_JPRB ,&
&  0.71032E+00_JPRB, 0.73320E+00_JPRB, 0.71602E+00_JPRB, 0.69240E+00_JPRB, 0.66794E+00_JPRB, 0.63906E+00_JPRB, 0.61534E+00_JPRB ,&
&  0.59644E+00_JPRB, 0.58377E+00_JPRB, 0.57356E+00_JPRB, 0.55941E+00_JPRB, 0.54280E+00_JPRB, 0.52935E+00_JPRB, 0.52135E+00_JPRB ,&
&  0.51861E+00_JPRB, 0.52049E+00_JPRB, 0.52334E+00_JPRB, 0.52593E+00_JPRB, 0.53067E+00_JPRB, 0.54016E+00_JPRB, 0.55339E+00_JPRB ,&
&  0.56666E+00_JPRB, 0.58825E+00_JPRB, 0.61816E+00_JPRB, 0.64702E+00_JPRB, 0.67347E+00_JPRB, 0.69008E+00_JPRB, 0.69699E+00_JPRB ,&
&  0.69149E+00_JPRB, 0.67400E+00_JPRB, 0.65226E+00_JPRB, 0.62996E+00_JPRB, 0.60515E+00_JPRB, 0.57666E+00_JPRB, 0.54752E+00_JPRB ,&
&  0.52338E+00_JPRB, 0.50498E+00_JPRB, 0.49270E+00_JPRB, 0.48708E+00_JPRB, 0.48531E+00_JPRB, 0.48755E+00_JPRB, 0.49221E+00_JPRB ,&
&  0.49804E+00_JPRB, 0.50432E+00_JPRB, 0.50918E+00_JPRB, 0.51494E+00_JPRB, 0.52053E+00_JPRB, 0.52689E+00_JPRB, 0.53361E+00_JPRB ,&
&  0.54117E+00_JPRB, 0.54837E+00_JPRB, 0.55556E+00_JPRB, 0.57082E+00_JPRB, 0.58681E+00_JPRB, 0.60093E+00_JPRB, 0.60847E+00_JPRB ,&
&  0.60733E+00_JPRB, 0.60558E+00_JPRB, 0.60407E+00_JPRB, 0.60070E+00_JPRB, 0.60317E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.54006E+00_JPRB, 0.53746E+00_JPRB, 0.53459E+00_JPRB, 0.53056E+00_JPRB, 0.52415E+00_JPRB, 0.51536E+00_JPRB, 0.50838E+00_JPRB ,&
&  0.50358E+00_JPRB, 0.49821E+00_JPRB, 0.49327E+00_JPRB, 0.48729E+00_JPRB, 0.48021E+00_JPRB, 0.47179E+00_JPRB, 0.46128E+00_JPRB ,&
&  0.45180E+00_JPRB, 0.44328E+00_JPRB, 0.43767E+00_JPRB, 0.43492E+00_JPRB, 0.43467E+00_JPRB, 0.43449E+00_JPRB, 0.43245E+00_JPRB ,&
&  0.43114E+00_JPRB, 0.43098E+00_JPRB, 0.43099E+00_JPRB, 0.43072E+00_JPRB, 0.43117E+00_JPRB, 0.43610E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.45879E+00_JPRB, 0.44123E+00_JPRB, 0.45760E+00_JPRB, 0.46405E+00_JPRB, 0.47876E+00_JPRB, 0.51439E+00_JPRB, 0.56351E+00_JPRB ,&
&  0.60379E+00_JPRB, 0.60665E+00_JPRB, 0.58865E+00_JPRB, 0.56892E+00_JPRB, 0.55992E+00_JPRB, 0.54860E+00_JPRB, 0.53544E+00_JPRB ,&
&  0.52300E+00_JPRB, 0.51412E+00_JPRB, 0.50684E+00_JPRB, 0.49995E+00_JPRB, 0.49096E+00_JPRB, 0.47982E+00_JPRB, 0.46956E+00_JPRB ,&
&  0.46276E+00_JPRB, 0.46065E+00_JPRB, 0.46243E+00_JPRB, 0.46613E+00_JPRB, 0.47023E+00_JPRB, 0.47711E+00_JPRB, 0.48612E+00_JPRB ,&
&  0.49507E+00_JPRB, 0.49910E+00_JPRB, 0.50255E+00_JPRB, 0.51044E+00_JPRB, 0.51975E+00_JPRB, 0.52648E+00_JPRB, 0.52939E+00_JPRB ,&
&  0.52889E+00_JPRB, 0.52289E+00_JPRB, 0.51353E+00_JPRB, 0.50146E+00_JPRB, 0.48823E+00_JPRB, 0.47499E+00_JPRB, 0.46347E+00_JPRB ,&
&  0.45426E+00_JPRB, 0.44631E+00_JPRB, 0.43940E+00_JPRB, 0.43449E+00_JPRB, 0.43235E+00_JPRB, 0.43226E+00_JPRB, 0.43393E+00_JPRB ,&
&  0.43810E+00_JPRB, 0.44331E+00_JPRB, 0.44774E+00_JPRB, 0.45202E+00_JPRB, 0.45590E+00_JPRB, 0.45929E+00_JPRB, 0.46397E+00_JPRB ,&
&  0.47048E+00_JPRB, 0.47551E+00_JPRB, 0.47981E+00_JPRB, 0.48683E+00_JPRB, 0.49792E+00_JPRB, 0.51158E+00_JPRB, 0.52347E+00_JPRB ,&
&  0.53044E+00_JPRB, 0.53243E+00_JPRB, 0.53545E+00_JPRB, 0.53711E+00_JPRB, 0.54071E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.49757E+00_JPRB, 0.49620E+00_JPRB, 0.49366E+00_JPRB, 0.49187E+00_JPRB, 0.48910E+00_JPRB, 0.48476E+00_JPRB, 0.47972E+00_JPRB ,&
&  0.47543E+00_JPRB, 0.47123E+00_JPRB, 0.46647E+00_JPRB, 0.46027E+00_JPRB, 0.45203E+00_JPRB, 0.44400E+00_JPRB, 0.43509E+00_JPRB ,&
&  0.42678E+00_JPRB, 0.41965E+00_JPRB, 0.41464E+00_JPRB, 0.41163E+00_JPRB, 0.40918E+00_JPRB, 0.40694E+00_JPRB, 0.40433E+00_JPRB ,&
&  0.40271E+00_JPRB, 0.40203E+00_JPRB, 0.40208E+00_JPRB, 0.40268E+00_JPRB, 0.40351E+00_JPRB, 0.40694E+00_JPRB, 0.40943E+00_JPRB ,&
&  0.41604E+00_JPRB, 0.43644E+00_JPRB, 0.44821E+00_JPRB, 0.45227E+00_JPRB, 0.45850E+00_JPRB, 0.47095E+00_JPRB, 0.48898E+00_JPRB ,&
&  0.50586E+00_JPRB, 0.51321E+00_JPRB, 0.50434E+00_JPRB, 0.48995E+00_JPRB, 0.48159E+00_JPRB, 0.47815E+00_JPRB, 0.47007E+00_JPRB ,&
&  0.46175E+00_JPRB, 0.45529E+00_JPRB, 0.45025E+00_JPRB, 0.44639E+00_JPRB, 0.44267E+00_JPRB, 0.43744E+00_JPRB, 0.43151E+00_JPRB ,&
&  0.42436E+00_JPRB, 0.41874E+00_JPRB, 0.41466E+00_JPRB, 0.41321E+00_JPRB, 0.41352E+00_JPRB, 0.41546E+00_JPRB, 0.41901E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.42977E+00_JPRB, 0.43174E+00_JPRB, 0.43201E+00_JPRB, 0.43232E+00_JPRB, 0.43359E+00_JPRB, 0.43417E+00_JPRB ,&
&  0.43299E+00_JPRB, 0.43023E+00_JPRB, 0.42620E+00_JPRB, 0.42109E+00_JPRB, 0.41536E+00_JPRB, 0.41045E+00_JPRB, 0.40637E+00_JPRB ,&
&  0.40268E+00_JPRB, 0.39899E+00_JPRB, 0.39542E+00_JPRB, 0.39337E+00_JPRB, 0.39216E+00_JPRB, 0.39168E+00_JPRB, 0.39270E+00_JPRB ,&
&  0.39531E+00_JPRB, 0.39859E+00_JPRB, 0.40231E+00_JPRB, 0.40584E+00_JPRB, 0.40923E+00_JPRB, 0.41206E+00_JPRB, 0.41552E+00_JPRB ,&
&  0.42112E+00_JPRB, 0.42737E+00_JPRB, 0.43345E+00_JPRB, 0.43916E+00_JPRB, 0.44666E+00_JPRB, 0.45740E+00_JPRB, 0.46924E+00_JPRB ,&
&  0.47742E+00_JPRB, 0.48201E+00_JPRB, 0.48638E+00_JPRB, 0.49152E+00_JPRB, 0.49611E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.44924E+00_JPRB, 0.45126E+00_JPRB, 0.45208E+00_JPRB, 0.45325E+00_JPRB, 0.45339E+00_JPRB, 0.45182E+00_JPRB, 0.44802E+00_JPRB ,&
&  0.44350E+00_JPRB, 0.43835E+00_JPRB, 0.43301E+00_JPRB, 0.42625E+00_JPRB, 0.41921E+00_JPRB, 0.41334E+00_JPRB, 0.40705E+00_JPRB ,&
&  0.40071E+00_JPRB, 0.39488E+00_JPRB, 0.39066E+00_JPRB, 0.38728E+00_JPRB, 0.38394E+00_JPRB, 0.38054E+00_JPRB, 0.37631E+00_JPRB ,&
&  0.37263E+00_JPRB, 0.36936E+00_JPRB, 0.36953E+00_JPRB, 0.37226E+00_JPRB, 0.37464E+00_JPRB, 0.37644E+00_JPRB, 0.37821E+00_JPRB ,&
&  0.37962E+00_JPRB, 0.40138E+00_JPRB, 0.39753E+00_JPRB, 0.41309E+00_JPRB, 0.41874E+00_JPRB, 0.41955E+00_JPRB, 0.42095E+00_JPRB ,&
&  0.42574E+00_JPRB, 0.43197E+00_JPRB, 0.43653E+00_JPRB, 0.43106E+00_JPRB, 0.42255E+00_JPRB, 0.41615E+00_JPRB, 0.40974E+00_JPRB ,&
&  0.40130E+00_JPRB, 0.39741E+00_JPRB, 0.39675E+00_JPRB, 0.39667E+00_JPRB, 0.39663E+00_JPRB, 0.39604E+00_JPRB, 0.39446E+00_JPRB ,&
&  0.39179E+00_JPRB, 0.38798E+00_JPRB, 0.38361E+00_JPRB, 0.37940E+00_JPRB, 0.37660E+00_JPRB, 0.37395E+00_JPRB, 0.37201E+00_JPRB ,&
&  0.37196E+00_JPRB, 0.37382E+00_JPRB, 0.37547E+00_JPRB, 0.37580E+00_JPRB, 0.37480E+00_JPRB, 0.37380E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.37207E+00_JPRB, 0.37050E+00_JPRB, 0.36844E+00_JPRB, 0.36583E+00_JPRB, 0.36305E+00_JPRB, 0.36114E+00_JPRB, 0.36020E+00_JPRB ,&
&  0.35912E+00_JPRB, 0.35703E+00_JPRB, 0.35420E+00_JPRB, 0.35353E+00_JPRB, 0.35352E+00_JPRB, 0.35389E+00_JPRB, 0.35447E+00_JPRB ,&
&  0.35575E+00_JPRB, 0.35819E+00_JPRB, 0.36108E+00_JPRB, 0.36406E+00_JPRB, 0.36714E+00_JPRB, 0.37144E+00_JPRB, 0.37632E+00_JPRB ,&
&  0.38165E+00_JPRB, 0.38735E+00_JPRB, 0.39289E+00_JPRB, 0.39806E+00_JPRB, 0.40281E+00_JPRB, 0.40903E+00_JPRB, 0.41750E+00_JPRB ,&
&  0.42497E+00_JPRB, 0.43068E+00_JPRB, 0.43486E+00_JPRB, 0.43973E+00_JPRB, 0.44510E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.40870E+00_JPRB, 0.41414E+00_JPRB, 0.41739E+00_JPRB, 0.41915E+00_JPRB, 0.41980E+00_JPRB, 0.41827E+00_JPRB, 0.41517E+00_JPRB ,&
&  0.41018E+00_JPRB, 0.40496E+00_JPRB, 0.39942E+00_JPRB, 0.39345E+00_JPRB, 0.38805E+00_JPRB, 0.38338E+00_JPRB, 0.37919E+00_JPRB ,&
&  0.37496E+00_JPRB, 0.37104E+00_JPRB, 0.36781E+00_JPRB, 0.36458E+00_JPRB, 0.36081E+00_JPRB, 0.35605E+00_JPRB, 0.35016E+00_JPRB ,&
&  0.34641E+00_JPRB, 0.34351E+00_JPRB, 0.34423E+00_JPRB, 0.34738E+00_JPRB, 0.34997E+00_JPRB, 0.35149E+00_JPRB, 0.35309E+00_JPRB ,&
&  0.35548E+00_JPRB, 0.36173E+00_JPRB, 0.36693E+00_JPRB, 0.37329E+00_JPRB, 0.37759E+00_JPRB, 0.37758E+00_JPRB, 0.37493E+00_JPRB ,&
&  0.37350E+00_JPRB, 0.37494E+00_JPRB, 0.37756E+00_JPRB, 0.37923E+00_JPRB, 0.37783E+00_JPRB, 0.37408E+00_JPRB, 0.36814E+00_JPRB ,&
&  0.35876E+00_JPRB, 0.35185E+00_JPRB, 0.35035E+00_JPRB, 0.35184E+00_JPRB, 0.35418E+00_JPRB, 0.35593E+00_JPRB, 0.35694E+00_JPRB ,&
&  0.35717E+00_JPRB, 0.35621E+00_JPRB, 0.35455E+00_JPRB, 0.35244E+00_JPRB, 0.34982E+00_JPRB, 0.34712E+00_JPRB, 0.34542E+00_JPRB ,&
&  0.34386E+00_JPRB, 0.34285E+00_JPRB, 0.34250E+00_JPRB, 0.34227E+00_JPRB, 0.34094E+00_JPRB, 0.33873E+00_JPRB, 0.33717E+00_JPRB ,&
&  0.33526E+00_JPRB, 0.33392E+00_JPRB, 0.33217E+00_JPRB, 0.33034E+00_JPRB, 0.32825E+00_JPRB, 0.32633E+00_JPRB, 0.32547E+00_JPRB ,&
&  0.32534E+00_JPRB, 0.32401E+00_JPRB, 0.32250E+00_JPRB, 0.32251E+00_JPRB, 0.32326E+00_JPRB, 0.32430E+00_JPRB, 0.32505E+00_JPRB ,&
&  0.32577E+00_JPRB, 0.32673E+00_JPRB, 0.32902E+00_JPRB, 0.33179E+00_JPRB, 0.33505E+00_JPRB, 0.33900E+00_JPRB, 0.34348E+00_JPRB ,&
&  0.34863E+00_JPRB, 0.35445E+00_JPRB, 0.36089E+00_JPRB, 0.36683E+00_JPRB, 0.37192E+00_JPRB, 0.37582E+00_JPRB, 0.38112E+00_JPRB ,&
&  0.38742E+00_JPRB, 0.39203E+00_JPRB, 0.39560E+00_JPRB, 0.39927E+00_JPRB, 0.40328E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.36913E+00_JPRB, 0.37470E+00_JPRB, 0.37899E+00_JPRB, 0.38147E+00_JPRB, 0.38176E+00_JPRB, 0.38045E+00_JPRB, 0.37738E+00_JPRB ,&
&  0.37279E+00_JPRB, 0.36806E+00_JPRB, 0.36341E+00_JPRB, 0.35964E+00_JPRB, 0.35711E+00_JPRB, 0.35491E+00_JPRB, 0.35290E+00_JPRB ,&
&  0.35029E+00_JPRB, 0.34731E+00_JPRB, 0.34390E+00_JPRB, 0.34089E+00_JPRB, 0.33721E+00_JPRB, 0.33224E+00_JPRB, 0.32840E+00_JPRB ,&
&  0.32562E+00_JPRB, 0.32315E+00_JPRB, 0.32240E+00_JPRB, 0.32384E+00_JPRB, 0.32647E+00_JPRB, 0.32894E+00_JPRB, 0.33189E+00_JPRB ,&
&  0.33424E+00_JPRB, 0.33686E+00_JPRB, 0.33938E+00_JPRB, 0.33994E+00_JPRB, 0.33992E+00_JPRB, 0.33968E+00_JPRB, 0.33909E+00_JPRB ,&
&  0.33849E+00_JPRB, 0.33796E+00_JPRB, 0.33705E+00_JPRB, 0.33711E+00_JPRB, 0.33703E+00_JPRB, 0.33566E+00_JPRB, 0.33314E+00_JPRB ,&
&  0.32890E+00_JPRB, 0.32312E+00_JPRB, 0.31873E+00_JPRB, 0.31677E+00_JPRB, 0.31636E+00_JPRB, 0.31683E+00_JPRB, 0.31781E+00_JPRB ,&
&  0.31901E+00_JPRB, 0.32075E+00_JPRB, 0.32244E+00_JPRB, 0.32327E+00_JPRB, 0.32303E+00_JPRB, 0.32268E+00_JPRB, 0.32113E+00_JPRB ,&
&  0.31964E+00_JPRB, 0.31861E+00_JPRB, 0.31800E+00_JPRB, 0.31734E+00_JPRB, 0.31490E+00_JPRB, 0.31177E+00_JPRB, 0.31068E+00_JPRB ,&
&  0.30861E+00_JPRB, 0.30724E+00_JPRB, 0.30536E+00_JPRB, 0.30414E+00_JPRB, 0.30283E+00_JPRB, 0.30114E+00_JPRB, 0.29945E+00_JPRB ,&
&  0.29822E+00_JPRB, 0.29754E+00_JPRB, 0.29721E+00_JPRB, 0.29643E+00_JPRB, 0.29683E+00_JPRB, 0.29754E+00_JPRB, 0.29906E+00_JPRB ,&
&  0.30038E+00_JPRB, 0.30158E+00_JPRB, 0.30334E+00_JPRB, 0.30530E+00_JPRB, 0.30821E+00_JPRB, 0.31161E+00_JPRB, 0.31551E+00_JPRB ,&
&  0.31896E+00_JPRB, 0.32351E+00_JPRB, 0.32859E+00_JPRB, 0.33340E+00_JPRB, 0.33721E+00_JPRB, 0.34011E+00_JPRB, 0.34426E+00_JPRB ,&
&  0.35065E+00_JPRB, 0.35582E+00_JPRB, 0.35857E+00_JPRB, 0.36159E+00_JPRB, 0.36447E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.33839E+00_JPRB, 0.34049E+00_JPRB, 0.34254E+00_JPRB, 0.34471E+00_JPRB, 0.34624E+00_JPRB, 0.34678E+00_JPRB, 0.34535E+00_JPRB ,&
&  0.34246E+00_JPRB, 0.33903E+00_JPRB, 0.33556E+00_JPRB, 0.33260E+00_JPRB, 0.33074E+00_JPRB, 0.32948E+00_JPRB, 0.32851E+00_JPRB ,&
&  0.32758E+00_JPRB, 0.32663E+00_JPRB, 0.32425E+00_JPRB, 0.32095E+00_JPRB, 0.31663E+00_JPRB, 0.31282E+00_JPRB, 0.31136E+00_JPRB ,&
&  0.30935E+00_JPRB, 0.30798E+00_JPRB, 0.30701E+00_JPRB, 0.30600E+00_JPRB, 0.30554E+00_JPRB, 0.30647E+00_JPRB, 0.30852E+00_JPRB ,&
&  0.31069E+00_JPRB, 0.31274E+00_JPRB, 0.31454E+00_JPRB, 0.31532E+00_JPRB, 0.31462E+00_JPRB, 0.31372E+00_JPRB, 0.31257E+00_JPRB ,&
&  0.31108E+00_JPRB, 0.30947E+00_JPRB, 0.30800E+00_JPRB, 0.30610E+00_JPRB, 0.30337E+00_JPRB, 0.29999E+00_JPRB, 0.29766E+00_JPRB ,&
&  0.29599E+00_JPRB, 0.29437E+00_JPRB, 0.29266E+00_JPRB, 0.29181E+00_JPRB, 0.29116E+00_JPRB, 0.29092E+00_JPRB, 0.29107E+00_JPRB ,&
&  0.29165E+00_JPRB, 0.29306E+00_JPRB, 0.29509E+00_JPRB, 0.29736E+00_JPRB, 0.29822E+00_JPRB, 0.29852E+00_JPRB, 0.29867E+00_JPRB ,&
&  0.29771E+00_JPRB, 0.29603E+00_JPRB, 0.29389E+00_JPRB, 0.29149E+00_JPRB, 0.28972E+00_JPRB, 0.28832E+00_JPRB, 0.28676E+00_JPRB ,&
&  0.28436E+00_JPRB, 0.28286E+00_JPRB, 0.28211E+00_JPRB, 0.28193E+00_JPRB, 0.28209E+00_JPRB, 0.28206E+00_JPRB, 0.28118E+00_JPRB ,&
&  0.28011E+00_JPRB, 0.27924E+00_JPRB, 0.27821E+00_JPRB, 0.27698E+00_JPRB, 0.27497E+00_JPRB, 0.27549E+00_JPRB, 0.27679E+00_JPRB ,&
&  0.27733E+00_JPRB, 0.27744E+00_JPRB, 0.27765E+00_JPRB, 0.27908E+00_JPRB, 0.28074E+00_JPRB, 0.28291E+00_JPRB, 0.28577E+00_JPRB ,&
&  0.28828E+00_JPRB, 0.29171E+00_JPRB, 0.29654E+00_JPRB, 0.30108E+00_JPRB, 0.30430E+00_JPRB, 0.30599E+00_JPRB, 0.30945E+00_JPRB ,&
&  0.31650E+00_JPRB, 0.32375E+00_JPRB, 0.32825E+00_JPRB, 0.33253E+00_JPRB, 0.33647E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.31083E+00_JPRB, 0.31202E+00_JPRB, 0.31304E+00_JPRB, 0.31425E+00_JPRB, 0.31585E+00_JPRB, 0.31681E+00_JPRB, 0.31755E+00_JPRB ,&
&  0.31682E+00_JPRB, 0.31481E+00_JPRB, 0.31243E+00_JPRB, 0.30998E+00_JPRB, 0.30881E+00_JPRB, 0.30850E+00_JPRB, 0.30844E+00_JPRB ,&
&  0.30778E+00_JPRB, 0.30742E+00_JPRB, 0.30535E+00_JPRB, 0.30195E+00_JPRB, 0.29787E+00_JPRB, 0.29495E+00_JPRB, 0.29400E+00_JPRB ,&
&  0.29325E+00_JPRB, 0.29218E+00_JPRB, 0.29153E+00_JPRB, 0.29050E+00_JPRB, 0.28757E+00_JPRB, 0.28617E+00_JPRB, 0.28661E+00_JPRB ,&
&  0.28755E+00_JPRB, 0.28923E+00_JPRB, 0.28938E+00_JPRB, 0.28723E+00_JPRB, 0.28947E+00_JPRB, 0.28940E+00_JPRB, 0.28754E+00_JPRB ,&
&  0.28510E+00_JPRB, 0.28257E+00_JPRB, 0.28028E+00_JPRB, 0.27786E+00_JPRB, 0.27530E+00_JPRB, 0.27337E+00_JPRB, 0.27216E+00_JPRB ,&
&  0.27194E+00_JPRB, 0.27240E+00_JPRB, 0.27314E+00_JPRB, 0.27425E+00_JPRB, 0.27498E+00_JPRB, 0.27543E+00_JPRB, 0.27559E+00_JPRB ,&
&  0.27586E+00_JPRB, 0.27622E+00_JPRB, 0.27678E+00_JPRB, 0.27715E+00_JPRB, 0.27753E+00_JPRB, 0.27775E+00_JPRB, 0.27741E+00_JPRB ,&
&  0.27591E+00_JPRB, 0.27359E+00_JPRB, 0.27069E+00_JPRB, 0.26942E+00_JPRB, 0.26915E+00_JPRB, 0.26703E+00_JPRB, 0.26409E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.26125E+00_JPRB, 0.26289E+00_JPRB, 0.26439E+00_JPRB, 0.26559E+00_JPRB, 0.26607E+00_JPRB, 0.26584E+00_JPRB ,&
&  0.26479E+00_JPRB, 0.26200E+00_JPRB, 0.25883E+00_JPRB, 0.25616E+00_JPRB, 0.25441E+00_JPRB, 0.25423E+00_JPRB, 0.25535E+00_JPRB ,&
&  0.25566E+00_JPRB, 0.25594E+00_JPRB, 0.25721E+00_JPRB, 0.25840E+00_JPRB, 0.25936E+00_JPRB, 0.26041E+00_JPRB, 0.26206E+00_JPRB ,&
&  0.26426E+00_JPRB, 0.26804E+00_JPRB, 0.27278E+00_JPRB, 0.27824E+00_JPRB, 0.28191E+00_JPRB, 0.28121E+00_JPRB, 0.28251E+00_JPRB ,&
&  0.28828E+00_JPRB, 0.29485E+00_JPRB, 0.30004E+00_JPRB, 0.30409E+00_JPRB, 0.30787E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.28883E+00_JPRB, 0.29019E+00_JPRB, 0.29119E+00_JPRB, 0.29195E+00_JPRB, 0.29342E+00_JPRB, 0.29457E+00_JPRB, 0.29603E+00_JPRB ,&
&  0.29706E+00_JPRB, 0.29566E+00_JPRB, 0.29384E+00_JPRB, 0.29227E+00_JPRB, 0.29151E+00_JPRB, 0.29052E+00_JPRB, 0.28964E+00_JPRB ,&
&  0.29004E+00_JPRB, 0.28915E+00_JPRB, 0.28617E+00_JPRB, 0.28373E+00_JPRB, 0.27996E+00_JPRB, 0.27756E+00_JPRB, 0.27723E+00_JPRB ,&
&  0.27888E+00_JPRB, 0.27835E+00_JPRB, 0.27713E+00_JPRB, 0.27554E+00_JPRB, 0.27363E+00_JPRB, 0.27259E+00_JPRB, 0.27143E+00_JPRB ,&
&  0.27074E+00_JPRB, 0.26934E+00_JPRB, 0.26481E+00_JPRB, 0.26526E+00_JPRB, 0.26505E+00_JPRB, 0.26536E+00_JPRB, 0.26501E+00_JPRB ,&
&  0.26450E+00_JPRB, 0.26351E+00_JPRB, 0.26166E+00_JPRB, 0.25956E+00_JPRB, 0.25783E+00_JPRB, 0.25695E+00_JPRB, 0.25705E+00_JPRB ,&
&  0.25776E+00_JPRB, 0.25926E+00_JPRB, 0.26142E+00_JPRB, 0.26377E+00_JPRB, 0.26478E+00_JPRB, 0.26558E+00_JPRB, 0.26685E+00_JPRB ,&
&  0.26749E+00_JPRB, 0.26767E+00_JPRB, 0.26764E+00_JPRB, 0.26736E+00_JPRB, 0.26665E+00_JPRB, 0.26533E+00_JPRB, 0.26420E+00_JPRB ,&
&  0.26184E+00_JPRB, 0.25882E+00_JPRB, 0.25667E+00_JPRB, 0.25410E+00_JPRB, 0.24715E+00_JPRB, 0.23681E+00_JPRB, 0.23108E+00_JPRB ,&
&  0.23176E+00_JPRB, 0.23672E+00_JPRB, 0.24161E+00_JPRB, 0.24550E+00_JPRB, 0.25217E+00_JPRB, 0.25223E+00_JPRB, 0.24756E+00_JPRB ,&
&  0.23707E+00_JPRB, 0.23321E+00_JPRB, 0.22987E+00_JPRB, 0.22476E+00_JPRB, 0.22170E+00_JPRB, 0.22142E+00_JPRB, 0.22170E+00_JPRB ,&
&  0.22397E+00_JPRB, 0.22671E+00_JPRB, 0.22995E+00_JPRB, 0.23190E+00_JPRB, 0.23225E+00_JPRB, 0.23131E+00_JPRB, 0.22986E+00_JPRB ,&
&  0.23020E+00_JPRB, 0.23513E+00_JPRB, 0.24445E+00_JPRB, 0.25284E+00_JPRB, 0.25799E+00_JPRB, 0.26015E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26709E+00_JPRB, 0.27314E+00_JPRB, 0.27893E+00_JPRB, 0.28297E+00_JPRB, 0.28625E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.26700E+00_JPRB, 0.26816E+00_JPRB, 0.26924E+00_JPRB, 0.27033E+00_JPRB, 0.27153E+00_JPRB, 0.27282E+00_JPRB, 0.27394E+00_JPRB ,&
&  0.27531E+00_JPRB, 0.27619E+00_JPRB, 0.27631E+00_JPRB, 0.27509E+00_JPRB, 0.27415E+00_JPRB, 0.27367E+00_JPRB, 0.27378E+00_JPRB ,&
&  0.27137E+00_JPRB, 0.26816E+00_JPRB, 0.26667E+00_JPRB, 0.26625E+00_JPRB, 0.26546E+00_JPRB, 0.26518E+00_JPRB, 0.26612E+00_JPRB ,&
&  0.26704E+00_JPRB, 0.26632E+00_JPRB, 0.26568E+00_JPRB, 0.26399E+00_JPRB, 0.26267E+00_JPRB, 0.26170E+00_JPRB, 0.26045E+00_JPRB ,&
&  0.25597E+00_JPRB, 0.24563E+00_JPRB, 0.23971E+00_JPRB, 0.24447E+00_JPRB, 0.25058E+00_JPRB, 0.24969E+00_JPRB, 0.24998E+00_JPRB ,&
&  0.24977E+00_JPRB, 0.24851E+00_JPRB, 0.24708E+00_JPRB, 0.24582E+00_JPRB, 0.24553E+00_JPRB, 0.24624E+00_JPRB, 0.24769E+00_JPRB ,&
&  0.24944E+00_JPRB, 0.25264E+00_JPRB, 0.25580E+00_JPRB, 0.25700E+00_JPRB, 0.25828E+00_JPRB, 0.25903E+00_JPRB, 0.25923E+00_JPRB ,&
&  0.25840E+00_JPRB, 0.25682E+00_JPRB, 0.25542E+00_JPRB, 0.25399E+00_JPRB, 0.25252E+00_JPRB, 0.25057E+00_JPRB, 0.24538E+00_JPRB ,&
&  0.24007E+00_JPRB, 0.23425E+00_JPRB, 0.22728E+00_JPRB, 0.22028E+00_JPRB, 0.21433E+00_JPRB, 0.21001E+00_JPRB, 0.20613E+00_JPRB ,&
&  0.20589E+00_JPRB, 0.21086E+00_JPRB, 0.22014E+00_JPRB, 0.23039E+00_JPRB, 0.23276E+00_JPRB, 0.22409E+00_JPRB, 0.21836E+00_JPRB ,&
&  0.21220E+00_JPRB, 0.20694E+00_JPRB, 0.20365E+00_JPRB, 0.20185E+00_JPRB, 0.20044E+00_JPRB, 0.20023E+00_JPRB, 0.20088E+00_JPRB ,&
&  0.20258E+00_JPRB, 0.20438E+00_JPRB, 0.20602E+00_JPRB, 0.20720E+00_JPRB, 0.20770E+00_JPRB, 0.20759E+00_JPRB, 0.20719E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.21109E+00_JPRB, 0.21601E+00_JPRB, 0.22162E+00_JPRB, 0.22645E+00_JPRB, 0.23110E+00_JPRB, 0.23723E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.25024E+00_JPRB, 0.25565E+00_JPRB, 0.26158E+00_JPRB, 0.26501E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.24906E+00_JPRB, 0.25161E+00_JPRB, 0.25232E+00_JPRB, 0.25371E+00_JPRB, 0.25396E+00_JPRB, 0.25500E+00_JPRB, 0.25555E+00_JPRB ,&
&  0.25768E+00_JPRB, 0.25968E+00_JPRB, 0.26008E+00_JPRB, 0.25816E+00_JPRB, 0.25662E+00_JPRB, 0.25280E+00_JPRB, 0.25109E+00_JPRB ,&
&  0.25027E+00_JPRB, 0.25110E+00_JPRB, 0.25119E+00_JPRB, 0.25121E+00_JPRB, 0.25111E+00_JPRB, 0.25107E+00_JPRB, 0.25173E+00_JPRB ,&
&  0.24950E+00_JPRB, 0.24795E+00_JPRB, 0.24999E+00_JPRB, 0.24878E+00_JPRB, 0.24780E+00_JPRB, 0.24610E+00_JPRB, 0.24517E+00_JPRB ,&
&  0.24049E+00_JPRB, 0.23418E+00_JPRB, 0.22815E+00_JPRB, 0.23237E+00_JPRB, 0.23962E+00_JPRB, 0.23638E+00_JPRB, 0.23592E+00_JPRB ,&
&  0.23579E+00_JPRB, 0.23500E+00_JPRB, 0.23508E+00_JPRB, 0.23508E+00_JPRB, 0.23646E+00_JPRB, 0.23780E+00_JPRB, 0.24044E+00_JPRB ,&
&  0.24390E+00_JPRB, 0.24863E+00_JPRB, 0.24866E+00_JPRB, 0.24136E+00_JPRB, 0.23856E+00_JPRB, 0.23783E+00_JPRB, 0.23416E+00_JPRB ,&
&  0.22958E+00_JPRB, 0.22847E+00_JPRB, 0.22711E+00_JPRB, 0.22328E+00_JPRB, 0.22174E+00_JPRB, 0.22108E+00_JPRB, 0.21771E+00_JPRB ,&
&  0.21353E+00_JPRB, 0.21044E+00_JPRB, 0.20641E+00_JPRB, 0.20239E+00_JPRB, 0.19964E+00_JPRB, 0.19839E+00_JPRB, 0.19863E+00_JPRB ,&
&  0.20099E+00_JPRB, 0.20306E+00_JPRB, 0.20956E+00_JPRB, 0.21331E+00_JPRB, 0.20913E+00_JPRB, 0.20662E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.19671E+00_JPRB, 0.19383E+00_JPRB, 0.19142E+00_JPRB, 0.18958E+00_JPRB, 0.18932E+00_JPRB, 0.19007E+00_JPRB, 0.19051E+00_JPRB ,&
&  0.19132E+00_JPRB, 0.19271E+00_JPRB, 0.19340E+00_JPRB, 0.19378E+00_JPRB, 0.19389E+00_JPRB, 0.19428E+00_JPRB, 0.19511E+00_JPRB ,&
&  0.19608E+00_JPRB, 0.19841E+00_JPRB, 0.20151E+00_JPRB, 0.20476E+00_JPRB, 0.20709E+00_JPRB, 0.20898E+00_JPRB, 0.21312E+00_JPRB ,&
&  0.21758E+00_JPRB, 0.22809E+00_JPRB, 0.24255E+00_JPRB, 0.25152E+00_JPRB, 0.24695E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.24549E+00_JPRB, 0.24713E+00_JPRB, 0.24669E+00_JPRB, 0.24678E+00_JPRB, 0.24613E+00_JPRB, 0.24665E+00_JPRB, 0.24520E+00_JPRB ,&
&  0.24452E+00_JPRB, 0.24268E+00_JPRB, 0.24107E+00_JPRB, 0.23744E+00_JPRB, 0.23545E+00_JPRB, 0.22911E+00_JPRB, 0.22711E+00_JPRB ,&
&  0.22515E+00_JPRB, 0.22574E+00_JPRB, 0.22581E+00_JPRB, 0.22627E+00_JPRB, 0.22694E+00_JPRB, 0.22850E+00_JPRB, 0.22812E+00_JPRB ,&
&  0.22676E+00_JPRB, 0.22471E+00_JPRB, 0.22227E+00_JPRB, 0.22007E+00_JPRB, 0.22052E+00_JPRB, 0.22004E+00_JPRB, 0.21887E+00_JPRB ,&
&  0.21792E+00_JPRB, 0.21868E+00_JPRB, 0.21917E+00_JPRB, 0.22069E+00_JPRB, 0.22571E+00_JPRB, 0.22362E+00_JPRB, 0.22280E+00_JPRB ,&
&  0.22326E+00_JPRB, 0.22387E+00_JPRB, 0.22524E+00_JPRB, 0.22648E+00_JPRB, 0.22978E+00_JPRB, 0.22939E+00_JPRB, 0.23292E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.23086E+00_JPRB, 0.22922E+00_JPRB, 0.22606E+00_JPRB, 0.22123E+00_JPRB, 0.21819E+00_JPRB, 0.21537E+00_JPRB ,&
&  0.21211E+00_JPRB, 0.20938E+00_JPRB, 0.20582E+00_JPRB, 0.20459E+00_JPRB, 0.20259E+00_JPRB, 0.20081E+00_JPRB, 0.19920E+00_JPRB ,&
&  0.19808E+00_JPRB, 0.19649E+00_JPRB, 0.19517E+00_JPRB, 0.19429E+00_JPRB, 0.19406E+00_JPRB, 0.19421E+00_JPRB, 0.19550E+00_JPRB ,&
&  0.19783E+00_JPRB, 0.19961E+00_JPRB, 0.20102E+00_JPRB, 0.20261E+00_JPRB, 0.20093E+00_JPRB, 0.19796E+00_JPRB, 0.19508E+00_JPRB ,&
&  0.19071E+00_JPRB, 0.18837E+00_JPRB, 0.18608E+00_JPRB, 0.18469E+00_JPRB, 0.18425E+00_JPRB, 0.18405E+00_JPRB, 0.18423E+00_JPRB ,&
&  0.18444E+00_JPRB, 0.18496E+00_JPRB, 0.18533E+00_JPRB, 0.18588E+00_JPRB, 0.18659E+00_JPRB, 0.18729E+00_JPRB, 0.18840E+00_JPRB ,&
&  0.19011E+00_JPRB, 0.19230E+00_JPRB, 0.19514E+00_JPRB, 0.19837E+00_JPRB, 0.20232E+00_JPRB, 0.20732E+00_JPRB, 0.21446E+00_JPRB ,&
&  0.22122E+00_JPRB, 0.23113E+00_JPRB, 0.23172E+00_JPRB, 0.24010E+00_JPRB, 0.24447E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.24275E+00_JPRB, 0.24498E+00_JPRB, 0.24546E+00_JPRB, 0.24605E+00_JPRB, 0.24494E+00_JPRB, 0.24176E+00_JPRB, 0.23842E+00_JPRB ,&
&  0.23190E+00_JPRB, 0.22682E+00_JPRB, 0.22072E+00_JPRB, 0.21514E+00_JPRB, 0.20978E+00_JPRB, 0.20649E+00_JPRB, 0.20439E+00_JPRB ,&
&  0.20397E+00_JPRB, 0.20518E+00_JPRB, 0.20602E+00_JPRB, 0.20655E+00_JPRB, 0.20669E+00_JPRB, 0.20756E+00_JPRB, 0.20733E+00_JPRB ,&
&  0.20667E+00_JPRB, 0.20564E+00_JPRB, 0.20490E+00_JPRB, 0.20567E+00_JPRB, 0.20785E+00_JPRB, 0.21005E+00_JPRB, 0.21190E+00_JPRB ,&
&  0.21336E+00_JPRB, 0.21235E+00_JPRB, 0.21234E+00_JPRB, 0.21155E+00_JPRB, 0.21146E+00_JPRB, 0.21136E+00_JPRB, 0.21187E+00_JPRB ,&
&  0.21361E+00_JPRB, 0.21499E+00_JPRB, 0.21786E+00_JPRB, 0.21929E+00_JPRB, 0.21841E+00_JPRB, 0.22178E+00_JPRB, 0.22305E+00_JPRB ,&
&  0.22294E+00_JPRB, 0.21958E+00_JPRB, 0.21571E+00_JPRB, 0.21251E+00_JPRB, 0.20902E+00_JPRB, 0.20613E+00_JPRB, 0.20376E+00_JPRB ,&
&  0.20160E+00_JPRB, 0.20054E+00_JPRB, 0.19819E+00_JPRB, 0.19619E+00_JPRB, 0.19488E+00_JPRB, 0.19350E+00_JPRB, 0.19265E+00_JPRB ,&
&  0.19215E+00_JPRB, 0.19120E+00_JPRB, 0.19104E+00_JPRB, 0.19048E+00_JPRB, 0.19055E+00_JPRB, 0.19064E+00_JPRB, 0.19095E+00_JPRB ,&
&  0.19184E+00_JPRB, 0.19305E+00_JPRB, 0.19294E+00_JPRB, 0.19286E+00_JPRB, 0.19104E+00_JPRB, 0.18906E+00_JPRB, 0.18742E+00_JPRB ,&
&  0.18513E+00_JPRB, 0.18348E+00_JPRB, 0.18246E+00_JPRB, 0.18143E+00_JPRB, 0.18122E+00_JPRB, 0.18117E+00_JPRB, 0.18102E+00_JPRB ,&
&  0.18115E+00_JPRB, 0.18147E+00_JPRB, 0.18201E+00_JPRB, 0.18326E+00_JPRB, 0.18433E+00_JPRB, 0.18589E+00_JPRB, 0.18760E+00_JPRB ,&
&  0.18892E+00_JPRB, 0.19141E+00_JPRB, 0.19402E+00_JPRB, 0.19705E+00_JPRB, 0.20325E+00_JPRB, 0.20836E+00_JPRB, 0.21738E+00_JPRB ,&
&  0.22359E+00_JPRB, 0.23208E+00_JPRB, 0.22935E+00_JPRB, 0.23547E+00_JPRB, 0.24025E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.24147E+00_JPRB, 0.24272E+00_JPRB, 0.24207E+00_JPRB, 0.24199E+00_JPRB, 0.24012E+00_JPRB, 0.23758E+00_JPRB, 0.23384E+00_JPRB ,&
&  0.23041E+00_JPRB, 0.22419E+00_JPRB, 0.21960E+00_JPRB, 0.21445E+00_JPRB, 0.21014E+00_JPRB, 0.20627E+00_JPRB, 0.20347E+00_JPRB ,&
&  0.20209E+00_JPRB, 0.20144E+00_JPRB, 0.20104E+00_JPRB, 0.19999E+00_JPRB, 0.19811E+00_JPRB, 0.19689E+00_JPRB, 0.19537E+00_JPRB ,&
&  0.19427E+00_JPRB, 0.19431E+00_JPRB, 0.19536E+00_JPRB, 0.19739E+00_JPRB, 0.20104E+00_JPRB, 0.20341E+00_JPRB, 0.20564E+00_JPRB ,&
&  0.20742E+00_JPRB, 0.20310E+00_JPRB, 0.20319E+00_JPRB, 0.20388E+00_JPRB, 0.20418E+00_JPRB, 0.20394E+00_JPRB, 0.20363E+00_JPRB ,&
&  0.20795E+00_JPRB, 0.20922E+00_JPRB, 0.21549E+00_JPRB, 0.21517E+00_JPRB, 0.21434E+00_JPRB, 0.21646E+00_JPRB, 0.21547E+00_JPRB ,&
&  0.21584E+00_JPRB, 0.21248E+00_JPRB, 0.20998E+00_JPRB, 0.20601E+00_JPRB, 0.20189E+00_JPRB, 0.19980E+00_JPRB, 0.19741E+00_JPRB ,&
&  0.19668E+00_JPRB, 0.19655E+00_JPRB, 0.19487E+00_JPRB, 0.19383E+00_JPRB, 0.19178E+00_JPRB, 0.18946E+00_JPRB, 0.18806E+00_JPRB ,&
&  0.18662E+00_JPRB, 0.18550E+00_JPRB, 0.18490E+00_JPRB, 0.18428E+00_JPRB, 0.18400E+00_JPRB, 0.18383E+00_JPRB, 0.18365E+00_JPRB ,&
&  0.18366E+00_JPRB, 0.18359E+00_JPRB, 0.18295E+00_JPRB, 0.18266E+00_JPRB, 0.18145E+00_JPRB, 0.18063E+00_JPRB, 0.17996E+00_JPRB ,&
&  0.17933E+00_JPRB, 0.17909E+00_JPRB, 0.17895E+00_JPRB, 0.17899E+00_JPRB, 0.17911E+00_JPRB, 0.17962E+00_JPRB, 0.18024E+00_JPRB ,&
&  0.18083E+00_JPRB, 0.18195E+00_JPRB, 0.18279E+00_JPRB, 0.18403E+00_JPRB, 0.18550E+00_JPRB, 0.18674E+00_JPRB, 0.18875E+00_JPRB ,&
&  0.19095E+00_JPRB, 0.19354E+00_JPRB, 0.19736E+00_JPRB, 0.20065E+00_JPRB, 0.20462E+00_JPRB, 0.20980E+00_JPRB, 0.21527E+00_JPRB ,&
&  0.22115E+00_JPRB, 0.22720E+00_JPRB, 0.22744E+00_JPRB, 0.23470E+00_JPRB, 0.23915E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.23117E+00_JPRB, 0.23187E+00_JPRB, 0.23257E+00_JPRB, 0.23274E+00_JPRB, 0.23272E+00_JPRB, 0.23161E+00_JPRB, 0.23018E+00_JPRB ,&
&  0.22569E+00_JPRB, 0.22250E+00_JPRB, 0.21885E+00_JPRB, 0.21520E+00_JPRB, 0.21196E+00_JPRB, 0.20841E+00_JPRB, 0.20534E+00_JPRB ,&
&  0.20255E+00_JPRB, 0.20041E+00_JPRB, 0.19858E+00_JPRB, 0.19622E+00_JPRB, 0.19329E+00_JPRB, 0.19060E+00_JPRB, 0.18906E+00_JPRB ,&
&  0.18873E+00_JPRB, 0.18921E+00_JPRB, 0.19090E+00_JPRB, 0.19254E+00_JPRB, 0.19429E+00_JPRB, 0.19560E+00_JPRB, 0.19695E+00_JPRB ,&
&  0.19642E+00_JPRB, 0.19616E+00_JPRB, 0.19589E+00_JPRB, 0.19531E+00_JPRB, 0.19625E+00_JPRB, 0.19749E+00_JPRB, 0.19836E+00_JPRB ,&
&  0.19939E+00_JPRB, 0.20166E+00_JPRB, 0.20342E+00_JPRB, 0.20501E+00_JPRB, 0.20525E+00_JPRB, 0.20475E+00_JPRB, 0.20326E+00_JPRB ,&
&  0.20128E+00_JPRB, 0.19915E+00_JPRB, 0.19719E+00_JPRB, 0.19547E+00_JPRB, 0.19413E+00_JPRB, 0.19256E+00_JPRB, 0.19119E+00_JPRB ,&
&  0.19053E+00_JPRB, 0.18992E+00_JPRB, 0.18906E+00_JPRB, 0.18821E+00_JPRB, 0.18690E+00_JPRB, 0.18575E+00_JPRB, 0.18456E+00_JPRB ,&
&  0.18348E+00_JPRB, 0.18248E+00_JPRB, 0.18155E+00_JPRB, 0.18059E+00_JPRB, 0.17993E+00_JPRB, 0.17933E+00_JPRB, 0.17876E+00_JPRB ,&
&  0.17815E+00_JPRB, 0.17796E+00_JPRB, 0.17753E+00_JPRB, 0.17739E+00_JPRB, 0.17710E+00_JPRB, 0.17692E+00_JPRB, 0.17713E+00_JPRB ,&
&  0.17774E+00_JPRB, 0.17836E+00_JPRB, 0.17864E+00_JPRB, 0.17902E+00_JPRB, 0.17941E+00_JPRB, 0.17992E+00_JPRB, 0.18077E+00_JPRB ,&
&  0.18142E+00_JPRB, 0.18219E+00_JPRB, 0.18279E+00_JPRB, 0.18353E+00_JPRB, 0.18503E+00_JPRB, 0.18656E+00_JPRB, 0.18831E+00_JPRB ,&
&  0.19052E+00_JPRB, 0.19282E+00_JPRB, 0.19555E+00_JPRB, 0.19816E+00_JPRB, 0.20165E+00_JPRB, 0.20339E+00_JPRB, 0.20694E+00_JPRB ,&
&  0.21081E+00_JPRB, 0.21140E+00_JPRB, 0.22487E+00_JPRB, 0.22615E+00_JPRB, 0.22799E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.20225E+00_JPRB, 0.20419E+00_JPRB, 0.20533E+00_JPRB, 0.20661E+00_JPRB, 0.20388E+00_JPRB, 0.21616E+00_JPRB, 0.21381E+00_JPRB ,&
&  0.21246E+00_JPRB, 0.20811E+00_JPRB, 0.20802E+00_JPRB, 0.20633E+00_JPRB, 0.20414E+00_JPRB, 0.20167E+00_JPRB, 0.19938E+00_JPRB ,&
&  0.19723E+00_JPRB, 0.19485E+00_JPRB, 0.19320E+00_JPRB, 0.19193E+00_JPRB, 0.19076E+00_JPRB, 0.18970E+00_JPRB, 0.18891E+00_JPRB ,&
&  0.18898E+00_JPRB, 0.18966E+00_JPRB, 0.19093E+00_JPRB, 0.19080E+00_JPRB, 0.19043E+00_JPRB, 0.18995E+00_JPRB, 0.18969E+00_JPRB ,&
&  0.18905E+00_JPRB, 0.18861E+00_JPRB, 0.18820E+00_JPRB, 0.18760E+00_JPRB, 0.18860E+00_JPRB, 0.18965E+00_JPRB, 0.19069E+00_JPRB ,&
&  0.19167E+00_JPRB, 0.19266E+00_JPRB, 0.19286E+00_JPRB, 0.19287E+00_JPRB, 0.19255E+00_JPRB, 0.19206E+00_JPRB, 0.19117E+00_JPRB ,&
&  0.19033E+00_JPRB, 0.18942E+00_JPRB, 0.18834E+00_JPRB, 0.18763E+00_JPRB, 0.18700E+00_JPRB, 0.18642E+00_JPRB, 0.18589E+00_JPRB ,&
&  0.18562E+00_JPRB, 0.18545E+00_JPRB, 0.18521E+00_JPRB, 0.18485E+00_JPRB, 0.18442E+00_JPRB, 0.18398E+00_JPRB, 0.18360E+00_JPRB ,&
&  0.18330E+00_JPRB, 0.18272E+00_JPRB, 0.18219E+00_JPRB, 0.18164E+00_JPRB, 0.18123E+00_JPRB, 0.18067E+00_JPRB, 0.18019E+00_JPRB ,&
&  0.17974E+00_JPRB, 0.17939E+00_JPRB, 0.17927E+00_JPRB, 0.17918E+00_JPRB, 0.17909E+00_JPRB, 0.17913E+00_JPRB, 0.17925E+00_JPRB ,&
&  0.17944E+00_JPRB, 0.17965E+00_JPRB, 0.17996E+00_JPRB, 0.18018E+00_JPRB, 0.18038E+00_JPRB, 0.18054E+00_JPRB, 0.18100E+00_JPRB ,&
&  0.18153E+00_JPRB, 0.18209E+00_JPRB, 0.18259E+00_JPRB, 0.18274E+00_JPRB, 0.18365E+00_JPRB, 0.18461E+00_JPRB, 0.18568E+00_JPRB ,&
&  0.18683E+00_JPRB, 0.18831E+00_JPRB, 0.18994E+00_JPRB, 0.19169E+00_JPRB, 0.19377E+00_JPRB, 0.19500E+00_JPRB, 0.19646E+00_JPRB ,&
&  0.19847E+00_JPRB, 0.20184E+00_JPRB, 0.20280E+00_JPRB, 0.20299E+00_JPRB, 0.20287E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.19040E+00_JPRB, 0.19009E+00_JPRB, 0.18973E+00_JPRB, 0.18932E+00_JPRB, 0.18814E+00_JPRB, 0.18751E+00_JPRB, 0.18699E+00_JPRB ,&
&  0.18665E+00_JPRB, 0.18643E+00_JPRB, 0.18649E+00_JPRB, 0.18663E+00_JPRB, 0.18679E+00_JPRB, 0.18693E+00_JPRB, 0.18707E+00_JPRB ,&
&  0.18703E+00_JPRB, 0.18703E+00_JPRB, 0.18619E+00_JPRB, 0.18551E+00_JPRB, 0.18507E+00_JPRB, 0.18474E+00_JPRB, 0.18431E+00_JPRB ,&
&  0.18397E+00_JPRB, 0.18357E+00_JPRB, 0.18309E+00_JPRB, 0.18281E+00_JPRB, 0.18287E+00_JPRB, 0.18274E+00_JPRB, 0.18280E+00_JPRB ,&
&  0.18283E+00_JPRB, 0.18312E+00_JPRB, 0.18328E+00_JPRB, 0.18346E+00_JPRB, 0.18399E+00_JPRB, 0.18450E+00_JPRB, 0.18468E+00_JPRB ,&
&  0.18482E+00_JPRB, 0.18469E+00_JPRB, 0.18455E+00_JPRB, 0.18444E+00_JPRB, 0.18441E+00_JPRB, 0.18443E+00_JPRB, 0.18442E+00_JPRB ,&
&  0.18431E+00_JPRB, 0.18412E+00_JPRB, 0.18387E+00_JPRB, 0.18356E+00_JPRB, 0.18330E+00_JPRB, 0.18320E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18323E+00_JPRB, 0.18311E+00_JPRB, 0.18290E+00_JPRB, 0.18254E+00_JPRB, 0.18219E+00_JPRB, 0.18184E+00_JPRB, 0.18151E+00_JPRB ,&
&  0.18119E+00_JPRB, 0.18105E+00_JPRB, 0.18089E+00_JPRB, 0.18077E+00_JPRB, 0.18069E+00_JPRB, 0.18056E+00_JPRB, 0.18044E+00_JPRB ,&
&  0.18033E+00_JPRB, 0.18023E+00_JPRB, 0.18011E+00_JPRB, 0.18001E+00_JPRB, 0.17994E+00_JPRB, 0.17989E+00_JPRB, 0.17986E+00_JPRB ,&
&  0.17986E+00_JPRB, 0.17986E+00_JPRB, 0.17988E+00_JPRB, 0.18002E+00_JPRB, 0.18020E+00_JPRB, 0.18042E+00_JPRB, 0.18068E+00_JPRB ,&
&  0.18097E+00_JPRB, 0.18130E+00_JPRB, 0.18165E+00_JPRB, 0.18190E+00_JPRB, 0.18232E+00_JPRB, 0.18274E+00_JPRB, 0.18317E+00_JPRB ,&
&  0.18361E+00_JPRB, 0.18415E+00_JPRB, 0.18475E+00_JPRB, 0.18542E+00_JPRB, 0.18620E+00_JPRB, 0.18662E+00_JPRB, 0.18716E+00_JPRB ,&
&  0.18778E+00_JPRB, 0.18862E+00_JPRB, 0.18906E+00_JPRB, 0.18937E+00_JPRB, 0.18970E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB, 0.18251E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB, 0.86432E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.87364E+00_JPRB, 0.87300E+00_JPRB, 0.87233E+00_JPRB, 0.87172E+00_JPRB, 0.87111E+00_JPRB, 0.87065E+00_JPRB, 0.87012E+00_JPRB ,&
&  0.86986E+00_JPRB, 0.86957E+00_JPRB, 0.86932E+00_JPRB, 0.86894E+00_JPRB, 0.86881E+00_JPRB, 0.86859E+00_JPRB, 0.86836E+00_JPRB ,&
&  0.86800E+00_JPRB, 0.86778E+00_JPRB, 0.86717E+00_JPRB, 0.86662E+00_JPRB, 0.86585E+00_JPRB, 0.86528E+00_JPRB, 0.86438E+00_JPRB ,&
&  0.86379E+00_JPRB, 0.86279E+00_JPRB, 0.86188E+00_JPRB, 0.86022E+00_JPRB, 0.85899E+00_JPRB, 0.85765E+00_JPRB, 0.85650E+00_JPRB ,&
&  0.85551E+00_JPRB, 0.85489E+00_JPRB, 0.85427E+00_JPRB, 0.85342E+00_JPRB, 0.85282E+00_JPRB, 0.85299E+00_JPRB, 0.85333E+00_JPRB ,&
&  0.85365E+00_JPRB, 0.85412E+00_JPRB, 0.85475E+00_JPRB, 0.85501E+00_JPRB, 0.85484E+00_JPRB, 0.85488E+00_JPRB, 0.85480E+00_JPRB ,&
&  0.85453E+00_JPRB, 0.85407E+00_JPRB, 0.85362E+00_JPRB, 0.85349E+00_JPRB, 0.85334E+00_JPRB, 0.85294E+00_JPRB, 0.85264E+00_JPRB ,&
&  0.85240E+00_JPRB, 0.85213E+00_JPRB, 0.85176E+00_JPRB, 0.85119E+00_JPRB, 0.85080E+00_JPRB, 0.85041E+00_JPRB, 0.84995E+00_JPRB ,&
&  0.84945E+00_JPRB, 0.84887E+00_JPRB, 0.84823E+00_JPRB, 0.84754E+00_JPRB, 0.84682E+00_JPRB, 0.84638E+00_JPRB, 0.84615E+00_JPRB ,&
&  0.84606E+00_JPRB, 0.84624E+00_JPRB, 0.84661E+00_JPRB, 0.84703E+00_JPRB, 0.84758E+00_JPRB, 0.84836E+00_JPRB, 0.84937E+00_JPRB ,&
&  0.85049E+00_JPRB, 0.85168E+00_JPRB, 0.85323E+00_JPRB, 0.85506E+00_JPRB, 0.85682E+00_JPRB, 0.85859E+00_JPRB, 0.86053E+00_JPRB ,&
&  0.86226E+00_JPRB, 0.86361E+00_JPRB, 0.86466E+00_JPRB, 0.86573E+00_JPRB, 0.86722E+00_JPRB, 0.86849E+00_JPRB, 0.86954E+00_JPRB ,&
&  0.87048E+00_JPRB, 0.87148E+00_JPRB, 0.87240E+00_JPRB, 0.87336E+00_JPRB, 0.87374E+00_JPRB, 0.87408E+00_JPRB, 0.87434E+00_JPRB ,&
&  0.87453E+00_JPRB, 0.87455E+00_JPRB, 0.87448E+00_JPRB, 0.87434E+00_JPRB, 0.87406E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.87331E+00_JPRB, 0.87167E+00_JPRB, 0.87039E+00_JPRB, 0.86952E+00_JPRB, 0.86877E+00_JPRB, 0.86836E+00_JPRB, 0.86810E+00_JPRB ,&
&  0.86807E+00_JPRB, 0.86765E+00_JPRB, 0.86690E+00_JPRB, 0.86679E+00_JPRB, 0.86744E+00_JPRB, 0.86831E+00_JPRB, 0.86806E+00_JPRB ,&
&  0.86794E+00_JPRB, 0.86755E+00_JPRB, 0.86742E+00_JPRB, 0.86680E+00_JPRB, 0.86561E+00_JPRB, 0.86397E+00_JPRB, 0.86200E+00_JPRB ,&
&  0.86049E+00_JPRB, 0.85854E+00_JPRB, 0.85644E+00_JPRB, 0.85221E+00_JPRB, 0.84863E+00_JPRB, 0.84515E+00_JPRB, 0.84181E+00_JPRB ,&
&  0.83731E+00_JPRB, 0.83342E+00_JPRB, 0.83038E+00_JPRB, 0.82786E+00_JPRB, 0.83155E+00_JPRB, 0.83401E+00_JPRB, 0.83543E+00_JPRB ,&
&  0.83553E+00_JPRB, 0.83157E+00_JPRB, 0.83043E+00_JPRB, 0.82992E+00_JPRB, 0.82972E+00_JPRB, 0.83330E+00_JPRB, 0.83579E+00_JPRB ,&
&  0.83785E+00_JPRB, 0.83950E+00_JPRB, 0.84312E+00_JPRB, 0.84493E+00_JPRB, 0.84617E+00_JPRB, 0.84675E+00_JPRB, 0.84653E+00_JPRB ,&
&  0.84710E+00_JPRB, 0.84771E+00_JPRB, 0.84814E+00_JPRB, 0.84782E+00_JPRB, 0.84739E+00_JPRB, 0.84698E+00_JPRB, 0.84605E+00_JPRB ,&
&  0.84449E+00_JPRB, 0.84280E+00_JPRB, 0.84115E+00_JPRB, 0.83941E+00_JPRB, 0.83743E+00_JPRB, 0.83609E+00_JPRB, 0.83498E+00_JPRB ,&
&  0.83400E+00_JPRB, 0.83332E+00_JPRB, 0.83305E+00_JPRB, 0.83301E+00_JPRB, 0.83326E+00_JPRB, 0.83430E+00_JPRB, 0.83634E+00_JPRB ,&
&  0.83919E+00_JPRB, 0.84269E+00_JPRB, 0.84670E+00_JPRB, 0.85078E+00_JPRB, 0.85422E+00_JPRB, 0.85725E+00_JPRB, 0.86086E+00_JPRB ,&
&  0.86473E+00_JPRB, 0.86782E+00_JPRB, 0.87046E+00_JPRB, 0.87288E+00_JPRB, 0.87571E+00_JPRB, 0.87743E+00_JPRB, 0.87839E+00_JPRB ,&
&  0.87910E+00_JPRB, 0.88044E+00_JPRB, 0.88183E+00_JPRB, 0.88344E+00_JPRB, 0.88425E+00_JPRB, 0.88416E+00_JPRB, 0.88343E+00_JPRB ,&
&  0.88239E+00_JPRB, 0.88135E+00_JPRB, 0.87978E+00_JPRB, 0.87766E+00_JPRB, 0.87545E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.86815E+00_JPRB, 0.86880E+00_JPRB, 0.87011E+00_JPRB, 0.87202E+00_JPRB, 0.87432E+00_JPRB, 0.87591E+00_JPRB, 0.87772E+00_JPRB ,&
&  0.87895E+00_JPRB, 0.87919E+00_JPRB, 0.87860E+00_JPRB, 0.87921E+00_JPRB, 0.87938E+00_JPRB, 0.87996E+00_JPRB, 0.87893E+00_JPRB ,&
&  0.87864E+00_JPRB, 0.87718E+00_JPRB, 0.87796E+00_JPRB, 0.87822E+00_JPRB, 0.87792E+00_JPRB, 0.87793E+00_JPRB, 0.87034E+00_JPRB ,&
&  0.86335E+00_JPRB, 0.85795E+00_JPRB, 0.85068E+00_JPRB, 0.84368E+00_JPRB, 0.83893E+00_JPRB, 0.81035E+00_JPRB, 0.80176E+00_JPRB ,&
&  0.81523E+00_JPRB, 0.82368E+00_JPRB, 0.83062E+00_JPRB, 0.83536E+00_JPRB, 0.81849E+00_JPRB, 0.79938E+00_JPRB, 0.78780E+00_JPRB ,&
&  0.76027E+00_JPRB, 0.76862E+00_JPRB, 0.77460E+00_JPRB, 0.77941E+00_JPRB, 0.78724E+00_JPRB, 0.78982E+00_JPRB, 0.79730E+00_JPRB ,&
&  0.81531E+00_JPRB, 0.82040E+00_JPRB, 0.82745E+00_JPRB, 0.83346E+00_JPRB, 0.83704E+00_JPRB, 0.84134E+00_JPRB, 0.85008E+00_JPRB ,&
&  0.85400E+00_JPRB, 0.85762E+00_JPRB, 0.85994E+00_JPRB, 0.85971E+00_JPRB, 0.86022E+00_JPRB, 0.85949E+00_JPRB, 0.85843E+00_JPRB ,&
&  0.85496E+00_JPRB, 0.85293E+00_JPRB, 0.85115E+00_JPRB, 0.84901E+00_JPRB, 0.84743E+00_JPRB, 0.84314E+00_JPRB, 0.84016E+00_JPRB ,&
&  0.83831E+00_JPRB, 0.83760E+00_JPRB, 0.83762E+00_JPRB, 0.83878E+00_JPRB, 0.84145E+00_JPRB, 0.84516E+00_JPRB, 0.84912E+00_JPRB ,&
&  0.85418E+00_JPRB, 0.85879E+00_JPRB, 0.86409E+00_JPRB, 0.86932E+00_JPRB, 0.87362E+00_JPRB, 0.87732E+00_JPRB, 0.88069E+00_JPRB ,&
&  0.88481E+00_JPRB, 0.88920E+00_JPRB, 0.89175E+00_JPRB, 0.89210E+00_JPRB, 0.89474E+00_JPRB, 0.89686E+00_JPRB, 0.89770E+00_JPRB ,&
&  0.89819E+00_JPRB, 0.89912E+00_JPRB, 0.89810E+00_JPRB, 0.89574E+00_JPRB, 0.89278E+00_JPRB, 0.88992E+00_JPRB, 0.88670E+00_JPRB ,&
&  0.88324E+00_JPRB, 0.87994E+00_JPRB, 0.87641E+00_JPRB, 0.87252E+00_JPRB, 0.86958E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.87234E+00_JPRB, 0.87637E+00_JPRB, 0.88162E+00_JPRB, 0.88458E+00_JPRB, 0.88855E+00_JPRB, 0.88995E+00_JPRB, 0.89231E+00_JPRB ,&
&  0.89280E+00_JPRB, 0.89392E+00_JPRB, 0.89462E+00_JPRB, 0.89468E+00_JPRB, 0.89438E+00_JPRB, 0.89311E+00_JPRB, 0.89198E+00_JPRB ,&
&  0.89138E+00_JPRB, 0.89028E+00_JPRB, 0.89077E+00_JPRB, 0.89079E+00_JPRB, 0.88994E+00_JPRB, 0.88919E+00_JPRB, 0.88039E+00_JPRB ,&
&  0.87077E+00_JPRB, 0.86138E+00_JPRB, 0.82552E+00_JPRB, 0.82482E+00_JPRB, 0.82064E+00_JPRB, 0.81854E+00_JPRB, 0.82123E+00_JPRB ,&
&  0.81639E+00_JPRB, 0.81071E+00_JPRB, 0.78989E+00_JPRB, 0.77008E+00_JPRB, 0.75625E+00_JPRB, 0.74134E+00_JPRB, 0.74146E+00_JPRB ,&
&  0.74051E+00_JPRB, 0.74623E+00_JPRB, 0.75162E+00_JPRB, 0.75487E+00_JPRB, 0.76513E+00_JPRB, 0.77046E+00_JPRB, 0.78176E+00_JPRB ,&
&  0.79588E+00_JPRB, 0.80519E+00_JPRB, 0.80350E+00_JPRB, 0.80098E+00_JPRB, 0.82267E+00_JPRB, 0.83766E+00_JPRB, 0.85116E+00_JPRB ,&
&  0.85970E+00_JPRB, 0.86137E+00_JPRB, 0.86547E+00_JPRB, 0.86558E+00_JPRB, 0.86734E+00_JPRB, 0.87159E+00_JPRB, 0.87304E+00_JPRB ,&
&  0.87519E+00_JPRB, 0.87383E+00_JPRB, 0.87028E+00_JPRB, 0.86541E+00_JPRB, 0.85755E+00_JPRB, 0.85241E+00_JPRB, 0.84943E+00_JPRB ,&
&  0.84953E+00_JPRB, 0.85283E+00_JPRB, 0.85768E+00_JPRB, 0.86435E+00_JPRB, 0.87121E+00_JPRB, 0.87675E+00_JPRB, 0.88074E+00_JPRB ,&
&  0.88388E+00_JPRB, 0.88596E+00_JPRB, 0.89279E+00_JPRB, 0.89412E+00_JPRB, 0.89790E+00_JPRB, 0.90726E+00_JPRB, 0.90731E+00_JPRB ,&
&  0.90980E+00_JPRB, 0.91569E+00_JPRB, 0.91957E+00_JPRB, 0.92297E+00_JPRB, 0.92533E+00_JPRB, 0.92653E+00_JPRB, 0.92783E+00_JPRB ,&
&  0.92813E+00_JPRB, 0.92645E+00_JPRB, 0.92236E+00_JPRB, 0.91675E+00_JPRB, 0.91085E+00_JPRB, 0.90479E+00_JPRB, 0.89689E+00_JPRB ,&
&  0.88901E+00_JPRB, 0.88101E+00_JPRB, 0.87475E+00_JPRB, 0.86987E+00_JPRB, 0.87013E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.88722E+00_JPRB, 0.89110E+00_JPRB, 0.89782E+00_JPRB, 0.89982E+00_JPRB, 0.90359E+00_JPRB, 0.90468E+00_JPRB, 0.90590E+00_JPRB ,&
&  0.90623E+00_JPRB, 0.90676E+00_JPRB, 0.90631E+00_JPRB, 0.90622E+00_JPRB, 0.90546E+00_JPRB, 0.90499E+00_JPRB, 0.90428E+00_JPRB ,&
&  0.90304E+00_JPRB, 0.90176E+00_JPRB, 0.90043E+00_JPRB, 0.89920E+00_JPRB, 0.89879E+00_JPRB, 0.89846E+00_JPRB, 0.89381E+00_JPRB ,&
&  0.88656E+00_JPRB, 0.87134E+00_JPRB, 0.84408E+00_JPRB, 0.84230E+00_JPRB, 0.83970E+00_JPRB, 0.84209E+00_JPRB, 0.84095E+00_JPRB ,&
&  0.82850E+00_JPRB, 0.77483E+00_JPRB, 0.78060E+00_JPRB, 0.76948E+00_JPRB, 0.76561E+00_JPRB, 0.76114E+00_JPRB, 0.74651E+00_JPRB ,&
&  0.74167E+00_JPRB, 0.73564E+00_JPRB, 0.73449E+00_JPRB, 0.74207E+00_JPRB, 0.74908E+00_JPRB, 0.75985E+00_JPRB, 0.77270E+00_JPRB ,&
&  0.76993E+00_JPRB, 0.78872E+00_JPRB, 0.80112E+00_JPRB, 0.80896E+00_JPRB, 0.83057E+00_JPRB, 0.84874E+00_JPRB, 0.85334E+00_JPRB ,&
&  0.85898E+00_JPRB, 0.85659E+00_JPRB, 0.86015E+00_JPRB, 0.86903E+00_JPRB, 0.87851E+00_JPRB, 0.87844E+00_JPRB, 0.87902E+00_JPRB ,&
&  0.87786E+00_JPRB, 0.87656E+00_JPRB, 0.87486E+00_JPRB, 0.87204E+00_JPRB, 0.87109E+00_JPRB, 0.87393E+00_JPRB, 0.87814E+00_JPRB ,&
&  0.88623E+00_JPRB, 0.89516E+00_JPRB, 0.90544E+00_JPRB, 0.91521E+00_JPRB, 0.92310E+00_JPRB, 0.93062E+00_JPRB, 0.93755E+00_JPRB ,&
&  0.94448E+00_JPRB, 0.94928E+00_JPRB, 0.95106E+00_JPRB, 0.95217E+00_JPRB, 0.95129E+00_JPRB, 0.94737E+00_JPRB, 0.94761E+00_JPRB ,&
&  0.95178E+00_JPRB, 0.95442E+00_JPRB, 0.95545E+00_JPRB, 0.95680E+00_JPRB, 0.95923E+00_JPRB, 0.96029E+00_JPRB, 0.96049E+00_JPRB ,&
&  0.96015E+00_JPRB, 0.95757E+00_JPRB, 0.95294E+00_JPRB, 0.94293E+00_JPRB, 0.93261E+00_JPRB, 0.92595E+00_JPRB, 0.91468E+00_JPRB ,&
&  0.90290E+00_JPRB, 0.88957E+00_JPRB, 0.88255E+00_JPRB, 0.87791E+00_JPRB, 0.88167E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.90834E+00_JPRB, 0.91439E+00_JPRB, 0.92195E+00_JPRB, 0.92571E+00_JPRB, 0.92982E+00_JPRB, 0.93063E+00_JPRB, 0.93244E+00_JPRB ,&
&  0.93335E+00_JPRB, 0.93476E+00_JPRB, 0.93527E+00_JPRB, 0.93518E+00_JPRB, 0.93401E+00_JPRB, 0.93247E+00_JPRB, 0.93007E+00_JPRB ,&
&  0.92623E+00_JPRB, 0.92250E+00_JPRB, 0.91673E+00_JPRB, 0.91179E+00_JPRB, 0.90921E+00_JPRB, 0.90972E+00_JPRB, 0.90687E+00_JPRB ,&
&  0.90044E+00_JPRB, 0.88738E+00_JPRB, 0.86709E+00_JPRB, 0.86137E+00_JPRB, 0.86300E+00_JPRB, 0.85152E+00_JPRB, 0.84889E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.84276E+00_JPRB, 0.83135E+00_JPRB, 0.82441E+00_JPRB, 0.80560E+00_JPRB, 0.73776E+00_JPRB, 0.73408E+00_JPRB ,&
&  0.73079E+00_JPRB, 0.72147E+00_JPRB, 0.72075E+00_JPRB, 0.72326E+00_JPRB, 0.73122E+00_JPRB, 0.75769E+00_JPRB, 0.77987E+00_JPRB ,&
&  0.77628E+00_JPRB, 0.79849E+00_JPRB, 0.80831E+00_JPRB, 0.81793E+00_JPRB, 0.83601E+00_JPRB, 0.85221E+00_JPRB, 0.85343E+00_JPRB ,&
&  0.85744E+00_JPRB, 0.86031E+00_JPRB, 0.86807E+00_JPRB, 0.87637E+00_JPRB, 0.88305E+00_JPRB, 0.88704E+00_JPRB, 0.89238E+00_JPRB ,&
&  0.89905E+00_JPRB, 0.90462E+00_JPRB, 0.91153E+00_JPRB, 0.91485E+00_JPRB, 0.91652E+00_JPRB, 0.91904E+00_JPRB, 0.92428E+00_JPRB ,&
&  0.93508E+00_JPRB, 0.94624E+00_JPRB, 0.96263E+00_JPRB, 0.96550E+00_JPRB, 0.97279E+00_JPRB, 0.97972E+00_JPRB, 0.99109E+00_JPRB ,&
&  0.99829E+00_JPRB, 0.10138E+01_JPRB, 0.10242E+01_JPRB, 0.10310E+01_JPRB, 0.10213E+01_JPRB, 0.10104E+01_JPRB, 0.10033E+01_JPRB ,&
&  0.99964E+00_JPRB, 0.10001E+01_JPRB, 0.99083E+00_JPRB, 0.98786E+00_JPRB, 0.98681E+00_JPRB, 0.98724E+00_JPRB, 0.98798E+00_JPRB ,&
&  0.98748E+00_JPRB, 0.98244E+00_JPRB, 0.97455E+00_JPRB, 0.96228E+00_JPRB, 0.95167E+00_JPRB, 0.94224E+00_JPRB, 0.92470E+00_JPRB ,&
&  0.90917E+00_JPRB, 0.89337E+00_JPRB, 0.88903E+00_JPRB, 0.88926E+00_JPRB, 0.89769E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.93294E+00_JPRB, 0.94468E+00_JPRB, 0.95562E+00_JPRB, 0.96454E+00_JPRB, 0.96776E+00_JPRB, 0.96768E+00_JPRB, 0.96463E+00_JPRB ,&
&  0.96123E+00_JPRB, 0.96015E+00_JPRB, 0.95844E+00_JPRB, 0.95711E+00_JPRB, 0.95525E+00_JPRB, 0.95174E+00_JPRB, 0.94754E+00_JPRB ,&
&  0.94391E+00_JPRB, 0.92709E+00_JPRB, 0.91871E+00_JPRB, 0.91790E+00_JPRB, 0.91505E+00_JPRB, 0.91380E+00_JPRB, 0.91261E+00_JPRB ,&
&  0.90479E+00_JPRB, 0.89443E+00_JPRB, 0.88725E+00_JPRB, 0.87712E+00_JPRB, 0.86671E+00_JPRB, 0.87317E+00_JPRB, 0.87211E+00_JPRB ,&
&  0.84840E+00_JPRB, 0.83058E+00_JPRB, 0.83797E+00_JPRB, 0.83947E+00_JPRB, 0.80321E+00_JPRB, 0.76659E+00_JPRB, 0.74575E+00_JPRB ,&
&  0.74989E+00_JPRB, 0.73288E+00_JPRB, 0.73026E+00_JPRB, 0.73250E+00_JPRB, 0.74403E+00_JPRB, 0.75608E+00_JPRB, 0.77918E+00_JPRB ,&
&  0.78508E+00_JPRB, 0.78920E+00_JPRB, 0.80930E+00_JPRB, 0.82928E+00_JPRB, 0.84069E+00_JPRB, 0.85201E+00_JPRB, 0.85613E+00_JPRB ,&
&  0.86577E+00_JPRB, 0.87524E+00_JPRB, 0.88393E+00_JPRB, 0.89253E+00_JPRB, 0.90374E+00_JPRB, 0.90880E+00_JPRB, 0.91696E+00_JPRB ,&
&  0.96226E+00_JPRB, 0.10001E+01_JPRB, 0.10210E+01_JPRB, 0.10369E+01_JPRB, 0.10424E+01_JPRB, 0.10407E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10301E+01_JPRB, 0.10371E+01_JPRB, 0.10485E+01_JPRB, 0.10572E+01_JPRB, 0.10604E+01_JPRB, 0.10538E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10399E+01_JPRB, 0.11280E+01_JPRB, 0.11506E+01_JPRB, 0.11371E+01_JPRB, 0.11063E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.10460E+01_JPRB, 0.10301E+01_JPRB, 0.10202E+01_JPRB, 0.10123E+01_JPRB, 0.10061E+01_JPRB, 0.99763E+00_JPRB, 0.99801E+00_JPRB ,&
&  0.99432E+00_JPRB, 0.98512E+00_JPRB, 0.97537E+00_JPRB, 0.96471E+00_JPRB, 0.95493E+00_JPRB, 0.94691E+00_JPRB, 0.93431E+00_JPRB ,&
&  0.92062E+00_JPRB, 0.90706E+00_JPRB, 0.90559E+00_JPRB, 0.91039E+00_JPRB, 0.92151E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.96380E+00_JPRB, 0.97965E+00_JPRB, 0.99468E+00_JPRB, 0.10052E+01_JPRB, 0.10035E+01_JPRB, 0.98421E+00_JPRB, 0.96718E+00_JPRB ,&
&  0.95580E+00_JPRB, 0.95190E+00_JPRB, 0.94853E+00_JPRB, 0.94372E+00_JPRB, 0.94048E+00_JPRB, 0.93739E+00_JPRB, 0.93529E+00_JPRB ,&
&  0.93431E+00_JPRB, 0.93004E+00_JPRB, 0.92044E+00_JPRB, 0.90992E+00_JPRB, 0.90491E+00_JPRB, 0.90500E+00_JPRB, 0.90649E+00_JPRB ,&
&  0.90317E+00_JPRB, 0.89929E+00_JPRB, 0.90104E+00_JPRB, 0.88822E+00_JPRB, 0.87776E+00_JPRB, 0.85927E+00_JPRB, 0.84915E+00_JPRB ,&
&  0.85174E+00_JPRB, 0.85416E+00_JPRB, 0.83330E+00_JPRB, 0.83227E+00_JPRB, 0.83605E+00_JPRB, 0.79949E+00_JPRB, 0.77251E+00_JPRB ,&
&  0.75738E+00_JPRB, 0.74627E+00_JPRB, 0.74143E+00_JPRB, 0.74383E+00_JPRB, 0.77232E+00_JPRB, 0.81218E+00_JPRB, 0.82174E+00_JPRB ,&
&  0.77971E+00_JPRB, 0.80052E+00_JPRB, 0.82785E+00_JPRB, 0.84165E+00_JPRB, 0.85203E+00_JPRB, 0.86669E+00_JPRB, 0.88110E+00_JPRB ,&
&  0.89405E+00_JPRB, 0.90539E+00_JPRB, 0.91272E+00_JPRB, 0.93087E+00_JPRB, 0.93898E+00_JPRB, 0.93562E+00_JPRB, 0.95672E+00_JPRB ,&
&  0.10747E+01_JPRB, 0.11671E+01_JPRB, 0.12112E+01_JPRB, 0.12246E+01_JPRB, 0.12207E+01_JPRB, 0.12004E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.11637E+01_JPRB, 0.11571E+01_JPRB, 0.11525E+01_JPRB, 0.11467E+01_JPRB, 0.11336E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.11141E+01_JPRB, 0.13605E+01_JPRB, 0.13357E+01_JPRB, 0.12709E+01_JPRB, 0.12219E+01_JPRB, 0.11682E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.10990E+01_JPRB, 0.10821E+01_JPRB, 0.10639E+01_JPRB, 0.10245E+01_JPRB, 0.99549E+00_JPRB, 0.98249E+00_JPRB ,&
&  0.97552E+00_JPRB, 0.96311E+00_JPRB, 0.95394E+00_JPRB, 0.94615E+00_JPRB, 0.94036E+00_JPRB, 0.93889E+00_JPRB, 0.93392E+00_JPRB ,&
&  0.92771E+00_JPRB, 0.92837E+00_JPRB, 0.93347E+00_JPRB, 0.94393E+00_JPRB, 0.95346E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.98804E+00_JPRB, 0.10227E+01_JPRB, 0.10459E+01_JPRB, 0.10522E+01_JPRB, 0.10426E+01_JPRB, 0.99304E+00_JPRB, 0.94918E+00_JPRB ,&
&  0.92340E+00_JPRB, 0.91447E+00_JPRB, 0.91435E+00_JPRB, 0.91405E+00_JPRB, 0.91497E+00_JPRB, 0.91646E+00_JPRB, 0.91551E+00_JPRB ,&
&  0.91227E+00_JPRB, 0.91157E+00_JPRB, 0.91143E+00_JPRB, 0.90713E+00_JPRB, 0.90034E+00_JPRB, 0.89617E+00_JPRB, 0.89642E+00_JPRB ,&
&  0.89635E+00_JPRB, 0.89439E+00_JPRB, 0.89397E+00_JPRB, 0.88905E+00_JPRB, 0.88743E+00_JPRB, 0.88019E+00_JPRB, 0.86647E+00_JPRB ,&
&  0.84751E+00_JPRB, 0.84857E+00_JPRB, 0.85948E+00_JPRB, 0.85696E+00_JPRB, 0.86149E+00_JPRB, 0.83729E+00_JPRB, 0.80310E+00_JPRB ,&
&  0.78080E+00_JPRB, 0.77267E+00_JPRB, 0.78901E+00_JPRB, 0.83408E+00_JPRB, 0.86127E+00_JPRB, 0.81785E+00_JPRB, 0.83527E+00_JPRB ,&
&  0.83742E+00_JPRB, 0.83916E+00_JPRB, 0.84479E+00_JPRB, 0.86289E+00_JPRB, 0.88278E+00_JPRB, 0.90038E+00_JPRB, 0.91404E+00_JPRB ,&
&  0.92841E+00_JPRB, 0.94206E+00_JPRB, 0.95410E+00_JPRB, 0.95521E+00_JPRB, 0.96297E+00_JPRB, 0.10180E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.12272E+01_JPRB, 0.12536E+01_JPRB, 0.13026E+01_JPRB, 0.13342E+01_JPRB, 0.13420E+01_JPRB, 0.13359E+01_JPRB ,&
&  0.13277E+01_JPRB, 0.12976E+01_JPRB, 0.12696E+01_JPRB, 0.12559E+01_JPRB, 0.12532E+01_JPRB, 0.12517E+01_JPRB, 0.12352E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.11705E+01_JPRB, 0.11301E+01_JPRB, 0.10713E+01_JPRB, 0.10677E+01_JPRB, 0.10907E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.10958E+01_JPRB, 0.10894E+01_JPRB, 0.10750E+01_JPRB, 0.10144E+01_JPRB, 0.97270E+00_JPRB, 0.94804E+00_JPRB ,&
&  0.93359E+00_JPRB, 0.91472E+00_JPRB, 0.91659E+00_JPRB, 0.91402E+00_JPRB, 0.92127E+00_JPRB, 0.93051E+00_JPRB, 0.92703E+00_JPRB ,&
&  0.92445E+00_JPRB, 0.92506E+00_JPRB, 0.93367E+00_JPRB, 0.94467E+00_JPRB, 0.96393E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.10873E+01_JPRB, 0.11039E+01_JPRB, 0.11008E+01_JPRB, 0.11002E+01_JPRB, 0.10759E+01_JPRB, 0.10440E+01_JPRB, 0.10073E+01_JPRB ,&
&  0.98126E+00_JPRB, 0.94944E+00_JPRB, 0.91931E+00_JPRB, 0.89286E+00_JPRB, 0.87940E+00_JPRB, 0.87542E+00_JPRB, 0.87950E+00_JPRB ,&
&  0.88718E+00_JPRB, 0.89515E+00_JPRB, 0.90011E+00_JPRB, 0.89731E+00_JPRB, 0.89182E+00_JPRB, 0.89015E+00_JPRB, 0.89237E+00_JPRB ,&
&  0.89266E+00_JPRB, 0.89277E+00_JPRB, 0.89166E+00_JPRB, 0.88852E+00_JPRB, 0.88421E+00_JPRB, 0.87898E+00_JPRB, 0.87843E+00_JPRB ,&
&  0.87839E+00_JPRB, 0.87799E+00_JPRB, 0.88436E+00_JPRB, 0.88507E+00_JPRB, 0.89215E+00_JPRB, 0.87235E+00_JPRB, 0.84117E+00_JPRB ,&
&  0.81642E+00_JPRB, 0.81750E+00_JPRB, 0.87404E+00_JPRB, 0.84605E+00_JPRB, 0.87028E+00_JPRB, 0.88614E+00_JPRB, 0.88278E+00_JPRB ,&
&  0.88082E+00_JPRB, 0.88347E+00_JPRB, 0.90048E+00_JPRB, 0.91510E+00_JPRB, 0.92955E+00_JPRB, 0.94693E+00_JPRB, 0.96258E+00_JPRB ,&
&  0.98722E+00_JPRB, 0.10160E+01_JPRB, 0.10356E+01_JPRB, 0.10337E+01_JPRB, 0.11181E+01_JPRB, 0.13596E+01_JPRB, 0.14185E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14848E+01_JPRB, 0.15375E+01_JPRB, 0.15763E+01_JPRB, 0.15516E+01_JPRB, 0.15325E+01_JPRB, 0.15196E+01_JPRB ,&
&  0.14821E+01_JPRB, 0.14615E+01_JPRB, 0.14483E+01_JPRB, 0.14668E+01_JPRB, 0.14555E+01_JPRB, 0.14390E+01_JPRB, 0.14272E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.13721E+01_JPRB, 0.13277E+01_JPRB, 0.12625E+01_JPRB, 0.12261E+01_JPRB, 0.12023E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.11627E+01_JPRB, 0.11532E+01_JPRB, 0.11305E+01_JPRB, 0.10631E+01_JPRB, 0.99447E+00_JPRB, 0.95230E+00_JPRB ,&
&  0.92260E+00_JPRB, 0.89885E+00_JPRB, 0.89994E+00_JPRB, 0.90987E+00_JPRB, 0.92634E+00_JPRB, 0.94233E+00_JPRB, 0.93874E+00_JPRB ,&
&  0.94577E+00_JPRB, 0.95711E+00_JPRB, 0.98238E+00_JPRB, 0.10170E+01_JPRB, 0.10541E+01_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.11957E+01_JPRB, 0.12011E+01_JPRB, 0.11828E+01_JPRB, 0.11688E+01_JPRB, 0.11440E+01_JPRB, 0.10961E+01_JPRB, 0.10296E+01_JPRB ,&
&  0.99632E+00_JPRB, 0.95213E+00_JPRB, 0.92376E+00_JPRB, 0.91170E+00_JPRB, 0.90278E+00_JPRB, 0.88822E+00_JPRB, 0.87269E+00_JPRB ,&
&  0.87234E+00_JPRB, 0.88608E+00_JPRB, 0.89990E+00_JPRB, 0.90554E+00_JPRB, 0.90864E+00_JPRB, 0.90680E+00_JPRB, 0.90760E+00_JPRB ,&
&  0.90791E+00_JPRB, 0.90850E+00_JPRB, 0.91113E+00_JPRB, 0.90361E+00_JPRB, 0.89856E+00_JPRB, 0.89807E+00_JPRB, 0.89769E+00_JPRB ,&
&  0.90223E+00_JPRB, 0.90126E+00_JPRB, 0.90844E+00_JPRB, 0.90652E+00_JPRB, 0.90300E+00_JPRB, 0.90513E+00_JPRB, 0.85617E+00_JPRB ,&
&  0.84148E+00_JPRB, 0.87200E+00_JPRB, 0.86165E+00_JPRB, 0.87248E+00_JPRB, 0.89397E+00_JPRB, 0.90671E+00_JPRB, 0.91606E+00_JPRB ,&
&  0.92166E+00_JPRB, 0.92887E+00_JPRB, 0.93994E+00_JPRB, 0.95626E+00_JPRB, 0.97852E+00_JPRB, 0.10013E+01_JPRB, 0.10224E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10591E+01_JPRB, 0.10808E+01_JPRB, 0.11789E+01_JPRB, 0.14331E+01_JPRB, 0.15535E+01_JPRB, 0.16334E+01_JPRB ,&
&  0.17188E+01_JPRB, 0.17693E+01_JPRB, 0.17855E+01_JPRB, 0.18102E+01_JPRB, 0.17952E+01_JPRB, 0.17772E+01_JPRB, 0.17911E+01_JPRB ,&
&  0.17762E+01_JPRB, 0.17938E+01_JPRB, 0.17944E+01_JPRB, 0.18106E+01_JPRB, 0.17858E+01_JPRB, 0.17790E+01_JPRB, 0.17722E+01_JPRB ,&
&  0.17458E+01_JPRB, 0.16931E+01_JPRB, 0.15978E+01_JPRB, 0.15020E+01_JPRB, 0.14158E+01_JPRB, 0.13664E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.12835E+01_JPRB, 0.12529E+01_JPRB, 0.12194E+01_JPRB, 0.11756E+01_JPRB, 0.11166E+01_JPRB, 0.10680E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.96573E+00_JPRB, 0.94522E+00_JPRB, 0.95240E+00_JPRB, 0.96298E+00_JPRB, 0.97999E+00_JPRB, 0.10088E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.10386E+01_JPRB, 0.10597E+01_JPRB, 0.10718E+01_JPRB, 0.11012E+01_JPRB, 0.11453E+01_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.13004E+01_JPRB, 0.12970E+01_JPRB, 0.12748E+01_JPRB, 0.12528E+01_JPRB, 0.12178E+01_JPRB, 0.11452E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10529E+01_JPRB, 0.10298E+01_JPRB, 0.97242E+00_JPRB, 0.94264E+00_JPRB, 0.91980E+00_JPRB, 0.89836E+00_JPRB ,&
&  0.89352E+00_JPRB, 0.90344E+00_JPRB, 0.91467E+00_JPRB, 0.92638E+00_JPRB, 0.93122E+00_JPRB, 0.93024E+00_JPRB, 0.92763E+00_JPRB ,&
&  0.92619E+00_JPRB, 0.93096E+00_JPRB, 0.93624E+00_JPRB, 0.94007E+00_JPRB, 0.94231E+00_JPRB, 0.94118E+00_JPRB, 0.93439E+00_JPRB ,&
&  0.92722E+00_JPRB, 0.92619E+00_JPRB, 0.93375E+00_JPRB, 0.92860E+00_JPRB, 0.92721E+00_JPRB, 0.92334E+00_JPRB, 0.91480E+00_JPRB ,&
&  0.88716E+00_JPRB, 0.88616E+00_JPRB, 0.89421E+00_JPRB, 0.90721E+00_JPRB, 0.92177E+00_JPRB, 0.93309E+00_JPRB, 0.94301E+00_JPRB ,&
&  0.95573E+00_JPRB, 0.97147E+00_JPRB, 0.99074E+00_JPRB, 0.10119E+01_JPRB, 0.10376E+01_JPRB, 0.10785E+01_JPRB, 0.11137E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.11844E+01_JPRB, 0.13651E+01_JPRB, 0.15201E+01_JPRB, 0.17514E+01_JPRB, 0.19376E+01_JPRB, 0.19986E+01_JPRB ,&
&  0.21568E+01_JPRB, 0.21947E+01_JPRB, 0.22207E+01_JPRB, 0.21993E+01_JPRB, 0.21669E+01_JPRB, 0.21566E+01_JPRB, 0.21820E+01_JPRB ,&
&  0.22184E+01_JPRB, 0.21800E+01_JPRB, 0.23019E+01_JPRB, 0.22698E+01_JPRB, 0.22231E+01_JPRB, 0.22070E+01_JPRB, 0.21755E+01_JPRB ,&
&  0.21168E+01_JPRB, 0.20236E+01_JPRB, 0.18991E+01_JPRB, 0.16966E+01_JPRB, 0.16123E+01_JPRB, 0.15378E+01_JPRB, 0.14201E+01_JPRB ,&
&  0.13527E+01_JPRB, 0.12801E+01_JPRB, 0.12142E+01_JPRB, 0.12010E+01_JPRB, 0.11234E+01_JPRB, 0.10672E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.98995E+00_JPRB, 0.98987E+00_JPRB, 0.10292E+01_JPRB, 0.10719E+01_JPRB, 0.11000E+01_JPRB, 0.11065E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11602E+01_JPRB, 0.12281E+01_JPRB, 0.12575E+01_JPRB, 0.12813E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.14525E+01_JPRB, 0.14361E+01_JPRB, 0.14055E+01_JPRB, 0.13671E+01_JPRB, 0.12944E+01_JPRB, 0.12042E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.11210E+01_JPRB, 0.11009E+01_JPRB, 0.10686E+01_JPRB, 0.10309E+01_JPRB, 0.98050E+00_JPRB, 0.92185E+00_JPRB ,&
&  0.91013E+00_JPRB, 0.93289E+00_JPRB, 0.95933E+00_JPRB, 0.97566E+00_JPRB, 0.97142E+00_JPRB, 0.96745E+00_JPRB, 0.97129E+00_JPRB ,&
&  0.97652E+00_JPRB, 0.98244E+00_JPRB, 0.99154E+00_JPRB, 0.99824E+00_JPRB, 0.99555E+00_JPRB, 0.99437E+00_JPRB, 0.99891E+00_JPRB ,&
&  0.97368E+00_JPRB, 0.95833E+00_JPRB, 0.95455E+00_JPRB, 0.94636E+00_JPRB, 0.97706E+00_JPRB, 0.99052E+00_JPRB, 0.99614E+00_JPRB ,&
&  0.98971E+00_JPRB, 0.97904E+00_JPRB, 0.97409E+00_JPRB, 0.98326E+00_JPRB, 0.99287E+00_JPRB, 0.99683E+00_JPRB, 0.10061E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10451E+01_JPRB, 0.10739E+01_JPRB, 0.11161E+01_JPRB, 0.11613E+01_JPRB, 0.12082E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.14291E+01_JPRB, 0.16279E+01_JPRB, 0.18618E+01_JPRB, 0.19156E+01_JPRB, 0.21240E+01_JPRB, 0.22976E+01_JPRB, 0.23437E+01_JPRB ,&
&  0.24874E+01_JPRB, 0.25258E+01_JPRB, 0.26560E+01_JPRB, 0.26926E+01_JPRB, 0.27294E+01_JPRB, 0.27572E+01_JPRB, 0.27670E+01_JPRB ,&
&  0.27786E+01_JPRB, 0.28150E+01_JPRB, 0.28654E+01_JPRB, 0.28447E+01_JPRB, 0.27509E+01_JPRB, 0.26397E+01_JPRB, 0.25041E+01_JPRB ,&
&  0.23833E+01_JPRB, 0.22510E+01_JPRB, 0.21181E+01_JPRB, 0.20088E+01_JPRB, 0.18746E+01_JPRB, 0.17180E+01_JPRB, 0.15604E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.12346E+01_JPRB, 0.11555E+01_JPRB, 0.11441E+01_JPRB, 0.11222E+01_JPRB, 0.10522E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.10763E+01_JPRB, 0.11535E+01_JPRB, 0.12039E+01_JPRB, 0.12865E+01_JPRB, 0.13394E+01_JPRB, 0.12591E+01_JPRB ,&
&  0.12048E+01_JPRB, 0.13091E+01_JPRB, 0.13747E+01_JPRB, 0.14160E+01_JPRB, 0.14451E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.15892E+01_JPRB, 0.15614E+01_JPRB, 0.15311E+01_JPRB, 0.14925E+01_JPRB, 0.14145E+01_JPRB, 0.13089E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.11693E+01_JPRB, 0.11396E+01_JPRB, 0.11116E+01_JPRB, 0.10833E+01_JPRB, 0.10506E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.10310E+01_JPRB, 0.10218E+01_JPRB, 0.10309E+01_JPRB, 0.10448E+01_JPRB, 0.10304E+01_JPRB, 0.10313E+01_JPRB ,&
&  0.10571E+01_JPRB, 0.10645E+01_JPRB, 0.10611E+01_JPRB, 0.10518E+01_JPRB, 0.10466E+01_JPRB, 0.10552E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.10109E+01_JPRB, 0.98309E+00_JPRB, 0.97616E+00_JPRB, 0.10092E+01_JPRB, 0.10571E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.11005E+01_JPRB, 0.10973E+01_JPRB, 0.10953E+01_JPRB, 0.10891E+01_JPRB, 0.10823E+01_JPRB, 0.10803E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.11036E+01_JPRB, 0.11306E+01_JPRB, 0.11815E+01_JPRB, 0.12334E+01_JPRB, 0.13380E+01_JPRB, 0.15293E+01_JPRB ,&
&  0.17586E+01_JPRB, 0.20009E+01_JPRB, 0.22094E+01_JPRB, 0.24682E+01_JPRB, 0.26699E+01_JPRB, 0.27506E+01_JPRB, 0.27828E+01_JPRB ,&
&  0.28979E+01_JPRB, 0.28816E+01_JPRB, 0.30890E+01_JPRB, 0.30829E+01_JPRB, 0.31110E+01_JPRB, 0.32424E+01_JPRB, 0.32383E+01_JPRB ,&
&  0.31857E+01_JPRB, 0.30923E+01_JPRB, 0.31878E+01_JPRB, 0.31603E+01_JPRB, 0.30763E+01_JPRB, 0.29707E+01_JPRB, 0.28045E+01_JPRB ,&
&  0.26411E+01_JPRB, 0.24718E+01_JPRB, 0.21503E+01_JPRB, 0.19133E+01_JPRB, 0.16505E+01_JPRB, 0.16296E+01_JPRB, 0.14946E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.11744E+01_JPRB, 0.11573E+01_JPRB, 0.11503E+01_JPRB, 0.11682E+01_JPRB, 0.11384E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.12043E+01_JPRB, 0.12565E+01_JPRB, 0.13356E+01_JPRB, 0.14612E+01_JPRB, 0.15675E+01_JPRB, 0.13603E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.15064E+01_JPRB, 0.15668E+01_JPRB, 0.15976E+01_JPRB, 0.16040E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.16278E+01_JPRB, 0.15830E+01_JPRB, 0.15406E+01_JPRB, 0.15026E+01_JPRB, 0.14532E+01_JPRB, 0.13913E+01_JPRB, 0.13131E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.11914E+01_JPRB, 0.11620E+01_JPRB, 0.11486E+01_JPRB, 0.11349E+01_JPRB, 0.11051E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.11174E+01_JPRB, 0.10559E+01_JPRB, 0.10831E+01_JPRB, 0.11832E+01_JPRB, 0.11197E+01_JPRB, 0.11158E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11395E+01_JPRB, 0.11257E+01_JPRB, 0.11072E+01_JPRB, 0.11002E+01_JPRB, 0.11133E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.10693E+01_JPRB, 0.10151E+01_JPRB, 0.99844E+00_JPRB, 0.10348E+01_JPRB, 0.10862E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11629E+01_JPRB, 0.11698E+01_JPRB, 0.11713E+01_JPRB, 0.11654E+01_JPRB, 0.11616E+01_JPRB, 0.11628E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.11830E+01_JPRB, 0.12147E+01_JPRB, 0.12685E+01_JPRB, 0.12991E+01_JPRB, 0.14658E+01_JPRB, 0.18586E+01_JPRB ,&
&  0.23981E+01_JPRB, 0.25523E+01_JPRB, 0.27808E+01_JPRB, 0.32960E+01_JPRB, 0.35083E+01_JPRB, 0.35636E+01_JPRB, 0.35708E+01_JPRB ,&
&  0.35953E+01_JPRB, 0.35610E+01_JPRB, 0.35572E+01_JPRB, 0.36917E+01_JPRB, 0.36729E+01_JPRB, 0.36820E+01_JPRB, 0.35728E+01_JPRB ,&
&  0.33814E+01_JPRB, 0.33287E+01_JPRB, 0.33260E+01_JPRB, 0.32154E+01_JPRB, 0.30559E+01_JPRB, 0.30061E+01_JPRB, 0.29226E+01_JPRB ,&
&  0.27618E+01_JPRB, 0.24659E+01_JPRB, 0.18914E+01_JPRB, 0.17047E+01_JPRB, 0.13482E+01_JPRB, 0.13490E+01_JPRB, 0.15462E+01_JPRB ,&
&  0.14933E+01_JPRB, 0.13811E+01_JPRB, 0.13122E+01_JPRB, 0.12902E+01_JPRB, 0.13024E+01_JPRB, 0.13669E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.13430E+01_JPRB, 0.14385E+01_JPRB, 0.15829E+01_JPRB, 0.17550E+01_JPRB, 0.18227E+01_JPRB, 0.16401E+01_JPRB ,&
&  0.17119E+01_JPRB, 0.17439E+01_JPRB, 0.17476E+01_JPRB, 0.17352E+01_JPRB, 0.16889E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.15529E+01_JPRB, 0.15097E+01_JPRB, 0.14620E+01_JPRB, 0.14469E+01_JPRB, 0.14511E+01_JPRB, 0.14337E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.12847E+01_JPRB, 0.12182E+01_JPRB, 0.11831E+01_JPRB, 0.11813E+01_JPRB, 0.11869E+01_JPRB, 0.11658E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.12316E+01_JPRB, 0.12719E+01_JPRB, 0.11613E+01_JPRB, 0.10813E+01_JPRB, 0.11524E+01_JPRB, 0.11330E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.11995E+01_JPRB, 0.11984E+01_JPRB, 0.12096E+01_JPRB, 0.12305E+01_JPRB, 0.12834E+01_JPRB, 0.13937E+01_JPRB ,&
&  0.14078E+01_JPRB, 0.12785E+01_JPRB, 0.11770E+01_JPRB, 0.11538E+01_JPRB, 0.12000E+01_JPRB, 0.11601E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.12201E+01_JPRB, 0.12341E+01_JPRB, 0.12508E+01_JPRB, 0.12627E+01_JPRB, 0.12707E+01_JPRB, 0.12822E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.13198E+01_JPRB, 0.13201E+01_JPRB, 0.13547E+01_JPRB, 0.14740E+01_JPRB, 0.17840E+01_JPRB, 0.22475E+01_JPRB ,&
&  0.26047E+01_JPRB, 0.27255E+01_JPRB, 0.28931E+01_JPRB, 0.33800E+01_JPRB, 0.37406E+01_JPRB, 0.39300E+01_JPRB, 0.41510E+01_JPRB ,&
&  0.42107E+01_JPRB, 0.42091E+01_JPRB, 0.41801E+01_JPRB, 0.39128E+01_JPRB, 0.35074E+01_JPRB, 0.32176E+01_JPRB, 0.30818E+01_JPRB ,&
&  0.29438E+01_JPRB, 0.29235E+01_JPRB, 0.30111E+01_JPRB, 0.29419E+01_JPRB, 0.29064E+01_JPRB, 0.27620E+01_JPRB, 0.28735E+01_JPRB ,&
&  0.28065E+01_JPRB, 0.17965E+01_JPRB, 0.14450E+01_JPRB, 0.15451E+01_JPRB, 0.13091E+01_JPRB, 0.13246E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.14405E+01_JPRB, 0.14318E+01_JPRB, 0.14926E+01_JPRB, 0.15535E+01_JPRB, 0.15822E+01_JPRB, 0.17283E+01_JPRB, 0.17907E+01_JPRB ,&
&  0.17463E+01_JPRB, 0.16598E+01_JPRB, 0.17887E+01_JPRB, 0.19738E+01_JPRB, 0.21350E+01_JPRB, 0.21224E+01_JPRB, 0.20238E+01_JPRB ,&
&  0.19632E+01_JPRB, 0.19155E+01_JPRB, 0.18551E+01_JPRB, 0.17472E+01_JPRB, 0.16275E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.15505E+01_JPRB, 0.15182E+01_JPRB, 0.14637E+01_JPRB, 0.14679E+01_JPRB, 0.14980E+01_JPRB, 0.14867E+01_JPRB, 0.14155E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.12255E+01_JPRB, 0.11948E+01_JPRB, 0.11935E+01_JPRB, 0.12102E+01_JPRB, 0.12158E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.12633E+01_JPRB, 0.11621E+01_JPRB, 0.11124E+01_JPRB, 0.10367E+01_JPRB, 0.10868E+01_JPRB, 0.11558E+01_JPRB ,&
&  0.12117E+01_JPRB, 0.12495E+01_JPRB, 0.13499E+01_JPRB, 0.14864E+01_JPRB, 0.16158E+01_JPRB, 0.17344E+01_JPRB, 0.19460E+01_JPRB ,&
&  0.19000E+01_JPRB, 0.17086E+01_JPRB, 0.14558E+01_JPRB, 0.14136E+01_JPRB, 0.13912E+01_JPRB, 0.12911E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.13618E+01_JPRB, 0.13805E+01_JPRB, 0.13908E+01_JPRB, 0.13887E+01_JPRB, 0.13582E+01_JPRB, 0.13470E+01_JPRB ,&
&  0.13515E+01_JPRB, 0.13728E+01_JPRB, 0.14035E+01_JPRB, 0.14565E+01_JPRB, 0.17620E+01_JPRB, 0.21643E+01_JPRB, 0.24814E+01_JPRB ,&
&  0.27875E+01_JPRB, 0.24888E+01_JPRB, 0.23128E+01_JPRB, 0.30518E+01_JPRB, 0.43601E+01_JPRB, 0.47901E+01_JPRB, 0.45759E+01_JPRB ,&
&  0.49259E+01_JPRB, 0.45816E+01_JPRB, 0.40032E+01_JPRB, 0.35608E+01_JPRB, 0.32627E+01_JPRB, 0.30095E+01_JPRB, 0.28904E+01_JPRB ,&
&  0.26976E+01_JPRB, 0.26653E+01_JPRB, 0.27835E+01_JPRB, 0.28617E+01_JPRB, 0.27925E+01_JPRB, 0.27040E+01_JPRB, 0.27440E+01_JPRB ,&
&  0.26099E+01_JPRB, 0.22005E+01_JPRB, 0.16170E+01_JPRB, 0.15429E+01_JPRB, 0.12895E+01_JPRB, 0.12804E+01_JPRB, 0.14576E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.15668E+01_JPRB, 0.16533E+01_JPRB, 0.16822E+01_JPRB, 0.19362E+01_JPRB, 0.21186E+01_JPRB, 0.23580E+01_JPRB ,&
&  0.24834E+01_JPRB, 0.21798E+01_JPRB, 0.23490E+01_JPRB, 0.25292E+01_JPRB, 0.25566E+01_JPRB, 0.23804E+01_JPRB, 0.21877E+01_JPRB ,&
&  0.20694E+01_JPRB, 0.19792E+01_JPRB, 0.18646E+01_JPRB, 0.17098E+01_JPRB, 0.16021E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.15807E+01_JPRB, 0.15804E+01_JPRB, 0.15269E+01_JPRB, 0.15276E+01_JPRB, 0.15257E+01_JPRB, 0.14813E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.13390E+01_JPRB, 0.12974E+01_JPRB, 0.12660E+01_JPRB, 0.12497E+01_JPRB, 0.12517E+01_JPRB, 0.12457E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11957E+01_JPRB, 0.10927E+01_JPRB, 0.10256E+01_JPRB, 0.98777E+00_JPRB, 0.10257E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.14453E+01_JPRB, 0.17584E+01_JPRB, 0.20309E+01_JPRB, 0.24002E+01_JPRB, 0.26955E+01_JPRB, 0.28668E+01_JPRB ,&
&  0.28673E+01_JPRB, 0.21989E+01_JPRB, 0.19882E+01_JPRB, 0.17753E+01_JPRB, 0.16806E+01_JPRB, 0.15742E+01_JPRB, 0.15207E+01_JPRB ,&
&  0.14800E+01_JPRB, 0.14725E+01_JPRB, 0.14632E+01_JPRB, 0.14326E+01_JPRB, 0.14091E+01_JPRB, 0.13978E+01_JPRB, 0.14299E+01_JPRB ,&
&  0.14634E+01_JPRB, 0.14620E+01_JPRB, 0.15130E+01_JPRB, 0.17560E+01_JPRB, 0.22522E+01_JPRB, 0.26765E+01_JPRB, 0.29246E+01_JPRB ,&
&  0.27688E+01_JPRB, 0.26690E+01_JPRB, 0.38966E+01_JPRB, 0.43718E+01_JPRB, 0.42781E+01_JPRB, 0.49963E+01_JPRB, 0.52449E+01_JPRB ,&
&  0.48664E+01_JPRB, 0.39489E+01_JPRB, 0.33966E+01_JPRB, 0.30153E+01_JPRB, 0.33997E+01_JPRB, 0.31450E+01_JPRB, 0.28379E+01_JPRB ,&
&  0.25846E+01_JPRB, 0.27153E+01_JPRB, 0.27221E+01_JPRB, 0.27838E+01_JPRB, 0.28695E+01_JPRB, 0.29226E+01_JPRB, 0.26258E+01_JPRB ,&
&  0.23641E+01_JPRB, 0.26615E+01_JPRB, 0.21753E+01_JPRB, 0.17194E+01_JPRB, 0.14912E+01_JPRB, 0.15098E+01_JPRB, 0.16142E+01_JPRB ,&
&  0.17125E+01_JPRB, 0.18192E+01_JPRB, 0.18787E+01_JPRB, 0.20494E+01_JPRB, 0.24708E+01_JPRB, 0.27223E+01_JPRB, 0.28491E+01_JPRB ,&
&  0.28121E+01_JPRB, 0.30678E+01_JPRB, 0.30749E+01_JPRB, 0.29855E+01_JPRB, 0.26893E+01_JPRB, 0.23325E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.20294E+01_JPRB, 0.19481E+01_JPRB, 0.17980E+01_JPRB, 0.16499E+01_JPRB, 0.15702E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.15994E+01_JPRB, 0.16160E+01_JPRB, 0.16049E+01_JPRB, 0.15822E+01_JPRB, 0.15473E+01_JPRB, 0.14987E+01_JPRB, 0.14579E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.13410E+01_JPRB, 0.13257E+01_JPRB, 0.13244E+01_JPRB, 0.13195E+01_JPRB, 0.12961E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.11626E+01_JPRB, 0.10455E+01_JPRB, 0.10499E+01_JPRB, 0.10605E+01_JPRB, 0.10539E+01_JPRB, 0.11555E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.17634E+01_JPRB, 0.22473E+01_JPRB, 0.27863E+01_JPRB, 0.31718E+01_JPRB, 0.33016E+01_JPRB, 0.32971E+01_JPRB ,&
&  0.29675E+01_JPRB, 0.26687E+01_JPRB, 0.23925E+01_JPRB, 0.21583E+01_JPRB, 0.19873E+01_JPRB, 0.18651E+01_JPRB, 0.17398E+01_JPRB ,&
&  0.16315E+01_JPRB, 0.15586E+01_JPRB, 0.15170E+01_JPRB, 0.14601E+01_JPRB, 0.14465E+01_JPRB, 0.14666E+01_JPRB, 0.15035E+01_JPRB ,&
&  0.14997E+01_JPRB, 0.15176E+01_JPRB, 0.16789E+01_JPRB, 0.20644E+01_JPRB, 0.22498E+01_JPRB, 0.25599E+01_JPRB, 0.28337E+01_JPRB ,&
&  0.37400E+01_JPRB, 0.40463E+01_JPRB, 0.39450E+01_JPRB, 0.49540E+01_JPRB, 0.34891E+01_JPRB, 0.41475E+01_JPRB, 0.48879E+01_JPRB ,&
&  0.44970E+01_JPRB, 0.40875E+01_JPRB, 0.40918E+01_JPRB, 0.34737E+01_JPRB, 0.29484E+01_JPRB, 0.38275E+01_JPRB, 0.32380E+01_JPRB ,&
&  0.28421E+01_JPRB, 0.28528E+01_JPRB, 0.29079E+01_JPRB, 0.31983E+01_JPRB, 0.27380E+01_JPRB, 0.24810E+01_JPRB, 0.20623E+01_JPRB ,&
&  0.21260E+01_JPRB, 0.19879E+01_JPRB, 0.21973E+01_JPRB, 0.19700E+01_JPRB, 0.20077E+01_JPRB, 0.19235E+01_JPRB, 0.18959E+01_JPRB ,&
&  0.20510E+01_JPRB, 0.21170E+01_JPRB, 0.21580E+01_JPRB, 0.25891E+01_JPRB, 0.33583E+01_JPRB, 0.30737E+01_JPRB, 0.33506E+01_JPRB ,&
&  0.35030E+01_JPRB, 0.34797E+01_JPRB, 0.31626E+01_JPRB, 0.28933E+01_JPRB, 0.24520E+01_JPRB, 0.21813E+01_JPRB, 0.20659E+01_JPRB ,&
&  0.19809E+01_JPRB, 0.18325E+01_JPRB, 0.16703E+01_JPRB, 0.16046E+01_JPRB, 0.15543E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.15908E+01_JPRB, 0.15775E+01_JPRB, 0.15709E+01_JPRB, 0.15466E+01_JPRB, 0.15404E+01_JPRB, 0.15050E+01_JPRB, 0.14695E+01_JPRB ,&
&  0.14198E+01_JPRB, 0.13720E+01_JPRB, 0.13622E+01_JPRB, 0.13548E+01_JPRB, 0.13453E+01_JPRB, 0.13367E+01_JPRB, 0.13002E+01_JPRB ,&
&  0.12973E+01_JPRB, 0.12483E+01_JPRB, 0.10964E+01_JPRB, 0.10568E+01_JPRB, 0.11016E+01_JPRB, 0.11150E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.16705E+01_JPRB, 0.23203E+01_JPRB, 0.27903E+01_JPRB, 0.32260E+01_JPRB, 0.34486E+01_JPRB, 0.33133E+01_JPRB, 0.31111E+01_JPRB ,&
&  0.30364E+01_JPRB, 0.25742E+01_JPRB, 0.23024E+01_JPRB, 0.21628E+01_JPRB, 0.20076E+01_JPRB, 0.18716E+01_JPRB, 0.17597E+01_JPRB ,&
&  0.16429E+01_JPRB, 0.15352E+01_JPRB, 0.14658E+01_JPRB, 0.14008E+01_JPRB, 0.13975E+01_JPRB, 0.14281E+01_JPRB, 0.14744E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.16231E+01_JPRB, 0.19196E+01_JPRB, 0.22948E+01_JPRB, 0.25596E+01_JPRB, 0.28340E+01_JPRB, 0.35008E+01_JPRB ,&
&  0.38649E+01_JPRB, 0.38346E+01_JPRB, 0.42102E+01_JPRB, 0.47315E+01_JPRB, 0.47585E+01_JPRB, 0.42551E+01_JPRB, 0.46064E+01_JPRB ,&
&  0.34939E+01_JPRB, 0.29997E+01_JPRB, 0.27899E+01_JPRB, 0.26687E+01_JPRB, 0.31950E+01_JPRB, 0.46803E+01_JPRB, 0.41969E+01_JPRB ,&
&  0.38198E+01_JPRB, 0.34217E+01_JPRB, 0.34578E+01_JPRB, 0.32384E+01_JPRB, 0.29324E+01_JPRB, 0.22640E+01_JPRB, 0.28262E+01_JPRB ,&
&  0.30285E+01_JPRB, 0.31776E+01_JPRB, 0.28963E+01_JPRB, 0.24142E+01_JPRB, 0.21111E+01_JPRB, 0.24053E+01_JPRB, 0.25227E+01_JPRB ,&
&  0.26135E+01_JPRB, 0.25900E+01_JPRB, 0.33481E+01_JPRB, 0.38438E+01_JPRB, 0.40328E+01_JPRB, 0.38267E+01_JPRB, 0.41144E+01_JPRB ,&
&  0.37516E+01_JPRB, 0.35892E+01_JPRB, 0.31957E+01_JPRB, 0.26498E+01_JPRB, 0.22799E+01_JPRB, 0.21103E+01_JPRB, 0.19893E+01_JPRB ,&
&  0.18573E+01_JPRB, 0.16816E+01_JPRB, 0.15867E+01_JPRB, 0.15331E+01_JPRB, 0.15252E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.15488E+01_JPRB, 0.15231E+01_JPRB, 0.14836E+01_JPRB, 0.14707E+01_JPRB, 0.14623E+01_JPRB, 0.14284E+01_JPRB, 0.14170E+01_JPRB ,&
&  0.14278E+01_JPRB, 0.13891E+01_JPRB, 0.13454E+01_JPRB, 0.13169E+01_JPRB, 0.13056E+01_JPRB, 0.13475E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.14132E+01_JPRB, 0.12110E+01_JPRB, 0.11627E+01_JPRB, 0.11908E+01_JPRB, 0.12614E+01_JPRB, 0.13887E+01_JPRB ,&
&  0.20805E+01_JPRB, 0.27597E+01_JPRB, 0.29770E+01_JPRB, 0.31863E+01_JPRB, 0.33282E+01_JPRB, 0.31519E+01_JPRB, 0.30163E+01_JPRB ,&
&  0.29005E+01_JPRB, 0.25457E+01_JPRB, 0.22603E+01_JPRB, 0.20708E+01_JPRB, 0.19150E+01_JPRB, 0.18034E+01_JPRB, 0.16994E+01_JPRB ,&
&  0.15767E+01_JPRB, 0.14808E+01_JPRB, 0.14124E+01_JPRB, 0.13731E+01_JPRB, 0.13431E+01_JPRB, 0.14180E+01_JPRB, 0.15230E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.18754E+01_JPRB, 0.21547E+01_JPRB, 0.25335E+01_JPRB, 0.29203E+01_JPRB, 0.31738E+01_JPRB, 0.42253E+01_JPRB ,&
&  0.41340E+01_JPRB, 0.41736E+01_JPRB, 0.43796E+01_JPRB, 0.54723E+01_JPRB, 0.45055E+01_JPRB, 0.40150E+01_JPRB, 0.41725E+01_JPRB ,&
&  0.31135E+01_JPRB, 0.27984E+01_JPRB, 0.30931E+01_JPRB, 0.35910E+01_JPRB, 0.35315E+01_JPRB, 0.38883E+01_JPRB, 0.51838E+01_JPRB ,&
&  0.44470E+01_JPRB, 0.40215E+01_JPRB, 0.39231E+01_JPRB, 0.40596E+01_JPRB, 0.22235E+01_JPRB, 0.18711E+01_JPRB, 0.28877E+01_JPRB ,&
&  0.30922E+01_JPRB, 0.21262E+01_JPRB, 0.16542E+01_JPRB, 0.20142E+01_JPRB, 0.16184E+01_JPRB, 0.19907E+01_JPRB, 0.29008E+01_JPRB ,&
&  0.31694E+01_JPRB, 0.34508E+01_JPRB, 0.52595E+01_JPRB, 0.50389E+01_JPRB, 0.46876E+01_JPRB, 0.55172E+01_JPRB, 0.46987E+01_JPRB ,&
&  0.41099E+01_JPRB, 0.31944E+01_JPRB, 0.28617E+01_JPRB, 0.23999E+01_JPRB, 0.21315E+01_JPRB, 0.19417E+01_JPRB, 0.18035E+01_JPRB ,&
&  0.16576E+01_JPRB, 0.15692E+01_JPRB, 0.15589E+01_JPRB, 0.15513E+01_JPRB, 0.15322E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.15191E+01_JPRB, 0.14600E+01_JPRB, 0.13968E+01_JPRB, 0.13920E+01_JPRB, 0.13713E+01_JPRB, 0.13413E+01_JPRB, 0.14201E+01_JPRB ,&
&  0.14744E+01_JPRB, 0.13751E+01_JPRB, 0.12894E+01_JPRB, 0.12685E+01_JPRB, 0.12909E+01_JPRB, 0.14238E+01_JPRB, 0.15054E+01_JPRB ,&
&  0.15530E+01_JPRB, 0.15345E+01_JPRB, 0.14927E+01_JPRB, 0.14721E+01_JPRB, 0.15148E+01_JPRB, 0.16125E+01_JPRB, 0.19138E+01_JPRB ,&
&  0.27349E+01_JPRB, 0.30184E+01_JPRB, 0.29148E+01_JPRB, 0.28736E+01_JPRB, 0.29209E+01_JPRB, 0.28378E+01_JPRB, 0.28954E+01_JPRB ,&
&  0.26973E+01_JPRB, 0.23781E+01_JPRB, 0.21312E+01_JPRB, 0.19514E+01_JPRB, 0.18649E+01_JPRB, 0.17788E+01_JPRB, 0.16784E+01_JPRB ,&
&  0.15388E+01_JPRB, 0.14361E+01_JPRB, 0.13946E+01_JPRB, 0.13488E+01_JPRB, 0.13712E+01_JPRB, 0.14725E+01_JPRB, 0.16128E+01_JPRB ,&
&  0.18222E+01_JPRB, 0.21138E+01_JPRB, 0.23766E+01_JPRB, 0.28245E+01_JPRB, 0.33932E+01_JPRB, 0.36046E+01_JPRB, 0.34194E+01_JPRB ,&
&  0.32580E+01_JPRB, 0.32636E+01_JPRB, 0.43044E+01_JPRB, 0.47916E+01_JPRB, 0.44722E+01_JPRB, 0.42732E+01_JPRB, 0.40433E+01_JPRB ,&
&  0.35615E+01_JPRB, 0.33679E+01_JPRB, 0.33675E+01_JPRB, 0.40175E+01_JPRB, 0.42301E+01_JPRB, 0.38840E+01_JPRB, 0.47932E+01_JPRB ,&
&  0.49095E+01_JPRB, 0.44025E+01_JPRB, 0.33435E+01_JPRB, 0.26811E+01_JPRB, 0.25091E+01_JPRB, 0.23127E+01_JPRB, 0.17175E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.15594E+01_JPRB, 0.14733E+01_JPRB, 0.15058E+01_JPRB, 0.14990E+01_JPRB, 0.21086E+01_JPRB, 0.29633E+01_JPRB ,&
&  0.42468E+01_JPRB, 0.51291E+01_JPRB, 0.59646E+01_JPRB, 0.57836E+01_JPRB, 0.52394E+01_JPRB, 0.63670E+01_JPRB, 0.50036E+01_JPRB ,&
&  0.34250E+01_JPRB, 0.27949E+01_JPRB, 0.24864E+01_JPRB, 0.21845E+01_JPRB, 0.19626E+01_JPRB, 0.17628E+01_JPRB, 0.16091E+01_JPRB ,&
&  0.15339E+01_JPRB, 0.15351E+01_JPRB, 0.15489E+01_JPRB, 0.15471E+01_JPRB, 0.15001E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.14901E+01_JPRB, 0.14341E+01_JPRB, 0.13753E+01_JPRB, 0.13312E+01_JPRB, 0.12800E+01_JPRB, 0.13059E+01_JPRB, 0.14542E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.13626E+01_JPRB, 0.12602E+01_JPRB, 0.12661E+01_JPRB, 0.13899E+01_JPRB, 0.15373E+01_JPRB, 0.15914E+01_JPRB ,&
&  0.16679E+01_JPRB, 0.16717E+01_JPRB, 0.18031E+01_JPRB, 0.19321E+01_JPRB, 0.21965E+01_JPRB, 0.23021E+01_JPRB, 0.28693E+01_JPRB ,&
&  0.33554E+01_JPRB, 0.30534E+01_JPRB, 0.27802E+01_JPRB, 0.26894E+01_JPRB, 0.27548E+01_JPRB, 0.27632E+01_JPRB, 0.27209E+01_JPRB ,&
&  0.24366E+01_JPRB, 0.21870E+01_JPRB, 0.19859E+01_JPRB, 0.18625E+01_JPRB, 0.17625E+01_JPRB, 0.16229E+01_JPRB, 0.15303E+01_JPRB ,&
&  0.14386E+01_JPRB, 0.14073E+01_JPRB, 0.13794E+01_JPRB, 0.13998E+01_JPRB, 0.15015E+01_JPRB, 0.16368E+01_JPRB, 0.18144E+01_JPRB ,&
&  0.20441E+01_JPRB, 0.23532E+01_JPRB, 0.26605E+01_JPRB, 0.31493E+01_JPRB, 0.30272E+01_JPRB, 0.26898E+01_JPRB, 0.30777E+01_JPRB ,&
&  0.34209E+01_JPRB, 0.37480E+01_JPRB, 0.37709E+01_JPRB, 0.47339E+01_JPRB, 0.46160E+01_JPRB, 0.40658E+01_JPRB, 0.38512E+01_JPRB ,&
&  0.34397E+01_JPRB, 0.37207E+01_JPRB, 0.37761E+01_JPRB, 0.41946E+01_JPRB, 0.52296E+01_JPRB, 0.48425E+01_JPRB, 0.50721E+01_JPRB ,&
&  0.53571E+01_JPRB, 0.46577E+01_JPRB, 0.37132E+01_JPRB, 0.29994E+01_JPRB, 0.37815E+01_JPRB, 0.45791E+01_JPRB, 0.20943E+01_JPRB ,&
&  0.17305E+01_JPRB, 0.16135E+01_JPRB, 0.15874E+01_JPRB, 0.14966E+01_JPRB, 0.15467E+01_JPRB, 0.22101E+01_JPRB, 0.43455E+01_JPRB ,&
&  0.45385E+01_JPRB, 0.67802E+01_JPRB, 0.67492E+01_JPRB, 0.64642E+01_JPRB, 0.56078E+01_JPRB, 0.62993E+01_JPRB, 0.43171E+01_JPRB ,&
&  0.34283E+01_JPRB, 0.28699E+01_JPRB, 0.24280E+01_JPRB, 0.21657E+01_JPRB, 0.18987E+01_JPRB, 0.16617E+01_JPRB, 0.15627E+01_JPRB ,&
&  0.15163E+01_JPRB, 0.14837E+01_JPRB, 0.14740E+01_JPRB, 0.14687E+01_JPRB, 0.14667E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.14809E+01_JPRB, 0.14214E+01_JPRB, 0.13706E+01_JPRB, 0.13321E+01_JPRB, 0.12957E+01_JPRB, 0.13923E+01_JPRB, 0.14998E+01_JPRB ,&
&  0.15229E+01_JPRB, 0.14142E+01_JPRB, 0.13424E+01_JPRB, 0.13871E+01_JPRB, 0.15251E+01_JPRB, 0.16893E+01_JPRB, 0.17849E+01_JPRB ,&
&  0.18888E+01_JPRB, 0.19990E+01_JPRB, 0.22185E+01_JPRB, 0.27446E+01_JPRB, 0.31467E+01_JPRB, 0.33825E+01_JPRB, 0.38799E+01_JPRB ,&
&  0.35228E+01_JPRB, 0.28158E+01_JPRB, 0.26796E+01_JPRB, 0.26783E+01_JPRB, 0.25504E+01_JPRB, 0.24795E+01_JPRB, 0.23423E+01_JPRB ,&
&  0.21883E+01_JPRB, 0.20328E+01_JPRB, 0.18818E+01_JPRB, 0.17797E+01_JPRB, 0.16939E+01_JPRB, 0.15723E+01_JPRB, 0.15170E+01_JPRB ,&
&  0.14666E+01_JPRB, 0.14430E+01_JPRB, 0.14803E+01_JPRB, 0.15250E+01_JPRB, 0.16699E+01_JPRB, 0.18078E+01_JPRB, 0.20120E+01_JPRB ,&
&  0.22810E+01_JPRB, 0.25535E+01_JPRB, 0.28458E+01_JPRB, 0.32085E+01_JPRB, 0.27485E+01_JPRB, 0.32327E+01_JPRB, 0.34070E+01_JPRB ,&
&  0.35974E+01_JPRB, 0.35838E+01_JPRB, 0.33826E+01_JPRB, 0.40189E+01_JPRB, 0.42808E+01_JPRB, 0.38270E+01_JPRB, 0.34740E+01_JPRB ,&
&  0.35722E+01_JPRB, 0.37090E+01_JPRB, 0.42621E+01_JPRB, 0.48796E+01_JPRB, 0.55294E+01_JPRB, 0.71680E+01_JPRB, 0.52035E+01_JPRB ,&
&  0.45268E+01_JPRB, 0.46805E+01_JPRB, 0.42495E+01_JPRB, 0.38116E+01_JPRB, 0.45839E+01_JPRB, 0.73882E+01_JPRB, 0.54193E+01_JPRB ,&
&  0.20517E+01_JPRB, 0.15900E+01_JPRB, 0.18008E+01_JPRB, 0.18389E+01_JPRB, 0.16244E+01_JPRB, 0.22521E+01_JPRB, 0.74352E+01_JPRB ,&
&  0.58571E+01_JPRB, 0.80499E+01_JPRB, 0.70118E+01_JPRB, 0.63214E+01_JPRB, 0.58145E+01_JPRB, 0.58607E+01_JPRB, 0.46127E+01_JPRB ,&
&  0.34532E+01_JPRB, 0.28572E+01_JPRB, 0.24147E+01_JPRB, 0.20898E+01_JPRB, 0.17947E+01_JPRB, 0.16123E+01_JPRB, 0.15281E+01_JPRB ,&
&  0.14777E+01_JPRB, 0.14614E+01_JPRB, 0.14551E+01_JPRB, 0.14469E+01_JPRB, 0.14634E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.14837E+01_JPRB, 0.13860E+01_JPRB, 0.13404E+01_JPRB, 0.13109E+01_JPRB, 0.13820E+01_JPRB, 0.14903E+01_JPRB, 0.15324E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.16432E+01_JPRB, 0.16411E+01_JPRB, 0.16420E+01_JPRB, 0.17590E+01_JPRB, 0.19741E+01_JPRB, 0.21467E+01_JPRB ,&
&  0.23670E+01_JPRB, 0.26180E+01_JPRB, 0.29688E+01_JPRB, 0.35329E+01_JPRB, 0.41180E+01_JPRB, 0.40991E+01_JPRB, 0.39766E+01_JPRB ,&
&  0.33796E+01_JPRB, 0.27347E+01_JPRB, 0.24243E+01_JPRB, 0.25277E+01_JPRB, 0.24906E+01_JPRB, 0.24777E+01_JPRB, 0.23586E+01_JPRB ,&
&  0.23188E+01_JPRB, 0.21533E+01_JPRB, 0.19532E+01_JPRB, 0.18369E+01_JPRB, 0.16646E+01_JPRB, 0.16026E+01_JPRB, 0.16548E+01_JPRB ,&
&  0.16176E+01_JPRB, 0.16289E+01_JPRB, 0.16806E+01_JPRB, 0.17520E+01_JPRB, 0.19117E+01_JPRB, 0.20619E+01_JPRB, 0.22725E+01_JPRB ,&
&  0.25148E+01_JPRB, 0.27264E+01_JPRB, 0.29751E+01_JPRB, 0.30471E+01_JPRB, 0.30876E+01_JPRB, 0.32137E+01_JPRB, 0.34957E+01_JPRB ,&
&  0.35899E+01_JPRB, 0.35557E+01_JPRB, 0.35007E+01_JPRB, 0.36236E+01_JPRB, 0.37053E+01_JPRB, 0.35537E+01_JPRB, 0.33833E+01_JPRB ,&
&  0.38050E+01_JPRB, 0.44734E+01_JPRB, 0.49924E+01_JPRB, 0.55838E+01_JPRB, 0.60308E+01_JPRB, 0.72717E+01_JPRB, 0.67557E+01_JPRB ,&
&  0.54435E+01_JPRB, 0.44733E+01_JPRB, 0.44762E+01_JPRB, 0.53608E+01_JPRB, 0.60669E+01_JPRB, 0.73666E+01_JPRB, 0.87709E+01_JPRB ,&
&  0.87070E+01_JPRB, 0.45853E+01_JPRB, 0.34000E+01_JPRB, 0.49756E+01_JPRB, 0.33234E+01_JPRB, 0.26089E+01_JPRB, 0.42552E+01_JPRB ,&
&  0.62026E+01_JPRB, 0.75689E+01_JPRB, 0.70293E+01_JPRB, 0.59974E+01_JPRB, 0.57925E+01_JPRB, 0.52133E+01_JPRB, 0.40394E+01_JPRB ,&
&  0.31014E+01_JPRB, 0.24942E+01_JPRB, 0.21167E+01_JPRB, 0.18578E+01_JPRB, 0.16264E+01_JPRB, 0.15332E+01_JPRB, 0.15220E+01_JPRB ,&
&  0.14791E+01_JPRB, 0.14988E+01_JPRB, 0.14993E+01_JPRB, 0.14792E+01_JPRB, 0.14934E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.14304E+01_JPRB, 0.13518E+01_JPRB, 0.13122E+01_JPRB, 0.13313E+01_JPRB, 0.14632E+01_JPRB, 0.16164E+01_JPRB, 0.16989E+01_JPRB ,&
&  0.17823E+01_JPRB, 0.18406E+01_JPRB, 0.19133E+01_JPRB, 0.19323E+01_JPRB, 0.21419E+01_JPRB, 0.25133E+01_JPRB, 0.28314E+01_JPRB ,&
&  0.30947E+01_JPRB, 0.33779E+01_JPRB, 0.37276E+01_JPRB, 0.43050E+01_JPRB, 0.51283E+01_JPRB, 0.61682E+01_JPRB, 0.36151E+01_JPRB ,&
&  0.29819E+01_JPRB, 0.27921E+01_JPRB, 0.24485E+01_JPRB, 0.26154E+01_JPRB, 0.25312E+01_JPRB, 0.25053E+01_JPRB, 0.23587E+01_JPRB ,&
&  0.24088E+01_JPRB, 0.22738E+01_JPRB, 0.21354E+01_JPRB, 0.21056E+01_JPRB, 0.19475E+01_JPRB, 0.20038E+01_JPRB, 0.20547E+01_JPRB ,&
&  0.20183E+01_JPRB, 0.20374E+01_JPRB, 0.21457E+01_JPRB, 0.22498E+01_JPRB, 0.23631E+01_JPRB, 0.24714E+01_JPRB, 0.26057E+01_JPRB ,&
&  0.27593E+01_JPRB, 0.29275E+01_JPRB, 0.31523E+01_JPRB, 0.30408E+01_JPRB, 0.30390E+01_JPRB, 0.31122E+01_JPRB, 0.33172E+01_JPRB ,&
&  0.32026E+01_JPRB, 0.29508E+01_JPRB, 0.31318E+01_JPRB, 0.33723E+01_JPRB, 0.33862E+01_JPRB, 0.32790E+01_JPRB, 0.35887E+01_JPRB ,&
&  0.45292E+01_JPRB, 0.54755E+01_JPRB, 0.65925E+01_JPRB, 0.60949E+01_JPRB, 0.62364E+01_JPRB, 0.67211E+01_JPRB, 0.67261E+01_JPRB ,&
&  0.64319E+01_JPRB, 0.57572E+01_JPRB, 0.57231E+01_JPRB, 0.62297E+01_JPRB, 0.62840E+01_JPRB, 0.64917E+01_JPRB, 0.67250E+01_JPRB ,&
&  0.83512E+01_JPRB, 0.10524E+02_JPRB, 0.90438E+01_JPRB, 0.58899E+01_JPRB, 0.44511E+01_JPRB, 0.34878E+01_JPRB, 0.42048E+01_JPRB ,&
&  0.66259E+01_JPRB, 0.63378E+01_JPRB, 0.57575E+01_JPRB, 0.61620E+01_JPRB, 0.48537E+01_JPRB, 0.39917E+01_JPRB, 0.30567E+01_JPRB ,&
&  0.23486E+01_JPRB, 0.18327E+01_JPRB, 0.16477E+01_JPRB, 0.14819E+01_JPRB, 0.14450E+01_JPRB, 0.14607E+01_JPRB, 0.15021E+01_JPRB ,&
&  0.15495E+01_JPRB, 0.15570E+01_JPRB, 0.15349E+01_JPRB, 0.15014E+01_JPRB, 0.14903E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.14229E+01_JPRB, 0.13700E+01_JPRB, 0.13391E+01_JPRB, 0.13989E+01_JPRB, 0.15530E+01_JPRB, 0.16916E+01_JPRB, 0.17528E+01_JPRB ,&
&  0.18070E+01_JPRB, 0.18804E+01_JPRB, 0.20512E+01_JPRB, 0.22305E+01_JPRB, 0.26006E+01_JPRB, 0.30020E+01_JPRB, 0.33732E+01_JPRB ,&
&  0.36609E+01_JPRB, 0.38949E+01_JPRB, 0.42167E+01_JPRB, 0.46989E+01_JPRB, 0.52386E+01_JPRB, 0.57127E+01_JPRB, 0.43355E+01_JPRB ,&
&  0.26342E+01_JPRB, 0.34983E+01_JPRB, 0.26686E+01_JPRB, 0.28860E+01_JPRB, 0.29463E+01_JPRB, 0.31677E+01_JPRB, 0.28768E+01_JPRB ,&
&  0.27988E+01_JPRB, 0.25805E+01_JPRB, 0.25028E+01_JPRB, 0.25257E+01_JPRB, 0.23115E+01_JPRB, 0.23720E+01_JPRB, 0.24739E+01_JPRB ,&
&  0.25526E+01_JPRB, 0.26540E+01_JPRB, 0.27094E+01_JPRB, 0.27271E+01_JPRB, 0.27673E+01_JPRB, 0.28101E+01_JPRB, 0.28852E+01_JPRB ,&
&  0.30262E+01_JPRB, 0.31909E+01_JPRB, 0.32326E+01_JPRB, 0.32119E+01_JPRB, 0.30544E+01_JPRB, 0.31138E+01_JPRB, 0.32133E+01_JPRB ,&
&  0.30161E+01_JPRB, 0.27628E+01_JPRB, 0.29316E+01_JPRB, 0.30371E+01_JPRB, 0.31829E+01_JPRB, 0.38508E+01_JPRB, 0.43951E+01_JPRB ,&
&  0.54569E+01_JPRB, 0.59009E+01_JPRB, 0.73827E+01_JPRB, 0.57436E+01_JPRB, 0.56281E+01_JPRB, 0.59540E+01_JPRB, 0.55553E+01_JPRB ,&
&  0.51357E+01_JPRB, 0.54034E+01_JPRB, 0.56235E+01_JPRB, 0.63726E+01_JPRB, 0.65801E+01_JPRB, 0.62226E+01_JPRB, 0.55703E+01_JPRB ,&
&  0.59974E+01_JPRB, 0.65529E+01_JPRB, 0.58472E+01_JPRB, 0.41348E+01_JPRB, 0.38130E+01_JPRB, 0.39450E+01_JPRB, 0.53663E+01_JPRB ,&
&  0.61368E+01_JPRB, 0.57185E+01_JPRB, 0.53367E+01_JPRB, 0.48133E+01_JPRB, 0.33702E+01_JPRB, 0.25952E+01_JPRB, 0.18780E+01_JPRB ,&
&  0.14034E+01_JPRB, 0.12234E+01_JPRB, 0.13617E+01_JPRB, 0.13241E+01_JPRB, 0.13653E+01_JPRB, 0.14279E+01_JPRB, 0.14481E+01_JPRB ,&
&  0.14893E+01_JPRB, 0.14950E+01_JPRB, 0.15021E+01_JPRB, 0.14801E+01_JPRB, 0.14637E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.13131E+01_JPRB, 0.12944E+01_JPRB, 0.12851E+01_JPRB, 0.13999E+01_JPRB, 0.15971E+01_JPRB, 0.17690E+01_JPRB, 0.18024E+01_JPRB ,&
&  0.18469E+01_JPRB, 0.17839E+01_JPRB, 0.20473E+01_JPRB, 0.23033E+01_JPRB, 0.26638E+01_JPRB, 0.30766E+01_JPRB, 0.35377E+01_JPRB ,&
&  0.39905E+01_JPRB, 0.44087E+01_JPRB, 0.47659E+01_JPRB, 0.51732E+01_JPRB, 0.57110E+01_JPRB, 0.62863E+01_JPRB, 0.56230E+01_JPRB ,&
&  0.40511E+01_JPRB, 0.29588E+01_JPRB, 0.29382E+01_JPRB, 0.29194E+01_JPRB, 0.29395E+01_JPRB, 0.31791E+01_JPRB, 0.32662E+01_JPRB ,&
&  0.32647E+01_JPRB, 0.28258E+01_JPRB, 0.26658E+01_JPRB, 0.27173E+01_JPRB, 0.24406E+01_JPRB, 0.25527E+01_JPRB, 0.27126E+01_JPRB ,&
&  0.28226E+01_JPRB, 0.29292E+01_JPRB, 0.29737E+01_JPRB, 0.29816E+01_JPRB, 0.30100E+01_JPRB, 0.30772E+01_JPRB, 0.31731E+01_JPRB ,&
&  0.33319E+01_JPRB, 0.35170E+01_JPRB, 0.36116E+01_JPRB, 0.34450E+01_JPRB, 0.33736E+01_JPRB, 0.34336E+01_JPRB, 0.33948E+01_JPRB ,&
&  0.32445E+01_JPRB, 0.32580E+01_JPRB, 0.34644E+01_JPRB, 0.38020E+01_JPRB, 0.39688E+01_JPRB, 0.47746E+01_JPRB, 0.51750E+01_JPRB ,&
&  0.57287E+01_JPRB, 0.60431E+01_JPRB, 0.67562E+01_JPRB, 0.51040E+01_JPRB, 0.51927E+01_JPRB, 0.53014E+01_JPRB, 0.51441E+01_JPRB ,&
&  0.50827E+01_JPRB, 0.54516E+01_JPRB, 0.58538E+01_JPRB, 0.61359E+01_JPRB, 0.62605E+01_JPRB, 0.54166E+01_JPRB, 0.52044E+01_JPRB ,&
&  0.50171E+01_JPRB, 0.55688E+01_JPRB, 0.46092E+01_JPRB, 0.41868E+01_JPRB, 0.37687E+01_JPRB, 0.40752E+01_JPRB, 0.45216E+01_JPRB ,&
&  0.50031E+01_JPRB, 0.45435E+01_JPRB, 0.38740E+01_JPRB, 0.29191E+01_JPRB, 0.18391E+01_JPRB, 0.13384E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.11413E+01_JPRB, 0.12097E+01_JPRB, 0.12380E+01_JPRB, 0.12725E+01_JPRB, 0.13163E+01_JPRB, 0.13236E+01_JPRB ,&
&  0.13261E+01_JPRB, 0.13085E+01_JPRB, 0.12865E+01_JPRB, 0.13303E+01_JPRB, 0.13459E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.11189E+01_JPRB, 0.10759E+01_JPRB, 0.10709E+01_JPRB, 0.11911E+01_JPRB, 0.13392E+01_JPRB, 0.14291E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.14535E+01_JPRB, 0.14910E+01_JPRB, 0.16832E+01_JPRB, 0.19323E+01_JPRB, 0.22210E+01_JPRB, 0.25971E+01_JPRB, 0.30009E+01_JPRB ,&
&  0.34228E+01_JPRB, 0.37446E+01_JPRB, 0.39739E+01_JPRB, 0.42285E+01_JPRB, 0.45722E+01_JPRB, 0.49694E+01_JPRB, 0.51382E+01_JPRB ,&
&  0.40944E+01_JPRB, 0.24599E+01_JPRB, 0.26742E+01_JPRB, 0.28303E+01_JPRB, 0.30691E+01_JPRB, 0.32719E+01_JPRB, 0.32532E+01_JPRB ,&
&  0.27746E+01_JPRB, 0.25935E+01_JPRB, 0.26397E+01_JPRB, 0.26881E+01_JPRB, 0.24062E+01_JPRB, 0.25210E+01_JPRB, 0.26764E+01_JPRB ,&
&  0.28566E+01_JPRB, 0.29971E+01_JPRB, 0.30860E+01_JPRB, 0.31650E+01_JPRB, 0.32303E+01_JPRB, 0.33267E+01_JPRB, 0.34973E+01_JPRB ,&
&  0.37118E+01_JPRB, 0.39098E+01_JPRB, 0.40544E+01_JPRB, 0.40447E+01_JPRB, 0.40304E+01_JPRB, 0.41295E+01_JPRB, 0.41014E+01_JPRB ,&
&  0.41088E+01_JPRB, 0.41158E+01_JPRB, 0.45219E+01_JPRB, 0.50689E+01_JPRB, 0.53857E+01_JPRB, 0.49332E+01_JPRB, 0.53527E+01_JPRB ,&
&  0.55256E+01_JPRB, 0.55135E+01_JPRB, 0.53019E+01_JPRB, 0.56398E+01_JPRB, 0.42130E+01_JPRB, 0.45631E+01_JPRB, 0.47222E+01_JPRB ,&
&  0.53601E+01_JPRB, 0.55276E+01_JPRB, 0.57112E+01_JPRB, 0.56062E+01_JPRB, 0.58019E+01_JPRB, 0.49115E+01_JPRB, 0.44858E+01_JPRB ,&
&  0.44725E+01_JPRB, 0.46210E+01_JPRB, 0.43108E+01_JPRB, 0.42303E+01_JPRB, 0.39179E+01_JPRB, 0.41107E+01_JPRB, 0.38413E+01_JPRB ,&
&  0.38878E+01_JPRB, 0.29654E+01_JPRB, 0.24168E+01_JPRB, 0.18110E+01_JPRB, 0.10783E+01_JPRB, 0.92483E+00_JPRB, 0.95709E+00_JPRB ,&
&  0.10001E+01_JPRB, 0.10519E+01_JPRB, 0.10622E+01_JPRB, 0.11100E+01_JPRB, 0.11473E+01_JPRB, 0.11361E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.11457E+01_JPRB, 0.11452E+01_JPRB, 0.11459E+01_JPRB, 0.11376E+01_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.87845E+00_JPRB, 0.87772E+00_JPRB, 0.89551E+00_JPRB, 0.95202E+00_JPRB, 0.10649E+01_JPRB, 0.11474E+01_JPRB, 0.12022E+01_JPRB ,&
&  0.12260E+01_JPRB, 0.12829E+01_JPRB, 0.13972E+01_JPRB, 0.14871E+01_JPRB, 0.16530E+01_JPRB, 0.18856E+01_JPRB, 0.21193E+01_JPRB ,&
&  0.24217E+01_JPRB, 0.26895E+01_JPRB, 0.28871E+01_JPRB, 0.31472E+01_JPRB, 0.33870E+01_JPRB, 0.37324E+01_JPRB, 0.39597E+01_JPRB ,&
&  0.39084E+01_JPRB, 0.31746E+01_JPRB, 0.28574E+01_JPRB, 0.22769E+01_JPRB, 0.21484E+01_JPRB, 0.25564E+01_JPRB, 0.23489E+01_JPRB ,&
&  0.22875E+01_JPRB, 0.22833E+01_JPRB, 0.22117E+01_JPRB, 0.22322E+01_JPRB, 0.22990E+01_JPRB, 0.24015E+01_JPRB, 0.25830E+01_JPRB ,&
&  0.27925E+01_JPRB, 0.29873E+01_JPRB, 0.31610E+01_JPRB, 0.33113E+01_JPRB, 0.34117E+01_JPRB, 0.35466E+01_JPRB, 0.37552E+01_JPRB ,&
&  0.39922E+01_JPRB, 0.41766E+01_JPRB, 0.43802E+01_JPRB, 0.46781E+01_JPRB, 0.46764E+01_JPRB, 0.48361E+01_JPRB, 0.49202E+01_JPRB ,&
&  0.49890E+01_JPRB, 0.49590E+01_JPRB, 0.50105E+01_JPRB, 0.51921E+01_JPRB, 0.51696E+01_JPRB, 0.46444E+01_JPRB, 0.46670E+01_JPRB ,&
&  0.49048E+01_JPRB, 0.47355E+01_JPRB, 0.42338E+01_JPRB, 0.48456E+01_JPRB, 0.35154E+01_JPRB, 0.41849E+01_JPRB, 0.48457E+01_JPRB ,&
&  0.49039E+01_JPRB, 0.48931E+01_JPRB, 0.47347E+01_JPRB, 0.47166E+01_JPRB, 0.47532E+01_JPRB, 0.44236E+01_JPRB, 0.42430E+01_JPRB ,&
&  0.40871E+01_JPRB, 0.34242E+01_JPRB, 0.32522E+01_JPRB, 0.35377E+01_JPRB, 0.38142E+01_JPRB, 0.34627E+01_JPRB, 0.27817E+01_JPRB ,&
&  0.22637E+01_JPRB, 0.22522E+01_JPRB, 0.20841E+01_JPRB, 0.22085E+01_JPRB, 0.14337E+01_JPRB, 0.88697E+00_JPRB, 0.88389E+00_JPRB ,&
&  0.90599E+00_JPRB, 0.93130E+00_JPRB, 0.92423E+00_JPRB, 0.97631E+00_JPRB, 0.99255E+00_JPRB, 0.97223E+00_JPRB, 0.97557E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.99657E+00_JPRB, 0.98670E+00_JPRB, 0.95525E+00_JPRB, 0.92862E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.82803E+00_JPRB, 0.81298E+00_JPRB, 0.81808E+00_JPRB, 0.81896E+00_JPRB, 0.83694E+00_JPRB, 0.84592E+00_JPRB, 0.87503E+00_JPRB ,&
&  0.92085E+00_JPRB, 0.97082E+00_JPRB, 0.97923E+00_JPRB, 0.97193E+00_JPRB, 0.98843E+00_JPRB, 0.10243E+01_JPRB, 0.11215E+01_JPRB ,&
&  0.13198E+01_JPRB, 0.15427E+01_JPRB, 0.16770E+01_JPRB, 0.18107E+01_JPRB, 0.20827E+01_JPRB, 0.25964E+01_JPRB, 0.30886E+01_JPRB ,&
&  0.34552E+01_JPRB, 0.36680E+01_JPRB, 0.35380E+01_JPRB, 0.35764E+01_JPRB, 0.28036E+01_JPRB, 0.23859E+01_JPRB, 0.26488E+01_JPRB ,&
&  0.28853E+01_JPRB, 0.27191E+01_JPRB, 0.21493E+01_JPRB, 0.20581E+01_JPRB, 0.20408E+01_JPRB, 0.21297E+01_JPRB, 0.23708E+01_JPRB ,&
&  0.26124E+01_JPRB, 0.28810E+01_JPRB, 0.30981E+01_JPRB, 0.32771E+01_JPRB, 0.34228E+01_JPRB, 0.35934E+01_JPRB, 0.37593E+01_JPRB ,&
&  0.39402E+01_JPRB, 0.40695E+01_JPRB, 0.42077E+01_JPRB, 0.44925E+01_JPRB, 0.47840E+01_JPRB, 0.50550E+01_JPRB, 0.52132E+01_JPRB ,&
&  0.52661E+01_JPRB, 0.50143E+01_JPRB, 0.50550E+01_JPRB, 0.51099E+01_JPRB, 0.48515E+01_JPRB, 0.44817E+01_JPRB, 0.43987E+01_JPRB ,&
&  0.43363E+01_JPRB, 0.41959E+01_JPRB, 0.33256E+01_JPRB, 0.40906E+01_JPRB, 0.47510E+01_JPRB, 0.44642E+01_JPRB, 0.43544E+01_JPRB ,&
&  0.41451E+01_JPRB, 0.39999E+01_JPRB, 0.37982E+01_JPRB, 0.38961E+01_JPRB, 0.42325E+01_JPRB, 0.54742E+01_JPRB, 0.43474E+01_JPRB ,&
&  0.36890E+01_JPRB, 0.24351E+01_JPRB, 0.21739E+01_JPRB, 0.24842E+01_JPRB, 0.27770E+01_JPRB, 0.25578E+01_JPRB, 0.18481E+01_JPRB ,&
&  0.16079E+01_JPRB, 0.18414E+01_JPRB, 0.19775E+01_JPRB, 0.25230E+01_JPRB, 0.26262E+01_JPRB, 0.93505E+00_JPRB, 0.80788E+00_JPRB ,&
&  0.80578E+00_JPRB, 0.80676E+00_JPRB, 0.79966E+00_JPRB, 0.82415E+00_JPRB, 0.82564E+00_JPRB, 0.83332E+00_JPRB, 0.86086E+00_JPRB ,&
&  0.87913E+00_JPRB, 0.87783E+00_JPRB, 0.86788E+00_JPRB, 0.83524E+00_JPRB, 0.83682E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.89182E+00_JPRB, 0.89094E+00_JPRB, 0.89898E+00_JPRB, 0.89116E+00_JPRB, 0.88278E+00_JPRB, 0.88385E+00_JPRB, 0.89084E+00_JPRB ,&
&  0.89536E+00_JPRB, 0.88384E+00_JPRB, 0.86878E+00_JPRB, 0.86044E+00_JPRB, 0.85508E+00_JPRB, 0.81585E+00_JPRB, 0.76140E+00_JPRB ,&
&  0.78184E+00_JPRB, 0.81239E+00_JPRB, 0.83238E+00_JPRB, 0.87544E+00_JPRB, 0.99059E+00_JPRB, 0.11718E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.19247E+01_JPRB, 0.24619E+01_JPRB, 0.28288E+01_JPRB, 0.28871E+01_JPRB, 0.32069E+01_JPRB, 0.26151E+01_JPRB, 0.27756E+01_JPRB ,&
&  0.32420E+01_JPRB, 0.25660E+01_JPRB, 0.21406E+01_JPRB, 0.18874E+01_JPRB, 0.18446E+01_JPRB, 0.19137E+01_JPRB, 0.20669E+01_JPRB ,&
&  0.23100E+01_JPRB, 0.25701E+01_JPRB, 0.27916E+01_JPRB, 0.30129E+01_JPRB, 0.32235E+01_JPRB, 0.34406E+01_JPRB, 0.36122E+01_JPRB ,&
&  0.37093E+01_JPRB, 0.38343E+01_JPRB, 0.40242E+01_JPRB, 0.41301E+01_JPRB, 0.45376E+01_JPRB, 0.47942E+01_JPRB, 0.49171E+01_JPRB ,&
&  0.48304E+01_JPRB, 0.46366E+01_JPRB, 0.46719E+01_JPRB, 0.44898E+01_JPRB, 0.42890E+01_JPRB, 0.40959E+01_JPRB, 0.38955E+01_JPRB ,&
&  0.37094E+01_JPRB, 0.34192E+01_JPRB, 0.28232E+01_JPRB, 0.32511E+01_JPRB, 0.35518E+01_JPRB, 0.33328E+01_JPRB, 0.39049E+01_JPRB ,&
&  0.36589E+01_JPRB, 0.35187E+01_JPRB, 0.34061E+01_JPRB, 0.32922E+01_JPRB, 0.32074E+01_JPRB, 0.38094E+01_JPRB, 0.38507E+01_JPRB ,&
&  0.26198E+01_JPRB, 0.17239E+01_JPRB, 0.14499E+01_JPRB, 0.15310E+01_JPRB, 0.17622E+01_JPRB, 0.18850E+01_JPRB, 0.17458E+01_JPRB ,&
&  0.16731E+01_JPRB, 0.15883E+01_JPRB, 0.17954E+01_JPRB, 0.23179E+01_JPRB, 0.15640E+01_JPRB, 0.91299E+00_JPRB, 0.73900E+00_JPRB ,&
&  0.70085E+00_JPRB, 0.69693E+00_JPRB, 0.70553E+00_JPRB, 0.73815E+00_JPRB, 0.76702E+00_JPRB, 0.79676E+00_JPRB, 0.82264E+00_JPRB ,&
&  0.83525E+00_JPRB, 0.85458E+00_JPRB, 0.86916E+00_JPRB, 0.88661E+00_JPRB, 0.89620E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.99850E+00_JPRB, 0.10187E+01_JPRB, 0.10148E+01_JPRB, 0.10261E+01_JPRB, 0.10389E+01_JPRB, 0.10620E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10616E+01_JPRB, 0.10387E+01_JPRB, 0.10448E+01_JPRB, 0.10473E+01_JPRB, 0.10491E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.10434E+01_JPRB, 0.10662E+01_JPRB, 0.10722E+01_JPRB, 0.10681E+01_JPRB, 0.10465E+01_JPRB, 0.10112E+01_JPRB, 0.98677E+00_JPRB ,&
&  0.10500E+01_JPRB, 0.11920E+01_JPRB, 0.13700E+01_JPRB, 0.14940E+01_JPRB, 0.17998E+01_JPRB, 0.18867E+01_JPRB, 0.21363E+01_JPRB ,&
&  0.27289E+01_JPRB, 0.22493E+01_JPRB, 0.21115E+01_JPRB, 0.17355E+01_JPRB, 0.16793E+01_JPRB, 0.16215E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.16379E+01_JPRB, 0.17388E+01_JPRB, 0.18615E+01_JPRB, 0.20020E+01_JPRB, 0.21195E+01_JPRB, 0.21630E+01_JPRB, 0.21837E+01_JPRB ,&
&  0.21961E+01_JPRB, 0.23822E+01_JPRB, 0.29031E+01_JPRB, 0.33158E+01_JPRB, 0.37979E+01_JPRB, 0.40312E+01_JPRB, 0.40661E+01_JPRB ,&
&  0.40176E+01_JPRB, 0.40210E+01_JPRB, 0.35261E+01_JPRB, 0.31545E+01_JPRB, 0.30876E+01_JPRB, 0.28274E+01_JPRB, 0.27716E+01_JPRB ,&
&  0.29141E+01_JPRB, 0.28559E+01_JPRB, 0.23429E+01_JPRB, 0.25963E+01_JPRB, 0.28905E+01_JPRB, 0.28589E+01_JPRB, 0.33266E+01_JPRB ,&
&  0.28943E+01_JPRB, 0.25923E+01_JPRB, 0.24637E+01_JPRB, 0.23940E+01_JPRB, 0.22969E+01_JPRB, 0.22065E+01_JPRB, 0.22739E+01_JPRB ,&
&  0.17636E+01_JPRB, 0.14395E+01_JPRB, 0.13293E+01_JPRB, 0.13812E+01_JPRB, 0.17431E+01_JPRB, 0.23952E+01_JPRB, 0.18750E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.17175E+01_JPRB, 0.23416E+01_JPRB, 0.33477E+01_JPRB, 0.15275E+01_JPRB, 0.95849E+00_JPRB, 0.69658E+00_JPRB ,&
&  0.68366E+00_JPRB, 0.71737E+00_JPRB, 0.74975E+00_JPRB, 0.79179E+00_JPRB, 0.81805E+00_JPRB, 0.83151E+00_JPRB, 0.85179E+00_JPRB ,&
&  0.89328E+00_JPRB, 0.93071E+00_JPRB, 0.98247E+00_JPRB, 0.99055E+00_JPRB, 0.97807E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.10653E+01_JPRB, 0.10946E+01_JPRB, 0.11031E+01_JPRB, 0.11291E+01_JPRB, 0.11630E+01_JPRB, 0.11806E+01_JPRB, 0.11899E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.11843E+01_JPRB, 0.11833E+01_JPRB, 0.12030E+01_JPRB, 0.12093E+01_JPRB, 0.12088E+01_JPRB, 0.12116E+01_JPRB ,&
&  0.12371E+01_JPRB, 0.12563E+01_JPRB, 0.13007E+01_JPRB, 0.13018E+01_JPRB, 0.12938E+01_JPRB, 0.12984E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.13844E+01_JPRB, 0.14250E+01_JPRB, 0.14626E+01_JPRB, 0.16062E+01_JPRB, 0.17922E+01_JPRB, 0.21161E+01_JPRB ,&
&  0.22567E+01_JPRB, 0.23061E+01_JPRB, 0.19335E+01_JPRB, 0.16158E+01_JPRB, 0.15765E+01_JPRB, 0.15443E+01_JPRB, 0.14442E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.12992E+01_JPRB, 0.12652E+01_JPRB, 0.12542E+01_JPRB, 0.12608E+01_JPRB, 0.12336E+01_JPRB, 0.12151E+01_JPRB ,&
&  0.11653E+01_JPRB, 0.11862E+01_JPRB, 0.14782E+01_JPRB, 0.18701E+01_JPRB, 0.22427E+01_JPRB, 0.25560E+01_JPRB, 0.23082E+01_JPRB ,&
&  0.22084E+01_JPRB, 0.23231E+01_JPRB, 0.21516E+01_JPRB, 0.20155E+01_JPRB, 0.18438E+01_JPRB, 0.16892E+01_JPRB, 0.16301E+01_JPRB ,&
&  0.18098E+01_JPRB, 0.20297E+01_JPRB, 0.19867E+01_JPRB, 0.23071E+01_JPRB, 0.24943E+01_JPRB, 0.25239E+01_JPRB, 0.22811E+01_JPRB ,&
&  0.19557E+01_JPRB, 0.18032E+01_JPRB, 0.17684E+01_JPRB, 0.17670E+01_JPRB, 0.17916E+01_JPRB, 0.16942E+01_JPRB, 0.15337E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.12860E+01_JPRB, 0.12638E+01_JPRB, 0.12400E+01_JPRB, 0.16896E+01_JPRB, 0.21254E+01_JPRB, 0.27277E+01_JPRB ,&
&  0.19117E+01_JPRB, 0.17675E+01_JPRB, 0.15376E+01_JPRB, 0.24027E+01_JPRB, 0.12749E+01_JPRB, 0.73285E+00_JPRB, 0.68984E+00_JPRB ,&
&  0.76529E+00_JPRB, 0.81312E+00_JPRB, 0.83959E+00_JPRB, 0.87733E+00_JPRB, 0.89713E+00_JPRB, 0.90282E+00_JPRB, 0.92156E+00_JPRB ,&
&  0.96979E+00_JPRB, 0.98810E+00_JPRB, 0.10063E+01_JPRB, 0.10126E+01_JPRB, 0.10370E+01_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.10989E+01_JPRB, 0.11402E+01_JPRB, 0.11678E+01_JPRB, 0.11965E+01_JPRB, 0.12325E+01_JPRB, 0.12577E+01_JPRB, 0.12773E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.12887E+01_JPRB, 0.13035E+01_JPRB, 0.13231E+01_JPRB, 0.13313E+01_JPRB, 0.13255E+01_JPRB, 0.13328E+01_JPRB ,&
&  0.13584E+01_JPRB, 0.13971E+01_JPRB, 0.14433E+01_JPRB, 0.14390E+01_JPRB, 0.14381E+01_JPRB, 0.14679E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.15111E+01_JPRB, 0.14971E+01_JPRB, 0.15084E+01_JPRB, 0.15312E+01_JPRB, 0.15739E+01_JPRB, 0.15998E+01_JPRB, 0.17377E+01_JPRB ,&
&  0.18377E+01_JPRB, 0.24664E+01_JPRB, 0.21448E+01_JPRB, 0.18097E+01_JPRB, 0.17628E+01_JPRB, 0.15950E+01_JPRB, 0.15445E+01_JPRB ,&
&  0.15032E+01_JPRB, 0.14224E+01_JPRB, 0.13696E+01_JPRB, 0.13230E+01_JPRB, 0.13157E+01_JPRB, 0.13255E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.13034E+01_JPRB, 0.13287E+01_JPRB, 0.14222E+01_JPRB, 0.15449E+01_JPRB, 0.16547E+01_JPRB, 0.16762E+01_JPRB, 0.15490E+01_JPRB ,&
&  0.14645E+01_JPRB, 0.13353E+01_JPRB, 0.11112E+01_JPRB, 0.11146E+01_JPRB, 0.10848E+01_JPRB, 0.91985E+00_JPRB, 0.85930E+00_JPRB ,&
&  0.11449E+01_JPRB, 0.13692E+01_JPRB, 0.17887E+01_JPRB, 0.19762E+01_JPRB, 0.20014E+01_JPRB, 0.20823E+01_JPRB, 0.17656E+01_JPRB ,&
&  0.15443E+01_JPRB, 0.14198E+01_JPRB, 0.14197E+01_JPRB, 0.13823E+01_JPRB, 0.14103E+01_JPRB, 0.13933E+01_JPRB, 0.12820E+01_JPRB ,&
&  0.11947E+01_JPRB, 0.11872E+01_JPRB, 0.12266E+01_JPRB, 0.13284E+01_JPRB, 0.15379E+01_JPRB, 0.29157E+01_JPRB, 0.44117E+01_JPRB ,&
&  0.22373E+01_JPRB, 0.15914E+01_JPRB, 0.93119E+00_JPRB, 0.11603E+01_JPRB, 0.87476E+00_JPRB, 0.71059E+00_JPRB, 0.74619E+00_JPRB ,&
&  0.87166E+00_JPRB, 0.93166E+00_JPRB, 0.94729E+00_JPRB, 0.94685E+00_JPRB, 0.93139E+00_JPRB, 0.91939E+00_JPRB, 0.91719E+00_JPRB ,&
&  0.94404E+00_JPRB, 0.97129E+00_JPRB, 0.10027E+01_JPRB, 0.10260E+01_JPRB, 0.10574E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.11084E+01_JPRB, 0.11492E+01_JPRB, 0.11840E+01_JPRB, 0.12244E+01_JPRB, 0.12681E+01_JPRB, 0.13086E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.14042E+01_JPRB, 0.14303E+01_JPRB, 0.14562E+01_JPRB, 0.14840E+01_JPRB, 0.15053E+01_JPRB, 0.15331E+01_JPRB ,&
&  0.15754E+01_JPRB, 0.16140E+01_JPRB, 0.16623E+01_JPRB, 0.16709E+01_JPRB, 0.16795E+01_JPRB, 0.17284E+01_JPRB, 0.17716E+01_JPRB ,&
&  0.17816E+01_JPRB, 0.18031E+01_JPRB, 0.18401E+01_JPRB, 0.18930E+01_JPRB, 0.19676E+01_JPRB, 0.21463E+01_JPRB, 0.21253E+01_JPRB ,&
&  0.24030E+01_JPRB, 0.25843E+01_JPRB, 0.23575E+01_JPRB, 0.20562E+01_JPRB, 0.19264E+01_JPRB, 0.18572E+01_JPRB, 0.18359E+01_JPRB ,&
&  0.18291E+01_JPRB, 0.17553E+01_JPRB, 0.17094E+01_JPRB, 0.16563E+01_JPRB, 0.16310E+01_JPRB, 0.16465E+01_JPRB, 0.16679E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.17547E+01_JPRB, 0.18264E+01_JPRB, 0.19230E+01_JPRB, 0.20244E+01_JPRB, 0.20005E+01_JPRB, 0.19274E+01_JPRB ,&
&  0.17897E+01_JPRB, 0.16498E+01_JPRB, 0.13010E+01_JPRB, 0.12067E+01_JPRB, 0.11281E+01_JPRB, 0.98735E+00_JPRB, 0.86950E+00_JPRB ,&
&  0.10745E+01_JPRB, 0.11898E+01_JPRB, 0.14191E+01_JPRB, 0.17419E+01_JPRB, 0.17108E+01_JPRB, 0.16334E+01_JPRB, 0.14482E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.11907E+01_JPRB, 0.11649E+01_JPRB, 0.11266E+01_JPRB, 0.11325E+01_JPRB, 0.11531E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10948E+01_JPRB, 0.11501E+01_JPRB, 0.12631E+01_JPRB, 0.15316E+01_JPRB, 0.17745E+01_JPRB, 0.27635E+01_JPRB ,&
&  0.19854E+01_JPRB, 0.11586E+01_JPRB, 0.11077E+01_JPRB, 0.96496E+00_JPRB, 0.84876E+00_JPRB, 0.75472E+00_JPRB, 0.81829E+00_JPRB ,&
&  0.97530E+00_JPRB, 0.11051E+01_JPRB, 0.11380E+01_JPRB, 0.10887E+01_JPRB, 0.10074E+01_JPRB, 0.96571E+00_JPRB, 0.94644E+00_JPRB ,&
&  0.96976E+00_JPRB, 0.99169E+00_JPRB, 0.10177E+01_JPRB, 0.10361E+01_JPRB, 0.10653E+01_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.10889E+01_JPRB, 0.11340E+01_JPRB, 0.11746E+01_JPRB, 0.12161E+01_JPRB, 0.12589E+01_JPRB, 0.13025E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.13854E+01_JPRB, 0.14316E+01_JPRB, 0.14837E+01_JPRB, 0.15350E+01_JPRB, 0.15857E+01_JPRB, 0.16383E+01_JPRB, 0.16856E+01_JPRB ,&
&  0.17422E+01_JPRB, 0.17875E+01_JPRB, 0.18445E+01_JPRB, 0.18702E+01_JPRB, 0.19023E+01_JPRB, 0.19531E+01_JPRB, 0.20046E+01_JPRB ,&
&  0.20255E+01_JPRB, 0.20557E+01_JPRB, 0.20928E+01_JPRB, 0.21493E+01_JPRB, 0.21818E+01_JPRB, 0.23501E+01_JPRB, 0.22244E+01_JPRB ,&
&  0.26861E+01_JPRB, 0.28871E+01_JPRB, 0.26616E+01_JPRB, 0.23624E+01_JPRB, 0.22118E+01_JPRB, 0.21896E+01_JPRB, 0.21843E+01_JPRB ,&
&  0.21995E+01_JPRB, 0.22583E+01_JPRB, 0.20915E+01_JPRB, 0.20030E+01_JPRB, 0.19669E+01_JPRB, 0.19888E+01_JPRB, 0.20603E+01_JPRB ,&
&  0.21431E+01_JPRB, 0.22336E+01_JPRB, 0.23281E+01_JPRB, 0.24202E+01_JPRB, 0.25012E+01_JPRB, 0.25428E+01_JPRB, 0.25294E+01_JPRB ,&
&  0.24485E+01_JPRB, 0.23422E+01_JPRB, 0.20997E+01_JPRB, 0.20476E+01_JPRB, 0.19265E+01_JPRB, 0.16019E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.12752E+01_JPRB, 0.13189E+01_JPRB, 0.15270E+01_JPRB, 0.14938E+01_JPRB, 0.13943E+01_JPRB, 0.12501E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.10226E+01_JPRB, 0.10171E+01_JPRB, 0.97863E+00_JPRB, 0.94950E+00_JPRB, 0.93807E+00_JPRB, 0.92648E+00_JPRB ,&
&  0.95482E+00_JPRB, 0.10037E+01_JPRB, 0.10755E+01_JPRB, 0.11709E+01_JPRB, 0.12849E+01_JPRB, 0.13503E+01_JPRB, 0.21991E+01_JPRB ,&
&  0.16764E+01_JPRB, 0.13160E+01_JPRB, 0.15194E+01_JPRB, 0.77385E+00_JPRB, 0.81217E+00_JPRB, 0.87242E+00_JPRB, 0.10072E+01_JPRB ,&
&  0.10930E+01_JPRB, 0.14788E+01_JPRB, 0.19475E+01_JPRB, 0.15673E+01_JPRB, 0.12995E+01_JPRB, 0.10391E+01_JPRB, 0.95866E+00_JPRB ,&
&  0.96853E+00_JPRB, 0.98224E+00_JPRB, 0.10033E+01_JPRB, 0.10197E+01_JPRB, 0.10480E+01_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.10546E+01_JPRB, 0.10923E+01_JPRB, 0.11360E+01_JPRB, 0.11768E+01_JPRB, 0.12106E+01_JPRB, 0.12435E+01_JPRB, 0.12815E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.13340E+01_JPRB, 0.13721E+01_JPRB, 0.14228E+01_JPRB, 0.14796E+01_JPRB, 0.15460E+01_JPRB, 0.16149E+01_JPRB ,&
&  0.17041E+01_JPRB, 0.17909E+01_JPRB, 0.19022E+01_JPRB, 0.20086E+01_JPRB, 0.20999E+01_JPRB, 0.21971E+01_JPRB, 0.23021E+01_JPRB ,&
&  0.24252E+01_JPRB, 0.25124E+01_JPRB, 0.25798E+01_JPRB, 0.26983E+01_JPRB, 0.28131E+01_JPRB, 0.29897E+01_JPRB, 0.25045E+01_JPRB ,&
&  0.30048E+01_JPRB, 0.33610E+01_JPRB, 0.30464E+01_JPRB, 0.26460E+01_JPRB, 0.24696E+01_JPRB, 0.24448E+01_JPRB, 0.24289E+01_JPRB ,&
&  0.24250E+01_JPRB, 0.23807E+01_JPRB, 0.21845E+01_JPRB, 0.22217E+01_JPRB, 0.22571E+01_JPRB, 0.22546E+01_JPRB, 0.23642E+01_JPRB ,&
&  0.24835E+01_JPRB, 0.26023E+01_JPRB, 0.27120E+01_JPRB, 0.28143E+01_JPRB, 0.29011E+01_JPRB, 0.29973E+01_JPRB, 0.30204E+01_JPRB ,&
&  0.30171E+01_JPRB, 0.30113E+01_JPRB, 0.29715E+01_JPRB, 0.26977E+01_JPRB, 0.25647E+01_JPRB, 0.23978E+01_JPRB, 0.19516E+01_JPRB ,&
&  0.15947E+01_JPRB, 0.13392E+01_JPRB, 0.12578E+01_JPRB, 0.13093E+01_JPRB, 0.12415E+01_JPRB, 0.11277E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.94175E+00_JPRB, 0.88410E+00_JPRB, 0.89924E+00_JPRB, 0.87705E+00_JPRB, 0.79372E+00_JPRB, 0.82057E+00_JPRB, 0.85219E+00_JPRB ,&
&  0.86263E+00_JPRB, 0.90502E+00_JPRB, 0.96966E+00_JPRB, 0.10297E+01_JPRB, 0.11752E+01_JPRB, 0.14788E+01_JPRB, 0.16848E+01_JPRB ,&
&  0.17320E+01_JPRB, 0.12162E+01_JPRB, 0.11160E+01_JPRB, 0.10797E+01_JPRB, 0.91466E+00_JPRB, 0.11413E+01_JPRB, 0.15094E+01_JPRB ,&
&  0.21419E+01_JPRB, 0.22352E+01_JPRB, 0.28084E+01_JPRB, 0.43001E+01_JPRB, 0.32830E+01_JPRB, 0.22401E+01_JPRB, 0.96131E+00_JPRB ,&
&  0.97500E+00_JPRB, 0.97932E+00_JPRB, 0.10084E+01_JPRB, 0.10196E+01_JPRB, 0.10402E+01_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.88723E+00_JPRB, 0.89644E+00_JPRB, 0.93723E+00_JPRB, 0.98363E+00_JPRB, 0.10286E+01_JPRB, 0.10727E+01_JPRB, 0.11114E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.12129E+01_JPRB, 0.12417E+01_JPRB, 0.12605E+01_JPRB, 0.12914E+01_JPRB, 0.13291E+01_JPRB, 0.13860E+01_JPRB ,&
&  0.14664E+01_JPRB, 0.15561E+01_JPRB, 0.16810E+01_JPRB, 0.17862E+01_JPRB, 0.19055E+01_JPRB, 0.20611E+01_JPRB, 0.22170E+01_JPRB ,&
&  0.23869E+01_JPRB, 0.25961E+01_JPRB, 0.27644E+01_JPRB, 0.29225E+01_JPRB, 0.31001E+01_JPRB, 0.31068E+01_JPRB, 0.22923E+01_JPRB ,&
&  0.32493E+01_JPRB, 0.35446E+01_JPRB, 0.33385E+01_JPRB, 0.29090E+01_JPRB, 0.26795E+01_JPRB, 0.26033E+01_JPRB, 0.26066E+01_JPRB ,&
&  0.25695E+01_JPRB, 0.24214E+01_JPRB, 0.22319E+01_JPRB, 0.21340E+01_JPRB, 0.23682E+01_JPRB, 0.23205E+01_JPRB, 0.24379E+01_JPRB ,&
&  0.25596E+01_JPRB, 0.26772E+01_JPRB, 0.27719E+01_JPRB, 0.28517E+01_JPRB, 0.29178E+01_JPRB, 0.30015E+01_JPRB, 0.30924E+01_JPRB ,&
&  0.31765E+01_JPRB, 0.32247E+01_JPRB, 0.33162E+01_JPRB, 0.30113E+01_JPRB, 0.28675E+01_JPRB, 0.30470E+01_JPRB, 0.27293E+01_JPRB ,&
&  0.18567E+01_JPRB, 0.13654E+01_JPRB, 0.13564E+01_JPRB, 0.12833E+01_JPRB, 0.11532E+01_JPRB, 0.10274E+01_JPRB, 0.94132E+00_JPRB ,&
&  0.87903E+00_JPRB, 0.83196E+00_JPRB, 0.81509E+00_JPRB, 0.79018E+00_JPRB, 0.78709E+00_JPRB, 0.80603E+00_JPRB, 0.80865E+00_JPRB ,&
&  0.85283E+00_JPRB, 0.84241E+00_JPRB, 0.85575E+00_JPRB, 0.90077E+00_JPRB, 0.10453E+01_JPRB, 0.13619E+01_JPRB, 0.19137E+01_JPRB ,&
&  0.20365E+01_JPRB, 0.11850E+01_JPRB, 0.10763E+01_JPRB, 0.10206E+01_JPRB, 0.10375E+01_JPRB, 0.11579E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.17320E+01_JPRB, 0.22461E+01_JPRB, 0.21147E+01_JPRB, 0.16575E+01_JPRB, 0.20176E+01_JPRB, 0.17260E+01_JPRB, 0.85242E+00_JPRB ,&
&  0.86278E+00_JPRB, 0.83885E+00_JPRB, 0.85348E+00_JPRB, 0.87001E+00_JPRB, 0.88137E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.72030E+00_JPRB, 0.71219E+00_JPRB, 0.72640E+00_JPRB, 0.75477E+00_JPRB, 0.78594E+00_JPRB, 0.84039E+00_JPRB, 0.90741E+00_JPRB ,&
&  0.98905E+00_JPRB, 0.10531E+01_JPRB, 0.10849E+01_JPRB, 0.11045E+01_JPRB, 0.11306E+01_JPRB, 0.11634E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.13327E+01_JPRB, 0.14201E+01_JPRB, 0.14935E+01_JPRB, 0.15826E+01_JPRB, 0.17347E+01_JPRB, 0.19180E+01_JPRB ,&
&  0.21371E+01_JPRB, 0.23724E+01_JPRB, 0.26373E+01_JPRB, 0.29126E+01_JPRB, 0.32014E+01_JPRB, 0.33393E+01_JPRB, 0.34305E+01_JPRB ,&
&  0.24565E+01_JPRB, 0.33648E+01_JPRB, 0.34183E+01_JPRB, 0.31226E+01_JPRB, 0.28850E+01_JPRB, 0.28665E+01_JPRB, 0.28213E+01_JPRB ,&
&  0.27346E+01_JPRB, 0.24750E+01_JPRB, 0.23117E+01_JPRB, 0.21633E+01_JPRB, 0.22302E+01_JPRB, 0.22625E+01_JPRB, 0.23613E+01_JPRB ,&
&  0.24584E+01_JPRB, 0.25353E+01_JPRB, 0.25950E+01_JPRB, 0.26592E+01_JPRB, 0.27561E+01_JPRB, 0.28991E+01_JPRB, 0.30902E+01_JPRB ,&
&  0.32618E+01_JPRB, 0.33985E+01_JPRB, 0.34963E+01_JPRB, 0.32500E+01_JPRB, 0.30302E+01_JPRB, 0.33261E+01_JPRB, 0.31767E+01_JPRB ,&
&  0.20847E+01_JPRB, 0.14518E+01_JPRB, 0.14799E+01_JPRB, 0.14649E+01_JPRB, 0.12591E+01_JPRB, 0.11216E+01_JPRB, 0.10169E+01_JPRB ,&
&  0.88554E+00_JPRB, 0.79568E+00_JPRB, 0.80625E+00_JPRB, 0.80542E+00_JPRB, 0.81617E+00_JPRB, 0.81882E+00_JPRB, 0.83705E+00_JPRB ,&
&  0.87837E+00_JPRB, 0.85879E+00_JPRB, 0.85960E+00_JPRB, 0.87455E+00_JPRB, 0.92744E+00_JPRB, 0.99340E+00_JPRB, 0.10521E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.10375E+01_JPRB, 0.10129E+01_JPRB, 0.10299E+01_JPRB, 0.10375E+01_JPRB, 0.10665E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.11547E+01_JPRB, 0.11990E+01_JPRB, 0.12286E+01_JPRB, 0.12109E+01_JPRB, 0.10926E+01_JPRB, 0.10103E+01_JPRB, 0.76925E+00_JPRB ,&
&  0.75578E+00_JPRB, 0.73572E+00_JPRB, 0.73624E+00_JPRB, 0.73501E+00_JPRB, 0.73703E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.62625E+00_JPRB, 0.62267E+00_JPRB, 0.63435E+00_JPRB, 0.65074E+00_JPRB, 0.67940E+00_JPRB, 0.72744E+00_JPRB, 0.79688E+00_JPRB ,&
&  0.87544E+00_JPRB, 0.92604E+00_JPRB, 0.97176E+00_JPRB, 0.97582E+00_JPRB, 0.98744E+00_JPRB, 0.10209E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11791E+01_JPRB, 0.12220E+01_JPRB, 0.12452E+01_JPRB, 0.12788E+01_JPRB, 0.13571E+01_JPRB, 0.14688E+01_JPRB ,&
&  0.16171E+01_JPRB, 0.18338E+01_JPRB, 0.21315E+01_JPRB, 0.25309E+01_JPRB, 0.29698E+01_JPRB, 0.31913E+01_JPRB, 0.32614E+01_JPRB ,&
&  0.15417E+01_JPRB, 0.25521E+01_JPRB, 0.34144E+01_JPRB, 0.33840E+01_JPRB, 0.32568E+01_JPRB, 0.31907E+01_JPRB, 0.31523E+01_JPRB ,&
&  0.27945E+01_JPRB, 0.24013E+01_JPRB, 0.21974E+01_JPRB, 0.23089E+01_JPRB, 0.21103E+01_JPRB, 0.21525E+01_JPRB, 0.22184E+01_JPRB ,&
&  0.23036E+01_JPRB, 0.23760E+01_JPRB, 0.24091E+01_JPRB, 0.24345E+01_JPRB, 0.25055E+01_JPRB, 0.26382E+01_JPRB, 0.28142E+01_JPRB ,&
&  0.30484E+01_JPRB, 0.33443E+01_JPRB, 0.35509E+01_JPRB, 0.33459E+01_JPRB, 0.32669E+01_JPRB, 0.34817E+01_JPRB, 0.42333E+01_JPRB ,&
&  0.27303E+01_JPRB, 0.17076E+01_JPRB, 0.16970E+01_JPRB, 0.17075E+01_JPRB, 0.14356E+01_JPRB, 0.12876E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.10062E+01_JPRB, 0.92404E+00_JPRB, 0.87814E+00_JPRB, 0.85171E+00_JPRB, 0.86566E+00_JPRB, 0.89627E+00_JPRB, 0.95224E+00_JPRB ,&
&  0.98745E+00_JPRB, 0.98562E+00_JPRB, 0.97512E+00_JPRB, 0.98328E+00_JPRB, 0.98784E+00_JPRB, 0.10005E+01_JPRB, 0.10019E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.99611E+00_JPRB, 0.98894E+00_JPRB, 0.10192E+01_JPRB, 0.10619E+01_JPRB, 0.10721E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.10266E+01_JPRB, 0.10343E+01_JPRB, 0.10296E+01_JPRB, 0.10031E+01_JPRB, 0.95691E+00_JPRB, 0.89381E+00_JPRB, 0.78954E+00_JPRB ,&
&  0.73882E+00_JPRB, 0.68137E+00_JPRB, 0.64483E+00_JPRB, 0.62688E+00_JPRB, 0.63630E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.60236E+00_JPRB, 0.59725E+00_JPRB, 0.60112E+00_JPRB, 0.63082E+00_JPRB, 0.63026E+00_JPRB, 0.65408E+00_JPRB, 0.68948E+00_JPRB ,&
&  0.76498E+00_JPRB, 0.83443E+00_JPRB, 0.90226E+00_JPRB, 0.90703E+00_JPRB, 0.90271E+00_JPRB, 0.88629E+00_JPRB, 0.89748E+00_JPRB ,&
&  0.92839E+00_JPRB, 0.97286E+00_JPRB, 0.10210E+01_JPRB, 0.10511E+01_JPRB, 0.10776E+01_JPRB, 0.11068E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.12836E+01_JPRB, 0.14592E+01_JPRB, 0.18040E+01_JPRB, 0.23836E+01_JPRB, 0.30448E+01_JPRB, 0.35776E+01_JPRB ,&
&  0.24554E+01_JPRB, 0.94221E+00_JPRB, 0.27985E+01_JPRB, 0.36351E+01_JPRB, 0.35751E+01_JPRB, 0.34704E+01_JPRB, 0.33051E+01_JPRB ,&
&  0.28585E+01_JPRB, 0.25259E+01_JPRB, 0.21550E+01_JPRB, 0.22543E+01_JPRB, 0.19924E+01_JPRB, 0.19511E+01_JPRB, 0.19985E+01_JPRB ,&
&  0.20847E+01_JPRB, 0.21704E+01_JPRB, 0.22563E+01_JPRB, 0.22734E+01_JPRB, 0.22635E+01_JPRB, 0.23018E+01_JPRB, 0.24138E+01_JPRB ,&
&  0.26341E+01_JPRB, 0.30051E+01_JPRB, 0.34820E+01_JPRB, 0.36166E+01_JPRB, 0.41436E+01_JPRB, 0.39353E+01_JPRB, 0.42308E+01_JPRB ,&
&  0.28768E+01_JPRB, 0.20497E+01_JPRB, 0.19368E+01_JPRB, 0.19558E+01_JPRB, 0.16530E+01_JPRB, 0.13894E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.11511E+01_JPRB, 0.10452E+01_JPRB, 0.10030E+01_JPRB, 0.96345E+00_JPRB, 0.93426E+00_JPRB, 0.96906E+00_JPRB, 0.10260E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.10958E+01_JPRB, 0.10913E+01_JPRB, 0.10910E+01_JPRB, 0.10863E+01_JPRB, 0.10660E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10679E+01_JPRB, 0.10834E+01_JPRB, 0.10999E+01_JPRB, 0.11167E+01_JPRB, 0.10987E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.10816E+01_JPRB, 0.10836E+01_JPRB, 0.10408E+01_JPRB, 0.10364E+01_JPRB, 0.10025E+01_JPRB, 0.95193E+00_JPRB, 0.90964E+00_JPRB ,&
&  0.83983E+00_JPRB, 0.73781E+00_JPRB, 0.66468E+00_JPRB, 0.63596E+00_JPRB, 0.61093E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.65356E+00_JPRB, 0.63015E+00_JPRB, 0.63339E+00_JPRB, 0.64915E+00_JPRB, 0.66301E+00_JPRB, 0.67471E+00_JPRB, 0.66047E+00_JPRB ,&
&  0.68700E+00_JPRB, 0.74362E+00_JPRB, 0.78442E+00_JPRB, 0.82490E+00_JPRB, 0.81730E+00_JPRB, 0.79739E+00_JPRB, 0.78466E+00_JPRB ,&
&  0.78410E+00_JPRB, 0.79321E+00_JPRB, 0.82285E+00_JPRB, 0.83983E+00_JPRB, 0.85832E+00_JPRB, 0.88468E+00_JPRB, 0.90180E+00_JPRB ,&
&  0.91661E+00_JPRB, 0.94265E+00_JPRB, 0.10021E+01_JPRB, 0.11381E+01_JPRB, 0.14156E+01_JPRB, 0.19801E+01_JPRB, 0.28626E+01_JPRB ,&
&  0.45387E+01_JPRB, 0.32807E+01_JPRB, 0.13470E+01_JPRB, 0.30654E+01_JPRB, 0.39313E+01_JPRB, 0.40227E+01_JPRB, 0.34134E+01_JPRB ,&
&  0.31927E+01_JPRB, 0.26807E+01_JPRB, 0.22702E+01_JPRB, 0.23147E+01_JPRB, 0.19452E+01_JPRB, 0.17764E+01_JPRB, 0.17227E+01_JPRB ,&
&  0.17586E+01_JPRB, 0.18605E+01_JPRB, 0.19775E+01_JPRB, 0.20621E+01_JPRB, 0.20937E+01_JPRB, 0.20889E+01_JPRB, 0.21073E+01_JPRB ,&
&  0.22373E+01_JPRB, 0.24688E+01_JPRB, 0.28951E+01_JPRB, 0.35677E+01_JPRB, 0.50275E+01_JPRB, 0.47112E+01_JPRB, 0.46911E+01_JPRB ,&
&  0.30511E+01_JPRB, 0.26320E+01_JPRB, 0.24660E+01_JPRB, 0.22683E+01_JPRB, 0.18511E+01_JPRB, 0.16308E+01_JPRB, 0.16349E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.11724E+01_JPRB, 0.11022E+01_JPRB, 0.10449E+01_JPRB, 0.10119E+01_JPRB, 0.10228E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.11264E+01_JPRB, 0.11171E+01_JPRB, 0.11114E+01_JPRB, 0.11076E+01_JPRB, 0.11013E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.11074E+01_JPRB, 0.11381E+01_JPRB, 0.11340E+01_JPRB, 0.11052E+01_JPRB, 0.10880E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11443E+01_JPRB, 0.10894E+01_JPRB, 0.10882E+01_JPRB, 0.10492E+01_JPRB, 0.10184E+01_JPRB, 0.98753E+00_JPRB ,&
&  0.94153E+00_JPRB, 0.84848E+00_JPRB, 0.75102E+00_JPRB, 0.66434E+00_JPRB, 0.65520E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.72569E+00_JPRB, 0.70495E+00_JPRB, 0.70984E+00_JPRB, 0.70449E+00_JPRB, 0.69434E+00_JPRB, 0.69302E+00_JPRB, 0.68731E+00_JPRB ,&
&  0.67920E+00_JPRB, 0.67880E+00_JPRB, 0.67902E+00_JPRB, 0.72150E+00_JPRB, 0.74347E+00_JPRB, 0.72723E+00_JPRB, 0.71018E+00_JPRB ,&
&  0.69513E+00_JPRB, 0.69532E+00_JPRB, 0.70675E+00_JPRB, 0.71580E+00_JPRB, 0.72404E+00_JPRB, 0.74189E+00_JPRB, 0.75613E+00_JPRB ,&
&  0.75293E+00_JPRB, 0.75588E+00_JPRB, 0.77580E+00_JPRB, 0.82491E+00_JPRB, 0.92836E+00_JPRB, 0.11354E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.26636E+01_JPRB, 0.28120E+01_JPRB, 0.11867E+01_JPRB, 0.24201E+01_JPRB, 0.40602E+01_JPRB, 0.43475E+01_JPRB, 0.41527E+01_JPRB ,&
&  0.38270E+01_JPRB, 0.30091E+01_JPRB, 0.25541E+01_JPRB, 0.24399E+01_JPRB, 0.20493E+01_JPRB, 0.17594E+01_JPRB, 0.15802E+01_JPRB ,&
&  0.14971E+01_JPRB, 0.15071E+01_JPRB, 0.15995E+01_JPRB, 0.17084E+01_JPRB, 0.18098E+01_JPRB, 0.18563E+01_JPRB, 0.18681E+01_JPRB ,&
&  0.18969E+01_JPRB, 0.20065E+01_JPRB, 0.22730E+01_JPRB, 0.25900E+01_JPRB, 0.41595E+01_JPRB, 0.50119E+01_JPRB, 0.52644E+01_JPRB ,&
&  0.39609E+01_JPRB, 0.34948E+01_JPRB, 0.30902E+01_JPRB, 0.26677E+01_JPRB, 0.21365E+01_JPRB, 0.19697E+01_JPRB, 0.19452E+01_JPRB ,&
&  0.16377E+01_JPRB, 0.13721E+01_JPRB, 0.12480E+01_JPRB, 0.11600E+01_JPRB, 0.10952E+01_JPRB, 0.10692E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.11032E+01_JPRB, 0.10921E+01_JPRB, 0.10857E+01_JPRB, 0.10916E+01_JPRB, 0.10972E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11028E+01_JPRB, 0.11201E+01_JPRB, 0.11474E+01_JPRB, 0.11424E+01_JPRB, 0.11514E+01_JPRB, 0.11765E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.11273E+01_JPRB, 0.11658E+01_JPRB, 0.11102E+01_JPRB, 0.10608E+01_JPRB, 0.10906E+01_JPRB, 0.10760E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.11737E+01_JPRB, 0.11084E+01_JPRB, 0.79367E+00_JPRB, 0.74018E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.78238E+00_JPRB, 0.75352E+00_JPRB, 0.74915E+00_JPRB, 0.74296E+00_JPRB, 0.72879E+00_JPRB, 0.71821E+00_JPRB, 0.71447E+00_JPRB ,&
&  0.69921E+00_JPRB, 0.67419E+00_JPRB, 0.65128E+00_JPRB, 0.65460E+00_JPRB, 0.67298E+00_JPRB, 0.68701E+00_JPRB, 0.67238E+00_JPRB ,&
&  0.65289E+00_JPRB, 0.64697E+00_JPRB, 0.66205E+00_JPRB, 0.67032E+00_JPRB, 0.67033E+00_JPRB, 0.67721E+00_JPRB, 0.67989E+00_JPRB ,&
&  0.67795E+00_JPRB, 0.67286E+00_JPRB, 0.67547E+00_JPRB, 0.69580E+00_JPRB, 0.74076E+00_JPRB, 0.83718E+00_JPRB, 0.99978E+00_JPRB ,&
&  0.15926E+01_JPRB, 0.21269E+01_JPRB, 0.27861E+01_JPRB, 0.42019E+01_JPRB, 0.53371E+01_JPRB, 0.51997E+01_JPRB, 0.50475E+01_JPRB ,&
&  0.48555E+01_JPRB, 0.40645E+01_JPRB, 0.32584E+01_JPRB, 0.26872E+01_JPRB, 0.21877E+01_JPRB, 0.18251E+01_JPRB, 0.15900E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.13910E+01_JPRB, 0.13797E+01_JPRB, 0.14139E+01_JPRB, 0.14722E+01_JPRB, 0.15342E+01_JPRB, 0.15673E+01_JPRB ,&
&  0.15933E+01_JPRB, 0.16702E+01_JPRB, 0.18467E+01_JPRB, 0.20414E+01_JPRB, 0.28797E+01_JPRB, 0.43516E+01_JPRB, 0.51162E+01_JPRB ,&
&  0.51027E+01_JPRB, 0.45841E+01_JPRB, 0.38458E+01_JPRB, 0.31670E+01_JPRB, 0.24093E+01_JPRB, 0.24450E+01_JPRB, 0.22020E+01_JPRB ,&
&  0.18970E+01_JPRB, 0.16455E+01_JPRB, 0.14543E+01_JPRB, 0.13182E+01_JPRB, 0.12101E+01_JPRB, 0.11270E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.10764E+01_JPRB, 0.10648E+01_JPRB, 0.10497E+01_JPRB, 0.10618E+01_JPRB, 0.10771E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.11132E+01_JPRB, 0.10939E+01_JPRB, 0.11190E+01_JPRB, 0.11533E+01_JPRB, 0.11538E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.11068E+01_JPRB, 0.11075E+01_JPRB, 0.10896E+01_JPRB, 0.10860E+01_JPRB, 0.10897E+01_JPRB, 0.10665E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10403E+01_JPRB, 0.95820E+00_JPRB, 0.85827E+00_JPRB, 0.81463E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.84085E+00_JPRB, 0.81128E+00_JPRB, 0.79216E+00_JPRB, 0.77603E+00_JPRB, 0.76835E+00_JPRB, 0.76804E+00_JPRB, 0.76221E+00_JPRB ,&
&  0.74941E+00_JPRB, 0.72317E+00_JPRB, 0.68879E+00_JPRB, 0.65852E+00_JPRB, 0.65952E+00_JPRB, 0.65763E+00_JPRB, 0.65523E+00_JPRB ,&
&  0.63941E+00_JPRB, 0.63358E+00_JPRB, 0.63963E+00_JPRB, 0.64173E+00_JPRB, 0.64007E+00_JPRB, 0.63836E+00_JPRB, 0.63837E+00_JPRB ,&
&  0.64380E+00_JPRB, 0.64762E+00_JPRB, 0.64704E+00_JPRB, 0.64986E+00_JPRB, 0.66672E+00_JPRB, 0.72293E+00_JPRB, 0.84273E+00_JPRB ,&
&  0.11836E+01_JPRB, 0.19470E+01_JPRB, 0.21847E+01_JPRB, 0.49312E+01_JPRB, 0.57833E+01_JPRB, 0.57549E+01_JPRB, 0.52054E+01_JPRB ,&
&  0.46169E+01_JPRB, 0.44960E+01_JPRB, 0.35610E+01_JPRB, 0.27542E+01_JPRB, 0.21436E+01_JPRB, 0.18063E+01_JPRB, 0.15610E+01_JPRB ,&
&  0.13973E+01_JPRB, 0.13126E+01_JPRB, 0.12827E+01_JPRB, 0.12714E+01_JPRB, 0.12631E+01_JPRB, 0.12791E+01_JPRB, 0.12976E+01_JPRB ,&
&  0.13236E+01_JPRB, 0.14200E+01_JPRB, 0.16083E+01_JPRB, 0.17582E+01_JPRB, 0.22503E+01_JPRB, 0.33938E+01_JPRB, 0.43773E+01_JPRB ,&
&  0.48418E+01_JPRB, 0.46007E+01_JPRB, 0.40973E+01_JPRB, 0.34112E+01_JPRB, 0.28101E+01_JPRB, 0.25843E+01_JPRB, 0.23296E+01_JPRB ,&
&  0.20692E+01_JPRB, 0.18863E+01_JPRB, 0.16393E+01_JPRB, 0.14620E+01_JPRB, 0.13701E+01_JPRB, 0.12565E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.10755E+01_JPRB, 0.10581E+01_JPRB, 0.10394E+01_JPRB, 0.10380E+01_JPRB, 0.10550E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10805E+01_JPRB, 0.10949E+01_JPRB, 0.10561E+01_JPRB, 0.10561E+01_JPRB, 0.10902E+01_JPRB, 0.11007E+01_JPRB, 0.10794E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.10729E+01_JPRB, 0.10796E+01_JPRB, 0.10643E+01_JPRB, 0.10385E+01_JPRB, 0.11070E+01_JPRB, 0.10777E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.98490E+00_JPRB, 0.93958E+00_JPRB, 0.89486E+00_JPRB, 0.86903E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.87033E+00_JPRB, 0.85100E+00_JPRB, 0.83484E+00_JPRB, 0.81530E+00_JPRB, 0.80320E+00_JPRB, 0.79715E+00_JPRB, 0.78877E+00_JPRB ,&
&  0.77177E+00_JPRB, 0.74749E+00_JPRB, 0.71869E+00_JPRB, 0.67963E+00_JPRB, 0.65650E+00_JPRB, 0.65199E+00_JPRB, 0.65820E+00_JPRB ,&
&  0.64641E+00_JPRB, 0.62817E+00_JPRB, 0.62812E+00_JPRB, 0.63226E+00_JPRB, 0.63177E+00_JPRB, 0.62434E+00_JPRB, 0.61698E+00_JPRB ,&
&  0.62275E+00_JPRB, 0.63091E+00_JPRB, 0.63614E+00_JPRB, 0.63774E+00_JPRB, 0.64183E+00_JPRB, 0.66371E+00_JPRB, 0.75688E+00_JPRB ,&
&  0.10626E+01_JPRB, 0.26672E+01_JPRB, 0.23502E+01_JPRB, 0.46222E+01_JPRB, 0.50032E+01_JPRB, 0.49695E+01_JPRB, 0.46046E+01_JPRB ,&
&  0.41244E+01_JPRB, 0.37502E+01_JPRB, 0.30438E+01_JPRB, 0.25133E+01_JPRB, 0.20413E+01_JPRB, 0.16917E+01_JPRB, 0.14657E+01_JPRB ,&
&  0.13241E+01_JPRB, 0.12230E+01_JPRB, 0.11643E+01_JPRB, 0.11440E+01_JPRB, 0.11365E+01_JPRB, 0.11319E+01_JPRB, 0.11397E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.12313E+01_JPRB, 0.14169E+01_JPRB, 0.18140E+01_JPRB, 0.21003E+01_JPRB, 0.29030E+01_JPRB, 0.40943E+01_JPRB ,&
&  0.47165E+01_JPRB, 0.45870E+01_JPRB, 0.38858E+01_JPRB, 0.32678E+01_JPRB, 0.28025E+01_JPRB, 0.24884E+01_JPRB, 0.22229E+01_JPRB ,&
&  0.20423E+01_JPRB, 0.19063E+01_JPRB, 0.17591E+01_JPRB, 0.15787E+01_JPRB, 0.14536E+01_JPRB, 0.13642E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.11805E+01_JPRB, 0.11054E+01_JPRB, 0.10619E+01_JPRB, 0.10282E+01_JPRB, 0.10110E+01_JPRB, 0.10213E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.10505E+01_JPRB, 0.10158E+01_JPRB, 0.10105E+01_JPRB, 0.10379E+01_JPRB, 0.10521E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10540E+01_JPRB, 0.10636E+01_JPRB, 0.10832E+01_JPRB, 0.10492E+01_JPRB, 0.11365E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.98653E+00_JPRB, 0.93754E+00_JPRB, 0.92033E+00_JPRB, 0.89899E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.88245E+00_JPRB, 0.86415E+00_JPRB, 0.85046E+00_JPRB, 0.84064E+00_JPRB, 0.83205E+00_JPRB, 0.81705E+00_JPRB, 0.80051E+00_JPRB ,&
&  0.78523E+00_JPRB, 0.76821E+00_JPRB, 0.73994E+00_JPRB, 0.69815E+00_JPRB, 0.66393E+00_JPRB, 0.64751E+00_JPRB, 0.65005E+00_JPRB ,&
&  0.65023E+00_JPRB, 0.63923E+00_JPRB, 0.62997E+00_JPRB, 0.62881E+00_JPRB, 0.62814E+00_JPRB, 0.62055E+00_JPRB, 0.61070E+00_JPRB ,&
&  0.60478E+00_JPRB, 0.60785E+00_JPRB, 0.61303E+00_JPRB, 0.61874E+00_JPRB, 0.62314E+00_JPRB, 0.63149E+00_JPRB, 0.69309E+00_JPRB ,&
&  0.10098E+01_JPRB, 0.11147E+01_JPRB, 0.28558E+01_JPRB, 0.39398E+01_JPRB, 0.39869E+01_JPRB, 0.38470E+01_JPRB, 0.38639E+01_JPRB ,&
&  0.35527E+01_JPRB, 0.30824E+01_JPRB, 0.25201E+01_JPRB, 0.20642E+01_JPRB, 0.17268E+01_JPRB, 0.15052E+01_JPRB, 0.13416E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.11251E+01_JPRB, 0.10591E+01_JPRB, 0.10220E+01_JPRB, 0.10261E+01_JPRB, 0.10385E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.11050E+01_JPRB, 0.12023E+01_JPRB, 0.15144E+01_JPRB, 0.21872E+01_JPRB, 0.27870E+01_JPRB, 0.37662E+01_JPRB ,&
&  0.46150E+01_JPRB, 0.41905E+01_JPRB, 0.34194E+01_JPRB, 0.28849E+01_JPRB, 0.24979E+01_JPRB, 0.22277E+01_JPRB, 0.20230E+01_JPRB ,&
&  0.18690E+01_JPRB, 0.17575E+01_JPRB, 0.16647E+01_JPRB, 0.15580E+01_JPRB, 0.14696E+01_JPRB, 0.14039E+01_JPRB, 0.13403E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.11884E+01_JPRB, 0.11153E+01_JPRB, 0.10486E+01_JPRB, 0.99621E+00_JPRB, 0.98543E+00_JPRB, 0.99050E+00_JPRB ,&
&  0.98819E+00_JPRB, 0.99251E+00_JPRB, 0.98279E+00_JPRB, 0.98737E+00_JPRB, 0.10207E+01_JPRB, 0.10310E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.10340E+01_JPRB, 0.10359E+01_JPRB, 0.10440E+01_JPRB, 0.99436E+00_JPRB, 0.10768E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.99155E+00_JPRB, 0.94523E+00_JPRB, 0.92820E+00_JPRB, 0.90922E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.88328E+00_JPRB, 0.86082E+00_JPRB, 0.84804E+00_JPRB, 0.83931E+00_JPRB, 0.83153E+00_JPRB, 0.82356E+00_JPRB, 0.80580E+00_JPRB ,&
&  0.78514E+00_JPRB, 0.77347E+00_JPRB, 0.75774E+00_JPRB, 0.73551E+00_JPRB, 0.70119E+00_JPRB, 0.67422E+00_JPRB, 0.66638E+00_JPRB ,&
&  0.66207E+00_JPRB, 0.65436E+00_JPRB, 0.63949E+00_JPRB, 0.63004E+00_JPRB, 0.62102E+00_JPRB, 0.61104E+00_JPRB, 0.60499E+00_JPRB ,&
&  0.59405E+00_JPRB, 0.58998E+00_JPRB, 0.59351E+00_JPRB, 0.60090E+00_JPRB, 0.60726E+00_JPRB, 0.61500E+00_JPRB, 0.63487E+00_JPRB ,&
&  0.90015E+00_JPRB, 0.20940E+01_JPRB, 0.28175E+01_JPRB, 0.32296E+01_JPRB, 0.32419E+01_JPRB, 0.29912E+01_JPRB, 0.29863E+01_JPRB ,&
&  0.26603E+01_JPRB, 0.23221E+01_JPRB, 0.19795E+01_JPRB, 0.16644E+01_JPRB, 0.14132E+01_JPRB, 0.12767E+01_JPRB, 0.12026E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.10510E+01_JPRB, 0.99242E+00_JPRB, 0.94242E+00_JPRB, 0.92473E+00_JPRB, 0.94477E+00_JPRB, 0.97647E+00_JPRB ,&
&  0.99245E+00_JPRB, 0.10060E+01_JPRB, 0.10483E+01_JPRB, 0.12326E+01_JPRB, 0.18007E+01_JPRB, 0.21081E+01_JPRB, 0.30386E+01_JPRB ,&
&  0.35527E+01_JPRB, 0.31904E+01_JPRB, 0.26875E+01_JPRB, 0.23224E+01_JPRB, 0.20273E+01_JPRB, 0.18028E+01_JPRB, 0.16413E+01_JPRB ,&
&  0.15493E+01_JPRB, 0.14837E+01_JPRB, 0.14285E+01_JPRB, 0.13740E+01_JPRB, 0.13244E+01_JPRB, 0.12873E+01_JPRB, 0.12588E+01_JPRB ,&
&  0.12292E+01_JPRB, 0.11889E+01_JPRB, 0.11388E+01_JPRB, 0.10768E+01_JPRB, 0.10132E+01_JPRB, 0.97523E+00_JPRB, 0.95540E+00_JPRB ,&
&  0.93781E+00_JPRB, 0.93987E+00_JPRB, 0.95086E+00_JPRB, 0.95798E+00_JPRB, 0.99185E+00_JPRB, 0.10116E+01_JPRB, 0.10183E+01_JPRB ,&
&  0.98677E+00_JPRB, 0.98522E+00_JPRB, 0.97842E+00_JPRB, 0.97024E+00_JPRB, 0.10071E+01_JPRB, 0.10300E+01_JPRB, 0.97048E+00_JPRB ,&
&  0.94506E+00_JPRB, 0.93188E+00_JPRB, 0.91592E+00_JPRB, 0.90255E+00_JPRB, 0.91065E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.89607E+00_JPRB, 0.86379E+00_JPRB, 0.84315E+00_JPRB, 0.82982E+00_JPRB, 0.82192E+00_JPRB, 0.81345E+00_JPRB, 0.79817E+00_JPRB ,&
&  0.78002E+00_JPRB, 0.76317E+00_JPRB, 0.74954E+00_JPRB, 0.73961E+00_JPRB, 0.71692E+00_JPRB, 0.67861E+00_JPRB, 0.64865E+00_JPRB ,&
&  0.63973E+00_JPRB, 0.63281E+00_JPRB, 0.62603E+00_JPRB, 0.61413E+00_JPRB, 0.60439E+00_JPRB, 0.60009E+00_JPRB, 0.59538E+00_JPRB ,&
&  0.59033E+00_JPRB, 0.58556E+00_JPRB, 0.58718E+00_JPRB, 0.59209E+00_JPRB, 0.59632E+00_JPRB, 0.60018E+00_JPRB, 0.60957E+00_JPRB ,&
&  0.74266E+00_JPRB, 0.14706E+01_JPRB, 0.18201E+01_JPRB, 0.23498E+01_JPRB, 0.25298E+01_JPRB, 0.23349E+01_JPRB, 0.22101E+01_JPRB ,&
&  0.20284E+01_JPRB, 0.18174E+01_JPRB, 0.16246E+01_JPRB, 0.14441E+01_JPRB, 0.12450E+01_JPRB, 0.11241E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.97700E+00_JPRB, 0.93078E+00_JPRB, 0.88481E+00_JPRB, 0.85497E+00_JPRB, 0.86182E+00_JPRB, 0.88213E+00_JPRB ,&
&  0.88602E+00_JPRB, 0.88656E+00_JPRB, 0.89357E+00_JPRB, 0.96517E+00_JPRB, 0.12578E+01_JPRB, 0.15451E+01_JPRB, 0.19502E+01_JPRB ,&
&  0.19838E+01_JPRB, 0.20680E+01_JPRB, 0.19026E+01_JPRB, 0.17190E+01_JPRB, 0.15620E+01_JPRB, 0.14402E+01_JPRB, 0.13484E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.12400E+01_JPRB, 0.12030E+01_JPRB, 0.11724E+01_JPRB, 0.11475E+01_JPRB, 0.11193E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.11011E+01_JPRB, 0.10850E+01_JPRB, 0.10620E+01_JPRB, 0.10323E+01_JPRB, 0.99442E+00_JPRB, 0.96157E+00_JPRB, 0.93553E+00_JPRB ,&
&  0.91067E+00_JPRB, 0.89068E+00_JPRB, 0.89299E+00_JPRB, 0.90999E+00_JPRB, 0.92443E+00_JPRB, 0.93580E+00_JPRB, 0.93543E+00_JPRB ,&
&  0.93125E+00_JPRB, 0.94136E+00_JPRB, 0.92352E+00_JPRB, 0.91813E+00_JPRB, 0.95025E+00_JPRB, 0.95607E+00_JPRB, 0.91386E+00_JPRB ,&
&  0.88778E+00_JPRB, 0.87031E+00_JPRB, 0.86580E+00_JPRB, 0.87031E+00_JPRB, 0.92493E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.92375E+00_JPRB, 0.88709E+00_JPRB, 0.84902E+00_JPRB, 0.82411E+00_JPRB, 0.80625E+00_JPRB, 0.79092E+00_JPRB, 0.77204E+00_JPRB ,&
&  0.75415E+00_JPRB, 0.73703E+00_JPRB, 0.72311E+00_JPRB, 0.71315E+00_JPRB, 0.70240E+00_JPRB, 0.67584E+00_JPRB, 0.63897E+00_JPRB ,&
&  0.61490E+00_JPRB, 0.60752E+00_JPRB, 0.60278E+00_JPRB, 0.59354E+00_JPRB, 0.58476E+00_JPRB, 0.58116E+00_JPRB, 0.57788E+00_JPRB ,&
&  0.57786E+00_JPRB, 0.57873E+00_JPRB, 0.58244E+00_JPRB, 0.58397E+00_JPRB, 0.58103E+00_JPRB, 0.58734E+00_JPRB, 0.58872E+00_JPRB ,&
&  0.68507E+00_JPRB, 0.95937E+00_JPRB, 0.12274E+01_JPRB, 0.15362E+01_JPRB, 0.17696E+01_JPRB, 0.18314E+01_JPRB, 0.17697E+01_JPRB ,&
&  0.16814E+01_JPRB, 0.15901E+01_JPRB, 0.14603E+01_JPRB, 0.13386E+01_JPRB, 0.12026E+01_JPRB, 0.10603E+01_JPRB, 0.97838E+00_JPRB ,&
&  0.94734E+00_JPRB, 0.91353E+00_JPRB, 0.88321E+00_JPRB, 0.85204E+00_JPRB, 0.82582E+00_JPRB, 0.81543E+00_JPRB, 0.81559E+00_JPRB ,&
&  0.81016E+00_JPRB, 0.80081E+00_JPRB, 0.78994E+00_JPRB, 0.79327E+00_JPRB, 0.84591E+00_JPRB, 0.99415E+00_JPRB, 0.11882E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.13174E+01_JPRB, 0.13350E+01_JPRB, 0.13138E+01_JPRB, 0.12564E+01_JPRB, 0.11826E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10179E+01_JPRB, 0.99965E+00_JPRB, 0.99236E+00_JPRB, 0.98812E+00_JPRB, 0.97875E+00_JPRB, 0.96359E+00_JPRB ,&
&  0.95174E+00_JPRB, 0.94688E+00_JPRB, 0.94657E+00_JPRB, 0.94005E+00_JPRB, 0.93011E+00_JPRB, 0.91741E+00_JPRB, 0.90669E+00_JPRB ,&
&  0.90056E+00_JPRB, 0.89215E+00_JPRB, 0.87795E+00_JPRB, 0.86659E+00_JPRB, 0.86967E+00_JPRB, 0.87101E+00_JPRB, 0.86796E+00_JPRB ,&
&  0.86491E+00_JPRB, 0.87545E+00_JPRB, 0.86690E+00_JPRB, 0.90735E+00_JPRB, 0.94586E+00_JPRB, 0.90043E+00_JPRB, 0.86804E+00_JPRB ,&
&  0.83393E+00_JPRB, 0.81639E+00_JPRB, 0.81852E+00_JPRB, 0.83922E+00_JPRB, 0.91094E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.79472E+00_JPRB, 0.78765E+00_JPRB, 0.77772E+00_JPRB, 0.76827E+00_JPRB, 0.75392E+00_JPRB, 0.73981E+00_JPRB, 0.72241E+00_JPRB ,&
&  0.70606E+00_JPRB, 0.69050E+00_JPRB, 0.67769E+00_JPRB, 0.67358E+00_JPRB, 0.66616E+00_JPRB, 0.65433E+00_JPRB, 0.62811E+00_JPRB ,&
&  0.60627E+00_JPRB, 0.59018E+00_JPRB, 0.58043E+00_JPRB, 0.57558E+00_JPRB, 0.57174E+00_JPRB, 0.56728E+00_JPRB, 0.56529E+00_JPRB ,&
&  0.56588E+00_JPRB, 0.56538E+00_JPRB, 0.56737E+00_JPRB, 0.56609E+00_JPRB, 0.56783E+00_JPRB, 0.56315E+00_JPRB, 0.55900E+00_JPRB ,&
&  0.60864E+00_JPRB, 0.72583E+00_JPRB, 0.88653E+00_JPRB, 0.10625E+01_JPRB, 0.12609E+01_JPRB, 0.13993E+01_JPRB, 0.14853E+01_JPRB ,&
&  0.15191E+01_JPRB, 0.14462E+01_JPRB, 0.13373E+01_JPRB, 0.12166E+01_JPRB, 0.11227E+01_JPRB, 0.10271E+01_JPRB, 0.93269E+00_JPRB ,&
&  0.88478E+00_JPRB, 0.85689E+00_JPRB, 0.83415E+00_JPRB, 0.81621E+00_JPRB, 0.80149E+00_JPRB, 0.79203E+00_JPRB, 0.78432E+00_JPRB ,&
&  0.77034E+00_JPRB, 0.75334E+00_JPRB, 0.73358E+00_JPRB, 0.72478E+00_JPRB, 0.72920E+00_JPRB, 0.77032E+00_JPRB, 0.86341E+00_JPRB ,&
&  0.90841E+00_JPRB, 0.92994E+00_JPRB, 0.94554E+00_JPRB, 0.97119E+00_JPRB, 0.98235E+00_JPRB, 0.97229E+00_JPRB, 0.92727E+00_JPRB ,&
&  0.88631E+00_JPRB, 0.85608E+00_JPRB, 0.84010E+00_JPRB, 0.83476E+00_JPRB, 0.83714E+00_JPRB, 0.83943E+00_JPRB, 0.84047E+00_JPRB ,&
&  0.83201E+00_JPRB, 0.82240E+00_JPRB, 0.81891E+00_JPRB, 0.81922E+00_JPRB, 0.82213E+00_JPRB, 0.82443E+00_JPRB, 0.82948E+00_JPRB ,&
&  0.84091E+00_JPRB, 0.85702E+00_JPRB, 0.87081E+00_JPRB, 0.86146E+00_JPRB, 0.83932E+00_JPRB, 0.82254E+00_JPRB, 0.81297E+00_JPRB ,&
&  0.80830E+00_JPRB, 0.81517E+00_JPRB, 0.82439E+00_JPRB, 0.86152E+00_JPRB, 0.87062E+00_JPRB, 0.84884E+00_JPRB, 0.81413E+00_JPRB ,&
&  0.78513E+00_JPRB, 0.76923E+00_JPRB, 0.76919E+00_JPRB, 0.78076E+00_JPRB, 0.76824E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.71408E+00_JPRB, 0.71026E+00_JPRB, 0.70463E+00_JPRB, 0.70038E+00_JPRB, 0.69154E+00_JPRB, 0.68165E+00_JPRB, 0.67118E+00_JPRB ,&
&  0.65809E+00_JPRB, 0.64514E+00_JPRB, 0.63427E+00_JPRB, 0.63072E+00_JPRB, 0.62898E+00_JPRB, 0.62774E+00_JPRB, 0.61987E+00_JPRB ,&
&  0.60650E+00_JPRB, 0.59385E+00_JPRB, 0.58276E+00_JPRB, 0.57535E+00_JPRB, 0.57206E+00_JPRB, 0.56827E+00_JPRB, 0.56353E+00_JPRB ,&
&  0.55978E+00_JPRB, 0.55885E+00_JPRB, 0.55676E+00_JPRB, 0.55441E+00_JPRB, 0.55113E+00_JPRB, 0.54432E+00_JPRB, 0.54252E+00_JPRB ,&
&  0.55998E+00_JPRB, 0.57231E+00_JPRB, 0.65880E+00_JPRB, 0.72999E+00_JPRB, 0.81814E+00_JPRB, 0.95737E+00_JPRB, 0.11152E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.12441E+01_JPRB, 0.11946E+01_JPRB, 0.11088E+01_JPRB, 0.10468E+01_JPRB, 0.10024E+01_JPRB, 0.94048E+00_JPRB ,&
&  0.87941E+00_JPRB, 0.83734E+00_JPRB, 0.80427E+00_JPRB, 0.77825E+00_JPRB, 0.75911E+00_JPRB, 0.74264E+00_JPRB, 0.72693E+00_JPRB ,&
&  0.70768E+00_JPRB, 0.68878E+00_JPRB, 0.67357E+00_JPRB, 0.66677E+00_JPRB, 0.66976E+00_JPRB, 0.67933E+00_JPRB, 0.70866E+00_JPRB ,&
&  0.73265E+00_JPRB, 0.74348E+00_JPRB, 0.74186E+00_JPRB, 0.75184E+00_JPRB, 0.77103E+00_JPRB, 0.78717E+00_JPRB, 0.78649E+00_JPRB ,&
&  0.76643E+00_JPRB, 0.73900E+00_JPRB, 0.71641E+00_JPRB, 0.70053E+00_JPRB, 0.69270E+00_JPRB, 0.69394E+00_JPRB, 0.70169E+00_JPRB ,&
&  0.70454E+00_JPRB, 0.70238E+00_JPRB, 0.70113E+00_JPRB, 0.70390E+00_JPRB, 0.70929E+00_JPRB, 0.71941E+00_JPRB, 0.73313E+00_JPRB ,&
&  0.74985E+00_JPRB, 0.76917E+00_JPRB, 0.79137E+00_JPRB, 0.80881E+00_JPRB, 0.81323E+00_JPRB, 0.81017E+00_JPRB, 0.80182E+00_JPRB ,&
&  0.79275E+00_JPRB, 0.78880E+00_JPRB, 0.78481E+00_JPRB, 0.79830E+00_JPRB, 0.82346E+00_JPRB, 0.81742E+00_JPRB, 0.78808E+00_JPRB ,&
&  0.76149E+00_JPRB, 0.73673E+00_JPRB, 0.72485E+00_JPRB, 0.67198E+00_JPRB, 0.70837E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.64552E+00_JPRB, 0.64238E+00_JPRB, 0.63633E+00_JPRB, 0.63077E+00_JPRB, 0.62694E+00_JPRB, 0.62128E+00_JPRB, 0.61675E+00_JPRB ,&
&  0.61037E+00_JPRB, 0.60525E+00_JPRB, 0.59696E+00_JPRB, 0.59145E+00_JPRB, 0.58876E+00_JPRB, 0.58883E+00_JPRB, 0.58873E+00_JPRB ,&
&  0.58444E+00_JPRB, 0.57669E+00_JPRB, 0.56861E+00_JPRB, 0.56208E+00_JPRB, 0.55658E+00_JPRB, 0.55041E+00_JPRB, 0.54414E+00_JPRB ,&
&  0.53834E+00_JPRB, 0.53327E+00_JPRB, 0.52880E+00_JPRB, 0.52744E+00_JPRB, 0.52626E+00_JPRB, 0.52407E+00_JPRB, 0.52058E+00_JPRB ,&
&  0.52620E+00_JPRB, 0.47957E+00_JPRB, 0.50370E+00_JPRB, 0.53097E+00_JPRB, 0.56778E+00_JPRB, 0.65924E+00_JPRB, 0.79110E+00_JPRB ,&
&  0.91801E+00_JPRB, 0.99990E+00_JPRB, 0.10194E+01_JPRB, 0.98162E+00_JPRB, 0.93925E+00_JPRB, 0.91393E+00_JPRB, 0.89303E+00_JPRB ,&
&  0.85211E+00_JPRB, 0.81480E+00_JPRB, 0.77717E+00_JPRB, 0.73900E+00_JPRB, 0.70373E+00_JPRB, 0.67231E+00_JPRB, 0.64724E+00_JPRB ,&
&  0.63071E+00_JPRB, 0.62198E+00_JPRB, 0.61759E+00_JPRB, 0.61591E+00_JPRB, 0.61730E+00_JPRB, 0.61932E+00_JPRB, 0.62072E+00_JPRB ,&
&  0.62501E+00_JPRB, 0.63369E+00_JPRB, 0.63526E+00_JPRB, 0.63690E+00_JPRB, 0.64476E+00_JPRB, 0.65848E+00_JPRB, 0.66337E+00_JPRB ,&
&  0.65889E+00_JPRB, 0.64780E+00_JPRB, 0.63452E+00_JPRB, 0.62167E+00_JPRB, 0.61096E+00_JPRB, 0.60179E+00_JPRB, 0.59802E+00_JPRB ,&
&  0.59574E+00_JPRB, 0.59637E+00_JPRB, 0.59857E+00_JPRB, 0.60484E+00_JPRB, 0.61300E+00_JPRB, 0.62180E+00_JPRB, 0.63074E+00_JPRB ,&
&  0.63982E+00_JPRB, 0.64959E+00_JPRB, 0.66372E+00_JPRB, 0.68038E+00_JPRB, 0.69495E+00_JPRB, 0.70328E+00_JPRB, 0.70654E+00_JPRB ,&
&  0.70657E+00_JPRB, 0.70691E+00_JPRB, 0.70325E+00_JPRB, 0.71408E+00_JPRB, 0.71912E+00_JPRB, 0.72443E+00_JPRB, 0.71684E+00_JPRB ,&
&  0.70371E+00_JPRB, 0.68916E+00_JPRB, 0.60702E+00_JPRB, 0.63868E+00_JPRB, 0.64350E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.59910E+00_JPRB, 0.59173E+00_JPRB, 0.58440E+00_JPRB, 0.57688E+00_JPRB, 0.57118E+00_JPRB, 0.56735E+00_JPRB, 0.56584E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.56850E+00_JPRB, 0.56705E+00_JPRB, 0.56101E+00_JPRB, 0.55492E+00_JPRB, 0.55120E+00_JPRB, 0.54984E+00_JPRB ,&
&  0.54782E+00_JPRB, 0.54367E+00_JPRB, 0.53808E+00_JPRB, 0.53258E+00_JPRB, 0.52615E+00_JPRB, 0.52049E+00_JPRB, 0.51543E+00_JPRB ,&
&  0.51099E+00_JPRB, 0.50833E+00_JPRB, 0.50607E+00_JPRB, 0.50426E+00_JPRB, 0.50371E+00_JPRB, 0.50220E+00_JPRB, 0.50474E+00_JPRB ,&
&  0.47666E+00_JPRB, 0.44665E+00_JPRB, 0.46156E+00_JPRB, 0.47558E+00_JPRB, 0.48995E+00_JPRB, 0.52887E+00_JPRB, 0.59729E+00_JPRB ,&
&  0.69111E+00_JPRB, 0.77523E+00_JPRB, 0.82811E+00_JPRB, 0.82749E+00_JPRB, 0.80609E+00_JPRB, 0.79411E+00_JPRB, 0.78879E+00_JPRB ,&
&  0.76798E+00_JPRB, 0.74271E+00_JPRB, 0.71907E+00_JPRB, 0.69100E+00_JPRB, 0.66111E+00_JPRB, 0.63494E+00_JPRB, 0.61456E+00_JPRB ,&
&  0.59935E+00_JPRB, 0.59046E+00_JPRB, 0.58661E+00_JPRB, 0.58340E+00_JPRB, 0.57992E+00_JPRB, 0.57777E+00_JPRB, 0.57427E+00_JPRB ,&
&  0.57231E+00_JPRB, 0.57170E+00_JPRB, 0.57347E+00_JPRB, 0.57370E+00_JPRB, 0.57523E+00_JPRB, 0.57556E+00_JPRB, 0.57357E+00_JPRB ,&
&  0.56900E+00_JPRB, 0.56464E+00_JPRB, 0.56208E+00_JPRB, 0.55671E+00_JPRB, 0.54637E+00_JPRB, 0.53638E+00_JPRB, 0.53128E+00_JPRB ,&
&  0.52990E+00_JPRB, 0.52960E+00_JPRB, 0.52978E+00_JPRB, 0.53223E+00_JPRB, 0.53673E+00_JPRB, 0.54244E+00_JPRB, 0.54887E+00_JPRB ,&
&  0.55596E+00_JPRB, 0.56250E+00_JPRB, 0.56865E+00_JPRB, 0.57603E+00_JPRB, 0.58715E+00_JPRB, 0.59833E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.61639E+00_JPRB, 0.62326E+00_JPRB, 0.62925E+00_JPRB, 0.63836E+00_JPRB, 0.64879E+00_JPRB, 0.65530E+00_JPRB, 0.65423E+00_JPRB ,&
&  0.64642E+00_JPRB, 0.63252E+00_JPRB, 0.62477E+00_JPRB, 0.61365E+00_JPRB, 0.60486E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.53942E+00_JPRB, 0.53453E+00_JPRB, 0.52856E+00_JPRB, 0.52231E+00_JPRB, 0.51630E+00_JPRB, 0.51343E+00_JPRB, 0.51278E+00_JPRB ,&
&  0.51520E+00_JPRB, 0.51651E+00_JPRB, 0.51913E+00_JPRB, 0.51770E+00_JPRB, 0.51491E+00_JPRB, 0.51237E+00_JPRB, 0.50974E+00_JPRB ,&
&  0.50678E+00_JPRB, 0.50283E+00_JPRB, 0.49880E+00_JPRB, 0.49555E+00_JPRB, 0.49192E+00_JPRB, 0.48703E+00_JPRB, 0.48239E+00_JPRB ,&
&  0.47861E+00_JPRB, 0.47643E+00_JPRB, 0.47483E+00_JPRB, 0.47340E+00_JPRB, 0.47193E+00_JPRB, 0.47178E+00_JPRB, 0.47076E+00_JPRB ,&
&  0.47335E+00_JPRB, 0.44500E+00_JPRB, 0.45785E+00_JPRB, 0.46251E+00_JPRB, 0.47048E+00_JPRB, 0.48812E+00_JPRB, 0.51267E+00_JPRB ,&
&  0.55094E+00_JPRB, 0.60259E+00_JPRB, 0.64871E+00_JPRB, 0.66844E+00_JPRB, 0.66510E+00_JPRB, 0.66335E+00_JPRB, 0.66950E+00_JPRB ,&
&  0.66818E+00_JPRB, 0.66025E+00_JPRB, 0.65402E+00_JPRB, 0.64400E+00_JPRB, 0.62797E+00_JPRB, 0.60713E+00_JPRB, 0.58864E+00_JPRB ,&
&  0.57706E+00_JPRB, 0.57012E+00_JPRB, 0.56368E+00_JPRB, 0.55647E+00_JPRB, 0.54666E+00_JPRB, 0.53664E+00_JPRB, 0.52752E+00_JPRB ,&
&  0.51945E+00_JPRB, 0.51066E+00_JPRB, 0.50513E+00_JPRB, 0.50199E+00_JPRB, 0.49853E+00_JPRB, 0.49650E+00_JPRB, 0.49402E+00_JPRB ,&
&  0.49021E+00_JPRB, 0.48572E+00_JPRB, 0.48301E+00_JPRB, 0.48205E+00_JPRB, 0.48008E+00_JPRB, 0.47819E+00_JPRB, 0.47611E+00_JPRB ,&
&  0.47592E+00_JPRB, 0.47600E+00_JPRB, 0.47513E+00_JPRB, 0.47479E+00_JPRB, 0.47594E+00_JPRB, 0.47938E+00_JPRB, 0.48380E+00_JPRB ,&
&  0.48898E+00_JPRB, 0.49485E+00_JPRB, 0.50037E+00_JPRB, 0.50545E+00_JPRB, 0.51243E+00_JPRB, 0.51950E+00_JPRB, 0.52838E+00_JPRB ,&
&  0.53819E+00_JPRB, 0.54714E+00_JPRB, 0.55528E+00_JPRB, 0.56339E+00_JPRB, 0.57042E+00_JPRB, 0.57288E+00_JPRB, 0.57347E+00_JPRB ,&
&  0.57228E+00_JPRB, 0.56469E+00_JPRB, 0.55773E+00_JPRB, 0.54991E+00_JPRB, 0.54303E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.49276E+00_JPRB, 0.48743E+00_JPRB, 0.48157E+00_JPRB, 0.47696E+00_JPRB, 0.47298E+00_JPRB, 0.46905E+00_JPRB, 0.46737E+00_JPRB ,&
&  0.46783E+00_JPRB, 0.46983E+00_JPRB, 0.47312E+00_JPRB, 0.47525E+00_JPRB, 0.47632E+00_JPRB, 0.47741E+00_JPRB, 0.47712E+00_JPRB ,&
&  0.47604E+00_JPRB, 0.47409E+00_JPRB, 0.47135E+00_JPRB, 0.46767E+00_JPRB, 0.46344E+00_JPRB, 0.45944E+00_JPRB, 0.45439E+00_JPRB ,&
&  0.45062E+00_JPRB, 0.44808E+00_JPRB, 0.44638E+00_JPRB, 0.44541E+00_JPRB, 0.44574E+00_JPRB, 0.44632E+00_JPRB, 0.44651E+00_JPRB ,&
&  0.44423E+00_JPRB, 0.44630E+00_JPRB, 0.45369E+00_JPRB, 0.45449E+00_JPRB, 0.46028E+00_JPRB, 0.47276E+00_JPRB, 0.48435E+00_JPRB ,&
&  0.49407E+00_JPRB, 0.51360E+00_JPRB, 0.54104E+00_JPRB, 0.56190E+00_JPRB, 0.56665E+00_JPRB, 0.56758E+00_JPRB, 0.57611E+00_JPRB ,&
&  0.58154E+00_JPRB, 0.58082E+00_JPRB, 0.58204E+00_JPRB, 0.58179E+00_JPRB, 0.57828E+00_JPRB, 0.57289E+00_JPRB, 0.56373E+00_JPRB ,&
&  0.55352E+00_JPRB, 0.54443E+00_JPRB, 0.53459E+00_JPRB, 0.52444E+00_JPRB, 0.51174E+00_JPRB, 0.49814E+00_JPRB, 0.48400E+00_JPRB ,&
&  0.47008E+00_JPRB, 0.45999E+00_JPRB, 0.45331E+00_JPRB, 0.44764E+00_JPRB, 0.44241E+00_JPRB, 0.43907E+00_JPRB, 0.43538E+00_JPRB ,&
&  0.43311E+00_JPRB, 0.43251E+00_JPRB, 0.43072E+00_JPRB, 0.42979E+00_JPRB, 0.43065E+00_JPRB, 0.43144E+00_JPRB, 0.43359E+00_JPRB ,&
&  0.43420E+00_JPRB, 0.43444E+00_JPRB, 0.43475E+00_JPRB, 0.43446E+00_JPRB, 0.43354E+00_JPRB, 0.43467E+00_JPRB, 0.43742E+00_JPRB ,&
&  0.44062E+00_JPRB, 0.44544E+00_JPRB, 0.45078E+00_JPRB, 0.45532E+00_JPRB, 0.45954E+00_JPRB, 0.46597E+00_JPRB, 0.47421E+00_JPRB ,&
&  0.48349E+00_JPRB, 0.49376E+00_JPRB, 0.50309E+00_JPRB, 0.51078E+00_JPRB, 0.51433E+00_JPRB, 0.51527E+00_JPRB, 0.51571E+00_JPRB ,&
&  0.51448E+00_JPRB, 0.50866E+00_JPRB, 0.50506E+00_JPRB, 0.50195E+00_JPRB, 0.49754E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.43686E+00_JPRB, 0.43440E+00_JPRB, 0.43233E+00_JPRB, 0.43126E+00_JPRB, 0.43058E+00_JPRB, 0.43034E+00_JPRB, 0.43093E+00_JPRB ,&
&  0.43260E+00_JPRB, 0.43412E+00_JPRB, 0.43841E+00_JPRB, 0.44098E+00_JPRB, 0.44234E+00_JPRB, 0.44413E+00_JPRB, 0.44524E+00_JPRB ,&
&  0.44421E+00_JPRB, 0.44319E+00_JPRB, 0.44174E+00_JPRB, 0.43896E+00_JPRB, 0.43454E+00_JPRB, 0.43167E+00_JPRB, 0.42886E+00_JPRB ,&
&  0.42489E+00_JPRB, 0.42031E+00_JPRB, 0.41618E+00_JPRB, 0.41360E+00_JPRB, 0.41281E+00_JPRB, 0.41155E+00_JPRB, 0.41128E+00_JPRB ,&
&  0.40998E+00_JPRB, 0.41761E+00_JPRB, 0.40725E+00_JPRB, 0.41700E+00_JPRB, 0.42537E+00_JPRB, 0.43374E+00_JPRB, 0.44270E+00_JPRB ,&
&  0.44894E+00_JPRB, 0.45667E+00_JPRB, 0.47121E+00_JPRB, 0.48491E+00_JPRB, 0.49096E+00_JPRB, 0.49028E+00_JPRB, 0.49304E+00_JPRB ,&
&  0.49525E+00_JPRB, 0.49228E+00_JPRB, 0.49224E+00_JPRB, 0.49604E+00_JPRB, 0.49992E+00_JPRB, 0.50100E+00_JPRB, 0.50059E+00_JPRB ,&
&  0.50001E+00_JPRB, 0.49456E+00_JPRB, 0.48454E+00_JPRB, 0.47351E+00_JPRB, 0.46315E+00_JPRB, 0.45213E+00_JPRB, 0.43931E+00_JPRB ,&
&  0.42561E+00_JPRB, 0.41453E+00_JPRB, 0.40539E+00_JPRB, 0.39905E+00_JPRB, 0.39516E+00_JPRB, 0.39163E+00_JPRB, 0.38847E+00_JPRB ,&
&  0.38702E+00_JPRB, 0.38665E+00_JPRB, 0.38607E+00_JPRB, 0.38538E+00_JPRB, 0.38612E+00_JPRB, 0.38743E+00_JPRB, 0.39144E+00_JPRB ,&
&  0.39602E+00_JPRB, 0.39872E+00_JPRB, 0.40025E+00_JPRB, 0.40147E+00_JPRB, 0.40057E+00_JPRB, 0.40143E+00_JPRB, 0.40347E+00_JPRB ,&
&  0.40569E+00_JPRB, 0.40859E+00_JPRB, 0.41071E+00_JPRB, 0.41287E+00_JPRB, 0.41572E+00_JPRB, 0.41992E+00_JPRB, 0.42634E+00_JPRB ,&
&  0.43284E+00_JPRB, 0.43994E+00_JPRB, 0.44763E+00_JPRB, 0.45414E+00_JPRB, 0.45818E+00_JPRB, 0.45969E+00_JPRB, 0.46038E+00_JPRB ,&
&  0.45888E+00_JPRB, 0.45397E+00_JPRB, 0.44851E+00_JPRB, 0.44412E+00_JPRB, 0.44024E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.39435E+00_JPRB, 0.39429E+00_JPRB, 0.39525E+00_JPRB, 0.39547E+00_JPRB, 0.39528E+00_JPRB, 0.39617E+00_JPRB, 0.39939E+00_JPRB ,&
&  0.40237E+00_JPRB, 0.40453E+00_JPRB, 0.40676E+00_JPRB, 0.40841E+00_JPRB, 0.40968E+00_JPRB, 0.41222E+00_JPRB, 0.41421E+00_JPRB ,&
&  0.41486E+00_JPRB, 0.41396E+00_JPRB, 0.41263E+00_JPRB, 0.41070E+00_JPRB, 0.40795E+00_JPRB, 0.40485E+00_JPRB, 0.40210E+00_JPRB ,&
&  0.39853E+00_JPRB, 0.39447E+00_JPRB, 0.39040E+00_JPRB, 0.38804E+00_JPRB, 0.38685E+00_JPRB, 0.38600E+00_JPRB, 0.38617E+00_JPRB ,&
&  0.38621E+00_JPRB, 0.38889E+00_JPRB, 0.39212E+00_JPRB, 0.39606E+00_JPRB, 0.39978E+00_JPRB, 0.40450E+00_JPRB, 0.40954E+00_JPRB ,&
&  0.41421E+00_JPRB, 0.41686E+00_JPRB, 0.42066E+00_JPRB, 0.42261E+00_JPRB, 0.42924E+00_JPRB, 0.43441E+00_JPRB, 0.43743E+00_JPRB ,&
&  0.43724E+00_JPRB, 0.43044E+00_JPRB, 0.42573E+00_JPRB, 0.42642E+00_JPRB, 0.42938E+00_JPRB, 0.43253E+00_JPRB, 0.43555E+00_JPRB ,&
&  0.43674E+00_JPRB, 0.43529E+00_JPRB, 0.43022E+00_JPRB, 0.42284E+00_JPRB, 0.41486E+00_JPRB, 0.40609E+00_JPRB, 0.39498E+00_JPRB ,&
&  0.38476E+00_JPRB, 0.37643E+00_JPRB, 0.36967E+00_JPRB, 0.36374E+00_JPRB, 0.35945E+00_JPRB, 0.35505E+00_JPRB, 0.35316E+00_JPRB ,&
&  0.35309E+00_JPRB, 0.35500E+00_JPRB, 0.35605E+00_JPRB, 0.35532E+00_JPRB, 0.35410E+00_JPRB, 0.35434E+00_JPRB, 0.35751E+00_JPRB ,&
&  0.36314E+00_JPRB, 0.36760E+00_JPRB, 0.36960E+00_JPRB, 0.37009E+00_JPRB, 0.36993E+00_JPRB, 0.37158E+00_JPRB, 0.37429E+00_JPRB ,&
&  0.37585E+00_JPRB, 0.37672E+00_JPRB, 0.37796E+00_JPRB, 0.37783E+00_JPRB, 0.37977E+00_JPRB, 0.38314E+00_JPRB, 0.38704E+00_JPRB ,&
&  0.39099E+00_JPRB, 0.39592E+00_JPRB, 0.40089E+00_JPRB, 0.40492E+00_JPRB, 0.40839E+00_JPRB, 0.40876E+00_JPRB, 0.40896E+00_JPRB ,&
&  0.40860E+00_JPRB, 0.40645E+00_JPRB, 0.40391E+00_JPRB, 0.40060E+00_JPRB, 0.39679E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.35923E+00_JPRB, 0.36219E+00_JPRB, 0.36592E+00_JPRB, 0.36656E+00_JPRB, 0.36669E+00_JPRB, 0.36811E+00_JPRB, 0.37143E+00_JPRB ,&
&  0.37396E+00_JPRB, 0.37506E+00_JPRB, 0.37563E+00_JPRB, 0.37613E+00_JPRB, 0.37736E+00_JPRB, 0.37990E+00_JPRB, 0.38135E+00_JPRB ,&
&  0.38112E+00_JPRB, 0.38058E+00_JPRB, 0.37969E+00_JPRB, 0.37878E+00_JPRB, 0.37687E+00_JPRB, 0.37406E+00_JPRB, 0.37149E+00_JPRB ,&
&  0.36799E+00_JPRB, 0.36463E+00_JPRB, 0.36186E+00_JPRB, 0.35946E+00_JPRB, 0.35773E+00_JPRB, 0.35701E+00_JPRB, 0.35803E+00_JPRB ,&
&  0.35947E+00_JPRB, 0.36159E+00_JPRB, 0.36458E+00_JPRB, 0.36829E+00_JPRB, 0.37190E+00_JPRB, 0.37591E+00_JPRB, 0.37947E+00_JPRB ,&
&  0.38225E+00_JPRB, 0.38479E+00_JPRB, 0.38705E+00_JPRB, 0.38593E+00_JPRB, 0.38504E+00_JPRB, 0.38874E+00_JPRB, 0.39316E+00_JPRB ,&
&  0.39216E+00_JPRB, 0.38717E+00_JPRB, 0.38105E+00_JPRB, 0.37807E+00_JPRB, 0.37754E+00_JPRB, 0.37668E+00_JPRB, 0.37539E+00_JPRB ,&
&  0.37450E+00_JPRB, 0.37337E+00_JPRB, 0.37109E+00_JPRB, 0.36745E+00_JPRB, 0.36234E+00_JPRB, 0.35603E+00_JPRB, 0.35005E+00_JPRB ,&
&  0.34623E+00_JPRB, 0.34349E+00_JPRB, 0.34276E+00_JPRB, 0.33978E+00_JPRB, 0.33442E+00_JPRB, 0.32911E+00_JPRB, 0.32757E+00_JPRB ,&
&  0.32729E+00_JPRB, 0.32828E+00_JPRB, 0.32885E+00_JPRB, 0.32827E+00_JPRB, 0.32872E+00_JPRB, 0.32925E+00_JPRB, 0.33098E+00_JPRB ,&
&  0.33656E+00_JPRB, 0.34072E+00_JPRB, 0.34113E+00_JPRB, 0.34036E+00_JPRB, 0.33852E+00_JPRB, 0.33829E+00_JPRB, 0.33955E+00_JPRB ,&
&  0.34096E+00_JPRB, 0.34187E+00_JPRB, 0.34249E+00_JPRB, 0.34205E+00_JPRB, 0.34211E+00_JPRB, 0.34360E+00_JPRB, 0.34557E+00_JPRB ,&
&  0.34808E+00_JPRB, 0.35117E+00_JPRB, 0.35388E+00_JPRB, 0.35664E+00_JPRB, 0.35924E+00_JPRB, 0.36062E+00_JPRB, 0.36184E+00_JPRB ,&
&  0.36295E+00_JPRB, 0.36292E+00_JPRB, 0.36216E+00_JPRB, 0.36135E+00_JPRB, 0.36028E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.33217E+00_JPRB, 0.33666E+00_JPRB, 0.34216E+00_JPRB, 0.34310E+00_JPRB, 0.34486E+00_JPRB, 0.34712E+00_JPRB, 0.35061E+00_JPRB ,&
&  0.35320E+00_JPRB, 0.35392E+00_JPRB, 0.35279E+00_JPRB, 0.35154E+00_JPRB, 0.35115E+00_JPRB, 0.35148E+00_JPRB, 0.35151E+00_JPRB ,&
&  0.35170E+00_JPRB, 0.35204E+00_JPRB, 0.35198E+00_JPRB, 0.35160E+00_JPRB, 0.35001E+00_JPRB, 0.34768E+00_JPRB, 0.34498E+00_JPRB ,&
&  0.34082E+00_JPRB, 0.33782E+00_JPRB, 0.33642E+00_JPRB, 0.33545E+00_JPRB, 0.33402E+00_JPRB, 0.33409E+00_JPRB, 0.33564E+00_JPRB ,&
&  0.33933E+00_JPRB, 0.34218E+00_JPRB, 0.34595E+00_JPRB, 0.34979E+00_JPRB, 0.35204E+00_JPRB, 0.35375E+00_JPRB, 0.35571E+00_JPRB ,&
&  0.35698E+00_JPRB, 0.35732E+00_JPRB, 0.35761E+00_JPRB, 0.35814E+00_JPRB, 0.35676E+00_JPRB, 0.35890E+00_JPRB, 0.36259E+00_JPRB ,&
&  0.36417E+00_JPRB, 0.36120E+00_JPRB, 0.35629E+00_JPRB, 0.35262E+00_JPRB, 0.34907E+00_JPRB, 0.34629E+00_JPRB, 0.34387E+00_JPRB ,&
&  0.34193E+00_JPRB, 0.34059E+00_JPRB, 0.33979E+00_JPRB, 0.33725E+00_JPRB, 0.33086E+00_JPRB, 0.32472E+00_JPRB, 0.32128E+00_JPRB ,&
&  0.31963E+00_JPRB, 0.31903E+00_JPRB, 0.32019E+00_JPRB, 0.31842E+00_JPRB, 0.31328E+00_JPRB, 0.30881E+00_JPRB, 0.30693E+00_JPRB ,&
&  0.30517E+00_JPRB, 0.30469E+00_JPRB, 0.30591E+00_JPRB, 0.30685E+00_JPRB, 0.30750E+00_JPRB, 0.30757E+00_JPRB, 0.30938E+00_JPRB ,&
&  0.31180E+00_JPRB, 0.31423E+00_JPRB, 0.31433E+00_JPRB, 0.31146E+00_JPRB, 0.30740E+00_JPRB, 0.30473E+00_JPRB, 0.30482E+00_JPRB ,&
&  0.30716E+00_JPRB, 0.30939E+00_JPRB, 0.31044E+00_JPRB, 0.31111E+00_JPRB, 0.31249E+00_JPRB, 0.31519E+00_JPRB, 0.31859E+00_JPRB ,&
&  0.32018E+00_JPRB, 0.32169E+00_JPRB, 0.32376E+00_JPRB, 0.32586E+00_JPRB, 0.32733E+00_JPRB, 0.32820E+00_JPRB, 0.32848E+00_JPRB ,&
&  0.32785E+00_JPRB, 0.32841E+00_JPRB, 0.32979E+00_JPRB, 0.33139E+00_JPRB, 0.33204E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.31300E+00_JPRB, 0.31597E+00_JPRB, 0.31870E+00_JPRB, 0.31997E+00_JPRB, 0.32147E+00_JPRB, 0.32341E+00_JPRB, 0.32697E+00_JPRB ,&
&  0.33130E+00_JPRB, 0.33300E+00_JPRB, 0.33130E+00_JPRB, 0.32831E+00_JPRB, 0.32657E+00_JPRB, 0.32606E+00_JPRB, 0.32626E+00_JPRB ,&
&  0.32645E+00_JPRB, 0.32657E+00_JPRB, 0.32584E+00_JPRB, 0.32390E+00_JPRB, 0.32101E+00_JPRB, 0.31794E+00_JPRB, 0.31539E+00_JPRB ,&
&  0.31450E+00_JPRB, 0.31548E+00_JPRB, 0.31725E+00_JPRB, 0.31692E+00_JPRB, 0.31680E+00_JPRB, 0.31815E+00_JPRB, 0.32053E+00_JPRB ,&
&  0.32472E+00_JPRB, 0.32653E+00_JPRB, 0.32796E+00_JPRB, 0.32438E+00_JPRB, 0.32824E+00_JPRB, 0.32975E+00_JPRB, 0.33008E+00_JPRB ,&
&  0.33120E+00_JPRB, 0.33215E+00_JPRB, 0.33340E+00_JPRB, 0.33310E+00_JPRB, 0.32960E+00_JPRB, 0.32989E+00_JPRB, 0.33385E+00_JPRB ,&
&  0.33701E+00_JPRB, 0.33557E+00_JPRB, 0.33192E+00_JPRB, 0.32865E+00_JPRB, 0.32588E+00_JPRB, 0.32369E+00_JPRB, 0.32179E+00_JPRB ,&
&  0.32078E+00_JPRB, 0.31950E+00_JPRB, 0.31673E+00_JPRB, 0.31053E+00_JPRB, 0.30378E+00_JPRB, 0.29915E+00_JPRB, 0.29567E+00_JPRB ,&
&  0.29515E+00_JPRB, 0.29687E+00_JPRB, 0.29864E+00_JPRB, 0.29847E+00_JPRB, 0.29630E+00_JPRB, 0.29262E+00_JPRB, 0.28806E+00_JPRB ,&
&  0.28277E+00_JPRB, 0.28114E+00_JPRB, 0.28130E+00_JPRB, 0.28090E+00_JPRB, 0.28160E+00_JPRB, 0.28378E+00_JPRB, 0.28637E+00_JPRB ,&
&  0.28957E+00_JPRB, 0.28948E+00_JPRB, 0.28773E+00_JPRB, 0.28498E+00_JPRB, 0.28216E+00_JPRB, 0.28198E+00_JPRB, 0.28310E+00_JPRB ,&
&  0.28429E+00_JPRB, 0.28552E+00_JPRB, 0.28718E+00_JPRB, 0.28921E+00_JPRB, 0.29155E+00_JPRB, 0.29442E+00_JPRB, 0.29662E+00_JPRB ,&
&  0.29655E+00_JPRB, 0.29538E+00_JPRB, 0.29668E+00_JPRB, 0.29907E+00_JPRB, 0.30041E+00_JPRB, 0.30117E+00_JPRB, 0.30162E+00_JPRB ,&
&  0.30097E+00_JPRB, 0.30046E+00_JPRB, 0.30206E+00_JPRB, 0.30648E+00_JPRB, 0.31040E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.29557E+00_JPRB, 0.29675E+00_JPRB, 0.29666E+00_JPRB, 0.29700E+00_JPRB, 0.29755E+00_JPRB, 0.29937E+00_JPRB, 0.30360E+00_JPRB ,&
&  0.30788E+00_JPRB, 0.30977E+00_JPRB, 0.30787E+00_JPRB, 0.30524E+00_JPRB, 0.30392E+00_JPRB, 0.30346E+00_JPRB, 0.30355E+00_JPRB ,&
&  0.30367E+00_JPRB, 0.30288E+00_JPRB, 0.30011E+00_JPRB, 0.29609E+00_JPRB, 0.29285E+00_JPRB, 0.29163E+00_JPRB, 0.29163E+00_JPRB ,&
&  0.29509E+00_JPRB, 0.29740E+00_JPRB, 0.29944E+00_JPRB, 0.30161E+00_JPRB, 0.30255E+00_JPRB, 0.30275E+00_JPRB, 0.30473E+00_JPRB ,&
&  0.30571E+00_JPRB, 0.30251E+00_JPRB, 0.29700E+00_JPRB, 0.30042E+00_JPRB, 0.30292E+00_JPRB, 0.30441E+00_JPRB, 0.30364E+00_JPRB ,&
&  0.30422E+00_JPRB, 0.30642E+00_JPRB, 0.30888E+00_JPRB, 0.31083E+00_JPRB, 0.31066E+00_JPRB, 0.30912E+00_JPRB, 0.31016E+00_JPRB ,&
&  0.31205E+00_JPRB, 0.31111E+00_JPRB, 0.30765E+00_JPRB, 0.30442E+00_JPRB, 0.30083E+00_JPRB, 0.29902E+00_JPRB, 0.29865E+00_JPRB ,&
&  0.29775E+00_JPRB, 0.29531E+00_JPRB, 0.29074E+00_JPRB, 0.28612E+00_JPRB, 0.28263E+00_JPRB, 0.27883E+00_JPRB, 0.27659E+00_JPRB ,&
&  0.27649E+00_JPRB, 0.27728E+00_JPRB, 0.27698E+00_JPRB, 0.27461E+00_JPRB, 0.26739E+00_JPRB, 0.25855E+00_JPRB, 0.25096E+00_JPRB ,&
&  0.24950E+00_JPRB, 0.25341E+00_JPRB, 0.26033E+00_JPRB, 0.26359E+00_JPRB, 0.26684E+00_JPRB, 0.26883E+00_JPRB, 0.27015E+00_JPRB ,&
&  0.26725E+00_JPRB, 0.25568E+00_JPRB, 0.25058E+00_JPRB, 0.24704E+00_JPRB, 0.24401E+00_JPRB, 0.24444E+00_JPRB, 0.24675E+00_JPRB ,&
&  0.25110E+00_JPRB, 0.25551E+00_JPRB, 0.25904E+00_JPRB, 0.26135E+00_JPRB, 0.26225E+00_JPRB, 0.26261E+00_JPRB, 0.26199E+00_JPRB ,&
&  0.26276E+00_JPRB, 0.26739E+00_JPRB, 0.27580E+00_JPRB, 0.28190E+00_JPRB, 0.28305E+00_JPRB, 0.28458E+00_JPRB, 0.28433E+00_JPRB ,&
&  0.28416E+00_JPRB, 0.28600E+00_JPRB, 0.28690E+00_JPRB, 0.28946E+00_JPRB, 0.29282E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.28145E+00_JPRB, 0.28247E+00_JPRB, 0.28231E+00_JPRB, 0.28103E+00_JPRB, 0.27948E+00_JPRB, 0.27935E+00_JPRB, 0.28056E+00_JPRB ,&
&  0.28340E+00_JPRB, 0.28537E+00_JPRB, 0.28567E+00_JPRB, 0.28561E+00_JPRB, 0.28478E+00_JPRB, 0.28298E+00_JPRB, 0.28165E+00_JPRB ,&
&  0.28067E+00_JPRB, 0.28025E+00_JPRB, 0.27989E+00_JPRB, 0.27990E+00_JPRB, 0.27991E+00_JPRB, 0.28060E+00_JPRB, 0.28112E+00_JPRB ,&
&  0.28438E+00_JPRB, 0.28397E+00_JPRB, 0.28326E+00_JPRB, 0.28315E+00_JPRB, 0.28317E+00_JPRB, 0.28353E+00_JPRB, 0.28352E+00_JPRB ,&
&  0.27981E+00_JPRB, 0.27237E+00_JPRB, 0.26663E+00_JPRB, 0.27364E+00_JPRB, 0.28373E+00_JPRB, 0.28395E+00_JPRB, 0.28447E+00_JPRB ,&
&  0.28597E+00_JPRB, 0.28881E+00_JPRB, 0.29122E+00_JPRB, 0.29234E+00_JPRB, 0.29157E+00_JPRB, 0.29077E+00_JPRB, 0.29115E+00_JPRB ,&
&  0.29208E+00_JPRB, 0.29024E+00_JPRB, 0.28414E+00_JPRB, 0.27689E+00_JPRB, 0.26958E+00_JPRB, 0.26687E+00_JPRB, 0.26537E+00_JPRB ,&
&  0.26255E+00_JPRB, 0.26076E+00_JPRB, 0.26020E+00_JPRB, 0.25975E+00_JPRB, 0.26021E+00_JPRB, 0.25744E+00_JPRB, 0.25287E+00_JPRB ,&
&  0.25038E+00_JPRB, 0.24577E+00_JPRB, 0.24081E+00_JPRB, 0.23553E+00_JPRB, 0.23116E+00_JPRB, 0.22576E+00_JPRB, 0.22200E+00_JPRB ,&
&  0.22150E+00_JPRB, 0.22649E+00_JPRB, 0.23734E+00_JPRB, 0.24693E+00_JPRB, 0.25003E+00_JPRB, 0.24261E+00_JPRB, 0.23716E+00_JPRB ,&
&  0.22973E+00_JPRB, 0.22279E+00_JPRB, 0.21887E+00_JPRB, 0.21829E+00_JPRB, 0.21870E+00_JPRB, 0.21976E+00_JPRB, 0.22201E+00_JPRB ,&
&  0.22496E+00_JPRB, 0.22846E+00_JPRB, 0.23100E+00_JPRB, 0.23309E+00_JPRB, 0.23459E+00_JPRB, 0.23498E+00_JPRB, 0.23464E+00_JPRB ,&
&  0.23435E+00_JPRB, 0.23585E+00_JPRB, 0.23830E+00_JPRB, 0.24304E+00_JPRB, 0.24566E+00_JPRB, 0.24939E+00_JPRB, 0.25387E+00_JPRB ,&
&  0.26081E+00_JPRB, 0.26837E+00_JPRB, 0.27273E+00_JPRB, 0.27584E+00_JPRB, 0.27904E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.26688E+00_JPRB, 0.27171E+00_JPRB, 0.27368E+00_JPRB, 0.27277E+00_JPRB, 0.27208E+00_JPRB, 0.27228E+00_JPRB, 0.27265E+00_JPRB ,&
&  0.27345E+00_JPRB, 0.27283E+00_JPRB, 0.27126E+00_JPRB, 0.27002E+00_JPRB, 0.26867E+00_JPRB, 0.26694E+00_JPRB, 0.26608E+00_JPRB ,&
&  0.26551E+00_JPRB, 0.26620E+00_JPRB, 0.26665E+00_JPRB, 0.26818E+00_JPRB, 0.26887E+00_JPRB, 0.27046E+00_JPRB, 0.26832E+00_JPRB ,&
&  0.26409E+00_JPRB, 0.26149E+00_JPRB, 0.26400E+00_JPRB, 0.26432E+00_JPRB, 0.26492E+00_JPRB, 0.26562E+00_JPRB, 0.26728E+00_JPRB ,&
&  0.26431E+00_JPRB, 0.26173E+00_JPRB, 0.26007E+00_JPRB, 0.26748E+00_JPRB, 0.27412E+00_JPRB, 0.27196E+00_JPRB, 0.27203E+00_JPRB ,&
&  0.27398E+00_JPRB, 0.27665E+00_JPRB, 0.27808E+00_JPRB, 0.27822E+00_JPRB, 0.27705E+00_JPRB, 0.27696E+00_JPRB, 0.27584E+00_JPRB ,&
&  0.27241E+00_JPRB, 0.26942E+00_JPRB, 0.25882E+00_JPRB, 0.24065E+00_JPRB, 0.23493E+00_JPRB, 0.23268E+00_JPRB, 0.22875E+00_JPRB ,&
&  0.22317E+00_JPRB, 0.22153E+00_JPRB, 0.22179E+00_JPRB, 0.21933E+00_JPRB, 0.21938E+00_JPRB, 0.21909E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21565E+00_JPRB, 0.21447E+00_JPRB, 0.21282E+00_JPRB, 0.21001E+00_JPRB, 0.20781E+00_JPRB, 0.20741E+00_JPRB, 0.20845E+00_JPRB ,&
&  0.21063E+00_JPRB, 0.21357E+00_JPRB, 0.22286E+00_JPRB, 0.22817E+00_JPRB, 0.22633E+00_JPRB, 0.22344E+00_JPRB, 0.21659E+00_JPRB ,&
&  0.20949E+00_JPRB, 0.20744E+00_JPRB, 0.20602E+00_JPRB, 0.20474E+00_JPRB, 0.20519E+00_JPRB, 0.20639E+00_JPRB, 0.20756E+00_JPRB ,&
&  0.20913E+00_JPRB, 0.21114E+00_JPRB, 0.21241E+00_JPRB, 0.21315E+00_JPRB, 0.21386E+00_JPRB, 0.21417E+00_JPRB, 0.21471E+00_JPRB ,&
&  0.21461E+00_JPRB, 0.21614E+00_JPRB, 0.21722E+00_JPRB, 0.21983E+00_JPRB, 0.22150E+00_JPRB, 0.22386E+00_JPRB, 0.22942E+00_JPRB ,&
&  0.23329E+00_JPRB, 0.24567E+00_JPRB, 0.26125E+00_JPRB, 0.26578E+00_JPRB, 0.26163E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.25711E+00_JPRB, 0.25930E+00_JPRB, 0.26075E+00_JPRB, 0.25890E+00_JPRB, 0.25888E+00_JPRB, 0.25900E+00_JPRB, 0.26013E+00_JPRB ,&
&  0.26056E+00_JPRB, 0.25995E+00_JPRB, 0.25966E+00_JPRB, 0.25767E+00_JPRB, 0.25607E+00_JPRB, 0.24989E+00_JPRB, 0.24775E+00_JPRB ,&
&  0.24593E+00_JPRB, 0.24711E+00_JPRB, 0.24785E+00_JPRB, 0.25006E+00_JPRB, 0.24976E+00_JPRB, 0.24798E+00_JPRB, 0.24399E+00_JPRB ,&
&  0.24140E+00_JPRB, 0.24034E+00_JPRB, 0.23897E+00_JPRB, 0.23973E+00_JPRB, 0.24234E+00_JPRB, 0.24481E+00_JPRB, 0.24658E+00_JPRB ,&
&  0.24810E+00_JPRB, 0.25092E+00_JPRB, 0.25357E+00_JPRB, 0.25686E+00_JPRB, 0.26298E+00_JPRB, 0.26278E+00_JPRB, 0.26420E+00_JPRB ,&
&  0.26523E+00_JPRB, 0.26616E+00_JPRB, 0.26660E+00_JPRB, 0.26586E+00_JPRB, 0.26388E+00_JPRB, 0.25746E+00_JPRB, 0.25566E+00_JPRB ,&
&  0.24362E+00_JPRB, 0.23642E+00_JPRB, 0.23020E+00_JPRB, 0.22587E+00_JPRB, 0.21924E+00_JPRB, 0.21577E+00_JPRB, 0.21183E+00_JPRB ,&
&  0.20848E+00_JPRB, 0.20481E+00_JPRB, 0.20216E+00_JPRB, 0.20118E+00_JPRB, 0.20088E+00_JPRB, 0.20054E+00_JPRB, 0.19960E+00_JPRB ,&
&  0.19974E+00_JPRB, 0.19921E+00_JPRB, 0.19881E+00_JPRB, 0.19839E+00_JPRB, 0.19810E+00_JPRB, 0.19817E+00_JPRB, 0.19908E+00_JPRB ,&
&  0.20136E+00_JPRB, 0.20329E+00_JPRB, 0.20545E+00_JPRB, 0.20785E+00_JPRB, 0.20754E+00_JPRB, 0.20555E+00_JPRB, 0.20253E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.19791E+00_JPRB, 0.19596E+00_JPRB, 0.19540E+00_JPRB, 0.19610E+00_JPRB, 0.19694E+00_JPRB, 0.19855E+00_JPRB ,&
&  0.19996E+00_JPRB, 0.20115E+00_JPRB, 0.20197E+00_JPRB, 0.20244E+00_JPRB, 0.20322E+00_JPRB, 0.20389E+00_JPRB, 0.20508E+00_JPRB ,&
&  0.20642E+00_JPRB, 0.20803E+00_JPRB, 0.20941E+00_JPRB, 0.21137E+00_JPRB, 0.21450E+00_JPRB, 0.21890E+00_JPRB, 0.22670E+00_JPRB ,&
&  0.23342E+00_JPRB, 0.25097E+00_JPRB, 0.25321E+00_JPRB, 0.25579E+00_JPRB, 0.25774E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.24661E+00_JPRB, 0.24846E+00_JPRB, 0.24918E+00_JPRB, 0.25003E+00_JPRB, 0.24988E+00_JPRB, 0.24989E+00_JPRB, 0.25055E+00_JPRB ,&
&  0.24640E+00_JPRB, 0.24377E+00_JPRB, 0.23912E+00_JPRB, 0.23510E+00_JPRB, 0.23120E+00_JPRB, 0.22818E+00_JPRB, 0.22643E+00_JPRB ,&
&  0.22545E+00_JPRB, 0.22573E+00_JPRB, 0.22552E+00_JPRB, 0.22589E+00_JPRB, 0.22438E+00_JPRB, 0.22301E+00_JPRB, 0.22180E+00_JPRB ,&
&  0.22068E+00_JPRB, 0.22169E+00_JPRB, 0.22267E+00_JPRB, 0.22647E+00_JPRB, 0.23181E+00_JPRB, 0.23759E+00_JPRB, 0.24200E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.24732E+00_JPRB, 0.25064E+00_JPRB, 0.25217E+00_JPRB, 0.25402E+00_JPRB, 0.25567E+00_JPRB, 0.25606E+00_JPRB ,&
&  0.25736E+00_JPRB, 0.25697E+00_JPRB, 0.25661E+00_JPRB, 0.25632E+00_JPRB, 0.24699E+00_JPRB, 0.24047E+00_JPRB, 0.23360E+00_JPRB ,&
&  0.22721E+00_JPRB, 0.22068E+00_JPRB, 0.21478E+00_JPRB, 0.21171E+00_JPRB, 0.20786E+00_JPRB, 0.20568E+00_JPRB, 0.20381E+00_JPRB ,&
&  0.20236E+00_JPRB, 0.20084E+00_JPRB, 0.19864E+00_JPRB, 0.19651E+00_JPRB, 0.19525E+00_JPRB, 0.19417E+00_JPRB, 0.19275E+00_JPRB ,&
&  0.19266E+00_JPRB, 0.19191E+00_JPRB, 0.19197E+00_JPRB, 0.19159E+00_JPRB, 0.19135E+00_JPRB, 0.19113E+00_JPRB, 0.19111E+00_JPRB ,&
&  0.19155E+00_JPRB, 0.19258E+00_JPRB, 0.19307E+00_JPRB, 0.19373E+00_JPRB, 0.19378E+00_JPRB, 0.19307E+00_JPRB, 0.19249E+00_JPRB ,&
&  0.19170E+00_JPRB, 0.19107E+00_JPRB, 0.19107E+00_JPRB, 0.19088E+00_JPRB, 0.19141E+00_JPRB, 0.19213E+00_JPRB, 0.19282E+00_JPRB ,&
&  0.19385E+00_JPRB, 0.19454E+00_JPRB, 0.19529E+00_JPRB, 0.19633E+00_JPRB, 0.19715E+00_JPRB, 0.19824E+00_JPRB, 0.19948E+00_JPRB ,&
&  0.20033E+00_JPRB, 0.20226E+00_JPRB, 0.20384E+00_JPRB, 0.20586E+00_JPRB, 0.21010E+00_JPRB, 0.21387E+00_JPRB, 0.22339E+00_JPRB ,&
&  0.23105E+00_JPRB, 0.24631E+00_JPRB, 0.24713E+00_JPRB, 0.24405E+00_JPRB, 0.24522E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.24071E+00_JPRB, 0.24209E+00_JPRB, 0.24237E+00_JPRB, 0.24309E+00_JPRB, 0.24314E+00_JPRB, 0.24450E+00_JPRB, 0.24425E+00_JPRB ,&
&  0.24437E+00_JPRB, 0.24082E+00_JPRB, 0.23870E+00_JPRB, 0.23597E+00_JPRB, 0.23360E+00_JPRB, 0.23060E+00_JPRB, 0.22771E+00_JPRB ,&
&  0.22548E+00_JPRB, 0.22345E+00_JPRB, 0.22121E+00_JPRB, 0.21823E+00_JPRB, 0.21518E+00_JPRB, 0.21310E+00_JPRB, 0.21185E+00_JPRB ,&
&  0.21065E+00_JPRB, 0.21205E+00_JPRB, 0.21425E+00_JPRB, 0.21823E+00_JPRB, 0.22330E+00_JPRB, 0.22721E+00_JPRB, 0.23260E+00_JPRB ,&
&  0.23846E+00_JPRB, 0.24040E+00_JPRB, 0.24387E+00_JPRB, 0.24531E+00_JPRB, 0.24508E+00_JPRB, 0.24599E+00_JPRB, 0.24431E+00_JPRB ,&
&  0.24434E+00_JPRB, 0.24021E+00_JPRB, 0.24476E+00_JPRB, 0.24040E+00_JPRB, 0.23269E+00_JPRB, 0.22972E+00_JPRB, 0.22314E+00_JPRB ,&
&  0.21903E+00_JPRB, 0.21427E+00_JPRB, 0.21129E+00_JPRB, 0.20973E+00_JPRB, 0.20593E+00_JPRB, 0.20396E+00_JPRB, 0.20076E+00_JPRB ,&
&  0.19780E+00_JPRB, 0.19700E+00_JPRB, 0.19529E+00_JPRB, 0.19474E+00_JPRB, 0.19390E+00_JPRB, 0.19259E+00_JPRB, 0.19116E+00_JPRB ,&
&  0.19023E+00_JPRB, 0.18945E+00_JPRB, 0.18909E+00_JPRB, 0.18853E+00_JPRB, 0.18771E+00_JPRB, 0.18688E+00_JPRB, 0.18587E+00_JPRB ,&
&  0.18526E+00_JPRB, 0.18503E+00_JPRB, 0.18491E+00_JPRB, 0.18506E+00_JPRB, 0.18518E+00_JPRB, 0.18505E+00_JPRB, 0.18529E+00_JPRB ,&
&  0.18564E+00_JPRB, 0.18636E+00_JPRB, 0.18686E+00_JPRB, 0.18756E+00_JPRB, 0.18811E+00_JPRB, 0.18885E+00_JPRB, 0.18930E+00_JPRB ,&
&  0.18989E+00_JPRB, 0.19087E+00_JPRB, 0.19174E+00_JPRB, 0.19279E+00_JPRB, 0.19392E+00_JPRB, 0.19482E+00_JPRB, 0.19639E+00_JPRB ,&
&  0.19798E+00_JPRB, 0.19954E+00_JPRB, 0.20188E+00_JPRB, 0.20413E+00_JPRB, 0.20704E+00_JPRB, 0.21170E+00_JPRB, 0.21687E+00_JPRB ,&
&  0.22340E+00_JPRB, 0.23275E+00_JPRB, 0.23297E+00_JPRB, 0.23716E+00_JPRB, 0.23916E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.24019E+00_JPRB, 0.24156E+00_JPRB, 0.24267E+00_JPRB, 0.24321E+00_JPRB, 0.24400E+00_JPRB, 0.24400E+00_JPRB, 0.24382E+00_JPRB ,&
&  0.24059E+00_JPRB, 0.23931E+00_JPRB, 0.23734E+00_JPRB, 0.23510E+00_JPRB, 0.23315E+00_JPRB, 0.22995E+00_JPRB, 0.22725E+00_JPRB ,&
&  0.22478E+00_JPRB, 0.22196E+00_JPRB, 0.21935E+00_JPRB, 0.21622E+00_JPRB, 0.21337E+00_JPRB, 0.21071E+00_JPRB, 0.20966E+00_JPRB ,&
&  0.20928E+00_JPRB, 0.21008E+00_JPRB, 0.21225E+00_JPRB, 0.21379E+00_JPRB, 0.21597E+00_JPRB, 0.21870E+00_JPRB, 0.22283E+00_JPRB ,&
&  0.22573E+00_JPRB, 0.22953E+00_JPRB, 0.23193E+00_JPRB, 0.23255E+00_JPRB, 0.23192E+00_JPRB, 0.23163E+00_JPRB, 0.23070E+00_JPRB ,&
&  0.22849E+00_JPRB, 0.22625E+00_JPRB, 0.22437E+00_JPRB, 0.22230E+00_JPRB, 0.22012E+00_JPRB, 0.21767E+00_JPRB, 0.21451E+00_JPRB ,&
&  0.21173E+00_JPRB, 0.20929E+00_JPRB, 0.20691E+00_JPRB, 0.20501E+00_JPRB, 0.20309E+00_JPRB, 0.20132E+00_JPRB, 0.19927E+00_JPRB ,&
&  0.19767E+00_JPRB, 0.19633E+00_JPRB, 0.19461E+00_JPRB, 0.19333E+00_JPRB, 0.19183E+00_JPRB, 0.19059E+00_JPRB, 0.18923E+00_JPRB ,&
&  0.18824E+00_JPRB, 0.18741E+00_JPRB, 0.18676E+00_JPRB, 0.18615E+00_JPRB, 0.18583E+00_JPRB, 0.18554E+00_JPRB, 0.18519E+00_JPRB ,&
&  0.18491E+00_JPRB, 0.18509E+00_JPRB, 0.18527E+00_JPRB, 0.18544E+00_JPRB, 0.18565E+00_JPRB, 0.18529E+00_JPRB, 0.18537E+00_JPRB ,&
&  0.18570E+00_JPRB, 0.18601E+00_JPRB, 0.18591E+00_JPRB, 0.18628E+00_JPRB, 0.18694E+00_JPRB, 0.18774E+00_JPRB, 0.18902E+00_JPRB ,&
&  0.19002E+00_JPRB, 0.19106E+00_JPRB, 0.19206E+00_JPRB, 0.19307E+00_JPRB, 0.19446E+00_JPRB, 0.19596E+00_JPRB, 0.19752E+00_JPRB ,&
&  0.19939E+00_JPRB, 0.20111E+00_JPRB, 0.20289E+00_JPRB, 0.20472E+00_JPRB, 0.20772E+00_JPRB, 0.21036E+00_JPRB, 0.21448E+00_JPRB ,&
&  0.21856E+00_JPRB, 0.21970E+00_JPRB, 0.23415E+00_JPRB, 0.23518E+00_JPRB, 0.23684E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.21885E+00_JPRB, 0.22099E+00_JPRB, 0.22217E+00_JPRB, 0.22330E+00_JPRB, 0.21915E+00_JPRB, 0.23105E+00_JPRB, 0.22865E+00_JPRB ,&
&  0.22816E+00_JPRB, 0.22369E+00_JPRB, 0.22467E+00_JPRB, 0.22429E+00_JPRB, 0.22301E+00_JPRB, 0.22152E+00_JPRB, 0.21973E+00_JPRB ,&
&  0.21792E+00_JPRB, 0.21556E+00_JPRB, 0.21373E+00_JPRB, 0.21220E+00_JPRB, 0.21079E+00_JPRB, 0.20960E+00_JPRB, 0.20839E+00_JPRB ,&
&  0.20842E+00_JPRB, 0.20930E+00_JPRB, 0.21058E+00_JPRB, 0.20958E+00_JPRB, 0.20990E+00_JPRB, 0.21078E+00_JPRB, 0.21204E+00_JPRB ,&
&  0.21304E+00_JPRB, 0.21460E+00_JPRB, 0.21565E+00_JPRB, 0.21603E+00_JPRB, 0.21694E+00_JPRB, 0.21626E+00_JPRB, 0.21577E+00_JPRB ,&
&  0.21496E+00_JPRB, 0.21380E+00_JPRB, 0.21235E+00_JPRB, 0.21109E+00_JPRB, 0.20970E+00_JPRB, 0.20796E+00_JPRB, 0.20616E+00_JPRB ,&
&  0.20456E+00_JPRB, 0.20316E+00_JPRB, 0.20159E+00_JPRB, 0.20030E+00_JPRB, 0.19908E+00_JPRB, 0.19785E+00_JPRB, 0.19670E+00_JPRB ,&
&  0.19577E+00_JPRB, 0.19495E+00_JPRB, 0.19406E+00_JPRB, 0.19318E+00_JPRB, 0.19221E+00_JPRB, 0.19130E+00_JPRB, 0.19042E+00_JPRB ,&
&  0.19009E+00_JPRB, 0.18981E+00_JPRB, 0.18950E+00_JPRB, 0.18917E+00_JPRB, 0.18916E+00_JPRB, 0.18912E+00_JPRB, 0.18902E+00_JPRB ,&
&  0.18888E+00_JPRB, 0.18863E+00_JPRB, 0.18855E+00_JPRB, 0.18847E+00_JPRB, 0.18839E+00_JPRB, 0.18831E+00_JPRB, 0.18842E+00_JPRB ,&
&  0.18861E+00_JPRB, 0.18887E+00_JPRB, 0.18946E+00_JPRB, 0.19006E+00_JPRB, 0.19070E+00_JPRB, 0.19127E+00_JPRB, 0.19212E+00_JPRB ,&
&  0.19306E+00_JPRB, 0.19395E+00_JPRB, 0.19470E+00_JPRB, 0.19531E+00_JPRB, 0.19642E+00_JPRB, 0.19748E+00_JPRB, 0.19852E+00_JPRB ,&
&  0.19965E+00_JPRB, 0.20112E+00_JPRB, 0.20264E+00_JPRB, 0.20421E+00_JPRB, 0.20608E+00_JPRB, 0.20755E+00_JPRB, 0.20906E+00_JPRB ,&
&  0.21085E+00_JPRB, 0.21427E+00_JPRB, 0.21638E+00_JPRB, 0.21768E+00_JPRB, 0.21895E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.20309E+00_JPRB, 0.20322E+00_JPRB, 0.20324E+00_JPRB, 0.20314E+00_JPRB, 0.20280E+00_JPRB, 0.20269E+00_JPRB, 0.20258E+00_JPRB ,&
&  0.20247E+00_JPRB, 0.20218E+00_JPRB, 0.20240E+00_JPRB, 0.20248E+00_JPRB, 0.20249E+00_JPRB, 0.20247E+00_JPRB, 0.20226E+00_JPRB ,&
&  0.20185E+00_JPRB, 0.20134E+00_JPRB, 0.20008E+00_JPRB, 0.19984E+00_JPRB, 0.19982E+00_JPRB, 0.20003E+00_JPRB, 0.20018E+00_JPRB ,&
&  0.20047E+00_JPRB, 0.20076E+00_JPRB, 0.20101E+00_JPRB, 0.20180E+00_JPRB, 0.20213E+00_JPRB, 0.20250E+00_JPRB, 0.20287E+00_JPRB ,&
&  0.20322E+00_JPRB, 0.20364E+00_JPRB, 0.20397E+00_JPRB, 0.20432E+00_JPRB, 0.20467E+00_JPRB, 0.20464E+00_JPRB, 0.20441E+00_JPRB ,&
&  0.20404E+00_JPRB, 0.20345E+00_JPRB, 0.20282E+00_JPRB, 0.20224E+00_JPRB, 0.20171E+00_JPRB, 0.20117E+00_JPRB, 0.20065E+00_JPRB ,&
&  0.20009E+00_JPRB, 0.19947E+00_JPRB, 0.19882E+00_JPRB, 0.19824E+00_JPRB, 0.19764E+00_JPRB, 0.19711E+00_JPRB, 0.19651E+00_JPRB ,&
&  0.19606E+00_JPRB, 0.19559E+00_JPRB, 0.19515E+00_JPRB, 0.19469E+00_JPRB, 0.19429E+00_JPRB, 0.19389E+00_JPRB, 0.19348E+00_JPRB ,&
&  0.19306E+00_JPRB, 0.19286E+00_JPRB, 0.19258E+00_JPRB, 0.19233E+00_JPRB, 0.19210E+00_JPRB, 0.19193E+00_JPRB, 0.19170E+00_JPRB ,&
&  0.19148E+00_JPRB, 0.19126E+00_JPRB, 0.19119E+00_JPRB, 0.19104E+00_JPRB, 0.19101E+00_JPRB, 0.19099E+00_JPRB, 0.19106E+00_JPRB ,&
&  0.19113E+00_JPRB, 0.19124E+00_JPRB, 0.19143E+00_JPRB, 0.19176E+00_JPRB, 0.19205E+00_JPRB, 0.19236E+00_JPRB, 0.19266E+00_JPRB ,&
&  0.19302E+00_JPRB, 0.19338E+00_JPRB, 0.19377E+00_JPRB, 0.19412E+00_JPRB, 0.19459E+00_JPRB, 0.19508E+00_JPRB, 0.19561E+00_JPRB ,&
&  0.19611E+00_JPRB, 0.19666E+00_JPRB, 0.19715E+00_JPRB, 0.19772E+00_JPRB, 0.19821E+00_JPRB, 0.19851E+00_JPRB, 0.19879E+00_JPRB ,&
&  0.19928E+00_JPRB, 0.19975E+00_JPRB, 0.20031E+00_JPRB, 0.20081E+00_JPRB, 0.20170E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB, 0.19499E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB, 0.84925E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.83673E+00_JPRB, 0.83712E+00_JPRB, 0.83744E+00_JPRB, 0.83776E+00_JPRB, 0.83803E+00_JPRB, 0.83830E+00_JPRB, 0.83860E+00_JPRB ,&
&  0.83879E+00_JPRB, 0.83921E+00_JPRB, 0.83977E+00_JPRB, 0.84042E+00_JPRB, 0.84103E+00_JPRB, 0.84164E+00_JPRB, 0.84228E+00_JPRB ,&
&  0.84289E+00_JPRB, 0.84327E+00_JPRB, 0.84367E+00_JPRB, 0.84428E+00_JPRB, 0.84510E+00_JPRB, 0.84608E+00_JPRB, 0.84744E+00_JPRB ,&
&  0.84803E+00_JPRB, 0.84776E+00_JPRB, 0.84754E+00_JPRB, 0.84826E+00_JPRB, 0.84807E+00_JPRB, 0.84797E+00_JPRB, 0.84794E+00_JPRB ,&
&  0.84785E+00_JPRB, 0.84770E+00_JPRB, 0.84740E+00_JPRB, 0.84713E+00_JPRB, 0.84710E+00_JPRB, 0.84733E+00_JPRB, 0.84765E+00_JPRB ,&
&  0.84810E+00_JPRB, 0.84847E+00_JPRB, 0.84937E+00_JPRB, 0.85031E+00_JPRB, 0.85140E+00_JPRB, 0.85217E+00_JPRB, 0.85306E+00_JPRB ,&
&  0.85364E+00_JPRB, 0.85426E+00_JPRB, 0.85525E+00_JPRB, 0.85660E+00_JPRB, 0.85805E+00_JPRB, 0.85951E+00_JPRB, 0.86056E+00_JPRB ,&
&  0.86218E+00_JPRB, 0.86427E+00_JPRB, 0.86670E+00_JPRB, 0.86927E+00_JPRB, 0.87121E+00_JPRB, 0.87317E+00_JPRB, 0.87536E+00_JPRB ,&
&  0.87726E+00_JPRB, 0.87832E+00_JPRB, 0.87957E+00_JPRB, 0.88089E+00_JPRB, 0.88181E+00_JPRB, 0.88098E+00_JPRB, 0.88033E+00_JPRB ,&
&  0.87978E+00_JPRB, 0.87888E+00_JPRB, 0.87637E+00_JPRB, 0.87429E+00_JPRB, 0.87240E+00_JPRB, 0.87042E+00_JPRB, 0.86818E+00_JPRB ,&
&  0.86614E+00_JPRB, 0.86426E+00_JPRB, 0.86219E+00_JPRB, 0.85996E+00_JPRB, 0.85800E+00_JPRB, 0.85612E+00_JPRB, 0.85427E+00_JPRB ,&
&  0.85251E+00_JPRB, 0.85102E+00_JPRB, 0.84958E+00_JPRB, 0.84825E+00_JPRB, 0.84682E+00_JPRB, 0.84545E+00_JPRB, 0.84402E+00_JPRB ,&
&  0.84273E+00_JPRB, 0.84139E+00_JPRB, 0.84015E+00_JPRB, 0.83893E+00_JPRB, 0.83809E+00_JPRB, 0.83754E+00_JPRB, 0.83706E+00_JPRB ,&
&  0.83669E+00_JPRB, 0.83650E+00_JPRB, 0.83654E+00_JPRB, 0.83652E+00_JPRB, 0.83653E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.82883E+00_JPRB, 0.82991E+00_JPRB, 0.83087E+00_JPRB, 0.83171E+00_JPRB, 0.83272E+00_JPRB, 0.83386E+00_JPRB, 0.83485E+00_JPRB ,&
&  0.83566E+00_JPRB, 0.83681E+00_JPRB, 0.83815E+00_JPRB, 0.83947E+00_JPRB, 0.84070E+00_JPRB, 0.84204E+00_JPRB, 0.84335E+00_JPRB ,&
&  0.84455E+00_JPRB, 0.84571E+00_JPRB, 0.84690E+00_JPRB, 0.84795E+00_JPRB, 0.84890E+00_JPRB, 0.84944E+00_JPRB, 0.85191E+00_JPRB ,&
&  0.85383E+00_JPRB, 0.85436E+00_JPRB, 0.85305E+00_JPRB, 0.84549E+00_JPRB, 0.84001E+00_JPRB, 0.83497E+00_JPRB, 0.83072E+00_JPRB ,&
&  0.82630E+00_JPRB, 0.82598E+00_JPRB, 0.82630E+00_JPRB, 0.82669E+00_JPRB, 0.82995E+00_JPRB, 0.82921E+00_JPRB, 0.82808E+00_JPRB ,&
&  0.82550E+00_JPRB, 0.81949E+00_JPRB, 0.82038E+00_JPRB, 0.82152E+00_JPRB, 0.82267E+00_JPRB, 0.82606E+00_JPRB, 0.82880E+00_JPRB ,&
&  0.83205E+00_JPRB, 0.83549E+00_JPRB, 0.84172E+00_JPRB, 0.84856E+00_JPRB, 0.85480E+00_JPRB, 0.86011E+00_JPRB, 0.86499E+00_JPRB ,&
&  0.86931E+00_JPRB, 0.87388E+00_JPRB, 0.87893E+00_JPRB, 0.88482E+00_JPRB, 0.88962E+00_JPRB, 0.89467E+00_JPRB, 0.89885E+00_JPRB ,&
&  0.90115E+00_JPRB, 0.90199E+00_JPRB, 0.90474E+00_JPRB, 0.90867E+00_JPRB, 0.91271E+00_JPRB, 0.91313E+00_JPRB, 0.91208E+00_JPRB ,&
&  0.90964E+00_JPRB, 0.90714E+00_JPRB, 0.90283E+00_JPRB, 0.89902E+00_JPRB, 0.89494E+00_JPRB, 0.89073E+00_JPRB, 0.88593E+00_JPRB ,&
&  0.88121E+00_JPRB, 0.87661E+00_JPRB, 0.87285E+00_JPRB, 0.86906E+00_JPRB, 0.86551E+00_JPRB, 0.86215E+00_JPRB, 0.85989E+00_JPRB ,&
&  0.85735E+00_JPRB, 0.85518E+00_JPRB, 0.85284E+00_JPRB, 0.85050E+00_JPRB, 0.84679E+00_JPRB, 0.84301E+00_JPRB, 0.83889E+00_JPRB ,&
&  0.83543E+00_JPRB, 0.83200E+00_JPRB, 0.82918E+00_JPRB, 0.82681E+00_JPRB, 0.82573E+00_JPRB, 0.82524E+00_JPRB, 0.82509E+00_JPRB ,&
&  0.82506E+00_JPRB, 0.82560E+00_JPRB, 0.82643E+00_JPRB, 0.82714E+00_JPRB, 0.82792E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.83310E+00_JPRB, 0.83463E+00_JPRB, 0.83604E+00_JPRB, 0.83686E+00_JPRB, 0.83834E+00_JPRB, 0.83974E+00_JPRB, 0.84111E+00_JPRB ,&
&  0.84243E+00_JPRB, 0.84328E+00_JPRB, 0.84406E+00_JPRB, 0.84497E+00_JPRB, 0.84584E+00_JPRB, 0.84763E+00_JPRB, 0.84929E+00_JPRB ,&
&  0.85070E+00_JPRB, 0.85155E+00_JPRB, 0.85389E+00_JPRB, 0.85598E+00_JPRB, 0.85695E+00_JPRB, 0.85772E+00_JPRB, 0.85175E+00_JPRB ,&
&  0.84792E+00_JPRB, 0.84522E+00_JPRB, 0.84159E+00_JPRB, 0.83380E+00_JPRB, 0.82987E+00_JPRB, 0.81005E+00_JPRB, 0.81623E+00_JPRB ,&
&  0.84130E+00_JPRB, 0.83928E+00_JPRB, 0.83326E+00_JPRB, 0.82332E+00_JPRB, 0.78861E+00_JPRB, 0.77367E+00_JPRB, 0.77087E+00_JPRB ,&
&  0.75128E+00_JPRB, 0.77021E+00_JPRB, 0.77573E+00_JPRB, 0.77820E+00_JPRB, 0.78265E+00_JPRB, 0.77954E+00_JPRB, 0.78013E+00_JPRB ,&
&  0.79319E+00_JPRB, 0.79393E+00_JPRB, 0.80168E+00_JPRB, 0.81137E+00_JPRB, 0.82202E+00_JPRB, 0.83218E+00_JPRB, 0.84355E+00_JPRB ,&
&  0.85260E+00_JPRB, 0.85923E+00_JPRB, 0.86394E+00_JPRB, 0.86583E+00_JPRB, 0.86816E+00_JPRB, 0.87109E+00_JPRB, 0.87673E+00_JPRB ,&
&  0.88558E+00_JPRB, 0.89496E+00_JPRB, 0.89982E+00_JPRB, 0.89858E+00_JPRB, 0.90060E+00_JPRB, 0.90221E+00_JPRB, 0.90400E+00_JPRB ,&
&  0.90409E+00_JPRB, 0.90287E+00_JPRB, 0.89895E+00_JPRB, 0.89807E+00_JPRB, 0.89749E+00_JPRB, 0.89535E+00_JPRB, 0.89202E+00_JPRB ,&
&  0.88802E+00_JPRB, 0.88231E+00_JPRB, 0.87763E+00_JPRB, 0.87370E+00_JPRB, 0.87042E+00_JPRB, 0.86747E+00_JPRB, 0.86659E+00_JPRB ,&
&  0.86413E+00_JPRB, 0.86220E+00_JPRB, 0.85827E+00_JPRB, 0.85278E+00_JPRB, 0.84619E+00_JPRB, 0.84023E+00_JPRB, 0.83386E+00_JPRB ,&
&  0.82982E+00_JPRB, 0.82720E+00_JPRB, 0.82578E+00_JPRB, 0.82413E+00_JPRB, 0.82412E+00_JPRB, 0.82426E+00_JPRB, 0.82516E+00_JPRB ,&
&  0.82599E+00_JPRB, 0.82743E+00_JPRB, 0.82864E+00_JPRB, 0.83025E+00_JPRB, 0.83131E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.83514E+00_JPRB, 0.83513E+00_JPRB, 0.83555E+00_JPRB, 0.83575E+00_JPRB, 0.83691E+00_JPRB, 0.83778E+00_JPRB, 0.84059E+00_JPRB ,&
&  0.84320E+00_JPRB, 0.84586E+00_JPRB, 0.84844E+00_JPRB, 0.85063E+00_JPRB, 0.85285E+00_JPRB, 0.85391E+00_JPRB, 0.85451E+00_JPRB ,&
&  0.85507E+00_JPRB, 0.85525E+00_JPRB, 0.85670E+00_JPRB, 0.85831E+00_JPRB, 0.85860E+00_JPRB, 0.85973E+00_JPRB, 0.85330E+00_JPRB ,&
&  0.84836E+00_JPRB, 0.84469E+00_JPRB, 0.82359E+00_JPRB, 0.83332E+00_JPRB, 0.83095E+00_JPRB, 0.83099E+00_JPRB, 0.82635E+00_JPRB ,&
&  0.80209E+00_JPRB, 0.79133E+00_JPRB, 0.76577E+00_JPRB, 0.74539E+00_JPRB, 0.74836E+00_JPRB, 0.73504E+00_JPRB, 0.73744E+00_JPRB ,&
&  0.73502E+00_JPRB, 0.72965E+00_JPRB, 0.72726E+00_JPRB, 0.72545E+00_JPRB, 0.72871E+00_JPRB, 0.73097E+00_JPRB, 0.73369E+00_JPRB ,&
&  0.74182E+00_JPRB, 0.74885E+00_JPRB, 0.75926E+00_JPRB, 0.76082E+00_JPRB, 0.77503E+00_JPRB, 0.79107E+00_JPRB, 0.80795E+00_JPRB ,&
&  0.81925E+00_JPRB, 0.81844E+00_JPRB, 0.82378E+00_JPRB, 0.83159E+00_JPRB, 0.83811E+00_JPRB, 0.84534E+00_JPRB, 0.85646E+00_JPRB ,&
&  0.87060E+00_JPRB, 0.88342E+00_JPRB, 0.88792E+00_JPRB, 0.88611E+00_JPRB, 0.88292E+00_JPRB, 0.88035E+00_JPRB, 0.88141E+00_JPRB ,&
&  0.88246E+00_JPRB, 0.88671E+00_JPRB, 0.88863E+00_JPRB, 0.89269E+00_JPRB, 0.89542E+00_JPRB, 0.90087E+00_JPRB, 0.90447E+00_JPRB ,&
&  0.90686E+00_JPRB, 0.90752E+00_JPRB, 0.91165E+00_JPRB, 0.90501E+00_JPRB, 0.90361E+00_JPRB, 0.90075E+00_JPRB, 0.89256E+00_JPRB ,&
&  0.88448E+00_JPRB, 0.87764E+00_JPRB, 0.87030E+00_JPRB, 0.86269E+00_JPRB, 0.85521E+00_JPRB, 0.84924E+00_JPRB, 0.84482E+00_JPRB ,&
&  0.84214E+00_JPRB, 0.83927E+00_JPRB, 0.83726E+00_JPRB, 0.83306E+00_JPRB, 0.83245E+00_JPRB, 0.82932E+00_JPRB, 0.82825E+00_JPRB ,&
&  0.82786E+00_JPRB, 0.82957E+00_JPRB, 0.83015E+00_JPRB, 0.83242E+00_JPRB, 0.83441E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.82431E+00_JPRB, 0.82783E+00_JPRB, 0.83420E+00_JPRB, 0.83955E+00_JPRB, 0.84463E+00_JPRB, 0.84874E+00_JPRB, 0.85077E+00_JPRB ,&
&  0.85241E+00_JPRB, 0.85289E+00_JPRB, 0.85354E+00_JPRB, 0.85441E+00_JPRB, 0.85611E+00_JPRB, 0.85746E+00_JPRB, 0.85937E+00_JPRB ,&
&  0.86011E+00_JPRB, 0.86061E+00_JPRB, 0.86076E+00_JPRB, 0.86085E+00_JPRB, 0.86138E+00_JPRB, 0.86333E+00_JPRB, 0.86194E+00_JPRB ,&
&  0.86200E+00_JPRB, 0.85577E+00_JPRB, 0.84032E+00_JPRB, 0.84527E+00_JPRB, 0.84625E+00_JPRB, 0.84593E+00_JPRB, 0.83536E+00_JPRB ,&
&  0.80757E+00_JPRB, 0.76607E+00_JPRB, 0.79031E+00_JPRB, 0.77652E+00_JPRB, 0.76142E+00_JPRB, 0.74567E+00_JPRB, 0.72157E+00_JPRB ,&
&  0.70686E+00_JPRB, 0.68740E+00_JPRB, 0.67265E+00_JPRB, 0.66714E+00_JPRB, 0.66603E+00_JPRB, 0.67273E+00_JPRB, 0.68229E+00_JPRB ,&
&  0.68645E+00_JPRB, 0.69921E+00_JPRB, 0.70706E+00_JPRB, 0.71796E+00_JPRB, 0.73174E+00_JPRB, 0.74977E+00_JPRB, 0.75732E+00_JPRB ,&
&  0.77035E+00_JPRB, 0.77537E+00_JPRB, 0.78836E+00_JPRB, 0.80620E+00_JPRB, 0.82932E+00_JPRB, 0.84440E+00_JPRB, 0.85465E+00_JPRB ,&
&  0.86698E+00_JPRB, 0.87403E+00_JPRB, 0.87837E+00_JPRB, 0.88065E+00_JPRB, 0.88564E+00_JPRB, 0.89061E+00_JPRB, 0.89593E+00_JPRB ,&
&  0.90068E+00_JPRB, 0.90874E+00_JPRB, 0.91745E+00_JPRB, 0.92765E+00_JPRB, 0.93370E+00_JPRB, 0.94428E+00_JPRB, 0.95654E+00_JPRB ,&
&  0.96783E+00_JPRB, 0.97360E+00_JPRB, 0.98491E+00_JPRB, 0.97508E+00_JPRB, 0.97013E+00_JPRB, 0.95082E+00_JPRB, 0.93602E+00_JPRB ,&
&  0.92484E+00_JPRB, 0.91119E+00_JPRB, 0.89903E+00_JPRB, 0.88784E+00_JPRB, 0.87880E+00_JPRB, 0.87116E+00_JPRB, 0.86452E+00_JPRB ,&
&  0.85745E+00_JPRB, 0.84828E+00_JPRB, 0.84212E+00_JPRB, 0.83511E+00_JPRB, 0.83222E+00_JPRB, 0.82738E+00_JPRB, 0.82513E+00_JPRB ,&
&  0.82359E+00_JPRB, 0.82323E+00_JPRB, 0.82232E+00_JPRB, 0.82379E+00_JPRB, 0.82439E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.82531E+00_JPRB, 0.83146E+00_JPRB, 0.83817E+00_JPRB, 0.84359E+00_JPRB, 0.84563E+00_JPRB, 0.84758E+00_JPRB, 0.84539E+00_JPRB ,&
&  0.84420E+00_JPRB, 0.84229E+00_JPRB, 0.84085E+00_JPRB, 0.84140E+00_JPRB, 0.84303E+00_JPRB, 0.84620E+00_JPRB, 0.84988E+00_JPRB ,&
&  0.85329E+00_JPRB, 0.85743E+00_JPRB, 0.85930E+00_JPRB, 0.86159E+00_JPRB, 0.86431E+00_JPRB, 0.86966E+00_JPRB, 0.87187E+00_JPRB ,&
&  0.87460E+00_JPRB, 0.86808E+00_JPRB, 0.85384E+00_JPRB, 0.85012E+00_JPRB, 0.85701E+00_JPRB, 0.84056E+00_JPRB, 0.83984E+00_JPRB ,&
&  0.84909E+00_JPRB, 0.85024E+00_JPRB, 0.83484E+00_JPRB, 0.82450E+00_JPRB, 0.79573E+00_JPRB, 0.71392E+00_JPRB, 0.71371E+00_JPRB ,&
&  0.69854E+00_JPRB, 0.67087E+00_JPRB, 0.65608E+00_JPRB, 0.64310E+00_JPRB, 0.63700E+00_JPRB, 0.64711E+00_JPRB, 0.66331E+00_JPRB ,&
&  0.66689E+00_JPRB, 0.68512E+00_JPRB, 0.69208E+00_JPRB, 0.70404E+00_JPRB, 0.71703E+00_JPRB, 0.73619E+00_JPRB, 0.74680E+00_JPRB ,&
&  0.75847E+00_JPRB, 0.77074E+00_JPRB, 0.79039E+00_JPRB, 0.81613E+00_JPRB, 0.83698E+00_JPRB, 0.84890E+00_JPRB, 0.84903E+00_JPRB ,&
&  0.86339E+00_JPRB, 0.85964E+00_JPRB, 0.86761E+00_JPRB, 0.87650E+00_JPRB, 0.88357E+00_JPRB, 0.89385E+00_JPRB, 0.89919E+00_JPRB ,&
&  0.91345E+00_JPRB, 0.93762E+00_JPRB, 0.96187E+00_JPRB, 0.97558E+00_JPRB, 0.98624E+00_JPRB, 0.99820E+00_JPRB, 0.10154E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10508E+01_JPRB, 0.10656E+01_JPRB, 0.10455E+01_JPRB, 0.10349E+01_JPRB, 0.10094E+01_JPRB, 0.99038E+00_JPRB ,&
&  0.96751E+00_JPRB, 0.95195E+00_JPRB, 0.92679E+00_JPRB, 0.91257E+00_JPRB, 0.89816E+00_JPRB, 0.88713E+00_JPRB, 0.87526E+00_JPRB ,&
&  0.86442E+00_JPRB, 0.85379E+00_JPRB, 0.84697E+00_JPRB, 0.84281E+00_JPRB, 0.83938E+00_JPRB, 0.83324E+00_JPRB, 0.82874E+00_JPRB ,&
&  0.82216E+00_JPRB, 0.81828E+00_JPRB, 0.81497E+00_JPRB, 0.81596E+00_JPRB, 0.81867E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.81384E+00_JPRB, 0.82159E+00_JPRB, 0.82608E+00_JPRB, 0.83341E+00_JPRB, 0.83225E+00_JPRB, 0.83447E+00_JPRB, 0.82726E+00_JPRB ,&
&  0.82216E+00_JPRB, 0.81917E+00_JPRB, 0.81772E+00_JPRB, 0.81954E+00_JPRB, 0.82294E+00_JPRB, 0.82789E+00_JPRB, 0.83447E+00_JPRB ,&
&  0.84147E+00_JPRB, 0.84207E+00_JPRB, 0.84784E+00_JPRB, 0.85759E+00_JPRB, 0.86229E+00_JPRB, 0.86908E+00_JPRB, 0.87318E+00_JPRB ,&
&  0.87440E+00_JPRB, 0.87207E+00_JPRB, 0.86960E+00_JPRB, 0.86005E+00_JPRB, 0.85695E+00_JPRB, 0.86191E+00_JPRB, 0.86588E+00_JPRB ,&
&  0.84303E+00_JPRB, 0.83351E+00_JPRB, 0.85518E+00_JPRB, 0.86159E+00_JPRB, 0.81107E+00_JPRB, 0.76547E+00_JPRB, 0.74420E+00_JPRB ,&
&  0.71449E+00_JPRB, 0.67175E+00_JPRB, 0.65606E+00_JPRB, 0.64552E+00_JPRB, 0.64849E+00_JPRB, 0.65276E+00_JPRB, 0.66471E+00_JPRB ,&
&  0.67245E+00_JPRB, 0.68265E+00_JPRB, 0.69777E+00_JPRB, 0.71749E+00_JPRB, 0.73047E+00_JPRB, 0.74573E+00_JPRB, 0.75769E+00_JPRB ,&
&  0.77117E+00_JPRB, 0.78944E+00_JPRB, 0.81510E+00_JPRB, 0.84717E+00_JPRB, 0.87108E+00_JPRB, 0.88963E+00_JPRB, 0.89855E+00_JPRB ,&
&  0.93461E+00_JPRB, 0.92175E+00_JPRB, 0.93131E+00_JPRB, 0.93705E+00_JPRB, 0.94142E+00_JPRB, 0.95101E+00_JPRB, 0.95951E+00_JPRB ,&
&  0.98297E+00_JPRB, 0.10181E+01_JPRB, 0.10288E+01_JPRB, 0.10342E+01_JPRB, 0.10374E+01_JPRB, 0.10446E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10838E+01_JPRB, 0.11178E+01_JPRB, 0.12022E+01_JPRB, 0.11592E+01_JPRB, 0.11252E+01_JPRB, 0.10882E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10137E+01_JPRB, 0.98163E+00_JPRB, 0.94997E+00_JPRB, 0.92174E+00_JPRB, 0.90205E+00_JPRB, 0.88285E+00_JPRB, 0.87564E+00_JPRB ,&
&  0.86957E+00_JPRB, 0.86246E+00_JPRB, 0.85797E+00_JPRB, 0.85396E+00_JPRB, 0.84510E+00_JPRB, 0.83532E+00_JPRB, 0.82812E+00_JPRB ,&
&  0.81689E+00_JPRB, 0.81066E+00_JPRB, 0.80544E+00_JPRB, 0.80592E+00_JPRB, 0.80814E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.80148E+00_JPRB, 0.81723E+00_JPRB, 0.82836E+00_JPRB, 0.83432E+00_JPRB, 0.82875E+00_JPRB, 0.83017E+00_JPRB, 0.82215E+00_JPRB ,&
&  0.81738E+00_JPRB, 0.81762E+00_JPRB, 0.81602E+00_JPRB, 0.81679E+00_JPRB, 0.81813E+00_JPRB, 0.81921E+00_JPRB, 0.82191E+00_JPRB ,&
&  0.82664E+00_JPRB, 0.83297E+00_JPRB, 0.83871E+00_JPRB, 0.84627E+00_JPRB, 0.85564E+00_JPRB, 0.86624E+00_JPRB, 0.87502E+00_JPRB ,&
&  0.87810E+00_JPRB, 0.87619E+00_JPRB, 0.88106E+00_JPRB, 0.87702E+00_JPRB, 0.87256E+00_JPRB, 0.86621E+00_JPRB, 0.86300E+00_JPRB ,&
&  0.87220E+00_JPRB, 0.88438E+00_JPRB, 0.87210E+00_JPRB, 0.86475E+00_JPRB, 0.85911E+00_JPRB, 0.81661E+00_JPRB, 0.77463E+00_JPRB ,&
&  0.73077E+00_JPRB, 0.69228E+00_JPRB, 0.67452E+00_JPRB, 0.66563E+00_JPRB, 0.67270E+00_JPRB, 0.68879E+00_JPRB, 0.70307E+00_JPRB ,&
&  0.69731E+00_JPRB, 0.71936E+00_JPRB, 0.74216E+00_JPRB, 0.75680E+00_JPRB, 0.76836E+00_JPRB, 0.78037E+00_JPRB, 0.79253E+00_JPRB ,&
&  0.81163E+00_JPRB, 0.84250E+00_JPRB, 0.89850E+00_JPRB, 0.96285E+00_JPRB, 0.10129E+01_JPRB, 0.10294E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.12267E+01_JPRB, 0.11480E+01_JPRB, 0.11111E+01_JPRB, 0.10986E+01_JPRB, 0.10824E+01_JPRB, 0.10783E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.11348E+01_JPRB, 0.11364E+01_JPRB, 0.11286E+01_JPRB, 0.11266E+01_JPRB, 0.11325E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.12266E+01_JPRB, 0.13713E+01_JPRB, 0.12208E+01_JPRB, 0.11407E+01_JPRB, 0.11001E+01_JPRB, 0.10599E+01_JPRB ,&
&  0.10253E+01_JPRB, 0.99570E+00_JPRB, 0.96494E+00_JPRB, 0.92865E+00_JPRB, 0.89960E+00_JPRB, 0.87173E+00_JPRB, 0.86181E+00_JPRB ,&
&  0.85669E+00_JPRB, 0.85101E+00_JPRB, 0.85068E+00_JPRB, 0.84541E+00_JPRB, 0.83707E+00_JPRB, 0.83251E+00_JPRB, 0.82536E+00_JPRB ,&
&  0.81197E+00_JPRB, 0.80006E+00_JPRB, 0.79193E+00_JPRB, 0.79036E+00_JPRB, 0.79164E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.81873E+00_JPRB, 0.82819E+00_JPRB, 0.83843E+00_JPRB, 0.85967E+00_JPRB, 0.86492E+00_JPRB, 0.85936E+00_JPRB, 0.83963E+00_JPRB ,&
&  0.82641E+00_JPRB, 0.82149E+00_JPRB, 0.82461E+00_JPRB, 0.82892E+00_JPRB, 0.83124E+00_JPRB, 0.83165E+00_JPRB, 0.82975E+00_JPRB ,&
&  0.82632E+00_JPRB, 0.82692E+00_JPRB, 0.83148E+00_JPRB, 0.83849E+00_JPRB, 0.84768E+00_JPRB, 0.85785E+00_JPRB, 0.86957E+00_JPRB ,&
&  0.87896E+00_JPRB, 0.88157E+00_JPRB, 0.88387E+00_JPRB, 0.88284E+00_JPRB, 0.88552E+00_JPRB, 0.89132E+00_JPRB, 0.88916E+00_JPRB ,&
&  0.87822E+00_JPRB, 0.88753E+00_JPRB, 0.89874E+00_JPRB, 0.88285E+00_JPRB, 0.87420E+00_JPRB, 0.84260E+00_JPRB, 0.80129E+00_JPRB ,&
&  0.76557E+00_JPRB, 0.72998E+00_JPRB, 0.72034E+00_JPRB, 0.72239E+00_JPRB, 0.73131E+00_JPRB, 0.72964E+00_JPRB, 0.75259E+00_JPRB ,&
&  0.77134E+00_JPRB, 0.78444E+00_JPRB, 0.79491E+00_JPRB, 0.80217E+00_JPRB, 0.81035E+00_JPRB, 0.82719E+00_JPRB, 0.86377E+00_JPRB ,&
&  0.93035E+00_JPRB, 0.10166E+01_JPRB, 0.11133E+01_JPRB, 0.10884E+01_JPRB, 0.11045E+01_JPRB, 0.11454E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.12604E+01_JPRB, 0.12616E+01_JPRB, 0.13156E+01_JPRB, 0.13496E+01_JPRB, 0.13826E+01_JPRB, 0.13723E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.13095E+01_JPRB, 0.12822E+01_JPRB, 0.12762E+01_JPRB, 0.12913E+01_JPRB, 0.13014E+01_JPRB, 0.12892E+01_JPRB ,&
&  0.12478E+01_JPRB, 0.11816E+01_JPRB, 0.11095E+01_JPRB, 0.10441E+01_JPRB, 0.10278E+01_JPRB, 0.10311E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.98909E+00_JPRB, 0.96250E+00_JPRB, 0.92787E+00_JPRB, 0.89250E+00_JPRB, 0.85521E+00_JPRB, 0.83960E+00_JPRB ,&
&  0.83198E+00_JPRB, 0.82791E+00_JPRB, 0.83274E+00_JPRB, 0.81813E+00_JPRB, 0.81550E+00_JPRB, 0.81471E+00_JPRB, 0.80003E+00_JPRB ,&
&  0.78856E+00_JPRB, 0.78399E+00_JPRB, 0.77608E+00_JPRB, 0.78156E+00_JPRB, 0.79419E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.84788E+00_JPRB, 0.87898E+00_JPRB, 0.90766E+00_JPRB, 0.91898E+00_JPRB, 0.92487E+00_JPRB, 0.92283E+00_JPRB, 0.90242E+00_JPRB ,&
&  0.89348E+00_JPRB, 0.86543E+00_JPRB, 0.84381E+00_JPRB, 0.82315E+00_JPRB, 0.80844E+00_JPRB, 0.80139E+00_JPRB, 0.80545E+00_JPRB ,&
&  0.81303E+00_JPRB, 0.82026E+00_JPRB, 0.82626E+00_JPRB, 0.83104E+00_JPRB, 0.83791E+00_JPRB, 0.84782E+00_JPRB, 0.86098E+00_JPRB ,&
&  0.87492E+00_JPRB, 0.88479E+00_JPRB, 0.89295E+00_JPRB, 0.89893E+00_JPRB, 0.90341E+00_JPRB, 0.90450E+00_JPRB, 0.89788E+00_JPRB ,&
&  0.89888E+00_JPRB, 0.89836E+00_JPRB, 0.90222E+00_JPRB, 0.89360E+00_JPRB, 0.88674E+00_JPRB, 0.86194E+00_JPRB, 0.81642E+00_JPRB ,&
&  0.79387E+00_JPRB, 0.78103E+00_JPRB, 0.78654E+00_JPRB, 0.78646E+00_JPRB, 0.79940E+00_JPRB, 0.81604E+00_JPRB, 0.82237E+00_JPRB ,&
&  0.82258E+00_JPRB, 0.82453E+00_JPRB, 0.84047E+00_JPRB, 0.85663E+00_JPRB, 0.87300E+00_JPRB, 0.90171E+00_JPRB, 0.93804E+00_JPRB ,&
&  0.99695E+00_JPRB, 0.10804E+01_JPRB, 0.12046E+01_JPRB, 0.12365E+01_JPRB, 0.13269E+01_JPRB, 0.15473E+01_JPRB, 0.14809E+01_JPRB ,&
&  0.15039E+01_JPRB, 0.15375E+01_JPRB, 0.16083E+01_JPRB, 0.17168E+01_JPRB, 0.16970E+01_JPRB, 0.16745E+01_JPRB, 0.16487E+01_JPRB ,&
&  0.15702E+01_JPRB, 0.15361E+01_JPRB, 0.15021E+01_JPRB, 0.15235E+01_JPRB, 0.15019E+01_JPRB, 0.14743E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.13655E+01_JPRB, 0.12662E+01_JPRB, 0.11748E+01_JPRB, 0.10938E+01_JPRB, 0.10540E+01_JPRB, 0.10324E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.98794E+00_JPRB, 0.97416E+00_JPRB, 0.95214E+00_JPRB, 0.91138E+00_JPRB, 0.86533E+00_JPRB, 0.85077E+00_JPRB ,&
&  0.83937E+00_JPRB, 0.82860E+00_JPRB, 0.82183E+00_JPRB, 0.81375E+00_JPRB, 0.81294E+00_JPRB, 0.80498E+00_JPRB, 0.79356E+00_JPRB ,&
&  0.79536E+00_JPRB, 0.78805E+00_JPRB, 0.78178E+00_JPRB, 0.78789E+00_JPRB, 0.81785E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.90662E+00_JPRB, 0.94532E+00_JPRB, 0.96112E+00_JPRB, 0.96974E+00_JPRB, 0.97808E+00_JPRB, 0.97599E+00_JPRB, 0.93714E+00_JPRB ,&
&  0.92116E+00_JPRB, 0.86345E+00_JPRB, 0.83384E+00_JPRB, 0.81513E+00_JPRB, 0.80660E+00_JPRB, 0.79678E+00_JPRB, 0.78694E+00_JPRB ,&
&  0.79189E+00_JPRB, 0.80786E+00_JPRB, 0.82452E+00_JPRB, 0.83401E+00_JPRB, 0.84258E+00_JPRB, 0.84964E+00_JPRB, 0.85786E+00_JPRB ,&
&  0.86993E+00_JPRB, 0.88369E+00_JPRB, 0.89926E+00_JPRB, 0.90706E+00_JPRB, 0.91097E+00_JPRB, 0.91267E+00_JPRB, 0.91119E+00_JPRB ,&
&  0.90941E+00_JPRB, 0.89902E+00_JPRB, 0.90448E+00_JPRB, 0.90222E+00_JPRB, 0.89035E+00_JPRB, 0.88420E+00_JPRB, 0.85332E+00_JPRB ,&
&  0.82515E+00_JPRB, 0.84303E+00_JPRB, 0.84638E+00_JPRB, 0.86174E+00_JPRB, 0.87147E+00_JPRB, 0.87717E+00_JPRB, 0.87987E+00_JPRB ,&
&  0.88004E+00_JPRB, 0.88068E+00_JPRB, 0.88804E+00_JPRB, 0.89934E+00_JPRB, 0.91513E+00_JPRB, 0.93633E+00_JPRB, 0.95761E+00_JPRB ,&
&  0.97888E+00_JPRB, 0.95947E+00_JPRB, 0.10427E+01_JPRB, 0.12447E+01_JPRB, 0.16461E+01_JPRB, 0.17216E+01_JPRB, 0.17884E+01_JPRB ,&
&  0.18979E+01_JPRB, 0.19018E+01_JPRB, 0.19483E+01_JPRB, 0.19840E+01_JPRB, 0.19741E+01_JPRB, 0.19469E+01_JPRB, 0.19417E+01_JPRB ,&
&  0.18687E+01_JPRB, 0.18732E+01_JPRB, 0.18281E+01_JPRB, 0.18027E+01_JPRB, 0.17395E+01_JPRB, 0.16940E+01_JPRB, 0.16273E+01_JPRB ,&
&  0.15384E+01_JPRB, 0.14522E+01_JPRB, 0.13429E+01_JPRB, 0.12285E+01_JPRB, 0.11391E+01_JPRB, 0.10956E+01_JPRB, 0.10573E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10175E+01_JPRB, 0.99840E+00_JPRB, 0.98615E+00_JPRB, 0.95767E+00_JPRB, 0.92712E+00_JPRB, 0.92298E+00_JPRB ,&
&  0.89457E+00_JPRB, 0.87361E+00_JPRB, 0.85510E+00_JPRB, 0.84579E+00_JPRB, 0.85198E+00_JPRB, 0.85397E+00_JPRB, 0.85221E+00_JPRB ,&
&  0.86213E+00_JPRB, 0.85403E+00_JPRB, 0.85352E+00_JPRB, 0.86308E+00_JPRB, 0.87953E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.96198E+00_JPRB, 0.99073E+00_JPRB, 0.10031E+01_JPRB, 0.10161E+01_JPRB, 0.10331E+01_JPRB, 0.10287E+01_JPRB, 0.10149E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.98327E+00_JPRB, 0.10135E+01_JPRB, 0.88528E+00_JPRB, 0.83473E+00_JPRB, 0.82323E+00_JPRB, 0.80930E+00_JPRB ,&
&  0.81049E+00_JPRB, 0.82123E+00_JPRB, 0.83317E+00_JPRB, 0.84494E+00_JPRB, 0.85846E+00_JPRB, 0.86226E+00_JPRB, 0.86593E+00_JPRB ,&
&  0.87381E+00_JPRB, 0.88831E+00_JPRB, 0.91263E+00_JPRB, 0.92548E+00_JPRB, 0.94011E+00_JPRB, 0.95331E+00_JPRB, 0.94850E+00_JPRB ,&
&  0.93311E+00_JPRB, 0.93400E+00_JPRB, 0.93333E+00_JPRB, 0.91624E+00_JPRB, 0.89892E+00_JPRB, 0.89355E+00_JPRB, 0.90955E+00_JPRB ,&
&  0.90927E+00_JPRB, 0.89741E+00_JPRB, 0.91610E+00_JPRB, 0.92312E+00_JPRB, 0.92341E+00_JPRB, 0.92034E+00_JPRB, 0.91723E+00_JPRB ,&
&  0.91593E+00_JPRB, 0.91324E+00_JPRB, 0.91744E+00_JPRB, 0.92397E+00_JPRB, 0.93788E+00_JPRB, 0.96061E+00_JPRB, 0.97440E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.10077E+01_JPRB, 0.11864E+01_JPRB, 0.13896E+01_JPRB, 0.17299E+01_JPRB, 0.19390E+01_JPRB, 0.20694E+01_JPRB ,&
&  0.23285E+01_JPRB, 0.22649E+01_JPRB, 0.23600E+01_JPRB, 0.23357E+01_JPRB, 0.22957E+01_JPRB, 0.22384E+01_JPRB, 0.21638E+01_JPRB ,&
&  0.21325E+01_JPRB, 0.20797E+01_JPRB, 0.21509E+01_JPRB, 0.20893E+01_JPRB, 0.19700E+01_JPRB, 0.18998E+01_JPRB, 0.18343E+01_JPRB ,&
&  0.17806E+01_JPRB, 0.17223E+01_JPRB, 0.16290E+01_JPRB, 0.14145E+01_JPRB, 0.13126E+01_JPRB, 0.12344E+01_JPRB, 0.11306E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.10322E+01_JPRB, 0.99252E+00_JPRB, 0.98635E+00_JPRB, 0.96381E+00_JPRB, 0.94321E+00_JPRB, 0.94316E+00_JPRB ,&
&  0.92402E+00_JPRB, 0.91151E+00_JPRB, 0.90450E+00_JPRB, 0.91228E+00_JPRB, 0.92045E+00_JPRB, 0.92304E+00_JPRB, 0.92530E+00_JPRB ,&
&  0.91260E+00_JPRB, 0.90068E+00_JPRB, 0.91290E+00_JPRB, 0.91709E+00_JPRB, 0.93476E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.99659E+00_JPRB, 0.10266E+01_JPRB, 0.10487E+01_JPRB, 0.10684E+01_JPRB, 0.10932E+01_JPRB, 0.11140E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.10996E+01_JPRB, 0.10968E+01_JPRB, 0.10770E+01_JPRB, 0.10402E+01_JPRB, 0.95020E+00_JPRB, 0.84296E+00_JPRB ,&
&  0.81504E+00_JPRB, 0.82973E+00_JPRB, 0.85012E+00_JPRB, 0.86027E+00_JPRB, 0.86541E+00_JPRB, 0.87240E+00_JPRB, 0.87406E+00_JPRB ,&
&  0.88261E+00_JPRB, 0.89626E+00_JPRB, 0.92225E+00_JPRB, 0.96129E+00_JPRB, 0.99879E+00_JPRB, 0.10066E+01_JPRB, 0.10033E+01_JPRB ,&
&  0.98479E+00_JPRB, 0.97531E+00_JPRB, 0.95383E+00_JPRB, 0.92784E+00_JPRB, 0.92826E+00_JPRB, 0.93503E+00_JPRB, 0.96057E+00_JPRB ,&
&  0.98560E+00_JPRB, 0.99499E+00_JPRB, 0.99367E+00_JPRB, 0.99264E+00_JPRB, 0.98853E+00_JPRB, 0.97811E+00_JPRB, 0.96473E+00_JPRB ,&
&  0.95336E+00_JPRB, 0.94546E+00_JPRB, 0.94111E+00_JPRB, 0.94511E+00_JPRB, 0.94686E+00_JPRB, 0.10033E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.11095E+01_JPRB, 0.12453E+01_JPRB, 0.14700E+01_JPRB, 0.15568E+01_JPRB, 0.18633E+01_JPRB, 0.20726E+01_JPRB, 0.22878E+01_JPRB ,&
&  0.24480E+01_JPRB, 0.23931E+01_JPRB, 0.26006E+01_JPRB, 0.26550E+01_JPRB, 0.26680E+01_JPRB, 0.25960E+01_JPRB, 0.24426E+01_JPRB ,&
&  0.23561E+01_JPRB, 0.24161E+01_JPRB, 0.24606E+01_JPRB, 0.24144E+01_JPRB, 0.22928E+01_JPRB, 0.22239E+01_JPRB, 0.21552E+01_JPRB ,&
&  0.21407E+01_JPRB, 0.21272E+01_JPRB, 0.20252E+01_JPRB, 0.18466E+01_JPRB, 0.16275E+01_JPRB, 0.14351E+01_JPRB, 0.12797E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.10323E+01_JPRB, 0.98739E+00_JPRB, 0.97705E+00_JPRB, 0.97542E+00_JPRB, 0.95025E+00_JPRB, 0.94901E+00_JPRB ,&
&  0.95970E+00_JPRB, 0.96175E+00_JPRB, 0.96746E+00_JPRB, 0.97827E+00_JPRB, 0.10122E+01_JPRB, 0.10263E+01_JPRB, 0.97228E+00_JPRB ,&
&  0.93510E+00_JPRB, 0.93027E+00_JPRB, 0.93400E+00_JPRB, 0.94096E+00_JPRB, 0.96549E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.10464E+01_JPRB, 0.10823E+01_JPRB, 0.11069E+01_JPRB, 0.11245E+01_JPRB, 0.11486E+01_JPRB, 0.11845E+01_JPRB, 0.11881E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.11650E+01_JPRB, 0.11591E+01_JPRB, 0.11461E+01_JPRB, 0.11174E+01_JPRB, 0.10749E+01_JPRB, 0.98315E+00_JPRB ,&
&  0.87797E+00_JPRB, 0.89948E+00_JPRB, 0.87740E+00_JPRB, 0.89141E+00_JPRB, 0.89854E+00_JPRB, 0.88905E+00_JPRB, 0.87843E+00_JPRB ,&
&  0.89200E+00_JPRB, 0.90729E+00_JPRB, 0.94106E+00_JPRB, 0.98179E+00_JPRB, 0.97924E+00_JPRB, 0.98445E+00_JPRB, 0.10168E+01_JPRB ,&
&  0.10133E+01_JPRB, 0.98730E+00_JPRB, 0.96044E+00_JPRB, 0.95337E+00_JPRB, 0.97555E+00_JPRB, 0.10136E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10591E+01_JPRB, 0.10493E+01_JPRB, 0.10407E+01_JPRB, 0.10324E+01_JPRB, 0.10189E+01_JPRB, 0.99978E+00_JPRB ,&
&  0.97961E+00_JPRB, 0.96688E+00_JPRB, 0.96142E+00_JPRB, 0.97075E+00_JPRB, 0.10002E+01_JPRB, 0.11068E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.14733E+01_JPRB, 0.16631E+01_JPRB, 0.18980E+01_JPRB, 0.21621E+01_JPRB, 0.23860E+01_JPRB, 0.25522E+01_JPRB ,&
&  0.27293E+01_JPRB, 0.27178E+01_JPRB, 0.30292E+01_JPRB, 0.30278E+01_JPRB, 0.29080E+01_JPRB, 0.28608E+01_JPRB, 0.26801E+01_JPRB ,&
&  0.25209E+01_JPRB, 0.23767E+01_JPRB, 0.24853E+01_JPRB, 0.25165E+01_JPRB, 0.25138E+01_JPRB, 0.25314E+01_JPRB, 0.24943E+01_JPRB ,&
&  0.24724E+01_JPRB, 0.24236E+01_JPRB, 0.21194E+01_JPRB, 0.18181E+01_JPRB, 0.14968E+01_JPRB, 0.14603E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.11736E+01_JPRB, 0.10196E+01_JPRB, 0.10079E+01_JPRB, 0.99874E+00_JPRB, 0.10166E+01_JPRB, 0.10098E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.10663E+01_JPRB, 0.10712E+01_JPRB, 0.11003E+01_JPRB, 0.11445E+01_JPRB, 0.11425E+01_JPRB, 0.10700E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.10037E+01_JPRB, 0.97395E+00_JPRB, 0.97472E+00_JPRB, 0.10024E+01_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.11174E+01_JPRB, 0.11458E+01_JPRB, 0.11652E+01_JPRB, 0.11852E+01_JPRB, 0.12134E+01_JPRB, 0.12447E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.12161E+01_JPRB, 0.12058E+01_JPRB, 0.11873E+01_JPRB, 0.11630E+01_JPRB, 0.11189E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.95237E+00_JPRB, 0.95499E+00_JPRB, 0.88033E+00_JPRB, 0.89192E+00_JPRB, 0.99478E+00_JPRB, 0.95230E+00_JPRB, 0.93893E+00_JPRB ,&
&  0.92174E+00_JPRB, 0.93971E+00_JPRB, 0.96450E+00_JPRB, 0.98426E+00_JPRB, 0.95697E+00_JPRB, 0.98048E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.10309E+01_JPRB, 0.10340E+01_JPRB, 0.10419E+01_JPRB, 0.10667E+01_JPRB, 0.11179E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.11489E+01_JPRB, 0.11362E+01_JPRB, 0.11168E+01_JPRB, 0.11007E+01_JPRB, 0.10845E+01_JPRB, 0.10647E+01_JPRB, 0.10421E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.10026E+01_JPRB, 0.10022E+01_JPRB, 0.10279E+01_JPRB, 0.10553E+01_JPRB, 0.11996E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.18052E+01_JPRB, 0.19359E+01_JPRB, 0.21653E+01_JPRB, 0.25777E+01_JPRB, 0.28161E+01_JPRB, 0.29543E+01_JPRB, 0.29959E+01_JPRB ,&
&  0.30407E+01_JPRB, 0.30389E+01_JPRB, 0.30080E+01_JPRB, 0.30366E+01_JPRB, 0.29340E+01_JPRB, 0.29548E+01_JPRB, 0.28098E+01_JPRB ,&
&  0.25577E+01_JPRB, 0.24928E+01_JPRB, 0.25690E+01_JPRB, 0.25861E+01_JPRB, 0.25649E+01_JPRB, 0.26072E+01_JPRB, 0.26357E+01_JPRB ,&
&  0.25665E+01_JPRB, 0.22779E+01_JPRB, 0.17552E+01_JPRB, 0.15886E+01_JPRB, 0.12367E+01_JPRB, 0.12197E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.13466E+01_JPRB, 0.12180E+01_JPRB, 0.11196E+01_JPRB, 0.10771E+01_JPRB, 0.10874E+01_JPRB, 0.12383E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.12085E+01_JPRB, 0.12562E+01_JPRB, 0.13204E+01_JPRB, 0.13640E+01_JPRB, 0.13536E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.11577E+01_JPRB, 0.10957E+01_JPRB, 0.10746E+01_JPRB, 0.10867E+01_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.12455E+01_JPRB, 0.12468E+01_JPRB, 0.12483E+01_JPRB, 0.12549E+01_JPRB, 0.12831E+01_JPRB, 0.13023E+01_JPRB, 0.12894E+01_JPRB ,&
&  0.12714E+01_JPRB, 0.12592E+01_JPRB, 0.12433E+01_JPRB, 0.12215E+01_JPRB, 0.11823E+01_JPRB, 0.11231E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.11286E+01_JPRB, 0.10911E+01_JPRB, 0.99165E+00_JPRB, 0.91030E+00_JPRB, 0.97312E+00_JPRB, 0.95909E+00_JPRB, 0.98252E+00_JPRB ,&
&  0.96690E+00_JPRB, 0.10122E+01_JPRB, 0.10293E+01_JPRB, 0.10102E+01_JPRB, 0.98554E+00_JPRB, 0.10508E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.12443E+01_JPRB, 0.12425E+01_JPRB, 0.12543E+01_JPRB, 0.12959E+01_JPRB, 0.12307E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.12301E+01_JPRB, 0.12029E+01_JPRB, 0.11741E+01_JPRB, 0.11586E+01_JPRB, 0.11412E+01_JPRB, 0.11306E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11206E+01_JPRB, 0.10915E+01_JPRB, 0.10911E+01_JPRB, 0.11183E+01_JPRB, 0.12634E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.17529E+01_JPRB, 0.18907E+01_JPRB, 0.20813E+01_JPRB, 0.25170E+01_JPRB, 0.27981E+01_JPRB, 0.29156E+01_JPRB, 0.30438E+01_JPRB ,&
&  0.31080E+01_JPRB, 0.31238E+01_JPRB, 0.31424E+01_JPRB, 0.31243E+01_JPRB, 0.31332E+01_JPRB, 0.31059E+01_JPRB, 0.30071E+01_JPRB ,&
&  0.27870E+01_JPRB, 0.26801E+01_JPRB, 0.27051E+01_JPRB, 0.25873E+01_JPRB, 0.25057E+01_JPRB, 0.23668E+01_JPRB, 0.24875E+01_JPRB ,&
&  0.24873E+01_JPRB, 0.16088E+01_JPRB, 0.13250E+01_JPRB, 0.14920E+01_JPRB, 0.12707E+01_JPRB, 0.12333E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.12297E+01_JPRB, 0.12203E+01_JPRB, 0.11997E+01_JPRB, 0.12475E+01_JPRB, 0.15781E+01_JPRB, 0.16223E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.15348E+01_JPRB, 0.16201E+01_JPRB, 0.16253E+01_JPRB, 0.16020E+01_JPRB, 0.16004E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.14271E+01_JPRB, 0.13424E+01_JPRB, 0.12752E+01_JPRB, 0.12469E+01_JPRB, 0.12441E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.14159E+01_JPRB, 0.13771E+01_JPRB, 0.13445E+01_JPRB, 0.13385E+01_JPRB, 0.13592E+01_JPRB, 0.13674E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.13270E+01_JPRB, 0.13060E+01_JPRB, 0.12831E+01_JPRB, 0.12464E+01_JPRB, 0.11893E+01_JPRB, 0.11274E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.11080E+01_JPRB, 0.98554E+00_JPRB, 0.94255E+00_JPRB, 0.89795E+00_JPRB, 0.93325E+00_JPRB, 0.97726E+00_JPRB ,&
&  0.10231E+01_JPRB, 0.10436E+01_JPRB, 0.10700E+01_JPRB, 0.10774E+01_JPRB, 0.10896E+01_JPRB, 0.12019E+01_JPRB, 0.13713E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.15281E+01_JPRB, 0.14486E+01_JPRB, 0.15101E+01_JPRB, 0.14501E+01_JPRB, 0.13469E+01_JPRB, 0.13689E+01_JPRB ,&
&  0.13260E+01_JPRB, 0.12875E+01_JPRB, 0.12429E+01_JPRB, 0.12305E+01_JPRB, 0.12242E+01_JPRB, 0.11974E+01_JPRB, 0.11694E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.11130E+01_JPRB, 0.11070E+01_JPRB, 0.10997E+01_JPRB, 0.11211E+01_JPRB, 0.12269E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.14700E+01_JPRB, 0.13887E+01_JPRB, 0.13601E+01_JPRB, 0.18162E+01_JPRB, 0.24940E+01_JPRB, 0.27519E+01_JPRB, 0.27089E+01_JPRB ,&
&  0.29341E+01_JPRB, 0.30635E+01_JPRB, 0.32053E+01_JPRB, 0.32891E+01_JPRB, 0.33231E+01_JPRB, 0.32251E+01_JPRB, 0.30753E+01_JPRB ,&
&  0.29151E+01_JPRB, 0.28215E+01_JPRB, 0.27389E+01_JPRB, 0.26712E+01_JPRB, 0.25709E+01_JPRB, 0.25878E+01_JPRB, 0.27077E+01_JPRB ,&
&  0.25409E+01_JPRB, 0.22537E+01_JPRB, 0.16160E+01_JPRB, 0.16104E+01_JPRB, 0.12962E+01_JPRB, 0.12484E+01_JPRB, 0.13488E+01_JPRB ,&
&  0.13024E+01_JPRB, 0.13335E+01_JPRB, 0.13332E+01_JPRB, 0.13205E+01_JPRB, 0.15417E+01_JPRB, 0.19186E+01_JPRB, 0.20194E+01_JPRB ,&
&  0.20903E+01_JPRB, 0.19614E+01_JPRB, 0.21174E+01_JPRB, 0.21154E+01_JPRB, 0.20687E+01_JPRB, 0.20165E+01_JPRB, 0.19284E+01_JPRB ,&
&  0.17899E+01_JPRB, 0.16417E+01_JPRB, 0.15319E+01_JPRB, 0.14754E+01_JPRB, 0.14404E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.15151E+01_JPRB, 0.14455E+01_JPRB, 0.13991E+01_JPRB, 0.13856E+01_JPRB, 0.13994E+01_JPRB, 0.14018E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.13512E+01_JPRB, 0.13228E+01_JPRB, 0.12970E+01_JPRB, 0.12390E+01_JPRB, 0.11861E+01_JPRB, 0.11220E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.10657E+01_JPRB, 0.10399E+01_JPRB, 0.93640E+00_JPRB, 0.89437E+00_JPRB, 0.85760E+00_JPRB, 0.88026E+00_JPRB, 0.94108E+00_JPRB ,&
&  0.10136E+01_JPRB, 0.10689E+01_JPRB, 0.11836E+01_JPRB, 0.12262E+01_JPRB, 0.12874E+01_JPRB, 0.14531E+01_JPRB, 0.16691E+01_JPRB ,&
&  0.19472E+01_JPRB, 0.16922E+01_JPRB, 0.16960E+01_JPRB, 0.16252E+01_JPRB, 0.15670E+01_JPRB, 0.14850E+01_JPRB, 0.14020E+01_JPRB ,&
&  0.13368E+01_JPRB, 0.12866E+01_JPRB, 0.12724E+01_JPRB, 0.12536E+01_JPRB, 0.12295E+01_JPRB, 0.12066E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.11631E+01_JPRB, 0.11307E+01_JPRB, 0.10961E+01_JPRB, 0.11828E+01_JPRB, 0.12887E+01_JPRB, 0.12999E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.13238E+01_JPRB, 0.17271E+01_JPRB, 0.20654E+01_JPRB, 0.23930E+01_JPRB, 0.27353E+01_JPRB, 0.29693E+01_JPRB ,&
&  0.31588E+01_JPRB, 0.32336E+01_JPRB, 0.32986E+01_JPRB, 0.29648E+01_JPRB, 0.32455E+01_JPRB, 0.31880E+01_JPRB, 0.30172E+01_JPRB ,&
&  0.28246E+01_JPRB, 0.27718E+01_JPRB, 0.26274E+01_JPRB, 0.26591E+01_JPRB, 0.27947E+01_JPRB, 0.30356E+01_JPRB, 0.27129E+01_JPRB ,&
&  0.23414E+01_JPRB, 0.25269E+01_JPRB, 0.22125E+01_JPRB, 0.18160E+01_JPRB, 0.14947E+01_JPRB, 0.14677E+01_JPRB, 0.14639E+01_JPRB ,&
&  0.14406E+01_JPRB, 0.14313E+01_JPRB, 0.14189E+01_JPRB, 0.15688E+01_JPRB, 0.20098E+01_JPRB, 0.23791E+01_JPRB, 0.23857E+01_JPRB ,&
&  0.23291E+01_JPRB, 0.25337E+01_JPRB, 0.25601E+01_JPRB, 0.25253E+01_JPRB, 0.24958E+01_JPRB, 0.24229E+01_JPRB, 0.22877E+01_JPRB ,&
&  0.21094E+01_JPRB, 0.19429E+01_JPRB, 0.18103E+01_JPRB, 0.16935E+01_JPRB, 0.15951E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.15593E+01_JPRB, 0.14833E+01_JPRB, 0.14425E+01_JPRB, 0.14366E+01_JPRB, 0.14543E+01_JPRB, 0.14433E+01_JPRB, 0.14104E+01_JPRB ,&
&  0.13655E+01_JPRB, 0.13165E+01_JPRB, 0.12583E+01_JPRB, 0.12055E+01_JPRB, 0.11681E+01_JPRB, 0.11140E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10658E+01_JPRB, 0.10071E+01_JPRB, 0.93864E+00_JPRB, 0.96067E+00_JPRB, 0.95983E+00_JPRB, 0.90791E+00_JPRB, 0.94517E+00_JPRB ,&
&  0.10151E+01_JPRB, 0.11360E+01_JPRB, 0.13056E+01_JPRB, 0.14914E+01_JPRB, 0.16426E+01_JPRB, 0.18638E+01_JPRB, 0.20558E+01_JPRB ,&
&  0.20155E+01_JPRB, 0.19549E+01_JPRB, 0.18700E+01_JPRB, 0.17582E+01_JPRB, 0.15938E+01_JPRB, 0.14742E+01_JPRB, 0.14002E+01_JPRB ,&
&  0.13430E+01_JPRB, 0.13130E+01_JPRB, 0.12990E+01_JPRB, 0.12799E+01_JPRB, 0.12736E+01_JPRB, 0.12676E+01_JPRB, 0.12686E+01_JPRB ,&
&  0.12213E+01_JPRB, 0.11751E+01_JPRB, 0.11228E+01_JPRB, 0.11366E+01_JPRB, 0.11622E+01_JPRB, 0.11962E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.15969E+01_JPRB, 0.17224E+01_JPRB, 0.22629E+01_JPRB, 0.21902E+01_JPRB, 0.23611E+01_JPRB, 0.28745E+01_JPRB ,&
&  0.31265E+01_JPRB, 0.32071E+01_JPRB, 0.32149E+01_JPRB, 0.26479E+01_JPRB, 0.23456E+01_JPRB, 0.30563E+01_JPRB, 0.30010E+01_JPRB ,&
&  0.28000E+01_JPRB, 0.27316E+01_JPRB, 0.27123E+01_JPRB, 0.30811E+01_JPRB, 0.26128E+01_JPRB, 0.23720E+01_JPRB, 0.19376E+01_JPRB ,&
&  0.20120E+01_JPRB, 0.17925E+01_JPRB, 0.20211E+01_JPRB, 0.18169E+01_JPRB, 0.17783E+01_JPRB, 0.16793E+01_JPRB, 0.15534E+01_JPRB ,&
&  0.15210E+01_JPRB, 0.15308E+01_JPRB, 0.16262E+01_JPRB, 0.21028E+01_JPRB, 0.27876E+01_JPRB, 0.27663E+01_JPRB, 0.29182E+01_JPRB ,&
&  0.30379E+01_JPRB, 0.30643E+01_JPRB, 0.28595E+01_JPRB, 0.28502E+01_JPRB, 0.27541E+01_JPRB, 0.26268E+01_JPRB, 0.24746E+01_JPRB ,&
&  0.23107E+01_JPRB, 0.21518E+01_JPRB, 0.19829E+01_JPRB, 0.18153E+01_JPRB, 0.16698E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.16211E+01_JPRB, 0.15595E+01_JPRB, 0.15348E+01_JPRB, 0.15312E+01_JPRB, 0.15192E+01_JPRB, 0.14808E+01_JPRB, 0.14279E+01_JPRB ,&
&  0.13629E+01_JPRB, 0.12826E+01_JPRB, 0.12124E+01_JPRB, 0.11620E+01_JPRB, 0.11378E+01_JPRB, 0.11112E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.10855E+01_JPRB, 0.10162E+01_JPRB, 0.98996E+00_JPRB, 0.10797E+01_JPRB, 0.10127E+01_JPRB, 0.95961E+00_JPRB ,&
&  0.10724E+01_JPRB, 0.12336E+01_JPRB, 0.14715E+01_JPRB, 0.17689E+01_JPRB, 0.20063E+01_JPRB, 0.22413E+01_JPRB, 0.23487E+01_JPRB ,&
&  0.23238E+01_JPRB, 0.21080E+01_JPRB, 0.18960E+01_JPRB, 0.17010E+01_JPRB, 0.15092E+01_JPRB, 0.13938E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.12852E+01_JPRB, 0.12799E+01_JPRB, 0.12648E+01_JPRB, 0.12252E+01_JPRB, 0.12184E+01_JPRB, 0.11735E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.11357E+01_JPRB, 0.11217E+01_JPRB, 0.11832E+01_JPRB, 0.11680E+01_JPRB, 0.11831E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.15771E+01_JPRB, 0.17356E+01_JPRB, 0.21259E+01_JPRB, 0.26688E+01_JPRB, 0.27718E+01_JPRB, 0.30569E+01_JPRB ,&
&  0.26713E+01_JPRB, 0.23024E+01_JPRB, 0.20199E+01_JPRB, 0.17568E+01_JPRB, 0.20997E+01_JPRB, 0.30294E+01_JPRB, 0.32329E+01_JPRB ,&
&  0.30920E+01_JPRB, 0.30839E+01_JPRB, 0.33134E+01_JPRB, 0.31810E+01_JPRB, 0.24956E+01_JPRB, 0.19152E+01_JPRB, 0.24152E+01_JPRB ,&
&  0.25422E+01_JPRB, 0.25733E+01_JPRB, 0.23547E+01_JPRB, 0.19584E+01_JPRB, 0.16846E+01_JPRB, 0.18889E+01_JPRB, 0.17880E+01_JPRB ,&
&  0.17022E+01_JPRB, 0.17154E+01_JPRB, 0.23545E+01_JPRB, 0.32603E+01_JPRB, 0.35652E+01_JPRB, 0.37508E+01_JPRB, 0.38044E+01_JPRB ,&
&  0.35407E+01_JPRB, 0.33744E+01_JPRB, 0.31707E+01_JPRB, 0.29380E+01_JPRB, 0.27304E+01_JPRB, 0.25225E+01_JPRB, 0.23557E+01_JPRB ,&
&  0.22158E+01_JPRB, 0.20879E+01_JPRB, 0.19510E+01_JPRB, 0.18191E+01_JPRB, 0.17063E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.16730E+01_JPRB, 0.16430E+01_JPRB, 0.16145E+01_JPRB, 0.15867E+01_JPRB, 0.15446E+01_JPRB, 0.14889E+01_JPRB, 0.14169E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.12491E+01_JPRB, 0.11806E+01_JPRB, 0.11471E+01_JPRB, 0.11203E+01_JPRB, 0.10856E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.12202E+01_JPRB, 0.11118E+01_JPRB, 0.10848E+01_JPRB, 0.11318E+01_JPRB, 0.11545E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.13662E+01_JPRB, 0.15744E+01_JPRB, 0.19094E+01_JPRB, 0.24198E+01_JPRB, 0.23508E+01_JPRB, 0.23721E+01_JPRB ,&
&  0.23156E+01_JPRB, 0.20857E+01_JPRB, 0.18921E+01_JPRB, 0.16594E+01_JPRB, 0.14670E+01_JPRB, 0.13691E+01_JPRB, 0.12893E+01_JPRB ,&
&  0.12687E+01_JPRB, 0.12755E+01_JPRB, 0.12622E+01_JPRB, 0.12237E+01_JPRB, 0.11846E+01_JPRB, 0.11533E+01_JPRB, 0.11255E+01_JPRB ,&
&  0.11223E+01_JPRB, 0.11206E+01_JPRB, 0.11278E+01_JPRB, 0.12297E+01_JPRB, 0.12512E+01_JPRB, 0.12249E+01_JPRB, 0.14410E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.18648E+01_JPRB, 0.19555E+01_JPRB, 0.22691E+01_JPRB, 0.26217E+01_JPRB, 0.27802E+01_JPRB, 0.29244E+01_JPRB ,&
&  0.23708E+01_JPRB, 0.21084E+01_JPRB, 0.22718E+01_JPRB, 0.24651E+01_JPRB, 0.23569E+01_JPRB, 0.25742E+01_JPRB, 0.39741E+01_JPRB ,&
&  0.29910E+01_JPRB, 0.32600E+01_JPRB, 0.37267E+01_JPRB, 0.38775E+01_JPRB, 0.19929E+01_JPRB, 0.14921E+01_JPRB, 0.22826E+01_JPRB ,&
&  0.23407E+01_JPRB, 0.15100E+01_JPRB, 0.11917E+01_JPRB, 0.14385E+01_JPRB, 0.11936E+01_JPRB, 0.13847E+01_JPRB, 0.17653E+01_JPRB ,&
&  0.19730E+01_JPRB, 0.21350E+01_JPRB, 0.31525E+01_JPRB, 0.36764E+01_JPRB, 0.38354E+01_JPRB, 0.51954E+01_JPRB, 0.45497E+01_JPRB ,&
&  0.41024E+01_JPRB, 0.34006E+01_JPRB, 0.31952E+01_JPRB, 0.28342E+01_JPRB, 0.25795E+01_JPRB, 0.23487E+01_JPRB, 0.21584E+01_JPRB ,&
&  0.20157E+01_JPRB, 0.18977E+01_JPRB, 0.18124E+01_JPRB, 0.17462E+01_JPRB, 0.17090E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.17316E+01_JPRB, 0.16946E+01_JPRB, 0.16581E+01_JPRB, 0.16132E+01_JPRB, 0.15558E+01_JPRB, 0.14776E+01_JPRB, 0.13772E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.12091E+01_JPRB, 0.11556E+01_JPRB, 0.11274E+01_JPRB, 0.10949E+01_JPRB, 0.10665E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.12345E+01_JPRB, 0.12684E+01_JPRB, 0.12412E+01_JPRB, 0.12715E+01_JPRB, 0.12975E+01_JPRB, 0.12395E+01_JPRB ,&
&  0.14441E+01_JPRB, 0.15675E+01_JPRB, 0.17207E+01_JPRB, 0.19636E+01_JPRB, 0.22088E+01_JPRB, 0.22026E+01_JPRB, 0.22815E+01_JPRB ,&
&  0.21846E+01_JPRB, 0.20355E+01_JPRB, 0.18130E+01_JPRB, 0.16023E+01_JPRB, 0.14551E+01_JPRB, 0.13457E+01_JPRB, 0.12956E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12917E+01_JPRB, 0.12763E+01_JPRB, 0.12229E+01_JPRB, 0.11917E+01_JPRB, 0.11594E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.11270E+01_JPRB, 0.11458E+01_JPRB, 0.11820E+01_JPRB, 0.13016E+01_JPRB, 0.15223E+01_JPRB, 0.16102E+01_JPRB, 0.15252E+01_JPRB ,&
&  0.14865E+01_JPRB, 0.16840E+01_JPRB, 0.20617E+01_JPRB, 0.22129E+01_JPRB, 0.24343E+01_JPRB, 0.26552E+01_JPRB, 0.26471E+01_JPRB ,&
&  0.25999E+01_JPRB, 0.27528E+01_JPRB, 0.26554E+01_JPRB, 0.29505E+01_JPRB, 0.30976E+01_JPRB, 0.26103E+01_JPRB, 0.31879E+01_JPRB ,&
&  0.33735E+01_JPRB, 0.34247E+01_JPRB, 0.29949E+01_JPRB, 0.24101E+01_JPRB, 0.27048E+01_JPRB, 0.21350E+01_JPRB, 0.13630E+01_JPRB ,&
&  0.11798E+01_JPRB, 0.10913E+01_JPRB, 0.10323E+01_JPRB, 0.10534E+01_JPRB, 0.10638E+01_JPRB, 0.13700E+01_JPRB, 0.17787E+01_JPRB ,&
&  0.25219E+01_JPRB, 0.30502E+01_JPRB, 0.34870E+01_JPRB, 0.37463E+01_JPRB, 0.36814E+01_JPRB, 0.48913E+01_JPRB, 0.43750E+01_JPRB ,&
&  0.35320E+01_JPRB, 0.31817E+01_JPRB, 0.29315E+01_JPRB, 0.26593E+01_JPRB, 0.24575E+01_JPRB, 0.22931E+01_JPRB, 0.21529E+01_JPRB ,&
&  0.20439E+01_JPRB, 0.19530E+01_JPRB, 0.18857E+01_JPRB, 0.18299E+01_JPRB, 0.17706E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.17348E+01_JPRB, 0.16898E+01_JPRB, 0.16461E+01_JPRB, 0.15803E+01_JPRB, 0.15045E+01_JPRB, 0.14069E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.11388E+01_JPRB, 0.11099E+01_JPRB, 0.10814E+01_JPRB, 0.10636E+01_JPRB, 0.10596E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.12514E+01_JPRB, 0.13822E+01_JPRB, 0.14442E+01_JPRB, 0.16518E+01_JPRB, 0.16726E+01_JPRB, 0.17437E+01_JPRB ,&
&  0.18827E+01_JPRB, 0.19112E+01_JPRB, 0.20218E+01_JPRB, 0.21102E+01_JPRB, 0.22428E+01_JPRB, 0.22233E+01_JPRB, 0.21944E+01_JPRB ,&
&  0.20842E+01_JPRB, 0.19761E+01_JPRB, 0.17669E+01_JPRB, 0.15732E+01_JPRB, 0.14242E+01_JPRB, 0.13355E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.12765E+01_JPRB, 0.13023E+01_JPRB, 0.12985E+01_JPRB, 0.12479E+01_JPRB, 0.12128E+01_JPRB, 0.11958E+01_JPRB, 0.11963E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.12397E+01_JPRB, 0.12970E+01_JPRB, 0.14367E+01_JPRB, 0.13905E+01_JPRB, 0.12651E+01_JPRB, 0.14364E+01_JPRB ,&
&  0.15751E+01_JPRB, 0.18786E+01_JPRB, 0.19728E+01_JPRB, 0.22223E+01_JPRB, 0.23278E+01_JPRB, 0.23222E+01_JPRB, 0.24955E+01_JPRB ,&
&  0.26679E+01_JPRB, 0.34136E+01_JPRB, 0.27303E+01_JPRB, 0.28066E+01_JPRB, 0.40797E+01_JPRB, 0.34889E+01_JPRB, 0.33321E+01_JPRB ,&
&  0.38320E+01_JPRB, 0.36296E+01_JPRB, 0.39602E+01_JPRB, 0.36628E+01_JPRB, 0.61690E+01_JPRB, 0.69770E+01_JPRB, 0.20570E+01_JPRB ,&
&  0.14121E+01_JPRB, 0.12222E+01_JPRB, 0.11180E+01_JPRB, 0.10281E+01_JPRB, 0.11305E+01_JPRB, 0.15741E+01_JPRB, 0.23996E+01_JPRB ,&
&  0.28403E+01_JPRB, 0.36162E+01_JPRB, 0.38571E+01_JPRB, 0.41909E+01_JPRB, 0.37573E+01_JPRB, 0.40320E+01_JPRB, 0.32523E+01_JPRB ,&
&  0.30875E+01_JPRB, 0.29378E+01_JPRB, 0.26641E+01_JPRB, 0.24277E+01_JPRB, 0.22838E+01_JPRB, 0.21804E+01_JPRB, 0.20841E+01_JPRB ,&
&  0.20016E+01_JPRB, 0.19440E+01_JPRB, 0.18897E+01_JPRB, 0.18361E+01_JPRB, 0.17838E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.16740E+01_JPRB, 0.16293E+01_JPRB, 0.15718E+01_JPRB, 0.14855E+01_JPRB, 0.14085E+01_JPRB, 0.13198E+01_JPRB, 0.12175E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.11271E+01_JPRB, 0.10918E+01_JPRB, 0.10587E+01_JPRB, 0.10555E+01_JPRB, 0.10711E+01_JPRB, 0.11216E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.13106E+01_JPRB, 0.14469E+01_JPRB, 0.17453E+01_JPRB, 0.20401E+01_JPRB, 0.21001E+01_JPRB, 0.25978E+01_JPRB ,&
&  0.25744E+01_JPRB, 0.24393E+01_JPRB, 0.24465E+01_JPRB, 0.23913E+01_JPRB, 0.23240E+01_JPRB, 0.22270E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.20082E+01_JPRB, 0.19210E+01_JPRB, 0.17119E+01_JPRB, 0.15397E+01_JPRB, 0.14290E+01_JPRB, 0.13420E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.13120E+01_JPRB, 0.13383E+01_JPRB, 0.13414E+01_JPRB, 0.12970E+01_JPRB, 0.12630E+01_JPRB, 0.12479E+01_JPRB, 0.12587E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.13651E+01_JPRB, 0.14429E+01_JPRB, 0.15457E+01_JPRB, 0.13491E+01_JPRB, 0.15348E+01_JPRB, 0.16196E+01_JPRB ,&
&  0.17715E+01_JPRB, 0.18982E+01_JPRB, 0.18809E+01_JPRB, 0.20077E+01_JPRB, 0.22061E+01_JPRB, 0.22531E+01_JPRB, 0.24307E+01_JPRB ,&
&  0.29194E+01_JPRB, 0.28678E+01_JPRB, 0.26188E+01_JPRB, 0.29330E+01_JPRB, 0.38322E+01_JPRB, 0.54620E+01_JPRB, 0.32170E+01_JPRB ,&
&  0.27778E+01_JPRB, 0.42164E+01_JPRB, 0.49841E+01_JPRB, 0.51258E+01_JPRB, 0.76299E+01_JPRB, 0.10593E+02_JPRB, 0.64436E+01_JPRB ,&
&  0.17880E+01_JPRB, 0.12535E+01_JPRB, 0.14380E+01_JPRB, 0.12777E+01_JPRB, 0.12080E+01_JPRB, 0.16994E+01_JPRB, 0.45055E+01_JPRB ,&
&  0.36058E+01_JPRB, 0.42545E+01_JPRB, 0.44209E+01_JPRB, 0.43017E+01_JPRB, 0.37358E+01_JPRB, 0.34675E+01_JPRB, 0.29865E+01_JPRB ,&
&  0.25688E+01_JPRB, 0.23848E+01_JPRB, 0.21566E+01_JPRB, 0.20263E+01_JPRB, 0.19806E+01_JPRB, 0.19513E+01_JPRB, 0.18854E+01_JPRB ,&
&  0.18400E+01_JPRB, 0.18093E+01_JPRB, 0.17779E+01_JPRB, 0.17510E+01_JPRB, 0.17191E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.15672E+01_JPRB, 0.15252E+01_JPRB, 0.14480E+01_JPRB, 0.13794E+01_JPRB, 0.13484E+01_JPRB, 0.12772E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.11758E+01_JPRB, 0.11523E+01_JPRB, 0.11223E+01_JPRB, 0.11248E+01_JPRB, 0.11077E+01_JPRB, 0.11364E+01_JPRB, 0.11975E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.14017E+01_JPRB, 0.15363E+01_JPRB, 0.18687E+01_JPRB, 0.23261E+01_JPRB, 0.24458E+01_JPRB, 0.30461E+01_JPRB ,&
&  0.35912E+01_JPRB, 0.33051E+01_JPRB, 0.28762E+01_JPRB, 0.26227E+01_JPRB, 0.24542E+01_JPRB, 0.23900E+01_JPRB, 0.21592E+01_JPRB ,&
&  0.19913E+01_JPRB, 0.19176E+01_JPRB, 0.16966E+01_JPRB, 0.15289E+01_JPRB, 0.14089E+01_JPRB, 0.13497E+01_JPRB, 0.13643E+01_JPRB ,&
&  0.13731E+01_JPRB, 0.14042E+01_JPRB, 0.14058E+01_JPRB, 0.13764E+01_JPRB, 0.13469E+01_JPRB, 0.13328E+01_JPRB, 0.13797E+01_JPRB ,&
&  0.14646E+01_JPRB, 0.15528E+01_JPRB, 0.16162E+01_JPRB, 0.16175E+01_JPRB, 0.16535E+01_JPRB, 0.16968E+01_JPRB, 0.18280E+01_JPRB ,&
&  0.19101E+01_JPRB, 0.19607E+01_JPRB, 0.19923E+01_JPRB, 0.20133E+01_JPRB, 0.21750E+01_JPRB, 0.23729E+01_JPRB, 0.26087E+01_JPRB ,&
&  0.29769E+01_JPRB, 0.31081E+01_JPRB, 0.29120E+01_JPRB, 0.31786E+01_JPRB, 0.38200E+01_JPRB, 0.47790E+01_JPRB, 0.46063E+01_JPRB ,&
&  0.41663E+01_JPRB, 0.41076E+01_JPRB, 0.50342E+01_JPRB, 0.66719E+01_JPRB, 0.76355E+01_JPRB, 0.79409E+01_JPRB, 0.96583E+01_JPRB ,&
&  0.80060E+01_JPRB, 0.35737E+01_JPRB, 0.27506E+01_JPRB, 0.34311E+01_JPRB, 0.24096E+01_JPRB, 0.24940E+01_JPRB, 0.35770E+01_JPRB ,&
&  0.38943E+01_JPRB, 0.45819E+01_JPRB, 0.47479E+01_JPRB, 0.38257E+01_JPRB, 0.34191E+01_JPRB, 0.30966E+01_JPRB, 0.25771E+01_JPRB ,&
&  0.22041E+01_JPRB, 0.19732E+01_JPRB, 0.18372E+01_JPRB, 0.17478E+01_JPRB, 0.16935E+01_JPRB, 0.16416E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.15701E+01_JPRB, 0.15725E+01_JPRB, 0.15774E+01_JPRB, 0.15826E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.14707E+01_JPRB, 0.14139E+01_JPRB, 0.13893E+01_JPRB, 0.13890E+01_JPRB, 0.13669E+01_JPRB, 0.13352E+01_JPRB, 0.13065E+01_JPRB ,&
&  0.12474E+01_JPRB, 0.12562E+01_JPRB, 0.12999E+01_JPRB, 0.12993E+01_JPRB, 0.12593E+01_JPRB, 0.12711E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.14213E+01_JPRB, 0.15911E+01_JPRB, 0.18251E+01_JPRB, 0.23323E+01_JPRB, 0.29414E+01_JPRB, 0.40355E+01_JPRB, 0.31382E+01_JPRB ,&
&  0.34981E+01_JPRB, 0.36950E+01_JPRB, 0.30393E+01_JPRB, 0.27816E+01_JPRB, 0.25994E+01_JPRB, 0.25023E+01_JPRB, 0.21728E+01_JPRB ,&
&  0.20528E+01_JPRB, 0.19148E+01_JPRB, 0.17139E+01_JPRB, 0.15408E+01_JPRB, 0.14234E+01_JPRB, 0.14465E+01_JPRB, 0.14563E+01_JPRB ,&
&  0.14790E+01_JPRB, 0.15207E+01_JPRB, 0.15001E+01_JPRB, 0.14928E+01_JPRB, 0.14742E+01_JPRB, 0.14685E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.16399E+01_JPRB, 0.17231E+01_JPRB, 0.17847E+01_JPRB, 0.17395E+01_JPRB, 0.18172E+01_JPRB, 0.19002E+01_JPRB, 0.20419E+01_JPRB ,&
&  0.20271E+01_JPRB, 0.18718E+01_JPRB, 0.19482E+01_JPRB, 0.21339E+01_JPRB, 0.23448E+01_JPRB, 0.24375E+01_JPRB, 0.26492E+01_JPRB ,&
&  0.32222E+01_JPRB, 0.37786E+01_JPRB, 0.41307E+01_JPRB, 0.36703E+01_JPRB, 0.38114E+01_JPRB, 0.46882E+01_JPRB, 0.57232E+01_JPRB ,&
&  0.62724E+01_JPRB, 0.63645E+01_JPRB, 0.68920E+01_JPRB, 0.68132E+01_JPRB, 0.65745E+01_JPRB, 0.62838E+01_JPRB, 0.66815E+01_JPRB ,&
&  0.76968E+01_JPRB, 0.83837E+01_JPRB, 0.63032E+01_JPRB, 0.48868E+01_JPRB, 0.46860E+01_JPRB, 0.39888E+01_JPRB, 0.37248E+01_JPRB ,&
&  0.43493E+01_JPRB, 0.44561E+01_JPRB, 0.42249E+01_JPRB, 0.42758E+01_JPRB, 0.32771E+01_JPRB, 0.26139E+01_JPRB, 0.19825E+01_JPRB ,&
&  0.15950E+01_JPRB, 0.14575E+01_JPRB, 0.14054E+01_JPRB, 0.13891E+01_JPRB, 0.14316E+01_JPRB, 0.14391E+01_JPRB, 0.14409E+01_JPRB ,&
&  0.14207E+01_JPRB, 0.14421E+01_JPRB, 0.14773E+01_JPRB, 0.15083E+01_JPRB, 0.15069E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.13986E+01_JPRB, 0.13871E+01_JPRB, 0.14063E+01_JPRB, 0.14260E+01_JPRB, 0.14233E+01_JPRB, 0.14448E+01_JPRB, 0.14397E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13645E+01_JPRB, 0.14136E+01_JPRB, 0.13983E+01_JPRB, 0.13823E+01_JPRB, 0.14066E+01_JPRB, 0.14905E+01_JPRB ,&
&  0.16110E+01_JPRB, 0.18637E+01_JPRB, 0.22882E+01_JPRB, 0.28639E+01_JPRB, 0.33273E+01_JPRB, 0.39136E+01_JPRB, 0.42188E+01_JPRB ,&
&  0.30920E+01_JPRB, 0.38180E+01_JPRB, 0.29006E+01_JPRB, 0.27595E+01_JPRB, 0.27120E+01_JPRB, 0.29306E+01_JPRB, 0.26274E+01_JPRB ,&
&  0.21747E+01_JPRB, 0.18941E+01_JPRB, 0.18145E+01_JPRB, 0.16980E+01_JPRB, 0.15581E+01_JPRB, 0.15453E+01_JPRB, 0.15716E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.15736E+01_JPRB, 0.15959E+01_JPRB, 0.16142E+01_JPRB, 0.16075E+01_JPRB, 0.16180E+01_JPRB, 0.16612E+01_JPRB ,&
&  0.17766E+01_JPRB, 0.18753E+01_JPRB, 0.19050E+01_JPRB, 0.19473E+01_JPRB, 0.19804E+01_JPRB, 0.21501E+01_JPRB, 0.23038E+01_JPRB ,&
&  0.22394E+01_JPRB, 0.20460E+01_JPRB, 0.20949E+01_JPRB, 0.22155E+01_JPRB, 0.23240E+01_JPRB, 0.26279E+01_JPRB, 0.28417E+01_JPRB ,&
&  0.35999E+01_JPRB, 0.38437E+01_JPRB, 0.46965E+01_JPRB, 0.38690E+01_JPRB, 0.38879E+01_JPRB, 0.47364E+01_JPRB, 0.50694E+01_JPRB ,&
&  0.50261E+01_JPRB, 0.60601E+01_JPRB, 0.63895E+01_JPRB, 0.66033E+01_JPRB, 0.64156E+01_JPRB, 0.58124E+01_JPRB, 0.54310E+01_JPRB ,&
&  0.56401E+01_JPRB, 0.60432E+01_JPRB, 0.59577E+01_JPRB, 0.49121E+01_JPRB, 0.45707E+01_JPRB, 0.40618E+01_JPRB, 0.42982E+01_JPRB ,&
&  0.49390E+01_JPRB, 0.50220E+01_JPRB, 0.45881E+01_JPRB, 0.38935E+01_JPRB, 0.23931E+01_JPRB, 0.17539E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.12445E+01_JPRB, 0.12031E+01_JPRB, 0.12326E+01_JPRB, 0.13036E+01_JPRB, 0.13656E+01_JPRB, 0.13931E+01_JPRB, 0.13856E+01_JPRB ,&
&  0.13901E+01_JPRB, 0.14124E+01_JPRB, 0.14259E+01_JPRB, 0.14265E+01_JPRB, 0.14176E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.14041E+01_JPRB, 0.14247E+01_JPRB, 0.14520E+01_JPRB, 0.14988E+01_JPRB, 0.15319E+01_JPRB, 0.16230E+01_JPRB, 0.17131E+01_JPRB ,&
&  0.16070E+01_JPRB, 0.12888E+01_JPRB, 0.13375E+01_JPRB, 0.13853E+01_JPRB, 0.14354E+01_JPRB, 0.15014E+01_JPRB, 0.16144E+01_JPRB ,&
&  0.18456E+01_JPRB, 0.21535E+01_JPRB, 0.25331E+01_JPRB, 0.29180E+01_JPRB, 0.33899E+01_JPRB, 0.39216E+01_JPRB, 0.40361E+01_JPRB ,&
&  0.32822E+01_JPRB, 0.28147E+01_JPRB, 0.29743E+01_JPRB, 0.29009E+01_JPRB, 0.28712E+01_JPRB, 0.29125E+01_JPRB, 0.29046E+01_JPRB ,&
&  0.23762E+01_JPRB, 0.20689E+01_JPRB, 0.20292E+01_JPRB, 0.20397E+01_JPRB, 0.16867E+01_JPRB, 0.17185E+01_JPRB, 0.17529E+01_JPRB ,&
&  0.17545E+01_JPRB, 0.17454E+01_JPRB, 0.17781E+01_JPRB, 0.18162E+01_JPRB, 0.17978E+01_JPRB, 0.18064E+01_JPRB, 0.18696E+01_JPRB ,&
&  0.20132E+01_JPRB, 0.21206E+01_JPRB, 0.21776E+01_JPRB, 0.21763E+01_JPRB, 0.22774E+01_JPRB, 0.24562E+01_JPRB, 0.25071E+01_JPRB ,&
&  0.23987E+01_JPRB, 0.23650E+01_JPRB, 0.24162E+01_JPRB, 0.25166E+01_JPRB, 0.24914E+01_JPRB, 0.29434E+01_JPRB, 0.32358E+01_JPRB ,&
&  0.36996E+01_JPRB, 0.40193E+01_JPRB, 0.47796E+01_JPRB, 0.37857E+01_JPRB, 0.41886E+01_JPRB, 0.46678E+01_JPRB, 0.48273E+01_JPRB ,&
&  0.47403E+01_JPRB, 0.52038E+01_JPRB, 0.53063E+01_JPRB, 0.54974E+01_JPRB, 0.55410E+01_JPRB, 0.48712E+01_JPRB, 0.51771E+01_JPRB ,&
&  0.49902E+01_JPRB, 0.56482E+01_JPRB, 0.53902E+01_JPRB, 0.51980E+01_JPRB, 0.43712E+01_JPRB, 0.41795E+01_JPRB, 0.43356E+01_JPRB ,&
&  0.48429E+01_JPRB, 0.46251E+01_JPRB, 0.36847E+01_JPRB, 0.27256E+01_JPRB, 0.15536E+01_JPRB, 0.12012E+01_JPRB, 0.11198E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11868E+01_JPRB, 0.12575E+01_JPRB, 0.13214E+01_JPRB, 0.13572E+01_JPRB, 0.13691E+01_JPRB, 0.13730E+01_JPRB ,&
&  0.13892E+01_JPRB, 0.13969E+01_JPRB, 0.13931E+01_JPRB, 0.13944E+01_JPRB, 0.13974E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.14007E+01_JPRB, 0.13970E+01_JPRB, 0.13969E+01_JPRB, 0.13671E+01_JPRB, 0.13298E+01_JPRB, 0.12873E+01_JPRB, 0.12136E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.11549E+01_JPRB, 0.12317E+01_JPRB, 0.13281E+01_JPRB, 0.14207E+01_JPRB, 0.15313E+01_JPRB, 0.17058E+01_JPRB ,&
&  0.19185E+01_JPRB, 0.20851E+01_JPRB, 0.22824E+01_JPRB, 0.25979E+01_JPRB, 0.29864E+01_JPRB, 0.31658E+01_JPRB, 0.33071E+01_JPRB ,&
&  0.30328E+01_JPRB, 0.22470E+01_JPRB, 0.27757E+01_JPRB, 0.30589E+01_JPRB, 0.31262E+01_JPRB, 0.31491E+01_JPRB, 0.29184E+01_JPRB ,&
&  0.23314E+01_JPRB, 0.22743E+01_JPRB, 0.23815E+01_JPRB, 0.23852E+01_JPRB, 0.19281E+01_JPRB, 0.19453E+01_JPRB, 0.19689E+01_JPRB ,&
&  0.19973E+01_JPRB, 0.20081E+01_JPRB, 0.20727E+01_JPRB, 0.21148E+01_JPRB, 0.21201E+01_JPRB, 0.21754E+01_JPRB, 0.22675E+01_JPRB ,&
&  0.23579E+01_JPRB, 0.24272E+01_JPRB, 0.25272E+01_JPRB, 0.25932E+01_JPRB, 0.26668E+01_JPRB, 0.27687E+01_JPRB, 0.27420E+01_JPRB ,&
&  0.26522E+01_JPRB, 0.25733E+01_JPRB, 0.27318E+01_JPRB, 0.30832E+01_JPRB, 0.33858E+01_JPRB, 0.31622E+01_JPRB, 0.35743E+01_JPRB ,&
&  0.37963E+01_JPRB, 0.40607E+01_JPRB, 0.41370E+01_JPRB, 0.49105E+01_JPRB, 0.35953E+01_JPRB, 0.41283E+01_JPRB, 0.39835E+01_JPRB ,&
&  0.43676E+01_JPRB, 0.42705E+01_JPRB, 0.41901E+01_JPRB, 0.42692E+01_JPRB, 0.43989E+01_JPRB, 0.40145E+01_JPRB, 0.43173E+01_JPRB ,&
&  0.46744E+01_JPRB, 0.49856E+01_JPRB, 0.51009E+01_JPRB, 0.52295E+01_JPRB, 0.48661E+01_JPRB, 0.47972E+01_JPRB, 0.46392E+01_JPRB ,&
&  0.47945E+01_JPRB, 0.35790E+01_JPRB, 0.25814E+01_JPRB, 0.17946E+01_JPRB, 0.12110E+01_JPRB, 0.10168E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.12044E+01_JPRB, 0.12585E+01_JPRB, 0.12984E+01_JPRB, 0.13242E+01_JPRB, 0.13439E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.13881E+01_JPRB, 0.13929E+01_JPRB, 0.13912E+01_JPRB, 0.13910E+01_JPRB, 0.13954E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.13556E+01_JPRB, 0.13259E+01_JPRB, 0.12703E+01_JPRB, 0.11860E+01_JPRB, 0.10969E+01_JPRB, 0.10119E+01_JPRB, 0.96419E+00_JPRB ,&
&  0.93194E+00_JPRB, 0.97025E+00_JPRB, 0.10668E+01_JPRB, 0.11956E+01_JPRB, 0.13410E+01_JPRB, 0.14863E+01_JPRB, 0.16182E+01_JPRB ,&
&  0.17007E+01_JPRB, 0.17526E+01_JPRB, 0.18675E+01_JPRB, 0.21358E+01_JPRB, 0.24447E+01_JPRB, 0.26929E+01_JPRB, 0.28701E+01_JPRB ,&
&  0.30285E+01_JPRB, 0.30591E+01_JPRB, 0.28737E+01_JPRB, 0.23326E+01_JPRB, 0.22092E+01_JPRB, 0.27093E+01_JPRB, 0.23397E+01_JPRB ,&
&  0.22339E+01_JPRB, 0.21484E+01_JPRB, 0.21527E+01_JPRB, 0.21732E+01_JPRB, 0.21251E+01_JPRB, 0.21024E+01_JPRB, 0.21584E+01_JPRB ,&
&  0.22033E+01_JPRB, 0.22706E+01_JPRB, 0.23393E+01_JPRB, 0.23846E+01_JPRB, 0.24526E+01_JPRB, 0.25439E+01_JPRB, 0.26271E+01_JPRB ,&
&  0.26948E+01_JPRB, 0.28025E+01_JPRB, 0.29258E+01_JPRB, 0.30461E+01_JPRB, 0.30288E+01_JPRB, 0.30896E+01_JPRB, 0.30646E+01_JPRB ,&
&  0.30263E+01_JPRB, 0.30078E+01_JPRB, 0.30822E+01_JPRB, 0.33515E+01_JPRB, 0.35122E+01_JPRB, 0.31988E+01_JPRB, 0.33257E+01_JPRB ,&
&  0.37492E+01_JPRB, 0.39759E+01_JPRB, 0.36300E+01_JPRB, 0.47451E+01_JPRB, 0.30091E+01_JPRB, 0.37309E+01_JPRB, 0.41080E+01_JPRB ,&
&  0.38346E+01_JPRB, 0.35295E+01_JPRB, 0.34930E+01_JPRB, 0.35670E+01_JPRB, 0.35516E+01_JPRB, 0.34878E+01_JPRB, 0.39335E+01_JPRB ,&
&  0.42753E+01_JPRB, 0.44717E+01_JPRB, 0.47915E+01_JPRB, 0.48324E+01_JPRB, 0.47982E+01_JPRB, 0.44031E+01_JPRB, 0.38163E+01_JPRB ,&
&  0.31162E+01_JPRB, 0.27701E+01_JPRB, 0.22443E+01_JPRB, 0.19249E+01_JPRB, 0.12761E+01_JPRB, 0.98401E+00_JPRB, 0.10529E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.11580E+01_JPRB, 0.12177E+01_JPRB, 0.12638E+01_JPRB, 0.12831E+01_JPRB, 0.13042E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.13423E+01_JPRB, 0.13417E+01_JPRB, 0.13464E+01_JPRB, 0.13488E+01_JPRB, 0.13575E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.11188E+01_JPRB, 0.10817E+01_JPRB, 0.10237E+01_JPRB, 0.96896E+00_JPRB, 0.91558E+00_JPRB, 0.85664E+00_JPRB, 0.84693E+00_JPRB ,&
&  0.82666E+00_JPRB, 0.84068E+00_JPRB, 0.88584E+00_JPRB, 0.94413E+00_JPRB, 0.10311E+01_JPRB, 0.11511E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.13251E+01_JPRB, 0.13944E+01_JPRB, 0.14995E+01_JPRB, 0.17137E+01_JPRB, 0.21165E+01_JPRB, 0.27870E+01_JPRB ,&
&  0.31751E+01_JPRB, 0.33648E+01_JPRB, 0.32682E+01_JPRB, 0.30171E+01_JPRB, 0.26478E+01_JPRB, 0.27698E+01_JPRB, 0.29685E+01_JPRB ,&
&  0.30817E+01_JPRB, 0.27128E+01_JPRB, 0.22638E+01_JPRB, 0.22208E+01_JPRB, 0.22233E+01_JPRB, 0.22369E+01_JPRB, 0.22972E+01_JPRB ,&
&  0.23691E+01_JPRB, 0.24257E+01_JPRB, 0.24837E+01_JPRB, 0.25681E+01_JPRB, 0.26413E+01_JPRB, 0.27154E+01_JPRB, 0.28105E+01_JPRB ,&
&  0.29074E+01_JPRB, 0.29931E+01_JPRB, 0.30687E+01_JPRB, 0.31572E+01_JPRB, 0.33131E+01_JPRB, 0.34721E+01_JPRB, 0.35393E+01_JPRB ,&
&  0.35588E+01_JPRB, 0.34308E+01_JPRB, 0.35760E+01_JPRB, 0.38238E+01_JPRB, 0.37931E+01_JPRB, 0.35662E+01_JPRB, 0.35809E+01_JPRB ,&
&  0.38462E+01_JPRB, 0.40464E+01_JPRB, 0.29650E+01_JPRB, 0.39082E+01_JPRB, 0.46001E+01_JPRB, 0.40787E+01_JPRB, 0.35808E+01_JPRB ,&
&  0.31549E+01_JPRB, 0.29232E+01_JPRB, 0.30464E+01_JPRB, 0.31353E+01_JPRB, 0.34129E+01_JPRB, 0.34015E+01_JPRB, 0.30022E+01_JPRB ,&
&  0.32257E+01_JPRB, 0.34469E+01_JPRB, 0.39075E+01_JPRB, 0.40929E+01_JPRB, 0.39052E+01_JPRB, 0.34244E+01_JPRB, 0.27235E+01_JPRB ,&
&  0.21227E+01_JPRB, 0.22164E+01_JPRB, 0.19287E+01_JPRB, 0.18661E+01_JPRB, 0.15407E+01_JPRB, 0.94862E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.10870E+01_JPRB, 0.11313E+01_JPRB, 0.11592E+01_JPRB, 0.11889E+01_JPRB, 0.12090E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.12318E+01_JPRB, 0.12118E+01_JPRB, 0.11961E+01_JPRB, 0.11844E+01_JPRB, 0.11598E+01_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.96982E+00_JPRB, 0.95217E+00_JPRB, 0.93360E+00_JPRB, 0.92386E+00_JPRB, 0.92468E+00_JPRB, 0.91409E+00_JPRB, 0.89250E+00_JPRB ,&
&  0.86284E+00_JPRB, 0.85072E+00_JPRB, 0.82536E+00_JPRB, 0.80244E+00_JPRB, 0.81468E+00_JPRB, 0.88594E+00_JPRB, 0.95835E+00_JPRB ,&
&  0.10009E+01_JPRB, 0.10167E+01_JPRB, 0.10065E+01_JPRB, 0.99143E+00_JPRB, 0.10219E+01_JPRB, 0.10724E+01_JPRB, 0.13945E+01_JPRB ,&
&  0.19518E+01_JPRB, 0.26982E+01_JPRB, 0.32318E+01_JPRB, 0.34944E+01_JPRB, 0.34235E+01_JPRB, 0.30929E+01_JPRB, 0.38180E+01_JPRB ,&
&  0.33761E+01_JPRB, 0.24697E+01_JPRB, 0.21738E+01_JPRB, 0.20556E+01_JPRB, 0.20496E+01_JPRB, 0.20937E+01_JPRB, 0.21700E+01_JPRB ,&
&  0.22562E+01_JPRB, 0.23450E+01_JPRB, 0.24367E+01_JPRB, 0.25456E+01_JPRB, 0.26598E+01_JPRB, 0.27857E+01_JPRB, 0.29116E+01_JPRB ,&
&  0.30618E+01_JPRB, 0.31561E+01_JPRB, 0.32896E+01_JPRB, 0.31864E+01_JPRB, 0.34772E+01_JPRB, 0.37368E+01_JPRB, 0.39060E+01_JPRB ,&
&  0.39417E+01_JPRB, 0.38977E+01_JPRB, 0.41034E+01_JPRB, 0.41939E+01_JPRB, 0.42422E+01_JPRB, 0.41736E+01_JPRB, 0.40412E+01_JPRB ,&
&  0.41504E+01_JPRB, 0.38814E+01_JPRB, 0.28971E+01_JPRB, 0.31971E+01_JPRB, 0.36137E+01_JPRB, 0.29197E+01_JPRB, 0.31511E+01_JPRB ,&
&  0.26764E+01_JPRB, 0.23568E+01_JPRB, 0.21454E+01_JPRB, 0.22035E+01_JPRB, 0.25681E+01_JPRB, 0.29295E+01_JPRB, 0.25510E+01_JPRB ,&
&  0.22044E+01_JPRB, 0.23444E+01_JPRB, 0.26892E+01_JPRB, 0.28799E+01_JPRB, 0.27796E+01_JPRB, 0.24724E+01_JPRB, 0.21157E+01_JPRB ,&
&  0.19210E+01_JPRB, 0.17843E+01_JPRB, 0.16545E+01_JPRB, 0.15068E+01_JPRB, 0.10812E+01_JPRB, 0.87136E+00_JPRB, 0.90455E+00_JPRB ,&
&  0.92865E+00_JPRB, 0.93205E+00_JPRB, 0.95073E+00_JPRB, 0.96828E+00_JPRB, 0.99784E+00_JPRB, 0.10048E+01_JPRB, 0.10278E+01_JPRB ,&
&  0.10405E+01_JPRB, 0.10369E+01_JPRB, 0.10319E+01_JPRB, 0.10324E+01_JPRB, 0.10033E+01_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.10377E+01_JPRB, 0.10421E+01_JPRB, 0.10564E+01_JPRB, 0.10726E+01_JPRB, 0.10942E+01_JPRB, 0.10776E+01_JPRB, 0.10524E+01_JPRB ,&
&  0.10094E+01_JPRB, 0.99939E+00_JPRB, 0.98946E+00_JPRB, 0.98507E+00_JPRB, 0.99965E+00_JPRB, 0.10340E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11183E+01_JPRB, 0.11026E+01_JPRB, 0.10818E+01_JPRB, 0.10287E+01_JPRB, 0.98066E+00_JPRB, 0.95157E+00_JPRB ,&
&  0.99962E+00_JPRB, 0.12877E+01_JPRB, 0.16537E+01_JPRB, 0.20082E+01_JPRB, 0.24790E+01_JPRB, 0.24537E+01_JPRB, 0.30188E+01_JPRB ,&
&  0.36098E+01_JPRB, 0.22532E+01_JPRB, 0.21008E+01_JPRB, 0.18414E+01_JPRB, 0.18285E+01_JPRB, 0.18254E+01_JPRB, 0.18919E+01_JPRB ,&
&  0.19969E+01_JPRB, 0.20669E+01_JPRB, 0.21260E+01_JPRB, 0.21955E+01_JPRB, 0.22872E+01_JPRB, 0.23767E+01_JPRB, 0.24260E+01_JPRB ,&
&  0.24633E+01_JPRB, 0.23928E+01_JPRB, 0.24495E+01_JPRB, 0.28806E+01_JPRB, 0.32977E+01_JPRB, 0.35422E+01_JPRB, 0.37382E+01_JPRB ,&
&  0.39964E+01_JPRB, 0.41878E+01_JPRB, 0.39558E+01_JPRB, 0.39849E+01_JPRB, 0.42268E+01_JPRB, 0.39298E+01_JPRB, 0.40182E+01_JPRB ,&
&  0.41560E+01_JPRB, 0.36920E+01_JPRB, 0.25676E+01_JPRB, 0.29986E+01_JPRB, 0.32411E+01_JPRB, 0.29573E+01_JPRB, 0.29118E+01_JPRB ,&
&  0.24338E+01_JPRB, 0.20859E+01_JPRB, 0.17126E+01_JPRB, 0.15838E+01_JPRB, 0.16615E+01_JPRB, 0.19372E+01_JPRB, 0.20101E+01_JPRB ,&
&  0.17021E+01_JPRB, 0.16692E+01_JPRB, 0.19649E+01_JPRB, 0.23007E+01_JPRB, 0.24384E+01_JPRB, 0.22250E+01_JPRB, 0.18147E+01_JPRB ,&
&  0.18525E+01_JPRB, 0.19139E+01_JPRB, 0.23540E+01_JPRB, 0.21817E+01_JPRB, 0.11570E+01_JPRB, 0.90145E+00_JPRB, 0.85878E+00_JPRB ,&
&  0.87834E+00_JPRB, 0.87951E+00_JPRB, 0.87561E+00_JPRB, 0.89284E+00_JPRB, 0.90469E+00_JPRB, 0.90880E+00_JPRB, 0.94942E+00_JPRB ,&
&  0.99756E+00_JPRB, 0.10027E+01_JPRB, 0.10135E+01_JPRB, 0.10179E+01_JPRB, 0.10264E+01_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.11476E+01_JPRB, 0.11679E+01_JPRB, 0.11961E+01_JPRB, 0.12045E+01_JPRB, 0.12124E+01_JPRB, 0.12038E+01_JPRB, 0.11942E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.11679E+01_JPRB, 0.11725E+01_JPRB, 0.11787E+01_JPRB, 0.11978E+01_JPRB, 0.12182E+01_JPRB, 0.12595E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.12746E+01_JPRB, 0.12872E+01_JPRB, 0.13122E+01_JPRB, 0.13316E+01_JPRB, 0.13204E+01_JPRB, 0.12772E+01_JPRB ,&
&  0.12510E+01_JPRB, 0.12445E+01_JPRB, 0.12626E+01_JPRB, 0.13045E+01_JPRB, 0.15238E+01_JPRB, 0.19008E+01_JPRB, 0.25418E+01_JPRB ,&
&  0.28594E+01_JPRB, 0.22268E+01_JPRB, 0.18568E+01_JPRB, 0.16134E+01_JPRB, 0.15826E+01_JPRB, 0.15892E+01_JPRB, 0.15003E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.14256E+01_JPRB, 0.14381E+01_JPRB, 0.14892E+01_JPRB, 0.15339E+01_JPRB, 0.15163E+01_JPRB, 0.14895E+01_JPRB ,&
&  0.13794E+01_JPRB, 0.13441E+01_JPRB, 0.14341E+01_JPRB, 0.16770E+01_JPRB, 0.22180E+01_JPRB, 0.25778E+01_JPRB, 0.24651E+01_JPRB ,&
&  0.27503E+01_JPRB, 0.32541E+01_JPRB, 0.34631E+01_JPRB, 0.35243E+01_JPRB, 0.34419E+01_JPRB, 0.32923E+01_JPRB, 0.31989E+01_JPRB ,&
&  0.31390E+01_JPRB, 0.28867E+01_JPRB, 0.24692E+01_JPRB, 0.27863E+01_JPRB, 0.28816E+01_JPRB, 0.28944E+01_JPRB, 0.25515E+01_JPRB ,&
&  0.21418E+01_JPRB, 0.18680E+01_JPRB, 0.16874E+01_JPRB, 0.15244E+01_JPRB, 0.14291E+01_JPRB, 0.15039E+01_JPRB, 0.14640E+01_JPRB ,&
&  0.13778E+01_JPRB, 0.14417E+01_JPRB, 0.15636E+01_JPRB, 0.17662E+01_JPRB, 0.22773E+01_JPRB, 0.21111E+01_JPRB, 0.21894E+01_JPRB ,&
&  0.19370E+01_JPRB, 0.19987E+01_JPRB, 0.16151E+01_JPRB, 0.23379E+01_JPRB, 0.11423E+01_JPRB, 0.81865E+00_JPRB, 0.82456E+00_JPRB ,&
&  0.87053E+00_JPRB, 0.89400E+00_JPRB, 0.92937E+00_JPRB, 0.96502E+00_JPRB, 0.97770E+00_JPRB, 0.99164E+00_JPRB, 0.10263E+01_JPRB ,&
&  0.10653E+01_JPRB, 0.10912E+01_JPRB, 0.11150E+01_JPRB, 0.11198E+01_JPRB, 0.11335E+01_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.12122E+01_JPRB, 0.12339E+01_JPRB, 0.12545E+01_JPRB, 0.12718E+01_JPRB, 0.12803E+01_JPRB, 0.12858E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.12899E+01_JPRB, 0.12885E+01_JPRB, 0.12903E+01_JPRB, 0.13090E+01_JPRB, 0.13396E+01_JPRB, 0.13574E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.14179E+01_JPRB, 0.14540E+01_JPRB, 0.14869E+01_JPRB, 0.15331E+01_JPRB, 0.15349E+01_JPRB, 0.15117E+01_JPRB ,&
&  0.14996E+01_JPRB, 0.14828E+01_JPRB, 0.14909E+01_JPRB, 0.15029E+01_JPRB, 0.15241E+01_JPRB, 0.15689E+01_JPRB, 0.18450E+01_JPRB ,&
&  0.21354E+01_JPRB, 0.23287E+01_JPRB, 0.20201E+01_JPRB, 0.18023E+01_JPRB, 0.17795E+01_JPRB, 0.16149E+01_JPRB, 0.15180E+01_JPRB ,&
&  0.13939E+01_JPRB, 0.12736E+01_JPRB, 0.12345E+01_JPRB, 0.12205E+01_JPRB, 0.12730E+01_JPRB, 0.13334E+01_JPRB, 0.13790E+01_JPRB ,&
&  0.14689E+01_JPRB, 0.15416E+01_JPRB, 0.16305E+01_JPRB, 0.16932E+01_JPRB, 0.17468E+01_JPRB, 0.19265E+01_JPRB, 0.18394E+01_JPRB ,&
&  0.18983E+01_JPRB, 0.21206E+01_JPRB, 0.22422E+01_JPRB, 0.23597E+01_JPRB, 0.22451E+01_JPRB, 0.21036E+01_JPRB, 0.18894E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.16563E+01_JPRB, 0.19654E+01_JPRB, 0.21765E+01_JPRB, 0.23353E+01_JPRB, 0.24116E+01_JPRB, 0.21721E+01_JPRB ,&
&  0.19325E+01_JPRB, 0.17648E+01_JPRB, 0.15997E+01_JPRB, 0.14511E+01_JPRB, 0.13711E+01_JPRB, 0.13656E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.12977E+01_JPRB, 0.13213E+01_JPRB, 0.13978E+01_JPRB, 0.15393E+01_JPRB, 0.17535E+01_JPRB, 0.30019E+01_JPRB, 0.36141E+01_JPRB ,&
&  0.22810E+01_JPRB, 0.18647E+01_JPRB, 0.11163E+01_JPRB, 0.16243E+01_JPRB, 0.97427E+00_JPRB, 0.79112E+00_JPRB, 0.84057E+00_JPRB ,&
&  0.94523E+00_JPRB, 0.10284E+01_JPRB, 0.10748E+01_JPRB, 0.11308E+01_JPRB, 0.10575E+01_JPRB, 0.10549E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.11206E+01_JPRB, 0.11467E+01_JPRB, 0.11668E+01_JPRB, 0.11904E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.12304E+01_JPRB, 0.12530E+01_JPRB, 0.12759E+01_JPRB, 0.13028E+01_JPRB, 0.13259E+01_JPRB, 0.13535E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.13858E+01_JPRB, 0.13972E+01_JPRB, 0.14035E+01_JPRB, 0.14209E+01_JPRB, 0.14407E+01_JPRB, 0.14765E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.16293E+01_JPRB, 0.16961E+01_JPRB, 0.17460E+01_JPRB, 0.17913E+01_JPRB, 0.18304E+01_JPRB, 0.18641E+01_JPRB ,&
&  0.18815E+01_JPRB, 0.18715E+01_JPRB, 0.18985E+01_JPRB, 0.19617E+01_JPRB, 0.20277E+01_JPRB, 0.21798E+01_JPRB, 0.20145E+01_JPRB ,&
&  0.23892E+01_JPRB, 0.24122E+01_JPRB, 0.22988E+01_JPRB, 0.21021E+01_JPRB, 0.20432E+01_JPRB, 0.19994E+01_JPRB, 0.19976E+01_JPRB ,&
&  0.20373E+01_JPRB, 0.19564E+01_JPRB, 0.18733E+01_JPRB, 0.17345E+01_JPRB, 0.16535E+01_JPRB, 0.16778E+01_JPRB, 0.17655E+01_JPRB ,&
&  0.18137E+01_JPRB, 0.18748E+01_JPRB, 0.18908E+01_JPRB, 0.18853E+01_JPRB, 0.18888E+01_JPRB, 0.18757E+01_JPRB, 0.17749E+01_JPRB ,&
&  0.16894E+01_JPRB, 0.16630E+01_JPRB, 0.14527E+01_JPRB, 0.14536E+01_JPRB, 0.14522E+01_JPRB, 0.13372E+01_JPRB, 0.11489E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10524E+01_JPRB, 0.11845E+01_JPRB, 0.16348E+01_JPRB, 0.18906E+01_JPRB, 0.19488E+01_JPRB, 0.18524E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.15015E+01_JPRB, 0.13072E+01_JPRB, 0.11984E+01_JPRB, 0.11822E+01_JPRB, 0.12367E+01_JPRB, 0.12635E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.12683E+01_JPRB, 0.12951E+01_JPRB, 0.13587E+01_JPRB, 0.16508E+01_JPRB, 0.21941E+01_JPRB, 0.33422E+01_JPRB ,&
&  0.23889E+01_JPRB, 0.13887E+01_JPRB, 0.13133E+01_JPRB, 0.13309E+01_JPRB, 0.93754E+00_JPRB, 0.74900E+00_JPRB, 0.82758E+00_JPRB ,&
&  0.10483E+01_JPRB, 0.11845E+01_JPRB, 0.11903E+01_JPRB, 0.11828E+01_JPRB, 0.11420E+01_JPRB, 0.11544E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.11580E+01_JPRB, 0.11729E+01_JPRB, 0.11877E+01_JPRB, 0.12083E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.12305E+01_JPRB, 0.12601E+01_JPRB, 0.12889E+01_JPRB, 0.13182E+01_JPRB, 0.13423E+01_JPRB, 0.13738E+01_JPRB, 0.13899E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.14241E+01_JPRB, 0.14508E+01_JPRB, 0.14802E+01_JPRB, 0.15168E+01_JPRB, 0.15604E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.16820E+01_JPRB, 0.17421E+01_JPRB, 0.18125E+01_JPRB, 0.18705E+01_JPRB, 0.19045E+01_JPRB, 0.19522E+01_JPRB, 0.19993E+01_JPRB ,&
&  0.20141E+01_JPRB, 0.20073E+01_JPRB, 0.20486E+01_JPRB, 0.21165E+01_JPRB, 0.21448E+01_JPRB, 0.22859E+01_JPRB, 0.19215E+01_JPRB ,&
&  0.24764E+01_JPRB, 0.26272E+01_JPRB, 0.26683E+01_JPRB, 0.24737E+01_JPRB, 0.23661E+01_JPRB, 0.23890E+01_JPRB, 0.24446E+01_JPRB ,&
&  0.25361E+01_JPRB, 0.26202E+01_JPRB, 0.25139E+01_JPRB, 0.23475E+01_JPRB, 0.22002E+01_JPRB, 0.22025E+01_JPRB, 0.22363E+01_JPRB ,&
&  0.22652E+01_JPRB, 0.22904E+01_JPRB, 0.22802E+01_JPRB, 0.22553E+01_JPRB, 0.22323E+01_JPRB, 0.21930E+01_JPRB, 0.21123E+01_JPRB ,&
&  0.20349E+01_JPRB, 0.19559E+01_JPRB, 0.17268E+01_JPRB, 0.18309E+01_JPRB, 0.17110E+01_JPRB, 0.14453E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.10525E+01_JPRB, 0.10467E+01_JPRB, 0.12518E+01_JPRB, 0.14293E+01_JPRB, 0.14897E+01_JPRB, 0.14958E+01_JPRB, 0.14434E+01_JPRB ,&
&  0.12922E+01_JPRB, 0.11585E+01_JPRB, 0.10775E+01_JPRB, 0.10628E+01_JPRB, 0.10894E+01_JPRB, 0.11385E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.11810E+01_JPRB, 0.11914E+01_JPRB, 0.11909E+01_JPRB, 0.11936E+01_JPRB, 0.13300E+01_JPRB, 0.16182E+01_JPRB, 0.27210E+01_JPRB ,&
&  0.21040E+01_JPRB, 0.15159E+01_JPRB, 0.20260E+01_JPRB, 0.78264E+00_JPRB, 0.80159E+00_JPRB, 0.69829E+00_JPRB, 0.77261E+00_JPRB ,&
&  0.91608E+00_JPRB, 0.12647E+01_JPRB, 0.16722E+01_JPRB, 0.14715E+01_JPRB, 0.13619E+01_JPRB, 0.12684E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.11632E+01_JPRB, 0.11643E+01_JPRB, 0.11734E+01_JPRB, 0.11807E+01_JPRB, 0.12017E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.12464E+01_JPRB, 0.12811E+01_JPRB, 0.12980E+01_JPRB, 0.13297E+01_JPRB, 0.13224E+01_JPRB, 0.13509E+01_JPRB, 0.13628E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.14220E+01_JPRB, 0.14602E+01_JPRB, 0.14992E+01_JPRB, 0.15504E+01_JPRB, 0.15999E+01_JPRB, 0.16550E+01_JPRB ,&
&  0.17199E+01_JPRB, 0.17934E+01_JPRB, 0.18571E+01_JPRB, 0.19410E+01_JPRB, 0.19874E+01_JPRB, 0.20663E+01_JPRB, 0.21414E+01_JPRB ,&
&  0.22512E+01_JPRB, 0.22674E+01_JPRB, 0.22989E+01_JPRB, 0.23472E+01_JPRB, 0.23574E+01_JPRB, 0.25088E+01_JPRB, 0.18515E+01_JPRB ,&
&  0.25832E+01_JPRB, 0.28169E+01_JPRB, 0.29730E+01_JPRB, 0.28465E+01_JPRB, 0.26800E+01_JPRB, 0.26434E+01_JPRB, 0.26632E+01_JPRB ,&
&  0.27331E+01_JPRB, 0.27735E+01_JPRB, 0.26677E+01_JPRB, 0.27303E+01_JPRB, 0.27494E+01_JPRB, 0.26758E+01_JPRB, 0.27421E+01_JPRB ,&
&  0.27888E+01_JPRB, 0.28315E+01_JPRB, 0.28619E+01_JPRB, 0.28921E+01_JPRB, 0.29112E+01_JPRB, 0.29726E+01_JPRB, 0.30042E+01_JPRB ,&
&  0.29758E+01_JPRB, 0.29679E+01_JPRB, 0.29405E+01_JPRB, 0.27758E+01_JPRB, 0.25962E+01_JPRB, 0.21841E+01_JPRB, 0.18068E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.11763E+01_JPRB, 0.12458E+01_JPRB, 0.13309E+01_JPRB, 0.12931E+01_JPRB, 0.12438E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.10383E+01_JPRB, 0.10118E+01_JPRB, 0.99999E+00_JPRB, 0.10228E+01_JPRB, 0.10609E+01_JPRB, 0.10889E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.11292E+01_JPRB, 0.10933E+01_JPRB, 0.10713E+01_JPRB, 0.11748E+01_JPRB, 0.14794E+01_JPRB, 0.16607E+01_JPRB ,&
&  0.19377E+01_JPRB, 0.14470E+01_JPRB, 0.12400E+01_JPRB, 0.11075E+01_JPRB, 0.80708E+00_JPRB, 0.73885E+00_JPRB, 0.88689E+00_JPRB ,&
&  0.14110E+01_JPRB, 0.17617E+01_JPRB, 0.24480E+01_JPRB, 0.40941E+01_JPRB, 0.29136E+01_JPRB, 0.22089E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.11668E+01_JPRB, 0.11689E+01_JPRB, 0.11714E+01_JPRB, 0.12059E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.11238E+01_JPRB, 0.11659E+01_JPRB, 0.12107E+01_JPRB, 0.12559E+01_JPRB, 0.12866E+01_JPRB, 0.13041E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.13493E+01_JPRB, 0.13474E+01_JPRB, 0.13850E+01_JPRB, 0.13832E+01_JPRB, 0.14217E+01_JPRB, 0.14407E+01_JPRB, 0.14877E+01_JPRB ,&
&  0.15529E+01_JPRB, 0.16324E+01_JPRB, 0.17232E+01_JPRB, 0.18021E+01_JPRB, 0.18810E+01_JPRB, 0.20120E+01_JPRB, 0.21770E+01_JPRB ,&
&  0.23618E+01_JPRB, 0.24809E+01_JPRB, 0.25780E+01_JPRB, 0.26464E+01_JPRB, 0.26101E+01_JPRB, 0.25618E+01_JPRB, 0.18364E+01_JPRB ,&
&  0.25279E+01_JPRB, 0.29320E+01_JPRB, 0.32862E+01_JPRB, 0.32949E+01_JPRB, 0.30715E+01_JPRB, 0.28864E+01_JPRB, 0.28902E+01_JPRB ,&
&  0.28860E+01_JPRB, 0.27885E+01_JPRB, 0.26470E+01_JPRB, 0.26186E+01_JPRB, 0.29053E+01_JPRB, 0.28174E+01_JPRB, 0.28870E+01_JPRB ,&
&  0.29530E+01_JPRB, 0.30368E+01_JPRB, 0.31443E+01_JPRB, 0.32769E+01_JPRB, 0.34076E+01_JPRB, 0.35366E+01_JPRB, 0.36792E+01_JPRB ,&
&  0.37850E+01_JPRB, 0.38176E+01_JPRB, 0.38430E+01_JPRB, 0.34223E+01_JPRB, 0.31428E+01_JPRB, 0.31309E+01_JPRB, 0.23835E+01_JPRB ,&
&  0.14999E+01_JPRB, 0.12817E+01_JPRB, 0.13199E+01_JPRB, 0.13332E+01_JPRB, 0.12758E+01_JPRB, 0.11509E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10024E+01_JPRB, 0.98226E+00_JPRB, 0.97993E+00_JPRB, 0.10214E+01_JPRB, 0.10552E+01_JPRB, 0.10867E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.11003E+01_JPRB, 0.10615E+01_JPRB, 0.10580E+01_JPRB, 0.11756E+01_JPRB, 0.14927E+01_JPRB, 0.19891E+01_JPRB ,&
&  0.21486E+01_JPRB, 0.12540E+01_JPRB, 0.11481E+01_JPRB, 0.97073E+00_JPRB, 0.87579E+00_JPRB, 0.88419E+00_JPRB, 0.10531E+01_JPRB ,&
&  0.15345E+01_JPRB, 0.23160E+01_JPRB, 0.25221E+01_JPRB, 0.21482E+01_JPRB, 0.25929E+01_JPRB, 0.19048E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.11181E+01_JPRB, 0.10964E+01_JPRB, 0.10671E+01_JPRB, 0.10639E+01_JPRB, 0.10818E+01_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.93379E+00_JPRB, 0.96371E+00_JPRB, 0.99680E+00_JPRB, 0.10441E+01_JPRB, 0.10981E+01_JPRB, 0.11389E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.12347E+01_JPRB, 0.12699E+01_JPRB, 0.12593E+01_JPRB, 0.12939E+01_JPRB, 0.13108E+01_JPRB, 0.13376E+01_JPRB ,&
&  0.13755E+01_JPRB, 0.14324E+01_JPRB, 0.15075E+01_JPRB, 0.16005E+01_JPRB, 0.17283E+01_JPRB, 0.18784E+01_JPRB, 0.20600E+01_JPRB ,&
&  0.22784E+01_JPRB, 0.24795E+01_JPRB, 0.26744E+01_JPRB, 0.28109E+01_JPRB, 0.29142E+01_JPRB, 0.29683E+01_JPRB, 0.30027E+01_JPRB ,&
&  0.16973E+01_JPRB, 0.27585E+01_JPRB, 0.31180E+01_JPRB, 0.33728E+01_JPRB, 0.35260E+01_JPRB, 0.33432E+01_JPRB, 0.33346E+01_JPRB ,&
&  0.31000E+01_JPRB, 0.27739E+01_JPRB, 0.26480E+01_JPRB, 0.25666E+01_JPRB, 0.26963E+01_JPRB, 0.26679E+01_JPRB, 0.26881E+01_JPRB ,&
&  0.27186E+01_JPRB, 0.28056E+01_JPRB, 0.29260E+01_JPRB, 0.30652E+01_JPRB, 0.32414E+01_JPRB, 0.34550E+01_JPRB, 0.37272E+01_JPRB ,&
&  0.40112E+01_JPRB, 0.42639E+01_JPRB, 0.44857E+01_JPRB, 0.39855E+01_JPRB, 0.35293E+01_JPRB, 0.35928E+01_JPRB, 0.30299E+01_JPRB ,&
&  0.18425E+01_JPRB, 0.13478E+01_JPRB, 0.13942E+01_JPRB, 0.14936E+01_JPRB, 0.13924E+01_JPRB, 0.12770E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.10334E+01_JPRB, 0.10738E+01_JPRB, 0.10613E+01_JPRB, 0.11108E+01_JPRB, 0.11203E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.11663E+01_JPRB, 0.11506E+01_JPRB, 0.11502E+01_JPRB, 0.11464E+01_JPRB, 0.11453E+01_JPRB, 0.11534E+01_JPRB ,&
&  0.11664E+01_JPRB, 0.10999E+01_JPRB, 0.10407E+01_JPRB, 0.10303E+01_JPRB, 0.10037E+01_JPRB, 0.98642E+00_JPRB, 0.10715E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.14060E+01_JPRB, 0.16954E+01_JPRB, 0.17619E+01_JPRB, 0.16575E+01_JPRB, 0.12796E+01_JPRB, 0.97686E+00_JPRB ,&
&  0.97928E+00_JPRB, 0.94117E+00_JPRB, 0.91172E+00_JPRB, 0.89703E+00_JPRB, 0.90795E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.84242E+00_JPRB, 0.86678E+00_JPRB, 0.86442E+00_JPRB, 0.89797E+00_JPRB, 0.92299E+00_JPRB, 0.95880E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.10672E+01_JPRB, 0.11158E+01_JPRB, 0.11208E+01_JPRB, 0.11605E+01_JPRB, 0.11862E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.12666E+01_JPRB, 0.13094E+01_JPRB, 0.13622E+01_JPRB, 0.14422E+01_JPRB, 0.15705E+01_JPRB, 0.17163E+01_JPRB ,&
&  0.18958E+01_JPRB, 0.21138E+01_JPRB, 0.23978E+01_JPRB, 0.26877E+01_JPRB, 0.29578E+01_JPRB, 0.31078E+01_JPRB, 0.31270E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.17298E+01_JPRB, 0.27272E+01_JPRB, 0.31301E+01_JPRB, 0.34231E+01_JPRB, 0.35154E+01_JPRB, 0.34533E+01_JPRB ,&
&  0.31124E+01_JPRB, 0.25983E+01_JPRB, 0.23972E+01_JPRB, 0.26027E+01_JPRB, 0.24857E+01_JPRB, 0.24837E+01_JPRB, 0.24708E+01_JPRB ,&
&  0.24734E+01_JPRB, 0.25145E+01_JPRB, 0.25910E+01_JPRB, 0.26904E+01_JPRB, 0.28635E+01_JPRB, 0.30998E+01_JPRB, 0.34249E+01_JPRB ,&
&  0.38286E+01_JPRB, 0.42465E+01_JPRB, 0.46178E+01_JPRB, 0.40671E+01_JPRB, 0.39784E+01_JPRB, 0.39376E+01_JPRB, 0.40564E+01_JPRB ,&
&  0.23120E+01_JPRB, 0.15137E+01_JPRB, 0.15687E+01_JPRB, 0.16779E+01_JPRB, 0.15284E+01_JPRB, 0.14461E+01_JPRB, 0.13753E+01_JPRB ,&
&  0.12866E+01_JPRB, 0.12097E+01_JPRB, 0.11964E+01_JPRB, 0.11743E+01_JPRB, 0.11750E+01_JPRB, 0.11999E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.12878E+01_JPRB, 0.12910E+01_JPRB, 0.12873E+01_JPRB, 0.12535E+01_JPRB, 0.12160E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.11943E+01_JPRB, 0.11644E+01_JPRB, 0.11541E+01_JPRB, 0.11807E+01_JPRB, 0.11993E+01_JPRB, 0.11962E+01_JPRB, 0.11930E+01_JPRB ,&
&  0.11612E+01_JPRB, 0.11607E+01_JPRB, 0.12702E+01_JPRB, 0.12985E+01_JPRB, 0.12603E+01_JPRB, 0.10703E+01_JPRB, 0.90286E+00_JPRB ,&
&  0.88667E+00_JPRB, 0.86072E+00_JPRB, 0.83805E+00_JPRB, 0.83454E+00_JPRB, 0.83249E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.77481E+00_JPRB, 0.76040E+00_JPRB, 0.74576E+00_JPRB, 0.75800E+00_JPRB, 0.77213E+00_JPRB, 0.79187E+00_JPRB, 0.85031E+00_JPRB ,&
&  0.90245E+00_JPRB, 0.92175E+00_JPRB, 0.95079E+00_JPRB, 0.96551E+00_JPRB, 0.99531E+00_JPRB, 0.10098E+01_JPRB, 0.10425E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.10913E+01_JPRB, 0.11358E+01_JPRB, 0.11803E+01_JPRB, 0.12188E+01_JPRB, 0.12739E+01_JPRB, 0.13554E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.15563E+01_JPRB, 0.17395E+01_JPRB, 0.20568E+01_JPRB, 0.25587E+01_JPRB, 0.31673E+01_JPRB, 0.36619E+01_JPRB ,&
&  0.28564E+01_JPRB, 0.11162E+01_JPRB, 0.22842E+01_JPRB, 0.29983E+01_JPRB, 0.31239E+01_JPRB, 0.32905E+01_JPRB, 0.31374E+01_JPRB ,&
&  0.28083E+01_JPRB, 0.24705E+01_JPRB, 0.22272E+01_JPRB, 0.23959E+01_JPRB, 0.23465E+01_JPRB, 0.23277E+01_JPRB, 0.23141E+01_JPRB ,&
&  0.22851E+01_JPRB, 0.22817E+01_JPRB, 0.22950E+01_JPRB, 0.23490E+01_JPRB, 0.24745E+01_JPRB, 0.26890E+01_JPRB, 0.29875E+01_JPRB ,&
&  0.34048E+01_JPRB, 0.39800E+01_JPRB, 0.45411E+01_JPRB, 0.41833E+01_JPRB, 0.43002E+01_JPRB, 0.38460E+01_JPRB, 0.33484E+01_JPRB ,&
&  0.21804E+01_JPRB, 0.16590E+01_JPRB, 0.16840E+01_JPRB, 0.18414E+01_JPRB, 0.16899E+01_JPRB, 0.14927E+01_JPRB, 0.15580E+01_JPRB ,&
&  0.14599E+01_JPRB, 0.13925E+01_JPRB, 0.13477E+01_JPRB, 0.13103E+01_JPRB, 0.13032E+01_JPRB, 0.13051E+01_JPRB, 0.13109E+01_JPRB ,&
&  0.13145E+01_JPRB, 0.13377E+01_JPRB, 0.13585E+01_JPRB, 0.13528E+01_JPRB, 0.13240E+01_JPRB, 0.13059E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.13236E+01_JPRB, 0.13196E+01_JPRB, 0.13360E+01_JPRB, 0.13529E+01_JPRB, 0.13563E+01_JPRB, 0.13119E+01_JPRB, 0.12958E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.12422E+01_JPRB, 0.12064E+01_JPRB, 0.11516E+01_JPRB, 0.11075E+01_JPRB, 0.10182E+01_JPRB, 0.91782E+00_JPRB ,&
&  0.88402E+00_JPRB, 0.84718E+00_JPRB, 0.82329E+00_JPRB, 0.80837E+00_JPRB, 0.79334E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.83442E+00_JPRB, 0.81375E+00_JPRB, 0.77993E+00_JPRB, 0.73172E+00_JPRB, 0.72791E+00_JPRB, 0.72622E+00_JPRB, 0.73790E+00_JPRB ,&
&  0.78733E+00_JPRB, 0.84736E+00_JPRB, 0.86447E+00_JPRB, 0.86503E+00_JPRB, 0.84396E+00_JPRB, 0.85335E+00_JPRB, 0.87227E+00_JPRB ,&
&  0.90246E+00_JPRB, 0.93194E+00_JPRB, 0.97382E+00_JPRB, 0.10277E+01_JPRB, 0.10759E+01_JPRB, 0.11168E+01_JPRB, 0.11526E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.12037E+01_JPRB, 0.12448E+01_JPRB, 0.13252E+01_JPRB, 0.15118E+01_JPRB, 0.19382E+01_JPRB, 0.27000E+01_JPRB ,&
&  0.44181E+01_JPRB, 0.37220E+01_JPRB, 0.16055E+01_JPRB, 0.27729E+01_JPRB, 0.30864E+01_JPRB, 0.33310E+01_JPRB, 0.28044E+01_JPRB ,&
&  0.26549E+01_JPRB, 0.23045E+01_JPRB, 0.20218E+01_JPRB, 0.21397E+01_JPRB, 0.20923E+01_JPRB, 0.20667E+01_JPRB, 0.20491E+01_JPRB ,&
&  0.19937E+01_JPRB, 0.19943E+01_JPRB, 0.20069E+01_JPRB, 0.20304E+01_JPRB, 0.20824E+01_JPRB, 0.22295E+01_JPRB, 0.25236E+01_JPRB ,&
&  0.29123E+01_JPRB, 0.34401E+01_JPRB, 0.42244E+01_JPRB, 0.45324E+01_JPRB, 0.49159E+01_JPRB, 0.37530E+01_JPRB, 0.29759E+01_JPRB ,&
&  0.20694E+01_JPRB, 0.19249E+01_JPRB, 0.20125E+01_JPRB, 0.20169E+01_JPRB, 0.18532E+01_JPRB, 0.17003E+01_JPRB, 0.17688E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.15561E+01_JPRB, 0.15076E+01_JPRB, 0.14544E+01_JPRB, 0.14179E+01_JPRB, 0.14010E+01_JPRB, 0.13891E+01_JPRB ,&
&  0.13783E+01_JPRB, 0.13901E+01_JPRB, 0.14050E+01_JPRB, 0.14141E+01_JPRB, 0.14128E+01_JPRB, 0.13980E+01_JPRB, 0.14053E+01_JPRB ,&
&  0.14153E+01_JPRB, 0.14292E+01_JPRB, 0.14468E+01_JPRB, 0.14664E+01_JPRB, 0.14270E+01_JPRB, 0.13628E+01_JPRB, 0.13620E+01_JPRB ,&
&  0.13418E+01_JPRB, 0.13356E+01_JPRB, 0.12893E+01_JPRB, 0.12752E+01_JPRB, 0.11831E+01_JPRB, 0.10776E+01_JPRB, 0.97860E+00_JPRB ,&
&  0.95606E+00_JPRB, 0.93538E+00_JPRB, 0.88646E+00_JPRB, 0.84011E+00_JPRB, 0.83086E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.90842E+00_JPRB, 0.91741E+00_JPRB, 0.90970E+00_JPRB, 0.86079E+00_JPRB, 0.81636E+00_JPRB, 0.77396E+00_JPRB, 0.74481E+00_JPRB ,&
&  0.73943E+00_JPRB, 0.74437E+00_JPRB, 0.77490E+00_JPRB, 0.80204E+00_JPRB, 0.79461E+00_JPRB, 0.76924E+00_JPRB, 0.76291E+00_JPRB ,&
&  0.77596E+00_JPRB, 0.80800E+00_JPRB, 0.84086E+00_JPRB, 0.89189E+00_JPRB, 0.93926E+00_JPRB, 0.97516E+00_JPRB, 0.10134E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.10157E+01_JPRB, 0.10275E+01_JPRB, 0.10486E+01_JPRB, 0.10925E+01_JPRB, 0.12110E+01_JPRB, 0.14087E+01_JPRB ,&
&  0.20499E+01_JPRB, 0.37667E+01_JPRB, 0.20525E+01_JPRB, 0.29103E+01_JPRB, 0.33863E+01_JPRB, 0.34051E+01_JPRB, 0.29467E+01_JPRB ,&
&  0.26415E+01_JPRB, 0.22636E+01_JPRB, 0.20260E+01_JPRB, 0.19851E+01_JPRB, 0.18020E+01_JPRB, 0.17618E+01_JPRB, 0.17315E+01_JPRB ,&
&  0.17288E+01_JPRB, 0.17509E+01_JPRB, 0.17682E+01_JPRB, 0.17847E+01_JPRB, 0.17798E+01_JPRB, 0.18082E+01_JPRB, 0.19585E+01_JPRB ,&
&  0.22932E+01_JPRB, 0.27798E+01_JPRB, 0.36152E+01_JPRB, 0.43220E+01_JPRB, 0.51893E+01_JPRB, 0.41039E+01_JPRB, 0.31167E+01_JPRB ,&
&  0.23649E+01_JPRB, 0.22924E+01_JPRB, 0.23218E+01_JPRB, 0.22949E+01_JPRB, 0.20733E+01_JPRB, 0.20451E+01_JPRB, 0.21025E+01_JPRB ,&
&  0.19145E+01_JPRB, 0.17589E+01_JPRB, 0.16775E+01_JPRB, 0.16157E+01_JPRB, 0.15458E+01_JPRB, 0.14928E+01_JPRB, 0.14586E+01_JPRB ,&
&  0.14361E+01_JPRB, 0.14316E+01_JPRB, 0.14467E+01_JPRB, 0.14595E+01_JPRB, 0.14753E+01_JPRB, 0.14763E+01_JPRB, 0.14735E+01_JPRB ,&
&  0.14597E+01_JPRB, 0.14592E+01_JPRB, 0.14670E+01_JPRB, 0.14666E+01_JPRB, 0.14411E+01_JPRB, 0.14003E+01_JPRB, 0.13925E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.14171E+01_JPRB, 0.13669E+01_JPRB, 0.13207E+01_JPRB, 0.13217E+01_JPRB, 0.12288E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.12481E+01_JPRB, 0.12750E+01_JPRB, 0.92303E+00_JPRB, 0.90518E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.96118E+00_JPRB, 0.96016E+00_JPRB, 0.94600E+00_JPRB, 0.94053E+00_JPRB, 0.90355E+00_JPRB, 0.85910E+00_JPRB, 0.81813E+00_JPRB ,&
&  0.78993E+00_JPRB, 0.77258E+00_JPRB, 0.77029E+00_JPRB, 0.76607E+00_JPRB, 0.76603E+00_JPRB, 0.75773E+00_JPRB, 0.75288E+00_JPRB ,&
&  0.74240E+00_JPRB, 0.75206E+00_JPRB, 0.77449E+00_JPRB, 0.81424E+00_JPRB, 0.85933E+00_JPRB, 0.87299E+00_JPRB, 0.88663E+00_JPRB ,&
&  0.89671E+00_JPRB, 0.89780E+00_JPRB, 0.90134E+00_JPRB, 0.91452E+00_JPRB, 0.94121E+00_JPRB, 0.98539E+00_JPRB, 0.10754E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.30801E+01_JPRB, 0.47124E+01_JPRB, 0.52769E+01_JPRB, 0.45763E+01_JPRB, 0.38404E+01_JPRB, 0.33773E+01_JPRB ,&
&  0.32747E+01_JPRB, 0.29656E+01_JPRB, 0.25525E+01_JPRB, 0.20663E+01_JPRB, 0.17312E+01_JPRB, 0.16059E+01_JPRB, 0.15661E+01_JPRB ,&
&  0.16136E+01_JPRB, 0.16408E+01_JPRB, 0.16464E+01_JPRB, 0.16353E+01_JPRB, 0.15972E+01_JPRB, 0.15717E+01_JPRB, 0.15868E+01_JPRB ,&
&  0.17114E+01_JPRB, 0.20249E+01_JPRB, 0.26328E+01_JPRB, 0.36790E+01_JPRB, 0.48200E+01_JPRB, 0.43404E+01_JPRB, 0.35604E+01_JPRB ,&
&  0.33119E+01_JPRB, 0.30598E+01_JPRB, 0.28270E+01_JPRB, 0.26942E+01_JPRB, 0.22227E+01_JPRB, 0.26129E+01_JPRB, 0.24902E+01_JPRB ,&
&  0.23321E+01_JPRB, 0.22210E+01_JPRB, 0.20547E+01_JPRB, 0.18743E+01_JPRB, 0.17369E+01_JPRB, 0.16072E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.14704E+01_JPRB, 0.14731E+01_JPRB, 0.14867E+01_JPRB, 0.15142E+01_JPRB, 0.15481E+01_JPRB, 0.15653E+01_JPRB, 0.15515E+01_JPRB ,&
&  0.15005E+01_JPRB, 0.14589E+01_JPRB, 0.13824E+01_JPRB, 0.13767E+01_JPRB, 0.13718E+01_JPRB, 0.13347E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.13312E+01_JPRB, 0.13782E+01_JPRB, 0.13806E+01_JPRB, 0.13591E+01_JPRB, 0.13314E+01_JPRB, 0.13080E+01_JPRB, 0.11969E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.11776E+01_JPRB, 0.10988E+01_JPRB, 0.10150E+01_JPRB, 0.98406E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.10140E+01_JPRB, 0.98028E+00_JPRB, 0.95977E+00_JPRB, 0.95652E+00_JPRB, 0.93374E+00_JPRB, 0.91662E+00_JPRB, 0.90697E+00_JPRB ,&
&  0.85995E+00_JPRB, 0.82296E+00_JPRB, 0.79017E+00_JPRB, 0.77530E+00_JPRB, 0.77299E+00_JPRB, 0.76730E+00_JPRB, 0.77290E+00_JPRB ,&
&  0.76704E+00_JPRB, 0.75523E+00_JPRB, 0.75747E+00_JPRB, 0.78246E+00_JPRB, 0.81381E+00_JPRB, 0.82075E+00_JPRB, 0.83816E+00_JPRB ,&
&  0.85834E+00_JPRB, 0.85890E+00_JPRB, 0.85090E+00_JPRB, 0.85104E+00_JPRB, 0.85888E+00_JPRB, 0.86680E+00_JPRB, 0.91769E+00_JPRB ,&
&  0.11667E+01_JPRB, 0.31181E+01_JPRB, 0.31367E+01_JPRB, 0.60409E+01_JPRB, 0.52279E+01_JPRB, 0.44630E+01_JPRB, 0.35117E+01_JPRB ,&
&  0.35637E+01_JPRB, 0.39164E+01_JPRB, 0.29545E+01_JPRB, 0.23321E+01_JPRB, 0.19113E+01_JPRB, 0.16345E+01_JPRB, 0.15353E+01_JPRB ,&
&  0.15488E+01_JPRB, 0.15894E+01_JPRB, 0.16108E+01_JPRB, 0.15633E+01_JPRB, 0.15063E+01_JPRB, 0.14760E+01_JPRB, 0.14700E+01_JPRB ,&
&  0.14813E+01_JPRB, 0.15606E+01_JPRB, 0.18497E+01_JPRB, 0.24528E+01_JPRB, 0.38651E+01_JPRB, 0.45297E+01_JPRB, 0.41285E+01_JPRB ,&
&  0.39647E+01_JPRB, 0.41064E+01_JPRB, 0.37567E+01_JPRB, 0.33648E+01_JPRB, 0.28521E+01_JPRB, 0.28530E+01_JPRB, 0.27238E+01_JPRB ,&
&  0.25872E+01_JPRB, 0.25492E+01_JPRB, 0.24707E+01_JPRB, 0.23044E+01_JPRB, 0.21032E+01_JPRB, 0.18913E+01_JPRB, 0.16947E+01_JPRB ,&
&  0.15573E+01_JPRB, 0.14925E+01_JPRB, 0.14634E+01_JPRB, 0.14914E+01_JPRB, 0.15539E+01_JPRB, 0.16009E+01_JPRB, 0.16154E+01_JPRB ,&
&  0.15761E+01_JPRB, 0.15139E+01_JPRB, 0.13736E+01_JPRB, 0.13022E+01_JPRB, 0.13008E+01_JPRB, 0.12669E+01_JPRB, 0.12489E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.13366E+01_JPRB, 0.13400E+01_JPRB, 0.13232E+01_JPRB, 0.13158E+01_JPRB, 0.13452E+01_JPRB, 0.12477E+01_JPRB ,&
&  0.11891E+01_JPRB, 0.11907E+01_JPRB, 0.11803E+01_JPRB, 0.11088E+01_JPRB, 0.10634E+01_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.10801E+01_JPRB, 0.10262E+01_JPRB, 0.98455E+00_JPRB, 0.96527E+00_JPRB, 0.95670E+00_JPRB, 0.94593E+00_JPRB, 0.93415E+00_JPRB ,&
&  0.90521E+00_JPRB, 0.86894E+00_JPRB, 0.83406E+00_JPRB, 0.80378E+00_JPRB, 0.79744E+00_JPRB, 0.79111E+00_JPRB, 0.78835E+00_JPRB ,&
&  0.78134E+00_JPRB, 0.77097E+00_JPRB, 0.76362E+00_JPRB, 0.77275E+00_JPRB, 0.78466E+00_JPRB, 0.79510E+00_JPRB, 0.81405E+00_JPRB ,&
&  0.82289E+00_JPRB, 0.83365E+00_JPRB, 0.83659E+00_JPRB, 0.83253E+00_JPRB, 0.82458E+00_JPRB, 0.81739E+00_JPRB, 0.83001E+00_JPRB ,&
&  0.11364E+01_JPRB, 0.39603E+01_JPRB, 0.28548E+01_JPRB, 0.55451E+01_JPRB, 0.48196E+01_JPRB, 0.43518E+01_JPRB, 0.34844E+01_JPRB ,&
&  0.33064E+01_JPRB, 0.36119E+01_JPRB, 0.30490E+01_JPRB, 0.25364E+01_JPRB, 0.21889E+01_JPRB, 0.19086E+01_JPRB, 0.17103E+01_JPRB ,&
&  0.16295E+01_JPRB, 0.15951E+01_JPRB, 0.15568E+01_JPRB, 0.15024E+01_JPRB, 0.14697E+01_JPRB, 0.14474E+01_JPRB, 0.14454E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.14492E+01_JPRB, 0.15435E+01_JPRB, 0.19294E+01_JPRB, 0.27656E+01_JPRB, 0.39359E+01_JPRB, 0.47143E+01_JPRB ,&
&  0.48399E+01_JPRB, 0.52631E+01_JPRB, 0.47337E+01_JPRB, 0.40926E+01_JPRB, 0.35993E+01_JPRB, 0.33042E+01_JPRB, 0.30437E+01_JPRB ,&
&  0.27804E+01_JPRB, 0.26226E+01_JPRB, 0.25725E+01_JPRB, 0.25328E+01_JPRB, 0.24482E+01_JPRB, 0.22919E+01_JPRB, 0.20875E+01_JPRB ,&
&  0.18749E+01_JPRB, 0.17219E+01_JPRB, 0.16141E+01_JPRB, 0.15636E+01_JPRB, 0.15610E+01_JPRB, 0.15876E+01_JPRB, 0.16046E+01_JPRB ,&
&  0.15637E+01_JPRB, 0.15177E+01_JPRB, 0.14065E+01_JPRB, 0.13088E+01_JPRB, 0.12930E+01_JPRB, 0.12603E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.13110E+01_JPRB, 0.12997E+01_JPRB, 0.12995E+01_JPRB, 0.12793E+01_JPRB, 0.13601E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.11909E+01_JPRB, 0.11777E+01_JPRB, 0.11503E+01_JPRB, 0.11266E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.11280E+01_JPRB, 0.10872E+01_JPRB, 0.10445E+01_JPRB, 0.10100E+01_JPRB, 0.98307E+00_JPRB, 0.96838E+00_JPRB, 0.95498E+00_JPRB ,&
&  0.92834E+00_JPRB, 0.89895E+00_JPRB, 0.87459E+00_JPRB, 0.85338E+00_JPRB, 0.83752E+00_JPRB, 0.82855E+00_JPRB, 0.81467E+00_JPRB ,&
&  0.79896E+00_JPRB, 0.77446E+00_JPRB, 0.77660E+00_JPRB, 0.78831E+00_JPRB, 0.79305E+00_JPRB, 0.79809E+00_JPRB, 0.82065E+00_JPRB ,&
&  0.81752E+00_JPRB, 0.81835E+00_JPRB, 0.82211E+00_JPRB, 0.82385E+00_JPRB, 0.81339E+00_JPRB, 0.81361E+00_JPRB, 0.81142E+00_JPRB ,&
&  0.10794E+01_JPRB, 0.15109E+01_JPRB, 0.34383E+01_JPRB, 0.48104E+01_JPRB, 0.43641E+01_JPRB, 0.39042E+01_JPRB, 0.35294E+01_JPRB ,&
&  0.34081E+01_JPRB, 0.32732E+01_JPRB, 0.28516E+01_JPRB, 0.25345E+01_JPRB, 0.22598E+01_JPRB, 0.20505E+01_JPRB, 0.18921E+01_JPRB ,&
&  0.17685E+01_JPRB, 0.16535E+01_JPRB, 0.15661E+01_JPRB, 0.14862E+01_JPRB, 0.14338E+01_JPRB, 0.14078E+01_JPRB, 0.14041E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.13924E+01_JPRB, 0.14140E+01_JPRB, 0.16120E+01_JPRB, 0.23182E+01_JPRB, 0.33058E+01_JPRB, 0.44384E+01_JPRB ,&
&  0.53510E+01_JPRB, 0.56939E+01_JPRB, 0.52059E+01_JPRB, 0.46178E+01_JPRB, 0.40568E+01_JPRB, 0.36316E+01_JPRB, 0.33030E+01_JPRB ,&
&  0.30023E+01_JPRB, 0.27446E+01_JPRB, 0.25430E+01_JPRB, 0.24343E+01_JPRB, 0.23656E+01_JPRB, 0.23092E+01_JPRB, 0.22378E+01_JPRB ,&
&  0.21354E+01_JPRB, 0.19927E+01_JPRB, 0.18428E+01_JPRB, 0.17197E+01_JPRB, 0.16396E+01_JPRB, 0.16090E+01_JPRB, 0.15887E+01_JPRB ,&
&  0.15286E+01_JPRB, 0.14747E+01_JPRB, 0.13976E+01_JPRB, 0.13369E+01_JPRB, 0.13393E+01_JPRB, 0.13227E+01_JPRB, 0.13025E+01_JPRB ,&
&  0.13091E+01_JPRB, 0.13156E+01_JPRB, 0.12930E+01_JPRB, 0.13010E+01_JPRB, 0.12680E+01_JPRB, 0.13407E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.12229E+01_JPRB, 0.12040E+01_JPRB, 0.11773E+01_JPRB, 0.11589E+01_JPRB, 0.11504E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.11616E+01_JPRB, 0.11390E+01_JPRB, 0.10921E+01_JPRB, 0.10461E+01_JPRB, 0.10125E+01_JPRB, 0.98818E+00_JPRB, 0.96772E+00_JPRB ,&
&  0.94450E+00_JPRB, 0.91599E+00_JPRB, 0.89149E+00_JPRB, 0.87752E+00_JPRB, 0.87270E+00_JPRB, 0.86717E+00_JPRB, 0.84936E+00_JPRB ,&
&  0.82624E+00_JPRB, 0.80620E+00_JPRB, 0.80447E+00_JPRB, 0.81078E+00_JPRB, 0.81772E+00_JPRB, 0.80700E+00_JPRB, 0.81800E+00_JPRB ,&
&  0.82462E+00_JPRB, 0.82315E+00_JPRB, 0.82080E+00_JPRB, 0.81720E+00_JPRB, 0.81159E+00_JPRB, 0.80791E+00_JPRB, 0.80239E+00_JPRB ,&
&  0.98691E+00_JPRB, 0.21863E+01_JPRB, 0.34617E+01_JPRB, 0.42038E+01_JPRB, 0.41064E+01_JPRB, 0.35568E+01_JPRB, 0.32284E+01_JPRB ,&
&  0.29423E+01_JPRB, 0.28057E+01_JPRB, 0.25262E+01_JPRB, 0.23216E+01_JPRB, 0.21278E+01_JPRB, 0.19599E+01_JPRB, 0.18142E+01_JPRB ,&
&  0.17066E+01_JPRB, 0.16192E+01_JPRB, 0.15383E+01_JPRB, 0.14699E+01_JPRB, 0.14196E+01_JPRB, 0.13801E+01_JPRB, 0.13599E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.13388E+01_JPRB, 0.13395E+01_JPRB, 0.14201E+01_JPRB, 0.19284E+01_JPRB, 0.23024E+01_JPRB, 0.36476E+01_JPRB ,&
&  0.47570E+01_JPRB, 0.45000E+01_JPRB, 0.42918E+01_JPRB, 0.39644E+01_JPRB, 0.35627E+01_JPRB, 0.32307E+01_JPRB, 0.28831E+01_JPRB ,&
&  0.25811E+01_JPRB, 0.23425E+01_JPRB, 0.21492E+01_JPRB, 0.20161E+01_JPRB, 0.19312E+01_JPRB, 0.18842E+01_JPRB, 0.18659E+01_JPRB ,&
&  0.18528E+01_JPRB, 0.18372E+01_JPRB, 0.18068E+01_JPRB, 0.17545E+01_JPRB, 0.16774E+01_JPRB, 0.15977E+01_JPRB, 0.15238E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.14036E+01_JPRB, 0.13595E+01_JPRB, 0.13220E+01_JPRB, 0.13517E+01_JPRB, 0.13880E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.13300E+01_JPRB, 0.13249E+01_JPRB, 0.13131E+01_JPRB, 0.12793E+01_JPRB, 0.12716E+01_JPRB, 0.13566E+01_JPRB, 0.13062E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.12231E+01_JPRB, 0.11970E+01_JPRB, 0.11683E+01_JPRB, 0.11718E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.11931E+01_JPRB, 0.11481E+01_JPRB, 0.11042E+01_JPRB, 0.10596E+01_JPRB, 0.10276E+01_JPRB, 0.99431E+00_JPRB, 0.96276E+00_JPRB ,&
&  0.93689E+00_JPRB, 0.92183E+00_JPRB, 0.90863E+00_JPRB, 0.89051E+00_JPRB, 0.88746E+00_JPRB, 0.88016E+00_JPRB, 0.86330E+00_JPRB ,&
&  0.83847E+00_JPRB, 0.82009E+00_JPRB, 0.81249E+00_JPRB, 0.81241E+00_JPRB, 0.82121E+00_JPRB, 0.82330E+00_JPRB, 0.82973E+00_JPRB ,&
&  0.83601E+00_JPRB, 0.81936E+00_JPRB, 0.80800E+00_JPRB, 0.80672E+00_JPRB, 0.80531E+00_JPRB, 0.80672E+00_JPRB, 0.80847E+00_JPRB ,&
&  0.93812E+00_JPRB, 0.16263E+01_JPRB, 0.23709E+01_JPRB, 0.33275E+01_JPRB, 0.34649E+01_JPRB, 0.31491E+01_JPRB, 0.28481E+01_JPRB ,&
&  0.26026E+01_JPRB, 0.24224E+01_JPRB, 0.22029E+01_JPRB, 0.19500E+01_JPRB, 0.17511E+01_JPRB, 0.16136E+01_JPRB, 0.15140E+01_JPRB ,&
&  0.14532E+01_JPRB, 0.14071E+01_JPRB, 0.13645E+01_JPRB, 0.13268E+01_JPRB, 0.12933E+01_JPRB, 0.12643E+01_JPRB, 0.12478E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.12294E+01_JPRB, 0.12277E+01_JPRB, 0.12507E+01_JPRB, 0.14902E+01_JPRB, 0.17310E+01_JPRB, 0.23237E+01_JPRB ,&
&  0.25778E+01_JPRB, 0.26739E+01_JPRB, 0.27624E+01_JPRB, 0.27203E+01_JPRB, 0.25701E+01_JPRB, 0.24200E+01_JPRB, 0.22622E+01_JPRB ,&
&  0.20889E+01_JPRB, 0.19132E+01_JPRB, 0.17934E+01_JPRB, 0.17074E+01_JPRB, 0.16276E+01_JPRB, 0.15675E+01_JPRB, 0.15306E+01_JPRB ,&
&  0.15228E+01_JPRB, 0.15452E+01_JPRB, 0.15700E+01_JPRB, 0.15839E+01_JPRB, 0.15650E+01_JPRB, 0.15261E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.13926E+01_JPRB, 0.13326E+01_JPRB, 0.13079E+01_JPRB, 0.13091E+01_JPRB, 0.13278E+01_JPRB, 0.13362E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.13515E+01_JPRB, 0.13576E+01_JPRB, 0.13337E+01_JPRB, 0.13117E+01_JPRB, 0.13055E+01_JPRB, 0.13722E+01_JPRB, 0.13406E+01_JPRB ,&
&  0.12942E+01_JPRB, 0.12510E+01_JPRB, 0.12227E+01_JPRB, 0.11936E+01_JPRB, 0.12245E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.12647E+01_JPRB, 0.11901E+01_JPRB, 0.11310E+01_JPRB, 0.10868E+01_JPRB, 0.10567E+01_JPRB, 0.10200E+01_JPRB, 0.97790E+00_JPRB ,&
&  0.94690E+00_JPRB, 0.93475E+00_JPRB, 0.92218E+00_JPRB, 0.89930E+00_JPRB, 0.88134E+00_JPRB, 0.86396E+00_JPRB, 0.84909E+00_JPRB ,&
&  0.82834E+00_JPRB, 0.81923E+00_JPRB, 0.81164E+00_JPRB, 0.81159E+00_JPRB, 0.80914E+00_JPRB, 0.81070E+00_JPRB, 0.82006E+00_JPRB ,&
&  0.82377E+00_JPRB, 0.80613E+00_JPRB, 0.80275E+00_JPRB, 0.80038E+00_JPRB, 0.79691E+00_JPRB, 0.79490E+00_JPRB, 0.79478E+00_JPRB ,&
&  0.92892E+00_JPRB, 0.12116E+01_JPRB, 0.16542E+01_JPRB, 0.21471E+01_JPRB, 0.23766E+01_JPRB, 0.24571E+01_JPRB, 0.23562E+01_JPRB ,&
&  0.21976E+01_JPRB, 0.20208E+01_JPRB, 0.18329E+01_JPRB, 0.16571E+01_JPRB, 0.14920E+01_JPRB, 0.13714E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.12129E+01_JPRB, 0.11898E+01_JPRB, 0.11710E+01_JPRB, 0.11522E+01_JPRB, 0.11299E+01_JPRB, 0.11173E+01_JPRB ,&
&  0.11146E+01_JPRB, 0.11016E+01_JPRB, 0.10983E+01_JPRB, 0.11165E+01_JPRB, 0.11638E+01_JPRB, 0.12634E+01_JPRB, 0.15364E+01_JPRB ,&
&  0.17045E+01_JPRB, 0.18215E+01_JPRB, 0.19426E+01_JPRB, 0.19769E+01_JPRB, 0.19200E+01_JPRB, 0.18356E+01_JPRB, 0.17230E+01_JPRB ,&
&  0.16346E+01_JPRB, 0.15691E+01_JPRB, 0.15072E+01_JPRB, 0.14477E+01_JPRB, 0.13986E+01_JPRB, 0.13634E+01_JPRB, 0.13394E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.13272E+01_JPRB, 0.13442E+01_JPRB, 0.13613E+01_JPRB, 0.13699E+01_JPRB, 0.13688E+01_JPRB, 0.13566E+01_JPRB ,&
&  0.13188E+01_JPRB, 0.12662E+01_JPRB, 0.12368E+01_JPRB, 0.12428E+01_JPRB, 0.12744E+01_JPRB, 0.12912E+01_JPRB, 0.12971E+01_JPRB ,&
&  0.13080E+01_JPRB, 0.13216E+01_JPRB, 0.12997E+01_JPRB, 0.13258E+01_JPRB, 0.13959E+01_JPRB, 0.13679E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.12583E+01_JPRB, 0.12262E+01_JPRB, 0.11926E+01_JPRB, 0.12576E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.10978E+01_JPRB, 0.10905E+01_JPRB, 0.10633E+01_JPRB, 0.10399E+01_JPRB, 0.10268E+01_JPRB, 0.10133E+01_JPRB, 0.98980E+00_JPRB ,&
&  0.96343E+00_JPRB, 0.93875E+00_JPRB, 0.92261E+00_JPRB, 0.89990E+00_JPRB, 0.87636E+00_JPRB, 0.86002E+00_JPRB, 0.84441E+00_JPRB ,&
&  0.82521E+00_JPRB, 0.81670E+00_JPRB, 0.81319E+00_JPRB, 0.80884E+00_JPRB, 0.80235E+00_JPRB, 0.79759E+00_JPRB, 0.80222E+00_JPRB ,&
&  0.80235E+00_JPRB, 0.80163E+00_JPRB, 0.80206E+00_JPRB, 0.79570E+00_JPRB, 0.78721E+00_JPRB, 0.78393E+00_JPRB, 0.77193E+00_JPRB ,&
&  0.82210E+00_JPRB, 0.91401E+00_JPRB, 0.11265E+01_JPRB, 0.13957E+01_JPRB, 0.16327E+01_JPRB, 0.18049E+01_JPRB, 0.18867E+01_JPRB ,&
&  0.18151E+01_JPRB, 0.16649E+01_JPRB, 0.15191E+01_JPRB, 0.14256E+01_JPRB, 0.13401E+01_JPRB, 0.12686E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.11377E+01_JPRB, 0.11146E+01_JPRB, 0.10924E+01_JPRB, 0.10687E+01_JPRB, 0.10473E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.10072E+01_JPRB, 0.10058E+01_JPRB, 0.10020E+01_JPRB, 0.10127E+01_JPRB, 0.10701E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.13516E+01_JPRB, 0.14625E+01_JPRB, 0.15162E+01_JPRB, 0.14897E+01_JPRB, 0.14438E+01_JPRB, 0.13850E+01_JPRB ,&
&  0.13358E+01_JPRB, 0.12824E+01_JPRB, 0.12499E+01_JPRB, 0.12173E+01_JPRB, 0.11824E+01_JPRB, 0.11547E+01_JPRB, 0.11406E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.11369E+01_JPRB, 0.11471E+01_JPRB, 0.11623E+01_JPRB, 0.11713E+01_JPRB, 0.11790E+01_JPRB, 0.11899E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.11862E+01_JPRB, 0.11633E+01_JPRB, 0.11688E+01_JPRB, 0.11965E+01_JPRB, 0.12180E+01_JPRB, 0.12312E+01_JPRB ,&
&  0.12435E+01_JPRB, 0.12591E+01_JPRB, 0.12708E+01_JPRB, 0.13074E+01_JPRB, 0.13048E+01_JPRB, 0.12701E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.11833E+01_JPRB, 0.11603E+01_JPRB, 0.11331E+01_JPRB, 0.10484E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.10287E+01_JPRB, 0.10167E+01_JPRB, 0.99506E+00_JPRB, 0.97452E+00_JPRB, 0.96520E+00_JPRB, 0.96195E+00_JPRB, 0.96005E+00_JPRB ,&
&  0.95165E+00_JPRB, 0.93926E+00_JPRB, 0.92491E+00_JPRB, 0.90782E+00_JPRB, 0.88858E+00_JPRB, 0.87115E+00_JPRB, 0.85520E+00_JPRB ,&
&  0.83254E+00_JPRB, 0.81835E+00_JPRB, 0.81539E+00_JPRB, 0.81194E+00_JPRB, 0.80894E+00_JPRB, 0.80339E+00_JPRB, 0.80159E+00_JPRB ,&
&  0.80315E+00_JPRB, 0.80417E+00_JPRB, 0.80429E+00_JPRB, 0.79615E+00_JPRB, 0.78880E+00_JPRB, 0.78701E+00_JPRB, 0.78011E+00_JPRB ,&
&  0.78834E+00_JPRB, 0.73577E+00_JPRB, 0.89527E+00_JPRB, 0.10624E+01_JPRB, 0.11782E+01_JPRB, 0.13234E+01_JPRB, 0.14620E+01_JPRB ,&
&  0.14868E+01_JPRB, 0.14283E+01_JPRB, 0.13379E+01_JPRB, 0.12634E+01_JPRB, 0.11985E+01_JPRB, 0.11518E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.10910E+01_JPRB, 0.10716E+01_JPRB, 0.10498E+01_JPRB, 0.10223E+01_JPRB, 0.99300E+00_JPRB, 0.97151E+00_JPRB, 0.95486E+00_JPRB ,&
&  0.94246E+00_JPRB, 0.93608E+00_JPRB, 0.93030E+00_JPRB, 0.93213E+00_JPRB, 0.93377E+00_JPRB, 0.95192E+00_JPRB, 0.10088E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10936E+01_JPRB, 0.11643E+01_JPRB, 0.11912E+01_JPRB, 0.11808E+01_JPRB, 0.11628E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.10610E+01_JPRB, 0.10362E+01_JPRB, 0.10139E+01_JPRB, 0.99730E+00_JPRB, 0.98878E+00_JPRB, 0.99407E+00_JPRB ,&
&  0.10006E+01_JPRB, 0.10019E+01_JPRB, 0.10065E+01_JPRB, 0.10152E+01_JPRB, 0.10211E+01_JPRB, 0.10236E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10621E+01_JPRB, 0.10897E+01_JPRB, 0.10993E+01_JPRB, 0.10955E+01_JPRB, 0.11000E+01_JPRB, 0.11182E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.11576E+01_JPRB, 0.11693E+01_JPRB, 0.11589E+01_JPRB, 0.12115E+01_JPRB, 0.11912E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.11082E+01_JPRB, 0.10919E+01_JPRB, 0.95436E+00_JPRB, 0.10047E+01_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.94537E+00_JPRB, 0.93555E+00_JPRB, 0.91814E+00_JPRB, 0.90064E+00_JPRB, 0.88846E+00_JPRB, 0.88365E+00_JPRB, 0.88184E+00_JPRB ,&
&  0.88545E+00_JPRB, 0.88829E+00_JPRB, 0.88763E+00_JPRB, 0.88312E+00_JPRB, 0.87575E+00_JPRB, 0.86412E+00_JPRB, 0.85339E+00_JPRB ,&
&  0.83743E+00_JPRB, 0.82097E+00_JPRB, 0.81423E+00_JPRB, 0.81371E+00_JPRB, 0.81154E+00_JPRB, 0.80353E+00_JPRB, 0.79731E+00_JPRB ,&
&  0.79350E+00_JPRB, 0.79403E+00_JPRB, 0.78619E+00_JPRB, 0.78038E+00_JPRB, 0.77374E+00_JPRB, 0.77461E+00_JPRB, 0.77386E+00_JPRB ,&
&  0.77295E+00_JPRB, 0.66725E+00_JPRB, 0.81287E+00_JPRB, 0.92054E+00_JPRB, 0.96712E+00_JPRB, 0.10584E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.11964E+01_JPRB, 0.12147E+01_JPRB, 0.11970E+01_JPRB, 0.11617E+01_JPRB, 0.11132E+01_JPRB, 0.10746E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10175E+01_JPRB, 0.98963E+00_JPRB, 0.95865E+00_JPRB, 0.92692E+00_JPRB, 0.90180E+00_JPRB, 0.87680E+00_JPRB, 0.85531E+00_JPRB ,&
&  0.84361E+00_JPRB, 0.84083E+00_JPRB, 0.83497E+00_JPRB, 0.83404E+00_JPRB, 0.83836E+00_JPRB, 0.84280E+00_JPRB, 0.86557E+00_JPRB ,&
&  0.88818E+00_JPRB, 0.90721E+00_JPRB, 0.92820E+00_JPRB, 0.94156E+00_JPRB, 0.94193E+00_JPRB, 0.94399E+00_JPRB, 0.93849E+00_JPRB ,&
&  0.93098E+00_JPRB, 0.91569E+00_JPRB, 0.89934E+00_JPRB, 0.88492E+00_JPRB, 0.87396E+00_JPRB, 0.86592E+00_JPRB, 0.86637E+00_JPRB ,&
&  0.87209E+00_JPRB, 0.86830E+00_JPRB, 0.86463E+00_JPRB, 0.86663E+00_JPRB, 0.87092E+00_JPRB, 0.87847E+00_JPRB, 0.89340E+00_JPRB ,&
&  0.91761E+00_JPRB, 0.94921E+00_JPRB, 0.97559E+00_JPRB, 0.98730E+00_JPRB, 0.99086E+00_JPRB, 0.99988E+00_JPRB, 0.10130E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10350E+01_JPRB, 0.10420E+01_JPRB, 0.10771E+01_JPRB, 0.10838E+01_JPRB, 0.10739E+01_JPRB, 0.10559E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.10127E+01_JPRB, 0.84423E+00_JPRB, 0.91341E+00_JPRB, 0.93047E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.87602E+00_JPRB, 0.86460E+00_JPRB, 0.84668E+00_JPRB, 0.83156E+00_JPRB, 0.81899E+00_JPRB, 0.81360E+00_JPRB, 0.81033E+00_JPRB ,&
&  0.81440E+00_JPRB, 0.82263E+00_JPRB, 0.83089E+00_JPRB, 0.83578E+00_JPRB, 0.83954E+00_JPRB, 0.83924E+00_JPRB, 0.83435E+00_JPRB ,&
&  0.82997E+00_JPRB, 0.81753E+00_JPRB, 0.80722E+00_JPRB, 0.80582E+00_JPRB, 0.80196E+00_JPRB, 0.79704E+00_JPRB, 0.79012E+00_JPRB ,&
&  0.78295E+00_JPRB, 0.77621E+00_JPRB, 0.76397E+00_JPRB, 0.75745E+00_JPRB, 0.75550E+00_JPRB, 0.75782E+00_JPRB, 0.76638E+00_JPRB ,&
&  0.68973E+00_JPRB, 0.61837E+00_JPRB, 0.71817E+00_JPRB, 0.83331E+00_JPRB, 0.88399E+00_JPRB, 0.93173E+00_JPRB, 0.97366E+00_JPRB ,&
&  0.10310E+01_JPRB, 0.10706E+01_JPRB, 0.10969E+01_JPRB, 0.10986E+01_JPRB, 0.10787E+01_JPRB, 0.10469E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.96384E+00_JPRB, 0.91023E+00_JPRB, 0.87189E+00_JPRB, 0.84870E+00_JPRB, 0.83015E+00_JPRB, 0.81176E+00_JPRB, 0.79665E+00_JPRB ,&
&  0.78627E+00_JPRB, 0.77876E+00_JPRB, 0.77282E+00_JPRB, 0.76874E+00_JPRB, 0.76951E+00_JPRB, 0.77255E+00_JPRB, 0.77925E+00_JPRB ,&
&  0.78890E+00_JPRB, 0.79815E+00_JPRB, 0.80494E+00_JPRB, 0.80686E+00_JPRB, 0.80915E+00_JPRB, 0.81760E+00_JPRB, 0.82258E+00_JPRB ,&
&  0.82665E+00_JPRB, 0.82477E+00_JPRB, 0.82015E+00_JPRB, 0.81357E+00_JPRB, 0.80435E+00_JPRB, 0.79518E+00_JPRB, 0.78937E+00_JPRB ,&
&  0.78738E+00_JPRB, 0.78218E+00_JPRB, 0.78144E+00_JPRB, 0.78549E+00_JPRB, 0.79189E+00_JPRB, 0.79936E+00_JPRB, 0.80710E+00_JPRB ,&
&  0.82039E+00_JPRB, 0.84239E+00_JPRB, 0.86073E+00_JPRB, 0.87312E+00_JPRB, 0.88149E+00_JPRB, 0.88851E+00_JPRB, 0.89862E+00_JPRB ,&
&  0.90829E+00_JPRB, 0.91807E+00_JPRB, 0.93030E+00_JPRB, 0.94829E+00_JPRB, 0.95485E+00_JPRB, 0.95219E+00_JPRB, 0.94308E+00_JPRB ,&
&  0.92703E+00_JPRB, 0.90530E+00_JPRB, 0.89779E+00_JPRB, 0.88775E+00_JPRB, 0.88081E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.77421E+00_JPRB, 0.76677E+00_JPRB, 0.75451E+00_JPRB, 0.74670E+00_JPRB, 0.74285E+00_JPRB, 0.74482E+00_JPRB, 0.74824E+00_JPRB ,&
&  0.75221E+00_JPRB, 0.75350E+00_JPRB, 0.76115E+00_JPRB, 0.76968E+00_JPRB, 0.77544E+00_JPRB, 0.77967E+00_JPRB, 0.77907E+00_JPRB ,&
&  0.77866E+00_JPRB, 0.77814E+00_JPRB, 0.77301E+00_JPRB, 0.76801E+00_JPRB, 0.76154E+00_JPRB, 0.76136E+00_JPRB, 0.76000E+00_JPRB ,&
&  0.75662E+00_JPRB, 0.75088E+00_JPRB, 0.74183E+00_JPRB, 0.73100E+00_JPRB, 0.72561E+00_JPRB, 0.72381E+00_JPRB, 0.72064E+00_JPRB ,&
&  0.72764E+00_JPRB, 0.65000E+00_JPRB, 0.69789E+00_JPRB, 0.74265E+00_JPRB, 0.79964E+00_JPRB, 0.84368E+00_JPRB, 0.88200E+00_JPRB ,&
&  0.92013E+00_JPRB, 0.95326E+00_JPRB, 0.97066E+00_JPRB, 0.97615E+00_JPRB, 0.95724E+00_JPRB, 0.91804E+00_JPRB, 0.87361E+00_JPRB ,&
&  0.83666E+00_JPRB, 0.80936E+00_JPRB, 0.78298E+00_JPRB, 0.76639E+00_JPRB, 0.75277E+00_JPRB, 0.73571E+00_JPRB, 0.71951E+00_JPRB ,&
&  0.70860E+00_JPRB, 0.70077E+00_JPRB, 0.69722E+00_JPRB, 0.69705E+00_JPRB, 0.69887E+00_JPRB, 0.70073E+00_JPRB, 0.70096E+00_JPRB ,&
&  0.70241E+00_JPRB, 0.70523E+00_JPRB, 0.70684E+00_JPRB, 0.70987E+00_JPRB, 0.71434E+00_JPRB, 0.72114E+00_JPRB, 0.72151E+00_JPRB ,&
&  0.71632E+00_JPRB, 0.71746E+00_JPRB, 0.71916E+00_JPRB, 0.71835E+00_JPRB, 0.71741E+00_JPRB, 0.71610E+00_JPRB, 0.71258E+00_JPRB ,&
&  0.70973E+00_JPRB, 0.71127E+00_JPRB, 0.71369E+00_JPRB, 0.71986E+00_JPRB, 0.72479E+00_JPRB, 0.73506E+00_JPRB, 0.74485E+00_JPRB ,&
&  0.75559E+00_JPRB, 0.77189E+00_JPRB, 0.78899E+00_JPRB, 0.80255E+00_JPRB, 0.81123E+00_JPRB, 0.81767E+00_JPRB, 0.82374E+00_JPRB ,&
&  0.82869E+00_JPRB, 0.83347E+00_JPRB, 0.83852E+00_JPRB, 0.84141E+00_JPRB, 0.83886E+00_JPRB, 0.83404E+00_JPRB, 0.82918E+00_JPRB ,&
&  0.81720E+00_JPRB, 0.80060E+00_JPRB, 0.79193E+00_JPRB, 0.78429E+00_JPRB, 0.77765E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.70413E+00_JPRB, 0.69863E+00_JPRB, 0.69302E+00_JPRB, 0.69192E+00_JPRB, 0.69060E+00_JPRB, 0.69205E+00_JPRB, 0.69423E+00_JPRB ,&
&  0.70056E+00_JPRB, 0.70468E+00_JPRB, 0.71410E+00_JPRB, 0.72212E+00_JPRB, 0.72636E+00_JPRB, 0.73132E+00_JPRB, 0.73439E+00_JPRB ,&
&  0.73568E+00_JPRB, 0.73728E+00_JPRB, 0.73802E+00_JPRB, 0.74222E+00_JPRB, 0.74265E+00_JPRB, 0.74176E+00_JPRB, 0.74038E+00_JPRB ,&
&  0.73744E+00_JPRB, 0.73363E+00_JPRB, 0.72641E+00_JPRB, 0.71714E+00_JPRB, 0.70592E+00_JPRB, 0.70001E+00_JPRB, 0.69244E+00_JPRB ,&
&  0.69224E+00_JPRB, 0.67427E+00_JPRB, 0.68865E+00_JPRB, 0.68766E+00_JPRB, 0.71595E+00_JPRB, 0.75960E+00_JPRB, 0.80525E+00_JPRB ,&
&  0.83189E+00_JPRB, 0.84189E+00_JPRB, 0.83733E+00_JPRB, 0.82718E+00_JPRB, 0.80659E+00_JPRB, 0.78662E+00_JPRB, 0.76414E+00_JPRB ,&
&  0.73898E+00_JPRB, 0.72393E+00_JPRB, 0.70717E+00_JPRB, 0.69477E+00_JPRB, 0.68900E+00_JPRB, 0.68446E+00_JPRB, 0.67663E+00_JPRB ,&
&  0.66479E+00_JPRB, 0.65430E+00_JPRB, 0.65249E+00_JPRB, 0.65494E+00_JPRB, 0.65817E+00_JPRB, 0.66022E+00_JPRB, 0.65880E+00_JPRB ,&
&  0.65265E+00_JPRB, 0.64839E+00_JPRB, 0.64815E+00_JPRB, 0.64969E+00_JPRB, 0.64757E+00_JPRB, 0.64198E+00_JPRB, 0.63569E+00_JPRB ,&
&  0.63463E+00_JPRB, 0.63745E+00_JPRB, 0.64097E+00_JPRB, 0.64550E+00_JPRB, 0.64851E+00_JPRB, 0.65212E+00_JPRB, 0.65362E+00_JPRB ,&
&  0.65797E+00_JPRB, 0.66226E+00_JPRB, 0.67131E+00_JPRB, 0.68020E+00_JPRB, 0.68256E+00_JPRB, 0.69187E+00_JPRB, 0.70192E+00_JPRB ,&
&  0.71284E+00_JPRB, 0.72595E+00_JPRB, 0.73742E+00_JPRB, 0.74680E+00_JPRB, 0.75606E+00_JPRB, 0.76171E+00_JPRB, 0.76863E+00_JPRB ,&
&  0.77704E+00_JPRB, 0.77922E+00_JPRB, 0.78509E+00_JPRB, 0.78541E+00_JPRB, 0.78024E+00_JPRB, 0.77182E+00_JPRB, 0.76007E+00_JPRB ,&
&  0.74627E+00_JPRB, 0.72806E+00_JPRB, 0.71730E+00_JPRB, 0.71051E+00_JPRB, 0.70649E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.62496E+00_JPRB, 0.62245E+00_JPRB, 0.62242E+00_JPRB, 0.62541E+00_JPRB, 0.62891E+00_JPRB, 0.63154E+00_JPRB, 0.63707E+00_JPRB ,&
&  0.64692E+00_JPRB, 0.65477E+00_JPRB, 0.66458E+00_JPRB, 0.67480E+00_JPRB, 0.68273E+00_JPRB, 0.68809E+00_JPRB, 0.69098E+00_JPRB ,&
&  0.68944E+00_JPRB, 0.68708E+00_JPRB, 0.69050E+00_JPRB, 0.69550E+00_JPRB, 0.69699E+00_JPRB, 0.69650E+00_JPRB, 0.69634E+00_JPRB ,&
&  0.69535E+00_JPRB, 0.69219E+00_JPRB, 0.68784E+00_JPRB, 0.68162E+00_JPRB, 0.67447E+00_JPRB, 0.67007E+00_JPRB, 0.66617E+00_JPRB ,&
&  0.65752E+00_JPRB, 0.66210E+00_JPRB, 0.61185E+00_JPRB, 0.62564E+00_JPRB, 0.63736E+00_JPRB, 0.66184E+00_JPRB, 0.68769E+00_JPRB ,&
&  0.70449E+00_JPRB, 0.71226E+00_JPRB, 0.71464E+00_JPRB, 0.71031E+00_JPRB, 0.69563E+00_JPRB, 0.68813E+00_JPRB, 0.68073E+00_JPRB ,&
&  0.67015E+00_JPRB, 0.66247E+00_JPRB, 0.64800E+00_JPRB, 0.63210E+00_JPRB, 0.62658E+00_JPRB, 0.62701E+00_JPRB, 0.62642E+00_JPRB ,&
&  0.62043E+00_JPRB, 0.61423E+00_JPRB, 0.61379E+00_JPRB, 0.61396E+00_JPRB, 0.61350E+00_JPRB, 0.60813E+00_JPRB, 0.60150E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.58098E+00_JPRB, 0.57402E+00_JPRB, 0.56919E+00_JPRB, 0.56518E+00_JPRB, 0.56064E+00_JPRB, 0.55613E+00_JPRB ,&
&  0.56085E+00_JPRB, 0.57360E+00_JPRB, 0.58549E+00_JPRB, 0.59493E+00_JPRB, 0.60086E+00_JPRB, 0.60457E+00_JPRB, 0.60964E+00_JPRB ,&
&  0.61574E+00_JPRB, 0.62629E+00_JPRB, 0.63416E+00_JPRB, 0.64141E+00_JPRB, 0.64685E+00_JPRB, 0.65199E+00_JPRB, 0.65793E+00_JPRB ,&
&  0.66335E+00_JPRB, 0.67219E+00_JPRB, 0.67976E+00_JPRB, 0.68843E+00_JPRB, 0.69423E+00_JPRB, 0.70092E+00_JPRB, 0.70775E+00_JPRB ,&
&  0.71494E+00_JPRB, 0.72099E+00_JPRB, 0.72325E+00_JPRB, 0.72128E+00_JPRB, 0.71347E+00_JPRB, 0.70337E+00_JPRB, 0.69296E+00_JPRB ,&
&  0.67957E+00_JPRB, 0.66358E+00_JPRB, 0.64747E+00_JPRB, 0.63695E+00_JPRB, 0.62813E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.56437E+00_JPRB, 0.56471E+00_JPRB, 0.56911E+00_JPRB, 0.57342E+00_JPRB, 0.57669E+00_JPRB, 0.58153E+00_JPRB, 0.59127E+00_JPRB ,&
&  0.60117E+00_JPRB, 0.61067E+00_JPRB, 0.61720E+00_JPRB, 0.62512E+00_JPRB, 0.63646E+00_JPRB, 0.64428E+00_JPRB, 0.65049E+00_JPRB ,&
&  0.65321E+00_JPRB, 0.65398E+00_JPRB, 0.65554E+00_JPRB, 0.65907E+00_JPRB, 0.65886E+00_JPRB, 0.65530E+00_JPRB, 0.65398E+00_JPRB ,&
&  0.65365E+00_JPRB, 0.65153E+00_JPRB, 0.65062E+00_JPRB, 0.65064E+00_JPRB, 0.64691E+00_JPRB, 0.64154E+00_JPRB, 0.63946E+00_JPRB ,&
&  0.63763E+00_JPRB, 0.62363E+00_JPRB, 0.61551E+00_JPRB, 0.60773E+00_JPRB, 0.61037E+00_JPRB, 0.62066E+00_JPRB, 0.63244E+00_JPRB ,&
&  0.63941E+00_JPRB, 0.64422E+00_JPRB, 0.64463E+00_JPRB, 0.64413E+00_JPRB, 0.63937E+00_JPRB, 0.62851E+00_JPRB, 0.62032E+00_JPRB ,&
&  0.61534E+00_JPRB, 0.60693E+00_JPRB, 0.58374E+00_JPRB, 0.56924E+00_JPRB, 0.56495E+00_JPRB, 0.56796E+00_JPRB, 0.57311E+00_JPRB ,&
&  0.57483E+00_JPRB, 0.57127E+00_JPRB, 0.56392E+00_JPRB, 0.55605E+00_JPRB, 0.54621E+00_JPRB, 0.53515E+00_JPRB, 0.52546E+00_JPRB ,&
&  0.52017E+00_JPRB, 0.51621E+00_JPRB, 0.51294E+00_JPRB, 0.51569E+00_JPRB, 0.51496E+00_JPRB, 0.51084E+00_JPRB, 0.51308E+00_JPRB ,&
&  0.52563E+00_JPRB, 0.54183E+00_JPRB, 0.55683E+00_JPRB, 0.56483E+00_JPRB, 0.56804E+00_JPRB, 0.57172E+00_JPRB, 0.57580E+00_JPRB ,&
&  0.58153E+00_JPRB, 0.58692E+00_JPRB, 0.59240E+00_JPRB, 0.60015E+00_JPRB, 0.60971E+00_JPRB, 0.61831E+00_JPRB, 0.62235E+00_JPRB ,&
&  0.62577E+00_JPRB, 0.62837E+00_JPRB, 0.63116E+00_JPRB, 0.63815E+00_JPRB, 0.64744E+00_JPRB, 0.65409E+00_JPRB, 0.65920E+00_JPRB ,&
&  0.66415E+00_JPRB, 0.66630E+00_JPRB, 0.66456E+00_JPRB, 0.65993E+00_JPRB, 0.65376E+00_JPRB, 0.64221E+00_JPRB, 0.62786E+00_JPRB ,&
&  0.61128E+00_JPRB, 0.59581E+00_JPRB, 0.58404E+00_JPRB, 0.57390E+00_JPRB, 0.56674E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.51784E+00_JPRB, 0.51702E+00_JPRB, 0.51950E+00_JPRB, 0.52555E+00_JPRB, 0.52968E+00_JPRB, 0.53346E+00_JPRB, 0.54136E+00_JPRB ,&
&  0.54882E+00_JPRB, 0.55559E+00_JPRB, 0.56337E+00_JPRB, 0.57157E+00_JPRB, 0.57786E+00_JPRB, 0.58708E+00_JPRB, 0.59603E+00_JPRB ,&
&  0.60199E+00_JPRB, 0.60465E+00_JPRB, 0.60647E+00_JPRB, 0.60804E+00_JPRB, 0.60829E+00_JPRB, 0.60604E+00_JPRB, 0.60567E+00_JPRB ,&
&  0.60375E+00_JPRB, 0.60146E+00_JPRB, 0.60324E+00_JPRB, 0.60310E+00_JPRB, 0.60393E+00_JPRB, 0.60554E+00_JPRB, 0.60421E+00_JPRB ,&
&  0.60036E+00_JPRB, 0.58592E+00_JPRB, 0.58019E+00_JPRB, 0.57642E+00_JPRB, 0.58001E+00_JPRB, 0.58036E+00_JPRB, 0.57821E+00_JPRB ,&
&  0.57090E+00_JPRB, 0.56623E+00_JPRB, 0.56794E+00_JPRB, 0.57031E+00_JPRB, 0.56447E+00_JPRB, 0.55158E+00_JPRB, 0.54244E+00_JPRB ,&
&  0.53603E+00_JPRB, 0.52486E+00_JPRB, 0.50685E+00_JPRB, 0.49526E+00_JPRB, 0.49016E+00_JPRB, 0.48549E+00_JPRB, 0.48184E+00_JPRB ,&
&  0.48017E+00_JPRB, 0.47935E+00_JPRB, 0.48028E+00_JPRB, 0.47930E+00_JPRB, 0.47571E+00_JPRB, 0.47167E+00_JPRB, 0.47042E+00_JPRB ,&
&  0.47352E+00_JPRB, 0.47616E+00_JPRB, 0.47968E+00_JPRB, 0.48246E+00_JPRB, 0.48209E+00_JPRB, 0.48224E+00_JPRB, 0.48598E+00_JPRB ,&
&  0.49256E+00_JPRB, 0.49912E+00_JPRB, 0.50522E+00_JPRB, 0.50755E+00_JPRB, 0.51122E+00_JPRB, 0.51870E+00_JPRB, 0.51961E+00_JPRB ,&
&  0.51791E+00_JPRB, 0.51939E+00_JPRB, 0.52357E+00_JPRB, 0.53228E+00_JPRB, 0.54569E+00_JPRB, 0.55497E+00_JPRB, 0.55715E+00_JPRB ,&
&  0.55802E+00_JPRB, 0.56095E+00_JPRB, 0.56562E+00_JPRB, 0.57637E+00_JPRB, 0.58795E+00_JPRB, 0.59617E+00_JPRB, 0.59953E+00_JPRB ,&
&  0.59629E+00_JPRB, 0.58915E+00_JPRB, 0.57650E+00_JPRB, 0.56186E+00_JPRB, 0.54889E+00_JPRB, 0.53869E+00_JPRB, 0.53023E+00_JPRB ,&
&  0.52176E+00_JPRB, 0.51763E+00_JPRB, 0.51667E+00_JPRB, 0.51799E+00_JPRB, 0.51843E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.48088E+00_JPRB, 0.48234E+00_JPRB, 0.48448E+00_JPRB, 0.48740E+00_JPRB, 0.49049E+00_JPRB, 0.49224E+00_JPRB, 0.49582E+00_JPRB ,&
&  0.49817E+00_JPRB, 0.50517E+00_JPRB, 0.51642E+00_JPRB, 0.52750E+00_JPRB, 0.53542E+00_JPRB, 0.54204E+00_JPRB, 0.54874E+00_JPRB ,&
&  0.55437E+00_JPRB, 0.56174E+00_JPRB, 0.56303E+00_JPRB, 0.56100E+00_JPRB, 0.55869E+00_JPRB, 0.55698E+00_JPRB, 0.55534E+00_JPRB ,&
&  0.55279E+00_JPRB, 0.55249E+00_JPRB, 0.55597E+00_JPRB, 0.55978E+00_JPRB, 0.56352E+00_JPRB, 0.56389E+00_JPRB, 0.56148E+00_JPRB ,&
&  0.56377E+00_JPRB, 0.55288E+00_JPRB, 0.54544E+00_JPRB, 0.53719E+00_JPRB, 0.53005E+00_JPRB, 0.51807E+00_JPRB, 0.50906E+00_JPRB ,&
&  0.49830E+00_JPRB, 0.48868E+00_JPRB, 0.48450E+00_JPRB, 0.48021E+00_JPRB, 0.47048E+00_JPRB, 0.45968E+00_JPRB, 0.45231E+00_JPRB ,&
&  0.44775E+00_JPRB, 0.44305E+00_JPRB, 0.43688E+00_JPRB, 0.43138E+00_JPRB, 0.42671E+00_JPRB, 0.42499E+00_JPRB, 0.42659E+00_JPRB ,&
&  0.43132E+00_JPRB, 0.43599E+00_JPRB, 0.43910E+00_JPRB, 0.44011E+00_JPRB, 0.43854E+00_JPRB, 0.43587E+00_JPRB, 0.43449E+00_JPRB ,&
&  0.43534E+00_JPRB, 0.43699E+00_JPRB, 0.43880E+00_JPRB, 0.44389E+00_JPRB, 0.44474E+00_JPRB, 0.44268E+00_JPRB, 0.43673E+00_JPRB ,&
&  0.43449E+00_JPRB, 0.43357E+00_JPRB, 0.43553E+00_JPRB, 0.44084E+00_JPRB, 0.44972E+00_JPRB, 0.45854E+00_JPRB, 0.46480E+00_JPRB ,&
&  0.46766E+00_JPRB, 0.46910E+00_JPRB, 0.47210E+00_JPRB, 0.47613E+00_JPRB, 0.47833E+00_JPRB, 0.47850E+00_JPRB, 0.47776E+00_JPRB ,&
&  0.48123E+00_JPRB, 0.48943E+00_JPRB, 0.49805E+00_JPRB, 0.50399E+00_JPRB, 0.50742E+00_JPRB, 0.50715E+00_JPRB, 0.50068E+00_JPRB ,&
&  0.48817E+00_JPRB, 0.47346E+00_JPRB, 0.45923E+00_JPRB, 0.44738E+00_JPRB, 0.43914E+00_JPRB, 0.43510E+00_JPRB, 0.43778E+00_JPRB ,&
&  0.44525E+00_JPRB, 0.45489E+00_JPRB, 0.46347E+00_JPRB, 0.46960E+00_JPRB, 0.47638E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.44640E+00_JPRB, 0.44628E+00_JPRB, 0.44795E+00_JPRB, 0.44709E+00_JPRB, 0.44442E+00_JPRB, 0.44198E+00_JPRB, 0.44494E+00_JPRB ,&
&  0.45152E+00_JPRB, 0.46349E+00_JPRB, 0.47960E+00_JPRB, 0.49382E+00_JPRB, 0.50231E+00_JPRB, 0.50734E+00_JPRB, 0.51500E+00_JPRB ,&
&  0.52228E+00_JPRB, 0.53043E+00_JPRB, 0.53089E+00_JPRB, 0.52235E+00_JPRB, 0.51518E+00_JPRB, 0.51148E+00_JPRB, 0.51116E+00_JPRB ,&
&  0.51046E+00_JPRB, 0.51201E+00_JPRB, 0.51483E+00_JPRB, 0.51770E+00_JPRB, 0.51642E+00_JPRB, 0.51385E+00_JPRB, 0.51653E+00_JPRB ,&
&  0.51344E+00_JPRB, 0.50243E+00_JPRB, 0.49098E+00_JPRB, 0.46539E+00_JPRB, 0.46378E+00_JPRB, 0.45264E+00_JPRB, 0.44437E+00_JPRB ,&
&  0.43325E+00_JPRB, 0.41935E+00_JPRB, 0.40918E+00_JPRB, 0.40277E+00_JPRB, 0.39978E+00_JPRB, 0.39741E+00_JPRB, 0.39576E+00_JPRB ,&
&  0.39503E+00_JPRB, 0.39575E+00_JPRB, 0.39817E+00_JPRB, 0.39618E+00_JPRB, 0.39512E+00_JPRB, 0.39611E+00_JPRB, 0.39824E+00_JPRB ,&
&  0.39906E+00_JPRB, 0.39993E+00_JPRB, 0.40076E+00_JPRB, 0.40055E+00_JPRB, 0.39901E+00_JPRB, 0.39641E+00_JPRB, 0.39404E+00_JPRB ,&
&  0.39321E+00_JPRB, 0.39458E+00_JPRB, 0.39529E+00_JPRB, 0.39138E+00_JPRB, 0.38355E+00_JPRB, 0.37313E+00_JPRB, 0.36627E+00_JPRB ,&
&  0.36353E+00_JPRB, 0.36604E+00_JPRB, 0.37451E+00_JPRB, 0.38610E+00_JPRB, 0.39668E+00_JPRB, 0.40316E+00_JPRB, 0.40747E+00_JPRB ,&
&  0.41213E+00_JPRB, 0.40958E+00_JPRB, 0.40547E+00_JPRB, 0.40223E+00_JPRB, 0.39937E+00_JPRB, 0.39520E+00_JPRB, 0.39521E+00_JPRB ,&
&  0.39847E+00_JPRB, 0.40195E+00_JPRB, 0.40336E+00_JPRB, 0.40252E+00_JPRB, 0.40050E+00_JPRB, 0.39628E+00_JPRB, 0.39039E+00_JPRB ,&
&  0.38291E+00_JPRB, 0.37414E+00_JPRB, 0.37260E+00_JPRB, 0.37385E+00_JPRB, 0.37786E+00_JPRB, 0.38570E+00_JPRB, 0.39643E+00_JPRB ,&
&  0.40760E+00_JPRB, 0.41645E+00_JPRB, 0.42606E+00_JPRB, 0.43492E+00_JPRB, 0.44295E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.42414E+00_JPRB, 0.42548E+00_JPRB, 0.42279E+00_JPRB, 0.42004E+00_JPRB, 0.41988E+00_JPRB, 0.42136E+00_JPRB, 0.42671E+00_JPRB ,&
&  0.43639E+00_JPRB, 0.44953E+00_JPRB, 0.46224E+00_JPRB, 0.47275E+00_JPRB, 0.48200E+00_JPRB, 0.48602E+00_JPRB, 0.49189E+00_JPRB ,&
&  0.49828E+00_JPRB, 0.50416E+00_JPRB, 0.50494E+00_JPRB, 0.49469E+00_JPRB, 0.48482E+00_JPRB, 0.47962E+00_JPRB, 0.47966E+00_JPRB ,&
&  0.48131E+00_JPRB, 0.48362E+00_JPRB, 0.48935E+00_JPRB, 0.49107E+00_JPRB, 0.48455E+00_JPRB, 0.48240E+00_JPRB, 0.47960E+00_JPRB ,&
&  0.47319E+00_JPRB, 0.46005E+00_JPRB, 0.42683E+00_JPRB, 0.41995E+00_JPRB, 0.41103E+00_JPRB, 0.40166E+00_JPRB, 0.39273E+00_JPRB ,&
&  0.38237E+00_JPRB, 0.37171E+00_JPRB, 0.36446E+00_JPRB, 0.36009E+00_JPRB, 0.36057E+00_JPRB, 0.36189E+00_JPRB, 0.36176E+00_JPRB ,&
&  0.36434E+00_JPRB, 0.36704E+00_JPRB, 0.36970E+00_JPRB, 0.37058E+00_JPRB, 0.37077E+00_JPRB, 0.37057E+00_JPRB, 0.36702E+00_JPRB ,&
&  0.36386E+00_JPRB, 0.36249E+00_JPRB, 0.36143E+00_JPRB, 0.36049E+00_JPRB, 0.36063E+00_JPRB, 0.36136E+00_JPRB, 0.36162E+00_JPRB ,&
&  0.35905E+00_JPRB, 0.35535E+00_JPRB, 0.35118E+00_JPRB, 0.34362E+00_JPRB, 0.32948E+00_JPRB, 0.31518E+00_JPRB, 0.30560E+00_JPRB ,&
&  0.30449E+00_JPRB, 0.31497E+00_JPRB, 0.33511E+00_JPRB, 0.34649E+00_JPRB, 0.35420E+00_JPRB, 0.35780E+00_JPRB, 0.36159E+00_JPRB ,&
&  0.35447E+00_JPRB, 0.33236E+00_JPRB, 0.31986E+00_JPRB, 0.31237E+00_JPRB, 0.30730E+00_JPRB, 0.30395E+00_JPRB, 0.30360E+00_JPRB ,&
&  0.30724E+00_JPRB, 0.31312E+00_JPRB, 0.31973E+00_JPRB, 0.32286E+00_JPRB, 0.32185E+00_JPRB, 0.31887E+00_JPRB, 0.31538E+00_JPRB ,&
&  0.31330E+00_JPRB, 0.31866E+00_JPRB, 0.33361E+00_JPRB, 0.35300E+00_JPRB, 0.36153E+00_JPRB, 0.36835E+00_JPRB, 0.37450E+00_JPRB ,&
&  0.38125E+00_JPRB, 0.39145E+00_JPRB, 0.40340E+00_JPRB, 0.41253E+00_JPRB, 0.42051E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.39424E+00_JPRB, 0.39789E+00_JPRB, 0.40003E+00_JPRB, 0.40321E+00_JPRB, 0.41086E+00_JPRB, 0.42144E+00_JPRB, 0.43347E+00_JPRB ,&
&  0.44513E+00_JPRB, 0.45506E+00_JPRB, 0.46023E+00_JPRB, 0.46564E+00_JPRB, 0.47064E+00_JPRB, 0.47292E+00_JPRB, 0.47781E+00_JPRB ,&
&  0.47838E+00_JPRB, 0.47470E+00_JPRB, 0.46844E+00_JPRB, 0.46037E+00_JPRB, 0.45609E+00_JPRB, 0.45724E+00_JPRB, 0.45987E+00_JPRB ,&
&  0.46135E+00_JPRB, 0.46241E+00_JPRB, 0.46209E+00_JPRB, 0.45814E+00_JPRB, 0.45451E+00_JPRB, 0.45228E+00_JPRB, 0.44762E+00_JPRB ,&
&  0.43616E+00_JPRB, 0.38564E+00_JPRB, 0.34845E+00_JPRB, 0.34534E+00_JPRB, 0.35995E+00_JPRB, 0.36180E+00_JPRB, 0.35739E+00_JPRB ,&
&  0.35208E+00_JPRB, 0.34705E+00_JPRB, 0.34408E+00_JPRB, 0.34178E+00_JPRB, 0.33903E+00_JPRB, 0.33651E+00_JPRB, 0.33707E+00_JPRB ,&
&  0.33841E+00_JPRB, 0.33638E+00_JPRB, 0.33362E+00_JPRB, 0.32731E+00_JPRB, 0.32290E+00_JPRB, 0.32191E+00_JPRB, 0.32117E+00_JPRB ,&
&  0.31717E+00_JPRB, 0.31291E+00_JPRB, 0.31318E+00_JPRB, 0.31478E+00_JPRB, 0.31720E+00_JPRB, 0.31881E+00_JPRB, 0.31575E+00_JPRB ,&
&  0.31139E+00_JPRB, 0.30421E+00_JPRB, 0.29615E+00_JPRB, 0.28623E+00_JPRB, 0.27703E+00_JPRB, 0.27007E+00_JPRB, 0.26419E+00_JPRB ,&
&  0.26385E+00_JPRB, 0.27115E+00_JPRB, 0.28596E+00_JPRB, 0.30108E+00_JPRB, 0.31379E+00_JPRB, 0.30657E+00_JPRB, 0.29719E+00_JPRB ,&
&  0.28603E+00_JPRB, 0.27622E+00_JPRB, 0.26925E+00_JPRB, 0.26628E+00_JPRB, 0.26515E+00_JPRB, 0.26512E+00_JPRB, 0.26650E+00_JPRB ,&
&  0.26904E+00_JPRB, 0.27203E+00_JPRB, 0.27437E+00_JPRB, 0.27546E+00_JPRB, 0.27581E+00_JPRB, 0.27591E+00_JPRB, 0.27524E+00_JPRB ,&
&  0.27449E+00_JPRB, 0.27721E+00_JPRB, 0.28436E+00_JPRB, 0.29330E+00_JPRB, 0.30160E+00_JPRB, 0.30970E+00_JPRB, 0.32278E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.35768E+00_JPRB, 0.37132E+00_JPRB, 0.38201E+00_JPRB, 0.39067E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.38333E+00_JPRB, 0.39236E+00_JPRB, 0.40381E+00_JPRB, 0.41400E+00_JPRB, 0.42339E+00_JPRB, 0.43213E+00_JPRB, 0.44012E+00_JPRB ,&
&  0.44492E+00_JPRB, 0.44772E+00_JPRB, 0.44892E+00_JPRB, 0.45272E+00_JPRB, 0.44952E+00_JPRB, 0.44192E+00_JPRB, 0.43609E+00_JPRB ,&
&  0.43112E+00_JPRB, 0.42733E+00_JPRB, 0.42194E+00_JPRB, 0.41910E+00_JPRB, 0.41874E+00_JPRB, 0.41865E+00_JPRB, 0.41280E+00_JPRB ,&
&  0.40173E+00_JPRB, 0.39983E+00_JPRB, 0.40401E+00_JPRB, 0.39791E+00_JPRB, 0.39178E+00_JPRB, 0.38575E+00_JPRB, 0.37956E+00_JPRB ,&
&  0.36252E+00_JPRB, 0.34170E+00_JPRB, 0.31549E+00_JPRB, 0.31340E+00_JPRB, 0.32907E+00_JPRB, 0.32155E+00_JPRB, 0.32568E+00_JPRB ,&
&  0.32692E+00_JPRB, 0.32688E+00_JPRB, 0.32543E+00_JPRB, 0.32359E+00_JPRB, 0.32023E+00_JPRB, 0.31955E+00_JPRB, 0.31785E+00_JPRB ,&
&  0.31119E+00_JPRB, 0.30600E+00_JPRB, 0.29736E+00_JPRB, 0.27872E+00_JPRB, 0.27492E+00_JPRB, 0.27473E+00_JPRB, 0.27073E+00_JPRB ,&
&  0.26331E+00_JPRB, 0.25902E+00_JPRB, 0.25799E+00_JPRB, 0.25296E+00_JPRB, 0.25523E+00_JPRB, 0.25770E+00_JPRB, 0.25749E+00_JPRB ,&
&  0.25729E+00_JPRB, 0.25671E+00_JPRB, 0.25438E+00_JPRB, 0.25036E+00_JPRB, 0.24702E+00_JPRB, 0.24609E+00_JPRB, 0.24734E+00_JPRB ,&
&  0.24998E+00_JPRB, 0.25427E+00_JPRB, 0.26583E+00_JPRB, 0.27452E+00_JPRB, 0.27287E+00_JPRB, 0.26907E+00_JPRB, 0.25967E+00_JPRB ,&
&  0.25034E+00_JPRB, 0.24704E+00_JPRB, 0.24413E+00_JPRB, 0.24249E+00_JPRB, 0.24273E+00_JPRB, 0.24426E+00_JPRB, 0.24488E+00_JPRB ,&
&  0.24612E+00_JPRB, 0.24755E+00_JPRB, 0.24854E+00_JPRB, 0.24900E+00_JPRB, 0.24973E+00_JPRB, 0.25059E+00_JPRB, 0.25231E+00_JPRB ,&
&  0.25484E+00_JPRB, 0.25935E+00_JPRB, 0.26525E+00_JPRB, 0.27162E+00_JPRB, 0.27631E+00_JPRB, 0.27872E+00_JPRB, 0.28385E+00_JPRB ,&
&  0.29013E+00_JPRB, 0.30664E+00_JPRB, 0.33585E+00_JPRB, 0.37912E+00_JPRB, 0.38166E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.38042E+00_JPRB, 0.39269E+00_JPRB, 0.40779E+00_JPRB, 0.41744E+00_JPRB, 0.42340E+00_JPRB, 0.42653E+00_JPRB, 0.42588E+00_JPRB ,&
&  0.42256E+00_JPRB, 0.42178E+00_JPRB, 0.43260E+00_JPRB, 0.41365E+00_JPRB, 0.39634E+00_JPRB, 0.36394E+00_JPRB, 0.34731E+00_JPRB ,&
&  0.33431E+00_JPRB, 0.33191E+00_JPRB, 0.33125E+00_JPRB, 0.33571E+00_JPRB, 0.33909E+00_JPRB, 0.34167E+00_JPRB, 0.33728E+00_JPRB ,&
&  0.33138E+00_JPRB, 0.32194E+00_JPRB, 0.31403E+00_JPRB, 0.30681E+00_JPRB, 0.30724E+00_JPRB, 0.30607E+00_JPRB, 0.30344E+00_JPRB ,&
&  0.30070E+00_JPRB, 0.29915E+00_JPRB, 0.29941E+00_JPRB, 0.30051E+00_JPRB, 0.31208E+00_JPRB, 0.31062E+00_JPRB, 0.31158E+00_JPRB ,&
&  0.31158E+00_JPRB, 0.31153E+00_JPRB, 0.31046E+00_JPRB, 0.30875E+00_JPRB, 0.30391E+00_JPRB, 0.28774E+00_JPRB, 0.28613E+00_JPRB ,&
&  0.27819E+00_JPRB, 0.26800E+00_JPRB, 0.25784E+00_JPRB, 0.25079E+00_JPRB, 0.24650E+00_JPRB, 0.24518E+00_JPRB, 0.24057E+00_JPRB ,&
&  0.23570E+00_JPRB, 0.22963E+00_JPRB, 0.22705E+00_JPRB, 0.22722E+00_JPRB, 0.22805E+00_JPRB, 0.22868E+00_JPRB, 0.22917E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23135E+00_JPRB, 0.23176E+00_JPRB, 0.23261E+00_JPRB, 0.23342E+00_JPRB, 0.23456E+00_JPRB, 0.23652E+00_JPRB ,&
&  0.23999E+00_JPRB, 0.24362E+00_JPRB, 0.24710E+00_JPRB, 0.25058E+00_JPRB, 0.24957E+00_JPRB, 0.24542E+00_JPRB, 0.24013E+00_JPRB ,&
&  0.23619E+00_JPRB, 0.23477E+00_JPRB, 0.23340E+00_JPRB, 0.23313E+00_JPRB, 0.23313E+00_JPRB, 0.23346E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.23495E+00_JPRB, 0.23571E+00_JPRB, 0.23662E+00_JPRB, 0.23783E+00_JPRB, 0.23946E+00_JPRB, 0.24095E+00_JPRB, 0.24300E+00_JPRB ,&
&  0.24608E+00_JPRB, 0.24946E+00_JPRB, 0.25413E+00_JPRB, 0.25831E+00_JPRB, 0.26467E+00_JPRB, 0.27314E+00_JPRB, 0.28708E+00_JPRB ,&
&  0.30158E+00_JPRB, 0.33429E+00_JPRB, 0.34857E+00_JPRB, 0.36204E+00_JPRB, 0.37034E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.37991E+00_JPRB, 0.38997E+00_JPRB, 0.40398E+00_JPRB, 0.41333E+00_JPRB, 0.42041E+00_JPRB, 0.41632E+00_JPRB, 0.41229E+00_JPRB ,&
&  0.40798E+00_JPRB, 0.39435E+00_JPRB, 0.37701E+00_JPRB, 0.34786E+00_JPRB, 0.32333E+00_JPRB, 0.30467E+00_JPRB, 0.29378E+00_JPRB ,&
&  0.28672E+00_JPRB, 0.28408E+00_JPRB, 0.28118E+00_JPRB, 0.28006E+00_JPRB, 0.27805E+00_JPRB, 0.27905E+00_JPRB, 0.27988E+00_JPRB ,&
&  0.27907E+00_JPRB, 0.27824E+00_JPRB, 0.27805E+00_JPRB, 0.28136E+00_JPRB, 0.28785E+00_JPRB, 0.29284E+00_JPRB, 0.29778E+00_JPRB ,&
&  0.30101E+00_JPRB, 0.29806E+00_JPRB, 0.29992E+00_JPRB, 0.29838E+00_JPRB, 0.29757E+00_JPRB, 0.29894E+00_JPRB, 0.30076E+00_JPRB ,&
&  0.30115E+00_JPRB, 0.30092E+00_JPRB, 0.29976E+00_JPRB, 0.29602E+00_JPRB, 0.28335E+00_JPRB, 0.27714E+00_JPRB, 0.26876E+00_JPRB ,&
&  0.26059E+00_JPRB, 0.25113E+00_JPRB, 0.24316E+00_JPRB, 0.23963E+00_JPRB, 0.23469E+00_JPRB, 0.23307E+00_JPRB, 0.22979E+00_JPRB ,&
&  0.22609E+00_JPRB, 0.22266E+00_JPRB, 0.22068E+00_JPRB, 0.21973E+00_JPRB, 0.21937E+00_JPRB, 0.21782E+00_JPRB, 0.21730E+00_JPRB ,&
&  0.21775E+00_JPRB, 0.21819E+00_JPRB, 0.21939E+00_JPRB, 0.22037E+00_JPRB, 0.22141E+00_JPRB, 0.22220E+00_JPRB, 0.22314E+00_JPRB ,&
&  0.22485E+00_JPRB, 0.22711E+00_JPRB, 0.22811E+00_JPRB, 0.22911E+00_JPRB, 0.22874E+00_JPRB, 0.22799E+00_JPRB, 0.22693E+00_JPRB ,&
&  0.22603E+00_JPRB, 0.22572E+00_JPRB, 0.22571E+00_JPRB, 0.22557E+00_JPRB, 0.22601E+00_JPRB, 0.22689E+00_JPRB, 0.22770E+00_JPRB ,&
&  0.22929E+00_JPRB, 0.23069E+00_JPRB, 0.23233E+00_JPRB, 0.23445E+00_JPRB, 0.23674E+00_JPRB, 0.23955E+00_JPRB, 0.24264E+00_JPRB ,&
&  0.24520E+00_JPRB, 0.24861E+00_JPRB, 0.25146E+00_JPRB, 0.25369E+00_JPRB, 0.26014E+00_JPRB, 0.26771E+00_JPRB, 0.28337E+00_JPRB ,&
&  0.30011E+00_JPRB, 0.33776E+00_JPRB, 0.37826E+00_JPRB, 0.36572E+00_JPRB, 0.37157E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.35240E+00_JPRB, 0.36372E+00_JPRB, 0.37232E+00_JPRB, 0.37775E+00_JPRB, 0.38294E+00_JPRB, 0.38472E+00_JPRB, 0.39191E+00_JPRB ,&
&  0.38640E+00_JPRB, 0.36873E+00_JPRB, 0.35760E+00_JPRB, 0.34086E+00_JPRB, 0.32048E+00_JPRB, 0.30355E+00_JPRB, 0.29465E+00_JPRB ,&
&  0.28924E+00_JPRB, 0.28470E+00_JPRB, 0.27941E+00_JPRB, 0.27371E+00_JPRB, 0.26805E+00_JPRB, 0.26378E+00_JPRB, 0.26127E+00_JPRB ,&
&  0.25894E+00_JPRB, 0.26015E+00_JPRB, 0.26350E+00_JPRB, 0.26851E+00_JPRB, 0.27571E+00_JPRB, 0.27997E+00_JPRB, 0.28350E+00_JPRB ,&
&  0.28448E+00_JPRB, 0.27409E+00_JPRB, 0.27739E+00_JPRB, 0.28059E+00_JPRB, 0.27975E+00_JPRB, 0.28439E+00_JPRB, 0.29228E+00_JPRB ,&
&  0.28878E+00_JPRB, 0.27871E+00_JPRB, 0.28062E+00_JPRB, 0.27352E+00_JPRB, 0.26479E+00_JPRB, 0.26402E+00_JPRB, 0.25818E+00_JPRB ,&
&  0.25672E+00_JPRB, 0.24972E+00_JPRB, 0.24462E+00_JPRB, 0.24134E+00_JPRB, 0.23441E+00_JPRB, 0.23172E+00_JPRB, 0.22778E+00_JPRB ,&
&  0.22396E+00_JPRB, 0.22163E+00_JPRB, 0.21870E+00_JPRB, 0.21758E+00_JPRB, 0.21653E+00_JPRB, 0.21506E+00_JPRB, 0.21384E+00_JPRB ,&
&  0.21310E+00_JPRB, 0.21243E+00_JPRB, 0.21242E+00_JPRB, 0.21237E+00_JPRB, 0.21222E+00_JPRB, 0.21249E+00_JPRB, 0.21254E+00_JPRB ,&
&  0.21320E+00_JPRB, 0.21408E+00_JPRB, 0.21480E+00_JPRB, 0.21604E+00_JPRB, 0.21647E+00_JPRB, 0.21711E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21790E+00_JPRB, 0.21854E+00_JPRB, 0.21867E+00_JPRB, 0.21926E+00_JPRB, 0.21989E+00_JPRB, 0.22138E+00_JPRB, 0.22284E+00_JPRB ,&
&  0.22466E+00_JPRB, 0.22691E+00_JPRB, 0.22926E+00_JPRB, 0.23176E+00_JPRB, 0.23487E+00_JPRB, 0.23761E+00_JPRB, 0.24133E+00_JPRB ,&
&  0.24539E+00_JPRB, 0.24930E+00_JPRB, 0.25427E+00_JPRB, 0.26024E+00_JPRB, 0.26844E+00_JPRB, 0.27851E+00_JPRB, 0.28700E+00_JPRB ,&
&  0.29655E+00_JPRB, 0.32111E+00_JPRB, 0.32741E+00_JPRB, 0.33399E+00_JPRB, 0.34333E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.34189E+00_JPRB, 0.34720E+00_JPRB, 0.35127E+00_JPRB, 0.35390E+00_JPRB, 0.35658E+00_JPRB, 0.35828E+00_JPRB, 0.35893E+00_JPRB ,&
&  0.35229E+00_JPRB, 0.34738E+00_JPRB, 0.34018E+00_JPRB, 0.33139E+00_JPRB, 0.32330E+00_JPRB, 0.31481E+00_JPRB, 0.30604E+00_JPRB ,&
&  0.29881E+00_JPRB, 0.29202E+00_JPRB, 0.28532E+00_JPRB, 0.27817E+00_JPRB, 0.27079E+00_JPRB, 0.26334E+00_JPRB, 0.26060E+00_JPRB ,&
&  0.26035E+00_JPRB, 0.26230E+00_JPRB, 0.26529E+00_JPRB, 0.26686E+00_JPRB, 0.26781E+00_JPRB, 0.26821E+00_JPRB, 0.26697E+00_JPRB ,&
&  0.26294E+00_JPRB, 0.26400E+00_JPRB, 0.26554E+00_JPRB, 0.26654E+00_JPRB, 0.26847E+00_JPRB, 0.27021E+00_JPRB, 0.27020E+00_JPRB ,&
&  0.26817E+00_JPRB, 0.26555E+00_JPRB, 0.26193E+00_JPRB, 0.25851E+00_JPRB, 0.25757E+00_JPRB, 0.25560E+00_JPRB, 0.25338E+00_JPRB ,&
&  0.25211E+00_JPRB, 0.24831E+00_JPRB, 0.24399E+00_JPRB, 0.24096E+00_JPRB, 0.23657E+00_JPRB, 0.23359E+00_JPRB, 0.22964E+00_JPRB ,&
&  0.22657E+00_JPRB, 0.22391E+00_JPRB, 0.22097E+00_JPRB, 0.21878E+00_JPRB, 0.21678E+00_JPRB, 0.21531E+00_JPRB, 0.21379E+00_JPRB ,&
&  0.21293E+00_JPRB, 0.21206E+00_JPRB, 0.21148E+00_JPRB, 0.21079E+00_JPRB, 0.21033E+00_JPRB, 0.20988E+00_JPRB, 0.20952E+00_JPRB ,&
&  0.20932E+00_JPRB, 0.20970E+00_JPRB, 0.20993E+00_JPRB, 0.21038E+00_JPRB, 0.21088E+00_JPRB, 0.21110E+00_JPRB, 0.21164E+00_JPRB ,&
&  0.21274E+00_JPRB, 0.21395E+00_JPRB, 0.21460E+00_JPRB, 0.21552E+00_JPRB, 0.21657E+00_JPRB, 0.21783E+00_JPRB, 0.21966E+00_JPRB ,&
&  0.22155E+00_JPRB, 0.22369E+00_JPRB, 0.22615E+00_JPRB, 0.22852E+00_JPRB, 0.23170E+00_JPRB, 0.23498E+00_JPRB, 0.23848E+00_JPRB ,&
&  0.24281E+00_JPRB, 0.24735E+00_JPRB, 0.25215E+00_JPRB, 0.25744E+00_JPRB, 0.26522E+00_JPRB, 0.27112E+00_JPRB, 0.27831E+00_JPRB ,&
&  0.28490E+00_JPRB, 0.28823E+00_JPRB, 0.32140E+00_JPRB, 0.32614E+00_JPRB, 0.33309E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.26685E+00_JPRB, 0.27310E+00_JPRB, 0.27900E+00_JPRB, 0.28749E+00_JPRB, 0.29083E+00_JPRB, 0.32693E+00_JPRB, 0.32186E+00_JPRB ,&
&  0.31908E+00_JPRB, 0.30718E+00_JPRB, 0.30643E+00_JPRB, 0.30318E+00_JPRB, 0.29883E+00_JPRB, 0.29419E+00_JPRB, 0.28934E+00_JPRB ,&
&  0.28483E+00_JPRB, 0.27942E+00_JPRB, 0.27506E+00_JPRB, 0.27156E+00_JPRB, 0.26846E+00_JPRB, 0.26537E+00_JPRB, 0.26272E+00_JPRB ,&
&  0.26171E+00_JPRB, 0.26175E+00_JPRB, 0.26105E+00_JPRB, 0.25615E+00_JPRB, 0.25242E+00_JPRB, 0.25021E+00_JPRB, 0.24966E+00_JPRB ,&
&  0.24969E+00_JPRB, 0.25193E+00_JPRB, 0.25356E+00_JPRB, 0.25503E+00_JPRB, 0.25716E+00_JPRB, 0.25632E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25414E+00_JPRB, 0.25274E+00_JPRB, 0.25135E+00_JPRB, 0.25008E+00_JPRB, 0.24882E+00_JPRB, 0.24689E+00_JPRB, 0.24538E+00_JPRB ,&
&  0.24397E+00_JPRB, 0.24198E+00_JPRB, 0.23893E+00_JPRB, 0.23693E+00_JPRB, 0.23451E+00_JPRB, 0.23247E+00_JPRB, 0.23033E+00_JPRB ,&
&  0.22872E+00_JPRB, 0.22730E+00_JPRB, 0.22568E+00_JPRB, 0.22373E+00_JPRB, 0.22183E+00_JPRB, 0.22017E+00_JPRB, 0.21861E+00_JPRB ,&
&  0.21781E+00_JPRB, 0.21695E+00_JPRB, 0.21611E+00_JPRB, 0.21520E+00_JPRB, 0.21477E+00_JPRB, 0.21429E+00_JPRB, 0.21384E+00_JPRB ,&
&  0.21348E+00_JPRB, 0.21320E+00_JPRB, 0.21317E+00_JPRB, 0.21321E+00_JPRB, 0.21329E+00_JPRB, 0.21366E+00_JPRB, 0.21411E+00_JPRB ,&
&  0.21474E+00_JPRB, 0.21552E+00_JPRB, 0.21665E+00_JPRB, 0.21783E+00_JPRB, 0.21886E+00_JPRB, 0.21965E+00_JPRB, 0.22062E+00_JPRB ,&
&  0.22221E+00_JPRB, 0.22403E+00_JPRB, 0.22604E+00_JPRB, 0.22746E+00_JPRB, 0.22972E+00_JPRB, 0.23178E+00_JPRB, 0.23386E+00_JPRB ,&
&  0.23592E+00_JPRB, 0.23852E+00_JPRB, 0.24156E+00_JPRB, 0.24494E+00_JPRB, 0.24901E+00_JPRB, 0.25204E+00_JPRB, 0.25490E+00_JPRB ,&
&  0.25782E+00_JPRB, 0.26232E+00_JPRB, 0.26418E+00_JPRB, 0.26494E+00_JPRB, 0.26580E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.24646E+00_JPRB, 0.24639E+00_JPRB, 0.24664E+00_JPRB, 0.24689E+00_JPRB, 0.24704E+00_JPRB, 0.24753E+00_JPRB, 0.24751E+00_JPRB ,&
&  0.24728E+00_JPRB, 0.24698E+00_JPRB, 0.24730E+00_JPRB, 0.24766E+00_JPRB, 0.24787E+00_JPRB, 0.24787E+00_JPRB, 0.24745E+00_JPRB ,&
&  0.24668E+00_JPRB, 0.24574E+00_JPRB, 0.24407E+00_JPRB, 0.24335E+00_JPRB, 0.24283E+00_JPRB, 0.24239E+00_JPRB, 0.24157E+00_JPRB ,&
&  0.24083E+00_JPRB, 0.23957E+00_JPRB, 0.23790E+00_JPRB, 0.23685E+00_JPRB, 0.23685E+00_JPRB, 0.23741E+00_JPRB, 0.23830E+00_JPRB ,&
&  0.23901E+00_JPRB, 0.23950E+00_JPRB, 0.23975E+00_JPRB, 0.24007E+00_JPRB, 0.24050E+00_JPRB, 0.24042E+00_JPRB, 0.24021E+00_JPRB ,&
&  0.23990E+00_JPRB, 0.23935E+00_JPRB, 0.23882E+00_JPRB, 0.23834E+00_JPRB, 0.23791E+00_JPRB, 0.23737E+00_JPRB, 0.23675E+00_JPRB ,&
&  0.23602E+00_JPRB, 0.23513E+00_JPRB, 0.23400E+00_JPRB, 0.23312E+00_JPRB, 0.23217E+00_JPRB, 0.23143E+00_JPRB, 0.23046E+00_JPRB ,&
&  0.22966E+00_JPRB, 0.22893E+00_JPRB, 0.22819E+00_JPRB, 0.22750E+00_JPRB, 0.22686E+00_JPRB, 0.22625E+00_JPRB, 0.22567E+00_JPRB ,&
&  0.22511E+00_JPRB, 0.22464E+00_JPRB, 0.22413E+00_JPRB, 0.22364E+00_JPRB, 0.22331E+00_JPRB, 0.22303E+00_JPRB, 0.22280E+00_JPRB ,&
&  0.22261E+00_JPRB, 0.22257E+00_JPRB, 0.22256E+00_JPRB, 0.22254E+00_JPRB, 0.22255E+00_JPRB, 0.22267E+00_JPRB, 0.22279E+00_JPRB ,&
&  0.22296E+00_JPRB, 0.22306E+00_JPRB, 0.22317E+00_JPRB, 0.22333E+00_JPRB, 0.22351E+00_JPRB, 0.22372E+00_JPRB, 0.22403E+00_JPRB ,&
&  0.22439E+00_JPRB, 0.22482E+00_JPRB, 0.22529E+00_JPRB, 0.22551E+00_JPRB, 0.22619E+00_JPRB, 0.22697E+00_JPRB, 0.22791E+00_JPRB ,&
&  0.22895E+00_JPRB, 0.23021E+00_JPRB, 0.23166E+00_JPRB, 0.23343E+00_JPRB, 0.23531E+00_JPRB, 0.23644E+00_JPRB, 0.23758E+00_JPRB ,&
&  0.23877E+00_JPRB, 0.24021E+00_JPRB, 0.24155E+00_JPRB, 0.24292E+00_JPRB, 0.24452E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB, 0.22927E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB, 0.74952E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.76361E+00_JPRB, 0.76300E+00_JPRB, 0.76241E+00_JPRB, 0.76174E+00_JPRB, 0.76087E+00_JPRB, 0.75998E+00_JPRB, 0.75924E+00_JPRB ,&
&  0.75846E+00_JPRB, 0.75797E+00_JPRB, 0.75699E+00_JPRB, 0.75598E+00_JPRB, 0.75492E+00_JPRB, 0.75398E+00_JPRB, 0.75295E+00_JPRB ,&
&  0.75186E+00_JPRB, 0.75075E+00_JPRB, 0.74989E+00_JPRB, 0.74887E+00_JPRB, 0.74775E+00_JPRB, 0.74671E+00_JPRB, 0.74573E+00_JPRB ,&
&  0.74486E+00_JPRB, 0.74385E+00_JPRB, 0.74283E+00_JPRB, 0.74160E+00_JPRB, 0.74082E+00_JPRB, 0.73968E+00_JPRB, 0.73877E+00_JPRB ,&
&  0.73771E+00_JPRB, 0.73699E+00_JPRB, 0.73617E+00_JPRB, 0.73542E+00_JPRB, 0.73449E+00_JPRB, 0.73435E+00_JPRB, 0.73382E+00_JPRB ,&
&  0.73357E+00_JPRB, 0.73330E+00_JPRB, 0.73334E+00_JPRB, 0.73326E+00_JPRB, 0.73328E+00_JPRB, 0.73340E+00_JPRB, 0.73402E+00_JPRB ,&
&  0.73453E+00_JPRB, 0.73514E+00_JPRB, 0.73607E+00_JPRB, 0.73695E+00_JPRB, 0.73774E+00_JPRB, 0.73854E+00_JPRB, 0.73943E+00_JPRB ,&
&  0.74068E+00_JPRB, 0.74194E+00_JPRB, 0.74314E+00_JPRB, 0.74440E+00_JPRB, 0.74564E+00_JPRB, 0.74663E+00_JPRB, 0.74744E+00_JPRB ,&
&  0.74794E+00_JPRB, 0.74858E+00_JPRB, 0.74915E+00_JPRB, 0.74974E+00_JPRB, 0.74986E+00_JPRB, 0.75024E+00_JPRB, 0.75051E+00_JPRB ,&
&  0.75075E+00_JPRB, 0.75040E+00_JPRB, 0.75048E+00_JPRB, 0.75037E+00_JPRB, 0.75048E+00_JPRB, 0.75043E+00_JPRB, 0.75090E+00_JPRB ,&
&  0.75126E+00_JPRB, 0.75197E+00_JPRB, 0.75274E+00_JPRB, 0.75380E+00_JPRB, 0.75458E+00_JPRB, 0.75546E+00_JPRB, 0.75624E+00_JPRB ,&
&  0.75712E+00_JPRB, 0.75805E+00_JPRB, 0.75901E+00_JPRB, 0.75966E+00_JPRB, 0.76030E+00_JPRB, 0.76104E+00_JPRB, 0.76180E+00_JPRB ,&
&  0.76276E+00_JPRB, 0.76332E+00_JPRB, 0.76393E+00_JPRB, 0.76448E+00_JPRB, 0.76470E+00_JPRB, 0.76485E+00_JPRB, 0.76487E+00_JPRB ,&
&  0.76478E+00_JPRB, 0.76460E+00_JPRB, 0.76436E+00_JPRB, 0.76419E+00_JPRB, 0.76402E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.77390E+00_JPRB, 0.77366E+00_JPRB, 0.77315E+00_JPRB, 0.77223E+00_JPRB, 0.77108E+00_JPRB, 0.76961E+00_JPRB, 0.76817E+00_JPRB ,&
&  0.76642E+00_JPRB, 0.76483E+00_JPRB, 0.76276E+00_JPRB, 0.76064E+00_JPRB, 0.75839E+00_JPRB, 0.75631E+00_JPRB, 0.75414E+00_JPRB ,&
&  0.75205E+00_JPRB, 0.74994E+00_JPRB, 0.74845E+00_JPRB, 0.74687E+00_JPRB, 0.74513E+00_JPRB, 0.74297E+00_JPRB, 0.74109E+00_JPRB ,&
&  0.73915E+00_JPRB, 0.73687E+00_JPRB, 0.73440E+00_JPRB, 0.73020E+00_JPRB, 0.72667E+00_JPRB, 0.72337E+00_JPRB, 0.72054E+00_JPRB ,&
&  0.71781E+00_JPRB, 0.71532E+00_JPRB, 0.71345E+00_JPRB, 0.71217E+00_JPRB, 0.71326E+00_JPRB, 0.71350E+00_JPRB, 0.71318E+00_JPRB ,&
&  0.71267E+00_JPRB, 0.71028E+00_JPRB, 0.70953E+00_JPRB, 0.70928E+00_JPRB, 0.70931E+00_JPRB, 0.71105E+00_JPRB, 0.71256E+00_JPRB ,&
&  0.71416E+00_JPRB, 0.71550E+00_JPRB, 0.71808E+00_JPRB, 0.72056E+00_JPRB, 0.72288E+00_JPRB, 0.72536E+00_JPRB, 0.72690E+00_JPRB ,&
&  0.72864E+00_JPRB, 0.73117E+00_JPRB, 0.73359E+00_JPRB, 0.73616E+00_JPRB, 0.73773E+00_JPRB, 0.73915E+00_JPRB, 0.74021E+00_JPRB ,&
&  0.74104E+00_JPRB, 0.74216E+00_JPRB, 0.74335E+00_JPRB, 0.74446E+00_JPRB, 0.74509E+00_JPRB, 0.74579E+00_JPRB, 0.74626E+00_JPRB ,&
&  0.74645E+00_JPRB, 0.74585E+00_JPRB, 0.74568E+00_JPRB, 0.74539E+00_JPRB, 0.74486E+00_JPRB, 0.74406E+00_JPRB, 0.74388E+00_JPRB ,&
&  0.74366E+00_JPRB, 0.74321E+00_JPRB, 0.74242E+00_JPRB, 0.74284E+00_JPRB, 0.74342E+00_JPRB, 0.74369E+00_JPRB, 0.74361E+00_JPRB ,&
&  0.74599E+00_JPRB, 0.74947E+00_JPRB, 0.75370E+00_JPRB, 0.75941E+00_JPRB, 0.76370E+00_JPRB, 0.76647E+00_JPRB, 0.76845E+00_JPRB ,&
&  0.76922E+00_JPRB, 0.76993E+00_JPRB, 0.77045E+00_JPRB, 0.77121E+00_JPRB, 0.77149E+00_JPRB, 0.77234E+00_JPRB, 0.77384E+00_JPRB ,&
&  0.77489E+00_JPRB, 0.77518E+00_JPRB, 0.77504E+00_JPRB, 0.77469E+00_JPRB, 0.77425E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.77832E+00_JPRB, 0.77692E+00_JPRB, 0.77550E+00_JPRB, 0.77319E+00_JPRB, 0.77191E+00_JPRB, 0.76978E+00_JPRB, 0.76799E+00_JPRB ,&
&  0.76575E+00_JPRB, 0.76323E+00_JPRB, 0.76125E+00_JPRB, 0.75959E+00_JPRB, 0.75781E+00_JPRB, 0.75605E+00_JPRB, 0.75420E+00_JPRB ,&
&  0.75226E+00_JPRB, 0.75005E+00_JPRB, 0.74909E+00_JPRB, 0.74764E+00_JPRB, 0.74599E+00_JPRB, 0.74408E+00_JPRB, 0.73955E+00_JPRB ,&
&  0.73500E+00_JPRB, 0.72944E+00_JPRB, 0.72365E+00_JPRB, 0.71860E+00_JPRB, 0.71567E+00_JPRB, 0.69420E+00_JPRB, 0.69063E+00_JPRB ,&
&  0.69853E+00_JPRB, 0.70321E+00_JPRB, 0.70866E+00_JPRB, 0.71418E+00_JPRB, 0.70329E+00_JPRB, 0.68953E+00_JPRB, 0.68128E+00_JPRB ,&
&  0.66147E+00_JPRB, 0.66871E+00_JPRB, 0.67220E+00_JPRB, 0.67569E+00_JPRB, 0.68114E+00_JPRB, 0.68135E+00_JPRB, 0.68493E+00_JPRB ,&
&  0.69795E+00_JPRB, 0.69824E+00_JPRB, 0.70009E+00_JPRB, 0.70206E+00_JPRB, 0.70398E+00_JPRB, 0.70595E+00_JPRB, 0.70967E+00_JPRB ,&
&  0.71276E+00_JPRB, 0.71637E+00_JPRB, 0.72040E+00_JPRB, 0.72248E+00_JPRB, 0.72450E+00_JPRB, 0.72714E+00_JPRB, 0.72956E+00_JPRB ,&
&  0.73076E+00_JPRB, 0.73224E+00_JPRB, 0.73393E+00_JPRB, 0.73566E+00_JPRB, 0.73769E+00_JPRB, 0.73916E+00_JPRB, 0.74042E+00_JPRB ,&
&  0.74144E+00_JPRB, 0.74125E+00_JPRB, 0.74125E+00_JPRB, 0.74156E+00_JPRB, 0.74152E+00_JPRB, 0.74122E+00_JPRB, 0.74154E+00_JPRB ,&
&  0.74238E+00_JPRB, 0.74304E+00_JPRB, 0.74416E+00_JPRB, 0.74586E+00_JPRB, 0.75009E+00_JPRB, 0.75590E+00_JPRB, 0.76727E+00_JPRB ,&
&  0.77647E+00_JPRB, 0.78448E+00_JPRB, 0.78739E+00_JPRB, 0.79032E+00_JPRB, 0.79290E+00_JPRB, 0.79376E+00_JPRB, 0.79402E+00_JPRB ,&
&  0.79292E+00_JPRB, 0.79075E+00_JPRB, 0.78822E+00_JPRB, 0.78560E+00_JPRB, 0.78375E+00_JPRB, 0.78215E+00_JPRB, 0.78157E+00_JPRB ,&
&  0.78178E+00_JPRB, 0.78286E+00_JPRB, 0.78307E+00_JPRB, 0.78148E+00_JPRB, 0.77960E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.78992E+00_JPRB, 0.78627E+00_JPRB, 0.78476E+00_JPRB, 0.78146E+00_JPRB, 0.77865E+00_JPRB, 0.77462E+00_JPRB, 0.77125E+00_JPRB ,&
&  0.76732E+00_JPRB, 0.76386E+00_JPRB, 0.76073E+00_JPRB, 0.75785E+00_JPRB, 0.75632E+00_JPRB, 0.75471E+00_JPRB, 0.75366E+00_JPRB ,&
&  0.75238E+00_JPRB, 0.75062E+00_JPRB, 0.74966E+00_JPRB, 0.74833E+00_JPRB, 0.74714E+00_JPRB, 0.74580E+00_JPRB, 0.74225E+00_JPRB ,&
&  0.73561E+00_JPRB, 0.72657E+00_JPRB, 0.70694E+00_JPRB, 0.70975E+00_JPRB, 0.70718E+00_JPRB, 0.70480E+00_JPRB, 0.70622E+00_JPRB ,&
&  0.69766E+00_JPRB, 0.69530E+00_JPRB, 0.68446E+00_JPRB, 0.67393E+00_JPRB, 0.66864E+00_JPRB, 0.65727E+00_JPRB, 0.65568E+00_JPRB ,&
&  0.65150E+00_JPRB, 0.64934E+00_JPRB, 0.65026E+00_JPRB, 0.64960E+00_JPRB, 0.65521E+00_JPRB, 0.65814E+00_JPRB, 0.66268E+00_JPRB ,&
&  0.67083E+00_JPRB, 0.67748E+00_JPRB, 0.68213E+00_JPRB, 0.67610E+00_JPRB, 0.67940E+00_JPRB, 0.68277E+00_JPRB, 0.68711E+00_JPRB ,&
&  0.69131E+00_JPRB, 0.69448E+00_JPRB, 0.69793E+00_JPRB, 0.70060E+00_JPRB, 0.70347E+00_JPRB, 0.70765E+00_JPRB, 0.71188E+00_JPRB ,&
&  0.71690E+00_JPRB, 0.72047E+00_JPRB, 0.72278E+00_JPRB, 0.72532E+00_JPRB, 0.72792E+00_JPRB, 0.73054E+00_JPRB, 0.73380E+00_JPRB ,&
&  0.73773E+00_JPRB, 0.74244E+00_JPRB, 0.74749E+00_JPRB, 0.75274E+00_JPRB, 0.75768E+00_JPRB, 0.76402E+00_JPRB, 0.77133E+00_JPRB ,&
&  0.78191E+00_JPRB, 0.79154E+00_JPRB, 0.80616E+00_JPRB, 0.81487E+00_JPRB, 0.83379E+00_JPRB, 0.85103E+00_JPRB, 0.86082E+00_JPRB ,&
&  0.85832E+00_JPRB, 0.85237E+00_JPRB, 0.84689E+00_JPRB, 0.84114E+00_JPRB, 0.83637E+00_JPRB, 0.83026E+00_JPRB, 0.82608E+00_JPRB ,&
&  0.82167E+00_JPRB, 0.81751E+00_JPRB, 0.81339E+00_JPRB, 0.80938E+00_JPRB, 0.80594E+00_JPRB, 0.80203E+00_JPRB, 0.79936E+00_JPRB ,&
&  0.79799E+00_JPRB, 0.79820E+00_JPRB, 0.79786E+00_JPRB, 0.79645E+00_JPRB, 0.79347E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.80442E+00_JPRB, 0.80328E+00_JPRB, 0.80431E+00_JPRB, 0.80229E+00_JPRB, 0.79874E+00_JPRB, 0.79483E+00_JPRB, 0.78718E+00_JPRB ,&
&  0.77937E+00_JPRB, 0.77159E+00_JPRB, 0.76452E+00_JPRB, 0.75912E+00_JPRB, 0.75459E+00_JPRB, 0.75118E+00_JPRB, 0.74979E+00_JPRB ,&
&  0.74785E+00_JPRB, 0.74756E+00_JPRB, 0.74744E+00_JPRB, 0.74769E+00_JPRB, 0.74895E+00_JPRB, 0.74965E+00_JPRB, 0.75014E+00_JPRB ,&
&  0.74643E+00_JPRB, 0.73605E+00_JPRB, 0.72175E+00_JPRB, 0.72449E+00_JPRB, 0.72630E+00_JPRB, 0.72997E+00_JPRB, 0.73414E+00_JPRB ,&
&  0.73373E+00_JPRB, 0.70410E+00_JPRB, 0.71162E+00_JPRB, 0.69655E+00_JPRB, 0.68765E+00_JPRB, 0.67799E+00_JPRB, 0.65674E+00_JPRB ,&
&  0.64934E+00_JPRB, 0.63729E+00_JPRB, 0.63049E+00_JPRB, 0.63056E+00_JPRB, 0.63347E+00_JPRB, 0.64116E+00_JPRB, 0.65082E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.66711E+00_JPRB, 0.66861E+00_JPRB, 0.66797E+00_JPRB, 0.67107E+00_JPRB, 0.67482E+00_JPRB, 0.67238E+00_JPRB ,&
&  0.67345E+00_JPRB, 0.67277E+00_JPRB, 0.67269E+00_JPRB, 0.67761E+00_JPRB, 0.68619E+00_JPRB, 0.69084E+00_JPRB, 0.69535E+00_JPRB ,&
&  0.70168E+00_JPRB, 0.70542E+00_JPRB, 0.70772E+00_JPRB, 0.71077E+00_JPRB, 0.71495E+00_JPRB, 0.71941E+00_JPRB, 0.72620E+00_JPRB ,&
&  0.73419E+00_JPRB, 0.74804E+00_JPRB, 0.76092E+00_JPRB, 0.77996E+00_JPRB, 0.79752E+00_JPRB, 0.82038E+00_JPRB, 0.84017E+00_JPRB ,&
&  0.86113E+00_JPRB, 0.87785E+00_JPRB, 0.89618E+00_JPRB, 0.91285E+00_JPRB, 0.92568E+00_JPRB, 0.91984E+00_JPRB, 0.90238E+00_JPRB ,&
&  0.88937E+00_JPRB, 0.87080E+00_JPRB, 0.85931E+00_JPRB, 0.85028E+00_JPRB, 0.84369E+00_JPRB, 0.84049E+00_JPRB, 0.83715E+00_JPRB ,&
&  0.83522E+00_JPRB, 0.83206E+00_JPRB, 0.82842E+00_JPRB, 0.82355E+00_JPRB, 0.81978E+00_JPRB, 0.81591E+00_JPRB, 0.81367E+00_JPRB ,&
&  0.81044E+00_JPRB, 0.80968E+00_JPRB, 0.80786E+00_JPRB, 0.80615E+00_JPRB, 0.80496E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.81982E+00_JPRB, 0.82296E+00_JPRB, 0.82000E+00_JPRB, 0.81667E+00_JPRB, 0.80781E+00_JPRB, 0.80002E+00_JPRB, 0.78830E+00_JPRB ,&
&  0.77672E+00_JPRB, 0.76522E+00_JPRB, 0.75484E+00_JPRB, 0.74741E+00_JPRB, 0.74141E+00_JPRB, 0.73726E+00_JPRB, 0.73567E+00_JPRB ,&
&  0.73528E+00_JPRB, 0.73639E+00_JPRB, 0.73763E+00_JPRB, 0.73934E+00_JPRB, 0.74297E+00_JPRB, 0.74713E+00_JPRB, 0.75159E+00_JPRB ,&
&  0.75310E+00_JPRB, 0.74910E+00_JPRB, 0.73640E+00_JPRB, 0.73603E+00_JPRB, 0.74374E+00_JPRB, 0.74068E+00_JPRB, 0.75638E+00_JPRB ,&
&  0.76718E+00_JPRB, 0.77358E+00_JPRB, 0.76423E+00_JPRB, 0.75765E+00_JPRB, 0.74453E+00_JPRB, 0.67917E+00_JPRB, 0.67333E+00_JPRB ,&
&  0.66153E+00_JPRB, 0.64477E+00_JPRB, 0.63720E+00_JPRB, 0.63095E+00_JPRB, 0.62954E+00_JPRB, 0.63865E+00_JPRB, 0.65208E+00_JPRB ,&
&  0.65529E+00_JPRB, 0.67686E+00_JPRB, 0.67617E+00_JPRB, 0.67444E+00_JPRB, 0.67500E+00_JPRB, 0.67833E+00_JPRB, 0.67507E+00_JPRB ,&
&  0.67609E+00_JPRB, 0.67665E+00_JPRB, 0.67776E+00_JPRB, 0.68458E+00_JPRB, 0.69165E+00_JPRB, 0.69749E+00_JPRB, 0.70561E+00_JPRB ,&
&  0.72145E+00_JPRB, 0.73247E+00_JPRB, 0.73885E+00_JPRB, 0.74468E+00_JPRB, 0.75405E+00_JPRB, 0.76538E+00_JPRB, 0.78079E+00_JPRB ,&
&  0.79196E+00_JPRB, 0.80558E+00_JPRB, 0.82055E+00_JPRB, 0.83625E+00_JPRB, 0.85148E+00_JPRB, 0.86793E+00_JPRB, 0.88536E+00_JPRB ,&
&  0.89415E+00_JPRB, 0.90640E+00_JPRB, 0.91831E+00_JPRB, 0.94013E+00_JPRB, 0.93818E+00_JPRB, 0.92019E+00_JPRB, 0.90615E+00_JPRB ,&
&  0.89516E+00_JPRB, 0.89481E+00_JPRB, 0.88450E+00_JPRB, 0.87776E+00_JPRB, 0.87035E+00_JPRB, 0.86731E+00_JPRB, 0.86218E+00_JPRB ,&
&  0.85841E+00_JPRB, 0.85337E+00_JPRB, 0.84672E+00_JPRB, 0.83892E+00_JPRB, 0.83338E+00_JPRB, 0.82769E+00_JPRB, 0.82419E+00_JPRB ,&
&  0.81964E+00_JPRB, 0.81782E+00_JPRB, 0.81678E+00_JPRB, 0.81684E+00_JPRB, 0.81616E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.83063E+00_JPRB, 0.83020E+00_JPRB, 0.82013E+00_JPRB, 0.81483E+00_JPRB, 0.80120E+00_JPRB, 0.78707E+00_JPRB, 0.76968E+00_JPRB ,&
&  0.75584E+00_JPRB, 0.74527E+00_JPRB, 0.73557E+00_JPRB, 0.73045E+00_JPRB, 0.72799E+00_JPRB, 0.72696E+00_JPRB, 0.72882E+00_JPRB ,&
&  0.73144E+00_JPRB, 0.72958E+00_JPRB, 0.73300E+00_JPRB, 0.73901E+00_JPRB, 0.74400E+00_JPRB, 0.74853E+00_JPRB, 0.75336E+00_JPRB ,&
&  0.75362E+00_JPRB, 0.75312E+00_JPRB, 0.75173E+00_JPRB, 0.74598E+00_JPRB, 0.74458E+00_JPRB, 0.75633E+00_JPRB, 0.77349E+00_JPRB ,&
&  0.77027E+00_JPRB, 0.77378E+00_JPRB, 0.78520E+00_JPRB, 0.79106E+00_JPRB, 0.77408E+00_JPRB, 0.73945E+00_JPRB, 0.70017E+00_JPRB ,&
&  0.67344E+00_JPRB, 0.64322E+00_JPRB, 0.62965E+00_JPRB, 0.62284E+00_JPRB, 0.62795E+00_JPRB, 0.63883E+00_JPRB, 0.65910E+00_JPRB ,&
&  0.67151E+00_JPRB, 0.67656E+00_JPRB, 0.68251E+00_JPRB, 0.68780E+00_JPRB, 0.68804E+00_JPRB, 0.69122E+00_JPRB, 0.69185E+00_JPRB ,&
&  0.69783E+00_JPRB, 0.70241E+00_JPRB, 0.70786E+00_JPRB, 0.71854E+00_JPRB, 0.73426E+00_JPRB, 0.75140E+00_JPRB, 0.77746E+00_JPRB ,&
&  0.83470E+00_JPRB, 0.85700E+00_JPRB, 0.87525E+00_JPRB, 0.88494E+00_JPRB, 0.89831E+00_JPRB, 0.90359E+00_JPRB, 0.89467E+00_JPRB ,&
&  0.89469E+00_JPRB, 0.91007E+00_JPRB, 0.91305E+00_JPRB, 0.91640E+00_JPRB, 0.92055E+00_JPRB, 0.92534E+00_JPRB, 0.93111E+00_JPRB ,&
&  0.92135E+00_JPRB, 0.93362E+00_JPRB, 0.10062E+01_JPRB, 0.10256E+01_JPRB, 0.10075E+01_JPRB, 0.98501E+00_JPRB, 0.96157E+00_JPRB ,&
&  0.94255E+00_JPRB, 0.92555E+00_JPRB, 0.91099E+00_JPRB, 0.89902E+00_JPRB, 0.89064E+00_JPRB, 0.88632E+00_JPRB, 0.88586E+00_JPRB ,&
&  0.87869E+00_JPRB, 0.87004E+00_JPRB, 0.85989E+00_JPRB, 0.85269E+00_JPRB, 0.84689E+00_JPRB, 0.84047E+00_JPRB, 0.83688E+00_JPRB ,&
&  0.83350E+00_JPRB, 0.83415E+00_JPRB, 0.83466E+00_JPRB, 0.83521E+00_JPRB, 0.83391E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.83071E+00_JPRB, 0.83098E+00_JPRB, 0.82620E+00_JPRB, 0.81175E+00_JPRB, 0.78528E+00_JPRB, 0.76025E+00_JPRB, 0.74215E+00_JPRB ,&
&  0.73349E+00_JPRB, 0.72855E+00_JPRB, 0.72268E+00_JPRB, 0.71994E+00_JPRB, 0.71932E+00_JPRB, 0.72061E+00_JPRB, 0.72379E+00_JPRB ,&
&  0.72881E+00_JPRB, 0.73378E+00_JPRB, 0.73679E+00_JPRB, 0.73855E+00_JPRB, 0.74310E+00_JPRB, 0.75080E+00_JPRB, 0.75946E+00_JPRB ,&
&  0.76536E+00_JPRB, 0.76659E+00_JPRB, 0.76980E+00_JPRB, 0.77048E+00_JPRB, 0.77119E+00_JPRB, 0.77248E+00_JPRB, 0.77382E+00_JPRB ,&
&  0.78353E+00_JPRB, 0.79458E+00_JPRB, 0.79442E+00_JPRB, 0.80350E+00_JPRB, 0.81332E+00_JPRB, 0.78099E+00_JPRB, 0.72951E+00_JPRB ,&
&  0.67869E+00_JPRB, 0.64225E+00_JPRB, 0.62657E+00_JPRB, 0.62271E+00_JPRB, 0.63241E+00_JPRB, 0.65460E+00_JPRB, 0.68164E+00_JPRB ,&
&  0.68570E+00_JPRB, 0.69220E+00_JPRB, 0.70299E+00_JPRB, 0.70883E+00_JPRB, 0.70924E+00_JPRB, 0.70832E+00_JPRB, 0.70955E+00_JPRB ,&
&  0.71594E+00_JPRB, 0.73241E+00_JPRB, 0.76002E+00_JPRB, 0.79865E+00_JPRB, 0.85361E+00_JPRB, 0.88063E+00_JPRB, 0.93245E+00_JPRB ,&
&  0.10131E+01_JPRB, 0.10209E+01_JPRB, 0.10360E+01_JPRB, 0.10378E+01_JPRB, 0.10371E+01_JPRB, 0.10301E+01_JPRB, 0.10287E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10230E+01_JPRB, 0.10095E+01_JPRB, 0.99407E+00_JPRB, 0.99049E+00_JPRB, 0.99727E+00_JPRB, 0.10157E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.10979E+01_JPRB, 0.12350E+01_JPRB, 0.11327E+01_JPRB, 0.10826E+01_JPRB, 0.10615E+01_JPRB, 0.10303E+01_JPRB ,&
&  0.10030E+01_JPRB, 0.98401E+00_JPRB, 0.97227E+00_JPRB, 0.95892E+00_JPRB, 0.93287E+00_JPRB, 0.91465E+00_JPRB, 0.90108E+00_JPRB ,&
&  0.89032E+00_JPRB, 0.87927E+00_JPRB, 0.87094E+00_JPRB, 0.86510E+00_JPRB, 0.85899E+00_JPRB, 0.85466E+00_JPRB, 0.85387E+00_JPRB ,&
&  0.85121E+00_JPRB, 0.84884E+00_JPRB, 0.84601E+00_JPRB, 0.83998E+00_JPRB, 0.83481E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.83124E+00_JPRB, 0.83105E+00_JPRB, 0.81747E+00_JPRB, 0.80141E+00_JPRB, 0.77080E+00_JPRB, 0.74788E+00_JPRB, 0.73233E+00_JPRB ,&
&  0.72457E+00_JPRB, 0.71757E+00_JPRB, 0.71290E+00_JPRB, 0.71091E+00_JPRB, 0.71177E+00_JPRB, 0.71575E+00_JPRB, 0.71917E+00_JPRB ,&
&  0.72244E+00_JPRB, 0.72809E+00_JPRB, 0.73525E+00_JPRB, 0.74121E+00_JPRB, 0.74670E+00_JPRB, 0.75337E+00_JPRB, 0.76188E+00_JPRB ,&
&  0.77187E+00_JPRB, 0.77835E+00_JPRB, 0.78303E+00_JPRB, 0.78839E+00_JPRB, 0.79893E+00_JPRB, 0.81405E+00_JPRB, 0.82293E+00_JPRB ,&
&  0.81786E+00_JPRB, 0.81379E+00_JPRB, 0.82419E+00_JPRB, 0.83339E+00_JPRB, 0.84226E+00_JPRB, 0.81851E+00_JPRB, 0.75326E+00_JPRB ,&
&  0.69346E+00_JPRB, 0.65635E+00_JPRB, 0.64986E+00_JPRB, 0.66216E+00_JPRB, 0.68629E+00_JPRB, 0.69804E+00_JPRB, 0.71092E+00_JPRB ,&
&  0.72105E+00_JPRB, 0.72414E+00_JPRB, 0.72648E+00_JPRB, 0.73266E+00_JPRB, 0.73563E+00_JPRB, 0.73575E+00_JPRB, 0.74079E+00_JPRB ,&
&  0.75695E+00_JPRB, 0.78401E+00_JPRB, 0.80558E+00_JPRB, 0.81225E+00_JPRB, 0.85581E+00_JPRB, 0.96366E+00_JPRB, 0.10629E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.11072E+01_JPRB, 0.11243E+01_JPRB, 0.11644E+01_JPRB, 0.11806E+01_JPRB, 0.11927E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.11719E+01_JPRB, 0.11408E+01_JPRB, 0.11129E+01_JPRB, 0.11064E+01_JPRB, 0.11296E+01_JPRB, 0.11761E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.11951E+01_JPRB, 0.10972E+01_JPRB, 0.10109E+01_JPRB, 0.98379E+00_JPRB, 0.99008E+00_JPRB, 0.10035E+01_JPRB ,&
&  0.99892E+00_JPRB, 0.99077E+00_JPRB, 0.98161E+00_JPRB, 0.97433E+00_JPRB, 0.93260E+00_JPRB, 0.90658E+00_JPRB, 0.88563E+00_JPRB ,&
&  0.87181E+00_JPRB, 0.86172E+00_JPRB, 0.86302E+00_JPRB, 0.85967E+00_JPRB, 0.85713E+00_JPRB, 0.85337E+00_JPRB, 0.84730E+00_JPRB ,&
&  0.84485E+00_JPRB, 0.84130E+00_JPRB, 0.83464E+00_JPRB, 0.82953E+00_JPRB, 0.82741E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.84753E+00_JPRB, 0.83607E+00_JPRB, 0.82433E+00_JPRB, 0.80358E+00_JPRB, 0.78758E+00_JPRB, 0.77814E+00_JPRB, 0.76993E+00_JPRB ,&
&  0.77102E+00_JPRB, 0.74890E+00_JPRB, 0.72617E+00_JPRB, 0.71138E+00_JPRB, 0.70410E+00_JPRB, 0.70340E+00_JPRB, 0.71013E+00_JPRB ,&
&  0.71971E+00_JPRB, 0.72846E+00_JPRB, 0.73715E+00_JPRB, 0.74743E+00_JPRB, 0.75752E+00_JPRB, 0.76535E+00_JPRB, 0.77335E+00_JPRB ,&
&  0.78413E+00_JPRB, 0.79548E+00_JPRB, 0.80747E+00_JPRB, 0.82523E+00_JPRB, 0.84185E+00_JPRB, 0.86071E+00_JPRB, 0.86739E+00_JPRB ,&
&  0.86362E+00_JPRB, 0.85529E+00_JPRB, 0.85391E+00_JPRB, 0.86208E+00_JPRB, 0.86610E+00_JPRB, 0.84538E+00_JPRB, 0.77338E+00_JPRB ,&
&  0.71343E+00_JPRB, 0.69298E+00_JPRB, 0.71004E+00_JPRB, 0.73433E+00_JPRB, 0.74413E+00_JPRB, 0.75282E+00_JPRB, 0.75365E+00_JPRB ,&
&  0.74951E+00_JPRB, 0.74755E+00_JPRB, 0.75598E+00_JPRB, 0.76611E+00_JPRB, 0.76824E+00_JPRB, 0.77146E+00_JPRB, 0.78161E+00_JPRB ,&
&  0.80830E+00_JPRB, 0.84100E+00_JPRB, 0.86723E+00_JPRB, 0.86432E+00_JPRB, 0.95739E+00_JPRB, 0.12548E+01_JPRB, 0.13023E+01_JPRB ,&
&  0.13310E+01_JPRB, 0.13478E+01_JPRB, 0.13810E+01_JPRB, 0.14041E+01_JPRB, 0.13791E+01_JPRB, 0.13660E+01_JPRB, 0.13599E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.13019E+01_JPRB, 0.12956E+01_JPRB, 0.13359E+01_JPRB, 0.13557E+01_JPRB, 0.13586E+01_JPRB, 0.13611E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.12628E+01_JPRB, 0.11761E+01_JPRB, 0.10972E+01_JPRB, 0.10584E+01_JPRB, 0.10344E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.10048E+01_JPRB, 0.99950E+00_JPRB, 0.10003E+01_JPRB, 0.99885E+00_JPRB, 0.96491E+00_JPRB, 0.91946E+00_JPRB, 0.89254E+00_JPRB ,&
&  0.87552E+00_JPRB, 0.86074E+00_JPRB, 0.84647E+00_JPRB, 0.84032E+00_JPRB, 0.84810E+00_JPRB, 0.85512E+00_JPRB, 0.85172E+00_JPRB ,&
&  0.85496E+00_JPRB, 0.84936E+00_JPRB, 0.84672E+00_JPRB, 0.84639E+00_JPRB, 0.85195E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.85117E+00_JPRB, 0.84278E+00_JPRB, 0.83114E+00_JPRB, 0.81845E+00_JPRB, 0.81306E+00_JPRB, 0.80700E+00_JPRB, 0.79056E+00_JPRB ,&
&  0.79536E+00_JPRB, 0.75592E+00_JPRB, 0.73196E+00_JPRB, 0.72275E+00_JPRB, 0.72245E+00_JPRB, 0.71751E+00_JPRB, 0.71125E+00_JPRB ,&
&  0.71480E+00_JPRB, 0.72793E+00_JPRB, 0.74336E+00_JPRB, 0.75567E+00_JPRB, 0.76804E+00_JPRB, 0.77782E+00_JPRB, 0.78664E+00_JPRB ,&
&  0.79870E+00_JPRB, 0.81382E+00_JPRB, 0.83346E+00_JPRB, 0.85387E+00_JPRB, 0.87177E+00_JPRB, 0.89316E+00_JPRB, 0.90183E+00_JPRB ,&
&  0.90608E+00_JPRB, 0.89950E+00_JPRB, 0.88572E+00_JPRB, 0.88041E+00_JPRB, 0.87492E+00_JPRB, 0.87010E+00_JPRB, 0.84033E+00_JPRB ,&
&  0.76694E+00_JPRB, 0.75972E+00_JPRB, 0.78391E+00_JPRB, 0.79217E+00_JPRB, 0.79062E+00_JPRB, 0.78407E+00_JPRB, 0.78204E+00_JPRB ,&
&  0.78313E+00_JPRB, 0.78399E+00_JPRB, 0.78913E+00_JPRB, 0.79049E+00_JPRB, 0.79507E+00_JPRB, 0.80232E+00_JPRB, 0.82859E+00_JPRB ,&
&  0.88368E+00_JPRB, 0.86727E+00_JPRB, 0.88640E+00_JPRB, 0.95532E+00_JPRB, 0.11957E+01_JPRB, 0.13433E+01_JPRB, 0.14481E+01_JPRB ,&
&  0.15986E+01_JPRB, 0.16340E+01_JPRB, 0.16285E+01_JPRB, 0.16262E+01_JPRB, 0.16268E+01_JPRB, 0.16285E+01_JPRB, 0.16341E+01_JPRB ,&
&  0.15869E+01_JPRB, 0.15781E+01_JPRB, 0.15774E+01_JPRB, 0.15989E+01_JPRB, 0.15806E+01_JPRB, 0.15520E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.14635E+01_JPRB, 0.13977E+01_JPRB, 0.13072E+01_JPRB, 0.12199E+01_JPRB, 0.11470E+01_JPRB, 0.11118E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10592E+01_JPRB, 0.10479E+01_JPRB, 0.10348E+01_JPRB, 0.10187E+01_JPRB, 0.10008E+01_JPRB, 0.99607E+00_JPRB ,&
&  0.95240E+00_JPRB, 0.92109E+00_JPRB, 0.88662E+00_JPRB, 0.87344E+00_JPRB, 0.87747E+00_JPRB, 0.88521E+00_JPRB, 0.89026E+00_JPRB ,&
&  0.89703E+00_JPRB, 0.88452E+00_JPRB, 0.88005E+00_JPRB, 0.87331E+00_JPRB, 0.86638E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.85667E+00_JPRB, 0.85522E+00_JPRB, 0.84670E+00_JPRB, 0.83989E+00_JPRB, 0.84230E+00_JPRB, 0.83881E+00_JPRB, 0.83267E+00_JPRB ,&
&  0.86728E+00_JPRB, 0.84285E+00_JPRB, 0.84854E+00_JPRB, 0.78132E+00_JPRB, 0.75367E+00_JPRB, 0.73975E+00_JPRB, 0.72299E+00_JPRB ,&
&  0.72215E+00_JPRB, 0.73078E+00_JPRB, 0.74137E+00_JPRB, 0.75446E+00_JPRB, 0.77081E+00_JPRB, 0.78268E+00_JPRB, 0.79242E+00_JPRB ,&
&  0.80459E+00_JPRB, 0.82068E+00_JPRB, 0.83978E+00_JPRB, 0.85677E+00_JPRB, 0.87543E+00_JPRB, 0.89919E+00_JPRB, 0.90887E+00_JPRB ,&
&  0.89095E+00_JPRB, 0.88138E+00_JPRB, 0.88151E+00_JPRB, 0.87516E+00_JPRB, 0.87977E+00_JPRB, 0.87826E+00_JPRB, 0.88392E+00_JPRB ,&
&  0.87392E+00_JPRB, 0.84396E+00_JPRB, 0.85059E+00_JPRB, 0.83941E+00_JPRB, 0.82969E+00_JPRB, 0.81946E+00_JPRB, 0.81046E+00_JPRB ,&
&  0.80572E+00_JPRB, 0.80712E+00_JPRB, 0.81473E+00_JPRB, 0.82157E+00_JPRB, 0.82992E+00_JPRB, 0.85417E+00_JPRB, 0.90338E+00_JPRB ,&
&  0.98139E+00_JPRB, 0.93458E+00_JPRB, 0.10534E+01_JPRB, 0.11200E+01_JPRB, 0.13648E+01_JPRB, 0.16193E+01_JPRB, 0.17229E+01_JPRB ,&
&  0.19806E+01_JPRB, 0.19989E+01_JPRB, 0.19638E+01_JPRB, 0.19306E+01_JPRB, 0.19353E+01_JPRB, 0.18912E+01_JPRB, 0.18166E+01_JPRB ,&
&  0.17747E+01_JPRB, 0.17063E+01_JPRB, 0.17776E+01_JPRB, 0.17581E+01_JPRB, 0.16991E+01_JPRB, 0.16568E+01_JPRB, 0.15890E+01_JPRB ,&
&  0.15360E+01_JPRB, 0.14814E+01_JPRB, 0.14302E+01_JPRB, 0.13167E+01_JPRB, 0.12768E+01_JPRB, 0.12371E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.10829E+01_JPRB, 0.10418E+01_JPRB, 0.10553E+01_JPRB, 0.10275E+01_JPRB, 0.10141E+01_JPRB, 0.10097E+01_JPRB ,&
&  0.98238E+00_JPRB, 0.94856E+00_JPRB, 0.94990E+00_JPRB, 0.94862E+00_JPRB, 0.93638E+00_JPRB, 0.92680E+00_JPRB, 0.93444E+00_JPRB ,&
&  0.91567E+00_JPRB, 0.90750E+00_JPRB, 0.91399E+00_JPRB, 0.89271E+00_JPRB, 0.87314E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.88568E+00_JPRB, 0.88171E+00_JPRB, 0.86993E+00_JPRB, 0.86776E+00_JPRB, 0.87485E+00_JPRB, 0.87992E+00_JPRB, 0.88460E+00_JPRB ,&
&  0.90518E+00_JPRB, 0.91139E+00_JPRB, 0.90823E+00_JPRB, 0.89066E+00_JPRB, 0.86280E+00_JPRB, 0.80262E+00_JPRB, 0.73396E+00_JPRB ,&
&  0.72174E+00_JPRB, 0.73975E+00_JPRB, 0.75968E+00_JPRB, 0.77019E+00_JPRB, 0.78493E+00_JPRB, 0.80051E+00_JPRB, 0.81362E+00_JPRB ,&
&  0.82594E+00_JPRB, 0.83583E+00_JPRB, 0.85108E+00_JPRB, 0.86966E+00_JPRB, 0.88635E+00_JPRB, 0.90744E+00_JPRB, 0.92645E+00_JPRB ,&
&  0.89535E+00_JPRB, 0.87706E+00_JPRB, 0.86983E+00_JPRB, 0.86457E+00_JPRB, 0.90162E+00_JPRB, 0.91333E+00_JPRB, 0.92132E+00_JPRB ,&
&  0.92452E+00_JPRB, 0.91830E+00_JPRB, 0.90262E+00_JPRB, 0.88962E+00_JPRB, 0.87638E+00_JPRB, 0.86727E+00_JPRB, 0.85595E+00_JPRB ,&
&  0.84927E+00_JPRB, 0.84939E+00_JPRB, 0.85958E+00_JPRB, 0.87182E+00_JPRB, 0.89133E+00_JPRB, 0.95117E+00_JPRB, 0.10509E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.12504E+01_JPRB, 0.13112E+01_JPRB, 0.13332E+01_JPRB, 0.15458E+01_JPRB, 0.17590E+01_JPRB, 0.18677E+01_JPRB ,&
&  0.20285E+01_JPRB, 0.19976E+01_JPRB, 0.20835E+01_JPRB, 0.21731E+01_JPRB, 0.22466E+01_JPRB, 0.21939E+01_JPRB, 0.20438E+01_JPRB ,&
&  0.19790E+01_JPRB, 0.20244E+01_JPRB, 0.20075E+01_JPRB, 0.19870E+01_JPRB, 0.19033E+01_JPRB, 0.18447E+01_JPRB, 0.17693E+01_JPRB ,&
&  0.17529E+01_JPRB, 0.17426E+01_JPRB, 0.16814E+01_JPRB, 0.16044E+01_JPRB, 0.15086E+01_JPRB, 0.14019E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.10731E+01_JPRB, 0.10288E+01_JPRB, 0.10281E+01_JPRB, 0.10412E+01_JPRB, 0.10169E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.10078E+01_JPRB, 0.10297E+01_JPRB, 0.10105E+01_JPRB, 0.98358E+00_JPRB, 0.95759E+00_JPRB, 0.94208E+00_JPRB ,&
&  0.91210E+00_JPRB, 0.92175E+00_JPRB, 0.92358E+00_JPRB, 0.91087E+00_JPRB, 0.89349E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.91181E+00_JPRB, 0.89475E+00_JPRB, 0.88414E+00_JPRB, 0.88944E+00_JPRB, 0.90085E+00_JPRB, 0.91522E+00_JPRB, 0.93661E+00_JPRB ,&
&  0.95871E+00_JPRB, 0.96703E+00_JPRB, 0.95042E+00_JPRB, 0.92269E+00_JPRB, 0.89864E+00_JPRB, 0.88124E+00_JPRB, 0.83904E+00_JPRB ,&
&  0.78402E+00_JPRB, 0.79870E+00_JPRB, 0.78955E+00_JPRB, 0.79806E+00_JPRB, 0.82577E+00_JPRB, 0.83959E+00_JPRB, 0.85486E+00_JPRB ,&
&  0.87804E+00_JPRB, 0.88636E+00_JPRB, 0.89451E+00_JPRB, 0.90448E+00_JPRB, 0.92067E+00_JPRB, 0.96231E+00_JPRB, 0.99865E+00_JPRB ,&
&  0.95832E+00_JPRB, 0.93126E+00_JPRB, 0.92575E+00_JPRB, 0.94314E+00_JPRB, 0.97426E+00_JPRB, 0.99673E+00_JPRB, 0.10018E+01_JPRB ,&
&  0.99815E+00_JPRB, 0.98735E+00_JPRB, 0.97233E+00_JPRB, 0.95764E+00_JPRB, 0.93917E+00_JPRB, 0.91983E+00_JPRB, 0.90726E+00_JPRB ,&
&  0.89965E+00_JPRB, 0.90122E+00_JPRB, 0.91372E+00_JPRB, 0.92551E+00_JPRB, 0.94961E+00_JPRB, 0.10667E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.14984E+01_JPRB, 0.16778E+01_JPRB, 0.17493E+01_JPRB, 0.18567E+01_JPRB, 0.19664E+01_JPRB, 0.20798E+01_JPRB, 0.21655E+01_JPRB ,&
&  0.22426E+01_JPRB, 0.21555E+01_JPRB, 0.23554E+01_JPRB, 0.24414E+01_JPRB, 0.24289E+01_JPRB, 0.24404E+01_JPRB, 0.22827E+01_JPRB ,&
&  0.21639E+01_JPRB, 0.20614E+01_JPRB, 0.21156E+01_JPRB, 0.21138E+01_JPRB, 0.20666E+01_JPRB, 0.20855E+01_JPRB, 0.20246E+01_JPRB ,&
&  0.19893E+01_JPRB, 0.19328E+01_JPRB, 0.17157E+01_JPRB, 0.15411E+01_JPRB, 0.13516E+01_JPRB, 0.13980E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.10674E+01_JPRB, 0.10699E+01_JPRB, 0.10710E+01_JPRB, 0.11037E+01_JPRB, 0.11382E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.11389E+01_JPRB, 0.11096E+01_JPRB, 0.10851E+01_JPRB, 0.10580E+01_JPRB, 0.10168E+01_JPRB, 0.98474E+00_JPRB ,&
&  0.98906E+00_JPRB, 0.99111E+00_JPRB, 0.97268E+00_JPRB, 0.94541E+00_JPRB, 0.92732E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.96165E+00_JPRB, 0.94351E+00_JPRB, 0.93292E+00_JPRB, 0.93575E+00_JPRB, 0.94572E+00_JPRB, 0.96552E+00_JPRB, 0.98873E+00_JPRB ,&
&  0.10057E+01_JPRB, 0.10029E+01_JPRB, 0.97346E+00_JPRB, 0.94720E+00_JPRB, 0.93015E+00_JPRB, 0.91482E+00_JPRB, 0.91297E+00_JPRB ,&
&  0.84303E+00_JPRB, 0.84662E+00_JPRB, 0.80603E+00_JPRB, 0.81916E+00_JPRB, 0.86714E+00_JPRB, 0.88306E+00_JPRB, 0.90078E+00_JPRB ,&
&  0.91302E+00_JPRB, 0.93523E+00_JPRB, 0.94255E+00_JPRB, 0.95410E+00_JPRB, 0.97790E+00_JPRB, 0.10380E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.10485E+01_JPRB, 0.10647E+01_JPRB, 0.10955E+01_JPRB, 0.10984E+01_JPRB, 0.11284E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11317E+01_JPRB, 0.10951E+01_JPRB, 0.10534E+01_JPRB, 0.10179E+01_JPRB, 0.99134E+00_JPRB, 0.97077E+00_JPRB, 0.96345E+00_JPRB ,&
&  0.96325E+00_JPRB, 0.96704E+00_JPRB, 0.97447E+00_JPRB, 0.99542E+00_JPRB, 0.10183E+01_JPRB, 0.11801E+01_JPRB, 0.14889E+01_JPRB ,&
&  0.18611E+01_JPRB, 0.19568E+01_JPRB, 0.21267E+01_JPRB, 0.25317E+01_JPRB, 0.26636E+01_JPRB, 0.26932E+01_JPRB, 0.26331E+01_JPRB ,&
&  0.25617E+01_JPRB, 0.24182E+01_JPRB, 0.23778E+01_JPRB, 0.24068E+01_JPRB, 0.23643E+01_JPRB, 0.24407E+01_JPRB, 0.23485E+01_JPRB ,&
&  0.21801E+01_JPRB, 0.21251E+01_JPRB, 0.21186E+01_JPRB, 0.20936E+01_JPRB, 0.20371E+01_JPRB, 0.20578E+01_JPRB, 0.20162E+01_JPRB ,&
&  0.19367E+01_JPRB, 0.17269E+01_JPRB, 0.13946E+01_JPRB, 0.13323E+01_JPRB, 0.11382E+01_JPRB, 0.11849E+01_JPRB, 0.13749E+01_JPRB ,&
&  0.13870E+01_JPRB, 0.13122E+01_JPRB, 0.12347E+01_JPRB, 0.12122E+01_JPRB, 0.12329E+01_JPRB, 0.13635E+01_JPRB, 0.13498E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.12822E+01_JPRB, 0.12725E+01_JPRB, 0.12402E+01_JPRB, 0.11914E+01_JPRB, 0.11486E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.10912E+01_JPRB, 0.10697E+01_JPRB, 0.10172E+01_JPRB, 0.97935E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.10462E+01_JPRB, 0.10246E+01_JPRB, 0.10045E+01_JPRB, 0.10031E+01_JPRB, 0.10123E+01_JPRB, 0.10292E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10288E+01_JPRB, 0.99172E+00_JPRB, 0.97304E+00_JPRB, 0.95807E+00_JPRB, 0.93767E+00_JPRB, 0.93295E+00_JPRB ,&
&  0.95208E+00_JPRB, 0.94490E+00_JPRB, 0.88991E+00_JPRB, 0.83868E+00_JPRB, 0.85970E+00_JPRB, 0.87266E+00_JPRB, 0.90680E+00_JPRB ,&
&  0.92907E+00_JPRB, 0.97618E+00_JPRB, 0.99943E+00_JPRB, 0.10176E+01_JPRB, 0.10608E+01_JPRB, 0.11507E+01_JPRB, 0.12333E+01_JPRB ,&
&  0.12559E+01_JPRB, 0.12784E+01_JPRB, 0.13243E+01_JPRB, 0.13742E+01_JPRB, 0.13803E+01_JPRB, 0.12935E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.11856E+01_JPRB, 0.11175E+01_JPRB, 0.10776E+01_JPRB, 0.10581E+01_JPRB, 0.10538E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.10689E+01_JPRB, 0.10453E+01_JPRB, 0.10433E+01_JPRB, 0.10743E+01_JPRB, 0.12513E+01_JPRB, 0.14774E+01_JPRB ,&
&  0.18349E+01_JPRB, 0.19105E+01_JPRB, 0.19942E+01_JPRB, 0.23505E+01_JPRB, 0.26221E+01_JPRB, 0.27542E+01_JPRB, 0.28049E+01_JPRB ,&
&  0.27512E+01_JPRB, 0.26512E+01_JPRB, 0.25550E+01_JPRB, 0.24223E+01_JPRB, 0.23999E+01_JPRB, 0.24227E+01_JPRB, 0.23567E+01_JPRB ,&
&  0.21932E+01_JPRB, 0.20787E+01_JPRB, 0.20443E+01_JPRB, 0.19625E+01_JPRB, 0.18924E+01_JPRB, 0.17643E+01_JPRB, 0.17842E+01_JPRB ,&
&  0.18226E+01_JPRB, 0.12503E+01_JPRB, 0.10955E+01_JPRB, 0.12344E+01_JPRB, 0.11129E+01_JPRB, 0.11501E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.12884E+01_JPRB, 0.13346E+01_JPRB, 0.13269E+01_JPRB, 0.14062E+01_JPRB, 0.15449E+01_JPRB, 0.15465E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.14380E+01_JPRB, 0.14466E+01_JPRB, 0.14129E+01_JPRB, 0.13548E+01_JPRB, 0.13090E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.11908E+01_JPRB, 0.11745E+01_JPRB, 0.11211E+01_JPRB, 0.10720E+01_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.11618E+01_JPRB, 0.11332E+01_JPRB, 0.11125E+01_JPRB, 0.11050E+01_JPRB, 0.11045E+01_JPRB, 0.11123E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.10498E+01_JPRB, 0.10082E+01_JPRB, 0.99927E+00_JPRB, 0.97984E+00_JPRB, 0.95803E+00_JPRB, 0.96079E+00_JPRB ,&
&  0.98533E+00_JPRB, 0.10137E+01_JPRB, 0.90784E+00_JPRB, 0.85702E+00_JPRB, 0.81571E+00_JPRB, 0.84016E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.93268E+00_JPRB, 0.97031E+00_JPRB, 0.99792E+00_JPRB, 0.10428E+01_JPRB, 0.11384E+01_JPRB, 0.12560E+01_JPRB, 0.13811E+01_JPRB ,&
&  0.14463E+01_JPRB, 0.15381E+01_JPRB, 0.15293E+01_JPRB, 0.16400E+01_JPRB, 0.15954E+01_JPRB, 0.14633E+01_JPRB, 0.14625E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.12976E+01_JPRB, 0.12259E+01_JPRB, 0.11901E+01_JPRB, 0.11673E+01_JPRB, 0.11378E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.10796E+01_JPRB, 0.10723E+01_JPRB, 0.10624E+01_JPRB, 0.11117E+01_JPRB, 0.12541E+01_JPRB, 0.13960E+01_JPRB ,&
&  0.16614E+01_JPRB, 0.14999E+01_JPRB, 0.13652E+01_JPRB, 0.17727E+01_JPRB, 0.22745E+01_JPRB, 0.23908E+01_JPRB, 0.23463E+01_JPRB ,&
&  0.25304E+01_JPRB, 0.25541E+01_JPRB, 0.25559E+01_JPRB, 0.25919E+01_JPRB, 0.25623E+01_JPRB, 0.25049E+01_JPRB, 0.23962E+01_JPRB ,&
&  0.21800E+01_JPRB, 0.20458E+01_JPRB, 0.19624E+01_JPRB, 0.19743E+01_JPRB, 0.18840E+01_JPRB, 0.18270E+01_JPRB, 0.17953E+01_JPRB ,&
&  0.16372E+01_JPRB, 0.15301E+01_JPRB, 0.12212E+01_JPRB, 0.12497E+01_JPRB, 0.10766E+01_JPRB, 0.10876E+01_JPRB, 0.12207E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.12912E+01_JPRB, 0.13214E+01_JPRB, 0.13202E+01_JPRB, 0.15893E+01_JPRB, 0.17685E+01_JPRB, 0.17957E+01_JPRB ,&
&  0.17484E+01_JPRB, 0.15652E+01_JPRB, 0.16042E+01_JPRB, 0.16420E+01_JPRB, 0.16208E+01_JPRB, 0.15829E+01_JPRB, 0.15319E+01_JPRB ,&
&  0.14586E+01_JPRB, 0.13865E+01_JPRB, 0.13369E+01_JPRB, 0.12668E+01_JPRB, 0.12044E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.12919E+01_JPRB, 0.12429E+01_JPRB, 0.12101E+01_JPRB, 0.11937E+01_JPRB, 0.11863E+01_JPRB, 0.11783E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.10687E+01_JPRB, 0.10386E+01_JPRB, 0.10254E+01_JPRB, 0.99775E+00_JPRB, 0.97868E+00_JPRB, 0.99152E+00_JPRB ,&
&  0.10218E+01_JPRB, 0.99909E+00_JPRB, 0.87788E+00_JPRB, 0.80879E+00_JPRB, 0.77182E+00_JPRB, 0.79171E+00_JPRB, 0.83238E+00_JPRB ,&
&  0.88838E+00_JPRB, 0.93427E+00_JPRB, 0.99521E+00_JPRB, 0.10814E+01_JPRB, 0.11986E+01_JPRB, 0.13271E+01_JPRB, 0.15427E+01_JPRB ,&
&  0.18620E+01_JPRB, 0.17339E+01_JPRB, 0.17977E+01_JPRB, 0.17580E+01_JPRB, 0.17329E+01_JPRB, 0.16246E+01_JPRB, 0.15143E+01_JPRB ,&
&  0.14064E+01_JPRB, 0.13323E+01_JPRB, 0.12575E+01_JPRB, 0.11918E+01_JPRB, 0.11380E+01_JPRB, 0.11012E+01_JPRB, 0.10873E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.11214E+01_JPRB, 0.11209E+01_JPRB, 0.11474E+01_JPRB, 0.12679E+01_JPRB, 0.13566E+01_JPRB, 0.14262E+01_JPRB ,&
&  0.14140E+01_JPRB, 0.14647E+01_JPRB, 0.18640E+01_JPRB, 0.20434E+01_JPRB, 0.21843E+01_JPRB, 0.23250E+01_JPRB, 0.23798E+01_JPRB ,&
&  0.24853E+01_JPRB, 0.23929E+01_JPRB, 0.23858E+01_JPRB, 0.21578E+01_JPRB, 0.24050E+01_JPRB, 0.23698E+01_JPRB, 0.22724E+01_JPRB ,&
&  0.21324E+01_JPRB, 0.20834E+01_JPRB, 0.19779E+01_JPRB, 0.19949E+01_JPRB, 0.19859E+01_JPRB, 0.20108E+01_JPRB, 0.16561E+01_JPRB ,&
&  0.14325E+01_JPRB, 0.15693E+01_JPRB, 0.14445E+01_JPRB, 0.13087E+01_JPRB, 0.11879E+01_JPRB, 0.12126E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.12647E+01_JPRB, 0.12788E+01_JPRB, 0.12693E+01_JPRB, 0.13537E+01_JPRB, 0.16944E+01_JPRB, 0.20482E+01_JPRB, 0.20832E+01_JPRB ,&
&  0.18936E+01_JPRB, 0.19453E+01_JPRB, 0.19872E+01_JPRB, 0.19908E+01_JPRB, 0.20109E+01_JPRB, 0.19595E+01_JPRB, 0.18731E+01_JPRB ,&
&  0.17675E+01_JPRB, 0.16634E+01_JPRB, 0.15665E+01_JPRB, 0.14647E+01_JPRB, 0.13625E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.14307E+01_JPRB, 0.13772E+01_JPRB, 0.13299E+01_JPRB, 0.12963E+01_JPRB, 0.12662E+01_JPRB, 0.12434E+01_JPRB, 0.12140E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.10902E+01_JPRB, 0.10770E+01_JPRB, 0.10575E+01_JPRB, 0.10328E+01_JPRB, 0.10186E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.10662E+01_JPRB, 0.99501E+00_JPRB, 0.87083E+00_JPRB, 0.86410E+00_JPRB, 0.84986E+00_JPRB, 0.80614E+00_JPRB, 0.82323E+00_JPRB ,&
&  0.86546E+00_JPRB, 0.91837E+00_JPRB, 0.98031E+00_JPRB, 0.11267E+01_JPRB, 0.13093E+01_JPRB, 0.15296E+01_JPRB, 0.17772E+01_JPRB ,&
&  0.18577E+01_JPRB, 0.19494E+01_JPRB, 0.20285E+01_JPRB, 0.19088E+01_JPRB, 0.17477E+01_JPRB, 0.15858E+01_JPRB, 0.14602E+01_JPRB ,&
&  0.13705E+01_JPRB, 0.12882E+01_JPRB, 0.12081E+01_JPRB, 0.11488E+01_JPRB, 0.11162E+01_JPRB, 0.10983E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.11080E+01_JPRB, 0.11069E+01_JPRB, 0.12047E+01_JPRB, 0.11920E+01_JPRB, 0.12280E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.15907E+01_JPRB, 0.17730E+01_JPRB, 0.18802E+01_JPRB, 0.21860E+01_JPRB, 0.18702E+01_JPRB, 0.20151E+01_JPRB, 0.24236E+01_JPRB ,&
&  0.25030E+01_JPRB, 0.24612E+01_JPRB, 0.25460E+01_JPRB, 0.20977E+01_JPRB, 0.18978E+01_JPRB, 0.24606E+01_JPRB, 0.24070E+01_JPRB ,&
&  0.22593E+01_JPRB, 0.22018E+01_JPRB, 0.21411E+01_JPRB, 0.23303E+01_JPRB, 0.17341E+01_JPRB, 0.15228E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.12644E+01_JPRB, 0.14276E+01_JPRB, 0.13779E+01_JPRB, 0.14054E+01_JPRB, 0.13780E+01_JPRB, 0.12863E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.12736E+01_JPRB, 0.13160E+01_JPRB, 0.16018E+01_JPRB, 0.22411E+01_JPRB, 0.22336E+01_JPRB, 0.24256E+01_JPRB ,&
&  0.24565E+01_JPRB, 0.26002E+01_JPRB, 0.24454E+01_JPRB, 0.24599E+01_JPRB, 0.24021E+01_JPRB, 0.22915E+01_JPRB, 0.21277E+01_JPRB ,&
&  0.19984E+01_JPRB, 0.18745E+01_JPRB, 0.17381E+01_JPRB, 0.16129E+01_JPRB, 0.15096E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.15346E+01_JPRB, 0.14646E+01_JPRB, 0.14036E+01_JPRB, 0.13494E+01_JPRB, 0.13161E+01_JPRB, 0.12845E+01_JPRB, 0.12381E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.10892E+01_JPRB, 0.10861E+01_JPRB, 0.10458E+01_JPRB, 0.10250E+01_JPRB, 0.10278E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.11123E+01_JPRB, 0.96976E+00_JPRB, 0.89906E+00_JPRB, 0.93047E+00_JPRB, 0.86543E+00_JPRB, 0.84055E+00_JPRB ,&
&  0.88958E+00_JPRB, 0.95144E+00_JPRB, 0.10758E+01_JPRB, 0.13320E+01_JPRB, 0.15987E+01_JPRB, 0.18681E+01_JPRB, 0.20983E+01_JPRB ,&
&  0.21732E+01_JPRB, 0.21559E+01_JPRB, 0.20393E+01_JPRB, 0.18757E+01_JPRB, 0.16494E+01_JPRB, 0.14724E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.12067E+01_JPRB, 0.11707E+01_JPRB, 0.11279E+01_JPRB, 0.11073E+01_JPRB, 0.10648E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10666E+01_JPRB, 0.10911E+01_JPRB, 0.11236E+01_JPRB, 0.12499E+01_JPRB, 0.12427E+01_JPRB, 0.12213E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.14978E+01_JPRB, 0.16158E+01_JPRB, 0.17870E+01_JPRB, 0.20237E+01_JPRB, 0.23789E+01_JPRB, 0.24643E+01_JPRB, 0.26536E+01_JPRB ,&
&  0.21634E+01_JPRB, 0.19232E+01_JPRB, 0.18819E+01_JPRB, 0.17521E+01_JPRB, 0.21572E+01_JPRB, 0.29572E+01_JPRB, 0.29575E+01_JPRB ,&
&  0.27727E+01_JPRB, 0.26468E+01_JPRB, 0.26227E+01_JPRB, 0.21836E+01_JPRB, 0.16917E+01_JPRB, 0.13938E+01_JPRB, 0.16245E+01_JPRB ,&
&  0.17232E+01_JPRB, 0.18433E+01_JPRB, 0.17124E+01_JPRB, 0.15025E+01_JPRB, 0.14078E+01_JPRB, 0.16822E+01_JPRB, 0.16706E+01_JPRB ,&
&  0.15280E+01_JPRB, 0.14693E+01_JPRB, 0.19708E+01_JPRB, 0.26707E+01_JPRB, 0.29983E+01_JPRB, 0.29940E+01_JPRB, 0.34149E+01_JPRB ,&
&  0.34834E+01_JPRB, 0.35544E+01_JPRB, 0.33934E+01_JPRB, 0.31646E+01_JPRB, 0.28899E+01_JPRB, 0.25929E+01_JPRB, 0.23611E+01_JPRB ,&
&  0.21816E+01_JPRB, 0.20101E+01_JPRB, 0.18508E+01_JPRB, 0.17143E+01_JPRB, 0.16143E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.16199E+01_JPRB, 0.15310E+01_JPRB, 0.14543E+01_JPRB, 0.14032E+01_JPRB, 0.13550E+01_JPRB, 0.12976E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.10940E+01_JPRB, 0.10811E+01_JPRB, 0.10387E+01_JPRB, 0.10297E+01_JPRB, 0.10441E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.12568E+01_JPRB, 0.10918E+01_JPRB, 0.10189E+01_JPRB, 0.10138E+01_JPRB, 0.99171E+00_JPRB, 0.89275E+00_JPRB ,&
&  0.94461E+00_JPRB, 0.10326E+01_JPRB, 0.12046E+01_JPRB, 0.15152E+01_JPRB, 0.19427E+01_JPRB, 0.20581E+01_JPRB, 0.21574E+01_JPRB ,&
&  0.22070E+01_JPRB, 0.20596E+01_JPRB, 0.19021E+01_JPRB, 0.17393E+01_JPRB, 0.15352E+01_JPRB, 0.13989E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.12588E+01_JPRB, 0.12413E+01_JPRB, 0.12176E+01_JPRB, 0.12072E+01_JPRB, 0.11545E+01_JPRB, 0.11065E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.10806E+01_JPRB, 0.10998E+01_JPRB, 0.11371E+01_JPRB, 0.12644E+01_JPRB, 0.13696E+01_JPRB, 0.13315E+01_JPRB, 0.15211E+01_JPRB ,&
&  0.15950E+01_JPRB, 0.17457E+01_JPRB, 0.18203E+01_JPRB, 0.20914E+01_JPRB, 0.23157E+01_JPRB, 0.23812E+01_JPRB, 0.25340E+01_JPRB ,&
&  0.23064E+01_JPRB, 0.24017E+01_JPRB, 0.28271E+01_JPRB, 0.33034E+01_JPRB, 0.28650E+01_JPRB, 0.28701E+01_JPRB, 0.38356E+01_JPRB ,&
&  0.28472E+01_JPRB, 0.28392E+01_JPRB, 0.29619E+01_JPRB, 0.29410E+01_JPRB, 0.14134E+01_JPRB, 0.12247E+01_JPRB, 0.17387E+01_JPRB ,&
&  0.18357E+01_JPRB, 0.12933E+01_JPRB, 0.10619E+01_JPRB, 0.12198E+01_JPRB, 0.10608E+01_JPRB, 0.13089E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.20472E+01_JPRB, 0.21357E+01_JPRB, 0.31804E+01_JPRB, 0.36480E+01_JPRB, 0.35202E+01_JPRB, 0.44014E+01_JPRB, 0.44661E+01_JPRB ,&
&  0.44276E+01_JPRB, 0.39705E+01_JPRB, 0.39511E+01_JPRB, 0.36047E+01_JPRB, 0.32113E+01_JPRB, 0.28494E+01_JPRB, 0.25519E+01_JPRB ,&
&  0.22769E+01_JPRB, 0.20792E+01_JPRB, 0.19436E+01_JPRB, 0.18068E+01_JPRB, 0.16989E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.15740E+01_JPRB, 0.14835E+01_JPRB, 0.13894E+01_JPRB, 0.13283E+01_JPRB, 0.12916E+01_JPRB, 0.12431E+01_JPRB, 0.11719E+01_JPRB ,&
&  0.10924E+01_JPRB, 0.11084E+01_JPRB, 0.10701E+01_JPRB, 0.10399E+01_JPRB, 0.10226E+01_JPRB, 0.10372E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.12075E+01_JPRB, 0.12252E+01_JPRB, 0.11524E+01_JPRB, 0.11273E+01_JPRB, 0.10810E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.10883E+01_JPRB, 0.12267E+01_JPRB, 0.14359E+01_JPRB, 0.17235E+01_JPRB, 0.19315E+01_JPRB, 0.20104E+01_JPRB, 0.21476E+01_JPRB ,&
&  0.20472E+01_JPRB, 0.18791E+01_JPRB, 0.17599E+01_JPRB, 0.16142E+01_JPRB, 0.14574E+01_JPRB, 0.13636E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.12575E+01_JPRB, 0.12411E+01_JPRB, 0.12308E+01_JPRB, 0.11688E+01_JPRB, 0.11144E+01_JPRB, 0.10865E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.11121E+01_JPRB, 0.11771E+01_JPRB, 0.12689E+01_JPRB, 0.14295E+01_JPRB, 0.14905E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.13819E+01_JPRB, 0.15020E+01_JPRB, 0.18758E+01_JPRB, 0.19477E+01_JPRB, 0.19510E+01_JPRB, 0.20293E+01_JPRB, 0.20462E+01_JPRB ,&
&  0.22359E+01_JPRB, 0.26969E+01_JPRB, 0.27324E+01_JPRB, 0.32967E+01_JPRB, 0.34568E+01_JPRB, 0.26650E+01_JPRB, 0.29609E+01_JPRB ,&
&  0.29567E+01_JPRB, 0.26283E+01_JPRB, 0.21260E+01_JPRB, 0.16879E+01_JPRB, 0.18955E+01_JPRB, 0.18077E+01_JPRB, 0.12207E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.10560E+01_JPRB, 0.10175E+01_JPRB, 0.10148E+01_JPRB, 0.10162E+01_JPRB, 0.13408E+01_JPRB, 0.18651E+01_JPRB ,&
&  0.27195E+01_JPRB, 0.34582E+01_JPRB, 0.41414E+01_JPRB, 0.42690E+01_JPRB, 0.38125E+01_JPRB, 0.46840E+01_JPRB, 0.49774E+01_JPRB ,&
&  0.41100E+01_JPRB, 0.37356E+01_JPRB, 0.36393E+01_JPRB, 0.33057E+01_JPRB, 0.29794E+01_JPRB, 0.26995E+01_JPRB, 0.24307E+01_JPRB ,&
&  0.22012E+01_JPRB, 0.20377E+01_JPRB, 0.19129E+01_JPRB, 0.17913E+01_JPRB, 0.16755E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.14579E+01_JPRB, 0.13950E+01_JPRB, 0.13517E+01_JPRB, 0.13158E+01_JPRB, 0.12699E+01_JPRB, 0.12268E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.11083E+01_JPRB, 0.11071E+01_JPRB, 0.10525E+01_JPRB, 0.10151E+01_JPRB, 0.99860E+00_JPRB, 0.10187E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.11078E+01_JPRB, 0.11808E+01_JPRB, 0.13467E+01_JPRB, 0.13566E+01_JPRB, 0.14800E+01_JPRB, 0.13523E+01_JPRB, 0.13727E+01_JPRB ,&
&  0.14009E+01_JPRB, 0.15639E+01_JPRB, 0.18068E+01_JPRB, 0.19408E+01_JPRB, 0.20075E+01_JPRB, 0.20952E+01_JPRB, 0.20443E+01_JPRB ,&
&  0.19009E+01_JPRB, 0.17971E+01_JPRB, 0.16401E+01_JPRB, 0.14828E+01_JPRB, 0.13782E+01_JPRB, 0.12944E+01_JPRB, 0.12449E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.12228E+01_JPRB, 0.11657E+01_JPRB, 0.11354E+01_JPRB, 0.11254E+01_JPRB, 0.11129E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11717E+01_JPRB, 0.12447E+01_JPRB, 0.13180E+01_JPRB, 0.12872E+01_JPRB, 0.11777E+01_JPRB, 0.13298E+01_JPRB ,&
&  0.14725E+01_JPRB, 0.16941E+01_JPRB, 0.17540E+01_JPRB, 0.19936E+01_JPRB, 0.19068E+01_JPRB, 0.18393E+01_JPRB, 0.19910E+01_JPRB ,&
&  0.22936E+01_JPRB, 0.29451E+01_JPRB, 0.26562E+01_JPRB, 0.30605E+01_JPRB, 0.39213E+01_JPRB, 0.30266E+01_JPRB, 0.25876E+01_JPRB ,&
&  0.27803E+01_JPRB, 0.24060E+01_JPRB, 0.21435E+01_JPRB, 0.20327E+01_JPRB, 0.30388E+01_JPRB, 0.39617E+01_JPRB, 0.17276E+01_JPRB ,&
&  0.13383E+01_JPRB, 0.12684E+01_JPRB, 0.12215E+01_JPRB, 0.11362E+01_JPRB, 0.11592E+01_JPRB, 0.15587E+01_JPRB, 0.25478E+01_JPRB ,&
&  0.31678E+01_JPRB, 0.44574E+01_JPRB, 0.49690E+01_JPRB, 0.48670E+01_JPRB, 0.39420E+01_JPRB, 0.42310E+01_JPRB, 0.38625E+01_JPRB ,&
&  0.40232E+01_JPRB, 0.38344E+01_JPRB, 0.35116E+01_JPRB, 0.31738E+01_JPRB, 0.28467E+01_JPRB, 0.25499E+01_JPRB, 0.22853E+01_JPRB ,&
&  0.20580E+01_JPRB, 0.18860E+01_JPRB, 0.17524E+01_JPRB, 0.16457E+01_JPRB, 0.15411E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.14444E+01_JPRB, 0.13887E+01_JPRB, 0.13396E+01_JPRB, 0.12868E+01_JPRB, 0.12521E+01_JPRB, 0.12017E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11353E+01_JPRB, 0.10928E+01_JPRB, 0.10354E+01_JPRB, 0.10037E+01_JPRB, 0.99790E+00_JPRB, 0.10137E+01_JPRB, 0.10487E+01_JPRB ,&
&  0.11101E+01_JPRB, 0.11693E+01_JPRB, 0.13374E+01_JPRB, 0.17050E+01_JPRB, 0.19044E+01_JPRB, 0.17752E+01_JPRB, 0.19300E+01_JPRB ,&
&  0.18779E+01_JPRB, 0.19857E+01_JPRB, 0.21673E+01_JPRB, 0.21392E+01_JPRB, 0.20992E+01_JPRB, 0.21411E+01_JPRB, 0.19514E+01_JPRB ,&
&  0.17852E+01_JPRB, 0.16786E+01_JPRB, 0.15142E+01_JPRB, 0.13990E+01_JPRB, 0.13145E+01_JPRB, 0.12835E+01_JPRB, 0.12467E+01_JPRB ,&
&  0.12312E+01_JPRB, 0.11907E+01_JPRB, 0.11651E+01_JPRB, 0.11677E+01_JPRB, 0.11758E+01_JPRB, 0.11744E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12260E+01_JPRB, 0.12729E+01_JPRB, 0.13440E+01_JPRB, 0.13967E+01_JPRB, 0.12117E+01_JPRB, 0.13623E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.14885E+01_JPRB, 0.15631E+01_JPRB, 0.15494E+01_JPRB, 0.17141E+01_JPRB, 0.18409E+01_JPRB, 0.18216E+01_JPRB, 0.19348E+01_JPRB ,&
&  0.24409E+01_JPRB, 0.24804E+01_JPRB, 0.25717E+01_JPRB, 0.30696E+01_JPRB, 0.34313E+01_JPRB, 0.42378E+01_JPRB, 0.23775E+01_JPRB ,&
&  0.19253E+01_JPRB, 0.23007E+01_JPRB, 0.24199E+01_JPRB, 0.26972E+01_JPRB, 0.35887E+01_JPRB, 0.58189E+01_JPRB, 0.49395E+01_JPRB ,&
&  0.17444E+01_JPRB, 0.13477E+01_JPRB, 0.15615E+01_JPRB, 0.15732E+01_JPRB, 0.14326E+01_JPRB, 0.19607E+01_JPRB, 0.49137E+01_JPRB ,&
&  0.45695E+01_JPRB, 0.53884E+01_JPRB, 0.52168E+01_JPRB, 0.44162E+01_JPRB, 0.37136E+01_JPRB, 0.37829E+01_JPRB, 0.39347E+01_JPRB ,&
&  0.37602E+01_JPRB, 0.34319E+01_JPRB, 0.31320E+01_JPRB, 0.28695E+01_JPRB, 0.25552E+01_JPRB, 0.22379E+01_JPRB, 0.19614E+01_JPRB ,&
&  0.17656E+01_JPRB, 0.16554E+01_JPRB, 0.15965E+01_JPRB, 0.15479E+01_JPRB, 0.14963E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.13718E+01_JPRB, 0.13220E+01_JPRB, 0.12846E+01_JPRB, 0.12560E+01_JPRB, 0.12445E+01_JPRB, 0.11963E+01_JPRB, 0.11398E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.10975E+01_JPRB, 0.10460E+01_JPRB, 0.10073E+01_JPRB, 0.10103E+01_JPRB, 0.10353E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.11711E+01_JPRB, 0.13755E+01_JPRB, 0.18426E+01_JPRB, 0.21612E+01_JPRB, 0.20592E+01_JPRB, 0.21994E+01_JPRB ,&
&  0.24573E+01_JPRB, 0.24245E+01_JPRB, 0.22157E+01_JPRB, 0.20856E+01_JPRB, 0.21544E+01_JPRB, 0.21698E+01_JPRB, 0.18961E+01_JPRB ,&
&  0.17205E+01_JPRB, 0.16080E+01_JPRB, 0.14774E+01_JPRB, 0.13895E+01_JPRB, 0.13489E+01_JPRB, 0.13177E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12359E+01_JPRB, 0.12400E+01_JPRB, 0.12459E+01_JPRB, 0.12668E+01_JPRB, 0.12570E+01_JPRB, 0.12899E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.13971E+01_JPRB, 0.14472E+01_JPRB, 0.13983E+01_JPRB, 0.14128E+01_JPRB, 0.14342E+01_JPRB, 0.14955E+01_JPRB ,&
&  0.15045E+01_JPRB, 0.15497E+01_JPRB, 0.16221E+01_JPRB, 0.16974E+01_JPRB, 0.18465E+01_JPRB, 0.19566E+01_JPRB, 0.21113E+01_JPRB ,&
&  0.24592E+01_JPRB, 0.26600E+01_JPRB, 0.26274E+01_JPRB, 0.28774E+01_JPRB, 0.32335E+01_JPRB, 0.37265E+01_JPRB, 0.29940E+01_JPRB ,&
&  0.24735E+01_JPRB, 0.22549E+01_JPRB, 0.28087E+01_JPRB, 0.40150E+01_JPRB, 0.48548E+01_JPRB, 0.66590E+01_JPRB, 0.90771E+01_JPRB ,&
&  0.98780E+01_JPRB, 0.50815E+01_JPRB, 0.39601E+01_JPRB, 0.49380E+01_JPRB, 0.36865E+01_JPRB, 0.31865E+01_JPRB, 0.44416E+01_JPRB ,&
&  0.51219E+01_JPRB, 0.54385E+01_JPRB, 0.49905E+01_JPRB, 0.38497E+01_JPRB, 0.35732E+01_JPRB, 0.37681E+01_JPRB, 0.36302E+01_JPRB ,&
&  0.32537E+01_JPRB, 0.29426E+01_JPRB, 0.26181E+01_JPRB, 0.22699E+01_JPRB, 0.19758E+01_JPRB, 0.17623E+01_JPRB, 0.16344E+01_JPRB ,&
&  0.15691E+01_JPRB, 0.15362E+01_JPRB, 0.15147E+01_JPRB, 0.14761E+01_JPRB, 0.14251E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.12935E+01_JPRB, 0.12744E+01_JPRB, 0.12765E+01_JPRB, 0.12750E+01_JPRB, 0.12549E+01_JPRB, 0.12025E+01_JPRB, 0.11972E+01_JPRB ,&
&  0.12312E+01_JPRB, 0.11390E+01_JPRB, 0.10652E+01_JPRB, 0.10331E+01_JPRB, 0.10465E+01_JPRB, 0.10830E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.11455E+01_JPRB, 0.12659E+01_JPRB, 0.16022E+01_JPRB, 0.20379E+01_JPRB, 0.24750E+01_JPRB, 0.32418E+01_JPRB, 0.22737E+01_JPRB ,&
&  0.24071E+01_JPRB, 0.24314E+01_JPRB, 0.21201E+01_JPRB, 0.20833E+01_JPRB, 0.21421E+01_JPRB, 0.21396E+01_JPRB, 0.18879E+01_JPRB ,&
&  0.17119E+01_JPRB, 0.16160E+01_JPRB, 0.15414E+01_JPRB, 0.14716E+01_JPRB, 0.14238E+01_JPRB, 0.14103E+01_JPRB, 0.13701E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.13720E+01_JPRB, 0.13491E+01_JPRB, 0.13671E+01_JPRB, 0.13835E+01_JPRB, 0.13804E+01_JPRB, 0.14285E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.15048E+01_JPRB, 0.15175E+01_JPRB, 0.14633E+01_JPRB, 0.15030E+01_JPRB, 0.15386E+01_JPRB, 0.15893E+01_JPRB ,&
&  0.15284E+01_JPRB, 0.14431E+01_JPRB, 0.16081E+01_JPRB, 0.18235E+01_JPRB, 0.19918E+01_JPRB, 0.20308E+01_JPRB, 0.22189E+01_JPRB ,&
&  0.26666E+01_JPRB, 0.30128E+01_JPRB, 0.31926E+01_JPRB, 0.28137E+01_JPRB, 0.30123E+01_JPRB, 0.34261E+01_JPRB, 0.37580E+01_JPRB ,&
&  0.40356E+01_JPRB, 0.39536E+01_JPRB, 0.45480E+01_JPRB, 0.51113E+01_JPRB, 0.56132E+01_JPRB, 0.71456E+01_JPRB, 0.74495E+01_JPRB ,&
&  0.89522E+01_JPRB, 0.11058E+02_JPRB, 0.92922E+01_JPRB, 0.71430E+01_JPRB, 0.60518E+01_JPRB, 0.51881E+01_JPRB, 0.51422E+01_JPRB ,&
&  0.56411E+01_JPRB, 0.50980E+01_JPRB, 0.43034E+01_JPRB, 0.42318E+01_JPRB, 0.35287E+01_JPRB, 0.33705E+01_JPRB, 0.30016E+01_JPRB ,&
&  0.24923E+01_JPRB, 0.21143E+01_JPRB, 0.17651E+01_JPRB, 0.15478E+01_JPRB, 0.14331E+01_JPRB, 0.14038E+01_JPRB, 0.13987E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.14022E+01_JPRB, 0.13931E+01_JPRB, 0.13723E+01_JPRB, 0.13330E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.12791E+01_JPRB, 0.12788E+01_JPRB, 0.12993E+01_JPRB, 0.13063E+01_JPRB, 0.12853E+01_JPRB, 0.12676E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.11517E+01_JPRB, 0.11274E+01_JPRB, 0.11310E+01_JPRB, 0.11572E+01_JPRB, 0.11736E+01_JPRB, 0.11935E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.14936E+01_JPRB, 0.18611E+01_JPRB, 0.22198E+01_JPRB, 0.25802E+01_JPRB, 0.30563E+01_JPRB, 0.29885E+01_JPRB ,&
&  0.22293E+01_JPRB, 0.26303E+01_JPRB, 0.21210E+01_JPRB, 0.22096E+01_JPRB, 0.22985E+01_JPRB, 0.24476E+01_JPRB, 0.21522E+01_JPRB ,&
&  0.18731E+01_JPRB, 0.17723E+01_JPRB, 0.18050E+01_JPRB, 0.17099E+01_JPRB, 0.15559E+01_JPRB, 0.14894E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.15071E+01_JPRB, 0.14989E+01_JPRB, 0.15251E+01_JPRB, 0.15397E+01_JPRB, 0.15326E+01_JPRB, 0.15425E+01_JPRB ,&
&  0.15581E+01_JPRB, 0.15668E+01_JPRB, 0.15468E+01_JPRB, 0.15660E+01_JPRB, 0.15921E+01_JPRB, 0.16637E+01_JPRB, 0.17178E+01_JPRB ,&
&  0.16787E+01_JPRB, 0.16126E+01_JPRB, 0.17294E+01_JPRB, 0.18691E+01_JPRB, 0.19980E+01_JPRB, 0.22461E+01_JPRB, 0.24000E+01_JPRB ,&
&  0.28558E+01_JPRB, 0.29622E+01_JPRB, 0.35450E+01_JPRB, 0.27754E+01_JPRB, 0.29066E+01_JPRB, 0.33089E+01_JPRB, 0.35013E+01_JPRB ,&
&  0.33337E+01_JPRB, 0.40218E+01_JPRB, 0.46197E+01_JPRB, 0.54517E+01_JPRB, 0.60727E+01_JPRB, 0.67370E+01_JPRB, 0.62713E+01_JPRB ,&
&  0.64444E+01_JPRB, 0.69894E+01_JPRB, 0.68359E+01_JPRB, 0.56440E+01_JPRB, 0.52767E+01_JPRB, 0.46831E+01_JPRB, 0.51885E+01_JPRB ,&
&  0.54396E+01_JPRB, 0.48996E+01_JPRB, 0.45662E+01_JPRB, 0.41652E+01_JPRB, 0.30193E+01_JPRB, 0.25717E+01_JPRB, 0.21027E+01_JPRB ,&
&  0.16885E+01_JPRB, 0.14339E+01_JPRB, 0.12306E+01_JPRB, 0.11955E+01_JPRB, 0.12122E+01_JPRB, 0.12382E+01_JPRB, 0.12871E+01_JPRB ,&
&  0.12967E+01_JPRB, 0.13021E+01_JPRB, 0.13108E+01_JPRB, 0.13072E+01_JPRB, 0.12914E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.12947E+01_JPRB, 0.12967E+01_JPRB, 0.13232E+01_JPRB, 0.13552E+01_JPRB, 0.13788E+01_JPRB, 0.14149E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.13555E+01_JPRB, 0.12082E+01_JPRB, 0.12418E+01_JPRB, 0.12634E+01_JPRB, 0.12857E+01_JPRB, 0.12813E+01_JPRB, 0.13337E+01_JPRB ,&
&  0.14975E+01_JPRB, 0.17493E+01_JPRB, 0.20895E+01_JPRB, 0.24861E+01_JPRB, 0.28853E+01_JPRB, 0.33558E+01_JPRB, 0.34511E+01_JPRB ,&
&  0.26488E+01_JPRB, 0.23286E+01_JPRB, 0.22826E+01_JPRB, 0.23757E+01_JPRB, 0.24142E+01_JPRB, 0.26001E+01_JPRB, 0.26265E+01_JPRB ,&
&  0.24347E+01_JPRB, 0.22176E+01_JPRB, 0.22632E+01_JPRB, 0.19822E+01_JPRB, 0.15934E+01_JPRB, 0.16062E+01_JPRB, 0.16432E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.16649E+01_JPRB, 0.16764E+01_JPRB, 0.17064E+01_JPRB, 0.17222E+01_JPRB, 0.16986E+01_JPRB, 0.16679E+01_JPRB ,&
&  0.16648E+01_JPRB, 0.16802E+01_JPRB, 0.16793E+01_JPRB, 0.16633E+01_JPRB, 0.17207E+01_JPRB, 0.17918E+01_JPRB, 0.18377E+01_JPRB ,&
&  0.18664E+01_JPRB, 0.19313E+01_JPRB, 0.20131E+01_JPRB, 0.21318E+01_JPRB, 0.21542E+01_JPRB, 0.24836E+01_JPRB, 0.26619E+01_JPRB ,&
&  0.29373E+01_JPRB, 0.31374E+01_JPRB, 0.35621E+01_JPRB, 0.27144E+01_JPRB, 0.30671E+01_JPRB, 0.33410E+01_JPRB, 0.36471E+01_JPRB ,&
&  0.37549E+01_JPRB, 0.43003E+01_JPRB, 0.46927E+01_JPRB, 0.51575E+01_JPRB, 0.55634E+01_JPRB, 0.52911E+01_JPRB, 0.54433E+01_JPRB ,&
&  0.51078E+01_JPRB, 0.57862E+01_JPRB, 0.55186E+01_JPRB, 0.51868E+01_JPRB, 0.44929E+01_JPRB, 0.42498E+01_JPRB, 0.45002E+01_JPRB ,&
&  0.47831E+01_JPRB, 0.42710E+01_JPRB, 0.37697E+01_JPRB, 0.30651E+01_JPRB, 0.21407E+01_JPRB, 0.17380E+01_JPRB, 0.14042E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.11256E+01_JPRB, 0.10872E+01_JPRB, 0.11406E+01_JPRB, 0.11734E+01_JPRB, 0.12455E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.12726E+01_JPRB, 0.12999E+01_JPRB, 0.13135E+01_JPRB, 0.13087E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.12621E+01_JPRB, 0.12533E+01_JPRB, 0.12455E+01_JPRB, 0.12524E+01_JPRB, 0.12762E+01_JPRB, 0.12885E+01_JPRB, 0.12595E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.13015E+01_JPRB, 0.13643E+01_JPRB, 0.13979E+01_JPRB, 0.14150E+01_JPRB, 0.14404E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.17081E+01_JPRB, 0.19695E+01_JPRB, 0.22839E+01_JPRB, 0.25234E+01_JPRB, 0.27368E+01_JPRB, 0.29819E+01_JPRB, 0.32161E+01_JPRB ,&
&  0.27397E+01_JPRB, 0.20288E+01_JPRB, 0.23465E+01_JPRB, 0.24906E+01_JPRB, 0.25506E+01_JPRB, 0.29054E+01_JPRB, 0.32820E+01_JPRB ,&
&  0.25207E+01_JPRB, 0.22641E+01_JPRB, 0.22770E+01_JPRB, 0.20840E+01_JPRB, 0.17027E+01_JPRB, 0.17320E+01_JPRB, 0.17787E+01_JPRB ,&
&  0.18073E+01_JPRB, 0.18205E+01_JPRB, 0.18618E+01_JPRB, 0.19030E+01_JPRB, 0.19184E+01_JPRB, 0.18923E+01_JPRB, 0.18539E+01_JPRB ,&
&  0.18671E+01_JPRB, 0.18942E+01_JPRB, 0.19288E+01_JPRB, 0.19297E+01_JPRB, 0.19644E+01_JPRB, 0.20365E+01_JPRB, 0.21012E+01_JPRB ,&
&  0.21622E+01_JPRB, 0.21819E+01_JPRB, 0.23075E+01_JPRB, 0.25549E+01_JPRB, 0.27902E+01_JPRB, 0.25788E+01_JPRB, 0.28510E+01_JPRB ,&
&  0.29404E+01_JPRB, 0.30576E+01_JPRB, 0.30484E+01_JPRB, 0.35710E+01_JPRB, 0.25374E+01_JPRB, 0.30407E+01_JPRB, 0.32442E+01_JPRB ,&
&  0.41177E+01_JPRB, 0.42436E+01_JPRB, 0.44981E+01_JPRB, 0.47815E+01_JPRB, 0.50222E+01_JPRB, 0.45627E+01_JPRB, 0.45297E+01_JPRB ,&
&  0.47010E+01_JPRB, 0.47943E+01_JPRB, 0.47779E+01_JPRB, 0.47557E+01_JPRB, 0.43622E+01_JPRB, 0.42801E+01_JPRB, 0.41184E+01_JPRB ,&
&  0.40609E+01_JPRB, 0.32986E+01_JPRB, 0.26662E+01_JPRB, 0.20223E+01_JPRB, 0.14421E+01_JPRB, 0.12408E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.11699E+01_JPRB, 0.11626E+01_JPRB, 0.11699E+01_JPRB, 0.12018E+01_JPRB, 0.12571E+01_JPRB, 0.12509E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.12786E+01_JPRB, 0.13005E+01_JPRB, 0.13065E+01_JPRB, 0.12927E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.12461E+01_JPRB, 0.12311E+01_JPRB, 0.12039E+01_JPRB, 0.12145E+01_JPRB, 0.12235E+01_JPRB, 0.12293E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.13494E+01_JPRB, 0.14308E+01_JPRB, 0.14892E+01_JPRB, 0.15144E+01_JPRB, 0.15494E+01_JPRB, 0.16456E+01_JPRB ,&
&  0.17842E+01_JPRB, 0.19908E+01_JPRB, 0.22478E+01_JPRB, 0.24353E+01_JPRB, 0.25760E+01_JPRB, 0.26353E+01_JPRB, 0.27343E+01_JPRB ,&
&  0.26776E+01_JPRB, 0.26568E+01_JPRB, 0.25081E+01_JPRB, 0.20809E+01_JPRB, 0.20600E+01_JPRB, 0.25926E+01_JPRB, 0.24137E+01_JPRB ,&
&  0.21370E+01_JPRB, 0.20109E+01_JPRB, 0.19516E+01_JPRB, 0.18919E+01_JPRB, 0.18392E+01_JPRB, 0.18718E+01_JPRB, 0.19287E+01_JPRB ,&
&  0.19509E+01_JPRB, 0.19999E+01_JPRB, 0.20490E+01_JPRB, 0.20853E+01_JPRB, 0.21073E+01_JPRB, 0.21461E+01_JPRB, 0.21581E+01_JPRB ,&
&  0.22191E+01_JPRB, 0.22805E+01_JPRB, 0.23517E+01_JPRB, 0.23859E+01_JPRB, 0.23616E+01_JPRB, 0.24273E+01_JPRB, 0.24851E+01_JPRB ,&
&  0.25568E+01_JPRB, 0.25731E+01_JPRB, 0.25839E+01_JPRB, 0.27324E+01_JPRB, 0.28255E+01_JPRB, 0.25184E+01_JPRB, 0.25859E+01_JPRB ,&
&  0.28858E+01_JPRB, 0.30153E+01_JPRB, 0.27658E+01_JPRB, 0.34753E+01_JPRB, 0.21890E+01_JPRB, 0.29642E+01_JPRB, 0.37754E+01_JPRB ,&
&  0.38801E+01_JPRB, 0.39919E+01_JPRB, 0.41407E+01_JPRB, 0.42571E+01_JPRB, 0.42576E+01_JPRB, 0.39213E+01_JPRB, 0.41644E+01_JPRB ,&
&  0.43856E+01_JPRB, 0.43212E+01_JPRB, 0.42628E+01_JPRB, 0.42001E+01_JPRB, 0.40507E+01_JPRB, 0.36536E+01_JPRB, 0.30899E+01_JPRB ,&
&  0.24792E+01_JPRB, 0.25466E+01_JPRB, 0.20655E+01_JPRB, 0.17152E+01_JPRB, 0.12465E+01_JPRB, 0.10730E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.11617E+01_JPRB, 0.11680E+01_JPRB, 0.11977E+01_JPRB, 0.12187E+01_JPRB, 0.12397E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.12514E+01_JPRB, 0.12676E+01_JPRB, 0.12807E+01_JPRB, 0.12768E+01_JPRB, 0.12637E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.11793E+01_JPRB, 0.11611E+01_JPRB, 0.11400E+01_JPRB, 0.11792E+01_JPRB, 0.11761E+01_JPRB, 0.11707E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.12153E+01_JPRB, 0.12607E+01_JPRB, 0.13223E+01_JPRB, 0.13906E+01_JPRB, 0.14705E+01_JPRB, 0.15739E+01_JPRB, 0.16645E+01_JPRB ,&
&  0.17675E+01_JPRB, 0.18669E+01_JPRB, 0.20339E+01_JPRB, 0.22447E+01_JPRB, 0.25147E+01_JPRB, 0.26597E+01_JPRB, 0.26768E+01_JPRB ,&
&  0.25456E+01_JPRB, 0.26066E+01_JPRB, 0.26908E+01_JPRB, 0.27622E+01_JPRB, 0.24378E+01_JPRB, 0.27044E+01_JPRB, 0.28181E+01_JPRB ,&
&  0.26932E+01_JPRB, 0.23050E+01_JPRB, 0.20015E+01_JPRB, 0.19549E+01_JPRB, 0.19425E+01_JPRB, 0.19905E+01_JPRB, 0.20617E+01_JPRB ,&
&  0.21092E+01_JPRB, 0.21697E+01_JPRB, 0.22115E+01_JPRB, 0.22587E+01_JPRB, 0.23351E+01_JPRB, 0.24590E+01_JPRB, 0.25213E+01_JPRB ,&
&  0.25678E+01_JPRB, 0.26629E+01_JPRB, 0.26978E+01_JPRB, 0.26611E+01_JPRB, 0.27366E+01_JPRB, 0.27973E+01_JPRB, 0.28563E+01_JPRB ,&
&  0.29645E+01_JPRB, 0.28677E+01_JPRB, 0.29483E+01_JPRB, 0.30707E+01_JPRB, 0.29972E+01_JPRB, 0.27523E+01_JPRB, 0.27614E+01_JPRB ,&
&  0.29508E+01_JPRB, 0.30693E+01_JPRB, 0.23058E+01_JPRB, 0.32723E+01_JPRB, 0.40418E+01_JPRB, 0.36477E+01_JPRB, 0.36848E+01_JPRB ,&
&  0.36747E+01_JPRB, 0.37771E+01_JPRB, 0.39363E+01_JPRB, 0.40870E+01_JPRB, 0.41438E+01_JPRB, 0.41446E+01_JPRB, 0.38110E+01_JPRB ,&
&  0.42509E+01_JPRB, 0.41208E+01_JPRB, 0.38883E+01_JPRB, 0.36453E+01_JPRB, 0.33188E+01_JPRB, 0.27777E+01_JPRB, 0.22325E+01_JPRB ,&
&  0.18585E+01_JPRB, 0.21216E+01_JPRB, 0.18178E+01_JPRB, 0.17070E+01_JPRB, 0.14817E+01_JPRB, 0.10308E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.11232E+01_JPRB, 0.11454E+01_JPRB, 0.11602E+01_JPRB, 0.11538E+01_JPRB, 0.11667E+01_JPRB, 0.11823E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.12018E+01_JPRB, 0.12071E+01_JPRB, 0.11988E+01_JPRB, 0.11934E+01_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.11393E+01_JPRB, 0.11346E+01_JPRB, 0.11206E+01_JPRB, 0.11353E+01_JPRB, 0.11243E+01_JPRB, 0.11492E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11208E+01_JPRB, 0.11399E+01_JPRB, 0.11631E+01_JPRB, 0.12099E+01_JPRB, 0.12919E+01_JPRB, 0.13852E+01_JPRB ,&
&  0.14741E+01_JPRB, 0.14840E+01_JPRB, 0.15208E+01_JPRB, 0.16528E+01_JPRB, 0.19709E+01_JPRB, 0.23123E+01_JPRB, 0.24482E+01_JPRB ,&
&  0.26059E+01_JPRB, 0.28202E+01_JPRB, 0.29917E+01_JPRB, 0.31978E+01_JPRB, 0.32898E+01_JPRB, 0.34220E+01_JPRB, 0.37228E+01_JPRB ,&
&  0.30613E+01_JPRB, 0.23414E+01_JPRB, 0.21017E+01_JPRB, 0.20046E+01_JPRB, 0.20189E+01_JPRB, 0.20688E+01_JPRB, 0.21218E+01_JPRB ,&
&  0.21738E+01_JPRB, 0.22412E+01_JPRB, 0.22995E+01_JPRB, 0.23932E+01_JPRB, 0.25330E+01_JPRB, 0.26964E+01_JPRB, 0.27890E+01_JPRB ,&
&  0.28311E+01_JPRB, 0.28916E+01_JPRB, 0.29644E+01_JPRB, 0.28539E+01_JPRB, 0.31588E+01_JPRB, 0.32924E+01_JPRB, 0.33676E+01_JPRB ,&
&  0.33596E+01_JPRB, 0.32674E+01_JPRB, 0.33633E+01_JPRB, 0.34050E+01_JPRB, 0.34175E+01_JPRB, 0.32152E+01_JPRB, 0.30830E+01_JPRB ,&
&  0.31080E+01_JPRB, 0.27480E+01_JPRB, 0.21216E+01_JPRB, 0.24502E+01_JPRB, 0.28332E+01_JPRB, 0.26442E+01_JPRB, 0.35251E+01_JPRB ,&
&  0.35175E+01_JPRB, 0.34757E+01_JPRB, 0.35560E+01_JPRB, 0.36780E+01_JPRB, 0.38770E+01_JPRB, 0.38148E+01_JPRB, 0.36778E+01_JPRB ,&
&  0.37441E+01_JPRB, 0.35591E+01_JPRB, 0.32510E+01_JPRB, 0.28521E+01_JPRB, 0.24411E+01_JPRB, 0.20773E+01_JPRB, 0.18538E+01_JPRB ,&
&  0.17287E+01_JPRB, 0.17099E+01_JPRB, 0.16198E+01_JPRB, 0.14328E+01_JPRB, 0.11344E+01_JPRB, 0.96026E+00_JPRB, 0.98663E+00_JPRB ,&
&  0.10499E+01_JPRB, 0.10648E+01_JPRB, 0.10793E+01_JPRB, 0.10751E+01_JPRB, 0.10784E+01_JPRB, 0.10793E+01_JPRB, 0.10847E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.11392E+01_JPRB, 0.11455E+01_JPRB, 0.11472E+01_JPRB, 0.11488E+01_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.10767E+01_JPRB, 0.10724E+01_JPRB, 0.10707E+01_JPRB, 0.10863E+01_JPRB, 0.10977E+01_JPRB, 0.10795E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.10466E+01_JPRB, 0.10589E+01_JPRB, 0.10660E+01_JPRB, 0.10884E+01_JPRB, 0.11153E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.11190E+01_JPRB, 0.10608E+01_JPRB, 0.10592E+01_JPRB, 0.11707E+01_JPRB, 0.13817E+01_JPRB, 0.17866E+01_JPRB ,&
&  0.22402E+01_JPRB, 0.24797E+01_JPRB, 0.26452E+01_JPRB, 0.27224E+01_JPRB, 0.27198E+01_JPRB, 0.26782E+01_JPRB, 0.36114E+01_JPRB ,&
&  0.38285E+01_JPRB, 0.22565E+01_JPRB, 0.21170E+01_JPRB, 0.19424E+01_JPRB, 0.19705E+01_JPRB, 0.19843E+01_JPRB, 0.20241E+01_JPRB ,&
&  0.20531E+01_JPRB, 0.21252E+01_JPRB, 0.22311E+01_JPRB, 0.23879E+01_JPRB, 0.25425E+01_JPRB, 0.27040E+01_JPRB, 0.28200E+01_JPRB ,&
&  0.28843E+01_JPRB, 0.29397E+01_JPRB, 0.30661E+01_JPRB, 0.30841E+01_JPRB, 0.31243E+01_JPRB, 0.32937E+01_JPRB, 0.34557E+01_JPRB ,&
&  0.35858E+01_JPRB, 0.38366E+01_JPRB, 0.35072E+01_JPRB, 0.35231E+01_JPRB, 0.36536E+01_JPRB, 0.32609E+01_JPRB, 0.31952E+01_JPRB ,&
&  0.31752E+01_JPRB, 0.26545E+01_JPRB, 0.19432E+01_JPRB, 0.21586E+01_JPRB, 0.24385E+01_JPRB, 0.26811E+01_JPRB, 0.30355E+01_JPRB ,&
&  0.29594E+01_JPRB, 0.28238E+01_JPRB, 0.28691E+01_JPRB, 0.30672E+01_JPRB, 0.31157E+01_JPRB, 0.30486E+01_JPRB, 0.29148E+01_JPRB ,&
&  0.28124E+01_JPRB, 0.26031E+01_JPRB, 0.24526E+01_JPRB, 0.22151E+01_JPRB, 0.20272E+01_JPRB, 0.18366E+01_JPRB, 0.16076E+01_JPRB ,&
&  0.16894E+01_JPRB, 0.19358E+01_JPRB, 0.24035E+01_JPRB, 0.19836E+01_JPRB, 0.11828E+01_JPRB, 0.97605E+00_JPRB, 0.93367E+00_JPRB ,&
&  0.95547E+00_JPRB, 0.96965E+00_JPRB, 0.10007E+01_JPRB, 0.10083E+01_JPRB, 0.10016E+01_JPRB, 0.99358E+00_JPRB, 0.99008E+00_JPRB ,&
&  0.10191E+01_JPRB, 0.10602E+01_JPRB, 0.10814E+01_JPRB, 0.10770E+01_JPRB, 0.10716E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.11184E+01_JPRB, 0.11250E+01_JPRB, 0.11525E+01_JPRB, 0.11621E+01_JPRB, 0.11598E+01_JPRB, 0.11656E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.11608E+01_JPRB, 0.11690E+01_JPRB, 0.11807E+01_JPRB, 0.11991E+01_JPRB, 0.12177E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.12262E+01_JPRB, 0.12221E+01_JPRB, 0.12575E+01_JPRB, 0.12807E+01_JPRB, 0.12751E+01_JPRB, 0.12708E+01_JPRB, 0.13090E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.14525E+01_JPRB, 0.14698E+01_JPRB, 0.14557E+01_JPRB, 0.15194E+01_JPRB, 0.17647E+01_JPRB, 0.26887E+01_JPRB ,&
&  0.31514E+01_JPRB, 0.22309E+01_JPRB, 0.18832E+01_JPRB, 0.17082E+01_JPRB, 0.17311E+01_JPRB, 0.17993E+01_JPRB, 0.17777E+01_JPRB ,&
&  0.17457E+01_JPRB, 0.17822E+01_JPRB, 0.18184E+01_JPRB, 0.18573E+01_JPRB, 0.19203E+01_JPRB, 0.19688E+01_JPRB, 0.19863E+01_JPRB ,&
&  0.19356E+01_JPRB, 0.19121E+01_JPRB, 0.19536E+01_JPRB, 0.20334E+01_JPRB, 0.23226E+01_JPRB, 0.27491E+01_JPRB, 0.26475E+01_JPRB ,&
&  0.26638E+01_JPRB, 0.29331E+01_JPRB, 0.31833E+01_JPRB, 0.33221E+01_JPRB, 0.32250E+01_JPRB, 0.29951E+01_JPRB, 0.28187E+01_JPRB ,&
&  0.26487E+01_JPRB, 0.23664E+01_JPRB, 0.20807E+01_JPRB, 0.24378E+01_JPRB, 0.25723E+01_JPRB, 0.27139E+01_JPRB, 0.25470E+01_JPRB ,&
&  0.24029E+01_JPRB, 0.21474E+01_JPRB, 0.21287E+01_JPRB, 0.22599E+01_JPRB, 0.23443E+01_JPRB, 0.22787E+01_JPRB, 0.21492E+01_JPRB ,&
&  0.19932E+01_JPRB, 0.19354E+01_JPRB, 0.20105E+01_JPRB, 0.19983E+01_JPRB, 0.20050E+01_JPRB, 0.16511E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.16459E+01_JPRB, 0.19065E+01_JPRB, 0.16121E+01_JPRB, 0.20042E+01_JPRB, 0.10631E+01_JPRB, 0.86948E+00_JPRB, 0.86364E+00_JPRB ,&
&  0.88929E+00_JPRB, 0.92616E+00_JPRB, 0.93981E+00_JPRB, 0.10411E+01_JPRB, 0.10342E+01_JPRB, 0.10365E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.10479E+01_JPRB, 0.10572E+01_JPRB, 0.10676E+01_JPRB, 0.10906E+01_JPRB, 0.11063E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.11827E+01_JPRB, 0.11894E+01_JPRB, 0.12016E+01_JPRB, 0.12200E+01_JPRB, 0.12313E+01_JPRB, 0.12401E+01_JPRB, 0.12417E+01_JPRB ,&
&  0.12329E+01_JPRB, 0.12350E+01_JPRB, 0.12496E+01_JPRB, 0.12710E+01_JPRB, 0.12938E+01_JPRB, 0.13137E+01_JPRB, 0.13398E+01_JPRB ,&
&  0.13454E+01_JPRB, 0.13578E+01_JPRB, 0.13943E+01_JPRB, 0.14312E+01_JPRB, 0.14482E+01_JPRB, 0.14487E+01_JPRB, 0.14560E+01_JPRB ,&
&  0.14468E+01_JPRB, 0.14596E+01_JPRB, 0.15030E+01_JPRB, 0.15329E+01_JPRB, 0.15764E+01_JPRB, 0.16947E+01_JPRB, 0.20726E+01_JPRB ,&
&  0.23607E+01_JPRB, 0.23870E+01_JPRB, 0.19470E+01_JPRB, 0.17373E+01_JPRB, 0.17194E+01_JPRB, 0.16375E+01_JPRB, 0.15617E+01_JPRB ,&
&  0.14852E+01_JPRB, 0.14536E+01_JPRB, 0.13938E+01_JPRB, 0.13977E+01_JPRB, 0.13774E+01_JPRB, 0.14065E+01_JPRB, 0.14003E+01_JPRB ,&
&  0.14537E+01_JPRB, 0.15131E+01_JPRB, 0.16335E+01_JPRB, 0.17375E+01_JPRB, 0.18794E+01_JPRB, 0.19629E+01_JPRB, 0.17935E+01_JPRB ,&
&  0.17736E+01_JPRB, 0.19717E+01_JPRB, 0.22538E+01_JPRB, 0.24873E+01_JPRB, 0.23834E+01_JPRB, 0.22044E+01_JPRB, 0.20770E+01_JPRB ,&
&  0.19645E+01_JPRB, 0.19620E+01_JPRB, 0.24050E+01_JPRB, 0.24911E+01_JPRB, 0.25318E+01_JPRB, 0.23784E+01_JPRB, 0.21260E+01_JPRB ,&
&  0.18348E+01_JPRB, 0.15102E+01_JPRB, 0.14854E+01_JPRB, 0.14903E+01_JPRB, 0.15121E+01_JPRB, 0.14681E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.13836E+01_JPRB, 0.13740E+01_JPRB, 0.16452E+01_JPRB, 0.19803E+01_JPRB, 0.22218E+01_JPRB, 0.27665E+01_JPRB, 0.24218E+01_JPRB ,&
&  0.16469E+01_JPRB, 0.15918E+01_JPRB, 0.95320E+00_JPRB, 0.13002E+01_JPRB, 0.85664E+00_JPRB, 0.80299E+00_JPRB, 0.87915E+00_JPRB ,&
&  0.99497E+00_JPRB, 0.10988E+01_JPRB, 0.11776E+01_JPRB, 0.11544E+01_JPRB, 0.10945E+01_JPRB, 0.10950E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.11370E+01_JPRB, 0.11474E+01_JPRB, 0.11582E+01_JPRB, 0.11652E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.12006E+01_JPRB, 0.12174E+01_JPRB, 0.12392E+01_JPRB, 0.12645E+01_JPRB, 0.12870E+01_JPRB, 0.13082E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13385E+01_JPRB, 0.13609E+01_JPRB, 0.13960E+01_JPRB, 0.14352E+01_JPRB, 0.14574E+01_JPRB, 0.14813E+01_JPRB, 0.14939E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.15794E+01_JPRB, 0.16097E+01_JPRB, 0.16459E+01_JPRB, 0.16682E+01_JPRB, 0.16913E+01_JPRB, 0.17225E+01_JPRB ,&
&  0.17415E+01_JPRB, 0.17961E+01_JPRB, 0.18879E+01_JPRB, 0.19574E+01_JPRB, 0.20691E+01_JPRB, 0.23580E+01_JPRB, 0.21828E+01_JPRB ,&
&  0.25650E+01_JPRB, 0.23700E+01_JPRB, 0.20943E+01_JPRB, 0.18341E+01_JPRB, 0.17567E+01_JPRB, 0.16882E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.15844E+01_JPRB, 0.15008E+01_JPRB, 0.15170E+01_JPRB, 0.15006E+01_JPRB, 0.14813E+01_JPRB, 0.15561E+01_JPRB, 0.16110E+01_JPRB ,&
&  0.16441E+01_JPRB, 0.16822E+01_JPRB, 0.17357E+01_JPRB, 0.17644E+01_JPRB, 0.18171E+01_JPRB, 0.18333E+01_JPRB, 0.17472E+01_JPRB ,&
&  0.16358E+01_JPRB, 0.15813E+01_JPRB, 0.15271E+01_JPRB, 0.15623E+01_JPRB, 0.15485E+01_JPRB, 0.13901E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.15331E+01_JPRB, 0.17470E+01_JPRB, 0.25580E+01_JPRB, 0.24449E+01_JPRB, 0.21798E+01_JPRB, 0.17983E+01_JPRB ,&
&  0.14981E+01_JPRB, 0.11557E+01_JPRB, 0.10687E+01_JPRB, 0.10925E+01_JPRB, 0.10631E+01_JPRB, 0.10534E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.99040E+00_JPRB, 0.10601E+01_JPRB, 0.13291E+01_JPRB, 0.17288E+01_JPRB, 0.21997E+01_JPRB, 0.23800E+01_JPRB, 0.30518E+01_JPRB ,&
&  0.19692E+01_JPRB, 0.11327E+01_JPRB, 0.12164E+01_JPRB, 0.10586E+01_JPRB, 0.90763E+00_JPRB, 0.77087E+00_JPRB, 0.88302E+00_JPRB ,&
&  0.10977E+01_JPRB, 0.11696E+01_JPRB, 0.11990E+01_JPRB, 0.11610E+01_JPRB, 0.11603E+01_JPRB, 0.11725E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11752E+01_JPRB, 0.11802E+01_JPRB, 0.11872E+01_JPRB, 0.11866E+01_JPRB, 0.11893E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.12248E+01_JPRB, 0.12497E+01_JPRB, 0.12832E+01_JPRB, 0.13232E+01_JPRB, 0.13498E+01_JPRB, 0.13857E+01_JPRB, 0.14067E+01_JPRB ,&
&  0.14317E+01_JPRB, 0.14773E+01_JPRB, 0.15226E+01_JPRB, 0.15714E+01_JPRB, 0.16213E+01_JPRB, 0.16677E+01_JPRB, 0.17146E+01_JPRB ,&
&  0.17555E+01_JPRB, 0.18020E+01_JPRB, 0.18338E+01_JPRB, 0.18621E+01_JPRB, 0.18681E+01_JPRB, 0.18760E+01_JPRB, 0.18702E+01_JPRB ,&
&  0.18595E+01_JPRB, 0.18721E+01_JPRB, 0.19067E+01_JPRB, 0.19379E+01_JPRB, 0.20101E+01_JPRB, 0.23026E+01_JPRB, 0.18721E+01_JPRB ,&
&  0.24413E+01_JPRB, 0.23396E+01_JPRB, 0.22251E+01_JPRB, 0.20296E+01_JPRB, 0.19452E+01_JPRB, 0.19552E+01_JPRB, 0.20053E+01_JPRB ,&
&  0.20914E+01_JPRB, 0.21986E+01_JPRB, 0.22768E+01_JPRB, 0.21301E+01_JPRB, 0.19523E+01_JPRB, 0.19709E+01_JPRB, 0.20116E+01_JPRB ,&
&  0.20374E+01_JPRB, 0.20410E+01_JPRB, 0.20474E+01_JPRB, 0.20241E+01_JPRB, 0.20031E+01_JPRB, 0.19733E+01_JPRB, 0.19082E+01_JPRB ,&
&  0.18132E+01_JPRB, 0.16884E+01_JPRB, 0.15208E+01_JPRB, 0.15940E+01_JPRB, 0.15927E+01_JPRB, 0.13584E+01_JPRB, 0.10064E+01_JPRB ,&
&  0.93313E+00_JPRB, 0.12650E+01_JPRB, 0.16828E+01_JPRB, 0.24688E+01_JPRB, 0.22350E+01_JPRB, 0.19155E+01_JPRB, 0.16696E+01_JPRB ,&
&  0.12919E+01_JPRB, 0.10260E+01_JPRB, 0.90595E+00_JPRB, 0.91758E+00_JPRB, 0.91262E+00_JPRB, 0.91642E+00_JPRB, 0.89714E+00_JPRB ,&
&  0.88272E+00_JPRB, 0.92678E+00_JPRB, 0.11024E+01_JPRB, 0.13483E+01_JPRB, 0.17011E+01_JPRB, 0.18244E+01_JPRB, 0.28863E+01_JPRB ,&
&  0.18014E+01_JPRB, 0.14244E+01_JPRB, 0.19152E+01_JPRB, 0.78953E+00_JPRB, 0.87372E+00_JPRB, 0.94555E+00_JPRB, 0.10523E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.13839E+01_JPRB, 0.16650E+01_JPRB, 0.14845E+01_JPRB, 0.13594E+01_JPRB, 0.12595E+01_JPRB, 0.12236E+01_JPRB ,&
&  0.12193E+01_JPRB, 0.12158E+01_JPRB, 0.12210E+01_JPRB, 0.12210E+01_JPRB, 0.12169E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.12855E+01_JPRB, 0.12995E+01_JPRB, 0.13036E+01_JPRB, 0.13476E+01_JPRB, 0.13591E+01_JPRB, 0.13880E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.14724E+01_JPRB, 0.15170E+01_JPRB, 0.15800E+01_JPRB, 0.16262E+01_JPRB, 0.16694E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.17871E+01_JPRB, 0.18572E+01_JPRB, 0.18822E+01_JPRB, 0.18996E+01_JPRB, 0.19026E+01_JPRB, 0.19254E+01_JPRB, 0.19367E+01_JPRB ,&
&  0.19565E+01_JPRB, 0.19664E+01_JPRB, 0.20076E+01_JPRB, 0.20693E+01_JPRB, 0.21617E+01_JPRB, 0.23791E+01_JPRB, 0.16848E+01_JPRB ,&
&  0.23345E+01_JPRB, 0.24477E+01_JPRB, 0.23318E+01_JPRB, 0.22931E+01_JPRB, 0.21832E+01_JPRB, 0.22363E+01_JPRB, 0.23476E+01_JPRB ,&
&  0.24950E+01_JPRB, 0.26371E+01_JPRB, 0.26093E+01_JPRB, 0.24599E+01_JPRB, 0.23926E+01_JPRB, 0.24375E+01_JPRB, 0.24862E+01_JPRB ,&
&  0.25180E+01_JPRB, 0.25366E+01_JPRB, 0.25614E+01_JPRB, 0.25694E+01_JPRB, 0.25861E+01_JPRB, 0.26156E+01_JPRB, 0.25964E+01_JPRB ,&
&  0.25309E+01_JPRB, 0.24653E+01_JPRB, 0.24649E+01_JPRB, 0.25716E+01_JPRB, 0.25293E+01_JPRB, 0.21692E+01_JPRB, 0.16631E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.99994E+00_JPRB, 0.15061E+01_JPRB, 0.22184E+01_JPRB, 0.19946E+01_JPRB, 0.17093E+01_JPRB, 0.14196E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.98638E+00_JPRB, 0.85027E+00_JPRB, 0.85725E+00_JPRB, 0.87808E+00_JPRB, 0.93503E+00_JPRB, 0.91630E+00_JPRB ,&
&  0.95064E+00_JPRB, 0.99548E+00_JPRB, 0.10573E+01_JPRB, 0.12079E+01_JPRB, 0.15080E+01_JPRB, 0.17482E+01_JPRB, 0.16642E+01_JPRB ,&
&  0.19071E+01_JPRB, 0.13695E+01_JPRB, 0.12458E+01_JPRB, 0.13201E+01_JPRB, 0.11546E+01_JPRB, 0.13398E+01_JPRB, 0.16366E+01_JPRB ,&
&  0.21084E+01_JPRB, 0.20037E+01_JPRB, 0.25897E+01_JPRB, 0.39009E+01_JPRB, 0.30724E+01_JPRB, 0.22432E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.12849E+01_JPRB, 0.12695E+01_JPRB, 0.12904E+01_JPRB, 0.12855E+01_JPRB, 0.12828E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.12039E+01_JPRB, 0.11957E+01_JPRB, 0.12064E+01_JPRB, 0.12153E+01_JPRB, 0.12177E+01_JPRB, 0.12354E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.13414E+01_JPRB, 0.14243E+01_JPRB, 0.14935E+01_JPRB, 0.15842E+01_JPRB, 0.16322E+01_JPRB, 0.16417E+01_JPRB ,&
&  0.16733E+01_JPRB, 0.17520E+01_JPRB, 0.18268E+01_JPRB, 0.18809E+01_JPRB, 0.18826E+01_JPRB, 0.19199E+01_JPRB, 0.20201E+01_JPRB ,&
&  0.21544E+01_JPRB, 0.22547E+01_JPRB, 0.23457E+01_JPRB, 0.24652E+01_JPRB, 0.25325E+01_JPRB, 0.24687E+01_JPRB, 0.17408E+01_JPRB ,&
&  0.22657E+01_JPRB, 0.24683E+01_JPRB, 0.25356E+01_JPRB, 0.25179E+01_JPRB, 0.25265E+01_JPRB, 0.24502E+01_JPRB, 0.25582E+01_JPRB ,&
&  0.27189E+01_JPRB, 0.28329E+01_JPRB, 0.26817E+01_JPRB, 0.24444E+01_JPRB, 0.26782E+01_JPRB, 0.27131E+01_JPRB, 0.28096E+01_JPRB ,&
&  0.28838E+01_JPRB, 0.29572E+01_JPRB, 0.30332E+01_JPRB, 0.31294E+01_JPRB, 0.32034E+01_JPRB, 0.32733E+01_JPRB, 0.33234E+01_JPRB ,&
&  0.33179E+01_JPRB, 0.32897E+01_JPRB, 0.33395E+01_JPRB, 0.31859E+01_JPRB, 0.30928E+01_JPRB, 0.31655E+01_JPRB, 0.26696E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.10824E+01_JPRB, 0.15118E+01_JPRB, 0.19284E+01_JPRB, 0.16881E+01_JPRB, 0.14618E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.93577E+00_JPRB, 0.89400E+00_JPRB, 0.95197E+00_JPRB, 0.10410E+01_JPRB, 0.11245E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11352E+01_JPRB, 0.11787E+01_JPRB, 0.12763E+01_JPRB, 0.14729E+01_JPRB, 0.18441E+01_JPRB, 0.23469E+01_JPRB ,&
&  0.22118E+01_JPRB, 0.14301E+01_JPRB, 0.13897E+01_JPRB, 0.13930E+01_JPRB, 0.14397E+01_JPRB, 0.15728E+01_JPRB, 0.18781E+01_JPRB ,&
&  0.23641E+01_JPRB, 0.28686E+01_JPRB, 0.27367E+01_JPRB, 0.22791E+01_JPRB, 0.28183E+01_JPRB, 0.19379E+01_JPRB, 0.11014E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.11649E+01_JPRB, 0.11955E+01_JPRB, 0.12032E+01_JPRB, 0.12069E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.10340E+01_JPRB, 0.10190E+01_JPRB, 0.10111E+01_JPRB, 0.10045E+01_JPRB, 0.10199E+01_JPRB, 0.10631E+01_JPRB, 0.11158E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.12494E+01_JPRB, 0.13411E+01_JPRB, 0.14043E+01_JPRB, 0.14808E+01_JPRB, 0.15179E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.15657E+01_JPRB, 0.16233E+01_JPRB, 0.17031E+01_JPRB, 0.17840E+01_JPRB, 0.18577E+01_JPRB, 0.19703E+01_JPRB, 0.21120E+01_JPRB ,&
&  0.22874E+01_JPRB, 0.24443E+01_JPRB, 0.26157E+01_JPRB, 0.28547E+01_JPRB, 0.29948E+01_JPRB, 0.30961E+01_JPRB, 0.32318E+01_JPRB ,&
&  0.17054E+01_JPRB, 0.23386E+01_JPRB, 0.24891E+01_JPRB, 0.25252E+01_JPRB, 0.27656E+01_JPRB, 0.28608E+01_JPRB, 0.29214E+01_JPRB ,&
&  0.30405E+01_JPRB, 0.29494E+01_JPRB, 0.27701E+01_JPRB, 0.25691E+01_JPRB, 0.27113E+01_JPRB, 0.27887E+01_JPRB, 0.28669E+01_JPRB ,&
&  0.28926E+01_JPRB, 0.29203E+01_JPRB, 0.30150E+01_JPRB, 0.31397E+01_JPRB, 0.32587E+01_JPRB, 0.33766E+01_JPRB, 0.35298E+01_JPRB ,&
&  0.36241E+01_JPRB, 0.36987E+01_JPRB, 0.38729E+01_JPRB, 0.35280E+01_JPRB, 0.32850E+01_JPRB, 0.34707E+01_JPRB, 0.31729E+01_JPRB ,&
&  0.21911E+01_JPRB, 0.13001E+01_JPRB, 0.14439E+01_JPRB, 0.17895E+01_JPRB, 0.15052E+01_JPRB, 0.12983E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.94995E+00_JPRB, 0.10245E+01_JPRB, 0.11175E+01_JPRB, 0.12176E+01_JPRB, 0.13039E+01_JPRB, 0.13290E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.12377E+01_JPRB, 0.11991E+01_JPRB, 0.12107E+01_JPRB, 0.12519E+01_JPRB, 0.13006E+01_JPRB, 0.13251E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.12972E+01_JPRB, 0.13036E+01_JPRB, 0.13379E+01_JPRB, 0.13500E+01_JPRB, 0.13293E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.16895E+01_JPRB, 0.19018E+01_JPRB, 0.21454E+01_JPRB, 0.20640E+01_JPRB, 0.18400E+01_JPRB, 0.13575E+01_JPRB, 0.95787E+00_JPRB ,&
&  0.98716E+00_JPRB, 0.10146E+01_JPRB, 0.10247E+01_JPRB, 0.10462E+01_JPRB, 0.10415E+01_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.92645E+00_JPRB, 0.91957E+00_JPRB, 0.91405E+00_JPRB, 0.92403E+00_JPRB, 0.93834E+00_JPRB, 0.99678E+00_JPRB, 0.10941E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.12666E+01_JPRB, 0.13313E+01_JPRB, 0.13808E+01_JPRB, 0.14141E+01_JPRB, 0.14163E+01_JPRB, 0.14431E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.15158E+01_JPRB, 0.15866E+01_JPRB, 0.16833E+01_JPRB, 0.17845E+01_JPRB, 0.19002E+01_JPRB, 0.20396E+01_JPRB ,&
&  0.22116E+01_JPRB, 0.24387E+01_JPRB, 0.26923E+01_JPRB, 0.30340E+01_JPRB, 0.33596E+01_JPRB, 0.36406E+01_JPRB, 0.37818E+01_JPRB ,&
&  0.17398E+01_JPRB, 0.18260E+01_JPRB, 0.24970E+01_JPRB, 0.24531E+01_JPRB, 0.26972E+01_JPRB, 0.30086E+01_JPRB, 0.33296E+01_JPRB ,&
&  0.31381E+01_JPRB, 0.26663E+01_JPRB, 0.25180E+01_JPRB, 0.26879E+01_JPRB, 0.26037E+01_JPRB, 0.26742E+01_JPRB, 0.27331E+01_JPRB ,&
&  0.27328E+01_JPRB, 0.27144E+01_JPRB, 0.27358E+01_JPRB, 0.28177E+01_JPRB, 0.29473E+01_JPRB, 0.30860E+01_JPRB, 0.32571E+01_JPRB ,&
&  0.34299E+01_JPRB, 0.36325E+01_JPRB, 0.38385E+01_JPRB, 0.32472E+01_JPRB, 0.34367E+01_JPRB, 0.37409E+01_JPRB, 0.40811E+01_JPRB ,&
&  0.31420E+01_JPRB, 0.18703E+01_JPRB, 0.16290E+01_JPRB, 0.18660E+01_JPRB, 0.15652E+01_JPRB, 0.14703E+01_JPRB, 0.12739E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.11405E+01_JPRB, 0.11912E+01_JPRB, 0.12810E+01_JPRB, 0.13726E+01_JPRB, 0.14458E+01_JPRB, 0.14579E+01_JPRB ,&
&  0.13872E+01_JPRB, 0.13437E+01_JPRB, 0.13236E+01_JPRB, 0.13322E+01_JPRB, 0.13559E+01_JPRB, 0.13728E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.13583E+01_JPRB, 0.13901E+01_JPRB, 0.14077E+01_JPRB, 0.14260E+01_JPRB, 0.14404E+01_JPRB, 0.14543E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.13261E+01_JPRB, 0.13918E+01_JPRB, 0.12731E+01_JPRB, 0.11214E+01_JPRB, 0.97316E+00_JPRB, 0.84051E+00_JPRB ,&
&  0.86792E+00_JPRB, 0.88279E+00_JPRB, 0.88593E+00_JPRB, 0.89944E+00_JPRB, 0.91951E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.82566E+00_JPRB, 0.83245E+00_JPRB, 0.82687E+00_JPRB, 0.85038E+00_JPRB, 0.85652E+00_JPRB, 0.93846E+00_JPRB, 0.10212E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.12550E+01_JPRB, 0.13257E+01_JPRB, 0.13592E+01_JPRB, 0.13543E+01_JPRB, 0.13288E+01_JPRB, 0.13109E+01_JPRB ,&
&  0.13413E+01_JPRB, 0.13685E+01_JPRB, 0.14051E+01_JPRB, 0.14584E+01_JPRB, 0.15215E+01_JPRB, 0.16085E+01_JPRB, 0.17059E+01_JPRB ,&
&  0.18516E+01_JPRB, 0.20665E+01_JPRB, 0.23546E+01_JPRB, 0.27649E+01_JPRB, 0.33163E+01_JPRB, 0.40661E+01_JPRB, 0.48676E+01_JPRB ,&
&  0.39477E+01_JPRB, 0.18956E+01_JPRB, 0.27894E+01_JPRB, 0.25044E+01_JPRB, 0.22947E+01_JPRB, 0.27567E+01_JPRB, 0.32362E+01_JPRB ,&
&  0.29737E+01_JPRB, 0.25859E+01_JPRB, 0.23495E+01_JPRB, 0.25273E+01_JPRB, 0.24857E+01_JPRB, 0.25333E+01_JPRB, 0.25663E+01_JPRB ,&
&  0.25816E+01_JPRB, 0.25674E+01_JPRB, 0.25599E+01_JPRB, 0.26259E+01_JPRB, 0.27435E+01_JPRB, 0.29003E+01_JPRB, 0.30678E+01_JPRB ,&
&  0.32701E+01_JPRB, 0.35008E+01_JPRB, 0.37903E+01_JPRB, 0.32498E+01_JPRB, 0.35959E+01_JPRB, 0.36401E+01_JPRB, 0.32364E+01_JPRB ,&
&  0.30723E+01_JPRB, 0.21379E+01_JPRB, 0.17628E+01_JPRB, 0.19735E+01_JPRB, 0.17376E+01_JPRB, 0.13589E+01_JPRB, 0.13617E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.13074E+01_JPRB, 0.13079E+01_JPRB, 0.14059E+01_JPRB, 0.14653E+01_JPRB, 0.15093E+01_JPRB, 0.14956E+01_JPRB ,&
&  0.14829E+01_JPRB, 0.14384E+01_JPRB, 0.14243E+01_JPRB, 0.14334E+01_JPRB, 0.14433E+01_JPRB, 0.14745E+01_JPRB, 0.14798E+01_JPRB ,&
&  0.14802E+01_JPRB, 0.15035E+01_JPRB, 0.15588E+01_JPRB, 0.16121E+01_JPRB, 0.16281E+01_JPRB, 0.15399E+01_JPRB, 0.14604E+01_JPRB ,&
&  0.13979E+01_JPRB, 0.13364E+01_JPRB, 0.11986E+01_JPRB, 0.10516E+01_JPRB, 0.98639E+00_JPRB, 0.89850E+00_JPRB, 0.82747E+00_JPRB ,&
&  0.83840E+00_JPRB, 0.83639E+00_JPRB, 0.82455E+00_JPRB, 0.82631E+00_JPRB, 0.82791E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.89322E+00_JPRB, 0.87446E+00_JPRB, 0.84092E+00_JPRB, 0.82422E+00_JPRB, 0.82383E+00_JPRB, 0.85970E+00_JPRB, 0.90868E+00_JPRB ,&
&  0.98524E+00_JPRB, 0.10902E+01_JPRB, 0.11789E+01_JPRB, 0.12279E+01_JPRB, 0.12258E+01_JPRB, 0.12401E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.12181E+01_JPRB, 0.12405E+01_JPRB, 0.12682E+01_JPRB, 0.12845E+01_JPRB, 0.13124E+01_JPRB, 0.13515E+01_JPRB, 0.13946E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.14620E+01_JPRB, 0.15317E+01_JPRB, 0.17561E+01_JPRB, 0.21554E+01_JPRB, 0.27809E+01_JPRB, 0.39195E+01_JPRB ,&
&  0.62342E+01_JPRB, 0.64751E+01_JPRB, 0.30988E+01_JPRB, 0.32919E+01_JPRB, 0.27687E+01_JPRB, 0.29664E+01_JPRB, 0.28082E+01_JPRB ,&
&  0.28993E+01_JPRB, 0.25703E+01_JPRB, 0.22855E+01_JPRB, 0.23548E+01_JPRB, 0.23093E+01_JPRB, 0.23428E+01_JPRB, 0.23681E+01_JPRB ,&
&  0.24288E+01_JPRB, 0.24484E+01_JPRB, 0.24222E+01_JPRB, 0.24283E+01_JPRB, 0.25173E+01_JPRB, 0.26579E+01_JPRB, 0.28276E+01_JPRB ,&
&  0.30536E+01_JPRB, 0.32607E+01_JPRB, 0.35573E+01_JPRB, 0.32712E+01_JPRB, 0.35874E+01_JPRB, 0.33343E+01_JPRB, 0.27161E+01_JPRB ,&
&  0.23238E+01_JPRB, 0.21188E+01_JPRB, 0.20797E+01_JPRB, 0.21094E+01_JPRB, 0.18978E+01_JPRB, 0.14876E+01_JPRB, 0.14756E+01_JPRB ,&
&  0.14086E+01_JPRB, 0.13884E+01_JPRB, 0.13886E+01_JPRB, 0.14264E+01_JPRB, 0.14871E+01_JPRB, 0.15109E+01_JPRB, 0.15238E+01_JPRB ,&
&  0.15101E+01_JPRB, 0.14809E+01_JPRB, 0.14725E+01_JPRB, 0.14744E+01_JPRB, 0.14829E+01_JPRB, 0.15162E+01_JPRB, 0.15353E+01_JPRB ,&
&  0.15567E+01_JPRB, 0.15977E+01_JPRB, 0.16577E+01_JPRB, 0.17146E+01_JPRB, 0.16854E+01_JPRB, 0.15920E+01_JPRB, 0.14832E+01_JPRB ,&
&  0.14442E+01_JPRB, 0.14343E+01_JPRB, 0.13255E+01_JPRB, 0.12447E+01_JPRB, 0.11493E+01_JPRB, 0.11074E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10379E+01_JPRB, 0.99447E+00_JPRB, 0.88813E+00_JPRB, 0.88923E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.99983E+00_JPRB, 0.95084E+00_JPRB, 0.94722E+00_JPRB, 0.94459E+00_JPRB, 0.93469E+00_JPRB, 0.93888E+00_JPRB, 0.89381E+00_JPRB ,&
&  0.88542E+00_JPRB, 0.93546E+00_JPRB, 0.10066E+01_JPRB, 0.10484E+01_JPRB, 0.10756E+01_JPRB, 0.10818E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.11161E+01_JPRB, 0.11483E+01_JPRB, 0.11710E+01_JPRB, 0.12013E+01_JPRB, 0.12264E+01_JPRB, 0.12571E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.12545E+01_JPRB, 0.12066E+01_JPRB, 0.12086E+01_JPRB, 0.13547E+01_JPRB, 0.16101E+01_JPRB, 0.20163E+01_JPRB ,&
&  0.35241E+01_JPRB, 0.79606E+01_JPRB, 0.36595E+01_JPRB, 0.36336E+01_JPRB, 0.34867E+01_JPRB, 0.32818E+01_JPRB, 0.30343E+01_JPRB ,&
&  0.30374E+01_JPRB, 0.25615E+01_JPRB, 0.21746E+01_JPRB, 0.21517E+01_JPRB, 0.20854E+01_JPRB, 0.20677E+01_JPRB, 0.20977E+01_JPRB ,&
&  0.21483E+01_JPRB, 0.21917E+01_JPRB, 0.22210E+01_JPRB, 0.21932E+01_JPRB, 0.22501E+01_JPRB, 0.23429E+01_JPRB, 0.24875E+01_JPRB ,&
&  0.26719E+01_JPRB, 0.29224E+01_JPRB, 0.32619E+01_JPRB, 0.31856E+01_JPRB, 0.34108E+01_JPRB, 0.32203E+01_JPRB, 0.26735E+01_JPRB ,&
&  0.23478E+01_JPRB, 0.23368E+01_JPRB, 0.22301E+01_JPRB, 0.21638E+01_JPRB, 0.19806E+01_JPRB, 0.16967E+01_JPRB, 0.16970E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.14783E+01_JPRB, 0.14780E+01_JPRB, 0.14787E+01_JPRB, 0.15044E+01_JPRB, 0.15122E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.15084E+01_JPRB, 0.14840E+01_JPRB, 0.14760E+01_JPRB, 0.14863E+01_JPRB, 0.14933E+01_JPRB, 0.15004E+01_JPRB, 0.15291E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.16073E+01_JPRB, 0.16813E+01_JPRB, 0.16956E+01_JPRB, 0.16612E+01_JPRB, 0.15640E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.14659E+01_JPRB, 0.13792E+01_JPRB, 0.13158E+01_JPRB, 0.12934E+01_JPRB, 0.12429E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.14240E+01_JPRB, 0.14857E+01_JPRB, 0.10319E+01_JPRB, 0.10278E+01_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.10667E+01_JPRB, 0.10180E+01_JPRB, 0.10192E+01_JPRB, 0.10510E+01_JPRB, 0.10539E+01_JPRB, 0.10315E+01_JPRB, 0.98252E+00_JPRB ,&
&  0.91795E+00_JPRB, 0.91070E+00_JPRB, 0.92881E+00_JPRB, 0.95160E+00_JPRB, 0.97195E+00_JPRB, 0.99246E+00_JPRB, 0.99341E+00_JPRB ,&
&  0.98223E+00_JPRB, 0.10031E+01_JPRB, 0.10270E+01_JPRB, 0.10642E+01_JPRB, 0.10996E+01_JPRB, 0.11337E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11709E+01_JPRB, 0.11439E+01_JPRB, 0.11355E+01_JPRB, 0.11584E+01_JPRB, 0.12396E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.22293E+01_JPRB, 0.66920E+01_JPRB, 0.60743E+01_JPRB, 0.46050E+01_JPRB, 0.36762E+01_JPRB, 0.32083E+01_JPRB, 0.30184E+01_JPRB ,&
&  0.33293E+01_JPRB, 0.33461E+01_JPRB, 0.28184E+01_JPRB, 0.22229E+01_JPRB, 0.19808E+01_JPRB, 0.19162E+01_JPRB, 0.19113E+01_JPRB ,&
&  0.18921E+01_JPRB, 0.19077E+01_JPRB, 0.19348E+01_JPRB, 0.19341E+01_JPRB, 0.19377E+01_JPRB, 0.20181E+01_JPRB, 0.21230E+01_JPRB ,&
&  0.22314E+01_JPRB, 0.24559E+01_JPRB, 0.28755E+01_JPRB, 0.32698E+01_JPRB, 0.32462E+01_JPRB, 0.33161E+01_JPRB, 0.31562E+01_JPRB ,&
&  0.32738E+01_JPRB, 0.29750E+01_JPRB, 0.25018E+01_JPRB, 0.21457E+01_JPRB, 0.18880E+01_JPRB, 0.19505E+01_JPRB, 0.18539E+01_JPRB ,&
&  0.16426E+01_JPRB, 0.15393E+01_JPRB, 0.15033E+01_JPRB, 0.15038E+01_JPRB, 0.15117E+01_JPRB, 0.15142E+01_JPRB, 0.14853E+01_JPRB ,&
&  0.14645E+01_JPRB, 0.14583E+01_JPRB, 0.14627E+01_JPRB, 0.14792E+01_JPRB, 0.14851E+01_JPRB, 0.14875E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.15183E+01_JPRB, 0.15512E+01_JPRB, 0.15681E+01_JPRB, 0.16244E+01_JPRB, 0.15951E+01_JPRB, 0.14955E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.13826E+01_JPRB, 0.14175E+01_JPRB, 0.14053E+01_JPRB, 0.13796E+01_JPRB, 0.13475E+01_JPRB, 0.13048E+01_JPRB, 0.12560E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.12609E+01_JPRB, 0.11782E+01_JPRB, 0.11051E+01_JPRB, 0.11091E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.11413E+01_JPRB, 0.11101E+01_JPRB, 0.10896E+01_JPRB, 0.10994E+01_JPRB, 0.11080E+01_JPRB, 0.10909E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.98586E+00_JPRB, 0.98728E+00_JPRB, 0.97551E+00_JPRB, 0.96757E+00_JPRB, 0.96602E+00_JPRB, 0.97045E+00_JPRB ,&
&  0.98047E+00_JPRB, 0.99573E+00_JPRB, 0.98880E+00_JPRB, 0.99441E+00_JPRB, 0.10206E+01_JPRB, 0.10548E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11001E+01_JPRB, 0.10925E+01_JPRB, 0.10918E+01_JPRB, 0.10901E+01_JPRB, 0.11270E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.17521E+01_JPRB, 0.53005E+01_JPRB, 0.34557E+01_JPRB, 0.47286E+01_JPRB, 0.36234E+01_JPRB, 0.31424E+01_JPRB, 0.26345E+01_JPRB ,&
&  0.28662E+01_JPRB, 0.34556E+01_JPRB, 0.26739E+01_JPRB, 0.21888E+01_JPRB, 0.19370E+01_JPRB, 0.18403E+01_JPRB, 0.17848E+01_JPRB ,&
&  0.17495E+01_JPRB, 0.17505E+01_JPRB, 0.17511E+01_JPRB, 0.17754E+01_JPRB, 0.17634E+01_JPRB, 0.18004E+01_JPRB, 0.18576E+01_JPRB ,&
&  0.19029E+01_JPRB, 0.20026E+01_JPRB, 0.22811E+01_JPRB, 0.26216E+01_JPRB, 0.33126E+01_JPRB, 0.36944E+01_JPRB, 0.38647E+01_JPRB ,&
&  0.39966E+01_JPRB, 0.35538E+01_JPRB, 0.29378E+01_JPRB, 0.23832E+01_JPRB, 0.20545E+01_JPRB, 0.19908E+01_JPRB, 0.19642E+01_JPRB ,&
&  0.17826E+01_JPRB, 0.16483E+01_JPRB, 0.15693E+01_JPRB, 0.15334E+01_JPRB, 0.15408E+01_JPRB, 0.15406E+01_JPRB, 0.14899E+01_JPRB ,&
&  0.14499E+01_JPRB, 0.14497E+01_JPRB, 0.14675E+01_JPRB, 0.14870E+01_JPRB, 0.14825E+01_JPRB, 0.14617E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.14771E+01_JPRB, 0.14917E+01_JPRB, 0.14978E+01_JPRB, 0.15834E+01_JPRB, 0.15467E+01_JPRB, 0.14755E+01_JPRB, 0.13980E+01_JPRB ,&
&  0.13891E+01_JPRB, 0.13937E+01_JPRB, 0.13838E+01_JPRB, 0.13688E+01_JPRB, 0.13879E+01_JPRB, 0.13959E+01_JPRB, 0.12833E+01_JPRB ,&
&  0.12568E+01_JPRB, 0.12369E+01_JPRB, 0.12084E+01_JPRB, 0.11634E+01_JPRB, 0.11654E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.11627E+01_JPRB, 0.11416E+01_JPRB, 0.11482E+01_JPRB, 0.11507E+01_JPRB, 0.11419E+01_JPRB, 0.11134E+01_JPRB, 0.10936E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.10745E+01_JPRB, 0.10566E+01_JPRB, 0.10276E+01_JPRB, 0.10107E+01_JPRB, 0.99306E+00_JPRB, 0.98749E+00_JPRB ,&
&  0.99547E+00_JPRB, 0.10012E+01_JPRB, 0.99365E+00_JPRB, 0.99340E+00_JPRB, 0.98722E+00_JPRB, 0.99995E+00_JPRB, 0.10181E+01_JPRB ,&
&  0.10370E+01_JPRB, 0.10483E+01_JPRB, 0.10605E+01_JPRB, 0.10638E+01_JPRB, 0.10607E+01_JPRB, 0.10667E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.15782E+01_JPRB, 0.56229E+01_JPRB, 0.30971E+01_JPRB, 0.47478E+01_JPRB, 0.35489E+01_JPRB, 0.31102E+01_JPRB, 0.25320E+01_JPRB ,&
&  0.26821E+01_JPRB, 0.31468E+01_JPRB, 0.25737E+01_JPRB, 0.20990E+01_JPRB, 0.19266E+01_JPRB, 0.18191E+01_JPRB, 0.17088E+01_JPRB ,&
&  0.16596E+01_JPRB, 0.16647E+01_JPRB, 0.16584E+01_JPRB, 0.16766E+01_JPRB, 0.16919E+01_JPRB, 0.16861E+01_JPRB, 0.17010E+01_JPRB ,&
&  0.16964E+01_JPRB, 0.17170E+01_JPRB, 0.18728E+01_JPRB, 0.24901E+01_JPRB, 0.30139E+01_JPRB, 0.39277E+01_JPRB, 0.48957E+01_JPRB ,&
&  0.47648E+01_JPRB, 0.41836E+01_JPRB, 0.31342E+01_JPRB, 0.26334E+01_JPRB, 0.22874E+01_JPRB, 0.21243E+01_JPRB, 0.20307E+01_JPRB ,&
&  0.19260E+01_JPRB, 0.18328E+01_JPRB, 0.16963E+01_JPRB, 0.16299E+01_JPRB, 0.16130E+01_JPRB, 0.15982E+01_JPRB, 0.15608E+01_JPRB ,&
&  0.15173E+01_JPRB, 0.15125E+01_JPRB, 0.15243E+01_JPRB, 0.15279E+01_JPRB, 0.15179E+01_JPRB, 0.14820E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.14825E+01_JPRB, 0.14841E+01_JPRB, 0.15003E+01_JPRB, 0.16182E+01_JPRB, 0.15771E+01_JPRB, 0.14803E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.14220E+01_JPRB, 0.13946E+01_JPRB, 0.13864E+01_JPRB, 0.14073E+01_JPRB, 0.14385E+01_JPRB, 0.15143E+01_JPRB, 0.13718E+01_JPRB ,&
&  0.12989E+01_JPRB, 0.12441E+01_JPRB, 0.12099E+01_JPRB, 0.11857E+01_JPRB, 0.11761E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.12126E+01_JPRB, 0.11960E+01_JPRB, 0.11985E+01_JPRB, 0.12150E+01_JPRB, 0.11950E+01_JPRB, 0.11600E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.11231E+01_JPRB, 0.11048E+01_JPRB, 0.10666E+01_JPRB, 0.10482E+01_JPRB, 0.10345E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10262E+01_JPRB, 0.10070E+01_JPRB, 0.99649E+00_JPRB, 0.99409E+00_JPRB, 0.99062E+00_JPRB, 0.10054E+01_JPRB ,&
&  0.10000E+01_JPRB, 0.10094E+01_JPRB, 0.10332E+01_JPRB, 0.10395E+01_JPRB, 0.10468E+01_JPRB, 0.10373E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.13793E+01_JPRB, 0.19994E+01_JPRB, 0.36953E+01_JPRB, 0.43400E+01_JPRB, 0.35142E+01_JPRB, 0.31180E+01_JPRB, 0.28504E+01_JPRB ,&
&  0.28091E+01_JPRB, 0.28022E+01_JPRB, 0.24428E+01_JPRB, 0.21336E+01_JPRB, 0.19585E+01_JPRB, 0.18034E+01_JPRB, 0.16671E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.15330E+01_JPRB, 0.15461E+01_JPRB, 0.15855E+01_JPRB, 0.16097E+01_JPRB, 0.16121E+01_JPRB, 0.15794E+01_JPRB ,&
&  0.15440E+01_JPRB, 0.15203E+01_JPRB, 0.15741E+01_JPRB, 0.19809E+01_JPRB, 0.29225E+01_JPRB, 0.40542E+01_JPRB, 0.48570E+01_JPRB ,&
&  0.49805E+01_JPRB, 0.42117E+01_JPRB, 0.31790E+01_JPRB, 0.27565E+01_JPRB, 0.24553E+01_JPRB, 0.22295E+01_JPRB, 0.20383E+01_JPRB ,&
&  0.19271E+01_JPRB, 0.18898E+01_JPRB, 0.18262E+01_JPRB, 0.17390E+01_JPRB, 0.16585E+01_JPRB, 0.16147E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.15549E+01_JPRB, 0.15644E+01_JPRB, 0.15823E+01_JPRB, 0.15590E+01_JPRB, 0.15295E+01_JPRB, 0.14902E+01_JPRB, 0.14711E+01_JPRB ,&
&  0.14879E+01_JPRB, 0.15123E+01_JPRB, 0.15470E+01_JPRB, 0.16152E+01_JPRB, 0.15864E+01_JPRB, 0.15324E+01_JPRB, 0.14732E+01_JPRB ,&
&  0.14290E+01_JPRB, 0.13877E+01_JPRB, 0.13713E+01_JPRB, 0.14428E+01_JPRB, 0.14649E+01_JPRB, 0.15576E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.13012E+01_JPRB, 0.12618E+01_JPRB, 0.12404E+01_JPRB, 0.12332E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.12746E+01_JPRB, 0.12639E+01_JPRB, 0.12741E+01_JPRB, 0.12861E+01_JPRB, 0.12712E+01_JPRB, 0.12322E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.11654E+01_JPRB, 0.11394E+01_JPRB, 0.11089E+01_JPRB, 0.10713E+01_JPRB, 0.10570E+01_JPRB, 0.10618E+01_JPRB, 0.10694E+01_JPRB ,&
&  0.10747E+01_JPRB, 0.10606E+01_JPRB, 0.10374E+01_JPRB, 0.10130E+01_JPRB, 0.10063E+01_JPRB, 0.10047E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.10063E+01_JPRB, 0.10045E+01_JPRB, 0.10120E+01_JPRB, 0.10201E+01_JPRB, 0.10287E+01_JPRB, 0.10224E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.27696E+01_JPRB, 0.33535E+01_JPRB, 0.35059E+01_JPRB, 0.33195E+01_JPRB, 0.29984E+01_JPRB, 0.28436E+01_JPRB ,&
&  0.25815E+01_JPRB, 0.24822E+01_JPRB, 0.23559E+01_JPRB, 0.22034E+01_JPRB, 0.20270E+01_JPRB, 0.18483E+01_JPRB, 0.16950E+01_JPRB ,&
&  0.15815E+01_JPRB, 0.15216E+01_JPRB, 0.14949E+01_JPRB, 0.15005E+01_JPRB, 0.15154E+01_JPRB, 0.15313E+01_JPRB, 0.15024E+01_JPRB ,&
&  0.14602E+01_JPRB, 0.14225E+01_JPRB, 0.14105E+01_JPRB, 0.16078E+01_JPRB, 0.23635E+01_JPRB, 0.28329E+01_JPRB, 0.39855E+01_JPRB ,&
&  0.44113E+01_JPRB, 0.33993E+01_JPRB, 0.27820E+01_JPRB, 0.24939E+01_JPRB, 0.22875E+01_JPRB, 0.20807E+01_JPRB, 0.19272E+01_JPRB ,&
&  0.18556E+01_JPRB, 0.18136E+01_JPRB, 0.18051E+01_JPRB, 0.17577E+01_JPRB, 0.16576E+01_JPRB, 0.15589E+01_JPRB, 0.15124E+01_JPRB ,&
&  0.14875E+01_JPRB, 0.14670E+01_JPRB, 0.14648E+01_JPRB, 0.14737E+01_JPRB, 0.14687E+01_JPRB, 0.14294E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.14533E+01_JPRB, 0.15216E+01_JPRB, 0.15388E+01_JPRB, 0.15556E+01_JPRB, 0.15689E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.13733E+01_JPRB, 0.13645E+01_JPRB, 0.14274E+01_JPRB, 0.15398E+01_JPRB, 0.16346E+01_JPRB, 0.15429E+01_JPRB ,&
&  0.14350E+01_JPRB, 0.13515E+01_JPRB, 0.13090E+01_JPRB, 0.12834E+01_JPRB, 0.12948E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.13154E+01_JPRB, 0.12994E+01_JPRB, 0.13048E+01_JPRB, 0.13088E+01_JPRB, 0.12956E+01_JPRB, 0.12573E+01_JPRB, 0.12230E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.11552E+01_JPRB, 0.11099E+01_JPRB, 0.10770E+01_JPRB, 0.10751E+01_JPRB, 0.10821E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10716E+01_JPRB, 0.10528E+01_JPRB, 0.10340E+01_JPRB, 0.10232E+01_JPRB, 0.10133E+01_JPRB, 0.99599E+00_JPRB ,&
&  0.99442E+00_JPRB, 0.99265E+00_JPRB, 0.99755E+00_JPRB, 0.10077E+01_JPRB, 0.10067E+01_JPRB, 0.10114E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.17923E+01_JPRB, 0.20957E+01_JPRB, 0.26514E+01_JPRB, 0.28827E+01_JPRB, 0.27566E+01_JPRB, 0.26027E+01_JPRB ,&
&  0.23996E+01_JPRB, 0.22604E+01_JPRB, 0.21873E+01_JPRB, 0.20998E+01_JPRB, 0.19604E+01_JPRB, 0.18092E+01_JPRB, 0.16733E+01_JPRB ,&
&  0.15771E+01_JPRB, 0.15217E+01_JPRB, 0.15030E+01_JPRB, 0.14794E+01_JPRB, 0.14543E+01_JPRB, 0.14513E+01_JPRB, 0.14517E+01_JPRB ,&
&  0.13961E+01_JPRB, 0.13532E+01_JPRB, 0.13245E+01_JPRB, 0.13556E+01_JPRB, 0.17021E+01_JPRB, 0.21388E+01_JPRB, 0.27516E+01_JPRB ,&
&  0.26226E+01_JPRB, 0.23549E+01_JPRB, 0.21765E+01_JPRB, 0.20406E+01_JPRB, 0.19459E+01_JPRB, 0.18154E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.16081E+01_JPRB, 0.15652E+01_JPRB, 0.15267E+01_JPRB, 0.15043E+01_JPRB, 0.14729E+01_JPRB, 0.14500E+01_JPRB, 0.13951E+01_JPRB ,&
&  0.13605E+01_JPRB, 0.13373E+01_JPRB, 0.13314E+01_JPRB, 0.13459E+01_JPRB, 0.13745E+01_JPRB, 0.13788E+01_JPRB, 0.13769E+01_JPRB ,&
&  0.13901E+01_JPRB, 0.14047E+01_JPRB, 0.14555E+01_JPRB, 0.15158E+01_JPRB, 0.15007E+01_JPRB, 0.14644E+01_JPRB, 0.14387E+01_JPRB ,&
&  0.14175E+01_JPRB, 0.13943E+01_JPRB, 0.13529E+01_JPRB, 0.14351E+01_JPRB, 0.15084E+01_JPRB, 0.15988E+01_JPRB, 0.15484E+01_JPRB ,&
&  0.14688E+01_JPRB, 0.13758E+01_JPRB, 0.13291E+01_JPRB, 0.13040E+01_JPRB, 0.13201E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.13397E+01_JPRB, 0.13399E+01_JPRB, 0.13305E+01_JPRB, 0.13224E+01_JPRB, 0.13076E+01_JPRB, 0.12746E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.11726E+01_JPRB, 0.11245E+01_JPRB, 0.10957E+01_JPRB, 0.10848E+01_JPRB, 0.10811E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.10693E+01_JPRB, 0.10597E+01_JPRB, 0.10560E+01_JPRB, 0.10445E+01_JPRB, 0.10343E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.99080E+00_JPRB, 0.97860E+00_JPRB, 0.97548E+00_JPRB, 0.98339E+00_JPRB, 0.99112E+00_JPRB, 0.99665E+00_JPRB, 0.10106E+01_JPRB ,&
&  0.10909E+01_JPRB, 0.11807E+01_JPRB, 0.13949E+01_JPRB, 0.17884E+01_JPRB, 0.20851E+01_JPRB, 0.22449E+01_JPRB, 0.21870E+01_JPRB ,&
&  0.20740E+01_JPRB, 0.19648E+01_JPRB, 0.18977E+01_JPRB, 0.18164E+01_JPRB, 0.17187E+01_JPRB, 0.16151E+01_JPRB, 0.15366E+01_JPRB ,&
&  0.14968E+01_JPRB, 0.14615E+01_JPRB, 0.14312E+01_JPRB, 0.14074E+01_JPRB, 0.13890E+01_JPRB, 0.13757E+01_JPRB, 0.13797E+01_JPRB ,&
&  0.13644E+01_JPRB, 0.13219E+01_JPRB, 0.12773E+01_JPRB, 0.12499E+01_JPRB, 0.13044E+01_JPRB, 0.14596E+01_JPRB, 0.17397E+01_JPRB ,&
&  0.18893E+01_JPRB, 0.18014E+01_JPRB, 0.17227E+01_JPRB, 0.16528E+01_JPRB, 0.15959E+01_JPRB, 0.15624E+01_JPRB, 0.15270E+01_JPRB ,&
&  0.15014E+01_JPRB, 0.14806E+01_JPRB, 0.14669E+01_JPRB, 0.14484E+01_JPRB, 0.14278E+01_JPRB, 0.14068E+01_JPRB, 0.13783E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.13390E+01_JPRB, 0.13242E+01_JPRB, 0.13300E+01_JPRB, 0.13381E+01_JPRB, 0.13426E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.13984E+01_JPRB, 0.14260E+01_JPRB, 0.14542E+01_JPRB, 0.14454E+01_JPRB, 0.14172E+01_JPRB, 0.14014E+01_JPRB ,&
&  0.13794E+01_JPRB, 0.13605E+01_JPRB, 0.13092E+01_JPRB, 0.13781E+01_JPRB, 0.14967E+01_JPRB, 0.14879E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.14564E+01_JPRB, 0.14174E+01_JPRB, 0.13507E+01_JPRB, 0.12857E+01_JPRB, 0.13034E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.12232E+01_JPRB, 0.12536E+01_JPRB, 0.12490E+01_JPRB, 0.12433E+01_JPRB, 0.12372E+01_JPRB, 0.12211E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11588E+01_JPRB, 0.11369E+01_JPRB, 0.11085E+01_JPRB, 0.10883E+01_JPRB, 0.10742E+01_JPRB, 0.10616E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10458E+01_JPRB, 0.10392E+01_JPRB, 0.10315E+01_JPRB, 0.10183E+01_JPRB, 0.10068E+01_JPRB, 0.98970E+00_JPRB ,&
&  0.97722E+00_JPRB, 0.97375E+00_JPRB, 0.97275E+00_JPRB, 0.97660E+00_JPRB, 0.98487E+00_JPRB, 0.99476E+00_JPRB, 0.10022E+01_JPRB ,&
&  0.10437E+01_JPRB, 0.10055E+01_JPRB, 0.10790E+01_JPRB, 0.12834E+01_JPRB, 0.15465E+01_JPRB, 0.17701E+01_JPRB, 0.18334E+01_JPRB ,&
&  0.17923E+01_JPRB, 0.17287E+01_JPRB, 0.16526E+01_JPRB, 0.15803E+01_JPRB, 0.15193E+01_JPRB, 0.14731E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.13426E+01_JPRB, 0.13331E+01_JPRB, 0.13188E+01_JPRB, 0.13014E+01_JPRB, 0.12878E+01_JPRB, 0.12855E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.12525E+01_JPRB, 0.12211E+01_JPRB, 0.12017E+01_JPRB, 0.12117E+01_JPRB, 0.12883E+01_JPRB, 0.14102E+01_JPRB ,&
&  0.15166E+01_JPRB, 0.14982E+01_JPRB, 0.14493E+01_JPRB, 0.14013E+01_JPRB, 0.13757E+01_JPRB, 0.13674E+01_JPRB, 0.13678E+01_JPRB ,&
&  0.13606E+01_JPRB, 0.13540E+01_JPRB, 0.13656E+01_JPRB, 0.13698E+01_JPRB, 0.13743E+01_JPRB, 0.13717E+01_JPRB, 0.13665E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.13308E+01_JPRB, 0.13194E+01_JPRB, 0.13150E+01_JPRB, 0.13212E+01_JPRB, 0.13221E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.13163E+01_JPRB, 0.13590E+01_JPRB, 0.13943E+01_JPRB, 0.13860E+01_JPRB, 0.13670E+01_JPRB, 0.13491E+01_JPRB ,&
&  0.13437E+01_JPRB, 0.13300E+01_JPRB, 0.13197E+01_JPRB, 0.13746E+01_JPRB, 0.13800E+01_JPRB, 0.13745E+01_JPRB, 0.13674E+01_JPRB ,&
&  0.13624E+01_JPRB, 0.13416E+01_JPRB, 0.13173E+01_JPRB, 0.12709E+01_JPRB, 0.11642E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.11179E+01_JPRB, 0.11495E+01_JPRB, 0.11643E+01_JPRB, 0.11655E+01_JPRB, 0.11708E+01_JPRB, 0.11738E+01_JPRB, 0.11716E+01_JPRB ,&
&  0.11694E+01_JPRB, 0.11634E+01_JPRB, 0.11483E+01_JPRB, 0.11242E+01_JPRB, 0.10995E+01_JPRB, 0.10724E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.10402E+01_JPRB, 0.10324E+01_JPRB, 0.10148E+01_JPRB, 0.99889E+00_JPRB, 0.98437E+00_JPRB, 0.97265E+00_JPRB, 0.96689E+00_JPRB ,&
&  0.96017E+00_JPRB, 0.95666E+00_JPRB, 0.96091E+00_JPRB, 0.96724E+00_JPRB, 0.97648E+00_JPRB, 0.98714E+00_JPRB, 0.10011E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.86485E+00_JPRB, 0.91193E+00_JPRB, 0.10464E+01_JPRB, 0.11956E+01_JPRB, 0.13590E+01_JPRB, 0.14974E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.15415E+01_JPRB, 0.15017E+01_JPRB, 0.14550E+01_JPRB, 0.14253E+01_JPRB, 0.13813E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.12458E+01_JPRB, 0.12496E+01_JPRB, 0.12479E+01_JPRB, 0.12339E+01_JPRB, 0.12181E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.12080E+01_JPRB, 0.12053E+01_JPRB, 0.12025E+01_JPRB, 0.11989E+01_JPRB, 0.12135E+01_JPRB, 0.12474E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.13115E+01_JPRB, 0.13037E+01_JPRB, 0.12845E+01_JPRB, 0.12586E+01_JPRB, 0.12443E+01_JPRB, 0.12506E+01_JPRB, 0.12569E+01_JPRB ,&
&  0.12562E+01_JPRB, 0.12543E+01_JPRB, 0.12618E+01_JPRB, 0.12682E+01_JPRB, 0.12815E+01_JPRB, 0.12953E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.13310E+01_JPRB, 0.13272E+01_JPRB, 0.13196E+01_JPRB, 0.13028E+01_JPRB, 0.12776E+01_JPRB, 0.12609E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.12569E+01_JPRB, 0.12795E+01_JPRB, 0.13047E+01_JPRB, 0.13088E+01_JPRB, 0.13075E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.13111E+01_JPRB, 0.13120E+01_JPRB, 0.12749E+01_JPRB, 0.13189E+01_JPRB, 0.13120E+01_JPRB, 0.12866E+01_JPRB ,&
&  0.12545E+01_JPRB, 0.12198E+01_JPRB, 0.12003E+01_JPRB, 0.10193E+01_JPRB, 0.10924E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.10429E+01_JPRB, 0.10626E+01_JPRB, 0.10666E+01_JPRB, 0.10801E+01_JPRB, 0.10893E+01_JPRB, 0.10960E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.11081E+01_JPRB, 0.10999E+01_JPRB, 0.10835E+01_JPRB, 0.10571E+01_JPRB, 0.10290E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.98817E+00_JPRB, 0.98278E+00_JPRB, 0.97232E+00_JPRB, 0.95969E+00_JPRB, 0.95173E+00_JPRB, 0.94702E+00_JPRB, 0.94782E+00_JPRB ,&
&  0.95455E+00_JPRB, 0.95778E+00_JPRB, 0.95186E+00_JPRB, 0.94969E+00_JPRB, 0.95764E+00_JPRB, 0.97276E+00_JPRB, 0.97903E+00_JPRB ,&
&  0.98152E+00_JPRB, 0.79256E+00_JPRB, 0.85122E+00_JPRB, 0.92725E+00_JPRB, 0.10148E+01_JPRB, 0.11329E+01_JPRB, 0.12462E+01_JPRB ,&
&  0.13186E+01_JPRB, 0.13565E+01_JPRB, 0.13467E+01_JPRB, 0.13208E+01_JPRB, 0.12913E+01_JPRB, 0.12649E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.11929E+01_JPRB, 0.11724E+01_JPRB, 0.11673E+01_JPRB, 0.11671E+01_JPRB, 0.11659E+01_JPRB, 0.11681E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.11581E+01_JPRB, 0.11554E+01_JPRB, 0.11545E+01_JPRB, 0.11556E+01_JPRB, 0.11541E+01_JPRB, 0.11525E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11412E+01_JPRB, 0.11337E+01_JPRB, 0.11216E+01_JPRB, 0.11187E+01_JPRB, 0.11257E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.11636E+01_JPRB, 0.11712E+01_JPRB, 0.11815E+01_JPRB, 0.11974E+01_JPRB, 0.12158E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.12498E+01_JPRB, 0.12486E+01_JPRB, 0.12385E+01_JPRB, 0.12345E+01_JPRB, 0.12136E+01_JPRB, 0.11899E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.11845E+01_JPRB, 0.11872E+01_JPRB, 0.12003E+01_JPRB, 0.12071E+01_JPRB, 0.12048E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.12215E+01_JPRB, 0.12114E+01_JPRB, 0.12070E+01_JPRB, 0.11921E+01_JPRB, 0.11807E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.11273E+01_JPRB, 0.11044E+01_JPRB, 0.89550E+00_JPRB, 0.99116E+00_JPRB, 0.10147E+01_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.10171E+01_JPRB, 0.10229E+01_JPRB, 0.10261E+01_JPRB, 0.10310E+01_JPRB, 0.10322E+01_JPRB, 0.10312E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.10413E+01_JPRB, 0.10368E+01_JPRB, 0.10281E+01_JPRB, 0.10244E+01_JPRB, 0.10135E+01_JPRB, 0.99763E+00_JPRB, 0.98419E+00_JPRB ,&
&  0.96794E+00_JPRB, 0.96068E+00_JPRB, 0.95605E+00_JPRB, 0.95114E+00_JPRB, 0.94324E+00_JPRB, 0.93377E+00_JPRB, 0.93659E+00_JPRB ,&
&  0.94310E+00_JPRB, 0.95021E+00_JPRB, 0.95167E+00_JPRB, 0.95480E+00_JPRB, 0.95993E+00_JPRB, 0.96573E+00_JPRB, 0.97612E+00_JPRB ,&
&  0.86266E+00_JPRB, 0.73890E+00_JPRB, 0.79742E+00_JPRB, 0.86099E+00_JPRB, 0.92758E+00_JPRB, 0.10322E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.12238E+01_JPRB, 0.12320E+01_JPRB, 0.12034E+01_JPRB, 0.11733E+01_JPRB, 0.11666E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.11527E+01_JPRB, 0.11483E+01_JPRB, 0.11470E+01_JPRB, 0.11492E+01_JPRB, 0.11547E+01_JPRB, 0.11572E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.11405E+01_JPRB, 0.11229E+01_JPRB, 0.10970E+01_JPRB, 0.10724E+01_JPRB, 0.10589E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10245E+01_JPRB, 0.10252E+01_JPRB, 0.10274E+01_JPRB, 0.10360E+01_JPRB, 0.10484E+01_JPRB, 0.10678E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11105E+01_JPRB, 0.11254E+01_JPRB, 0.11413E+01_JPRB, 0.11492E+01_JPRB, 0.11594E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11928E+01_JPRB, 0.11830E+01_JPRB, 0.11734E+01_JPRB, 0.11644E+01_JPRB, 0.11627E+01_JPRB, 0.11623E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.11368E+01_JPRB, 0.11334E+01_JPRB, 0.11378E+01_JPRB, 0.11348E+01_JPRB, 0.11263E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.11143E+01_JPRB, 0.11001E+01_JPRB, 0.10837E+01_JPRB, 0.10730E+01_JPRB, 0.10598E+01_JPRB, 0.10513E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.10194E+01_JPRB, 0.10231E+01_JPRB, 0.10162E+01_JPRB, 0.10125E+01_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.97848E+00_JPRB, 0.98237E+00_JPRB, 0.98642E+00_JPRB, 0.98768E+00_JPRB, 0.98472E+00_JPRB, 0.98301E+00_JPRB, 0.98541E+00_JPRB ,&
&  0.98731E+00_JPRB, 0.97999E+00_JPRB, 0.97249E+00_JPRB, 0.96651E+00_JPRB, 0.96360E+00_JPRB, 0.95362E+00_JPRB, 0.93868E+00_JPRB ,&
&  0.92786E+00_JPRB, 0.92594E+00_JPRB, 0.93054E+00_JPRB, 0.93266E+00_JPRB, 0.92313E+00_JPRB, 0.91321E+00_JPRB, 0.90949E+00_JPRB ,&
&  0.91024E+00_JPRB, 0.91598E+00_JPRB, 0.92574E+00_JPRB, 0.93077E+00_JPRB, 0.93857E+00_JPRB, 0.95198E+00_JPRB, 0.95160E+00_JPRB ,&
&  0.94956E+00_JPRB, 0.81299E+00_JPRB, 0.85436E+00_JPRB, 0.87166E+00_JPRB, 0.90728E+00_JPRB, 0.96527E+00_JPRB, 0.10200E+01_JPRB ,&
&  0.10753E+01_JPRB, 0.11066E+01_JPRB, 0.11208E+01_JPRB, 0.11136E+01_JPRB, 0.10943E+01_JPRB, 0.10766E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10900E+01_JPRB, 0.11035E+01_JPRB, 0.11115E+01_JPRB, 0.11133E+01_JPRB, 0.11137E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.10945E+01_JPRB, 0.10661E+01_JPRB, 0.10303E+01_JPRB, 0.99942E+00_JPRB, 0.98070E+00_JPRB, 0.96182E+00_JPRB ,&
&  0.93958E+00_JPRB, 0.92745E+00_JPRB, 0.92682E+00_JPRB, 0.93182E+00_JPRB, 0.94349E+00_JPRB, 0.95720E+00_JPRB, 0.96831E+00_JPRB ,&
&  0.98017E+00_JPRB, 0.99832E+00_JPRB, 0.10184E+01_JPRB, 0.10378E+01_JPRB, 0.10557E+01_JPRB, 0.10708E+01_JPRB, 0.10814E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.10853E+01_JPRB, 0.10737E+01_JPRB, 0.10656E+01_JPRB, 0.10638E+01_JPRB, 0.10704E+01_JPRB, 0.10733E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10592E+01_JPRB, 0.10548E+01_JPRB, 0.10426E+01_JPRB, 0.10284E+01_JPRB, 0.10143E+01_JPRB, 0.99944E+00_JPRB ,&
&  0.98764E+00_JPRB, 0.98135E+00_JPRB, 0.97561E+00_JPRB, 0.96754E+00_JPRB, 0.95632E+00_JPRB, 0.95015E+00_JPRB, 0.94502E+00_JPRB ,&
&  0.94078E+00_JPRB, 0.93647E+00_JPRB, 0.93864E+00_JPRB, 0.95041E+00_JPRB, 0.96714E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.92649E+00_JPRB, 0.93322E+00_JPRB, 0.94332E+00_JPRB, 0.95490E+00_JPRB, 0.95953E+00_JPRB, 0.95998E+00_JPRB, 0.95816E+00_JPRB ,&
&  0.95935E+00_JPRB, 0.95159E+00_JPRB, 0.94585E+00_JPRB, 0.94216E+00_JPRB, 0.94110E+00_JPRB, 0.93692E+00_JPRB, 0.92665E+00_JPRB ,&
&  0.91794E+00_JPRB, 0.91415E+00_JPRB, 0.91781E+00_JPRB, 0.91840E+00_JPRB, 0.90214E+00_JPRB, 0.89636E+00_JPRB, 0.88759E+00_JPRB ,&
&  0.88570E+00_JPRB, 0.88474E+00_JPRB, 0.88661E+00_JPRB, 0.89151E+00_JPRB, 0.90668E+00_JPRB, 0.91732E+00_JPRB, 0.92249E+00_JPRB ,&
&  0.91927E+00_JPRB, 0.85150E+00_JPRB, 0.88094E+00_JPRB, 0.89036E+00_JPRB, 0.91129E+00_JPRB, 0.93501E+00_JPRB, 0.96067E+00_JPRB ,&
&  0.99586E+00_JPRB, 0.10175E+01_JPRB, 0.10295E+01_JPRB, 0.10347E+01_JPRB, 0.10371E+01_JPRB, 0.10351E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.10325E+01_JPRB, 0.10536E+01_JPRB, 0.10653E+01_JPRB, 0.10719E+01_JPRB, 0.10756E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10497E+01_JPRB, 0.10211E+01_JPRB, 0.98568E+00_JPRB, 0.94523E+00_JPRB, 0.91498E+00_JPRB, 0.89650E+00_JPRB ,&
&  0.88549E+00_JPRB, 0.88459E+00_JPRB, 0.88639E+00_JPRB, 0.88947E+00_JPRB, 0.89458E+00_JPRB, 0.90492E+00_JPRB, 0.91023E+00_JPRB ,&
&  0.91836E+00_JPRB, 0.92569E+00_JPRB, 0.93783E+00_JPRB, 0.95798E+00_JPRB, 0.97812E+00_JPRB, 0.98959E+00_JPRB, 0.99106E+00_JPRB ,&
&  0.99804E+00_JPRB, 0.10024E+01_JPRB, 0.99973E+00_JPRB, 0.10003E+01_JPRB, 0.99702E+00_JPRB, 0.99919E+00_JPRB, 0.99769E+00_JPRB ,&
&  0.98947E+00_JPRB, 0.98303E+00_JPRB, 0.97462E+00_JPRB, 0.95823E+00_JPRB, 0.94310E+00_JPRB, 0.93115E+00_JPRB, 0.92082E+00_JPRB ,&
&  0.91793E+00_JPRB, 0.92052E+00_JPRB, 0.92501E+00_JPRB, 0.91913E+00_JPRB, 0.90413E+00_JPRB, 0.89438E+00_JPRB, 0.88312E+00_JPRB ,&
&  0.87566E+00_JPRB, 0.87801E+00_JPRB, 0.87860E+00_JPRB, 0.88719E+00_JPRB, 0.90804E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.87200E+00_JPRB, 0.88568E+00_JPRB, 0.90286E+00_JPRB, 0.92058E+00_JPRB, 0.93222E+00_JPRB, 0.93957E+00_JPRB, 0.93816E+00_JPRB ,&
&  0.92766E+00_JPRB, 0.91724E+00_JPRB, 0.91931E+00_JPRB, 0.91943E+00_JPRB, 0.91828E+00_JPRB, 0.91609E+00_JPRB, 0.90886E+00_JPRB ,&
&  0.89852E+00_JPRB, 0.88947E+00_JPRB, 0.88526E+00_JPRB, 0.88164E+00_JPRB, 0.87099E+00_JPRB, 0.86248E+00_JPRB, 0.85319E+00_JPRB ,&
&  0.85187E+00_JPRB, 0.85335E+00_JPRB, 0.85501E+00_JPRB, 0.85391E+00_JPRB, 0.86053E+00_JPRB, 0.86936E+00_JPRB, 0.87294E+00_JPRB ,&
&  0.86647E+00_JPRB, 0.88038E+00_JPRB, 0.79409E+00_JPRB, 0.86286E+00_JPRB, 0.88601E+00_JPRB, 0.89997E+00_JPRB, 0.91766E+00_JPRB ,&
&  0.93191E+00_JPRB, 0.94671E+00_JPRB, 0.95803E+00_JPRB, 0.96679E+00_JPRB, 0.98851E+00_JPRB, 0.10032E+01_JPRB, 0.10044E+01_JPRB ,&
&  0.10056E+01_JPRB, 0.10042E+01_JPRB, 0.10062E+01_JPRB, 0.10086E+01_JPRB, 0.10158E+01_JPRB, 0.10185E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.10077E+01_JPRB, 0.99535E+00_JPRB, 0.97577E+00_JPRB, 0.95224E+00_JPRB, 0.91556E+00_JPRB, 0.88358E+00_JPRB, 0.86270E+00_JPRB ,&
&  0.84625E+00_JPRB, 0.84098E+00_JPRB, 0.83530E+00_JPRB, 0.82986E+00_JPRB, 0.82815E+00_JPRB, 0.83020E+00_JPRB, 0.82915E+00_JPRB ,&
&  0.83554E+00_JPRB, 0.84401E+00_JPRB, 0.84819E+00_JPRB, 0.85618E+00_JPRB, 0.86698E+00_JPRB, 0.88237E+00_JPRB, 0.89724E+00_JPRB ,&
&  0.91145E+00_JPRB, 0.92186E+00_JPRB, 0.92474E+00_JPRB, 0.92594E+00_JPRB, 0.92419E+00_JPRB, 0.92343E+00_JPRB, 0.91662E+00_JPRB ,&
&  0.91125E+00_JPRB, 0.90849E+00_JPRB, 0.90196E+00_JPRB, 0.88933E+00_JPRB, 0.88082E+00_JPRB, 0.87335E+00_JPRB, 0.87075E+00_JPRB ,&
&  0.87064E+00_JPRB, 0.87141E+00_JPRB, 0.86870E+00_JPRB, 0.86929E+00_JPRB, 0.86260E+00_JPRB, 0.84161E+00_JPRB, 0.82508E+00_JPRB ,&
&  0.82391E+00_JPRB, 0.82908E+00_JPRB, 0.83388E+00_JPRB, 0.84308E+00_JPRB, 0.85884E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.88331E+00_JPRB, 0.89013E+00_JPRB, 0.90051E+00_JPRB, 0.91545E+00_JPRB, 0.93105E+00_JPRB, 0.94152E+00_JPRB, 0.94203E+00_JPRB ,&
&  0.93153E+00_JPRB, 0.91839E+00_JPRB, 0.91411E+00_JPRB, 0.91355E+00_JPRB, 0.91059E+00_JPRB, 0.90905E+00_JPRB, 0.90414E+00_JPRB ,&
&  0.89223E+00_JPRB, 0.87424E+00_JPRB, 0.86536E+00_JPRB, 0.85076E+00_JPRB, 0.83687E+00_JPRB, 0.82517E+00_JPRB, 0.82423E+00_JPRB ,&
&  0.83208E+00_JPRB, 0.84148E+00_JPRB, 0.85318E+00_JPRB, 0.86616E+00_JPRB, 0.87217E+00_JPRB, 0.86929E+00_JPRB, 0.86490E+00_JPRB ,&
&  0.86110E+00_JPRB, 0.85832E+00_JPRB, 0.86789E+00_JPRB, 0.87598E+00_JPRB, 0.88107E+00_JPRB, 0.89166E+00_JPRB, 0.90152E+00_JPRB ,&
&  0.91115E+00_JPRB, 0.92426E+00_JPRB, 0.94475E+00_JPRB, 0.96051E+00_JPRB, 0.97324E+00_JPRB, 0.98580E+00_JPRB, 0.98794E+00_JPRB ,&
&  0.99249E+00_JPRB, 0.10032E+01_JPRB, 0.10056E+01_JPRB, 0.99701E+00_JPRB, 0.98358E+00_JPRB, 0.97604E+00_JPRB, 0.97304E+00_JPRB ,&
&  0.96553E+00_JPRB, 0.94683E+00_JPRB, 0.93203E+00_JPRB, 0.91941E+00_JPRB, 0.89847E+00_JPRB, 0.87621E+00_JPRB, 0.85064E+00_JPRB ,&
&  0.82188E+00_JPRB, 0.80045E+00_JPRB, 0.78358E+00_JPRB, 0.76828E+00_JPRB, 0.76369E+00_JPRB, 0.75397E+00_JPRB, 0.74703E+00_JPRB ,&
&  0.75159E+00_JPRB, 0.77050E+00_JPRB, 0.78508E+00_JPRB, 0.79085E+00_JPRB, 0.79448E+00_JPRB, 0.81150E+00_JPRB, 0.83211E+00_JPRB ,&
&  0.84858E+00_JPRB, 0.85642E+00_JPRB, 0.85387E+00_JPRB, 0.85745E+00_JPRB, 0.86355E+00_JPRB, 0.86830E+00_JPRB, 0.86228E+00_JPRB ,&
&  0.85368E+00_JPRB, 0.84822E+00_JPRB, 0.84699E+00_JPRB, 0.84311E+00_JPRB, 0.84427E+00_JPRB, 0.84386E+00_JPRB, 0.84075E+00_JPRB ,&
&  0.83488E+00_JPRB, 0.82567E+00_JPRB, 0.81365E+00_JPRB, 0.80584E+00_JPRB, 0.79460E+00_JPRB, 0.77744E+00_JPRB, 0.77731E+00_JPRB ,&
&  0.78829E+00_JPRB, 0.80640E+00_JPRB, 0.82415E+00_JPRB, 0.84570E+00_JPRB, 0.87112E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.90580E+00_JPRB, 0.91443E+00_JPRB, 0.92055E+00_JPRB, 0.92523E+00_JPRB, 0.93498E+00_JPRB, 0.94716E+00_JPRB, 0.94788E+00_JPRB ,&
&  0.93364E+00_JPRB, 0.91994E+00_JPRB, 0.91022E+00_JPRB, 0.90754E+00_JPRB, 0.90924E+00_JPRB, 0.91148E+00_JPRB, 0.90532E+00_JPRB ,&
&  0.88298E+00_JPRB, 0.85904E+00_JPRB, 0.84406E+00_JPRB, 0.82930E+00_JPRB, 0.81983E+00_JPRB, 0.81350E+00_JPRB, 0.81954E+00_JPRB ,&
&  0.82880E+00_JPRB, 0.84392E+00_JPRB, 0.85598E+00_JPRB, 0.86446E+00_JPRB, 0.86733E+00_JPRB, 0.85491E+00_JPRB, 0.85529E+00_JPRB ,&
&  0.86126E+00_JPRB, 0.86256E+00_JPRB, 0.87678E+00_JPRB, 0.87557E+00_JPRB, 0.86128E+00_JPRB, 0.84942E+00_JPRB, 0.83809E+00_JPRB ,&
&  0.83532E+00_JPRB, 0.84692E+00_JPRB, 0.87309E+00_JPRB, 0.91096E+00_JPRB, 0.93429E+00_JPRB, 0.94356E+00_JPRB, 0.93942E+00_JPRB ,&
&  0.94053E+00_JPRB, 0.95185E+00_JPRB, 0.95857E+00_JPRB, 0.95065E+00_JPRB, 0.93307E+00_JPRB, 0.91895E+00_JPRB, 0.90699E+00_JPRB ,&
&  0.89916E+00_JPRB, 0.88927E+00_JPRB, 0.88216E+00_JPRB, 0.87253E+00_JPRB, 0.84975E+00_JPRB, 0.82264E+00_JPRB, 0.79877E+00_JPRB ,&
&  0.78076E+00_JPRB, 0.76299E+00_JPRB, 0.74154E+00_JPRB, 0.73397E+00_JPRB, 0.73691E+00_JPRB, 0.71352E+00_JPRB, 0.68873E+00_JPRB ,&
&  0.69238E+00_JPRB, 0.72409E+00_JPRB, 0.74927E+00_JPRB, 0.76002E+00_JPRB, 0.76178E+00_JPRB, 0.76698E+00_JPRB, 0.77140E+00_JPRB ,&
&  0.77920E+00_JPRB, 0.78918E+00_JPRB, 0.79706E+00_JPRB, 0.80922E+00_JPRB, 0.82154E+00_JPRB, 0.82015E+00_JPRB, 0.80566E+00_JPRB ,&
&  0.79012E+00_JPRB, 0.78155E+00_JPRB, 0.77836E+00_JPRB, 0.78100E+00_JPRB, 0.78456E+00_JPRB, 0.78259E+00_JPRB, 0.77683E+00_JPRB ,&
&  0.76506E+00_JPRB, 0.74670E+00_JPRB, 0.73089E+00_JPRB, 0.72145E+00_JPRB, 0.72030E+00_JPRB, 0.72621E+00_JPRB, 0.74225E+00_JPRB ,&
&  0.77356E+00_JPRB, 0.82141E+00_JPRB, 0.85782E+00_JPRB, 0.87959E+00_JPRB, 0.88710E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.93058E+00_JPRB, 0.93952E+00_JPRB, 0.94971E+00_JPRB, 0.96349E+00_JPRB, 0.97637E+00_JPRB, 0.97384E+00_JPRB, 0.96151E+00_JPRB ,&
&  0.93797E+00_JPRB, 0.92877E+00_JPRB, 0.93312E+00_JPRB, 0.93528E+00_JPRB, 0.93678E+00_JPRB, 0.93430E+00_JPRB, 0.91752E+00_JPRB ,&
&  0.87853E+00_JPRB, 0.84718E+00_JPRB, 0.83180E+00_JPRB, 0.82349E+00_JPRB, 0.82037E+00_JPRB, 0.82642E+00_JPRB, 0.83349E+00_JPRB ,&
&  0.84874E+00_JPRB, 0.85743E+00_JPRB, 0.86876E+00_JPRB, 0.87791E+00_JPRB, 0.88047E+00_JPRB, 0.88400E+00_JPRB, 0.88435E+00_JPRB ,&
&  0.86920E+00_JPRB, 0.86870E+00_JPRB, 0.86897E+00_JPRB, 0.85829E+00_JPRB, 0.83518E+00_JPRB, 0.81196E+00_JPRB, 0.78401E+00_JPRB ,&
&  0.76525E+00_JPRB, 0.75603E+00_JPRB, 0.78179E+00_JPRB, 0.84458E+00_JPRB, 0.88947E+00_JPRB, 0.89784E+00_JPRB, 0.88869E+00_JPRB ,&
&  0.88929E+00_JPRB, 0.89629E+00_JPRB, 0.90008E+00_JPRB, 0.88897E+00_JPRB, 0.86959E+00_JPRB, 0.85269E+00_JPRB, 0.83702E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83374E+00_JPRB, 0.81794E+00_JPRB, 0.78650E+00_JPRB, 0.75838E+00_JPRB, 0.73820E+00_JPRB, 0.72731E+00_JPRB ,&
&  0.72025E+00_JPRB, 0.70976E+00_JPRB, 0.70266E+00_JPRB, 0.70642E+00_JPRB, 0.68583E+00_JPRB, 0.66769E+00_JPRB, 0.67138E+00_JPRB ,&
&  0.68777E+00_JPRB, 0.69377E+00_JPRB, 0.70360E+00_JPRB, 0.70752E+00_JPRB, 0.70543E+00_JPRB, 0.70262E+00_JPRB, 0.70474E+00_JPRB ,&
&  0.71454E+00_JPRB, 0.72571E+00_JPRB, 0.73412E+00_JPRB, 0.73963E+00_JPRB, 0.74504E+00_JPRB, 0.73310E+00_JPRB, 0.71193E+00_JPRB ,&
&  0.69492E+00_JPRB, 0.69237E+00_JPRB, 0.69748E+00_JPRB, 0.69731E+00_JPRB, 0.69602E+00_JPRB, 0.69643E+00_JPRB, 0.69555E+00_JPRB ,&
&  0.69325E+00_JPRB, 0.68636E+00_JPRB, 0.67780E+00_JPRB, 0.67755E+00_JPRB, 0.68842E+00_JPRB, 0.71156E+00_JPRB, 0.74746E+00_JPRB ,&
&  0.79998E+00_JPRB, 0.83944E+00_JPRB, 0.87853E+00_JPRB, 0.89594E+00_JPRB, 0.90153E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.96180E+00_JPRB, 0.98483E+00_JPRB, 0.10043E+01_JPRB, 0.10176E+01_JPRB, 0.10199E+01_JPRB, 0.10132E+01_JPRB, 0.99706E+00_JPRB ,&
&  0.96679E+00_JPRB, 0.94946E+00_JPRB, 0.95822E+00_JPRB, 0.97268E+00_JPRB, 0.98116E+00_JPRB, 0.98070E+00_JPRB, 0.94833E+00_JPRB ,&
&  0.89227E+00_JPRB, 0.87052E+00_JPRB, 0.86659E+00_JPRB, 0.87531E+00_JPRB, 0.89245E+00_JPRB, 0.89802E+00_JPRB, 0.88445E+00_JPRB ,&
&  0.87099E+00_JPRB, 0.87173E+00_JPRB, 0.87393E+00_JPRB, 0.88068E+00_JPRB, 0.87027E+00_JPRB, 0.84897E+00_JPRB, 0.83901E+00_JPRB ,&
&  0.83537E+00_JPRB, 0.82621E+00_JPRB, 0.81641E+00_JPRB, 0.76731E+00_JPRB, 0.78735E+00_JPRB, 0.77465E+00_JPRB, 0.75480E+00_JPRB ,&
&  0.73624E+00_JPRB, 0.73813E+00_JPRB, 0.78056E+00_JPRB, 0.82814E+00_JPRB, 0.84016E+00_JPRB, 0.84242E+00_JPRB, 0.83612E+00_JPRB ,&
&  0.83178E+00_JPRB, 0.83201E+00_JPRB, 0.83353E+00_JPRB, 0.82548E+00_JPRB, 0.80538E+00_JPRB, 0.77939E+00_JPRB, 0.75115E+00_JPRB ,&
&  0.73468E+00_JPRB, 0.71099E+00_JPRB, 0.66826E+00_JPRB, 0.64856E+00_JPRB, 0.64729E+00_JPRB, 0.65567E+00_JPRB, 0.65955E+00_JPRB ,&
&  0.65632E+00_JPRB, 0.64823E+00_JPRB, 0.65032E+00_JPRB, 0.64585E+00_JPRB, 0.62345E+00_JPRB, 0.60933E+00_JPRB, 0.60284E+00_JPRB ,&
&  0.59175E+00_JPRB, 0.59180E+00_JPRB, 0.59789E+00_JPRB, 0.61052E+00_JPRB, 0.62156E+00_JPRB, 0.63681E+00_JPRB, 0.64383E+00_JPRB ,&
&  0.64665E+00_JPRB, 0.64263E+00_JPRB, 0.63616E+00_JPRB, 0.62392E+00_JPRB, 0.60518E+00_JPRB, 0.59799E+00_JPRB, 0.59993E+00_JPRB ,&
&  0.60975E+00_JPRB, 0.61697E+00_JPRB, 0.61749E+00_JPRB, 0.61710E+00_JPRB, 0.60592E+00_JPRB, 0.59346E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.58046E+00_JPRB, 0.58430E+00_JPRB, 0.60123E+00_JPRB, 0.62767E+00_JPRB, 0.66086E+00_JPRB, 0.70104E+00_JPRB, 0.74658E+00_JPRB ,&
&  0.77841E+00_JPRB, 0.79669E+00_JPRB, 0.82137E+00_JPRB, 0.87050E+00_JPRB, 0.91883E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.95995E+00_JPRB, 0.98811E+00_JPRB, 0.10182E+01_JPRB, 0.10354E+01_JPRB, 0.10366E+01_JPRB, 0.10337E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.98800E+00_JPRB, 0.96295E+00_JPRB, 0.95889E+00_JPRB, 0.96274E+00_JPRB, 0.97401E+00_JPRB, 0.99196E+00_JPRB, 0.99214E+00_JPRB ,&
&  0.96368E+00_JPRB, 0.94285E+00_JPRB, 0.93208E+00_JPRB, 0.92382E+00_JPRB, 0.90559E+00_JPRB, 0.87986E+00_JPRB, 0.86289E+00_JPRB ,&
&  0.85767E+00_JPRB, 0.85689E+00_JPRB, 0.84758E+00_JPRB, 0.83365E+00_JPRB, 0.80201E+00_JPRB, 0.76281E+00_JPRB, 0.73116E+00_JPRB ,&
&  0.71901E+00_JPRB, 0.70698E+00_JPRB, 0.66691E+00_JPRB, 0.67944E+00_JPRB, 0.68332E+00_JPRB, 0.68316E+00_JPRB, 0.67919E+00_JPRB ,&
&  0.68961E+00_JPRB, 0.72287E+00_JPRB, 0.75646E+00_JPRB, 0.76616E+00_JPRB, 0.77502E+00_JPRB, 0.77629E+00_JPRB, 0.77712E+00_JPRB ,&
&  0.78125E+00_JPRB, 0.77332E+00_JPRB, 0.74384E+00_JPRB, 0.69992E+00_JPRB, 0.65668E+00_JPRB, 0.62315E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.57581E+00_JPRB, 0.56419E+00_JPRB, 0.55495E+00_JPRB, 0.55194E+00_JPRB, 0.56014E+00_JPRB, 0.57549E+00_JPRB, 0.57010E+00_JPRB ,&
&  0.55073E+00_JPRB, 0.53223E+00_JPRB, 0.50867E+00_JPRB, 0.47706E+00_JPRB, 0.44402E+00_JPRB, 0.40704E+00_JPRB, 0.38746E+00_JPRB ,&
&  0.39271E+00_JPRB, 0.43252E+00_JPRB, 0.48425E+00_JPRB, 0.50591E+00_JPRB, 0.53440E+00_JPRB, 0.56239E+00_JPRB, 0.55160E+00_JPRB ,&
&  0.50108E+00_JPRB, 0.43607E+00_JPRB, 0.41362E+00_JPRB, 0.39979E+00_JPRB, 0.39290E+00_JPRB, 0.38869E+00_JPRB, 0.38735E+00_JPRB ,&
&  0.39236E+00_JPRB, 0.40362E+00_JPRB, 0.41423E+00_JPRB, 0.41213E+00_JPRB, 0.40253E+00_JPRB, 0.39798E+00_JPRB, 0.39991E+00_JPRB ,&
&  0.41211E+00_JPRB, 0.44224E+00_JPRB, 0.50507E+00_JPRB, 0.61684E+00_JPRB, 0.65651E+00_JPRB, 0.69165E+00_JPRB, 0.73135E+00_JPRB ,&
&  0.76983E+00_JPRB, 0.80496E+00_JPRB, 0.83446E+00_JPRB, 0.87041E+00_JPRB, 0.91857E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.89445E+00_JPRB, 0.91461E+00_JPRB, 0.93736E+00_JPRB, 0.94941E+00_JPRB, 0.95098E+00_JPRB, 0.94834E+00_JPRB, 0.93987E+00_JPRB ,&
&  0.92508E+00_JPRB, 0.90717E+00_JPRB, 0.89377E+00_JPRB, 0.88431E+00_JPRB, 0.87940E+00_JPRB, 0.88119E+00_JPRB, 0.88511E+00_JPRB ,&
&  0.88272E+00_JPRB, 0.87738E+00_JPRB, 0.86409E+00_JPRB, 0.84706E+00_JPRB, 0.82913E+00_JPRB, 0.81533E+00_JPRB, 0.80489E+00_JPRB ,&
&  0.79476E+00_JPRB, 0.77988E+00_JPRB, 0.75692E+00_JPRB, 0.72133E+00_JPRB, 0.67926E+00_JPRB, 0.64088E+00_JPRB, 0.60946E+00_JPRB ,&
&  0.57985E+00_JPRB, 0.51450E+00_JPRB, 0.47848E+00_JPRB, 0.48665E+00_JPRB, 0.57230E+00_JPRB, 0.60660E+00_JPRB, 0.63204E+00_JPRB ,&
&  0.66732E+00_JPRB, 0.70524E+00_JPRB, 0.72345E+00_JPRB, 0.72340E+00_JPRB, 0.71744E+00_JPRB, 0.70445E+00_JPRB, 0.68435E+00_JPRB ,&
&  0.65900E+00_JPRB, 0.62829E+00_JPRB, 0.60201E+00_JPRB, 0.57207E+00_JPRB, 0.54169E+00_JPRB, 0.51962E+00_JPRB, 0.50440E+00_JPRB ,&
&  0.49400E+00_JPRB, 0.48402E+00_JPRB, 0.47586E+00_JPRB, 0.46574E+00_JPRB, 0.45644E+00_JPRB, 0.43499E+00_JPRB, 0.41131E+00_JPRB ,&
&  0.38967E+00_JPRB, 0.37077E+00_JPRB, 0.35206E+00_JPRB, 0.33346E+00_JPRB, 0.31830E+00_JPRB, 0.30819E+00_JPRB, 0.30048E+00_JPRB ,&
&  0.29878E+00_JPRB, 0.30683E+00_JPRB, 0.32790E+00_JPRB, 0.36066E+00_JPRB, 0.39332E+00_JPRB, 0.37212E+00_JPRB, 0.35373E+00_JPRB ,&
&  0.33115E+00_JPRB, 0.31808E+00_JPRB, 0.31321E+00_JPRB, 0.31263E+00_JPRB, 0.31384E+00_JPRB, 0.31608E+00_JPRB, 0.31870E+00_JPRB ,&
&  0.32276E+00_JPRB, 0.32675E+00_JPRB, 0.33074E+00_JPRB, 0.33396E+00_JPRB, 0.33614E+00_JPRB, 0.33942E+00_JPRB, 0.34463E+00_JPRB ,&
&  0.35322E+00_JPRB, 0.36685E+00_JPRB, 0.38769E+00_JPRB, 0.41468E+00_JPRB, 0.44346E+00_JPRB, 0.46939E+00_JPRB, 0.52272E+00_JPRB ,&
&  0.61022E+00_JPRB, 0.72049E+00_JPRB, 0.78147E+00_JPRB, 0.84110E+00_JPRB, 0.87098E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.82441E+00_JPRB, 0.83904E+00_JPRB, 0.85081E+00_JPRB, 0.85429E+00_JPRB, 0.85395E+00_JPRB, 0.84909E+00_JPRB, 0.83693E+00_JPRB ,&
&  0.81722E+00_JPRB, 0.79084E+00_JPRB, 0.76501E+00_JPRB, 0.74447E+00_JPRB, 0.73130E+00_JPRB, 0.73320E+00_JPRB, 0.74085E+00_JPRB ,&
&  0.75701E+00_JPRB, 0.78140E+00_JPRB, 0.78567E+00_JPRB, 0.77797E+00_JPRB, 0.74157E+00_JPRB, 0.70356E+00_JPRB, 0.66900E+00_JPRB ,&
&  0.63788E+00_JPRB, 0.60762E+00_JPRB, 0.59637E+00_JPRB, 0.57688E+00_JPRB, 0.56331E+00_JPRB, 0.54632E+00_JPRB, 0.52113E+00_JPRB ,&
&  0.47991E+00_JPRB, 0.44996E+00_JPRB, 0.43586E+00_JPRB, 0.44419E+00_JPRB, 0.52707E+00_JPRB, 0.58830E+00_JPRB, 0.60117E+00_JPRB ,&
&  0.63473E+00_JPRB, 0.65778E+00_JPRB, 0.65203E+00_JPRB, 0.63124E+00_JPRB, 0.60905E+00_JPRB, 0.58536E+00_JPRB, 0.56498E+00_JPRB ,&
&  0.54479E+00_JPRB, 0.51834E+00_JPRB, 0.46772E+00_JPRB, 0.39583E+00_JPRB, 0.36081E+00_JPRB, 0.34762E+00_JPRB, 0.33318E+00_JPRB ,&
&  0.32243E+00_JPRB, 0.31491E+00_JPRB, 0.31172E+00_JPRB, 0.30727E+00_JPRB, 0.30677E+00_JPRB, 0.30294E+00_JPRB, 0.29920E+00_JPRB ,&
&  0.29359E+00_JPRB, 0.28937E+00_JPRB, 0.28400E+00_JPRB, 0.27878E+00_JPRB, 0.27439E+00_JPRB, 0.27373E+00_JPRB, 0.27373E+00_JPRB ,&
&  0.27508E+00_JPRB, 0.27957E+00_JPRB, 0.29299E+00_JPRB, 0.30020E+00_JPRB, 0.30091E+00_JPRB, 0.29777E+00_JPRB, 0.29056E+00_JPRB ,&
&  0.28293E+00_JPRB, 0.28158E+00_JPRB, 0.28034E+00_JPRB, 0.28123E+00_JPRB, 0.28466E+00_JPRB, 0.28882E+00_JPRB, 0.29196E+00_JPRB ,&
&  0.29451E+00_JPRB, 0.29761E+00_JPRB, 0.30039E+00_JPRB, 0.30340E+00_JPRB, 0.30715E+00_JPRB, 0.31146E+00_JPRB, 0.31731E+00_JPRB ,&
&  0.32322E+00_JPRB, 0.33162E+00_JPRB, 0.34277E+00_JPRB, 0.35903E+00_JPRB, 0.37415E+00_JPRB, 0.38439E+00_JPRB, 0.39926E+00_JPRB ,&
&  0.41642E+00_JPRB, 0.46579E+00_JPRB, 0.60212E+00_JPRB, 0.82250E+00_JPRB, 0.85470E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.74228E+00_JPRB, 0.75594E+00_JPRB, 0.77108E+00_JPRB, 0.77543E+00_JPRB, 0.78606E+00_JPRB, 0.78486E+00_JPRB, 0.76716E+00_JPRB ,&
&  0.73914E+00_JPRB, 0.71205E+00_JPRB, 0.69743E+00_JPRB, 0.67685E+00_JPRB, 0.64793E+00_JPRB, 0.58873E+00_JPRB, 0.56624E+00_JPRB ,&
&  0.54710E+00_JPRB, 0.54625E+00_JPRB, 0.53623E+00_JPRB, 0.53575E+00_JPRB, 0.52399E+00_JPRB, 0.51213E+00_JPRB, 0.49244E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.45799E+00_JPRB, 0.44622E+00_JPRB, 0.44142E+00_JPRB, 0.43897E+00_JPRB, 0.43334E+00_JPRB, 0.41941E+00_JPRB ,&
&  0.40974E+00_JPRB, 0.40755E+00_JPRB, 0.40943E+00_JPRB, 0.41548E+00_JPRB, 0.45556E+00_JPRB, 0.51314E+00_JPRB, 0.51494E+00_JPRB ,&
&  0.53624E+00_JPRB, 0.54837E+00_JPRB, 0.56044E+00_JPRB, 0.54835E+00_JPRB, 0.51453E+00_JPRB, 0.47279E+00_JPRB, 0.42008E+00_JPRB ,&
&  0.37000E+00_JPRB, 0.34026E+00_JPRB, 0.31598E+00_JPRB, 0.29837E+00_JPRB, 0.28574E+00_JPRB, 0.28095E+00_JPRB, 0.27471E+00_JPRB ,&
&  0.27081E+00_JPRB, 0.26778E+00_JPRB, 0.26663E+00_JPRB, 0.26627E+00_JPRB, 0.26543E+00_JPRB, 0.26304E+00_JPRB, 0.26237E+00_JPRB ,&
&  0.26309E+00_JPRB, 0.26311E+00_JPRB, 0.26264E+00_JPRB, 0.26358E+00_JPRB, 0.26461E+00_JPRB, 0.26628E+00_JPRB, 0.26796E+00_JPRB ,&
&  0.27057E+00_JPRB, 0.27205E+00_JPRB, 0.27429E+00_JPRB, 0.27686E+00_JPRB, 0.27582E+00_JPRB, 0.27486E+00_JPRB, 0.27218E+00_JPRB ,&
&  0.26892E+00_JPRB, 0.26907E+00_JPRB, 0.26885E+00_JPRB, 0.26978E+00_JPRB, 0.27197E+00_JPRB, 0.27469E+00_JPRB, 0.27813E+00_JPRB ,&
&  0.28136E+00_JPRB, 0.28470E+00_JPRB, 0.28730E+00_JPRB, 0.28988E+00_JPRB, 0.29421E+00_JPRB, 0.29864E+00_JPRB, 0.30428E+00_JPRB ,&
&  0.31012E+00_JPRB, 0.31680E+00_JPRB, 0.32580E+00_JPRB, 0.33744E+00_JPRB, 0.35483E+00_JPRB, 0.37359E+00_JPRB, 0.40304E+00_JPRB ,&
&  0.42703E+00_JPRB, 0.52261E+00_JPRB, 0.75614E+00_JPRB, 0.69002E+00_JPRB, 0.71267E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.70335E+00_JPRB, 0.72160E+00_JPRB, 0.74605E+00_JPRB, 0.73943E+00_JPRB, 0.75926E+00_JPRB, 0.74600E+00_JPRB, 0.72230E+00_JPRB ,&
&  0.67505E+00_JPRB, 0.61976E+00_JPRB, 0.57274E+00_JPRB, 0.52085E+00_JPRB, 0.47938E+00_JPRB, 0.44977E+00_JPRB, 0.43478E+00_JPRB ,&
&  0.42639E+00_JPRB, 0.42348E+00_JPRB, 0.42055E+00_JPRB, 0.41892E+00_JPRB, 0.41264E+00_JPRB, 0.40839E+00_JPRB, 0.40181E+00_JPRB ,&
&  0.39586E+00_JPRB, 0.39269E+00_JPRB, 0.38958E+00_JPRB, 0.39456E+00_JPRB, 0.40035E+00_JPRB, 0.40605E+00_JPRB, 0.40834E+00_JPRB ,&
&  0.40978E+00_JPRB, 0.41126E+00_JPRB, 0.41439E+00_JPRB, 0.41205E+00_JPRB, 0.42366E+00_JPRB, 0.44434E+00_JPRB, 0.46074E+00_JPRB ,&
&  0.47674E+00_JPRB, 0.48962E+00_JPRB, 0.48460E+00_JPRB, 0.45972E+00_JPRB, 0.39287E+00_JPRB, 0.35876E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.30736E+00_JPRB, 0.29226E+00_JPRB, 0.28179E+00_JPRB, 0.27595E+00_JPRB, 0.27080E+00_JPRB, 0.26836E+00_JPRB, 0.26575E+00_JPRB ,&
&  0.26333E+00_JPRB, 0.26213E+00_JPRB, 0.26045E+00_JPRB, 0.25954E+00_JPRB, 0.25831E+00_JPRB, 0.25666E+00_JPRB, 0.25568E+00_JPRB ,&
&  0.25613E+00_JPRB, 0.25609E+00_JPRB, 0.25753E+00_JPRB, 0.25864E+00_JPRB, 0.26073E+00_JPRB, 0.26239E+00_JPRB, 0.26488E+00_JPRB ,&
&  0.26718E+00_JPRB, 0.26919E+00_JPRB, 0.26915E+00_JPRB, 0.26895E+00_JPRB, 0.26815E+00_JPRB, 0.26786E+00_JPRB, 0.26720E+00_JPRB ,&
&  0.26612E+00_JPRB, 0.26659E+00_JPRB, 0.26730E+00_JPRB, 0.26800E+00_JPRB, 0.26966E+00_JPRB, 0.27177E+00_JPRB, 0.27468E+00_JPRB ,&
&  0.27851E+00_JPRB, 0.28242E+00_JPRB, 0.28581E+00_JPRB, 0.28954E+00_JPRB, 0.29411E+00_JPRB, 0.29910E+00_JPRB, 0.30515E+00_JPRB ,&
&  0.30993E+00_JPRB, 0.31898E+00_JPRB, 0.33189E+00_JPRB, 0.34562E+00_JPRB, 0.36898E+00_JPRB, 0.38844E+00_JPRB, 0.42109E+00_JPRB ,&
&  0.46708E+00_JPRB, 0.62969E+00_JPRB, 0.91053E+00_JPRB, 0.69218E+00_JPRB, 0.68609E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.62011E+00_JPRB, 0.63025E+00_JPRB, 0.64449E+00_JPRB, 0.65618E+00_JPRB, 0.67971E+00_JPRB, 0.69301E+00_JPRB, 0.73696E+00_JPRB ,&
&  0.69153E+00_JPRB, 0.61367E+00_JPRB, 0.58150E+00_JPRB, 0.54584E+00_JPRB, 0.50584E+00_JPRB, 0.47666E+00_JPRB, 0.45621E+00_JPRB ,&
&  0.44355E+00_JPRB, 0.43264E+00_JPRB, 0.42190E+00_JPRB, 0.40884E+00_JPRB, 0.39560E+00_JPRB, 0.38137E+00_JPRB, 0.37115E+00_JPRB ,&
&  0.36529E+00_JPRB, 0.36437E+00_JPRB, 0.36381E+00_JPRB, 0.36750E+00_JPRB, 0.37242E+00_JPRB, 0.37864E+00_JPRB, 0.38633E+00_JPRB ,&
&  0.40539E+00_JPRB, 0.41417E+00_JPRB, 0.41729E+00_JPRB, 0.41672E+00_JPRB, 0.41719E+00_JPRB, 0.42026E+00_JPRB, 0.41094E+00_JPRB ,&
&  0.41508E+00_JPRB, 0.43239E+00_JPRB, 0.43748E+00_JPRB, 0.41604E+00_JPRB, 0.36126E+00_JPRB, 0.32936E+00_JPRB, 0.30624E+00_JPRB ,&
&  0.29849E+00_JPRB, 0.28771E+00_JPRB, 0.28100E+00_JPRB, 0.27431E+00_JPRB, 0.26941E+00_JPRB, 0.26748E+00_JPRB, 0.26543E+00_JPRB ,&
&  0.26421E+00_JPRB, 0.26372E+00_JPRB, 0.26169E+00_JPRB, 0.26090E+00_JPRB, 0.25949E+00_JPRB, 0.25863E+00_JPRB, 0.25825E+00_JPRB ,&
&  0.25804E+00_JPRB, 0.25772E+00_JPRB, 0.25800E+00_JPRB, 0.25857E+00_JPRB, 0.25953E+00_JPRB, 0.26030E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26213E+00_JPRB, 0.26319E+00_JPRB, 0.26357E+00_JPRB, 0.26436E+00_JPRB, 0.26501E+00_JPRB, 0.26656E+00_JPRB, 0.26736E+00_JPRB ,&
&  0.26816E+00_JPRB, 0.26891E+00_JPRB, 0.26909E+00_JPRB, 0.27004E+00_JPRB, 0.27126E+00_JPRB, 0.27355E+00_JPRB, 0.27607E+00_JPRB ,&
&  0.27927E+00_JPRB, 0.28313E+00_JPRB, 0.28724E+00_JPRB, 0.29137E+00_JPRB, 0.29650E+00_JPRB, 0.30169E+00_JPRB, 0.30877E+00_JPRB ,&
&  0.31634E+00_JPRB, 0.32676E+00_JPRB, 0.34150E+00_JPRB, 0.35880E+00_JPRB, 0.38038E+00_JPRB, 0.39902E+00_JPRB, 0.41782E+00_JPRB ,&
&  0.45035E+00_JPRB, 0.54504E+00_JPRB, 0.60492E+00_JPRB, 0.59865E+00_JPRB, 0.61217E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.59629E+00_JPRB, 0.59964E+00_JPRB, 0.60314E+00_JPRB, 0.60805E+00_JPRB, 0.61341E+00_JPRB, 0.61825E+00_JPRB, 0.62201E+00_JPRB ,&
&  0.60331E+00_JPRB, 0.58017E+00_JPRB, 0.56098E+00_JPRB, 0.53800E+00_JPRB, 0.51700E+00_JPRB, 0.49868E+00_JPRB, 0.47833E+00_JPRB ,&
&  0.45782E+00_JPRB, 0.43747E+00_JPRB, 0.41466E+00_JPRB, 0.39586E+00_JPRB, 0.37744E+00_JPRB, 0.36256E+00_JPRB, 0.35408E+00_JPRB ,&
&  0.34904E+00_JPRB, 0.34746E+00_JPRB, 0.34857E+00_JPRB, 0.35041E+00_JPRB, 0.35243E+00_JPRB, 0.35662E+00_JPRB, 0.36282E+00_JPRB ,&
&  0.37144E+00_JPRB, 0.38741E+00_JPRB, 0.40152E+00_JPRB, 0.40623E+00_JPRB, 0.40181E+00_JPRB, 0.39254E+00_JPRB, 0.38440E+00_JPRB ,&
&  0.37483E+00_JPRB, 0.36288E+00_JPRB, 0.34759E+00_JPRB, 0.33232E+00_JPRB, 0.31810E+00_JPRB, 0.30775E+00_JPRB, 0.29869E+00_JPRB ,&
&  0.29085E+00_JPRB, 0.28383E+00_JPRB, 0.27830E+00_JPRB, 0.27464E+00_JPRB, 0.27149E+00_JPRB, 0.26907E+00_JPRB, 0.26663E+00_JPRB ,&
&  0.26558E+00_JPRB, 0.26451E+00_JPRB, 0.26387E+00_JPRB, 0.26351E+00_JPRB, 0.26289E+00_JPRB, 0.26263E+00_JPRB, 0.26245E+00_JPRB ,&
&  0.26203E+00_JPRB, 0.26132E+00_JPRB, 0.26094E+00_JPRB, 0.26070E+00_JPRB, 0.26126E+00_JPRB, 0.26177E+00_JPRB, 0.26253E+00_JPRB ,&
&  0.26361E+00_JPRB, 0.26572E+00_JPRB, 0.26698E+00_JPRB, 0.26820E+00_JPRB, 0.26884E+00_JPRB, 0.26947E+00_JPRB, 0.27029E+00_JPRB ,&
&  0.27160E+00_JPRB, 0.27311E+00_JPRB, 0.27441E+00_JPRB, 0.27616E+00_JPRB, 0.27795E+00_JPRB, 0.28041E+00_JPRB, 0.28312E+00_JPRB ,&
&  0.28607E+00_JPRB, 0.28974E+00_JPRB, 0.29417E+00_JPRB, 0.29807E+00_JPRB, 0.30440E+00_JPRB, 0.31081E+00_JPRB, 0.31809E+00_JPRB ,&
&  0.32580E+00_JPRB, 0.33418E+00_JPRB, 0.34547E+00_JPRB, 0.35894E+00_JPRB, 0.37623E+00_JPRB, 0.38372E+00_JPRB, 0.39347E+00_JPRB ,&
&  0.40870E+00_JPRB, 0.44037E+00_JPRB, 0.57729E+00_JPRB, 0.59085E+00_JPRB, 0.59620E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.37539E+00_JPRB, 0.38114E+00_JPRB, 0.38489E+00_JPRB, 0.39372E+00_JPRB, 0.40373E+00_JPRB, 0.50328E+00_JPRB, 0.49839E+00_JPRB ,&
&  0.49791E+00_JPRB, 0.45462E+00_JPRB, 0.44709E+00_JPRB, 0.43512E+00_JPRB, 0.41986E+00_JPRB, 0.40105E+00_JPRB, 0.38946E+00_JPRB ,&
&  0.37985E+00_JPRB, 0.36846E+00_JPRB, 0.35796E+00_JPRB, 0.35166E+00_JPRB, 0.34666E+00_JPRB, 0.34209E+00_JPRB, 0.33920E+00_JPRB ,&
&  0.33764E+00_JPRB, 0.33751E+00_JPRB, 0.33769E+00_JPRB, 0.33761E+00_JPRB, 0.34057E+00_JPRB, 0.34398E+00_JPRB, 0.34748E+00_JPRB ,&
&  0.34871E+00_JPRB, 0.35343E+00_JPRB, 0.35466E+00_JPRB, 0.35169E+00_JPRB, 0.34703E+00_JPRB, 0.33954E+00_JPRB, 0.33201E+00_JPRB ,&
&  0.32428E+00_JPRB, 0.31729E+00_JPRB, 0.31140E+00_JPRB, 0.30610E+00_JPRB, 0.29995E+00_JPRB, 0.29387E+00_JPRB, 0.28981E+00_JPRB ,&
&  0.28600E+00_JPRB, 0.28206E+00_JPRB, 0.27887E+00_JPRB, 0.27730E+00_JPRB, 0.27554E+00_JPRB, 0.27434E+00_JPRB, 0.27339E+00_JPRB ,&
&  0.27311E+00_JPRB, 0.27294E+00_JPRB, 0.27298E+00_JPRB, 0.27322E+00_JPRB, 0.27317E+00_JPRB, 0.27306E+00_JPRB, 0.27300E+00_JPRB ,&
&  0.27363E+00_JPRB, 0.27370E+00_JPRB, 0.27367E+00_JPRB, 0.27351E+00_JPRB, 0.27438E+00_JPRB, 0.27475E+00_JPRB, 0.27528E+00_JPRB ,&
&  0.27599E+00_JPRB, 0.27692E+00_JPRB, 0.27761E+00_JPRB, 0.27837E+00_JPRB, 0.27911E+00_JPRB, 0.28005E+00_JPRB, 0.28115E+00_JPRB ,&
&  0.28228E+00_JPRB, 0.28394E+00_JPRB, 0.28566E+00_JPRB, 0.28715E+00_JPRB, 0.28850E+00_JPRB, 0.29031E+00_JPRB, 0.29237E+00_JPRB ,&
&  0.29533E+00_JPRB, 0.29818E+00_JPRB, 0.30108E+00_JPRB, 0.30261E+00_JPRB, 0.30530E+00_JPRB, 0.30761E+00_JPRB, 0.31019E+00_JPRB ,&
&  0.31336E+00_JPRB, 0.31866E+00_JPRB, 0.32659E+00_JPRB, 0.33545E+00_JPRB, 0.34562E+00_JPRB, 0.35299E+00_JPRB, 0.35766E+00_JPRB ,&
&  0.36146E+00_JPRB, 0.36564E+00_JPRB, 0.37030E+00_JPRB, 0.37268E+00_JPRB, 0.37446E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.34120E+00_JPRB, 0.34109E+00_JPRB, 0.34115E+00_JPRB, 0.34076E+00_JPRB, 0.33902E+00_JPRB, 0.33757E+00_JPRB, 0.33622E+00_JPRB ,&
&  0.33536E+00_JPRB, 0.33428E+00_JPRB, 0.33373E+00_JPRB, 0.33340E+00_JPRB, 0.33283E+00_JPRB, 0.33208E+00_JPRB, 0.33110E+00_JPRB ,&
&  0.32981E+00_JPRB, 0.32816E+00_JPRB, 0.32626E+00_JPRB, 0.32566E+00_JPRB, 0.32542E+00_JPRB, 0.32518E+00_JPRB, 0.32486E+00_JPRB ,&
&  0.32439E+00_JPRB, 0.32400E+00_JPRB, 0.32324E+00_JPRB, 0.32335E+00_JPRB, 0.32328E+00_JPRB, 0.32323E+00_JPRB, 0.32327E+00_JPRB ,&
&  0.32340E+00_JPRB, 0.32368E+00_JPRB, 0.32355E+00_JPRB, 0.32314E+00_JPRB, 0.32233E+00_JPRB, 0.32063E+00_JPRB, 0.31791E+00_JPRB ,&
&  0.31483E+00_JPRB, 0.31175E+00_JPRB, 0.30927E+00_JPRB, 0.30700E+00_JPRB, 0.30492E+00_JPRB, 0.30303E+00_JPRB, 0.30146E+00_JPRB ,&
&  0.29955E+00_JPRB, 0.29766E+00_JPRB, 0.29589E+00_JPRB, 0.29457E+00_JPRB, 0.29311E+00_JPRB, 0.29181E+00_JPRB, 0.29055E+00_JPRB ,&
&  0.28967E+00_JPRB, 0.28890E+00_JPRB, 0.28824E+00_JPRB, 0.28788E+00_JPRB, 0.28761E+00_JPRB, 0.28738E+00_JPRB, 0.28711E+00_JPRB ,&
&  0.28687E+00_JPRB, 0.28674E+00_JPRB, 0.28667E+00_JPRB, 0.28668E+00_JPRB, 0.28683E+00_JPRB, 0.28678E+00_JPRB, 0.28670E+00_JPRB ,&
&  0.28670E+00_JPRB, 0.28708E+00_JPRB, 0.28730E+00_JPRB, 0.28763E+00_JPRB, 0.28812E+00_JPRB, 0.28880E+00_JPRB, 0.28955E+00_JPRB ,&
&  0.29030E+00_JPRB, 0.29117E+00_JPRB, 0.29210E+00_JPRB, 0.29327E+00_JPRB, 0.29454E+00_JPRB, 0.29579E+00_JPRB, 0.29701E+00_JPRB ,&
&  0.29831E+00_JPRB, 0.29952E+00_JPRB, 0.30066E+00_JPRB, 0.30144E+00_JPRB, 0.30264E+00_JPRB, 0.30391E+00_JPRB, 0.30539E+00_JPRB ,&
&  0.30711E+00_JPRB, 0.30940E+00_JPRB, 0.31270E+00_JPRB, 0.31664E+00_JPRB, 0.32174E+00_JPRB, 0.32538E+00_JPRB, 0.32894E+00_JPRB ,&
&  0.33141E+00_JPRB, 0.33339E+00_JPRB, 0.33493E+00_JPRB, 0.33682E+00_JPRB, 0.33892E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB, 0.30569E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2020:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2020
