!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2030 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2030* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB, 0.55806E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.57962E+00_JPRB, 0.57879E+00_JPRB, 0.57802E+00_JPRB, 0.57733E+00_JPRB, 0.57667E+00_JPRB, 0.57616E+00_JPRB, 0.57574E+00_JPRB ,&
&  0.57515E+00_JPRB, 0.57458E+00_JPRB, 0.57384E+00_JPRB, 0.57316E+00_JPRB, 0.57250E+00_JPRB, 0.57176E+00_JPRB, 0.57084E+00_JPRB ,&
&  0.56976E+00_JPRB, 0.56878E+00_JPRB, 0.56772E+00_JPRB, 0.56648E+00_JPRB, 0.56524E+00_JPRB, 0.56386E+00_JPRB, 0.56224E+00_JPRB ,&
&  0.56039E+00_JPRB, 0.55856E+00_JPRB, 0.55663E+00_JPRB, 0.55460E+00_JPRB, 0.55236E+00_JPRB, 0.54983E+00_JPRB, 0.54753E+00_JPRB ,&
&  0.54582E+00_JPRB, 0.54398E+00_JPRB, 0.54222E+00_JPRB, 0.54043E+00_JPRB, 0.53896E+00_JPRB, 0.53781E+00_JPRB, 0.53660E+00_JPRB ,&
&  0.53555E+00_JPRB, 0.53464E+00_JPRB, 0.53448E+00_JPRB, 0.53460E+00_JPRB, 0.53488E+00_JPRB, 0.53585E+00_JPRB, 0.53746E+00_JPRB ,&
&  0.53881E+00_JPRB, 0.53994E+00_JPRB, 0.54136E+00_JPRB, 0.54299E+00_JPRB, 0.54469E+00_JPRB, 0.54640E+00_JPRB, 0.54801E+00_JPRB ,&
&  0.54926E+00_JPRB, 0.55026E+00_JPRB, 0.55134E+00_JPRB, 0.55227E+00_JPRB, 0.55281E+00_JPRB, 0.55329E+00_JPRB, 0.55383E+00_JPRB ,&
&  0.55426E+00_JPRB, 0.55460E+00_JPRB, 0.55504E+00_JPRB, 0.55580E+00_JPRB, 0.55650E+00_JPRB, 0.55730E+00_JPRB, 0.55835E+00_JPRB ,&
&  0.55976E+00_JPRB, 0.56085E+00_JPRB, 0.56218E+00_JPRB, 0.56382E+00_JPRB, 0.56589E+00_JPRB, 0.56832E+00_JPRB, 0.57033E+00_JPRB ,&
&  0.57243E+00_JPRB, 0.57465E+00_JPRB, 0.57704E+00_JPRB, 0.57884E+00_JPRB, 0.58077E+00_JPRB, 0.58260E+00_JPRB, 0.58436E+00_JPRB ,&
&  0.58506E+00_JPRB, 0.58603E+00_JPRB, 0.58722E+00_JPRB, 0.58840E+00_JPRB, 0.58813E+00_JPRB, 0.58799E+00_JPRB, 0.58790E+00_JPRB ,&
&  0.58857E+00_JPRB, 0.58754E+00_JPRB, 0.58711E+00_JPRB, 0.58647E+00_JPRB, 0.58620E+00_JPRB, 0.58509E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.58346E+00_JPRB, 0.58281E+00_JPRB, 0.58196E+00_JPRB, 0.58111E+00_JPRB, 0.58028E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.60105E+00_JPRB, 0.59952E+00_JPRB, 0.59787E+00_JPRB, 0.59614E+00_JPRB, 0.59415E+00_JPRB, 0.59259E+00_JPRB, 0.59107E+00_JPRB ,&
&  0.58949E+00_JPRB, 0.58807E+00_JPRB, 0.58657E+00_JPRB, 0.58530E+00_JPRB, 0.58413E+00_JPRB, 0.58351E+00_JPRB, 0.58247E+00_JPRB ,&
&  0.58123E+00_JPRB, 0.57954E+00_JPRB, 0.57802E+00_JPRB, 0.57561E+00_JPRB, 0.57290E+00_JPRB, 0.56931E+00_JPRB, 0.56598E+00_JPRB ,&
&  0.56222E+00_JPRB, 0.55779E+00_JPRB, 0.55386E+00_JPRB, 0.54987E+00_JPRB, 0.54604E+00_JPRB, 0.54195E+00_JPRB, 0.53900E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.53436E+00_JPRB, 0.53195E+00_JPRB, 0.52995E+00_JPRB, 0.52789E+00_JPRB, 0.52617E+00_JPRB, 0.52449E+00_JPRB ,&
&  0.52154E+00_JPRB, 0.51833E+00_JPRB, 0.51811E+00_JPRB, 0.51833E+00_JPRB, 0.51940E+00_JPRB, 0.52204E+00_JPRB, 0.52690E+00_JPRB ,&
&  0.53338E+00_JPRB, 0.53950E+00_JPRB, 0.54622E+00_JPRB, 0.55214E+00_JPRB, 0.55678E+00_JPRB, 0.56041E+00_JPRB, 0.56333E+00_JPRB ,&
&  0.56523E+00_JPRB, 0.56661E+00_JPRB, 0.56726E+00_JPRB, 0.56708E+00_JPRB, 0.56610E+00_JPRB, 0.56568E+00_JPRB, 0.56569E+00_JPRB ,&
&  0.56600E+00_JPRB, 0.56441E+00_JPRB, 0.56401E+00_JPRB, 0.56462E+00_JPRB, 0.56610E+00_JPRB, 0.56764E+00_JPRB, 0.57046E+00_JPRB ,&
&  0.57431E+00_JPRB, 0.57897E+00_JPRB, 0.58344E+00_JPRB, 0.58926E+00_JPRB, 0.59598E+00_JPRB, 0.60359E+00_JPRB, 0.60840E+00_JPRB ,&
&  0.61420E+00_JPRB, 0.62038E+00_JPRB, 0.62682E+00_JPRB, 0.62870E+00_JPRB, 0.63087E+00_JPRB, 0.63273E+00_JPRB, 0.63495E+00_JPRB ,&
&  0.63316E+00_JPRB, 0.63299E+00_JPRB, 0.63324E+00_JPRB, 0.63501E+00_JPRB, 0.63282E+00_JPRB, 0.63210E+00_JPRB, 0.63127E+00_JPRB ,&
&  0.63251E+00_JPRB, 0.62826E+00_JPRB, 0.62594E+00_JPRB, 0.62284E+00_JPRB, 0.62103E+00_JPRB, 0.61657E+00_JPRB, 0.61312E+00_JPRB ,&
&  0.61025E+00_JPRB, 0.60854E+00_JPRB, 0.60625E+00_JPRB, 0.60432E+00_JPRB, 0.60278E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.62563E+00_JPRB, 0.62205E+00_JPRB, 0.61668E+00_JPRB, 0.60986E+00_JPRB, 0.60338E+00_JPRB, 0.59843E+00_JPRB, 0.59311E+00_JPRB ,&
&  0.58873E+00_JPRB, 0.58471E+00_JPRB, 0.58179E+00_JPRB, 0.57954E+00_JPRB, 0.57736E+00_JPRB, 0.57768E+00_JPRB, 0.57798E+00_JPRB ,&
&  0.57828E+00_JPRB, 0.57763E+00_JPRB, 0.57890E+00_JPRB, 0.57783E+00_JPRB, 0.57579E+00_JPRB, 0.57157E+00_JPRB, 0.56568E+00_JPRB ,&
&  0.55927E+00_JPRB, 0.55205E+00_JPRB, 0.54397E+00_JPRB, 0.53483E+00_JPRB, 0.52618E+00_JPRB, 0.49604E+00_JPRB, 0.48899E+00_JPRB ,&
&  0.49178E+00_JPRB, 0.48710E+00_JPRB, 0.48312E+00_JPRB, 0.48068E+00_JPRB, 0.47508E+00_JPRB, 0.47056E+00_JPRB, 0.46660E+00_JPRB ,&
&  0.44329E+00_JPRB, 0.44300E+00_JPRB, 0.44084E+00_JPRB, 0.43949E+00_JPRB, 0.44319E+00_JPRB, 0.45522E+00_JPRB, 0.47458E+00_JPRB ,&
&  0.50986E+00_JPRB, 0.52463E+00_JPRB, 0.53738E+00_JPRB, 0.55216E+00_JPRB, 0.56381E+00_JPRB, 0.57252E+00_JPRB, 0.58122E+00_JPRB ,&
&  0.58838E+00_JPRB, 0.59530E+00_JPRB, 0.60026E+00_JPRB, 0.60258E+00_JPRB, 0.60263E+00_JPRB, 0.60231E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.60098E+00_JPRB, 0.60023E+00_JPRB, 0.60034E+00_JPRB, 0.60042E+00_JPRB, 0.60015E+00_JPRB, 0.60118E+00_JPRB, 0.60558E+00_JPRB ,&
&  0.61210E+00_JPRB, 0.62166E+00_JPRB, 0.63016E+00_JPRB, 0.64209E+00_JPRB, 0.65325E+00_JPRB, 0.66722E+00_JPRB, 0.67320E+00_JPRB ,&
&  0.68040E+00_JPRB, 0.68523E+00_JPRB, 0.69159E+00_JPRB, 0.69212E+00_JPRB, 0.69257E+00_JPRB, 0.69205E+00_JPRB, 0.69292E+00_JPRB ,&
&  0.69000E+00_JPRB, 0.68820E+00_JPRB, 0.68569E+00_JPRB, 0.68605E+00_JPRB, 0.68263E+00_JPRB, 0.68262E+00_JPRB, 0.68093E+00_JPRB ,&
&  0.68324E+00_JPRB, 0.67874E+00_JPRB, 0.67728E+00_JPRB, 0.67148E+00_JPRB, 0.66684E+00_JPRB, 0.65811E+00_JPRB, 0.65054E+00_JPRB ,&
&  0.64409E+00_JPRB, 0.63922E+00_JPRB, 0.63512E+00_JPRB, 0.63166E+00_JPRB, 0.62844E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.64496E+00_JPRB, 0.63471E+00_JPRB, 0.62331E+00_JPRB, 0.61362E+00_JPRB, 0.60454E+00_JPRB, 0.59775E+00_JPRB, 0.59118E+00_JPRB ,&
&  0.58609E+00_JPRB, 0.58114E+00_JPRB, 0.57735E+00_JPRB, 0.57379E+00_JPRB, 0.57097E+00_JPRB, 0.56904E+00_JPRB, 0.56719E+00_JPRB ,&
&  0.56635E+00_JPRB, 0.56523E+00_JPRB, 0.56543E+00_JPRB, 0.56494E+00_JPRB, 0.56501E+00_JPRB, 0.56216E+00_JPRB, 0.55607E+00_JPRB ,&
&  0.54897E+00_JPRB, 0.54136E+00_JPRB, 0.51651E+00_JPRB, 0.50822E+00_JPRB, 0.48757E+00_JPRB, 0.47257E+00_JPRB, 0.46389E+00_JPRB ,&
&  0.44787E+00_JPRB, 0.43911E+00_JPRB, 0.42260E+00_JPRB, 0.41335E+00_JPRB, 0.41195E+00_JPRB, 0.40180E+00_JPRB, 0.39796E+00_JPRB ,&
&  0.39071E+00_JPRB, 0.37726E+00_JPRB, 0.37213E+00_JPRB, 0.36979E+00_JPRB, 0.37378E+00_JPRB, 0.38462E+00_JPRB, 0.40458E+00_JPRB ,&
&  0.44224E+00_JPRB, 0.48406E+00_JPRB, 0.54103E+00_JPRB, 0.55733E+00_JPRB, 0.56076E+00_JPRB, 0.57417E+00_JPRB, 0.58507E+00_JPRB ,&
&  0.59171E+00_JPRB, 0.60638E+00_JPRB, 0.62188E+00_JPRB, 0.62825E+00_JPRB, 0.63238E+00_JPRB, 0.63379E+00_JPRB, 0.63498E+00_JPRB ,&
&  0.63614E+00_JPRB, 0.64200E+00_JPRB, 0.64540E+00_JPRB, 0.64895E+00_JPRB, 0.65258E+00_JPRB, 0.65689E+00_JPRB, 0.66601E+00_JPRB ,&
&  0.67402E+00_JPRB, 0.69106E+00_JPRB, 0.70540E+00_JPRB, 0.72772E+00_JPRB, 0.73755E+00_JPRB, 0.74925E+00_JPRB, 0.75278E+00_JPRB ,&
&  0.75840E+00_JPRB, 0.76178E+00_JPRB, 0.76446E+00_JPRB, 0.75856E+00_JPRB, 0.75758E+00_JPRB, 0.76253E+00_JPRB, 0.76145E+00_JPRB ,&
&  0.75951E+00_JPRB, 0.75868E+00_JPRB, 0.75702E+00_JPRB, 0.75607E+00_JPRB, 0.75568E+00_JPRB, 0.75807E+00_JPRB, 0.75843E+00_JPRB ,&
&  0.76333E+00_JPRB, 0.76101E+00_JPRB, 0.75889E+00_JPRB, 0.75208E+00_JPRB, 0.74295E+00_JPRB, 0.72957E+00_JPRB, 0.71225E+00_JPRB ,&
&  0.69730E+00_JPRB, 0.68562E+00_JPRB, 0.67338E+00_JPRB, 0.66412E+00_JPRB, 0.65431E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.61967E+00_JPRB, 0.60644E+00_JPRB, 0.59523E+00_JPRB, 0.58719E+00_JPRB, 0.58025E+00_JPRB, 0.57571E+00_JPRB, 0.57141E+00_JPRB ,&
&  0.56905E+00_JPRB, 0.56626E+00_JPRB, 0.56495E+00_JPRB, 0.56331E+00_JPRB, 0.56280E+00_JPRB, 0.56269E+00_JPRB, 0.56325E+00_JPRB ,&
&  0.56410E+00_JPRB, 0.56400E+00_JPRB, 0.56427E+00_JPRB, 0.56161E+00_JPRB, 0.55937E+00_JPRB, 0.55576E+00_JPRB, 0.55211E+00_JPRB ,&
&  0.54421E+00_JPRB, 0.53471E+00_JPRB, 0.51667E+00_JPRB, 0.50727E+00_JPRB, 0.48859E+00_JPRB, 0.47548E+00_JPRB, 0.46226E+00_JPRB ,&
&  0.45047E+00_JPRB, 0.42686E+00_JPRB, 0.43490E+00_JPRB, 0.41965E+00_JPRB, 0.40518E+00_JPRB, 0.38988E+00_JPRB, 0.37141E+00_JPRB ,&
&  0.36420E+00_JPRB, 0.34625E+00_JPRB, 0.33955E+00_JPRB, 0.33964E+00_JPRB, 0.34211E+00_JPRB, 0.35383E+00_JPRB, 0.37227E+00_JPRB ,&
&  0.40842E+00_JPRB, 0.50040E+00_JPRB, 0.53331E+00_JPRB, 0.54823E+00_JPRB, 0.54555E+00_JPRB, 0.54755E+00_JPRB, 0.54302E+00_JPRB ,&
&  0.55908E+00_JPRB, 0.58052E+00_JPRB, 0.60264E+00_JPRB, 0.61772E+00_JPRB, 0.63602E+00_JPRB, 0.64598E+00_JPRB, 0.65483E+00_JPRB ,&
&  0.66539E+00_JPRB, 0.68183E+00_JPRB, 0.69655E+00_JPRB, 0.71115E+00_JPRB, 0.72898E+00_JPRB, 0.74676E+00_JPRB, 0.76762E+00_JPRB ,&
&  0.78063E+00_JPRB, 0.80798E+00_JPRB, 0.81808E+00_JPRB, 0.83186E+00_JPRB, 0.83461E+00_JPRB, 0.83371E+00_JPRB, 0.83519E+00_JPRB ,&
&  0.83919E+00_JPRB, 0.84415E+00_JPRB, 0.84820E+00_JPRB, 0.84713E+00_JPRB, 0.84688E+00_JPRB, 0.83668E+00_JPRB, 0.83338E+00_JPRB ,&
&  0.83435E+00_JPRB, 0.83385E+00_JPRB, 0.83206E+00_JPRB, 0.82739E+00_JPRB, 0.82065E+00_JPRB, 0.81350E+00_JPRB, 0.81001E+00_JPRB ,&
&  0.80525E+00_JPRB, 0.79628E+00_JPRB, 0.78642E+00_JPRB, 0.77527E+00_JPRB, 0.76361E+00_JPRB, 0.75136E+00_JPRB, 0.73117E+00_JPRB ,&
&  0.70654E+00_JPRB, 0.68540E+00_JPRB, 0.66546E+00_JPRB, 0.65024E+00_JPRB, 0.63408E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.60017E+00_JPRB, 0.58755E+00_JPRB, 0.57677E+00_JPRB, 0.57072E+00_JPRB, 0.56486E+00_JPRB, 0.56234E+00_JPRB, 0.55885E+00_JPRB ,&
&  0.55703E+00_JPRB, 0.55534E+00_JPRB, 0.55475E+00_JPRB, 0.55428E+00_JPRB, 0.55458E+00_JPRB, 0.55511E+00_JPRB, 0.55646E+00_JPRB ,&
&  0.55790E+00_JPRB, 0.56012E+00_JPRB, 0.56227E+00_JPRB, 0.56132E+00_JPRB, 0.56015E+00_JPRB, 0.55768E+00_JPRB, 0.55459E+00_JPRB ,&
&  0.54789E+00_JPRB, 0.53888E+00_JPRB, 0.52140E+00_JPRB, 0.50966E+00_JPRB, 0.49907E+00_JPRB, 0.48352E+00_JPRB, 0.48044E+00_JPRB ,&
&  0.47616E+00_JPRB, 0.46375E+00_JPRB, 0.44608E+00_JPRB, 0.43799E+00_JPRB, 0.42577E+00_JPRB, 0.38607E+00_JPRB, 0.38671E+00_JPRB ,&
&  0.37568E+00_JPRB, 0.34961E+00_JPRB, 0.34188E+00_JPRB, 0.33647E+00_JPRB, 0.33814E+00_JPRB, 0.34907E+00_JPRB, 0.36569E+00_JPRB ,&
&  0.39997E+00_JPRB, 0.50642E+00_JPRB, 0.53322E+00_JPRB, 0.54288E+00_JPRB, 0.53308E+00_JPRB, 0.53731E+00_JPRB, 0.53769E+00_JPRB ,&
&  0.54896E+00_JPRB, 0.56002E+00_JPRB, 0.57368E+00_JPRB, 0.58567E+00_JPRB, 0.59807E+00_JPRB, 0.60456E+00_JPRB, 0.61777E+00_JPRB ,&
&  0.64264E+00_JPRB, 0.67368E+00_JPRB, 0.70172E+00_JPRB, 0.73645E+00_JPRB, 0.77890E+00_JPRB, 0.85440E+00_JPRB, 0.93175E+00_JPRB ,&
&  0.93830E+00_JPRB, 0.92925E+00_JPRB, 0.92683E+00_JPRB, 0.92007E+00_JPRB, 0.91948E+00_JPRB, 0.91913E+00_JPRB, 0.92443E+00_JPRB ,&
&  0.93587E+00_JPRB, 0.94594E+00_JPRB, 0.93030E+00_JPRB, 0.90090E+00_JPRB, 0.88857E+00_JPRB, 0.86779E+00_JPRB, 0.86622E+00_JPRB ,&
&  0.86302E+00_JPRB, 0.87658E+00_JPRB, 0.87112E+00_JPRB, 0.86012E+00_JPRB, 0.84980E+00_JPRB, 0.83839E+00_JPRB, 0.83247E+00_JPRB ,&
&  0.82481E+00_JPRB, 0.81155E+00_JPRB, 0.79659E+00_JPRB, 0.78377E+00_JPRB, 0.77195E+00_JPRB, 0.76238E+00_JPRB, 0.74792E+00_JPRB ,&
&  0.72046E+00_JPRB, 0.68622E+00_JPRB, 0.65995E+00_JPRB, 0.63546E+00_JPRB, 0.61556E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.58996E+00_JPRB, 0.58181E+00_JPRB, 0.57687E+00_JPRB, 0.57636E+00_JPRB, 0.57504E+00_JPRB, 0.57479E+00_JPRB, 0.57117E+00_JPRB ,&
&  0.56749E+00_JPRB, 0.56364E+00_JPRB, 0.56011E+00_JPRB, 0.55698E+00_JPRB, 0.55509E+00_JPRB, 0.55321E+00_JPRB, 0.55214E+00_JPRB ,&
&  0.55133E+00_JPRB, 0.54490E+00_JPRB, 0.54302E+00_JPRB, 0.54540E+00_JPRB, 0.54801E+00_JPRB, 0.54974E+00_JPRB, 0.55243E+00_JPRB ,&
&  0.54870E+00_JPRB, 0.54091E+00_JPRB, 0.53104E+00_JPRB, 0.51819E+00_JPRB, 0.50869E+00_JPRB, 0.50614E+00_JPRB, 0.49912E+00_JPRB ,&
&  0.48256E+00_JPRB, 0.46837E+00_JPRB, 0.47430E+00_JPRB, 0.46970E+00_JPRB, 0.44246E+00_JPRB, 0.42823E+00_JPRB, 0.41106E+00_JPRB ,&
&  0.38761E+00_JPRB, 0.35527E+00_JPRB, 0.34777E+00_JPRB, 0.34461E+00_JPRB, 0.35124E+00_JPRB, 0.36222E+00_JPRB, 0.38646E+00_JPRB ,&
&  0.42879E+00_JPRB, 0.51263E+00_JPRB, 0.53592E+00_JPRB, 0.54806E+00_JPRB, 0.55367E+00_JPRB, 0.55331E+00_JPRB, 0.55299E+00_JPRB ,&
&  0.56067E+00_JPRB, 0.56811E+00_JPRB, 0.58005E+00_JPRB, 0.59347E+00_JPRB, 0.61082E+00_JPRB, 0.62569E+00_JPRB, 0.63713E+00_JPRB ,&
&  0.67244E+00_JPRB, 0.69265E+00_JPRB, 0.71231E+00_JPRB, 0.77322E+00_JPRB, 0.88497E+00_JPRB, 0.10026E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10292E+01_JPRB, 0.99402E+00_JPRB, 0.97359E+00_JPRB, 0.96455E+00_JPRB, 0.96255E+00_JPRB, 0.96593E+00_JPRB ,&
&  0.97464E+00_JPRB, 0.99371E+00_JPRB, 0.10531E+01_JPRB, 0.10063E+01_JPRB, 0.97850E+00_JPRB, 0.94983E+00_JPRB, 0.91692E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.87768E+00_JPRB, 0.86868E+00_JPRB, 0.85694E+00_JPRB, 0.84740E+00_JPRB, 0.84040E+00_JPRB, 0.84376E+00_JPRB ,&
&  0.84027E+00_JPRB, 0.82975E+00_JPRB, 0.80868E+00_JPRB, 0.79632E+00_JPRB, 0.78402E+00_JPRB, 0.77154E+00_JPRB, 0.75832E+00_JPRB ,&
&  0.72858E+00_JPRB, 0.68901E+00_JPRB, 0.65757E+00_JPRB, 0.62961E+00_JPRB, 0.60661E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.59665E+00_JPRB, 0.59336E+00_JPRB, 0.59583E+00_JPRB, 0.60313E+00_JPRB, 0.60776E+00_JPRB, 0.60034E+00_JPRB, 0.59076E+00_JPRB ,&
&  0.58364E+00_JPRB, 0.58067E+00_JPRB, 0.57813E+00_JPRB, 0.57308E+00_JPRB, 0.56917E+00_JPRB, 0.56540E+00_JPRB, 0.56182E+00_JPRB ,&
&  0.55830E+00_JPRB, 0.55425E+00_JPRB, 0.54929E+00_JPRB, 0.54210E+00_JPRB, 0.53801E+00_JPRB, 0.53909E+00_JPRB, 0.54380E+00_JPRB ,&
&  0.54824E+00_JPRB, 0.54741E+00_JPRB, 0.54412E+00_JPRB, 0.53349E+00_JPRB, 0.52203E+00_JPRB, 0.51291E+00_JPRB, 0.51294E+00_JPRB ,&
&  0.50568E+00_JPRB, 0.49071E+00_JPRB, 0.47406E+00_JPRB, 0.48057E+00_JPRB, 0.48303E+00_JPRB, 0.45540E+00_JPRB, 0.42759E+00_JPRB ,&
&  0.40126E+00_JPRB, 0.37840E+00_JPRB, 0.36799E+00_JPRB, 0.36708E+00_JPRB, 0.37893E+00_JPRB, 0.40073E+00_JPRB, 0.43010E+00_JPRB ,&
&  0.47159E+00_JPRB, 0.51358E+00_JPRB, 0.53374E+00_JPRB, 0.54936E+00_JPRB, 0.55217E+00_JPRB, 0.55000E+00_JPRB, 0.54768E+00_JPRB ,&
&  0.54992E+00_JPRB, 0.55523E+00_JPRB, 0.56449E+00_JPRB, 0.57208E+00_JPRB, 0.58644E+00_JPRB, 0.61637E+00_JPRB, 0.68022E+00_JPRB ,&
&  0.83535E+00_JPRB, 0.84686E+00_JPRB, 0.78342E+00_JPRB, 0.85480E+00_JPRB, 0.97312E+00_JPRB, 0.10768E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.11512E+01_JPRB, 0.10953E+01_JPRB, 0.10499E+01_JPRB, 0.10247E+01_JPRB, 0.10073E+01_JPRB, 0.99356E+00_JPRB, 0.98910E+00_JPRB ,&
&  0.99403E+00_JPRB, 0.10142E+01_JPRB, 0.11052E+01_JPRB, 0.98063E+00_JPRB, 0.91703E+00_JPRB, 0.89602E+00_JPRB, 0.87173E+00_JPRB ,&
&  0.84948E+00_JPRB, 0.83833E+00_JPRB, 0.84162E+00_JPRB, 0.85200E+00_JPRB, 0.82434E+00_JPRB, 0.79974E+00_JPRB, 0.78526E+00_JPRB ,&
&  0.78620E+00_JPRB, 0.78062E+00_JPRB, 0.77726E+00_JPRB, 0.78381E+00_JPRB, 0.78223E+00_JPRB, 0.76203E+00_JPRB, 0.74853E+00_JPRB ,&
&  0.72607E+00_JPRB, 0.68940E+00_JPRB, 0.66338E+00_JPRB, 0.63828E+00_JPRB, 0.61243E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.59797E+00_JPRB, 0.61178E+00_JPRB, 0.63028E+00_JPRB, 0.64240E+00_JPRB, 0.64472E+00_JPRB, 0.61796E+00_JPRB, 0.59062E+00_JPRB ,&
&  0.57459E+00_JPRB, 0.56875E+00_JPRB, 0.56962E+00_JPRB, 0.56977E+00_JPRB, 0.56883E+00_JPRB, 0.56830E+00_JPRB, 0.56655E+00_JPRB ,&
&  0.56061E+00_JPRB, 0.55323E+00_JPRB, 0.54662E+00_JPRB, 0.54048E+00_JPRB, 0.53577E+00_JPRB, 0.53459E+00_JPRB, 0.53653E+00_JPRB ,&
&  0.54070E+00_JPRB, 0.54316E+00_JPRB, 0.54352E+00_JPRB, 0.53842E+00_JPRB, 0.53535E+00_JPRB, 0.53248E+00_JPRB, 0.52282E+00_JPRB ,&
&  0.50576E+00_JPRB, 0.50326E+00_JPRB, 0.49880E+00_JPRB, 0.49345E+00_JPRB, 0.49728E+00_JPRB, 0.47979E+00_JPRB, 0.45022E+00_JPRB ,&
&  0.42246E+00_JPRB, 0.40280E+00_JPRB, 0.39934E+00_JPRB, 0.40913E+00_JPRB, 0.43346E+00_JPRB, 0.46370E+00_JPRB, 0.48294E+00_JPRB ,&
&  0.50072E+00_JPRB, 0.51602E+00_JPRB, 0.52700E+00_JPRB, 0.53682E+00_JPRB, 0.54384E+00_JPRB, 0.54666E+00_JPRB, 0.54669E+00_JPRB ,&
&  0.54502E+00_JPRB, 0.54485E+00_JPRB, 0.54661E+00_JPRB, 0.53219E+00_JPRB, 0.54676E+00_JPRB, 0.63364E+00_JPRB, 0.72274E+00_JPRB ,&
&  0.82426E+00_JPRB, 0.85488E+00_JPRB, 0.88488E+00_JPRB, 0.97030E+00_JPRB, 0.10474E+01_JPRB, 0.11106E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11682E+01_JPRB, 0.11092E+01_JPRB, 0.10558E+01_JPRB, 0.10380E+01_JPRB, 0.10303E+01_JPRB, 0.10219E+01_JPRB, 0.98669E+00_JPRB ,&
&  0.96036E+00_JPRB, 0.91642E+00_JPRB, 0.85830E+00_JPRB, 0.80809E+00_JPRB, 0.80041E+00_JPRB, 0.82108E+00_JPRB, 0.85332E+00_JPRB ,&
&  0.85833E+00_JPRB, 0.85758E+00_JPRB, 0.85912E+00_JPRB, 0.87168E+00_JPRB, 0.80615E+00_JPRB, 0.76464E+00_JPRB, 0.73968E+00_JPRB ,&
&  0.73496E+00_JPRB, 0.73455E+00_JPRB, 0.73576E+00_JPRB, 0.75236E+00_JPRB, 0.75710E+00_JPRB, 0.74978E+00_JPRB, 0.72007E+00_JPRB ,&
&  0.67947E+00_JPRB, 0.64625E+00_JPRB, 0.62165E+00_JPRB, 0.60480E+00_JPRB, 0.59842E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.66354E+00_JPRB, 0.67457E+00_JPRB, 0.67933E+00_JPRB, 0.68142E+00_JPRB, 0.67216E+00_JPRB, 0.65710E+00_JPRB, 0.64169E+00_JPRB ,&
&  0.63637E+00_JPRB, 0.62309E+00_JPRB, 0.60891E+00_JPRB, 0.58997E+00_JPRB, 0.57336E+00_JPRB, 0.55924E+00_JPRB, 0.55598E+00_JPRB ,&
&  0.55801E+00_JPRB, 0.56007E+00_JPRB, 0.55675E+00_JPRB, 0.54862E+00_JPRB, 0.54099E+00_JPRB, 0.53692E+00_JPRB, 0.53729E+00_JPRB ,&
&  0.53991E+00_JPRB, 0.54377E+00_JPRB, 0.54596E+00_JPRB, 0.54572E+00_JPRB, 0.54513E+00_JPRB, 0.54320E+00_JPRB, 0.53843E+00_JPRB ,&
&  0.52713E+00_JPRB, 0.51973E+00_JPRB, 0.51621E+00_JPRB, 0.51492E+00_JPRB, 0.51981E+00_JPRB, 0.50338E+00_JPRB, 0.47468E+00_JPRB ,&
&  0.44678E+00_JPRB, 0.43763E+00_JPRB, 0.44968E+00_JPRB, 0.48132E+00_JPRB, 0.50516E+00_JPRB, 0.52519E+00_JPRB, 0.53863E+00_JPRB ,&
&  0.53768E+00_JPRB, 0.53492E+00_JPRB, 0.54197E+00_JPRB, 0.55911E+00_JPRB, 0.56154E+00_JPRB, 0.56200E+00_JPRB, 0.55893E+00_JPRB ,&
&  0.55447E+00_JPRB, 0.55394E+00_JPRB, 0.55394E+00_JPRB, 0.54444E+00_JPRB, 0.61771E+00_JPRB, 0.75912E+00_JPRB, 0.80767E+00_JPRB ,&
&  0.89739E+00_JPRB, 0.96273E+00_JPRB, 0.10521E+01_JPRB, 0.10749E+01_JPRB, 0.10605E+01_JPRB, 0.10881E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11421E+01_JPRB, 0.11084E+01_JPRB, 0.10968E+01_JPRB, 0.10565E+01_JPRB, 0.10126E+01_JPRB, 0.97103E+00_JPRB ,&
&  0.96183E+00_JPRB, 0.94096E+00_JPRB, 0.89718E+00_JPRB, 0.84634E+00_JPRB, 0.82786E+00_JPRB, 0.82622E+00_JPRB, 0.84110E+00_JPRB ,&
&  0.84877E+00_JPRB, 0.86321E+00_JPRB, 0.87375E+00_JPRB, 0.88585E+00_JPRB, 0.82910E+00_JPRB, 0.76040E+00_JPRB, 0.72231E+00_JPRB ,&
&  0.70901E+00_JPRB, 0.70441E+00_JPRB, 0.69581E+00_JPRB, 0.70807E+00_JPRB, 0.72344E+00_JPRB, 0.71806E+00_JPRB, 0.68042E+00_JPRB ,&
&  0.64895E+00_JPRB, 0.63084E+00_JPRB, 0.62312E+00_JPRB, 0.63679E+00_JPRB, 0.65303E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.71087E+00_JPRB, 0.72356E+00_JPRB, 0.72065E+00_JPRB, 0.71380E+00_JPRB, 0.71074E+00_JPRB, 0.69799E+00_JPRB, 0.67266E+00_JPRB ,&
&  0.66041E+00_JPRB, 0.63125E+00_JPRB, 0.61468E+00_JPRB, 0.60199E+00_JPRB, 0.59158E+00_JPRB, 0.57689E+00_JPRB, 0.55601E+00_JPRB ,&
&  0.54837E+00_JPRB, 0.55202E+00_JPRB, 0.55999E+00_JPRB, 0.55962E+00_JPRB, 0.55523E+00_JPRB, 0.54961E+00_JPRB, 0.54705E+00_JPRB ,&
&  0.54775E+00_JPRB, 0.55113E+00_JPRB, 0.55675E+00_JPRB, 0.55577E+00_JPRB, 0.55177E+00_JPRB, 0.54882E+00_JPRB, 0.55029E+00_JPRB ,&
&  0.54460E+00_JPRB, 0.53505E+00_JPRB, 0.53667E+00_JPRB, 0.53694E+00_JPRB, 0.54282E+00_JPRB, 0.54607E+00_JPRB, 0.52379E+00_JPRB ,&
&  0.49071E+00_JPRB, 0.49814E+00_JPRB, 0.53255E+00_JPRB, 0.55632E+00_JPRB, 0.57640E+00_JPRB, 0.58259E+00_JPRB, 0.58636E+00_JPRB ,&
&  0.58860E+00_JPRB, 0.59183E+00_JPRB, 0.59212E+00_JPRB, 0.59348E+00_JPRB, 0.59304E+00_JPRB, 0.59074E+00_JPRB, 0.58688E+00_JPRB ,&
&  0.58507E+00_JPRB, 0.55292E+00_JPRB, 0.54884E+00_JPRB, 0.57738E+00_JPRB, 0.73635E+00_JPRB, 0.78971E+00_JPRB, 0.83977E+00_JPRB ,&
&  0.93796E+00_JPRB, 0.10056E+01_JPRB, 0.10895E+01_JPRB, 0.11410E+01_JPRB, 0.11463E+01_JPRB, 0.11507E+01_JPRB, 0.12085E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.12337E+01_JPRB, 0.12103E+01_JPRB, 0.11676E+01_JPRB, 0.10954E+01_JPRB, 0.10640E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10463E+01_JPRB, 0.10008E+01_JPRB, 0.94334E+00_JPRB, 0.90294E+00_JPRB, 0.90415E+00_JPRB, 0.91141E+00_JPRB ,&
&  0.91530E+00_JPRB, 0.93006E+00_JPRB, 0.92574E+00_JPRB, 0.91305E+00_JPRB, 0.87171E+00_JPRB, 0.83753E+00_JPRB, 0.82843E+00_JPRB ,&
&  0.80875E+00_JPRB, 0.77321E+00_JPRB, 0.71977E+00_JPRB, 0.70726E+00_JPRB, 0.71471E+00_JPRB, 0.70676E+00_JPRB, 0.68164E+00_JPRB ,&
&  0.67064E+00_JPRB, 0.66120E+00_JPRB, 0.65819E+00_JPRB, 0.67606E+00_JPRB, 0.69046E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.72525E+00_JPRB, 0.73403E+00_JPRB, 0.72963E+00_JPRB, 0.72108E+00_JPRB, 0.72914E+00_JPRB, 0.71916E+00_JPRB, 0.71640E+00_JPRB ,&
&  0.72789E+00_JPRB, 0.71543E+00_JPRB, 0.73155E+00_JPRB, 0.65920E+00_JPRB, 0.61009E+00_JPRB, 0.59107E+00_JPRB, 0.57005E+00_JPRB ,&
&  0.56269E+00_JPRB, 0.56779E+00_JPRB, 0.57035E+00_JPRB, 0.57112E+00_JPRB, 0.57088E+00_JPRB, 0.56243E+00_JPRB, 0.55885E+00_JPRB ,&
&  0.56035E+00_JPRB, 0.56573E+00_JPRB, 0.57229E+00_JPRB, 0.57061E+00_JPRB, 0.56724E+00_JPRB, 0.56488E+00_JPRB, 0.56195E+00_JPRB ,&
&  0.55575E+00_JPRB, 0.55802E+00_JPRB, 0.56316E+00_JPRB, 0.56051E+00_JPRB, 0.56396E+00_JPRB, 0.57306E+00_JPRB, 0.57270E+00_JPRB ,&
&  0.57923E+00_JPRB, 0.57257E+00_JPRB, 0.60571E+00_JPRB, 0.62482E+00_JPRB, 0.63334E+00_JPRB, 0.63125E+00_JPRB, 0.62903E+00_JPRB ,&
&  0.63181E+00_JPRB, 0.63826E+00_JPRB, 0.64622E+00_JPRB, 0.65280E+00_JPRB, 0.65160E+00_JPRB, 0.65187E+00_JPRB, 0.65082E+00_JPRB ,&
&  0.65219E+00_JPRB, 0.59758E+00_JPRB, 0.66171E+00_JPRB, 0.69873E+00_JPRB, 0.81545E+00_JPRB, 0.95066E+00_JPRB, 0.97866E+00_JPRB ,&
&  0.11808E+01_JPRB, 0.11645E+01_JPRB, 0.12137E+01_JPRB, 0.12192E+01_JPRB, 0.12242E+01_JPRB, 0.12561E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.14054E+01_JPRB, 0.13446E+01_JPRB, 0.13610E+01_JPRB, 0.12663E+01_JPRB, 0.12002E+01_JPRB, 0.11935E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11851E+01_JPRB, 0.11857E+01_JPRB, 0.11650E+01_JPRB, 0.10507E+01_JPRB, 0.10227E+01_JPRB, 0.10122E+01_JPRB, 0.95905E+00_JPRB ,&
&  0.97704E+00_JPRB, 0.97164E+00_JPRB, 0.93215E+00_JPRB, 0.97393E+00_JPRB, 0.90327E+00_JPRB, 0.86445E+00_JPRB, 0.87392E+00_JPRB ,&
&  0.91872E+00_JPRB, 0.84632E+00_JPRB, 0.78490E+00_JPRB, 0.75670E+00_JPRB, 0.73241E+00_JPRB, 0.71202E+00_JPRB, 0.71013E+00_JPRB ,&
&  0.68969E+00_JPRB, 0.69277E+00_JPRB, 0.71803E+00_JPRB, 0.71753E+00_JPRB, 0.71445E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.73915E+00_JPRB, 0.73503E+00_JPRB, 0.72825E+00_JPRB, 0.72485E+00_JPRB, 0.75001E+00_JPRB, 0.76058E+00_JPRB, 0.76176E+00_JPRB ,&
&  0.77835E+00_JPRB, 0.77238E+00_JPRB, 0.76765E+00_JPRB, 0.75095E+00_JPRB, 0.72986E+00_JPRB, 0.67619E+00_JPRB, 0.59694E+00_JPRB ,&
&  0.56964E+00_JPRB, 0.57351E+00_JPRB, 0.58510E+00_JPRB, 0.58823E+00_JPRB, 0.58154E+00_JPRB, 0.57295E+00_JPRB, 0.56955E+00_JPRB ,&
&  0.57474E+00_JPRB, 0.58054E+00_JPRB, 0.58810E+00_JPRB, 0.59076E+00_JPRB, 0.58830E+00_JPRB, 0.58767E+00_JPRB, 0.58684E+00_JPRB ,&
&  0.57926E+00_JPRB, 0.58335E+00_JPRB, 0.59039E+00_JPRB, 0.58081E+00_JPRB, 0.60328E+00_JPRB, 0.62299E+00_JPRB, 0.62699E+00_JPRB ,&
&  0.64942E+00_JPRB, 0.66947E+00_JPRB, 0.67821E+00_JPRB, 0.68822E+00_JPRB, 0.68373E+00_JPRB, 0.67869E+00_JPRB, 0.68255E+00_JPRB ,&
&  0.69423E+00_JPRB, 0.70566E+00_JPRB, 0.71694E+00_JPRB, 0.72546E+00_JPRB, 0.72773E+00_JPRB, 0.73381E+00_JPRB, 0.76705E+00_JPRB ,&
&  0.78107E+00_JPRB, 0.81692E+00_JPRB, 0.89312E+00_JPRB, 0.90049E+00_JPRB, 0.99794E+00_JPRB, 0.10855E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.11954E+01_JPRB, 0.11877E+01_JPRB, 0.13194E+01_JPRB, 0.14157E+01_JPRB, 0.15019E+01_JPRB, 0.15396E+01_JPRB, 0.15875E+01_JPRB ,&
&  0.17069E+01_JPRB, 0.17770E+01_JPRB, 0.15849E+01_JPRB, 0.14843E+01_JPRB, 0.13990E+01_JPRB, 0.13943E+01_JPRB, 0.13420E+01_JPRB ,&
&  0.13480E+01_JPRB, 0.13780E+01_JPRB, 0.13596E+01_JPRB, 0.13252E+01_JPRB, 0.12503E+01_JPRB, 0.11675E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10416E+01_JPRB, 0.95092E+00_JPRB, 0.90636E+00_JPRB, 0.92714E+00_JPRB, 0.93134E+00_JPRB, 0.85742E+00_JPRB, 0.90126E+00_JPRB ,&
&  0.99050E+00_JPRB, 0.94171E+00_JPRB, 0.86684E+00_JPRB, 0.80708E+00_JPRB, 0.78146E+00_JPRB, 0.75870E+00_JPRB, 0.71678E+00_JPRB ,&
&  0.68634E+00_JPRB, 0.72109E+00_JPRB, 0.74374E+00_JPRB, 0.74237E+00_JPRB, 0.73859E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.75650E+00_JPRB, 0.74290E+00_JPRB, 0.72717E+00_JPRB, 0.73241E+00_JPRB, 0.76285E+00_JPRB, 0.77618E+00_JPRB, 0.79569E+00_JPRB ,&
&  0.80625E+00_JPRB, 0.80784E+00_JPRB, 0.80002E+00_JPRB, 0.77199E+00_JPRB, 0.75674E+00_JPRB, 0.74991E+00_JPRB, 0.70668E+00_JPRB ,&
&  0.64405E+00_JPRB, 0.62762E+00_JPRB, 0.61009E+00_JPRB, 0.60726E+00_JPRB, 0.60615E+00_JPRB, 0.59431E+00_JPRB, 0.58857E+00_JPRB ,&
&  0.59632E+00_JPRB, 0.59724E+00_JPRB, 0.59976E+00_JPRB, 0.60289E+00_JPRB, 0.60537E+00_JPRB, 0.61061E+00_JPRB, 0.61290E+00_JPRB ,&
&  0.60535E+00_JPRB, 0.61537E+00_JPRB, 0.62935E+00_JPRB, 0.62382E+00_JPRB, 0.66022E+00_JPRB, 0.70043E+00_JPRB, 0.72319E+00_JPRB ,&
&  0.74461E+00_JPRB, 0.75099E+00_JPRB, 0.74697E+00_JPRB, 0.74268E+00_JPRB, 0.73303E+00_JPRB, 0.73722E+00_JPRB, 0.75267E+00_JPRB ,&
&  0.77388E+00_JPRB, 0.78773E+00_JPRB, 0.79831E+00_JPRB, 0.81208E+00_JPRB, 0.80969E+00_JPRB, 0.85149E+00_JPRB, 0.94340E+00_JPRB ,&
&  0.10372E+01_JPRB, 0.11209E+01_JPRB, 0.11753E+01_JPRB, 0.12309E+01_JPRB, 0.13211E+01_JPRB, 0.13507E+01_JPRB, 0.13054E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.13250E+01_JPRB, 0.16913E+01_JPRB, 0.17476E+01_JPRB, 0.17409E+01_JPRB, 0.18129E+01_JPRB, 0.17887E+01_JPRB ,&
&  0.17947E+01_JPRB, 0.17081E+01_JPRB, 0.16371E+01_JPRB, 0.15897E+01_JPRB, 0.15552E+01_JPRB, 0.16174E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.15697E+01_JPRB, 0.15654E+01_JPRB, 0.13816E+01_JPRB, 0.12307E+01_JPRB, 0.10734E+01_JPRB, 0.11174E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.89150E+00_JPRB, 0.93457E+00_JPRB, 0.97839E+00_JPRB, 0.10019E+01_JPRB, 0.10557E+01_JPRB, 0.10907E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.11432E+01_JPRB, 0.10192E+01_JPRB, 0.92072E+00_JPRB, 0.86920E+00_JPRB, 0.82931E+00_JPRB, 0.75248E+00_JPRB ,&
&  0.73327E+00_JPRB, 0.76548E+00_JPRB, 0.78667E+00_JPRB, 0.78554E+00_JPRB, 0.77081E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.78968E+00_JPRB, 0.77197E+00_JPRB, 0.74712E+00_JPRB, 0.75725E+00_JPRB, 0.78148E+00_JPRB, 0.78992E+00_JPRB, 0.81217E+00_JPRB ,&
&  0.83527E+00_JPRB, 0.83552E+00_JPRB, 0.80543E+00_JPRB, 0.78151E+00_JPRB, 0.76951E+00_JPRB, 0.76983E+00_JPRB, 0.77914E+00_JPRB ,&
&  0.71228E+00_JPRB, 0.67563E+00_JPRB, 0.62926E+00_JPRB, 0.62639E+00_JPRB, 0.66481E+00_JPRB, 0.64049E+00_JPRB, 0.63573E+00_JPRB ,&
&  0.62746E+00_JPRB, 0.62833E+00_JPRB, 0.62015E+00_JPRB, 0.61618E+00_JPRB, 0.61730E+00_JPRB, 0.62601E+00_JPRB, 0.64062E+00_JPRB ,&
&  0.64558E+00_JPRB, 0.67771E+00_JPRB, 0.71220E+00_JPRB, 0.71367E+00_JPRB, 0.75493E+00_JPRB, 0.79117E+00_JPRB, 0.82719E+00_JPRB ,&
&  0.83855E+00_JPRB, 0.81911E+00_JPRB, 0.79666E+00_JPRB, 0.79207E+00_JPRB, 0.79552E+00_JPRB, 0.81745E+00_JPRB, 0.84719E+00_JPRB ,&
&  0.88014E+00_JPRB, 0.90694E+00_JPRB, 0.92431E+00_JPRB, 0.93831E+00_JPRB, 0.93376E+00_JPRB, 0.10012E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.14690E+01_JPRB, 0.15430E+01_JPRB, 0.18071E+01_JPRB, 0.19246E+01_JPRB, 0.18810E+01_JPRB, 0.17055E+01_JPRB ,&
&  0.16036E+01_JPRB, 0.15130E+01_JPRB, 0.16466E+01_JPRB, 0.17687E+01_JPRB, 0.17486E+01_JPRB, 0.18931E+01_JPRB, 0.18408E+01_JPRB ,&
&  0.16898E+01_JPRB, 0.15818E+01_JPRB, 0.15180E+01_JPRB, 0.15618E+01_JPRB, 0.15484E+01_JPRB, 0.16375E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.16471E+01_JPRB, 0.14538E+01_JPRB, 0.11242E+01_JPRB, 0.10474E+01_JPRB, 0.85593E+00_JPRB, 0.86777E+00_JPRB, 0.10402E+01_JPRB ,&
&  0.10395E+01_JPRB, 0.10417E+01_JPRB, 0.10523E+01_JPRB, 0.11178E+01_JPRB, 0.11554E+01_JPRB, 0.13825E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.13313E+01_JPRB, 0.13277E+01_JPRB, 0.12741E+01_JPRB, 0.11894E+01_JPRB, 0.10954E+01_JPRB, 0.97767E+00_JPRB, 0.88480E+00_JPRB ,&
&  0.85574E+00_JPRB, 0.84254E+00_JPRB, 0.84661E+00_JPRB, 0.83045E+00_JPRB, 0.80355E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.84442E+00_JPRB, 0.82781E+00_JPRB, 0.79120E+00_JPRB, 0.79451E+00_JPRB, 0.80287E+00_JPRB, 0.81285E+00_JPRB, 0.83948E+00_JPRB ,&
&  0.86363E+00_JPRB, 0.84441E+00_JPRB, 0.80619E+00_JPRB, 0.79178E+00_JPRB, 0.78820E+00_JPRB, 0.78733E+00_JPRB, 0.80018E+00_JPRB ,&
&  0.82624E+00_JPRB, 0.79342E+00_JPRB, 0.71378E+00_JPRB, 0.64977E+00_JPRB, 0.67015E+00_JPRB, 0.66607E+00_JPRB, 0.68479E+00_JPRB ,&
&  0.67479E+00_JPRB, 0.69662E+00_JPRB, 0.67781E+00_JPRB, 0.66936E+00_JPRB, 0.66511E+00_JPRB, 0.67990E+00_JPRB, 0.71486E+00_JPRB ,&
&  0.76236E+00_JPRB, 0.84359E+00_JPRB, 0.91918E+00_JPRB, 0.92482E+00_JPRB, 0.95783E+00_JPRB, 0.91202E+00_JPRB, 0.92743E+00_JPRB ,&
&  0.91957E+00_JPRB, 0.88867E+00_JPRB, 0.86259E+00_JPRB, 0.86424E+00_JPRB, 0.88682E+00_JPRB, 0.92256E+00_JPRB, 0.97471E+00_JPRB ,&
&  0.10322E+01_JPRB, 0.10583E+01_JPRB, 0.10409E+01_JPRB, 0.10306E+01_JPRB, 0.10665E+01_JPRB, 0.11490E+01_JPRB, 0.12203E+01_JPRB ,&
&  0.14074E+01_JPRB, 0.15171E+01_JPRB, 0.15337E+01_JPRB, 0.18777E+01_JPRB, 0.21347E+01_JPRB, 0.21724E+01_JPRB, 0.20657E+01_JPRB ,&
&  0.19475E+01_JPRB, 0.18330E+01_JPRB, 0.19082E+01_JPRB, 0.19453E+01_JPRB, 0.19531E+01_JPRB, 0.19549E+01_JPRB, 0.18493E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.15360E+01_JPRB, 0.15300E+01_JPRB, 0.15161E+01_JPRB, 0.15297E+01_JPRB, 0.14810E+01_JPRB, 0.15292E+01_JPRB ,&
&  0.15995E+01_JPRB, 0.99633E+00_JPRB, 0.83950E+00_JPRB, 0.95436E+00_JPRB, 0.83462E+00_JPRB, 0.85198E+00_JPRB, 0.95062E+00_JPRB ,&
&  0.94255E+00_JPRB, 0.95283E+00_JPRB, 0.10877E+01_JPRB, 0.12503E+01_JPRB, 0.14386E+01_JPRB, 0.17674E+01_JPRB, 0.17908E+01_JPRB ,&
&  0.16730E+01_JPRB, 0.15279E+01_JPRB, 0.15602E+01_JPRB, 0.15267E+01_JPRB, 0.14013E+01_JPRB, 0.12093E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.97490E+00_JPRB, 0.94418E+00_JPRB, 0.92944E+00_JPRB, 0.89973E+00_JPRB, 0.85873E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.92377E+00_JPRB, 0.89872E+00_JPRB, 0.83495E+00_JPRB, 0.82137E+00_JPRB, 0.83536E+00_JPRB, 0.86366E+00_JPRB, 0.89158E+00_JPRB ,&
&  0.89300E+00_JPRB, 0.85433E+00_JPRB, 0.82537E+00_JPRB, 0.81633E+00_JPRB, 0.80867E+00_JPRB, 0.79889E+00_JPRB, 0.79761E+00_JPRB ,&
&  0.80344E+00_JPRB, 0.81237E+00_JPRB, 0.73676E+00_JPRB, 0.70278E+00_JPRB, 0.66002E+00_JPRB, 0.67543E+00_JPRB, 0.70094E+00_JPRB ,&
&  0.73416E+00_JPRB, 0.73059E+00_JPRB, 0.71976E+00_JPRB, 0.72665E+00_JPRB, 0.73531E+00_JPRB, 0.76458E+00_JPRB, 0.86472E+00_JPRB ,&
&  0.98449E+00_JPRB, 0.11225E+01_JPRB, 0.11473E+01_JPRB, 0.11898E+01_JPRB, 0.11224E+01_JPRB, 0.10245E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.96516E+00_JPRB, 0.93457E+00_JPRB, 0.94295E+00_JPRB, 0.97334E+00_JPRB, 0.99867E+00_JPRB, 0.10492E+01_JPRB ,&
&  0.10881E+01_JPRB, 0.11027E+01_JPRB, 0.10913E+01_JPRB, 0.10675E+01_JPRB, 0.11228E+01_JPRB, 0.11768E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.12057E+01_JPRB, 0.10502E+01_JPRB, 0.14832E+01_JPRB, 0.22015E+01_JPRB, 0.22767E+01_JPRB, 0.19814E+01_JPRB ,&
&  0.20419E+01_JPRB, 0.20191E+01_JPRB, 0.21259E+01_JPRB, 0.22311E+01_JPRB, 0.21914E+01_JPRB, 0.20512E+01_JPRB, 0.18852E+01_JPRB ,&
&  0.16985E+01_JPRB, 0.16147E+01_JPRB, 0.16208E+01_JPRB, 0.18068E+01_JPRB, 0.18217E+01_JPRB, 0.18015E+01_JPRB, 0.17427E+01_JPRB ,&
&  0.15031E+01_JPRB, 0.13234E+01_JPRB, 0.98719E+00_JPRB, 0.10052E+01_JPRB, 0.82393E+00_JPRB, 0.82934E+00_JPRB, 0.92945E+00_JPRB ,&
&  0.91270E+00_JPRB, 0.97697E+00_JPRB, 0.11303E+01_JPRB, 0.13204E+01_JPRB, 0.18267E+01_JPRB, 0.21977E+01_JPRB, 0.23110E+01_JPRB ,&
&  0.22031E+01_JPRB, 0.19233E+01_JPRB, 0.19660E+01_JPRB, 0.18221E+01_JPRB, 0.17011E+01_JPRB, 0.15072E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.10969E+01_JPRB, 0.10593E+01_JPRB, 0.10073E+01_JPRB, 0.94738E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.97381E+00_JPRB, 0.92156E+00_JPRB, 0.83368E+00_JPRB, 0.82204E+00_JPRB, 0.84685E+00_JPRB, 0.90531E+00_JPRB, 0.93606E+00_JPRB ,&
&  0.91276E+00_JPRB, 0.86915E+00_JPRB, 0.84257E+00_JPRB, 0.83100E+00_JPRB, 0.81381E+00_JPRB, 0.79257E+00_JPRB, 0.78417E+00_JPRB ,&
&  0.78109E+00_JPRB, 0.77001E+00_JPRB, 0.71188E+00_JPRB, 0.67587E+00_JPRB, 0.64354E+00_JPRB, 0.64941E+00_JPRB, 0.67963E+00_JPRB ,&
&  0.70982E+00_JPRB, 0.73660E+00_JPRB, 0.77428E+00_JPRB, 0.80257E+00_JPRB, 0.82090E+00_JPRB, 0.90195E+00_JPRB, 0.10739E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.13327E+01_JPRB, 0.13661E+01_JPRB, 0.12850E+01_JPRB, 0.12006E+01_JPRB, 0.11295E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.99513E+00_JPRB, 0.93869E+00_JPRB, 0.92008E+00_JPRB, 0.93131E+00_JPRB, 0.99181E+00_JPRB, 0.10553E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11911E+01_JPRB, 0.11705E+01_JPRB, 0.11169E+01_JPRB, 0.11153E+01_JPRB, 0.11903E+01_JPRB, 0.12101E+01_JPRB, 0.11941E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10197E+01_JPRB, 0.12267E+01_JPRB, 0.15007E+01_JPRB, 0.19639E+01_JPRB, 0.22852E+01_JPRB, 0.21956E+01_JPRB ,&
&  0.22146E+01_JPRB, 0.21712E+01_JPRB, 0.21909E+01_JPRB, 0.18339E+01_JPRB, 0.20263E+01_JPRB, 0.19360E+01_JPRB, 0.18093E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.16854E+01_JPRB, 0.17877E+01_JPRB, 0.20020E+01_JPRB, 0.21464E+01_JPRB, 0.22311E+01_JPRB, 0.17299E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.15159E+01_JPRB, 0.13882E+01_JPRB, 0.11838E+01_JPRB, 0.10242E+01_JPRB, 0.10431E+01_JPRB, 0.10405E+01_JPRB ,&
&  0.10448E+01_JPRB, 0.10808E+01_JPRB, 0.12632E+01_JPRB, 0.16692E+01_JPRB, 0.23683E+01_JPRB, 0.28673E+01_JPRB, 0.28740E+01_JPRB ,&
&  0.25537E+01_JPRB, 0.26070E+01_JPRB, 0.24063E+01_JPRB, 0.21159E+01_JPRB, 0.19076E+01_JPRB, 0.17321E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.13867E+01_JPRB, 0.12856E+01_JPRB, 0.12112E+01_JPRB, 0.11218E+01_JPRB, 0.10244E+01_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.10152E+01_JPRB, 0.95836E+00_JPRB, 0.87213E+00_JPRB, 0.86117E+00_JPRB, 0.90718E+00_JPRB, 0.96717E+00_JPRB, 0.97428E+00_JPRB ,&
&  0.90971E+00_JPRB, 0.86801E+00_JPRB, 0.84126E+00_JPRB, 0.83848E+00_JPRB, 0.82652E+00_JPRB, 0.80031E+00_JPRB, 0.77980E+00_JPRB ,&
&  0.78585E+00_JPRB, 0.75025E+00_JPRB, 0.69239E+00_JPRB, 0.69421E+00_JPRB, 0.68717E+00_JPRB, 0.64438E+00_JPRB, 0.65981E+00_JPRB ,&
&  0.69395E+00_JPRB, 0.74081E+00_JPRB, 0.79430E+00_JPRB, 0.87520E+00_JPRB, 0.98786E+00_JPRB, 0.11542E+01_JPRB, 0.13674E+01_JPRB ,&
&  0.14367E+01_JPRB, 0.14639E+01_JPRB, 0.14707E+01_JPRB, 0.13677E+01_JPRB, 0.11980E+01_JPRB, 0.10848E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.96293E+00_JPRB, 0.91826E+00_JPRB, 0.93314E+00_JPRB, 0.10002E+01_JPRB, 0.11067E+01_JPRB, 0.11840E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12163E+01_JPRB, 0.12096E+01_JPRB, 0.12039E+01_JPRB, 0.12997E+01_JPRB, 0.10602E+01_JPRB, 0.10339E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.12023E+01_JPRB, 0.12393E+01_JPRB, 0.15528E+01_JPRB, 0.15209E+01_JPRB, 0.17905E+01_JPRB, 0.21472E+01_JPRB ,&
&  0.23323E+01_JPRB, 0.23560E+01_JPRB, 0.23490E+01_JPRB, 0.15328E+01_JPRB, 0.13912E+01_JPRB, 0.17877E+01_JPRB, 0.18333E+01_JPRB ,&
&  0.17967E+01_JPRB, 0.19280E+01_JPRB, 0.20924E+01_JPRB, 0.24064E+01_JPRB, 0.18734E+01_JPRB, 0.16321E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.12261E+01_JPRB, 0.14345E+01_JPRB, 0.13261E+01_JPRB, 0.13697E+01_JPRB, 0.13170E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.14178E+01_JPRB, 0.17207E+01_JPRB, 0.23967E+01_JPRB, 0.32280E+01_JPRB, 0.31588E+01_JPRB, 0.34838E+01_JPRB ,&
&  0.34455E+01_JPRB, 0.31022E+01_JPRB, 0.25674E+01_JPRB, 0.23273E+01_JPRB, 0.21306E+01_JPRB, 0.19681E+01_JPRB, 0.17916E+01_JPRB ,&
&  0.16070E+01_JPRB, 0.14619E+01_JPRB, 0.13311E+01_JPRB, 0.12005E+01_JPRB, 0.10841E+01_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.11004E+01_JPRB, 0.10402E+01_JPRB, 0.93454E+00_JPRB, 0.93183E+00_JPRB, 0.98653E+00_JPRB, 0.10130E+01_JPRB, 0.96495E+00_JPRB ,&
&  0.91150E+00_JPRB, 0.86526E+00_JPRB, 0.84969E+00_JPRB, 0.84153E+00_JPRB, 0.82650E+00_JPRB, 0.81816E+00_JPRB, 0.80012E+00_JPRB ,&
&  0.82250E+00_JPRB, 0.81603E+00_JPRB, 0.73980E+00_JPRB, 0.69974E+00_JPRB, 0.71544E+00_JPRB, 0.66972E+00_JPRB, 0.64352E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.78811E+00_JPRB, 0.89476E+00_JPRB, 0.10362E+01_JPRB, 0.12039E+01_JPRB, 0.15005E+01_JPRB, 0.16592E+01_JPRB ,&
&  0.16245E+01_JPRB, 0.15790E+01_JPRB, 0.14011E+01_JPRB, 0.12258E+01_JPRB, 0.10763E+01_JPRB, 0.99344E+00_JPRB, 0.94147E+00_JPRB ,&
&  0.91051E+00_JPRB, 0.90785E+00_JPRB, 0.96794E+00_JPRB, 0.10683E+01_JPRB, 0.11755E+01_JPRB, 0.12227E+01_JPRB, 0.12284E+01_JPRB ,&
&  0.12644E+01_JPRB, 0.13466E+01_JPRB, 0.13651E+01_JPRB, 0.13088E+01_JPRB, 0.12081E+01_JPRB, 0.10564E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.11397E+01_JPRB, 0.12214E+01_JPRB, 0.14962E+01_JPRB, 0.17272E+01_JPRB, 0.18005E+01_JPRB, 0.21664E+01_JPRB ,&
&  0.18225E+01_JPRB, 0.15198E+01_JPRB, 0.12964E+01_JPRB, 0.10605E+01_JPRB, 0.14065E+01_JPRB, 0.19543E+01_JPRB, 0.21136E+01_JPRB ,&
&  0.21637E+01_JPRB, 0.23079E+01_JPRB, 0.25157E+01_JPRB, 0.22852E+01_JPRB, 0.17211E+01_JPRB, 0.14480E+01_JPRB, 0.17708E+01_JPRB ,&
&  0.18820E+01_JPRB, 0.20099E+01_JPRB, 0.19672E+01_JPRB, 0.17121E+01_JPRB, 0.14927E+01_JPRB, 0.16939E+01_JPRB, 0.16943E+01_JPRB ,&
&  0.18351E+01_JPRB, 0.20023E+01_JPRB, 0.28978E+01_JPRB, 0.38158E+01_JPRB, 0.40959E+01_JPRB, 0.42967E+01_JPRB, 0.44988E+01_JPRB ,&
&  0.39581E+01_JPRB, 0.35333E+01_JPRB, 0.30039E+01_JPRB, 0.25781E+01_JPRB, 0.22951E+01_JPRB, 0.21065E+01_JPRB, 0.19583E+01_JPRB ,&
&  0.17800E+01_JPRB, 0.16481E+01_JPRB, 0.14968E+01_JPRB, 0.13107E+01_JPRB, 0.11726E+01_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.12114E+01_JPRB, 0.11223E+01_JPRB, 0.10010E+01_JPRB, 0.99481E+00_JPRB, 0.10479E+01_JPRB, 0.10254E+01_JPRB, 0.95077E+00_JPRB ,&
&  0.92176E+00_JPRB, 0.87895E+00_JPRB, 0.86408E+00_JPRB, 0.84731E+00_JPRB, 0.82804E+00_JPRB, 0.81590E+00_JPRB, 0.82328E+00_JPRB ,&
&  0.87343E+00_JPRB, 0.95089E+00_JPRB, 0.85922E+00_JPRB, 0.79185E+00_JPRB, 0.76461E+00_JPRB, 0.73947E+00_JPRB, 0.66797E+00_JPRB ,&
&  0.76560E+00_JPRB, 0.91138E+00_JPRB, 0.10535E+01_JPRB, 0.12400E+01_JPRB, 0.15152E+01_JPRB, 0.16002E+01_JPRB, 0.16300E+01_JPRB ,&
&  0.15955E+01_JPRB, 0.14574E+01_JPRB, 0.12736E+01_JPRB, 0.11107E+01_JPRB, 0.10096E+01_JPRB, 0.96133E+00_JPRB, 0.92664E+00_JPRB ,&
&  0.92408E+00_JPRB, 0.97665E+00_JPRB, 0.10516E+01_JPRB, 0.11394E+01_JPRB, 0.12500E+01_JPRB, 0.13128E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.14162E+01_JPRB, 0.14877E+01_JPRB, 0.15010E+01_JPRB, 0.14584E+01_JPRB, 0.14284E+01_JPRB, 0.12632E+01_JPRB, 0.14881E+01_JPRB ,&
&  0.14330E+01_JPRB, 0.14776E+01_JPRB, 0.14351E+01_JPRB, 0.17539E+01_JPRB, 0.17512E+01_JPRB, 0.17410E+01_JPRB, 0.19543E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.14241E+01_JPRB, 0.15687E+01_JPRB, 0.17105E+01_JPRB, 0.17417E+01_JPRB, 0.19237E+01_JPRB, 0.27342E+01_JPRB ,&
&  0.22991E+01_JPRB, 0.25955E+01_JPRB, 0.28206E+01_JPRB, 0.27623E+01_JPRB, 0.13353E+01_JPRB, 0.10978E+01_JPRB, 0.17594E+01_JPRB ,&
&  0.19765E+01_JPRB, 0.13847E+01_JPRB, 0.10762E+01_JPRB, 0.13316E+01_JPRB, 0.10715E+01_JPRB, 0.13043E+01_JPRB, 0.19211E+01_JPRB ,&
&  0.23593E+01_JPRB, 0.29622E+01_JPRB, 0.46522E+01_JPRB, 0.49505E+01_JPRB, 0.46543E+01_JPRB, 0.55283E+01_JPRB, 0.46449E+01_JPRB ,&
&  0.41231E+01_JPRB, 0.32099E+01_JPRB, 0.30103E+01_JPRB, 0.26680E+01_JPRB, 0.24113E+01_JPRB, 0.22554E+01_JPRB, 0.21129E+01_JPRB ,&
&  0.19438E+01_JPRB, 0.17879E+01_JPRB, 0.16239E+01_JPRB, 0.14507E+01_JPRB, 0.13153E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.12869E+01_JPRB, 0.11829E+01_JPRB, 0.10754E+01_JPRB, 0.10536E+01_JPRB, 0.10810E+01_JPRB, 0.10050E+01_JPRB, 0.94914E+00_JPRB ,&
&  0.90545E+00_JPRB, 0.88343E+00_JPRB, 0.86305E+00_JPRB, 0.83770E+00_JPRB, 0.81662E+00_JPRB, 0.82303E+00_JPRB, 0.84918E+00_JPRB ,&
&  0.90067E+00_JPRB, 0.10126E+01_JPRB, 0.10985E+01_JPRB, 0.10357E+01_JPRB, 0.99181E+00_JPRB, 0.92766E+00_JPRB, 0.86684E+00_JPRB ,&
&  0.96146E+00_JPRB, 0.11699E+01_JPRB, 0.12941E+01_JPRB, 0.13723E+01_JPRB, 0.14446E+01_JPRB, 0.14257E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.14490E+01_JPRB, 0.12929E+01_JPRB, 0.11667E+01_JPRB, 0.10597E+01_JPRB, 0.10004E+01_JPRB, 0.96544E+00_JPRB, 0.97145E+00_JPRB ,&
&  0.10055E+01_JPRB, 0.10482E+01_JPRB, 0.10943E+01_JPRB, 0.12153E+01_JPRB, 0.13574E+01_JPRB, 0.14243E+01_JPRB, 0.14840E+01_JPRB ,&
&  0.15404E+01_JPRB, 0.15655E+01_JPRB, 0.15740E+01_JPRB, 0.15945E+01_JPRB, 0.17014E+01_JPRB, 0.15588E+01_JPRB, 0.13630E+01_JPRB ,&
&  0.12751E+01_JPRB, 0.14279E+01_JPRB, 0.19337E+01_JPRB, 0.19171E+01_JPRB, 0.17291E+01_JPRB, 0.17866E+01_JPRB, 0.17774E+01_JPRB ,&
&  0.17340E+01_JPRB, 0.19394E+01_JPRB, 0.20393E+01_JPRB, 0.22453E+01_JPRB, 0.23390E+01_JPRB, 0.19034E+01_JPRB, 0.25656E+01_JPRB ,&
&  0.27497E+01_JPRB, 0.26939E+01_JPRB, 0.20789E+01_JPRB, 0.14998E+01_JPRB, 0.15778E+01_JPRB, 0.12936E+01_JPRB, 0.95261E+00_JPRB ,&
&  0.90906E+00_JPRB, 0.87604E+00_JPRB, 0.81487E+00_JPRB, 0.85699E+00_JPRB, 0.87855E+00_JPRB, 0.12662E+01_JPRB, 0.20030E+01_JPRB ,&
&  0.32884E+01_JPRB, 0.46050E+01_JPRB, 0.54868E+01_JPRB, 0.54126E+01_JPRB, 0.49794E+01_JPRB, 0.56076E+01_JPRB, 0.47897E+01_JPRB ,&
&  0.34532E+01_JPRB, 0.29220E+01_JPRB, 0.27572E+01_JPRB, 0.26447E+01_JPRB, 0.25467E+01_JPRB, 0.24430E+01_JPRB, 0.22723E+01_JPRB ,&
&  0.20647E+01_JPRB, 0.19056E+01_JPRB, 0.17540E+01_JPRB, 0.15640E+01_JPRB, 0.14156E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.13445E+01_JPRB, 0.12418E+01_JPRB, 0.11796E+01_JPRB, 0.11324E+01_JPRB, 0.11008E+01_JPRB, 0.99138E+00_JPRB, 0.94635E+00_JPRB ,&
&  0.88905E+00_JPRB, 0.87279E+00_JPRB, 0.83887E+00_JPRB, 0.82055E+00_JPRB, 0.81073E+00_JPRB, 0.83764E+00_JPRB, 0.89030E+00_JPRB ,&
&  0.96245E+00_JPRB, 0.11371E+01_JPRB, 0.13730E+01_JPRB, 0.14706E+01_JPRB, 0.15794E+01_JPRB, 0.15247E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.15914E+01_JPRB, 0.15485E+01_JPRB, 0.14879E+01_JPRB, 0.14708E+01_JPRB, 0.14230E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.13053E+01_JPRB, 0.11877E+01_JPRB, 0.10839E+01_JPRB, 0.10227E+01_JPRB, 0.99878E+00_JPRB, 0.99750E+00_JPRB, 0.10141E+01_JPRB ,&
&  0.10673E+01_JPRB, 0.10797E+01_JPRB, 0.11527E+01_JPRB, 0.13073E+01_JPRB, 0.14396E+01_JPRB, 0.14955E+01_JPRB, 0.15744E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.16452E+01_JPRB, 0.16267E+01_JPRB, 0.15889E+01_JPRB, 0.13911E+01_JPRB, 0.11987E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.16031E+01_JPRB, 0.19346E+01_JPRB, 0.21228E+01_JPRB, 0.22710E+01_JPRB, 0.18626E+01_JPRB, 0.17674E+01_JPRB, 0.17829E+01_JPRB ,&
&  0.19212E+01_JPRB, 0.27207E+01_JPRB, 0.24200E+01_JPRB, 0.22533E+01_JPRB, 0.27140E+01_JPRB, 0.22707E+01_JPRB, 0.23577E+01_JPRB ,&
&  0.28034E+01_JPRB, 0.26680E+01_JPRB, 0.21472E+01_JPRB, 0.17198E+01_JPRB, 0.22959E+01_JPRB, 0.25391E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.10066E+01_JPRB, 0.94315E+00_JPRB, 0.89596E+00_JPRB, 0.86187E+00_JPRB, 0.89887E+00_JPRB, 0.13772E+01_JPRB, 0.27451E+01_JPRB ,&
&  0.39965E+01_JPRB, 0.57658E+01_JPRB, 0.60271E+01_JPRB, 0.57704E+01_JPRB, 0.51280E+01_JPRB, 0.50520E+01_JPRB, 0.40401E+01_JPRB ,&
&  0.34412E+01_JPRB, 0.30457E+01_JPRB, 0.28217E+01_JPRB, 0.27083E+01_JPRB, 0.26006E+01_JPRB, 0.24900E+01_JPRB, 0.23378E+01_JPRB ,&
&  0.21480E+01_JPRB, 0.19854E+01_JPRB, 0.18286E+01_JPRB, 0.16295E+01_JPRB, 0.14677E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.13765E+01_JPRB, 0.13023E+01_JPRB, 0.12684E+01_JPRB, 0.12244E+01_JPRB, 0.11310E+01_JPRB, 0.10175E+01_JPRB, 0.92609E+00_JPRB ,&
&  0.87971E+00_JPRB, 0.85160E+00_JPRB, 0.82897E+00_JPRB, 0.82133E+00_JPRB, 0.82873E+00_JPRB, 0.86817E+00_JPRB, 0.93056E+00_JPRB ,&
&  0.10613E+01_JPRB, 0.12790E+01_JPRB, 0.15132E+01_JPRB, 0.19393E+01_JPRB, 0.21054E+01_JPRB, 0.19583E+01_JPRB, 0.21589E+01_JPRB ,&
&  0.21330E+01_JPRB, 0.20997E+01_JPRB, 0.18859E+01_JPRB, 0.16674E+01_JPRB, 0.15265E+01_JPRB, 0.14412E+01_JPRB, 0.13466E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.11400E+01_JPRB, 0.10613E+01_JPRB, 0.10535E+01_JPRB, 0.10526E+01_JPRB, 0.10507E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.10875E+01_JPRB, 0.12079E+01_JPRB, 0.13619E+01_JPRB, 0.14750E+01_JPRB, 0.15656E+01_JPRB, 0.16742E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.17037E+01_JPRB, 0.16610E+01_JPRB, 0.15998E+01_JPRB, 0.13377E+01_JPRB, 0.15034E+01_JPRB, 0.15625E+01_JPRB ,&
&  0.17098E+01_JPRB, 0.18990E+01_JPRB, 0.19639E+01_JPRB, 0.21215E+01_JPRB, 0.19325E+01_JPRB, 0.17909E+01_JPRB, 0.17555E+01_JPRB ,&
&  0.21953E+01_JPRB, 0.22839E+01_JPRB, 0.20571E+01_JPRB, 0.22191E+01_JPRB, 0.26830E+01_JPRB, 0.32947E+01_JPRB, 0.21946E+01_JPRB ,&
&  0.19759E+01_JPRB, 0.25428E+01_JPRB, 0.22255E+01_JPRB, 0.20303E+01_JPRB, 0.25739E+01_JPRB, 0.36186E+01_JPRB, 0.26403E+01_JPRB ,&
&  0.12558E+01_JPRB, 0.10183E+01_JPRB, 0.10695E+01_JPRB, 0.11627E+01_JPRB, 0.11770E+01_JPRB, 0.18514E+01_JPRB, 0.53746E+01_JPRB ,&
&  0.57097E+01_JPRB, 0.68266E+01_JPRB, 0.62735E+01_JPRB, 0.55472E+01_JPRB, 0.51184E+01_JPRB, 0.47308E+01_JPRB, 0.40017E+01_JPRB ,&
&  0.34523E+01_JPRB, 0.31191E+01_JPRB, 0.28621E+01_JPRB, 0.26753E+01_JPRB, 0.24896E+01_JPRB, 0.23046E+01_JPRB, 0.21382E+01_JPRB ,&
&  0.20215E+01_JPRB, 0.18990E+01_JPRB, 0.17828E+01_JPRB, 0.16228E+01_JPRB, 0.14799E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.13649E+01_JPRB, 0.13095E+01_JPRB, 0.12856E+01_JPRB, 0.12368E+01_JPRB, 0.11534E+01_JPRB, 0.10757E+01_JPRB, 0.96623E+00_JPRB ,&
&  0.89069E+00_JPRB, 0.84705E+00_JPRB, 0.82996E+00_JPRB, 0.83554E+00_JPRB, 0.86434E+00_JPRB, 0.91432E+00_JPRB, 0.97350E+00_JPRB ,&
&  0.11655E+01_JPRB, 0.13543E+01_JPRB, 0.16533E+01_JPRB, 0.20753E+01_JPRB, 0.23281E+01_JPRB, 0.21612E+01_JPRB, 0.23819E+01_JPRB ,&
&  0.28853E+01_JPRB, 0.25329E+01_JPRB, 0.21177E+01_JPRB, 0.18983E+01_JPRB, 0.17236E+01_JPRB, 0.16189E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.13076E+01_JPRB, 0.12558E+01_JPRB, 0.11848E+01_JPRB, 0.11735E+01_JPRB, 0.11375E+01_JPRB, 0.11464E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.11448E+01_JPRB, 0.12685E+01_JPRB, 0.13941E+01_JPRB, 0.15082E+01_JPRB, 0.16139E+01_JPRB, 0.17419E+01_JPRB ,&
&  0.18047E+01_JPRB, 0.17806E+01_JPRB, 0.17078E+01_JPRB, 0.15735E+01_JPRB, 0.15451E+01_JPRB, 0.15687E+01_JPRB, 0.16742E+01_JPRB ,&
&  0.17294E+01_JPRB, 0.18504E+01_JPRB, 0.19095E+01_JPRB, 0.19242E+01_JPRB, 0.18445E+01_JPRB, 0.17811E+01_JPRB, 0.17694E+01_JPRB ,&
&  0.20625E+01_JPRB, 0.20765E+01_JPRB, 0.18903E+01_JPRB, 0.22985E+01_JPRB, 0.28357E+01_JPRB, 0.32330E+01_JPRB, 0.30291E+01_JPRB ,&
&  0.26419E+01_JPRB, 0.22473E+01_JPRB, 0.21820E+01_JPRB, 0.25847E+01_JPRB, 0.30017E+01_JPRB, 0.36917E+01_JPRB, 0.49647E+01_JPRB ,&
&  0.52669E+01_JPRB, 0.30285E+01_JPRB, 0.25197E+01_JPRB, 0.34282E+01_JPRB, 0.29472E+01_JPRB, 0.30632E+01_JPRB, 0.46994E+01_JPRB ,&
&  0.59658E+01_JPRB, 0.67317E+01_JPRB, 0.63341E+01_JPRB, 0.54765E+01_JPRB, 0.50293E+01_JPRB, 0.43783E+01_JPRB, 0.38257E+01_JPRB ,&
&  0.32846E+01_JPRB, 0.28521E+01_JPRB, 0.25454E+01_JPRB, 0.23081E+01_JPRB, 0.20883E+01_JPRB, 0.19210E+01_JPRB, 0.18464E+01_JPRB ,&
&  0.17959E+01_JPRB, 0.17306E+01_JPRB, 0.16622E+01_JPRB, 0.15432E+01_JPRB, 0.14337E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.13260E+01_JPRB, 0.12885E+01_JPRB, 0.12627E+01_JPRB, 0.12158E+01_JPRB, 0.11963E+01_JPRB, 0.11668E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.98038E+00_JPRB, 0.86293E+00_JPRB, 0.85271E+00_JPRB, 0.87715E+00_JPRB, 0.91843E+00_JPRB, 0.97586E+00_JPRB, 0.10897E+01_JPRB ,&
&  0.12733E+01_JPRB, 0.14602E+01_JPRB, 0.18342E+01_JPRB, 0.23038E+01_JPRB, 0.27563E+01_JPRB, 0.35329E+01_JPRB, 0.24755E+01_JPRB ,&
&  0.25090E+01_JPRB, 0.24050E+01_JPRB, 0.20944E+01_JPRB, 0.20183E+01_JPRB, 0.18976E+01_JPRB, 0.18527E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.14354E+01_JPRB, 0.13799E+01_JPRB, 0.13054E+01_JPRB, 0.12325E+01_JPRB, 0.11932E+01_JPRB, 0.11727E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.12175E+01_JPRB, 0.13251E+01_JPRB, 0.14326E+01_JPRB, 0.15479E+01_JPRB, 0.16755E+01_JPRB, 0.17699E+01_JPRB ,&
&  0.18659E+01_JPRB, 0.18678E+01_JPRB, 0.17942E+01_JPRB, 0.16368E+01_JPRB, 0.16274E+01_JPRB, 0.16361E+01_JPRB, 0.16525E+01_JPRB ,&
&  0.15698E+01_JPRB, 0.15146E+01_JPRB, 0.16977E+01_JPRB, 0.18080E+01_JPRB, 0.17639E+01_JPRB, 0.16602E+01_JPRB, 0.16603E+01_JPRB ,&
&  0.19807E+01_JPRB, 0.22119E+01_JPRB, 0.24850E+01_JPRB, 0.26482E+01_JPRB, 0.30236E+01_JPRB, 0.32814E+01_JPRB, 0.34367E+01_JPRB ,&
&  0.35308E+01_JPRB, 0.31963E+01_JPRB, 0.31424E+01_JPRB, 0.33857E+01_JPRB, 0.35190E+01_JPRB, 0.40176E+01_JPRB, 0.47609E+01_JPRB ,&
&  0.57623E+01_JPRB, 0.72158E+01_JPRB, 0.74338E+01_JPRB, 0.60405E+01_JPRB, 0.53661E+01_JPRB, 0.46591E+01_JPRB, 0.51381E+01_JPRB ,&
&  0.60725E+01_JPRB, 0.58665E+01_JPRB, 0.55585E+01_JPRB, 0.56899E+01_JPRB, 0.43246E+01_JPRB, 0.36757E+01_JPRB, 0.32367E+01_JPRB ,&
&  0.29114E+01_JPRB, 0.25167E+01_JPRB, 0.21996E+01_JPRB, 0.19512E+01_JPRB, 0.17222E+01_JPRB, 0.16203E+01_JPRB, 0.16145E+01_JPRB ,&
&  0.15955E+01_JPRB, 0.15722E+01_JPRB, 0.15308E+01_JPRB, 0.14513E+01_JPRB, 0.13826E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.12882E+01_JPRB, 0.12455E+01_JPRB, 0.12201E+01_JPRB, 0.11929E+01_JPRB, 0.12247E+01_JPRB, 0.12246E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.11742E+01_JPRB, 0.92165E+00_JPRB, 0.90261E+00_JPRB, 0.93017E+00_JPRB, 0.98425E+00_JPRB, 0.10632E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.13475E+01_JPRB, 0.15568E+01_JPRB, 0.19730E+01_JPRB, 0.25015E+01_JPRB, 0.29751E+01_JPRB, 0.33518E+01_JPRB, 0.33792E+01_JPRB ,&
&  0.21114E+01_JPRB, 0.24705E+01_JPRB, 0.21013E+01_JPRB, 0.21803E+01_JPRB, 0.21893E+01_JPRB, 0.21233E+01_JPRB, 0.17593E+01_JPRB ,&
&  0.15304E+01_JPRB, 0.14532E+01_JPRB, 0.14009E+01_JPRB, 0.13826E+01_JPRB, 0.13008E+01_JPRB, 0.12176E+01_JPRB, 0.11956E+01_JPRB ,&
&  0.12413E+01_JPRB, 0.13403E+01_JPRB, 0.14521E+01_JPRB, 0.15470E+01_JPRB, 0.16682E+01_JPRB, 0.17829E+01_JPRB, 0.18618E+01_JPRB ,&
&  0.19499E+01_JPRB, 0.19611E+01_JPRB, 0.18324E+01_JPRB, 0.17590E+01_JPRB, 0.17195E+01_JPRB, 0.17262E+01_JPRB, 0.17068E+01_JPRB ,&
&  0.16017E+01_JPRB, 0.15197E+01_JPRB, 0.16224E+01_JPRB, 0.16245E+01_JPRB, 0.15708E+01_JPRB, 0.16467E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.21417E+01_JPRB, 0.23631E+01_JPRB, 0.31390E+01_JPRB, 0.29132E+01_JPRB, 0.29743E+01_JPRB, 0.32959E+01_JPRB, 0.32582E+01_JPRB ,&
&  0.30110E+01_JPRB, 0.32283E+01_JPRB, 0.33537E+01_JPRB, 0.38741E+01_JPRB, 0.43229E+01_JPRB, 0.48576E+01_JPRB, 0.51368E+01_JPRB ,&
&  0.56740E+01_JPRB, 0.63523E+01_JPRB, 0.71079E+01_JPRB, 0.57715E+01_JPRB, 0.52235E+01_JPRB, 0.49316E+01_JPRB, 0.53602E+01_JPRB ,&
&  0.57134E+01_JPRB, 0.54010E+01_JPRB, 0.53480E+01_JPRB, 0.48157E+01_JPRB, 0.35985E+01_JPRB, 0.31249E+01_JPRB, 0.27398E+01_JPRB ,&
&  0.24424E+01_JPRB, 0.21245E+01_JPRB, 0.18620E+01_JPRB, 0.16468E+01_JPRB, 0.15170E+01_JPRB, 0.14887E+01_JPRB, 0.14971E+01_JPRB ,&
&  0.14924E+01_JPRB, 0.14797E+01_JPRB, 0.14560E+01_JPRB, 0.13943E+01_JPRB, 0.13361E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.12682E+01_JPRB, 0.12446E+01_JPRB, 0.12301E+01_JPRB, 0.11854E+01_JPRB, 0.12211E+01_JPRB, 0.13200E+01_JPRB, 0.13921E+01_JPRB ,&
&  0.13398E+01_JPRB, 0.96515E+00_JPRB, 0.95543E+00_JPRB, 0.98529E+00_JPRB, 0.10378E+01_JPRB, 0.11437E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.14227E+01_JPRB, 0.16800E+01_JPRB, 0.21360E+01_JPRB, 0.26734E+01_JPRB, 0.31207E+01_JPRB, 0.36521E+01_JPRB, 0.38028E+01_JPRB ,&
&  0.26428E+01_JPRB, 0.20884E+01_JPRB, 0.20655E+01_JPRB, 0.22075E+01_JPRB, 0.21453E+01_JPRB, 0.22500E+01_JPRB, 0.22277E+01_JPRB ,&
&  0.19897E+01_JPRB, 0.17188E+01_JPRB, 0.17091E+01_JPRB, 0.16771E+01_JPRB, 0.13597E+01_JPRB, 0.13349E+01_JPRB, 0.13470E+01_JPRB ,&
&  0.14105E+01_JPRB, 0.15032E+01_JPRB, 0.16042E+01_JPRB, 0.16921E+01_JPRB, 0.18155E+01_JPRB, 0.19083E+01_JPRB, 0.19937E+01_JPRB ,&
&  0.20339E+01_JPRB, 0.20592E+01_JPRB, 0.19607E+01_JPRB, 0.18428E+01_JPRB, 0.18030E+01_JPRB, 0.17720E+01_JPRB, 0.17110E+01_JPRB ,&
&  0.16557E+01_JPRB, 0.16795E+01_JPRB, 0.16951E+01_JPRB, 0.16508E+01_JPRB, 0.16176E+01_JPRB, 0.18463E+01_JPRB, 0.20631E+01_JPRB ,&
&  0.23702E+01_JPRB, 0.27730E+01_JPRB, 0.36214E+01_JPRB, 0.28998E+01_JPRB, 0.29419E+01_JPRB, 0.31501E+01_JPRB, 0.31895E+01_JPRB ,&
&  0.31257E+01_JPRB, 0.33896E+01_JPRB, 0.37541E+01_JPRB, 0.42508E+01_JPRB, 0.48050E+01_JPRB, 0.49446E+01_JPRB, 0.61562E+01_JPRB ,&
&  0.58250E+01_JPRB, 0.67044E+01_JPRB, 0.60599E+01_JPRB, 0.57400E+01_JPRB, 0.48464E+01_JPRB, 0.48025E+01_JPRB, 0.47057E+01_JPRB ,&
&  0.48691E+01_JPRB, 0.45827E+01_JPRB, 0.41357E+01_JPRB, 0.36598E+01_JPRB, 0.28636E+01_JPRB, 0.25375E+01_JPRB, 0.22100E+01_JPRB ,&
&  0.19164E+01_JPRB, 0.17094E+01_JPRB, 0.16106E+01_JPRB, 0.15356E+01_JPRB, 0.15070E+01_JPRB, 0.14954E+01_JPRB, 0.14866E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.14425E+01_JPRB, 0.14137E+01_JPRB, 0.13546E+01_JPRB, 0.13052E+01_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.12481E+01_JPRB, 0.12207E+01_JPRB, 0.11844E+01_JPRB, 0.10994E+01_JPRB, 0.10488E+01_JPRB, 0.10498E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.10191E+01_JPRB, 0.99080E+00_JPRB, 0.10077E+01_JPRB, 0.10554E+01_JPRB, 0.11204E+01_JPRB, 0.12293E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.15663E+01_JPRB, 0.17849E+01_JPRB, 0.21276E+01_JPRB, 0.25058E+01_JPRB, 0.28362E+01_JPRB, 0.30979E+01_JPRB, 0.32464E+01_JPRB ,&
&  0.26228E+01_JPRB, 0.18433E+01_JPRB, 0.22242E+01_JPRB, 0.24044E+01_JPRB, 0.24001E+01_JPRB, 0.25928E+01_JPRB, 0.25363E+01_JPRB ,&
&  0.21168E+01_JPRB, 0.19780E+01_JPRB, 0.20179E+01_JPRB, 0.18376E+01_JPRB, 0.14596E+01_JPRB, 0.14763E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.16095E+01_JPRB, 0.16755E+01_JPRB, 0.17612E+01_JPRB, 0.18535E+01_JPRB, 0.19491E+01_JPRB, 0.20722E+01_JPRB, 0.21557E+01_JPRB ,&
&  0.21775E+01_JPRB, 0.21730E+01_JPRB, 0.21310E+01_JPRB, 0.19922E+01_JPRB, 0.18926E+01_JPRB, 0.18383E+01_JPRB, 0.17764E+01_JPRB ,&
&  0.17609E+01_JPRB, 0.17471E+01_JPRB, 0.18150E+01_JPRB, 0.19699E+01_JPRB, 0.22202E+01_JPRB, 0.22018E+01_JPRB, 0.24817E+01_JPRB ,&
&  0.27051E+01_JPRB, 0.32033E+01_JPRB, 0.33641E+01_JPRB, 0.37914E+01_JPRB, 0.24548E+01_JPRB, 0.28085E+01_JPRB, 0.27236E+01_JPRB ,&
&  0.34325E+01_JPRB, 0.36162E+01_JPRB, 0.41097E+01_JPRB, 0.47268E+01_JPRB, 0.55998E+01_JPRB, 0.57292E+01_JPRB, 0.61309E+01_JPRB ,&
&  0.59360E+01_JPRB, 0.56235E+01_JPRB, 0.50075E+01_JPRB, 0.49420E+01_JPRB, 0.46802E+01_JPRB, 0.49963E+01_JPRB, 0.42177E+01_JPRB ,&
&  0.41133E+01_JPRB, 0.37674E+01_JPRB, 0.34170E+01_JPRB, 0.30799E+01_JPRB, 0.23896E+01_JPRB, 0.19667E+01_JPRB, 0.17078E+01_JPRB ,&
&  0.15892E+01_JPRB, 0.15366E+01_JPRB, 0.14966E+01_JPRB, 0.14746E+01_JPRB, 0.14781E+01_JPRB, 0.14807E+01_JPRB, 0.14754E+01_JPRB ,&
&  0.14551E+01_JPRB, 0.14246E+01_JPRB, 0.13880E+01_JPRB, 0.13352E+01_JPRB, 0.12906E+01_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.12114E+01_JPRB, 0.11757E+01_JPRB, 0.11234E+01_JPRB, 0.10585E+01_JPRB, 0.10261E+01_JPRB, 0.10064E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.10093E+01_JPRB, 0.10350E+01_JPRB, 0.10585E+01_JPRB, 0.10982E+01_JPRB, 0.11511E+01_JPRB, 0.12454E+01_JPRB, 0.13375E+01_JPRB ,&
&  0.14526E+01_JPRB, 0.16193E+01_JPRB, 0.19176E+01_JPRB, 0.22888E+01_JPRB, 0.25999E+01_JPRB, 0.27726E+01_JPRB, 0.26390E+01_JPRB ,&
&  0.25202E+01_JPRB, 0.24731E+01_JPRB, 0.23051E+01_JPRB, 0.18910E+01_JPRB, 0.18953E+01_JPRB, 0.23957E+01_JPRB, 0.21653E+01_JPRB ,&
&  0.18890E+01_JPRB, 0.18169E+01_JPRB, 0.17485E+01_JPRB, 0.16708E+01_JPRB, 0.15973E+01_JPRB, 0.16173E+01_JPRB, 0.16732E+01_JPRB ,&
&  0.17400E+01_JPRB, 0.18089E+01_JPRB, 0.19099E+01_JPRB, 0.20119E+01_JPRB, 0.21210E+01_JPRB, 0.22754E+01_JPRB, 0.23602E+01_JPRB ,&
&  0.23564E+01_JPRB, 0.23402E+01_JPRB, 0.22840E+01_JPRB, 0.21578E+01_JPRB, 0.20070E+01_JPRB, 0.19803E+01_JPRB, 0.19985E+01_JPRB ,&
&  0.20335E+01_JPRB, 0.20895E+01_JPRB, 0.21695E+01_JPRB, 0.24070E+01_JPRB, 0.26280E+01_JPRB, 0.24947E+01_JPRB, 0.25490E+01_JPRB ,&
&  0.29422E+01_JPRB, 0.34181E+01_JPRB, 0.31029E+01_JPRB, 0.36586E+01_JPRB, 0.21687E+01_JPRB, 0.26615E+01_JPRB, 0.32737E+01_JPRB ,&
&  0.34430E+01_JPRB, 0.38576E+01_JPRB, 0.44656E+01_JPRB, 0.52088E+01_JPRB, 0.60280E+01_JPRB, 0.57348E+01_JPRB, 0.56736E+01_JPRB ,&
&  0.50982E+01_JPRB, 0.43550E+01_JPRB, 0.41593E+01_JPRB, 0.40540E+01_JPRB, 0.42557E+01_JPRB, 0.43266E+01_JPRB, 0.33214E+01_JPRB ,&
&  0.28038E+01_JPRB, 0.30916E+01_JPRB, 0.30880E+01_JPRB, 0.28376E+01_JPRB, 0.20189E+01_JPRB, 0.15728E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.14675E+01_JPRB, 0.14379E+01_JPRB, 0.14301E+01_JPRB, 0.14393E+01_JPRB, 0.14577E+01_JPRB, 0.14628E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.14058E+01_JPRB, 0.13641E+01_JPRB, 0.13120E+01_JPRB, 0.12596E+01_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.11617E+01_JPRB, 0.11229E+01_JPRB, 0.10656E+01_JPRB, 0.10123E+01_JPRB, 0.94890E+00_JPRB, 0.91805E+00_JPRB, 0.91668E+00_JPRB ,&
&  0.94256E+00_JPRB, 0.98042E+00_JPRB, 0.10081E+01_JPRB, 0.10176E+01_JPRB, 0.10513E+01_JPRB, 0.10733E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.11856E+01_JPRB, 0.13159E+01_JPRB, 0.15618E+01_JPRB, 0.19538E+01_JPRB, 0.23414E+01_JPRB, 0.27080E+01_JPRB, 0.26032E+01_JPRB ,&
&  0.24791E+01_JPRB, 0.23922E+01_JPRB, 0.23533E+01_JPRB, 0.23486E+01_JPRB, 0.21040E+01_JPRB, 0.23328E+01_JPRB, 0.23347E+01_JPRB ,&
&  0.23294E+01_JPRB, 0.21522E+01_JPRB, 0.18391E+01_JPRB, 0.17643E+01_JPRB, 0.17301E+01_JPRB, 0.17459E+01_JPRB, 0.18002E+01_JPRB ,&
&  0.18675E+01_JPRB, 0.19385E+01_JPRB, 0.20554E+01_JPRB, 0.22003E+01_JPRB, 0.23742E+01_JPRB, 0.25344E+01_JPRB, 0.25989E+01_JPRB ,&
&  0.25883E+01_JPRB, 0.25705E+01_JPRB, 0.25274E+01_JPRB, 0.23919E+01_JPRB, 0.23335E+01_JPRB, 0.23691E+01_JPRB, 0.24482E+01_JPRB ,&
&  0.25554E+01_JPRB, 0.25187E+01_JPRB, 0.26397E+01_JPRB, 0.29155E+01_JPRB, 0.29907E+01_JPRB, 0.29160E+01_JPRB, 0.29381E+01_JPRB ,&
&  0.32038E+01_JPRB, 0.35837E+01_JPRB, 0.25522E+01_JPRB, 0.30957E+01_JPRB, 0.37487E+01_JPRB, 0.33014E+01_JPRB, 0.33144E+01_JPRB ,&
&  0.36911E+01_JPRB, 0.41096E+01_JPRB, 0.46876E+01_JPRB, 0.53389E+01_JPRB, 0.60850E+01_JPRB, 0.66739E+01_JPRB, 0.46908E+01_JPRB ,&
&  0.41207E+01_JPRB, 0.34619E+01_JPRB, 0.33346E+01_JPRB, 0.33864E+01_JPRB, 0.35259E+01_JPRB, 0.33671E+01_JPRB, 0.25507E+01_JPRB ,&
&  0.23221E+01_JPRB, 0.25221E+01_JPRB, 0.24216E+01_JPRB, 0.23423E+01_JPRB, 0.20802E+01_JPRB, 0.13730E+01_JPRB, 0.13938E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.13824E+01_JPRB, 0.13858E+01_JPRB, 0.13986E+01_JPRB, 0.14146E+01_JPRB, 0.14198E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.13922E+01_JPRB, 0.13483E+01_JPRB, 0.13008E+01_JPRB, 0.12509E+01_JPRB, 0.12037E+01_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.11185E+01_JPRB, 0.10697E+01_JPRB, 0.10049E+01_JPRB, 0.93423E+00_JPRB, 0.87122E+00_JPRB, 0.84721E+00_JPRB, 0.83851E+00_JPRB ,&
&  0.84872E+00_JPRB, 0.88163E+00_JPRB, 0.90270E+00_JPRB, 0.90180E+00_JPRB, 0.89780E+00_JPRB, 0.91376E+00_JPRB, 0.96781E+00_JPRB ,&
&  0.10363E+01_JPRB, 0.11216E+01_JPRB, 0.12954E+01_JPRB, 0.16696E+01_JPRB, 0.21565E+01_JPRB, 0.25727E+01_JPRB, 0.25857E+01_JPRB ,&
&  0.24677E+01_JPRB, 0.24874E+01_JPRB, 0.25148E+01_JPRB, 0.25923E+01_JPRB, 0.26652E+01_JPRB, 0.28124E+01_JPRB, 0.29840E+01_JPRB ,&
&  0.27134E+01_JPRB, 0.21825E+01_JPRB, 0.19494E+01_JPRB, 0.18120E+01_JPRB, 0.18078E+01_JPRB, 0.18564E+01_JPRB, 0.19143E+01_JPRB ,&
&  0.19854E+01_JPRB, 0.20926E+01_JPRB, 0.22427E+01_JPRB, 0.24694E+01_JPRB, 0.27045E+01_JPRB, 0.28425E+01_JPRB, 0.29080E+01_JPRB ,&
&  0.29433E+01_JPRB, 0.29663E+01_JPRB, 0.30308E+01_JPRB, 0.27621E+01_JPRB, 0.28661E+01_JPRB, 0.29568E+01_JPRB, 0.30408E+01_JPRB ,&
&  0.30254E+01_JPRB, 0.29413E+01_JPRB, 0.31621E+01_JPRB, 0.33289E+01_JPRB, 0.34954E+01_JPRB, 0.35000E+01_JPRB, 0.34406E+01_JPRB ,&
&  0.35979E+01_JPRB, 0.33371E+01_JPRB, 0.23107E+01_JPRB, 0.23322E+01_JPRB, 0.25922E+01_JPRB, 0.25255E+01_JPRB, 0.33430E+01_JPRB ,&
&  0.35734E+01_JPRB, 0.38558E+01_JPRB, 0.41874E+01_JPRB, 0.45189E+01_JPRB, 0.48712E+01_JPRB, 0.50560E+01_JPRB, 0.38852E+01_JPRB ,&
&  0.31466E+01_JPRB, 0.27876E+01_JPRB, 0.27069E+01_JPRB, 0.26398E+01_JPRB, 0.25736E+01_JPRB, 0.23594E+01_JPRB, 0.22210E+01_JPRB ,&
&  0.21692E+01_JPRB, 0.21140E+01_JPRB, 0.19898E+01_JPRB, 0.17502E+01_JPRB, 0.15081E+01_JPRB, 0.13070E+01_JPRB, 0.13386E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.13269E+01_JPRB, 0.13309E+01_JPRB, 0.13341E+01_JPRB, 0.13425E+01_JPRB, 0.13396E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.12947E+01_JPRB, 0.12603E+01_JPRB, 0.12258E+01_JPRB, 0.11874E+01_JPRB, 0.11497E+01_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.10736E+01_JPRB, 0.10127E+01_JPRB, 0.93643E+00_JPRB, 0.87341E+00_JPRB, 0.86330E+00_JPRB, 0.86703E+00_JPRB, 0.85130E+00_JPRB ,&
&  0.84594E+00_JPRB, 0.84325E+00_JPRB, 0.86647E+00_JPRB, 0.87291E+00_JPRB, 0.87933E+00_JPRB, 0.89938E+00_JPRB, 0.92567E+00_JPRB ,&
&  0.94611E+00_JPRB, 0.96239E+00_JPRB, 0.10474E+01_JPRB, 0.12367E+01_JPRB, 0.16018E+01_JPRB, 0.20961E+01_JPRB, 0.24823E+01_JPRB ,&
&  0.27263E+01_JPRB, 0.29348E+01_JPRB, 0.31398E+01_JPRB, 0.33317E+01_JPRB, 0.32330E+01_JPRB, 0.34294E+01_JPRB, 0.38848E+01_JPRB ,&
&  0.29082E+01_JPRB, 0.19868E+01_JPRB, 0.19941E+01_JPRB, 0.17956E+01_JPRB, 0.18180E+01_JPRB, 0.19210E+01_JPRB, 0.20073E+01_JPRB ,&
&  0.21217E+01_JPRB, 0.22583E+01_JPRB, 0.24235E+01_JPRB, 0.26483E+01_JPRB, 0.28787E+01_JPRB, 0.30412E+01_JPRB, 0.31651E+01_JPRB ,&
&  0.32684E+01_JPRB, 0.33832E+01_JPRB, 0.35506E+01_JPRB, 0.35294E+01_JPRB, 0.34709E+01_JPRB, 0.36011E+01_JPRB, 0.36683E+01_JPRB ,&
&  0.37568E+01_JPRB, 0.37726E+01_JPRB, 0.33504E+01_JPRB, 0.34760E+01_JPRB, 0.39009E+01_JPRB, 0.37212E+01_JPRB, 0.36999E+01_JPRB ,&
&  0.39617E+01_JPRB, 0.34009E+01_JPRB, 0.21517E+01_JPRB, 0.21169E+01_JPRB, 0.24072E+01_JPRB, 0.27810E+01_JPRB, 0.31951E+01_JPRB ,&
&  0.34569E+01_JPRB, 0.36523E+01_JPRB, 0.36950E+01_JPRB, 0.37372E+01_JPRB, 0.35631E+01_JPRB, 0.33111E+01_JPRB, 0.30132E+01_JPRB ,&
&  0.25480E+01_JPRB, 0.24036E+01_JPRB, 0.22859E+01_JPRB, 0.22365E+01_JPRB, 0.22572E+01_JPRB, 0.21515E+01_JPRB, 0.20177E+01_JPRB ,&
&  0.21831E+01_JPRB, 0.23081E+01_JPRB, 0.25486E+01_JPRB, 0.19379E+01_JPRB, 0.14244E+01_JPRB, 0.12859E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.13150E+01_JPRB, 0.13052E+01_JPRB, 0.13005E+01_JPRB, 0.12937E+01_JPRB, 0.12891E+01_JPRB, 0.12676E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.11972E+01_JPRB, 0.11715E+01_JPRB, 0.11566E+01_JPRB, 0.11355E+01_JPRB, 0.11031E+01_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.99688E+00_JPRB, 0.91299E+00_JPRB, 0.86461E+00_JPRB, 0.90755E+00_JPRB, 0.88100E+00_JPRB, 0.91070E+00_JPRB, 0.90316E+00_JPRB ,&
&  0.90855E+00_JPRB, 0.91676E+00_JPRB, 0.94355E+00_JPRB, 0.98021E+00_JPRB, 0.10185E+01_JPRB, 0.10488E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.10944E+01_JPRB, 0.11136E+01_JPRB, 0.11554E+01_JPRB, 0.12434E+01_JPRB, 0.14002E+01_JPRB, 0.17036E+01_JPRB ,&
&  0.20267E+01_JPRB, 0.22277E+01_JPRB, 0.23316E+01_JPRB, 0.23873E+01_JPRB, 0.27011E+01_JPRB, 0.33802E+01_JPRB, 0.39413E+01_JPRB ,&
&  0.23759E+01_JPRB, 0.20790E+01_JPRB, 0.18008E+01_JPRB, 0.16267E+01_JPRB, 0.16935E+01_JPRB, 0.18901E+01_JPRB, 0.19976E+01_JPRB ,&
&  0.20729E+01_JPRB, 0.21986E+01_JPRB, 0.23540E+01_JPRB, 0.25440E+01_JPRB, 0.27094E+01_JPRB, 0.28726E+01_JPRB, 0.30466E+01_JPRB ,&
&  0.32387E+01_JPRB, 0.34889E+01_JPRB, 0.37492E+01_JPRB, 0.40149E+01_JPRB, 0.40111E+01_JPRB, 0.44753E+01_JPRB, 0.45685E+01_JPRB ,&
&  0.46276E+01_JPRB, 0.44251E+01_JPRB, 0.38725E+01_JPRB, 0.39587E+01_JPRB, 0.42241E+01_JPRB, 0.41954E+01_JPRB, 0.39355E+01_JPRB ,&
&  0.39763E+01_JPRB, 0.32238E+01_JPRB, 0.23644E+01_JPRB, 0.24175E+01_JPRB, 0.25360E+01_JPRB, 0.27642E+01_JPRB, 0.27698E+01_JPRB ,&
&  0.29838E+01_JPRB, 0.30239E+01_JPRB, 0.29523E+01_JPRB, 0.27858E+01_JPRB, 0.25827E+01_JPRB, 0.24694E+01_JPRB, 0.24337E+01_JPRB ,&
&  0.22248E+01_JPRB, 0.21545E+01_JPRB, 0.21112E+01_JPRB, 0.21946E+01_JPRB, 0.23288E+01_JPRB, 0.19605E+01_JPRB, 0.19525E+01_JPRB ,&
&  0.20843E+01_JPRB, 0.22476E+01_JPRB, 0.19558E+01_JPRB, 0.21138E+01_JPRB, 0.13251E+01_JPRB, 0.11803E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.12173E+01_JPRB, 0.12182E+01_JPRB, 0.12096E+01_JPRB, 0.12119E+01_JPRB, 0.11957E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10795E+01_JPRB, 0.10793E+01_JPRB, 0.10694E+01_JPRB, 0.10520E+01_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.90067E+00_JPRB, 0.85063E+00_JPRB, 0.90431E+00_JPRB, 0.87985E+00_JPRB, 0.88440E+00_JPRB, 0.90837E+00_JPRB, 0.93271E+00_JPRB ,&
&  0.94987E+00_JPRB, 0.97767E+00_JPRB, 0.10219E+01_JPRB, 0.10831E+01_JPRB, 0.11395E+01_JPRB, 0.11835E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.12719E+01_JPRB, 0.12959E+01_JPRB, 0.13110E+01_JPRB, 0.13246E+01_JPRB, 0.13626E+01_JPRB, 0.14088E+01_JPRB, 0.14881E+01_JPRB ,&
&  0.16016E+01_JPRB, 0.16464E+01_JPRB, 0.16665E+01_JPRB, 0.17390E+01_JPRB, 0.18283E+01_JPRB, 0.20602E+01_JPRB, 0.26132E+01_JPRB ,&
&  0.20540E+01_JPRB, 0.20262E+01_JPRB, 0.18464E+01_JPRB, 0.17373E+01_JPRB, 0.17818E+01_JPRB, 0.17383E+01_JPRB, 0.17950E+01_JPRB ,&
&  0.19300E+01_JPRB, 0.19676E+01_JPRB, 0.20496E+01_JPRB, 0.21199E+01_JPRB, 0.22184E+01_JPRB, 0.23550E+01_JPRB, 0.24973E+01_JPRB ,&
&  0.26859E+01_JPRB, 0.29117E+01_JPRB, 0.30886E+01_JPRB, 0.33727E+01_JPRB, 0.36572E+01_JPRB, 0.39569E+01_JPRB, 0.41012E+01_JPRB ,&
&  0.41916E+01_JPRB, 0.42425E+01_JPRB, 0.39472E+01_JPRB, 0.39865E+01_JPRB, 0.39954E+01_JPRB, 0.38128E+01_JPRB, 0.33858E+01_JPRB ,&
&  0.29848E+01_JPRB, 0.27560E+01_JPRB, 0.26344E+01_JPRB, 0.25231E+01_JPRB, 0.25088E+01_JPRB, 0.24513E+01_JPRB, 0.24552E+01_JPRB ,&
&  0.25135E+01_JPRB, 0.24034E+01_JPRB, 0.22575E+01_JPRB, 0.20597E+01_JPRB, 0.19633E+01_JPRB, 0.19404E+01_JPRB, 0.19786E+01_JPRB ,&
&  0.19361E+01_JPRB, 0.19083E+01_JPRB, 0.20097E+01_JPRB, 0.22989E+01_JPRB, 0.26710E+01_JPRB, 0.34700E+01_JPRB, 0.25729E+01_JPRB ,&
&  0.18834E+01_JPRB, 0.17420E+01_JPRB, 0.11650E+01_JPRB, 0.14640E+01_JPRB, 0.10963E+01_JPRB, 0.10330E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.10999E+01_JPRB, 0.11504E+01_JPRB, 0.11192E+01_JPRB, 0.10890E+01_JPRB, 0.10745E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.97732E+00_JPRB, 0.91748E+00_JPRB, 0.93112E+00_JPRB, 0.87930E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.76146E+00_JPRB, 0.78650E+00_JPRB, 0.89620E+00_JPRB, 0.84636E+00_JPRB, 0.85500E+00_JPRB, 0.88436E+00_JPRB, 0.92522E+00_JPRB ,&
&  0.96959E+00_JPRB, 0.10238E+01_JPRB, 0.10812E+01_JPRB, 0.11568E+01_JPRB, 0.12244E+01_JPRB, 0.12829E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.14346E+01_JPRB, 0.14783E+01_JPRB, 0.15060E+01_JPRB, 0.15215E+01_JPRB, 0.15649E+01_JPRB, 0.16196E+01_JPRB ,&
&  0.17314E+01_JPRB, 0.17928E+01_JPRB, 0.18655E+01_JPRB, 0.19800E+01_JPRB, 0.20766E+01_JPRB, 0.22343E+01_JPRB, 0.20756E+01_JPRB ,&
&  0.23624E+01_JPRB, 0.20016E+01_JPRB, 0.18759E+01_JPRB, 0.18387E+01_JPRB, 0.17744E+01_JPRB, 0.17182E+01_JPRB, 0.17110E+01_JPRB ,&
&  0.17707E+01_JPRB, 0.17142E+01_JPRB, 0.17301E+01_JPRB, 0.17149E+01_JPRB, 0.17681E+01_JPRB, 0.18889E+01_JPRB, 0.19731E+01_JPRB ,&
&  0.21333E+01_JPRB, 0.23308E+01_JPRB, 0.25428E+01_JPRB, 0.27045E+01_JPRB, 0.28496E+01_JPRB, 0.28582E+01_JPRB, 0.28553E+01_JPRB ,&
&  0.29269E+01_JPRB, 0.30911E+01_JPRB, 0.30461E+01_JPRB, 0.31284E+01_JPRB, 0.31176E+01_JPRB, 0.28682E+01_JPRB, 0.22892E+01_JPRB ,&
&  0.20405E+01_JPRB, 0.22697E+01_JPRB, 0.22189E+01_JPRB, 0.26576E+01_JPRB, 0.24119E+01_JPRB, 0.21753E+01_JPRB, 0.21768E+01_JPRB ,&
&  0.20925E+01_JPRB, 0.19150E+01_JPRB, 0.17721E+01_JPRB, 0.15663E+01_JPRB, 0.15286E+01_JPRB, 0.15424E+01_JPRB, 0.15964E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.15656E+01_JPRB, 0.16393E+01_JPRB, 0.19613E+01_JPRB, 0.28721E+01_JPRB, 0.29642E+01_JPRB, 0.31588E+01_JPRB ,&
&  0.18033E+01_JPRB, 0.10858E+01_JPRB, 0.11358E+01_JPRB, 0.10573E+01_JPRB, 0.94118E+00_JPRB, 0.86308E+00_JPRB, 0.92112E+00_JPRB ,&
&  0.10017E+01_JPRB, 0.10050E+01_JPRB, 0.98359E+00_JPRB, 0.99237E+00_JPRB, 0.99170E+00_JPRB, 0.99874E+00_JPRB, 0.92270E+00_JPRB ,&
&  0.90467E+00_JPRB, 0.84582E+00_JPRB, 0.81016E+00_JPRB, 0.79399E+00_JPRB, 0.75204E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.70090E+00_JPRB, 0.77872E+00_JPRB, 0.86191E+00_JPRB, 0.81388E+00_JPRB, 0.84044E+00_JPRB, 0.88215E+00_JPRB, 0.93595E+00_JPRB ,&
&  0.99496E+00_JPRB, 0.10610E+01_JPRB, 0.11289E+01_JPRB, 0.12068E+01_JPRB, 0.12709E+01_JPRB, 0.13316E+01_JPRB, 0.13955E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.14980E+01_JPRB, 0.15563E+01_JPRB, 0.15900E+01_JPRB, 0.16308E+01_JPRB, 0.16641E+01_JPRB, 0.17046E+01_JPRB ,&
&  0.18276E+01_JPRB, 0.19221E+01_JPRB, 0.20320E+01_JPRB, 0.21494E+01_JPRB, 0.22323E+01_JPRB, 0.25113E+01_JPRB, 0.21296E+01_JPRB ,&
&  0.23926E+01_JPRB, 0.20107E+01_JPRB, 0.18654E+01_JPRB, 0.18722E+01_JPRB, 0.18178E+01_JPRB, 0.17932E+01_JPRB, 0.17837E+01_JPRB ,&
&  0.17795E+01_JPRB, 0.17930E+01_JPRB, 0.17841E+01_JPRB, 0.17496E+01_JPRB, 0.15900E+01_JPRB, 0.16615E+01_JPRB, 0.17460E+01_JPRB ,&
&  0.18530E+01_JPRB, 0.19892E+01_JPRB, 0.20928E+01_JPRB, 0.21574E+01_JPRB, 0.21912E+01_JPRB, 0.21470E+01_JPRB, 0.21007E+01_JPRB ,&
&  0.20459E+01_JPRB, 0.20696E+01_JPRB, 0.19633E+01_JPRB, 0.20919E+01_JPRB, 0.22984E+01_JPRB, 0.19608E+01_JPRB, 0.15156E+01_JPRB ,&
&  0.13373E+01_JPRB, 0.17373E+01_JPRB, 0.21952E+01_JPRB, 0.25786E+01_JPRB, 0.22197E+01_JPRB, 0.20185E+01_JPRB, 0.19149E+01_JPRB ,&
&  0.17555E+01_JPRB, 0.15662E+01_JPRB, 0.14026E+01_JPRB, 0.12395E+01_JPRB, 0.11812E+01_JPRB, 0.11712E+01_JPRB, 0.12043E+01_JPRB ,&
&  0.11935E+01_JPRB, 0.11974E+01_JPRB, 0.12573E+01_JPRB, 0.13956E+01_JPRB, 0.18410E+01_JPRB, 0.21716E+01_JPRB, 0.38393E+01_JPRB ,&
&  0.19413E+01_JPRB, 0.13241E+01_JPRB, 0.20008E+01_JPRB, 0.67146E+00_JPRB, 0.72914E+00_JPRB, 0.75127E+00_JPRB, 0.78569E+00_JPRB ,&
&  0.73372E+00_JPRB, 0.92791E+00_JPRB, 0.12706E+01_JPRB, 0.12785E+01_JPRB, 0.13303E+01_JPRB, 0.14042E+01_JPRB, 0.88864E+00_JPRB ,&
&  0.74990E+00_JPRB, 0.74786E+00_JPRB, 0.75620E+00_JPRB, 0.70414E+00_JPRB, 0.69415E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.65160E+00_JPRB, 0.71610E+00_JPRB, 0.80052E+00_JPRB, 0.86980E+00_JPRB, 0.87492E+00_JPRB, 0.89725E+00_JPRB, 0.93558E+00_JPRB ,&
&  0.97712E+00_JPRB, 0.10241E+01_JPRB, 0.10696E+01_JPRB, 0.11263E+01_JPRB, 0.11681E+01_JPRB, 0.12217E+01_JPRB, 0.13003E+01_JPRB ,&
&  0.14056E+01_JPRB, 0.14640E+01_JPRB, 0.15252E+01_JPRB, 0.15952E+01_JPRB, 0.16623E+01_JPRB, 0.17268E+01_JPRB, 0.17994E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.19447E+01_JPRB, 0.20347E+01_JPRB, 0.22207E+01_JPRB, 0.23987E+01_JPRB, 0.28689E+01_JPRB, 0.21396E+01_JPRB ,&
&  0.23053E+01_JPRB, 0.19845E+01_JPRB, 0.18603E+01_JPRB, 0.18858E+01_JPRB, 0.18596E+01_JPRB, 0.18290E+01_JPRB, 0.18355E+01_JPRB ,&
&  0.19247E+01_JPRB, 0.19837E+01_JPRB, 0.19584E+01_JPRB, 0.19912E+01_JPRB, 0.18443E+01_JPRB, 0.18111E+01_JPRB, 0.19775E+01_JPRB ,&
&  0.21335E+01_JPRB, 0.22718E+01_JPRB, 0.23237E+01_JPRB, 0.24039E+01_JPRB, 0.24090E+01_JPRB, 0.23582E+01_JPRB, 0.22942E+01_JPRB ,&
&  0.22137E+01_JPRB, 0.21905E+01_JPRB, 0.22365E+01_JPRB, 0.24051E+01_JPRB, 0.24761E+01_JPRB, 0.21965E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.13512E+01_JPRB, 0.12618E+01_JPRB, 0.18708E+01_JPRB, 0.22975E+01_JPRB, 0.20032E+01_JPRB, 0.18391E+01_JPRB, 0.16328E+01_JPRB ,&
&  0.15254E+01_JPRB, 0.13548E+01_JPRB, 0.12185E+01_JPRB, 0.10753E+01_JPRB, 0.10166E+01_JPRB, 0.99585E+00_JPRB, 0.10449E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.10466E+01_JPRB, 0.10621E+01_JPRB, 0.11323E+01_JPRB, 0.15308E+01_JPRB, 0.21726E+01_JPRB, 0.22757E+01_JPRB ,&
&  0.22459E+01_JPRB, 0.14639E+01_JPRB, 0.11878E+01_JPRB, 0.11598E+01_JPRB, 0.71932E+00_JPRB, 0.73175E+00_JPRB, 0.76754E+00_JPRB ,&
&  0.95936E+00_JPRB, 0.10337E+01_JPRB, 0.13600E+01_JPRB, 0.35827E+01_JPRB, 0.40583E+01_JPRB, 0.50413E+01_JPRB, 0.20641E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.70495E+00_JPRB, 0.64415E+00_JPRB, 0.64971E+00_JPRB, 0.64057E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.61493E+00_JPRB, 0.66724E+00_JPRB, 0.71500E+00_JPRB, 0.78514E+00_JPRB, 0.82019E+00_JPRB, 0.84109E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85258E+00_JPRB, 0.86404E+00_JPRB, 0.89526E+00_JPRB, 0.96909E+00_JPRB, 0.10502E+01_JPRB, 0.11361E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.13654E+01_JPRB, 0.14913E+01_JPRB, 0.15260E+01_JPRB, 0.15863E+01_JPRB, 0.16759E+01_JPRB, 0.17574E+01_JPRB, 0.18153E+01_JPRB ,&
&  0.18806E+01_JPRB, 0.19944E+01_JPRB, 0.21301E+01_JPRB, 0.23065E+01_JPRB, 0.25321E+01_JPRB, 0.27841E+01_JPRB, 0.20673E+01_JPRB ,&
&  0.22715E+01_JPRB, 0.19987E+01_JPRB, 0.19752E+01_JPRB, 0.19989E+01_JPRB, 0.19090E+01_JPRB, 0.18312E+01_JPRB, 0.18788E+01_JPRB ,&
&  0.20283E+01_JPRB, 0.22057E+01_JPRB, 0.22448E+01_JPRB, 0.21669E+01_JPRB, 0.22757E+01_JPRB, 0.21682E+01_JPRB, 0.24737E+01_JPRB ,&
&  0.27267E+01_JPRB, 0.28839E+01_JPRB, 0.29589E+01_JPRB, 0.30302E+01_JPRB, 0.31158E+01_JPRB, 0.32336E+01_JPRB, 0.33735E+01_JPRB ,&
&  0.34512E+01_JPRB, 0.34823E+01_JPRB, 0.35394E+01_JPRB, 0.35971E+01_JPRB, 0.32174E+01_JPRB, 0.34019E+01_JPRB, 0.27296E+01_JPRB ,&
&  0.18282E+01_JPRB, 0.14872E+01_JPRB, 0.18543E+01_JPRB, 0.20409E+01_JPRB, 0.18329E+01_JPRB, 0.16523E+01_JPRB, 0.14862E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.12072E+01_JPRB, 0.11003E+01_JPRB, 0.99384E+00_JPRB, 0.91514E+00_JPRB, 0.87660E+00_JPRB, 0.87254E+00_JPRB ,&
&  0.88210E+00_JPRB, 0.88304E+00_JPRB, 0.89698E+00_JPRB, 0.98303E+00_JPRB, 0.12521E+01_JPRB, 0.18732E+01_JPRB, 0.27678E+01_JPRB ,&
&  0.31517E+01_JPRB, 0.15197E+01_JPRB, 0.11103E+01_JPRB, 0.94762E+00_JPRB, 0.86765E+00_JPRB, 0.88314E+00_JPRB, 0.91302E+00_JPRB ,&
&  0.10518E+01_JPRB, 0.13713E+01_JPRB, 0.12585E+01_JPRB, 0.14128E+01_JPRB, 0.26605E+01_JPRB, 0.28482E+01_JPRB, 0.16725E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.79059E+00_JPRB, 0.65343E+00_JPRB, 0.61442E+00_JPRB, 0.60723E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.61598E+00_JPRB, 0.63420E+00_JPRB, 0.66119E+00_JPRB, 0.69497E+00_JPRB, 0.70007E+00_JPRB, 0.70940E+00_JPRB, 0.70293E+00_JPRB ,&
&  0.70676E+00_JPRB, 0.73422E+00_JPRB, 0.79856E+00_JPRB, 0.90709E+00_JPRB, 0.99355E+00_JPRB, 0.11104E+01_JPRB, 0.12746E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.14126E+01_JPRB, 0.14521E+01_JPRB, 0.15126E+01_JPRB, 0.15806E+01_JPRB, 0.16621E+01_JPRB, 0.17445E+01_JPRB ,&
&  0.18483E+01_JPRB, 0.20373E+01_JPRB, 0.22191E+01_JPRB, 0.24389E+01_JPRB, 0.26551E+01_JPRB, 0.28456E+01_JPRB, 0.32604E+01_JPRB ,&
&  0.17585E+01_JPRB, 0.21832E+01_JPRB, 0.22117E+01_JPRB, 0.21655E+01_JPRB, 0.18498E+01_JPRB, 0.17323E+01_JPRB, 0.18943E+01_JPRB ,&
&  0.23073E+01_JPRB, 0.26349E+01_JPRB, 0.29508E+01_JPRB, 0.23508E+01_JPRB, 0.25042E+01_JPRB, 0.26084E+01_JPRB, 0.29697E+01_JPRB ,&
&  0.31116E+01_JPRB, 0.32442E+01_JPRB, 0.34153E+01_JPRB, 0.36097E+01_JPRB, 0.38151E+01_JPRB, 0.40626E+01_JPRB, 0.42914E+01_JPRB ,&
&  0.44618E+01_JPRB, 0.45273E+01_JPRB, 0.46894E+01_JPRB, 0.42060E+01_JPRB, 0.36651E+01_JPRB, 0.38521E+01_JPRB, 0.36975E+01_JPRB ,&
&  0.30453E+01_JPRB, 0.20952E+01_JPRB, 0.20057E+01_JPRB, 0.20772E+01_JPRB, 0.18410E+01_JPRB, 0.16487E+01_JPRB, 0.14634E+01_JPRB ,&
&  0.12476E+01_JPRB, 0.10576E+01_JPRB, 0.97932E+00_JPRB, 0.90184E+00_JPRB, 0.84107E+00_JPRB, 0.81807E+00_JPRB, 0.83352E+00_JPRB ,&
&  0.85520E+00_JPRB, 0.86281E+00_JPRB, 0.85729E+00_JPRB, 0.88107E+00_JPRB, 0.97732E+00_JPRB, 0.10720E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.10810E+01_JPRB, 0.10310E+01_JPRB, 0.10292E+01_JPRB, 0.10161E+01_JPRB, 0.98077E+00_JPRB, 0.99570E+00_JPRB ,&
&  0.99270E+00_JPRB, 0.97849E+00_JPRB, 0.10744E+01_JPRB, 0.13053E+01_JPRB, 0.13157E+01_JPRB, 0.13652E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.92608E+00_JPRB, 0.79635E+00_JPRB, 0.70702E+00_JPRB, 0.64933E+00_JPRB, 0.60368E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.60707E+00_JPRB, 0.60656E+00_JPRB, 0.60832E+00_JPRB, 0.62766E+00_JPRB, 0.63349E+00_JPRB, 0.64081E+00_JPRB, 0.64171E+00_JPRB ,&
&  0.67621E+00_JPRB, 0.74152E+00_JPRB, 0.84197E+00_JPRB, 0.95961E+00_JPRB, 0.10551E+01_JPRB, 0.11701E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.13107E+01_JPRB, 0.13421E+01_JPRB, 0.14078E+01_JPRB, 0.14814E+01_JPRB, 0.15804E+01_JPRB, 0.16890E+01_JPRB ,&
&  0.18131E+01_JPRB, 0.19773E+01_JPRB, 0.22016E+01_JPRB, 0.24701E+01_JPRB, 0.27591E+01_JPRB, 0.30275E+01_JPRB, 0.34134E+01_JPRB ,&
&  0.13667E+01_JPRB, 0.17405E+01_JPRB, 0.23407E+01_JPRB, 0.24630E+01_JPRB, 0.18982E+01_JPRB, 0.17123E+01_JPRB, 0.16343E+01_JPRB ,&
&  0.20077E+01_JPRB, 0.24225E+01_JPRB, 0.25379E+01_JPRB, 0.26197E+01_JPRB, 0.26031E+01_JPRB, 0.28882E+01_JPRB, 0.31126E+01_JPRB ,&
&  0.31917E+01_JPRB, 0.32975E+01_JPRB, 0.34936E+01_JPRB, 0.37358E+01_JPRB, 0.39746E+01_JPRB, 0.41949E+01_JPRB, 0.44284E+01_JPRB ,&
&  0.46690E+01_JPRB, 0.48246E+01_JPRB, 0.50045E+01_JPRB, 0.42334E+01_JPRB, 0.43977E+01_JPRB, 0.46542E+01_JPRB, 0.52185E+01_JPRB ,&
&  0.42656E+01_JPRB, 0.26172E+01_JPRB, 0.20153E+01_JPRB, 0.20946E+01_JPRB, 0.18745E+01_JPRB, 0.17093E+01_JPRB, 0.14500E+01_JPRB ,&
&  0.11909E+01_JPRB, 0.10257E+01_JPRB, 0.95521E+00_JPRB, 0.93486E+00_JPRB, 0.91054E+00_JPRB, 0.90253E+00_JPRB, 0.90939E+00_JPRB ,&
&  0.92075E+00_JPRB, 0.93413E+00_JPRB, 0.92062E+00_JPRB, 0.91989E+00_JPRB, 0.98369E+00_JPRB, 0.10453E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.10873E+01_JPRB, 0.10983E+01_JPRB, 0.11203E+01_JPRB, 0.11487E+01_JPRB, 0.11481E+01_JPRB, 0.11368E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.95616E+00_JPRB, 0.93104E+00_JPRB, 0.90770E+00_JPRB, 0.88266E+00_JPRB, 0.90394E+00_JPRB, 0.85056E+00_JPRB ,&
&  0.81931E+00_JPRB, 0.80788E+00_JPRB, 0.77417E+00_JPRB, 0.70059E+00_JPRB, 0.62452E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.63437E+00_JPRB, 0.62868E+00_JPRB, 0.62131E+00_JPRB, 0.62903E+00_JPRB, 0.62389E+00_JPRB, 0.63612E+00_JPRB, 0.65551E+00_JPRB ,&
&  0.69352E+00_JPRB, 0.76430E+00_JPRB, 0.85714E+00_JPRB, 0.94408E+00_JPRB, 0.10453E+01_JPRB, 0.11331E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11829E+01_JPRB, 0.12082E+01_JPRB, 0.12397E+01_JPRB, 0.13072E+01_JPRB, 0.13796E+01_JPRB, 0.14783E+01_JPRB, 0.16115E+01_JPRB ,&
&  0.17743E+01_JPRB, 0.19860E+01_JPRB, 0.22559E+01_JPRB, 0.25830E+01_JPRB, 0.30122E+01_JPRB, 0.35210E+01_JPRB, 0.41398E+01_JPRB ,&
&  0.31141E+01_JPRB, 0.16632E+01_JPRB, 0.28310E+01_JPRB, 0.29552E+01_JPRB, 0.22434E+01_JPRB, 0.20102E+01_JPRB, 0.17188E+01_JPRB ,&
&  0.16695E+01_JPRB, 0.21845E+01_JPRB, 0.22941E+01_JPRB, 0.24268E+01_JPRB, 0.25462E+01_JPRB, 0.28120E+01_JPRB, 0.30116E+01_JPRB ,&
&  0.31221E+01_JPRB, 0.32336E+01_JPRB, 0.33674E+01_JPRB, 0.35204E+01_JPRB, 0.37138E+01_JPRB, 0.39157E+01_JPRB, 0.41598E+01_JPRB ,&
&  0.44441E+01_JPRB, 0.46560E+01_JPRB, 0.48113E+01_JPRB, 0.41356E+01_JPRB, 0.44406E+01_JPRB, 0.42459E+01_JPRB, 0.40346E+01_JPRB ,&
&  0.35450E+01_JPRB, 0.22177E+01_JPRB, 0.16795E+01_JPRB, 0.19937E+01_JPRB, 0.18535E+01_JPRB, 0.14200E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.10821E+01_JPRB, 0.10554E+01_JPRB, 0.10754E+01_JPRB, 0.10261E+01_JPRB, 0.98879E+00_JPRB, 0.96628E+00_JPRB ,&
&  0.10083E+01_JPRB, 0.10117E+01_JPRB, 0.99647E+00_JPRB, 0.10112E+01_JPRB, 0.10486E+01_JPRB, 0.11050E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.12019E+01_JPRB, 0.12177E+01_JPRB, 0.12521E+01_JPRB, 0.12758E+01_JPRB, 0.12050E+01_JPRB, 0.11684E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10487E+01_JPRB, 0.10189E+01_JPRB, 0.99070E+00_JPRB, 0.93634E+00_JPRB, 0.93182E+00_JPRB, 0.95447E+00_JPRB ,&
&  0.98211E+00_JPRB, 0.98262E+00_JPRB, 0.90205E+00_JPRB, 0.75699E+00_JPRB, 0.66470E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.65726E+00_JPRB, 0.62653E+00_JPRB, 0.62064E+00_JPRB, 0.64143E+00_JPRB, 0.63912E+00_JPRB, 0.64724E+00_JPRB, 0.67125E+00_JPRB ,&
&  0.70871E+00_JPRB, 0.76291E+00_JPRB, 0.81257E+00_JPRB, 0.87714E+00_JPRB, 0.96195E+00_JPRB, 0.10501E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.11094E+01_JPRB, 0.11416E+01_JPRB, 0.11881E+01_JPRB, 0.12677E+01_JPRB, 0.13562E+01_JPRB, 0.14618E+01_JPRB ,&
&  0.16221E+01_JPRB, 0.18458E+01_JPRB, 0.20921E+01_JPRB, 0.23628E+01_JPRB, 0.27414E+01_JPRB, 0.33457E+01_JPRB, 0.42890E+01_JPRB ,&
&  0.68653E+01_JPRB, 0.74274E+01_JPRB, 0.24685E+01_JPRB, 0.30833E+01_JPRB, 0.27644E+01_JPRB, 0.24041E+01_JPRB, 0.18988E+01_JPRB ,&
&  0.17920E+01_JPRB, 0.20735E+01_JPRB, 0.21328E+01_JPRB, 0.22905E+01_JPRB, 0.22870E+01_JPRB, 0.25037E+01_JPRB, 0.26932E+01_JPRB ,&
&  0.29191E+01_JPRB, 0.30593E+01_JPRB, 0.31281E+01_JPRB, 0.31616E+01_JPRB, 0.32497E+01_JPRB, 0.33930E+01_JPRB, 0.36401E+01_JPRB ,&
&  0.40119E+01_JPRB, 0.44184E+01_JPRB, 0.47295E+01_JPRB, 0.43702E+01_JPRB, 0.45736E+01_JPRB, 0.40035E+01_JPRB, 0.35328E+01_JPRB ,&
&  0.27108E+01_JPRB, 0.22662E+01_JPRB, 0.20838E+01_JPRB, 0.20602E+01_JPRB, 0.17796E+01_JPRB, 0.15732E+01_JPRB, 0.16519E+01_JPRB ,&
&  0.15142E+01_JPRB, 0.13297E+01_JPRB, 0.12156E+01_JPRB, 0.11557E+01_JPRB, 0.11235E+01_JPRB, 0.10734E+01_JPRB, 0.10684E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.11111E+01_JPRB, 0.10893E+01_JPRB, 0.11177E+01_JPRB, 0.11467E+01_JPRB, 0.11894E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.12967E+01_JPRB, 0.13171E+01_JPRB, 0.13230E+01_JPRB, 0.12530E+01_JPRB, 0.11807E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.11302E+01_JPRB, 0.11407E+01_JPRB, 0.11453E+01_JPRB, 0.11081E+01_JPRB, 0.11129E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.11501E+01_JPRB, 0.10372E+01_JPRB, 0.80513E+00_JPRB, 0.70189E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.71088E+00_JPRB, 0.68182E+00_JPRB, 0.67966E+00_JPRB, 0.69097E+00_JPRB, 0.69471E+00_JPRB, 0.67860E+00_JPRB, 0.68022E+00_JPRB ,&
&  0.69643E+00_JPRB, 0.72131E+00_JPRB, 0.72671E+00_JPRB, 0.76981E+00_JPRB, 0.86374E+00_JPRB, 0.96172E+00_JPRB, 0.99378E+00_JPRB ,&
&  0.99840E+00_JPRB, 0.99725E+00_JPRB, 0.10013E+01_JPRB, 0.10306E+01_JPRB, 0.10793E+01_JPRB, 0.11788E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.14278E+01_JPRB, 0.15848E+01_JPRB, 0.17092E+01_JPRB, 0.17974E+01_JPRB, 0.19952E+01_JPRB, 0.23957E+01_JPRB, 0.34637E+01_JPRB ,&
&  0.54503E+01_JPRB, 0.10914E+02_JPRB, 0.28492E+01_JPRB, 0.29890E+01_JPRB, 0.30584E+01_JPRB, 0.27949E+01_JPRB, 0.24552E+01_JPRB ,&
&  0.24080E+01_JPRB, 0.21758E+01_JPRB, 0.21258E+01_JPRB, 0.21551E+01_JPRB, 0.20268E+01_JPRB, 0.21002E+01_JPRB, 0.22246E+01_JPRB ,&
&  0.25173E+01_JPRB, 0.27188E+01_JPRB, 0.28411E+01_JPRB, 0.28267E+01_JPRB, 0.27929E+01_JPRB, 0.27697E+01_JPRB, 0.28319E+01_JPRB ,&
&  0.31337E+01_JPRB, 0.37144E+01_JPRB, 0.43434E+01_JPRB, 0.44481E+01_JPRB, 0.47836E+01_JPRB, 0.42053E+01_JPRB, 0.38092E+01_JPRB ,&
&  0.31197E+01_JPRB, 0.28030E+01_JPRB, 0.27328E+01_JPRB, 0.24854E+01_JPRB, 0.20444E+01_JPRB, 0.18410E+01_JPRB, 0.18441E+01_JPRB ,&
&  0.17428E+01_JPRB, 0.15623E+01_JPRB, 0.13662E+01_JPRB, 0.12274E+01_JPRB, 0.11992E+01_JPRB, 0.12100E+01_JPRB, 0.11819E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.12608E+01_JPRB, 0.12381E+01_JPRB, 0.12862E+01_JPRB, 0.13057E+01_JPRB, 0.13125E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.13583E+01_JPRB, 0.13723E+01_JPRB, 0.13201E+01_JPRB, 0.12133E+01_JPRB, 0.11473E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.12103E+01_JPRB, 0.12032E+01_JPRB, 0.11679E+01_JPRB, 0.12029E+01_JPRB, 0.11913E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11954E+01_JPRB, 0.12381E+01_JPRB, 0.12469E+01_JPRB, 0.92205E+00_JPRB, 0.78312E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.81229E+00_JPRB, 0.77612E+00_JPRB, 0.74895E+00_JPRB, 0.72630E+00_JPRB, 0.71057E+00_JPRB, 0.71214E+00_JPRB, 0.71727E+00_JPRB ,&
&  0.71692E+00_JPRB, 0.72478E+00_JPRB, 0.72383E+00_JPRB, 0.73184E+00_JPRB, 0.75744E+00_JPRB, 0.81086E+00_JPRB, 0.87674E+00_JPRB ,&
&  0.89398E+00_JPRB, 0.87445E+00_JPRB, 0.86748E+00_JPRB, 0.86243E+00_JPRB, 0.88977E+00_JPRB, 0.94100E+00_JPRB, 0.10452E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.13531E+01_JPRB, 0.14164E+01_JPRB, 0.14106E+01_JPRB, 0.14528E+01_JPRB, 0.15911E+01_JPRB, 0.24788E+01_JPRB ,&
&  0.44088E+01_JPRB, 0.97382E+01_JPRB, 0.58565E+01_JPRB, 0.45882E+01_JPRB, 0.34041E+01_JPRB, 0.29436E+01_JPRB, 0.28416E+01_JPRB ,&
&  0.31982E+01_JPRB, 0.30035E+01_JPRB, 0.26493E+01_JPRB, 0.20811E+01_JPRB, 0.18378E+01_JPRB, 0.17363E+01_JPRB, 0.18164E+01_JPRB ,&
&  0.20568E+01_JPRB, 0.22811E+01_JPRB, 0.24356E+01_JPRB, 0.24736E+01_JPRB, 0.24200E+01_JPRB, 0.23644E+01_JPRB, 0.23223E+01_JPRB ,&
&  0.23024E+01_JPRB, 0.25319E+01_JPRB, 0.34135E+01_JPRB, 0.43778E+01_JPRB, 0.49222E+01_JPRB, 0.45527E+01_JPRB, 0.42961E+01_JPRB ,&
&  0.41574E+01_JPRB, 0.36728E+01_JPRB, 0.33615E+01_JPRB, 0.28742E+01_JPRB, 0.22883E+01_JPRB, 0.23172E+01_JPRB, 0.19903E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.17044E+01_JPRB, 0.15330E+01_JPRB, 0.13480E+01_JPRB, 0.13318E+01_JPRB, 0.13247E+01_JPRB, 0.12812E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.13629E+01_JPRB, 0.13439E+01_JPRB, 0.13563E+01_JPRB, 0.13623E+01_JPRB, 0.13748E+01_JPRB, 0.13773E+01_JPRB ,&
&  0.13752E+01_JPRB, 0.14047E+01_JPRB, 0.13284E+01_JPRB, 0.12450E+01_JPRB, 0.11936E+01_JPRB, 0.11356E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.12545E+01_JPRB, 0.12539E+01_JPRB, 0.12576E+01_JPRB, 0.12284E+01_JPRB, 0.12295E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.11641E+01_JPRB, 0.11297E+01_JPRB, 0.11039E+01_JPRB, 0.10097E+01_JPRB, 0.88734E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.88234E+00_JPRB, 0.84766E+00_JPRB, 0.77743E+00_JPRB, 0.72001E+00_JPRB, 0.70521E+00_JPRB, 0.71243E+00_JPRB, 0.72963E+00_JPRB ,&
&  0.74094E+00_JPRB, 0.74727E+00_JPRB, 0.74716E+00_JPRB, 0.74056E+00_JPRB, 0.74446E+00_JPRB, 0.72049E+00_JPRB, 0.74399E+00_JPRB ,&
&  0.78627E+00_JPRB, 0.80504E+00_JPRB, 0.81918E+00_JPRB, 0.78643E+00_JPRB, 0.78126E+00_JPRB, 0.77912E+00_JPRB, 0.82827E+00_JPRB ,&
&  0.94421E+00_JPRB, 0.11148E+01_JPRB, 0.12239E+01_JPRB, 0.12468E+01_JPRB, 0.12602E+01_JPRB, 0.13760E+01_JPRB, 0.19460E+01_JPRB ,&
&  0.35676E+01_JPRB, 0.82037E+01_JPRB, 0.34266E+01_JPRB, 0.58326E+01_JPRB, 0.40860E+01_JPRB, 0.31758E+01_JPRB, 0.24988E+01_JPRB ,&
&  0.26598E+01_JPRB, 0.32112E+01_JPRB, 0.24803E+01_JPRB, 0.19466E+01_JPRB, 0.16885E+01_JPRB, 0.16089E+01_JPRB, 0.15921E+01_JPRB ,&
&  0.16686E+01_JPRB, 0.18579E+01_JPRB, 0.20365E+01_JPRB, 0.21193E+01_JPRB, 0.20771E+01_JPRB, 0.19791E+01_JPRB, 0.19475E+01_JPRB ,&
&  0.19244E+01_JPRB, 0.19357E+01_JPRB, 0.23096E+01_JPRB, 0.29058E+01_JPRB, 0.43462E+01_JPRB, 0.48589E+01_JPRB, 0.49876E+01_JPRB ,&
&  0.50890E+01_JPRB, 0.46674E+01_JPRB, 0.39454E+01_JPRB, 0.32854E+01_JPRB, 0.27033E+01_JPRB, 0.25816E+01_JPRB, 0.22013E+01_JPRB ,&
&  0.19587E+01_JPRB, 0.17903E+01_JPRB, 0.16336E+01_JPRB, 0.14855E+01_JPRB, 0.14123E+01_JPRB, 0.13848E+01_JPRB, 0.13466E+01_JPRB ,&
&  0.13922E+01_JPRB, 0.14547E+01_JPRB, 0.14370E+01_JPRB, 0.13974E+01_JPRB, 0.13820E+01_JPRB, 0.13848E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.14245E+01_JPRB, 0.13444E+01_JPRB, 0.12179E+01_JPRB, 0.11827E+01_JPRB, 0.11750E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.13173E+01_JPRB, 0.13125E+01_JPRB, 0.12658E+01_JPRB, 0.12562E+01_JPRB, 0.12961E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.11008E+01_JPRB, 0.10582E+01_JPRB, 0.10145E+01_JPRB, 0.95361E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.89447E+00_JPRB, 0.83145E+00_JPRB, 0.75976E+00_JPRB, 0.72359E+00_JPRB, 0.72270E+00_JPRB, 0.71186E+00_JPRB, 0.72586E+00_JPRB ,&
&  0.75541E+00_JPRB, 0.76140E+00_JPRB, 0.76241E+00_JPRB, 0.75103E+00_JPRB, 0.74052E+00_JPRB, 0.71431E+00_JPRB, 0.68616E+00_JPRB ,&
&  0.71127E+00_JPRB, 0.78498E+00_JPRB, 0.81852E+00_JPRB, 0.80139E+00_JPRB, 0.77570E+00_JPRB, 0.75070E+00_JPRB, 0.74970E+00_JPRB ,&
&  0.79083E+00_JPRB, 0.89224E+00_JPRB, 0.10109E+01_JPRB, 0.10777E+01_JPRB, 0.11056E+01_JPRB, 0.11556E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.26855E+01_JPRB, 0.85658E+01_JPRB, 0.33800E+01_JPRB, 0.63880E+01_JPRB, 0.46589E+01_JPRB, 0.36100E+01_JPRB, 0.26102E+01_JPRB ,&
&  0.24178E+01_JPRB, 0.26014E+01_JPRB, 0.22612E+01_JPRB, 0.19190E+01_JPRB, 0.16803E+01_JPRB, 0.15853E+01_JPRB, 0.15657E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.15634E+01_JPRB, 0.16313E+01_JPRB, 0.16990E+01_JPRB, 0.17057E+01_JPRB, 0.16568E+01_JPRB, 0.16109E+01_JPRB ,&
&  0.15912E+01_JPRB, 0.16003E+01_JPRB, 0.17866E+01_JPRB, 0.22464E+01_JPRB, 0.32397E+01_JPRB, 0.45654E+01_JPRB, 0.59604E+01_JPRB ,&
&  0.64005E+01_JPRB, 0.58865E+01_JPRB, 0.47819E+01_JPRB, 0.39576E+01_JPRB, 0.33481E+01_JPRB, 0.30851E+01_JPRB, 0.26382E+01_JPRB ,&
&  0.22783E+01_JPRB, 0.20306E+01_JPRB, 0.18282E+01_JPRB, 0.16914E+01_JPRB, 0.15674E+01_JPRB, 0.15263E+01_JPRB, 0.14875E+01_JPRB ,&
&  0.14988E+01_JPRB, 0.15826E+01_JPRB, 0.15795E+01_JPRB, 0.15143E+01_JPRB, 0.14289E+01_JPRB, 0.13949E+01_JPRB, 0.13608E+01_JPRB ,&
&  0.13535E+01_JPRB, 0.13929E+01_JPRB, 0.13653E+01_JPRB, 0.12682E+01_JPRB, 0.12164E+01_JPRB, 0.12117E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.13738E+01_JPRB, 0.13682E+01_JPRB, 0.13429E+01_JPRB, 0.13063E+01_JPRB, 0.13567E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.11136E+01_JPRB, 0.10634E+01_JPRB, 0.99598E+00_JPRB, 0.96122E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.91573E+00_JPRB, 0.83564E+00_JPRB, 0.77924E+00_JPRB, 0.74138E+00_JPRB, 0.73733E+00_JPRB, 0.73010E+00_JPRB, 0.73022E+00_JPRB ,&
&  0.75234E+00_JPRB, 0.76532E+00_JPRB, 0.77790E+00_JPRB, 0.76651E+00_JPRB, 0.74552E+00_JPRB, 0.74205E+00_JPRB, 0.72089E+00_JPRB ,&
&  0.71190E+00_JPRB, 0.76110E+00_JPRB, 0.81105E+00_JPRB, 0.82060E+00_JPRB, 0.79907E+00_JPRB, 0.75758E+00_JPRB, 0.73878E+00_JPRB ,&
&  0.74651E+00_JPRB, 0.77563E+00_JPRB, 0.83606E+00_JPRB, 0.90416E+00_JPRB, 0.95092E+00_JPRB, 0.98466E+00_JPRB, 0.10848E+01_JPRB ,&
&  0.16323E+01_JPRB, 0.27566E+01_JPRB, 0.49765E+01_JPRB, 0.62644E+01_JPRB, 0.49668E+01_JPRB, 0.37884E+01_JPRB, 0.29232E+01_JPRB ,&
&  0.25247E+01_JPRB, 0.23453E+01_JPRB, 0.21377E+01_JPRB, 0.18897E+01_JPRB, 0.16918E+01_JPRB, 0.15455E+01_JPRB, 0.14975E+01_JPRB ,&
&  0.14450E+01_JPRB, 0.14026E+01_JPRB, 0.13688E+01_JPRB, 0.13606E+01_JPRB, 0.13547E+01_JPRB, 0.13440E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.13342E+01_JPRB, 0.13532E+01_JPRB, 0.14360E+01_JPRB, 0.17704E+01_JPRB, 0.27754E+01_JPRB, 0.40878E+01_JPRB, 0.60236E+01_JPRB ,&
&  0.72176E+01_JPRB, 0.65431E+01_JPRB, 0.54653E+01_JPRB, 0.45743E+01_JPRB, 0.39316E+01_JPRB, 0.35722E+01_JPRB, 0.31964E+01_JPRB ,&
&  0.28425E+01_JPRB, 0.24879E+01_JPRB, 0.21913E+01_JPRB, 0.19994E+01_JPRB, 0.17897E+01_JPRB, 0.16890E+01_JPRB, 0.16880E+01_JPRB ,&
&  0.16456E+01_JPRB, 0.16437E+01_JPRB, 0.16556E+01_JPRB, 0.17104E+01_JPRB, 0.16207E+01_JPRB, 0.15156E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.13485E+01_JPRB, 0.13577E+01_JPRB, 0.13401E+01_JPRB, 0.13109E+01_JPRB, 0.12991E+01_JPRB, 0.12930E+01_JPRB, 0.12932E+01_JPRB ,&
&  0.13254E+01_JPRB, 0.13754E+01_JPRB, 0.13947E+01_JPRB, 0.13949E+01_JPRB, 0.13030E+01_JPRB, 0.13698E+01_JPRB, 0.12751E+01_JPRB ,&
&  0.11881E+01_JPRB, 0.11318E+01_JPRB, 0.10907E+01_JPRB, 0.10375E+01_JPRB, 0.99960E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.10115E+01_JPRB, 0.93069E+00_JPRB, 0.85106E+00_JPRB, 0.80057E+00_JPRB, 0.76373E+00_JPRB, 0.74769E+00_JPRB, 0.73863E+00_JPRB ,&
&  0.74626E+00_JPRB, 0.75542E+00_JPRB, 0.77086E+00_JPRB, 0.76879E+00_JPRB, 0.75087E+00_JPRB, 0.75962E+00_JPRB, 0.77937E+00_JPRB ,&
&  0.77442E+00_JPRB, 0.78600E+00_JPRB, 0.80756E+00_JPRB, 0.79724E+00_JPRB, 0.77425E+00_JPRB, 0.74942E+00_JPRB, 0.72926E+00_JPRB ,&
&  0.73133E+00_JPRB, 0.73738E+00_JPRB, 0.75516E+00_JPRB, 0.79213E+00_JPRB, 0.84442E+00_JPRB, 0.88736E+00_JPRB, 0.92401E+00_JPRB ,&
&  0.11257E+01_JPRB, 0.31193E+01_JPRB, 0.46153E+01_JPRB, 0.54229E+01_JPRB, 0.48875E+01_JPRB, 0.37535E+01_JPRB, 0.30915E+01_JPRB ,&
&  0.24734E+01_JPRB, 0.21777E+01_JPRB, 0.19836E+01_JPRB, 0.18114E+01_JPRB, 0.16299E+01_JPRB, 0.15159E+01_JPRB, 0.14568E+01_JPRB ,&
&  0.14013E+01_JPRB, 0.13224E+01_JPRB, 0.12344E+01_JPRB, 0.11606E+01_JPRB, 0.11337E+01_JPRB, 0.11274E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.11649E+01_JPRB, 0.12171E+01_JPRB, 0.13564E+01_JPRB, 0.21084E+01_JPRB, 0.25705E+01_JPRB, 0.45594E+01_JPRB ,&
&  0.55917E+01_JPRB, 0.48561E+01_JPRB, 0.43768E+01_JPRB, 0.40293E+01_JPRB, 0.36544E+01_JPRB, 0.33191E+01_JPRB, 0.31070E+01_JPRB ,&
&  0.28929E+01_JPRB, 0.26497E+01_JPRB, 0.24439E+01_JPRB, 0.22899E+01_JPRB, 0.21354E+01_JPRB, 0.20139E+01_JPRB, 0.18693E+01_JPRB ,&
&  0.18127E+01_JPRB, 0.17397E+01_JPRB, 0.17118E+01_JPRB, 0.17546E+01_JPRB, 0.17835E+01_JPRB, 0.16943E+01_JPRB, 0.15791E+01_JPRB ,&
&  0.14570E+01_JPRB, 0.13922E+01_JPRB, 0.13560E+01_JPRB, 0.13300E+01_JPRB, 0.13411E+01_JPRB, 0.13573E+01_JPRB, 0.13575E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.13485E+01_JPRB, 0.13912E+01_JPRB, 0.13837E+01_JPRB, 0.13204E+01_JPRB, 0.13871E+01_JPRB, 0.13014E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.11309E+01_JPRB, 0.10809E+01_JPRB, 0.10364E+01_JPRB, 0.10504E+01_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.10590E+01_JPRB, 0.10078E+01_JPRB, 0.94280E+00_JPRB, 0.89013E+00_JPRB, 0.84472E+00_JPRB, 0.81384E+00_JPRB, 0.80448E+00_JPRB ,&
&  0.79650E+00_JPRB, 0.78479E+00_JPRB, 0.78969E+00_JPRB, 0.78794E+00_JPRB, 0.78678E+00_JPRB, 0.78839E+00_JPRB, 0.80787E+00_JPRB ,&
&  0.82154E+00_JPRB, 0.82121E+00_JPRB, 0.81598E+00_JPRB, 0.80115E+00_JPRB, 0.78229E+00_JPRB, 0.76006E+00_JPRB, 0.74501E+00_JPRB ,&
&  0.73640E+00_JPRB, 0.73637E+00_JPRB, 0.74287E+00_JPRB, 0.75832E+00_JPRB, 0.79262E+00_JPRB, 0.83958E+00_JPRB, 0.88420E+00_JPRB ,&
&  0.94446E+00_JPRB, 0.18303E+01_JPRB, 0.26388E+01_JPRB, 0.40962E+01_JPRB, 0.45020E+01_JPRB, 0.37385E+01_JPRB, 0.30475E+01_JPRB ,&
&  0.24970E+01_JPRB, 0.21280E+01_JPRB, 0.18899E+01_JPRB, 0.17179E+01_JPRB, 0.15459E+01_JPRB, 0.14447E+01_JPRB, 0.13719E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.12356E+01_JPRB, 0.11522E+01_JPRB, 0.10853E+01_JPRB, 0.10570E+01_JPRB, 0.10538E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10382E+01_JPRB, 0.10458E+01_JPRB, 0.10583E+01_JPRB, 0.10966E+01_JPRB, 0.14128E+01_JPRB, 0.17671E+01_JPRB, 0.25931E+01_JPRB ,&
&  0.26432E+01_JPRB, 0.25116E+01_JPRB, 0.24889E+01_JPRB, 0.24597E+01_JPRB, 0.24879E+01_JPRB, 0.25016E+01_JPRB, 0.25411E+01_JPRB ,&
&  0.25677E+01_JPRB, 0.25401E+01_JPRB, 0.24662E+01_JPRB, 0.24053E+01_JPRB, 0.23402E+01_JPRB, 0.22657E+01_JPRB, 0.21292E+01_JPRB ,&
&  0.20198E+01_JPRB, 0.19296E+01_JPRB, 0.18691E+01_JPRB, 0.18113E+01_JPRB, 0.18031E+01_JPRB, 0.17601E+01_JPRB, 0.16741E+01_JPRB ,&
&  0.15573E+01_JPRB, 0.14463E+01_JPRB, 0.14040E+01_JPRB, 0.13855E+01_JPRB, 0.13678E+01_JPRB, 0.13441E+01_JPRB, 0.13445E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.13730E+01_JPRB, 0.13726E+01_JPRB, 0.13716E+01_JPRB, 0.13181E+01_JPRB, 0.13656E+01_JPRB, 0.13062E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.11477E+01_JPRB, 0.10753E+01_JPRB, 0.10356E+01_JPRB, 0.10768E+01_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.11476E+01_JPRB, 0.10955E+01_JPRB, 0.10385E+01_JPRB, 0.96445E+00_JPRB, 0.90379E+00_JPRB, 0.87507E+00_JPRB, 0.85655E+00_JPRB ,&
&  0.83422E+00_JPRB, 0.81367E+00_JPRB, 0.81066E+00_JPRB, 0.81732E+00_JPRB, 0.82422E+00_JPRB, 0.82878E+00_JPRB, 0.82534E+00_JPRB ,&
&  0.82471E+00_JPRB, 0.82725E+00_JPRB, 0.81735E+00_JPRB, 0.80142E+00_JPRB, 0.78411E+00_JPRB, 0.76942E+00_JPRB, 0.75820E+00_JPRB ,&
&  0.75139E+00_JPRB, 0.75346E+00_JPRB, 0.75544E+00_JPRB, 0.76663E+00_JPRB, 0.78876E+00_JPRB, 0.82043E+00_JPRB, 0.86494E+00_JPRB ,&
&  0.92036E+00_JPRB, 0.10475E+01_JPRB, 0.15971E+01_JPRB, 0.23912E+01_JPRB, 0.30725E+01_JPRB, 0.31196E+01_JPRB, 0.27765E+01_JPRB ,&
&  0.24531E+01_JPRB, 0.21347E+01_JPRB, 0.18882E+01_JPRB, 0.16833E+01_JPRB, 0.15161E+01_JPRB, 0.13892E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.12193E+01_JPRB, 0.11561E+01_JPRB, 0.10986E+01_JPRB, 0.10471E+01_JPRB, 0.10203E+01_JPRB, 0.10015E+01_JPRB, 0.98466E+00_JPRB ,&
&  0.96966E+00_JPRB, 0.96051E+00_JPRB, 0.95634E+00_JPRB, 0.95900E+00_JPRB, 0.99878E+00_JPRB, 0.11008E+01_JPRB, 0.13337E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.15281E+01_JPRB, 0.15455E+01_JPRB, 0.15429E+01_JPRB, 0.15724E+01_JPRB, 0.16476E+01_JPRB, 0.17680E+01_JPRB ,&
&  0.19155E+01_JPRB, 0.20491E+01_JPRB, 0.21532E+01_JPRB, 0.22181E+01_JPRB, 0.22282E+01_JPRB, 0.21929E+01_JPRB, 0.21282E+01_JPRB ,&
&  0.20578E+01_JPRB, 0.19541E+01_JPRB, 0.18534E+01_JPRB, 0.17711E+01_JPRB, 0.17246E+01_JPRB, 0.16928E+01_JPRB, 0.16392E+01_JPRB ,&
&  0.15604E+01_JPRB, 0.14697E+01_JPRB, 0.14066E+01_JPRB, 0.13749E+01_JPRB, 0.13565E+01_JPRB, 0.13321E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.13284E+01_JPRB, 0.12929E+01_JPRB, 0.13247E+01_JPRB, 0.13579E+01_JPRB, 0.13072E+01_JPRB, 0.12600E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.11448E+01_JPRB, 0.10916E+01_JPRB, 0.10501E+01_JPRB, 0.11402E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.10659E+01_JPRB, 0.10562E+01_JPRB, 0.10158E+01_JPRB, 0.96055E+00_JPRB, 0.90693E+00_JPRB, 0.86293E+00_JPRB, 0.83884E+00_JPRB ,&
&  0.82158E+00_JPRB, 0.81066E+00_JPRB, 0.81630E+00_JPRB, 0.83140E+00_JPRB, 0.84928E+00_JPRB, 0.85540E+00_JPRB, 0.85082E+00_JPRB ,&
&  0.83747E+00_JPRB, 0.81582E+00_JPRB, 0.79992E+00_JPRB, 0.78933E+00_JPRB, 0.77661E+00_JPRB, 0.76991E+00_JPRB, 0.76630E+00_JPRB ,&
&  0.76833E+00_JPRB, 0.77299E+00_JPRB, 0.77577E+00_JPRB, 0.78539E+00_JPRB, 0.80217E+00_JPRB, 0.82941E+00_JPRB, 0.86267E+00_JPRB ,&
&  0.88901E+00_JPRB, 0.82898E+00_JPRB, 0.10436E+01_JPRB, 0.15113E+01_JPRB, 0.20507E+01_JPRB, 0.23325E+01_JPRB, 0.23176E+01_JPRB ,&
&  0.21540E+01_JPRB, 0.19898E+01_JPRB, 0.18178E+01_JPRB, 0.16264E+01_JPRB, 0.14601E+01_JPRB, 0.13283E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.10678E+01_JPRB, 0.10305E+01_JPRB, 0.10030E+01_JPRB, 0.97704E+00_JPRB, 0.93955E+00_JPRB, 0.91304E+00_JPRB ,&
&  0.90168E+00_JPRB, 0.89456E+00_JPRB, 0.89149E+00_JPRB, 0.89117E+00_JPRB, 0.90811E+00_JPRB, 0.93988E+00_JPRB, 0.98154E+00_JPRB ,&
&  0.10243E+01_JPRB, 0.10658E+01_JPRB, 0.10878E+01_JPRB, 0.10967E+01_JPRB, 0.11013E+01_JPRB, 0.11584E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.14466E+01_JPRB, 0.16146E+01_JPRB, 0.17231E+01_JPRB, 0.17482E+01_JPRB, 0.17097E+01_JPRB, 0.16396E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.15287E+01_JPRB, 0.14704E+01_JPRB, 0.14210E+01_JPRB, 0.13946E+01_JPRB, 0.13983E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.14024E+01_JPRB, 0.13805E+01_JPRB, 0.13546E+01_JPRB, 0.13484E+01_JPRB, 0.13481E+01_JPRB, 0.13326E+01_JPRB, 0.13077E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.12568E+01_JPRB, 0.12375E+01_JPRB, 0.12579E+01_JPRB, 0.12268E+01_JPRB, 0.12029E+01_JPRB, 0.11677E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.10984E+01_JPRB, 0.10524E+01_JPRB, 0.10115E+01_JPRB, 0.99847E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.94502E+00_JPRB, 0.98009E+00_JPRB, 0.98345E+00_JPRB, 0.95137E+00_JPRB, 0.91255E+00_JPRB, 0.88392E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.85612E+00_JPRB, 0.84895E+00_JPRB, 0.85220E+00_JPRB, 0.86501E+00_JPRB, 0.87873E+00_JPRB, 0.88116E+00_JPRB, 0.86231E+00_JPRB ,&
&  0.83640E+00_JPRB, 0.81395E+00_JPRB, 0.79995E+00_JPRB, 0.79232E+00_JPRB, 0.78471E+00_JPRB, 0.78059E+00_JPRB, 0.78788E+00_JPRB ,&
&  0.79722E+00_JPRB, 0.80901E+00_JPRB, 0.82263E+00_JPRB, 0.83487E+00_JPRB, 0.84539E+00_JPRB, 0.85625E+00_JPRB, 0.88083E+00_JPRB ,&
&  0.90313E+00_JPRB, 0.74653E+00_JPRB, 0.81651E+00_JPRB, 0.10647E+01_JPRB, 0.13837E+01_JPRB, 0.17457E+01_JPRB, 0.18935E+01_JPRB ,&
&  0.18466E+01_JPRB, 0.17388E+01_JPRB, 0.16112E+01_JPRB, 0.14750E+01_JPRB, 0.13428E+01_JPRB, 0.12328E+01_JPRB, 0.11377E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10170E+01_JPRB, 0.98113E+00_JPRB, 0.94771E+00_JPRB, 0.92292E+00_JPRB, 0.90029E+00_JPRB, 0.88078E+00_JPRB ,&
&  0.86385E+00_JPRB, 0.84807E+00_JPRB, 0.84130E+00_JPRB, 0.84102E+00_JPRB, 0.83685E+00_JPRB, 0.84187E+00_JPRB, 0.83939E+00_JPRB ,&
&  0.85933E+00_JPRB, 0.88587E+00_JPRB, 0.89792E+00_JPRB, 0.90097E+00_JPRB, 0.90837E+00_JPRB, 0.92718E+00_JPRB, 0.95540E+00_JPRB ,&
&  0.99158E+00_JPRB, 0.10649E+01_JPRB, 0.11472E+01_JPRB, 0.12247E+01_JPRB, 0.12723E+01_JPRB, 0.12831E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.12651E+01_JPRB, 0.12464E+01_JPRB, 0.12198E+01_JPRB, 0.11865E+01_JPRB, 0.11704E+01_JPRB, 0.11792E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.12301E+01_JPRB, 0.12479E+01_JPRB, 0.12685E+01_JPRB, 0.12685E+01_JPRB, 0.12468E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11620E+01_JPRB, 0.11300E+01_JPRB, 0.10760E+01_JPRB, 0.11277E+01_JPRB, 0.11210E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.10309E+01_JPRB, 0.10006E+01_JPRB, 0.82886E+00_JPRB, 0.90624E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.88906E+00_JPRB, 0.92111E+00_JPRB, 0.93273E+00_JPRB, 0.92501E+00_JPRB, 0.90409E+00_JPRB, 0.89051E+00_JPRB, 0.88018E+00_JPRB ,&
&  0.87284E+00_JPRB, 0.86706E+00_JPRB, 0.87521E+00_JPRB, 0.88101E+00_JPRB, 0.88148E+00_JPRB, 0.87069E+00_JPRB, 0.85832E+00_JPRB ,&
&  0.84050E+00_JPRB, 0.81952E+00_JPRB, 0.80351E+00_JPRB, 0.78962E+00_JPRB, 0.78628E+00_JPRB, 0.79289E+00_JPRB, 0.80780E+00_JPRB ,&
&  0.82792E+00_JPRB, 0.84239E+00_JPRB, 0.85497E+00_JPRB, 0.86912E+00_JPRB, 0.87386E+00_JPRB, 0.87672E+00_JPRB, 0.88293E+00_JPRB ,&
&  0.89397E+00_JPRB, 0.71424E+00_JPRB, 0.77525E+00_JPRB, 0.87269E+00_JPRB, 0.99850E+00_JPRB, 0.11958E+01_JPRB, 0.13576E+01_JPRB ,&
&  0.14141E+01_JPRB, 0.13470E+01_JPRB, 0.12803E+01_JPRB, 0.12081E+01_JPRB, 0.11377E+01_JPRB, 0.10767E+01_JPRB, 0.10172E+01_JPRB ,&
&  0.96977E+00_JPRB, 0.94069E+00_JPRB, 0.92550E+00_JPRB, 0.91000E+00_JPRB, 0.89208E+00_JPRB, 0.87072E+00_JPRB, 0.84738E+00_JPRB ,&
&  0.82383E+00_JPRB, 0.80748E+00_JPRB, 0.79430E+00_JPRB, 0.78477E+00_JPRB, 0.78260E+00_JPRB, 0.78454E+00_JPRB, 0.78543E+00_JPRB ,&
&  0.79385E+00_JPRB, 0.80186E+00_JPRB, 0.80607E+00_JPRB, 0.80485E+00_JPRB, 0.80350E+00_JPRB, 0.80602E+00_JPRB, 0.82467E+00_JPRB ,&
&  0.84306E+00_JPRB, 0.87441E+00_JPRB, 0.91594E+00_JPRB, 0.97286E+00_JPRB, 0.10201E+01_JPRB, 0.10439E+01_JPRB, 0.10562E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.10528E+01_JPRB, 0.10319E+01_JPRB, 0.10105E+01_JPRB, 0.99300E+00_JPRB, 0.98635E+00_JPRB, 0.98389E+00_JPRB ,&
&  0.99239E+00_JPRB, 0.10092E+01_JPRB, 0.10251E+01_JPRB, 0.10467E+01_JPRB, 0.10690E+01_JPRB, 0.10701E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.10197E+01_JPRB, 0.10065E+01_JPRB, 0.99565E+00_JPRB, 0.10141E+01_JPRB, 0.10159E+01_JPRB, 0.10080E+01_JPRB, 0.98473E+00_JPRB ,&
&  0.96645E+00_JPRB, 0.94963E+00_JPRB, 0.74237E+00_JPRB, 0.83460E+00_JPRB, 0.85582E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.87520E+00_JPRB, 0.87134E+00_JPRB, 0.86581E+00_JPRB, 0.87084E+00_JPRB, 0.87571E+00_JPRB, 0.87517E+00_JPRB, 0.87404E+00_JPRB ,&
&  0.87063E+00_JPRB, 0.86580E+00_JPRB, 0.87269E+00_JPRB, 0.87892E+00_JPRB, 0.87845E+00_JPRB, 0.86997E+00_JPRB, 0.86076E+00_JPRB ,&
&  0.85334E+00_JPRB, 0.84277E+00_JPRB, 0.83229E+00_JPRB, 0.82708E+00_JPRB, 0.82836E+00_JPRB, 0.84110E+00_JPRB, 0.86115E+00_JPRB ,&
&  0.87611E+00_JPRB, 0.87490E+00_JPRB, 0.87858E+00_JPRB, 0.89047E+00_JPRB, 0.89444E+00_JPRB, 0.89787E+00_JPRB, 0.90616E+00_JPRB ,&
&  0.78823E+00_JPRB, 0.66920E+00_JPRB, 0.74287E+00_JPRB, 0.81504E+00_JPRB, 0.88043E+00_JPRB, 0.96298E+00_JPRB, 0.10726E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11016E+01_JPRB, 0.10752E+01_JPRB, 0.10384E+01_JPRB, 0.99745E+00_JPRB, 0.96717E+00_JPRB, 0.94299E+00_JPRB ,&
&  0.93466E+00_JPRB, 0.92116E+00_JPRB, 0.90075E+00_JPRB, 0.88354E+00_JPRB, 0.86638E+00_JPRB, 0.84418E+00_JPRB, 0.82264E+00_JPRB ,&
&  0.80062E+00_JPRB, 0.78142E+00_JPRB, 0.77243E+00_JPRB, 0.76583E+00_JPRB, 0.76473E+00_JPRB, 0.76208E+00_JPRB, 0.75510E+00_JPRB ,&
&  0.75043E+00_JPRB, 0.75056E+00_JPRB, 0.75855E+00_JPRB, 0.76618E+00_JPRB, 0.76830E+00_JPRB, 0.76680E+00_JPRB, 0.77642E+00_JPRB ,&
&  0.78602E+00_JPRB, 0.80359E+00_JPRB, 0.82614E+00_JPRB, 0.86043E+00_JPRB, 0.88870E+00_JPRB, 0.90333E+00_JPRB, 0.91361E+00_JPRB ,&
&  0.92518E+00_JPRB, 0.92184E+00_JPRB, 0.90539E+00_JPRB, 0.89228E+00_JPRB, 0.88522E+00_JPRB, 0.87619E+00_JPRB, 0.87122E+00_JPRB ,&
&  0.87611E+00_JPRB, 0.88214E+00_JPRB, 0.88268E+00_JPRB, 0.89059E+00_JPRB, 0.90316E+00_JPRB, 0.91508E+00_JPRB, 0.92244E+00_JPRB ,&
&  0.90569E+00_JPRB, 0.90791E+00_JPRB, 0.90719E+00_JPRB, 0.91852E+00_JPRB, 0.93124E+00_JPRB, 0.92026E+00_JPRB, 0.90713E+00_JPRB ,&
&  0.89405E+00_JPRB, 0.86837E+00_JPRB, 0.86530E+00_JPRB, 0.86399E+00_JPRB, 0.86608E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.82114E+00_JPRB, 0.81270E+00_JPRB, 0.80749E+00_JPRB, 0.81721E+00_JPRB, 0.83386E+00_JPRB, 0.84672E+00_JPRB, 0.84799E+00_JPRB ,&
&  0.84847E+00_JPRB, 0.85319E+00_JPRB, 0.86128E+00_JPRB, 0.86293E+00_JPRB, 0.85502E+00_JPRB, 0.83711E+00_JPRB, 0.82728E+00_JPRB ,&
&  0.82378E+00_JPRB, 0.82339E+00_JPRB, 0.82215E+00_JPRB, 0.82547E+00_JPRB, 0.84388E+00_JPRB, 0.87344E+00_JPRB, 0.90147E+00_JPRB ,&
&  0.91508E+00_JPRB, 0.90934E+00_JPRB, 0.90305E+00_JPRB, 0.89827E+00_JPRB, 0.89759E+00_JPRB, 0.90317E+00_JPRB, 0.90447E+00_JPRB ,&
&  0.89852E+00_JPRB, 0.75572E+00_JPRB, 0.81806E+00_JPRB, 0.84511E+00_JPRB, 0.87795E+00_JPRB, 0.91707E+00_JPRB, 0.98027E+00_JPRB ,&
&  0.10060E+01_JPRB, 0.99224E+00_JPRB, 0.96139E+00_JPRB, 0.93814E+00_JPRB, 0.91806E+00_JPRB, 0.90750E+00_JPRB, 0.89933E+00_JPRB ,&
&  0.89556E+00_JPRB, 0.89150E+00_JPRB, 0.86853E+00_JPRB, 0.83215E+00_JPRB, 0.81020E+00_JPRB, 0.79349E+00_JPRB, 0.77811E+00_JPRB ,&
&  0.76344E+00_JPRB, 0.74608E+00_JPRB, 0.72966E+00_JPRB, 0.71632E+00_JPRB, 0.71518E+00_JPRB, 0.71102E+00_JPRB, 0.69645E+00_JPRB ,&
&  0.68957E+00_JPRB, 0.68612E+00_JPRB, 0.69285E+00_JPRB, 0.70316E+00_JPRB, 0.70759E+00_JPRB, 0.70357E+00_JPRB, 0.71405E+00_JPRB ,&
&  0.73778E+00_JPRB, 0.75404E+00_JPRB, 0.76832E+00_JPRB, 0.78630E+00_JPRB, 0.79712E+00_JPRB, 0.80020E+00_JPRB, 0.80520E+00_JPRB ,&
&  0.81555E+00_JPRB, 0.81655E+00_JPRB, 0.80918E+00_JPRB, 0.80351E+00_JPRB, 0.80513E+00_JPRB, 0.80629E+00_JPRB, 0.81548E+00_JPRB ,&
&  0.82417E+00_JPRB, 0.82719E+00_JPRB, 0.82316E+00_JPRB, 0.82551E+00_JPRB, 0.83164E+00_JPRB, 0.84259E+00_JPRB, 0.84981E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85291E+00_JPRB, 0.83969E+00_JPRB, 0.84113E+00_JPRB, 0.84823E+00_JPRB, 0.84665E+00_JPRB, 0.83336E+00_JPRB ,&
&  0.82080E+00_JPRB, 0.81073E+00_JPRB, 0.81213E+00_JPRB, 0.81268E+00_JPRB, 0.81747E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.80212E+00_JPRB, 0.79819E+00_JPRB, 0.79700E+00_JPRB, 0.80827E+00_JPRB, 0.82623E+00_JPRB, 0.83103E+00_JPRB, 0.83331E+00_JPRB ,&
&  0.84656E+00_JPRB, 0.85669E+00_JPRB, 0.85671E+00_JPRB, 0.85516E+00_JPRB, 0.84892E+00_JPRB, 0.83444E+00_JPRB, 0.82305E+00_JPRB ,&
&  0.82753E+00_JPRB, 0.83229E+00_JPRB, 0.83337E+00_JPRB, 0.85626E+00_JPRB, 0.88694E+00_JPRB, 0.92373E+00_JPRB, 0.94857E+00_JPRB ,&
&  0.95720E+00_JPRB, 0.94965E+00_JPRB, 0.94597E+00_JPRB, 0.94095E+00_JPRB, 0.93578E+00_JPRB, 0.93779E+00_JPRB, 0.93234E+00_JPRB ,&
&  0.92233E+00_JPRB, 0.84255E+00_JPRB, 0.87640E+00_JPRB, 0.88714E+00_JPRB, 0.90491E+00_JPRB, 0.93332E+00_JPRB, 0.97655E+00_JPRB ,&
&  0.98652E+00_JPRB, 0.97869E+00_JPRB, 0.96013E+00_JPRB, 0.93001E+00_JPRB, 0.91062E+00_JPRB, 0.90038E+00_JPRB, 0.88207E+00_JPRB ,&
&  0.85379E+00_JPRB, 0.81636E+00_JPRB, 0.78286E+00_JPRB, 0.75976E+00_JPRB, 0.74841E+00_JPRB, 0.74362E+00_JPRB, 0.73590E+00_JPRB ,&
&  0.73064E+00_JPRB, 0.72593E+00_JPRB, 0.71265E+00_JPRB, 0.70064E+00_JPRB, 0.68786E+00_JPRB, 0.67828E+00_JPRB, 0.66326E+00_JPRB ,&
&  0.65440E+00_JPRB, 0.64267E+00_JPRB, 0.64114E+00_JPRB, 0.66065E+00_JPRB, 0.68200E+00_JPRB, 0.68761E+00_JPRB, 0.70197E+00_JPRB ,&
&  0.72181E+00_JPRB, 0.73131E+00_JPRB, 0.73405E+00_JPRB, 0.73196E+00_JPRB, 0.72517E+00_JPRB, 0.72509E+00_JPRB, 0.73390E+00_JPRB ,&
&  0.74231E+00_JPRB, 0.75343E+00_JPRB, 0.75818E+00_JPRB, 0.76357E+00_JPRB, 0.77070E+00_JPRB, 0.77617E+00_JPRB, 0.78701E+00_JPRB ,&
&  0.79264E+00_JPRB, 0.79657E+00_JPRB, 0.80761E+00_JPRB, 0.81898E+00_JPRB, 0.82429E+00_JPRB, 0.82732E+00_JPRB, 0.83747E+00_JPRB ,&
&  0.84209E+00_JPRB, 0.84089E+00_JPRB, 0.83216E+00_JPRB, 0.81984E+00_JPRB, 0.81169E+00_JPRB, 0.79926E+00_JPRB, 0.77928E+00_JPRB ,&
&  0.76045E+00_JPRB, 0.75756E+00_JPRB, 0.76640E+00_JPRB, 0.77625E+00_JPRB, 0.79313E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.78284E+00_JPRB, 0.79432E+00_JPRB, 0.80750E+00_JPRB, 0.82025E+00_JPRB, 0.83338E+00_JPRB, 0.83286E+00_JPRB, 0.83485E+00_JPRB ,&
&  0.85548E+00_JPRB, 0.85986E+00_JPRB, 0.86018E+00_JPRB, 0.85206E+00_JPRB, 0.84600E+00_JPRB, 0.84573E+00_JPRB, 0.83636E+00_JPRB ,&
&  0.83133E+00_JPRB, 0.83691E+00_JPRB, 0.85418E+00_JPRB, 0.88109E+00_JPRB, 0.90739E+00_JPRB, 0.94257E+00_JPRB, 0.96864E+00_JPRB ,&
&  0.97236E+00_JPRB, 0.96114E+00_JPRB, 0.96142E+00_JPRB, 0.96495E+00_JPRB, 0.96512E+00_JPRB, 0.94831E+00_JPRB, 0.94446E+00_JPRB ,&
&  0.92398E+00_JPRB, 0.94056E+00_JPRB, 0.81006E+00_JPRB, 0.88167E+00_JPRB, 0.92716E+00_JPRB, 0.96602E+00_JPRB, 0.98836E+00_JPRB ,&
&  0.98386E+00_JPRB, 0.97017E+00_JPRB, 0.94008E+00_JPRB, 0.90693E+00_JPRB, 0.89562E+00_JPRB, 0.87603E+00_JPRB, 0.85257E+00_JPRB ,&
&  0.80324E+00_JPRB, 0.73999E+00_JPRB, 0.70766E+00_JPRB, 0.69549E+00_JPRB, 0.68900E+00_JPRB, 0.68866E+00_JPRB, 0.69193E+00_JPRB ,&
&  0.69368E+00_JPRB, 0.69248E+00_JPRB, 0.67965E+00_JPRB, 0.66117E+00_JPRB, 0.63804E+00_JPRB, 0.62531E+00_JPRB, 0.62246E+00_JPRB ,&
&  0.61908E+00_JPRB, 0.60517E+00_JPRB, 0.60803E+00_JPRB, 0.63779E+00_JPRB, 0.66252E+00_JPRB, 0.68143E+00_JPRB, 0.69132E+00_JPRB ,&
&  0.69916E+00_JPRB, 0.69477E+00_JPRB, 0.69218E+00_JPRB, 0.68330E+00_JPRB, 0.65941E+00_JPRB, 0.65277E+00_JPRB, 0.67386E+00_JPRB ,&
&  0.69779E+00_JPRB, 0.71165E+00_JPRB, 0.72018E+00_JPRB, 0.73298E+00_JPRB, 0.74680E+00_JPRB, 0.75482E+00_JPRB, 0.76115E+00_JPRB ,&
&  0.76538E+00_JPRB, 0.76966E+00_JPRB, 0.78322E+00_JPRB, 0.79866E+00_JPRB, 0.80619E+00_JPRB, 0.80799E+00_JPRB, 0.80845E+00_JPRB ,&
&  0.80612E+00_JPRB, 0.79881E+00_JPRB, 0.79020E+00_JPRB, 0.78225E+00_JPRB, 0.76261E+00_JPRB, 0.73872E+00_JPRB, 0.71994E+00_JPRB ,&
&  0.70968E+00_JPRB, 0.71909E+00_JPRB, 0.72656E+00_JPRB, 0.73908E+00_JPRB, 0.76583E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.81660E+00_JPRB, 0.83042E+00_JPRB, 0.84183E+00_JPRB, 0.86232E+00_JPRB, 0.88068E+00_JPRB, 0.87773E+00_JPRB, 0.87123E+00_JPRB ,&
&  0.86736E+00_JPRB, 0.86390E+00_JPRB, 0.85769E+00_JPRB, 0.84879E+00_JPRB, 0.84283E+00_JPRB, 0.84247E+00_JPRB, 0.84282E+00_JPRB ,&
&  0.84392E+00_JPRB, 0.86344E+00_JPRB, 0.89568E+00_JPRB, 0.92075E+00_JPRB, 0.94821E+00_JPRB, 0.99064E+00_JPRB, 0.10103E+01_JPRB ,&
&  0.99258E+00_JPRB, 0.99091E+00_JPRB, 0.98006E+00_JPRB, 0.96579E+00_JPRB, 0.96805E+00_JPRB, 0.95952E+00_JPRB, 0.93235E+00_JPRB ,&
&  0.92701E+00_JPRB, 0.92476E+00_JPRB, 0.92096E+00_JPRB, 0.95149E+00_JPRB, 0.97757E+00_JPRB, 0.98268E+00_JPRB, 0.98528E+00_JPRB ,&
&  0.96255E+00_JPRB, 0.92550E+00_JPRB, 0.88524E+00_JPRB, 0.86333E+00_JPRB, 0.86017E+00_JPRB, 0.84048E+00_JPRB, 0.81268E+00_JPRB ,&
&  0.75499E+00_JPRB, 0.70716E+00_JPRB, 0.68030E+00_JPRB, 0.66382E+00_JPRB, 0.65787E+00_JPRB, 0.65412E+00_JPRB, 0.64847E+00_JPRB ,&
&  0.65268E+00_JPRB, 0.64606E+00_JPRB, 0.61588E+00_JPRB, 0.58358E+00_JPRB, 0.57651E+00_JPRB, 0.58502E+00_JPRB, 0.60563E+00_JPRB ,&
&  0.62058E+00_JPRB, 0.61506E+00_JPRB, 0.62137E+00_JPRB, 0.62912E+00_JPRB, 0.64701E+00_JPRB, 0.66628E+00_JPRB, 0.66922E+00_JPRB ,&
&  0.67113E+00_JPRB, 0.66727E+00_JPRB, 0.65503E+00_JPRB, 0.64628E+00_JPRB, 0.63467E+00_JPRB, 0.62956E+00_JPRB, 0.64040E+00_JPRB ,&
&  0.65762E+00_JPRB, 0.66722E+00_JPRB, 0.67764E+00_JPRB, 0.69136E+00_JPRB, 0.70344E+00_JPRB, 0.71255E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.72960E+00_JPRB, 0.74457E+00_JPRB, 0.75984E+00_JPRB, 0.77335E+00_JPRB, 0.77376E+00_JPRB, 0.75953E+00_JPRB, 0.73401E+00_JPRB ,&
&  0.71702E+00_JPRB, 0.70387E+00_JPRB, 0.70121E+00_JPRB, 0.70765E+00_JPRB, 0.70291E+00_JPRB, 0.68324E+00_JPRB, 0.67317E+00_JPRB ,&
&  0.68370E+00_JPRB, 0.70760E+00_JPRB, 0.73026E+00_JPRB, 0.76105E+00_JPRB, 0.79330E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.85039E+00_JPRB, 0.88159E+00_JPRB, 0.90502E+00_JPRB, 0.91256E+00_JPRB, 0.90970E+00_JPRB, 0.90096E+00_JPRB, 0.87890E+00_JPRB ,&
&  0.85482E+00_JPRB, 0.83816E+00_JPRB, 0.82045E+00_JPRB, 0.79435E+00_JPRB, 0.77271E+00_JPRB, 0.76845E+00_JPRB, 0.78111E+00_JPRB ,&
&  0.82739E+00_JPRB, 0.86832E+00_JPRB, 0.88461E+00_JPRB, 0.89780E+00_JPRB, 0.94691E+00_JPRB, 0.98799E+00_JPRB, 0.99625E+00_JPRB ,&
&  0.98974E+00_JPRB, 0.99049E+00_JPRB, 0.95389E+00_JPRB, 0.92109E+00_JPRB, 0.92203E+00_JPRB, 0.92859E+00_JPRB, 0.91198E+00_JPRB ,&
&  0.93905E+00_JPRB, 0.95866E+00_JPRB, 0.95561E+00_JPRB, 0.93674E+00_JPRB, 0.93338E+00_JPRB, 0.92343E+00_JPRB, 0.91077E+00_JPRB ,&
&  0.88840E+00_JPRB, 0.84008E+00_JPRB, 0.80534E+00_JPRB, 0.80020E+00_JPRB, 0.79291E+00_JPRB, 0.78652E+00_JPRB, 0.77391E+00_JPRB ,&
&  0.73370E+00_JPRB, 0.69712E+00_JPRB, 0.66904E+00_JPRB, 0.64962E+00_JPRB, 0.63708E+00_JPRB, 0.62688E+00_JPRB, 0.61572E+00_JPRB ,&
&  0.60569E+00_JPRB, 0.57565E+00_JPRB, 0.54488E+00_JPRB, 0.53649E+00_JPRB, 0.54081E+00_JPRB, 0.54809E+00_JPRB, 0.57624E+00_JPRB ,&
&  0.60127E+00_JPRB, 0.59961E+00_JPRB, 0.59600E+00_JPRB, 0.59906E+00_JPRB, 0.60906E+00_JPRB, 0.61124E+00_JPRB, 0.60266E+00_JPRB ,&
&  0.59893E+00_JPRB, 0.59750E+00_JPRB, 0.58831E+00_JPRB, 0.58142E+00_JPRB, 0.58104E+00_JPRB, 0.58486E+00_JPRB, 0.59874E+00_JPRB ,&
&  0.60706E+00_JPRB, 0.60781E+00_JPRB, 0.62091E+00_JPRB, 0.63789E+00_JPRB, 0.64877E+00_JPRB, 0.65764E+00_JPRB, 0.66425E+00_JPRB ,&
&  0.67846E+00_JPRB, 0.69849E+00_JPRB, 0.71366E+00_JPRB, 0.71593E+00_JPRB, 0.69331E+00_JPRB, 0.66928E+00_JPRB, 0.65054E+00_JPRB ,&
&  0.64230E+00_JPRB, 0.64673E+00_JPRB, 0.65459E+00_JPRB, 0.66227E+00_JPRB, 0.66914E+00_JPRB, 0.67387E+00_JPRB, 0.67350E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.72450E+00_JPRB, 0.74945E+00_JPRB, 0.77911E+00_JPRB, 0.81031E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.91402E+00_JPRB, 0.93126E+00_JPRB, 0.94245E+00_JPRB, 0.94242E+00_JPRB, 0.92629E+00_JPRB, 0.89935E+00_JPRB, 0.87653E+00_JPRB ,&
&  0.85064E+00_JPRB, 0.82543E+00_JPRB, 0.80150E+00_JPRB, 0.76852E+00_JPRB, 0.75072E+00_JPRB, 0.75395E+00_JPRB, 0.80271E+00_JPRB ,&
&  0.86811E+00_JPRB, 0.87311E+00_JPRB, 0.86702E+00_JPRB, 0.89181E+00_JPRB, 0.94343E+00_JPRB, 0.97454E+00_JPRB, 0.99475E+00_JPRB ,&
&  0.99587E+00_JPRB, 0.96179E+00_JPRB, 0.92904E+00_JPRB, 0.91684E+00_JPRB, 0.94178E+00_JPRB, 0.96108E+00_JPRB, 0.96720E+00_JPRB ,&
&  0.96382E+00_JPRB, 0.94468E+00_JPRB, 0.92867E+00_JPRB, 0.91545E+00_JPRB, 0.92122E+00_JPRB, 0.89658E+00_JPRB, 0.84386E+00_JPRB ,&
&  0.77740E+00_JPRB, 0.75030E+00_JPRB, 0.73602E+00_JPRB, 0.72032E+00_JPRB, 0.70956E+00_JPRB, 0.71026E+00_JPRB, 0.70183E+00_JPRB ,&
&  0.68821E+00_JPRB, 0.67146E+00_JPRB, 0.65486E+00_JPRB, 0.63572E+00_JPRB, 0.62044E+00_JPRB, 0.59900E+00_JPRB, 0.57893E+00_JPRB ,&
&  0.56038E+00_JPRB, 0.53726E+00_JPRB, 0.51564E+00_JPRB, 0.49989E+00_JPRB, 0.49228E+00_JPRB, 0.50286E+00_JPRB, 0.54371E+00_JPRB ,&
&  0.57554E+00_JPRB, 0.57267E+00_JPRB, 0.57073E+00_JPRB, 0.57591E+00_JPRB, 0.57041E+00_JPRB, 0.56166E+00_JPRB, 0.55713E+00_JPRB ,&
&  0.55280E+00_JPRB, 0.54953E+00_JPRB, 0.54874E+00_JPRB, 0.55148E+00_JPRB, 0.55417E+00_JPRB, 0.55228E+00_JPRB, 0.55372E+00_JPRB ,&
&  0.55707E+00_JPRB, 0.56165E+00_JPRB, 0.57854E+00_JPRB, 0.60295E+00_JPRB, 0.62437E+00_JPRB, 0.64040E+00_JPRB, 0.65525E+00_JPRB ,&
&  0.67734E+00_JPRB, 0.69090E+00_JPRB, 0.67860E+00_JPRB, 0.66716E+00_JPRB, 0.66174E+00_JPRB, 0.66750E+00_JPRB, 0.67826E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.71104E+00_JPRB, 0.71723E+00_JPRB, 0.71854E+00_JPRB, 0.72147E+00_JPRB, 0.72219E+00_JPRB, 0.73498E+00_JPRB ,&
&  0.76919E+00_JPRB, 0.79855E+00_JPRB, 0.80556E+00_JPRB, 0.83780E+00_JPRB, 0.87926E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.95086E+00_JPRB, 0.95457E+00_JPRB, 0.94426E+00_JPRB, 0.93954E+00_JPRB, 0.92869E+00_JPRB, 0.91478E+00_JPRB, 0.89247E+00_JPRB ,&
&  0.86640E+00_JPRB, 0.83832E+00_JPRB, 0.81133E+00_JPRB, 0.78902E+00_JPRB, 0.79223E+00_JPRB, 0.82412E+00_JPRB, 0.85126E+00_JPRB ,&
&  0.84032E+00_JPRB, 0.83626E+00_JPRB, 0.85288E+00_JPRB, 0.88293E+00_JPRB, 0.91550E+00_JPRB, 0.94549E+00_JPRB, 0.96648E+00_JPRB ,&
&  0.98315E+00_JPRB, 0.96485E+00_JPRB, 0.96985E+00_JPRB, 0.97339E+00_JPRB, 0.96858E+00_JPRB, 0.95149E+00_JPRB, 0.92333E+00_JPRB ,&
&  0.89532E+00_JPRB, 0.86808E+00_JPRB, 0.83460E+00_JPRB, 0.77768E+00_JPRB, 0.85165E+00_JPRB, 0.77873E+00_JPRB, 0.75134E+00_JPRB ,&
&  0.72387E+00_JPRB, 0.70497E+00_JPRB, 0.68294E+00_JPRB, 0.65998E+00_JPRB, 0.65214E+00_JPRB, 0.64751E+00_JPRB, 0.63923E+00_JPRB ,&
&  0.63200E+00_JPRB, 0.62354E+00_JPRB, 0.61095E+00_JPRB, 0.59578E+00_JPRB, 0.58192E+00_JPRB, 0.56752E+00_JPRB, 0.55429E+00_JPRB ,&
&  0.53468E+00_JPRB, 0.50595E+00_JPRB, 0.47572E+00_JPRB, 0.45318E+00_JPRB, 0.44113E+00_JPRB, 0.45065E+00_JPRB, 0.48484E+00_JPRB ,&
&  0.51978E+00_JPRB, 0.52603E+00_JPRB, 0.53651E+00_JPRB, 0.54070E+00_JPRB, 0.53307E+00_JPRB, 0.52287E+00_JPRB, 0.50538E+00_JPRB ,&
&  0.49282E+00_JPRB, 0.49193E+00_JPRB, 0.49423E+00_JPRB, 0.49952E+00_JPRB, 0.50445E+00_JPRB, 0.51100E+00_JPRB, 0.52005E+00_JPRB ,&
&  0.52816E+00_JPRB, 0.53093E+00_JPRB, 0.54318E+00_JPRB, 0.56994E+00_JPRB, 0.60150E+00_JPRB, 0.63395E+00_JPRB, 0.66226E+00_JPRB ,&
&  0.68210E+00_JPRB, 0.69634E+00_JPRB, 0.70513E+00_JPRB, 0.72039E+00_JPRB, 0.73624E+00_JPRB, 0.75173E+00_JPRB, 0.75947E+00_JPRB ,&
&  0.75652E+00_JPRB, 0.76060E+00_JPRB, 0.77888E+00_JPRB, 0.80786E+00_JPRB, 0.82781E+00_JPRB, 0.86173E+00_JPRB, 0.88299E+00_JPRB ,&
&  0.90532E+00_JPRB, 0.91089E+00_JPRB, 0.90534E+00_JPRB, 0.91768E+00_JPRB, 0.93404E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.93358E+00_JPRB, 0.94391E+00_JPRB, 0.94156E+00_JPRB, 0.94017E+00_JPRB, 0.94146E+00_JPRB, 0.93403E+00_JPRB, 0.92016E+00_JPRB ,&
&  0.90614E+00_JPRB, 0.87939E+00_JPRB, 0.84819E+00_JPRB, 0.83375E+00_JPRB, 0.84028E+00_JPRB, 0.83941E+00_JPRB, 0.82063E+00_JPRB ,&
&  0.81882E+00_JPRB, 0.83213E+00_JPRB, 0.85462E+00_JPRB, 0.87963E+00_JPRB, 0.91323E+00_JPRB, 0.94818E+00_JPRB, 0.97408E+00_JPRB ,&
&  0.98454E+00_JPRB, 0.98337E+00_JPRB, 0.97638E+00_JPRB, 0.94997E+00_JPRB, 0.91440E+00_JPRB, 0.88753E+00_JPRB, 0.86700E+00_JPRB ,&
&  0.84108E+00_JPRB, 0.79136E+00_JPRB, 0.72350E+00_JPRB, 0.71816E+00_JPRB, 0.65758E+00_JPRB, 0.64015E+00_JPRB, 0.66325E+00_JPRB ,&
&  0.67878E+00_JPRB, 0.66091E+00_JPRB, 0.63618E+00_JPRB, 0.61558E+00_JPRB, 0.61099E+00_JPRB, 0.60887E+00_JPRB, 0.60301E+00_JPRB ,&
&  0.59839E+00_JPRB, 0.59100E+00_JPRB, 0.57627E+00_JPRB, 0.56289E+00_JPRB, 0.55024E+00_JPRB, 0.53760E+00_JPRB, 0.52852E+00_JPRB ,&
&  0.49878E+00_JPRB, 0.45398E+00_JPRB, 0.42048E+00_JPRB, 0.40531E+00_JPRB, 0.39686E+00_JPRB, 0.40093E+00_JPRB, 0.41493E+00_JPRB ,&
&  0.42694E+00_JPRB, 0.42896E+00_JPRB, 0.42422E+00_JPRB, 0.41100E+00_JPRB, 0.38840E+00_JPRB, 0.34671E+00_JPRB, 0.32191E+00_JPRB ,&
&  0.33975E+00_JPRB, 0.37092E+00_JPRB, 0.40382E+00_JPRB, 0.41811E+00_JPRB, 0.42703E+00_JPRB, 0.42790E+00_JPRB, 0.41161E+00_JPRB ,&
&  0.38120E+00_JPRB, 0.35560E+00_JPRB, 0.35185E+00_JPRB, 0.35406E+00_JPRB, 0.36050E+00_JPRB, 0.37158E+00_JPRB, 0.38844E+00_JPRB ,&
&  0.41367E+00_JPRB, 0.45046E+00_JPRB, 0.47315E+00_JPRB, 0.47584E+00_JPRB, 0.46844E+00_JPRB, 0.45752E+00_JPRB, 0.45481E+00_JPRB ,&
&  0.46972E+00_JPRB, 0.52931E+00_JPRB, 0.64957E+00_JPRB, 0.79329E+00_JPRB, 0.79678E+00_JPRB, 0.84181E+00_JPRB, 0.87645E+00_JPRB ,&
&  0.90654E+00_JPRB, 0.92029E+00_JPRB, 0.90729E+00_JPRB, 0.90792E+00_JPRB, 0.91393E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.89032E+00_JPRB, 0.91011E+00_JPRB, 0.92076E+00_JPRB, 0.93445E+00_JPRB, 0.93447E+00_JPRB, 0.90420E+00_JPRB, 0.88976E+00_JPRB ,&
&  0.89419E+00_JPRB, 0.88874E+00_JPRB, 0.87689E+00_JPRB, 0.86911E+00_JPRB, 0.85970E+00_JPRB, 0.84976E+00_JPRB, 0.84170E+00_JPRB ,&
&  0.83336E+00_JPRB, 0.82793E+00_JPRB, 0.83438E+00_JPRB, 0.84779E+00_JPRB, 0.85868E+00_JPRB, 0.86448E+00_JPRB, 0.86950E+00_JPRB ,&
&  0.86923E+00_JPRB, 0.83656E+00_JPRB, 0.79114E+00_JPRB, 0.75796E+00_JPRB, 0.74199E+00_JPRB, 0.72856E+00_JPRB, 0.69486E+00_JPRB ,&
&  0.62408E+00_JPRB, 0.50298E+00_JPRB, 0.44931E+00_JPRB, 0.49225E+00_JPRB, 0.60640E+00_JPRB, 0.57460E+00_JPRB, 0.59629E+00_JPRB ,&
&  0.62761E+00_JPRB, 0.64395E+00_JPRB, 0.63969E+00_JPRB, 0.62887E+00_JPRB, 0.62487E+00_JPRB, 0.62815E+00_JPRB, 0.62492E+00_JPRB ,&
&  0.62154E+00_JPRB, 0.60545E+00_JPRB, 0.59558E+00_JPRB, 0.58321E+00_JPRB, 0.56436E+00_JPRB, 0.53791E+00_JPRB, 0.52132E+00_JPRB ,&
&  0.48138E+00_JPRB, 0.44224E+00_JPRB, 0.43013E+00_JPRB, 0.42952E+00_JPRB, 0.39833E+00_JPRB, 0.38575E+00_JPRB, 0.37007E+00_JPRB ,&
&  0.33264E+00_JPRB, 0.31191E+00_JPRB, 0.28141E+00_JPRB, 0.26265E+00_JPRB, 0.24979E+00_JPRB, 0.23488E+00_JPRB, 0.23268E+00_JPRB ,&
&  0.23874E+00_JPRB, 0.25289E+00_JPRB, 0.27347E+00_JPRB, 0.30358E+00_JPRB, 0.32977E+00_JPRB, 0.29454E+00_JPRB, 0.27909E+00_JPRB ,&
&  0.26405E+00_JPRB, 0.26052E+00_JPRB, 0.26759E+00_JPRB, 0.27554E+00_JPRB, 0.28334E+00_JPRB, 0.29245E+00_JPRB, 0.30175E+00_JPRB ,&
&  0.30856E+00_JPRB, 0.31164E+00_JPRB, 0.31190E+00_JPRB, 0.31096E+00_JPRB, 0.30909E+00_JPRB, 0.30930E+00_JPRB, 0.31259E+00_JPRB ,&
&  0.31972E+00_JPRB, 0.33272E+00_JPRB, 0.35688E+00_JPRB, 0.39253E+00_JPRB, 0.43828E+00_JPRB, 0.49888E+00_JPRB, 0.59032E+00_JPRB ,&
&  0.71446E+00_JPRB, 0.87729E+00_JPRB, 0.92470E+00_JPRB, 0.93406E+00_JPRB, 0.90221E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.89337E+00_JPRB, 0.89016E+00_JPRB, 0.89679E+00_JPRB, 0.88636E+00_JPRB, 0.86736E+00_JPRB, 0.85846E+00_JPRB, 0.87543E+00_JPRB ,&
&  0.89504E+00_JPRB, 0.86512E+00_JPRB, 0.84296E+00_JPRB, 0.82353E+00_JPRB, 0.80913E+00_JPRB, 0.79484E+00_JPRB, 0.77303E+00_JPRB ,&
&  0.75289E+00_JPRB, 0.74527E+00_JPRB, 0.74295E+00_JPRB, 0.73763E+00_JPRB, 0.72724E+00_JPRB, 0.71714E+00_JPRB, 0.69952E+00_JPRB ,&
&  0.66585E+00_JPRB, 0.63026E+00_JPRB, 0.61205E+00_JPRB, 0.58819E+00_JPRB, 0.57144E+00_JPRB, 0.55199E+00_JPRB, 0.52442E+00_JPRB ,&
&  0.47819E+00_JPRB, 0.42029E+00_JPRB, 0.38757E+00_JPRB, 0.42613E+00_JPRB, 0.51793E+00_JPRB, 0.51051E+00_JPRB, 0.53144E+00_JPRB ,&
&  0.56690E+00_JPRB, 0.58829E+00_JPRB, 0.59907E+00_JPRB, 0.60249E+00_JPRB, 0.61174E+00_JPRB, 0.61809E+00_JPRB, 0.61436E+00_JPRB ,&
&  0.59842E+00_JPRB, 0.54573E+00_JPRB, 0.50126E+00_JPRB, 0.39610E+00_JPRB, 0.32620E+00_JPRB, 0.28903E+00_JPRB, 0.25836E+00_JPRB ,&
&  0.23789E+00_JPRB, 0.22651E+00_JPRB, 0.22410E+00_JPRB, 0.22212E+00_JPRB, 0.22795E+00_JPRB, 0.23584E+00_JPRB, 0.23639E+00_JPRB ,&
&  0.23188E+00_JPRB, 0.22311E+00_JPRB, 0.21153E+00_JPRB, 0.20534E+00_JPRB, 0.20187E+00_JPRB, 0.20223E+00_JPRB, 0.20453E+00_JPRB ,&
&  0.20554E+00_JPRB, 0.20854E+00_JPRB, 0.22125E+00_JPRB, 0.23165E+00_JPRB, 0.24522E+00_JPRB, 0.22955E+00_JPRB, 0.21577E+00_JPRB ,&
&  0.21010E+00_JPRB, 0.21574E+00_JPRB, 0.22392E+00_JPRB, 0.22993E+00_JPRB, 0.23617E+00_JPRB, 0.24113E+00_JPRB, 0.24597E+00_JPRB ,&
&  0.25126E+00_JPRB, 0.25513E+00_JPRB, 0.25675E+00_JPRB, 0.25853E+00_JPRB, 0.26007E+00_JPRB, 0.26250E+00_JPRB, 0.26533E+00_JPRB ,&
&  0.27017E+00_JPRB, 0.28006E+00_JPRB, 0.29279E+00_JPRB, 0.31397E+00_JPRB, 0.33383E+00_JPRB, 0.35345E+00_JPRB, 0.37692E+00_JPRB ,&
&  0.39701E+00_JPRB, 0.47856E+00_JPRB, 0.67104E+00_JPRB, 0.98839E+00_JPRB, 0.98196E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.83829E+00_JPRB, 0.85023E+00_JPRB, 0.86667E+00_JPRB, 0.84318E+00_JPRB, 0.82807E+00_JPRB, 0.83126E+00_JPRB, 0.83191E+00_JPRB ,&
&  0.80830E+00_JPRB, 0.77198E+00_JPRB, 0.72181E+00_JPRB, 0.67937E+00_JPRB, 0.61558E+00_JPRB, 0.53322E+00_JPRB, 0.49520E+00_JPRB ,&
&  0.45988E+00_JPRB, 0.45580E+00_JPRB, 0.45745E+00_JPRB, 0.47345E+00_JPRB, 0.48186E+00_JPRB, 0.47829E+00_JPRB, 0.45571E+00_JPRB ,&
&  0.42829E+00_JPRB, 0.40160E+00_JPRB, 0.38387E+00_JPRB, 0.37043E+00_JPRB, 0.37045E+00_JPRB, 0.36845E+00_JPRB, 0.36192E+00_JPRB ,&
&  0.35431E+00_JPRB, 0.35984E+00_JPRB, 0.37798E+00_JPRB, 0.40496E+00_JPRB, 0.45197E+00_JPRB, 0.43173E+00_JPRB, 0.44051E+00_JPRB ,&
&  0.47130E+00_JPRB, 0.49351E+00_JPRB, 0.50234E+00_JPRB, 0.50556E+00_JPRB, 0.49784E+00_JPRB, 0.51974E+00_JPRB, 0.46291E+00_JPRB ,&
&  0.35668E+00_JPRB, 0.29736E+00_JPRB, 0.25474E+00_JPRB, 0.22619E+00_JPRB, 0.20613E+00_JPRB, 0.19412E+00_JPRB, 0.18047E+00_JPRB ,&
&  0.17395E+00_JPRB, 0.16892E+00_JPRB, 0.16801E+00_JPRB, 0.17058E+00_JPRB, 0.17402E+00_JPRB, 0.17838E+00_JPRB, 0.18007E+00_JPRB ,&
&  0.18269E+00_JPRB, 0.18292E+00_JPRB, 0.18234E+00_JPRB, 0.18255E+00_JPRB, 0.18146E+00_JPRB, 0.18230E+00_JPRB, 0.18221E+00_JPRB ,&
&  0.18177E+00_JPRB, 0.18120E+00_JPRB, 0.18299E+00_JPRB, 0.18423E+00_JPRB, 0.18516E+00_JPRB, 0.18210E+00_JPRB, 0.18094E+00_JPRB ,&
&  0.18175E+00_JPRB, 0.18442E+00_JPRB, 0.18842E+00_JPRB, 0.19281E+00_JPRB, 0.19886E+00_JPRB, 0.20343E+00_JPRB, 0.20877E+00_JPRB ,&
&  0.21335E+00_JPRB, 0.21794E+00_JPRB, 0.22239E+00_JPRB, 0.22694E+00_JPRB, 0.23198E+00_JPRB, 0.23699E+00_JPRB, 0.24210E+00_JPRB ,&
&  0.24751E+00_JPRB, 0.25642E+00_JPRB, 0.26722E+00_JPRB, 0.28597E+00_JPRB, 0.31107E+00_JPRB, 0.33648E+00_JPRB, 0.38199E+00_JPRB ,&
&  0.43777E+00_JPRB, 0.66438E+00_JPRB, 0.96903E+00_JPRB, 0.80427E+00_JPRB, 0.82134E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.82702E+00_JPRB, 0.84513E+00_JPRB, 0.86066E+00_JPRB, 0.85787E+00_JPRB, 0.84040E+00_JPRB, 0.78278E+00_JPRB, 0.71928E+00_JPRB ,&
&  0.64061E+00_JPRB, 0.57514E+00_JPRB, 0.50551E+00_JPRB, 0.44575E+00_JPRB, 0.39511E+00_JPRB, 0.35590E+00_JPRB, 0.33109E+00_JPRB ,&
&  0.31311E+00_JPRB, 0.30511E+00_JPRB, 0.30122E+00_JPRB, 0.30562E+00_JPRB, 0.30741E+00_JPRB, 0.31279E+00_JPRB, 0.30702E+00_JPRB ,&
&  0.29990E+00_JPRB, 0.29235E+00_JPRB, 0.29019E+00_JPRB, 0.29873E+00_JPRB, 0.31238E+00_JPRB, 0.32552E+00_JPRB, 0.34278E+00_JPRB ,&
&  0.36114E+00_JPRB, 0.36704E+00_JPRB, 0.38199E+00_JPRB, 0.38552E+00_JPRB, 0.37136E+00_JPRB, 0.37350E+00_JPRB, 0.37957E+00_JPRB ,&
&  0.39301E+00_JPRB, 0.41157E+00_JPRB, 0.42213E+00_JPRB, 0.44151E+00_JPRB, 0.38642E+00_JPRB, 0.32641E+00_JPRB, 0.27368E+00_JPRB ,&
&  0.23423E+00_JPRB, 0.21140E+00_JPRB, 0.19451E+00_JPRB, 0.18597E+00_JPRB, 0.17711E+00_JPRB, 0.17231E+00_JPRB, 0.16739E+00_JPRB ,&
&  0.16526E+00_JPRB, 0.16554E+00_JPRB, 0.16536E+00_JPRB, 0.16548E+00_JPRB, 0.16646E+00_JPRB, 0.16782E+00_JPRB, 0.16760E+00_JPRB ,&
&  0.16722E+00_JPRB, 0.16718E+00_JPRB, 0.16686E+00_JPRB, 0.16652E+00_JPRB, 0.16563E+00_JPRB, 0.16555E+00_JPRB, 0.16359E+00_JPRB ,&
&  0.16258E+00_JPRB, 0.16095E+00_JPRB, 0.16006E+00_JPRB, 0.16016E+00_JPRB, 0.16077E+00_JPRB, 0.16134E+00_JPRB, 0.16228E+00_JPRB ,&
&  0.16392E+00_JPRB, 0.16539E+00_JPRB, 0.16779E+00_JPRB, 0.16990E+00_JPRB, 0.17339E+00_JPRB, 0.17629E+00_JPRB, 0.18034E+00_JPRB ,&
&  0.18467E+00_JPRB, 0.18990E+00_JPRB, 0.19608E+00_JPRB, 0.20311E+00_JPRB, 0.20882E+00_JPRB, 0.21434E+00_JPRB, 0.22023E+00_JPRB ,&
&  0.22475E+00_JPRB, 0.23287E+00_JPRB, 0.24190E+00_JPRB, 0.25704E+00_JPRB, 0.28882E+00_JPRB, 0.31168E+00_JPRB, 0.35933E+00_JPRB ,&
&  0.43613E+00_JPRB, 0.68667E+00_JPRB, 0.92078E+00_JPRB, 0.77711E+00_JPRB, 0.80873E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.70397E+00_JPRB, 0.73503E+00_JPRB, 0.75842E+00_JPRB, 0.75994E+00_JPRB, 0.75798E+00_JPRB, 0.72026E+00_JPRB, 0.66147E+00_JPRB ,&
&  0.55566E+00_JPRB, 0.48637E+00_JPRB, 0.44171E+00_JPRB, 0.39949E+00_JPRB, 0.36147E+00_JPRB, 0.33524E+00_JPRB, 0.31758E+00_JPRB ,&
&  0.30429E+00_JPRB, 0.29691E+00_JPRB, 0.29135E+00_JPRB, 0.28671E+00_JPRB, 0.28156E+00_JPRB, 0.27662E+00_JPRB, 0.26800E+00_JPRB ,&
&  0.26260E+00_JPRB, 0.25951E+00_JPRB, 0.25782E+00_JPRB, 0.26021E+00_JPRB, 0.26713E+00_JPRB, 0.27337E+00_JPRB, 0.28369E+00_JPRB ,&
&  0.29496E+00_JPRB, 0.30942E+00_JPRB, 0.31946E+00_JPRB, 0.32346E+00_JPRB, 0.32675E+00_JPRB, 0.32558E+00_JPRB, 0.30398E+00_JPRB ,&
&  0.30873E+00_JPRB, 0.32958E+00_JPRB, 0.34767E+00_JPRB, 0.35537E+00_JPRB, 0.30898E+00_JPRB, 0.26761E+00_JPRB, 0.22679E+00_JPRB ,&
&  0.20812E+00_JPRB, 0.19200E+00_JPRB, 0.18332E+00_JPRB, 0.17652E+00_JPRB, 0.17097E+00_JPRB, 0.16830E+00_JPRB, 0.16598E+00_JPRB ,&
&  0.16418E+00_JPRB, 0.16382E+00_JPRB, 0.16272E+00_JPRB, 0.16259E+00_JPRB, 0.16195E+00_JPRB, 0.16215E+00_JPRB, 0.16106E+00_JPRB ,&
&  0.15990E+00_JPRB, 0.15894E+00_JPRB, 0.15833E+00_JPRB, 0.15802E+00_JPRB, 0.15703E+00_JPRB, 0.15606E+00_JPRB, 0.15374E+00_JPRB ,&
&  0.15219E+00_JPRB, 0.15070E+00_JPRB, 0.14964E+00_JPRB, 0.14995E+00_JPRB, 0.15082E+00_JPRB, 0.15333E+00_JPRB, 0.15472E+00_JPRB ,&
&  0.15499E+00_JPRB, 0.15510E+00_JPRB, 0.15504E+00_JPRB, 0.15591E+00_JPRB, 0.15718E+00_JPRB, 0.15937E+00_JPRB, 0.16245E+00_JPRB ,&
&  0.16618E+00_JPRB, 0.17196E+00_JPRB, 0.17759E+00_JPRB, 0.18373E+00_JPRB, 0.18929E+00_JPRB, 0.19440E+00_JPRB, 0.20150E+00_JPRB ,&
&  0.20864E+00_JPRB, 0.21739E+00_JPRB, 0.23069E+00_JPRB, 0.24443E+00_JPRB, 0.26951E+00_JPRB, 0.28884E+00_JPRB, 0.31831E+00_JPRB ,&
&  0.38655E+00_JPRB, 0.53101E+00_JPRB, 0.61736E+00_JPRB, 0.63194E+00_JPRB, 0.66573E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.51902E+00_JPRB, 0.53082E+00_JPRB, 0.54600E+00_JPRB, 0.55518E+00_JPRB, 0.56601E+00_JPRB, 0.53360E+00_JPRB, 0.50794E+00_JPRB ,&
&  0.45505E+00_JPRB, 0.42452E+00_JPRB, 0.39907E+00_JPRB, 0.37571E+00_JPRB, 0.35482E+00_JPRB, 0.33550E+00_JPRB, 0.31804E+00_JPRB ,&
&  0.30689E+00_JPRB, 0.30035E+00_JPRB, 0.29157E+00_JPRB, 0.28142E+00_JPRB, 0.27048E+00_JPRB, 0.26089E+00_JPRB, 0.25635E+00_JPRB ,&
&  0.25143E+00_JPRB, 0.24796E+00_JPRB, 0.24649E+00_JPRB, 0.24611E+00_JPRB, 0.24780E+00_JPRB, 0.24763E+00_JPRB, 0.24796E+00_JPRB ,&
&  0.24392E+00_JPRB, 0.25133E+00_JPRB, 0.26124E+00_JPRB, 0.26579E+00_JPRB, 0.27453E+00_JPRB, 0.27351E+00_JPRB, 0.26897E+00_JPRB ,&
&  0.26130E+00_JPRB, 0.25275E+00_JPRB, 0.24270E+00_JPRB, 0.23024E+00_JPRB, 0.21987E+00_JPRB, 0.21246E+00_JPRB, 0.20503E+00_JPRB ,&
&  0.20075E+00_JPRB, 0.19501E+00_JPRB, 0.18773E+00_JPRB, 0.18133E+00_JPRB, 0.17576E+00_JPRB, 0.17136E+00_JPRB, 0.16724E+00_JPRB ,&
&  0.16449E+00_JPRB, 0.16304E+00_JPRB, 0.16167E+00_JPRB, 0.16170E+00_JPRB, 0.16077E+00_JPRB, 0.15971E+00_JPRB, 0.15862E+00_JPRB ,&
&  0.15811E+00_JPRB, 0.15678E+00_JPRB, 0.15568E+00_JPRB, 0.15470E+00_JPRB, 0.15312E+00_JPRB, 0.15176E+00_JPRB, 0.15021E+00_JPRB ,&
&  0.14892E+00_JPRB, 0.14834E+00_JPRB, 0.14802E+00_JPRB, 0.14804E+00_JPRB, 0.14848E+00_JPRB, 0.14849E+00_JPRB, 0.14888E+00_JPRB ,&
&  0.14931E+00_JPRB, 0.14992E+00_JPRB, 0.14991E+00_JPRB, 0.15034E+00_JPRB, 0.15138E+00_JPRB, 0.15280E+00_JPRB, 0.15627E+00_JPRB ,&
&  0.16018E+00_JPRB, 0.16514E+00_JPRB, 0.17042E+00_JPRB, 0.17618E+00_JPRB, 0.18188E+00_JPRB, 0.18736E+00_JPRB, 0.19345E+00_JPRB ,&
&  0.20137E+00_JPRB, 0.20859E+00_JPRB, 0.21726E+00_JPRB, 0.23065E+00_JPRB, 0.26117E+00_JPRB, 0.27684E+00_JPRB, 0.29057E+00_JPRB ,&
&  0.31505E+00_JPRB, 0.34622E+00_JPRB, 0.45006E+00_JPRB, 0.48207E+00_JPRB, 0.49946E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.26390E+00_JPRB, 0.26346E+00_JPRB, 0.26353E+00_JPRB, 0.26912E+00_JPRB, 0.27679E+00_JPRB, 0.35443E+00_JPRB, 0.34454E+00_JPRB ,&
&  0.33656E+00_JPRB, 0.30899E+00_JPRB, 0.30606E+00_JPRB, 0.29938E+00_JPRB, 0.29013E+00_JPRB, 0.28311E+00_JPRB, 0.27919E+00_JPRB ,&
&  0.27474E+00_JPRB, 0.26966E+00_JPRB, 0.26723E+00_JPRB, 0.26424E+00_JPRB, 0.26122E+00_JPRB, 0.25758E+00_JPRB, 0.25472E+00_JPRB ,&
&  0.25183E+00_JPRB, 0.24978E+00_JPRB, 0.24758E+00_JPRB, 0.24296E+00_JPRB, 0.24101E+00_JPRB, 0.23738E+00_JPRB, 0.23204E+00_JPRB ,&
&  0.22500E+00_JPRB, 0.22148E+00_JPRB, 0.21758E+00_JPRB, 0.21418E+00_JPRB, 0.21796E+00_JPRB, 0.21738E+00_JPRB, 0.21544E+00_JPRB ,&
&  0.21178E+00_JPRB, 0.20800E+00_JPRB, 0.20297E+00_JPRB, 0.19810E+00_JPRB, 0.19417E+00_JPRB, 0.19095E+00_JPRB, 0.18824E+00_JPRB ,&
&  0.18612E+00_JPRB, 0.18437E+00_JPRB, 0.18169E+00_JPRB, 0.17927E+00_JPRB, 0.17670E+00_JPRB, 0.17446E+00_JPRB, 0.17204E+00_JPRB ,&
&  0.17017E+00_JPRB, 0.16841E+00_JPRB, 0.16659E+00_JPRB, 0.16472E+00_JPRB, 0.16326E+00_JPRB, 0.16133E+00_JPRB, 0.15990E+00_JPRB ,&
&  0.15868E+00_JPRB, 0.15705E+00_JPRB, 0.15584E+00_JPRB, 0.15438E+00_JPRB, 0.15330E+00_JPRB, 0.15240E+00_JPRB, 0.15152E+00_JPRB ,&
&  0.15037E+00_JPRB, 0.14959E+00_JPRB, 0.14932E+00_JPRB, 0.14913E+00_JPRB, 0.14884E+00_JPRB, 0.14894E+00_JPRB, 0.14937E+00_JPRB ,&
&  0.14995E+00_JPRB, 0.15065E+00_JPRB, 0.15173E+00_JPRB, 0.15296E+00_JPRB, 0.15477E+00_JPRB, 0.15721E+00_JPRB, 0.16101E+00_JPRB ,&
&  0.16511E+00_JPRB, 0.16961E+00_JPRB, 0.17407E+00_JPRB, 0.17867E+00_JPRB, 0.18346E+00_JPRB, 0.18821E+00_JPRB, 0.19372E+00_JPRB ,&
&  0.19965E+00_JPRB, 0.20435E+00_JPRB, 0.20890E+00_JPRB, 0.21375E+00_JPRB, 0.22460E+00_JPRB, 0.23257E+00_JPRB, 0.23846E+00_JPRB ,&
&  0.24510E+00_JPRB, 0.25437E+00_JPRB, 0.25888E+00_JPRB, 0.26166E+00_JPRB, 0.26296E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.23468E+00_JPRB, 0.23765E+00_JPRB, 0.23966E+00_JPRB, 0.24094E+00_JPRB, 0.24060E+00_JPRB, 0.24151E+00_JPRB, 0.24243E+00_JPRB ,&
&  0.24367E+00_JPRB, 0.24530E+00_JPRB, 0.24651E+00_JPRB, 0.24757E+00_JPRB, 0.24803E+00_JPRB, 0.24805E+00_JPRB, 0.24807E+00_JPRB ,&
&  0.24783E+00_JPRB, 0.24756E+00_JPRB, 0.24325E+00_JPRB, 0.23938E+00_JPRB, 0.23652E+00_JPRB, 0.23479E+00_JPRB, 0.23333E+00_JPRB ,&
&  0.23217E+00_JPRB, 0.23079E+00_JPRB, 0.22930E+00_JPRB, 0.22719E+00_JPRB, 0.22378E+00_JPRB, 0.22044E+00_JPRB, 0.21698E+00_JPRB ,&
&  0.21386E+00_JPRB, 0.21142E+00_JPRB, 0.20917E+00_JPRB, 0.20694E+00_JPRB, 0.20574E+00_JPRB, 0.20426E+00_JPRB, 0.20270E+00_JPRB ,&
&  0.20061E+00_JPRB, 0.19832E+00_JPRB, 0.19615E+00_JPRB, 0.19392E+00_JPRB, 0.19178E+00_JPRB, 0.18967E+00_JPRB, 0.18812E+00_JPRB ,&
&  0.18652E+00_JPRB, 0.18533E+00_JPRB, 0.18402E+00_JPRB, 0.18283E+00_JPRB, 0.18157E+00_JPRB, 0.18038E+00_JPRB, 0.17891E+00_JPRB ,&
&  0.17790E+00_JPRB, 0.17685E+00_JPRB, 0.17578E+00_JPRB, 0.17480E+00_JPRB, 0.17382E+00_JPRB, 0.17287E+00_JPRB, 0.17179E+00_JPRB ,&
&  0.17079E+00_JPRB, 0.17015E+00_JPRB, 0.16948E+00_JPRB, 0.16873E+00_JPRB, 0.16805E+00_JPRB, 0.16759E+00_JPRB, 0.16719E+00_JPRB ,&
&  0.16684E+00_JPRB, 0.16682E+00_JPRB, 0.16702E+00_JPRB, 0.16721E+00_JPRB, 0.16741E+00_JPRB, 0.16785E+00_JPRB, 0.16851E+00_JPRB ,&
&  0.16927E+00_JPRB, 0.17009E+00_JPRB, 0.17113E+00_JPRB, 0.17261E+00_JPRB, 0.17425E+00_JPRB, 0.17613E+00_JPRB, 0.17816E+00_JPRB ,&
&  0.18037E+00_JPRB, 0.18270E+00_JPRB, 0.18513E+00_JPRB, 0.18739E+00_JPRB, 0.18993E+00_JPRB, 0.19241E+00_JPRB, 0.19483E+00_JPRB ,&
&  0.19737E+00_JPRB, 0.19992E+00_JPRB, 0.20261E+00_JPRB, 0.20510E+00_JPRB, 0.20912E+00_JPRB, 0.21174E+00_JPRB, 0.21443E+00_JPRB ,&
&  0.21682E+00_JPRB, 0.21963E+00_JPRB, 0.22245E+00_JPRB, 0.22591E+00_JPRB, 0.22937E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB, 0.19637E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB, 0.60434E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.64745E+00_JPRB, 0.64524E+00_JPRB, 0.64274E+00_JPRB, 0.64050E+00_JPRB, 0.63777E+00_JPRB, 0.63548E+00_JPRB, 0.63308E+00_JPRB ,&
&  0.63088E+00_JPRB, 0.62847E+00_JPRB, 0.62609E+00_JPRB, 0.62344E+00_JPRB, 0.62089E+00_JPRB, 0.61842E+00_JPRB, 0.61607E+00_JPRB ,&
&  0.61370E+00_JPRB, 0.61148E+00_JPRB, 0.60921E+00_JPRB, 0.60674E+00_JPRB, 0.60459E+00_JPRB, 0.60211E+00_JPRB, 0.59904E+00_JPRB ,&
&  0.59445E+00_JPRB, 0.58964E+00_JPRB, 0.58534E+00_JPRB, 0.58234E+00_JPRB, 0.58005E+00_JPRB, 0.57813E+00_JPRB, 0.57643E+00_JPRB ,&
&  0.57524E+00_JPRB, 0.57457E+00_JPRB, 0.57413E+00_JPRB, 0.57345E+00_JPRB, 0.57329E+00_JPRB, 0.57424E+00_JPRB, 0.57467E+00_JPRB ,&
&  0.57474E+00_JPRB, 0.57486E+00_JPRB, 0.57576E+00_JPRB, 0.57644E+00_JPRB, 0.57708E+00_JPRB, 0.57798E+00_JPRB, 0.57980E+00_JPRB ,&
&  0.58161E+00_JPRB, 0.58325E+00_JPRB, 0.58512E+00_JPRB, 0.58714E+00_JPRB, 0.58920E+00_JPRB, 0.59129E+00_JPRB, 0.59411E+00_JPRB ,&
&  0.59726E+00_JPRB, 0.60002E+00_JPRB, 0.60249E+00_JPRB, 0.60463E+00_JPRB, 0.60647E+00_JPRB, 0.60750E+00_JPRB, 0.60822E+00_JPRB ,&
&  0.60833E+00_JPRB, 0.60931E+00_JPRB, 0.60961E+00_JPRB, 0.61001E+00_JPRB, 0.60983E+00_JPRB, 0.61078E+00_JPRB, 0.61139E+00_JPRB ,&
&  0.61291E+00_JPRB, 0.61393E+00_JPRB, 0.61570E+00_JPRB, 0.61744E+00_JPRB, 0.62009E+00_JPRB, 0.62308E+00_JPRB, 0.62634E+00_JPRB ,&
&  0.62990E+00_JPRB, 0.63391E+00_JPRB, 0.63844E+00_JPRB, 0.64123E+00_JPRB, 0.64401E+00_JPRB, 0.64677E+00_JPRB, 0.64972E+00_JPRB ,&
&  0.65196E+00_JPRB, 0.65408E+00_JPRB, 0.65629E+00_JPRB, 0.65808E+00_JPRB, 0.65916E+00_JPRB, 0.66002E+00_JPRB, 0.66085E+00_JPRB ,&
&  0.66129E+00_JPRB, 0.66165E+00_JPRB, 0.66181E+00_JPRB, 0.66203E+00_JPRB, 0.66129E+00_JPRB, 0.66003E+00_JPRB, 0.65850E+00_JPRB ,&
&  0.65723E+00_JPRB, 0.65538E+00_JPRB, 0.65359E+00_JPRB, 0.65167E+00_JPRB, 0.64981E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.72777E+00_JPRB, 0.72409E+00_JPRB, 0.72021E+00_JPRB, 0.71672E+00_JPRB, 0.71153E+00_JPRB, 0.70630E+00_JPRB, 0.70086E+00_JPRB ,&
&  0.69505E+00_JPRB, 0.68755E+00_JPRB, 0.67931E+00_JPRB, 0.67050E+00_JPRB, 0.66161E+00_JPRB, 0.65375E+00_JPRB, 0.64727E+00_JPRB ,&
&  0.64114E+00_JPRB, 0.63499E+00_JPRB, 0.62968E+00_JPRB, 0.62292E+00_JPRB, 0.61613E+00_JPRB, 0.61008E+00_JPRB, 0.60582E+00_JPRB ,&
&  0.59932E+00_JPRB, 0.59192E+00_JPRB, 0.58411E+00_JPRB, 0.57559E+00_JPRB, 0.56769E+00_JPRB, 0.55985E+00_JPRB, 0.55211E+00_JPRB ,&
&  0.54483E+00_JPRB, 0.53932E+00_JPRB, 0.53446E+00_JPRB, 0.52953E+00_JPRB, 0.52469E+00_JPRB, 0.52327E+00_JPRB, 0.52173E+00_JPRB ,&
&  0.52035E+00_JPRB, 0.52000E+00_JPRB, 0.52333E+00_JPRB, 0.52630E+00_JPRB, 0.52942E+00_JPRB, 0.53264E+00_JPRB, 0.53816E+00_JPRB ,&
&  0.54331E+00_JPRB, 0.54938E+00_JPRB, 0.55712E+00_JPRB, 0.56480E+00_JPRB, 0.57102E+00_JPRB, 0.57557E+00_JPRB, 0.58168E+00_JPRB ,&
&  0.58592E+00_JPRB, 0.59162E+00_JPRB, 0.59559E+00_JPRB, 0.59747E+00_JPRB, 0.59712E+00_JPRB, 0.59696E+00_JPRB, 0.59787E+00_JPRB ,&
&  0.59813E+00_JPRB, 0.60139E+00_JPRB, 0.60468E+00_JPRB, 0.60816E+00_JPRB, 0.60987E+00_JPRB, 0.61447E+00_JPRB, 0.61808E+00_JPRB ,&
&  0.62427E+00_JPRB, 0.63304E+00_JPRB, 0.64197E+00_JPRB, 0.65165E+00_JPRB, 0.66346E+00_JPRB, 0.67697E+00_JPRB, 0.68793E+00_JPRB ,&
&  0.69832E+00_JPRB, 0.70724E+00_JPRB, 0.71461E+00_JPRB, 0.71888E+00_JPRB, 0.72288E+00_JPRB, 0.72677E+00_JPRB, 0.73092E+00_JPRB ,&
&  0.73284E+00_JPRB, 0.73518E+00_JPRB, 0.73770E+00_JPRB, 0.73970E+00_JPRB, 0.73951E+00_JPRB, 0.73919E+00_JPRB, 0.73963E+00_JPRB ,&
&  0.74013E+00_JPRB, 0.74032E+00_JPRB, 0.74075E+00_JPRB, 0.74186E+00_JPRB, 0.74225E+00_JPRB, 0.74182E+00_JPRB, 0.74124E+00_JPRB ,&
&  0.74076E+00_JPRB, 0.73895E+00_JPRB, 0.73635E+00_JPRB, 0.73370E+00_JPRB, 0.73140E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.76532E+00_JPRB, 0.76408E+00_JPRB, 0.76272E+00_JPRB, 0.76122E+00_JPRB, 0.75787E+00_JPRB, 0.75413E+00_JPRB, 0.74922E+00_JPRB ,&
&  0.74347E+00_JPRB, 0.73498E+00_JPRB, 0.72592E+00_JPRB, 0.71589E+00_JPRB, 0.70617E+00_JPRB, 0.69735E+00_JPRB, 0.68880E+00_JPRB ,&
&  0.68009E+00_JPRB, 0.66929E+00_JPRB, 0.65725E+00_JPRB, 0.64127E+00_JPRB, 0.62672E+00_JPRB, 0.61477E+00_JPRB, 0.60175E+00_JPRB ,&
&  0.58366E+00_JPRB, 0.56404E+00_JPRB, 0.54669E+00_JPRB, 0.52949E+00_JPRB, 0.51716E+00_JPRB, 0.48512E+00_JPRB, 0.47504E+00_JPRB ,&
&  0.46847E+00_JPRB, 0.46043E+00_JPRB, 0.45626E+00_JPRB, 0.45254E+00_JPRB, 0.45096E+00_JPRB, 0.44496E+00_JPRB, 0.44179E+00_JPRB ,&
&  0.42735E+00_JPRB, 0.43082E+00_JPRB, 0.43495E+00_JPRB, 0.43975E+00_JPRB, 0.44924E+00_JPRB, 0.45934E+00_JPRB, 0.47555E+00_JPRB ,&
&  0.50513E+00_JPRB, 0.51776E+00_JPRB, 0.53700E+00_JPRB, 0.55202E+00_JPRB, 0.56143E+00_JPRB, 0.56666E+00_JPRB, 0.57310E+00_JPRB ,&
&  0.57979E+00_JPRB, 0.58994E+00_JPRB, 0.59607E+00_JPRB, 0.60014E+00_JPRB, 0.60011E+00_JPRB, 0.59874E+00_JPRB, 0.59847E+00_JPRB ,&
&  0.59751E+00_JPRB, 0.60056E+00_JPRB, 0.60617E+00_JPRB, 0.61757E+00_JPRB, 0.63372E+00_JPRB, 0.65440E+00_JPRB, 0.67558E+00_JPRB ,&
&  0.69625E+00_JPRB, 0.71810E+00_JPRB, 0.73609E+00_JPRB, 0.75317E+00_JPRB, 0.77155E+00_JPRB, 0.78636E+00_JPRB, 0.79900E+00_JPRB ,&
&  0.80747E+00_JPRB, 0.81090E+00_JPRB, 0.80851E+00_JPRB, 0.80520E+00_JPRB, 0.80010E+00_JPRB, 0.79623E+00_JPRB, 0.79573E+00_JPRB ,&
&  0.79572E+00_JPRB, 0.79699E+00_JPRB, 0.79893E+00_JPRB, 0.79782E+00_JPRB, 0.79629E+00_JPRB, 0.79436E+00_JPRB, 0.79287E+00_JPRB ,&
&  0.79074E+00_JPRB, 0.78903E+00_JPRB, 0.78747E+00_JPRB, 0.78576E+00_JPRB, 0.78400E+00_JPRB, 0.78062E+00_JPRB, 0.77761E+00_JPRB ,&
&  0.77439E+00_JPRB, 0.77306E+00_JPRB, 0.77053E+00_JPRB, 0.76850E+00_JPRB, 0.76653E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.72382E+00_JPRB, 0.72369E+00_JPRB, 0.72497E+00_JPRB, 0.72541E+00_JPRB, 0.72734E+00_JPRB, 0.72845E+00_JPRB, 0.73018E+00_JPRB ,&
&  0.73101E+00_JPRB, 0.73171E+00_JPRB, 0.73071E+00_JPRB, 0.72852E+00_JPRB, 0.72464E+00_JPRB, 0.72031E+00_JPRB, 0.71517E+00_JPRB ,&
&  0.70829E+00_JPRB, 0.69561E+00_JPRB, 0.67818E+00_JPRB, 0.65840E+00_JPRB, 0.63558E+00_JPRB, 0.61090E+00_JPRB, 0.58375E+00_JPRB ,&
&  0.55710E+00_JPRB, 0.53306E+00_JPRB, 0.50294E+00_JPRB, 0.49076E+00_JPRB, 0.47164E+00_JPRB, 0.45670E+00_JPRB, 0.44545E+00_JPRB ,&
&  0.43137E+00_JPRB, 0.42144E+00_JPRB, 0.41096E+00_JPRB, 0.40092E+00_JPRB, 0.39573E+00_JPRB, 0.38625E+00_JPRB, 0.38225E+00_JPRB ,&
&  0.37951E+00_JPRB, 0.37821E+00_JPRB, 0.37873E+00_JPRB, 0.38133E+00_JPRB, 0.38742E+00_JPRB, 0.39514E+00_JPRB, 0.40945E+00_JPRB ,&
&  0.44051E+00_JPRB, 0.48382E+00_JPRB, 0.54965E+00_JPRB, 0.56258E+00_JPRB, 0.56287E+00_JPRB, 0.57302E+00_JPRB, 0.58445E+00_JPRB ,&
&  0.59573E+00_JPRB, 0.60622E+00_JPRB, 0.61498E+00_JPRB, 0.61895E+00_JPRB, 0.62396E+00_JPRB, 0.62839E+00_JPRB, 0.63334E+00_JPRB ,&
&  0.63855E+00_JPRB, 0.65042E+00_JPRB, 0.66738E+00_JPRB, 0.68767E+00_JPRB, 0.70964E+00_JPRB, 0.72930E+00_JPRB, 0.74645E+00_JPRB ,&
&  0.76045E+00_JPRB, 0.77496E+00_JPRB, 0.79504E+00_JPRB, 0.81615E+00_JPRB, 0.83822E+00_JPRB, 0.85064E+00_JPRB, 0.86118E+00_JPRB ,&
&  0.86757E+00_JPRB, 0.86800E+00_JPRB, 0.86110E+00_JPRB, 0.85224E+00_JPRB, 0.84575E+00_JPRB, 0.85037E+00_JPRB, 0.84914E+00_JPRB ,&
&  0.84821E+00_JPRB, 0.84175E+00_JPRB, 0.83134E+00_JPRB, 0.81985E+00_JPRB, 0.81251E+00_JPRB, 0.80618E+00_JPRB, 0.80243E+00_JPRB ,&
&  0.79791E+00_JPRB, 0.79235E+00_JPRB, 0.78513E+00_JPRB, 0.77559E+00_JPRB, 0.76559E+00_JPRB, 0.75457E+00_JPRB, 0.74579E+00_JPRB ,&
&  0.73777E+00_JPRB, 0.73293E+00_JPRB, 0.72862E+00_JPRB, 0.72629E+00_JPRB, 0.72402E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.69689E+00_JPRB, 0.70291E+00_JPRB, 0.70937E+00_JPRB, 0.71365E+00_JPRB, 0.71698E+00_JPRB, 0.71917E+00_JPRB, 0.71993E+00_JPRB ,&
&  0.71998E+00_JPRB, 0.72008E+00_JPRB, 0.72037E+00_JPRB, 0.72039E+00_JPRB, 0.72052E+00_JPRB, 0.72087E+00_JPRB, 0.72040E+00_JPRB ,&
&  0.71902E+00_JPRB, 0.70941E+00_JPRB, 0.69156E+00_JPRB, 0.67153E+00_JPRB, 0.64957E+00_JPRB, 0.62112E+00_JPRB, 0.58695E+00_JPRB ,&
&  0.55694E+00_JPRB, 0.53518E+00_JPRB, 0.51234E+00_JPRB, 0.49753E+00_JPRB, 0.48118E+00_JPRB, 0.47136E+00_JPRB, 0.45737E+00_JPRB ,&
&  0.45088E+00_JPRB, 0.42042E+00_JPRB, 0.41747E+00_JPRB, 0.39811E+00_JPRB, 0.38400E+00_JPRB, 0.37230E+00_JPRB, 0.36356E+00_JPRB ,&
&  0.36152E+00_JPRB, 0.35806E+00_JPRB, 0.35800E+00_JPRB, 0.36221E+00_JPRB, 0.36742E+00_JPRB, 0.37870E+00_JPRB, 0.39138E+00_JPRB ,&
&  0.41842E+00_JPRB, 0.50799E+00_JPRB, 0.54177E+00_JPRB, 0.55253E+00_JPRB, 0.55934E+00_JPRB, 0.57557E+00_JPRB, 0.59105E+00_JPRB ,&
&  0.61269E+00_JPRB, 0.62719E+00_JPRB, 0.64170E+00_JPRB, 0.65069E+00_JPRB, 0.67197E+00_JPRB, 0.69050E+00_JPRB, 0.70632E+00_JPRB ,&
&  0.72270E+00_JPRB, 0.73821E+00_JPRB, 0.75560E+00_JPRB, 0.76699E+00_JPRB, 0.76838E+00_JPRB, 0.76866E+00_JPRB, 0.76838E+00_JPRB ,&
&  0.77396E+00_JPRB, 0.78099E+00_JPRB, 0.80568E+00_JPRB, 0.83827E+00_JPRB, 0.87450E+00_JPRB, 0.89900E+00_JPRB, 0.91209E+00_JPRB ,&
&  0.92162E+00_JPRB, 0.92718E+00_JPRB, 0.93560E+00_JPRB, 0.94784E+00_JPRB, 0.93984E+00_JPRB, 0.92270E+00_JPRB, 0.91330E+00_JPRB ,&
&  0.90247E+00_JPRB, 0.87908E+00_JPRB, 0.86147E+00_JPRB, 0.84869E+00_JPRB, 0.83850E+00_JPRB, 0.83005E+00_JPRB, 0.81938E+00_JPRB ,&
&  0.80443E+00_JPRB, 0.78780E+00_JPRB, 0.76921E+00_JPRB, 0.74709E+00_JPRB, 0.72820E+00_JPRB, 0.70877E+00_JPRB, 0.69719E+00_JPRB ,&
&  0.68787E+00_JPRB, 0.68564E+00_JPRB, 0.68357E+00_JPRB, 0.68712E+00_JPRB, 0.68976E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.67787E+00_JPRB, 0.69148E+00_JPRB, 0.70122E+00_JPRB, 0.70562E+00_JPRB, 0.70865E+00_JPRB, 0.71216E+00_JPRB, 0.71304E+00_JPRB ,&
&  0.71392E+00_JPRB, 0.71333E+00_JPRB, 0.71392E+00_JPRB, 0.71386E+00_JPRB, 0.71515E+00_JPRB, 0.71495E+00_JPRB, 0.71379E+00_JPRB ,&
&  0.71164E+00_JPRB, 0.70778E+00_JPRB, 0.69782E+00_JPRB, 0.68079E+00_JPRB, 0.65937E+00_JPRB, 0.63537E+00_JPRB, 0.60112E+00_JPRB ,&
&  0.56613E+00_JPRB, 0.54212E+00_JPRB, 0.52143E+00_JPRB, 0.50597E+00_JPRB, 0.49696E+00_JPRB, 0.48431E+00_JPRB, 0.48149E+00_JPRB ,&
&  0.47890E+00_JPRB, 0.46741E+00_JPRB, 0.45214E+00_JPRB, 0.43594E+00_JPRB, 0.41896E+00_JPRB, 0.37906E+00_JPRB, 0.37536E+00_JPRB ,&
&  0.36584E+00_JPRB, 0.35644E+00_JPRB, 0.35416E+00_JPRB, 0.35564E+00_JPRB, 0.35970E+00_JPRB, 0.37261E+00_JPRB, 0.38870E+00_JPRB ,&
&  0.41004E+00_JPRB, 0.51500E+00_JPRB, 0.53966E+00_JPRB, 0.55291E+00_JPRB, 0.57463E+00_JPRB, 0.59732E+00_JPRB, 0.61548E+00_JPRB ,&
&  0.63359E+00_JPRB, 0.65021E+00_JPRB, 0.67326E+00_JPRB, 0.69552E+00_JPRB, 0.71935E+00_JPRB, 0.74029E+00_JPRB, 0.75416E+00_JPRB ,&
&  0.76679E+00_JPRB, 0.77021E+00_JPRB, 0.79753E+00_JPRB, 0.80714E+00_JPRB, 0.80073E+00_JPRB, 0.82199E+00_JPRB, 0.83461E+00_JPRB ,&
&  0.83283E+00_JPRB, 0.82422E+00_JPRB, 0.84237E+00_JPRB, 0.88602E+00_JPRB, 0.94845E+00_JPRB, 0.99687E+00_JPRB, 0.10114E+01_JPRB ,&
&  0.10171E+01_JPRB, 0.10113E+01_JPRB, 0.10031E+01_JPRB, 0.10016E+01_JPRB, 0.97634E+00_JPRB, 0.94139E+00_JPRB, 0.91723E+00_JPRB ,&
&  0.89715E+00_JPRB, 0.89671E+00_JPRB, 0.88922E+00_JPRB, 0.88343E+00_JPRB, 0.87043E+00_JPRB, 0.85661E+00_JPRB, 0.83323E+00_JPRB ,&
&  0.79788E+00_JPRB, 0.75855E+00_JPRB, 0.72737E+00_JPRB, 0.70115E+00_JPRB, 0.68574E+00_JPRB, 0.66934E+00_JPRB, 0.66113E+00_JPRB ,&
&  0.65349E+00_JPRB, 0.65225E+00_JPRB, 0.65139E+00_JPRB, 0.65674E+00_JPRB, 0.66459E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.67431E+00_JPRB, 0.69199E+00_JPRB, 0.70833E+00_JPRB, 0.71855E+00_JPRB, 0.72046E+00_JPRB, 0.72111E+00_JPRB, 0.71944E+00_JPRB ,&
&  0.71897E+00_JPRB, 0.71893E+00_JPRB, 0.71804E+00_JPRB, 0.71445E+00_JPRB, 0.71062E+00_JPRB, 0.70543E+00_JPRB, 0.70232E+00_JPRB ,&
&  0.69817E+00_JPRB, 0.68704E+00_JPRB, 0.67929E+00_JPRB, 0.67111E+00_JPRB, 0.65691E+00_JPRB, 0.63817E+00_JPRB, 0.61039E+00_JPRB ,&
&  0.57698E+00_JPRB, 0.55012E+00_JPRB, 0.53127E+00_JPRB, 0.51466E+00_JPRB, 0.50251E+00_JPRB, 0.49557E+00_JPRB, 0.49141E+00_JPRB ,&
&  0.48929E+00_JPRB, 0.48282E+00_JPRB, 0.47898E+00_JPRB, 0.45764E+00_JPRB, 0.42976E+00_JPRB, 0.42105E+00_JPRB, 0.40011E+00_JPRB ,&
&  0.37849E+00_JPRB, 0.36124E+00_JPRB, 0.35735E+00_JPRB, 0.35786E+00_JPRB, 0.36451E+00_JPRB, 0.37865E+00_JPRB, 0.40977E+00_JPRB ,&
&  0.45980E+00_JPRB, 0.54490E+00_JPRB, 0.56497E+00_JPRB, 0.58758E+00_JPRB, 0.61028E+00_JPRB, 0.63199E+00_JPRB, 0.64682E+00_JPRB ,&
&  0.66039E+00_JPRB, 0.67442E+00_JPRB, 0.70039E+00_JPRB, 0.72753E+00_JPRB, 0.76908E+00_JPRB, 0.80720E+00_JPRB, 0.83913E+00_JPRB ,&
&  0.86805E+00_JPRB, 0.83924E+00_JPRB, 0.84651E+00_JPRB, 0.85440E+00_JPRB, 0.88675E+00_JPRB, 0.91634E+00_JPRB, 0.91028E+00_JPRB ,&
&  0.90953E+00_JPRB, 0.94421E+00_JPRB, 0.94908E+00_JPRB, 0.97348E+00_JPRB, 0.10282E+01_JPRB, 0.10883E+01_JPRB, 0.11114E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.10889E+01_JPRB, 0.11368E+01_JPRB, 0.10896E+01_JPRB, 0.10602E+01_JPRB, 0.10135E+01_JPRB, 0.95697E+00_JPRB ,&
&  0.92670E+00_JPRB, 0.91227E+00_JPRB, 0.90981E+00_JPRB, 0.90766E+00_JPRB, 0.89408E+00_JPRB, 0.87034E+00_JPRB, 0.82578E+00_JPRB ,&
&  0.76606E+00_JPRB, 0.72255E+00_JPRB, 0.69274E+00_JPRB, 0.67470E+00_JPRB, 0.66171E+00_JPRB, 0.64803E+00_JPRB, 0.63592E+00_JPRB ,&
&  0.62542E+00_JPRB, 0.62338E+00_JPRB, 0.63033E+00_JPRB, 0.64617E+00_JPRB, 0.66175E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.69510E+00_JPRB, 0.71613E+00_JPRB, 0.72871E+00_JPRB, 0.72617E+00_JPRB, 0.71196E+00_JPRB, 0.69170E+00_JPRB, 0.67664E+00_JPRB ,&
&  0.67139E+00_JPRB, 0.67239E+00_JPRB, 0.67530E+00_JPRB, 0.67298E+00_JPRB, 0.67197E+00_JPRB, 0.67201E+00_JPRB, 0.67400E+00_JPRB ,&
&  0.67509E+00_JPRB, 0.67557E+00_JPRB, 0.67483E+00_JPRB, 0.66654E+00_JPRB, 0.65388E+00_JPRB, 0.63644E+00_JPRB, 0.61443E+00_JPRB ,&
&  0.58826E+00_JPRB, 0.56564E+00_JPRB, 0.54671E+00_JPRB, 0.53185E+00_JPRB, 0.51787E+00_JPRB, 0.51089E+00_JPRB, 0.50750E+00_JPRB ,&
&  0.50527E+00_JPRB, 0.50494E+00_JPRB, 0.49751E+00_JPRB, 0.48710E+00_JPRB, 0.46494E+00_JPRB, 0.43857E+00_JPRB, 0.41358E+00_JPRB ,&
&  0.39302E+00_JPRB, 0.38069E+00_JPRB, 0.37416E+00_JPRB, 0.37324E+00_JPRB, 0.38369E+00_JPRB, 0.40596E+00_JPRB, 0.45315E+00_JPRB ,&
&  0.54048E+00_JPRB, 0.57255E+00_JPRB, 0.58983E+00_JPRB, 0.60839E+00_JPRB, 0.62562E+00_JPRB, 0.63987E+00_JPRB, 0.65380E+00_JPRB ,&
&  0.66747E+00_JPRB, 0.68098E+00_JPRB, 0.70160E+00_JPRB, 0.71871E+00_JPRB, 0.74743E+00_JPRB, 0.76782E+00_JPRB, 0.84567E+00_JPRB ,&
&  0.99834E+00_JPRB, 0.10070E+01_JPRB, 0.93676E+00_JPRB, 0.94803E+00_JPRB, 0.98728E+00_JPRB, 0.10253E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.10980E+01_JPRB, 0.11011E+01_JPRB, 0.11042E+01_JPRB, 0.11254E+01_JPRB, 0.11482E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.11658E+01_JPRB, 0.12208E+01_JPRB, 0.10470E+01_JPRB, 0.96956E+00_JPRB, 0.94919E+00_JPRB, 0.91779E+00_JPRB ,&
&  0.89819E+00_JPRB, 0.89711E+00_JPRB, 0.91471E+00_JPRB, 0.92566E+00_JPRB, 0.88275E+00_JPRB, 0.82023E+00_JPRB, 0.75072E+00_JPRB ,&
&  0.70954E+00_JPRB, 0.68674E+00_JPRB, 0.66735E+00_JPRB, 0.65797E+00_JPRB, 0.65058E+00_JPRB, 0.63835E+00_JPRB, 0.61702E+00_JPRB ,&
&  0.61363E+00_JPRB, 0.62429E+00_JPRB, 0.63895E+00_JPRB, 0.66478E+00_JPRB, 0.68195E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.68978E+00_JPRB, 0.72264E+00_JPRB, 0.72743E+00_JPRB, 0.70715E+00_JPRB, 0.68495E+00_JPRB, 0.64675E+00_JPRB, 0.61278E+00_JPRB ,&
&  0.59780E+00_JPRB, 0.59117E+00_JPRB, 0.59079E+00_JPRB, 0.59192E+00_JPRB, 0.59636E+00_JPRB, 0.60453E+00_JPRB, 0.61299E+00_JPRB ,&
&  0.61759E+00_JPRB, 0.62106E+00_JPRB, 0.62508E+00_JPRB, 0.63014E+00_JPRB, 0.62900E+00_JPRB, 0.61717E+00_JPRB, 0.60205E+00_JPRB ,&
&  0.57981E+00_JPRB, 0.56235E+00_JPRB, 0.54906E+00_JPRB, 0.53885E+00_JPRB, 0.53277E+00_JPRB, 0.53162E+00_JPRB, 0.53337E+00_JPRB ,&
&  0.53142E+00_JPRB, 0.52992E+00_JPRB, 0.52865E+00_JPRB, 0.51988E+00_JPRB, 0.49920E+00_JPRB, 0.46933E+00_JPRB, 0.43617E+00_JPRB ,&
&  0.41385E+00_JPRB, 0.40577E+00_JPRB, 0.40935E+00_JPRB, 0.42246E+00_JPRB, 0.45977E+00_JPRB, 0.52653E+00_JPRB, 0.55711E+00_JPRB ,&
&  0.58295E+00_JPRB, 0.59874E+00_JPRB, 0.61323E+00_JPRB, 0.63099E+00_JPRB, 0.64563E+00_JPRB, 0.65854E+00_JPRB, 0.67189E+00_JPRB ,&
&  0.69365E+00_JPRB, 0.71698E+00_JPRB, 0.74770E+00_JPRB, 0.70278E+00_JPRB, 0.71850E+00_JPRB, 0.82878E+00_JPRB, 0.90800E+00_JPRB ,&
&  0.10016E+01_JPRB, 0.10274E+01_JPRB, 0.10399E+01_JPRB, 0.11200E+01_JPRB, 0.11576E+01_JPRB, 0.12091E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.12054E+01_JPRB, 0.11797E+01_JPRB, 0.11541E+01_JPRB, 0.11390E+01_JPRB, 0.11561E+01_JPRB, 0.11920E+01_JPRB, 0.11768E+01_JPRB ,&
&  0.11576E+01_JPRB, 0.11402E+01_JPRB, 0.10540E+01_JPRB, 0.93691E+00_JPRB, 0.89451E+00_JPRB, 0.88738E+00_JPRB, 0.89659E+00_JPRB ,&
&  0.89261E+00_JPRB, 0.89928E+00_JPRB, 0.91204E+00_JPRB, 0.91873E+00_JPRB, 0.84547E+00_JPRB, 0.76492E+00_JPRB, 0.69225E+00_JPRB ,&
&  0.65461E+00_JPRB, 0.64528E+00_JPRB, 0.63663E+00_JPRB, 0.64186E+00_JPRB, 0.63713E+00_JPRB, 0.62513E+00_JPRB, 0.61234E+00_JPRB ,&
&  0.61152E+00_JPRB, 0.61679E+00_JPRB, 0.63735E+00_JPRB, 0.66040E+00_JPRB, 0.67412E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.73193E+00_JPRB, 0.74117E+00_JPRB, 0.71599E+00_JPRB, 0.69754E+00_JPRB, 0.67672E+00_JPRB, 0.65294E+00_JPRB, 0.62659E+00_JPRB ,&
&  0.61877E+00_JPRB, 0.59302E+00_JPRB, 0.57236E+00_JPRB, 0.55523E+00_JPRB, 0.54667E+00_JPRB, 0.54396E+00_JPRB, 0.55281E+00_JPRB ,&
&  0.57094E+00_JPRB, 0.59322E+00_JPRB, 0.60688E+00_JPRB, 0.60756E+00_JPRB, 0.60327E+00_JPRB, 0.58998E+00_JPRB, 0.58045E+00_JPRB ,&
&  0.56397E+00_JPRB, 0.55455E+00_JPRB, 0.55501E+00_JPRB, 0.56071E+00_JPRB, 0.56709E+00_JPRB, 0.56638E+00_JPRB, 0.55828E+00_JPRB ,&
&  0.54750E+00_JPRB, 0.54189E+00_JPRB, 0.54387E+00_JPRB, 0.54420E+00_JPRB, 0.53120E+00_JPRB, 0.49517E+00_JPRB, 0.46016E+00_JPRB ,&
&  0.43756E+00_JPRB, 0.43579E+00_JPRB, 0.46630E+00_JPRB, 0.53381E+00_JPRB, 0.56506E+00_JPRB, 0.58543E+00_JPRB, 0.59754E+00_JPRB ,&
&  0.59257E+00_JPRB, 0.59800E+00_JPRB, 0.62726E+00_JPRB, 0.68112E+00_JPRB, 0.69219E+00_JPRB, 0.70606E+00_JPRB, 0.72215E+00_JPRB ,&
&  0.75211E+00_JPRB, 0.75489E+00_JPRB, 0.77668E+00_JPRB, 0.76725E+00_JPRB, 0.89230E+00_JPRB, 0.11596E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.11690E+01_JPRB, 0.11953E+01_JPRB, 0.12816E+01_JPRB, 0.13151E+01_JPRB, 0.13149E+01_JPRB, 0.13209E+01_JPRB, 0.13035E+01_JPRB ,&
&  0.12468E+01_JPRB, 0.12151E+01_JPRB, 0.11770E+01_JPRB, 0.11804E+01_JPRB, 0.11815E+01_JPRB, 0.11935E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.11795E+01_JPRB, 0.11567E+01_JPRB, 0.10820E+01_JPRB, 0.98713E+00_JPRB, 0.93779E+00_JPRB, 0.89577E+00_JPRB, 0.86882E+00_JPRB ,&
&  0.85315E+00_JPRB, 0.86895E+00_JPRB, 0.89913E+00_JPRB, 0.92111E+00_JPRB, 0.86527E+00_JPRB, 0.76779E+00_JPRB, 0.67840E+00_JPRB ,&
&  0.63077E+00_JPRB, 0.61545E+00_JPRB, 0.59752E+00_JPRB, 0.59370E+00_JPRB, 0.59974E+00_JPRB, 0.60410E+00_JPRB, 0.60834E+00_JPRB ,&
&  0.61437E+00_JPRB, 0.62614E+00_JPRB, 0.66001E+00_JPRB, 0.69574E+00_JPRB, 0.71624E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.73090E+00_JPRB, 0.74334E+00_JPRB, 0.73016E+00_JPRB, 0.72379E+00_JPRB, 0.71408E+00_JPRB, 0.68721E+00_JPRB, 0.64647E+00_JPRB ,&
&  0.63994E+00_JPRB, 0.58853E+00_JPRB, 0.56780E+00_JPRB, 0.55784E+00_JPRB, 0.55681E+00_JPRB, 0.55062E+00_JPRB, 0.54076E+00_JPRB ,&
&  0.54559E+00_JPRB, 0.56410E+00_JPRB, 0.59251E+00_JPRB, 0.60526E+00_JPRB, 0.60616E+00_JPRB, 0.59459E+00_JPRB, 0.57539E+00_JPRB ,&
&  0.55607E+00_JPRB, 0.55105E+00_JPRB, 0.56240E+00_JPRB, 0.58152E+00_JPRB, 0.59251E+00_JPRB, 0.59017E+00_JPRB, 0.58239E+00_JPRB ,&
&  0.57205E+00_JPRB, 0.56245E+00_JPRB, 0.55854E+00_JPRB, 0.55762E+00_JPRB, 0.55315E+00_JPRB, 0.54103E+00_JPRB, 0.50472E+00_JPRB ,&
&  0.48157E+00_JPRB, 0.48999E+00_JPRB, 0.55465E+00_JPRB, 0.59534E+00_JPRB, 0.61733E+00_JPRB, 0.62635E+00_JPRB, 0.63668E+00_JPRB ,&
&  0.65265E+00_JPRB, 0.67352E+00_JPRB, 0.70019E+00_JPRB, 0.73381E+00_JPRB, 0.75811E+00_JPRB, 0.78245E+00_JPRB, 0.80883E+00_JPRB ,&
&  0.86889E+00_JPRB, 0.76039E+00_JPRB, 0.73774E+00_JPRB, 0.79457E+00_JPRB, 0.10943E+01_JPRB, 0.12229E+01_JPRB, 0.12752E+01_JPRB ,&
&  0.13508E+01_JPRB, 0.13194E+01_JPRB, 0.13580E+01_JPRB, 0.14223E+01_JPRB, 0.14570E+01_JPRB, 0.14601E+01_JPRB, 0.14414E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.13633E+01_JPRB, 0.12967E+01_JPRB, 0.12572E+01_JPRB, 0.12247E+01_JPRB, 0.12365E+01_JPRB, 0.12205E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.11719E+01_JPRB, 0.11016E+01_JPRB, 0.10235E+01_JPRB, 0.96369E+00_JPRB, 0.92593E+00_JPRB, 0.89597E+00_JPRB ,&
&  0.87564E+00_JPRB, 0.88030E+00_JPRB, 0.90066E+00_JPRB, 0.94137E+00_JPRB, 0.92491E+00_JPRB, 0.84287E+00_JPRB, 0.75904E+00_JPRB ,&
&  0.68521E+00_JPRB, 0.64703E+00_JPRB, 0.61071E+00_JPRB, 0.60186E+00_JPRB, 0.61174E+00_JPRB, 0.62660E+00_JPRB, 0.64433E+00_JPRB ,&
&  0.66560E+00_JPRB, 0.68098E+00_JPRB, 0.69613E+00_JPRB, 0.71712E+00_JPRB, 0.72214E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.73493E+00_JPRB, 0.75503E+00_JPRB, 0.75050E+00_JPRB, 0.75270E+00_JPRB, 0.74872E+00_JPRB, 0.71027E+00_JPRB, 0.68227E+00_JPRB ,&
&  0.70807E+00_JPRB, 0.66028E+00_JPRB, 0.68252E+00_JPRB, 0.61179E+00_JPRB, 0.58350E+00_JPRB, 0.57555E+00_JPRB, 0.55977E+00_JPRB ,&
&  0.55849E+00_JPRB, 0.57199E+00_JPRB, 0.58840E+00_JPRB, 0.60204E+00_JPRB, 0.61073E+00_JPRB, 0.59638E+00_JPRB, 0.57934E+00_JPRB ,&
&  0.55859E+00_JPRB, 0.55422E+00_JPRB, 0.56813E+00_JPRB, 0.58393E+00_JPRB, 0.59784E+00_JPRB, 0.60213E+00_JPRB, 0.59770E+00_JPRB ,&
&  0.58388E+00_JPRB, 0.57842E+00_JPRB, 0.57843E+00_JPRB, 0.57439E+00_JPRB, 0.57158E+00_JPRB, 0.56515E+00_JPRB, 0.55665E+00_JPRB ,&
&  0.55022E+00_JPRB, 0.55914E+00_JPRB, 0.60385E+00_JPRB, 0.63548E+00_JPRB, 0.65260E+00_JPRB, 0.65811E+00_JPRB, 0.67093E+00_JPRB ,&
&  0.68413E+00_JPRB, 0.70344E+00_JPRB, 0.73832E+00_JPRB, 0.78638E+00_JPRB, 0.82985E+00_JPRB, 0.87171E+00_JPRB, 0.92163E+00_JPRB ,&
&  0.99232E+00_JPRB, 0.83044E+00_JPRB, 0.94330E+00_JPRB, 0.97447E+00_JPRB, 0.11527E+01_JPRB, 0.13597E+01_JPRB, 0.13685E+01_JPRB ,&
&  0.15473E+01_JPRB, 0.15064E+01_JPRB, 0.15440E+01_JPRB, 0.15616E+01_JPRB, 0.15925E+01_JPRB, 0.16101E+01_JPRB, 0.16073E+01_JPRB ,&
&  0.16412E+01_JPRB, 0.14805E+01_JPRB, 0.14721E+01_JPRB, 0.13944E+01_JPRB, 0.13357E+01_JPRB, 0.13256E+01_JPRB, 0.12732E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.12049E+01_JPRB, 0.11603E+01_JPRB, 0.10376E+01_JPRB, 0.10011E+01_JPRB, 0.97117E+00_JPRB, 0.90265E+00_JPRB ,&
&  0.87839E+00_JPRB, 0.85112E+00_JPRB, 0.83184E+00_JPRB, 0.91681E+00_JPRB, 0.92957E+00_JPRB, 0.85648E+00_JPRB, 0.76309E+00_JPRB ,&
&  0.70303E+00_JPRB, 0.66363E+00_JPRB, 0.68065E+00_JPRB, 0.68289E+00_JPRB, 0.66946E+00_JPRB, 0.67157E+00_JPRB, 0.71356E+00_JPRB ,&
&  0.70734E+00_JPRB, 0.71875E+00_JPRB, 0.74770E+00_JPRB, 0.74883E+00_JPRB, 0.73612E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.75325E+00_JPRB, 0.78044E+00_JPRB, 0.76921E+00_JPRB, 0.77691E+00_JPRB, 0.78376E+00_JPRB, 0.74776E+00_JPRB, 0.70826E+00_JPRB ,&
&  0.71235E+00_JPRB, 0.71086E+00_JPRB, 0.71266E+00_JPRB, 0.71209E+00_JPRB, 0.71456E+00_JPRB, 0.66226E+00_JPRB, 0.57443E+00_JPRB ,&
&  0.54727E+00_JPRB, 0.55918E+00_JPRB, 0.58587E+00_JPRB, 0.60515E+00_JPRB, 0.60664E+00_JPRB, 0.59727E+00_JPRB, 0.58949E+00_JPRB ,&
&  0.58119E+00_JPRB, 0.57634E+00_JPRB, 0.58524E+00_JPRB, 0.59259E+00_JPRB, 0.59775E+00_JPRB, 0.60727E+00_JPRB, 0.61229E+00_JPRB ,&
&  0.60211E+00_JPRB, 0.60240E+00_JPRB, 0.59931E+00_JPRB, 0.58447E+00_JPRB, 0.59527E+00_JPRB, 0.58792E+00_JPRB, 0.58034E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.61501E+00_JPRB, 0.63910E+00_JPRB, 0.66775E+00_JPRB, 0.68749E+00_JPRB, 0.69680E+00_JPRB, 0.70886E+00_JPRB ,&
&  0.73063E+00_JPRB, 0.76432E+00_JPRB, 0.80658E+00_JPRB, 0.85983E+00_JPRB, 0.92504E+00_JPRB, 0.95129E+00_JPRB, 0.10667E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.11233E+01_JPRB, 0.11594E+01_JPRB, 0.10999E+01_JPRB, 0.12719E+01_JPRB, 0.14558E+01_JPRB, 0.14716E+01_JPRB ,&
&  0.15577E+01_JPRB, 0.14945E+01_JPRB, 0.16162E+01_JPRB, 0.17581E+01_JPRB, 0.18693E+01_JPRB, 0.18486E+01_JPRB, 0.18089E+01_JPRB ,&
&  0.18635E+01_JPRB, 0.18352E+01_JPRB, 0.16367E+01_JPRB, 0.15558E+01_JPRB, 0.14726E+01_JPRB, 0.14451E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.13093E+01_JPRB, 0.12892E+01_JPRB, 0.12423E+01_JPRB, 0.11830E+01_JPRB, 0.11118E+01_JPRB, 0.10473E+01_JPRB, 0.98618E+00_JPRB ,&
&  0.91793E+00_JPRB, 0.82621E+00_JPRB, 0.79742E+00_JPRB, 0.85469E+00_JPRB, 0.91569E+00_JPRB, 0.82951E+00_JPRB, 0.72918E+00_JPRB ,&
&  0.69348E+00_JPRB, 0.70184E+00_JPRB, 0.76004E+00_JPRB, 0.76402E+00_JPRB, 0.75058E+00_JPRB, 0.74255E+00_JPRB, 0.71892E+00_JPRB ,&
&  0.69316E+00_JPRB, 0.74353E+00_JPRB, 0.76089E+00_JPRB, 0.75527E+00_JPRB, 0.74449E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.78719E+00_JPRB, 0.82722E+00_JPRB, 0.81151E+00_JPRB, 0.81497E+00_JPRB, 0.82793E+00_JPRB, 0.78084E+00_JPRB, 0.76014E+00_JPRB ,&
&  0.75700E+00_JPRB, 0.75123E+00_JPRB, 0.74048E+00_JPRB, 0.73552E+00_JPRB, 0.73373E+00_JPRB, 0.72192E+00_JPRB, 0.65904E+00_JPRB ,&
&  0.58972E+00_JPRB, 0.59518E+00_JPRB, 0.59153E+00_JPRB, 0.61055E+00_JPRB, 0.63121E+00_JPRB, 0.61811E+00_JPRB, 0.60908E+00_JPRB ,&
&  0.61111E+00_JPRB, 0.60387E+00_JPRB, 0.60084E+00_JPRB, 0.60550E+00_JPRB, 0.61670E+00_JPRB, 0.63578E+00_JPRB, 0.64520E+00_JPRB ,&
&  0.62954E+00_JPRB, 0.62220E+00_JPRB, 0.61349E+00_JPRB, 0.60849E+00_JPRB, 0.61737E+00_JPRB, 0.61844E+00_JPRB, 0.61776E+00_JPRB ,&
&  0.62859E+00_JPRB, 0.64978E+00_JPRB, 0.67167E+00_JPRB, 0.70415E+00_JPRB, 0.73259E+00_JPRB, 0.75391E+00_JPRB, 0.78201E+00_JPRB ,&
&  0.84003E+00_JPRB, 0.90010E+00_JPRB, 0.93631E+00_JPRB, 0.98953E+00_JPRB, 0.10425E+01_JPRB, 0.11439E+01_JPRB, 0.12295E+01_JPRB ,&
&  0.12743E+01_JPRB, 0.12842E+01_JPRB, 0.12746E+01_JPRB, 0.13185E+01_JPRB, 0.14904E+01_JPRB, 0.16044E+01_JPRB, 0.16608E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.16999E+01_JPRB, 0.20341E+01_JPRB, 0.21437E+01_JPRB, 0.20910E+01_JPRB, 0.21010E+01_JPRB, 0.19853E+01_JPRB ,&
&  0.19504E+01_JPRB, 0.18319E+01_JPRB, 0.17449E+01_JPRB, 0.16670E+01_JPRB, 0.15906E+01_JPRB, 0.15899E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.14784E+01_JPRB, 0.14367E+01_JPRB, 0.12579E+01_JPRB, 0.11268E+01_JPRB, 0.99105E+00_JPRB, 0.10379E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.92612E+00_JPRB, 0.80384E+00_JPRB, 0.85260E+00_JPRB, 0.93198E+00_JPRB, 0.10170E+01_JPRB, 0.10183E+01_JPRB, 0.86050E+00_JPRB ,&
&  0.77585E+00_JPRB, 0.79157E+00_JPRB, 0.80470E+00_JPRB, 0.82224E+00_JPRB, 0.82947E+00_JPRB, 0.83217E+00_JPRB, 0.76793E+00_JPRB ,&
&  0.72621E+00_JPRB, 0.76286E+00_JPRB, 0.77738E+00_JPRB, 0.77218E+00_JPRB, 0.77141E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.83382E+00_JPRB, 0.88288E+00_JPRB, 0.88572E+00_JPRB, 0.86898E+00_JPRB, 0.87263E+00_JPRB, 0.82298E+00_JPRB, 0.80101E+00_JPRB ,&
&  0.80325E+00_JPRB, 0.79607E+00_JPRB, 0.77092E+00_JPRB, 0.76299E+00_JPRB, 0.75786E+00_JPRB, 0.74512E+00_JPRB, 0.72367E+00_JPRB ,&
&  0.62868E+00_JPRB, 0.62408E+00_JPRB, 0.59299E+00_JPRB, 0.60604E+00_JPRB, 0.67713E+00_JPRB, 0.66106E+00_JPRB, 0.65265E+00_JPRB ,&
&  0.64467E+00_JPRB, 0.64014E+00_JPRB, 0.62653E+00_JPRB, 0.62577E+00_JPRB, 0.63800E+00_JPRB, 0.66988E+00_JPRB, 0.70174E+00_JPRB ,&
&  0.69126E+00_JPRB, 0.67019E+00_JPRB, 0.65388E+00_JPRB, 0.64628E+00_JPRB, 0.64822E+00_JPRB, 0.65681E+00_JPRB, 0.67361E+00_JPRB ,&
&  0.68880E+00_JPRB, 0.70164E+00_JPRB, 0.72939E+00_JPRB, 0.77202E+00_JPRB, 0.80752E+00_JPRB, 0.83912E+00_JPRB, 0.90596E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.10864E+01_JPRB, 0.11274E+01_JPRB, 0.11727E+01_JPRB, 0.11982E+01_JPRB, 0.12926E+01_JPRB, 0.13660E+01_JPRB ,&
&  0.15107E+01_JPRB, 0.14932E+01_JPRB, 0.15265E+01_JPRB, 0.17434E+01_JPRB, 0.19214E+01_JPRB, 0.19722E+01_JPRB, 0.19546E+01_JPRB ,&
&  0.19934E+01_JPRB, 0.19420E+01_JPRB, 0.20642E+01_JPRB, 0.21990E+01_JPRB, 0.21426E+01_JPRB, 0.22189E+01_JPRB, 0.20719E+01_JPRB ,&
&  0.18548E+01_JPRB, 0.17337E+01_JPRB, 0.16840E+01_JPRB, 0.16640E+01_JPRB, 0.15946E+01_JPRB, 0.16524E+01_JPRB, 0.16105E+01_JPRB ,&
&  0.16191E+01_JPRB, 0.14479E+01_JPRB, 0.11170E+01_JPRB, 0.10369E+01_JPRB, 0.85016E+00_JPRB, 0.87046E+00_JPRB, 0.10522E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.96057E+00_JPRB, 0.99672E+00_JPRB, 0.10982E+01_JPRB, 0.12286E+01_JPRB, 0.14143E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.95182E+00_JPRB, 0.94797E+00_JPRB, 0.95313E+00_JPRB, 0.95114E+00_JPRB, 0.96743E+00_JPRB, 0.93930E+00_JPRB ,&
&  0.89875E+00_JPRB, 0.86070E+00_JPRB, 0.84432E+00_JPRB, 0.82788E+00_JPRB, 0.82678E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.88531E+00_JPRB, 0.92652E+00_JPRB, 0.94404E+00_JPRB, 0.92207E+00_JPRB, 0.91500E+00_JPRB, 0.87786E+00_JPRB, 0.84465E+00_JPRB ,&
&  0.84209E+00_JPRB, 0.83141E+00_JPRB, 0.80243E+00_JPRB, 0.79364E+00_JPRB, 0.78374E+00_JPRB, 0.76271E+00_JPRB, 0.74517E+00_JPRB ,&
&  0.74935E+00_JPRB, 0.71401E+00_JPRB, 0.65126E+00_JPRB, 0.61473E+00_JPRB, 0.66866E+00_JPRB, 0.66623E+00_JPRB, 0.68114E+00_JPRB ,&
&  0.67094E+00_JPRB, 0.68890E+00_JPRB, 0.68431E+00_JPRB, 0.68426E+00_JPRB, 0.71076E+00_JPRB, 0.75362E+00_JPRB, 0.79119E+00_JPRB ,&
&  0.79682E+00_JPRB, 0.79179E+00_JPRB, 0.76849E+00_JPRB, 0.75880E+00_JPRB, 0.77682E+00_JPRB, 0.75830E+00_JPRB, 0.77960E+00_JPRB ,&
&  0.78575E+00_JPRB, 0.78741E+00_JPRB, 0.80416E+00_JPRB, 0.84731E+00_JPRB, 0.89156E+00_JPRB, 0.94286E+00_JPRB, 0.10401E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.11851E+01_JPRB, 0.11869E+01_JPRB, 0.12100E+01_JPRB, 0.12872E+01_JPRB, 0.13944E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.15051E+01_JPRB, 0.14862E+01_JPRB, 0.14631E+01_JPRB, 0.17793E+01_JPRB, 0.20309E+01_JPRB, 0.20525E+01_JPRB, 0.20463E+01_JPRB ,&
&  0.21115E+01_JPRB, 0.21468E+01_JPRB, 0.22831E+01_JPRB, 0.23433E+01_JPRB, 0.23498E+01_JPRB, 0.23057E+01_JPRB, 0.20960E+01_JPRB ,&
&  0.18325E+01_JPRB, 0.16974E+01_JPRB, 0.16983E+01_JPRB, 0.16739E+01_JPRB, 0.16550E+01_JPRB, 0.15743E+01_JPRB, 0.16615E+01_JPRB ,&
&  0.17758E+01_JPRB, 0.10542E+01_JPRB, 0.86811E+00_JPRB, 0.99922E+00_JPRB, 0.87850E+00_JPRB, 0.89660E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.96752E+00_JPRB, 0.95675E+00_JPRB, 0.11167E+01_JPRB, 0.12430E+01_JPRB, 0.14310E+01_JPRB, 0.16958E+01_JPRB, 0.15163E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.12180E+01_JPRB, 0.12200E+01_JPRB, 0.11914E+01_JPRB, 0.12091E+01_JPRB, 0.11912E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.10372E+01_JPRB, 0.97985E+00_JPRB, 0.93375E+00_JPRB, 0.90128E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.98295E+00_JPRB, 0.10038E+01_JPRB, 0.10031E+01_JPRB, 0.98038E+00_JPRB, 0.96101E+00_JPRB, 0.94573E+00_JPRB, 0.90455E+00_JPRB ,&
&  0.88108E+00_JPRB, 0.86140E+00_JPRB, 0.83298E+00_JPRB, 0.82242E+00_JPRB, 0.80041E+00_JPRB, 0.77629E+00_JPRB, 0.76021E+00_JPRB ,&
&  0.76322E+00_JPRB, 0.76033E+00_JPRB, 0.66701E+00_JPRB, 0.63482E+00_JPRB, 0.61053E+00_JPRB, 0.64016E+00_JPRB, 0.66795E+00_JPRB ,&
&  0.70428E+00_JPRB, 0.71912E+00_JPRB, 0.73663E+00_JPRB, 0.77203E+00_JPRB, 0.83371E+00_JPRB, 0.87239E+00_JPRB, 0.93005E+00_JPRB ,&
&  0.94521E+00_JPRB, 0.96666E+00_JPRB, 0.90917E+00_JPRB, 0.91875E+00_JPRB, 0.93670E+00_JPRB, 0.90363E+00_JPRB, 0.93450E+00_JPRB ,&
&  0.92198E+00_JPRB, 0.92720E+00_JPRB, 0.93966E+00_JPRB, 0.95209E+00_JPRB, 0.97887E+00_JPRB, 0.10062E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.11681E+01_JPRB, 0.11509E+01_JPRB, 0.11542E+01_JPRB, 0.12063E+01_JPRB, 0.12829E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.14045E+01_JPRB, 0.12597E+01_JPRB, 0.10233E+01_JPRB, 0.14885E+01_JPRB, 0.20506E+01_JPRB, 0.19955E+01_JPRB, 0.17969E+01_JPRB ,&
&  0.19599E+01_JPRB, 0.21033E+01_JPRB, 0.23201E+01_JPRB, 0.24042E+01_JPRB, 0.24012E+01_JPRB, 0.23063E+01_JPRB, 0.21436E+01_JPRB ,&
&  0.19523E+01_JPRB, 0.19184E+01_JPRB, 0.19945E+01_JPRB, 0.21638E+01_JPRB, 0.21502E+01_JPRB, 0.21048E+01_JPRB, 0.20364E+01_JPRB ,&
&  0.17785E+01_JPRB, 0.15299E+01_JPRB, 0.10973E+01_JPRB, 0.11320E+01_JPRB, 0.91528E+00_JPRB, 0.91679E+00_JPRB, 0.10161E+01_JPRB ,&
&  0.98387E+00_JPRB, 0.10480E+01_JPRB, 0.12294E+01_JPRB, 0.14008E+01_JPRB, 0.18213E+01_JPRB, 0.19772E+01_JPRB, 0.19020E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.14477E+01_JPRB, 0.14862E+01_JPRB, 0.14755E+01_JPRB, 0.14909E+01_JPRB, 0.14409E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12032E+01_JPRB, 0.11485E+01_JPRB, 0.10733E+01_JPRB, 0.10026E+01_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.11040E+01_JPRB, 0.10887E+01_JPRB, 0.10514E+01_JPRB, 0.10033E+01_JPRB, 0.97022E+00_JPRB, 0.95719E+00_JPRB, 0.93750E+00_JPRB ,&
&  0.91959E+00_JPRB, 0.88825E+00_JPRB, 0.86364E+00_JPRB, 0.84101E+00_JPRB, 0.81901E+00_JPRB, 0.79425E+00_JPRB, 0.78251E+00_JPRB ,&
&  0.78347E+00_JPRB, 0.73198E+00_JPRB, 0.63036E+00_JPRB, 0.58892E+00_JPRB, 0.56857E+00_JPRB, 0.59518E+00_JPRB, 0.63851E+00_JPRB ,&
&  0.69065E+00_JPRB, 0.72630E+00_JPRB, 0.79336E+00_JPRB, 0.88598E+00_JPRB, 0.98670E+00_JPRB, 0.10643E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.11754E+01_JPRB, 0.11771E+01_JPRB, 0.11274E+01_JPRB, 0.11273E+01_JPRB, 0.10872E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10146E+01_JPRB, 0.99006E+00_JPRB, 0.97172E+00_JPRB, 0.95531E+00_JPRB, 0.96157E+00_JPRB, 0.10039E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.11689E+01_JPRB, 0.11238E+01_JPRB, 0.11568E+01_JPRB, 0.12368E+01_JPRB, 0.12973E+01_JPRB, 0.13175E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.12572E+01_JPRB, 0.15725E+01_JPRB, 0.17661E+01_JPRB, 0.20249E+01_JPRB, 0.20773E+01_JPRB, 0.19841E+01_JPRB ,&
&  0.21034E+01_JPRB, 0.21652E+01_JPRB, 0.22839E+01_JPRB, 0.19023E+01_JPRB, 0.21602E+01_JPRB, 0.21513E+01_JPRB, 0.20799E+01_JPRB ,&
&  0.20307E+01_JPRB, 0.21301E+01_JPRB, 0.22271E+01_JPRB, 0.24554E+01_JPRB, 0.26286E+01_JPRB, 0.27671E+01_JPRB, 0.21424E+01_JPRB ,&
&  0.17150E+01_JPRB, 0.18316E+01_JPRB, 0.16643E+01_JPRB, 0.13886E+01_JPRB, 0.11728E+01_JPRB, 0.11854E+01_JPRB, 0.11789E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.12783E+01_JPRB, 0.14625E+01_JPRB, 0.18749E+01_JPRB, 0.23506E+01_JPRB, 0.25376E+01_JPRB, 0.23457E+01_JPRB ,&
&  0.19645E+01_JPRB, 0.19906E+01_JPRB, 0.19093E+01_JPRB, 0.18300E+01_JPRB, 0.17982E+01_JPRB, 0.17169E+01_JPRB, 0.15899E+01_JPRB ,&
&  0.14532E+01_JPRB, 0.13529E+01_JPRB, 0.12711E+01_JPRB, 0.11880E+01_JPRB, 0.11260E+01_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.11833E+01_JPRB, 0.11538E+01_JPRB, 0.11038E+01_JPRB, 0.10513E+01_JPRB, 0.10089E+01_JPRB, 0.98820E+00_JPRB, 0.97033E+00_JPRB ,&
&  0.96672E+00_JPRB, 0.90700E+00_JPRB, 0.88199E+00_JPRB, 0.85761E+00_JPRB, 0.83733E+00_JPRB, 0.81999E+00_JPRB, 0.81508E+00_JPRB ,&
&  0.81295E+00_JPRB, 0.71201E+00_JPRB, 0.60651E+00_JPRB, 0.60200E+00_JPRB, 0.60894E+00_JPRB, 0.60088E+00_JPRB, 0.63632E+00_JPRB ,&
&  0.68688E+00_JPRB, 0.75306E+00_JPRB, 0.85524E+00_JPRB, 0.10347E+01_JPRB, 0.11953E+01_JPRB, 0.13275E+01_JPRB, 0.14513E+01_JPRB ,&
&  0.14566E+01_JPRB, 0.14693E+01_JPRB, 0.14389E+01_JPRB, 0.13235E+01_JPRB, 0.12076E+01_JPRB, 0.11319E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10180E+01_JPRB, 0.99941E+00_JPRB, 0.99762E+00_JPRB, 0.10168E+01_JPRB, 0.10774E+01_JPRB, 0.11666E+01_JPRB ,&
&  0.12080E+01_JPRB, 0.11756E+01_JPRB, 0.11542E+01_JPRB, 0.12349E+01_JPRB, 0.11847E+01_JPRB, 0.11633E+01_JPRB, 0.11343E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.16198E+01_JPRB, 0.16668E+01_JPRB, 0.18408E+01_JPRB, 0.16877E+01_JPRB, 0.17999E+01_JPRB, 0.20820E+01_JPRB ,&
&  0.22488E+01_JPRB, 0.22698E+01_JPRB, 0.23417E+01_JPRB, 0.16721E+01_JPRB, 0.15308E+01_JPRB, 0.20662E+01_JPRB, 0.21435E+01_JPRB ,&
&  0.21303E+01_JPRB, 0.23650E+01_JPRB, 0.26467E+01_JPRB, 0.30406E+01_JPRB, 0.23460E+01_JPRB, 0.20745E+01_JPRB, 0.16057E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.14565E+01_JPRB, 0.16570E+01_JPRB, 0.15435E+01_JPRB, 0.15377E+01_JPRB, 0.14992E+01_JPRB, 0.14190E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.15425E+01_JPRB, 0.18976E+01_JPRB, 0.25940E+01_JPRB, 0.32283E+01_JPRB, 0.29957E+01_JPRB, 0.29550E+01_JPRB ,&
&  0.28237E+01_JPRB, 0.26522E+01_JPRB, 0.23583E+01_JPRB, 0.22861E+01_JPRB, 0.21431E+01_JPRB, 0.19450E+01_JPRB, 0.17200E+01_JPRB ,&
&  0.15402E+01_JPRB, 0.14355E+01_JPRB, 0.13403E+01_JPRB, 0.12610E+01_JPRB, 0.12110E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.12605E+01_JPRB, 0.12174E+01_JPRB, 0.11677E+01_JPRB, 0.11137E+01_JPRB, 0.10703E+01_JPRB, 0.10394E+01_JPRB, 0.10175E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.94756E+00_JPRB, 0.89077E+00_JPRB, 0.86468E+00_JPRB, 0.85416E+00_JPRB, 0.84940E+00_JPRB, 0.85069E+00_JPRB ,&
&  0.85516E+00_JPRB, 0.79781E+00_JPRB, 0.65126E+00_JPRB, 0.61156E+00_JPRB, 0.65028E+00_JPRB, 0.63643E+00_JPRB, 0.64184E+00_JPRB ,&
&  0.71612E+00_JPRB, 0.83509E+00_JPRB, 0.10154E+01_JPRB, 0.12863E+01_JPRB, 0.14961E+01_JPRB, 0.16930E+01_JPRB, 0.17907E+01_JPRB ,&
&  0.18107E+01_JPRB, 0.16670E+01_JPRB, 0.14616E+01_JPRB, 0.12770E+01_JPRB, 0.11590E+01_JPRB, 0.10895E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.10273E+01_JPRB, 0.10267E+01_JPRB, 0.10244E+01_JPRB, 0.10658E+01_JPRB, 0.11071E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.12082E+01_JPRB, 0.11899E+01_JPRB, 0.12176E+01_JPRB, 0.12865E+01_JPRB, 0.12895E+01_JPRB, 0.12277E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.15088E+01_JPRB, 0.16586E+01_JPRB, 0.17956E+01_JPRB, 0.19419E+01_JPRB, 0.20127E+01_JPRB, 0.22657E+01_JPRB ,&
&  0.19473E+01_JPRB, 0.16022E+01_JPRB, 0.13651E+01_JPRB, 0.11772E+01_JPRB, 0.15674E+01_JPRB, 0.22447E+01_JPRB, 0.24969E+01_JPRB ,&
&  0.26932E+01_JPRB, 0.29447E+01_JPRB, 0.32111E+01_JPRB, 0.28809E+01_JPRB, 0.22621E+01_JPRB, 0.18211E+01_JPRB, 0.21746E+01_JPRB ,&
&  0.22773E+01_JPRB, 0.23088E+01_JPRB, 0.21654E+01_JPRB, 0.19052E+01_JPRB, 0.15785E+01_JPRB, 0.18110E+01_JPRB, 0.17709E+01_JPRB ,&
&  0.17651E+01_JPRB, 0.18696E+01_JPRB, 0.28805E+01_JPRB, 0.38093E+01_JPRB, 0.39368E+01_JPRB, 0.37743E+01_JPRB, 0.38051E+01_JPRB ,&
&  0.34499E+01_JPRB, 0.32406E+01_JPRB, 0.28899E+01_JPRB, 0.25340E+01_JPRB, 0.22205E+01_JPRB, 0.19584E+01_JPRB, 0.17476E+01_JPRB ,&
&  0.16072E+01_JPRB, 0.15024E+01_JPRB, 0.13945E+01_JPRB, 0.13360E+01_JPRB, 0.12943E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.13604E+01_JPRB, 0.13082E+01_JPRB, 0.12589E+01_JPRB, 0.12075E+01_JPRB, 0.11612E+01_JPRB, 0.11029E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.98740E+00_JPRB, 0.92062E+00_JPRB, 0.89345E+00_JPRB, 0.90205E+00_JPRB, 0.90879E+00_JPRB, 0.90680E+00_JPRB ,&
&  0.90541E+00_JPRB, 0.94293E+00_JPRB, 0.75918E+00_JPRB, 0.67921E+00_JPRB, 0.68564E+00_JPRB, 0.68387E+00_JPRB, 0.65910E+00_JPRB ,&
&  0.79443E+00_JPRB, 0.10130E+01_JPRB, 0.12594E+01_JPRB, 0.15565E+01_JPRB, 0.18842E+01_JPRB, 0.18614E+01_JPRB, 0.18795E+01_JPRB ,&
&  0.18660E+01_JPRB, 0.16891E+01_JPRB, 0.14876E+01_JPRB, 0.13259E+01_JPRB, 0.12128E+01_JPRB, 0.11490E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11232E+01_JPRB, 0.11194E+01_JPRB, 0.11117E+01_JPRB, 0.11217E+01_JPRB, 0.11236E+01_JPRB, 0.11519E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.12008E+01_JPRB, 0.12326E+01_JPRB, 0.13380E+01_JPRB, 0.14399E+01_JPRB, 0.13387E+01_JPRB, 0.15781E+01_JPRB ,&
&  0.14770E+01_JPRB, 0.16419E+01_JPRB, 0.17677E+01_JPRB, 0.20849E+01_JPRB, 0.21658E+01_JPRB, 0.20848E+01_JPRB, 0.21915E+01_JPRB ,&
&  0.18056E+01_JPRB, 0.16312E+01_JPRB, 0.17832E+01_JPRB, 0.19532E+01_JPRB, 0.20065E+01_JPRB, 0.21578E+01_JPRB, 0.32628E+01_JPRB ,&
&  0.27946E+01_JPRB, 0.31326E+01_JPRB, 0.33750E+01_JPRB, 0.33342E+01_JPRB, 0.16896E+01_JPRB, 0.13616E+01_JPRB, 0.20233E+01_JPRB ,&
&  0.22414E+01_JPRB, 0.15671E+01_JPRB, 0.11534E+01_JPRB, 0.13776E+01_JPRB, 0.10723E+01_JPRB, 0.13136E+01_JPRB, 0.18159E+01_JPRB ,&
&  0.20647E+01_JPRB, 0.24131E+01_JPRB, 0.38291E+01_JPRB, 0.44643E+01_JPRB, 0.44905E+01_JPRB, 0.50042E+01_JPRB, 0.42777E+01_JPRB ,&
&  0.39032E+01_JPRB, 0.32384E+01_JPRB, 0.30808E+01_JPRB, 0.26812E+01_JPRB, 0.23347E+01_JPRB, 0.20907E+01_JPRB, 0.18957E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.16133E+01_JPRB, 0.15144E+01_JPRB, 0.14552E+01_JPRB, 0.14099E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.15092E+01_JPRB, 0.14362E+01_JPRB, 0.13682E+01_JPRB, 0.13036E+01_JPRB, 0.12346E+01_JPRB, 0.11794E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.10262E+01_JPRB, 0.97131E+00_JPRB, 0.93063E+00_JPRB, 0.93185E+00_JPRB, 0.94064E+00_JPRB, 0.92877E+00_JPRB ,&
&  0.91019E+00_JPRB, 0.91314E+00_JPRB, 0.87344E+00_JPRB, 0.81608E+00_JPRB, 0.78168E+00_JPRB, 0.77493E+00_JPRB, 0.81587E+00_JPRB ,&
&  0.10495E+01_JPRB, 0.14267E+01_JPRB, 0.16712E+01_JPRB, 0.18737E+01_JPRB, 0.19963E+01_JPRB, 0.19252E+01_JPRB, 0.20294E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.18503E+01_JPRB, 0.16597E+01_JPRB, 0.14736E+01_JPRB, 0.13494E+01_JPRB, 0.12705E+01_JPRB, 0.12237E+01_JPRB ,&
&  0.12081E+01_JPRB, 0.12058E+01_JPRB, 0.11919E+01_JPRB, 0.11633E+01_JPRB, 0.11464E+01_JPRB, 0.11711E+01_JPRB, 0.12284E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.12076E+01_JPRB, 0.12550E+01_JPRB, 0.13340E+01_JPRB, 0.15448E+01_JPRB, 0.15945E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.12390E+01_JPRB, 0.13575E+01_JPRB, 0.18384E+01_JPRB, 0.20333E+01_JPRB, 0.20421E+01_JPRB, 0.20415E+01_JPRB, 0.19552E+01_JPRB ,&
&  0.19390E+01_JPRB, 0.22101E+01_JPRB, 0.23213E+01_JPRB, 0.26001E+01_JPRB, 0.26703E+01_JPRB, 0.21284E+01_JPRB, 0.28393E+01_JPRB ,&
&  0.30941E+01_JPRB, 0.30938E+01_JPRB, 0.25425E+01_JPRB, 0.18349E+01_JPRB, 0.19556E+01_JPRB, 0.15775E+01_JPRB, 0.11105E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.99695E+00_JPRB, 0.90911E+00_JPRB, 0.91184E+00_JPRB, 0.89456E+00_JPRB, 0.12549E+01_JPRB, 0.18518E+01_JPRB ,&
&  0.26890E+01_JPRB, 0.36285E+01_JPRB, 0.46054E+01_JPRB, 0.50748E+01_JPRB, 0.47800E+01_JPRB, 0.51364E+01_JPRB, 0.45861E+01_JPRB ,&
&  0.36110E+01_JPRB, 0.31517E+01_JPRB, 0.29437E+01_JPRB, 0.26217E+01_JPRB, 0.23670E+01_JPRB, 0.21926E+01_JPRB, 0.20470E+01_JPRB ,&
&  0.18991E+01_JPRB, 0.17679E+01_JPRB, 0.16764E+01_JPRB, 0.16213E+01_JPRB, 0.15733E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.16116E+01_JPRB, 0.15340E+01_JPRB, 0.14628E+01_JPRB, 0.14000E+01_JPRB, 0.13353E+01_JPRB, 0.12777E+01_JPRB, 0.12288E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.10591E+01_JPRB, 0.10005E+01_JPRB, 0.96510E+00_JPRB, 0.95683E+00_JPRB, 0.96151E+00_JPRB, 0.95607E+00_JPRB ,&
&  0.94035E+00_JPRB, 0.93338E+00_JPRB, 0.94528E+00_JPRB, 0.96521E+00_JPRB, 0.10095E+01_JPRB, 0.10872E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.16269E+01_JPRB, 0.20884E+01_JPRB, 0.22393E+01_JPRB, 0.23388E+01_JPRB, 0.24121E+01_JPRB, 0.23759E+01_JPRB, 0.23068E+01_JPRB ,&
&  0.21732E+01_JPRB, 0.20088E+01_JPRB, 0.18088E+01_JPRB, 0.15950E+01_JPRB, 0.14399E+01_JPRB, 0.13516E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.12890E+01_JPRB, 0.12629E+01_JPRB, 0.12132E+01_JPRB, 0.11685E+01_JPRB, 0.12007E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.12526E+01_JPRB, 0.12738E+01_JPRB, 0.13432E+01_JPRB, 0.13785E+01_JPRB, 0.13212E+01_JPRB, 0.11428E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.16721E+01_JPRB, 0.17470E+01_JPRB, 0.21464E+01_JPRB, 0.21613E+01_JPRB, 0.20131E+01_JPRB, 0.20282E+01_JPRB ,&
&  0.21622E+01_JPRB, 0.27772E+01_JPRB, 0.24254E+01_JPRB, 0.24512E+01_JPRB, 0.29634E+01_JPRB, 0.24332E+01_JPRB, 0.25878E+01_JPRB ,&
&  0.30373E+01_JPRB, 0.28607E+01_JPRB, 0.24304E+01_JPRB, 0.18734E+01_JPRB, 0.24902E+01_JPRB, 0.27001E+01_JPRB, 0.13555E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.10442E+01_JPRB, 0.98698E+00_JPRB, 0.94307E+00_JPRB, 0.95244E+00_JPRB, 0.14539E+01_JPRB, 0.26907E+01_JPRB ,&
&  0.32900E+01_JPRB, 0.48099E+01_JPRB, 0.55304E+01_JPRB, 0.56772E+01_JPRB, 0.50355E+01_JPRB, 0.44226E+01_JPRB, 0.36457E+01_JPRB ,&
&  0.33816E+01_JPRB, 0.32823E+01_JPRB, 0.30078E+01_JPRB, 0.27952E+01_JPRB, 0.25810E+01_JPRB, 0.23892E+01_JPRB, 0.22132E+01_JPRB ,&
&  0.20364E+01_JPRB, 0.19065E+01_JPRB, 0.18093E+01_JPRB, 0.17652E+01_JPRB, 0.16927E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.16966E+01_JPRB, 0.16321E+01_JPRB, 0.15373E+01_JPRB, 0.14657E+01_JPRB, 0.13958E+01_JPRB, 0.13205E+01_JPRB, 0.12715E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.11061E+01_JPRB, 0.10396E+01_JPRB, 0.10121E+01_JPRB, 0.99760E+00_JPRB, 0.98185E+00_JPRB, 0.97433E+00_JPRB ,&
&  0.96220E+00_JPRB, 0.96290E+00_JPRB, 0.10130E+01_JPRB, 0.12368E+01_JPRB, 0.13913E+01_JPRB, 0.15195E+01_JPRB, 0.17410E+01_JPRB ,&
&  0.23951E+01_JPRB, 0.26447E+01_JPRB, 0.25771E+01_JPRB, 0.25585E+01_JPRB, 0.24308E+01_JPRB, 0.23196E+01_JPRB, 0.21525E+01_JPRB ,&
&  0.20010E+01_JPRB, 0.18711E+01_JPRB, 0.17307E+01_JPRB, 0.15817E+01_JPRB, 0.14627E+01_JPRB, 0.14039E+01_JPRB, 0.13719E+01_JPRB ,&
&  0.13764E+01_JPRB, 0.13534E+01_JPRB, 0.13018E+01_JPRB, 0.12146E+01_JPRB, 0.11731E+01_JPRB, 0.12212E+01_JPRB, 0.12703E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.13655E+01_JPRB, 0.14167E+01_JPRB, 0.13920E+01_JPRB, 0.11776E+01_JPRB, 0.13001E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.14934E+01_JPRB, 0.14892E+01_JPRB, 0.18028E+01_JPRB, 0.20469E+01_JPRB, 0.19629E+01_JPRB, 0.19254E+01_JPRB ,&
&  0.23803E+01_JPRB, 0.23921E+01_JPRB, 0.22116E+01_JPRB, 0.23311E+01_JPRB, 0.25971E+01_JPRB, 0.32154E+01_JPRB, 0.22137E+01_JPRB ,&
&  0.20467E+01_JPRB, 0.24558E+01_JPRB, 0.21889E+01_JPRB, 0.17828E+01_JPRB, 0.23244E+01_JPRB, 0.34324E+01_JPRB, 0.28258E+01_JPRB ,&
&  0.13257E+01_JPRB, 0.10648E+01_JPRB, 0.11549E+01_JPRB, 0.12927E+01_JPRB, 0.12002E+01_JPRB, 0.18709E+01_JPRB, 0.50485E+01_JPRB ,&
&  0.48442E+01_JPRB, 0.60661E+01_JPRB, 0.59939E+01_JPRB, 0.55662E+01_JPRB, 0.49186E+01_JPRB, 0.41636E+01_JPRB, 0.37312E+01_JPRB ,&
&  0.34383E+01_JPRB, 0.32975E+01_JPRB, 0.31675E+01_JPRB, 0.30172E+01_JPRB, 0.28191E+01_JPRB, 0.25522E+01_JPRB, 0.23115E+01_JPRB ,&
&  0.21262E+01_JPRB, 0.19949E+01_JPRB, 0.18861E+01_JPRB, 0.18122E+01_JPRB, 0.17453E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.17933E+01_JPRB, 0.16833E+01_JPRB, 0.15708E+01_JPRB, 0.14971E+01_JPRB, 0.14134E+01_JPRB, 0.13332E+01_JPRB, 0.13158E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.11695E+01_JPRB, 0.10894E+01_JPRB, 0.10540E+01_JPRB, 0.10306E+01_JPRB, 0.10069E+01_JPRB, 0.99907E+00_JPRB ,&
&  0.99588E+00_JPRB, 0.10115E+01_JPRB, 0.10529E+01_JPRB, 0.12508E+01_JPRB, 0.14646E+01_JPRB, 0.14554E+01_JPRB, 0.17916E+01_JPRB ,&
&  0.27173E+01_JPRB, 0.28672E+01_JPRB, 0.26782E+01_JPRB, 0.25571E+01_JPRB, 0.23904E+01_JPRB, 0.23127E+01_JPRB, 0.20939E+01_JPRB ,&
&  0.18422E+01_JPRB, 0.16825E+01_JPRB, 0.15543E+01_JPRB, 0.15074E+01_JPRB, 0.14758E+01_JPRB, 0.14412E+01_JPRB, 0.14185E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.13211E+01_JPRB, 0.12499E+01_JPRB, 0.11671E+01_JPRB, 0.11778E+01_JPRB, 0.12505E+01_JPRB, 0.13174E+01_JPRB ,&
&  0.13769E+01_JPRB, 0.14573E+01_JPRB, 0.14614E+01_JPRB, 0.13532E+01_JPRB, 0.13159E+01_JPRB, 0.13173E+01_JPRB, 0.13667E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.14327E+01_JPRB, 0.15462E+01_JPRB, 0.16861E+01_JPRB, 0.18624E+01_JPRB, 0.18983E+01_JPRB, 0.19106E+01_JPRB ,&
&  0.21696E+01_JPRB, 0.21590E+01_JPRB, 0.20012E+01_JPRB, 0.21991E+01_JPRB, 0.24353E+01_JPRB, 0.29786E+01_JPRB, 0.29498E+01_JPRB ,&
&  0.24596E+01_JPRB, 0.19867E+01_JPRB, 0.18551E+01_JPRB, 0.19478E+01_JPRB, 0.22962E+01_JPRB, 0.32240E+01_JPRB, 0.49303E+01_JPRB ,&
&  0.50143E+01_JPRB, 0.32953E+01_JPRB, 0.27170E+01_JPRB, 0.36766E+01_JPRB, 0.31682E+01_JPRB, 0.30449E+01_JPRB, 0.42669E+01_JPRB ,&
&  0.54187E+01_JPRB, 0.62973E+01_JPRB, 0.60987E+01_JPRB, 0.55086E+01_JPRB, 0.49473E+01_JPRB, 0.42017E+01_JPRB, 0.38645E+01_JPRB ,&
&  0.36332E+01_JPRB, 0.34187E+01_JPRB, 0.33547E+01_JPRB, 0.32079E+01_JPRB, 0.29373E+01_JPRB, 0.26021E+01_JPRB, 0.23051E+01_JPRB ,&
&  0.21257E+01_JPRB, 0.20185E+01_JPRB, 0.19439E+01_JPRB, 0.18973E+01_JPRB, 0.18539E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.18040E+01_JPRB, 0.16609E+01_JPRB, 0.15573E+01_JPRB, 0.14747E+01_JPRB, 0.13837E+01_JPRB, 0.13291E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.13410E+01_JPRB, 0.12193E+01_JPRB, 0.11453E+01_JPRB, 0.10848E+01_JPRB, 0.10440E+01_JPRB, 0.10283E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.10240E+01_JPRB, 0.10416E+01_JPRB, 0.10832E+01_JPRB, 0.12945E+01_JPRB, 0.16685E+01_JPRB, 0.22581E+01_JPRB, 0.17962E+01_JPRB ,&
&  0.26708E+01_JPRB, 0.32309E+01_JPRB, 0.29025E+01_JPRB, 0.25692E+01_JPRB, 0.23017E+01_JPRB, 0.23104E+01_JPRB, 0.20426E+01_JPRB ,&
&  0.17153E+01_JPRB, 0.15334E+01_JPRB, 0.14424E+01_JPRB, 0.14083E+01_JPRB, 0.14058E+01_JPRB, 0.13820E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.12806E+01_JPRB, 0.12378E+01_JPRB, 0.11864E+01_JPRB, 0.11673E+01_JPRB, 0.12079E+01_JPRB, 0.12874E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.14943E+01_JPRB, 0.15375E+01_JPRB, 0.15081E+01_JPRB, 0.13813E+01_JPRB, 0.13650E+01_JPRB, 0.13534E+01_JPRB, 0.13559E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.12712E+01_JPRB, 0.14592E+01_JPRB, 0.16672E+01_JPRB, 0.17791E+01_JPRB, 0.17383E+01_JPRB, 0.17311E+01_JPRB ,&
&  0.20422E+01_JPRB, 0.22956E+01_JPRB, 0.24872E+01_JPRB, 0.24097E+01_JPRB, 0.26653E+01_JPRB, 0.29823E+01_JPRB, 0.32337E+01_JPRB ,&
&  0.31235E+01_JPRB, 0.24768E+01_JPRB, 0.22292E+01_JPRB, 0.22450E+01_JPRB, 0.23268E+01_JPRB, 0.31332E+01_JPRB, 0.44956E+01_JPRB ,&
&  0.59839E+01_JPRB, 0.81587E+01_JPRB, 0.81477E+01_JPRB, 0.63905E+01_JPRB, 0.55295E+01_JPRB, 0.50109E+01_JPRB, 0.47402E+01_JPRB ,&
&  0.61306E+01_JPRB, 0.63997E+01_JPRB, 0.60571E+01_JPRB, 0.66489E+01_JPRB, 0.51657E+01_JPRB, 0.43228E+01_JPRB, 0.38979E+01_JPRB ,&
&  0.37444E+01_JPRB, 0.35376E+01_JPRB, 0.33347E+01_JPRB, 0.31034E+01_JPRB, 0.27527E+01_JPRB, 0.24316E+01_JPRB, 0.22181E+01_JPRB ,&
&  0.20926E+01_JPRB, 0.20211E+01_JPRB, 0.19687E+01_JPRB, 0.19397E+01_JPRB, 0.19031E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.17198E+01_JPRB, 0.15875E+01_JPRB, 0.14538E+01_JPRB, 0.13865E+01_JPRB, 0.13290E+01_JPRB, 0.13183E+01_JPRB, 0.13492E+01_JPRB ,&
&  0.14251E+01_JPRB, 0.12575E+01_JPRB, 0.11537E+01_JPRB, 0.10762E+01_JPRB, 0.10443E+01_JPRB, 0.10454E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10951E+01_JPRB, 0.11999E+01_JPRB, 0.13669E+01_JPRB, 0.17456E+01_JPRB, 0.21985E+01_JPRB, 0.23463E+01_JPRB ,&
&  0.29029E+01_JPRB, 0.41986E+01_JPRB, 0.31465E+01_JPRB, 0.25566E+01_JPRB, 0.23993E+01_JPRB, 0.26916E+01_JPRB, 0.22349E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.15606E+01_JPRB, 0.15147E+01_JPRB, 0.13926E+01_JPRB, 0.13528E+01_JPRB, 0.13112E+01_JPRB, 0.12817E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.12165E+01_JPRB, 0.11827E+01_JPRB, 0.12013E+01_JPRB, 0.12661E+01_JPRB, 0.13603E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.16289E+01_JPRB, 0.15919E+01_JPRB, 0.14850E+01_JPRB, 0.14277E+01_JPRB, 0.14036E+01_JPRB, 0.14086E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.13763E+01_JPRB, 0.13328E+01_JPRB, 0.14442E+01_JPRB, 0.15029E+01_JPRB, 0.15330E+01_JPRB, 0.16483E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.20282E+01_JPRB, 0.21462E+01_JPRB, 0.27012E+01_JPRB, 0.24988E+01_JPRB, 0.27946E+01_JPRB, 0.30523E+01_JPRB, 0.29051E+01_JPRB ,&
&  0.24808E+01_JPRB, 0.23113E+01_JPRB, 0.21272E+01_JPRB, 0.22535E+01_JPRB, 0.27511E+01_JPRB, 0.38089E+01_JPRB, 0.48777E+01_JPRB ,&
&  0.60553E+01_JPRB, 0.72153E+01_JPRB, 0.80903E+01_JPRB, 0.64833E+01_JPRB, 0.58193E+01_JPRB, 0.55095E+01_JPRB, 0.56264E+01_JPRB ,&
&  0.65611E+01_JPRB, 0.66651E+01_JPRB, 0.66915E+01_JPRB, 0.62278E+01_JPRB, 0.47797E+01_JPRB, 0.40316E+01_JPRB, 0.36667E+01_JPRB ,&
&  0.34543E+01_JPRB, 0.32482E+01_JPRB, 0.29018E+01_JPRB, 0.26221E+01_JPRB, 0.24171E+01_JPRB, 0.22531E+01_JPRB, 0.20943E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.19270E+01_JPRB, 0.18935E+01_JPRB, 0.18803E+01_JPRB, 0.18272E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.15569E+01_JPRB, 0.14585E+01_JPRB, 0.13895E+01_JPRB, 0.13628E+01_JPRB, 0.13248E+01_JPRB, 0.13427E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.12192E+01_JPRB, 0.11120E+01_JPRB, 0.10721E+01_JPRB, 0.10604E+01_JPRB, 0.10563E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.11883E+01_JPRB, 0.13465E+01_JPRB, 0.15453E+01_JPRB, 0.19324E+01_JPRB, 0.25106E+01_JPRB, 0.31243E+01_JPRB ,&
&  0.37038E+01_JPRB, 0.36020E+01_JPRB, 0.30457E+01_JPRB, 0.24933E+01_JPRB, 0.23192E+01_JPRB, 0.25346E+01_JPRB, 0.24368E+01_JPRB ,&
&  0.20432E+01_JPRB, 0.17080E+01_JPRB, 0.15667E+01_JPRB, 0.14186E+01_JPRB, 0.13176E+01_JPRB, 0.12959E+01_JPRB, 0.12642E+01_JPRB ,&
&  0.12199E+01_JPRB, 0.11858E+01_JPRB, 0.11794E+01_JPRB, 0.12397E+01_JPRB, 0.13318E+01_JPRB, 0.14729E+01_JPRB, 0.16805E+01_JPRB ,&
&  0.17203E+01_JPRB, 0.16260E+01_JPRB, 0.15398E+01_JPRB, 0.14491E+01_JPRB, 0.14426E+01_JPRB, 0.14478E+01_JPRB, 0.14457E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.14604E+01_JPRB, 0.14977E+01_JPRB, 0.15045E+01_JPRB, 0.14735E+01_JPRB, 0.16575E+01_JPRB, 0.17529E+01_JPRB ,&
&  0.20169E+01_JPRB, 0.22960E+01_JPRB, 0.29330E+01_JPRB, 0.24470E+01_JPRB, 0.28888E+01_JPRB, 0.29615E+01_JPRB, 0.27104E+01_JPRB ,&
&  0.23551E+01_JPRB, 0.22200E+01_JPRB, 0.22527E+01_JPRB, 0.25259E+01_JPRB, 0.34134E+01_JPRB, 0.45516E+01_JPRB, 0.65633E+01_JPRB ,&
&  0.66032E+01_JPRB, 0.74059E+01_JPRB, 0.69552E+01_JPRB, 0.68154E+01_JPRB, 0.60167E+01_JPRB, 0.57585E+01_JPRB, 0.57248E+01_JPRB ,&
&  0.61917E+01_JPRB, 0.60722E+01_JPRB, 0.55456E+01_JPRB, 0.49103E+01_JPRB, 0.39624E+01_JPRB, 0.33742E+01_JPRB, 0.30845E+01_JPRB ,&
&  0.27973E+01_JPRB, 0.25219E+01_JPRB, 0.23088E+01_JPRB, 0.22193E+01_JPRB, 0.21172E+01_JPRB, 0.20053E+01_JPRB, 0.19184E+01_JPRB ,&
&  0.18679E+01_JPRB, 0.18219E+01_JPRB, 0.17850E+01_JPRB, 0.17373E+01_JPRB, 0.16676E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.14129E+01_JPRB, 0.13559E+01_JPRB, 0.13243E+01_JPRB, 0.12802E+01_JPRB, 0.12371E+01_JPRB, 0.12286E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.11140E+01_JPRB, 0.10957E+01_JPRB, 0.10931E+01_JPRB, 0.10800E+01_JPRB, 0.10675E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.12790E+01_JPRB, 0.14527E+01_JPRB, 0.17302E+01_JPRB, 0.20934E+01_JPRB, 0.26510E+01_JPRB, 0.32747E+01_JPRB ,&
&  0.33697E+01_JPRB, 0.23798E+01_JPRB, 0.25988E+01_JPRB, 0.24667E+01_JPRB, 0.22315E+01_JPRB, 0.23370E+01_JPRB, 0.22902E+01_JPRB ,&
&  0.18325E+01_JPRB, 0.16777E+01_JPRB, 0.16447E+01_JPRB, 0.15490E+01_JPRB, 0.13036E+01_JPRB, 0.12756E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.12046E+01_JPRB, 0.11907E+01_JPRB, 0.12207E+01_JPRB, 0.13348E+01_JPRB, 0.14862E+01_JPRB, 0.17066E+01_JPRB, 0.18269E+01_JPRB ,&
&  0.17657E+01_JPRB, 0.17094E+01_JPRB, 0.16745E+01_JPRB, 0.15775E+01_JPRB, 0.15587E+01_JPRB, 0.15671E+01_JPRB, 0.15634E+01_JPRB ,&
&  0.15528E+01_JPRB, 0.15420E+01_JPRB, 0.15946E+01_JPRB, 0.16949E+01_JPRB, 0.17868E+01_JPRB, 0.17253E+01_JPRB, 0.19292E+01_JPRB ,&
&  0.21511E+01_JPRB, 0.25761E+01_JPRB, 0.28936E+01_JPRB, 0.33414E+01_JPRB, 0.24689E+01_JPRB, 0.26673E+01_JPRB, 0.22830E+01_JPRB ,&
&  0.23763E+01_JPRB, 0.24773E+01_JPRB, 0.28610E+01_JPRB, 0.34393E+01_JPRB, 0.46991E+01_JPRB, 0.58854E+01_JPRB, 0.76993E+01_JPRB ,&
&  0.67465E+01_JPRB, 0.64026E+01_JPRB, 0.60971E+01_JPRB, 0.65619E+01_JPRB, 0.61535E+01_JPRB, 0.62767E+01_JPRB, 0.56827E+01_JPRB ,&
&  0.55649E+01_JPRB, 0.49273E+01_JPRB, 0.43071E+01_JPRB, 0.38996E+01_JPRB, 0.31545E+01_JPRB, 0.25452E+01_JPRB, 0.22003E+01_JPRB ,&
&  0.20074E+01_JPRB, 0.19490E+01_JPRB, 0.19142E+01_JPRB, 0.18874E+01_JPRB, 0.18338E+01_JPRB, 0.17833E+01_JPRB, 0.17457E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.16622E+01_JPRB, 0.16167E+01_JPRB, 0.15661E+01_JPRB, 0.14993E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.13149E+01_JPRB, 0.12705E+01_JPRB, 0.12442E+01_JPRB, 0.12069E+01_JPRB, 0.11787E+01_JPRB, 0.11678E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.11352E+01_JPRB, 0.11219E+01_JPRB, 0.11184E+01_JPRB, 0.11018E+01_JPRB, 0.10746E+01_JPRB, 0.10759E+01_JPRB, 0.11311E+01_JPRB ,&
&  0.12523E+01_JPRB, 0.14345E+01_JPRB, 0.16456E+01_JPRB, 0.18704E+01_JPRB, 0.22214E+01_JPRB, 0.25181E+01_JPRB, 0.26588E+01_JPRB ,&
&  0.28524E+01_JPRB, 0.29036E+01_JPRB, 0.24434E+01_JPRB, 0.17614E+01_JPRB, 0.16409E+01_JPRB, 0.19771E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.15684E+01_JPRB, 0.14854E+01_JPRB, 0.14421E+01_JPRB, 0.13904E+01_JPRB, 0.13057E+01_JPRB, 0.12682E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.12727E+01_JPRB, 0.13790E+01_JPRB, 0.15862E+01_JPRB, 0.17920E+01_JPRB, 0.19325E+01_JPRB, 0.19325E+01_JPRB ,&
&  0.19174E+01_JPRB, 0.19389E+01_JPRB, 0.19273E+01_JPRB, 0.18357E+01_JPRB, 0.17645E+01_JPRB, 0.17768E+01_JPRB, 0.17874E+01_JPRB ,&
&  0.17910E+01_JPRB, 0.17898E+01_JPRB, 0.17975E+01_JPRB, 0.18858E+01_JPRB, 0.19534E+01_JPRB, 0.18547E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.23710E+01_JPRB, 0.29633E+01_JPRB, 0.30095E+01_JPRB, 0.35864E+01_JPRB, 0.21950E+01_JPRB, 0.24384E+01_JPRB, 0.26364E+01_JPRB ,&
&  0.26513E+01_JPRB, 0.31677E+01_JPRB, 0.37163E+01_JPRB, 0.44745E+01_JPRB, 0.56542E+01_JPRB, 0.64081E+01_JPRB, 0.70965E+01_JPRB ,&
&  0.59452E+01_JPRB, 0.53075E+01_JPRB, 0.52703E+01_JPRB, 0.56164E+01_JPRB, 0.57443E+01_JPRB, 0.56791E+01_JPRB, 0.47063E+01_JPRB ,&
&  0.39124E+01_JPRB, 0.39287E+01_JPRB, 0.34084E+01_JPRB, 0.32194E+01_JPRB, 0.23431E+01_JPRB, 0.18452E+01_JPRB, 0.17648E+01_JPRB ,&
&  0.16992E+01_JPRB, 0.16501E+01_JPRB, 0.16406E+01_JPRB, 0.16253E+01_JPRB, 0.16059E+01_JPRB, 0.15873E+01_JPRB, 0.15547E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.15017E+01_JPRB, 0.14711E+01_JPRB, 0.14311E+01_JPRB, 0.13780E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.12405E+01_JPRB, 0.12095E+01_JPRB, 0.11973E+01_JPRB, 0.11530E+01_JPRB, 0.11144E+01_JPRB, 0.11060E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.11103E+01_JPRB, 0.11003E+01_JPRB, 0.10839E+01_JPRB, 0.10719E+01_JPRB, 0.10892E+01_JPRB, 0.11487E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.15627E+01_JPRB, 0.17756E+01_JPRB, 0.19896E+01_JPRB, 0.22728E+01_JPRB, 0.23915E+01_JPRB, 0.23838E+01_JPRB ,&
&  0.24593E+01_JPRB, 0.25626E+01_JPRB, 0.25147E+01_JPRB, 0.24726E+01_JPRB, 0.20620E+01_JPRB, 0.22419E+01_JPRB, 0.22958E+01_JPRB ,&
&  0.21776E+01_JPRB, 0.17462E+01_JPRB, 0.14547E+01_JPRB, 0.13939E+01_JPRB, 0.13392E+01_JPRB, 0.13254E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.14155E+01_JPRB, 0.15307E+01_JPRB, 0.17153E+01_JPRB, 0.19225E+01_JPRB, 0.20616E+01_JPRB, 0.21206E+01_JPRB, 0.21415E+01_JPRB ,&
&  0.21754E+01_JPRB, 0.22180E+01_JPRB, 0.22618E+01_JPRB, 0.21987E+01_JPRB, 0.21795E+01_JPRB, 0.21753E+01_JPRB, 0.22282E+01_JPRB ,&
&  0.22819E+01_JPRB, 0.21889E+01_JPRB, 0.21964E+01_JPRB, 0.23232E+01_JPRB, 0.23385E+01_JPRB, 0.23119E+01_JPRB, 0.24158E+01_JPRB ,&
&  0.28247E+01_JPRB, 0.35054E+01_JPRB, 0.28558E+01_JPRB, 0.30366E+01_JPRB, 0.35597E+01_JPRB, 0.29933E+01_JPRB, 0.27720E+01_JPRB ,&
&  0.31155E+01_JPRB, 0.36002E+01_JPRB, 0.41143E+01_JPRB, 0.48024E+01_JPRB, 0.58434E+01_JPRB, 0.71694E+01_JPRB, 0.59454E+01_JPRB ,&
&  0.49841E+01_JPRB, 0.44019E+01_JPRB, 0.44639E+01_JPRB, 0.45931E+01_JPRB, 0.45859E+01_JPRB, 0.44243E+01_JPRB, 0.37086E+01_JPRB ,&
&  0.32102E+01_JPRB, 0.32159E+01_JPRB, 0.29176E+01_JPRB, 0.25807E+01_JPRB, 0.23045E+01_JPRB, 0.16693E+01_JPRB, 0.15723E+01_JPRB ,&
&  0.14688E+01_JPRB, 0.14649E+01_JPRB, 0.14763E+01_JPRB, 0.14759E+01_JPRB, 0.14629E+01_JPRB, 0.14350E+01_JPRB, 0.14098E+01_JPRB ,&
&  0.13928E+01_JPRB, 0.13723E+01_JPRB, 0.13531E+01_JPRB, 0.13290E+01_JPRB, 0.12844E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.11843E+01_JPRB, 0.11650E+01_JPRB, 0.11479E+01_JPRB, 0.11107E+01_JPRB, 0.10708E+01_JPRB, 0.10620E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10110E+01_JPRB, 0.98262E+00_JPRB, 0.10015E+01_JPRB, 0.10478E+01_JPRB, 0.11352E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.15052E+01_JPRB, 0.17253E+01_JPRB, 0.19093E+01_JPRB, 0.21698E+01_JPRB, 0.23202E+01_JPRB, 0.24162E+01_JPRB ,&
&  0.25283E+01_JPRB, 0.26565E+01_JPRB, 0.27593E+01_JPRB, 0.28354E+01_JPRB, 0.28601E+01_JPRB, 0.29519E+01_JPRB, 0.31593E+01_JPRB ,&
&  0.26510E+01_JPRB, 0.19036E+01_JPRB, 0.16412E+01_JPRB, 0.14914E+01_JPRB, 0.14631E+01_JPRB, 0.14910E+01_JPRB, 0.15697E+01_JPRB ,&
&  0.16979E+01_JPRB, 0.18444E+01_JPRB, 0.20299E+01_JPRB, 0.21764E+01_JPRB, 0.22963E+01_JPRB, 0.23817E+01_JPRB, 0.24396E+01_JPRB ,&
&  0.25386E+01_JPRB, 0.26438E+01_JPRB, 0.27985E+01_JPRB, 0.26900E+01_JPRB, 0.29208E+01_JPRB, 0.29180E+01_JPRB, 0.29354E+01_JPRB ,&
&  0.28620E+01_JPRB, 0.26793E+01_JPRB, 0.27647E+01_JPRB, 0.28917E+01_JPRB, 0.30234E+01_JPRB, 0.30564E+01_JPRB, 0.30635E+01_JPRB ,&
&  0.34698E+01_JPRB, 0.36128E+01_JPRB, 0.27774E+01_JPRB, 0.25106E+01_JPRB, 0.26417E+01_JPRB, 0.23565E+01_JPRB, 0.29226E+01_JPRB ,&
&  0.31634E+01_JPRB, 0.35392E+01_JPRB, 0.39269E+01_JPRB, 0.45253E+01_JPRB, 0.52217E+01_JPRB, 0.57367E+01_JPRB, 0.49829E+01_JPRB ,&
&  0.38686E+01_JPRB, 0.34625E+01_JPRB, 0.34503E+01_JPRB, 0.34062E+01_JPRB, 0.33739E+01_JPRB, 0.33213E+01_JPRB, 0.31235E+01_JPRB ,&
&  0.28821E+01_JPRB, 0.26165E+01_JPRB, 0.24050E+01_JPRB, 0.19136E+01_JPRB, 0.16318E+01_JPRB, 0.14489E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.13181E+01_JPRB, 0.13556E+01_JPRB, 0.13725E+01_JPRB, 0.13743E+01_JPRB, 0.13657E+01_JPRB, 0.13361E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.12886E+01_JPRB, 0.12698E+01_JPRB, 0.12530E+01_JPRB, 0.12168E+01_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.10946E+01_JPRB, 0.10646E+01_JPRB, 0.10429E+01_JPRB, 0.10191E+01_JPRB, 0.98119E+00_JPRB, 0.95504E+00_JPRB, 0.94610E+00_JPRB ,&
&  0.90415E+00_JPRB, 0.90431E+00_JPRB, 0.88701E+00_JPRB, 0.90339E+00_JPRB, 0.95060E+00_JPRB, 0.99734E+00_JPRB, 0.10627E+01_JPRB ,&
&  0.11152E+01_JPRB, 0.11908E+01_JPRB, 0.13196E+01_JPRB, 0.15915E+01_JPRB, 0.19044E+01_JPRB, 0.21547E+01_JPRB, 0.24537E+01_JPRB ,&
&  0.26073E+01_JPRB, 0.27493E+01_JPRB, 0.29921E+01_JPRB, 0.33305E+01_JPRB, 0.36057E+01_JPRB, 0.38664E+01_JPRB, 0.38676E+01_JPRB ,&
&  0.29113E+01_JPRB, 0.17759E+01_JPRB, 0.16830E+01_JPRB, 0.15360E+01_JPRB, 0.15774E+01_JPRB, 0.16570E+01_JPRB, 0.17443E+01_JPRB ,&
&  0.18862E+01_JPRB, 0.20312E+01_JPRB, 0.21840E+01_JPRB, 0.23108E+01_JPRB, 0.24588E+01_JPRB, 0.26170E+01_JPRB, 0.27059E+01_JPRB ,&
&  0.27920E+01_JPRB, 0.28798E+01_JPRB, 0.29525E+01_JPRB, 0.28923E+01_JPRB, 0.30432E+01_JPRB, 0.32487E+01_JPRB, 0.34130E+01_JPRB ,&
&  0.35724E+01_JPRB, 0.35880E+01_JPRB, 0.30549E+01_JPRB, 0.31650E+01_JPRB, 0.35948E+01_JPRB, 0.33964E+01_JPRB, 0.35223E+01_JPRB ,&
&  0.41109E+01_JPRB, 0.42644E+01_JPRB, 0.25945E+01_JPRB, 0.23137E+01_JPRB, 0.24392E+01_JPRB, 0.25487E+01_JPRB, 0.27570E+01_JPRB ,&
&  0.29345E+01_JPRB, 0.32590E+01_JPRB, 0.36108E+01_JPRB, 0.39159E+01_JPRB, 0.40758E+01_JPRB, 0.38676E+01_JPRB, 0.36009E+01_JPRB ,&
&  0.29508E+01_JPRB, 0.26687E+01_JPRB, 0.26739E+01_JPRB, 0.27155E+01_JPRB, 0.28253E+01_JPRB, 0.29452E+01_JPRB, 0.26367E+01_JPRB ,&
&  0.25325E+01_JPRB, 0.25038E+01_JPRB, 0.27316E+01_JPRB, 0.21880E+01_JPRB, 0.15157E+01_JPRB, 0.12949E+01_JPRB, 0.11921E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.12884E+01_JPRB, 0.13091E+01_JPRB, 0.13227E+01_JPRB, 0.13182E+01_JPRB, 0.13028E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.12285E+01_JPRB, 0.11967E+01_JPRB, 0.11633E+01_JPRB, 0.11317E+01_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.94696E+00_JPRB, 0.91815E+00_JPRB, 0.90551E+00_JPRB, 0.85398E+00_JPRB, 0.86669E+00_JPRB, 0.82981E+00_JPRB, 0.86948E+00_JPRB ,&
&  0.84450E+00_JPRB, 0.84581E+00_JPRB, 0.85062E+00_JPRB, 0.87247E+00_JPRB, 0.89392E+00_JPRB, 0.91861E+00_JPRB, 0.93988E+00_JPRB ,&
&  0.96559E+00_JPRB, 0.10239E+01_JPRB, 0.11117E+01_JPRB, 0.12196E+01_JPRB, 0.13624E+01_JPRB, 0.15126E+01_JPRB, 0.16540E+01_JPRB ,&
&  0.18405E+01_JPRB, 0.20115E+01_JPRB, 0.21350E+01_JPRB, 0.24029E+01_JPRB, 0.30423E+01_JPRB, 0.38268E+01_JPRB, 0.39670E+01_JPRB ,&
&  0.25469E+01_JPRB, 0.18165E+01_JPRB, 0.15683E+01_JPRB, 0.14478E+01_JPRB, 0.15315E+01_JPRB, 0.17138E+01_JPRB, 0.17881E+01_JPRB ,&
&  0.18741E+01_JPRB, 0.19801E+01_JPRB, 0.21099E+01_JPRB, 0.22502E+01_JPRB, 0.24228E+01_JPRB, 0.25920E+01_JPRB, 0.26803E+01_JPRB ,&
&  0.26944E+01_JPRB, 0.27499E+01_JPRB, 0.27673E+01_JPRB, 0.29473E+01_JPRB, 0.33068E+01_JPRB, 0.37310E+01_JPRB, 0.40293E+01_JPRB ,&
&  0.41897E+01_JPRB, 0.41306E+01_JPRB, 0.35407E+01_JPRB, 0.35983E+01_JPRB, 0.38040E+01_JPRB, 0.37669E+01_JPRB, 0.36712E+01_JPRB ,&
&  0.40686E+01_JPRB, 0.39437E+01_JPRB, 0.27434E+01_JPRB, 0.25718E+01_JPRB, 0.25360E+01_JPRB, 0.25415E+01_JPRB, 0.24493E+01_JPRB ,&
&  0.26868E+01_JPRB, 0.29451E+01_JPRB, 0.31200E+01_JPRB, 0.31479E+01_JPRB, 0.30275E+01_JPRB, 0.27882E+01_JPRB, 0.27394E+01_JPRB ,&
&  0.24936E+01_JPRB, 0.23989E+01_JPRB, 0.24252E+01_JPRB, 0.25132E+01_JPRB, 0.29265E+01_JPRB, 0.26481E+01_JPRB, 0.25630E+01_JPRB ,&
&  0.22243E+01_JPRB, 0.22005E+01_JPRB, 0.18337E+01_JPRB, 0.20979E+01_JPRB, 0.13706E+01_JPRB, 0.10765E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.11529E+01_JPRB, 0.11555E+01_JPRB, 0.11751E+01_JPRB, 0.11810E+01_JPRB, 0.11658E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.11174E+01_JPRB, 0.10725E+01_JPRB, 0.10360E+01_JPRB, 0.99661E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.89939E+00_JPRB, 0.89545E+00_JPRB, 0.86787E+00_JPRB, 0.82301E+00_JPRB, 0.79502E+00_JPRB, 0.77337E+00_JPRB, 0.78367E+00_JPRB ,&
&  0.79573E+00_JPRB, 0.82373E+00_JPRB, 0.84942E+00_JPRB, 0.87439E+00_JPRB, 0.88577E+00_JPRB, 0.89370E+00_JPRB, 0.91609E+00_JPRB ,&
&  0.94611E+00_JPRB, 0.97448E+00_JPRB, 0.10136E+01_JPRB, 0.10665E+01_JPRB, 0.11228E+01_JPRB, 0.11956E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.13074E+01_JPRB, 0.13635E+01_JPRB, 0.14675E+01_JPRB, 0.18297E+01_JPRB, 0.22357E+01_JPRB, 0.28114E+01_JPRB ,&
&  0.21469E+01_JPRB, 0.19560E+01_JPRB, 0.16590E+01_JPRB, 0.15266E+01_JPRB, 0.15686E+01_JPRB, 0.15519E+01_JPRB, 0.15827E+01_JPRB ,&
&  0.16390E+01_JPRB, 0.16723E+01_JPRB, 0.17092E+01_JPRB, 0.17512E+01_JPRB, 0.17991E+01_JPRB, 0.18994E+01_JPRB, 0.18573E+01_JPRB ,&
&  0.18441E+01_JPRB, 0.19224E+01_JPRB, 0.20239E+01_JPRB, 0.23201E+01_JPRB, 0.27561E+01_JPRB, 0.31737E+01_JPRB, 0.32151E+01_JPRB ,&
&  0.32559E+01_JPRB, 0.34840E+01_JPRB, 0.34204E+01_JPRB, 0.34580E+01_JPRB, 0.34260E+01_JPRB, 0.33375E+01_JPRB, 0.33812E+01_JPRB ,&
&  0.32679E+01_JPRB, 0.30914E+01_JPRB, 0.29271E+01_JPRB, 0.26147E+01_JPRB, 0.24200E+01_JPRB, 0.22441E+01_JPRB, 0.22275E+01_JPRB ,&
&  0.23291E+01_JPRB, 0.24775E+01_JPRB, 0.25490E+01_JPRB, 0.24426E+01_JPRB, 0.23259E+01_JPRB, 0.21742E+01_JPRB, 0.22062E+01_JPRB ,&
&  0.21922E+01_JPRB, 0.20551E+01_JPRB, 0.20835E+01_JPRB, 0.23072E+01_JPRB, 0.25898E+01_JPRB, 0.42905E+01_JPRB, 0.37096E+01_JPRB ,&
&  0.21208E+01_JPRB, 0.17212E+01_JPRB, 0.11226E+01_JPRB, 0.16042E+01_JPRB, 0.10368E+01_JPRB, 0.92977E+00_JPRB, 0.95499E+00_JPRB ,&
&  0.97051E+00_JPRB, 0.98817E+00_JPRB, 0.99956E+00_JPRB, 0.10060E+01_JPRB, 0.10083E+01_JPRB, 0.10115E+01_JPRB, 0.99791E+00_JPRB ,&
&  0.98668E+00_JPRB, 0.96819E+00_JPRB, 0.96376E+00_JPRB, 0.92696E+00_JPRB, 0.91005E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.85220E+00_JPRB, 0.85052E+00_JPRB, 0.81397E+00_JPRB, 0.76125E+00_JPRB, 0.73820E+00_JPRB, 0.74203E+00_JPRB, 0.76083E+00_JPRB ,&
&  0.78064E+00_JPRB, 0.80617E+00_JPRB, 0.83634E+00_JPRB, 0.86641E+00_JPRB, 0.89415E+00_JPRB, 0.92455E+00_JPRB, 0.96088E+00_JPRB ,&
&  0.99562E+00_JPRB, 0.10378E+01_JPRB, 0.10820E+01_JPRB, 0.11296E+01_JPRB, 0.11748E+01_JPRB, 0.12507E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.14832E+01_JPRB, 0.15918E+01_JPRB, 0.17106E+01_JPRB, 0.18677E+01_JPRB, 0.20257E+01_JPRB, 0.19393E+01_JPRB ,&
&  0.22843E+01_JPRB, 0.19696E+01_JPRB, 0.17100E+01_JPRB, 0.15677E+01_JPRB, 0.14839E+01_JPRB, 0.14202E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.12724E+01_JPRB, 0.12245E+01_JPRB, 0.11840E+01_JPRB, 0.11604E+01_JPRB, 0.12388E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.13607E+01_JPRB, 0.15522E+01_JPRB, 0.17215E+01_JPRB, 0.20009E+01_JPRB, 0.22342E+01_JPRB, 0.24028E+01_JPRB, 0.23111E+01_JPRB ,&
&  0.23417E+01_JPRB, 0.25629E+01_JPRB, 0.25404E+01_JPRB, 0.26175E+01_JPRB, 0.26079E+01_JPRB, 0.25068E+01_JPRB, 0.24586E+01_JPRB ,&
&  0.22410E+01_JPRB, 0.22779E+01_JPRB, 0.24479E+01_JPRB, 0.26899E+01_JPRB, 0.23319E+01_JPRB, 0.20960E+01_JPRB, 0.21047E+01_JPRB ,&
&  0.20602E+01_JPRB, 0.20992E+01_JPRB, 0.21079E+01_JPRB, 0.19846E+01_JPRB, 0.18807E+01_JPRB, 0.17968E+01_JPRB, 0.18476E+01_JPRB ,&
&  0.18809E+01_JPRB, 0.17502E+01_JPRB, 0.18528E+01_JPRB, 0.21952E+01_JPRB, 0.26964E+01_JPRB, 0.29581E+01_JPRB, 0.35703E+01_JPRB ,&
&  0.20525E+01_JPRB, 0.11731E+01_JPRB, 0.11477E+01_JPRB, 0.11600E+01_JPRB, 0.90603E+00_JPRB, 0.78131E+00_JPRB, 0.83089E+00_JPRB ,&
&  0.86571E+00_JPRB, 0.88919E+00_JPRB, 0.87291E+00_JPRB, 0.89857E+00_JPRB, 0.95443E+00_JPRB, 0.95277E+00_JPRB, 0.90417E+00_JPRB ,&
&  0.91547E+00_JPRB, 0.87943E+00_JPRB, 0.86378E+00_JPRB, 0.86160E+00_JPRB, 0.85941E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.79525E+00_JPRB, 0.78178E+00_JPRB, 0.74049E+00_JPRB, 0.69357E+00_JPRB, 0.67552E+00_JPRB, 0.69136E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.74610E+00_JPRB, 0.77750E+00_JPRB, 0.80947E+00_JPRB, 0.84532E+00_JPRB, 0.88282E+00_JPRB, 0.91964E+00_JPRB, 0.94905E+00_JPRB ,&
&  0.97358E+00_JPRB, 0.10055E+01_JPRB, 0.10366E+01_JPRB, 0.10680E+01_JPRB, 0.10949E+01_JPRB, 0.11123E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.12084E+01_JPRB, 0.12912E+01_JPRB, 0.13959E+01_JPRB, 0.15138E+01_JPRB, 0.16278E+01_JPRB, 0.17930E+01_JPRB, 0.16001E+01_JPRB ,&
&  0.20066E+01_JPRB, 0.18956E+01_JPRB, 0.17279E+01_JPRB, 0.15806E+01_JPRB, 0.14620E+01_JPRB, 0.13727E+01_JPRB, 0.13021E+01_JPRB ,&
&  0.12514E+01_JPRB, 0.13003E+01_JPRB, 0.12398E+01_JPRB, 0.12533E+01_JPRB, 0.10748E+01_JPRB, 0.11345E+01_JPRB, 0.12297E+01_JPRB ,&
&  0.14075E+01_JPRB, 0.16129E+01_JPRB, 0.17902E+01_JPRB, 0.19510E+01_JPRB, 0.20243E+01_JPRB, 0.20821E+01_JPRB, 0.19837E+01_JPRB ,&
&  0.19891E+01_JPRB, 0.19908E+01_JPRB, 0.18732E+01_JPRB, 0.19380E+01_JPRB, 0.20147E+01_JPRB, 0.17635E+01_JPRB, 0.15952E+01_JPRB ,&
&  0.14823E+01_JPRB, 0.17452E+01_JPRB, 0.23528E+01_JPRB, 0.26773E+01_JPRB, 0.22832E+01_JPRB, 0.21361E+01_JPRB, 0.20148E+01_JPRB ,&
&  0.18661E+01_JPRB, 0.18410E+01_JPRB, 0.18023E+01_JPRB, 0.17083E+01_JPRB, 0.15583E+01_JPRB, 0.14739E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.15656E+01_JPRB, 0.14634E+01_JPRB, 0.14702E+01_JPRB, 0.17867E+01_JPRB, 0.20430E+01_JPRB, 0.19850E+01_JPRB, 0.34100E+01_JPRB ,&
&  0.20212E+01_JPRB, 0.14335E+01_JPRB, 0.19699E+01_JPRB, 0.75554E+00_JPRB, 0.77189E+00_JPRB, 0.68832E+00_JPRB, 0.69765E+00_JPRB ,&
&  0.59177E+00_JPRB, 0.69911E+00_JPRB, 0.10740E+01_JPRB, 0.13544E+01_JPRB, 0.15455E+01_JPRB, 0.15485E+01_JPRB, 0.95232E+00_JPRB ,&
&  0.86010E+00_JPRB, 0.84315E+00_JPRB, 0.83233E+00_JPRB, 0.81527E+00_JPRB, 0.79559E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.72472E+00_JPRB, 0.68538E+00_JPRB, 0.66272E+00_JPRB, 0.64374E+00_JPRB, 0.63314E+00_JPRB, 0.65466E+00_JPRB, 0.67470E+00_JPRB ,&
&  0.70540E+00_JPRB, 0.74092E+00_JPRB, 0.78288E+00_JPRB, 0.82141E+00_JPRB, 0.87057E+00_JPRB, 0.92140E+00_JPRB, 0.96820E+00_JPRB ,&
&  0.10126E+01_JPRB, 0.10559E+01_JPRB, 0.11019E+01_JPRB, 0.11351E+01_JPRB, 0.11686E+01_JPRB, 0.12002E+01_JPRB, 0.12305E+01_JPRB ,&
&  0.12592E+01_JPRB, 0.13426E+01_JPRB, 0.14704E+01_JPRB, 0.16142E+01_JPRB, 0.17452E+01_JPRB, 0.19551E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.17313E+01_JPRB, 0.17620E+01_JPRB, 0.17214E+01_JPRB, 0.16084E+01_JPRB, 0.15012E+01_JPRB, 0.14302E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.14074E+01_JPRB, 0.14826E+01_JPRB, 0.14393E+01_JPRB, 0.14846E+01_JPRB, 0.14142E+01_JPRB, 0.13557E+01_JPRB, 0.15164E+01_JPRB ,&
&  0.16435E+01_JPRB, 0.18544E+01_JPRB, 0.19896E+01_JPRB, 0.20904E+01_JPRB, 0.21661E+01_JPRB, 0.22295E+01_JPRB, 0.22552E+01_JPRB ,&
&  0.22679E+01_JPRB, 0.22107E+01_JPRB, 0.21317E+01_JPRB, 0.22207E+01_JPRB, 0.22703E+01_JPRB, 0.19594E+01_JPRB, 0.18010E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.13068E+01_JPRB, 0.20275E+01_JPRB, 0.25577E+01_JPRB, 0.23145E+01_JPRB, 0.21684E+01_JPRB, 0.19361E+01_JPRB ,&
&  0.17139E+01_JPRB, 0.15991E+01_JPRB, 0.15299E+01_JPRB, 0.14428E+01_JPRB, 0.13283E+01_JPRB, 0.12549E+01_JPRB, 0.12527E+01_JPRB ,&
&  0.12421E+01_JPRB, 0.11305E+01_JPRB, 0.11053E+01_JPRB, 0.12651E+01_JPRB, 0.14064E+01_JPRB, 0.17276E+01_JPRB, 0.18534E+01_JPRB ,&
&  0.21236E+01_JPRB, 0.18884E+01_JPRB, 0.13870E+01_JPRB, 0.12138E+01_JPRB, 0.79171E+00_JPRB, 0.65792E+00_JPRB, 0.61419E+00_JPRB ,&
&  0.62571E+00_JPRB, 0.56474E+00_JPRB, 0.10655E+01_JPRB, 0.38800E+01_JPRB, 0.37959E+01_JPRB, 0.47816E+01_JPRB, 0.25492E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.83633E+00_JPRB, 0.75299E+00_JPRB, 0.75072E+00_JPRB, 0.73690E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.63081E+00_JPRB, 0.60579E+00_JPRB, 0.57890E+00_JPRB, 0.58457E+00_JPRB, 0.59224E+00_JPRB, 0.60319E+00_JPRB, 0.63638E+00_JPRB ,&
&  0.70331E+00_JPRB, 0.72291E+00_JPRB, 0.75633E+00_JPRB, 0.81217E+00_JPRB, 0.89477E+00_JPRB, 0.97297E+00_JPRB, 0.10501E+01_JPRB ,&
&  0.11317E+01_JPRB, 0.12037E+01_JPRB, 0.12563E+01_JPRB, 0.12948E+01_JPRB, 0.13476E+01_JPRB, 0.14055E+01_JPRB, 0.14756E+01_JPRB ,&
&  0.15256E+01_JPRB, 0.16138E+01_JPRB, 0.17613E+01_JPRB, 0.19173E+01_JPRB, 0.20655E+01_JPRB, 0.21277E+01_JPRB, 0.13583E+01_JPRB ,&
&  0.15226E+01_JPRB, 0.16028E+01_JPRB, 0.16520E+01_JPRB, 0.16311E+01_JPRB, 0.15405E+01_JPRB, 0.14712E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.16414E+01_JPRB, 0.17481E+01_JPRB, 0.17137E+01_JPRB, 0.16479E+01_JPRB, 0.18193E+01_JPRB, 0.17253E+01_JPRB, 0.20231E+01_JPRB ,&
&  0.21744E+01_JPRB, 0.22937E+01_JPRB, 0.24554E+01_JPRB, 0.25502E+01_JPRB, 0.26693E+01_JPRB, 0.28118E+01_JPRB, 0.29458E+01_JPRB ,&
&  0.30078E+01_JPRB, 0.29994E+01_JPRB, 0.30861E+01_JPRB, 0.33857E+01_JPRB, 0.31312E+01_JPRB, 0.34890E+01_JPRB, 0.28477E+01_JPRB ,&
&  0.19225E+01_JPRB, 0.16295E+01_JPRB, 0.20207E+01_JPRB, 0.23757E+01_JPRB, 0.22627E+01_JPRB, 0.21573E+01_JPRB, 0.19425E+01_JPRB ,&
&  0.16370E+01_JPRB, 0.14462E+01_JPRB, 0.13693E+01_JPRB, 0.12961E+01_JPRB, 0.12066E+01_JPRB, 0.11289E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10086E+01_JPRB, 0.93301E+00_JPRB, 0.96512E+00_JPRB, 0.99970E+00_JPRB, 0.10964E+01_JPRB, 0.14808E+01_JPRB, 0.22822E+01_JPRB ,&
&  0.35890E+01_JPRB, 0.19927E+01_JPRB, 0.14497E+01_JPRB, 0.11526E+01_JPRB, 0.86133E+00_JPRB, 0.72171E+00_JPRB, 0.64493E+00_JPRB ,&
&  0.61681E+00_JPRB, 0.75256E+00_JPRB, 0.88716E+00_JPRB, 0.15564E+01_JPRB, 0.26693E+01_JPRB, 0.27074E+01_JPRB, 0.18538E+01_JPRB ,&
&  0.11683E+01_JPRB, 0.83398E+00_JPRB, 0.67879E+00_JPRB, 0.65227E+00_JPRB, 0.63262E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.57552E+00_JPRB, 0.56161E+00_JPRB, 0.55854E+00_JPRB, 0.55531E+00_JPRB, 0.59461E+00_JPRB, 0.62181E+00_JPRB, 0.64849E+00_JPRB ,&
&  0.69126E+00_JPRB, 0.72147E+00_JPRB, 0.76290E+00_JPRB, 0.84197E+00_JPRB, 0.93534E+00_JPRB, 0.10235E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11783E+01_JPRB, 0.12213E+01_JPRB, 0.12660E+01_JPRB, 0.13492E+01_JPRB, 0.14297E+01_JPRB, 0.15446E+01_JPRB ,&
&  0.16831E+01_JPRB, 0.18333E+01_JPRB, 0.20085E+01_JPRB, 0.21700E+01_JPRB, 0.23527E+01_JPRB, 0.25252E+01_JPRB, 0.26778E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.15272E+01_JPRB, 0.15905E+01_JPRB, 0.15905E+01_JPRB, 0.15177E+01_JPRB, 0.15444E+01_JPRB, 0.17490E+01_JPRB ,&
&  0.19870E+01_JPRB, 0.20298E+01_JPRB, 0.22131E+01_JPRB, 0.18639E+01_JPRB, 0.21097E+01_JPRB, 0.21711E+01_JPRB, 0.24227E+01_JPRB ,&
&  0.25998E+01_JPRB, 0.27331E+01_JPRB, 0.29066E+01_JPRB, 0.30931E+01_JPRB, 0.32996E+01_JPRB, 0.35280E+01_JPRB, 0.38082E+01_JPRB ,&
&  0.40771E+01_JPRB, 0.44152E+01_JPRB, 0.48560E+01_JPRB, 0.48520E+01_JPRB, 0.45180E+01_JPRB, 0.48505E+01_JPRB, 0.45804E+01_JPRB ,&
&  0.38421E+01_JPRB, 0.27344E+01_JPRB, 0.26158E+01_JPRB, 0.24720E+01_JPRB, 0.22639E+01_JPRB, 0.21481E+01_JPRB, 0.19199E+01_JPRB ,&
&  0.16159E+01_JPRB, 0.14344E+01_JPRB, 0.13082E+01_JPRB, 0.12281E+01_JPRB, 0.11362E+01_JPRB, 0.10641E+01_JPRB, 0.10141E+01_JPRB ,&
&  0.96784E+00_JPRB, 0.97421E+00_JPRB, 0.99059E+00_JPRB, 0.91981E+00_JPRB, 0.89858E+00_JPRB, 0.10290E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.15496E+01_JPRB, 0.13431E+01_JPRB, 0.11296E+01_JPRB, 0.11091E+01_JPRB, 0.99953E+00_JPRB, 0.88091E+00_JPRB, 0.82433E+00_JPRB ,&
&  0.80412E+00_JPRB, 0.77691E+00_JPRB, 0.88217E+00_JPRB, 0.13181E+01_JPRB, 0.16303E+01_JPRB, 0.18396E+01_JPRB, 0.15286E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.84695E+00_JPRB, 0.68769E+00_JPRB, 0.64681E+00_JPRB, 0.62529E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.61593E+00_JPRB, 0.60261E+00_JPRB, 0.60333E+00_JPRB, 0.59991E+00_JPRB, 0.63682E+00_JPRB, 0.67977E+00_JPRB, 0.69374E+00_JPRB ,&
&  0.69515E+00_JPRB, 0.72165E+00_JPRB, 0.74951E+00_JPRB, 0.79663E+00_JPRB, 0.88444E+00_JPRB, 0.94395E+00_JPRB, 0.10123E+01_JPRB ,&
&  0.10802E+01_JPRB, 0.11096E+01_JPRB, 0.11222E+01_JPRB, 0.11685E+01_JPRB, 0.12688E+01_JPRB, 0.13822E+01_JPRB, 0.15255E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.19266E+01_JPRB, 0.22029E+01_JPRB, 0.25070E+01_JPRB, 0.28462E+01_JPRB, 0.31837E+01_JPRB, 0.33802E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.13518E+01_JPRB, 0.15656E+01_JPRB, 0.15459E+01_JPRB, 0.15059E+01_JPRB, 0.16108E+01_JPRB, 0.17849E+01_JPRB ,&
&  0.18653E+01_JPRB, 0.16997E+01_JPRB, 0.17982E+01_JPRB, 0.22330E+01_JPRB, 0.23013E+01_JPRB, 0.24714E+01_JPRB, 0.26815E+01_JPRB ,&
&  0.28467E+01_JPRB, 0.30282E+01_JPRB, 0.31872E+01_JPRB, 0.33274E+01_JPRB, 0.35058E+01_JPRB, 0.37386E+01_JPRB, 0.40513E+01_JPRB ,&
&  0.44519E+01_JPRB, 0.50382E+01_JPRB, 0.56118E+01_JPRB, 0.51860E+01_JPRB, 0.53381E+01_JPRB, 0.56603E+01_JPRB, 0.64158E+01_JPRB ,&
&  0.57151E+01_JPRB, 0.39146E+01_JPRB, 0.29632E+01_JPRB, 0.25338E+01_JPRB, 0.20662E+01_JPRB, 0.18979E+01_JPRB, 0.17009E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.14042E+01_JPRB, 0.12893E+01_JPRB, 0.12012E+01_JPRB, 0.10958E+01_JPRB, 0.10400E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.10706E+01_JPRB, 0.10232E+01_JPRB, 0.95501E+00_JPRB, 0.93753E+00_JPRB, 0.99719E+00_JPRB, 0.11004E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11267E+01_JPRB, 0.11271E+01_JPRB, 0.11382E+01_JPRB, 0.11348E+01_JPRB, 0.11256E+01_JPRB, 0.10855E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.96006E+00_JPRB, 0.94641E+00_JPRB, 0.92630E+00_JPRB, 0.10738E+01_JPRB, 0.13619E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.11951E+01_JPRB, 0.93492E+00_JPRB, 0.76976E+00_JPRB, 0.71886E+00_JPRB, 0.69171E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.73772E+00_JPRB, 0.69189E+00_JPRB, 0.68167E+00_JPRB, 0.66585E+00_JPRB, 0.68290E+00_JPRB, 0.71465E+00_JPRB, 0.73694E+00_JPRB ,&
&  0.74655E+00_JPRB, 0.74821E+00_JPRB, 0.75524E+00_JPRB, 0.78569E+00_JPRB, 0.82438E+00_JPRB, 0.86710E+00_JPRB, 0.92003E+00_JPRB ,&
&  0.95915E+00_JPRB, 0.98488E+00_JPRB, 0.97928E+00_JPRB, 0.97715E+00_JPRB, 0.10395E+01_JPRB, 0.11319E+01_JPRB, 0.12554E+01_JPRB ,&
&  0.14112E+01_JPRB, 0.16017E+01_JPRB, 0.18852E+01_JPRB, 0.23206E+01_JPRB, 0.29705E+01_JPRB, 0.37606E+01_JPRB, 0.45881E+01_JPRB ,&
&  0.37235E+01_JPRB, 0.16674E+01_JPRB, 0.18139E+01_JPRB, 0.18669E+01_JPRB, 0.16794E+01_JPRB, 0.17924E+01_JPRB, 0.18510E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.17539E+01_JPRB, 0.16268E+01_JPRB, 0.17597E+01_JPRB, 0.21466E+01_JPRB, 0.24122E+01_JPRB, 0.26407E+01_JPRB ,&
&  0.28199E+01_JPRB, 0.29529E+01_JPRB, 0.30304E+01_JPRB, 0.31108E+01_JPRB, 0.32613E+01_JPRB, 0.34950E+01_JPRB, 0.38088E+01_JPRB ,&
&  0.42364E+01_JPRB, 0.47874E+01_JPRB, 0.54513E+01_JPRB, 0.50131E+01_JPRB, 0.56705E+01_JPRB, 0.56801E+01_JPRB, 0.57498E+01_JPRB ,&
&  0.52453E+01_JPRB, 0.35276E+01_JPRB, 0.26443E+01_JPRB, 0.23972E+01_JPRB, 0.18846E+01_JPRB, 0.14144E+01_JPRB, 0.14689E+01_JPRB ,&
&  0.14112E+01_JPRB, 0.13498E+01_JPRB, 0.13022E+01_JPRB, 0.11929E+01_JPRB, 0.10948E+01_JPRB, 0.10542E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.12139E+01_JPRB, 0.11344E+01_JPRB, 0.11131E+01_JPRB, 0.10821E+01_JPRB, 0.11181E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.12014E+01_JPRB, 0.12394E+01_JPRB, 0.12632E+01_JPRB, 0.12963E+01_JPRB, 0.13310E+01_JPRB, 0.12680E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11481E+01_JPRB, 0.10802E+01_JPRB, 0.10336E+01_JPRB, 0.98330E+00_JPRB, 0.10018E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.95286E+00_JPRB, 0.83178E+00_JPRB, 0.80279E+00_JPRB, 0.80811E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.86454E+00_JPRB, 0.76001E+00_JPRB, 0.70674E+00_JPRB, 0.70248E+00_JPRB, 0.68951E+00_JPRB, 0.69788E+00_JPRB, 0.70982E+00_JPRB ,&
&  0.70786E+00_JPRB, 0.72964E+00_JPRB, 0.72748E+00_JPRB, 0.74819E+00_JPRB, 0.77135E+00_JPRB, 0.78530E+00_JPRB, 0.82201E+00_JPRB ,&
&  0.85536E+00_JPRB, 0.88925E+00_JPRB, 0.92372E+00_JPRB, 0.90444E+00_JPRB, 0.92848E+00_JPRB, 0.97127E+00_JPRB, 0.10468E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.13510E+01_JPRB, 0.15533E+01_JPRB, 0.17972E+01_JPRB, 0.21520E+01_JPRB, 0.27888E+01_JPRB, 0.39532E+01_JPRB ,&
&  0.68035E+01_JPRB, 0.58159E+01_JPRB, 0.17035E+01_JPRB, 0.21821E+01_JPRB, 0.21062E+01_JPRB, 0.21253E+01_JPRB, 0.18522E+01_JPRB ,&
&  0.19878E+01_JPRB, 0.20879E+01_JPRB, 0.17272E+01_JPRB, 0.15073E+01_JPRB, 0.16489E+01_JPRB, 0.20264E+01_JPRB, 0.23225E+01_JPRB ,&
&  0.25067E+01_JPRB, 0.26450E+01_JPRB, 0.26989E+01_JPRB, 0.27502E+01_JPRB, 0.28313E+01_JPRB, 0.29942E+01_JPRB, 0.32541E+01_JPRB ,&
&  0.36387E+01_JPRB, 0.41840E+01_JPRB, 0.49154E+01_JPRB, 0.48612E+01_JPRB, 0.59834E+01_JPRB, 0.57383E+01_JPRB, 0.53351E+01_JPRB ,&
&  0.43698E+01_JPRB, 0.34938E+01_JPRB, 0.29051E+01_JPRB, 0.23002E+01_JPRB, 0.16499E+01_JPRB, 0.13254E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.11712E+01_JPRB, 0.11349E+01_JPRB, 0.11223E+01_JPRB, 0.10718E+01_JPRB, 0.11458E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.13168E+01_JPRB, 0.12672E+01_JPRB, 0.12516E+01_JPRB, 0.12391E+01_JPRB, 0.12463E+01_JPRB, 0.12800E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.13268E+01_JPRB, 0.13362E+01_JPRB, 0.13666E+01_JPRB, 0.13969E+01_JPRB, 0.13260E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12166E+01_JPRB, 0.12332E+01_JPRB, 0.11976E+01_JPRB, 0.11661E+01_JPRB, 0.10897E+01_JPRB, 0.10291E+01_JPRB, 0.10053E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10064E+01_JPRB, 0.94372E+00_JPRB, 0.87653E+00_JPRB, 0.89910E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.92366E+00_JPRB, 0.86074E+00_JPRB, 0.76182E+00_JPRB, 0.72951E+00_JPRB, 0.72181E+00_JPRB, 0.70540E+00_JPRB, 0.68388E+00_JPRB ,&
&  0.65961E+00_JPRB, 0.65046E+00_JPRB, 0.67990E+00_JPRB, 0.71497E+00_JPRB, 0.74193E+00_JPRB, 0.75168E+00_JPRB, 0.77362E+00_JPRB ,&
&  0.78701E+00_JPRB, 0.81441E+00_JPRB, 0.86482E+00_JPRB, 0.88743E+00_JPRB, 0.89299E+00_JPRB, 0.90714E+00_JPRB, 0.93253E+00_JPRB ,&
&  0.97729E+00_JPRB, 0.10564E+01_JPRB, 0.11931E+01_JPRB, 0.13492E+01_JPRB, 0.15534E+01_JPRB, 0.19163E+01_JPRB, 0.26729E+01_JPRB ,&
&  0.41857E+01_JPRB, 0.77798E+01_JPRB, 0.18219E+01_JPRB, 0.25728E+01_JPRB, 0.25579E+01_JPRB, 0.23932E+01_JPRB, 0.22320E+01_JPRB ,&
&  0.22444E+01_JPRB, 0.22046E+01_JPRB, 0.19732E+01_JPRB, 0.15881E+01_JPRB, 0.12889E+01_JPRB, 0.14569E+01_JPRB, 0.17874E+01_JPRB ,&
&  0.20858E+01_JPRB, 0.22327E+01_JPRB, 0.22879E+01_JPRB, 0.22735E+01_JPRB, 0.22982E+01_JPRB, 0.23641E+01_JPRB, 0.25106E+01_JPRB ,&
&  0.27805E+01_JPRB, 0.32167E+01_JPRB, 0.38666E+01_JPRB, 0.41412E+01_JPRB, 0.54981E+01_JPRB, 0.58453E+01_JPRB, 0.55807E+01_JPRB ,&
&  0.44876E+01_JPRB, 0.40097E+01_JPRB, 0.35011E+01_JPRB, 0.25332E+01_JPRB, 0.18000E+01_JPRB, 0.14298E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.11652E+01_JPRB, 0.11444E+01_JPRB, 0.11192E+01_JPRB, 0.11376E+01_JPRB, 0.12228E+01_JPRB, 0.13187E+01_JPRB ,&
&  0.13485E+01_JPRB, 0.13491E+01_JPRB, 0.13096E+01_JPRB, 0.13020E+01_JPRB, 0.13049E+01_JPRB, 0.13115E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.13677E+01_JPRB, 0.13850E+01_JPRB, 0.13856E+01_JPRB, 0.13536E+01_JPRB, 0.13666E+01_JPRB, 0.13242E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.13110E+01_JPRB, 0.12750E+01_JPRB, 0.12195E+01_JPRB, 0.12266E+01_JPRB, 0.11754E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.11449E+01_JPRB, 0.11051E+01_JPRB, 0.97200E+00_JPRB, 0.97477E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.98150E+00_JPRB, 0.92723E+00_JPRB, 0.85446E+00_JPRB, 0.77935E+00_JPRB, 0.77468E+00_JPRB, 0.77826E+00_JPRB, 0.73844E+00_JPRB ,&
&  0.67994E+00_JPRB, 0.67393E+00_JPRB, 0.68118E+00_JPRB, 0.70579E+00_JPRB, 0.72389E+00_JPRB, 0.73725E+00_JPRB, 0.77769E+00_JPRB ,&
&  0.77618E+00_JPRB, 0.77302E+00_JPRB, 0.79868E+00_JPRB, 0.83977E+00_JPRB, 0.85751E+00_JPRB, 0.87173E+00_JPRB, 0.89889E+00_JPRB ,&
&  0.91219E+00_JPRB, 0.90931E+00_JPRB, 0.94351E+00_JPRB, 0.10180E+01_JPRB, 0.11529E+01_JPRB, 0.13948E+01_JPRB, 0.18757E+01_JPRB ,&
&  0.32672E+01_JPRB, 0.80823E+01_JPRB, 0.46661E+01_JPRB, 0.45501E+01_JPRB, 0.33727E+01_JPRB, 0.27571E+01_JPRB, 0.25245E+01_JPRB ,&
&  0.27686E+01_JPRB, 0.25705E+01_JPRB, 0.22946E+01_JPRB, 0.16958E+01_JPRB, 0.13459E+01_JPRB, 0.12060E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.15484E+01_JPRB, 0.17161E+01_JPRB, 0.18341E+01_JPRB, 0.18412E+01_JPRB, 0.18498E+01_JPRB, 0.18981E+01_JPRB, 0.19615E+01_JPRB ,&
&  0.21018E+01_JPRB, 0.23906E+01_JPRB, 0.28941E+01_JPRB, 0.35432E+01_JPRB, 0.43351E+01_JPRB, 0.51673E+01_JPRB, 0.59820E+01_JPRB ,&
&  0.56903E+01_JPRB, 0.49718E+01_JPRB, 0.40501E+01_JPRB, 0.28045E+01_JPRB, 0.19124E+01_JPRB, 0.17399E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.12141E+01_JPRB, 0.12018E+01_JPRB, 0.12215E+01_JPRB, 0.12572E+01_JPRB, 0.13227E+01_JPRB, 0.13758E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.13694E+01_JPRB, 0.13501E+01_JPRB, 0.13415E+01_JPRB, 0.13366E+01_JPRB, 0.13362E+01_JPRB, 0.13467E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.13871E+01_JPRB, 0.13384E+01_JPRB, 0.13260E+01_JPRB, 0.13279E+01_JPRB, 0.13057E+01_JPRB, 0.12431E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.13274E+01_JPRB, 0.13379E+01_JPRB, 0.13146E+01_JPRB, 0.13037E+01_JPRB, 0.13052E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.12458E+01_JPRB, 0.12181E+01_JPRB, 0.11226E+01_JPRB, 0.10672E+01_JPRB, 0.10486E+01_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.10442E+01_JPRB, 0.99989E+00_JPRB, 0.94265E+00_JPRB, 0.87139E+00_JPRB, 0.83366E+00_JPRB, 0.83294E+00_JPRB, 0.83773E+00_JPRB ,&
&  0.80092E+00_JPRB, 0.77381E+00_JPRB, 0.76181E+00_JPRB, 0.77609E+00_JPRB, 0.80335E+00_JPRB, 0.81891E+00_JPRB, 0.83028E+00_JPRB ,&
&  0.82440E+00_JPRB, 0.82040E+00_JPRB, 0.80541E+00_JPRB, 0.80867E+00_JPRB, 0.86203E+00_JPRB, 0.87717E+00_JPRB, 0.87514E+00_JPRB ,&
&  0.89553E+00_JPRB, 0.88537E+00_JPRB, 0.87121E+00_JPRB, 0.88805E+00_JPRB, 0.95058E+00_JPRB, 0.11158E+01_JPRB, 0.14959E+01_JPRB ,&
&  0.29710E+01_JPRB, 0.73558E+01_JPRB, 0.31079E+01_JPRB, 0.53263E+01_JPRB, 0.37414E+01_JPRB, 0.30583E+01_JPRB, 0.24469E+01_JPRB ,&
&  0.24719E+01_JPRB, 0.28481E+01_JPRB, 0.22173E+01_JPRB, 0.17929E+01_JPRB, 0.14674E+01_JPRB, 0.12918E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.14274E+01_JPRB, 0.14693E+01_JPRB, 0.15021E+01_JPRB, 0.15269E+01_JPRB, 0.15028E+01_JPRB, 0.15116E+01_JPRB, 0.15563E+01_JPRB ,&
&  0.16160E+01_JPRB, 0.17669E+01_JPRB, 0.20884E+01_JPRB, 0.25856E+01_JPRB, 0.37252E+01_JPRB, 0.49924E+01_JPRB, 0.63680E+01_JPRB ,&
&  0.63953E+01_JPRB, 0.62648E+01_JPRB, 0.46981E+01_JPRB, 0.31267E+01_JPRB, 0.21710E+01_JPRB, 0.18437E+01_JPRB, 0.15475E+01_JPRB ,&
&  0.14034E+01_JPRB, 0.13365E+01_JPRB, 0.12735E+01_JPRB, 0.12717E+01_JPRB, 0.13593E+01_JPRB, 0.14346E+01_JPRB, 0.14310E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.13923E+01_JPRB, 0.13694E+01_JPRB, 0.13495E+01_JPRB, 0.13573E+01_JPRB, 0.13581E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.13417E+01_JPRB, 0.13588E+01_JPRB, 0.13233E+01_JPRB, 0.13263E+01_JPRB, 0.13271E+01_JPRB, 0.13113E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.13165E+01_JPRB, 0.13238E+01_JPRB, 0.13075E+01_JPRB, 0.13045E+01_JPRB, 0.13609E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.12992E+01_JPRB, 0.12601E+01_JPRB, 0.11837E+01_JPRB, 0.11385E+01_JPRB, 0.11108E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.11053E+01_JPRB, 0.10483E+01_JPRB, 0.10035E+01_JPRB, 0.94607E+00_JPRB, 0.90390E+00_JPRB, 0.88151E+00_JPRB, 0.87921E+00_JPRB ,&
&  0.86387E+00_JPRB, 0.83129E+00_JPRB, 0.81100E+00_JPRB, 0.81106E+00_JPRB, 0.81379E+00_JPRB, 0.82747E+00_JPRB, 0.83842E+00_JPRB ,&
&  0.83695E+00_JPRB, 0.82734E+00_JPRB, 0.82319E+00_JPRB, 0.82231E+00_JPRB, 0.84792E+00_JPRB, 0.89425E+00_JPRB, 0.91416E+00_JPRB ,&
&  0.90358E+00_JPRB, 0.89024E+00_JPRB, 0.86770E+00_JPRB, 0.85442E+00_JPRB, 0.87316E+00_JPRB, 0.98341E+00_JPRB, 0.13137E+01_JPRB ,&
&  0.28377E+01_JPRB, 0.87296E+01_JPRB, 0.33640E+01_JPRB, 0.58957E+01_JPRB, 0.40422E+01_JPRB, 0.32128E+01_JPRB, 0.25129E+01_JPRB ,&
&  0.24611E+01_JPRB, 0.26665E+01_JPRB, 0.22648E+01_JPRB, 0.19269E+01_JPRB, 0.16639E+01_JPRB, 0.14787E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.14416E+01_JPRB, 0.14390E+01_JPRB, 0.14072E+01_JPRB, 0.13965E+01_JPRB, 0.13625E+01_JPRB, 0.13474E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.13806E+01_JPRB, 0.15297E+01_JPRB, 0.21381E+01_JPRB, 0.30251E+01_JPRB, 0.47718E+01_JPRB, 0.72954E+01_JPRB ,&
&  0.70756E+01_JPRB, 0.73429E+01_JPRB, 0.52600E+01_JPRB, 0.34996E+01_JPRB, 0.23832E+01_JPRB, 0.19386E+01_JPRB, 0.17430E+01_JPRB ,&
&  0.15715E+01_JPRB, 0.14489E+01_JPRB, 0.13613E+01_JPRB, 0.13379E+01_JPRB, 0.13943E+01_JPRB, 0.14291E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.14521E+01_JPRB, 0.13875E+01_JPRB, 0.13528E+01_JPRB, 0.13402E+01_JPRB, 0.13541E+01_JPRB, 0.13576E+01_JPRB, 0.13470E+01_JPRB ,&
&  0.13376E+01_JPRB, 0.13344E+01_JPRB, 0.13124E+01_JPRB, 0.13257E+01_JPRB, 0.13385E+01_JPRB, 0.13263E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.13056E+01_JPRB, 0.12984E+01_JPRB, 0.13144E+01_JPRB, 0.13191E+01_JPRB, 0.13975E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.13042E+01_JPRB, 0.12726E+01_JPRB, 0.12148E+01_JPRB, 0.11796E+01_JPRB, 0.11578E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.11519E+01_JPRB, 0.11056E+01_JPRB, 0.10696E+01_JPRB, 0.10248E+01_JPRB, 0.96723E+00_JPRB, 0.93771E+00_JPRB, 0.91746E+00_JPRB ,&
&  0.90433E+00_JPRB, 0.87265E+00_JPRB, 0.84882E+00_JPRB, 0.84512E+00_JPRB, 0.84963E+00_JPRB, 0.84621E+00_JPRB, 0.84858E+00_JPRB ,&
&  0.85835E+00_JPRB, 0.86340E+00_JPRB, 0.85407E+00_JPRB, 0.84191E+00_JPRB, 0.83731E+00_JPRB, 0.87316E+00_JPRB, 0.89723E+00_JPRB ,&
&  0.90795E+00_JPRB, 0.89232E+00_JPRB, 0.86500E+00_JPRB, 0.83600E+00_JPRB, 0.83466E+00_JPRB, 0.89154E+00_JPRB, 0.11488E+01_JPRB ,&
&  0.24205E+01_JPRB, 0.33829E+01_JPRB, 0.61104E+01_JPRB, 0.62997E+01_JPRB, 0.43116E+01_JPRB, 0.33858E+01_JPRB, 0.29057E+01_JPRB ,&
&  0.27777E+01_JPRB, 0.26006E+01_JPRB, 0.22686E+01_JPRB, 0.20185E+01_JPRB, 0.18080E+01_JPRB, 0.16318E+01_JPRB, 0.14939E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.13939E+01_JPRB, 0.13823E+01_JPRB, 0.13692E+01_JPRB, 0.13554E+01_JPRB, 0.13477E+01_JPRB, 0.13481E+01_JPRB ,&
&  0.13463E+01_JPRB, 0.13358E+01_JPRB, 0.13562E+01_JPRB, 0.16036E+01_JPRB, 0.26388E+01_JPRB, 0.42525E+01_JPRB, 0.65446E+01_JPRB ,&
&  0.75318E+01_JPRB, 0.70533E+01_JPRB, 0.49802E+01_JPRB, 0.35823E+01_JPRB, 0.25226E+01_JPRB, 0.19656E+01_JPRB, 0.17868E+01_JPRB ,&
&  0.16963E+01_JPRB, 0.16064E+01_JPRB, 0.14831E+01_JPRB, 0.14279E+01_JPRB, 0.14147E+01_JPRB, 0.14082E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.13949E+01_JPRB, 0.13735E+01_JPRB, 0.13548E+01_JPRB, 0.13394E+01_JPRB, 0.13185E+01_JPRB, 0.13038E+01_JPRB ,&
&  0.12955E+01_JPRB, 0.12821E+01_JPRB, 0.12758E+01_JPRB, 0.13019E+01_JPRB, 0.13620E+01_JPRB, 0.13702E+01_JPRB, 0.13451E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.12859E+01_JPRB, 0.12796E+01_JPRB, 0.13133E+01_JPRB, 0.12969E+01_JPRB, 0.14076E+01_JPRB, 0.13488E+01_JPRB ,&
&  0.13199E+01_JPRB, 0.12911E+01_JPRB, 0.12457E+01_JPRB, 0.12005E+01_JPRB, 0.11687E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.11634E+01_JPRB, 0.11131E+01_JPRB, 0.10761E+01_JPRB, 0.10502E+01_JPRB, 0.10123E+01_JPRB, 0.97453E+00_JPRB, 0.96323E+00_JPRB ,&
&  0.95847E+00_JPRB, 0.93614E+00_JPRB, 0.90843E+00_JPRB, 0.87389E+00_JPRB, 0.86282E+00_JPRB, 0.86734E+00_JPRB, 0.87444E+00_JPRB ,&
&  0.87644E+00_JPRB, 0.87963E+00_JPRB, 0.87902E+00_JPRB, 0.87546E+00_JPRB, 0.87046E+00_JPRB, 0.89014E+00_JPRB, 0.90096E+00_JPRB ,&
&  0.89568E+00_JPRB, 0.89556E+00_JPRB, 0.87189E+00_JPRB, 0.84743E+00_JPRB, 0.83788E+00_JPRB, 0.83714E+00_JPRB, 0.90167E+00_JPRB ,&
&  0.17751E+01_JPRB, 0.42654E+01_JPRB, 0.61317E+01_JPRB, 0.55672E+01_JPRB, 0.42033E+01_JPRB, 0.32350E+01_JPRB, 0.29595E+01_JPRB ,&
&  0.27309E+01_JPRB, 0.25245E+01_JPRB, 0.22533E+01_JPRB, 0.20415E+01_JPRB, 0.18730E+01_JPRB, 0.17263E+01_JPRB, 0.15758E+01_JPRB ,&
&  0.14820E+01_JPRB, 0.14394E+01_JPRB, 0.14085E+01_JPRB, 0.13946E+01_JPRB, 0.13823E+01_JPRB, 0.13881E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.13979E+01_JPRB, 0.13905E+01_JPRB, 0.13809E+01_JPRB, 0.14539E+01_JPRB, 0.20390E+01_JPRB, 0.24651E+01_JPRB, 0.43835E+01_JPRB ,&
&  0.61040E+01_JPRB, 0.49721E+01_JPRB, 0.38071E+01_JPRB, 0.31180E+01_JPRB, 0.25181E+01_JPRB, 0.20021E+01_JPRB, 0.17530E+01_JPRB ,&
&  0.16539E+01_JPRB, 0.15873E+01_JPRB, 0.15011E+01_JPRB, 0.14392E+01_JPRB, 0.13902E+01_JPRB, 0.13697E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.13354E+01_JPRB, 0.13340E+01_JPRB, 0.13286E+01_JPRB, 0.13237E+01_JPRB, 0.13042E+01_JPRB, 0.12827E+01_JPRB, 0.12631E+01_JPRB ,&
&  0.12483E+01_JPRB, 0.12271E+01_JPRB, 0.12229E+01_JPRB, 0.12455E+01_JPRB, 0.13338E+01_JPRB, 0.14013E+01_JPRB, 0.14003E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.13011E+01_JPRB, 0.13013E+01_JPRB, 0.13130E+01_JPRB, 0.13473E+01_JPRB, 0.14711E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.13151E+01_JPRB, 0.12787E+01_JPRB, 0.12342E+01_JPRB, 0.12020E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.12069E+01_JPRB, 0.11512E+01_JPRB, 0.11022E+01_JPRB, 0.10590E+01_JPRB, 0.10422E+01_JPRB, 0.10155E+01_JPRB, 0.98544E+00_JPRB ,&
&  0.96973E+00_JPRB, 0.96770E+00_JPRB, 0.95838E+00_JPRB, 0.93740E+00_JPRB, 0.91830E+00_JPRB, 0.91022E+00_JPRB, 0.90929E+00_JPRB ,&
&  0.90831E+00_JPRB, 0.90824E+00_JPRB, 0.90629E+00_JPRB, 0.89752E+00_JPRB, 0.89861E+00_JPRB, 0.90635E+00_JPRB, 0.91482E+00_JPRB ,&
&  0.89687E+00_JPRB, 0.88105E+00_JPRB, 0.86871E+00_JPRB, 0.85548E+00_JPRB, 0.85158E+00_JPRB, 0.83579E+00_JPRB, 0.82910E+00_JPRB ,&
&  0.12177E+01_JPRB, 0.27020E+01_JPRB, 0.38911E+01_JPRB, 0.48381E+01_JPRB, 0.41260E+01_JPRB, 0.33335E+01_JPRB, 0.29788E+01_JPRB ,&
&  0.26969E+01_JPRB, 0.23978E+01_JPRB, 0.21439E+01_JPRB, 0.19824E+01_JPRB, 0.18780E+01_JPRB, 0.17543E+01_JPRB, 0.16061E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.14280E+01_JPRB, 0.14110E+01_JPRB, 0.14135E+01_JPRB, 0.14118E+01_JPRB, 0.14217E+01_JPRB, 0.14252E+01_JPRB ,&
&  0.14166E+01_JPRB, 0.13980E+01_JPRB, 0.13679E+01_JPRB, 0.13519E+01_JPRB, 0.15525E+01_JPRB, 0.17394E+01_JPRB, 0.23706E+01_JPRB ,&
&  0.26795E+01_JPRB, 0.28185E+01_JPRB, 0.26841E+01_JPRB, 0.25591E+01_JPRB, 0.23499E+01_JPRB, 0.19992E+01_JPRB, 0.17223E+01_JPRB ,&
&  0.15688E+01_JPRB, 0.15022E+01_JPRB, 0.14401E+01_JPRB, 0.14065E+01_JPRB, 0.13715E+01_JPRB, 0.13427E+01_JPRB, 0.13295E+01_JPRB ,&
&  0.13165E+01_JPRB, 0.13025E+01_JPRB, 0.12878E+01_JPRB, 0.12873E+01_JPRB, 0.12675E+01_JPRB, 0.12306E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11654E+01_JPRB, 0.11755E+01_JPRB, 0.12149E+01_JPRB, 0.12596E+01_JPRB, 0.12933E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.13553E+01_JPRB, 0.13577E+01_JPRB, 0.13293E+01_JPRB, 0.13525E+01_JPRB, 0.13918E+01_JPRB, 0.14928E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.13672E+01_JPRB, 0.13338E+01_JPRB, 0.12989E+01_JPRB, 0.12413E+01_JPRB, 0.12542E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.12060E+01_JPRB, 0.11739E+01_JPRB, 0.11363E+01_JPRB, 0.10876E+01_JPRB, 0.10655E+01_JPRB, 0.10457E+01_JPRB, 0.10248E+01_JPRB ,&
&  0.10137E+01_JPRB, 0.10034E+01_JPRB, 0.99536E+00_JPRB, 0.97822E+00_JPRB, 0.96031E+00_JPRB, 0.94852E+00_JPRB, 0.93539E+00_JPRB ,&
&  0.93502E+00_JPRB, 0.93489E+00_JPRB, 0.92682E+00_JPRB, 0.91780E+00_JPRB, 0.91274E+00_JPRB, 0.90839E+00_JPRB, 0.91150E+00_JPRB ,&
&  0.90345E+00_JPRB, 0.89195E+00_JPRB, 0.89062E+00_JPRB, 0.88165E+00_JPRB, 0.86676E+00_JPRB, 0.85482E+00_JPRB, 0.82581E+00_JPRB ,&
&  0.92615E+00_JPRB, 0.14517E+01_JPRB, 0.23390E+01_JPRB, 0.33381E+01_JPRB, 0.32646E+01_JPRB, 0.30709E+01_JPRB, 0.27445E+01_JPRB ,&
&  0.24410E+01_JPRB, 0.22112E+01_JPRB, 0.19572E+01_JPRB, 0.17819E+01_JPRB, 0.17133E+01_JPRB, 0.16458E+01_JPRB, 0.15742E+01_JPRB ,&
&  0.14983E+01_JPRB, 0.14332E+01_JPRB, 0.14001E+01_JPRB, 0.14035E+01_JPRB, 0.13941E+01_JPRB, 0.13852E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.13494E+01_JPRB, 0.13222E+01_JPRB, 0.12821E+01_JPRB, 0.12427E+01_JPRB, 0.12110E+01_JPRB, 0.12104E+01_JPRB, 0.13575E+01_JPRB ,&
&  0.15856E+01_JPRB, 0.16962E+01_JPRB, 0.17549E+01_JPRB, 0.18303E+01_JPRB, 0.18702E+01_JPRB, 0.18191E+01_JPRB, 0.16733E+01_JPRB ,&
&  0.15522E+01_JPRB, 0.14753E+01_JPRB, 0.13994E+01_JPRB, 0.13929E+01_JPRB, 0.13669E+01_JPRB, 0.13326E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.12481E+01_JPRB, 0.12219E+01_JPRB, 0.12027E+01_JPRB, 0.11825E+01_JPRB, 0.11549E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.10967E+01_JPRB, 0.11046E+01_JPRB, 0.11374E+01_JPRB, 0.11815E+01_JPRB, 0.12146E+01_JPRB, 0.12498E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.13145E+01_JPRB, 0.12897E+01_JPRB, 0.13191E+01_JPRB, 0.14514E+01_JPRB, 0.14507E+01_JPRB, 0.13953E+01_JPRB ,&
&  0.13519E+01_JPRB, 0.13125E+01_JPRB, 0.12857E+01_JPRB, 0.12180E+01_JPRB, 0.11941E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.10784E+01_JPRB, 0.10956E+01_JPRB, 0.10796E+01_JPRB, 0.10443E+01_JPRB, 0.10245E+01_JPRB, 0.10165E+01_JPRB, 0.10149E+01_JPRB ,&
&  0.10107E+01_JPRB, 0.99977E+00_JPRB, 0.10036E+01_JPRB, 0.10079E+01_JPRB, 0.10021E+01_JPRB, 0.98645E+00_JPRB, 0.97452E+00_JPRB ,&
&  0.96975E+00_JPRB, 0.97197E+00_JPRB, 0.97590E+00_JPRB, 0.96481E+00_JPRB, 0.94206E+00_JPRB, 0.92606E+00_JPRB, 0.91260E+00_JPRB ,&
&  0.91134E+00_JPRB, 0.91156E+00_JPRB, 0.90918E+00_JPRB, 0.90954E+00_JPRB, 0.88603E+00_JPRB, 0.85927E+00_JPRB, 0.83774E+00_JPRB ,&
&  0.84243E+00_JPRB, 0.90747E+00_JPRB, 0.13471E+01_JPRB, 0.19472E+01_JPRB, 0.22268E+01_JPRB, 0.21794E+01_JPRB, 0.19396E+01_JPRB ,&
&  0.17847E+01_JPRB, 0.16922E+01_JPRB, 0.16202E+01_JPRB, 0.15293E+01_JPRB, 0.14665E+01_JPRB, 0.14246E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.13571E+01_JPRB, 0.13201E+01_JPRB, 0.12902E+01_JPRB, 0.12562E+01_JPRB, 0.12283E+01_JPRB, 0.12148E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11733E+01_JPRB, 0.11577E+01_JPRB, 0.11394E+01_JPRB, 0.10983E+01_JPRB, 0.10768E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.11436E+01_JPRB, 0.11977E+01_JPRB, 0.12338E+01_JPRB, 0.13246E+01_JPRB, 0.14080E+01_JPRB, 0.14791E+01_JPRB, 0.14765E+01_JPRB ,&
&  0.14378E+01_JPRB, 0.14048E+01_JPRB, 0.13660E+01_JPRB, 0.13494E+01_JPRB, 0.13262E+01_JPRB, 0.12787E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.11972E+01_JPRB, 0.11737E+01_JPRB, 0.11426E+01_JPRB, 0.11170E+01_JPRB, 0.10888E+01_JPRB, 0.10541E+01_JPRB, 0.10235E+01_JPRB ,&
&  0.10018E+01_JPRB, 0.99475E+00_JPRB, 0.99926E+00_JPRB, 0.10214E+01_JPRB, 0.10553E+01_JPRB, 0.10965E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.11769E+01_JPRB, 0.12081E+01_JPRB, 0.12442E+01_JPRB, 0.13139E+01_JPRB, 0.13264E+01_JPRB, 0.13255E+01_JPRB, 0.13305E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.12596E+01_JPRB, 0.12087E+01_JPRB, 0.11297E+01_JPRB, 0.10307E+01_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.10213E+01_JPRB, 0.10319E+01_JPRB, 0.10259E+01_JPRB, 0.10114E+01_JPRB, 0.99992E+00_JPRB, 0.99279E+00_JPRB, 0.99547E+00_JPRB ,&
&  0.99398E+00_JPRB, 0.98829E+00_JPRB, 0.99037E+00_JPRB, 0.99079E+00_JPRB, 0.99023E+00_JPRB, 0.98442E+00_JPRB, 0.98621E+00_JPRB ,&
&  0.98457E+00_JPRB, 0.98447E+00_JPRB, 0.98646E+00_JPRB, 0.97908E+00_JPRB, 0.95804E+00_JPRB, 0.94279E+00_JPRB, 0.92797E+00_JPRB ,&
&  0.92669E+00_JPRB, 0.92617E+00_JPRB, 0.92648E+00_JPRB, 0.91929E+00_JPRB, 0.89692E+00_JPRB, 0.87667E+00_JPRB, 0.86347E+00_JPRB ,&
&  0.86499E+00_JPRB, 0.76638E+00_JPRB, 0.95565E+00_JPRB, 0.13817E+01_JPRB, 0.15967E+01_JPRB, 0.14808E+01_JPRB, 0.14176E+01_JPRB ,&
&  0.13963E+01_JPRB, 0.13842E+01_JPRB, 0.13642E+01_JPRB, 0.13205E+01_JPRB, 0.12446E+01_JPRB, 0.12078E+01_JPRB, 0.11670E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.11290E+01_JPRB, 0.11149E+01_JPRB, 0.11057E+01_JPRB, 0.10838E+01_JPRB, 0.10623E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10490E+01_JPRB, 0.10473E+01_JPRB, 0.10430E+01_JPRB, 0.10265E+01_JPRB, 0.10304E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10400E+01_JPRB, 0.10491E+01_JPRB, 0.10569E+01_JPRB, 0.10801E+01_JPRB, 0.11269E+01_JPRB, 0.12127E+01_JPRB, 0.12668E+01_JPRB ,&
&  0.12688E+01_JPRB, 0.12558E+01_JPRB, 0.12563E+01_JPRB, 0.12540E+01_JPRB, 0.12385E+01_JPRB, 0.12025E+01_JPRB, 0.11618E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.11014E+01_JPRB, 0.10724E+01_JPRB, 0.10413E+01_JPRB, 0.10094E+01_JPRB, 0.97942E+00_JPRB, 0.95441E+00_JPRB ,&
&  0.93774E+00_JPRB, 0.93076E+00_JPRB, 0.93903E+00_JPRB, 0.96168E+00_JPRB, 0.99252E+00_JPRB, 0.10284E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.11093E+01_JPRB, 0.11531E+01_JPRB, 0.11944E+01_JPRB, 0.11746E+01_JPRB, 0.12478E+01_JPRB, 0.12622E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.11996E+01_JPRB, 0.11245E+01_JPRB, 0.90581E+00_JPRB, 0.98709E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.99258E+00_JPRB, 0.99515E+00_JPRB, 0.99586E+00_JPRB, 0.98157E+00_JPRB, 0.96372E+00_JPRB, 0.95613E+00_JPRB, 0.96176E+00_JPRB ,&
&  0.95640E+00_JPRB, 0.94243E+00_JPRB, 0.94659E+00_JPRB, 0.94864E+00_JPRB, 0.95376E+00_JPRB, 0.95484E+00_JPRB, 0.95610E+00_JPRB ,&
&  0.95764E+00_JPRB, 0.95978E+00_JPRB, 0.95713E+00_JPRB, 0.96496E+00_JPRB, 0.95721E+00_JPRB, 0.95131E+00_JPRB, 0.94811E+00_JPRB ,&
&  0.93954E+00_JPRB, 0.93083E+00_JPRB, 0.92318E+00_JPRB, 0.91209E+00_JPRB, 0.90251E+00_JPRB, 0.89060E+00_JPRB, 0.87310E+00_JPRB ,&
&  0.88397E+00_JPRB, 0.75336E+00_JPRB, 0.90243E+00_JPRB, 0.11708E+01_JPRB, 0.11682E+01_JPRB, 0.11219E+01_JPRB, 0.11343E+01_JPRB ,&
&  0.11591E+01_JPRB, 0.11666E+01_JPRB, 0.11600E+01_JPRB, 0.11274E+01_JPRB, 0.10562E+01_JPRB, 0.10238E+01_JPRB, 0.10081E+01_JPRB ,&
&  0.99072E+00_JPRB, 0.97694E+00_JPRB, 0.96710E+00_JPRB, 0.96741E+00_JPRB, 0.96791E+00_JPRB, 0.96446E+00_JPRB, 0.95806E+00_JPRB ,&
&  0.95777E+00_JPRB, 0.96285E+00_JPRB, 0.96202E+00_JPRB, 0.96107E+00_JPRB, 0.95991E+00_JPRB, 0.94804E+00_JPRB, 0.93844E+00_JPRB ,&
&  0.93009E+00_JPRB, 0.92723E+00_JPRB, 0.92503E+00_JPRB, 0.93400E+00_JPRB, 0.95618E+00_JPRB, 0.10120E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.10935E+01_JPRB, 0.10979E+01_JPRB, 0.11010E+01_JPRB, 0.11056E+01_JPRB, 0.11003E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10242E+01_JPRB, 0.10004E+01_JPRB, 0.98110E+00_JPRB, 0.96011E+00_JPRB, 0.94402E+00_JPRB, 0.93641E+00_JPRB ,&
&  0.92518E+00_JPRB, 0.91114E+00_JPRB, 0.91345E+00_JPRB, 0.92976E+00_JPRB, 0.94715E+00_JPRB, 0.96240E+00_JPRB, 0.98860E+00_JPRB ,&
&  0.10309E+01_JPRB, 0.10744E+01_JPRB, 0.11088E+01_JPRB, 0.11500E+01_JPRB, 0.11697E+01_JPRB, 0.11779E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.11514E+01_JPRB, 0.11271E+01_JPRB, 0.85281E+00_JPRB, 0.96006E+00_JPRB, 0.98348E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.98333E+00_JPRB, 0.97280E+00_JPRB, 0.96010E+00_JPRB, 0.94226E+00_JPRB, 0.92420E+00_JPRB, 0.91268E+00_JPRB, 0.91687E+00_JPRB ,&
&  0.91427E+00_JPRB, 0.90128E+00_JPRB, 0.91114E+00_JPRB, 0.92356E+00_JPRB, 0.93303E+00_JPRB, 0.94123E+00_JPRB, 0.94266E+00_JPRB ,&
&  0.94445E+00_JPRB, 0.94016E+00_JPRB, 0.94280E+00_JPRB, 0.95503E+00_JPRB, 0.97411E+00_JPRB, 0.97605E+00_JPRB, 0.97305E+00_JPRB ,&
&  0.95081E+00_JPRB, 0.93949E+00_JPRB, 0.92897E+00_JPRB, 0.91600E+00_JPRB, 0.90356E+00_JPRB, 0.89761E+00_JPRB, 0.89764E+00_JPRB ,&
&  0.78333E+00_JPRB, 0.70266E+00_JPRB, 0.86639E+00_JPRB, 0.98853E+00_JPRB, 0.96871E+00_JPRB, 0.94502E+00_JPRB, 0.96285E+00_JPRB ,&
&  0.96947E+00_JPRB, 0.96264E+00_JPRB, 0.96549E+00_JPRB, 0.96759E+00_JPRB, 0.93969E+00_JPRB, 0.90275E+00_JPRB, 0.88695E+00_JPRB ,&
&  0.87847E+00_JPRB, 0.88444E+00_JPRB, 0.88763E+00_JPRB, 0.89607E+00_JPRB, 0.89656E+00_JPRB, 0.88876E+00_JPRB, 0.88486E+00_JPRB ,&
&  0.88202E+00_JPRB, 0.88798E+00_JPRB, 0.89248E+00_JPRB, 0.89394E+00_JPRB, 0.89171E+00_JPRB, 0.88826E+00_JPRB, 0.88033E+00_JPRB ,&
&  0.87296E+00_JPRB, 0.86725E+00_JPRB, 0.86210E+00_JPRB, 0.86548E+00_JPRB, 0.87807E+00_JPRB, 0.90535E+00_JPRB, 0.94600E+00_JPRB ,&
&  0.97285E+00_JPRB, 0.98443E+00_JPRB, 0.99131E+00_JPRB, 0.99156E+00_JPRB, 0.10014E+01_JPRB, 0.10050E+01_JPRB, 0.99074E+00_JPRB ,&
&  0.97132E+00_JPRB, 0.95504E+00_JPRB, 0.94874E+00_JPRB, 0.94135E+00_JPRB, 0.93620E+00_JPRB, 0.92941E+00_JPRB, 0.91890E+00_JPRB ,&
&  0.90763E+00_JPRB, 0.89286E+00_JPRB, 0.88210E+00_JPRB, 0.88295E+00_JPRB, 0.89597E+00_JPRB, 0.90786E+00_JPRB, 0.93119E+00_JPRB ,&
&  0.96977E+00_JPRB, 0.10149E+01_JPRB, 0.10515E+01_JPRB, 0.10917E+01_JPRB, 0.11156E+01_JPRB, 0.11257E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.10564E+01_JPRB, 0.10386E+01_JPRB, 0.10111E+01_JPRB, 0.99580E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.93102E+00_JPRB, 0.92356E+00_JPRB, 0.90879E+00_JPRB, 0.88220E+00_JPRB, 0.84786E+00_JPRB, 0.85513E+00_JPRB, 0.86510E+00_JPRB ,&
&  0.85735E+00_JPRB, 0.84717E+00_JPRB, 0.85769E+00_JPRB, 0.88292E+00_JPRB, 0.90696E+00_JPRB, 0.92015E+00_JPRB, 0.92117E+00_JPRB ,&
&  0.91482E+00_JPRB, 0.91567E+00_JPRB, 0.92349E+00_JPRB, 0.94090E+00_JPRB, 0.94054E+00_JPRB, 0.92298E+00_JPRB, 0.91219E+00_JPRB ,&
&  0.90949E+00_JPRB, 0.90766E+00_JPRB, 0.90505E+00_JPRB, 0.88802E+00_JPRB, 0.87673E+00_JPRB, 0.87322E+00_JPRB, 0.87276E+00_JPRB ,&
&  0.87290E+00_JPRB, 0.74075E+00_JPRB, 0.88190E+00_JPRB, 0.88402E+00_JPRB, 0.88047E+00_JPRB, 0.85591E+00_JPRB, 0.85079E+00_JPRB ,&
&  0.85275E+00_JPRB, 0.86750E+00_JPRB, 0.86596E+00_JPRB, 0.85150E+00_JPRB, 0.83443E+00_JPRB, 0.81033E+00_JPRB, 0.79402E+00_JPRB ,&
&  0.78569E+00_JPRB, 0.79020E+00_JPRB, 0.80158E+00_JPRB, 0.81431E+00_JPRB, 0.82710E+00_JPRB, 0.82917E+00_JPRB, 0.83211E+00_JPRB ,&
&  0.83263E+00_JPRB, 0.83464E+00_JPRB, 0.83342E+00_JPRB, 0.83440E+00_JPRB, 0.83240E+00_JPRB, 0.82656E+00_JPRB, 0.81664E+00_JPRB ,&
&  0.80271E+00_JPRB, 0.79381E+00_JPRB, 0.79193E+00_JPRB, 0.79465E+00_JPRB, 0.80635E+00_JPRB, 0.82943E+00_JPRB, 0.85766E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.90605E+00_JPRB, 0.91741E+00_JPRB, 0.92108E+00_JPRB, 0.92943E+00_JPRB, 0.93728E+00_JPRB, 0.93254E+00_JPRB ,&
&  0.91856E+00_JPRB, 0.90282E+00_JPRB, 0.89232E+00_JPRB, 0.88113E+00_JPRB, 0.87461E+00_JPRB, 0.86912E+00_JPRB, 0.86069E+00_JPRB ,&
&  0.85076E+00_JPRB, 0.84886E+00_JPRB, 0.85053E+00_JPRB, 0.84074E+00_JPRB, 0.84430E+00_JPRB, 0.85688E+00_JPRB, 0.87905E+00_JPRB ,&
&  0.90692E+00_JPRB, 0.94299E+00_JPRB, 0.97768E+00_JPRB, 0.10059E+01_JPRB, 0.10230E+01_JPRB, 0.10285E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.97983E+00_JPRB, 0.96016E+00_JPRB, 0.94860E+00_JPRB, 0.94732E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.87845E+00_JPRB, 0.87865E+00_JPRB, 0.86807E+00_JPRB, 0.82936E+00_JPRB, 0.79963E+00_JPRB, 0.80791E+00_JPRB, 0.81538E+00_JPRB ,&
&  0.81477E+00_JPRB, 0.83027E+00_JPRB, 0.85548E+00_JPRB, 0.88074E+00_JPRB, 0.89752E+00_JPRB, 0.89681E+00_JPRB, 0.89993E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.89361E+00_JPRB, 0.90323E+00_JPRB, 0.90824E+00_JPRB, 0.89763E+00_JPRB, 0.87696E+00_JPRB, 0.87524E+00_JPRB ,&
&  0.88174E+00_JPRB, 0.87733E+00_JPRB, 0.85900E+00_JPRB, 0.84655E+00_JPRB, 0.83645E+00_JPRB, 0.84225E+00_JPRB, 0.85102E+00_JPRB ,&
&  0.84863E+00_JPRB, 0.82247E+00_JPRB, 0.86414E+00_JPRB, 0.86887E+00_JPRB, 0.86365E+00_JPRB, 0.83717E+00_JPRB, 0.83775E+00_JPRB ,&
&  0.84638E+00_JPRB, 0.85154E+00_JPRB, 0.83485E+00_JPRB, 0.81551E+00_JPRB, 0.80001E+00_JPRB, 0.78569E+00_JPRB, 0.76032E+00_JPRB ,&
&  0.75184E+00_JPRB, 0.74986E+00_JPRB, 0.76437E+00_JPRB, 0.77996E+00_JPRB, 0.79808E+00_JPRB, 0.80604E+00_JPRB, 0.81662E+00_JPRB ,&
&  0.81763E+00_JPRB, 0.81018E+00_JPRB, 0.80223E+00_JPRB, 0.79454E+00_JPRB, 0.79076E+00_JPRB, 0.78320E+00_JPRB, 0.77217E+00_JPRB ,&
&  0.76200E+00_JPRB, 0.75928E+00_JPRB, 0.76354E+00_JPRB, 0.77280E+00_JPRB, 0.78668E+00_JPRB, 0.81129E+00_JPRB, 0.83349E+00_JPRB ,&
&  0.85957E+00_JPRB, 0.87072E+00_JPRB, 0.87969E+00_JPRB, 0.89431E+00_JPRB, 0.90244E+00_JPRB, 0.90110E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.88982E+00_JPRB, 0.87866E+00_JPRB, 0.87820E+00_JPRB, 0.87227E+00_JPRB, 0.86812E+00_JPRB, 0.85557E+00_JPRB, 0.83722E+00_JPRB ,&
&  0.82963E+00_JPRB, 0.82454E+00_JPRB, 0.81633E+00_JPRB, 0.81365E+00_JPRB, 0.81136E+00_JPRB, 0.83047E+00_JPRB, 0.86415E+00_JPRB ,&
&  0.88393E+00_JPRB, 0.91017E+00_JPRB, 0.94620E+00_JPRB, 0.96914E+00_JPRB, 0.96951E+00_JPRB, 0.96858E+00_JPRB, 0.95672E+00_JPRB ,&
&  0.93777E+00_JPRB, 0.92400E+00_JPRB, 0.91051E+00_JPRB, 0.90482E+00_JPRB, 0.89982E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.82682E+00_JPRB, 0.84872E+00_JPRB, 0.81625E+00_JPRB, 0.79814E+00_JPRB, 0.77659E+00_JPRB, 0.76931E+00_JPRB, 0.75949E+00_JPRB ,&
&  0.77114E+00_JPRB, 0.78927E+00_JPRB, 0.81374E+00_JPRB, 0.82186E+00_JPRB, 0.82854E+00_JPRB, 0.83100E+00_JPRB, 0.84700E+00_JPRB ,&
&  0.84135E+00_JPRB, 0.83254E+00_JPRB, 0.83530E+00_JPRB, 0.82472E+00_JPRB, 0.81594E+00_JPRB, 0.82150E+00_JPRB, 0.83157E+00_JPRB ,&
&  0.84240E+00_JPRB, 0.83360E+00_JPRB, 0.81586E+00_JPRB, 0.78914E+00_JPRB, 0.77644E+00_JPRB, 0.77365E+00_JPRB, 0.78194E+00_JPRB ,&
&  0.78981E+00_JPRB, 0.82925E+00_JPRB, 0.76017E+00_JPRB, 0.81916E+00_JPRB, 0.81986E+00_JPRB, 0.80900E+00_JPRB, 0.80403E+00_JPRB ,&
&  0.80685E+00_JPRB, 0.79604E+00_JPRB, 0.78921E+00_JPRB, 0.78325E+00_JPRB, 0.76099E+00_JPRB, 0.74351E+00_JPRB, 0.72585E+00_JPRB ,&
&  0.71048E+00_JPRB, 0.70092E+00_JPRB, 0.70722E+00_JPRB, 0.72630E+00_JPRB, 0.74548E+00_JPRB, 0.75612E+00_JPRB, 0.76242E+00_JPRB ,&
&  0.76563E+00_JPRB, 0.75800E+00_JPRB, 0.74682E+00_JPRB, 0.74515E+00_JPRB, 0.74938E+00_JPRB, 0.75241E+00_JPRB, 0.75324E+00_JPRB ,&
&  0.75114E+00_JPRB, 0.72808E+00_JPRB, 0.71459E+00_JPRB, 0.73112E+00_JPRB, 0.75097E+00_JPRB, 0.77456E+00_JPRB, 0.80514E+00_JPRB ,&
&  0.82940E+00_JPRB, 0.83956E+00_JPRB, 0.84077E+00_JPRB, 0.84796E+00_JPRB, 0.85428E+00_JPRB, 0.86310E+00_JPRB, 0.86932E+00_JPRB ,&
&  0.85552E+00_JPRB, 0.85038E+00_JPRB, 0.84949E+00_JPRB, 0.84750E+00_JPRB, 0.83569E+00_JPRB, 0.80984E+00_JPRB, 0.77533E+00_JPRB ,&
&  0.75897E+00_JPRB, 0.74331E+00_JPRB, 0.73219E+00_JPRB, 0.72782E+00_JPRB, 0.72807E+00_JPRB, 0.75517E+00_JPRB, 0.79496E+00_JPRB ,&
&  0.83490E+00_JPRB, 0.86727E+00_JPRB, 0.89518E+00_JPRB, 0.92045E+00_JPRB, 0.91998E+00_JPRB, 0.89869E+00_JPRB, 0.88931E+00_JPRB ,&
&  0.87415E+00_JPRB, 0.85715E+00_JPRB, 0.84478E+00_JPRB, 0.83518E+00_JPRB, 0.82541E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.82396E+00_JPRB, 0.82568E+00_JPRB, 0.81107E+00_JPRB, 0.80063E+00_JPRB, 0.78235E+00_JPRB, 0.75853E+00_JPRB, 0.73837E+00_JPRB ,&
&  0.74087E+00_JPRB, 0.75767E+00_JPRB, 0.77354E+00_JPRB, 0.78478E+00_JPRB, 0.80027E+00_JPRB, 0.80320E+00_JPRB, 0.80892E+00_JPRB ,&
&  0.80350E+00_JPRB, 0.76579E+00_JPRB, 0.75960E+00_JPRB, 0.76697E+00_JPRB, 0.76874E+00_JPRB, 0.77535E+00_JPRB, 0.78711E+00_JPRB ,&
&  0.79775E+00_JPRB, 0.79332E+00_JPRB, 0.78489E+00_JPRB, 0.75482E+00_JPRB, 0.74785E+00_JPRB, 0.75274E+00_JPRB, 0.75806E+00_JPRB ,&
&  0.76912E+00_JPRB, 0.79083E+00_JPRB, 0.79198E+00_JPRB, 0.80281E+00_JPRB, 0.80800E+00_JPRB, 0.80839E+00_JPRB, 0.80307E+00_JPRB ,&
&  0.77582E+00_JPRB, 0.74915E+00_JPRB, 0.73876E+00_JPRB, 0.72254E+00_JPRB, 0.69872E+00_JPRB, 0.68235E+00_JPRB, 0.67574E+00_JPRB ,&
&  0.67340E+00_JPRB, 0.67976E+00_JPRB, 0.68648E+00_JPRB, 0.70074E+00_JPRB, 0.70612E+00_JPRB, 0.71053E+00_JPRB, 0.71656E+00_JPRB ,&
&  0.71717E+00_JPRB, 0.71416E+00_JPRB, 0.71458E+00_JPRB, 0.72412E+00_JPRB, 0.72863E+00_JPRB, 0.72459E+00_JPRB, 0.71841E+00_JPRB ,&
&  0.70354E+00_JPRB, 0.68939E+00_JPRB, 0.69633E+00_JPRB, 0.71249E+00_JPRB, 0.72151E+00_JPRB, 0.75000E+00_JPRB, 0.78617E+00_JPRB ,&
&  0.81703E+00_JPRB, 0.83667E+00_JPRB, 0.83699E+00_JPRB, 0.83212E+00_JPRB, 0.82943E+00_JPRB, 0.83642E+00_JPRB, 0.84786E+00_JPRB ,&
&  0.83095E+00_JPRB, 0.82112E+00_JPRB, 0.81823E+00_JPRB, 0.79754E+00_JPRB, 0.75935E+00_JPRB, 0.72579E+00_JPRB, 0.70127E+00_JPRB ,&
&  0.68197E+00_JPRB, 0.67130E+00_JPRB, 0.67518E+00_JPRB, 0.67764E+00_JPRB, 0.68914E+00_JPRB, 0.72493E+00_JPRB, 0.76323E+00_JPRB ,&
&  0.80535E+00_JPRB, 0.82754E+00_JPRB, 0.83553E+00_JPRB, 0.84489E+00_JPRB, 0.85203E+00_JPRB, 0.84171E+00_JPRB, 0.83266E+00_JPRB ,&
&  0.82993E+00_JPRB, 0.82105E+00_JPRB, 0.81152E+00_JPRB, 0.80207E+00_JPRB, 0.81014E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.81537E+00_JPRB, 0.80812E+00_JPRB, 0.79447E+00_JPRB, 0.76652E+00_JPRB, 0.74768E+00_JPRB, 0.72039E+00_JPRB, 0.70540E+00_JPRB ,&
&  0.70839E+00_JPRB, 0.71071E+00_JPRB, 0.71648E+00_JPRB, 0.72571E+00_JPRB, 0.72342E+00_JPRB, 0.71846E+00_JPRB, 0.71319E+00_JPRB ,&
&  0.71539E+00_JPRB, 0.71554E+00_JPRB, 0.72398E+00_JPRB, 0.73919E+00_JPRB, 0.75518E+00_JPRB, 0.76470E+00_JPRB, 0.77537E+00_JPRB ,&
&  0.78359E+00_JPRB, 0.77772E+00_JPRB, 0.77291E+00_JPRB, 0.74136E+00_JPRB, 0.72294E+00_JPRB, 0.72783E+00_JPRB, 0.73565E+00_JPRB ,&
&  0.74628E+00_JPRB, 0.75840E+00_JPRB, 0.76650E+00_JPRB, 0.77495E+00_JPRB, 0.76107E+00_JPRB, 0.75602E+00_JPRB, 0.74419E+00_JPRB ,&
&  0.69943E+00_JPRB, 0.65598E+00_JPRB, 0.63254E+00_JPRB, 0.62360E+00_JPRB, 0.61587E+00_JPRB, 0.62116E+00_JPRB, 0.62994E+00_JPRB ,&
&  0.64507E+00_JPRB, 0.65861E+00_JPRB, 0.66183E+00_JPRB, 0.66444E+00_JPRB, 0.66592E+00_JPRB, 0.66571E+00_JPRB, 0.66759E+00_JPRB ,&
&  0.66666E+00_JPRB, 0.66700E+00_JPRB, 0.67477E+00_JPRB, 0.67483E+00_JPRB, 0.66474E+00_JPRB, 0.66038E+00_JPRB, 0.65648E+00_JPRB ,&
&  0.65298E+00_JPRB, 0.64997E+00_JPRB, 0.64648E+00_JPRB, 0.65732E+00_JPRB, 0.68439E+00_JPRB, 0.71508E+00_JPRB, 0.74103E+00_JPRB ,&
&  0.75653E+00_JPRB, 0.76954E+00_JPRB, 0.77304E+00_JPRB, 0.77470E+00_JPRB, 0.77418E+00_JPRB, 0.76510E+00_JPRB, 0.76127E+00_JPRB ,&
&  0.75332E+00_JPRB, 0.73352E+00_JPRB, 0.70693E+00_JPRB, 0.66619E+00_JPRB, 0.62661E+00_JPRB, 0.60372E+00_JPRB, 0.60297E+00_JPRB ,&
&  0.61500E+00_JPRB, 0.62757E+00_JPRB, 0.63372E+00_JPRB, 0.64119E+00_JPRB, 0.65499E+00_JPRB, 0.67175E+00_JPRB, 0.69004E+00_JPRB ,&
&  0.70166E+00_JPRB, 0.70219E+00_JPRB, 0.70078E+00_JPRB, 0.71014E+00_JPRB, 0.72243E+00_JPRB, 0.74170E+00_JPRB, 0.76260E+00_JPRB ,&
&  0.78063E+00_JPRB, 0.78091E+00_JPRB, 0.78179E+00_JPRB, 0.78366E+00_JPRB, 0.80423E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.80677E+00_JPRB, 0.80494E+00_JPRB, 0.77531E+00_JPRB, 0.74259E+00_JPRB, 0.72396E+00_JPRB, 0.70398E+00_JPRB, 0.68813E+00_JPRB ,&
&  0.68680E+00_JPRB, 0.68400E+00_JPRB, 0.68144E+00_JPRB, 0.67809E+00_JPRB, 0.67595E+00_JPRB, 0.67293E+00_JPRB, 0.66101E+00_JPRB ,&
&  0.65635E+00_JPRB, 0.66251E+00_JPRB, 0.68456E+00_JPRB, 0.70913E+00_JPRB, 0.72876E+00_JPRB, 0.75483E+00_JPRB, 0.76654E+00_JPRB ,&
&  0.77924E+00_JPRB, 0.79226E+00_JPRB, 0.79403E+00_JPRB, 0.75966E+00_JPRB, 0.73146E+00_JPRB, 0.73231E+00_JPRB, 0.74393E+00_JPRB ,&
&  0.74491E+00_JPRB, 0.74882E+00_JPRB, 0.75097E+00_JPRB, 0.74932E+00_JPRB, 0.72937E+00_JPRB, 0.69137E+00_JPRB, 0.66031E+00_JPRB ,&
&  0.63359E+00_JPRB, 0.61264E+00_JPRB, 0.60480E+00_JPRB, 0.59085E+00_JPRB, 0.58692E+00_JPRB, 0.59672E+00_JPRB, 0.61182E+00_JPRB ,&
&  0.62787E+00_JPRB, 0.63578E+00_JPRB, 0.64300E+00_JPRB, 0.63877E+00_JPRB, 0.63466E+00_JPRB, 0.63832E+00_JPRB, 0.63910E+00_JPRB ,&
&  0.63981E+00_JPRB, 0.63129E+00_JPRB, 0.62796E+00_JPRB, 0.62019E+00_JPRB, 0.61554E+00_JPRB, 0.61450E+00_JPRB, 0.61615E+00_JPRB ,&
&  0.61666E+00_JPRB, 0.62199E+00_JPRB, 0.62371E+00_JPRB, 0.62639E+00_JPRB, 0.63632E+00_JPRB, 0.65343E+00_JPRB, 0.67037E+00_JPRB ,&
&  0.68299E+00_JPRB, 0.69286E+00_JPRB, 0.70026E+00_JPRB, 0.70625E+00_JPRB, 0.70631E+00_JPRB, 0.70144E+00_JPRB, 0.68721E+00_JPRB ,&
&  0.65660E+00_JPRB, 0.62387E+00_JPRB, 0.59893E+00_JPRB, 0.57910E+00_JPRB, 0.57609E+00_JPRB, 0.58790E+00_JPRB, 0.60634E+00_JPRB ,&
&  0.62016E+00_JPRB, 0.62346E+00_JPRB, 0.61634E+00_JPRB, 0.61330E+00_JPRB, 0.61655E+00_JPRB, 0.62487E+00_JPRB, 0.63139E+00_JPRB ,&
&  0.63000E+00_JPRB, 0.62701E+00_JPRB, 0.63186E+00_JPRB, 0.64081E+00_JPRB, 0.65703E+00_JPRB, 0.67687E+00_JPRB, 0.69795E+00_JPRB ,&
&  0.71855E+00_JPRB, 0.73428E+00_JPRB, 0.75403E+00_JPRB, 0.77385E+00_JPRB, 0.79898E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.78648E+00_JPRB, 0.75761E+00_JPRB, 0.72731E+00_JPRB, 0.70183E+00_JPRB, 0.69027E+00_JPRB, 0.67329E+00_JPRB, 0.65894E+00_JPRB ,&
&  0.65819E+00_JPRB, 0.66588E+00_JPRB, 0.66079E+00_JPRB, 0.65431E+00_JPRB, 0.64503E+00_JPRB, 0.63774E+00_JPRB, 0.62917E+00_JPRB ,&
&  0.62514E+00_JPRB, 0.63595E+00_JPRB, 0.66108E+00_JPRB, 0.67318E+00_JPRB, 0.67884E+00_JPRB, 0.70897E+00_JPRB, 0.74056E+00_JPRB ,&
&  0.75049E+00_JPRB, 0.76500E+00_JPRB, 0.76866E+00_JPRB, 0.74199E+00_JPRB, 0.72448E+00_JPRB, 0.71772E+00_JPRB, 0.72158E+00_JPRB ,&
&  0.71748E+00_JPRB, 0.72212E+00_JPRB, 0.71327E+00_JPRB, 0.64568E+00_JPRB, 0.64483E+00_JPRB, 0.61469E+00_JPRB, 0.60373E+00_JPRB ,&
&  0.59742E+00_JPRB, 0.60130E+00_JPRB, 0.59563E+00_JPRB, 0.58422E+00_JPRB, 0.57948E+00_JPRB, 0.58284E+00_JPRB, 0.58554E+00_JPRB ,&
&  0.58281E+00_JPRB, 0.57314E+00_JPRB, 0.57795E+00_JPRB, 0.58433E+00_JPRB, 0.58791E+00_JPRB, 0.59539E+00_JPRB, 0.60645E+00_JPRB ,&
&  0.59669E+00_JPRB, 0.58692E+00_JPRB, 0.57813E+00_JPRB, 0.57870E+00_JPRB, 0.58538E+00_JPRB, 0.59872E+00_JPRB, 0.60217E+00_JPRB ,&
&  0.59936E+00_JPRB, 0.59806E+00_JPRB, 0.59956E+00_JPRB, 0.60649E+00_JPRB, 0.62104E+00_JPRB, 0.63521E+00_JPRB, 0.63627E+00_JPRB ,&
&  0.63762E+00_JPRB, 0.64262E+00_JPRB, 0.64866E+00_JPRB, 0.64961E+00_JPRB, 0.64483E+00_JPRB, 0.62935E+00_JPRB, 0.61168E+00_JPRB ,&
&  0.59507E+00_JPRB, 0.57665E+00_JPRB, 0.56185E+00_JPRB, 0.55818E+00_JPRB, 0.57301E+00_JPRB, 0.60380E+00_JPRB, 0.62134E+00_JPRB ,&
&  0.62601E+00_JPRB, 0.62823E+00_JPRB, 0.63086E+00_JPRB, 0.63070E+00_JPRB, 0.63152E+00_JPRB, 0.62842E+00_JPRB, 0.62042E+00_JPRB ,&
&  0.60815E+00_JPRB, 0.59744E+00_JPRB, 0.59654E+00_JPRB, 0.60367E+00_JPRB, 0.61154E+00_JPRB, 0.62658E+00_JPRB, 0.64555E+00_JPRB ,&
&  0.66924E+00_JPRB, 0.70820E+00_JPRB, 0.74053E+00_JPRB, 0.76518E+00_JPRB, 0.78207E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.76310E+00_JPRB, 0.73524E+00_JPRB, 0.71642E+00_JPRB, 0.70208E+00_JPRB, 0.68362E+00_JPRB, 0.66955E+00_JPRB, 0.66389E+00_JPRB ,&
&  0.67204E+00_JPRB, 0.67049E+00_JPRB, 0.66076E+00_JPRB, 0.65283E+00_JPRB, 0.63737E+00_JPRB, 0.62357E+00_JPRB, 0.61804E+00_JPRB ,&
&  0.62637E+00_JPRB, 0.64218E+00_JPRB, 0.66172E+00_JPRB, 0.66888E+00_JPRB, 0.66748E+00_JPRB, 0.67579E+00_JPRB, 0.69514E+00_JPRB ,&
&  0.72397E+00_JPRB, 0.74414E+00_JPRB, 0.75049E+00_JPRB, 0.73608E+00_JPRB, 0.71758E+00_JPRB, 0.70906E+00_JPRB, 0.69510E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.68114E+00_JPRB, 0.61488E+00_JPRB, 0.60716E+00_JPRB, 0.58468E+00_JPRB, 0.56988E+00_JPRB, 0.56571E+00_JPRB ,&
&  0.57844E+00_JPRB, 0.58140E+00_JPRB, 0.57169E+00_JPRB, 0.55818E+00_JPRB, 0.54953E+00_JPRB, 0.53857E+00_JPRB, 0.52894E+00_JPRB ,&
&  0.52302E+00_JPRB, 0.52621E+00_JPRB, 0.53529E+00_JPRB, 0.54477E+00_JPRB, 0.55561E+00_JPRB, 0.56629E+00_JPRB, 0.56317E+00_JPRB ,&
&  0.55159E+00_JPRB, 0.54983E+00_JPRB, 0.55771E+00_JPRB, 0.57351E+00_JPRB, 0.59054E+00_JPRB, 0.59801E+00_JPRB, 0.60014E+00_JPRB ,&
&  0.59015E+00_JPRB, 0.57215E+00_JPRB, 0.55706E+00_JPRB, 0.54486E+00_JPRB, 0.51632E+00_JPRB, 0.45450E+00_JPRB, 0.43251E+00_JPRB ,&
&  0.44465E+00_JPRB, 0.50202E+00_JPRB, 0.56904E+00_JPRB, 0.56226E+00_JPRB, 0.56067E+00_JPRB, 0.54612E+00_JPRB, 0.50781E+00_JPRB ,&
&  0.46096E+00_JPRB, 0.40064E+00_JPRB, 0.38183E+00_JPRB, 0.37937E+00_JPRB, 0.37975E+00_JPRB, 0.38799E+00_JPRB, 0.40143E+00_JPRB ,&
&  0.41885E+00_JPRB, 0.44355E+00_JPRB, 0.45974E+00_JPRB, 0.46538E+00_JPRB, 0.46276E+00_JPRB, 0.44646E+00_JPRB, 0.43194E+00_JPRB ,&
&  0.42317E+00_JPRB, 0.44968E+00_JPRB, 0.51080E+00_JPRB, 0.59284E+00_JPRB, 0.59566E+00_JPRB, 0.62408E+00_JPRB, 0.65108E+00_JPRB ,&
&  0.67658E+00_JPRB, 0.71787E+00_JPRB, 0.74387E+00_JPRB, 0.74883E+00_JPRB, 0.76633E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.72233E+00_JPRB, 0.70862E+00_JPRB, 0.69899E+00_JPRB, 0.69223E+00_JPRB, 0.68466E+00_JPRB, 0.68281E+00_JPRB, 0.68179E+00_JPRB ,&
&  0.67590E+00_JPRB, 0.66769E+00_JPRB, 0.66922E+00_JPRB, 0.65285E+00_JPRB, 0.62901E+00_JPRB, 0.61307E+00_JPRB, 0.61101E+00_JPRB ,&
&  0.61483E+00_JPRB, 0.62196E+00_JPRB, 0.63082E+00_JPRB, 0.63856E+00_JPRB, 0.64307E+00_JPRB, 0.65460E+00_JPRB, 0.67519E+00_JPRB ,&
&  0.69904E+00_JPRB, 0.71239E+00_JPRB, 0.71606E+00_JPRB, 0.70732E+00_JPRB, 0.69463E+00_JPRB, 0.68184E+00_JPRB, 0.67040E+00_JPRB ,&
&  0.65855E+00_JPRB, 0.55686E+00_JPRB, 0.49141E+00_JPRB, 0.49964E+00_JPRB, 0.56408E+00_JPRB, 0.54760E+00_JPRB, 0.54738E+00_JPRB ,&
&  0.55997E+00_JPRB, 0.55866E+00_JPRB, 0.54449E+00_JPRB, 0.52684E+00_JPRB, 0.51055E+00_JPRB, 0.49893E+00_JPRB, 0.49125E+00_JPRB ,&
&  0.48829E+00_JPRB, 0.49572E+00_JPRB, 0.50335E+00_JPRB, 0.51504E+00_JPRB, 0.51736E+00_JPRB, 0.50387E+00_JPRB, 0.48751E+00_JPRB ,&
&  0.47204E+00_JPRB, 0.46633E+00_JPRB, 0.46915E+00_JPRB, 0.47698E+00_JPRB, 0.48721E+00_JPRB, 0.49512E+00_JPRB, 0.49216E+00_JPRB ,&
&  0.46453E+00_JPRB, 0.44088E+00_JPRB, 0.41255E+00_JPRB, 0.38574E+00_JPRB, 0.36272E+00_JPRB, 0.34747E+00_JPRB, 0.33919E+00_JPRB ,&
&  0.33758E+00_JPRB, 0.34660E+00_JPRB, 0.36892E+00_JPRB, 0.41793E+00_JPRB, 0.47844E+00_JPRB, 0.39884E+00_JPRB, 0.36080E+00_JPRB ,&
&  0.34045E+00_JPRB, 0.32707E+00_JPRB, 0.32194E+00_JPRB, 0.32330E+00_JPRB, 0.32848E+00_JPRB, 0.33559E+00_JPRB, 0.34120E+00_JPRB ,&
&  0.34581E+00_JPRB, 0.34916E+00_JPRB, 0.35000E+00_JPRB, 0.35016E+00_JPRB, 0.35123E+00_JPRB, 0.35229E+00_JPRB, 0.35103E+00_JPRB ,&
&  0.35012E+00_JPRB, 0.35800E+00_JPRB, 0.37776E+00_JPRB, 0.40822E+00_JPRB, 0.44257E+00_JPRB, 0.48050E+00_JPRB, 0.53399E+00_JPRB ,&
&  0.59956E+00_JPRB, 0.67108E+00_JPRB, 0.69149E+00_JPRB, 0.70654E+00_JPRB, 0.71720E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.66139E+00_JPRB, 0.66417E+00_JPRB, 0.66712E+00_JPRB, 0.66850E+00_JPRB, 0.66255E+00_JPRB, 0.65480E+00_JPRB, 0.65020E+00_JPRB ,&
&  0.65564E+00_JPRB, 0.65634E+00_JPRB, 0.63372E+00_JPRB, 0.61645E+00_JPRB, 0.60679E+00_JPRB, 0.60334E+00_JPRB, 0.60081E+00_JPRB ,&
&  0.59461E+00_JPRB, 0.59238E+00_JPRB, 0.59313E+00_JPRB, 0.60370E+00_JPRB, 0.61250E+00_JPRB, 0.62007E+00_JPRB, 0.62714E+00_JPRB ,&
&  0.62137E+00_JPRB, 0.63254E+00_JPRB, 0.65353E+00_JPRB, 0.65187E+00_JPRB, 0.64846E+00_JPRB, 0.64065E+00_JPRB, 0.62555E+00_JPRB ,&
&  0.57555E+00_JPRB, 0.51100E+00_JPRB, 0.45287E+00_JPRB, 0.46636E+00_JPRB, 0.53959E+00_JPRB, 0.54569E+00_JPRB, 0.53298E+00_JPRB ,&
&  0.54403E+00_JPRB, 0.53851E+00_JPRB, 0.52019E+00_JPRB, 0.50620E+00_JPRB, 0.49034E+00_JPRB, 0.48007E+00_JPRB, 0.47246E+00_JPRB ,&
&  0.46927E+00_JPRB, 0.46909E+00_JPRB, 0.45245E+00_JPRB, 0.40174E+00_JPRB, 0.37426E+00_JPRB, 0.35676E+00_JPRB, 0.33840E+00_JPRB ,&
&  0.32353E+00_JPRB, 0.31518E+00_JPRB, 0.31459E+00_JPRB, 0.31507E+00_JPRB, 0.32102E+00_JPRB, 0.32464E+00_JPRB, 0.32784E+00_JPRB ,&
&  0.32855E+00_JPRB, 0.32764E+00_JPRB, 0.32376E+00_JPRB, 0.31936E+00_JPRB, 0.31621E+00_JPRB, 0.31608E+00_JPRB, 0.31562E+00_JPRB ,&
&  0.31848E+00_JPRB, 0.32217E+00_JPRB, 0.33941E+00_JPRB, 0.35723E+00_JPRB, 0.35261E+00_JPRB, 0.33755E+00_JPRB, 0.32404E+00_JPRB ,&
&  0.31315E+00_JPRB, 0.30768E+00_JPRB, 0.30411E+00_JPRB, 0.30358E+00_JPRB, 0.30715E+00_JPRB, 0.31234E+00_JPRB, 0.31353E+00_JPRB ,&
&  0.31309E+00_JPRB, 0.31361E+00_JPRB, 0.31349E+00_JPRB, 0.31376E+00_JPRB, 0.31422E+00_JPRB, 0.31708E+00_JPRB, 0.32260E+00_JPRB ,&
&  0.32908E+00_JPRB, 0.34078E+00_JPRB, 0.35371E+00_JPRB, 0.37089E+00_JPRB, 0.38355E+00_JPRB, 0.39345E+00_JPRB, 0.40790E+00_JPRB ,&
&  0.42698E+00_JPRB, 0.46785E+00_JPRB, 0.53709E+00_JPRB, 0.65969E+00_JPRB, 0.65239E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.62762E+00_JPRB, 0.63830E+00_JPRB, 0.64969E+00_JPRB, 0.64085E+00_JPRB, 0.62749E+00_JPRB, 0.61994E+00_JPRB, 0.62003E+00_JPRB ,&
&  0.62039E+00_JPRB, 0.61790E+00_JPRB, 0.60637E+00_JPRB, 0.59230E+00_JPRB, 0.57398E+00_JPRB, 0.52676E+00_JPRB, 0.51028E+00_JPRB ,&
&  0.49311E+00_JPRB, 0.49596E+00_JPRB, 0.49966E+00_JPRB, 0.50531E+00_JPRB, 0.51816E+00_JPRB, 0.52828E+00_JPRB, 0.52916E+00_JPRB ,&
&  0.51810E+00_JPRB, 0.49928E+00_JPRB, 0.48082E+00_JPRB, 0.46684E+00_JPRB, 0.46627E+00_JPRB, 0.46562E+00_JPRB, 0.45795E+00_JPRB ,&
&  0.44446E+00_JPRB, 0.43894E+00_JPRB, 0.43894E+00_JPRB, 0.45861E+00_JPRB, 0.50852E+00_JPRB, 0.52641E+00_JPRB, 0.50317E+00_JPRB ,&
&  0.50768E+00_JPRB, 0.50197E+00_JPRB, 0.49754E+00_JPRB, 0.49811E+00_JPRB, 0.48703E+00_JPRB, 0.46409E+00_JPRB, 0.42998E+00_JPRB ,&
&  0.38871E+00_JPRB, 0.36738E+00_JPRB, 0.34629E+00_JPRB, 0.32522E+00_JPRB, 0.30777E+00_JPRB, 0.29788E+00_JPRB, 0.28960E+00_JPRB ,&
&  0.28432E+00_JPRB, 0.27970E+00_JPRB, 0.27683E+00_JPRB, 0.27724E+00_JPRB, 0.27963E+00_JPRB, 0.28094E+00_JPRB, 0.28242E+00_JPRB ,&
&  0.28450E+00_JPRB, 0.28589E+00_JPRB, 0.28681E+00_JPRB, 0.28872E+00_JPRB, 0.29063E+00_JPRB, 0.29392E+00_JPRB, 0.29805E+00_JPRB ,&
&  0.30321E+00_JPRB, 0.30694E+00_JPRB, 0.31039E+00_JPRB, 0.31739E+00_JPRB, 0.31806E+00_JPRB, 0.31653E+00_JPRB, 0.30811E+00_JPRB ,&
&  0.30181E+00_JPRB, 0.29912E+00_JPRB, 0.29628E+00_JPRB, 0.29554E+00_JPRB, 0.29694E+00_JPRB, 0.29927E+00_JPRB, 0.30166E+00_JPRB ,&
&  0.30335E+00_JPRB, 0.30333E+00_JPRB, 0.30223E+00_JPRB, 0.30190E+00_JPRB, 0.30235E+00_JPRB, 0.30240E+00_JPRB, 0.30649E+00_JPRB ,&
&  0.31574E+00_JPRB, 0.32912E+00_JPRB, 0.34383E+00_JPRB, 0.36336E+00_JPRB, 0.38214E+00_JPRB, 0.39926E+00_JPRB, 0.42777E+00_JPRB ,&
&  0.46590E+00_JPRB, 0.57045E+00_JPRB, 0.63769E+00_JPRB, 0.58204E+00_JPRB, 0.60757E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.60117E+00_JPRB, 0.60850E+00_JPRB, 0.61865E+00_JPRB, 0.62282E+00_JPRB, 0.63040E+00_JPRB, 0.61860E+00_JPRB, 0.61109E+00_JPRB ,&
&  0.58083E+00_JPRB, 0.56024E+00_JPRB, 0.52983E+00_JPRB, 0.49296E+00_JPRB, 0.46810E+00_JPRB, 0.44923E+00_JPRB, 0.44181E+00_JPRB ,&
&  0.43076E+00_JPRB, 0.42430E+00_JPRB, 0.41683E+00_JPRB, 0.41354E+00_JPRB, 0.41043E+00_JPRB, 0.41053E+00_JPRB, 0.40925E+00_JPRB ,&
&  0.40494E+00_JPRB, 0.39912E+00_JPRB, 0.39208E+00_JPRB, 0.39412E+00_JPRB, 0.40675E+00_JPRB, 0.42116E+00_JPRB, 0.43857E+00_JPRB ,&
&  0.45895E+00_JPRB, 0.46908E+00_JPRB, 0.47951E+00_JPRB, 0.48351E+00_JPRB, 0.46806E+00_JPRB, 0.47088E+00_JPRB, 0.47507E+00_JPRB ,&
&  0.48287E+00_JPRB, 0.48777E+00_JPRB, 0.48383E+00_JPRB, 0.47754E+00_JPRB, 0.42533E+00_JPRB, 0.39342E+00_JPRB, 0.36465E+00_JPRB ,&
&  0.34343E+00_JPRB, 0.33010E+00_JPRB, 0.31766E+00_JPRB, 0.30690E+00_JPRB, 0.29482E+00_JPRB, 0.28735E+00_JPRB, 0.28179E+00_JPRB ,&
&  0.27750E+00_JPRB, 0.27592E+00_JPRB, 0.27454E+00_JPRB, 0.27438E+00_JPRB, 0.27532E+00_JPRB, 0.27483E+00_JPRB, 0.27510E+00_JPRB ,&
&  0.27532E+00_JPRB, 0.27504E+00_JPRB, 0.27570E+00_JPRB, 0.27761E+00_JPRB, 0.28008E+00_JPRB, 0.28389E+00_JPRB, 0.28473E+00_JPRB ,&
&  0.28594E+00_JPRB, 0.28700E+00_JPRB, 0.28716E+00_JPRB, 0.28954E+00_JPRB, 0.29090E+00_JPRB, 0.29104E+00_JPRB, 0.28943E+00_JPRB ,&
&  0.28766E+00_JPRB, 0.28625E+00_JPRB, 0.28647E+00_JPRB, 0.28727E+00_JPRB, 0.28924E+00_JPRB, 0.29206E+00_JPRB, 0.29252E+00_JPRB ,&
&  0.29343E+00_JPRB, 0.29272E+00_JPRB, 0.29195E+00_JPRB, 0.29455E+00_JPRB, 0.29749E+00_JPRB, 0.30059E+00_JPRB, 0.30458E+00_JPRB ,&
&  0.30614E+00_JPRB, 0.31412E+00_JPRB, 0.32910E+00_JPRB, 0.35238E+00_JPRB, 0.39146E+00_JPRB, 0.40879E+00_JPRB, 0.44913E+00_JPRB ,&
&  0.50331E+00_JPRB, 0.63749E+00_JPRB, 0.70140E+00_JPRB, 0.56752E+00_JPRB, 0.58531E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.54223E+00_JPRB, 0.55455E+00_JPRB, 0.57065E+00_JPRB, 0.58065E+00_JPRB, 0.59367E+00_JPRB, 0.59572E+00_JPRB, 0.58828E+00_JPRB ,&
&  0.57083E+00_JPRB, 0.54363E+00_JPRB, 0.52751E+00_JPRB, 0.50597E+00_JPRB, 0.48485E+00_JPRB, 0.46291E+00_JPRB, 0.45300E+00_JPRB ,&
&  0.44106E+00_JPRB, 0.42680E+00_JPRB, 0.41246E+00_JPRB, 0.39978E+00_JPRB, 0.38745E+00_JPRB, 0.37628E+00_JPRB, 0.37003E+00_JPRB ,&
&  0.36364E+00_JPRB, 0.36347E+00_JPRB, 0.36288E+00_JPRB, 0.36675E+00_JPRB, 0.37833E+00_JPRB, 0.39667E+00_JPRB, 0.42256E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.46395E+00_JPRB, 0.44965E+00_JPRB, 0.44361E+00_JPRB, 0.43605E+00_JPRB, 0.43000E+00_JPRB, 0.42001E+00_JPRB ,&
&  0.42759E+00_JPRB, 0.43596E+00_JPRB, 0.45072E+00_JPRB, 0.43615E+00_JPRB, 0.38946E+00_JPRB, 0.37041E+00_JPRB, 0.34666E+00_JPRB ,&
&  0.33622E+00_JPRB, 0.32655E+00_JPRB, 0.31859E+00_JPRB, 0.31025E+00_JPRB, 0.29972E+00_JPRB, 0.29376E+00_JPRB, 0.28717E+00_JPRB ,&
&  0.28308E+00_JPRB, 0.28197E+00_JPRB, 0.27969E+00_JPRB, 0.28037E+00_JPRB, 0.28067E+00_JPRB, 0.28013E+00_JPRB, 0.27983E+00_JPRB ,&
&  0.27895E+00_JPRB, 0.27801E+00_JPRB, 0.27780E+00_JPRB, 0.27866E+00_JPRB, 0.27951E+00_JPRB, 0.28029E+00_JPRB, 0.27761E+00_JPRB ,&
&  0.27637E+00_JPRB, 0.27517E+00_JPRB, 0.27520E+00_JPRB, 0.27726E+00_JPRB, 0.27801E+00_JPRB, 0.27941E+00_JPRB, 0.27986E+00_JPRB ,&
&  0.28022E+00_JPRB, 0.28105E+00_JPRB, 0.28063E+00_JPRB, 0.28153E+00_JPRB, 0.28177E+00_JPRB, 0.28301E+00_JPRB, 0.28368E+00_JPRB ,&
&  0.28441E+00_JPRB, 0.28683E+00_JPRB, 0.28932E+00_JPRB, 0.29317E+00_JPRB, 0.29830E+00_JPRB, 0.30112E+00_JPRB, 0.30592E+00_JPRB ,&
&  0.31123E+00_JPRB, 0.32109E+00_JPRB, 0.34112E+00_JPRB, 0.36343E+00_JPRB, 0.39173E+00_JPRB, 0.40249E+00_JPRB, 0.40790E+00_JPRB ,&
&  0.41695E+00_JPRB, 0.44561E+00_JPRB, 0.49614E+00_JPRB, 0.52428E+00_JPRB, 0.53318E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.53063E+00_JPRB, 0.53371E+00_JPRB, 0.53602E+00_JPRB, 0.53377E+00_JPRB, 0.53410E+00_JPRB, 0.53447E+00_JPRB, 0.53191E+00_JPRB ,&
&  0.52379E+00_JPRB, 0.52198E+00_JPRB, 0.51675E+00_JPRB, 0.50861E+00_JPRB, 0.50032E+00_JPRB, 0.48641E+00_JPRB, 0.46968E+00_JPRB ,&
&  0.44744E+00_JPRB, 0.42744E+00_JPRB, 0.41285E+00_JPRB, 0.39864E+00_JPRB, 0.38480E+00_JPRB, 0.37130E+00_JPRB, 0.36692E+00_JPRB ,&
&  0.36264E+00_JPRB, 0.36105E+00_JPRB, 0.36085E+00_JPRB, 0.36387E+00_JPRB, 0.37119E+00_JPRB, 0.38043E+00_JPRB, 0.39070E+00_JPRB ,&
&  0.40167E+00_JPRB, 0.41208E+00_JPRB, 0.41295E+00_JPRB, 0.40484E+00_JPRB, 0.39782E+00_JPRB, 0.39378E+00_JPRB, 0.39041E+00_JPRB ,&
&  0.38649E+00_JPRB, 0.37852E+00_JPRB, 0.36879E+00_JPRB, 0.35830E+00_JPRB, 0.34781E+00_JPRB, 0.34016E+00_JPRB, 0.33122E+00_JPRB ,&
&  0.32463E+00_JPRB, 0.31842E+00_JPRB, 0.31143E+00_JPRB, 0.30538E+00_JPRB, 0.30124E+00_JPRB, 0.29790E+00_JPRB, 0.29437E+00_JPRB ,&
&  0.29294E+00_JPRB, 0.29148E+00_JPRB, 0.28990E+00_JPRB, 0.28851E+00_JPRB, 0.28659E+00_JPRB, 0.28527E+00_JPRB, 0.28409E+00_JPRB ,&
&  0.28326E+00_JPRB, 0.28247E+00_JPRB, 0.28204E+00_JPRB, 0.28152E+00_JPRB, 0.28105E+00_JPRB, 0.27987E+00_JPRB, 0.27851E+00_JPRB ,&
&  0.27784E+00_JPRB, 0.27800E+00_JPRB, 0.27726E+00_JPRB, 0.27698E+00_JPRB, 0.27671E+00_JPRB, 0.27582E+00_JPRB, 0.27650E+00_JPRB ,&
&  0.27806E+00_JPRB, 0.27975E+00_JPRB, 0.27981E+00_JPRB, 0.28089E+00_JPRB, 0.28216E+00_JPRB, 0.28379E+00_JPRB, 0.28668E+00_JPRB ,&
&  0.28881E+00_JPRB, 0.29105E+00_JPRB, 0.29395E+00_JPRB, 0.29627E+00_JPRB, 0.30010E+00_JPRB, 0.30486E+00_JPRB, 0.31078E+00_JPRB ,&
&  0.31893E+00_JPRB, 0.32880E+00_JPRB, 0.34315E+00_JPRB, 0.36086E+00_JPRB, 0.38726E+00_JPRB, 0.39486E+00_JPRB, 0.40135E+00_JPRB ,&
&  0.40537E+00_JPRB, 0.41235E+00_JPRB, 0.49509E+00_JPRB, 0.50903E+00_JPRB, 0.52393E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.40905E+00_JPRB, 0.41848E+00_JPRB, 0.42582E+00_JPRB, 0.43242E+00_JPRB, 0.42641E+00_JPRB, 0.49083E+00_JPRB, 0.46923E+00_JPRB ,&
&  0.45888E+00_JPRB, 0.43302E+00_JPRB, 0.43583E+00_JPRB, 0.43269E+00_JPRB, 0.42704E+00_JPRB, 0.42141E+00_JPRB, 0.41647E+00_JPRB ,&
&  0.41129E+00_JPRB, 0.40401E+00_JPRB, 0.40009E+00_JPRB, 0.39468E+00_JPRB, 0.38935E+00_JPRB, 0.38383E+00_JPRB, 0.38044E+00_JPRB ,&
&  0.37793E+00_JPRB, 0.37645E+00_JPRB, 0.37532E+00_JPRB, 0.37257E+00_JPRB, 0.37400E+00_JPRB, 0.37438E+00_JPRB, 0.37175E+00_JPRB ,&
&  0.36693E+00_JPRB, 0.36266E+00_JPRB, 0.35770E+00_JPRB, 0.35265E+00_JPRB, 0.35065E+00_JPRB, 0.34864E+00_JPRB, 0.34642E+00_JPRB ,&
&  0.34312E+00_JPRB, 0.33936E+00_JPRB, 0.33521E+00_JPRB, 0.33173E+00_JPRB, 0.32823E+00_JPRB, 0.32447E+00_JPRB, 0.32083E+00_JPRB ,&
&  0.31688E+00_JPRB, 0.31290E+00_JPRB, 0.30890E+00_JPRB, 0.30642E+00_JPRB, 0.30434E+00_JPRB, 0.30246E+00_JPRB, 0.30073E+00_JPRB ,&
&  0.30014E+00_JPRB, 0.29939E+00_JPRB, 0.29843E+00_JPRB, 0.29719E+00_JPRB, 0.29614E+00_JPRB, 0.29500E+00_JPRB, 0.29414E+00_JPRB ,&
&  0.29415E+00_JPRB, 0.29358E+00_JPRB, 0.29298E+00_JPRB, 0.29216E+00_JPRB, 0.29184E+00_JPRB, 0.29090E+00_JPRB, 0.28974E+00_JPRB ,&
&  0.28871E+00_JPRB, 0.28777E+00_JPRB, 0.28766E+00_JPRB, 0.28751E+00_JPRB, 0.28734E+00_JPRB, 0.28717E+00_JPRB, 0.28769E+00_JPRB ,&
&  0.28834E+00_JPRB, 0.28916E+00_JPRB, 0.29046E+00_JPRB, 0.29190E+00_JPRB, 0.29319E+00_JPRB, 0.29447E+00_JPRB, 0.29629E+00_JPRB ,&
&  0.29850E+00_JPRB, 0.30073E+00_JPRB, 0.30325E+00_JPRB, 0.30527E+00_JPRB, 0.30911E+00_JPRB, 0.31328E+00_JPRB, 0.31795E+00_JPRB ,&
&  0.32347E+00_JPRB, 0.33121E+00_JPRB, 0.34045E+00_JPRB, 0.34893E+00_JPRB, 0.35895E+00_JPRB, 0.36812E+00_JPRB, 0.37562E+00_JPRB ,&
&  0.38206E+00_JPRB, 0.38880E+00_JPRB, 0.39311E+00_JPRB, 0.39822E+00_JPRB, 0.40362E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.36780E+00_JPRB, 0.37113E+00_JPRB, 0.37364E+00_JPRB, 0.37516E+00_JPRB, 0.37450E+00_JPRB, 0.37553E+00_JPRB, 0.37522E+00_JPRB ,&
&  0.37465E+00_JPRB, 0.37393E+00_JPRB, 0.37458E+00_JPRB, 0.37534E+00_JPRB, 0.37578E+00_JPRB, 0.37609E+00_JPRB, 0.37614E+00_JPRB ,&
&  0.37549E+00_JPRB, 0.37435E+00_JPRB, 0.36909E+00_JPRB, 0.36613E+00_JPRB, 0.36422E+00_JPRB, 0.36311E+00_JPRB, 0.36240E+00_JPRB ,&
&  0.36112E+00_JPRB, 0.35974E+00_JPRB, 0.35772E+00_JPRB, 0.35758E+00_JPRB, 0.35678E+00_JPRB, 0.35571E+00_JPRB, 0.35410E+00_JPRB ,&
&  0.35249E+00_JPRB, 0.35124E+00_JPRB, 0.34975E+00_JPRB, 0.34821E+00_JPRB, 0.34647E+00_JPRB, 0.34431E+00_JPRB, 0.34140E+00_JPRB ,&
&  0.33843E+00_JPRB, 0.33545E+00_JPRB, 0.33298E+00_JPRB, 0.33062E+00_JPRB, 0.32858E+00_JPRB, 0.32688E+00_JPRB, 0.32556E+00_JPRB ,&
&  0.32409E+00_JPRB, 0.32238E+00_JPRB, 0.32064E+00_JPRB, 0.31917E+00_JPRB, 0.31773E+00_JPRB, 0.31647E+00_JPRB, 0.31577E+00_JPRB ,&
&  0.31537E+00_JPRB, 0.31490E+00_JPRB, 0.31432E+00_JPRB, 0.31374E+00_JPRB, 0.31306E+00_JPRB, 0.31243E+00_JPRB, 0.31179E+00_JPRB ,&
&  0.31123E+00_JPRB, 0.31097E+00_JPRB, 0.31056E+00_JPRB, 0.31014E+00_JPRB, 0.30988E+00_JPRB, 0.30943E+00_JPRB, 0.30894E+00_JPRB ,&
&  0.30840E+00_JPRB, 0.30819E+00_JPRB, 0.30792E+00_JPRB, 0.30756E+00_JPRB, 0.30721E+00_JPRB, 0.30698E+00_JPRB, 0.30703E+00_JPRB ,&
&  0.30703E+00_JPRB, 0.30695E+00_JPRB, 0.30704E+00_JPRB, 0.30793E+00_JPRB, 0.30888E+00_JPRB, 0.31004E+00_JPRB, 0.31131E+00_JPRB ,&
&  0.31294E+00_JPRB, 0.31465E+00_JPRB, 0.31660E+00_JPRB, 0.31845E+00_JPRB, 0.32100E+00_JPRB, 0.32352E+00_JPRB, 0.32606E+00_JPRB ,&
&  0.32849E+00_JPRB, 0.33104E+00_JPRB, 0.33366E+00_JPRB, 0.33641E+00_JPRB, 0.34017E+00_JPRB, 0.34304E+00_JPRB, 0.34628E+00_JPRB ,&
&  0.34940E+00_JPRB, 0.35302E+00_JPRB, 0.35654E+00_JPRB, 0.35968E+00_JPRB, 0.36271E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB ,&
&  0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB, 0.33361E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB, 0.60095E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.61013E+00_JPRB, 0.61007E+00_JPRB, 0.61010E+00_JPRB, 0.61028E+00_JPRB, 0.61010E+00_JPRB, 0.60965E+00_JPRB, 0.60890E+00_JPRB ,&
&  0.60805E+00_JPRB, 0.60708E+00_JPRB, 0.60573E+00_JPRB, 0.60433E+00_JPRB, 0.60295E+00_JPRB, 0.60156E+00_JPRB, 0.59975E+00_JPRB ,&
&  0.59778E+00_JPRB, 0.59552E+00_JPRB, 0.59314E+00_JPRB, 0.59081E+00_JPRB, 0.58909E+00_JPRB, 0.58771E+00_JPRB, 0.58678E+00_JPRB ,&
&  0.58583E+00_JPRB, 0.58503E+00_JPRB, 0.58427E+00_JPRB, 0.58373E+00_JPRB, 0.58352E+00_JPRB, 0.58316E+00_JPRB, 0.58287E+00_JPRB ,&
&  0.58263E+00_JPRB, 0.58261E+00_JPRB, 0.58239E+00_JPRB, 0.58216E+00_JPRB, 0.58189E+00_JPRB, 0.58209E+00_JPRB, 0.58212E+00_JPRB ,&
&  0.58237E+00_JPRB, 0.58248E+00_JPRB, 0.58255E+00_JPRB, 0.58249E+00_JPRB, 0.58258E+00_JPRB, 0.58268E+00_JPRB, 0.58271E+00_JPRB ,&
&  0.58283E+00_JPRB, 0.58330E+00_JPRB, 0.58441E+00_JPRB, 0.58588E+00_JPRB, 0.58733E+00_JPRB, 0.58895E+00_JPRB, 0.59061E+00_JPRB ,&
&  0.59233E+00_JPRB, 0.59384E+00_JPRB, 0.59564E+00_JPRB, 0.59741E+00_JPRB, 0.59946E+00_JPRB, 0.60137E+00_JPRB, 0.60341E+00_JPRB ,&
&  0.60532E+00_JPRB, 0.60742E+00_JPRB, 0.60981E+00_JPRB, 0.61230E+00_JPRB, 0.61465E+00_JPRB, 0.61673E+00_JPRB, 0.61895E+00_JPRB ,&
&  0.62130E+00_JPRB, 0.62355E+00_JPRB, 0.62525E+00_JPRB, 0.62709E+00_JPRB, 0.62899E+00_JPRB, 0.63089E+00_JPRB, 0.63226E+00_JPRB ,&
&  0.63367E+00_JPRB, 0.63524E+00_JPRB, 0.63680E+00_JPRB, 0.63728E+00_JPRB, 0.63762E+00_JPRB, 0.63776E+00_JPRB, 0.63752E+00_JPRB ,&
&  0.63651E+00_JPRB, 0.63536E+00_JPRB, 0.63417E+00_JPRB, 0.63281E+00_JPRB, 0.63106E+00_JPRB, 0.62910E+00_JPRB, 0.62687E+00_JPRB ,&
&  0.62453E+00_JPRB, 0.62176E+00_JPRB, 0.61912E+00_JPRB, 0.61717E+00_JPRB, 0.61591E+00_JPRB, 0.61447E+00_JPRB, 0.61297E+00_JPRB ,&
&  0.61159E+00_JPRB, 0.61110E+00_JPRB, 0.61083E+00_JPRB, 0.61054E+00_JPRB, 0.61039E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.61291E+00_JPRB, 0.61338E+00_JPRB, 0.61409E+00_JPRB, 0.61439E+00_JPRB, 0.61330E+00_JPRB, 0.61226E+00_JPRB, 0.61108E+00_JPRB ,&
&  0.61024E+00_JPRB, 0.60942E+00_JPRB, 0.60795E+00_JPRB, 0.60601E+00_JPRB, 0.60406E+00_JPRB, 0.60217E+00_JPRB, 0.60022E+00_JPRB ,&
&  0.59840E+00_JPRB, 0.59646E+00_JPRB, 0.59425E+00_JPRB, 0.59022E+00_JPRB, 0.58636E+00_JPRB, 0.58317E+00_JPRB, 0.58174E+00_JPRB ,&
&  0.58013E+00_JPRB, 0.57855E+00_JPRB, 0.57632E+00_JPRB, 0.57366E+00_JPRB, 0.57103E+00_JPRB, 0.56865E+00_JPRB, 0.56622E+00_JPRB ,&
&  0.56413E+00_JPRB, 0.56233E+00_JPRB, 0.56092E+00_JPRB, 0.55925E+00_JPRB, 0.55996E+00_JPRB, 0.56070E+00_JPRB, 0.56131E+00_JPRB ,&
&  0.56186E+00_JPRB, 0.56178E+00_JPRB, 0.56297E+00_JPRB, 0.56421E+00_JPRB, 0.56538E+00_JPRB, 0.56743E+00_JPRB, 0.56862E+00_JPRB ,&
&  0.56883E+00_JPRB, 0.56932E+00_JPRB, 0.57227E+00_JPRB, 0.57456E+00_JPRB, 0.57643E+00_JPRB, 0.57809E+00_JPRB, 0.58080E+00_JPRB ,&
&  0.58354E+00_JPRB, 0.58631E+00_JPRB, 0.58939E+00_JPRB, 0.59334E+00_JPRB, 0.59720E+00_JPRB, 0.60157E+00_JPRB, 0.60607E+00_JPRB ,&
&  0.61031E+00_JPRB, 0.61476E+00_JPRB, 0.61929E+00_JPRB, 0.62372E+00_JPRB, 0.62777E+00_JPRB, 0.63127E+00_JPRB, 0.63529E+00_JPRB ,&
&  0.63957E+00_JPRB, 0.64453E+00_JPRB, 0.64768E+00_JPRB, 0.65104E+00_JPRB, 0.65473E+00_JPRB, 0.65975E+00_JPRB, 0.66184E+00_JPRB ,&
&  0.66376E+00_JPRB, 0.66553E+00_JPRB, 0.66745E+00_JPRB, 0.66837E+00_JPRB, 0.66928E+00_JPRB, 0.67015E+00_JPRB, 0.67072E+00_JPRB ,&
&  0.67053E+00_JPRB, 0.66943E+00_JPRB, 0.66737E+00_JPRB, 0.66427E+00_JPRB, 0.66058E+00_JPRB, 0.65679E+00_JPRB, 0.65224E+00_JPRB ,&
&  0.64812E+00_JPRB, 0.64305E+00_JPRB, 0.63815E+00_JPRB, 0.63456E+00_JPRB, 0.63211E+00_JPRB, 0.62805E+00_JPRB, 0.62374E+00_JPRB ,&
&  0.61986E+00_JPRB, 0.61692E+00_JPRB, 0.61542E+00_JPRB, 0.61412E+00_JPRB, 0.61347E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.61728E+00_JPRB, 0.61786E+00_JPRB, 0.61819E+00_JPRB, 0.61795E+00_JPRB, 0.61619E+00_JPRB, 0.61464E+00_JPRB, 0.61291E+00_JPRB ,&
&  0.61149E+00_JPRB, 0.60958E+00_JPRB, 0.60801E+00_JPRB, 0.60661E+00_JPRB, 0.60535E+00_JPRB, 0.60453E+00_JPRB, 0.60369E+00_JPRB ,&
&  0.60317E+00_JPRB, 0.60225E+00_JPRB, 0.60156E+00_JPRB, 0.59979E+00_JPRB, 0.59633E+00_JPRB, 0.59085E+00_JPRB, 0.58474E+00_JPRB ,&
&  0.57893E+00_JPRB, 0.57364E+00_JPRB, 0.56795E+00_JPRB, 0.56245E+00_JPRB, 0.55778E+00_JPRB, 0.52788E+00_JPRB, 0.51938E+00_JPRB ,&
&  0.52129E+00_JPRB, 0.52299E+00_JPRB, 0.52721E+00_JPRB, 0.53257E+00_JPRB, 0.53452E+00_JPRB, 0.52650E+00_JPRB, 0.52255E+00_JPRB ,&
&  0.50114E+00_JPRB, 0.50387E+00_JPRB, 0.50638E+00_JPRB, 0.50936E+00_JPRB, 0.51554E+00_JPRB, 0.52064E+00_JPRB, 0.52977E+00_JPRB ,&
&  0.55042E+00_JPRB, 0.55409E+00_JPRB, 0.55854E+00_JPRB, 0.56370E+00_JPRB, 0.56796E+00_JPRB, 0.57176E+00_JPRB, 0.57630E+00_JPRB ,&
&  0.57994E+00_JPRB, 0.58376E+00_JPRB, 0.58768E+00_JPRB, 0.59215E+00_JPRB, 0.59752E+00_JPRB, 0.60325E+00_JPRB, 0.60928E+00_JPRB ,&
&  0.61501E+00_JPRB, 0.62186E+00_JPRB, 0.62899E+00_JPRB, 0.63616E+00_JPRB, 0.64314E+00_JPRB, 0.65002E+00_JPRB, 0.65762E+00_JPRB ,&
&  0.66465E+00_JPRB, 0.67203E+00_JPRB, 0.67716E+00_JPRB, 0.68267E+00_JPRB, 0.68642E+00_JPRB, 0.69072E+00_JPRB, 0.69182E+00_JPRB ,&
&  0.69339E+00_JPRB, 0.69164E+00_JPRB, 0.69153E+00_JPRB, 0.69037E+00_JPRB, 0.68941E+00_JPRB, 0.68901E+00_JPRB, 0.68813E+00_JPRB ,&
&  0.68791E+00_JPRB, 0.68668E+00_JPRB, 0.68538E+00_JPRB, 0.68248E+00_JPRB, 0.67812E+00_JPRB, 0.67254E+00_JPRB, 0.66668E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.65927E+00_JPRB, 0.65650E+00_JPRB, 0.65419E+00_JPRB, 0.65009E+00_JPRB, 0.64450E+00_JPRB, 0.63822E+00_JPRB ,&
&  0.63283E+00_JPRB, 0.62763E+00_JPRB, 0.62342E+00_JPRB, 0.62002E+00_JPRB, 0.61816E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.62482E+00_JPRB, 0.62623E+00_JPRB, 0.62700E+00_JPRB, 0.62740E+00_JPRB, 0.62592E+00_JPRB, 0.62448E+00_JPRB, 0.62291E+00_JPRB ,&
&  0.62117E+00_JPRB, 0.61948E+00_JPRB, 0.61770E+00_JPRB, 0.61629E+00_JPRB, 0.61463E+00_JPRB, 0.61333E+00_JPRB, 0.61129E+00_JPRB ,&
&  0.61041E+00_JPRB, 0.60857E+00_JPRB, 0.60669E+00_JPRB, 0.60260E+00_JPRB, 0.59842E+00_JPRB, 0.59346E+00_JPRB, 0.58427E+00_JPRB ,&
&  0.57414E+00_JPRB, 0.56478E+00_JPRB, 0.54005E+00_JPRB, 0.53412E+00_JPRB, 0.52193E+00_JPRB, 0.51515E+00_JPRB, 0.51212E+00_JPRB ,&
&  0.50675E+00_JPRB, 0.50285E+00_JPRB, 0.49238E+00_JPRB, 0.48319E+00_JPRB, 0.47674E+00_JPRB, 0.46562E+00_JPRB, 0.46018E+00_JPRB ,&
&  0.45322E+00_JPRB, 0.44994E+00_JPRB, 0.45137E+00_JPRB, 0.45534E+00_JPRB, 0.46595E+00_JPRB, 0.47791E+00_JPRB, 0.49413E+00_JPRB ,&
&  0.51685E+00_JPRB, 0.53730E+00_JPRB, 0.55661E+00_JPRB, 0.55856E+00_JPRB, 0.56588E+00_JPRB, 0.57221E+00_JPRB, 0.57682E+00_JPRB ,&
&  0.57387E+00_JPRB, 0.57402E+00_JPRB, 0.58487E+00_JPRB, 0.58920E+00_JPRB, 0.59452E+00_JPRB, 0.60075E+00_JPRB, 0.60722E+00_JPRB ,&
&  0.61422E+00_JPRB, 0.62202E+00_JPRB, 0.63024E+00_JPRB, 0.64012E+00_JPRB, 0.65233E+00_JPRB, 0.66465E+00_JPRB, 0.67489E+00_JPRB ,&
&  0.68395E+00_JPRB, 0.69505E+00_JPRB, 0.70309E+00_JPRB, 0.71064E+00_JPRB, 0.71779E+00_JPRB, 0.72787E+00_JPRB, 0.73587E+00_JPRB ,&
&  0.74041E+00_JPRB, 0.73783E+00_JPRB, 0.73118E+00_JPRB, 0.71780E+00_JPRB, 0.71123E+00_JPRB, 0.71679E+00_JPRB, 0.71421E+00_JPRB ,&
&  0.71322E+00_JPRB, 0.70995E+00_JPRB, 0.70743E+00_JPRB, 0.70312E+00_JPRB, 0.69848E+00_JPRB, 0.69160E+00_JPRB, 0.68324E+00_JPRB ,&
&  0.67635E+00_JPRB, 0.67305E+00_JPRB, 0.67007E+00_JPRB, 0.66693E+00_JPRB, 0.66037E+00_JPRB, 0.65349E+00_JPRB, 0.64673E+00_JPRB ,&
&  0.64073E+00_JPRB, 0.63527E+00_JPRB, 0.63011E+00_JPRB, 0.62679E+00_JPRB, 0.62508E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.64085E+00_JPRB, 0.64245E+00_JPRB, 0.64196E+00_JPRB, 0.64156E+00_JPRB, 0.63957E+00_JPRB, 0.63781E+00_JPRB, 0.63509E+00_JPRB ,&
&  0.63253E+00_JPRB, 0.62997E+00_JPRB, 0.62755E+00_JPRB, 0.62540E+00_JPRB, 0.62320E+00_JPRB, 0.62106E+00_JPRB, 0.61966E+00_JPRB ,&
&  0.61818E+00_JPRB, 0.61569E+00_JPRB, 0.61169E+00_JPRB, 0.60663E+00_JPRB, 0.60224E+00_JPRB, 0.59740E+00_JPRB, 0.59113E+00_JPRB ,&
&  0.58438E+00_JPRB, 0.57445E+00_JPRB, 0.55812E+00_JPRB, 0.55142E+00_JPRB, 0.54177E+00_JPRB, 0.53832E+00_JPRB, 0.53215E+00_JPRB ,&
&  0.52876E+00_JPRB, 0.49326E+00_JPRB, 0.49328E+00_JPRB, 0.48010E+00_JPRB, 0.47080E+00_JPRB, 0.46360E+00_JPRB, 0.44788E+00_JPRB ,&
&  0.44095E+00_JPRB, 0.43184E+00_JPRB, 0.42812E+00_JPRB, 0.43188E+00_JPRB, 0.43911E+00_JPRB, 0.45396E+00_JPRB, 0.47319E+00_JPRB ,&
&  0.49572E+00_JPRB, 0.54340E+00_JPRB, 0.55547E+00_JPRB, 0.56406E+00_JPRB, 0.57285E+00_JPRB, 0.57954E+00_JPRB, 0.57151E+00_JPRB ,&
&  0.57008E+00_JPRB, 0.56848E+00_JPRB, 0.57240E+00_JPRB, 0.58066E+00_JPRB, 0.59081E+00_JPRB, 0.59469E+00_JPRB, 0.60045E+00_JPRB ,&
&  0.60591E+00_JPRB, 0.61480E+00_JPRB, 0.62436E+00_JPRB, 0.63845E+00_JPRB, 0.65898E+00_JPRB, 0.68025E+00_JPRB, 0.69843E+00_JPRB ,&
&  0.71312E+00_JPRB, 0.72150E+00_JPRB, 0.72523E+00_JPRB, 0.73359E+00_JPRB, 0.74593E+00_JPRB, 0.76395E+00_JPRB, 0.77536E+00_JPRB ,&
&  0.78235E+00_JPRB, 0.78856E+00_JPRB, 0.78658E+00_JPRB, 0.78043E+00_JPRB, 0.76653E+00_JPRB, 0.75292E+00_JPRB, 0.74623E+00_JPRB ,&
&  0.74910E+00_JPRB, 0.74434E+00_JPRB, 0.73667E+00_JPRB, 0.72499E+00_JPRB, 0.71742E+00_JPRB, 0.70758E+00_JPRB, 0.69746E+00_JPRB ,&
&  0.68883E+00_JPRB, 0.68396E+00_JPRB, 0.67795E+00_JPRB, 0.67295E+00_JPRB, 0.66454E+00_JPRB, 0.65781E+00_JPRB, 0.65267E+00_JPRB ,&
&  0.64753E+00_JPRB, 0.64390E+00_JPRB, 0.63992E+00_JPRB, 0.63874E+00_JPRB, 0.63984E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.65747E+00_JPRB, 0.65782E+00_JPRB, 0.65706E+00_JPRB, 0.65622E+00_JPRB, 0.65471E+00_JPRB, 0.65407E+00_JPRB, 0.65392E+00_JPRB ,&
&  0.65286E+00_JPRB, 0.65184E+00_JPRB, 0.65062E+00_JPRB, 0.64922E+00_JPRB, 0.64756E+00_JPRB, 0.64608E+00_JPRB, 0.64443E+00_JPRB ,&
&  0.64100E+00_JPRB, 0.63622E+00_JPRB, 0.62946E+00_JPRB, 0.62292E+00_JPRB, 0.61701E+00_JPRB, 0.61361E+00_JPRB, 0.60810E+00_JPRB ,&
&  0.60254E+00_JPRB, 0.59410E+00_JPRB, 0.58100E+00_JPRB, 0.57193E+00_JPRB, 0.56956E+00_JPRB, 0.56188E+00_JPRB, 0.56151E+00_JPRB ,&
&  0.55831E+00_JPRB, 0.55280E+00_JPRB, 0.53741E+00_JPRB, 0.53164E+00_JPRB, 0.52166E+00_JPRB, 0.47228E+00_JPRB, 0.46157E+00_JPRB ,&
&  0.44682E+00_JPRB, 0.42956E+00_JPRB, 0.42285E+00_JPRB, 0.42056E+00_JPRB, 0.42399E+00_JPRB, 0.44027E+00_JPRB, 0.46169E+00_JPRB ,&
&  0.48309E+00_JPRB, 0.54281E+00_JPRB, 0.55839E+00_JPRB, 0.57128E+00_JPRB, 0.58041E+00_JPRB, 0.58529E+00_JPRB, 0.58130E+00_JPRB ,&
&  0.57890E+00_JPRB, 0.57844E+00_JPRB, 0.58433E+00_JPRB, 0.59473E+00_JPRB, 0.60226E+00_JPRB, 0.60063E+00_JPRB, 0.60039E+00_JPRB ,&
&  0.59624E+00_JPRB, 0.60230E+00_JPRB, 0.61497E+00_JPRB, 0.63291E+00_JPRB, 0.65728E+00_JPRB, 0.67587E+00_JPRB, 0.70105E+00_JPRB ,&
&  0.72280E+00_JPRB, 0.73732E+00_JPRB, 0.76059E+00_JPRB, 0.78377E+00_JPRB, 0.79984E+00_JPRB, 0.81007E+00_JPRB, 0.81220E+00_JPRB ,&
&  0.80947E+00_JPRB, 0.82723E+00_JPRB, 0.83643E+00_JPRB, 0.83314E+00_JPRB, 0.82365E+00_JPRB, 0.80547E+00_JPRB, 0.78682E+00_JPRB ,&
&  0.77829E+00_JPRB, 0.78170E+00_JPRB, 0.77370E+00_JPRB, 0.75202E+00_JPRB, 0.73482E+00_JPRB, 0.72058E+00_JPRB, 0.70914E+00_JPRB ,&
&  0.70053E+00_JPRB, 0.69537E+00_JPRB, 0.68792E+00_JPRB, 0.68139E+00_JPRB, 0.67387E+00_JPRB, 0.66722E+00_JPRB, 0.66347E+00_JPRB ,&
&  0.66057E+00_JPRB, 0.66026E+00_JPRB, 0.65931E+00_JPRB, 0.65844E+00_JPRB, 0.65892E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.67445E+00_JPRB, 0.67616E+00_JPRB, 0.67647E+00_JPRB, 0.67606E+00_JPRB, 0.67635E+00_JPRB, 0.67866E+00_JPRB, 0.67780E+00_JPRB ,&
&  0.67521E+00_JPRB, 0.67293E+00_JPRB, 0.66890E+00_JPRB, 0.66776E+00_JPRB, 0.66607E+00_JPRB, 0.66387E+00_JPRB, 0.66189E+00_JPRB ,&
&  0.65828E+00_JPRB, 0.64594E+00_JPRB, 0.63809E+00_JPRB, 0.63514E+00_JPRB, 0.63176E+00_JPRB, 0.63006E+00_JPRB, 0.62837E+00_JPRB ,&
&  0.62238E+00_JPRB, 0.61365E+00_JPRB, 0.60482E+00_JPRB, 0.59015E+00_JPRB, 0.57943E+00_JPRB, 0.58035E+00_JPRB, 0.58584E+00_JPRB ,&
&  0.57962E+00_JPRB, 0.56322E+00_JPRB, 0.56558E+00_JPRB, 0.56320E+00_JPRB, 0.54041E+00_JPRB, 0.53153E+00_JPRB, 0.49219E+00_JPRB ,&
&  0.46480E+00_JPRB, 0.43976E+00_JPRB, 0.43079E+00_JPRB, 0.42496E+00_JPRB, 0.43015E+00_JPRB, 0.44359E+00_JPRB, 0.47520E+00_JPRB ,&
&  0.50806E+00_JPRB, 0.54292E+00_JPRB, 0.55592E+00_JPRB, 0.56995E+00_JPRB, 0.57794E+00_JPRB, 0.59082E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.60707E+00_JPRB, 0.61185E+00_JPRB, 0.62019E+00_JPRB, 0.62742E+00_JPRB, 0.64283E+00_JPRB, 0.65541E+00_JPRB, 0.66108E+00_JPRB ,&
&  0.67124E+00_JPRB, 0.67402E+00_JPRB, 0.68463E+00_JPRB, 0.69778E+00_JPRB, 0.71456E+00_JPRB, 0.72357E+00_JPRB, 0.73408E+00_JPRB ,&
&  0.74664E+00_JPRB, 0.77572E+00_JPRB, 0.81791E+00_JPRB, 0.86985E+00_JPRB, 0.88449E+00_JPRB, 0.88012E+00_JPRB, 0.85807E+00_JPRB ,&
&  0.83168E+00_JPRB, 0.85397E+00_JPRB, 0.95900E+00_JPRB, 0.94532E+00_JPRB, 0.93303E+00_JPRB, 0.91500E+00_JPRB, 0.87338E+00_JPRB ,&
&  0.82484E+00_JPRB, 0.80550E+00_JPRB, 0.78754E+00_JPRB, 0.75652E+00_JPRB, 0.73194E+00_JPRB, 0.71508E+00_JPRB, 0.70582E+00_JPRB ,&
&  0.69997E+00_JPRB, 0.69916E+00_JPRB, 0.69419E+00_JPRB, 0.69069E+00_JPRB, 0.68387E+00_JPRB, 0.67666E+00_JPRB, 0.67379E+00_JPRB ,&
&  0.67151E+00_JPRB, 0.67246E+00_JPRB, 0.67449E+00_JPRB, 0.67564E+00_JPRB, 0.67572E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.69032E+00_JPRB, 0.69677E+00_JPRB, 0.70165E+00_JPRB, 0.71076E+00_JPRB, 0.71924E+00_JPRB, 0.72197E+00_JPRB, 0.71778E+00_JPRB ,&
&  0.70590E+00_JPRB, 0.69094E+00_JPRB, 0.67942E+00_JPRB, 0.67123E+00_JPRB, 0.66627E+00_JPRB, 0.66331E+00_JPRB, 0.66037E+00_JPRB ,&
&  0.65822E+00_JPRB, 0.65583E+00_JPRB, 0.64946E+00_JPRB, 0.64051E+00_JPRB, 0.63578E+00_JPRB, 0.63563E+00_JPRB, 0.63588E+00_JPRB ,&
&  0.63352E+00_JPRB, 0.62715E+00_JPRB, 0.61884E+00_JPRB, 0.60477E+00_JPRB, 0.59234E+00_JPRB, 0.59464E+00_JPRB, 0.58900E+00_JPRB ,&
&  0.58423E+00_JPRB, 0.58203E+00_JPRB, 0.57912E+00_JPRB, 0.57494E+00_JPRB, 0.56934E+00_JPRB, 0.55145E+00_JPRB, 0.51798E+00_JPRB ,&
&  0.48352E+00_JPRB, 0.45605E+00_JPRB, 0.43972E+00_JPRB, 0.43474E+00_JPRB, 0.44389E+00_JPRB, 0.46798E+00_JPRB, 0.50155E+00_JPRB ,&
&  0.53892E+00_JPRB, 0.55630E+00_JPRB, 0.57273E+00_JPRB, 0.59003E+00_JPRB, 0.60403E+00_JPRB, 0.62187E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.65614E+00_JPRB, 0.68101E+00_JPRB, 0.71449E+00_JPRB, 0.75823E+00_JPRB, 0.80979E+00_JPRB, 0.79883E+00_JPRB, 0.83991E+00_JPRB ,&
&  0.95768E+00_JPRB, 0.95989E+00_JPRB, 0.92270E+00_JPRB, 0.87985E+00_JPRB, 0.85336E+00_JPRB, 0.84426E+00_JPRB, 0.84808E+00_JPRB ,&
&  0.85647E+00_JPRB, 0.87502E+00_JPRB, 0.90691E+00_JPRB, 0.94518E+00_JPRB, 0.95725E+00_JPRB, 0.95124E+00_JPRB, 0.91924E+00_JPRB ,&
&  0.89053E+00_JPRB, 0.91398E+00_JPRB, 0.11284E+01_JPRB, 0.10377E+01_JPRB, 0.95632E+00_JPRB, 0.91687E+00_JPRB, 0.86874E+00_JPRB ,&
&  0.83346E+00_JPRB, 0.80582E+00_JPRB, 0.77605E+00_JPRB, 0.75180E+00_JPRB, 0.72397E+00_JPRB, 0.69813E+00_JPRB, 0.67560E+00_JPRB ,&
&  0.66822E+00_JPRB, 0.66730E+00_JPRB, 0.66812E+00_JPRB, 0.66940E+00_JPRB, 0.66661E+00_JPRB, 0.66722E+00_JPRB, 0.66990E+00_JPRB ,&
&  0.66696E+00_JPRB, 0.66767E+00_JPRB, 0.67352E+00_JPRB, 0.67956E+00_JPRB, 0.68478E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.67322E+00_JPRB, 0.69298E+00_JPRB, 0.72462E+00_JPRB, 0.74607E+00_JPRB, 0.75487E+00_JPRB, 0.74051E+00_JPRB, 0.72109E+00_JPRB ,&
&  0.69924E+00_JPRB, 0.67215E+00_JPRB, 0.65600E+00_JPRB, 0.64667E+00_JPRB, 0.64140E+00_JPRB, 0.63757E+00_JPRB, 0.63444E+00_JPRB ,&
&  0.63223E+00_JPRB, 0.63051E+00_JPRB, 0.63142E+00_JPRB, 0.63553E+00_JPRB, 0.63835E+00_JPRB, 0.63661E+00_JPRB, 0.63364E+00_JPRB ,&
&  0.63146E+00_JPRB, 0.62740E+00_JPRB, 0.61716E+00_JPRB, 0.60270E+00_JPRB, 0.59757E+00_JPRB, 0.60648E+00_JPRB, 0.60519E+00_JPRB ,&
&  0.59935E+00_JPRB, 0.59194E+00_JPRB, 0.59147E+00_JPRB, 0.59331E+00_JPRB, 0.58891E+00_JPRB, 0.57487E+00_JPRB, 0.54402E+00_JPRB ,&
&  0.50574E+00_JPRB, 0.47480E+00_JPRB, 0.46533E+00_JPRB, 0.47940E+00_JPRB, 0.50796E+00_JPRB, 0.54695E+00_JPRB, 0.56420E+00_JPRB ,&
&  0.57744E+00_JPRB, 0.58928E+00_JPRB, 0.60132E+00_JPRB, 0.62187E+00_JPRB, 0.64629E+00_JPRB, 0.66814E+00_JPRB, 0.68954E+00_JPRB ,&
&  0.72204E+00_JPRB, 0.75833E+00_JPRB, 0.80360E+00_JPRB, 0.77570E+00_JPRB, 0.80037E+00_JPRB, 0.86593E+00_JPRB, 0.90049E+00_JPRB ,&
&  0.91905E+00_JPRB, 0.95752E+00_JPRB, 0.98226E+00_JPRB, 0.10151E+01_JPRB, 0.10026E+01_JPRB, 0.98334E+00_JPRB, 0.96194E+00_JPRB ,&
&  0.95861E+00_JPRB, 0.95564E+00_JPRB, 0.96364E+00_JPRB, 0.10026E+01_JPRB, 0.10401E+01_JPRB, 0.10409E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.96812E+00_JPRB, 0.90030E+00_JPRB, 0.83717E+00_JPRB, 0.79605E+00_JPRB, 0.79445E+00_JPRB, 0.81063E+00_JPRB, 0.81853E+00_JPRB ,&
&  0.81487E+00_JPRB, 0.80824E+00_JPRB, 0.79040E+00_JPRB, 0.77471E+00_JPRB, 0.72330E+00_JPRB, 0.68619E+00_JPRB, 0.65706E+00_JPRB ,&
&  0.64602E+00_JPRB, 0.64043E+00_JPRB, 0.63946E+00_JPRB, 0.64749E+00_JPRB, 0.64570E+00_JPRB, 0.64927E+00_JPRB, 0.65313E+00_JPRB ,&
&  0.65246E+00_JPRB, 0.65026E+00_JPRB, 0.65302E+00_JPRB, 0.65739E+00_JPRB, 0.66346E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.74305E+00_JPRB, 0.76015E+00_JPRB, 0.77512E+00_JPRB, 0.78209E+00_JPRB, 0.77254E+00_JPRB, 0.76356E+00_JPRB, 0.75421E+00_JPRB ,&
&  0.74869E+00_JPRB, 0.69807E+00_JPRB, 0.66329E+00_JPRB, 0.63834E+00_JPRB, 0.62001E+00_JPRB, 0.60339E+00_JPRB, 0.59866E+00_JPRB ,&
&  0.60046E+00_JPRB, 0.60473E+00_JPRB, 0.60944E+00_JPRB, 0.61475E+00_JPRB, 0.61972E+00_JPRB, 0.62134E+00_JPRB, 0.62478E+00_JPRB ,&
&  0.63042E+00_JPRB, 0.63283E+00_JPRB, 0.62596E+00_JPRB, 0.62040E+00_JPRB, 0.61980E+00_JPRB, 0.62430E+00_JPRB, 0.62153E+00_JPRB ,&
&  0.61717E+00_JPRB, 0.61158E+00_JPRB, 0.60864E+00_JPRB, 0.61137E+00_JPRB, 0.61040E+00_JPRB, 0.59820E+00_JPRB, 0.56224E+00_JPRB ,&
&  0.51676E+00_JPRB, 0.50041E+00_JPRB, 0.51883E+00_JPRB, 0.56594E+00_JPRB, 0.58449E+00_JPRB, 0.59758E+00_JPRB, 0.61141E+00_JPRB ,&
&  0.60593E+00_JPRB, 0.60512E+00_JPRB, 0.62367E+00_JPRB, 0.66880E+00_JPRB, 0.68927E+00_JPRB, 0.70836E+00_JPRB, 0.73217E+00_JPRB ,&
&  0.77543E+00_JPRB, 0.80965E+00_JPRB, 0.84933E+00_JPRB, 0.81694E+00_JPRB, 0.89391E+00_JPRB, 0.10657E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.10446E+01_JPRB, 0.10933E+01_JPRB, 0.11231E+01_JPRB, 0.11208E+01_JPRB, 0.11023E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10753E+01_JPRB, 0.10694E+01_JPRB, 0.10815E+01_JPRB, 0.11349E+01_JPRB, 0.11339E+01_JPRB, 0.11146E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.96450E+00_JPRB, 0.89842E+00_JPRB, 0.84967E+00_JPRB, 0.83730E+00_JPRB, 0.84066E+00_JPRB, 0.83669E+00_JPRB ,&
&  0.82077E+00_JPRB, 0.81796E+00_JPRB, 0.81949E+00_JPRB, 0.81632E+00_JPRB, 0.76322E+00_JPRB, 0.70325E+00_JPRB, 0.65895E+00_JPRB ,&
&  0.62729E+00_JPRB, 0.60868E+00_JPRB, 0.59552E+00_JPRB, 0.60112E+00_JPRB, 0.61684E+00_JPRB, 0.63655E+00_JPRB, 0.63879E+00_JPRB ,&
&  0.64343E+00_JPRB, 0.64639E+00_JPRB, 0.65949E+00_JPRB, 0.68844E+00_JPRB, 0.71812E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.78508E+00_JPRB, 0.80381E+00_JPRB, 0.81787E+00_JPRB, 0.81015E+00_JPRB, 0.79396E+00_JPRB, 0.78231E+00_JPRB, 0.76899E+00_JPRB ,&
&  0.76162E+00_JPRB, 0.68431E+00_JPRB, 0.64910E+00_JPRB, 0.63100E+00_JPRB, 0.61810E+00_JPRB, 0.59419E+00_JPRB, 0.57560E+00_JPRB ,&
&  0.57382E+00_JPRB, 0.58512E+00_JPRB, 0.60253E+00_JPRB, 0.61520E+00_JPRB, 0.62678E+00_JPRB, 0.63521E+00_JPRB, 0.63908E+00_JPRB ,&
&  0.64344E+00_JPRB, 0.64811E+00_JPRB, 0.64668E+00_JPRB, 0.64435E+00_JPRB, 0.64425E+00_JPRB, 0.64952E+00_JPRB, 0.65661E+00_JPRB ,&
&  0.65911E+00_JPRB, 0.65640E+00_JPRB, 0.64352E+00_JPRB, 0.64000E+00_JPRB, 0.63855E+00_JPRB, 0.63972E+00_JPRB, 0.63335E+00_JPRB ,&
&  0.54720E+00_JPRB, 0.54771E+00_JPRB, 0.59610E+00_JPRB, 0.60285E+00_JPRB, 0.61652E+00_JPRB, 0.62198E+00_JPRB, 0.62544E+00_JPRB ,&
&  0.63365E+00_JPRB, 0.65160E+00_JPRB, 0.67920E+00_JPRB, 0.71171E+00_JPRB, 0.73715E+00_JPRB, 0.75702E+00_JPRB, 0.77874E+00_JPRB ,&
&  0.83572E+00_JPRB, 0.78421E+00_JPRB, 0.77139E+00_JPRB, 0.83887E+00_JPRB, 0.98933E+00_JPRB, 0.10872E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.12219E+01_JPRB, 0.12171E+01_JPRB, 0.12170E+01_JPRB, 0.12375E+01_JPRB, 0.12416E+01_JPRB, 0.12354E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.12684E+01_JPRB, 0.12933E+01_JPRB, 0.13132E+01_JPRB, 0.12656E+01_JPRB, 0.12281E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.10683E+01_JPRB, 0.99664E+00_JPRB, 0.93957E+00_JPRB, 0.90002E+00_JPRB, 0.89712E+00_JPRB, 0.89147E+00_JPRB ,&
&  0.87395E+00_JPRB, 0.86371E+00_JPRB, 0.84927E+00_JPRB, 0.83525E+00_JPRB, 0.79883E+00_JPRB, 0.76353E+00_JPRB, 0.74518E+00_JPRB ,&
&  0.69259E+00_JPRB, 0.65521E+00_JPRB, 0.61302E+00_JPRB, 0.61366E+00_JPRB, 0.64610E+00_JPRB, 0.66620E+00_JPRB, 0.67447E+00_JPRB ,&
&  0.69853E+00_JPRB, 0.71430E+00_JPRB, 0.72268E+00_JPRB, 0.74012E+00_JPRB, 0.76288E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.81073E+00_JPRB, 0.81699E+00_JPRB, 0.81770E+00_JPRB, 0.81113E+00_JPRB, 0.81278E+00_JPRB, 0.80810E+00_JPRB, 0.81038E+00_JPRB ,&
&  0.83471E+00_JPRB, 0.78706E+00_JPRB, 0.82375E+00_JPRB, 0.70647E+00_JPRB, 0.64513E+00_JPRB, 0.61219E+00_JPRB, 0.58969E+00_JPRB ,&
&  0.58738E+00_JPRB, 0.59939E+00_JPRB, 0.61133E+00_JPRB, 0.62396E+00_JPRB, 0.64534E+00_JPRB, 0.64941E+00_JPRB, 0.65074E+00_JPRB ,&
&  0.65666E+00_JPRB, 0.66335E+00_JPRB, 0.67538E+00_JPRB, 0.66975E+00_JPRB, 0.67408E+00_JPRB, 0.68335E+00_JPRB, 0.70908E+00_JPRB ,&
&  0.68540E+00_JPRB, 0.68100E+00_JPRB, 0.69734E+00_JPRB, 0.69734E+00_JPRB, 0.69710E+00_JPRB, 0.69170E+00_JPRB, 0.68645E+00_JPRB ,&
&  0.68548E+00_JPRB, 0.63796E+00_JPRB, 0.66013E+00_JPRB, 0.66474E+00_JPRB, 0.66708E+00_JPRB, 0.66987E+00_JPRB, 0.66843E+00_JPRB ,&
&  0.67308E+00_JPRB, 0.68777E+00_JPRB, 0.71252E+00_JPRB, 0.74026E+00_JPRB, 0.77418E+00_JPRB, 0.80710E+00_JPRB, 0.83507E+00_JPRB ,&
&  0.90456E+00_JPRB, 0.82638E+00_JPRB, 0.95247E+00_JPRB, 0.10201E+01_JPRB, 0.11475E+01_JPRB, 0.13158E+01_JPRB, 0.14031E+01_JPRB ,&
&  0.15762E+01_JPRB, 0.15370E+01_JPRB, 0.15235E+01_JPRB, 0.14943E+01_JPRB, 0.14540E+01_JPRB, 0.14337E+01_JPRB, 0.14338E+01_JPRB ,&
&  0.14789E+01_JPRB, 0.14938E+01_JPRB, 0.15942E+01_JPRB, 0.15533E+01_JPRB, 0.14432E+01_JPRB, 0.13722E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.11931E+01_JPRB, 0.11502E+01_JPRB, 0.10485E+01_JPRB, 0.10230E+01_JPRB, 0.10069E+01_JPRB, 0.94776E+00_JPRB ,&
&  0.91924E+00_JPRB, 0.87445E+00_JPRB, 0.82224E+00_JPRB, 0.82902E+00_JPRB, 0.79054E+00_JPRB, 0.76173E+00_JPRB, 0.75687E+00_JPRB ,&
&  0.73323E+00_JPRB, 0.68278E+00_JPRB, 0.68429E+00_JPRB, 0.70724E+00_JPRB, 0.70548E+00_JPRB, 0.70151E+00_JPRB, 0.72856E+00_JPRB ,&
&  0.74348E+00_JPRB, 0.75912E+00_JPRB, 0.79546E+00_JPRB, 0.80253E+00_JPRB, 0.81180E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.82588E+00_JPRB, 0.81236E+00_JPRB, 0.80502E+00_JPRB, 0.80734E+00_JPRB, 0.82587E+00_JPRB, 0.85691E+00_JPRB, 0.84752E+00_JPRB ,&
&  0.86816E+00_JPRB, 0.87529E+00_JPRB, 0.86316E+00_JPRB, 0.83993E+00_JPRB, 0.80342E+00_JPRB, 0.71303E+00_JPRB, 0.61261E+00_JPRB ,&
&  0.58586E+00_JPRB, 0.60212E+00_JPRB, 0.62375E+00_JPRB, 0.63372E+00_JPRB, 0.65047E+00_JPRB, 0.65282E+00_JPRB, 0.65601E+00_JPRB ,&
&  0.66353E+00_JPRB, 0.67321E+00_JPRB, 0.68577E+00_JPRB, 0.69814E+00_JPRB, 0.69999E+00_JPRB, 0.72483E+00_JPRB, 0.76321E+00_JPRB ,&
&  0.72145E+00_JPRB, 0.69558E+00_JPRB, 0.69112E+00_JPRB, 0.67813E+00_JPRB, 0.71193E+00_JPRB, 0.72550E+00_JPRB, 0.73172E+00_JPRB ,&
&  0.73422E+00_JPRB, 0.73253E+00_JPRB, 0.72229E+00_JPRB, 0.71820E+00_JPRB, 0.71718E+00_JPRB, 0.72419E+00_JPRB, 0.73463E+00_JPRB ,&
&  0.75329E+00_JPRB, 0.77453E+00_JPRB, 0.80277E+00_JPRB, 0.84304E+00_JPRB, 0.89166E+00_JPRB, 0.91034E+00_JPRB, 0.97563E+00_JPRB ,&
&  0.10406E+01_JPRB, 0.11616E+01_JPRB, 0.13046E+01_JPRB, 0.12797E+01_JPRB, 0.14189E+01_JPRB, 0.15511E+01_JPRB, 0.15798E+01_JPRB ,&
&  0.16765E+01_JPRB, 0.16436E+01_JPRB, 0.17064E+01_JPRB, 0.17592E+01_JPRB, 0.17994E+01_JPRB, 0.17886E+01_JPRB, 0.17256E+01_JPRB ,&
&  0.17083E+01_JPRB, 0.18683E+01_JPRB, 0.18929E+01_JPRB, 0.18024E+01_JPRB, 0.16444E+01_JPRB, 0.15418E+01_JPRB, 0.14265E+01_JPRB ,&
&  0.13864E+01_JPRB, 0.13902E+01_JPRB, 0.13444E+01_JPRB, 0.12945E+01_JPRB, 0.12068E+01_JPRB, 0.11124E+01_JPRB, 0.10393E+01_JPRB ,&
&  0.95687E+00_JPRB, 0.84768E+00_JPRB, 0.79503E+00_JPRB, 0.78346E+00_JPRB, 0.77755E+00_JPRB, 0.74325E+00_JPRB, 0.74411E+00_JPRB ,&
&  0.74964E+00_JPRB, 0.74264E+00_JPRB, 0.77720E+00_JPRB, 0.79861E+00_JPRB, 0.78937E+00_JPRB, 0.77195E+00_JPRB, 0.74558E+00_JPRB ,&
&  0.73114E+00_JPRB, 0.77250E+00_JPRB, 0.81351E+00_JPRB, 0.83357E+00_JPRB, 0.83519E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.84133E+00_JPRB, 0.82340E+00_JPRB, 0.79965E+00_JPRB, 0.80350E+00_JPRB, 0.83988E+00_JPRB, 0.87496E+00_JPRB, 0.89562E+00_JPRB ,&
&  0.89051E+00_JPRB, 0.88995E+00_JPRB, 0.87775E+00_JPRB, 0.86209E+00_JPRB, 0.83510E+00_JPRB, 0.80252E+00_JPRB, 0.72739E+00_JPRB ,&
&  0.63887E+00_JPRB, 0.65052E+00_JPRB, 0.62753E+00_JPRB, 0.63542E+00_JPRB, 0.66877E+00_JPRB, 0.66729E+00_JPRB, 0.66943E+00_JPRB ,&
&  0.68015E+00_JPRB, 0.68986E+00_JPRB, 0.70463E+00_JPRB, 0.71173E+00_JPRB, 0.72164E+00_JPRB, 0.78692E+00_JPRB, 0.78244E+00_JPRB ,&
&  0.72109E+00_JPRB, 0.68566E+00_JPRB, 0.68466E+00_JPRB, 0.69880E+00_JPRB, 0.72391E+00_JPRB, 0.75055E+00_JPRB, 0.75631E+00_JPRB ,&
&  0.75551E+00_JPRB, 0.75042E+00_JPRB, 0.73566E+00_JPRB, 0.73459E+00_JPRB, 0.74420E+00_JPRB, 0.76360E+00_JPRB, 0.78299E+00_JPRB ,&
&  0.79951E+00_JPRB, 0.82450E+00_JPRB, 0.86328E+00_JPRB, 0.92149E+00_JPRB, 0.97340E+00_JPRB, 0.10688E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.12573E+01_JPRB, 0.13588E+01_JPRB, 0.14600E+01_JPRB, 0.15890E+01_JPRB, 0.17613E+01_JPRB, 0.18112E+01_JPRB, 0.18276E+01_JPRB ,&
&  0.18733E+01_JPRB, 0.18347E+01_JPRB, 0.20936E+01_JPRB, 0.21410E+01_JPRB, 0.21129E+01_JPRB, 0.21233E+01_JPRB, 0.20088E+01_JPRB ,&
&  0.19207E+01_JPRB, 0.18712E+01_JPRB, 0.18421E+01_JPRB, 0.17889E+01_JPRB, 0.17240E+01_JPRB, 0.17231E+01_JPRB, 0.16170E+01_JPRB ,&
&  0.16140E+01_JPRB, 0.15783E+01_JPRB, 0.13619E+01_JPRB, 0.11991E+01_JPRB, 0.10465E+01_JPRB, 0.10611E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.95105E+00_JPRB, 0.82413E+00_JPRB, 0.81521E+00_JPRB, 0.80109E+00_JPRB, 0.80838E+00_JPRB, 0.83691E+00_JPRB, 0.83972E+00_JPRB ,&
&  0.85182E+00_JPRB, 0.88234E+00_JPRB, 0.89404E+00_JPRB, 0.94150E+00_JPRB, 0.92066E+00_JPRB, 0.87531E+00_JPRB, 0.77338E+00_JPRB ,&
&  0.74480E+00_JPRB, 0.77758E+00_JPRB, 0.81206E+00_JPRB, 0.83790E+00_JPRB, 0.84756E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.87470E+00_JPRB, 0.85943E+00_JPRB, 0.83685E+00_JPRB, 0.83747E+00_JPRB, 0.87028E+00_JPRB, 0.90578E+00_JPRB, 0.92535E+00_JPRB ,&
&  0.91827E+00_JPRB, 0.91726E+00_JPRB, 0.89705E+00_JPRB, 0.87975E+00_JPRB, 0.85999E+00_JPRB, 0.82743E+00_JPRB, 0.81793E+00_JPRB ,&
&  0.69238E+00_JPRB, 0.68920E+00_JPRB, 0.62282E+00_JPRB, 0.62095E+00_JPRB, 0.69791E+00_JPRB, 0.69462E+00_JPRB, 0.70317E+00_JPRB ,&
&  0.70135E+00_JPRB, 0.71855E+00_JPRB, 0.72195E+00_JPRB, 0.72753E+00_JPRB, 0.77414E+00_JPRB, 0.82713E+00_JPRB, 0.79918E+00_JPRB ,&
&  0.75359E+00_JPRB, 0.72385E+00_JPRB, 0.72668E+00_JPRB, 0.73144E+00_JPRB, 0.73997E+00_JPRB, 0.75884E+00_JPRB, 0.76811E+00_JPRB ,&
&  0.76490E+00_JPRB, 0.77618E+00_JPRB, 0.78162E+00_JPRB, 0.79898E+00_JPRB, 0.82372E+00_JPRB, 0.86007E+00_JPRB, 0.88901E+00_JPRB ,&
&  0.92194E+00_JPRB, 0.96502E+00_JPRB, 0.10127E+01_JPRB, 0.10816E+01_JPRB, 0.11293E+01_JPRB, 0.12614E+01_JPRB, 0.14377E+01_JPRB ,&
&  0.16835E+01_JPRB, 0.17687E+01_JPRB, 0.18713E+01_JPRB, 0.21976E+01_JPRB, 0.23076E+01_JPRB, 0.22740E+01_JPRB, 0.21360E+01_JPRB ,&
&  0.20858E+01_JPRB, 0.20392E+01_JPRB, 0.20808E+01_JPRB, 0.21609E+01_JPRB, 0.21589E+01_JPRB, 0.22263E+01_JPRB, 0.21310E+01_JPRB ,&
&  0.19239E+01_JPRB, 0.18296E+01_JPRB, 0.17823E+01_JPRB, 0.17819E+01_JPRB, 0.17061E+01_JPRB, 0.17236E+01_JPRB, 0.16551E+01_JPRB ,&
&  0.16698E+01_JPRB, 0.14720E+01_JPRB, 0.11358E+01_JPRB, 0.10492E+01_JPRB, 0.87722E+00_JPRB, 0.89838E+00_JPRB, 0.10817E+01_JPRB ,&
&  0.10522E+01_JPRB, 0.95650E+00_JPRB, 0.88931E+00_JPRB, 0.85791E+00_JPRB, 0.87289E+00_JPRB, 0.10689E+01_JPRB, 0.11280E+01_JPRB ,&
&  0.10309E+01_JPRB, 0.10658E+01_JPRB, 0.11242E+01_JPRB, 0.11693E+01_JPRB, 0.11362E+01_JPRB, 0.10765E+01_JPRB, 0.96398E+00_JPRB ,&
&  0.92807E+00_JPRB, 0.91089E+00_JPRB, 0.90950E+00_JPRB, 0.90239E+00_JPRB, 0.88923E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.95696E+00_JPRB, 0.93300E+00_JPRB, 0.90885E+00_JPRB, 0.90694E+00_JPRB, 0.92835E+00_JPRB, 0.95861E+00_JPRB, 0.97581E+00_JPRB ,&
&  0.96370E+00_JPRB, 0.95474E+00_JPRB, 0.92883E+00_JPRB, 0.90596E+00_JPRB, 0.88394E+00_JPRB, 0.85603E+00_JPRB, 0.83803E+00_JPRB ,&
&  0.83912E+00_JPRB, 0.78623E+00_JPRB, 0.67911E+00_JPRB, 0.62058E+00_JPRB, 0.66511E+00_JPRB, 0.68306E+00_JPRB, 0.71524E+00_JPRB ,&
&  0.71229E+00_JPRB, 0.75220E+00_JPRB, 0.75590E+00_JPRB, 0.77193E+00_JPRB, 0.85138E+00_JPRB, 0.86851E+00_JPRB, 0.85368E+00_JPRB ,&
&  0.84849E+00_JPRB, 0.81630E+00_JPRB, 0.80534E+00_JPRB, 0.81471E+00_JPRB, 0.82819E+00_JPRB, 0.78950E+00_JPRB, 0.80681E+00_JPRB ,&
&  0.82461E+00_JPRB, 0.84685E+00_JPRB, 0.86339E+00_JPRB, 0.89882E+00_JPRB, 0.94856E+00_JPRB, 0.99300E+00_JPRB, 0.10428E+01_JPRB ,&
&  0.10923E+01_JPRB, 0.11275E+01_JPRB, 0.11446E+01_JPRB, 0.11960E+01_JPRB, 0.12815E+01_JPRB, 0.14347E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.16648E+01_JPRB, 0.18017E+01_JPRB, 0.18556E+01_JPRB, 0.22371E+01_JPRB, 0.24535E+01_JPRB, 0.24386E+01_JPRB, 0.23680E+01_JPRB ,&
&  0.23289E+01_JPRB, 0.22796E+01_JPRB, 0.23616E+01_JPRB, 0.23887E+01_JPRB, 0.24020E+01_JPRB, 0.23725E+01_JPRB, 0.22032E+01_JPRB ,&
&  0.19874E+01_JPRB, 0.18431E+01_JPRB, 0.17676E+01_JPRB, 0.16839E+01_JPRB, 0.16080E+01_JPRB, 0.15375E+01_JPRB, 0.15769E+01_JPRB ,&
&  0.17008E+01_JPRB, 0.10495E+01_JPRB, 0.87365E+00_JPRB, 0.98436E+00_JPRB, 0.86284E+00_JPRB, 0.88896E+00_JPRB, 0.98686E+00_JPRB ,&
&  0.97345E+00_JPRB, 0.96243E+00_JPRB, 0.99606E+00_JPRB, 0.10116E+01_JPRB, 0.11055E+01_JPRB, 0.14254E+01_JPRB, 0.15090E+01_JPRB ,&
&  0.14234E+01_JPRB, 0.13276E+01_JPRB, 0.14012E+01_JPRB, 0.13953E+01_JPRB, 0.13692E+01_JPRB, 0.13308E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.11708E+01_JPRB, 0.11237E+01_JPRB, 0.10709E+01_JPRB, 0.10210E+01_JPRB, 0.98421E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.10950E+01_JPRB, 0.10664E+01_JPRB, 0.10440E+01_JPRB, 0.10299E+01_JPRB, 0.10329E+01_JPRB, 0.10536E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10181E+01_JPRB, 0.98025E+00_JPRB, 0.94725E+00_JPRB, 0.91898E+00_JPRB, 0.89812E+00_JPRB, 0.87856E+00_JPRB ,&
&  0.85551E+00_JPRB, 0.83581E+00_JPRB, 0.70089E+00_JPRB, 0.64504E+00_JPRB, 0.61402E+00_JPRB, 0.65556E+00_JPRB, 0.69841E+00_JPRB ,&
&  0.72453E+00_JPRB, 0.75638E+00_JPRB, 0.78585E+00_JPRB, 0.88916E+00_JPRB, 0.10030E+01_JPRB, 0.97827E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.99474E+00_JPRB, 0.91677E+00_JPRB, 0.92948E+00_JPRB, 0.88483E+00_JPRB, 0.84741E+00_JPRB, 0.92131E+00_JPRB ,&
&  0.95124E+00_JPRB, 0.98005E+00_JPRB, 0.10063E+01_JPRB, 0.10344E+01_JPRB, 0.10717E+01_JPRB, 0.10880E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.12131E+01_JPRB, 0.12587E+01_JPRB, 0.12889E+01_JPRB, 0.13570E+01_JPRB, 0.14531E+01_JPRB, 0.14707E+01_JPRB ,&
&  0.15783E+01_JPRB, 0.15027E+01_JPRB, 0.12800E+01_JPRB, 0.16918E+01_JPRB, 0.24202E+01_JPRB, 0.25174E+01_JPRB, 0.23586E+01_JPRB ,&
&  0.25124E+01_JPRB, 0.25445E+01_JPRB, 0.26846E+01_JPRB, 0.28132E+01_JPRB, 0.27492E+01_JPRB, 0.26045E+01_JPRB, 0.24041E+01_JPRB ,&
&  0.21496E+01_JPRB, 0.19770E+01_JPRB, 0.18810E+01_JPRB, 0.19471E+01_JPRB, 0.18620E+01_JPRB, 0.18456E+01_JPRB, 0.18318E+01_JPRB ,&
&  0.16341E+01_JPRB, 0.14145E+01_JPRB, 0.10530E+01_JPRB, 0.10780E+01_JPRB, 0.89306E+00_JPRB, 0.91168E+00_JPRB, 0.10145E+01_JPRB ,&
&  0.99604E+00_JPRB, 0.10757E+01_JPRB, 0.11592E+01_JPRB, 0.12102E+01_JPRB, 0.15116E+01_JPRB, 0.18056E+01_JPRB, 0.20719E+01_JPRB ,&
&  0.20319E+01_JPRB, 0.17263E+01_JPRB, 0.17891E+01_JPRB, 0.17309E+01_JPRB, 0.17201E+01_JPRB, 0.16730E+01_JPRB, 0.15957E+01_JPRB ,&
&  0.14939E+01_JPRB, 0.13780E+01_JPRB, 0.12750E+01_JPRB, 0.11903E+01_JPRB, 0.11331E+01_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.12035E+01_JPRB, 0.11603E+01_JPRB, 0.11327E+01_JPRB, 0.11190E+01_JPRB, 0.11219E+01_JPRB, 0.11388E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.10692E+01_JPRB, 0.10233E+01_JPRB, 0.98244E+00_JPRB, 0.95100E+00_JPRB, 0.93090E+00_JPRB, 0.91321E+00_JPRB ,&
&  0.87871E+00_JPRB, 0.79556E+00_JPRB, 0.66602E+00_JPRB, 0.60733E+00_JPRB, 0.58554E+00_JPRB, 0.61030E+00_JPRB, 0.67536E+00_JPRB ,&
&  0.73829E+00_JPRB, 0.79085E+00_JPRB, 0.88636E+00_JPRB, 0.10381E+01_JPRB, 0.11537E+01_JPRB, 0.11885E+01_JPRB, 0.12348E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.11429E+01_JPRB, 0.10992E+01_JPRB, 0.10440E+01_JPRB, 0.10503E+01_JPRB, 0.10656E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10382E+01_JPRB, 0.10522E+01_JPRB, 0.10867E+01_JPRB, 0.11269E+01_JPRB, 0.11863E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.14152E+01_JPRB, 0.13658E+01_JPRB, 0.13407E+01_JPRB, 0.14306E+01_JPRB, 0.15053E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.13960E+01_JPRB, 0.13639E+01_JPRB, 0.16283E+01_JPRB, 0.19597E+01_JPRB, 0.23383E+01_JPRB, 0.26158E+01_JPRB, 0.26801E+01_JPRB ,&
&  0.27821E+01_JPRB, 0.27423E+01_JPRB, 0.28020E+01_JPRB, 0.25606E+01_JPRB, 0.27822E+01_JPRB, 0.27105E+01_JPRB, 0.25784E+01_JPRB ,&
&  0.24736E+01_JPRB, 0.24199E+01_JPRB, 0.22973E+01_JPRB, 0.23202E+01_JPRB, 0.24372E+01_JPRB, 0.25031E+01_JPRB, 0.20060E+01_JPRB ,&
&  0.16293E+01_JPRB, 0.16981E+01_JPRB, 0.14532E+01_JPRB, 0.12789E+01_JPRB, 0.11491E+01_JPRB, 0.11730E+01_JPRB, 0.11933E+01_JPRB ,&
&  0.12275E+01_JPRB, 0.13016E+01_JPRB, 0.13285E+01_JPRB, 0.15067E+01_JPRB, 0.18443E+01_JPRB, 0.21783E+01_JPRB, 0.24103E+01_JPRB ,&
&  0.22152E+01_JPRB, 0.23283E+01_JPRB, 0.22552E+01_JPRB, 0.21656E+01_JPRB, 0.21568E+01_JPRB, 0.20926E+01_JPRB, 0.19692E+01_JPRB ,&
&  0.17946E+01_JPRB, 0.16282E+01_JPRB, 0.14878E+01_JPRB, 0.13679E+01_JPRB, 0.12719E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.14281E+01_JPRB, 0.13655E+01_JPRB, 0.13207E+01_JPRB, 0.12942E+01_JPRB, 0.12905E+01_JPRB, 0.12963E+01_JPRB, 0.12648E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.11407E+01_JPRB, 0.10883E+01_JPRB, 0.10419E+01_JPRB, 0.99752E+00_JPRB, 0.96582E+00_JPRB, 0.94305E+00_JPRB ,&
&  0.92125E+00_JPRB, 0.78325E+00_JPRB, 0.64213E+00_JPRB, 0.62287E+00_JPRB, 0.62152E+00_JPRB, 0.60253E+00_JPRB, 0.65273E+00_JPRB ,&
&  0.73639E+00_JPRB, 0.85605E+00_JPRB, 0.98924E+00_JPRB, 0.12197E+01_JPRB, 0.13498E+01_JPRB, 0.14412E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.14313E+01_JPRB, 0.13649E+01_JPRB, 0.13048E+01_JPRB, 0.12258E+01_JPRB, 0.11398E+01_JPRB, 0.11218E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10830E+01_JPRB, 0.10870E+01_JPRB, 0.11223E+01_JPRB, 0.11950E+01_JPRB, 0.12815E+01_JPRB, 0.13875E+01_JPRB, 0.15196E+01_JPRB ,&
&  0.15370E+01_JPRB, 0.14841E+01_JPRB, 0.13610E+01_JPRB, 0.13683E+01_JPRB, 0.13134E+01_JPRB, 0.13319E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.16139E+01_JPRB, 0.17253E+01_JPRB, 0.17137E+01_JPRB, 0.20687E+01_JPRB, 0.20237E+01_JPRB, 0.22310E+01_JPRB, 0.27890E+01_JPRB ,&
&  0.29157E+01_JPRB, 0.29113E+01_JPRB, 0.29841E+01_JPRB, 0.23931E+01_JPRB, 0.20861E+01_JPRB, 0.28648E+01_JPRB, 0.29736E+01_JPRB ,&
&  0.30233E+01_JPRB, 0.31340E+01_JPRB, 0.31003E+01_JPRB, 0.32432E+01_JPRB, 0.27121E+01_JPRB, 0.22583E+01_JPRB, 0.17030E+01_JPRB ,&
&  0.16230E+01_JPRB, 0.15385E+01_JPRB, 0.16754E+01_JPRB, 0.15332E+01_JPRB, 0.15619E+01_JPRB, 0.14387E+01_JPRB, 0.13813E+01_JPRB ,&
&  0.14800E+01_JPRB, 0.15455E+01_JPRB, 0.16835E+01_JPRB, 0.19707E+01_JPRB, 0.23737E+01_JPRB, 0.23527E+01_JPRB, 0.26763E+01_JPRB ,&
&  0.27848E+01_JPRB, 0.27307E+01_JPRB, 0.24137E+01_JPRB, 0.24353E+01_JPRB, 0.23439E+01_JPRB, 0.22508E+01_JPRB, 0.21325E+01_JPRB ,&
&  0.19813E+01_JPRB, 0.18364E+01_JPRB, 0.17149E+01_JPRB, 0.16040E+01_JPRB, 0.15089E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.16406E+01_JPRB, 0.15809E+01_JPRB, 0.15317E+01_JPRB, 0.14998E+01_JPRB, 0.14785E+01_JPRB, 0.14396E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.12610E+01_JPRB, 0.11914E+01_JPRB, 0.11258E+01_JPRB, 0.10718E+01_JPRB, 0.10147E+01_JPRB, 0.98669E+00_JPRB, 0.97827E+00_JPRB ,&
&  0.96133E+00_JPRB, 0.87744E+00_JPRB, 0.69867E+00_JPRB, 0.64664E+00_JPRB, 0.66470E+00_JPRB, 0.63981E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.78562E+00_JPRB, 0.96881E+00_JPRB, 0.11652E+01_JPRB, 0.14631E+01_JPRB, 0.15966E+01_JPRB, 0.17356E+01_JPRB, 0.17692E+01_JPRB ,&
&  0.17894E+01_JPRB, 0.15478E+01_JPRB, 0.13637E+01_JPRB, 0.12276E+01_JPRB, 0.11339E+01_JPRB, 0.10875E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.11522E+01_JPRB, 0.12410E+01_JPRB, 0.12906E+01_JPRB, 0.13787E+01_JPRB, 0.14582E+01_JPRB, 0.15313E+01_JPRB ,&
&  0.15312E+01_JPRB, 0.14816E+01_JPRB, 0.14453E+01_JPRB, 0.15201E+01_JPRB, 0.15224E+01_JPRB, 0.14416E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.16593E+01_JPRB, 0.17449E+01_JPRB, 0.19797E+01_JPRB, 0.23037E+01_JPRB, 0.23824E+01_JPRB, 0.27469E+01_JPRB ,&
&  0.22861E+01_JPRB, 0.20359E+01_JPRB, 0.17883E+01_JPRB, 0.15749E+01_JPRB, 0.20109E+01_JPRB, 0.33060E+01_JPRB, 0.34966E+01_JPRB ,&
&  0.34920E+01_JPRB, 0.36943E+01_JPRB, 0.37593E+01_JPRB, 0.32899E+01_JPRB, 0.26870E+01_JPRB, 0.21773E+01_JPRB, 0.26215E+01_JPRB ,&
&  0.26772E+01_JPRB, 0.29730E+01_JPRB, 0.26632E+01_JPRB, 0.20864E+01_JPRB, 0.17808E+01_JPRB, 0.20616E+01_JPRB, 0.20528E+01_JPRB ,&
&  0.20729E+01_JPRB, 0.21503E+01_JPRB, 0.28456E+01_JPRB, 0.29266E+01_JPRB, 0.28825E+01_JPRB, 0.29815E+01_JPRB, 0.32653E+01_JPRB ,&
&  0.30305E+01_JPRB, 0.29424E+01_JPRB, 0.28000E+01_JPRB, 0.26127E+01_JPRB, 0.24631E+01_JPRB, 0.23213E+01_JPRB, 0.21870E+01_JPRB ,&
&  0.20760E+01_JPRB, 0.19710E+01_JPRB, 0.18596E+01_JPRB, 0.17760E+01_JPRB, 0.17026E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.17819E+01_JPRB, 0.17259E+01_JPRB, 0.16812E+01_JPRB, 0.16318E+01_JPRB, 0.15733E+01_JPRB, 0.14911E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.12229E+01_JPRB, 0.11589E+01_JPRB, 0.11060E+01_JPRB, 0.10493E+01_JPRB, 0.10152E+01_JPRB, 0.10003E+01_JPRB ,&
&  0.99124E+00_JPRB, 0.10286E+01_JPRB, 0.81329E+00_JPRB, 0.71086E+00_JPRB, 0.71094E+00_JPRB, 0.71980E+00_JPRB, 0.70241E+00_JPRB ,&
&  0.89258E+00_JPRB, 0.11309E+01_JPRB, 0.13024E+01_JPRB, 0.16288E+01_JPRB, 0.19660E+01_JPRB, 0.18617E+01_JPRB, 0.17980E+01_JPRB ,&
&  0.17525E+01_JPRB, 0.15443E+01_JPRB, 0.13767E+01_JPRB, 0.12741E+01_JPRB, 0.12034E+01_JPRB, 0.11850E+01_JPRB, 0.11975E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.13334E+01_JPRB, 0.13514E+01_JPRB, 0.13874E+01_JPRB, 0.14397E+01_JPRB, 0.15371E+01_JPRB, 0.16101E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.15195E+01_JPRB, 0.15487E+01_JPRB, 0.16807E+01_JPRB, 0.17068E+01_JPRB, 0.16207E+01_JPRB, 0.17906E+01_JPRB ,&
&  0.17718E+01_JPRB, 0.19102E+01_JPRB, 0.18948E+01_JPRB, 0.21118E+01_JPRB, 0.22822E+01_JPRB, 0.23025E+01_JPRB, 0.24775E+01_JPRB ,&
&  0.19618E+01_JPRB, 0.17784E+01_JPRB, 0.18634E+01_JPRB, 0.20388E+01_JPRB, 0.22291E+01_JPRB, 0.27790E+01_JPRB, 0.41859E+01_JPRB ,&
&  0.34943E+01_JPRB, 0.37385E+01_JPRB, 0.37606E+01_JPRB, 0.36451E+01_JPRB, 0.20468E+01_JPRB, 0.17215E+01_JPRB, 0.28965E+01_JPRB ,&
&  0.32722E+01_JPRB, 0.20990E+01_JPRB, 0.14731E+01_JPRB, 0.17014E+01_JPRB, 0.13374E+01_JPRB, 0.17210E+01_JPRB, 0.24791E+01_JPRB ,&
&  0.27996E+01_JPRB, 0.30244E+01_JPRB, 0.40263E+01_JPRB, 0.36342E+01_JPRB, 0.34209E+01_JPRB, 0.42608E+01_JPRB, 0.36775E+01_JPRB ,&
&  0.33834E+01_JPRB, 0.28986E+01_JPRB, 0.28630E+01_JPRB, 0.25672E+01_JPRB, 0.23803E+01_JPRB, 0.22639E+01_JPRB, 0.21961E+01_JPRB ,&
&  0.21530E+01_JPRB, 0.20992E+01_JPRB, 0.20257E+01_JPRB, 0.19389E+01_JPRB, 0.18485E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.18333E+01_JPRB, 0.17904E+01_JPRB, 0.17541E+01_JPRB, 0.16970E+01_JPRB, 0.16176E+01_JPRB, 0.15190E+01_JPRB, 0.14191E+01_JPRB ,&
&  0.13473E+01_JPRB, 0.12680E+01_JPRB, 0.12021E+01_JPRB, 0.11474E+01_JPRB, 0.10918E+01_JPRB, 0.10483E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.99744E+00_JPRB, 0.99603E+00_JPRB, 0.96796E+00_JPRB, 0.87967E+00_JPRB, 0.86559E+00_JPRB, 0.91369E+00_JPRB, 0.93965E+00_JPRB ,&
&  0.11549E+01_JPRB, 0.13761E+01_JPRB, 0.15666E+01_JPRB, 0.17983E+01_JPRB, 0.19327E+01_JPRB, 0.18211E+01_JPRB, 0.18453E+01_JPRB ,&
&  0.17472E+01_JPRB, 0.15844E+01_JPRB, 0.14492E+01_JPRB, 0.13846E+01_JPRB, 0.13582E+01_JPRB, 0.13610E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.13981E+01_JPRB, 0.14149E+01_JPRB, 0.13936E+01_JPRB, 0.13868E+01_JPRB, 0.14743E+01_JPRB, 0.16019E+01_JPRB, 0.16568E+01_JPRB ,&
&  0.16062E+01_JPRB, 0.16012E+01_JPRB, 0.16410E+01_JPRB, 0.17620E+01_JPRB, 0.19456E+01_JPRB, 0.19346E+01_JPRB, 0.16861E+01_JPRB ,&
&  0.15331E+01_JPRB, 0.16462E+01_JPRB, 0.20485E+01_JPRB, 0.20451E+01_JPRB, 0.21081E+01_JPRB, 0.22143E+01_JPRB, 0.22983E+01_JPRB ,&
&  0.22384E+01_JPRB, 0.22592E+01_JPRB, 0.22483E+01_JPRB, 0.25580E+01_JPRB, 0.28627E+01_JPRB, 0.28696E+01_JPRB, 0.39973E+01_JPRB ,&
&  0.40392E+01_JPRB, 0.37195E+01_JPRB, 0.29120E+01_JPRB, 0.22744E+01_JPRB, 0.23552E+01_JPRB, 0.19919E+01_JPRB, 0.14109E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.13007E+01_JPRB, 0.11539E+01_JPRB, 0.11300E+01_JPRB, 0.10694E+01_JPRB, 0.15572E+01_JPRB, 0.25488E+01_JPRB ,&
&  0.37607E+01_JPRB, 0.44213E+01_JPRB, 0.47760E+01_JPRB, 0.43883E+01_JPRB, 0.39166E+01_JPRB, 0.46869E+01_JPRB, 0.40608E+01_JPRB ,&
&  0.30851E+01_JPRB, 0.27543E+01_JPRB, 0.26814E+01_JPRB, 0.24866E+01_JPRB, 0.23905E+01_JPRB, 0.23065E+01_JPRB, 0.22320E+01_JPRB ,&
&  0.21578E+01_JPRB, 0.20970E+01_JPRB, 0.20323E+01_JPRB, 0.19574E+01_JPRB, 0.18915E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.18485E+01_JPRB, 0.18129E+01_JPRB, 0.17669E+01_JPRB, 0.17003E+01_JPRB, 0.16138E+01_JPRB, 0.15266E+01_JPRB, 0.14537E+01_JPRB ,&
&  0.14130E+01_JPRB, 0.13483E+01_JPRB, 0.12758E+01_JPRB, 0.12093E+01_JPRB, 0.11439E+01_JPRB, 0.10878E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10168E+01_JPRB, 0.10668E+01_JPRB, 0.10611E+01_JPRB, 0.11726E+01_JPRB, 0.13539E+01_JPRB, 0.14787E+01_JPRB ,&
&  0.16892E+01_JPRB, 0.18523E+01_JPRB, 0.20807E+01_JPRB, 0.21481E+01_JPRB, 0.22039E+01_JPRB, 0.21735E+01_JPRB, 0.20565E+01_JPRB ,&
&  0.18846E+01_JPRB, 0.17893E+01_JPRB, 0.17009E+01_JPRB, 0.16375E+01_JPRB, 0.15704E+01_JPRB, 0.15120E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.14538E+01_JPRB, 0.14646E+01_JPRB, 0.14144E+01_JPRB, 0.14294E+01_JPRB, 0.15252E+01_JPRB, 0.16286E+01_JPRB, 0.16901E+01_JPRB ,&
&  0.17055E+01_JPRB, 0.17105E+01_JPRB, 0.17661E+01_JPRB, 0.18715E+01_JPRB, 0.17540E+01_JPRB, 0.15128E+01_JPRB, 0.16464E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.19944E+01_JPRB, 0.19686E+01_JPRB, 0.22576E+01_JPRB, 0.22230E+01_JPRB, 0.21471E+01_JPRB, 0.23396E+01_JPRB ,&
&  0.24957E+01_JPRB, 0.30019E+01_JPRB, 0.25819E+01_JPRB, 0.26182E+01_JPRB, 0.36484E+01_JPRB, 0.34386E+01_JPRB, 0.36937E+01_JPRB ,&
&  0.40279E+01_JPRB, 0.34196E+01_JPRB, 0.29169E+01_JPRB, 0.26875E+01_JPRB, 0.39034E+01_JPRB, 0.40743E+01_JPRB, 0.19391E+01_JPRB ,&
&  0.15553E+01_JPRB, 0.13654E+01_JPRB, 0.12399E+01_JPRB, 0.11165E+01_JPRB, 0.10851E+01_JPRB, 0.16832E+01_JPRB, 0.36494E+01_JPRB ,&
&  0.40958E+01_JPRB, 0.58715E+01_JPRB, 0.59371E+01_JPRB, 0.53533E+01_JPRB, 0.44510E+01_JPRB, 0.44853E+01_JPRB, 0.35260E+01_JPRB ,&
&  0.30667E+01_JPRB, 0.28539E+01_JPRB, 0.26409E+01_JPRB, 0.24596E+01_JPRB, 0.22715E+01_JPRB, 0.21603E+01_JPRB, 0.21085E+01_JPRB ,&
&  0.20622E+01_JPRB, 0.20161E+01_JPRB, 0.19752E+01_JPRB, 0.19296E+01_JPRB, 0.18893E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.17729E+01_JPRB, 0.17262E+01_JPRB, 0.16777E+01_JPRB, 0.16134E+01_JPRB, 0.15539E+01_JPRB, 0.15099E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.14600E+01_JPRB, 0.14050E+01_JPRB, 0.13331E+01_JPRB, 0.12523E+01_JPRB, 0.11771E+01_JPRB, 0.11158E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.10598E+01_JPRB, 0.10575E+01_JPRB, 0.10944E+01_JPRB, 0.12597E+01_JPRB, 0.15133E+01_JPRB, 0.16946E+01_JPRB, 0.20278E+01_JPRB ,&
&  0.24257E+01_JPRB, 0.26425E+01_JPRB, 0.29064E+01_JPRB, 0.28986E+01_JPRB, 0.27883E+01_JPRB, 0.26214E+01_JPRB, 0.24084E+01_JPRB ,&
&  0.22377E+01_JPRB, 0.21331E+01_JPRB, 0.19982E+01_JPRB, 0.18583E+01_JPRB, 0.17268E+01_JPRB, 0.16270E+01_JPRB, 0.15577E+01_JPRB ,&
&  0.15613E+01_JPRB, 0.15039E+01_JPRB, 0.14486E+01_JPRB, 0.14672E+01_JPRB, 0.15528E+01_JPRB, 0.16750E+01_JPRB, 0.17581E+01_JPRB ,&
&  0.17840E+01_JPRB, 0.17756E+01_JPRB, 0.18158E+01_JPRB, 0.18906E+01_JPRB, 0.15859E+01_JPRB, 0.17685E+01_JPRB, 0.17810E+01_JPRB ,&
&  0.18086E+01_JPRB, 0.18862E+01_JPRB, 0.18613E+01_JPRB, 0.21170E+01_JPRB, 0.22172E+01_JPRB, 0.21433E+01_JPRB, 0.22687E+01_JPRB ,&
&  0.27659E+01_JPRB, 0.26322E+01_JPRB, 0.25094E+01_JPRB, 0.29186E+01_JPRB, 0.36194E+01_JPRB, 0.50098E+01_JPRB, 0.33775E+01_JPRB ,&
&  0.29332E+01_JPRB, 0.33784E+01_JPRB, 0.31869E+01_JPRB, 0.30530E+01_JPRB, 0.39819E+01_JPRB, 0.50128E+01_JPRB, 0.43085E+01_JPRB ,&
&  0.18327E+01_JPRB, 0.13334E+01_JPRB, 0.14065E+01_JPRB, 0.14654E+01_JPRB, 0.14161E+01_JPRB, 0.20786E+01_JPRB, 0.59270E+01_JPRB ,&
&  0.52715E+01_JPRB, 0.67450E+01_JPRB, 0.62558E+01_JPRB, 0.53453E+01_JPRB, 0.45564E+01_JPRB, 0.41997E+01_JPRB, 0.36127E+01_JPRB ,&
&  0.30261E+01_JPRB, 0.26854E+01_JPRB, 0.24089E+01_JPRB, 0.22154E+01_JPRB, 0.21037E+01_JPRB, 0.20193E+01_JPRB, 0.19652E+01_JPRB ,&
&  0.19177E+01_JPRB, 0.18801E+01_JPRB, 0.18561E+01_JPRB, 0.18392E+01_JPRB, 0.18143E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.16748E+01_JPRB, 0.16175E+01_JPRB, 0.15604E+01_JPRB, 0.15273E+01_JPRB, 0.15068E+01_JPRB, 0.14784E+01_JPRB, 0.14579E+01_JPRB ,&
&  0.14418E+01_JPRB, 0.13848E+01_JPRB, 0.13236E+01_JPRB, 0.12489E+01_JPRB, 0.11775E+01_JPRB, 0.11259E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.10923E+01_JPRB, 0.10925E+01_JPRB, 0.11223E+01_JPRB, 0.13201E+01_JPRB, 0.17098E+01_JPRB, 0.18260E+01_JPRB, 0.22305E+01_JPRB ,&
&  0.35419E+01_JPRB, 0.39833E+01_JPRB, 0.37328E+01_JPRB, 0.35667E+01_JPRB, 0.34110E+01_JPRB, 0.32836E+01_JPRB, 0.29854E+01_JPRB ,&
&  0.26643E+01_JPRB, 0.23746E+01_JPRB, 0.21198E+01_JPRB, 0.19252E+01_JPRB, 0.17718E+01_JPRB, 0.16671E+01_JPRB, 0.16199E+01_JPRB ,&
&  0.15759E+01_JPRB, 0.15159E+01_JPRB, 0.14652E+01_JPRB, 0.14795E+01_JPRB, 0.15887E+01_JPRB, 0.17069E+01_JPRB, 0.17983E+01_JPRB ,&
&  0.18167E+01_JPRB, 0.18064E+01_JPRB, 0.18072E+01_JPRB, 0.17579E+01_JPRB, 0.17622E+01_JPRB, 0.17874E+01_JPRB, 0.18383E+01_JPRB ,&
&  0.18308E+01_JPRB, 0.19232E+01_JPRB, 0.20287E+01_JPRB, 0.20685E+01_JPRB, 0.21248E+01_JPRB, 0.21871E+01_JPRB, 0.23303E+01_JPRB ,&
&  0.26508E+01_JPRB, 0.26522E+01_JPRB, 0.27500E+01_JPRB, 0.34236E+01_JPRB, 0.38937E+01_JPRB, 0.44545E+01_JPRB, 0.42755E+01_JPRB ,&
&  0.36032E+01_JPRB, 0.28563E+01_JPRB, 0.28116E+01_JPRB, 0.33089E+01_JPRB, 0.38217E+01_JPRB, 0.43390E+01_JPRB, 0.65648E+01_JPRB ,&
&  0.65400E+01_JPRB, 0.43262E+01_JPRB, 0.35864E+01_JPRB, 0.47030E+01_JPRB, 0.39913E+01_JPRB, 0.36275E+01_JPRB, 0.47352E+01_JPRB ,&
&  0.57816E+01_JPRB, 0.69345E+01_JPRB, 0.62523E+01_JPRB, 0.50254E+01_JPRB, 0.45673E+01_JPRB, 0.40415E+01_JPRB, 0.33814E+01_JPRB ,&
&  0.28073E+01_JPRB, 0.25039E+01_JPRB, 0.22982E+01_JPRB, 0.20985E+01_JPRB, 0.19386E+01_JPRB, 0.18495E+01_JPRB, 0.18063E+01_JPRB ,&
&  0.17710E+01_JPRB, 0.17535E+01_JPRB, 0.17473E+01_JPRB, 0.17454E+01_JPRB, 0.17175E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.15437E+01_JPRB, 0.15026E+01_JPRB, 0.14830E+01_JPRB, 0.14466E+01_JPRB, 0.14077E+01_JPRB, 0.13774E+01_JPRB, 0.13688E+01_JPRB ,&
&  0.13574E+01_JPRB, 0.13136E+01_JPRB, 0.12628E+01_JPRB, 0.12053E+01_JPRB, 0.11440E+01_JPRB, 0.11090E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.11184E+01_JPRB, 0.12066E+01_JPRB, 0.14615E+01_JPRB, 0.18770E+01_JPRB, 0.25479E+01_JPRB, 0.23809E+01_JPRB ,&
&  0.36460E+01_JPRB, 0.48901E+01_JPRB, 0.45958E+01_JPRB, 0.43081E+01_JPRB, 0.40122E+01_JPRB, 0.38244E+01_JPRB, 0.32975E+01_JPRB ,&
&  0.28039E+01_JPRB, 0.24554E+01_JPRB, 0.21353E+01_JPRB, 0.19093E+01_JPRB, 0.17680E+01_JPRB, 0.16851E+01_JPRB, 0.16118E+01_JPRB ,&
&  0.15608E+01_JPRB, 0.14974E+01_JPRB, 0.14561E+01_JPRB, 0.14831E+01_JPRB, 0.15690E+01_JPRB, 0.16982E+01_JPRB, 0.17810E+01_JPRB ,&
&  0.18146E+01_JPRB, 0.18076E+01_JPRB, 0.17874E+01_JPRB, 0.16880E+01_JPRB, 0.17253E+01_JPRB, 0.17684E+01_JPRB, 0.18259E+01_JPRB ,&
&  0.17765E+01_JPRB, 0.17027E+01_JPRB, 0.18917E+01_JPRB, 0.20745E+01_JPRB, 0.21485E+01_JPRB, 0.21284E+01_JPRB, 0.22375E+01_JPRB ,&
&  0.26718E+01_JPRB, 0.30536E+01_JPRB, 0.37307E+01_JPRB, 0.39469E+01_JPRB, 0.41410E+01_JPRB, 0.43019E+01_JPRB, 0.43826E+01_JPRB ,&
&  0.44301E+01_JPRB, 0.37352E+01_JPRB, 0.34962E+01_JPRB, 0.35590E+01_JPRB, 0.35831E+01_JPRB, 0.40481E+01_JPRB, 0.57547E+01_JPRB ,&
&  0.74455E+01_JPRB, 0.85387E+01_JPRB, 0.87903E+01_JPRB, 0.73790E+01_JPRB, 0.62307E+01_JPRB, 0.53418E+01_JPRB, 0.51133E+01_JPRB ,&
&  0.63960E+01_JPRB, 0.65841E+01_JPRB, 0.57466E+01_JPRB, 0.58239E+01_JPRB, 0.43815E+01_JPRB, 0.38847E+01_JPRB, 0.32492E+01_JPRB ,&
&  0.27821E+01_JPRB, 0.24347E+01_JPRB, 0.21650E+01_JPRB, 0.19663E+01_JPRB, 0.18281E+01_JPRB, 0.17155E+01_JPRB, 0.16470E+01_JPRB ,&
&  0.16141E+01_JPRB, 0.16218E+01_JPRB, 0.16354E+01_JPRB, 0.16306E+01_JPRB, 0.15903E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.14655E+01_JPRB, 0.14457E+01_JPRB, 0.14199E+01_JPRB, 0.13713E+01_JPRB, 0.13353E+01_JPRB, 0.13127E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.12389E+01_JPRB, 0.11959E+01_JPRB, 0.11463E+01_JPRB, 0.11064E+01_JPRB, 0.10812E+01_JPRB, 0.10849E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.12628E+01_JPRB, 0.14851E+01_JPRB, 0.17505E+01_JPRB, 0.21986E+01_JPRB, 0.28835E+01_JPRB, 0.33557E+01_JPRB ,&
&  0.38654E+01_JPRB, 0.59820E+01_JPRB, 0.49566E+01_JPRB, 0.44980E+01_JPRB, 0.41784E+01_JPRB, 0.38802E+01_JPRB, 0.32452E+01_JPRB ,&
&  0.25496E+01_JPRB, 0.22468E+01_JPRB, 0.20527E+01_JPRB, 0.18912E+01_JPRB, 0.17771E+01_JPRB, 0.16712E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.15518E+01_JPRB, 0.15119E+01_JPRB, 0.14787E+01_JPRB, 0.15162E+01_JPRB, 0.16113E+01_JPRB, 0.17085E+01_JPRB, 0.17937E+01_JPRB ,&
&  0.18233E+01_JPRB, 0.18036E+01_JPRB, 0.17300E+01_JPRB, 0.16871E+01_JPRB, 0.16804E+01_JPRB, 0.17378E+01_JPRB, 0.18117E+01_JPRB ,&
&  0.18163E+01_JPRB, 0.17612E+01_JPRB, 0.18718E+01_JPRB, 0.19254E+01_JPRB, 0.19626E+01_JPRB, 0.21452E+01_JPRB, 0.22489E+01_JPRB ,&
&  0.27303E+01_JPRB, 0.28101E+01_JPRB, 0.39525E+01_JPRB, 0.36888E+01_JPRB, 0.37174E+01_JPRB, 0.39429E+01_JPRB, 0.37841E+01_JPRB ,&
&  0.33427E+01_JPRB, 0.32868E+01_JPRB, 0.31713E+01_JPRB, 0.33176E+01_JPRB, 0.37371E+01_JPRB, 0.45179E+01_JPRB, 0.60144E+01_JPRB ,&
&  0.66321E+01_JPRB, 0.64683E+01_JPRB, 0.65224E+01_JPRB, 0.54319E+01_JPRB, 0.51999E+01_JPRB, 0.49012E+01_JPRB, 0.49277E+01_JPRB ,&
&  0.61443E+01_JPRB, 0.60808E+01_JPRB, 0.56053E+01_JPRB, 0.50518E+01_JPRB, 0.38223E+01_JPRB, 0.32891E+01_JPRB, 0.27473E+01_JPRB ,&
&  0.23942E+01_JPRB, 0.21007E+01_JPRB, 0.19369E+01_JPRB, 0.18038E+01_JPRB, 0.17023E+01_JPRB, 0.15987E+01_JPRB, 0.15513E+01_JPRB ,&
&  0.15320E+01_JPRB, 0.15324E+01_JPRB, 0.15371E+01_JPRB, 0.15204E+01_JPRB, 0.14907E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.14515E+01_JPRB, 0.14532E+01_JPRB, 0.14483E+01_JPRB, 0.14650E+01_JPRB, 0.15063E+01_JPRB, 0.15708E+01_JPRB, 0.16033E+01_JPRB ,&
&  0.14572E+01_JPRB, 0.11695E+01_JPRB, 0.11430E+01_JPRB, 0.11114E+01_JPRB, 0.10785E+01_JPRB, 0.10823E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.14581E+01_JPRB, 0.16605E+01_JPRB, 0.20097E+01_JPRB, 0.25321E+01_JPRB, 0.33984E+01_JPRB, 0.43869E+01_JPRB ,&
&  0.47045E+01_JPRB, 0.51259E+01_JPRB, 0.49768E+01_JPRB, 0.42088E+01_JPRB, 0.37906E+01_JPRB, 0.35767E+01_JPRB, 0.32961E+01_JPRB ,&
&  0.26269E+01_JPRB, 0.22628E+01_JPRB, 0.21767E+01_JPRB, 0.21547E+01_JPRB, 0.17013E+01_JPRB, 0.16319E+01_JPRB, 0.15967E+01_JPRB ,&
&  0.15772E+01_JPRB, 0.15774E+01_JPRB, 0.15824E+01_JPRB, 0.16352E+01_JPRB, 0.17143E+01_JPRB, 0.17936E+01_JPRB, 0.18531E+01_JPRB ,&
&  0.18715E+01_JPRB, 0.18513E+01_JPRB, 0.17982E+01_JPRB, 0.16888E+01_JPRB, 0.16821E+01_JPRB, 0.17287E+01_JPRB, 0.17916E+01_JPRB ,&
&  0.18309E+01_JPRB, 0.18895E+01_JPRB, 0.19309E+01_JPRB, 0.19500E+01_JPRB, 0.19298E+01_JPRB, 0.22295E+01_JPRB, 0.24094E+01_JPRB ,&
&  0.27262E+01_JPRB, 0.31072E+01_JPRB, 0.42437E+01_JPRB, 0.31834E+01_JPRB, 0.33603E+01_JPRB, 0.35186E+01_JPRB, 0.33894E+01_JPRB ,&
&  0.30459E+01_JPRB, 0.29882E+01_JPRB, 0.28870E+01_JPRB, 0.30614E+01_JPRB, 0.38487E+01_JPRB, 0.48434E+01_JPRB, 0.66361E+01_JPRB ,&
&  0.56022E+01_JPRB, 0.56248E+01_JPRB, 0.47965E+01_JPRB, 0.48327E+01_JPRB, 0.47461E+01_JPRB, 0.45084E+01_JPRB, 0.42692E+01_JPRB ,&
&  0.51400E+01_JPRB, 0.47018E+01_JPRB, 0.40889E+01_JPRB, 0.34898E+01_JPRB, 0.26504E+01_JPRB, 0.22543E+01_JPRB, 0.19035E+01_JPRB ,&
&  0.17469E+01_JPRB, 0.16668E+01_JPRB, 0.16533E+01_JPRB, 0.15958E+01_JPRB, 0.15563E+01_JPRB, 0.15188E+01_JPRB, 0.15100E+01_JPRB ,&
&  0.14926E+01_JPRB, 0.14829E+01_JPRB, 0.14714E+01_JPRB, 0.14593E+01_JPRB, 0.14494E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.14800E+01_JPRB, 0.14624E+01_JPRB, 0.14442E+01_JPRB, 0.14323E+01_JPRB, 0.14221E+01_JPRB, 0.13902E+01_JPRB, 0.13222E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.11579E+01_JPRB, 0.11372E+01_JPRB, 0.11212E+01_JPRB, 0.11249E+01_JPRB, 0.11737E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.13783E+01_JPRB, 0.15739E+01_JPRB, 0.18437E+01_JPRB, 0.22178E+01_JPRB, 0.27169E+01_JPRB, 0.35211E+01_JPRB, 0.45058E+01_JPRB ,&
&  0.47577E+01_JPRB, 0.35564E+01_JPRB, 0.39774E+01_JPRB, 0.38096E+01_JPRB, 0.35682E+01_JPRB, 0.32957E+01_JPRB, 0.27835E+01_JPRB ,&
&  0.22640E+01_JPRB, 0.21106E+01_JPRB, 0.20505E+01_JPRB, 0.19132E+01_JPRB, 0.16408E+01_JPRB, 0.16513E+01_JPRB, 0.16796E+01_JPRB ,&
&  0.17137E+01_JPRB, 0.17413E+01_JPRB, 0.17444E+01_JPRB, 0.17982E+01_JPRB, 0.18470E+01_JPRB, 0.19179E+01_JPRB, 0.19959E+01_JPRB ,&
&  0.20217E+01_JPRB, 0.20193E+01_JPRB, 0.19910E+01_JPRB, 0.18711E+01_JPRB, 0.18382E+01_JPRB, 0.18964E+01_JPRB, 0.19504E+01_JPRB ,&
&  0.19837E+01_JPRB, 0.19639E+01_JPRB, 0.20456E+01_JPRB, 0.22469E+01_JPRB, 0.24959E+01_JPRB, 0.23853E+01_JPRB, 0.27490E+01_JPRB ,&
&  0.30096E+01_JPRB, 0.34901E+01_JPRB, 0.34234E+01_JPRB, 0.39042E+01_JPRB, 0.26605E+01_JPRB, 0.29199E+01_JPRB, 0.27069E+01_JPRB ,&
&  0.28804E+01_JPRB, 0.27532E+01_JPRB, 0.28876E+01_JPRB, 0.34122E+01_JPRB, 0.44486E+01_JPRB, 0.57213E+01_JPRB, 0.60474E+01_JPRB ,&
&  0.42851E+01_JPRB, 0.38762E+01_JPRB, 0.41016E+01_JPRB, 0.46507E+01_JPRB, 0.46867E+01_JPRB, 0.44989E+01_JPRB, 0.37924E+01_JPRB ,&
&  0.37962E+01_JPRB, 0.32668E+01_JPRB, 0.28795E+01_JPRB, 0.24266E+01_JPRB, 0.17391E+01_JPRB, 0.15209E+01_JPRB, 0.14683E+01_JPRB ,&
&  0.14877E+01_JPRB, 0.15141E+01_JPRB, 0.15253E+01_JPRB, 0.15343E+01_JPRB, 0.15150E+01_JPRB, 0.14910E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.14666E+01_JPRB, 0.14674E+01_JPRB, 0.14681E+01_JPRB, 0.14656E+01_JPRB, 0.14729E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.14166E+01_JPRB, 0.13849E+01_JPRB, 0.13557E+01_JPRB, 0.13302E+01_JPRB, 0.13137E+01_JPRB, 0.12700E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.11567E+01_JPRB, 0.11562E+01_JPRB, 0.11701E+01_JPRB, 0.12098E+01_JPRB, 0.12557E+01_JPRB, 0.13384E+01_JPRB ,&
&  0.14833E+01_JPRB, 0.16942E+01_JPRB, 0.19780E+01_JPRB, 0.23279E+01_JPRB, 0.27791E+01_JPRB, 0.33220E+01_JPRB, 0.37915E+01_JPRB ,&
&  0.42759E+01_JPRB, 0.44441E+01_JPRB, 0.36828E+01_JPRB, 0.26440E+01_JPRB, 0.25202E+01_JPRB, 0.25045E+01_JPRB, 0.21113E+01_JPRB ,&
&  0.20083E+01_JPRB, 0.19072E+01_JPRB, 0.18225E+01_JPRB, 0.17771E+01_JPRB, 0.17651E+01_JPRB, 0.18024E+01_JPRB, 0.18503E+01_JPRB ,&
&  0.18847E+01_JPRB, 0.19110E+01_JPRB, 0.19485E+01_JPRB, 0.20280E+01_JPRB, 0.20960E+01_JPRB, 0.21730E+01_JPRB, 0.22328E+01_JPRB ,&
&  0.22668E+01_JPRB, 0.23215E+01_JPRB, 0.23076E+01_JPRB, 0.22418E+01_JPRB, 0.21695E+01_JPRB, 0.22247E+01_JPRB, 0.22985E+01_JPRB ,&
&  0.23627E+01_JPRB, 0.23826E+01_JPRB, 0.24416E+01_JPRB, 0.27075E+01_JPRB, 0.29409E+01_JPRB, 0.27413E+01_JPRB, 0.28733E+01_JPRB ,&
&  0.33379E+01_JPRB, 0.36801E+01_JPRB, 0.28772E+01_JPRB, 0.33865E+01_JPRB, 0.20917E+01_JPRB, 0.26063E+01_JPRB, 0.28710E+01_JPRB ,&
&  0.28005E+01_JPRB, 0.29519E+01_JPRB, 0.32108E+01_JPRB, 0.39026E+01_JPRB, 0.48795E+01_JPRB, 0.55790E+01_JPRB, 0.48772E+01_JPRB ,&
&  0.34178E+01_JPRB, 0.33214E+01_JPRB, 0.36083E+01_JPRB, 0.39126E+01_JPRB, 0.39822E+01_JPRB, 0.36831E+01_JPRB, 0.26968E+01_JPRB ,&
&  0.20399E+01_JPRB, 0.23399E+01_JPRB, 0.23040E+01_JPRB, 0.21521E+01_JPRB, 0.14925E+01_JPRB, 0.12968E+01_JPRB, 0.13615E+01_JPRB ,&
&  0.14444E+01_JPRB, 0.15063E+01_JPRB, 0.15386E+01_JPRB, 0.15348E+01_JPRB, 0.15086E+01_JPRB, 0.14842E+01_JPRB, 0.14779E+01_JPRB ,&
&  0.14759E+01_JPRB, 0.14647E+01_JPRB, 0.14600E+01_JPRB, 0.14595E+01_JPRB, 0.14461E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.12934E+01_JPRB, 0.12514E+01_JPRB, 0.12266E+01_JPRB, 0.12102E+01_JPRB, 0.12074E+01_JPRB, 0.11807E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.11058E+01_JPRB, 0.11257E+01_JPRB, 0.11729E+01_JPRB, 0.12360E+01_JPRB, 0.12996E+01_JPRB, 0.14199E+01_JPRB ,&
&  0.15770E+01_JPRB, 0.18009E+01_JPRB, 0.20874E+01_JPRB, 0.25086E+01_JPRB, 0.28693E+01_JPRB, 0.32239E+01_JPRB, 0.33527E+01_JPRB ,&
&  0.35314E+01_JPRB, 0.35811E+01_JPRB, 0.33536E+01_JPRB, 0.31252E+01_JPRB, 0.24813E+01_JPRB, 0.24691E+01_JPRB, 0.25969E+01_JPRB ,&
&  0.27080E+01_JPRB, 0.22586E+01_JPRB, 0.19308E+01_JPRB, 0.19212E+01_JPRB, 0.19246E+01_JPRB, 0.19808E+01_JPRB, 0.20434E+01_JPRB ,&
&  0.20946E+01_JPRB, 0.21462E+01_JPRB, 0.21977E+01_JPRB, 0.22777E+01_JPRB, 0.23638E+01_JPRB, 0.24092E+01_JPRB, 0.24572E+01_JPRB ,&
&  0.24816E+01_JPRB, 0.25189E+01_JPRB, 0.25485E+01_JPRB, 0.25457E+01_JPRB, 0.26036E+01_JPRB, 0.26975E+01_JPRB, 0.28620E+01_JPRB ,&
&  0.30323E+01_JPRB, 0.30130E+01_JPRB, 0.32207E+01_JPRB, 0.35552E+01_JPRB, 0.36102E+01_JPRB, 0.35006E+01_JPRB, 0.35133E+01_JPRB ,&
&  0.38666E+01_JPRB, 0.42047E+01_JPRB, 0.24557E+01_JPRB, 0.27348E+01_JPRB, 0.32174E+01_JPRB, 0.28664E+01_JPRB, 0.27354E+01_JPRB ,&
&  0.29039E+01_JPRB, 0.31728E+01_JPRB, 0.35884E+01_JPRB, 0.40949E+01_JPRB, 0.49021E+01_JPRB, 0.63050E+01_JPRB, 0.43066E+01_JPRB ,&
&  0.28216E+01_JPRB, 0.26977E+01_JPRB, 0.28814E+01_JPRB, 0.30364E+01_JPRB, 0.30436E+01_JPRB, 0.26134E+01_JPRB, 0.17984E+01_JPRB ,&
&  0.15521E+01_JPRB, 0.20792E+01_JPRB, 0.21560E+01_JPRB, 0.21284E+01_JPRB, 0.17889E+01_JPRB, 0.12300E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.14336E+01_JPRB, 0.14815E+01_JPRB, 0.14735E+01_JPRB, 0.14723E+01_JPRB, 0.14699E+01_JPRB, 0.14601E+01_JPRB, 0.14576E+01_JPRB ,&
&  0.14546E+01_JPRB, 0.14498E+01_JPRB, 0.14458E+01_JPRB, 0.14083E+01_JPRB, 0.13433E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.11562E+01_JPRB, 0.11244E+01_JPRB, 0.11074E+01_JPRB, 0.10827E+01_JPRB, 0.10667E+01_JPRB, 0.10445E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.10456E+01_JPRB, 0.10763E+01_JPRB, 0.11227E+01_JPRB, 0.11888E+01_JPRB, 0.12798E+01_JPRB, 0.14389E+01_JPRB ,&
&  0.16034E+01_JPRB, 0.18563E+01_JPRB, 0.21695E+01_JPRB, 0.25408E+01_JPRB, 0.28480E+01_JPRB, 0.29874E+01_JPRB, 0.31647E+01_JPRB ,&
&  0.32906E+01_JPRB, 0.33350E+01_JPRB, 0.32887E+01_JPRB, 0.32176E+01_JPRB, 0.31150E+01_JPRB, 0.31568E+01_JPRB, 0.33683E+01_JPRB ,&
&  0.31247E+01_JPRB, 0.23659E+01_JPRB, 0.21374E+01_JPRB, 0.20563E+01_JPRB, 0.20896E+01_JPRB, 0.21368E+01_JPRB, 0.22151E+01_JPRB ,&
&  0.22780E+01_JPRB, 0.23246E+01_JPRB, 0.23761E+01_JPRB, 0.24384E+01_JPRB, 0.25454E+01_JPRB, 0.26134E+01_JPRB, 0.26966E+01_JPRB ,&
&  0.27741E+01_JPRB, 0.28569E+01_JPRB, 0.29990E+01_JPRB, 0.28652E+01_JPRB, 0.31334E+01_JPRB, 0.33398E+01_JPRB, 0.35342E+01_JPRB ,&
&  0.36085E+01_JPRB, 0.35963E+01_JPRB, 0.38908E+01_JPRB, 0.41169E+01_JPRB, 0.42948E+01_JPRB, 0.43643E+01_JPRB, 0.42795E+01_JPRB ,&
&  0.44688E+01_JPRB, 0.42016E+01_JPRB, 0.24377E+01_JPRB, 0.22109E+01_JPRB, 0.24426E+01_JPRB, 0.22864E+01_JPRB, 0.27714E+01_JPRB ,&
&  0.29503E+01_JPRB, 0.32552E+01_JPRB, 0.36037E+01_JPRB, 0.40614E+01_JPRB, 0.46188E+01_JPRB, 0.57406E+01_JPRB, 0.38431E+01_JPRB ,&
&  0.21425E+01_JPRB, 0.20148E+01_JPRB, 0.20690E+01_JPRB, 0.21361E+01_JPRB, 0.22088E+01_JPRB, 0.19953E+01_JPRB, 0.17051E+01_JPRB ,&
&  0.16963E+01_JPRB, 0.18054E+01_JPRB, 0.18701E+01_JPRB, 0.16503E+01_JPRB, 0.13021E+01_JPRB, 0.11541E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.12917E+01_JPRB, 0.13373E+01_JPRB, 0.13414E+01_JPRB, 0.13570E+01_JPRB, 0.13552E+01_JPRB, 0.13503E+01_JPRB, 0.13468E+01_JPRB ,&
&  0.13351E+01_JPRB, 0.13249E+01_JPRB, 0.12973E+01_JPRB, 0.12402E+01_JPRB, 0.11898E+01_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.97108E+00_JPRB, 0.97109E+00_JPRB, 0.95326E+00_JPRB, 0.93047E+00_JPRB, 0.91860E+00_JPRB, 0.89760E+00_JPRB, 0.92498E+00_JPRB ,&
&  0.92857E+00_JPRB, 0.96112E+00_JPRB, 0.99540E+00_JPRB, 0.10348E+01_JPRB, 0.11014E+01_JPRB, 0.11738E+01_JPRB, 0.12784E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.17286E+01_JPRB, 0.20850E+01_JPRB, 0.25324E+01_JPRB, 0.27326E+01_JPRB, 0.28793E+01_JPRB, 0.30722E+01_JPRB ,&
&  0.32982E+01_JPRB, 0.34909E+01_JPRB, 0.36152E+01_JPRB, 0.37714E+01_JPRB, 0.39529E+01_JPRB, 0.39577E+01_JPRB, 0.38888E+01_JPRB ,&
&  0.31920E+01_JPRB, 0.21945E+01_JPRB, 0.22426E+01_JPRB, 0.20710E+01_JPRB, 0.21701E+01_JPRB, 0.22733E+01_JPRB, 0.23473E+01_JPRB ,&
&  0.24219E+01_JPRB, 0.24743E+01_JPRB, 0.25501E+01_JPRB, 0.26433E+01_JPRB, 0.27726E+01_JPRB, 0.29006E+01_JPRB, 0.30357E+01_JPRB ,&
&  0.31454E+01_JPRB, 0.32537E+01_JPRB, 0.33306E+01_JPRB, 0.32784E+01_JPRB, 0.34839E+01_JPRB, 0.37783E+01_JPRB, 0.40473E+01_JPRB ,&
&  0.42636E+01_JPRB, 0.42587E+01_JPRB, 0.38528E+01_JPRB, 0.40375E+01_JPRB, 0.43459E+01_JPRB, 0.41942E+01_JPRB, 0.43836E+01_JPRB ,&
&  0.48689E+01_JPRB, 0.43884E+01_JPRB, 0.22983E+01_JPRB, 0.20622E+01_JPRB, 0.22896E+01_JPRB, 0.24755E+01_JPRB, 0.26540E+01_JPRB ,&
&  0.28194E+01_JPRB, 0.29826E+01_JPRB, 0.32048E+01_JPRB, 0.34639E+01_JPRB, 0.36082E+01_JPRB, 0.37610E+01_JPRB, 0.30372E+01_JPRB ,&
&  0.17410E+01_JPRB, 0.16536E+01_JPRB, 0.17618E+01_JPRB, 0.18993E+01_JPRB, 0.22324E+01_JPRB, 0.21777E+01_JPRB, 0.16974E+01_JPRB ,&
&  0.18139E+01_JPRB, 0.21630E+01_JPRB, 0.26173E+01_JPRB, 0.21381E+01_JPRB, 0.12741E+01_JPRB, 0.10585E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.11098E+01_JPRB, 0.11367E+01_JPRB, 0.11568E+01_JPRB, 0.11820E+01_JPRB, 0.11682E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.11017E+01_JPRB, 0.10693E+01_JPRB, 0.10237E+01_JPRB, 0.98998E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.80960E+00_JPRB, 0.77071E+00_JPRB, 0.72509E+00_JPRB, 0.71779E+00_JPRB, 0.72031E+00_JPRB, 0.70503E+00_JPRB, 0.74784E+00_JPRB ,&
&  0.72479E+00_JPRB, 0.76302E+00_JPRB, 0.78659E+00_JPRB, 0.79879E+00_JPRB, 0.86226E+00_JPRB, 0.92647E+00_JPRB, 0.10074E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.12856E+01_JPRB, 0.15029E+01_JPRB, 0.17797E+01_JPRB, 0.21207E+01_JPRB, 0.23091E+01_JPRB, 0.24946E+01_JPRB ,&
&  0.27456E+01_JPRB, 0.30729E+01_JPRB, 0.35283E+01_JPRB, 0.40539E+01_JPRB, 0.46604E+01_JPRB, 0.45197E+01_JPRB, 0.39051E+01_JPRB ,&
&  0.28385E+01_JPRB, 0.21600E+01_JPRB, 0.19866E+01_JPRB, 0.18536E+01_JPRB, 0.19836E+01_JPRB, 0.22402E+01_JPRB, 0.23634E+01_JPRB ,&
&  0.24533E+01_JPRB, 0.25554E+01_JPRB, 0.26662E+01_JPRB, 0.27912E+01_JPRB, 0.29342E+01_JPRB, 0.30978E+01_JPRB, 0.31940E+01_JPRB ,&
&  0.32587E+01_JPRB, 0.33252E+01_JPRB, 0.33838E+01_JPRB, 0.35703E+01_JPRB, 0.38421E+01_JPRB, 0.41903E+01_JPRB, 0.42801E+01_JPRB ,&
&  0.43044E+01_JPRB, 0.42125E+01_JPRB, 0.36844E+01_JPRB, 0.36649E+01_JPRB, 0.37951E+01_JPRB, 0.38632E+01_JPRB, 0.39812E+01_JPRB ,&
&  0.44092E+01_JPRB, 0.36152E+01_JPRB, 0.23349E+01_JPRB, 0.23446E+01_JPRB, 0.24638E+01_JPRB, 0.26287E+01_JPRB, 0.24323E+01_JPRB ,&
&  0.24842E+01_JPRB, 0.25442E+01_JPRB, 0.26575E+01_JPRB, 0.26779E+01_JPRB, 0.26974E+01_JPRB, 0.26141E+01_JPRB, 0.22744E+01_JPRB ,&
&  0.17537E+01_JPRB, 0.16225E+01_JPRB, 0.16048E+01_JPRB, 0.17109E+01_JPRB, 0.22156E+01_JPRB, 0.22140E+01_JPRB, 0.19702E+01_JPRB ,&
&  0.17722E+01_JPRB, 0.19984E+01_JPRB, 0.17479E+01_JPRB, 0.20624E+01_JPRB, 0.10900E+01_JPRB, 0.90211E+00_JPRB, 0.87615E+00_JPRB ,&
&  0.90722E+00_JPRB, 0.91578E+00_JPRB, 0.93566E+00_JPRB, 0.92456E+00_JPRB, 0.93918E+00_JPRB, 0.97671E+00_JPRB, 0.98613E+00_JPRB ,&
&  0.93626E+00_JPRB, 0.92045E+00_JPRB, 0.88079E+00_JPRB, 0.83670E+00_JPRB, 0.82488E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.67086E+00_JPRB, 0.65911E+00_JPRB, 0.65299E+00_JPRB, 0.64658E+00_JPRB, 0.65759E+00_JPRB, 0.66417E+00_JPRB, 0.67290E+00_JPRB ,&
&  0.68054E+00_JPRB, 0.70440E+00_JPRB, 0.72579E+00_JPRB, 0.75360E+00_JPRB, 0.78256E+00_JPRB, 0.83418E+00_JPRB, 0.89145E+00_JPRB ,&
&  0.95901E+00_JPRB, 0.10341E+01_JPRB, 0.11189E+01_JPRB, 0.12428E+01_JPRB, 0.14193E+01_JPRB, 0.15559E+01_JPRB, 0.17058E+01_JPRB ,&
&  0.19040E+01_JPRB, 0.21991E+01_JPRB, 0.25426E+01_JPRB, 0.30062E+01_JPRB, 0.35471E+01_JPRB, 0.37410E+01_JPRB, 0.35713E+01_JPRB ,&
&  0.25823E+01_JPRB, 0.22868E+01_JPRB, 0.20671E+01_JPRB, 0.18890E+01_JPRB, 0.19303E+01_JPRB, 0.18903E+01_JPRB, 0.19415E+01_JPRB ,&
&  0.20265E+01_JPRB, 0.20539E+01_JPRB, 0.20637E+01_JPRB, 0.21094E+01_JPRB, 0.21473E+01_JPRB, 0.21823E+01_JPRB, 0.21712E+01_JPRB ,&
&  0.21755E+01_JPRB, 0.22733E+01_JPRB, 0.26462E+01_JPRB, 0.31190E+01_JPRB, 0.34656E+01_JPRB, 0.36337E+01_JPRB, 0.34615E+01_JPRB ,&
&  0.32067E+01_JPRB, 0.30413E+01_JPRB, 0.28734E+01_JPRB, 0.28507E+01_JPRB, 0.29644E+01_JPRB, 0.30394E+01_JPRB, 0.32957E+01_JPRB ,&
&  0.31717E+01_JPRB, 0.27098E+01_JPRB, 0.25587E+01_JPRB, 0.24899E+01_JPRB, 0.25049E+01_JPRB, 0.24650E+01_JPRB, 0.23000E+01_JPRB ,&
&  0.23455E+01_JPRB, 0.22689E+01_JPRB, 0.22564E+01_JPRB, 0.21338E+01_JPRB, 0.19581E+01_JPRB, 0.17843E+01_JPRB, 0.16911E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.14485E+01_JPRB, 0.14890E+01_JPRB, 0.17462E+01_JPRB, 0.22086E+01_JPRB, 0.36346E+01_JPRB, 0.31250E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.14893E+01_JPRB, 0.97818E+00_JPRB, 0.12667E+01_JPRB, 0.86260E+00_JPRB, 0.77080E+00_JPRB, 0.76087E+00_JPRB ,&
&  0.76605E+00_JPRB, 0.75994E+00_JPRB, 0.73989E+00_JPRB, 0.73593E+00_JPRB, 0.74838E+00_JPRB, 0.76065E+00_JPRB, 0.75320E+00_JPRB ,&
&  0.71482E+00_JPRB, 0.69851E+00_JPRB, 0.73118E+00_JPRB, 0.68274E+00_JPRB, 0.67147E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.61435E+00_JPRB, 0.61573E+00_JPRB, 0.63368E+00_JPRB, 0.65070E+00_JPRB, 0.66278E+00_JPRB, 0.67603E+00_JPRB, 0.69492E+00_JPRB ,&
&  0.71093E+00_JPRB, 0.72973E+00_JPRB, 0.74938E+00_JPRB, 0.77861E+00_JPRB, 0.81735E+00_JPRB, 0.87393E+00_JPRB, 0.93889E+00_JPRB ,&
&  0.10194E+01_JPRB, 0.11068E+01_JPRB, 0.12065E+01_JPRB, 0.13162E+01_JPRB, 0.14517E+01_JPRB, 0.16389E+01_JPRB, 0.18960E+01_JPRB ,&
&  0.20517E+01_JPRB, 0.22523E+01_JPRB, 0.25080E+01_JPRB, 0.27510E+01_JPRB, 0.28305E+01_JPRB, 0.28152E+01_JPRB, 0.23887E+01_JPRB ,&
&  0.27499E+01_JPRB, 0.22981E+01_JPRB, 0.20226E+01_JPRB, 0.18257E+01_JPRB, 0.16971E+01_JPRB, 0.15897E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.14791E+01_JPRB, 0.14050E+01_JPRB, 0.13304E+01_JPRB, 0.12845E+01_JPRB, 0.12495E+01_JPRB, 0.12645E+01_JPRB, 0.12548E+01_JPRB ,&
&  0.13786E+01_JPRB, 0.16714E+01_JPRB, 0.20560E+01_JPRB, 0.25325E+01_JPRB, 0.28064E+01_JPRB, 0.27796E+01_JPRB, 0.23314E+01_JPRB ,&
&  0.20679E+01_JPRB, 0.19641E+01_JPRB, 0.19505E+01_JPRB, 0.20779E+01_JPRB, 0.21215E+01_JPRB, 0.22504E+01_JPRB, 0.23404E+01_JPRB ,&
&  0.23394E+01_JPRB, 0.21342E+01_JPRB, 0.21770E+01_JPRB, 0.26490E+01_JPRB, 0.25681E+01_JPRB, 0.23655E+01_JPRB, 0.22800E+01_JPRB ,&
&  0.21663E+01_JPRB, 0.20920E+01_JPRB, 0.20458E+01_JPRB, 0.19177E+01_JPRB, 0.17319E+01_JPRB, 0.15383E+01_JPRB, 0.14770E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.12953E+01_JPRB, 0.12820E+01_JPRB, 0.14605E+01_JPRB, 0.19327E+01_JPRB, 0.24874E+01_JPRB, 0.35137E+01_JPRB ,&
&  0.18606E+01_JPRB, 0.10650E+01_JPRB, 0.10257E+01_JPRB, 0.94758E+00_JPRB, 0.80839E+00_JPRB, 0.70238E+00_JPRB, 0.74301E+00_JPRB ,&
&  0.80531E+00_JPRB, 0.87194E+00_JPRB, 0.86127E+00_JPRB, 0.75069E+00_JPRB, 0.70879E+00_JPRB, 0.69176E+00_JPRB, 0.66856E+00_JPRB ,&
&  0.64890E+00_JPRB, 0.61181E+00_JPRB, 0.62891E+00_JPRB, 0.61273E+00_JPRB, 0.61388E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.57364E+00_JPRB, 0.57354E+00_JPRB, 0.58454E+00_JPRB, 0.59547E+00_JPRB, 0.60066E+00_JPRB, 0.60709E+00_JPRB, 0.61417E+00_JPRB ,&
&  0.62196E+00_JPRB, 0.62603E+00_JPRB, 0.63176E+00_JPRB, 0.64252E+00_JPRB, 0.66361E+00_JPRB, 0.70924E+00_JPRB, 0.76504E+00_JPRB ,&
&  0.81785E+00_JPRB, 0.87165E+00_JPRB, 0.92962E+00_JPRB, 0.98217E+00_JPRB, 0.10377E+01_JPRB, 0.10797E+01_JPRB, 0.11734E+01_JPRB ,&
&  0.12941E+01_JPRB, 0.14399E+01_JPRB, 0.15949E+01_JPRB, 0.17253E+01_JPRB, 0.18679E+01_JPRB, 0.19406E+01_JPRB, 0.17022E+01_JPRB ,&
&  0.22969E+01_JPRB, 0.21089E+01_JPRB, 0.18958E+01_JPRB, 0.17044E+01_JPRB, 0.15711E+01_JPRB, 0.14512E+01_JPRB, 0.13426E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.13421E+01_JPRB, 0.12805E+01_JPRB, 0.13654E+01_JPRB, 0.11111E+01_JPRB, 0.11420E+01_JPRB, 0.13558E+01_JPRB ,&
&  0.15371E+01_JPRB, 0.17369E+01_JPRB, 0.19837E+01_JPRB, 0.22334E+01_JPRB, 0.23818E+01_JPRB, 0.23420E+01_JPRB, 0.21017E+01_JPRB ,&
&  0.19622E+01_JPRB, 0.19198E+01_JPRB, 0.18614E+01_JPRB, 0.20341E+01_JPRB, 0.19832E+01_JPRB, 0.18895E+01_JPRB, 0.19332E+01_JPRB ,&
&  0.17333E+01_JPRB, 0.17817E+01_JPRB, 0.20516E+01_JPRB, 0.25651E+01_JPRB, 0.24591E+01_JPRB, 0.23566E+01_JPRB, 0.22508E+01_JPRB ,&
&  0.21166E+01_JPRB, 0.20147E+01_JPRB, 0.19311E+01_JPRB, 0.17906E+01_JPRB, 0.16211E+01_JPRB, 0.14495E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.11863E+01_JPRB, 0.11386E+01_JPRB, 0.11734E+01_JPRB, 0.13580E+01_JPRB, 0.16656E+01_JPRB, 0.29356E+01_JPRB ,&
&  0.18131E+01_JPRB, 0.14058E+01_JPRB, 0.15133E+01_JPRB, 0.59482E+00_JPRB, 0.66621E+00_JPRB, 0.66491E+00_JPRB, 0.76707E+00_JPRB ,&
&  0.91765E+00_JPRB, 0.14288E+01_JPRB, 0.21176E+01_JPRB, 0.18865E+01_JPRB, 0.14525E+01_JPRB, 0.95756E+00_JPRB, 0.60481E+00_JPRB ,&
&  0.58592E+00_JPRB, 0.58147E+00_JPRB, 0.58752E+00_JPRB, 0.58237E+00_JPRB, 0.58074E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.51640E+00_JPRB, 0.50806E+00_JPRB, 0.51886E+00_JPRB, 0.53107E+00_JPRB, 0.52892E+00_JPRB, 0.53045E+00_JPRB, 0.52612E+00_JPRB ,&
&  0.51758E+00_JPRB, 0.51114E+00_JPRB, 0.53625E+00_JPRB, 0.58381E+00_JPRB, 0.65240E+00_JPRB, 0.74344E+00_JPRB, 0.83918E+00_JPRB ,&
&  0.91598E+00_JPRB, 0.10117E+01_JPRB, 0.11051E+01_JPRB, 0.11857E+01_JPRB, 0.12205E+01_JPRB, 0.12503E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.13301E+01_JPRB, 0.14501E+01_JPRB, 0.15849E+01_JPRB, 0.17959E+01_JPRB, 0.21417E+01_JPRB, 0.13578E+01_JPRB ,&
&  0.18672E+01_JPRB, 0.18354E+01_JPRB, 0.17621E+01_JPRB, 0.16231E+01_JPRB, 0.15542E+01_JPRB, 0.14442E+01_JPRB, 0.13746E+01_JPRB ,&
&  0.14236E+01_JPRB, 0.15390E+01_JPRB, 0.15237E+01_JPRB, 0.15904E+01_JPRB, 0.16315E+01_JPRB, 0.16505E+01_JPRB, 0.19082E+01_JPRB ,&
&  0.21296E+01_JPRB, 0.24187E+01_JPRB, 0.27278E+01_JPRB, 0.29386E+01_JPRB, 0.31119E+01_JPRB, 0.31865E+01_JPRB, 0.31776E+01_JPRB ,&
&  0.29763E+01_JPRB, 0.28391E+01_JPRB, 0.29375E+01_JPRB, 0.31767E+01_JPRB, 0.27991E+01_JPRB, 0.26326E+01_JPRB, 0.22369E+01_JPRB ,&
&  0.17033E+01_JPRB, 0.15814E+01_JPRB, 0.18204E+01_JPRB, 0.22723E+01_JPRB, 0.23285E+01_JPRB, 0.22235E+01_JPRB, 0.20275E+01_JPRB ,&
&  0.18889E+01_JPRB, 0.17552E+01_JPRB, 0.16540E+01_JPRB, 0.15361E+01_JPRB, 0.14136E+01_JPRB, 0.12896E+01_JPRB, 0.11700E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.10550E+01_JPRB, 0.99443E+00_JPRB, 0.10320E+01_JPRB, 0.11750E+01_JPRB, 0.15714E+01_JPRB, 0.17591E+01_JPRB ,&
&  0.19048E+01_JPRB, 0.12970E+01_JPRB, 0.10539E+01_JPRB, 0.96253E+00_JPRB, 0.66401E+00_JPRB, 0.65692E+00_JPRB, 0.81363E+00_JPRB ,&
&  0.12349E+01_JPRB, 0.17573E+01_JPRB, 0.28843E+01_JPRB, 0.51052E+01_JPRB, 0.28390E+01_JPRB, 0.22040E+01_JPRB, 0.58268E+00_JPRB ,&
&  0.56193E+00_JPRB, 0.56200E+00_JPRB, 0.55984E+00_JPRB, 0.54458E+00_JPRB, 0.53993E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.49475E+00_JPRB, 0.48746E+00_JPRB, 0.49865E+00_JPRB, 0.49746E+00_JPRB, 0.50167E+00_JPRB, 0.50981E+00_JPRB, 0.51448E+00_JPRB ,&
&  0.51976E+00_JPRB, 0.53835E+00_JPRB, 0.63849E+00_JPRB, 0.78075E+00_JPRB, 0.92224E+00_JPRB, 0.10362E+01_JPRB, 0.11023E+01_JPRB ,&
&  0.11821E+01_JPRB, 0.12518E+01_JPRB, 0.12996E+01_JPRB, 0.13428E+01_JPRB, 0.14243E+01_JPRB, 0.15311E+01_JPRB, 0.16246E+01_JPRB ,&
&  0.17201E+01_JPRB, 0.18202E+01_JPRB, 0.19382E+01_JPRB, 0.21164E+01_JPRB, 0.22893E+01_JPRB, 0.22912E+01_JPRB, 0.13847E+01_JPRB ,&
&  0.16861E+01_JPRB, 0.17570E+01_JPRB, 0.17932E+01_JPRB, 0.16940E+01_JPRB, 0.15846E+01_JPRB, 0.15454E+01_JPRB, 0.16803E+01_JPRB ,&
&  0.18246E+01_JPRB, 0.18921E+01_JPRB, 0.18444E+01_JPRB, 0.18669E+01_JPRB, 0.24403E+01_JPRB, 0.24763E+01_JPRB, 0.29495E+01_JPRB ,&
&  0.32765E+01_JPRB, 0.36272E+01_JPRB, 0.38722E+01_JPRB, 0.41409E+01_JPRB, 0.43662E+01_JPRB, 0.45282E+01_JPRB, 0.47579E+01_JPRB ,&
&  0.48386E+01_JPRB, 0.49426E+01_JPRB, 0.49567E+01_JPRB, 0.48323E+01_JPRB, 0.39493E+01_JPRB, 0.41725E+01_JPRB, 0.28510E+01_JPRB ,&
&  0.20332E+01_JPRB, 0.17646E+01_JPRB, 0.19494E+01_JPRB, 0.20369E+01_JPRB, 0.19586E+01_JPRB, 0.17563E+01_JPRB, 0.14970E+01_JPRB ,&
&  0.14141E+01_JPRB, 0.13970E+01_JPRB, 0.13812E+01_JPRB, 0.13110E+01_JPRB, 0.12584E+01_JPRB, 0.11574E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10408E+01_JPRB, 0.98437E+00_JPRB, 0.92314E+00_JPRB, 0.97054E+00_JPRB, 0.11616E+01_JPRB, 0.16726E+01_JPRB, 0.24947E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.13845E+01_JPRB, 0.11048E+01_JPRB, 0.90474E+00_JPRB, 0.79770E+00_JPRB, 0.78647E+00_JPRB, 0.84995E+00_JPRB ,&
&  0.10730E+01_JPRB, 0.18861E+01_JPRB, 0.27421E+01_JPRB, 0.20231E+01_JPRB, 0.22692E+01_JPRB, 0.15752E+01_JPRB, 0.67878E+00_JPRB ,&
&  0.54738E+00_JPRB, 0.52746E+00_JPRB, 0.53183E+00_JPRB, 0.52357E+00_JPRB, 0.51092E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.54224E+00_JPRB, 0.53100E+00_JPRB, 0.52973E+00_JPRB, 0.54053E+00_JPRB, 0.54752E+00_JPRB, 0.57401E+00_JPRB, 0.60651E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.70471E+00_JPRB, 0.88170E+00_JPRB, 0.10380E+01_JPRB, 0.11177E+01_JPRB, 0.11795E+01_JPRB, 0.11701E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.12140E+01_JPRB, 0.12180E+01_JPRB, 0.12739E+01_JPRB, 0.13657E+01_JPRB, 0.14909E+01_JPRB, 0.16530E+01_JPRB ,&
&  0.18357E+01_JPRB, 0.20164E+01_JPRB, 0.21856E+01_JPRB, 0.23424E+01_JPRB, 0.25564E+01_JPRB, 0.26029E+01_JPRB, 0.25686E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.18005E+01_JPRB, 0.18336E+01_JPRB, 0.17590E+01_JPRB, 0.16538E+01_JPRB, 0.16936E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.22138E+01_JPRB, 0.22482E+01_JPRB, 0.26118E+01_JPRB, 0.24035E+01_JPRB, 0.29397E+01_JPRB, 0.32614E+01_JPRB, 0.36171E+01_JPRB ,&
&  0.39565E+01_JPRB, 0.42040E+01_JPRB, 0.44675E+01_JPRB, 0.47224E+01_JPRB, 0.49754E+01_JPRB, 0.52007E+01_JPRB, 0.54621E+01_JPRB ,&
&  0.57705E+01_JPRB, 0.61906E+01_JPRB, 0.66326E+01_JPRB, 0.63916E+01_JPRB, 0.54264E+01_JPRB, 0.50663E+01_JPRB, 0.39895E+01_JPRB ,&
&  0.26818E+01_JPRB, 0.17107E+01_JPRB, 0.16996E+01_JPRB, 0.17571E+01_JPRB, 0.16056E+01_JPRB, 0.14419E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11760E+01_JPRB, 0.11419E+01_JPRB, 0.11694E+01_JPRB, 0.11896E+01_JPRB, 0.11421E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10138E+01_JPRB, 0.96213E+00_JPRB, 0.95373E+00_JPRB, 0.10417E+01_JPRB, 0.12249E+01_JPRB, 0.13827E+01_JPRB ,&
&  0.14085E+01_JPRB, 0.12051E+01_JPRB, 0.10469E+01_JPRB, 0.10032E+01_JPRB, 0.94204E+00_JPRB, 0.89975E+00_JPRB, 0.87038E+00_JPRB ,&
&  0.90834E+00_JPRB, 0.10802E+01_JPRB, 0.14886E+01_JPRB, 0.18778E+01_JPRB, 0.14676E+01_JPRB, 0.11638E+01_JPRB, 0.68871E+00_JPRB ,&
&  0.56382E+00_JPRB, 0.54445E+00_JPRB, 0.56132E+00_JPRB, 0.56520E+00_JPRB, 0.56573E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.65362E+00_JPRB, 0.63936E+00_JPRB, 0.61924E+00_JPRB, 0.63506E+00_JPRB, 0.66915E+00_JPRB, 0.71419E+00_JPRB, 0.73559E+00_JPRB ,&
&  0.76513E+00_JPRB, 0.87886E+00_JPRB, 0.10285E+01_JPRB, 0.11630E+01_JPRB, 0.11975E+01_JPRB, 0.11792E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.11109E+01_JPRB, 0.11283E+01_JPRB, 0.11574E+01_JPRB, 0.12266E+01_JPRB, 0.13609E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.18181E+01_JPRB, 0.21057E+01_JPRB, 0.24287E+01_JPRB, 0.27843E+01_JPRB, 0.31241E+01_JPRB, 0.33308E+01_JPRB, 0.32226E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.14404E+01_JPRB, 0.19628E+01_JPRB, 0.19625E+01_JPRB, 0.18799E+01_JPRB, 0.18826E+01_JPRB, 0.20447E+01_JPRB ,&
&  0.22062E+01_JPRB, 0.22267E+01_JPRB, 0.24907E+01_JPRB, 0.28954E+01_JPRB, 0.30811E+01_JPRB, 0.34931E+01_JPRB, 0.37242E+01_JPRB ,&
&  0.39605E+01_JPRB, 0.42063E+01_JPRB, 0.44769E+01_JPRB, 0.47526E+01_JPRB, 0.50235E+01_JPRB, 0.52786E+01_JPRB, 0.56084E+01_JPRB ,&
&  0.60557E+01_JPRB, 0.65492E+01_JPRB, 0.69610E+01_JPRB, 0.62840E+01_JPRB, 0.66534E+01_JPRB, 0.58184E+01_JPRB, 0.49998E+01_JPRB ,&
&  0.29459E+01_JPRB, 0.17598E+01_JPRB, 0.17294E+01_JPRB, 0.17869E+01_JPRB, 0.15504E+01_JPRB, 0.13436E+01_JPRB, 0.11293E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.97855E+00_JPRB, 0.10365E+01_JPRB, 0.11128E+01_JPRB, 0.11033E+01_JPRB, 0.11450E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.10872E+01_JPRB, 0.10597E+01_JPRB, 0.10610E+01_JPRB, 0.10911E+01_JPRB, 0.11383E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.11133E+01_JPRB, 0.10878E+01_JPRB, 0.10701E+01_JPRB, 0.10387E+01_JPRB, 0.10037E+01_JPRB, 0.94525E+00_JPRB ,&
&  0.88176E+00_JPRB, 0.88306E+00_JPRB, 0.98014E+00_JPRB, 0.11086E+01_JPRB, 0.10331E+01_JPRB, 0.89553E+00_JPRB, 0.72072E+00_JPRB ,&
&  0.70707E+00_JPRB, 0.72768E+00_JPRB, 0.72650E+00_JPRB, 0.70678E+00_JPRB, 0.68398E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.77880E+00_JPRB, 0.74876E+00_JPRB, 0.70871E+00_JPRB, 0.69716E+00_JPRB, 0.74859E+00_JPRB, 0.79929E+00_JPRB, 0.80350E+00_JPRB ,&
&  0.81756E+00_JPRB, 0.89093E+00_JPRB, 0.10209E+01_JPRB, 0.11231E+01_JPRB, 0.11595E+01_JPRB, 0.11563E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.10576E+01_JPRB, 0.10484E+01_JPRB, 0.10601E+01_JPRB, 0.10900E+01_JPRB, 0.11519E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.17143E+01_JPRB, 0.21131E+01_JPRB, 0.26867E+01_JPRB, 0.34960E+01_JPRB, 0.44548E+01_JPRB, 0.52960E+01_JPRB ,&
&  0.40056E+01_JPRB, 0.15295E+01_JPRB, 0.18315E+01_JPRB, 0.23249E+01_JPRB, 0.21529E+01_JPRB, 0.21794E+01_JPRB, 0.21345E+01_JPRB ,&
&  0.22075E+01_JPRB, 0.22880E+01_JPRB, 0.23840E+01_JPRB, 0.27383E+01_JPRB, 0.30476E+01_JPRB, 0.34008E+01_JPRB, 0.35597E+01_JPRB ,&
&  0.37034E+01_JPRB, 0.38860E+01_JPRB, 0.40765E+01_JPRB, 0.42754E+01_JPRB, 0.44983E+01_JPRB, 0.47433E+01_JPRB, 0.51168E+01_JPRB ,&
&  0.55952E+01_JPRB, 0.60657E+01_JPRB, 0.63913E+01_JPRB, 0.55033E+01_JPRB, 0.58043E+01_JPRB, 0.46091E+01_JPRB, 0.34630E+01_JPRB ,&
&  0.26719E+01_JPRB, 0.21136E+01_JPRB, 0.19006E+01_JPRB, 0.18827E+01_JPRB, 0.15857E+01_JPRB, 0.12043E+01_JPRB, 0.11206E+01_JPRB ,&
&  0.10013E+01_JPRB, 0.97358E+00_JPRB, 0.10130E+01_JPRB, 0.98716E+00_JPRB, 0.10556E+01_JPRB, 0.10904E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.11129E+01_JPRB, 0.11365E+01_JPRB, 0.11581E+01_JPRB, 0.11988E+01_JPRB, 0.11941E+01_JPRB, 0.12018E+01_JPRB, 0.11776E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.11878E+01_JPRB, 0.11889E+01_JPRB, 0.11926E+01_JPRB, 0.11847E+01_JPRB, 0.10888E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.98461E+00_JPRB, 0.96132E+00_JPRB, 0.91501E+00_JPRB, 0.94714E+00_JPRB, 0.93664E+00_JPRB, 0.93676E+00_JPRB, 0.96673E+00_JPRB ,&
&  0.97520E+00_JPRB, 0.97555E+00_JPRB, 0.91783E+00_JPRB, 0.85795E+00_JPRB, 0.82118E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.86429E+00_JPRB, 0.80122E+00_JPRB, 0.76622E+00_JPRB, 0.77478E+00_JPRB, 0.82576E+00_JPRB, 0.87778E+00_JPRB, 0.87422E+00_JPRB ,&
&  0.85931E+00_JPRB, 0.88418E+00_JPRB, 0.96181E+00_JPRB, 0.10302E+01_JPRB, 0.10420E+01_JPRB, 0.10490E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.10205E+01_JPRB, 0.10144E+01_JPRB, 0.99954E+00_JPRB, 0.10082E+01_JPRB, 0.10174E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.12949E+01_JPRB, 0.15738E+01_JPRB, 0.19944E+01_JPRB, 0.26108E+01_JPRB, 0.37416E+01_JPRB, 0.55005E+01_JPRB ,&
&  0.81927E+01_JPRB, 0.53607E+01_JPRB, 0.17643E+01_JPRB, 0.23381E+01_JPRB, 0.24832E+01_JPRB, 0.25293E+01_JPRB, 0.21774E+01_JPRB ,&
&  0.23440E+01_JPRB, 0.24434E+01_JPRB, 0.22503E+01_JPRB, 0.25017E+01_JPRB, 0.28135E+01_JPRB, 0.30626E+01_JPRB, 0.32222E+01_JPRB ,&
&  0.32836E+01_JPRB, 0.33896E+01_JPRB, 0.34832E+01_JPRB, 0.35623E+01_JPRB, 0.36619E+01_JPRB, 0.38249E+01_JPRB, 0.41008E+01_JPRB ,&
&  0.46231E+01_JPRB, 0.52716E+01_JPRB, 0.58183E+01_JPRB, 0.50733E+01_JPRB, 0.50857E+01_JPRB, 0.39328E+01_JPRB, 0.32553E+01_JPRB ,&
&  0.27991E+01_JPRB, 0.27054E+01_JPRB, 0.25203E+01_JPRB, 0.20663E+01_JPRB, 0.16433E+01_JPRB, 0.13797E+01_JPRB, 0.13362E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.10217E+01_JPRB, 0.10138E+01_JPRB, 0.10588E+01_JPRB, 0.10750E+01_JPRB, 0.10947E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.11806E+01_JPRB, 0.12346E+01_JPRB, 0.12667E+01_JPRB, 0.12926E+01_JPRB, 0.13357E+01_JPRB, 0.13288E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.13031E+01_JPRB, 0.13031E+01_JPRB, 0.13061E+01_JPRB, 0.12392E+01_JPRB, 0.11437E+01_JPRB, 0.11335E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.10998E+01_JPRB, 0.10455E+01_JPRB, 0.10442E+01_JPRB, 0.10426E+01_JPRB, 0.10962E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11881E+01_JPRB, 0.11500E+01_JPRB, 0.10681E+01_JPRB, 0.93277E+00_JPRB, 0.91670E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.93622E+00_JPRB, 0.91097E+00_JPRB, 0.87654E+00_JPRB, 0.85302E+00_JPRB, 0.87788E+00_JPRB, 0.91527E+00_JPRB, 0.91479E+00_JPRB ,&
&  0.88727E+00_JPRB, 0.86971E+00_JPRB, 0.90917E+00_JPRB, 0.95147E+00_JPRB, 0.95011E+00_JPRB, 0.97342E+00_JPRB, 0.98722E+00_JPRB ,&
&  0.10271E+01_JPRB, 0.10267E+01_JPRB, 0.10104E+01_JPRB, 0.98800E+00_JPRB, 0.98728E+00_JPRB, 0.97518E+00_JPRB, 0.98049E+00_JPRB ,&
&  0.10110E+01_JPRB, 0.10821E+01_JPRB, 0.12320E+01_JPRB, 0.14652E+01_JPRB, 0.17601E+01_JPRB, 0.23006E+01_JPRB, 0.33871E+01_JPRB ,&
&  0.46020E+01_JPRB, 0.58942E+01_JPRB, 0.18877E+01_JPRB, 0.25633E+01_JPRB, 0.31566E+01_JPRB, 0.30382E+01_JPRB, 0.27134E+01_JPRB ,&
&  0.27103E+01_JPRB, 0.26683E+01_JPRB, 0.24723E+01_JPRB, 0.22570E+01_JPRB, 0.23714E+01_JPRB, 0.25533E+01_JPRB, 0.25502E+01_JPRB ,&
&  0.26251E+01_JPRB, 0.27657E+01_JPRB, 0.29804E+01_JPRB, 0.30293E+01_JPRB, 0.30227E+01_JPRB, 0.29963E+01_JPRB, 0.31038E+01_JPRB ,&
&  0.34693E+01_JPRB, 0.41040E+01_JPRB, 0.50484E+01_JPRB, 0.47068E+01_JPRB, 0.46961E+01_JPRB, 0.41072E+01_JPRB, 0.36210E+01_JPRB ,&
&  0.32287E+01_JPRB, 0.31861E+01_JPRB, 0.29733E+01_JPRB, 0.23992E+01_JPRB, 0.18506E+01_JPRB, 0.16042E+01_JPRB, 0.15287E+01_JPRB ,&
&  0.13138E+01_JPRB, 0.11718E+01_JPRB, 0.11555E+01_JPRB, 0.11591E+01_JPRB, 0.11986E+01_JPRB, 0.11887E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.12021E+01_JPRB, 0.12402E+01_JPRB, 0.12902E+01_JPRB, 0.13401E+01_JPRB, 0.14017E+01_JPRB, 0.14271E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.14166E+01_JPRB, 0.13990E+01_JPRB, 0.13352E+01_JPRB, 0.12501E+01_JPRB, 0.12066E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.12149E+01_JPRB, 0.11541E+01_JPRB, 0.10986E+01_JPRB, 0.11299E+01_JPRB, 0.11348E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.13187E+01_JPRB, 0.13741E+01_JPRB, 0.13059E+01_JPRB, 0.10236E+01_JPRB, 0.10014E+01_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.10621E+01_JPRB, 0.10115E+01_JPRB, 0.97854E+00_JPRB, 0.94691E+00_JPRB, 0.92190E+00_JPRB, 0.95584E+00_JPRB, 0.95022E+00_JPRB ,&
&  0.92711E+00_JPRB, 0.88975E+00_JPRB, 0.90553E+00_JPRB, 0.95763E+00_JPRB, 0.96282E+00_JPRB, 0.94758E+00_JPRB, 0.93770E+00_JPRB ,&
&  0.93894E+00_JPRB, 0.95603E+00_JPRB, 0.96029E+00_JPRB, 0.97670E+00_JPRB, 0.98115E+00_JPRB, 0.97300E+00_JPRB, 0.96972E+00_JPRB ,&
&  0.99027E+00_JPRB, 0.10231E+01_JPRB, 0.10856E+01_JPRB, 0.11670E+01_JPRB, 0.12794E+01_JPRB, 0.14529E+01_JPRB, 0.20039E+01_JPRB ,&
&  0.27344E+01_JPRB, 0.46195E+01_JPRB, 0.36849E+01_JPRB, 0.44089E+01_JPRB, 0.41656E+01_JPRB, 0.38187E+01_JPRB, 0.34168E+01_JPRB ,&
&  0.36763E+01_JPRB, 0.33500E+01_JPRB, 0.31300E+01_JPRB, 0.23515E+01_JPRB, 0.20146E+01_JPRB, 0.19874E+01_JPRB, 0.19165E+01_JPRB ,&
&  0.18640E+01_JPRB, 0.18705E+01_JPRB, 0.21647E+01_JPRB, 0.23060E+01_JPRB, 0.23629E+01_JPRB, 0.23783E+01_JPRB, 0.24065E+01_JPRB ,&
&  0.25994E+01_JPRB, 0.29408E+01_JPRB, 0.38342E+01_JPRB, 0.44991E+01_JPRB, 0.43667E+01_JPRB, 0.42779E+01_JPRB, 0.41762E+01_JPRB ,&
&  0.42040E+01_JPRB, 0.37836E+01_JPRB, 0.33569E+01_JPRB, 0.27281E+01_JPRB, 0.21192E+01_JPRB, 0.20066E+01_JPRB, 0.16992E+01_JPRB ,&
&  0.14695E+01_JPRB, 0.13183E+01_JPRB, 0.12876E+01_JPRB, 0.12808E+01_JPRB, 0.13004E+01_JPRB, 0.12721E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.12470E+01_JPRB, 0.12402E+01_JPRB, 0.12813E+01_JPRB, 0.13178E+01_JPRB, 0.13602E+01_JPRB, 0.13935E+01_JPRB ,&
&  0.14306E+01_JPRB, 0.14541E+01_JPRB, 0.14011E+01_JPRB, 0.13452E+01_JPRB, 0.12777E+01_JPRB, 0.12371E+01_JPRB, 0.12025E+01_JPRB ,&
&  0.12171E+01_JPRB, 0.12447E+01_JPRB, 0.12339E+01_JPRB, 0.11851E+01_JPRB, 0.11538E+01_JPRB, 0.11356E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.12056E+01_JPRB, 0.11725E+01_JPRB, 0.11154E+01_JPRB, 0.10959E+01_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.11467E+01_JPRB, 0.11173E+01_JPRB, 0.10587E+01_JPRB, 0.10389E+01_JPRB, 0.99484E+00_JPRB, 0.10148E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.98729E+00_JPRB, 0.95361E+00_JPRB, 0.92611E+00_JPRB, 0.94689E+00_JPRB, 0.97060E+00_JPRB, 0.96179E+00_JPRB, 0.92708E+00_JPRB ,&
&  0.91545E+00_JPRB, 0.91171E+00_JPRB, 0.90976E+00_JPRB, 0.91521E+00_JPRB, 0.93420E+00_JPRB, 0.96190E+00_JPRB, 0.96300E+00_JPRB ,&
&  0.96117E+00_JPRB, 0.10052E+01_JPRB, 0.10619E+01_JPRB, 0.10857E+01_JPRB, 0.11204E+01_JPRB, 0.11904E+01_JPRB, 0.14519E+01_JPRB ,&
&  0.18668E+01_JPRB, 0.45432E+01_JPRB, 0.28080E+01_JPRB, 0.56620E+01_JPRB, 0.45097E+01_JPRB, 0.40382E+01_JPRB, 0.33050E+01_JPRB ,&
&  0.30698E+01_JPRB, 0.32146E+01_JPRB, 0.26058E+01_JPRB, 0.21696E+01_JPRB, 0.18270E+01_JPRB, 0.16616E+01_JPRB, 0.15188E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.13684E+01_JPRB, 0.15043E+01_JPRB, 0.16641E+01_JPRB, 0.17630E+01_JPRB, 0.18144E+01_JPRB, 0.18901E+01_JPRB ,&
&  0.20218E+01_JPRB, 0.21892E+01_JPRB, 0.27787E+01_JPRB, 0.35410E+01_JPRB, 0.42376E+01_JPRB, 0.44793E+01_JPRB, 0.48223E+01_JPRB ,&
&  0.49402E+01_JPRB, 0.45456E+01_JPRB, 0.37855E+01_JPRB, 0.30362E+01_JPRB, 0.24998E+01_JPRB, 0.23026E+01_JPRB, 0.20339E+01_JPRB ,&
&  0.17829E+01_JPRB, 0.15544E+01_JPRB, 0.14902E+01_JPRB, 0.14314E+01_JPRB, 0.14333E+01_JPRB, 0.13970E+01_JPRB, 0.13144E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.12538E+01_JPRB, 0.12812E+01_JPRB, 0.13100E+01_JPRB, 0.13461E+01_JPRB, 0.13529E+01_JPRB, 0.13546E+01_JPRB ,&
&  0.13867E+01_JPRB, 0.14283E+01_JPRB, 0.13981E+01_JPRB, 0.13853E+01_JPRB, 0.13309E+01_JPRB, 0.12902E+01_JPRB, 0.12719E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.13018E+01_JPRB, 0.12539E+01_JPRB, 0.11972E+01_JPRB, 0.11665E+01_JPRB, 0.11694E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.11469E+01_JPRB, 0.11859E+01_JPRB, 0.11510E+01_JPRB, 0.11477E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.11829E+01_JPRB, 0.11748E+01_JPRB, 0.11012E+01_JPRB, 0.10743E+01_JPRB, 0.10349E+01_JPRB, 0.10175E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.10137E+01_JPRB, 0.98834E+00_JPRB, 0.95629E+00_JPRB, 0.94120E+00_JPRB, 0.95432E+00_JPRB, 0.96938E+00_JPRB, 0.93843E+00_JPRB ,&
&  0.90261E+00_JPRB, 0.88825E+00_JPRB, 0.87856E+00_JPRB, 0.88081E+00_JPRB, 0.89458E+00_JPRB, 0.91213E+00_JPRB, 0.92437E+00_JPRB ,&
&  0.92199E+00_JPRB, 0.96945E+00_JPRB, 0.10382E+01_JPRB, 0.10803E+01_JPRB, 0.10944E+01_JPRB, 0.11573E+01_JPRB, 0.13592E+01_JPRB ,&
&  0.18650E+01_JPRB, 0.64185E+01_JPRB, 0.32301E+01_JPRB, 0.59409E+01_JPRB, 0.46738E+01_JPRB, 0.40266E+01_JPRB, 0.31943E+01_JPRB ,&
&  0.28970E+01_JPRB, 0.27077E+01_JPRB, 0.23342E+01_JPRB, 0.20390E+01_JPRB, 0.18146E+01_JPRB, 0.16244E+01_JPRB, 0.14673E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.12992E+01_JPRB, 0.13247E+01_JPRB, 0.14218E+01_JPRB, 0.14784E+01_JPRB, 0.15021E+01_JPRB, 0.15378E+01_JPRB ,&
&  0.16226E+01_JPRB, 0.17454E+01_JPRB, 0.20994E+01_JPRB, 0.30746E+01_JPRB, 0.38292E+01_JPRB, 0.45250E+01_JPRB, 0.64527E+01_JPRB ,&
&  0.61808E+01_JPRB, 0.56883E+01_JPRB, 0.42403E+01_JPRB, 0.33841E+01_JPRB, 0.28214E+01_JPRB, 0.24955E+01_JPRB, 0.23213E+01_JPRB ,&
&  0.21365E+01_JPRB, 0.18874E+01_JPRB, 0.17675E+01_JPRB, 0.16830E+01_JPRB, 0.15985E+01_JPRB, 0.15185E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.13754E+01_JPRB, 0.13359E+01_JPRB, 0.13536E+01_JPRB, 0.13715E+01_JPRB, 0.13876E+01_JPRB, 0.13768E+01_JPRB, 0.13643E+01_JPRB ,&
&  0.13719E+01_JPRB, 0.14000E+01_JPRB, 0.13815E+01_JPRB, 0.13872E+01_JPRB, 0.13692E+01_JPRB, 0.13373E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.13710E+01_JPRB, 0.12957E+01_JPRB, 0.12526E+01_JPRB, 0.12041E+01_JPRB, 0.12293E+01_JPRB, 0.11476E+01_JPRB ,&
&  0.11161E+01_JPRB, 0.11028E+01_JPRB, 0.11605E+01_JPRB, 0.11673E+01_JPRB, 0.11859E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.11997E+01_JPRB, 0.11822E+01_JPRB, 0.11430E+01_JPRB, 0.10836E+01_JPRB, 0.10580E+01_JPRB, 0.10307E+01_JPRB, 0.10140E+01_JPRB ,&
&  0.10069E+01_JPRB, 0.99144E+00_JPRB, 0.97602E+00_JPRB, 0.95335E+00_JPRB, 0.94362E+00_JPRB, 0.95577E+00_JPRB, 0.95059E+00_JPRB ,&
&  0.93269E+00_JPRB, 0.90881E+00_JPRB, 0.89943E+00_JPRB, 0.88952E+00_JPRB, 0.87841E+00_JPRB, 0.87814E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.89863E+00_JPRB, 0.92057E+00_JPRB, 0.96247E+00_JPRB, 0.10032E+01_JPRB, 0.10183E+01_JPRB, 0.10626E+01_JPRB, 0.13042E+01_JPRB ,&
&  0.20220E+01_JPRB, 0.21997E+01_JPRB, 0.48759E+01_JPRB, 0.58185E+01_JPRB, 0.47961E+01_JPRB, 0.41610E+01_JPRB, 0.34875E+01_JPRB ,&
&  0.30403E+01_JPRB, 0.25175E+01_JPRB, 0.20924E+01_JPRB, 0.18252E+01_JPRB, 0.16773E+01_JPRB, 0.15088E+01_JPRB, 0.14109E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.13221E+01_JPRB, 0.13177E+01_JPRB, 0.13355E+01_JPRB, 0.13344E+01_JPRB, 0.13313E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.14936E+01_JPRB, 0.16877E+01_JPRB, 0.23141E+01_JPRB, 0.35143E+01_JPRB, 0.47046E+01_JPRB, 0.64278E+01_JPRB ,&
&  0.72537E+01_JPRB, 0.66090E+01_JPRB, 0.46995E+01_JPRB, 0.36523E+01_JPRB, 0.30859E+01_JPRB, 0.27033E+01_JPRB, 0.25066E+01_JPRB ,&
&  0.23579E+01_JPRB, 0.21647E+01_JPRB, 0.20090E+01_JPRB, 0.19694E+01_JPRB, 0.18298E+01_JPRB, 0.16852E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.14570E+01_JPRB, 0.14084E+01_JPRB, 0.13748E+01_JPRB, 0.13806E+01_JPRB, 0.13750E+01_JPRB, 0.13591E+01_JPRB ,&
&  0.13544E+01_JPRB, 0.13645E+01_JPRB, 0.13559E+01_JPRB, 0.13487E+01_JPRB, 0.13726E+01_JPRB, 0.13787E+01_JPRB, 0.13826E+01_JPRB ,&
&  0.13815E+01_JPRB, 0.13909E+01_JPRB, 0.13423E+01_JPRB, 0.13141E+01_JPRB, 0.12373E+01_JPRB, 0.12750E+01_JPRB, 0.11945E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.11021E+01_JPRB, 0.11047E+01_JPRB, 0.11535E+01_JPRB, 0.11955E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.11783E+01_JPRB, 0.11657E+01_JPRB, 0.11415E+01_JPRB, 0.11065E+01_JPRB, 0.10763E+01_JPRB, 0.10581E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10351E+01_JPRB, 0.10332E+01_JPRB, 0.10215E+01_JPRB, 0.99274E+00_JPRB, 0.96794E+00_JPRB, 0.96102E+00_JPRB, 0.96390E+00_JPRB ,&
&  0.95634E+00_JPRB, 0.94460E+00_JPRB, 0.94684E+00_JPRB, 0.92882E+00_JPRB, 0.90681E+00_JPRB, 0.89267E+00_JPRB, 0.88911E+00_JPRB ,&
&  0.89556E+00_JPRB, 0.90745E+00_JPRB, 0.93775E+00_JPRB, 0.96958E+00_JPRB, 0.97823E+00_JPRB, 0.98271E+00_JPRB, 0.10451E+01_JPRB ,&
&  0.17558E+01_JPRB, 0.37754E+01_JPRB, 0.57226E+01_JPRB, 0.53485E+01_JPRB, 0.45772E+01_JPRB, 0.38325E+01_JPRB, 0.34285E+01_JPRB ,&
&  0.28231E+01_JPRB, 0.24456E+01_JPRB, 0.20690E+01_JPRB, 0.17784E+01_JPRB, 0.15787E+01_JPRB, 0.14550E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.13200E+01_JPRB, 0.12895E+01_JPRB, 0.12816E+01_JPRB, 0.12734E+01_JPRB, 0.12601E+01_JPRB, 0.12385E+01_JPRB, 0.12416E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.13642E+01_JPRB, 0.14770E+01_JPRB, 0.18077E+01_JPRB, 0.29762E+01_JPRB, 0.36435E+01_JPRB, 0.55476E+01_JPRB ,&
&  0.74967E+01_JPRB, 0.62507E+01_JPRB, 0.48108E+01_JPRB, 0.38566E+01_JPRB, 0.32694E+01_JPRB, 0.28859E+01_JPRB, 0.26557E+01_JPRB ,&
&  0.24243E+01_JPRB, 0.22714E+01_JPRB, 0.21294E+01_JPRB, 0.19914E+01_JPRB, 0.18963E+01_JPRB, 0.17678E+01_JPRB, 0.16511E+01_JPRB ,&
&  0.16020E+01_JPRB, 0.15405E+01_JPRB, 0.14732E+01_JPRB, 0.13972E+01_JPRB, 0.13690E+01_JPRB, 0.13544E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.13354E+01_JPRB, 0.13314E+01_JPRB, 0.13246E+01_JPRB, 0.13608E+01_JPRB, 0.13967E+01_JPRB, 0.13928E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.13477E+01_JPRB, 0.13671E+01_JPRB, 0.13287E+01_JPRB, 0.13045E+01_JPRB, 0.13690E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.11290E+01_JPRB, 0.10974E+01_JPRB, 0.11030E+01_JPRB, 0.11697E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.11753E+01_JPRB, 0.11677E+01_JPRB, 0.11450E+01_JPRB, 0.11230E+01_JPRB, 0.11107E+01_JPRB, 0.10906E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.10515E+01_JPRB, 0.10409E+01_JPRB, 0.10250E+01_JPRB, 0.10033E+01_JPRB, 0.98329E+00_JPRB, 0.97419E+00_JPRB ,&
&  0.97136E+00_JPRB, 0.96042E+00_JPRB, 0.96327E+00_JPRB, 0.96361E+00_JPRB, 0.95976E+00_JPRB, 0.94821E+00_JPRB, 0.93699E+00_JPRB ,&
&  0.92940E+00_JPRB, 0.91466E+00_JPRB, 0.91266E+00_JPRB, 0.93865E+00_JPRB, 0.96210E+00_JPRB, 0.97140E+00_JPRB, 0.99040E+00_JPRB ,&
&  0.13393E+01_JPRB, 0.28951E+01_JPRB, 0.41404E+01_JPRB, 0.44441E+01_JPRB, 0.41535E+01_JPRB, 0.35997E+01_JPRB, 0.30641E+01_JPRB ,&
&  0.25921E+01_JPRB, 0.22248E+01_JPRB, 0.19839E+01_JPRB, 0.17958E+01_JPRB, 0.16302E+01_JPRB, 0.14939E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.13177E+01_JPRB, 0.12742E+01_JPRB, 0.12456E+01_JPRB, 0.12291E+01_JPRB, 0.12160E+01_JPRB, 0.12052E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.12421E+01_JPRB, 0.12837E+01_JPRB, 0.13479E+01_JPRB, 0.14569E+01_JPRB, 0.19880E+01_JPRB, 0.29029E+01_JPRB, 0.41388E+01_JPRB ,&
&  0.45025E+01_JPRB, 0.45748E+01_JPRB, 0.41582E+01_JPRB, 0.35614E+01_JPRB, 0.30759E+01_JPRB, 0.27221E+01_JPRB, 0.24949E+01_JPRB ,&
&  0.22799E+01_JPRB, 0.21197E+01_JPRB, 0.19984E+01_JPRB, 0.18527E+01_JPRB, 0.17114E+01_JPRB, 0.16271E+01_JPRB, 0.16215E+01_JPRB ,&
&  0.16343E+01_JPRB, 0.15942E+01_JPRB, 0.15157E+01_JPRB, 0.14214E+01_JPRB, 0.13286E+01_JPRB, 0.12866E+01_JPRB, 0.12715E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.12674E+01_JPRB, 0.12935E+01_JPRB, 0.13216E+01_JPRB, 0.13393E+01_JPRB, 0.13459E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.13535E+01_JPRB, 0.13566E+01_JPRB, 0.13396E+01_JPRB, 0.13493E+01_JPRB, 0.13358E+01_JPRB, 0.13924E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.11623E+01_JPRB, 0.11278E+01_JPRB, 0.11097E+01_JPRB, 0.11560E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11819E+01_JPRB, 0.11720E+01_JPRB, 0.11514E+01_JPRB, 0.11333E+01_JPRB, 0.11090E+01_JPRB, 0.10871E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10389E+01_JPRB, 0.10285E+01_JPRB, 0.10188E+01_JPRB, 0.10129E+01_JPRB, 0.10005E+01_JPRB, 0.98224E+00_JPRB ,&
&  0.96365E+00_JPRB, 0.95260E+00_JPRB, 0.94816E+00_JPRB, 0.95093E+00_JPRB, 0.96007E+00_JPRB, 0.96645E+00_JPRB, 0.96175E+00_JPRB ,&
&  0.95200E+00_JPRB, 0.93586E+00_JPRB, 0.91756E+00_JPRB, 0.93615E+00_JPRB, 0.95754E+00_JPRB, 0.96691E+00_JPRB, 0.96855E+00_JPRB ,&
&  0.11804E+01_JPRB, 0.18880E+01_JPRB, 0.27136E+01_JPRB, 0.29448E+01_JPRB, 0.29413E+01_JPRB, 0.29351E+01_JPRB, 0.26616E+01_JPRB ,&
&  0.23693E+01_JPRB, 0.20657E+01_JPRB, 0.17987E+01_JPRB, 0.16478E+01_JPRB, 0.15502E+01_JPRB, 0.14536E+01_JPRB, 0.13751E+01_JPRB ,&
&  0.12902E+01_JPRB, 0.12399E+01_JPRB, 0.12067E+01_JPRB, 0.11931E+01_JPRB, 0.11873E+01_JPRB, 0.11873E+01_JPRB, 0.11934E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.12073E+01_JPRB, 0.12343E+01_JPRB, 0.12852E+01_JPRB, 0.14052E+01_JPRB, 0.17845E+01_JPRB, 0.23825E+01_JPRB ,&
&  0.27896E+01_JPRB, 0.30909E+01_JPRB, 0.32910E+01_JPRB, 0.32517E+01_JPRB, 0.29654E+01_JPRB, 0.26342E+01_JPRB, 0.23881E+01_JPRB ,&
&  0.21856E+01_JPRB, 0.19839E+01_JPRB, 0.18092E+01_JPRB, 0.16639E+01_JPRB, 0.15386E+01_JPRB, 0.14843E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.15484E+01_JPRB, 0.15243E+01_JPRB, 0.14558E+01_JPRB, 0.13756E+01_JPRB, 0.12834E+01_JPRB, 0.12276E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12219E+01_JPRB, 0.12424E+01_JPRB, 0.12694E+01_JPRB, 0.12927E+01_JPRB, 0.13052E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.13138E+01_JPRB, 0.13186E+01_JPRB, 0.12600E+01_JPRB, 0.12720E+01_JPRB, 0.13701E+01_JPRB, 0.13484E+01_JPRB, 0.13174E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.11835E+01_JPRB, 0.11298E+01_JPRB, 0.10939E+01_JPRB, 0.11397E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.10348E+01_JPRB, 0.10586E+01_JPRB, 0.10660E+01_JPRB, 0.10652E+01_JPRB, 0.10516E+01_JPRB, 0.10299E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.98837E+00_JPRB, 0.98081E+00_JPRB, 0.97698E+00_JPRB, 0.97899E+00_JPRB, 0.98425E+00_JPRB, 0.98160E+00_JPRB ,&
&  0.96919E+00_JPRB, 0.95207E+00_JPRB, 0.94024E+00_JPRB, 0.93829E+00_JPRB, 0.94722E+00_JPRB, 0.95767E+00_JPRB, 0.96439E+00_JPRB ,&
&  0.96659E+00_JPRB, 0.95860E+00_JPRB, 0.95360E+00_JPRB, 0.96578E+00_JPRB, 0.97854E+00_JPRB, 0.97375E+00_JPRB, 0.96076E+00_JPRB ,&
&  0.10523E+01_JPRB, 0.12506E+01_JPRB, 0.15637E+01_JPRB, 0.18353E+01_JPRB, 0.20769E+01_JPRB, 0.21641E+01_JPRB, 0.20933E+01_JPRB ,&
&  0.19397E+01_JPRB, 0.18013E+01_JPRB, 0.16412E+01_JPRB, 0.14960E+01_JPRB, 0.13891E+01_JPRB, 0.13305E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.11940E+01_JPRB, 0.11515E+01_JPRB, 0.11270E+01_JPRB, 0.11172E+01_JPRB, 0.11083E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.10793E+01_JPRB, 0.10911E+01_JPRB, 0.11231E+01_JPRB, 0.11783E+01_JPRB, 0.13226E+01_JPRB, 0.15920E+01_JPRB ,&
&  0.18403E+01_JPRB, 0.21013E+01_JPRB, 0.23565E+01_JPRB, 0.25532E+01_JPRB, 0.25443E+01_JPRB, 0.24124E+01_JPRB, 0.22331E+01_JPRB ,&
&  0.20273E+01_JPRB, 0.17821E+01_JPRB, 0.15594E+01_JPRB, 0.14275E+01_JPRB, 0.13825E+01_JPRB, 0.13894E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.14466E+01_JPRB, 0.14202E+01_JPRB, 0.13465E+01_JPRB, 0.12720E+01_JPRB, 0.12165E+01_JPRB, 0.11701E+01_JPRB, 0.11537E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.11539E+01_JPRB, 0.11718E+01_JPRB, 0.11917E+01_JPRB, 0.12184E+01_JPRB, 0.12361E+01_JPRB, 0.12378E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.12265E+01_JPRB, 0.12189E+01_JPRB, 0.12484E+01_JPRB, 0.12427E+01_JPRB, 0.12375E+01_JPRB, 0.12225E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11419E+01_JPRB, 0.10870E+01_JPRB, 0.10531E+01_JPRB, 0.97964E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.96685E+00_JPRB, 0.98803E+00_JPRB, 0.99576E+00_JPRB, 0.99556E+00_JPRB, 0.99204E+00_JPRB, 0.97769E+00_JPRB, 0.96003E+00_JPRB ,&
&  0.94070E+00_JPRB, 0.93301E+00_JPRB, 0.93046E+00_JPRB, 0.93459E+00_JPRB, 0.93961E+00_JPRB, 0.94728E+00_JPRB, 0.95818E+00_JPRB ,&
&  0.96828E+00_JPRB, 0.97184E+00_JPRB, 0.97412E+00_JPRB, 0.97070E+00_JPRB, 0.96771E+00_JPRB, 0.96902E+00_JPRB, 0.97236E+00_JPRB ,&
&  0.98029E+00_JPRB, 0.98185E+00_JPRB, 0.97642E+00_JPRB, 0.98577E+00_JPRB, 0.10036E+01_JPRB, 0.99576E+00_JPRB, 0.98075E+00_JPRB ,&
&  0.10116E+01_JPRB, 0.97720E+00_JPRB, 0.10513E+01_JPRB, 0.12848E+01_JPRB, 0.15195E+01_JPRB, 0.16986E+01_JPRB, 0.17264E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.15562E+01_JPRB, 0.14576E+01_JPRB, 0.13592E+01_JPRB, 0.12873E+01_JPRB, 0.12244E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11532E+01_JPRB, 0.11228E+01_JPRB, 0.10812E+01_JPRB, 0.10437E+01_JPRB, 0.10118E+01_JPRB, 0.98176E+00_JPRB, 0.96261E+00_JPRB ,&
&  0.95488E+00_JPRB, 0.96150E+00_JPRB, 0.97579E+00_JPRB, 0.10014E+01_JPRB, 0.10495E+01_JPRB, 0.11195E+01_JPRB, 0.12397E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.15365E+01_JPRB, 0.16872E+01_JPRB, 0.18191E+01_JPRB, 0.19273E+01_JPRB, 0.19377E+01_JPRB, 0.18413E+01_JPRB ,&
&  0.16738E+01_JPRB, 0.15335E+01_JPRB, 0.14245E+01_JPRB, 0.13500E+01_JPRB, 0.13325E+01_JPRB, 0.13472E+01_JPRB, 0.13562E+01_JPRB ,&
&  0.13347E+01_JPRB, 0.12825E+01_JPRB, 0.12144E+01_JPRB, 0.11512E+01_JPRB, 0.11122E+01_JPRB, 0.10862E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.10383E+01_JPRB, 0.10449E+01_JPRB, 0.10660E+01_JPRB, 0.10954E+01_JPRB, 0.11264E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11470E+01_JPRB, 0.11514E+01_JPRB, 0.11607E+01_JPRB, 0.11183E+01_JPRB, 0.11594E+01_JPRB, 0.11677E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11083E+01_JPRB, 0.10611E+01_JPRB, 0.89194E+00_JPRB, 0.94403E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.93450E+00_JPRB, 0.93789E+00_JPRB, 0.94070E+00_JPRB, 0.93547E+00_JPRB, 0.93059E+00_JPRB, 0.92218E+00_JPRB, 0.90534E+00_JPRB ,&
&  0.88953E+00_JPRB, 0.88385E+00_JPRB, 0.88443E+00_JPRB, 0.88612E+00_JPRB, 0.88916E+00_JPRB, 0.89505E+00_JPRB, 0.90583E+00_JPRB ,&
&  0.91879E+00_JPRB, 0.93198E+00_JPRB, 0.94741E+00_JPRB, 0.96104E+00_JPRB, 0.96245E+00_JPRB, 0.95726E+00_JPRB, 0.95544E+00_JPRB ,&
&  0.95988E+00_JPRB, 0.95756E+00_JPRB, 0.94875E+00_JPRB, 0.94984E+00_JPRB, 0.96212E+00_JPRB, 0.95679E+00_JPRB, 0.94094E+00_JPRB ,&
&  0.94765E+00_JPRB, 0.84559E+00_JPRB, 0.91141E+00_JPRB, 0.10236E+01_JPRB, 0.11893E+01_JPRB, 0.13255E+01_JPRB, 0.13850E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.13004E+01_JPRB, 0.12349E+01_JPRB, 0.11617E+01_JPRB, 0.11172E+01_JPRB, 0.10895E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10247E+01_JPRB, 0.98432E+00_JPRB, 0.93872E+00_JPRB, 0.89593E+00_JPRB, 0.86233E+00_JPRB, 0.84276E+00_JPRB, 0.83302E+00_JPRB ,&
&  0.82907E+00_JPRB, 0.83834E+00_JPRB, 0.84565E+00_JPRB, 0.86091E+00_JPRB, 0.88855E+00_JPRB, 0.93321E+00_JPRB, 0.99634E+00_JPRB ,&
&  0.10843E+01_JPRB, 0.11891E+01_JPRB, 0.12913E+01_JPRB, 0.13635E+01_JPRB, 0.14079E+01_JPRB, 0.14384E+01_JPRB, 0.14303E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.13158E+01_JPRB, 0.13015E+01_JPRB, 0.12936E+01_JPRB, 0.12718E+01_JPRB, 0.12477E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11044E+01_JPRB, 0.10526E+01_JPRB, 0.10053E+01_JPRB, 0.97439E+00_JPRB, 0.95655E+00_JPRB, 0.93139E+00_JPRB ,&
&  0.91527E+00_JPRB, 0.91250E+00_JPRB, 0.92214E+00_JPRB, 0.94569E+00_JPRB, 0.97801E+00_JPRB, 0.10091E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.10377E+01_JPRB, 0.10402E+01_JPRB, 0.10659E+01_JPRB, 0.10726E+01_JPRB, 0.10754E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10633E+01_JPRB, 0.84187E+00_JPRB, 0.90895E+00_JPRB, 0.91985E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.94418E+00_JPRB, 0.92209E+00_JPRB, 0.90955E+00_JPRB, 0.90179E+00_JPRB, 0.89602E+00_JPRB, 0.88848E+00_JPRB, 0.87424E+00_JPRB ,&
&  0.85936E+00_JPRB, 0.85147E+00_JPRB, 0.84484E+00_JPRB, 0.83673E+00_JPRB, 0.83890E+00_JPRB, 0.84559E+00_JPRB, 0.85730E+00_JPRB ,&
&  0.86967E+00_JPRB, 0.88497E+00_JPRB, 0.89917E+00_JPRB, 0.91661E+00_JPRB, 0.92688E+00_JPRB, 0.93312E+00_JPRB, 0.93540E+00_JPRB ,&
&  0.93654E+00_JPRB, 0.93567E+00_JPRB, 0.92625E+00_JPRB, 0.91424E+00_JPRB, 0.91252E+00_JPRB, 0.91866E+00_JPRB, 0.91492E+00_JPRB ,&
&  0.81902E+00_JPRB, 0.75056E+00_JPRB, 0.82553E+00_JPRB, 0.91443E+00_JPRB, 0.10137E+01_JPRB, 0.11019E+01_JPRB, 0.11440E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.10792E+01_JPRB, 0.10360E+01_JPRB, 0.98644E+00_JPRB, 0.96071E+00_JPRB, 0.94746E+00_JPRB, 0.93323E+00_JPRB ,&
&  0.92113E+00_JPRB, 0.90581E+00_JPRB, 0.88309E+00_JPRB, 0.84734E+00_JPRB, 0.81967E+00_JPRB, 0.79926E+00_JPRB, 0.79182E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.79304E+00_JPRB, 0.79626E+00_JPRB, 0.80009E+00_JPRB, 0.80992E+00_JPRB, 0.82792E+00_JPRB, 0.85299E+00_JPRB ,&
&  0.89254E+00_JPRB, 0.94225E+00_JPRB, 0.10040E+01_JPRB, 0.10474E+01_JPRB, 0.10723E+01_JPRB, 0.10938E+01_JPRB, 0.11219E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.11316E+01_JPRB, 0.11007E+01_JPRB, 0.10846E+01_JPRB, 0.10786E+01_JPRB, 0.10681E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.96010E+00_JPRB, 0.91166E+00_JPRB, 0.88095E+00_JPRB, 0.87157E+00_JPRB, 0.86733E+00_JPRB, 0.85920E+00_JPRB ,&
&  0.85220E+00_JPRB, 0.85094E+00_JPRB, 0.85721E+00_JPRB, 0.87015E+00_JPRB, 0.88773E+00_JPRB, 0.90880E+00_JPRB, 0.92555E+00_JPRB ,&
&  0.94039E+00_JPRB, 0.95377E+00_JPRB, 0.96313E+00_JPRB, 0.98050E+00_JPRB, 0.99827E+00_JPRB, 0.10066E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.99219E+00_JPRB, 0.98071E+00_JPRB, 0.97389E+00_JPRB, 0.96277E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.90081E+00_JPRB, 0.88001E+00_JPRB, 0.86745E+00_JPRB, 0.86172E+00_JPRB, 0.85185E+00_JPRB, 0.83190E+00_JPRB, 0.81542E+00_JPRB ,&
&  0.79977E+00_JPRB, 0.79046E+00_JPRB, 0.78590E+00_JPRB, 0.78475E+00_JPRB, 0.79098E+00_JPRB, 0.79730E+00_JPRB, 0.80522E+00_JPRB ,&
&  0.81759E+00_JPRB, 0.83686E+00_JPRB, 0.85276E+00_JPRB, 0.86637E+00_JPRB, 0.87782E+00_JPRB, 0.88873E+00_JPRB, 0.89020E+00_JPRB ,&
&  0.89109E+00_JPRB, 0.89723E+00_JPRB, 0.89712E+00_JPRB, 0.88224E+00_JPRB, 0.86762E+00_JPRB, 0.87320E+00_JPRB, 0.87243E+00_JPRB ,&
&  0.86472E+00_JPRB, 0.75278E+00_JPRB, 0.81978E+00_JPRB, 0.85949E+00_JPRB, 0.90597E+00_JPRB, 0.95630E+00_JPRB, 0.98350E+00_JPRB ,&
&  0.97637E+00_JPRB, 0.94694E+00_JPRB, 0.91644E+00_JPRB, 0.88344E+00_JPRB, 0.87304E+00_JPRB, 0.86339E+00_JPRB, 0.84958E+00_JPRB ,&
&  0.84017E+00_JPRB, 0.83136E+00_JPRB, 0.81881E+00_JPRB, 0.80274E+00_JPRB, 0.78332E+00_JPRB, 0.76848E+00_JPRB, 0.75504E+00_JPRB ,&
&  0.74932E+00_JPRB, 0.73947E+00_JPRB, 0.73134E+00_JPRB, 0.73200E+00_JPRB, 0.74486E+00_JPRB, 0.75323E+00_JPRB, 0.75858E+00_JPRB ,&
&  0.76753E+00_JPRB, 0.78199E+00_JPRB, 0.80447E+00_JPRB, 0.83199E+00_JPRB, 0.85443E+00_JPRB, 0.87149E+00_JPRB, 0.89392E+00_JPRB ,&
&  0.92686E+00_JPRB, 0.94882E+00_JPRB, 0.94839E+00_JPRB, 0.92945E+00_JPRB, 0.91436E+00_JPRB, 0.90757E+00_JPRB, 0.88875E+00_JPRB ,&
&  0.85866E+00_JPRB, 0.83441E+00_JPRB, 0.82198E+00_JPRB, 0.81530E+00_JPRB, 0.80825E+00_JPRB, 0.80106E+00_JPRB, 0.79992E+00_JPRB ,&
&  0.80101E+00_JPRB, 0.80059E+00_JPRB, 0.79944E+00_JPRB, 0.80247E+00_JPRB, 0.80916E+00_JPRB, 0.81791E+00_JPRB, 0.82970E+00_JPRB ,&
&  0.84645E+00_JPRB, 0.86142E+00_JPRB, 0.86886E+00_JPRB, 0.87933E+00_JPRB, 0.89230E+00_JPRB, 0.89823E+00_JPRB, 0.91284E+00_JPRB ,&
&  0.91337E+00_JPRB, 0.91000E+00_JPRB, 0.90871E+00_JPRB, 0.91370E+00_JPRB, 0.91459E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.85586E+00_JPRB, 0.83464E+00_JPRB, 0.82148E+00_JPRB, 0.80914E+00_JPRB, 0.80052E+00_JPRB, 0.78206E+00_JPRB, 0.77136E+00_JPRB ,&
&  0.76398E+00_JPRB, 0.75892E+00_JPRB, 0.75688E+00_JPRB, 0.75659E+00_JPRB, 0.76714E+00_JPRB, 0.77433E+00_JPRB, 0.77757E+00_JPRB ,&
&  0.78590E+00_JPRB, 0.80443E+00_JPRB, 0.82173E+00_JPRB, 0.83578E+00_JPRB, 0.84617E+00_JPRB, 0.85291E+00_JPRB, 0.85139E+00_JPRB ,&
&  0.85374E+00_JPRB, 0.86057E+00_JPRB, 0.86340E+00_JPRB, 0.85142E+00_JPRB, 0.83671E+00_JPRB, 0.82908E+00_JPRB, 0.83102E+00_JPRB ,&
&  0.82021E+00_JPRB, 0.79169E+00_JPRB, 0.81202E+00_JPRB, 0.82683E+00_JPRB, 0.84514E+00_JPRB, 0.86333E+00_JPRB, 0.88187E+00_JPRB ,&
&  0.88883E+00_JPRB, 0.86894E+00_JPRB, 0.84033E+00_JPRB, 0.82032E+00_JPRB, 0.82438E+00_JPRB, 0.82025E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.79654E+00_JPRB, 0.78768E+00_JPRB, 0.77674E+00_JPRB, 0.76154E+00_JPRB, 0.74820E+00_JPRB, 0.73826E+00_JPRB, 0.73058E+00_JPRB ,&
&  0.71659E+00_JPRB, 0.70512E+00_JPRB, 0.69897E+00_JPRB, 0.69253E+00_JPRB, 0.68792E+00_JPRB, 0.69337E+00_JPRB, 0.69942E+00_JPRB ,&
&  0.70307E+00_JPRB, 0.70941E+00_JPRB, 0.72249E+00_JPRB, 0.74217E+00_JPRB, 0.76048E+00_JPRB, 0.78002E+00_JPRB, 0.80146E+00_JPRB ,&
&  0.82351E+00_JPRB, 0.84350E+00_JPRB, 0.85131E+00_JPRB, 0.85533E+00_JPRB, 0.84749E+00_JPRB, 0.83478E+00_JPRB, 0.81062E+00_JPRB ,&
&  0.78364E+00_JPRB, 0.77453E+00_JPRB, 0.77224E+00_JPRB, 0.77564E+00_JPRB, 0.77227E+00_JPRB, 0.76897E+00_JPRB, 0.76767E+00_JPRB ,&
&  0.76399E+00_JPRB, 0.76031E+00_JPRB, 0.75490E+00_JPRB, 0.74997E+00_JPRB, 0.75326E+00_JPRB, 0.75927E+00_JPRB, 0.77299E+00_JPRB ,&
&  0.79171E+00_JPRB, 0.81157E+00_JPRB, 0.83116E+00_JPRB, 0.84365E+00_JPRB, 0.84725E+00_JPRB, 0.84927E+00_JPRB, 0.85244E+00_JPRB ,&
&  0.85343E+00_JPRB, 0.85150E+00_JPRB, 0.85750E+00_JPRB, 0.86198E+00_JPRB, 0.86662E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.79305E+00_JPRB, 0.76767E+00_JPRB, 0.74908E+00_JPRB, 0.74219E+00_JPRB, 0.73841E+00_JPRB, 0.72572E+00_JPRB, 0.71956E+00_JPRB ,&
&  0.72482E+00_JPRB, 0.72614E+00_JPRB, 0.73273E+00_JPRB, 0.73192E+00_JPRB, 0.73110E+00_JPRB, 0.73381E+00_JPRB, 0.72622E+00_JPRB ,&
&  0.72641E+00_JPRB, 0.73591E+00_JPRB, 0.74725E+00_JPRB, 0.76251E+00_JPRB, 0.77530E+00_JPRB, 0.78484E+00_JPRB, 0.79251E+00_JPRB ,&
&  0.79525E+00_JPRB, 0.80132E+00_JPRB, 0.80445E+00_JPRB, 0.79941E+00_JPRB, 0.78699E+00_JPRB, 0.77701E+00_JPRB, 0.77478E+00_JPRB ,&
&  0.77271E+00_JPRB, 0.77516E+00_JPRB, 0.73159E+00_JPRB, 0.75818E+00_JPRB, 0.76696E+00_JPRB, 0.77540E+00_JPRB, 0.78373E+00_JPRB ,&
&  0.78382E+00_JPRB, 0.77228E+00_JPRB, 0.76490E+00_JPRB, 0.74923E+00_JPRB, 0.74677E+00_JPRB, 0.75261E+00_JPRB, 0.74748E+00_JPRB ,&
&  0.73808E+00_JPRB, 0.72284E+00_JPRB, 0.70554E+00_JPRB, 0.69321E+00_JPRB, 0.68974E+00_JPRB, 0.68604E+00_JPRB, 0.67559E+00_JPRB ,&
&  0.66909E+00_JPRB, 0.66813E+00_JPRB, 0.66632E+00_JPRB, 0.65983E+00_JPRB, 0.64422E+00_JPRB, 0.63059E+00_JPRB, 0.62914E+00_JPRB ,&
&  0.63496E+00_JPRB, 0.64511E+00_JPRB, 0.66310E+00_JPRB, 0.68534E+00_JPRB, 0.70808E+00_JPRB, 0.72905E+00_JPRB, 0.74743E+00_JPRB ,&
&  0.75501E+00_JPRB, 0.75862E+00_JPRB, 0.76259E+00_JPRB, 0.76880E+00_JPRB, 0.77024E+00_JPRB, 0.76125E+00_JPRB, 0.73782E+00_JPRB ,&
&  0.71320E+00_JPRB, 0.70559E+00_JPRB, 0.70874E+00_JPRB, 0.71736E+00_JPRB, 0.72137E+00_JPRB, 0.72089E+00_JPRB, 0.71480E+00_JPRB ,&
&  0.71114E+00_JPRB, 0.70287E+00_JPRB, 0.69344E+00_JPRB, 0.68856E+00_JPRB, 0.68779E+00_JPRB, 0.69372E+00_JPRB, 0.71174E+00_JPRB ,&
&  0.73387E+00_JPRB, 0.75830E+00_JPRB, 0.77901E+00_JPRB, 0.78596E+00_JPRB, 0.78964E+00_JPRB, 0.78769E+00_JPRB, 0.77769E+00_JPRB ,&
&  0.78174E+00_JPRB, 0.80099E+00_JPRB, 0.81076E+00_JPRB, 0.81203E+00_JPRB, 0.80905E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.73538E+00_JPRB, 0.70666E+00_JPRB, 0.69527E+00_JPRB, 0.69743E+00_JPRB, 0.70076E+00_JPRB, 0.69601E+00_JPRB, 0.70276E+00_JPRB ,&
&  0.71354E+00_JPRB, 0.72186E+00_JPRB, 0.72637E+00_JPRB, 0.72221E+00_JPRB, 0.71940E+00_JPRB, 0.71608E+00_JPRB, 0.70078E+00_JPRB ,&
&  0.69546E+00_JPRB, 0.69362E+00_JPRB, 0.69815E+00_JPRB, 0.70608E+00_JPRB, 0.71496E+00_JPRB, 0.72822E+00_JPRB, 0.73802E+00_JPRB ,&
&  0.74423E+00_JPRB, 0.74792E+00_JPRB, 0.75198E+00_JPRB, 0.75194E+00_JPRB, 0.74785E+00_JPRB, 0.73776E+00_JPRB, 0.72994E+00_JPRB ,&
&  0.72513E+00_JPRB, 0.71560E+00_JPRB, 0.70479E+00_JPRB, 0.70377E+00_JPRB, 0.70737E+00_JPRB, 0.71129E+00_JPRB, 0.71340E+00_JPRB ,&
&  0.70448E+00_JPRB, 0.69071E+00_JPRB, 0.69112E+00_JPRB, 0.69011E+00_JPRB, 0.69397E+00_JPRB, 0.70566E+00_JPRB, 0.70814E+00_JPRB ,&
&  0.70257E+00_JPRB, 0.68563E+00_JPRB, 0.66408E+00_JPRB, 0.65922E+00_JPRB, 0.66052E+00_JPRB, 0.65090E+00_JPRB, 0.64205E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.63773E+00_JPRB, 0.63184E+00_JPRB, 0.62359E+00_JPRB, 0.61025E+00_JPRB, 0.59313E+00_JPRB, 0.58483E+00_JPRB ,&
&  0.58553E+00_JPRB, 0.60388E+00_JPRB, 0.63573E+00_JPRB, 0.67038E+00_JPRB, 0.70178E+00_JPRB, 0.71637E+00_JPRB, 0.71495E+00_JPRB ,&
&  0.70088E+00_JPRB, 0.69618E+00_JPRB, 0.70085E+00_JPRB, 0.70586E+00_JPRB, 0.70770E+00_JPRB, 0.69671E+00_JPRB, 0.67749E+00_JPRB ,&
&  0.66306E+00_JPRB, 0.65318E+00_JPRB, 0.65243E+00_JPRB, 0.65762E+00_JPRB, 0.66239E+00_JPRB, 0.66359E+00_JPRB, 0.65810E+00_JPRB ,&
&  0.65285E+00_JPRB, 0.64682E+00_JPRB, 0.64249E+00_JPRB, 0.64396E+00_JPRB, 0.64754E+00_JPRB, 0.65184E+00_JPRB, 0.66616E+00_JPRB ,&
&  0.68710E+00_JPRB, 0.70724E+00_JPRB, 0.71144E+00_JPRB, 0.70675E+00_JPRB, 0.70575E+00_JPRB, 0.70250E+00_JPRB, 0.69935E+00_JPRB ,&
&  0.70918E+00_JPRB, 0.73184E+00_JPRB, 0.74805E+00_JPRB, 0.75653E+00_JPRB, 0.75619E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.65424E+00_JPRB, 0.63752E+00_JPRB, 0.63859E+00_JPRB, 0.64673E+00_JPRB, 0.65130E+00_JPRB, 0.65918E+00_JPRB, 0.67849E+00_JPRB ,&
&  0.68915E+00_JPRB, 0.69576E+00_JPRB, 0.69819E+00_JPRB, 0.69297E+00_JPRB, 0.69201E+00_JPRB, 0.68468E+00_JPRB, 0.67162E+00_JPRB ,&
&  0.66605E+00_JPRB, 0.66491E+00_JPRB, 0.66663E+00_JPRB, 0.67081E+00_JPRB, 0.67357E+00_JPRB, 0.67724E+00_JPRB, 0.68132E+00_JPRB ,&
&  0.68362E+00_JPRB, 0.68373E+00_JPRB, 0.68273E+00_JPRB, 0.68276E+00_JPRB, 0.68150E+00_JPRB, 0.67329E+00_JPRB, 0.66386E+00_JPRB ,&
&  0.65398E+00_JPRB, 0.64527E+00_JPRB, 0.64227E+00_JPRB, 0.64607E+00_JPRB, 0.65064E+00_JPRB, 0.65321E+00_JPRB, 0.65090E+00_JPRB ,&
&  0.64745E+00_JPRB, 0.64794E+00_JPRB, 0.65318E+00_JPRB, 0.66037E+00_JPRB, 0.66839E+00_JPRB, 0.67769E+00_JPRB, 0.68256E+00_JPRB ,&
&  0.67615E+00_JPRB, 0.65703E+00_JPRB, 0.64509E+00_JPRB, 0.64389E+00_JPRB, 0.63818E+00_JPRB, 0.62031E+00_JPRB, 0.60840E+00_JPRB ,&
&  0.60334E+00_JPRB, 0.59405E+00_JPRB, 0.58432E+00_JPRB, 0.57163E+00_JPRB, 0.56027E+00_JPRB, 0.55587E+00_JPRB, 0.55486E+00_JPRB ,&
&  0.55849E+00_JPRB, 0.57963E+00_JPRB, 0.61748E+00_JPRB, 0.65295E+00_JPRB, 0.66756E+00_JPRB, 0.66012E+00_JPRB, 0.64143E+00_JPRB ,&
&  0.63168E+00_JPRB, 0.62689E+00_JPRB, 0.62783E+00_JPRB, 0.62822E+00_JPRB, 0.62543E+00_JPRB, 0.61642E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.59706E+00_JPRB, 0.58508E+00_JPRB, 0.58054E+00_JPRB, 0.57813E+00_JPRB, 0.57550E+00_JPRB, 0.57616E+00_JPRB, 0.57615E+00_JPRB ,&
&  0.57766E+00_JPRB, 0.58190E+00_JPRB, 0.59307E+00_JPRB, 0.59777E+00_JPRB, 0.59659E+00_JPRB, 0.58588E+00_JPRB, 0.58782E+00_JPRB ,&
&  0.59185E+00_JPRB, 0.59012E+00_JPRB, 0.58543E+00_JPRB, 0.58521E+00_JPRB, 0.58532E+00_JPRB, 0.58762E+00_JPRB, 0.60037E+00_JPRB ,&
&  0.62805E+00_JPRB, 0.65157E+00_JPRB, 0.67074E+00_JPRB, 0.67581E+00_JPRB, 0.67162E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.59446E+00_JPRB, 0.60005E+00_JPRB, 0.61387E+00_JPRB, 0.62522E+00_JPRB, 0.62898E+00_JPRB, 0.63896E+00_JPRB, 0.65001E+00_JPRB ,&
&  0.65816E+00_JPRB, 0.65909E+00_JPRB, 0.65497E+00_JPRB, 0.64501E+00_JPRB, 0.63338E+00_JPRB, 0.62371E+00_JPRB, 0.62007E+00_JPRB ,&
&  0.62233E+00_JPRB, 0.62529E+00_JPRB, 0.63083E+00_JPRB, 0.63407E+00_JPRB, 0.63818E+00_JPRB, 0.63750E+00_JPRB, 0.63858E+00_JPRB ,&
&  0.64102E+00_JPRB, 0.64246E+00_JPRB, 0.64302E+00_JPRB, 0.63905E+00_JPRB, 0.63572E+00_JPRB, 0.63266E+00_JPRB, 0.62319E+00_JPRB ,&
&  0.61298E+00_JPRB, 0.60664E+00_JPRB, 0.61053E+00_JPRB, 0.62139E+00_JPRB, 0.62803E+00_JPRB, 0.62434E+00_JPRB, 0.62225E+00_JPRB ,&
&  0.62256E+00_JPRB, 0.62883E+00_JPRB, 0.63895E+00_JPRB, 0.65252E+00_JPRB, 0.66066E+00_JPRB, 0.67105E+00_JPRB, 0.67156E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.65547E+00_JPRB, 0.65218E+00_JPRB, 0.65012E+00_JPRB, 0.61819E+00_JPRB, 0.58313E+00_JPRB, 0.56608E+00_JPRB ,&
&  0.55684E+00_JPRB, 0.54573E+00_JPRB, 0.52979E+00_JPRB, 0.51894E+00_JPRB, 0.51828E+00_JPRB, 0.52375E+00_JPRB, 0.53558E+00_JPRB ,&
&  0.54442E+00_JPRB, 0.55779E+00_JPRB, 0.57459E+00_JPRB, 0.58591E+00_JPRB, 0.58636E+00_JPRB, 0.57343E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.55303E+00_JPRB, 0.55037E+00_JPRB, 0.54848E+00_JPRB, 0.54650E+00_JPRB, 0.54481E+00_JPRB, 0.54422E+00_JPRB, 0.54338E+00_JPRB ,&
&  0.53929E+00_JPRB, 0.53217E+00_JPRB, 0.52361E+00_JPRB, 0.51923E+00_JPRB, 0.51950E+00_JPRB, 0.52072E+00_JPRB, 0.52325E+00_JPRB ,&
&  0.53218E+00_JPRB, 0.54351E+00_JPRB, 0.55204E+00_JPRB, 0.55457E+00_JPRB, 0.54997E+00_JPRB, 0.54084E+00_JPRB, 0.53548E+00_JPRB ,&
&  0.53186E+00_JPRB, 0.52937E+00_JPRB, 0.52596E+00_JPRB, 0.52477E+00_JPRB, 0.52526E+00_JPRB, 0.53402E+00_JPRB, 0.55155E+00_JPRB ,&
&  0.57226E+00_JPRB, 0.59831E+00_JPRB, 0.60968E+00_JPRB, 0.60781E+00_JPRB, 0.60064E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.56183E+00_JPRB, 0.57537E+00_JPRB, 0.59219E+00_JPRB, 0.60371E+00_JPRB, 0.60844E+00_JPRB, 0.61346E+00_JPRB, 0.61552E+00_JPRB ,&
&  0.60926E+00_JPRB, 0.60631E+00_JPRB, 0.60067E+00_JPRB, 0.59303E+00_JPRB, 0.58644E+00_JPRB, 0.58334E+00_JPRB, 0.58401E+00_JPRB ,&
&  0.58624E+00_JPRB, 0.58816E+00_JPRB, 0.58494E+00_JPRB, 0.58353E+00_JPRB, 0.58805E+00_JPRB, 0.59410E+00_JPRB, 0.59822E+00_JPRB ,&
&  0.60037E+00_JPRB, 0.60249E+00_JPRB, 0.60772E+00_JPRB, 0.61094E+00_JPRB, 0.61093E+00_JPRB, 0.60392E+00_JPRB, 0.59841E+00_JPRB ,&
&  0.59286E+00_JPRB, 0.59077E+00_JPRB, 0.59613E+00_JPRB, 0.58874E+00_JPRB, 0.60270E+00_JPRB, 0.60365E+00_JPRB, 0.60825E+00_JPRB ,&
&  0.61576E+00_JPRB, 0.62965E+00_JPRB, 0.64140E+00_JPRB, 0.65080E+00_JPRB, 0.65344E+00_JPRB, 0.65615E+00_JPRB, 0.64576E+00_JPRB ,&
&  0.63176E+00_JPRB, 0.61312E+00_JPRB, 0.59412E+00_JPRB, 0.57238E+00_JPRB, 0.54997E+00_JPRB, 0.53274E+00_JPRB, 0.52489E+00_JPRB ,&
&  0.51530E+00_JPRB, 0.50107E+00_JPRB, 0.48773E+00_JPRB, 0.48283E+00_JPRB, 0.48770E+00_JPRB, 0.49790E+00_JPRB, 0.51326E+00_JPRB ,&
&  0.52301E+00_JPRB, 0.52543E+00_JPRB, 0.52247E+00_JPRB, 0.51887E+00_JPRB, 0.51499E+00_JPRB, 0.50683E+00_JPRB, 0.50190E+00_JPRB ,&
&  0.49809E+00_JPRB, 0.49271E+00_JPRB, 0.49016E+00_JPRB, 0.49141E+00_JPRB, 0.49408E+00_JPRB, 0.49615E+00_JPRB, 0.49549E+00_JPRB ,&
&  0.49353E+00_JPRB, 0.48872E+00_JPRB, 0.48076E+00_JPRB, 0.47742E+00_JPRB, 0.47902E+00_JPRB, 0.48418E+00_JPRB, 0.49168E+00_JPRB ,&
&  0.49403E+00_JPRB, 0.50054E+00_JPRB, 0.50644E+00_JPRB, 0.50800E+00_JPRB, 0.50997E+00_JPRB, 0.51111E+00_JPRB, 0.51054E+00_JPRB ,&
&  0.50816E+00_JPRB, 0.50277E+00_JPRB, 0.49605E+00_JPRB, 0.49426E+00_JPRB, 0.49664E+00_JPRB, 0.50483E+00_JPRB, 0.51493E+00_JPRB ,&
&  0.52452E+00_JPRB, 0.53958E+00_JPRB, 0.55075E+00_JPRB, 0.55309E+00_JPRB, 0.55527E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.53800E+00_JPRB, 0.55025E+00_JPRB, 0.56440E+00_JPRB, 0.57378E+00_JPRB, 0.57236E+00_JPRB, 0.57181E+00_JPRB, 0.57727E+00_JPRB ,&
&  0.56973E+00_JPRB, 0.56163E+00_JPRB, 0.55547E+00_JPRB, 0.54971E+00_JPRB, 0.54824E+00_JPRB, 0.54764E+00_JPRB, 0.55126E+00_JPRB ,&
&  0.55397E+00_JPRB, 0.55336E+00_JPRB, 0.55227E+00_JPRB, 0.55372E+00_JPRB, 0.56115E+00_JPRB, 0.57024E+00_JPRB, 0.57787E+00_JPRB ,&
&  0.57992E+00_JPRB, 0.58423E+00_JPRB, 0.59078E+00_JPRB, 0.59635E+00_JPRB, 0.59901E+00_JPRB, 0.60033E+00_JPRB, 0.59866E+00_JPRB ,&
&  0.59546E+00_JPRB, 0.59105E+00_JPRB, 0.57372E+00_JPRB, 0.58514E+00_JPRB, 0.58972E+00_JPRB, 0.59000E+00_JPRB, 0.59373E+00_JPRB ,&
&  0.60086E+00_JPRB, 0.61027E+00_JPRB, 0.61521E+00_JPRB, 0.61713E+00_JPRB, 0.61223E+00_JPRB, 0.60232E+00_JPRB, 0.57716E+00_JPRB ,&
&  0.55609E+00_JPRB, 0.53456E+00_JPRB, 0.52101E+00_JPRB, 0.51223E+00_JPRB, 0.50484E+00_JPRB, 0.49878E+00_JPRB, 0.49390E+00_JPRB ,&
&  0.48709E+00_JPRB, 0.48113E+00_JPRB, 0.47744E+00_JPRB, 0.47705E+00_JPRB, 0.48069E+00_JPRB, 0.49115E+00_JPRB, 0.50403E+00_JPRB ,&
&  0.50926E+00_JPRB, 0.50038E+00_JPRB, 0.49180E+00_JPRB, 0.48197E+00_JPRB, 0.46463E+00_JPRB, 0.43912E+00_JPRB, 0.42265E+00_JPRB ,&
&  0.41941E+00_JPRB, 0.43200E+00_JPRB, 0.45188E+00_JPRB, 0.45867E+00_JPRB, 0.46511E+00_JPRB, 0.46245E+00_JPRB, 0.45666E+00_JPRB ,&
&  0.44558E+00_JPRB, 0.42389E+00_JPRB, 0.41389E+00_JPRB, 0.41162E+00_JPRB, 0.41028E+00_JPRB, 0.41210E+00_JPRB, 0.41532E+00_JPRB ,&
&  0.42051E+00_JPRB, 0.42687E+00_JPRB, 0.43127E+00_JPRB, 0.43282E+00_JPRB, 0.43535E+00_JPRB, 0.43565E+00_JPRB, 0.43445E+00_JPRB ,&
&  0.43134E+00_JPRB, 0.43936E+00_JPRB, 0.45924E+00_JPRB, 0.48589E+00_JPRB, 0.49276E+00_JPRB, 0.50147E+00_JPRB, 0.51031E+00_JPRB ,&
&  0.51596E+00_JPRB, 0.52221E+00_JPRB, 0.52348E+00_JPRB, 0.52404E+00_JPRB, 0.52900E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.50985E+00_JPRB, 0.51704E+00_JPRB, 0.52610E+00_JPRB, 0.53763E+00_JPRB, 0.53988E+00_JPRB, 0.54131E+00_JPRB, 0.55001E+00_JPRB ,&
&  0.54956E+00_JPRB, 0.54602E+00_JPRB, 0.54295E+00_JPRB, 0.54385E+00_JPRB, 0.54690E+00_JPRB, 0.55019E+00_JPRB, 0.55086E+00_JPRB ,&
&  0.54243E+00_JPRB, 0.53783E+00_JPRB, 0.54136E+00_JPRB, 0.54521E+00_JPRB, 0.55036E+00_JPRB, 0.55693E+00_JPRB, 0.56293E+00_JPRB ,&
&  0.56657E+00_JPRB, 0.57087E+00_JPRB, 0.57432E+00_JPRB, 0.58122E+00_JPRB, 0.59117E+00_JPRB, 0.59822E+00_JPRB, 0.60357E+00_JPRB ,&
&  0.58542E+00_JPRB, 0.53685E+00_JPRB, 0.50729E+00_JPRB, 0.51270E+00_JPRB, 0.55717E+00_JPRB, 0.56834E+00_JPRB, 0.56905E+00_JPRB ,&
&  0.57165E+00_JPRB, 0.57888E+00_JPRB, 0.57860E+00_JPRB, 0.57320E+00_JPRB, 0.56099E+00_JPRB, 0.54121E+00_JPRB, 0.51738E+00_JPRB ,&
&  0.50182E+00_JPRB, 0.49207E+00_JPRB, 0.48514E+00_JPRB, 0.47761E+00_JPRB, 0.47378E+00_JPRB, 0.46883E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.45748E+00_JPRB, 0.45678E+00_JPRB, 0.45822E+00_JPRB, 0.45896E+00_JPRB, 0.46005E+00_JPRB, 0.46004E+00_JPRB, 0.45330E+00_JPRB ,&
&  0.44150E+00_JPRB, 0.42471E+00_JPRB, 0.40901E+00_JPRB, 0.39186E+00_JPRB, 0.37773E+00_JPRB, 0.36965E+00_JPRB, 0.36273E+00_JPRB ,&
&  0.36051E+00_JPRB, 0.36838E+00_JPRB, 0.38900E+00_JPRB, 0.41122E+00_JPRB, 0.43768E+00_JPRB, 0.41081E+00_JPRB, 0.38767E+00_JPRB ,&
&  0.37776E+00_JPRB, 0.37277E+00_JPRB, 0.37003E+00_JPRB, 0.37065E+00_JPRB, 0.37282E+00_JPRB, 0.37341E+00_JPRB, 0.37368E+00_JPRB ,&
&  0.37569E+00_JPRB, 0.37894E+00_JPRB, 0.38097E+00_JPRB, 0.38280E+00_JPRB, 0.38557E+00_JPRB, 0.38866E+00_JPRB, 0.39020E+00_JPRB ,&
&  0.39126E+00_JPRB, 0.39716E+00_JPRB, 0.40804E+00_JPRB, 0.42079E+00_JPRB, 0.43292E+00_JPRB, 0.44548E+00_JPRB, 0.46253E+00_JPRB ,&
&  0.48257E+00_JPRB, 0.50444E+00_JPRB, 0.50546E+00_JPRB, 0.50766E+00_JPRB, 0.50769E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.49263E+00_JPRB, 0.50461E+00_JPRB, 0.51031E+00_JPRB, 0.51610E+00_JPRB, 0.51874E+00_JPRB, 0.52031E+00_JPRB, 0.52842E+00_JPRB ,&
&  0.53337E+00_JPRB, 0.53751E+00_JPRB, 0.54034E+00_JPRB, 0.54274E+00_JPRB, 0.53993E+00_JPRB, 0.53411E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53243E+00_JPRB, 0.53303E+00_JPRB, 0.53238E+00_JPRB, 0.53249E+00_JPRB, 0.53309E+00_JPRB, 0.53784E+00_JPRB, 0.54521E+00_JPRB ,&
&  0.54699E+00_JPRB, 0.55103E+00_JPRB, 0.56487E+00_JPRB, 0.57262E+00_JPRB, 0.57754E+00_JPRB, 0.57623E+00_JPRB, 0.56942E+00_JPRB ,&
&  0.53856E+00_JPRB, 0.50948E+00_JPRB, 0.48414E+00_JPRB, 0.48513E+00_JPRB, 0.53461E+00_JPRB, 0.53953E+00_JPRB, 0.53992E+00_JPRB ,&
&  0.53851E+00_JPRB, 0.54229E+00_JPRB, 0.54252E+00_JPRB, 0.52757E+00_JPRB, 0.51679E+00_JPRB, 0.50274E+00_JPRB, 0.48991E+00_JPRB ,&
&  0.47772E+00_JPRB, 0.47047E+00_JPRB, 0.45626E+00_JPRB, 0.42733E+00_JPRB, 0.41131E+00_JPRB, 0.40310E+00_JPRB, 0.38919E+00_JPRB ,&
&  0.37832E+00_JPRB, 0.37612E+00_JPRB, 0.37551E+00_JPRB, 0.37337E+00_JPRB, 0.37705E+00_JPRB, 0.37927E+00_JPRB, 0.37676E+00_JPRB ,&
&  0.37156E+00_JPRB, 0.36736E+00_JPRB, 0.36281E+00_JPRB, 0.35707E+00_JPRB, 0.35424E+00_JPRB, 0.35381E+00_JPRB, 0.35504E+00_JPRB ,&
&  0.36049E+00_JPRB, 0.36779E+00_JPRB, 0.38328E+00_JPRB, 0.39375E+00_JPRB, 0.38670E+00_JPRB, 0.37377E+00_JPRB, 0.36383E+00_JPRB ,&
&  0.35643E+00_JPRB, 0.35360E+00_JPRB, 0.35136E+00_JPRB, 0.34887E+00_JPRB, 0.34965E+00_JPRB, 0.35222E+00_JPRB, 0.35364E+00_JPRB ,&
&  0.35369E+00_JPRB, 0.35611E+00_JPRB, 0.35771E+00_JPRB, 0.35936E+00_JPRB, 0.36075E+00_JPRB, 0.36298E+00_JPRB, 0.36603E+00_JPRB ,&
&  0.36855E+00_JPRB, 0.37437E+00_JPRB, 0.38052E+00_JPRB, 0.38769E+00_JPRB, 0.39231E+00_JPRB, 0.39584E+00_JPRB, 0.40384E+00_JPRB ,&
&  0.41000E+00_JPRB, 0.42572E+00_JPRB, 0.45863E+00_JPRB, 0.49180E+00_JPRB, 0.48130E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.47991E+00_JPRB, 0.48902E+00_JPRB, 0.49685E+00_JPRB, 0.50304E+00_JPRB, 0.50737E+00_JPRB, 0.50859E+00_JPRB, 0.51176E+00_JPRB ,&
&  0.51985E+00_JPRB, 0.52480E+00_JPRB, 0.52914E+00_JPRB, 0.52791E+00_JPRB, 0.52735E+00_JPRB, 0.50999E+00_JPRB, 0.50706E+00_JPRB ,&
&  0.50056E+00_JPRB, 0.49596E+00_JPRB, 0.48908E+00_JPRB, 0.48868E+00_JPRB, 0.48829E+00_JPRB, 0.49077E+00_JPRB, 0.49180E+00_JPRB ,&
&  0.49319E+00_JPRB, 0.49289E+00_JPRB, 0.49033E+00_JPRB, 0.48885E+00_JPRB, 0.48983E+00_JPRB, 0.49056E+00_JPRB, 0.48709E+00_JPRB ,&
&  0.48227E+00_JPRB, 0.48075E+00_JPRB, 0.48352E+00_JPRB, 0.48923E+00_JPRB, 0.52733E+00_JPRB, 0.52066E+00_JPRB, 0.51936E+00_JPRB ,&
&  0.51598E+00_JPRB, 0.51149E+00_JPRB, 0.51049E+00_JPRB, 0.50326E+00_JPRB, 0.49315E+00_JPRB, 0.46846E+00_JPRB, 0.45495E+00_JPRB ,&
&  0.42144E+00_JPRB, 0.40374E+00_JPRB, 0.38839E+00_JPRB, 0.37850E+00_JPRB, 0.36693E+00_JPRB, 0.36015E+00_JPRB, 0.35443E+00_JPRB ,&
&  0.35001E+00_JPRB, 0.34627E+00_JPRB, 0.34294E+00_JPRB, 0.34385E+00_JPRB, 0.34594E+00_JPRB, 0.34675E+00_JPRB, 0.34770E+00_JPRB ,&
&  0.34824E+00_JPRB, 0.34879E+00_JPRB, 0.34939E+00_JPRB, 0.35107E+00_JPRB, 0.35335E+00_JPRB, 0.35639E+00_JPRB, 0.36031E+00_JPRB ,&
&  0.36651E+00_JPRB, 0.37111E+00_JPRB, 0.37243E+00_JPRB, 0.37570E+00_JPRB, 0.37533E+00_JPRB, 0.36812E+00_JPRB, 0.36051E+00_JPRB ,&
&  0.35232E+00_JPRB, 0.34807E+00_JPRB, 0.34377E+00_JPRB, 0.34065E+00_JPRB, 0.33984E+00_JPRB, 0.33965E+00_JPRB, 0.34187E+00_JPRB ,&
&  0.34435E+00_JPRB, 0.34671E+00_JPRB, 0.34840E+00_JPRB, 0.35016E+00_JPRB, 0.35241E+00_JPRB, 0.35394E+00_JPRB, 0.35596E+00_JPRB ,&
&  0.35862E+00_JPRB, 0.36218E+00_JPRB, 0.36746E+00_JPRB, 0.37369E+00_JPRB, 0.38242E+00_JPRB, 0.39448E+00_JPRB, 0.41448E+00_JPRB ,&
&  0.43893E+00_JPRB, 0.49139E+00_JPRB, 0.49322E+00_JPRB, 0.46649E+00_JPRB, 0.47343E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.45580E+00_JPRB, 0.46354E+00_JPRB, 0.47062E+00_JPRB, 0.47836E+00_JPRB, 0.48804E+00_JPRB, 0.49247E+00_JPRB, 0.49445E+00_JPRB ,&
&  0.48995E+00_JPRB, 0.48623E+00_JPRB, 0.47603E+00_JPRB, 0.46795E+00_JPRB, 0.45896E+00_JPRB, 0.45204E+00_JPRB, 0.44583E+00_JPRB ,&
&  0.44132E+00_JPRB, 0.43793E+00_JPRB, 0.43206E+00_JPRB, 0.42833E+00_JPRB, 0.42464E+00_JPRB, 0.42517E+00_JPRB, 0.42621E+00_JPRB ,&
&  0.42649E+00_JPRB, 0.42757E+00_JPRB, 0.42590E+00_JPRB, 0.43109E+00_JPRB, 0.44061E+00_JPRB, 0.45172E+00_JPRB, 0.46625E+00_JPRB ,&
&  0.48646E+00_JPRB, 0.49693E+00_JPRB, 0.51236E+00_JPRB, 0.51184E+00_JPRB, 0.51010E+00_JPRB, 0.50648E+00_JPRB, 0.50332E+00_JPRB ,&
&  0.49842E+00_JPRB, 0.49129E+00_JPRB, 0.48570E+00_JPRB, 0.47828E+00_JPRB, 0.44696E+00_JPRB, 0.42355E+00_JPRB, 0.40533E+00_JPRB ,&
&  0.39062E+00_JPRB, 0.37668E+00_JPRB, 0.36383E+00_JPRB, 0.35755E+00_JPRB, 0.35171E+00_JPRB, 0.34824E+00_JPRB, 0.34455E+00_JPRB ,&
&  0.34054E+00_JPRB, 0.33963E+00_JPRB, 0.33775E+00_JPRB, 0.33696E+00_JPRB, 0.33829E+00_JPRB, 0.33798E+00_JPRB, 0.33877E+00_JPRB ,&
&  0.33961E+00_JPRB, 0.34126E+00_JPRB, 0.34492E+00_JPRB, 0.34762E+00_JPRB, 0.35031E+00_JPRB, 0.35274E+00_JPRB, 0.35372E+00_JPRB ,&
&  0.35596E+00_JPRB, 0.35798E+00_JPRB, 0.35786E+00_JPRB, 0.35692E+00_JPRB, 0.35472E+00_JPRB, 0.34944E+00_JPRB, 0.34434E+00_JPRB ,&
&  0.33877E+00_JPRB, 0.33477E+00_JPRB, 0.33354E+00_JPRB, 0.33231E+00_JPRB, 0.33339E+00_JPRB, 0.33413E+00_JPRB, 0.33479E+00_JPRB ,&
&  0.33682E+00_JPRB, 0.33882E+00_JPRB, 0.34134E+00_JPRB, 0.34518E+00_JPRB, 0.34787E+00_JPRB, 0.35006E+00_JPRB, 0.35270E+00_JPRB ,&
&  0.35204E+00_JPRB, 0.35676E+00_JPRB, 0.36277E+00_JPRB, 0.37172E+00_JPRB, 0.38896E+00_JPRB, 0.40001E+00_JPRB, 0.42417E+00_JPRB ,&
&  0.45558E+00_JPRB, 0.51350E+00_JPRB, 0.50447E+00_JPRB, 0.44178E+00_JPRB, 0.44844E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.44152E+00_JPRB, 0.44576E+00_JPRB, 0.44915E+00_JPRB, 0.45351E+00_JPRB, 0.45873E+00_JPRB, 0.46373E+00_JPRB, 0.46713E+00_JPRB ,&
&  0.46967E+00_JPRB, 0.46842E+00_JPRB, 0.47019E+00_JPRB, 0.46466E+00_JPRB, 0.45920E+00_JPRB, 0.45191E+00_JPRB, 0.44532E+00_JPRB ,&
&  0.43906E+00_JPRB, 0.42982E+00_JPRB, 0.42006E+00_JPRB, 0.41158E+00_JPRB, 0.40427E+00_JPRB, 0.39901E+00_JPRB, 0.39518E+00_JPRB ,&
&  0.39251E+00_JPRB, 0.39377E+00_JPRB, 0.39586E+00_JPRB, 0.40203E+00_JPRB, 0.41233E+00_JPRB, 0.42443E+00_JPRB, 0.44338E+00_JPRB ,&
&  0.46939E+00_JPRB, 0.47146E+00_JPRB, 0.47859E+00_JPRB, 0.47786E+00_JPRB, 0.47375E+00_JPRB, 0.47248E+00_JPRB, 0.47017E+00_JPRB ,&
&  0.47101E+00_JPRB, 0.46043E+00_JPRB, 0.46985E+00_JPRB, 0.45023E+00_JPRB, 0.42025E+00_JPRB, 0.40484E+00_JPRB, 0.38601E+00_JPRB ,&
&  0.37357E+00_JPRB, 0.36287E+00_JPRB, 0.35501E+00_JPRB, 0.35250E+00_JPRB, 0.34796E+00_JPRB, 0.34595E+00_JPRB, 0.34131E+00_JPRB ,&
&  0.33679E+00_JPRB, 0.33509E+00_JPRB, 0.33170E+00_JPRB, 0.33193E+00_JPRB, 0.33231E+00_JPRB, 0.33162E+00_JPRB, 0.33177E+00_JPRB ,&
&  0.33230E+00_JPRB, 0.33387E+00_JPRB, 0.33672E+00_JPRB, 0.33925E+00_JPRB, 0.34078E+00_JPRB, 0.34164E+00_JPRB, 0.34139E+00_JPRB ,&
&  0.34202E+00_JPRB, 0.34163E+00_JPRB, 0.34058E+00_JPRB, 0.33901E+00_JPRB, 0.33646E+00_JPRB, 0.33375E+00_JPRB, 0.33108E+00_JPRB ,&
&  0.32881E+00_JPRB, 0.32775E+00_JPRB, 0.32672E+00_JPRB, 0.32666E+00_JPRB, 0.32703E+00_JPRB, 0.32894E+00_JPRB, 0.33070E+00_JPRB ,&
&  0.33279E+00_JPRB, 0.33595E+00_JPRB, 0.33863E+00_JPRB, 0.34197E+00_JPRB, 0.34485E+00_JPRB, 0.34607E+00_JPRB, 0.35059E+00_JPRB ,&
&  0.35653E+00_JPRB, 0.36509E+00_JPRB, 0.37525E+00_JPRB, 0.38448E+00_JPRB, 0.39606E+00_JPRB, 0.41133E+00_JPRB, 0.43122E+00_JPRB ,&
&  0.45035E+00_JPRB, 0.46752E+00_JPRB, 0.45873E+00_JPRB, 0.43521E+00_JPRB, 0.43674E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.43561E+00_JPRB, 0.43522E+00_JPRB, 0.43686E+00_JPRB, 0.43909E+00_JPRB, 0.44474E+00_JPRB, 0.45155E+00_JPRB, 0.45883E+00_JPRB ,&
&  0.45891E+00_JPRB, 0.46272E+00_JPRB, 0.46127E+00_JPRB, 0.45743E+00_JPRB, 0.45143E+00_JPRB, 0.44460E+00_JPRB, 0.43984E+00_JPRB ,&
&  0.43325E+00_JPRB, 0.42472E+00_JPRB, 0.41650E+00_JPRB, 0.40786E+00_JPRB, 0.39988E+00_JPRB, 0.39161E+00_JPRB, 0.38779E+00_JPRB ,&
&  0.38427E+00_JPRB, 0.38349E+00_JPRB, 0.38556E+00_JPRB, 0.38779E+00_JPRB, 0.39304E+00_JPRB, 0.40029E+00_JPRB, 0.41001E+00_JPRB ,&
&  0.41906E+00_JPRB, 0.43030E+00_JPRB, 0.43637E+00_JPRB, 0.43618E+00_JPRB, 0.43618E+00_JPRB, 0.43428E+00_JPRB, 0.43094E+00_JPRB ,&
&  0.42383E+00_JPRB, 0.41616E+00_JPRB, 0.40694E+00_JPRB, 0.39718E+00_JPRB, 0.38716E+00_JPRB, 0.37847E+00_JPRB, 0.37037E+00_JPRB ,&
&  0.36383E+00_JPRB, 0.35828E+00_JPRB, 0.35297E+00_JPRB, 0.34847E+00_JPRB, 0.34501E+00_JPRB, 0.34253E+00_JPRB, 0.33907E+00_JPRB ,&
&  0.33635E+00_JPRB, 0.33390E+00_JPRB, 0.33090E+00_JPRB, 0.32895E+00_JPRB, 0.32716E+00_JPRB, 0.32601E+00_JPRB, 0.32505E+00_JPRB ,&
&  0.32524E+00_JPRB, 0.32546E+00_JPRB, 0.32665E+00_JPRB, 0.32785E+00_JPRB, 0.32914E+00_JPRB, 0.32935E+00_JPRB, 0.32929E+00_JPRB ,&
&  0.32892E+00_JPRB, 0.32850E+00_JPRB, 0.32660E+00_JPRB, 0.32508E+00_JPRB, 0.32396E+00_JPRB, 0.32273E+00_JPRB, 0.32247E+00_JPRB ,&
&  0.32321E+00_JPRB, 0.32398E+00_JPRB, 0.32447E+00_JPRB, 0.32624E+00_JPRB, 0.32796E+00_JPRB, 0.33060E+00_JPRB, 0.33387E+00_JPRB ,&
&  0.33617E+00_JPRB, 0.33890E+00_JPRB, 0.34192E+00_JPRB, 0.34542E+00_JPRB, 0.34973E+00_JPRB, 0.35409E+00_JPRB, 0.35935E+00_JPRB ,&
&  0.36534E+00_JPRB, 0.37130E+00_JPRB, 0.37802E+00_JPRB, 0.38550E+00_JPRB, 0.39454E+00_JPRB, 0.39929E+00_JPRB, 0.40752E+00_JPRB ,&
&  0.41294E+00_JPRB, 0.40536E+00_JPRB, 0.44548E+00_JPRB, 0.43891E+00_JPRB, 0.43531E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.38087E+00_JPRB, 0.38722E+00_JPRB, 0.39093E+00_JPRB, 0.39415E+00_JPRB, 0.38213E+00_JPRB, 0.41937E+00_JPRB, 0.41715E+00_JPRB ,&
&  0.41864E+00_JPRB, 0.40948E+00_JPRB, 0.41281E+00_JPRB, 0.41314E+00_JPRB, 0.41140E+00_JPRB, 0.40848E+00_JPRB, 0.40563E+00_JPRB ,&
&  0.40303E+00_JPRB, 0.39883E+00_JPRB, 0.39360E+00_JPRB, 0.38897E+00_JPRB, 0.38544E+00_JPRB, 0.38245E+00_JPRB, 0.37982E+00_JPRB ,&
&  0.37918E+00_JPRB, 0.38057E+00_JPRB, 0.38314E+00_JPRB, 0.38047E+00_JPRB, 0.38050E+00_JPRB, 0.38163E+00_JPRB, 0.38340E+00_JPRB ,&
&  0.38491E+00_JPRB, 0.38649E+00_JPRB, 0.38640E+00_JPRB, 0.38443E+00_JPRB, 0.38418E+00_JPRB, 0.38237E+00_JPRB, 0.38054E+00_JPRB ,&
&  0.37803E+00_JPRB, 0.37529E+00_JPRB, 0.37196E+00_JPRB, 0.36931E+00_JPRB, 0.36631E+00_JPRB, 0.36262E+00_JPRB, 0.35911E+00_JPRB ,&
&  0.35610E+00_JPRB, 0.35334E+00_JPRB, 0.35003E+00_JPRB, 0.34696E+00_JPRB, 0.34400E+00_JPRB, 0.34100E+00_JPRB, 0.33819E+00_JPRB ,&
&  0.33603E+00_JPRB, 0.33421E+00_JPRB, 0.33244E+00_JPRB, 0.33055E+00_JPRB, 0.32878E+00_JPRB, 0.32706E+00_JPRB, 0.32575E+00_JPRB ,&
&  0.32581E+00_JPRB, 0.32513E+00_JPRB, 0.32449E+00_JPRB, 0.32431E+00_JPRB, 0.32465E+00_JPRB, 0.32418E+00_JPRB, 0.32404E+00_JPRB ,&
&  0.32404E+00_JPRB, 0.32411E+00_JPRB, 0.32436E+00_JPRB, 0.32468E+00_JPRB, 0.32513E+00_JPRB, 0.32574E+00_JPRB, 0.32673E+00_JPRB ,&
&  0.32792E+00_JPRB, 0.32937E+00_JPRB, 0.33095E+00_JPRB, 0.33254E+00_JPRB, 0.33417E+00_JPRB, 0.33589E+00_JPRB, 0.33786E+00_JPRB ,&
&  0.33980E+00_JPRB, 0.34167E+00_JPRB, 0.34346E+00_JPRB, 0.34522E+00_JPRB, 0.34837E+00_JPRB, 0.35136E+00_JPRB, 0.35454E+00_JPRB ,&
&  0.35799E+00_JPRB, 0.36207E+00_JPRB, 0.36637E+00_JPRB, 0.37112E+00_JPRB, 0.37662E+00_JPRB, 0.37713E+00_JPRB, 0.37740E+00_JPRB ,&
&  0.37714E+00_JPRB, 0.38029E+00_JPRB, 0.38083E+00_JPRB, 0.37918E+00_JPRB, 0.37787E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.35571E+00_JPRB, 0.35555E+00_JPRB, 0.35529E+00_JPRB, 0.35492E+00_JPRB, 0.35250E+00_JPRB, 0.35254E+00_JPRB, 0.35274E+00_JPRB ,&
&  0.35345E+00_JPRB, 0.35549E+00_JPRB, 0.35776E+00_JPRB, 0.35945E+00_JPRB, 0.36059E+00_JPRB, 0.36135E+00_JPRB, 0.36198E+00_JPRB ,&
&  0.36215E+00_JPRB, 0.36261E+00_JPRB, 0.35817E+00_JPRB, 0.35653E+00_JPRB, 0.35587E+00_JPRB, 0.35572E+00_JPRB, 0.35588E+00_JPRB ,&
&  0.35591E+00_JPRB, 0.35578E+00_JPRB, 0.35499E+00_JPRB, 0.35536E+00_JPRB, 0.35578E+00_JPRB, 0.35603E+00_JPRB, 0.35629E+00_JPRB ,&
&  0.35656E+00_JPRB, 0.35683E+00_JPRB, 0.35678E+00_JPRB, 0.35651E+00_JPRB, 0.35647E+00_JPRB, 0.35608E+00_JPRB, 0.35526E+00_JPRB ,&
&  0.35423E+00_JPRB, 0.35289E+00_JPRB, 0.35157E+00_JPRB, 0.35035E+00_JPRB, 0.34930E+00_JPRB, 0.34841E+00_JPRB, 0.34754E+00_JPRB ,&
&  0.34644E+00_JPRB, 0.34493E+00_JPRB, 0.34332E+00_JPRB, 0.34174E+00_JPRB, 0.34021E+00_JPRB, 0.33882E+00_JPRB, 0.33732E+00_JPRB ,&
&  0.33620E+00_JPRB, 0.33527E+00_JPRB, 0.33436E+00_JPRB, 0.33358E+00_JPRB, 0.33279E+00_JPRB, 0.33202E+00_JPRB, 0.33129E+00_JPRB ,&
&  0.33025E+00_JPRB, 0.32975E+00_JPRB, 0.32917E+00_JPRB, 0.32897E+00_JPRB, 0.32916E+00_JPRB, 0.32932E+00_JPRB, 0.32959E+00_JPRB ,&
&  0.32994E+00_JPRB, 0.33032E+00_JPRB, 0.33051E+00_JPRB, 0.33078E+00_JPRB, 0.33117E+00_JPRB, 0.33168E+00_JPRB, 0.33227E+00_JPRB ,&
&  0.33290E+00_JPRB, 0.33359E+00_JPRB, 0.33435E+00_JPRB, 0.33520E+00_JPRB, 0.33607E+00_JPRB, 0.33704E+00_JPRB, 0.33795E+00_JPRB ,&
&  0.33888E+00_JPRB, 0.33974E+00_JPRB, 0.34062E+00_JPRB, 0.34130E+00_JPRB, 0.34234E+00_JPRB, 0.34333E+00_JPRB, 0.34452E+00_JPRB ,&
&  0.34571E+00_JPRB, 0.34699E+00_JPRB, 0.34812E+00_JPRB, 0.34936E+00_JPRB, 0.35076E+00_JPRB, 0.35035E+00_JPRB, 0.35010E+00_JPRB ,&
&  0.34991E+00_JPRB, 0.35014E+00_JPRB, 0.34985E+00_JPRB, 0.35011E+00_JPRB, 0.35106E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB, 0.33736E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB, 0.72804E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.74005E+00_JPRB, 0.74037E+00_JPRB, 0.74060E+00_JPRB, 0.74061E+00_JPRB, 0.74054E+00_JPRB, 0.74066E+00_JPRB, 0.74071E+00_JPRB ,&
&  0.74072E+00_JPRB, 0.74069E+00_JPRB, 0.74099E+00_JPRB, 0.74086E+00_JPRB, 0.74071E+00_JPRB, 0.74031E+00_JPRB, 0.73992E+00_JPRB ,&
&  0.73944E+00_JPRB, 0.73899E+00_JPRB, 0.73847E+00_JPRB, 0.73809E+00_JPRB, 0.73781E+00_JPRB, 0.73774E+00_JPRB, 0.73764E+00_JPRB ,&
&  0.73731E+00_JPRB, 0.73664E+00_JPRB, 0.73612E+00_JPRB, 0.73527E+00_JPRB, 0.73422E+00_JPRB, 0.73266E+00_JPRB, 0.73080E+00_JPRB ,&
&  0.72852E+00_JPRB, 0.72638E+00_JPRB, 0.72465E+00_JPRB, 0.72310E+00_JPRB, 0.72188E+00_JPRB, 0.72094E+00_JPRB, 0.72024E+00_JPRB ,&
&  0.71937E+00_JPRB, 0.71881E+00_JPRB, 0.71829E+00_JPRB, 0.71787E+00_JPRB, 0.71704E+00_JPRB, 0.71633E+00_JPRB, 0.71554E+00_JPRB ,&
&  0.71473E+00_JPRB, 0.71368E+00_JPRB, 0.71295E+00_JPRB, 0.71217E+00_JPRB, 0.71127E+00_JPRB, 0.70966E+00_JPRB, 0.70822E+00_JPRB ,&
&  0.70688E+00_JPRB, 0.70550E+00_JPRB, 0.70396E+00_JPRB, 0.70256E+00_JPRB, 0.70155E+00_JPRB, 0.70059E+00_JPRB, 0.69950E+00_JPRB ,&
&  0.69925E+00_JPRB, 0.69982E+00_JPRB, 0.70036E+00_JPRB, 0.70066E+00_JPRB, 0.70144E+00_JPRB, 0.70237E+00_JPRB, 0.70339E+00_JPRB ,&
&  0.70434E+00_JPRB, 0.70561E+00_JPRB, 0.70742E+00_JPRB, 0.70909E+00_JPRB, 0.71090E+00_JPRB, 0.71276E+00_JPRB, 0.71487E+00_JPRB ,&
&  0.71680E+00_JPRB, 0.71871E+00_JPRB, 0.72041E+00_JPRB, 0.72233E+00_JPRB, 0.72364E+00_JPRB, 0.72491E+00_JPRB, 0.72582E+00_JPRB ,&
&  0.72679E+00_JPRB, 0.72753E+00_JPRB, 0.72835E+00_JPRB, 0.72906E+00_JPRB, 0.73003E+00_JPRB, 0.73084E+00_JPRB, 0.73169E+00_JPRB ,&
&  0.73251E+00_JPRB, 0.73347E+00_JPRB, 0.73426E+00_JPRB, 0.73500E+00_JPRB, 0.73560E+00_JPRB, 0.73644E+00_JPRB, 0.73711E+00_JPRB ,&
&  0.73771E+00_JPRB, 0.73833E+00_JPRB, 0.73895E+00_JPRB, 0.73952E+00_JPRB, 0.73987E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.73670E+00_JPRB, 0.73909E+00_JPRB, 0.74044E+00_JPRB, 0.74169E+00_JPRB, 0.74273E+00_JPRB, 0.74441E+00_JPRB, 0.74560E+00_JPRB ,&
&  0.74674E+00_JPRB, 0.74829E+00_JPRB, 0.75063E+00_JPRB, 0.75179E+00_JPRB, 0.75257E+00_JPRB, 0.75272E+00_JPRB, 0.75274E+00_JPRB ,&
&  0.75220E+00_JPRB, 0.75124E+00_JPRB, 0.75018E+00_JPRB, 0.74924E+00_JPRB, 0.74841E+00_JPRB, 0.74777E+00_JPRB, 0.74671E+00_JPRB ,&
&  0.74474E+00_JPRB, 0.74227E+00_JPRB, 0.73951E+00_JPRB, 0.73513E+00_JPRB, 0.73025E+00_JPRB, 0.72515E+00_JPRB, 0.71979E+00_JPRB ,&
&  0.71389E+00_JPRB, 0.70934E+00_JPRB, 0.70512E+00_JPRB, 0.70117E+00_JPRB, 0.69983E+00_JPRB, 0.69833E+00_JPRB, 0.69675E+00_JPRB ,&
&  0.69515E+00_JPRB, 0.69245E+00_JPRB, 0.69151E+00_JPRB, 0.69090E+00_JPRB, 0.69077E+00_JPRB, 0.69179E+00_JPRB, 0.69251E+00_JPRB ,&
&  0.69296E+00_JPRB, 0.69273E+00_JPRB, 0.69240E+00_JPRB, 0.69270E+00_JPRB, 0.69341E+00_JPRB, 0.69335E+00_JPRB, 0.69206E+00_JPRB ,&
&  0.69106E+00_JPRB, 0.68946E+00_JPRB, 0.68760E+00_JPRB, 0.68542E+00_JPRB, 0.68433E+00_JPRB, 0.68287E+00_JPRB, 0.68121E+00_JPRB ,&
&  0.68021E+00_JPRB, 0.68010E+00_JPRB, 0.68026E+00_JPRB, 0.68058E+00_JPRB, 0.68170E+00_JPRB, 0.68315E+00_JPRB, 0.68459E+00_JPRB ,&
&  0.68612E+00_JPRB, 0.68872E+00_JPRB, 0.69170E+00_JPRB, 0.69445E+00_JPRB, 0.69736E+00_JPRB, 0.70032E+00_JPRB, 0.70363E+00_JPRB ,&
&  0.70626E+00_JPRB, 0.70828E+00_JPRB, 0.70925E+00_JPRB, 0.71117E+00_JPRB, 0.71288E+00_JPRB, 0.71516E+00_JPRB, 0.71693E+00_JPRB ,&
&  0.71938E+00_JPRB, 0.72101E+00_JPRB, 0.72210E+00_JPRB, 0.72318E+00_JPRB, 0.72435E+00_JPRB, 0.72523E+00_JPRB, 0.72574E+00_JPRB ,&
&  0.72604E+00_JPRB, 0.72646E+00_JPRB, 0.72674E+00_JPRB, 0.72686E+00_JPRB, 0.72715E+00_JPRB, 0.72760E+00_JPRB, 0.72813E+00_JPRB ,&
&  0.72893E+00_JPRB, 0.73040E+00_JPRB, 0.73231E+00_JPRB, 0.73366E+00_JPRB, 0.73502E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.72680E+00_JPRB, 0.72875E+00_JPRB, 0.73026E+00_JPRB, 0.73272E+00_JPRB, 0.73702E+00_JPRB, 0.74147E+00_JPRB, 0.74402E+00_JPRB ,&
&  0.74749E+00_JPRB, 0.75158E+00_JPRB, 0.75622E+00_JPRB, 0.75899E+00_JPRB, 0.76108E+00_JPRB, 0.76208E+00_JPRB, 0.76192E+00_JPRB ,&
&  0.76058E+00_JPRB, 0.75855E+00_JPRB, 0.75729E+00_JPRB, 0.75569E+00_JPRB, 0.75377E+00_JPRB, 0.75157E+00_JPRB, 0.74572E+00_JPRB ,&
&  0.74066E+00_JPRB, 0.73708E+00_JPRB, 0.73005E+00_JPRB, 0.72204E+00_JPRB, 0.71569E+00_JPRB, 0.67123E+00_JPRB, 0.65655E+00_JPRB ,&
&  0.65572E+00_JPRB, 0.65182E+00_JPRB, 0.65285E+00_JPRB, 0.65455E+00_JPRB, 0.65946E+00_JPRB, 0.65190E+00_JPRB, 0.64685E+00_JPRB ,&
&  0.61954E+00_JPRB, 0.62058E+00_JPRB, 0.61624E+00_JPRB, 0.61392E+00_JPRB, 0.61665E+00_JPRB, 0.61838E+00_JPRB, 0.62718E+00_JPRB ,&
&  0.65135E+00_JPRB, 0.65788E+00_JPRB, 0.66114E+00_JPRB, 0.66407E+00_JPRB, 0.66772E+00_JPRB, 0.66999E+00_JPRB, 0.67185E+00_JPRB ,&
&  0.67254E+00_JPRB, 0.67228E+00_JPRB, 0.67234E+00_JPRB, 0.67149E+00_JPRB, 0.67073E+00_JPRB, 0.66927E+00_JPRB, 0.66831E+00_JPRB ,&
&  0.66769E+00_JPRB, 0.66859E+00_JPRB, 0.67031E+00_JPRB, 0.67206E+00_JPRB, 0.67459E+00_JPRB, 0.67633E+00_JPRB, 0.67798E+00_JPRB ,&
&  0.67968E+00_JPRB, 0.68133E+00_JPRB, 0.68330E+00_JPRB, 0.68537E+00_JPRB, 0.68793E+00_JPRB, 0.68960E+00_JPRB, 0.69198E+00_JPRB ,&
&  0.69388E+00_JPRB, 0.69632E+00_JPRB, 0.69840E+00_JPRB, 0.70278E+00_JPRB, 0.70616E+00_JPRB, 0.71095E+00_JPRB, 0.71515E+00_JPRB ,&
&  0.72007E+00_JPRB, 0.72332E+00_JPRB, 0.72625E+00_JPRB, 0.72831E+00_JPRB, 0.73068E+00_JPRB, 0.73246E+00_JPRB, 0.73327E+00_JPRB ,&
&  0.73349E+00_JPRB, 0.73257E+00_JPRB, 0.73143E+00_JPRB, 0.72961E+00_JPRB, 0.72801E+00_JPRB, 0.72590E+00_JPRB, 0.72413E+00_JPRB ,&
&  0.72188E+00_JPRB, 0.72140E+00_JPRB, 0.72208E+00_JPRB, 0.72368E+00_JPRB, 0.72515E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.73488E+00_JPRB, 0.73476E+00_JPRB, 0.73447E+00_JPRB, 0.73720E+00_JPRB, 0.74056E+00_JPRB, 0.74656E+00_JPRB, 0.74807E+00_JPRB ,&
&  0.75339E+00_JPRB, 0.75886E+00_JPRB, 0.76506E+00_JPRB, 0.76794E+00_JPRB, 0.77046E+00_JPRB, 0.77029E+00_JPRB, 0.76856E+00_JPRB ,&
&  0.76651E+00_JPRB, 0.76260E+00_JPRB, 0.76008E+00_JPRB, 0.75595E+00_JPRB, 0.75041E+00_JPRB, 0.74644E+00_JPRB, 0.73885E+00_JPRB ,&
&  0.73431E+00_JPRB, 0.72819E+00_JPRB, 0.68603E+00_JPRB, 0.67440E+00_JPRB, 0.65378E+00_JPRB, 0.64140E+00_JPRB, 0.63489E+00_JPRB ,&
&  0.62584E+00_JPRB, 0.61696E+00_JPRB, 0.60481E+00_JPRB, 0.59075E+00_JPRB, 0.57617E+00_JPRB, 0.55508E+00_JPRB, 0.54627E+00_JPRB ,&
&  0.53755E+00_JPRB, 0.53603E+00_JPRB, 0.53815E+00_JPRB, 0.54336E+00_JPRB, 0.55499E+00_JPRB, 0.56646E+00_JPRB, 0.58129E+00_JPRB ,&
&  0.60218E+00_JPRB, 0.62294E+00_JPRB, 0.63325E+00_JPRB, 0.63158E+00_JPRB, 0.64949E+00_JPRB, 0.65835E+00_JPRB, 0.66338E+00_JPRB ,&
&  0.66555E+00_JPRB, 0.66353E+00_JPRB, 0.66905E+00_JPRB, 0.66880E+00_JPRB, 0.66868E+00_JPRB, 0.66913E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.67312E+00_JPRB, 0.67615E+00_JPRB, 0.67857E+00_JPRB, 0.68073E+00_JPRB, 0.68210E+00_JPRB, 0.68254E+00_JPRB, 0.68289E+00_JPRB ,&
&  0.68350E+00_JPRB, 0.68431E+00_JPRB, 0.68545E+00_JPRB, 0.68671E+00_JPRB, 0.68883E+00_JPRB, 0.69019E+00_JPRB, 0.69120E+00_JPRB ,&
&  0.69333E+00_JPRB, 0.69654E+00_JPRB, 0.70015E+00_JPRB, 0.69982E+00_JPRB, 0.70404E+00_JPRB, 0.71440E+00_JPRB, 0.72015E+00_JPRB ,&
&  0.72916E+00_JPRB, 0.73587E+00_JPRB, 0.74360E+00_JPRB, 0.74963E+00_JPRB, 0.75493E+00_JPRB, 0.75745E+00_JPRB, 0.75757E+00_JPRB ,&
&  0.75696E+00_JPRB, 0.75507E+00_JPRB, 0.75372E+00_JPRB, 0.75046E+00_JPRB, 0.74758E+00_JPRB, 0.74332E+00_JPRB, 0.74016E+00_JPRB ,&
&  0.73606E+00_JPRB, 0.73429E+00_JPRB, 0.73074E+00_JPRB, 0.73194E+00_JPRB, 0.73294E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.74755E+00_JPRB, 0.74650E+00_JPRB, 0.74794E+00_JPRB, 0.75391E+00_JPRB, 0.76445E+00_JPRB, 0.77650E+00_JPRB, 0.77646E+00_JPRB ,&
&  0.77735E+00_JPRB, 0.77608E+00_JPRB, 0.77928E+00_JPRB, 0.77933E+00_JPRB, 0.77893E+00_JPRB, 0.77625E+00_JPRB, 0.77412E+00_JPRB ,&
&  0.77194E+00_JPRB, 0.76515E+00_JPRB, 0.75620E+00_JPRB, 0.75136E+00_JPRB, 0.74922E+00_JPRB, 0.74758E+00_JPRB, 0.74069E+00_JPRB ,&
&  0.73256E+00_JPRB, 0.71570E+00_JPRB, 0.68420E+00_JPRB, 0.66498E+00_JPRB, 0.64284E+00_JPRB, 0.63433E+00_JPRB, 0.61980E+00_JPRB ,&
&  0.62026E+00_JPRB, 0.57313E+00_JPRB, 0.57649E+00_JPRB, 0.55712E+00_JPRB, 0.54432E+00_JPRB, 0.52967E+00_JPRB, 0.51180E+00_JPRB ,&
&  0.50673E+00_JPRB, 0.49996E+00_JPRB, 0.49885E+00_JPRB, 0.50795E+00_JPRB, 0.51897E+00_JPRB, 0.53731E+00_JPRB, 0.55643E+00_JPRB ,&
&  0.56788E+00_JPRB, 0.61602E+00_JPRB, 0.63115E+00_JPRB, 0.63700E+00_JPRB, 0.65312E+00_JPRB, 0.66409E+00_JPRB, 0.66538E+00_JPRB ,&
&  0.66643E+00_JPRB, 0.66585E+00_JPRB, 0.67309E+00_JPRB, 0.67767E+00_JPRB, 0.68428E+00_JPRB, 0.68407E+00_JPRB, 0.68445E+00_JPRB ,&
&  0.68776E+00_JPRB, 0.69183E+00_JPRB, 0.69651E+00_JPRB, 0.70066E+00_JPRB, 0.70443E+00_JPRB, 0.70689E+00_JPRB, 0.70787E+00_JPRB ,&
&  0.70807E+00_JPRB, 0.70723E+00_JPRB, 0.70712E+00_JPRB, 0.70850E+00_JPRB, 0.71006E+00_JPRB, 0.71331E+00_JPRB, 0.71538E+00_JPRB ,&
&  0.71768E+00_JPRB, 0.71860E+00_JPRB, 0.71803E+00_JPRB, 0.71950E+00_JPRB, 0.72431E+00_JPRB, 0.72934E+00_JPRB, 0.73814E+00_JPRB ,&
&  0.75048E+00_JPRB, 0.75821E+00_JPRB, 0.76680E+00_JPRB, 0.77428E+00_JPRB, 0.78067E+00_JPRB, 0.78210E+00_JPRB, 0.78098E+00_JPRB ,&
&  0.77969E+00_JPRB, 0.77791E+00_JPRB, 0.77563E+00_JPRB, 0.77129E+00_JPRB, 0.76654E+00_JPRB, 0.75975E+00_JPRB, 0.75374E+00_JPRB ,&
&  0.74749E+00_JPRB, 0.74541E+00_JPRB, 0.74086E+00_JPRB, 0.74203E+00_JPRB, 0.74462E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.76699E+00_JPRB, 0.76850E+00_JPRB, 0.77399E+00_JPRB, 0.78190E+00_JPRB, 0.79027E+00_JPRB, 0.80022E+00_JPRB, 0.80454E+00_JPRB ,&
&  0.80792E+00_JPRB, 0.80688E+00_JPRB, 0.80383E+00_JPRB, 0.79180E+00_JPRB, 0.78392E+00_JPRB, 0.78000E+00_JPRB, 0.77543E+00_JPRB ,&
&  0.76903E+00_JPRB, 0.76431E+00_JPRB, 0.76047E+00_JPRB, 0.75807E+00_JPRB, 0.75345E+00_JPRB, 0.75102E+00_JPRB, 0.73989E+00_JPRB ,&
&  0.71912E+00_JPRB, 0.70003E+00_JPRB, 0.67245E+00_JPRB, 0.65048E+00_JPRB, 0.63536E+00_JPRB, 0.62016E+00_JPRB, 0.61879E+00_JPRB ,&
&  0.61463E+00_JPRB, 0.61112E+00_JPRB, 0.60612E+00_JPRB, 0.60221E+00_JPRB, 0.60252E+00_JPRB, 0.52458E+00_JPRB, 0.50683E+00_JPRB ,&
&  0.49525E+00_JPRB, 0.48522E+00_JPRB, 0.48285E+00_JPRB, 0.48294E+00_JPRB, 0.49029E+00_JPRB, 0.51423E+00_JPRB, 0.54535E+00_JPRB ,&
&  0.56342E+00_JPRB, 0.62848E+00_JPRB, 0.64139E+00_JPRB, 0.65278E+00_JPRB, 0.67345E+00_JPRB, 0.68641E+00_JPRB, 0.68932E+00_JPRB ,&
&  0.69726E+00_JPRB, 0.71207E+00_JPRB, 0.73061E+00_JPRB, 0.74864E+00_JPRB, 0.75061E+00_JPRB, 0.75631E+00_JPRB, 0.75639E+00_JPRB ,&
&  0.75239E+00_JPRB, 0.74434E+00_JPRB, 0.74198E+00_JPRB, 0.74131E+00_JPRB, 0.73973E+00_JPRB, 0.74012E+00_JPRB, 0.73707E+00_JPRB ,&
&  0.73954E+00_JPRB, 0.73821E+00_JPRB, 0.74458E+00_JPRB, 0.74734E+00_JPRB, 0.75343E+00_JPRB, 0.75969E+00_JPRB, 0.76252E+00_JPRB ,&
&  0.76225E+00_JPRB, 0.76301E+00_JPRB, 0.75843E+00_JPRB, 0.75018E+00_JPRB, 0.74772E+00_JPRB, 0.74490E+00_JPRB, 0.74848E+00_JPRB ,&
&  0.75512E+00_JPRB, 0.76720E+00_JPRB, 0.77453E+00_JPRB, 0.78260E+00_JPRB, 0.78953E+00_JPRB, 0.79054E+00_JPRB, 0.79320E+00_JPRB ,&
&  0.79445E+00_JPRB, 0.79629E+00_JPRB, 0.79467E+00_JPRB, 0.79057E+00_JPRB, 0.78319E+00_JPRB, 0.77279E+00_JPRB, 0.76162E+00_JPRB ,&
&  0.75545E+00_JPRB, 0.75508E+00_JPRB, 0.75350E+00_JPRB, 0.75613E+00_JPRB, 0.76146E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.78462E+00_JPRB, 0.78680E+00_JPRB, 0.79602E+00_JPRB, 0.80522E+00_JPRB, 0.81644E+00_JPRB, 0.82694E+00_JPRB, 0.83103E+00_JPRB ,&
&  0.84128E+00_JPRB, 0.83861E+00_JPRB, 0.82444E+00_JPRB, 0.80682E+00_JPRB, 0.78747E+00_JPRB, 0.77408E+00_JPRB, 0.76827E+00_JPRB ,&
&  0.76805E+00_JPRB, 0.75952E+00_JPRB, 0.75396E+00_JPRB, 0.74981E+00_JPRB, 0.74351E+00_JPRB, 0.73869E+00_JPRB, 0.72649E+00_JPRB ,&
&  0.70276E+00_JPRB, 0.68807E+00_JPRB, 0.67243E+00_JPRB, 0.64970E+00_JPRB, 0.63050E+00_JPRB, 0.61991E+00_JPRB, 0.62139E+00_JPRB ,&
&  0.61869E+00_JPRB, 0.60849E+00_JPRB, 0.60883E+00_JPRB, 0.60348E+00_JPRB, 0.58903E+00_JPRB, 0.58321E+00_JPRB, 0.52702E+00_JPRB ,&
&  0.50848E+00_JPRB, 0.48658E+00_JPRB, 0.47882E+00_JPRB, 0.47821E+00_JPRB, 0.49362E+00_JPRB, 0.51825E+00_JPRB, 0.56291E+00_JPRB ,&
&  0.60219E+00_JPRB, 0.63792E+00_JPRB, 0.66588E+00_JPRB, 0.68561E+00_JPRB, 0.70557E+00_JPRB, 0.72378E+00_JPRB, 0.73588E+00_JPRB ,&
&  0.75807E+00_JPRB, 0.78796E+00_JPRB, 0.81760E+00_JPRB, 0.84613E+00_JPRB, 0.86511E+00_JPRB, 0.87113E+00_JPRB, 0.87635E+00_JPRB ,&
&  0.87025E+00_JPRB, 0.80388E+00_JPRB, 0.77815E+00_JPRB, 0.76292E+00_JPRB, 0.75236E+00_JPRB, 0.75260E+00_JPRB, 0.74751E+00_JPRB ,&
&  0.75012E+00_JPRB, 0.75554E+00_JPRB, 0.75811E+00_JPRB, 0.76205E+00_JPRB, 0.76765E+00_JPRB, 0.76980E+00_JPRB, 0.77624E+00_JPRB ,&
&  0.78756E+00_JPRB, 0.80334E+00_JPRB, 0.83175E+00_JPRB, 0.81025E+00_JPRB, 0.78406E+00_JPRB, 0.75929E+00_JPRB, 0.74699E+00_JPRB ,&
&  0.74576E+00_JPRB, 0.75196E+00_JPRB, 0.76322E+00_JPRB, 0.76928E+00_JPRB, 0.77605E+00_JPRB, 0.78184E+00_JPRB, 0.79686E+00_JPRB ,&
&  0.80299E+00_JPRB, 0.80833E+00_JPRB, 0.80988E+00_JPRB, 0.80789E+00_JPRB, 0.79929E+00_JPRB, 0.78722E+00_JPRB, 0.77503E+00_JPRB ,&
&  0.77059E+00_JPRB, 0.77211E+00_JPRB, 0.77356E+00_JPRB, 0.77710E+00_JPRB, 0.78115E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.79051E+00_JPRB, 0.80124E+00_JPRB, 0.82144E+00_JPRB, 0.83995E+00_JPRB, 0.85346E+00_JPRB, 0.84365E+00_JPRB, 0.83071E+00_JPRB ,&
&  0.82811E+00_JPRB, 0.82675E+00_JPRB, 0.81418E+00_JPRB, 0.80033E+00_JPRB, 0.77952E+00_JPRB, 0.76330E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.75733E+00_JPRB, 0.74986E+00_JPRB, 0.73846E+00_JPRB, 0.72352E+00_JPRB, 0.71295E+00_JPRB, 0.70979E+00_JPRB, 0.70185E+00_JPRB ,&
&  0.68835E+00_JPRB, 0.68100E+00_JPRB, 0.67263E+00_JPRB, 0.65642E+00_JPRB, 0.64089E+00_JPRB, 0.63139E+00_JPRB, 0.62727E+00_JPRB ,&
&  0.62452E+00_JPRB, 0.62274E+00_JPRB, 0.62218E+00_JPRB, 0.61738E+00_JPRB, 0.61462E+00_JPRB, 0.60907E+00_JPRB, 0.56729E+00_JPRB ,&
&  0.52472E+00_JPRB, 0.49604E+00_JPRB, 0.47972E+00_JPRB, 0.47878E+00_JPRB, 0.50058E+00_JPRB, 0.54467E+00_JPRB, 0.59253E+00_JPRB ,&
&  0.63248E+00_JPRB, 0.67805E+00_JPRB, 0.71384E+00_JPRB, 0.73409E+00_JPRB, 0.75019E+00_JPRB, 0.77081E+00_JPRB, 0.79544E+00_JPRB ,&
&  0.83052E+00_JPRB, 0.87507E+00_JPRB, 0.91562E+00_JPRB, 0.95606E+00_JPRB, 0.98452E+00_JPRB, 0.95561E+00_JPRB, 0.10164E+01_JPRB ,&
&  0.12211E+01_JPRB, 0.10180E+01_JPRB, 0.86067E+00_JPRB, 0.80257E+00_JPRB, 0.79095E+00_JPRB, 0.79564E+00_JPRB, 0.80262E+00_JPRB ,&
&  0.81327E+00_JPRB, 0.81541E+00_JPRB, 0.81035E+00_JPRB, 0.80630E+00_JPRB, 0.81067E+00_JPRB, 0.81996E+00_JPRB, 0.84119E+00_JPRB ,&
&  0.87598E+00_JPRB, 0.10338E+01_JPRB, 0.12806E+01_JPRB, 0.99340E+00_JPRB, 0.88756E+00_JPRB, 0.81102E+00_JPRB, 0.74899E+00_JPRB ,&
&  0.72450E+00_JPRB, 0.71962E+00_JPRB, 0.72788E+00_JPRB, 0.73607E+00_JPRB, 0.74004E+00_JPRB, 0.74742E+00_JPRB, 0.75939E+00_JPRB ,&
&  0.77633E+00_JPRB, 0.79340E+00_JPRB, 0.80454E+00_JPRB, 0.81048E+00_JPRB, 0.80750E+00_JPRB, 0.79533E+00_JPRB, 0.77685E+00_JPRB ,&
&  0.77515E+00_JPRB, 0.78399E+00_JPRB, 0.79048E+00_JPRB, 0.79407E+00_JPRB, 0.79154E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.79265E+00_JPRB, 0.82731E+00_JPRB, 0.87267E+00_JPRB, 0.88413E+00_JPRB, 0.87826E+00_JPRB, 0.83482E+00_JPRB, 0.79089E+00_JPRB ,&
&  0.76618E+00_JPRB, 0.75777E+00_JPRB, 0.75395E+00_JPRB, 0.74209E+00_JPRB, 0.73005E+00_JPRB, 0.72337E+00_JPRB, 0.71830E+00_JPRB ,&
&  0.71385E+00_JPRB, 0.70938E+00_JPRB, 0.70344E+00_JPRB, 0.69122E+00_JPRB, 0.68495E+00_JPRB, 0.68665E+00_JPRB, 0.68524E+00_JPRB ,&
&  0.68031E+00_JPRB, 0.67890E+00_JPRB, 0.66725E+00_JPRB, 0.65138E+00_JPRB, 0.64418E+00_JPRB, 0.64380E+00_JPRB, 0.64641E+00_JPRB ,&
&  0.63734E+00_JPRB, 0.62956E+00_JPRB, 0.63560E+00_JPRB, 0.64420E+00_JPRB, 0.63788E+00_JPRB, 0.62844E+00_JPRB, 0.58152E+00_JPRB ,&
&  0.53401E+00_JPRB, 0.50450E+00_JPRB, 0.49683E+00_JPRB, 0.52603E+00_JPRB, 0.57835E+00_JPRB, 0.64208E+00_JPRB, 0.68601E+00_JPRB ,&
&  0.70992E+00_JPRB, 0.74257E+00_JPRB, 0.77281E+00_JPRB, 0.80535E+00_JPRB, 0.84622E+00_JPRB, 0.89104E+00_JPRB, 0.94301E+00_JPRB ,&
&  0.98404E+00_JPRB, 0.10088E+01_JPRB, 0.10072E+01_JPRB, 0.92295E+00_JPRB, 0.90576E+00_JPRB, 0.90909E+00_JPRB, 0.98670E+00_JPRB ,&
&  0.10695E+01_JPRB, 0.99098E+00_JPRB, 0.93388E+00_JPRB, 0.93107E+00_JPRB, 0.92317E+00_JPRB, 0.93592E+00_JPRB, 0.94083E+00_JPRB ,&
&  0.92659E+00_JPRB, 0.89970E+00_JPRB, 0.87068E+00_JPRB, 0.84819E+00_JPRB, 0.85010E+00_JPRB, 0.87227E+00_JPRB, 0.89584E+00_JPRB ,&
&  0.94386E+00_JPRB, 0.10481E+01_JPRB, 0.10244E+01_JPRB, 0.81441E+00_JPRB, 0.78166E+00_JPRB, 0.76550E+00_JPRB, 0.74845E+00_JPRB ,&
&  0.72241E+00_JPRB, 0.70708E+00_JPRB, 0.70733E+00_JPRB, 0.71068E+00_JPRB, 0.70834E+00_JPRB, 0.70574E+00_JPRB, 0.70905E+00_JPRB ,&
&  0.71735E+00_JPRB, 0.73217E+00_JPRB, 0.75163E+00_JPRB, 0.78051E+00_JPRB, 0.79388E+00_JPRB, 0.78997E+00_JPRB, 0.77252E+00_JPRB ,&
&  0.76510E+00_JPRB, 0.77051E+00_JPRB, 0.77406E+00_JPRB, 0.77382E+00_JPRB, 0.77798E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.90580E+00_JPRB, 0.91708E+00_JPRB, 0.91863E+00_JPRB, 0.91888E+00_JPRB, 0.87644E+00_JPRB, 0.85250E+00_JPRB, 0.81292E+00_JPRB ,&
&  0.80234E+00_JPRB, 0.77834E+00_JPRB, 0.74118E+00_JPRB, 0.69891E+00_JPRB, 0.67490E+00_JPRB, 0.66452E+00_JPRB, 0.66507E+00_JPRB ,&
&  0.67189E+00_JPRB, 0.68040E+00_JPRB, 0.68217E+00_JPRB, 0.67138E+00_JPRB, 0.67322E+00_JPRB, 0.67657E+00_JPRB, 0.67554E+00_JPRB ,&
&  0.67461E+00_JPRB, 0.67729E+00_JPRB, 0.67228E+00_JPRB, 0.66325E+00_JPRB, 0.65878E+00_JPRB, 0.65749E+00_JPRB, 0.65247E+00_JPRB ,&
&  0.64106E+00_JPRB, 0.63457E+00_JPRB, 0.64359E+00_JPRB, 0.66622E+00_JPRB, 0.64852E+00_JPRB, 0.63885E+00_JPRB, 0.58285E+00_JPRB ,&
&  0.53974E+00_JPRB, 0.53141E+00_JPRB, 0.56414E+00_JPRB, 0.63089E+00_JPRB, 0.66614E+00_JPRB, 0.70163E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.74331E+00_JPRB, 0.79371E+00_JPRB, 0.88708E+00_JPRB, 0.93172E+00_JPRB, 0.98938E+00_JPRB, 0.10458E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.11222E+01_JPRB, 0.11387E+01_JPRB, 0.10880E+01_JPRB, 0.11134E+01_JPRB, 0.12033E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10371E+01_JPRB, 0.10429E+01_JPRB, 0.10972E+01_JPRB, 0.10534E+01_JPRB, 0.10714E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.98270E+00_JPRB, 0.94273E+00_JPRB, 0.91330E+00_JPRB, 0.90401E+00_JPRB, 0.90420E+00_JPRB, 0.92094E+00_JPRB, 0.95170E+00_JPRB ,&
&  0.99971E+00_JPRB, 0.10182E+01_JPRB, 0.95190E+00_JPRB, 0.82366E+00_JPRB, 0.77442E+00_JPRB, 0.75283E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.71940E+00_JPRB, 0.70829E+00_JPRB, 0.71029E+00_JPRB, 0.71136E+00_JPRB, 0.70860E+00_JPRB, 0.69816E+00_JPRB, 0.68880E+00_JPRB ,&
&  0.68559E+00_JPRB, 0.69039E+00_JPRB, 0.69490E+00_JPRB, 0.72467E+00_JPRB, 0.75532E+00_JPRB, 0.77537E+00_JPRB, 0.77117E+00_JPRB ,&
&  0.76882E+00_JPRB, 0.78238E+00_JPRB, 0.80309E+00_JPRB, 0.84264E+00_JPRB, 0.87096E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.94284E+00_JPRB, 0.95890E+00_JPRB, 0.96625E+00_JPRB, 0.95390E+00_JPRB, 0.91833E+00_JPRB, 0.87429E+00_JPRB, 0.78743E+00_JPRB ,&
&  0.78187E+00_JPRB, 0.73643E+00_JPRB, 0.69945E+00_JPRB, 0.67188E+00_JPRB, 0.66205E+00_JPRB, 0.64855E+00_JPRB, 0.64374E+00_JPRB ,&
&  0.64402E+00_JPRB, 0.65962E+00_JPRB, 0.66976E+00_JPRB, 0.66352E+00_JPRB, 0.66798E+00_JPRB, 0.67263E+00_JPRB, 0.67093E+00_JPRB ,&
&  0.67651E+00_JPRB, 0.68423E+00_JPRB, 0.68882E+00_JPRB, 0.68720E+00_JPRB, 0.67469E+00_JPRB, 0.66574E+00_JPRB, 0.65618E+00_JPRB ,&
&  0.65018E+00_JPRB, 0.64803E+00_JPRB, 0.65447E+00_JPRB, 0.67157E+00_JPRB, 0.65873E+00_JPRB, 0.64371E+00_JPRB, 0.61599E+00_JPRB ,&
&  0.56643E+00_JPRB, 0.58342E+00_JPRB, 0.64478E+00_JPRB, 0.67648E+00_JPRB, 0.70908E+00_JPRB, 0.74285E+00_JPRB, 0.79269E+00_JPRB ,&
&  0.83512E+00_JPRB, 0.88013E+00_JPRB, 0.93279E+00_JPRB, 0.99712E+00_JPRB, 0.10754E+01_JPRB, 0.11519E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.12727E+01_JPRB, 0.10740E+01_JPRB, 0.10579E+01_JPRB, 0.12197E+01_JPRB, 0.14775E+01_JPRB, 0.12989E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.11820E+01_JPRB, 0.11275E+01_JPRB, 0.11054E+01_JPRB, 0.11673E+01_JPRB, 0.11795E+01_JPRB, 0.12261E+01_JPRB, 0.12581E+01_JPRB ,&
&  0.11562E+01_JPRB, 0.10934E+01_JPRB, 0.10358E+01_JPRB, 0.10220E+01_JPRB, 0.10261E+01_JPRB, 0.10376E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.10346E+01_JPRB, 0.10125E+01_JPRB, 0.95967E+00_JPRB, 0.88556E+00_JPRB, 0.83220E+00_JPRB, 0.80050E+00_JPRB, 0.77618E+00_JPRB ,&
&  0.75603E+00_JPRB, 0.73491E+00_JPRB, 0.71956E+00_JPRB, 0.71333E+00_JPRB, 0.71384E+00_JPRB, 0.72395E+00_JPRB, 0.75084E+00_JPRB ,&
&  0.73703E+00_JPRB, 0.72916E+00_JPRB, 0.71634E+00_JPRB, 0.73744E+00_JPRB, 0.77557E+00_JPRB, 0.80878E+00_JPRB, 0.81745E+00_JPRB ,&
&  0.83670E+00_JPRB, 0.84744E+00_JPRB, 0.84467E+00_JPRB, 0.87209E+00_JPRB, 0.89741E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.97561E+00_JPRB, 0.99729E+00_JPRB, 0.10033E+01_JPRB, 0.10038E+01_JPRB, 0.96118E+00_JPRB, 0.88937E+00_JPRB, 0.82271E+00_JPRB ,&
&  0.88187E+00_JPRB, 0.80906E+00_JPRB, 0.80105E+00_JPRB, 0.70065E+00_JPRB, 0.66984E+00_JPRB, 0.65585E+00_JPRB, 0.63571E+00_JPRB ,&
&  0.61372E+00_JPRB, 0.62598E+00_JPRB, 0.64425E+00_JPRB, 0.65213E+00_JPRB, 0.65992E+00_JPRB, 0.66884E+00_JPRB, 0.66658E+00_JPRB ,&
&  0.67351E+00_JPRB, 0.68796E+00_JPRB, 0.70812E+00_JPRB, 0.70290E+00_JPRB, 0.68725E+00_JPRB, 0.68288E+00_JPRB, 0.67654E+00_JPRB ,&
&  0.66207E+00_JPRB, 0.65658E+00_JPRB, 0.66661E+00_JPRB, 0.67956E+00_JPRB, 0.68064E+00_JPRB, 0.65589E+00_JPRB, 0.63863E+00_JPRB ,&
&  0.63598E+00_JPRB, 0.64144E+00_JPRB, 0.68763E+00_JPRB, 0.71967E+00_JPRB, 0.75995E+00_JPRB, 0.80817E+00_JPRB, 0.85667E+00_JPRB ,&
&  0.91371E+00_JPRB, 0.97903E+00_JPRB, 0.10542E+01_JPRB, 0.11494E+01_JPRB, 0.12448E+01_JPRB, 0.13502E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.12075E+01_JPRB, 0.14491E+01_JPRB, 0.15537E+01_JPRB, 0.17589E+01_JPRB, 0.17369E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.16710E+01_JPRB, 0.14843E+01_JPRB, 0.13669E+01_JPRB, 0.12796E+01_JPRB, 0.12762E+01_JPRB, 0.13558E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.15096E+01_JPRB, 0.14597E+01_JPRB, 0.13629E+01_JPRB, 0.13202E+01_JPRB, 0.12963E+01_JPRB, 0.12770E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.11832E+01_JPRB, 0.11199E+01_JPRB, 0.10546E+01_JPRB, 0.96884E+00_JPRB, 0.92398E+00_JPRB, 0.88364E+00_JPRB, 0.84023E+00_JPRB ,&
&  0.81515E+00_JPRB, 0.77116E+00_JPRB, 0.72930E+00_JPRB, 0.70435E+00_JPRB, 0.68861E+00_JPRB, 0.68943E+00_JPRB, 0.71350E+00_JPRB ,&
&  0.72847E+00_JPRB, 0.73342E+00_JPRB, 0.78656E+00_JPRB, 0.83426E+00_JPRB, 0.86219E+00_JPRB, 0.88901E+00_JPRB, 0.93467E+00_JPRB ,&
&  0.88280E+00_JPRB, 0.89251E+00_JPRB, 0.94228E+00_JPRB, 0.95443E+00_JPRB, 0.96258E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.10370E+01_JPRB, 0.10643E+01_JPRB, 0.10589E+01_JPRB, 0.10559E+01_JPRB, 0.98543E+00_JPRB, 0.95505E+00_JPRB, 0.87595E+00_JPRB ,&
&  0.89474E+00_JPRB, 0.86201E+00_JPRB, 0.79509E+00_JPRB, 0.77219E+00_JPRB, 0.77732E+00_JPRB, 0.70136E+00_JPRB, 0.64101E+00_JPRB ,&
&  0.60710E+00_JPRB, 0.60380E+00_JPRB, 0.63183E+00_JPRB, 0.64956E+00_JPRB, 0.65491E+00_JPRB, 0.66254E+00_JPRB, 0.65664E+00_JPRB ,&
&  0.68556E+00_JPRB, 0.69697E+00_JPRB, 0.71637E+00_JPRB, 0.72245E+00_JPRB, 0.71645E+00_JPRB, 0.71831E+00_JPRB, 0.72442E+00_JPRB ,&
&  0.70334E+00_JPRB, 0.69664E+00_JPRB, 0.68382E+00_JPRB, 0.67054E+00_JPRB, 0.71394E+00_JPRB, 0.69957E+00_JPRB, 0.68001E+00_JPRB ,&
&  0.68094E+00_JPRB, 0.70148E+00_JPRB, 0.73364E+00_JPRB, 0.77698E+00_JPRB, 0.82212E+00_JPRB, 0.86964E+00_JPRB, 0.92500E+00_JPRB ,&
&  0.98079E+00_JPRB, 0.10588E+01_JPRB, 0.11281E+01_JPRB, 0.12539E+01_JPRB, 0.13596E+01_JPRB, 0.13403E+01_JPRB, 0.15519E+01_JPRB ,&
&  0.16771E+01_JPRB, 0.18394E+01_JPRB, 0.20102E+01_JPRB, 0.18771E+01_JPRB, 0.20877E+01_JPRB, 0.20272E+01_JPRB, 0.18456E+01_JPRB ,&
&  0.18618E+01_JPRB, 0.17219E+01_JPRB, 0.16826E+01_JPRB, 0.16751E+01_JPRB, 0.17608E+01_JPRB, 0.17122E+01_JPRB, 0.16882E+01_JPRB ,&
&  0.17648E+01_JPRB, 0.19926E+01_JPRB, 0.18867E+01_JPRB, 0.18919E+01_JPRB, 0.18160E+01_JPRB, 0.17397E+01_JPRB, 0.16058E+01_JPRB ,&
&  0.15025E+01_JPRB, 0.13833E+01_JPRB, 0.12711E+01_JPRB, 0.11800E+01_JPRB, 0.10692E+01_JPRB, 0.98456E+00_JPRB, 0.91725E+00_JPRB ,&
&  0.84640E+00_JPRB, 0.76080E+00_JPRB, 0.72818E+00_JPRB, 0.70680E+00_JPRB, 0.69603E+00_JPRB, 0.67289E+00_JPRB, 0.68213E+00_JPRB ,&
&  0.71836E+00_JPRB, 0.78583E+00_JPRB, 0.88727E+00_JPRB, 0.95062E+00_JPRB, 0.10315E+01_JPRB, 0.11094E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.94450E+00_JPRB, 0.10173E+01_JPRB, 0.10325E+01_JPRB, 0.10345E+01_JPRB, 0.10325E+01_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.11795E+01_JPRB, 0.11891E+01_JPRB, 0.11661E+01_JPRB, 0.11249E+01_JPRB, 0.10274E+01_JPRB, 0.98110E+00_JPRB, 0.91946E+00_JPRB ,&
&  0.91867E+00_JPRB, 0.87998E+00_JPRB, 0.81490E+00_JPRB, 0.79021E+00_JPRB, 0.78906E+00_JPRB, 0.77092E+00_JPRB, 0.69917E+00_JPRB ,&
&  0.64692E+00_JPRB, 0.64394E+00_JPRB, 0.63366E+00_JPRB, 0.65978E+00_JPRB, 0.68813E+00_JPRB, 0.67975E+00_JPRB, 0.69369E+00_JPRB ,&
&  0.72778E+00_JPRB, 0.73513E+00_JPRB, 0.73434E+00_JPRB, 0.73434E+00_JPRB, 0.73811E+00_JPRB, 0.74986E+00_JPRB, 0.75287E+00_JPRB ,&
&  0.72633E+00_JPRB, 0.70619E+00_JPRB, 0.68622E+00_JPRB, 0.68412E+00_JPRB, 0.73160E+00_JPRB, 0.75044E+00_JPRB, 0.74166E+00_JPRB ,&
&  0.73804E+00_JPRB, 0.74362E+00_JPRB, 0.76185E+00_JPRB, 0.80745E+00_JPRB, 0.86004E+00_JPRB, 0.91454E+00_JPRB, 0.98028E+00_JPRB ,&
&  0.10531E+01_JPRB, 0.11380E+01_JPRB, 0.12122E+01_JPRB, 0.13068E+01_JPRB, 0.13166E+01_JPRB, 0.14463E+01_JPRB, 0.16980E+01_JPRB ,&
&  0.18326E+01_JPRB, 0.19961E+01_JPRB, 0.21325E+01_JPRB, 0.23489E+01_JPRB, 0.25146E+01_JPRB, 0.23664E+01_JPRB, 0.21977E+01_JPRB ,&
&  0.21385E+01_JPRB, 0.20564E+01_JPRB, 0.25303E+01_JPRB, 0.23436E+01_JPRB, 0.21642E+01_JPRB, 0.21225E+01_JPRB, 0.20535E+01_JPRB ,&
&  0.20955E+01_JPRB, 0.22010E+01_JPRB, 0.22473E+01_JPRB, 0.22817E+01_JPRB, 0.22726E+01_JPRB, 0.22680E+01_JPRB, 0.21337E+01_JPRB ,&
&  0.20426E+01_JPRB, 0.18687E+01_JPRB, 0.15093E+01_JPRB, 0.12403E+01_JPRB, 0.10327E+01_JPRB, 0.10232E+01_JPRB, 0.95269E+00_JPRB ,&
&  0.85910E+00_JPRB, 0.76322E+00_JPRB, 0.74557E+00_JPRB, 0.72531E+00_JPRB, 0.72293E+00_JPRB, 0.71157E+00_JPRB, 0.69960E+00_JPRB ,&
&  0.73510E+00_JPRB, 0.84096E+00_JPRB, 0.92301E+00_JPRB, 0.10559E+01_JPRB, 0.12042E+01_JPRB, 0.13271E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.12316E+01_JPRB, 0.12506E+01_JPRB, 0.12420E+01_JPRB, 0.12085E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.14405E+01_JPRB, 0.13867E+01_JPRB, 0.13163E+01_JPRB, 0.12119E+01_JPRB, 0.10782E+01_JPRB, 0.99813E+00_JPRB, 0.93141E+00_JPRB ,&
&  0.91922E+00_JPRB, 0.88294E+00_JPRB, 0.82794E+00_JPRB, 0.80565E+00_JPRB, 0.79901E+00_JPRB, 0.79137E+00_JPRB, 0.78512E+00_JPRB ,&
&  0.70530E+00_JPRB, 0.71729E+00_JPRB, 0.68243E+00_JPRB, 0.71305E+00_JPRB, 0.81416E+00_JPRB, 0.74284E+00_JPRB, 0.75113E+00_JPRB ,&
&  0.75424E+00_JPRB, 0.77080E+00_JPRB, 0.75584E+00_JPRB, 0.75484E+00_JPRB, 0.77847E+00_JPRB, 0.80067E+00_JPRB, 0.80055E+00_JPRB ,&
&  0.77335E+00_JPRB, 0.74065E+00_JPRB, 0.72763E+00_JPRB, 0.72991E+00_JPRB, 0.77618E+00_JPRB, 0.81175E+00_JPRB, 0.82809E+00_JPRB ,&
&  0.82934E+00_JPRB, 0.82820E+00_JPRB, 0.82954E+00_JPRB, 0.85623E+00_JPRB, 0.90045E+00_JPRB, 0.95458E+00_JPRB, 0.10376E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.12144E+01_JPRB, 0.13058E+01_JPRB, 0.14039E+01_JPRB, 0.14174E+01_JPRB, 0.16146E+01_JPRB, 0.18930E+01_JPRB ,&
&  0.21202E+01_JPRB, 0.21016E+01_JPRB, 0.21900E+01_JPRB, 0.26758E+01_JPRB, 0.29800E+01_JPRB, 0.29252E+01_JPRB, 0.27221E+01_JPRB ,&
&  0.25184E+01_JPRB, 0.23700E+01_JPRB, 0.24747E+01_JPRB, 0.24510E+01_JPRB, 0.22823E+01_JPRB, 0.23524E+01_JPRB, 0.23377E+01_JPRB ,&
&  0.23409E+01_JPRB, 0.24249E+01_JPRB, 0.25556E+01_JPRB, 0.26482E+01_JPRB, 0.25630E+01_JPRB, 0.25994E+01_JPRB, 0.25126E+01_JPRB ,&
&  0.24148E+01_JPRB, 0.20620E+01_JPRB, 0.15039E+01_JPRB, 0.12194E+01_JPRB, 0.93622E+00_JPRB, 0.90518E+00_JPRB, 0.10332E+01_JPRB ,&
&  0.10015E+01_JPRB, 0.94344E+00_JPRB, 0.87320E+00_JPRB, 0.82946E+00_JPRB, 0.80371E+00_JPRB, 0.83401E+00_JPRB, 0.85449E+00_JPRB ,&
&  0.87658E+00_JPRB, 0.10106E+01_JPRB, 0.12281E+01_JPRB, 0.14643E+01_JPRB, 0.16007E+01_JPRB, 0.16486E+01_JPRB, 0.15059E+01_JPRB ,&
&  0.15949E+01_JPRB, 0.15669E+01_JPRB, 0.15627E+01_JPRB, 0.15648E+01_JPRB, 0.15212E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.16992E+01_JPRB, 0.16081E+01_JPRB, 0.14704E+01_JPRB, 0.12985E+01_JPRB, 0.11097E+01_JPRB, 0.99701E+00_JPRB, 0.94948E+00_JPRB ,&
&  0.93511E+00_JPRB, 0.90260E+00_JPRB, 0.85005E+00_JPRB, 0.82170E+00_JPRB, 0.81099E+00_JPRB, 0.80235E+00_JPRB, 0.81286E+00_JPRB ,&
&  0.85371E+00_JPRB, 0.87277E+00_JPRB, 0.77781E+00_JPRB, 0.71231E+00_JPRB, 0.75726E+00_JPRB, 0.74936E+00_JPRB, 0.76702E+00_JPRB ,&
&  0.77079E+00_JPRB, 0.81378E+00_JPRB, 0.78079E+00_JPRB, 0.79739E+00_JPRB, 0.86489E+00_JPRB, 0.90702E+00_JPRB, 0.92897E+00_JPRB ,&
&  0.92608E+00_JPRB, 0.90312E+00_JPRB, 0.88329E+00_JPRB, 0.90445E+00_JPRB, 0.95323E+00_JPRB, 0.92266E+00_JPRB, 0.94689E+00_JPRB ,&
&  0.93733E+00_JPRB, 0.93471E+00_JPRB, 0.91864E+00_JPRB, 0.94185E+00_JPRB, 0.99872E+00_JPRB, 0.10931E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.12305E+01_JPRB, 0.12905E+01_JPRB, 0.13260E+01_JPRB, 0.14243E+01_JPRB, 0.15679E+01_JPRB, 0.17549E+01_JPRB, 0.17980E+01_JPRB ,&
&  0.19296E+01_JPRB, 0.19505E+01_JPRB, 0.19756E+01_JPRB, 0.24166E+01_JPRB, 0.28220E+01_JPRB, 0.29334E+01_JPRB, 0.29335E+01_JPRB ,&
&  0.28701E+01_JPRB, 0.27864E+01_JPRB, 0.28674E+01_JPRB, 0.27634E+01_JPRB, 0.26830E+01_JPRB, 0.27166E+01_JPRB, 0.27599E+01_JPRB ,&
&  0.26875E+01_JPRB, 0.27478E+01_JPRB, 0.28860E+01_JPRB, 0.28689E+01_JPRB, 0.28168E+01_JPRB, 0.27019E+01_JPRB, 0.27074E+01_JPRB ,&
&  0.26628E+01_JPRB, 0.16509E+01_JPRB, 0.12998E+01_JPRB, 0.13516E+01_JPRB, 0.10425E+01_JPRB, 0.92058E+00_JPRB, 0.98224E+00_JPRB ,&
&  0.94900E+00_JPRB, 0.93697E+00_JPRB, 0.97155E+00_JPRB, 0.99253E+00_JPRB, 0.10083E+01_JPRB, 0.11280E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.14374E+01_JPRB, 0.17826E+01_JPRB, 0.20035E+01_JPRB, 0.20566E+01_JPRB, 0.20435E+01_JPRB, 0.19983E+01_JPRB ,&
&  0.19402E+01_JPRB, 0.18771E+01_JPRB, 0.18350E+01_JPRB, 0.18138E+01_JPRB, 0.17755E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.19428E+01_JPRB, 0.18371E+01_JPRB, 0.16546E+01_JPRB, 0.14295E+01_JPRB, 0.11853E+01_JPRB, 0.10427E+01_JPRB, 0.10175E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.97545E+00_JPRB, 0.92543E+00_JPRB, 0.88374E+00_JPRB, 0.86522E+00_JPRB, 0.85051E+00_JPRB, 0.85294E+00_JPRB ,&
&  0.85497E+00_JPRB, 0.90354E+00_JPRB, 0.78205E+00_JPRB, 0.69286E+00_JPRB, 0.64465E+00_JPRB, 0.68154E+00_JPRB, 0.71850E+00_JPRB ,&
&  0.77075E+00_JPRB, 0.82623E+00_JPRB, 0.84557E+00_JPRB, 0.95865E+00_JPRB, 0.10669E+01_JPRB, 0.11223E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11930E+01_JPRB, 0.11222E+01_JPRB, 0.11572E+01_JPRB, 0.11401E+01_JPRB, 0.10639E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.10939E+01_JPRB, 0.10868E+01_JPRB, 0.10806E+01_JPRB, 0.11140E+01_JPRB, 0.11685E+01_JPRB, 0.11950E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.12618E+01_JPRB, 0.13509E+01_JPRB, 0.14403E+01_JPRB, 0.15558E+01_JPRB, 0.16841E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.17615E+01_JPRB, 0.16202E+01_JPRB, 0.14796E+01_JPRB, 0.18835E+01_JPRB, 0.26415E+01_JPRB, 0.29627E+01_JPRB, 0.28560E+01_JPRB ,&
&  0.30835E+01_JPRB, 0.31594E+01_JPRB, 0.32524E+01_JPRB, 0.31806E+01_JPRB, 0.30708E+01_JPRB, 0.30623E+01_JPRB, 0.31607E+01_JPRB ,&
&  0.31463E+01_JPRB, 0.32395E+01_JPRB, 0.32904E+01_JPRB, 0.32668E+01_JPRB, 0.31846E+01_JPRB, 0.31921E+01_JPRB, 0.32164E+01_JPRB ,&
&  0.29513E+01_JPRB, 0.24869E+01_JPRB, 0.17502E+01_JPRB, 0.16726E+01_JPRB, 0.12058E+01_JPRB, 0.10097E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.96349E+00_JPRB, 0.99144E+00_JPRB, 0.10853E+01_JPRB, 0.11659E+01_JPRB, 0.14252E+01_JPRB, 0.16932E+01_JPRB, 0.19846E+01_JPRB ,&
&  0.21578E+01_JPRB, 0.20919E+01_JPRB, 0.24431E+01_JPRB, 0.25093E+01_JPRB, 0.25060E+01_JPRB, 0.25099E+01_JPRB, 0.24976E+01_JPRB ,&
&  0.24183E+01_JPRB, 0.23035E+01_JPRB, 0.22055E+01_JPRB, 0.21100E+01_JPRB, 0.20275E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.20737E+01_JPRB, 0.19388E+01_JPRB, 0.17236E+01_JPRB, 0.14855E+01_JPRB, 0.12425E+01_JPRB, 0.10802E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.10969E+01_JPRB, 0.10672E+01_JPRB, 0.10086E+01_JPRB, 0.96852E+00_JPRB, 0.94310E+00_JPRB, 0.91783E+00_JPRB ,&
&  0.89241E+00_JPRB, 0.83886E+00_JPRB, 0.71015E+00_JPRB, 0.63247E+00_JPRB, 0.61138E+00_JPRB, 0.61716E+00_JPRB, 0.67632E+00_JPRB ,&
&  0.75636E+00_JPRB, 0.90110E+00_JPRB, 0.10515E+01_JPRB, 0.12277E+01_JPRB, 0.14388E+01_JPRB, 0.16408E+01_JPRB, 0.16996E+01_JPRB ,&
&  0.17312E+01_JPRB, 0.14855E+01_JPRB, 0.14771E+01_JPRB, 0.14065E+01_JPRB, 0.13813E+01_JPRB, 0.13318E+01_JPRB, 0.12966E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.12219E+01_JPRB, 0.12070E+01_JPRB, 0.12057E+01_JPRB, 0.12063E+01_JPRB, 0.12075E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.13528E+01_JPRB, 0.14252E+01_JPRB, 0.14866E+01_JPRB, 0.15930E+01_JPRB, 0.17285E+01_JPRB, 0.17690E+01_JPRB ,&
&  0.16961E+01_JPRB, 0.16729E+01_JPRB, 0.19490E+01_JPRB, 0.21848E+01_JPRB, 0.24820E+01_JPRB, 0.28869E+01_JPRB, 0.31215E+01_JPRB ,&
&  0.32700E+01_JPRB, 0.31761E+01_JPRB, 0.32033E+01_JPRB, 0.30034E+01_JPRB, 0.33716E+01_JPRB, 0.33955E+01_JPRB, 0.34622E+01_JPRB ,&
&  0.35198E+01_JPRB, 0.37061E+01_JPRB, 0.35871E+01_JPRB, 0.34906E+01_JPRB, 0.35691E+01_JPRB, 0.38045E+01_JPRB, 0.34262E+01_JPRB ,&
&  0.29122E+01_JPRB, 0.31584E+01_JPRB, 0.27718E+01_JPRB, 0.22882E+01_JPRB, 0.17661E+01_JPRB, 0.15257E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.12115E+01_JPRB, 0.11651E+01_JPRB, 0.12616E+01_JPRB, 0.15914E+01_JPRB, 0.21269E+01_JPRB, 0.25591E+01_JPRB, 0.28014E+01_JPRB ,&
&  0.28467E+01_JPRB, 0.34068E+01_JPRB, 0.33504E+01_JPRB, 0.31930E+01_JPRB, 0.31154E+01_JPRB, 0.30858E+01_JPRB, 0.29931E+01_JPRB ,&
&  0.27786E+01_JPRB, 0.26003E+01_JPRB, 0.24499E+01_JPRB, 0.23164E+01_JPRB, 0.21831E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.21272E+01_JPRB, 0.20047E+01_JPRB, 0.17936E+01_JPRB, 0.15398E+01_JPRB, 0.13083E+01_JPRB, 0.11534E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11747E+01_JPRB, 0.11866E+01_JPRB, 0.11803E+01_JPRB, 0.11528E+01_JPRB, 0.11016E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.99667E+00_JPRB, 0.83648E+00_JPRB, 0.69287E+00_JPRB, 0.64759E+00_JPRB, 0.62250E+00_JPRB, 0.61361E+00_JPRB, 0.65449E+00_JPRB ,&
&  0.77812E+00_JPRB, 0.10741E+01_JPRB, 0.14014E+01_JPRB, 0.17640E+01_JPRB, 0.20734E+01_JPRB, 0.22693E+01_JPRB, 0.22062E+01_JPRB ,&
&  0.20197E+01_JPRB, 0.19615E+01_JPRB, 0.18541E+01_JPRB, 0.17146E+01_JPRB, 0.16190E+01_JPRB, 0.15226E+01_JPRB, 0.14390E+01_JPRB ,&
&  0.13671E+01_JPRB, 0.13109E+01_JPRB, 0.13098E+01_JPRB, 0.13141E+01_JPRB, 0.13214E+01_JPRB, 0.13334E+01_JPRB, 0.13537E+01_JPRB ,&
&  0.13870E+01_JPRB, 0.14411E+01_JPRB, 0.14882E+01_JPRB, 0.15895E+01_JPRB, 0.14681E+01_JPRB, 0.15525E+01_JPRB, 0.16921E+01_JPRB ,&
&  0.19881E+01_JPRB, 0.20178E+01_JPRB, 0.19876E+01_JPRB, 0.21934E+01_JPRB, 0.21091E+01_JPRB, 0.23437E+01_JPRB, 0.29109E+01_JPRB ,&
&  0.31065E+01_JPRB, 0.31135E+01_JPRB, 0.32731E+01_JPRB, 0.28253E+01_JPRB, 0.27267E+01_JPRB, 0.38404E+01_JPRB, 0.40379E+01_JPRB ,&
&  0.40484E+01_JPRB, 0.40532E+01_JPRB, 0.38861E+01_JPRB, 0.39568E+01_JPRB, 0.35107E+01_JPRB, 0.31320E+01_JPRB, 0.24240E+01_JPRB ,&
&  0.26089E+01_JPRB, 0.25417E+01_JPRB, 0.27954E+01_JPRB, 0.25482E+01_JPRB, 0.23947E+01_JPRB, 0.21265E+01_JPRB, 0.17598E+01_JPRB ,&
&  0.15523E+01_JPRB, 0.14852E+01_JPRB, 0.16786E+01_JPRB, 0.24390E+01_JPRB, 0.33683E+01_JPRB, 0.33697E+01_JPRB, 0.40074E+01_JPRB ,&
&  0.43529E+01_JPRB, 0.44161E+01_JPRB, 0.38337E+01_JPRB, 0.37258E+01_JPRB, 0.34983E+01_JPRB, 0.32824E+01_JPRB, 0.30257E+01_JPRB ,&
&  0.28141E+01_JPRB, 0.26588E+01_JPRB, 0.25165E+01_JPRB, 0.23716E+01_JPRB, 0.22311E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.21720E+01_JPRB, 0.20599E+01_JPRB, 0.18539E+01_JPRB, 0.16000E+01_JPRB, 0.13756E+01_JPRB, 0.12355E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.12586E+01_JPRB, 0.12699E+01_JPRB, 0.12608E+01_JPRB, 0.12398E+01_JPRB, 0.12146E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.94694E+00_JPRB, 0.74610E+00_JPRB, 0.66541E+00_JPRB, 0.67936E+00_JPRB, 0.65816E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.92500E+00_JPRB, 0.13874E+01_JPRB, 0.19742E+01_JPRB, 0.24614E+01_JPRB, 0.27932E+01_JPRB, 0.27836E+01_JPRB, 0.25791E+01_JPRB ,&
&  0.25018E+01_JPRB, 0.22945E+01_JPRB, 0.20160E+01_JPRB, 0.17895E+01_JPRB, 0.16432E+01_JPRB, 0.15488E+01_JPRB, 0.14553E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.13229E+01_JPRB, 0.13121E+01_JPRB, 0.13070E+01_JPRB, 0.13224E+01_JPRB, 0.13242E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.14491E+01_JPRB, 0.15381E+01_JPRB, 0.16577E+01_JPRB, 0.15867E+01_JPRB, 0.16424E+01_JPRB, 0.18598E+01_JPRB ,&
&  0.20374E+01_JPRB, 0.20665E+01_JPRB, 0.20632E+01_JPRB, 0.21167E+01_JPRB, 0.22772E+01_JPRB, 0.23420E+01_JPRB, 0.27735E+01_JPRB ,&
&  0.26663E+01_JPRB, 0.25209E+01_JPRB, 0.24754E+01_JPRB, 0.23363E+01_JPRB, 0.28967E+01_JPRB, 0.40835E+01_JPRB, 0.45847E+01_JPRB ,&
&  0.45158E+01_JPRB, 0.44219E+01_JPRB, 0.43418E+01_JPRB, 0.39567E+01_JPRB, 0.32764E+01_JPRB, 0.25779E+01_JPRB, 0.30774E+01_JPRB ,&
&  0.33402E+01_JPRB, 0.35234E+01_JPRB, 0.33416E+01_JPRB, 0.27579E+01_JPRB, 0.24279E+01_JPRB, 0.27450E+01_JPRB, 0.24971E+01_JPRB ,&
&  0.21027E+01_JPRB, 0.20330E+01_JPRB, 0.28864E+01_JPRB, 0.40013E+01_JPRB, 0.45063E+01_JPRB, 0.45006E+01_JPRB, 0.50979E+01_JPRB ,&
&  0.49021E+01_JPRB, 0.46853E+01_JPRB, 0.42991E+01_JPRB, 0.38460E+01_JPRB, 0.34673E+01_JPRB, 0.31540E+01_JPRB, 0.29003E+01_JPRB ,&
&  0.27563E+01_JPRB, 0.26676E+01_JPRB, 0.25510E+01_JPRB, 0.24306E+01_JPRB, 0.22866E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.22126E+01_JPRB, 0.20911E+01_JPRB, 0.18759E+01_JPRB, 0.16186E+01_JPRB, 0.14374E+01_JPRB, 0.13235E+01_JPRB, 0.12780E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.12863E+01_JPRB, 0.12873E+01_JPRB, 0.12757E+01_JPRB, 0.12611E+01_JPRB, 0.12446E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.11478E+01_JPRB, 0.88349E+00_JPRB, 0.76471E+00_JPRB, 0.75135E+00_JPRB, 0.77711E+00_JPRB, 0.80440E+00_JPRB ,&
&  0.12236E+01_JPRB, 0.20110E+01_JPRB, 0.26756E+01_JPRB, 0.31200E+01_JPRB, 0.33145E+01_JPRB, 0.28481E+01_JPRB, 0.26744E+01_JPRB ,&
&  0.26051E+01_JPRB, 0.24065E+01_JPRB, 0.21421E+01_JPRB, 0.18929E+01_JPRB, 0.16814E+01_JPRB, 0.15617E+01_JPRB, 0.14874E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.13675E+01_JPRB, 0.13480E+01_JPRB, 0.13594E+01_JPRB, 0.13643E+01_JPRB, 0.13695E+01_JPRB, 0.13953E+01_JPRB ,&
&  0.14355E+01_JPRB, 0.14914E+01_JPRB, 0.15587E+01_JPRB, 0.17207E+01_JPRB, 0.17681E+01_JPRB, 0.18814E+01_JPRB, 0.22381E+01_JPRB ,&
&  0.22557E+01_JPRB, 0.23092E+01_JPRB, 0.22281E+01_JPRB, 0.22326E+01_JPRB, 0.22811E+01_JPRB, 0.23576E+01_JPRB, 0.27177E+01_JPRB ,&
&  0.26572E+01_JPRB, 0.26616E+01_JPRB, 0.28665E+01_JPRB, 0.30030E+01_JPRB, 0.31317E+01_JPRB, 0.36948E+01_JPRB, 0.51395E+01_JPRB ,&
&  0.44711E+01_JPRB, 0.44814E+01_JPRB, 0.45096E+01_JPRB, 0.43705E+01_JPRB, 0.24956E+01_JPRB, 0.19919E+01_JPRB, 0.30555E+01_JPRB ,&
&  0.32259E+01_JPRB, 0.20400E+01_JPRB, 0.14980E+01_JPRB, 0.19837E+01_JPRB, 0.15773E+01_JPRB, 0.19186E+01_JPRB, 0.27560E+01_JPRB ,&
&  0.29454E+01_JPRB, 0.27592E+01_JPRB, 0.38117E+01_JPRB, 0.44598E+01_JPRB, 0.46336E+01_JPRB, 0.55435E+01_JPRB, 0.55760E+01_JPRB ,&
&  0.53126E+01_JPRB, 0.43635E+01_JPRB, 0.42492E+01_JPRB, 0.37611E+01_JPRB, 0.33778E+01_JPRB, 0.31432E+01_JPRB, 0.29550E+01_JPRB ,&
&  0.27444E+01_JPRB, 0.25536E+01_JPRB, 0.24387E+01_JPRB, 0.23915E+01_JPRB, 0.23119E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.22026E+01_JPRB, 0.20782E+01_JPRB, 0.18550E+01_JPRB, 0.16059E+01_JPRB, 0.14609E+01_JPRB, 0.13614E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.12946E+01_JPRB, 0.12959E+01_JPRB, 0.12893E+01_JPRB, 0.12772E+01_JPRB, 0.12662E+01_JPRB, 0.12646E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.12312E+01_JPRB, 0.11569E+01_JPRB, 0.10357E+01_JPRB, 0.93197E+00_JPRB, 0.91776E+00_JPRB, 0.94176E+00_JPRB, 0.10793E+01_JPRB ,&
&  0.15628E+01_JPRB, 0.25681E+01_JPRB, 0.32866E+01_JPRB, 0.34673E+01_JPRB, 0.31075E+01_JPRB, 0.27095E+01_JPRB, 0.27525E+01_JPRB ,&
&  0.26493E+01_JPRB, 0.24279E+01_JPRB, 0.21576E+01_JPRB, 0.19560E+01_JPRB, 0.17832E+01_JPRB, 0.16659E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.15089E+01_JPRB, 0.14500E+01_JPRB, 0.14387E+01_JPRB, 0.14310E+01_JPRB, 0.14269E+01_JPRB, 0.14346E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.14920E+01_JPRB, 0.15490E+01_JPRB, 0.16378E+01_JPRB, 0.17917E+01_JPRB, 0.21213E+01_JPRB, 0.23078E+01_JPRB, 0.22334E+01_JPRB ,&
&  0.21251E+01_JPRB, 0.21734E+01_JPRB, 0.24056E+01_JPRB, 0.22752E+01_JPRB, 0.22352E+01_JPRB, 0.23211E+01_JPRB, 0.26032E+01_JPRB ,&
&  0.30191E+01_JPRB, 0.32676E+01_JPRB, 0.30568E+01_JPRB, 0.32098E+01_JPRB, 0.34008E+01_JPRB, 0.31888E+01_JPRB, 0.42494E+01_JPRB ,&
&  0.46588E+01_JPRB, 0.41131E+01_JPRB, 0.33520E+01_JPRB, 0.27623E+01_JPRB, 0.30191E+01_JPRB, 0.24599E+01_JPRB, 0.15273E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.12230E+01_JPRB, 0.11093E+01_JPRB, 0.11959E+01_JPRB, 0.11949E+01_JPRB, 0.16958E+01_JPRB, 0.26996E+01_JPRB ,&
&  0.36643E+01_JPRB, 0.38788E+01_JPRB, 0.44543E+01_JPRB, 0.46495E+01_JPRB, 0.45958E+01_JPRB, 0.55654E+01_JPRB, 0.56965E+01_JPRB ,&
&  0.45569E+01_JPRB, 0.39956E+01_JPRB, 0.39017E+01_JPRB, 0.34750E+01_JPRB, 0.32540E+01_JPRB, 0.30812E+01_JPRB, 0.28525E+01_JPRB ,&
&  0.25707E+01_JPRB, 0.23227E+01_JPRB, 0.22249E+01_JPRB, 0.22451E+01_JPRB, 0.22761E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.21720E+01_JPRB, 0.20486E+01_JPRB, 0.18509E+01_JPRB, 0.16359E+01_JPRB, 0.15179E+01_JPRB, 0.14378E+01_JPRB, 0.13822E+01_JPRB ,&
&  0.13547E+01_JPRB, 0.13327E+01_JPRB, 0.13190E+01_JPRB, 0.13120E+01_JPRB, 0.13104E+01_JPRB, 0.13145E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.12788E+01_JPRB, 0.11948E+01_JPRB, 0.11408E+01_JPRB, 0.10838E+01_JPRB, 0.11208E+01_JPRB, 0.11534E+01_JPRB, 0.13812E+01_JPRB ,&
&  0.20413E+01_JPRB, 0.31387E+01_JPRB, 0.37504E+01_JPRB, 0.36435E+01_JPRB, 0.32774E+01_JPRB, 0.29733E+01_JPRB, 0.27679E+01_JPRB ,&
&  0.25296E+01_JPRB, 0.23558E+01_JPRB, 0.21730E+01_JPRB, 0.20170E+01_JPRB, 0.18885E+01_JPRB, 0.17855E+01_JPRB, 0.16936E+01_JPRB ,&
&  0.16074E+01_JPRB, 0.15748E+01_JPRB, 0.15345E+01_JPRB, 0.15148E+01_JPRB, 0.14979E+01_JPRB, 0.14898E+01_JPRB, 0.15007E+01_JPRB ,&
&  0.15479E+01_JPRB, 0.16388E+01_JPRB, 0.18002E+01_JPRB, 0.20733E+01_JPRB, 0.21389E+01_JPRB, 0.19584E+01_JPRB, 0.20992E+01_JPRB ,&
&  0.21973E+01_JPRB, 0.23503E+01_JPRB, 0.22950E+01_JPRB, 0.23808E+01_JPRB, 0.22818E+01_JPRB, 0.22485E+01_JPRB, 0.25510E+01_JPRB ,&
&  0.30311E+01_JPRB, 0.35837E+01_JPRB, 0.30923E+01_JPRB, 0.30086E+01_JPRB, 0.37485E+01_JPRB, 0.35409E+01_JPRB, 0.38825E+01_JPRB ,&
&  0.44000E+01_JPRB, 0.38372E+01_JPRB, 0.34338E+01_JPRB, 0.32032E+01_JPRB, 0.45320E+01_JPRB, 0.48297E+01_JPRB, 0.23850E+01_JPRB ,&
&  0.16681E+01_JPRB, 0.13596E+01_JPRB, 0.12007E+01_JPRB, 0.11288E+01_JPRB, 0.11687E+01_JPRB, 0.17718E+01_JPRB, 0.33259E+01_JPRB ,&
&  0.38420E+01_JPRB, 0.47245E+01_JPRB, 0.51651E+01_JPRB, 0.51686E+01_JPRB, 0.49648E+01_JPRB, 0.50231E+01_JPRB, 0.44624E+01_JPRB ,&
&  0.44080E+01_JPRB, 0.40281E+01_JPRB, 0.35758E+01_JPRB, 0.31943E+01_JPRB, 0.28924E+01_JPRB, 0.25592E+01_JPRB, 0.22852E+01_JPRB ,&
&  0.21000E+01_JPRB, 0.20364E+01_JPRB, 0.20297E+01_JPRB, 0.21020E+01_JPRB, 0.22223E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.21392E+01_JPRB, 0.20714E+01_JPRB, 0.18982E+01_JPRB, 0.17052E+01_JPRB, 0.15921E+01_JPRB, 0.15160E+01_JPRB, 0.14665E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.13726E+01_JPRB, 0.13376E+01_JPRB, 0.13214E+01_JPRB, 0.13275E+01_JPRB, 0.13266E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.12213E+01_JPRB, 0.11617E+01_JPRB, 0.12412E+01_JPRB, 0.13161E+01_JPRB, 0.14533E+01_JPRB, 0.17951E+01_JPRB ,&
&  0.27163E+01_JPRB, 0.39020E+01_JPRB, 0.41009E+01_JPRB, 0.38650E+01_JPRB, 0.34501E+01_JPRB, 0.30914E+01_JPRB, 0.25120E+01_JPRB ,&
&  0.22154E+01_JPRB, 0.21712E+01_JPRB, 0.20617E+01_JPRB, 0.20368E+01_JPRB, 0.19862E+01_JPRB, 0.18873E+01_JPRB, 0.17809E+01_JPRB ,&
&  0.17002E+01_JPRB, 0.16281E+01_JPRB, 0.15888E+01_JPRB, 0.15629E+01_JPRB, 0.15400E+01_JPRB, 0.15246E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.16574E+01_JPRB, 0.17767E+01_JPRB, 0.19946E+01_JPRB, 0.22025E+01_JPRB, 0.18751E+01_JPRB, 0.20329E+01_JPRB, 0.20942E+01_JPRB ,&
&  0.21253E+01_JPRB, 0.21641E+01_JPRB, 0.20661E+01_JPRB, 0.21202E+01_JPRB, 0.22278E+01_JPRB, 0.22501E+01_JPRB, 0.24648E+01_JPRB ,&
&  0.31913E+01_JPRB, 0.30040E+01_JPRB, 0.27721E+01_JPRB, 0.28774E+01_JPRB, 0.34860E+01_JPRB, 0.50312E+01_JPRB, 0.36135E+01_JPRB ,&
&  0.33627E+01_JPRB, 0.38851E+01_JPRB, 0.37357E+01_JPRB, 0.39052E+01_JPRB, 0.51574E+01_JPRB, 0.66899E+01_JPRB, 0.63169E+01_JPRB ,&
&  0.24710E+01_JPRB, 0.15921E+01_JPRB, 0.15556E+01_JPRB, 0.15162E+01_JPRB, 0.14278E+01_JPRB, 0.18896E+01_JPRB, 0.55289E+01_JPRB ,&
&  0.47613E+01_JPRB, 0.55901E+01_JPRB, 0.56292E+01_JPRB, 0.51787E+01_JPRB, 0.50312E+01_JPRB, 0.48665E+01_JPRB, 0.43937E+01_JPRB ,&
&  0.38833E+01_JPRB, 0.34154E+01_JPRB, 0.29576E+01_JPRB, 0.25081E+01_JPRB, 0.21548E+01_JPRB, 0.19258E+01_JPRB, 0.18310E+01_JPRB ,&
&  0.18131E+01_JPRB, 0.18433E+01_JPRB, 0.18973E+01_JPRB, 0.19896E+01_JPRB, 0.21162E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.21115E+01_JPRB, 0.20858E+01_JPRB, 0.19200E+01_JPRB, 0.17513E+01_JPRB, 0.16426E+01_JPRB, 0.15809E+01_JPRB, 0.15030E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.14017E+01_JPRB, 0.13315E+01_JPRB, 0.12931E+01_JPRB, 0.12960E+01_JPRB, 0.12918E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.12314E+01_JPRB, 0.11902E+01_JPRB, 0.12268E+01_JPRB, 0.14118E+01_JPRB, 0.13682E+01_JPRB, 0.17192E+01_JPRB ,&
&  0.32015E+01_JPRB, 0.48477E+01_JPRB, 0.42674E+01_JPRB, 0.38587E+01_JPRB, 0.33978E+01_JPRB, 0.30909E+01_JPRB, 0.22626E+01_JPRB ,&
&  0.20455E+01_JPRB, 0.20409E+01_JPRB, 0.20110E+01_JPRB, 0.20269E+01_JPRB, 0.19871E+01_JPRB, 0.19053E+01_JPRB, 0.18077E+01_JPRB ,&
&  0.16950E+01_JPRB, 0.16295E+01_JPRB, 0.16212E+01_JPRB, 0.16067E+01_JPRB, 0.15813E+01_JPRB, 0.15984E+01_JPRB, 0.16691E+01_JPRB ,&
&  0.17562E+01_JPRB, 0.18958E+01_JPRB, 0.21302E+01_JPRB, 0.21096E+01_JPRB, 0.20220E+01_JPRB, 0.20242E+01_JPRB, 0.20807E+01_JPRB ,&
&  0.20570E+01_JPRB, 0.20734E+01_JPRB, 0.20669E+01_JPRB, 0.20312E+01_JPRB, 0.21258E+01_JPRB, 0.22853E+01_JPRB, 0.24694E+01_JPRB ,&
&  0.28297E+01_JPRB, 0.26764E+01_JPRB, 0.24988E+01_JPRB, 0.27552E+01_JPRB, 0.33906E+01_JPRB, 0.46040E+01_JPRB, 0.47057E+01_JPRB ,&
&  0.42765E+01_JPRB, 0.34084E+01_JPRB, 0.36144E+01_JPRB, 0.48619E+01_JPRB, 0.57609E+01_JPRB, 0.66157E+01_JPRB, 0.92893E+01_JPRB ,&
&  0.91971E+01_JPRB, 0.48942E+01_JPRB, 0.38121E+01_JPRB, 0.48564E+01_JPRB, 0.32503E+01_JPRB, 0.23630E+01_JPRB, 0.37674E+01_JPRB ,&
&  0.47075E+01_JPRB, 0.54489E+01_JPRB, 0.55862E+01_JPRB, 0.52263E+01_JPRB, 0.51403E+01_JPRB, 0.46759E+01_JPRB, 0.38052E+01_JPRB ,&
&  0.31327E+01_JPRB, 0.25426E+01_JPRB, 0.21350E+01_JPRB, 0.19162E+01_JPRB, 0.18275E+01_JPRB, 0.17809E+01_JPRB, 0.17581E+01_JPRB ,&
&  0.17598E+01_JPRB, 0.17914E+01_JPRB, 0.18569E+01_JPRB, 0.19751E+01_JPRB, 0.20857E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.20734E+01_JPRB, 0.20267E+01_JPRB, 0.18842E+01_JPRB, 0.17516E+01_JPRB, 0.16629E+01_JPRB, 0.15905E+01_JPRB, 0.14974E+01_JPRB ,&
&  0.15950E+01_JPRB, 0.14254E+01_JPRB, 0.13089E+01_JPRB, 0.12620E+01_JPRB, 0.12437E+01_JPRB, 0.12462E+01_JPRB, 0.12505E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12315E+01_JPRB, 0.12261E+01_JPRB, 0.12740E+01_JPRB, 0.14419E+01_JPRB, 0.19536E+01_JPRB, 0.19295E+01_JPRB ,&
&  0.33566E+01_JPRB, 0.56603E+01_JPRB, 0.44152E+01_JPRB, 0.38646E+01_JPRB, 0.34605E+01_JPRB, 0.32444E+01_JPRB, 0.22897E+01_JPRB ,&
&  0.20178E+01_JPRB, 0.19743E+01_JPRB, 0.19504E+01_JPRB, 0.19385E+01_JPRB, 0.19279E+01_JPRB, 0.18703E+01_JPRB, 0.17353E+01_JPRB ,&
&  0.16464E+01_JPRB, 0.16141E+01_JPRB, 0.16290E+01_JPRB, 0.16346E+01_JPRB, 0.16432E+01_JPRB, 0.16960E+01_JPRB, 0.17423E+01_JPRB ,&
&  0.18381E+01_JPRB, 0.20868E+01_JPRB, 0.22503E+01_JPRB, 0.20646E+01_JPRB, 0.20349E+01_JPRB, 0.20629E+01_JPRB, 0.20974E+01_JPRB ,&
&  0.19817E+01_JPRB, 0.18538E+01_JPRB, 0.19626E+01_JPRB, 0.20610E+01_JPRB, 0.21436E+01_JPRB, 0.22114E+01_JPRB, 0.22921E+01_JPRB ,&
&  0.26066E+01_JPRB, 0.27455E+01_JPRB, 0.30133E+01_JPRB, 0.30278E+01_JPRB, 0.34495E+01_JPRB, 0.43391E+01_JPRB, 0.47558E+01_JPRB ,&
&  0.51288E+01_JPRB, 0.46842E+01_JPRB, 0.52081E+01_JPRB, 0.59859E+01_JPRB, 0.62574E+01_JPRB, 0.63699E+01_JPRB, 0.73340E+01_JPRB ,&
&  0.84247E+01_JPRB, 0.10009E+02_JPRB, 0.88271E+01_JPRB, 0.67822E+01_JPRB, 0.49305E+01_JPRB, 0.34206E+01_JPRB, 0.39097E+01_JPRB ,&
&  0.49766E+01_JPRB, 0.51097E+01_JPRB, 0.51072E+01_JPRB, 0.64761E+01_JPRB, 0.48661E+01_JPRB, 0.36473E+01_JPRB, 0.26832E+01_JPRB ,&
&  0.21280E+01_JPRB, 0.19197E+01_JPRB, 0.18685E+01_JPRB, 0.18113E+01_JPRB, 0.17873E+01_JPRB, 0.17642E+01_JPRB, 0.17698E+01_JPRB ,&
&  0.18071E+01_JPRB, 0.18650E+01_JPRB, 0.19356E+01_JPRB, 0.20219E+01_JPRB, 0.20840E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.19738E+01_JPRB, 0.18942E+01_JPRB, 0.17928E+01_JPRB, 0.16961E+01_JPRB, 0.16061E+01_JPRB, 0.15242E+01_JPRB, 0.14759E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.13720E+01_JPRB, 0.12566E+01_JPRB, 0.12220E+01_JPRB, 0.12123E+01_JPRB, 0.12095E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12413E+01_JPRB, 0.12613E+01_JPRB, 0.13151E+01_JPRB, 0.14343E+01_JPRB, 0.17329E+01_JPRB, 0.24571E+01_JPRB, 0.31343E+01_JPRB ,&
&  0.37738E+01_JPRB, 0.67185E+01_JPRB, 0.45220E+01_JPRB, 0.38373E+01_JPRB, 0.35070E+01_JPRB, 0.33547E+01_JPRB, 0.25709E+01_JPRB ,&
&  0.20716E+01_JPRB, 0.19425E+01_JPRB, 0.18945E+01_JPRB, 0.18447E+01_JPRB, 0.18079E+01_JPRB, 0.17308E+01_JPRB, 0.16729E+01_JPRB ,&
&  0.16413E+01_JPRB, 0.16418E+01_JPRB, 0.16551E+01_JPRB, 0.16713E+01_JPRB, 0.16882E+01_JPRB, 0.17605E+01_JPRB, 0.18656E+01_JPRB ,&
&  0.20756E+01_JPRB, 0.23145E+01_JPRB, 0.22065E+01_JPRB, 0.20844E+01_JPRB, 0.20262E+01_JPRB, 0.20718E+01_JPRB, 0.20986E+01_JPRB ,&
&  0.20050E+01_JPRB, 0.18936E+01_JPRB, 0.19792E+01_JPRB, 0.20075E+01_JPRB, 0.20156E+01_JPRB, 0.21667E+01_JPRB, 0.22133E+01_JPRB ,&
&  0.26061E+01_JPRB, 0.28185E+01_JPRB, 0.37555E+01_JPRB, 0.36572E+01_JPRB, 0.34267E+01_JPRB, 0.41566E+01_JPRB, 0.42639E+01_JPRB ,&
&  0.39823E+01_JPRB, 0.45721E+01_JPRB, 0.54646E+01_JPRB, 0.64454E+01_JPRB, 0.67726E+01_JPRB, 0.69455E+01_JPRB, 0.68037E+01_JPRB ,&
&  0.61519E+01_JPRB, 0.64011E+01_JPRB, 0.52737E+01_JPRB, 0.43313E+01_JPRB, 0.43383E+01_JPRB, 0.37457E+01_JPRB, 0.44271E+01_JPRB ,&
&  0.53817E+01_JPRB, 0.54020E+01_JPRB, 0.56770E+01_JPRB, 0.55133E+01_JPRB, 0.34957E+01_JPRB, 0.24545E+01_JPRB, 0.19262E+01_JPRB ,&
&  0.17942E+01_JPRB, 0.18106E+01_JPRB, 0.18343E+01_JPRB, 0.18140E+01_JPRB, 0.18039E+01_JPRB, 0.18064E+01_JPRB, 0.18425E+01_JPRB ,&
&  0.19082E+01_JPRB, 0.19786E+01_JPRB, 0.20124E+01_JPRB, 0.20225E+01_JPRB, 0.20258E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.18729E+01_JPRB, 0.18065E+01_JPRB, 0.17331E+01_JPRB, 0.16679E+01_JPRB, 0.16288E+01_JPRB, 0.15527E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.17389E+01_JPRB, 0.13089E+01_JPRB, 0.12053E+01_JPRB, 0.11816E+01_JPRB, 0.11870E+01_JPRB, 0.12125E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.13998E+01_JPRB, 0.15338E+01_JPRB, 0.18205E+01_JPRB, 0.24053E+01_JPRB, 0.34742E+01_JPRB, 0.45198E+01_JPRB ,&
&  0.57575E+01_JPRB, 0.56479E+01_JPRB, 0.45293E+01_JPRB, 0.34633E+01_JPRB, 0.32392E+01_JPRB, 0.31913E+01_JPRB, 0.30473E+01_JPRB ,&
&  0.23683E+01_JPRB, 0.20844E+01_JPRB, 0.19736E+01_JPRB, 0.18566E+01_JPRB, 0.17123E+01_JPRB, 0.16899E+01_JPRB, 0.16918E+01_JPRB ,&
&  0.17012E+01_JPRB, 0.17111E+01_JPRB, 0.17311E+01_JPRB, 0.17630E+01_JPRB, 0.18418E+01_JPRB, 0.19942E+01_JPRB, 0.21698E+01_JPRB ,&
&  0.23589E+01_JPRB, 0.24297E+01_JPRB, 0.22828E+01_JPRB, 0.21328E+01_JPRB, 0.20634E+01_JPRB, 0.21012E+01_JPRB, 0.21066E+01_JPRB ,&
&  0.20617E+01_JPRB, 0.20531E+01_JPRB, 0.20766E+01_JPRB, 0.20719E+01_JPRB, 0.19966E+01_JPRB, 0.22183E+01_JPRB, 0.23459E+01_JPRB ,&
&  0.27444E+01_JPRB, 0.31988E+01_JPRB, 0.42571E+01_JPRB, 0.35645E+01_JPRB, 0.37848E+01_JPRB, 0.39943E+01_JPRB, 0.41094E+01_JPRB ,&
&  0.41048E+01_JPRB, 0.49835E+01_JPRB, 0.59742E+01_JPRB, 0.65262E+01_JPRB, 0.72200E+01_JPRB, 0.69043E+01_JPRB, 0.71803E+01_JPRB ,&
&  0.52970E+01_JPRB, 0.49571E+01_JPRB, 0.36384E+01_JPRB, 0.38614E+01_JPRB, 0.39987E+01_JPRB, 0.39063E+01_JPRB, 0.42209E+01_JPRB ,&
&  0.49552E+01_JPRB, 0.48124E+01_JPRB, 0.42906E+01_JPRB, 0.34173E+01_JPRB, 0.21498E+01_JPRB, 0.17561E+01_JPRB, 0.16398E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.17874E+01_JPRB, 0.18291E+01_JPRB, 0.18522E+01_JPRB, 0.18483E+01_JPRB, 0.18631E+01_JPRB, 0.19004E+01_JPRB ,&
&  0.19553E+01_JPRB, 0.19887E+01_JPRB, 0.19938E+01_JPRB, 0.19631E+01_JPRB, 0.19280E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.17604E+01_JPRB, 0.17027E+01_JPRB, 0.16496E+01_JPRB, 0.15717E+01_JPRB, 0.14423E+01_JPRB, 0.13486E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.12167E+01_JPRB, 0.12270E+01_JPRB, 0.12407E+01_JPRB, 0.12717E+01_JPRB, 0.13330E+01_JPRB, 0.14207E+01_JPRB ,&
&  0.15252E+01_JPRB, 0.16966E+01_JPRB, 0.20129E+01_JPRB, 0.24878E+01_JPRB, 0.30805E+01_JPRB, 0.39836E+01_JPRB, 0.52664E+01_JPRB ,&
&  0.55870E+01_JPRB, 0.35560E+01_JPRB, 0.36644E+01_JPRB, 0.32535E+01_JPRB, 0.31220E+01_JPRB, 0.28272E+01_JPRB, 0.25960E+01_JPRB ,&
&  0.23851E+01_JPRB, 0.22570E+01_JPRB, 0.21942E+01_JPRB, 0.20448E+01_JPRB, 0.18112E+01_JPRB, 0.18363E+01_JPRB, 0.18658E+01_JPRB ,&
&  0.18866E+01_JPRB, 0.19290E+01_JPRB, 0.19900E+01_JPRB, 0.20889E+01_JPRB, 0.22389E+01_JPRB, 0.23939E+01_JPRB, 0.25149E+01_JPRB ,&
&  0.25664E+01_JPRB, 0.25967E+01_JPRB, 0.25406E+01_JPRB, 0.24298E+01_JPRB, 0.23007E+01_JPRB, 0.22790E+01_JPRB, 0.22188E+01_JPRB ,&
&  0.21731E+01_JPRB, 0.21267E+01_JPRB, 0.21900E+01_JPRB, 0.23289E+01_JPRB, 0.24277E+01_JPRB, 0.23487E+01_JPRB, 0.26655E+01_JPRB ,&
&  0.30304E+01_JPRB, 0.35598E+01_JPRB, 0.38883E+01_JPRB, 0.48306E+01_JPRB, 0.34150E+01_JPRB, 0.36775E+01_JPRB, 0.36506E+01_JPRB ,&
&  0.47945E+01_JPRB, 0.53491E+01_JPRB, 0.59271E+01_JPRB, 0.64239E+01_JPRB, 0.73637E+01_JPRB, 0.67668E+01_JPRB, 0.62097E+01_JPRB ,&
&  0.48379E+01_JPRB, 0.37060E+01_JPRB, 0.30658E+01_JPRB, 0.34015E+01_JPRB, 0.38258E+01_JPRB, 0.43819E+01_JPRB, 0.41091E+01_JPRB ,&
&  0.41201E+01_JPRB, 0.35160E+01_JPRB, 0.28319E+01_JPRB, 0.25227E+01_JPRB, 0.18015E+01_JPRB, 0.16283E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.17051E+01_JPRB, 0.18143E+01_JPRB, 0.18460E+01_JPRB, 0.18446E+01_JPRB, 0.18439E+01_JPRB, 0.18491E+01_JPRB, 0.18659E+01_JPRB ,&
&  0.18816E+01_JPRB, 0.18965E+01_JPRB, 0.18927E+01_JPRB, 0.18556E+01_JPRB, 0.18071E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.16518E+01_JPRB, 0.15871E+01_JPRB, 0.14760E+01_JPRB, 0.13442E+01_JPRB, 0.12479E+01_JPRB, 0.11957E+01_JPRB, 0.11769E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12366E+01_JPRB, 0.12815E+01_JPRB, 0.13410E+01_JPRB, 0.14005E+01_JPRB, 0.14707E+01_JPRB, 0.16023E+01_JPRB ,&
&  0.17559E+01_JPRB, 0.20078E+01_JPRB, 0.23814E+01_JPRB, 0.28058E+01_JPRB, 0.31951E+01_JPRB, 0.35942E+01_JPRB, 0.39893E+01_JPRB ,&
&  0.44222E+01_JPRB, 0.44034E+01_JPRB, 0.37414E+01_JPRB, 0.25697E+01_JPRB, 0.23039E+01_JPRB, 0.24316E+01_JPRB, 0.23459E+01_JPRB ,&
&  0.22911E+01_JPRB, 0.22034E+01_JPRB, 0.21355E+01_JPRB, 0.21027E+01_JPRB, 0.20954E+01_JPRB, 0.21388E+01_JPRB, 0.21811E+01_JPRB ,&
&  0.22487E+01_JPRB, 0.23238E+01_JPRB, 0.24105E+01_JPRB, 0.25098E+01_JPRB, 0.25879E+01_JPRB, 0.26666E+01_JPRB, 0.27545E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.29356E+01_JPRB, 0.30208E+01_JPRB, 0.30144E+01_JPRB, 0.28301E+01_JPRB, 0.27373E+01_JPRB, 0.26124E+01_JPRB ,&
&  0.25541E+01_JPRB, 0.25076E+01_JPRB, 0.25392E+01_JPRB, 0.27129E+01_JPRB, 0.28423E+01_JPRB, 0.26514E+01_JPRB, 0.28020E+01_JPRB ,&
&  0.34010E+01_JPRB, 0.40423E+01_JPRB, 0.37716E+01_JPRB, 0.47183E+01_JPRB, 0.29312E+01_JPRB, 0.35653E+01_JPRB, 0.43533E+01_JPRB ,&
&  0.45610E+01_JPRB, 0.50175E+01_JPRB, 0.56188E+01_JPRB, 0.61008E+01_JPRB, 0.62435E+01_JPRB, 0.57746E+01_JPRB, 0.52823E+01_JPRB ,&
&  0.42800E+01_JPRB, 0.32106E+01_JPRB, 0.27563E+01_JPRB, 0.28630E+01_JPRB, 0.33483E+01_JPRB, 0.39112E+01_JPRB, 0.30743E+01_JPRB ,&
&  0.23460E+01_JPRB, 0.25055E+01_JPRB, 0.23612E+01_JPRB, 0.24253E+01_JPRB, 0.18654E+01_JPRB, 0.15866E+01_JPRB, 0.16722E+01_JPRB ,&
&  0.17776E+01_JPRB, 0.17979E+01_JPRB, 0.18225E+01_JPRB, 0.18048E+01_JPRB, 0.18040E+01_JPRB, 0.17989E+01_JPRB, 0.17927E+01_JPRB ,&
&  0.17974E+01_JPRB, 0.18115E+01_JPRB, 0.18065E+01_JPRB, 0.17566E+01_JPRB, 0.16985E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.14867E+01_JPRB, 0.13603E+01_JPRB, 0.12256E+01_JPRB, 0.11320E+01_JPRB, 0.10904E+01_JPRB, 0.10871E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.12086E+01_JPRB, 0.12784E+01_JPRB, 0.13666E+01_JPRB, 0.14512E+01_JPRB, 0.15471E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.18417E+01_JPRB, 0.20636E+01_JPRB, 0.23573E+01_JPRB, 0.26817E+01_JPRB, 0.29434E+01_JPRB, 0.31880E+01_JPRB, 0.34200E+01_JPRB ,&
&  0.36399E+01_JPRB, 0.37130E+01_JPRB, 0.35836E+01_JPRB, 0.33805E+01_JPRB, 0.27352E+01_JPRB, 0.28757E+01_JPRB, 0.30379E+01_JPRB ,&
&  0.31031E+01_JPRB, 0.25746E+01_JPRB, 0.23686E+01_JPRB, 0.23415E+01_JPRB, 0.23714E+01_JPRB, 0.24364E+01_JPRB, 0.24921E+01_JPRB ,&
&  0.25553E+01_JPRB, 0.26141E+01_JPRB, 0.26697E+01_JPRB, 0.27167E+01_JPRB, 0.27645E+01_JPRB, 0.28608E+01_JPRB, 0.30326E+01_JPRB ,&
&  0.31171E+01_JPRB, 0.32033E+01_JPRB, 0.32673E+01_JPRB, 0.33001E+01_JPRB, 0.34768E+01_JPRB, 0.35968E+01_JPRB, 0.35084E+01_JPRB ,&
&  0.34104E+01_JPRB, 0.31970E+01_JPRB, 0.32896E+01_JPRB, 0.35502E+01_JPRB, 0.35938E+01_JPRB, 0.34119E+01_JPRB, 0.34824E+01_JPRB ,&
&  0.40314E+01_JPRB, 0.46601E+01_JPRB, 0.34102E+01_JPRB, 0.39024E+01_JPRB, 0.46029E+01_JPRB, 0.39682E+01_JPRB, 0.36898E+01_JPRB ,&
&  0.34536E+01_JPRB, 0.37548E+01_JPRB, 0.46042E+01_JPRB, 0.49921E+01_JPRB, 0.49360E+01_JPRB, 0.57086E+01_JPRB, 0.46243E+01_JPRB ,&
&  0.36964E+01_JPRB, 0.27411E+01_JPRB, 0.23926E+01_JPRB, 0.23528E+01_JPRB, 0.25998E+01_JPRB, 0.26736E+01_JPRB, 0.18501E+01_JPRB ,&
&  0.16281E+01_JPRB, 0.20433E+01_JPRB, 0.20667E+01_JPRB, 0.22479E+01_JPRB, 0.21261E+01_JPRB, 0.15088E+01_JPRB, 0.15945E+01_JPRB ,&
&  0.16715E+01_JPRB, 0.17249E+01_JPRB, 0.17351E+01_JPRB, 0.17296E+01_JPRB, 0.17223E+01_JPRB, 0.17164E+01_JPRB, 0.17167E+01_JPRB ,&
&  0.17205E+01_JPRB, 0.17180E+01_JPRB, 0.17023E+01_JPRB, 0.16508E+01_JPRB, 0.15761E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.12011E+01_JPRB, 0.10856E+01_JPRB, 0.10057E+01_JPRB, 0.98151E+00_JPRB, 0.99292E+00_JPRB, 0.10133E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.11096E+01_JPRB, 0.11799E+01_JPRB, 0.12441E+01_JPRB, 0.13219E+01_JPRB, 0.14020E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.16551E+01_JPRB, 0.17959E+01_JPRB, 0.20065E+01_JPRB, 0.23043E+01_JPRB, 0.25342E+01_JPRB, 0.27418E+01_JPRB, 0.30302E+01_JPRB ,&
&  0.33331E+01_JPRB, 0.35349E+01_JPRB, 0.37454E+01_JPRB, 0.38911E+01_JPRB, 0.38246E+01_JPRB, 0.37159E+01_JPRB, 0.38633E+01_JPRB ,&
&  0.32313E+01_JPRB, 0.25465E+01_JPRB, 0.24818E+01_JPRB, 0.24800E+01_JPRB, 0.25598E+01_JPRB, 0.26019E+01_JPRB, 0.26424E+01_JPRB ,&
&  0.26914E+01_JPRB, 0.27248E+01_JPRB, 0.27563E+01_JPRB, 0.27990E+01_JPRB, 0.28805E+01_JPRB, 0.30235E+01_JPRB, 0.32171E+01_JPRB ,&
&  0.33084E+01_JPRB, 0.34210E+01_JPRB, 0.35683E+01_JPRB, 0.33249E+01_JPRB, 0.36459E+01_JPRB, 0.40958E+01_JPRB, 0.43568E+01_JPRB ,&
&  0.43382E+01_JPRB, 0.41004E+01_JPRB, 0.42108E+01_JPRB, 0.43583E+01_JPRB, 0.44321E+01_JPRB, 0.43816E+01_JPRB, 0.43859E+01_JPRB ,&
&  0.46960E+01_JPRB, 0.45533E+01_JPRB, 0.33718E+01_JPRB, 0.31719E+01_JPRB, 0.33227E+01_JPRB, 0.27942E+01_JPRB, 0.28823E+01_JPRB ,&
&  0.25223E+01_JPRB, 0.27110E+01_JPRB, 0.34497E+01_JPRB, 0.40489E+01_JPRB, 0.43599E+01_JPRB, 0.50125E+01_JPRB, 0.42309E+01_JPRB ,&
&  0.28615E+01_JPRB, 0.21171E+01_JPRB, 0.17733E+01_JPRB, 0.16374E+01_JPRB, 0.17644E+01_JPRB, 0.18054E+01_JPRB, 0.16096E+01_JPRB ,&
&  0.16358E+01_JPRB, 0.16702E+01_JPRB, 0.17197E+01_JPRB, 0.16100E+01_JPRB, 0.14207E+01_JPRB, 0.13535E+01_JPRB, 0.14330E+01_JPRB ,&
&  0.14764E+01_JPRB, 0.15174E+01_JPRB, 0.15403E+01_JPRB, 0.15421E+01_JPRB, 0.15490E+01_JPRB, 0.15511E+01_JPRB, 0.15558E+01_JPRB ,&
&  0.15607E+01_JPRB, 0.15480E+01_JPRB, 0.15079E+01_JPRB, 0.14445E+01_JPRB, 0.13364E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.94647E+00_JPRB, 0.87883E+00_JPRB, 0.85938E+00_JPRB, 0.86941E+00_JPRB, 0.88337E+00_JPRB, 0.89828E+00_JPRB, 0.91546E+00_JPRB ,&
&  0.93195E+00_JPRB, 0.94660E+00_JPRB, 0.96929E+00_JPRB, 0.98901E+00_JPRB, 0.10509E+01_JPRB, 0.11248E+01_JPRB, 0.12179E+01_JPRB ,&
&  0.13516E+01_JPRB, 0.14735E+01_JPRB, 0.15808E+01_JPRB, 0.17459E+01_JPRB, 0.18664E+01_JPRB, 0.19955E+01_JPRB, 0.21939E+01_JPRB ,&
&  0.24481E+01_JPRB, 0.27588E+01_JPRB, 0.30613E+01_JPRB, 0.34058E+01_JPRB, 0.38796E+01_JPRB, 0.43384E+01_JPRB, 0.44603E+01_JPRB ,&
&  0.33942E+01_JPRB, 0.23508E+01_JPRB, 0.24941E+01_JPRB, 0.23527E+01_JPRB, 0.24619E+01_JPRB, 0.25560E+01_JPRB, 0.25947E+01_JPRB ,&
&  0.26508E+01_JPRB, 0.26913E+01_JPRB, 0.27289E+01_JPRB, 0.27722E+01_JPRB, 0.28745E+01_JPRB, 0.30076E+01_JPRB, 0.31797E+01_JPRB ,&
&  0.33047E+01_JPRB, 0.34048E+01_JPRB, 0.34405E+01_JPRB, 0.34522E+01_JPRB, 0.37624E+01_JPRB, 0.42200E+01_JPRB, 0.45529E+01_JPRB ,&
&  0.47525E+01_JPRB, 0.49586E+01_JPRB, 0.45823E+01_JPRB, 0.46533E+01_JPRB, 0.46189E+01_JPRB, 0.42123E+01_JPRB, 0.43195E+01_JPRB ,&
&  0.46211E+01_JPRB, 0.43107E+01_JPRB, 0.28480E+01_JPRB, 0.27293E+01_JPRB, 0.27078E+01_JPRB, 0.25150E+01_JPRB, 0.24075E+01_JPRB ,&
&  0.19562E+01_JPRB, 0.20220E+01_JPRB, 0.23563E+01_JPRB, 0.31189E+01_JPRB, 0.35365E+01_JPRB, 0.37498E+01_JPRB, 0.32920E+01_JPRB ,&
&  0.22195E+01_JPRB, 0.17021E+01_JPRB, 0.14435E+01_JPRB, 0.14691E+01_JPRB, 0.17684E+01_JPRB, 0.21131E+01_JPRB, 0.16858E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.20600E+01_JPRB, 0.26024E+01_JPRB, 0.19401E+01_JPRB, 0.12886E+01_JPRB, 0.11775E+01_JPRB, 0.11974E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.12500E+01_JPRB, 0.12653E+01_JPRB, 0.12729E+01_JPRB, 0.12841E+01_JPRB, 0.13069E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.12663E+01_JPRB, 0.12311E+01_JPRB, 0.11341E+01_JPRB, 0.10313E+01_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.77768E+00_JPRB, 0.76918E+00_JPRB, 0.77209E+00_JPRB, 0.78937E+00_JPRB, 0.79176E+00_JPRB, 0.79478E+00_JPRB, 0.83193E+00_JPRB ,&
&  0.82447E+00_JPRB, 0.81878E+00_JPRB, 0.83001E+00_JPRB, 0.81486E+00_JPRB, 0.85585E+00_JPRB, 0.89942E+00_JPRB, 0.96626E+00_JPRB ,&
&  0.10340E+01_JPRB, 0.10804E+01_JPRB, 0.11175E+01_JPRB, 0.11696E+01_JPRB, 0.12690E+01_JPRB, 0.13487E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.15449E+01_JPRB, 0.17314E+01_JPRB, 0.19331E+01_JPRB, 0.23698E+01_JPRB, 0.33626E+01_JPRB, 0.41733E+01_JPRB, 0.41976E+01_JPRB ,&
&  0.31566E+01_JPRB, 0.22693E+01_JPRB, 0.21139E+01_JPRB, 0.19337E+01_JPRB, 0.20396E+01_JPRB, 0.22547E+01_JPRB, 0.23275E+01_JPRB ,&
&  0.23233E+01_JPRB, 0.23563E+01_JPRB, 0.23720E+01_JPRB, 0.23967E+01_JPRB, 0.24324E+01_JPRB, 0.24563E+01_JPRB, 0.24250E+01_JPRB ,&
&  0.23370E+01_JPRB, 0.22250E+01_JPRB, 0.23060E+01_JPRB, 0.27787E+01_JPRB, 0.32948E+01_JPRB, 0.40075E+01_JPRB, 0.39334E+01_JPRB ,&
&  0.37731E+01_JPRB, 0.37562E+01_JPRB, 0.37933E+01_JPRB, 0.37628E+01_JPRB, 0.36453E+01_JPRB, 0.34977E+01_JPRB, 0.35916E+01_JPRB ,&
&  0.38267E+01_JPRB, 0.35878E+01_JPRB, 0.24509E+01_JPRB, 0.24566E+01_JPRB, 0.23631E+01_JPRB, 0.23493E+01_JPRB, 0.20202E+01_JPRB ,&
&  0.16237E+01_JPRB, 0.15643E+01_JPRB, 0.16972E+01_JPRB, 0.20061E+01_JPRB, 0.25430E+01_JPRB, 0.27183E+01_JPRB, 0.23785E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.14482E+01_JPRB, 0.12772E+01_JPRB, 0.13416E+01_JPRB, 0.18124E+01_JPRB, 0.21928E+01_JPRB, 0.23645E+01_JPRB ,&
&  0.16189E+01_JPRB, 0.18399E+01_JPRB, 0.16229E+01_JPRB, 0.19812E+01_JPRB, 0.10594E+01_JPRB, 0.91503E+00_JPRB, 0.90480E+00_JPRB ,&
&  0.92105E+00_JPRB, 0.91522E+00_JPRB, 0.90751E+00_JPRB, 0.88879E+00_JPRB, 0.87192E+00_JPRB, 0.94205E+00_JPRB, 0.10071E+01_JPRB ,&
&  0.95193E+00_JPRB, 0.98293E+00_JPRB, 0.93735E+00_JPRB, 0.86133E+00_JPRB, 0.82258E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.73395E+00_JPRB, 0.74590E+00_JPRB, 0.75150E+00_JPRB, 0.76447E+00_JPRB, 0.78946E+00_JPRB, 0.81096E+00_JPRB, 0.83868E+00_JPRB ,&
&  0.87043E+00_JPRB, 0.89136E+00_JPRB, 0.91377E+00_JPRB, 0.94184E+00_JPRB, 0.96601E+00_JPRB, 0.10064E+01_JPRB, 0.10445E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.11297E+01_JPRB, 0.11689E+01_JPRB, 0.12290E+01_JPRB, 0.13032E+01_JPRB, 0.14130E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.16938E+01_JPRB, 0.18843E+01_JPRB, 0.21131E+01_JPRB, 0.23084E+01_JPRB, 0.24190E+01_JPRB, 0.25066E+01_JPRB, 0.29046E+01_JPRB ,&
&  0.24729E+01_JPRB, 0.23376E+01_JPRB, 0.21895E+01_JPRB, 0.19590E+01_JPRB, 0.19381E+01_JPRB, 0.17874E+01_JPRB, 0.17363E+01_JPRB ,&
&  0.17407E+01_JPRB, 0.16840E+01_JPRB, 0.16505E+01_JPRB, 0.16211E+01_JPRB, 0.15785E+01_JPRB, 0.14904E+01_JPRB, 0.13884E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.14687E+01_JPRB, 0.16867E+01_JPRB, 0.21359E+01_JPRB, 0.25914E+01_JPRB, 0.27766E+01_JPRB, 0.25807E+01_JPRB ,&
&  0.24855E+01_JPRB, 0.25038E+01_JPRB, 0.24405E+01_JPRB, 0.23485E+01_JPRB, 0.24617E+01_JPRB, 0.25538E+01_JPRB, 0.26519E+01_JPRB ,&
&  0.25703E+01_JPRB, 0.23114E+01_JPRB, 0.21678E+01_JPRB, 0.20419E+01_JPRB, 0.20183E+01_JPRB, 0.19718E+01_JPRB, 0.17877E+01_JPRB ,&
&  0.15150E+01_JPRB, 0.13477E+01_JPRB, 0.13001E+01_JPRB, 0.13932E+01_JPRB, 0.17018E+01_JPRB, 0.17490E+01_JPRB, 0.15906E+01_JPRB ,&
&  0.13153E+01_JPRB, 0.11762E+01_JPRB, 0.11544E+01_JPRB, 0.12558E+01_JPRB, 0.16377E+01_JPRB, 0.36606E+01_JPRB, 0.47586E+01_JPRB ,&
&  0.17748E+01_JPRB, 0.14182E+01_JPRB, 0.84484E+00_JPRB, 0.11877E+01_JPRB, 0.79870E+00_JPRB, 0.73811E+00_JPRB, 0.73540E+00_JPRB ,&
&  0.72851E+00_JPRB, 0.72720E+00_JPRB, 0.72790E+00_JPRB, 0.71576E+00_JPRB, 0.71106E+00_JPRB, 0.72552E+00_JPRB, 0.77449E+00_JPRB ,&
&  0.77559E+00_JPRB, 0.78335E+00_JPRB, 0.76090E+00_JPRB, 0.75696E+00_JPRB, 0.74185E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.74024E+00_JPRB, 0.76113E+00_JPRB, 0.78843E+00_JPRB, 0.82228E+00_JPRB, 0.84971E+00_JPRB, 0.88314E+00_JPRB, 0.91347E+00_JPRB ,&
&  0.95038E+00_JPRB, 0.98350E+00_JPRB, 0.10144E+01_JPRB, 0.10481E+01_JPRB, 0.10862E+01_JPRB, 0.11279E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.12897E+01_JPRB, 0.13509E+01_JPRB, 0.14203E+01_JPRB, 0.15017E+01_JPRB, 0.16016E+01_JPRB, 0.16998E+01_JPRB ,&
&  0.17953E+01_JPRB, 0.18981E+01_JPRB, 0.20136E+01_JPRB, 0.21065E+01_JPRB, 0.21504E+01_JPRB, 0.22720E+01_JPRB, 0.21219E+01_JPRB ,&
&  0.24480E+01_JPRB, 0.22074E+01_JPRB, 0.20461E+01_JPRB, 0.18577E+01_JPRB, 0.16889E+01_JPRB, 0.15365E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.13735E+01_JPRB, 0.13472E+01_JPRB, 0.13418E+01_JPRB, 0.12162E+01_JPRB, 0.11967E+01_JPRB, 0.12694E+01_JPRB ,&
&  0.13997E+01_JPRB, 0.15926E+01_JPRB, 0.17228E+01_JPRB, 0.18772E+01_JPRB, 0.20673E+01_JPRB, 0.19999E+01_JPRB, 0.18847E+01_JPRB ,&
&  0.17595E+01_JPRB, 0.16866E+01_JPRB, 0.13840E+01_JPRB, 0.15680E+01_JPRB, 0.16384E+01_JPRB, 0.17120E+01_JPRB, 0.18304E+01_JPRB ,&
&  0.17198E+01_JPRB, 0.14310E+01_JPRB, 0.14923E+01_JPRB, 0.17769E+01_JPRB, 0.17380E+01_JPRB, 0.16472E+01_JPRB, 0.15651E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.13219E+01_JPRB, 0.12204E+01_JPRB, 0.11807E+01_JPRB, 0.12598E+01_JPRB, 0.12543E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.10450E+01_JPRB, 0.10845E+01_JPRB, 0.11862E+01_JPRB, 0.15673E+01_JPRB, 0.20324E+01_JPRB, 0.33123E+01_JPRB ,&
&  0.18636E+01_JPRB, 0.97320E+00_JPRB, 0.10118E+01_JPRB, 0.94195E+00_JPRB, 0.79840E+00_JPRB, 0.70303E+00_JPRB, 0.71931E+00_JPRB ,&
&  0.74551E+00_JPRB, 0.75646E+00_JPRB, 0.71379E+00_JPRB, 0.71421E+00_JPRB, 0.71545E+00_JPRB, 0.71863E+00_JPRB, 0.72798E+00_JPRB ,&
&  0.72962E+00_JPRB, 0.73097E+00_JPRB, 0.73855E+00_JPRB, 0.73334E+00_JPRB, 0.73199E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.65099E+00_JPRB, 0.66493E+00_JPRB, 0.68573E+00_JPRB, 0.70485E+00_JPRB, 0.71878E+00_JPRB, 0.73898E+00_JPRB, 0.75611E+00_JPRB ,&
&  0.77541E+00_JPRB, 0.78793E+00_JPRB, 0.80499E+00_JPRB, 0.82181E+00_JPRB, 0.83996E+00_JPRB, 0.86378E+00_JPRB, 0.88961E+00_JPRB ,&
&  0.91384E+00_JPRB, 0.95213E+00_JPRB, 0.99148E+00_JPRB, 0.10231E+01_JPRB, 0.10874E+01_JPRB, 0.12045E+01_JPRB, 0.13423E+01_JPRB ,&
&  0.14649E+01_JPRB, 0.16326E+01_JPRB, 0.17925E+01_JPRB, 0.18980E+01_JPRB, 0.19467E+01_JPRB, 0.21242E+01_JPRB, 0.18460E+01_JPRB ,&
&  0.21976E+01_JPRB, 0.21072E+01_JPRB, 0.19178E+01_JPRB, 0.18086E+01_JPRB, 0.17032E+01_JPRB, 0.16075E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.16315E+01_JPRB, 0.19674E+01_JPRB, 0.19008E+01_JPRB, 0.19397E+01_JPRB, 0.16338E+01_JPRB, 0.15670E+01_JPRB, 0.17369E+01_JPRB ,&
&  0.19430E+01_JPRB, 0.20268E+01_JPRB, 0.20587E+01_JPRB, 0.21677E+01_JPRB, 0.22885E+01_JPRB, 0.23478E+01_JPRB, 0.23674E+01_JPRB ,&
&  0.22811E+01_JPRB, 0.22008E+01_JPRB, 0.20695E+01_JPRB, 0.24342E+01_JPRB, 0.21682E+01_JPRB, 0.19035E+01_JPRB, 0.17436E+01_JPRB ,&
&  0.13608E+01_JPRB, 0.12642E+01_JPRB, 0.13341E+01_JPRB, 0.14185E+01_JPRB, 0.14104E+01_JPRB, 0.14166E+01_JPRB, 0.13959E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.13001E+01_JPRB, 0.12225E+01_JPRB, 0.11969E+01_JPRB, 0.12037E+01_JPRB, 0.11511E+01_JPRB, 0.10427E+01_JPRB ,&
&  0.98916E+00_JPRB, 0.10220E+01_JPRB, 0.10365E+01_JPRB, 0.10914E+01_JPRB, 0.12954E+01_JPRB, 0.13933E+01_JPRB, 0.21472E+01_JPRB ,&
&  0.14976E+01_JPRB, 0.12885E+01_JPRB, 0.16969E+01_JPRB, 0.60605E+00_JPRB, 0.68567E+00_JPRB, 0.77757E+00_JPRB, 0.97096E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.12750E+01_JPRB, 0.15976E+01_JPRB, 0.12724E+01_JPRB, 0.99290E+00_JPRB, 0.70929E+00_JPRB, 0.68023E+00_JPRB ,&
&  0.67350E+00_JPRB, 0.66402E+00_JPRB, 0.66549E+00_JPRB, 0.65671E+00_JPRB, 0.65218E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.55574E+00_JPRB, 0.56266E+00_JPRB, 0.57675E+00_JPRB, 0.58735E+00_JPRB, 0.59785E+00_JPRB, 0.60777E+00_JPRB, 0.61582E+00_JPRB ,&
&  0.63499E+00_JPRB, 0.64867E+00_JPRB, 0.68181E+00_JPRB, 0.71452E+00_JPRB, 0.74633E+00_JPRB, 0.80303E+00_JPRB, 0.85813E+00_JPRB ,&
&  0.93888E+00_JPRB, 0.10422E+01_JPRB, 0.11032E+01_JPRB, 0.11719E+01_JPRB, 0.12183E+01_JPRB, 0.12767E+01_JPRB, 0.13315E+01_JPRB ,&
&  0.14912E+01_JPRB, 0.17930E+01_JPRB, 0.20133E+01_JPRB, 0.22473E+01_JPRB, 0.22850E+01_JPRB, 0.25130E+01_JPRB, 0.18909E+01_JPRB ,&
&  0.22490E+01_JPRB, 0.22876E+01_JPRB, 0.21648E+01_JPRB, 0.21094E+01_JPRB, 0.20872E+01_JPRB, 0.20838E+01_JPRB, 0.20889E+01_JPRB ,&
&  0.22003E+01_JPRB, 0.23331E+01_JPRB, 0.22451E+01_JPRB, 0.22736E+01_JPRB, 0.22920E+01_JPRB, 0.22387E+01_JPRB, 0.25023E+01_JPRB ,&
&  0.27404E+01_JPRB, 0.29326E+01_JPRB, 0.31676E+01_JPRB, 0.34535E+01_JPRB, 0.36527E+01_JPRB, 0.37752E+01_JPRB, 0.38239E+01_JPRB ,&
&  0.38441E+01_JPRB, 0.39633E+01_JPRB, 0.42916E+01_JPRB, 0.42726E+01_JPRB, 0.33967E+01_JPRB, 0.27765E+01_JPRB, 0.21330E+01_JPRB ,&
&  0.14457E+01_JPRB, 0.11758E+01_JPRB, 0.11487E+01_JPRB, 0.11808E+01_JPRB, 0.11622E+01_JPRB, 0.11625E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11934E+01_JPRB, 0.11926E+01_JPRB, 0.11878E+01_JPRB, 0.11498E+01_JPRB, 0.10474E+01_JPRB, 0.94119E+00_JPRB ,&
&  0.92957E+00_JPRB, 0.93747E+00_JPRB, 0.96765E+00_JPRB, 0.10446E+01_JPRB, 0.12068E+01_JPRB, 0.15126E+01_JPRB, 0.15292E+01_JPRB ,&
&  0.15971E+01_JPRB, 0.10138E+01_JPRB, 0.92533E+00_JPRB, 0.94595E+00_JPRB, 0.72587E+00_JPRB, 0.97653E+00_JPRB, 0.13952E+01_JPRB ,&
&  0.20265E+01_JPRB, 0.22360E+01_JPRB, 0.28868E+01_JPRB, 0.41926E+01_JPRB, 0.26532E+01_JPRB, 0.17322E+01_JPRB, 0.62827E+00_JPRB ,&
&  0.61949E+00_JPRB, 0.59615E+00_JPRB, 0.58866E+00_JPRB, 0.57176E+00_JPRB, 0.56293E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.45350E+00_JPRB, 0.44753E+00_JPRB, 0.46133E+00_JPRB, 0.48181E+00_JPRB, 0.49824E+00_JPRB, 0.53464E+00_JPRB, 0.57935E+00_JPRB ,&
&  0.62949E+00_JPRB, 0.69574E+00_JPRB, 0.76156E+00_JPRB, 0.82140E+00_JPRB, 0.91424E+00_JPRB, 0.10349E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.12445E+01_JPRB, 0.13498E+01_JPRB, 0.14323E+01_JPRB, 0.14990E+01_JPRB, 0.15822E+01_JPRB, 0.16891E+01_JPRB, 0.18338E+01_JPRB ,&
&  0.19856E+01_JPRB, 0.21537E+01_JPRB, 0.23714E+01_JPRB, 0.25183E+01_JPRB, 0.26015E+01_JPRB, 0.26378E+01_JPRB, 0.20275E+01_JPRB ,&
&  0.24262E+01_JPRB, 0.26323E+01_JPRB, 0.26835E+01_JPRB, 0.26192E+01_JPRB, 0.25306E+01_JPRB, 0.24876E+01_JPRB, 0.25384E+01_JPRB ,&
&  0.26178E+01_JPRB, 0.26063E+01_JPRB, 0.25071E+01_JPRB, 0.24702E+01_JPRB, 0.30249E+01_JPRB, 0.30957E+01_JPRB, 0.34006E+01_JPRB ,&
&  0.36824E+01_JPRB, 0.39243E+01_JPRB, 0.42388E+01_JPRB, 0.45682E+01_JPRB, 0.49106E+01_JPRB, 0.52022E+01_JPRB, 0.54640E+01_JPRB ,&
&  0.57121E+01_JPRB, 0.59829E+01_JPRB, 0.62384E+01_JPRB, 0.53734E+01_JPRB, 0.42650E+01_JPRB, 0.34867E+01_JPRB, 0.21756E+01_JPRB ,&
&  0.13812E+01_JPRB, 0.11814E+01_JPRB, 0.11731E+01_JPRB, 0.11201E+01_JPRB, 0.10200E+01_JPRB, 0.98230E+00_JPRB, 0.96029E+00_JPRB ,&
&  0.98594E+00_JPRB, 0.10426E+01_JPRB, 0.10765E+01_JPRB, 0.10768E+01_JPRB, 0.10227E+01_JPRB, 0.93090E+00_JPRB, 0.85148E+00_JPRB ,&
&  0.87912E+00_JPRB, 0.89555E+00_JPRB, 0.92078E+00_JPRB, 0.10439E+01_JPRB, 0.12402E+01_JPRB, 0.16481E+01_JPRB, 0.21001E+01_JPRB ,&
&  0.18639E+01_JPRB, 0.10108E+01_JPRB, 0.85912E+00_JPRB, 0.80061E+00_JPRB, 0.75806E+00_JPRB, 0.82294E+00_JPRB, 0.98442E+00_JPRB ,&
&  0.13699E+01_JPRB, 0.21950E+01_JPRB, 0.23975E+01_JPRB, 0.15772E+01_JPRB, 0.20678E+01_JPRB, 0.13794E+01_JPRB, 0.60525E+00_JPRB ,&
&  0.58037E+00_JPRB, 0.55860E+00_JPRB, 0.53006E+00_JPRB, 0.49680E+00_JPRB, 0.47126E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.45276E+00_JPRB, 0.43350E+00_JPRB, 0.44682E+00_JPRB, 0.47390E+00_JPRB, 0.50541E+00_JPRB, 0.55583E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.68659E+00_JPRB, 0.75779E+00_JPRB, 0.87570E+00_JPRB, 0.92572E+00_JPRB, 0.98417E+00_JPRB, 0.10768E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.13942E+01_JPRB, 0.15097E+01_JPRB, 0.16235E+01_JPRB, 0.17569E+01_JPRB, 0.18970E+01_JPRB, 0.20208E+01_JPRB ,&
&  0.21843E+01_JPRB, 0.23500E+01_JPRB, 0.25665E+01_JPRB, 0.27952E+01_JPRB, 0.30062E+01_JPRB, 0.30550E+01_JPRB, 0.31916E+01_JPRB ,&
&  0.20452E+01_JPRB, 0.28545E+01_JPRB, 0.29056E+01_JPRB, 0.28661E+01_JPRB, 0.27447E+01_JPRB, 0.26443E+01_JPRB, 0.26834E+01_JPRB ,&
&  0.27465E+01_JPRB, 0.26361E+01_JPRB, 0.27265E+01_JPRB, 0.28063E+01_JPRB, 0.34408E+01_JPRB, 0.36152E+01_JPRB, 0.38542E+01_JPRB ,&
&  0.41063E+01_JPRB, 0.43424E+01_JPRB, 0.46132E+01_JPRB, 0.49399E+01_JPRB, 0.52706E+01_JPRB, 0.55813E+01_JPRB, 0.58945E+01_JPRB ,&
&  0.61798E+01_JPRB, 0.64475E+01_JPRB, 0.67301E+01_JPRB, 0.55852E+01_JPRB, 0.46290E+01_JPRB, 0.37455E+01_JPRB, 0.24422E+01_JPRB ,&
&  0.15433E+01_JPRB, 0.11756E+01_JPRB, 0.11719E+01_JPRB, 0.11705E+01_JPRB, 0.10251E+01_JPRB, 0.94863E+00_JPRB, 0.85459E+00_JPRB ,&
&  0.85307E+00_JPRB, 0.88194E+00_JPRB, 0.89916E+00_JPRB, 0.90139E+00_JPRB, 0.90312E+00_JPRB, 0.82949E+00_JPRB, 0.78769E+00_JPRB ,&
&  0.81866E+00_JPRB, 0.82659E+00_JPRB, 0.85894E+00_JPRB, 0.92493E+00_JPRB, 0.10282E+01_JPRB, 0.10989E+01_JPRB, 0.10938E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.94301E+00_JPRB, 0.87931E+00_JPRB, 0.88185E+00_JPRB, 0.83251E+00_JPRB, 0.78463E+00_JPRB, 0.79400E+00_JPRB ,&
&  0.87528E+00_JPRB, 0.98749E+00_JPRB, 0.10765E+01_JPRB, 0.10471E+01_JPRB, 0.10566E+01_JPRB, 0.85094E+00_JPRB, 0.65842E+00_JPRB ,&
&  0.61861E+00_JPRB, 0.60940E+00_JPRB, 0.56137E+00_JPRB, 0.52606E+00_JPRB, 0.48677E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.56137E+00_JPRB, 0.49358E+00_JPRB, 0.48993E+00_JPRB, 0.53639E+00_JPRB, 0.58316E+00_JPRB, 0.63368E+00_JPRB, 0.68536E+00_JPRB ,&
&  0.74628E+00_JPRB, 0.78739E+00_JPRB, 0.85974E+00_JPRB, 0.90954E+00_JPRB, 0.95646E+00_JPRB, 0.10029E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.11637E+01_JPRB, 0.12814E+01_JPRB, 0.13993E+01_JPRB, 0.15480E+01_JPRB, 0.17081E+01_JPRB, 0.18572E+01_JPRB, 0.20214E+01_JPRB ,&
&  0.21862E+01_JPRB, 0.23703E+01_JPRB, 0.25619E+01_JPRB, 0.27813E+01_JPRB, 0.29859E+01_JPRB, 0.30671E+01_JPRB, 0.30276E+01_JPRB ,&
&  0.14900E+01_JPRB, 0.23864E+01_JPRB, 0.30209E+01_JPRB, 0.29165E+01_JPRB, 0.28086E+01_JPRB, 0.26552E+01_JPRB, 0.26341E+01_JPRB ,&
&  0.24893E+01_JPRB, 0.22768E+01_JPRB, 0.24148E+01_JPRB, 0.31837E+01_JPRB, 0.35563E+01_JPRB, 0.37837E+01_JPRB, 0.39270E+01_JPRB ,&
&  0.41041E+01_JPRB, 0.42397E+01_JPRB, 0.44013E+01_JPRB, 0.45763E+01_JPRB, 0.47724E+01_JPRB, 0.49608E+01_JPRB, 0.51327E+01_JPRB ,&
&  0.52815E+01_JPRB, 0.53688E+01_JPRB, 0.53428E+01_JPRB, 0.42254E+01_JPRB, 0.41469E+01_JPRB, 0.35119E+01_JPRB, 0.29794E+01_JPRB ,&
&  0.18288E+01_JPRB, 0.12635E+01_JPRB, 0.12162E+01_JPRB, 0.12126E+01_JPRB, 0.10580E+01_JPRB, 0.96051E+00_JPRB, 0.83648E+00_JPRB ,&
&  0.82200E+00_JPRB, 0.81725E+00_JPRB, 0.83011E+00_JPRB, 0.82827E+00_JPRB, 0.82940E+00_JPRB, 0.76360E+00_JPRB, 0.77363E+00_JPRB ,&
&  0.80190E+00_JPRB, 0.81421E+00_JPRB, 0.84488E+00_JPRB, 0.89400E+00_JPRB, 0.95709E+00_JPRB, 0.10112E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.99243E+00_JPRB, 0.97595E+00_JPRB, 0.96221E+00_JPRB, 0.94294E+00_JPRB, 0.92108E+00_JPRB, 0.89399E+00_JPRB ,&
&  0.85973E+00_JPRB, 0.85904E+00_JPRB, 0.88120E+00_JPRB, 0.80550E+00_JPRB, 0.84318E+00_JPRB, 0.76897E+00_JPRB, 0.75453E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.72010E+00_JPRB, 0.67364E+00_JPRB, 0.64890E+00_JPRB, 0.61046E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.69113E+00_JPRB, 0.60347E+00_JPRB, 0.53814E+00_JPRB, 0.55378E+00_JPRB, 0.62448E+00_JPRB, 0.68478E+00_JPRB, 0.72146E+00_JPRB ,&
&  0.76702E+00_JPRB, 0.78757E+00_JPRB, 0.80383E+00_JPRB, 0.82440E+00_JPRB, 0.86764E+00_JPRB, 0.90552E+00_JPRB, 0.97644E+00_JPRB ,&
&  0.10444E+01_JPRB, 0.11133E+01_JPRB, 0.11876E+01_JPRB, 0.13051E+01_JPRB, 0.14556E+01_JPRB, 0.16063E+01_JPRB, 0.17511E+01_JPRB ,&
&  0.19218E+01_JPRB, 0.21346E+01_JPRB, 0.24199E+01_JPRB, 0.27579E+01_JPRB, 0.32286E+01_JPRB, 0.37470E+01_JPRB, 0.41405E+01_JPRB ,&
&  0.28627E+01_JPRB, 0.12267E+01_JPRB, 0.25793E+01_JPRB, 0.30198E+01_JPRB, 0.27894E+01_JPRB, 0.26629E+01_JPRB, 0.24790E+01_JPRB ,&
&  0.23147E+01_JPRB, 0.21622E+01_JPRB, 0.21456E+01_JPRB, 0.26982E+01_JPRB, 0.33902E+01_JPRB, 0.36428E+01_JPRB, 0.37377E+01_JPRB ,&
&  0.38279E+01_JPRB, 0.38652E+01_JPRB, 0.39264E+01_JPRB, 0.40175E+01_JPRB, 0.41308E+01_JPRB, 0.42028E+01_JPRB, 0.42528E+01_JPRB ,&
&  0.42866E+01_JPRB, 0.42663E+01_JPRB, 0.41067E+01_JPRB, 0.29809E+01_JPRB, 0.26859E+01_JPRB, 0.23375E+01_JPRB, 0.21214E+01_JPRB ,&
&  0.18573E+01_JPRB, 0.14351E+01_JPRB, 0.13007E+01_JPRB, 0.12538E+01_JPRB, 0.11182E+01_JPRB, 0.96040E+00_JPRB, 0.87668E+00_JPRB ,&
&  0.85284E+00_JPRB, 0.82572E+00_JPRB, 0.80248E+00_JPRB, 0.77995E+00_JPRB, 0.81460E+00_JPRB, 0.81326E+00_JPRB, 0.80509E+00_JPRB ,&
&  0.80383E+00_JPRB, 0.85498E+00_JPRB, 0.90778E+00_JPRB, 0.95374E+00_JPRB, 0.10015E+01_JPRB, 0.10553E+01_JPRB, 0.10665E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10767E+01_JPRB, 0.10638E+01_JPRB, 0.10348E+01_JPRB, 0.10100E+01_JPRB, 0.96982E+00_JPRB, 0.96125E+00_JPRB ,&
&  0.95523E+00_JPRB, 0.95477E+00_JPRB, 0.93364E+00_JPRB, 0.90721E+00_JPRB, 0.92083E+00_JPRB, 0.90252E+00_JPRB, 0.90236E+00_JPRB ,&
&  0.89314E+00_JPRB, 0.87033E+00_JPRB, 0.81468E+00_JPRB, 0.76673E+00_JPRB, 0.74272E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.77879E+00_JPRB, 0.71383E+00_JPRB, 0.63642E+00_JPRB, 0.58150E+00_JPRB, 0.59908E+00_JPRB, 0.67138E+00_JPRB, 0.73215E+00_JPRB ,&
&  0.75763E+00_JPRB, 0.76316E+00_JPRB, 0.75809E+00_JPRB, 0.76254E+00_JPRB, 0.78592E+00_JPRB, 0.79603E+00_JPRB, 0.83521E+00_JPRB ,&
&  0.89684E+00_JPRB, 0.95130E+00_JPRB, 0.99378E+00_JPRB, 0.10655E+01_JPRB, 0.11479E+01_JPRB, 0.12828E+01_JPRB, 0.14083E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.16321E+01_JPRB, 0.18174E+01_JPRB, 0.20785E+01_JPRB, 0.24487E+01_JPRB, 0.30964E+01_JPRB, 0.42191E+01_JPRB ,&
&  0.62492E+01_JPRB, 0.41552E+01_JPRB, 0.16769E+01_JPRB, 0.28976E+01_JPRB, 0.30258E+01_JPRB, 0.28577E+01_JPRB, 0.24411E+01_JPRB ,&
&  0.24104E+01_JPRB, 0.21214E+01_JPRB, 0.19228E+01_JPRB, 0.21319E+01_JPRB, 0.25130E+01_JPRB, 0.29992E+01_JPRB, 0.32924E+01_JPRB ,&
&  0.33688E+01_JPRB, 0.34072E+01_JPRB, 0.34391E+01_JPRB, 0.35110E+01_JPRB, 0.36187E+01_JPRB, 0.37158E+01_JPRB, 0.37370E+01_JPRB ,&
&  0.37642E+01_JPRB, 0.38248E+01_JPRB, 0.37297E+01_JPRB, 0.27812E+01_JPRB, 0.25730E+01_JPRB, 0.24712E+01_JPRB, 0.23377E+01_JPRB ,&
&  0.20411E+01_JPRB, 0.18185E+01_JPRB, 0.16285E+01_JPRB, 0.13778E+01_JPRB, 0.11893E+01_JPRB, 0.10810E+01_JPRB, 0.10385E+01_JPRB ,&
&  0.98449E+00_JPRB, 0.91004E+00_JPRB, 0.87821E+00_JPRB, 0.84778E+00_JPRB, 0.86439E+00_JPRB, 0.90273E+00_JPRB, 0.91546E+00_JPRB ,&
&  0.94677E+00_JPRB, 0.99132E+00_JPRB, 0.10220E+01_JPRB, 0.10501E+01_JPRB, 0.10886E+01_JPRB, 0.11224E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.11633E+01_JPRB, 0.11467E+01_JPRB, 0.11219E+01_JPRB, 0.10718E+01_JPRB, 0.10275E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.10252E+01_JPRB, 0.10237E+01_JPRB, 0.99953E+00_JPRB, 0.10019E+01_JPRB, 0.98914E+00_JPRB, 0.10016E+01_JPRB, 0.10254E+01_JPRB ,&
&  0.10448E+01_JPRB, 0.10161E+01_JPRB, 0.92592E+00_JPRB, 0.83232E+00_JPRB, 0.80493E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.82574E+00_JPRB, 0.80245E+00_JPRB, 0.75275E+00_JPRB, 0.65873E+00_JPRB, 0.61571E+00_JPRB, 0.62765E+00_JPRB, 0.69692E+00_JPRB ,&
&  0.73005E+00_JPRB, 0.74358E+00_JPRB, 0.73127E+00_JPRB, 0.71940E+00_JPRB, 0.71788E+00_JPRB, 0.73005E+00_JPRB, 0.76400E+00_JPRB ,&
&  0.80898E+00_JPRB, 0.81732E+00_JPRB, 0.81084E+00_JPRB, 0.82122E+00_JPRB, 0.87504E+00_JPRB, 0.95237E+00_JPRB, 0.10834E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.12570E+01_JPRB, 0.13321E+01_JPRB, 0.15000E+01_JPRB, 0.17032E+01_JPRB, 0.18150E+01_JPRB, 0.21370E+01_JPRB ,&
&  0.34848E+01_JPRB, 0.54442E+01_JPRB, 0.20060E+01_JPRB, 0.30251E+01_JPRB, 0.34884E+01_JPRB, 0.31242E+01_JPRB, 0.28320E+01_JPRB ,&
&  0.27308E+01_JPRB, 0.22342E+01_JPRB, 0.17554E+01_JPRB, 0.17271E+01_JPRB, 0.19090E+01_JPRB, 0.20865E+01_JPRB, 0.23702E+01_JPRB ,&
&  0.26316E+01_JPRB, 0.28008E+01_JPRB, 0.29375E+01_JPRB, 0.30512E+01_JPRB, 0.31980E+01_JPRB, 0.33928E+01_JPRB, 0.35654E+01_JPRB ,&
&  0.36916E+01_JPRB, 0.38734E+01_JPRB, 0.39923E+01_JPRB, 0.31834E+01_JPRB, 0.30400E+01_JPRB, 0.32429E+01_JPRB, 0.30575E+01_JPRB ,&
&  0.26370E+01_JPRB, 0.23281E+01_JPRB, 0.19704E+01_JPRB, 0.15948E+01_JPRB, 0.13376E+01_JPRB, 0.12334E+01_JPRB, 0.11782E+01_JPRB ,&
&  0.10946E+01_JPRB, 0.10165E+01_JPRB, 0.99905E+00_JPRB, 0.99163E+00_JPRB, 0.99304E+00_JPRB, 0.10073E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10955E+01_JPRB, 0.11428E+01_JPRB, 0.11738E+01_JPRB, 0.11881E+01_JPRB, 0.11998E+01_JPRB, 0.12132E+01_JPRB, 0.12026E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.12022E+01_JPRB, 0.11885E+01_JPRB, 0.11649E+01_JPRB, 0.11351E+01_JPRB, 0.11179E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.10805E+01_JPRB, 0.10300E+01_JPRB, 0.10055E+01_JPRB, 0.10291E+01_JPRB, 0.99881E+00_JPRB, 0.10433E+01_JPRB ,&
&  0.11095E+01_JPRB, 0.11375E+01_JPRB, 0.10689E+01_JPRB, 0.86597E+00_JPRB, 0.83682E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.86826E+00_JPRB, 0.85479E+00_JPRB, 0.83465E+00_JPRB, 0.76719E+00_JPRB, 0.69743E+00_JPRB, 0.66074E+00_JPRB, 0.67399E+00_JPRB ,&
&  0.71427E+00_JPRB, 0.74312E+00_JPRB, 0.74682E+00_JPRB, 0.73911E+00_JPRB, 0.74113E+00_JPRB, 0.75721E+00_JPRB, 0.77991E+00_JPRB ,&
&  0.79714E+00_JPRB, 0.79514E+00_JPRB, 0.77679E+00_JPRB, 0.77758E+00_JPRB, 0.78426E+00_JPRB, 0.80467E+00_JPRB, 0.86959E+00_JPRB ,&
&  0.96098E+00_JPRB, 0.10155E+01_JPRB, 0.10344E+01_JPRB, 0.10930E+01_JPRB, 0.12493E+01_JPRB, 0.13128E+01_JPRB, 0.14022E+01_JPRB ,&
&  0.19246E+01_JPRB, 0.41275E+01_JPRB, 0.43524E+01_JPRB, 0.49345E+01_JPRB, 0.43637E+01_JPRB, 0.36149E+01_JPRB, 0.34421E+01_JPRB ,&
&  0.34555E+01_JPRB, 0.27996E+01_JPRB, 0.21966E+01_JPRB, 0.17286E+01_JPRB, 0.16903E+01_JPRB, 0.17667E+01_JPRB, 0.17900E+01_JPRB ,&
&  0.19058E+01_JPRB, 0.21759E+01_JPRB, 0.23463E+01_JPRB, 0.25097E+01_JPRB, 0.26659E+01_JPRB, 0.28448E+01_JPRB, 0.31016E+01_JPRB ,&
&  0.34045E+01_JPRB, 0.37894E+01_JPRB, 0.42623E+01_JPRB, 0.39891E+01_JPRB, 0.35069E+01_JPRB, 0.40249E+01_JPRB, 0.41714E+01_JPRB ,&
&  0.39636E+01_JPRB, 0.30829E+01_JPRB, 0.24025E+01_JPRB, 0.18976E+01_JPRB, 0.15493E+01_JPRB, 0.15529E+01_JPRB, 0.13553E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.11451E+01_JPRB, 0.11175E+01_JPRB, 0.11230E+01_JPRB, 0.11319E+01_JPRB, 0.11215E+01_JPRB, 0.11420E+01_JPRB ,&
&  0.11840E+01_JPRB, 0.12225E+01_JPRB, 0.12437E+01_JPRB, 0.12589E+01_JPRB, 0.12760E+01_JPRB, 0.12873E+01_JPRB, 0.12679E+01_JPRB ,&
&  0.12360E+01_JPRB, 0.12173E+01_JPRB, 0.11538E+01_JPRB, 0.11490E+01_JPRB, 0.11488E+01_JPRB, 0.11533E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.11151E+01_JPRB, 0.10780E+01_JPRB, 0.10501E+01_JPRB, 0.10552E+01_JPRB, 0.10358E+01_JPRB, 0.99835E+00_JPRB ,&
&  0.10032E+01_JPRB, 0.98149E+00_JPRB, 0.94136E+00_JPRB, 0.90785E+00_JPRB, 0.88419E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.92573E+00_JPRB, 0.91213E+00_JPRB, 0.90314E+00_JPRB, 0.86804E+00_JPRB, 0.82228E+00_JPRB, 0.77892E+00_JPRB, 0.75342E+00_JPRB ,&
&  0.75333E+00_JPRB, 0.76769E+00_JPRB, 0.78125E+00_JPRB, 0.77266E+00_JPRB, 0.77164E+00_JPRB, 0.79110E+00_JPRB, 0.80584E+00_JPRB ,&
&  0.81837E+00_JPRB, 0.81971E+00_JPRB, 0.81263E+00_JPRB, 0.81117E+00_JPRB, 0.80627E+00_JPRB, 0.79162E+00_JPRB, 0.79323E+00_JPRB ,&
&  0.83014E+00_JPRB, 0.86804E+00_JPRB, 0.88640E+00_JPRB, 0.90050E+00_JPRB, 0.98747E+00_JPRB, 0.10604E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.35072E+01_JPRB, 0.29617E+01_JPRB, 0.57158E+01_JPRB, 0.48264E+01_JPRB, 0.40188E+01_JPRB, 0.34110E+01_JPRB ,&
&  0.32792E+01_JPRB, 0.33756E+01_JPRB, 0.26173E+01_JPRB, 0.20910E+01_JPRB, 0.17927E+01_JPRB, 0.17085E+01_JPRB, 0.16361E+01_JPRB ,&
&  0.15749E+01_JPRB, 0.16171E+01_JPRB, 0.17146E+01_JPRB, 0.18644E+01_JPRB, 0.20529E+01_JPRB, 0.22203E+01_JPRB, 0.24210E+01_JPRB ,&
&  0.26584E+01_JPRB, 0.30496E+01_JPRB, 0.36720E+01_JPRB, 0.37929E+01_JPRB, 0.38697E+01_JPRB, 0.45094E+01_JPRB, 0.51861E+01_JPRB ,&
&  0.51238E+01_JPRB, 0.41450E+01_JPRB, 0.29773E+01_JPRB, 0.22243E+01_JPRB, 0.18580E+01_JPRB, 0.17511E+01_JPRB, 0.15360E+01_JPRB ,&
&  0.13605E+01_JPRB, 0.12716E+01_JPRB, 0.12375E+01_JPRB, 0.12238E+01_JPRB, 0.12310E+01_JPRB, 0.12322E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.12522E+01_JPRB, 0.12868E+01_JPRB, 0.12893E+01_JPRB, 0.12827E+01_JPRB, 0.12843E+01_JPRB, 0.12873E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.12676E+01_JPRB, 0.12536E+01_JPRB, 0.11743E+01_JPRB, 0.11384E+01_JPRB, 0.11418E+01_JPRB, 0.11413E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.11572E+01_JPRB, 0.11559E+01_JPRB, 0.11171E+01_JPRB, 0.10634E+01_JPRB, 0.10660E+01_JPRB, 0.10764E+01_JPRB, 0.10059E+01_JPRB ,&
&  0.98681E+00_JPRB, 0.97239E+00_JPRB, 0.96433E+00_JPRB, 0.95595E+00_JPRB, 0.94073E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.94099E+00_JPRB, 0.92709E+00_JPRB, 0.92236E+00_JPRB, 0.90549E+00_JPRB, 0.87410E+00_JPRB, 0.84022E+00_JPRB, 0.81815E+00_JPRB ,&
&  0.80188E+00_JPRB, 0.80119E+00_JPRB, 0.81156E+00_JPRB, 0.80378E+00_JPRB, 0.80831E+00_JPRB, 0.82338E+00_JPRB, 0.83319E+00_JPRB ,&
&  0.83217E+00_JPRB, 0.83064E+00_JPRB, 0.82146E+00_JPRB, 0.81626E+00_JPRB, 0.81291E+00_JPRB, 0.79884E+00_JPRB, 0.78995E+00_JPRB ,&
&  0.79358E+00_JPRB, 0.81044E+00_JPRB, 0.82599E+00_JPRB, 0.84114E+00_JPRB, 0.88618E+00_JPRB, 0.93721E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.12771E+01_JPRB, 0.39398E+01_JPRB, 0.27796E+01_JPRB, 0.53498E+01_JPRB, 0.46577E+01_JPRB, 0.37939E+01_JPRB, 0.31217E+01_JPRB ,&
&  0.30386E+01_JPRB, 0.29949E+01_JPRB, 0.26370E+01_JPRB, 0.23611E+01_JPRB, 0.21059E+01_JPRB, 0.18572E+01_JPRB, 0.16763E+01_JPRB ,&
&  0.15262E+01_JPRB, 0.14486E+01_JPRB, 0.14391E+01_JPRB, 0.14769E+01_JPRB, 0.15812E+01_JPRB, 0.17138E+01_JPRB, 0.18445E+01_JPRB ,&
&  0.19892E+01_JPRB, 0.22183E+01_JPRB, 0.26544E+01_JPRB, 0.33692E+01_JPRB, 0.35668E+01_JPRB, 0.41078E+01_JPRB, 0.60580E+01_JPRB ,&
&  0.62522E+01_JPRB, 0.54079E+01_JPRB, 0.37473E+01_JPRB, 0.25779E+01_JPRB, 0.21086E+01_JPRB, 0.19162E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.15500E+01_JPRB, 0.14082E+01_JPRB, 0.13588E+01_JPRB, 0.13008E+01_JPRB, 0.12704E+01_JPRB, 0.12942E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.13534E+01_JPRB, 0.13382E+01_JPRB, 0.13342E+01_JPRB, 0.12998E+01_JPRB, 0.12712E+01_JPRB, 0.12458E+01_JPRB, 0.12406E+01_JPRB ,&
&  0.12502E+01_JPRB, 0.12600E+01_JPRB, 0.12113E+01_JPRB, 0.11607E+01_JPRB, 0.11512E+01_JPRB, 0.11559E+01_JPRB, 0.11476E+01_JPRB ,&
&  0.11879E+01_JPRB, 0.11797E+01_JPRB, 0.11391E+01_JPRB, 0.11058E+01_JPRB, 0.10742E+01_JPRB, 0.10876E+01_JPRB, 0.99878E+00_JPRB ,&
&  0.96848E+00_JPRB, 0.95659E+00_JPRB, 0.95675E+00_JPRB, 0.96359E+00_JPRB, 0.96043E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.95368E+00_JPRB, 0.93532E+00_JPRB, 0.93048E+00_JPRB, 0.92414E+00_JPRB, 0.90838E+00_JPRB, 0.88883E+00_JPRB, 0.88076E+00_JPRB ,&
&  0.87142E+00_JPRB, 0.86396E+00_JPRB, 0.85238E+00_JPRB, 0.83457E+00_JPRB, 0.83376E+00_JPRB, 0.83533E+00_JPRB, 0.83527E+00_JPRB ,&
&  0.83266E+00_JPRB, 0.83077E+00_JPRB, 0.81750E+00_JPRB, 0.81082E+00_JPRB, 0.81209E+00_JPRB, 0.80672E+00_JPRB, 0.79397E+00_JPRB ,&
&  0.78212E+00_JPRB, 0.78202E+00_JPRB, 0.79636E+00_JPRB, 0.81331E+00_JPRB, 0.84710E+00_JPRB, 0.87062E+00_JPRB, 0.93241E+00_JPRB ,&
&  0.12631E+01_JPRB, 0.13034E+01_JPRB, 0.33893E+01_JPRB, 0.46055E+01_JPRB, 0.44568E+01_JPRB, 0.37646E+01_JPRB, 0.32870E+01_JPRB ,&
&  0.30439E+01_JPRB, 0.27930E+01_JPRB, 0.25979E+01_JPRB, 0.23494E+01_JPRB, 0.20788E+01_JPRB, 0.17841E+01_JPRB, 0.16186E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.14301E+01_JPRB, 0.13952E+01_JPRB, 0.13985E+01_JPRB, 0.14232E+01_JPRB, 0.14574E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.15910E+01_JPRB, 0.16796E+01_JPRB, 0.18655E+01_JPRB, 0.24044E+01_JPRB, 0.32414E+01_JPRB, 0.39271E+01_JPRB, 0.53870E+01_JPRB ,&
&  0.67192E+01_JPRB, 0.62431E+01_JPRB, 0.44326E+01_JPRB, 0.30016E+01_JPRB, 0.23545E+01_JPRB, 0.20905E+01_JPRB, 0.18891E+01_JPRB ,&
&  0.17278E+01_JPRB, 0.15603E+01_JPRB, 0.14451E+01_JPRB, 0.13611E+01_JPRB, 0.13040E+01_JPRB, 0.12842E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.13975E+01_JPRB, 0.13595E+01_JPRB, 0.13158E+01_JPRB, 0.12797E+01_JPRB, 0.12482E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.12604E+01_JPRB, 0.12324E+01_JPRB, 0.11925E+01_JPRB, 0.11819E+01_JPRB, 0.11960E+01_JPRB, 0.12055E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.11807E+01_JPRB, 0.11348E+01_JPRB, 0.11143E+01_JPRB, 0.10566E+01_JPRB, 0.10719E+01_JPRB, 0.99479E+00_JPRB ,&
&  0.95756E+00_JPRB, 0.94839E+00_JPRB, 0.95646E+00_JPRB, 0.96411E+00_JPRB, 0.97808E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.96489E+00_JPRB, 0.93650E+00_JPRB, 0.92726E+00_JPRB, 0.93091E+00_JPRB, 0.92756E+00_JPRB, 0.93105E+00_JPRB, 0.93146E+00_JPRB ,&
&  0.92005E+00_JPRB, 0.89498E+00_JPRB, 0.87476E+00_JPRB, 0.85430E+00_JPRB, 0.84727E+00_JPRB, 0.84038E+00_JPRB, 0.83972E+00_JPRB ,&
&  0.83153E+00_JPRB, 0.82523E+00_JPRB, 0.81412E+00_JPRB, 0.81090E+00_JPRB, 0.81044E+00_JPRB, 0.80343E+00_JPRB, 0.77922E+00_JPRB ,&
&  0.76983E+00_JPRB, 0.77129E+00_JPRB, 0.78780E+00_JPRB, 0.80774E+00_JPRB, 0.83195E+00_JPRB, 0.84433E+00_JPRB, 0.88045E+00_JPRB ,&
&  0.11931E+01_JPRB, 0.22349E+01_JPRB, 0.35509E+01_JPRB, 0.37332E+01_JPRB, 0.36981E+01_JPRB, 0.33284E+01_JPRB, 0.30063E+01_JPRB ,&
&  0.27007E+01_JPRB, 0.25677E+01_JPRB, 0.25456E+01_JPRB, 0.22893E+01_JPRB, 0.18991E+01_JPRB, 0.15900E+01_JPRB, 0.14652E+01_JPRB ,&
&  0.13958E+01_JPRB, 0.13414E+01_JPRB, 0.13149E+01_JPRB, 0.13151E+01_JPRB, 0.13195E+01_JPRB, 0.13272E+01_JPRB, 0.13487E+01_JPRB ,&
&  0.13731E+01_JPRB, 0.14078E+01_JPRB, 0.14723E+01_JPRB, 0.17349E+01_JPRB, 0.27153E+01_JPRB, 0.30660E+01_JPRB, 0.46646E+01_JPRB ,&
&  0.67677E+01_JPRB, 0.60444E+01_JPRB, 0.46596E+01_JPRB, 0.34171E+01_JPRB, 0.27086E+01_JPRB, 0.23542E+01_JPRB, 0.20848E+01_JPRB ,&
&  0.18754E+01_JPRB, 0.17002E+01_JPRB, 0.15499E+01_JPRB, 0.14527E+01_JPRB, 0.13687E+01_JPRB, 0.13099E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.13737E+01_JPRB, 0.13509E+01_JPRB, 0.13118E+01_JPRB, 0.12787E+01_JPRB, 0.12685E+01_JPRB, 0.12608E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.12630E+01_JPRB, 0.12446E+01_JPRB, 0.11916E+01_JPRB, 0.11820E+01_JPRB, 0.12045E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.11688E+01_JPRB, 0.11324E+01_JPRB, 0.11341E+01_JPRB, 0.11004E+01_JPRB, 0.10873E+01_JPRB, 0.10910E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.95176E+00_JPRB, 0.93833E+00_JPRB, 0.95687E+00_JPRB, 0.98001E+00_JPRB, 0.10116E+01_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.99008E+00_JPRB, 0.94450E+00_JPRB, 0.92017E+00_JPRB, 0.92105E+00_JPRB, 0.92940E+00_JPRB, 0.92824E+00_JPRB, 0.93230E+00_JPRB ,&
&  0.92160E+00_JPRB, 0.89993E+00_JPRB, 0.87611E+00_JPRB, 0.85616E+00_JPRB, 0.84567E+00_JPRB, 0.83867E+00_JPRB, 0.83330E+00_JPRB ,&
&  0.82792E+00_JPRB, 0.82153E+00_JPRB, 0.81202E+00_JPRB, 0.81416E+00_JPRB, 0.81609E+00_JPRB, 0.81314E+00_JPRB, 0.78715E+00_JPRB ,&
&  0.77164E+00_JPRB, 0.77393E+00_JPRB, 0.78673E+00_JPRB, 0.79798E+00_JPRB, 0.81413E+00_JPRB, 0.82951E+00_JPRB, 0.85844E+00_JPRB ,&
&  0.10329E+01_JPRB, 0.19486E+01_JPRB, 0.26960E+01_JPRB, 0.31783E+01_JPRB, 0.33200E+01_JPRB, 0.31504E+01_JPRB, 0.28220E+01_JPRB ,&
&  0.25721E+01_JPRB, 0.24309E+01_JPRB, 0.23007E+01_JPRB, 0.19829E+01_JPRB, 0.16439E+01_JPRB, 0.14552E+01_JPRB, 0.13562E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.12915E+01_JPRB, 0.12725E+01_JPRB, 0.12582E+01_JPRB, 0.12451E+01_JPRB, 0.12469E+01_JPRB, 0.12573E+01_JPRB ,&
&  0.12695E+01_JPRB, 0.12794E+01_JPRB, 0.13026E+01_JPRB, 0.13415E+01_JPRB, 0.17820E+01_JPRB, 0.24561E+01_JPRB, 0.35192E+01_JPRB ,&
&  0.40391E+01_JPRB, 0.41231E+01_JPRB, 0.37592E+01_JPRB, 0.32774E+01_JPRB, 0.28530E+01_JPRB, 0.25116E+01_JPRB, 0.22168E+01_JPRB ,&
&  0.19576E+01_JPRB, 0.17307E+01_JPRB, 0.15455E+01_JPRB, 0.14255E+01_JPRB, 0.13501E+01_JPRB, 0.12886E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.12886E+01_JPRB, 0.12917E+01_JPRB, 0.12844E+01_JPRB, 0.12617E+01_JPRB, 0.12386E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.12184E+01_JPRB, 0.12228E+01_JPRB, 0.11965E+01_JPRB, 0.11682E+01_JPRB, 0.11503E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11426E+01_JPRB, 0.11193E+01_JPRB, 0.10978E+01_JPRB, 0.10903E+01_JPRB, 0.10745E+01_JPRB, 0.10829E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.95524E+00_JPRB, 0.93931E+00_JPRB, 0.94054E+00_JPRB, 0.95784E+00_JPRB, 0.99798E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.10046E+01_JPRB, 0.97151E+00_JPRB, 0.93916E+00_JPRB, 0.92145E+00_JPRB, 0.91574E+00_JPRB, 0.91414E+00_JPRB, 0.91312E+00_JPRB ,&
&  0.90861E+00_JPRB, 0.89893E+00_JPRB, 0.87512E+00_JPRB, 0.85597E+00_JPRB, 0.84198E+00_JPRB, 0.83620E+00_JPRB, 0.83118E+00_JPRB ,&
&  0.82779E+00_JPRB, 0.82344E+00_JPRB, 0.81572E+00_JPRB, 0.81356E+00_JPRB, 0.81484E+00_JPRB, 0.80850E+00_JPRB, 0.78774E+00_JPRB ,&
&  0.76528E+00_JPRB, 0.76510E+00_JPRB, 0.77580E+00_JPRB, 0.78701E+00_JPRB, 0.80289E+00_JPRB, 0.81635E+00_JPRB, 0.84462E+00_JPRB ,&
&  0.99593E+00_JPRB, 0.15094E+01_JPRB, 0.22457E+01_JPRB, 0.26530E+01_JPRB, 0.28188E+01_JPRB, 0.28355E+01_JPRB, 0.26349E+01_JPRB ,&
&  0.23770E+01_JPRB, 0.21148E+01_JPRB, 0.19017E+01_JPRB, 0.16775E+01_JPRB, 0.15157E+01_JPRB, 0.13917E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.12605E+01_JPRB, 0.12441E+01_JPRB, 0.12272E+01_JPRB, 0.12068E+01_JPRB, 0.11909E+01_JPRB, 0.11813E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.11975E+01_JPRB, 0.12170E+01_JPRB, 0.12341E+01_JPRB, 0.12920E+01_JPRB, 0.15380E+01_JPRB, 0.21018E+01_JPRB ,&
&  0.26130E+01_JPRB, 0.29062E+01_JPRB, 0.28613E+01_JPRB, 0.27541E+01_JPRB, 0.26511E+01_JPRB, 0.24329E+01_JPRB, 0.22032E+01_JPRB ,&
&  0.19764E+01_JPRB, 0.17679E+01_JPRB, 0.15561E+01_JPRB, 0.13860E+01_JPRB, 0.12764E+01_JPRB, 0.12127E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.11984E+01_JPRB, 0.12082E+01_JPRB, 0.12183E+01_JPRB, 0.12099E+01_JPRB, 0.11878E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.11352E+01_JPRB, 0.11103E+01_JPRB, 0.11026E+01_JPRB, 0.10889E+01_JPRB, 0.10718E+01_JPRB, 0.10646E+01_JPRB, 0.10609E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.10696E+01_JPRB, 0.10343E+01_JPRB, 0.10457E+01_JPRB, 0.11109E+01_JPRB, 0.10624E+01_JPRB, 0.10099E+01_JPRB ,&
&  0.96821E+00_JPRB, 0.94576E+00_JPRB, 0.93774E+00_JPRB, 0.95160E+00_JPRB, 0.99329E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.90237E+00_JPRB, 0.89970E+00_JPRB, 0.89674E+00_JPRB, 0.89919E+00_JPRB, 0.90028E+00_JPRB, 0.89186E+00_JPRB, 0.88471E+00_JPRB ,&
&  0.88245E+00_JPRB, 0.87689E+00_JPRB, 0.86861E+00_JPRB, 0.85593E+00_JPRB, 0.83978E+00_JPRB, 0.82700E+00_JPRB, 0.82193E+00_JPRB ,&
&  0.81482E+00_JPRB, 0.81118E+00_JPRB, 0.80928E+00_JPRB, 0.80359E+00_JPRB, 0.79699E+00_JPRB, 0.78699E+00_JPRB, 0.76936E+00_JPRB ,&
&  0.74594E+00_JPRB, 0.73716E+00_JPRB, 0.74773E+00_JPRB, 0.75983E+00_JPRB, 0.77050E+00_JPRB, 0.77685E+00_JPRB, 0.80103E+00_JPRB ,&
&  0.95285E+00_JPRB, 0.12754E+01_JPRB, 0.16063E+01_JPRB, 0.19896E+01_JPRB, 0.22139E+01_JPRB, 0.21554E+01_JPRB, 0.20935E+01_JPRB ,&
&  0.19699E+01_JPRB, 0.18385E+01_JPRB, 0.17093E+01_JPRB, 0.15689E+01_JPRB, 0.14466E+01_JPRB, 0.13485E+01_JPRB, 0.12858E+01_JPRB ,&
&  0.12412E+01_JPRB, 0.12034E+01_JPRB, 0.11731E+01_JPRB, 0.11414E+01_JPRB, 0.11175E+01_JPRB, 0.11011E+01_JPRB, 0.10982E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.11116E+01_JPRB, 0.11264E+01_JPRB, 0.11417E+01_JPRB, 0.11644E+01_JPRB, 0.12350E+01_JPRB, 0.14792E+01_JPRB ,&
&  0.18053E+01_JPRB, 0.21135E+01_JPRB, 0.22549E+01_JPRB, 0.22175E+01_JPRB, 0.21629E+01_JPRB, 0.20866E+01_JPRB, 0.19172E+01_JPRB ,&
&  0.17111E+01_JPRB, 0.15718E+01_JPRB, 0.14637E+01_JPRB, 0.13691E+01_JPRB, 0.12980E+01_JPRB, 0.12395E+01_JPRB, 0.11945E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.11593E+01_JPRB, 0.11550E+01_JPRB, 0.11502E+01_JPRB, 0.11374E+01_JPRB, 0.11073E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.98986E+00_JPRB, 0.98700E+00_JPRB, 0.97618E+00_JPRB, 0.96560E+00_JPRB, 0.96782E+00_JPRB, 0.98006E+00_JPRB ,&
&  0.98510E+00_JPRB, 0.99632E+00_JPRB, 0.10033E+01_JPRB, 0.10430E+01_JPRB, 0.10429E+01_JPRB, 0.10185E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.97495E+00_JPRB, 0.94390E+00_JPRB, 0.91960E+00_JPRB, 0.91403E+00_JPRB, 0.86514E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.88372E+00_JPRB, 0.87910E+00_JPRB, 0.87198E+00_JPRB, 0.87186E+00_JPRB, 0.87458E+00_JPRB, 0.86195E+00_JPRB, 0.84942E+00_JPRB ,&
&  0.84225E+00_JPRB, 0.84207E+00_JPRB, 0.84399E+00_JPRB, 0.84291E+00_JPRB, 0.83377E+00_JPRB, 0.81838E+00_JPRB, 0.80812E+00_JPRB ,&
&  0.79563E+00_JPRB, 0.78549E+00_JPRB, 0.78220E+00_JPRB, 0.78024E+00_JPRB, 0.77308E+00_JPRB, 0.76481E+00_JPRB, 0.75741E+00_JPRB ,&
&  0.74086E+00_JPRB, 0.72831E+00_JPRB, 0.72563E+00_JPRB, 0.73059E+00_JPRB, 0.73738E+00_JPRB, 0.73977E+00_JPRB, 0.77424E+00_JPRB ,&
&  0.94222E+00_JPRB, 0.10367E+01_JPRB, 0.10714E+01_JPRB, 0.13463E+01_JPRB, 0.15851E+01_JPRB, 0.16222E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.17000E+01_JPRB, 0.16910E+01_JPRB, 0.16295E+01_JPRB, 0.15028E+01_JPRB, 0.13929E+01_JPRB, 0.13031E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.10797E+01_JPRB, 0.10365E+01_JPRB, 0.10038E+01_JPRB, 0.98621E+00_JPRB, 0.96539E+00_JPRB, 0.95560E+00_JPRB ,&
&  0.95722E+00_JPRB, 0.96996E+00_JPRB, 0.98661E+00_JPRB, 0.10026E+01_JPRB, 0.10285E+01_JPRB, 0.10713E+01_JPRB, 0.11739E+01_JPRB ,&
&  0.13536E+01_JPRB, 0.15991E+01_JPRB, 0.17730E+01_JPRB, 0.18231E+01_JPRB, 0.17894E+01_JPRB, 0.17087E+01_JPRB, 0.15926E+01_JPRB ,&
&  0.14661E+01_JPRB, 0.13609E+01_JPRB, 0.12839E+01_JPRB, 0.12400E+01_JPRB, 0.12066E+01_JPRB, 0.11790E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.11279E+01_JPRB, 0.11112E+01_JPRB, 0.10986E+01_JPRB, 0.10826E+01_JPRB, 0.10611E+01_JPRB, 0.10210E+01_JPRB, 0.96550E+00_JPRB ,&
&  0.92162E+00_JPRB, 0.89327E+00_JPRB, 0.88388E+00_JPRB, 0.87689E+00_JPRB, 0.87405E+00_JPRB, 0.88740E+00_JPRB, 0.90631E+00_JPRB ,&
&  0.91775E+00_JPRB, 0.93129E+00_JPRB, 0.95268E+00_JPRB, 0.94480E+00_JPRB, 0.98669E+00_JPRB, 0.98398E+00_JPRB, 0.98409E+00_JPRB ,&
&  0.97210E+00_JPRB, 0.95114E+00_JPRB, 0.92774E+00_JPRB, 0.80529E+00_JPRB, 0.86699E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.86819E+00_JPRB, 0.86266E+00_JPRB, 0.84789E+00_JPRB, 0.83810E+00_JPRB, 0.83145E+00_JPRB, 0.82306E+00_JPRB, 0.80948E+00_JPRB ,&
&  0.80123E+00_JPRB, 0.79905E+00_JPRB, 0.79781E+00_JPRB, 0.79773E+00_JPRB, 0.79639E+00_JPRB, 0.78823E+00_JPRB, 0.77639E+00_JPRB ,&
&  0.76507E+00_JPRB, 0.75576E+00_JPRB, 0.75685E+00_JPRB, 0.75520E+00_JPRB, 0.74857E+00_JPRB, 0.74304E+00_JPRB, 0.73637E+00_JPRB ,&
&  0.72077E+00_JPRB, 0.70366E+00_JPRB, 0.69060E+00_JPRB, 0.69409E+00_JPRB, 0.70165E+00_JPRB, 0.69856E+00_JPRB, 0.72910E+00_JPRB ,&
&  0.88444E+00_JPRB, 0.87604E+00_JPRB, 0.92323E+00_JPRB, 0.10806E+01_JPRB, 0.11841E+01_JPRB, 0.12782E+01_JPRB, 0.13474E+01_JPRB ,&
&  0.14048E+01_JPRB, 0.14454E+01_JPRB, 0.13592E+01_JPRB, 0.12483E+01_JPRB, 0.11374E+01_JPRB, 0.10414E+01_JPRB, 0.97105E+00_JPRB ,&
&  0.92277E+00_JPRB, 0.89424E+00_JPRB, 0.87433E+00_JPRB, 0.85332E+00_JPRB, 0.83681E+00_JPRB, 0.82258E+00_JPRB, 0.81490E+00_JPRB ,&
&  0.81212E+00_JPRB, 0.81670E+00_JPRB, 0.82839E+00_JPRB, 0.85227E+00_JPRB, 0.87893E+00_JPRB, 0.90903E+00_JPRB, 0.95352E+00_JPRB ,&
&  0.10329E+01_JPRB, 0.11688E+01_JPRB, 0.13148E+01_JPRB, 0.13913E+01_JPRB, 0.13902E+01_JPRB, 0.13397E+01_JPRB, 0.12618E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11230E+01_JPRB, 0.10834E+01_JPRB, 0.10556E+01_JPRB, 0.10297E+01_JPRB, 0.10081E+01_JPRB, 0.99095E+00_JPRB ,&
&  0.97715E+00_JPRB, 0.96125E+00_JPRB, 0.95138E+00_JPRB, 0.93725E+00_JPRB, 0.91457E+00_JPRB, 0.88280E+00_JPRB, 0.85138E+00_JPRB ,&
&  0.82208E+00_JPRB, 0.79922E+00_JPRB, 0.78676E+00_JPRB, 0.78673E+00_JPRB, 0.79088E+00_JPRB, 0.80523E+00_JPRB, 0.82697E+00_JPRB ,&
&  0.84438E+00_JPRB, 0.85867E+00_JPRB, 0.87162E+00_JPRB, 0.90377E+00_JPRB, 0.92454E+00_JPRB, 0.94231E+00_JPRB, 0.95256E+00_JPRB ,&
&  0.95426E+00_JPRB, 0.95309E+00_JPRB, 0.77593E+00_JPRB, 0.84485E+00_JPRB, 0.86079E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.86305E+00_JPRB, 0.84203E+00_JPRB, 0.81713E+00_JPRB, 0.79884E+00_JPRB, 0.78847E+00_JPRB, 0.78335E+00_JPRB, 0.77728E+00_JPRB ,&
&  0.76958E+00_JPRB, 0.76844E+00_JPRB, 0.76907E+00_JPRB, 0.76942E+00_JPRB, 0.76876E+00_JPRB, 0.76345E+00_JPRB, 0.75296E+00_JPRB ,&
&  0.74131E+00_JPRB, 0.72805E+00_JPRB, 0.72557E+00_JPRB, 0.72426E+00_JPRB, 0.71722E+00_JPRB, 0.71197E+00_JPRB, 0.71066E+00_JPRB ,&
&  0.70355E+00_JPRB, 0.69214E+00_JPRB, 0.67214E+00_JPRB, 0.66633E+00_JPRB, 0.66986E+00_JPRB, 0.67080E+00_JPRB, 0.68455E+00_JPRB ,&
&  0.71543E+00_JPRB, 0.74281E+00_JPRB, 0.82686E+00_JPRB, 0.88953E+00_JPRB, 0.93299E+00_JPRB, 0.97723E+00_JPRB, 0.10243E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.10192E+01_JPRB, 0.97638E+00_JPRB, 0.90982E+00_JPRB, 0.86044E+00_JPRB, 0.82415E+00_JPRB, 0.79851E+00_JPRB ,&
&  0.78643E+00_JPRB, 0.77710E+00_JPRB, 0.76728E+00_JPRB, 0.75821E+00_JPRB, 0.75279E+00_JPRB, 0.74472E+00_JPRB, 0.74018E+00_JPRB ,&
&  0.74127E+00_JPRB, 0.74629E+00_JPRB, 0.75009E+00_JPRB, 0.75773E+00_JPRB, 0.77259E+00_JPRB, 0.79528E+00_JPRB, 0.82335E+00_JPRB ,&
&  0.85582E+00_JPRB, 0.90441E+00_JPRB, 0.98018E+00_JPRB, 0.10422E+01_JPRB, 0.10605E+01_JPRB, 0.10362E+01_JPRB, 0.99257E+00_JPRB ,&
&  0.95358E+00_JPRB, 0.92684E+00_JPRB, 0.90704E+00_JPRB, 0.88949E+00_JPRB, 0.86826E+00_JPRB, 0.84894E+00_JPRB, 0.83156E+00_JPRB ,&
&  0.82285E+00_JPRB, 0.81115E+00_JPRB, 0.79880E+00_JPRB, 0.78441E+00_JPRB, 0.77129E+00_JPRB, 0.76292E+00_JPRB, 0.75732E+00_JPRB ,&
&  0.75126E+00_JPRB, 0.74163E+00_JPRB, 0.73576E+00_JPRB, 0.73782E+00_JPRB, 0.74103E+00_JPRB, 0.74898E+00_JPRB, 0.76226E+00_JPRB ,&
&  0.77910E+00_JPRB, 0.79514E+00_JPRB, 0.81123E+00_JPRB, 0.84249E+00_JPRB, 0.87406E+00_JPRB, 0.90088E+00_JPRB, 0.91720E+00_JPRB ,&
&  0.92727E+00_JPRB, 0.92089E+00_JPRB, 0.91528E+00_JPRB, 0.89950E+00_JPRB, 0.87996E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.79614E+00_JPRB, 0.77123E+00_JPRB, 0.75090E+00_JPRB, 0.73677E+00_JPRB, 0.72810E+00_JPRB, 0.72534E+00_JPRB, 0.72126E+00_JPRB ,&
&  0.71793E+00_JPRB, 0.71324E+00_JPRB, 0.71043E+00_JPRB, 0.71301E+00_JPRB, 0.71431E+00_JPRB, 0.71449E+00_JPRB, 0.71458E+00_JPRB ,&
&  0.70740E+00_JPRB, 0.69271E+00_JPRB, 0.68113E+00_JPRB, 0.67739E+00_JPRB, 0.67748E+00_JPRB, 0.67265E+00_JPRB, 0.67064E+00_JPRB ,&
&  0.67361E+00_JPRB, 0.66532E+00_JPRB, 0.65118E+00_JPRB, 0.64196E+00_JPRB, 0.64286E+00_JPRB, 0.64082E+00_JPRB, 0.64252E+00_JPRB ,&
&  0.69420E+00_JPRB, 0.65744E+00_JPRB, 0.72465E+00_JPRB, 0.73953E+00_JPRB, 0.75458E+00_JPRB, 0.76711E+00_JPRB, 0.77437E+00_JPRB ,&
&  0.77198E+00_JPRB, 0.76173E+00_JPRB, 0.74922E+00_JPRB, 0.73380E+00_JPRB, 0.71919E+00_JPRB, 0.70435E+00_JPRB, 0.69271E+00_JPRB ,&
&  0.68718E+00_JPRB, 0.68123E+00_JPRB, 0.67971E+00_JPRB, 0.68163E+00_JPRB, 0.68757E+00_JPRB, 0.68911E+00_JPRB, 0.69067E+00_JPRB ,&
&  0.69312E+00_JPRB, 0.69481E+00_JPRB, 0.69475E+00_JPRB, 0.69401E+00_JPRB, 0.69304E+00_JPRB, 0.69830E+00_JPRB, 0.70757E+00_JPRB ,&
&  0.71952E+00_JPRB, 0.74193E+00_JPRB, 0.77600E+00_JPRB, 0.80660E+00_JPRB, 0.81914E+00_JPRB, 0.81310E+00_JPRB, 0.79602E+00_JPRB ,&
&  0.78571E+00_JPRB, 0.78442E+00_JPRB, 0.78464E+00_JPRB, 0.77768E+00_JPRB, 0.76714E+00_JPRB, 0.75708E+00_JPRB, 0.74209E+00_JPRB ,&
&  0.72333E+00_JPRB, 0.70722E+00_JPRB, 0.69614E+00_JPRB, 0.68939E+00_JPRB, 0.68639E+00_JPRB, 0.68276E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.67864E+00_JPRB, 0.67721E+00_JPRB, 0.67816E+00_JPRB, 0.68366E+00_JPRB, 0.68884E+00_JPRB, 0.69558E+00_JPRB, 0.70418E+00_JPRB ,&
&  0.71611E+00_JPRB, 0.72955E+00_JPRB, 0.74244E+00_JPRB, 0.76341E+00_JPRB, 0.79344E+00_JPRB, 0.81577E+00_JPRB, 0.82991E+00_JPRB ,&
&  0.84260E+00_JPRB, 0.84646E+00_JPRB, 0.84049E+00_JPRB, 0.83046E+00_JPRB, 0.81511E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.73771E+00_JPRB, 0.72270E+00_JPRB, 0.71078E+00_JPRB, 0.70137E+00_JPRB, 0.69482E+00_JPRB, 0.69266E+00_JPRB, 0.68980E+00_JPRB ,&
&  0.68686E+00_JPRB, 0.68391E+00_JPRB, 0.67605E+00_JPRB, 0.66891E+00_JPRB, 0.66868E+00_JPRB, 0.67499E+00_JPRB, 0.67962E+00_JPRB ,&
&  0.67594E+00_JPRB, 0.66406E+00_JPRB, 0.65291E+00_JPRB, 0.64997E+00_JPRB, 0.65215E+00_JPRB, 0.65401E+00_JPRB, 0.65322E+00_JPRB ,&
&  0.65477E+00_JPRB, 0.65601E+00_JPRB, 0.64765E+00_JPRB, 0.62975E+00_JPRB, 0.62179E+00_JPRB, 0.61757E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.64899E+00_JPRB, 0.67069E+00_JPRB, 0.67853E+00_JPRB, 0.67298E+00_JPRB, 0.67263E+00_JPRB, 0.67105E+00_JPRB, 0.66570E+00_JPRB ,&
&  0.66014E+00_JPRB, 0.65291E+00_JPRB, 0.64659E+00_JPRB, 0.64173E+00_JPRB, 0.63845E+00_JPRB, 0.63635E+00_JPRB, 0.63639E+00_JPRB ,&
&  0.63279E+00_JPRB, 0.62747E+00_JPRB, 0.62815E+00_JPRB, 0.63431E+00_JPRB, 0.64295E+00_JPRB, 0.64834E+00_JPRB, 0.65338E+00_JPRB ,&
&  0.65759E+00_JPRB, 0.65984E+00_JPRB, 0.66268E+00_JPRB, 0.66310E+00_JPRB, 0.65897E+00_JPRB, 0.65816E+00_JPRB, 0.65824E+00_JPRB ,&
&  0.66138E+00_JPRB, 0.66896E+00_JPRB, 0.68479E+00_JPRB, 0.70413E+00_JPRB, 0.71228E+00_JPRB, 0.70660E+00_JPRB, 0.69815E+00_JPRB ,&
&  0.69703E+00_JPRB, 0.70314E+00_JPRB, 0.71018E+00_JPRB, 0.71426E+00_JPRB, 0.71339E+00_JPRB, 0.70444E+00_JPRB, 0.69280E+00_JPRB ,&
&  0.68012E+00_JPRB, 0.66727E+00_JPRB, 0.65570E+00_JPRB, 0.64799E+00_JPRB, 0.64173E+00_JPRB, 0.63695E+00_JPRB, 0.63402E+00_JPRB ,&
&  0.63295E+00_JPRB, 0.63324E+00_JPRB, 0.63715E+00_JPRB, 0.64476E+00_JPRB, 0.65009E+00_JPRB, 0.65985E+00_JPRB, 0.67137E+00_JPRB ,&
&  0.68293E+00_JPRB, 0.69391E+00_JPRB, 0.70687E+00_JPRB, 0.72506E+00_JPRB, 0.74595E+00_JPRB, 0.76206E+00_JPRB, 0.77460E+00_JPRB ,&
&  0.78258E+00_JPRB, 0.78165E+00_JPRB, 0.77328E+00_JPRB, 0.76334E+00_JPRB, 0.75197E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.68468E+00_JPRB, 0.67638E+00_JPRB, 0.66947E+00_JPRB, 0.66332E+00_JPRB, 0.65797E+00_JPRB, 0.65570E+00_JPRB, 0.65360E+00_JPRB ,&
&  0.65168E+00_JPRB, 0.64868E+00_JPRB, 0.64639E+00_JPRB, 0.64186E+00_JPRB, 0.63850E+00_JPRB, 0.64164E+00_JPRB, 0.64764E+00_JPRB ,&
&  0.64614E+00_JPRB, 0.63861E+00_JPRB, 0.63261E+00_JPRB, 0.63215E+00_JPRB, 0.63522E+00_JPRB, 0.63738E+00_JPRB, 0.63974E+00_JPRB ,&
&  0.63910E+00_JPRB, 0.63801E+00_JPRB, 0.63557E+00_JPRB, 0.61870E+00_JPRB, 0.60335E+00_JPRB, 0.59591E+00_JPRB, 0.59266E+00_JPRB ,&
&  0.59717E+00_JPRB, 0.61896E+00_JPRB, 0.59745E+00_JPRB, 0.60743E+00_JPRB, 0.60315E+00_JPRB, 0.59624E+00_JPRB, 0.59163E+00_JPRB ,&
&  0.58917E+00_JPRB, 0.58681E+00_JPRB, 0.58453E+00_JPRB, 0.58523E+00_JPRB, 0.58631E+00_JPRB, 0.58557E+00_JPRB, 0.58579E+00_JPRB ,&
&  0.58437E+00_JPRB, 0.58611E+00_JPRB, 0.58820E+00_JPRB, 0.59344E+00_JPRB, 0.59724E+00_JPRB, 0.59973E+00_JPRB, 0.60315E+00_JPRB ,&
&  0.60814E+00_JPRB, 0.61279E+00_JPRB, 0.61711E+00_JPRB, 0.62078E+00_JPRB, 0.62229E+00_JPRB, 0.62201E+00_JPRB, 0.61773E+00_JPRB ,&
&  0.61434E+00_JPRB, 0.61448E+00_JPRB, 0.61981E+00_JPRB, 0.63196E+00_JPRB, 0.64245E+00_JPRB, 0.64330E+00_JPRB, 0.64063E+00_JPRB ,&
&  0.64051E+00_JPRB, 0.64483E+00_JPRB, 0.64667E+00_JPRB, 0.64633E+00_JPRB, 0.64474E+00_JPRB, 0.64215E+00_JPRB, 0.63722E+00_JPRB ,&
&  0.63017E+00_JPRB, 0.62378E+00_JPRB, 0.61703E+00_JPRB, 0.60447E+00_JPRB, 0.59526E+00_JPRB, 0.59440E+00_JPRB, 0.59629E+00_JPRB ,&
&  0.59601E+00_JPRB, 0.59463E+00_JPRB, 0.59620E+00_JPRB, 0.60235E+00_JPRB, 0.60375E+00_JPRB, 0.61259E+00_JPRB, 0.62827E+00_JPRB ,&
&  0.64369E+00_JPRB, 0.65687E+00_JPRB, 0.66745E+00_JPRB, 0.67837E+00_JPRB, 0.68920E+00_JPRB, 0.69762E+00_JPRB, 0.70463E+00_JPRB ,&
&  0.70759E+00_JPRB, 0.70532E+00_JPRB, 0.69915E+00_JPRB, 0.69555E+00_JPRB, 0.69246E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.65320E+00_JPRB, 0.65149E+00_JPRB, 0.64947E+00_JPRB, 0.64673E+00_JPRB, 0.64286E+00_JPRB, 0.63940E+00_JPRB, 0.63433E+00_JPRB ,&
&  0.62940E+00_JPRB, 0.62563E+00_JPRB, 0.62401E+00_JPRB, 0.62325E+00_JPRB, 0.62436E+00_JPRB, 0.62869E+00_JPRB, 0.63590E+00_JPRB ,&
&  0.63581E+00_JPRB, 0.62831E+00_JPRB, 0.62449E+00_JPRB, 0.62424E+00_JPRB, 0.62640E+00_JPRB, 0.62788E+00_JPRB, 0.63014E+00_JPRB ,&
&  0.63139E+00_JPRB, 0.63145E+00_JPRB, 0.62970E+00_JPRB, 0.62108E+00_JPRB, 0.60307E+00_JPRB, 0.59252E+00_JPRB, 0.58392E+00_JPRB ,&
&  0.57992E+00_JPRB, 0.57943E+00_JPRB, 0.57501E+00_JPRB, 0.57387E+00_JPRB, 0.57251E+00_JPRB, 0.57053E+00_JPRB, 0.57040E+00_JPRB ,&
&  0.56868E+00_JPRB, 0.56637E+00_JPRB, 0.56535E+00_JPRB, 0.56608E+00_JPRB, 0.56794E+00_JPRB, 0.56946E+00_JPRB, 0.56902E+00_JPRB ,&
&  0.56956E+00_JPRB, 0.57282E+00_JPRB, 0.57324E+00_JPRB, 0.57280E+00_JPRB, 0.57209E+00_JPRB, 0.57117E+00_JPRB, 0.57098E+00_JPRB ,&
&  0.57340E+00_JPRB, 0.57895E+00_JPRB, 0.58405E+00_JPRB, 0.58945E+00_JPRB, 0.59218E+00_JPRB, 0.59351E+00_JPRB, 0.59354E+00_JPRB ,&
&  0.59006E+00_JPRB, 0.58560E+00_JPRB, 0.58393E+00_JPRB, 0.58712E+00_JPRB, 0.59378E+00_JPRB, 0.60003E+00_JPRB, 0.60262E+00_JPRB ,&
&  0.60440E+00_JPRB, 0.60529E+00_JPRB, 0.60330E+00_JPRB, 0.60253E+00_JPRB, 0.60027E+00_JPRB, 0.59605E+00_JPRB, 0.59370E+00_JPRB ,&
&  0.59133E+00_JPRB, 0.58881E+00_JPRB, 0.58606E+00_JPRB, 0.57651E+00_JPRB, 0.56889E+00_JPRB, 0.56487E+00_JPRB, 0.56257E+00_JPRB ,&
&  0.56215E+00_JPRB, 0.56164E+00_JPRB, 0.56166E+00_JPRB, 0.56285E+00_JPRB, 0.56533E+00_JPRB, 0.57354E+00_JPRB, 0.58895E+00_JPRB ,&
&  0.61074E+00_JPRB, 0.63050E+00_JPRB, 0.64099E+00_JPRB, 0.64811E+00_JPRB, 0.65249E+00_JPRB, 0.65454E+00_JPRB, 0.65663E+00_JPRB ,&
&  0.65997E+00_JPRB, 0.66113E+00_JPRB, 0.65906E+00_JPRB, 0.65647E+00_JPRB, 0.65523E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.62136E+00_JPRB, 0.62159E+00_JPRB, 0.62000E+00_JPRB, 0.61830E+00_JPRB, 0.61643E+00_JPRB, 0.61472E+00_JPRB, 0.61250E+00_JPRB ,&
&  0.60760E+00_JPRB, 0.60122E+00_JPRB, 0.59754E+00_JPRB, 0.59625E+00_JPRB, 0.60238E+00_JPRB, 0.61101E+00_JPRB, 0.61858E+00_JPRB ,&
&  0.61722E+00_JPRB, 0.60716E+00_JPRB, 0.60385E+00_JPRB, 0.60559E+00_JPRB, 0.60802E+00_JPRB, 0.61064E+00_JPRB, 0.61412E+00_JPRB ,&
&  0.61802E+00_JPRB, 0.62186E+00_JPRB, 0.62343E+00_JPRB, 0.61991E+00_JPRB, 0.61208E+00_JPRB, 0.59741E+00_JPRB, 0.58525E+00_JPRB ,&
&  0.57750E+00_JPRB, 0.57183E+00_JPRB, 0.56714E+00_JPRB, 0.56581E+00_JPRB, 0.56337E+00_JPRB, 0.55961E+00_JPRB, 0.55458E+00_JPRB ,&
&  0.55059E+00_JPRB, 0.54909E+00_JPRB, 0.55222E+00_JPRB, 0.55477E+00_JPRB, 0.55600E+00_JPRB, 0.55539E+00_JPRB, 0.55177E+00_JPRB ,&
&  0.54927E+00_JPRB, 0.54781E+00_JPRB, 0.54645E+00_JPRB, 0.54687E+00_JPRB, 0.54336E+00_JPRB, 0.54053E+00_JPRB, 0.54199E+00_JPRB ,&
&  0.54632E+00_JPRB, 0.55051E+00_JPRB, 0.55469E+00_JPRB, 0.55849E+00_JPRB, 0.56069E+00_JPRB, 0.56353E+00_JPRB, 0.56497E+00_JPRB ,&
&  0.56358E+00_JPRB, 0.55920E+00_JPRB, 0.55515E+00_JPRB, 0.55521E+00_JPRB, 0.55724E+00_JPRB, 0.55971E+00_JPRB, 0.56298E+00_JPRB ,&
&  0.56324E+00_JPRB, 0.55790E+00_JPRB, 0.55273E+00_JPRB, 0.54825E+00_JPRB, 0.54421E+00_JPRB, 0.54128E+00_JPRB, 0.54003E+00_JPRB ,&
&  0.53989E+00_JPRB, 0.53668E+00_JPRB, 0.53049E+00_JPRB, 0.52550E+00_JPRB, 0.52534E+00_JPRB, 0.52555E+00_JPRB, 0.52259E+00_JPRB ,&
&  0.52405E+00_JPRB, 0.52743E+00_JPRB, 0.52663E+00_JPRB, 0.52476E+00_JPRB, 0.52675E+00_JPRB, 0.53278E+00_JPRB, 0.54544E+00_JPRB ,&
&  0.56230E+00_JPRB, 0.58130E+00_JPRB, 0.59864E+00_JPRB, 0.61305E+00_JPRB, 0.62129E+00_JPRB, 0.62310E+00_JPRB, 0.62125E+00_JPRB ,&
&  0.62074E+00_JPRB, 0.62186E+00_JPRB, 0.62131E+00_JPRB, 0.61995E+00_JPRB, 0.62032E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.58959E+00_JPRB, 0.58953E+00_JPRB, 0.58968E+00_JPRB, 0.59078E+00_JPRB, 0.59151E+00_JPRB, 0.59037E+00_JPRB, 0.58992E+00_JPRB ,&
&  0.58658E+00_JPRB, 0.58185E+00_JPRB, 0.57930E+00_JPRB, 0.58160E+00_JPRB, 0.59175E+00_JPRB, 0.60395E+00_JPRB, 0.61081E+00_JPRB ,&
&  0.60537E+00_JPRB, 0.59182E+00_JPRB, 0.58837E+00_JPRB, 0.59061E+00_JPRB, 0.59291E+00_JPRB, 0.59701E+00_JPRB, 0.60296E+00_JPRB ,&
&  0.60843E+00_JPRB, 0.61210E+00_JPRB, 0.61615E+00_JPRB, 0.61765E+00_JPRB, 0.61724E+00_JPRB, 0.60939E+00_JPRB, 0.59786E+00_JPRB ,&
&  0.58685E+00_JPRB, 0.57870E+00_JPRB, 0.57251E+00_JPRB, 0.56856E+00_JPRB, 0.56337E+00_JPRB, 0.55855E+00_JPRB, 0.55240E+00_JPRB ,&
&  0.54994E+00_JPRB, 0.55041E+00_JPRB, 0.55205E+00_JPRB, 0.55043E+00_JPRB, 0.54767E+00_JPRB, 0.54163E+00_JPRB, 0.53613E+00_JPRB ,&
&  0.53324E+00_JPRB, 0.53326E+00_JPRB, 0.53316E+00_JPRB, 0.53082E+00_JPRB, 0.52597E+00_JPRB, 0.52224E+00_JPRB, 0.52273E+00_JPRB ,&
&  0.52816E+00_JPRB, 0.52773E+00_JPRB, 0.52671E+00_JPRB, 0.52820E+00_JPRB, 0.53117E+00_JPRB, 0.53363E+00_JPRB, 0.53295E+00_JPRB ,&
&  0.53191E+00_JPRB, 0.53117E+00_JPRB, 0.53072E+00_JPRB, 0.53198E+00_JPRB, 0.53170E+00_JPRB, 0.52920E+00_JPRB, 0.52321E+00_JPRB ,&
&  0.51433E+00_JPRB, 0.50342E+00_JPRB, 0.49906E+00_JPRB, 0.49589E+00_JPRB, 0.49284E+00_JPRB, 0.49249E+00_JPRB, 0.49431E+00_JPRB ,&
&  0.49317E+00_JPRB, 0.48703E+00_JPRB, 0.48447E+00_JPRB, 0.48565E+00_JPRB, 0.48592E+00_JPRB, 0.48953E+00_JPRB, 0.49490E+00_JPRB ,&
&  0.50697E+00_JPRB, 0.51408E+00_JPRB, 0.51623E+00_JPRB, 0.51394E+00_JPRB, 0.51398E+00_JPRB, 0.51571E+00_JPRB, 0.52044E+00_JPRB ,&
&  0.52757E+00_JPRB, 0.53545E+00_JPRB, 0.54385E+00_JPRB, 0.55314E+00_JPRB, 0.56424E+00_JPRB, 0.57468E+00_JPRB, 0.57892E+00_JPRB ,&
&  0.57961E+00_JPRB, 0.58060E+00_JPRB, 0.58255E+00_JPRB, 0.58469E+00_JPRB, 0.58809E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.55149E+00_JPRB, 0.55393E+00_JPRB, 0.55646E+00_JPRB, 0.55831E+00_JPRB, 0.56130E+00_JPRB, 0.56652E+00_JPRB, 0.56863E+00_JPRB ,&
&  0.56649E+00_JPRB, 0.56402E+00_JPRB, 0.56392E+00_JPRB, 0.56793E+00_JPRB, 0.57575E+00_JPRB, 0.58356E+00_JPRB, 0.59145E+00_JPRB ,&
&  0.58641E+00_JPRB, 0.57252E+00_JPRB, 0.56756E+00_JPRB, 0.56969E+00_JPRB, 0.57474E+00_JPRB, 0.58026E+00_JPRB, 0.58688E+00_JPRB ,&
&  0.59313E+00_JPRB, 0.59707E+00_JPRB, 0.60125E+00_JPRB, 0.60514E+00_JPRB, 0.60755E+00_JPRB, 0.60532E+00_JPRB, 0.60064E+00_JPRB ,&
&  0.59499E+00_JPRB, 0.58868E+00_JPRB, 0.58243E+00_JPRB, 0.56694E+00_JPRB, 0.56455E+00_JPRB, 0.55587E+00_JPRB, 0.54894E+00_JPRB ,&
&  0.54446E+00_JPRB, 0.54064E+00_JPRB, 0.53806E+00_JPRB, 0.53461E+00_JPRB, 0.52945E+00_JPRB, 0.52080E+00_JPRB, 0.51731E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.51457E+00_JPRB, 0.50841E+00_JPRB, 0.50458E+00_JPRB, 0.50033E+00_JPRB, 0.49679E+00_JPRB, 0.49990E+00_JPRB ,&
&  0.50432E+00_JPRB, 0.50113E+00_JPRB, 0.49676E+00_JPRB, 0.49669E+00_JPRB, 0.50135E+00_JPRB, 0.50313E+00_JPRB, 0.50395E+00_JPRB ,&
&  0.50535E+00_JPRB, 0.50516E+00_JPRB, 0.49956E+00_JPRB, 0.49373E+00_JPRB, 0.48527E+00_JPRB, 0.47340E+00_JPRB, 0.46944E+00_JPRB ,&
&  0.46598E+00_JPRB, 0.46065E+00_JPRB, 0.45835E+00_JPRB, 0.45760E+00_JPRB, 0.45745E+00_JPRB, 0.45884E+00_JPRB, 0.45957E+00_JPRB ,&
&  0.46028E+00_JPRB, 0.45691E+00_JPRB, 0.45205E+00_JPRB, 0.44829E+00_JPRB, 0.44308E+00_JPRB, 0.44722E+00_JPRB, 0.45893E+00_JPRB ,&
&  0.47277E+00_JPRB, 0.48149E+00_JPRB, 0.48715E+00_JPRB, 0.49063E+00_JPRB, 0.49368E+00_JPRB, 0.49482E+00_JPRB, 0.49707E+00_JPRB ,&
&  0.50003E+00_JPRB, 0.50136E+00_JPRB, 0.50029E+00_JPRB, 0.50189E+00_JPRB, 0.50653E+00_JPRB, 0.51292E+00_JPRB, 0.52025E+00_JPRB ,&
&  0.52728E+00_JPRB, 0.53315E+00_JPRB, 0.53804E+00_JPRB, 0.54398E+00_JPRB, 0.54796E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.51976E+00_JPRB, 0.52446E+00_JPRB, 0.52914E+00_JPRB, 0.53419E+00_JPRB, 0.54116E+00_JPRB, 0.54508E+00_JPRB, 0.54652E+00_JPRB ,&
&  0.54785E+00_JPRB, 0.54814E+00_JPRB, 0.54927E+00_JPRB, 0.55351E+00_JPRB, 0.56031E+00_JPRB, 0.56573E+00_JPRB, 0.57223E+00_JPRB ,&
&  0.57218E+00_JPRB, 0.56232E+00_JPRB, 0.55388E+00_JPRB, 0.55365E+00_JPRB, 0.55951E+00_JPRB, 0.56554E+00_JPRB, 0.57286E+00_JPRB ,&
&  0.57996E+00_JPRB, 0.58488E+00_JPRB, 0.58748E+00_JPRB, 0.58872E+00_JPRB, 0.59037E+00_JPRB, 0.58946E+00_JPRB, 0.58614E+00_JPRB ,&
&  0.58129E+00_JPRB, 0.57082E+00_JPRB, 0.55117E+00_JPRB, 0.54885E+00_JPRB, 0.54012E+00_JPRB, 0.53271E+00_JPRB, 0.52765E+00_JPRB ,&
&  0.52367E+00_JPRB, 0.52009E+00_JPRB, 0.51576E+00_JPRB, 0.51144E+00_JPRB, 0.50632E+00_JPRB, 0.49780E+00_JPRB, 0.49278E+00_JPRB ,&
&  0.48806E+00_JPRB, 0.48429E+00_JPRB, 0.48029E+00_JPRB, 0.47924E+00_JPRB, 0.47836E+00_JPRB, 0.48000E+00_JPRB, 0.48045E+00_JPRB ,&
&  0.47636E+00_JPRB, 0.47173E+00_JPRB, 0.46832E+00_JPRB, 0.46984E+00_JPRB, 0.47243E+00_JPRB, 0.47419E+00_JPRB, 0.47479E+00_JPRB ,&
&  0.47154E+00_JPRB, 0.46563E+00_JPRB, 0.45939E+00_JPRB, 0.45258E+00_JPRB, 0.44412E+00_JPRB, 0.41849E+00_JPRB, 0.40803E+00_JPRB ,&
&  0.40660E+00_JPRB, 0.41655E+00_JPRB, 0.42745E+00_JPRB, 0.42752E+00_JPRB, 0.43211E+00_JPRB, 0.43305E+00_JPRB, 0.43046E+00_JPRB ,&
&  0.41982E+00_JPRB, 0.40683E+00_JPRB, 0.39959E+00_JPRB, 0.39677E+00_JPRB, 0.39480E+00_JPRB, 0.39809E+00_JPRB, 0.40588E+00_JPRB ,&
&  0.41785E+00_JPRB, 0.43002E+00_JPRB, 0.43896E+00_JPRB, 0.44289E+00_JPRB, 0.44128E+00_JPRB, 0.43821E+00_JPRB, 0.43413E+00_JPRB ,&
&  0.43054E+00_JPRB, 0.43798E+00_JPRB, 0.45417E+00_JPRB, 0.47354E+00_JPRB, 0.47631E+00_JPRB, 0.48259E+00_JPRB, 0.48700E+00_JPRB ,&
&  0.49313E+00_JPRB, 0.50086E+00_JPRB, 0.50716E+00_JPRB, 0.51329E+00_JPRB, 0.51686E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.49470E+00_JPRB, 0.50006E+00_JPRB, 0.50724E+00_JPRB, 0.51521E+00_JPRB, 0.52108E+00_JPRB, 0.52423E+00_JPRB, 0.52759E+00_JPRB ,&
&  0.53150E+00_JPRB, 0.53322E+00_JPRB, 0.53426E+00_JPRB, 0.53687E+00_JPRB, 0.54255E+00_JPRB, 0.54618E+00_JPRB, 0.55178E+00_JPRB ,&
&  0.55424E+00_JPRB, 0.55172E+00_JPRB, 0.54441E+00_JPRB, 0.54254E+00_JPRB, 0.54653E+00_JPRB, 0.55163E+00_JPRB, 0.55679E+00_JPRB ,&
&  0.56186E+00_JPRB, 0.56642E+00_JPRB, 0.56839E+00_JPRB, 0.56853E+00_JPRB, 0.56825E+00_JPRB, 0.56769E+00_JPRB, 0.56473E+00_JPRB ,&
&  0.55741E+00_JPRB, 0.51575E+00_JPRB, 0.48966E+00_JPRB, 0.49686E+00_JPRB, 0.50730E+00_JPRB, 0.49974E+00_JPRB, 0.49818E+00_JPRB ,&
&  0.49672E+00_JPRB, 0.49254E+00_JPRB, 0.48645E+00_JPRB, 0.48114E+00_JPRB, 0.47492E+00_JPRB, 0.46667E+00_JPRB, 0.46027E+00_JPRB ,&
&  0.45810E+00_JPRB, 0.46373E+00_JPRB, 0.46478E+00_JPRB, 0.46517E+00_JPRB, 0.46167E+00_JPRB, 0.46167E+00_JPRB, 0.45862E+00_JPRB ,&
&  0.45127E+00_JPRB, 0.44315E+00_JPRB, 0.43976E+00_JPRB, 0.43878E+00_JPRB, 0.43997E+00_JPRB, 0.44280E+00_JPRB, 0.44131E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.41690E+00_JPRB, 0.39762E+00_JPRB, 0.37887E+00_JPRB, 0.36556E+00_JPRB, 0.35977E+00_JPRB, 0.35563E+00_JPRB ,&
&  0.35554E+00_JPRB, 0.36337E+00_JPRB, 0.38386E+00_JPRB, 0.40754E+00_JPRB, 0.41681E+00_JPRB, 0.39785E+00_JPRB, 0.38354E+00_JPRB ,&
&  0.37527E+00_JPRB, 0.37061E+00_JPRB, 0.36649E+00_JPRB, 0.36561E+00_JPRB, 0.36680E+00_JPRB, 0.36981E+00_JPRB, 0.37386E+00_JPRB ,&
&  0.37962E+00_JPRB, 0.38490E+00_JPRB, 0.38773E+00_JPRB, 0.38906E+00_JPRB, 0.38993E+00_JPRB, 0.39094E+00_JPRB, 0.39043E+00_JPRB ,&
&  0.39081E+00_JPRB, 0.39565E+00_JPRB, 0.40412E+00_JPRB, 0.41418E+00_JPRB, 0.42292E+00_JPRB, 0.43314E+00_JPRB, 0.44734E+00_JPRB ,&
&  0.46242E+00_JPRB, 0.47270E+00_JPRB, 0.47680E+00_JPRB, 0.48390E+00_JPRB, 0.48971E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.48188E+00_JPRB, 0.48980E+00_JPRB, 0.49457E+00_JPRB, 0.50432E+00_JPRB, 0.50762E+00_JPRB, 0.51145E+00_JPRB, 0.51505E+00_JPRB ,&
&  0.52240E+00_JPRB, 0.52634E+00_JPRB, 0.52443E+00_JPRB, 0.52415E+00_JPRB, 0.52329E+00_JPRB, 0.52130E+00_JPRB, 0.52272E+00_JPRB ,&
&  0.52334E+00_JPRB, 0.52586E+00_JPRB, 0.52316E+00_JPRB, 0.52054E+00_JPRB, 0.52270E+00_JPRB, 0.52785E+00_JPRB, 0.53120E+00_JPRB ,&
&  0.52537E+00_JPRB, 0.52302E+00_JPRB, 0.53117E+00_JPRB, 0.53240E+00_JPRB, 0.53277E+00_JPRB, 0.53413E+00_JPRB, 0.53338E+00_JPRB ,&
&  0.51827E+00_JPRB, 0.49082E+00_JPRB, 0.46331E+00_JPRB, 0.46795E+00_JPRB, 0.48296E+00_JPRB, 0.47401E+00_JPRB, 0.47200E+00_JPRB ,&
&  0.46918E+00_JPRB, 0.46429E+00_JPRB, 0.45983E+00_JPRB, 0.45721E+00_JPRB, 0.45442E+00_JPRB, 0.44895E+00_JPRB, 0.44747E+00_JPRB ,&
&  0.44565E+00_JPRB, 0.45609E+00_JPRB, 0.44872E+00_JPRB, 0.42273E+00_JPRB, 0.40531E+00_JPRB, 0.39546E+00_JPRB, 0.38243E+00_JPRB ,&
&  0.36682E+00_JPRB, 0.35894E+00_JPRB, 0.35836E+00_JPRB, 0.35549E+00_JPRB, 0.35616E+00_JPRB, 0.35736E+00_JPRB, 0.35845E+00_JPRB ,&
&  0.35870E+00_JPRB, 0.35787E+00_JPRB, 0.35268E+00_JPRB, 0.34605E+00_JPRB, 0.34252E+00_JPRB, 0.34270E+00_JPRB, 0.34637E+00_JPRB ,&
&  0.35384E+00_JPRB, 0.36095E+00_JPRB, 0.37721E+00_JPRB, 0.38517E+00_JPRB, 0.37481E+00_JPRB, 0.37134E+00_JPRB, 0.36748E+00_JPRB ,&
&  0.35498E+00_JPRB, 0.35056E+00_JPRB, 0.34691E+00_JPRB, 0.34502E+00_JPRB, 0.34620E+00_JPRB, 0.35033E+00_JPRB, 0.35187E+00_JPRB ,&
&  0.35476E+00_JPRB, 0.35834E+00_JPRB, 0.35985E+00_JPRB, 0.36095E+00_JPRB, 0.36192E+00_JPRB, 0.36444E+00_JPRB, 0.36702E+00_JPRB ,&
&  0.36892E+00_JPRB, 0.37248E+00_JPRB, 0.37670E+00_JPRB, 0.38214E+00_JPRB, 0.38692E+00_JPRB, 0.38961E+00_JPRB, 0.39889E+00_JPRB ,&
&  0.40903E+00_JPRB, 0.42938E+00_JPRB, 0.46170E+00_JPRB, 0.48477E+00_JPRB, 0.47601E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.48182E+00_JPRB, 0.48599E+00_JPRB, 0.48596E+00_JPRB, 0.48924E+00_JPRB, 0.49525E+00_JPRB, 0.50200E+00_JPRB, 0.50797E+00_JPRB ,&
&  0.51369E+00_JPRB, 0.51335E+00_JPRB, 0.50837E+00_JPRB, 0.49986E+00_JPRB, 0.49289E+00_JPRB, 0.47614E+00_JPRB, 0.46793E+00_JPRB ,&
&  0.46097E+00_JPRB, 0.46146E+00_JPRB, 0.46176E+00_JPRB, 0.46368E+00_JPRB, 0.46446E+00_JPRB, 0.46963E+00_JPRB, 0.47076E+00_JPRB ,&
&  0.46741E+00_JPRB, 0.46162E+00_JPRB, 0.45558E+00_JPRB, 0.45426E+00_JPRB, 0.45691E+00_JPRB, 0.45851E+00_JPRB, 0.45487E+00_JPRB ,&
&  0.45107E+00_JPRB, 0.45103E+00_JPRB, 0.45145E+00_JPRB, 0.45287E+00_JPRB, 0.46214E+00_JPRB, 0.45493E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.44301E+00_JPRB, 0.43720E+00_JPRB, 0.43350E+00_JPRB, 0.43406E+00_JPRB, 0.43822E+00_JPRB, 0.43562E+00_JPRB, 0.43874E+00_JPRB ,&
&  0.42884E+00_JPRB, 0.42059E+00_JPRB, 0.40479E+00_JPRB, 0.39167E+00_JPRB, 0.37617E+00_JPRB, 0.36562E+00_JPRB, 0.35614E+00_JPRB ,&
&  0.34817E+00_JPRB, 0.34110E+00_JPRB, 0.33461E+00_JPRB, 0.33277E+00_JPRB, 0.33389E+00_JPRB, 0.33354E+00_JPRB, 0.33244E+00_JPRB ,&
&  0.33266E+00_JPRB, 0.33351E+00_JPRB, 0.33417E+00_JPRB, 0.33531E+00_JPRB, 0.33739E+00_JPRB, 0.34066E+00_JPRB, 0.34520E+00_JPRB ,&
&  0.35215E+00_JPRB, 0.35743E+00_JPRB, 0.36019E+00_JPRB, 0.36216E+00_JPRB, 0.35893E+00_JPRB, 0.35288E+00_JPRB, 0.34726E+00_JPRB ,&
&  0.34100E+00_JPRB, 0.33817E+00_JPRB, 0.33539E+00_JPRB, 0.33409E+00_JPRB, 0.33371E+00_JPRB, 0.33463E+00_JPRB, 0.33678E+00_JPRB ,&
&  0.33893E+00_JPRB, 0.34191E+00_JPRB, 0.34386E+00_JPRB, 0.34616E+00_JPRB, 0.34885E+00_JPRB, 0.35114E+00_JPRB, 0.35349E+00_JPRB ,&
&  0.35616E+00_JPRB, 0.35827E+00_JPRB, 0.36201E+00_JPRB, 0.36670E+00_JPRB, 0.37589E+00_JPRB, 0.38575E+00_JPRB, 0.40299E+00_JPRB ,&
&  0.42217E+00_JPRB, 0.45226E+00_JPRB, 0.45868E+00_JPRB, 0.47143E+00_JPRB, 0.48094E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.47589E+00_JPRB, 0.47853E+00_JPRB, 0.48203E+00_JPRB, 0.48790E+00_JPRB, 0.49545E+00_JPRB, 0.50272E+00_JPRB, 0.50473E+00_JPRB ,&
&  0.49449E+00_JPRB, 0.48166E+00_JPRB, 0.46290E+00_JPRB, 0.44850E+00_JPRB, 0.43393E+00_JPRB, 0.42329E+00_JPRB, 0.41494E+00_JPRB ,&
&  0.40892E+00_JPRB, 0.40883E+00_JPRB, 0.40872E+00_JPRB, 0.40988E+00_JPRB, 0.40817E+00_JPRB, 0.40883E+00_JPRB, 0.40854E+00_JPRB ,&
&  0.40665E+00_JPRB, 0.40636E+00_JPRB, 0.40628E+00_JPRB, 0.40957E+00_JPRB, 0.41647E+00_JPRB, 0.42260E+00_JPRB, 0.42862E+00_JPRB ,&
&  0.43585E+00_JPRB, 0.43704E+00_JPRB, 0.44089E+00_JPRB, 0.43894E+00_JPRB, 0.43548E+00_JPRB, 0.43109E+00_JPRB, 0.42480E+00_JPRB ,&
&  0.42121E+00_JPRB, 0.41778E+00_JPRB, 0.41721E+00_JPRB, 0.41755E+00_JPRB, 0.41749E+00_JPRB, 0.42252E+00_JPRB, 0.41818E+00_JPRB ,&
&  0.41148E+00_JPRB, 0.40028E+00_JPRB, 0.38599E+00_JPRB, 0.37501E+00_JPRB, 0.36368E+00_JPRB, 0.35560E+00_JPRB, 0.34840E+00_JPRB ,&
&  0.34184E+00_JPRB, 0.33751E+00_JPRB, 0.33289E+00_JPRB, 0.32942E+00_JPRB, 0.32828E+00_JPRB, 0.32579E+00_JPRB, 0.32428E+00_JPRB ,&
&  0.32361E+00_JPRB, 0.32400E+00_JPRB, 0.32624E+00_JPRB, 0.32819E+00_JPRB, 0.33095E+00_JPRB, 0.33389E+00_JPRB, 0.33627E+00_JPRB ,&
&  0.33954E+00_JPRB, 0.34225E+00_JPRB, 0.34262E+00_JPRB, 0.34207E+00_JPRB, 0.33965E+00_JPRB, 0.33632E+00_JPRB, 0.33363E+00_JPRB ,&
&  0.33061E+00_JPRB, 0.32855E+00_JPRB, 0.32793E+00_JPRB, 0.32685E+00_JPRB, 0.32743E+00_JPRB, 0.32878E+00_JPRB, 0.32963E+00_JPRB ,&
&  0.33167E+00_JPRB, 0.33332E+00_JPRB, 0.33536E+00_JPRB, 0.33914E+00_JPRB, 0.34241E+00_JPRB, 0.34530E+00_JPRB, 0.34810E+00_JPRB ,&
&  0.34785E+00_JPRB, 0.34974E+00_JPRB, 0.35253E+00_JPRB, 0.35809E+00_JPRB, 0.37490E+00_JPRB, 0.38515E+00_JPRB, 0.40892E+00_JPRB ,&
&  0.42615E+00_JPRB, 0.45752E+00_JPRB, 0.45357E+00_JPRB, 0.46323E+00_JPRB, 0.47159E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.46964E+00_JPRB, 0.47697E+00_JPRB, 0.48349E+00_JPRB, 0.48898E+00_JPRB, 0.49244E+00_JPRB, 0.49589E+00_JPRB, 0.49269E+00_JPRB ,&
&  0.48663E+00_JPRB, 0.47275E+00_JPRB, 0.46171E+00_JPRB, 0.45001E+00_JPRB, 0.44048E+00_JPRB, 0.43104E+00_JPRB, 0.42238E+00_JPRB ,&
&  0.41379E+00_JPRB, 0.40651E+00_JPRB, 0.40021E+00_JPRB, 0.39455E+00_JPRB, 0.38696E+00_JPRB, 0.38018E+00_JPRB, 0.37472E+00_JPRB ,&
&  0.37020E+00_JPRB, 0.37012E+00_JPRB, 0.37283E+00_JPRB, 0.37761E+00_JPRB, 0.38665E+00_JPRB, 0.39508E+00_JPRB, 0.40440E+00_JPRB ,&
&  0.41834E+00_JPRB, 0.41971E+00_JPRB, 0.41846E+00_JPRB, 0.41792E+00_JPRB, 0.41286E+00_JPRB, 0.40635E+00_JPRB, 0.39761E+00_JPRB ,&
&  0.40119E+00_JPRB, 0.39919E+00_JPRB, 0.40818E+00_JPRB, 0.40796E+00_JPRB, 0.40659E+00_JPRB, 0.40864E+00_JPRB, 0.40172E+00_JPRB ,&
&  0.39789E+00_JPRB, 0.38802E+00_JPRB, 0.38029E+00_JPRB, 0.37120E+00_JPRB, 0.36129E+00_JPRB, 0.35552E+00_JPRB, 0.34783E+00_JPRB ,&
&  0.34239E+00_JPRB, 0.33863E+00_JPRB, 0.33354E+00_JPRB, 0.33056E+00_JPRB, 0.32780E+00_JPRB, 0.32482E+00_JPRB, 0.32324E+00_JPRB ,&
&  0.32205E+00_JPRB, 0.32163E+00_JPRB, 0.32192E+00_JPRB, 0.32221E+00_JPRB, 0.32288E+00_JPRB, 0.32400E+00_JPRB, 0.32479E+00_JPRB ,&
&  0.32570E+00_JPRB, 0.32612E+00_JPRB, 0.32576E+00_JPRB, 0.32543E+00_JPRB, 0.32406E+00_JPRB, 0.32322E+00_JPRB, 0.32199E+00_JPRB ,&
&  0.32117E+00_JPRB, 0.32152E+00_JPRB, 0.32177E+00_JPRB, 0.32253E+00_JPRB, 0.32324E+00_JPRB, 0.32502E+00_JPRB, 0.32665E+00_JPRB ,&
&  0.32841E+00_JPRB, 0.33076E+00_JPRB, 0.33258E+00_JPRB, 0.33506E+00_JPRB, 0.33791E+00_JPRB, 0.33958E+00_JPRB, 0.34292E+00_JPRB ,&
&  0.34606E+00_JPRB, 0.35015E+00_JPRB, 0.35771E+00_JPRB, 0.36577E+00_JPRB, 0.37661E+00_JPRB, 0.38849E+00_JPRB, 0.40292E+00_JPRB ,&
&  0.42107E+00_JPRB, 0.44472E+00_JPRB, 0.44504E+00_JPRB, 0.45653E+00_JPRB, 0.46325E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.45497E+00_JPRB, 0.46105E+00_JPRB, 0.46613E+00_JPRB, 0.47068E+00_JPRB, 0.47497E+00_JPRB, 0.47654E+00_JPRB, 0.47565E+00_JPRB ,&
&  0.46616E+00_JPRB, 0.45948E+00_JPRB, 0.45102E+00_JPRB, 0.44304E+00_JPRB, 0.43563E+00_JPRB, 0.42561E+00_JPRB, 0.41782E+00_JPRB ,&
&  0.40984E+00_JPRB, 0.40178E+00_JPRB, 0.39443E+00_JPRB, 0.38567E+00_JPRB, 0.37663E+00_JPRB, 0.36790E+00_JPRB, 0.36294E+00_JPRB ,&
&  0.36018E+00_JPRB, 0.36031E+00_JPRB, 0.36395E+00_JPRB, 0.36803E+00_JPRB, 0.37323E+00_JPRB, 0.37983E+00_JPRB, 0.38700E+00_JPRB ,&
&  0.39142E+00_JPRB, 0.39368E+00_JPRB, 0.39285E+00_JPRB, 0.39052E+00_JPRB, 0.38802E+00_JPRB, 0.38626E+00_JPRB, 0.38346E+00_JPRB ,&
&  0.37984E+00_JPRB, 0.37806E+00_JPRB, 0.37870E+00_JPRB, 0.38161E+00_JPRB, 0.38281E+00_JPRB, 0.38184E+00_JPRB, 0.37788E+00_JPRB ,&
&  0.37297E+00_JPRB, 0.36784E+00_JPRB, 0.36224E+00_JPRB, 0.35692E+00_JPRB, 0.35257E+00_JPRB, 0.34782E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.33932E+00_JPRB, 0.33598E+00_JPRB, 0.33228E+00_JPRB, 0.32889E+00_JPRB, 0.32603E+00_JPRB, 0.32385E+00_JPRB, 0.32198E+00_JPRB ,&
&  0.32067E+00_JPRB, 0.31968E+00_JPRB, 0.31895E+00_JPRB, 0.31842E+00_JPRB, 0.31790E+00_JPRB, 0.31695E+00_JPRB, 0.31601E+00_JPRB ,&
&  0.31508E+00_JPRB, 0.31480E+00_JPRB, 0.31419E+00_JPRB, 0.31423E+00_JPRB, 0.31455E+00_JPRB, 0.31471E+00_JPRB, 0.31529E+00_JPRB ,&
&  0.31656E+00_JPRB, 0.31788E+00_JPRB, 0.31837E+00_JPRB, 0.31958E+00_JPRB, 0.32076E+00_JPRB, 0.32235E+00_JPRB, 0.32455E+00_JPRB ,&
&  0.32581E+00_JPRB, 0.32721E+00_JPRB, 0.32886E+00_JPRB, 0.33063E+00_JPRB, 0.33388E+00_JPRB, 0.33740E+00_JPRB, 0.34130E+00_JPRB ,&
&  0.34635E+00_JPRB, 0.35113E+00_JPRB, 0.35667E+00_JPRB, 0.36330E+00_JPRB, 0.37595E+00_JPRB, 0.38289E+00_JPRB, 0.39109E+00_JPRB ,&
&  0.39844E+00_JPRB, 0.39898E+00_JPRB, 0.43605E+00_JPRB, 0.43942E+00_JPRB, 0.44589E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.37830E+00_JPRB, 0.38526E+00_JPRB, 0.38923E+00_JPRB, 0.39297E+00_JPRB, 0.38533E+00_JPRB, 0.42547E+00_JPRB, 0.42460E+00_JPRB ,&
&  0.42426E+00_JPRB, 0.41221E+00_JPRB, 0.41157E+00_JPRB, 0.40748E+00_JPRB, 0.40216E+00_JPRB, 0.39601E+00_JPRB, 0.39065E+00_JPRB ,&
&  0.38563E+00_JPRB, 0.37987E+00_JPRB, 0.37532E+00_JPRB, 0.37108E+00_JPRB, 0.36763E+00_JPRB, 0.36460E+00_JPRB, 0.36249E+00_JPRB ,&
&  0.36208E+00_JPRB, 0.36390E+00_JPRB, 0.36765E+00_JPRB, 0.36918E+00_JPRB, 0.36942E+00_JPRB, 0.36960E+00_JPRB, 0.36988E+00_JPRB ,&
&  0.36952E+00_JPRB, 0.36896E+00_JPRB, 0.36716E+00_JPRB, 0.36401E+00_JPRB, 0.36192E+00_JPRB, 0.36074E+00_JPRB, 0.35995E+00_JPRB ,&
&  0.35929E+00_JPRB, 0.35884E+00_JPRB, 0.35833E+00_JPRB, 0.35825E+00_JPRB, 0.35764E+00_JPRB, 0.35642E+00_JPRB, 0.35422E+00_JPRB ,&
&  0.35186E+00_JPRB, 0.34930E+00_JPRB, 0.34603E+00_JPRB, 0.34340E+00_JPRB, 0.34083E+00_JPRB, 0.33823E+00_JPRB, 0.33558E+00_JPRB ,&
&  0.33410E+00_JPRB, 0.33266E+00_JPRB, 0.33081E+00_JPRB, 0.32848E+00_JPRB, 0.32677E+00_JPRB, 0.32498E+00_JPRB, 0.32344E+00_JPRB ,&
&  0.32248E+00_JPRB, 0.32142E+00_JPRB, 0.32035E+00_JPRB, 0.31924E+00_JPRB, 0.31849E+00_JPRB, 0.31754E+00_JPRB, 0.31663E+00_JPRB ,&
&  0.31581E+00_JPRB, 0.31482E+00_JPRB, 0.31458E+00_JPRB, 0.31445E+00_JPRB, 0.31452E+00_JPRB, 0.31451E+00_JPRB, 0.31485E+00_JPRB ,&
&  0.31539E+00_JPRB, 0.31621E+00_JPRB, 0.31725E+00_JPRB, 0.31820E+00_JPRB, 0.31910E+00_JPRB, 0.32003E+00_JPRB, 0.32146E+00_JPRB ,&
&  0.32295E+00_JPRB, 0.32449E+00_JPRB, 0.32607E+00_JPRB, 0.32726E+00_JPRB, 0.33022E+00_JPRB, 0.33298E+00_JPRB, 0.33590E+00_JPRB ,&
&  0.33926E+00_JPRB, 0.34331E+00_JPRB, 0.34776E+00_JPRB, 0.35262E+00_JPRB, 0.35922E+00_JPRB, 0.36282E+00_JPRB, 0.36528E+00_JPRB ,&
&  0.36800E+00_JPRB, 0.37294E+00_JPRB, 0.37381E+00_JPRB, 0.37472E+00_JPRB, 0.37626E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.35086E+00_JPRB, 0.35110E+00_JPRB, 0.35100E+00_JPRB, 0.35060E+00_JPRB, 0.34702E+00_JPRB, 0.34660E+00_JPRB, 0.34612E+00_JPRB ,&
&  0.34597E+00_JPRB, 0.34604E+00_JPRB, 0.34683E+00_JPRB, 0.34780E+00_JPRB, 0.34880E+00_JPRB, 0.34979E+00_JPRB, 0.35072E+00_JPRB ,&
&  0.35132E+00_JPRB, 0.35207E+00_JPRB, 0.35040E+00_JPRB, 0.34929E+00_JPRB, 0.34907E+00_JPRB, 0.34907E+00_JPRB, 0.34897E+00_JPRB ,&
&  0.34876E+00_JPRB, 0.34852E+00_JPRB, 0.34784E+00_JPRB, 0.34739E+00_JPRB, 0.34719E+00_JPRB, 0.34692E+00_JPRB, 0.34652E+00_JPRB ,&
&  0.34608E+00_JPRB, 0.34573E+00_JPRB, 0.34532E+00_JPRB, 0.34481E+00_JPRB, 0.34479E+00_JPRB, 0.34482E+00_JPRB, 0.34431E+00_JPRB ,&
&  0.34369E+00_JPRB, 0.34267E+00_JPRB, 0.34174E+00_JPRB, 0.34088E+00_JPRB, 0.34018E+00_JPRB, 0.33991E+00_JPRB, 0.33958E+00_JPRB ,&
&  0.33909E+00_JPRB, 0.33840E+00_JPRB, 0.33774E+00_JPRB, 0.33685E+00_JPRB, 0.33605E+00_JPRB, 0.33529E+00_JPRB, 0.33441E+00_JPRB ,&
&  0.33344E+00_JPRB, 0.33243E+00_JPRB, 0.33139E+00_JPRB, 0.33020E+00_JPRB, 0.32901E+00_JPRB, 0.32778E+00_JPRB, 0.32645E+00_JPRB ,&
&  0.32512E+00_JPRB, 0.32459E+00_JPRB, 0.32397E+00_JPRB, 0.32345E+00_JPRB, 0.32305E+00_JPRB, 0.32276E+00_JPRB, 0.32245E+00_JPRB ,&
&  0.32221E+00_JPRB, 0.32196E+00_JPRB, 0.32184E+00_JPRB, 0.32173E+00_JPRB, 0.32173E+00_JPRB, 0.32181E+00_JPRB, 0.32203E+00_JPRB ,&
&  0.32228E+00_JPRB, 0.32259E+00_JPRB, 0.32301E+00_JPRB, 0.32376E+00_JPRB, 0.32449E+00_JPRB, 0.32535E+00_JPRB, 0.32619E+00_JPRB ,&
&  0.32712E+00_JPRB, 0.32804E+00_JPRB, 0.32898E+00_JPRB, 0.32988E+00_JPRB, 0.33113E+00_JPRB, 0.33232E+00_JPRB, 0.33365E+00_JPRB ,&
&  0.33496E+00_JPRB, 0.33639E+00_JPRB, 0.33779E+00_JPRB, 0.33930E+00_JPRB, 0.34105E+00_JPRB, 0.34151E+00_JPRB, 0.34201E+00_JPRB ,&
&  0.34262E+00_JPRB, 0.34353E+00_JPRB, 0.34432E+00_JPRB, 0.34544E+00_JPRB, 0.34735E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB, 0.33367E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB, 0.73124E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.73115E+00_JPRB, 0.73196E+00_JPRB, 0.73268E+00_JPRB, 0.73308E+00_JPRB, 0.73329E+00_JPRB, 0.73342E+00_JPRB, 0.73346E+00_JPRB ,&
&  0.73330E+00_JPRB, 0.73348E+00_JPRB, 0.73385E+00_JPRB, 0.73425E+00_JPRB, 0.73458E+00_JPRB, 0.73479E+00_JPRB, 0.73471E+00_JPRB ,&
&  0.73511E+00_JPRB, 0.73723E+00_JPRB, 0.73848E+00_JPRB, 0.73902E+00_JPRB, 0.73948E+00_JPRB, 0.73983E+00_JPRB, 0.74001E+00_JPRB ,&
&  0.74025E+00_JPRB, 0.74050E+00_JPRB, 0.74077E+00_JPRB, 0.74099E+00_JPRB, 0.74124E+00_JPRB, 0.74152E+00_JPRB, 0.74185E+00_JPRB ,&
&  0.74240E+00_JPRB, 0.74362E+00_JPRB, 0.74530E+00_JPRB, 0.74716E+00_JPRB, 0.74944E+00_JPRB, 0.75243E+00_JPRB, 0.75489E+00_JPRB ,&
&  0.75663E+00_JPRB, 0.75748E+00_JPRB, 0.75638E+00_JPRB, 0.75379E+00_JPRB, 0.75032E+00_JPRB, 0.74661E+00_JPRB, 0.74302E+00_JPRB ,&
&  0.73894E+00_JPRB, 0.73524E+00_JPRB, 0.73193E+00_JPRB, 0.72905E+00_JPRB, 0.72636E+00_JPRB, 0.72402E+00_JPRB, 0.72176E+00_JPRB ,&
&  0.72003E+00_JPRB, 0.71834E+00_JPRB, 0.71689E+00_JPRB, 0.71549E+00_JPRB, 0.71403E+00_JPRB, 0.71220E+00_JPRB, 0.71028E+00_JPRB ,&
&  0.70888E+00_JPRB, 0.70808E+00_JPRB, 0.70756E+00_JPRB, 0.70703E+00_JPRB, 0.70685E+00_JPRB, 0.70749E+00_JPRB, 0.70844E+00_JPRB ,&
&  0.70930E+00_JPRB, 0.71045E+00_JPRB, 0.71179E+00_JPRB, 0.71272E+00_JPRB, 0.71335E+00_JPRB, 0.71356E+00_JPRB, 0.71352E+00_JPRB ,&
&  0.71335E+00_JPRB, 0.71367E+00_JPRB, 0.71413E+00_JPRB, 0.71504E+00_JPRB, 0.71597E+00_JPRB, 0.71711E+00_JPRB, 0.71841E+00_JPRB ,&
&  0.71941E+00_JPRB, 0.72055E+00_JPRB, 0.72155E+00_JPRB, 0.72282E+00_JPRB, 0.72394E+00_JPRB, 0.72479E+00_JPRB, 0.72537E+00_JPRB ,&
&  0.72584E+00_JPRB, 0.72612E+00_JPRB, 0.72644E+00_JPRB, 0.72661E+00_JPRB, 0.72692E+00_JPRB, 0.72727E+00_JPRB, 0.72763E+00_JPRB ,&
&  0.72797E+00_JPRB, 0.72849E+00_JPRB, 0.72891E+00_JPRB, 0.72973E+00_JPRB, 0.73044E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.73746E+00_JPRB, 0.73880E+00_JPRB, 0.73986E+00_JPRB, 0.74038E+00_JPRB, 0.74034E+00_JPRB, 0.73957E+00_JPRB, 0.73825E+00_JPRB ,&
&  0.73617E+00_JPRB, 0.73455E+00_JPRB, 0.73365E+00_JPRB, 0.73260E+00_JPRB, 0.73100E+00_JPRB, 0.73181E+00_JPRB, 0.73332E+00_JPRB ,&
&  0.73217E+00_JPRB, 0.73045E+00_JPRB, 0.72838E+00_JPRB, 0.72655E+00_JPRB, 0.72461E+00_JPRB, 0.72247E+00_JPRB, 0.72130E+00_JPRB ,&
&  0.72077E+00_JPRB, 0.71972E+00_JPRB, 0.71794E+00_JPRB, 0.71525E+00_JPRB, 0.71450E+00_JPRB, 0.71382E+00_JPRB, 0.71304E+00_JPRB ,&
&  0.71166E+00_JPRB, 0.70811E+00_JPRB, 0.70489E+00_JPRB, 0.70283E+00_JPRB, 0.70696E+00_JPRB, 0.71152E+00_JPRB, 0.71659E+00_JPRB ,&
&  0.72205E+00_JPRB, 0.72501E+00_JPRB, 0.72912E+00_JPRB, 0.73174E+00_JPRB, 0.73156E+00_JPRB, 0.73407E+00_JPRB, 0.73564E+00_JPRB ,&
&  0.73489E+00_JPRB, 0.73188E+00_JPRB, 0.72968E+00_JPRB, 0.72912E+00_JPRB, 0.72888E+00_JPRB, 0.72823E+00_JPRB, 0.72721E+00_JPRB ,&
&  0.72676E+00_JPRB, 0.72596E+00_JPRB, 0.72458E+00_JPRB, 0.72254E+00_JPRB, 0.71977E+00_JPRB, 0.71660E+00_JPRB, 0.71303E+00_JPRB ,&
&  0.71022E+00_JPRB, 0.70695E+00_JPRB, 0.70380E+00_JPRB, 0.70069E+00_JPRB, 0.69837E+00_JPRB, 0.69546E+00_JPRB, 0.69261E+00_JPRB ,&
&  0.69140E+00_JPRB, 0.69291E+00_JPRB, 0.69526E+00_JPRB, 0.69592E+00_JPRB, 0.69655E+00_JPRB, 0.69664E+00_JPRB, 0.69793E+00_JPRB ,&
&  0.69944E+00_JPRB, 0.70265E+00_JPRB, 0.70618E+00_JPRB, 0.71101E+00_JPRB, 0.71558E+00_JPRB, 0.72007E+00_JPRB, 0.72364E+00_JPRB ,&
&  0.72683E+00_JPRB, 0.72893E+00_JPRB, 0.73025E+00_JPRB, 0.73093E+00_JPRB, 0.73180E+00_JPRB, 0.73229E+00_JPRB, 0.73217E+00_JPRB ,&
&  0.73178E+00_JPRB, 0.73164E+00_JPRB, 0.73128E+00_JPRB, 0.73066E+00_JPRB, 0.73047E+00_JPRB, 0.72998E+00_JPRB, 0.72998E+00_JPRB ,&
&  0.73008E+00_JPRB, 0.73096E+00_JPRB, 0.73194E+00_JPRB, 0.73362E+00_JPRB, 0.73558E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.72912E+00_JPRB, 0.73247E+00_JPRB, 0.73514E+00_JPRB, 0.73622E+00_JPRB, 0.73657E+00_JPRB, 0.73685E+00_JPRB, 0.73662E+00_JPRB ,&
&  0.73616E+00_JPRB, 0.73525E+00_JPRB, 0.73521E+00_JPRB, 0.73485E+00_JPRB, 0.73437E+00_JPRB, 0.73802E+00_JPRB, 0.73860E+00_JPRB ,&
&  0.73748E+00_JPRB, 0.73522E+00_JPRB, 0.73321E+00_JPRB, 0.72996E+00_JPRB, 0.72560E+00_JPRB, 0.71980E+00_JPRB, 0.70897E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.69567E+00_JPRB, 0.68808E+00_JPRB, 0.68658E+00_JPRB, 0.68490E+00_JPRB, 0.66005E+00_JPRB, 0.66041E+00_JPRB ,&
&  0.68622E+00_JPRB, 0.69658E+00_JPRB, 0.69946E+00_JPRB, 0.70354E+00_JPRB, 0.67184E+00_JPRB, 0.63827E+00_JPRB, 0.61676E+00_JPRB ,&
&  0.58297E+00_JPRB, 0.61368E+00_JPRB, 0.63175E+00_JPRB, 0.63653E+00_JPRB, 0.63376E+00_JPRB, 0.62760E+00_JPRB, 0.64805E+00_JPRB ,&
&  0.69789E+00_JPRB, 0.70676E+00_JPRB, 0.71894E+00_JPRB, 0.73232E+00_JPRB, 0.74545E+00_JPRB, 0.75419E+00_JPRB, 0.76431E+00_JPRB ,&
&  0.76942E+00_JPRB, 0.77263E+00_JPRB, 0.77067E+00_JPRB, 0.76641E+00_JPRB, 0.75674E+00_JPRB, 0.74893E+00_JPRB, 0.74133E+00_JPRB ,&
&  0.73714E+00_JPRB, 0.73261E+00_JPRB, 0.72861E+00_JPRB, 0.72255E+00_JPRB, 0.71598E+00_JPRB, 0.70860E+00_JPRB, 0.70126E+00_JPRB ,&
&  0.69394E+00_JPRB, 0.68622E+00_JPRB, 0.68158E+00_JPRB, 0.67816E+00_JPRB, 0.67975E+00_JPRB, 0.68010E+00_JPRB, 0.68367E+00_JPRB ,&
&  0.68696E+00_JPRB, 0.69195E+00_JPRB, 0.69751E+00_JPRB, 0.70494E+00_JPRB, 0.71088E+00_JPRB, 0.71551E+00_JPRB, 0.71827E+00_JPRB ,&
&  0.72050E+00_JPRB, 0.72151E+00_JPRB, 0.72190E+00_JPRB, 0.72145E+00_JPRB, 0.72156E+00_JPRB, 0.72190E+00_JPRB, 0.72179E+00_JPRB ,&
&  0.72225E+00_JPRB, 0.72274E+00_JPRB, 0.72349E+00_JPRB, 0.72378E+00_JPRB, 0.72371E+00_JPRB, 0.72205E+00_JPRB, 0.72015E+00_JPRB ,&
&  0.71822E+00_JPRB, 0.71837E+00_JPRB, 0.71935E+00_JPRB, 0.72199E+00_JPRB, 0.72492E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.74075E+00_JPRB, 0.74156E+00_JPRB, 0.74299E+00_JPRB, 0.74386E+00_JPRB, 0.74423E+00_JPRB, 0.74408E+00_JPRB, 0.74352E+00_JPRB ,&
&  0.74328E+00_JPRB, 0.74342E+00_JPRB, 0.74297E+00_JPRB, 0.74161E+00_JPRB, 0.74162E+00_JPRB, 0.74215E+00_JPRB, 0.74207E+00_JPRB ,&
&  0.73959E+00_JPRB, 0.73627E+00_JPRB, 0.73354E+00_JPRB, 0.72924E+00_JPRB, 0.72505E+00_JPRB, 0.72115E+00_JPRB, 0.71167E+00_JPRB ,&
&  0.70543E+00_JPRB, 0.69572E+00_JPRB, 0.66024E+00_JPRB, 0.66438E+00_JPRB, 0.65805E+00_JPRB, 0.65724E+00_JPRB, 0.66139E+00_JPRB ,&
&  0.65043E+00_JPRB, 0.64876E+00_JPRB, 0.63441E+00_JPRB, 0.61312E+00_JPRB, 0.60268E+00_JPRB, 0.58426E+00_JPRB, 0.58540E+00_JPRB ,&
&  0.58195E+00_JPRB, 0.58809E+00_JPRB, 0.60180E+00_JPRB, 0.60584E+00_JPRB, 0.60195E+00_JPRB, 0.59870E+00_JPRB, 0.60835E+00_JPRB ,&
&  0.63598E+00_JPRB, 0.67725E+00_JPRB, 0.73097E+00_JPRB, 0.75934E+00_JPRB, 0.78760E+00_JPRB, 0.81073E+00_JPRB, 0.83197E+00_JPRB ,&
&  0.84757E+00_JPRB, 0.85964E+00_JPRB, 0.86041E+00_JPRB, 0.85858E+00_JPRB, 0.84691E+00_JPRB, 0.82910E+00_JPRB, 0.80675E+00_JPRB ,&
&  0.78522E+00_JPRB, 0.76953E+00_JPRB, 0.75791E+00_JPRB, 0.75158E+00_JPRB, 0.74889E+00_JPRB, 0.74662E+00_JPRB, 0.74557E+00_JPRB ,&
&  0.74025E+00_JPRB, 0.72489E+00_JPRB, 0.71246E+00_JPRB, 0.69975E+00_JPRB, 0.69654E+00_JPRB, 0.69532E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.70501E+00_JPRB, 0.71548E+00_JPRB, 0.72753E+00_JPRB, 0.73133E+00_JPRB, 0.73744E+00_JPRB, 0.74495E+00_JPRB, 0.74393E+00_JPRB ,&
&  0.74311E+00_JPRB, 0.74094E+00_JPRB, 0.73686E+00_JPRB, 0.73196E+00_JPRB, 0.72660E+00_JPRB, 0.72174E+00_JPRB, 0.71811E+00_JPRB ,&
&  0.71722E+00_JPRB, 0.71875E+00_JPRB, 0.72434E+00_JPRB, 0.73063E+00_JPRB, 0.73788E+00_JPRB, 0.74209E+00_JPRB, 0.74223E+00_JPRB ,&
&  0.73849E+00_JPRB, 0.73661E+00_JPRB, 0.73533E+00_JPRB, 0.73749E+00_JPRB, 0.73943E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.76084E+00_JPRB, 0.76300E+00_JPRB, 0.76790E+00_JPRB, 0.77359E+00_JPRB, 0.77838E+00_JPRB, 0.77951E+00_JPRB, 0.77179E+00_JPRB ,&
&  0.76483E+00_JPRB, 0.75794E+00_JPRB, 0.75247E+00_JPRB, 0.74867E+00_JPRB, 0.74704E+00_JPRB, 0.74374E+00_JPRB, 0.74011E+00_JPRB ,&
&  0.73714E+00_JPRB, 0.73600E+00_JPRB, 0.73627E+00_JPRB, 0.73616E+00_JPRB, 0.73801E+00_JPRB, 0.73735E+00_JPRB, 0.73442E+00_JPRB ,&
&  0.73160E+00_JPRB, 0.71751E+00_JPRB, 0.68937E+00_JPRB, 0.68065E+00_JPRB, 0.67061E+00_JPRB, 0.67150E+00_JPRB, 0.67174E+00_JPRB ,&
&  0.66578E+00_JPRB, 0.60075E+00_JPRB, 0.61102E+00_JPRB, 0.59553E+00_JPRB, 0.58680E+00_JPRB, 0.58752E+00_JPRB, 0.57247E+00_JPRB ,&
&  0.56290E+00_JPRB, 0.55063E+00_JPRB, 0.55073E+00_JPRB, 0.55821E+00_JPRB, 0.55649E+00_JPRB, 0.56658E+00_JPRB, 0.58082E+00_JPRB ,&
&  0.60565E+00_JPRB, 0.71875E+00_JPRB, 0.77059E+00_JPRB, 0.80088E+00_JPRB, 0.82639E+00_JPRB, 0.85554E+00_JPRB, 0.88370E+00_JPRB ,&
&  0.90569E+00_JPRB, 0.93263E+00_JPRB, 0.94427E+00_JPRB, 0.93956E+00_JPRB, 0.94728E+00_JPRB, 0.93845E+00_JPRB, 0.91582E+00_JPRB ,&
&  0.88191E+00_JPRB, 0.83708E+00_JPRB, 0.80676E+00_JPRB, 0.78844E+00_JPRB, 0.77498E+00_JPRB, 0.76454E+00_JPRB, 0.75790E+00_JPRB ,&
&  0.76208E+00_JPRB, 0.75666E+00_JPRB, 0.74887E+00_JPRB, 0.73379E+00_JPRB, 0.72591E+00_JPRB, 0.72193E+00_JPRB, 0.72420E+00_JPRB ,&
&  0.72912E+00_JPRB, 0.74029E+00_JPRB, 0.75341E+00_JPRB, 0.76363E+00_JPRB, 0.76880E+00_JPRB, 0.76777E+00_JPRB, 0.77193E+00_JPRB ,&
&  0.77099E+00_JPRB, 0.76713E+00_JPRB, 0.76272E+00_JPRB, 0.75807E+00_JPRB, 0.75366E+00_JPRB, 0.75116E+00_JPRB, 0.75440E+00_JPRB ,&
&  0.75853E+00_JPRB, 0.76118E+00_JPRB, 0.76205E+00_JPRB, 0.76466E+00_JPRB, 0.76940E+00_JPRB, 0.77357E+00_JPRB, 0.77511E+00_JPRB ,&
&  0.76926E+00_JPRB, 0.76319E+00_JPRB, 0.75852E+00_JPRB, 0.75909E+00_JPRB, 0.75977E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.75502E+00_JPRB, 0.76531E+00_JPRB, 0.78339E+00_JPRB, 0.79840E+00_JPRB, 0.80844E+00_JPRB, 0.80761E+00_JPRB, 0.79552E+00_JPRB ,&
&  0.78375E+00_JPRB, 0.77515E+00_JPRB, 0.76654E+00_JPRB, 0.75874E+00_JPRB, 0.74998E+00_JPRB, 0.74331E+00_JPRB, 0.73856E+00_JPRB ,&
&  0.73605E+00_JPRB, 0.73658E+00_JPRB, 0.73724E+00_JPRB, 0.73781E+00_JPRB, 0.73971E+00_JPRB, 0.74242E+00_JPRB, 0.74240E+00_JPRB ,&
&  0.74167E+00_JPRB, 0.73038E+00_JPRB, 0.70573E+00_JPRB, 0.68756E+00_JPRB, 0.68141E+00_JPRB, 0.67007E+00_JPRB, 0.67466E+00_JPRB ,&
&  0.67457E+00_JPRB, 0.67314E+00_JPRB, 0.66521E+00_JPRB, 0.66117E+00_JPRB, 0.63706E+00_JPRB, 0.55333E+00_JPRB, 0.55838E+00_JPRB ,&
&  0.54514E+00_JPRB, 0.52681E+00_JPRB, 0.52358E+00_JPRB, 0.52313E+00_JPRB, 0.52458E+00_JPRB, 0.55019E+00_JPRB, 0.57780E+00_JPRB ,&
&  0.60352E+00_JPRB, 0.72983E+00_JPRB, 0.76480E+00_JPRB, 0.80264E+00_JPRB, 0.83660E+00_JPRB, 0.87759E+00_JPRB, 0.92022E+00_JPRB ,&
&  0.94728E+00_JPRB, 0.97651E+00_JPRB, 0.99926E+00_JPRB, 0.10159E+01_JPRB, 0.10218E+01_JPRB, 0.10256E+01_JPRB, 0.10283E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.93729E+00_JPRB, 0.88632E+00_JPRB, 0.85676E+00_JPRB, 0.82928E+00_JPRB, 0.81087E+00_JPRB, 0.79618E+00_JPRB ,&
&  0.79186E+00_JPRB, 0.77550E+00_JPRB, 0.77365E+00_JPRB, 0.76418E+00_JPRB, 0.75906E+00_JPRB, 0.75528E+00_JPRB, 0.75071E+00_JPRB ,&
&  0.74309E+00_JPRB, 0.74662E+00_JPRB, 0.76261E+00_JPRB, 0.77072E+00_JPRB, 0.77633E+00_JPRB, 0.77149E+00_JPRB, 0.76299E+00_JPRB ,&
&  0.74838E+00_JPRB, 0.74614E+00_JPRB, 0.73960E+00_JPRB, 0.73847E+00_JPRB, 0.74061E+00_JPRB, 0.75233E+00_JPRB, 0.76752E+00_JPRB ,&
&  0.77726E+00_JPRB, 0.78044E+00_JPRB, 0.78004E+00_JPRB, 0.78062E+00_JPRB, 0.77920E+00_JPRB, 0.77598E+00_JPRB, 0.76973E+00_JPRB ,&
&  0.76377E+00_JPRB, 0.76078E+00_JPRB, 0.75472E+00_JPRB, 0.75076E+00_JPRB, 0.75068E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.74634E+00_JPRB, 0.77067E+00_JPRB, 0.79921E+00_JPRB, 0.81808E+00_JPRB, 0.82578E+00_JPRB, 0.81293E+00_JPRB, 0.79440E+00_JPRB ,&
&  0.77984E+00_JPRB, 0.76949E+00_JPRB, 0.76110E+00_JPRB, 0.75549E+00_JPRB, 0.74911E+00_JPRB, 0.74278E+00_JPRB, 0.73762E+00_JPRB ,&
&  0.73409E+00_JPRB, 0.72273E+00_JPRB, 0.71968E+00_JPRB, 0.72389E+00_JPRB, 0.72647E+00_JPRB, 0.73220E+00_JPRB, 0.73692E+00_JPRB ,&
&  0.73388E+00_JPRB, 0.72449E+00_JPRB, 0.71619E+00_JPRB, 0.69706E+00_JPRB, 0.67859E+00_JPRB, 0.67374E+00_JPRB, 0.68104E+00_JPRB ,&
&  0.66528E+00_JPRB, 0.64005E+00_JPRB, 0.66369E+00_JPRB, 0.68184E+00_JPRB, 0.64550E+00_JPRB, 0.60947E+00_JPRB, 0.56455E+00_JPRB ,&
&  0.55070E+00_JPRB, 0.52186E+00_JPRB, 0.51490E+00_JPRB, 0.51407E+00_JPRB, 0.52369E+00_JPRB, 0.54530E+00_JPRB, 0.58906E+00_JPRB ,&
&  0.63992E+00_JPRB, 0.72979E+00_JPRB, 0.76172E+00_JPRB, 0.80925E+00_JPRB, 0.84895E+00_JPRB, 0.88376E+00_JPRB, 0.90855E+00_JPRB ,&
&  0.94833E+00_JPRB, 0.10046E+01_JPRB, 0.10422E+01_JPRB, 0.10546E+01_JPRB, 0.10724E+01_JPRB, 0.10993E+01_JPRB, 0.11502E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.10744E+01_JPRB, 0.98688E+00_JPRB, 0.93050E+00_JPRB, 0.89472E+00_JPRB, 0.88249E+00_JPRB, 0.83814E+00_JPRB ,&
&  0.82373E+00_JPRB, 0.81984E+00_JPRB, 0.80939E+00_JPRB, 0.80495E+00_JPRB, 0.80072E+00_JPRB, 0.79615E+00_JPRB, 0.78783E+00_JPRB ,&
&  0.77662E+00_JPRB, 0.78242E+00_JPRB, 0.84947E+00_JPRB, 0.86501E+00_JPRB, 0.85017E+00_JPRB, 0.80434E+00_JPRB, 0.75627E+00_JPRB ,&
&  0.72311E+00_JPRB, 0.71031E+00_JPRB, 0.70596E+00_JPRB, 0.70477E+00_JPRB, 0.71400E+00_JPRB, 0.72672E+00_JPRB, 0.74593E+00_JPRB ,&
&  0.74683E+00_JPRB, 0.75759E+00_JPRB, 0.76828E+00_JPRB, 0.77427E+00_JPRB, 0.77409E+00_JPRB, 0.76826E+00_JPRB, 0.76395E+00_JPRB ,&
&  0.76027E+00_JPRB, 0.75627E+00_JPRB, 0.74895E+00_JPRB, 0.74063E+00_JPRB, 0.73853E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.72637E+00_JPRB, 0.76644E+00_JPRB, 0.81403E+00_JPRB, 0.84388E+00_JPRB, 0.84376E+00_JPRB, 0.81671E+00_JPRB, 0.78720E+00_JPRB ,&
&  0.76757E+00_JPRB, 0.75831E+00_JPRB, 0.75178E+00_JPRB, 0.74634E+00_JPRB, 0.74158E+00_JPRB, 0.73659E+00_JPRB, 0.73154E+00_JPRB ,&
&  0.72769E+00_JPRB, 0.72399E+00_JPRB, 0.71740E+00_JPRB, 0.70877E+00_JPRB, 0.70452E+00_JPRB, 0.70692E+00_JPRB, 0.71297E+00_JPRB ,&
&  0.71863E+00_JPRB, 0.71699E+00_JPRB, 0.71586E+00_JPRB, 0.70487E+00_JPRB, 0.69080E+00_JPRB, 0.67442E+00_JPRB, 0.66299E+00_JPRB ,&
&  0.65901E+00_JPRB, 0.66302E+00_JPRB, 0.65545E+00_JPRB, 0.67811E+00_JPRB, 0.67175E+00_JPRB, 0.63079E+00_JPRB, 0.58867E+00_JPRB ,&
&  0.56207E+00_JPRB, 0.53020E+00_JPRB, 0.51800E+00_JPRB, 0.52346E+00_JPRB, 0.55633E+00_JPRB, 0.59497E+00_JPRB, 0.64888E+00_JPRB ,&
&  0.74127E+00_JPRB, 0.80774E+00_JPRB, 0.85361E+00_JPRB, 0.89235E+00_JPRB, 0.92258E+00_JPRB, 0.95622E+00_JPRB, 0.99608E+00_JPRB ,&
&  0.10482E+01_JPRB, 0.10973E+01_JPRB, 0.11271E+01_JPRB, 0.11088E+01_JPRB, 0.11054E+01_JPRB, 0.10531E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.15312E+01_JPRB, 0.13046E+01_JPRB, 0.11470E+01_JPRB, 0.10644E+01_JPRB, 0.10146E+01_JPRB, 0.97315E+00_JPRB, 0.93479E+00_JPRB ,&
&  0.89613E+00_JPRB, 0.87370E+00_JPRB, 0.86248E+00_JPRB, 0.82681E+00_JPRB, 0.79201E+00_JPRB, 0.79105E+00_JPRB, 0.83469E+00_JPRB ,&
&  0.94324E+00_JPRB, 0.11022E+01_JPRB, 0.13015E+01_JPRB, 0.94932E+00_JPRB, 0.83906E+00_JPRB, 0.76000E+00_JPRB, 0.72295E+00_JPRB ,&
&  0.71227E+00_JPRB, 0.71154E+00_JPRB, 0.71405E+00_JPRB, 0.71123E+00_JPRB, 0.70968E+00_JPRB, 0.71369E+00_JPRB, 0.71578E+00_JPRB ,&
&  0.71417E+00_JPRB, 0.72511E+00_JPRB, 0.74060E+00_JPRB, 0.75276E+00_JPRB, 0.76436E+00_JPRB, 0.76671E+00_JPRB, 0.76610E+00_JPRB ,&
&  0.76359E+00_JPRB, 0.75498E+00_JPRB, 0.74393E+00_JPRB, 0.72475E+00_JPRB, 0.71567E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.72142E+00_JPRB, 0.76691E+00_JPRB, 0.82918E+00_JPRB, 0.86551E+00_JPRB, 0.86241E+00_JPRB, 0.82034E+00_JPRB, 0.76727E+00_JPRB ,&
&  0.73054E+00_JPRB, 0.70813E+00_JPRB, 0.70090E+00_JPRB, 0.69776E+00_JPRB, 0.69764E+00_JPRB, 0.69810E+00_JPRB, 0.69705E+00_JPRB ,&
&  0.69470E+00_JPRB, 0.69305E+00_JPRB, 0.69075E+00_JPRB, 0.68886E+00_JPRB, 0.68564E+00_JPRB, 0.68398E+00_JPRB, 0.68596E+00_JPRB ,&
&  0.69036E+00_JPRB, 0.69144E+00_JPRB, 0.69102E+00_JPRB, 0.68930E+00_JPRB, 0.69112E+00_JPRB, 0.69626E+00_JPRB, 0.69453E+00_JPRB ,&
&  0.67598E+00_JPRB, 0.66179E+00_JPRB, 0.66907E+00_JPRB, 0.68625E+00_JPRB, 0.68411E+00_JPRB, 0.65181E+00_JPRB, 0.60394E+00_JPRB ,&
&  0.57029E+00_JPRB, 0.54773E+00_JPRB, 0.56613E+00_JPRB, 0.61224E+00_JPRB, 0.66841E+00_JPRB, 0.78652E+00_JPRB, 0.87637E+00_JPRB ,&
&  0.95064E+00_JPRB, 0.99197E+00_JPRB, 0.10179E+01_JPRB, 0.10452E+01_JPRB, 0.10986E+01_JPRB, 0.11546E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.12360E+01_JPRB, 0.12646E+01_JPRB, 0.12871E+01_JPRB, 0.10400E+01_JPRB, 0.10210E+01_JPRB, 0.11203E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.13024E+01_JPRB, 0.12323E+01_JPRB, 0.11922E+01_JPRB, 0.12076E+01_JPRB, 0.11356E+01_JPRB, 0.10823E+01_JPRB, 0.10430E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.99220E+00_JPRB, 0.92938E+00_JPRB, 0.86256E+00_JPRB, 0.85708E+00_JPRB, 0.86133E+00_JPRB, 0.87525E+00_JPRB ,&
&  0.91323E+00_JPRB, 0.93957E+00_JPRB, 0.87229E+00_JPRB, 0.72166E+00_JPRB, 0.68381E+00_JPRB, 0.67054E+00_JPRB, 0.67415E+00_JPRB ,&
&  0.68159E+00_JPRB, 0.68954E+00_JPRB, 0.70045E+00_JPRB, 0.70364E+00_JPRB, 0.70481E+00_JPRB, 0.69965E+00_JPRB, 0.69245E+00_JPRB ,&
&  0.68397E+00_JPRB, 0.68613E+00_JPRB, 0.69736E+00_JPRB, 0.69964E+00_JPRB, 0.71701E+00_JPRB, 0.73275E+00_JPRB, 0.74794E+00_JPRB ,&
&  0.74548E+00_JPRB, 0.73899E+00_JPRB, 0.73093E+00_JPRB, 0.71283E+00_JPRB, 0.70159E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.76334E+00_JPRB, 0.80902E+00_JPRB, 0.84893E+00_JPRB, 0.87086E+00_JPRB, 0.86350E+00_JPRB, 0.83238E+00_JPRB, 0.76423E+00_JPRB ,&
&  0.73795E+00_JPRB, 0.70004E+00_JPRB, 0.68546E+00_JPRB, 0.67327E+00_JPRB, 0.66322E+00_JPRB, 0.65547E+00_JPRB, 0.65382E+00_JPRB ,&
&  0.65760E+00_JPRB, 0.66158E+00_JPRB, 0.66424E+00_JPRB, 0.66767E+00_JPRB, 0.66808E+00_JPRB, 0.66977E+00_JPRB, 0.67236E+00_JPRB ,&
&  0.67396E+00_JPRB, 0.67532E+00_JPRB, 0.67852E+00_JPRB, 0.68270E+00_JPRB, 0.68843E+00_JPRB, 0.69321E+00_JPRB, 0.69072E+00_JPRB ,&
&  0.67839E+00_JPRB, 0.66652E+00_JPRB, 0.67640E+00_JPRB, 0.69347E+00_JPRB, 0.69188E+00_JPRB, 0.67116E+00_JPRB, 0.62032E+00_JPRB ,&
&  0.58571E+00_JPRB, 0.59780E+00_JPRB, 0.68656E+00_JPRB, 0.82880E+00_JPRB, 0.89586E+00_JPRB, 0.96102E+00_JPRB, 0.99917E+00_JPRB ,&
&  0.95637E+00_JPRB, 0.96530E+00_JPRB, 0.10285E+01_JPRB, 0.12081E+01_JPRB, 0.12463E+01_JPRB, 0.12974E+01_JPRB, 0.13593E+01_JPRB ,&
&  0.14526E+01_JPRB, 0.14715E+01_JPRB, 0.14543E+01_JPRB, 0.12626E+01_JPRB, 0.14345E+01_JPRB, 0.17298E+01_JPRB, 0.15549E+01_JPRB ,&
&  0.15131E+01_JPRB, 0.14683E+01_JPRB, 0.14953E+01_JPRB, 0.14014E+01_JPRB, 0.12645E+01_JPRB, 0.11850E+01_JPRB, 0.11636E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.10901E+01_JPRB, 0.97833E+00_JPRB, 0.94160E+00_JPRB, 0.94904E+00_JPRB, 0.92324E+00_JPRB, 0.89970E+00_JPRB ,&
&  0.87310E+00_JPRB, 0.82024E+00_JPRB, 0.77677E+00_JPRB, 0.72899E+00_JPRB, 0.69731E+00_JPRB, 0.69079E+00_JPRB, 0.69399E+00_JPRB ,&
&  0.69783E+00_JPRB, 0.70128E+00_JPRB, 0.71037E+00_JPRB, 0.72135E+00_JPRB, 0.72306E+00_JPRB, 0.70564E+00_JPRB, 0.70180E+00_JPRB ,&
&  0.69560E+00_JPRB, 0.68325E+00_JPRB, 0.67431E+00_JPRB, 0.68164E+00_JPRB, 0.68980E+00_JPRB, 0.70494E+00_JPRB, 0.72280E+00_JPRB ,&
&  0.73613E+00_JPRB, 0.74411E+00_JPRB, 0.74052E+00_JPRB, 0.74273E+00_JPRB, 0.74097E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.80740E+00_JPRB, 0.82841E+00_JPRB, 0.85634E+00_JPRB, 0.87730E+00_JPRB, 0.88781E+00_JPRB, 0.83057E+00_JPRB, 0.77610E+00_JPRB ,&
&  0.79122E+00_JPRB, 0.72077E+00_JPRB, 0.68474E+00_JPRB, 0.67269E+00_JPRB, 0.66867E+00_JPRB, 0.65093E+00_JPRB, 0.63886E+00_JPRB ,&
&  0.63509E+00_JPRB, 0.64248E+00_JPRB, 0.65296E+00_JPRB, 0.65963E+00_JPRB, 0.66466E+00_JPRB, 0.66899E+00_JPRB, 0.67095E+00_JPRB ,&
&  0.67308E+00_JPRB, 0.67605E+00_JPRB, 0.67963E+00_JPRB, 0.68261E+00_JPRB, 0.68358E+00_JPRB, 0.68632E+00_JPRB, 0.68514E+00_JPRB ,&
&  0.67589E+00_JPRB, 0.67026E+00_JPRB, 0.68105E+00_JPRB, 0.70167E+00_JPRB, 0.71036E+00_JPRB, 0.71036E+00_JPRB, 0.68563E+00_JPRB ,&
&  0.62758E+00_JPRB, 0.68269E+00_JPRB, 0.88108E+00_JPRB, 0.95172E+00_JPRB, 0.99982E+00_JPRB, 0.10613E+01_JPRB, 0.11147E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11923E+01_JPRB, 0.12237E+01_JPRB, 0.12685E+01_JPRB, 0.13272E+01_JPRB, 0.13916E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.17223E+01_JPRB, 0.13969E+01_JPRB, 0.13089E+01_JPRB, 0.13855E+01_JPRB, 0.19004E+01_JPRB, 0.19942E+01_JPRB, 0.17821E+01_JPRB ,&
&  0.17217E+01_JPRB, 0.16461E+01_JPRB, 0.15789E+01_JPRB, 0.14587E+01_JPRB, 0.13662E+01_JPRB, 0.13329E+01_JPRB, 0.13112E+01_JPRB ,&
&  0.12305E+01_JPRB, 0.11899E+01_JPRB, 0.10638E+01_JPRB, 0.10380E+01_JPRB, 0.10361E+01_JPRB, 0.98693E+00_JPRB, 0.91651E+00_JPRB ,&
&  0.85850E+00_JPRB, 0.81333E+00_JPRB, 0.78110E+00_JPRB, 0.75696E+00_JPRB, 0.73808E+00_JPRB, 0.72713E+00_JPRB, 0.71962E+00_JPRB ,&
&  0.71667E+00_JPRB, 0.71732E+00_JPRB, 0.71311E+00_JPRB, 0.72377E+00_JPRB, 0.73117E+00_JPRB, 0.72030E+00_JPRB, 0.72839E+00_JPRB ,&
&  0.73184E+00_JPRB, 0.72352E+00_JPRB, 0.70574E+00_JPRB, 0.71370E+00_JPRB, 0.73256E+00_JPRB, 0.76449E+00_JPRB, 0.79455E+00_JPRB ,&
&  0.83238E+00_JPRB, 0.82746E+00_JPRB, 0.79442E+00_JPRB, 0.78669E+00_JPRB, 0.78774E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.84335E+00_JPRB, 0.85838E+00_JPRB, 0.86978E+00_JPRB, 0.88822E+00_JPRB, 0.90304E+00_JPRB, 0.85562E+00_JPRB, 0.81971E+00_JPRB ,&
&  0.87795E+00_JPRB, 0.79037E+00_JPRB, 0.83515E+00_JPRB, 0.74609E+00_JPRB, 0.70118E+00_JPRB, 0.67650E+00_JPRB, 0.65838E+00_JPRB ,&
&  0.64331E+00_JPRB, 0.65211E+00_JPRB, 0.66495E+00_JPRB, 0.67601E+00_JPRB, 0.69397E+00_JPRB, 0.68859E+00_JPRB, 0.68327E+00_JPRB ,&
&  0.67935E+00_JPRB, 0.67894E+00_JPRB, 0.68755E+00_JPRB, 0.68739E+00_JPRB, 0.68526E+00_JPRB, 0.68299E+00_JPRB, 0.67299E+00_JPRB ,&
&  0.65313E+00_JPRB, 0.65373E+00_JPRB, 0.67708E+00_JPRB, 0.69971E+00_JPRB, 0.72315E+00_JPRB, 0.73480E+00_JPRB, 0.74949E+00_JPRB ,&
&  0.76725E+00_JPRB, 0.79373E+00_JPRB, 0.92829E+00_JPRB, 0.97271E+00_JPRB, 0.10211E+01_JPRB, 0.10566E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.11806E+01_JPRB, 0.12043E+01_JPRB, 0.12509E+01_JPRB, 0.13411E+01_JPRB, 0.14435E+01_JPRB, 0.16284E+01_JPRB ,&
&  0.19046E+01_JPRB, 0.14678E+01_JPRB, 0.17221E+01_JPRB, 0.17187E+01_JPRB, 0.19695E+01_JPRB, 0.21568E+01_JPRB, 0.21490E+01_JPRB ,&
&  0.23279E+01_JPRB, 0.20464E+01_JPRB, 0.18431E+01_JPRB, 0.16337E+01_JPRB, 0.14725E+01_JPRB, 0.14360E+01_JPRB, 0.14109E+01_JPRB ,&
&  0.14387E+01_JPRB, 0.14015E+01_JPRB, 0.12131E+01_JPRB, 0.11251E+01_JPRB, 0.10947E+01_JPRB, 0.10735E+01_JPRB, 0.99762E+00_JPRB ,&
&  0.95590E+00_JPRB, 0.92854E+00_JPRB, 0.88855E+00_JPRB, 0.84569E+00_JPRB, 0.81339E+00_JPRB, 0.79550E+00_JPRB, 0.77166E+00_JPRB ,&
&  0.76770E+00_JPRB, 0.75593E+00_JPRB, 0.71854E+00_JPRB, 0.71685E+00_JPRB, 0.71993E+00_JPRB, 0.69796E+00_JPRB, 0.70836E+00_JPRB ,&
&  0.73279E+00_JPRB, 0.73920E+00_JPRB, 0.78624E+00_JPRB, 0.85115E+00_JPRB, 0.87227E+00_JPRB, 0.87973E+00_JPRB, 0.96281E+00_JPRB ,&
&  0.89726E+00_JPRB, 0.89746E+00_JPRB, 0.89238E+00_JPRB, 0.85665E+00_JPRB, 0.83926E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.89471E+00_JPRB, 0.88896E+00_JPRB, 0.88598E+00_JPRB, 0.90004E+00_JPRB, 0.91609E+00_JPRB, 0.92208E+00_JPRB, 0.89034E+00_JPRB ,&
&  0.94124E+00_JPRB, 0.90973E+00_JPRB, 0.89163E+00_JPRB, 0.88330E+00_JPRB, 0.86522E+00_JPRB, 0.77215E+00_JPRB, 0.67626E+00_JPRB ,&
&  0.65282E+00_JPRB, 0.65414E+00_JPRB, 0.68087E+00_JPRB, 0.69023E+00_JPRB, 0.70523E+00_JPRB, 0.70504E+00_JPRB, 0.69707E+00_JPRB ,&
&  0.69358E+00_JPRB, 0.69067E+00_JPRB, 0.70071E+00_JPRB, 0.70232E+00_JPRB, 0.69453E+00_JPRB, 0.69390E+00_JPRB, 0.69059E+00_JPRB ,&
&  0.66004E+00_JPRB, 0.64614E+00_JPRB, 0.65439E+00_JPRB, 0.66064E+00_JPRB, 0.72186E+00_JPRB, 0.75076E+00_JPRB, 0.77639E+00_JPRB ,&
&  0.80417E+00_JPRB, 0.84336E+00_JPRB, 0.90884E+00_JPRB, 0.96289E+00_JPRB, 0.99957E+00_JPRB, 0.10271E+01_JPRB, 0.10680E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.11449E+01_JPRB, 0.11988E+01_JPRB, 0.12901E+01_JPRB, 0.14283E+01_JPRB, 0.14047E+01_JPRB, 0.17227E+01_JPRB ,&
&  0.18601E+01_JPRB, 0.20618E+01_JPRB, 0.22662E+01_JPRB, 0.21280E+01_JPRB, 0.23259E+01_JPRB, 0.24443E+01_JPRB, 0.23550E+01_JPRB ,&
&  0.22822E+01_JPRB, 0.20287E+01_JPRB, 0.19753E+01_JPRB, 0.19355E+01_JPRB, 0.19302E+01_JPRB, 0.18007E+01_JPRB, 0.16721E+01_JPRB ,&
&  0.17537E+01_JPRB, 0.19801E+01_JPRB, 0.15822E+01_JPRB, 0.14418E+01_JPRB, 0.13248E+01_JPRB, 0.12872E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.11964E+01_JPRB, 0.11545E+01_JPRB, 0.10942E+01_JPRB, 0.97897E+00_JPRB, 0.92720E+00_JPRB, 0.86690E+00_JPRB ,&
&  0.83722E+00_JPRB, 0.79652E+00_JPRB, 0.73811E+00_JPRB, 0.72374E+00_JPRB, 0.70649E+00_JPRB, 0.69386E+00_JPRB, 0.71361E+00_JPRB ,&
&  0.78218E+00_JPRB, 0.86010E+00_JPRB, 0.10258E+01_JPRB, 0.10880E+01_JPRB, 0.11252E+01_JPRB, 0.11694E+01_JPRB, 0.10163E+01_JPRB ,&
&  0.92089E+00_JPRB, 0.99820E+00_JPRB, 0.96805E+00_JPRB, 0.93162E+00_JPRB, 0.90577E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.98683E+00_JPRB, 0.96360E+00_JPRB, 0.94064E+00_JPRB, 0.93869E+00_JPRB, 0.95009E+00_JPRB, 0.94881E+00_JPRB, 0.95508E+00_JPRB ,&
&  0.97355E+00_JPRB, 0.96456E+00_JPRB, 0.93651E+00_JPRB, 0.92059E+00_JPRB, 0.90048E+00_JPRB, 0.89658E+00_JPRB, 0.79774E+00_JPRB ,&
&  0.71301E+00_JPRB, 0.70209E+00_JPRB, 0.68354E+00_JPRB, 0.69776E+00_JPRB, 0.73583E+00_JPRB, 0.72834E+00_JPRB, 0.72517E+00_JPRB ,&
&  0.71998E+00_JPRB, 0.71408E+00_JPRB, 0.71825E+00_JPRB, 0.72165E+00_JPRB, 0.72979E+00_JPRB, 0.75605E+00_JPRB, 0.76733E+00_JPRB ,&
&  0.71538E+00_JPRB, 0.65694E+00_JPRB, 0.63509E+00_JPRB, 0.64832E+00_JPRB, 0.70445E+00_JPRB, 0.76128E+00_JPRB, 0.79794E+00_JPRB ,&
&  0.82768E+00_JPRB, 0.85995E+00_JPRB, 0.90259E+00_JPRB, 0.95824E+00_JPRB, 0.99058E+00_JPRB, 0.10222E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.11647E+01_JPRB, 0.12482E+01_JPRB, 0.13771E+01_JPRB, 0.13894E+01_JPRB, 0.15101E+01_JPRB, 0.17762E+01_JPRB ,&
&  0.19449E+01_JPRB, 0.21201E+01_JPRB, 0.23277E+01_JPRB, 0.26849E+01_JPRB, 0.28803E+01_JPRB, 0.27537E+01_JPRB, 0.25550E+01_JPRB ,&
&  0.24181E+01_JPRB, 0.21573E+01_JPRB, 0.24245E+01_JPRB, 0.22429E+01_JPRB, 0.21002E+01_JPRB, 0.20966E+01_JPRB, 0.19980E+01_JPRB ,&
&  0.19735E+01_JPRB, 0.20464E+01_JPRB, 0.19756E+01_JPRB, 0.19345E+01_JPRB, 0.17923E+01_JPRB, 0.17621E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.16386E+01_JPRB, 0.14168E+01_JPRB, 0.12215E+01_JPRB, 0.10417E+01_JPRB, 0.10036E+01_JPRB, 0.92859E+00_JPRB ,&
&  0.87440E+00_JPRB, 0.81875E+00_JPRB, 0.79713E+00_JPRB, 0.76787E+00_JPRB, 0.76136E+00_JPRB, 0.79675E+00_JPRB, 0.81637E+00_JPRB ,&
&  0.90089E+00_JPRB, 0.10885E+01_JPRB, 0.11943E+01_JPRB, 0.12617E+01_JPRB, 0.13163E+01_JPRB, 0.13636E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.10676E+01_JPRB, 0.10527E+01_JPRB, 0.10250E+01_JPRB, 0.10022E+01_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.11229E+01_JPRB, 0.10777E+01_JPRB, 0.10365E+01_JPRB, 0.10118E+01_JPRB, 0.99657E+00_JPRB, 0.98866E+00_JPRB, 0.10035E+01_JPRB ,&
&  0.10194E+01_JPRB, 0.10048E+01_JPRB, 0.98210E+00_JPRB, 0.96339E+00_JPRB, 0.94287E+00_JPRB, 0.94277E+00_JPRB, 0.95548E+00_JPRB ,&
&  0.81871E+00_JPRB, 0.78844E+00_JPRB, 0.70609E+00_JPRB, 0.70706E+00_JPRB, 0.82468E+00_JPRB, 0.77308E+00_JPRB, 0.75498E+00_JPRB ,&
&  0.73514E+00_JPRB, 0.74532E+00_JPRB, 0.75160E+00_JPRB, 0.79768E+00_JPRB, 0.84557E+00_JPRB, 0.90154E+00_JPRB, 0.90821E+00_JPRB ,&
&  0.82791E+00_JPRB, 0.73089E+00_JPRB, 0.68808E+00_JPRB, 0.68019E+00_JPRB, 0.72067E+00_JPRB, 0.76118E+00_JPRB, 0.83471E+00_JPRB ,&
&  0.87810E+00_JPRB, 0.91299E+00_JPRB, 0.94752E+00_JPRB, 0.99611E+00_JPRB, 0.10312E+01_JPRB, 0.10732E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.12746E+01_JPRB, 0.13600E+01_JPRB, 0.14765E+01_JPRB, 0.15175E+01_JPRB, 0.16402E+01_JPRB, 0.18659E+01_JPRB ,&
&  0.20348E+01_JPRB, 0.21607E+01_JPRB, 0.24659E+01_JPRB, 0.32600E+01_JPRB, 0.36834E+01_JPRB, 0.36408E+01_JPRB, 0.32430E+01_JPRB ,&
&  0.28280E+01_JPRB, 0.24325E+01_JPRB, 0.23820E+01_JPRB, 0.22505E+01_JPRB, 0.21068E+01_JPRB, 0.22022E+01_JPRB, 0.21412E+01_JPRB ,&
&  0.20527E+01_JPRB, 0.20682E+01_JPRB, 0.20671E+01_JPRB, 0.21261E+01_JPRB, 0.20370E+01_JPRB, 0.20546E+01_JPRB, 0.19990E+01_JPRB ,&
&  0.20253E+01_JPRB, 0.18702E+01_JPRB, 0.14411E+01_JPRB, 0.12334E+01_JPRB, 0.10379E+01_JPRB, 0.97912E+00_JPRB, 0.11197E+01_JPRB ,&
&  0.10133E+01_JPRB, 0.98498E+00_JPRB, 0.98932E+00_JPRB, 0.98403E+00_JPRB, 0.10395E+01_JPRB, 0.12194E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.10656E+01_JPRB, 0.12151E+01_JPRB, 0.14630E+01_JPRB, 0.16500E+01_JPRB, 0.17347E+01_JPRB, 0.16823E+01_JPRB, 0.13613E+01_JPRB ,&
&  0.13104E+01_JPRB, 0.12557E+01_JPRB, 0.12252E+01_JPRB, 0.11795E+01_JPRB, 0.11484E+01_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.13328E+01_JPRB, 0.12765E+01_JPRB, 0.12127E+01_JPRB, 0.11368E+01_JPRB, 0.10931E+01_JPRB, 0.10822E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10876E+01_JPRB, 0.10537E+01_JPRB, 0.10298E+01_JPRB, 0.10099E+01_JPRB, 0.10019E+01_JPRB, 0.10094E+01_JPRB, 0.10203E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.98114E+00_JPRB, 0.83218E+00_JPRB, 0.73648E+00_JPRB, 0.77806E+00_JPRB, 0.75508E+00_JPRB, 0.76290E+00_JPRB ,&
&  0.76034E+00_JPRB, 0.82848E+00_JPRB, 0.86745E+00_JPRB, 0.96242E+00_JPRB, 0.10722E+01_JPRB, 0.11764E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.10757E+01_JPRB, 0.92937E+00_JPRB, 0.83476E+00_JPRB, 0.81025E+00_JPRB, 0.84776E+00_JPRB, 0.82332E+00_JPRB, 0.88850E+00_JPRB ,&
&  0.92880E+00_JPRB, 0.97106E+00_JPRB, 0.10168E+01_JPRB, 0.10554E+01_JPRB, 0.11106E+01_JPRB, 0.11803E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.13379E+01_JPRB, 0.14047E+01_JPRB, 0.14362E+01_JPRB, 0.14910E+01_JPRB, 0.16193E+01_JPRB, 0.17697E+01_JPRB, 0.17789E+01_JPRB ,&
&  0.19357E+01_JPRB, 0.20933E+01_JPRB, 0.23904E+01_JPRB, 0.31546E+01_JPRB, 0.37202E+01_JPRB, 0.38688E+01_JPRB, 0.37063E+01_JPRB ,&
&  0.33770E+01_JPRB, 0.30369E+01_JPRB, 0.29589E+01_JPRB, 0.26778E+01_JPRB, 0.25153E+01_JPRB, 0.25005E+01_JPRB, 0.24509E+01_JPRB ,&
&  0.23396E+01_JPRB, 0.23042E+01_JPRB, 0.23302E+01_JPRB, 0.22728E+01_JPRB, 0.22659E+01_JPRB, 0.22422E+01_JPRB, 0.23051E+01_JPRB ,&
&  0.25365E+01_JPRB, 0.17687E+01_JPRB, 0.14212E+01_JPRB, 0.14717E+01_JPRB, 0.12271E+01_JPRB, 0.10901E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.10888E+01_JPRB, 0.12829E+01_JPRB, 0.13047E+01_JPRB, 0.15489E+01_JPRB, 0.18650E+01_JPRB, 0.16854E+01_JPRB ,&
&  0.16275E+01_JPRB, 0.16232E+01_JPRB, 0.20078E+01_JPRB, 0.23076E+01_JPRB, 0.23532E+01_JPRB, 0.21528E+01_JPRB, 0.18750E+01_JPRB ,&
&  0.16522E+01_JPRB, 0.15373E+01_JPRB, 0.14812E+01_JPRB, 0.14300E+01_JPRB, 0.13903E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.15777E+01_JPRB, 0.15003E+01_JPRB, 0.14344E+01_JPRB, 0.13640E+01_JPRB, 0.13127E+01_JPRB, 0.12731E+01_JPRB, 0.12260E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11153E+01_JPRB, 0.10864E+01_JPRB, 0.10753E+01_JPRB, 0.10763E+01_JPRB, 0.10893E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11146E+01_JPRB, 0.11028E+01_JPRB, 0.91967E+00_JPRB, 0.79450E+00_JPRB, 0.70971E+00_JPRB, 0.72436E+00_JPRB, 0.74589E+00_JPRB ,&
&  0.80442E+00_JPRB, 0.90491E+00_JPRB, 0.10965E+01_JPRB, 0.13379E+01_JPRB, 0.14843E+01_JPRB, 0.15375E+01_JPRB, 0.15374E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.12663E+01_JPRB, 0.10785E+01_JPRB, 0.10464E+01_JPRB, 0.99984E+00_JPRB, 0.94826E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10757E+01_JPRB, 0.11143E+01_JPRB, 0.11748E+01_JPRB, 0.12281E+01_JPRB, 0.12708E+01_JPRB, 0.13128E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.14020E+01_JPRB, 0.14469E+01_JPRB, 0.14884E+01_JPRB, 0.15807E+01_JPRB, 0.17158E+01_JPRB, 0.17649E+01_JPRB ,&
&  0.19703E+01_JPRB, 0.19892E+01_JPRB, 0.20695E+01_JPRB, 0.30072E+01_JPRB, 0.42891E+01_JPRB, 0.43272E+01_JPRB, 0.39482E+01_JPRB ,&
&  0.38552E+01_JPRB, 0.37813E+01_JPRB, 0.38614E+01_JPRB, 0.37683E+01_JPRB, 0.34535E+01_JPRB, 0.33218E+01_JPRB, 0.31557E+01_JPRB ,&
&  0.29849E+01_JPRB, 0.28732E+01_JPRB, 0.27850E+01_JPRB, 0.28221E+01_JPRB, 0.28266E+01_JPRB, 0.28986E+01_JPRB, 0.29924E+01_JPRB ,&
&  0.29820E+01_JPRB, 0.26492E+01_JPRB, 0.18847E+01_JPRB, 0.17977E+01_JPRB, 0.14479E+01_JPRB, 0.13017E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.13470E+01_JPRB, 0.15730E+01_JPRB, 0.16198E+01_JPRB, 0.22641E+01_JPRB, 0.26021E+01_JPRB, 0.26010E+01_JPRB ,&
&  0.25260E+01_JPRB, 0.24234E+01_JPRB, 0.28557E+01_JPRB, 0.30206E+01_JPRB, 0.29784E+01_JPRB, 0.27658E+01_JPRB, 0.24598E+01_JPRB ,&
&  0.21365E+01_JPRB, 0.19358E+01_JPRB, 0.18330E+01_JPRB, 0.17571E+01_JPRB, 0.16638E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.18315E+01_JPRB, 0.17273E+01_JPRB, 0.16310E+01_JPRB, 0.15581E+01_JPRB, 0.14960E+01_JPRB, 0.14252E+01_JPRB, 0.13342E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12012E+01_JPRB, 0.11821E+01_JPRB, 0.11612E+01_JPRB, 0.11495E+01_JPRB, 0.11520E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11648E+01_JPRB, 0.10964E+01_JPRB, 0.90291E+00_JPRB, 0.76833E+00_JPRB, 0.68257E+00_JPRB, 0.66942E+00_JPRB, 0.71693E+00_JPRB ,&
&  0.81807E+00_JPRB, 0.11120E+01_JPRB, 0.15417E+01_JPRB, 0.18121E+01_JPRB, 0.19891E+01_JPRB, 0.20724E+01_JPRB, 0.20200E+01_JPRB ,&
&  0.20484E+01_JPRB, 0.16640E+01_JPRB, 0.15376E+01_JPRB, 0.13421E+01_JPRB, 0.12510E+01_JPRB, 0.12203E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11339E+01_JPRB, 0.11425E+01_JPRB, 0.11595E+01_JPRB, 0.12049E+01_JPRB, 0.12366E+01_JPRB, 0.12854E+01_JPRB, 0.13687E+01_JPRB ,&
&  0.14690E+01_JPRB, 0.15280E+01_JPRB, 0.15089E+01_JPRB, 0.15006E+01_JPRB, 0.17333E+01_JPRB, 0.20278E+01_JPRB, 0.21965E+01_JPRB ,&
&  0.22278E+01_JPRB, 0.24062E+01_JPRB, 0.35657E+01_JPRB, 0.41882E+01_JPRB, 0.44871E+01_JPRB, 0.48999E+01_JPRB, 0.48570E+01_JPRB ,&
&  0.44530E+01_JPRB, 0.40731E+01_JPRB, 0.38886E+01_JPRB, 0.34705E+01_JPRB, 0.39084E+01_JPRB, 0.39888E+01_JPRB, 0.39273E+01_JPRB ,&
&  0.38100E+01_JPRB, 0.36365E+01_JPRB, 0.34928E+01_JPRB, 0.34881E+01_JPRB, 0.35699E+01_JPRB, 0.36950E+01_JPRB, 0.35503E+01_JPRB ,&
&  0.31763E+01_JPRB, 0.33346E+01_JPRB, 0.28561E+01_JPRB, 0.23295E+01_JPRB, 0.19200E+01_JPRB, 0.17862E+01_JPRB, 0.16717E+01_JPRB ,&
&  0.15492E+01_JPRB, 0.15998E+01_JPRB, 0.19740E+01_JPRB, 0.24416E+01_JPRB, 0.31534E+01_JPRB, 0.36577E+01_JPRB, 0.36328E+01_JPRB ,&
&  0.34089E+01_JPRB, 0.37392E+01_JPRB, 0.36092E+01_JPRB, 0.33650E+01_JPRB, 0.31933E+01_JPRB, 0.29375E+01_JPRB, 0.27058E+01_JPRB ,&
&  0.24573E+01_JPRB, 0.22921E+01_JPRB, 0.21762E+01_JPRB, 0.20689E+01_JPRB, 0.19504E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.19593E+01_JPRB, 0.18514E+01_JPRB, 0.17469E+01_JPRB, 0.16545E+01_JPRB, 0.15587E+01_JPRB, 0.14738E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.13204E+01_JPRB, 0.12863E+01_JPRB, 0.12481E+01_JPRB, 0.12162E+01_JPRB, 0.11989E+01_JPRB, 0.11830E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.10484E+01_JPRB, 0.88121E+00_JPRB, 0.81317E+00_JPRB, 0.75317E+00_JPRB, 0.69824E+00_JPRB, 0.72953E+00_JPRB ,&
&  0.92516E+00_JPRB, 0.14733E+01_JPRB, 0.19954E+01_JPRB, 0.23381E+01_JPRB, 0.25238E+01_JPRB, 0.25704E+01_JPRB, 0.24690E+01_JPRB ,&
&  0.22092E+01_JPRB, 0.21829E+01_JPRB, 0.20286E+01_JPRB, 0.18390E+01_JPRB, 0.16360E+01_JPRB, 0.15013E+01_JPRB, 0.14031E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.13053E+01_JPRB, 0.13179E+01_JPRB, 0.13458E+01_JPRB, 0.14059E+01_JPRB, 0.14849E+01_JPRB, 0.15787E+01_JPRB ,&
&  0.16193E+01_JPRB, 0.16398E+01_JPRB, 0.15955E+01_JPRB, 0.16294E+01_JPRB, 0.17390E+01_JPRB, 0.22770E+01_JPRB, 0.25502E+01_JPRB ,&
&  0.31232E+01_JPRB, 0.36145E+01_JPRB, 0.37629E+01_JPRB, 0.44707E+01_JPRB, 0.39118E+01_JPRB, 0.44846E+01_JPRB, 0.50976E+01_JPRB ,&
&  0.45801E+01_JPRB, 0.41072E+01_JPRB, 0.39621E+01_JPRB, 0.33745E+01_JPRB, 0.32126E+01_JPRB, 0.45647E+01_JPRB, 0.47119E+01_JPRB ,&
&  0.46975E+01_JPRB, 0.48627E+01_JPRB, 0.45762E+01_JPRB, 0.43490E+01_JPRB, 0.40741E+01_JPRB, 0.37573E+01_JPRB, 0.28972E+01_JPRB ,&
&  0.29446E+01_JPRB, 0.27241E+01_JPRB, 0.28181E+01_JPRB, 0.25041E+01_JPRB, 0.23229E+01_JPRB, 0.21146E+01_JPRB, 0.19114E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.18848E+01_JPRB, 0.29283E+01_JPRB, 0.43129E+01_JPRB, 0.51045E+01_JPRB, 0.45303E+01_JPRB, 0.51138E+01_JPRB ,&
&  0.49477E+01_JPRB, 0.42200E+01_JPRB, 0.34069E+01_JPRB, 0.31026E+01_JPRB, 0.28531E+01_JPRB, 0.26480E+01_JPRB, 0.24954E+01_JPRB ,&
&  0.23575E+01_JPRB, 0.22349E+01_JPRB, 0.21460E+01_JPRB, 0.20991E+01_JPRB, 0.20401E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.19911E+01_JPRB, 0.18904E+01_JPRB, 0.17802E+01_JPRB, 0.16868E+01_JPRB, 0.15957E+01_JPRB, 0.15119E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.14016E+01_JPRB, 0.13505E+01_JPRB, 0.12782E+01_JPRB, 0.12291E+01_JPRB, 0.12050E+01_JPRB, 0.12044E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.11768E+01_JPRB, 0.10911E+01_JPRB, 0.92974E+00_JPRB, 0.85937E+00_JPRB, 0.86543E+00_JPRB, 0.81272E+00_JPRB, 0.82375E+00_JPRB ,&
&  0.12420E+01_JPRB, 0.21305E+01_JPRB, 0.26090E+01_JPRB, 0.27636E+01_JPRB, 0.29352E+01_JPRB, 0.29611E+01_JPRB, 0.27583E+01_JPRB ,&
&  0.26942E+01_JPRB, 0.24574E+01_JPRB, 0.22678E+01_JPRB, 0.20825E+01_JPRB, 0.18974E+01_JPRB, 0.17097E+01_JPRB, 0.15546E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.14407E+01_JPRB, 0.14441E+01_JPRB, 0.14363E+01_JPRB, 0.15084E+01_JPRB, 0.15766E+01_JPRB, 0.16290E+01_JPRB ,&
&  0.16560E+01_JPRB, 0.16808E+01_JPRB, 0.16684E+01_JPRB, 0.17054E+01_JPRB, 0.20147E+01_JPRB, 0.26257E+01_JPRB, 0.31476E+01_JPRB ,&
&  0.34857E+01_JPRB, 0.37951E+01_JPRB, 0.41322E+01_JPRB, 0.44561E+01_JPRB, 0.51628E+01_JPRB, 0.51255E+01_JPRB, 0.53221E+01_JPRB ,&
&  0.41748E+01_JPRB, 0.34492E+01_JPRB, 0.28836E+01_JPRB, 0.23829E+01_JPRB, 0.35086E+01_JPRB, 0.56423E+01_JPRB, 0.62186E+01_JPRB ,&
&  0.61538E+01_JPRB, 0.63232E+01_JPRB, 0.60499E+01_JPRB, 0.50807E+01_JPRB, 0.42909E+01_JPRB, 0.35059E+01_JPRB, 0.39744E+01_JPRB ,&
&  0.38776E+01_JPRB, 0.37813E+01_JPRB, 0.33425E+01_JPRB, 0.27066E+01_JPRB, 0.22960E+01_JPRB, 0.24437E+01_JPRB, 0.22469E+01_JPRB ,&
&  0.21418E+01_JPRB, 0.25306E+01_JPRB, 0.50795E+01_JPRB, 0.64739E+01_JPRB, 0.63082E+01_JPRB, 0.56393E+01_JPRB, 0.62500E+01_JPRB ,&
&  0.49740E+01_JPRB, 0.39577E+01_JPRB, 0.32719E+01_JPRB, 0.27878E+01_JPRB, 0.25271E+01_JPRB, 0.23791E+01_JPRB, 0.23112E+01_JPRB ,&
&  0.22710E+01_JPRB, 0.22429E+01_JPRB, 0.21771E+01_JPRB, 0.21063E+01_JPRB, 0.20480E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.20353E+01_JPRB, 0.19336E+01_JPRB, 0.18410E+01_JPRB, 0.17413E+01_JPRB, 0.16410E+01_JPRB, 0.15680E+01_JPRB, 0.15185E+01_JPRB ,&
&  0.14508E+01_JPRB, 0.13861E+01_JPRB, 0.13249E+01_JPRB, 0.12832E+01_JPRB, 0.12642E+01_JPRB, 0.12522E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.11945E+01_JPRB, 0.10277E+01_JPRB, 0.95498E+00_JPRB, 0.95502E+00_JPRB, 0.95587E+00_JPRB, 0.10646E+01_JPRB ,&
&  0.19227E+01_JPRB, 0.29533E+01_JPRB, 0.30543E+01_JPRB, 0.29332E+01_JPRB, 0.31127E+01_JPRB, 0.26851E+01_JPRB, 0.25441E+01_JPRB ,&
&  0.26224E+01_JPRB, 0.24318E+01_JPRB, 0.22837E+01_JPRB, 0.21301E+01_JPRB, 0.19862E+01_JPRB, 0.18575E+01_JPRB, 0.17410E+01_JPRB ,&
&  0.16502E+01_JPRB, 0.15957E+01_JPRB, 0.15981E+01_JPRB, 0.16590E+01_JPRB, 0.16938E+01_JPRB, 0.17096E+01_JPRB, 0.17155E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.16883E+01_JPRB, 0.16857E+01_JPRB, 0.18679E+01_JPRB, 0.22899E+01_JPRB, 0.29875E+01_JPRB, 0.41875E+01_JPRB ,&
&  0.42024E+01_JPRB, 0.45904E+01_JPRB, 0.45265E+01_JPRB, 0.54679E+01_JPRB, 0.59744E+01_JPRB, 0.56830E+01_JPRB, 0.54138E+01_JPRB ,&
&  0.40613E+01_JPRB, 0.33706E+01_JPRB, 0.31342E+01_JPRB, 0.33380E+01_JPRB, 0.41609E+01_JPRB, 0.52995E+01_JPRB, 0.67878E+01_JPRB ,&
&  0.59347E+01_JPRB, 0.59174E+01_JPRB, 0.58856E+01_JPRB, 0.54231E+01_JPRB, 0.33431E+01_JPRB, 0.26437E+01_JPRB, 0.38451E+01_JPRB ,&
&  0.38831E+01_JPRB, 0.23613E+01_JPRB, 0.16877E+01_JPRB, 0.19926E+01_JPRB, 0.15378E+01_JPRB, 0.18066E+01_JPRB, 0.25391E+01_JPRB ,&
&  0.28806E+01_JPRB, 0.38153E+01_JPRB, 0.64466E+01_JPRB, 0.62030E+01_JPRB, 0.61505E+01_JPRB, 0.69696E+01_JPRB, 0.57705E+01_JPRB ,&
&  0.45212E+01_JPRB, 0.34358E+01_JPRB, 0.31814E+01_JPRB, 0.27551E+01_JPRB, 0.25244E+01_JPRB, 0.23909E+01_JPRB, 0.23797E+01_JPRB ,&
&  0.24429E+01_JPRB, 0.24481E+01_JPRB, 0.23760E+01_JPRB, 0.22502E+01_JPRB, 0.21299E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.21079E+01_JPRB, 0.20139E+01_JPRB, 0.19194E+01_JPRB, 0.18270E+01_JPRB, 0.17413E+01_JPRB, 0.16692E+01_JPRB, 0.15750E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.13899E+01_JPRB, 0.13628E+01_JPRB, 0.13328E+01_JPRB, 0.13143E+01_JPRB, 0.13123E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.12556E+01_JPRB, 0.12251E+01_JPRB, 0.11787E+01_JPRB, 0.11092E+01_JPRB, 0.10830E+01_JPRB, 0.10650E+01_JPRB, 0.14798E+01_JPRB ,&
&  0.28782E+01_JPRB, 0.38117E+01_JPRB, 0.34812E+01_JPRB, 0.29976E+01_JPRB, 0.26964E+01_JPRB, 0.24014E+01_JPRB, 0.25743E+01_JPRB ,&
&  0.25370E+01_JPRB, 0.24154E+01_JPRB, 0.22632E+01_JPRB, 0.21300E+01_JPRB, 0.20150E+01_JPRB, 0.19110E+01_JPRB, 0.18430E+01_JPRB ,&
&  0.17715E+01_JPRB, 0.17143E+01_JPRB, 0.17353E+01_JPRB, 0.17226E+01_JPRB, 0.17170E+01_JPRB, 0.17005E+01_JPRB, 0.17367E+01_JPRB ,&
&  0.17269E+01_JPRB, 0.17298E+01_JPRB, 0.17929E+01_JPRB, 0.20626E+01_JPRB, 0.27834E+01_JPRB, 0.37409E+01_JPRB, 0.40416E+01_JPRB ,&
&  0.41210E+01_JPRB, 0.43982E+01_JPRB, 0.52523E+01_JPRB, 0.56114E+01_JPRB, 0.58876E+01_JPRB, 0.58223E+01_JPRB, 0.51147E+01_JPRB ,&
&  0.45829E+01_JPRB, 0.41659E+01_JPRB, 0.35621E+01_JPRB, 0.38306E+01_JPRB, 0.49622E+01_JPRB, 0.51355E+01_JPRB, 0.60591E+01_JPRB ,&
&  0.60805E+01_JPRB, 0.54834E+01_JPRB, 0.43695E+01_JPRB, 0.35237E+01_JPRB, 0.38875E+01_JPRB, 0.29480E+01_JPRB, 0.19233E+01_JPRB ,&
&  0.16938E+01_JPRB, 0.14971E+01_JPRB, 0.13153E+01_JPRB, 0.13177E+01_JPRB, 0.12765E+01_JPRB, 0.17182E+01_JPRB, 0.27284E+01_JPRB ,&
&  0.39982E+01_JPRB, 0.50879E+01_JPRB, 0.60403E+01_JPRB, 0.55925E+01_JPRB, 0.52497E+01_JPRB, 0.61834E+01_JPRB, 0.55874E+01_JPRB ,&
&  0.36282E+01_JPRB, 0.29944E+01_JPRB, 0.29261E+01_JPRB, 0.26718E+01_JPRB, 0.24930E+01_JPRB, 0.24610E+01_JPRB, 0.25659E+01_JPRB ,&
&  0.26806E+01_JPRB, 0.26720E+01_JPRB, 0.25625E+01_JPRB, 0.23765E+01_JPRB, 0.22163E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.21064E+01_JPRB, 0.19855E+01_JPRB, 0.19051E+01_JPRB, 0.18557E+01_JPRB, 0.18036E+01_JPRB, 0.17070E+01_JPRB, 0.15454E+01_JPRB ,&
&  0.14259E+01_JPRB, 0.13786E+01_JPRB, 0.13699E+01_JPRB, 0.13505E+01_JPRB, 0.13302E+01_JPRB, 0.13197E+01_JPRB, 0.12965E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.12315E+01_JPRB, 0.12521E+01_JPRB, 0.12523E+01_JPRB, 0.12930E+01_JPRB, 0.13277E+01_JPRB, 0.20722E+01_JPRB ,&
&  0.39307E+01_JPRB, 0.45164E+01_JPRB, 0.36917E+01_JPRB, 0.28741E+01_JPRB, 0.26279E+01_JPRB, 0.25873E+01_JPRB, 0.26099E+01_JPRB ,&
&  0.24199E+01_JPRB, 0.22322E+01_JPRB, 0.20059E+01_JPRB, 0.18875E+01_JPRB, 0.18646E+01_JPRB, 0.18418E+01_JPRB, 0.18200E+01_JPRB ,&
&  0.17605E+01_JPRB, 0.17369E+01_JPRB, 0.17055E+01_JPRB, 0.16815E+01_JPRB, 0.16874E+01_JPRB, 0.17212E+01_JPRB, 0.17787E+01_JPRB ,&
&  0.18217E+01_JPRB, 0.18691E+01_JPRB, 0.20346E+01_JPRB, 0.25188E+01_JPRB, 0.32029E+01_JPRB, 0.35181E+01_JPRB, 0.41186E+01_JPRB ,&
&  0.47533E+01_JPRB, 0.52573E+01_JPRB, 0.50631E+01_JPRB, 0.59710E+01_JPRB, 0.58897E+01_JPRB, 0.53564E+01_JPRB, 0.49210E+01_JPRB ,&
&  0.45711E+01_JPRB, 0.48925E+01_JPRB, 0.39688E+01_JPRB, 0.36631E+01_JPRB, 0.53758E+01_JPRB, 0.57057E+01_JPRB, 0.60299E+01_JPRB ,&
&  0.64767E+01_JPRB, 0.55977E+01_JPRB, 0.46725E+01_JPRB, 0.44357E+01_JPRB, 0.63780E+01_JPRB, 0.62263E+01_JPRB, 0.24654E+01_JPRB ,&
&  0.17782E+01_JPRB, 0.15535E+01_JPRB, 0.13167E+01_JPRB, 0.12151E+01_JPRB, 0.12755E+01_JPRB, 0.18067E+01_JPRB, 0.35099E+01_JPRB ,&
&  0.40669E+01_JPRB, 0.56383E+01_JPRB, 0.57986E+01_JPRB, 0.54999E+01_JPRB, 0.48322E+01_JPRB, 0.46373E+01_JPRB, 0.40179E+01_JPRB ,&
&  0.35704E+01_JPRB, 0.30997E+01_JPRB, 0.26472E+01_JPRB, 0.23757E+01_JPRB, 0.22754E+01_JPRB, 0.23517E+01_JPRB, 0.25480E+01_JPRB ,&
&  0.26535E+01_JPRB, 0.26616E+01_JPRB, 0.25688E+01_JPRB, 0.24074E+01_JPRB, 0.22361E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.20224E+01_JPRB, 0.19672E+01_JPRB, 0.19343E+01_JPRB, 0.18680E+01_JPRB, 0.17816E+01_JPRB, 0.16626E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.14013E+01_JPRB, 0.13952E+01_JPRB, 0.13626E+01_JPRB, 0.13218E+01_JPRB, 0.12959E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.12516E+01_JPRB, 0.12197E+01_JPRB, 0.12277E+01_JPRB, 0.13685E+01_JPRB, 0.15661E+01_JPRB, 0.19545E+01_JPRB, 0.29005E+01_JPRB ,&
&  0.48052E+01_JPRB, 0.49664E+01_JPRB, 0.39021E+01_JPRB, 0.31795E+01_JPRB, 0.28263E+01_JPRB, 0.27581E+01_JPRB, 0.24686E+01_JPRB ,&
&  0.21868E+01_JPRB, 0.20015E+01_JPRB, 0.18190E+01_JPRB, 0.18121E+01_JPRB, 0.18319E+01_JPRB, 0.18495E+01_JPRB, 0.18299E+01_JPRB ,&
&  0.18223E+01_JPRB, 0.17471E+01_JPRB, 0.17250E+01_JPRB, 0.17101E+01_JPRB, 0.17380E+01_JPRB, 0.18150E+01_JPRB, 0.18954E+01_JPRB ,&
&  0.19815E+01_JPRB, 0.21014E+01_JPRB, 0.24004E+01_JPRB, 0.29151E+01_JPRB, 0.30422E+01_JPRB, 0.38013E+01_JPRB, 0.42406E+01_JPRB ,&
&  0.46279E+01_JPRB, 0.47772E+01_JPRB, 0.46516E+01_JPRB, 0.52726E+01_JPRB, 0.56315E+01_JPRB, 0.52161E+01_JPRB, 0.47826E+01_JPRB ,&
&  0.49750E+01_JPRB, 0.43714E+01_JPRB, 0.39180E+01_JPRB, 0.36075E+01_JPRB, 0.45369E+01_JPRB, 0.71299E+01_JPRB, 0.56174E+01_JPRB ,&
&  0.54670E+01_JPRB, 0.62284E+01_JPRB, 0.56013E+01_JPRB, 0.56794E+01_JPRB, 0.67899E+01_JPRB, 0.83576E+01_JPRB, 0.65047E+01_JPRB ,&
&  0.23348E+01_JPRB, 0.15832E+01_JPRB, 0.16041E+01_JPRB, 0.15598E+01_JPRB, 0.13834E+01_JPRB, 0.19519E+01_JPRB, 0.61311E+01_JPRB ,&
&  0.50673E+01_JPRB, 0.65765E+01_JPRB, 0.62919E+01_JPRB, 0.55647E+01_JPRB, 0.50090E+01_JPRB, 0.41714E+01_JPRB, 0.36531E+01_JPRB ,&
&  0.29736E+01_JPRB, 0.25452E+01_JPRB, 0.23244E+01_JPRB, 0.22093E+01_JPRB, 0.21995E+01_JPRB, 0.22461E+01_JPRB, 0.23832E+01_JPRB ,&
&  0.24748E+01_JPRB, 0.24918E+01_JPRB, 0.24026E+01_JPRB, 0.22625E+01_JPRB, 0.21252E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.19616E+01_JPRB, 0.19360E+01_JPRB, 0.18902E+01_JPRB, 0.17983E+01_JPRB, 0.17061E+01_JPRB, 0.15781E+01_JPRB, 0.14766E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.14255E+01_JPRB, 0.13997E+01_JPRB, 0.13630E+01_JPRB, 0.13168E+01_JPRB, 0.12800E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.12524E+01_JPRB, 0.12170E+01_JPRB, 0.12137E+01_JPRB, 0.13727E+01_JPRB, 0.18901E+01_JPRB, 0.25233E+01_JPRB, 0.32150E+01_JPRB ,&
&  0.55379E+01_JPRB, 0.54273E+01_JPRB, 0.36158E+01_JPRB, 0.31184E+01_JPRB, 0.29576E+01_JPRB, 0.28798E+01_JPRB, 0.24117E+01_JPRB ,&
&  0.21257E+01_JPRB, 0.19710E+01_JPRB, 0.18170E+01_JPRB, 0.17855E+01_JPRB, 0.18046E+01_JPRB, 0.18603E+01_JPRB, 0.18851E+01_JPRB ,&
&  0.18154E+01_JPRB, 0.18015E+01_JPRB, 0.18368E+01_JPRB, 0.18446E+01_JPRB, 0.18748E+01_JPRB, 0.19735E+01_JPRB, 0.20707E+01_JPRB ,&
&  0.22033E+01_JPRB, 0.24082E+01_JPRB, 0.27664E+01_JPRB, 0.30444E+01_JPRB, 0.34393E+01_JPRB, 0.37860E+01_JPRB, 0.42255E+01_JPRB ,&
&  0.42625E+01_JPRB, 0.44060E+01_JPRB, 0.46015E+01_JPRB, 0.48211E+01_JPRB, 0.49912E+01_JPRB, 0.49535E+01_JPRB, 0.48036E+01_JPRB ,&
&  0.49078E+01_JPRB, 0.42515E+01_JPRB, 0.37274E+01_JPRB, 0.35914E+01_JPRB, 0.42748E+01_JPRB, 0.64459E+01_JPRB, 0.71175E+01_JPRB ,&
&  0.72241E+01_JPRB, 0.54300E+01_JPRB, 0.53685E+01_JPRB, 0.66626E+01_JPRB, 0.66793E+01_JPRB, 0.75056E+01_JPRB, 0.10261E+02_JPRB ,&
&  0.90832E+01_JPRB, 0.44913E+01_JPRB, 0.35404E+01_JPRB, 0.45937E+01_JPRB, 0.27016E+01_JPRB, 0.21749E+01_JPRB, 0.38574E+01_JPRB ,&
&  0.50085E+01_JPRB, 0.60256E+01_JPRB, 0.58837E+01_JPRB, 0.53484E+01_JPRB, 0.48696E+01_JPRB, 0.37289E+01_JPRB, 0.29873E+01_JPRB ,&
&  0.23999E+01_JPRB, 0.21879E+01_JPRB, 0.20859E+01_JPRB, 0.19781E+01_JPRB, 0.19399E+01_JPRB, 0.20134E+01_JPRB, 0.21261E+01_JPRB ,&
&  0.22213E+01_JPRB, 0.22478E+01_JPRB, 0.22164E+01_JPRB, 0.21313E+01_JPRB, 0.20268E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.18798E+01_JPRB, 0.18651E+01_JPRB, 0.18362E+01_JPRB, 0.17373E+01_JPRB, 0.16327E+01_JPRB, 0.15281E+01_JPRB, 0.14933E+01_JPRB ,&
&  0.14578E+01_JPRB, 0.14259E+01_JPRB, 0.13903E+01_JPRB, 0.13411E+01_JPRB, 0.13166E+01_JPRB, 0.12851E+01_JPRB, 0.12874E+01_JPRB ,&
&  0.13008E+01_JPRB, 0.12809E+01_JPRB, 0.13494E+01_JPRB, 0.17486E+01_JPRB, 0.25750E+01_JPRB, 0.43145E+01_JPRB, 0.37415E+01_JPRB ,&
&  0.55842E+01_JPRB, 0.58861E+01_JPRB, 0.36765E+01_JPRB, 0.33370E+01_JPRB, 0.32132E+01_JPRB, 0.32042E+01_JPRB, 0.26377E+01_JPRB ,&
&  0.22458E+01_JPRB, 0.20735E+01_JPRB, 0.19609E+01_JPRB, 0.18534E+01_JPRB, 0.18484E+01_JPRB, 0.18861E+01_JPRB, 0.18426E+01_JPRB ,&
&  0.18608E+01_JPRB, 0.19297E+01_JPRB, 0.19617E+01_JPRB, 0.19771E+01_JPRB, 0.20452E+01_JPRB, 0.21471E+01_JPRB, 0.22750E+01_JPRB ,&
&  0.24789E+01_JPRB, 0.27312E+01_JPRB, 0.31023E+01_JPRB, 0.32279E+01_JPRB, 0.34795E+01_JPRB, 0.38399E+01_JPRB, 0.40262E+01_JPRB ,&
&  0.38936E+01_JPRB, 0.37481E+01_JPRB, 0.41617E+01_JPRB, 0.45229E+01_JPRB, 0.46034E+01_JPRB, 0.45907E+01_JPRB, 0.46714E+01_JPRB ,&
&  0.49958E+01_JPRB, 0.48111E+01_JPRB, 0.48804E+01_JPRB, 0.43049E+01_JPRB, 0.50037E+01_JPRB, 0.65245E+01_JPRB, 0.67214E+01_JPRB ,&
&  0.81410E+01_JPRB, 0.73944E+01_JPRB, 0.73804E+01_JPRB, 0.71482E+01_JPRB, 0.66514E+01_JPRB, 0.69661E+01_JPRB, 0.78493E+01_JPRB ,&
&  0.84012E+01_JPRB, 0.92918E+01_JPRB, 0.87415E+01_JPRB, 0.52672E+01_JPRB, 0.33431E+01_JPRB, 0.26919E+01_JPRB, 0.32180E+01_JPRB ,&
&  0.45076E+01_JPRB, 0.47702E+01_JPRB, 0.46148E+01_JPRB, 0.63042E+01_JPRB, 0.40552E+01_JPRB, 0.29045E+01_JPRB, 0.22649E+01_JPRB ,&
&  0.19554E+01_JPRB, 0.18684E+01_JPRB, 0.18026E+01_JPRB, 0.16932E+01_JPRB, 0.16716E+01_JPRB, 0.17322E+01_JPRB, 0.18320E+01_JPRB ,&
&  0.19421E+01_JPRB, 0.20135E+01_JPRB, 0.20483E+01_JPRB, 0.19981E+01_JPRB, 0.19254E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.17547E+01_JPRB, 0.17545E+01_JPRB, 0.17004E+01_JPRB, 0.16221E+01_JPRB, 0.15586E+01_JPRB, 0.15049E+01_JPRB, 0.14973E+01_JPRB ,&
&  0.14183E+01_JPRB, 0.13780E+01_JPRB, 0.13476E+01_JPRB, 0.13214E+01_JPRB, 0.13385E+01_JPRB, 0.13642E+01_JPRB, 0.14161E+01_JPRB ,&
&  0.14895E+01_JPRB, 0.16144E+01_JPRB, 0.19560E+01_JPRB, 0.27700E+01_JPRB, 0.39452E+01_JPRB, 0.56097E+01_JPRB, 0.58136E+01_JPRB ,&
&  0.54956E+01_JPRB, 0.64453E+01_JPRB, 0.36470E+01_JPRB, 0.36396E+01_JPRB, 0.37170E+01_JPRB, 0.42077E+01_JPRB, 0.34865E+01_JPRB ,&
&  0.26133E+01_JPRB, 0.23574E+01_JPRB, 0.22179E+01_JPRB, 0.19959E+01_JPRB, 0.19221E+01_JPRB, 0.18964E+01_JPRB, 0.19109E+01_JPRB ,&
&  0.19948E+01_JPRB, 0.20801E+01_JPRB, 0.21290E+01_JPRB, 0.21867E+01_JPRB, 0.22756E+01_JPRB, 0.23779E+01_JPRB, 0.25646E+01_JPRB ,&
&  0.28280E+01_JPRB, 0.31139E+01_JPRB, 0.32597E+01_JPRB, 0.33630E+01_JPRB, 0.34624E+01_JPRB, 0.37248E+01_JPRB, 0.38732E+01_JPRB ,&
&  0.38193E+01_JPRB, 0.36876E+01_JPRB, 0.39651E+01_JPRB, 0.40565E+01_JPRB, 0.40998E+01_JPRB, 0.44999E+01_JPRB, 0.45535E+01_JPRB ,&
&  0.50690E+01_JPRB, 0.48123E+01_JPRB, 0.55467E+01_JPRB, 0.51806E+01_JPRB, 0.54368E+01_JPRB, 0.61041E+01_JPRB, 0.55742E+01_JPRB ,&
&  0.57643E+01_JPRB, 0.66495E+01_JPRB, 0.69504E+01_JPRB, 0.70445E+01_JPRB, 0.69591E+01_JPRB, 0.68985E+01_JPRB, 0.66582E+01_JPRB ,&
&  0.58939E+01_JPRB, 0.57861E+01_JPRB, 0.51126E+01_JPRB, 0.33525E+01_JPRB, 0.26229E+01_JPRB, 0.25097E+01_JPRB, 0.28649E+01_JPRB ,&
&  0.37281E+01_JPRB, 0.39920E+01_JPRB, 0.44893E+01_JPRB, 0.42413E+01_JPRB, 0.27077E+01_JPRB, 0.20159E+01_JPRB, 0.17977E+01_JPRB ,&
&  0.17073E+01_JPRB, 0.16375E+01_JPRB, 0.15377E+01_JPRB, 0.14465E+01_JPRB, 0.14511E+01_JPRB, 0.15274E+01_JPRB, 0.16199E+01_JPRB ,&
&  0.17257E+01_JPRB, 0.17929E+01_JPRB, 0.17958E+01_JPRB, 0.17644E+01_JPRB, 0.17586E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.17197E+01_JPRB, 0.17184E+01_JPRB, 0.17076E+01_JPRB, 0.17213E+01_JPRB, 0.17476E+01_JPRB, 0.17911E+01_JPRB, 0.17672E+01_JPRB ,&
&  0.15937E+01_JPRB, 0.13326E+01_JPRB, 0.13538E+01_JPRB, 0.14115E+01_JPRB, 0.15313E+01_JPRB, 0.16518E+01_JPRB, 0.18196E+01_JPRB ,&
&  0.20832E+01_JPRB, 0.24563E+01_JPRB, 0.31076E+01_JPRB, 0.41017E+01_JPRB, 0.53616E+01_JPRB, 0.68955E+01_JPRB, 0.73590E+01_JPRB ,&
&  0.70837E+01_JPRB, 0.48153E+01_JPRB, 0.39039E+01_JPRB, 0.37868E+01_JPRB, 0.35895E+01_JPRB, 0.40525E+01_JPRB, 0.38278E+01_JPRB ,&
&  0.30202E+01_JPRB, 0.26479E+01_JPRB, 0.24983E+01_JPRB, 0.22838E+01_JPRB, 0.21030E+01_JPRB, 0.21439E+01_JPRB, 0.22164E+01_JPRB ,&
&  0.23201E+01_JPRB, 0.23847E+01_JPRB, 0.24054E+01_JPRB, 0.24413E+01_JPRB, 0.25351E+01_JPRB, 0.26575E+01_JPRB, 0.28936E+01_JPRB ,&
&  0.31561E+01_JPRB, 0.33641E+01_JPRB, 0.34472E+01_JPRB, 0.33786E+01_JPRB, 0.34701E+01_JPRB, 0.36158E+01_JPRB, 0.36928E+01_JPRB ,&
&  0.37205E+01_JPRB, 0.38296E+01_JPRB, 0.39399E+01_JPRB, 0.40465E+01_JPRB, 0.39309E+01_JPRB, 0.44357E+01_JPRB, 0.44776E+01_JPRB ,&
&  0.47566E+01_JPRB, 0.46616E+01_JPRB, 0.54092E+01_JPRB, 0.48739E+01_JPRB, 0.54161E+01_JPRB, 0.54644E+01_JPRB, 0.52470E+01_JPRB ,&
&  0.54869E+01_JPRB, 0.64547E+01_JPRB, 0.68225E+01_JPRB, 0.68443E+01_JPRB, 0.68704E+01_JPRB, 0.61063E+01_JPRB, 0.64430E+01_JPRB ,&
&  0.50263E+01_JPRB, 0.50332E+01_JPRB, 0.33454E+01_JPRB, 0.24910E+01_JPRB, 0.20379E+01_JPRB, 0.24162E+01_JPRB, 0.29145E+01_JPRB ,&
&  0.35260E+01_JPRB, 0.34205E+01_JPRB, 0.29171E+01_JPRB, 0.27100E+01_JPRB, 0.21065E+01_JPRB, 0.17235E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.15855E+01_JPRB, 0.14909E+01_JPRB, 0.14067E+01_JPRB, 0.13855E+01_JPRB, 0.14034E+01_JPRB, 0.15013E+01_JPRB, 0.15709E+01_JPRB ,&
&  0.16498E+01_JPRB, 0.16738E+01_JPRB, 0.16749E+01_JPRB, 0.16864E+01_JPRB, 0.16980E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.16910E+01_JPRB, 0.16572E+01_JPRB, 0.16425E+01_JPRB, 0.16258E+01_JPRB, 0.16008E+01_JPRB, 0.15260E+01_JPRB, 0.14713E+01_JPRB ,&
&  0.14150E+01_JPRB, 0.13862E+01_JPRB, 0.15147E+01_JPRB, 0.16948E+01_JPRB, 0.19237E+01_JPRB, 0.21763E+01_JPRB, 0.24811E+01_JPRB ,&
&  0.28788E+01_JPRB, 0.33905E+01_JPRB, 0.40388E+01_JPRB, 0.46378E+01_JPRB, 0.52759E+01_JPRB, 0.61068E+01_JPRB, 0.66677E+01_JPRB ,&
&  0.59340E+01_JPRB, 0.36173E+01_JPRB, 0.37846E+01_JPRB, 0.38079E+01_JPRB, 0.36786E+01_JPRB, 0.36804E+01_JPRB, 0.33775E+01_JPRB ,&
&  0.29481E+01_JPRB, 0.27903E+01_JPRB, 0.27276E+01_JPRB, 0.26215E+01_JPRB, 0.24188E+01_JPRB, 0.24786E+01_JPRB, 0.25390E+01_JPRB ,&
&  0.25626E+01_JPRB, 0.25409E+01_JPRB, 0.25362E+01_JPRB, 0.26449E+01_JPRB, 0.28505E+01_JPRB, 0.29806E+01_JPRB, 0.31683E+01_JPRB ,&
&  0.33279E+01_JPRB, 0.35283E+01_JPRB, 0.36127E+01_JPRB, 0.36299E+01_JPRB, 0.37486E+01_JPRB, 0.38372E+01_JPRB, 0.38511E+01_JPRB ,&
&  0.38322E+01_JPRB, 0.38252E+01_JPRB, 0.39752E+01_JPRB, 0.43886E+01_JPRB, 0.46325E+01_JPRB, 0.42435E+01_JPRB, 0.44569E+01_JPRB ,&
&  0.46221E+01_JPRB, 0.48165E+01_JPRB, 0.47674E+01_JPRB, 0.61472E+01_JPRB, 0.47592E+01_JPRB, 0.49029E+01_JPRB, 0.45371E+01_JPRB ,&
&  0.53422E+01_JPRB, 0.58861E+01_JPRB, 0.63755E+01_JPRB, 0.65126E+01_JPRB, 0.63188E+01_JPRB, 0.57657E+01_JPRB, 0.61702E+01_JPRB ,&
&  0.53282E+01_JPRB, 0.44776E+01_JPRB, 0.31030E+01_JPRB, 0.22419E+01_JPRB, 0.22546E+01_JPRB, 0.38434E+01_JPRB, 0.38417E+01_JPRB ,&
&  0.34073E+01_JPRB, 0.26387E+01_JPRB, 0.24564E+01_JPRB, 0.28021E+01_JPRB, 0.20845E+01_JPRB, 0.16093E+01_JPRB, 0.14784E+01_JPRB ,&
&  0.14842E+01_JPRB, 0.14544E+01_JPRB, 0.13953E+01_JPRB, 0.13960E+01_JPRB, 0.14591E+01_JPRB, 0.15543E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.16508E+01_JPRB, 0.16357E+01_JPRB, 0.16434E+01_JPRB, 0.16593E+01_JPRB, 0.17017E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.15792E+01_JPRB, 0.15174E+01_JPRB, 0.14682E+01_JPRB, 0.14207E+01_JPRB, 0.13755E+01_JPRB, 0.13531E+01_JPRB, 0.13730E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.15245E+01_JPRB, 0.16798E+01_JPRB, 0.18717E+01_JPRB, 0.21181E+01_JPRB, 0.24225E+01_JPRB, 0.27601E+01_JPRB ,&
&  0.31930E+01_JPRB, 0.35996E+01_JPRB, 0.38183E+01_JPRB, 0.39704E+01_JPRB, 0.41540E+01_JPRB, 0.44226E+01_JPRB, 0.46472E+01_JPRB ,&
&  0.48376E+01_JPRB, 0.46073E+01_JPRB, 0.40059E+01_JPRB, 0.29477E+01_JPRB, 0.26808E+01_JPRB, 0.30467E+01_JPRB, 0.29641E+01_JPRB ,&
&  0.28785E+01_JPRB, 0.27650E+01_JPRB, 0.26802E+01_JPRB, 0.26059E+01_JPRB, 0.25887E+01_JPRB, 0.26306E+01_JPRB, 0.26645E+01_JPRB ,&
&  0.26586E+01_JPRB, 0.26721E+01_JPRB, 0.27365E+01_JPRB, 0.28820E+01_JPRB, 0.30973E+01_JPRB, 0.32359E+01_JPRB, 0.33766E+01_JPRB ,&
&  0.34901E+01_JPRB, 0.36898E+01_JPRB, 0.37922E+01_JPRB, 0.39127E+01_JPRB, 0.39527E+01_JPRB, 0.40648E+01_JPRB, 0.42556E+01_JPRB ,&
&  0.43314E+01_JPRB, 0.44424E+01_JPRB, 0.46522E+01_JPRB, 0.49880E+01_JPRB, 0.51446E+01_JPRB, 0.45760E+01_JPRB, 0.44714E+01_JPRB ,&
&  0.48186E+01_JPRB, 0.52165E+01_JPRB, 0.49476E+01_JPRB, 0.63896E+01_JPRB, 0.41861E+01_JPRB, 0.46347E+01_JPRB, 0.47343E+01_JPRB ,&
&  0.47137E+01_JPRB, 0.50679E+01_JPRB, 0.54586E+01_JPRB, 0.57324E+01_JPRB, 0.53990E+01_JPRB, 0.49519E+01_JPRB, 0.59225E+01_JPRB ,&
&  0.54053E+01_JPRB, 0.39214E+01_JPRB, 0.28272E+01_JPRB, 0.19513E+01_JPRB, 0.25061E+01_JPRB, 0.44465E+01_JPRB, 0.33515E+01_JPRB ,&
&  0.22265E+01_JPRB, 0.22545E+01_JPRB, 0.24759E+01_JPRB, 0.27485E+01_JPRB, 0.20971E+01_JPRB, 0.14613E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.14629E+01_JPRB, 0.14891E+01_JPRB, 0.15176E+01_JPRB, 0.15639E+01_JPRB, 0.16236E+01_JPRB, 0.16744E+01_JPRB ,&
&  0.16749E+01_JPRB, 0.16636E+01_JPRB, 0.16910E+01_JPRB, 0.16839E+01_JPRB, 0.16427E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.13268E+01_JPRB, 0.12567E+01_JPRB, 0.12046E+01_JPRB, 0.11898E+01_JPRB, 0.12024E+01_JPRB, 0.12356E+01_JPRB, 0.12946E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.15286E+01_JPRB, 0.16588E+01_JPRB, 0.17966E+01_JPRB, 0.19711E+01_JPRB, 0.21579E+01_JPRB, 0.23762E+01_JPRB ,&
&  0.26134E+01_JPRB, 0.28588E+01_JPRB, 0.29576E+01_JPRB, 0.29433E+01_JPRB, 0.29591E+01_JPRB, 0.29875E+01_JPRB, 0.31875E+01_JPRB ,&
&  0.36515E+01_JPRB, 0.40433E+01_JPRB, 0.42173E+01_JPRB, 0.39658E+01_JPRB, 0.31734E+01_JPRB, 0.32167E+01_JPRB, 0.35038E+01_JPRB ,&
&  0.35177E+01_JPRB, 0.30788E+01_JPRB, 0.27322E+01_JPRB, 0.26951E+01_JPRB, 0.26948E+01_JPRB, 0.27358E+01_JPRB, 0.27672E+01_JPRB ,&
&  0.27708E+01_JPRB, 0.28053E+01_JPRB, 0.28989E+01_JPRB, 0.30083E+01_JPRB, 0.31700E+01_JPRB, 0.33380E+01_JPRB, 0.35157E+01_JPRB ,&
&  0.36135E+01_JPRB, 0.37243E+01_JPRB, 0.37082E+01_JPRB, 0.36163E+01_JPRB, 0.37019E+01_JPRB, 0.40528E+01_JPRB, 0.45567E+01_JPRB ,&
&  0.48862E+01_JPRB, 0.49040E+01_JPRB, 0.52700E+01_JPRB, 0.56706E+01_JPRB, 0.56115E+01_JPRB, 0.52047E+01_JPRB, 0.51574E+01_JPRB ,&
&  0.53749E+01_JPRB, 0.59841E+01_JPRB, 0.45117E+01_JPRB, 0.53963E+01_JPRB, 0.55754E+01_JPRB, 0.49103E+01_JPRB, 0.42946E+01_JPRB ,&
&  0.39478E+01_JPRB, 0.36475E+01_JPRB, 0.31880E+01_JPRB, 0.34188E+01_JPRB, 0.36498E+01_JPRB, 0.40430E+01_JPRB, 0.48989E+01_JPRB ,&
&  0.46476E+01_JPRB, 0.33657E+01_JPRB, 0.23074E+01_JPRB, 0.17959E+01_JPRB, 0.21886E+01_JPRB, 0.31236E+01_JPRB, 0.23090E+01_JPRB ,&
&  0.18940E+01_JPRB, 0.20889E+01_JPRB, 0.21899E+01_JPRB, 0.25156E+01_JPRB, 0.23184E+01_JPRB, 0.13574E+01_JPRB, 0.13673E+01_JPRB ,&
&  0.14125E+01_JPRB, 0.14682E+01_JPRB, 0.14882E+01_JPRB, 0.15085E+01_JPRB, 0.15223E+01_JPRB, 0.15579E+01_JPRB, 0.15863E+01_JPRB ,&
&  0.15883E+01_JPRB, 0.15772E+01_JPRB, 0.15425E+01_JPRB, 0.14825E+01_JPRB, 0.13987E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.10375E+01_JPRB, 0.10290E+01_JPRB, 0.10205E+01_JPRB, 0.10495E+01_JPRB, 0.10829E+01_JPRB, 0.11008E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.11856E+01_JPRB, 0.12359E+01_JPRB, 0.12858E+01_JPRB, 0.13332E+01_JPRB, 0.14672E+01_JPRB, 0.16101E+01_JPRB, 0.17738E+01_JPRB ,&
&  0.18615E+01_JPRB, 0.19298E+01_JPRB, 0.19114E+01_JPRB, 0.18288E+01_JPRB, 0.17509E+01_JPRB, 0.15877E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.15056E+01_JPRB, 0.19517E+01_JPRB, 0.26812E+01_JPRB, 0.34367E+01_JPRB, 0.41623E+01_JPRB, 0.37847E+01_JPRB, 0.42505E+01_JPRB ,&
&  0.36116E+01_JPRB, 0.27414E+01_JPRB, 0.26242E+01_JPRB, 0.26737E+01_JPRB, 0.27727E+01_JPRB, 0.27695E+01_JPRB, 0.28138E+01_JPRB ,&
&  0.28435E+01_JPRB, 0.28854E+01_JPRB, 0.29565E+01_JPRB, 0.30913E+01_JPRB, 0.32057E+01_JPRB, 0.32874E+01_JPRB, 0.33882E+01_JPRB ,&
&  0.34775E+01_JPRB, 0.35122E+01_JPRB, 0.35689E+01_JPRB, 0.33449E+01_JPRB, 0.36302E+01_JPRB, 0.41968E+01_JPRB, 0.46883E+01_JPRB ,&
&  0.49405E+01_JPRB, 0.50904E+01_JPRB, 0.55800E+01_JPRB, 0.58140E+01_JPRB, 0.59641E+01_JPRB, 0.59186E+01_JPRB, 0.59502E+01_JPRB ,&
&  0.60111E+01_JPRB, 0.55472E+01_JPRB, 0.41364E+01_JPRB, 0.45399E+01_JPRB, 0.41211E+01_JPRB, 0.34183E+01_JPRB, 0.35063E+01_JPRB ,&
&  0.31524E+01_JPRB, 0.27274E+01_JPRB, 0.21755E+01_JPRB, 0.21669E+01_JPRB, 0.25100E+01_JPRB, 0.29338E+01_JPRB, 0.38996E+01_JPRB ,&
&  0.32320E+01_JPRB, 0.24560E+01_JPRB, 0.17868E+01_JPRB, 0.16777E+01_JPRB, 0.18404E+01_JPRB, 0.20940E+01_JPRB, 0.20609E+01_JPRB ,&
&  0.20296E+01_JPRB, 0.19579E+01_JPRB, 0.21055E+01_JPRB, 0.24066E+01_JPRB, 0.15411E+01_JPRB, 0.11844E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.12640E+01_JPRB, 0.12679E+01_JPRB, 0.12591E+01_JPRB, 0.12731E+01_JPRB, 0.13021E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.11976E+01_JPRB, 0.11274E+01_JPRB, 0.10811E+01_JPRB, 0.10485E+01_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.78639E+00_JPRB, 0.79163E+00_JPRB, 0.80142E+00_JPRB, 0.81833E+00_JPRB, 0.82716E+00_JPRB, 0.83349E+00_JPRB, 0.82648E+00_JPRB ,&
&  0.82309E+00_JPRB, 0.81506E+00_JPRB, 0.81820E+00_JPRB, 0.82900E+00_JPRB, 0.87234E+00_JPRB, 0.94684E+00_JPRB, 0.10398E+01_JPRB ,&
&  0.10666E+01_JPRB, 0.10596E+01_JPRB, 0.10005E+01_JPRB, 0.94524E+00_JPRB, 0.94798E+00_JPRB, 0.92527E+00_JPRB, 0.88005E+00_JPRB ,&
&  0.85592E+00_JPRB, 0.95220E+00_JPRB, 0.12153E+01_JPRB, 0.16429E+01_JPRB, 0.24138E+01_JPRB, 0.29718E+01_JPRB, 0.40015E+01_JPRB ,&
&  0.40456E+01_JPRB, 0.24454E+01_JPRB, 0.24245E+01_JPRB, 0.22958E+01_JPRB, 0.24266E+01_JPRB, 0.25024E+01_JPRB, 0.25266E+01_JPRB ,&
&  0.25954E+01_JPRB, 0.26709E+01_JPRB, 0.27446E+01_JPRB, 0.28207E+01_JPRB, 0.28742E+01_JPRB, 0.28702E+01_JPRB, 0.28716E+01_JPRB ,&
&  0.28821E+01_JPRB, 0.28623E+01_JPRB, 0.29256E+01_JPRB, 0.31301E+01_JPRB, 0.36687E+01_JPRB, 0.42708E+01_JPRB, 0.43779E+01_JPRB ,&
&  0.44579E+01_JPRB, 0.48189E+01_JPRB, 0.47832E+01_JPRB, 0.51401E+01_JPRB, 0.52995E+01_JPRB, 0.51715E+01_JPRB, 0.55380E+01_JPRB ,&
&  0.57707E+01_JPRB, 0.48764E+01_JPRB, 0.33340E+01_JPRB, 0.35544E+01_JPRB, 0.33232E+01_JPRB, 0.30485E+01_JPRB, 0.31423E+01_JPRB ,&
&  0.28922E+01_JPRB, 0.25912E+01_JPRB, 0.22839E+01_JPRB, 0.21043E+01_JPRB, 0.21336E+01_JPRB, 0.20660E+01_JPRB, 0.21999E+01_JPRB ,&
&  0.20206E+01_JPRB, 0.19695E+01_JPRB, 0.18444E+01_JPRB, 0.18212E+01_JPRB, 0.21447E+01_JPRB, 0.26377E+01_JPRB, 0.22307E+01_JPRB ,&
&  0.21453E+01_JPRB, 0.21698E+01_JPRB, 0.25434E+01_JPRB, 0.26988E+01_JPRB, 0.13913E+01_JPRB, 0.97563E+00_JPRB, 0.90029E+00_JPRB ,&
&  0.91182E+00_JPRB, 0.90508E+00_JPRB, 0.89464E+00_JPRB, 0.87701E+00_JPRB, 0.91488E+00_JPRB, 0.98155E+00_JPRB, 0.95310E+00_JPRB ,&
&  0.93962E+00_JPRB, 0.86741E+00_JPRB, 0.83289E+00_JPRB, 0.81983E+00_JPRB, 0.81005E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.71814E+00_JPRB, 0.71134E+00_JPRB, 0.70516E+00_JPRB, 0.69868E+00_JPRB, 0.71286E+00_JPRB, 0.72344E+00_JPRB, 0.74495E+00_JPRB ,&
&  0.76901E+00_JPRB, 0.78151E+00_JPRB, 0.80008E+00_JPRB, 0.81080E+00_JPRB, 0.84243E+00_JPRB, 0.86983E+00_JPRB, 0.91377E+00_JPRB ,&
&  0.95943E+00_JPRB, 0.98699E+00_JPRB, 0.10156E+01_JPRB, 0.10453E+01_JPRB, 0.10603E+01_JPRB, 0.10800E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.10524E+01_JPRB, 0.10946E+01_JPRB, 0.12801E+01_JPRB, 0.16315E+01_JPRB, 0.20833E+01_JPRB, 0.30191E+01_JPRB ,&
&  0.32860E+01_JPRB, 0.22141E+01_JPRB, 0.18435E+01_JPRB, 0.16296E+01_JPRB, 0.16964E+01_JPRB, 0.18005E+01_JPRB, 0.17899E+01_JPRB ,&
&  0.18028E+01_JPRB, 0.18095E+01_JPRB, 0.17994E+01_JPRB, 0.17625E+01_JPRB, 0.16671E+01_JPRB, 0.15508E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.14838E+01_JPRB, 0.14761E+01_JPRB, 0.16944E+01_JPRB, 0.22473E+01_JPRB, 0.27337E+01_JPRB, 0.35081E+01_JPRB, 0.33388E+01_JPRB ,&
&  0.33981E+01_JPRB, 0.35840E+01_JPRB, 0.35571E+01_JPRB, 0.38215E+01_JPRB, 0.39415E+01_JPRB, 0.41039E+01_JPRB, 0.41314E+01_JPRB ,&
&  0.44167E+01_JPRB, 0.38945E+01_JPRB, 0.28097E+01_JPRB, 0.30920E+01_JPRB, 0.29628E+01_JPRB, 0.29873E+01_JPRB, 0.28470E+01_JPRB ,&
&  0.26642E+01_JPRB, 0.24462E+01_JPRB, 0.22595E+01_JPRB, 0.21228E+01_JPRB, 0.21202E+01_JPRB, 0.20613E+01_JPRB, 0.18935E+01_JPRB ,&
&  0.18817E+01_JPRB, 0.19188E+01_JPRB, 0.18906E+01_JPRB, 0.18566E+01_JPRB, 0.23199E+01_JPRB, 0.26226E+01_JPRB, 0.33122E+01_JPRB ,&
&  0.22129E+01_JPRB, 0.20149E+01_JPRB, 0.15165E+01_JPRB, 0.19261E+01_JPRB, 0.10751E+01_JPRB, 0.75619E+00_JPRB, 0.73780E+00_JPRB ,&
&  0.71893E+00_JPRB, 0.69446E+00_JPRB, 0.67388E+00_JPRB, 0.66809E+00_JPRB, 0.68196E+00_JPRB, 0.72243E+00_JPRB, 0.71287E+00_JPRB ,&
&  0.71386E+00_JPRB, 0.71421E+00_JPRB, 0.72535E+00_JPRB, 0.74416E+00_JPRB, 0.73591E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.76822E+00_JPRB, 0.77536E+00_JPRB, 0.79284E+00_JPRB, 0.81940E+00_JPRB, 0.83797E+00_JPRB, 0.85549E+00_JPRB, 0.87264E+00_JPRB ,&
&  0.90609E+00_JPRB, 0.94270E+00_JPRB, 0.97603E+00_JPRB, 0.10089E+01_JPRB, 0.10437E+01_JPRB, 0.10718E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.11648E+01_JPRB, 0.12123E+01_JPRB, 0.12440E+01_JPRB, 0.12812E+01_JPRB, 0.13147E+01_JPRB, 0.13056E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.13446E+01_JPRB, 0.13973E+01_JPRB, 0.14428E+01_JPRB, 0.15326E+01_JPRB, 0.16488E+01_JPRB, 0.17125E+01_JPRB, 0.20691E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.21369E+01_JPRB, 0.17673E+01_JPRB, 0.15108E+01_JPRB, 0.14362E+01_JPRB, 0.12832E+01_JPRB, 0.11966E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.12039E+01_JPRB, 0.11837E+01_JPRB, 0.11384E+01_JPRB, 0.10732E+01_JPRB, 0.11364E+01_JPRB, 0.12852E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.16911E+01_JPRB, 0.18933E+01_JPRB, 0.20695E+01_JPRB, 0.23027E+01_JPRB, 0.24551E+01_JPRB, 0.24322E+01_JPRB ,&
&  0.24471E+01_JPRB, 0.24190E+01_JPRB, 0.23144E+01_JPRB, 0.26730E+01_JPRB, 0.29505E+01_JPRB, 0.30817E+01_JPRB, 0.31246E+01_JPRB ,&
&  0.31439E+01_JPRB, 0.28384E+01_JPRB, 0.27674E+01_JPRB, 0.27076E+01_JPRB, 0.25939E+01_JPRB, 0.26332E+01_JPRB, 0.25077E+01_JPRB ,&
&  0.23279E+01_JPRB, 0.21993E+01_JPRB, 0.21609E+01_JPRB, 0.21708E+01_JPRB, 0.22330E+01_JPRB, 0.22510E+01_JPRB, 0.20948E+01_JPRB ,&
&  0.19787E+01_JPRB, 0.19187E+01_JPRB, 0.18764E+01_JPRB, 0.18767E+01_JPRB, 0.21135E+01_JPRB, 0.45446E+01_JPRB, 0.56119E+01_JPRB ,&
&  0.21422E+01_JPRB, 0.14805E+01_JPRB, 0.82054E+00_JPRB, 0.99508E+00_JPRB, 0.78964E+00_JPRB, 0.71705E+00_JPRB, 0.78400E+00_JPRB ,&
&  0.79669E+00_JPRB, 0.78028E+00_JPRB, 0.71160E+00_JPRB, 0.64649E+00_JPRB, 0.67142E+00_JPRB, 0.68859E+00_JPRB, 0.68871E+00_JPRB ,&
&  0.70770E+00_JPRB, 0.71603E+00_JPRB, 0.73069E+00_JPRB, 0.75767E+00_JPRB, 0.75667E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.82470E+00_JPRB, 0.84840E+00_JPRB, 0.87764E+00_JPRB, 0.91541E+00_JPRB, 0.95301E+00_JPRB, 0.98473E+00_JPRB, 0.10091E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10912E+01_JPRB, 0.11228E+01_JPRB, 0.11409E+01_JPRB, 0.11776E+01_JPRB, 0.12086E+01_JPRB, 0.12613E+01_JPRB ,&
&  0.12985E+01_JPRB, 0.13736E+01_JPRB, 0.14394E+01_JPRB, 0.15002E+01_JPRB, 0.15527E+01_JPRB, 0.16091E+01_JPRB, 0.16621E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.17892E+01_JPRB, 0.18505E+01_JPRB, 0.18833E+01_JPRB, 0.19668E+01_JPRB, 0.21565E+01_JPRB, 0.19175E+01_JPRB ,&
&  0.23367E+01_JPRB, 0.20237E+01_JPRB, 0.17471E+01_JPRB, 0.15657E+01_JPRB, 0.14399E+01_JPRB, 0.13663E+01_JPRB, 0.13336E+01_JPRB ,&
&  0.13742E+01_JPRB, 0.14781E+01_JPRB, 0.15673E+01_JPRB, 0.15550E+01_JPRB, 0.14689E+01_JPRB, 0.16270E+01_JPRB, 0.19726E+01_JPRB ,&
&  0.21620E+01_JPRB, 0.22718E+01_JPRB, 0.23551E+01_JPRB, 0.23851E+01_JPRB, 0.24942E+01_JPRB, 0.26502E+01_JPRB, 0.27338E+01_JPRB ,&
&  0.27313E+01_JPRB, 0.26490E+01_JPRB, 0.23673E+01_JPRB, 0.28669E+01_JPRB, 0.28484E+01_JPRB, 0.27171E+01_JPRB, 0.24697E+01_JPRB ,&
&  0.22325E+01_JPRB, 0.20170E+01_JPRB, 0.21268E+01_JPRB, 0.23053E+01_JPRB, 0.22429E+01_JPRB, 0.21963E+01_JPRB, 0.20528E+01_JPRB ,&
&  0.18993E+01_JPRB, 0.18706E+01_JPRB, 0.19154E+01_JPRB, 0.20194E+01_JPRB, 0.21134E+01_JPRB, 0.21202E+01_JPRB, 0.19817E+01_JPRB ,&
&  0.18417E+01_JPRB, 0.17073E+01_JPRB, 0.16373E+01_JPRB, 0.16294E+01_JPRB, 0.18295E+01_JPRB, 0.23133E+01_JPRB, 0.34207E+01_JPRB ,&
&  0.17835E+01_JPRB, 0.98306E+00_JPRB, 0.95882E+00_JPRB, 0.82374E+00_JPRB, 0.81627E+00_JPRB, 0.73350E+00_JPRB, 0.88109E+00_JPRB ,&
&  0.10895E+01_JPRB, 0.12367E+01_JPRB, 0.12100E+01_JPRB, 0.94570E+00_JPRB, 0.79122E+00_JPRB, 0.74912E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.76019E+00_JPRB, 0.76462E+00_JPRB, 0.77446E+00_JPRB, 0.78818E+00_JPRB, 0.80629E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.80288E+00_JPRB, 0.82072E+00_JPRB, 0.84255E+00_JPRB, 0.86607E+00_JPRB, 0.89446E+00_JPRB, 0.92406E+00_JPRB, 0.94605E+00_JPRB ,&
&  0.97213E+00_JPRB, 0.10027E+01_JPRB, 0.10318E+01_JPRB, 0.10536E+01_JPRB, 0.10814E+01_JPRB, 0.11281E+01_JPRB, 0.11839E+01_JPRB ,&
&  0.12222E+01_JPRB, 0.13103E+01_JPRB, 0.13742E+01_JPRB, 0.14695E+01_JPRB, 0.15658E+01_JPRB, 0.16729E+01_JPRB, 0.17593E+01_JPRB ,&
&  0.18544E+01_JPRB, 0.19943E+01_JPRB, 0.20719E+01_JPRB, 0.20794E+01_JPRB, 0.21510E+01_JPRB, 0.24401E+01_JPRB, 0.21022E+01_JPRB ,&
&  0.23340E+01_JPRB, 0.21503E+01_JPRB, 0.19234E+01_JPRB, 0.18328E+01_JPRB, 0.17673E+01_JPRB, 0.17839E+01_JPRB, 0.18452E+01_JPRB ,&
&  0.19577E+01_JPRB, 0.21612E+01_JPRB, 0.20890E+01_JPRB, 0.21143E+01_JPRB, 0.21576E+01_JPRB, 0.24359E+01_JPRB, 0.27207E+01_JPRB ,&
&  0.28347E+01_JPRB, 0.30430E+01_JPRB, 0.31514E+01_JPRB, 0.32540E+01_JPRB, 0.34174E+01_JPRB, 0.35895E+01_JPRB, 0.37115E+01_JPRB ,&
&  0.36699E+01_JPRB, 0.35963E+01_JPRB, 0.33277E+01_JPRB, 0.35629E+01_JPRB, 0.33628E+01_JPRB, 0.27343E+01_JPRB, 0.21551E+01_JPRB ,&
&  0.16236E+01_JPRB, 0.16867E+01_JPRB, 0.17963E+01_JPRB, 0.19201E+01_JPRB, 0.18132E+01_JPRB, 0.17155E+01_JPRB, 0.14841E+01_JPRB ,&
&  0.13942E+01_JPRB, 0.13918E+01_JPRB, 0.14850E+01_JPRB, 0.16239E+01_JPRB, 0.17062E+01_JPRB, 0.16959E+01_JPRB, 0.15838E+01_JPRB ,&
&  0.14788E+01_JPRB, 0.13632E+01_JPRB, 0.13374E+01_JPRB, 0.13839E+01_JPRB, 0.14405E+01_JPRB, 0.14726E+01_JPRB, 0.20301E+01_JPRB ,&
&  0.13898E+01_JPRB, 0.11964E+01_JPRB, 0.13698E+01_JPRB, 0.59546E+00_JPRB, 0.75054E+00_JPRB, 0.93771E+00_JPRB, 0.12106E+01_JPRB ,&
&  0.14036E+01_JPRB, 0.19260E+01_JPRB, 0.23416E+01_JPRB, 0.17843E+01_JPRB, 0.12739E+01_JPRB, 0.80078E+00_JPRB, 0.71123E+00_JPRB ,&
&  0.72932E+00_JPRB, 0.73881E+00_JPRB, 0.75307E+00_JPRB, 0.75921E+00_JPRB, 0.78023E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.72604E+00_JPRB, 0.72411E+00_JPRB, 0.72693E+00_JPRB, 0.71850E+00_JPRB, 0.71622E+00_JPRB, 0.71882E+00_JPRB, 0.71003E+00_JPRB ,&
&  0.71737E+00_JPRB, 0.73818E+00_JPRB, 0.74716E+00_JPRB, 0.77743E+00_JPRB, 0.80217E+00_JPRB, 0.87031E+00_JPRB, 0.97315E+00_JPRB ,&
&  0.10643E+01_JPRB, 0.12358E+01_JPRB, 0.13539E+01_JPRB, 0.14867E+01_JPRB, 0.16065E+01_JPRB, 0.17370E+01_JPRB, 0.18501E+01_JPRB ,&
&  0.19947E+01_JPRB, 0.21811E+01_JPRB, 0.23681E+01_JPRB, 0.25103E+01_JPRB, 0.25733E+01_JPRB, 0.29808E+01_JPRB, 0.23353E+01_JPRB ,&
&  0.24791E+01_JPRB, 0.24401E+01_JPRB, 0.22894E+01_JPRB, 0.22125E+01_JPRB, 0.21386E+01_JPRB, 0.21043E+01_JPRB, 0.20871E+01_JPRB ,&
&  0.21435E+01_JPRB, 0.21882E+01_JPRB, 0.22085E+01_JPRB, 0.24254E+01_JPRB, 0.26736E+01_JPRB, 0.28871E+01_JPRB, 0.31285E+01_JPRB ,&
&  0.33476E+01_JPRB, 0.35670E+01_JPRB, 0.38378E+01_JPRB, 0.41207E+01_JPRB, 0.44097E+01_JPRB, 0.45902E+01_JPRB, 0.48055E+01_JPRB ,&
&  0.49148E+01_JPRB, 0.49895E+01_JPRB, 0.46650E+01_JPRB, 0.40787E+01_JPRB, 0.34489E+01_JPRB, 0.27309E+01_JPRB, 0.20320E+01_JPRB ,&
&  0.15560E+01_JPRB, 0.14750E+01_JPRB, 0.14763E+01_JPRB, 0.14702E+01_JPRB, 0.13114E+01_JPRB, 0.12076E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.99979E+00_JPRB, 0.10580E+01_JPRB, 0.11548E+01_JPRB, 0.11953E+01_JPRB, 0.11867E+01_JPRB, 0.11632E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.10671E+01_JPRB, 0.10997E+01_JPRB, 0.11602E+01_JPRB, 0.12795E+01_JPRB, 0.14718E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.92937E+00_JPRB, 0.86847E+00_JPRB, 0.84461E+00_JPRB, 0.78968E+00_JPRB, 0.99920E+00_JPRB, 0.13534E+01_JPRB ,&
&  0.19857E+01_JPRB, 0.21863E+01_JPRB, 0.23866E+01_JPRB, 0.29222E+01_JPRB, 0.25400E+01_JPRB, 0.19099E+01_JPRB, 0.64347E+00_JPRB ,&
&  0.64807E+00_JPRB, 0.65669E+00_JPRB, 0.69078E+00_JPRB, 0.70334E+00_JPRB, 0.71097E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.60705E+00_JPRB, 0.58816E+00_JPRB, 0.57982E+00_JPRB, 0.57708E+00_JPRB, 0.56122E+00_JPRB, 0.56428E+00_JPRB, 0.56439E+00_JPRB ,&
&  0.58684E+00_JPRB, 0.61947E+00_JPRB, 0.67504E+00_JPRB, 0.74767E+00_JPRB, 0.84974E+00_JPRB, 0.94394E+00_JPRB, 0.10655E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.12757E+01_JPRB, 0.13888E+01_JPRB, 0.15091E+01_JPRB, 0.16415E+01_JPRB, 0.17645E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.20689E+01_JPRB, 0.22412E+01_JPRB, 0.24510E+01_JPRB, 0.26379E+01_JPRB, 0.27821E+01_JPRB, 0.28938E+01_JPRB, 0.22254E+01_JPRB ,&
&  0.26228E+01_JPRB, 0.26981E+01_JPRB, 0.27066E+01_JPRB, 0.26009E+01_JPRB, 0.24360E+01_JPRB, 0.22819E+01_JPRB, 0.22485E+01_JPRB ,&
&  0.22500E+01_JPRB, 0.22314E+01_JPRB, 0.21957E+01_JPRB, 0.22437E+01_JPRB, 0.27502E+01_JPRB, 0.29504E+01_JPRB, 0.31922E+01_JPRB ,&
&  0.34455E+01_JPRB, 0.37311E+01_JPRB, 0.40268E+01_JPRB, 0.43298E+01_JPRB, 0.45369E+01_JPRB, 0.47636E+01_JPRB, 0.50003E+01_JPRB ,&
&  0.51849E+01_JPRB, 0.53431E+01_JPRB, 0.51175E+01_JPRB, 0.40037E+01_JPRB, 0.32992E+01_JPRB, 0.26877E+01_JPRB, 0.17831E+01_JPRB ,&
&  0.13879E+01_JPRB, 0.12849E+01_JPRB, 0.12836E+01_JPRB, 0.12406E+01_JPRB, 0.11044E+01_JPRB, 0.10048E+01_JPRB, 0.92953E+00_JPRB ,&
&  0.81896E+00_JPRB, 0.78974E+00_JPRB, 0.78719E+00_JPRB, 0.85734E+00_JPRB, 0.89812E+00_JPRB, 0.85824E+00_JPRB, 0.84183E+00_JPRB ,&
&  0.80713E+00_JPRB, 0.78473E+00_JPRB, 0.86081E+00_JPRB, 0.98735E+00_JPRB, 0.12546E+01_JPRB, 0.15522E+01_JPRB, 0.17684E+01_JPRB ,&
&  0.17290E+01_JPRB, 0.94090E+00_JPRB, 0.82147E+00_JPRB, 0.77771E+00_JPRB, 0.77809E+00_JPRB, 0.83087E+00_JPRB, 0.92614E+00_JPRB ,&
&  0.11519E+01_JPRB, 0.15503E+01_JPRB, 0.13425E+01_JPRB, 0.10325E+01_JPRB, 0.10619E+01_JPRB, 0.98503E+00_JPRB, 0.59048E+00_JPRB ,&
&  0.59564E+00_JPRB, 0.59105E+00_JPRB, 0.60998E+00_JPRB, 0.61704E+00_JPRB, 0.62134E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.53508E+00_JPRB, 0.51132E+00_JPRB, 0.48864E+00_JPRB, 0.48000E+00_JPRB, 0.47443E+00_JPRB, 0.46878E+00_JPRB, 0.48656E+00_JPRB ,&
&  0.52845E+00_JPRB, 0.57009E+00_JPRB, 0.63931E+00_JPRB, 0.74670E+00_JPRB, 0.86621E+00_JPRB, 0.95594E+00_JPRB, 0.10496E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.12260E+01_JPRB, 0.13016E+01_JPRB, 0.13798E+01_JPRB, 0.14821E+01_JPRB, 0.16135E+01_JPRB, 0.17936E+01_JPRB ,&
&  0.20049E+01_JPRB, 0.22133E+01_JPRB, 0.24649E+01_JPRB, 0.27600E+01_JPRB, 0.30459E+01_JPRB, 0.31757E+01_JPRB, 0.34202E+01_JPRB ,&
&  0.22906E+01_JPRB, 0.28893E+01_JPRB, 0.28877E+01_JPRB, 0.27934E+01_JPRB, 0.26657E+01_JPRB, 0.25431E+01_JPRB, 0.24651E+01_JPRB ,&
&  0.24101E+01_JPRB, 0.22130E+01_JPRB, 0.21622E+01_JPRB, 0.21720E+01_JPRB, 0.24677E+01_JPRB, 0.27057E+01_JPRB, 0.29412E+01_JPRB ,&
&  0.31898E+01_JPRB, 0.34731E+01_JPRB, 0.37695E+01_JPRB, 0.40552E+01_JPRB, 0.42828E+01_JPRB, 0.45336E+01_JPRB, 0.47230E+01_JPRB ,&
&  0.49547E+01_JPRB, 0.51512E+01_JPRB, 0.52121E+01_JPRB, 0.42522E+01_JPRB, 0.33748E+01_JPRB, 0.28854E+01_JPRB, 0.20826E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.11503E+01_JPRB, 0.11699E+01_JPRB, 0.11943E+01_JPRB, 0.10963E+01_JPRB, 0.10125E+01_JPRB, 0.90849E+00_JPRB ,&
&  0.82303E+00_JPRB, 0.72878E+00_JPRB, 0.69014E+00_JPRB, 0.71338E+00_JPRB, 0.77370E+00_JPRB, 0.73035E+00_JPRB, 0.67548E+00_JPRB ,&
&  0.64168E+00_JPRB, 0.63565E+00_JPRB, 0.71507E+00_JPRB, 0.81122E+00_JPRB, 0.94518E+00_JPRB, 0.10206E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.87915E+00_JPRB, 0.83214E+00_JPRB, 0.82994E+00_JPRB, 0.81428E+00_JPRB, 0.81434E+00_JPRB, 0.83071E+00_JPRB ,&
&  0.86663E+00_JPRB, 0.93624E+00_JPRB, 0.94915E+00_JPRB, 0.88393E+00_JPRB, 0.78688E+00_JPRB, 0.72783E+00_JPRB, 0.67924E+00_JPRB ,&
&  0.68185E+00_JPRB, 0.66726E+00_JPRB, 0.65050E+00_JPRB, 0.61768E+00_JPRB, 0.57837E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.55123E+00_JPRB, 0.51161E+00_JPRB, 0.46199E+00_JPRB, 0.42296E+00_JPRB, 0.45300E+00_JPRB, 0.46043E+00_JPRB, 0.48569E+00_JPRB ,&
&  0.50070E+00_JPRB, 0.54067E+00_JPRB, 0.58700E+00_JPRB, 0.65496E+00_JPRB, 0.75237E+00_JPRB, 0.85415E+00_JPRB, 0.94476E+00_JPRB ,&
&  0.10281E+01_JPRB, 0.11068E+01_JPRB, 0.11664E+01_JPRB, 0.11978E+01_JPRB, 0.12481E+01_JPRB, 0.13293E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.15863E+01_JPRB, 0.17782E+01_JPRB, 0.20322E+01_JPRB, 0.23512E+01_JPRB, 0.27154E+01_JPRB, 0.29379E+01_JPRB, 0.31177E+01_JPRB ,&
&  0.16424E+01_JPRB, 0.25946E+01_JPRB, 0.31151E+01_JPRB, 0.29015E+01_JPRB, 0.28052E+01_JPRB, 0.27082E+01_JPRB, 0.26147E+01_JPRB ,&
&  0.23891E+01_JPRB, 0.21158E+01_JPRB, 0.20209E+01_JPRB, 0.22419E+01_JPRB, 0.21442E+01_JPRB, 0.23632E+01_JPRB, 0.25327E+01_JPRB ,&
&  0.27160E+01_JPRB, 0.29130E+01_JPRB, 0.31442E+01_JPRB, 0.34403E+01_JPRB, 0.37474E+01_JPRB, 0.40037E+01_JPRB, 0.42224E+01_JPRB ,&
&  0.45193E+01_JPRB, 0.47946E+01_JPRB, 0.48898E+01_JPRB, 0.37744E+01_JPRB, 0.34107E+01_JPRB, 0.31816E+01_JPRB, 0.28131E+01_JPRB ,&
&  0.16817E+01_JPRB, 0.11751E+01_JPRB, 0.11824E+01_JPRB, 0.12271E+01_JPRB, 0.11554E+01_JPRB, 0.10829E+01_JPRB, 0.10023E+01_JPRB ,&
&  0.89082E+00_JPRB, 0.80690E+00_JPRB, 0.74085E+00_JPRB, 0.71277E+00_JPRB, 0.73989E+00_JPRB, 0.73802E+00_JPRB, 0.70069E+00_JPRB ,&
&  0.66091E+00_JPRB, 0.66585E+00_JPRB, 0.72067E+00_JPRB, 0.77014E+00_JPRB, 0.84628E+00_JPRB, 0.88933E+00_JPRB, 0.94957E+00_JPRB ,&
&  0.95571E+00_JPRB, 0.85569E+00_JPRB, 0.84209E+00_JPRB, 0.85384E+00_JPRB, 0.86000E+00_JPRB, 0.86634E+00_JPRB, 0.86802E+00_JPRB ,&
&  0.86569E+00_JPRB, 0.88631E+00_JPRB, 0.89202E+00_JPRB, 0.89583E+00_JPRB, 0.86409E+00_JPRB, 0.82983E+00_JPRB, 0.80516E+00_JPRB ,&
&  0.77417E+00_JPRB, 0.73807E+00_JPRB, 0.70840E+00_JPRB, 0.65539E+00_JPRB, 0.59998E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.58205E+00_JPRB, 0.53422E+00_JPRB, 0.48624E+00_JPRB, 0.46965E+00_JPRB, 0.50310E+00_JPRB, 0.53508E+00_JPRB, 0.53564E+00_JPRB ,&
&  0.53292E+00_JPRB, 0.52473E+00_JPRB, 0.52666E+00_JPRB, 0.54943E+00_JPRB, 0.61467E+00_JPRB, 0.71571E+00_JPRB, 0.80523E+00_JPRB ,&
&  0.88269E+00_JPRB, 0.93984E+00_JPRB, 0.98335E+00_JPRB, 0.99656E+00_JPRB, 0.98556E+00_JPRB, 0.10128E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.11638E+01_JPRB, 0.12930E+01_JPRB, 0.14665E+01_JPRB, 0.17264E+01_JPRB, 0.21147E+01_JPRB, 0.25928E+01_JPRB, 0.30837E+01_JPRB ,&
&  0.21610E+01_JPRB, 0.90421E+00_JPRB, 0.28159E+01_JPRB, 0.33655E+01_JPRB, 0.31405E+01_JPRB, 0.30796E+01_JPRB, 0.27932E+01_JPRB ,&
&  0.25019E+01_JPRB, 0.24173E+01_JPRB, 0.21868E+01_JPRB, 0.22455E+01_JPRB, 0.20598E+01_JPRB, 0.20829E+01_JPRB, 0.21165E+01_JPRB ,&
&  0.21873E+01_JPRB, 0.22830E+01_JPRB, 0.24413E+01_JPRB, 0.26399E+01_JPRB, 0.28703E+01_JPRB, 0.32361E+01_JPRB, 0.35026E+01_JPRB ,&
&  0.37390E+01_JPRB, 0.39758E+01_JPRB, 0.40408E+01_JPRB, 0.29398E+01_JPRB, 0.23773E+01_JPRB, 0.20885E+01_JPRB, 0.18795E+01_JPRB ,&
&  0.16037E+01_JPRB, 0.12794E+01_JPRB, 0.12412E+01_JPRB, 0.12824E+01_JPRB, 0.12397E+01_JPRB, 0.11353E+01_JPRB, 0.11189E+01_JPRB ,&
&  0.99192E+00_JPRB, 0.90378E+00_JPRB, 0.82935E+00_JPRB, 0.77051E+00_JPRB, 0.77028E+00_JPRB, 0.77890E+00_JPRB, 0.78227E+00_JPRB ,&
&  0.75804E+00_JPRB, 0.75202E+00_JPRB, 0.77543E+00_JPRB, 0.77840E+00_JPRB, 0.78557E+00_JPRB, 0.79396E+00_JPRB, 0.80834E+00_JPRB ,&
&  0.85654E+00_JPRB, 0.84413E+00_JPRB, 0.84948E+00_JPRB, 0.87067E+00_JPRB, 0.88333E+00_JPRB, 0.87155E+00_JPRB, 0.88224E+00_JPRB ,&
&  0.90019E+00_JPRB, 0.91843E+00_JPRB, 0.90994E+00_JPRB, 0.92218E+00_JPRB, 0.90720E+00_JPRB, 0.89466E+00_JPRB, 0.89053E+00_JPRB ,&
&  0.88287E+00_JPRB, 0.85894E+00_JPRB, 0.80836E+00_JPRB, 0.70937E+00_JPRB, 0.64322E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.63674E+00_JPRB, 0.55768E+00_JPRB, 0.51465E+00_JPRB, 0.52064E+00_JPRB, 0.55284E+00_JPRB, 0.56720E+00_JPRB, 0.57287E+00_JPRB ,&
&  0.56458E+00_JPRB, 0.55126E+00_JPRB, 0.52992E+00_JPRB, 0.52179E+00_JPRB, 0.53486E+00_JPRB, 0.56452E+00_JPRB, 0.63350E+00_JPRB ,&
&  0.70181E+00_JPRB, 0.76714E+00_JPRB, 0.81674E+00_JPRB, 0.81306E+00_JPRB, 0.76826E+00_JPRB, 0.75869E+00_JPRB, 0.78146E+00_JPRB ,&
&  0.82283E+00_JPRB, 0.89812E+00_JPRB, 0.10043E+01_JPRB, 0.11531E+01_JPRB, 0.13948E+01_JPRB, 0.18168E+01_JPRB, 0.24869E+01_JPRB ,&
&  0.37310E+01_JPRB, 0.25104E+01_JPRB, 0.15017E+01_JPRB, 0.30693E+01_JPRB, 0.37073E+01_JPRB, 0.38684E+01_JPRB, 0.31614E+01_JPRB ,&
&  0.30273E+01_JPRB, 0.27955E+01_JPRB, 0.25167E+01_JPRB, 0.24445E+01_JPRB, 0.21446E+01_JPRB, 0.20331E+01_JPRB, 0.19335E+01_JPRB ,&
&  0.18530E+01_JPRB, 0.18986E+01_JPRB, 0.19939E+01_JPRB, 0.20874E+01_JPRB, 0.21879E+01_JPRB, 0.24265E+01_JPRB, 0.27398E+01_JPRB ,&
&  0.30542E+01_JPRB, 0.32536E+01_JPRB, 0.33242E+01_JPRB, 0.25034E+01_JPRB, 0.20893E+01_JPRB, 0.18913E+01_JPRB, 0.17869E+01_JPRB ,&
&  0.15833E+01_JPRB, 0.14630E+01_JPRB, 0.14309E+01_JPRB, 0.13731E+01_JPRB, 0.13010E+01_JPRB, 0.12263E+01_JPRB, 0.12322E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.10218E+01_JPRB, 0.95163E+00_JPRB, 0.90630E+00_JPRB, 0.88745E+00_JPRB, 0.88150E+00_JPRB, 0.86852E+00_JPRB ,&
&  0.87080E+00_JPRB, 0.86938E+00_JPRB, 0.86952E+00_JPRB, 0.84294E+00_JPRB, 0.81376E+00_JPRB, 0.79551E+00_JPRB, 0.77429E+00_JPRB ,&
&  0.80038E+00_JPRB, 0.83129E+00_JPRB, 0.84953E+00_JPRB, 0.87861E+00_JPRB, 0.88191E+00_JPRB, 0.87710E+00_JPRB, 0.90308E+00_JPRB ,&
&  0.91299E+00_JPRB, 0.93330E+00_JPRB, 0.92455E+00_JPRB, 0.93292E+00_JPRB, 0.91611E+00_JPRB, 0.90169E+00_JPRB, 0.90294E+00_JPRB ,&
&  0.93591E+00_JPRB, 0.96849E+00_JPRB, 0.93030E+00_JPRB, 0.75432E+00_JPRB, 0.68155E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.62082E+00_JPRB, 0.58347E+00_JPRB, 0.54096E+00_JPRB, 0.55053E+00_JPRB, 0.55995E+00_JPRB, 0.57394E+00_JPRB, 0.58212E+00_JPRB ,&
&  0.58808E+00_JPRB, 0.59065E+00_JPRB, 0.59379E+00_JPRB, 0.58070E+00_JPRB, 0.54788E+00_JPRB, 0.53524E+00_JPRB, 0.53827E+00_JPRB ,&
&  0.55906E+00_JPRB, 0.60574E+00_JPRB, 0.65843E+00_JPRB, 0.66842E+00_JPRB, 0.65680E+00_JPRB, 0.64370E+00_JPRB, 0.63897E+00_JPRB ,&
&  0.64878E+00_JPRB, 0.67163E+00_JPRB, 0.70775E+00_JPRB, 0.78184E+00_JPRB, 0.91459E+00_JPRB, 0.11297E+01_JPRB, 0.15269E+01_JPRB ,&
&  0.25446E+01_JPRB, 0.27740E+01_JPRB, 0.11609E+01_JPRB, 0.24645E+01_JPRB, 0.38297E+01_JPRB, 0.41220E+01_JPRB, 0.39272E+01_JPRB ,&
&  0.37002E+01_JPRB, 0.32725E+01_JPRB, 0.29135E+01_JPRB, 0.25465E+01_JPRB, 0.21538E+01_JPRB, 0.19434E+01_JPRB, 0.17712E+01_JPRB ,&
&  0.16373E+01_JPRB, 0.16370E+01_JPRB, 0.17254E+01_JPRB, 0.17927E+01_JPRB, 0.18094E+01_JPRB, 0.18789E+01_JPRB, 0.21020E+01_JPRB ,&
&  0.24453E+01_JPRB, 0.28005E+01_JPRB, 0.29643E+01_JPRB, 0.23658E+01_JPRB, 0.20844E+01_JPRB, 0.20856E+01_JPRB, 0.20047E+01_JPRB ,&
&  0.18529E+01_JPRB, 0.16700E+01_JPRB, 0.15459E+01_JPRB, 0.14632E+01_JPRB, 0.13692E+01_JPRB, 0.13301E+01_JPRB, 0.13444E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.11472E+01_JPRB, 0.10705E+01_JPRB, 0.10198E+01_JPRB, 0.98564E+00_JPRB, 0.96556E+00_JPRB, 0.94772E+00_JPRB ,&
&  0.93637E+00_JPRB, 0.94383E+00_JPRB, 0.95665E+00_JPRB, 0.93960E+00_JPRB, 0.90152E+00_JPRB, 0.85890E+00_JPRB, 0.81782E+00_JPRB ,&
&  0.80916E+00_JPRB, 0.81861E+00_JPRB, 0.85004E+00_JPRB, 0.86853E+00_JPRB, 0.88181E+00_JPRB, 0.89673E+00_JPRB, 0.91238E+00_JPRB ,&
&  0.93165E+00_JPRB, 0.94693E+00_JPRB, 0.92168E+00_JPRB, 0.90272E+00_JPRB, 0.91628E+00_JPRB, 0.89353E+00_JPRB, 0.87253E+00_JPRB ,&
&  0.90508E+00_JPRB, 0.96858E+00_JPRB, 0.10464E+01_JPRB, 0.75912E+00_JPRB, 0.67913E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.61948E+00_JPRB, 0.57601E+00_JPRB, 0.56985E+00_JPRB, 0.57225E+00_JPRB, 0.58129E+00_JPRB, 0.59687E+00_JPRB, 0.61465E+00_JPRB ,&
&  0.62187E+00_JPRB, 0.62384E+00_JPRB, 0.62367E+00_JPRB, 0.61786E+00_JPRB, 0.59988E+00_JPRB, 0.58076E+00_JPRB, 0.56116E+00_JPRB ,&
&  0.55340E+00_JPRB, 0.55900E+00_JPRB, 0.58045E+00_JPRB, 0.59098E+00_JPRB, 0.59113E+00_JPRB, 0.58691E+00_JPRB, 0.58793E+00_JPRB ,&
&  0.59418E+00_JPRB, 0.59775E+00_JPRB, 0.60121E+00_JPRB, 0.62871E+00_JPRB, 0.68927E+00_JPRB, 0.80959E+00_JPRB, 0.10490E+01_JPRB ,&
&  0.17493E+01_JPRB, 0.23542E+01_JPRB, 0.22446E+01_JPRB, 0.37409E+01_JPRB, 0.46401E+01_JPRB, 0.46060E+01_JPRB, 0.45442E+01_JPRB ,&
&  0.42720E+01_JPRB, 0.37827E+01_JPRB, 0.35064E+01_JPRB, 0.27630E+01_JPRB, 0.22584E+01_JPRB, 0.19018E+01_JPRB, 0.16424E+01_JPRB ,&
&  0.14382E+01_JPRB, 0.13578E+01_JPRB, 0.14021E+01_JPRB, 0.15060E+01_JPRB, 0.15773E+01_JPRB, 0.16042E+01_JPRB, 0.16932E+01_JPRB ,&
&  0.19297E+01_JPRB, 0.23693E+01_JPRB, 0.27316E+01_JPRB, 0.25318E+01_JPRB, 0.20829E+01_JPRB, 0.22784E+01_JPRB, 0.24910E+01_JPRB ,&
&  0.27901E+01_JPRB, 0.20867E+01_JPRB, 0.17003E+01_JPRB, 0.15518E+01_JPRB, 0.14290E+01_JPRB, 0.15226E+01_JPRB, 0.14592E+01_JPRB ,&
&  0.13654E+01_JPRB, 0.12649E+01_JPRB, 0.11745E+01_JPRB, 0.11138E+01_JPRB, 0.10667E+01_JPRB, 0.10313E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.99395E+00_JPRB, 0.98735E+00_JPRB, 0.99662E+00_JPRB, 0.99633E+00_JPRB, 0.97861E+00_JPRB, 0.94191E+00_JPRB, 0.89203E+00_JPRB ,&
&  0.85120E+00_JPRB, 0.83823E+00_JPRB, 0.82394E+00_JPRB, 0.84135E+00_JPRB, 0.86754E+00_JPRB, 0.88229E+00_JPRB, 0.88209E+00_JPRB ,&
&  0.90058E+00_JPRB, 0.92209E+00_JPRB, 0.91772E+00_JPRB, 0.90177E+00_JPRB, 0.89606E+00_JPRB, 0.89042E+00_JPRB, 0.85010E+00_JPRB ,&
&  0.82822E+00_JPRB, 0.81417E+00_JPRB, 0.76451E+00_JPRB, 0.72116E+00_JPRB, 0.66530E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.63118E+00_JPRB, 0.60459E+00_JPRB, 0.59837E+00_JPRB, 0.60462E+00_JPRB, 0.61324E+00_JPRB, 0.62411E+00_JPRB, 0.64068E+00_JPRB ,&
&  0.64943E+00_JPRB, 0.65113E+00_JPRB, 0.64958E+00_JPRB, 0.64593E+00_JPRB, 0.63334E+00_JPRB, 0.61787E+00_JPRB, 0.60058E+00_JPRB ,&
&  0.58423E+00_JPRB, 0.55721E+00_JPRB, 0.54378E+00_JPRB, 0.55022E+00_JPRB, 0.55281E+00_JPRB, 0.55399E+00_JPRB, 0.55694E+00_JPRB ,&
&  0.55814E+00_JPRB, 0.56460E+00_JPRB, 0.56506E+00_JPRB, 0.57628E+00_JPRB, 0.60479E+00_JPRB, 0.68139E+00_JPRB, 0.82809E+00_JPRB ,&
&  0.12844E+01_JPRB, 0.21180E+01_JPRB, 0.18052E+01_JPRB, 0.42871E+01_JPRB, 0.48740E+01_JPRB, 0.48275E+01_JPRB, 0.43708E+01_JPRB ,&
&  0.40306E+01_JPRB, 0.41677E+01_JPRB, 0.31929E+01_JPRB, 0.25586E+01_JPRB, 0.22071E+01_JPRB, 0.18981E+01_JPRB, 0.15740E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.11917E+01_JPRB, 0.11459E+01_JPRB, 0.11888E+01_JPRB, 0.12919E+01_JPRB, 0.13853E+01_JPRB, 0.14599E+01_JPRB ,&
&  0.15971E+01_JPRB, 0.19188E+01_JPRB, 0.23856E+01_JPRB, 0.23487E+01_JPRB, 0.22072E+01_JPRB, 0.24973E+01_JPRB, 0.30689E+01_JPRB ,&
&  0.36731E+01_JPRB, 0.29469E+01_JPRB, 0.20296E+01_JPRB, 0.16868E+01_JPRB, 0.15916E+01_JPRB, 0.16406E+01_JPRB, 0.16059E+01_JPRB ,&
&  0.14982E+01_JPRB, 0.14000E+01_JPRB, 0.12855E+01_JPRB, 0.12065E+01_JPRB, 0.11501E+01_JPRB, 0.10868E+01_JPRB, 0.10436E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10099E+01_JPRB, 0.10025E+01_JPRB, 0.99086E+00_JPRB, 0.98524E+00_JPRB, 0.97712E+00_JPRB, 0.95634E+00_JPRB ,&
&  0.91776E+00_JPRB, 0.87307E+00_JPRB, 0.82426E+00_JPRB, 0.81406E+00_JPRB, 0.83578E+00_JPRB, 0.85191E+00_JPRB, 0.85910E+00_JPRB ,&
&  0.89586E+00_JPRB, 0.90990E+00_JPRB, 0.89638E+00_JPRB, 0.86763E+00_JPRB, 0.87014E+00_JPRB, 0.90111E+00_JPRB, 0.85266E+00_JPRB ,&
&  0.83365E+00_JPRB, 0.81054E+00_JPRB, 0.77833E+00_JPRB, 0.74059E+00_JPRB, 0.68603E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.67552E+00_JPRB, 0.65489E+00_JPRB, 0.64322E+00_JPRB, 0.64132E+00_JPRB, 0.63790E+00_JPRB, 0.64117E+00_JPRB, 0.65309E+00_JPRB ,&
&  0.67048E+00_JPRB, 0.67589E+00_JPRB, 0.67344E+00_JPRB, 0.66979E+00_JPRB, 0.66131E+00_JPRB, 0.64685E+00_JPRB, 0.63011E+00_JPRB ,&
&  0.61522E+00_JPRB, 0.59430E+00_JPRB, 0.56579E+00_JPRB, 0.54316E+00_JPRB, 0.54369E+00_JPRB, 0.54860E+00_JPRB, 0.55162E+00_JPRB ,&
&  0.55266E+00_JPRB, 0.56051E+00_JPRB, 0.56398E+00_JPRB, 0.57498E+00_JPRB, 0.58550E+00_JPRB, 0.62866E+00_JPRB, 0.76461E+00_JPRB ,&
&  0.11174E+01_JPRB, 0.25698E+01_JPRB, 0.19449E+01_JPRB, 0.39860E+01_JPRB, 0.45564E+01_JPRB, 0.44425E+01_JPRB, 0.39501E+01_JPRB ,&
&  0.36501E+01_JPRB, 0.34230E+01_JPRB, 0.28061E+01_JPRB, 0.22868E+01_JPRB, 0.19591E+01_JPRB, 0.17106E+01_JPRB, 0.14980E+01_JPRB ,&
&  0.13333E+01_JPRB, 0.12271E+01_JPRB, 0.11606E+01_JPRB, 0.11190E+01_JPRB, 0.11344E+01_JPRB, 0.11925E+01_JPRB, 0.12719E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.15459E+01_JPRB, 0.19437E+01_JPRB, 0.23398E+01_JPRB, 0.21983E+01_JPRB, 0.25122E+01_JPRB, 0.36290E+01_JPRB ,&
&  0.42237E+01_JPRB, 0.41308E+01_JPRB, 0.27419E+01_JPRB, 0.19910E+01_JPRB, 0.17559E+01_JPRB, 0.17778E+01_JPRB, 0.17640E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.15757E+01_JPRB, 0.14467E+01_JPRB, 0.13097E+01_JPRB, 0.12090E+01_JPRB, 0.11365E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.10184E+01_JPRB, 0.99731E+00_JPRB, 0.98004E+00_JPRB, 0.96000E+00_JPRB, 0.94871E+00_JPRB, 0.94974E+00_JPRB, 0.95276E+00_JPRB ,&
&  0.94824E+00_JPRB, 0.92205E+00_JPRB, 0.86173E+00_JPRB, 0.82008E+00_JPRB, 0.81591E+00_JPRB, 0.82691E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.87991E+00_JPRB, 0.89215E+00_JPRB, 0.87072E+00_JPRB, 0.86064E+00_JPRB, 0.85452E+00_JPRB, 0.91626E+00_JPRB, 0.85923E+00_JPRB ,&
&  0.83307E+00_JPRB, 0.81036E+00_JPRB, 0.79789E+00_JPRB, 0.77225E+00_JPRB, 0.72305E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.71221E+00_JPRB, 0.68785E+00_JPRB, 0.67613E+00_JPRB, 0.66204E+00_JPRB, 0.65155E+00_JPRB, 0.64939E+00_JPRB, 0.65846E+00_JPRB ,&
&  0.67769E+00_JPRB, 0.69186E+00_JPRB, 0.68499E+00_JPRB, 0.68059E+00_JPRB, 0.67700E+00_JPRB, 0.66547E+00_JPRB, 0.65014E+00_JPRB ,&
&  0.63467E+00_JPRB, 0.62039E+00_JPRB, 0.60115E+00_JPRB, 0.57751E+00_JPRB, 0.56098E+00_JPRB, 0.55941E+00_JPRB, 0.55702E+00_JPRB ,&
&  0.55598E+00_JPRB, 0.56133E+00_JPRB, 0.56561E+00_JPRB, 0.57317E+00_JPRB, 0.58182E+00_JPRB, 0.60652E+00_JPRB, 0.73151E+00_JPRB ,&
&  0.11480E+01_JPRB, 0.88229E+00_JPRB, 0.24724E+01_JPRB, 0.34829E+01_JPRB, 0.36785E+01_JPRB, 0.34424E+01_JPRB, 0.33160E+01_JPRB ,&
&  0.31375E+01_JPRB, 0.28023E+01_JPRB, 0.24863E+01_JPRB, 0.22023E+01_JPRB, 0.18469E+01_JPRB, 0.16108E+01_JPRB, 0.14679E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.12702E+01_JPRB, 0.11937E+01_JPRB, 0.11164E+01_JPRB, 0.10615E+01_JPRB, 0.10534E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.12665E+01_JPRB, 0.15501E+01_JPRB, 0.19438E+01_JPRB, 0.21741E+01_JPRB, 0.24468E+01_JPRB, 0.30132E+01_JPRB ,&
&  0.39071E+01_JPRB, 0.43008E+01_JPRB, 0.33097E+01_JPRB, 0.24919E+01_JPRB, 0.20419E+01_JPRB, 0.19354E+01_JPRB, 0.19323E+01_JPRB ,&
&  0.18382E+01_JPRB, 0.17170E+01_JPRB, 0.15717E+01_JPRB, 0.14093E+01_JPRB, 0.12713E+01_JPRB, 0.11782E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.99688E+00_JPRB, 0.96433E+00_JPRB, 0.94071E+00_JPRB, 0.92503E+00_JPRB, 0.91900E+00_JPRB, 0.92325E+00_JPRB ,&
&  0.93675E+00_JPRB, 0.93731E+00_JPRB, 0.90667E+00_JPRB, 0.86666E+00_JPRB, 0.85338E+00_JPRB, 0.85827E+00_JPRB, 0.86817E+00_JPRB ,&
&  0.89458E+00_JPRB, 0.89566E+00_JPRB, 0.87216E+00_JPRB, 0.86417E+00_JPRB, 0.83284E+00_JPRB, 0.86649E+00_JPRB, 0.84239E+00_JPRB ,&
&  0.81936E+00_JPRB, 0.80643E+00_JPRB, 0.80561E+00_JPRB, 0.78967E+00_JPRB, 0.75231E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.73831E+00_JPRB, 0.71390E+00_JPRB, 0.69695E+00_JPRB, 0.67332E+00_JPRB, 0.66129E+00_JPRB, 0.65947E+00_JPRB, 0.66844E+00_JPRB ,&
&  0.67676E+00_JPRB, 0.68231E+00_JPRB, 0.67570E+00_JPRB, 0.66880E+00_JPRB, 0.65999E+00_JPRB, 0.65274E+00_JPRB, 0.64285E+00_JPRB ,&
&  0.63683E+00_JPRB, 0.63171E+00_JPRB, 0.62205E+00_JPRB, 0.60761E+00_JPRB, 0.58953E+00_JPRB, 0.57979E+00_JPRB, 0.57042E+00_JPRB ,&
&  0.56246E+00_JPRB, 0.55878E+00_JPRB, 0.55913E+00_JPRB, 0.56795E+00_JPRB, 0.58077E+00_JPRB, 0.60198E+00_JPRB, 0.68332E+00_JPRB ,&
&  0.11719E+01_JPRB, 0.18054E+01_JPRB, 0.29043E+01_JPRB, 0.30559E+01_JPRB, 0.27729E+01_JPRB, 0.24072E+01_JPRB, 0.23604E+01_JPRB ,&
&  0.22699E+01_JPRB, 0.21475E+01_JPRB, 0.19885E+01_JPRB, 0.18746E+01_JPRB, 0.17529E+01_JPRB, 0.16696E+01_JPRB, 0.15406E+01_JPRB ,&
&  0.14222E+01_JPRB, 0.12959E+01_JPRB, 0.11629E+01_JPRB, 0.10532E+01_JPRB, 0.97922E+00_JPRB, 0.94583E+00_JPRB, 0.95446E+00_JPRB ,&
&  0.98376E+00_JPRB, 0.10287E+01_JPRB, 0.11950E+01_JPRB, 0.15403E+01_JPRB, 0.20610E+01_JPRB, 0.21669E+01_JPRB, 0.28631E+01_JPRB ,&
&  0.38063E+01_JPRB, 0.37631E+01_JPRB, 0.33586E+01_JPRB, 0.29368E+01_JPRB, 0.24818E+01_JPRB, 0.23007E+01_JPRB, 0.22524E+01_JPRB ,&
&  0.20997E+01_JPRB, 0.18559E+01_JPRB, 0.16100E+01_JPRB, 0.14248E+01_JPRB, 0.12976E+01_JPRB, 0.12194E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.10737E+01_JPRB, 0.10190E+01_JPRB, 0.97464E+00_JPRB, 0.94677E+00_JPRB, 0.93396E+00_JPRB, 0.93020E+00_JPRB ,&
&  0.93351E+00_JPRB, 0.93423E+00_JPRB, 0.93114E+00_JPRB, 0.90900E+00_JPRB, 0.92057E+00_JPRB, 0.94947E+00_JPRB, 0.95931E+00_JPRB ,&
&  0.94216E+00_JPRB, 0.92429E+00_JPRB, 0.89438E+00_JPRB, 0.86793E+00_JPRB, 0.85609E+00_JPRB, 0.87035E+00_JPRB, 0.83643E+00_JPRB ,&
&  0.79964E+00_JPRB, 0.78936E+00_JPRB, 0.80186E+00_JPRB, 0.79750E+00_JPRB, 0.78052E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.76061E+00_JPRB, 0.73382E+00_JPRB, 0.70958E+00_JPRB, 0.68743E+00_JPRB, 0.67028E+00_JPRB, 0.66455E+00_JPRB, 0.66684E+00_JPRB ,&
&  0.66964E+00_JPRB, 0.66592E+00_JPRB, 0.65817E+00_JPRB, 0.64793E+00_JPRB, 0.64032E+00_JPRB, 0.63322E+00_JPRB, 0.62698E+00_JPRB ,&
&  0.61874E+00_JPRB, 0.61995E+00_JPRB, 0.61617E+00_JPRB, 0.60741E+00_JPRB, 0.59683E+00_JPRB, 0.58389E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.56282E+00_JPRB, 0.55866E+00_JPRB, 0.55766E+00_JPRB, 0.56201E+00_JPRB, 0.57616E+00_JPRB, 0.59489E+00_JPRB, 0.63787E+00_JPRB ,&
&  0.99574E+00_JPRB, 0.17094E+01_JPRB, 0.21915E+01_JPRB, 0.25305E+01_JPRB, 0.22613E+01_JPRB, 0.19176E+01_JPRB, 0.17143E+01_JPRB ,&
&  0.16284E+01_JPRB, 0.16177E+01_JPRB, 0.15431E+01_JPRB, 0.14470E+01_JPRB, 0.13298E+01_JPRB, 0.12698E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.10610E+01_JPRB, 0.10002E+01_JPRB, 0.96314E+00_JPRB, 0.92987E+00_JPRB, 0.89419E+00_JPRB, 0.87250E+00_JPRB ,&
&  0.87244E+00_JPRB, 0.88507E+00_JPRB, 0.94245E+00_JPRB, 0.11545E+01_JPRB, 0.16073E+01_JPRB, 0.19649E+01_JPRB, 0.24125E+01_JPRB ,&
&  0.25351E+01_JPRB, 0.26503E+01_JPRB, 0.27295E+01_JPRB, 0.26734E+01_JPRB, 0.24888E+01_JPRB, 0.23711E+01_JPRB, 0.23499E+01_JPRB ,&
&  0.22164E+01_JPRB, 0.18913E+01_JPRB, 0.15803E+01_JPRB, 0.13882E+01_JPRB, 0.12923E+01_JPRB, 0.12454E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.11481E+01_JPRB, 0.11019E+01_JPRB, 0.10489E+01_JPRB, 0.10011E+01_JPRB, 0.96190E+00_JPRB, 0.93912E+00_JPRB ,&
&  0.92202E+00_JPRB, 0.91374E+00_JPRB, 0.92400E+00_JPRB, 0.94325E+00_JPRB, 0.96254E+00_JPRB, 0.97958E+00_JPRB, 0.98871E+00_JPRB ,&
&  0.99300E+00_JPRB, 0.97707E+00_JPRB, 0.91315E+00_JPRB, 0.87723E+00_JPRB, 0.85391E+00_JPRB, 0.84738E+00_JPRB, 0.80672E+00_JPRB ,&
&  0.77534E+00_JPRB, 0.76328E+00_JPRB, 0.77332E+00_JPRB, 0.78093E+00_JPRB, 0.78290E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.83341E+00_JPRB, 0.78049E+00_JPRB, 0.73484E+00_JPRB, 0.69963E+00_JPRB, 0.67968E+00_JPRB, 0.66857E+00_JPRB, 0.65997E+00_JPRB ,&
&  0.65423E+00_JPRB, 0.64612E+00_JPRB, 0.63268E+00_JPRB, 0.62232E+00_JPRB, 0.61487E+00_JPRB, 0.61055E+00_JPRB, 0.60612E+00_JPRB ,&
&  0.59474E+00_JPRB, 0.59278E+00_JPRB, 0.59304E+00_JPRB, 0.58810E+00_JPRB, 0.58107E+00_JPRB, 0.57263E+00_JPRB, 0.56147E+00_JPRB ,&
&  0.55314E+00_JPRB, 0.54998E+00_JPRB, 0.55110E+00_JPRB, 0.55535E+00_JPRB, 0.56059E+00_JPRB, 0.57519E+00_JPRB, 0.62022E+00_JPRB ,&
&  0.92384E+00_JPRB, 0.13260E+01_JPRB, 0.16648E+01_JPRB, 0.18237E+01_JPRB, 0.15566E+01_JPRB, 0.14046E+01_JPRB, 0.13187E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.13207E+01_JPRB, 0.12462E+01_JPRB, 0.11230E+01_JPRB, 0.10079E+01_JPRB, 0.94482E+00_JPRB, 0.90235E+00_JPRB ,&
&  0.86441E+00_JPRB, 0.83342E+00_JPRB, 0.80579E+00_JPRB, 0.80005E+00_JPRB, 0.79622E+00_JPRB, 0.78696E+00_JPRB, 0.77798E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.77931E+00_JPRB, 0.79623E+00_JPRB, 0.87878E+00_JPRB, 0.11059E+01_JPRB, 0.13568E+01_JPRB, 0.16036E+01_JPRB ,&
&  0.17192E+01_JPRB, 0.18340E+01_JPRB, 0.19723E+01_JPRB, 0.20695E+01_JPRB, 0.20749E+01_JPRB, 0.20650E+01_JPRB, 0.20547E+01_JPRB ,&
&  0.19977E+01_JPRB, 0.18040E+01_JPRB, 0.15469E+01_JPRB, 0.13581E+01_JPRB, 0.12556E+01_JPRB, 0.12032E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.11390E+01_JPRB, 0.11107E+01_JPRB, 0.10720E+01_JPRB, 0.10208E+01_JPRB, 0.96666E+00_JPRB, 0.92820E+00_JPRB, 0.89998E+00_JPRB ,&
&  0.88241E+00_JPRB, 0.88472E+00_JPRB, 0.90567E+00_JPRB, 0.94188E+00_JPRB, 0.96712E+00_JPRB, 0.97839E+00_JPRB, 0.97466E+00_JPRB ,&
&  0.96520E+00_JPRB, 0.96570E+00_JPRB, 0.89908E+00_JPRB, 0.87478E+00_JPRB, 0.89907E+00_JPRB, 0.83842E+00_JPRB, 0.78137E+00_JPRB ,&
&  0.75355E+00_JPRB, 0.73396E+00_JPRB, 0.72956E+00_JPRB, 0.74716E+00_JPRB, 0.85789E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.76835E+00_JPRB, 0.75160E+00_JPRB, 0.72269E+00_JPRB, 0.69519E+00_JPRB, 0.67749E+00_JPRB, 0.66656E+00_JPRB, 0.65934E+00_JPRB ,&
&  0.65036E+00_JPRB, 0.63820E+00_JPRB, 0.62375E+00_JPRB, 0.60655E+00_JPRB, 0.59532E+00_JPRB, 0.58946E+00_JPRB, 0.58523E+00_JPRB ,&
&  0.57864E+00_JPRB, 0.56787E+00_JPRB, 0.56441E+00_JPRB, 0.56130E+00_JPRB, 0.55978E+00_JPRB, 0.55729E+00_JPRB, 0.55113E+00_JPRB ,&
&  0.54363E+00_JPRB, 0.53984E+00_JPRB, 0.54236E+00_JPRB, 0.54359E+00_JPRB, 0.54589E+00_JPRB, 0.55417E+00_JPRB, 0.58520E+00_JPRB ,&
&  0.77818E+00_JPRB, 0.99841E+00_JPRB, 0.12338E+01_JPRB, 0.14276E+01_JPRB, 0.13145E+01_JPRB, 0.12092E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.10204E+01_JPRB, 0.94552E+00_JPRB, 0.86132E+00_JPRB, 0.79016E+00_JPRB, 0.76071E+00_JPRB, 0.74383E+00_JPRB ,&
&  0.73003E+00_JPRB, 0.71786E+00_JPRB, 0.70798E+00_JPRB, 0.70276E+00_JPRB, 0.70250E+00_JPRB, 0.69996E+00_JPRB, 0.69712E+00_JPRB ,&
&  0.69768E+00_JPRB, 0.70374E+00_JPRB, 0.71298E+00_JPRB, 0.73638E+00_JPRB, 0.82437E+00_JPRB, 0.98269E+00_JPRB, 0.11515E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.13392E+01_JPRB, 0.14660E+01_JPRB, 0.15702E+01_JPRB, 0.16343E+01_JPRB, 0.16552E+01_JPRB, 0.16457E+01_JPRB ,&
&  0.16261E+01_JPRB, 0.15580E+01_JPRB, 0.14105E+01_JPRB, 0.12540E+01_JPRB, 0.11520E+01_JPRB, 0.10858E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.99981E+00_JPRB, 0.95665E+00_JPRB, 0.91512E+00_JPRB, 0.88017E+00_JPRB, 0.85636E+00_JPRB, 0.83473E+00_JPRB, 0.81824E+00_JPRB ,&
&  0.82348E+00_JPRB, 0.84967E+00_JPRB, 0.88733E+00_JPRB, 0.91180E+00_JPRB, 0.91924E+00_JPRB, 0.92056E+00_JPRB, 0.91180E+00_JPRB ,&
&  0.90277E+00_JPRB, 0.90198E+00_JPRB, 0.88754E+00_JPRB, 0.88402E+00_JPRB, 0.85322E+00_JPRB, 0.81583E+00_JPRB, 0.76176E+00_JPRB ,&
&  0.72931E+00_JPRB, 0.70996E+00_JPRB, 0.69368E+00_JPRB, 0.70334E+00_JPRB, 0.75284E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.69764E+00_JPRB, 0.69698E+00_JPRB, 0.69209E+00_JPRB, 0.68248E+00_JPRB, 0.67659E+00_JPRB, 0.66966E+00_JPRB, 0.65839E+00_JPRB ,&
&  0.64531E+00_JPRB, 0.63338E+00_JPRB, 0.62108E+00_JPRB, 0.60542E+00_JPRB, 0.58890E+00_JPRB, 0.58184E+00_JPRB, 0.57928E+00_JPRB ,&
&  0.57399E+00_JPRB, 0.56466E+00_JPRB, 0.55597E+00_JPRB, 0.55362E+00_JPRB, 0.55241E+00_JPRB, 0.55129E+00_JPRB, 0.54715E+00_JPRB ,&
&  0.54080E+00_JPRB, 0.53931E+00_JPRB, 0.54190E+00_JPRB, 0.54171E+00_JPRB, 0.53712E+00_JPRB, 0.53942E+00_JPRB, 0.56634E+00_JPRB ,&
&  0.70235E+00_JPRB, 0.73363E+00_JPRB, 0.88015E+00_JPRB, 0.11113E+01_JPRB, 0.10261E+01_JPRB, 0.98025E+00_JPRB, 0.89756E+00_JPRB ,&
&  0.81485E+00_JPRB, 0.76766E+00_JPRB, 0.73596E+00_JPRB, 0.70332E+00_JPRB, 0.67289E+00_JPRB, 0.65654E+00_JPRB, 0.65665E+00_JPRB ,&
&  0.66561E+00_JPRB, 0.66931E+00_JPRB, 0.66852E+00_JPRB, 0.65912E+00_JPRB, 0.65197E+00_JPRB, 0.64578E+00_JPRB, 0.64147E+00_JPRB ,&
&  0.63856E+00_JPRB, 0.64182E+00_JPRB, 0.64872E+00_JPRB, 0.65679E+00_JPRB, 0.68333E+00_JPRB, 0.76168E+00_JPRB, 0.88781E+00_JPRB ,&
&  0.10005E+01_JPRB, 0.10932E+01_JPRB, 0.11795E+01_JPRB, 0.12667E+01_JPRB, 0.13305E+01_JPRB, 0.13861E+01_JPRB, 0.14120E+01_JPRB ,&
&  0.13876E+01_JPRB, 0.13236E+01_JPRB, 0.12386E+01_JPRB, 0.11397E+01_JPRB, 0.10476E+01_JPRB, 0.97329E+00_JPRB, 0.91586E+00_JPRB ,&
&  0.87207E+00_JPRB, 0.84262E+00_JPRB, 0.81960E+00_JPRB, 0.79599E+00_JPRB, 0.78267E+00_JPRB, 0.78078E+00_JPRB, 0.78916E+00_JPRB ,&
&  0.80751E+00_JPRB, 0.82822E+00_JPRB, 0.84123E+00_JPRB, 0.84357E+00_JPRB, 0.83606E+00_JPRB, 0.82881E+00_JPRB, 0.82194E+00_JPRB ,&
&  0.82402E+00_JPRB, 0.83435E+00_JPRB, 0.84079E+00_JPRB, 0.83507E+00_JPRB, 0.84578E+00_JPRB, 0.82616E+00_JPRB, 0.78330E+00_JPRB ,&
&  0.74027E+00_JPRB, 0.70807E+00_JPRB, 0.68860E+00_JPRB, 0.62046E+00_JPRB, 0.69496E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.66043E+00_JPRB, 0.66206E+00_JPRB, 0.66222E+00_JPRB, 0.66134E+00_JPRB, 0.65898E+00_JPRB, 0.65211E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.62737E+00_JPRB, 0.61505E+00_JPRB, 0.60526E+00_JPRB, 0.59511E+00_JPRB, 0.58038E+00_JPRB, 0.57027E+00_JPRB, 0.56755E+00_JPRB ,&
&  0.56493E+00_JPRB, 0.55852E+00_JPRB, 0.55120E+00_JPRB, 0.54956E+00_JPRB, 0.54687E+00_JPRB, 0.54296E+00_JPRB, 0.53756E+00_JPRB ,&
&  0.53218E+00_JPRB, 0.53014E+00_JPRB, 0.52986E+00_JPRB, 0.53004E+00_JPRB, 0.52622E+00_JPRB, 0.52873E+00_JPRB, 0.55701E+00_JPRB ,&
&  0.66604E+00_JPRB, 0.62259E+00_JPRB, 0.74144E+00_JPRB, 0.86889E+00_JPRB, 0.79266E+00_JPRB, 0.73288E+00_JPRB, 0.68423E+00_JPRB ,&
&  0.64629E+00_JPRB, 0.62332E+00_JPRB, 0.61061E+00_JPRB, 0.59926E+00_JPRB, 0.59072E+00_JPRB, 0.58370E+00_JPRB, 0.58449E+00_JPRB ,&
&  0.58954E+00_JPRB, 0.60182E+00_JPRB, 0.60965E+00_JPRB, 0.60978E+00_JPRB, 0.60344E+00_JPRB, 0.59594E+00_JPRB, 0.58901E+00_JPRB ,&
&  0.58440E+00_JPRB, 0.58384E+00_JPRB, 0.58659E+00_JPRB, 0.59317E+00_JPRB, 0.60832E+00_JPRB, 0.64435E+00_JPRB, 0.71577E+00_JPRB ,&
&  0.80640E+00_JPRB, 0.88059E+00_JPRB, 0.95023E+00_JPRB, 0.10349E+01_JPRB, 0.11161E+01_JPRB, 0.11734E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.11673E+01_JPRB, 0.11100E+01_JPRB, 0.10566E+01_JPRB, 0.10010E+01_JPRB, 0.94651E+00_JPRB, 0.89998E+00_JPRB, 0.86717E+00_JPRB ,&
&  0.84520E+00_JPRB, 0.83044E+00_JPRB, 0.82199E+00_JPRB, 0.81653E+00_JPRB, 0.81678E+00_JPRB, 0.81903E+00_JPRB, 0.82319E+00_JPRB ,&
&  0.81952E+00_JPRB, 0.81124E+00_JPRB, 0.78958E+00_JPRB, 0.75965E+00_JPRB, 0.73893E+00_JPRB, 0.72864E+00_JPRB, 0.72504E+00_JPRB ,&
&  0.72849E+00_JPRB, 0.73966E+00_JPRB, 0.75337E+00_JPRB, 0.78385E+00_JPRB, 0.79200E+00_JPRB, 0.79522E+00_JPRB, 0.78111E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.71223E+00_JPRB, 0.59640E+00_JPRB, 0.65021E+00_JPRB, 0.66320E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.66407E+00_JPRB, 0.65708E+00_JPRB, 0.65085E+00_JPRB, 0.64447E+00_JPRB, 0.63626E+00_JPRB, 0.62610E+00_JPRB, 0.61426E+00_JPRB ,&
&  0.60292E+00_JPRB, 0.59219E+00_JPRB, 0.58153E+00_JPRB, 0.57231E+00_JPRB, 0.56354E+00_JPRB, 0.55895E+00_JPRB, 0.55741E+00_JPRB ,&
&  0.55760E+00_JPRB, 0.55640E+00_JPRB, 0.55369E+00_JPRB, 0.55287E+00_JPRB, 0.55190E+00_JPRB, 0.54848E+00_JPRB, 0.54369E+00_JPRB ,&
&  0.53641E+00_JPRB, 0.53008E+00_JPRB, 0.52713E+00_JPRB, 0.52861E+00_JPRB, 0.52691E+00_JPRB, 0.53093E+00_JPRB, 0.54849E+00_JPRB ,&
&  0.55570E+00_JPRB, 0.55819E+00_JPRB, 0.62991E+00_JPRB, 0.72102E+00_JPRB, 0.66670E+00_JPRB, 0.61321E+00_JPRB, 0.59040E+00_JPRB ,&
&  0.57248E+00_JPRB, 0.56005E+00_JPRB, 0.55150E+00_JPRB, 0.54655E+00_JPRB, 0.54623E+00_JPRB, 0.54629E+00_JPRB, 0.54720E+00_JPRB ,&
&  0.54978E+00_JPRB, 0.55585E+00_JPRB, 0.56230E+00_JPRB, 0.56659E+00_JPRB, 0.56556E+00_JPRB, 0.56149E+00_JPRB, 0.55501E+00_JPRB ,&
&  0.54899E+00_JPRB, 0.54490E+00_JPRB, 0.54402E+00_JPRB, 0.54825E+00_JPRB, 0.55734E+00_JPRB, 0.57524E+00_JPRB, 0.61089E+00_JPRB ,&
&  0.66581E+00_JPRB, 0.72296E+00_JPRB, 0.77578E+00_JPRB, 0.83656E+00_JPRB, 0.91209E+00_JPRB, 0.98305E+00_JPRB, 0.10278E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.97970E+00_JPRB, 0.92475E+00_JPRB, 0.87759E+00_JPRB, 0.84171E+00_JPRB, 0.81843E+00_JPRB, 0.80454E+00_JPRB ,&
&  0.79930E+00_JPRB, 0.79656E+00_JPRB, 0.79549E+00_JPRB, 0.79522E+00_JPRB, 0.79408E+00_JPRB, 0.78833E+00_JPRB, 0.77794E+00_JPRB ,&
&  0.75997E+00_JPRB, 0.73222E+00_JPRB, 0.70081E+00_JPRB, 0.67536E+00_JPRB, 0.65815E+00_JPRB, 0.65018E+00_JPRB, 0.64944E+00_JPRB ,&
&  0.65542E+00_JPRB, 0.66997E+00_JPRB, 0.68876E+00_JPRB, 0.72391E+00_JPRB, 0.74887E+00_JPRB, 0.76500E+00_JPRB, 0.77084E+00_JPRB ,&
&  0.75634E+00_JPRB, 0.72771E+00_JPRB, 0.69898E+00_JPRB, 0.68287E+00_JPRB, 0.67501E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.65832E+00_JPRB, 0.64166E+00_JPRB, 0.62516E+00_JPRB, 0.61355E+00_JPRB, 0.60432E+00_JPRB, 0.59535E+00_JPRB, 0.58547E+00_JPRB ,&
&  0.57404E+00_JPRB, 0.56272E+00_JPRB, 0.55242E+00_JPRB, 0.54305E+00_JPRB, 0.53823E+00_JPRB, 0.53583E+00_JPRB, 0.53606E+00_JPRB ,&
&  0.53883E+00_JPRB, 0.54368E+00_JPRB, 0.54491E+00_JPRB, 0.54374E+00_JPRB, 0.54409E+00_JPRB, 0.54343E+00_JPRB, 0.54071E+00_JPRB ,&
&  0.53511E+00_JPRB, 0.52855E+00_JPRB, 0.52442E+00_JPRB, 0.52416E+00_JPRB, 0.52396E+00_JPRB, 0.52650E+00_JPRB, 0.53175E+00_JPRB ,&
&  0.55019E+00_JPRB, 0.52291E+00_JPRB, 0.57171E+00_JPRB, 0.63145E+00_JPRB, 0.61400E+00_JPRB, 0.58677E+00_JPRB, 0.56417E+00_JPRB ,&
&  0.54645E+00_JPRB, 0.53647E+00_JPRB, 0.53098E+00_JPRB, 0.53104E+00_JPRB, 0.53429E+00_JPRB, 0.53679E+00_JPRB, 0.53864E+00_JPRB ,&
&  0.53730E+00_JPRB, 0.53803E+00_JPRB, 0.53633E+00_JPRB, 0.53484E+00_JPRB, 0.53456E+00_JPRB, 0.53529E+00_JPRB, 0.53493E+00_JPRB ,&
&  0.53205E+00_JPRB, 0.52728E+00_JPRB, 0.52264E+00_JPRB, 0.52136E+00_JPRB, 0.52274E+00_JPRB, 0.52791E+00_JPRB, 0.54043E+00_JPRB ,&
&  0.56354E+00_JPRB, 0.59296E+00_JPRB, 0.62833E+00_JPRB, 0.67425E+00_JPRB, 0.73714E+00_JPRB, 0.80205E+00_JPRB, 0.85301E+00_JPRB ,&
&  0.87084E+00_JPRB, 0.86002E+00_JPRB, 0.83310E+00_JPRB, 0.79936E+00_JPRB, 0.76778E+00_JPRB, 0.73802E+00_JPRB, 0.71228E+00_JPRB ,&
&  0.69344E+00_JPRB, 0.67911E+00_JPRB, 0.66871E+00_JPRB, 0.66211E+00_JPRB, 0.65549E+00_JPRB, 0.64622E+00_JPRB, 0.63684E+00_JPRB ,&
&  0.62626E+00_JPRB, 0.61619E+00_JPRB, 0.60785E+00_JPRB, 0.60027E+00_JPRB, 0.59293E+00_JPRB, 0.58721E+00_JPRB, 0.58602E+00_JPRB ,&
&  0.59093E+00_JPRB, 0.60138E+00_JPRB, 0.61958E+00_JPRB, 0.64534E+00_JPRB, 0.67380E+00_JPRB, 0.69662E+00_JPRB, 0.71086E+00_JPRB ,&
&  0.71350E+00_JPRB, 0.70962E+00_JPRB, 0.69755E+00_JPRB, 0.68436E+00_JPRB, 0.67296E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.62931E+00_JPRB, 0.61586E+00_JPRB, 0.60220E+00_JPRB, 0.59105E+00_JPRB, 0.58192E+00_JPRB, 0.57210E+00_JPRB, 0.56124E+00_JPRB ,&
&  0.55219E+00_JPRB, 0.54278E+00_JPRB, 0.53584E+00_JPRB, 0.53003E+00_JPRB, 0.52585E+00_JPRB, 0.52446E+00_JPRB, 0.52516E+00_JPRB ,&
&  0.52798E+00_JPRB, 0.53397E+00_JPRB, 0.53646E+00_JPRB, 0.53422E+00_JPRB, 0.53559E+00_JPRB, 0.53761E+00_JPRB, 0.53749E+00_JPRB ,&
&  0.53396E+00_JPRB, 0.52951E+00_JPRB, 0.52476E+00_JPRB, 0.52169E+00_JPRB, 0.52135E+00_JPRB, 0.52371E+00_JPRB, 0.52317E+00_JPRB ,&
&  0.52499E+00_JPRB, 0.53549E+00_JPRB, 0.55953E+00_JPRB, 0.58153E+00_JPRB, 0.58206E+00_JPRB, 0.56773E+00_JPRB, 0.55041E+00_JPRB ,&
&  0.53264E+00_JPRB, 0.52304E+00_JPRB, 0.52133E+00_JPRB, 0.52319E+00_JPRB, 0.52658E+00_JPRB, 0.52902E+00_JPRB, 0.53233E+00_JPRB ,&
&  0.53012E+00_JPRB, 0.52841E+00_JPRB, 0.52445E+00_JPRB, 0.52034E+00_JPRB, 0.51949E+00_JPRB, 0.52023E+00_JPRB, 0.52128E+00_JPRB ,&
&  0.52079E+00_JPRB, 0.52144E+00_JPRB, 0.52151E+00_JPRB, 0.51969E+00_JPRB, 0.52041E+00_JPRB, 0.52243E+00_JPRB, 0.52619E+00_JPRB ,&
&  0.53371E+00_JPRB, 0.54585E+00_JPRB, 0.56136E+00_JPRB, 0.58504E+00_JPRB, 0.62403E+00_JPRB, 0.66914E+00_JPRB, 0.70590E+00_JPRB ,&
&  0.72545E+00_JPRB, 0.72845E+00_JPRB, 0.71997E+00_JPRB, 0.70378E+00_JPRB, 0.68758E+00_JPRB, 0.67139E+00_JPRB, 0.65306E+00_JPRB ,&
&  0.63360E+00_JPRB, 0.61862E+00_JPRB, 0.60642E+00_JPRB, 0.59709E+00_JPRB, 0.58887E+00_JPRB, 0.58191E+00_JPRB, 0.57743E+00_JPRB ,&
&  0.57534E+00_JPRB, 0.57382E+00_JPRB, 0.57064E+00_JPRB, 0.56679E+00_JPRB, 0.56399E+00_JPRB, 0.55975E+00_JPRB, 0.55732E+00_JPRB ,&
&  0.55905E+00_JPRB, 0.56502E+00_JPRB, 0.57594E+00_JPRB, 0.59414E+00_JPRB, 0.61264E+00_JPRB, 0.62883E+00_JPRB, 0.64203E+00_JPRB ,&
&  0.65181E+00_JPRB, 0.65666E+00_JPRB, 0.65563E+00_JPRB, 0.65070E+00_JPRB, 0.64201E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.57837E+00_JPRB, 0.57187E+00_JPRB, 0.56379E+00_JPRB, 0.55549E+00_JPRB, 0.54754E+00_JPRB, 0.53969E+00_JPRB, 0.53247E+00_JPRB ,&
&  0.52637E+00_JPRB, 0.52259E+00_JPRB, 0.52029E+00_JPRB, 0.51858E+00_JPRB, 0.51504E+00_JPRB, 0.51346E+00_JPRB, 0.51421E+00_JPRB ,&
&  0.51687E+00_JPRB, 0.51959E+00_JPRB, 0.52099E+00_JPRB, 0.51980E+00_JPRB, 0.52169E+00_JPRB, 0.52334E+00_JPRB, 0.52296E+00_JPRB ,&
&  0.52131E+00_JPRB, 0.51938E+00_JPRB, 0.51606E+00_JPRB, 0.51252E+00_JPRB, 0.51225E+00_JPRB, 0.51343E+00_JPRB, 0.51121E+00_JPRB ,&
&  0.50600E+00_JPRB, 0.52004E+00_JPRB, 0.51373E+00_JPRB, 0.54057E+00_JPRB, 0.54928E+00_JPRB, 0.54203E+00_JPRB, 0.53023E+00_JPRB ,&
&  0.52163E+00_JPRB, 0.51793E+00_JPRB, 0.51950E+00_JPRB, 0.52187E+00_JPRB, 0.52369E+00_JPRB, 0.52496E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.51589E+00_JPRB, 0.50903E+00_JPRB, 0.50610E+00_JPRB, 0.50512E+00_JPRB, 0.50414E+00_JPRB, 0.50423E+00_JPRB, 0.50606E+00_JPRB ,&
&  0.50780E+00_JPRB, 0.50907E+00_JPRB, 0.51023E+00_JPRB, 0.51246E+00_JPRB, 0.51372E+00_JPRB, 0.51413E+00_JPRB, 0.51588E+00_JPRB ,&
&  0.51918E+00_JPRB, 0.52440E+00_JPRB, 0.53067E+00_JPRB, 0.53878E+00_JPRB, 0.55617E+00_JPRB, 0.58196E+00_JPRB, 0.60528E+00_JPRB ,&
&  0.61873E+00_JPRB, 0.62374E+00_JPRB, 0.62295E+00_JPRB, 0.61571E+00_JPRB, 0.60422E+00_JPRB, 0.59382E+00_JPRB, 0.58425E+00_JPRB ,&
&  0.57243E+00_JPRB, 0.56399E+00_JPRB, 0.55653E+00_JPRB, 0.54784E+00_JPRB, 0.54011E+00_JPRB, 0.53464E+00_JPRB, 0.53149E+00_JPRB ,&
&  0.53076E+00_JPRB, 0.53220E+00_JPRB, 0.53236E+00_JPRB, 0.53307E+00_JPRB, 0.53546E+00_JPRB, 0.53820E+00_JPRB, 0.53740E+00_JPRB ,&
&  0.53545E+00_JPRB, 0.53499E+00_JPRB, 0.53742E+00_JPRB, 0.54339E+00_JPRB, 0.55068E+00_JPRB, 0.55933E+00_JPRB, 0.56835E+00_JPRB ,&
&  0.57657E+00_JPRB, 0.58251E+00_JPRB, 0.58534E+00_JPRB, 0.58617E+00_JPRB, 0.58349E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.52884E+00_JPRB, 0.52698E+00_JPRB, 0.52433E+00_JPRB, 0.52016E+00_JPRB, 0.51555E+00_JPRB, 0.51145E+00_JPRB, 0.50814E+00_JPRB ,&
&  0.50603E+00_JPRB, 0.50546E+00_JPRB, 0.50603E+00_JPRB, 0.50632E+00_JPRB, 0.50627E+00_JPRB, 0.50721E+00_JPRB, 0.50767E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.50959E+00_JPRB, 0.50658E+00_JPRB, 0.50552E+00_JPRB, 0.50777E+00_JPRB, 0.51235E+00_JPRB, 0.51359E+00_JPRB ,&
&  0.51067E+00_JPRB, 0.50967E+00_JPRB, 0.50857E+00_JPRB, 0.50561E+00_JPRB, 0.50367E+00_JPRB, 0.50323E+00_JPRB, 0.50035E+00_JPRB ,&
&  0.49567E+00_JPRB, 0.50104E+00_JPRB, 0.50333E+00_JPRB, 0.51413E+00_JPRB, 0.51999E+00_JPRB, 0.52284E+00_JPRB, 0.52398E+00_JPRB ,&
&  0.52132E+00_JPRB, 0.51913E+00_JPRB, 0.51739E+00_JPRB, 0.51674E+00_JPRB, 0.51391E+00_JPRB, 0.50956E+00_JPRB, 0.50160E+00_JPRB ,&
&  0.48811E+00_JPRB, 0.48175E+00_JPRB, 0.48247E+00_JPRB, 0.48652E+00_JPRB, 0.48545E+00_JPRB, 0.48551E+00_JPRB, 0.48851E+00_JPRB ,&
&  0.49261E+00_JPRB, 0.49578E+00_JPRB, 0.49774E+00_JPRB, 0.49886E+00_JPRB, 0.50031E+00_JPRB, 0.50168E+00_JPRB, 0.50234E+00_JPRB ,&
&  0.50332E+00_JPRB, 0.50541E+00_JPRB, 0.50861E+00_JPRB, 0.51297E+00_JPRB, 0.51934E+00_JPRB, 0.53024E+00_JPRB, 0.54174E+00_JPRB ,&
&  0.54911E+00_JPRB, 0.55098E+00_JPRB, 0.55013E+00_JPRB, 0.54582E+00_JPRB, 0.53846E+00_JPRB, 0.53134E+00_JPRB, 0.52479E+00_JPRB ,&
&  0.51594E+00_JPRB, 0.50821E+00_JPRB, 0.50214E+00_JPRB, 0.49892E+00_JPRB, 0.49736E+00_JPRB, 0.49720E+00_JPRB, 0.49820E+00_JPRB ,&
&  0.49928E+00_JPRB, 0.50228E+00_JPRB, 0.50885E+00_JPRB, 0.51275E+00_JPRB, 0.51445E+00_JPRB, 0.51755E+00_JPRB, 0.51894E+00_JPRB ,&
&  0.51687E+00_JPRB, 0.51503E+00_JPRB, 0.51397E+00_JPRB, 0.51632E+00_JPRB, 0.51858E+00_JPRB, 0.51902E+00_JPRB, 0.52000E+00_JPRB ,&
&  0.52354E+00_JPRB, 0.52735E+00_JPRB, 0.52861E+00_JPRB, 0.52922E+00_JPRB, 0.52992E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.48727E+00_JPRB, 0.48756E+00_JPRB, 0.48699E+00_JPRB, 0.48583E+00_JPRB, 0.48464E+00_JPRB, 0.48371E+00_JPRB, 0.48387E+00_JPRB ,&
&  0.48422E+00_JPRB, 0.48513E+00_JPRB, 0.48640E+00_JPRB, 0.48745E+00_JPRB, 0.48941E+00_JPRB, 0.49169E+00_JPRB, 0.49368E+00_JPRB ,&
&  0.49566E+00_JPRB, 0.49490E+00_JPRB, 0.49114E+00_JPRB, 0.48763E+00_JPRB, 0.48832E+00_JPRB, 0.49397E+00_JPRB, 0.50004E+00_JPRB ,&
&  0.50116E+00_JPRB, 0.50173E+00_JPRB, 0.50156E+00_JPRB, 0.49973E+00_JPRB, 0.49769E+00_JPRB, 0.49759E+00_JPRB, 0.49465E+00_JPRB ,&
&  0.49017E+00_JPRB, 0.49073E+00_JPRB, 0.49017E+00_JPRB, 0.49102E+00_JPRB, 0.49505E+00_JPRB, 0.50236E+00_JPRB, 0.50564E+00_JPRB ,&
&  0.50680E+00_JPRB, 0.50483E+00_JPRB, 0.50027E+00_JPRB, 0.49512E+00_JPRB, 0.48865E+00_JPRB, 0.47823E+00_JPRB, 0.46650E+00_JPRB ,&
&  0.45710E+00_JPRB, 0.45582E+00_JPRB, 0.46129E+00_JPRB, 0.46687E+00_JPRB, 0.46489E+00_JPRB, 0.46280E+00_JPRB, 0.46397E+00_JPRB ,&
&  0.46767E+00_JPRB, 0.47272E+00_JPRB, 0.47604E+00_JPRB, 0.47543E+00_JPRB, 0.47425E+00_JPRB, 0.47617E+00_JPRB, 0.47894E+00_JPRB ,&
&  0.48013E+00_JPRB, 0.48142E+00_JPRB, 0.48239E+00_JPRB, 0.48418E+00_JPRB, 0.48579E+00_JPRB, 0.48727E+00_JPRB, 0.49078E+00_JPRB ,&
&  0.49267E+00_JPRB, 0.49245E+00_JPRB, 0.48920E+00_JPRB, 0.48349E+00_JPRB, 0.47803E+00_JPRB, 0.47333E+00_JPRB, 0.47120E+00_JPRB ,&
&  0.46869E+00_JPRB, 0.46866E+00_JPRB, 0.46810E+00_JPRB, 0.46543E+00_JPRB, 0.46240E+00_JPRB, 0.46244E+00_JPRB, 0.46619E+00_JPRB ,&
&  0.47125E+00_JPRB, 0.47560E+00_JPRB, 0.48206E+00_JPRB, 0.48495E+00_JPRB, 0.48684E+00_JPRB, 0.48946E+00_JPRB, 0.49028E+00_JPRB ,&
&  0.48753E+00_JPRB, 0.48511E+00_JPRB, 0.48445E+00_JPRB, 0.48731E+00_JPRB, 0.48831E+00_JPRB, 0.48737E+00_JPRB, 0.48422E+00_JPRB ,&
&  0.48304E+00_JPRB, 0.48473E+00_JPRB, 0.48528E+00_JPRB, 0.48489E+00_JPRB, 0.48566E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.45845E+00_JPRB, 0.46087E+00_JPRB, 0.46347E+00_JPRB, 0.46580E+00_JPRB, 0.46725E+00_JPRB, 0.46822E+00_JPRB, 0.46736E+00_JPRB ,&
&  0.46729E+00_JPRB, 0.46913E+00_JPRB, 0.47153E+00_JPRB, 0.47359E+00_JPRB, 0.47497E+00_JPRB, 0.47739E+00_JPRB, 0.47994E+00_JPRB ,&
&  0.48169E+00_JPRB, 0.48058E+00_JPRB, 0.47605E+00_JPRB, 0.47339E+00_JPRB, 0.47210E+00_JPRB, 0.47763E+00_JPRB, 0.48675E+00_JPRB ,&
&  0.49078E+00_JPRB, 0.49305E+00_JPRB, 0.49572E+00_JPRB, 0.49596E+00_JPRB, 0.49601E+00_JPRB, 0.49615E+00_JPRB, 0.49197E+00_JPRB ,&
&  0.48693E+00_JPRB, 0.48311E+00_JPRB, 0.47957E+00_JPRB, 0.47684E+00_JPRB, 0.47883E+00_JPRB, 0.47936E+00_JPRB, 0.48020E+00_JPRB ,&
&  0.48069E+00_JPRB, 0.47830E+00_JPRB, 0.47336E+00_JPRB, 0.46549E+00_JPRB, 0.45605E+00_JPRB, 0.44396E+00_JPRB, 0.43516E+00_JPRB ,&
&  0.43122E+00_JPRB, 0.43497E+00_JPRB, 0.44405E+00_JPRB, 0.44757E+00_JPRB, 0.44314E+00_JPRB, 0.44119E+00_JPRB, 0.44401E+00_JPRB ,&
&  0.44904E+00_JPRB, 0.45500E+00_JPRB, 0.45757E+00_JPRB, 0.45686E+00_JPRB, 0.45472E+00_JPRB, 0.45640E+00_JPRB, 0.46106E+00_JPRB ,&
&  0.46494E+00_JPRB, 0.46676E+00_JPRB, 0.46563E+00_JPRB, 0.46442E+00_JPRB, 0.46185E+00_JPRB, 0.45680E+00_JPRB, 0.45030E+00_JPRB ,&
&  0.44432E+00_JPRB, 0.44061E+00_JPRB, 0.43766E+00_JPRB, 0.43457E+00_JPRB, 0.43323E+00_JPRB, 0.43340E+00_JPRB, 0.43552E+00_JPRB ,&
&  0.43628E+00_JPRB, 0.43804E+00_JPRB, 0.43819E+00_JPRB, 0.43404E+00_JPRB, 0.43081E+00_JPRB, 0.43262E+00_JPRB, 0.43451E+00_JPRB ,&
&  0.43751E+00_JPRB, 0.44062E+00_JPRB, 0.44363E+00_JPRB, 0.44462E+00_JPRB, 0.44616E+00_JPRB, 0.44918E+00_JPRB, 0.45112E+00_JPRB ,&
&  0.45214E+00_JPRB, 0.45283E+00_JPRB, 0.45481E+00_JPRB, 0.45823E+00_JPRB, 0.46160E+00_JPRB, 0.46435E+00_JPRB, 0.46392E+00_JPRB ,&
&  0.46248E+00_JPRB, 0.46055E+00_JPRB, 0.46053E+00_JPRB, 0.46035E+00_JPRB, 0.45849E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.45113E+00_JPRB, 0.45075E+00_JPRB, 0.45406E+00_JPRB, 0.45759E+00_JPRB, 0.45815E+00_JPRB, 0.45738E+00_JPRB, 0.45439E+00_JPRB ,&
&  0.45241E+00_JPRB, 0.45356E+00_JPRB, 0.45549E+00_JPRB, 0.45706E+00_JPRB, 0.45980E+00_JPRB, 0.46350E+00_JPRB, 0.46640E+00_JPRB ,&
&  0.46660E+00_JPRB, 0.46439E+00_JPRB, 0.46062E+00_JPRB, 0.46032E+00_JPRB, 0.45997E+00_JPRB, 0.46246E+00_JPRB, 0.47066E+00_JPRB ,&
&  0.47661E+00_JPRB, 0.48060E+00_JPRB, 0.48389E+00_JPRB, 0.48426E+00_JPRB, 0.48367E+00_JPRB, 0.48562E+00_JPRB, 0.48213E+00_JPRB ,&
&  0.47569E+00_JPRB, 0.47028E+00_JPRB, 0.46394E+00_JPRB, 0.45672E+00_JPRB, 0.46006E+00_JPRB, 0.45845E+00_JPRB, 0.45607E+00_JPRB ,&
&  0.45180E+00_JPRB, 0.44551E+00_JPRB, 0.43767E+00_JPRB, 0.43011E+00_JPRB, 0.42427E+00_JPRB, 0.41879E+00_JPRB, 0.41230E+00_JPRB ,&
&  0.41002E+00_JPRB, 0.41276E+00_JPRB, 0.41873E+00_JPRB, 0.42025E+00_JPRB, 0.41636E+00_JPRB, 0.41639E+00_JPRB, 0.42302E+00_JPRB ,&
&  0.43056E+00_JPRB, 0.43446E+00_JPRB, 0.43454E+00_JPRB, 0.43511E+00_JPRB, 0.43576E+00_JPRB, 0.43733E+00_JPRB, 0.43812E+00_JPRB ,&
&  0.43694E+00_JPRB, 0.43499E+00_JPRB, 0.43209E+00_JPRB, 0.42598E+00_JPRB, 0.41890E+00_JPRB, 0.41230E+00_JPRB, 0.40992E+00_JPRB ,&
&  0.41280E+00_JPRB, 0.41352E+00_JPRB, 0.41517E+00_JPRB, 0.41773E+00_JPRB, 0.41857E+00_JPRB, 0.41805E+00_JPRB, 0.41729E+00_JPRB ,&
&  0.41902E+00_JPRB, 0.41728E+00_JPRB, 0.41775E+00_JPRB, 0.41483E+00_JPRB, 0.41028E+00_JPRB, 0.40545E+00_JPRB, 0.40074E+00_JPRB ,&
&  0.39826E+00_JPRB, 0.39791E+00_JPRB, 0.40013E+00_JPRB, 0.40107E+00_JPRB, 0.40326E+00_JPRB, 0.40587E+00_JPRB, 0.40812E+00_JPRB ,&
&  0.40956E+00_JPRB, 0.41162E+00_JPRB, 0.41544E+00_JPRB, 0.42235E+00_JPRB, 0.42948E+00_JPRB, 0.43568E+00_JPRB, 0.44160E+00_JPRB ,&
&  0.44680E+00_JPRB, 0.45027E+00_JPRB, 0.45601E+00_JPRB, 0.45669E+00_JPRB, 0.45508E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.44898E+00_JPRB, 0.44562E+00_JPRB, 0.44510E+00_JPRB, 0.44542E+00_JPRB, 0.44401E+00_JPRB, 0.44223E+00_JPRB, 0.43885E+00_JPRB ,&
&  0.43606E+00_JPRB, 0.43637E+00_JPRB, 0.43786E+00_JPRB, 0.44077E+00_JPRB, 0.44582E+00_JPRB, 0.45116E+00_JPRB, 0.45094E+00_JPRB ,&
&  0.44795E+00_JPRB, 0.44684E+00_JPRB, 0.44919E+00_JPRB, 0.45077E+00_JPRB, 0.45063E+00_JPRB, 0.45138E+00_JPRB, 0.45547E+00_JPRB ,&
&  0.46044E+00_JPRB, 0.46282E+00_JPRB, 0.46597E+00_JPRB, 0.46559E+00_JPRB, 0.46170E+00_JPRB, 0.46235E+00_JPRB, 0.46054E+00_JPRB ,&
&  0.45554E+00_JPRB, 0.44898E+00_JPRB, 0.43906E+00_JPRB, 0.43953E+00_JPRB, 0.43578E+00_JPRB, 0.43258E+00_JPRB, 0.42756E+00_JPRB ,&
&  0.42184E+00_JPRB, 0.41661E+00_JPRB, 0.41091E+00_JPRB, 0.40637E+00_JPRB, 0.40427E+00_JPRB, 0.40408E+00_JPRB, 0.40171E+00_JPRB ,&
&  0.40052E+00_JPRB, 0.40333E+00_JPRB, 0.40458E+00_JPRB, 0.39967E+00_JPRB, 0.39790E+00_JPRB, 0.40363E+00_JPRB, 0.40907E+00_JPRB ,&
&  0.41181E+00_JPRB, 0.41214E+00_JPRB, 0.40918E+00_JPRB, 0.40832E+00_JPRB, 0.40731E+00_JPRB, 0.40532E+00_JPRB, 0.40161E+00_JPRB ,&
&  0.39571E+00_JPRB, 0.39268E+00_JPRB, 0.39056E+00_JPRB, 0.38647E+00_JPRB, 0.37834E+00_JPRB, 0.36274E+00_JPRB, 0.35420E+00_JPRB ,&
&  0.35612E+00_JPRB, 0.36569E+00_JPRB, 0.37927E+00_JPRB, 0.38787E+00_JPRB, 0.39659E+00_JPRB, 0.40241E+00_JPRB, 0.40243E+00_JPRB ,&
&  0.40075E+00_JPRB, 0.38148E+00_JPRB, 0.37068E+00_JPRB, 0.36063E+00_JPRB, 0.35168E+00_JPRB, 0.34656E+00_JPRB, 0.34446E+00_JPRB ,&
&  0.34741E+00_JPRB, 0.35300E+00_JPRB, 0.35906E+00_JPRB, 0.36252E+00_JPRB, 0.36281E+00_JPRB, 0.36072E+00_JPRB, 0.35777E+00_JPRB ,&
&  0.35892E+00_JPRB, 0.36773E+00_JPRB, 0.38330E+00_JPRB, 0.39592E+00_JPRB, 0.40635E+00_JPRB, 0.41769E+00_JPRB, 0.42687E+00_JPRB ,&
&  0.43333E+00_JPRB, 0.43942E+00_JPRB, 0.44641E+00_JPRB, 0.44876E+00_JPRB, 0.44941E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.43300E+00_JPRB, 0.42823E+00_JPRB, 0.42770E+00_JPRB, 0.42766E+00_JPRB, 0.42590E+00_JPRB, 0.42510E+00_JPRB, 0.42372E+00_JPRB ,&
&  0.42279E+00_JPRB, 0.42065E+00_JPRB, 0.41896E+00_JPRB, 0.41981E+00_JPRB, 0.42237E+00_JPRB, 0.42517E+00_JPRB, 0.42607E+00_JPRB ,&
&  0.42622E+00_JPRB, 0.42810E+00_JPRB, 0.43195E+00_JPRB, 0.43548E+00_JPRB, 0.43585E+00_JPRB, 0.43605E+00_JPRB, 0.43799E+00_JPRB ,&
&  0.44038E+00_JPRB, 0.44128E+00_JPRB, 0.44431E+00_JPRB, 0.44194E+00_JPRB, 0.43725E+00_JPRB, 0.43590E+00_JPRB, 0.43465E+00_JPRB ,&
&  0.42689E+00_JPRB, 0.40178E+00_JPRB, 0.38629E+00_JPRB, 0.39262E+00_JPRB, 0.40728E+00_JPRB, 0.40547E+00_JPRB, 0.40289E+00_JPRB ,&
&  0.40018E+00_JPRB, 0.39642E+00_JPRB, 0.39334E+00_JPRB, 0.39223E+00_JPRB, 0.39279E+00_JPRB, 0.39317E+00_JPRB, 0.39217E+00_JPRB ,&
&  0.39320E+00_JPRB, 0.39966E+00_JPRB, 0.40595E+00_JPRB, 0.40064E+00_JPRB, 0.39390E+00_JPRB, 0.38534E+00_JPRB, 0.37993E+00_JPRB ,&
&  0.37494E+00_JPRB, 0.37027E+00_JPRB, 0.36694E+00_JPRB, 0.36601E+00_JPRB, 0.36577E+00_JPRB, 0.36327E+00_JPRB, 0.35900E+00_JPRB ,&
&  0.35304E+00_JPRB, 0.34301E+00_JPRB, 0.33338E+00_JPRB, 0.32387E+00_JPRB, 0.31780E+00_JPRB, 0.31319E+00_JPRB, 0.30920E+00_JPRB ,&
&  0.31208E+00_JPRB, 0.32680E+00_JPRB, 0.34851E+00_JPRB, 0.36972E+00_JPRB, 0.38331E+00_JPRB, 0.36297E+00_JPRB, 0.35362E+00_JPRB ,&
&  0.34130E+00_JPRB, 0.33036E+00_JPRB, 0.32254E+00_JPRB, 0.31685E+00_JPRB, 0.31254E+00_JPRB, 0.30987E+00_JPRB, 0.30939E+00_JPRB ,&
&  0.31284E+00_JPRB, 0.31754E+00_JPRB, 0.32194E+00_JPRB, 0.32375E+00_JPRB, 0.32373E+00_JPRB, 0.32209E+00_JPRB, 0.32121E+00_JPRB ,&
&  0.32321E+00_JPRB, 0.33043E+00_JPRB, 0.34152E+00_JPRB, 0.35779E+00_JPRB, 0.37173E+00_JPRB, 0.38584E+00_JPRB, 0.40115E+00_JPRB ,&
&  0.41660E+00_JPRB, 0.42868E+00_JPRB, 0.43219E+00_JPRB, 0.43589E+00_JPRB, 0.43639E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.42105E+00_JPRB, 0.41867E+00_JPRB, 0.41650E+00_JPRB, 0.41665E+00_JPRB, 0.41658E+00_JPRB, 0.41855E+00_JPRB, 0.41683E+00_JPRB ,&
&  0.41197E+00_JPRB, 0.40728E+00_JPRB, 0.40614E+00_JPRB, 0.40653E+00_JPRB, 0.40556E+00_JPRB, 0.40394E+00_JPRB, 0.40499E+00_JPRB ,&
&  0.40347E+00_JPRB, 0.40610E+00_JPRB, 0.40970E+00_JPRB, 0.41164E+00_JPRB, 0.41226E+00_JPRB, 0.41230E+00_JPRB, 0.41269E+00_JPRB ,&
&  0.40628E+00_JPRB, 0.40396E+00_JPRB, 0.40989E+00_JPRB, 0.40668E+00_JPRB, 0.40235E+00_JPRB, 0.39865E+00_JPRB, 0.39739E+00_JPRB ,&
&  0.38769E+00_JPRB, 0.37436E+00_JPRB, 0.36205E+00_JPRB, 0.37044E+00_JPRB, 0.38737E+00_JPRB, 0.38269E+00_JPRB, 0.38032E+00_JPRB ,&
&  0.38024E+00_JPRB, 0.37954E+00_JPRB, 0.38063E+00_JPRB, 0.38138E+00_JPRB, 0.38304E+00_JPRB, 0.38368E+00_JPRB, 0.38579E+00_JPRB ,&
&  0.38783E+00_JPRB, 0.39578E+00_JPRB, 0.39856E+00_JPRB, 0.37566E+00_JPRB, 0.35743E+00_JPRB, 0.34357E+00_JPRB, 0.32999E+00_JPRB ,&
&  0.31788E+00_JPRB, 0.31711E+00_JPRB, 0.31292E+00_JPRB, 0.30406E+00_JPRB, 0.30431E+00_JPRB, 0.30531E+00_JPRB, 0.30420E+00_JPRB ,&
&  0.30134E+00_JPRB, 0.29966E+00_JPRB, 0.29820E+00_JPRB, 0.29567E+00_JPRB, 0.29490E+00_JPRB, 0.29665E+00_JPRB, 0.30141E+00_JPRB ,&
&  0.30850E+00_JPRB, 0.31563E+00_JPRB, 0.33196E+00_JPRB, 0.34525E+00_JPRB, 0.33937E+00_JPRB, 0.33385E+00_JPRB, 0.32728E+00_JPRB ,&
&  0.31390E+00_JPRB, 0.30759E+00_JPRB, 0.30117E+00_JPRB, 0.29590E+00_JPRB, 0.29342E+00_JPRB, 0.29339E+00_JPRB, 0.29392E+00_JPRB ,&
&  0.29500E+00_JPRB, 0.29743E+00_JPRB, 0.29838E+00_JPRB, 0.29813E+00_JPRB, 0.29740E+00_JPRB, 0.29841E+00_JPRB, 0.30077E+00_JPRB ,&
&  0.30473E+00_JPRB, 0.31136E+00_JPRB, 0.31914E+00_JPRB, 0.32887E+00_JPRB, 0.33753E+00_JPRB, 0.34566E+00_JPRB, 0.35870E+00_JPRB ,&
&  0.36736E+00_JPRB, 0.38873E+00_JPRB, 0.41575E+00_JPRB, 0.43531E+00_JPRB, 0.42556E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.39839E+00_JPRB, 0.39626E+00_JPRB, 0.39348E+00_JPRB, 0.39297E+00_JPRB, 0.39386E+00_JPRB, 0.39605E+00_JPRB, 0.39320E+00_JPRB ,&
&  0.38805E+00_JPRB, 0.38590E+00_JPRB, 0.38704E+00_JPRB, 0.38596E+00_JPRB, 0.38603E+00_JPRB, 0.37674E+00_JPRB, 0.37147E+00_JPRB ,&
&  0.36435E+00_JPRB, 0.36266E+00_JPRB, 0.36114E+00_JPRB, 0.36235E+00_JPRB, 0.36426E+00_JPRB, 0.36534E+00_JPRB, 0.36068E+00_JPRB ,&
&  0.35603E+00_JPRB, 0.35195E+00_JPRB, 0.34732E+00_JPRB, 0.34534E+00_JPRB, 0.34667E+00_JPRB, 0.34772E+00_JPRB, 0.34752E+00_JPRB ,&
&  0.34833E+00_JPRB, 0.35160E+00_JPRB, 0.35486E+00_JPRB, 0.35901E+00_JPRB, 0.37191E+00_JPRB, 0.36875E+00_JPRB, 0.36751E+00_JPRB ,&
&  0.36830E+00_JPRB, 0.36925E+00_JPRB, 0.37094E+00_JPRB, 0.37234E+00_JPRB, 0.37792E+00_JPRB, 0.38033E+00_JPRB, 0.38241E+00_JPRB ,&
&  0.36936E+00_JPRB, 0.36351E+00_JPRB, 0.35655E+00_JPRB, 0.34686E+00_JPRB, 0.33298E+00_JPRB, 0.32226E+00_JPRB, 0.31257E+00_JPRB ,&
&  0.30450E+00_JPRB, 0.29687E+00_JPRB, 0.28950E+00_JPRB, 0.28670E+00_JPRB, 0.28560E+00_JPRB, 0.28378E+00_JPRB, 0.28344E+00_JPRB ,&
&  0.28402E+00_JPRB, 0.28492E+00_JPRB, 0.28541E+00_JPRB, 0.28721E+00_JPRB, 0.29022E+00_JPRB, 0.29502E+00_JPRB, 0.30012E+00_JPRB ,&
&  0.30755E+00_JPRB, 0.31245E+00_JPRB, 0.31845E+00_JPRB, 0.32427E+00_JPRB, 0.32278E+00_JPRB, 0.31691E+00_JPRB, 0.31091E+00_JPRB ,&
&  0.30275E+00_JPRB, 0.29810E+00_JPRB, 0.29322E+00_JPRB, 0.28964E+00_JPRB, 0.28752E+00_JPRB, 0.28612E+00_JPRB, 0.28611E+00_JPRB ,&
&  0.28583E+00_JPRB, 0.28615E+00_JPRB, 0.28535E+00_JPRB, 0.28563E+00_JPRB, 0.28666E+00_JPRB, 0.28899E+00_JPRB, 0.29208E+00_JPRB ,&
&  0.29691E+00_JPRB, 0.30246E+00_JPRB, 0.30887E+00_JPRB, 0.31577E+00_JPRB, 0.32630E+00_JPRB, 0.33936E+00_JPRB, 0.35716E+00_JPRB ,&
&  0.37297E+00_JPRB, 0.39792E+00_JPRB, 0.40726E+00_JPRB, 0.40428E+00_JPRB, 0.40253E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.38048E+00_JPRB, 0.38211E+00_JPRB, 0.38285E+00_JPRB, 0.38499E+00_JPRB, 0.38753E+00_JPRB, 0.38818E+00_JPRB, 0.38821E+00_JPRB ,&
&  0.37857E+00_JPRB, 0.37149E+00_JPRB, 0.36462E+00_JPRB, 0.35456E+00_JPRB, 0.34657E+00_JPRB, 0.33964E+00_JPRB, 0.33469E+00_JPRB ,&
&  0.32883E+00_JPRB, 0.32632E+00_JPRB, 0.32277E+00_JPRB, 0.32177E+00_JPRB, 0.32082E+00_JPRB, 0.32105E+00_JPRB, 0.31908E+00_JPRB ,&
&  0.31716E+00_JPRB, 0.31876E+00_JPRB, 0.31967E+00_JPRB, 0.32416E+00_JPRB, 0.33103E+00_JPRB, 0.33867E+00_JPRB, 0.34621E+00_JPRB ,&
&  0.35586E+00_JPRB, 0.35965E+00_JPRB, 0.36295E+00_JPRB, 0.36378E+00_JPRB, 0.36300E+00_JPRB, 0.36311E+00_JPRB, 0.36157E+00_JPRB ,&
&  0.36214E+00_JPRB, 0.36237E+00_JPRB, 0.36471E+00_JPRB, 0.36625E+00_JPRB, 0.36445E+00_JPRB, 0.36319E+00_JPRB, 0.35868E+00_JPRB ,&
&  0.35105E+00_JPRB, 0.34510E+00_JPRB, 0.33825E+00_JPRB, 0.33273E+00_JPRB, 0.32386E+00_JPRB, 0.31624E+00_JPRB, 0.30888E+00_JPRB ,&
&  0.30272E+00_JPRB, 0.29828E+00_JPRB, 0.29280E+00_JPRB, 0.28835E+00_JPRB, 0.28622E+00_JPRB, 0.28353E+00_JPRB, 0.28236E+00_JPRB ,&
&  0.28234E+00_JPRB, 0.28289E+00_JPRB, 0.28567E+00_JPRB, 0.28784E+00_JPRB, 0.29121E+00_JPRB, 0.29516E+00_JPRB, 0.29734E+00_JPRB ,&
&  0.30147E+00_JPRB, 0.30506E+00_JPRB, 0.30705E+00_JPRB, 0.30866E+00_JPRB, 0.30768E+00_JPRB, 0.30395E+00_JPRB, 0.30038E+00_JPRB ,&
&  0.29541E+00_JPRB, 0.29202E+00_JPRB, 0.28985E+00_JPRB, 0.28740E+00_JPRB, 0.28604E+00_JPRB, 0.28479E+00_JPRB, 0.28331E+00_JPRB ,&
&  0.28215E+00_JPRB, 0.28157E+00_JPRB, 0.28104E+00_JPRB, 0.28241E+00_JPRB, 0.28380E+00_JPRB, 0.28642E+00_JPRB, 0.28957E+00_JPRB ,&
&  0.29236E+00_JPRB, 0.29673E+00_JPRB, 0.30146E+00_JPRB, 0.30799E+00_JPRB, 0.32048E+00_JPRB, 0.32961E+00_JPRB, 0.34987E+00_JPRB ,&
&  0.36013E+00_JPRB, 0.37831E+00_JPRB, 0.38666E+00_JPRB, 0.37954E+00_JPRB, 0.37990E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.36851E+00_JPRB, 0.37093E+00_JPRB, 0.37133E+00_JPRB, 0.37317E+00_JPRB, 0.37458E+00_JPRB, 0.37490E+00_JPRB, 0.37398E+00_JPRB ,&
&  0.37202E+00_JPRB, 0.36714E+00_JPRB, 0.36283E+00_JPRB, 0.35604E+00_JPRB, 0.35004E+00_JPRB, 0.34315E+00_JPRB, 0.33728E+00_JPRB ,&
&  0.33063E+00_JPRB, 0.32448E+00_JPRB, 0.31802E+00_JPRB, 0.31249E+00_JPRB, 0.30825E+00_JPRB, 0.30473E+00_JPRB, 0.30136E+00_JPRB ,&
&  0.29849E+00_JPRB, 0.30025E+00_JPRB, 0.30348E+00_JPRB, 0.30931E+00_JPRB, 0.31817E+00_JPRB, 0.32583E+00_JPRB, 0.33430E+00_JPRB ,&
&  0.34801E+00_JPRB, 0.35010E+00_JPRB, 0.35040E+00_JPRB, 0.35312E+00_JPRB, 0.35294E+00_JPRB, 0.35365E+00_JPRB, 0.34436E+00_JPRB ,&
&  0.34914E+00_JPRB, 0.35161E+00_JPRB, 0.36089E+00_JPRB, 0.36358E+00_JPRB, 0.35732E+00_JPRB, 0.35356E+00_JPRB, 0.34586E+00_JPRB ,&
&  0.34062E+00_JPRB, 0.33610E+00_JPRB, 0.33308E+00_JPRB, 0.33138E+00_JPRB, 0.32527E+00_JPRB, 0.32075E+00_JPRB, 0.31314E+00_JPRB ,&
&  0.30713E+00_JPRB, 0.30345E+00_JPRB, 0.29872E+00_JPRB, 0.29596E+00_JPRB, 0.29362E+00_JPRB, 0.29123E+00_JPRB, 0.28939E+00_JPRB ,&
&  0.28837E+00_JPRB, 0.28752E+00_JPRB, 0.28838E+00_JPRB, 0.28932E+00_JPRB, 0.29115E+00_JPRB, 0.29262E+00_JPRB, 0.29280E+00_JPRB ,&
&  0.29306E+00_JPRB, 0.29365E+00_JPRB, 0.29396E+00_JPRB, 0.29498E+00_JPRB, 0.29442E+00_JPRB, 0.29321E+00_JPRB, 0.29175E+00_JPRB ,&
&  0.29003E+00_JPRB, 0.28926E+00_JPRB, 0.28792E+00_JPRB, 0.28704E+00_JPRB, 0.28452E+00_JPRB, 0.28281E+00_JPRB, 0.28190E+00_JPRB ,&
&  0.28107E+00_JPRB, 0.28164E+00_JPRB, 0.28179E+00_JPRB, 0.28285E+00_JPRB, 0.28447E+00_JPRB, 0.28578E+00_JPRB, 0.28825E+00_JPRB ,&
&  0.29131E+00_JPRB, 0.29485E+00_JPRB, 0.30122E+00_JPRB, 0.30755E+00_JPRB, 0.31455E+00_JPRB, 0.32212E+00_JPRB, 0.32730E+00_JPRB ,&
&  0.33524E+00_JPRB, 0.34783E+00_JPRB, 0.34962E+00_JPRB, 0.35768E+00_JPRB, 0.36431E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.34977E+00_JPRB, 0.35502E+00_JPRB, 0.35835E+00_JPRB, 0.36052E+00_JPRB, 0.36278E+00_JPRB, 0.36199E+00_JPRB, 0.36013E+00_JPRB ,&
&  0.35548E+00_JPRB, 0.35393E+00_JPRB, 0.35098E+00_JPRB, 0.34752E+00_JPRB, 0.34361E+00_JPRB, 0.33829E+00_JPRB, 0.33420E+00_JPRB ,&
&  0.32948E+00_JPRB, 0.32421E+00_JPRB, 0.31819E+00_JPRB, 0.31122E+00_JPRB, 0.30486E+00_JPRB, 0.29882E+00_JPRB, 0.29599E+00_JPRB ,&
&  0.29481E+00_JPRB, 0.29611E+00_JPRB, 0.29977E+00_JPRB, 0.30380E+00_JPRB, 0.30864E+00_JPRB, 0.31405E+00_JPRB, 0.32077E+00_JPRB ,&
&  0.32631E+00_JPRB, 0.33278E+00_JPRB, 0.33666E+00_JPRB, 0.33816E+00_JPRB, 0.33676E+00_JPRB, 0.33693E+00_JPRB, 0.33618E+00_JPRB ,&
&  0.33518E+00_JPRB, 0.33391E+00_JPRB, 0.33402E+00_JPRB, 0.33508E+00_JPRB, 0.33511E+00_JPRB, 0.33395E+00_JPRB, 0.33168E+00_JPRB ,&
&  0.32927E+00_JPRB, 0.32719E+00_JPRB, 0.32408E+00_JPRB, 0.32141E+00_JPRB, 0.31946E+00_JPRB, 0.31679E+00_JPRB, 0.31303E+00_JPRB ,&
&  0.30975E+00_JPRB, 0.30654E+00_JPRB, 0.30281E+00_JPRB, 0.29982E+00_JPRB, 0.29685E+00_JPRB, 0.29473E+00_JPRB, 0.29280E+00_JPRB ,&
&  0.29209E+00_JPRB, 0.29127E+00_JPRB, 0.29109E+00_JPRB, 0.29102E+00_JPRB, 0.29160E+00_JPRB, 0.29120E+00_JPRB, 0.29060E+00_JPRB ,&
&  0.28991E+00_JPRB, 0.28972E+00_JPRB, 0.28901E+00_JPRB, 0.28873E+00_JPRB, 0.28845E+00_JPRB, 0.28743E+00_JPRB, 0.28684E+00_JPRB ,&
&  0.28666E+00_JPRB, 0.28631E+00_JPRB, 0.28535E+00_JPRB, 0.28507E+00_JPRB, 0.28442E+00_JPRB, 0.28357E+00_JPRB, 0.28385E+00_JPRB ,&
&  0.28358E+00_JPRB, 0.28379E+00_JPRB, 0.28414E+00_JPRB, 0.28466E+00_JPRB, 0.28594E+00_JPRB, 0.28751E+00_JPRB, 0.28964E+00_JPRB ,&
&  0.29303E+00_JPRB, 0.29603E+00_JPRB, 0.29898E+00_JPRB, 0.30232E+00_JPRB, 0.30928E+00_JPRB, 0.31124E+00_JPRB, 0.31522E+00_JPRB ,&
&  0.31827E+00_JPRB, 0.31219E+00_JPRB, 0.33625E+00_JPRB, 0.33771E+00_JPRB, 0.34139E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.30447E+00_JPRB, 0.30968E+00_JPRB, 0.31291E+00_JPRB, 0.31498E+00_JPRB, 0.30768E+00_JPRB, 0.32864E+00_JPRB, 0.32697E+00_JPRB ,&
&  0.32794E+00_JPRB, 0.32442E+00_JPRB, 0.32713E+00_JPRB, 0.32658E+00_JPRB, 0.32452E+00_JPRB, 0.32159E+00_JPRB, 0.31858E+00_JPRB ,&
&  0.31536E+00_JPRB, 0.31138E+00_JPRB, 0.30786E+00_JPRB, 0.30437E+00_JPRB, 0.30148E+00_JPRB, 0.29887E+00_JPRB, 0.29756E+00_JPRB ,&
&  0.29779E+00_JPRB, 0.29961E+00_JPRB, 0.30272E+00_JPRB, 0.30403E+00_JPRB, 0.30550E+00_JPRB, 0.30709E+00_JPRB, 0.30885E+00_JPRB ,&
&  0.31012E+00_JPRB, 0.31205E+00_JPRB, 0.31301E+00_JPRB, 0.31245E+00_JPRB, 0.31182E+00_JPRB, 0.31206E+00_JPRB, 0.31261E+00_JPRB ,&
&  0.31390E+00_JPRB, 0.31485E+00_JPRB, 0.31553E+00_JPRB, 0.31629E+00_JPRB, 0.31705E+00_JPRB, 0.31686E+00_JPRB, 0.31569E+00_JPRB ,&
&  0.31468E+00_JPRB, 0.31384E+00_JPRB, 0.31181E+00_JPRB, 0.30990E+00_JPRB, 0.30809E+00_JPRB, 0.30619E+00_JPRB, 0.30417E+00_JPRB ,&
&  0.30275E+00_JPRB, 0.30132E+00_JPRB, 0.29953E+00_JPRB, 0.29747E+00_JPRB, 0.29560E+00_JPRB, 0.29397E+00_JPRB, 0.29273E+00_JPRB ,&
&  0.29259E+00_JPRB, 0.29192E+00_JPRB, 0.29134E+00_JPRB, 0.29076E+00_JPRB, 0.29065E+00_JPRB, 0.29006E+00_JPRB, 0.28961E+00_JPRB ,&
&  0.28925E+00_JPRB, 0.28877E+00_JPRB, 0.28822E+00_JPRB, 0.28766E+00_JPRB, 0.28727E+00_JPRB, 0.28678E+00_JPRB, 0.28626E+00_JPRB ,&
&  0.28590E+00_JPRB, 0.28561E+00_JPRB, 0.28532E+00_JPRB, 0.28473E+00_JPRB, 0.28415E+00_JPRB, 0.28346E+00_JPRB, 0.28339E+00_JPRB ,&
&  0.28349E+00_JPRB, 0.28375E+00_JPRB, 0.28426E+00_JPRB, 0.28435E+00_JPRB, 0.28537E+00_JPRB, 0.28634E+00_JPRB, 0.28751E+00_JPRB ,&
&  0.28869E+00_JPRB, 0.29060E+00_JPRB, 0.29273E+00_JPRB, 0.29510E+00_JPRB, 0.29907E+00_JPRB, 0.29996E+00_JPRB, 0.30063E+00_JPRB ,&
&  0.30153E+00_JPRB, 0.30593E+00_JPRB, 0.30634E+00_JPRB, 0.30542E+00_JPRB, 0.30542E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.29516E+00_JPRB, 0.29541E+00_JPRB, 0.29527E+00_JPRB, 0.29488E+00_JPRB, 0.29200E+00_JPRB, 0.29068E+00_JPRB, 0.28956E+00_JPRB ,&
&  0.28871E+00_JPRB, 0.28830E+00_JPRB, 0.28881E+00_JPRB, 0.28949E+00_JPRB, 0.29022E+00_JPRB, 0.29099E+00_JPRB, 0.29170E+00_JPRB ,&
&  0.29210E+00_JPRB, 0.29254E+00_JPRB, 0.29119E+00_JPRB, 0.29052E+00_JPRB, 0.29028E+00_JPRB, 0.29030E+00_JPRB, 0.29027E+00_JPRB ,&
&  0.29035E+00_JPRB, 0.29041E+00_JPRB, 0.29035E+00_JPRB, 0.29081E+00_JPRB, 0.29158E+00_JPRB, 0.29219E+00_JPRB, 0.29283E+00_JPRB ,&
&  0.29347E+00_JPRB, 0.29442E+00_JPRB, 0.29530E+00_JPRB, 0.29614E+00_JPRB, 0.29734E+00_JPRB, 0.29856E+00_JPRB, 0.29936E+00_JPRB ,&
&  0.30003E+00_JPRB, 0.30036E+00_JPRB, 0.30075E+00_JPRB, 0.30106E+00_JPRB, 0.30151E+00_JPRB, 0.30203E+00_JPRB, 0.30220E+00_JPRB ,&
&  0.30219E+00_JPRB, 0.30209E+00_JPRB, 0.30187E+00_JPRB, 0.30122E+00_JPRB, 0.30052E+00_JPRB, 0.29984E+00_JPRB, 0.29915E+00_JPRB ,&
&  0.29856E+00_JPRB, 0.29787E+00_JPRB, 0.29711E+00_JPRB, 0.29631E+00_JPRB, 0.29546E+00_JPRB, 0.29464E+00_JPRB, 0.29381E+00_JPRB ,&
&  0.29296E+00_JPRB, 0.29242E+00_JPRB, 0.29182E+00_JPRB, 0.29130E+00_JPRB, 0.29087E+00_JPRB, 0.29043E+00_JPRB, 0.29000E+00_JPRB ,&
&  0.28958E+00_JPRB, 0.28908E+00_JPRB, 0.28862E+00_JPRB, 0.28811E+00_JPRB, 0.28767E+00_JPRB, 0.28716E+00_JPRB, 0.28674E+00_JPRB ,&
&  0.28633E+00_JPRB, 0.28597E+00_JPRB, 0.28567E+00_JPRB, 0.28552E+00_JPRB, 0.28542E+00_JPRB, 0.28543E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28549E+00_JPRB, 0.28550E+00_JPRB, 0.28549E+00_JPRB, 0.28528E+00_JPRB, 0.28552E+00_JPRB, 0.28577E+00_JPRB, 0.28614E+00_JPRB ,&
&  0.28649E+00_JPRB, 0.28709E+00_JPRB, 0.28772E+00_JPRB, 0.28844E+00_JPRB, 0.28962E+00_JPRB, 0.29004E+00_JPRB, 0.29038E+00_JPRB ,&
&  0.29073E+00_JPRB, 0.29151E+00_JPRB, 0.29213E+00_JPRB, 0.29245E+00_JPRB, 0.29316E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB, 0.29009E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB, 0.89781E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.87268E+00_JPRB, 0.86902E+00_JPRB, 0.86516E+00_JPRB, 0.86113E+00_JPRB, 0.85674E+00_JPRB, 0.85173E+00_JPRB, 0.84609E+00_JPRB ,&
&  0.84094E+00_JPRB, 0.83617E+00_JPRB, 0.83189E+00_JPRB, 0.82810E+00_JPRB, 0.82488E+00_JPRB, 0.82240E+00_JPRB, 0.82158E+00_JPRB ,&
&  0.82162E+00_JPRB, 0.82177E+00_JPRB, 0.82224E+00_JPRB, 0.82256E+00_JPRB, 0.82247E+00_JPRB, 0.82221E+00_JPRB, 0.82178E+00_JPRB ,&
&  0.82156E+00_JPRB, 0.82148E+00_JPRB, 0.82147E+00_JPRB, 0.82192E+00_JPRB, 0.82247E+00_JPRB, 0.82306E+00_JPRB, 0.82356E+00_JPRB ,&
&  0.82458E+00_JPRB, 0.82583E+00_JPRB, 0.82759E+00_JPRB, 0.82981E+00_JPRB, 0.83441E+00_JPRB, 0.83927E+00_JPRB, 0.84432E+00_JPRB ,&
&  0.84904E+00_JPRB, 0.85373E+00_JPRB, 0.85785E+00_JPRB, 0.86147E+00_JPRB, 0.86484E+00_JPRB, 0.86904E+00_JPRB, 0.87393E+00_JPRB ,&
&  0.87922E+00_JPRB, 0.88503E+00_JPRB, 0.89166E+00_JPRB, 0.89827E+00_JPRB, 0.90430E+00_JPRB, 0.91049E+00_JPRB, 0.91706E+00_JPRB ,&
&  0.92352E+00_JPRB, 0.92993E+00_JPRB, 0.93632E+00_JPRB, 0.94285E+00_JPRB, 0.94925E+00_JPRB, 0.95667E+00_JPRB, 0.96538E+00_JPRB ,&
&  0.97502E+00_JPRB, 0.98269E+00_JPRB, 0.99143E+00_JPRB, 0.10008E+01_JPRB, 0.10144E+01_JPRB, 0.10211E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10424E+01_JPRB, 0.10481E+01_JPRB, 0.10538E+01_JPRB, 0.10591E+01_JPRB, 0.10637E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.10715E+01_JPRB, 0.10717E+01_JPRB, 0.10709E+01_JPRB, 0.10687E+01_JPRB, 0.10661E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10492E+01_JPRB, 0.10444E+01_JPRB, 0.10374E+01_JPRB, 0.10305E+01_JPRB, 0.10232E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.99670E+00_JPRB, 0.98636E+00_JPRB, 0.97528E+00_JPRB, 0.96451E+00_JPRB, 0.95329E+00_JPRB, 0.94107E+00_JPRB ,&
&  0.92753E+00_JPRB, 0.91482E+00_JPRB, 0.90065E+00_JPRB, 0.88659E+00_JPRB, 0.87649E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.85258E+00_JPRB, 0.84059E+00_JPRB, 0.83551E+00_JPRB, 0.83227E+00_JPRB, 0.82957E+00_JPRB, 0.82601E+00_JPRB, 0.82187E+00_JPRB ,&
&  0.81877E+00_JPRB, 0.81508E+00_JPRB, 0.81217E+00_JPRB, 0.81176E+00_JPRB, 0.81027E+00_JPRB, 0.80953E+00_JPRB, 0.80867E+00_JPRB ,&
&  0.80821E+00_JPRB, 0.80764E+00_JPRB, 0.80773E+00_JPRB, 0.80635E+00_JPRB, 0.80422E+00_JPRB, 0.80122E+00_JPRB, 0.79880E+00_JPRB ,&
&  0.79659E+00_JPRB, 0.79505E+00_JPRB, 0.79275E+00_JPRB, 0.79084E+00_JPRB, 0.79175E+00_JPRB, 0.79203E+00_JPRB, 0.79161E+00_JPRB ,&
&  0.79477E+00_JPRB, 0.79656E+00_JPRB, 0.79578E+00_JPRB, 0.79269E+00_JPRB, 0.78897E+00_JPRB, 0.78690E+00_JPRB, 0.78900E+00_JPRB ,&
&  0.79309E+00_JPRB, 0.80529E+00_JPRB, 0.81520E+00_JPRB, 0.82454E+00_JPRB, 0.83328E+00_JPRB, 0.84394E+00_JPRB, 0.85651E+00_JPRB ,&
&  0.86962E+00_JPRB, 0.87997E+00_JPRB, 0.89165E+00_JPRB, 0.90587E+00_JPRB, 0.92131E+00_JPRB, 0.93767E+00_JPRB, 0.95573E+00_JPRB ,&
&  0.97119E+00_JPRB, 0.98700E+00_JPRB, 0.10054E+01_JPRB, 0.10255E+01_JPRB, 0.10428E+01_JPRB, 0.10653E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11434E+01_JPRB, 0.11629E+01_JPRB, 0.11822E+01_JPRB, 0.12068E+01_JPRB, 0.12212E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.12435E+01_JPRB, 0.12511E+01_JPRB, 0.12483E+01_JPRB, 0.12433E+01_JPRB, 0.12398E+01_JPRB, 0.12347E+01_JPRB, 0.12295E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.12228E+01_JPRB, 0.12152E+01_JPRB, 0.12072E+01_JPRB, 0.11965E+01_JPRB, 0.11857E+01_JPRB, 0.11659E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.11338E+01_JPRB, 0.11244E+01_JPRB, 0.11123E+01_JPRB, 0.11003E+01_JPRB, 0.10894E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.10561E+01_JPRB, 0.10430E+01_JPRB, 0.10290E+01_JPRB, 0.10174E+01_JPRB, 0.10009E+01_JPRB, 0.98225E+00_JPRB ,&
&  0.96040E+00_JPRB, 0.94143E+00_JPRB, 0.91927E+00_JPRB, 0.89332E+00_JPRB, 0.86798E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.87056E+00_JPRB, 0.84232E+00_JPRB, 0.82743E+00_JPRB, 0.82329E+00_JPRB, 0.82125E+00_JPRB, 0.81913E+00_JPRB, 0.81692E+00_JPRB ,&
&  0.81315E+00_JPRB, 0.81044E+00_JPRB, 0.81147E+00_JPRB, 0.81301E+00_JPRB, 0.81312E+00_JPRB, 0.81295E+00_JPRB, 0.81119E+00_JPRB ,&
&  0.80893E+00_JPRB, 0.80488E+00_JPRB, 0.80070E+00_JPRB, 0.79343E+00_JPRB, 0.78561E+00_JPRB, 0.77587E+00_JPRB, 0.76215E+00_JPRB ,&
&  0.75156E+00_JPRB, 0.74231E+00_JPRB, 0.73014E+00_JPRB, 0.72701E+00_JPRB, 0.72647E+00_JPRB, 0.69870E+00_JPRB, 0.69374E+00_JPRB ,&
&  0.70570E+00_JPRB, 0.70851E+00_JPRB, 0.70916E+00_JPRB, 0.70930E+00_JPRB, 0.70168E+00_JPRB, 0.68671E+00_JPRB, 0.68115E+00_JPRB ,&
&  0.65888E+00_JPRB, 0.67979E+00_JPRB, 0.69402E+00_JPRB, 0.70586E+00_JPRB, 0.71699E+00_JPRB, 0.73162E+00_JPRB, 0.76298E+00_JPRB ,&
&  0.82224E+00_JPRB, 0.84693E+00_JPRB, 0.87578E+00_JPRB, 0.90357E+00_JPRB, 0.93954E+00_JPRB, 0.97692E+00_JPRB, 0.10116E+01_JPRB ,&
&  0.10423E+01_JPRB, 0.10755E+01_JPRB, 0.11141E+01_JPRB, 0.11635E+01_JPRB, 0.12041E+01_JPRB, 0.12413E+01_JPRB, 0.12686E+01_JPRB ,&
&  0.12993E+01_JPRB, 0.13216E+01_JPRB, 0.13378E+01_JPRB, 0.13476E+01_JPRB, 0.13587E+01_JPRB, 0.13640E+01_JPRB, 0.13705E+01_JPRB ,&
&  0.13737E+01_JPRB, 0.13737E+01_JPRB, 0.13636E+01_JPRB, 0.13446E+01_JPRB, 0.13321E+01_JPRB, 0.13271E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.12964E+01_JPRB, 0.12816E+01_JPRB, 0.12635E+01_JPRB, 0.12402E+01_JPRB, 0.12199E+01_JPRB, 0.11950E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.11572E+01_JPRB, 0.11433E+01_JPRB, 0.11267E+01_JPRB, 0.11096E+01_JPRB, 0.10927E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10430E+01_JPRB, 0.10315E+01_JPRB, 0.10193E+01_JPRB, 0.10108E+01_JPRB, 0.99888E+00_JPRB, 0.98811E+00_JPRB ,&
&  0.97587E+00_JPRB, 0.96479E+00_JPRB, 0.94749E+00_JPRB, 0.92460E+00_JPRB, 0.89508E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.90622E+00_JPRB, 0.87567E+00_JPRB, 0.84324E+00_JPRB, 0.82504E+00_JPRB, 0.82072E+00_JPRB, 0.81692E+00_JPRB, 0.81371E+00_JPRB ,&
&  0.81193E+00_JPRB, 0.81443E+00_JPRB, 0.81719E+00_JPRB, 0.81834E+00_JPRB, 0.81710E+00_JPRB, 0.81380E+00_JPRB, 0.80901E+00_JPRB ,&
&  0.80229E+00_JPRB, 0.79370E+00_JPRB, 0.78675E+00_JPRB, 0.77940E+00_JPRB, 0.77120E+00_JPRB, 0.76260E+00_JPRB, 0.74873E+00_JPRB ,&
&  0.73556E+00_JPRB, 0.72350E+00_JPRB, 0.68145E+00_JPRB, 0.68122E+00_JPRB, 0.67442E+00_JPRB, 0.67326E+00_JPRB, 0.67810E+00_JPRB ,&
&  0.67544E+00_JPRB, 0.66245E+00_JPRB, 0.64747E+00_JPRB, 0.62755E+00_JPRB, 0.61361E+00_JPRB, 0.59846E+00_JPRB, 0.59214E+00_JPRB ,&
&  0.59086E+00_JPRB, 0.59588E+00_JPRB, 0.60849E+00_JPRB, 0.61985E+00_JPRB, 0.63202E+00_JPRB, 0.64243E+00_JPRB, 0.66220E+00_JPRB ,&
&  0.71230E+00_JPRB, 0.78100E+00_JPRB, 0.86462E+00_JPRB, 0.91034E+00_JPRB, 0.96620E+00_JPRB, 0.10197E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.11417E+01_JPRB, 0.11870E+01_JPRB, 0.12364E+01_JPRB, 0.12850E+01_JPRB, 0.13253E+01_JPRB, 0.13760E+01_JPRB ,&
&  0.14178E+01_JPRB, 0.14533E+01_JPRB, 0.14694E+01_JPRB, 0.14781E+01_JPRB, 0.14786E+01_JPRB, 0.14664E+01_JPRB, 0.14534E+01_JPRB ,&
&  0.14434E+01_JPRB, 0.14310E+01_JPRB, 0.14143E+01_JPRB, 0.13771E+01_JPRB, 0.13582E+01_JPRB, 0.13516E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.13057E+01_JPRB, 0.12771E+01_JPRB, 0.12490E+01_JPRB, 0.12098E+01_JPRB, 0.11770E+01_JPRB, 0.11653E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.11378E+01_JPRB, 0.11280E+01_JPRB, 0.11108E+01_JPRB, 0.10838E+01_JPRB, 0.10572E+01_JPRB, 0.10351E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.99704E+00_JPRB, 0.98048E+00_JPRB, 0.96385E+00_JPRB, 0.95363E+00_JPRB, 0.94566E+00_JPRB, 0.94354E+00_JPRB ,&
&  0.94482E+00_JPRB, 0.94700E+00_JPRB, 0.94651E+00_JPRB, 0.94197E+00_JPRB, 0.92769E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.99411E+00_JPRB, 0.98112E+00_JPRB, 0.95382E+00_JPRB, 0.91737E+00_JPRB, 0.88862E+00_JPRB, 0.86814E+00_JPRB, 0.85771E+00_JPRB ,&
&  0.85094E+00_JPRB, 0.84771E+00_JPRB, 0.84377E+00_JPRB, 0.83961E+00_JPRB, 0.83282E+00_JPRB, 0.82473E+00_JPRB, 0.81610E+00_JPRB ,&
&  0.80209E+00_JPRB, 0.78871E+00_JPRB, 0.77533E+00_JPRB, 0.76672E+00_JPRB, 0.75958E+00_JPRB, 0.75399E+00_JPRB, 0.74881E+00_JPRB ,&
&  0.74325E+00_JPRB, 0.73615E+00_JPRB, 0.71318E+00_JPRB, 0.70540E+00_JPRB, 0.69637E+00_JPRB, 0.69603E+00_JPRB, 0.69145E+00_JPRB ,&
&  0.70321E+00_JPRB, 0.63777E+00_JPRB, 0.63178E+00_JPRB, 0.61514E+00_JPRB, 0.60346E+00_JPRB, 0.58974E+00_JPRB, 0.57467E+00_JPRB ,&
&  0.56690E+00_JPRB, 0.55730E+00_JPRB, 0.55569E+00_JPRB, 0.56736E+00_JPRB, 0.57663E+00_JPRB, 0.59444E+00_JPRB, 0.61278E+00_JPRB ,&
&  0.64418E+00_JPRB, 0.78701E+00_JPRB, 0.85930E+00_JPRB, 0.91117E+00_JPRB, 0.96453E+00_JPRB, 0.10129E+01_JPRB, 0.10395E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.11090E+01_JPRB, 0.11431E+01_JPRB, 0.11590E+01_JPRB, 0.12472E+01_JPRB, 0.13111E+01_JPRB, 0.13804E+01_JPRB ,&
&  0.14441E+01_JPRB, 0.15058E+01_JPRB, 0.15590E+01_JPRB, 0.15997E+01_JPRB, 0.16321E+01_JPRB, 0.16323E+01_JPRB, 0.16118E+01_JPRB ,&
&  0.15962E+01_JPRB, 0.15829E+01_JPRB, 0.15743E+01_JPRB, 0.15249E+01_JPRB, 0.14932E+01_JPRB, 0.14723E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.13143E+01_JPRB, 0.12632E+01_JPRB, 0.12100E+01_JPRB, 0.11616E+01_JPRB, 0.11340E+01_JPRB, 0.11223E+01_JPRB ,&
&  0.11198E+01_JPRB, 0.11119E+01_JPRB, 0.10943E+01_JPRB, 0.10666E+01_JPRB, 0.10366E+01_JPRB, 0.10170E+01_JPRB, 0.99457E+00_JPRB ,&
&  0.97246E+00_JPRB, 0.94716E+00_JPRB, 0.93155E+00_JPRB, 0.92104E+00_JPRB, 0.91845E+00_JPRB, 0.92178E+00_JPRB, 0.93032E+00_JPRB ,&
&  0.94045E+00_JPRB, 0.95318E+00_JPRB, 0.96798E+00_JPRB, 0.98203E+00_JPRB, 0.99205E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.99111E+00_JPRB, 0.99912E+00_JPRB, 0.10005E+01_JPRB, 0.99315E+00_JPRB, 0.97805E+00_JPRB, 0.95802E+00_JPRB, 0.93793E+00_JPRB ,&
&  0.91893E+00_JPRB, 0.90407E+00_JPRB, 0.88933E+00_JPRB, 0.87752E+00_JPRB, 0.86325E+00_JPRB, 0.85277E+00_JPRB, 0.84073E+00_JPRB ,&
&  0.83294E+00_JPRB, 0.82883E+00_JPRB, 0.81900E+00_JPRB, 0.80791E+00_JPRB, 0.78969E+00_JPRB, 0.77376E+00_JPRB, 0.76128E+00_JPRB ,&
&  0.75003E+00_JPRB, 0.74026E+00_JPRB, 0.71837E+00_JPRB, 0.70285E+00_JPRB, 0.69457E+00_JPRB, 0.68539E+00_JPRB, 0.69073E+00_JPRB ,&
&  0.68761E+00_JPRB, 0.69169E+00_JPRB, 0.69528E+00_JPRB, 0.68814E+00_JPRB, 0.67927E+00_JPRB, 0.56938E+00_JPRB, 0.54211E+00_JPRB ,&
&  0.53534E+00_JPRB, 0.52793E+00_JPRB, 0.52756E+00_JPRB, 0.53539E+00_JPRB, 0.54797E+00_JPRB, 0.58213E+00_JPRB, 0.60596E+00_JPRB ,&
&  0.62003E+00_JPRB, 0.76976E+00_JPRB, 0.83239E+00_JPRB, 0.88696E+00_JPRB, 0.92837E+00_JPRB, 0.96021E+00_JPRB, 0.97976E+00_JPRB ,&
&  0.99998E+00_JPRB, 0.10291E+01_JPRB, 0.10465E+01_JPRB, 0.10837E+01_JPRB, 0.11555E+01_JPRB, 0.12755E+01_JPRB, 0.13861E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.15771E+01_JPRB, 0.16373E+01_JPRB, 0.16688E+01_JPRB, 0.17182E+01_JPRB, 0.17496E+01_JPRB, 0.17578E+01_JPRB ,&
&  0.17715E+01_JPRB, 0.17477E+01_JPRB, 0.17567E+01_JPRB, 0.17274E+01_JPRB, 0.17016E+01_JPRB, 0.16718E+01_JPRB, 0.15764E+01_JPRB ,&
&  0.14638E+01_JPRB, 0.13883E+01_JPRB, 0.13190E+01_JPRB, 0.12316E+01_JPRB, 0.11671E+01_JPRB, 0.11207E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.10917E+01_JPRB, 0.10896E+01_JPRB, 0.10632E+01_JPRB, 0.10318E+01_JPRB, 0.10102E+01_JPRB, 0.99636E+00_JPRB, 0.97195E+00_JPRB ,&
&  0.95085E+00_JPRB, 0.93042E+00_JPRB, 0.91906E+00_JPRB, 0.90830E+00_JPRB, 0.90140E+00_JPRB, 0.90017E+00_JPRB, 0.90384E+00_JPRB ,&
&  0.91250E+00_JPRB, 0.92496E+00_JPRB, 0.94229E+00_JPRB, 0.96220E+00_JPRB, 0.98347E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.97706E+00_JPRB, 0.99191E+00_JPRB, 0.99906E+00_JPRB, 0.99646E+00_JPRB, 0.98139E+00_JPRB, 0.96168E+00_JPRB, 0.94401E+00_JPRB ,&
&  0.92669E+00_JPRB, 0.91036E+00_JPRB, 0.89472E+00_JPRB, 0.88259E+00_JPRB, 0.87172E+00_JPRB, 0.86335E+00_JPRB, 0.85668E+00_JPRB ,&
&  0.85163E+00_JPRB, 0.82934E+00_JPRB, 0.81942E+00_JPRB, 0.81736E+00_JPRB, 0.81727E+00_JPRB, 0.81331E+00_JPRB, 0.80624E+00_JPRB ,&
&  0.78318E+00_JPRB, 0.75162E+00_JPRB, 0.72193E+00_JPRB, 0.69593E+00_JPRB, 0.67465E+00_JPRB, 0.66523E+00_JPRB, 0.66898E+00_JPRB ,&
&  0.67194E+00_JPRB, 0.66453E+00_JPRB, 0.66630E+00_JPRB, 0.68575E+00_JPRB, 0.68899E+00_JPRB, 0.62451E+00_JPRB, 0.54298E+00_JPRB ,&
&  0.53370E+00_JPRB, 0.51442E+00_JPRB, 0.51436E+00_JPRB, 0.52221E+00_JPRB, 0.53759E+00_JPRB, 0.56074E+00_JPRB, 0.59948E+00_JPRB ,&
&  0.64491E+00_JPRB, 0.75144E+00_JPRB, 0.81371E+00_JPRB, 0.85846E+00_JPRB, 0.89737E+00_JPRB, 0.93040E+00_JPRB, 0.94605E+00_JPRB ,&
&  0.95567E+00_JPRB, 0.97742E+00_JPRB, 0.10157E+01_JPRB, 0.10615E+01_JPRB, 0.11420E+01_JPRB, 0.13083E+01_JPRB, 0.14552E+01_JPRB ,&
&  0.16557E+01_JPRB, 0.17670E+01_JPRB, 0.17637E+01_JPRB, 0.16998E+01_JPRB, 0.17421E+01_JPRB, 0.18297E+01_JPRB, 0.18528E+01_JPRB ,&
&  0.19244E+01_JPRB, 0.20111E+01_JPRB, 0.19917E+01_JPRB, 0.19032E+01_JPRB, 0.18474E+01_JPRB, 0.18142E+01_JPRB, 0.16805E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.14768E+01_JPRB, 0.14846E+01_JPRB, 0.13390E+01_JPRB, 0.12393E+01_JPRB, 0.11456E+01_JPRB, 0.10924E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10302E+01_JPRB, 0.99348E+00_JPRB, 0.96543E+00_JPRB, 0.95085E+00_JPRB, 0.94187E+00_JPRB, 0.93166E+00_JPRB ,&
&  0.92001E+00_JPRB, 0.90956E+00_JPRB, 0.89914E+00_JPRB, 0.89216E+00_JPRB, 0.88456E+00_JPRB, 0.88168E+00_JPRB, 0.88245E+00_JPRB ,&
&  0.88830E+00_JPRB, 0.90005E+00_JPRB, 0.91681E+00_JPRB, 0.93634E+00_JPRB, 0.95973E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.98954E+00_JPRB, 0.10008E+01_JPRB, 0.99223E+00_JPRB, 0.96753E+00_JPRB, 0.93166E+00_JPRB, 0.88688E+00_JPRB, 0.85404E+00_JPRB ,&
&  0.83495E+00_JPRB, 0.82613E+00_JPRB, 0.81823E+00_JPRB, 0.80985E+00_JPRB, 0.80638E+00_JPRB, 0.80540E+00_JPRB, 0.80664E+00_JPRB ,&
&  0.80993E+00_JPRB, 0.81304E+00_JPRB, 0.81178E+00_JPRB, 0.80097E+00_JPRB, 0.79756E+00_JPRB, 0.79738E+00_JPRB, 0.79968E+00_JPRB ,&
&  0.79144E+00_JPRB, 0.76421E+00_JPRB, 0.73142E+00_JPRB, 0.70659E+00_JPRB, 0.68259E+00_JPRB, 0.67381E+00_JPRB, 0.66061E+00_JPRB ,&
&  0.63917E+00_JPRB, 0.63427E+00_JPRB, 0.65251E+00_JPRB, 0.68378E+00_JPRB, 0.67814E+00_JPRB, 0.69454E+00_JPRB, 0.57828E+00_JPRB ,&
&  0.54169E+00_JPRB, 0.52123E+00_JPRB, 0.51671E+00_JPRB, 0.52992E+00_JPRB, 0.56538E+00_JPRB, 0.60745E+00_JPRB, 0.64261E+00_JPRB ,&
&  0.67988E+00_JPRB, 0.74782E+00_JPRB, 0.78672E+00_JPRB, 0.81887E+00_JPRB, 0.85753E+00_JPRB, 0.89733E+00_JPRB, 0.92678E+00_JPRB ,&
&  0.94673E+00_JPRB, 0.98196E+00_JPRB, 0.10412E+01_JPRB, 0.11354E+01_JPRB, 0.13371E+01_JPRB, 0.13309E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.18072E+01_JPRB, 0.20211E+01_JPRB, 0.18983E+01_JPRB, 0.17901E+01_JPRB, 0.17599E+01_JPRB, 0.18155E+01_JPRB, 0.19106E+01_JPRB ,&
&  0.20901E+01_JPRB, 0.22459E+01_JPRB, 0.22432E+01_JPRB, 0.20418E+01_JPRB, 0.19403E+01_JPRB, 0.18824E+01_JPRB, 0.17033E+01_JPRB ,&
&  0.15055E+01_JPRB, 0.15379E+01_JPRB, 0.16082E+01_JPRB, 0.13702E+01_JPRB, 0.12062E+01_JPRB, 0.11008E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.10065E+01_JPRB, 0.97968E+00_JPRB, 0.95441E+00_JPRB, 0.92286E+00_JPRB, 0.89473E+00_JPRB, 0.87043E+00_JPRB ,&
&  0.85345E+00_JPRB, 0.84182E+00_JPRB, 0.84125E+00_JPRB, 0.84644E+00_JPRB, 0.84974E+00_JPRB, 0.85372E+00_JPRB, 0.86522E+00_JPRB ,&
&  0.88111E+00_JPRB, 0.88918E+00_JPRB, 0.90498E+00_JPRB, 0.92755E+00_JPRB, 0.95913E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.95271E+00_JPRB, 0.95606E+00_JPRB, 0.94217E+00_JPRB, 0.91379E+00_JPRB, 0.87613E+00_JPRB, 0.82480E+00_JPRB, 0.78325E+00_JPRB ,&
&  0.75853E+00_JPRB, 0.74402E+00_JPRB, 0.73511E+00_JPRB, 0.73056E+00_JPRB, 0.73083E+00_JPRB, 0.73584E+00_JPRB, 0.73590E+00_JPRB ,&
&  0.73816E+00_JPRB, 0.74047E+00_JPRB, 0.74511E+00_JPRB, 0.75200E+00_JPRB, 0.75547E+00_JPRB, 0.75750E+00_JPRB, 0.76029E+00_JPRB ,&
&  0.76270E+00_JPRB, 0.75047E+00_JPRB, 0.72298E+00_JPRB, 0.70245E+00_JPRB, 0.70418E+00_JPRB, 0.72005E+00_JPRB, 0.70097E+00_JPRB ,&
&  0.65630E+00_JPRB, 0.63547E+00_JPRB, 0.63499E+00_JPRB, 0.69144E+00_JPRB, 0.73125E+00_JPRB, 0.74368E+00_JPRB, 0.62658E+00_JPRB ,&
&  0.56369E+00_JPRB, 0.53956E+00_JPRB, 0.54452E+00_JPRB, 0.57833E+00_JPRB, 0.63331E+00_JPRB, 0.68809E+00_JPRB, 0.76046E+00_JPRB ,&
&  0.80842E+00_JPRB, 0.84858E+00_JPRB, 0.86045E+00_JPRB, 0.88482E+00_JPRB, 0.92550E+00_JPRB, 0.96279E+00_JPRB, 0.10036E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10753E+01_JPRB, 0.11948E+01_JPRB, 0.10871E+01_JPRB, 0.11472E+01_JPRB, 0.12222E+01_JPRB, 0.13664E+01_JPRB ,&
&  0.16322E+01_JPRB, 0.17397E+01_JPRB, 0.17721E+01_JPRB, 0.19053E+01_JPRB, 0.18236E+01_JPRB, 0.17718E+01_JPRB, 0.18240E+01_JPRB ,&
&  0.21083E+01_JPRB, 0.23642E+01_JPRB, 0.23208E+01_JPRB, 0.21229E+01_JPRB, 0.20522E+01_JPRB, 0.20092E+01_JPRB, 0.17912E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.13654E+01_JPRB, 0.12352E+01_JPRB, 0.10814E+01_JPRB, 0.10198E+01_JPRB, 0.98341E+00_JPRB, 0.96212E+00_JPRB ,&
&  0.94893E+00_JPRB, 0.93286E+00_JPRB, 0.92168E+00_JPRB, 0.86848E+00_JPRB, 0.82012E+00_JPRB, 0.76937E+00_JPRB, 0.73619E+00_JPRB ,&
&  0.71582E+00_JPRB, 0.71628E+00_JPRB, 0.73079E+00_JPRB, 0.75600E+00_JPRB, 0.80349E+00_JPRB, 0.84780E+00_JPRB, 0.87943E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.87991E+00_JPRB, 0.87722E+00_JPRB, 0.88894E+00_JPRB, 0.92748E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.90996E+00_JPRB, 0.87559E+00_JPRB, 0.85357E+00_JPRB, 0.85805E+00_JPRB, 0.81169E+00_JPRB, 0.76328E+00_JPRB, 0.73444E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.72897E+00_JPRB, 0.72738E+00_JPRB, 0.69924E+00_JPRB, 0.67442E+00_JPRB, 0.66155E+00_JPRB, 0.65671E+00_JPRB ,&
&  0.66813E+00_JPRB, 0.68301E+00_JPRB, 0.69584E+00_JPRB, 0.69598E+00_JPRB, 0.69433E+00_JPRB, 0.69709E+00_JPRB, 0.70683E+00_JPRB ,&
&  0.71926E+00_JPRB, 0.72406E+00_JPRB, 0.71535E+00_JPRB, 0.70770E+00_JPRB, 0.72749E+00_JPRB, 0.75121E+00_JPRB, 0.72279E+00_JPRB ,&
&  0.68997E+00_JPRB, 0.66244E+00_JPRB, 0.65611E+00_JPRB, 0.69742E+00_JPRB, 0.76766E+00_JPRB, 0.79259E+00_JPRB, 0.67663E+00_JPRB ,&
&  0.59561E+00_JPRB, 0.58307E+00_JPRB, 0.63037E+00_JPRB, 0.74634E+00_JPRB, 0.79820E+00_JPRB, 0.85358E+00_JPRB, 0.92254E+00_JPRB ,&
&  0.87654E+00_JPRB, 0.87990E+00_JPRB, 0.90816E+00_JPRB, 0.10479E+01_JPRB, 0.10880E+01_JPRB, 0.11067E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.11772E+01_JPRB, 0.13263E+01_JPRB, 0.11708E+01_JPRB, 0.12590E+01_JPRB, 0.15009E+01_JPRB, 0.15201E+01_JPRB ,&
&  0.16429E+01_JPRB, 0.17041E+01_JPRB, 0.19147E+01_JPRB, 0.18955E+01_JPRB, 0.17796E+01_JPRB, 0.17555E+01_JPRB, 0.19029E+01_JPRB ,&
&  0.22637E+01_JPRB, 0.25621E+01_JPRB, 0.24985E+01_JPRB, 0.23955E+01_JPRB, 0.23285E+01_JPRB, 0.22178E+01_JPRB, 0.19729E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.13716E+01_JPRB, 0.11790E+01_JPRB, 0.10137E+01_JPRB, 0.93510E+00_JPRB, 0.90575E+00_JPRB, 0.87574E+00_JPRB ,&
&  0.86843E+00_JPRB, 0.86086E+00_JPRB, 0.85300E+00_JPRB, 0.79617E+00_JPRB, 0.74807E+00_JPRB, 0.71486E+00_JPRB, 0.68980E+00_JPRB ,&
&  0.68307E+00_JPRB, 0.67281E+00_JPRB, 0.66696E+00_JPRB, 0.68993E+00_JPRB, 0.75501E+00_JPRB, 0.83186E+00_JPRB, 0.86998E+00_JPRB ,&
&  0.85812E+00_JPRB, 0.83164E+00_JPRB, 0.85304E+00_JPRB, 0.89733E+00_JPRB, 0.92202E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.92776E+00_JPRB, 0.92128E+00_JPRB, 0.91514E+00_JPRB, 0.90322E+00_JPRB, 0.87897E+00_JPRB, 0.81950E+00_JPRB, 0.78192E+00_JPRB ,&
&  0.79064E+00_JPRB, 0.76365E+00_JPRB, 0.74997E+00_JPRB, 0.71325E+00_JPRB, 0.67944E+00_JPRB, 0.64548E+00_JPRB, 0.62979E+00_JPRB ,&
&  0.61591E+00_JPRB, 0.62926E+00_JPRB, 0.65867E+00_JPRB, 0.66877E+00_JPRB, 0.67067E+00_JPRB, 0.67079E+00_JPRB, 0.68352E+00_JPRB ,&
&  0.69790E+00_JPRB, 0.71459E+00_JPRB, 0.72349E+00_JPRB, 0.73044E+00_JPRB, 0.74730E+00_JPRB, 0.77004E+00_JPRB, 0.76558E+00_JPRB ,&
&  0.75712E+00_JPRB, 0.73059E+00_JPRB, 0.70937E+00_JPRB, 0.73472E+00_JPRB, 0.79863E+00_JPRB, 0.85913E+00_JPRB, 0.82379E+00_JPRB ,&
&  0.66872E+00_JPRB, 0.67682E+00_JPRB, 0.84482E+00_JPRB, 0.90946E+00_JPRB, 0.97023E+00_JPRB, 0.10089E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.11463E+01_JPRB, 0.11785E+01_JPRB, 0.12040E+01_JPRB, 0.12228E+01_JPRB, 0.12469E+01_JPRB, 0.12465E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.11129E+01_JPRB, 0.10757E+01_JPRB, 0.11600E+01_JPRB, 0.15000E+01_JPRB, 0.16153E+01_JPRB, 0.16066E+01_JPRB ,&
&  0.16949E+01_JPRB, 0.17104E+01_JPRB, 0.18122E+01_JPRB, 0.19645E+01_JPRB, 0.20176E+01_JPRB, 0.21351E+01_JPRB, 0.23067E+01_JPRB ,&
&  0.25777E+01_JPRB, 0.28722E+01_JPRB, 0.28904E+01_JPRB, 0.28248E+01_JPRB, 0.26776E+01_JPRB, 0.24517E+01_JPRB, 0.20992E+01_JPRB ,&
&  0.16739E+01_JPRB, 0.13784E+01_JPRB, 0.11379E+01_JPRB, 0.97228E+00_JPRB, 0.87799E+00_JPRB, 0.83605E+00_JPRB, 0.80620E+00_JPRB ,&
&  0.80473E+00_JPRB, 0.80151E+00_JPRB, 0.79123E+00_JPRB, 0.74941E+00_JPRB, 0.72090E+00_JPRB, 0.73300E+00_JPRB, 0.74583E+00_JPRB ,&
&  0.75803E+00_JPRB, 0.75325E+00_JPRB, 0.71591E+00_JPRB, 0.73612E+00_JPRB, 0.80019E+00_JPRB, 0.86497E+00_JPRB, 0.88916E+00_JPRB ,&
&  0.88242E+00_JPRB, 0.85585E+00_JPRB, 0.87013E+00_JPRB, 0.90307E+00_JPRB, 0.91641E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.96006E+00_JPRB, 0.96467E+00_JPRB, 0.94844E+00_JPRB, 0.93572E+00_JPRB, 0.91575E+00_JPRB, 0.84277E+00_JPRB, 0.82075E+00_JPRB ,&
&  0.89155E+00_JPRB, 0.83229E+00_JPRB, 0.88180E+00_JPRB, 0.77322E+00_JPRB, 0.74442E+00_JPRB, 0.70754E+00_JPRB, 0.68040E+00_JPRB ,&
&  0.64495E+00_JPRB, 0.63263E+00_JPRB, 0.64890E+00_JPRB, 0.66788E+00_JPRB, 0.66944E+00_JPRB, 0.66369E+00_JPRB, 0.66803E+00_JPRB ,&
&  0.67675E+00_JPRB, 0.70295E+00_JPRB, 0.73835E+00_JPRB, 0.75543E+00_JPRB, 0.76887E+00_JPRB, 0.79340E+00_JPRB, 0.80443E+00_JPRB ,&
&  0.77850E+00_JPRB, 0.77488E+00_JPRB, 0.77359E+00_JPRB, 0.78999E+00_JPRB, 0.83020E+00_JPRB, 0.89226E+00_JPRB, 0.93322E+00_JPRB ,&
&  0.94617E+00_JPRB, 0.83797E+00_JPRB, 0.95065E+00_JPRB, 0.98369E+00_JPRB, 0.10485E+01_JPRB, 0.10866E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.11498E+01_JPRB, 0.11898E+01_JPRB, 0.12340E+01_JPRB, 0.12742E+01_JPRB, 0.13158E+01_JPRB, 0.13603E+01_JPRB, 0.13724E+01_JPRB ,&
&  0.14697E+01_JPRB, 0.11235E+01_JPRB, 0.12027E+01_JPRB, 0.12784E+01_JPRB, 0.16767E+01_JPRB, 0.18700E+01_JPRB, 0.18578E+01_JPRB ,&
&  0.21556E+01_JPRB, 0.20769E+01_JPRB, 0.22427E+01_JPRB, 0.23952E+01_JPRB, 0.25864E+01_JPRB, 0.28341E+01_JPRB, 0.29783E+01_JPRB ,&
&  0.30279E+01_JPRB, 0.31444E+01_JPRB, 0.32996E+01_JPRB, 0.30691E+01_JPRB, 0.27297E+01_JPRB, 0.23570E+01_JPRB, 0.19186E+01_JPRB ,&
&  0.16223E+01_JPRB, 0.13650E+01_JPRB, 0.11469E+01_JPRB, 0.99894E+00_JPRB, 0.88551E+00_JPRB, 0.81256E+00_JPRB, 0.77508E+00_JPRB ,&
&  0.75487E+00_JPRB, 0.73726E+00_JPRB, 0.73595E+00_JPRB, 0.72013E+00_JPRB, 0.71082E+00_JPRB, 0.72904E+00_JPRB, 0.77345E+00_JPRB ,&
&  0.78869E+00_JPRB, 0.80204E+00_JPRB, 0.85428E+00_JPRB, 0.95437E+00_JPRB, 0.97597E+00_JPRB, 0.10009E+01_JPRB, 0.10956E+01_JPRB ,&
&  0.97919E+00_JPRB, 0.97443E+00_JPRB, 0.10186E+01_JPRB, 0.99754E+00_JPRB, 0.96400E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.10322E+01_JPRB, 0.10242E+01_JPRB, 0.99007E+00_JPRB, 0.96824E+00_JPRB, 0.96223E+00_JPRB, 0.92189E+00_JPRB, 0.88546E+00_JPRB ,&
&  0.94613E+00_JPRB, 0.93469E+00_JPRB, 0.93361E+00_JPRB, 0.91989E+00_JPRB, 0.90406E+00_JPRB, 0.83175E+00_JPRB, 0.77794E+00_JPRB ,&
&  0.72832E+00_JPRB, 0.70460E+00_JPRB, 0.71755E+00_JPRB, 0.72020E+00_JPRB, 0.71602E+00_JPRB, 0.69897E+00_JPRB, 0.68985E+00_JPRB ,&
&  0.68252E+00_JPRB, 0.69669E+00_JPRB, 0.73483E+00_JPRB, 0.76728E+00_JPRB, 0.77337E+00_JPRB, 0.78175E+00_JPRB, 0.80501E+00_JPRB ,&
&  0.78438E+00_JPRB, 0.79007E+00_JPRB, 0.79967E+00_JPRB, 0.76575E+00_JPRB, 0.85899E+00_JPRB, 0.90533E+00_JPRB, 0.93215E+00_JPRB ,&
&  0.98092E+00_JPRB, 0.10234E+01_JPRB, 0.10455E+01_JPRB, 0.10549E+01_JPRB, 0.10789E+01_JPRB, 0.10943E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.11840E+01_JPRB, 0.12339E+01_JPRB, 0.12952E+01_JPRB, 0.14119E+01_JPRB, 0.13549E+01_JPRB, 0.14867E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.16051E+01_JPRB, 0.17125E+01_JPRB, 0.15843E+01_JPRB, 0.19799E+01_JPRB, 0.22197E+01_JPRB, 0.21023E+01_JPRB ,&
&  0.21880E+01_JPRB, 0.21689E+01_JPRB, 0.26167E+01_JPRB, 0.29738E+01_JPRB, 0.32757E+01_JPRB, 0.35452E+01_JPRB, 0.36451E+01_JPRB ,&
&  0.36088E+01_JPRB, 0.38051E+01_JPRB, 0.37355E+01_JPRB, 0.33686E+01_JPRB, 0.28820E+01_JPRB, 0.25014E+01_JPRB, 0.20281E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.13263E+01_JPRB, 0.11341E+01_JPRB, 0.10388E+01_JPRB, 0.88078E+00_JPRB, 0.82879E+00_JPRB, 0.77464E+00_JPRB ,&
&  0.72199E+00_JPRB, 0.69205E+00_JPRB, 0.70816E+00_JPRB, 0.73112E+00_JPRB, 0.72223E+00_JPRB, 0.74052E+00_JPRB, 0.75948E+00_JPRB ,&
&  0.83238E+00_JPRB, 0.98342E+00_JPRB, 0.12670E+01_JPRB, 0.13190E+01_JPRB, 0.12996E+01_JPRB, 0.12846E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.11648E+01_JPRB, 0.11171E+01_JPRB, 0.10632E+01_JPRB, 0.10269E+01_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.10888E+01_JPRB, 0.10699E+01_JPRB, 0.10457E+01_JPRB, 0.10236E+01_JPRB, 0.10087E+01_JPRB, 0.98304E+00_JPRB, 0.96437E+00_JPRB ,&
&  0.97322E+00_JPRB, 0.96433E+00_JPRB, 0.96098E+00_JPRB, 0.95274E+00_JPRB, 0.93332E+00_JPRB, 0.94354E+00_JPRB, 0.86451E+00_JPRB ,&
&  0.79130E+00_JPRB, 0.76888E+00_JPRB, 0.74764E+00_JPRB, 0.76147E+00_JPRB, 0.79048E+00_JPRB, 0.77815E+00_JPRB, 0.76088E+00_JPRB ,&
&  0.73984E+00_JPRB, 0.72421E+00_JPRB, 0.73254E+00_JPRB, 0.75469E+00_JPRB, 0.76817E+00_JPRB, 0.78135E+00_JPRB, 0.79826E+00_JPRB ,&
&  0.76865E+00_JPRB, 0.73922E+00_JPRB, 0.72826E+00_JPRB, 0.73811E+00_JPRB, 0.80797E+00_JPRB, 0.90825E+00_JPRB, 0.96657E+00_JPRB ,&
&  0.98929E+00_JPRB, 0.10079E+01_JPRB, 0.10499E+01_JPRB, 0.10703E+01_JPRB, 0.10829E+01_JPRB, 0.11087E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.11687E+01_JPRB, 0.12080E+01_JPRB, 0.12602E+01_JPRB, 0.13464E+01_JPRB, 0.13632E+01_JPRB, 0.14149E+01_JPRB, 0.16165E+01_JPRB ,&
&  0.17446E+01_JPRB, 0.18620E+01_JPRB, 0.19244E+01_JPRB, 0.20753E+01_JPRB, 0.23396E+01_JPRB, 0.23430E+01_JPRB, 0.23427E+01_JPRB ,&
&  0.25264E+01_JPRB, 0.26161E+01_JPRB, 0.32843E+01_JPRB, 0.35774E+01_JPRB, 0.37150E+01_JPRB, 0.39722E+01_JPRB, 0.39714E+01_JPRB ,&
&  0.39970E+01_JPRB, 0.39700E+01_JPRB, 0.38976E+01_JPRB, 0.35388E+01_JPRB, 0.29966E+01_JPRB, 0.25858E+01_JPRB, 0.20246E+01_JPRB ,&
&  0.16606E+01_JPRB, 0.13439E+01_JPRB, 0.10407E+01_JPRB, 0.91640E+00_JPRB, 0.80553E+00_JPRB, 0.78173E+00_JPRB, 0.74774E+00_JPRB ,&
&  0.70292E+00_JPRB, 0.70993E+00_JPRB, 0.74889E+00_JPRB, 0.84016E+00_JPRB, 0.85356E+00_JPRB, 0.79164E+00_JPRB, 0.84960E+00_JPRB ,&
&  0.96947E+00_JPRB, 0.13195E+01_JPRB, 0.14530E+01_JPRB, 0.15925E+01_JPRB, 0.15261E+01_JPRB, 0.15051E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.12498E+01_JPRB, 0.13281E+01_JPRB, 0.12249E+01_JPRB, 0.11504E+01_JPRB, 0.11132E+01_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.11325E+01_JPRB, 0.11038E+01_JPRB, 0.10907E+01_JPRB, 0.10915E+01_JPRB, 0.10878E+01_JPRB, 0.10498E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10118E+01_JPRB, 0.10021E+01_JPRB, 0.99643E+00_JPRB, 0.97892E+00_JPRB, 0.98495E+00_JPRB, 0.10137E+01_JPRB ,&
&  0.91478E+00_JPRB, 0.89533E+00_JPRB, 0.80980E+00_JPRB, 0.79753E+00_JPRB, 0.90492E+00_JPRB, 0.86388E+00_JPRB, 0.85088E+00_JPRB ,&
&  0.81091E+00_JPRB, 0.78353E+00_JPRB, 0.74883E+00_JPRB, 0.74956E+00_JPRB, 0.76756E+00_JPRB, 0.79113E+00_JPRB, 0.80158E+00_JPRB ,&
&  0.75941E+00_JPRB, 0.71884E+00_JPRB, 0.71150E+00_JPRB, 0.72522E+00_JPRB, 0.77785E+00_JPRB, 0.86077E+00_JPRB, 0.96223E+00_JPRB ,&
&  0.10041E+01_JPRB, 0.10218E+01_JPRB, 0.10522E+01_JPRB, 0.10883E+01_JPRB, 0.11306E+01_JPRB, 0.11889E+01_JPRB, 0.12259E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12759E+01_JPRB, 0.13232E+01_JPRB, 0.14059E+01_JPRB, 0.13709E+01_JPRB, 0.14593E+01_JPRB, 0.18114E+01_JPRB ,&
&  0.20934E+01_JPRB, 0.20142E+01_JPRB, 0.21541E+01_JPRB, 0.28631E+01_JPRB, 0.32258E+01_JPRB, 0.32669E+01_JPRB, 0.31706E+01_JPRB ,&
&  0.32595E+01_JPRB, 0.34103E+01_JPRB, 0.36205E+01_JPRB, 0.39034E+01_JPRB, 0.40783E+01_JPRB, 0.42476E+01_JPRB, 0.41917E+01_JPRB ,&
&  0.41078E+01_JPRB, 0.40571E+01_JPRB, 0.38935E+01_JPRB, 0.35610E+01_JPRB, 0.29221E+01_JPRB, 0.24534E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.16136E+01_JPRB, 0.12441E+01_JPRB, 0.88677E+00_JPRB, 0.75354E+00_JPRB, 0.70563E+00_JPRB, 0.76075E+00_JPRB, 0.97432E+00_JPRB ,&
&  0.76869E+00_JPRB, 0.84182E+00_JPRB, 0.95031E+00_JPRB, 0.11161E+01_JPRB, 0.12238E+01_JPRB, 0.12228E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.13222E+01_JPRB, 0.17527E+01_JPRB, 0.19113E+01_JPRB, 0.17606E+01_JPRB, 0.17421E+01_JPRB, 0.16168E+01_JPRB ,&
&  0.16809E+01_JPRB, 0.15011E+01_JPRB, 0.13425E+01_JPRB, 0.12245E+01_JPRB, 0.11832E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.11489E+01_JPRB, 0.11440E+01_JPRB, 0.11546E+01_JPRB, 0.11780E+01_JPRB, 0.11922E+01_JPRB, 0.11358E+01_JPRB, 0.11196E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.10737E+01_JPRB, 0.10517E+01_JPRB, 0.10427E+01_JPRB, 0.10283E+01_JPRB, 0.10194E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.10552E+01_JPRB, 0.94867E+00_JPRB, 0.82106E+00_JPRB, 0.85869E+00_JPRB, 0.84797E+00_JPRB, 0.85581E+00_JPRB ,&
&  0.83728E+00_JPRB, 0.84069E+00_JPRB, 0.81071E+00_JPRB, 0.78848E+00_JPRB, 0.81139E+00_JPRB, 0.84670E+00_JPRB, 0.85826E+00_JPRB ,&
&  0.83129E+00_JPRB, 0.78350E+00_JPRB, 0.76379E+00_JPRB, 0.78842E+00_JPRB, 0.87712E+00_JPRB, 0.87029E+00_JPRB, 0.97891E+00_JPRB ,&
&  0.10372E+01_JPRB, 0.10794E+01_JPRB, 0.11208E+01_JPRB, 0.11804E+01_JPRB, 0.12421E+01_JPRB, 0.13169E+01_JPRB, 0.13623E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13913E+01_JPRB, 0.13586E+01_JPRB, 0.14297E+01_JPRB, 0.15102E+01_JPRB, 0.16105E+01_JPRB, 0.18235E+01_JPRB ,&
&  0.20282E+01_JPRB, 0.21041E+01_JPRB, 0.24679E+01_JPRB, 0.32798E+01_JPRB, 0.36464E+01_JPRB, 0.36386E+01_JPRB, 0.38146E+01_JPRB ,&
&  0.40591E+01_JPRB, 0.43022E+01_JPRB, 0.45528E+01_JPRB, 0.46078E+01_JPRB, 0.46282E+01_JPRB, 0.44181E+01_JPRB, 0.42580E+01_JPRB ,&
&  0.40678E+01_JPRB, 0.40413E+01_JPRB, 0.39630E+01_JPRB, 0.34459E+01_JPRB, 0.27996E+01_JPRB, 0.22766E+01_JPRB, 0.19911E+01_JPRB ,&
&  0.18991E+01_JPRB, 0.10313E+01_JPRB, 0.74591E+00_JPRB, 0.62580E+00_JPRB, 0.61260E+00_JPRB, 0.65100E+00_JPRB, 0.80114E+00_JPRB ,&
&  0.75228E+00_JPRB, 0.95182E+00_JPRB, 0.13378E+01_JPRB, 0.15638E+01_JPRB, 0.19109E+01_JPRB, 0.23060E+01_JPRB, 0.17739E+01_JPRB ,&
&  0.14713E+01_JPRB, 0.15907E+01_JPRB, 0.21701E+01_JPRB, 0.22946E+01_JPRB, 0.20546E+01_JPRB, 0.20039E+01_JPRB, 0.20230E+01_JPRB ,&
&  0.18673E+01_JPRB, 0.17437E+01_JPRB, 0.14932E+01_JPRB, 0.12738E+01_JPRB, 0.11877E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.12588E+01_JPRB, 0.12488E+01_JPRB, 0.12764E+01_JPRB, 0.13028E+01_JPRB, 0.12924E+01_JPRB, 0.12394E+01_JPRB, 0.12155E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.11345E+01_JPRB, 0.10999E+01_JPRB, 0.10815E+01_JPRB, 0.10669E+01_JPRB, 0.10578E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.10757E+01_JPRB, 0.94870E+00_JPRB, 0.84584E+00_JPRB, 0.75205E+00_JPRB, 0.77731E+00_JPRB, 0.81708E+00_JPRB ,&
&  0.87710E+00_JPRB, 0.91057E+00_JPRB, 0.95041E+00_JPRB, 0.99010E+00_JPRB, 0.10320E+01_JPRB, 0.10005E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.95414E+00_JPRB, 0.87480E+00_JPRB, 0.94599E+00_JPRB, 0.99253E+00_JPRB, 0.96918E+00_JPRB, 0.11236E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.12446E+01_JPRB, 0.12886E+01_JPRB, 0.13393E+01_JPRB, 0.13873E+01_JPRB, 0.13864E+01_JPRB, 0.13563E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.13069E+01_JPRB, 0.13616E+01_JPRB, 0.14100E+01_JPRB, 0.15959E+01_JPRB, 0.17325E+01_JPRB, 0.19522E+01_JPRB ,&
&  0.22244E+01_JPRB, 0.22441E+01_JPRB, 0.23986E+01_JPRB, 0.33294E+01_JPRB, 0.43940E+01_JPRB, 0.44259E+01_JPRB, 0.45282E+01_JPRB ,&
&  0.48226E+01_JPRB, 0.49001E+01_JPRB, 0.48288E+01_JPRB, 0.46037E+01_JPRB, 0.45099E+01_JPRB, 0.44300E+01_JPRB, 0.43433E+01_JPRB ,&
&  0.40987E+01_JPRB, 0.40007E+01_JPRB, 0.38619E+01_JPRB, 0.34690E+01_JPRB, 0.28771E+01_JPRB, 0.25469E+01_JPRB, 0.22450E+01_JPRB ,&
&  0.19630E+01_JPRB, 0.13002E+01_JPRB, 0.86714E+00_JPRB, 0.72707E+00_JPRB, 0.69438E+00_JPRB, 0.71362E+00_JPRB, 0.86247E+00_JPRB ,&
&  0.91101E+00_JPRB, 0.12248E+01_JPRB, 0.17590E+01_JPRB, 0.20327E+01_JPRB, 0.31304E+01_JPRB, 0.38426E+01_JPRB, 0.35144E+01_JPRB ,&
&  0.27585E+01_JPRB, 0.23130E+01_JPRB, 0.27304E+01_JPRB, 0.27297E+01_JPRB, 0.24841E+01_JPRB, 0.22540E+01_JPRB, 0.21951E+01_JPRB ,&
&  0.20459E+01_JPRB, 0.19223E+01_JPRB, 0.17371E+01_JPRB, 0.14691E+01_JPRB, 0.13159E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.14917E+01_JPRB, 0.14248E+01_JPRB, 0.13929E+01_JPRB, 0.13751E+01_JPRB, 0.13035E+01_JPRB, 0.12542E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.11938E+01_JPRB, 0.11549E+01_JPRB, 0.11435E+01_JPRB, 0.11361E+01_JPRB, 0.11311E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10095E+01_JPRB, 0.87510E+00_JPRB, 0.77432E+00_JPRB, 0.73349E+00_JPRB, 0.78554E+00_JPRB, 0.88150E+00_JPRB ,&
&  0.10251E+01_JPRB, 0.11780E+01_JPRB, 0.13304E+01_JPRB, 0.13820E+01_JPRB, 0.14211E+01_JPRB, 0.14056E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.14783E+01_JPRB, 0.12168E+01_JPRB, 0.11824E+01_JPRB, 0.11765E+01_JPRB, 0.12265E+01_JPRB, 0.12538E+01_JPRB, 0.12904E+01_JPRB ,&
&  0.12868E+01_JPRB, 0.13081E+01_JPRB, 0.13277E+01_JPRB, 0.13455E+01_JPRB, 0.13586E+01_JPRB, 0.13568E+01_JPRB, 0.13475E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.13656E+01_JPRB, 0.13724E+01_JPRB, 0.16387E+01_JPRB, 0.22136E+01_JPRB, 0.25409E+01_JPRB, 0.26502E+01_JPRB ,&
&  0.25595E+01_JPRB, 0.26970E+01_JPRB, 0.39901E+01_JPRB, 0.45107E+01_JPRB, 0.48103E+01_JPRB, 0.54404E+01_JPRB, 0.56528E+01_JPRB ,&
&  0.54041E+01_JPRB, 0.48911E+01_JPRB, 0.44294E+01_JPRB, 0.39831E+01_JPRB, 0.46166E+01_JPRB, 0.47087E+01_JPRB, 0.45177E+01_JPRB ,&
&  0.40563E+01_JPRB, 0.39573E+01_JPRB, 0.35913E+01_JPRB, 0.32359E+01_JPRB, 0.29780E+01_JPRB, 0.29863E+01_JPRB, 0.24037E+01_JPRB ,&
&  0.19020E+01_JPRB, 0.18026E+01_JPRB, 0.13630E+01_JPRB, 0.11467E+01_JPRB, 0.10742E+01_JPRB, 0.10848E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.13743E+01_JPRB, 0.17398E+01_JPRB, 0.23108E+01_JPRB, 0.29053E+01_JPRB, 0.42065E+01_JPRB, 0.51359E+01_JPRB, 0.50585E+01_JPRB ,&
&  0.40176E+01_JPRB, 0.38148E+01_JPRB, 0.33900E+01_JPRB, 0.31787E+01_JPRB, 0.29098E+01_JPRB, 0.25728E+01_JPRB, 0.23825E+01_JPRB ,&
&  0.21698E+01_JPRB, 0.20196E+01_JPRB, 0.19016E+01_JPRB, 0.17239E+01_JPRB, 0.16071E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.18010E+01_JPRB, 0.16933E+01_JPRB, 0.15410E+01_JPRB, 0.14311E+01_JPRB, 0.13317E+01_JPRB, 0.12718E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.12039E+01_JPRB, 0.12106E+01_JPRB, 0.12264E+01_JPRB, 0.12226E+01_JPRB, 0.11949E+01_JPRB, 0.11099E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.93042E+00_JPRB, 0.82560E+00_JPRB, 0.81756E+00_JPRB, 0.89709E+00_JPRB, 0.97410E+00_JPRB, 0.10850E+01_JPRB ,&
&  0.13129E+01_JPRB, 0.16641E+01_JPRB, 0.18428E+01_JPRB, 0.20174E+01_JPRB, 0.20815E+01_JPRB, 0.20461E+01_JPRB, 0.19530E+01_JPRB ,&
&  0.17667E+01_JPRB, 0.17162E+01_JPRB, 0.16791E+01_JPRB, 0.15871E+01_JPRB, 0.14929E+01_JPRB, 0.14380E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.13617E+01_JPRB, 0.13316E+01_JPRB, 0.13472E+01_JPRB, 0.13565E+01_JPRB, 0.14075E+01_JPRB, 0.14452E+01_JPRB, 0.14908E+01_JPRB ,&
&  0.14714E+01_JPRB, 0.14696E+01_JPRB, 0.16698E+01_JPRB, 0.23899E+01_JPRB, 0.30380E+01_JPRB, 0.35124E+01_JPRB, 0.33952E+01_JPRB ,&
&  0.37571E+01_JPRB, 0.40919E+01_JPRB, 0.43079E+01_JPRB, 0.48795E+01_JPRB, 0.41576E+01_JPRB, 0.47121E+01_JPRB, 0.54539E+01_JPRB ,&
&  0.52697E+01_JPRB, 0.47715E+01_JPRB, 0.44972E+01_JPRB, 0.42769E+01_JPRB, 0.44750E+01_JPRB, 0.57314E+01_JPRB, 0.51995E+01_JPRB ,&
&  0.44160E+01_JPRB, 0.40229E+01_JPRB, 0.35429E+01_JPRB, 0.34409E+01_JPRB, 0.29664E+01_JPRB, 0.26183E+01_JPRB, 0.18939E+01_JPRB ,&
&  0.19096E+01_JPRB, 0.17236E+01_JPRB, 0.17356E+01_JPRB, 0.15997E+01_JPRB, 0.16494E+01_JPRB, 0.16044E+01_JPRB, 0.16729E+01_JPRB ,&
&  0.20323E+01_JPRB, 0.24681E+01_JPRB, 0.36340E+01_JPRB, 0.45528E+01_JPRB, 0.66705E+01_JPRB, 0.63134E+01_JPRB, 0.62269E+01_JPRB ,&
&  0.53980E+01_JPRB, 0.49288E+01_JPRB, 0.40017E+01_JPRB, 0.38320E+01_JPRB, 0.33986E+01_JPRB, 0.29985E+01_JPRB, 0.26350E+01_JPRB ,&
&  0.23047E+01_JPRB, 0.20870E+01_JPRB, 0.19446E+01_JPRB, 0.18524E+01_JPRB, 0.18449E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.17488E+01_JPRB, 0.16555E+01_JPRB, 0.15498E+01_JPRB, 0.14386E+01_JPRB, 0.13211E+01_JPRB, 0.12551E+01_JPRB, 0.12165E+01_JPRB ,&
&  0.11987E+01_JPRB, 0.12156E+01_JPRB, 0.12317E+01_JPRB, 0.12366E+01_JPRB, 0.12372E+01_JPRB, 0.12306E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.94532E+00_JPRB, 0.83048E+00_JPRB, 0.82645E+00_JPRB, 0.10449E+01_JPRB, 0.12280E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.18327E+01_JPRB, 0.23249E+01_JPRB, 0.24299E+01_JPRB, 0.25916E+01_JPRB, 0.27993E+01_JPRB, 0.28824E+01_JPRB, 0.26321E+01_JPRB ,&
&  0.24752E+01_JPRB, 0.22697E+01_JPRB, 0.19814E+01_JPRB, 0.17558E+01_JPRB, 0.15749E+01_JPRB, 0.14489E+01_JPRB, 0.13491E+01_JPRB ,&
&  0.12899E+01_JPRB, 0.12681E+01_JPRB, 0.12646E+01_JPRB, 0.12874E+01_JPRB, 0.13585E+01_JPRB, 0.13614E+01_JPRB, 0.14482E+01_JPRB ,&
&  0.15492E+01_JPRB, 0.16683E+01_JPRB, 0.21051E+01_JPRB, 0.29792E+01_JPRB, 0.39989E+01_JPRB, 0.45838E+01_JPRB, 0.46356E+01_JPRB ,&
&  0.44148E+01_JPRB, 0.44302E+01_JPRB, 0.46395E+01_JPRB, 0.46701E+01_JPRB, 0.47393E+01_JPRB, 0.49549E+01_JPRB, 0.54418E+01_JPRB ,&
&  0.43326E+01_JPRB, 0.36187E+01_JPRB, 0.31819E+01_JPRB, 0.35687E+01_JPRB, 0.52184E+01_JPRB, 0.66324E+01_JPRB, 0.65070E+01_JPRB ,&
&  0.55551E+01_JPRB, 0.47556E+01_JPRB, 0.40975E+01_JPRB, 0.35658E+01_JPRB, 0.35117E+01_JPRB, 0.24422E+01_JPRB, 0.26189E+01_JPRB ,&
&  0.26651E+01_JPRB, 0.26852E+01_JPRB, 0.24585E+01_JPRB, 0.20865E+01_JPRB, 0.20083E+01_JPRB, 0.23252E+01_JPRB, 0.26945E+01_JPRB ,&
&  0.33271E+01_JPRB, 0.39686E+01_JPRB, 0.66348E+01_JPRB, 0.76855E+01_JPRB, 0.85958E+01_JPRB, 0.73269E+01_JPRB, 0.65428E+01_JPRB ,&
&  0.52540E+01_JPRB, 0.48855E+01_JPRB, 0.47066E+01_JPRB, 0.42878E+01_JPRB, 0.37881E+01_JPRB, 0.33976E+01_JPRB, 0.29068E+01_JPRB ,&
&  0.25182E+01_JPRB, 0.22129E+01_JPRB, 0.19882E+01_JPRB, 0.18467E+01_JPRB, 0.18009E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.17243E+01_JPRB, 0.16488E+01_JPRB, 0.15348E+01_JPRB, 0.13673E+01_JPRB, 0.12710E+01_JPRB, 0.12364E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.12713E+01_JPRB, 0.12580E+01_JPRB, 0.12395E+01_JPRB, 0.12449E+01_JPRB, 0.12230E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.10196E+01_JPRB, 0.88634E+00_JPRB, 0.93840E+00_JPRB, 0.12639E+01_JPRB, 0.15246E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.26642E+01_JPRB, 0.28044E+01_JPRB, 0.24977E+01_JPRB, 0.26023E+01_JPRB, 0.33256E+01_JPRB, 0.31330E+01_JPRB, 0.28353E+01_JPRB ,&
&  0.26603E+01_JPRB, 0.23171E+01_JPRB, 0.19674E+01_JPRB, 0.17010E+01_JPRB, 0.15159E+01_JPRB, 0.13983E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12448E+01_JPRB, 0.12765E+01_JPRB, 0.13403E+01_JPRB, 0.13534E+01_JPRB, 0.13788E+01_JPRB, 0.15201E+01_JPRB ,&
&  0.16683E+01_JPRB, 0.18376E+01_JPRB, 0.23054E+01_JPRB, 0.31275E+01_JPRB, 0.42644E+01_JPRB, 0.51775E+01_JPRB, 0.62697E+01_JPRB ,&
&  0.55767E+01_JPRB, 0.56452E+01_JPRB, 0.51701E+01_JPRB, 0.57276E+01_JPRB, 0.53001E+01_JPRB, 0.52908E+01_JPRB, 0.53615E+01_JPRB ,&
&  0.39734E+01_JPRB, 0.34113E+01_JPRB, 0.37522E+01_JPRB, 0.51628E+01_JPRB, 0.54174E+01_JPRB, 0.60354E+01_JPRB, 0.73752E+01_JPRB ,&
&  0.58915E+01_JPRB, 0.50479E+01_JPRB, 0.46033E+01_JPRB, 0.45582E+01_JPRB, 0.29519E+01_JPRB, 0.23401E+01_JPRB, 0.35227E+01_JPRB ,&
&  0.33056E+01_JPRB, 0.19796E+01_JPRB, 0.15325E+01_JPRB, 0.18045E+01_JPRB, 0.14960E+01_JPRB, 0.19457E+01_JPRB, 0.33466E+01_JPRB ,&
&  0.42616E+01_JPRB, 0.52472E+01_JPRB, 0.85450E+01_JPRB, 0.77705E+01_JPRB, 0.79341E+01_JPRB, 0.79549E+01_JPRB, 0.63288E+01_JPRB ,&
&  0.55783E+01_JPRB, 0.44044E+01_JPRB, 0.44152E+01_JPRB, 0.39582E+01_JPRB, 0.36708E+01_JPRB, 0.34673E+01_JPRB, 0.30768E+01_JPRB ,&
&  0.26551E+01_JPRB, 0.22988E+01_JPRB, 0.20269E+01_JPRB, 0.18578E+01_JPRB, 0.17834E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.16269E+01_JPRB, 0.15345E+01_JPRB, 0.14819E+01_JPRB, 0.13847E+01_JPRB, 0.13097E+01_JPRB, 0.12894E+01_JPRB, 0.13281E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.12654E+01_JPRB, 0.12073E+01_JPRB, 0.11927E+01_JPRB, 0.12094E+01_JPRB, 0.12008E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.10339E+01_JPRB, 0.10430E+01_JPRB, 0.11978E+01_JPRB, 0.17695E+01_JPRB, 0.18772E+01_JPRB, 0.25512E+01_JPRB ,&
&  0.34289E+01_JPRB, 0.29598E+01_JPRB, 0.25445E+01_JPRB, 0.26306E+01_JPRB, 0.26794E+01_JPRB, 0.25724E+01_JPRB, 0.25889E+01_JPRB ,&
&  0.23152E+01_JPRB, 0.20367E+01_JPRB, 0.18319E+01_JPRB, 0.16760E+01_JPRB, 0.15004E+01_JPRB, 0.13622E+01_JPRB, 0.12867E+01_JPRB ,&
&  0.12738E+01_JPRB, 0.13016E+01_JPRB, 0.13907E+01_JPRB, 0.13624E+01_JPRB, 0.13900E+01_JPRB, 0.14515E+01_JPRB, 0.16104E+01_JPRB ,&
&  0.17886E+01_JPRB, 0.20001E+01_JPRB, 0.24062E+01_JPRB, 0.32765E+01_JPRB, 0.50772E+01_JPRB, 0.64564E+01_JPRB, 0.63473E+01_JPRB ,&
&  0.57354E+01_JPRB, 0.55361E+01_JPRB, 0.58536E+01_JPRB, 0.59362E+01_JPRB, 0.52666E+01_JPRB, 0.53495E+01_JPRB, 0.51502E+01_JPRB ,&
&  0.45438E+01_JPRB, 0.42233E+01_JPRB, 0.41793E+01_JPRB, 0.54845E+01_JPRB, 0.62770E+01_JPRB, 0.62434E+01_JPRB, 0.70520E+01_JPRB ,&
&  0.64500E+01_JPRB, 0.53493E+01_JPRB, 0.41662E+01_JPRB, 0.34270E+01_JPRB, 0.34514E+01_JPRB, 0.33011E+01_JPRB, 0.24702E+01_JPRB ,&
&  0.20724E+01_JPRB, 0.17270E+01_JPRB, 0.14840E+01_JPRB, 0.14946E+01_JPRB, 0.13502E+01_JPRB, 0.18633E+01_JPRB, 0.30953E+01_JPRB ,&
&  0.50336E+01_JPRB, 0.61176E+01_JPRB, 0.76821E+01_JPRB, 0.73192E+01_JPRB, 0.72508E+01_JPRB, 0.83122E+01_JPRB, 0.77430E+01_JPRB ,&
&  0.49041E+01_JPRB, 0.40285E+01_JPRB, 0.40240E+01_JPRB, 0.37773E+01_JPRB, 0.35505E+01_JPRB, 0.33336E+01_JPRB, 0.29686E+01_JPRB ,&
&  0.25344E+01_JPRB, 0.21386E+01_JPRB, 0.19345E+01_JPRB, 0.18555E+01_JPRB, 0.17650E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.13771E+01_JPRB, 0.13563E+01_JPRB, 0.13766E+01_JPRB, 0.13576E+01_JPRB, 0.13250E+01_JPRB, 0.13380E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.11736E+01_JPRB, 0.11507E+01_JPRB, 0.11727E+01_JPRB, 0.11832E+01_JPRB, 0.11832E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.10683E+01_JPRB, 0.11458E+01_JPRB, 0.15285E+01_JPRB, 0.25299E+01_JPRB, 0.25178E+01_JPRB, 0.36517E+01_JPRB ,&
&  0.39538E+01_JPRB, 0.29253E+01_JPRB, 0.25551E+01_JPRB, 0.23952E+01_JPRB, 0.24270E+01_JPRB, 0.22235E+01_JPRB, 0.20413E+01_JPRB ,&
&  0.17228E+01_JPRB, 0.16956E+01_JPRB, 0.15918E+01_JPRB, 0.14941E+01_JPRB, 0.13848E+01_JPRB, 0.13199E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.14477E+01_JPRB, 0.14215E+01_JPRB, 0.14255E+01_JPRB, 0.14992E+01_JPRB, 0.15800E+01_JPRB, 0.17337E+01_JPRB ,&
&  0.19274E+01_JPRB, 0.21585E+01_JPRB, 0.25890E+01_JPRB, 0.39605E+01_JPRB, 0.52045E+01_JPRB, 0.55254E+01_JPRB, 0.61175E+01_JPRB ,&
&  0.63129E+01_JPRB, 0.64526E+01_JPRB, 0.56862E+01_JPRB, 0.62186E+01_JPRB, 0.53104E+01_JPRB, 0.50652E+01_JPRB, 0.49563E+01_JPRB ,&
&  0.45256E+01_JPRB, 0.47956E+01_JPRB, 0.45813E+01_JPRB, 0.48761E+01_JPRB, 0.63621E+01_JPRB, 0.70875E+01_JPRB, 0.69421E+01_JPRB ,&
&  0.67418E+01_JPRB, 0.57030E+01_JPRB, 0.46729E+01_JPRB, 0.42964E+01_JPRB, 0.56504E+01_JPRB, 0.67761E+01_JPRB, 0.31853E+01_JPRB ,&
&  0.21732E+01_JPRB, 0.20038E+01_JPRB, 0.15773E+01_JPRB, 0.13744E+01_JPRB, 0.14225E+01_JPRB, 0.19508E+01_JPRB, 0.38153E+01_JPRB ,&
&  0.44455E+01_JPRB, 0.72438E+01_JPRB, 0.78048E+01_JPRB, 0.81112E+01_JPRB, 0.76362E+01_JPRB, 0.73561E+01_JPRB, 0.57223E+01_JPRB ,&
&  0.56856E+01_JPRB, 0.49807E+01_JPRB, 0.42021E+01_JPRB, 0.37691E+01_JPRB, 0.33267E+01_JPRB, 0.29088E+01_JPRB, 0.26042E+01_JPRB ,&
&  0.23568E+01_JPRB, 0.20578E+01_JPRB, 0.18694E+01_JPRB, 0.17315E+01_JPRB, 0.15271E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.11577E+01_JPRB, 0.12033E+01_JPRB, 0.12762E+01_JPRB, 0.12951E+01_JPRB, 0.12801E+01_JPRB, 0.12995E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.11367E+01_JPRB, 0.11477E+01_JPRB, 0.11841E+01_JPRB, 0.12351E+01_JPRB, 0.12718E+01_JPRB, 0.12260E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11501E+01_JPRB, 0.13002E+01_JPRB, 0.22079E+01_JPRB, 0.31789E+01_JPRB, 0.41071E+01_JPRB, 0.47411E+01_JPRB ,&
&  0.43118E+01_JPRB, 0.30690E+01_JPRB, 0.25651E+01_JPRB, 0.24719E+01_JPRB, 0.23502E+01_JPRB, 0.22475E+01_JPRB, 0.18017E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.15132E+01_JPRB, 0.14010E+01_JPRB, 0.13710E+01_JPRB, 0.13439E+01_JPRB, 0.13478E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.15001E+01_JPRB, 0.14921E+01_JPRB, 0.15278E+01_JPRB, 0.15951E+01_JPRB, 0.16899E+01_JPRB, 0.17606E+01_JPRB, 0.18855E+01_JPRB ,&
&  0.20956E+01_JPRB, 0.24076E+01_JPRB, 0.30451E+01_JPRB, 0.45415E+01_JPRB, 0.46841E+01_JPRB, 0.59575E+01_JPRB, 0.61487E+01_JPRB ,&
&  0.61264E+01_JPRB, 0.60290E+01_JPRB, 0.54475E+01_JPRB, 0.56233E+01_JPRB, 0.52160E+01_JPRB, 0.49739E+01_JPRB, 0.47054E+01_JPRB ,&
&  0.48952E+01_JPRB, 0.46844E+01_JPRB, 0.54760E+01_JPRB, 0.53427E+01_JPRB, 0.59285E+01_JPRB, 0.90034E+01_JPRB, 0.70629E+01_JPRB ,&
&  0.60205E+01_JPRB, 0.62833E+01_JPRB, 0.53848E+01_JPRB, 0.53501E+01_JPRB, 0.68226E+01_JPRB, 0.87871E+01_JPRB, 0.75927E+01_JPRB ,&
&  0.28795E+01_JPRB, 0.19650E+01_JPRB, 0.17993E+01_JPRB, 0.16136E+01_JPRB, 0.14046E+01_JPRB, 0.19408E+01_JPRB, 0.75175E+01_JPRB ,&
&  0.53883E+01_JPRB, 0.75011E+01_JPRB, 0.76401E+01_JPRB, 0.75271E+01_JPRB, 0.75755E+01_JPRB, 0.66572E+01_JPRB, 0.57608E+01_JPRB ,&
&  0.47008E+01_JPRB, 0.38296E+01_JPRB, 0.31853E+01_JPRB, 0.27050E+01_JPRB, 0.23927E+01_JPRB, 0.22292E+01_JPRB, 0.21827E+01_JPRB ,&
&  0.20945E+01_JPRB, 0.18740E+01_JPRB, 0.16654E+01_JPRB, 0.14066E+01_JPRB, 0.11938E+01_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.11809E+01_JPRB, 0.12089E+01_JPRB, 0.12508E+01_JPRB, 0.13091E+01_JPRB, 0.13488E+01_JPRB, 0.13304E+01_JPRB, 0.12680E+01_JPRB ,&
&  0.12650E+01_JPRB, 0.13019E+01_JPRB, 0.13855E+01_JPRB, 0.14408E+01_JPRB, 0.15034E+01_JPRB, 0.14881E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.13249E+01_JPRB, 0.13669E+01_JPRB, 0.18514E+01_JPRB, 0.29539E+01_JPRB, 0.36785E+01_JPRB, 0.43596E+01_JPRB, 0.52955E+01_JPRB ,&
&  0.51724E+01_JPRB, 0.32643E+01_JPRB, 0.24151E+01_JPRB, 0.24524E+01_JPRB, 0.24557E+01_JPRB, 0.25283E+01_JPRB, 0.18952E+01_JPRB ,&
&  0.15919E+01_JPRB, 0.14832E+01_JPRB, 0.13854E+01_JPRB, 0.13872E+01_JPRB, 0.14065E+01_JPRB, 0.14406E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.15633E+01_JPRB, 0.16602E+01_JPRB, 0.17737E+01_JPRB, 0.18694E+01_JPRB, 0.19392E+01_JPRB, 0.19929E+01_JPRB, 0.21101E+01_JPRB ,&
&  0.23583E+01_JPRB, 0.27625E+01_JPRB, 0.34552E+01_JPRB, 0.44047E+01_JPRB, 0.52116E+01_JPRB, 0.54976E+01_JPRB, 0.58551E+01_JPRB ,&
&  0.58438E+01_JPRB, 0.58203E+01_JPRB, 0.55545E+01_JPRB, 0.51973E+01_JPRB, 0.48470E+01_JPRB, 0.48971E+01_JPRB, 0.46836E+01_JPRB ,&
&  0.50557E+01_JPRB, 0.54642E+01_JPRB, 0.61455E+01_JPRB, 0.59981E+01_JPRB, 0.62851E+01_JPRB, 0.85609E+01_JPRB, 0.89746E+01_JPRB ,&
&  0.74344E+01_JPRB, 0.57450E+01_JPRB, 0.53532E+01_JPRB, 0.60940E+01_JPRB, 0.70230E+01_JPRB, 0.80978E+01_JPRB, 0.12075E+02_JPRB ,&
&  0.99381E+01_JPRB, 0.43257E+01_JPRB, 0.24586E+01_JPRB, 0.25570E+01_JPRB, 0.14883E+01_JPRB, 0.14208E+01_JPRB, 0.31048E+01_JPRB ,&
&  0.43316E+01_JPRB, 0.54556E+01_JPRB, 0.56250E+01_JPRB, 0.57809E+01_JPRB, 0.58001E+01_JPRB, 0.47430E+01_JPRB, 0.35425E+01_JPRB ,&
&  0.26027E+01_JPRB, 0.20833E+01_JPRB, 0.18067E+01_JPRB, 0.16798E+01_JPRB, 0.17189E+01_JPRB, 0.18104E+01_JPRB, 0.18823E+01_JPRB ,&
&  0.18458E+01_JPRB, 0.16536E+01_JPRB, 0.13750E+01_JPRB, 0.11673E+01_JPRB, 0.11295E+01_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.13784E+01_JPRB, 0.13743E+01_JPRB, 0.13982E+01_JPRB, 0.14596E+01_JPRB, 0.14891E+01_JPRB, 0.15104E+01_JPRB, 0.16402E+01_JPRB ,&
&  0.17505E+01_JPRB, 0.18283E+01_JPRB, 0.18805E+01_JPRB, 0.19165E+01_JPRB, 0.18942E+01_JPRB, 0.17744E+01_JPRB, 0.16792E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.21002E+01_JPRB, 0.29478E+01_JPRB, 0.41086E+01_JPRB, 0.46702E+01_JPRB, 0.61439E+01_JPRB, 0.50813E+01_JPRB ,&
&  0.49937E+01_JPRB, 0.41074E+01_JPRB, 0.26828E+01_JPRB, 0.27822E+01_JPRB, 0.27825E+01_JPRB, 0.28930E+01_JPRB, 0.20320E+01_JPRB ,&
&  0.16920E+01_JPRB, 0.15818E+01_JPRB, 0.14835E+01_JPRB, 0.14856E+01_JPRB, 0.15224E+01_JPRB, 0.15951E+01_JPRB, 0.16733E+01_JPRB ,&
&  0.18030E+01_JPRB, 0.19756E+01_JPRB, 0.21200E+01_JPRB, 0.21906E+01_JPRB, 0.22285E+01_JPRB, 0.22756E+01_JPRB, 0.24429E+01_JPRB ,&
&  0.27155E+01_JPRB, 0.31493E+01_JPRB, 0.38454E+01_JPRB, 0.45808E+01_JPRB, 0.50717E+01_JPRB, 0.50821E+01_JPRB, 0.53601E+01_JPRB ,&
&  0.51698E+01_JPRB, 0.48693E+01_JPRB, 0.49813E+01_JPRB, 0.49013E+01_JPRB, 0.46864E+01_JPRB, 0.46028E+01_JPRB, 0.44557E+01_JPRB ,&
&  0.53528E+01_JPRB, 0.67385E+01_JPRB, 0.83507E+01_JPRB, 0.73597E+01_JPRB, 0.72857E+01_JPRB, 0.87463E+01_JPRB, 0.91635E+01_JPRB ,&
&  0.82463E+01_JPRB, 0.66007E+01_JPRB, 0.62733E+01_JPRB, 0.59716E+01_JPRB, 0.62398E+01_JPRB, 0.69556E+01_JPRB, 0.88942E+01_JPRB ,&
&  0.97317E+01_JPRB, 0.98855E+01_JPRB, 0.51150E+01_JPRB, 0.21749E+01_JPRB, 0.15335E+01_JPRB, 0.17629E+01_JPRB, 0.23207E+01_JPRB ,&
&  0.31374E+01_JPRB, 0.35333E+01_JPRB, 0.36729E+01_JPRB, 0.46552E+01_JPRB, 0.41057E+01_JPRB, 0.31762E+01_JPRB, 0.24195E+01_JPRB ,&
&  0.19211E+01_JPRB, 0.15585E+01_JPRB, 0.13801E+01_JPRB, 0.13759E+01_JPRB, 0.15123E+01_JPRB, 0.16439E+01_JPRB, 0.16997E+01_JPRB ,&
&  0.15685E+01_JPRB, 0.13471E+01_JPRB, 0.12493E+01_JPRB, 0.12505E+01_JPRB, 0.13145E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.16267E+01_JPRB, 0.16780E+01_JPRB, 0.17632E+01_JPRB, 0.18836E+01_JPRB, 0.20019E+01_JPRB, 0.21520E+01_JPRB, 0.23332E+01_JPRB ,&
&  0.24519E+01_JPRB, 0.24689E+01_JPRB, 0.24494E+01_JPRB, 0.23797E+01_JPRB, 0.23416E+01_JPRB, 0.23361E+01_JPRB, 0.25103E+01_JPRB ,&
&  0.30263E+01_JPRB, 0.38379E+01_JPRB, 0.46753E+01_JPRB, 0.52633E+01_JPRB, 0.55253E+01_JPRB, 0.63346E+01_JPRB, 0.64511E+01_JPRB ,&
&  0.46784E+01_JPRB, 0.54189E+01_JPRB, 0.30507E+01_JPRB, 0.32279E+01_JPRB, 0.34094E+01_JPRB, 0.37924E+01_JPRB, 0.26669E+01_JPRB ,&
&  0.19590E+01_JPRB, 0.18250E+01_JPRB, 0.17595E+01_JPRB, 0.17615E+01_JPRB, 0.17699E+01_JPRB, 0.18813E+01_JPRB, 0.20414E+01_JPRB ,&
&  0.22036E+01_JPRB, 0.23272E+01_JPRB, 0.24163E+01_JPRB, 0.24642E+01_JPRB, 0.25229E+01_JPRB, 0.26085E+01_JPRB, 0.28532E+01_JPRB ,&
&  0.31727E+01_JPRB, 0.36507E+01_JPRB, 0.42151E+01_JPRB, 0.48040E+01_JPRB, 0.47790E+01_JPRB, 0.47342E+01_JPRB, 0.48942E+01_JPRB ,&
&  0.47758E+01_JPRB, 0.45124E+01_JPRB, 0.45978E+01_JPRB, 0.44093E+01_JPRB, 0.42110E+01_JPRB, 0.44643E+01_JPRB, 0.45283E+01_JPRB ,&
&  0.60543E+01_JPRB, 0.70925E+01_JPRB, 0.94018E+01_JPRB, 0.86982E+01_JPRB, 0.77467E+01_JPRB, 0.81886E+01_JPRB, 0.74238E+01_JPRB ,&
&  0.62251E+01_JPRB, 0.59097E+01_JPRB, 0.56898E+01_JPRB, 0.53845E+01_JPRB, 0.54799E+01_JPRB, 0.60018E+01_JPRB, 0.69805E+01_JPRB ,&
&  0.74746E+01_JPRB, 0.70988E+01_JPRB, 0.48381E+01_JPRB, 0.22503E+01_JPRB, 0.16966E+01_JPRB, 0.17091E+01_JPRB, 0.20878E+01_JPRB ,&
&  0.28115E+01_JPRB, 0.30236E+01_JPRB, 0.30020E+01_JPRB, 0.28839E+01_JPRB, 0.23107E+01_JPRB, 0.18270E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.13880E+01_JPRB, 0.13334E+01_JPRB, 0.13189E+01_JPRB, 0.14257E+01_JPRB, 0.15274E+01_JPRB, 0.15509E+01_JPRB, 0.13999E+01_JPRB ,&
&  0.13020E+01_JPRB, 0.12953E+01_JPRB, 0.13908E+01_JPRB, 0.14883E+01_JPRB, 0.15704E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.18886E+01_JPRB, 0.20086E+01_JPRB, 0.21692E+01_JPRB, 0.23988E+01_JPRB, 0.27045E+01_JPRB, 0.30579E+01_JPRB, 0.31338E+01_JPRB ,&
&  0.31347E+01_JPRB, 0.30872E+01_JPRB, 0.32176E+01_JPRB, 0.33581E+01_JPRB, 0.35304E+01_JPRB, 0.37974E+01_JPRB, 0.41969E+01_JPRB ,&
&  0.46985E+01_JPRB, 0.51986E+01_JPRB, 0.56788E+01_JPRB, 0.61630E+01_JPRB, 0.66652E+01_JPRB, 0.76392E+01_JPRB, 0.80568E+01_JPRB ,&
&  0.74081E+01_JPRB, 0.43699E+01_JPRB, 0.37065E+01_JPRB, 0.32841E+01_JPRB, 0.33730E+01_JPRB, 0.37813E+01_JPRB, 0.34728E+01_JPRB ,&
&  0.26858E+01_JPRB, 0.23583E+01_JPRB, 0.22458E+01_JPRB, 0.21942E+01_JPRB, 0.21082E+01_JPRB, 0.22505E+01_JPRB, 0.23866E+01_JPRB ,&
&  0.24876E+01_JPRB, 0.25515E+01_JPRB, 0.26249E+01_JPRB, 0.26814E+01_JPRB, 0.27616E+01_JPRB, 0.29216E+01_JPRB, 0.32279E+01_JPRB ,&
&  0.35474E+01_JPRB, 0.39402E+01_JPRB, 0.43394E+01_JPRB, 0.45381E+01_JPRB, 0.44696E+01_JPRB, 0.44177E+01_JPRB, 0.44576E+01_JPRB ,&
&  0.44493E+01_JPRB, 0.44683E+01_JPRB, 0.44130E+01_JPRB, 0.42842E+01_JPRB, 0.41006E+01_JPRB, 0.46889E+01_JPRB, 0.52978E+01_JPRB ,&
&  0.66286E+01_JPRB, 0.75088E+01_JPRB, 0.94621E+01_JPRB, 0.83473E+01_JPRB, 0.73981E+01_JPRB, 0.68724E+01_JPRB, 0.61518E+01_JPRB ,&
&  0.51882E+01_JPRB, 0.50273E+01_JPRB, 0.46816E+01_JPRB, 0.40412E+01_JPRB, 0.38454E+01_JPRB, 0.39605E+01_JPRB, 0.51875E+01_JPRB ,&
&  0.57741E+01_JPRB, 0.56840E+01_JPRB, 0.33751E+01_JPRB, 0.21424E+01_JPRB, 0.17652E+01_JPRB, 0.17261E+01_JPRB, 0.21204E+01_JPRB ,&
&  0.27558E+01_JPRB, 0.26382E+01_JPRB, 0.22692E+01_JPRB, 0.21577E+01_JPRB, 0.17806E+01_JPRB, 0.14323E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.12222E+01_JPRB, 0.12102E+01_JPRB, 0.12399E+01_JPRB, 0.13245E+01_JPRB, 0.13529E+01_JPRB, 0.14067E+01_JPRB, 0.13925E+01_JPRB ,&
&  0.14627E+01_JPRB, 0.15491E+01_JPRB, 0.16341E+01_JPRB, 0.17165E+01_JPRB, 0.17891E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.19803E+01_JPRB, 0.21278E+01_JPRB, 0.23069E+01_JPRB, 0.25185E+01_JPRB, 0.27521E+01_JPRB, 0.29601E+01_JPRB, 0.31189E+01_JPRB ,&
&  0.33426E+01_JPRB, 0.35881E+01_JPRB, 0.38430E+01_JPRB, 0.40573E+01_JPRB, 0.42286E+01_JPRB, 0.44423E+01_JPRB, 0.46831E+01_JPRB ,&
&  0.49093E+01_JPRB, 0.51442E+01_JPRB, 0.54337E+01_JPRB, 0.57889E+01_JPRB, 0.62662E+01_JPRB, 0.69040E+01_JPRB, 0.72629E+01_JPRB ,&
&  0.59201E+01_JPRB, 0.35201E+01_JPRB, 0.38170E+01_JPRB, 0.34526E+01_JPRB, 0.33978E+01_JPRB, 0.34686E+01_JPRB, 0.32032E+01_JPRB ,&
&  0.28410E+01_JPRB, 0.27169E+01_JPRB, 0.27273E+01_JPRB, 0.26223E+01_JPRB, 0.24294E+01_JPRB, 0.25090E+01_JPRB, 0.25850E+01_JPRB ,&
&  0.26510E+01_JPRB, 0.27130E+01_JPRB, 0.27702E+01_JPRB, 0.28089E+01_JPRB, 0.29095E+01_JPRB, 0.30927E+01_JPRB, 0.33772E+01_JPRB ,&
&  0.36967E+01_JPRB, 0.39610E+01_JPRB, 0.41932E+01_JPRB, 0.43419E+01_JPRB, 0.42891E+01_JPRB, 0.42465E+01_JPRB, 0.43026E+01_JPRB ,&
&  0.43933E+01_JPRB, 0.43843E+01_JPRB, 0.45266E+01_JPRB, 0.48198E+01_JPRB, 0.51996E+01_JPRB, 0.52683E+01_JPRB, 0.62992E+01_JPRB ,&
&  0.72303E+01_JPRB, 0.76342E+01_JPRB, 0.82113E+01_JPRB, 0.96036E+01_JPRB, 0.63317E+01_JPRB, 0.57601E+01_JPRB, 0.49135E+01_JPRB ,&
&  0.49509E+01_JPRB, 0.45821E+01_JPRB, 0.39058E+01_JPRB, 0.29728E+01_JPRB, 0.26283E+01_JPRB, 0.25396E+01_JPRB, 0.33401E+01_JPRB ,&
&  0.43507E+01_JPRB, 0.42420E+01_JPRB, 0.28998E+01_JPRB, 0.22554E+01_JPRB, 0.20338E+01_JPRB, 0.23176E+01_JPRB, 0.27046E+01_JPRB ,&
&  0.28060E+01_JPRB, 0.22975E+01_JPRB, 0.20789E+01_JPRB, 0.23046E+01_JPRB, 0.17190E+01_JPRB, 0.11729E+01_JPRB, 0.94750E+00_JPRB ,&
&  0.94406E+00_JPRB, 0.99284E+00_JPRB, 0.10779E+01_JPRB, 0.11948E+01_JPRB, 0.12528E+01_JPRB, 0.13247E+01_JPRB, 0.13735E+01_JPRB ,&
&  0.15149E+01_JPRB, 0.16191E+01_JPRB, 0.17085E+01_JPRB, 0.17700E+01_JPRB, 0.18686E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.18355E+01_JPRB, 0.19743E+01_JPRB, 0.20977E+01_JPRB, 0.22301E+01_JPRB, 0.23516E+01_JPRB, 0.24675E+01_JPRB, 0.26315E+01_JPRB ,&
&  0.28225E+01_JPRB, 0.30440E+01_JPRB, 0.32557E+01_JPRB, 0.34220E+01_JPRB, 0.36030E+01_JPRB, 0.37274E+01_JPRB, 0.38379E+01_JPRB ,&
&  0.38857E+01_JPRB, 0.39518E+01_JPRB, 0.40004E+01_JPRB, 0.40942E+01_JPRB, 0.42491E+01_JPRB, 0.44289E+01_JPRB, 0.46028E+01_JPRB ,&
&  0.46762E+01_JPRB, 0.43980E+01_JPRB, 0.36361E+01_JPRB, 0.25959E+01_JPRB, 0.23604E+01_JPRB, 0.27023E+01_JPRB, 0.26977E+01_JPRB ,&
&  0.26622E+01_JPRB, 0.25826E+01_JPRB, 0.25084E+01_JPRB, 0.25472E+01_JPRB, 0.25606E+01_JPRB, 0.25919E+01_JPRB, 0.26130E+01_JPRB ,&
&  0.26475E+01_JPRB, 0.27093E+01_JPRB, 0.27388E+01_JPRB, 0.27753E+01_JPRB, 0.28627E+01_JPRB, 0.30365E+01_JPRB, 0.32869E+01_JPRB ,&
&  0.35878E+01_JPRB, 0.38432E+01_JPRB, 0.40022E+01_JPRB, 0.41258E+01_JPRB, 0.40906E+01_JPRB, 0.41525E+01_JPRB, 0.43454E+01_JPRB ,&
&  0.46233E+01_JPRB, 0.48007E+01_JPRB, 0.51245E+01_JPRB, 0.56483E+01_JPRB, 0.60106E+01_JPRB, 0.58461E+01_JPRB, 0.64701E+01_JPRB ,&
&  0.71767E+01_JPRB, 0.74071E+01_JPRB, 0.73348E+01_JPRB, 0.92668E+01_JPRB, 0.59584E+01_JPRB, 0.58732E+01_JPRB, 0.54720E+01_JPRB ,&
&  0.49746E+01_JPRB, 0.47787E+01_JPRB, 0.34555E+01_JPRB, 0.23887E+01_JPRB, 0.20251E+01_JPRB, 0.19604E+01_JPRB, 0.26359E+01_JPRB ,&
&  0.34468E+01_JPRB, 0.29849E+01_JPRB, 0.22987E+01_JPRB, 0.20516E+01_JPRB, 0.20863E+01_JPRB, 0.27796E+01_JPRB, 0.31175E+01_JPRB ,&
&  0.24628E+01_JPRB, 0.20697E+01_JPRB, 0.19973E+01_JPRB, 0.25669E+01_JPRB, 0.20278E+01_JPRB, 0.10565E+01_JPRB, 0.82756E+00_JPRB ,&
&  0.83085E+00_JPRB, 0.89134E+00_JPRB, 0.96216E+00_JPRB, 0.10286E+01_JPRB, 0.10857E+01_JPRB, 0.11558E+01_JPRB, 0.12343E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.14521E+01_JPRB, 0.15506E+01_JPRB, 0.16457E+01_JPRB, 0.17320E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.13667E+01_JPRB, 0.14870E+01_JPRB, 0.16010E+01_JPRB, 0.16809E+01_JPRB, 0.17234E+01_JPRB, 0.17570E+01_JPRB, 0.17694E+01_JPRB ,&
&  0.17897E+01_JPRB, 0.17924E+01_JPRB, 0.18578E+01_JPRB, 0.20084E+01_JPRB, 0.22656E+01_JPRB, 0.24244E+01_JPRB, 0.25943E+01_JPRB ,&
&  0.25929E+01_JPRB, 0.25944E+01_JPRB, 0.25896E+01_JPRB, 0.26304E+01_JPRB, 0.27143E+01_JPRB, 0.28356E+01_JPRB, 0.31368E+01_JPRB ,&
&  0.37466E+01_JPRB, 0.41275E+01_JPRB, 0.40061E+01_JPRB, 0.35548E+01_JPRB, 0.28711E+01_JPRB, 0.27918E+01_JPRB, 0.32153E+01_JPRB ,&
&  0.32074E+01_JPRB, 0.28007E+01_JPRB, 0.25310E+01_JPRB, 0.24851E+01_JPRB, 0.24809E+01_JPRB, 0.24871E+01_JPRB, 0.24964E+01_JPRB ,&
&  0.25279E+01_JPRB, 0.25701E+01_JPRB, 0.25945E+01_JPRB, 0.26337E+01_JPRB, 0.27172E+01_JPRB, 0.28422E+01_JPRB, 0.30006E+01_JPRB ,&
&  0.32171E+01_JPRB, 0.33984E+01_JPRB, 0.35046E+01_JPRB, 0.35604E+01_JPRB, 0.36841E+01_JPRB, 0.39852E+01_JPRB, 0.44411E+01_JPRB ,&
&  0.48662E+01_JPRB, 0.50837E+01_JPRB, 0.55826E+01_JPRB, 0.60844E+01_JPRB, 0.61347E+01_JPRB, 0.60824E+01_JPRB, 0.63722E+01_JPRB ,&
&  0.65751E+01_JPRB, 0.66244E+01_JPRB, 0.57985E+01_JPRB, 0.79861E+01_JPRB, 0.81950E+01_JPRB, 0.69470E+01_JPRB, 0.56218E+01_JPRB ,&
&  0.50228E+01_JPRB, 0.45661E+01_JPRB, 0.36414E+01_JPRB, 0.26509E+01_JPRB, 0.22088E+01_JPRB, 0.24312E+01_JPRB, 0.28703E+01_JPRB ,&
&  0.30312E+01_JPRB, 0.23603E+01_JPRB, 0.20805E+01_JPRB, 0.20565E+01_JPRB, 0.20081E+01_JPRB, 0.27805E+01_JPRB, 0.25946E+01_JPRB ,&
&  0.19009E+01_JPRB, 0.17320E+01_JPRB, 0.18255E+01_JPRB, 0.24026E+01_JPRB, 0.30441E+01_JPRB, 0.98325E+00_JPRB, 0.72475E+00_JPRB ,&
&  0.73484E+00_JPRB, 0.77069E+00_JPRB, 0.81667E+00_JPRB, 0.83892E+00_JPRB, 0.84283E+00_JPRB, 0.88658E+00_JPRB, 0.95608E+00_JPRB ,&
&  0.99505E+00_JPRB, 0.10292E+01_JPRB, 0.10731E+01_JPRB, 0.11819E+01_JPRB, 0.12652E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.84675E+00_JPRB, 0.90577E+00_JPRB, 0.94989E+00_JPRB, 0.99186E+00_JPRB, 0.10107E+01_JPRB, 0.10010E+01_JPRB, 0.95084E+00_JPRB ,&
&  0.91275E+00_JPRB, 0.88157E+00_JPRB, 0.88555E+00_JPRB, 0.87921E+00_JPRB, 0.93623E+00_JPRB, 0.10401E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.12661E+01_JPRB, 0.11659E+01_JPRB, 0.11469E+01_JPRB, 0.11686E+01_JPRB, 0.11623E+01_JPRB, 0.12098E+01_JPRB, 0.14051E+01_JPRB ,&
&  0.17037E+01_JPRB, 0.20489E+01_JPRB, 0.25495E+01_JPRB, 0.34048E+01_JPRB, 0.37712E+01_JPRB, 0.32335E+01_JPRB, 0.37013E+01_JPRB ,&
&  0.32717E+01_JPRB, 0.25680E+01_JPRB, 0.23795E+01_JPRB, 0.22189E+01_JPRB, 0.22207E+01_JPRB, 0.22297E+01_JPRB, 0.22819E+01_JPRB ,&
&  0.23428E+01_JPRB, 0.24044E+01_JPRB, 0.24527E+01_JPRB, 0.24992E+01_JPRB, 0.25593E+01_JPRB, 0.26294E+01_JPRB, 0.27279E+01_JPRB ,&
&  0.28022E+01_JPRB, 0.28588E+01_JPRB, 0.29723E+01_JPRB, 0.30311E+01_JPRB, 0.33877E+01_JPRB, 0.38913E+01_JPRB, 0.43916E+01_JPRB ,&
&  0.47351E+01_JPRB, 0.49802E+01_JPRB, 0.54293E+01_JPRB, 0.55736E+01_JPRB, 0.56396E+01_JPRB, 0.56963E+01_JPRB, 0.56213E+01_JPRB ,&
&  0.53537E+01_JPRB, 0.44225E+01_JPRB, 0.40833E+01_JPRB, 0.62996E+01_JPRB, 0.61403E+01_JPRB, 0.49329E+01_JPRB, 0.49816E+01_JPRB ,&
&  0.45960E+01_JPRB, 0.43042E+01_JPRB, 0.36756E+01_JPRB, 0.29562E+01_JPRB, 0.24497E+01_JPRB, 0.23122E+01_JPRB, 0.27310E+01_JPRB ,&
&  0.23504E+01_JPRB, 0.18922E+01_JPRB, 0.19685E+01_JPRB, 0.21270E+01_JPRB, 0.21319E+01_JPRB, 0.24186E+01_JPRB, 0.22078E+01_JPRB ,&
&  0.18394E+01_JPRB, 0.15779E+01_JPRB, 0.18857E+01_JPRB, 0.25422E+01_JPRB, 0.17179E+01_JPRB, 0.81381E+00_JPRB, 0.62993E+00_JPRB ,&
&  0.63610E+00_JPRB, 0.64886E+00_JPRB, 0.66044E+00_JPRB, 0.66669E+00_JPRB, 0.67224E+00_JPRB, 0.69184E+00_JPRB, 0.70467E+00_JPRB ,&
&  0.71656E+00_JPRB, 0.74810E+00_JPRB, 0.76297E+00_JPRB, 0.78515E+00_JPRB, 0.82067E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.70649E+00_JPRB, 0.71170E+00_JPRB, 0.70851E+00_JPRB, 0.71365E+00_JPRB, 0.72310E+00_JPRB, 0.74554E+00_JPRB, 0.76109E+00_JPRB ,&
&  0.76577E+00_JPRB, 0.77805E+00_JPRB, 0.76194E+00_JPRB, 0.77158E+00_JPRB, 0.80785E+00_JPRB, 0.84038E+00_JPRB, 0.87477E+00_JPRB ,&
&  0.95366E+00_JPRB, 0.97029E+00_JPRB, 0.96886E+00_JPRB, 0.97913E+00_JPRB, 0.96802E+00_JPRB, 0.10411E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.10276E+01_JPRB, 0.12128E+01_JPRB, 0.17127E+01_JPRB, 0.22378E+01_JPRB, 0.23906E+01_JPRB, 0.29457E+01_JPRB ,&
&  0.30554E+01_JPRB, 0.20432E+01_JPRB, 0.19648E+01_JPRB, 0.16577E+01_JPRB, 0.16493E+01_JPRB, 0.17176E+01_JPRB, 0.17442E+01_JPRB ,&
&  0.17866E+01_JPRB, 0.17762E+01_JPRB, 0.17857E+01_JPRB, 0.18194E+01_JPRB, 0.18369E+01_JPRB, 0.18595E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.18170E+01_JPRB, 0.18835E+01_JPRB, 0.21094E+01_JPRB, 0.25133E+01_JPRB, 0.29819E+01_JPRB, 0.36053E+01_JPRB, 0.38860E+01_JPRB ,&
&  0.40901E+01_JPRB, 0.43241E+01_JPRB, 0.41157E+01_JPRB, 0.42523E+01_JPRB, 0.45981E+01_JPRB, 0.43153E+01_JPRB, 0.41244E+01_JPRB ,&
&  0.39184E+01_JPRB, 0.31079E+01_JPRB, 0.30294E+01_JPRB, 0.44015E+01_JPRB, 0.46180E+01_JPRB, 0.43389E+01_JPRB, 0.43606E+01_JPRB ,&
&  0.41376E+01_JPRB, 0.37782E+01_JPRB, 0.31906E+01_JPRB, 0.27396E+01_JPRB, 0.23489E+01_JPRB, 0.20096E+01_JPRB, 0.20819E+01_JPRB ,&
&  0.17302E+01_JPRB, 0.17436E+01_JPRB, 0.20738E+01_JPRB, 0.23329E+01_JPRB, 0.25635E+01_JPRB, 0.30383E+01_JPRB, 0.23033E+01_JPRB ,&
&  0.16792E+01_JPRB, 0.16613E+01_JPRB, 0.24435E+01_JPRB, 0.31933E+01_JPRB, 0.14629E+01_JPRB, 0.82649E+00_JPRB, 0.54846E+00_JPRB ,&
&  0.56408E+00_JPRB, 0.58274E+00_JPRB, 0.58786E+00_JPRB, 0.59326E+00_JPRB, 0.60792E+00_JPRB, 0.61983E+00_JPRB, 0.64246E+00_JPRB ,&
&  0.65532E+00_JPRB, 0.66547E+00_JPRB, 0.67666E+00_JPRB, 0.69675E+00_JPRB, 0.69686E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.77264E+00_JPRB, 0.78873E+00_JPRB, 0.80170E+00_JPRB, 0.82865E+00_JPRB, 0.86693E+00_JPRB, 0.88741E+00_JPRB, 0.91778E+00_JPRB ,&
&  0.94079E+00_JPRB, 0.97318E+00_JPRB, 0.10058E+01_JPRB, 0.10443E+01_JPRB, 0.10799E+01_JPRB, 0.11033E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.11965E+01_JPRB, 0.12224E+01_JPRB, 0.12718E+01_JPRB, 0.12819E+01_JPRB, 0.13382E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.13230E+01_JPRB, 0.13397E+01_JPRB, 0.14523E+01_JPRB, 0.15906E+01_JPRB, 0.18429E+01_JPRB, 0.23839E+01_JPRB ,&
&  0.23097E+01_JPRB, 0.17230E+01_JPRB, 0.14588E+01_JPRB, 0.12289E+01_JPRB, 0.12044E+01_JPRB, 0.12107E+01_JPRB, 0.11798E+01_JPRB ,&
&  0.11676E+01_JPRB, 0.11223E+01_JPRB, 0.11077E+01_JPRB, 0.11146E+01_JPRB, 0.11857E+01_JPRB, 0.13428E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.15018E+01_JPRB, 0.17331E+01_JPRB, 0.20178E+01_JPRB, 0.23948E+01_JPRB, 0.26715E+01_JPRB, 0.33666E+01_JPRB, 0.34217E+01_JPRB ,&
&  0.33352E+01_JPRB, 0.32560E+01_JPRB, 0.28143E+01_JPRB, 0.29614E+01_JPRB, 0.32739E+01_JPRB, 0.31583E+01_JPRB, 0.28163E+01_JPRB ,&
&  0.27955E+01_JPRB, 0.26452E+01_JPRB, 0.27586E+01_JPRB, 0.36376E+01_JPRB, 0.39176E+01_JPRB, 0.39746E+01_JPRB, 0.37715E+01_JPRB ,&
&  0.34891E+01_JPRB, 0.30729E+01_JPRB, 0.26989E+01_JPRB, 0.25019E+01_JPRB, 0.23809E+01_JPRB, 0.20853E+01_JPRB, 0.18368E+01_JPRB ,&
&  0.16705E+01_JPRB, 0.18627E+01_JPRB, 0.21664E+01_JPRB, 0.24131E+01_JPRB, 0.30537E+01_JPRB, 0.28081E+01_JPRB, 0.30745E+01_JPRB ,&
&  0.16250E+01_JPRB, 0.16072E+01_JPRB, 0.12298E+01_JPRB, 0.16599E+01_JPRB, 0.98314E+00_JPRB, 0.62843E+00_JPRB, 0.57923E+00_JPRB ,&
&  0.59423E+00_JPRB, 0.60401E+00_JPRB, 0.61948E+00_JPRB, 0.63099E+00_JPRB, 0.64531E+00_JPRB, 0.67095E+00_JPRB, 0.69835E+00_JPRB ,&
&  0.72699E+00_JPRB, 0.72307E+00_JPRB, 0.72877E+00_JPRB, 0.74067E+00_JPRB, 0.76208E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.84070E+00_JPRB, 0.87361E+00_JPRB, 0.90684E+00_JPRB, 0.94072E+00_JPRB, 0.97205E+00_JPRB, 0.10027E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10993E+01_JPRB, 0.11363E+01_JPRB, 0.11777E+01_JPRB, 0.12169E+01_JPRB, 0.12456E+01_JPRB, 0.12822E+01_JPRB ,&
&  0.13196E+01_JPRB, 0.13472E+01_JPRB, 0.13557E+01_JPRB, 0.14134E+01_JPRB, 0.14684E+01_JPRB, 0.15005E+01_JPRB, 0.15473E+01_JPRB ,&
&  0.15900E+01_JPRB, 0.15433E+01_JPRB, 0.15219E+01_JPRB, 0.15643E+01_JPRB, 0.16082E+01_JPRB, 0.15660E+01_JPRB, 0.18352E+01_JPRB ,&
&  0.15318E+01_JPRB, 0.15448E+01_JPRB, 0.13659E+01_JPRB, 0.11811E+01_JPRB, 0.11224E+01_JPRB, 0.98165E+00_JPRB, 0.90490E+00_JPRB ,&
&  0.96137E+00_JPRB, 0.98247E+00_JPRB, 0.10112E+01_JPRB, 0.10750E+01_JPRB, 0.12248E+01_JPRB, 0.14893E+01_JPRB, 0.16996E+01_JPRB ,&
&  0.19593E+01_JPRB, 0.22175E+01_JPRB, 0.25151E+01_JPRB, 0.28418E+01_JPRB, 0.32028E+01_JPRB, 0.34105E+01_JPRB, 0.33633E+01_JPRB ,&
&  0.32015E+01_JPRB, 0.30048E+01_JPRB, 0.22971E+01_JPRB, 0.25910E+01_JPRB, 0.30480E+01_JPRB, 0.29638E+01_JPRB, 0.25224E+01_JPRB ,&
&  0.22836E+01_JPRB, 0.22741E+01_JPRB, 0.25651E+01_JPRB, 0.29053E+01_JPRB, 0.31203E+01_JPRB, 0.31988E+01_JPRB, 0.30370E+01_JPRB ,&
&  0.28331E+01_JPRB, 0.24581E+01_JPRB, 0.22914E+01_JPRB, 0.23292E+01_JPRB, 0.22872E+01_JPRB, 0.21284E+01_JPRB, 0.18311E+01_JPRB ,&
&  0.17648E+01_JPRB, 0.19160E+01_JPRB, 0.21284E+01_JPRB, 0.23667E+01_JPRB, 0.27295E+01_JPRB, 0.41717E+01_JPRB, 0.40459E+01_JPRB ,&
&  0.16578E+01_JPRB, 0.12952E+01_JPRB, 0.70646E+00_JPRB, 0.79606E+00_JPRB, 0.80474E+00_JPRB, 0.69877E+00_JPRB, 0.73818E+00_JPRB ,&
&  0.77498E+00_JPRB, 0.74615E+00_JPRB, 0.73873E+00_JPRB, 0.71733E+00_JPRB, 0.72187E+00_JPRB, 0.72207E+00_JPRB, 0.74107E+00_JPRB ,&
&  0.75454E+00_JPRB, 0.76199E+00_JPRB, 0.77830E+00_JPRB, 0.79864E+00_JPRB, 0.81598E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.87811E+00_JPRB, 0.91613E+00_JPRB, 0.95435E+00_JPRB, 0.98992E+00_JPRB, 0.10252E+01_JPRB, 0.10598E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11690E+01_JPRB, 0.12071E+01_JPRB, 0.12535E+01_JPRB, 0.12982E+01_JPRB, 0.13282E+01_JPRB, 0.13845E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.14972E+01_JPRB, 0.15520E+01_JPRB, 0.16376E+01_JPRB, 0.17100E+01_JPRB, 0.17648E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.18750E+01_JPRB, 0.18800E+01_JPRB, 0.18863E+01_JPRB, 0.19452E+01_JPRB, 0.19922E+01_JPRB, 0.22210E+01_JPRB, 0.19270E+01_JPRB ,&
&  0.18896E+01_JPRB, 0.16426E+01_JPRB, 0.14891E+01_JPRB, 0.13790E+01_JPRB, 0.12876E+01_JPRB, 0.12111E+01_JPRB, 0.11659E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.11821E+01_JPRB, 0.12191E+01_JPRB, 0.12874E+01_JPRB, 0.14347E+01_JPRB, 0.17213E+01_JPRB, 0.19586E+01_JPRB ,&
&  0.22217E+01_JPRB, 0.25272E+01_JPRB, 0.28131E+01_JPRB, 0.31113E+01_JPRB, 0.34436E+01_JPRB, 0.37832E+01_JPRB, 0.40569E+01_JPRB ,&
&  0.40862E+01_JPRB, 0.40016E+01_JPRB, 0.34027E+01_JPRB, 0.37594E+01_JPRB, 0.40063E+01_JPRB, 0.35859E+01_JPRB, 0.27284E+01_JPRB ,&
&  0.21650E+01_JPRB, 0.18411E+01_JPRB, 0.19309E+01_JPRB, 0.22509E+01_JPRB, 0.23823E+01_JPRB, 0.24066E+01_JPRB, 0.22941E+01_JPRB ,&
&  0.21413E+01_JPRB, 0.19247E+01_JPRB, 0.18484E+01_JPRB, 0.18912E+01_JPRB, 0.19913E+01_JPRB, 0.20414E+01_JPRB, 0.19121E+01_JPRB ,&
&  0.18765E+01_JPRB, 0.18600E+01_JPRB, 0.18682E+01_JPRB, 0.18993E+01_JPRB, 0.21185E+01_JPRB, 0.20769E+01_JPRB, 0.22732E+01_JPRB ,&
&  0.12578E+01_JPRB, 0.95465E+00_JPRB, 0.91851E+00_JPRB, 0.76988E+00_JPRB, 0.84099E+00_JPRB, 0.83225E+00_JPRB, 0.99686E+00_JPRB ,&
&  0.11293E+01_JPRB, 0.11824E+01_JPRB, 0.11129E+01_JPRB, 0.93478E+00_JPRB, 0.79953E+00_JPRB, 0.74864E+00_JPRB, 0.75533E+00_JPRB ,&
&  0.76466E+00_JPRB, 0.77161E+00_JPRB, 0.79129E+00_JPRB, 0.81934E+00_JPRB, 0.84567E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.84433E+00_JPRB, 0.88137E+00_JPRB, 0.92174E+00_JPRB, 0.95643E+00_JPRB, 0.99807E+00_JPRB, 0.10552E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.12049E+01_JPRB, 0.12413E+01_JPRB, 0.12888E+01_JPRB, 0.13489E+01_JPRB, 0.14337E+01_JPRB, 0.15403E+01_JPRB ,&
&  0.16138E+01_JPRB, 0.17137E+01_JPRB, 0.17980E+01_JPRB, 0.19267E+01_JPRB, 0.20187E+01_JPRB, 0.21603E+01_JPRB, 0.22770E+01_JPRB ,&
&  0.23391E+01_JPRB, 0.23713E+01_JPRB, 0.24128E+01_JPRB, 0.24906E+01_JPRB, 0.24984E+01_JPRB, 0.26427E+01_JPRB, 0.21451E+01_JPRB ,&
&  0.21278E+01_JPRB, 0.20017E+01_JPRB, 0.18256E+01_JPRB, 0.17410E+01_JPRB, 0.16672E+01_JPRB, 0.16213E+01_JPRB, 0.16084E+01_JPRB ,&
&  0.16112E+01_JPRB, 0.15471E+01_JPRB, 0.14224E+01_JPRB, 0.14202E+01_JPRB, 0.15287E+01_JPRB, 0.17947E+01_JPRB, 0.20496E+01_JPRB ,&
&  0.23272E+01_JPRB, 0.26108E+01_JPRB, 0.29021E+01_JPRB, 0.32766E+01_JPRB, 0.37645E+01_JPRB, 0.42824E+01_JPRB, 0.47944E+01_JPRB ,&
&  0.49897E+01_JPRB, 0.50779E+01_JPRB, 0.46478E+01_JPRB, 0.46926E+01_JPRB, 0.47846E+01_JPRB, 0.39102E+01_JPRB, 0.27400E+01_JPRB ,&
&  0.17443E+01_JPRB, 0.16154E+01_JPRB, 0.17594E+01_JPRB, 0.18791E+01_JPRB, 0.17648E+01_JPRB, 0.16680E+01_JPRB, 0.15350E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.12889E+01_JPRB, 0.13065E+01_JPRB, 0.14191E+01_JPRB, 0.15616E+01_JPRB, 0.17370E+01_JPRB, 0.17878E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.16583E+01_JPRB, 0.15826E+01_JPRB, 0.15768E+01_JPRB, 0.15708E+01_JPRB, 0.14436E+01_JPRB, 0.15878E+01_JPRB ,&
&  0.11303E+01_JPRB, 0.10616E+01_JPRB, 0.10303E+01_JPRB, 0.67165E+00_JPRB, 0.83192E+00_JPRB, 0.10334E+01_JPRB, 0.13008E+01_JPRB ,&
&  0.14187E+01_JPRB, 0.18675E+01_JPRB, 0.22468E+01_JPRB, 0.16111E+01_JPRB, 0.12155E+01_JPRB, 0.80359E+00_JPRB, 0.73976E+00_JPRB ,&
&  0.75038E+00_JPRB, 0.75007E+00_JPRB, 0.76324E+00_JPRB, 0.78729E+00_JPRB, 0.81245E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.76802E+00_JPRB, 0.79644E+00_JPRB, 0.83195E+00_JPRB, 0.84738E+00_JPRB, 0.86486E+00_JPRB, 0.89143E+00_JPRB, 0.92026E+00_JPRB ,&
&  0.94688E+00_JPRB, 0.97656E+00_JPRB, 0.10005E+01_JPRB, 0.10355E+01_JPRB, 0.10813E+01_JPRB, 0.11868E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.14836E+01_JPRB, 0.16317E+01_JPRB, 0.18123E+01_JPRB, 0.19322E+01_JPRB, 0.20982E+01_JPRB, 0.22783E+01_JPRB, 0.24171E+01_JPRB ,&
&  0.25641E+01_JPRB, 0.26417E+01_JPRB, 0.27134E+01_JPRB, 0.28834E+01_JPRB, 0.28383E+01_JPRB, 0.29130E+01_JPRB, 0.22207E+01_JPRB ,&
&  0.22707E+01_JPRB, 0.22399E+01_JPRB, 0.21298E+01_JPRB, 0.20730E+01_JPRB, 0.19880E+01_JPRB, 0.18659E+01_JPRB, 0.17797E+01_JPRB ,&
&  0.17188E+01_JPRB, 0.15633E+01_JPRB, 0.13894E+01_JPRB, 0.14013E+01_JPRB, 0.14665E+01_JPRB, 0.16322E+01_JPRB, 0.19387E+01_JPRB ,&
&  0.22407E+01_JPRB, 0.25229E+01_JPRB, 0.28125E+01_JPRB, 0.32302E+01_JPRB, 0.37832E+01_JPRB, 0.43715E+01_JPRB, 0.49917E+01_JPRB ,&
&  0.54172E+01_JPRB, 0.57380E+01_JPRB, 0.58176E+01_JPRB, 0.49419E+01_JPRB, 0.46743E+01_JPRB, 0.35712E+01_JPRB, 0.23637E+01_JPRB ,&
&  0.17540E+01_JPRB, 0.15770E+01_JPRB, 0.15910E+01_JPRB, 0.15758E+01_JPRB, 0.13745E+01_JPRB, 0.12472E+01_JPRB, 0.11047E+01_JPRB ,&
&  0.99425E+00_JPRB, 0.96819E+00_JPRB, 0.10121E+01_JPRB, 0.11415E+01_JPRB, 0.12754E+01_JPRB, 0.14442E+01_JPRB, 0.14994E+01_JPRB ,&
&  0.14546E+01_JPRB, 0.13517E+01_JPRB, 0.13124E+01_JPRB, 0.13192E+01_JPRB, 0.13727E+01_JPRB, 0.14569E+01_JPRB, 0.12616E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.86329E+00_JPRB, 0.81520E+00_JPRB, 0.86485E+00_JPRB, 0.87915E+00_JPRB, 0.11330E+01_JPRB, 0.15698E+01_JPRB ,&
&  0.22034E+01_JPRB, 0.22935E+01_JPRB, 0.27206E+01_JPRB, 0.35739E+01_JPRB, 0.26251E+01_JPRB, 0.18896E+01_JPRB, 0.66022E+00_JPRB ,&
&  0.68332E+00_JPRB, 0.68822E+00_JPRB, 0.69835E+00_JPRB, 0.70953E+00_JPRB, 0.73246E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.63121E+00_JPRB, 0.66685E+00_JPRB, 0.66282E+00_JPRB, 0.67092E+00_JPRB, 0.68168E+00_JPRB, 0.69957E+00_JPRB, 0.73484E+00_JPRB ,&
&  0.75147E+00_JPRB, 0.81128E+00_JPRB, 0.88032E+00_JPRB, 0.94342E+00_JPRB, 0.10205E+01_JPRB, 0.11273E+01_JPRB, 0.12692E+01_JPRB ,&
&  0.14349E+01_JPRB, 0.15425E+01_JPRB, 0.16589E+01_JPRB, 0.17595E+01_JPRB, 0.19115E+01_JPRB, 0.20984E+01_JPRB, 0.22411E+01_JPRB ,&
&  0.24394E+01_JPRB, 0.25903E+01_JPRB, 0.27547E+01_JPRB, 0.29806E+01_JPRB, 0.28650E+01_JPRB, 0.28132E+01_JPRB, 0.19641E+01_JPRB ,&
&  0.22600E+01_JPRB, 0.23132E+01_JPRB, 0.22777E+01_JPRB, 0.22252E+01_JPRB, 0.21069E+01_JPRB, 0.19177E+01_JPRB, 0.17936E+01_JPRB ,&
&  0.16745E+01_JPRB, 0.15421E+01_JPRB, 0.13835E+01_JPRB, 0.12928E+01_JPRB, 0.13990E+01_JPRB, 0.14398E+01_JPRB, 0.17044E+01_JPRB ,&
&  0.20121E+01_JPRB, 0.23599E+01_JPRB, 0.26924E+01_JPRB, 0.30959E+01_JPRB, 0.36254E+01_JPRB, 0.42332E+01_JPRB, 0.48534E+01_JPRB ,&
&  0.53198E+01_JPRB, 0.57089E+01_JPRB, 0.56533E+01_JPRB, 0.47041E+01_JPRB, 0.41669E+01_JPRB, 0.31445E+01_JPRB, 0.20787E+01_JPRB ,&
&  0.16702E+01_JPRB, 0.15478E+01_JPRB, 0.15503E+01_JPRB, 0.14530E+01_JPRB, 0.12252E+01_JPRB, 0.10646E+01_JPRB, 0.92244E+00_JPRB ,&
&  0.80557E+00_JPRB, 0.79783E+00_JPRB, 0.88438E+00_JPRB, 0.10092E+01_JPRB, 0.11125E+01_JPRB, 0.12197E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.10783E+01_JPRB, 0.10576E+01_JPRB, 0.11455E+01_JPRB, 0.13617E+01_JPRB, 0.15834E+01_JPRB, 0.17013E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.86628E+00_JPRB, 0.81676E+00_JPRB, 0.81518E+00_JPRB, 0.84433E+00_JPRB, 0.91868E+00_JPRB, 0.10437E+01_JPRB ,&
&  0.12307E+01_JPRB, 0.16350E+01_JPRB, 0.15507E+01_JPRB, 0.12514E+01_JPRB, 0.17263E+01_JPRB, 0.14572E+01_JPRB, 0.59973E+00_JPRB ,&
&  0.58331E+00_JPRB, 0.57200E+00_JPRB, 0.58660E+00_JPRB, 0.59756E+00_JPRB, 0.62013E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.58670E+00_JPRB, 0.57664E+00_JPRB, 0.55917E+00_JPRB, 0.57337E+00_JPRB, 0.57901E+00_JPRB, 0.60598E+00_JPRB, 0.63846E+00_JPRB ,&
&  0.68488E+00_JPRB, 0.77046E+00_JPRB, 0.86559E+00_JPRB, 0.95243E+00_JPRB, 0.10222E+01_JPRB, 0.11007E+01_JPRB, 0.11740E+01_JPRB ,&
&  0.12424E+01_JPRB, 0.13024E+01_JPRB, 0.13886E+01_JPRB, 0.14980E+01_JPRB, 0.16396E+01_JPRB, 0.18064E+01_JPRB, 0.20075E+01_JPRB ,&
&  0.22321E+01_JPRB, 0.24671E+01_JPRB, 0.27567E+01_JPRB, 0.29112E+01_JPRB, 0.30125E+01_JPRB, 0.31043E+01_JPRB, 0.31133E+01_JPRB ,&
&  0.20130E+01_JPRB, 0.24921E+01_JPRB, 0.23904E+01_JPRB, 0.22650E+01_JPRB, 0.21563E+01_JPRB, 0.19797E+01_JPRB, 0.18243E+01_JPRB ,&
&  0.17108E+01_JPRB, 0.15345E+01_JPRB, 0.14463E+01_JPRB, 0.12582E+01_JPRB, 0.12637E+01_JPRB, 0.13213E+01_JPRB, 0.14629E+01_JPRB ,&
&  0.17066E+01_JPRB, 0.20703E+01_JPRB, 0.25134E+01_JPRB, 0.29365E+01_JPRB, 0.33558E+01_JPRB, 0.38220E+01_JPRB, 0.44739E+01_JPRB ,&
&  0.51876E+01_JPRB, 0.56798E+01_JPRB, 0.59798E+01_JPRB, 0.51783E+01_JPRB, 0.41184E+01_JPRB, 0.35520E+01_JPRB, 0.26290E+01_JPRB ,&
&  0.17390E+01_JPRB, 0.14321E+01_JPRB, 0.14592E+01_JPRB, 0.14290E+01_JPRB, 0.12229E+01_JPRB, 0.10708E+01_JPRB, 0.92289E+00_JPRB ,&
&  0.78817E+00_JPRB, 0.71873E+00_JPRB, 0.74427E+00_JPRB, 0.87489E+00_JPRB, 0.94395E+00_JPRB, 0.97977E+00_JPRB, 0.96052E+00_JPRB ,&
&  0.93652E+00_JPRB, 0.87670E+00_JPRB, 0.82024E+00_JPRB, 0.83226E+00_JPRB, 0.89151E+00_JPRB, 0.93770E+00_JPRB, 0.92562E+00_JPRB ,&
&  0.88841E+00_JPRB, 0.86931E+00_JPRB, 0.86095E+00_JPRB, 0.86846E+00_JPRB, 0.86637E+00_JPRB, 0.87477E+00_JPRB, 0.89102E+00_JPRB ,&
&  0.91020E+00_JPRB, 0.94327E+00_JPRB, 0.91204E+00_JPRB, 0.87242E+00_JPRB, 0.91665E+00_JPRB, 0.96307E+00_JPRB, 0.65325E+00_JPRB ,&
&  0.57222E+00_JPRB, 0.54117E+00_JPRB, 0.55153E+00_JPRB, 0.54841E+00_JPRB, 0.59730E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.60135E+00_JPRB, 0.58000E+00_JPRB, 0.56904E+00_JPRB, 0.56173E+00_JPRB, 0.54685E+00_JPRB, 0.58113E+00_JPRB, 0.59693E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.70069E+00_JPRB, 0.78132E+00_JPRB, 0.84384E+00_JPRB, 0.87935E+00_JPRB, 0.93620E+00_JPRB, 0.98686E+00_JPRB ,&
&  0.10272E+01_JPRB, 0.10601E+01_JPRB, 0.11045E+01_JPRB, 0.11610E+01_JPRB, 0.12540E+01_JPRB, 0.13939E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.21142E+01_JPRB, 0.24064E+01_JPRB, 0.26866E+01_JPRB, 0.29020E+01_JPRB, 0.30692E+01_JPRB, 0.30187E+01_JPRB ,&
&  0.16121E+01_JPRB, 0.23291E+01_JPRB, 0.26914E+01_JPRB, 0.24477E+01_JPRB, 0.22409E+01_JPRB, 0.20345E+01_JPRB, 0.18294E+01_JPRB ,&
&  0.16478E+01_JPRB, 0.14757E+01_JPRB, 0.13946E+01_JPRB, 0.13587E+01_JPRB, 0.11982E+01_JPRB, 0.12215E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.14307E+01_JPRB, 0.16907E+01_JPRB, 0.21729E+01_JPRB, 0.26984E+01_JPRB, 0.29925E+01_JPRB, 0.32852E+01_JPRB, 0.38256E+01_JPRB ,&
&  0.46542E+01_JPRB, 0.52983E+01_JPRB, 0.57643E+01_JPRB, 0.43652E+01_JPRB, 0.37179E+01_JPRB, 0.35983E+01_JPRB, 0.32426E+01_JPRB ,&
&  0.20061E+01_JPRB, 0.14582E+01_JPRB, 0.14767E+01_JPRB, 0.14759E+01_JPRB, 0.12823E+01_JPRB, 0.11618E+01_JPRB, 0.10247E+01_JPRB ,&
&  0.86180E+00_JPRB, 0.75545E+00_JPRB, 0.67882E+00_JPRB, 0.71684E+00_JPRB, 0.75682E+00_JPRB, 0.78197E+00_JPRB, 0.74199E+00_JPRB ,&
&  0.78282E+00_JPRB, 0.76680E+00_JPRB, 0.75590E+00_JPRB, 0.76209E+00_JPRB, 0.83756E+00_JPRB, 0.87420E+00_JPRB, 0.89077E+00_JPRB ,&
&  0.91347E+00_JPRB, 0.93108E+00_JPRB, 0.93051E+00_JPRB, 0.93390E+00_JPRB, 0.93527E+00_JPRB, 0.92580E+00_JPRB, 0.91342E+00_JPRB ,&
&  0.88829E+00_JPRB, 0.87051E+00_JPRB, 0.82771E+00_JPRB, 0.78074E+00_JPRB, 0.71766E+00_JPRB, 0.71578E+00_JPRB, 0.65168E+00_JPRB ,&
&  0.60969E+00_JPRB, 0.58318E+00_JPRB, 0.56748E+00_JPRB, 0.56533E+00_JPRB, 0.57780E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.62731E+00_JPRB, 0.62271E+00_JPRB, 0.61652E+00_JPRB, 0.60418E+00_JPRB, 0.58767E+00_JPRB, 0.58836E+00_JPRB, 0.57706E+00_JPRB ,&
&  0.58193E+00_JPRB, 0.61695E+00_JPRB, 0.66987E+00_JPRB, 0.71238E+00_JPRB, 0.75325E+00_JPRB, 0.78689E+00_JPRB, 0.82749E+00_JPRB ,&
&  0.84753E+00_JPRB, 0.85386E+00_JPRB, 0.85373E+00_JPRB, 0.85820E+00_JPRB, 0.88296E+00_JPRB, 0.95935E+00_JPRB, 0.10891E+01_JPRB ,&
&  0.12769E+01_JPRB, 0.15269E+01_JPRB, 0.18453E+01_JPRB, 0.22046E+01_JPRB, 0.25864E+01_JPRB, 0.29719E+01_JPRB, 0.31924E+01_JPRB ,&
&  0.20061E+01_JPRB, 0.93424E+00_JPRB, 0.25277E+01_JPRB, 0.27845E+01_JPRB, 0.25018E+01_JPRB, 0.23151E+01_JPRB, 0.20043E+01_JPRB ,&
&  0.17225E+01_JPRB, 0.16039E+01_JPRB, 0.14467E+01_JPRB, 0.13334E+01_JPRB, 0.11907E+01_JPRB, 0.11756E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.13980E+01_JPRB, 0.17341E+01_JPRB, 0.23051E+01_JPRB, 0.26913E+01_JPRB, 0.28437E+01_JPRB, 0.31918E+01_JPRB ,&
&  0.38952E+01_JPRB, 0.45061E+01_JPRB, 0.47598E+01_JPRB, 0.32426E+01_JPRB, 0.22940E+01_JPRB, 0.20582E+01_JPRB, 0.19169E+01_JPRB ,&
&  0.18258E+01_JPRB, 0.15306E+01_JPRB, 0.15057E+01_JPRB, 0.15301E+01_JPRB, 0.13913E+01_JPRB, 0.12259E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.91605E+00_JPRB, 0.80443E+00_JPRB, 0.74903E+00_JPRB, 0.71421E+00_JPRB, 0.72009E+00_JPRB, 0.71398E+00_JPRB ,&
&  0.71473E+00_JPRB, 0.73860E+00_JPRB, 0.74981E+00_JPRB, 0.76906E+00_JPRB, 0.82273E+00_JPRB, 0.87614E+00_JPRB, 0.91018E+00_JPRB ,&
&  0.93877E+00_JPRB, 0.96806E+00_JPRB, 0.98357E+00_JPRB, 0.98636E+00_JPRB, 0.98289E+00_JPRB, 0.95940E+00_JPRB, 0.95994E+00_JPRB ,&
&  0.95919E+00_JPRB, 0.95095E+00_JPRB, 0.90664E+00_JPRB, 0.87124E+00_JPRB, 0.79613E+00_JPRB, 0.74087E+00_JPRB, 0.66834E+00_JPRB ,&
&  0.63877E+00_JPRB, 0.59722E+00_JPRB, 0.57059E+00_JPRB, 0.57153E+00_JPRB, 0.58977E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.64555E+00_JPRB, 0.64578E+00_JPRB, 0.64948E+00_JPRB, 0.65320E+00_JPRB, 0.64980E+00_JPRB, 0.65071E+00_JPRB, 0.62763E+00_JPRB ,&
&  0.59487E+00_JPRB, 0.58537E+00_JPRB, 0.61372E+00_JPRB, 0.61887E+00_JPRB, 0.62157E+00_JPRB, 0.65218E+00_JPRB, 0.68724E+00_JPRB ,&
&  0.70996E+00_JPRB, 0.72163E+00_JPRB, 0.72199E+00_JPRB, 0.72178E+00_JPRB, 0.70897E+00_JPRB, 0.71388E+00_JPRB, 0.75815E+00_JPRB ,&
&  0.84090E+00_JPRB, 0.99169E+00_JPRB, 0.12204E+01_JPRB, 0.15129E+01_JPRB, 0.18688E+01_JPRB, 0.23855E+01_JPRB, 0.28993E+01_JPRB ,&
&  0.38433E+01_JPRB, 0.25451E+01_JPRB, 0.13884E+01_JPRB, 0.25485E+01_JPRB, 0.28547E+01_JPRB, 0.28013E+01_JPRB, 0.22823E+01_JPRB ,&
&  0.21068E+01_JPRB, 0.19714E+01_JPRB, 0.18076E+01_JPRB, 0.15987E+01_JPRB, 0.13230E+01_JPRB, 0.12171E+01_JPRB, 0.11283E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.11533E+01_JPRB, 0.13380E+01_JPRB, 0.17313E+01_JPRB, 0.22411E+01_JPRB, 0.25053E+01_JPRB, 0.26599E+01_JPRB ,&
&  0.31395E+01_JPRB, 0.37594E+01_JPRB, 0.39081E+01_JPRB, 0.26370E+01_JPRB, 0.19737E+01_JPRB, 0.18068E+01_JPRB, 0.17693E+01_JPRB ,&
&  0.16952E+01_JPRB, 0.17022E+01_JPRB, 0.17300E+01_JPRB, 0.16022E+01_JPRB, 0.14406E+01_JPRB, 0.13217E+01_JPRB, 0.13212E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.10763E+01_JPRB, 0.97057E+00_JPRB, 0.85800E+00_JPRB, 0.76195E+00_JPRB, 0.73657E+00_JPRB, 0.72545E+00_JPRB ,&
&  0.73423E+00_JPRB, 0.76174E+00_JPRB, 0.76830E+00_JPRB, 0.79634E+00_JPRB, 0.82675E+00_JPRB, 0.85807E+00_JPRB, 0.88856E+00_JPRB ,&
&  0.91683E+00_JPRB, 0.94809E+00_JPRB, 0.97639E+00_JPRB, 0.98186E+00_JPRB, 0.97166E+00_JPRB, 0.95581E+00_JPRB, 0.97148E+00_JPRB ,&
&  0.97921E+00_JPRB, 0.99083E+00_JPRB, 0.96969E+00_JPRB, 0.95879E+00_JPRB, 0.89622E+00_JPRB, 0.83565E+00_JPRB, 0.76969E+00_JPRB ,&
&  0.74769E+00_JPRB, 0.69822E+00_JPRB, 0.66308E+00_JPRB, 0.62465E+00_JPRB, 0.63271E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.67171E+00_JPRB, 0.66408E+00_JPRB, 0.66613E+00_JPRB, 0.67093E+00_JPRB, 0.67231E+00_JPRB, 0.67595E+00_JPRB, 0.67622E+00_JPRB ,&
&  0.64089E+00_JPRB, 0.60814E+00_JPRB, 0.59085E+00_JPRB, 0.57787E+00_JPRB, 0.57118E+00_JPRB, 0.58814E+00_JPRB, 0.59948E+00_JPRB ,&
&  0.60162E+00_JPRB, 0.60402E+00_JPRB, 0.63334E+00_JPRB, 0.64372E+00_JPRB, 0.64175E+00_JPRB, 0.63234E+00_JPRB, 0.63190E+00_JPRB ,&
&  0.65418E+00_JPRB, 0.71149E+00_JPRB, 0.82799E+00_JPRB, 0.98931E+00_JPRB, 0.11755E+01_JPRB, 0.15117E+01_JPRB, 0.19553E+01_JPRB ,&
&  0.27130E+01_JPRB, 0.28104E+01_JPRB, 0.12044E+01_JPRB, 0.21854E+01_JPRB, 0.30130E+01_JPRB, 0.30863E+01_JPRB, 0.28238E+01_JPRB ,&
&  0.25864E+01_JPRB, 0.23737E+01_JPRB, 0.23999E+01_JPRB, 0.21433E+01_JPRB, 0.16436E+01_JPRB, 0.13457E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10045E+01_JPRB, 0.10634E+01_JPRB, 0.12568E+01_JPRB, 0.16580E+01_JPRB, 0.20326E+01_JPRB, 0.22186E+01_JPRB ,&
&  0.24472E+01_JPRB, 0.30291E+01_JPRB, 0.34312E+01_JPRB, 0.24125E+01_JPRB, 0.19661E+01_JPRB, 0.20067E+01_JPRB, 0.19873E+01_JPRB ,&
&  0.18970E+01_JPRB, 0.18630E+01_JPRB, 0.18227E+01_JPRB, 0.16865E+01_JPRB, 0.15098E+01_JPRB, 0.14323E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.13155E+01_JPRB, 0.12078E+01_JPRB, 0.11062E+01_JPRB, 0.99988E+00_JPRB, 0.88125E+00_JPRB, 0.78987E+00_JPRB, 0.77629E+00_JPRB ,&
&  0.77734E+00_JPRB, 0.80274E+00_JPRB, 0.80638E+00_JPRB, 0.82435E+00_JPRB, 0.84327E+00_JPRB, 0.86243E+00_JPRB, 0.87803E+00_JPRB ,&
&  0.89662E+00_JPRB, 0.92058E+00_JPRB, 0.94507E+00_JPRB, 0.94745E+00_JPRB, 0.94587E+00_JPRB, 0.94289E+00_JPRB, 0.94801E+00_JPRB ,&
&  0.96145E+00_JPRB, 0.97627E+00_JPRB, 0.96754E+00_JPRB, 0.96599E+00_JPRB, 0.97151E+00_JPRB, 0.93836E+00_JPRB, 0.89494E+00_JPRB ,&
&  0.91834E+00_JPRB, 0.94241E+00_JPRB, 0.87101E+00_JPRB, 0.72297E+00_JPRB, 0.68968E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.72306E+00_JPRB, 0.70791E+00_JPRB, 0.69788E+00_JPRB, 0.69648E+00_JPRB, 0.69394E+00_JPRB, 0.69430E+00_JPRB, 0.70351E+00_JPRB ,&
&  0.69026E+00_JPRB, 0.66567E+00_JPRB, 0.63741E+00_JPRB, 0.60616E+00_JPRB, 0.58265E+00_JPRB, 0.58468E+00_JPRB, 0.59291E+00_JPRB ,&
&  0.59587E+00_JPRB, 0.59070E+00_JPRB, 0.59067E+00_JPRB, 0.60076E+00_JPRB, 0.60035E+00_JPRB, 0.59386E+00_JPRB, 0.58686E+00_JPRB ,&
&  0.58640E+00_JPRB, 0.59986E+00_JPRB, 0.64284E+00_JPRB, 0.71971E+00_JPRB, 0.81840E+00_JPRB, 0.98780E+00_JPRB, 0.12426E+01_JPRB ,&
&  0.18279E+01_JPRB, 0.22245E+01_JPRB, 0.23794E+01_JPRB, 0.35915E+01_JPRB, 0.37686E+01_JPRB, 0.35480E+01_JPRB, 0.34859E+01_JPRB ,&
&  0.33426E+01_JPRB, 0.30901E+01_JPRB, 0.32493E+01_JPRB, 0.25429E+01_JPRB, 0.19837E+01_JPRB, 0.15281E+01_JPRB, 0.12351E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.97324E+00_JPRB, 0.94743E+00_JPRB, 0.99139E+00_JPRB, 0.11877E+01_JPRB, 0.15363E+01_JPRB, 0.18159E+01_JPRB ,&
&  0.20202E+01_JPRB, 0.24109E+01_JPRB, 0.29868E+01_JPRB, 0.25338E+01_JPRB, 0.20780E+01_JPRB, 0.23649E+01_JPRB, 0.26457E+01_JPRB ,&
&  0.29647E+01_JPRB, 0.22570E+01_JPRB, 0.19814E+01_JPRB, 0.18042E+01_JPRB, 0.15836E+01_JPRB, 0.16939E+01_JPRB, 0.15306E+01_JPRB ,&
&  0.14083E+01_JPRB, 0.13165E+01_JPRB, 0.12201E+01_JPRB, 0.11254E+01_JPRB, 0.10247E+01_JPRB, 0.92488E+00_JPRB, 0.87040E+00_JPRB ,&
&  0.86041E+00_JPRB, 0.86744E+00_JPRB, 0.86421E+00_JPRB, 0.87293E+00_JPRB, 0.87510E+00_JPRB, 0.88709E+00_JPRB, 0.89822E+00_JPRB ,&
&  0.91098E+00_JPRB, 0.91867E+00_JPRB, 0.90346E+00_JPRB, 0.90708E+00_JPRB, 0.91043E+00_JPRB, 0.90734E+00_JPRB, 0.90022E+00_JPRB ,&
&  0.90886E+00_JPRB, 0.92928E+00_JPRB, 0.94098E+00_JPRB, 0.94907E+00_JPRB, 0.96058E+00_JPRB, 0.94298E+00_JPRB, 0.90280E+00_JPRB ,&
&  0.88157E+00_JPRB, 0.87704E+00_JPRB, 0.85485E+00_JPRB, 0.78496E+00_JPRB, 0.74958E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.75312E+00_JPRB, 0.73855E+00_JPRB, 0.72862E+00_JPRB, 0.71775E+00_JPRB, 0.70467E+00_JPRB, 0.70187E+00_JPRB, 0.70986E+00_JPRB ,&
&  0.70805E+00_JPRB, 0.69286E+00_JPRB, 0.67136E+00_JPRB, 0.64936E+00_JPRB, 0.62178E+00_JPRB, 0.60686E+00_JPRB, 0.59784E+00_JPRB ,&
&  0.59184E+00_JPRB, 0.58888E+00_JPRB, 0.58206E+00_JPRB, 0.58074E+00_JPRB, 0.57979E+00_JPRB, 0.56749E+00_JPRB, 0.55767E+00_JPRB ,&
&  0.55299E+00_JPRB, 0.55851E+00_JPRB, 0.57506E+00_JPRB, 0.61378E+00_JPRB, 0.67459E+00_JPRB, 0.78195E+00_JPRB, 0.92394E+00_JPRB ,&
&  0.12884E+01_JPRB, 0.18585E+01_JPRB, 0.16854E+01_JPRB, 0.37454E+01_JPRB, 0.37151E+01_JPRB, 0.36499E+01_JPRB, 0.34484E+01_JPRB ,&
&  0.34272E+01_JPRB, 0.38055E+01_JPRB, 0.32947E+01_JPRB, 0.26974E+01_JPRB, 0.20914E+01_JPRB, 0.16559E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.10117E+01_JPRB, 0.95417E+00_JPRB, 0.92660E+00_JPRB, 0.94123E+00_JPRB, 0.11301E+01_JPRB, 0.14383E+01_JPRB ,&
&  0.16734E+01_JPRB, 0.19589E+01_JPRB, 0.24912E+01_JPRB, 0.23831E+01_JPRB, 0.22313E+01_JPRB, 0.26202E+01_JPRB, 0.33617E+01_JPRB ,&
&  0.40208E+01_JPRB, 0.31110E+01_JPRB, 0.23363E+01_JPRB, 0.20112E+01_JPRB, 0.18269E+01_JPRB, 0.17852E+01_JPRB, 0.16343E+01_JPRB ,&
&  0.14716E+01_JPRB, 0.13812E+01_JPRB, 0.13052E+01_JPRB, 0.12307E+01_JPRB, 0.11465E+01_JPRB, 0.10642E+01_JPRB, 0.98529E+00_JPRB ,&
&  0.94577E+00_JPRB, 0.93294E+00_JPRB, 0.92759E+00_JPRB, 0.93539E+00_JPRB, 0.93865E+00_JPRB, 0.94924E+00_JPRB, 0.95142E+00_JPRB ,&
&  0.95721E+00_JPRB, 0.94717E+00_JPRB, 0.89925E+00_JPRB, 0.88303E+00_JPRB, 0.88178E+00_JPRB, 0.87496E+00_JPRB, 0.86428E+00_JPRB ,&
&  0.87743E+00_JPRB, 0.89466E+00_JPRB, 0.90000E+00_JPRB, 0.90304E+00_JPRB, 0.93111E+00_JPRB, 0.93917E+00_JPRB, 0.90888E+00_JPRB ,&
&  0.87441E+00_JPRB, 0.84458E+00_JPRB, 0.81774E+00_JPRB, 0.78634E+00_JPRB, 0.77161E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.76327E+00_JPRB, 0.75213E+00_JPRB, 0.74159E+00_JPRB, 0.72766E+00_JPRB, 0.70843E+00_JPRB, 0.69916E+00_JPRB, 0.70304E+00_JPRB ,&
&  0.70861E+00_JPRB, 0.69940E+00_JPRB, 0.68397E+00_JPRB, 0.66316E+00_JPRB, 0.63676E+00_JPRB, 0.62144E+00_JPRB, 0.61066E+00_JPRB ,&
&  0.59526E+00_JPRB, 0.58217E+00_JPRB, 0.56534E+00_JPRB, 0.55892E+00_JPRB, 0.55909E+00_JPRB, 0.55035E+00_JPRB, 0.53247E+00_JPRB ,&
&  0.53309E+00_JPRB, 0.54313E+00_JPRB, 0.55604E+00_JPRB, 0.57521E+00_JPRB, 0.61764E+00_JPRB, 0.69684E+00_JPRB, 0.82191E+00_JPRB ,&
&  0.10851E+01_JPRB, 0.22055E+01_JPRB, 0.15459E+01_JPRB, 0.28824E+01_JPRB, 0.29834E+01_JPRB, 0.31933E+01_JPRB, 0.32536E+01_JPRB ,&
&  0.33458E+01_JPRB, 0.34038E+01_JPRB, 0.31217E+01_JPRB, 0.25420E+01_JPRB, 0.19739E+01_JPRB, 0.16048E+01_JPRB, 0.13696E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.10922E+01_JPRB, 0.10067E+01_JPRB, 0.94945E+00_JPRB, 0.91240E+00_JPRB, 0.92610E+00_JPRB, 0.10795E+01_JPRB ,&
&  0.13475E+01_JPRB, 0.15944E+01_JPRB, 0.19963E+01_JPRB, 0.24197E+01_JPRB, 0.22357E+01_JPRB, 0.25046E+01_JPRB, 0.36029E+01_JPRB ,&
&  0.43114E+01_JPRB, 0.43415E+01_JPRB, 0.29725E+01_JPRB, 0.23617E+01_JPRB, 0.21043E+01_JPRB, 0.19496E+01_JPRB, 0.17170E+01_JPRB ,&
&  0.15021E+01_JPRB, 0.13931E+01_JPRB, 0.13339E+01_JPRB, 0.12848E+01_JPRB, 0.12237E+01_JPRB, 0.11383E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.98105E+00_JPRB, 0.96021E+00_JPRB, 0.95746E+00_JPRB, 0.96321E+00_JPRB, 0.97779E+00_JPRB, 0.99435E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.10016E+01_JPRB, 0.99007E+00_JPRB, 0.92669E+00_JPRB, 0.89119E+00_JPRB, 0.87873E+00_JPRB, 0.86672E+00_JPRB, 0.85310E+00_JPRB ,&
&  0.86076E+00_JPRB, 0.86887E+00_JPRB, 0.87137E+00_JPRB, 0.88530E+00_JPRB, 0.91051E+00_JPRB, 0.94077E+00_JPRB, 0.88003E+00_JPRB ,&
&  0.87366E+00_JPRB, 0.85055E+00_JPRB, 0.82336E+00_JPRB, 0.79375E+00_JPRB, 0.77883E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.75951E+00_JPRB, 0.74963E+00_JPRB, 0.73887E+00_JPRB, 0.72639E+00_JPRB, 0.71194E+00_JPRB, 0.70108E+00_JPRB, 0.69729E+00_JPRB ,&
&  0.69607E+00_JPRB, 0.69560E+00_JPRB, 0.68624E+00_JPRB, 0.66422E+00_JPRB, 0.64087E+00_JPRB, 0.61981E+00_JPRB, 0.60728E+00_JPRB ,&
&  0.59637E+00_JPRB, 0.58233E+00_JPRB, 0.56559E+00_JPRB, 0.55014E+00_JPRB, 0.54545E+00_JPRB, 0.54489E+00_JPRB, 0.53464E+00_JPRB ,&
&  0.54005E+00_JPRB, 0.54919E+00_JPRB, 0.55719E+00_JPRB, 0.56623E+00_JPRB, 0.59838E+00_JPRB, 0.66448E+00_JPRB, 0.77874E+00_JPRB ,&
&  0.10042E+01_JPRB, 0.87066E+00_JPRB, 0.17416E+01_JPRB, 0.21493E+01_JPRB, 0.23667E+01_JPRB, 0.25532E+01_JPRB, 0.29477E+01_JPRB ,&
&  0.32389E+01_JPRB, 0.31293E+01_JPRB, 0.27879E+01_JPRB, 0.23220E+01_JPRB, 0.18960E+01_JPRB, 0.15840E+01_JPRB, 0.13751E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.11403E+01_JPRB, 0.10495E+01_JPRB, 0.97108E+00_JPRB, 0.91906E+00_JPRB, 0.89072E+00_JPRB, 0.90547E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.12064E+01_JPRB, 0.14814E+01_JPRB, 0.19667E+01_JPRB, 0.22131E+01_JPRB, 0.23839E+01_JPRB, 0.27995E+01_JPRB ,&
&  0.37074E+01_JPRB, 0.42770E+01_JPRB, 0.34219E+01_JPRB, 0.27265E+01_JPRB, 0.23535E+01_JPRB, 0.20324E+01_JPRB, 0.17509E+01_JPRB ,&
&  0.15285E+01_JPRB, 0.13928E+01_JPRB, 0.13110E+01_JPRB, 0.12623E+01_JPRB, 0.12320E+01_JPRB, 0.11887E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10303E+01_JPRB, 0.10147E+01_JPRB, 0.10120E+01_JPRB, 0.10143E+01_JPRB, 0.10268E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.10147E+01_JPRB, 0.99050E+00_JPRB, 0.94824E+00_JPRB, 0.90817E+00_JPRB, 0.89131E+00_JPRB, 0.87279E+00_JPRB, 0.85185E+00_JPRB ,&
&  0.84568E+00_JPRB, 0.84747E+00_JPRB, 0.85347E+00_JPRB, 0.87541E+00_JPRB, 0.88553E+00_JPRB, 0.91199E+00_JPRB, 0.86083E+00_JPRB ,&
&  0.84792E+00_JPRB, 0.82563E+00_JPRB, 0.80342E+00_JPRB, 0.78225E+00_JPRB, 0.77353E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.75603E+00_JPRB, 0.74558E+00_JPRB, 0.73256E+00_JPRB, 0.71870E+00_JPRB, 0.70359E+00_JPRB, 0.69149E+00_JPRB, 0.68503E+00_JPRB ,&
&  0.68110E+00_JPRB, 0.67959E+00_JPRB, 0.67877E+00_JPRB, 0.66328E+00_JPRB, 0.64345E+00_JPRB, 0.62468E+00_JPRB, 0.61293E+00_JPRB ,&
&  0.60449E+00_JPRB, 0.59473E+00_JPRB, 0.58236E+00_JPRB, 0.56706E+00_JPRB, 0.55903E+00_JPRB, 0.55336E+00_JPRB, 0.54877E+00_JPRB ,&
&  0.55391E+00_JPRB, 0.55800E+00_JPRB, 0.56346E+00_JPRB, 0.57246E+00_JPRB, 0.59815E+00_JPRB, 0.65736E+00_JPRB, 0.76138E+00_JPRB ,&
&  0.99963E+00_JPRB, 0.15478E+01_JPRB, 0.20147E+01_JPRB, 0.19025E+01_JPRB, 0.19194E+01_JPRB, 0.19812E+01_JPRB, 0.23381E+01_JPRB ,&
&  0.25692E+01_JPRB, 0.26348E+01_JPRB, 0.24030E+01_JPRB, 0.20785E+01_JPRB, 0.17843E+01_JPRB, 0.15562E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.10951E+01_JPRB, 0.99324E+00_JPRB, 0.92174E+00_JPRB, 0.87589E+00_JPRB, 0.84673E+00_JPRB, 0.83541E+00_JPRB ,&
&  0.85108E+00_JPRB, 0.92006E+00_JPRB, 0.10812E+01_JPRB, 0.14102E+01_JPRB, 0.21164E+01_JPRB, 0.21237E+01_JPRB, 0.26126E+01_JPRB ,&
&  0.35248E+01_JPRB, 0.34521E+01_JPRB, 0.31189E+01_JPRB, 0.26929E+01_JPRB, 0.23303E+01_JPRB, 0.20272E+01_JPRB, 0.17746E+01_JPRB ,&
&  0.15891E+01_JPRB, 0.14542E+01_JPRB, 0.13391E+01_JPRB, 0.12537E+01_JPRB, 0.11990E+01_JPRB, 0.11759E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.11285E+01_JPRB, 0.10965E+01_JPRB, 0.10735E+01_JPRB, 0.10618E+01_JPRB, 0.10497E+01_JPRB, 0.10403E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.98124E+00_JPRB, 0.94951E+00_JPRB, 0.92923E+00_JPRB, 0.89434E+00_JPRB, 0.88247E+00_JPRB, 0.86790E+00_JPRB, 0.84278E+00_JPRB ,&
&  0.82038E+00_JPRB, 0.82099E+00_JPRB, 0.83748E+00_JPRB, 0.85471E+00_JPRB, 0.88852E+00_JPRB, 0.91098E+00_JPRB, 0.85741E+00_JPRB ,&
&  0.83395E+00_JPRB, 0.80213E+00_JPRB, 0.77575E+00_JPRB, 0.76435E+00_JPRB, 0.76776E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.74115E+00_JPRB, 0.72718E+00_JPRB, 0.71373E+00_JPRB, 0.69998E+00_JPRB, 0.68428E+00_JPRB, 0.67265E+00_JPRB, 0.66274E+00_JPRB ,&
&  0.65580E+00_JPRB, 0.65209E+00_JPRB, 0.65291E+00_JPRB, 0.65362E+00_JPRB, 0.64975E+00_JPRB, 0.63899E+00_JPRB, 0.62130E+00_JPRB ,&
&  0.60966E+00_JPRB, 0.59892E+00_JPRB, 0.58561E+00_JPRB, 0.56936E+00_JPRB, 0.55364E+00_JPRB, 0.54570E+00_JPRB, 0.54380E+00_JPRB ,&
&  0.54779E+00_JPRB, 0.55235E+00_JPRB, 0.55583E+00_JPRB, 0.56771E+00_JPRB, 0.59168E+00_JPRB, 0.64762E+00_JPRB, 0.73259E+00_JPRB ,&
&  0.90373E+00_JPRB, 0.14394E+01_JPRB, 0.15898E+01_JPRB, 0.16151E+01_JPRB, 0.16729E+01_JPRB, 0.16854E+01_JPRB, 0.18507E+01_JPRB ,&
&  0.20082E+01_JPRB, 0.21044E+01_JPRB, 0.20405E+01_JPRB, 0.18119E+01_JPRB, 0.16064E+01_JPRB, 0.14653E+01_JPRB, 0.13253E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.10719E+01_JPRB, 0.98045E+00_JPRB, 0.91837E+00_JPRB, 0.87039E+00_JPRB, 0.83383E+00_JPRB, 0.81192E+00_JPRB ,&
&  0.80494E+00_JPRB, 0.81391E+00_JPRB, 0.86101E+00_JPRB, 0.97633E+00_JPRB, 0.14391E+01_JPRB, 0.18540E+01_JPRB, 0.22167E+01_JPRB ,&
&  0.23626E+01_JPRB, 0.24302E+01_JPRB, 0.24504E+01_JPRB, 0.22951E+01_JPRB, 0.20898E+01_JPRB, 0.18556E+01_JPRB, 0.16850E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.14420E+01_JPRB, 0.13272E+01_JPRB, 0.12222E+01_JPRB, 0.11468E+01_JPRB, 0.11046E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.10375E+01_JPRB, 0.10216E+01_JPRB, 0.10071E+01_JPRB, 0.99002E+00_JPRB, 0.96809E+00_JPRB, 0.93999E+00_JPRB ,&
&  0.90739E+00_JPRB, 0.87535E+00_JPRB, 0.85685E+00_JPRB, 0.84278E+00_JPRB, 0.83190E+00_JPRB, 0.81633E+00_JPRB, 0.79765E+00_JPRB ,&
&  0.79249E+00_JPRB, 0.80858E+00_JPRB, 0.81770E+00_JPRB, 0.83605E+00_JPRB, 0.85534E+00_JPRB, 0.87082E+00_JPRB, 0.83479E+00_JPRB ,&
&  0.80231E+00_JPRB, 0.77126E+00_JPRB, 0.74836E+00_JPRB, 0.73685E+00_JPRB, 0.74995E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.72759E+00_JPRB, 0.70855E+00_JPRB, 0.68906E+00_JPRB, 0.66753E+00_JPRB, 0.64989E+00_JPRB, 0.63913E+00_JPRB, 0.63158E+00_JPRB ,&
&  0.62742E+00_JPRB, 0.62590E+00_JPRB, 0.62458E+00_JPRB, 0.62264E+00_JPRB, 0.61948E+00_JPRB, 0.61514E+00_JPRB, 0.60626E+00_JPRB ,&
&  0.59507E+00_JPRB, 0.58119E+00_JPRB, 0.56952E+00_JPRB, 0.55981E+00_JPRB, 0.54266E+00_JPRB, 0.53129E+00_JPRB, 0.53145E+00_JPRB ,&
&  0.53590E+00_JPRB, 0.54074E+00_JPRB, 0.54531E+00_JPRB, 0.55563E+00_JPRB, 0.58290E+00_JPRB, 0.63989E+00_JPRB, 0.70680E+00_JPRB ,&
&  0.84969E+00_JPRB, 0.10429E+01_JPRB, 0.12170E+01_JPRB, 0.12584E+01_JPRB, 0.12909E+01_JPRB, 0.13616E+01_JPRB, 0.14581E+01_JPRB ,&
&  0.15171E+01_JPRB, 0.16464E+01_JPRB, 0.16206E+01_JPRB, 0.14730E+01_JPRB, 0.13142E+01_JPRB, 0.11968E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.10089E+01_JPRB, 0.96260E+00_JPRB, 0.92799E+00_JPRB, 0.89490E+00_JPRB, 0.86110E+00_JPRB, 0.82825E+00_JPRB ,&
&  0.80531E+00_JPRB, 0.79048E+00_JPRB, 0.78830E+00_JPRB, 0.81393E+00_JPRB, 0.92383E+00_JPRB, 0.11339E+01_JPRB, 0.14201E+01_JPRB ,&
&  0.16513E+01_JPRB, 0.17850E+01_JPRB, 0.18916E+01_JPRB, 0.18873E+01_JPRB, 0.18074E+01_JPRB, 0.17046E+01_JPRB, 0.15967E+01_JPRB ,&
&  0.14580E+01_JPRB, 0.12981E+01_JPRB, 0.11768E+01_JPRB, 0.10891E+01_JPRB, 0.10275E+01_JPRB, 0.98750E+00_JPRB, 0.96310E+00_JPRB ,&
&  0.94456E+00_JPRB, 0.93950E+00_JPRB, 0.95726E+00_JPRB, 0.97490E+00_JPRB, 0.97225E+00_JPRB, 0.94004E+00_JPRB, 0.89422E+00_JPRB ,&
&  0.85287E+00_JPRB, 0.82027E+00_JPRB, 0.79549E+00_JPRB, 0.77616E+00_JPRB, 0.76100E+00_JPRB, 0.74956E+00_JPRB, 0.73683E+00_JPRB ,&
&  0.72913E+00_JPRB, 0.74525E+00_JPRB, 0.75331E+00_JPRB, 0.79365E+00_JPRB, 0.83439E+00_JPRB, 0.80911E+00_JPRB, 0.77385E+00_JPRB ,&
&  0.74653E+00_JPRB, 0.72875E+00_JPRB, 0.71197E+00_JPRB, 0.70015E+00_JPRB, 0.74580E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.64132E+00_JPRB, 0.63615E+00_JPRB, 0.62434E+00_JPRB, 0.61264E+00_JPRB, 0.60351E+00_JPRB, 0.59918E+00_JPRB, 0.59536E+00_JPRB ,&
&  0.59305E+00_JPRB, 0.59186E+00_JPRB, 0.59118E+00_JPRB, 0.58843E+00_JPRB, 0.58309E+00_JPRB, 0.57511E+00_JPRB, 0.56517E+00_JPRB ,&
&  0.55589E+00_JPRB, 0.54662E+00_JPRB, 0.53828E+00_JPRB, 0.52964E+00_JPRB, 0.51866E+00_JPRB, 0.51201E+00_JPRB, 0.51406E+00_JPRB ,&
&  0.52033E+00_JPRB, 0.52755E+00_JPRB, 0.53455E+00_JPRB, 0.54387E+00_JPRB, 0.57732E+00_JPRB, 0.62899E+00_JPRB, 0.67674E+00_JPRB ,&
&  0.80024E+00_JPRB, 0.85698E+00_JPRB, 0.93844E+00_JPRB, 0.10206E+01_JPRB, 0.11110E+01_JPRB, 0.11422E+01_JPRB, 0.11666E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.12460E+01_JPRB, 0.12468E+01_JPRB, 0.11906E+01_JPRB, 0.10780E+01_JPRB, 0.96561E+00_JPRB, 0.89471E+00_JPRB ,&
&  0.85907E+00_JPRB, 0.84172E+00_JPRB, 0.82572E+00_JPRB, 0.81023E+00_JPRB, 0.79683E+00_JPRB, 0.78341E+00_JPRB, 0.76977E+00_JPRB ,&
&  0.75679E+00_JPRB, 0.74651E+00_JPRB, 0.74024E+00_JPRB, 0.73880E+00_JPRB, 0.75529E+00_JPRB, 0.81912E+00_JPRB, 0.96263E+00_JPRB ,&
&  0.11630E+01_JPRB, 0.13514E+01_JPRB, 0.14548E+01_JPRB, 0.15160E+01_JPRB, 0.15496E+01_JPRB, 0.14871E+01_JPRB, 0.13370E+01_JPRB ,&
&  0.11951E+01_JPRB, 0.10705E+01_JPRB, 0.97700E+00_JPRB, 0.90840E+00_JPRB, 0.86043E+00_JPRB, 0.82945E+00_JPRB, 0.81235E+00_JPRB ,&
&  0.80547E+00_JPRB, 0.80489E+00_JPRB, 0.81122E+00_JPRB, 0.81786E+00_JPRB, 0.82032E+00_JPRB, 0.81297E+00_JPRB, 0.79628E+00_JPRB ,&
&  0.77251E+00_JPRB, 0.75458E+00_JPRB, 0.73755E+00_JPRB, 0.71865E+00_JPRB, 0.70172E+00_JPRB, 0.68562E+00_JPRB, 0.67274E+00_JPRB ,&
&  0.66463E+00_JPRB, 0.67030E+00_JPRB, 0.68535E+00_JPRB, 0.72307E+00_JPRB, 0.71283E+00_JPRB, 0.69591E+00_JPRB, 0.67831E+00_JPRB ,&
&  0.66884E+00_JPRB, 0.66426E+00_JPRB, 0.65249E+00_JPRB, 0.64282E+00_JPRB, 0.64035E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.59483E+00_JPRB, 0.58967E+00_JPRB, 0.57974E+00_JPRB, 0.57076E+00_JPRB, 0.56693E+00_JPRB, 0.56635E+00_JPRB, 0.56699E+00_JPRB ,&
&  0.56553E+00_JPRB, 0.56181E+00_JPRB, 0.55681E+00_JPRB, 0.55318E+00_JPRB, 0.54818E+00_JPRB, 0.54130E+00_JPRB, 0.53274E+00_JPRB ,&
&  0.52445E+00_JPRB, 0.51686E+00_JPRB, 0.51081E+00_JPRB, 0.50445E+00_JPRB, 0.49961E+00_JPRB, 0.49905E+00_JPRB, 0.50072E+00_JPRB ,&
&  0.50741E+00_JPRB, 0.51629E+00_JPRB, 0.52450E+00_JPRB, 0.54283E+00_JPRB, 0.56790E+00_JPRB, 0.60798E+00_JPRB, 0.65354E+00_JPRB ,&
&  0.75503E+00_JPRB, 0.71118E+00_JPRB, 0.76960E+00_JPRB, 0.88943E+00_JPRB, 0.97075E+00_JPRB, 0.99638E+00_JPRB, 0.96381E+00_JPRB ,&
&  0.94638E+00_JPRB, 0.95264E+00_JPRB, 0.95671E+00_JPRB, 0.95317E+00_JPRB, 0.91933E+00_JPRB, 0.84193E+00_JPRB, 0.76199E+00_JPRB ,&
&  0.70632E+00_JPRB, 0.68524E+00_JPRB, 0.67516E+00_JPRB, 0.66141E+00_JPRB, 0.64743E+00_JPRB, 0.63911E+00_JPRB, 0.63677E+00_JPRB ,&
&  0.63499E+00_JPRB, 0.63264E+00_JPRB, 0.62885E+00_JPRB, 0.62446E+00_JPRB, 0.62332E+00_JPRB, 0.63824E+00_JPRB, 0.69371E+00_JPRB ,&
&  0.80541E+00_JPRB, 0.93600E+00_JPRB, 0.10582E+01_JPRB, 0.11533E+01_JPRB, 0.11743E+01_JPRB, 0.11437E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.93395E+00_JPRB, 0.87380E+00_JPRB, 0.82507E+00_JPRB, 0.78562E+00_JPRB, 0.75439E+00_JPRB, 0.73112E+00_JPRB ,&
&  0.71798E+00_JPRB, 0.70891E+00_JPRB, 0.70218E+00_JPRB, 0.69358E+00_JPRB, 0.68533E+00_JPRB, 0.67964E+00_JPRB, 0.67689E+00_JPRB ,&
&  0.67173E+00_JPRB, 0.66546E+00_JPRB, 0.66017E+00_JPRB, 0.65469E+00_JPRB, 0.64844E+00_JPRB, 0.64170E+00_JPRB, 0.63535E+00_JPRB ,&
&  0.63120E+00_JPRB, 0.63001E+00_JPRB, 0.62972E+00_JPRB, 0.64416E+00_JPRB, 0.65400E+00_JPRB, 0.63915E+00_JPRB, 0.62671E+00_JPRB ,&
&  0.62055E+00_JPRB, 0.61221E+00_JPRB, 0.60712E+00_JPRB, 0.56653E+00_JPRB, 0.59329E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.55119E+00_JPRB, 0.55223E+00_JPRB, 0.54949E+00_JPRB, 0.54238E+00_JPRB, 0.53739E+00_JPRB, 0.53443E+00_JPRB, 0.53345E+00_JPRB ,&
&  0.53335E+00_JPRB, 0.53226E+00_JPRB, 0.52901E+00_JPRB, 0.52304E+00_JPRB, 0.51775E+00_JPRB, 0.51129E+00_JPRB, 0.50457E+00_JPRB ,&
&  0.49874E+00_JPRB, 0.49357E+00_JPRB, 0.49042E+00_JPRB, 0.48665E+00_JPRB, 0.48534E+00_JPRB, 0.48690E+00_JPRB, 0.48901E+00_JPRB ,&
&  0.49242E+00_JPRB, 0.49583E+00_JPRB, 0.50704E+00_JPRB, 0.52552E+00_JPRB, 0.55763E+00_JPRB, 0.57980E+00_JPRB, 0.62182E+00_JPRB ,&
&  0.71501E+00_JPRB, 0.62905E+00_JPRB, 0.69991E+00_JPRB, 0.79054E+00_JPRB, 0.85056E+00_JPRB, 0.87845E+00_JPRB, 0.82485E+00_JPRB ,&
&  0.78239E+00_JPRB, 0.75946E+00_JPRB, 0.75156E+00_JPRB, 0.75057E+00_JPRB, 0.74953E+00_JPRB, 0.72833E+00_JPRB, 0.70040E+00_JPRB ,&
&  0.66996E+00_JPRB, 0.64429E+00_JPRB, 0.62182E+00_JPRB, 0.59805E+00_JPRB, 0.57648E+00_JPRB, 0.56222E+00_JPRB, 0.55503E+00_JPRB ,&
&  0.55281E+00_JPRB, 0.55246E+00_JPRB, 0.55185E+00_JPRB, 0.55214E+00_JPRB, 0.55405E+00_JPRB, 0.55841E+00_JPRB, 0.56857E+00_JPRB ,&
&  0.60189E+00_JPRB, 0.66291E+00_JPRB, 0.73274E+00_JPRB, 0.79991E+00_JPRB, 0.84778E+00_JPRB, 0.86920E+00_JPRB, 0.87641E+00_JPRB ,&
&  0.86258E+00_JPRB, 0.83281E+00_JPRB, 0.79624E+00_JPRB, 0.75976E+00_JPRB, 0.72738E+00_JPRB, 0.70081E+00_JPRB, 0.67715E+00_JPRB ,&
&  0.65617E+00_JPRB, 0.63574E+00_JPRB, 0.61865E+00_JPRB, 0.60586E+00_JPRB, 0.59428E+00_JPRB, 0.58428E+00_JPRB, 0.57847E+00_JPRB ,&
&  0.57592E+00_JPRB, 0.57525E+00_JPRB, 0.57481E+00_JPRB, 0.57663E+00_JPRB, 0.57909E+00_JPRB, 0.58108E+00_JPRB, 0.58092E+00_JPRB ,&
&  0.57766E+00_JPRB, 0.57665E+00_JPRB, 0.57448E+00_JPRB, 0.58518E+00_JPRB, 0.58505E+00_JPRB, 0.58212E+00_JPRB, 0.57751E+00_JPRB ,&
&  0.57426E+00_JPRB, 0.56535E+00_JPRB, 0.51044E+00_JPRB, 0.53579E+00_JPRB, 0.54535E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.52160E+00_JPRB, 0.52102E+00_JPRB, 0.52179E+00_JPRB, 0.52101E+00_JPRB, 0.51948E+00_JPRB, 0.51732E+00_JPRB, 0.51399E+00_JPRB ,&
&  0.50929E+00_JPRB, 0.50612E+00_JPRB, 0.50332E+00_JPRB, 0.49942E+00_JPRB, 0.49511E+00_JPRB, 0.49153E+00_JPRB, 0.48835E+00_JPRB ,&
&  0.48495E+00_JPRB, 0.48113E+00_JPRB, 0.47830E+00_JPRB, 0.47626E+00_JPRB, 0.47416E+00_JPRB, 0.47336E+00_JPRB, 0.47438E+00_JPRB ,&
&  0.47424E+00_JPRB, 0.47762E+00_JPRB, 0.49152E+00_JPRB, 0.51136E+00_JPRB, 0.53877E+00_JPRB, 0.56439E+00_JPRB, 0.58953E+00_JPRB ,&
&  0.59122E+00_JPRB, 0.54002E+00_JPRB, 0.59185E+00_JPRB, 0.65977E+00_JPRB, 0.72819E+00_JPRB, 0.77239E+00_JPRB, 0.73879E+00_JPRB ,&
&  0.68869E+00_JPRB, 0.64734E+00_JPRB, 0.62453E+00_JPRB, 0.61360E+00_JPRB, 0.61519E+00_JPRB, 0.61576E+00_JPRB, 0.61596E+00_JPRB ,&
&  0.60971E+00_JPRB, 0.59431E+00_JPRB, 0.57580E+00_JPRB, 0.55765E+00_JPRB, 0.54380E+00_JPRB, 0.53295E+00_JPRB, 0.52399E+00_JPRB ,&
&  0.51878E+00_JPRB, 0.51668E+00_JPRB, 0.51571E+00_JPRB, 0.51543E+00_JPRB, 0.51614E+00_JPRB, 0.51910E+00_JPRB, 0.52264E+00_JPRB ,&
&  0.52728E+00_JPRB, 0.54050E+00_JPRB, 0.56981E+00_JPRB, 0.60669E+00_JPRB, 0.64425E+00_JPRB, 0.67287E+00_JPRB, 0.69265E+00_JPRB ,&
&  0.70628E+00_JPRB, 0.71112E+00_JPRB, 0.70417E+00_JPRB, 0.68599E+00_JPRB, 0.66272E+00_JPRB, 0.64026E+00_JPRB, 0.61877E+00_JPRB ,&
&  0.59749E+00_JPRB, 0.57422E+00_JPRB, 0.55185E+00_JPRB, 0.53749E+00_JPRB, 0.52937E+00_JPRB, 0.52189E+00_JPRB, 0.51592E+00_JPRB ,&
&  0.51337E+00_JPRB, 0.51340E+00_JPRB, 0.51474E+00_JPRB, 0.51637E+00_JPRB, 0.51838E+00_JPRB, 0.52149E+00_JPRB, 0.52406E+00_JPRB ,&
&  0.52558E+00_JPRB, 0.52900E+00_JPRB, 0.53403E+00_JPRB, 0.54624E+00_JPRB, 0.55394E+00_JPRB, 0.55617E+00_JPRB, 0.55375E+00_JPRB ,&
&  0.54634E+00_JPRB, 0.53651E+00_JPRB, 0.52643E+00_JPRB, 0.52038E+00_JPRB, 0.52010E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.49943E+00_JPRB, 0.49856E+00_JPRB, 0.49808E+00_JPRB, 0.49643E+00_JPRB, 0.49491E+00_JPRB, 0.49346E+00_JPRB, 0.48973E+00_JPRB ,&
&  0.48824E+00_JPRB, 0.48421E+00_JPRB, 0.48037E+00_JPRB, 0.47835E+00_JPRB, 0.47588E+00_JPRB, 0.47293E+00_JPRB, 0.47085E+00_JPRB ,&
&  0.46854E+00_JPRB, 0.46565E+00_JPRB, 0.46332E+00_JPRB, 0.46032E+00_JPRB, 0.45606E+00_JPRB, 0.45302E+00_JPRB, 0.45332E+00_JPRB ,&
&  0.45380E+00_JPRB, 0.45680E+00_JPRB, 0.46853E+00_JPRB, 0.48102E+00_JPRB, 0.49777E+00_JPRB, 0.51625E+00_JPRB, 0.53184E+00_JPRB ,&
&  0.56031E+00_JPRB, 0.49806E+00_JPRB, 0.54705E+00_JPRB, 0.58741E+00_JPRB, 0.64552E+00_JPRB, 0.68895E+00_JPRB, 0.67431E+00_JPRB ,&
&  0.63182E+00_JPRB, 0.58609E+00_JPRB, 0.55827E+00_JPRB, 0.54572E+00_JPRB, 0.55129E+00_JPRB, 0.55820E+00_JPRB, 0.55724E+00_JPRB ,&
&  0.54972E+00_JPRB, 0.53365E+00_JPRB, 0.51518E+00_JPRB, 0.50356E+00_JPRB, 0.49712E+00_JPRB, 0.49184E+00_JPRB, 0.48666E+00_JPRB ,&
&  0.48267E+00_JPRB, 0.48062E+00_JPRB, 0.47945E+00_JPRB, 0.47879E+00_JPRB, 0.47921E+00_JPRB, 0.48089E+00_JPRB, 0.48327E+00_JPRB ,&
&  0.48485E+00_JPRB, 0.48498E+00_JPRB, 0.48818E+00_JPRB, 0.49914E+00_JPRB, 0.51551E+00_JPRB, 0.53221E+00_JPRB, 0.54581E+00_JPRB ,&
&  0.55677E+00_JPRB, 0.56628E+00_JPRB, 0.57202E+00_JPRB, 0.57088E+00_JPRB, 0.56282E+00_JPRB, 0.55062E+00_JPRB, 0.53739E+00_JPRB ,&
&  0.52349E+00_JPRB, 0.50917E+00_JPRB, 0.49330E+00_JPRB, 0.48352E+00_JPRB, 0.47887E+00_JPRB, 0.47623E+00_JPRB, 0.47442E+00_JPRB ,&
&  0.47332E+00_JPRB, 0.47273E+00_JPRB, 0.47332E+00_JPRB, 0.47431E+00_JPRB, 0.47490E+00_JPRB, 0.47570E+00_JPRB, 0.47764E+00_JPRB ,&
&  0.48139E+00_JPRB, 0.48779E+00_JPRB, 0.49600E+00_JPRB, 0.50817E+00_JPRB, 0.51872E+00_JPRB, 0.52474E+00_JPRB, 0.52710E+00_JPRB ,&
&  0.52411E+00_JPRB, 0.51773E+00_JPRB, 0.50944E+00_JPRB, 0.50207E+00_JPRB, 0.50044E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.48276E+00_JPRB, 0.47855E+00_JPRB, 0.47568E+00_JPRB, 0.47136E+00_JPRB, 0.46799E+00_JPRB, 0.46548E+00_JPRB, 0.46291E+00_JPRB ,&
&  0.46122E+00_JPRB, 0.46159E+00_JPRB, 0.46032E+00_JPRB, 0.45944E+00_JPRB, 0.45797E+00_JPRB, 0.45461E+00_JPRB, 0.45071E+00_JPRB ,&
&  0.44857E+00_JPRB, 0.44667E+00_JPRB, 0.44493E+00_JPRB, 0.44283E+00_JPRB, 0.44043E+00_JPRB, 0.43727E+00_JPRB, 0.43680E+00_JPRB ,&
&  0.43711E+00_JPRB, 0.43997E+00_JPRB, 0.44538E+00_JPRB, 0.45174E+00_JPRB, 0.46291E+00_JPRB, 0.47890E+00_JPRB, 0.49434E+00_JPRB ,&
&  0.51418E+00_JPRB, 0.50187E+00_JPRB, 0.53398E+00_JPRB, 0.55308E+00_JPRB, 0.58610E+00_JPRB, 0.61442E+00_JPRB, 0.61640E+00_JPRB ,&
&  0.60063E+00_JPRB, 0.57105E+00_JPRB, 0.54764E+00_JPRB, 0.53577E+00_JPRB, 0.53583E+00_JPRB, 0.53389E+00_JPRB, 0.52247E+00_JPRB ,&
&  0.50494E+00_JPRB, 0.48475E+00_JPRB, 0.46967E+00_JPRB, 0.46145E+00_JPRB, 0.45776E+00_JPRB, 0.45566E+00_JPRB, 0.45241E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.44563E+00_JPRB, 0.44379E+00_JPRB, 0.44281E+00_JPRB, 0.44199E+00_JPRB, 0.44188E+00_JPRB, 0.44247E+00_JPRB ,&
&  0.44307E+00_JPRB, 0.44316E+00_JPRB, 0.44279E+00_JPRB, 0.44383E+00_JPRB, 0.44794E+00_JPRB, 0.45457E+00_JPRB, 0.46138E+00_JPRB ,&
&  0.46622E+00_JPRB, 0.47103E+00_JPRB, 0.47525E+00_JPRB, 0.47817E+00_JPRB, 0.47718E+00_JPRB, 0.47170E+00_JPRB, 0.46410E+00_JPRB ,&
&  0.45737E+00_JPRB, 0.45128E+00_JPRB, 0.44542E+00_JPRB, 0.44086E+00_JPRB, 0.43846E+00_JPRB, 0.43820E+00_JPRB, 0.43874E+00_JPRB ,&
&  0.43941E+00_JPRB, 0.44023E+00_JPRB, 0.44139E+00_JPRB, 0.44302E+00_JPRB, 0.44384E+00_JPRB, 0.44477E+00_JPRB, 0.44601E+00_JPRB ,&
&  0.44994E+00_JPRB, 0.45506E+00_JPRB, 0.46242E+00_JPRB, 0.47363E+00_JPRB, 0.48436E+00_JPRB, 0.49197E+00_JPRB, 0.49784E+00_JPRB ,&
&  0.50005E+00_JPRB, 0.49896E+00_JPRB, 0.49582E+00_JPRB, 0.49205E+00_JPRB, 0.48794E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.45432E+00_JPRB, 0.44832E+00_JPRB, 0.44408E+00_JPRB, 0.44105E+00_JPRB, 0.44062E+00_JPRB, 0.43894E+00_JPRB, 0.43529E+00_JPRB ,&
&  0.43251E+00_JPRB, 0.43398E+00_JPRB, 0.43718E+00_JPRB, 0.43863E+00_JPRB, 0.43781E+00_JPRB, 0.43587E+00_JPRB, 0.43233E+00_JPRB ,&
&  0.42773E+00_JPRB, 0.42537E+00_JPRB, 0.42512E+00_JPRB, 0.42616E+00_JPRB, 0.42566E+00_JPRB, 0.42309E+00_JPRB, 0.42080E+00_JPRB ,&
&  0.42077E+00_JPRB, 0.42239E+00_JPRB, 0.42404E+00_JPRB, 0.42709E+00_JPRB, 0.43523E+00_JPRB, 0.45049E+00_JPRB, 0.46407E+00_JPRB ,&
&  0.47454E+00_JPRB, 0.48007E+00_JPRB, 0.47569E+00_JPRB, 0.49850E+00_JPRB, 0.51730E+00_JPRB, 0.54136E+00_JPRB, 0.55889E+00_JPRB ,&
&  0.56419E+00_JPRB, 0.55707E+00_JPRB, 0.54059E+00_JPRB, 0.52658E+00_JPRB, 0.51614E+00_JPRB, 0.50140E+00_JPRB, 0.47934E+00_JPRB ,&
&  0.45673E+00_JPRB, 0.43880E+00_JPRB, 0.42792E+00_JPRB, 0.42253E+00_JPRB, 0.42111E+00_JPRB, 0.42064E+00_JPRB, 0.41779E+00_JPRB ,&
&  0.41446E+00_JPRB, 0.41138E+00_JPRB, 0.40995E+00_JPRB, 0.40879E+00_JPRB, 0.40745E+00_JPRB, 0.40636E+00_JPRB, 0.40589E+00_JPRB ,&
&  0.40582E+00_JPRB, 0.40570E+00_JPRB, 0.40581E+00_JPRB, 0.40622E+00_JPRB, 0.40638E+00_JPRB, 0.40705E+00_JPRB, 0.40908E+00_JPRB ,&
&  0.41133E+00_JPRB, 0.41333E+00_JPRB, 0.41504E+00_JPRB, 0.41618E+00_JPRB, 0.41652E+00_JPRB, 0.41568E+00_JPRB, 0.41284E+00_JPRB ,&
&  0.40926E+00_JPRB, 0.40746E+00_JPRB, 0.40641E+00_JPRB, 0.40464E+00_JPRB, 0.40294E+00_JPRB, 0.40294E+00_JPRB, 0.40444E+00_JPRB ,&
&  0.40620E+00_JPRB, 0.40853E+00_JPRB, 0.41062E+00_JPRB, 0.41247E+00_JPRB, 0.41322E+00_JPRB, 0.41432E+00_JPRB, 0.41561E+00_JPRB ,&
&  0.41780E+00_JPRB, 0.42121E+00_JPRB, 0.42608E+00_JPRB, 0.43262E+00_JPRB, 0.43986E+00_JPRB, 0.44582E+00_JPRB, 0.45201E+00_JPRB ,&
&  0.45764E+00_JPRB, 0.46090E+00_JPRB, 0.46176E+00_JPRB, 0.46193E+00_JPRB, 0.45971E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.42099E+00_JPRB, 0.41700E+00_JPRB, 0.41411E+00_JPRB, 0.41345E+00_JPRB, 0.41497E+00_JPRB, 0.41459E+00_JPRB, 0.41175E+00_JPRB ,&
&  0.40928E+00_JPRB, 0.41062E+00_JPRB, 0.41457E+00_JPRB, 0.41723E+00_JPRB, 0.41856E+00_JPRB, 0.41949E+00_JPRB, 0.41776E+00_JPRB ,&
&  0.41320E+00_JPRB, 0.40899E+00_JPRB, 0.40944E+00_JPRB, 0.41156E+00_JPRB, 0.41235E+00_JPRB, 0.41003E+00_JPRB, 0.40738E+00_JPRB ,&
&  0.40704E+00_JPRB, 0.40716E+00_JPRB, 0.40962E+00_JPRB, 0.41217E+00_JPRB, 0.41884E+00_JPRB, 0.43142E+00_JPRB, 0.44274E+00_JPRB ,&
&  0.45051E+00_JPRB, 0.45030E+00_JPRB, 0.44965E+00_JPRB, 0.45603E+00_JPRB, 0.46412E+00_JPRB, 0.48144E+00_JPRB, 0.50094E+00_JPRB ,&
&  0.51216E+00_JPRB, 0.51229E+00_JPRB, 0.49887E+00_JPRB, 0.48273E+00_JPRB, 0.46746E+00_JPRB, 0.45020E+00_JPRB, 0.43298E+00_JPRB ,&
&  0.41727E+00_JPRB, 0.40633E+00_JPRB, 0.39998E+00_JPRB, 0.39694E+00_JPRB, 0.39551E+00_JPRB, 0.39391E+00_JPRB, 0.39046E+00_JPRB ,&
&  0.38630E+00_JPRB, 0.38300E+00_JPRB, 0.38063E+00_JPRB, 0.37902E+00_JPRB, 0.37762E+00_JPRB, 0.37694E+00_JPRB, 0.37608E+00_JPRB ,&
&  0.37565E+00_JPRB, 0.37596E+00_JPRB, 0.37663E+00_JPRB, 0.37631E+00_JPRB, 0.37567E+00_JPRB, 0.37454E+00_JPRB, 0.37429E+00_JPRB ,&
&  0.37515E+00_JPRB, 0.37494E+00_JPRB, 0.37489E+00_JPRB, 0.37607E+00_JPRB, 0.37697E+00_JPRB, 0.37783E+00_JPRB, 0.37766E+00_JPRB ,&
&  0.37691E+00_JPRB, 0.37670E+00_JPRB, 0.37684E+00_JPRB, 0.37654E+00_JPRB, 0.37608E+00_JPRB, 0.37598E+00_JPRB, 0.37648E+00_JPRB ,&
&  0.37844E+00_JPRB, 0.38085E+00_JPRB, 0.38279E+00_JPRB, 0.38462E+00_JPRB, 0.38643E+00_JPRB, 0.38883E+00_JPRB, 0.38964E+00_JPRB ,&
&  0.39007E+00_JPRB, 0.39154E+00_JPRB, 0.39489E+00_JPRB, 0.39941E+00_JPRB, 0.40395E+00_JPRB, 0.40746E+00_JPRB, 0.41130E+00_JPRB ,&
&  0.41505E+00_JPRB, 0.41829E+00_JPRB, 0.42047E+00_JPRB, 0.42131E+00_JPRB, 0.42180E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.38983E+00_JPRB, 0.38851E+00_JPRB, 0.38804E+00_JPRB, 0.38854E+00_JPRB, 0.39039E+00_JPRB, 0.39010E+00_JPRB, 0.38931E+00_JPRB ,&
&  0.38879E+00_JPRB, 0.39110E+00_JPRB, 0.39547E+00_JPRB, 0.39837E+00_JPRB, 0.39939E+00_JPRB, 0.39972E+00_JPRB, 0.39907E+00_JPRB ,&
&  0.39803E+00_JPRB, 0.39720E+00_JPRB, 0.39609E+00_JPRB, 0.39695E+00_JPRB, 0.39646E+00_JPRB, 0.39461E+00_JPRB, 0.39479E+00_JPRB ,&
&  0.39433E+00_JPRB, 0.39477E+00_JPRB, 0.39771E+00_JPRB, 0.40063E+00_JPRB, 0.40690E+00_JPRB, 0.41839E+00_JPRB, 0.42849E+00_JPRB ,&
&  0.43358E+00_JPRB, 0.42874E+00_JPRB, 0.42312E+00_JPRB, 0.42359E+00_JPRB, 0.42741E+00_JPRB, 0.43580E+00_JPRB, 0.44637E+00_JPRB ,&
&  0.45290E+00_JPRB, 0.44878E+00_JPRB, 0.43681E+00_JPRB, 0.42434E+00_JPRB, 0.41319E+00_JPRB, 0.40343E+00_JPRB, 0.39493E+00_JPRB ,&
&  0.38754E+00_JPRB, 0.38248E+00_JPRB, 0.38000E+00_JPRB, 0.37913E+00_JPRB, 0.37795E+00_JPRB, 0.37400E+00_JPRB, 0.36926E+00_JPRB ,&
&  0.36369E+00_JPRB, 0.35967E+00_JPRB, 0.35646E+00_JPRB, 0.35522E+00_JPRB, 0.35513E+00_JPRB, 0.35458E+00_JPRB, 0.35318E+00_JPRB ,&
&  0.35152E+00_JPRB, 0.35085E+00_JPRB, 0.35021E+00_JPRB, 0.34864E+00_JPRB, 0.34667E+00_JPRB, 0.34568E+00_JPRB, 0.34454E+00_JPRB ,&
&  0.34312E+00_JPRB, 0.34327E+00_JPRB, 0.34471E+00_JPRB, 0.34684E+00_JPRB, 0.34726E+00_JPRB, 0.34853E+00_JPRB, 0.35044E+00_JPRB ,&
&  0.34968E+00_JPRB, 0.34686E+00_JPRB, 0.34498E+00_JPRB, 0.34498E+00_JPRB, 0.34662E+00_JPRB, 0.34923E+00_JPRB, 0.35051E+00_JPRB ,&
&  0.35167E+00_JPRB, 0.35195E+00_JPRB, 0.35324E+00_JPRB, 0.35514E+00_JPRB, 0.35774E+00_JPRB, 0.36008E+00_JPRB, 0.36142E+00_JPRB ,&
&  0.36212E+00_JPRB, 0.36278E+00_JPRB, 0.36524E+00_JPRB, 0.36899E+00_JPRB, 0.37235E+00_JPRB, 0.37610E+00_JPRB, 0.37907E+00_JPRB ,&
&  0.38130E+00_JPRB, 0.38368E+00_JPRB, 0.38545E+00_JPRB, 0.38672E+00_JPRB, 0.38835E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.36935E+00_JPRB, 0.37025E+00_JPRB, 0.37174E+00_JPRB, 0.37236E+00_JPRB, 0.37354E+00_JPRB, 0.37397E+00_JPRB, 0.37511E+00_JPRB ,&
&  0.37574E+00_JPRB, 0.37775E+00_JPRB, 0.38047E+00_JPRB, 0.38071E+00_JPRB, 0.38052E+00_JPRB, 0.38049E+00_JPRB, 0.38131E+00_JPRB ,&
&  0.38372E+00_JPRB, 0.38457E+00_JPRB, 0.38410E+00_JPRB, 0.38437E+00_JPRB, 0.38280E+00_JPRB, 0.38118E+00_JPRB, 0.38264E+00_JPRB ,&
&  0.38275E+00_JPRB, 0.38340E+00_JPRB, 0.38605E+00_JPRB, 0.38893E+00_JPRB, 0.39491E+00_JPRB, 0.40308E+00_JPRB, 0.41221E+00_JPRB ,&
&  0.41806E+00_JPRB, 0.41514E+00_JPRB, 0.40676E+00_JPRB, 0.40082E+00_JPRB, 0.39958E+00_JPRB, 0.40093E+00_JPRB, 0.40424E+00_JPRB ,&
&  0.40416E+00_JPRB, 0.39806E+00_JPRB, 0.38924E+00_JPRB, 0.38366E+00_JPRB, 0.38051E+00_JPRB, 0.37586E+00_JPRB, 0.36959E+00_JPRB ,&
&  0.36466E+00_JPRB, 0.36261E+00_JPRB, 0.36241E+00_JPRB, 0.36123E+00_JPRB, 0.35863E+00_JPRB, 0.35431E+00_JPRB, 0.34903E+00_JPRB ,&
&  0.34423E+00_JPRB, 0.33959E+00_JPRB, 0.33651E+00_JPRB, 0.33457E+00_JPRB, 0.33426E+00_JPRB, 0.33409E+00_JPRB, 0.33272E+00_JPRB ,&
&  0.33017E+00_JPRB, 0.32838E+00_JPRB, 0.32616E+00_JPRB, 0.32246E+00_JPRB, 0.31983E+00_JPRB, 0.31894E+00_JPRB, 0.31784E+00_JPRB ,&
&  0.31638E+00_JPRB, 0.31635E+00_JPRB, 0.31672E+00_JPRB, 0.31670E+00_JPRB, 0.31810E+00_JPRB, 0.32027E+00_JPRB, 0.31794E+00_JPRB ,&
&  0.31555E+00_JPRB, 0.31392E+00_JPRB, 0.31405E+00_JPRB, 0.31605E+00_JPRB, 0.31931E+00_JPRB, 0.32275E+00_JPRB, 0.32549E+00_JPRB ,&
&  0.32662E+00_JPRB, 0.32690E+00_JPRB, 0.32728E+00_JPRB, 0.32777E+00_JPRB, 0.32972E+00_JPRB, 0.33324E+00_JPRB, 0.33710E+00_JPRB ,&
&  0.33879E+00_JPRB, 0.34152E+00_JPRB, 0.34470E+00_JPRB, 0.34876E+00_JPRB, 0.35129E+00_JPRB, 0.35381E+00_JPRB, 0.35616E+00_JPRB ,&
&  0.35792E+00_JPRB, 0.36007E+00_JPRB, 0.36288E+00_JPRB, 0.36511E+00_JPRB, 0.36736E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.35389E+00_JPRB, 0.35571E+00_JPRB, 0.35679E+00_JPRB, 0.35865E+00_JPRB, 0.36186E+00_JPRB, 0.36347E+00_JPRB, 0.36324E+00_JPRB ,&
&  0.36341E+00_JPRB, 0.36519E+00_JPRB, 0.36670E+00_JPRB, 0.36744E+00_JPRB, 0.36518E+00_JPRB, 0.36487E+00_JPRB, 0.36688E+00_JPRB ,&
&  0.36950E+00_JPRB, 0.37149E+00_JPRB, 0.37172E+00_JPRB, 0.37187E+00_JPRB, 0.37057E+00_JPRB, 0.36872E+00_JPRB, 0.36993E+00_JPRB ,&
&  0.37094E+00_JPRB, 0.37135E+00_JPRB, 0.37315E+00_JPRB, 0.37551E+00_JPRB, 0.38021E+00_JPRB, 0.38582E+00_JPRB, 0.39137E+00_JPRB ,&
&  0.39659E+00_JPRB, 0.39655E+00_JPRB, 0.38819E+00_JPRB, 0.37901E+00_JPRB, 0.37862E+00_JPRB, 0.37913E+00_JPRB, 0.37870E+00_JPRB ,&
&  0.37637E+00_JPRB, 0.37310E+00_JPRB, 0.37091E+00_JPRB, 0.36902E+00_JPRB, 0.36626E+00_JPRB, 0.36076E+00_JPRB, 0.35459E+00_JPRB ,&
&  0.34945E+00_JPRB, 0.34748E+00_JPRB, 0.34587E+00_JPRB, 0.34285E+00_JPRB, 0.33834E+00_JPRB, 0.33444E+00_JPRB, 0.33034E+00_JPRB ,&
&  0.32377E+00_JPRB, 0.31757E+00_JPRB, 0.31405E+00_JPRB, 0.31277E+00_JPRB, 0.31267E+00_JPRB, 0.31180E+00_JPRB, 0.30937E+00_JPRB ,&
&  0.30662E+00_JPRB, 0.30347E+00_JPRB, 0.30152E+00_JPRB, 0.30096E+00_JPRB, 0.30225E+00_JPRB, 0.30019E+00_JPRB, 0.29489E+00_JPRB ,&
&  0.29209E+00_JPRB, 0.29094E+00_JPRB, 0.29192E+00_JPRB, 0.29105E+00_JPRB, 0.29033E+00_JPRB, 0.28925E+00_JPRB, 0.28982E+00_JPRB ,&
&  0.29116E+00_JPRB, 0.29116E+00_JPRB, 0.29414E+00_JPRB, 0.29957E+00_JPRB, 0.30435E+00_JPRB, 0.30714E+00_JPRB, 0.30887E+00_JPRB ,&
&  0.30951E+00_JPRB, 0.30921E+00_JPRB, 0.30892E+00_JPRB, 0.30995E+00_JPRB, 0.31286E+00_JPRB, 0.31681E+00_JPRB, 0.32035E+00_JPRB ,&
&  0.32107E+00_JPRB, 0.32246E+00_JPRB, 0.32603E+00_JPRB, 0.33138E+00_JPRB, 0.33444E+00_JPRB, 0.33674E+00_JPRB, 0.33854E+00_JPRB ,&
&  0.33933E+00_JPRB, 0.34107E+00_JPRB, 0.34509E+00_JPRB, 0.34781E+00_JPRB, 0.35059E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.33790E+00_JPRB, 0.33895E+00_JPRB, 0.34113E+00_JPRB, 0.34381E+00_JPRB, 0.34525E+00_JPRB, 0.34483E+00_JPRB, 0.34373E+00_JPRB ,&
&  0.34420E+00_JPRB, 0.34692E+00_JPRB, 0.34984E+00_JPRB, 0.35097E+00_JPRB, 0.35161E+00_JPRB, 0.35227E+00_JPRB, 0.35434E+00_JPRB ,&
&  0.35720E+00_JPRB, 0.36051E+00_JPRB, 0.36062E+00_JPRB, 0.36056E+00_JPRB, 0.35965E+00_JPRB, 0.35818E+00_JPRB, 0.35928E+00_JPRB ,&
&  0.36235E+00_JPRB, 0.36233E+00_JPRB, 0.36212E+00_JPRB, 0.36171E+00_JPRB, 0.36437E+00_JPRB, 0.36906E+00_JPRB, 0.37279E+00_JPRB ,&
&  0.37399E+00_JPRB, 0.36817E+00_JPRB, 0.35934E+00_JPRB, 0.35922E+00_JPRB, 0.36092E+00_JPRB, 0.36246E+00_JPRB, 0.36237E+00_JPRB ,&
&  0.36179E+00_JPRB, 0.36065E+00_JPRB, 0.35921E+00_JPRB, 0.35749E+00_JPRB, 0.35489E+00_JPRB, 0.34896E+00_JPRB, 0.34225E+00_JPRB ,&
&  0.33689E+00_JPRB, 0.33267E+00_JPRB, 0.32876E+00_JPRB, 0.32317E+00_JPRB, 0.31756E+00_JPRB, 0.31346E+00_JPRB, 0.30852E+00_JPRB ,&
&  0.30265E+00_JPRB, 0.29877E+00_JPRB, 0.29744E+00_JPRB, 0.29701E+00_JPRB, 0.29671E+00_JPRB, 0.29549E+00_JPRB, 0.29391E+00_JPRB ,&
&  0.29130E+00_JPRB, 0.28929E+00_JPRB, 0.28741E+00_JPRB, 0.28602E+00_JPRB, 0.27850E+00_JPRB, 0.27088E+00_JPRB, 0.26502E+00_JPRB ,&
&  0.26484E+00_JPRB, 0.26728E+00_JPRB, 0.26969E+00_JPRB, 0.27113E+00_JPRB, 0.27437E+00_JPRB, 0.27547E+00_JPRB, 0.27471E+00_JPRB ,&
&  0.27057E+00_JPRB, 0.26860E+00_JPRB, 0.27063E+00_JPRB, 0.27239E+00_JPRB, 0.27330E+00_JPRB, 0.27525E+00_JPRB, 0.27684E+00_JPRB ,&
&  0.28008E+00_JPRB, 0.28204E+00_JPRB, 0.28456E+00_JPRB, 0.28751E+00_JPRB, 0.28981E+00_JPRB, 0.29068E+00_JPRB, 0.28861E+00_JPRB ,&
&  0.28914E+00_JPRB, 0.29589E+00_JPRB, 0.30691E+00_JPRB, 0.31615E+00_JPRB, 0.31940E+00_JPRB, 0.32225E+00_JPRB, 0.32458E+00_JPRB ,&
&  0.32599E+00_JPRB, 0.33023E+00_JPRB, 0.33376E+00_JPRB, 0.33566E+00_JPRB, 0.33671E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.32603E+00_JPRB, 0.32531E+00_JPRB, 0.32390E+00_JPRB, 0.32278E+00_JPRB, 0.32114E+00_JPRB, 0.31964E+00_JPRB, 0.32048E+00_JPRB ,&
&  0.32366E+00_JPRB, 0.32782E+00_JPRB, 0.33183E+00_JPRB, 0.33489E+00_JPRB, 0.33720E+00_JPRB, 0.33838E+00_JPRB, 0.34014E+00_JPRB ,&
&  0.34213E+00_JPRB, 0.34577E+00_JPRB, 0.34760E+00_JPRB, 0.34912E+00_JPRB, 0.34820E+00_JPRB, 0.34718E+00_JPRB, 0.34819E+00_JPRB ,&
&  0.35146E+00_JPRB, 0.35274E+00_JPRB, 0.35176E+00_JPRB, 0.34952E+00_JPRB, 0.35042E+00_JPRB, 0.35347E+00_JPRB, 0.35422E+00_JPRB ,&
&  0.34759E+00_JPRB, 0.33262E+00_JPRB, 0.32304E+00_JPRB, 0.32794E+00_JPRB, 0.33744E+00_JPRB, 0.34170E+00_JPRB, 0.34512E+00_JPRB ,&
&  0.34719E+00_JPRB, 0.34712E+00_JPRB, 0.34562E+00_JPRB, 0.34298E+00_JPRB, 0.33904E+00_JPRB, 0.33264E+00_JPRB, 0.32543E+00_JPRB ,&
&  0.31927E+00_JPRB, 0.31644E+00_JPRB, 0.31259E+00_JPRB, 0.30568E+00_JPRB, 0.29926E+00_JPRB, 0.29544E+00_JPRB, 0.29024E+00_JPRB ,&
&  0.28476E+00_JPRB, 0.28106E+00_JPRB, 0.28027E+00_JPRB, 0.27950E+00_JPRB, 0.27891E+00_JPRB, 0.27754E+00_JPRB, 0.27401E+00_JPRB ,&
&  0.27109E+00_JPRB, 0.26614E+00_JPRB, 0.26028E+00_JPRB, 0.25478E+00_JPRB, 0.25128E+00_JPRB, 0.24730E+00_JPRB, 0.24206E+00_JPRB ,&
&  0.23949E+00_JPRB, 0.24273E+00_JPRB, 0.25353E+00_JPRB, 0.26448E+00_JPRB, 0.26499E+00_JPRB, 0.25338E+00_JPRB, 0.25069E+00_JPRB ,&
&  0.25120E+00_JPRB, 0.25139E+00_JPRB, 0.25198E+00_JPRB, 0.25386E+00_JPRB, 0.25472E+00_JPRB, 0.25415E+00_JPRB, 0.25479E+00_JPRB ,&
&  0.25751E+00_JPRB, 0.26023E+00_JPRB, 0.26156E+00_JPRB, 0.26331E+00_JPRB, 0.26446E+00_JPRB, 0.26435E+00_JPRB, 0.26392E+00_JPRB ,&
&  0.26485E+00_JPRB, 0.26879E+00_JPRB, 0.27489E+00_JPRB, 0.28338E+00_JPRB, 0.28890E+00_JPRB, 0.29394E+00_JPRB, 0.29995E+00_JPRB ,&
&  0.30778E+00_JPRB, 0.31428E+00_JPRB, 0.31955E+00_JPRB, 0.32416E+00_JPRB, 0.32570E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.31458E+00_JPRB, 0.31562E+00_JPRB, 0.31107E+00_JPRB, 0.30730E+00_JPRB, 0.30535E+00_JPRB, 0.30492E+00_JPRB, 0.30671E+00_JPRB ,&
&  0.31073E+00_JPRB, 0.31406E+00_JPRB, 0.31726E+00_JPRB, 0.32010E+00_JPRB, 0.32135E+00_JPRB, 0.32101E+00_JPRB, 0.32221E+00_JPRB ,&
&  0.32345E+00_JPRB, 0.32677E+00_JPRB, 0.32938E+00_JPRB, 0.33313E+00_JPRB, 0.33345E+00_JPRB, 0.33419E+00_JPRB, 0.33552E+00_JPRB ,&
&  0.33495E+00_JPRB, 0.33493E+00_JPRB, 0.33623E+00_JPRB, 0.33349E+00_JPRB, 0.33193E+00_JPRB, 0.32963E+00_JPRB, 0.32829E+00_JPRB ,&
&  0.32113E+00_JPRB, 0.31329E+00_JPRB, 0.30684E+00_JPRB, 0.31211E+00_JPRB, 0.32007E+00_JPRB, 0.32118E+00_JPRB, 0.32623E+00_JPRB ,&
&  0.32818E+00_JPRB, 0.32605E+00_JPRB, 0.32399E+00_JPRB, 0.31996E+00_JPRB, 0.31628E+00_JPRB, 0.31200E+00_JPRB, 0.30850E+00_JPRB ,&
&  0.30443E+00_JPRB, 0.30514E+00_JPRB, 0.29964E+00_JPRB, 0.28403E+00_JPRB, 0.27696E+00_JPRB, 0.27246E+00_JPRB, 0.26429E+00_JPRB ,&
&  0.25561E+00_JPRB, 0.25392E+00_JPRB, 0.25372E+00_JPRB, 0.25031E+00_JPRB, 0.24976E+00_JPRB, 0.25006E+00_JPRB, 0.24983E+00_JPRB ,&
&  0.24800E+00_JPRB, 0.24766E+00_JPRB, 0.24617E+00_JPRB, 0.24388E+00_JPRB, 0.24150E+00_JPRB, 0.24049E+00_JPRB, 0.24093E+00_JPRB ,&
&  0.24262E+00_JPRB, 0.24403E+00_JPRB, 0.25023E+00_JPRB, 0.25360E+00_JPRB, 0.24671E+00_JPRB, 0.24500E+00_JPRB, 0.24712E+00_JPRB ,&
&  0.24221E+00_JPRB, 0.24143E+00_JPRB, 0.24148E+00_JPRB, 0.24147E+00_JPRB, 0.24191E+00_JPRB, 0.24291E+00_JPRB, 0.24250E+00_JPRB ,&
&  0.24324E+00_JPRB, 0.24511E+00_JPRB, 0.24570E+00_JPRB, 0.24580E+00_JPRB, 0.24542E+00_JPRB, 0.24599E+00_JPRB, 0.24755E+00_JPRB ,&
&  0.24923E+00_JPRB, 0.25260E+00_JPRB, 0.25606E+00_JPRB, 0.26124E+00_JPRB, 0.26533E+00_JPRB, 0.26863E+00_JPRB, 0.27491E+00_JPRB ,&
&  0.27978E+00_JPRB, 0.29090E+00_JPRB, 0.30894E+00_JPRB, 0.31651E+00_JPRB, 0.30931E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.30296E+00_JPRB, 0.30150E+00_JPRB, 0.29938E+00_JPRB, 0.29763E+00_JPRB, 0.29875E+00_JPRB, 0.30015E+00_JPRB, 0.30226E+00_JPRB ,&
&  0.30457E+00_JPRB, 0.30538E+00_JPRB, 0.30555E+00_JPRB, 0.30274E+00_JPRB, 0.30144E+00_JPRB, 0.29613E+00_JPRB, 0.29444E+00_JPRB ,&
&  0.29323E+00_JPRB, 0.29602E+00_JPRB, 0.29946E+00_JPRB, 0.30457E+00_JPRB, 0.30641E+00_JPRB, 0.30837E+00_JPRB, 0.30705E+00_JPRB ,&
&  0.30454E+00_JPRB, 0.30164E+00_JPRB, 0.29867E+00_JPRB, 0.29637E+00_JPRB, 0.29567E+00_JPRB, 0.29341E+00_JPRB, 0.29121E+00_JPRB ,&
&  0.29138E+00_JPRB, 0.29319E+00_JPRB, 0.29386E+00_JPRB, 0.29458E+00_JPRB, 0.30035E+00_JPRB, 0.30028E+00_JPRB, 0.30296E+00_JPRB ,&
&  0.30402E+00_JPRB, 0.30523E+00_JPRB, 0.30508E+00_JPRB, 0.30384E+00_JPRB, 0.30267E+00_JPRB, 0.29549E+00_JPRB, 0.29272E+00_JPRB ,&
&  0.28581E+00_JPRB, 0.28194E+00_JPRB, 0.27457E+00_JPRB, 0.26753E+00_JPRB, 0.25906E+00_JPRB, 0.25270E+00_JPRB, 0.24730E+00_JPRB ,&
&  0.24299E+00_JPRB, 0.23973E+00_JPRB, 0.23715E+00_JPRB, 0.23684E+00_JPRB, 0.23686E+00_JPRB, 0.23623E+00_JPRB, 0.23643E+00_JPRB ,&
&  0.23763E+00_JPRB, 0.23892E+00_JPRB, 0.23878E+00_JPRB, 0.23945E+00_JPRB, 0.24000E+00_JPRB, 0.24079E+00_JPRB, 0.24246E+00_JPRB ,&
&  0.24500E+00_JPRB, 0.24691E+00_JPRB, 0.24719E+00_JPRB, 0.24968E+00_JPRB, 0.24893E+00_JPRB, 0.24588E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.23950E+00_JPRB, 0.23798E+00_JPRB, 0.23612E+00_JPRB, 0.23501E+00_JPRB, 0.23478E+00_JPRB, 0.23487E+00_JPRB, 0.23576E+00_JPRB ,&
&  0.23674E+00_JPRB, 0.23781E+00_JPRB, 0.23817E+00_JPRB, 0.23864E+00_JPRB, 0.23901E+00_JPRB, 0.24000E+00_JPRB, 0.24203E+00_JPRB ,&
&  0.24488E+00_JPRB, 0.24816E+00_JPRB, 0.25135E+00_JPRB, 0.25519E+00_JPRB, 0.26149E+00_JPRB, 0.26954E+00_JPRB, 0.28161E+00_JPRB ,&
&  0.29006E+00_JPRB, 0.30022E+00_JPRB, 0.29748E+00_JPRB, 0.30237E+00_JPRB, 0.30390E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.29409E+00_JPRB, 0.29398E+00_JPRB, 0.29278E+00_JPRB, 0.29219E+00_JPRB, 0.29272E+00_JPRB, 0.29367E+00_JPRB, 0.29517E+00_JPRB ,&
&  0.29182E+00_JPRB, 0.28986E+00_JPRB, 0.28558E+00_JPRB, 0.28087E+00_JPRB, 0.27656E+00_JPRB, 0.27369E+00_JPRB, 0.27212E+00_JPRB ,&
&  0.27206E+00_JPRB, 0.27480E+00_JPRB, 0.27656E+00_JPRB, 0.27911E+00_JPRB, 0.27912E+00_JPRB, 0.27927E+00_JPRB, 0.27738E+00_JPRB ,&
&  0.27538E+00_JPRB, 0.27464E+00_JPRB, 0.27487E+00_JPRB, 0.27618E+00_JPRB, 0.27877E+00_JPRB, 0.28014E+00_JPRB, 0.28199E+00_JPRB ,&
&  0.28460E+00_JPRB, 0.28323E+00_JPRB, 0.28377E+00_JPRB, 0.28297E+00_JPRB, 0.28377E+00_JPRB, 0.28537E+00_JPRB, 0.28635E+00_JPRB ,&
&  0.28822E+00_JPRB, 0.28997E+00_JPRB, 0.29038E+00_JPRB, 0.28960E+00_JPRB, 0.28485E+00_JPRB, 0.28299E+00_JPRB, 0.27835E+00_JPRB ,&
&  0.27251E+00_JPRB, 0.26709E+00_JPRB, 0.26021E+00_JPRB, 0.25524E+00_JPRB, 0.24900E+00_JPRB, 0.24536E+00_JPRB, 0.24254E+00_JPRB ,&
&  0.23992E+00_JPRB, 0.23904E+00_JPRB, 0.23771E+00_JPRB, 0.23615E+00_JPRB, 0.23575E+00_JPRB, 0.23410E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.23456E+00_JPRB, 0.23502E+00_JPRB, 0.23636E+00_JPRB, 0.23685E+00_JPRB, 0.23772E+00_JPRB, 0.23843E+00_JPRB, 0.23876E+00_JPRB ,&
&  0.23991E+00_JPRB, 0.24165E+00_JPRB, 0.24149E+00_JPRB, 0.24201E+00_JPRB, 0.24107E+00_JPRB, 0.23880E+00_JPRB, 0.23643E+00_JPRB ,&
&  0.23376E+00_JPRB, 0.23171E+00_JPRB, 0.23112E+00_JPRB, 0.23019E+00_JPRB, 0.23031E+00_JPRB, 0.23073E+00_JPRB, 0.23078E+00_JPRB ,&
&  0.23149E+00_JPRB, 0.23212E+00_JPRB, 0.23313E+00_JPRB, 0.23558E+00_JPRB, 0.23716E+00_JPRB, 0.23928E+00_JPRB, 0.24155E+00_JPRB ,&
&  0.24330E+00_JPRB, 0.24663E+00_JPRB, 0.24953E+00_JPRB, 0.25355E+00_JPRB, 0.26302E+00_JPRB, 0.27054E+00_JPRB, 0.28405E+00_JPRB ,&
&  0.28763E+00_JPRB, 0.29442E+00_JPRB, 0.29012E+00_JPRB, 0.29310E+00_JPRB, 0.29459E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.28838E+00_JPRB, 0.28935E+00_JPRB, 0.28900E+00_JPRB, 0.29003E+00_JPRB, 0.29073E+00_JPRB, 0.29189E+00_JPRB, 0.29157E+00_JPRB ,&
&  0.29136E+00_JPRB, 0.28789E+00_JPRB, 0.28605E+00_JPRB, 0.28252E+00_JPRB, 0.27939E+00_JPRB, 0.27588E+00_JPRB, 0.27307E+00_JPRB ,&
&  0.27132E+00_JPRB, 0.26993E+00_JPRB, 0.26868E+00_JPRB, 0.26689E+00_JPRB, 0.26455E+00_JPRB, 0.26215E+00_JPRB, 0.25999E+00_JPRB ,&
&  0.25797E+00_JPRB, 0.25821E+00_JPRB, 0.25982E+00_JPRB, 0.26228E+00_JPRB, 0.26688E+00_JPRB, 0.26983E+00_JPRB, 0.27367E+00_JPRB ,&
&  0.27903E+00_JPRB, 0.27400E+00_JPRB, 0.27427E+00_JPRB, 0.27499E+00_JPRB, 0.27502E+00_JPRB, 0.27517E+00_JPRB, 0.27349E+00_JPRB ,&
&  0.27943E+00_JPRB, 0.27983E+00_JPRB, 0.28414E+00_JPRB, 0.28253E+00_JPRB, 0.27700E+00_JPRB, 0.27357E+00_JPRB, 0.26822E+00_JPRB ,&
&  0.26496E+00_JPRB, 0.25990E+00_JPRB, 0.25649E+00_JPRB, 0.25311E+00_JPRB, 0.24862E+00_JPRB, 0.24681E+00_JPRB, 0.24368E+00_JPRB ,&
&  0.24189E+00_JPRB, 0.24157E+00_JPRB, 0.24038E+00_JPRB, 0.24045E+00_JPRB, 0.23937E+00_JPRB, 0.23785E+00_JPRB, 0.23693E+00_JPRB ,&
&  0.23615E+00_JPRB, 0.23528E+00_JPRB, 0.23502E+00_JPRB, 0.23466E+00_JPRB, 0.23479E+00_JPRB, 0.23469E+00_JPRB, 0.23457E+00_JPRB ,&
&  0.23467E+00_JPRB, 0.23477E+00_JPRB, 0.23416E+00_JPRB, 0.23405E+00_JPRB, 0.23291E+00_JPRB, 0.23190E+00_JPRB, 0.23027E+00_JPRB ,&
&  0.22922E+00_JPRB, 0.22852E+00_JPRB, 0.22737E+00_JPRB, 0.22663E+00_JPRB, 0.22631E+00_JPRB, 0.22683E+00_JPRB, 0.22753E+00_JPRB ,&
&  0.22836E+00_JPRB, 0.23006E+00_JPRB, 0.23143E+00_JPRB, 0.23328E+00_JPRB, 0.23531E+00_JPRB, 0.23683E+00_JPRB, 0.23910E+00_JPRB ,&
&  0.24131E+00_JPRB, 0.24361E+00_JPRB, 0.24769E+00_JPRB, 0.25207E+00_JPRB, 0.25780E+00_JPRB, 0.26320E+00_JPRB, 0.26495E+00_JPRB ,&
&  0.26661E+00_JPRB, 0.27179E+00_JPRB, 0.27460E+00_JPRB, 0.28557E+00_JPRB, 0.28736E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.27620E+00_JPRB, 0.27818E+00_JPRB, 0.28011E+00_JPRB, 0.28195E+00_JPRB, 0.28432E+00_JPRB, 0.28534E+00_JPRB, 0.28549E+00_JPRB ,&
&  0.28304E+00_JPRB, 0.28277E+00_JPRB, 0.28130E+00_JPRB, 0.27921E+00_JPRB, 0.27752E+00_JPRB, 0.27341E+00_JPRB, 0.27010E+00_JPRB ,&
&  0.26690E+00_JPRB, 0.26385E+00_JPRB, 0.26131E+00_JPRB, 0.25773E+00_JPRB, 0.25438E+00_JPRB, 0.25095E+00_JPRB, 0.25041E+00_JPRB ,&
&  0.25037E+00_JPRB, 0.25132E+00_JPRB, 0.25367E+00_JPRB, 0.25526E+00_JPRB, 0.25762E+00_JPRB, 0.26006E+00_JPRB, 0.26368E+00_JPRB ,&
&  0.26685E+00_JPRB, 0.26908E+00_JPRB, 0.26920E+00_JPRB, 0.26769E+00_JPRB, 0.26684E+00_JPRB, 0.26689E+00_JPRB, 0.26682E+00_JPRB ,&
&  0.26642E+00_JPRB, 0.26661E+00_JPRB, 0.26639E+00_JPRB, 0.26631E+00_JPRB, 0.26503E+00_JPRB, 0.26233E+00_JPRB, 0.25945E+00_JPRB ,&
&  0.25671E+00_JPRB, 0.25425E+00_JPRB, 0.25178E+00_JPRB, 0.24958E+00_JPRB, 0.24831E+00_JPRB, 0.24696E+00_JPRB, 0.24514E+00_JPRB ,&
&  0.24412E+00_JPRB, 0.24303E+00_JPRB, 0.24184E+00_JPRB, 0.24100E+00_JPRB, 0.24010E+00_JPRB, 0.23950E+00_JPRB, 0.23879E+00_JPRB ,&
&  0.23830E+00_JPRB, 0.23744E+00_JPRB, 0.23671E+00_JPRB, 0.23588E+00_JPRB, 0.23554E+00_JPRB, 0.23452E+00_JPRB, 0.23353E+00_JPRB ,&
&  0.23229E+00_JPRB, 0.23151E+00_JPRB, 0.23004E+00_JPRB, 0.22916E+00_JPRB, 0.22811E+00_JPRB, 0.22712E+00_JPRB, 0.22638E+00_JPRB ,&
&  0.22627E+00_JPRB, 0.22603E+00_JPRB, 0.22514E+00_JPRB, 0.22478E+00_JPRB, 0.22461E+00_JPRB, 0.22504E+00_JPRB, 0.22671E+00_JPRB ,&
&  0.22776E+00_JPRB, 0.22891E+00_JPRB, 0.22999E+00_JPRB, 0.23077E+00_JPRB, 0.23222E+00_JPRB, 0.23357E+00_JPRB, 0.23497E+00_JPRB ,&
&  0.23665E+00_JPRB, 0.23804E+00_JPRB, 0.23967E+00_JPRB, 0.24153E+00_JPRB, 0.24655E+00_JPRB, 0.24944E+00_JPRB, 0.25220E+00_JPRB ,&
&  0.25361E+00_JPRB, 0.25019E+00_JPRB, 0.26598E+00_JPRB, 0.26890E+00_JPRB, 0.27122E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.25048E+00_JPRB, 0.25355E+00_JPRB, 0.25506E+00_JPRB, 0.25581E+00_JPRB, 0.24551E+00_JPRB, 0.25733E+00_JPRB, 0.25376E+00_JPRB ,&
&  0.25340E+00_JPRB, 0.25381E+00_JPRB, 0.25670E+00_JPRB, 0.25748E+00_JPRB, 0.25733E+00_JPRB, 0.25683E+00_JPRB, 0.25594E+00_JPRB ,&
&  0.25503E+00_JPRB, 0.25369E+00_JPRB, 0.25301E+00_JPRB, 0.25220E+00_JPRB, 0.25157E+00_JPRB, 0.25089E+00_JPRB, 0.25042E+00_JPRB ,&
&  0.25060E+00_JPRB, 0.25173E+00_JPRB, 0.25366E+00_JPRB, 0.25277E+00_JPRB, 0.25257E+00_JPRB, 0.25285E+00_JPRB, 0.25343E+00_JPRB ,&
&  0.25357E+00_JPRB, 0.25351E+00_JPRB, 0.25303E+00_JPRB, 0.25159E+00_JPRB, 0.25193E+00_JPRB, 0.25237E+00_JPRB, 0.25287E+00_JPRB ,&
&  0.25312E+00_JPRB, 0.25307E+00_JPRB, 0.25223E+00_JPRB, 0.25142E+00_JPRB, 0.25060E+00_JPRB, 0.24953E+00_JPRB, 0.24816E+00_JPRB ,&
&  0.24703E+00_JPRB, 0.24618E+00_JPRB, 0.24507E+00_JPRB, 0.24394E+00_JPRB, 0.24297E+00_JPRB, 0.24202E+00_JPRB, 0.24108E+00_JPRB ,&
&  0.24048E+00_JPRB, 0.24004E+00_JPRB, 0.23956E+00_JPRB, 0.23885E+00_JPRB, 0.23808E+00_JPRB, 0.23735E+00_JPRB, 0.23670E+00_JPRB ,&
&  0.23629E+00_JPRB, 0.23555E+00_JPRB, 0.23482E+00_JPRB, 0.23400E+00_JPRB, 0.23340E+00_JPRB, 0.23235E+00_JPRB, 0.23144E+00_JPRB ,&
&  0.23054E+00_JPRB, 0.22972E+00_JPRB, 0.22906E+00_JPRB, 0.22857E+00_JPRB, 0.22811E+00_JPRB, 0.22775E+00_JPRB, 0.22745E+00_JPRB ,&
&  0.22720E+00_JPRB, 0.22696E+00_JPRB, 0.22682E+00_JPRB, 0.22660E+00_JPRB, 0.22634E+00_JPRB, 0.22620E+00_JPRB, 0.22648E+00_JPRB ,&
&  0.22670E+00_JPRB, 0.22682E+00_JPRB, 0.22685E+00_JPRB, 0.22634E+00_JPRB, 0.22681E+00_JPRB, 0.22721E+00_JPRB, 0.22770E+00_JPRB ,&
&  0.22811E+00_JPRB, 0.22907E+00_JPRB, 0.23025E+00_JPRB, 0.23172E+00_JPRB, 0.23465E+00_JPRB, 0.23712E+00_JPRB, 0.23980E+00_JPRB ,&
&  0.24280E+00_JPRB, 0.24723E+00_JPRB, 0.24880E+00_JPRB, 0.24971E+00_JPRB, 0.25022E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.23867E+00_JPRB, 0.23845E+00_JPRB, 0.23804E+00_JPRB, 0.23680E+00_JPRB, 0.23342E+00_JPRB, 0.23274E+00_JPRB, 0.23263E+00_JPRB ,&
&  0.23281E+00_JPRB, 0.23292E+00_JPRB, 0.23413E+00_JPRB, 0.23543E+00_JPRB, 0.23674E+00_JPRB, 0.23776E+00_JPRB, 0.23866E+00_JPRB ,&
&  0.23915E+00_JPRB, 0.23966E+00_JPRB, 0.23809E+00_JPRB, 0.23731E+00_JPRB, 0.23705E+00_JPRB, 0.23701E+00_JPRB, 0.23698E+00_JPRB ,&
&  0.23689E+00_JPRB, 0.23686E+00_JPRB, 0.23671E+00_JPRB, 0.23701E+00_JPRB, 0.23745E+00_JPRB, 0.23781E+00_JPRB, 0.23823E+00_JPRB ,&
&  0.23856E+00_JPRB, 0.23885E+00_JPRB, 0.23906E+00_JPRB, 0.23922E+00_JPRB, 0.23972E+00_JPRB, 0.24003E+00_JPRB, 0.24003E+00_JPRB ,&
&  0.23990E+00_JPRB, 0.23954E+00_JPRB, 0.23912E+00_JPRB, 0.23868E+00_JPRB, 0.23837E+00_JPRB, 0.23806E+00_JPRB, 0.23774E+00_JPRB ,&
&  0.23741E+00_JPRB, 0.23707E+00_JPRB, 0.23670E+00_JPRB, 0.23627E+00_JPRB, 0.23586E+00_JPRB, 0.23557E+00_JPRB, 0.23523E+00_JPRB ,&
&  0.23495E+00_JPRB, 0.23468E+00_JPRB, 0.23439E+00_JPRB, 0.23399E+00_JPRB, 0.23357E+00_JPRB, 0.23314E+00_JPRB, 0.23266E+00_JPRB ,&
&  0.23205E+00_JPRB, 0.23161E+00_JPRB, 0.23118E+00_JPRB, 0.23080E+00_JPRB, 0.23047E+00_JPRB, 0.23012E+00_JPRB, 0.22981E+00_JPRB ,&
&  0.22949E+00_JPRB, 0.22919E+00_JPRB, 0.22881E+00_JPRB, 0.22846E+00_JPRB, 0.22815E+00_JPRB, 0.22786E+00_JPRB, 0.22760E+00_JPRB ,&
&  0.22736E+00_JPRB, 0.22711E+00_JPRB, 0.22688E+00_JPRB, 0.22685E+00_JPRB, 0.22676E+00_JPRB, 0.22675E+00_JPRB, 0.22674E+00_JPRB ,&
&  0.22674E+00_JPRB, 0.22672E+00_JPRB, 0.22669E+00_JPRB, 0.22650E+00_JPRB, 0.22666E+00_JPRB, 0.22691E+00_JPRB, 0.22732E+00_JPRB ,&
&  0.22779E+00_JPRB, 0.22841E+00_JPRB, 0.22908E+00_JPRB, 0.22980E+00_JPRB, 0.23066E+00_JPRB, 0.23136E+00_JPRB, 0.23215E+00_JPRB ,&
&  0.23304E+00_JPRB, 0.23394E+00_JPRB, 0.23458E+00_JPRB, 0.23534E+00_JPRB, 0.23650E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB ,&
&  0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB, 0.23084E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB ,&
&  0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB, 0.91921E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.93040E+00_JPRB, 0.92900E+00_JPRB, 0.92778E+00_JPRB, 0.92710E+00_JPRB, 0.92673E+00_JPRB, 0.92640E+00_JPRB, 0.92492E+00_JPRB ,&
&  0.92309E+00_JPRB, 0.92083E+00_JPRB, 0.91849E+00_JPRB, 0.91565E+00_JPRB, 0.91210E+00_JPRB, 0.90789E+00_JPRB, 0.90407E+00_JPRB ,&
&  0.90052E+00_JPRB, 0.89758E+00_JPRB, 0.89448E+00_JPRB, 0.89233E+00_JPRB, 0.88994E+00_JPRB, 0.88749E+00_JPRB, 0.88493E+00_JPRB ,&
&  0.88290E+00_JPRB, 0.88101E+00_JPRB, 0.87920E+00_JPRB, 0.87766E+00_JPRB, 0.87708E+00_JPRB, 0.87627E+00_JPRB, 0.87536E+00_JPRB ,&
&  0.87461E+00_JPRB, 0.87424E+00_JPRB, 0.87399E+00_JPRB, 0.87372E+00_JPRB, 0.87357E+00_JPRB, 0.87393E+00_JPRB, 0.87427E+00_JPRB ,&
&  0.87485E+00_JPRB, 0.87618E+00_JPRB, 0.87809E+00_JPRB, 0.87985E+00_JPRB, 0.88180E+00_JPRB, 0.88448E+00_JPRB, 0.88782E+00_JPRB ,&
&  0.89080E+00_JPRB, 0.89367E+00_JPRB, 0.89692E+00_JPRB, 0.90063E+00_JPRB, 0.90422E+00_JPRB, 0.90812E+00_JPRB, 0.91185E+00_JPRB ,&
&  0.91556E+00_JPRB, 0.91858E+00_JPRB, 0.92151E+00_JPRB, 0.92297E+00_JPRB, 0.92504E+00_JPRB, 0.92678E+00_JPRB, 0.92875E+00_JPRB ,&
&  0.92926E+00_JPRB, 0.92954E+00_JPRB, 0.92998E+00_JPRB, 0.93076E+00_JPRB, 0.93121E+00_JPRB, 0.93182E+00_JPRB, 0.93255E+00_JPRB ,&
&  0.93366E+00_JPRB, 0.93424E+00_JPRB, 0.93518E+00_JPRB, 0.93582E+00_JPRB, 0.93667E+00_JPRB, 0.93681E+00_JPRB, 0.93737E+00_JPRB ,&
&  0.93794E+00_JPRB, 0.93888E+00_JPRB, 0.93896E+00_JPRB, 0.93921E+00_JPRB, 0.93929E+00_JPRB, 0.93953E+00_JPRB, 0.93973E+00_JPRB ,&
&  0.94005E+00_JPRB, 0.94021E+00_JPRB, 0.94032E+00_JPRB, 0.93980E+00_JPRB, 0.93888E+00_JPRB, 0.93800E+00_JPRB, 0.93730E+00_JPRB ,&
&  0.93685E+00_JPRB, 0.93638E+00_JPRB, 0.93628E+00_JPRB, 0.93650E+00_JPRB, 0.93716E+00_JPRB, 0.93703E+00_JPRB, 0.93687E+00_JPRB ,&
&  0.93655E+00_JPRB, 0.93611E+00_JPRB, 0.93492E+00_JPRB, 0.93350E+00_JPRB, 0.93194E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.93048E+00_JPRB, 0.92829E+00_JPRB, 0.92687E+00_JPRB, 0.92643E+00_JPRB, 0.92646E+00_JPRB, 0.92622E+00_JPRB, 0.92571E+00_JPRB ,&
&  0.92349E+00_JPRB, 0.91850E+00_JPRB, 0.91270E+00_JPRB, 0.90626E+00_JPRB, 0.89834E+00_JPRB, 0.89113E+00_JPRB, 0.88639E+00_JPRB ,&
&  0.88149E+00_JPRB, 0.87657E+00_JPRB, 0.87179E+00_JPRB, 0.86788E+00_JPRB, 0.86344E+00_JPRB, 0.85871E+00_JPRB, 0.85423E+00_JPRB ,&
&  0.85069E+00_JPRB, 0.84666E+00_JPRB, 0.84210E+00_JPRB, 0.83617E+00_JPRB, 0.83240E+00_JPRB, 0.82835E+00_JPRB, 0.82473E+00_JPRB ,&
&  0.81985E+00_JPRB, 0.81775E+00_JPRB, 0.81686E+00_JPRB, 0.81736E+00_JPRB, 0.82232E+00_JPRB, 0.82585E+00_JPRB, 0.82665E+00_JPRB ,&
&  0.82574E+00_JPRB, 0.82070E+00_JPRB, 0.82128E+00_JPRB, 0.82264E+00_JPRB, 0.82549E+00_JPRB, 0.83161E+00_JPRB, 0.83933E+00_JPRB ,&
&  0.85040E+00_JPRB, 0.86379E+00_JPRB, 0.87791E+00_JPRB, 0.88886E+00_JPRB, 0.89951E+00_JPRB, 0.90911E+00_JPRB, 0.91616E+00_JPRB ,&
&  0.92566E+00_JPRB, 0.93370E+00_JPRB, 0.94192E+00_JPRB, 0.94919E+00_JPRB, 0.95753E+00_JPRB, 0.96403E+00_JPRB, 0.96874E+00_JPRB ,&
&  0.96979E+00_JPRB, 0.96925E+00_JPRB, 0.96857E+00_JPRB, 0.96810E+00_JPRB, 0.96595E+00_JPRB, 0.96486E+00_JPRB, 0.96480E+00_JPRB ,&
&  0.96556E+00_JPRB, 0.96557E+00_JPRB, 0.96723E+00_JPRB, 0.96910E+00_JPRB, 0.97131E+00_JPRB, 0.97099E+00_JPRB, 0.97150E+00_JPRB ,&
&  0.97158E+00_JPRB, 0.97191E+00_JPRB, 0.97044E+00_JPRB, 0.96743E+00_JPRB, 0.96496E+00_JPRB, 0.96381E+00_JPRB, 0.96176E+00_JPRB ,&
&  0.95969E+00_JPRB, 0.95809E+00_JPRB, 0.95736E+00_JPRB, 0.95557E+00_JPRB, 0.95293E+00_JPRB, 0.95055E+00_JPRB, 0.94933E+00_JPRB ,&
&  0.94791E+00_JPRB, 0.94601E+00_JPRB, 0.94530E+00_JPRB, 0.94488E+00_JPRB, 0.94672E+00_JPRB, 0.94651E+00_JPRB, 0.94598E+00_JPRB ,&
&  0.94437E+00_JPRB, 0.94333E+00_JPRB, 0.94095E+00_JPRB, 0.93751E+00_JPRB, 0.93340E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.94207E+00_JPRB, 0.94182E+00_JPRB, 0.94384E+00_JPRB, 0.94311E+00_JPRB, 0.94116E+00_JPRB, 0.93704E+00_JPRB, 0.93442E+00_JPRB ,&
&  0.92906E+00_JPRB, 0.92029E+00_JPRB, 0.90720E+00_JPRB, 0.89599E+00_JPRB, 0.88491E+00_JPRB, 0.87762E+00_JPRB, 0.87082E+00_JPRB ,&
&  0.86402E+00_JPRB, 0.85837E+00_JPRB, 0.85242E+00_JPRB, 0.84491E+00_JPRB, 0.83549E+00_JPRB, 0.82458E+00_JPRB, 0.80997E+00_JPRB ,&
&  0.79973E+00_JPRB, 0.78999E+00_JPRB, 0.78196E+00_JPRB, 0.77700E+00_JPRB, 0.77679E+00_JPRB, 0.75257E+00_JPRB, 0.74956E+00_JPRB ,&
&  0.76474E+00_JPRB, 0.75041E+00_JPRB, 0.73569E+00_JPRB, 0.71158E+00_JPRB, 0.70168E+00_JPRB, 0.71423E+00_JPRB, 0.72437E+00_JPRB ,&
&  0.71764E+00_JPRB, 0.75778E+00_JPRB, 0.76151E+00_JPRB, 0.76282E+00_JPRB, 0.75955E+00_JPRB, 0.75163E+00_JPRB, 0.76300E+00_JPRB ,&
&  0.79150E+00_JPRB, 0.80361E+00_JPRB, 0.82944E+00_JPRB, 0.85827E+00_JPRB, 0.87896E+00_JPRB, 0.89995E+00_JPRB, 0.92111E+00_JPRB ,&
&  0.94176E+00_JPRB, 0.95695E+00_JPRB, 0.96979E+00_JPRB, 0.98126E+00_JPRB, 0.99850E+00_JPRB, 0.10153E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10456E+01_JPRB, 0.10525E+01_JPRB, 0.10595E+01_JPRB, 0.10694E+01_JPRB, 0.10780E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.10995E+01_JPRB, 0.11031E+01_JPRB, 0.11045E+01_JPRB, 0.11045E+01_JPRB, 0.11007E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.10856E+01_JPRB, 0.10753E+01_JPRB, 0.10618E+01_JPRB, 0.10513E+01_JPRB, 0.10431E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10256E+01_JPRB, 0.10168E+01_JPRB, 0.10091E+01_JPRB, 0.99935E+00_JPRB, 0.99020E+00_JPRB, 0.98086E+00_JPRB, 0.97455E+00_JPRB ,&
&  0.96979E+00_JPRB, 0.96599E+00_JPRB, 0.96419E+00_JPRB, 0.96183E+00_JPRB, 0.96163E+00_JPRB, 0.96043E+00_JPRB, 0.96140E+00_JPRB ,&
&  0.96061E+00_JPRB, 0.95623E+00_JPRB, 0.94980E+00_JPRB, 0.94529E+00_JPRB, 0.94192E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.94499E+00_JPRB, 0.94785E+00_JPRB, 0.95519E+00_JPRB, 0.95383E+00_JPRB, 0.95069E+00_JPRB, 0.94128E+00_JPRB, 0.93693E+00_JPRB ,&
&  0.92754E+00_JPRB, 0.91787E+00_JPRB, 0.89770E+00_JPRB, 0.88447E+00_JPRB, 0.87231E+00_JPRB, 0.86455E+00_JPRB, 0.85832E+00_JPRB ,&
&  0.85323E+00_JPRB, 0.84669E+00_JPRB, 0.83881E+00_JPRB, 0.82718E+00_JPRB, 0.81256E+00_JPRB, 0.79597E+00_JPRB, 0.77216E+00_JPRB ,&
&  0.75258E+00_JPRB, 0.73891E+00_JPRB, 0.72139E+00_JPRB, 0.73007E+00_JPRB, 0.72138E+00_JPRB, 0.71459E+00_JPRB, 0.70978E+00_JPRB ,&
&  0.69597E+00_JPRB, 0.68004E+00_JPRB, 0.65695E+00_JPRB, 0.64626E+00_JPRB, 0.63967E+00_JPRB, 0.63104E+00_JPRB, 0.64895E+00_JPRB ,&
&  0.66245E+00_JPRB, 0.69700E+00_JPRB, 0.71114E+00_JPRB, 0.71407E+00_JPRB, 0.71863E+00_JPRB, 0.71612E+00_JPRB, 0.72498E+00_JPRB ,&
&  0.73626E+00_JPRB, 0.75047E+00_JPRB, 0.79855E+00_JPRB, 0.83077E+00_JPRB, 0.85484E+00_JPRB, 0.87623E+00_JPRB, 0.89202E+00_JPRB ,&
&  0.91206E+00_JPRB, 0.92454E+00_JPRB, 0.93788E+00_JPRB, 0.95449E+00_JPRB, 0.97961E+00_JPRB, 0.10088E+01_JPRB, 0.10349E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10720E+01_JPRB, 0.10905E+01_JPRB, 0.11118E+01_JPRB, 0.11408E+01_JPRB, 0.11733E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.12430E+01_JPRB, 0.12781E+01_JPRB, 0.13070E+01_JPRB, 0.13326E+01_JPRB, 0.13464E+01_JPRB, 0.13489E+01_JPRB, 0.13462E+01_JPRB ,&
&  0.13273E+01_JPRB, 0.13051E+01_JPRB, 0.12748E+01_JPRB, 0.12287E+01_JPRB, 0.11936E+01_JPRB, 0.11750E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.11129E+01_JPRB, 0.10868E+01_JPRB, 0.10632E+01_JPRB, 0.10376E+01_JPRB, 0.10147E+01_JPRB, 0.99329E+00_JPRB, 0.97898E+00_JPRB ,&
&  0.96753E+00_JPRB, 0.96071E+00_JPRB, 0.95665E+00_JPRB, 0.95425E+00_JPRB, 0.95478E+00_JPRB, 0.95405E+00_JPRB, 0.95745E+00_JPRB ,&
&  0.95772E+00_JPRB, 0.95445E+00_JPRB, 0.94721E+00_JPRB, 0.94382E+00_JPRB, 0.94191E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.95143E+00_JPRB, 0.95239E+00_JPRB, 0.95039E+00_JPRB, 0.93912E+00_JPRB, 0.93265E+00_JPRB, 0.92538E+00_JPRB, 0.92444E+00_JPRB ,&
&  0.91609E+00_JPRB, 0.90784E+00_JPRB, 0.88841E+00_JPRB, 0.87607E+00_JPRB, 0.86202E+00_JPRB, 0.85343E+00_JPRB, 0.84534E+00_JPRB ,&
&  0.83915E+00_JPRB, 0.83167E+00_JPRB, 0.82622E+00_JPRB, 0.81459E+00_JPRB, 0.80329E+00_JPRB, 0.78586E+00_JPRB, 0.76661E+00_JPRB ,&
&  0.74640E+00_JPRB, 0.72578E+00_JPRB, 0.71019E+00_JPRB, 0.71388E+00_JPRB, 0.70481E+00_JPRB, 0.69791E+00_JPRB, 0.68907E+00_JPRB ,&
&  0.68091E+00_JPRB, 0.64253E+00_JPRB, 0.65782E+00_JPRB, 0.64286E+00_JPRB, 0.62672E+00_JPRB, 0.60533E+00_JPRB, 0.58732E+00_JPRB ,&
&  0.59207E+00_JPRB, 0.59569E+00_JPRB, 0.61046E+00_JPRB, 0.63269E+00_JPRB, 0.64269E+00_JPRB, 0.66070E+00_JPRB, 0.67236E+00_JPRB ,&
&  0.66559E+00_JPRB, 0.75001E+00_JPRB, 0.79426E+00_JPRB, 0.82019E+00_JPRB, 0.84595E+00_JPRB, 0.86609E+00_JPRB, 0.87846E+00_JPRB ,&
&  0.90237E+00_JPRB, 0.92846E+00_JPRB, 0.95865E+00_JPRB, 0.98981E+00_JPRB, 0.10340E+01_JPRB, 0.10649E+01_JPRB, 0.10946E+01_JPRB ,&
&  0.11186E+01_JPRB, 0.11308E+01_JPRB, 0.11510E+01_JPRB, 0.11819E+01_JPRB, 0.12214E+01_JPRB, 0.12656E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.14218E+01_JPRB, 0.14773E+01_JPRB, 0.15256E+01_JPRB, 0.15634E+01_JPRB, 0.15868E+01_JPRB, 0.16110E+01_JPRB ,&
&  0.15933E+01_JPRB, 0.15673E+01_JPRB, 0.15246E+01_JPRB, 0.14657E+01_JPRB, 0.13932E+01_JPRB, 0.13132E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.11461E+01_JPRB, 0.11052E+01_JPRB, 0.10660E+01_JPRB, 0.10315E+01_JPRB, 0.10032E+01_JPRB, 0.98731E+00_JPRB ,&
&  0.97340E+00_JPRB, 0.96478E+00_JPRB, 0.95914E+00_JPRB, 0.95430E+00_JPRB, 0.95165E+00_JPRB, 0.94600E+00_JPRB, 0.94413E+00_JPRB ,&
&  0.94376E+00_JPRB, 0.94608E+00_JPRB, 0.94478E+00_JPRB, 0.94564E+00_JPRB, 0.94766E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.96334E+00_JPRB, 0.96562E+00_JPRB, 0.95936E+00_JPRB, 0.95131E+00_JPRB, 0.94626E+00_JPRB, 0.93378E+00_JPRB, 0.92774E+00_JPRB ,&
&  0.92345E+00_JPRB, 0.91927E+00_JPRB, 0.90747E+00_JPRB, 0.89440E+00_JPRB, 0.87659E+00_JPRB, 0.86296E+00_JPRB, 0.84803E+00_JPRB ,&
&  0.83530E+00_JPRB, 0.82066E+00_JPRB, 0.80415E+00_JPRB, 0.78894E+00_JPRB, 0.77738E+00_JPRB, 0.76266E+00_JPRB, 0.75187E+00_JPRB ,&
&  0.73968E+00_JPRB, 0.72408E+00_JPRB, 0.71342E+00_JPRB, 0.71382E+00_JPRB, 0.70352E+00_JPRB, 0.69287E+00_JPRB, 0.69364E+00_JPRB ,&
&  0.69893E+00_JPRB, 0.69251E+00_JPRB, 0.68351E+00_JPRB, 0.66697E+00_JPRB, 0.63429E+00_JPRB, 0.57162E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58122E+00_JPRB, 0.57012E+00_JPRB, 0.57475E+00_JPRB, 0.58622E+00_JPRB, 0.59843E+00_JPRB, 0.63146E+00_JPRB, 0.63427E+00_JPRB ,&
&  0.62081E+00_JPRB, 0.74067E+00_JPRB, 0.76726E+00_JPRB, 0.79894E+00_JPRB, 0.82890E+00_JPRB, 0.85110E+00_JPRB, 0.87220E+00_JPRB ,&
&  0.89269E+00_JPRB, 0.92280E+00_JPRB, 0.95162E+00_JPRB, 0.99469E+00_JPRB, 0.10384E+01_JPRB, 0.10921E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11864E+01_JPRB, 0.12120E+01_JPRB, 0.12323E+01_JPRB, 0.12567E+01_JPRB, 0.13002E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.14386E+01_JPRB, 0.14878E+01_JPRB, 0.15948E+01_JPRB, 0.16764E+01_JPRB, 0.17189E+01_JPRB, 0.17774E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.18243E+01_JPRB, 0.18162E+01_JPRB, 0.17839E+01_JPRB, 0.16592E+01_JPRB, 0.15247E+01_JPRB, 0.14333E+01_JPRB, 0.13502E+01_JPRB ,&
&  0.12860E+01_JPRB, 0.12211E+01_JPRB, 0.11546E+01_JPRB, 0.11044E+01_JPRB, 0.10650E+01_JPRB, 0.10369E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.99221E+00_JPRB, 0.97844E+00_JPRB, 0.97267E+00_JPRB, 0.95908E+00_JPRB, 0.94763E+00_JPRB, 0.93501E+00_JPRB ,&
&  0.93215E+00_JPRB, 0.93221E+00_JPRB, 0.93980E+00_JPRB, 0.94729E+00_JPRB, 0.95191E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.97980E+00_JPRB, 0.99747E+00_JPRB, 0.10025E+01_JPRB, 0.10030E+01_JPRB, 0.10028E+01_JPRB, 0.98299E+00_JPRB, 0.96541E+00_JPRB ,&
&  0.95265E+00_JPRB, 0.94207E+00_JPRB, 0.93041E+00_JPRB, 0.91762E+00_JPRB, 0.90263E+00_JPRB, 0.88995E+00_JPRB, 0.87802E+00_JPRB ,&
&  0.86723E+00_JPRB, 0.82168E+00_JPRB, 0.79936E+00_JPRB, 0.78153E+00_JPRB, 0.76399E+00_JPRB, 0.75476E+00_JPRB, 0.75019E+00_JPRB ,&
&  0.74086E+00_JPRB, 0.73561E+00_JPRB, 0.73196E+00_JPRB, 0.72414E+00_JPRB, 0.71311E+00_JPRB, 0.70025E+00_JPRB, 0.69491E+00_JPRB ,&
&  0.69870E+00_JPRB, 0.68466E+00_JPRB, 0.67426E+00_JPRB, 0.65583E+00_JPRB, 0.61850E+00_JPRB, 0.59845E+00_JPRB, 0.59886E+00_JPRB ,&
&  0.59365E+00_JPRB, 0.56654E+00_JPRB, 0.56872E+00_JPRB, 0.57278E+00_JPRB, 0.57510E+00_JPRB, 0.58948E+00_JPRB, 0.61803E+00_JPRB ,&
&  0.64455E+00_JPRB, 0.73735E+00_JPRB, 0.77668E+00_JPRB, 0.82070E+00_JPRB, 0.85452E+00_JPRB, 0.88312E+00_JPRB, 0.90806E+00_JPRB ,&
&  0.93229E+00_JPRB, 0.96731E+00_JPRB, 0.10057E+01_JPRB, 0.10562E+01_JPRB, 0.11080E+01_JPRB, 0.11501E+01_JPRB, 0.11956E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.12427E+01_JPRB, 0.12637E+01_JPRB, 0.12671E+01_JPRB, 0.12869E+01_JPRB, 0.13433E+01_JPRB, 0.13619E+01_JPRB ,&
&  0.14276E+01_JPRB, 0.15324E+01_JPRB, 0.17027E+01_JPRB, 0.18734E+01_JPRB, 0.19534E+01_JPRB, 0.20410E+01_JPRB, 0.20775E+01_JPRB ,&
&  0.19863E+01_JPRB, 0.19338E+01_JPRB, 0.19841E+01_JPRB, 0.18347E+01_JPRB, 0.16722E+01_JPRB, 0.15350E+01_JPRB, 0.14413E+01_JPRB ,&
&  0.13476E+01_JPRB, 0.12555E+01_JPRB, 0.11972E+01_JPRB, 0.11524E+01_JPRB, 0.11072E+01_JPRB, 0.10681E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.10585E+01_JPRB, 0.10513E+01_JPRB, 0.10387E+01_JPRB, 0.10238E+01_JPRB, 0.99414E+00_JPRB, 0.96323E+00_JPRB, 0.93689E+00_JPRB ,&
&  0.92997E+00_JPRB, 0.92814E+00_JPRB, 0.93759E+00_JPRB, 0.94605E+00_JPRB, 0.96004E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.99020E+00_JPRB, 0.10050E+01_JPRB, 0.10094E+01_JPRB, 0.10155E+01_JPRB, 0.10262E+01_JPRB, 0.97551E+00_JPRB, 0.93794E+00_JPRB ,&
&  0.91586E+00_JPRB, 0.91813E+00_JPRB, 0.91773E+00_JPRB, 0.91152E+00_JPRB, 0.89968E+00_JPRB, 0.88642E+00_JPRB, 0.87562E+00_JPRB ,&
&  0.86616E+00_JPRB, 0.84394E+00_JPRB, 0.80579E+00_JPRB, 0.75441E+00_JPRB, 0.74027E+00_JPRB, 0.73573E+00_JPRB, 0.73751E+00_JPRB ,&
&  0.73659E+00_JPRB, 0.73187E+00_JPRB, 0.73400E+00_JPRB, 0.72143E+00_JPRB, 0.71299E+00_JPRB, 0.68916E+00_JPRB, 0.67119E+00_JPRB ,&
&  0.67267E+00_JPRB, 0.66753E+00_JPRB, 0.67422E+00_JPRB, 0.66343E+00_JPRB, 0.65080E+00_JPRB, 0.64081E+00_JPRB, 0.58471E+00_JPRB ,&
&  0.57549E+00_JPRB, 0.56158E+00_JPRB, 0.55722E+00_JPRB, 0.55719E+00_JPRB, 0.57899E+00_JPRB, 0.59770E+00_JPRB, 0.61996E+00_JPRB ,&
&  0.63368E+00_JPRB, 0.74340E+00_JPRB, 0.77888E+00_JPRB, 0.80895E+00_JPRB, 0.83809E+00_JPRB, 0.87465E+00_JPRB, 0.92327E+00_JPRB ,&
&  0.97081E+00_JPRB, 0.10182E+01_JPRB, 0.10560E+01_JPRB, 0.11088E+01_JPRB, 0.11608E+01_JPRB, 0.11480E+01_JPRB, 0.12348E+01_JPRB ,&
&  0.15717E+01_JPRB, 0.14882E+01_JPRB, 0.14141E+01_JPRB, 0.13783E+01_JPRB, 0.13854E+01_JPRB, 0.14060E+01_JPRB, 0.14446E+01_JPRB ,&
&  0.15597E+01_JPRB, 0.17546E+01_JPRB, 0.20492E+01_JPRB, 0.22600E+01_JPRB, 0.22721E+01_JPRB, 0.23061E+01_JPRB, 0.22813E+01_JPRB ,&
&  0.21326E+01_JPRB, 0.20365E+01_JPRB, 0.21293E+01_JPRB, 0.17780E+01_JPRB, 0.15315E+01_JPRB, 0.14022E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.12885E+01_JPRB, 0.12252E+01_JPRB, 0.11895E+01_JPRB, 0.11454E+01_JPRB, 0.10889E+01_JPRB, 0.10561E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.10391E+01_JPRB, 0.10338E+01_JPRB, 0.10236E+01_JPRB, 0.10023E+01_JPRB, 0.97859E+00_JPRB, 0.97032E+00_JPRB ,&
&  0.96842E+00_JPRB, 0.96361E+00_JPRB, 0.95971E+00_JPRB, 0.96059E+00_JPRB, 0.97022E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.99186E+00_JPRB, 0.10042E+01_JPRB, 0.10074E+01_JPRB, 0.99626E+00_JPRB, 0.98040E+00_JPRB, 0.90961E+00_JPRB, 0.83258E+00_JPRB ,&
&  0.77645E+00_JPRB, 0.77890E+00_JPRB, 0.80159E+00_JPRB, 0.81988E+00_JPRB, 0.81926E+00_JPRB, 0.80260E+00_JPRB, 0.79465E+00_JPRB ,&
&  0.80010E+00_JPRB, 0.79820E+00_JPRB, 0.77226E+00_JPRB, 0.75479E+00_JPRB, 0.72543E+00_JPRB, 0.71959E+00_JPRB, 0.72150E+00_JPRB ,&
&  0.72418E+00_JPRB, 0.72188E+00_JPRB, 0.72191E+00_JPRB, 0.71939E+00_JPRB, 0.71899E+00_JPRB, 0.71174E+00_JPRB, 0.69719E+00_JPRB ,&
&  0.67701E+00_JPRB, 0.67367E+00_JPRB, 0.67799E+00_JPRB, 0.68584E+00_JPRB, 0.68081E+00_JPRB, 0.67076E+00_JPRB, 0.59995E+00_JPRB ,&
&  0.57278E+00_JPRB, 0.55452E+00_JPRB, 0.55728E+00_JPRB, 0.57266E+00_JPRB, 0.60639E+00_JPRB, 0.60881E+00_JPRB, 0.69118E+00_JPRB ,&
&  0.75252E+00_JPRB, 0.78795E+00_JPRB, 0.80381E+00_JPRB, 0.81834E+00_JPRB, 0.84850E+00_JPRB, 0.91130E+00_JPRB, 0.99002E+00_JPRB ,&
&  0.10428E+01_JPRB, 0.10808E+01_JPRB, 0.10788E+01_JPRB, 0.95405E+00_JPRB, 0.95951E+00_JPRB, 0.10432E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.13186E+01_JPRB, 0.13553E+01_JPRB, 0.13638E+01_JPRB, 0.14265E+01_JPRB, 0.14337E+01_JPRB, 0.14387E+01_JPRB, 0.14405E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.18544E+01_JPRB, 0.22448E+01_JPRB, 0.24511E+01_JPRB, 0.23990E+01_JPRB, 0.24279E+01_JPRB, 0.23427E+01_JPRB ,&
&  0.22599E+01_JPRB, 0.20321E+01_JPRB, 0.18315E+01_JPRB, 0.15887E+01_JPRB, 0.13967E+01_JPRB, 0.12877E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.11708E+01_JPRB, 0.11236E+01_JPRB, 0.11217E+01_JPRB, 0.10967E+01_JPRB, 0.10425E+01_JPRB, 0.99110E+00_JPRB, 0.95192E+00_JPRB ,&
&  0.92449E+00_JPRB, 0.91154E+00_JPRB, 0.91187E+00_JPRB, 0.89400E+00_JPRB, 0.89617E+00_JPRB, 0.91151E+00_JPRB, 0.91957E+00_JPRB ,&
&  0.91945E+00_JPRB, 0.91866E+00_JPRB, 0.93034E+00_JPRB, 0.94711E+00_JPRB, 0.96421E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.10400E+01_JPRB, 0.10122E+01_JPRB, 0.97455E+00_JPRB, 0.96808E+00_JPRB, 0.94284E+00_JPRB, 0.92419E+00_JPRB, 0.84439E+00_JPRB ,&
&  0.74305E+00_JPRB, 0.71305E+00_JPRB, 0.73666E+00_JPRB, 0.74458E+00_JPRB, 0.72705E+00_JPRB, 0.72104E+00_JPRB, 0.72652E+00_JPRB ,&
&  0.73354E+00_JPRB, 0.75850E+00_JPRB, 0.75611E+00_JPRB, 0.73924E+00_JPRB, 0.72850E+00_JPRB, 0.72315E+00_JPRB, 0.72622E+00_JPRB ,&
&  0.72685E+00_JPRB, 0.72610E+00_JPRB, 0.71846E+00_JPRB, 0.70550E+00_JPRB, 0.70287E+00_JPRB, 0.70072E+00_JPRB, 0.69737E+00_JPRB ,&
&  0.69669E+00_JPRB, 0.69224E+00_JPRB, 0.70007E+00_JPRB, 0.70427E+00_JPRB, 0.69671E+00_JPRB, 0.67067E+00_JPRB, 0.60758E+00_JPRB ,&
&  0.57491E+00_JPRB, 0.56173E+00_JPRB, 0.59203E+00_JPRB, 0.65887E+00_JPRB, 0.71192E+00_JPRB, 0.76060E+00_JPRB, 0.79781E+00_JPRB ,&
&  0.77588E+00_JPRB, 0.78269E+00_JPRB, 0.82299E+00_JPRB, 0.93852E+00_JPRB, 0.99180E+00_JPRB, 0.10601E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.12137E+01_JPRB, 0.12196E+01_JPRB, 0.10661E+01_JPRB, 0.11047E+01_JPRB, 0.13387E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.14804E+01_JPRB, 0.15618E+01_JPRB, 0.15429E+01_JPRB, 0.14575E+01_JPRB, 0.14707E+01_JPRB, 0.15777E+01_JPRB ,&
&  0.17542E+01_JPRB, 0.20312E+01_JPRB, 0.25555E+01_JPRB, 0.28480E+01_JPRB, 0.28046E+01_JPRB, 0.27714E+01_JPRB, 0.27147E+01_JPRB ,&
&  0.26669E+01_JPRB, 0.24183E+01_JPRB, 0.20965E+01_JPRB, 0.17978E+01_JPRB, 0.15405E+01_JPRB, 0.13393E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.11473E+01_JPRB, 0.11522E+01_JPRB, 0.11308E+01_JPRB, 0.10661E+01_JPRB, 0.98830E+00_JPRB, 0.92348E+00_JPRB ,&
&  0.88295E+00_JPRB, 0.84599E+00_JPRB, 0.80746E+00_JPRB, 0.79536E+00_JPRB, 0.83854E+00_JPRB, 0.89676E+00_JPRB, 0.91846E+00_JPRB ,&
&  0.90396E+00_JPRB, 0.90107E+00_JPRB, 0.94450E+00_JPRB, 0.98919E+00_JPRB, 0.10059E+01_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.10857E+01_JPRB, 0.10451E+01_JPRB, 0.10097E+01_JPRB, 0.97849E+00_JPRB, 0.95132E+00_JPRB, 0.90127E+00_JPRB, 0.81345E+00_JPRB ,&
&  0.72739E+00_JPRB, 0.69132E+00_JPRB, 0.70017E+00_JPRB, 0.68429E+00_JPRB, 0.66118E+00_JPRB, 0.64730E+00_JPRB, 0.64954E+00_JPRB ,&
&  0.66121E+00_JPRB, 0.69301E+00_JPRB, 0.71573E+00_JPRB, 0.72226E+00_JPRB, 0.71923E+00_JPRB, 0.71505E+00_JPRB, 0.72258E+00_JPRB ,&
&  0.73347E+00_JPRB, 0.74478E+00_JPRB, 0.74989E+00_JPRB, 0.74316E+00_JPRB, 0.73402E+00_JPRB, 0.72994E+00_JPRB, 0.72801E+00_JPRB ,&
&  0.73012E+00_JPRB, 0.72906E+00_JPRB, 0.71577E+00_JPRB, 0.71584E+00_JPRB, 0.69500E+00_JPRB, 0.69170E+00_JPRB, 0.65593E+00_JPRB ,&
&  0.60788E+00_JPRB, 0.61678E+00_JPRB, 0.73612E+00_JPRB, 0.78593E+00_JPRB, 0.80262E+00_JPRB, 0.84038E+00_JPRB, 0.89269E+00_JPRB ,&
&  0.94063E+00_JPRB, 0.98495E+00_JPRB, 0.10266E+01_JPRB, 0.10736E+01_JPRB, 0.11210E+01_JPRB, 0.11937E+01_JPRB, 0.12729E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.11957E+01_JPRB, 0.11429E+01_JPRB, 0.12238E+01_JPRB, 0.15690E+01_JPRB, 0.16815E+01_JPRB, 0.16990E+01_JPRB ,&
&  0.17523E+01_JPRB, 0.17251E+01_JPRB, 0.16978E+01_JPRB, 0.17118E+01_JPRB, 0.17864E+01_JPRB, 0.19180E+01_JPRB, 0.21320E+01_JPRB ,&
&  0.23250E+01_JPRB, 0.26877E+01_JPRB, 0.32279E+01_JPRB, 0.33612E+01_JPRB, 0.33135E+01_JPRB, 0.32835E+01_JPRB, 0.31772E+01_JPRB ,&
&  0.30436E+01_JPRB, 0.27781E+01_JPRB, 0.23452E+01_JPRB, 0.19811E+01_JPRB, 0.16777E+01_JPRB, 0.14470E+01_JPRB, 0.12631E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.11574E+01_JPRB, 0.11585E+01_JPRB, 0.11437E+01_JPRB, 0.11046E+01_JPRB, 0.10418E+01_JPRB, 0.96823E+00_JPRB ,&
&  0.88589E+00_JPRB, 0.84334E+00_JPRB, 0.79930E+00_JPRB, 0.82625E+00_JPRB, 0.89348E+00_JPRB, 0.95983E+00_JPRB, 0.98038E+00_JPRB ,&
&  0.99310E+00_JPRB, 0.98359E+00_JPRB, 0.99021E+00_JPRB, 0.10235E+01_JPRB, 0.10649E+01_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.11080E+01_JPRB, 0.10596E+01_JPRB, 0.10038E+01_JPRB, 0.94166E+00_JPRB, 0.88808E+00_JPRB, 0.84519E+00_JPRB, 0.79416E+00_JPRB ,&
&  0.82983E+00_JPRB, 0.74855E+00_JPRB, 0.81849E+00_JPRB, 0.71179E+00_JPRB, 0.68245E+00_JPRB, 0.67358E+00_JPRB, 0.65446E+00_JPRB ,&
&  0.64307E+00_JPRB, 0.64719E+00_JPRB, 0.65841E+00_JPRB, 0.67254E+00_JPRB, 0.67328E+00_JPRB, 0.67328E+00_JPRB, 0.68350E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.73000E+00_JPRB, 0.76003E+00_JPRB, 0.77560E+00_JPRB, 0.78023E+00_JPRB, 0.77573E+00_JPRB, 0.77696E+00_JPRB ,&
&  0.74024E+00_JPRB, 0.72839E+00_JPRB, 0.73048E+00_JPRB, 0.72759E+00_JPRB, 0.72276E+00_JPRB, 0.70842E+00_JPRB, 0.72194E+00_JPRB ,&
&  0.70900E+00_JPRB, 0.69009E+00_JPRB, 0.79981E+00_JPRB, 0.82135E+00_JPRB, 0.85420E+00_JPRB, 0.89135E+00_JPRB, 0.93215E+00_JPRB ,&
&  0.97430E+00_JPRB, 0.10229E+01_JPRB, 0.10776E+01_JPRB, 0.11212E+01_JPRB, 0.11970E+01_JPRB, 0.13097E+01_JPRB, 0.14158E+01_JPRB ,&
&  0.15284E+01_JPRB, 0.12621E+01_JPRB, 0.13857E+01_JPRB, 0.14782E+01_JPRB, 0.17638E+01_JPRB, 0.20768E+01_JPRB, 0.21374E+01_JPRB ,&
&  0.23506E+01_JPRB, 0.23180E+01_JPRB, 0.22701E+01_JPRB, 0.22262E+01_JPRB, 0.23635E+01_JPRB, 0.25605E+01_JPRB, 0.27850E+01_JPRB ,&
&  0.30711E+01_JPRB, 0.34552E+01_JPRB, 0.40244E+01_JPRB, 0.40324E+01_JPRB, 0.39479E+01_JPRB, 0.38328E+01_JPRB, 0.36248E+01_JPRB ,&
&  0.33444E+01_JPRB, 0.30062E+01_JPRB, 0.25792E+01_JPRB, 0.21146E+01_JPRB, 0.17970E+01_JPRB, 0.15288E+01_JPRB, 0.13109E+01_JPRB ,&
&  0.11624E+01_JPRB, 0.10892E+01_JPRB, 0.10921E+01_JPRB, 0.10795E+01_JPRB, 0.10687E+01_JPRB, 0.10245E+01_JPRB, 0.95326E+00_JPRB ,&
&  0.87776E+00_JPRB, 0.90448E+00_JPRB, 0.96434E+00_JPRB, 0.10816E+01_JPRB, 0.11285E+01_JPRB, 0.11334E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.11154E+01_JPRB, 0.11814E+01_JPRB, 0.11610E+01_JPRB, 0.11423E+01_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.11526E+01_JPRB, 0.10722E+01_JPRB, 0.95498E+00_JPRB, 0.88785E+00_JPRB, 0.87786E+00_JPRB, 0.86716E+00_JPRB, 0.83504E+00_JPRB ,&
&  0.86157E+00_JPRB, 0.84542E+00_JPRB, 0.87004E+00_JPRB, 0.90549E+00_JPRB, 0.93690E+00_JPRB, 0.84492E+00_JPRB, 0.71680E+00_JPRB ,&
&  0.67682E+00_JPRB, 0.68595E+00_JPRB, 0.69520E+00_JPRB, 0.69558E+00_JPRB, 0.69999E+00_JPRB, 0.69685E+00_JPRB, 0.69830E+00_JPRB ,&
&  0.70486E+00_JPRB, 0.72005E+00_JPRB, 0.75798E+00_JPRB, 0.77379E+00_JPRB, 0.77906E+00_JPRB, 0.78129E+00_JPRB, 0.79061E+00_JPRB ,&
&  0.75501E+00_JPRB, 0.74037E+00_JPRB, 0.73064E+00_JPRB, 0.70578E+00_JPRB, 0.74089E+00_JPRB, 0.75451E+00_JPRB, 0.76450E+00_JPRB ,&
&  0.77686E+00_JPRB, 0.80037E+00_JPRB, 0.83389E+00_JPRB, 0.86710E+00_JPRB, 0.90043E+00_JPRB, 0.93555E+00_JPRB, 0.97100E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.10494E+01_JPRB, 0.10989E+01_JPRB, 0.11663E+01_JPRB, 0.13253E+01_JPRB, 0.13739E+01_JPRB, 0.15251E+01_JPRB ,&
&  0.16058E+01_JPRB, 0.17679E+01_JPRB, 0.19563E+01_JPRB, 0.19058E+01_JPRB, 0.20554E+01_JPRB, 0.22805E+01_JPRB, 0.22593E+01_JPRB ,&
&  0.23107E+01_JPRB, 0.22907E+01_JPRB, 0.24308E+01_JPRB, 0.26372E+01_JPRB, 0.29235E+01_JPRB, 0.31685E+01_JPRB, 0.33929E+01_JPRB ,&
&  0.37631E+01_JPRB, 0.44457E+01_JPRB, 0.47126E+01_JPRB, 0.45589E+01_JPRB, 0.43712E+01_JPRB, 0.41656E+01_JPRB, 0.38475E+01_JPRB ,&
&  0.35050E+01_JPRB, 0.31116E+01_JPRB, 0.26626E+01_JPRB, 0.22055E+01_JPRB, 0.18078E+01_JPRB, 0.15905E+01_JPRB, 0.13653E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.10212E+01_JPRB, 0.10360E+01_JPRB, 0.10620E+01_JPRB, 0.10561E+01_JPRB, 0.99197E+00_JPRB, 0.93582E+00_JPRB ,&
&  0.97335E+00_JPRB, 0.10996E+01_JPRB, 0.14408E+01_JPRB, 0.15498E+01_JPRB, 0.15599E+01_JPRB, 0.14761E+01_JPRB, 0.12082E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.12228E+01_JPRB, 0.12108E+01_JPRB, 0.12160E+01_JPRB, 0.11987E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.12107E+01_JPRB, 0.10868E+01_JPRB, 0.92525E+00_JPRB, 0.87119E+00_JPRB, 0.88665E+00_JPRB, 0.88312E+00_JPRB, 0.87646E+00_JPRB ,&
&  0.88845E+00_JPRB, 0.89468E+00_JPRB, 0.90511E+00_JPRB, 0.92210E+00_JPRB, 0.93570E+00_JPRB, 0.96931E+00_JPRB, 0.88728E+00_JPRB ,&
&  0.78942E+00_JPRB, 0.76647E+00_JPRB, 0.74175E+00_JPRB, 0.74492E+00_JPRB, 0.79237E+00_JPRB, 0.79852E+00_JPRB, 0.79910E+00_JPRB ,&
&  0.81089E+00_JPRB, 0.80601E+00_JPRB, 0.81747E+00_JPRB, 0.80804E+00_JPRB, 0.79382E+00_JPRB, 0.79653E+00_JPRB, 0.80145E+00_JPRB ,&
&  0.75488E+00_JPRB, 0.72004E+00_JPRB, 0.69877E+00_JPRB, 0.68867E+00_JPRB, 0.72152E+00_JPRB, 0.77060E+00_JPRB, 0.79873E+00_JPRB ,&
&  0.81596E+00_JPRB, 0.83739E+00_JPRB, 0.86554E+00_JPRB, 0.90129E+00_JPRB, 0.93985E+00_JPRB, 0.97730E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10675E+01_JPRB, 0.11146E+01_JPRB, 0.12049E+01_JPRB, 0.12849E+01_JPRB, 0.14280E+01_JPRB, 0.16733E+01_JPRB ,&
&  0.18262E+01_JPRB, 0.19821E+01_JPRB, 0.21831E+01_JPRB, 0.24909E+01_JPRB, 0.27024E+01_JPRB, 0.26516E+01_JPRB, 0.25810E+01_JPRB ,&
&  0.26161E+01_JPRB, 0.26370E+01_JPRB, 0.30437E+01_JPRB, 0.33358E+01_JPRB, 0.35375E+01_JPRB, 0.38037E+01_JPRB, 0.40045E+01_JPRB ,&
&  0.43693E+01_JPRB, 0.47552E+01_JPRB, 0.48275E+01_JPRB, 0.46631E+01_JPRB, 0.44601E+01_JPRB, 0.42711E+01_JPRB, 0.39223E+01_JPRB ,&
&  0.35453E+01_JPRB, 0.30838E+01_JPRB, 0.25382E+01_JPRB, 0.20838E+01_JPRB, 0.16794E+01_JPRB, 0.15640E+01_JPRB, 0.14295E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.10702E+01_JPRB, 0.11002E+01_JPRB, 0.12008E+01_JPRB, 0.11973E+01_JPRB, 0.11006E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.13507E+01_JPRB, 0.15971E+01_JPRB, 0.17708E+01_JPRB, 0.17772E+01_JPRB, 0.16294E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.12769E+01_JPRB, 0.12815E+01_JPRB, 0.12767E+01_JPRB, 0.12601E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.12938E+01_JPRB, 0.11342E+01_JPRB, 0.98472E+00_JPRB, 0.90125E+00_JPRB, 0.90850E+00_JPRB, 0.91695E+00_JPRB, 0.92009E+00_JPRB ,&
&  0.93770E+00_JPRB, 0.95315E+00_JPRB, 0.96786E+00_JPRB, 0.98079E+00_JPRB, 0.98603E+00_JPRB, 0.98495E+00_JPRB, 0.10263E+01_JPRB ,&
&  0.88126E+00_JPRB, 0.83998E+00_JPRB, 0.79812E+00_JPRB, 0.81597E+00_JPRB, 0.96714E+00_JPRB, 0.99018E+00_JPRB, 0.10294E+01_JPRB ,&
&  0.10194E+01_JPRB, 0.10242E+01_JPRB, 0.10019E+01_JPRB, 0.96235E+00_JPRB, 0.92368E+00_JPRB, 0.90571E+00_JPRB, 0.87140E+00_JPRB ,&
&  0.79805E+00_JPRB, 0.74842E+00_JPRB, 0.73422E+00_JPRB, 0.73513E+00_JPRB, 0.77010E+00_JPRB, 0.79937E+00_JPRB, 0.85441E+00_JPRB ,&
&  0.88208E+00_JPRB, 0.91214E+00_JPRB, 0.94567E+00_JPRB, 0.98041E+00_JPRB, 0.10081E+01_JPRB, 0.10393E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.11130E+01_JPRB, 0.11555E+01_JPRB, 0.12275E+01_JPRB, 0.13298E+01_JPRB, 0.13528E+01_JPRB, 0.15227E+01_JPRB, 0.19092E+01_JPRB ,&
&  0.22216E+01_JPRB, 0.23246E+01_JPRB, 0.27671E+01_JPRB, 0.38051E+01_JPRB, 0.40127E+01_JPRB, 0.38407E+01_JPRB, 0.36199E+01_JPRB ,&
&  0.35175E+01_JPRB, 0.34493E+01_JPRB, 0.35991E+01_JPRB, 0.38186E+01_JPRB, 0.40095E+01_JPRB, 0.43333E+01_JPRB, 0.45783E+01_JPRB ,&
&  0.46878E+01_JPRB, 0.47169E+01_JPRB, 0.46345E+01_JPRB, 0.44760E+01_JPRB, 0.41925E+01_JPRB, 0.39866E+01_JPRB, 0.37412E+01_JPRB ,&
&  0.34227E+01_JPRB, 0.30077E+01_JPRB, 0.23734E+01_JPRB, 0.19700E+01_JPRB, 0.15138E+01_JPRB, 0.13939E+01_JPRB, 0.16402E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.11858E+01_JPRB, 0.12796E+01_JPRB, 0.14537E+01_JPRB, 0.14997E+01_JPRB, 0.14697E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.13320E+01_JPRB, 0.18737E+01_JPRB, 0.20255E+01_JPRB, 0.19839E+01_JPRB, 0.17953E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.14502E+01_JPRB, 0.13875E+01_JPRB, 0.13799E+01_JPRB, 0.13750E+01_JPRB, 0.13589E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.13618E+01_JPRB, 0.11946E+01_JPRB, 0.10585E+01_JPRB, 0.96261E+00_JPRB, 0.95503E+00_JPRB, 0.97537E+00_JPRB, 0.99185E+00_JPRB ,&
&  0.10103E+01_JPRB, 0.10261E+01_JPRB, 0.10383E+01_JPRB, 0.10498E+01_JPRB, 0.10507E+01_JPRB, 0.10223E+01_JPRB, 0.99823E+00_JPRB ,&
&  0.98442E+00_JPRB, 0.91551E+00_JPRB, 0.88382E+00_JPRB, 0.87162E+00_JPRB, 0.10027E+01_JPRB, 0.11063E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.13383E+01_JPRB, 0.14226E+01_JPRB, 0.13929E+01_JPRB, 0.13079E+01_JPRB, 0.12189E+01_JPRB, 0.11453E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.91972E+00_JPRB, 0.86623E+00_JPRB, 0.85025E+00_JPRB, 0.88599E+00_JPRB, 0.94833E+00_JPRB, 0.88904E+00_JPRB, 0.95986E+00_JPRB ,&
&  0.99733E+00_JPRB, 0.10404E+01_JPRB, 0.10624E+01_JPRB, 0.10875E+01_JPRB, 0.11469E+01_JPRB, 0.12198E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.13525E+01_JPRB, 0.13723E+01_JPRB, 0.13667E+01_JPRB, 0.14465E+01_JPRB, 0.15351E+01_JPRB, 0.16543E+01_JPRB, 0.19465E+01_JPRB ,&
&  0.23884E+01_JPRB, 0.26284E+01_JPRB, 0.32324E+01_JPRB, 0.45055E+01_JPRB, 0.47790E+01_JPRB, 0.46004E+01_JPRB, 0.45891E+01_JPRB ,&
&  0.45588E+01_JPRB, 0.44407E+01_JPRB, 0.43745E+01_JPRB, 0.42947E+01_JPRB, 0.45088E+01_JPRB, 0.48491E+01_JPRB, 0.50486E+01_JPRB ,&
&  0.48310E+01_JPRB, 0.44822E+01_JPRB, 0.43216E+01_JPRB, 0.40371E+01_JPRB, 0.37669E+01_JPRB, 0.34521E+01_JPRB, 0.34791E+01_JPRB ,&
&  0.35117E+01_JPRB, 0.25480E+01_JPRB, 0.20979E+01_JPRB, 0.19726E+01_JPRB, 0.15074E+01_JPRB, 0.13251E+01_JPRB, 0.14453E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.11636E+01_JPRB, 0.13857E+01_JPRB, 0.16250E+01_JPRB, 0.18988E+01_JPRB, 0.20294E+01_JPRB, 0.19467E+01_JPRB ,&
&  0.17541E+01_JPRB, 0.18279E+01_JPRB, 0.24542E+01_JPRB, 0.25092E+01_JPRB, 0.22838E+01_JPRB, 0.20082E+01_JPRB, 0.17585E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.15976E+01_JPRB, 0.15614E+01_JPRB, 0.15278E+01_JPRB, 0.14881E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.15050E+01_JPRB, 0.13336E+01_JPRB, 0.11734E+01_JPRB, 0.10776E+01_JPRB, 0.10685E+01_JPRB, 0.10841E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10786E+01_JPRB, 0.10952E+01_JPRB, 0.11002E+01_JPRB, 0.10940E+01_JPRB, 0.10597E+01_JPRB, 0.10046E+01_JPRB ,&
&  0.97301E+00_JPRB, 0.98006E+00_JPRB, 0.92666E+00_JPRB, 0.95078E+00_JPRB, 0.10173E+01_JPRB, 0.12250E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.16384E+01_JPRB, 0.17923E+01_JPRB, 0.18374E+01_JPRB, 0.18359E+01_JPRB, 0.17525E+01_JPRB, 0.15992E+01_JPRB, 0.14456E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.12521E+01_JPRB, 0.11502E+01_JPRB, 0.12297E+01_JPRB, 0.12331E+01_JPRB, 0.11454E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.13085E+01_JPRB, 0.13223E+01_JPRB, 0.13781E+01_JPRB, 0.14381E+01_JPRB, 0.14578E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.14490E+01_JPRB, 0.14925E+01_JPRB, 0.14870E+01_JPRB, 0.15663E+01_JPRB, 0.18168E+01_JPRB, 0.22525E+01_JPRB ,&
&  0.27996E+01_JPRB, 0.28541E+01_JPRB, 0.30028E+01_JPRB, 0.43310E+01_JPRB, 0.54691E+01_JPRB, 0.54169E+01_JPRB, 0.52752E+01_JPRB ,&
&  0.52967E+01_JPRB, 0.49945E+01_JPRB, 0.46936E+01_JPRB, 0.45889E+01_JPRB, 0.51052E+01_JPRB, 0.54026E+01_JPRB, 0.54110E+01_JPRB ,&
&  0.50319E+01_JPRB, 0.45448E+01_JPRB, 0.43031E+01_JPRB, 0.40215E+01_JPRB, 0.35993E+01_JPRB, 0.34426E+01_JPRB, 0.35175E+01_JPRB ,&
&  0.34699E+01_JPRB, 0.30894E+01_JPRB, 0.23999E+01_JPRB, 0.21662E+01_JPRB, 0.17761E+01_JPRB, 0.16057E+01_JPRB, 0.17161E+01_JPRB ,&
&  0.15195E+01_JPRB, 0.15338E+01_JPRB, 0.16603E+01_JPRB, 0.17797E+01_JPRB, 0.28431E+01_JPRB, 0.31466E+01_JPRB, 0.34022E+01_JPRB ,&
&  0.32814E+01_JPRB, 0.30443E+01_JPRB, 0.36548E+01_JPRB, 0.34528E+01_JPRB, 0.29035E+01_JPRB, 0.23651E+01_JPRB, 0.19745E+01_JPRB ,&
&  0.18308E+01_JPRB, 0.18235E+01_JPRB, 0.17631E+01_JPRB, 0.17027E+01_JPRB, 0.16544E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.16340E+01_JPRB, 0.14522E+01_JPRB, 0.12905E+01_JPRB, 0.11835E+01_JPRB, 0.11452E+01_JPRB, 0.11521E+01_JPRB, 0.11514E+01_JPRB ,&
&  0.11270E+01_JPRB, 0.11217E+01_JPRB, 0.11243E+01_JPRB, 0.11059E+01_JPRB, 0.10798E+01_JPRB, 0.10467E+01_JPRB, 0.10003E+01_JPRB ,&
&  0.96721E+00_JPRB, 0.97679E+00_JPRB, 0.94353E+00_JPRB, 0.98186E+00_JPRB, 0.11030E+01_JPRB, 0.14026E+01_JPRB, 0.17774E+01_JPRB ,&
&  0.21260E+01_JPRB, 0.23068E+01_JPRB, 0.23860E+01_JPRB, 0.24171E+01_JPRB, 0.24502E+01_JPRB, 0.23715E+01_JPRB, 0.22075E+01_JPRB ,&
&  0.22137E+01_JPRB, 0.18622E+01_JPRB, 0.18887E+01_JPRB, 0.17790E+01_JPRB, 0.17691E+01_JPRB, 0.17156E+01_JPRB, 0.16505E+01_JPRB ,&
&  0.15971E+01_JPRB, 0.15719E+01_JPRB, 0.15402E+01_JPRB, 0.15392E+01_JPRB, 0.15552E+01_JPRB, 0.15666E+01_JPRB, 0.15901E+01_JPRB ,&
&  0.16188E+01_JPRB, 0.15808E+01_JPRB, 0.15155E+01_JPRB, 0.15214E+01_JPRB, 0.19371E+01_JPRB, 0.26531E+01_JPRB, 0.32981E+01_JPRB ,&
&  0.32983E+01_JPRB, 0.31153E+01_JPRB, 0.47593E+01_JPRB, 0.54520E+01_JPRB, 0.54004E+01_JPRB, 0.59342E+01_JPRB, 0.59293E+01_JPRB ,&
&  0.54199E+01_JPRB, 0.47581E+01_JPRB, 0.42493E+01_JPRB, 0.43797E+01_JPRB, 0.58477E+01_JPRB, 0.61248E+01_JPRB, 0.60186E+01_JPRB ,&
&  0.54367E+01_JPRB, 0.48668E+01_JPRB, 0.43235E+01_JPRB, 0.39455E+01_JPRB, 0.37381E+01_JPRB, 0.39042E+01_JPRB, 0.36991E+01_JPRB ,&
&  0.35010E+01_JPRB, 0.37714E+01_JPRB, 0.31587E+01_JPRB, 0.25188E+01_JPRB, 0.22040E+01_JPRB, 0.21291E+01_JPRB, 0.21154E+01_JPRB ,&
&  0.20248E+01_JPRB, 0.20621E+01_JPRB, 0.21648E+01_JPRB, 0.27590E+01_JPRB, 0.43434E+01_JPRB, 0.45279E+01_JPRB, 0.44709E+01_JPRB ,&
&  0.45254E+01_JPRB, 0.53749E+01_JPRB, 0.47731E+01_JPRB, 0.40745E+01_JPRB, 0.33476E+01_JPRB, 0.27371E+01_JPRB, 0.22916E+01_JPRB ,&
&  0.20491E+01_JPRB, 0.20125E+01_JPRB, 0.19631E+01_JPRB, 0.18756E+01_JPRB, 0.18041E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.18077E+01_JPRB, 0.15877E+01_JPRB, 0.14722E+01_JPRB, 0.13572E+01_JPRB, 0.12968E+01_JPRB, 0.12516E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11157E+01_JPRB, 0.11245E+01_JPRB, 0.11058E+01_JPRB, 0.10865E+01_JPRB, 0.10643E+01_JPRB, 0.10243E+01_JPRB, 0.98381E+00_JPRB ,&
&  0.97363E+00_JPRB, 0.97347E+00_JPRB, 0.99937E+00_JPRB, 0.11388E+01_JPRB, 0.13151E+01_JPRB, 0.15786E+01_JPRB, 0.20368E+01_JPRB ,&
&  0.24756E+01_JPRB, 0.27526E+01_JPRB, 0.28227E+01_JPRB, 0.30395E+01_JPRB, 0.32274E+01_JPRB, 0.33558E+01_JPRB, 0.32354E+01_JPRB ,&
&  0.28904E+01_JPRB, 0.28066E+01_JPRB, 0.27498E+01_JPRB, 0.25481E+01_JPRB, 0.23908E+01_JPRB, 0.22499E+01_JPRB, 0.20959E+01_JPRB ,&
&  0.19341E+01_JPRB, 0.17896E+01_JPRB, 0.17045E+01_JPRB, 0.16749E+01_JPRB, 0.16735E+01_JPRB, 0.17234E+01_JPRB, 0.17152E+01_JPRB ,&
&  0.16456E+01_JPRB, 0.16123E+01_JPRB, 0.15779E+01_JPRB, 0.19020E+01_JPRB, 0.27138E+01_JPRB, 0.39787E+01_JPRB, 0.43724E+01_JPRB ,&
&  0.46518E+01_JPRB, 0.43304E+01_JPRB, 0.47240E+01_JPRB, 0.54730E+01_JPRB, 0.46536E+01_JPRB, 0.51806E+01_JPRB, 0.54224E+01_JPRB ,&
&  0.48735E+01_JPRB, 0.45932E+01_JPRB, 0.50745E+01_JPRB, 0.58018E+01_JPRB, 0.66540E+01_JPRB, 0.79621E+01_JPRB, 0.77076E+01_JPRB ,&
&  0.64633E+01_JPRB, 0.54366E+01_JPRB, 0.44513E+01_JPRB, 0.41793E+01_JPRB, 0.37350E+01_JPRB, 0.36961E+01_JPRB, 0.30707E+01_JPRB ,&
&  0.32237E+01_JPRB, 0.28178E+01_JPRB, 0.29916E+01_JPRB, 0.26880E+01_JPRB, 0.26460E+01_JPRB, 0.24963E+01_JPRB, 0.23926E+01_JPRB ,&
&  0.24583E+01_JPRB, 0.26213E+01_JPRB, 0.30797E+01_JPRB, 0.44889E+01_JPRB, 0.62924E+01_JPRB, 0.53609E+01_JPRB, 0.62884E+01_JPRB ,&
&  0.78804E+01_JPRB, 0.74857E+01_JPRB, 0.52661E+01_JPRB, 0.43865E+01_JPRB, 0.35665E+01_JPRB, 0.29909E+01_JPRB, 0.26659E+01_JPRB ,&
&  0.23922E+01_JPRB, 0.22618E+01_JPRB, 0.21829E+01_JPRB, 0.20957E+01_JPRB, 0.19991E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.19080E+01_JPRB, 0.16928E+01_JPRB, 0.15960E+01_JPRB, 0.15469E+01_JPRB, 0.14546E+01_JPRB, 0.12881E+01_JPRB, 0.11880E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.11617E+01_JPRB, 0.10904E+01_JPRB, 0.10429E+01_JPRB, 0.10291E+01_JPRB, 0.10248E+01_JPRB, 0.99571E+00_JPRB ,&
&  0.10172E+01_JPRB, 0.11119E+01_JPRB, 0.12256E+01_JPRB, 0.13729E+01_JPRB, 0.16044E+01_JPRB, 0.18310E+01_JPRB, 0.20741E+01_JPRB ,&
&  0.30377E+01_JPRB, 0.32828E+01_JPRB, 0.31300E+01_JPRB, 0.33327E+01_JPRB, 0.37039E+01_JPRB, 0.40566E+01_JPRB, 0.40545E+01_JPRB ,&
&  0.40886E+01_JPRB, 0.39180E+01_JPRB, 0.32498E+01_JPRB, 0.27742E+01_JPRB, 0.24598E+01_JPRB, 0.22787E+01_JPRB, 0.20846E+01_JPRB ,&
&  0.18542E+01_JPRB, 0.16707E+01_JPRB, 0.16048E+01_JPRB, 0.15740E+01_JPRB, 0.15918E+01_JPRB, 0.15214E+01_JPRB, 0.15155E+01_JPRB ,&
&  0.15495E+01_JPRB, 0.16306E+01_JPRB, 0.17678E+01_JPRB, 0.23535E+01_JPRB, 0.36288E+01_JPRB, 0.51118E+01_JPRB, 0.55269E+01_JPRB ,&
&  0.49711E+01_JPRB, 0.43980E+01_JPRB, 0.48296E+01_JPRB, 0.53380E+01_JPRB, 0.52749E+01_JPRB, 0.50343E+01_JPRB, 0.47179E+01_JPRB ,&
&  0.39449E+01_JPRB, 0.39992E+01_JPRB, 0.48009E+01_JPRB, 0.60915E+01_JPRB, 0.79259E+01_JPRB, 0.99315E+01_JPRB, 0.96369E+01_JPRB ,&
&  0.77759E+01_JPRB, 0.62856E+01_JPRB, 0.51798E+01_JPRB, 0.42457E+01_JPRB, 0.42312E+01_JPRB, 0.33332E+01_JPRB, 0.38332E+01_JPRB ,&
&  0.39504E+01_JPRB, 0.38346E+01_JPRB, 0.32052E+01_JPRB, 0.26842E+01_JPRB, 0.23522E+01_JPRB, 0.27031E+01_JPRB, 0.27517E+01_JPRB ,&
&  0.31163E+01_JPRB, 0.34989E+01_JPRB, 0.61107E+01_JPRB, 0.72828E+01_JPRB, 0.76742E+01_JPRB, 0.73028E+01_JPRB, 0.93420E+01_JPRB ,&
&  0.86366E+01_JPRB, 0.73685E+01_JPRB, 0.57657E+01_JPRB, 0.46717E+01_JPRB, 0.38219E+01_JPRB, 0.33294E+01_JPRB, 0.30218E+01_JPRB ,&
&  0.27042E+01_JPRB, 0.25341E+01_JPRB, 0.24077E+01_JPRB, 0.22817E+01_JPRB, 0.21308E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.18718E+01_JPRB, 0.17564E+01_JPRB, 0.17208E+01_JPRB, 0.15868E+01_JPRB, 0.13756E+01_JPRB, 0.12354E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.11458E+01_JPRB, 0.10768E+01_JPRB, 0.10292E+01_JPRB, 0.10332E+01_JPRB, 0.10446E+01_JPRB, 0.10642E+01_JPRB ,&
&  0.11664E+01_JPRB, 0.14066E+01_JPRB, 0.16024E+01_JPRB, 0.18024E+01_JPRB, 0.19710E+01_JPRB, 0.21074E+01_JPRB, 0.24380E+01_JPRB ,&
&  0.36120E+01_JPRB, 0.33636E+01_JPRB, 0.27774E+01_JPRB, 0.29459E+01_JPRB, 0.37533E+01_JPRB, 0.39339E+01_JPRB, 0.40096E+01_JPRB ,&
&  0.39510E+01_JPRB, 0.34976E+01_JPRB, 0.29355E+01_JPRB, 0.24594E+01_JPRB, 0.21602E+01_JPRB, 0.19720E+01_JPRB, 0.17372E+01_JPRB ,&
&  0.15883E+01_JPRB, 0.15453E+01_JPRB, 0.15686E+01_JPRB, 0.15880E+01_JPRB, 0.15610E+01_JPRB, 0.15429E+01_JPRB, 0.15716E+01_JPRB ,&
&  0.16183E+01_JPRB, 0.17213E+01_JPRB, 0.19608E+01_JPRB, 0.27544E+01_JPRB, 0.41264E+01_JPRB, 0.57071E+01_JPRB, 0.65409E+01_JPRB ,&
&  0.53549E+01_JPRB, 0.50478E+01_JPRB, 0.52811E+01_JPRB, 0.64852E+01_JPRB, 0.51627E+01_JPRB, 0.48484E+01_JPRB, 0.46327E+01_JPRB ,&
&  0.42669E+01_JPRB, 0.48709E+01_JPRB, 0.63600E+01_JPRB, 0.79018E+01_JPRB, 0.81244E+01_JPRB, 0.94044E+01_JPRB, 0.10904E+02_JPRB ,&
&  0.81132E+01_JPRB, 0.64678E+01_JPRB, 0.56039E+01_JPRB, 0.53389E+01_JPRB, 0.36136E+01_JPRB, 0.26643E+01_JPRB, 0.39471E+01_JPRB ,&
&  0.39416E+01_JPRB, 0.22966E+01_JPRB, 0.17236E+01_JPRB, 0.19852E+01_JPRB, 0.15285E+01_JPRB, 0.18488E+01_JPRB, 0.29406E+01_JPRB ,&
&  0.38665E+01_JPRB, 0.47834E+01_JPRB, 0.97583E+01_JPRB, 0.89520E+01_JPRB, 0.93475E+01_JPRB, 0.10432E+02_JPRB, 0.10846E+02_JPRB ,&
&  0.94769E+01_JPRB, 0.59608E+01_JPRB, 0.53139E+01_JPRB, 0.46112E+01_JPRB, 0.37409E+01_JPRB, 0.31124E+01_JPRB, 0.26709E+01_JPRB ,&
&  0.23315E+01_JPRB, 0.21375E+01_JPRB, 0.20283E+01_JPRB, 0.19803E+01_JPRB, 0.19801E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.16049E+01_JPRB, 0.15662E+01_JPRB, 0.15015E+01_JPRB, 0.13427E+01_JPRB, 0.12257E+01_JPRB, 0.11893E+01_JPRB, 0.11785E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.10833E+01_JPRB, 0.10400E+01_JPRB, 0.10490E+01_JPRB, 0.10687E+01_JPRB, 0.10945E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.12667E+01_JPRB, 0.15023E+01_JPRB, 0.20431E+01_JPRB, 0.24412E+01_JPRB, 0.26626E+01_JPRB, 0.22842E+01_JPRB, 0.29730E+01_JPRB ,&
&  0.37390E+01_JPRB, 0.29661E+01_JPRB, 0.24598E+01_JPRB, 0.27769E+01_JPRB, 0.31987E+01_JPRB, 0.32418E+01_JPRB, 0.32523E+01_JPRB ,&
&  0.30967E+01_JPRB, 0.27810E+01_JPRB, 0.24868E+01_JPRB, 0.23035E+01_JPRB, 0.20332E+01_JPRB, 0.17342E+01_JPRB, 0.16278E+01_JPRB ,&
&  0.15941E+01_JPRB, 0.16567E+01_JPRB, 0.17281E+01_JPRB, 0.16942E+01_JPRB, 0.16994E+01_JPRB, 0.16954E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.17879E+01_JPRB, 0.19328E+01_JPRB, 0.23199E+01_JPRB, 0.34272E+01_JPRB, 0.53463E+01_JPRB, 0.74430E+01_JPRB, 0.62495E+01_JPRB ,&
&  0.48232E+01_JPRB, 0.44605E+01_JPRB, 0.57510E+01_JPRB, 0.61158E+01_JPRB, 0.49360E+01_JPRB, 0.49377E+01_JPRB, 0.47694E+01_JPRB ,&
&  0.53061E+01_JPRB, 0.61370E+01_JPRB, 0.65190E+01_JPRB, 0.78512E+01_JPRB, 0.93755E+01_JPRB, 0.95007E+01_JPRB, 0.97457E+01_JPRB ,&
&  0.81483E+01_JPRB, 0.63645E+01_JPRB, 0.47879E+01_JPRB, 0.39842E+01_JPRB, 0.46923E+01_JPRB, 0.36369E+01_JPRB, 0.23877E+01_JPRB ,&
&  0.18897E+01_JPRB, 0.15088E+01_JPRB, 0.12266E+01_JPRB, 0.12106E+01_JPRB, 0.11168E+01_JPRB, 0.16454E+01_JPRB, 0.28606E+01_JPRB ,&
&  0.57096E+01_JPRB, 0.80346E+01_JPRB, 0.10677E+02_JPRB, 0.10817E+02_JPRB, 0.99351E+01_JPRB, 0.11044E+02_JPRB, 0.11843E+02_JPRB ,&
&  0.73008E+01_JPRB, 0.51699E+01_JPRB, 0.48093E+01_JPRB, 0.38581E+01_JPRB, 0.28605E+01_JPRB, 0.22949E+01_JPRB, 0.19618E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.16801E+01_JPRB, 0.16151E+01_JPRB, 0.16166E+01_JPRB, 0.16445E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.13662E+01_JPRB, 0.13382E+01_JPRB, 0.13110E+01_JPRB, 0.12289E+01_JPRB, 0.11566E+01_JPRB, 0.11655E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.10705E+01_JPRB, 0.10674E+01_JPRB, 0.10998E+01_JPRB, 0.11408E+01_JPRB, 0.12157E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.17753E+01_JPRB, 0.25098E+01_JPRB, 0.32301E+01_JPRB, 0.36584E+01_JPRB, 0.28803E+01_JPRB, 0.35156E+01_JPRB ,&
&  0.36117E+01_JPRB, 0.27682E+01_JPRB, 0.25078E+01_JPRB, 0.27676E+01_JPRB, 0.30002E+01_JPRB, 0.27170E+01_JPRB, 0.26003E+01_JPRB ,&
&  0.24873E+01_JPRB, 0.23403E+01_JPRB, 0.21973E+01_JPRB, 0.20984E+01_JPRB, 0.18745E+01_JPRB, 0.18469E+01_JPRB, 0.19079E+01_JPRB ,&
&  0.19383E+01_JPRB, 0.20057E+01_JPRB, 0.19737E+01_JPRB, 0.19383E+01_JPRB, 0.19301E+01_JPRB, 0.19394E+01_JPRB, 0.19875E+01_JPRB ,&
&  0.20954E+01_JPRB, 0.23745E+01_JPRB, 0.31316E+01_JPRB, 0.48459E+01_JPRB, 0.62768E+01_JPRB, 0.63011E+01_JPRB, 0.57648E+01_JPRB ,&
&  0.53071E+01_JPRB, 0.54444E+01_JPRB, 0.56144E+01_JPRB, 0.63324E+01_JPRB, 0.50703E+01_JPRB, 0.48919E+01_JPRB, 0.48747E+01_JPRB ,&
&  0.54459E+01_JPRB, 0.69359E+01_JPRB, 0.73189E+01_JPRB, 0.82229E+01_JPRB, 0.11382E+02_JPRB, 0.11552E+02_JPRB, 0.91155E+01_JPRB ,&
&  0.79884E+01_JPRB, 0.62357E+01_JPRB, 0.52266E+01_JPRB, 0.47766E+01_JPRB, 0.72871E+01_JPRB, 0.78974E+01_JPRB, 0.27411E+01_JPRB ,&
&  0.16521E+01_JPRB, 0.13054E+01_JPRB, 0.97735E+00_JPRB, 0.95178E+00_JPRB, 0.10830E+01_JPRB, 0.17006E+01_JPRB, 0.44985E+01_JPRB ,&
&  0.53617E+01_JPRB, 0.87556E+01_JPRB, 0.98805E+01_JPRB, 0.10624E+02_JPRB, 0.98769E+01_JPRB, 0.98376E+01_JPRB, 0.75179E+01_JPRB ,&
&  0.60356E+01_JPRB, 0.46494E+01_JPRB, 0.35037E+01_JPRB, 0.26984E+01_JPRB, 0.21158E+01_JPRB, 0.18033E+01_JPRB, 0.16480E+01_JPRB ,&
&  0.15797E+01_JPRB, 0.15415E+01_JPRB, 0.14879E+01_JPRB, 0.14355E+01_JPRB, 0.13751E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.11630E+01_JPRB, 0.11917E+01_JPRB, 0.12260E+01_JPRB, 0.12211E+01_JPRB, 0.12172E+01_JPRB, 0.12557E+01_JPRB, 0.12367E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.11491E+01_JPRB, 0.11678E+01_JPRB, 0.12130E+01_JPRB, 0.12861E+01_JPRB, 0.13965E+01_JPRB, 0.15287E+01_JPRB ,&
&  0.17788E+01_JPRB, 0.23449E+01_JPRB, 0.31818E+01_JPRB, 0.42397E+01_JPRB, 0.45633E+01_JPRB, 0.41057E+01_JPRB, 0.41521E+01_JPRB ,&
&  0.36716E+01_JPRB, 0.28772E+01_JPRB, 0.25439E+01_JPRB, 0.27462E+01_JPRB, 0.27340E+01_JPRB, 0.25017E+01_JPRB, 0.22433E+01_JPRB ,&
&  0.22485E+01_JPRB, 0.22577E+01_JPRB, 0.22186E+01_JPRB, 0.22330E+01_JPRB, 0.22949E+01_JPRB, 0.23769E+01_JPRB, 0.24482E+01_JPRB ,&
&  0.24810E+01_JPRB, 0.24176E+01_JPRB, 0.23163E+01_JPRB, 0.22556E+01_JPRB, 0.22464E+01_JPRB, 0.22948E+01_JPRB, 0.24067E+01_JPRB ,&
&  0.26364E+01_JPRB, 0.32001E+01_JPRB, 0.44291E+01_JPRB, 0.59763E+01_JPRB, 0.57995E+01_JPRB, 0.62495E+01_JPRB, 0.57867E+01_JPRB ,&
&  0.53515E+01_JPRB, 0.53974E+01_JPRB, 0.52157E+01_JPRB, 0.56682E+01_JPRB, 0.53125E+01_JPRB, 0.52279E+01_JPRB, 0.52092E+01_JPRB ,&
&  0.67023E+01_JPRB, 0.77408E+01_JPRB, 0.90839E+01_JPRB, 0.97010E+01_JPRB, 0.11691E+02_JPRB, 0.14597E+02_JPRB, 0.93311E+01_JPRB ,&
&  0.70032E+01_JPRB, 0.66598E+01_JPRB, 0.57637E+01_JPRB, 0.53913E+01_JPRB, 0.68443E+01_JPRB, 0.96584E+01_JPRB, 0.72655E+01_JPRB ,&
&  0.22884E+01_JPRB, 0.14324E+01_JPRB, 0.12695E+01_JPRB, 0.12766E+01_JPRB, 0.10770E+01_JPRB, 0.17809E+01_JPRB, 0.66156E+01_JPRB ,&
&  0.57305E+01_JPRB, 0.76432E+01_JPRB, 0.85678E+01_JPRB, 0.91996E+01_JPRB, 0.90797E+01_JPRB, 0.67370E+01_JPRB, 0.47682E+01_JPRB ,&
&  0.31022E+01_JPRB, 0.22528E+01_JPRB, 0.19220E+01_JPRB, 0.16908E+01_JPRB, 0.14313E+01_JPRB, 0.13031E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.12136E+01_JPRB, 0.11845E+01_JPRB, 0.11468E+01_JPRB, 0.11463E+01_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.11198E+01_JPRB, 0.12024E+01_JPRB, 0.12753E+01_JPRB, 0.13495E+01_JPRB, 0.14009E+01_JPRB, 0.14037E+01_JPRB, 0.13752E+01_JPRB ,&
&  0.13575E+01_JPRB, 0.13417E+01_JPRB, 0.13708E+01_JPRB, 0.14284E+01_JPRB, 0.15554E+01_JPRB, 0.17521E+01_JPRB, 0.20582E+01_JPRB ,&
&  0.25488E+01_JPRB, 0.32528E+01_JPRB, 0.41089E+01_JPRB, 0.49250E+01_JPRB, 0.50243E+01_JPRB, 0.43335E+01_JPRB, 0.42438E+01_JPRB ,&
&  0.38623E+01_JPRB, 0.31456E+01_JPRB, 0.26320E+01_JPRB, 0.28371E+01_JPRB, 0.29352E+01_JPRB, 0.28367E+01_JPRB, 0.25766E+01_JPRB ,&
&  0.25838E+01_JPRB, 0.26177E+01_JPRB, 0.26604E+01_JPRB, 0.27515E+01_JPRB, 0.28380E+01_JPRB, 0.29019E+01_JPRB, 0.29150E+01_JPRB ,&
&  0.28218E+01_JPRB, 0.27002E+01_JPRB, 0.26420E+01_JPRB, 0.26562E+01_JPRB, 0.27225E+01_JPRB, 0.28174E+01_JPRB, 0.30294E+01_JPRB ,&
&  0.34906E+01_JPRB, 0.43256E+01_JPRB, 0.54335E+01_JPRB, 0.59011E+01_JPRB, 0.57961E+01_JPRB, 0.56592E+01_JPRB, 0.53888E+01_JPRB ,&
&  0.51832E+01_JPRB, 0.53117E+01_JPRB, 0.53262E+01_JPRB, 0.53815E+01_JPRB, 0.53733E+01_JPRB, 0.56714E+01_JPRB, 0.60303E+01_JPRB ,&
&  0.79172E+01_JPRB, 0.94149E+01_JPRB, 0.96813E+01_JPRB, 0.97065E+01_JPRB, 0.10220E+02_JPRB, 0.11635E+02_JPRB, 0.10691E+02_JPRB ,&
&  0.81629E+01_JPRB, 0.62089E+01_JPRB, 0.55646E+01_JPRB, 0.59159E+01_JPRB, 0.62765E+01_JPRB, 0.74002E+01_JPRB, 0.91694E+01_JPRB ,&
&  0.82164E+01_JPRB, 0.31532E+01_JPRB, 0.20304E+01_JPRB, 0.29344E+01_JPRB, 0.17745E+01_JPRB, 0.18857E+01_JPRB, 0.33677E+01_JPRB ,&
&  0.43403E+01_JPRB, 0.53040E+01_JPRB, 0.60233E+01_JPRB, 0.69056E+01_JPRB, 0.58629E+01_JPRB, 0.41437E+01_JPRB, 0.27599E+01_JPRB ,&
&  0.18878E+01_JPRB, 0.15516E+01_JPRB, 0.14680E+01_JPRB, 0.12785E+01_JPRB, 0.11488E+01_JPRB, 0.11177E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10389E+01_JPRB, 0.10591E+01_JPRB, 0.10607E+01_JPRB, 0.10601E+01_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.12191E+01_JPRB, 0.13190E+01_JPRB, 0.14575E+01_JPRB, 0.15449E+01_JPRB, 0.15751E+01_JPRB, 0.15881E+01_JPRB, 0.16243E+01_JPRB ,&
&  0.16642E+01_JPRB, 0.17167E+01_JPRB, 0.18145E+01_JPRB, 0.19712E+01_JPRB, 0.22397E+01_JPRB, 0.25828E+01_JPRB, 0.30283E+01_JPRB ,&
&  0.35830E+01_JPRB, 0.41998E+01_JPRB, 0.48463E+01_JPRB, 0.54729E+01_JPRB, 0.58821E+01_JPRB, 0.62318E+01_JPRB, 0.40869E+01_JPRB ,&
&  0.33765E+01_JPRB, 0.36358E+01_JPRB, 0.28760E+01_JPRB, 0.31605E+01_JPRB, 0.32862E+01_JPRB, 0.32825E+01_JPRB, 0.29621E+01_JPRB ,&
&  0.29163E+01_JPRB, 0.29606E+01_JPRB, 0.30247E+01_JPRB, 0.30606E+01_JPRB, 0.30802E+01_JPRB, 0.30910E+01_JPRB, 0.30539E+01_JPRB ,&
&  0.29998E+01_JPRB, 0.29847E+01_JPRB, 0.30374E+01_JPRB, 0.31284E+01_JPRB, 0.32589E+01_JPRB, 0.34673E+01_JPRB, 0.38210E+01_JPRB ,&
&  0.43369E+01_JPRB, 0.49636E+01_JPRB, 0.55270E+01_JPRB, 0.53817E+01_JPRB, 0.52747E+01_JPRB, 0.51500E+01_JPRB, 0.51750E+01_JPRB ,&
&  0.50159E+01_JPRB, 0.47585E+01_JPRB, 0.51339E+01_JPRB, 0.55357E+01_JPRB, 0.56711E+01_JPRB, 0.59739E+01_JPRB, 0.67022E+01_JPRB ,&
&  0.87165E+01_JPRB, 0.98714E+01_JPRB, 0.10302E+02_JPRB, 0.89225E+01_JPRB, 0.84689E+01_JPRB, 0.92726E+01_JPRB, 0.95444E+01_JPRB ,&
&  0.82908E+01_JPRB, 0.67147E+01_JPRB, 0.59629E+01_JPRB, 0.55933E+01_JPRB, 0.53395E+01_JPRB, 0.57981E+01_JPRB, 0.64578E+01_JPRB ,&
&  0.69437E+01_JPRB, 0.77179E+01_JPRB, 0.61834E+01_JPRB, 0.36382E+01_JPRB, 0.25451E+01_JPRB, 0.25550E+01_JPRB, 0.31467E+01_JPRB ,&
&  0.35759E+01_JPRB, 0.37565E+01_JPRB, 0.37146E+01_JPRB, 0.53772E+01_JPRB, 0.31091E+01_JPRB, 0.21431E+01_JPRB, 0.15280E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12616E+01_JPRB, 0.12522E+01_JPRB, 0.11522E+01_JPRB, 0.10719E+01_JPRB, 0.10945E+01_JPRB, 0.10436E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.11049E+01_JPRB, 0.11166E+01_JPRB, 0.11299E+01_JPRB, 0.11725E+01_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.14478E+01_JPRB, 0.15428E+01_JPRB, 0.16638E+01_JPRB, 0.17457E+01_JPRB, 0.18416E+01_JPRB, 0.19271E+01_JPRB, 0.20560E+01_JPRB ,&
&  0.22043E+01_JPRB, 0.23799E+01_JPRB, 0.25794E+01_JPRB, 0.28371E+01_JPRB, 0.31779E+01_JPRB, 0.35552E+01_JPRB, 0.39756E+01_JPRB ,&
&  0.43759E+01_JPRB, 0.47334E+01_JPRB, 0.50801E+01_JPRB, 0.54488E+01_JPRB, 0.57437E+01_JPRB, 0.61203E+01_JPRB, 0.52967E+01_JPRB ,&
&  0.31366E+01_JPRB, 0.43830E+01_JPRB, 0.30898E+01_JPRB, 0.34881E+01_JPRB, 0.35713E+01_JPRB, 0.37152E+01_JPRB, 0.32808E+01_JPRB ,&
&  0.30560E+01_JPRB, 0.30547E+01_JPRB, 0.30576E+01_JPRB, 0.30893E+01_JPRB, 0.31117E+01_JPRB, 0.31361E+01_JPRB, 0.31551E+01_JPRB ,&
&  0.32020E+01_JPRB, 0.32995E+01_JPRB, 0.34321E+01_JPRB, 0.35753E+01_JPRB, 0.37931E+01_JPRB, 0.40910E+01_JPRB, 0.43857E+01_JPRB ,&
&  0.46949E+01_JPRB, 0.50061E+01_JPRB, 0.50977E+01_JPRB, 0.49503E+01_JPRB, 0.48831E+01_JPRB, 0.50080E+01_JPRB, 0.52784E+01_JPRB ,&
&  0.51977E+01_JPRB, 0.50236E+01_JPRB, 0.53274E+01_JPRB, 0.55087E+01_JPRB, 0.56082E+01_JPRB, 0.63883E+01_JPRB, 0.70342E+01_JPRB ,&
&  0.85463E+01_JPRB, 0.88065E+01_JPRB, 0.10035E+02_JPRB, 0.84135E+01_JPRB, 0.72087E+01_JPRB, 0.75272E+01_JPRB, 0.70015E+01_JPRB ,&
&  0.59584E+01_JPRB, 0.56179E+01_JPRB, 0.50108E+01_JPRB, 0.43135E+01_JPRB, 0.43612E+01_JPRB, 0.45950E+01_JPRB, 0.51463E+01_JPRB ,&
&  0.56660E+01_JPRB, 0.57821E+01_JPRB, 0.42980E+01_JPRB, 0.27013E+01_JPRB, 0.24203E+01_JPRB, 0.25880E+01_JPRB, 0.29841E+01_JPRB ,&
&  0.30012E+01_JPRB, 0.29747E+01_JPRB, 0.31422E+01_JPRB, 0.29069E+01_JPRB, 0.17184E+01_JPRB, 0.12925E+01_JPRB, 0.11264E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.11764E+01_JPRB, 0.11779E+01_JPRB, 0.11481E+01_JPRB, 0.10743E+01_JPRB, 0.11244E+01_JPRB, 0.11348E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.12392E+01_JPRB, 0.12895E+01_JPRB, 0.13395E+01_JPRB, 0.13842E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.17219E+01_JPRB, 0.18547E+01_JPRB, 0.20581E+01_JPRB, 0.22417E+01_JPRB, 0.23985E+01_JPRB, 0.25313E+01_JPRB, 0.26425E+01_JPRB ,&
&  0.27507E+01_JPRB, 0.28930E+01_JPRB, 0.31456E+01_JPRB, 0.34092E+01_JPRB, 0.37366E+01_JPRB, 0.41047E+01_JPRB, 0.44544E+01_JPRB ,&
&  0.47524E+01_JPRB, 0.50300E+01_JPRB, 0.53103E+01_JPRB, 0.56936E+01_JPRB, 0.61595E+01_JPRB, 0.69582E+01_JPRB, 0.66907E+01_JPRB ,&
&  0.47301E+01_JPRB, 0.34770E+01_JPRB, 0.33005E+01_JPRB, 0.33942E+01_JPRB, 0.33228E+01_JPRB, 0.36308E+01_JPRB, 0.36917E+01_JPRB ,&
&  0.33004E+01_JPRB, 0.31530E+01_JPRB, 0.31261E+01_JPRB, 0.31322E+01_JPRB, 0.30811E+01_JPRB, 0.31556E+01_JPRB, 0.32414E+01_JPRB ,&
&  0.33811E+01_JPRB, 0.35492E+01_JPRB, 0.37197E+01_JPRB, 0.39229E+01_JPRB, 0.41446E+01_JPRB, 0.43872E+01_JPRB, 0.45976E+01_JPRB ,&
&  0.47679E+01_JPRB, 0.49136E+01_JPRB, 0.49043E+01_JPRB, 0.47505E+01_JPRB, 0.48443E+01_JPRB, 0.50657E+01_JPRB, 0.51892E+01_JPRB ,&
&  0.52347E+01_JPRB, 0.54108E+01_JPRB, 0.57185E+01_JPRB, 0.58712E+01_JPRB, 0.58171E+01_JPRB, 0.66010E+01_JPRB, 0.70729E+01_JPRB ,&
&  0.77591E+01_JPRB, 0.78776E+01_JPRB, 0.91053E+01_JPRB, 0.77374E+01_JPRB, 0.67265E+01_JPRB, 0.63454E+01_JPRB, 0.56435E+01_JPRB ,&
&  0.49256E+01_JPRB, 0.46732E+01_JPRB, 0.42257E+01_JPRB, 0.32356E+01_JPRB, 0.34945E+01_JPRB, 0.34303E+01_JPRB, 0.45168E+01_JPRB ,&
&  0.52498E+01_JPRB, 0.57733E+01_JPRB, 0.35070E+01_JPRB, 0.26672E+01_JPRB, 0.23639E+01_JPRB, 0.26085E+01_JPRB, 0.26675E+01_JPRB ,&
&  0.24953E+01_JPRB, 0.22050E+01_JPRB, 0.20389E+01_JPRB, 0.19056E+01_JPRB, 0.12289E+01_JPRB, 0.10527E+01_JPRB, 0.10078E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.11357E+01_JPRB, 0.11488E+01_JPRB, 0.11360E+01_JPRB, 0.11523E+01_JPRB, 0.12385E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.14347E+01_JPRB, 0.15036E+01_JPRB, 0.15487E+01_JPRB, 0.16115E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.17898E+01_JPRB, 0.19261E+01_JPRB, 0.21369E+01_JPRB, 0.23393E+01_JPRB, 0.25092E+01_JPRB, 0.26206E+01_JPRB, 0.26858E+01_JPRB ,&
&  0.27934E+01_JPRB, 0.29485E+01_JPRB, 0.31668E+01_JPRB, 0.33726E+01_JPRB, 0.36171E+01_JPRB, 0.38673E+01_JPRB, 0.41149E+01_JPRB ,&
&  0.43431E+01_JPRB, 0.45515E+01_JPRB, 0.48128E+01_JPRB, 0.51422E+01_JPRB, 0.55308E+01_JPRB, 0.61313E+01_JPRB, 0.61543E+01_JPRB ,&
&  0.47788E+01_JPRB, 0.28001E+01_JPRB, 0.30387E+01_JPRB, 0.30644E+01_JPRB, 0.31701E+01_JPRB, 0.35459E+01_JPRB, 0.36198E+01_JPRB ,&
&  0.31076E+01_JPRB, 0.30310E+01_JPRB, 0.31232E+01_JPRB, 0.31062E+01_JPRB, 0.30150E+01_JPRB, 0.31843E+01_JPRB, 0.33317E+01_JPRB ,&
&  0.35039E+01_JPRB, 0.36567E+01_JPRB, 0.38028E+01_JPRB, 0.39648E+01_JPRB, 0.41166E+01_JPRB, 0.42857E+01_JPRB, 0.44780E+01_JPRB ,&
&  0.46794E+01_JPRB, 0.48307E+01_JPRB, 0.48041E+01_JPRB, 0.47745E+01_JPRB, 0.48553E+01_JPRB, 0.50768E+01_JPRB, 0.52847E+01_JPRB ,&
&  0.54375E+01_JPRB, 0.54993E+01_JPRB, 0.59306E+01_JPRB, 0.64545E+01_JPRB, 0.66965E+01_JPRB, 0.65097E+01_JPRB, 0.68890E+01_JPRB ,&
&  0.71416E+01_JPRB, 0.70253E+01_JPRB, 0.74498E+01_JPRB, 0.82092E+01_JPRB, 0.57880E+01_JPRB, 0.55940E+01_JPRB, 0.45873E+01_JPRB ,&
&  0.46550E+01_JPRB, 0.42749E+01_JPRB, 0.37564E+01_JPRB, 0.31545E+01_JPRB, 0.28371E+01_JPRB, 0.28165E+01_JPRB, 0.36689E+01_JPRB ,&
&  0.46768E+01_JPRB, 0.41157E+01_JPRB, 0.28460E+01_JPRB, 0.24085E+01_JPRB, 0.24085E+01_JPRB, 0.29938E+01_JPRB, 0.29438E+01_JPRB ,&
&  0.24505E+01_JPRB, 0.20639E+01_JPRB, 0.19434E+01_JPRB, 0.24550E+01_JPRB, 0.14722E+01_JPRB, 0.10156E+01_JPRB, 0.92890E+00_JPRB ,&
&  0.10045E+01_JPRB, 0.10820E+01_JPRB, 0.11525E+01_JPRB, 0.12060E+01_JPRB, 0.12563E+01_JPRB, 0.13298E+01_JPRB, 0.13670E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.14893E+01_JPRB, 0.15799E+01_JPRB, 0.16349E+01_JPRB, 0.17042E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.17773E+01_JPRB, 0.18831E+01_JPRB, 0.19963E+01_JPRB, 0.21270E+01_JPRB, 0.22677E+01_JPRB, 0.23821E+01_JPRB, 0.24594E+01_JPRB ,&
&  0.25520E+01_JPRB, 0.26737E+01_JPRB, 0.28179E+01_JPRB, 0.29411E+01_JPRB, 0.31232E+01_JPRB, 0.32944E+01_JPRB, 0.34645E+01_JPRB ,&
&  0.36318E+01_JPRB, 0.38196E+01_JPRB, 0.40559E+01_JPRB, 0.42535E+01_JPRB, 0.43585E+01_JPRB, 0.44386E+01_JPRB, 0.45229E+01_JPRB ,&
&  0.45878E+01_JPRB, 0.38404E+01_JPRB, 0.33050E+01_JPRB, 0.25271E+01_JPRB, 0.21078E+01_JPRB, 0.26564E+01_JPRB, 0.26033E+01_JPRB ,&
&  0.25786E+01_JPRB, 0.25257E+01_JPRB, 0.25460E+01_JPRB, 0.26387E+01_JPRB, 0.27967E+01_JPRB, 0.29710E+01_JPRB, 0.31248E+01_JPRB ,&
&  0.32541E+01_JPRB, 0.33571E+01_JPRB, 0.34428E+01_JPRB, 0.35434E+01_JPRB, 0.36648E+01_JPRB, 0.37811E+01_JPRB, 0.39025E+01_JPRB ,&
&  0.40646E+01_JPRB, 0.42645E+01_JPRB, 0.44013E+01_JPRB, 0.45619E+01_JPRB, 0.46766E+01_JPRB, 0.49856E+01_JPRB, 0.53151E+01_JPRB ,&
&  0.55349E+01_JPRB, 0.56689E+01_JPRB, 0.60588E+01_JPRB, 0.65015E+01_JPRB, 0.65710E+01_JPRB, 0.59114E+01_JPRB, 0.61189E+01_JPRB ,&
&  0.64017E+01_JPRB, 0.61803E+01_JPRB, 0.60452E+01_JPRB, 0.77221E+01_JPRB, 0.48555E+01_JPRB, 0.51561E+01_JPRB, 0.49933E+01_JPRB ,&
&  0.42922E+01_JPRB, 0.40099E+01_JPRB, 0.34478E+01_JPRB, 0.26553E+01_JPRB, 0.23147E+01_JPRB, 0.23557E+01_JPRB, 0.30654E+01_JPRB ,&
&  0.35711E+01_JPRB, 0.29115E+01_JPRB, 0.21245E+01_JPRB, 0.18731E+01_JPRB, 0.21151E+01_JPRB, 0.29864E+01_JPRB, 0.30891E+01_JPRB ,&
&  0.23276E+01_JPRB, 0.20741E+01_JPRB, 0.18766E+01_JPRB, 0.30068E+01_JPRB, 0.21565E+01_JPRB, 0.10090E+01_JPRB, 0.81358E+00_JPRB ,&
&  0.84640E+00_JPRB, 0.92582E+00_JPRB, 0.10440E+01_JPRB, 0.11307E+01_JPRB, 0.11774E+01_JPRB, 0.12342E+01_JPRB, 0.12910E+01_JPRB ,&
&  0.13465E+01_JPRB, 0.14368E+01_JPRB, 0.15413E+01_JPRB, 0.16195E+01_JPRB, 0.16967E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.13876E+01_JPRB, 0.14687E+01_JPRB, 0.15340E+01_JPRB, 0.15731E+01_JPRB, 0.15989E+01_JPRB, 0.16020E+01_JPRB, 0.15706E+01_JPRB ,&
&  0.15352E+01_JPRB, 0.15831E+01_JPRB, 0.17176E+01_JPRB, 0.20135E+01_JPRB, 0.22725E+01_JPRB, 0.24427E+01_JPRB, 0.25989E+01_JPRB ,&
&  0.27494E+01_JPRB, 0.28037E+01_JPRB, 0.27185E+01_JPRB, 0.26752E+01_JPRB, 0.25466E+01_JPRB, 0.23930E+01_JPRB, 0.25870E+01_JPRB ,&
&  0.29767E+01_JPRB, 0.34058E+01_JPRB, 0.36163E+01_JPRB, 0.38278E+01_JPRB, 0.30573E+01_JPRB, 0.26132E+01_JPRB, 0.29325E+01_JPRB ,&
&  0.32688E+01_JPRB, 0.27811E+01_JPRB, 0.21962E+01_JPRB, 0.21844E+01_JPRB, 0.22237E+01_JPRB, 0.23314E+01_JPRB, 0.25112E+01_JPRB ,&
&  0.26346E+01_JPRB, 0.27202E+01_JPRB, 0.27991E+01_JPRB, 0.28675E+01_JPRB, 0.29353E+01_JPRB, 0.30024E+01_JPRB, 0.30845E+01_JPRB ,&
&  0.31495E+01_JPRB, 0.32093E+01_JPRB, 0.34007E+01_JPRB, 0.36901E+01_JPRB, 0.40176E+01_JPRB, 0.44498E+01_JPRB, 0.48191E+01_JPRB ,&
&  0.50455E+01_JPRB, 0.52331E+01_JPRB, 0.58378E+01_JPRB, 0.62290E+01_JPRB, 0.59073E+01_JPRB, 0.53795E+01_JPRB, 0.55656E+01_JPRB ,&
&  0.55686E+01_JPRB, 0.51929E+01_JPRB, 0.41640E+01_JPRB, 0.60045E+01_JPRB, 0.60501E+01_JPRB, 0.51964E+01_JPRB, 0.42796E+01_JPRB ,&
&  0.37815E+01_JPRB, 0.35610E+01_JPRB, 0.31472E+01_JPRB, 0.26059E+01_JPRB, 0.21512E+01_JPRB, 0.26318E+01_JPRB, 0.29496E+01_JPRB ,&
&  0.30149E+01_JPRB, 0.23485E+01_JPRB, 0.17010E+01_JPRB, 0.15187E+01_JPRB, 0.16264E+01_JPRB, 0.25536E+01_JPRB, 0.25259E+01_JPRB ,&
&  0.19348E+01_JPRB, 0.17325E+01_JPRB, 0.16053E+01_JPRB, 0.23693E+01_JPRB, 0.35329E+01_JPRB, 0.10685E+01_JPRB, 0.71380E+00_JPRB ,&
&  0.71562E+00_JPRB, 0.75476E+00_JPRB, 0.79808E+00_JPRB, 0.81509E+00_JPRB, 0.82302E+00_JPRB, 0.86836E+00_JPRB, 0.94152E+00_JPRB ,&
&  0.10318E+01_JPRB, 0.11151E+01_JPRB, 0.11746E+01_JPRB, 0.12532E+01_JPRB, 0.13193E+01_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.94115E+00_JPRB, 0.98034E+00_JPRB, 0.99001E+00_JPRB, 0.10067E+01_JPRB, 0.10031E+01_JPRB, 0.10023E+01_JPRB, 0.96681E+00_JPRB ,&
&  0.94677E+00_JPRB, 0.98033E+00_JPRB, 0.10761E+01_JPRB, 0.12516E+01_JPRB, 0.13192E+01_JPRB, 0.13718E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.14626E+01_JPRB, 0.13813E+01_JPRB, 0.13186E+01_JPRB, 0.12126E+01_JPRB, 0.11617E+01_JPRB, 0.11752E+01_JPRB, 0.12859E+01_JPRB ,&
&  0.14739E+01_JPRB, 0.17271E+01_JPRB, 0.20038E+01_JPRB, 0.23405E+01_JPRB, 0.31500E+01_JPRB, 0.29210E+01_JPRB, 0.30185E+01_JPRB ,&
&  0.36101E+01_JPRB, 0.26275E+01_JPRB, 0.21014E+01_JPRB, 0.17972E+01_JPRB, 0.16645E+01_JPRB, 0.16111E+01_JPRB, 0.16815E+01_JPRB ,&
&  0.17214E+01_JPRB, 0.17894E+01_JPRB, 0.18708E+01_JPRB, 0.19043E+01_JPRB, 0.19365E+01_JPRB, 0.19098E+01_JPRB, 0.18666E+01_JPRB ,&
&  0.18108E+01_JPRB, 0.19192E+01_JPRB, 0.21487E+01_JPRB, 0.23417E+01_JPRB, 0.28005E+01_JPRB, 0.31904E+01_JPRB, 0.35198E+01_JPRB ,&
&  0.37039E+01_JPRB, 0.39659E+01_JPRB, 0.44964E+01_JPRB, 0.46524E+01_JPRB, 0.47425E+01_JPRB, 0.45214E+01_JPRB, 0.43805E+01_JPRB ,&
&  0.41138E+01_JPRB, 0.33339E+01_JPRB, 0.27959E+01_JPRB, 0.41941E+01_JPRB, 0.43355E+01_JPRB, 0.34626E+01_JPRB, 0.35813E+01_JPRB ,&
&  0.32957E+01_JPRB, 0.30968E+01_JPRB, 0.29440E+01_JPRB, 0.24761E+01_JPRB, 0.19699E+01_JPRB, 0.21934E+01_JPRB, 0.27490E+01_JPRB ,&
&  0.24125E+01_JPRB, 0.17350E+01_JPRB, 0.13331E+01_JPRB, 0.12876E+01_JPRB, 0.14343E+01_JPRB, 0.20599E+01_JPRB, 0.22300E+01_JPRB ,&
&  0.20243E+01_JPRB, 0.15108E+01_JPRB, 0.17566E+01_JPRB, 0.27916E+01_JPRB, 0.19842E+01_JPRB, 0.95863E+00_JPRB, 0.61393E+00_JPRB ,&
&  0.58627E+00_JPRB, 0.60187E+00_JPRB, 0.62382E+00_JPRB, 0.63419E+00_JPRB, 0.64018E+00_JPRB, 0.65457E+00_JPRB, 0.70255E+00_JPRB ,&
&  0.74638E+00_JPRB, 0.79230E+00_JPRB, 0.83080E+00_JPRB, 0.88244E+00_JPRB, 0.91586E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.87091E+00_JPRB, 0.89495E+00_JPRB, 0.92405E+00_JPRB, 0.94327E+00_JPRB, 0.96861E+00_JPRB, 0.99212E+00_JPRB, 0.10449E+01_JPRB ,&
&  0.10978E+01_JPRB, 0.11448E+01_JPRB, 0.12067E+01_JPRB, 0.12730E+01_JPRB, 0.12939E+01_JPRB, 0.13132E+01_JPRB, 0.13310E+01_JPRB ,&
&  0.13566E+01_JPRB, 0.13706E+01_JPRB, 0.13487E+01_JPRB, 0.13384E+01_JPRB, 0.13250E+01_JPRB, 0.14126E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.13940E+01_JPRB, 0.14042E+01_JPRB, 0.14812E+01_JPRB, 0.17286E+01_JPRB, 0.19984E+01_JPRB, 0.21455E+01_JPRB ,&
&  0.27954E+01_JPRB, 0.21727E+01_JPRB, 0.18558E+01_JPRB, 0.13697E+01_JPRB, 0.12667E+01_JPRB, 0.12706E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.12120E+01_JPRB, 0.12511E+01_JPRB, 0.13274E+01_JPRB, 0.13318E+01_JPRB, 0.13963E+01_JPRB, 0.14437E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.15376E+01_JPRB, 0.16223E+01_JPRB, 0.17196E+01_JPRB, 0.18875E+01_JPRB, 0.20378E+01_JPRB, 0.24005E+01_JPRB, 0.25598E+01_JPRB ,&
&  0.26218E+01_JPRB, 0.27512E+01_JPRB, 0.25886E+01_JPRB, 0.26456E+01_JPRB, 0.29961E+01_JPRB, 0.30014E+01_JPRB, 0.29183E+01_JPRB ,&
&  0.29826E+01_JPRB, 0.26746E+01_JPRB, 0.20367E+01_JPRB, 0.30115E+01_JPRB, 0.33452E+01_JPRB, 0.30692E+01_JPRB, 0.30080E+01_JPRB ,&
&  0.27996E+01_JPRB, 0.24954E+01_JPRB, 0.22665E+01_JPRB, 0.21340E+01_JPRB, 0.18242E+01_JPRB, 0.17013E+01_JPRB, 0.19076E+01_JPRB ,&
&  0.16781E+01_JPRB, 0.13913E+01_JPRB, 0.12596E+01_JPRB, 0.13679E+01_JPRB, 0.18575E+01_JPRB, 0.31165E+01_JPRB, 0.27835E+01_JPRB ,&
&  0.18819E+01_JPRB, 0.15111E+01_JPRB, 0.22609E+01_JPRB, 0.33525E+01_JPRB, 0.17634E+01_JPRB, 0.10221E+01_JPRB, 0.60547E+00_JPRB ,&
&  0.54602E+00_JPRB, 0.56951E+00_JPRB, 0.59884E+00_JPRB, 0.62386E+00_JPRB, 0.65459E+00_JPRB, 0.68329E+00_JPRB, 0.70779E+00_JPRB ,&
&  0.73147E+00_JPRB, 0.76187E+00_JPRB, 0.79004E+00_JPRB, 0.82477E+00_JPRB, 0.85617E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.10342E+01_JPRB, 0.10913E+01_JPRB, 0.11353E+01_JPRB, 0.11724E+01_JPRB, 0.12069E+01_JPRB, 0.12420E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.13238E+01_JPRB, 0.13618E+01_JPRB, 0.13968E+01_JPRB, 0.14405E+01_JPRB, 0.14953E+01_JPRB, 0.15461E+01_JPRB, 0.15681E+01_JPRB ,&
&  0.15734E+01_JPRB, 0.16253E+01_JPRB, 0.16481E+01_JPRB, 0.17156E+01_JPRB, 0.17375E+01_JPRB, 0.17839E+01_JPRB, 0.17850E+01_JPRB ,&
&  0.17692E+01_JPRB, 0.17603E+01_JPRB, 0.17630E+01_JPRB, 0.17764E+01_JPRB, 0.17818E+01_JPRB, 0.19113E+01_JPRB, 0.21107E+01_JPRB ,&
&  0.21740E+01_JPRB, 0.21470E+01_JPRB, 0.15274E+01_JPRB, 0.12372E+01_JPRB, 0.12299E+01_JPRB, 0.12515E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.12272E+01_JPRB, 0.12573E+01_JPRB, 0.13003E+01_JPRB, 0.13166E+01_JPRB, 0.14114E+01_JPRB, 0.15820E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.17610E+01_JPRB, 0.18550E+01_JPRB, 0.19073E+01_JPRB, 0.20794E+01_JPRB, 0.20993E+01_JPRB, 0.25161E+01_JPRB, 0.27230E+01_JPRB ,&
&  0.27732E+01_JPRB, 0.26371E+01_JPRB, 0.20574E+01_JPRB, 0.20635E+01_JPRB, 0.22644E+01_JPRB, 0.22073E+01_JPRB, 0.21382E+01_JPRB ,&
&  0.24798E+01_JPRB, 0.24413E+01_JPRB, 0.21588E+01_JPRB, 0.27100E+01_JPRB, 0.28662E+01_JPRB, 0.27866E+01_JPRB, 0.24572E+01_JPRB ,&
&  0.21496E+01_JPRB, 0.16826E+01_JPRB, 0.16526E+01_JPRB, 0.16625E+01_JPRB, 0.15345E+01_JPRB, 0.14109E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.12363E+01_JPRB, 0.12891E+01_JPRB, 0.15245E+01_JPRB, 0.24844E+01_JPRB, 0.30244E+01_JPRB, 0.35546E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.15302E+01_JPRB, 0.11986E+01_JPRB, 0.15559E+01_JPRB, 0.12182E+01_JPRB, 0.74236E+00_JPRB, 0.58961E+00_JPRB ,&
&  0.61151E+00_JPRB, 0.66111E+00_JPRB, 0.70516E+00_JPRB, 0.74031E+00_JPRB, 0.77395E+00_JPRB, 0.80952E+00_JPRB, 0.83144E+00_JPRB ,&
&  0.85786E+00_JPRB, 0.88813E+00_JPRB, 0.91767E+00_JPRB, 0.96004E+00_JPRB, 0.99180E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.11347E+01_JPRB, 0.11828E+01_JPRB, 0.12256E+01_JPRB, 0.12668E+01_JPRB, 0.13125E+01_JPRB, 0.13624E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.14477E+01_JPRB, 0.14891E+01_JPRB, 0.15251E+01_JPRB, 0.15600E+01_JPRB, 0.16055E+01_JPRB, 0.16510E+01_JPRB, 0.16809E+01_JPRB ,&
&  0.17110E+01_JPRB, 0.17635E+01_JPRB, 0.18161E+01_JPRB, 0.18681E+01_JPRB, 0.18959E+01_JPRB, 0.19252E+01_JPRB, 0.19230E+01_JPRB ,&
&  0.19054E+01_JPRB, 0.18976E+01_JPRB, 0.19179E+01_JPRB, 0.19511E+01_JPRB, 0.20016E+01_JPRB, 0.20101E+01_JPRB, 0.21838E+01_JPRB ,&
&  0.20256E+01_JPRB, 0.22115E+01_JPRB, 0.17075E+01_JPRB, 0.14858E+01_JPRB, 0.15000E+01_JPRB, 0.14094E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.14387E+01_JPRB, 0.14290E+01_JPRB, 0.14507E+01_JPRB, 0.14712E+01_JPRB, 0.15544E+01_JPRB, 0.16911E+01_JPRB, 0.17911E+01_JPRB ,&
&  0.19448E+01_JPRB, 0.21208E+01_JPRB, 0.22592E+01_JPRB, 0.24196E+01_JPRB, 0.27681E+01_JPRB, 0.30656E+01_JPRB, 0.32115E+01_JPRB ,&
&  0.31747E+01_JPRB, 0.30230E+01_JPRB, 0.23052E+01_JPRB, 0.23787E+01_JPRB, 0.25173E+01_JPRB, 0.23399E+01_JPRB, 0.21401E+01_JPRB ,&
&  0.22365E+01_JPRB, 0.19164E+01_JPRB, 0.21903E+01_JPRB, 0.22458E+01_JPRB, 0.21475E+01_JPRB, 0.19038E+01_JPRB, 0.15764E+01_JPRB ,&
&  0.12851E+01_JPRB, 0.10634E+01_JPRB, 0.10875E+01_JPRB, 0.12466E+01_JPRB, 0.11993E+01_JPRB, 0.10188E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.99275E+00_JPRB, 0.10821E+01_JPRB, 0.14263E+01_JPRB, 0.21329E+01_JPRB, 0.37046E+01_JPRB, 0.33209E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.13104E+01_JPRB, 0.74277E+00_JPRB, 0.77291E+00_JPRB, 0.79273E+00_JPRB, 0.64519E+00_JPRB, 0.63776E+00_JPRB ,&
&  0.74243E+00_JPRB, 0.85062E+00_JPRB, 0.87800E+00_JPRB, 0.83757E+00_JPRB, 0.83529E+00_JPRB, 0.86016E+00_JPRB, 0.89512E+00_JPRB ,&
&  0.93121E+00_JPRB, 0.96624E+00_JPRB, 0.10030E+01_JPRB, 0.10439E+01_JPRB, 0.10869E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.11794E+01_JPRB, 0.12284E+01_JPRB, 0.12726E+01_JPRB, 0.13136E+01_JPRB, 0.13628E+01_JPRB, 0.14167E+01_JPRB, 0.14692E+01_JPRB ,&
&  0.15171E+01_JPRB, 0.15619E+01_JPRB, 0.16105E+01_JPRB, 0.16576E+01_JPRB, 0.17041E+01_JPRB, 0.17537E+01_JPRB, 0.18138E+01_JPRB ,&
&  0.18706E+01_JPRB, 0.19146E+01_JPRB, 0.19823E+01_JPRB, 0.20615E+01_JPRB, 0.21170E+01_JPRB, 0.21557E+01_JPRB, 0.21764E+01_JPRB ,&
&  0.21992E+01_JPRB, 0.22575E+01_JPRB, 0.22951E+01_JPRB, 0.23236E+01_JPRB, 0.23766E+01_JPRB, 0.25988E+01_JPRB, 0.23583E+01_JPRB ,&
&  0.25853E+01_JPRB, 0.22022E+01_JPRB, 0.18334E+01_JPRB, 0.17158E+01_JPRB, 0.16852E+01_JPRB, 0.16696E+01_JPRB, 0.16634E+01_JPRB ,&
&  0.16847E+01_JPRB, 0.16041E+01_JPRB, 0.15828E+01_JPRB, 0.15833E+01_JPRB, 0.16481E+01_JPRB, 0.17657E+01_JPRB, 0.18946E+01_JPRB ,&
&  0.20999E+01_JPRB, 0.23616E+01_JPRB, 0.26643E+01_JPRB, 0.29907E+01_JPRB, 0.33808E+01_JPRB, 0.37090E+01_JPRB, 0.39112E+01_JPRB ,&
&  0.39476E+01_JPRB, 0.38957E+01_JPRB, 0.33079E+01_JPRB, 0.34822E+01_JPRB, 0.34383E+01_JPRB, 0.28612E+01_JPRB, 0.23627E+01_JPRB ,&
&  0.19758E+01_JPRB, 0.15637E+01_JPRB, 0.16350E+01_JPRB, 0.16645E+01_JPRB, 0.15106E+01_JPRB, 0.12744E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.81163E+00_JPRB, 0.72794E+00_JPRB, 0.73090E+00_JPRB, 0.86874E+00_JPRB, 0.87302E+00_JPRB, 0.78034E+00_JPRB, 0.80576E+00_JPRB ,&
&  0.82900E+00_JPRB, 0.84635E+00_JPRB, 0.95978E+00_JPRB, 0.12648E+01_JPRB, 0.17269E+01_JPRB, 0.17431E+01_JPRB, 0.17845E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.10064E+01_JPRB, 0.92045E+00_JPRB, 0.75100E+00_JPRB, 0.77815E+00_JPRB, 0.76701E+00_JPRB, 0.86124E+00_JPRB ,&
&  0.10224E+01_JPRB, 0.12411E+01_JPRB, 0.12158E+01_JPRB, 0.10208E+01_JPRB, 0.88355E+00_JPRB, 0.86930E+00_JPRB, 0.90515E+00_JPRB ,&
&  0.94453E+00_JPRB, 0.98181E+00_JPRB, 0.10272E+01_JPRB, 0.10764E+01_JPRB, 0.11269E+01_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.11860E+01_JPRB, 0.12328E+01_JPRB, 0.12736E+01_JPRB, 0.13106E+01_JPRB, 0.13601E+01_JPRB, 0.14123E+01_JPRB, 0.14670E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.15749E+01_JPRB, 0.16329E+01_JPRB, 0.16897E+01_JPRB, 0.17532E+01_JPRB, 0.18192E+01_JPRB, 0.18909E+01_JPRB ,&
&  0.19596E+01_JPRB, 0.20257E+01_JPRB, 0.20888E+01_JPRB, 0.21997E+01_JPRB, 0.23238E+01_JPRB, 0.24181E+01_JPRB, 0.24864E+01_JPRB ,&
&  0.25224E+01_JPRB, 0.25966E+01_JPRB, 0.26857E+01_JPRB, 0.27574E+01_JPRB, 0.27807E+01_JPRB, 0.29682E+01_JPRB, 0.26922E+01_JPRB ,&
&  0.26079E+01_JPRB, 0.22926E+01_JPRB, 0.20130E+01_JPRB, 0.19516E+01_JPRB, 0.19050E+01_JPRB, 0.18902E+01_JPRB, 0.18680E+01_JPRB ,&
&  0.18330E+01_JPRB, 0.17935E+01_JPRB, 0.17231E+01_JPRB, 0.16901E+01_JPRB, 0.17114E+01_JPRB, 0.18137E+01_JPRB, 0.19676E+01_JPRB ,&
&  0.21783E+01_JPRB, 0.24654E+01_JPRB, 0.28096E+01_JPRB, 0.31845E+01_JPRB, 0.36123E+01_JPRB, 0.39962E+01_JPRB, 0.43506E+01_JPRB ,&
&  0.46169E+01_JPRB, 0.47705E+01_JPRB, 0.43769E+01_JPRB, 0.43301E+01_JPRB, 0.40880E+01_JPRB, 0.28582E+01_JPRB, 0.20987E+01_JPRB ,&
&  0.15211E+01_JPRB, 0.14469E+01_JPRB, 0.14779E+01_JPRB, 0.13832E+01_JPRB, 0.11064E+01_JPRB, 0.94592E+00_JPRB, 0.76896E+00_JPRB ,&
&  0.63021E+00_JPRB, 0.58202E+00_JPRB, 0.59544E+00_JPRB, 0.67100E+00_JPRB, 0.66720E+00_JPRB, 0.63318E+00_JPRB, 0.68100E+00_JPRB ,&
&  0.71387E+00_JPRB, 0.79445E+00_JPRB, 0.92573E+00_JPRB, 0.11332E+01_JPRB, 0.13395E+01_JPRB, 0.13444E+01_JPRB, 0.13729E+01_JPRB ,&
&  0.11395E+01_JPRB, 0.11075E+01_JPRB, 0.95902E+00_JPRB, 0.69441E+00_JPRB, 0.85248E+00_JPRB, 0.96718E+00_JPRB, 0.11037E+01_JPRB ,&
&  0.12367E+01_JPRB, 0.16381E+01_JPRB, 0.18511E+01_JPRB, 0.14652E+01_JPRB, 0.11760E+01_JPRB, 0.91504E+00_JPRB, 0.90191E+00_JPRB ,&
&  0.94643E+00_JPRB, 0.97260E+00_JPRB, 0.10182E+01_JPRB, 0.10728E+01_JPRB, 0.11312E+01_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.11247E+01_JPRB, 0.11753E+01_JPRB, 0.12110E+01_JPRB, 0.12289E+01_JPRB, 0.12707E+01_JPRB, 0.13339E+01_JPRB, 0.13843E+01_JPRB ,&
&  0.14391E+01_JPRB, 0.14889E+01_JPRB, 0.15506E+01_JPRB, 0.16092E+01_JPRB, 0.16719E+01_JPRB, 0.17370E+01_JPRB, 0.18121E+01_JPRB ,&
&  0.18835E+01_JPRB, 0.19914E+01_JPRB, 0.20698E+01_JPRB, 0.21975E+01_JPRB, 0.23365E+01_JPRB, 0.25034E+01_JPRB, 0.26077E+01_JPRB ,&
&  0.26470E+01_JPRB, 0.27368E+01_JPRB, 0.28440E+01_JPRB, 0.29645E+01_JPRB, 0.29724E+01_JPRB, 0.30063E+01_JPRB, 0.23123E+01_JPRB ,&
&  0.24637E+01_JPRB, 0.23443E+01_JPRB, 0.22152E+01_JPRB, 0.21542E+01_JPRB, 0.20764E+01_JPRB, 0.20049E+01_JPRB, 0.19303E+01_JPRB ,&
&  0.19000E+01_JPRB, 0.18313E+01_JPRB, 0.16973E+01_JPRB, 0.17307E+01_JPRB, 0.17495E+01_JPRB, 0.18071E+01_JPRB, 0.19649E+01_JPRB ,&
&  0.21364E+01_JPRB, 0.23888E+01_JPRB, 0.27155E+01_JPRB, 0.31041E+01_JPRB, 0.35294E+01_JPRB, 0.39521E+01_JPRB, 0.43676E+01_JPRB ,&
&  0.47557E+01_JPRB, 0.50643E+01_JPRB, 0.51134E+01_JPRB, 0.42108E+01_JPRB, 0.35336E+01_JPRB, 0.25233E+01_JPRB, 0.19316E+01_JPRB ,&
&  0.15996E+01_JPRB, 0.14517E+01_JPRB, 0.13878E+01_JPRB, 0.12605E+01_JPRB, 0.96819E+00_JPRB, 0.80405E+00_JPRB, 0.65328E+00_JPRB ,&
&  0.59514E+00_JPRB, 0.55791E+00_JPRB, 0.56207E+00_JPRB, 0.57005E+00_JPRB, 0.57752E+00_JPRB, 0.57785E+00_JPRB, 0.61552E+00_JPRB ,&
&  0.65759E+00_JPRB, 0.74439E+00_JPRB, 0.90090E+00_JPRB, 0.11270E+01_JPRB, 0.13393E+01_JPRB, 0.14401E+01_JPRB, 0.12397E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.96157E+00_JPRB, 0.89829E+00_JPRB, 0.91629E+00_JPRB, 0.93835E+00_JPRB, 0.11210E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.15944E+01_JPRB, 0.16073E+01_JPRB, 0.18791E+01_JPRB, 0.24324E+01_JPRB, 0.21919E+01_JPRB, 0.19150E+01_JPRB, 0.81332E+00_JPRB ,&
&  0.87629E+00_JPRB, 0.90546E+00_JPRB, 0.96079E+00_JPRB, 0.10172E+01_JPRB, 0.10861E+01_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.89335E+00_JPRB, 0.93973E+00_JPRB, 0.98273E+00_JPRB, 0.10056E+01_JPRB, 0.10642E+01_JPRB, 0.11309E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.12981E+01_JPRB, 0.13566E+01_JPRB, 0.14272E+01_JPRB, 0.14894E+01_JPRB, 0.15509E+01_JPRB, 0.16171E+01_JPRB ,&
&  0.16821E+01_JPRB, 0.17757E+01_JPRB, 0.18714E+01_JPRB, 0.20042E+01_JPRB, 0.21447E+01_JPRB, 0.23173E+01_JPRB, 0.24776E+01_JPRB ,&
&  0.26021E+01_JPRB, 0.27256E+01_JPRB, 0.28703E+01_JPRB, 0.29701E+01_JPRB, 0.29423E+01_JPRB, 0.28773E+01_JPRB, 0.20856E+01_JPRB ,&
&  0.23635E+01_JPRB, 0.24036E+01_JPRB, 0.23873E+01_JPRB, 0.23220E+01_JPRB, 0.22186E+01_JPRB, 0.20972E+01_JPRB, 0.20170E+01_JPRB ,&
&  0.19434E+01_JPRB, 0.18290E+01_JPRB, 0.16964E+01_JPRB, 0.16014E+01_JPRB, 0.17426E+01_JPRB, 0.17871E+01_JPRB, 0.19142E+01_JPRB ,&
&  0.20886E+01_JPRB, 0.23076E+01_JPRB, 0.25760E+01_JPRB, 0.29578E+01_JPRB, 0.33801E+01_JPRB, 0.37984E+01_JPRB, 0.42283E+01_JPRB ,&
&  0.45621E+01_JPRB, 0.47315E+01_JPRB, 0.47755E+01_JPRB, 0.38222E+01_JPRB, 0.31685E+01_JPRB, 0.25032E+01_JPRB, 0.18243E+01_JPRB ,&
&  0.15298E+01_JPRB, 0.14157E+01_JPRB, 0.13668E+01_JPRB, 0.12044E+01_JPRB, 0.95016E+00_JPRB, 0.76004E+00_JPRB, 0.62692E+00_JPRB ,&
&  0.57615E+00_JPRB, 0.53766E+00_JPRB, 0.53980E+00_JPRB, 0.55079E+00_JPRB, 0.57028E+00_JPRB, 0.58713E+00_JPRB, 0.61912E+00_JPRB ,&
&  0.66480E+00_JPRB, 0.73370E+00_JPRB, 0.84510E+00_JPRB, 0.10289E+01_JPRB, 0.12894E+01_JPRB, 0.15748E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.97404E+00_JPRB, 0.88071E+00_JPRB, 0.83292E+00_JPRB, 0.83740E+00_JPRB, 0.89133E+00_JPRB, 0.96231E+00_JPRB ,&
&  0.10962E+01_JPRB, 0.11701E+01_JPRB, 0.10452E+01_JPRB, 0.87229E+00_JPRB, 0.11221E+01_JPRB, 0.11219E+01_JPRB, 0.65681E+00_JPRB ,&
&  0.72184E+00_JPRB, 0.71203E+00_JPRB, 0.75228E+00_JPRB, 0.80102E+00_JPRB, 0.85998E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.67506E+00_JPRB, 0.74441E+00_JPRB, 0.78128E+00_JPRB, 0.82704E+00_JPRB, 0.86830E+00_JPRB, 0.92768E+00_JPRB, 0.98641E+00_JPRB ,&
&  0.10471E+01_JPRB, 0.11150E+01_JPRB, 0.11716E+01_JPRB, 0.12394E+01_JPRB, 0.13011E+01_JPRB, 0.13582E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.15500E+01_JPRB, 0.16313E+01_JPRB, 0.17251E+01_JPRB, 0.18270E+01_JPRB, 0.19602E+01_JPRB, 0.21446E+01_JPRB ,&
&  0.23678E+01_JPRB, 0.25864E+01_JPRB, 0.27856E+01_JPRB, 0.29541E+01_JPRB, 0.30487E+01_JPRB, 0.31751E+01_JPRB, 0.32147E+01_JPRB ,&
&  0.21179E+01_JPRB, 0.26099E+01_JPRB, 0.25091E+01_JPRB, 0.23740E+01_JPRB, 0.22655E+01_JPRB, 0.21581E+01_JPRB, 0.20991E+01_JPRB ,&
&  0.20191E+01_JPRB, 0.18176E+01_JPRB, 0.17281E+01_JPRB, 0.16118E+01_JPRB, 0.16519E+01_JPRB, 0.17160E+01_JPRB, 0.18310E+01_JPRB ,&
&  0.20002E+01_JPRB, 0.22257E+01_JPRB, 0.24706E+01_JPRB, 0.27344E+01_JPRB, 0.30947E+01_JPRB, 0.35389E+01_JPRB, 0.39432E+01_JPRB ,&
&  0.43666E+01_JPRB, 0.47960E+01_JPRB, 0.50539E+01_JPRB, 0.42619E+01_JPRB, 0.35086E+01_JPRB, 0.31247E+01_JPRB, 0.23019E+01_JPRB ,&
&  0.15734E+01_JPRB, 0.12949E+01_JPRB, 0.12804E+01_JPRB, 0.12047E+01_JPRB, 0.96161E+00_JPRB, 0.79051E+00_JPRB, 0.67321E+00_JPRB ,&
&  0.61898E+00_JPRB, 0.57179E+00_JPRB, 0.56068E+00_JPRB, 0.57139E+00_JPRB, 0.58979E+00_JPRB, 0.61342E+00_JPRB, 0.66052E+00_JPRB ,&
&  0.69754E+00_JPRB, 0.73241E+00_JPRB, 0.77422E+00_JPRB, 0.81577E+00_JPRB, 0.89540E+00_JPRB, 0.10256E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.98765E+00_JPRB, 0.93939E+00_JPRB, 0.88669E+00_JPRB, 0.86402E+00_JPRB, 0.82160E+00_JPRB, 0.79994E+00_JPRB, 0.80699E+00_JPRB ,&
&  0.82356E+00_JPRB, 0.83184E+00_JPRB, 0.78141E+00_JPRB, 0.74029E+00_JPRB, 0.71970E+00_JPRB, 0.68651E+00_JPRB, 0.63374E+00_JPRB ,&
&  0.61538E+00_JPRB, 0.59955E+00_JPRB, 0.58174E+00_JPRB, 0.59523E+00_JPRB, 0.63043E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.57169E+00_JPRB, 0.59788E+00_JPRB, 0.63103E+00_JPRB, 0.66661E+00_JPRB, 0.70060E+00_JPRB, 0.73544E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.84897E+00_JPRB, 0.89989E+00_JPRB, 0.93257E+00_JPRB, 0.98059E+00_JPRB, 0.10381E+01_JPRB, 0.11090E+01_JPRB, 0.11797E+01_JPRB ,&
&  0.12511E+01_JPRB, 0.13393E+01_JPRB, 0.14127E+01_JPRB, 0.14879E+01_JPRB, 0.15660E+01_JPRB, 0.16534E+01_JPRB, 0.17848E+01_JPRB ,&
&  0.19552E+01_JPRB, 0.21790E+01_JPRB, 0.24600E+01_JPRB, 0.27376E+01_JPRB, 0.29945E+01_JPRB, 0.32445E+01_JPRB, 0.33402E+01_JPRB ,&
&  0.18910E+01_JPRB, 0.25143E+01_JPRB, 0.28908E+01_JPRB, 0.25552E+01_JPRB, 0.22980E+01_JPRB, 0.21380E+01_JPRB, 0.20439E+01_JPRB ,&
&  0.18984E+01_JPRB, 0.17064E+01_JPRB, 0.16221E+01_JPRB, 0.16807E+01_JPRB, 0.15719E+01_JPRB, 0.16117E+01_JPRB, 0.16764E+01_JPRB ,&
&  0.17983E+01_JPRB, 0.19680E+01_JPRB, 0.21327E+01_JPRB, 0.23293E+01_JPRB, 0.26166E+01_JPRB, 0.29761E+01_JPRB, 0.34183E+01_JPRB ,&
&  0.39775E+01_JPRB, 0.46241E+01_JPRB, 0.52255E+01_JPRB, 0.41997E+01_JPRB, 0.38154E+01_JPRB, 0.35915E+01_JPRB, 0.30719E+01_JPRB ,&
&  0.18644E+01_JPRB, 0.13125E+01_JPRB, 0.12858E+01_JPRB, 0.12273E+01_JPRB, 0.10383E+01_JPRB, 0.89759E+00_JPRB, 0.80042E+00_JPRB ,&
&  0.73027E+00_JPRB, 0.67636E+00_JPRB, 0.64893E+00_JPRB, 0.64387E+00_JPRB, 0.65123E+00_JPRB, 0.68087E+00_JPRB, 0.71966E+00_JPRB ,&
&  0.75420E+00_JPRB, 0.76272E+00_JPRB, 0.80150E+00_JPRB, 0.81961E+00_JPRB, 0.87094E+00_JPRB, 0.91544E+00_JPRB, 0.94596E+00_JPRB ,&
&  0.95679E+00_JPRB, 0.94540E+00_JPRB, 0.93356E+00_JPRB, 0.92144E+00_JPRB, 0.91068E+00_JPRB, 0.89710E+00_JPRB, 0.88154E+00_JPRB ,&
&  0.86029E+00_JPRB, 0.85958E+00_JPRB, 0.83357E+00_JPRB, 0.81157E+00_JPRB, 0.80404E+00_JPRB, 0.76057E+00_JPRB, 0.73753E+00_JPRB ,&
&  0.71660E+00_JPRB, 0.66271E+00_JPRB, 0.62710E+00_JPRB, 0.59947E+00_JPRB, 0.57681E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.60885E+00_JPRB, 0.57210E+00_JPRB, 0.57756E+00_JPRB, 0.58748E+00_JPRB, 0.61337E+00_JPRB, 0.64055E+00_JPRB, 0.63821E+00_JPRB ,&
&  0.65363E+00_JPRB, 0.67869E+00_JPRB, 0.71346E+00_JPRB, 0.74642E+00_JPRB, 0.78495E+00_JPRB, 0.82911E+00_JPRB, 0.88058E+00_JPRB ,&
&  0.94812E+00_JPRB, 0.10307E+01_JPRB, 0.11196E+01_JPRB, 0.12015E+01_JPRB, 0.12823E+01_JPRB, 0.13492E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.15129E+01_JPRB, 0.16621E+01_JPRB, 0.18872E+01_JPRB, 0.21743E+01_JPRB, 0.25485E+01_JPRB, 0.29969E+01_JPRB, 0.33188E+01_JPRB ,&
&  0.22855E+01_JPRB, 0.10881E+01_JPRB, 0.26926E+01_JPRB, 0.29754E+01_JPRB, 0.25229E+01_JPRB, 0.23134E+01_JPRB, 0.20613E+01_JPRB ,&
&  0.18939E+01_JPRB, 0.18347E+01_JPRB, 0.16685E+01_JPRB, 0.16294E+01_JPRB, 0.15259E+01_JPRB, 0.15262E+01_JPRB, 0.15370E+01_JPRB ,&
&  0.15650E+01_JPRB, 0.16451E+01_JPRB, 0.18092E+01_JPRB, 0.19119E+01_JPRB, 0.20652E+01_JPRB, 0.23332E+01_JPRB, 0.27346E+01_JPRB ,&
&  0.33015E+01_JPRB, 0.39961E+01_JPRB, 0.46968E+01_JPRB, 0.36631E+01_JPRB, 0.28807E+01_JPRB, 0.23154E+01_JPRB, 0.19192E+01_JPRB ,&
&  0.16784E+01_JPRB, 0.13525E+01_JPRB, 0.12794E+01_JPRB, 0.12369E+01_JPRB, 0.11188E+01_JPRB, 0.96389E+00_JPRB, 0.93101E+00_JPRB ,&
&  0.86202E+00_JPRB, 0.81202E+00_JPRB, 0.78141E+00_JPRB, 0.76109E+00_JPRB, 0.75739E+00_JPRB, 0.77611E+00_JPRB, 0.79811E+00_JPRB ,&
&  0.82631E+00_JPRB, 0.84662E+00_JPRB, 0.87728E+00_JPRB, 0.91289E+00_JPRB, 0.92405E+00_JPRB, 0.92854E+00_JPRB, 0.94697E+00_JPRB ,&
&  0.94829E+00_JPRB, 0.95937E+00_JPRB, 0.97191E+00_JPRB, 0.97660E+00_JPRB, 0.97712E+00_JPRB, 0.96137E+00_JPRB, 0.96514E+00_JPRB ,&
&  0.95775E+00_JPRB, 0.94855E+00_JPRB, 0.92372E+00_JPRB, 0.92125E+00_JPRB, 0.91792E+00_JPRB, 0.90459E+00_JPRB, 0.91090E+00_JPRB ,&
&  0.89494E+00_JPRB, 0.85534E+00_JPRB, 0.78343E+00_JPRB, 0.70697E+00_JPRB, 0.66244E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.74763E+00_JPRB, 0.68691E+00_JPRB, 0.64283E+00_JPRB, 0.61983E+00_JPRB, 0.60626E+00_JPRB, 0.60276E+00_JPRB, 0.60266E+00_JPRB ,&
&  0.60882E+00_JPRB, 0.61554E+00_JPRB, 0.60782E+00_JPRB, 0.61822E+00_JPRB, 0.63981E+00_JPRB, 0.66881E+00_JPRB, 0.70591E+00_JPRB ,&
&  0.73651E+00_JPRB, 0.77184E+00_JPRB, 0.81162E+00_JPRB, 0.86432E+00_JPRB, 0.92955E+00_JPRB, 0.10024E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.12385E+01_JPRB, 0.13680E+01_JPRB, 0.15565E+01_JPRB, 0.18224E+01_JPRB, 0.22241E+01_JPRB, 0.26950E+01_JPRB ,&
&  0.33793E+01_JPRB, 0.23969E+01_JPRB, 0.15674E+01_JPRB, 0.27099E+01_JPRB, 0.28356E+01_JPRB, 0.26602E+01_JPRB, 0.21912E+01_JPRB ,&
&  0.21637E+01_JPRB, 0.21534E+01_JPRB, 0.18895E+01_JPRB, 0.17119E+01_JPRB, 0.15061E+01_JPRB, 0.14659E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.13884E+01_JPRB, 0.14791E+01_JPRB, 0.15548E+01_JPRB, 0.16185E+01_JPRB, 0.17873E+01_JPRB, 0.21388E+01_JPRB ,&
&  0.26253E+01_JPRB, 0.32674E+01_JPRB, 0.39956E+01_JPRB, 0.33632E+01_JPRB, 0.25531E+01_JPRB, 0.19530E+01_JPRB, 0.17209E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.14894E+01_JPRB, 0.13683E+01_JPRB, 0.12587E+01_JPRB, 0.11400E+01_JPRB, 0.10282E+01_JPRB, 0.10102E+01_JPRB ,&
&  0.97288E+00_JPRB, 0.91683E+00_JPRB, 0.87551E+00_JPRB, 0.83915E+00_JPRB, 0.83101E+00_JPRB, 0.84193E+00_JPRB, 0.85635E+00_JPRB ,&
&  0.87426E+00_JPRB, 0.89685E+00_JPRB, 0.93045E+00_JPRB, 0.96888E+00_JPRB, 0.97986E+00_JPRB, 0.97220E+00_JPRB, 0.96937E+00_JPRB ,&
&  0.96556E+00_JPRB, 0.96572E+00_JPRB, 0.98397E+00_JPRB, 0.99317E+00_JPRB, 0.98987E+00_JPRB, 0.97232E+00_JPRB, 0.98662E+00_JPRB ,&
&  0.98601E+00_JPRB, 0.99285E+00_JPRB, 0.97875E+00_JPRB, 0.98609E+00_JPRB, 0.98398E+00_JPRB, 0.10024E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10345E+01_JPRB, 0.10180E+01_JPRB, 0.96722E+00_JPRB, 0.84591E+00_JPRB, 0.79379E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.81076E+00_JPRB, 0.77199E+00_JPRB, 0.74258E+00_JPRB, 0.71629E+00_JPRB, 0.69036E+00_JPRB, 0.66591E+00_JPRB, 0.64659E+00_JPRB ,&
&  0.62418E+00_JPRB, 0.62153E+00_JPRB, 0.61922E+00_JPRB, 0.61332E+00_JPRB, 0.60605E+00_JPRB, 0.61076E+00_JPRB, 0.62454E+00_JPRB ,&
&  0.63552E+00_JPRB, 0.64610E+00_JPRB, 0.66248E+00_JPRB, 0.67310E+00_JPRB, 0.68722E+00_JPRB, 0.71478E+00_JPRB, 0.75754E+00_JPRB ,&
&  0.81694E+00_JPRB, 0.88634E+00_JPRB, 0.97897E+00_JPRB, 0.10969E+01_JPRB, 0.12199E+01_JPRB, 0.14142E+01_JPRB, 0.16565E+01_JPRB ,&
&  0.22557E+01_JPRB, 0.23425E+01_JPRB, 0.12862E+01_JPRB, 0.23521E+01_JPRB, 0.30030E+01_JPRB, 0.28372E+01_JPRB, 0.26143E+01_JPRB ,&
&  0.26133E+01_JPRB, 0.24466E+01_JPRB, 0.21707E+01_JPRB, 0.18155E+01_JPRB, 0.15167E+01_JPRB, 0.14270E+01_JPRB, 0.13763E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.12533E+01_JPRB, 0.12581E+01_JPRB, 0.12731E+01_JPRB, 0.12962E+01_JPRB, 0.13657E+01_JPRB, 0.16081E+01_JPRB ,&
&  0.19985E+01_JPRB, 0.26016E+01_JPRB, 0.32868E+01_JPRB, 0.30992E+01_JPRB, 0.23674E+01_JPRB, 0.20271E+01_JPRB, 0.18599E+01_JPRB ,&
&  0.17945E+01_JPRB, 0.16821E+01_JPRB, 0.14284E+01_JPRB, 0.12695E+01_JPRB, 0.11449E+01_JPRB, 0.10539E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.98180E+00_JPRB, 0.94975E+00_JPRB, 0.90712E+00_JPRB, 0.88381E+00_JPRB, 0.87496E+00_JPRB, 0.88102E+00_JPRB ,&
&  0.89337E+00_JPRB, 0.90837E+00_JPRB, 0.92891E+00_JPRB, 0.95514E+00_JPRB, 0.97236E+00_JPRB, 0.98036E+00_JPRB, 0.97807E+00_JPRB ,&
&  0.97661E+00_JPRB, 0.98130E+00_JPRB, 0.98992E+00_JPRB, 0.99364E+00_JPRB, 0.99225E+00_JPRB, 0.98729E+00_JPRB, 0.97968E+00_JPRB ,&
&  0.97659E+00_JPRB, 0.97615E+00_JPRB, 0.95623E+00_JPRB, 0.94705E+00_JPRB, 0.97868E+00_JPRB, 0.98656E+00_JPRB, 0.99285E+00_JPRB ,&
&  0.10287E+01_JPRB, 0.10535E+01_JPRB, 0.10318E+01_JPRB, 0.90943E+00_JPRB, 0.85330E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.83875E+00_JPRB, 0.80741E+00_JPRB, 0.78307E+00_JPRB, 0.76439E+00_JPRB, 0.74221E+00_JPRB, 0.72034E+00_JPRB, 0.70166E+00_JPRB ,&
&  0.67755E+00_JPRB, 0.66312E+00_JPRB, 0.65397E+00_JPRB, 0.64386E+00_JPRB, 0.63333E+00_JPRB, 0.62368E+00_JPRB, 0.61624E+00_JPRB ,&
&  0.60908E+00_JPRB, 0.60322E+00_JPRB, 0.60026E+00_JPRB, 0.59401E+00_JPRB, 0.58900E+00_JPRB, 0.59058E+00_JPRB, 0.60152E+00_JPRB ,&
&  0.63238E+00_JPRB, 0.68049E+00_JPRB, 0.74271E+00_JPRB, 0.81446E+00_JPRB, 0.87800E+00_JPRB, 0.96371E+00_JPRB, 0.10690E+01_JPRB ,&
&  0.15196E+01_JPRB, 0.18125E+01_JPRB, 0.21199E+01_JPRB, 0.34857E+01_JPRB, 0.35680E+01_JPRB, 0.31844E+01_JPRB, 0.31546E+01_JPRB ,&
&  0.32397E+01_JPRB, 0.30196E+01_JPRB, 0.27341E+01_JPRB, 0.21081E+01_JPRB, 0.16706E+01_JPRB, 0.14391E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.11370E+01_JPRB, 0.11230E+01_JPRB, 0.11098E+01_JPRB, 0.11153E+01_JPRB, 0.12028E+01_JPRB, 0.13258E+01_JPRB ,&
&  0.15741E+01_JPRB, 0.20109E+01_JPRB, 0.26300E+01_JPRB, 0.30165E+01_JPRB, 0.23767E+01_JPRB, 0.22122E+01_JPRB, 0.22328E+01_JPRB ,&
&  0.26398E+01_JPRB, 0.21011E+01_JPRB, 0.15930E+01_JPRB, 0.13126E+01_JPRB, 0.11481E+01_JPRB, 0.10859E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.99349E+00_JPRB, 0.97139E+00_JPRB, 0.94656E+00_JPRB, 0.92829E+00_JPRB, 0.91040E+00_JPRB, 0.90110E+00_JPRB ,&
&  0.89999E+00_JPRB, 0.90485E+00_JPRB, 0.91109E+00_JPRB, 0.92368E+00_JPRB, 0.94287E+00_JPRB, 0.95877E+00_JPRB, 0.96640E+00_JPRB ,&
&  0.97268E+00_JPRB, 0.98235E+00_JPRB, 0.96171E+00_JPRB, 0.95991E+00_JPRB, 0.97007E+00_JPRB, 0.96664E+00_JPRB, 0.93649E+00_JPRB ,&
&  0.92368E+00_JPRB, 0.93814E+00_JPRB, 0.93745E+00_JPRB, 0.92802E+00_JPRB, 0.93801E+00_JPRB, 0.94617E+00_JPRB, 0.92076E+00_JPRB ,&
&  0.92237E+00_JPRB, 0.93123E+00_JPRB, 0.92142E+00_JPRB, 0.89751E+00_JPRB, 0.87117E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.85169E+00_JPRB, 0.82669E+00_JPRB, 0.80170E+00_JPRB, 0.77822E+00_JPRB, 0.75226E+00_JPRB, 0.73140E+00_JPRB, 0.71931E+00_JPRB ,&
&  0.70148E+00_JPRB, 0.68621E+00_JPRB, 0.67546E+00_JPRB, 0.67008E+00_JPRB, 0.66550E+00_JPRB, 0.65339E+00_JPRB, 0.63128E+00_JPRB ,&
&  0.60925E+00_JPRB, 0.59403E+00_JPRB, 0.58454E+00_JPRB, 0.57505E+00_JPRB, 0.56584E+00_JPRB, 0.56148E+00_JPRB, 0.56545E+00_JPRB ,&
&  0.57736E+00_JPRB, 0.59779E+00_JPRB, 0.63734E+00_JPRB, 0.68439E+00_JPRB, 0.72782E+00_JPRB, 0.77145E+00_JPRB, 0.80531E+00_JPRB ,&
&  0.10550E+01_JPRB, 0.15243E+01_JPRB, 0.15024E+01_JPRB, 0.29450E+01_JPRB, 0.31662E+01_JPRB, 0.31116E+01_JPRB, 0.31376E+01_JPRB ,&
&  0.31773E+01_JPRB, 0.35267E+01_JPRB, 0.29897E+01_JPRB, 0.24103E+01_JPRB, 0.19163E+01_JPRB, 0.15929E+01_JPRB, 0.13656E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.11103E+01_JPRB, 0.10576E+01_JPRB, 0.10406E+01_JPRB, 0.10377E+01_JPRB, 0.10811E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.13691E+01_JPRB, 0.16638E+01_JPRB, 0.21117E+01_JPRB, 0.24134E+01_JPRB, 0.23062E+01_JPRB, 0.23329E+01_JPRB, 0.25776E+01_JPRB ,&
&  0.32147E+01_JPRB, 0.27951E+01_JPRB, 0.18920E+01_JPRB, 0.14092E+01_JPRB, 0.11873E+01_JPRB, 0.10984E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.10068E+01_JPRB, 0.98022E+00_JPRB, 0.95978E+00_JPRB, 0.94674E+00_JPRB, 0.93611E+00_JPRB, 0.93050E+00_JPRB, 0.92302E+00_JPRB ,&
&  0.91277E+00_JPRB, 0.90757E+00_JPRB, 0.90506E+00_JPRB, 0.90494E+00_JPRB, 0.90915E+00_JPRB, 0.92149E+00_JPRB, 0.93680E+00_JPRB ,&
&  0.95169E+00_JPRB, 0.96557E+00_JPRB, 0.94835E+00_JPRB, 0.93159E+00_JPRB, 0.93424E+00_JPRB, 0.92830E+00_JPRB, 0.90400E+00_JPRB ,&
&  0.90772E+00_JPRB, 0.92200E+00_JPRB, 0.91931E+00_JPRB, 0.89937E+00_JPRB, 0.90803E+00_JPRB, 0.92988E+00_JPRB, 0.89228E+00_JPRB ,&
&  0.88182E+00_JPRB, 0.88398E+00_JPRB, 0.88853E+00_JPRB, 0.87871E+00_JPRB, 0.86665E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.83203E+00_JPRB, 0.81778E+00_JPRB, 0.79779E+00_JPRB, 0.77520E+00_JPRB, 0.75187E+00_JPRB, 0.73184E+00_JPRB, 0.72048E+00_JPRB ,&
&  0.70684E+00_JPRB, 0.69296E+00_JPRB, 0.68067E+00_JPRB, 0.67323E+00_JPRB, 0.66958E+00_JPRB, 0.65773E+00_JPRB, 0.63678E+00_JPRB ,&
&  0.61472E+00_JPRB, 0.59524E+00_JPRB, 0.57862E+00_JPRB, 0.56786E+00_JPRB, 0.55678E+00_JPRB, 0.55197E+00_JPRB, 0.55232E+00_JPRB ,&
&  0.55577E+00_JPRB, 0.56665E+00_JPRB, 0.59026E+00_JPRB, 0.62603E+00_JPRB, 0.66299E+00_JPRB, 0.69533E+00_JPRB, 0.71731E+00_JPRB ,&
&  0.85928E+00_JPRB, 0.18170E+01_JPRB, 0.13941E+01_JPRB, 0.22519E+01_JPRB, 0.24695E+01_JPRB, 0.26319E+01_JPRB, 0.28469E+01_JPRB ,&
&  0.29420E+01_JPRB, 0.30893E+01_JPRB, 0.28893E+01_JPRB, 0.25175E+01_JPRB, 0.20909E+01_JPRB, 0.17905E+01_JPRB, 0.15539E+01_JPRB ,&
&  0.13769E+01_JPRB, 0.12530E+01_JPRB, 0.11634E+01_JPRB, 0.10968E+01_JPRB, 0.10556E+01_JPRB, 0.10603E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.13490E+01_JPRB, 0.15767E+01_JPRB, 0.18468E+01_JPRB, 0.21608E+01_JPRB, 0.20664E+01_JPRB, 0.23036E+01_JPRB, 0.30131E+01_JPRB ,&
&  0.35064E+01_JPRB, 0.35018E+01_JPRB, 0.23793E+01_JPRB, 0.16184E+01_JPRB, 0.12800E+01_JPRB, 0.11171E+01_JPRB, 0.10223E+01_JPRB ,&
&  0.96965E+00_JPRB, 0.94174E+00_JPRB, 0.92593E+00_JPRB, 0.92170E+00_JPRB, 0.92449E+00_JPRB, 0.93265E+00_JPRB, 0.93227E+00_JPRB ,&
&  0.92948E+00_JPRB, 0.92551E+00_JPRB, 0.92040E+00_JPRB, 0.91205E+00_JPRB, 0.90075E+00_JPRB, 0.89681E+00_JPRB, 0.90096E+00_JPRB ,&
&  0.91410E+00_JPRB, 0.93092E+00_JPRB, 0.92496E+00_JPRB, 0.91348E+00_JPRB, 0.90858E+00_JPRB, 0.90167E+00_JPRB, 0.88761E+00_JPRB ,&
&  0.89542E+00_JPRB, 0.90010E+00_JPRB, 0.89688E+00_JPRB, 0.89647E+00_JPRB, 0.89133E+00_JPRB, 0.92556E+00_JPRB, 0.87887E+00_JPRB ,&
&  0.86423E+00_JPRB, 0.86235E+00_JPRB, 0.86111E+00_JPRB, 0.84820E+00_JPRB, 0.83947E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.79146E+00_JPRB, 0.78013E+00_JPRB, 0.76254E+00_JPRB, 0.74395E+00_JPRB, 0.73110E+00_JPRB, 0.71665E+00_JPRB, 0.70515E+00_JPRB ,&
&  0.69465E+00_JPRB, 0.68492E+00_JPRB, 0.67686E+00_JPRB, 0.66794E+00_JPRB, 0.65661E+00_JPRB, 0.64130E+00_JPRB, 0.62627E+00_JPRB ,&
&  0.61082E+00_JPRB, 0.59604E+00_JPRB, 0.58068E+00_JPRB, 0.56466E+00_JPRB, 0.55148E+00_JPRB, 0.54285E+00_JPRB, 0.53732E+00_JPRB ,&
&  0.53706E+00_JPRB, 0.54161E+00_JPRB, 0.55571E+00_JPRB, 0.58264E+00_JPRB, 0.61769E+00_JPRB, 0.65238E+00_JPRB, 0.68714E+00_JPRB ,&
&  0.83129E+00_JPRB, 0.77852E+00_JPRB, 0.15876E+01_JPRB, 0.19143E+01_JPRB, 0.18940E+01_JPRB, 0.19571E+01_JPRB, 0.23589E+01_JPRB ,&
&  0.26049E+01_JPRB, 0.27071E+01_JPRB, 0.27072E+01_JPRB, 0.25450E+01_JPRB, 0.22235E+01_JPRB, 0.19842E+01_JPRB, 0.17964E+01_JPRB ,&
&  0.16546E+01_JPRB, 0.15379E+01_JPRB, 0.14033E+01_JPRB, 0.12734E+01_JPRB, 0.11793E+01_JPRB, 0.11586E+01_JPRB, 0.12083E+01_JPRB ,&
&  0.14118E+01_JPRB, 0.15784E+01_JPRB, 0.17070E+01_JPRB, 0.18892E+01_JPRB, 0.19230E+01_JPRB, 0.22172E+01_JPRB, 0.25950E+01_JPRB ,&
&  0.34389E+01_JPRB, 0.35621E+01_JPRB, 0.26697E+01_JPRB, 0.19314E+01_JPRB, 0.14792E+01_JPRB, 0.12379E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.97933E+00_JPRB, 0.92695E+00_JPRB, 0.89488E+00_JPRB, 0.88907E+00_JPRB, 0.90426E+00_JPRB, 0.92916E+00_JPRB, 0.94928E+00_JPRB ,&
&  0.97096E+00_JPRB, 0.98486E+00_JPRB, 0.98289E+00_JPRB, 0.97122E+00_JPRB, 0.93936E+00_JPRB, 0.90088E+00_JPRB, 0.87633E+00_JPRB ,&
&  0.87293E+00_JPRB, 0.88382E+00_JPRB, 0.88818E+00_JPRB, 0.88132E+00_JPRB, 0.87980E+00_JPRB, 0.87746E+00_JPRB, 0.87328E+00_JPRB ,&
&  0.87658E+00_JPRB, 0.88031E+00_JPRB, 0.87890E+00_JPRB, 0.88696E+00_JPRB, 0.87018E+00_JPRB, 0.90388E+00_JPRB, 0.87254E+00_JPRB ,&
&  0.85226E+00_JPRB, 0.83956E+00_JPRB, 0.82862E+00_JPRB, 0.81149E+00_JPRB, 0.79700E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.74811E+00_JPRB, 0.73634E+00_JPRB, 0.71874E+00_JPRB, 0.70174E+00_JPRB, 0.69203E+00_JPRB, 0.68527E+00_JPRB, 0.68040E+00_JPRB ,&
&  0.67685E+00_JPRB, 0.66733E+00_JPRB, 0.65728E+00_JPRB, 0.64347E+00_JPRB, 0.62581E+00_JPRB, 0.61078E+00_JPRB, 0.60120E+00_JPRB ,&
&  0.58991E+00_JPRB, 0.58398E+00_JPRB, 0.57625E+00_JPRB, 0.56194E+00_JPRB, 0.54609E+00_JPRB, 0.53304E+00_JPRB, 0.52484E+00_JPRB ,&
&  0.52100E+00_JPRB, 0.52400E+00_JPRB, 0.53296E+00_JPRB, 0.55278E+00_JPRB, 0.57473E+00_JPRB, 0.60115E+00_JPRB, 0.64848E+00_JPRB ,&
&  0.83308E+00_JPRB, 0.14949E+01_JPRB, 0.19403E+01_JPRB, 0.17414E+01_JPRB, 0.16029E+01_JPRB, 0.14954E+01_JPRB, 0.17471E+01_JPRB ,&
&  0.19601E+01_JPRB, 0.21814E+01_JPRB, 0.23138E+01_JPRB, 0.23585E+01_JPRB, 0.21981E+01_JPRB, 0.20792E+01_JPRB, 0.19604E+01_JPRB ,&
&  0.18388E+01_JPRB, 0.16989E+01_JPRB, 0.15088E+01_JPRB, 0.13249E+01_JPRB, 0.12221E+01_JPRB, 0.12094E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.14191E+01_JPRB, 0.15516E+01_JPRB, 0.16061E+01_JPRB, 0.17477E+01_JPRB, 0.19907E+01_JPRB, 0.20015E+01_JPRB, 0.25591E+01_JPRB ,&
&  0.32440E+01_JPRB, 0.29403E+01_JPRB, 0.24672E+01_JPRB, 0.19855E+01_JPRB, 0.16153E+01_JPRB, 0.13177E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.10259E+01_JPRB, 0.94755E+00_JPRB, 0.89405E+00_JPRB, 0.86878E+00_JPRB, 0.88457E+00_JPRB, 0.93009E+00_JPRB, 0.97040E+00_JPRB ,&
&  0.10238E+01_JPRB, 0.10788E+01_JPRB, 0.11096E+01_JPRB, 0.11051E+01_JPRB, 0.10616E+01_JPRB, 0.99354E+00_JPRB, 0.92732E+00_JPRB ,&
&  0.88474E+00_JPRB, 0.86128E+00_JPRB, 0.86194E+00_JPRB, 0.85171E+00_JPRB, 0.85608E+00_JPRB, 0.86815E+00_JPRB, 0.86515E+00_JPRB ,&
&  0.86075E+00_JPRB, 0.86695E+00_JPRB, 0.88203E+00_JPRB, 0.87469E+00_JPRB, 0.86858E+00_JPRB, 0.90639E+00_JPRB, 0.87457E+00_JPRB ,&
&  0.84473E+00_JPRB, 0.81767E+00_JPRB, 0.79742E+00_JPRB, 0.76887E+00_JPRB, 0.75361E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.72372E+00_JPRB, 0.70587E+00_JPRB, 0.68027E+00_JPRB, 0.65894E+00_JPRB, 0.64712E+00_JPRB, 0.64099E+00_JPRB, 0.63977E+00_JPRB ,&
&  0.63967E+00_JPRB, 0.63163E+00_JPRB, 0.62088E+00_JPRB, 0.60475E+00_JPRB, 0.59043E+00_JPRB, 0.57735E+00_JPRB, 0.56621E+00_JPRB ,&
&  0.55701E+00_JPRB, 0.55197E+00_JPRB, 0.54675E+00_JPRB, 0.54081E+00_JPRB, 0.53213E+00_JPRB, 0.51987E+00_JPRB, 0.51190E+00_JPRB ,&
&  0.50694E+00_JPRB, 0.50669E+00_JPRB, 0.51150E+00_JPRB, 0.52461E+00_JPRB, 0.54168E+00_JPRB, 0.56374E+00_JPRB, 0.59804E+00_JPRB ,&
&  0.71846E+00_JPRB, 0.12489E+01_JPRB, 0.13570E+01_JPRB, 0.13258E+01_JPRB, 0.13572E+01_JPRB, 0.13257E+01_JPRB, 0.14052E+01_JPRB ,&
&  0.15618E+01_JPRB, 0.17120E+01_JPRB, 0.18491E+01_JPRB, 0.19056E+01_JPRB, 0.18316E+01_JPRB, 0.17585E+01_JPRB, 0.16597E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.14285E+01_JPRB, 0.13026E+01_JPRB, 0.11857E+01_JPRB, 0.10988E+01_JPRB, 0.10882E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.12291E+01_JPRB, 0.13000E+01_JPRB, 0.13526E+01_JPRB, 0.14764E+01_JPRB, 0.17525E+01_JPRB, 0.19261E+01_JPRB, 0.21637E+01_JPRB ,&
&  0.22179E+01_JPRB, 0.21423E+01_JPRB, 0.20077E+01_JPRB, 0.18285E+01_JPRB, 0.15916E+01_JPRB, 0.13649E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.10481E+01_JPRB, 0.95688E+00_JPRB, 0.90042E+00_JPRB, 0.87646E+00_JPRB, 0.88764E+00_JPRB, 0.93027E+00_JPRB, 0.98805E+00_JPRB ,&
&  0.10599E+01_JPRB, 0.11324E+01_JPRB, 0.11798E+01_JPRB, 0.11991E+01_JPRB, 0.11822E+01_JPRB, 0.11375E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.99999E+00_JPRB, 0.94177E+00_JPRB, 0.90323E+00_JPRB, 0.88770E+00_JPRB, 0.88608E+00_JPRB, 0.88670E+00_JPRB, 0.88567E+00_JPRB ,&
&  0.88959E+00_JPRB, 0.89793E+00_JPRB, 0.89444E+00_JPRB, 0.87737E+00_JPRB, 0.86116E+00_JPRB, 0.88088E+00_JPRB, 0.85237E+00_JPRB ,&
&  0.82291E+00_JPRB, 0.79060E+00_JPRB, 0.76366E+00_JPRB, 0.73301E+00_JPRB, 0.73091E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.74680E+00_JPRB, 0.71209E+00_JPRB, 0.67016E+00_JPRB, 0.63908E+00_JPRB, 0.61862E+00_JPRB, 0.60524E+00_JPRB, 0.59935E+00_JPRB ,&
&  0.59840E+00_JPRB, 0.58809E+00_JPRB, 0.57017E+00_JPRB, 0.55446E+00_JPRB, 0.54593E+00_JPRB, 0.53933E+00_JPRB, 0.53093E+00_JPRB ,&
&  0.52434E+00_JPRB, 0.51841E+00_JPRB, 0.51365E+00_JPRB, 0.51185E+00_JPRB, 0.51101E+00_JPRB, 0.50868E+00_JPRB, 0.50232E+00_JPRB ,&
&  0.49321E+00_JPRB, 0.48952E+00_JPRB, 0.49033E+00_JPRB, 0.49912E+00_JPRB, 0.51355E+00_JPRB, 0.53237E+00_JPRB, 0.56194E+00_JPRB ,&
&  0.68182E+00_JPRB, 0.80518E+00_JPRB, 0.90361E+00_JPRB, 0.92647E+00_JPRB, 0.97059E+00_JPRB, 0.10652E+01_JPRB, 0.11468E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.13299E+01_JPRB, 0.14681E+01_JPRB, 0.15385E+01_JPRB, 0.14900E+01_JPRB, 0.13959E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.12315E+01_JPRB, 0.11740E+01_JPRB, 0.11164E+01_JPRB, 0.10613E+01_JPRB, 0.10283E+01_JPRB, 0.10289E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.10682E+01_JPRB, 0.10547E+01_JPRB, 0.11169E+01_JPRB, 0.12197E+01_JPRB, 0.13785E+01_JPRB, 0.14925E+01_JPRB, 0.16031E+01_JPRB ,&
&  0.16896E+01_JPRB, 0.17220E+01_JPRB, 0.17279E+01_JPRB, 0.16853E+01_JPRB, 0.15719E+01_JPRB, 0.14336E+01_JPRB, 0.12862E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.10475E+01_JPRB, 0.99389E+00_JPRB, 0.96750E+00_JPRB, 0.96338E+00_JPRB, 0.97396E+00_JPRB, 0.10008E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10811E+01_JPRB, 0.11139E+01_JPRB, 0.11292E+01_JPRB, 0.11249E+01_JPRB, 0.11172E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.10873E+01_JPRB, 0.10612E+01_JPRB, 0.10262E+01_JPRB, 0.98808E+00_JPRB, 0.95437E+00_JPRB, 0.93050E+00_JPRB, 0.91152E+00_JPRB ,&
&  0.89213E+00_JPRB, 0.88215E+00_JPRB, 0.85367E+00_JPRB, 0.85155E+00_JPRB, 0.87154E+00_JPRB, 0.84829E+00_JPRB, 0.81807E+00_JPRB ,&
&  0.78176E+00_JPRB, 0.74954E+00_JPRB, 0.72036E+00_JPRB, 0.69116E+00_JPRB, 0.73688E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.64461E+00_JPRB, 0.63802E+00_JPRB, 0.62072E+00_JPRB, 0.59886E+00_JPRB, 0.58348E+00_JPRB, 0.57019E+00_JPRB, 0.55980E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.54394E+00_JPRB, 0.53041E+00_JPRB, 0.51988E+00_JPRB, 0.51796E+00_JPRB, 0.51483E+00_JPRB, 0.50927E+00_JPRB ,&
&  0.49976E+00_JPRB, 0.49052E+00_JPRB, 0.48415E+00_JPRB, 0.48220E+00_JPRB, 0.48345E+00_JPRB, 0.48598E+00_JPRB, 0.48658E+00_JPRB ,&
&  0.48279E+00_JPRB, 0.47655E+00_JPRB, 0.47489E+00_JPRB, 0.47706E+00_JPRB, 0.48446E+00_JPRB, 0.49749E+00_JPRB, 0.52419E+00_JPRB ,&
&  0.60974E+00_JPRB, 0.62746E+00_JPRB, 0.67217E+00_JPRB, 0.69930E+00_JPRB, 0.74649E+00_JPRB, 0.83202E+00_JPRB, 0.92906E+00_JPRB ,&
&  0.98477E+00_JPRB, 0.10444E+01_JPRB, 0.11381E+01_JPRB, 0.12200E+01_JPRB, 0.12249E+01_JPRB, 0.11783E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.97851E+00_JPRB, 0.93200E+00_JPRB, 0.89424E+00_JPRB, 0.87201E+00_JPRB, 0.86423E+00_JPRB, 0.86004E+00_JPRB ,&
&  0.84196E+00_JPRB, 0.84529E+00_JPRB, 0.89529E+00_JPRB, 0.97451E+00_JPRB, 0.10720E+01_JPRB, 0.11709E+01_JPRB, 0.12752E+01_JPRB ,&
&  0.13631E+01_JPRB, 0.14150E+01_JPRB, 0.14543E+01_JPRB, 0.14832E+01_JPRB, 0.14653E+01_JPRB, 0.14346E+01_JPRB, 0.13484E+01_JPRB ,&
&  0.12433E+01_JPRB, 0.11540E+01_JPRB, 0.10937E+01_JPRB, 0.10465E+01_JPRB, 0.10125E+01_JPRB, 0.99101E+00_JPRB, 0.97994E+00_JPRB ,&
&  0.97995E+00_JPRB, 0.98574E+00_JPRB, 0.98712E+00_JPRB, 0.98279E+00_JPRB, 0.97651E+00_JPRB, 0.97253E+00_JPRB, 0.96935E+00_JPRB ,&
&  0.96718E+00_JPRB, 0.96469E+00_JPRB, 0.95497E+00_JPRB, 0.93363E+00_JPRB, 0.90324E+00_JPRB, 0.87282E+00_JPRB, 0.84365E+00_JPRB ,&
&  0.81896E+00_JPRB, 0.80800E+00_JPRB, 0.79327E+00_JPRB, 0.79447E+00_JPRB, 0.77901E+00_JPRB, 0.76111E+00_JPRB, 0.74051E+00_JPRB ,&
&  0.71668E+00_JPRB, 0.69284E+00_JPRB, 0.66287E+00_JPRB, 0.63446E+00_JPRB, 0.62641E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.57891E+00_JPRB, 0.58201E+00_JPRB, 0.57916E+00_JPRB, 0.56482E+00_JPRB, 0.54966E+00_JPRB, 0.54099E+00_JPRB, 0.53152E+00_JPRB ,&
&  0.52113E+00_JPRB, 0.51280E+00_JPRB, 0.50086E+00_JPRB, 0.49457E+00_JPRB, 0.49203E+00_JPRB, 0.49135E+00_JPRB, 0.48863E+00_JPRB ,&
&  0.48178E+00_JPRB, 0.47165E+00_JPRB, 0.46406E+00_JPRB, 0.46106E+00_JPRB, 0.46230E+00_JPRB, 0.46506E+00_JPRB, 0.46711E+00_JPRB ,&
&  0.46504E+00_JPRB, 0.46289E+00_JPRB, 0.46043E+00_JPRB, 0.45886E+00_JPRB, 0.45990E+00_JPRB, 0.46970E+00_JPRB, 0.48943E+00_JPRB ,&
&  0.55209E+00_JPRB, 0.52445E+00_JPRB, 0.54610E+00_JPRB, 0.58683E+00_JPRB, 0.63135E+00_JPRB, 0.70091E+00_JPRB, 0.77967E+00_JPRB ,&
&  0.83331E+00_JPRB, 0.85098E+00_JPRB, 0.88868E+00_JPRB, 0.95132E+00_JPRB, 0.96340E+00_JPRB, 0.95916E+00_JPRB, 0.93318E+00_JPRB ,&
&  0.88731E+00_JPRB, 0.82837E+00_JPRB, 0.77079E+00_JPRB, 0.72969E+00_JPRB, 0.71229E+00_JPRB, 0.70619E+00_JPRB, 0.69843E+00_JPRB ,&
&  0.67655E+00_JPRB, 0.67019E+00_JPRB, 0.69417E+00_JPRB, 0.72962E+00_JPRB, 0.77184E+00_JPRB, 0.83952E+00_JPRB, 0.92790E+00_JPRB ,&
&  0.10061E+01_JPRB, 0.10862E+01_JPRB, 0.11672E+01_JPRB, 0.12289E+01_JPRB, 0.12778E+01_JPRB, 0.12901E+01_JPRB, 0.12540E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.11336E+01_JPRB, 0.10711E+01_JPRB, 0.10121E+01_JPRB, 0.96298E+00_JPRB, 0.93157E+00_JPRB, 0.91180E+00_JPRB ,&
&  0.89811E+00_JPRB, 0.88692E+00_JPRB, 0.86968E+00_JPRB, 0.84564E+00_JPRB, 0.82075E+00_JPRB, 0.79953E+00_JPRB, 0.78083E+00_JPRB ,&
&  0.76466E+00_JPRB, 0.74813E+00_JPRB, 0.73549E+00_JPRB, 0.72714E+00_JPRB, 0.71826E+00_JPRB, 0.71050E+00_JPRB, 0.70389E+00_JPRB ,&
&  0.69583E+00_JPRB, 0.69418E+00_JPRB, 0.68316E+00_JPRB, 0.68929E+00_JPRB, 0.69635E+00_JPRB, 0.68423E+00_JPRB, 0.67221E+00_JPRB ,&
&  0.65401E+00_JPRB, 0.63435E+00_JPRB, 0.61272E+00_JPRB, 0.56073E+00_JPRB, 0.57748E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.53062E+00_JPRB, 0.53398E+00_JPRB, 0.53230E+00_JPRB, 0.52523E+00_JPRB, 0.51524E+00_JPRB, 0.50836E+00_JPRB, 0.49865E+00_JPRB ,&
&  0.48807E+00_JPRB, 0.48285E+00_JPRB, 0.47641E+00_JPRB, 0.47313E+00_JPRB, 0.47324E+00_JPRB, 0.46914E+00_JPRB, 0.46525E+00_JPRB ,&
&  0.46154E+00_JPRB, 0.45480E+00_JPRB, 0.45033E+00_JPRB, 0.44688E+00_JPRB, 0.44679E+00_JPRB, 0.44748E+00_JPRB, 0.44726E+00_JPRB ,&
&  0.44435E+00_JPRB, 0.43988E+00_JPRB, 0.43604E+00_JPRB, 0.43333E+00_JPRB, 0.43438E+00_JPRB, 0.44158E+00_JPRB, 0.45629E+00_JPRB ,&
&  0.49959E+00_JPRB, 0.45776E+00_JPRB, 0.48284E+00_JPRB, 0.51047E+00_JPRB, 0.55163E+00_JPRB, 0.59467E+00_JPRB, 0.62441E+00_JPRB ,&
&  0.65898E+00_JPRB, 0.69012E+00_JPRB, 0.72238E+00_JPRB, 0.75673E+00_JPRB, 0.76388E+00_JPRB, 0.75636E+00_JPRB, 0.74592E+00_JPRB ,&
&  0.72628E+00_JPRB, 0.69890E+00_JPRB, 0.66988E+00_JPRB, 0.64023E+00_JPRB, 0.62197E+00_JPRB, 0.61309E+00_JPRB, 0.60043E+00_JPRB ,&
&  0.58061E+00_JPRB, 0.56404E+00_JPRB, 0.55492E+00_JPRB, 0.55701E+00_JPRB, 0.57201E+00_JPRB, 0.60155E+00_JPRB, 0.65754E+00_JPRB ,&
&  0.72120E+00_JPRB, 0.79504E+00_JPRB, 0.87555E+00_JPRB, 0.94168E+00_JPRB, 0.99126E+00_JPRB, 0.10150E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10077E+01_JPRB, 0.98434E+00_JPRB, 0.95891E+00_JPRB, 0.92646E+00_JPRB, 0.88408E+00_JPRB, 0.84336E+00_JPRB, 0.81055E+00_JPRB ,&
&  0.78498E+00_JPRB, 0.75887E+00_JPRB, 0.73073E+00_JPRB, 0.69999E+00_JPRB, 0.66744E+00_JPRB, 0.63745E+00_JPRB, 0.61245E+00_JPRB ,&
&  0.59146E+00_JPRB, 0.57366E+00_JPRB, 0.56284E+00_JPRB, 0.55919E+00_JPRB, 0.56323E+00_JPRB, 0.56968E+00_JPRB, 0.57411E+00_JPRB ,&
&  0.57824E+00_JPRB, 0.58501E+00_JPRB, 0.58764E+00_JPRB, 0.59907E+00_JPRB, 0.59975E+00_JPRB, 0.59695E+00_JPRB, 0.59437E+00_JPRB ,&
&  0.58737E+00_JPRB, 0.57284E+00_JPRB, 0.50839E+00_JPRB, 0.52896E+00_JPRB, 0.52942E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.49371E+00_JPRB, 0.48809E+00_JPRB, 0.48337E+00_JPRB, 0.48447E+00_JPRB, 0.48313E+00_JPRB, 0.47900E+00_JPRB, 0.47229E+00_JPRB ,&
&  0.46156E+00_JPRB, 0.45467E+00_JPRB, 0.45005E+00_JPRB, 0.44914E+00_JPRB, 0.45222E+00_JPRB, 0.45017E+00_JPRB, 0.44490E+00_JPRB ,&
&  0.44088E+00_JPRB, 0.43628E+00_JPRB, 0.43288E+00_JPRB, 0.42898E+00_JPRB, 0.42568E+00_JPRB, 0.42397E+00_JPRB, 0.42282E+00_JPRB ,&
&  0.42047E+00_JPRB, 0.41620E+00_JPRB, 0.41213E+00_JPRB, 0.40950E+00_JPRB, 0.41104E+00_JPRB, 0.41699E+00_JPRB, 0.42633E+00_JPRB ,&
&  0.42852E+00_JPRB, 0.41457E+00_JPRB, 0.43829E+00_JPRB, 0.46852E+00_JPRB, 0.49169E+00_JPRB, 0.51724E+00_JPRB, 0.54068E+00_JPRB ,&
&  0.56352E+00_JPRB, 0.57248E+00_JPRB, 0.58370E+00_JPRB, 0.59689E+00_JPRB, 0.60717E+00_JPRB, 0.60722E+00_JPRB, 0.60417E+00_JPRB ,&
&  0.60001E+00_JPRB, 0.59430E+00_JPRB, 0.58703E+00_JPRB, 0.57513E+00_JPRB, 0.56218E+00_JPRB, 0.54873E+00_JPRB, 0.53498E+00_JPRB ,&
&  0.52165E+00_JPRB, 0.50950E+00_JPRB, 0.50127E+00_JPRB, 0.49824E+00_JPRB, 0.50129E+00_JPRB, 0.50814E+00_JPRB, 0.52144E+00_JPRB ,&
&  0.53809E+00_JPRB, 0.56720E+00_JPRB, 0.62004E+00_JPRB, 0.67650E+00_JPRB, 0.72262E+00_JPRB, 0.75461E+00_JPRB, 0.77957E+00_JPRB ,&
&  0.78699E+00_JPRB, 0.78405E+00_JPRB, 0.77764E+00_JPRB, 0.76532E+00_JPRB, 0.74643E+00_JPRB, 0.72064E+00_JPRB, 0.68997E+00_JPRB ,&
&  0.66217E+00_JPRB, 0.63780E+00_JPRB, 0.61396E+00_JPRB, 0.59059E+00_JPRB, 0.56743E+00_JPRB, 0.54591E+00_JPRB, 0.52742E+00_JPRB ,&
&  0.51143E+00_JPRB, 0.49721E+00_JPRB, 0.48665E+00_JPRB, 0.48127E+00_JPRB, 0.48243E+00_JPRB, 0.48810E+00_JPRB, 0.49581E+00_JPRB ,&
&  0.50233E+00_JPRB, 0.51149E+00_JPRB, 0.51913E+00_JPRB, 0.52787E+00_JPRB, 0.53487E+00_JPRB, 0.53463E+00_JPRB, 0.53378E+00_JPRB ,&
&  0.53108E+00_JPRB, 0.52246E+00_JPRB, 0.51448E+00_JPRB, 0.50503E+00_JPRB, 0.49727E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.45489E+00_JPRB, 0.44760E+00_JPRB, 0.44648E+00_JPRB, 0.45000E+00_JPRB, 0.45428E+00_JPRB, 0.45409E+00_JPRB, 0.44987E+00_JPRB ,&
&  0.43970E+00_JPRB, 0.43074E+00_JPRB, 0.42406E+00_JPRB, 0.42160E+00_JPRB, 0.42432E+00_JPRB, 0.42512E+00_JPRB, 0.42217E+00_JPRB ,&
&  0.41866E+00_JPRB, 0.41209E+00_JPRB, 0.40627E+00_JPRB, 0.40048E+00_JPRB, 0.39622E+00_JPRB, 0.39418E+00_JPRB, 0.39321E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.38923E+00_JPRB, 0.38806E+00_JPRB, 0.38723E+00_JPRB, 0.38840E+00_JPRB, 0.39250E+00_JPRB, 0.39880E+00_JPRB ,&
&  0.41213E+00_JPRB, 0.40076E+00_JPRB, 0.42482E+00_JPRB, 0.43095E+00_JPRB, 0.44145E+00_JPRB, 0.45571E+00_JPRB, 0.47570E+00_JPRB ,&
&  0.49387E+00_JPRB, 0.50053E+00_JPRB, 0.50368E+00_JPRB, 0.50634E+00_JPRB, 0.50831E+00_JPRB, 0.51297E+00_JPRB, 0.51249E+00_JPRB ,&
&  0.50855E+00_JPRB, 0.50636E+00_JPRB, 0.50534E+00_JPRB, 0.50296E+00_JPRB, 0.50056E+00_JPRB, 0.49561E+00_JPRB, 0.48904E+00_JPRB ,&
&  0.48222E+00_JPRB, 0.47520E+00_JPRB, 0.46891E+00_JPRB, 0.46255E+00_JPRB, 0.45734E+00_JPRB, 0.45490E+00_JPRB, 0.45570E+00_JPRB ,&
&  0.45801E+00_JPRB, 0.46415E+00_JPRB, 0.47993E+00_JPRB, 0.50432E+00_JPRB, 0.53314E+00_JPRB, 0.56081E+00_JPRB, 0.58293E+00_JPRB ,&
&  0.59613E+00_JPRB, 0.60238E+00_JPRB, 0.60225E+00_JPRB, 0.59839E+00_JPRB, 0.59010E+00_JPRB, 0.57934E+00_JPRB, 0.56484E+00_JPRB ,&
&  0.55060E+00_JPRB, 0.53703E+00_JPRB, 0.52313E+00_JPRB, 0.50932E+00_JPRB, 0.49446E+00_JPRB, 0.47906E+00_JPRB, 0.46517E+00_JPRB ,&
&  0.45411E+00_JPRB, 0.44609E+00_JPRB, 0.44027E+00_JPRB, 0.43678E+00_JPRB, 0.43505E+00_JPRB, 0.43643E+00_JPRB, 0.43947E+00_JPRB ,&
&  0.44387E+00_JPRB, 0.44993E+00_JPRB, 0.45686E+00_JPRB, 0.46325E+00_JPRB, 0.47025E+00_JPRB, 0.47357E+00_JPRB, 0.47594E+00_JPRB ,&
&  0.47574E+00_JPRB, 0.47199E+00_JPRB, 0.46895E+00_JPRB, 0.46502E+00_JPRB, 0.46100E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.41812E+00_JPRB, 0.41476E+00_JPRB, 0.41729E+00_JPRB, 0.42190E+00_JPRB, 0.42900E+00_JPRB, 0.42886E+00_JPRB, 0.42685E+00_JPRB ,&
&  0.42091E+00_JPRB, 0.41287E+00_JPRB, 0.40516E+00_JPRB, 0.39821E+00_JPRB, 0.39576E+00_JPRB, 0.39680E+00_JPRB, 0.39686E+00_JPRB ,&
&  0.39532E+00_JPRB, 0.39029E+00_JPRB, 0.38355E+00_JPRB, 0.37719E+00_JPRB, 0.37261E+00_JPRB, 0.37006E+00_JPRB, 0.36809E+00_JPRB ,&
&  0.36707E+00_JPRB, 0.36525E+00_JPRB, 0.36603E+00_JPRB, 0.36599E+00_JPRB, 0.36653E+00_JPRB, 0.37009E+00_JPRB, 0.37449E+00_JPRB ,&
&  0.37920E+00_JPRB, 0.40208E+00_JPRB, 0.39666E+00_JPRB, 0.39453E+00_JPRB, 0.39932E+00_JPRB, 0.40951E+00_JPRB, 0.42597E+00_JPRB ,&
&  0.44253E+00_JPRB, 0.45039E+00_JPRB, 0.45268E+00_JPRB, 0.44947E+00_JPRB, 0.44617E+00_JPRB, 0.44565E+00_JPRB, 0.44776E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44507E+00_JPRB, 0.43986E+00_JPRB, 0.43526E+00_JPRB, 0.43363E+00_JPRB, 0.43258E+00_JPRB, 0.43076E+00_JPRB ,&
&  0.42809E+00_JPRB, 0.42451E+00_JPRB, 0.41874E+00_JPRB, 0.41177E+00_JPRB, 0.40508E+00_JPRB, 0.40113E+00_JPRB, 0.40055E+00_JPRB ,&
&  0.40275E+00_JPRB, 0.40707E+00_JPRB, 0.41448E+00_JPRB, 0.42462E+00_JPRB, 0.43560E+00_JPRB, 0.44903E+00_JPRB, 0.46130E+00_JPRB ,&
&  0.47012E+00_JPRB, 0.47422E+00_JPRB, 0.47455E+00_JPRB, 0.47277E+00_JPRB, 0.46942E+00_JPRB, 0.46479E+00_JPRB, 0.46003E+00_JPRB ,&
&  0.45534E+00_JPRB, 0.45015E+00_JPRB, 0.44436E+00_JPRB, 0.43776E+00_JPRB, 0.42924E+00_JPRB, 0.42036E+00_JPRB, 0.41322E+00_JPRB ,&
&  0.40849E+00_JPRB, 0.40529E+00_JPRB, 0.40301E+00_JPRB, 0.40098E+00_JPRB, 0.39984E+00_JPRB, 0.40000E+00_JPRB, 0.40174E+00_JPRB ,&
&  0.40477E+00_JPRB, 0.40881E+00_JPRB, 0.41381E+00_JPRB, 0.41872E+00_JPRB, 0.42242E+00_JPRB, 0.42577E+00_JPRB, 0.42918E+00_JPRB ,&
&  0.43058E+00_JPRB, 0.42885E+00_JPRB, 0.42747E+00_JPRB, 0.42637E+00_JPRB, 0.42313E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.38692E+00_JPRB, 0.38669E+00_JPRB, 0.39171E+00_JPRB, 0.39687E+00_JPRB, 0.40273E+00_JPRB, 0.40514E+00_JPRB, 0.40299E+00_JPRB ,&
&  0.39995E+00_JPRB, 0.39578E+00_JPRB, 0.38942E+00_JPRB, 0.38149E+00_JPRB, 0.37436E+00_JPRB, 0.37260E+00_JPRB, 0.37298E+00_JPRB ,&
&  0.37323E+00_JPRB, 0.37054E+00_JPRB, 0.36590E+00_JPRB, 0.36090E+00_JPRB, 0.35541E+00_JPRB, 0.35060E+00_JPRB, 0.34700E+00_JPRB ,&
&  0.34595E+00_JPRB, 0.34543E+00_JPRB, 0.34629E+00_JPRB, 0.34726E+00_JPRB, 0.34832E+00_JPRB, 0.35083E+00_JPRB, 0.35765E+00_JPRB ,&
&  0.35796E+00_JPRB, 0.36801E+00_JPRB, 0.35706E+00_JPRB, 0.36046E+00_JPRB, 0.36434E+00_JPRB, 0.37015E+00_JPRB, 0.37909E+00_JPRB ,&
&  0.39030E+00_JPRB, 0.39789E+00_JPRB, 0.40230E+00_JPRB, 0.40184E+00_JPRB, 0.39923E+00_JPRB, 0.39947E+00_JPRB, 0.40058E+00_JPRB ,&
&  0.40131E+00_JPRB, 0.40126E+00_JPRB, 0.39896E+00_JPRB, 0.39367E+00_JPRB, 0.38651E+00_JPRB, 0.38049E+00_JPRB, 0.37623E+00_JPRB ,&
&  0.37523E+00_JPRB, 0.37444E+00_JPRB, 0.37212E+00_JPRB, 0.36913E+00_JPRB, 0.36589E+00_JPRB, 0.36383E+00_JPRB, 0.36413E+00_JPRB ,&
&  0.36594E+00_JPRB, 0.36781E+00_JPRB, 0.36956E+00_JPRB, 0.37360E+00_JPRB, 0.37969E+00_JPRB, 0.38570E+00_JPRB, 0.39134E+00_JPRB ,&
&  0.39592E+00_JPRB, 0.39712E+00_JPRB, 0.39613E+00_JPRB, 0.39395E+00_JPRB, 0.39227E+00_JPRB, 0.39099E+00_JPRB, 0.39063E+00_JPRB ,&
&  0.38993E+00_JPRB, 0.38850E+00_JPRB, 0.38584E+00_JPRB, 0.38211E+00_JPRB, 0.37841E+00_JPRB, 0.37497E+00_JPRB, 0.37264E+00_JPRB ,&
&  0.37147E+00_JPRB, 0.37076E+00_JPRB, 0.36980E+00_JPRB, 0.36877E+00_JPRB, 0.36886E+00_JPRB, 0.36936E+00_JPRB, 0.37058E+00_JPRB ,&
&  0.37241E+00_JPRB, 0.37489E+00_JPRB, 0.37822E+00_JPRB, 0.38147E+00_JPRB, 0.38146E+00_JPRB, 0.38178E+00_JPRB, 0.38436E+00_JPRB ,&
&  0.38707E+00_JPRB, 0.38820E+00_JPRB, 0.38827E+00_JPRB, 0.38826E+00_JPRB, 0.38844E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.36144E+00_JPRB, 0.36346E+00_JPRB, 0.36768E+00_JPRB, 0.37405E+00_JPRB, 0.37769E+00_JPRB, 0.38208E+00_JPRB, 0.38220E+00_JPRB ,&
&  0.37949E+00_JPRB, 0.37575E+00_JPRB, 0.37120E+00_JPRB, 0.36530E+00_JPRB, 0.35893E+00_JPRB, 0.35418E+00_JPRB, 0.35258E+00_JPRB ,&
&  0.35242E+00_JPRB, 0.35084E+00_JPRB, 0.34737E+00_JPRB, 0.34292E+00_JPRB, 0.33808E+00_JPRB, 0.33410E+00_JPRB, 0.33111E+00_JPRB ,&
&  0.33039E+00_JPRB, 0.33112E+00_JPRB, 0.33264E+00_JPRB, 0.33368E+00_JPRB, 0.33391E+00_JPRB, 0.33638E+00_JPRB, 0.34161E+00_JPRB ,&
&  0.34086E+00_JPRB, 0.33988E+00_JPRB, 0.33164E+00_JPRB, 0.33099E+00_JPRB, 0.33320E+00_JPRB, 0.33622E+00_JPRB, 0.34046E+00_JPRB ,&
&  0.34570E+00_JPRB, 0.35048E+00_JPRB, 0.35538E+00_JPRB, 0.35925E+00_JPRB, 0.36174E+00_JPRB, 0.36369E+00_JPRB, 0.36500E+00_JPRB ,&
&  0.36577E+00_JPRB, 0.36509E+00_JPRB, 0.36273E+00_JPRB, 0.35834E+00_JPRB, 0.35278E+00_JPRB, 0.34886E+00_JPRB, 0.34718E+00_JPRB ,&
&  0.34568E+00_JPRB, 0.34542E+00_JPRB, 0.34494E+00_JPRB, 0.34353E+00_JPRB, 0.34157E+00_JPRB, 0.34070E+00_JPRB, 0.34106E+00_JPRB ,&
&  0.34154E+00_JPRB, 0.34109E+00_JPRB, 0.33991E+00_JPRB, 0.34013E+00_JPRB, 0.34250E+00_JPRB, 0.34581E+00_JPRB, 0.34850E+00_JPRB ,&
&  0.35031E+00_JPRB, 0.35001E+00_JPRB, 0.34848E+00_JPRB, 0.34581E+00_JPRB, 0.34379E+00_JPRB, 0.34363E+00_JPRB, 0.34509E+00_JPRB ,&
&  0.34627E+00_JPRB, 0.34635E+00_JPRB, 0.34631E+00_JPRB, 0.34448E+00_JPRB, 0.34248E+00_JPRB, 0.34173E+00_JPRB, 0.34144E+00_JPRB ,&
&  0.34070E+00_JPRB, 0.33980E+00_JPRB, 0.33907E+00_JPRB, 0.33888E+00_JPRB, 0.33941E+00_JPRB, 0.33942E+00_JPRB, 0.34016E+00_JPRB ,&
&  0.34279E+00_JPRB, 0.34600E+00_JPRB, 0.34880E+00_JPRB, 0.35042E+00_JPRB, 0.35082E+00_JPRB, 0.35032E+00_JPRB, 0.35230E+00_JPRB ,&
&  0.35571E+00_JPRB, 0.35838E+00_JPRB, 0.35856E+00_JPRB, 0.35812E+00_JPRB, 0.35959E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.33971E+00_JPRB, 0.34239E+00_JPRB, 0.34686E+00_JPRB, 0.35201E+00_JPRB, 0.35528E+00_JPRB, 0.36022E+00_JPRB, 0.36120E+00_JPRB ,&
&  0.36056E+00_JPRB, 0.35872E+00_JPRB, 0.35430E+00_JPRB, 0.34883E+00_JPRB, 0.34338E+00_JPRB, 0.33872E+00_JPRB, 0.33548E+00_JPRB ,&
&  0.33366E+00_JPRB, 0.33200E+00_JPRB, 0.33001E+00_JPRB, 0.32707E+00_JPRB, 0.32366E+00_JPRB, 0.32063E+00_JPRB, 0.31838E+00_JPRB ,&
&  0.31784E+00_JPRB, 0.31815E+00_JPRB, 0.31883E+00_JPRB, 0.31905E+00_JPRB, 0.31875E+00_JPRB, 0.32007E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32160E+00_JPRB, 0.31733E+00_JPRB, 0.31366E+00_JPRB, 0.31327E+00_JPRB, 0.31328E+00_JPRB, 0.31368E+00_JPRB, 0.31489E+00_JPRB ,&
&  0.31620E+00_JPRB, 0.31723E+00_JPRB, 0.31970E+00_JPRB, 0.32278E+00_JPRB, 0.32581E+00_JPRB, 0.32959E+00_JPRB, 0.33162E+00_JPRB ,&
&  0.33207E+00_JPRB, 0.33124E+00_JPRB, 0.32955E+00_JPRB, 0.32690E+00_JPRB, 0.32515E+00_JPRB, 0.32480E+00_JPRB, 0.32408E+00_JPRB ,&
&  0.32481E+00_JPRB, 0.32550E+00_JPRB, 0.32598E+00_JPRB, 0.32432E+00_JPRB, 0.32165E+00_JPRB, 0.32095E+00_JPRB, 0.32124E+00_JPRB ,&
&  0.32120E+00_JPRB, 0.31995E+00_JPRB, 0.31764E+00_JPRB, 0.31625E+00_JPRB, 0.31552E+00_JPRB, 0.31585E+00_JPRB, 0.31592E+00_JPRB ,&
&  0.31538E+00_JPRB, 0.31376E+00_JPRB, 0.31257E+00_JPRB, 0.31196E+00_JPRB, 0.31173E+00_JPRB, 0.31212E+00_JPRB, 0.31197E+00_JPRB ,&
&  0.31282E+00_JPRB, 0.31360E+00_JPRB, 0.31268E+00_JPRB, 0.31184E+00_JPRB, 0.31109E+00_JPRB, 0.31097E+00_JPRB, 0.31069E+00_JPRB ,&
&  0.30997E+00_JPRB, 0.30919E+00_JPRB, 0.30884E+00_JPRB, 0.30909E+00_JPRB, 0.30835E+00_JPRB, 0.30710E+00_JPRB, 0.30785E+00_JPRB ,&
&  0.31138E+00_JPRB, 0.31525E+00_JPRB, 0.31852E+00_JPRB, 0.32052E+00_JPRB, 0.32108E+00_JPRB, 0.32241E+00_JPRB, 0.32529E+00_JPRB ,&
&  0.32774E+00_JPRB, 0.32892E+00_JPRB, 0.33157E+00_JPRB, 0.33381E+00_JPRB, 0.33610E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.32081E+00_JPRB, 0.32469E+00_JPRB, 0.32737E+00_JPRB, 0.33088E+00_JPRB, 0.33392E+00_JPRB, 0.33922E+00_JPRB, 0.34136E+00_JPRB ,&
&  0.34131E+00_JPRB, 0.34121E+00_JPRB, 0.33981E+00_JPRB, 0.33709E+00_JPRB, 0.33275E+00_JPRB, 0.32815E+00_JPRB, 0.32375E+00_JPRB ,&
&  0.32031E+00_JPRB, 0.31752E+00_JPRB, 0.31537E+00_JPRB, 0.31348E+00_JPRB, 0.31090E+00_JPRB, 0.30850E+00_JPRB, 0.30675E+00_JPRB ,&
&  0.30615E+00_JPRB, 0.30579E+00_JPRB, 0.30576E+00_JPRB, 0.30599E+00_JPRB, 0.30569E+00_JPRB, 0.30514E+00_JPRB, 0.30514E+00_JPRB ,&
&  0.30378E+00_JPRB, 0.29979E+00_JPRB, 0.29731E+00_JPRB, 0.29643E+00_JPRB, 0.29555E+00_JPRB, 0.29601E+00_JPRB, 0.29574E+00_JPRB ,&
&  0.29569E+00_JPRB, 0.29585E+00_JPRB, 0.29779E+00_JPRB, 0.30057E+00_JPRB, 0.30170E+00_JPRB, 0.30340E+00_JPRB, 0.30576E+00_JPRB ,&
&  0.30685E+00_JPRB, 0.30602E+00_JPRB, 0.30397E+00_JPRB, 0.30365E+00_JPRB, 0.30494E+00_JPRB, 0.30528E+00_JPRB, 0.30578E+00_JPRB ,&
&  0.30642E+00_JPRB, 0.30753E+00_JPRB, 0.30797E+00_JPRB, 0.30561E+00_JPRB, 0.30324E+00_JPRB, 0.30235E+00_JPRB, 0.30220E+00_JPRB ,&
&  0.30181E+00_JPRB, 0.30058E+00_JPRB, 0.29800E+00_JPRB, 0.29549E+00_JPRB, 0.29211E+00_JPRB, 0.28942E+00_JPRB, 0.28799E+00_JPRB ,&
&  0.28828E+00_JPRB, 0.28870E+00_JPRB, 0.29011E+00_JPRB, 0.29131E+00_JPRB, 0.29195E+00_JPRB, 0.29192E+00_JPRB, 0.29031E+00_JPRB ,&
&  0.28847E+00_JPRB, 0.28774E+00_JPRB, 0.28649E+00_JPRB, 0.28510E+00_JPRB, 0.28525E+00_JPRB, 0.28519E+00_JPRB, 0.28474E+00_JPRB ,&
&  0.28435E+00_JPRB, 0.28504E+00_JPRB, 0.28563E+00_JPRB, 0.28488E+00_JPRB, 0.28361E+00_JPRB, 0.28228E+00_JPRB, 0.28282E+00_JPRB ,&
&  0.28459E+00_JPRB, 0.28723E+00_JPRB, 0.29022E+00_JPRB, 0.29336E+00_JPRB, 0.29644E+00_JPRB, 0.29915E+00_JPRB, 0.30199E+00_JPRB ,&
&  0.30541E+00_JPRB, 0.30644E+00_JPRB, 0.30884E+00_JPRB, 0.31250E+00_JPRB, 0.31596E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.29903E+00_JPRB, 0.30461E+00_JPRB, 0.30750E+00_JPRB, 0.30972E+00_JPRB, 0.31234E+00_JPRB, 0.31728E+00_JPRB, 0.32078E+00_JPRB ,&
&  0.32163E+00_JPRB, 0.32190E+00_JPRB, 0.32178E+00_JPRB, 0.32087E+00_JPRB, 0.31916E+00_JPRB, 0.31742E+00_JPRB, 0.31440E+00_JPRB ,&
&  0.31055E+00_JPRB, 0.30699E+00_JPRB, 0.30380E+00_JPRB, 0.30083E+00_JPRB, 0.29808E+00_JPRB, 0.29561E+00_JPRB, 0.29385E+00_JPRB ,&
&  0.29365E+00_JPRB, 0.29396E+00_JPRB, 0.29346E+00_JPRB, 0.29357E+00_JPRB, 0.29353E+00_JPRB, 0.29235E+00_JPRB, 0.29047E+00_JPRB ,&
&  0.28803E+00_JPRB, 0.28415E+00_JPRB, 0.28020E+00_JPRB, 0.27939E+00_JPRB, 0.27905E+00_JPRB, 0.27808E+00_JPRB, 0.27669E+00_JPRB ,&
&  0.27798E+00_JPRB, 0.28015E+00_JPRB, 0.28291E+00_JPRB, 0.28458E+00_JPRB, 0.28578E+00_JPRB, 0.28784E+00_JPRB, 0.29039E+00_JPRB ,&
&  0.29110E+00_JPRB, 0.28827E+00_JPRB, 0.28558E+00_JPRB, 0.28538E+00_JPRB, 0.28563E+00_JPRB, 0.28542E+00_JPRB, 0.28600E+00_JPRB ,&
&  0.28679E+00_JPRB, 0.28850E+00_JPRB, 0.28951E+00_JPRB, 0.28808E+00_JPRB, 0.28556E+00_JPRB, 0.28374E+00_JPRB, 0.28256E+00_JPRB ,&
&  0.28196E+00_JPRB, 0.28131E+00_JPRB, 0.28018E+00_JPRB, 0.27833E+00_JPRB, 0.27617E+00_JPRB, 0.27391E+00_JPRB, 0.27152E+00_JPRB ,&
&  0.26943E+00_JPRB, 0.26951E+00_JPRB, 0.27246E+00_JPRB, 0.27405E+00_JPRB, 0.27332E+00_JPRB, 0.27177E+00_JPRB, 0.27027E+00_JPRB ,&
&  0.26944E+00_JPRB, 0.26648E+00_JPRB, 0.26380E+00_JPRB, 0.26177E+00_JPRB, 0.26056E+00_JPRB, 0.25889E+00_JPRB, 0.25844E+00_JPRB ,&
&  0.25924E+00_JPRB, 0.26155E+00_JPRB, 0.26208E+00_JPRB, 0.26175E+00_JPRB, 0.26102E+00_JPRB, 0.26046E+00_JPRB, 0.26156E+00_JPRB ,&
&  0.26310E+00_JPRB, 0.26576E+00_JPRB, 0.26881E+00_JPRB, 0.27284E+00_JPRB, 0.27622E+00_JPRB, 0.28012E+00_JPRB, 0.28320E+00_JPRB ,&
&  0.28509E+00_JPRB, 0.28559E+00_JPRB, 0.28688E+00_JPRB, 0.28916E+00_JPRB, 0.29297E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.27630E+00_JPRB, 0.28252E+00_JPRB, 0.28768E+00_JPRB, 0.29024E+00_JPRB, 0.29170E+00_JPRB, 0.29416E+00_JPRB, 0.29686E+00_JPRB ,&
&  0.29899E+00_JPRB, 0.30043E+00_JPRB, 0.30131E+00_JPRB, 0.30151E+00_JPRB, 0.30085E+00_JPRB, 0.29948E+00_JPRB, 0.29775E+00_JPRB ,&
&  0.29573E+00_JPRB, 0.29273E+00_JPRB, 0.28990E+00_JPRB, 0.28781E+00_JPRB, 0.28615E+00_JPRB, 0.28430E+00_JPRB, 0.28289E+00_JPRB ,&
&  0.28160E+00_JPRB, 0.28156E+00_JPRB, 0.28118E+00_JPRB, 0.28040E+00_JPRB, 0.27945E+00_JPRB, 0.27807E+00_JPRB, 0.27699E+00_JPRB ,&
&  0.27376E+00_JPRB, 0.26856E+00_JPRB, 0.26691E+00_JPRB, 0.26535E+00_JPRB, 0.26554E+00_JPRB, 0.26476E+00_JPRB, 0.26364E+00_JPRB ,&
&  0.26529E+00_JPRB, 0.26852E+00_JPRB, 0.27114E+00_JPRB, 0.27252E+00_JPRB, 0.27355E+00_JPRB, 0.27529E+00_JPRB, 0.27719E+00_JPRB ,&
&  0.27500E+00_JPRB, 0.27191E+00_JPRB, 0.26962E+00_JPRB, 0.26870E+00_JPRB, 0.26882E+00_JPRB, 0.27000E+00_JPRB, 0.27215E+00_JPRB ,&
&  0.27404E+00_JPRB, 0.27510E+00_JPRB, 0.27467E+00_JPRB, 0.27292E+00_JPRB, 0.27071E+00_JPRB, 0.26969E+00_JPRB, 0.26919E+00_JPRB ,&
&  0.26890E+00_JPRB, 0.26830E+00_JPRB, 0.26757E+00_JPRB, 0.26590E+00_JPRB, 0.25814E+00_JPRB, 0.25518E+00_JPRB, 0.25248E+00_JPRB ,&
&  0.25344E+00_JPRB, 0.25195E+00_JPRB, 0.24752E+00_JPRB, 0.25520E+00_JPRB, 0.26310E+00_JPRB, 0.25676E+00_JPRB, 0.24977E+00_JPRB ,&
&  0.24010E+00_JPRB, 0.24376E+00_JPRB, 0.24096E+00_JPRB, 0.23502E+00_JPRB, 0.23384E+00_JPRB, 0.23446E+00_JPRB, 0.23396E+00_JPRB ,&
&  0.23492E+00_JPRB, 0.23450E+00_JPRB, 0.23568E+00_JPRB, 0.23617E+00_JPRB, 0.23651E+00_JPRB, 0.23789E+00_JPRB, 0.23919E+00_JPRB ,&
&  0.24245E+00_JPRB, 0.24745E+00_JPRB, 0.25250E+00_JPRB, 0.25262E+00_JPRB, 0.25639E+00_JPRB, 0.25937E+00_JPRB, 0.26108E+00_JPRB ,&
&  0.26245E+00_JPRB, 0.26465E+00_JPRB, 0.26664E+00_JPRB, 0.26840E+00_JPRB, 0.27130E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.26074E+00_JPRB, 0.26349E+00_JPRB, 0.26677E+00_JPRB, 0.26997E+00_JPRB, 0.27220E+00_JPRB, 0.27445E+00_JPRB, 0.27649E+00_JPRB ,&
&  0.27830E+00_JPRB, 0.27961E+00_JPRB, 0.28055E+00_JPRB, 0.28119E+00_JPRB, 0.28167E+00_JPRB, 0.28158E+00_JPRB, 0.28105E+00_JPRB ,&
&  0.28049E+00_JPRB, 0.27938E+00_JPRB, 0.27774E+00_JPRB, 0.27642E+00_JPRB, 0.27559E+00_JPRB, 0.27437E+00_JPRB, 0.27258E+00_JPRB ,&
&  0.27073E+00_JPRB, 0.26898E+00_JPRB, 0.26743E+00_JPRB, 0.26571E+00_JPRB, 0.26541E+00_JPRB, 0.26604E+00_JPRB, 0.26664E+00_JPRB ,&
&  0.26020E+00_JPRB, 0.25457E+00_JPRB, 0.25718E+00_JPRB, 0.25828E+00_JPRB, 0.25361E+00_JPRB, 0.25504E+00_JPRB, 0.25582E+00_JPRB ,&
&  0.25683E+00_JPRB, 0.25838E+00_JPRB, 0.26005E+00_JPRB, 0.26081E+00_JPRB, 0.26138E+00_JPRB, 0.26238E+00_JPRB, 0.26205E+00_JPRB ,&
&  0.26000E+00_JPRB, 0.25820E+00_JPRB, 0.25653E+00_JPRB, 0.25476E+00_JPRB, 0.25437E+00_JPRB, 0.25661E+00_JPRB, 0.25805E+00_JPRB ,&
&  0.25693E+00_JPRB, 0.25508E+00_JPRB, 0.25328E+00_JPRB, 0.25378E+00_JPRB, 0.25480E+00_JPRB, 0.25533E+00_JPRB, 0.25253E+00_JPRB ,&
&  0.25192E+00_JPRB, 0.24867E+00_JPRB, 0.24586E+00_JPRB, 0.24302E+00_JPRB, 0.24200E+00_JPRB, 0.23777E+00_JPRB, 0.23306E+00_JPRB ,&
&  0.23229E+00_JPRB, 0.23749E+00_JPRB, 0.24467E+00_JPRB, 0.24547E+00_JPRB, 0.22758E+00_JPRB, 0.23197E+00_JPRB, 0.23403E+00_JPRB ,&
&  0.22908E+00_JPRB, 0.22261E+00_JPRB, 0.21777E+00_JPRB, 0.21572E+00_JPRB, 0.21497E+00_JPRB, 0.21428E+00_JPRB, 0.21492E+00_JPRB ,&
&  0.21609E+00_JPRB, 0.21656E+00_JPRB, 0.21719E+00_JPRB, 0.21876E+00_JPRB, 0.22031E+00_JPRB, 0.22194E+00_JPRB, 0.22345E+00_JPRB ,&
&  0.22688E+00_JPRB, 0.23089E+00_JPRB, 0.23370E+00_JPRB, 0.23716E+00_JPRB, 0.24083E+00_JPRB, 0.24434E+00_JPRB, 0.24734E+00_JPRB ,&
&  0.25056E+00_JPRB, 0.25152E+00_JPRB, 0.25348E+00_JPRB, 0.25721E+00_JPRB, 0.25902E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.25320E+00_JPRB, 0.25656E+00_JPRB, 0.25867E+00_JPRB, 0.26074E+00_JPRB, 0.26236E+00_JPRB, 0.26402E+00_JPRB, 0.26482E+00_JPRB ,&
&  0.26559E+00_JPRB, 0.26564E+00_JPRB, 0.26623E+00_JPRB, 0.26632E+00_JPRB, 0.26665E+00_JPRB, 0.26540E+00_JPRB, 0.26501E+00_JPRB ,&
&  0.26462E+00_JPRB, 0.26530E+00_JPRB, 0.26552E+00_JPRB, 0.26555E+00_JPRB, 0.26473E+00_JPRB, 0.26359E+00_JPRB, 0.26172E+00_JPRB ,&
&  0.25788E+00_JPRB, 0.25481E+00_JPRB, 0.25474E+00_JPRB, 0.25389E+00_JPRB, 0.25429E+00_JPRB, 0.25412E+00_JPRB, 0.25464E+00_JPRB ,&
&  0.24913E+00_JPRB, 0.24775E+00_JPRB, 0.24919E+00_JPRB, 0.24767E+00_JPRB, 0.24855E+00_JPRB, 0.24910E+00_JPRB, 0.25120E+00_JPRB ,&
&  0.25159E+00_JPRB, 0.25178E+00_JPRB, 0.25279E+00_JPRB, 0.25212E+00_JPRB, 0.25250E+00_JPRB, 0.25334E+00_JPRB, 0.25224E+00_JPRB ,&
&  0.24844E+00_JPRB, 0.24570E+00_JPRB, 0.24159E+00_JPRB, 0.23223E+00_JPRB, 0.23109E+00_JPRB, 0.23236E+00_JPRB, 0.23128E+00_JPRB ,&
&  0.22820E+00_JPRB, 0.22943E+00_JPRB, 0.22891E+00_JPRB, 0.22670E+00_JPRB, 0.22925E+00_JPRB, 0.23299E+00_JPRB, 0.23309E+00_JPRB ,&
&  0.23129E+00_JPRB, 0.23079E+00_JPRB, 0.22994E+00_JPRB, 0.22831E+00_JPRB, 0.22633E+00_JPRB, 0.22558E+00_JPRB, 0.22488E+00_JPRB ,&
&  0.22443E+00_JPRB, 0.22233E+00_JPRB, 0.22602E+00_JPRB, 0.22613E+00_JPRB, 0.21951E+00_JPRB, 0.22390E+00_JPRB, 0.22169E+00_JPRB ,&
&  0.21260E+00_JPRB, 0.20869E+00_JPRB, 0.20636E+00_JPRB, 0.20434E+00_JPRB, 0.20389E+00_JPRB, 0.20385E+00_JPRB, 0.20347E+00_JPRB ,&
&  0.20451E+00_JPRB, 0.20592E+00_JPRB, 0.20654E+00_JPRB, 0.20745E+00_JPRB, 0.20876E+00_JPRB, 0.21073E+00_JPRB, 0.21335E+00_JPRB ,&
&  0.21567E+00_JPRB, 0.21878E+00_JPRB, 0.22022E+00_JPRB, 0.22261E+00_JPRB, 0.22435E+00_JPRB, 0.22638E+00_JPRB, 0.23143E+00_JPRB ,&
&  0.23521E+00_JPRB, 0.24263E+00_JPRB, 0.25232E+00_JPRB, 0.25402E+00_JPRB, 0.24981E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.24799E+00_JPRB, 0.25051E+00_JPRB, 0.25265E+00_JPRB, 0.25446E+00_JPRB, 0.25642E+00_JPRB, 0.25802E+00_JPRB, 0.25885E+00_JPRB ,&
&  0.25900E+00_JPRB, 0.25816E+00_JPRB, 0.25815E+00_JPRB, 0.25618E+00_JPRB, 0.25430E+00_JPRB, 0.24988E+00_JPRB, 0.24988E+00_JPRB ,&
&  0.24902E+00_JPRB, 0.25008E+00_JPRB, 0.24957E+00_JPRB, 0.24993E+00_JPRB, 0.24888E+00_JPRB, 0.24821E+00_JPRB, 0.24574E+00_JPRB ,&
&  0.24227E+00_JPRB, 0.23988E+00_JPRB, 0.23767E+00_JPRB, 0.23724E+00_JPRB, 0.23829E+00_JPRB, 0.23893E+00_JPRB, 0.23842E+00_JPRB ,&
&  0.23851E+00_JPRB, 0.23871E+00_JPRB, 0.23905E+00_JPRB, 0.23872E+00_JPRB, 0.24456E+00_JPRB, 0.24721E+00_JPRB, 0.24856E+00_JPRB ,&
&  0.24817E+00_JPRB, 0.24822E+00_JPRB, 0.24823E+00_JPRB, 0.24695E+00_JPRB, 0.24573E+00_JPRB, 0.24012E+00_JPRB, 0.23679E+00_JPRB ,&
&  0.22847E+00_JPRB, 0.22231E+00_JPRB, 0.21774E+00_JPRB, 0.21490E+00_JPRB, 0.21374E+00_JPRB, 0.21390E+00_JPRB, 0.21358E+00_JPRB ,&
&  0.21173E+00_JPRB, 0.20996E+00_JPRB, 0.20881E+00_JPRB, 0.20977E+00_JPRB, 0.21136E+00_JPRB, 0.21306E+00_JPRB, 0.21504E+00_JPRB ,&
&  0.21670E+00_JPRB, 0.21713E+00_JPRB, 0.21703E+00_JPRB, 0.21762E+00_JPRB, 0.21807E+00_JPRB, 0.21878E+00_JPRB, 0.21920E+00_JPRB ,&
&  0.22044E+00_JPRB, 0.21913E+00_JPRB, 0.21991E+00_JPRB, 0.22166E+00_JPRB, 0.21853E+00_JPRB, 0.21366E+00_JPRB, 0.20894E+00_JPRB ,&
&  0.20433E+00_JPRB, 0.20175E+00_JPRB, 0.19957E+00_JPRB, 0.19813E+00_JPRB, 0.19775E+00_JPRB, 0.19774E+00_JPRB, 0.19832E+00_JPRB ,&
&  0.19912E+00_JPRB, 0.20033E+00_JPRB, 0.20107E+00_JPRB, 0.20216E+00_JPRB, 0.20351E+00_JPRB, 0.20508E+00_JPRB, 0.20695E+00_JPRB ,&
&  0.20928E+00_JPRB, 0.21143E+00_JPRB, 0.21296E+00_JPRB, 0.21506E+00_JPRB, 0.21901E+00_JPRB, 0.22406E+00_JPRB, 0.23157E+00_JPRB ,&
&  0.23947E+00_JPRB, 0.24432E+00_JPRB, 0.23918E+00_JPRB, 0.24342E+00_JPRB, 0.24659E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.24791E+00_JPRB, 0.25012E+00_JPRB, 0.25177E+00_JPRB, 0.25352E+00_JPRB, 0.25489E+00_JPRB, 0.25612E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25370E+00_JPRB, 0.25210E+00_JPRB, 0.24903E+00_JPRB, 0.24666E+00_JPRB, 0.24361E+00_JPRB, 0.24006E+00_JPRB, 0.23908E+00_JPRB ,&
&  0.23708E+00_JPRB, 0.23673E+00_JPRB, 0.23490E+00_JPRB, 0.23408E+00_JPRB, 0.23246E+00_JPRB, 0.23134E+00_JPRB, 0.22947E+00_JPRB ,&
&  0.22627E+00_JPRB, 0.22475E+00_JPRB, 0.22330E+00_JPRB, 0.22354E+00_JPRB, 0.22524E+00_JPRB, 0.22699E+00_JPRB, 0.22891E+00_JPRB ,&
&  0.23174E+00_JPRB, 0.23270E+00_JPRB, 0.23543E+00_JPRB, 0.23756E+00_JPRB, 0.24163E+00_JPRB, 0.24510E+00_JPRB, 0.24601E+00_JPRB ,&
&  0.24599E+00_JPRB, 0.24302E+00_JPRB, 0.24182E+00_JPRB, 0.23867E+00_JPRB, 0.23155E+00_JPRB, 0.22803E+00_JPRB, 0.22257E+00_JPRB ,&
&  0.21647E+00_JPRB, 0.21192E+00_JPRB, 0.20870E+00_JPRB, 0.20776E+00_JPRB, 0.20660E+00_JPRB, 0.20631E+00_JPRB, 0.20626E+00_JPRB ,&
&  0.20538E+00_JPRB, 0.20434E+00_JPRB, 0.20304E+00_JPRB, 0.20235E+00_JPRB, 0.20296E+00_JPRB, 0.20406E+00_JPRB, 0.20642E+00_JPRB ,&
&  0.20875E+00_JPRB, 0.21009E+00_JPRB, 0.21154E+00_JPRB, 0.21203E+00_JPRB, 0.21216E+00_JPRB, 0.21246E+00_JPRB, 0.21249E+00_JPRB ,&
&  0.21339E+00_JPRB, 0.21357E+00_JPRB, 0.21289E+00_JPRB, 0.21180E+00_JPRB, 0.20888E+00_JPRB, 0.20498E+00_JPRB, 0.20204E+00_JPRB ,&
&  0.19948E+00_JPRB, 0.19781E+00_JPRB, 0.19725E+00_JPRB, 0.19662E+00_JPRB, 0.19693E+00_JPRB, 0.19740E+00_JPRB, 0.19778E+00_JPRB ,&
&  0.19848E+00_JPRB, 0.19911E+00_JPRB, 0.19972E+00_JPRB, 0.20128E+00_JPRB, 0.20255E+00_JPRB, 0.20449E+00_JPRB, 0.20652E+00_JPRB ,&
&  0.20791E+00_JPRB, 0.21002E+00_JPRB, 0.21183E+00_JPRB, 0.21380E+00_JPRB, 0.21991E+00_JPRB, 0.22451E+00_JPRB, 0.23396E+00_JPRB ,&
&  0.23938E+00_JPRB, 0.23920E+00_JPRB, 0.23713E+00_JPRB, 0.24481E+00_JPRB, 0.24649E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.24761E+00_JPRB, 0.24869E+00_JPRB, 0.24879E+00_JPRB, 0.24996E+00_JPRB, 0.25063E+00_JPRB, 0.25136E+00_JPRB, 0.24988E+00_JPRB ,&
&  0.24867E+00_JPRB, 0.24741E+00_JPRB, 0.24638E+00_JPRB, 0.24290E+00_JPRB, 0.24021E+00_JPRB, 0.23677E+00_JPRB, 0.23379E+00_JPRB ,&
&  0.23112E+00_JPRB, 0.22853E+00_JPRB, 0.22609E+00_JPRB, 0.22399E+00_JPRB, 0.22195E+00_JPRB, 0.22007E+00_JPRB, 0.21817E+00_JPRB ,&
&  0.21534E+00_JPRB, 0.21454E+00_JPRB, 0.21444E+00_JPRB, 0.21593E+00_JPRB, 0.21914E+00_JPRB, 0.22095E+00_JPRB, 0.22321E+00_JPRB ,&
&  0.22526E+00_JPRB, 0.22389E+00_JPRB, 0.22780E+00_JPRB, 0.23150E+00_JPRB, 0.23460E+00_JPRB, 0.23732E+00_JPRB, 0.23996E+00_JPRB ,&
&  0.24061E+00_JPRB, 0.23334E+00_JPRB, 0.23423E+00_JPRB, 0.22817E+00_JPRB, 0.22262E+00_JPRB, 0.22072E+00_JPRB, 0.21634E+00_JPRB ,&
&  0.21501E+00_JPRB, 0.21151E+00_JPRB, 0.20925E+00_JPRB, 0.20716E+00_JPRB, 0.20444E+00_JPRB, 0.20386E+00_JPRB, 0.20288E+00_JPRB ,&
&  0.20242E+00_JPRB, 0.20273E+00_JPRB, 0.20185E+00_JPRB, 0.20167E+00_JPRB, 0.20120E+00_JPRB, 0.20081E+00_JPRB, 0.20153E+00_JPRB ,&
&  0.20260E+00_JPRB, 0.20376E+00_JPRB, 0.20483E+00_JPRB, 0.20500E+00_JPRB, 0.20488E+00_JPRB, 0.20485E+00_JPRB, 0.20471E+00_JPRB ,&
&  0.20511E+00_JPRB, 0.20516E+00_JPRB, 0.20466E+00_JPRB, 0.20407E+00_JPRB, 0.20266E+00_JPRB, 0.20123E+00_JPRB, 0.19984E+00_JPRB ,&
&  0.19857E+00_JPRB, 0.19788E+00_JPRB, 0.19728E+00_JPRB, 0.19708E+00_JPRB, 0.19707E+00_JPRB, 0.19745E+00_JPRB, 0.19785E+00_JPRB ,&
&  0.19821E+00_JPRB, 0.19917E+00_JPRB, 0.19985E+00_JPRB, 0.20101E+00_JPRB, 0.20241E+00_JPRB, 0.20365E+00_JPRB, 0.20554E+00_JPRB ,&
&  0.20761E+00_JPRB, 0.20984E+00_JPRB, 0.21311E+00_JPRB, 0.21584E+00_JPRB, 0.21935E+00_JPRB, 0.22372E+00_JPRB, 0.22674E+00_JPRB ,&
&  0.22993E+00_JPRB, 0.23234E+00_JPRB, 0.23171E+00_JPRB, 0.24065E+00_JPRB, 0.24562E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.23713E+00_JPRB, 0.23824E+00_JPRB, 0.23881E+00_JPRB, 0.23924E+00_JPRB, 0.24054E+00_JPRB, 0.24050E+00_JPRB, 0.24021E+00_JPRB ,&
&  0.23762E+00_JPRB, 0.23677E+00_JPRB, 0.23529E+00_JPRB, 0.23309E+00_JPRB, 0.23080E+00_JPRB, 0.22673E+00_JPRB, 0.22442E+00_JPRB ,&
&  0.22223E+00_JPRB, 0.21990E+00_JPRB, 0.21849E+00_JPRB, 0.21648E+00_JPRB, 0.21474E+00_JPRB, 0.21268E+00_JPRB, 0.21204E+00_JPRB ,&
&  0.21092E+00_JPRB, 0.21079E+00_JPRB, 0.21176E+00_JPRB, 0.21241E+00_JPRB, 0.21378E+00_JPRB, 0.21521E+00_JPRB, 0.21709E+00_JPRB ,&
&  0.21726E+00_JPRB, 0.21797E+00_JPRB, 0.21877E+00_JPRB, 0.21937E+00_JPRB, 0.22172E+00_JPRB, 0.22338E+00_JPRB, 0.22403E+00_JPRB ,&
&  0.22355E+00_JPRB, 0.22243E+00_JPRB, 0.22029E+00_JPRB, 0.21780E+00_JPRB, 0.21506E+00_JPRB, 0.21215E+00_JPRB, 0.20936E+00_JPRB ,&
&  0.20726E+00_JPRB, 0.20528E+00_JPRB, 0.20378E+00_JPRB, 0.20255E+00_JPRB, 0.20184E+00_JPRB, 0.20127E+00_JPRB, 0.20069E+00_JPRB ,&
&  0.20041E+00_JPRB, 0.20021E+00_JPRB, 0.19984E+00_JPRB, 0.19997E+00_JPRB, 0.19995E+00_JPRB, 0.20012E+00_JPRB, 0.20032E+00_JPRB ,&
&  0.20069E+00_JPRB, 0.20083E+00_JPRB, 0.20084E+00_JPRB, 0.20056E+00_JPRB, 0.20027E+00_JPRB, 0.19991E+00_JPRB, 0.19953E+00_JPRB ,&
&  0.19913E+00_JPRB, 0.19914E+00_JPRB, 0.19878E+00_JPRB, 0.19853E+00_JPRB, 0.19818E+00_JPRB, 0.19753E+00_JPRB, 0.19732E+00_JPRB ,&
&  0.19737E+00_JPRB, 0.19750E+00_JPRB, 0.19723E+00_JPRB, 0.19732E+00_JPRB, 0.19753E+00_JPRB, 0.19786E+00_JPRB, 0.19890E+00_JPRB ,&
&  0.19952E+00_JPRB, 0.20015E+00_JPRB, 0.20073E+00_JPRB, 0.20157E+00_JPRB, 0.20309E+00_JPRB, 0.20460E+00_JPRB, 0.20629E+00_JPRB ,&
&  0.20833E+00_JPRB, 0.21022E+00_JPRB, 0.21211E+00_JPRB, 0.21396E+00_JPRB, 0.21709E+00_JPRB, 0.21916E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22410E+00_JPRB, 0.22180E+00_JPRB, 0.23124E+00_JPRB, 0.23176E+00_JPRB, 0.23299E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.21922E+00_JPRB, 0.21893E+00_JPRB, 0.21751E+00_JPRB, 0.21589E+00_JPRB, 0.20665E+00_JPRB, 0.21360E+00_JPRB, 0.21298E+00_JPRB ,&
&  0.21387E+00_JPRB, 0.21493E+00_JPRB, 0.21703E+00_JPRB, 0.21761E+00_JPRB, 0.21756E+00_JPRB, 0.21771E+00_JPRB, 0.21763E+00_JPRB ,&
&  0.21742E+00_JPRB, 0.21661E+00_JPRB, 0.21630E+00_JPRB, 0.21569E+00_JPRB, 0.21504E+00_JPRB, 0.21384E+00_JPRB, 0.21252E+00_JPRB ,&
&  0.21191E+00_JPRB, 0.21197E+00_JPRB, 0.21242E+00_JPRB, 0.21009E+00_JPRB, 0.20896E+00_JPRB, 0.20820E+00_JPRB, 0.20777E+00_JPRB ,&
&  0.20713E+00_JPRB, 0.20690E+00_JPRB, 0.20690E+00_JPRB, 0.20685E+00_JPRB, 0.20867E+00_JPRB, 0.20926E+00_JPRB, 0.20973E+00_JPRB ,&
&  0.20986E+00_JPRB, 0.20975E+00_JPRB, 0.20898E+00_JPRB, 0.20825E+00_JPRB, 0.20751E+00_JPRB, 0.20615E+00_JPRB, 0.20474E+00_JPRB ,&
&  0.20354E+00_JPRB, 0.20264E+00_JPRB, 0.20158E+00_JPRB, 0.20078E+00_JPRB, 0.20013E+00_JPRB, 0.19952E+00_JPRB, 0.19913E+00_JPRB ,&
&  0.19905E+00_JPRB, 0.19909E+00_JPRB, 0.19911E+00_JPRB, 0.19914E+00_JPRB, 0.19914E+00_JPRB, 0.19918E+00_JPRB, 0.19926E+00_JPRB ,&
&  0.19980E+00_JPRB, 0.19984E+00_JPRB, 0.19967E+00_JPRB, 0.19934E+00_JPRB, 0.19927E+00_JPRB, 0.19891E+00_JPRB, 0.19867E+00_JPRB ,&
&  0.19847E+00_JPRB, 0.19821E+00_JPRB, 0.19809E+00_JPRB, 0.19807E+00_JPRB, 0.19808E+00_JPRB, 0.19807E+00_JPRB, 0.19819E+00_JPRB ,&
&  0.19837E+00_JPRB, 0.19861E+00_JPRB, 0.19908E+00_JPRB, 0.19943E+00_JPRB, 0.19976E+00_JPRB, 0.20005E+00_JPRB, 0.20066E+00_JPRB ,&
&  0.20131E+00_JPRB, 0.20194E+00_JPRB, 0.20256E+00_JPRB, 0.20295E+00_JPRB, 0.20387E+00_JPRB, 0.20481E+00_JPRB, 0.20584E+00_JPRB ,&
&  0.20696E+00_JPRB, 0.20838E+00_JPRB, 0.20985E+00_JPRB, 0.21134E+00_JPRB, 0.21334E+00_JPRB, 0.21490E+00_JPRB, 0.21643E+00_JPRB ,&
&  0.21814E+00_JPRB, 0.22135E+00_JPRB, 0.22203E+00_JPRB, 0.22196E+00_JPRB, 0.22095E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.21433E+00_JPRB, 0.21423E+00_JPRB, 0.21401E+00_JPRB, 0.21343E+00_JPRB, 0.21241E+00_JPRB, 0.21216E+00_JPRB, 0.21183E+00_JPRB ,&
&  0.21138E+00_JPRB, 0.21048E+00_JPRB, 0.21046E+00_JPRB, 0.21035E+00_JPRB, 0.21021E+00_JPRB, 0.21004E+00_JPRB, 0.20970E+00_JPRB ,&
&  0.20901E+00_JPRB, 0.20796E+00_JPRB, 0.20491E+00_JPRB, 0.20344E+00_JPRB, 0.20248E+00_JPRB, 0.20193E+00_JPRB, 0.20149E+00_JPRB ,&
&  0.20121E+00_JPRB, 0.20106E+00_JPRB, 0.20091E+00_JPRB, 0.20147E+00_JPRB, 0.20172E+00_JPRB, 0.20182E+00_JPRB, 0.20188E+00_JPRB ,&
&  0.20188E+00_JPRB, 0.20200E+00_JPRB, 0.20202E+00_JPRB, 0.20201E+00_JPRB, 0.20217E+00_JPRB, 0.20219E+00_JPRB, 0.20196E+00_JPRB ,&
&  0.20171E+00_JPRB, 0.20140E+00_JPRB, 0.20112E+00_JPRB, 0.20075E+00_JPRB, 0.20049E+00_JPRB, 0.20027E+00_JPRB, 0.20021E+00_JPRB ,&
&  0.20002E+00_JPRB, 0.19989E+00_JPRB, 0.19975E+00_JPRB, 0.19969E+00_JPRB, 0.19958E+00_JPRB, 0.19955E+00_JPRB, 0.19940E+00_JPRB ,&
&  0.19953E+00_JPRB, 0.19958E+00_JPRB, 0.19969E+00_JPRB, 0.19976E+00_JPRB, 0.19982E+00_JPRB, 0.19981E+00_JPRB, 0.19975E+00_JPRB ,&
&  0.19956E+00_JPRB, 0.19945E+00_JPRB, 0.19932E+00_JPRB, 0.19921E+00_JPRB, 0.19919E+00_JPRB, 0.19918E+00_JPRB, 0.19919E+00_JPRB ,&
&  0.19921E+00_JPRB, 0.19935E+00_JPRB, 0.19940E+00_JPRB, 0.19948E+00_JPRB, 0.19957E+00_JPRB, 0.19966E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19984E+00_JPRB, 0.19996E+00_JPRB, 0.20009E+00_JPRB, 0.20027E+00_JPRB, 0.20049E+00_JPRB, 0.20080E+00_JPRB, 0.20117E+00_JPRB ,&
&  0.20154E+00_JPRB, 0.20193E+00_JPRB, 0.20231E+00_JPRB, 0.20268E+00_JPRB, 0.20327E+00_JPRB, 0.20387E+00_JPRB, 0.20463E+00_JPRB ,&
&  0.20535E+00_JPRB, 0.20605E+00_JPRB, 0.20670E+00_JPRB, 0.20740E+00_JPRB, 0.20839E+00_JPRB, 0.20886E+00_JPRB, 0.20926E+00_JPRB ,&
&  0.20991E+00_JPRB, 0.21061E+00_JPRB, 0.21127E+00_JPRB, 0.21184E+00_JPRB, 0.21275E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB, 0.20181E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB, 0.85419E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.85784E+00_JPRB, 0.85546E+00_JPRB, 0.85331E+00_JPRB, 0.85104E+00_JPRB, 0.84863E+00_JPRB, 0.84670E+00_JPRB, 0.84488E+00_JPRB ,&
&  0.84327E+00_JPRB, 0.84245E+00_JPRB, 0.84202E+00_JPRB, 0.84182E+00_JPRB, 0.84189E+00_JPRB, 0.84188E+00_JPRB, 0.84197E+00_JPRB ,&
&  0.84192E+00_JPRB, 0.84203E+00_JPRB, 0.84202E+00_JPRB, 0.84167E+00_JPRB, 0.84099E+00_JPRB, 0.84008E+00_JPRB, 0.83888E+00_JPRB ,&
&  0.83718E+00_JPRB, 0.83530E+00_JPRB, 0.83352E+00_JPRB, 0.83158E+00_JPRB, 0.82982E+00_JPRB, 0.82781E+00_JPRB, 0.82568E+00_JPRB ,&
&  0.82377E+00_JPRB, 0.82267E+00_JPRB, 0.82178E+00_JPRB, 0.82089E+00_JPRB, 0.82045E+00_JPRB, 0.82134E+00_JPRB, 0.82209E+00_JPRB ,&
&  0.82292E+00_JPRB, 0.82391E+00_JPRB, 0.82607E+00_JPRB, 0.82821E+00_JPRB, 0.83022E+00_JPRB, 0.83231E+00_JPRB, 0.83443E+00_JPRB ,&
&  0.83550E+00_JPRB, 0.83606E+00_JPRB, 0.83628E+00_JPRB, 0.83655E+00_JPRB, 0.83632E+00_JPRB, 0.83634E+00_JPRB, 0.83637E+00_JPRB ,&
&  0.83627E+00_JPRB, 0.83620E+00_JPRB, 0.83589E+00_JPRB, 0.83483E+00_JPRB, 0.83207E+00_JPRB, 0.83009E+00_JPRB, 0.82929E+00_JPRB ,&
&  0.82940E+00_JPRB, 0.82971E+00_JPRB, 0.83047E+00_JPRB, 0.83165E+00_JPRB, 0.83361E+00_JPRB, 0.83578E+00_JPRB, 0.83835E+00_JPRB ,&
&  0.84112E+00_JPRB, 0.84450E+00_JPRB, 0.84880E+00_JPRB, 0.85326E+00_JPRB, 0.85819E+00_JPRB, 0.86397E+00_JPRB, 0.86926E+00_JPRB ,&
&  0.87474E+00_JPRB, 0.88053E+00_JPRB, 0.88543E+00_JPRB, 0.89016E+00_JPRB, 0.89478E+00_JPRB, 0.89951E+00_JPRB, 0.90305E+00_JPRB ,&
&  0.90476E+00_JPRB, 0.90597E+00_JPRB, 0.90684E+00_JPRB, 0.90611E+00_JPRB, 0.90430E+00_JPRB, 0.90281E+00_JPRB, 0.90124E+00_JPRB ,&
&  0.89926E+00_JPRB, 0.89677E+00_JPRB, 0.89438E+00_JPRB, 0.89171E+00_JPRB, 0.88801E+00_JPRB, 0.88397E+00_JPRB, 0.88011E+00_JPRB ,&
&  0.87630E+00_JPRB, 0.87213E+00_JPRB, 0.86830E+00_JPRB, 0.86461E+00_JPRB, 0.86116E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.88374E+00_JPRB, 0.87481E+00_JPRB, 0.86668E+00_JPRB, 0.85925E+00_JPRB, 0.85316E+00_JPRB, 0.84850E+00_JPRB, 0.84456E+00_JPRB ,&
&  0.84115E+00_JPRB, 0.83957E+00_JPRB, 0.83793E+00_JPRB, 0.83641E+00_JPRB, 0.83499E+00_JPRB, 0.83313E+00_JPRB, 0.83086E+00_JPRB ,&
&  0.82879E+00_JPRB, 0.82742E+00_JPRB, 0.82602E+00_JPRB, 0.82419E+00_JPRB, 0.82085E+00_JPRB, 0.81579E+00_JPRB, 0.80985E+00_JPRB ,&
&  0.80288E+00_JPRB, 0.79447E+00_JPRB, 0.78485E+00_JPRB, 0.77478E+00_JPRB, 0.76961E+00_JPRB, 0.76361E+00_JPRB, 0.75785E+00_JPRB ,&
&  0.75279E+00_JPRB, 0.75124E+00_JPRB, 0.75012E+00_JPRB, 0.74941E+00_JPRB, 0.74959E+00_JPRB, 0.75138E+00_JPRB, 0.75237E+00_JPRB ,&
&  0.75306E+00_JPRB, 0.75526E+00_JPRB, 0.76198E+00_JPRB, 0.76916E+00_JPRB, 0.77752E+00_JPRB, 0.78770E+00_JPRB, 0.79925E+00_JPRB ,&
&  0.80801E+00_JPRB, 0.81334E+00_JPRB, 0.81863E+00_JPRB, 0.82354E+00_JPRB, 0.82675E+00_JPRB, 0.82893E+00_JPRB, 0.83026E+00_JPRB ,&
&  0.83321E+00_JPRB, 0.83511E+00_JPRB, 0.83693E+00_JPRB, 0.83817E+00_JPRB, 0.83742E+00_JPRB, 0.83650E+00_JPRB, 0.83525E+00_JPRB ,&
&  0.83298E+00_JPRB, 0.83262E+00_JPRB, 0.83411E+00_JPRB, 0.83580E+00_JPRB, 0.83951E+00_JPRB, 0.84510E+00_JPRB, 0.85176E+00_JPRB ,&
&  0.86091E+00_JPRB, 0.87350E+00_JPRB, 0.88891E+00_JPRB, 0.90603E+00_JPRB, 0.92592E+00_JPRB, 0.95082E+00_JPRB, 0.97105E+00_JPRB ,&
&  0.98948E+00_JPRB, 0.10052E+01_JPRB, 0.10193E+01_JPRB, 0.10294E+01_JPRB, 0.10366E+01_JPRB, 0.10434E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10416E+01_JPRB, 0.10392E+01_JPRB, 0.10373E+01_JPRB, 0.10322E+01_JPRB, 0.10259E+01_JPRB, 0.10193E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.10038E+01_JPRB, 0.99434E+00_JPRB, 0.98498E+00_JPRB, 0.97584E+00_JPRB, 0.96547E+00_JPRB, 0.95541E+00_JPRB, 0.94543E+00_JPRB ,&
&  0.93536E+00_JPRB, 0.92460E+00_JPRB, 0.91428E+00_JPRB, 0.90423E+00_JPRB, 0.89409E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.91574E+00_JPRB, 0.90062E+00_JPRB, 0.88686E+00_JPRB, 0.87468E+00_JPRB, 0.86412E+00_JPRB, 0.85573E+00_JPRB, 0.84893E+00_JPRB ,&
&  0.84331E+00_JPRB, 0.84003E+00_JPRB, 0.83680E+00_JPRB, 0.83330E+00_JPRB, 0.82974E+00_JPRB, 0.82937E+00_JPRB, 0.82691E+00_JPRB ,&
&  0.82314E+00_JPRB, 0.81820E+00_JPRB, 0.81344E+00_JPRB, 0.80619E+00_JPRB, 0.79572E+00_JPRB, 0.78324E+00_JPRB, 0.76782E+00_JPRB ,&
&  0.75808E+00_JPRB, 0.74551E+00_JPRB, 0.73223E+00_JPRB, 0.71865E+00_JPRB, 0.70919E+00_JPRB, 0.68742E+00_JPRB, 0.67994E+00_JPRB ,&
&  0.67507E+00_JPRB, 0.65910E+00_JPRB, 0.65065E+00_JPRB, 0.64658E+00_JPRB, 0.66080E+00_JPRB, 0.68018E+00_JPRB, 0.69103E+00_JPRB ,&
&  0.68020E+00_JPRB, 0.68944E+00_JPRB, 0.68655E+00_JPRB, 0.68372E+00_JPRB, 0.68498E+00_JPRB, 0.68968E+00_JPRB, 0.71430E+00_JPRB ,&
&  0.75865E+00_JPRB, 0.77599E+00_JPRB, 0.78647E+00_JPRB, 0.79663E+00_JPRB, 0.80358E+00_JPRB, 0.81032E+00_JPRB, 0.81543E+00_JPRB ,&
&  0.82095E+00_JPRB, 0.82563E+00_JPRB, 0.83191E+00_JPRB, 0.83469E+00_JPRB, 0.83951E+00_JPRB, 0.84430E+00_JPRB, 0.84859E+00_JPRB ,&
&  0.84560E+00_JPRB, 0.84460E+00_JPRB, 0.84593E+00_JPRB, 0.85023E+00_JPRB, 0.85783E+00_JPRB, 0.87172E+00_JPRB, 0.88770E+00_JPRB ,&
&  0.90515E+00_JPRB, 0.91891E+00_JPRB, 0.94146E+00_JPRB, 0.97202E+00_JPRB, 0.10092E+01_JPRB, 0.10550E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.11845E+01_JPRB, 0.12206E+01_JPRB, 0.12390E+01_JPRB, 0.12488E+01_JPRB, 0.12554E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.12464E+01_JPRB, 0.12394E+01_JPRB, 0.12295E+01_JPRB, 0.12159E+01_JPRB, 0.11979E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.11533E+01_JPRB, 0.11296E+01_JPRB, 0.11063E+01_JPRB, 0.10834E+01_JPRB, 0.10606E+01_JPRB, 0.10395E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.10025E+01_JPRB, 0.98402E+00_JPRB, 0.96673E+00_JPRB, 0.94990E+00_JPRB, 0.93294E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.94724E+00_JPRB, 0.93056E+00_JPRB, 0.91289E+00_JPRB, 0.89685E+00_JPRB, 0.88337E+00_JPRB, 0.87305E+00_JPRB, 0.86573E+00_JPRB ,&
&  0.85935E+00_JPRB, 0.85148E+00_JPRB, 0.84306E+00_JPRB, 0.83405E+00_JPRB, 0.82617E+00_JPRB, 0.82282E+00_JPRB, 0.82022E+00_JPRB ,&
&  0.81835E+00_JPRB, 0.81204E+00_JPRB, 0.80358E+00_JPRB, 0.78996E+00_JPRB, 0.77517E+00_JPRB, 0.76147E+00_JPRB, 0.74506E+00_JPRB ,&
&  0.73252E+00_JPRB, 0.71969E+00_JPRB, 0.70047E+00_JPRB, 0.68953E+00_JPRB, 0.66253E+00_JPRB, 0.64435E+00_JPRB, 0.63270E+00_JPRB ,&
&  0.62411E+00_JPRB, 0.62547E+00_JPRB, 0.63022E+00_JPRB, 0.63937E+00_JPRB, 0.64335E+00_JPRB, 0.64049E+00_JPRB, 0.64071E+00_JPRB ,&
&  0.63381E+00_JPRB, 0.62899E+00_JPRB, 0.62569E+00_JPRB, 0.62508E+00_JPRB, 0.62829E+00_JPRB, 0.63628E+00_JPRB, 0.65309E+00_JPRB ,&
&  0.68884E+00_JPRB, 0.73450E+00_JPRB, 0.77575E+00_JPRB, 0.79663E+00_JPRB, 0.79752E+00_JPRB, 0.80452E+00_JPRB, 0.80535E+00_JPRB ,&
&  0.80912E+00_JPRB, 0.81014E+00_JPRB, 0.81585E+00_JPRB, 0.82442E+00_JPRB, 0.83444E+00_JPRB, 0.84376E+00_JPRB, 0.85413E+00_JPRB ,&
&  0.85848E+00_JPRB, 0.86278E+00_JPRB, 0.86111E+00_JPRB, 0.86798E+00_JPRB, 0.88381E+00_JPRB, 0.90176E+00_JPRB, 0.91027E+00_JPRB ,&
&  0.92504E+00_JPRB, 0.94486E+00_JPRB, 0.98233E+00_JPRB, 0.10391E+01_JPRB, 0.11008E+01_JPRB, 0.11582E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.13274E+01_JPRB, 0.13879E+01_JPRB, 0.14127E+01_JPRB, 0.14125E+01_JPRB, 0.14185E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.14014E+01_JPRB, 0.13894E+01_JPRB, 0.13835E+01_JPRB, 0.13776E+01_JPRB, 0.13590E+01_JPRB, 0.13298E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.12280E+01_JPRB, 0.11928E+01_JPRB, 0.11571E+01_JPRB, 0.11233E+01_JPRB, 0.10903E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.10139E+01_JPRB, 0.99419E+00_JPRB, 0.97809E+00_JPRB, 0.96282E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.97827E+00_JPRB, 0.96718E+00_JPRB, 0.95595E+00_JPRB, 0.94126E+00_JPRB, 0.92616E+00_JPRB, 0.91240E+00_JPRB, 0.90136E+00_JPRB ,&
&  0.89200E+00_JPRB, 0.88169E+00_JPRB, 0.87135E+00_JPRB, 0.85879E+00_JPRB, 0.84687E+00_JPRB, 0.83623E+00_JPRB, 0.82779E+00_JPRB ,&
&  0.82100E+00_JPRB, 0.81005E+00_JPRB, 0.79949E+00_JPRB, 0.78566E+00_JPRB, 0.77237E+00_JPRB, 0.75675E+00_JPRB, 0.73862E+00_JPRB ,&
&  0.72507E+00_JPRB, 0.70547E+00_JPRB, 0.68900E+00_JPRB, 0.67945E+00_JPRB, 0.65371E+00_JPRB, 0.64053E+00_JPRB, 0.63329E+00_JPRB ,&
&  0.64817E+00_JPRB, 0.64566E+00_JPRB, 0.66028E+00_JPRB, 0.64349E+00_JPRB, 0.63129E+00_JPRB, 0.62000E+00_JPRB, 0.60793E+00_JPRB ,&
&  0.60030E+00_JPRB, 0.59200E+00_JPRB, 0.59309E+00_JPRB, 0.59856E+00_JPRB, 0.60228E+00_JPRB, 0.61435E+00_JPRB, 0.63057E+00_JPRB ,&
&  0.66704E+00_JPRB, 0.76589E+00_JPRB, 0.78557E+00_JPRB, 0.80674E+00_JPRB, 0.81015E+00_JPRB, 0.81571E+00_JPRB, 0.81920E+00_JPRB ,&
&  0.82315E+00_JPRB, 0.82443E+00_JPRB, 0.82512E+00_JPRB, 0.82801E+00_JPRB, 0.84474E+00_JPRB, 0.85633E+00_JPRB, 0.87093E+00_JPRB ,&
&  0.87700E+00_JPRB, 0.88471E+00_JPRB, 0.88778E+00_JPRB, 0.89669E+00_JPRB, 0.91940E+00_JPRB, 0.93120E+00_JPRB, 0.93349E+00_JPRB ,&
&  0.95664E+00_JPRB, 0.98913E+00_JPRB, 0.10402E+01_JPRB, 0.11031E+01_JPRB, 0.11506E+01_JPRB, 0.11911E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.12935E+01_JPRB, 0.13626E+01_JPRB, 0.14358E+01_JPRB, 0.14695E+01_JPRB, 0.14749E+01_JPRB, 0.14657E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.14498E+01_JPRB, 0.14290E+01_JPRB, 0.14171E+01_JPRB, 0.14110E+01_JPRB, 0.13881E+01_JPRB, 0.13595E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.12575E+01_JPRB, 0.12293E+01_JPRB, 0.11969E+01_JPRB, 0.11627E+01_JPRB, 0.11292E+01_JPRB, 0.10964E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.10396E+01_JPRB, 0.10160E+01_JPRB, 0.10005E+01_JPRB, 0.98683E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.10034E+01_JPRB, 0.99266E+00_JPRB, 0.98265E+00_JPRB, 0.97288E+00_JPRB, 0.96178E+00_JPRB, 0.95054E+00_JPRB, 0.93877E+00_JPRB ,&
&  0.92852E+00_JPRB, 0.91862E+00_JPRB, 0.90793E+00_JPRB, 0.89398E+00_JPRB, 0.87940E+00_JPRB, 0.86303E+00_JPRB, 0.84988E+00_JPRB ,&
&  0.83630E+00_JPRB, 0.82191E+00_JPRB, 0.80159E+00_JPRB, 0.78260E+00_JPRB, 0.76955E+00_JPRB, 0.75670E+00_JPRB, 0.74055E+00_JPRB ,&
&  0.72628E+00_JPRB, 0.70724E+00_JPRB, 0.68541E+00_JPRB, 0.66716E+00_JPRB, 0.64361E+00_JPRB, 0.63869E+00_JPRB, 0.67191E+00_JPRB ,&
&  0.68845E+00_JPRB, 0.68866E+00_JPRB, 0.68400E+00_JPRB, 0.67434E+00_JPRB, 0.66439E+00_JPRB, 0.61760E+00_JPRB, 0.62437E+00_JPRB ,&
&  0.60565E+00_JPRB, 0.58980E+00_JPRB, 0.58863E+00_JPRB, 0.59019E+00_JPRB, 0.59770E+00_JPRB, 0.61513E+00_JPRB, 0.63450E+00_JPRB ,&
&  0.67015E+00_JPRB, 0.78331E+00_JPRB, 0.80480E+00_JPRB, 0.82514E+00_JPRB, 0.83125E+00_JPRB, 0.83671E+00_JPRB, 0.84234E+00_JPRB ,&
&  0.84550E+00_JPRB, 0.85180E+00_JPRB, 0.85855E+00_JPRB, 0.86863E+00_JPRB, 0.88072E+00_JPRB, 0.88639E+00_JPRB, 0.90344E+00_JPRB ,&
&  0.91606E+00_JPRB, 0.92796E+00_JPRB, 0.93467E+00_JPRB, 0.93380E+00_JPRB, 0.94489E+00_JPRB, 0.94038E+00_JPRB, 0.94030E+00_JPRB ,&
&  0.98603E+00_JPRB, 0.10536E+01_JPRB, 0.11124E+01_JPRB, 0.11665E+01_JPRB, 0.12143E+01_JPRB, 0.12676E+01_JPRB, 0.13326E+01_JPRB ,&
&  0.13782E+01_JPRB, 0.14433E+01_JPRB, 0.15208E+01_JPRB, 0.15473E+01_JPRB, 0.15462E+01_JPRB, 0.15416E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.15502E+01_JPRB, 0.15578E+01_JPRB, 0.15165E+01_JPRB, 0.14921E+01_JPRB, 0.14399E+01_JPRB, 0.13728E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.11921E+01_JPRB, 0.11562E+01_JPRB, 0.11300E+01_JPRB, 0.11105E+01_JPRB, 0.10973E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.10595E+01_JPRB, 0.10436E+01_JPRB, 0.10278E+01_JPRB, 0.10143E+01_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.10112E+01_JPRB, 0.10038E+01_JPRB, 0.99301E+00_JPRB, 0.98357E+00_JPRB, 0.97111E+00_JPRB, 0.96210E+00_JPRB, 0.95301E+00_JPRB ,&
&  0.94737E+00_JPRB, 0.94385E+00_JPRB, 0.93953E+00_JPRB, 0.93490E+00_JPRB, 0.92702E+00_JPRB, 0.91687E+00_JPRB, 0.90333E+00_JPRB ,&
&  0.88366E+00_JPRB, 0.83902E+00_JPRB, 0.80700E+00_JPRB, 0.79205E+00_JPRB, 0.77699E+00_JPRB, 0.76429E+00_JPRB, 0.75168E+00_JPRB ,&
&  0.73327E+00_JPRB, 0.71689E+00_JPRB, 0.69737E+00_JPRB, 0.67103E+00_JPRB, 0.65806E+00_JPRB, 0.66811E+00_JPRB, 0.67353E+00_JPRB ,&
&  0.68082E+00_JPRB, 0.69835E+00_JPRB, 0.69608E+00_JPRB, 0.68522E+00_JPRB, 0.68640E+00_JPRB, 0.68800E+00_JPRB, 0.64764E+00_JPRB ,&
&  0.61686E+00_JPRB, 0.59276E+00_JPRB, 0.58903E+00_JPRB, 0.58956E+00_JPRB, 0.59810E+00_JPRB, 0.61682E+00_JPRB, 0.66357E+00_JPRB ,&
&  0.74360E+00_JPRB, 0.84030E+00_JPRB, 0.83963E+00_JPRB, 0.84251E+00_JPRB, 0.85095E+00_JPRB, 0.85864E+00_JPRB, 0.86716E+00_JPRB ,&
&  0.87605E+00_JPRB, 0.89064E+00_JPRB, 0.90920E+00_JPRB, 0.92778E+00_JPRB, 0.95027E+00_JPRB, 0.96603E+00_JPRB, 0.97862E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.99036E+00_JPRB, 0.98935E+00_JPRB, 0.97972E+00_JPRB, 0.97160E+00_JPRB, 0.95462E+00_JPRB, 0.97907E+00_JPRB ,&
&  0.10379E+01_JPRB, 0.10972E+01_JPRB, 0.11604E+01_JPRB, 0.12125E+01_JPRB, 0.12678E+01_JPRB, 0.13419E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.15970E+01_JPRB, 0.17344E+01_JPRB, 0.17173E+01_JPRB, 0.16903E+01_JPRB, 0.16750E+01_JPRB, 0.16808E+01_JPRB ,&
&  0.16812E+01_JPRB, 0.16995E+01_JPRB, 0.16263E+01_JPRB, 0.15174E+01_JPRB, 0.14205E+01_JPRB, 0.13084E+01_JPRB, 0.12421E+01_JPRB ,&
&  0.11956E+01_JPRB, 0.11253E+01_JPRB, 0.10907E+01_JPRB, 0.10714E+01_JPRB, 0.10591E+01_JPRB, 0.10447E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.10246E+01_JPRB, 0.10244E+01_JPRB, 0.10206E+01_JPRB, 0.10164E+01_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.99641E+00_JPRB, 0.99995E+00_JPRB, 0.99840E+00_JPRB, 0.99027E+00_JPRB, 0.96591E+00_JPRB, 0.93039E+00_JPRB, 0.90696E+00_JPRB ,&
&  0.89190E+00_JPRB, 0.88855E+00_JPRB, 0.88377E+00_JPRB, 0.87948E+00_JPRB, 0.88099E+00_JPRB, 0.88826E+00_JPRB, 0.89517E+00_JPRB ,&
&  0.89563E+00_JPRB, 0.88346E+00_JPRB, 0.86078E+00_JPRB, 0.82608E+00_JPRB, 0.79834E+00_JPRB, 0.77777E+00_JPRB, 0.76561E+00_JPRB ,&
&  0.75619E+00_JPRB, 0.74315E+00_JPRB, 0.72714E+00_JPRB, 0.70040E+00_JPRB, 0.68217E+00_JPRB, 0.66807E+00_JPRB, 0.67549E+00_JPRB ,&
&  0.69539E+00_JPRB, 0.70112E+00_JPRB, 0.69606E+00_JPRB, 0.70949E+00_JPRB, 0.71331E+00_JPRB, 0.72274E+00_JPRB, 0.67132E+00_JPRB ,&
&  0.63383E+00_JPRB, 0.61754E+00_JPRB, 0.60559E+00_JPRB, 0.60284E+00_JPRB, 0.61589E+00_JPRB, 0.64323E+00_JPRB, 0.69898E+00_JPRB ,&
&  0.81552E+00_JPRB, 0.84673E+00_JPRB, 0.85863E+00_JPRB, 0.87480E+00_JPRB, 0.88745E+00_JPRB, 0.89341E+00_JPRB, 0.90572E+00_JPRB ,&
&  0.93005E+00_JPRB, 0.96882E+00_JPRB, 0.10081E+01_JPRB, 0.10143E+01_JPRB, 0.10146E+01_JPRB, 0.10167E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.13009E+01_JPRB, 0.11506E+01_JPRB, 0.10596E+01_JPRB, 0.10152E+01_JPRB, 0.99660E+00_JPRB, 0.10292E+01_JPRB ,&
&  0.10839E+01_JPRB, 0.11441E+01_JPRB, 0.12257E+01_JPRB, 0.12893E+01_JPRB, 0.13562E+01_JPRB, 0.14483E+01_JPRB, 0.15772E+01_JPRB ,&
&  0.17079E+01_JPRB, 0.18793E+01_JPRB, 0.22457E+01_JPRB, 0.20045E+01_JPRB, 0.18644E+01_JPRB, 0.18155E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.17447E+01_JPRB, 0.17456E+01_JPRB, 0.16303E+01_JPRB, 0.14735E+01_JPRB, 0.13925E+01_JPRB, 0.13028E+01_JPRB, 0.12367E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.11139E+01_JPRB, 0.10817E+01_JPRB, 0.10574E+01_JPRB, 0.10501E+01_JPRB, 0.10419E+01_JPRB, 0.10308E+01_JPRB ,&
&  0.10165E+01_JPRB, 0.10045E+01_JPRB, 0.99622E+00_JPRB, 0.99055E+00_JPRB, 0.98984E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.96516E+00_JPRB, 0.98745E+00_JPRB, 0.10053E+01_JPRB, 0.98565E+00_JPRB, 0.95183E+00_JPRB, 0.88881E+00_JPRB, 0.83960E+00_JPRB ,&
&  0.80405E+00_JPRB, 0.79327E+00_JPRB, 0.78803E+00_JPRB, 0.78881E+00_JPRB, 0.79597E+00_JPRB, 0.80932E+00_JPRB, 0.81831E+00_JPRB ,&
&  0.82822E+00_JPRB, 0.82807E+00_JPRB, 0.82727E+00_JPRB, 0.81613E+00_JPRB, 0.79623E+00_JPRB, 0.77310E+00_JPRB, 0.75725E+00_JPRB ,&
&  0.74778E+00_JPRB, 0.74149E+00_JPRB, 0.73254E+00_JPRB, 0.71826E+00_JPRB, 0.70840E+00_JPRB, 0.69480E+00_JPRB, 0.69709E+00_JPRB ,&
&  0.68876E+00_JPRB, 0.69412E+00_JPRB, 0.71107E+00_JPRB, 0.71892E+00_JPRB, 0.73339E+00_JPRB, 0.74529E+00_JPRB, 0.70649E+00_JPRB ,&
&  0.66364E+00_JPRB, 0.64373E+00_JPRB, 0.63885E+00_JPRB, 0.64271E+00_JPRB, 0.68883E+00_JPRB, 0.79618E+00_JPRB, 0.84737E+00_JPRB ,&
&  0.87551E+00_JPRB, 0.89304E+00_JPRB, 0.91251E+00_JPRB, 0.93646E+00_JPRB, 0.94507E+00_JPRB, 0.95369E+00_JPRB, 0.97911E+00_JPRB ,&
&  0.10235E+01_JPRB, 0.10926E+01_JPRB, 0.11468E+01_JPRB, 0.10341E+01_JPRB, 0.98613E+00_JPRB, 0.10255E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.11688E+01_JPRB, 0.11528E+01_JPRB, 0.11258E+01_JPRB, 0.11504E+01_JPRB, 0.11549E+01_JPRB, 0.11600E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.12397E+01_JPRB, 0.13692E+01_JPRB, 0.14388E+01_JPRB, 0.15462E+01_JPRB, 0.17028E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.20011E+01_JPRB, 0.21175E+01_JPRB, 0.21724E+01_JPRB, 0.20967E+01_JPRB, 0.20393E+01_JPRB, 0.20369E+01_JPRB, 0.19985E+01_JPRB ,&
&  0.18552E+01_JPRB, 0.17518E+01_JPRB, 0.16650E+01_JPRB, 0.15751E+01_JPRB, 0.14505E+01_JPRB, 0.13463E+01_JPRB, 0.12393E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.10591E+01_JPRB, 0.10208E+01_JPRB, 0.99503E+00_JPRB, 0.98728E+00_JPRB, 0.98496E+00_JPRB, 0.97318E+00_JPRB ,&
&  0.95468E+00_JPRB, 0.93440E+00_JPRB, 0.92492E+00_JPRB, 0.93185E+00_JPRB, 0.95521E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.10138E+01_JPRB, 0.10089E+01_JPRB, 0.97364E+00_JPRB, 0.97766E+00_JPRB, 0.94593E+00_JPRB, 0.91356E+00_JPRB, 0.86587E+00_JPRB ,&
&  0.83330E+00_JPRB, 0.79488E+00_JPRB, 0.76732E+00_JPRB, 0.73200E+00_JPRB, 0.71087E+00_JPRB, 0.70150E+00_JPRB, 0.71927E+00_JPRB ,&
&  0.74442E+00_JPRB, 0.76258E+00_JPRB, 0.77366E+00_JPRB, 0.77605E+00_JPRB, 0.77268E+00_JPRB, 0.76195E+00_JPRB, 0.75042E+00_JPRB ,&
&  0.74036E+00_JPRB, 0.73875E+00_JPRB, 0.73267E+00_JPRB, 0.72555E+00_JPRB, 0.71747E+00_JPRB, 0.70969E+00_JPRB, 0.70649E+00_JPRB ,&
&  0.69629E+00_JPRB, 0.69204E+00_JPRB, 0.69970E+00_JPRB, 0.71950E+00_JPRB, 0.74217E+00_JPRB, 0.75646E+00_JPRB, 0.72478E+00_JPRB ,&
&  0.69038E+00_JPRB, 0.68371E+00_JPRB, 0.69858E+00_JPRB, 0.79534E+00_JPRB, 0.83200E+00_JPRB, 0.85156E+00_JPRB, 0.88201E+00_JPRB ,&
&  0.86746E+00_JPRB, 0.86746E+00_JPRB, 0.88957E+00_JPRB, 0.96304E+00_JPRB, 0.98848E+00_JPRB, 0.10067E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.11816E+01_JPRB, 0.12561E+01_JPRB, 0.11552E+01_JPRB, 0.12199E+01_JPRB, 0.14146E+01_JPRB, 0.13062E+01_JPRB ,&
&  0.12666E+01_JPRB, 0.12325E+01_JPRB, 0.12652E+01_JPRB, 0.12981E+01_JPRB, 0.12936E+01_JPRB, 0.13288E+01_JPRB, 0.13615E+01_JPRB ,&
&  0.13481E+01_JPRB, 0.14182E+01_JPRB, 0.15748E+01_JPRB, 0.17292E+01_JPRB, 0.18734E+01_JPRB, 0.20615E+01_JPRB, 0.22561E+01_JPRB ,&
&  0.24929E+01_JPRB, 0.27029E+01_JPRB, 0.28503E+01_JPRB, 0.28233E+01_JPRB, 0.27523E+01_JPRB, 0.26204E+01_JPRB, 0.24291E+01_JPRB ,&
&  0.21654E+01_JPRB, 0.19760E+01_JPRB, 0.18492E+01_JPRB, 0.17468E+01_JPRB, 0.16198E+01_JPRB, 0.14541E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.97646E+00_JPRB, 0.93522E+00_JPRB, 0.92421E+00_JPRB, 0.92976E+00_JPRB, 0.93482E+00_JPRB, 0.92871E+00_JPRB ,&
&  0.91853E+00_JPRB, 0.91139E+00_JPRB, 0.90760E+00_JPRB, 0.92161E+00_JPRB, 0.98235E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.10505E+01_JPRB, 0.10301E+01_JPRB, 0.10069E+01_JPRB, 0.10006E+01_JPRB, 0.99013E+00_JPRB, 0.94130E+00_JPRB, 0.86295E+00_JPRB ,&
&  0.86916E+00_JPRB, 0.82561E+00_JPRB, 0.81469E+00_JPRB, 0.79990E+00_JPRB, 0.77689E+00_JPRB, 0.73277E+00_JPRB, 0.69867E+00_JPRB ,&
&  0.69737E+00_JPRB, 0.71632E+00_JPRB, 0.73690E+00_JPRB, 0.75406E+00_JPRB, 0.76033E+00_JPRB, 0.75583E+00_JPRB, 0.74805E+00_JPRB ,&
&  0.74127E+00_JPRB, 0.73939E+00_JPRB, 0.74091E+00_JPRB, 0.74277E+00_JPRB, 0.73980E+00_JPRB, 0.73554E+00_JPRB, 0.73576E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.72839E+00_JPRB, 0.72284E+00_JPRB, 0.73285E+00_JPRB, 0.74322E+00_JPRB, 0.76642E+00_JPRB, 0.78403E+00_JPRB ,&
&  0.72521E+00_JPRB, 0.73955E+00_JPRB, 0.82973E+00_JPRB, 0.85100E+00_JPRB, 0.87407E+00_JPRB, 0.89459E+00_JPRB, 0.91189E+00_JPRB ,&
&  0.93174E+00_JPRB, 0.95998E+00_JPRB, 0.99395E+00_JPRB, 0.10254E+01_JPRB, 0.10499E+01_JPRB, 0.10854E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.11507E+01_JPRB, 0.11180E+01_JPRB, 0.11962E+01_JPRB, 0.15500E+01_JPRB, 0.15928E+01_JPRB, 0.15754E+01_JPRB ,&
&  0.15810E+01_JPRB, 0.15183E+01_JPRB, 0.15037E+01_JPRB, 0.15453E+01_JPRB, 0.15719E+01_JPRB, 0.16511E+01_JPRB, 0.17663E+01_JPRB ,&
&  0.17756E+01_JPRB, 0.18946E+01_JPRB, 0.20471E+01_JPRB, 0.21805E+01_JPRB, 0.23292E+01_JPRB, 0.25520E+01_JPRB, 0.28058E+01_JPRB ,&
&  0.30737E+01_JPRB, 0.33013E+01_JPRB, 0.34000E+01_JPRB, 0.33456E+01_JPRB, 0.31936E+01_JPRB, 0.29981E+01_JPRB, 0.27503E+01_JPRB ,&
&  0.24289E+01_JPRB, 0.22034E+01_JPRB, 0.20532E+01_JPRB, 0.19065E+01_JPRB, 0.17723E+01_JPRB, 0.15948E+01_JPRB, 0.13359E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.95231E+00_JPRB, 0.93041E+00_JPRB, 0.92781E+00_JPRB, 0.93537E+00_JPRB, 0.95100E+00_JPRB, 0.96515E+00_JPRB ,&
&  0.97135E+00_JPRB, 0.95471E+00_JPRB, 0.95297E+00_JPRB, 0.98651E+00_JPRB, 0.10278E+01_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.10212E+01_JPRB, 0.10128E+01_JPRB, 0.10172E+01_JPRB, 0.10340E+01_JPRB, 0.10181E+01_JPRB, 0.96843E+00_JPRB, 0.91686E+00_JPRB ,&
&  0.10313E+01_JPRB, 0.10204E+01_JPRB, 0.10742E+01_JPRB, 0.91654E+00_JPRB, 0.86055E+00_JPRB, 0.83207E+00_JPRB, 0.78974E+00_JPRB ,&
&  0.75611E+00_JPRB, 0.73554E+00_JPRB, 0.72748E+00_JPRB, 0.74015E+00_JPRB, 0.74637E+00_JPRB, 0.74237E+00_JPRB, 0.73771E+00_JPRB ,&
&  0.73422E+00_JPRB, 0.73648E+00_JPRB, 0.74532E+00_JPRB, 0.74873E+00_JPRB, 0.75067E+00_JPRB, 0.75331E+00_JPRB, 0.75922E+00_JPRB ,&
&  0.74143E+00_JPRB, 0.73709E+00_JPRB, 0.74142E+00_JPRB, 0.74342E+00_JPRB, 0.75688E+00_JPRB, 0.76760E+00_JPRB, 0.79181E+00_JPRB ,&
&  0.83918E+00_JPRB, 0.81654E+00_JPRB, 0.90597E+00_JPRB, 0.92178E+00_JPRB, 0.94067E+00_JPRB, 0.95798E+00_JPRB, 0.97257E+00_JPRB ,&
&  0.98543E+00_JPRB, 0.10076E+01_JPRB, 0.10381E+01_JPRB, 0.10641E+01_JPRB, 0.10981E+01_JPRB, 0.11546E+01_JPRB, 0.12108E+01_JPRB ,&
&  0.13423E+01_JPRB, 0.11919E+01_JPRB, 0.12863E+01_JPRB, 0.13443E+01_JPRB, 0.15924E+01_JPRB, 0.18665E+01_JPRB, 0.18478E+01_JPRB ,&
&  0.21164E+01_JPRB, 0.19947E+01_JPRB, 0.18984E+01_JPRB, 0.18175E+01_JPRB, 0.18717E+01_JPRB, 0.19804E+01_JPRB, 0.21107E+01_JPRB ,&
&  0.22294E+01_JPRB, 0.23450E+01_JPRB, 0.25415E+01_JPRB, 0.27067E+01_JPRB, 0.28828E+01_JPRB, 0.30935E+01_JPRB, 0.32943E+01_JPRB ,&
&  0.34741E+01_JPRB, 0.35876E+01_JPRB, 0.36508E+01_JPRB, 0.34529E+01_JPRB, 0.32798E+01_JPRB, 0.30953E+01_JPRB, 0.27263E+01_JPRB ,&
&  0.24405E+01_JPRB, 0.22275E+01_JPRB, 0.20522E+01_JPRB, 0.19218E+01_JPRB, 0.16938E+01_JPRB, 0.15050E+01_JPRB, 0.13234E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10000E+01_JPRB, 0.10063E+01_JPRB, 0.10210E+01_JPRB, 0.10208E+01_JPRB, 0.10300E+01_JPRB, 0.10336E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.10097E+01_JPRB, 0.10402E+01_JPRB, 0.10401E+01_JPRB, 0.10315E+01_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.10839E+01_JPRB, 0.11000E+01_JPRB, 0.11073E+01_JPRB, 0.10966E+01_JPRB, 0.10723E+01_JPRB, 0.10868E+01_JPRB, 0.10470E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11658E+01_JPRB, 0.11614E+01_JPRB, 0.11635E+01_JPRB, 0.11766E+01_JPRB, 0.10855E+01_JPRB, 0.92367E+00_JPRB ,&
&  0.84441E+00_JPRB, 0.83001E+00_JPRB, 0.82452E+00_JPRB, 0.81811E+00_JPRB, 0.81577E+00_JPRB, 0.80928E+00_JPRB, 0.80619E+00_JPRB ,&
&  0.80313E+00_JPRB, 0.79612E+00_JPRB, 0.79817E+00_JPRB, 0.79815E+00_JPRB, 0.79303E+00_JPRB, 0.79068E+00_JPRB, 0.79554E+00_JPRB ,&
&  0.76870E+00_JPRB, 0.75276E+00_JPRB, 0.74250E+00_JPRB, 0.73061E+00_JPRB, 0.75611E+00_JPRB, 0.77931E+00_JPRB, 0.81596E+00_JPRB ,&
&  0.86780E+00_JPRB, 0.92035E+00_JPRB, 0.95377E+00_JPRB, 0.98331E+00_JPRB, 0.10073E+01_JPRB, 0.10294E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.10698E+01_JPRB, 0.10920E+01_JPRB, 0.11213E+01_JPRB, 0.11439E+01_JPRB, 0.12021E+01_JPRB, 0.12496E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.15129E+01_JPRB, 0.16624E+01_JPRB, 0.17691E+01_JPRB, 0.16222E+01_JPRB, 0.18055E+01_JPRB, 0.20427E+01_JPRB, 0.20594E+01_JPRB ,&
&  0.21355E+01_JPRB, 0.21346E+01_JPRB, 0.22036E+01_JPRB, 0.23055E+01_JPRB, 0.25678E+01_JPRB, 0.27614E+01_JPRB, 0.28529E+01_JPRB ,&
&  0.29346E+01_JPRB, 0.32249E+01_JPRB, 0.34273E+01_JPRB, 0.35433E+01_JPRB, 0.35900E+01_JPRB, 0.36390E+01_JPRB, 0.37212E+01_JPRB ,&
&  0.37829E+01_JPRB, 0.38425E+01_JPRB, 0.38268E+01_JPRB, 0.37028E+01_JPRB, 0.35115E+01_JPRB, 0.32513E+01_JPRB, 0.28962E+01_JPRB ,&
&  0.25819E+01_JPRB, 0.21968E+01_JPRB, 0.18836E+01_JPRB, 0.16940E+01_JPRB, 0.15189E+01_JPRB, 0.13484E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.11067E+01_JPRB, 0.11935E+01_JPRB, 0.12016E+01_JPRB, 0.12021E+01_JPRB, 0.11767E+01_JPRB, 0.10900E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10950E+01_JPRB, 0.11111E+01_JPRB, 0.11008E+01_JPRB, 0.10862E+01_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.12546E+01_JPRB, 0.12467E+01_JPRB, 0.12321E+01_JPRB, 0.12099E+01_JPRB, 0.12062E+01_JPRB, 0.12120E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.12307E+01_JPRB, 0.12566E+01_JPRB, 0.12548E+01_JPRB, 0.12308E+01_JPRB, 0.12024E+01_JPRB, 0.11850E+01_JPRB, 0.10786E+01_JPRB ,&
&  0.95009E+00_JPRB, 0.91317E+00_JPRB, 0.90322E+00_JPRB, 0.92055E+00_JPRB, 0.94872E+00_JPRB, 0.95646E+00_JPRB, 0.96037E+00_JPRB ,&
&  0.96563E+00_JPRB, 0.94551E+00_JPRB, 0.94392E+00_JPRB, 0.93844E+00_JPRB, 0.92298E+00_JPRB, 0.90805E+00_JPRB, 0.89327E+00_JPRB ,&
&  0.83762E+00_JPRB, 0.80070E+00_JPRB, 0.78239E+00_JPRB, 0.77265E+00_JPRB, 0.79392E+00_JPRB, 0.83339E+00_JPRB, 0.88534E+00_JPRB ,&
&  0.94522E+00_JPRB, 0.99345E+00_JPRB, 0.10382E+01_JPRB, 0.10793E+01_JPRB, 0.11082E+01_JPRB, 0.11360E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.12093E+01_JPRB, 0.12246E+01_JPRB, 0.12506E+01_JPRB, 0.12748E+01_JPRB, 0.15039E+01_JPRB, 0.18388E+01_JPRB ,&
&  0.18727E+01_JPRB, 0.19615E+01_JPRB, 0.20820E+01_JPRB, 0.23040E+01_JPRB, 0.24274E+01_JPRB, 0.23855E+01_JPRB, 0.24359E+01_JPRB ,&
&  0.26483E+01_JPRB, 0.28941E+01_JPRB, 0.33996E+01_JPRB, 0.34387E+01_JPRB, 0.35253E+01_JPRB, 0.36433E+01_JPRB, 0.36169E+01_JPRB ,&
&  0.37104E+01_JPRB, 0.38102E+01_JPRB, 0.39819E+01_JPRB, 0.39773E+01_JPRB, 0.39109E+01_JPRB, 0.39394E+01_JPRB, 0.39324E+01_JPRB ,&
&  0.39786E+01_JPRB, 0.40151E+01_JPRB, 0.38332E+01_JPRB, 0.36683E+01_JPRB, 0.33506E+01_JPRB, 0.33464E+01_JPRB, 0.31056E+01_JPRB ,&
&  0.26220E+01_JPRB, 0.19794E+01_JPRB, 0.18049E+01_JPRB, 0.16165E+01_JPRB, 0.14928E+01_JPRB, 0.13624E+01_JPRB, 0.12632E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.13206E+01_JPRB, 0.13434E+01_JPRB, 0.13595E+01_JPRB, 0.13751E+01_JPRB, 0.13445E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.12452E+01_JPRB, 0.12640E+01_JPRB, 0.12618E+01_JPRB, 0.12578E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.14971E+01_JPRB, 0.14541E+01_JPRB, 0.14183E+01_JPRB, 0.13971E+01_JPRB, 0.13735E+01_JPRB, 0.13398E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.13420E+01_JPRB, 0.13437E+01_JPRB, 0.12998E+01_JPRB, 0.12496E+01_JPRB, 0.12137E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.10685E+01_JPRB, 0.10787E+01_JPRB, 0.11407E+01_JPRB, 0.12915E+01_JPRB, 0.13075E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.12934E+01_JPRB, 0.12714E+01_JPRB, 0.12418E+01_JPRB, 0.11917E+01_JPRB, 0.11520E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.99447E+00_JPRB, 0.91753E+00_JPRB, 0.88977E+00_JPRB, 0.87691E+00_JPRB, 0.90728E+00_JPRB, 0.94617E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.11325E+01_JPRB, 0.11868E+01_JPRB, 0.12414E+01_JPRB, 0.12801E+01_JPRB, 0.13161E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.13601E+01_JPRB, 0.13715E+01_JPRB, 0.13921E+01_JPRB, 0.13971E+01_JPRB, 0.14767E+01_JPRB, 0.19480E+01_JPRB, 0.24843E+01_JPRB ,&
&  0.25489E+01_JPRB, 0.26583E+01_JPRB, 0.33761E+01_JPRB, 0.42274E+01_JPRB, 0.39235E+01_JPRB, 0.35363E+01_JPRB, 0.34183E+01_JPRB ,&
&  0.36498E+01_JPRB, 0.39898E+01_JPRB, 0.43522E+01_JPRB, 0.44650E+01_JPRB, 0.44178E+01_JPRB, 0.44989E+01_JPRB, 0.44982E+01_JPRB ,&
&  0.44388E+01_JPRB, 0.44409E+01_JPRB, 0.43542E+01_JPRB, 0.42267E+01_JPRB, 0.41111E+01_JPRB, 0.41898E+01_JPRB, 0.41775E+01_JPRB ,&
&  0.41211E+01_JPRB, 0.39584E+01_JPRB, 0.36687E+01_JPRB, 0.35090E+01_JPRB, 0.29623E+01_JPRB, 0.28768E+01_JPRB, 0.32105E+01_JPRB ,&
&  0.26730E+01_JPRB, 0.21338E+01_JPRB, 0.18465E+01_JPRB, 0.16899E+01_JPRB, 0.16465E+01_JPRB, 0.17206E+01_JPRB, 0.17087E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.15624E+01_JPRB, 0.17019E+01_JPRB, 0.17687E+01_JPRB, 0.17820E+01_JPRB, 0.17567E+01_JPRB, 0.15977E+01_JPRB ,&
&  0.15992E+01_JPRB, 0.15513E+01_JPRB, 0.15538E+01_JPRB, 0.15520E+01_JPRB, 0.15336E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.17498E+01_JPRB, 0.16613E+01_JPRB, 0.16096E+01_JPRB, 0.15424E+01_JPRB, 0.14656E+01_JPRB, 0.13958E+01_JPRB, 0.13575E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.14056E+01_JPRB, 0.13872E+01_JPRB, 0.13323E+01_JPRB, 0.12884E+01_JPRB, 0.12584E+01_JPRB, 0.12688E+01_JPRB ,&
&  0.12683E+01_JPRB, 0.12399E+01_JPRB, 0.12384E+01_JPRB, 0.12531E+01_JPRB, 0.14988E+01_JPRB, 0.17195E+01_JPRB, 0.19124E+01_JPRB ,&
&  0.19740E+01_JPRB, 0.20022E+01_JPRB, 0.19692E+01_JPRB, 0.18559E+01_JPRB, 0.17298E+01_JPRB, 0.16121E+01_JPRB, 0.14824E+01_JPRB ,&
&  0.13420E+01_JPRB, 0.12080E+01_JPRB, 0.11367E+01_JPRB, 0.11244E+01_JPRB, 0.11703E+01_JPRB, 0.11213E+01_JPRB, 0.11941E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.13384E+01_JPRB, 0.13907E+01_JPRB, 0.14454E+01_JPRB, 0.14760E+01_JPRB, 0.15001E+01_JPRB, 0.15059E+01_JPRB ,&
&  0.15309E+01_JPRB, 0.15126E+01_JPRB, 0.14171E+01_JPRB, 0.14904E+01_JPRB, 0.17400E+01_JPRB, 0.24896E+01_JPRB, 0.32567E+01_JPRB ,&
&  0.36725E+01_JPRB, 0.44189E+01_JPRB, 0.54353E+01_JPRB, 0.59322E+01_JPRB, 0.52546E+01_JPRB, 0.47661E+01_JPRB, 0.47692E+01_JPRB ,&
&  0.50552E+01_JPRB, 0.53883E+01_JPRB, 0.55169E+01_JPRB, 0.52645E+01_JPRB, 0.52859E+01_JPRB, 0.53312E+01_JPRB, 0.53234E+01_JPRB ,&
&  0.51972E+01_JPRB, 0.49515E+01_JPRB, 0.47069E+01_JPRB, 0.44865E+01_JPRB, 0.43369E+01_JPRB, 0.40296E+01_JPRB, 0.39473E+01_JPRB ,&
&  0.40314E+01_JPRB, 0.34428E+01_JPRB, 0.31264E+01_JPRB, 0.32665E+01_JPRB, 0.27207E+01_JPRB, 0.26649E+01_JPRB, 0.28604E+01_JPRB ,&
&  0.24581E+01_JPRB, 0.20835E+01_JPRB, 0.19115E+01_JPRB, 0.17878E+01_JPRB, 0.18309E+01_JPRB, 0.21938E+01_JPRB, 0.24676E+01_JPRB ,&
&  0.23361E+01_JPRB, 0.22033E+01_JPRB, 0.25124E+01_JPRB, 0.27040E+01_JPRB, 0.28073E+01_JPRB, 0.26792E+01_JPRB, 0.24645E+01_JPRB ,&
&  0.22951E+01_JPRB, 0.21818E+01_JPRB, 0.20897E+01_JPRB, 0.19857E+01_JPRB, 0.18756E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.19273E+01_JPRB, 0.18778E+01_JPRB, 0.17898E+01_JPRB, 0.16682E+01_JPRB, 0.15639E+01_JPRB, 0.14877E+01_JPRB, 0.14242E+01_JPRB ,&
&  0.13969E+01_JPRB, 0.13637E+01_JPRB, 0.13391E+01_JPRB, 0.12982E+01_JPRB, 0.12819E+01_JPRB, 0.12835E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.13262E+01_JPRB, 0.12820E+01_JPRB, 0.13118E+01_JPRB, 0.15553E+01_JPRB, 0.22554E+01_JPRB, 0.27725E+01_JPRB ,&
&  0.29694E+01_JPRB, 0.29763E+01_JPRB, 0.28806E+01_JPRB, 0.28075E+01_JPRB, 0.26435E+01_JPRB, 0.23765E+01_JPRB, 0.22061E+01_JPRB ,&
&  0.20376E+01_JPRB, 0.18377E+01_JPRB, 0.15894E+01_JPRB, 0.15653E+01_JPRB, 0.15294E+01_JPRB, 0.14302E+01_JPRB, 0.15419E+01_JPRB ,&
&  0.15962E+01_JPRB, 0.16161E+01_JPRB, 0.16162E+01_JPRB, 0.16153E+01_JPRB, 0.16080E+01_JPRB, 0.15287E+01_JPRB, 0.14503E+01_JPRB ,&
&  0.14313E+01_JPRB, 0.14753E+01_JPRB, 0.15459E+01_JPRB, 0.18137E+01_JPRB, 0.25006E+01_JPRB, 0.36088E+01_JPRB, 0.45189E+01_JPRB ,&
&  0.51945E+01_JPRB, 0.55411E+01_JPRB, 0.52545E+01_JPRB, 0.57317E+01_JPRB, 0.64743E+01_JPRB, 0.62227E+01_JPRB, 0.58912E+01_JPRB ,&
&  0.58857E+01_JPRB, 0.58488E+01_JPRB, 0.57192E+01_JPRB, 0.57191E+01_JPRB, 0.60397E+01_JPRB, 0.61433E+01_JPRB, 0.60195E+01_JPRB ,&
&  0.52579E+01_JPRB, 0.49795E+01_JPRB, 0.48587E+01_JPRB, 0.46925E+01_JPRB, 0.41996E+01_JPRB, 0.40047E+01_JPRB, 0.40814E+01_JPRB ,&
&  0.39936E+01_JPRB, 0.39480E+01_JPRB, 0.31834E+01_JPRB, 0.30196E+01_JPRB, 0.23721E+01_JPRB, 0.22976E+01_JPRB, 0.25808E+01_JPRB ,&
&  0.23765E+01_JPRB, 0.22040E+01_JPRB, 0.20001E+01_JPRB, 0.20125E+01_JPRB, 0.26368E+01_JPRB, 0.33457E+01_JPRB, 0.41815E+01_JPRB ,&
&  0.40848E+01_JPRB, 0.34418E+01_JPRB, 0.36789E+01_JPRB, 0.38175E+01_JPRB, 0.39763E+01_JPRB, 0.37399E+01_JPRB, 0.33102E+01_JPRB ,&
&  0.29574E+01_JPRB, 0.26809E+01_JPRB, 0.24127E+01_JPRB, 0.21687E+01_JPRB, 0.20128E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.20006E+01_JPRB, 0.19409E+01_JPRB, 0.18292E+01_JPRB, 0.16916E+01_JPRB, 0.15886E+01_JPRB, 0.15094E+01_JPRB, 0.14450E+01_JPRB ,&
&  0.13791E+01_JPRB, 0.13231E+01_JPRB, 0.12905E+01_JPRB, 0.12702E+01_JPRB, 0.12649E+01_JPRB, 0.12689E+01_JPRB, 0.12774E+01_JPRB ,&
&  0.12851E+01_JPRB, 0.13456E+01_JPRB, 0.13221E+01_JPRB, 0.13731E+01_JPRB, 0.17626E+01_JPRB, 0.26326E+01_JPRB, 0.34781E+01_JPRB ,&
&  0.38487E+01_JPRB, 0.37601E+01_JPRB, 0.36410E+01_JPRB, 0.35499E+01_JPRB, 0.35146E+01_JPRB, 0.33924E+01_JPRB, 0.32148E+01_JPRB ,&
&  0.29788E+01_JPRB, 0.23717E+01_JPRB, 0.21554E+01_JPRB, 0.19847E+01_JPRB, 0.19571E+01_JPRB, 0.19317E+01_JPRB, 0.18720E+01_JPRB ,&
&  0.17830E+01_JPRB, 0.16904E+01_JPRB, 0.16216E+01_JPRB, 0.15468E+01_JPRB, 0.15042E+01_JPRB, 0.14381E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.17098E+01_JPRB, 0.21557E+01_JPRB, 0.29109E+01_JPRB, 0.42027E+01_JPRB, 0.56870E+01_JPRB, 0.58774E+01_JPRB ,&
&  0.49712E+01_JPRB, 0.49529E+01_JPRB, 0.69419E+01_JPRB, 0.65268E+01_JPRB, 0.66670E+01_JPRB, 0.73527E+01_JPRB, 0.69144E+01_JPRB ,&
&  0.62391E+01_JPRB, 0.57554E+01_JPRB, 0.55531E+01_JPRB, 0.57553E+01_JPRB, 0.72837E+01_JPRB, 0.74320E+01_JPRB, 0.68437E+01_JPRB ,&
&  0.57926E+01_JPRB, 0.55275E+01_JPRB, 0.51150E+01_JPRB, 0.46586E+01_JPRB, 0.43740E+01_JPRB, 0.45870E+01_JPRB, 0.43471E+01_JPRB ,&
&  0.40811E+01_JPRB, 0.44585E+01_JPRB, 0.35963E+01_JPRB, 0.27998E+01_JPRB, 0.23609E+01_JPRB, 0.24291E+01_JPRB, 0.25929E+01_JPRB ,&
&  0.25936E+01_JPRB, 0.25417E+01_JPRB, 0.25919E+01_JPRB, 0.33270E+01_JPRB, 0.48994E+01_JPRB, 0.61363E+01_JPRB, 0.59271E+01_JPRB ,&
&  0.55757E+01_JPRB, 0.59018E+01_JPRB, 0.55332E+01_JPRB, 0.54725E+01_JPRB, 0.53579E+01_JPRB, 0.46626E+01_JPRB, 0.37702E+01_JPRB ,&
&  0.30169E+01_JPRB, 0.25393E+01_JPRB, 0.22822E+01_JPRB, 0.21352E+01_JPRB, 0.20503E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.21096E+01_JPRB, 0.19847E+01_JPRB, 0.18524E+01_JPRB, 0.17221E+01_JPRB, 0.16029E+01_JPRB, 0.15046E+01_JPRB, 0.14221E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.12831E+01_JPRB, 0.12670E+01_JPRB, 0.12741E+01_JPRB, 0.12881E+01_JPRB, 0.12758E+01_JPRB, 0.12871E+01_JPRB ,&
&  0.13327E+01_JPRB, 0.13819E+01_JPRB, 0.13737E+01_JPRB, 0.15328E+01_JPRB, 0.19346E+01_JPRB, 0.25704E+01_JPRB, 0.34933E+01_JPRB ,&
&  0.41127E+01_JPRB, 0.39048E+01_JPRB, 0.36217E+01_JPRB, 0.38521E+01_JPRB, 0.42338E+01_JPRB, 0.44144E+01_JPRB, 0.43095E+01_JPRB ,&
&  0.37201E+01_JPRB, 0.32406E+01_JPRB, 0.29049E+01_JPRB, 0.25676E+01_JPRB, 0.23588E+01_JPRB, 0.21530E+01_JPRB, 0.19029E+01_JPRB ,&
&  0.16888E+01_JPRB, 0.15636E+01_JPRB, 0.15041E+01_JPRB, 0.14823E+01_JPRB, 0.15172E+01_JPRB, 0.15755E+01_JPRB, 0.16911E+01_JPRB ,&
&  0.19112E+01_JPRB, 0.23192E+01_JPRB, 0.30515E+01_JPRB, 0.40929E+01_JPRB, 0.47478E+01_JPRB, 0.65599E+01_JPRB, 0.62593E+01_JPRB ,&
&  0.61155E+01_JPRB, 0.62383E+01_JPRB, 0.62206E+01_JPRB, 0.67363E+01_JPRB, 0.59127E+01_JPRB, 0.64405E+01_JPRB, 0.67019E+01_JPRB ,&
&  0.63911E+01_JPRB, 0.61729E+01_JPRB, 0.64289E+01_JPRB, 0.64029E+01_JPRB, 0.73255E+01_JPRB, 0.91214E+01_JPRB, 0.82621E+01_JPRB ,&
&  0.68667E+01_JPRB, 0.61517E+01_JPRB, 0.53661E+01_JPRB, 0.49617E+01_JPRB, 0.42216E+01_JPRB, 0.42224E+01_JPRB, 0.37424E+01_JPRB ,&
&  0.35377E+01_JPRB, 0.29383E+01_JPRB, 0.31968E+01_JPRB, 0.27409E+01_JPRB, 0.26158E+01_JPRB, 0.25974E+01_JPRB, 0.26752E+01_JPRB ,&
&  0.29347E+01_JPRB, 0.32704E+01_JPRB, 0.42188E+01_JPRB, 0.59786E+01_JPRB, 0.82182E+01_JPRB, 0.72808E+01_JPRB, 0.74773E+01_JPRB ,&
&  0.74031E+01_JPRB, 0.72584E+01_JPRB, 0.61226E+01_JPRB, 0.56954E+01_JPRB, 0.50445E+01_JPRB, 0.44930E+01_JPRB, 0.39075E+01_JPRB ,&
&  0.33058E+01_JPRB, 0.28060E+01_JPRB, 0.25280E+01_JPRB, 0.23602E+01_JPRB, 0.22301E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.20504E+01_JPRB, 0.18963E+01_JPRB, 0.17737E+01_JPRB, 0.16579E+01_JPRB, 0.15499E+01_JPRB, 0.14387E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.12101E+01_JPRB, 0.12216E+01_JPRB, 0.12461E+01_JPRB, 0.12718E+01_JPRB, 0.13089E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.14772E+01_JPRB, 0.14830E+01_JPRB, 0.15976E+01_JPRB, 0.19834E+01_JPRB, 0.24930E+01_JPRB, 0.30696E+01_JPRB ,&
&  0.40978E+01_JPRB, 0.34550E+01_JPRB, 0.30868E+01_JPRB, 0.35505E+01_JPRB, 0.47232E+01_JPRB, 0.54925E+01_JPRB, 0.53323E+01_JPRB ,&
&  0.51536E+01_JPRB, 0.43897E+01_JPRB, 0.34683E+01_JPRB, 0.27782E+01_JPRB, 0.22292E+01_JPRB, 0.18377E+01_JPRB, 0.15500E+01_JPRB ,&
&  0.14092E+01_JPRB, 0.13556E+01_JPRB, 0.13613E+01_JPRB, 0.13904E+01_JPRB, 0.14754E+01_JPRB, 0.15465E+01_JPRB, 0.18614E+01_JPRB ,&
&  0.22216E+01_JPRB, 0.26954E+01_JPRB, 0.33165E+01_JPRB, 0.38711E+01_JPRB, 0.45154E+01_JPRB, 0.66519E+01_JPRB, 0.76607E+01_JPRB ,&
&  0.62822E+01_JPRB, 0.60201E+01_JPRB, 0.64917E+01_JPRB, 0.67550E+01_JPRB, 0.69346E+01_JPRB, 0.63861E+01_JPRB, 0.62295E+01_JPRB ,&
&  0.49515E+01_JPRB, 0.44180E+01_JPRB, 0.45141E+01_JPRB, 0.54424E+01_JPRB, 0.82895E+01_JPRB, 0.10883E+02_JPRB, 0.10062E+02_JPRB ,&
&  0.84356E+01_JPRB, 0.72416E+01_JPRB, 0.61986E+01_JPRB, 0.48149E+01_JPRB, 0.43426E+01_JPRB, 0.37835E+01_JPRB, 0.44346E+01_JPRB ,&
&  0.42993E+01_JPRB, 0.39923E+01_JPRB, 0.33574E+01_JPRB, 0.26411E+01_JPRB, 0.22386E+01_JPRB, 0.27314E+01_JPRB, 0.30412E+01_JPRB ,&
&  0.36099E+01_JPRB, 0.43509E+01_JPRB, 0.75804E+01_JPRB, 0.88168E+01_JPRB, 0.92163E+01_JPRB, 0.78132E+01_JPRB, 0.88429E+01_JPRB ,&
&  0.79587E+01_JPRB, 0.76747E+01_JPRB, 0.64488E+01_JPRB, 0.49241E+01_JPRB, 0.39338E+01_JPRB, 0.33821E+01_JPRB, 0.30989E+01_JPRB ,&
&  0.29873E+01_JPRB, 0.28718E+01_JPRB, 0.26733E+01_JPRB, 0.24546E+01_JPRB, 0.22625E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.17493E+01_JPRB, 0.16038E+01_JPRB, 0.15080E+01_JPRB, 0.14301E+01_JPRB, 0.13440E+01_JPRB, 0.12620E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.11489E+01_JPRB, 0.11486E+01_JPRB, 0.11792E+01_JPRB, 0.12366E+01_JPRB, 0.12961E+01_JPRB, 0.13593E+01_JPRB, 0.14100E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.16186E+01_JPRB, 0.16794E+01_JPRB, 0.18269E+01_JPRB, 0.21899E+01_JPRB, 0.24682E+01_JPRB, 0.30432E+01_JPRB ,&
&  0.39041E+01_JPRB, 0.30636E+01_JPRB, 0.28099E+01_JPRB, 0.33603E+01_JPRB, 0.51132E+01_JPRB, 0.55839E+01_JPRB, 0.51864E+01_JPRB ,&
&  0.51388E+01_JPRB, 0.40918E+01_JPRB, 0.28796E+01_JPRB, 0.22235E+01_JPRB, 0.18075E+01_JPRB, 0.15040E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.13584E+01_JPRB, 0.14034E+01_JPRB, 0.14599E+01_JPRB, 0.15327E+01_JPRB, 0.17313E+01_JPRB, 0.21127E+01_JPRB ,&
&  0.24997E+01_JPRB, 0.28979E+01_JPRB, 0.32062E+01_JPRB, 0.35063E+01_JPRB, 0.42719E+01_JPRB, 0.64674E+01_JPRB, 0.91009E+01_JPRB ,&
&  0.76999E+01_JPRB, 0.73950E+01_JPRB, 0.72950E+01_JPRB, 0.75213E+01_JPRB, 0.66851E+01_JPRB, 0.61556E+01_JPRB, 0.58249E+01_JPRB ,&
&  0.46350E+01_JPRB, 0.45092E+01_JPRB, 0.55838E+01_JPRB, 0.73446E+01_JPRB, 0.86378E+01_JPRB, 0.10282E+02_JPRB, 0.12057E+02_JPRB ,&
&  0.90750E+01_JPRB, 0.76180E+01_JPRB, 0.66998E+01_JPRB, 0.59200E+01_JPRB, 0.35483E+01_JPRB, 0.26566E+01_JPRB, 0.37421E+01_JPRB ,&
&  0.35109E+01_JPRB, 0.20967E+01_JPRB, 0.16957E+01_JPRB, 0.21528E+01_JPRB, 0.16185E+01_JPRB, 0.20255E+01_JPRB, 0.34527E+01_JPRB ,&
&  0.44753E+01_JPRB, 0.57444E+01_JPRB, 0.10726E+02_JPRB, 0.95654E+01_JPRB, 0.96713E+01_JPRB, 0.10912E+02_JPRB, 0.10483E+02_JPRB ,&
&  0.95133E+01_JPRB, 0.59527E+01_JPRB, 0.41919E+01_JPRB, 0.31888E+01_JPRB, 0.26637E+01_JPRB, 0.25865E+01_JPRB, 0.25280E+01_JPRB ,&
&  0.24783E+01_JPRB, 0.23563E+01_JPRB, 0.22343E+01_JPRB, 0.20561E+01_JPRB, 0.19148E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.13670E+01_JPRB, 0.12705E+01_JPRB, 0.12421E+01_JPRB, 0.12147E+01_JPRB, 0.11541E+01_JPRB, 0.11148E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.11350E+01_JPRB, 0.11977E+01_JPRB, 0.12844E+01_JPRB, 0.13904E+01_JPRB, 0.14864E+01_JPRB, 0.15283E+01_JPRB ,&
&  0.15819E+01_JPRB, 0.17410E+01_JPRB, 0.20182E+01_JPRB, 0.23631E+01_JPRB, 0.28377E+01_JPRB, 0.26527E+01_JPRB, 0.37126E+01_JPRB ,&
&  0.37318E+01_JPRB, 0.28059E+01_JPRB, 0.28131E+01_JPRB, 0.34884E+01_JPRB, 0.43686E+01_JPRB, 0.43807E+01_JPRB, 0.40288E+01_JPRB ,&
&  0.35217E+01_JPRB, 0.27209E+01_JPRB, 0.21417E+01_JPRB, 0.17791E+01_JPRB, 0.15474E+01_JPRB, 0.14252E+01_JPRB, 0.14435E+01_JPRB ,&
&  0.14502E+01_JPRB, 0.15168E+01_JPRB, 0.15962E+01_JPRB, 0.15831E+01_JPRB, 0.17346E+01_JPRB, 0.19628E+01_JPRB, 0.23108E+01_JPRB ,&
&  0.26290E+01_JPRB, 0.29056E+01_JPRB, 0.31388E+01_JPRB, 0.35605E+01_JPRB, 0.52199E+01_JPRB, 0.74749E+01_JPRB, 0.80116E+01_JPRB ,&
&  0.73894E+01_JPRB, 0.70461E+01_JPRB, 0.80320E+01_JPRB, 0.75260E+01_JPRB, 0.66576E+01_JPRB, 0.66484E+01_JPRB, 0.60881E+01_JPRB ,&
&  0.60376E+01_JPRB, 0.65072E+01_JPRB, 0.70556E+01_JPRB, 0.85222E+01_JPRB, 0.10485E+02_JPRB, 0.10699E+02_JPRB, 0.10928E+02_JPRB ,&
&  0.97933E+01_JPRB, 0.76909E+01_JPRB, 0.54598E+01_JPRB, 0.40153E+01_JPRB, 0.39174E+01_JPRB, 0.34228E+01_JPRB, 0.23469E+01_JPRB ,&
&  0.19521E+01_JPRB, 0.16956E+01_JPRB, 0.13039E+01_JPRB, 0.14340E+01_JPRB, 0.13414E+01_JPRB, 0.18283E+01_JPRB, 0.34168E+01_JPRB ,&
&  0.63058E+01_JPRB, 0.83945E+01_JPRB, 0.10827E+02_JPRB, 0.96220E+01_JPRB, 0.95395E+01_JPRB, 0.11464E+02_JPRB, 0.11175E+02_JPRB ,&
&  0.52187E+01_JPRB, 0.32477E+01_JPRB, 0.25978E+01_JPRB, 0.23666E+01_JPRB, 0.22674E+01_JPRB, 0.21943E+01_JPRB, 0.21160E+01_JPRB ,&
&  0.19867E+01_JPRB, 0.18706E+01_JPRB, 0.18132E+01_JPRB, 0.17185E+01_JPRB, 0.15395E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.11817E+01_JPRB, 0.11598E+01_JPRB, 0.11638E+01_JPRB, 0.11378E+01_JPRB, 0.10769E+01_JPRB, 0.10856E+01_JPRB, 0.11039E+01_JPRB ,&
&  0.11483E+01_JPRB, 0.12149E+01_JPRB, 0.13093E+01_JPRB, 0.14464E+01_JPRB, 0.16353E+01_JPRB, 0.17289E+01_JPRB, 0.17457E+01_JPRB ,&
&  0.18026E+01_JPRB, 0.20481E+01_JPRB, 0.25066E+01_JPRB, 0.31723E+01_JPRB, 0.37317E+01_JPRB, 0.31531E+01_JPRB, 0.44180E+01_JPRB ,&
&  0.35325E+01_JPRB, 0.27485E+01_JPRB, 0.28021E+01_JPRB, 0.32238E+01_JPRB, 0.35414E+01_JPRB, 0.33003E+01_JPRB, 0.27840E+01_JPRB ,&
&  0.22902E+01_JPRB, 0.20269E+01_JPRB, 0.17918E+01_JPRB, 0.16251E+01_JPRB, 0.14877E+01_JPRB, 0.14861E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.15844E+01_JPRB, 0.16848E+01_JPRB, 0.17133E+01_JPRB, 0.18543E+01_JPRB, 0.21262E+01_JPRB, 0.23812E+01_JPRB, 0.26391E+01_JPRB ,&
&  0.28181E+01_JPRB, 0.30607E+01_JPRB, 0.34886E+01_JPRB, 0.47005E+01_JPRB, 0.58660E+01_JPRB, 0.62865E+01_JPRB, 0.72604E+01_JPRB ,&
&  0.79856E+01_JPRB, 0.82815E+01_JPRB, 0.75106E+01_JPRB, 0.81224E+01_JPRB, 0.72964E+01_JPRB, 0.66698E+01_JPRB, 0.65617E+01_JPRB ,&
&  0.68906E+01_JPRB, 0.83870E+01_JPRB, 0.88348E+01_JPRB, 0.95163E+01_JPRB, 0.12979E+02_JPRB, 0.13780E+02_JPRB, 0.11117E+02_JPRB ,&
&  0.10056E+02_JPRB, 0.74576E+01_JPRB, 0.54873E+01_JPRB, 0.42531E+01_JPRB, 0.54333E+01_JPRB, 0.63620E+01_JPRB, 0.32816E+01_JPRB ,&
&  0.22324E+01_JPRB, 0.17637E+01_JPRB, 0.13477E+01_JPRB, 0.13248E+01_JPRB, 0.13884E+01_JPRB, 0.18988E+01_JPRB, 0.47888E+01_JPRB ,&
&  0.59294E+01_JPRB, 0.98227E+01_JPRB, 0.11370E+02_JPRB, 0.10654E+02_JPRB, 0.99230E+01_JPRB, 0.88291E+01_JPRB, 0.51542E+01_JPRB ,&
&  0.29659E+01_JPRB, 0.20381E+01_JPRB, 0.19578E+01_JPRB, 0.21263E+01_JPRB, 0.21654E+01_JPRB, 0.20344E+01_JPRB, 0.18533E+01_JPRB ,&
&  0.17335E+01_JPRB, 0.16878E+01_JPRB, 0.15951E+01_JPRB, 0.15015E+01_JPRB, 0.13372E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.11811E+01_JPRB, 0.12092E+01_JPRB, 0.12419E+01_JPRB, 0.11716E+01_JPRB, 0.11562E+01_JPRB, 0.11787E+01_JPRB, 0.11764E+01_JPRB ,&
&  0.12545E+01_JPRB, 0.13727E+01_JPRB, 0.15082E+01_JPRB, 0.17497E+01_JPRB, 0.19933E+01_JPRB, 0.20382E+01_JPRB, 0.20442E+01_JPRB ,&
&  0.21686E+01_JPRB, 0.25855E+01_JPRB, 0.32753E+01_JPRB, 0.41621E+01_JPRB, 0.43436E+01_JPRB, 0.40764E+01_JPRB, 0.47481E+01_JPRB ,&
&  0.35960E+01_JPRB, 0.28744E+01_JPRB, 0.26920E+01_JPRB, 0.29601E+01_JPRB, 0.29997E+01_JPRB, 0.28237E+01_JPRB, 0.22898E+01_JPRB ,&
&  0.19533E+01_JPRB, 0.19159E+01_JPRB, 0.16761E+01_JPRB, 0.17225E+01_JPRB, 0.16314E+01_JPRB, 0.16839E+01_JPRB, 0.17181E+01_JPRB ,&
&  0.18047E+01_JPRB, 0.18920E+01_JPRB, 0.19961E+01_JPRB, 0.22215E+01_JPRB, 0.25293E+01_JPRB, 0.28054E+01_JPRB, 0.30557E+01_JPRB ,&
&  0.32207E+01_JPRB, 0.36478E+01_JPRB, 0.44472E+01_JPRB, 0.56776E+01_JPRB, 0.53952E+01_JPRB, 0.69697E+01_JPRB, 0.73149E+01_JPRB ,&
&  0.77629E+01_JPRB, 0.78051E+01_JPRB, 0.70889E+01_JPRB, 0.73647E+01_JPRB, 0.74868E+01_JPRB, 0.71682E+01_JPRB, 0.71835E+01_JPRB ,&
&  0.87620E+01_JPRB, 0.96332E+01_JPRB, 0.10631E+02_JPRB, 0.11362E+02_JPRB, 0.14413E+02_JPRB, 0.19061E+02_JPRB, 0.11540E+02_JPRB ,&
&  0.83154E+01_JPRB, 0.75855E+01_JPRB, 0.57369E+01_JPRB, 0.48632E+01_JPRB, 0.62822E+01_JPRB, 0.78033E+01_JPRB, 0.77328E+01_JPRB ,&
&  0.32360E+01_JPRB, 0.19078E+01_JPRB, 0.17627E+01_JPRB, 0.17608E+01_JPRB, 0.14881E+01_JPRB, 0.21045E+01_JPRB, 0.80553E+01_JPRB ,&
&  0.63843E+01_JPRB, 0.10083E+02_JPRB, 0.10716E+02_JPRB, 0.99675E+01_JPRB, 0.97536E+01_JPRB, 0.65774E+01_JPRB, 0.35927E+01_JPRB ,&
&  0.17624E+01_JPRB, 0.17064E+01_JPRB, 0.19134E+01_JPRB, 0.18483E+01_JPRB, 0.18109E+01_JPRB, 0.16967E+01_JPRB, 0.15033E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.14529E+01_JPRB, 0.13519E+01_JPRB, 0.12618E+01_JPRB, 0.12046E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.12465E+01_JPRB, 0.12900E+01_JPRB, 0.12945E+01_JPRB, 0.12958E+01_JPRB, 0.13159E+01_JPRB, 0.13368E+01_JPRB, 0.13417E+01_JPRB ,&
&  0.14461E+01_JPRB, 0.16343E+01_JPRB, 0.19073E+01_JPRB, 0.22673E+01_JPRB, 0.24905E+01_JPRB, 0.24926E+01_JPRB, 0.25411E+01_JPRB ,&
&  0.28429E+01_JPRB, 0.34247E+01_JPRB, 0.41598E+01_JPRB, 0.47048E+01_JPRB, 0.45647E+01_JPRB, 0.41571E+01_JPRB, 0.45159E+01_JPRB ,&
&  0.39499E+01_JPRB, 0.30922E+01_JPRB, 0.25208E+01_JPRB, 0.27170E+01_JPRB, 0.29236E+01_JPRB, 0.29655E+01_JPRB, 0.24409E+01_JPRB ,&
&  0.22314E+01_JPRB, 0.21723E+01_JPRB, 0.20378E+01_JPRB, 0.20446E+01_JPRB, 0.20473E+01_JPRB, 0.21157E+01_JPRB, 0.21631E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.22465E+01_JPRB, 0.24420E+01_JPRB, 0.27127E+01_JPRB, 0.30286E+01_JPRB, 0.33159E+01_JPRB, 0.36255E+01_JPRB ,&
&  0.40112E+01_JPRB, 0.46403E+01_JPRB, 0.54314E+01_JPRB, 0.56661E+01_JPRB, 0.60289E+01_JPRB, 0.66052E+01_JPRB, 0.71279E+01_JPRB ,&
&  0.74812E+01_JPRB, 0.76758E+01_JPRB, 0.74864E+01_JPRB, 0.72806E+01_JPRB, 0.74793E+01_JPRB, 0.78395E+01_JPRB, 0.86337E+01_JPRB ,&
&  0.10461E+02_JPRB, 0.11591E+02_JPRB, 0.11708E+02_JPRB, 0.12259E+02_JPRB, 0.13384E+02_JPRB, 0.14256E+02_JPRB, 0.11931E+02_JPRB ,&
&  0.89918E+01_JPRB, 0.66601E+01_JPRB, 0.56259E+01_JPRB, 0.62029E+01_JPRB, 0.79189E+01_JPRB, 0.84839E+01_JPRB, 0.11549E+02_JPRB ,&
&  0.11674E+02_JPRB, 0.46118E+01_JPRB, 0.31249E+01_JPRB, 0.42232E+01_JPRB, 0.21798E+01_JPRB, 0.20541E+01_JPRB, 0.40551E+01_JPRB ,&
&  0.66561E+01_JPRB, 0.89790E+01_JPRB, 0.89352E+01_JPRB, 0.81706E+01_JPRB, 0.80905E+01_JPRB, 0.45404E+01_JPRB, 0.21906E+01_JPRB ,&
&  0.15651E+01_JPRB, 0.16479E+01_JPRB, 0.16501E+01_JPRB, 0.14782E+01_JPRB, 0.14177E+01_JPRB, 0.14668E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.13138E+01_JPRB, 0.13143E+01_JPRB, 0.13227E+01_JPRB, 0.12857E+01_JPRB, 0.12345E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.13959E+01_JPRB, 0.14375E+01_JPRB, 0.15143E+01_JPRB, 0.15705E+01_JPRB, 0.15887E+01_JPRB, 0.16392E+01_JPRB, 0.17220E+01_JPRB ,&
&  0.19262E+01_JPRB, 0.22218E+01_JPRB, 0.25660E+01_JPRB, 0.29054E+01_JPRB, 0.30218E+01_JPRB, 0.30574E+01_JPRB, 0.32628E+01_JPRB ,&
&  0.36716E+01_JPRB, 0.42390E+01_JPRB, 0.47976E+01_JPRB, 0.52289E+01_JPRB, 0.52770E+01_JPRB, 0.54477E+01_JPRB, 0.45089E+01_JPRB ,&
&  0.37831E+01_JPRB, 0.35439E+01_JPRB, 0.27269E+01_JPRB, 0.29548E+01_JPRB, 0.31401E+01_JPRB, 0.31982E+01_JPRB, 0.28389E+01_JPRB ,&
&  0.26457E+01_JPRB, 0.25805E+01_JPRB, 0.24815E+01_JPRB, 0.24579E+01_JPRB, 0.25279E+01_JPRB, 0.26081E+01_JPRB, 0.26817E+01_JPRB ,&
&  0.27824E+01_JPRB, 0.28769E+01_JPRB, 0.30673E+01_JPRB, 0.33253E+01_JPRB, 0.36064E+01_JPRB, 0.39280E+01_JPRB, 0.43556E+01_JPRB ,&
&  0.48119E+01_JPRB, 0.52112E+01_JPRB, 0.56058E+01_JPRB, 0.55471E+01_JPRB, 0.58944E+01_JPRB, 0.63800E+01_JPRB, 0.68056E+01_JPRB ,&
&  0.67298E+01_JPRB, 0.65684E+01_JPRB, 0.71628E+01_JPRB, 0.79370E+01_JPRB, 0.84689E+01_JPRB, 0.87971E+01_JPRB, 0.95974E+01_JPRB ,&
&  0.11310E+02_JPRB, 0.12367E+02_JPRB, 0.12609E+02_JPRB, 0.11090E+02_JPRB, 0.10287E+02_JPRB, 0.98380E+01_JPRB, 0.97821E+01_JPRB ,&
&  0.85619E+01_JPRB, 0.71535E+01_JPRB, 0.67673E+01_JPRB, 0.76020E+01_JPRB, 0.86324E+01_JPRB, 0.89752E+01_JPRB, 0.10548E+02_JPRB ,&
&  0.12182E+02_JPRB, 0.10860E+02_JPRB, 0.85640E+01_JPRB, 0.53657E+01_JPRB, 0.34176E+01_JPRB, 0.29970E+01_JPRB, 0.39692E+01_JPRB ,&
&  0.62215E+01_JPRB, 0.66890E+01_JPRB, 0.64380E+01_JPRB, 0.79141E+01_JPRB, 0.55963E+01_JPRB, 0.29579E+01_JPRB, 0.16643E+01_JPRB ,&
&  0.13719E+01_JPRB, 0.14057E+01_JPRB, 0.14032E+01_JPRB, 0.13235E+01_JPRB, 0.13432E+01_JPRB, 0.13785E+01_JPRB, 0.13177E+01_JPRB ,&
&  0.13542E+01_JPRB, 0.14083E+01_JPRB, 0.13653E+01_JPRB, 0.13295E+01_JPRB, 0.13584E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.16833E+01_JPRB, 0.17756E+01_JPRB, 0.18604E+01_JPRB, 0.19169E+01_JPRB, 0.20220E+01_JPRB, 0.20976E+01_JPRB, 0.22679E+01_JPRB ,&
&  0.25548E+01_JPRB, 0.28634E+01_JPRB, 0.31574E+01_JPRB, 0.33372E+01_JPRB, 0.34192E+01_JPRB, 0.35598E+01_JPRB, 0.38497E+01_JPRB ,&
&  0.41842E+01_JPRB, 0.45881E+01_JPRB, 0.49521E+01_JPRB, 0.53467E+01_JPRB, 0.56339E+01_JPRB, 0.63363E+01_JPRB, 0.56904E+01_JPRB ,&
&  0.36126E+01_JPRB, 0.45899E+01_JPRB, 0.30888E+01_JPRB, 0.33951E+01_JPRB, 0.35138E+01_JPRB, 0.36720E+01_JPRB, 0.32247E+01_JPRB ,&
&  0.29326E+01_JPRB, 0.29135E+01_JPRB, 0.28865E+01_JPRB, 0.28175E+01_JPRB, 0.28023E+01_JPRB, 0.29655E+01_JPRB, 0.31396E+01_JPRB ,&
&  0.32906E+01_JPRB, 0.34054E+01_JPRB, 0.35726E+01_JPRB, 0.37956E+01_JPRB, 0.40525E+01_JPRB, 0.43619E+01_JPRB, 0.47208E+01_JPRB ,&
&  0.49998E+01_JPRB, 0.52691E+01_JPRB, 0.53388E+01_JPRB, 0.53916E+01_JPRB, 0.55485E+01_JPRB, 0.58664E+01_JPRB, 0.62426E+01_JPRB ,&
&  0.64062E+01_JPRB, 0.65340E+01_JPRB, 0.72346E+01_JPRB, 0.77119E+01_JPRB, 0.79938E+01_JPRB, 0.87222E+01_JPRB, 0.91147E+01_JPRB ,&
&  0.10549E+02_JPRB, 0.10507E+02_JPRB, 0.10673E+02_JPRB, 0.89996E+01_JPRB, 0.74947E+01_JPRB, 0.69414E+01_JPRB, 0.68439E+01_JPRB ,&
&  0.63470E+01_JPRB, 0.66090E+01_JPRB, 0.69199E+01_JPRB, 0.75547E+01_JPRB, 0.84243E+01_JPRB, 0.92312E+01_JPRB, 0.91144E+01_JPRB ,&
&  0.89910E+01_JPRB, 0.64600E+01_JPRB, 0.48146E+01_JPRB, 0.41673E+01_JPRB, 0.31974E+01_JPRB, 0.29911E+01_JPRB, 0.41236E+01_JPRB ,&
&  0.55334E+01_JPRB, 0.56646E+01_JPRB, 0.59225E+01_JPRB, 0.56155E+01_JPRB, 0.32199E+01_JPRB, 0.18910E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.12306E+01_JPRB, 0.12773E+01_JPRB, 0.12793E+01_JPRB, 0.12731E+01_JPRB, 0.13298E+01_JPRB, 0.12769E+01_JPRB, 0.13455E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.14339E+01_JPRB, 0.14788E+01_JPRB, 0.14973E+01_JPRB, 0.15706E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.20107E+01_JPRB, 0.22132E+01_JPRB, 0.24251E+01_JPRB, 0.25923E+01_JPRB, 0.26238E+01_JPRB, 0.26722E+01_JPRB, 0.27936E+01_JPRB ,&
&  0.29477E+01_JPRB, 0.31398E+01_JPRB, 0.33398E+01_JPRB, 0.35141E+01_JPRB, 0.36632E+01_JPRB, 0.39026E+01_JPRB, 0.42470E+01_JPRB ,&
&  0.46384E+01_JPRB, 0.50526E+01_JPRB, 0.54658E+01_JPRB, 0.59688E+01_JPRB, 0.66063E+01_JPRB, 0.76075E+01_JPRB, 0.74824E+01_JPRB ,&
&  0.52124E+01_JPRB, 0.38728E+01_JPRB, 0.37081E+01_JPRB, 0.35164E+01_JPRB, 0.33925E+01_JPRB, 0.36781E+01_JPRB, 0.36544E+01_JPRB ,&
&  0.33164E+01_JPRB, 0.31579E+01_JPRB, 0.31313E+01_JPRB, 0.30577E+01_JPRB, 0.29572E+01_JPRB, 0.31551E+01_JPRB, 0.33425E+01_JPRB ,&
&  0.35228E+01_JPRB, 0.36443E+01_JPRB, 0.37800E+01_JPRB, 0.39545E+01_JPRB, 0.41560E+01_JPRB, 0.43745E+01_JPRB, 0.46260E+01_JPRB ,&
&  0.47703E+01_JPRB, 0.49383E+01_JPRB, 0.50243E+01_JPRB, 0.50765E+01_JPRB, 0.52003E+01_JPRB, 0.53092E+01_JPRB, 0.54615E+01_JPRB ,&
&  0.57836E+01_JPRB, 0.62756E+01_JPRB, 0.67759E+01_JPRB, 0.70829E+01_JPRB, 0.71316E+01_JPRB, 0.78339E+01_JPRB, 0.81508E+01_JPRB ,&
&  0.84893E+01_JPRB, 0.80230E+01_JPRB, 0.81494E+01_JPRB, 0.70496E+01_JPRB, 0.62684E+01_JPRB, 0.56120E+01_JPRB, 0.54349E+01_JPRB ,&
&  0.52628E+01_JPRB, 0.53021E+01_JPRB, 0.52128E+01_JPRB, 0.56222E+01_JPRB, 0.67178E+01_JPRB, 0.73993E+01_JPRB, 0.76677E+01_JPRB ,&
&  0.64632E+01_JPRB, 0.48840E+01_JPRB, 0.34718E+01_JPRB, 0.35736E+01_JPRB, 0.29930E+01_JPRB, 0.33853E+01_JPRB, 0.40319E+01_JPRB ,&
&  0.44701E+01_JPRB, 0.37792E+01_JPRB, 0.38913E+01_JPRB, 0.36037E+01_JPRB, 0.21146E+01_JPRB, 0.13856E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.10646E+01_JPRB, 0.10931E+01_JPRB, 0.10944E+01_JPRB, 0.11699E+01_JPRB, 0.12344E+01_JPRB, 0.13380E+01_JPRB ,&
&  0.13826E+01_JPRB, 0.14636E+01_JPRB, 0.15598E+01_JPRB, 0.16586E+01_JPRB, 0.18141E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.20390E+01_JPRB, 0.22070E+01_JPRB, 0.23911E+01_JPRB, 0.24538E+01_JPRB, 0.25215E+01_JPRB, 0.26025E+01_JPRB, 0.27132E+01_JPRB ,&
&  0.29118E+01_JPRB, 0.30979E+01_JPRB, 0.32944E+01_JPRB, 0.34665E+01_JPRB, 0.36194E+01_JPRB, 0.38583E+01_JPRB, 0.41482E+01_JPRB ,&
&  0.44060E+01_JPRB, 0.46538E+01_JPRB, 0.49056E+01_JPRB, 0.52411E+01_JPRB, 0.56065E+01_JPRB, 0.61128E+01_JPRB, 0.63514E+01_JPRB ,&
&  0.49459E+01_JPRB, 0.29225E+01_JPRB, 0.31221E+01_JPRB, 0.31568E+01_JPRB, 0.32880E+01_JPRB, 0.36278E+01_JPRB, 0.36950E+01_JPRB ,&
&  0.32643E+01_JPRB, 0.31468E+01_JPRB, 0.31963E+01_JPRB, 0.30762E+01_JPRB, 0.28205E+01_JPRB, 0.30266E+01_JPRB, 0.32333E+01_JPRB ,&
&  0.34322E+01_JPRB, 0.36027E+01_JPRB, 0.37392E+01_JPRB, 0.38891E+01_JPRB, 0.40150E+01_JPRB, 0.41322E+01_JPRB, 0.42412E+01_JPRB ,&
&  0.43468E+01_JPRB, 0.43900E+01_JPRB, 0.44781E+01_JPRB, 0.46723E+01_JPRB, 0.47279E+01_JPRB, 0.47379E+01_JPRB, 0.47418E+01_JPRB ,&
&  0.49420E+01_JPRB, 0.52530E+01_JPRB, 0.57637E+01_JPRB, 0.62123E+01_JPRB, 0.65598E+01_JPRB, 0.65002E+01_JPRB, 0.65907E+01_JPRB ,&
&  0.63042E+01_JPRB, 0.57734E+01_JPRB, 0.60068E+01_JPRB, 0.63353E+01_JPRB, 0.48186E+01_JPRB, 0.45257E+01_JPRB, 0.41380E+01_JPRB ,&
&  0.41865E+01_JPRB, 0.37709E+01_JPRB, 0.35017E+01_JPRB, 0.37674E+01_JPRB, 0.49650E+01_JPRB, 0.53293E+01_JPRB, 0.55552E+01_JPRB ,&
&  0.47232E+01_JPRB, 0.30835E+01_JPRB, 0.27889E+01_JPRB, 0.33341E+01_JPRB, 0.34237E+01_JPRB, 0.48352E+01_JPRB, 0.41057E+01_JPRB ,&
&  0.28976E+01_JPRB, 0.23911E+01_JPRB, 0.27365E+01_JPRB, 0.27920E+01_JPRB, 0.17161E+01_JPRB, 0.11833E+01_JPRB, 0.99540E+00_JPRB ,&
&  0.92554E+00_JPRB, 0.91116E+00_JPRB, 0.95969E+00_JPRB, 0.94958E+00_JPRB, 0.10186E+01_JPRB, 0.10815E+01_JPRB, 0.12186E+01_JPRB ,&
&  0.13368E+01_JPRB, 0.14497E+01_JPRB, 0.15501E+01_JPRB, 0.16847E+01_JPRB, 0.18411E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.14416E+01_JPRB, 0.15465E+01_JPRB, 0.15960E+01_JPRB, 0.18020E+01_JPRB, 0.20401E+01_JPRB, 0.22339E+01_JPRB, 0.24107E+01_JPRB ,&
&  0.26347E+01_JPRB, 0.29114E+01_JPRB, 0.31323E+01_JPRB, 0.32934E+01_JPRB, 0.34612E+01_JPRB, 0.36758E+01_JPRB, 0.37407E+01_JPRB ,&
&  0.37411E+01_JPRB, 0.37388E+01_JPRB, 0.37575E+01_JPRB, 0.37552E+01_JPRB, 0.37785E+01_JPRB, 0.37907E+01_JPRB, 0.40789E+01_JPRB ,&
&  0.39767E+01_JPRB, 0.35872E+01_JPRB, 0.31928E+01_JPRB, 0.23414E+01_JPRB, 0.19492E+01_JPRB, 0.24527E+01_JPRB, 0.24380E+01_JPRB ,&
&  0.26786E+01_JPRB, 0.26670E+01_JPRB, 0.25353E+01_JPRB, 0.26037E+01_JPRB, 0.26593E+01_JPRB, 0.27886E+01_JPRB, 0.29785E+01_JPRB ,&
&  0.31224E+01_JPRB, 0.32775E+01_JPRB, 0.34197E+01_JPRB, 0.35444E+01_JPRB, 0.36371E+01_JPRB, 0.37016E+01_JPRB, 0.37159E+01_JPRB ,&
&  0.37352E+01_JPRB, 0.37856E+01_JPRB, 0.38825E+01_JPRB, 0.41425E+01_JPRB, 0.42229E+01_JPRB, 0.43042E+01_JPRB, 0.42295E+01_JPRB ,&
&  0.42849E+01_JPRB, 0.44189E+01_JPRB, 0.47596E+01_JPRB, 0.50306E+01_JPRB, 0.50830E+01_JPRB, 0.47988E+01_JPRB, 0.46759E+01_JPRB ,&
&  0.44871E+01_JPRB, 0.42246E+01_JPRB, 0.43508E+01_JPRB, 0.48586E+01_JPRB, 0.35228E+01_JPRB, 0.36742E+01_JPRB, 0.36967E+01_JPRB ,&
&  0.31620E+01_JPRB, 0.27866E+01_JPRB, 0.25510E+01_JPRB, 0.24132E+01_JPRB, 0.31214E+01_JPRB, 0.38103E+01_JPRB, 0.45072E+01_JPRB ,&
&  0.36096E+01_JPRB, 0.23825E+01_JPRB, 0.23190E+01_JPRB, 0.30280E+01_JPRB, 0.35845E+01_JPRB, 0.47844E+01_JPRB, 0.33972E+01_JPRB ,&
&  0.20678E+01_JPRB, 0.21334E+01_JPRB, 0.25229E+01_JPRB, 0.29493E+01_JPRB, 0.19457E+01_JPRB, 0.11082E+01_JPRB, 0.90035E+00_JPRB ,&
&  0.85560E+00_JPRB, 0.85409E+00_JPRB, 0.86337E+00_JPRB, 0.86092E+00_JPRB, 0.90544E+00_JPRB, 0.95260E+00_JPRB, 0.97307E+00_JPRB ,&
&  0.10018E+01_JPRB, 0.10247E+01_JPRB, 0.10831E+01_JPRB, 0.11797E+01_JPRB, 0.13242E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.92722E+00_JPRB, 0.94332E+00_JPRB, 0.93641E+00_JPRB, 0.98395E+00_JPRB, 0.10740E+01_JPRB, 0.12538E+01_JPRB, 0.14362E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.18957E+01_JPRB, 0.22561E+01_JPRB, 0.26018E+01_JPRB, 0.27572E+01_JPRB, 0.27341E+01_JPRB, 0.24744E+01_JPRB ,&
&  0.21202E+01_JPRB, 0.17369E+01_JPRB, 0.14925E+01_JPRB, 0.13815E+01_JPRB, 0.14328E+01_JPRB, 0.14500E+01_JPRB, 0.15322E+01_JPRB ,&
&  0.17010E+01_JPRB, 0.19887E+01_JPRB, 0.22166E+01_JPRB, 0.27697E+01_JPRB, 0.28879E+01_JPRB, 0.23154E+01_JPRB, 0.23899E+01_JPRB ,&
&  0.31536E+01_JPRB, 0.31181E+01_JPRB, 0.20575E+01_JPRB, 0.19424E+01_JPRB, 0.21028E+01_JPRB, 0.22335E+01_JPRB, 0.24443E+01_JPRB ,&
&  0.25489E+01_JPRB, 0.26582E+01_JPRB, 0.27501E+01_JPRB, 0.27993E+01_JPRB, 0.28169E+01_JPRB, 0.27686E+01_JPRB, 0.27177E+01_JPRB ,&
&  0.26829E+01_JPRB, 0.26554E+01_JPRB, 0.28170E+01_JPRB, 0.30442E+01_JPRB, 0.32931E+01_JPRB, 0.34775E+01_JPRB, 0.35206E+01_JPRB ,&
&  0.35519E+01_JPRB, 0.36169E+01_JPRB, 0.39661E+01_JPRB, 0.40438E+01_JPRB, 0.38025E+01_JPRB, 0.35801E+01_JPRB, 0.32949E+01_JPRB ,&
&  0.29895E+01_JPRB, 0.25527E+01_JPRB, 0.20732E+01_JPRB, 0.32133E+01_JPRB, 0.36818E+01_JPRB, 0.31931E+01_JPRB, 0.26659E+01_JPRB ,&
&  0.23724E+01_JPRB, 0.21505E+01_JPRB, 0.19410E+01_JPRB, 0.18957E+01_JPRB, 0.22349E+01_JPRB, 0.34723E+01_JPRB, 0.36130E+01_JPRB ,&
&  0.30039E+01_JPRB, 0.18854E+01_JPRB, 0.16521E+01_JPRB, 0.20074E+01_JPRB, 0.24536E+01_JPRB, 0.31500E+01_JPRB, 0.21672E+01_JPRB ,&
&  0.16171E+01_JPRB, 0.19790E+01_JPRB, 0.22844E+01_JPRB, 0.27810E+01_JPRB, 0.27864E+01_JPRB, 0.89644E+00_JPRB, 0.81697E+00_JPRB ,&
&  0.79250E+00_JPRB, 0.78158E+00_JPRB, 0.79971E+00_JPRB, 0.79091E+00_JPRB, 0.79926E+00_JPRB, 0.78566E+00_JPRB, 0.76797E+00_JPRB ,&
&  0.76974E+00_JPRB, 0.79267E+00_JPRB, 0.82350E+00_JPRB, 0.84649E+00_JPRB, 0.89179E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.83712E+00_JPRB, 0.83963E+00_JPRB, 0.84585E+00_JPRB, 0.85671E+00_JPRB, 0.86671E+00_JPRB, 0.87705E+00_JPRB, 0.91705E+00_JPRB ,&
&  0.98411E+00_JPRB, 0.10829E+01_JPRB, 0.12233E+01_JPRB, 0.13287E+01_JPRB, 0.13099E+01_JPRB, 0.11961E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10500E+01_JPRB, 0.10832E+01_JPRB, 0.11413E+01_JPRB, 0.12127E+01_JPRB, 0.12259E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.13470E+01_JPRB, 0.14730E+01_JPRB, 0.15790E+01_JPRB, 0.23464E+01_JPRB, 0.22875E+01_JPRB, 0.20436E+01_JPRB ,&
&  0.25993E+01_JPRB, 0.26561E+01_JPRB, 0.22047E+01_JPRB, 0.16780E+01_JPRB, 0.15180E+01_JPRB, 0.14840E+01_JPRB, 0.15232E+01_JPRB ,&
&  0.15134E+01_JPRB, 0.16234E+01_JPRB, 0.17165E+01_JPRB, 0.18199E+01_JPRB, 0.18920E+01_JPRB, 0.19132E+01_JPRB, 0.19766E+01_JPRB ,&
&  0.20353E+01_JPRB, 0.20999E+01_JPRB, 0.21835E+01_JPRB, 0.20115E+01_JPRB, 0.22062E+01_JPRB, 0.24256E+01_JPRB, 0.25795E+01_JPRB ,&
&  0.26470E+01_JPRB, 0.26869E+01_JPRB, 0.29074E+01_JPRB, 0.28269E+01_JPRB, 0.26998E+01_JPRB, 0.24661E+01_JPRB, 0.22646E+01_JPRB ,&
&  0.20959E+01_JPRB, 0.15759E+01_JPRB, 0.11035E+01_JPRB, 0.16396E+01_JPRB, 0.20108E+01_JPRB, 0.17525E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.17305E+01_JPRB, 0.16206E+01_JPRB, 0.15152E+01_JPRB, 0.15226E+01_JPRB, 0.15634E+01_JPRB, 0.21495E+01_JPRB, 0.26246E+01_JPRB ,&
&  0.21552E+01_JPRB, 0.14703E+01_JPRB, 0.12504E+01_JPRB, 0.13430E+01_JPRB, 0.15948E+01_JPRB, 0.19493E+01_JPRB, 0.20846E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.16376E+01_JPRB, 0.20787E+01_JPRB, 0.26218E+01_JPRB, 0.14243E+01_JPRB, 0.71793E+00_JPRB, 0.65847E+00_JPRB ,&
&  0.65506E+00_JPRB, 0.66522E+00_JPRB, 0.68878E+00_JPRB, 0.70529E+00_JPRB, 0.71712E+00_JPRB, 0.72546E+00_JPRB, 0.74566E+00_JPRB ,&
&  0.76345E+00_JPRB, 0.78381E+00_JPRB, 0.79615E+00_JPRB, 0.81103E+00_JPRB, 0.82717E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.10658E+01_JPRB, 0.10899E+01_JPRB, 0.11247E+01_JPRB, 0.11605E+01_JPRB, 0.11878E+01_JPRB, 0.12250E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.13957E+01_JPRB, 0.14489E+01_JPRB, 0.14692E+01_JPRB, 0.14821E+01_JPRB, 0.14974E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.15262E+01_JPRB, 0.15467E+01_JPRB, 0.16330E+01_JPRB, 0.16692E+01_JPRB, 0.17116E+01_JPRB, 0.17633E+01_JPRB, 0.17661E+01_JPRB ,&
&  0.17415E+01_JPRB, 0.17259E+01_JPRB, 0.17340E+01_JPRB, 0.17171E+01_JPRB, 0.17965E+01_JPRB, 0.18205E+01_JPRB, 0.17762E+01_JPRB ,&
&  0.19583E+01_JPRB, 0.19825E+01_JPRB, 0.20547E+01_JPRB, 0.15118E+01_JPRB, 0.14273E+01_JPRB, 0.14355E+01_JPRB, 0.13154E+01_JPRB ,&
&  0.12964E+01_JPRB, 0.13596E+01_JPRB, 0.14377E+01_JPRB, 0.15162E+01_JPRB, 0.16295E+01_JPRB, 0.16870E+01_JPRB, 0.17299E+01_JPRB ,&
&  0.17824E+01_JPRB, 0.18446E+01_JPRB, 0.18834E+01_JPRB, 0.18505E+01_JPRB, 0.18570E+01_JPRB, 0.21528E+01_JPRB, 0.22773E+01_JPRB ,&
&  0.23220E+01_JPRB, 0.23718E+01_JPRB, 0.20075E+01_JPRB, 0.18778E+01_JPRB, 0.19641E+01_JPRB, 0.17596E+01_JPRB, 0.16799E+01_JPRB ,&
&  0.18193E+01_JPRB, 0.15782E+01_JPRB, 0.91005E+00_JPRB, 0.11833E+01_JPRB, 0.13848E+01_JPRB, 0.14430E+01_JPRB, 0.14609E+01_JPRB ,&
&  0.13721E+01_JPRB, 0.12159E+01_JPRB, 0.11511E+01_JPRB, 0.12271E+01_JPRB, 0.12969E+01_JPRB, 0.13049E+01_JPRB, 0.14536E+01_JPRB ,&
&  0.13404E+01_JPRB, 0.11867E+01_JPRB, 0.11123E+01_JPRB, 0.13286E+01_JPRB, 0.20288E+01_JPRB, 0.33273E+01_JPRB, 0.29268E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.18232E+01_JPRB, 0.29862E+01_JPRB, 0.39725E+01_JPRB, 0.14964E+01_JPRB, 0.89027E+00_JPRB, 0.63487E+00_JPRB ,&
&  0.66879E+00_JPRB, 0.68568E+00_JPRB, 0.69804E+00_JPRB, 0.72732E+00_JPRB, 0.77309E+00_JPRB, 0.79959E+00_JPRB, 0.81297E+00_JPRB ,&
&  0.84772E+00_JPRB, 0.89658E+00_JPRB, 0.93443E+00_JPRB, 0.98044E+00_JPRB, 0.10223E+01_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.12908E+01_JPRB, 0.13295E+01_JPRB, 0.13607E+01_JPRB, 0.13951E+01_JPRB, 0.14365E+01_JPRB, 0.14770E+01_JPRB, 0.15073E+01_JPRB ,&
&  0.15448E+01_JPRB, 0.15733E+01_JPRB, 0.15873E+01_JPRB, 0.16113E+01_JPRB, 0.16194E+01_JPRB, 0.16942E+01_JPRB, 0.17240E+01_JPRB ,&
&  0.18016E+01_JPRB, 0.18500E+01_JPRB, 0.18710E+01_JPRB, 0.18790E+01_JPRB, 0.19007E+01_JPRB, 0.18840E+01_JPRB, 0.18669E+01_JPRB ,&
&  0.18973E+01_JPRB, 0.19634E+01_JPRB, 0.20211E+01_JPRB, 0.20727E+01_JPRB, 0.20823E+01_JPRB, 0.21584E+01_JPRB, 0.22705E+01_JPRB ,&
&  0.19077E+01_JPRB, 0.22017E+01_JPRB, 0.20059E+01_JPRB, 0.16700E+01_JPRB, 0.16540E+01_JPRB, 0.16713E+01_JPRB, 0.15980E+01_JPRB ,&
&  0.15395E+01_JPRB, 0.15678E+01_JPRB, 0.15900E+01_JPRB, 0.16132E+01_JPRB, 0.16741E+01_JPRB, 0.17429E+01_JPRB, 0.17893E+01_JPRB ,&
&  0.18685E+01_JPRB, 0.19299E+01_JPRB, 0.19816E+01_JPRB, 0.20671E+01_JPRB, 0.20224E+01_JPRB, 0.23500E+01_JPRB, 0.24493E+01_JPRB ,&
&  0.24604E+01_JPRB, 0.23646E+01_JPRB, 0.18346E+01_JPRB, 0.17526E+01_JPRB, 0.18149E+01_JPRB, 0.17331E+01_JPRB, 0.16398E+01_JPRB ,&
&  0.17812E+01_JPRB, 0.16949E+01_JPRB, 0.12207E+01_JPRB, 0.13095E+01_JPRB, 0.13346E+01_JPRB, 0.13364E+01_JPRB, 0.12328E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.98238E+00_JPRB, 0.93785E+00_JPRB, 0.96918E+00_JPRB, 0.98871E+00_JPRB, 0.98054E+00_JPRB, 0.10303E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.10730E+01_JPRB, 0.11236E+01_JPRB, 0.12643E+01_JPRB, 0.21717E+01_JPRB, 0.28443E+01_JPRB, 0.38691E+01_JPRB ,&
&  0.16976E+01_JPRB, 0.17219E+01_JPRB, 0.15095E+01_JPRB, 0.20786E+01_JPRB, 0.11961E+01_JPRB, 0.71935E+00_JPRB, 0.67138E+00_JPRB ,&
&  0.74489E+00_JPRB, 0.78177E+00_JPRB, 0.79518E+00_JPRB, 0.82241E+00_JPRB, 0.88115E+00_JPRB, 0.92212E+00_JPRB, 0.96340E+00_JPRB ,&
&  0.99052E+00_JPRB, 0.10549E+01_JPRB, 0.11373E+01_JPRB, 0.12012E+01_JPRB, 0.12390E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.14021E+01_JPRB, 0.14529E+01_JPRB, 0.14992E+01_JPRB, 0.15463E+01_JPRB, 0.15924E+01_JPRB, 0.16383E+01_JPRB, 0.16771E+01_JPRB ,&
&  0.17086E+01_JPRB, 0.17308E+01_JPRB, 0.17428E+01_JPRB, 0.17597E+01_JPRB, 0.17692E+01_JPRB, 0.18336E+01_JPRB, 0.18546E+01_JPRB ,&
&  0.19289E+01_JPRB, 0.19634E+01_JPRB, 0.19593E+01_JPRB, 0.20073E+01_JPRB, 0.19848E+01_JPRB, 0.19474E+01_JPRB, 0.19414E+01_JPRB ,&
&  0.19611E+01_JPRB, 0.19673E+01_JPRB, 0.19731E+01_JPRB, 0.19931E+01_JPRB, 0.20072E+01_JPRB, 0.19774E+01_JPRB, 0.21624E+01_JPRB ,&
&  0.17677E+01_JPRB, 0.24297E+01_JPRB, 0.23517E+01_JPRB, 0.20470E+01_JPRB, 0.19242E+01_JPRB, 0.17708E+01_JPRB, 0.17415E+01_JPRB ,&
&  0.17390E+01_JPRB, 0.16961E+01_JPRB, 0.16872E+01_JPRB, 0.17081E+01_JPRB, 0.17609E+01_JPRB, 0.18376E+01_JPRB, 0.19043E+01_JPRB ,&
&  0.19932E+01_JPRB, 0.20723E+01_JPRB, 0.21370E+01_JPRB, 0.21992E+01_JPRB, 0.23245E+01_JPRB, 0.24110E+01_JPRB, 0.24382E+01_JPRB ,&
&  0.24303E+01_JPRB, 0.24356E+01_JPRB, 0.19267E+01_JPRB, 0.18892E+01_JPRB, 0.20316E+01_JPRB, 0.19505E+01_JPRB, 0.17468E+01_JPRB ,&
&  0.17656E+01_JPRB, 0.16426E+01_JPRB, 0.15096E+01_JPRB, 0.13779E+01_JPRB, 0.12911E+01_JPRB, 0.12076E+01_JPRB, 0.10999E+01_JPRB ,&
&  0.91488E+00_JPRB, 0.78529E+00_JPRB, 0.75685E+00_JPRB, 0.80198E+00_JPRB, 0.82503E+00_JPRB, 0.83877E+00_JPRB, 0.83967E+00_JPRB ,&
&  0.88163E+00_JPRB, 0.95465E+00_JPRB, 0.10875E+01_JPRB, 0.13807E+01_JPRB, 0.18511E+01_JPRB, 0.35779E+01_JPRB, 0.36239E+01_JPRB ,&
&  0.16407E+01_JPRB, 0.15154E+01_JPRB, 0.88508E+00_JPRB, 0.98813E+00_JPRB, 0.99040E+00_JPRB, 0.80295E+00_JPRB, 0.80566E+00_JPRB ,&
&  0.92350E+00_JPRB, 0.99564E+00_JPRB, 0.10284E+01_JPRB, 0.98243E+00_JPRB, 0.10175E+01_JPRB, 0.10211E+01_JPRB, 0.10438E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.11479E+01_JPRB, 0.12195E+01_JPRB, 0.12871E+01_JPRB, 0.13463E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.14485E+01_JPRB, 0.15035E+01_JPRB, 0.15614E+01_JPRB, 0.16254E+01_JPRB, 0.16910E+01_JPRB, 0.17442E+01_JPRB, 0.17948E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.18775E+01_JPRB, 0.18906E+01_JPRB, 0.18998E+01_JPRB, 0.19074E+01_JPRB, 0.19675E+01_JPRB, 0.20160E+01_JPRB ,&
&  0.20215E+01_JPRB, 0.20568E+01_JPRB, 0.20799E+01_JPRB, 0.21141E+01_JPRB, 0.21065E+01_JPRB, 0.20613E+01_JPRB, 0.20831E+01_JPRB ,&
&  0.20838E+01_JPRB, 0.20986E+01_JPRB, 0.21153E+01_JPRB, 0.21737E+01_JPRB, 0.22527E+01_JPRB, 0.24718E+01_JPRB, 0.22125E+01_JPRB ,&
&  0.24560E+01_JPRB, 0.26053E+01_JPRB, 0.24912E+01_JPRB, 0.22541E+01_JPRB, 0.20246E+01_JPRB, 0.19666E+01_JPRB, 0.19350E+01_JPRB ,&
&  0.19317E+01_JPRB, 0.18547E+01_JPRB, 0.18191E+01_JPRB, 0.17966E+01_JPRB, 0.18077E+01_JPRB, 0.18752E+01_JPRB, 0.19302E+01_JPRB ,&
&  0.20147E+01_JPRB, 0.21480E+01_JPRB, 0.22803E+01_JPRB, 0.24017E+01_JPRB, 0.25600E+01_JPRB, 0.27209E+01_JPRB, 0.28138E+01_JPRB ,&
&  0.28554E+01_JPRB, 0.28974E+01_JPRB, 0.26342E+01_JPRB, 0.26548E+01_JPRB, 0.27768E+01_JPRB, 0.25159E+01_JPRB, 0.21190E+01_JPRB ,&
&  0.19398E+01_JPRB, 0.15637E+01_JPRB, 0.13498E+01_JPRB, 0.14474E+01_JPRB, 0.12952E+01_JPRB, 0.11399E+01_JPRB, 0.97022E+00_JPRB ,&
&  0.78590E+00_JPRB, 0.69614E+00_JPRB, 0.69726E+00_JPRB, 0.74691E+00_JPRB, 0.75096E+00_JPRB, 0.77663E+00_JPRB, 0.83560E+00_JPRB ,&
&  0.92648E+00_JPRB, 0.10332E+01_JPRB, 0.11896E+01_JPRB, 0.14574E+01_JPRB, 0.18180E+01_JPRB, 0.18718E+01_JPRB, 0.20277E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.11633E+01_JPRB, 0.11528E+01_JPRB, 0.10520E+01_JPRB, 0.10678E+01_JPRB, 0.10056E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.14013E+01_JPRB, 0.14194E+01_JPRB, 0.12196E+01_JPRB, 0.11335E+01_JPRB, 0.11040E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11948E+01_JPRB, 0.12590E+01_JPRB, 0.13308E+01_JPRB, 0.13937E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.14406E+01_JPRB, 0.14984E+01_JPRB, 0.15598E+01_JPRB, 0.16214E+01_JPRB, 0.16748E+01_JPRB, 0.17209E+01_JPRB, 0.17601E+01_JPRB ,&
&  0.18087E+01_JPRB, 0.18567E+01_JPRB, 0.18945E+01_JPRB, 0.19191E+01_JPRB, 0.19453E+01_JPRB, 0.19919E+01_JPRB, 0.20462E+01_JPRB ,&
&  0.21104E+01_JPRB, 0.21649E+01_JPRB, 0.22496E+01_JPRB, 0.22749E+01_JPRB, 0.23001E+01_JPRB, 0.23395E+01_JPRB, 0.24256E+01_JPRB ,&
&  0.24760E+01_JPRB, 0.24862E+01_JPRB, 0.25104E+01_JPRB, 0.25963E+01_JPRB, 0.26821E+01_JPRB, 0.28526E+01_JPRB, 0.26060E+01_JPRB ,&
&  0.27395E+01_JPRB, 0.28392E+01_JPRB, 0.26914E+01_JPRB, 0.25260E+01_JPRB, 0.22831E+01_JPRB, 0.21946E+01_JPRB, 0.21468E+01_JPRB ,&
&  0.20961E+01_JPRB, 0.20401E+01_JPRB, 0.19611E+01_JPRB, 0.19106E+01_JPRB, 0.18863E+01_JPRB, 0.18969E+01_JPRB, 0.19620E+01_JPRB ,&
&  0.20300E+01_JPRB, 0.21332E+01_JPRB, 0.22986E+01_JPRB, 0.24680E+01_JPRB, 0.26999E+01_JPRB, 0.29262E+01_JPRB, 0.30933E+01_JPRB ,&
&  0.32616E+01_JPRB, 0.34256E+01_JPRB, 0.32237E+01_JPRB, 0.33081E+01_JPRB, 0.34941E+01_JPRB, 0.30974E+01_JPRB, 0.24062E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.14751E+01_JPRB, 0.14643E+01_JPRB, 0.14959E+01_JPRB, 0.12866E+01_JPRB, 0.11079E+01_JPRB, 0.97731E+00_JPRB ,&
&  0.82493E+00_JPRB, 0.71845E+00_JPRB, 0.73288E+00_JPRB, 0.73257E+00_JPRB, 0.73308E+00_JPRB, 0.78553E+00_JPRB, 0.86229E+00_JPRB ,&
&  0.96562E+00_JPRB, 0.11256E+01_JPRB, 0.13068E+01_JPRB, 0.15133E+01_JPRB, 0.16621E+01_JPRB, 0.16176E+01_JPRB, 0.16489E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.13761E+01_JPRB, 0.13251E+01_JPRB, 0.98746E+00_JPRB, 0.12270E+01_JPRB, 0.13029E+01_JPRB, 0.14284E+01_JPRB ,&
&  0.13874E+01_JPRB, 0.18143E+01_JPRB, 0.21663E+01_JPRB, 0.17198E+01_JPRB, 0.14261E+01_JPRB, 0.11464E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.11867E+01_JPRB, 0.12440E+01_JPRB, 0.13128E+01_JPRB, 0.13783E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.13767E+01_JPRB, 0.14393E+01_JPRB, 0.15051E+01_JPRB, 0.15616E+01_JPRB, 0.16058E+01_JPRB, 0.16165E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.16479E+01_JPRB, 0.16837E+01_JPRB, 0.17291E+01_JPRB, 0.17786E+01_JPRB, 0.18210E+01_JPRB, 0.18659E+01_JPRB, 0.19239E+01_JPRB ,&
&  0.19931E+01_JPRB, 0.20747E+01_JPRB, 0.21837E+01_JPRB, 0.22903E+01_JPRB, 0.24191E+01_JPRB, 0.25454E+01_JPRB, 0.26832E+01_JPRB ,&
&  0.28091E+01_JPRB, 0.29368E+01_JPRB, 0.29992E+01_JPRB, 0.30893E+01_JPRB, 0.31451E+01_JPRB, 0.32761E+01_JPRB, 0.26360E+01_JPRB ,&
&  0.28769E+01_JPRB, 0.29436E+01_JPRB, 0.28928E+01_JPRB, 0.27015E+01_JPRB, 0.24463E+01_JPRB, 0.23154E+01_JPRB, 0.22155E+01_JPRB ,&
&  0.21566E+01_JPRB, 0.20576E+01_JPRB, 0.18925E+01_JPRB, 0.19129E+01_JPRB, 0.19332E+01_JPRB, 0.18684E+01_JPRB, 0.19463E+01_JPRB ,&
&  0.20230E+01_JPRB, 0.21341E+01_JPRB, 0.22729E+01_JPRB, 0.24851E+01_JPRB, 0.27546E+01_JPRB, 0.30071E+01_JPRB, 0.32634E+01_JPRB ,&
&  0.35302E+01_JPRB, 0.38097E+01_JPRB, 0.40195E+01_JPRB, 0.36649E+01_JPRB, 0.36561E+01_JPRB, 0.32420E+01_JPRB, 0.24307E+01_JPRB ,&
&  0.16615E+01_JPRB, 0.14499E+01_JPRB, 0.14774E+01_JPRB, 0.15164E+01_JPRB, 0.13125E+01_JPRB, 0.11049E+01_JPRB, 0.96800E+00_JPRB ,&
&  0.83627E+00_JPRB, 0.77173E+00_JPRB, 0.76375E+00_JPRB, 0.74310E+00_JPRB, 0.74711E+00_JPRB, 0.80068E+00_JPRB, 0.87695E+00_JPRB ,&
&  0.97336E+00_JPRB, 0.11109E+01_JPRB, 0.12866E+01_JPRB, 0.14821E+01_JPRB, 0.16744E+01_JPRB, 0.17413E+01_JPRB, 0.15425E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.13079E+01_JPRB, 0.13232E+01_JPRB, 0.14094E+01_JPRB, 0.14020E+01_JPRB, 0.16348E+01_JPRB, 0.18738E+01_JPRB ,&
&  0.20991E+01_JPRB, 0.19255E+01_JPRB, 0.22203E+01_JPRB, 0.30391E+01_JPRB, 0.27936E+01_JPRB, 0.23369E+01_JPRB, 0.10315E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10726E+01_JPRB, 0.11354E+01_JPRB, 0.12163E+01_JPRB, 0.13070E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.10887E+01_JPRB, 0.11680E+01_JPRB, 0.12505E+01_JPRB, 0.13612E+01_JPRB, 0.14628E+01_JPRB, 0.14977E+01_JPRB, 0.14949E+01_JPRB ,&
&  0.14976E+01_JPRB, 0.15130E+01_JPRB, 0.15487E+01_JPRB, 0.15976E+01_JPRB, 0.16446E+01_JPRB, 0.16872E+01_JPRB, 0.17377E+01_JPRB ,&
&  0.18038E+01_JPRB, 0.18795E+01_JPRB, 0.19704E+01_JPRB, 0.20698E+01_JPRB, 0.21758E+01_JPRB, 0.23248E+01_JPRB, 0.25054E+01_JPRB ,&
&  0.27027E+01_JPRB, 0.28949E+01_JPRB, 0.30443E+01_JPRB, 0.31896E+01_JPRB, 0.33142E+01_JPRB, 0.33012E+01_JPRB, 0.23569E+01_JPRB ,&
&  0.28973E+01_JPRB, 0.29515E+01_JPRB, 0.30097E+01_JPRB, 0.28322E+01_JPRB, 0.25311E+01_JPRB, 0.23556E+01_JPRB, 0.22704E+01_JPRB ,&
&  0.21773E+01_JPRB, 0.20451E+01_JPRB, 0.18934E+01_JPRB, 0.17831E+01_JPRB, 0.18957E+01_JPRB, 0.18144E+01_JPRB, 0.19036E+01_JPRB ,&
&  0.19927E+01_JPRB, 0.21023E+01_JPRB, 0.22299E+01_JPRB, 0.24346E+01_JPRB, 0.27136E+01_JPRB, 0.29999E+01_JPRB, 0.33144E+01_JPRB ,&
&  0.36289E+01_JPRB, 0.39061E+01_JPRB, 0.40607E+01_JPRB, 0.35608E+01_JPRB, 0.33822E+01_JPRB, 0.31306E+01_JPRB, 0.21323E+01_JPRB ,&
&  0.16846E+01_JPRB, 0.15913E+01_JPRB, 0.16584E+01_JPRB, 0.15907E+01_JPRB, 0.13882E+01_JPRB, 0.12089E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.86366E+00_JPRB, 0.78294E+00_JPRB, 0.77384E+00_JPRB, 0.76889E+00_JPRB, 0.78129E+00_JPRB, 0.83997E+00_JPRB, 0.91313E+00_JPRB ,&
&  0.99053E+00_JPRB, 0.10878E+01_JPRB, 0.11785E+01_JPRB, 0.12681E+01_JPRB, 0.13686E+01_JPRB, 0.15353E+01_JPRB, 0.16397E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.12772E+01_JPRB, 0.12422E+01_JPRB, 0.12415E+01_JPRB, 0.12591E+01_JPRB, 0.13233E+01_JPRB, 0.13936E+01_JPRB ,&
&  0.15247E+01_JPRB, 0.15980E+01_JPRB, 0.13482E+01_JPRB, 0.11244E+01_JPRB, 0.14097E+01_JPRB, 0.15216E+01_JPRB, 0.87530E+00_JPRB ,&
&  0.85665E+00_JPRB, 0.88489E+00_JPRB, 0.91791E+00_JPRB, 0.97410E+00_JPRB, 0.10300E+01_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.85671E+00_JPRB, 0.88681E+00_JPRB, 0.99121E+00_JPRB, 0.10341E+01_JPRB, 0.11463E+01_JPRB, 0.12090E+01_JPRB, 0.12801E+01_JPRB ,&
&  0.13140E+01_JPRB, 0.13369E+01_JPRB, 0.13579E+01_JPRB, 0.13861E+01_JPRB, 0.14217E+01_JPRB, 0.14677E+01_JPRB, 0.15121E+01_JPRB ,&
&  0.15645E+01_JPRB, 0.16276E+01_JPRB, 0.17013E+01_JPRB, 0.17806E+01_JPRB, 0.18837E+01_JPRB, 0.20049E+01_JPRB, 0.21737E+01_JPRB ,&
&  0.23885E+01_JPRB, 0.26416E+01_JPRB, 0.29025E+01_JPRB, 0.31625E+01_JPRB, 0.34312E+01_JPRB, 0.36391E+01_JPRB, 0.37484E+01_JPRB ,&
&  0.24594E+01_JPRB, 0.30531E+01_JPRB, 0.30137E+01_JPRB, 0.28355E+01_JPRB, 0.26045E+01_JPRB, 0.24528E+01_JPRB, 0.23677E+01_JPRB ,&
&  0.22801E+01_JPRB, 0.20763E+01_JPRB, 0.19783E+01_JPRB, 0.18261E+01_JPRB, 0.17785E+01_JPRB, 0.17478E+01_JPRB, 0.18246E+01_JPRB ,&
&  0.18989E+01_JPRB, 0.20161E+01_JPRB, 0.21719E+01_JPRB, 0.23296E+01_JPRB, 0.25868E+01_JPRB, 0.29321E+01_JPRB, 0.32948E+01_JPRB ,&
&  0.36777E+01_JPRB, 0.40737E+01_JPRB, 0.43087E+01_JPRB, 0.38194E+01_JPRB, 0.33495E+01_JPRB, 0.31652E+01_JPRB, 0.23374E+01_JPRB ,&
&  0.19030E+01_JPRB, 0.16660E+01_JPRB, 0.17762E+01_JPRB, 0.17654E+01_JPRB, 0.15304E+01_JPRB, 0.13699E+01_JPRB, 0.12106E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.86814E+00_JPRB, 0.82276E+00_JPRB, 0.82608E+00_JPRB, 0.84834E+00_JPRB, 0.89445E+00_JPRB, 0.93409E+00_JPRB ,&
&  0.97918E+00_JPRB, 0.10296E+01_JPRB, 0.10749E+01_JPRB, 0.11251E+01_JPRB, 0.11490E+01_JPRB, 0.11705E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.11687E+01_JPRB, 0.12034E+01_JPRB, 0.11920E+01_JPRB, 0.11935E+01_JPRB, 0.11646E+01_JPRB, 0.11604E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.11906E+01_JPRB, 0.12000E+01_JPRB, 0.11780E+01_JPRB, 0.11311E+01_JPRB, 0.10447E+01_JPRB, 0.96223E+00_JPRB, 0.80665E+00_JPRB ,&
&  0.78919E+00_JPRB, 0.77430E+00_JPRB, 0.79623E+00_JPRB, 0.81420E+00_JPRB, 0.84023E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.73576E+00_JPRB, 0.76354E+00_JPRB, 0.79196E+00_JPRB, 0.82454E+00_JPRB, 0.87664E+00_JPRB, 0.94016E+00_JPRB, 0.10241E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.11472E+01_JPRB, 0.11599E+01_JPRB, 0.11696E+01_JPRB, 0.11900E+01_JPRB, 0.12308E+01_JPRB, 0.12805E+01_JPRB ,&
&  0.13299E+01_JPRB, 0.13757E+01_JPRB, 0.14201E+01_JPRB, 0.14730E+01_JPRB, 0.15383E+01_JPRB, 0.16343E+01_JPRB, 0.17524E+01_JPRB ,&
&  0.19058E+01_JPRB, 0.21477E+01_JPRB, 0.24647E+01_JPRB, 0.28336E+01_JPRB, 0.32339E+01_JPRB, 0.35627E+01_JPRB, 0.37504E+01_JPRB ,&
&  0.20212E+01_JPRB, 0.26348E+01_JPRB, 0.31890E+01_JPRB, 0.29679E+01_JPRB, 0.27324E+01_JPRB, 0.25957E+01_JPRB, 0.25101E+01_JPRB ,&
&  0.23259E+01_JPRB, 0.20916E+01_JPRB, 0.19740E+01_JPRB, 0.19631E+01_JPRB, 0.17391E+01_JPRB, 0.17034E+01_JPRB, 0.17348E+01_JPRB ,&
&  0.17871E+01_JPRB, 0.18446E+01_JPRB, 0.19547E+01_JPRB, 0.21105E+01_JPRB, 0.23391E+01_JPRB, 0.26960E+01_JPRB, 0.31074E+01_JPRB ,&
&  0.35601E+01_JPRB, 0.40836E+01_JPRB, 0.45577E+01_JPRB, 0.41238E+01_JPRB, 0.40706E+01_JPRB, 0.41878E+01_JPRB, 0.38027E+01_JPRB ,&
&  0.25565E+01_JPRB, 0.19678E+01_JPRB, 0.20275E+01_JPRB, 0.20157E+01_JPRB, 0.17213E+01_JPRB, 0.15654E+01_JPRB, 0.14169E+01_JPRB ,&
&  0.12592E+01_JPRB, 0.11020E+01_JPRB, 0.93929E+00_JPRB, 0.90313E+00_JPRB, 0.91938E+00_JPRB, 0.95564E+00_JPRB, 0.98914E+00_JPRB ,&
&  0.10077E+01_JPRB, 0.10161E+01_JPRB, 0.10354E+01_JPRB, 0.10611E+01_JPRB, 0.11040E+01_JPRB, 0.11228E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.11537E+01_JPRB, 0.11671E+01_JPRB, 0.11885E+01_JPRB, 0.12093E+01_JPRB, 0.12162E+01_JPRB, 0.12039E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.11778E+01_JPRB, 0.11903E+01_JPRB, 0.11735E+01_JPRB, 0.11199E+01_JPRB, 0.10102E+01_JPRB, 0.92143E+00_JPRB ,&
&  0.83218E+00_JPRB, 0.82029E+00_JPRB, 0.79224E+00_JPRB, 0.76620E+00_JPRB, 0.74195E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.72228E+00_JPRB, 0.71960E+00_JPRB, 0.73096E+00_JPRB, 0.74004E+00_JPRB, 0.75870E+00_JPRB, 0.79510E+00_JPRB, 0.84189E+00_JPRB ,&
&  0.88288E+00_JPRB, 0.95053E+00_JPRB, 0.99949E+00_JPRB, 0.10018E+01_JPRB, 0.99604E+00_JPRB, 0.10154E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.11570E+01_JPRB, 0.11925E+01_JPRB, 0.12249E+01_JPRB, 0.12652E+01_JPRB, 0.13081E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.15889E+01_JPRB, 0.18161E+01_JPRB, 0.21752E+01_JPRB, 0.26715E+01_JPRB, 0.32560E+01_JPRB, 0.36683E+01_JPRB ,&
&  0.25958E+01_JPRB, 0.12158E+01_JPRB, 0.27952E+01_JPRB, 0.33164E+01_JPRB, 0.29738E+01_JPRB, 0.28775E+01_JPRB, 0.26819E+01_JPRB ,&
&  0.24707E+01_JPRB, 0.23567E+01_JPRB, 0.21657E+01_JPRB, 0.20232E+01_JPRB, 0.17595E+01_JPRB, 0.16759E+01_JPRB, 0.16400E+01_JPRB ,&
&  0.16494E+01_JPRB, 0.16748E+01_JPRB, 0.17411E+01_JPRB, 0.18488E+01_JPRB, 0.20145E+01_JPRB, 0.22946E+01_JPRB, 0.27423E+01_JPRB ,&
&  0.32352E+01_JPRB, 0.38313E+01_JPRB, 0.46209E+01_JPRB, 0.44886E+01_JPRB, 0.46862E+01_JPRB, 0.42998E+01_JPRB, 0.36205E+01_JPRB ,&
&  0.28616E+01_JPRB, 0.23270E+01_JPRB, 0.23280E+01_JPRB, 0.23396E+01_JPRB, 0.19874E+01_JPRB, 0.16858E+01_JPRB, 0.16680E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.13465E+01_JPRB, 0.11646E+01_JPRB, 0.10415E+01_JPRB, 0.10094E+01_JPRB, 0.10353E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.10652E+01_JPRB, 0.10603E+01_JPRB, 0.10639E+01_JPRB, 0.10795E+01_JPRB, 0.11076E+01_JPRB, 0.11266E+01_JPRB, 0.11319E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11766E+01_JPRB, 0.11996E+01_JPRB, 0.12221E+01_JPRB, 0.12375E+01_JPRB, 0.12162E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.12208E+01_JPRB, 0.12175E+01_JPRB, 0.12042E+01_JPRB, 0.12057E+01_JPRB, 0.11760E+01_JPRB, 0.11075E+01_JPRB, 0.10763E+01_JPRB ,&
&  0.99919E+00_JPRB, 0.92468E+00_JPRB, 0.83388E+00_JPRB, 0.77611E+00_JPRB, 0.75366E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.84084E+00_JPRB, 0.79773E+00_JPRB, 0.75287E+00_JPRB, 0.74175E+00_JPRB, 0.74190E+00_JPRB, 0.73945E+00_JPRB, 0.76163E+00_JPRB ,&
&  0.78214E+00_JPRB, 0.81342E+00_JPRB, 0.82730E+00_JPRB, 0.85647E+00_JPRB, 0.86728E+00_JPRB, 0.87331E+00_JPRB, 0.89579E+00_JPRB ,&
&  0.91824E+00_JPRB, 0.94595E+00_JPRB, 0.97616E+00_JPRB, 0.10104E+01_JPRB, 0.10451E+01_JPRB, 0.10612E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.11873E+01_JPRB, 0.12844E+01_JPRB, 0.14856E+01_JPRB, 0.18469E+01_JPRB, 0.23731E+01_JPRB, 0.30048E+01_JPRB ,&
&  0.36425E+01_JPRB, 0.25632E+01_JPRB, 0.17285E+01_JPRB, 0.31094E+01_JPRB, 0.34134E+01_JPRB, 0.33610E+01_JPRB, 0.28409E+01_JPRB ,&
&  0.28600E+01_JPRB, 0.26965E+01_JPRB, 0.24898E+01_JPRB, 0.22349E+01_JPRB, 0.18891E+01_JPRB, 0.17157E+01_JPRB, 0.15910E+01_JPRB ,&
&  0.15242E+01_JPRB, 0.14939E+01_JPRB, 0.15242E+01_JPRB, 0.16314E+01_JPRB, 0.17558E+01_JPRB, 0.19319E+01_JPRB, 0.22892E+01_JPRB ,&
&  0.27636E+01_JPRB, 0.33636E+01_JPRB, 0.43087E+01_JPRB, 0.47647E+01_JPRB, 0.54516E+01_JPRB, 0.46871E+01_JPRB, 0.38564E+01_JPRB ,&
&  0.30463E+01_JPRB, 0.29121E+01_JPRB, 0.29976E+01_JPRB, 0.26932E+01_JPRB, 0.22012E+01_JPRB, 0.19695E+01_JPRB, 0.19552E+01_JPRB ,&
&  0.16953E+01_JPRB, 0.15577E+01_JPRB, 0.14394E+01_JPRB, 0.12775E+01_JPRB, 0.11773E+01_JPRB, 0.11488E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11192E+01_JPRB, 0.11060E+01_JPRB, 0.11104E+01_JPRB, 0.11230E+01_JPRB, 0.11325E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.11635E+01_JPRB, 0.11837E+01_JPRB, 0.12085E+01_JPRB, 0.12231E+01_JPRB, 0.12118E+01_JPRB, 0.11897E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.12441E+01_JPRB, 0.12272E+01_JPRB, 0.12273E+01_JPRB, 0.12073E+01_JPRB, 0.11935E+01_JPRB, 0.11787E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11262E+01_JPRB, 0.10316E+01_JPRB, 0.91604E+00_JPRB, 0.88252E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.94722E+00_JPRB, 0.91249E+00_JPRB, 0.87054E+00_JPRB, 0.82862E+00_JPRB, 0.80658E+00_JPRB, 0.78427E+00_JPRB, 0.75626E+00_JPRB ,&
&  0.74075E+00_JPRB, 0.73393E+00_JPRB, 0.72211E+00_JPRB, 0.73434E+00_JPRB, 0.76436E+00_JPRB, 0.78008E+00_JPRB, 0.79475E+00_JPRB ,&
&  0.81208E+00_JPRB, 0.82938E+00_JPRB, 0.82913E+00_JPRB, 0.83514E+00_JPRB, 0.85670E+00_JPRB, 0.87367E+00_JPRB, 0.87772E+00_JPRB ,&
&  0.89117E+00_JPRB, 0.92195E+00_JPRB, 0.96646E+00_JPRB, 0.10516E+01_JPRB, 0.12268E+01_JPRB, 0.15171E+01_JPRB, 0.19396E+01_JPRB ,&
&  0.26115E+01_JPRB, 0.26894E+01_JPRB, 0.14005E+01_JPRB, 0.26273E+01_JPRB, 0.36527E+01_JPRB, 0.37073E+01_JPRB, 0.33930E+01_JPRB ,&
&  0.33636E+01_JPRB, 0.32396E+01_JPRB, 0.30305E+01_JPRB, 0.25184E+01_JPRB, 0.21719E+01_JPRB, 0.19095E+01_JPRB, 0.16933E+01_JPRB ,&
&  0.15204E+01_JPRB, 0.14000E+01_JPRB, 0.13488E+01_JPRB, 0.13974E+01_JPRB, 0.15054E+01_JPRB, 0.16042E+01_JPRB, 0.18058E+01_JPRB ,&
&  0.21737E+01_JPRB, 0.26648E+01_JPRB, 0.34875E+01_JPRB, 0.43127E+01_JPRB, 0.56239E+01_JPRB, 0.56368E+01_JPRB, 0.45996E+01_JPRB ,&
&  0.36455E+01_JPRB, 0.35991E+01_JPRB, 0.35699E+01_JPRB, 0.31452E+01_JPRB, 0.24576E+01_JPRB, 0.22459E+01_JPRB, 0.22035E+01_JPRB ,&
&  0.19458E+01_JPRB, 0.17433E+01_JPRB, 0.16478E+01_JPRB, 0.15437E+01_JPRB, 0.13995E+01_JPRB, 0.12778E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.11863E+01_JPRB, 0.11726E+01_JPRB, 0.11588E+01_JPRB, 0.11518E+01_JPRB, 0.11492E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11857E+01_JPRB, 0.12055E+01_JPRB, 0.12067E+01_JPRB, 0.12063E+01_JPRB, 0.12058E+01_JPRB, 0.12084E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12136E+01_JPRB, 0.11941E+01_JPRB, 0.11615E+01_JPRB, 0.12007E+01_JPRB, 0.11940E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.12260E+01_JPRB, 0.11514E+01_JPRB, 0.10150E+01_JPRB, 0.98285E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.98633E+00_JPRB, 0.95903E+00_JPRB, 0.93179E+00_JPRB, 0.90895E+00_JPRB, 0.89020E+00_JPRB, 0.87096E+00_JPRB, 0.84038E+00_JPRB ,&
&  0.80175E+00_JPRB, 0.77633E+00_JPRB, 0.73906E+00_JPRB, 0.72531E+00_JPRB, 0.72941E+00_JPRB, 0.73395E+00_JPRB, 0.74622E+00_JPRB ,&
&  0.75646E+00_JPRB, 0.76778E+00_JPRB, 0.77489E+00_JPRB, 0.77718E+00_JPRB, 0.78044E+00_JPRB, 0.78514E+00_JPRB, 0.78494E+00_JPRB ,&
&  0.77964E+00_JPRB, 0.78713E+00_JPRB, 0.80689E+00_JPRB, 0.84023E+00_JPRB, 0.91316E+00_JPRB, 0.10530E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.17981E+01_JPRB, 0.21545E+01_JPRB, 0.23577E+01_JPRB, 0.36455E+01_JPRB, 0.43876E+01_JPRB, 0.42336E+01_JPRB, 0.42216E+01_JPRB ,&
&  0.43207E+01_JPRB, 0.42421E+01_JPRB, 0.38445E+01_JPRB, 0.28797E+01_JPRB, 0.22875E+01_JPRB, 0.19859E+01_JPRB, 0.17574E+01_JPRB ,&
&  0.15566E+01_JPRB, 0.14032E+01_JPRB, 0.12851E+01_JPRB, 0.12413E+01_JPRB, 0.12752E+01_JPRB, 0.13656E+01_JPRB, 0.14705E+01_JPRB ,&
&  0.16809E+01_JPRB, 0.20477E+01_JPRB, 0.26015E+01_JPRB, 0.33042E+01_JPRB, 0.45908E+01_JPRB, 0.55546E+01_JPRB, 0.53078E+01_JPRB ,&
&  0.49781E+01_JPRB, 0.45634E+01_JPRB, 0.40544E+01_JPRB, 0.36262E+01_JPRB, 0.27549E+01_JPRB, 0.26676E+01_JPRB, 0.23753E+01_JPRB ,&
&  0.20839E+01_JPRB, 0.18738E+01_JPRB, 0.17534E+01_JPRB, 0.16823E+01_JPRB, 0.15799E+01_JPRB, 0.14385E+01_JPRB, 0.13226E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.12360E+01_JPRB, 0.12274E+01_JPRB, 0.12134E+01_JPRB, 0.12015E+01_JPRB, 0.11949E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11805E+01_JPRB, 0.11626E+01_JPRB, 0.11745E+01_JPRB, 0.11758E+01_JPRB, 0.11683E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11866E+01_JPRB, 0.11848E+01_JPRB, 0.11748E+01_JPRB, 0.11656E+01_JPRB, 0.11570E+01_JPRB, 0.11368E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.11025E+01_JPRB, 0.10428E+01_JPRB, 0.10220E+01_JPRB, 0.10058E+01_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.99236E+00_JPRB, 0.96865E+00_JPRB, 0.95172E+00_JPRB, 0.93232E+00_JPRB, 0.91327E+00_JPRB, 0.90119E+00_JPRB, 0.89034E+00_JPRB ,&
&  0.87159E+00_JPRB, 0.83066E+00_JPRB, 0.78790E+00_JPRB, 0.76404E+00_JPRB, 0.75365E+00_JPRB, 0.73617E+00_JPRB, 0.73456E+00_JPRB ,&
&  0.73675E+00_JPRB, 0.74492E+00_JPRB, 0.74422E+00_JPRB, 0.74369E+00_JPRB, 0.74351E+00_JPRB, 0.74525E+00_JPRB, 0.74367E+00_JPRB ,&
&  0.73409E+00_JPRB, 0.72953E+00_JPRB, 0.73555E+00_JPRB, 0.74907E+00_JPRB, 0.77911E+00_JPRB, 0.84618E+00_JPRB, 0.93971E+00_JPRB ,&
&  0.12556E+01_JPRB, 0.17070E+01_JPRB, 0.17973E+01_JPRB, 0.35639E+01_JPRB, 0.42409E+01_JPRB, 0.45593E+01_JPRB, 0.43694E+01_JPRB ,&
&  0.44797E+01_JPRB, 0.50091E+01_JPRB, 0.41773E+01_JPRB, 0.29975E+01_JPRB, 0.22145E+01_JPRB, 0.18512E+01_JPRB, 0.16437E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.13303E+01_JPRB, 0.12298E+01_JPRB, 0.11717E+01_JPRB, 0.11663E+01_JPRB, 0.11891E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.13515E+01_JPRB, 0.15966E+01_JPRB, 0.19806E+01_JPRB, 0.23338E+01_JPRB, 0.32751E+01_JPRB, 0.43745E+01_JPRB, 0.50048E+01_JPRB ,&
&  0.53002E+01_JPRB, 0.52652E+01_JPRB, 0.43728E+01_JPRB, 0.38121E+01_JPRB, 0.31375E+01_JPRB, 0.27817E+01_JPRB, 0.24195E+01_JPRB ,&
&  0.21640E+01_JPRB, 0.19631E+01_JPRB, 0.18165E+01_JPRB, 0.17298E+01_JPRB, 0.16442E+01_JPRB, 0.15426E+01_JPRB, 0.14192E+01_JPRB ,&
&  0.13146E+01_JPRB, 0.12656E+01_JPRB, 0.12454E+01_JPRB, 0.12334E+01_JPRB, 0.12313E+01_JPRB, 0.12258E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.12014E+01_JPRB, 0.11864E+01_JPRB, 0.11529E+01_JPRB, 0.11520E+01_JPRB, 0.11369E+01_JPRB, 0.11036E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.11736E+01_JPRB, 0.11717E+01_JPRB, 0.11423E+01_JPRB, 0.11527E+01_JPRB, 0.11683E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.10644E+01_JPRB, 0.10358E+01_JPRB, 0.10194E+01_JPRB, 0.10084E+01_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.98322E+00_JPRB, 0.95200E+00_JPRB, 0.92963E+00_JPRB, 0.91108E+00_JPRB, 0.89381E+00_JPRB, 0.87804E+00_JPRB, 0.87279E+00_JPRB ,&
&  0.86658E+00_JPRB, 0.85447E+00_JPRB, 0.82639E+00_JPRB, 0.79310E+00_JPRB, 0.77503E+00_JPRB, 0.76515E+00_JPRB, 0.74641E+00_JPRB ,&
&  0.73663E+00_JPRB, 0.73568E+00_JPRB, 0.73466E+00_JPRB, 0.72738E+00_JPRB, 0.71970E+00_JPRB, 0.71381E+00_JPRB, 0.70953E+00_JPRB ,&
&  0.69568E+00_JPRB, 0.68938E+00_JPRB, 0.68974E+00_JPRB, 0.69124E+00_JPRB, 0.70058E+00_JPRB, 0.74972E+00_JPRB, 0.84653E+00_JPRB ,&
&  0.10329E+01_JPRB, 0.19382E+01_JPRB, 0.17789E+01_JPRB, 0.28734E+01_JPRB, 0.33009E+01_JPRB, 0.38045E+01_JPRB, 0.41230E+01_JPRB ,&
&  0.44763E+01_JPRB, 0.48662E+01_JPRB, 0.39522E+01_JPRB, 0.28910E+01_JPRB, 0.20236E+01_JPRB, 0.15652E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.12704E+01_JPRB, 0.11896E+01_JPRB, 0.11261E+01_JPRB, 0.11013E+01_JPRB, 0.10998E+01_JPRB, 0.11049E+01_JPRB, 0.11188E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.13238E+01_JPRB, 0.16644E+01_JPRB, 0.21235E+01_JPRB, 0.23855E+01_JPRB, 0.31311E+01_JPRB, 0.42306E+01_JPRB ,&
&  0.50047E+01_JPRB, 0.54875E+01_JPRB, 0.45956E+01_JPRB, 0.38557E+01_JPRB, 0.32921E+01_JPRB, 0.28614E+01_JPRB, 0.24953E+01_JPRB ,&
&  0.22125E+01_JPRB, 0.20035E+01_JPRB, 0.18496E+01_JPRB, 0.17399E+01_JPRB, 0.16616E+01_JPRB, 0.16005E+01_JPRB, 0.15158E+01_JPRB ,&
&  0.14198E+01_JPRB, 0.13482E+01_JPRB, 0.13033E+01_JPRB, 0.12785E+01_JPRB, 0.12698E+01_JPRB, 0.12524E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.11792E+01_JPRB, 0.11493E+01_JPRB, 0.11424E+01_JPRB, 0.11276E+01_JPRB, 0.10877E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.11405E+01_JPRB, 0.11474E+01_JPRB, 0.11477E+01_JPRB, 0.11589E+01_JPRB, 0.12174E+01_JPRB, 0.11283E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10472E+01_JPRB, 0.10255E+01_JPRB, 0.10155E+01_JPRB, 0.10027E+01_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.96223E+00_JPRB, 0.93029E+00_JPRB, 0.90402E+00_JPRB, 0.87831E+00_JPRB, 0.85894E+00_JPRB, 0.83845E+00_JPRB, 0.82488E+00_JPRB ,&
&  0.81873E+00_JPRB, 0.81860E+00_JPRB, 0.81560E+00_JPRB, 0.80239E+00_JPRB, 0.78000E+00_JPRB, 0.76110E+00_JPRB, 0.74668E+00_JPRB ,&
&  0.73251E+00_JPRB, 0.72715E+00_JPRB, 0.72397E+00_JPRB, 0.71284E+00_JPRB, 0.70047E+00_JPRB, 0.68895E+00_JPRB, 0.67735E+00_JPRB ,&
&  0.66706E+00_JPRB, 0.66207E+00_JPRB, 0.65860E+00_JPRB, 0.65547E+00_JPRB, 0.65424E+00_JPRB, 0.68177E+00_JPRB, 0.81657E+00_JPRB ,&
&  0.11091E+01_JPRB, 0.98149E+00_JPRB, 0.22365E+01_JPRB, 0.25940E+01_JPRB, 0.27202E+01_JPRB, 0.29368E+01_JPRB, 0.34902E+01_JPRB ,&
&  0.40562E+01_JPRB, 0.40333E+01_JPRB, 0.34901E+01_JPRB, 0.27035E+01_JPRB, 0.19997E+01_JPRB, 0.15439E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.10618E+01_JPRB, 0.10192E+01_JPRB, 0.99659E+00_JPRB, 0.99467E+00_JPRB, 0.10076E+01_JPRB, 0.10266E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.11606E+01_JPRB, 0.14067E+01_JPRB, 0.18748E+01_JPRB, 0.23349E+01_JPRB, 0.26892E+01_JPRB, 0.33891E+01_JPRB ,&
&  0.44422E+01_JPRB, 0.47860E+01_JPRB, 0.40127E+01_JPRB, 0.34309E+01_JPRB, 0.30512E+01_JPRB, 0.27763E+01_JPRB, 0.25016E+01_JPRB ,&
&  0.22393E+01_JPRB, 0.19879E+01_JPRB, 0.18050E+01_JPRB, 0.16768E+01_JPRB, 0.16043E+01_JPRB, 0.15766E+01_JPRB, 0.15509E+01_JPRB ,&
&  0.15157E+01_JPRB, 0.14639E+01_JPRB, 0.14120E+01_JPRB, 0.13634E+01_JPRB, 0.13237E+01_JPRB, 0.12839E+01_JPRB, 0.12284E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.11553E+01_JPRB, 0.11249E+01_JPRB, 0.11018E+01_JPRB, 0.10942E+01_JPRB, 0.10826E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10956E+01_JPRB, 0.11020E+01_JPRB, 0.11063E+01_JPRB, 0.11188E+01_JPRB, 0.11096E+01_JPRB, 0.11645E+01_JPRB, 0.11245E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.10582E+01_JPRB, 0.10208E+01_JPRB, 0.10000E+01_JPRB, 0.98544E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.93571E+00_JPRB, 0.91025E+00_JPRB, 0.87608E+00_JPRB, 0.84606E+00_JPRB, 0.82232E+00_JPRB, 0.79755E+00_JPRB, 0.77994E+00_JPRB ,&
&  0.76731E+00_JPRB, 0.76890E+00_JPRB, 0.77501E+00_JPRB, 0.77788E+00_JPRB, 0.77008E+00_JPRB, 0.75026E+00_JPRB, 0.73003E+00_JPRB ,&
&  0.71483E+00_JPRB, 0.70841E+00_JPRB, 0.70537E+00_JPRB, 0.69831E+00_JPRB, 0.68798E+00_JPRB, 0.67500E+00_JPRB, 0.66330E+00_JPRB ,&
&  0.65248E+00_JPRB, 0.64512E+00_JPRB, 0.63790E+00_JPRB, 0.63214E+00_JPRB, 0.62577E+00_JPRB, 0.64001E+00_JPRB, 0.76704E+00_JPRB ,&
&  0.11684E+01_JPRB, 0.16900E+01_JPRB, 0.21924E+01_JPRB, 0.20688E+01_JPRB, 0.20616E+01_JPRB, 0.21726E+01_JPRB, 0.25812E+01_JPRB ,&
&  0.28105E+01_JPRB, 0.29135E+01_JPRB, 0.27587E+01_JPRB, 0.23668E+01_JPRB, 0.18895E+01_JPRB, 0.15547E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.99256E+00_JPRB, 0.92637E+00_JPRB, 0.90593E+00_JPRB, 0.91074E+00_JPRB, 0.92692E+00_JPRB, 0.94297E+00_JPRB ,&
&  0.96442E+00_JPRB, 0.10188E+01_JPRB, 0.12016E+01_JPRB, 0.16272E+01_JPRB, 0.23347E+01_JPRB, 0.24198E+01_JPRB, 0.32034E+01_JPRB ,&
&  0.40471E+01_JPRB, 0.36782E+01_JPRB, 0.33413E+01_JPRB, 0.30537E+01_JPRB, 0.28239E+01_JPRB, 0.26204E+01_JPRB, 0.23045E+01_JPRB ,&
&  0.20514E+01_JPRB, 0.18185E+01_JPRB, 0.16455E+01_JPRB, 0.15479E+01_JPRB, 0.14890E+01_JPRB, 0.14661E+01_JPRB, 0.14619E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.14373E+01_JPRB, 0.14208E+01_JPRB, 0.13845E+01_JPRB, 0.13332E+01_JPRB, 0.12801E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.11266E+01_JPRB, 0.10872E+01_JPRB, 0.10490E+01_JPRB, 0.10435E+01_JPRB, 0.10481E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10327E+01_JPRB, 0.10322E+01_JPRB, 0.10498E+01_JPRB, 0.10516E+01_JPRB, 0.10704E+01_JPRB, 0.11271E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.10889E+01_JPRB, 0.10434E+01_JPRB, 0.10036E+01_JPRB, 0.97270E+00_JPRB, 0.95522E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.91327E+00_JPRB, 0.88132E+00_JPRB, 0.84662E+00_JPRB, 0.81519E+00_JPRB, 0.78659E+00_JPRB, 0.76463E+00_JPRB, 0.75075E+00_JPRB ,&
&  0.73863E+00_JPRB, 0.73198E+00_JPRB, 0.73554E+00_JPRB, 0.74254E+00_JPRB, 0.74186E+00_JPRB, 0.73026E+00_JPRB, 0.71305E+00_JPRB ,&
&  0.69510E+00_JPRB, 0.68094E+00_JPRB, 0.67053E+00_JPRB, 0.66568E+00_JPRB, 0.65921E+00_JPRB, 0.65198E+00_JPRB, 0.64047E+00_JPRB ,&
&  0.63108E+00_JPRB, 0.62367E+00_JPRB, 0.61304E+00_JPRB, 0.60187E+00_JPRB, 0.59347E+00_JPRB, 0.59921E+00_JPRB, 0.69378E+00_JPRB ,&
&  0.99431E+00_JPRB, 0.14390E+01_JPRB, 0.14977E+01_JPRB, 0.15512E+01_JPRB, 0.17073E+01_JPRB, 0.17471E+01_JPRB, 0.19042E+01_JPRB ,&
&  0.19801E+01_JPRB, 0.20299E+01_JPRB, 0.20566E+01_JPRB, 0.19345E+01_JPRB, 0.16793E+01_JPRB, 0.14753E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.10004E+01_JPRB, 0.92071E+00_JPRB, 0.88577E+00_JPRB, 0.88202E+00_JPRB, 0.89235E+00_JPRB, 0.89873E+00_JPRB ,&
&  0.90186E+00_JPRB, 0.90668E+00_JPRB, 0.96840E+00_JPRB, 0.11344E+01_JPRB, 0.15551E+01_JPRB, 0.18563E+01_JPRB, 0.23177E+01_JPRB ,&
&  0.24252E+01_JPRB, 0.23951E+01_JPRB, 0.23310E+01_JPRB, 0.22578E+01_JPRB, 0.21639E+01_JPRB, 0.20213E+01_JPRB, 0.18946E+01_JPRB ,&
&  0.17417E+01_JPRB, 0.15866E+01_JPRB, 0.14659E+01_JPRB, 0.13855E+01_JPRB, 0.13353E+01_JPRB, 0.13057E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.12671E+01_JPRB, 0.12477E+01_JPRB, 0.12315E+01_JPRB, 0.12191E+01_JPRB, 0.11863E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10334E+01_JPRB, 0.99913E+00_JPRB, 0.97965E+00_JPRB, 0.96939E+00_JPRB, 0.96117E+00_JPRB, 0.95378E+00_JPRB ,&
&  0.95873E+00_JPRB, 0.97018E+00_JPRB, 0.97686E+00_JPRB, 0.97593E+00_JPRB, 0.98338E+00_JPRB, 0.10260E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.98995E+00_JPRB, 0.96479E+00_JPRB, 0.93633E+00_JPRB, 0.93333E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.90365E+00_JPRB, 0.87536E+00_JPRB, 0.83889E+00_JPRB, 0.80350E+00_JPRB, 0.77096E+00_JPRB, 0.74535E+00_JPRB, 0.73000E+00_JPRB ,&
&  0.71909E+00_JPRB, 0.71189E+00_JPRB, 0.70846E+00_JPRB, 0.70643E+00_JPRB, 0.70303E+00_JPRB, 0.69879E+00_JPRB, 0.69142E+00_JPRB ,&
&  0.68146E+00_JPRB, 0.66896E+00_JPRB, 0.65750E+00_JPRB, 0.64927E+00_JPRB, 0.64301E+00_JPRB, 0.63876E+00_JPRB, 0.63239E+00_JPRB ,&
&  0.62110E+00_JPRB, 0.60820E+00_JPRB, 0.59242E+00_JPRB, 0.57666E+00_JPRB, 0.56606E+00_JPRB, 0.56769E+00_JPRB, 0.62753E+00_JPRB ,&
&  0.84185E+00_JPRB, 0.98354E+00_JPRB, 0.11080E+01_JPRB, 0.11951E+01_JPRB, 0.13200E+01_JPRB, 0.13942E+01_JPRB, 0.14385E+01_JPRB ,&
&  0.14649E+01_JPRB, 0.15011E+01_JPRB, 0.15694E+01_JPRB, 0.15609E+01_JPRB, 0.14393E+01_JPRB, 0.13150E+01_JPRB, 0.11977E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.10010E+01_JPRB, 0.92741E+00_JPRB, 0.89176E+00_JPRB, 0.87912E+00_JPRB, 0.88020E+00_JPRB, 0.87917E+00_JPRB ,&
&  0.87276E+00_JPRB, 0.85434E+00_JPRB, 0.84562E+00_JPRB, 0.86315E+00_JPRB, 0.92791E+00_JPRB, 0.10617E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.13866E+01_JPRB, 0.14174E+01_JPRB, 0.14473E+01_JPRB, 0.14757E+01_JPRB, 0.14479E+01_JPRB, 0.14067E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.13270E+01_JPRB, 0.12832E+01_JPRB, 0.12450E+01_JPRB, 0.11973E+01_JPRB, 0.11559E+01_JPRB, 0.11261E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10804E+01_JPRB, 0.10654E+01_JPRB, 0.10367E+01_JPRB, 0.10147E+01_JPRB, 0.99188E+00_JPRB, 0.95918E+00_JPRB ,&
&  0.91489E+00_JPRB, 0.88688E+00_JPRB, 0.87044E+00_JPRB, 0.86755E+00_JPRB, 0.87032E+00_JPRB, 0.87465E+00_JPRB, 0.87377E+00_JPRB ,&
&  0.87605E+00_JPRB, 0.89161E+00_JPRB, 0.88578E+00_JPRB, 0.89290E+00_JPRB, 0.93532E+00_JPRB, 0.93617E+00_JPRB, 0.93697E+00_JPRB ,&
&  0.93370E+00_JPRB, 0.92254E+00_JPRB, 0.90284E+00_JPRB, 0.87410E+00_JPRB, 0.90598E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.78619E+00_JPRB, 0.78527E+00_JPRB, 0.76652E+00_JPRB, 0.74471E+00_JPRB, 0.72510E+00_JPRB, 0.71027E+00_JPRB, 0.69806E+00_JPRB ,&
&  0.68912E+00_JPRB, 0.68208E+00_JPRB, 0.67749E+00_JPRB, 0.67441E+00_JPRB, 0.67003E+00_JPRB, 0.66353E+00_JPRB, 0.65637E+00_JPRB ,&
&  0.64997E+00_JPRB, 0.64214E+00_JPRB, 0.63056E+00_JPRB, 0.62223E+00_JPRB, 0.61723E+00_JPRB, 0.61077E+00_JPRB, 0.60369E+00_JPRB ,&
&  0.59485E+00_JPRB, 0.57849E+00_JPRB, 0.56196E+00_JPRB, 0.55033E+00_JPRB, 0.54096E+00_JPRB, 0.53927E+00_JPRB, 0.57931E+00_JPRB ,&
&  0.71160E+00_JPRB, 0.73377E+00_JPRB, 0.83509E+00_JPRB, 0.93270E+00_JPRB, 0.10564E+01_JPRB, 0.11259E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11605E+01_JPRB, 0.12038E+01_JPRB, 0.12382E+01_JPRB, 0.12851E+01_JPRB, 0.12390E+01_JPRB, 0.11539E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.95169E+00_JPRB, 0.90423E+00_JPRB, 0.87159E+00_JPRB, 0.85737E+00_JPRB, 0.85121E+00_JPRB, 0.83942E+00_JPRB ,&
&  0.82523E+00_JPRB, 0.80273E+00_JPRB, 0.78328E+00_JPRB, 0.77751E+00_JPRB, 0.76754E+00_JPRB, 0.79828E+00_JPRB, 0.84941E+00_JPRB ,&
&  0.89742E+00_JPRB, 0.96559E+00_JPRB, 0.10024E+01_JPRB, 0.10267E+01_JPRB, 0.10251E+01_JPRB, 0.10211E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.10017E+01_JPRB, 0.99043E+00_JPRB, 0.97183E+00_JPRB, 0.94877E+00_JPRB, 0.92778E+00_JPRB, 0.90949E+00_JPRB ,&
&  0.89376E+00_JPRB, 0.88003E+00_JPRB, 0.86923E+00_JPRB, 0.84714E+00_JPRB, 0.81498E+00_JPRB, 0.78744E+00_JPRB, 0.76631E+00_JPRB ,&
&  0.74843E+00_JPRB, 0.74097E+00_JPRB, 0.74034E+00_JPRB, 0.74878E+00_JPRB, 0.75736E+00_JPRB, 0.76623E+00_JPRB, 0.77131E+00_JPRB ,&
&  0.77495E+00_JPRB, 0.78696E+00_JPRB, 0.80675E+00_JPRB, 0.83513E+00_JPRB, 0.85036E+00_JPRB, 0.85983E+00_JPRB, 0.86367E+00_JPRB ,&
&  0.86129E+00_JPRB, 0.85458E+00_JPRB, 0.83555E+00_JPRB, 0.81221E+00_JPRB, 0.77672E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.71549E+00_JPRB, 0.71482E+00_JPRB, 0.70442E+00_JPRB, 0.68915E+00_JPRB, 0.67634E+00_JPRB, 0.66537E+00_JPRB, 0.65679E+00_JPRB ,&
&  0.64421E+00_JPRB, 0.63308E+00_JPRB, 0.62877E+00_JPRB, 0.62816E+00_JPRB, 0.62685E+00_JPRB, 0.62193E+00_JPRB, 0.61543E+00_JPRB ,&
&  0.60792E+00_JPRB, 0.59945E+00_JPRB, 0.58950E+00_JPRB, 0.57950E+00_JPRB, 0.57122E+00_JPRB, 0.56165E+00_JPRB, 0.55355E+00_JPRB ,&
&  0.54557E+00_JPRB, 0.53796E+00_JPRB, 0.52672E+00_JPRB, 0.51362E+00_JPRB, 0.50724E+00_JPRB, 0.50231E+00_JPRB, 0.53633E+00_JPRB ,&
&  0.64365E+00_JPRB, 0.57307E+00_JPRB, 0.63880E+00_JPRB, 0.74184E+00_JPRB, 0.85683E+00_JPRB, 0.96473E+00_JPRB, 0.99704E+00_JPRB ,&
&  0.10081E+01_JPRB, 0.10253E+01_JPRB, 0.10383E+01_JPRB, 0.10596E+01_JPRB, 0.10679E+01_JPRB, 0.10238E+01_JPRB, 0.96153E+00_JPRB ,&
&  0.91061E+00_JPRB, 0.86716E+00_JPRB, 0.83967E+00_JPRB, 0.81438E+00_JPRB, 0.80045E+00_JPRB, 0.79250E+00_JPRB, 0.77535E+00_JPRB ,&
&  0.75318E+00_JPRB, 0.73909E+00_JPRB, 0.72947E+00_JPRB, 0.72919E+00_JPRB, 0.73184E+00_JPRB, 0.73450E+00_JPRB, 0.72223E+00_JPRB ,&
&  0.71011E+00_JPRB, 0.72743E+00_JPRB, 0.76189E+00_JPRB, 0.78182E+00_JPRB, 0.79353E+00_JPRB, 0.79257E+00_JPRB, 0.78502E+00_JPRB ,&
&  0.78103E+00_JPRB, 0.77816E+00_JPRB, 0.77901E+00_JPRB, 0.77332E+00_JPRB, 0.76334E+00_JPRB, 0.74729E+00_JPRB, 0.72903E+00_JPRB ,&
&  0.71213E+00_JPRB, 0.69747E+00_JPRB, 0.68546E+00_JPRB, 0.67427E+00_JPRB, 0.66111E+00_JPRB, 0.64459E+00_JPRB, 0.63472E+00_JPRB ,&
&  0.62945E+00_JPRB, 0.62897E+00_JPRB, 0.63267E+00_JPRB, 0.64023E+00_JPRB, 0.65141E+00_JPRB, 0.66187E+00_JPRB, 0.67466E+00_JPRB ,&
&  0.68578E+00_JPRB, 0.69759E+00_JPRB, 0.70826E+00_JPRB, 0.73267E+00_JPRB, 0.77011E+00_JPRB, 0.78100E+00_JPRB, 0.79021E+00_JPRB ,&
&  0.79137E+00_JPRB, 0.78886E+00_JPRB, 0.78010E+00_JPRB, 0.69192E+00_JPRB, 0.71405E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.65398E+00_JPRB, 0.65091E+00_JPRB, 0.64254E+00_JPRB, 0.63305E+00_JPRB, 0.62529E+00_JPRB, 0.61761E+00_JPRB, 0.61147E+00_JPRB ,&
&  0.60270E+00_JPRB, 0.59176E+00_JPRB, 0.58366E+00_JPRB, 0.58017E+00_JPRB, 0.57860E+00_JPRB, 0.57514E+00_JPRB, 0.56904E+00_JPRB ,&
&  0.56117E+00_JPRB, 0.55249E+00_JPRB, 0.54399E+00_JPRB, 0.53362E+00_JPRB, 0.52392E+00_JPRB, 0.51482E+00_JPRB, 0.50689E+00_JPRB ,&
&  0.49896E+00_JPRB, 0.49050E+00_JPRB, 0.48223E+00_JPRB, 0.47614E+00_JPRB, 0.47161E+00_JPRB, 0.47043E+00_JPRB, 0.49881E+00_JPRB ,&
&  0.58334E+00_JPRB, 0.50338E+00_JPRB, 0.54839E+00_JPRB, 0.61066E+00_JPRB, 0.71055E+00_JPRB, 0.83190E+00_JPRB, 0.88558E+00_JPRB ,&
&  0.90412E+00_JPRB, 0.91336E+00_JPRB, 0.91211E+00_JPRB, 0.90947E+00_JPRB, 0.91954E+00_JPRB, 0.91389E+00_JPRB, 0.88100E+00_JPRB ,&
&  0.84182E+00_JPRB, 0.80564E+00_JPRB, 0.78106E+00_JPRB, 0.76312E+00_JPRB, 0.74497E+00_JPRB, 0.72767E+00_JPRB, 0.70564E+00_JPRB ,&
&  0.67855E+00_JPRB, 0.66244E+00_JPRB, 0.65357E+00_JPRB, 0.64717E+00_JPRB, 0.64469E+00_JPRB, 0.63812E+00_JPRB, 0.61802E+00_JPRB ,&
&  0.59866E+00_JPRB, 0.58505E+00_JPRB, 0.58041E+00_JPRB, 0.58548E+00_JPRB, 0.59345E+00_JPRB, 0.59855E+00_JPRB, 0.59663E+00_JPRB ,&
&  0.59180E+00_JPRB, 0.58407E+00_JPRB, 0.58001E+00_JPRB, 0.57819E+00_JPRB, 0.57610E+00_JPRB, 0.57448E+00_JPRB, 0.57020E+00_JPRB ,&
&  0.56417E+00_JPRB, 0.56108E+00_JPRB, 0.56160E+00_JPRB, 0.56274E+00_JPRB, 0.56181E+00_JPRB, 0.55868E+00_JPRB, 0.55490E+00_JPRB ,&
&  0.54995E+00_JPRB, 0.54642E+00_JPRB, 0.54693E+00_JPRB, 0.55259E+00_JPRB, 0.56139E+00_JPRB, 0.57142E+00_JPRB, 0.58252E+00_JPRB ,&
&  0.59470E+00_JPRB, 0.60742E+00_JPRB, 0.61925E+00_JPRB, 0.64595E+00_JPRB, 0.66189E+00_JPRB, 0.67919E+00_JPRB, 0.69564E+00_JPRB ,&
&  0.70003E+00_JPRB, 0.69690E+00_JPRB, 0.60228E+00_JPRB, 0.64087E+00_JPRB, 0.64902E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.61123E+00_JPRB, 0.59991E+00_JPRB, 0.58669E+00_JPRB, 0.57779E+00_JPRB, 0.57387E+00_JPRB, 0.56973E+00_JPRB, 0.56578E+00_JPRB ,&
&  0.55966E+00_JPRB, 0.55282E+00_JPRB, 0.54594E+00_JPRB, 0.53969E+00_JPRB, 0.53455E+00_JPRB, 0.53100E+00_JPRB, 0.52642E+00_JPRB ,&
&  0.51961E+00_JPRB, 0.51158E+00_JPRB, 0.50299E+00_JPRB, 0.49315E+00_JPRB, 0.48048E+00_JPRB, 0.47010E+00_JPRB, 0.46181E+00_JPRB ,&
&  0.45535E+00_JPRB, 0.45023E+00_JPRB, 0.44782E+00_JPRB, 0.44514E+00_JPRB, 0.44241E+00_JPRB, 0.44255E+00_JPRB, 0.46289E+00_JPRB ,&
&  0.48784E+00_JPRB, 0.46166E+00_JPRB, 0.49294E+00_JPRB, 0.53755E+00_JPRB, 0.61147E+00_JPRB, 0.72737E+00_JPRB, 0.80154E+00_JPRB ,&
&  0.83788E+00_JPRB, 0.85262E+00_JPRB, 0.85476E+00_JPRB, 0.84190E+00_JPRB, 0.83042E+00_JPRB, 0.83321E+00_JPRB, 0.82798E+00_JPRB ,&
&  0.80376E+00_JPRB, 0.77045E+00_JPRB, 0.74016E+00_JPRB, 0.71740E+00_JPRB, 0.69030E+00_JPRB, 0.66556E+00_JPRB, 0.64018E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.60251E+00_JPRB, 0.59356E+00_JPRB, 0.58677E+00_JPRB, 0.57958E+00_JPRB, 0.56651E+00_JPRB, 0.54927E+00_JPRB ,&
&  0.53462E+00_JPRB, 0.52133E+00_JPRB, 0.50942E+00_JPRB, 0.49973E+00_JPRB, 0.49329E+00_JPRB, 0.48931E+00_JPRB, 0.48627E+00_JPRB ,&
&  0.48571E+00_JPRB, 0.48680E+00_JPRB, 0.48914E+00_JPRB, 0.49140E+00_JPRB, 0.49187E+00_JPRB, 0.49039E+00_JPRB, 0.48906E+00_JPRB ,&
&  0.48802E+00_JPRB, 0.48433E+00_JPRB, 0.48366E+00_JPRB, 0.48845E+00_JPRB, 0.49459E+00_JPRB, 0.49819E+00_JPRB, 0.49918E+00_JPRB ,&
&  0.49716E+00_JPRB, 0.49515E+00_JPRB, 0.49491E+00_JPRB, 0.49729E+00_JPRB, 0.50120E+00_JPRB, 0.50741E+00_JPRB, 0.51506E+00_JPRB ,&
&  0.52291E+00_JPRB, 0.53300E+00_JPRB, 0.54614E+00_JPRB, 0.56575E+00_JPRB, 0.58170E+00_JPRB, 0.59526E+00_JPRB, 0.60783E+00_JPRB ,&
&  0.61238E+00_JPRB, 0.61469E+00_JPRB, 0.61970E+00_JPRB, 0.61868E+00_JPRB, 0.61675E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.54340E+00_JPRB, 0.53696E+00_JPRB, 0.52897E+00_JPRB, 0.52192E+00_JPRB, 0.51806E+00_JPRB, 0.51488E+00_JPRB, 0.51002E+00_JPRB ,&
&  0.50601E+00_JPRB, 0.50250E+00_JPRB, 0.49887E+00_JPRB, 0.49563E+00_JPRB, 0.49265E+00_JPRB, 0.49026E+00_JPRB, 0.48633E+00_JPRB ,&
&  0.47985E+00_JPRB, 0.47162E+00_JPRB, 0.46230E+00_JPRB, 0.45426E+00_JPRB, 0.44451E+00_JPRB, 0.43435E+00_JPRB, 0.42657E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.41564E+00_JPRB, 0.41368E+00_JPRB, 0.41128E+00_JPRB, 0.41172E+00_JPRB, 0.41461E+00_JPRB, 0.42689E+00_JPRB ,&
&  0.46011E+00_JPRB, 0.44599E+00_JPRB, 0.47709E+00_JPRB, 0.50662E+00_JPRB, 0.56675E+00_JPRB, 0.65878E+00_JPRB, 0.72063E+00_JPRB ,&
&  0.75735E+00_JPRB, 0.78663E+00_JPRB, 0.80667E+00_JPRB, 0.80689E+00_JPRB, 0.79143E+00_JPRB, 0.78461E+00_JPRB, 0.78193E+00_JPRB ,&
&  0.76835E+00_JPRB, 0.74220E+00_JPRB, 0.70542E+00_JPRB, 0.66581E+00_JPRB, 0.63090E+00_JPRB, 0.60449E+00_JPRB, 0.58488E+00_JPRB ,&
&  0.57016E+00_JPRB, 0.55808E+00_JPRB, 0.54834E+00_JPRB, 0.53455E+00_JPRB, 0.51849E+00_JPRB, 0.50211E+00_JPRB, 0.48584E+00_JPRB ,&
&  0.46945E+00_JPRB, 0.45380E+00_JPRB, 0.44054E+00_JPRB, 0.42991E+00_JPRB, 0.42347E+00_JPRB, 0.41973E+00_JPRB, 0.41758E+00_JPRB ,&
&  0.41697E+00_JPRB, 0.41866E+00_JPRB, 0.42168E+00_JPRB, 0.42512E+00_JPRB, 0.42773E+00_JPRB, 0.42800E+00_JPRB, 0.42532E+00_JPRB ,&
&  0.42378E+00_JPRB, 0.42284E+00_JPRB, 0.42328E+00_JPRB, 0.42601E+00_JPRB, 0.42787E+00_JPRB, 0.43020E+00_JPRB, 0.43428E+00_JPRB ,&
&  0.43769E+00_JPRB, 0.43919E+00_JPRB, 0.44009E+00_JPRB, 0.44135E+00_JPRB, 0.44311E+00_JPRB, 0.44639E+00_JPRB, 0.45159E+00_JPRB ,&
&  0.45774E+00_JPRB, 0.46477E+00_JPRB, 0.47514E+00_JPRB, 0.48871E+00_JPRB, 0.50173E+00_JPRB, 0.51326E+00_JPRB, 0.52353E+00_JPRB ,&
&  0.52883E+00_JPRB, 0.53330E+00_JPRB, 0.54010E+00_JPRB, 0.54356E+00_JPRB, 0.54593E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.48751E+00_JPRB, 0.48462E+00_JPRB, 0.48092E+00_JPRB, 0.47739E+00_JPRB, 0.47443E+00_JPRB, 0.46953E+00_JPRB, 0.46400E+00_JPRB ,&
&  0.46064E+00_JPRB, 0.45789E+00_JPRB, 0.45608E+00_JPRB, 0.45297E+00_JPRB, 0.45130E+00_JPRB, 0.45024E+00_JPRB, 0.44581E+00_JPRB ,&
&  0.44038E+00_JPRB, 0.43245E+00_JPRB, 0.42457E+00_JPRB, 0.41727E+00_JPRB, 0.41021E+00_JPRB, 0.40351E+00_JPRB, 0.39611E+00_JPRB ,&
&  0.38913E+00_JPRB, 0.38385E+00_JPRB, 0.38070E+00_JPRB, 0.37808E+00_JPRB, 0.37921E+00_JPRB, 0.38522E+00_JPRB, 0.39531E+00_JPRB ,&
&  0.41031E+00_JPRB, 0.43395E+00_JPRB, 0.45550E+00_JPRB, 0.47817E+00_JPRB, 0.52327E+00_JPRB, 0.59112E+00_JPRB, 0.64174E+00_JPRB ,&
&  0.66653E+00_JPRB, 0.69408E+00_JPRB, 0.73075E+00_JPRB, 0.75557E+00_JPRB, 0.75798E+00_JPRB, 0.74621E+00_JPRB, 0.73722E+00_JPRB ,&
&  0.72490E+00_JPRB, 0.70013E+00_JPRB, 0.67213E+00_JPRB, 0.64019E+00_JPRB, 0.60856E+00_JPRB, 0.58278E+00_JPRB, 0.56306E+00_JPRB ,&
&  0.54648E+00_JPRB, 0.53174E+00_JPRB, 0.51742E+00_JPRB, 0.50177E+00_JPRB, 0.48464E+00_JPRB, 0.46548E+00_JPRB, 0.44497E+00_JPRB ,&
&  0.42547E+00_JPRB, 0.40943E+00_JPRB, 0.39746E+00_JPRB, 0.39041E+00_JPRB, 0.38580E+00_JPRB, 0.38171E+00_JPRB, 0.37864E+00_JPRB ,&
&  0.37777E+00_JPRB, 0.37862E+00_JPRB, 0.38082E+00_JPRB, 0.38376E+00_JPRB, 0.38649E+00_JPRB, 0.38750E+00_JPRB, 0.38483E+00_JPRB ,&
&  0.38188E+00_JPRB, 0.38152E+00_JPRB, 0.38203E+00_JPRB, 0.37935E+00_JPRB, 0.37617E+00_JPRB, 0.37712E+00_JPRB, 0.38060E+00_JPRB ,&
&  0.38628E+00_JPRB, 0.39131E+00_JPRB, 0.39507E+00_JPRB, 0.39794E+00_JPRB, 0.39989E+00_JPRB, 0.40338E+00_JPRB, 0.40785E+00_JPRB ,&
&  0.41357E+00_JPRB, 0.42073E+00_JPRB, 0.43065E+00_JPRB, 0.44293E+00_JPRB, 0.45412E+00_JPRB, 0.46360E+00_JPRB, 0.46959E+00_JPRB ,&
&  0.47353E+00_JPRB, 0.47642E+00_JPRB, 0.47930E+00_JPRB, 0.48313E+00_JPRB, 0.48663E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.43424E+00_JPRB, 0.43427E+00_JPRB, 0.43481E+00_JPRB, 0.43442E+00_JPRB, 0.43060E+00_JPRB, 0.42661E+00_JPRB, 0.42229E+00_JPRB ,&
&  0.42009E+00_JPRB, 0.41864E+00_JPRB, 0.41824E+00_JPRB, 0.41523E+00_JPRB, 0.41302E+00_JPRB, 0.41175E+00_JPRB, 0.40774E+00_JPRB ,&
&  0.40341E+00_JPRB, 0.39444E+00_JPRB, 0.38503E+00_JPRB, 0.37825E+00_JPRB, 0.37397E+00_JPRB, 0.37076E+00_JPRB, 0.36453E+00_JPRB ,&
&  0.35969E+00_JPRB, 0.35578E+00_JPRB, 0.35366E+00_JPRB, 0.35350E+00_JPRB, 0.35576E+00_JPRB, 0.36204E+00_JPRB, 0.37205E+00_JPRB ,&
&  0.38069E+00_JPRB, 0.40140E+00_JPRB, 0.40651E+00_JPRB, 0.43011E+00_JPRB, 0.45888E+00_JPRB, 0.50316E+00_JPRB, 0.54837E+00_JPRB ,&
&  0.57344E+00_JPRB, 0.58809E+00_JPRB, 0.61344E+00_JPRB, 0.64511E+00_JPRB, 0.66098E+00_JPRB, 0.66434E+00_JPRB, 0.65829E+00_JPRB ,&
&  0.64885E+00_JPRB, 0.63899E+00_JPRB, 0.62780E+00_JPRB, 0.61235E+00_JPRB, 0.59081E+00_JPRB, 0.56726E+00_JPRB, 0.54506E+00_JPRB ,&
&  0.52507E+00_JPRB, 0.50586E+00_JPRB, 0.48662E+00_JPRB, 0.46554E+00_JPRB, 0.44358E+00_JPRB, 0.42236E+00_JPRB, 0.40191E+00_JPRB ,&
&  0.38552E+00_JPRB, 0.37320E+00_JPRB, 0.36418E+00_JPRB, 0.35738E+00_JPRB, 0.35198E+00_JPRB, 0.34693E+00_JPRB, 0.34398E+00_JPRB ,&
&  0.34305E+00_JPRB, 0.34422E+00_JPRB, 0.34689E+00_JPRB, 0.34929E+00_JPRB, 0.35220E+00_JPRB, 0.35260E+00_JPRB, 0.34863E+00_JPRB ,&
&  0.34497E+00_JPRB, 0.34419E+00_JPRB, 0.33948E+00_JPRB, 0.33662E+00_JPRB, 0.33594E+00_JPRB, 0.33665E+00_JPRB, 0.33924E+00_JPRB ,&
&  0.34414E+00_JPRB, 0.35059E+00_JPRB, 0.35650E+00_JPRB, 0.36165E+00_JPRB, 0.36626E+00_JPRB, 0.37076E+00_JPRB, 0.37579E+00_JPRB ,&
&  0.38103E+00_JPRB, 0.38610E+00_JPRB, 0.39376E+00_JPRB, 0.40232E+00_JPRB, 0.40969E+00_JPRB, 0.41772E+00_JPRB, 0.42276E+00_JPRB ,&
&  0.42490E+00_JPRB, 0.42577E+00_JPRB, 0.42805E+00_JPRB, 0.43151E+00_JPRB, 0.43383E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.38959E+00_JPRB, 0.39132E+00_JPRB, 0.39336E+00_JPRB, 0.39334E+00_JPRB, 0.39221E+00_JPRB, 0.39163E+00_JPRB, 0.39004E+00_JPRB ,&
&  0.38714E+00_JPRB, 0.38530E+00_JPRB, 0.38522E+00_JPRB, 0.38354E+00_JPRB, 0.38043E+00_JPRB, 0.37667E+00_JPRB, 0.37248E+00_JPRB ,&
&  0.36892E+00_JPRB, 0.36520E+00_JPRB, 0.35901E+00_JPRB, 0.35120E+00_JPRB, 0.34455E+00_JPRB, 0.34017E+00_JPRB, 0.33764E+00_JPRB ,&
&  0.33633E+00_JPRB, 0.33593E+00_JPRB, 0.33531E+00_JPRB, 0.33587E+00_JPRB, 0.33816E+00_JPRB, 0.34343E+00_JPRB, 0.35113E+00_JPRB ,&
&  0.35857E+00_JPRB, 0.36770E+00_JPRB, 0.37721E+00_JPRB, 0.38839E+00_JPRB, 0.40276E+00_JPRB, 0.42388E+00_JPRB, 0.44929E+00_JPRB ,&
&  0.47721E+00_JPRB, 0.48750E+00_JPRB, 0.49025E+00_JPRB, 0.50004E+00_JPRB, 0.51841E+00_JPRB, 0.53351E+00_JPRB, 0.54411E+00_JPRB ,&
&  0.55032E+00_JPRB, 0.55095E+00_JPRB, 0.54847E+00_JPRB, 0.54029E+00_JPRB, 0.52710E+00_JPRB, 0.51001E+00_JPRB, 0.49127E+00_JPRB ,&
&  0.47297E+00_JPRB, 0.45417E+00_JPRB, 0.43435E+00_JPRB, 0.41476E+00_JPRB, 0.39665E+00_JPRB, 0.37996E+00_JPRB, 0.36462E+00_JPRB ,&
&  0.35161E+00_JPRB, 0.34159E+00_JPRB, 0.33376E+00_JPRB, 0.32802E+00_JPRB, 0.32324E+00_JPRB, 0.31947E+00_JPRB, 0.31774E+00_JPRB ,&
&  0.31750E+00_JPRB, 0.31845E+00_JPRB, 0.31890E+00_JPRB, 0.32041E+00_JPRB, 0.32311E+00_JPRB, 0.32384E+00_JPRB, 0.32175E+00_JPRB ,&
&  0.31693E+00_JPRB, 0.31249E+00_JPRB, 0.30804E+00_JPRB, 0.30739E+00_JPRB, 0.30750E+00_JPRB, 0.30930E+00_JPRB, 0.31197E+00_JPRB ,&
&  0.31671E+00_JPRB, 0.32287E+00_JPRB, 0.32859E+00_JPRB, 0.33301E+00_JPRB, 0.33793E+00_JPRB, 0.34228E+00_JPRB, 0.34640E+00_JPRB ,&
&  0.35072E+00_JPRB, 0.35514E+00_JPRB, 0.36089E+00_JPRB, 0.36668E+00_JPRB, 0.37200E+00_JPRB, 0.37626E+00_JPRB, 0.37938E+00_JPRB ,&
&  0.38151E+00_JPRB, 0.38227E+00_JPRB, 0.38287E+00_JPRB, 0.38482E+00_JPRB, 0.38753E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.35139E+00_JPRB, 0.35367E+00_JPRB, 0.35646E+00_JPRB, 0.35842E+00_JPRB, 0.35885E+00_JPRB, 0.35807E+00_JPRB, 0.35706E+00_JPRB ,&
&  0.35640E+00_JPRB, 0.35549E+00_JPRB, 0.35338E+00_JPRB, 0.35040E+00_JPRB, 0.34753E+00_JPRB, 0.34456E+00_JPRB, 0.34250E+00_JPRB ,&
&  0.34055E+00_JPRB, 0.33824E+00_JPRB, 0.33392E+00_JPRB, 0.32746E+00_JPRB, 0.32080E+00_JPRB, 0.31708E+00_JPRB, 0.31553E+00_JPRB ,&
&  0.31577E+00_JPRB, 0.31666E+00_JPRB, 0.31933E+00_JPRB, 0.32109E+00_JPRB, 0.32350E+00_JPRB, 0.32764E+00_JPRB, 0.33356E+00_JPRB ,&
&  0.33891E+00_JPRB, 0.34418E+00_JPRB, 0.35009E+00_JPRB, 0.35573E+00_JPRB, 0.36153E+00_JPRB, 0.36997E+00_JPRB, 0.38121E+00_JPRB ,&
&  0.39799E+00_JPRB, 0.41191E+00_JPRB, 0.41229E+00_JPRB, 0.41216E+00_JPRB, 0.41840E+00_JPRB, 0.42799E+00_JPRB, 0.43672E+00_JPRB ,&
&  0.44389E+00_JPRB, 0.44590E+00_JPRB, 0.44514E+00_JPRB, 0.44114E+00_JPRB, 0.43542E+00_JPRB, 0.42796E+00_JPRB, 0.41764E+00_JPRB ,&
&  0.40518E+00_JPRB, 0.39143E+00_JPRB, 0.37784E+00_JPRB, 0.36429E+00_JPRB, 0.35094E+00_JPRB, 0.33886E+00_JPRB, 0.32851E+00_JPRB ,&
&  0.31958E+00_JPRB, 0.31259E+00_JPRB, 0.30617E+00_JPRB, 0.30183E+00_JPRB, 0.29875E+00_JPRB, 0.29560E+00_JPRB, 0.29459E+00_JPRB ,&
&  0.29311E+00_JPRB, 0.29197E+00_JPRB, 0.29154E+00_JPRB, 0.29276E+00_JPRB, 0.29561E+00_JPRB, 0.29731E+00_JPRB, 0.29566E+00_JPRB ,&
&  0.29206E+00_JPRB, 0.28992E+00_JPRB, 0.28859E+00_JPRB, 0.28774E+00_JPRB, 0.28548E+00_JPRB, 0.28561E+00_JPRB, 0.28834E+00_JPRB ,&
&  0.29383E+00_JPRB, 0.29952E+00_JPRB, 0.30406E+00_JPRB, 0.30692E+00_JPRB, 0.31009E+00_JPRB, 0.31303E+00_JPRB, 0.31560E+00_JPRB ,&
&  0.31807E+00_JPRB, 0.31976E+00_JPRB, 0.32094E+00_JPRB, 0.32249E+00_JPRB, 0.32502E+00_JPRB, 0.32745E+00_JPRB, 0.32984E+00_JPRB ,&
&  0.33267E+00_JPRB, 0.33723E+00_JPRB, 0.34185E+00_JPRB, 0.34531E+00_JPRB, 0.34911E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.32502E+00_JPRB, 0.32767E+00_JPRB, 0.32974E+00_JPRB, 0.32919E+00_JPRB, 0.32661E+00_JPRB, 0.32511E+00_JPRB, 0.32507E+00_JPRB ,&
&  0.32520E+00_JPRB, 0.32450E+00_JPRB, 0.32241E+00_JPRB, 0.32189E+00_JPRB, 0.32176E+00_JPRB, 0.32089E+00_JPRB, 0.32048E+00_JPRB ,&
&  0.31926E+00_JPRB, 0.31597E+00_JPRB, 0.31107E+00_JPRB, 0.30515E+00_JPRB, 0.30096E+00_JPRB, 0.29916E+00_JPRB, 0.29955E+00_JPRB ,&
&  0.30113E+00_JPRB, 0.30225E+00_JPRB, 0.30500E+00_JPRB, 0.30699E+00_JPRB, 0.30920E+00_JPRB, 0.31241E+00_JPRB, 0.31597E+00_JPRB ,&
&  0.31835E+00_JPRB, 0.32096E+00_JPRB, 0.32394E+00_JPRB, 0.32703E+00_JPRB, 0.33117E+00_JPRB, 0.33779E+00_JPRB, 0.34423E+00_JPRB ,&
&  0.35492E+00_JPRB, 0.36438E+00_JPRB, 0.36123E+00_JPRB, 0.35594E+00_JPRB, 0.35551E+00_JPRB, 0.35921E+00_JPRB, 0.36459E+00_JPRB ,&
&  0.37071E+00_JPRB, 0.37659E+00_JPRB, 0.38118E+00_JPRB, 0.38261E+00_JPRB, 0.38056E+00_JPRB, 0.37546E+00_JPRB, 0.36750E+00_JPRB ,&
&  0.35796E+00_JPRB, 0.34774E+00_JPRB, 0.33725E+00_JPRB, 0.32633E+00_JPRB, 0.31675E+00_JPRB, 0.30809E+00_JPRB, 0.30060E+00_JPRB ,&
&  0.29383E+00_JPRB, 0.28758E+00_JPRB, 0.28286E+00_JPRB, 0.27959E+00_JPRB, 0.27767E+00_JPRB, 0.27560E+00_JPRB, 0.27515E+00_JPRB ,&
&  0.27393E+00_JPRB, 0.27317E+00_JPRB, 0.27265E+00_JPRB, 0.27290E+00_JPRB, 0.27408E+00_JPRB, 0.27496E+00_JPRB, 0.27418E+00_JPRB ,&
&  0.27381E+00_JPRB, 0.27542E+00_JPRB, 0.27525E+00_JPRB, 0.27252E+00_JPRB, 0.26910E+00_JPRB, 0.26847E+00_JPRB, 0.27040E+00_JPRB ,&
&  0.27394E+00_JPRB, 0.27777E+00_JPRB, 0.28109E+00_JPRB, 0.28383E+00_JPRB, 0.28613E+00_JPRB, 0.28897E+00_JPRB, 0.29158E+00_JPRB ,&
&  0.29386E+00_JPRB, 0.29528E+00_JPRB, 0.29730E+00_JPRB, 0.29945E+00_JPRB, 0.30156E+00_JPRB, 0.30298E+00_JPRB, 0.30276E+00_JPRB ,&
&  0.30218E+00_JPRB, 0.30408E+00_JPRB, 0.30954E+00_JPRB, 0.31717E+00_JPRB, 0.32196E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.30173E+00_JPRB, 0.30381E+00_JPRB, 0.30460E+00_JPRB, 0.30369E+00_JPRB, 0.30232E+00_JPRB, 0.30251E+00_JPRB, 0.30285E+00_JPRB ,&
&  0.30356E+00_JPRB, 0.30330E+00_JPRB, 0.30128E+00_JPRB, 0.29943E+00_JPRB, 0.30083E+00_JPRB, 0.30053E+00_JPRB, 0.30019E+00_JPRB ,&
&  0.29863E+00_JPRB, 0.29656E+00_JPRB, 0.29361E+00_JPRB, 0.29036E+00_JPRB, 0.28671E+00_JPRB, 0.28517E+00_JPRB, 0.28653E+00_JPRB ,&
&  0.28891E+00_JPRB, 0.28986E+00_JPRB, 0.29058E+00_JPRB, 0.29117E+00_JPRB, 0.29270E+00_JPRB, 0.29478E+00_JPRB, 0.29625E+00_JPRB ,&
&  0.29722E+00_JPRB, 0.30001E+00_JPRB, 0.30173E+00_JPRB, 0.30162E+00_JPRB, 0.30586E+00_JPRB, 0.31100E+00_JPRB, 0.31685E+00_JPRB ,&
&  0.32339E+00_JPRB, 0.32785E+00_JPRB, 0.32810E+00_JPRB, 0.32628E+00_JPRB, 0.32490E+00_JPRB, 0.32618E+00_JPRB, 0.32747E+00_JPRB ,&
&  0.32951E+00_JPRB, 0.33470E+00_JPRB, 0.34083E+00_JPRB, 0.34451E+00_JPRB, 0.34501E+00_JPRB, 0.34138E+00_JPRB, 0.33363E+00_JPRB ,&
&  0.32238E+00_JPRB, 0.30984E+00_JPRB, 0.29909E+00_JPRB, 0.29034E+00_JPRB, 0.28372E+00_JPRB, 0.27881E+00_JPRB, 0.27488E+00_JPRB ,&
&  0.27058E+00_JPRB, 0.26733E+00_JPRB, 0.26686E+00_JPRB, 0.26714E+00_JPRB, 0.26636E+00_JPRB, 0.26324E+00_JPRB, 0.26110E+00_JPRB ,&
&  0.25921E+00_JPRB, 0.25961E+00_JPRB, 0.26089E+00_JPRB, 0.26140E+00_JPRB, 0.26161E+00_JPRB, 0.26163E+00_JPRB, 0.26156E+00_JPRB ,&
&  0.26262E+00_JPRB, 0.26206E+00_JPRB, 0.26048E+00_JPRB, 0.25807E+00_JPRB, 0.25594E+00_JPRB, 0.25566E+00_JPRB, 0.25650E+00_JPRB ,&
&  0.25785E+00_JPRB, 0.26012E+00_JPRB, 0.26279E+00_JPRB, 0.26543E+00_JPRB, 0.26749E+00_JPRB, 0.26854E+00_JPRB, 0.26917E+00_JPRB ,&
&  0.26971E+00_JPRB, 0.27015E+00_JPRB, 0.27225E+00_JPRB, 0.27695E+00_JPRB, 0.28114E+00_JPRB, 0.28406E+00_JPRB, 0.28528E+00_JPRB ,&
&  0.28435E+00_JPRB, 0.28470E+00_JPRB, 0.28785E+00_JPRB, 0.29384E+00_JPRB, 0.29823E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.28329E+00_JPRB, 0.28524E+00_JPRB, 0.28547E+00_JPRB, 0.28455E+00_JPRB, 0.28354E+00_JPRB, 0.28413E+00_JPRB, 0.28525E+00_JPRB ,&
&  0.28582E+00_JPRB, 0.28585E+00_JPRB, 0.28448E+00_JPRB, 0.28186E+00_JPRB, 0.27969E+00_JPRB, 0.27956E+00_JPRB, 0.27806E+00_JPRB ,&
&  0.27709E+00_JPRB, 0.27819E+00_JPRB, 0.27788E+00_JPRB, 0.27732E+00_JPRB, 0.27605E+00_JPRB, 0.27584E+00_JPRB, 0.27739E+00_JPRB ,&
&  0.27987E+00_JPRB, 0.28034E+00_JPRB, 0.28073E+00_JPRB, 0.28123E+00_JPRB, 0.28233E+00_JPRB, 0.28389E+00_JPRB, 0.28449E+00_JPRB ,&
&  0.28449E+00_JPRB, 0.28243E+00_JPRB, 0.28121E+00_JPRB, 0.28344E+00_JPRB, 0.28772E+00_JPRB, 0.29376E+00_JPRB, 0.29832E+00_JPRB ,&
&  0.30214E+00_JPRB, 0.30449E+00_JPRB, 0.30526E+00_JPRB, 0.30478E+00_JPRB, 0.30477E+00_JPRB, 0.30726E+00_JPRB, 0.30961E+00_JPRB ,&
&  0.31092E+00_JPRB, 0.31192E+00_JPRB, 0.31177E+00_JPRB, 0.31184E+00_JPRB, 0.30946E+00_JPRB, 0.30470E+00_JPRB, 0.29578E+00_JPRB ,&
&  0.28577E+00_JPRB, 0.27726E+00_JPRB, 0.27075E+00_JPRB, 0.26570E+00_JPRB, 0.26246E+00_JPRB, 0.26232E+00_JPRB, 0.26208E+00_JPRB ,&
&  0.26127E+00_JPRB, 0.26047E+00_JPRB, 0.26165E+00_JPRB, 0.26155E+00_JPRB, 0.25489E+00_JPRB, 0.25048E+00_JPRB, 0.24501E+00_JPRB ,&
&  0.24339E+00_JPRB, 0.24298E+00_JPRB, 0.24231E+00_JPRB, 0.24600E+00_JPRB, 0.25112E+00_JPRB, 0.25227E+00_JPRB, 0.25053E+00_JPRB ,&
&  0.24327E+00_JPRB, 0.23899E+00_JPRB, 0.23593E+00_JPRB, 0.23287E+00_JPRB, 0.23129E+00_JPRB, 0.23200E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23585E+00_JPRB, 0.23726E+00_JPRB, 0.24017E+00_JPRB, 0.24195E+00_JPRB, 0.24198E+00_JPRB, 0.24234E+00_JPRB, 0.24172E+00_JPRB ,&
&  0.24414E+00_JPRB, 0.25001E+00_JPRB, 0.25820E+00_JPRB, 0.25989E+00_JPRB, 0.26544E+00_JPRB, 0.26930E+00_JPRB, 0.27068E+00_JPRB ,&
&  0.27128E+00_JPRB, 0.27111E+00_JPRB, 0.27038E+00_JPRB, 0.27407E+00_JPRB, 0.27977E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.26837E+00_JPRB, 0.26961E+00_JPRB, 0.26989E+00_JPRB, 0.26947E+00_JPRB, 0.26888E+00_JPRB, 0.26894E+00_JPRB, 0.26918E+00_JPRB ,&
&  0.26886E+00_JPRB, 0.26785E+00_JPRB, 0.26639E+00_JPRB, 0.26386E+00_JPRB, 0.26070E+00_JPRB, 0.25879E+00_JPRB, 0.25992E+00_JPRB ,&
&  0.26194E+00_JPRB, 0.26367E+00_JPRB, 0.26556E+00_JPRB, 0.26680E+00_JPRB, 0.26764E+00_JPRB, 0.26886E+00_JPRB, 0.27053E+00_JPRB ,&
&  0.27174E+00_JPRB, 0.27215E+00_JPRB, 0.27334E+00_JPRB, 0.27417E+00_JPRB, 0.27617E+00_JPRB, 0.27790E+00_JPRB, 0.27783E+00_JPRB ,&
&  0.27321E+00_JPRB, 0.26516E+00_JPRB, 0.26059E+00_JPRB, 0.26356E+00_JPRB, 0.27060E+00_JPRB, 0.27724E+00_JPRB, 0.28180E+00_JPRB ,&
&  0.28453E+00_JPRB, 0.28537E+00_JPRB, 0.28534E+00_JPRB, 0.28665E+00_JPRB, 0.28912E+00_JPRB, 0.29189E+00_JPRB, 0.29191E+00_JPRB ,&
&  0.29023E+00_JPRB, 0.28893E+00_JPRB, 0.28701E+00_JPRB, 0.28305E+00_JPRB, 0.27703E+00_JPRB, 0.27080E+00_JPRB, 0.26499E+00_JPRB ,&
&  0.25944E+00_JPRB, 0.25415E+00_JPRB, 0.25004E+00_JPRB, 0.24746E+00_JPRB, 0.24689E+00_JPRB, 0.24866E+00_JPRB, 0.24788E+00_JPRB ,&
&  0.24888E+00_JPRB, 0.24670E+00_JPRB, 0.24436E+00_JPRB, 0.24116E+00_JPRB, 0.23830E+00_JPRB, 0.23291E+00_JPRB, 0.22597E+00_JPRB ,&
&  0.22266E+00_JPRB, 0.22616E+00_JPRB, 0.23481E+00_JPRB, 0.24063E+00_JPRB, 0.23459E+00_JPRB, 0.22955E+00_JPRB, 0.22724E+00_JPRB ,&
&  0.22333E+00_JPRB, 0.21842E+00_JPRB, 0.21506E+00_JPRB, 0.21419E+00_JPRB, 0.21435E+00_JPRB, 0.21452E+00_JPRB, 0.21575E+00_JPRB ,&
&  0.21780E+00_JPRB, 0.21956E+00_JPRB, 0.22048E+00_JPRB, 0.22177E+00_JPRB, 0.22232E+00_JPRB, 0.22240E+00_JPRB, 0.22249E+00_JPRB ,&
&  0.22480E+00_JPRB, 0.22947E+00_JPRB, 0.23533E+00_JPRB, 0.24166E+00_JPRB, 0.24624E+00_JPRB, 0.25041E+00_JPRB, 0.25337E+00_JPRB ,&
&  0.25571E+00_JPRB, 0.25521E+00_JPRB, 0.25825E+00_JPRB, 0.26285E+00_JPRB, 0.26612E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.25178E+00_JPRB, 0.25575E+00_JPRB, 0.25692E+00_JPRB, 0.25663E+00_JPRB, 0.25522E+00_JPRB, 0.25384E+00_JPRB, 0.25324E+00_JPRB ,&
&  0.25269E+00_JPRB, 0.25131E+00_JPRB, 0.25030E+00_JPRB, 0.24969E+00_JPRB, 0.24903E+00_JPRB, 0.24809E+00_JPRB, 0.24855E+00_JPRB ,&
&  0.25047E+00_JPRB, 0.25336E+00_JPRB, 0.25592E+00_JPRB, 0.25783E+00_JPRB, 0.25818E+00_JPRB, 0.25903E+00_JPRB, 0.26012E+00_JPRB ,&
&  0.26019E+00_JPRB, 0.25988E+00_JPRB, 0.26303E+00_JPRB, 0.26274E+00_JPRB, 0.26347E+00_JPRB, 0.26303E+00_JPRB, 0.26288E+00_JPRB ,&
&  0.25675E+00_JPRB, 0.25082E+00_JPRB, 0.24929E+00_JPRB, 0.25213E+00_JPRB, 0.26012E+00_JPRB, 0.26146E+00_JPRB, 0.26589E+00_JPRB ,&
&  0.26813E+00_JPRB, 0.26887E+00_JPRB, 0.27085E+00_JPRB, 0.27238E+00_JPRB, 0.27413E+00_JPRB, 0.27376E+00_JPRB, 0.27170E+00_JPRB ,&
&  0.26844E+00_JPRB, 0.26762E+00_JPRB, 0.26442E+00_JPRB, 0.24875E+00_JPRB, 0.23975E+00_JPRB, 0.23457E+00_JPRB, 0.22893E+00_JPRB ,&
&  0.22364E+00_JPRB, 0.22131E+00_JPRB, 0.22114E+00_JPRB, 0.21931E+00_JPRB, 0.22036E+00_JPRB, 0.22327E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22400E+00_JPRB, 0.22453E+00_JPRB, 0.22419E+00_JPRB, 0.22284E+00_JPRB, 0.22010E+00_JPRB, 0.21837E+00_JPRB, 0.21737E+00_JPRB ,&
&  0.21803E+00_JPRB, 0.21745E+00_JPRB, 0.22209E+00_JPRB, 0.22286E+00_JPRB, 0.21555E+00_JPRB, 0.21558E+00_JPRB, 0.21486E+00_JPRB ,&
&  0.20840E+00_JPRB, 0.20488E+00_JPRB, 0.20281E+00_JPRB, 0.20196E+00_JPRB, 0.20323E+00_JPRB, 0.20499E+00_JPRB, 0.20564E+00_JPRB ,&
&  0.20571E+00_JPRB, 0.20709E+00_JPRB, 0.20771E+00_JPRB, 0.20840E+00_JPRB, 0.20882E+00_JPRB, 0.20991E+00_JPRB, 0.21155E+00_JPRB ,&
&  0.21357E+00_JPRB, 0.21736E+00_JPRB, 0.22097E+00_JPRB, 0.22494E+00_JPRB, 0.22714E+00_JPRB, 0.22872E+00_JPRB, 0.23282E+00_JPRB ,&
&  0.23437E+00_JPRB, 0.24251E+00_JPRB, 0.25392E+00_JPRB, 0.25382E+00_JPRB, 0.24636E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.24296E+00_JPRB, 0.24455E+00_JPRB, 0.24550E+00_JPRB, 0.24433E+00_JPRB, 0.24444E+00_JPRB, 0.24408E+00_JPRB, 0.24395E+00_JPRB ,&
&  0.24366E+00_JPRB, 0.24282E+00_JPRB, 0.24310E+00_JPRB, 0.24056E+00_JPRB, 0.23912E+00_JPRB, 0.23622E+00_JPRB, 0.23607E+00_JPRB ,&
&  0.23492E+00_JPRB, 0.23550E+00_JPRB, 0.23647E+00_JPRB, 0.23862E+00_JPRB, 0.23930E+00_JPRB, 0.24114E+00_JPRB, 0.24001E+00_JPRB ,&
&  0.24053E+00_JPRB, 0.24109E+00_JPRB, 0.23962E+00_JPRB, 0.23922E+00_JPRB, 0.23937E+00_JPRB, 0.23930E+00_JPRB, 0.23946E+00_JPRB ,&
&  0.24208E+00_JPRB, 0.24447E+00_JPRB, 0.24683E+00_JPRB, 0.24785E+00_JPRB, 0.25259E+00_JPRB, 0.25398E+00_JPRB, 0.25671E+00_JPRB ,&
&  0.25772E+00_JPRB, 0.25883E+00_JPRB, 0.26006E+00_JPRB, 0.25923E+00_JPRB, 0.25786E+00_JPRB, 0.25312E+00_JPRB, 0.25164E+00_JPRB ,&
&  0.24160E+00_JPRB, 0.23454E+00_JPRB, 0.22763E+00_JPRB, 0.22110E+00_JPRB, 0.21595E+00_JPRB, 0.21245E+00_JPRB, 0.20967E+00_JPRB ,&
&  0.20744E+00_JPRB, 0.20515E+00_JPRB, 0.20254E+00_JPRB, 0.20285E+00_JPRB, 0.20383E+00_JPRB, 0.20536E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20893E+00_JPRB, 0.21001E+00_JPRB, 0.21042E+00_JPRB, 0.21126E+00_JPRB, 0.21126E+00_JPRB, 0.21149E+00_JPRB, 0.21160E+00_JPRB ,&
&  0.21316E+00_JPRB, 0.21349E+00_JPRB, 0.21342E+00_JPRB, 0.21558E+00_JPRB, 0.21327E+00_JPRB, 0.20994E+00_JPRB, 0.20661E+00_JPRB ,&
&  0.20289E+00_JPRB, 0.20061E+00_JPRB, 0.19858E+00_JPRB, 0.19779E+00_JPRB, 0.19830E+00_JPRB, 0.19926E+00_JPRB, 0.20040E+00_JPRB ,&
&  0.20048E+00_JPRB, 0.20122E+00_JPRB, 0.20177E+00_JPRB, 0.20287E+00_JPRB, 0.20403E+00_JPRB, 0.20540E+00_JPRB, 0.20696E+00_JPRB ,&
&  0.20932E+00_JPRB, 0.21206E+00_JPRB, 0.21471E+00_JPRB, 0.21739E+00_JPRB, 0.22174E+00_JPRB, 0.22655E+00_JPRB, 0.23402E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.24371E+00_JPRB, 0.23959E+00_JPRB, 0.24452E+00_JPRB, 0.24501E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.23685E+00_JPRB, 0.23749E+00_JPRB, 0.23495E+00_JPRB, 0.23496E+00_JPRB, 0.23551E+00_JPRB, 0.23571E+00_JPRB, 0.23617E+00_JPRB ,&
&  0.23337E+00_JPRB, 0.23284E+00_JPRB, 0.23056E+00_JPRB, 0.22899E+00_JPRB, 0.22669E+00_JPRB, 0.22451E+00_JPRB, 0.22298E+00_JPRB ,&
&  0.22104E+00_JPRB, 0.22070E+00_JPRB, 0.21990E+00_JPRB, 0.22145E+00_JPRB, 0.22170E+00_JPRB, 0.22395E+00_JPRB, 0.22569E+00_JPRB ,&
&  0.22647E+00_JPRB, 0.22771E+00_JPRB, 0.22583E+00_JPRB, 0.22493E+00_JPRB, 0.22658E+00_JPRB, 0.22780E+00_JPRB, 0.23045E+00_JPRB ,&
&  0.23304E+00_JPRB, 0.23656E+00_JPRB, 0.24253E+00_JPRB, 0.24452E+00_JPRB, 0.24731E+00_JPRB, 0.24891E+00_JPRB, 0.24864E+00_JPRB ,&
&  0.24824E+00_JPRB, 0.24743E+00_JPRB, 0.24814E+00_JPRB, 0.24786E+00_JPRB, 0.24036E+00_JPRB, 0.23490E+00_JPRB, 0.22897E+00_JPRB ,&
&  0.22379E+00_JPRB, 0.21863E+00_JPRB, 0.21390E+00_JPRB, 0.21074E+00_JPRB, 0.20697E+00_JPRB, 0.20442E+00_JPRB, 0.20240E+00_JPRB ,&
&  0.20045E+00_JPRB, 0.19946E+00_JPRB, 0.19797E+00_JPRB, 0.19753E+00_JPRB, 0.19799E+00_JPRB, 0.19905E+00_JPRB, 0.20062E+00_JPRB ,&
&  0.20247E+00_JPRB, 0.20370E+00_JPRB, 0.20524E+00_JPRB, 0.20661E+00_JPRB, 0.20757E+00_JPRB, 0.20806E+00_JPRB, 0.20752E+00_JPRB ,&
&  0.20768E+00_JPRB, 0.20801E+00_JPRB, 0.20747E+00_JPRB, 0.20757E+00_JPRB, 0.20620E+00_JPRB, 0.20457E+00_JPRB, 0.20293E+00_JPRB ,&
&  0.20054E+00_JPRB, 0.19871E+00_JPRB, 0.19810E+00_JPRB, 0.19777E+00_JPRB, 0.19871E+00_JPRB, 0.20010E+00_JPRB, 0.20094E+00_JPRB ,&
&  0.20126E+00_JPRB, 0.20104E+00_JPRB, 0.20094E+00_JPRB, 0.20242E+00_JPRB, 0.20397E+00_JPRB, 0.20595E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20864E+00_JPRB, 0.21030E+00_JPRB, 0.21169E+00_JPRB, 0.21355E+00_JPRB, 0.22005E+00_JPRB, 0.22466E+00_JPRB, 0.23352E+00_JPRB ,&
&  0.23613E+00_JPRB, 0.23267E+00_JPRB, 0.22877E+00_JPRB, 0.23511E+00_JPRB, 0.23651E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.23536E+00_JPRB, 0.23800E+00_JPRB, 0.23613E+00_JPRB, 0.23561E+00_JPRB, 0.23451E+00_JPRB, 0.23398E+00_JPRB, 0.23224E+00_JPRB ,&
&  0.23198E+00_JPRB, 0.23046E+00_JPRB, 0.23036E+00_JPRB, 0.22872E+00_JPRB, 0.22710E+00_JPRB, 0.22467E+00_JPRB, 0.22224E+00_JPRB ,&
&  0.21993E+00_JPRB, 0.21797E+00_JPRB, 0.21673E+00_JPRB, 0.21649E+00_JPRB, 0.21555E+00_JPRB, 0.21611E+00_JPRB, 0.21937E+00_JPRB ,&
&  0.22014E+00_JPRB, 0.22158E+00_JPRB, 0.22148E+00_JPRB, 0.22271E+00_JPRB, 0.22437E+00_JPRB, 0.22438E+00_JPRB, 0.22653E+00_JPRB ,&
&  0.22655E+00_JPRB, 0.22822E+00_JPRB, 0.23595E+00_JPRB, 0.23982E+00_JPRB, 0.24165E+00_JPRB, 0.24488E+00_JPRB, 0.24592E+00_JPRB ,&
&  0.24391E+00_JPRB, 0.23731E+00_JPRB, 0.24021E+00_JPRB, 0.23637E+00_JPRB, 0.22913E+00_JPRB, 0.22659E+00_JPRB, 0.22088E+00_JPRB ,&
&  0.21832E+00_JPRB, 0.21453E+00_JPRB, 0.21176E+00_JPRB, 0.20961E+00_JPRB, 0.20586E+00_JPRB, 0.20358E+00_JPRB, 0.20095E+00_JPRB ,&
&  0.19859E+00_JPRB, 0.19776E+00_JPRB, 0.19619E+00_JPRB, 0.19622E+00_JPRB, 0.19660E+00_JPRB, 0.19728E+00_JPRB, 0.19839E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.20096E+00_JPRB, 0.20207E+00_JPRB, 0.20298E+00_JPRB, 0.20375E+00_JPRB, 0.20415E+00_JPRB, 0.20389E+00_JPRB ,&
&  0.20393E+00_JPRB, 0.20376E+00_JPRB, 0.20314E+00_JPRB, 0.20295E+00_JPRB, 0.20180E+00_JPRB, 0.20138E+00_JPRB, 0.20091E+00_JPRB ,&
&  0.20071E+00_JPRB, 0.20075E+00_JPRB, 0.20062E+00_JPRB, 0.20085E+00_JPRB, 0.20114E+00_JPRB, 0.20197E+00_JPRB, 0.20257E+00_JPRB ,&
&  0.20305E+00_JPRB, 0.20414E+00_JPRB, 0.20479E+00_JPRB, 0.20572E+00_JPRB, 0.20703E+00_JPRB, 0.20753E+00_JPRB, 0.20904E+00_JPRB ,&
&  0.21029E+00_JPRB, 0.21180E+00_JPRB, 0.21466E+00_JPRB, 0.21745E+00_JPRB, 0.22154E+00_JPRB, 0.22630E+00_JPRB, 0.22736E+00_JPRB ,&
&  0.22785E+00_JPRB, 0.22682E+00_JPRB, 0.22353E+00_JPRB, 0.22966E+00_JPRB, 0.23258E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.22758E+00_JPRB, 0.22966E+00_JPRB, 0.23101E+00_JPRB, 0.23194E+00_JPRB, 0.23362E+00_JPRB, 0.23441E+00_JPRB, 0.23508E+00_JPRB ,&
&  0.23299E+00_JPRB, 0.23254E+00_JPRB, 0.23156E+00_JPRB, 0.23029E+00_JPRB, 0.22950E+00_JPRB, 0.22668E+00_JPRB, 0.22405E+00_JPRB ,&
&  0.22125E+00_JPRB, 0.21882E+00_JPRB, 0.21763E+00_JPRB, 0.21647E+00_JPRB, 0.21521E+00_JPRB, 0.21417E+00_JPRB, 0.21726E+00_JPRB ,&
&  0.21919E+00_JPRB, 0.22033E+00_JPRB, 0.22200E+00_JPRB, 0.22131E+00_JPRB, 0.22272E+00_JPRB, 0.22497E+00_JPRB, 0.22806E+00_JPRB ,&
&  0.22952E+00_JPRB, 0.23264E+00_JPRB, 0.23441E+00_JPRB, 0.23529E+00_JPRB, 0.23665E+00_JPRB, 0.23770E+00_JPRB, 0.23746E+00_JPRB ,&
&  0.23594E+00_JPRB, 0.23393E+00_JPRB, 0.23144E+00_JPRB, 0.22816E+00_JPRB, 0.22480E+00_JPRB, 0.22164E+00_JPRB, 0.21847E+00_JPRB ,&
&  0.21629E+00_JPRB, 0.21429E+00_JPRB, 0.21205E+00_JPRB, 0.21001E+00_JPRB, 0.20804E+00_JPRB, 0.20626E+00_JPRB, 0.20445E+00_JPRB ,&
&  0.20305E+00_JPRB, 0.20193E+00_JPRB, 0.20050E+00_JPRB, 0.19986E+00_JPRB, 0.19923E+00_JPRB, 0.19902E+00_JPRB, 0.19897E+00_JPRB ,&
&  0.19961E+00_JPRB, 0.20035E+00_JPRB, 0.20105E+00_JPRB, 0.20145E+00_JPRB, 0.20184E+00_JPRB, 0.20201E+00_JPRB, 0.20236E+00_JPRB ,&
&  0.20269E+00_JPRB, 0.20306E+00_JPRB, 0.20252E+00_JPRB, 0.20203E+00_JPRB, 0.20139E+00_JPRB, 0.20066E+00_JPRB, 0.20044E+00_JPRB ,&
&  0.20079E+00_JPRB, 0.20108E+00_JPRB, 0.20079E+00_JPRB, 0.20111E+00_JPRB, 0.20172E+00_JPRB, 0.20234E+00_JPRB, 0.20400E+00_JPRB ,&
&  0.20488E+00_JPRB, 0.20572E+00_JPRB, 0.20651E+00_JPRB, 0.20713E+00_JPRB, 0.20810E+00_JPRB, 0.20887E+00_JPRB, 0.20985E+00_JPRB ,&
&  0.21103E+00_JPRB, 0.21213E+00_JPRB, 0.21364E+00_JPRB, 0.21561E+00_JPRB, 0.22051E+00_JPRB, 0.22339E+00_JPRB, 0.22556E+00_JPRB ,&
&  0.22598E+00_JPRB, 0.22003E+00_JPRB, 0.22718E+00_JPRB, 0.22532E+00_JPRB, 0.22458E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.22175E+00_JPRB, 0.22350E+00_JPRB, 0.22470E+00_JPRB, 0.22644E+00_JPRB, 0.21973E+00_JPRB, 0.22646E+00_JPRB, 0.22262E+00_JPRB ,&
&  0.21870E+00_JPRB, 0.21371E+00_JPRB, 0.21439E+00_JPRB, 0.21390E+00_JPRB, 0.21299E+00_JPRB, 0.21286E+00_JPRB, 0.21323E+00_JPRB ,&
&  0.21375E+00_JPRB, 0.21392E+00_JPRB, 0.21491E+00_JPRB, 0.21567E+00_JPRB, 0.21629E+00_JPRB, 0.21680E+00_JPRB, 0.21745E+00_JPRB ,&
&  0.21835E+00_JPRB, 0.21972E+00_JPRB, 0.22112E+00_JPRB, 0.21856E+00_JPRB, 0.21819E+00_JPRB, 0.21836E+00_JPRB, 0.21896E+00_JPRB ,&
&  0.21914E+00_JPRB, 0.22035E+00_JPRB, 0.22136E+00_JPRB, 0.22216E+00_JPRB, 0.22461E+00_JPRB, 0.22488E+00_JPRB, 0.22480E+00_JPRB ,&
&  0.22438E+00_JPRB, 0.22340E+00_JPRB, 0.22203E+00_JPRB, 0.22075E+00_JPRB, 0.21943E+00_JPRB, 0.21766E+00_JPRB, 0.21597E+00_JPRB ,&
&  0.21459E+00_JPRB, 0.21348E+00_JPRB, 0.21200E+00_JPRB, 0.21056E+00_JPRB, 0.20927E+00_JPRB, 0.20805E+00_JPRB, 0.20701E+00_JPRB ,&
&  0.20626E+00_JPRB, 0.20562E+00_JPRB, 0.20493E+00_JPRB, 0.20441E+00_JPRB, 0.20384E+00_JPRB, 0.20331E+00_JPRB, 0.20280E+00_JPRB ,&
&  0.20302E+00_JPRB, 0.20316E+00_JPRB, 0.20331E+00_JPRB, 0.20348E+00_JPRB, 0.20405E+00_JPRB, 0.20417E+00_JPRB, 0.20433E+00_JPRB ,&
&  0.20444E+00_JPRB, 0.20433E+00_JPRB, 0.20416E+00_JPRB, 0.20387E+00_JPRB, 0.20355E+00_JPRB, 0.20314E+00_JPRB, 0.20295E+00_JPRB ,&
&  0.20293E+00_JPRB, 0.20306E+00_JPRB, 0.20349E+00_JPRB, 0.20379E+00_JPRB, 0.20409E+00_JPRB, 0.20435E+00_JPRB, 0.20495E+00_JPRB ,&
&  0.20554E+00_JPRB, 0.20612E+00_JPRB, 0.20667E+00_JPRB, 0.20648E+00_JPRB, 0.20677E+00_JPRB, 0.20704E+00_JPRB, 0.20730E+00_JPRB ,&
&  0.20745E+00_JPRB, 0.20801E+00_JPRB, 0.20893E+00_JPRB, 0.21023E+00_JPRB, 0.21270E+00_JPRB, 0.21468E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21846E+00_JPRB, 0.22251E+00_JPRB, 0.22379E+00_JPRB, 0.22393E+00_JPRB, 0.22348E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.21139E+00_JPRB, 0.21242E+00_JPRB, 0.21311E+00_JPRB, 0.21362E+00_JPRB, 0.21329E+00_JPRB, 0.21329E+00_JPRB, 0.21326E+00_JPRB ,&
&  0.21311E+00_JPRB, 0.21296E+00_JPRB, 0.21328E+00_JPRB, 0.21355E+00_JPRB, 0.21388E+00_JPRB, 0.21424E+00_JPRB, 0.21473E+00_JPRB ,&
&  0.21491E+00_JPRB, 0.21535E+00_JPRB, 0.21370E+00_JPRB, 0.21232E+00_JPRB, 0.21138E+00_JPRB, 0.21078E+00_JPRB, 0.21044E+00_JPRB ,&
&  0.21019E+00_JPRB, 0.21004E+00_JPRB, 0.20988E+00_JPRB, 0.21110E+00_JPRB, 0.21188E+00_JPRB, 0.21256E+00_JPRB, 0.21308E+00_JPRB ,&
&  0.21327E+00_JPRB, 0.21359E+00_JPRB, 0.21387E+00_JPRB, 0.21415E+00_JPRB, 0.21449E+00_JPRB, 0.21451E+00_JPRB, 0.21441E+00_JPRB ,&
&  0.21421E+00_JPRB, 0.21385E+00_JPRB, 0.21343E+00_JPRB, 0.21302E+00_JPRB, 0.21269E+00_JPRB, 0.21228E+00_JPRB, 0.21184E+00_JPRB ,&
&  0.21142E+00_JPRB, 0.21093E+00_JPRB, 0.21047E+00_JPRB, 0.21002E+00_JPRB, 0.20959E+00_JPRB, 0.20923E+00_JPRB, 0.20883E+00_JPRB ,&
&  0.20854E+00_JPRB, 0.20829E+00_JPRB, 0.20802E+00_JPRB, 0.20782E+00_JPRB, 0.20749E+00_JPRB, 0.20738E+00_JPRB, 0.20709E+00_JPRB ,&
&  0.20699E+00_JPRB, 0.20688E+00_JPRB, 0.20681E+00_JPRB, 0.20672E+00_JPRB, 0.20673E+00_JPRB, 0.20671E+00_JPRB, 0.20671E+00_JPRB ,&
&  0.20669E+00_JPRB, 0.20689E+00_JPRB, 0.20675E+00_JPRB, 0.20656E+00_JPRB, 0.20636E+00_JPRB, 0.20616E+00_JPRB, 0.20598E+00_JPRB ,&
&  0.20581E+00_JPRB, 0.20563E+00_JPRB, 0.20552E+00_JPRB, 0.20557E+00_JPRB, 0.20564E+00_JPRB, 0.20574E+00_JPRB, 0.20585E+00_JPRB ,&
&  0.20596E+00_JPRB, 0.20610E+00_JPRB, 0.20623E+00_JPRB, 0.20616E+00_JPRB, 0.20618E+00_JPRB, 0.20620E+00_JPRB, 0.20623E+00_JPRB ,&
&  0.20624E+00_JPRB, 0.20630E+00_JPRB, 0.20641E+00_JPRB, 0.20656E+00_JPRB, 0.20693E+00_JPRB, 0.20706E+00_JPRB, 0.20726E+00_JPRB ,&
&  0.20753E+00_JPRB, 0.20803E+00_JPRB, 0.20846E+00_JPRB, 0.20893E+00_JPRB, 0.20945E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB, 0.20805E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB, 0.84996E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.84646E+00_JPRB, 0.84541E+00_JPRB, 0.84447E+00_JPRB, 0.84357E+00_JPRB, 0.84261E+00_JPRB, 0.84196E+00_JPRB, 0.84146E+00_JPRB ,&
&  0.84094E+00_JPRB, 0.84050E+00_JPRB, 0.84000E+00_JPRB, 0.83963E+00_JPRB, 0.83941E+00_JPRB, 0.83905E+00_JPRB, 0.83870E+00_JPRB ,&
&  0.83821E+00_JPRB, 0.83774E+00_JPRB, 0.83744E+00_JPRB, 0.83672E+00_JPRB, 0.83604E+00_JPRB, 0.83542E+00_JPRB, 0.83433E+00_JPRB ,&
&  0.83283E+00_JPRB, 0.83134E+00_JPRB, 0.82973E+00_JPRB, 0.82837E+00_JPRB, 0.82732E+00_JPRB, 0.82582E+00_JPRB, 0.82407E+00_JPRB ,&
&  0.82256E+00_JPRB, 0.82154E+00_JPRB, 0.82086E+00_JPRB, 0.82041E+00_JPRB, 0.82013E+00_JPRB, 0.82002E+00_JPRB, 0.81989E+00_JPRB ,&
&  0.81989E+00_JPRB, 0.82028E+00_JPRB, 0.82112E+00_JPRB, 0.82230E+00_JPRB, 0.82377E+00_JPRB, 0.82554E+00_JPRB, 0.82807E+00_JPRB ,&
&  0.83081E+00_JPRB, 0.83363E+00_JPRB, 0.83659E+00_JPRB, 0.83927E+00_JPRB, 0.84169E+00_JPRB, 0.84383E+00_JPRB, 0.84546E+00_JPRB ,&
&  0.84724E+00_JPRB, 0.84909E+00_JPRB, 0.85099E+00_JPRB, 0.85244E+00_JPRB, 0.85371E+00_JPRB, 0.85489E+00_JPRB, 0.85619E+00_JPRB ,&
&  0.85692E+00_JPRB, 0.85745E+00_JPRB, 0.85808E+00_JPRB, 0.85891E+00_JPRB, 0.85950E+00_JPRB, 0.85991E+00_JPRB, 0.86036E+00_JPRB ,&
&  0.86102E+00_JPRB, 0.86152E+00_JPRB, 0.86177E+00_JPRB, 0.86204E+00_JPRB, 0.86254E+00_JPRB, 0.86324E+00_JPRB, 0.86396E+00_JPRB ,&
&  0.86458E+00_JPRB, 0.86517E+00_JPRB, 0.86556E+00_JPRB, 0.86572E+00_JPRB, 0.86559E+00_JPRB, 0.86509E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.86251E+00_JPRB, 0.86095E+00_JPRB, 0.85963E+00_JPRB, 0.85877E+00_JPRB, 0.85786E+00_JPRB, 0.85708E+00_JPRB, 0.85652E+00_JPRB ,&
&  0.85630E+00_JPRB, 0.85617E+00_JPRB, 0.85602E+00_JPRB, 0.85600E+00_JPRB, 0.85586E+00_JPRB, 0.85543E+00_JPRB, 0.85479E+00_JPRB ,&
&  0.85394E+00_JPRB, 0.85275E+00_JPRB, 0.85129E+00_JPRB, 0.84965E+00_JPRB, 0.84797E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.83627E+00_JPRB, 0.83368E+00_JPRB, 0.83182E+00_JPRB, 0.83057E+00_JPRB, 0.82949E+00_JPRB, 0.82847E+00_JPRB, 0.82739E+00_JPRB ,&
&  0.82632E+00_JPRB, 0.82640E+00_JPRB, 0.82603E+00_JPRB, 0.82582E+00_JPRB, 0.82556E+00_JPRB, 0.82603E+00_JPRB, 0.82615E+00_JPRB ,&
&  0.82586E+00_JPRB, 0.82446E+00_JPRB, 0.82345E+00_JPRB, 0.82156E+00_JPRB, 0.82138E+00_JPRB, 0.82115E+00_JPRB, 0.82249E+00_JPRB ,&
&  0.82168E+00_JPRB, 0.82050E+00_JPRB, 0.81824E+00_JPRB, 0.81197E+00_JPRB, 0.80673E+00_JPRB, 0.80157E+00_JPRB, 0.79709E+00_JPRB ,&
&  0.79276E+00_JPRB, 0.78950E+00_JPRB, 0.78812E+00_JPRB, 0.78789E+00_JPRB, 0.79176E+00_JPRB, 0.79293E+00_JPRB, 0.79339E+00_JPRB ,&
&  0.79242E+00_JPRB, 0.78761E+00_JPRB, 0.78884E+00_JPRB, 0.79042E+00_JPRB, 0.79276E+00_JPRB, 0.79809E+00_JPRB, 0.80278E+00_JPRB ,&
&  0.80820E+00_JPRB, 0.81431E+00_JPRB, 0.82410E+00_JPRB, 0.83110E+00_JPRB, 0.83799E+00_JPRB, 0.84387E+00_JPRB, 0.84692E+00_JPRB ,&
&  0.85003E+00_JPRB, 0.85279E+00_JPRB, 0.85561E+00_JPRB, 0.85783E+00_JPRB, 0.85954E+00_JPRB, 0.86142E+00_JPRB, 0.86363E+00_JPRB ,&
&  0.86512E+00_JPRB, 0.86634E+00_JPRB, 0.86770E+00_JPRB, 0.86906E+00_JPRB, 0.86995E+00_JPRB, 0.87086E+00_JPRB, 0.87197E+00_JPRB ,&
&  0.87311E+00_JPRB, 0.87301E+00_JPRB, 0.87291E+00_JPRB, 0.87365E+00_JPRB, 0.87446E+00_JPRB, 0.87527E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.87527E+00_JPRB, 0.87494E+00_JPRB, 0.87494E+00_JPRB, 0.87509E+00_JPRB, 0.87482E+00_JPRB, 0.87452E+00_JPRB, 0.87347E+00_JPRB ,&
&  0.87201E+00_JPRB, 0.86986E+00_JPRB, 0.86796E+00_JPRB, 0.86703E+00_JPRB, 0.86598E+00_JPRB, 0.86576E+00_JPRB, 0.86594E+00_JPRB ,&
&  0.86665E+00_JPRB, 0.86552E+00_JPRB, 0.86402E+00_JPRB, 0.86189E+00_JPRB, 0.85973E+00_JPRB, 0.85728E+00_JPRB, 0.85493E+00_JPRB ,&
&  0.85256E+00_JPRB, 0.84982E+00_JPRB, 0.84651E+00_JPRB, 0.84298E+00_JPRB, 0.83953E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.83854E+00_JPRB, 0.83390E+00_JPRB, 0.83058E+00_JPRB, 0.82843E+00_JPRB, 0.82645E+00_JPRB, 0.82451E+00_JPRB, 0.82313E+00_JPRB ,&
&  0.82162E+00_JPRB, 0.81903E+00_JPRB, 0.81761E+00_JPRB, 0.81658E+00_JPRB, 0.81511E+00_JPRB, 0.81568E+00_JPRB, 0.81580E+00_JPRB ,&
&  0.81657E+00_JPRB, 0.81606E+00_JPRB, 0.81849E+00_JPRB, 0.81708E+00_JPRB, 0.81587E+00_JPRB, 0.81421E+00_JPRB, 0.80970E+00_JPRB ,&
&  0.80841E+00_JPRB, 0.80691E+00_JPRB, 0.80194E+00_JPRB, 0.79569E+00_JPRB, 0.79214E+00_JPRB, 0.77307E+00_JPRB, 0.77455E+00_JPRB ,&
&  0.78389E+00_JPRB, 0.76984E+00_JPRB, 0.76232E+00_JPRB, 0.75458E+00_JPRB, 0.74291E+00_JPRB, 0.74289E+00_JPRB, 0.74806E+00_JPRB ,&
&  0.74249E+00_JPRB, 0.76254E+00_JPRB, 0.76556E+00_JPRB, 0.76747E+00_JPRB, 0.76967E+00_JPRB, 0.76636E+00_JPRB, 0.77184E+00_JPRB ,&
&  0.79343E+00_JPRB, 0.79733E+00_JPRB, 0.80705E+00_JPRB, 0.81574E+00_JPRB, 0.82673E+00_JPRB, 0.83686E+00_JPRB, 0.84816E+00_JPRB ,&
&  0.85658E+00_JPRB, 0.86257E+00_JPRB, 0.86688E+00_JPRB, 0.86784E+00_JPRB, 0.86883E+00_JPRB, 0.86908E+00_JPRB, 0.87061E+00_JPRB ,&
&  0.87070E+00_JPRB, 0.87221E+00_JPRB, 0.87475E+00_JPRB, 0.87772E+00_JPRB, 0.88170E+00_JPRB, 0.88416E+00_JPRB, 0.88656E+00_JPRB ,&
&  0.88811E+00_JPRB, 0.88845E+00_JPRB, 0.88809E+00_JPRB, 0.88796E+00_JPRB, 0.88665E+00_JPRB, 0.88590E+00_JPRB, 0.88514E+00_JPRB ,&
&  0.88552E+00_JPRB, 0.88518E+00_JPRB, 0.88572E+00_JPRB, 0.88610E+00_JPRB, 0.88729E+00_JPRB, 0.88955E+00_JPRB, 0.89126E+00_JPRB ,&
&  0.89386E+00_JPRB, 0.89501E+00_JPRB, 0.89550E+00_JPRB, 0.89526E+00_JPRB, 0.89476E+00_JPRB, 0.89522E+00_JPRB, 0.89546E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.89276E+00_JPRB, 0.88831E+00_JPRB, 0.88346E+00_JPRB, 0.87923E+00_JPRB, 0.87540E+00_JPRB, 0.87133E+00_JPRB ,&
&  0.86758E+00_JPRB, 0.86250E+00_JPRB, 0.85682E+00_JPRB, 0.85034E+00_JPRB, 0.84400E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.85671E+00_JPRB, 0.84953E+00_JPRB, 0.84303E+00_JPRB, 0.83834E+00_JPRB, 0.83387E+00_JPRB, 0.83010E+00_JPRB, 0.82679E+00_JPRB ,&
&  0.82383E+00_JPRB, 0.82130E+00_JPRB, 0.81966E+00_JPRB, 0.81789E+00_JPRB, 0.81662E+00_JPRB, 0.81500E+00_JPRB, 0.81420E+00_JPRB ,&
&  0.81618E+00_JPRB, 0.81608E+00_JPRB, 0.81865E+00_JPRB, 0.81662E+00_JPRB, 0.81489E+00_JPRB, 0.81039E+00_JPRB, 0.79920E+00_JPRB ,&
&  0.79384E+00_JPRB, 0.78920E+00_JPRB, 0.77379E+00_JPRB, 0.78809E+00_JPRB, 0.78159E+00_JPRB, 0.77714E+00_JPRB, 0.76986E+00_JPRB ,&
&  0.75366E+00_JPRB, 0.74577E+00_JPRB, 0.73589E+00_JPRB, 0.73558E+00_JPRB, 0.74223E+00_JPRB, 0.74000E+00_JPRB, 0.74534E+00_JPRB ,&
&  0.74098E+00_JPRB, 0.73951E+00_JPRB, 0.74072E+00_JPRB, 0.74082E+00_JPRB, 0.74656E+00_JPRB, 0.74725E+00_JPRB, 0.75167E+00_JPRB ,&
&  0.76163E+00_JPRB, 0.77545E+00_JPRB, 0.79528E+00_JPRB, 0.80227E+00_JPRB, 0.81970E+00_JPRB, 0.83390E+00_JPRB, 0.84705E+00_JPRB ,&
&  0.85570E+00_JPRB, 0.86096E+00_JPRB, 0.86481E+00_JPRB, 0.86651E+00_JPRB, 0.86653E+00_JPRB, 0.86327E+00_JPRB, 0.86347E+00_JPRB ,&
&  0.86451E+00_JPRB, 0.86682E+00_JPRB, 0.86977E+00_JPRB, 0.87253E+00_JPRB, 0.87653E+00_JPRB, 0.88013E+00_JPRB, 0.88486E+00_JPRB ,&
&  0.88901E+00_JPRB, 0.89318E+00_JPRB, 0.89418E+00_JPRB, 0.89592E+00_JPRB, 0.89698E+00_JPRB, 0.89781E+00_JPRB, 0.89877E+00_JPRB ,&
&  0.89993E+00_JPRB, 0.90185E+00_JPRB, 0.90572E+00_JPRB, 0.90305E+00_JPRB, 0.90564E+00_JPRB, 0.91804E+00_JPRB, 0.92115E+00_JPRB ,&
&  0.92532E+00_JPRB, 0.92591E+00_JPRB, 0.92392E+00_JPRB, 0.92174E+00_JPRB, 0.91962E+00_JPRB, 0.91907E+00_JPRB, 0.91881E+00_JPRB ,&
&  0.91837E+00_JPRB, 0.91297E+00_JPRB, 0.90732E+00_JPRB, 0.90233E+00_JPRB, 0.89699E+00_JPRB, 0.89297E+00_JPRB, 0.88886E+00_JPRB ,&
&  0.88601E+00_JPRB, 0.88350E+00_JPRB, 0.88016E+00_JPRB, 0.87402E+00_JPRB, 0.86600E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.87878E+00_JPRB, 0.87030E+00_JPRB, 0.86340E+00_JPRB, 0.85754E+00_JPRB, 0.85292E+00_JPRB, 0.84896E+00_JPRB, 0.84507E+00_JPRB ,&
&  0.84136E+00_JPRB, 0.83693E+00_JPRB, 0.83241E+00_JPRB, 0.82897E+00_JPRB, 0.82626E+00_JPRB, 0.82370E+00_JPRB, 0.82099E+00_JPRB ,&
&  0.81970E+00_JPRB, 0.81809E+00_JPRB, 0.81952E+00_JPRB, 0.81686E+00_JPRB, 0.81829E+00_JPRB, 0.81346E+00_JPRB, 0.80093E+00_JPRB ,&
&  0.78868E+00_JPRB, 0.77576E+00_JPRB, 0.76405E+00_JPRB, 0.77704E+00_JPRB, 0.77604E+00_JPRB, 0.77603E+00_JPRB, 0.75869E+00_JPRB ,&
&  0.74207E+00_JPRB, 0.72848E+00_JPRB, 0.75338E+00_JPRB, 0.74502E+00_JPRB, 0.74072E+00_JPRB, 0.73234E+00_JPRB, 0.71302E+00_JPRB ,&
&  0.70999E+00_JPRB, 0.69850E+00_JPRB, 0.69778E+00_JPRB, 0.70291E+00_JPRB, 0.70972E+00_JPRB, 0.71956E+00_JPRB, 0.72920E+00_JPRB ,&
&  0.72816E+00_JPRB, 0.76238E+00_JPRB, 0.78633E+00_JPRB, 0.80431E+00_JPRB, 0.82213E+00_JPRB, 0.83680E+00_JPRB, 0.84680E+00_JPRB ,&
&  0.85596E+00_JPRB, 0.86274E+00_JPRB, 0.86925E+00_JPRB, 0.87158E+00_JPRB, 0.88215E+00_JPRB, 0.88476E+00_JPRB, 0.88849E+00_JPRB ,&
&  0.89280E+00_JPRB, 0.89290E+00_JPRB, 0.89174E+00_JPRB, 0.88843E+00_JPRB, 0.88711E+00_JPRB, 0.88483E+00_JPRB, 0.88625E+00_JPRB ,&
&  0.88686E+00_JPRB, 0.88851E+00_JPRB, 0.89363E+00_JPRB, 0.89828E+00_JPRB, 0.90259E+00_JPRB, 0.90662E+00_JPRB, 0.91119E+00_JPRB ,&
&  0.91553E+00_JPRB, 0.92379E+00_JPRB, 0.93175E+00_JPRB, 0.94600E+00_JPRB, 0.95853E+00_JPRB, 0.96505E+00_JPRB, 0.96994E+00_JPRB ,&
&  0.97569E+00_JPRB, 0.96962E+00_JPRB, 0.96374E+00_JPRB, 0.95984E+00_JPRB, 0.95768E+00_JPRB, 0.95669E+00_JPRB, 0.95713E+00_JPRB ,&
&  0.95616E+00_JPRB, 0.95120E+00_JPRB, 0.94517E+00_JPRB, 0.93807E+00_JPRB, 0.93109E+00_JPRB, 0.92564E+00_JPRB, 0.92108E+00_JPRB ,&
&  0.91699E+00_JPRB, 0.91260E+00_JPRB, 0.90538E+00_JPRB, 0.89651E+00_JPRB, 0.88745E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.90872E+00_JPRB, 0.89984E+00_JPRB, 0.89284E+00_JPRB, 0.88667E+00_JPRB, 0.88212E+00_JPRB, 0.87826E+00_JPRB, 0.87483E+00_JPRB ,&
&  0.87209E+00_JPRB, 0.86657E+00_JPRB, 0.86010E+00_JPRB, 0.85215E+00_JPRB, 0.84653E+00_JPRB, 0.84148E+00_JPRB, 0.83592E+00_JPRB ,&
&  0.82835E+00_JPRB, 0.82025E+00_JPRB, 0.81175E+00_JPRB, 0.80500E+00_JPRB, 0.80682E+00_JPRB, 0.80296E+00_JPRB, 0.79730E+00_JPRB ,&
&  0.78578E+00_JPRB, 0.77189E+00_JPRB, 0.76174E+00_JPRB, 0.76671E+00_JPRB, 0.77018E+00_JPRB, 0.75916E+00_JPRB, 0.75626E+00_JPRB ,&
&  0.77028E+00_JPRB, 0.77185E+00_JPRB, 0.76308E+00_JPRB, 0.75899E+00_JPRB, 0.72776E+00_JPRB, 0.68639E+00_JPRB, 0.71316E+00_JPRB ,&
&  0.71373E+00_JPRB, 0.69399E+00_JPRB, 0.69075E+00_JPRB, 0.68851E+00_JPRB, 0.69677E+00_JPRB, 0.71464E+00_JPRB, 0.73098E+00_JPRB ,&
&  0.73420E+00_JPRB, 0.79310E+00_JPRB, 0.81923E+00_JPRB, 0.83793E+00_JPRB, 0.85067E+00_JPRB, 0.86109E+00_JPRB, 0.86918E+00_JPRB ,&
&  0.87725E+00_JPRB, 0.88967E+00_JPRB, 0.90618E+00_JPRB, 0.92265E+00_JPRB, 0.93875E+00_JPRB, 0.95040E+00_JPRB, 0.96253E+00_JPRB ,&
&  0.97793E+00_JPRB, 0.97998E+00_JPRB, 0.96641E+00_JPRB, 0.95134E+00_JPRB, 0.93771E+00_JPRB, 0.92517E+00_JPRB, 0.91404E+00_JPRB ,&
&  0.90928E+00_JPRB, 0.90615E+00_JPRB, 0.91248E+00_JPRB, 0.91374E+00_JPRB, 0.91752E+00_JPRB, 0.92287E+00_JPRB, 0.93620E+00_JPRB ,&
&  0.94511E+00_JPRB, 0.95823E+00_JPRB, 0.97387E+00_JPRB, 0.99450E+00_JPRB, 0.10112E+01_JPRB, 0.10193E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.10247E+01_JPRB, 0.10305E+01_JPRB, 0.10309E+01_JPRB, 0.10358E+01_JPRB, 0.10451E+01_JPRB, 0.10495E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.10573E+01_JPRB, 0.10567E+01_JPRB, 0.10514E+01_JPRB, 0.10419E+01_JPRB, 0.10286E+01_JPRB, 0.10136E+01_JPRB, 0.99563E+00_JPRB ,&
&  0.97655E+00_JPRB, 0.95580E+00_JPRB, 0.93816E+00_JPRB, 0.92655E+00_JPRB, 0.91678E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.96190E+00_JPRB, 0.94472E+00_JPRB, 0.93223E+00_JPRB, 0.92472E+00_JPRB, 0.91850E+00_JPRB, 0.91336E+00_JPRB, 0.90832E+00_JPRB ,&
&  0.90320E+00_JPRB, 0.89767E+00_JPRB, 0.89018E+00_JPRB, 0.88200E+00_JPRB, 0.87517E+00_JPRB, 0.86841E+00_JPRB, 0.86292E+00_JPRB ,&
&  0.85371E+00_JPRB, 0.83294E+00_JPRB, 0.82959E+00_JPRB, 0.81858E+00_JPRB, 0.80059E+00_JPRB, 0.79011E+00_JPRB, 0.78193E+00_JPRB ,&
&  0.77244E+00_JPRB, 0.76900E+00_JPRB, 0.77039E+00_JPRB, 0.76505E+00_JPRB, 0.76320E+00_JPRB, 0.77639E+00_JPRB, 0.77952E+00_JPRB ,&
&  0.74980E+00_JPRB, 0.74936E+00_JPRB, 0.78758E+00_JPRB, 0.78865E+00_JPRB, 0.74436E+00_JPRB, 0.73077E+00_JPRB, 0.73879E+00_JPRB ,&
&  0.73753E+00_JPRB, 0.70741E+00_JPRB, 0.70264E+00_JPRB, 0.70134E+00_JPRB, 0.71469E+00_JPRB, 0.72786E+00_JPRB, 0.75994E+00_JPRB ,&
&  0.78426E+00_JPRB, 0.83880E+00_JPRB, 0.86549E+00_JPRB, 0.88435E+00_JPRB, 0.89852E+00_JPRB, 0.91291E+00_JPRB, 0.92947E+00_JPRB ,&
&  0.95330E+00_JPRB, 0.98418E+00_JPRB, 0.10238E+01_JPRB, 0.10591E+01_JPRB, 0.10925E+01_JPRB, 0.11083E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11324E+01_JPRB, 0.10970E+01_JPRB, 0.10674E+01_JPRB, 0.10349E+01_JPRB, 0.10042E+01_JPRB, 0.97260E+00_JPRB ,&
&  0.96435E+00_JPRB, 0.97162E+00_JPRB, 0.96274E+00_JPRB, 0.95619E+00_JPRB, 0.95457E+00_JPRB, 0.95627E+00_JPRB, 0.96342E+00_JPRB ,&
&  0.96868E+00_JPRB, 0.10043E+01_JPRB, 0.11142E+01_JPRB, 0.11432E+01_JPRB, 0.11547E+01_JPRB, 0.11568E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.11755E+01_JPRB, 0.11913E+01_JPRB, 0.11992E+01_JPRB, 0.12082E+01_JPRB, 0.12006E+01_JPRB, 0.12082E+01_JPRB ,&
&  0.12009E+01_JPRB, 0.11915E+01_JPRB, 0.11799E+01_JPRB, 0.11709E+01_JPRB, 0.11597E+01_JPRB, 0.11489E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.10790E+01_JPRB, 0.10495E+01_JPRB, 0.10180E+01_JPRB, 0.98740E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.10398E+01_JPRB, 0.10005E+01_JPRB, 0.97375E+00_JPRB, 0.96184E+00_JPRB, 0.95393E+00_JPRB, 0.93576E+00_JPRB, 0.92167E+00_JPRB ,&
&  0.91122E+00_JPRB, 0.90743E+00_JPRB, 0.89878E+00_JPRB, 0.88840E+00_JPRB, 0.88163E+00_JPRB, 0.87557E+00_JPRB, 0.87227E+00_JPRB ,&
&  0.86813E+00_JPRB, 0.85812E+00_JPRB, 0.83725E+00_JPRB, 0.81505E+00_JPRB, 0.79964E+00_JPRB, 0.78686E+00_JPRB, 0.77618E+00_JPRB ,&
&  0.76678E+00_JPRB, 0.76505E+00_JPRB, 0.77568E+00_JPRB, 0.77548E+00_JPRB, 0.77520E+00_JPRB, 0.77056E+00_JPRB, 0.76685E+00_JPRB ,&
&  0.77139E+00_JPRB, 0.78660E+00_JPRB, 0.76480E+00_JPRB, 0.77011E+00_JPRB, 0.78709E+00_JPRB, 0.76930E+00_JPRB, 0.75445E+00_JPRB ,&
&  0.74169E+00_JPRB, 0.73074E+00_JPRB, 0.72573E+00_JPRB, 0.72641E+00_JPRB, 0.74843E+00_JPRB, 0.77446E+00_JPRB, 0.81617E+00_JPRB ,&
&  0.86118E+00_JPRB, 0.89405E+00_JPRB, 0.91459E+00_JPRB, 0.93296E+00_JPRB, 0.95104E+00_JPRB, 0.97199E+00_JPRB, 0.10085E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.11317E+01_JPRB, 0.11933E+01_JPRB, 0.12351E+01_JPRB, 0.12768E+01_JPRB, 0.12485E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.14196E+01_JPRB, 0.13484E+01_JPRB, 0.12856E+01_JPRB, 0.12178E+01_JPRB, 0.11580E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.10582E+01_JPRB, 0.10430E+01_JPRB, 0.10228E+01_JPRB, 0.10106E+01_JPRB, 0.10178E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.12701E+01_JPRB, 0.15980E+01_JPRB, 0.14105E+01_JPRB, 0.13416E+01_JPRB, 0.13297E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.13829E+01_JPRB, 0.13937E+01_JPRB, 0.13711E+01_JPRB, 0.13316E+01_JPRB, 0.12835E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.12260E+01_JPRB, 0.11997E+01_JPRB, 0.11835E+01_JPRB, 0.11793E+01_JPRB, 0.11682E+01_JPRB, 0.11567E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11197E+01_JPRB, 0.11148E+01_JPRB, 0.11022E+01_JPRB, 0.10815E+01_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.10645E+01_JPRB, 0.10698E+01_JPRB, 0.10574E+01_JPRB, 0.10398E+01_JPRB, 0.10192E+01_JPRB, 0.97138E+00_JPRB, 0.93398E+00_JPRB ,&
&  0.91408E+00_JPRB, 0.90629E+00_JPRB, 0.90455E+00_JPRB, 0.90011E+00_JPRB, 0.89593E+00_JPRB, 0.88970E+00_JPRB, 0.87909E+00_JPRB ,&
&  0.86502E+00_JPRB, 0.85476E+00_JPRB, 0.84583E+00_JPRB, 0.83092E+00_JPRB, 0.81136E+00_JPRB, 0.79086E+00_JPRB, 0.77464E+00_JPRB ,&
&  0.76321E+00_JPRB, 0.75884E+00_JPRB, 0.76784E+00_JPRB, 0.77737E+00_JPRB, 0.78735E+00_JPRB, 0.79113E+00_JPRB, 0.78785E+00_JPRB ,&
&  0.77248E+00_JPRB, 0.76966E+00_JPRB, 0.78227E+00_JPRB, 0.78291E+00_JPRB, 0.79028E+00_JPRB, 0.79239E+00_JPRB, 0.77960E+00_JPRB ,&
&  0.76803E+00_JPRB, 0.75957E+00_JPRB, 0.76481E+00_JPRB, 0.78155E+00_JPRB, 0.82949E+00_JPRB, 0.88160E+00_JPRB, 0.90714E+00_JPRB ,&
&  0.93189E+00_JPRB, 0.94759E+00_JPRB, 0.96337E+00_JPRB, 0.98681E+00_JPRB, 0.10214E+01_JPRB, 0.10672E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.13459E+01_JPRB, 0.14206E+01_JPRB, 0.13387E+01_JPRB, 0.13084E+01_JPRB, 0.13423E+01_JPRB, 0.13954E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.14901E+01_JPRB, 0.14812E+01_JPRB, 0.14882E+01_JPRB, 0.14238E+01_JPRB, 0.13439E+01_JPRB, 0.12408E+01_JPRB ,&
&  0.11636E+01_JPRB, 0.11148E+01_JPRB, 0.10964E+01_JPRB, 0.10825E+01_JPRB, 0.10976E+01_JPRB, 0.11331E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.14105E+01_JPRB, 0.14744E+01_JPRB, 0.14349E+01_JPRB, 0.14625E+01_JPRB, 0.15227E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.15663E+01_JPRB, 0.15601E+01_JPRB, 0.15662E+01_JPRB, 0.15512E+01_JPRB, 0.14582E+01_JPRB, 0.13729E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.11033E+01_JPRB, 0.10629E+01_JPRB, 0.10523E+01_JPRB, 0.10453E+01_JPRB, 0.10360E+01_JPRB, 0.10154E+01_JPRB ,&
&  0.99327E+00_JPRB, 0.98950E+00_JPRB, 0.10030E+01_JPRB, 0.10221E+01_JPRB, 0.10480E+01_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.11113E+01_JPRB, 0.11339E+01_JPRB, 0.11466E+01_JPRB, 0.11479E+01_JPRB, 0.11019E+01_JPRB, 0.10546E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.97947E+00_JPRB, 0.94983E+00_JPRB, 0.92970E+00_JPRB, 0.90663E+00_JPRB, 0.88626E+00_JPRB, 0.87526E+00_JPRB, 0.87217E+00_JPRB ,&
&  0.87079E+00_JPRB, 0.86018E+00_JPRB, 0.84651E+00_JPRB, 0.82888E+00_JPRB, 0.81410E+00_JPRB, 0.80073E+00_JPRB, 0.78838E+00_JPRB ,&
&  0.77783E+00_JPRB, 0.77614E+00_JPRB, 0.78075E+00_JPRB, 0.79199E+00_JPRB, 0.80035E+00_JPRB, 0.80461E+00_JPRB, 0.80577E+00_JPRB ,&
&  0.79965E+00_JPRB, 0.78975E+00_JPRB, 0.79438E+00_JPRB, 0.80523E+00_JPRB, 0.81369E+00_JPRB, 0.82222E+00_JPRB, 0.80512E+00_JPRB ,&
&  0.79712E+00_JPRB, 0.80415E+00_JPRB, 0.83089E+00_JPRB, 0.89388E+00_JPRB, 0.92984E+00_JPRB, 0.95145E+00_JPRB, 0.96531E+00_JPRB ,&
&  0.95581E+00_JPRB, 0.95770E+00_JPRB, 0.98377E+00_JPRB, 0.10387E+01_JPRB, 0.11094E+01_JPRB, 0.12344E+01_JPRB, 0.13886E+01_JPRB ,&
&  0.14928E+01_JPRB, 0.15426E+01_JPRB, 0.15702E+01_JPRB, 0.14819E+01_JPRB, 0.14883E+01_JPRB, 0.16670E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.17585E+01_JPRB, 0.17689E+01_JPRB, 0.18044E+01_JPRB, 0.17440E+01_JPRB, 0.15813E+01_JPRB, 0.14601E+01_JPRB, 0.13712E+01_JPRB ,&
&  0.12839E+01_JPRB, 0.12346E+01_JPRB, 0.12289E+01_JPRB, 0.12618E+01_JPRB, 0.13354E+01_JPRB, 0.14797E+01_JPRB, 0.16613E+01_JPRB ,&
&  0.18351E+01_JPRB, 0.19383E+01_JPRB, 0.20002E+01_JPRB, 0.19857E+01_JPRB, 0.19578E+01_JPRB, 0.19169E+01_JPRB, 0.18859E+01_JPRB ,&
&  0.18487E+01_JPRB, 0.18368E+01_JPRB, 0.18204E+01_JPRB, 0.18019E+01_JPRB, 0.16824E+01_JPRB, 0.14853E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.11553E+01_JPRB, 0.10782E+01_JPRB, 0.10253E+01_JPRB, 0.10140E+01_JPRB, 0.10184E+01_JPRB, 0.10221E+01_JPRB, 0.99666E+00_JPRB ,&
&  0.98467E+00_JPRB, 0.99706E+00_JPRB, 0.10182E+01_JPRB, 0.10391E+01_JPRB, 0.10731E+01_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.12135E+01_JPRB, 0.12362E+01_JPRB, 0.12448E+01_JPRB, 0.12315E+01_JPRB, 0.11936E+01_JPRB, 0.11299E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.98446E+00_JPRB, 0.96499E+00_JPRB, 0.94264E+00_JPRB, 0.92235E+00_JPRB, 0.89468E+00_JPRB, 0.86777E+00_JPRB ,&
&  0.85917E+00_JPRB, 0.86202E+00_JPRB, 0.86006E+00_JPRB, 0.84997E+00_JPRB, 0.83773E+00_JPRB, 0.82464E+00_JPRB, 0.81344E+00_JPRB ,&
&  0.80553E+00_JPRB, 0.80553E+00_JPRB, 0.81653E+00_JPRB, 0.83117E+00_JPRB, 0.83850E+00_JPRB, 0.84193E+00_JPRB, 0.84836E+00_JPRB ,&
&  0.84717E+00_JPRB, 0.83724E+00_JPRB, 0.83464E+00_JPRB, 0.83984E+00_JPRB, 0.84434E+00_JPRB, 0.86664E+00_JPRB, 0.86667E+00_JPRB ,&
&  0.83895E+00_JPRB, 0.86653E+00_JPRB, 0.92497E+00_JPRB, 0.95877E+00_JPRB, 0.98074E+00_JPRB, 0.99657E+00_JPRB, 0.10096E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.10418E+01_JPRB, 0.10669E+01_JPRB, 0.10967E+01_JPRB, 0.12221E+01_JPRB, 0.15067E+01_JPRB, 0.16221E+01_JPRB ,&
&  0.17047E+01_JPRB, 0.15532E+01_JPRB, 0.14783E+01_JPRB, 0.14991E+01_JPRB, 0.16996E+01_JPRB, 0.18212E+01_JPRB, 0.19073E+01_JPRB ,&
&  0.20170E+01_JPRB, 0.20190E+01_JPRB, 0.19852E+01_JPRB, 0.18965E+01_JPRB, 0.17505E+01_JPRB, 0.16422E+01_JPRB, 0.16057E+01_JPRB ,&
&  0.15567E+01_JPRB, 0.15548E+01_JPRB, 0.16350E+01_JPRB, 0.18101E+01_JPRB, 0.20139E+01_JPRB, 0.21854E+01_JPRB, 0.23116E+01_JPRB ,&
&  0.24322E+01_JPRB, 0.24874E+01_JPRB, 0.24573E+01_JPRB, 0.23997E+01_JPRB, 0.23003E+01_JPRB, 0.22410E+01_JPRB, 0.22156E+01_JPRB ,&
&  0.21845E+01_JPRB, 0.21577E+01_JPRB, 0.21030E+01_JPRB, 0.20279E+01_JPRB, 0.19027E+01_JPRB, 0.17815E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.12044E+01_JPRB, 0.10869E+01_JPRB, 0.10560E+01_JPRB, 0.10605E+01_JPRB, 0.10772E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.11014E+01_JPRB, 0.11079E+01_JPRB, 0.11392E+01_JPRB, 0.11796E+01_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.13244E+01_JPRB, 0.13321E+01_JPRB, 0.13222E+01_JPRB, 0.13071E+01_JPRB, 0.12776E+01_JPRB, 0.12181E+01_JPRB, 0.11618E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11490E+01_JPRB, 0.11492E+01_JPRB, 0.10277E+01_JPRB, 0.98313E+00_JPRB, 0.96398E+00_JPRB, 0.92784E+00_JPRB ,&
&  0.90627E+00_JPRB, 0.89706E+00_JPRB, 0.88552E+00_JPRB, 0.88106E+00_JPRB, 0.87400E+00_JPRB, 0.85982E+00_JPRB, 0.84969E+00_JPRB ,&
&  0.84764E+00_JPRB, 0.85553E+00_JPRB, 0.87156E+00_JPRB, 0.88448E+00_JPRB, 0.89665E+00_JPRB, 0.90588E+00_JPRB, 0.91145E+00_JPRB ,&
&  0.89375E+00_JPRB, 0.88598E+00_JPRB, 0.88514E+00_JPRB, 0.87813E+00_JPRB, 0.88379E+00_JPRB, 0.90076E+00_JPRB, 0.92936E+00_JPRB ,&
&  0.94839E+00_JPRB, 0.93933E+00_JPRB, 0.99176E+00_JPRB, 0.10193E+01_JPRB, 0.10378E+01_JPRB, 0.10512E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10899E+01_JPRB, 0.11179E+01_JPRB, 0.11586E+01_JPRB, 0.13923E+01_JPRB, 0.18463E+01_JPRB, 0.18720E+01_JPRB ,&
&  0.18493E+01_JPRB, 0.16018E+01_JPRB, 0.17525E+01_JPRB, 0.18248E+01_JPRB, 0.19729E+01_JPRB, 0.21161E+01_JPRB, 0.21411E+01_JPRB ,&
&  0.23697E+01_JPRB, 0.24679E+01_JPRB, 0.23846E+01_JPRB, 0.22021E+01_JPRB, 0.20400E+01_JPRB, 0.19622E+01_JPRB, 0.19335E+01_JPRB ,&
&  0.19587E+01_JPRB, 0.19280E+01_JPRB, 0.21507E+01_JPRB, 0.23402E+01_JPRB, 0.24419E+01_JPRB, 0.25418E+01_JPRB, 0.26590E+01_JPRB ,&
&  0.27358E+01_JPRB, 0.27399E+01_JPRB, 0.27255E+01_JPRB, 0.25813E+01_JPRB, 0.25059E+01_JPRB, 0.24463E+01_JPRB, 0.23422E+01_JPRB ,&
&  0.22750E+01_JPRB, 0.22217E+01_JPRB, 0.21485E+01_JPRB, 0.21623E+01_JPRB, 0.19889E+01_JPRB, 0.18628E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.13645E+01_JPRB, 0.12779E+01_JPRB, 0.12473E+01_JPRB, 0.12202E+01_JPRB, 0.12175E+01_JPRB, 0.12130E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.12047E+01_JPRB, 0.12618E+01_JPRB, 0.12865E+01_JPRB, 0.13023E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.14217E+01_JPRB, 0.14289E+01_JPRB, 0.14196E+01_JPRB, 0.14043E+01_JPRB, 0.13868E+01_JPRB, 0.13561E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.12553E+01_JPRB, 0.12112E+01_JPRB, 0.11810E+01_JPRB, 0.11725E+01_JPRB, 0.10977E+01_JPRB, 0.98449E+00_JPRB ,&
&  0.94019E+00_JPRB, 0.93737E+00_JPRB, 0.94283E+00_JPRB, 0.93961E+00_JPRB, 0.93460E+00_JPRB, 0.92748E+00_JPRB, 0.92692E+00_JPRB ,&
&  0.93292E+00_JPRB, 0.94297E+00_JPRB, 0.96317E+00_JPRB, 0.98717E+00_JPRB, 0.99379E+00_JPRB, 0.99893E+00_JPRB, 0.10079E+01_JPRB ,&
&  0.97819E+00_JPRB, 0.94889E+00_JPRB, 0.93090E+00_JPRB, 0.90720E+00_JPRB, 0.94094E+00_JPRB, 0.96819E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.10351E+01_JPRB, 0.10620E+01_JPRB, 0.10913E+01_JPRB, 0.11113E+01_JPRB, 0.11194E+01_JPRB, 0.11266E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.11440E+01_JPRB, 0.11677E+01_JPRB, 0.12026E+01_JPRB, 0.12514E+01_JPRB, 0.17077E+01_JPRB, 0.22059E+01_JPRB, 0.23069E+01_JPRB ,&
&  0.22087E+01_JPRB, 0.23637E+01_JPRB, 0.24823E+01_JPRB, 0.23649E+01_JPRB, 0.24832E+01_JPRB, 0.25648E+01_JPRB, 0.25070E+01_JPRB ,&
&  0.26270E+01_JPRB, 0.26431E+01_JPRB, 0.26060E+01_JPRB, 0.25541E+01_JPRB, 0.25213E+01_JPRB, 0.24579E+01_JPRB, 0.24118E+01_JPRB ,&
&  0.24551E+01_JPRB, 0.26400E+01_JPRB, 0.26827E+01_JPRB, 0.27483E+01_JPRB, 0.27644E+01_JPRB, 0.28113E+01_JPRB, 0.28582E+01_JPRB ,&
&  0.28840E+01_JPRB, 0.29201E+01_JPRB, 0.29191E+01_JPRB, 0.28376E+01_JPRB, 0.27285E+01_JPRB, 0.26129E+01_JPRB, 0.24726E+01_JPRB ,&
&  0.23615E+01_JPRB, 0.21434E+01_JPRB, 0.19745E+01_JPRB, 0.19254E+01_JPRB, 0.18805E+01_JPRB, 0.17395E+01_JPRB, 0.16973E+01_JPRB ,&
&  0.16553E+01_JPRB, 0.15824E+01_JPRB, 0.16729E+01_JPRB, 0.15833E+01_JPRB, 0.14931E+01_JPRB, 0.13851E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.12288E+01_JPRB, 0.13026E+01_JPRB, 0.13773E+01_JPRB, 0.14173E+01_JPRB, 0.14212E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.14964E+01_JPRB, 0.15085E+01_JPRB, 0.15119E+01_JPRB, 0.15079E+01_JPRB, 0.14863E+01_JPRB, 0.14508E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.13676E+01_JPRB, 0.13210E+01_JPRB, 0.12798E+01_JPRB, 0.12481E+01_JPRB, 0.12338E+01_JPRB, 0.12123E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10080E+01_JPRB, 0.98365E+00_JPRB, 0.98151E+00_JPRB, 0.10023E+01_JPRB, 0.10128E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.10873E+01_JPRB, 0.11045E+01_JPRB, 0.11212E+01_JPRB, 0.11323E+01_JPRB, 0.11498E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.10537E+01_JPRB, 0.10173E+01_JPRB, 0.10027E+01_JPRB, 0.10392E+01_JPRB, 0.10901E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.11998E+01_JPRB, 0.12214E+01_JPRB, 0.12350E+01_JPRB, 0.12357E+01_JPRB, 0.12376E+01_JPRB, 0.12454E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.12885E+01_JPRB, 0.13097E+01_JPRB, 0.13895E+01_JPRB, 0.21098E+01_JPRB, 0.29162E+01_JPRB, 0.32287E+01_JPRB ,&
&  0.31094E+01_JPRB, 0.31193E+01_JPRB, 0.31978E+01_JPRB, 0.33738E+01_JPRB, 0.33484E+01_JPRB, 0.31315E+01_JPRB, 0.29437E+01_JPRB ,&
&  0.29688E+01_JPRB, 0.28539E+01_JPRB, 0.30527E+01_JPRB, 0.30377E+01_JPRB, 0.30682E+01_JPRB, 0.31247E+01_JPRB, 0.30855E+01_JPRB ,&
&  0.31334E+01_JPRB, 0.31441E+01_JPRB, 0.31155E+01_JPRB, 0.30216E+01_JPRB, 0.29834E+01_JPRB, 0.30241E+01_JPRB, 0.30463E+01_JPRB ,&
&  0.31241E+01_JPRB, 0.31828E+01_JPRB, 0.30407E+01_JPRB, 0.29277E+01_JPRB, 0.27877E+01_JPRB, 0.29068E+01_JPRB, 0.26912E+01_JPRB ,&
&  0.23392E+01_JPRB, 0.19168E+01_JPRB, 0.19154E+01_JPRB, 0.19234E+01_JPRB, 0.19527E+01_JPRB, 0.19046E+01_JPRB, 0.17907E+01_JPRB ,&
&  0.17898E+01_JPRB, 0.19635E+01_JPRB, 0.20380E+01_JPRB, 0.18342E+01_JPRB, 0.17069E+01_JPRB, 0.15580E+01_JPRB, 0.13686E+01_JPRB ,&
&  0.13560E+01_JPRB, 0.14598E+01_JPRB, 0.14962E+01_JPRB, 0.14884E+01_JPRB, 0.14788E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.16420E+01_JPRB, 0.16029E+01_JPRB, 0.15855E+01_JPRB, 0.15794E+01_JPRB, 0.15540E+01_JPRB, 0.15219E+01_JPRB, 0.14827E+01_JPRB ,&
&  0.14495E+01_JPRB, 0.14175E+01_JPRB, 0.13846E+01_JPRB, 0.13556E+01_JPRB, 0.13368E+01_JPRB, 0.13074E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.11511E+01_JPRB, 0.11073E+01_JPRB, 0.10782E+01_JPRB, 0.10823E+01_JPRB, 0.11581E+01_JPRB, 0.11517E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.11888E+01_JPRB, 0.12111E+01_JPRB, 0.12086E+01_JPRB, 0.12162E+01_JPRB, 0.12461E+01_JPRB, 0.13180E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.12676E+01_JPRB, 0.12264E+01_JPRB, 0.12118E+01_JPRB, 0.12573E+01_JPRB, 0.12976E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.13831E+01_JPRB, 0.13808E+01_JPRB, 0.13745E+01_JPRB, 0.13686E+01_JPRB, 0.13680E+01_JPRB, 0.13829E+01_JPRB, 0.14138E+01_JPRB ,&
&  0.14441E+01_JPRB, 0.14352E+01_JPRB, 0.14458E+01_JPRB, 0.16240E+01_JPRB, 0.26479E+01_JPRB, 0.37255E+01_JPRB, 0.43193E+01_JPRB ,&
&  0.44749E+01_JPRB, 0.42686E+01_JPRB, 0.43532E+01_JPRB, 0.49069E+01_JPRB, 0.46959E+01_JPRB, 0.41902E+01_JPRB, 0.37524E+01_JPRB ,&
&  0.35648E+01_JPRB, 0.34016E+01_JPRB, 0.35271E+01_JPRB, 0.36377E+01_JPRB, 0.35310E+01_JPRB, 0.35166E+01_JPRB, 0.34795E+01_JPRB ,&
&  0.34595E+01_JPRB, 0.34777E+01_JPRB, 0.33555E+01_JPRB, 0.32264E+01_JPRB, 0.31270E+01_JPRB, 0.32122E+01_JPRB, 0.32454E+01_JPRB ,&
&  0.33978E+01_JPRB, 0.33440E+01_JPRB, 0.29980E+01_JPRB, 0.29811E+01_JPRB, 0.24530E+01_JPRB, 0.23373E+01_JPRB, 0.26398E+01_JPRB ,&
&  0.22888E+01_JPRB, 0.20263E+01_JPRB, 0.19937E+01_JPRB, 0.20887E+01_JPRB, 0.21636E+01_JPRB, 0.23148E+01_JPRB, 0.22649E+01_JPRB ,&
&  0.19811E+01_JPRB, 0.20720E+01_JPRB, 0.24068E+01_JPRB, 0.21650E+01_JPRB, 0.19732E+01_JPRB, 0.18445E+01_JPRB, 0.16704E+01_JPRB ,&
&  0.16951E+01_JPRB, 0.16831E+01_JPRB, 0.16659E+01_JPRB, 0.16417E+01_JPRB, 0.16402E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.17572E+01_JPRB, 0.16276E+01_JPRB, 0.16202E+01_JPRB, 0.15939E+01_JPRB, 0.15678E+01_JPRB, 0.15400E+01_JPRB, 0.15261E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.15098E+01_JPRB, 0.14856E+01_JPRB, 0.14556E+01_JPRB, 0.14313E+01_JPRB, 0.13975E+01_JPRB, 0.13584E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.12846E+01_JPRB, 0.12672E+01_JPRB, 0.12197E+01_JPRB, 0.12803E+01_JPRB, 0.12802E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.13896E+01_JPRB, 0.13977E+01_JPRB, 0.14168E+01_JPRB, 0.14684E+01_JPRB, 0.15947E+01_JPRB, 0.17492E+01_JPRB ,&
&  0.17920E+01_JPRB, 0.17035E+01_JPRB, 0.16524E+01_JPRB, 0.16773E+01_JPRB, 0.17573E+01_JPRB, 0.16266E+01_JPRB, 0.16225E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.15300E+01_JPRB, 0.14983E+01_JPRB, 0.14890E+01_JPRB, 0.15183E+01_JPRB, 0.15686E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.15993E+01_JPRB, 0.15328E+01_JPRB, 0.15452E+01_JPRB, 0.20071E+01_JPRB, 0.31830E+01_JPRB, 0.44276E+01_JPRB, 0.49914E+01_JPRB ,&
&  0.51137E+01_JPRB, 0.50439E+01_JPRB, 0.50940E+01_JPRB, 0.56186E+01_JPRB, 0.53359E+01_JPRB, 0.47977E+01_JPRB, 0.44940E+01_JPRB ,&
&  0.42947E+01_JPRB, 0.41782E+01_JPRB, 0.42242E+01_JPRB, 0.39819E+01_JPRB, 0.36906E+01_JPRB, 0.35131E+01_JPRB, 0.33662E+01_JPRB ,&
&  0.32174E+01_JPRB, 0.32030E+01_JPRB, 0.31902E+01_JPRB, 0.30244E+01_JPRB, 0.29448E+01_JPRB, 0.28705E+01_JPRB, 0.31120E+01_JPRB ,&
&  0.34684E+01_JPRB, 0.27373E+01_JPRB, 0.23284E+01_JPRB, 0.25476E+01_JPRB, 0.20140E+01_JPRB, 0.19524E+01_JPRB, 0.21594E+01_JPRB ,&
&  0.20455E+01_JPRB, 0.20807E+01_JPRB, 0.23327E+01_JPRB, 0.23350E+01_JPRB, 0.24896E+01_JPRB, 0.27862E+01_JPRB, 0.27869E+01_JPRB ,&
&  0.25821E+01_JPRB, 0.24346E+01_JPRB, 0.29607E+01_JPRB, 0.26627E+01_JPRB, 0.23774E+01_JPRB, 0.22434E+01_JPRB, 0.21244E+01_JPRB ,&
&  0.20508E+01_JPRB, 0.19803E+01_JPRB, 0.18972E+01_JPRB, 0.18398E+01_JPRB, 0.18228E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.17605E+01_JPRB, 0.16292E+01_JPRB, 0.16755E+01_JPRB, 0.16687E+01_JPRB, 0.16550E+01_JPRB, 0.16445E+01_JPRB, 0.16434E+01_JPRB ,&
&  0.16261E+01_JPRB, 0.16115E+01_JPRB, 0.15774E+01_JPRB, 0.15184E+01_JPRB, 0.14759E+01_JPRB, 0.14398E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.13824E+01_JPRB, 0.14174E+01_JPRB, 0.13594E+01_JPRB, 0.13436E+01_JPRB, 0.13099E+01_JPRB, 0.13618E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.15142E+01_JPRB, 0.16186E+01_JPRB, 0.17468E+01_JPRB, 0.18998E+01_JPRB, 0.20385E+01_JPRB, 0.21562E+01_JPRB, 0.23574E+01_JPRB ,&
&  0.24284E+01_JPRB, 0.23109E+01_JPRB, 0.21641E+01_JPRB, 0.21786E+01_JPRB, 0.21332E+01_JPRB, 0.19379E+01_JPRB, 0.18820E+01_JPRB ,&
&  0.17824E+01_JPRB, 0.17180E+01_JPRB, 0.16787E+01_JPRB, 0.16869E+01_JPRB, 0.16761E+01_JPRB, 0.16040E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.14950E+01_JPRB, 0.15458E+01_JPRB, 0.18742E+01_JPRB, 0.26385E+01_JPRB, 0.39069E+01_JPRB, 0.52608E+01_JPRB, 0.56096E+01_JPRB ,&
&  0.57098E+01_JPRB, 0.52671E+01_JPRB, 0.42296E+01_JPRB, 0.47184E+01_JPRB, 0.58020E+01_JPRB, 0.54846E+01_JPRB, 0.49399E+01_JPRB ,&
&  0.48764E+01_JPRB, 0.45336E+01_JPRB, 0.43691E+01_JPRB, 0.39497E+01_JPRB, 0.35954E+01_JPRB, 0.33132E+01_JPRB, 0.30751E+01_JPRB ,&
&  0.28908E+01_JPRB, 0.28406E+01_JPRB, 0.28401E+01_JPRB, 0.28727E+01_JPRB, 0.28422E+01_JPRB, 0.30144E+01_JPRB, 0.33758E+01_JPRB ,&
&  0.33974E+01_JPRB, 0.31742E+01_JPRB, 0.25041E+01_JPRB, 0.24389E+01_JPRB, 0.19184E+01_JPRB, 0.18635E+01_JPRB, 0.20516E+01_JPRB ,&
&  0.20580E+01_JPRB, 0.24580E+01_JPRB, 0.30473E+01_JPRB, 0.31773E+01_JPRB, 0.37175E+01_JPRB, 0.38627E+01_JPRB, 0.37654E+01_JPRB ,&
&  0.35408E+01_JPRB, 0.31839E+01_JPRB, 0.37058E+01_JPRB, 0.32085E+01_JPRB, 0.27525E+01_JPRB, 0.25615E+01_JPRB, 0.24291E+01_JPRB ,&
&  0.23344E+01_JPRB, 0.22006E+01_JPRB, 0.20554E+01_JPRB, 0.19586E+01_JPRB, 0.18807E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.17726E+01_JPRB, 0.16867E+01_JPRB, 0.17317E+01_JPRB, 0.16996E+01_JPRB, 0.17166E+01_JPRB, 0.16921E+01_JPRB, 0.16990E+01_JPRB ,&
&  0.16855E+01_JPRB, 0.16393E+01_JPRB, 0.15751E+01_JPRB, 0.15177E+01_JPRB, 0.14768E+01_JPRB, 0.14404E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.14068E+01_JPRB, 0.14035E+01_JPRB, 0.13081E+01_JPRB, 0.12581E+01_JPRB, 0.12398E+01_JPRB, 0.13229E+01_JPRB, 0.14803E+01_JPRB ,&
&  0.16917E+01_JPRB, 0.19088E+01_JPRB, 0.22219E+01_JPRB, 0.24446E+01_JPRB, 0.26253E+01_JPRB, 0.27809E+01_JPRB, 0.29960E+01_JPRB ,&
&  0.31860E+01_JPRB, 0.27413E+01_JPRB, 0.25771E+01_JPRB, 0.23382E+01_JPRB, 0.21930E+01_JPRB, 0.20634E+01_JPRB, 0.19469E+01_JPRB ,&
&  0.18574E+01_JPRB, 0.18082E+01_JPRB, 0.17532E+01_JPRB, 0.16635E+01_JPRB, 0.15716E+01_JPRB, 0.15164E+01_JPRB, 0.15087E+01_JPRB ,&
&  0.15726E+01_JPRB, 0.18235E+01_JPRB, 0.25640E+01_JPRB, 0.36320E+01_JPRB, 0.52574E+01_JPRB, 0.65183E+01_JPRB, 0.64399E+01_JPRB ,&
&  0.51750E+01_JPRB, 0.47650E+01_JPRB, 0.63896E+01_JPRB, 0.55806E+01_JPRB, 0.56757E+01_JPRB, 0.60209E+01_JPRB, 0.55553E+01_JPRB ,&
&  0.48533E+01_JPRB, 0.40836E+01_JPRB, 0.39503E+01_JPRB, 0.36885E+01_JPRB, 0.44577E+01_JPRB, 0.38467E+01_JPRB, 0.33796E+01_JPRB ,&
&  0.31017E+01_JPRB, 0.30661E+01_JPRB, 0.29972E+01_JPRB, 0.30592E+01_JPRB, 0.33263E+01_JPRB, 0.38846E+01_JPRB, 0.40248E+01_JPRB ,&
&  0.36716E+01_JPRB, 0.37186E+01_JPRB, 0.31355E+01_JPRB, 0.26048E+01_JPRB, 0.22663E+01_JPRB, 0.22539E+01_JPRB, 0.23223E+01_JPRB ,&
&  0.25715E+01_JPRB, 0.29533E+01_JPRB, 0.38848E+01_JPRB, 0.48218E+01_JPRB, 0.55514E+01_JPRB, 0.54449E+01_JPRB, 0.46985E+01_JPRB ,&
&  0.43929E+01_JPRB, 0.49492E+01_JPRB, 0.48655E+01_JPRB, 0.37625E+01_JPRB, 0.30900E+01_JPRB, 0.28610E+01_JPRB, 0.26881E+01_JPRB ,&
&  0.24564E+01_JPRB, 0.22183E+01_JPRB, 0.20460E+01_JPRB, 0.19524E+01_JPRB, 0.18999E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.18763E+01_JPRB, 0.17934E+01_JPRB, 0.17041E+01_JPRB, 0.16735E+01_JPRB, 0.17045E+01_JPRB, 0.16675E+01_JPRB, 0.16336E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.15241E+01_JPRB, 0.15066E+01_JPRB, 0.15002E+01_JPRB, 0.14806E+01_JPRB, 0.14496E+01_JPRB, 0.14324E+01_JPRB ,&
&  0.14142E+01_JPRB, 0.13464E+01_JPRB, 0.12591E+01_JPRB, 0.12708E+01_JPRB, 0.12758E+01_JPRB, 0.13038E+01_JPRB, 0.15264E+01_JPRB ,&
&  0.18861E+01_JPRB, 0.23311E+01_JPRB, 0.27054E+01_JPRB, 0.29173E+01_JPRB, 0.30047E+01_JPRB, 0.31219E+01_JPRB, 0.32904E+01_JPRB ,&
&  0.30775E+01_JPRB, 0.29837E+01_JPRB, 0.27841E+01_JPRB, 0.25024E+01_JPRB, 0.22728E+01_JPRB, 0.20789E+01_JPRB, 0.19704E+01_JPRB ,&
&  0.18929E+01_JPRB, 0.18576E+01_JPRB, 0.17548E+01_JPRB, 0.16344E+01_JPRB, 0.16134E+01_JPRB, 0.16144E+01_JPRB, 0.16269E+01_JPRB ,&
&  0.17138E+01_JPRB, 0.22446E+01_JPRB, 0.31736E+01_JPRB, 0.43497E+01_JPRB, 0.46212E+01_JPRB, 0.57264E+01_JPRB, 0.57654E+01_JPRB ,&
&  0.67380E+01_JPRB, 0.65416E+01_JPRB, 0.62284E+01_JPRB, 0.66998E+01_JPRB, 0.55898E+01_JPRB, 0.56551E+01_JPRB, 0.54966E+01_JPRB ,&
&  0.48606E+01_JPRB, 0.45668E+01_JPRB, 0.49458E+01_JPRB, 0.44239E+01_JPRB, 0.45532E+01_JPRB, 0.50819E+01_JPRB, 0.44982E+01_JPRB ,&
&  0.42206E+01_JPRB, 0.40091E+01_JPRB, 0.37536E+01_JPRB, 0.39440E+01_JPRB, 0.39122E+01_JPRB, 0.40319E+01_JPRB, 0.34998E+01_JPRB ,&
&  0.33862E+01_JPRB, 0.28975E+01_JPRB, 0.31304E+01_JPRB, 0.28524E+01_JPRB, 0.28215E+01_JPRB, 0.27683E+01_JPRB, 0.28216E+01_JPRB ,&
&  0.33262E+01_JPRB, 0.38093E+01_JPRB, 0.51871E+01_JPRB, 0.65910E+01_JPRB, 0.77484E+01_JPRB, 0.66097E+01_JPRB, 0.58846E+01_JPRB ,&
&  0.61849E+01_JPRB, 0.63548E+01_JPRB, 0.52180E+01_JPRB, 0.40400E+01_JPRB, 0.33794E+01_JPRB, 0.31093E+01_JPRB, 0.29152E+01_JPRB ,&
&  0.26884E+01_JPRB, 0.24644E+01_JPRB, 0.22846E+01_JPRB, 0.21515E+01_JPRB, 0.20456E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.17527E+01_JPRB, 0.17026E+01_JPRB, 0.15945E+01_JPRB, 0.15949E+01_JPRB, 0.16466E+01_JPRB, 0.16125E+01_JPRB, 0.15745E+01_JPRB ,&
&  0.15144E+01_JPRB, 0.14742E+01_JPRB, 0.14863E+01_JPRB, 0.14938E+01_JPRB, 0.14787E+01_JPRB, 0.14604E+01_JPRB, 0.14286E+01_JPRB ,&
&  0.14044E+01_JPRB, 0.13615E+01_JPRB, 0.12860E+01_JPRB, 0.12849E+01_JPRB, 0.13561E+01_JPRB, 0.14206E+01_JPRB, 0.16165E+01_JPRB ,&
&  0.23106E+01_JPRB, 0.28725E+01_JPRB, 0.30821E+01_JPRB, 0.31314E+01_JPRB, 0.31076E+01_JPRB, 0.31296E+01_JPRB, 0.30832E+01_JPRB ,&
&  0.29429E+01_JPRB, 0.26516E+01_JPRB, 0.24667E+01_JPRB, 0.23261E+01_JPRB, 0.21937E+01_JPRB, 0.20711E+01_JPRB, 0.19633E+01_JPRB ,&
&  0.18653E+01_JPRB, 0.16964E+01_JPRB, 0.15387E+01_JPRB, 0.14474E+01_JPRB, 0.14706E+01_JPRB, 0.14663E+01_JPRB, 0.15577E+01_JPRB ,&
&  0.19182E+01_JPRB, 0.24945E+01_JPRB, 0.32089E+01_JPRB, 0.40672E+01_JPRB, 0.45547E+01_JPRB, 0.56298E+01_JPRB, 0.64451E+01_JPRB ,&
&  0.63459E+01_JPRB, 0.58589E+01_JPRB, 0.65356E+01_JPRB, 0.72231E+01_JPRB, 0.70225E+01_JPRB, 0.60556E+01_JPRB, 0.57223E+01_JPRB ,&
&  0.43509E+01_JPRB, 0.35728E+01_JPRB, 0.33697E+01_JPRB, 0.37019E+01_JPRB, 0.52113E+01_JPRB, 0.67331E+01_JPRB, 0.63973E+01_JPRB ,&
&  0.62124E+01_JPRB, 0.58322E+01_JPRB, 0.53697E+01_JPRB, 0.46876E+01_JPRB, 0.43135E+01_JPRB, 0.40097E+01_JPRB, 0.49970E+01_JPRB ,&
&  0.49181E+01_JPRB, 0.48873E+01_JPRB, 0.41576E+01_JPRB, 0.34067E+01_JPRB, 0.29255E+01_JPRB, 0.33229E+01_JPRB, 0.36400E+01_JPRB ,&
&  0.42407E+01_JPRB, 0.50279E+01_JPRB, 0.87826E+01_JPRB, 0.10007E+02_JPRB, 0.94874E+01_JPRB, 0.84140E+01_JPRB, 0.83277E+01_JPRB ,&
&  0.75244E+01_JPRB, 0.72356E+01_JPRB, 0.58297E+01_JPRB, 0.42627E+01_JPRB, 0.33335E+01_JPRB, 0.28241E+01_JPRB, 0.25437E+01_JPRB ,&
&  0.24467E+01_JPRB, 0.23545E+01_JPRB, 0.22100E+01_JPRB, 0.20333E+01_JPRB, 0.19041E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.16348E+01_JPRB, 0.15908E+01_JPRB, 0.14776E+01_JPRB, 0.14661E+01_JPRB, 0.15172E+01_JPRB, 0.15544E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.15079E+01_JPRB, 0.15149E+01_JPRB, 0.15356E+01_JPRB, 0.15161E+01_JPRB, 0.14852E+01_JPRB, 0.14705E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14477E+01_JPRB, 0.15016E+01_JPRB, 0.14215E+01_JPRB, 0.14778E+01_JPRB, 0.16098E+01_JPRB, 0.17783E+01_JPRB, 0.20446E+01_JPRB ,&
&  0.29606E+01_JPRB, 0.31943E+01_JPRB, 0.29664E+01_JPRB, 0.29447E+01_JPRB, 0.30919E+01_JPRB, 0.27938E+01_JPRB, 0.26539E+01_JPRB ,&
&  0.25321E+01_JPRB, 0.23027E+01_JPRB, 0.21826E+01_JPRB, 0.20935E+01_JPRB, 0.20610E+01_JPRB, 0.20320E+01_JPRB, 0.19294E+01_JPRB ,&
&  0.17606E+01_JPRB, 0.15621E+01_JPRB, 0.14568E+01_JPRB, 0.14470E+01_JPRB, 0.13864E+01_JPRB, 0.14081E+01_JPRB, 0.15287E+01_JPRB ,&
&  0.19714E+01_JPRB, 0.25541E+01_JPRB, 0.29634E+01_JPRB, 0.34927E+01_JPRB, 0.41121E+01_JPRB, 0.53163E+01_JPRB, 0.73663E+01_JPRB ,&
&  0.68579E+01_JPRB, 0.67928E+01_JPRB, 0.68280E+01_JPRB, 0.76838E+01_JPRB, 0.68079E+01_JPRB, 0.59184E+01_JPRB, 0.54616E+01_JPRB ,&
&  0.41009E+01_JPRB, 0.35295E+01_JPRB, 0.39447E+01_JPRB, 0.48139E+01_JPRB, 0.51593E+01_JPRB, 0.59285E+01_JPRB, 0.74747E+01_JPRB ,&
&  0.66552E+01_JPRB, 0.64851E+01_JPRB, 0.62518E+01_JPRB, 0.59504E+01_JPRB, 0.36574E+01_JPRB, 0.30656E+01_JPRB, 0.44784E+01_JPRB ,&
&  0.47193E+01_JPRB, 0.31224E+01_JPRB, 0.21160E+01_JPRB, 0.26624E+01_JPRB, 0.20298E+01_JPRB, 0.24124E+01_JPRB, 0.37152E+01_JPRB ,&
&  0.52346E+01_JPRB, 0.73231E+01_JPRB, 0.12854E+02_JPRB, 0.11471E+02_JPRB, 0.10558E+02_JPRB, 0.11675E+02_JPRB, 0.97183E+01_JPRB ,&
&  0.94400E+01_JPRB, 0.67016E+01_JPRB, 0.52628E+01_JPRB, 0.36239E+01_JPRB, 0.25532E+01_JPRB, 0.20975E+01_JPRB, 0.19504E+01_JPRB ,&
&  0.18974E+01_JPRB, 0.19066E+01_JPRB, 0.19060E+01_JPRB, 0.18221E+01_JPRB, 0.17208E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.16858E+01_JPRB, 0.15578E+01_JPRB, 0.13838E+01_JPRB, 0.13623E+01_JPRB, 0.13994E+01_JPRB, 0.14561E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.14824E+01_JPRB, 0.15374E+01_JPRB, 0.16267E+01_JPRB, 0.16119E+01_JPRB, 0.15464E+01_JPRB, 0.15220E+01_JPRB, 0.14869E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.16123E+01_JPRB, 0.17995E+01_JPRB, 0.20190E+01_JPRB, 0.22760E+01_JPRB, 0.24275E+01_JPRB, 0.30350E+01_JPRB ,&
&  0.34930E+01_JPRB, 0.31078E+01_JPRB, 0.28072E+01_JPRB, 0.27682E+01_JPRB, 0.26729E+01_JPRB, 0.23205E+01_JPRB, 0.23024E+01_JPRB ,&
&  0.21983E+01_JPRB, 0.20625E+01_JPRB, 0.19846E+01_JPRB, 0.19251E+01_JPRB, 0.19423E+01_JPRB, 0.19523E+01_JPRB, 0.18806E+01_JPRB ,&
&  0.17017E+01_JPRB, 0.15755E+01_JPRB, 0.14968E+01_JPRB, 0.13934E+01_JPRB, 0.13370E+01_JPRB, 0.13729E+01_JPRB, 0.15471E+01_JPRB ,&
&  0.19549E+01_JPRB, 0.24306E+01_JPRB, 0.26881E+01_JPRB, 0.30256E+01_JPRB, 0.44249E+01_JPRB, 0.54058E+01_JPRB, 0.55840E+01_JPRB ,&
&  0.55301E+01_JPRB, 0.56368E+01_JPRB, 0.72311E+01_JPRB, 0.73139E+01_JPRB, 0.68471E+01_JPRB, 0.65670E+01_JPRB, 0.56758E+01_JPRB ,&
&  0.50227E+01_JPRB, 0.46101E+01_JPRB, 0.47654E+01_JPRB, 0.54760E+01_JPRB, 0.61800E+01_JPRB, 0.61173E+01_JPRB, 0.73977E+01_JPRB ,&
&  0.75746E+01_JPRB, 0.71669E+01_JPRB, 0.57170E+01_JPRB, 0.44442E+01_JPRB, 0.40896E+01_JPRB, 0.32215E+01_JPRB, 0.23297E+01_JPRB ,&
&  0.20919E+01_JPRB, 0.20300E+01_JPRB, 0.18480E+01_JPRB, 0.18876E+01_JPRB, 0.17455E+01_JPRB, 0.22003E+01_JPRB, 0.34250E+01_JPRB ,&
&  0.69714E+01_JPRB, 0.10919E+02_JPRB, 0.12956E+02_JPRB, 0.10825E+02_JPRB, 0.97944E+01_JPRB, 0.11644E+02_JPRB, 0.11735E+02_JPRB ,&
&  0.71682E+01_JPRB, 0.52031E+01_JPRB, 0.35010E+01_JPRB, 0.24335E+01_JPRB, 0.19678E+01_JPRB, 0.17402E+01_JPRB, 0.16772E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.16467E+01_JPRB, 0.16674E+01_JPRB, 0.16986E+01_JPRB, 0.16751E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.16587E+01_JPRB, 0.14740E+01_JPRB, 0.12928E+01_JPRB, 0.12498E+01_JPRB, 0.13167E+01_JPRB, 0.13535E+01_JPRB, 0.14019E+01_JPRB ,&
&  0.14792E+01_JPRB, 0.16085E+01_JPRB, 0.17062E+01_JPRB, 0.16873E+01_JPRB, 0.16293E+01_JPRB, 0.15817E+01_JPRB, 0.15715E+01_JPRB ,&
&  0.16651E+01_JPRB, 0.19238E+01_JPRB, 0.24106E+01_JPRB, 0.29237E+01_JPRB, 0.34299E+01_JPRB, 0.33669E+01_JPRB, 0.38736E+01_JPRB ,&
&  0.34442E+01_JPRB, 0.28860E+01_JPRB, 0.28195E+01_JPRB, 0.26815E+01_JPRB, 0.25037E+01_JPRB, 0.21724E+01_JPRB, 0.20709E+01_JPRB ,&
&  0.19487E+01_JPRB, 0.19421E+01_JPRB, 0.18882E+01_JPRB, 0.19116E+01_JPRB, 0.19284E+01_JPRB, 0.18955E+01_JPRB, 0.17850E+01_JPRB ,&
&  0.16386E+01_JPRB, 0.15976E+01_JPRB, 0.14494E+01_JPRB, 0.14352E+01_JPRB, 0.14305E+01_JPRB, 0.15126E+01_JPRB, 0.16988E+01_JPRB ,&
&  0.20345E+01_JPRB, 0.23928E+01_JPRB, 0.26753E+01_JPRB, 0.33967E+01_JPRB, 0.40183E+01_JPRB, 0.39410E+01_JPRB, 0.49802E+01_JPRB ,&
&  0.58778E+01_JPRB, 0.65135E+01_JPRB, 0.65679E+01_JPRB, 0.74822E+01_JPRB, 0.71950E+01_JPRB, 0.65239E+01_JPRB, 0.59884E+01_JPRB ,&
&  0.52542E+01_JPRB, 0.54724E+01_JPRB, 0.65005E+01_JPRB, 0.70930E+01_JPRB, 0.85205E+01_JPRB, 0.80505E+01_JPRB, 0.83922E+01_JPRB ,&
&  0.87522E+01_JPRB, 0.80596E+01_JPRB, 0.66084E+01_JPRB, 0.53877E+01_JPRB, 0.59301E+01_JPRB, 0.58030E+01_JPRB, 0.33545E+01_JPRB ,&
&  0.28424E+01_JPRB, 0.25177E+01_JPRB, 0.21239E+01_JPRB, 0.18654E+01_JPRB, 0.18676E+01_JPRB, 0.24158E+01_JPRB, 0.45725E+01_JPRB ,&
&  0.62898E+01_JPRB, 0.11475E+02_JPRB, 0.13524E+02_JPRB, 0.11407E+02_JPRB, 0.94610E+01_JPRB, 0.10796E+02_JPRB, 0.76380E+01_JPRB ,&
&  0.62501E+01_JPRB, 0.42981E+01_JPRB, 0.24740E+01_JPRB, 0.18933E+01_JPRB, 0.17736E+01_JPRB, 0.16893E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.15039E+01_JPRB, 0.14886E+01_JPRB, 0.15302E+01_JPRB, 0.15675E+01_JPRB, 0.15727E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.14563E+01_JPRB, 0.13098E+01_JPRB, 0.12078E+01_JPRB, 0.11458E+01_JPRB, 0.12305E+01_JPRB, 0.13539E+01_JPRB, 0.14031E+01_JPRB ,&
&  0.15424E+01_JPRB, 0.17241E+01_JPRB, 0.18203E+01_JPRB, 0.18142E+01_JPRB, 0.17183E+01_JPRB, 0.17054E+01_JPRB, 0.18208E+01_JPRB ,&
&  0.21078E+01_JPRB, 0.25987E+01_JPRB, 0.33034E+01_JPRB, 0.40561E+01_JPRB, 0.43900E+01_JPRB, 0.45336E+01_JPRB, 0.42121E+01_JPRB ,&
&  0.32330E+01_JPRB, 0.27858E+01_JPRB, 0.28148E+01_JPRB, 0.27451E+01_JPRB, 0.23633E+01_JPRB, 0.21744E+01_JPRB, 0.19161E+01_JPRB ,&
&  0.18887E+01_JPRB, 0.19219E+01_JPRB, 0.19093E+01_JPRB, 0.19216E+01_JPRB, 0.18873E+01_JPRB, 0.18268E+01_JPRB, 0.17630E+01_JPRB ,&
&  0.16666E+01_JPRB, 0.15394E+01_JPRB, 0.14730E+01_JPRB, 0.14807E+01_JPRB, 0.16142E+01_JPRB, 0.18296E+01_JPRB, 0.20598E+01_JPRB ,&
&  0.23466E+01_JPRB, 0.26573E+01_JPRB, 0.30690E+01_JPRB, 0.38334E+01_JPRB, 0.34440E+01_JPRB, 0.43632E+01_JPRB, 0.50494E+01_JPRB ,&
&  0.54097E+01_JPRB, 0.57312E+01_JPRB, 0.56596E+01_JPRB, 0.64240E+01_JPRB, 0.68522E+01_JPRB, 0.63851E+01_JPRB, 0.57490E+01_JPRB ,&
&  0.58397E+01_JPRB, 0.61487E+01_JPRB, 0.83483E+01_JPRB, 0.95264E+01_JPRB, 0.10426E+02_JPRB, 0.13069E+02_JPRB, 0.97429E+01_JPRB ,&
&  0.83592E+01_JPRB, 0.87692E+01_JPRB, 0.75381E+01_JPRB, 0.63876E+01_JPRB, 0.65575E+01_JPRB, 0.81101E+01_JPRB, 0.95401E+01_JPRB ,&
&  0.43532E+01_JPRB, 0.27283E+01_JPRB, 0.27365E+01_JPRB, 0.24864E+01_JPRB, 0.21325E+01_JPRB, 0.26666E+01_JPRB, 0.76004E+01_JPRB ,&
&  0.64174E+01_JPRB, 0.10661E+02_JPRB, 0.12093E+02_JPRB, 0.10281E+02_JPRB, 0.90505E+01_JPRB, 0.83077E+01_JPRB, 0.65959E+01_JPRB ,&
&  0.46898E+01_JPRB, 0.31477E+01_JPRB, 0.21289E+01_JPRB, 0.17607E+01_JPRB, 0.16805E+01_JPRB, 0.16313E+01_JPRB, 0.15548E+01_JPRB ,&
&  0.14830E+01_JPRB, 0.14321E+01_JPRB, 0.14548E+01_JPRB, 0.14124E+01_JPRB, 0.14782E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.13315E+01_JPRB, 0.12567E+01_JPRB, 0.12631E+01_JPRB, 0.12753E+01_JPRB, 0.13431E+01_JPRB, 0.14157E+01_JPRB, 0.15276E+01_JPRB ,&
&  0.17141E+01_JPRB, 0.19264E+01_JPRB, 0.20865E+01_JPRB, 0.20858E+01_JPRB, 0.20167E+01_JPRB, 0.21564E+01_JPRB, 0.24907E+01_JPRB ,&
&  0.29549E+01_JPRB, 0.35276E+01_JPRB, 0.41852E+01_JPRB, 0.46297E+01_JPRB, 0.48662E+01_JPRB, 0.44475E+01_JPRB, 0.38842E+01_JPRB ,&
&  0.30866E+01_JPRB, 0.26664E+01_JPRB, 0.24807E+01_JPRB, 0.24800E+01_JPRB, 0.24165E+01_JPRB, 0.22041E+01_JPRB, 0.18768E+01_JPRB ,&
&  0.18385E+01_JPRB, 0.18304E+01_JPRB, 0.18044E+01_JPRB, 0.18142E+01_JPRB, 0.18260E+01_JPRB, 0.18447E+01_JPRB, 0.17969E+01_JPRB ,&
&  0.16587E+01_JPRB, 0.16018E+01_JPRB, 0.15865E+01_JPRB, 0.17010E+01_JPRB, 0.19825E+01_JPRB, 0.22838E+01_JPRB, 0.25711E+01_JPRB ,&
&  0.28660E+01_JPRB, 0.31489E+01_JPRB, 0.35096E+01_JPRB, 0.37399E+01_JPRB, 0.41615E+01_JPRB, 0.45738E+01_JPRB, 0.48828E+01_JPRB ,&
&  0.49750E+01_JPRB, 0.53523E+01_JPRB, 0.57006E+01_JPRB, 0.59769E+01_JPRB, 0.62549E+01_JPRB, 0.61172E+01_JPRB, 0.58670E+01_JPRB ,&
&  0.65619E+01_JPRB, 0.80322E+01_JPRB, 0.98289E+01_JPRB, 0.10613E+02_JPRB, 0.11091E+02_JPRB, 0.13220E+02_JPRB, 0.13014E+02_JPRB ,&
&  0.10350E+02_JPRB, 0.78653E+01_JPRB, 0.70853E+01_JPRB, 0.75925E+01_JPRB, 0.81843E+01_JPRB, 0.10058E+02_JPRB, 0.15817E+02_JPRB ,&
&  0.16096E+02_JPRB, 0.71875E+01_JPRB, 0.46538E+01_JPRB, 0.52729E+01_JPRB, 0.30387E+01_JPRB, 0.27085E+01_JPRB, 0.46891E+01_JPRB ,&
&  0.64769E+01_JPRB, 0.95106E+01_JPRB, 0.10992E+02_JPRB, 0.86800E+01_JPRB, 0.76781E+01_JPRB, 0.58169E+01_JPRB, 0.43659E+01_JPRB ,&
&  0.32980E+01_JPRB, 0.23391E+01_JPRB, 0.17818E+01_JPRB, 0.16185E+01_JPRB, 0.15115E+01_JPRB, 0.15184E+01_JPRB, 0.15588E+01_JPRB ,&
&  0.14884E+01_JPRB, 0.13754E+01_JPRB, 0.13798E+01_JPRB, 0.13760E+01_JPRB, 0.13415E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.13642E+01_JPRB, 0.13596E+01_JPRB, 0.14190E+01_JPRB, 0.14945E+01_JPRB, 0.15778E+01_JPRB, 0.16827E+01_JPRB, 0.18048E+01_JPRB ,&
&  0.20291E+01_JPRB, 0.22932E+01_JPRB, 0.24643E+01_JPRB, 0.25759E+01_JPRB, 0.27144E+01_JPRB, 0.29971E+01_JPRB, 0.33743E+01_JPRB ,&
&  0.38135E+01_JPRB, 0.42629E+01_JPRB, 0.46987E+01_JPRB, 0.51634E+01_JPRB, 0.57865E+01_JPRB, 0.61184E+01_JPRB, 0.35945E+01_JPRB ,&
&  0.29196E+01_JPRB, 0.29233E+01_JPRB, 0.24573E+01_JPRB, 0.25891E+01_JPRB, 0.26447E+01_JPRB, 0.25815E+01_JPRB, 0.20628E+01_JPRB ,&
&  0.17967E+01_JPRB, 0.17626E+01_JPRB, 0.17509E+01_JPRB, 0.17847E+01_JPRB, 0.18222E+01_JPRB, 0.19484E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.19723E+01_JPRB, 0.20311E+01_JPRB, 0.21222E+01_JPRB, 0.23922E+01_JPRB, 0.27532E+01_JPRB, 0.30350E+01_JPRB, 0.32256E+01_JPRB ,&
&  0.33793E+01_JPRB, 0.35669E+01_JPRB, 0.38522E+01_JPRB, 0.40577E+01_JPRB, 0.45030E+01_JPRB, 0.47401E+01_JPRB, 0.49338E+01_JPRB ,&
&  0.48426E+01_JPRB, 0.46784E+01_JPRB, 0.51877E+01_JPRB, 0.55945E+01_JPRB, 0.57861E+01_JPRB, 0.58087E+01_JPRB, 0.61881E+01_JPRB ,&
&  0.79766E+01_JPRB, 0.10037E+02_JPRB, 0.12275E+02_JPRB, 0.11063E+02_JPRB, 0.10641E+02_JPRB, 0.11759E+02_JPRB, 0.12819E+02_JPRB ,&
&  0.11000E+02_JPRB, 0.91315E+01_JPRB, 0.89796E+01_JPRB, 0.91878E+01_JPRB, 0.98580E+01_JPRB, 0.10772E+02_JPRB, 0.14074E+02_JPRB ,&
&  0.16560E+02_JPRB, 0.12776E+02_JPRB, 0.74218E+01_JPRB, 0.40895E+01_JPRB, 0.30533E+01_JPRB, 0.32436E+01_JPRB, 0.41999E+01_JPRB ,&
&  0.67828E+01_JPRB, 0.82645E+01_JPRB, 0.95152E+01_JPRB, 0.95061E+01_JPRB, 0.51941E+01_JPRB, 0.34250E+01_JPRB, 0.26763E+01_JPRB ,&
&  0.19017E+01_JPRB, 0.15161E+01_JPRB, 0.14786E+01_JPRB, 0.14276E+01_JPRB, 0.13582E+01_JPRB, 0.14251E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.14947E+01_JPRB, 0.13275E+01_JPRB, 0.13356E+01_JPRB, 0.13288E+01_JPRB, 0.13306E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.14204E+01_JPRB, 0.14616E+01_JPRB, 0.15508E+01_JPRB, 0.16930E+01_JPRB, 0.18754E+01_JPRB, 0.20766E+01_JPRB, 0.21913E+01_JPRB ,&
&  0.23458E+01_JPRB, 0.25847E+01_JPRB, 0.28111E+01_JPRB, 0.30357E+01_JPRB, 0.32679E+01_JPRB, 0.35436E+01_JPRB, 0.38433E+01_JPRB ,&
&  0.41674E+01_JPRB, 0.44754E+01_JPRB, 0.47885E+01_JPRB, 0.51984E+01_JPRB, 0.57299E+01_JPRB, 0.60161E+01_JPRB, 0.44430E+01_JPRB ,&
&  0.29323E+01_JPRB, 0.40490E+01_JPRB, 0.27101E+01_JPRB, 0.29618E+01_JPRB, 0.31543E+01_JPRB, 0.34373E+01_JPRB, 0.26668E+01_JPRB ,&
&  0.19611E+01_JPRB, 0.18844E+01_JPRB, 0.18925E+01_JPRB, 0.18613E+01_JPRB, 0.17793E+01_JPRB, 0.19931E+01_JPRB, 0.22350E+01_JPRB ,&
&  0.23640E+01_JPRB, 0.25320E+01_JPRB, 0.27662E+01_JPRB, 0.30815E+01_JPRB, 0.33119E+01_JPRB, 0.35003E+01_JPRB, 0.36494E+01_JPRB ,&
&  0.37979E+01_JPRB, 0.40112E+01_JPRB, 0.41882E+01_JPRB, 0.44640E+01_JPRB, 0.46222E+01_JPRB, 0.48859E+01_JPRB, 0.50646E+01_JPRB ,&
&  0.49486E+01_JPRB, 0.47859E+01_JPRB, 0.50424E+01_JPRB, 0.51226E+01_JPRB, 0.54950E+01_JPRB, 0.65848E+01_JPRB, 0.73508E+01_JPRB ,&
&  0.93709E+01_JPRB, 0.10433E+02_JPRB, 0.12522E+02_JPRB, 0.10208E+02_JPRB, 0.94617E+01_JPRB, 0.10092E+02_JPRB, 0.98147E+01_JPRB ,&
&  0.88699E+01_JPRB, 0.93370E+01_JPRB, 0.95537E+01_JPRB, 0.10020E+02_JPRB, 0.10601E+02_JPRB, 0.11236E+02_JPRB, 0.12543E+02_JPRB ,&
&  0.10874E+02_JPRB, 0.58118E+01_JPRB, 0.37550E+01_JPRB, 0.29262E+01_JPRB, 0.25864E+01_JPRB, 0.30308E+01_JPRB, 0.48157E+01_JPRB ,&
&  0.74491E+01_JPRB, 0.80661E+01_JPRB, 0.87130E+01_JPRB, 0.61510E+01_JPRB, 0.29056E+01_JPRB, 0.19925E+01_JPRB, 0.15197E+01_JPRB ,&
&  0.11627E+01_JPRB, 0.10952E+01_JPRB, 0.12003E+01_JPRB, 0.12383E+01_JPRB, 0.12045E+01_JPRB, 0.12653E+01_JPRB, 0.13395E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.12963E+01_JPRB, 0.13706E+01_JPRB, 0.13587E+01_JPRB, 0.13783E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.14478E+01_JPRB, 0.15152E+01_JPRB, 0.16679E+01_JPRB, 0.18784E+01_JPRB, 0.21890E+01_JPRB, 0.25510E+01_JPRB, 0.27708E+01_JPRB ,&
&  0.27028E+01_JPRB, 0.26271E+01_JPRB, 0.29531E+01_JPRB, 0.32590E+01_JPRB, 0.34875E+01_JPRB, 0.37672E+01_JPRB, 0.40596E+01_JPRB ,&
&  0.44007E+01_JPRB, 0.47561E+01_JPRB, 0.51510E+01_JPRB, 0.56361E+01_JPRB, 0.62049E+01_JPRB, 0.67484E+01_JPRB, 0.65914E+01_JPRB ,&
&  0.46141E+01_JPRB, 0.33742E+01_JPRB, 0.33341E+01_JPRB, 0.30512E+01_JPRB, 0.29820E+01_JPRB, 0.32887E+01_JPRB, 0.32654E+01_JPRB ,&
&  0.26245E+01_JPRB, 0.22005E+01_JPRB, 0.22060E+01_JPRB, 0.21845E+01_JPRB, 0.16008E+01_JPRB, 0.18067E+01_JPRB, 0.21895E+01_JPRB ,&
&  0.24259E+01_JPRB, 0.26520E+01_JPRB, 0.29393E+01_JPRB, 0.32391E+01_JPRB, 0.34505E+01_JPRB, 0.37136E+01_JPRB, 0.39542E+01_JPRB ,&
&  0.41484E+01_JPRB, 0.43746E+01_JPRB, 0.44884E+01_JPRB, 0.45306E+01_JPRB, 0.47363E+01_JPRB, 0.49739E+01_JPRB, 0.50802E+01_JPRB ,&
&  0.51149E+01_JPRB, 0.53609E+01_JPRB, 0.56468E+01_JPRB, 0.59775E+01_JPRB, 0.62402E+01_JPRB, 0.74865E+01_JPRB, 0.83925E+01_JPRB ,&
&  0.94345E+01_JPRB, 0.97611E+01_JPRB, 0.10815E+02_JPRB, 0.87679E+01_JPRB, 0.83351E+01_JPRB, 0.83598E+01_JPRB, 0.83464E+01_JPRB ,&
&  0.81332E+01_JPRB, 0.87193E+01_JPRB, 0.89437E+01_JPRB, 0.97501E+01_JPRB, 0.10637E+02_JPRB, 0.10545E+02_JPRB, 0.11036E+02_JPRB ,&
&  0.67975E+01_JPRB, 0.39896E+01_JPRB, 0.24978E+01_JPRB, 0.25233E+01_JPRB, 0.26666E+01_JPRB, 0.32509E+01_JPRB, 0.43814E+01_JPRB ,&
&  0.58996E+01_JPRB, 0.48522E+01_JPRB, 0.44686E+01_JPRB, 0.38374E+01_JPRB, 0.19534E+01_JPRB, 0.13059E+01_JPRB, 0.10617E+01_JPRB ,&
&  0.95636E+00_JPRB, 0.97907E+00_JPRB, 0.10558E+01_JPRB, 0.10701E+01_JPRB, 0.10425E+01_JPRB, 0.10981E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.12275E+01_JPRB, 0.12615E+01_JPRB, 0.13434E+01_JPRB, 0.13529E+01_JPRB, 0.14154E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.13580E+01_JPRB, 0.14815E+01_JPRB, 0.15922E+01_JPRB, 0.17271E+01_JPRB, 0.18840E+01_JPRB, 0.20997E+01_JPRB, 0.22205E+01_JPRB ,&
&  0.22489E+01_JPRB, 0.22909E+01_JPRB, 0.25120E+01_JPRB, 0.27740E+01_JPRB, 0.30010E+01_JPRB, 0.31405E+01_JPRB, 0.33011E+01_JPRB ,&
&  0.34638E+01_JPRB, 0.36175E+01_JPRB, 0.37792E+01_JPRB, 0.39344E+01_JPRB, 0.41887E+01_JPRB, 0.45548E+01_JPRB, 0.48970E+01_JPRB ,&
&  0.43448E+01_JPRB, 0.27378E+01_JPRB, 0.30308E+01_JPRB, 0.29022E+01_JPRB, 0.29348E+01_JPRB, 0.30718E+01_JPRB, 0.32451E+01_JPRB ,&
&  0.26274E+01_JPRB, 0.23481E+01_JPRB, 0.23595E+01_JPRB, 0.24010E+01_JPRB, 0.15215E+01_JPRB, 0.16090E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.22389E+01_JPRB, 0.25430E+01_JPRB, 0.28443E+01_JPRB, 0.30925E+01_JPRB, 0.33446E+01_JPRB, 0.36698E+01_JPRB, 0.39771E+01_JPRB ,&
&  0.42360E+01_JPRB, 0.44346E+01_JPRB, 0.45810E+01_JPRB, 0.47292E+01_JPRB, 0.48934E+01_JPRB, 0.51420E+01_JPRB, 0.52472E+01_JPRB ,&
&  0.54399E+01_JPRB, 0.56018E+01_JPRB, 0.62050E+01_JPRB, 0.69492E+01_JPRB, 0.75480E+01_JPRB, 0.75138E+01_JPRB, 0.80736E+01_JPRB ,&
&  0.82998E+01_JPRB, 0.81814E+01_JPRB, 0.84679E+01_JPRB, 0.86920E+01_JPRB, 0.63947E+01_JPRB, 0.63966E+01_JPRB, 0.66451E+01_JPRB ,&
&  0.75117E+01_JPRB, 0.75959E+01_JPRB, 0.79340E+01_JPRB, 0.84568E+01_JPRB, 0.87118E+01_JPRB, 0.87012E+01_JPRB, 0.80718E+01_JPRB ,&
&  0.54108E+01_JPRB, 0.30617E+01_JPRB, 0.22294E+01_JPRB, 0.24438E+01_JPRB, 0.30116E+01_JPRB, 0.42467E+01_JPRB, 0.36859E+01_JPRB ,&
&  0.41800E+01_JPRB, 0.28668E+01_JPRB, 0.27482E+01_JPRB, 0.32624E+01_JPRB, 0.18856E+01_JPRB, 0.12474E+01_JPRB, 0.99116E+00_JPRB ,&
&  0.90942E+00_JPRB, 0.93654E+00_JPRB, 0.94142E+00_JPRB, 0.93068E+00_JPRB, 0.92564E+00_JPRB, 0.98414E+00_JPRB, 0.10615E+01_JPRB ,&
&  0.11001E+01_JPRB, 0.11439E+01_JPRB, 0.12015E+01_JPRB, 0.12632E+01_JPRB, 0.13405E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.10828E+01_JPRB, 0.11147E+01_JPRB, 0.11859E+01_JPRB, 0.13071E+01_JPRB, 0.14258E+01_JPRB, 0.15473E+01_JPRB, 0.16455E+01_JPRB ,&
&  0.17976E+01_JPRB, 0.19301E+01_JPRB, 0.20704E+01_JPRB, 0.21764E+01_JPRB, 0.22653E+01_JPRB, 0.24528E+01_JPRB, 0.25720E+01_JPRB ,&
&  0.25988E+01_JPRB, 0.25504E+01_JPRB, 0.25498E+01_JPRB, 0.25997E+01_JPRB, 0.27936E+01_JPRB, 0.31441E+01_JPRB, 0.35689E+01_JPRB ,&
&  0.40102E+01_JPRB, 0.37499E+01_JPRB, 0.32754E+01_JPRB, 0.20924E+01_JPRB, 0.17801E+01_JPRB, 0.21076E+01_JPRB, 0.21224E+01_JPRB ,&
&  0.22221E+01_JPRB, 0.22597E+01_JPRB, 0.19430E+01_JPRB, 0.17400E+01_JPRB, 0.15491E+01_JPRB, 0.15012E+01_JPRB, 0.16703E+01_JPRB ,&
&  0.18994E+01_JPRB, 0.21706E+01_JPRB, 0.24739E+01_JPRB, 0.27056E+01_JPRB, 0.29559E+01_JPRB, 0.32907E+01_JPRB, 0.36538E+01_JPRB ,&
&  0.39630E+01_JPRB, 0.42240E+01_JPRB, 0.44370E+01_JPRB, 0.47775E+01_JPRB, 0.50197E+01_JPRB, 0.53522E+01_JPRB, 0.56034E+01_JPRB ,&
&  0.58912E+01_JPRB, 0.60592E+01_JPRB, 0.65647E+01_JPRB, 0.71248E+01_JPRB, 0.72535E+01_JPRB, 0.66874E+01_JPRB, 0.65913E+01_JPRB ,&
&  0.66842E+01_JPRB, 0.64424E+01_JPRB, 0.65025E+01_JPRB, 0.70806E+01_JPRB, 0.50567E+01_JPRB, 0.53874E+01_JPRB, 0.61203E+01_JPRB ,&
&  0.61507E+01_JPRB, 0.63743E+01_JPRB, 0.62027E+01_JPRB, 0.54539E+01_JPRB, 0.52350E+01_JPRB, 0.54866E+01_JPRB, 0.53674E+01_JPRB ,&
&  0.41997E+01_JPRB, 0.27222E+01_JPRB, 0.20338E+01_JPRB, 0.18484E+01_JPRB, 0.25042E+01_JPRB, 0.42199E+01_JPRB, 0.31888E+01_JPRB ,&
&  0.25877E+01_JPRB, 0.23686E+01_JPRB, 0.22076E+01_JPRB, 0.34034E+01_JPRB, 0.24391E+01_JPRB, 0.14820E+01_JPRB, 0.10695E+01_JPRB ,&
&  0.90239E+00_JPRB, 0.87960E+00_JPRB, 0.88362E+00_JPRB, 0.86665E+00_JPRB, 0.88699E+00_JPRB, 0.93712E+00_JPRB, 0.94939E+00_JPRB ,&
&  0.10193E+01_JPRB, 0.10374E+01_JPRB, 0.10762E+01_JPRB, 0.11084E+01_JPRB, 0.10768E+01_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.73769E+00_JPRB, 0.73167E+00_JPRB, 0.76638E+00_JPRB, 0.87342E+00_JPRB, 0.95662E+00_JPRB, 0.10261E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.14482E+01_JPRB, 0.16005E+01_JPRB, 0.16528E+01_JPRB, 0.16009E+01_JPRB, 0.15467E+01_JPRB, 0.15362E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.13806E+01_JPRB, 0.12519E+01_JPRB, 0.12027E+01_JPRB, 0.12493E+01_JPRB, 0.13908E+01_JPRB, 0.16132E+01_JPRB ,&
&  0.19914E+01_JPRB, 0.22854E+01_JPRB, 0.25007E+01_JPRB, 0.31317E+01_JPRB, 0.30410E+01_JPRB, 0.19430E+01_JPRB, 0.22750E+01_JPRB ,&
&  0.31796E+01_JPRB, 0.32037E+01_JPRB, 0.20499E+01_JPRB, 0.17221E+01_JPRB, 0.15332E+01_JPRB, 0.14106E+01_JPRB, 0.14508E+01_JPRB ,&
&  0.16038E+01_JPRB, 0.17711E+01_JPRB, 0.19860E+01_JPRB, 0.22590E+01_JPRB, 0.25511E+01_JPRB, 0.28190E+01_JPRB, 0.31479E+01_JPRB ,&
&  0.34339E+01_JPRB, 0.36022E+01_JPRB, 0.38111E+01_JPRB, 0.41245E+01_JPRB, 0.45663E+01_JPRB, 0.50996E+01_JPRB, 0.55897E+01_JPRB ,&
&  0.59885E+01_JPRB, 0.62150E+01_JPRB, 0.66910E+01_JPRB, 0.69806E+01_JPRB, 0.64674E+01_JPRB, 0.59181E+01_JPRB, 0.54394E+01_JPRB ,&
&  0.50426E+01_JPRB, 0.44432E+01_JPRB, 0.40711E+01_JPRB, 0.52661E+01_JPRB, 0.55320E+01_JPRB, 0.49898E+01_JPRB, 0.49725E+01_JPRB ,&
&  0.50527E+01_JPRB, 0.50419E+01_JPRB, 0.45158E+01_JPRB, 0.35306E+01_JPRB, 0.28206E+01_JPRB, 0.32592E+01_JPRB, 0.35560E+01_JPRB ,&
&  0.30322E+01_JPRB, 0.21350E+01_JPRB, 0.17004E+01_JPRB, 0.15938E+01_JPRB, 0.17306E+01_JPRB, 0.29073E+01_JPRB, 0.22458E+01_JPRB ,&
&  0.16648E+01_JPRB, 0.18273E+01_JPRB, 0.17842E+01_JPRB, 0.26328E+01_JPRB, 0.33035E+01_JPRB, 0.17982E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.86330E+00_JPRB, 0.84154E+00_JPRB, 0.84174E+00_JPRB, 0.84017E+00_JPRB, 0.84346E+00_JPRB, 0.86412E+00_JPRB, 0.86650E+00_JPRB ,&
&  0.88252E+00_JPRB, 0.86670E+00_JPRB, 0.86769E+00_JPRB, 0.82098E+00_JPRB, 0.75155E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.75226E+00_JPRB, 0.74926E+00_JPRB, 0.73698E+00_JPRB, 0.73512E+00_JPRB, 0.75049E+00_JPRB, 0.77479E+00_JPRB, 0.80955E+00_JPRB ,&
&  0.86303E+00_JPRB, 0.92361E+00_JPRB, 0.97958E+00_JPRB, 0.10399E+01_JPRB, 0.10459E+01_JPRB, 0.96445E+00_JPRB, 0.94592E+00_JPRB ,&
&  0.93387E+00_JPRB, 0.93904E+00_JPRB, 0.93004E+00_JPRB, 0.10235E+01_JPRB, 0.10413E+01_JPRB, 0.10473E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.13441E+01_JPRB, 0.14494E+01_JPRB, 0.15886E+01_JPRB, 0.25060E+01_JPRB, 0.24038E+01_JPRB, 0.19569E+01_JPRB ,&
&  0.29512E+01_JPRB, 0.29558E+01_JPRB, 0.21007E+01_JPRB, 0.15887E+01_JPRB, 0.14181E+01_JPRB, 0.12801E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.13031E+01_JPRB, 0.14197E+01_JPRB, 0.16384E+01_JPRB, 0.18635E+01_JPRB, 0.20479E+01_JPRB, 0.22227E+01_JPRB, 0.24229E+01_JPRB ,&
&  0.25685E+01_JPRB, 0.27212E+01_JPRB, 0.27894E+01_JPRB, 0.29298E+01_JPRB, 0.34536E+01_JPRB, 0.39596E+01_JPRB, 0.43602E+01_JPRB ,&
&  0.46843E+01_JPRB, 0.50019E+01_JPRB, 0.54998E+01_JPRB, 0.53468E+01_JPRB, 0.49047E+01_JPRB, 0.46151E+01_JPRB, 0.39237E+01_JPRB ,&
&  0.32340E+01_JPRB, 0.25840E+01_JPRB, 0.23883E+01_JPRB, 0.31342E+01_JPRB, 0.33592E+01_JPRB, 0.30272E+01_JPRB, 0.35664E+01_JPRB ,&
&  0.37391E+01_JPRB, 0.38407E+01_JPRB, 0.31799E+01_JPRB, 0.27457E+01_JPRB, 0.22366E+01_JPRB, 0.22485E+01_JPRB, 0.24777E+01_JPRB ,&
&  0.20402E+01_JPRB, 0.15266E+01_JPRB, 0.13231E+01_JPRB, 0.12704E+01_JPRB, 0.13256E+01_JPRB, 0.18762E+01_JPRB, 0.21549E+01_JPRB ,&
&  0.20072E+01_JPRB, 0.17365E+01_JPRB, 0.17501E+01_JPRB, 0.26193E+01_JPRB, 0.21311E+01_JPRB, 0.12866E+01_JPRB, 0.84861E+00_JPRB ,&
&  0.74726E+00_JPRB, 0.75097E+00_JPRB, 0.74428E+00_JPRB, 0.73974E+00_JPRB, 0.75080E+00_JPRB, 0.77094E+00_JPRB, 0.75597E+00_JPRB ,&
&  0.75262E+00_JPRB, 0.73578E+00_JPRB, 0.72354E+00_JPRB, 0.71495E+00_JPRB, 0.72163E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.91298E+00_JPRB, 0.92398E+00_JPRB, 0.93640E+00_JPRB, 0.95127E+00_JPRB, 0.98079E+00_JPRB, 0.10145E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.11430E+01_JPRB, 0.11813E+01_JPRB, 0.12057E+01_JPRB, 0.12231E+01_JPRB, 0.12359E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.13042E+01_JPRB, 0.13020E+01_JPRB, 0.13076E+01_JPRB, 0.13332E+01_JPRB, 0.13490E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.13020E+01_JPRB, 0.12787E+01_JPRB, 0.13084E+01_JPRB, 0.13561E+01_JPRB, 0.15870E+01_JPRB, 0.17671E+01_JPRB, 0.17799E+01_JPRB ,&
&  0.21806E+01_JPRB, 0.23816E+01_JPRB, 0.22163E+01_JPRB, 0.15811E+01_JPRB, 0.14221E+01_JPRB, 0.13373E+01_JPRB, 0.12574E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.13025E+01_JPRB, 0.13662E+01_JPRB, 0.14382E+01_JPRB, 0.15253E+01_JPRB, 0.15596E+01_JPRB, 0.16274E+01_JPRB ,&
&  0.16476E+01_JPRB, 0.17015E+01_JPRB, 0.17881E+01_JPRB, 0.19867E+01_JPRB, 0.23612E+01_JPRB, 0.28302E+01_JPRB, 0.28955E+01_JPRB ,&
&  0.29393E+01_JPRB, 0.32096E+01_JPRB, 0.30254E+01_JPRB, 0.29505E+01_JPRB, 0.28513E+01_JPRB, 0.24673E+01_JPRB, 0.20788E+01_JPRB ,&
&  0.19885E+01_JPRB, 0.18337E+01_JPRB, 0.15098E+01_JPRB, 0.19173E+01_JPRB, 0.22266E+01_JPRB, 0.23239E+01_JPRB, 0.26744E+01_JPRB ,&
&  0.28809E+01_JPRB, 0.25149E+01_JPRB, 0.19958E+01_JPRB, 0.19590E+01_JPRB, 0.19145E+01_JPRB, 0.15904E+01_JPRB, 0.15894E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.12623E+01_JPRB, 0.12269E+01_JPRB, 0.12639E+01_JPRB, 0.16035E+01_JPRB, 0.27329E+01_JPRB, 0.29235E+01_JPRB ,&
&  0.22283E+01_JPRB, 0.18882E+01_JPRB, 0.22421E+01_JPRB, 0.34821E+01_JPRB, 0.18912E+01_JPRB, 0.12126E+01_JPRB, 0.78023E+00_JPRB ,&
&  0.75078E+00_JPRB, 0.75600E+00_JPRB, 0.76276E+00_JPRB, 0.76966E+00_JPRB, 0.77957E+00_JPRB, 0.80050E+00_JPRB, 0.79224E+00_JPRB ,&
&  0.78371E+00_JPRB, 0.77770E+00_JPRB, 0.79290E+00_JPRB, 0.81868E+00_JPRB, 0.87094E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.10547E+01_JPRB, 0.10624E+01_JPRB, 0.10853E+01_JPRB, 0.11134E+01_JPRB, 0.11411E+01_JPRB, 0.11743E+01_JPRB, 0.12234E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.13221E+01_JPRB, 0.13626E+01_JPRB, 0.13934E+01_JPRB, 0.14030E+01_JPRB, 0.14196E+01_JPRB, 0.14389E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.14513E+01_JPRB, 0.14761E+01_JPRB, 0.15170E+01_JPRB, 0.15693E+01_JPRB, 0.15916E+01_JPRB, 0.16173E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.16993E+01_JPRB, 0.17621E+01_JPRB, 0.18384E+01_JPRB, 0.19563E+01_JPRB, 0.21499E+01_JPRB, 0.23227E+01_JPRB ,&
&  0.20558E+01_JPRB, 0.25138E+01_JPRB, 0.20961E+01_JPRB, 0.16500E+01_JPRB, 0.15399E+01_JPRB, 0.15576E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.14618E+01_JPRB, 0.14816E+01_JPRB, 0.15096E+01_JPRB, 0.15367E+01_JPRB, 0.15657E+01_JPRB, 0.16142E+01_JPRB, 0.16183E+01_JPRB ,&
&  0.16545E+01_JPRB, 0.16803E+01_JPRB, 0.17120E+01_JPRB, 0.17970E+01_JPRB, 0.18071E+01_JPRB, 0.22400E+01_JPRB, 0.21658E+01_JPRB ,&
&  0.20231E+01_JPRB, 0.20397E+01_JPRB, 0.16106E+01_JPRB, 0.16522E+01_JPRB, 0.16615E+01_JPRB, 0.14438E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.18180E+01_JPRB, 0.14736E+01_JPRB, 0.17426E+01_JPRB, 0.19262E+01_JPRB, 0.20155E+01_JPRB, 0.20777E+01_JPRB ,&
&  0.20447E+01_JPRB, 0.17527E+01_JPRB, 0.15008E+01_JPRB, 0.13881E+01_JPRB, 0.13464E+01_JPRB, 0.12743E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.11532E+01_JPRB, 0.11403E+01_JPRB, 0.11462E+01_JPRB, 0.11988E+01_JPRB, 0.18612E+01_JPRB, 0.26074E+01_JPRB, 0.38197E+01_JPRB ,&
&  0.20583E+01_JPRB, 0.18068E+01_JPRB, 0.14230E+01_JPRB, 0.21094E+01_JPRB, 0.14386E+01_JPRB, 0.86122E+00_JPRB, 0.81921E+00_JPRB ,&
&  0.87544E+00_JPRB, 0.91744E+00_JPRB, 0.92053E+00_JPRB, 0.90805E+00_JPRB, 0.90984E+00_JPRB, 0.91557E+00_JPRB, 0.89106E+00_JPRB ,&
&  0.87760E+00_JPRB, 0.89672E+00_JPRB, 0.93695E+00_JPRB, 0.95954E+00_JPRB, 0.10083E+01_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.11362E+01_JPRB, 0.11719E+01_JPRB, 0.12100E+01_JPRB, 0.12510E+01_JPRB, 0.12903E+01_JPRB, 0.13278E+01_JPRB, 0.13683E+01_JPRB ,&
&  0.14172E+01_JPRB, 0.14523E+01_JPRB, 0.14864E+01_JPRB, 0.15065E+01_JPRB, 0.15171E+01_JPRB, 0.15398E+01_JPRB, 0.15485E+01_JPRB ,&
&  0.15538E+01_JPRB, 0.15602E+01_JPRB, 0.15888E+01_JPRB, 0.16094E+01_JPRB, 0.16284E+01_JPRB, 0.16367E+01_JPRB, 0.16729E+01_JPRB ,&
&  0.16804E+01_JPRB, 0.17429E+01_JPRB, 0.17899E+01_JPRB, 0.18509E+01_JPRB, 0.19036E+01_JPRB, 0.19358E+01_JPRB, 0.21092E+01_JPRB ,&
&  0.19053E+01_JPRB, 0.27950E+01_JPRB, 0.25163E+01_JPRB, 0.19359E+01_JPRB, 0.18421E+01_JPRB, 0.17126E+01_JPRB, 0.16861E+01_JPRB ,&
&  0.16871E+01_JPRB, 0.16925E+01_JPRB, 0.17294E+01_JPRB, 0.17940E+01_JPRB, 0.18534E+01_JPRB, 0.19142E+01_JPRB, 0.19431E+01_JPRB ,&
&  0.19873E+01_JPRB, 0.20301E+01_JPRB, 0.20758E+01_JPRB, 0.21032E+01_JPRB, 0.21558E+01_JPRB, 0.21980E+01_JPRB, 0.21407E+01_JPRB ,&
&  0.19858E+01_JPRB, 0.18387E+01_JPRB, 0.13548E+01_JPRB, 0.13146E+01_JPRB, 0.13943E+01_JPRB, 0.13265E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.15351E+01_JPRB, 0.17005E+01_JPRB, 0.16314E+01_JPRB, 0.16673E+01_JPRB, 0.16753E+01_JPRB, 0.15983E+01_JPRB, 0.15369E+01_JPRB ,&
&  0.13782E+01_JPRB, 0.11934E+01_JPRB, 0.10757E+01_JPRB, 0.10072E+01_JPRB, 0.10198E+01_JPRB, 0.10179E+01_JPRB, 0.10019E+01_JPRB ,&
&  0.97280E+00_JPRB, 0.10194E+01_JPRB, 0.11295E+01_JPRB, 0.13029E+01_JPRB, 0.17238E+01_JPRB, 0.35080E+01_JPRB, 0.36762E+01_JPRB ,&
&  0.16379E+01_JPRB, 0.14718E+01_JPRB, 0.93354E+00_JPRB, 0.10582E+01_JPRB, 0.10595E+01_JPRB, 0.96383E+00_JPRB, 0.10623E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.11931E+01_JPRB, 0.11251E+01_JPRB, 0.10374E+01_JPRB, 0.10018E+01_JPRB, 0.99044E+00_JPRB, 0.96512E+00_JPRB ,&
&  0.96505E+00_JPRB, 0.10004E+01_JPRB, 0.10350E+01_JPRB, 0.10658E+01_JPRB, 0.11002E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.12073E+01_JPRB, 0.12417E+01_JPRB, 0.12765E+01_JPRB, 0.13253E+01_JPRB, 0.13770E+01_JPRB, 0.14259E+01_JPRB, 0.14662E+01_JPRB ,&
&  0.15082E+01_JPRB, 0.15401E+01_JPRB, 0.15763E+01_JPRB, 0.16045E+01_JPRB, 0.16268E+01_JPRB, 0.16463E+01_JPRB, 0.16566E+01_JPRB ,&
&  0.16636E+01_JPRB, 0.16793E+01_JPRB, 0.17103E+01_JPRB, 0.17427E+01_JPRB, 0.17713E+01_JPRB, 0.18001E+01_JPRB, 0.18407E+01_JPRB ,&
&  0.18609E+01_JPRB, 0.18846E+01_JPRB, 0.19384E+01_JPRB, 0.20200E+01_JPRB, 0.21267E+01_JPRB, 0.24023E+01_JPRB, 0.23187E+01_JPRB ,&
&  0.26652E+01_JPRB, 0.29609E+01_JPRB, 0.28255E+01_JPRB, 0.22057E+01_JPRB, 0.19955E+01_JPRB, 0.19063E+01_JPRB, 0.18948E+01_JPRB ,&
&  0.18978E+01_JPRB, 0.18755E+01_JPRB, 0.18869E+01_JPRB, 0.19446E+01_JPRB, 0.20342E+01_JPRB, 0.21331E+01_JPRB, 0.21870E+01_JPRB ,&
&  0.22575E+01_JPRB, 0.23397E+01_JPRB, 0.24302E+01_JPRB, 0.25142E+01_JPRB, 0.26455E+01_JPRB, 0.27171E+01_JPRB, 0.26756E+01_JPRB ,&
&  0.25471E+01_JPRB, 0.24474E+01_JPRB, 0.20846E+01_JPRB, 0.19761E+01_JPRB, 0.20334E+01_JPRB, 0.17979E+01_JPRB, 0.16016E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.15255E+01_JPRB, 0.13683E+01_JPRB, 0.15225E+01_JPRB, 0.14636E+01_JPRB, 0.13172E+01_JPRB, 0.12412E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.90998E+00_JPRB, 0.80681E+00_JPRB, 0.78796E+00_JPRB, 0.81022E+00_JPRB, 0.86137E+00_JPRB, 0.88859E+00_JPRB ,&
&  0.94886E+00_JPRB, 0.10510E+01_JPRB, 0.11753E+01_JPRB, 0.13823E+01_JPRB, 0.18312E+01_JPRB, 0.18845E+01_JPRB, 0.21522E+01_JPRB ,&
&  0.14801E+01_JPRB, 0.12122E+01_JPRB, 0.12308E+01_JPRB, 0.11357E+01_JPRB, 0.11802E+01_JPRB, 0.11841E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.16704E+01_JPRB, 0.15385E+01_JPRB, 0.13415E+01_JPRB, 0.11465E+01_JPRB, 0.10659E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.10812E+01_JPRB, 0.11163E+01_JPRB, 0.11410E+01_JPRB, 0.11726E+01_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.12148E+01_JPRB, 0.12504E+01_JPRB, 0.12807E+01_JPRB, 0.13185E+01_JPRB, 0.13638E+01_JPRB, 0.14141E+01_JPRB, 0.14549E+01_JPRB ,&
&  0.14986E+01_JPRB, 0.15362E+01_JPRB, 0.15755E+01_JPRB, 0.16185E+01_JPRB, 0.16579E+01_JPRB, 0.17056E+01_JPRB, 0.17394E+01_JPRB ,&
&  0.17845E+01_JPRB, 0.18062E+01_JPRB, 0.18423E+01_JPRB, 0.18908E+01_JPRB, 0.19478E+01_JPRB, 0.20132E+01_JPRB, 0.20854E+01_JPRB ,&
&  0.21308E+01_JPRB, 0.21482E+01_JPRB, 0.21717E+01_JPRB, 0.22052E+01_JPRB, 0.22213E+01_JPRB, 0.24266E+01_JPRB, 0.24238E+01_JPRB ,&
&  0.30732E+01_JPRB, 0.33511E+01_JPRB, 0.31175E+01_JPRB, 0.25022E+01_JPRB, 0.22850E+01_JPRB, 0.21740E+01_JPRB, 0.21410E+01_JPRB ,&
&  0.21088E+01_JPRB, 0.20825E+01_JPRB, 0.20349E+01_JPRB, 0.20519E+01_JPRB, 0.21299E+01_JPRB, 0.22747E+01_JPRB, 0.24040E+01_JPRB ,&
&  0.24993E+01_JPRB, 0.26245E+01_JPRB, 0.27850E+01_JPRB, 0.29696E+01_JPRB, 0.31942E+01_JPRB, 0.33200E+01_JPRB, 0.33595E+01_JPRB ,&
&  0.33399E+01_JPRB, 0.32924E+01_JPRB, 0.29941E+01_JPRB, 0.30955E+01_JPRB, 0.31629E+01_JPRB, 0.25558E+01_JPRB, 0.20683E+01_JPRB ,&
&  0.17068E+01_JPRB, 0.14660E+01_JPRB, 0.14463E+01_JPRB, 0.14432E+01_JPRB, 0.13065E+01_JPRB, 0.12107E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.91068E+00_JPRB, 0.80896E+00_JPRB, 0.75421E+00_JPRB, 0.71461E+00_JPRB, 0.72640E+00_JPRB, 0.77060E+00_JPRB, 0.87475E+00_JPRB ,&
&  0.97289E+00_JPRB, 0.10846E+01_JPRB, 0.12362E+01_JPRB, 0.13985E+01_JPRB, 0.16335E+01_JPRB, 0.16642E+01_JPRB, 0.18455E+01_JPRB ,&
&  0.15456E+01_JPRB, 0.14456E+01_JPRB, 0.14031E+01_JPRB, 0.10303E+01_JPRB, 0.12330E+01_JPRB, 0.13304E+01_JPRB, 0.14876E+01_JPRB ,&
&  0.15310E+01_JPRB, 0.20073E+01_JPRB, 0.23475E+01_JPRB, 0.19817E+01_JPRB, 0.14860E+01_JPRB, 0.11141E+01_JPRB, 0.10331E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10893E+01_JPRB, 0.11287E+01_JPRB, 0.11547E+01_JPRB, 0.11840E+01_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.11508E+01_JPRB, 0.11836E+01_JPRB, 0.12151E+01_JPRB, 0.12404E+01_JPRB, 0.12582E+01_JPRB, 0.12886E+01_JPRB, 0.13344E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.14225E+01_JPRB, 0.14690E+01_JPRB, 0.15170E+01_JPRB, 0.15675E+01_JPRB, 0.16226E+01_JPRB, 0.16938E+01_JPRB ,&
&  0.17893E+01_JPRB, 0.18841E+01_JPRB, 0.19644E+01_JPRB, 0.20771E+01_JPRB, 0.21862E+01_JPRB, 0.22909E+01_JPRB, 0.24513E+01_JPRB ,&
&  0.25973E+01_JPRB, 0.27478E+01_JPRB, 0.28680E+01_JPRB, 0.29418E+01_JPRB, 0.29357E+01_JPRB, 0.31521E+01_JPRB, 0.29085E+01_JPRB ,&
&  0.34277E+01_JPRB, 0.37071E+01_JPRB, 0.33964E+01_JPRB, 0.27757E+01_JPRB, 0.25770E+01_JPRB, 0.24331E+01_JPRB, 0.23174E+01_JPRB ,&
&  0.22685E+01_JPRB, 0.21779E+01_JPRB, 0.20180E+01_JPRB, 0.20712E+01_JPRB, 0.21473E+01_JPRB, 0.22964E+01_JPRB, 0.24983E+01_JPRB ,&
&  0.26616E+01_JPRB, 0.28437E+01_JPRB, 0.30374E+01_JPRB, 0.32478E+01_JPRB, 0.35124E+01_JPRB, 0.37272E+01_JPRB, 0.38922E+01_JPRB ,&
&  0.40181E+01_JPRB, 0.41019E+01_JPRB, 0.41627E+01_JPRB, 0.39185E+01_JPRB, 0.38807E+01_JPRB, 0.34111E+01_JPRB, 0.25555E+01_JPRB ,&
&  0.16718E+01_JPRB, 0.14186E+01_JPRB, 0.14270E+01_JPRB, 0.13919E+01_JPRB, 0.12769E+01_JPRB, 0.11033E+01_JPRB, 0.10218E+01_JPRB ,&
&  0.85752E+00_JPRB, 0.76323E+00_JPRB, 0.73565E+00_JPRB, 0.71598E+00_JPRB, 0.71728E+00_JPRB, 0.78076E+00_JPRB, 0.84809E+00_JPRB ,&
&  0.96074E+00_JPRB, 0.11082E+01_JPRB, 0.12588E+01_JPRB, 0.14284E+01_JPRB, 0.17066E+01_JPRB, 0.19552E+01_JPRB, 0.17208E+01_JPRB ,&
&  0.16245E+01_JPRB, 0.13932E+01_JPRB, 0.13602E+01_JPRB, 0.14673E+01_JPRB, 0.13549E+01_JPRB, 0.15646E+01_JPRB, 0.17920E+01_JPRB ,&
&  0.21295E+01_JPRB, 0.20214E+01_JPRB, 0.22769E+01_JPRB, 0.32891E+01_JPRB, 0.27868E+01_JPRB, 0.23975E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.10570E+01_JPRB, 0.11045E+01_JPRB, 0.11241E+01_JPRB, 0.11432E+01_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.98904E+00_JPRB, 0.10417E+01_JPRB, 0.10647E+01_JPRB, 0.11173E+01_JPRB, 0.11326E+01_JPRB, 0.11403E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.12189E+01_JPRB, 0.12572E+01_JPRB, 0.13091E+01_JPRB, 0.13623E+01_JPRB, 0.14104E+01_JPRB, 0.14479E+01_JPRB, 0.15038E+01_JPRB ,&
&  0.15866E+01_JPRB, 0.16889E+01_JPRB, 0.18195E+01_JPRB, 0.19910E+01_JPRB, 0.21307E+01_JPRB, 0.22821E+01_JPRB, 0.24720E+01_JPRB ,&
&  0.26452E+01_JPRB, 0.28937E+01_JPRB, 0.31344E+01_JPRB, 0.33623E+01_JPRB, 0.35157E+01_JPRB, 0.35616E+01_JPRB, 0.27350E+01_JPRB ,&
&  0.37217E+01_JPRB, 0.36680E+01_JPRB, 0.34788E+01_JPRB, 0.29995E+01_JPRB, 0.27910E+01_JPRB, 0.26215E+01_JPRB, 0.24822E+01_JPRB ,&
&  0.23739E+01_JPRB, 0.22225E+01_JPRB, 0.20483E+01_JPRB, 0.19335E+01_JPRB, 0.21365E+01_JPRB, 0.22145E+01_JPRB, 0.24072E+01_JPRB ,&
&  0.26078E+01_JPRB, 0.28036E+01_JPRB, 0.29975E+01_JPRB, 0.32042E+01_JPRB, 0.34772E+01_JPRB, 0.37874E+01_JPRB, 0.40796E+01_JPRB ,&
&  0.43419E+01_JPRB, 0.44897E+01_JPRB, 0.47040E+01_JPRB, 0.42097E+01_JPRB, 0.40790E+01_JPRB, 0.38334E+01_JPRB, 0.24642E+01_JPRB ,&
&  0.15592E+01_JPRB, 0.14482E+01_JPRB, 0.15113E+01_JPRB, 0.14521E+01_JPRB, 0.12831E+01_JPRB, 0.10935E+01_JPRB, 0.95717E+00_JPRB ,&
&  0.82905E+00_JPRB, 0.79075E+00_JPRB, 0.76175E+00_JPRB, 0.74742E+00_JPRB, 0.75063E+00_JPRB, 0.80089E+00_JPRB, 0.85523E+00_JPRB ,&
&  0.96607E+00_JPRB, 0.10607E+01_JPRB, 0.11697E+01_JPRB, 0.12834E+01_JPRB, 0.14487E+01_JPRB, 0.16483E+01_JPRB, 0.18372E+01_JPRB ,&
&  0.15275E+01_JPRB, 0.13281E+01_JPRB, 0.12732E+01_JPRB, 0.13070E+01_JPRB, 0.13210E+01_JPRB, 0.14017E+01_JPRB, 0.15056E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.17504E+01_JPRB, 0.15799E+01_JPRB, 0.13082E+01_JPRB, 0.15933E+01_JPRB, 0.16236E+01_JPRB, 0.83461E+00_JPRB ,&
&  0.84112E+00_JPRB, 0.88802E+00_JPRB, 0.92349E+00_JPRB, 0.96085E+00_JPRB, 0.99499E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.80605E+00_JPRB, 0.82227E+00_JPRB, 0.83045E+00_JPRB, 0.88818E+00_JPRB, 0.91222E+00_JPRB, 0.95123E+00_JPRB, 0.98351E+00_JPRB ,&
&  0.10245E+01_JPRB, 0.10776E+01_JPRB, 0.11381E+01_JPRB, 0.11960E+01_JPRB, 0.12501E+01_JPRB, 0.12959E+01_JPRB, 0.13413E+01_JPRB ,&
&  0.13891E+01_JPRB, 0.14451E+01_JPRB, 0.15192E+01_JPRB, 0.16224E+01_JPRB, 0.17812E+01_JPRB, 0.19661E+01_JPRB, 0.21703E+01_JPRB ,&
&  0.23859E+01_JPRB, 0.26584E+01_JPRB, 0.29607E+01_JPRB, 0.32859E+01_JPRB, 0.36209E+01_JPRB, 0.38617E+01_JPRB, 0.40399E+01_JPRB ,&
&  0.29761E+01_JPRB, 0.35513E+01_JPRB, 0.34578E+01_JPRB, 0.31471E+01_JPRB, 0.29449E+01_JPRB, 0.28334E+01_JPRB, 0.26877E+01_JPRB ,&
&  0.25508E+01_JPRB, 0.23099E+01_JPRB, 0.21736E+01_JPRB, 0.20334E+01_JPRB, 0.20684E+01_JPRB, 0.21152E+01_JPRB, 0.22480E+01_JPRB ,&
&  0.24412E+01_JPRB, 0.25947E+01_JPRB, 0.27802E+01_JPRB, 0.29674E+01_JPRB, 0.32085E+01_JPRB, 0.35860E+01_JPRB, 0.40209E+01_JPRB ,&
&  0.44940E+01_JPRB, 0.48699E+01_JPRB, 0.51445E+01_JPRB, 0.45394E+01_JPRB, 0.41668E+01_JPRB, 0.38021E+01_JPRB, 0.24205E+01_JPRB ,&
&  0.16626E+01_JPRB, 0.14257E+01_JPRB, 0.15084E+01_JPRB, 0.15423E+01_JPRB, 0.13345E+01_JPRB, 0.11592E+01_JPRB, 0.99692E+00_JPRB ,&
&  0.87711E+00_JPRB, 0.81360E+00_JPRB, 0.79322E+00_JPRB, 0.77430E+00_JPRB, 0.78309E+00_JPRB, 0.82109E+00_JPRB, 0.86024E+00_JPRB ,&
&  0.93231E+00_JPRB, 0.97038E+00_JPRB, 0.10503E+01_JPRB, 0.11171E+01_JPRB, 0.11857E+01_JPRB, 0.12325E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.12767E+01_JPRB, 0.12825E+01_JPRB, 0.12767E+01_JPRB, 0.12955E+01_JPRB, 0.12892E+01_JPRB, 0.12955E+01_JPRB, 0.13177E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.13542E+01_JPRB, 0.13043E+01_JPRB, 0.12431E+01_JPRB, 0.11424E+01_JPRB, 0.10577E+01_JPRB, 0.82697E+00_JPRB ,&
&  0.78576E+00_JPRB, 0.74563E+00_JPRB, 0.77552E+00_JPRB, 0.81062E+00_JPRB, 0.80704E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.71327E+00_JPRB, 0.70392E+00_JPRB, 0.69337E+00_JPRB, 0.70808E+00_JPRB, 0.72427E+00_JPRB, 0.76236E+00_JPRB, 0.77020E+00_JPRB ,&
&  0.79806E+00_JPRB, 0.87056E+00_JPRB, 0.96534E+00_JPRB, 0.10347E+01_JPRB, 0.10910E+01_JPRB, 0.11468E+01_JPRB, 0.12022E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.12637E+01_JPRB, 0.12886E+01_JPRB, 0.13224E+01_JPRB, 0.13875E+01_JPRB, 0.14934E+01_JPRB, 0.16627E+01_JPRB ,&
&  0.18479E+01_JPRB, 0.20620E+01_JPRB, 0.23316E+01_JPRB, 0.26600E+01_JPRB, 0.30159E+01_JPRB, 0.32539E+01_JPRB, 0.34520E+01_JPRB ,&
&  0.19615E+01_JPRB, 0.29986E+01_JPRB, 0.36269E+01_JPRB, 0.33884E+01_JPRB, 0.31274E+01_JPRB, 0.29812E+01_JPRB, 0.27934E+01_JPRB ,&
&  0.25130E+01_JPRB, 0.22289E+01_JPRB, 0.20937E+01_JPRB, 0.21895E+01_JPRB, 0.20189E+01_JPRB, 0.20270E+01_JPRB, 0.20599E+01_JPRB ,&
&  0.21993E+01_JPRB, 0.23022E+01_JPRB, 0.23565E+01_JPRB, 0.24376E+01_JPRB, 0.25715E+01_JPRB, 0.28841E+01_JPRB, 0.34248E+01_JPRB ,&
&  0.41672E+01_JPRB, 0.49601E+01_JPRB, 0.53969E+01_JPRB, 0.46222E+01_JPRB, 0.47438E+01_JPRB, 0.43493E+01_JPRB, 0.35569E+01_JPRB ,&
&  0.21043E+01_JPRB, 0.15385E+01_JPRB, 0.15744E+01_JPRB, 0.15974E+01_JPRB, 0.14142E+01_JPRB, 0.12735E+01_JPRB, 0.11299E+01_JPRB ,&
&  0.98487E+00_JPRB, 0.90904E+00_JPRB, 0.86470E+00_JPRB, 0.83888E+00_JPRB, 0.83388E+00_JPRB, 0.84863E+00_JPRB, 0.87828E+00_JPRB ,&
&  0.90135E+00_JPRB, 0.92814E+00_JPRB, 0.97285E+00_JPRB, 0.10236E+01_JPRB, 0.10811E+01_JPRB, 0.11305E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.12395E+01_JPRB, 0.12634E+01_JPRB, 0.12802E+01_JPRB, 0.12883E+01_JPRB, 0.12955E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12795E+01_JPRB, 0.12769E+01_JPRB, 0.12523E+01_JPRB, 0.11836E+01_JPRB, 0.11170E+01_JPRB, 0.10560E+01_JPRB ,&
&  0.95846E+00_JPRB, 0.83829E+00_JPRB, 0.74413E+00_JPRB, 0.72566E+00_JPRB, 0.71623E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.67781E+00_JPRB, 0.64931E+00_JPRB, 0.64466E+00_JPRB, 0.65421E+00_JPRB, 0.67565E+00_JPRB, 0.68970E+00_JPRB, 0.69155E+00_JPRB ,&
&  0.69951E+00_JPRB, 0.72124E+00_JPRB, 0.74689E+00_JPRB, 0.82000E+00_JPRB, 0.90872E+00_JPRB, 0.98321E+00_JPRB, 0.10351E+01_JPRB ,&
&  0.10757E+01_JPRB, 0.10896E+01_JPRB, 0.10841E+01_JPRB, 0.10854E+01_JPRB, 0.10915E+01_JPRB, 0.11208E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.14912E+01_JPRB, 0.17030E+01_JPRB, 0.20026E+01_JPRB, 0.24173E+01_JPRB, 0.28581E+01_JPRB, 0.31949E+01_JPRB ,&
&  0.23198E+01_JPRB, 0.10831E+01_JPRB, 0.32589E+01_JPRB, 0.39286E+01_JPRB, 0.34641E+01_JPRB, 0.32797E+01_JPRB, 0.29252E+01_JPRB ,&
&  0.24772E+01_JPRB, 0.22605E+01_JPRB, 0.20911E+01_JPRB, 0.21196E+01_JPRB, 0.19830E+01_JPRB, 0.19785E+01_JPRB, 0.20014E+01_JPRB ,&
&  0.20563E+01_JPRB, 0.21530E+01_JPRB, 0.21613E+01_JPRB, 0.21268E+01_JPRB, 0.21195E+01_JPRB, 0.22220E+01_JPRB, 0.25740E+01_JPRB ,&
&  0.33055E+01_JPRB, 0.44007E+01_JPRB, 0.54280E+01_JPRB, 0.48504E+01_JPRB, 0.50050E+01_JPRB, 0.39582E+01_JPRB, 0.28728E+01_JPRB ,&
&  0.21414E+01_JPRB, 0.17187E+01_JPRB, 0.16567E+01_JPRB, 0.16649E+01_JPRB, 0.15221E+01_JPRB, 0.13463E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.10226E+01_JPRB, 0.94793E+00_JPRB, 0.90797E+00_JPRB, 0.89132E+00_JPRB, 0.89436E+00_JPRB, 0.90691E+00_JPRB ,&
&  0.91140E+00_JPRB, 0.92337E+00_JPRB, 0.94327E+00_JPRB, 0.98392E+00_JPRB, 0.10357E+01_JPRB, 0.11008E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.11957E+01_JPRB, 0.12165E+01_JPRB, 0.12269E+01_JPRB, 0.12359E+01_JPRB, 0.12134E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.12418E+01_JPRB, 0.12623E+01_JPRB, 0.12925E+01_JPRB, 0.12466E+01_JPRB, 0.12012E+01_JPRB, 0.12013E+01_JPRB ,&
&  0.11638E+01_JPRB, 0.10565E+01_JPRB, 0.89861E+00_JPRB, 0.76411E+00_JPRB, 0.71105E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.76253E+00_JPRB, 0.72327E+00_JPRB, 0.69078E+00_JPRB, 0.68481E+00_JPRB, 0.70121E+00_JPRB, 0.69728E+00_JPRB, 0.69569E+00_JPRB ,&
&  0.69069E+00_JPRB, 0.68593E+00_JPRB, 0.67234E+00_JPRB, 0.68544E+00_JPRB, 0.73209E+00_JPRB, 0.80972E+00_JPRB, 0.86694E+00_JPRB ,&
&  0.91612E+00_JPRB, 0.93079E+00_JPRB, 0.92733E+00_JPRB, 0.92206E+00_JPRB, 0.91592E+00_JPRB, 0.91693E+00_JPRB, 0.93259E+00_JPRB ,&
&  0.97885E+00_JPRB, 0.10563E+01_JPRB, 0.11628E+01_JPRB, 0.13086E+01_JPRB, 0.15340E+01_JPRB, 0.19558E+01_JPRB, 0.26174E+01_JPRB ,&
&  0.36892E+01_JPRB, 0.26266E+01_JPRB, 0.15079E+01_JPRB, 0.34488E+01_JPRB, 0.38864E+01_JPRB, 0.38419E+01_JPRB, 0.31581E+01_JPRB ,&
&  0.28613E+01_JPRB, 0.24829E+01_JPRB, 0.21792E+01_JPRB, 0.21827E+01_JPRB, 0.19637E+01_JPRB, 0.19234E+01_JPRB, 0.19633E+01_JPRB ,&
&  0.19908E+01_JPRB, 0.20214E+01_JPRB, 0.20464E+01_JPRB, 0.20053E+01_JPRB, 0.19217E+01_JPRB, 0.18741E+01_JPRB, 0.19818E+01_JPRB ,&
&  0.24312E+01_JPRB, 0.33671E+01_JPRB, 0.48877E+01_JPRB, 0.49706E+01_JPRB, 0.51126E+01_JPRB, 0.36296E+01_JPRB, 0.26691E+01_JPRB ,&
&  0.21935E+01_JPRB, 0.20925E+01_JPRB, 0.19784E+01_JPRB, 0.17872E+01_JPRB, 0.15725E+01_JPRB, 0.14415E+01_JPRB, 0.14468E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.11806E+01_JPRB, 0.10813E+01_JPRB, 0.10205E+01_JPRB, 0.98073E+00_JPRB, 0.95917E+00_JPRB, 0.94830E+00_JPRB ,&
&  0.95481E+00_JPRB, 0.95661E+00_JPRB, 0.97024E+00_JPRB, 0.99090E+00_JPRB, 0.10180E+01_JPRB, 0.10585E+01_JPRB, 0.11083E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.11593E+01_JPRB, 0.11654E+01_JPRB, 0.11637E+01_JPRB, 0.11395E+01_JPRB, 0.11239E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.11507E+01_JPRB, 0.11702E+01_JPRB, 0.11758E+01_JPRB, 0.12136E+01_JPRB, 0.12333E+01_JPRB, 0.12389E+01_JPRB, 0.12566E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.12317E+01_JPRB, 0.10899E+01_JPRB, 0.86755E+00_JPRB, 0.79917E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.86243E+00_JPRB, 0.83332E+00_JPRB, 0.81694E+00_JPRB, 0.81734E+00_JPRB, 0.81874E+00_JPRB, 0.80054E+00_JPRB, 0.77512E+00_JPRB ,&
&  0.74102E+00_JPRB, 0.71524E+00_JPRB, 0.67959E+00_JPRB, 0.64448E+00_JPRB, 0.63858E+00_JPRB, 0.66392E+00_JPRB, 0.71137E+00_JPRB ,&
&  0.75325E+00_JPRB, 0.76864E+00_JPRB, 0.77848E+00_JPRB, 0.78554E+00_JPRB, 0.79271E+00_JPRB, 0.79600E+00_JPRB, 0.79731E+00_JPRB ,&
&  0.81023E+00_JPRB, 0.83680E+00_JPRB, 0.87662E+00_JPRB, 0.93016E+00_JPRB, 0.10161E+01_JPRB, 0.11821E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.22364E+01_JPRB, 0.26636E+01_JPRB, 0.12011E+01_JPRB, 0.26490E+01_JPRB, 0.38870E+01_JPRB, 0.39223E+01_JPRB, 0.36825E+01_JPRB ,&
&  0.33310E+01_JPRB, 0.27793E+01_JPRB, 0.25422E+01_JPRB, 0.22604E+01_JPRB, 0.19870E+01_JPRB, 0.18871E+01_JPRB, 0.18635E+01_JPRB ,&
&  0.18586E+01_JPRB, 0.18026E+01_JPRB, 0.17716E+01_JPRB, 0.17763E+01_JPRB, 0.17266E+01_JPRB, 0.16501E+01_JPRB, 0.16523E+01_JPRB ,&
&  0.18620E+01_JPRB, 0.25078E+01_JPRB, 0.39353E+01_JPRB, 0.47735E+01_JPRB, 0.50482E+01_JPRB, 0.39569E+01_JPRB, 0.29783E+01_JPRB ,&
&  0.25902E+01_JPRB, 0.26309E+01_JPRB, 0.24237E+01_JPRB, 0.20285E+01_JPRB, 0.16957E+01_JPRB, 0.15869E+01_JPRB, 0.15586E+01_JPRB ,&
&  0.14308E+01_JPRB, 0.13147E+01_JPRB, 0.12360E+01_JPRB, 0.11704E+01_JPRB, 0.11184E+01_JPRB, 0.10677E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.10172E+01_JPRB, 0.10180E+01_JPRB, 0.10199E+01_JPRB, 0.10211E+01_JPRB, 0.10322E+01_JPRB, 0.10601E+01_JPRB ,&
&  0.10955E+01_JPRB, 0.11149E+01_JPRB, 0.11236E+01_JPRB, 0.11035E+01_JPRB, 0.10891E+01_JPRB, 0.10911E+01_JPRB, 0.11118E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.11404E+01_JPRB, 0.11259E+01_JPRB, 0.11340E+01_JPRB, 0.11901E+01_JPRB, 0.12068E+01_JPRB, 0.12413E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.13327E+01_JPRB, 0.12841E+01_JPRB, 0.98110E+00_JPRB, 0.91393E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.94548E+00_JPRB, 0.91183E+00_JPRB, 0.90751E+00_JPRB, 0.90974E+00_JPRB, 0.90900E+00_JPRB, 0.89866E+00_JPRB, 0.87473E+00_JPRB ,&
&  0.84348E+00_JPRB, 0.81753E+00_JPRB, 0.78771E+00_JPRB, 0.74868E+00_JPRB, 0.69309E+00_JPRB, 0.64604E+00_JPRB, 0.64352E+00_JPRB ,&
&  0.67489E+00_JPRB, 0.68157E+00_JPRB, 0.68645E+00_JPRB, 0.69462E+00_JPRB, 0.70641E+00_JPRB, 0.72027E+00_JPRB, 0.73041E+00_JPRB ,&
&  0.73641E+00_JPRB, 0.74815E+00_JPRB, 0.76471E+00_JPRB, 0.78584E+00_JPRB, 0.82218E+00_JPRB, 0.89934E+00_JPRB, 0.10567E+01_JPRB ,&
&  0.14855E+01_JPRB, 0.21559E+01_JPRB, 0.23383E+01_JPRB, 0.39367E+01_JPRB, 0.45175E+01_JPRB, 0.43135E+01_JPRB, 0.43688E+01_JPRB ,&
&  0.42874E+01_JPRB, 0.37504E+01_JPRB, 0.31674E+01_JPRB, 0.24471E+01_JPRB, 0.20700E+01_JPRB, 0.18957E+01_JPRB, 0.17950E+01_JPRB ,&
&  0.17342E+01_JPRB, 0.16581E+01_JPRB, 0.15543E+01_JPRB, 0.15055E+01_JPRB, 0.14709E+01_JPRB, 0.14305E+01_JPRB, 0.14264E+01_JPRB ,&
&  0.15190E+01_JPRB, 0.19016E+01_JPRB, 0.29254E+01_JPRB, 0.45293E+01_JPRB, 0.50197E+01_JPRB, 0.45075E+01_JPRB, 0.37218E+01_JPRB ,&
&  0.37096E+01_JPRB, 0.34958E+01_JPRB, 0.31285E+01_JPRB, 0.24885E+01_JPRB, 0.19822E+01_JPRB, 0.19286E+01_JPRB, 0.16768E+01_JPRB ,&
&  0.15175E+01_JPRB, 0.14102E+01_JPRB, 0.13457E+01_JPRB, 0.12988E+01_JPRB, 0.12572E+01_JPRB, 0.12014E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.10686E+01_JPRB, 0.10560E+01_JPRB, 0.10493E+01_JPRB, 0.10390E+01_JPRB, 0.10309E+01_JPRB, 0.10330E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10721E+01_JPRB, 0.10443E+01_JPRB, 0.10403E+01_JPRB, 0.10488E+01_JPRB, 0.10525E+01_JPRB, 0.10538E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10870E+01_JPRB, 0.11107E+01_JPRB, 0.11188E+01_JPRB, 0.11438E+01_JPRB, 0.11713E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11777E+01_JPRB, 0.11247E+01_JPRB, 0.10369E+01_JPRB, 0.99161E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.98664E+00_JPRB, 0.95628E+00_JPRB, 0.94087E+00_JPRB, 0.93186E+00_JPRB, 0.92778E+00_JPRB, 0.92336E+00_JPRB, 0.90453E+00_JPRB ,&
&  0.88275E+00_JPRB, 0.86837E+00_JPRB, 0.84474E+00_JPRB, 0.81954E+00_JPRB, 0.77108E+00_JPRB, 0.70847E+00_JPRB, 0.68015E+00_JPRB ,&
&  0.66638E+00_JPRB, 0.66189E+00_JPRB, 0.67012E+00_JPRB, 0.67528E+00_JPRB, 0.67870E+00_JPRB, 0.68654E+00_JPRB, 0.69420E+00_JPRB ,&
&  0.70353E+00_JPRB, 0.70931E+00_JPRB, 0.71923E+00_JPRB, 0.73019E+00_JPRB, 0.74845E+00_JPRB, 0.80305E+00_JPRB, 0.89966E+00_JPRB ,&
&  0.11652E+01_JPRB, 0.19444E+01_JPRB, 0.16842E+01_JPRB, 0.38266E+01_JPRB, 0.43129E+01_JPRB, 0.43672E+01_JPRB, 0.41522E+01_JPRB ,&
&  0.42100E+01_JPRB, 0.41892E+01_JPRB, 0.31807E+01_JPRB, 0.25308E+01_JPRB, 0.21023E+01_JPRB, 0.18866E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.16695E+01_JPRB, 0.15765E+01_JPRB, 0.14576E+01_JPRB, 0.13582E+01_JPRB, 0.13066E+01_JPRB, 0.12708E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.12947E+01_JPRB, 0.14975E+01_JPRB, 0.21610E+01_JPRB, 0.35900E+01_JPRB, 0.48412E+01_JPRB, 0.49853E+01_JPRB, 0.44671E+01_JPRB ,&
&  0.44936E+01_JPRB, 0.45657E+01_JPRB, 0.41259E+01_JPRB, 0.32012E+01_JPRB, 0.25431E+01_JPRB, 0.22212E+01_JPRB, 0.18902E+01_JPRB ,&
&  0.16768E+01_JPRB, 0.15459E+01_JPRB, 0.14679E+01_JPRB, 0.14018E+01_JPRB, 0.13453E+01_JPRB, 0.12852E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.10903E+01_JPRB, 0.10674E+01_JPRB, 0.10555E+01_JPRB, 0.10514E+01_JPRB, 0.10455E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.10567E+01_JPRB, 0.10122E+01_JPRB, 0.10002E+01_JPRB, 0.10091E+01_JPRB, 0.10144E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10689E+01_JPRB, 0.10805E+01_JPRB, 0.10769E+01_JPRB, 0.11026E+01_JPRB, 0.11544E+01_JPRB, 0.11486E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.11197E+01_JPRB, 0.10880E+01_JPRB, 0.10483E+01_JPRB, 0.10165E+01_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.99489E+00_JPRB, 0.97312E+00_JPRB, 0.94778E+00_JPRB, 0.92550E+00_JPRB, 0.92207E+00_JPRB, 0.92112E+00_JPRB, 0.90638E+00_JPRB ,&
&  0.89128E+00_JPRB, 0.87199E+00_JPRB, 0.84271E+00_JPRB, 0.80965E+00_JPRB, 0.77216E+00_JPRB, 0.72662E+00_JPRB, 0.68596E+00_JPRB ,&
&  0.66807E+00_JPRB, 0.65268E+00_JPRB, 0.65356E+00_JPRB, 0.66052E+00_JPRB, 0.67016E+00_JPRB, 0.67624E+00_JPRB, 0.68376E+00_JPRB ,&
&  0.69124E+00_JPRB, 0.69636E+00_JPRB, 0.70231E+00_JPRB, 0.70791E+00_JPRB, 0.71595E+00_JPRB, 0.75061E+00_JPRB, 0.85368E+00_JPRB ,&
&  0.10760E+01_JPRB, 0.23436E+01_JPRB, 0.17155E+01_JPRB, 0.31347E+01_JPRB, 0.35939E+01_JPRB, 0.38540E+01_JPRB, 0.36807E+01_JPRB ,&
&  0.37530E+01_JPRB, 0.35310E+01_JPRB, 0.29149E+01_JPRB, 0.24428E+01_JPRB, 0.21018E+01_JPRB, 0.18838E+01_JPRB, 0.17348E+01_JPRB ,&
&  0.16297E+01_JPRB, 0.15135E+01_JPRB, 0.13887E+01_JPRB, 0.12729E+01_JPRB, 0.12064E+01_JPRB, 0.11694E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.12501E+01_JPRB, 0.16204E+01_JPRB, 0.27909E+01_JPRB, 0.41532E+01_JPRB, 0.49617E+01_JPRB, 0.51118E+01_JPRB ,&
&  0.52039E+01_JPRB, 0.55666E+01_JPRB, 0.50914E+01_JPRB, 0.40521E+01_JPRB, 0.32333E+01_JPRB, 0.26452E+01_JPRB, 0.21956E+01_JPRB ,&
&  0.19201E+01_JPRB, 0.17561E+01_JPRB, 0.16439E+01_JPRB, 0.15487E+01_JPRB, 0.14533E+01_JPRB, 0.13473E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.10954E+01_JPRB, 0.10611E+01_JPRB, 0.10530E+01_JPRB, 0.10535E+01_JPRB, 0.10541E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.10637E+01_JPRB, 0.10220E+01_JPRB, 0.10037E+01_JPRB, 0.10026E+01_JPRB, 0.10113E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10658E+01_JPRB, 0.10562E+01_JPRB, 0.10590E+01_JPRB, 0.10864E+01_JPRB, 0.11593E+01_JPRB, 0.11461E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.10943E+01_JPRB, 0.10628E+01_JPRB, 0.10426E+01_JPRB, 0.10202E+01_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.99379E+00_JPRB, 0.97053E+00_JPRB, 0.95584E+00_JPRB, 0.94392E+00_JPRB, 0.94290E+00_JPRB, 0.93737E+00_JPRB, 0.91710E+00_JPRB ,&
&  0.88631E+00_JPRB, 0.85020E+00_JPRB, 0.81290E+00_JPRB, 0.78202E+00_JPRB, 0.75773E+00_JPRB, 0.73088E+00_JPRB, 0.70841E+00_JPRB ,&
&  0.68976E+00_JPRB, 0.66887E+00_JPRB, 0.65552E+00_JPRB, 0.64937E+00_JPRB, 0.65147E+00_JPRB, 0.65687E+00_JPRB, 0.66524E+00_JPRB ,&
&  0.67560E+00_JPRB, 0.68244E+00_JPRB, 0.68405E+00_JPRB, 0.68681E+00_JPRB, 0.68797E+00_JPRB, 0.71937E+00_JPRB, 0.82601E+00_JPRB ,&
&  0.10900E+01_JPRB, 0.10524E+01_JPRB, 0.19731E+01_JPRB, 0.24515E+01_JPRB, 0.27863E+01_JPRB, 0.31237E+01_JPRB, 0.33764E+01_JPRB ,&
&  0.34994E+01_JPRB, 0.31122E+01_JPRB, 0.26137E+01_JPRB, 0.22675E+01_JPRB, 0.19896E+01_JPRB, 0.17924E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.13939E+01_JPRB, 0.13068E+01_JPRB, 0.12005E+01_JPRB, 0.11295E+01_JPRB, 0.10970E+01_JPRB, 0.10828E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.10999E+01_JPRB, 0.12818E+01_JPRB, 0.19077E+01_JPRB, 0.34337E+01_JPRB, 0.46235E+01_JPRB, 0.54039E+01_JPRB ,&
&  0.63431E+01_JPRB, 0.67325E+01_JPRB, 0.60223E+01_JPRB, 0.48818E+01_JPRB, 0.37715E+01_JPRB, 0.30270E+01_JPRB, 0.24194E+01_JPRB ,&
&  0.20694E+01_JPRB, 0.18738E+01_JPRB, 0.17582E+01_JPRB, 0.16630E+01_JPRB, 0.15637E+01_JPRB, 0.14277E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.11572E+01_JPRB, 0.10876E+01_JPRB, 0.10446E+01_JPRB, 0.10211E+01_JPRB, 0.10113E+01_JPRB, 0.10148E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10614E+01_JPRB, 0.10293E+01_JPRB, 0.10043E+01_JPRB, 0.10032E+01_JPRB, 0.10198E+01_JPRB, 0.10364E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.10285E+01_JPRB, 0.10114E+01_JPRB, 0.10221E+01_JPRB, 0.10505E+01_JPRB, 0.11437E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.10930E+01_JPRB, 0.10500E+01_JPRB, 0.10330E+01_JPRB, 0.10214E+01_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.98827E+00_JPRB, 0.96806E+00_JPRB, 0.96681E+00_JPRB, 0.98322E+00_JPRB, 0.99837E+00_JPRB, 0.98364E+00_JPRB, 0.93114E+00_JPRB ,&
&  0.88058E+00_JPRB, 0.83710E+00_JPRB, 0.79700E+00_JPRB, 0.76564E+00_JPRB, 0.74462E+00_JPRB, 0.72898E+00_JPRB, 0.71308E+00_JPRB ,&
&  0.70081E+00_JPRB, 0.68935E+00_JPRB, 0.66904E+00_JPRB, 0.65056E+00_JPRB, 0.64200E+00_JPRB, 0.64042E+00_JPRB, 0.64483E+00_JPRB ,&
&  0.65038E+00_JPRB, 0.65660E+00_JPRB, 0.66063E+00_JPRB, 0.66358E+00_JPRB, 0.66549E+00_JPRB, 0.69209E+00_JPRB, 0.78381E+00_JPRB ,&
&  0.10859E+01_JPRB, 0.16865E+01_JPRB, 0.20377E+01_JPRB, 0.20721E+01_JPRB, 0.22607E+01_JPRB, 0.25964E+01_JPRB, 0.30199E+01_JPRB ,&
&  0.30203E+01_JPRB, 0.27331E+01_JPRB, 0.23022E+01_JPRB, 0.20008E+01_JPRB, 0.17708E+01_JPRB, 0.16070E+01_JPRB, 0.14533E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.12465E+01_JPRB, 0.11638E+01_JPRB, 0.10910E+01_JPRB, 0.10589E+01_JPRB, 0.10498E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10328E+01_JPRB, 0.11043E+01_JPRB, 0.15117E+01_JPRB, 0.27795E+01_JPRB, 0.36277E+01_JPRB, 0.51151E+01_JPRB ,&
&  0.66087E+01_JPRB, 0.65638E+01_JPRB, 0.59283E+01_JPRB, 0.49711E+01_JPRB, 0.39211E+01_JPRB, 0.30406E+01_JPRB, 0.24856E+01_JPRB ,&
&  0.21381E+01_JPRB, 0.18643E+01_JPRB, 0.16959E+01_JPRB, 0.15878E+01_JPRB, 0.14989E+01_JPRB, 0.14145E+01_JPRB, 0.13123E+01_JPRB ,&
&  0.12059E+01_JPRB, 0.11167E+01_JPRB, 0.10539E+01_JPRB, 0.10105E+01_JPRB, 0.98718E+00_JPRB, 0.98085E+00_JPRB, 0.98300E+00_JPRB ,&
&  0.98296E+00_JPRB, 0.99761E+00_JPRB, 0.10093E+01_JPRB, 0.99418E+00_JPRB, 0.99890E+00_JPRB, 0.10150E+01_JPRB, 0.10264E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.98579E+00_JPRB, 0.97900E+00_JPRB, 0.98437E+00_JPRB, 0.10551E+01_JPRB, 0.11550E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.10852E+01_JPRB, 0.10546E+01_JPRB, 0.10239E+01_JPRB, 0.10178E+01_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.97695E+00_JPRB, 0.95233E+00_JPRB, 0.94366E+00_JPRB, 0.96772E+00_JPRB, 0.99656E+00_JPRB, 0.98981E+00_JPRB, 0.94150E+00_JPRB ,&
&  0.87463E+00_JPRB, 0.82052E+00_JPRB, 0.78263E+00_JPRB, 0.75599E+00_JPRB, 0.73537E+00_JPRB, 0.71743E+00_JPRB, 0.69788E+00_JPRB ,&
&  0.68205E+00_JPRB, 0.67258E+00_JPRB, 0.66574E+00_JPRB, 0.65711E+00_JPRB, 0.64263E+00_JPRB, 0.63152E+00_JPRB, 0.62769E+00_JPRB ,&
&  0.62566E+00_JPRB, 0.62543E+00_JPRB, 0.63104E+00_JPRB, 0.63802E+00_JPRB, 0.64224E+00_JPRB, 0.65891E+00_JPRB, 0.71911E+00_JPRB ,&
&  0.93108E+00_JPRB, 0.14558E+01_JPRB, 0.14799E+01_JPRB, 0.16464E+01_JPRB, 0.19662E+01_JPRB, 0.22555E+01_JPRB, 0.25288E+01_JPRB ,&
&  0.25349E+01_JPRB, 0.23868E+01_JPRB, 0.20924E+01_JPRB, 0.17973E+01_JPRB, 0.15571E+01_JPRB, 0.14089E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.12146E+01_JPRB, 0.11294E+01_JPRB, 0.10542E+01_JPRB, 0.99807E+00_JPRB, 0.95990E+00_JPRB, 0.94366E+00_JPRB, 0.94491E+00_JPRB ,&
&  0.94679E+00_JPRB, 0.94444E+00_JPRB, 0.95478E+00_JPRB, 0.10959E+01_JPRB, 0.17607E+01_JPRB, 0.26943E+01_JPRB, 0.37302E+01_JPRB ,&
&  0.40805E+01_JPRB, 0.42550E+01_JPRB, 0.42058E+01_JPRB, 0.39455E+01_JPRB, 0.34806E+01_JPRB, 0.30214E+01_JPRB, 0.25258E+01_JPRB ,&
&  0.21599E+01_JPRB, 0.19112E+01_JPRB, 0.17296E+01_JPRB, 0.15756E+01_JPRB, 0.14374E+01_JPRB, 0.13268E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.11916E+01_JPRB, 0.11401E+01_JPRB, 0.10909E+01_JPRB, 0.10457E+01_JPRB, 0.10116E+01_JPRB, 0.99227E+00_JPRB, 0.98906E+00_JPRB ,&
&  0.98236E+00_JPRB, 0.96618E+00_JPRB, 0.96328E+00_JPRB, 0.96495E+00_JPRB, 0.97162E+00_JPRB, 0.97202E+00_JPRB, 0.96842E+00_JPRB ,&
&  0.97094E+00_JPRB, 0.98103E+00_JPRB, 0.97672E+00_JPRB, 0.98421E+00_JPRB, 0.10405E+01_JPRB, 0.11153E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.10817E+01_JPRB, 0.10532E+01_JPRB, 0.10319E+01_JPRB, 0.10154E+01_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.10632E+01_JPRB, 0.97453E+00_JPRB, 0.92351E+00_JPRB, 0.91019E+00_JPRB, 0.91665E+00_JPRB, 0.90438E+00_JPRB, 0.87566E+00_JPRB ,&
&  0.84343E+00_JPRB, 0.81389E+00_JPRB, 0.78771E+00_JPRB, 0.75899E+00_JPRB, 0.73080E+00_JPRB, 0.70644E+00_JPRB, 0.68316E+00_JPRB ,&
&  0.66098E+00_JPRB, 0.64770E+00_JPRB, 0.63868E+00_JPRB, 0.63236E+00_JPRB, 0.62771E+00_JPRB, 0.61946E+00_JPRB, 0.61092E+00_JPRB ,&
&  0.60441E+00_JPRB, 0.59502E+00_JPRB, 0.59181E+00_JPRB, 0.59916E+00_JPRB, 0.60829E+00_JPRB, 0.62238E+00_JPRB, 0.66709E+00_JPRB ,&
&  0.82934E+00_JPRB, 0.10652E+01_JPRB, 0.12329E+01_JPRB, 0.13854E+01_JPRB, 0.15423E+01_JPRB, 0.17242E+01_JPRB, 0.17819E+01_JPRB ,&
&  0.16900E+01_JPRB, 0.16982E+01_JPRB, 0.16523E+01_JPRB, 0.15037E+01_JPRB, 0.13360E+01_JPRB, 0.12227E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10044E+01_JPRB, 0.93680E+00_JPRB, 0.88319E+00_JPRB, 0.85074E+00_JPRB, 0.83758E+00_JPRB, 0.83679E+00_JPRB ,&
&  0.84141E+00_JPRB, 0.84913E+00_JPRB, 0.85370E+00_JPRB, 0.87363E+00_JPRB, 0.10240E+01_JPRB, 0.14358E+01_JPRB, 0.20811E+01_JPRB ,&
&  0.24100E+01_JPRB, 0.25280E+01_JPRB, 0.26156E+01_JPRB, 0.26281E+01_JPRB, 0.24659E+01_JPRB, 0.22857E+01_JPRB, 0.20865E+01_JPRB ,&
&  0.18870E+01_JPRB, 0.16764E+01_JPRB, 0.14960E+01_JPRB, 0.13538E+01_JPRB, 0.12430E+01_JPRB, 0.11674E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.10952E+01_JPRB, 0.10846E+01_JPRB, 0.10638E+01_JPRB, 0.10367E+01_JPRB, 0.10162E+01_JPRB, 0.10022E+01_JPRB ,&
&  0.98586E+00_JPRB, 0.97086E+00_JPRB, 0.95823E+00_JPRB, 0.94818E+00_JPRB, 0.94330E+00_JPRB, 0.93798E+00_JPRB, 0.93633E+00_JPRB ,&
&  0.93846E+00_JPRB, 0.94970E+00_JPRB, 0.94011E+00_JPRB, 0.96134E+00_JPRB, 0.10300E+01_JPRB, 0.10537E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.10853E+01_JPRB, 0.10469E+01_JPRB, 0.10321E+01_JPRB, 0.11263E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.96558E+00_JPRB, 0.92521E+00_JPRB, 0.87288E+00_JPRB, 0.84319E+00_JPRB, 0.82368E+00_JPRB, 0.79702E+00_JPRB, 0.77607E+00_JPRB ,&
&  0.75927E+00_JPRB, 0.74281E+00_JPRB, 0.72264E+00_JPRB, 0.69922E+00_JPRB, 0.67674E+00_JPRB, 0.66036E+00_JPRB, 0.64661E+00_JPRB ,&
&  0.63354E+00_JPRB, 0.62093E+00_JPRB, 0.60943E+00_JPRB, 0.60078E+00_JPRB, 0.59165E+00_JPRB, 0.58457E+00_JPRB, 0.57868E+00_JPRB ,&
&  0.57181E+00_JPRB, 0.56378E+00_JPRB, 0.55346E+00_JPRB, 0.55714E+00_JPRB, 0.56930E+00_JPRB, 0.58711E+00_JPRB, 0.61115E+00_JPRB ,&
&  0.70716E+00_JPRB, 0.82032E+00_JPRB, 0.96869E+00_JPRB, 0.11320E+01_JPRB, 0.12533E+01_JPRB, 0.13422E+01_JPRB, 0.13350E+01_JPRB ,&
&  0.12580E+01_JPRB, 0.12656E+01_JPRB, 0.12337E+01_JPRB, 0.11695E+01_JPRB, 0.11232E+01_JPRB, 0.10270E+01_JPRB, 0.93893E+00_JPRB ,&
&  0.87656E+00_JPRB, 0.83605E+00_JPRB, 0.80335E+00_JPRB, 0.77170E+00_JPRB, 0.75312E+00_JPRB, 0.74911E+00_JPRB, 0.75593E+00_JPRB ,&
&  0.76332E+00_JPRB, 0.77309E+00_JPRB, 0.78178E+00_JPRB, 0.79246E+00_JPRB, 0.81556E+00_JPRB, 0.93548E+00_JPRB, 0.12481E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.16411E+01_JPRB, 0.17410E+01_JPRB, 0.17820E+01_JPRB, 0.17341E+01_JPRB, 0.16406E+01_JPRB, 0.15256E+01_JPRB ,&
&  0.14025E+01_JPRB, 0.12705E+01_JPRB, 0.11494E+01_JPRB, 0.10544E+01_JPRB, 0.98064E+00_JPRB, 0.93172E+00_JPRB, 0.90557E+00_JPRB ,&
&  0.90185E+00_JPRB, 0.91473E+00_JPRB, 0.93529E+00_JPRB, 0.95497E+00_JPRB, 0.96529E+00_JPRB, 0.96404E+00_JPRB, 0.95725E+00_JPRB ,&
&  0.94517E+00_JPRB, 0.92939E+00_JPRB, 0.91208E+00_JPRB, 0.89767E+00_JPRB, 0.88591E+00_JPRB, 0.87750E+00_JPRB, 0.87255E+00_JPRB ,&
&  0.87035E+00_JPRB, 0.87287E+00_JPRB, 0.88062E+00_JPRB, 0.91152E+00_JPRB, 0.92640E+00_JPRB, 0.94809E+00_JPRB, 0.96294E+00_JPRB ,&
&  0.97883E+00_JPRB, 0.98203E+00_JPRB, 0.96560E+00_JPRB, 0.93020E+00_JPRB, 0.93067E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.84272E+00_JPRB, 0.83003E+00_JPRB, 0.80235E+00_JPRB, 0.77291E+00_JPRB, 0.74838E+00_JPRB, 0.72538E+00_JPRB, 0.70495E+00_JPRB ,&
&  0.68739E+00_JPRB, 0.67094E+00_JPRB, 0.65334E+00_JPRB, 0.63695E+00_JPRB, 0.62111E+00_JPRB, 0.60815E+00_JPRB, 0.59610E+00_JPRB ,&
&  0.58744E+00_JPRB, 0.58089E+00_JPRB, 0.57426E+00_JPRB, 0.56679E+00_JPRB, 0.55940E+00_JPRB, 0.55230E+00_JPRB, 0.54474E+00_JPRB ,&
&  0.53860E+00_JPRB, 0.53507E+00_JPRB, 0.52606E+00_JPRB, 0.52347E+00_JPRB, 0.53283E+00_JPRB, 0.54843E+00_JPRB, 0.56411E+00_JPRB ,&
&  0.62675E+00_JPRB, 0.64366E+00_JPRB, 0.67706E+00_JPRB, 0.82080E+00_JPRB, 0.94316E+00_JPRB, 0.10757E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.11416E+01_JPRB, 0.11003E+01_JPRB, 0.10459E+01_JPRB, 0.96303E+00_JPRB, 0.89452E+00_JPRB, 0.84414E+00_JPRB ,&
&  0.79980E+00_JPRB, 0.75771E+00_JPRB, 0.72520E+00_JPRB, 0.70061E+00_JPRB, 0.68329E+00_JPRB, 0.67678E+00_JPRB, 0.67989E+00_JPRB ,&
&  0.68511E+00_JPRB, 0.69024E+00_JPRB, 0.69702E+00_JPRB, 0.70947E+00_JPRB, 0.72560E+00_JPRB, 0.75459E+00_JPRB, 0.85948E+00_JPRB ,&
&  0.10266E+01_JPRB, 0.11552E+01_JPRB, 0.12679E+01_JPRB, 0.13311E+01_JPRB, 0.13257E+01_JPRB, 0.12660E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.95146E+00_JPRB, 0.89462E+00_JPRB, 0.85858E+00_JPRB, 0.83784E+00_JPRB, 0.82250E+00_JPRB, 0.80885E+00_JPRB ,&
&  0.79651E+00_JPRB, 0.78921E+00_JPRB, 0.79074E+00_JPRB, 0.80061E+00_JPRB, 0.81606E+00_JPRB, 0.82869E+00_JPRB, 0.83337E+00_JPRB ,&
&  0.83182E+00_JPRB, 0.82655E+00_JPRB, 0.82153E+00_JPRB, 0.81507E+00_JPRB, 0.80677E+00_JPRB, 0.79700E+00_JPRB, 0.79141E+00_JPRB ,&
&  0.79083E+00_JPRB, 0.79524E+00_JPRB, 0.80186E+00_JPRB, 0.81842E+00_JPRB, 0.84825E+00_JPRB, 0.85642E+00_JPRB, 0.86039E+00_JPRB ,&
&  0.86390E+00_JPRB, 0.85998E+00_JPRB, 0.85169E+00_JPRB, 0.76180E+00_JPRB, 0.82716E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.72413E+00_JPRB, 0.72105E+00_JPRB, 0.70327E+00_JPRB, 0.68336E+00_JPRB, 0.66316E+00_JPRB, 0.64365E+00_JPRB, 0.62843E+00_JPRB ,&
&  0.61634E+00_JPRB, 0.60173E+00_JPRB, 0.58911E+00_JPRB, 0.58035E+00_JPRB, 0.57128E+00_JPRB, 0.56289E+00_JPRB, 0.55335E+00_JPRB ,&
&  0.54558E+00_JPRB, 0.54038E+00_JPRB, 0.53642E+00_JPRB, 0.53200E+00_JPRB, 0.52718E+00_JPRB, 0.52167E+00_JPRB, 0.51637E+00_JPRB ,&
&  0.51159E+00_JPRB, 0.50835E+00_JPRB, 0.50274E+00_JPRB, 0.49915E+00_JPRB, 0.50356E+00_JPRB, 0.51491E+00_JPRB, 0.52579E+00_JPRB ,&
&  0.57915E+00_JPRB, 0.53607E+00_JPRB, 0.55255E+00_JPRB, 0.61722E+00_JPRB, 0.69767E+00_JPRB, 0.80276E+00_JPRB, 0.88990E+00_JPRB ,&
&  0.92567E+00_JPRB, 0.95786E+00_JPRB, 0.93832E+00_JPRB, 0.90229E+00_JPRB, 0.84230E+00_JPRB, 0.78454E+00_JPRB, 0.74177E+00_JPRB ,&
&  0.71694E+00_JPRB, 0.69797E+00_JPRB, 0.67633E+00_JPRB, 0.65277E+00_JPRB, 0.63160E+00_JPRB, 0.61723E+00_JPRB, 0.61120E+00_JPRB ,&
&  0.61263E+00_JPRB, 0.61552E+00_JPRB, 0.61623E+00_JPRB, 0.62045E+00_JPRB, 0.63152E+00_JPRB, 0.64879E+00_JPRB, 0.68045E+00_JPRB ,&
&  0.74397E+00_JPRB, 0.82989E+00_JPRB, 0.90465E+00_JPRB, 0.97435E+00_JPRB, 0.10160E+01_JPRB, 0.10237E+01_JPRB, 0.97394E+00_JPRB ,&
&  0.90664E+00_JPRB, 0.84453E+00_JPRB, 0.80290E+00_JPRB, 0.77956E+00_JPRB, 0.76479E+00_JPRB, 0.75354E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.71891E+00_JPRB, 0.69768E+00_JPRB, 0.67965E+00_JPRB, 0.66299E+00_JPRB, 0.65197E+00_JPRB, 0.64948E+00_JPRB, 0.65330E+00_JPRB ,&
&  0.65878E+00_JPRB, 0.66291E+00_JPRB, 0.66654E+00_JPRB, 0.67103E+00_JPRB, 0.67463E+00_JPRB, 0.67800E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.68572E+00_JPRB, 0.69286E+00_JPRB, 0.69970E+00_JPRB, 0.71845E+00_JPRB, 0.73280E+00_JPRB, 0.74047E+00_JPRB, 0.74491E+00_JPRB ,&
&  0.74542E+00_JPRB, 0.74968E+00_JPRB, 0.65597E+00_JPRB, 0.69730E+00_JPRB, 0.71497E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.63281E+00_JPRB, 0.63110E+00_JPRB, 0.62489E+00_JPRB, 0.61062E+00_JPRB, 0.59408E+00_JPRB, 0.57950E+00_JPRB, 0.56982E+00_JPRB ,&
&  0.56317E+00_JPRB, 0.55449E+00_JPRB, 0.54708E+00_JPRB, 0.54187E+00_JPRB, 0.53561E+00_JPRB, 0.52814E+00_JPRB, 0.51799E+00_JPRB ,&
&  0.50924E+00_JPRB, 0.50234E+00_JPRB, 0.49744E+00_JPRB, 0.49455E+00_JPRB, 0.49278E+00_JPRB, 0.49094E+00_JPRB, 0.48811E+00_JPRB ,&
&  0.48544E+00_JPRB, 0.48416E+00_JPRB, 0.48279E+00_JPRB, 0.48147E+00_JPRB, 0.48253E+00_JPRB, 0.48978E+00_JPRB, 0.49907E+00_JPRB ,&
&  0.50106E+00_JPRB, 0.48293E+00_JPRB, 0.49677E+00_JPRB, 0.52528E+00_JPRB, 0.56647E+00_JPRB, 0.63759E+00_JPRB, 0.71258E+00_JPRB ,&
&  0.75691E+00_JPRB, 0.78098E+00_JPRB, 0.76203E+00_JPRB, 0.73664E+00_JPRB, 0.71052E+00_JPRB, 0.67935E+00_JPRB, 0.65360E+00_JPRB ,&
&  0.63320E+00_JPRB, 0.61950E+00_JPRB, 0.60844E+00_JPRB, 0.59320E+00_JPRB, 0.57538E+00_JPRB, 0.56096E+00_JPRB, 0.55236E+00_JPRB ,&
&  0.54940E+00_JPRB, 0.55145E+00_JPRB, 0.55459E+00_JPRB, 0.55739E+00_JPRB, 0.56254E+00_JPRB, 0.57292E+00_JPRB, 0.58789E+00_JPRB ,&
&  0.60433E+00_JPRB, 0.63284E+00_JPRB, 0.67258E+00_JPRB, 0.71154E+00_JPRB, 0.74806E+00_JPRB, 0.77293E+00_JPRB, 0.78550E+00_JPRB ,&
&  0.78197E+00_JPRB, 0.76225E+00_JPRB, 0.73656E+00_JPRB, 0.70986E+00_JPRB, 0.67942E+00_JPRB, 0.64372E+00_JPRB, 0.60647E+00_JPRB ,&
&  0.57537E+00_JPRB, 0.55181E+00_JPRB, 0.53631E+00_JPRB, 0.52901E+00_JPRB, 0.52637E+00_JPRB, 0.52859E+00_JPRB, 0.53398E+00_JPRB ,&
&  0.54111E+00_JPRB, 0.54890E+00_JPRB, 0.55494E+00_JPRB, 0.56158E+00_JPRB, 0.56766E+00_JPRB, 0.57434E+00_JPRB, 0.58132E+00_JPRB ,&
&  0.58912E+00_JPRB, 0.59647E+00_JPRB, 0.60357E+00_JPRB, 0.61675E+00_JPRB, 0.62902E+00_JPRB, 0.64011E+00_JPRB, 0.64562E+00_JPRB ,&
&  0.64292E+00_JPRB, 0.64047E+00_JPRB, 0.63759E+00_JPRB, 0.63306E+00_JPRB, 0.63472E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.56789E+00_JPRB, 0.56566E+00_JPRB, 0.56307E+00_JPRB, 0.55907E+00_JPRB, 0.55227E+00_JPRB, 0.54283E+00_JPRB, 0.53518E+00_JPRB ,&
&  0.52999E+00_JPRB, 0.52438E+00_JPRB, 0.51932E+00_JPRB, 0.51326E+00_JPRB, 0.50605E+00_JPRB, 0.49734E+00_JPRB, 0.48633E+00_JPRB ,&
&  0.47625E+00_JPRB, 0.46718E+00_JPRB, 0.46115E+00_JPRB, 0.45823E+00_JPRB, 0.45794E+00_JPRB, 0.45773E+00_JPRB, 0.45555E+00_JPRB ,&
&  0.45412E+00_JPRB, 0.45395E+00_JPRB, 0.45409E+00_JPRB, 0.45370E+00_JPRB, 0.45400E+00_JPRB, 0.45920E+00_JPRB, 0.46481E+00_JPRB ,&
&  0.48377E+00_JPRB, 0.46701E+00_JPRB, 0.48460E+00_JPRB, 0.49128E+00_JPRB, 0.50638E+00_JPRB, 0.54461E+00_JPRB, 0.59771E+00_JPRB ,&
&  0.64154E+00_JPRB, 0.64458E+00_JPRB, 0.62458E+00_JPRB, 0.60311E+00_JPRB, 0.59384E+00_JPRB, 0.58186E+00_JPRB, 0.56772E+00_JPRB ,&
&  0.55433E+00_JPRB, 0.54466E+00_JPRB, 0.53673E+00_JPRB, 0.52923E+00_JPRB, 0.51954E+00_JPRB, 0.50758E+00_JPRB, 0.49662E+00_JPRB ,&
&  0.48939E+00_JPRB, 0.48719E+00_JPRB, 0.48914E+00_JPRB, 0.49313E+00_JPRB, 0.49752E+00_JPRB, 0.50491E+00_JPRB, 0.51473E+00_JPRB ,&
&  0.52487E+00_JPRB, 0.53000E+00_JPRB, 0.53506E+00_JPRB, 0.54648E+00_JPRB, 0.56055E+00_JPRB, 0.57147E+00_JPRB, 0.57752E+00_JPRB ,&
&  0.57907E+00_JPRB, 0.57329E+00_JPRB, 0.56263E+00_JPRB, 0.54828E+00_JPRB, 0.53207E+00_JPRB, 0.51563E+00_JPRB, 0.50123E+00_JPRB ,&
&  0.48971E+00_JPRB, 0.48005E+00_JPRB, 0.47185E+00_JPRB, 0.46611E+00_JPRB, 0.46342E+00_JPRB, 0.46302E+00_JPRB, 0.46468E+00_JPRB ,&
&  0.46932E+00_JPRB, 0.47540E+00_JPRB, 0.48081E+00_JPRB, 0.48598E+00_JPRB, 0.49060E+00_JPRB, 0.49448E+00_JPRB, 0.49956E+00_JPRB ,&
&  0.50641E+00_JPRB, 0.51161E+00_JPRB, 0.51580E+00_JPRB, 0.52211E+00_JPRB, 0.53123E+00_JPRB, 0.54208E+00_JPRB, 0.55182E+00_JPRB ,&
&  0.55777E+00_JPRB, 0.56000E+00_JPRB, 0.56321E+00_JPRB, 0.56484E+00_JPRB, 0.56839E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.52225E+00_JPRB, 0.52123E+00_JPRB, 0.51905E+00_JPRB, 0.51750E+00_JPRB, 0.51476E+00_JPRB, 0.51027E+00_JPRB, 0.50492E+00_JPRB ,&
&  0.50037E+00_JPRB, 0.49600E+00_JPRB, 0.49113E+00_JPRB, 0.48476E+00_JPRB, 0.47627E+00_JPRB, 0.46788E+00_JPRB, 0.45855E+00_JPRB ,&
&  0.44979E+00_JPRB, 0.44224E+00_JPRB, 0.43687E+00_JPRB, 0.43359E+00_JPRB, 0.43096E+00_JPRB, 0.42852E+00_JPRB, 0.42570E+00_JPRB ,&
&  0.42391E+00_JPRB, 0.42310E+00_JPRB, 0.42320E+00_JPRB, 0.42389E+00_JPRB, 0.42471E+00_JPRB, 0.42826E+00_JPRB, 0.43087E+00_JPRB ,&
&  0.43806E+00_JPRB, 0.46140E+00_JPRB, 0.47419E+00_JPRB, 0.47823E+00_JPRB, 0.48415E+00_JPRB, 0.49740E+00_JPRB, 0.51726E+00_JPRB ,&
&  0.53597E+00_JPRB, 0.54405E+00_JPRB, 0.53426E+00_JPRB, 0.51837E+00_JPRB, 0.50929E+00_JPRB, 0.50567E+00_JPRB, 0.49708E+00_JPRB ,&
&  0.48823E+00_JPRB, 0.48129E+00_JPRB, 0.47584E+00_JPRB, 0.47162E+00_JPRB, 0.46759E+00_JPRB, 0.46198E+00_JPRB, 0.45562E+00_JPRB ,&
&  0.44798E+00_JPRB, 0.44196E+00_JPRB, 0.43763E+00_JPRB, 0.43612E+00_JPRB, 0.43646E+00_JPRB, 0.43854E+00_JPRB, 0.44237E+00_JPRB ,&
&  0.44857E+00_JPRB, 0.45445E+00_JPRB, 0.45682E+00_JPRB, 0.45731E+00_JPRB, 0.45821E+00_JPRB, 0.46058E+00_JPRB, 0.46234E+00_JPRB ,&
&  0.46193E+00_JPRB, 0.45953E+00_JPRB, 0.45534E+00_JPRB, 0.44966E+00_JPRB, 0.44307E+00_JPRB, 0.43729E+00_JPRB, 0.43247E+00_JPRB ,&
&  0.42816E+00_JPRB, 0.42394E+00_JPRB, 0.41993E+00_JPRB, 0.41763E+00_JPRB, 0.41633E+00_JPRB, 0.41588E+00_JPRB, 0.41704E+00_JPRB ,&
&  0.41991E+00_JPRB, 0.42357E+00_JPRB, 0.42782E+00_JPRB, 0.43194E+00_JPRB, 0.43593E+00_JPRB, 0.43916E+00_JPRB, 0.44294E+00_JPRB ,&
&  0.44886E+00_JPRB, 0.45546E+00_JPRB, 0.46184E+00_JPRB, 0.46764E+00_JPRB, 0.47470E+00_JPRB, 0.48409E+00_JPRB, 0.49445E+00_JPRB ,&
&  0.50158E+00_JPRB, 0.50585E+00_JPRB, 0.51051E+00_JPRB, 0.51585E+00_JPRB, 0.52062E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.47146E+00_JPRB, 0.47366E+00_JPRB, 0.47478E+00_JPRB, 0.47621E+00_JPRB, 0.47655E+00_JPRB, 0.47509E+00_JPRB, 0.47126E+00_JPRB ,&
&  0.46660E+00_JPRB, 0.46129E+00_JPRB, 0.45581E+00_JPRB, 0.44891E+00_JPRB, 0.44167E+00_JPRB, 0.43557E+00_JPRB, 0.42900E+00_JPRB ,&
&  0.42234E+00_JPRB, 0.41620E+00_JPRB, 0.41167E+00_JPRB, 0.40806E+00_JPRB, 0.40447E+00_JPRB, 0.40078E+00_JPRB, 0.39625E+00_JPRB ,&
&  0.39230E+00_JPRB, 0.38882E+00_JPRB, 0.38896E+00_JPRB, 0.39181E+00_JPRB, 0.39429E+00_JPRB, 0.39616E+00_JPRB, 0.39800E+00_JPRB ,&
&  0.39957E+00_JPRB, 0.42327E+00_JPRB, 0.41979E+00_JPRB, 0.43643E+00_JPRB, 0.44252E+00_JPRB, 0.44327E+00_JPRB, 0.44482E+00_JPRB ,&
&  0.45002E+00_JPRB, 0.45681E+00_JPRB, 0.46172E+00_JPRB, 0.45575E+00_JPRB, 0.44643E+00_JPRB, 0.43947E+00_JPRB, 0.43257E+00_JPRB ,&
&  0.42358E+00_JPRB, 0.41939E+00_JPRB, 0.41861E+00_JPRB, 0.41844E+00_JPRB, 0.41834E+00_JPRB, 0.41770E+00_JPRB, 0.41601E+00_JPRB ,&
&  0.41314E+00_JPRB, 0.40907E+00_JPRB, 0.40439E+00_JPRB, 0.39992E+00_JPRB, 0.39696E+00_JPRB, 0.39421E+00_JPRB, 0.39226E+00_JPRB ,&
&  0.39233E+00_JPRB, 0.39449E+00_JPRB, 0.39652E+00_JPRB, 0.39704E+00_JPRB, 0.39605E+00_JPRB, 0.39499E+00_JPRB, 0.39441E+00_JPRB ,&
&  0.39343E+00_JPRB, 0.39194E+00_JPRB, 0.38992E+00_JPRB, 0.38725E+00_JPRB, 0.38435E+00_JPRB, 0.38229E+00_JPRB, 0.38114E+00_JPRB ,&
&  0.37981E+00_JPRB, 0.37747E+00_JPRB, 0.37441E+00_JPRB, 0.37371E+00_JPRB, 0.37376E+00_JPRB, 0.37419E+00_JPRB, 0.37482E+00_JPRB ,&
&  0.37621E+00_JPRB, 0.37888E+00_JPRB, 0.38209E+00_JPRB, 0.38544E+00_JPRB, 0.38893E+00_JPRB, 0.39366E+00_JPRB, 0.39891E+00_JPRB ,&
&  0.40453E+00_JPRB, 0.41050E+00_JPRB, 0.41633E+00_JPRB, 0.42181E+00_JPRB, 0.42666E+00_JPRB, 0.43258E+00_JPRB, 0.44039E+00_JPRB ,&
&  0.44717E+00_JPRB, 0.45240E+00_JPRB, 0.45656E+00_JPRB, 0.46165E+00_JPRB, 0.46723E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.42937E+00_JPRB, 0.43493E+00_JPRB, 0.43833E+00_JPRB, 0.44028E+00_JPRB, 0.44112E+00_JPRB, 0.43970E+00_JPRB, 0.43658E+00_JPRB ,&
&  0.43147E+00_JPRB, 0.42608E+00_JPRB, 0.42038E+00_JPRB, 0.41431E+00_JPRB, 0.40879E+00_JPRB, 0.40398E+00_JPRB, 0.39961E+00_JPRB ,&
&  0.39516E+00_JPRB, 0.39099E+00_JPRB, 0.38755E+00_JPRB, 0.38414E+00_JPRB, 0.38012E+00_JPRB, 0.37506E+00_JPRB, 0.36887E+00_JPRB ,&
&  0.36487E+00_JPRB, 0.36176E+00_JPRB, 0.36239E+00_JPRB, 0.36562E+00_JPRB, 0.36831E+00_JPRB, 0.36987E+00_JPRB, 0.37154E+00_JPRB ,&
&  0.37406E+00_JPRB, 0.38090E+00_JPRB, 0.38666E+00_JPRB, 0.39367E+00_JPRB, 0.39832E+00_JPRB, 0.39837E+00_JPRB, 0.39555E+00_JPRB ,&
&  0.39408E+00_JPRB, 0.39566E+00_JPRB, 0.39852E+00_JPRB, 0.40031E+00_JPRB, 0.39869E+00_JPRB, 0.39460E+00_JPRB, 0.38817E+00_JPRB ,&
&  0.37819E+00_JPRB, 0.37089E+00_JPRB, 0.36931E+00_JPRB, 0.37087E+00_JPRB, 0.37330E+00_JPRB, 0.37512E+00_JPRB, 0.37615E+00_JPRB ,&
&  0.37637E+00_JPRB, 0.37535E+00_JPRB, 0.37358E+00_JPRB, 0.37134E+00_JPRB, 0.36859E+00_JPRB, 0.36581E+00_JPRB, 0.36405E+00_JPRB ,&
&  0.36248E+00_JPRB, 0.36150E+00_JPRB, 0.36128E+00_JPRB, 0.36116E+00_JPRB, 0.35985E+00_JPRB, 0.35754E+00_JPRB, 0.35588E+00_JPRB ,&
&  0.35386E+00_JPRB, 0.35246E+00_JPRB, 0.35063E+00_JPRB, 0.34870E+00_JPRB, 0.34649E+00_JPRB, 0.34447E+00_JPRB, 0.34349E+00_JPRB ,&
&  0.34329E+00_JPRB, 0.34190E+00_JPRB, 0.34032E+00_JPRB, 0.34036E+00_JPRB, 0.34122E+00_JPRB, 0.34236E+00_JPRB, 0.34315E+00_JPRB ,&
&  0.34389E+00_JPRB, 0.34489E+00_JPRB, 0.34735E+00_JPRB, 0.35037E+00_JPRB, 0.35392E+00_JPRB, 0.35822E+00_JPRB, 0.36305E+00_JPRB ,&
&  0.36851E+00_JPRB, 0.37460E+00_JPRB, 0.38126E+00_JPRB, 0.38747E+00_JPRB, 0.39281E+00_JPRB, 0.39675E+00_JPRB, 0.40191E+00_JPRB ,&
&  0.40802E+00_JPRB, 0.41234E+00_JPRB, 0.41578E+00_JPRB, 0.41957E+00_JPRB, 0.42380E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.38849E+00_JPRB, 0.39420E+00_JPRB, 0.39858E+00_JPRB, 0.40116E+00_JPRB, 0.40154E+00_JPRB, 0.40028E+00_JPRB, 0.39719E+00_JPRB ,&
&  0.39248E+00_JPRB, 0.38759E+00_JPRB, 0.38282E+00_JPRB, 0.37897E+00_JPRB, 0.37638E+00_JPRB, 0.37410E+00_JPRB, 0.37196E+00_JPRB ,&
&  0.36916E+00_JPRB, 0.36600E+00_JPRB, 0.36243E+00_JPRB, 0.35928E+00_JPRB, 0.35540E+00_JPRB, 0.35018E+00_JPRB, 0.34611E+00_JPRB ,&
&  0.34312E+00_JPRB, 0.34049E+00_JPRB, 0.33966E+00_JPRB, 0.34110E+00_JPRB, 0.34376E+00_JPRB, 0.34627E+00_JPRB, 0.34928E+00_JPRB ,&
&  0.35171E+00_JPRB, 0.35453E+00_JPRB, 0.35729E+00_JPRB, 0.35796E+00_JPRB, 0.35800E+00_JPRB, 0.35784E+00_JPRB, 0.35729E+00_JPRB ,&
&  0.35672E+00_JPRB, 0.35618E+00_JPRB, 0.35526E+00_JPRB, 0.35530E+00_JPRB, 0.35514E+00_JPRB, 0.35366E+00_JPRB, 0.35096E+00_JPRB ,&
&  0.34647E+00_JPRB, 0.34042E+00_JPRB, 0.33587E+00_JPRB, 0.33390E+00_JPRB, 0.33353E+00_JPRB, 0.33404E+00_JPRB, 0.33507E+00_JPRB ,&
&  0.33628E+00_JPRB, 0.33805E+00_JPRB, 0.33977E+00_JPRB, 0.34063E+00_JPRB, 0.34039E+00_JPRB, 0.34003E+00_JPRB, 0.33843E+00_JPRB ,&
&  0.33691E+00_JPRB, 0.33585E+00_JPRB, 0.33524E+00_JPRB, 0.33458E+00_JPRB, 0.33210E+00_JPRB, 0.32888E+00_JPRB, 0.32770E+00_JPRB ,&
&  0.32552E+00_JPRB, 0.32407E+00_JPRB, 0.32211E+00_JPRB, 0.32076E+00_JPRB, 0.31932E+00_JPRB, 0.31751E+00_JPRB, 0.31574E+00_JPRB ,&
&  0.31448E+00_JPRB, 0.31379E+00_JPRB, 0.31345E+00_JPRB, 0.31272E+00_JPRB, 0.31319E+00_JPRB, 0.31404E+00_JPRB, 0.31566E+00_JPRB ,&
&  0.31697E+00_JPRB, 0.31813E+00_JPRB, 0.31992E+00_JPRB, 0.32204E+00_JPRB, 0.32518E+00_JPRB, 0.32883E+00_JPRB, 0.33302E+00_JPRB ,&
&  0.33672E+00_JPRB, 0.34152E+00_JPRB, 0.34684E+00_JPRB, 0.35189E+00_JPRB, 0.35592E+00_JPRB, 0.35894E+00_JPRB, 0.36321E+00_JPRB ,&
&  0.36973E+00_JPRB, 0.37488E+00_JPRB, 0.37754E+00_JPRB, 0.38057E+00_JPRB, 0.38364E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.35635E+00_JPRB, 0.35853E+00_JPRB, 0.36067E+00_JPRB, 0.36290E+00_JPRB, 0.36449E+00_JPRB, 0.36509E+00_JPRB, 0.36367E+00_JPRB ,&
&  0.36074E+00_JPRB, 0.35724E+00_JPRB, 0.35367E+00_JPRB, 0.35063E+00_JPRB, 0.34873E+00_JPRB, 0.34740E+00_JPRB, 0.34632E+00_JPRB ,&
&  0.34528E+00_JPRB, 0.34421E+00_JPRB, 0.34172E+00_JPRB, 0.33829E+00_JPRB, 0.33378E+00_JPRB, 0.32977E+00_JPRB, 0.32817E+00_JPRB ,&
&  0.32603E+00_JPRB, 0.32456E+00_JPRB, 0.32346E+00_JPRB, 0.32234E+00_JPRB, 0.32179E+00_JPRB, 0.32268E+00_JPRB, 0.32476E+00_JPRB ,&
&  0.32698E+00_JPRB, 0.32911E+00_JPRB, 0.33099E+00_JPRB, 0.33173E+00_JPRB, 0.33100E+00_JPRB, 0.33014E+00_JPRB, 0.32903E+00_JPRB ,&
&  0.32756E+00_JPRB, 0.32591E+00_JPRB, 0.32437E+00_JPRB, 0.32241E+00_JPRB, 0.31959E+00_JPRB, 0.31608E+00_JPRB, 0.31366E+00_JPRB ,&
&  0.31192E+00_JPRB, 0.31025E+00_JPRB, 0.30850E+00_JPRB, 0.30764E+00_JPRB, 0.30696E+00_JPRB, 0.30670E+00_JPRB, 0.30685E+00_JPRB ,&
&  0.30744E+00_JPRB, 0.30889E+00_JPRB, 0.31099E+00_JPRB, 0.31335E+00_JPRB, 0.31428E+00_JPRB, 0.31461E+00_JPRB, 0.31474E+00_JPRB ,&
&  0.31373E+00_JPRB, 0.31197E+00_JPRB, 0.30973E+00_JPRB, 0.30720E+00_JPRB, 0.30531E+00_JPRB, 0.30384E+00_JPRB, 0.30219E+00_JPRB ,&
&  0.29968E+00_JPRB, 0.29812E+00_JPRB, 0.29736E+00_JPRB, 0.29718E+00_JPRB, 0.29732E+00_JPRB, 0.29726E+00_JPRB, 0.29633E+00_JPRB ,&
&  0.29521E+00_JPRB, 0.29432E+00_JPRB, 0.29332E+00_JPRB, 0.29213E+00_JPRB, 0.29012E+00_JPRB, 0.29076E+00_JPRB, 0.29216E+00_JPRB ,&
&  0.29267E+00_JPRB, 0.29269E+00_JPRB, 0.29289E+00_JPRB, 0.29446E+00_JPRB, 0.29632E+00_JPRB, 0.29868E+00_JPRB, 0.30168E+00_JPRB ,&
&  0.30434E+00_JPRB, 0.30799E+00_JPRB, 0.31306E+00_JPRB, 0.31784E+00_JPRB, 0.32127E+00_JPRB, 0.32307E+00_JPRB, 0.32677E+00_JPRB ,&
&  0.33415E+00_JPRB, 0.34155E+00_JPRB, 0.34601E+00_JPRB, 0.35027E+00_JPRB, 0.35431E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.32781E+00_JPRB, 0.32905E+00_JPRB, 0.33006E+00_JPRB, 0.33125E+00_JPRB, 0.33284E+00_JPRB, 0.33384E+00_JPRB, 0.33462E+00_JPRB ,&
&  0.33391E+00_JPRB, 0.33187E+00_JPRB, 0.32944E+00_JPRB, 0.32692E+00_JPRB, 0.32570E+00_JPRB, 0.32533E+00_JPRB, 0.32521E+00_JPRB ,&
&  0.32449E+00_JPRB, 0.32407E+00_JPRB, 0.32190E+00_JPRB, 0.31837E+00_JPRB, 0.31408E+00_JPRB, 0.31098E+00_JPRB, 0.30995E+00_JPRB ,&
&  0.30915E+00_JPRB, 0.30799E+00_JPRB, 0.30721E+00_JPRB, 0.30601E+00_JPRB, 0.30282E+00_JPRB, 0.30130E+00_JPRB, 0.30173E+00_JPRB ,&
&  0.30267E+00_JPRB, 0.30439E+00_JPRB, 0.30448E+00_JPRB, 0.30242E+00_JPRB, 0.30464E+00_JPRB, 0.30459E+00_JPRB, 0.30271E+00_JPRB ,&
&  0.30023E+00_JPRB, 0.29767E+00_JPRB, 0.29532E+00_JPRB, 0.29284E+00_JPRB, 0.29017E+00_JPRB, 0.28815E+00_JPRB, 0.28688E+00_JPRB ,&
&  0.28666E+00_JPRB, 0.28717E+00_JPRB, 0.28800E+00_JPRB, 0.28916E+00_JPRB, 0.28990E+00_JPRB, 0.29034E+00_JPRB, 0.29051E+00_JPRB ,&
&  0.29080E+00_JPRB, 0.29119E+00_JPRB, 0.29179E+00_JPRB, 0.29216E+00_JPRB, 0.29254E+00_JPRB, 0.29273E+00_JPRB, 0.29231E+00_JPRB ,&
&  0.29071E+00_JPRB, 0.28826E+00_JPRB, 0.28519E+00_JPRB, 0.28380E+00_JPRB, 0.28348E+00_JPRB, 0.28122E+00_JPRB, 0.27814E+00_JPRB ,&
&  0.27554E+00_JPRB, 0.27534E+00_JPRB, 0.27712E+00_JPRB, 0.27869E+00_JPRB, 0.27995E+00_JPRB, 0.28045E+00_JPRB, 0.28021E+00_JPRB ,&
&  0.27913E+00_JPRB, 0.27619E+00_JPRB, 0.27298E+00_JPRB, 0.27025E+00_JPRB, 0.26844E+00_JPRB, 0.26826E+00_JPRB, 0.26942E+00_JPRB ,&
&  0.26974E+00_JPRB, 0.27008E+00_JPRB, 0.27142E+00_JPRB, 0.27273E+00_JPRB, 0.27383E+00_JPRB, 0.27496E+00_JPRB, 0.27668E+00_JPRB ,&
&  0.27895E+00_JPRB, 0.28291E+00_JPRB, 0.28790E+00_JPRB, 0.29370E+00_JPRB, 0.29765E+00_JPRB, 0.29702E+00_JPRB, 0.29856E+00_JPRB ,&
&  0.30470E+00_JPRB, 0.31156E+00_JPRB, 0.31686E+00_JPRB, 0.32094E+00_JPRB, 0.32478E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.30521E+00_JPRB, 0.30663E+00_JPRB, 0.30762E+00_JPRB, 0.30832E+00_JPRB, 0.30969E+00_JPRB, 0.31080E+00_JPRB, 0.31226E+00_JPRB ,&
&  0.31329E+00_JPRB, 0.31188E+00_JPRB, 0.31005E+00_JPRB, 0.30844E+00_JPRB, 0.30763E+00_JPRB, 0.30655E+00_JPRB, 0.30555E+00_JPRB ,&
&  0.30586E+00_JPRB, 0.30490E+00_JPRB, 0.30181E+00_JPRB, 0.29920E+00_JPRB, 0.29522E+00_JPRB, 0.29269E+00_JPRB, 0.29235E+00_JPRB ,&
&  0.29403E+00_JPRB, 0.29341E+00_JPRB, 0.29199E+00_JPRB, 0.29020E+00_JPRB, 0.28811E+00_JPRB, 0.28695E+00_JPRB, 0.28573E+00_JPRB ,&
&  0.28497E+00_JPRB, 0.28340E+00_JPRB, 0.27889E+00_JPRB, 0.27926E+00_JPRB, 0.27908E+00_JPRB, 0.27947E+00_JPRB, 0.27916E+00_JPRB ,&
&  0.27866E+00_JPRB, 0.27767E+00_JPRB, 0.27578E+00_JPRB, 0.27362E+00_JPRB, 0.27184E+00_JPRB, 0.27095E+00_JPRB, 0.27108E+00_JPRB ,&
&  0.27188E+00_JPRB, 0.27355E+00_JPRB, 0.27587E+00_JPRB, 0.27829E+00_JPRB, 0.27934E+00_JPRB, 0.28018E+00_JPRB, 0.28152E+00_JPRB ,&
&  0.28220E+00_JPRB, 0.28238E+00_JPRB, 0.28230E+00_JPRB, 0.28191E+00_JPRB, 0.28106E+00_JPRB, 0.27958E+00_JPRB, 0.27833E+00_JPRB ,&
&  0.27578E+00_JPRB, 0.27257E+00_JPRB, 0.27026E+00_JPRB, 0.26754E+00_JPRB, 0.26021E+00_JPRB, 0.24985E+00_JPRB, 0.24412E+00_JPRB ,&
&  0.24496E+00_JPRB, 0.25004E+00_JPRB, 0.25472E+00_JPRB, 0.25880E+00_JPRB, 0.26590E+00_JPRB, 0.26593E+00_JPRB, 0.26097E+00_JPRB ,&
&  0.24986E+00_JPRB, 0.24645E+00_JPRB, 0.24335E+00_JPRB, 0.23803E+00_JPRB, 0.23488E+00_JPRB, 0.23468E+00_JPRB, 0.23490E+00_JPRB ,&
&  0.23726E+00_JPRB, 0.23997E+00_JPRB, 0.24331E+00_JPRB, 0.24536E+00_JPRB, 0.24576E+00_JPRB, 0.24491E+00_JPRB, 0.24350E+00_JPRB ,&
&  0.24403E+00_JPRB, 0.24922E+00_JPRB, 0.25881E+00_JPRB, 0.26699E+00_JPRB, 0.27245E+00_JPRB, 0.27483E+00_JPRB, 0.27729E+00_JPRB ,&
&  0.28234E+00_JPRB, 0.28877E+00_JPRB, 0.29485E+00_JPRB, 0.29909E+00_JPRB, 0.30251E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.28260E+00_JPRB, 0.28386E+00_JPRB, 0.28498E+00_JPRB, 0.28604E+00_JPRB, 0.28716E+00_JPRB, 0.28839E+00_JPRB, 0.28948E+00_JPRB ,&
&  0.29086E+00_JPRB, 0.29173E+00_JPRB, 0.29183E+00_JPRB, 0.29057E+00_JPRB, 0.28959E+00_JPRB, 0.28891E+00_JPRB, 0.28877E+00_JPRB ,&
&  0.28619E+00_JPRB, 0.28288E+00_JPRB, 0.28131E+00_JPRB, 0.28081E+00_JPRB, 0.27992E+00_JPRB, 0.27962E+00_JPRB, 0.28059E+00_JPRB ,&
&  0.28151E+00_JPRB, 0.28065E+00_JPRB, 0.27990E+00_JPRB, 0.27802E+00_JPRB, 0.27655E+00_JPRB, 0.27548E+00_JPRB, 0.27412E+00_JPRB ,&
&  0.26926E+00_JPRB, 0.25878E+00_JPRB, 0.25297E+00_JPRB, 0.25808E+00_JPRB, 0.26396E+00_JPRB, 0.26300E+00_JPRB, 0.26336E+00_JPRB ,&
&  0.26321E+00_JPRB, 0.26196E+00_JPRB, 0.26052E+00_JPRB, 0.25926E+00_JPRB, 0.25902E+00_JPRB, 0.25981E+00_JPRB, 0.26141E+00_JPRB ,&
&  0.26338E+00_JPRB, 0.26688E+00_JPRB, 0.27023E+00_JPRB, 0.27145E+00_JPRB, 0.27278E+00_JPRB, 0.27357E+00_JPRB, 0.27377E+00_JPRB ,&
&  0.27279E+00_JPRB, 0.27104E+00_JPRB, 0.26941E+00_JPRB, 0.26776E+00_JPRB, 0.26607E+00_JPRB, 0.26390E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25301E+00_JPRB, 0.24703E+00_JPRB, 0.23988E+00_JPRB, 0.23271E+00_JPRB, 0.22670E+00_JPRB, 0.22234E+00_JPRB, 0.21835E+00_JPRB ,&
&  0.21820E+00_JPRB, 0.22352E+00_JPRB, 0.23323E+00_JPRB, 0.24371E+00_JPRB, 0.24525E+00_JPRB, 0.23695E+00_JPRB, 0.23132E+00_JPRB ,&
&  0.22503E+00_JPRB, 0.21959E+00_JPRB, 0.21623E+00_JPRB, 0.21437E+00_JPRB, 0.21289E+00_JPRB, 0.21265E+00_JPRB, 0.21332E+00_JPRB ,&
&  0.21512E+00_JPRB, 0.21701E+00_JPRB, 0.21872E+00_JPRB, 0.21998E+00_JPRB, 0.22052E+00_JPRB, 0.22042E+00_JPRB, 0.22000E+00_JPRB ,&
&  0.22096E+00_JPRB, 0.22420E+00_JPRB, 0.22931E+00_JPRB, 0.23509E+00_JPRB, 0.24012E+00_JPRB, 0.24505E+00_JPRB, 0.25145E+00_JPRB ,&
&  0.25943E+00_JPRB, 0.26474E+00_JPRB, 0.27046E+00_JPRB, 0.27673E+00_JPRB, 0.28045E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.26371E+00_JPRB, 0.26651E+00_JPRB, 0.26727E+00_JPRB, 0.26871E+00_JPRB, 0.26892E+00_JPRB, 0.26992E+00_JPRB, 0.27039E+00_JPRB ,&
&  0.27249E+00_JPRB, 0.27446E+00_JPRB, 0.27484E+00_JPRB, 0.27283E+00_JPRB, 0.27112E+00_JPRB, 0.26698E+00_JPRB, 0.26508E+00_JPRB ,&
&  0.26411E+00_JPRB, 0.26489E+00_JPRB, 0.26496E+00_JPRB, 0.26495E+00_JPRB, 0.26481E+00_JPRB, 0.26473E+00_JPRB, 0.26542E+00_JPRB ,&
&  0.26309E+00_JPRB, 0.26129E+00_JPRB, 0.26329E+00_JPRB, 0.26193E+00_JPRB, 0.26083E+00_JPRB, 0.25901E+00_JPRB, 0.25799E+00_JPRB ,&
&  0.25311E+00_JPRB, 0.24661E+00_JPRB, 0.24072E+00_JPRB, 0.24525E+00_JPRB, 0.25252E+00_JPRB, 0.24902E+00_JPRB, 0.24862E+00_JPRB ,&
&  0.24859E+00_JPRB, 0.24787E+00_JPRB, 0.24805E+00_JPRB, 0.24812E+00_JPRB, 0.24966E+00_JPRB, 0.25117E+00_JPRB, 0.25407E+00_JPRB ,&
&  0.25785E+00_JPRB, 0.26293E+00_JPRB, 0.26299E+00_JPRB, 0.25540E+00_JPRB, 0.25266E+00_JPRB, 0.25198E+00_JPRB, 0.24816E+00_JPRB ,&
&  0.24336E+00_JPRB, 0.24231E+00_JPRB, 0.24081E+00_JPRB, 0.23658E+00_JPRB, 0.23482E+00_JPRB, 0.23411E+00_JPRB, 0.23050E+00_JPRB ,&
&  0.22608E+00_JPRB, 0.22281E+00_JPRB, 0.21862E+00_JPRB, 0.21447E+00_JPRB, 0.21164E+00_JPRB, 0.21039E+00_JPRB, 0.21069E+00_JPRB ,&
&  0.21322E+00_JPRB, 0.21536E+00_JPRB, 0.22209E+00_JPRB, 0.22588E+00_JPRB, 0.22146E+00_JPRB, 0.21914E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.20888E+00_JPRB, 0.20592E+00_JPRB, 0.20341E+00_JPRB, 0.20149E+00_JPRB, 0.20123E+00_JPRB, 0.20199E+00_JPRB, 0.20239E+00_JPRB ,&
&  0.20321E+00_JPRB, 0.20469E+00_JPRB, 0.20542E+00_JPRB, 0.20583E+00_JPRB, 0.20596E+00_JPRB, 0.20639E+00_JPRB, 0.20729E+00_JPRB ,&
&  0.20832E+00_JPRB, 0.21080E+00_JPRB, 0.21401E+00_JPRB, 0.21733E+00_JPRB, 0.21978E+00_JPRB, 0.22181E+00_JPRB, 0.22624E+00_JPRB ,&
&  0.23095E+00_JPRB, 0.24198E+00_JPRB, 0.25710E+00_JPRB, 0.26611E+00_JPRB, 0.26130E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.25999E+00_JPRB, 0.26180E+00_JPRB, 0.26134E+00_JPRB, 0.26142E+00_JPRB, 0.26073E+00_JPRB, 0.26115E+00_JPRB, 0.25957E+00_JPRB ,&
&  0.25875E+00_JPRB, 0.25669E+00_JPRB, 0.25492E+00_JPRB, 0.25097E+00_JPRB, 0.24880E+00_JPRB, 0.24234E+00_JPRB, 0.24023E+00_JPRB ,&
&  0.23810E+00_JPRB, 0.23864E+00_JPRB, 0.23865E+00_JPRB, 0.23909E+00_JPRB, 0.23968E+00_JPRB, 0.24121E+00_JPRB, 0.24071E+00_JPRB ,&
&  0.23926E+00_JPRB, 0.23719E+00_JPRB, 0.23471E+00_JPRB, 0.23248E+00_JPRB, 0.23293E+00_JPRB, 0.23238E+00_JPRB, 0.23113E+00_JPRB ,&
&  0.23017E+00_JPRB, 0.23095E+00_JPRB, 0.23143E+00_JPRB, 0.23298E+00_JPRB, 0.23803E+00_JPRB, 0.23578E+00_JPRB, 0.23503E+00_JPRB ,&
&  0.23562E+00_JPRB, 0.23638E+00_JPRB, 0.23793E+00_JPRB, 0.23935E+00_JPRB, 0.24297E+00_JPRB, 0.24262E+00_JPRB, 0.24641E+00_JPRB ,&
&  0.24533E+00_JPRB, 0.24481E+00_JPRB, 0.24327E+00_JPRB, 0.24002E+00_JPRB, 0.23487E+00_JPRB, 0.23164E+00_JPRB, 0.22863E+00_JPRB ,&
&  0.22516E+00_JPRB, 0.22224E+00_JPRB, 0.21843E+00_JPRB, 0.21712E+00_JPRB, 0.21497E+00_JPRB, 0.21305E+00_JPRB, 0.21134E+00_JPRB ,&
&  0.21016E+00_JPRB, 0.20849E+00_JPRB, 0.20711E+00_JPRB, 0.20618E+00_JPRB, 0.20597E+00_JPRB, 0.20614E+00_JPRB, 0.20749E+00_JPRB ,&
&  0.20994E+00_JPRB, 0.21178E+00_JPRB, 0.21325E+00_JPRB, 0.21498E+00_JPRB, 0.21324E+00_JPRB, 0.21016E+00_JPRB, 0.20721E+00_JPRB ,&
&  0.20259E+00_JPRB, 0.20016E+00_JPRB, 0.19775E+00_JPRB, 0.19627E+00_JPRB, 0.19580E+00_JPRB, 0.19556E+00_JPRB, 0.19572E+00_JPRB ,&
&  0.19589E+00_JPRB, 0.19643E+00_JPRB, 0.19681E+00_JPRB, 0.19740E+00_JPRB, 0.19817E+00_JPRB, 0.19894E+00_JPRB, 0.20014E+00_JPRB ,&
&  0.20198E+00_JPRB, 0.20431E+00_JPRB, 0.20728E+00_JPRB, 0.21059E+00_JPRB, 0.21477E+00_JPRB, 0.22005E+00_JPRB, 0.22753E+00_JPRB ,&
&  0.23449E+00_JPRB, 0.24435E+00_JPRB, 0.24487E+00_JPRB, 0.25414E+00_JPRB, 0.25888E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.25718E+00_JPRB, 0.25955E+00_JPRB, 0.26002E+00_JPRB, 0.26054E+00_JPRB, 0.25922E+00_JPRB, 0.25585E+00_JPRB, 0.25227E+00_JPRB ,&
&  0.24541E+00_JPRB, 0.24010E+00_JPRB, 0.23374E+00_JPRB, 0.22796E+00_JPRB, 0.22238E+00_JPRB, 0.21897E+00_JPRB, 0.21677E+00_JPRB ,&
&  0.21623E+00_JPRB, 0.21739E+00_JPRB, 0.21817E+00_JPRB, 0.21867E+00_JPRB, 0.21872E+00_JPRB, 0.21954E+00_JPRB, 0.21919E+00_JPRB ,&
&  0.21845E+00_JPRB, 0.21741E+00_JPRB, 0.21667E+00_JPRB, 0.21739E+00_JPRB, 0.21961E+00_JPRB, 0.22184E+00_JPRB, 0.22372E+00_JPRB ,&
&  0.22515E+00_JPRB, 0.22402E+00_JPRB, 0.22399E+00_JPRB, 0.22318E+00_JPRB, 0.22318E+00_JPRB, 0.22322E+00_JPRB, 0.22389E+00_JPRB ,&
&  0.22586E+00_JPRB, 0.22742E+00_JPRB, 0.23055E+00_JPRB, 0.23209E+00_JPRB, 0.23128E+00_JPRB, 0.23506E+00_JPRB, 0.23664E+00_JPRB ,&
&  0.23668E+00_JPRB, 0.23320E+00_JPRB, 0.22914E+00_JPRB, 0.22576E+00_JPRB, 0.22203E+00_JPRB, 0.21890E+00_JPRB, 0.21635E+00_JPRB ,&
&  0.21404E+00_JPRB, 0.21291E+00_JPRB, 0.21040E+00_JPRB, 0.20827E+00_JPRB, 0.20688E+00_JPRB, 0.20541E+00_JPRB, 0.20452E+00_JPRB ,&
&  0.20400E+00_JPRB, 0.20301E+00_JPRB, 0.20286E+00_JPRB, 0.20227E+00_JPRB, 0.20233E+00_JPRB, 0.20241E+00_JPRB, 0.20273E+00_JPRB ,&
&  0.20365E+00_JPRB, 0.20494E+00_JPRB, 0.20483E+00_JPRB, 0.20477E+00_JPRB, 0.20285E+00_JPRB, 0.20078E+00_JPRB, 0.19907E+00_JPRB ,&
&  0.19665E+00_JPRB, 0.19493E+00_JPRB, 0.19386E+00_JPRB, 0.19277E+00_JPRB, 0.19254E+00_JPRB, 0.19247E+00_JPRB, 0.19229E+00_JPRB ,&
&  0.19243E+00_JPRB, 0.19276E+00_JPRB, 0.19334E+00_JPRB, 0.19468E+00_JPRB, 0.19583E+00_JPRB, 0.19750E+00_JPRB, 0.19934E+00_JPRB ,&
&  0.20075E+00_JPRB, 0.20340E+00_JPRB, 0.20615E+00_JPRB, 0.20929E+00_JPRB, 0.21593E+00_JPRB, 0.22137E+00_JPRB, 0.23092E+00_JPRB ,&
&  0.23729E+00_JPRB, 0.24558E+00_JPRB, 0.24261E+00_JPRB, 0.24936E+00_JPRB, 0.25449E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.25559E+00_JPRB, 0.25687E+00_JPRB, 0.25616E+00_JPRB, 0.25602E+00_JPRB, 0.25396E+00_JPRB, 0.25120E+00_JPRB, 0.24709E+00_JPRB ,&
&  0.24346E+00_JPRB, 0.23708E+00_JPRB, 0.23232E+00_JPRB, 0.22696E+00_JPRB, 0.22247E+00_JPRB, 0.21843E+00_JPRB, 0.21549E+00_JPRB ,&
&  0.21401E+00_JPRB, 0.21328E+00_JPRB, 0.21282E+00_JPRB, 0.21171E+00_JPRB, 0.20973E+00_JPRB, 0.20843E+00_JPRB, 0.20684E+00_JPRB ,&
&  0.20569E+00_JPRB, 0.20574E+00_JPRB, 0.20685E+00_JPRB, 0.20894E+00_JPRB, 0.21270E+00_JPRB, 0.21507E+00_JPRB, 0.21733E+00_JPRB ,&
&  0.21901E+00_JPRB, 0.21442E+00_JPRB, 0.21457E+00_JPRB, 0.21535E+00_JPRB, 0.21573E+00_JPRB, 0.21560E+00_JPRB, 0.21555E+00_JPRB ,&
&  0.22026E+00_JPRB, 0.22165E+00_JPRB, 0.22829E+00_JPRB, 0.22790E+00_JPRB, 0.22711E+00_JPRB, 0.22953E+00_JPRB, 0.22868E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.22579E+00_JPRB, 0.22320E+00_JPRB, 0.21897E+00_JPRB, 0.21455E+00_JPRB, 0.21230E+00_JPRB, 0.20973E+00_JPRB ,&
&  0.20893E+00_JPRB, 0.20879E+00_JPRB, 0.20700E+00_JPRB, 0.20588E+00_JPRB, 0.20369E+00_JPRB, 0.20121E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19820E+00_JPRB, 0.19702E+00_JPRB, 0.19639E+00_JPRB, 0.19574E+00_JPRB, 0.19543E+00_JPRB, 0.19524E+00_JPRB, 0.19504E+00_JPRB ,&
&  0.19505E+00_JPRB, 0.19499E+00_JPRB, 0.19432E+00_JPRB, 0.19403E+00_JPRB, 0.19276E+00_JPRB, 0.19190E+00_JPRB, 0.19119E+00_JPRB ,&
&  0.19051E+00_JPRB, 0.19026E+00_JPRB, 0.19011E+00_JPRB, 0.19016E+00_JPRB, 0.19029E+00_JPRB, 0.19085E+00_JPRB, 0.19152E+00_JPRB ,&
&  0.19216E+00_JPRB, 0.19334E+00_JPRB, 0.19422E+00_JPRB, 0.19553E+00_JPRB, 0.19710E+00_JPRB, 0.19842E+00_JPRB, 0.20056E+00_JPRB ,&
&  0.20290E+00_JPRB, 0.20564E+00_JPRB, 0.20967E+00_JPRB, 0.21306E+00_JPRB, 0.21724E+00_JPRB, 0.22265E+00_JPRB, 0.22832E+00_JPRB ,&
&  0.23436E+00_JPRB, 0.24040E+00_JPRB, 0.24055E+00_JPRB, 0.24837E+00_JPRB, 0.25312E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.24428E+00_JPRB, 0.24501E+00_JPRB, 0.24575E+00_JPRB, 0.24593E+00_JPRB, 0.24591E+00_JPRB, 0.24472E+00_JPRB, 0.24318E+00_JPRB ,&
&  0.23850E+00_JPRB, 0.23520E+00_JPRB, 0.23139E+00_JPRB, 0.22757E+00_JPRB, 0.22417E+00_JPRB, 0.22037E+00_JPRB, 0.21713E+00_JPRB ,&
&  0.21418E+00_JPRB, 0.21191E+00_JPRB, 0.20998E+00_JPRB, 0.20754E+00_JPRB, 0.20451E+00_JPRB, 0.20175E+00_JPRB, 0.20020E+00_JPRB ,&
&  0.19990E+00_JPRB, 0.20043E+00_JPRB, 0.20220E+00_JPRB, 0.20390E+00_JPRB, 0.20570E+00_JPRB, 0.20705E+00_JPRB, 0.20845E+00_JPRB ,&
&  0.20793E+00_JPRB, 0.20768E+00_JPRB, 0.20741E+00_JPRB, 0.20680E+00_JPRB, 0.20783E+00_JPRB, 0.20919E+00_JPRB, 0.21016E+00_JPRB ,&
&  0.21130E+00_JPRB, 0.21378E+00_JPRB, 0.21573E+00_JPRB, 0.21748E+00_JPRB, 0.21779E+00_JPRB, 0.21736E+00_JPRB, 0.21588E+00_JPRB ,&
&  0.21386E+00_JPRB, 0.21164E+00_JPRB, 0.20957E+00_JPRB, 0.20776E+00_JPRB, 0.20636E+00_JPRB, 0.20468E+00_JPRB, 0.20320E+00_JPRB ,&
&  0.20248E+00_JPRB, 0.20183E+00_JPRB, 0.20090E+00_JPRB, 0.20000E+00_JPRB, 0.19860E+00_JPRB, 0.19738E+00_JPRB, 0.19611E+00_JPRB ,&
&  0.19498E+00_JPRB, 0.19392E+00_JPRB, 0.19293E+00_JPRB, 0.19192E+00_JPRB, 0.19122E+00_JPRB, 0.19059E+00_JPRB, 0.18998E+00_JPRB ,&
&  0.18934E+00_JPRB, 0.18914E+00_JPRB, 0.18868E+00_JPRB, 0.18853E+00_JPRB, 0.18822E+00_JPRB, 0.18803E+00_JPRB, 0.18825E+00_JPRB ,&
&  0.18888E+00_JPRB, 0.18953E+00_JPRB, 0.18982E+00_JPRB, 0.19022E+00_JPRB, 0.19065E+00_JPRB, 0.19120E+00_JPRB, 0.19211E+00_JPRB ,&
&  0.19281E+00_JPRB, 0.19362E+00_JPRB, 0.19423E+00_JPRB, 0.19502E+00_JPRB, 0.19659E+00_JPRB, 0.19820E+00_JPRB, 0.20003E+00_JPRB ,&
&  0.20236E+00_JPRB, 0.20477E+00_JPRB, 0.20762E+00_JPRB, 0.21033E+00_JPRB, 0.21398E+00_JPRB, 0.21573E+00_JPRB, 0.21935E+00_JPRB ,&
&  0.22333E+00_JPRB, 0.22375E+00_JPRB, 0.23767E+00_JPRB, 0.23899E+00_JPRB, 0.24090E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.21432E+00_JPRB, 0.21627E+00_JPRB, 0.21734E+00_JPRB, 0.21853E+00_JPRB, 0.21529E+00_JPRB, 0.22778E+00_JPRB, 0.22536E+00_JPRB ,&
&  0.22398E+00_JPRB, 0.21976E+00_JPRB, 0.21975E+00_JPRB, 0.21805E+00_JPRB, 0.21581E+00_JPRB, 0.21327E+00_JPRB, 0.21091E+00_JPRB ,&
&  0.20870E+00_JPRB, 0.20623E+00_JPRB, 0.20455E+00_JPRB, 0.20326E+00_JPRB, 0.20208E+00_JPRB, 0.20100E+00_JPRB, 0.20021E+00_JPRB ,&
&  0.20030E+00_JPRB, 0.20103E+00_JPRB, 0.20236E+00_JPRB, 0.20222E+00_JPRB, 0.20182E+00_JPRB, 0.20131E+00_JPRB, 0.20103E+00_JPRB ,&
&  0.20036E+00_JPRB, 0.19989E+00_JPRB, 0.19946E+00_JPRB, 0.19883E+00_JPRB, 0.19992E+00_JPRB, 0.20110E+00_JPRB, 0.20224E+00_JPRB ,&
&  0.20333E+00_JPRB, 0.20441E+00_JPRB, 0.20466E+00_JPRB, 0.20469E+00_JPRB, 0.20437E+00_JPRB, 0.20390E+00_JPRB, 0.20302E+00_JPRB ,&
&  0.20218E+00_JPRB, 0.20126E+00_JPRB, 0.20013E+00_JPRB, 0.19940E+00_JPRB, 0.19874E+00_JPRB, 0.19813E+00_JPRB, 0.19757E+00_JPRB ,&
&  0.19729E+00_JPRB, 0.19710E+00_JPRB, 0.19686E+00_JPRB, 0.19647E+00_JPRB, 0.19602E+00_JPRB, 0.19556E+00_JPRB, 0.19516E+00_JPRB ,&
&  0.19484E+00_JPRB, 0.19423E+00_JPRB, 0.19368E+00_JPRB, 0.19309E+00_JPRB, 0.19266E+00_JPRB, 0.19206E+00_JPRB, 0.19156E+00_JPRB ,&
&  0.19107E+00_JPRB, 0.19069E+00_JPRB, 0.19055E+00_JPRB, 0.19044E+00_JPRB, 0.19035E+00_JPRB, 0.19038E+00_JPRB, 0.19050E+00_JPRB ,&
&  0.19069E+00_JPRB, 0.19090E+00_JPRB, 0.19123E+00_JPRB, 0.19147E+00_JPRB, 0.19169E+00_JPRB, 0.19186E+00_JPRB, 0.19234E+00_JPRB ,&
&  0.19290E+00_JPRB, 0.19349E+00_JPRB, 0.19400E+00_JPRB, 0.19413E+00_JPRB, 0.19505E+00_JPRB, 0.19606E+00_JPRB, 0.19717E+00_JPRB ,&
&  0.19836E+00_JPRB, 0.19990E+00_JPRB, 0.20159E+00_JPRB, 0.20344E+00_JPRB, 0.20564E+00_JPRB, 0.20692E+00_JPRB, 0.20846E+00_JPRB ,&
&  0.21059E+00_JPRB, 0.21416E+00_JPRB, 0.21513E+00_JPRB, 0.21528E+00_JPRB, 0.21508E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.20206E+00_JPRB, 0.20173E+00_JPRB, 0.20134E+00_JPRB, 0.20089E+00_JPRB, 0.19960E+00_JPRB, 0.19889E+00_JPRB, 0.19833E+00_JPRB ,&
&  0.19795E+00_JPRB, 0.19770E+00_JPRB, 0.19777E+00_JPRB, 0.19792E+00_JPRB, 0.19809E+00_JPRB, 0.19825E+00_JPRB, 0.19840E+00_JPRB ,&
&  0.19837E+00_JPRB, 0.19837E+00_JPRB, 0.19748E+00_JPRB, 0.19675E+00_JPRB, 0.19628E+00_JPRB, 0.19591E+00_JPRB, 0.19546E+00_JPRB ,&
&  0.19509E+00_JPRB, 0.19466E+00_JPRB, 0.19416E+00_JPRB, 0.19386E+00_JPRB, 0.19394E+00_JPRB, 0.19380E+00_JPRB, 0.19387E+00_JPRB ,&
&  0.19392E+00_JPRB, 0.19423E+00_JPRB, 0.19442E+00_JPRB, 0.19462E+00_JPRB, 0.19520E+00_JPRB, 0.19576E+00_JPRB, 0.19598E+00_JPRB ,&
&  0.19614E+00_JPRB, 0.19601E+00_JPRB, 0.19588E+00_JPRB, 0.19578E+00_JPRB, 0.19575E+00_JPRB, 0.19579E+00_JPRB, 0.19578E+00_JPRB ,&
&  0.19567E+00_JPRB, 0.19549E+00_JPRB, 0.19525E+00_JPRB, 0.19494E+00_JPRB, 0.19468E+00_JPRB, 0.19459E+00_JPRB, 0.19463E+00_JPRB ,&
&  0.19466E+00_JPRB, 0.19455E+00_JPRB, 0.19433E+00_JPRB, 0.19396E+00_JPRB, 0.19359E+00_JPRB, 0.19321E+00_JPRB, 0.19286E+00_JPRB ,&
&  0.19252E+00_JPRB, 0.19236E+00_JPRB, 0.19220E+00_JPRB, 0.19208E+00_JPRB, 0.19199E+00_JPRB, 0.19186E+00_JPRB, 0.19172E+00_JPRB ,&
&  0.19161E+00_JPRB, 0.19151E+00_JPRB, 0.19138E+00_JPRB, 0.19128E+00_JPRB, 0.19121E+00_JPRB, 0.19114E+00_JPRB, 0.19111E+00_JPRB ,&
&  0.19111E+00_JPRB, 0.19111E+00_JPRB, 0.19112E+00_JPRB, 0.19127E+00_JPRB, 0.19145E+00_JPRB, 0.19168E+00_JPRB, 0.19195E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19259E+00_JPRB, 0.19294E+00_JPRB, 0.19320E+00_JPRB, 0.19362E+00_JPRB, 0.19404E+00_JPRB, 0.19449E+00_JPRB ,&
&  0.19493E+00_JPRB, 0.19548E+00_JPRB, 0.19610E+00_JPRB, 0.19680E+00_JPRB, 0.19762E+00_JPRB, 0.19805E+00_JPRB, 0.19862E+00_JPRB ,&
&  0.19928E+00_JPRB, 0.20019E+00_JPRB, 0.20065E+00_JPRB, 0.20098E+00_JPRB, 0.20132E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB, 0.19378E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB, 0.85629E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.86358E+00_JPRB, 0.86309E+00_JPRB, 0.86258E+00_JPRB, 0.86214E+00_JPRB, 0.86171E+00_JPRB, 0.86142E+00_JPRB, 0.86106E+00_JPRB ,&
&  0.86096E+00_JPRB, 0.86084E+00_JPRB, 0.86073E+00_JPRB, 0.86048E+00_JPRB, 0.86045E+00_JPRB, 0.86032E+00_JPRB, 0.86018E+00_JPRB ,&
&  0.85991E+00_JPRB, 0.85978E+00_JPRB, 0.85928E+00_JPRB, 0.85881E+00_JPRB, 0.85814E+00_JPRB, 0.85767E+00_JPRB, 0.85691E+00_JPRB ,&
&  0.85642E+00_JPRB, 0.85553E+00_JPRB, 0.85471E+00_JPRB, 0.85319E+00_JPRB, 0.85206E+00_JPRB, 0.85084E+00_JPRB, 0.84979E+00_JPRB ,&
&  0.84889E+00_JPRB, 0.84833E+00_JPRB, 0.84778E+00_JPRB, 0.84699E+00_JPRB, 0.84638E+00_JPRB, 0.84653E+00_JPRB, 0.84686E+00_JPRB ,&
&  0.84720E+00_JPRB, 0.84767E+00_JPRB, 0.84829E+00_JPRB, 0.84855E+00_JPRB, 0.84841E+00_JPRB, 0.84851E+00_JPRB, 0.84846E+00_JPRB ,&
&  0.84824E+00_JPRB, 0.84783E+00_JPRB, 0.84745E+00_JPRB, 0.84739E+00_JPRB, 0.84734E+00_JPRB, 0.84702E+00_JPRB, 0.84683E+00_JPRB ,&
&  0.84664E+00_JPRB, 0.84641E+00_JPRB, 0.84607E+00_JPRB, 0.84553E+00_JPRB, 0.84512E+00_JPRB, 0.84471E+00_JPRB, 0.84424E+00_JPRB ,&
&  0.84372E+00_JPRB, 0.84309E+00_JPRB, 0.84241E+00_JPRB, 0.84169E+00_JPRB, 0.84092E+00_JPRB, 0.84036E+00_JPRB, 0.84000E+00_JPRB ,&
&  0.83977E+00_JPRB, 0.83978E+00_JPRB, 0.83994E+00_JPRB, 0.84016E+00_JPRB, 0.84051E+00_JPRB, 0.84105E+00_JPRB, 0.84182E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.84369E+00_JPRB, 0.84498E+00_JPRB, 0.84655E+00_JPRB, 0.84806E+00_JPRB, 0.84960E+00_JPRB, 0.85130E+00_JPRB ,&
&  0.85281E+00_JPRB, 0.85397E+00_JPRB, 0.85484E+00_JPRB, 0.85575E+00_JPRB, 0.85710E+00_JPRB, 0.85822E+00_JPRB, 0.85916E+00_JPRB ,&
&  0.85999E+00_JPRB, 0.86092E+00_JPRB, 0.86178E+00_JPRB, 0.86269E+00_JPRB, 0.86305E+00_JPRB, 0.86342E+00_JPRB, 0.86374E+00_JPRB ,&
&  0.86399E+00_JPRB, 0.86406E+00_JPRB, 0.86407E+00_JPRB, 0.86403E+00_JPRB, 0.86386E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.86180E+00_JPRB, 0.86061E+00_JPRB, 0.85976E+00_JPRB, 0.85927E+00_JPRB, 0.85893E+00_JPRB, 0.85884E+00_JPRB, 0.85888E+00_JPRB ,&
&  0.85914E+00_JPRB, 0.85899E+00_JPRB, 0.85845E+00_JPRB, 0.85852E+00_JPRB, 0.85931E+00_JPRB, 0.86028E+00_JPRB, 0.86009E+00_JPRB ,&
&  0.86006E+00_JPRB, 0.85983E+00_JPRB, 0.85982E+00_JPRB, 0.85927E+00_JPRB, 0.85823E+00_JPRB, 0.85675E+00_JPRB, 0.85506E+00_JPRB ,&
&  0.85377E+00_JPRB, 0.85203E+00_JPRB, 0.85015E+00_JPRB, 0.84617E+00_JPRB, 0.84278E+00_JPRB, 0.83948E+00_JPRB, 0.83637E+00_JPRB ,&
&  0.83204E+00_JPRB, 0.82830E+00_JPRB, 0.82540E+00_JPRB, 0.82305E+00_JPRB, 0.82681E+00_JPRB, 0.82935E+00_JPRB, 0.83082E+00_JPRB ,&
&  0.83101E+00_JPRB, 0.82708E+00_JPRB, 0.82596E+00_JPRB, 0.82545E+00_JPRB, 0.82526E+00_JPRB, 0.82874E+00_JPRB, 0.83115E+00_JPRB ,&
&  0.83318E+00_JPRB, 0.83484E+00_JPRB, 0.83850E+00_JPRB, 0.84032E+00_JPRB, 0.84162E+00_JPRB, 0.84236E+00_JPRB, 0.84225E+00_JPRB ,&
&  0.84286E+00_JPRB, 0.84349E+00_JPRB, 0.84401E+00_JPRB, 0.84381E+00_JPRB, 0.84335E+00_JPRB, 0.84291E+00_JPRB, 0.84198E+00_JPRB ,&
&  0.84044E+00_JPRB, 0.83872E+00_JPRB, 0.83703E+00_JPRB, 0.83523E+00_JPRB, 0.83319E+00_JPRB, 0.83172E+00_JPRB, 0.83045E+00_JPRB ,&
&  0.82926E+00_JPRB, 0.82834E+00_JPRB, 0.82778E+00_JPRB, 0.82743E+00_JPRB, 0.82733E+00_JPRB, 0.82792E+00_JPRB, 0.82940E+00_JPRB ,&
&  0.83159E+00_JPRB, 0.83438E+00_JPRB, 0.83765E+00_JPRB, 0.84115E+00_JPRB, 0.84415E+00_JPRB, 0.84681E+00_JPRB, 0.85004E+00_JPRB ,&
&  0.85355E+00_JPRB, 0.85632E+00_JPRB, 0.85858E+00_JPRB, 0.86063E+00_JPRB, 0.86313E+00_JPRB, 0.86460E+00_JPRB, 0.86535E+00_JPRB ,&
&  0.86591E+00_JPRB, 0.86717E+00_JPRB, 0.86850E+00_JPRB, 0.87006E+00_JPRB, 0.87083E+00_JPRB, 0.87081E+00_JPRB, 0.87018E+00_JPRB ,&
&  0.86927E+00_JPRB, 0.86841E+00_JPRB, 0.86707E+00_JPRB, 0.86530E+00_JPRB, 0.86349E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.85669E+00_JPRB, 0.85789E+00_JPRB, 0.85973E+00_JPRB, 0.86208E+00_JPRB, 0.86482E+00_JPRB, 0.86678E+00_JPRB, 0.86891E+00_JPRB ,&
&  0.87043E+00_JPRB, 0.87092E+00_JPRB, 0.87054E+00_JPRB, 0.87130E+00_JPRB, 0.87161E+00_JPRB, 0.87226E+00_JPRB, 0.87138E+00_JPRB ,&
&  0.87122E+00_JPRB, 0.86998E+00_JPRB, 0.87093E+00_JPRB, 0.87137E+00_JPRB, 0.87126E+00_JPRB, 0.87147E+00_JPRB, 0.86427E+00_JPRB ,&
&  0.85765E+00_JPRB, 0.85266E+00_JPRB, 0.84583E+00_JPRB, 0.83919E+00_JPRB, 0.83472E+00_JPRB, 0.80765E+00_JPRB, 0.79959E+00_JPRB ,&
&  0.81310E+00_JPRB, 0.82153E+00_JPRB, 0.82832E+00_JPRB, 0.83287E+00_JPRB, 0.81556E+00_JPRB, 0.79674E+00_JPRB, 0.78543E+00_JPRB ,&
&  0.75905E+00_JPRB, 0.76752E+00_JPRB, 0.77354E+00_JPRB, 0.77832E+00_JPRB, 0.78592E+00_JPRB, 0.78826E+00_JPRB, 0.79539E+00_JPRB ,&
&  0.81254E+00_JPRB, 0.81739E+00_JPRB, 0.82420E+00_JPRB, 0.83004E+00_JPRB, 0.83371E+00_JPRB, 0.83805E+00_JPRB, 0.84670E+00_JPRB ,&
&  0.85052E+00_JPRB, 0.85409E+00_JPRB, 0.85640E+00_JPRB, 0.85612E+00_JPRB, 0.85640E+00_JPRB, 0.85543E+00_JPRB, 0.85415E+00_JPRB ,&
&  0.85046E+00_JPRB, 0.84836E+00_JPRB, 0.84655E+00_JPRB, 0.84438E+00_JPRB, 0.84274E+00_JPRB, 0.83832E+00_JPRB, 0.83511E+00_JPRB ,&
&  0.83304E+00_JPRB, 0.83199E+00_JPRB, 0.83156E+00_JPRB, 0.83219E+00_JPRB, 0.83417E+00_JPRB, 0.83698E+00_JPRB, 0.84001E+00_JPRB ,&
&  0.84407E+00_JPRB, 0.84776E+00_JPRB, 0.85236E+00_JPRB, 0.85699E+00_JPRB, 0.86072E+00_JPRB, 0.86382E+00_JPRB, 0.86670E+00_JPRB ,&
&  0.87031E+00_JPRB, 0.87421E+00_JPRB, 0.87644E+00_JPRB, 0.87661E+00_JPRB, 0.87902E+00_JPRB, 0.88087E+00_JPRB, 0.88157E+00_JPRB ,&
&  0.88202E+00_JPRB, 0.88298E+00_JPRB, 0.88213E+00_JPRB, 0.88003E+00_JPRB, 0.87731E+00_JPRB, 0.87468E+00_JPRB, 0.87179E+00_JPRB ,&
&  0.86866E+00_JPRB, 0.86572E+00_JPRB, 0.86269E+00_JPRB, 0.85953E+00_JPRB, 0.85735E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.86213E+00_JPRB, 0.86674E+00_JPRB, 0.87247E+00_JPRB, 0.87590E+00_JPRB, 0.88022E+00_JPRB, 0.88195E+00_JPRB, 0.88457E+00_JPRB ,&
&  0.88531E+00_JPRB, 0.88658E+00_JPRB, 0.88743E+00_JPRB, 0.88760E+00_JPRB, 0.88744E+00_JPRB, 0.88630E+00_JPRB, 0.88531E+00_JPRB ,&
&  0.88483E+00_JPRB, 0.88385E+00_JPRB, 0.88445E+00_JPRB, 0.88462E+00_JPRB, 0.88398E+00_JPRB, 0.88347E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.86603E+00_JPRB, 0.85720E+00_JPRB, 0.82297E+00_JPRB, 0.82253E+00_JPRB, 0.81874E+00_JPRB, 0.81690E+00_JPRB, 0.81954E+00_JPRB ,&
&  0.81489E+00_JPRB, 0.80945E+00_JPRB, 0.78886E+00_JPRB, 0.76931E+00_JPRB, 0.75589E+00_JPRB, 0.74158E+00_JPRB, 0.74215E+00_JPRB ,&
&  0.74155E+00_JPRB, 0.74737E+00_JPRB, 0.75269E+00_JPRB, 0.75570E+00_JPRB, 0.76554E+00_JPRB, 0.77045E+00_JPRB, 0.78129E+00_JPRB ,&
&  0.79492E+00_JPRB, 0.80362E+00_JPRB, 0.80132E+00_JPRB, 0.79889E+00_JPRB, 0.82014E+00_JPRB, 0.83486E+00_JPRB, 0.84811E+00_JPRB ,&
&  0.85660E+00_JPRB, 0.85826E+00_JPRB, 0.86217E+00_JPRB, 0.86205E+00_JPRB, 0.86337E+00_JPRB, 0.86713E+00_JPRB, 0.86811E+00_JPRB ,&
&  0.86991E+00_JPRB, 0.86828E+00_JPRB, 0.86449E+00_JPRB, 0.85942E+00_JPRB, 0.85138E+00_JPRB, 0.84603E+00_JPRB, 0.84269E+00_JPRB ,&
&  0.84224E+00_JPRB, 0.84460E+00_JPRB, 0.84821E+00_JPRB, 0.85346E+00_JPRB, 0.85898E+00_JPRB, 0.86329E+00_JPRB, 0.86629E+00_JPRB ,&
&  0.86857E+00_JPRB, 0.86999E+00_JPRB, 0.87615E+00_JPRB, 0.87748E+00_JPRB, 0.88075E+00_JPRB, 0.88909E+00_JPRB, 0.88879E+00_JPRB ,&
&  0.89094E+00_JPRB, 0.89639E+00_JPRB, 0.89994E+00_JPRB, 0.90306E+00_JPRB, 0.90523E+00_JPRB, 0.90631E+00_JPRB, 0.90764E+00_JPRB ,&
&  0.90808E+00_JPRB, 0.90685E+00_JPRB, 0.90334E+00_JPRB, 0.89843E+00_JPRB, 0.89312E+00_JPRB, 0.88762E+00_JPRB, 0.88043E+00_JPRB ,&
&  0.87338E+00_JPRB, 0.86637E+00_JPRB, 0.86137E+00_JPRB, 0.85788E+00_JPRB, 0.85910E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.87862E+00_JPRB, 0.88312E+00_JPRB, 0.89022E+00_JPRB, 0.89267E+00_JPRB, 0.89671E+00_JPRB, 0.89814E+00_JPRB, 0.89955E+00_JPRB ,&
&  0.90015E+00_JPRB, 0.90079E+00_JPRB, 0.90050E+00_JPRB, 0.90050E+00_JPRB, 0.89987E+00_JPRB, 0.89945E+00_JPRB, 0.89882E+00_JPRB ,&
&  0.89762E+00_JPRB, 0.89639E+00_JPRB, 0.89515E+00_JPRB, 0.89405E+00_JPRB, 0.89382E+00_JPRB, 0.89368E+00_JPRB, 0.88935E+00_JPRB ,&
&  0.88248E+00_JPRB, 0.86787E+00_JPRB, 0.84156E+00_JPRB, 0.83978E+00_JPRB, 0.83740E+00_JPRB, 0.83984E+00_JPRB, 0.83872E+00_JPRB ,&
&  0.82585E+00_JPRB, 0.77372E+00_JPRB, 0.77940E+00_JPRB, 0.76879E+00_JPRB, 0.76560E+00_JPRB, 0.76175E+00_JPRB, 0.74754E+00_JPRB ,&
&  0.74284E+00_JPRB, 0.73698E+00_JPRB, 0.73585E+00_JPRB, 0.74347E+00_JPRB, 0.75035E+00_JPRB, 0.76075E+00_JPRB, 0.77310E+00_JPRB ,&
&  0.76999E+00_JPRB, 0.78748E+00_JPRB, 0.79959E+00_JPRB, 0.80739E+00_JPRB, 0.82845E+00_JPRB, 0.84634E+00_JPRB, 0.85087E+00_JPRB ,&
&  0.85636E+00_JPRB, 0.85396E+00_JPRB, 0.85707E+00_JPRB, 0.86548E+00_JPRB, 0.87445E+00_JPRB, 0.87387E+00_JPRB, 0.87388E+00_JPRB ,&
&  0.87208E+00_JPRB, 0.87012E+00_JPRB, 0.86762E+00_JPRB, 0.86435E+00_JPRB, 0.86273E+00_JPRB, 0.86459E+00_JPRB, 0.86759E+00_JPRB ,&
&  0.87413E+00_JPRB, 0.88116E+00_JPRB, 0.88943E+00_JPRB, 0.89720E+00_JPRB, 0.90334E+00_JPRB, 0.90928E+00_JPRB, 0.91495E+00_JPRB ,&
&  0.92063E+00_JPRB, 0.92461E+00_JPRB, 0.92569E+00_JPRB, 0.92640E+00_JPRB, 0.92567E+00_JPRB, 0.92231E+00_JPRB, 0.92285E+00_JPRB ,&
&  0.92678E+00_JPRB, 0.92940E+00_JPRB, 0.93057E+00_JPRB, 0.93207E+00_JPRB, 0.93463E+00_JPRB, 0.93592E+00_JPRB, 0.93653E+00_JPRB ,&
&  0.93655E+00_JPRB, 0.93452E+00_JPRB, 0.93064E+00_JPRB, 0.92177E+00_JPRB, 0.91264E+00_JPRB, 0.90694E+00_JPRB, 0.89703E+00_JPRB ,&
&  0.88698E+00_JPRB, 0.87568E+00_JPRB, 0.87062E+00_JPRB, 0.86760E+00_JPRB, 0.87236E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.90081E+00_JPRB, 0.90735E+00_JPRB, 0.91512E+00_JPRB, 0.91917E+00_JPRB, 0.92342E+00_JPRB, 0.92452E+00_JPRB, 0.92653E+00_JPRB ,&
&  0.92765E+00_JPRB, 0.92917E+00_JPRB, 0.92978E+00_JPRB, 0.92978E+00_JPRB, 0.92873E+00_JPRB, 0.92730E+00_JPRB, 0.92502E+00_JPRB ,&
&  0.92129E+00_JPRB, 0.91767E+00_JPRB, 0.91205E+00_JPRB, 0.90726E+00_JPRB, 0.90488E+00_JPRB, 0.90545E+00_JPRB, 0.90276E+00_JPRB ,&
&  0.89656E+00_JPRB, 0.88389E+00_JPRB, 0.86411E+00_JPRB, 0.85846E+00_JPRB, 0.85995E+00_JPRB, 0.84848E+00_JPRB, 0.84530E+00_JPRB ,&
&  0.84123E+00_JPRB, 0.83926E+00_JPRB, 0.82783E+00_JPRB, 0.82076E+00_JPRB, 0.80312E+00_JPRB, 0.73848E+00_JPRB, 0.73522E+00_JPRB ,&
&  0.73233E+00_JPRB, 0.72333E+00_JPRB, 0.72270E+00_JPRB, 0.72523E+00_JPRB, 0.73304E+00_JPRB, 0.75915E+00_JPRB, 0.78086E+00_JPRB ,&
&  0.77685E+00_JPRB, 0.79752E+00_JPRB, 0.80691E+00_JPRB, 0.81626E+00_JPRB, 0.83384E+00_JPRB, 0.84976E+00_JPRB, 0.85117E+00_JPRB ,&
&  0.85514E+00_JPRB, 0.85800E+00_JPRB, 0.86538E+00_JPRB, 0.87324E+00_JPRB, 0.87930E+00_JPRB, 0.88241E+00_JPRB, 0.88655E+00_JPRB ,&
&  0.89163E+00_JPRB, 0.89558E+00_JPRB, 0.90074E+00_JPRB, 0.90266E+00_JPRB, 0.90305E+00_JPRB, 0.90417E+00_JPRB, 0.90797E+00_JPRB ,&
&  0.91669E+00_JPRB, 0.92564E+00_JPRB, 0.93934E+00_JPRB, 0.94039E+00_JPRB, 0.94573E+00_JPRB, 0.95089E+00_JPRB, 0.96012E+00_JPRB ,&
&  0.96573E+00_JPRB, 0.97958E+00_JPRB, 0.98976E+00_JPRB, 0.99528E+00_JPRB, 0.98748E+00_JPRB, 0.97841E+00_JPRB, 0.97244E+00_JPRB ,&
&  0.96976E+00_JPRB, 0.96994E+00_JPRB, 0.96162E+00_JPRB, 0.95960E+00_JPRB, 0.95942E+00_JPRB, 0.96042E+00_JPRB, 0.96172E+00_JPRB ,&
&  0.96190E+00_JPRB, 0.95767E+00_JPRB, 0.95069E+00_JPRB, 0.93969E+00_JPRB, 0.93035E+00_JPRB, 0.92224E+00_JPRB, 0.90700E+00_JPRB ,&
&  0.89416E+00_JPRB, 0.88137E+00_JPRB, 0.87912E+00_JPRB, 0.88066E+00_JPRB, 0.88980E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.92506E+00_JPRB, 0.93682E+00_JPRB, 0.94781E+00_JPRB, 0.95692E+00_JPRB, 0.96052E+00_JPRB, 0.96090E+00_JPRB, 0.95828E+00_JPRB ,&
&  0.95524E+00_JPRB, 0.95431E+00_JPRB, 0.95275E+00_JPRB, 0.95156E+00_JPRB, 0.94990E+00_JPRB, 0.94657E+00_JPRB, 0.94252E+00_JPRB ,&
&  0.93895E+00_JPRB, 0.92264E+00_JPRB, 0.91445E+00_JPRB, 0.91360E+00_JPRB, 0.91074E+00_JPRB, 0.90953E+00_JPRB, 0.90838E+00_JPRB ,&
&  0.90078E+00_JPRB, 0.89072E+00_JPRB, 0.88356E+00_JPRB, 0.87336E+00_JPRB, 0.86299E+00_JPRB, 0.86959E+00_JPRB, 0.86855E+00_JPRB ,&
&  0.84444E+00_JPRB, 0.82626E+00_JPRB, 0.83365E+00_JPRB, 0.83541E+00_JPRB, 0.80055E+00_JPRB, 0.76524E+00_JPRB, 0.74606E+00_JPRB ,&
&  0.75107E+00_JPRB, 0.73479E+00_JPRB, 0.73233E+00_JPRB, 0.73478E+00_JPRB, 0.74608E+00_JPRB, 0.75750E+00_JPRB, 0.77954E+00_JPRB ,&
&  0.78483E+00_JPRB, 0.78845E+00_JPRB, 0.80801E+00_JPRB, 0.82769E+00_JPRB, 0.83892E+00_JPRB, 0.85027E+00_JPRB, 0.85487E+00_JPRB ,&
&  0.86461E+00_JPRB, 0.87416E+00_JPRB, 0.88264E+00_JPRB, 0.89067E+00_JPRB, 0.90058E+00_JPRB, 0.90431E+00_JPRB, 0.91058E+00_JPRB ,&
&  0.95053E+00_JPRB, 0.98306E+00_JPRB, 0.99998E+00_JPRB, 0.10124E+01_JPRB, 0.10151E+01_JPRB, 0.10124E+01_JPRB, 0.99992E+00_JPRB ,&
&  0.99623E+00_JPRB, 0.10011E+01_JPRB, 0.10052E+01_JPRB, 0.10134E+01_JPRB, 0.10192E+01_JPRB, 0.10202E+01_JPRB, 0.10135E+01_JPRB ,&
&  0.10002E+01_JPRB, 0.10037E+01_JPRB, 0.10869E+01_JPRB, 0.11068E+01_JPRB, 0.10935E+01_JPRB, 0.10650E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.10119E+01_JPRB, 0.99825E+00_JPRB, 0.98998E+00_JPRB, 0.98357E+00_JPRB, 0.97890E+00_JPRB, 0.97164E+00_JPRB, 0.97260E+00_JPRB ,&
&  0.96980E+00_JPRB, 0.96165E+00_JPRB, 0.95326E+00_JPRB, 0.94380E+00_JPRB, 0.93510E+00_JPRB, 0.92834E+00_JPRB, 0.91796E+00_JPRB ,&
&  0.90717E+00_JPRB, 0.89612E+00_JPRB, 0.89622E+00_JPRB, 0.90189E+00_JPRB, 0.91349E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.95416E+00_JPRB, 0.96980E+00_JPRB, 0.98502E+00_JPRB, 0.99600E+00_JPRB, 0.99521E+00_JPRB, 0.97745E+00_JPRB, 0.96155E+00_JPRB ,&
&  0.95090E+00_JPRB, 0.94687E+00_JPRB, 0.94344E+00_JPRB, 0.93876E+00_JPRB, 0.93571E+00_JPRB, 0.93278E+00_JPRB, 0.93078E+00_JPRB ,&
&  0.92976E+00_JPRB, 0.92553E+00_JPRB, 0.91610E+00_JPRB, 0.90583E+00_JPRB, 0.90092E+00_JPRB, 0.90092E+00_JPRB, 0.90224E+00_JPRB ,&
&  0.89878E+00_JPRB, 0.89494E+00_JPRB, 0.89659E+00_JPRB, 0.88367E+00_JPRB, 0.87307E+00_JPRB, 0.85480E+00_JPRB, 0.84489E+00_JPRB ,&
&  0.84769E+00_JPRB, 0.84990E+00_JPRB, 0.82869E+00_JPRB, 0.82779E+00_JPRB, 0.83156E+00_JPRB, 0.79723E+00_JPRB, 0.77189E+00_JPRB ,&
&  0.75803E+00_JPRB, 0.74790E+00_JPRB, 0.74351E+00_JPRB, 0.74583E+00_JPRB, 0.77369E+00_JPRB, 0.81296E+00_JPRB, 0.82185E+00_JPRB ,&
&  0.77969E+00_JPRB, 0.80022E+00_JPRB, 0.82724E+00_JPRB, 0.84091E+00_JPRB, 0.85130E+00_JPRB, 0.86606E+00_JPRB, 0.88060E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.90436E+00_JPRB, 0.91075E+00_JPRB, 0.92623E+00_JPRB, 0.93152E+00_JPRB, 0.92696E+00_JPRB, 0.94504E+00_JPRB ,&
&  0.10503E+01_JPRB, 0.11323E+01_JPRB, 0.11687E+01_JPRB, 0.11779E+01_JPRB, 0.11727E+01_JPRB, 0.11536E+01_JPRB, 0.11395E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11168E+01_JPRB, 0.11085E+01_JPRB, 0.11022E+01_JPRB, 0.10953E+01_JPRB, 0.10836E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.10746E+01_JPRB, 0.13062E+01_JPRB, 0.12855E+01_JPRB, 0.12239E+01_JPRB, 0.11766E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.10656E+01_JPRB, 0.10505E+01_JPRB, 0.10338E+01_JPRB, 0.99901E+00_JPRB, 0.97337E+00_JPRB, 0.96260E+00_JPRB ,&
&  0.95653E+00_JPRB, 0.94480E+00_JPRB, 0.93628E+00_JPRB, 0.92914E+00_JPRB, 0.92424E+00_JPRB, 0.92386E+00_JPRB, 0.92052E+00_JPRB ,&
&  0.91579E+00_JPRB, 0.91746E+00_JPRB, 0.92335E+00_JPRB, 0.93425E+00_JPRB, 0.94391E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.97909E+00_JPRB, 0.10125E+01_JPRB, 0.10350E+01_JPRB, 0.10421E+01_JPRB, 0.10336E+01_JPRB, 0.98671E+00_JPRB, 0.94501E+00_JPRB ,&
&  0.92046E+00_JPRB, 0.91182E+00_JPRB, 0.91146E+00_JPRB, 0.91095E+00_JPRB, 0.91172E+00_JPRB, 0.91307E+00_JPRB, 0.91206E+00_JPRB ,&
&  0.90887E+00_JPRB, 0.90805E+00_JPRB, 0.90762E+00_JPRB, 0.90323E+00_JPRB, 0.89650E+00_JPRB, 0.89238E+00_JPRB, 0.89248E+00_JPRB ,&
&  0.89219E+00_JPRB, 0.88996E+00_JPRB, 0.88913E+00_JPRB, 0.88395E+00_JPRB, 0.88208E+00_JPRB, 0.87489E+00_JPRB, 0.86097E+00_JPRB ,&
&  0.84212E+00_JPRB, 0.84351E+00_JPRB, 0.85422E+00_JPRB, 0.85156E+00_JPRB, 0.85574E+00_JPRB, 0.83343E+00_JPRB, 0.80178E+00_JPRB ,&
&  0.78103E+00_JPRB, 0.77358E+00_JPRB, 0.78978E+00_JPRB, 0.83445E+00_JPRB, 0.86101E+00_JPRB, 0.81745E+00_JPRB, 0.83478E+00_JPRB ,&
&  0.83762E+00_JPRB, 0.83954E+00_JPRB, 0.84513E+00_JPRB, 0.86302E+00_JPRB, 0.88281E+00_JPRB, 0.90027E+00_JPRB, 0.91372E+00_JPRB ,&
&  0.92737E+00_JPRB, 0.93925E+00_JPRB, 0.94820E+00_JPRB, 0.94366E+00_JPRB, 0.94867E+00_JPRB, 0.99520E+00_JPRB, 0.10540E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.11762E+01_JPRB, 0.11989E+01_JPRB, 0.12428E+01_JPRB, 0.12702E+01_JPRB, 0.12758E+01_JPRB, 0.12686E+01_JPRB ,&
&  0.12590E+01_JPRB, 0.12301E+01_JPRB, 0.12033E+01_JPRB, 0.11891E+01_JPRB, 0.11864E+01_JPRB, 0.11884E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.11303E+01_JPRB, 0.10978E+01_JPRB, 0.10466E+01_JPRB, 0.10451E+01_JPRB, 0.10672E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10717E+01_JPRB, 0.10650E+01_JPRB, 0.10495E+01_JPRB, 0.99409E+00_JPRB, 0.95616E+00_JPRB, 0.93428E+00_JPRB ,&
&  0.92086E+00_JPRB, 0.90269E+00_JPRB, 0.90503E+00_JPRB, 0.90202E+00_JPRB, 0.90931E+00_JPRB, 0.91843E+00_JPRB, 0.91626E+00_JPRB ,&
&  0.91494E+00_JPRB, 0.91662E+00_JPRB, 0.92558E+00_JPRB, 0.93688E+00_JPRB, 0.95563E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.10739E+01_JPRB, 0.10906E+01_JPRB, 0.10890E+01_JPRB, 0.10891E+01_JPRB, 0.10660E+01_JPRB, 0.10354E+01_JPRB, 0.10002E+01_JPRB ,&
&  0.97595E+00_JPRB, 0.94558E+00_JPRB, 0.91684E+00_JPRB, 0.89157E+00_JPRB, 0.87854E+00_JPRB, 0.87457E+00_JPRB, 0.87820E+00_JPRB ,&
&  0.88507E+00_JPRB, 0.89233E+00_JPRB, 0.89684E+00_JPRB, 0.89405E+00_JPRB, 0.88866E+00_JPRB, 0.88695E+00_JPRB, 0.88883E+00_JPRB ,&
&  0.88853E+00_JPRB, 0.88780E+00_JPRB, 0.88616E+00_JPRB, 0.88290E+00_JPRB, 0.87844E+00_JPRB, 0.87268E+00_JPRB, 0.87137E+00_JPRB ,&
&  0.87114E+00_JPRB, 0.87122E+00_JPRB, 0.87754E+00_JPRB, 0.87763E+00_JPRB, 0.88487E+00_JPRB, 0.86624E+00_JPRB, 0.83849E+00_JPRB ,&
&  0.81592E+00_JPRB, 0.81739E+00_JPRB, 0.87325E+00_JPRB, 0.84454E+00_JPRB, 0.86835E+00_JPRB, 0.88473E+00_JPRB, 0.88133E+00_JPRB ,&
&  0.87983E+00_JPRB, 0.88352E+00_JPRB, 0.90062E+00_JPRB, 0.91500E+00_JPRB, 0.92938E+00_JPRB, 0.94654E+00_JPRB, 0.96135E+00_JPRB ,&
&  0.98260E+00_JPRB, 0.10055E+01_JPRB, 0.10176E+01_JPRB, 0.10102E+01_JPRB, 0.10822E+01_JPRB, 0.12937E+01_JPRB, 0.13431E+01_JPRB ,&
&  0.13777E+01_JPRB, 0.14042E+01_JPRB, 0.14512E+01_JPRB, 0.14866E+01_JPRB, 0.14632E+01_JPRB, 0.14443E+01_JPRB, 0.14315E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.13760E+01_JPRB, 0.13618E+01_JPRB, 0.13777E+01_JPRB, 0.13722E+01_JPRB, 0.13648E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.13237E+01_JPRB, 0.12869E+01_JPRB, 0.12307E+01_JPRB, 0.12003E+01_JPRB, 0.11803E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.11440E+01_JPRB, 0.11332E+01_JPRB, 0.11087E+01_JPRB, 0.10433E+01_JPRB, 0.97907E+00_JPRB, 0.94108E+00_JPRB ,&
&  0.91363E+00_JPRB, 0.89054E+00_JPRB, 0.89212E+00_JPRB, 0.90138E+00_JPRB, 0.91676E+00_JPRB, 0.93167E+00_JPRB, 0.92938E+00_JPRB ,&
&  0.93681E+00_JPRB, 0.94803E+00_JPRB, 0.97267E+00_JPRB, 0.10060E+01_JPRB, 0.10416E+01_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.11777E+01_JPRB, 0.11838E+01_JPRB, 0.11675E+01_JPRB, 0.11542E+01_JPRB, 0.11305E+01_JPRB, 0.10847E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.99150E+00_JPRB, 0.94831E+00_JPRB, 0.92167E+00_JPRB, 0.91017E+00_JPRB, 0.90119E+00_JPRB, 0.88670E+00_JPRB, 0.87152E+00_JPRB ,&
&  0.87097E+00_JPRB, 0.88377E+00_JPRB, 0.89603E+00_JPRB, 0.90073E+00_JPRB, 0.90315E+00_JPRB, 0.90082E+00_JPRB, 0.90104E+00_JPRB ,&
&  0.90072E+00_JPRB, 0.90087E+00_JPRB, 0.90326E+00_JPRB, 0.89568E+00_JPRB, 0.89078E+00_JPRB, 0.88951E+00_JPRB, 0.88833E+00_JPRB ,&
&  0.89224E+00_JPRB, 0.89135E+00_JPRB, 0.89916E+00_JPRB, 0.89734E+00_JPRB, 0.89465E+00_JPRB, 0.89770E+00_JPRB, 0.85074E+00_JPRB ,&
&  0.84028E+00_JPRB, 0.87101E+00_JPRB, 0.85951E+00_JPRB, 0.87076E+00_JPRB, 0.89186E+00_JPRB, 0.90458E+00_JPRB, 0.91408E+00_JPRB ,&
&  0.91975E+00_JPRB, 0.92711E+00_JPRB, 0.93797E+00_JPRB, 0.95358E+00_JPRB, 0.97422E+00_JPRB, 0.99488E+00_JPRB, 0.10135E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10353E+01_JPRB, 0.10495E+01_JPRB, 0.11336E+01_JPRB, 0.13667E+01_JPRB, 0.14671E+01_JPRB, 0.15392E+01_JPRB ,&
&  0.16200E+01_JPRB, 0.16658E+01_JPRB, 0.16794E+01_JPRB, 0.17005E+01_JPRB, 0.16852E+01_JPRB, 0.16672E+01_JPRB, 0.16783E+01_JPRB ,&
&  0.16630E+01_JPRB, 0.16767E+01_JPRB, 0.16774E+01_JPRB, 0.16964E+01_JPRB, 0.16837E+01_JPRB, 0.16881E+01_JPRB, 0.16910E+01_JPRB ,&
&  0.16720E+01_JPRB, 0.16268E+01_JPRB, 0.15426E+01_JPRB, 0.14574E+01_JPRB, 0.13801E+01_JPRB, 0.13361E+01_JPRB, 0.12998E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12331E+01_JPRB, 0.12009E+01_JPRB, 0.11574E+01_JPRB, 0.10987E+01_JPRB, 0.10508E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.95450E+00_JPRB, 0.93393E+00_JPRB, 0.94160E+00_JPRB, 0.95209E+00_JPRB, 0.96812E+00_JPRB, 0.99554E+00_JPRB, 0.99490E+00_JPRB ,&
&  0.10232E+01_JPRB, 0.10441E+01_JPRB, 0.10575E+01_JPRB, 0.10866E+01_JPRB, 0.11292E+01_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.12793E+01_JPRB, 0.12765E+01_JPRB, 0.12555E+01_JPRB, 0.12342E+01_JPRB, 0.12009E+01_JPRB, 0.11324E+01_JPRB, 0.10780E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.10467E+01_JPRB, 0.10239E+01_JPRB, 0.96840E+00_JPRB, 0.93966E+00_JPRB, 0.91714E+00_JPRB, 0.89583E+00_JPRB ,&
&  0.89005E+00_JPRB, 0.89809E+00_JPRB, 0.90764E+00_JPRB, 0.91784E+00_JPRB, 0.92078E+00_JPRB, 0.91923E+00_JPRB, 0.91621E+00_JPRB ,&
&  0.91501E+00_JPRB, 0.92006E+00_JPRB, 0.92384E+00_JPRB, 0.92711E+00_JPRB, 0.92893E+00_JPRB, 0.92819E+00_JPRB, 0.92068E+00_JPRB ,&
&  0.91485E+00_JPRB, 0.91424E+00_JPRB, 0.91994E+00_JPRB, 0.91604E+00_JPRB, 0.91603E+00_JPRB, 0.91322E+00_JPRB, 0.90670E+00_JPRB ,&
&  0.88154E+00_JPRB, 0.88296E+00_JPRB, 0.89114E+00_JPRB, 0.90444E+00_JPRB, 0.91928E+00_JPRB, 0.93104E+00_JPRB, 0.94140E+00_JPRB ,&
&  0.95439E+00_JPRB, 0.96995E+00_JPRB, 0.98843E+00_JPRB, 0.10080E+01_JPRB, 0.10308E+01_JPRB, 0.10633E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11377E+01_JPRB, 0.12954E+01_JPRB, 0.14328E+01_JPRB, 0.16463E+01_JPRB, 0.18186E+01_JPRB, 0.18699E+01_JPRB ,&
&  0.20186E+01_JPRB, 0.20543E+01_JPRB, 0.20768E+01_JPRB, 0.20535E+01_JPRB, 0.20213E+01_JPRB, 0.20103E+01_JPRB, 0.20302E+01_JPRB ,&
&  0.20630E+01_JPRB, 0.20336E+01_JPRB, 0.21497E+01_JPRB, 0.21319E+01_JPRB, 0.21031E+01_JPRB, 0.21007E+01_JPRB, 0.20766E+01_JPRB ,&
&  0.20240E+01_JPRB, 0.19366E+01_JPRB, 0.18195E+01_JPRB, 0.16323E+01_JPRB, 0.15525E+01_JPRB, 0.14829E+01_JPRB, 0.13763E+01_JPRB ,&
&  0.13163E+01_JPRB, 0.12515E+01_JPRB, 0.11938E+01_JPRB, 0.11819E+01_JPRB, 0.11080E+01_JPRB, 0.10534E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.97810E+00_JPRB, 0.97789E+00_JPRB, 0.10117E+01_JPRB, 0.10511E+01_JPRB, 0.10785E+01_JPRB, 0.10865E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.11400E+01_JPRB, 0.12072E+01_JPRB, 0.12368E+01_JPRB, 0.12605E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.14251E+01_JPRB, 0.14095E+01_JPRB, 0.13803E+01_JPRB, 0.13437E+01_JPRB, 0.12755E+01_JPRB, 0.11909E+01_JPRB, 0.11282E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.11126E+01_JPRB, 0.10926E+01_JPRB, 0.10606E+01_JPRB, 0.10232E+01_JPRB, 0.97378E+00_JPRB, 0.91856E+00_JPRB ,&
&  0.90594E+00_JPRB, 0.92468E+00_JPRB, 0.94432E+00_JPRB, 0.95779E+00_JPRB, 0.95459E+00_JPRB, 0.95126E+00_JPRB, 0.95328E+00_JPRB ,&
&  0.95710E+00_JPRB, 0.96269E+00_JPRB, 0.97082E+00_JPRB, 0.97679E+00_JPRB, 0.97575E+00_JPRB, 0.97540E+00_JPRB, 0.97638E+00_JPRB ,&
&  0.95264E+00_JPRB, 0.93761E+00_JPRB, 0.93222E+00_JPRB, 0.92919E+00_JPRB, 0.95552E+00_JPRB, 0.96970E+00_JPRB, 0.97761E+00_JPRB ,&
&  0.97503E+00_JPRB, 0.96831E+00_JPRB, 0.96678E+00_JPRB, 0.97780E+00_JPRB, 0.98881E+00_JPRB, 0.99384E+00_JPRB, 0.10037E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10409E+01_JPRB, 0.10662E+01_JPRB, 0.11020E+01_JPRB, 0.11411E+01_JPRB, 0.11690E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.13542E+01_JPRB, 0.15324E+01_JPRB, 0.17369E+01_JPRB, 0.17866E+01_JPRB, 0.19866E+01_JPRB, 0.21487E+01_JPRB, 0.21914E+01_JPRB ,&
&  0.23197E+01_JPRB, 0.23537E+01_JPRB, 0.24723E+01_JPRB, 0.25040E+01_JPRB, 0.25380E+01_JPRB, 0.25635E+01_JPRB, 0.25703E+01_JPRB ,&
&  0.25838E+01_JPRB, 0.26294E+01_JPRB, 0.26866E+01_JPRB, 0.26834E+01_JPRB, 0.26123E+01_JPRB, 0.25167E+01_JPRB, 0.23923E+01_JPRB ,&
&  0.22757E+01_JPRB, 0.21454E+01_JPRB, 0.20148E+01_JPRB, 0.19094E+01_JPRB, 0.17817E+01_JPRB, 0.16376E+01_JPRB, 0.14951E+01_JPRB ,&
&  0.13583E+01_JPRB, 0.12034E+01_JPRB, 0.11355E+01_JPRB, 0.11293E+01_JPRB, 0.11094E+01_JPRB, 0.10429E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.10107E+01_JPRB, 0.10579E+01_JPRB, 0.11238E+01_JPRB, 0.11694E+01_JPRB, 0.12460E+01_JPRB, 0.12986E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.12826E+01_JPRB, 0.13488E+01_JPRB, 0.13900E+01_JPRB, 0.14180E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.15565E+01_JPRB, 0.15294E+01_JPRB, 0.15002E+01_JPRB, 0.14642E+01_JPRB, 0.13920E+01_JPRB, 0.12934E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.11613E+01_JPRB, 0.11313E+01_JPRB, 0.11026E+01_JPRB, 0.10742E+01_JPRB, 0.10403E+01_JPRB, 0.10066E+01_JPRB ,&
&  0.98624E+00_JPRB, 0.10047E+01_JPRB, 0.99680E+00_JPRB, 0.10004E+01_JPRB, 0.10114E+01_JPRB, 0.10015E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.10272E+01_JPRB, 0.10259E+01_JPRB, 0.10202E+01_JPRB, 0.10174E+01_JPRB, 0.10238E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.99927E+00_JPRB, 0.97221E+00_JPRB, 0.95019E+00_JPRB, 0.94638E+00_JPRB, 0.97715E+00_JPRB, 0.10220E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10803E+01_JPRB, 0.10818E+01_JPRB, 0.10833E+01_JPRB, 0.10801E+01_JPRB, 0.10756E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10977E+01_JPRB, 0.11219E+01_JPRB, 0.11668E+01_JPRB, 0.12080E+01_JPRB, 0.12829E+01_JPRB, 0.14387E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.18707E+01_JPRB, 0.20461E+01_JPRB, 0.22886E+01_JPRB, 0.24820E+01_JPRB, 0.25576E+01_JPRB, 0.25870E+01_JPRB ,&
&  0.26887E+01_JPRB, 0.26727E+01_JPRB, 0.28648E+01_JPRB, 0.28583E+01_JPRB, 0.28837E+01_JPRB, 0.30038E+01_JPRB, 0.30018E+01_JPRB ,&
&  0.29621E+01_JPRB, 0.28954E+01_JPRB, 0.29989E+01_JPRB, 0.29893E+01_JPRB, 0.29266E+01_JPRB, 0.28350E+01_JPRB, 0.26780E+01_JPRB ,&
&  0.25221E+01_JPRB, 0.23565E+01_JPRB, 0.20486E+01_JPRB, 0.18205E+01_JPRB, 0.15759E+01_JPRB, 0.15540E+01_JPRB, 0.14319E+01_JPRB ,&
&  0.13041E+01_JPRB, 0.11464E+01_JPRB, 0.11340E+01_JPRB, 0.11328E+01_JPRB, 0.11528E+01_JPRB, 0.11255E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.11734E+01_JPRB, 0.12173E+01_JPRB, 0.12866E+01_JPRB, 0.14037E+01_JPRB, 0.15118E+01_JPRB, 0.13243E+01_JPRB ,&
&  0.13360E+01_JPRB, 0.14733E+01_JPRB, 0.15342E+01_JPRB, 0.15648E+01_JPRB, 0.15709E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.15971E+01_JPRB, 0.15537E+01_JPRB, 0.15135E+01_JPRB, 0.14783E+01_JPRB, 0.14321E+01_JPRB, 0.13749E+01_JPRB, 0.13016E+01_JPRB ,&
&  0.12363E+01_JPRB, 0.11850E+01_JPRB, 0.11549E+01_JPRB, 0.11399E+01_JPRB, 0.11247E+01_JPRB, 0.10927E+01_JPRB, 0.10969E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.10739E+01_JPRB, 0.10222E+01_JPRB, 0.10385E+01_JPRB, 0.11116E+01_JPRB, 0.10661E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10830E+01_JPRB, 0.10717E+01_JPRB, 0.10572E+01_JPRB, 0.10516E+01_JPRB, 0.10593E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.10040E+01_JPRB, 0.96406E+00_JPRB, 0.95669E+00_JPRB, 0.99456E+00_JPRB, 0.10453E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11386E+01_JPRB, 0.11505E+01_JPRB, 0.11557E+01_JPRB, 0.11530E+01_JPRB, 0.11514E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.11742E+01_JPRB, 0.12055E+01_JPRB, 0.12532E+01_JPRB, 0.12749E+01_JPRB, 0.14060E+01_JPRB, 0.17386E+01_JPRB ,&
&  0.22109E+01_JPRB, 0.23568E+01_JPRB, 0.25440E+01_JPRB, 0.30145E+01_JPRB, 0.32281E+01_JPRB, 0.32867E+01_JPRB, 0.32959E+01_JPRB ,&
&  0.33178E+01_JPRB, 0.32866E+01_JPRB, 0.32868E+01_JPRB, 0.34083E+01_JPRB, 0.33879E+01_JPRB, 0.33933E+01_JPRB, 0.32988E+01_JPRB ,&
&  0.31443E+01_JPRB, 0.31200E+01_JPRB, 0.31381E+01_JPRB, 0.30523E+01_JPRB, 0.29176E+01_JPRB, 0.28728E+01_JPRB, 0.27960E+01_JPRB ,&
&  0.26483E+01_JPRB, 0.23592E+01_JPRB, 0.18109E+01_JPRB, 0.16296E+01_JPRB, 0.13025E+01_JPRB, 0.13031E+01_JPRB, 0.14814E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.13333E+01_JPRB, 0.12738E+01_JPRB, 0.12576E+01_JPRB, 0.12725E+01_JPRB, 0.13319E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.12513E+01_JPRB, 0.12974E+01_JPRB, 0.13784E+01_JPRB, 0.15115E+01_JPRB, 0.16810E+01_JPRB, 0.17581E+01_JPRB, 0.15936E+01_JPRB ,&
&  0.16681E+01_JPRB, 0.17030E+01_JPRB, 0.17094E+01_JPRB, 0.16987E+01_JPRB, 0.16551E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.15322E+01_JPRB, 0.14896E+01_JPRB, 0.14436E+01_JPRB, 0.14291E+01_JPRB, 0.14325E+01_JPRB, 0.14167E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12130E+01_JPRB, 0.11778E+01_JPRB, 0.11738E+01_JPRB, 0.11759E+01_JPRB, 0.11501E+01_JPRB, 0.11447E+01_JPRB ,&
&  0.11875E+01_JPRB, 0.11984E+01_JPRB, 0.11037E+01_JPRB, 0.10398E+01_JPRB, 0.10894E+01_JPRB, 0.10759E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.10764E+01_JPRB, 0.11223E+01_JPRB, 0.11184E+01_JPRB, 0.11228E+01_JPRB, 0.11343E+01_JPRB, 0.11695E+01_JPRB, 0.12465E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.11509E+01_JPRB, 0.10748E+01_JPRB, 0.10682E+01_JPRB, 0.11234E+01_JPRB, 0.11019E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11895E+01_JPRB, 0.12099E+01_JPRB, 0.12304E+01_JPRB, 0.12449E+01_JPRB, 0.12547E+01_JPRB, 0.12674E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.13063E+01_JPRB, 0.13066E+01_JPRB, 0.13403E+01_JPRB, 0.14529E+01_JPRB, 0.17522E+01_JPRB, 0.21786E+01_JPRB ,&
&  0.24930E+01_JPRB, 0.25806E+01_JPRB, 0.26932E+01_JPRB, 0.31321E+01_JPRB, 0.34797E+01_JPRB, 0.36496E+01_JPRB, 0.38423E+01_JPRB ,&
&  0.38925E+01_JPRB, 0.38906E+01_JPRB, 0.38598E+01_JPRB, 0.36095E+01_JPRB, 0.32353E+01_JPRB, 0.29821E+01_JPRB, 0.28742E+01_JPRB ,&
&  0.27709E+01_JPRB, 0.27758E+01_JPRB, 0.28745E+01_JPRB, 0.28183E+01_JPRB, 0.27917E+01_JPRB, 0.26689E+01_JPRB, 0.27858E+01_JPRB ,&
&  0.27193E+01_JPRB, 0.17501E+01_JPRB, 0.14086E+01_JPRB, 0.14904E+01_JPRB, 0.12712E+01_JPRB, 0.12834E+01_JPRB, 0.14147E+01_JPRB ,&
&  0.13865E+01_JPRB, 0.13814E+01_JPRB, 0.14401E+01_JPRB, 0.14958E+01_JPRB, 0.15227E+01_JPRB, 0.16541E+01_JPRB, 0.17089E+01_JPRB ,&
&  0.16654E+01_JPRB, 0.15851E+01_JPRB, 0.17029E+01_JPRB, 0.18838E+01_JPRB, 0.20508E+01_JPRB, 0.20517E+01_JPRB, 0.19671E+01_JPRB ,&
&  0.19150E+01_JPRB, 0.18737E+01_JPRB, 0.18187E+01_JPRB, 0.17172E+01_JPRB, 0.16036E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.15406E+01_JPRB, 0.15052E+01_JPRB, 0.14517E+01_JPRB, 0.14538E+01_JPRB, 0.14812E+01_JPRB, 0.14700E+01_JPRB, 0.14019E+01_JPRB ,&
&  0.12864E+01_JPRB, 0.12204E+01_JPRB, 0.11908E+01_JPRB, 0.11878E+01_JPRB, 0.12015E+01_JPRB, 0.12034E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.11581E+01_JPRB, 0.12160E+01_JPRB, 0.11262E+01_JPRB, 0.10782E+01_JPRB, 0.10119E+01_JPRB, 0.10459E+01_JPRB, 0.10920E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11602E+01_JPRB, 0.12349E+01_JPRB, 0.13359E+01_JPRB, 0.14302E+01_JPRB, 0.15153E+01_JPRB, 0.16741E+01_JPRB ,&
&  0.16346E+01_JPRB, 0.14773E+01_JPRB, 0.12941E+01_JPRB, 0.12761E+01_JPRB, 0.12791E+01_JPRB, 0.12083E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.13165E+01_JPRB, 0.13436E+01_JPRB, 0.13583E+01_JPRB, 0.13593E+01_JPRB, 0.13314E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.13285E+01_JPRB, 0.13526E+01_JPRB, 0.13874E+01_JPRB, 0.14395E+01_JPRB, 0.17446E+01_JPRB, 0.21647E+01_JPRB, 0.24944E+01_JPRB ,&
&  0.27654E+01_JPRB, 0.24328E+01_JPRB, 0.22274E+01_JPRB, 0.28917E+01_JPRB, 0.40787E+01_JPRB, 0.44597E+01_JPRB, 0.42391E+01_JPRB ,&
&  0.45315E+01_JPRB, 0.41982E+01_JPRB, 0.36791E+01_JPRB, 0.33005E+01_JPRB, 0.30383E+01_JPRB, 0.28316E+01_JPRB, 0.27399E+01_JPRB ,&
&  0.25758E+01_JPRB, 0.25618E+01_JPRB, 0.26848E+01_JPRB, 0.27632E+01_JPRB, 0.27235E+01_JPRB, 0.26768E+01_JPRB, 0.27388E+01_JPRB ,&
&  0.25929E+01_JPRB, 0.21691E+01_JPRB, 0.15972E+01_JPRB, 0.15070E+01_JPRB, 0.12641E+01_JPRB, 0.12518E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.15056E+01_JPRB, 0.15817E+01_JPRB, 0.16077E+01_JPRB, 0.18335E+01_JPRB, 0.19964E+01_JPRB, 0.22163E+01_JPRB ,&
&  0.23384E+01_JPRB, 0.20629E+01_JPRB, 0.22323E+01_JPRB, 0.24240E+01_JPRB, 0.24695E+01_JPRB, 0.23147E+01_JPRB, 0.21373E+01_JPRB ,&
&  0.20283E+01_JPRB, 0.19468E+01_JPRB, 0.18408E+01_JPRB, 0.16942E+01_JPRB, 0.15919E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.15764E+01_JPRB, 0.15693E+01_JPRB, 0.15153E+01_JPRB, 0.15140E+01_JPRB, 0.15098E+01_JPRB, 0.14649E+01_JPRB, 0.13987E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.12923E+01_JPRB, 0.12632E+01_JPRB, 0.12470E+01_JPRB, 0.12476E+01_JPRB, 0.12405E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.11751E+01_JPRB, 0.10789E+01_JPRB, 0.10183E+01_JPRB, 0.98388E+00_JPRB, 0.10095E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.13280E+01_JPRB, 0.15751E+01_JPRB, 0.17942E+01_JPRB, 0.20866E+01_JPRB, 0.23147E+01_JPRB, 0.24482E+01_JPRB ,&
&  0.24317E+01_JPRB, 0.18867E+01_JPRB, 0.17375E+01_JPRB, 0.15831E+01_JPRB, 0.15270E+01_JPRB, 0.14539E+01_JPRB, 0.14254E+01_JPRB ,&
&  0.14059E+01_JPRB, 0.14135E+01_JPRB, 0.14146E+01_JPRB, 0.13901E+01_JPRB, 0.13684E+01_JPRB, 0.13596E+01_JPRB, 0.13971E+01_JPRB ,&
&  0.14402E+01_JPRB, 0.14470E+01_JPRB, 0.15003E+01_JPRB, 0.17470E+01_JPRB, 0.22568E+01_JPRB, 0.27134E+01_JPRB, 0.29918E+01_JPRB ,&
&  0.28301E+01_JPRB, 0.26937E+01_JPRB, 0.37633E+01_JPRB, 0.41789E+01_JPRB, 0.40302E+01_JPRB, 0.46502E+01_JPRB, 0.48168E+01_JPRB ,&
&  0.44360E+01_JPRB, 0.36175E+01_JPRB, 0.31756E+01_JPRB, 0.28692E+01_JPRB, 0.32757E+01_JPRB, 0.30660E+01_JPRB, 0.27586E+01_JPRB ,&
&  0.24998E+01_JPRB, 0.26349E+01_JPRB, 0.26698E+01_JPRB, 0.27599E+01_JPRB, 0.28913E+01_JPRB, 0.29773E+01_JPRB, 0.27105E+01_JPRB ,&
&  0.24278E+01_JPRB, 0.26553E+01_JPRB, 0.21632E+01_JPRB, 0.16993E+01_JPRB, 0.14731E+01_JPRB, 0.14788E+01_JPRB, 0.15629E+01_JPRB ,&
&  0.16486E+01_JPRB, 0.17347E+01_JPRB, 0.17843E+01_JPRB, 0.19399E+01_JPRB, 0.23164E+01_JPRB, 0.25464E+01_JPRB, 0.26682E+01_JPRB ,&
&  0.26457E+01_JPRB, 0.29069E+01_JPRB, 0.29415E+01_JPRB, 0.28868E+01_JPRB, 0.26212E+01_JPRB, 0.22878E+01_JPRB, 0.20924E+01_JPRB ,&
&  0.20060E+01_JPRB, 0.19330E+01_JPRB, 0.17918E+01_JPRB, 0.16495E+01_JPRB, 0.15709E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.16002E+01_JPRB, 0.16108E+01_JPRB, 0.15969E+01_JPRB, 0.15706E+01_JPRB, 0.15322E+01_JPRB, 0.14830E+01_JPRB, 0.14446E+01_JPRB ,&
&  0.13969E+01_JPRB, 0.13389E+01_JPRB, 0.13271E+01_JPRB, 0.13277E+01_JPRB, 0.13225E+01_JPRB, 0.12986E+01_JPRB, 0.12598E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.11607E+01_JPRB, 0.10493E+01_JPRB, 0.10505E+01_JPRB, 0.10541E+01_JPRB, 0.10408E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.16345E+01_JPRB, 0.20419E+01_JPRB, 0.24751E+01_JPRB, 0.27542E+01_JPRB, 0.28353E+01_JPRB, 0.28336E+01_JPRB ,&
&  0.25631E+01_JPRB, 0.23284E+01_JPRB, 0.21297E+01_JPRB, 0.19465E+01_JPRB, 0.18187E+01_JPRB, 0.17291E+01_JPRB, 0.16325E+01_JPRB ,&
&  0.15472E+01_JPRB, 0.14930E+01_JPRB, 0.14637E+01_JPRB, 0.14160E+01_JPRB, 0.14083E+01_JPRB, 0.14347E+01_JPRB, 0.14829E+01_JPRB ,&
&  0.14908E+01_JPRB, 0.15161E+01_JPRB, 0.16805E+01_JPRB, 0.20848E+01_JPRB, 0.22895E+01_JPRB, 0.26406E+01_JPRB, 0.29570E+01_JPRB ,&
&  0.38845E+01_JPRB, 0.41748E+01_JPRB, 0.39488E+01_JPRB, 0.48079E+01_JPRB, 0.33235E+01_JPRB, 0.38726E+01_JPRB, 0.44947E+01_JPRB ,&
&  0.41558E+01_JPRB, 0.38617E+01_JPRB, 0.39633E+01_JPRB, 0.34628E+01_JPRB, 0.29695E+01_JPRB, 0.39021E+01_JPRB, 0.32730E+01_JPRB ,&
&  0.28376E+01_JPRB, 0.28436E+01_JPRB, 0.29546E+01_JPRB, 0.32837E+01_JPRB, 0.28742E+01_JPRB, 0.26124E+01_JPRB, 0.21669E+01_JPRB ,&
&  0.22029E+01_JPRB, 0.20441E+01_JPRB, 0.22225E+01_JPRB, 0.19785E+01_JPRB, 0.19976E+01_JPRB, 0.18890E+01_JPRB, 0.18360E+01_JPRB ,&
&  0.19628E+01_JPRB, 0.20109E+01_JPRB, 0.20464E+01_JPRB, 0.24425E+01_JPRB, 0.31492E+01_JPRB, 0.29078E+01_JPRB, 0.31681E+01_JPRB ,&
&  0.33425E+01_JPRB, 0.33637E+01_JPRB, 0.30787E+01_JPRB, 0.28340E+01_JPRB, 0.24138E+01_JPRB, 0.21576E+01_JPRB, 0.20502E+01_JPRB ,&
&  0.19719E+01_JPRB, 0.18324E+01_JPRB, 0.16762E+01_JPRB, 0.16120E+01_JPRB, 0.15604E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.15982E+01_JPRB, 0.15813E+01_JPRB, 0.15709E+01_JPRB, 0.15412E+01_JPRB, 0.15296E+01_JPRB, 0.14932E+01_JPRB, 0.14609E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.13786E+01_JPRB, 0.13715E+01_JPRB, 0.13655E+01_JPRB, 0.13555E+01_JPRB, 0.13476E+01_JPRB, 0.13134E+01_JPRB ,&
&  0.13132E+01_JPRB, 0.12573E+01_JPRB, 0.11055E+01_JPRB, 0.10642E+01_JPRB, 0.10982E+01_JPRB, 0.11074E+01_JPRB, 0.11907E+01_JPRB ,&
&  0.16109E+01_JPRB, 0.22153E+01_JPRB, 0.26162E+01_JPRB, 0.29269E+01_JPRB, 0.30515E+01_JPRB, 0.29071E+01_JPRB, 0.27325E+01_JPRB ,&
&  0.26585E+01_JPRB, 0.22894E+01_JPRB, 0.20740E+01_JPRB, 0.19730E+01_JPRB, 0.18546E+01_JPRB, 0.17465E+01_JPRB, 0.16572E+01_JPRB ,&
&  0.15623E+01_JPRB, 0.14743E+01_JPRB, 0.14181E+01_JPRB, 0.13647E+01_JPRB, 0.13720E+01_JPRB, 0.14131E+01_JPRB, 0.14724E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.16325E+01_JPRB, 0.19437E+01_JPRB, 0.23461E+01_JPRB, 0.26561E+01_JPRB, 0.29810E+01_JPRB, 0.37334E+01_JPRB ,&
&  0.41305E+01_JPRB, 0.40479E+01_JPRB, 0.42722E+01_JPRB, 0.46294E+01_JPRB, 0.44956E+01_JPRB, 0.39825E+01_JPRB, 0.42992E+01_JPRB ,&
&  0.33390E+01_JPRB, 0.29534E+01_JPRB, 0.28245E+01_JPRB, 0.28005E+01_JPRB, 0.34419E+01_JPRB, 0.50650E+01_JPRB, 0.45329E+01_JPRB ,&
&  0.40902E+01_JPRB, 0.36117E+01_JPRB, 0.36715E+01_JPRB, 0.34514E+01_JPRB, 0.30968E+01_JPRB, 0.24008E+01_JPRB, 0.29598E+01_JPRB ,&
&  0.31421E+01_JPRB, 0.33015E+01_JPRB, 0.30096E+01_JPRB, 0.24814E+01_JPRB, 0.21414E+01_JPRB, 0.23740E+01_JPRB, 0.24356E+01_JPRB ,&
&  0.24922E+01_JPRB, 0.24596E+01_JPRB, 0.31278E+01_JPRB, 0.35908E+01_JPRB, 0.38149E+01_JPRB, 0.36977E+01_JPRB, 0.39932E+01_JPRB ,&
&  0.36772E+01_JPRB, 0.35321E+01_JPRB, 0.31455E+01_JPRB, 0.26147E+01_JPRB, 0.22610E+01_JPRB, 0.21014E+01_JPRB, 0.19869E+01_JPRB ,&
&  0.18607E+01_JPRB, 0.16897E+01_JPRB, 0.15975E+01_JPRB, 0.15452E+01_JPRB, 0.15358E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.15623E+01_JPRB, 0.15348E+01_JPRB, 0.14923E+01_JPRB, 0.14736E+01_JPRB, 0.14592E+01_JPRB, 0.14240E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.14429E+01_JPRB, 0.14072E+01_JPRB, 0.13628E+01_JPRB, 0.13329E+01_JPRB, 0.13221E+01_JPRB, 0.13702E+01_JPRB, 0.14157E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.14297E+01_JPRB, 0.12232E+01_JPRB, 0.11737E+01_JPRB, 0.11982E+01_JPRB, 0.12709E+01_JPRB, 0.13930E+01_JPRB ,&
&  0.20799E+01_JPRB, 0.27323E+01_JPRB, 0.28767E+01_JPRB, 0.29681E+01_JPRB, 0.29886E+01_JPRB, 0.28070E+01_JPRB, 0.26848E+01_JPRB ,&
&  0.25901E+01_JPRB, 0.23092E+01_JPRB, 0.20689E+01_JPRB, 0.19177E+01_JPRB, 0.17874E+01_JPRB, 0.16953E+01_JPRB, 0.16134E+01_JPRB ,&
&  0.15135E+01_JPRB, 0.14351E+01_JPRB, 0.13788E+01_JPRB, 0.13547E+01_JPRB, 0.13400E+01_JPRB, 0.14282E+01_JPRB, 0.15434E+01_JPRB ,&
&  0.16637E+01_JPRB, 0.19090E+01_JPRB, 0.22092E+01_JPRB, 0.26159E+01_JPRB, 0.30534E+01_JPRB, 0.34241E+01_JPRB, 0.46314E+01_JPRB ,&
&  0.45190E+01_JPRB, 0.45350E+01_JPRB, 0.45727E+01_JPRB, 0.54342E+01_JPRB, 0.43300E+01_JPRB, 0.38087E+01_JPRB, 0.39722E+01_JPRB ,&
&  0.30575E+01_JPRB, 0.28107E+01_JPRB, 0.32057E+01_JPRB, 0.39307E+01_JPRB, 0.39905E+01_JPRB, 0.44060E+01_JPRB, 0.58065E+01_JPRB ,&
&  0.50810E+01_JPRB, 0.45033E+01_JPRB, 0.43166E+01_JPRB, 0.43697E+01_JPRB, 0.23881E+01_JPRB, 0.20189E+01_JPRB, 0.31044E+01_JPRB ,&
&  0.33035E+01_JPRB, 0.22776E+01_JPRB, 0.17708E+01_JPRB, 0.21259E+01_JPRB, 0.16937E+01_JPRB, 0.20200E+01_JPRB, 0.28195E+01_JPRB ,&
&  0.30250E+01_JPRB, 0.32684E+01_JPRB, 0.48785E+01_JPRB, 0.47212E+01_JPRB, 0.45139E+01_JPRB, 0.55964E+01_JPRB, 0.47037E+01_JPRB ,&
&  0.41055E+01_JPRB, 0.31806E+01_JPRB, 0.28396E+01_JPRB, 0.23871E+01_JPRB, 0.21292E+01_JPRB, 0.19449E+01_JPRB, 0.18095E+01_JPRB ,&
&  0.16666E+01_JPRB, 0.15806E+01_JPRB, 0.15719E+01_JPRB, 0.15653E+01_JPRB, 0.15457E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.15362E+01_JPRB, 0.14768E+01_JPRB, 0.14113E+01_JPRB, 0.14016E+01_JPRB, 0.13765E+01_JPRB, 0.13474E+01_JPRB, 0.14384E+01_JPRB ,&
&  0.15053E+01_JPRB, 0.14006E+01_JPRB, 0.13090E+01_JPRB, 0.12885E+01_JPRB, 0.13168E+01_JPRB, 0.14644E+01_JPRB, 0.15548E+01_JPRB ,&
&  0.16067E+01_JPRB, 0.15730E+01_JPRB, 0.15070E+01_JPRB, 0.14926E+01_JPRB, 0.15494E+01_JPRB, 0.16629E+01_JPRB, 0.19712E+01_JPRB ,&
&  0.28088E+01_JPRB, 0.30416E+01_JPRB, 0.28400E+01_JPRB, 0.27160E+01_JPRB, 0.26936E+01_JPRB, 0.25897E+01_JPRB, 0.26320E+01_JPRB ,&
&  0.24734E+01_JPRB, 0.22012E+01_JPRB, 0.19949E+01_JPRB, 0.18520E+01_JPRB, 0.17847E+01_JPRB, 0.17154E+01_JPRB, 0.16322E+01_JPRB ,&
&  0.15113E+01_JPRB, 0.14184E+01_JPRB, 0.13855E+01_JPRB, 0.13515E+01_JPRB, 0.13883E+01_JPRB, 0.15060E+01_JPRB, 0.16586E+01_JPRB ,&
&  0.18718E+01_JPRB, 0.21722E+01_JPRB, 0.24654E+01_JPRB, 0.29496E+01_JPRB, 0.36669E+01_JPRB, 0.40015E+01_JPRB, 0.37701E+01_JPRB ,&
&  0.35347E+01_JPRB, 0.35165E+01_JPRB, 0.45676E+01_JPRB, 0.48441E+01_JPRB, 0.43740E+01_JPRB, 0.41164E+01_JPRB, 0.38951E+01_JPRB ,&
&  0.34768E+01_JPRB, 0.33464E+01_JPRB, 0.35928E+01_JPRB, 0.44803E+01_JPRB, 0.49073E+01_JPRB, 0.45981E+01_JPRB, 0.57549E+01_JPRB ,&
&  0.58452E+01_JPRB, 0.51765E+01_JPRB, 0.37770E+01_JPRB, 0.29464E+01_JPRB, 0.27446E+01_JPRB, 0.25260E+01_JPRB, 0.18563E+01_JPRB ,&
&  0.16940E+01_JPRB, 0.16822E+01_JPRB, 0.15886E+01_JPRB, 0.16186E+01_JPRB, 0.15925E+01_JPRB, 0.21345E+01_JPRB, 0.28837E+01_JPRB ,&
&  0.40089E+01_JPRB, 0.48040E+01_JPRB, 0.55521E+01_JPRB, 0.54517E+01_JPRB, 0.50911E+01_JPRB, 0.65704E+01_JPRB, 0.50479E+01_JPRB ,&
&  0.34088E+01_JPRB, 0.27672E+01_JPRB, 0.24694E+01_JPRB, 0.21812E+01_JPRB, 0.19667E+01_JPRB, 0.17726E+01_JPRB, 0.16223E+01_JPRB ,&
&  0.15465E+01_JPRB, 0.15468E+01_JPRB, 0.15615E+01_JPRB, 0.15611E+01_JPRB, 0.15164E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.15103E+01_JPRB, 0.14542E+01_JPRB, 0.13944E+01_JPRB, 0.13479E+01_JPRB, 0.12942E+01_JPRB, 0.13260E+01_JPRB, 0.14876E+01_JPRB ,&
&  0.15608E+01_JPRB, 0.13939E+01_JPRB, 0.12854E+01_JPRB, 0.12959E+01_JPRB, 0.14330E+01_JPRB, 0.15939E+01_JPRB, 0.16534E+01_JPRB ,&
&  0.17369E+01_JPRB, 0.17330E+01_JPRB, 0.18454E+01_JPRB, 0.19975E+01_JPRB, 0.22931E+01_JPRB, 0.24191E+01_JPRB, 0.30127E+01_JPRB ,&
&  0.34996E+01_JPRB, 0.30828E+01_JPRB, 0.26847E+01_JPRB, 0.25464E+01_JPRB, 0.25608E+01_JPRB, 0.25481E+01_JPRB, 0.25141E+01_JPRB ,&
&  0.22770E+01_JPRB, 0.20666E+01_JPRB, 0.18982E+01_JPRB, 0.18038E+01_JPRB, 0.17328E+01_JPRB, 0.16118E+01_JPRB, 0.15274E+01_JPRB ,&
&  0.14421E+01_JPRB, 0.14105E+01_JPRB, 0.13863E+01_JPRB, 0.14132E+01_JPRB, 0.15300E+01_JPRB, 0.16905E+01_JPRB, 0.18875E+01_JPRB ,&
&  0.21303E+01_JPRB, 0.24654E+01_JPRB, 0.28146E+01_JPRB, 0.33689E+01_JPRB, 0.33146E+01_JPRB, 0.29244E+01_JPRB, 0.33334E+01_JPRB ,&
&  0.36742E+01_JPRB, 0.39943E+01_JPRB, 0.39561E+01_JPRB, 0.48178E+01_JPRB, 0.45689E+01_JPRB, 0.39610E+01_JPRB, 0.37365E+01_JPRB ,&
&  0.33868E+01_JPRB, 0.38900E+01_JPRB, 0.42897E+01_JPRB, 0.48673E+01_JPRB, 0.62744E+01_JPRB, 0.59347E+01_JPRB, 0.63252E+01_JPRB ,&
&  0.66055E+01_JPRB, 0.56053E+01_JPRB, 0.42908E+01_JPRB, 0.33787E+01_JPRB, 0.42188E+01_JPRB, 0.51209E+01_JPRB, 0.22918E+01_JPRB ,&
&  0.18843E+01_JPRB, 0.17538E+01_JPRB, 0.17238E+01_JPRB, 0.16210E+01_JPRB, 0.16467E+01_JPRB, 0.22334E+01_JPRB, 0.41502E+01_JPRB ,&
&  0.42624E+01_JPRB, 0.63302E+01_JPRB, 0.63011E+01_JPRB, 0.61043E+01_JPRB, 0.54479E+01_JPRB, 0.63571E+01_JPRB, 0.42982E+01_JPRB ,&
&  0.34052E+01_JPRB, 0.28481E+01_JPRB, 0.24201E+01_JPRB, 0.21662E+01_JPRB, 0.19065E+01_JPRB, 0.16754E+01_JPRB, 0.15772E+01_JPRB ,&
&  0.15292E+01_JPRB, 0.14968E+01_JPRB, 0.14889E+01_JPRB, 0.14855E+01_JPRB, 0.14857E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.15020E+01_JPRB, 0.14434E+01_JPRB, 0.13942E+01_JPRB, 0.13546E+01_JPRB, 0.13188E+01_JPRB, 0.14255E+01_JPRB, 0.15439E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.14588E+01_JPRB, 0.13839E+01_JPRB, 0.14345E+01_JPRB, 0.15844E+01_JPRB, 0.17639E+01_JPRB, 0.18684E+01_JPRB ,&
&  0.19814E+01_JPRB, 0.20960E+01_JPRB, 0.23227E+01_JPRB, 0.28984E+01_JPRB, 0.33444E+01_JPRB, 0.36139E+01_JPRB, 0.41300E+01_JPRB ,&
&  0.36780E+01_JPRB, 0.28253E+01_JPRB, 0.25744E+01_JPRB, 0.25473E+01_JPRB, 0.24209E+01_JPRB, 0.23381E+01_JPRB, 0.22364E+01_JPRB ,&
&  0.21018E+01_JPRB, 0.19692E+01_JPRB, 0.18487E+01_JPRB, 0.17849E+01_JPRB, 0.17174E+01_JPRB, 0.16017E+01_JPRB, 0.15464E+01_JPRB ,&
&  0.14960E+01_JPRB, 0.14714E+01_JPRB, 0.15083E+01_JPRB, 0.15534E+01_JPRB, 0.17204E+01_JPRB, 0.18857E+01_JPRB, 0.21154E+01_JPRB ,&
&  0.24168E+01_JPRB, 0.27184E+01_JPRB, 0.30490E+01_JPRB, 0.34451E+01_JPRB, 0.29566E+01_JPRB, 0.34933E+01_JPRB, 0.36835E+01_JPRB ,&
&  0.38662E+01_JPRB, 0.37847E+01_JPRB, 0.34952E+01_JPRB, 0.40585E+01_JPRB, 0.42456E+01_JPRB, 0.37446E+01_JPRB, 0.34341E+01_JPRB ,&
&  0.37578E+01_JPRB, 0.41696E+01_JPRB, 0.50687E+01_JPRB, 0.58942E+01_JPRB, 0.67486E+01_JPRB, 0.90374E+01_JPRB, 0.65084E+01_JPRB ,&
&  0.55721E+01_JPRB, 0.56307E+01_JPRB, 0.49569E+01_JPRB, 0.43743E+01_JPRB, 0.51892E+01_JPRB, 0.83183E+01_JPRB, 0.60619E+01_JPRB ,&
&  0.22538E+01_JPRB, 0.17317E+01_JPRB, 0.19660E+01_JPRB, 0.20041E+01_JPRB, 0.17631E+01_JPRB, 0.23093E+01_JPRB, 0.69567E+01_JPRB ,&
&  0.54655E+01_JPRB, 0.74783E+01_JPRB, 0.65476E+01_JPRB, 0.59893E+01_JPRB, 0.56613E+01_JPRB, 0.58235E+01_JPRB, 0.45743E+01_JPRB ,&
&  0.34209E+01_JPRB, 0.28363E+01_JPRB, 0.24057E+01_JPRB, 0.20921E+01_JPRB, 0.18055E+01_JPRB, 0.16266E+01_JPRB, 0.15426E+01_JPRB ,&
&  0.14925E+01_JPRB, 0.14777E+01_JPRB, 0.14729E+01_JPRB, 0.14668E+01_JPRB, 0.14839E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.15072E+01_JPRB, 0.14123E+01_JPRB, 0.13689E+01_JPRB, 0.13398E+01_JPRB, 0.14152E+01_JPRB, 0.15334E+01_JPRB, 0.15862E+01_JPRB ,&
&  0.16881E+01_JPRB, 0.17173E+01_JPRB, 0.17158E+01_JPRB, 0.17174E+01_JPRB, 0.18441E+01_JPRB, 0.20772E+01_JPRB, 0.22654E+01_JPRB ,&
&  0.25049E+01_JPRB, 0.27769E+01_JPRB, 0.31563E+01_JPRB, 0.37699E+01_JPRB, 0.44106E+01_JPRB, 0.43989E+01_JPRB, 0.42496E+01_JPRB ,&
&  0.35249E+01_JPRB, 0.27498E+01_JPRB, 0.23829E+01_JPRB, 0.24588E+01_JPRB, 0.24059E+01_JPRB, 0.23671E+01_JPRB, 0.23117E+01_JPRB ,&
&  0.23025E+01_JPRB, 0.21698E+01_JPRB, 0.20110E+01_JPRB, 0.19272E+01_JPRB, 0.17615E+01_JPRB, 0.16973E+01_JPRB, 0.17601E+01_JPRB ,&
&  0.17204E+01_JPRB, 0.17254E+01_JPRB, 0.17639E+01_JPRB, 0.18321E+01_JPRB, 0.20156E+01_JPRB, 0.21910E+01_JPRB, 0.24323E+01_JPRB ,&
&  0.27045E+01_JPRB, 0.29331E+01_JPRB, 0.32145E+01_JPRB, 0.32788E+01_JPRB, 0.33174E+01_JPRB, 0.34578E+01_JPRB, 0.37582E+01_JPRB ,&
&  0.38411E+01_JPRB, 0.37336E+01_JPRB, 0.35931E+01_JPRB, 0.36557E+01_JPRB, 0.36910E+01_JPRB, 0.35285E+01_JPRB, 0.34927E+01_JPRB ,&
&  0.42474E+01_JPRB, 0.53263E+01_JPRB, 0.61445E+01_JPRB, 0.69359E+01_JPRB, 0.74679E+01_JPRB, 0.91708E+01_JPRB, 0.84692E+01_JPRB ,&
&  0.66830E+01_JPRB, 0.53629E+01_JPRB, 0.52324E+01_JPRB, 0.61587E+01_JPRB, 0.69052E+01_JPRB, 0.82872E+01_JPRB, 0.98085E+01_JPRB ,&
&  0.97162E+01_JPRB, 0.51045E+01_JPRB, 0.37568E+01_JPRB, 0.54246E+01_JPRB, 0.35905E+01_JPRB, 0.26713E+01_JPRB, 0.40898E+01_JPRB ,&
&  0.58305E+01_JPRB, 0.70789E+01_JPRB, 0.66377E+01_JPRB, 0.57570E+01_JPRB, 0.56727E+01_JPRB, 0.51639E+01_JPRB, 0.40034E+01_JPRB ,&
&  0.30814E+01_JPRB, 0.24881E+01_JPRB, 0.21211E+01_JPRB, 0.18699E+01_JPRB, 0.16429E+01_JPRB, 0.15496E+01_JPRB, 0.15389E+01_JPRB ,&
&  0.14969E+01_JPRB, 0.15170E+01_JPRB, 0.15185E+01_JPRB, 0.14997E+01_JPRB, 0.15147E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.14583E+01_JPRB, 0.13826E+01_JPRB, 0.13461E+01_JPRB, 0.13683E+01_JPRB, 0.15081E+01_JPRB, 0.16776E+01_JPRB, 0.17736E+01_JPRB ,&
&  0.18714E+01_JPRB, 0.19386E+01_JPRB, 0.20171E+01_JPRB, 0.20398E+01_JPRB, 0.22683E+01_JPRB, 0.26703E+01_JPRB, 0.30173E+01_JPRB ,&
&  0.33040E+01_JPRB, 0.36128E+01_JPRB, 0.39915E+01_JPRB, 0.46175E+01_JPRB, 0.55154E+01_JPRB, 0.66540E+01_JPRB, 0.38723E+01_JPRB ,&
&  0.31421E+01_JPRB, 0.28372E+01_JPRB, 0.24374E+01_JPRB, 0.26216E+01_JPRB, 0.25520E+01_JPRB, 0.25394E+01_JPRB, 0.24187E+01_JPRB ,&
&  0.25091E+01_JPRB, 0.24053E+01_JPRB, 0.22909E+01_JPRB, 0.22854E+01_JPRB, 0.21238E+01_JPRB, 0.22102E+01_JPRB, 0.22995E+01_JPRB ,&
&  0.22535E+01_JPRB, 0.22532E+01_JPRB, 0.23530E+01_JPRB, 0.24541E+01_JPRB, 0.25754E+01_JPRB, 0.26938E+01_JPRB, 0.28437E+01_JPRB ,&
&  0.30101E+01_JPRB, 0.31960E+01_JPRB, 0.34505E+01_JPRB, 0.32991E+01_JPRB, 0.32785E+01_JPRB, 0.33499E+01_JPRB, 0.35689E+01_JPRB ,&
&  0.34250E+01_JPRB, 0.31039E+01_JPRB, 0.32265E+01_JPRB, 0.34318E+01_JPRB, 0.34409E+01_JPRB, 0.34061E+01_JPRB, 0.39926E+01_JPRB ,&
&  0.53596E+01_JPRB, 0.67381E+01_JPRB, 0.83017E+01_JPRB, 0.76425E+01_JPRB, 0.77798E+01_JPRB, 0.83860E+01_JPRB, 0.82536E+01_JPRB ,&
&  0.77721E+01_JPRB, 0.67987E+01_JPRB, 0.66427E+01_JPRB, 0.71433E+01_JPRB, 0.71267E+01_JPRB, 0.72456E+01_JPRB, 0.74650E+01_JPRB ,&
&  0.92341E+01_JPRB, 0.11567E+02_JPRB, 0.98148E+01_JPRB, 0.62772E+01_JPRB, 0.45702E+01_JPRB, 0.33913E+01_JPRB, 0.39957E+01_JPRB ,&
&  0.62454E+01_JPRB, 0.60436E+01_JPRB, 0.55880E+01_JPRB, 0.61009E+01_JPRB, 0.48100E+01_JPRB, 0.39725E+01_JPRB, 0.30426E+01_JPRB ,&
&  0.23487E+01_JPRB, 0.18431E+01_JPRB, 0.16630E+01_JPRB, 0.15019E+01_JPRB, 0.14665E+01_JPRB, 0.14814E+01_JPRB, 0.15226E+01_JPRB ,&
&  0.15698E+01_JPRB, 0.15776E+01_JPRB, 0.15560E+01_JPRB, 0.15239E+01_JPRB, 0.15152E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.14540E+01_JPRB, 0.14043E+01_JPRB, 0.13777E+01_JPRB, 0.14444E+01_JPRB, 0.16114E+01_JPRB, 0.17653E+01_JPRB, 0.18341E+01_JPRB ,&
&  0.18968E+01_JPRB, 0.19812E+01_JPRB, 0.21683E+01_JPRB, 0.23677E+01_JPRB, 0.27703E+01_JPRB, 0.32058E+01_JPRB, 0.36104E+01_JPRB ,&
&  0.39213E+01_JPRB, 0.41740E+01_JPRB, 0.45222E+01_JPRB, 0.50473E+01_JPRB, 0.56368E+01_JPRB, 0.61550E+01_JPRB, 0.46519E+01_JPRB ,&
&  0.27813E+01_JPRB, 0.35931E+01_JPRB, 0.27048E+01_JPRB, 0.29726E+01_JPRB, 0.30846E+01_JPRB, 0.33549E+01_JPRB, 0.30666E+01_JPRB ,&
&  0.30186E+01_JPRB, 0.28148E+01_JPRB, 0.27520E+01_JPRB, 0.27965E+01_JPRB, 0.25845E+01_JPRB, 0.26920E+01_JPRB, 0.28203E+01_JPRB ,&
&  0.29021E+01_JPRB, 0.30054E+01_JPRB, 0.30535E+01_JPRB, 0.30567E+01_JPRB, 0.30853E+01_JPRB, 0.31258E+01_JPRB, 0.32009E+01_JPRB ,&
&  0.33592E+01_JPRB, 0.35558E+01_JPRB, 0.36025E+01_JPRB, 0.35576E+01_JPRB, 0.33581E+01_JPRB, 0.34080E+01_JPRB, 0.35037E+01_JPRB ,&
&  0.32562E+01_JPRB, 0.29365E+01_JPRB, 0.30926E+01_JPRB, 0.32315E+01_JPRB, 0.34549E+01_JPRB, 0.43435E+01_JPRB, 0.52218E+01_JPRB ,&
&  0.66737E+01_JPRB, 0.73551E+01_JPRB, 0.93091E+01_JPRB, 0.71358E+01_JPRB, 0.69213E+01_JPRB, 0.73038E+01_JPRB, 0.66784E+01_JPRB ,&
&  0.60660E+01_JPRB, 0.62987E+01_JPRB, 0.64573E+01_JPRB, 0.72378E+01_JPRB, 0.73965E+01_JPRB, 0.68915E+01_JPRB, 0.61209E+01_JPRB ,&
&  0.65372E+01_JPRB, 0.70838E+01_JPRB, 0.61751E+01_JPRB, 0.42435E+01_JPRB, 0.37848E+01_JPRB, 0.37890E+01_JPRB, 0.51214E+01_JPRB ,&
&  0.58978E+01_JPRB, 0.55989E+01_JPRB, 0.53358E+01_JPRB, 0.48862E+01_JPRB, 0.33695E+01_JPRB, 0.25941E+01_JPRB, 0.18835E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.12409E+01_JPRB, 0.13846E+01_JPRB, 0.13491E+01_JPRB, 0.13881E+01_JPRB, 0.14497E+01_JPRB, 0.14692E+01_JPRB ,&
&  0.15101E+01_JPRB, 0.15162E+01_JPRB, 0.15247E+01_JPRB, 0.15056E+01_JPRB, 0.14924E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.13425E+01_JPRB, 0.13277E+01_JPRB, 0.13239E+01_JPRB, 0.14462E+01_JPRB, 0.16541E+01_JPRB, 0.18351E+01_JPRB, 0.18719E+01_JPRB ,&
&  0.19230E+01_JPRB, 0.18749E+01_JPRB, 0.21657E+01_JPRB, 0.24495E+01_JPRB, 0.28411E+01_JPRB, 0.32896E+01_JPRB, 0.37909E+01_JPRB ,&
&  0.42828E+01_JPRB, 0.47347E+01_JPRB, 0.51229E+01_JPRB, 0.55691E+01_JPRB, 0.61608E+01_JPRB, 0.67924E+01_JPRB, 0.60748E+01_JPRB ,&
&  0.43546E+01_JPRB, 0.31005E+01_JPRB, 0.30029E+01_JPRB, 0.30560E+01_JPRB, 0.31356E+01_JPRB, 0.34329E+01_JPRB, 0.35516E+01_JPRB ,&
&  0.35761E+01_JPRB, 0.31219E+01_JPRB, 0.29683E+01_JPRB, 0.30470E+01_JPRB, 0.27645E+01_JPRB, 0.29127E+01_JPRB, 0.31052E+01_JPRB ,&
&  0.32338E+01_JPRB, 0.33495E+01_JPRB, 0.33899E+01_JPRB, 0.33869E+01_JPRB, 0.34083E+01_JPRB, 0.34803E+01_JPRB, 0.35888E+01_JPRB ,&
&  0.37836E+01_JPRB, 0.40198E+01_JPRB, 0.41439E+01_JPRB, 0.39396E+01_JPRB, 0.38408E+01_JPRB, 0.39042E+01_JPRB, 0.38447E+01_JPRB ,&
&  0.36424E+01_JPRB, 0.36290E+01_JPRB, 0.38926E+01_JPRB, 0.43728E+01_JPRB, 0.46619E+01_JPRB, 0.57196E+01_JPRB, 0.63377E+01_JPRB ,&
&  0.70953E+01_JPRB, 0.75387E+01_JPRB, 0.84293E+01_JPRB, 0.62363E+01_JPRB, 0.62918E+01_JPRB, 0.63528E+01_JPRB, 0.60459E+01_JPRB ,&
&  0.58887E+01_JPRB, 0.62493E+01_JPRB, 0.66291E+01_JPRB, 0.68820E+01_JPRB, 0.69627E+01_JPRB, 0.59597E+01_JPRB, 0.56793E+01_JPRB ,&
&  0.53980E+01_JPRB, 0.59025E+01_JPRB, 0.47729E+01_JPRB, 0.42286E+01_JPRB, 0.37393E+01_JPRB, 0.40039E+01_JPRB, 0.44202E+01_JPRB ,&
&  0.49305E+01_JPRB, 0.45867E+01_JPRB, 0.39643E+01_JPRB, 0.30036E+01_JPRB, 0.18468E+01_JPRB, 0.13504E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.11642E+01_JPRB, 0.12335E+01_JPRB, 0.12624E+01_JPRB, 0.12940E+01_JPRB, 0.13378E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.13439E+01_JPRB, 0.13270E+01_JPRB, 0.13092E+01_JPRB, 0.13565E+01_JPRB, 0.13746E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.11469E+01_JPRB, 0.11073E+01_JPRB, 0.11054E+01_JPRB, 0.12309E+01_JPRB, 0.13855E+01_JPRB, 0.14803E+01_JPRB, 0.14922E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.15633E+01_JPRB, 0.17727E+01_JPRB, 0.20449E+01_JPRB, 0.23573E+01_JPRB, 0.27644E+01_JPRB, 0.32016E+01_JPRB ,&
&  0.36549E+01_JPRB, 0.39993E+01_JPRB, 0.42454E+01_JPRB, 0.45202E+01_JPRB, 0.48930E+01_JPRB, 0.53228E+01_JPRB, 0.54996E+01_JPRB ,&
&  0.43651E+01_JPRB, 0.25835E+01_JPRB, 0.28035E+01_JPRB, 0.30216E+01_JPRB, 0.33184E+01_JPRB, 0.35605E+01_JPRB, 0.35642E+01_JPRB ,&
&  0.30602E+01_JPRB, 0.28770E+01_JPRB, 0.29425E+01_JPRB, 0.30111E+01_JPRB, 0.27224E+01_JPRB, 0.28766E+01_JPRB, 0.30701E+01_JPRB ,&
&  0.32828E+01_JPRB, 0.34463E+01_JPRB, 0.35461E+01_JPRB, 0.36339E+01_JPRB, 0.37155E+01_JPRB, 0.38378E+01_JPRB, 0.40489E+01_JPRB ,&
&  0.43260E+01_JPRB, 0.45801E+01_JPRB, 0.47783E+01_JPRB, 0.47789E+01_JPRB, 0.47574E+01_JPRB, 0.48829E+01_JPRB, 0.48590E+01_JPRB ,&
&  0.48728E+01_JPRB, 0.48901E+01_JPRB, 0.54198E+01_JPRB, 0.61485E+01_JPRB, 0.65833E+01_JPRB, 0.60514E+01_JPRB, 0.66176E+01_JPRB ,&
&  0.68394E+01_JPRB, 0.68013E+01_JPRB, 0.64770E+01_JPRB, 0.68194E+01_JPRB, 0.49930E+01_JPRB, 0.53412E+01_JPRB, 0.54366E+01_JPRB ,&
&  0.61188E+01_JPRB, 0.62554E+01_JPRB, 0.64124E+01_JPRB, 0.62460E+01_JPRB, 0.64112E+01_JPRB, 0.54019E+01_JPRB, 0.49045E+01_JPRB ,&
&  0.48385E+01_JPRB, 0.49143E+01_JPRB, 0.45028E+01_JPRB, 0.43469E+01_JPRB, 0.39713E+01_JPRB, 0.41390E+01_JPRB, 0.38307E+01_JPRB ,&
&  0.38995E+01_JPRB, 0.30676E+01_JPRB, 0.25276E+01_JPRB, 0.18913E+01_JPRB, 0.10985E+01_JPRB, 0.94342E+00_JPRB, 0.97691E+00_JPRB ,&
&  0.10224E+01_JPRB, 0.10748E+01_JPRB, 0.10840E+01_JPRB, 0.11322E+01_JPRB, 0.11681E+01_JPRB, 0.11555E+01_JPRB, 0.11196E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.11638E+01_JPRB, 0.11665E+01_JPRB, 0.11702E+01_JPRB, 0.11639E+01_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.90267E+00_JPRB, 0.90327E+00_JPRB, 0.92272E+00_JPRB, 0.98182E+00_JPRB, 0.11004E+01_JPRB, 0.11889E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.12784E+01_JPRB, 0.13401E+01_JPRB, 0.14633E+01_JPRB, 0.15621E+01_JPRB, 0.17416E+01_JPRB, 0.19908E+01_JPRB, 0.22410E+01_JPRB ,&
&  0.25637E+01_JPRB, 0.28480E+01_JPRB, 0.30563E+01_JPRB, 0.33294E+01_JPRB, 0.35811E+01_JPRB, 0.39453E+01_JPRB, 0.41822E+01_JPRB ,&
&  0.41271E+01_JPRB, 0.33362E+01_JPRB, 0.30086E+01_JPRB, 0.24222E+01_JPRB, 0.23236E+01_JPRB, 0.27895E+01_JPRB, 0.25720E+01_JPRB ,&
&  0.25168E+01_JPRB, 0.25248E+01_JPRB, 0.24617E+01_JPRB, 0.25066E+01_JPRB, 0.26063E+01_JPRB, 0.27469E+01_JPRB, 0.29730E+01_JPRB ,&
&  0.32296E+01_JPRB, 0.34643E+01_JPRB, 0.36745E+01_JPRB, 0.38620E+01_JPRB, 0.39988E+01_JPRB, 0.41782E+01_JPRB, 0.44422E+01_JPRB ,&
&  0.47470E+01_JPRB, 0.49946E+01_JPRB, 0.52796E+01_JPRB, 0.56754E+01_JPRB, 0.56821E+01_JPRB, 0.58946E+01_JPRB, 0.60103E+01_JPRB ,&
&  0.60985E+01_JPRB, 0.60652E+01_JPRB, 0.61430E+01_JPRB, 0.63929E+01_JPRB, 0.63712E+01_JPRB, 0.57124E+01_JPRB, 0.57322E+01_JPRB ,&
&  0.60017E+01_JPRB, 0.57307E+01_JPRB, 0.50644E+01_JPRB, 0.57541E+01_JPRB, 0.40885E+01_JPRB, 0.48152E+01_JPRB, 0.55230E+01_JPRB ,&
&  0.55547E+01_JPRB, 0.55143E+01_JPRB, 0.53107E+01_JPRB, 0.52647E+01_JPRB, 0.52776E+01_JPRB, 0.48946E+01_JPRB, 0.46754E+01_JPRB ,&
&  0.44566E+01_JPRB, 0.36674E+01_JPRB, 0.34261E+01_JPRB, 0.36982E+01_JPRB, 0.39691E+01_JPRB, 0.35812E+01_JPRB, 0.28323E+01_JPRB ,&
&  0.23114E+01_JPRB, 0.23853E+01_JPRB, 0.22273E+01_JPRB, 0.23566E+01_JPRB, 0.14862E+01_JPRB, 0.91175E+00_JPRB, 0.90570E+00_JPRB ,&
&  0.92771E+00_JPRB, 0.95246E+00_JPRB, 0.94337E+00_JPRB, 0.99578E+00_JPRB, 0.10115E+01_JPRB, 0.98988E+00_JPRB, 0.99279E+00_JPRB ,&
&  0.10191E+01_JPRB, 0.10155E+01_JPRB, 0.10077E+01_JPRB, 0.97780E+00_JPRB, 0.95202E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.85088E+00_JPRB, 0.83630E+00_JPRB, 0.84236E+00_JPRB, 0.84389E+00_JPRB, 0.86307E+00_JPRB, 0.87345E+00_JPRB, 0.90523E+00_JPRB ,&
&  0.95443E+00_JPRB, 0.10073E+01_JPRB, 0.10168E+01_JPRB, 0.10103E+01_JPRB, 0.10291E+01_JPRB, 0.10683E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.13831E+01_JPRB, 0.16208E+01_JPRB, 0.17622E+01_JPRB, 0.18996E+01_JPRB, 0.21800E+01_JPRB, 0.27146E+01_JPRB, 0.32288E+01_JPRB ,&
&  0.36154E+01_JPRB, 0.38410E+01_JPRB, 0.37107E+01_JPRB, 0.37632E+01_JPRB, 0.29630E+01_JPRB, 0.25881E+01_JPRB, 0.28797E+01_JPRB ,&
&  0.31517E+01_JPRB, 0.29866E+01_JPRB, 0.23816E+01_JPRB, 0.23051E+01_JPRB, 0.23132E+01_JPRB, 0.24356E+01_JPRB, 0.27406E+01_JPRB ,&
&  0.30431E+01_JPRB, 0.33747E+01_JPRB, 0.36470E+01_JPRB, 0.38782E+01_JPRB, 0.40716E+01_JPRB, 0.42965E+01_JPRB, 0.45171E+01_JPRB ,&
&  0.47552E+01_JPRB, 0.49385E+01_JPRB, 0.51356E+01_JPRB, 0.55336E+01_JPRB, 0.59314E+01_JPRB, 0.62861E+01_JPRB, 0.64896E+01_JPRB ,&
&  0.65498E+01_JPRB, 0.62175E+01_JPRB, 0.62695E+01_JPRB, 0.63421E+01_JPRB, 0.60014E+01_JPRB, 0.55102E+01_JPRB, 0.53687E+01_JPRB ,&
&  0.52395E+01_JPRB, 0.49852E+01_JPRB, 0.38917E+01_JPRB, 0.47485E+01_JPRB, 0.54643E+01_JPRB, 0.50699E+01_JPRB, 0.49199E+01_JPRB ,&
&  0.46670E+01_JPRB, 0.44960E+01_JPRB, 0.42571E+01_JPRB, 0.43461E+01_JPRB, 0.46979E+01_JPRB, 0.60608E+01_JPRB, 0.47971E+01_JPRB ,&
&  0.40489E+01_JPRB, 0.26219E+01_JPRB, 0.22892E+01_JPRB, 0.26108E+01_JPRB, 0.29294E+01_JPRB, 0.26866E+01_JPRB, 0.19045E+01_JPRB ,&
&  0.16792E+01_JPRB, 0.19568E+01_JPRB, 0.21057E+01_JPRB, 0.26685E+01_JPRB, 0.27350E+01_JPRB, 0.97160E+00_JPRB, 0.82924E+00_JPRB ,&
&  0.82452E+00_JPRB, 0.82460E+00_JPRB, 0.81635E+00_JPRB, 0.84135E+00_JPRB, 0.84278E+00_JPRB, 0.84989E+00_JPRB, 0.87803E+00_JPRB ,&
&  0.89770E+00_JPRB, 0.89750E+00_JPRB, 0.88877E+00_JPRB, 0.85651E+00_JPRB, 0.85905E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.91746E+00_JPRB, 0.91752E+00_JPRB, 0.92677E+00_JPRB, 0.91904E+00_JPRB, 0.91057E+00_JPRB, 0.91204E+00_JPRB, 0.91952E+00_JPRB ,&
&  0.92442E+00_JPRB, 0.91224E+00_JPRB, 0.89620E+00_JPRB, 0.88762E+00_JPRB, 0.88312E+00_JPRB, 0.84236E+00_JPRB, 0.78555E+00_JPRB ,&
&  0.80709E+00_JPRB, 0.83962E+00_JPRB, 0.86043E+00_JPRB, 0.90479E+00_JPRB, 0.10233E+01_JPRB, 0.12111E+01_JPRB, 0.14850E+01_JPRB ,&
&  0.20016E+01_JPRB, 0.25662E+01_JPRB, 0.29545E+01_JPRB, 0.30219E+01_JPRB, 0.33665E+01_JPRB, 0.27838E+01_JPRB, 0.30030E+01_JPRB ,&
&  0.35422E+01_JPRB, 0.28221E+01_JPRB, 0.23709E+01_JPRB, 0.21114E+01_JPRB, 0.20889E+01_JPRB, 0.21956E+01_JPRB, 0.23959E+01_JPRB ,&
&  0.27032E+01_JPRB, 0.30318E+01_JPRB, 0.33145E+01_JPRB, 0.35978E+01_JPRB, 0.38708E+01_JPRB, 0.41505E+01_JPRB, 0.43820E+01_JPRB ,&
&  0.45299E+01_JPRB, 0.47131E+01_JPRB, 0.49672E+01_JPRB, 0.51357E+01_JPRB, 0.56679E+01_JPRB, 0.60040E+01_JPRB, 0.61688E+01_JPRB ,&
&  0.60531E+01_JPRB, 0.57957E+01_JPRB, 0.58327E+01_JPRB, 0.55773E+01_JPRB, 0.52909E+01_JPRB, 0.50152E+01_JPRB, 0.47201E+01_JPRB ,&
&  0.44435E+01_JPRB, 0.40282E+01_JPRB, 0.32795E+01_JPRB, 0.37411E+01_JPRB, 0.40448E+01_JPRB, 0.37623E+01_JPRB, 0.43888E+01_JPRB ,&
&  0.40951E+01_JPRB, 0.39240E+01_JPRB, 0.37929E+01_JPRB, 0.36581E+01_JPRB, 0.35556E+01_JPRB, 0.42053E+01_JPRB, 0.42442E+01_JPRB ,&
&  0.28714E+01_JPRB, 0.18517E+01_JPRB, 0.15332E+01_JPRB, 0.16109E+01_JPRB, 0.18617E+01_JPRB, 0.19866E+01_JPRB, 0.18207E+01_JPRB ,&
&  0.17800E+01_JPRB, 0.17027E+01_JPRB, 0.19356E+01_JPRB, 0.25118E+01_JPRB, 0.16819E+01_JPRB, 0.95887E+00_JPRB, 0.75966E+00_JPRB ,&
&  0.71581E+00_JPRB, 0.71121E+00_JPRB, 0.71983E+00_JPRB, 0.75312E+00_JPRB, 0.78246E+00_JPRB, 0.81304E+00_JPRB, 0.83999E+00_JPRB ,&
&  0.85401E+00_JPRB, 0.87493E+00_JPRB, 0.89097E+00_JPRB, 0.91007E+00_JPRB, 0.92081E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.10293E+01_JPRB, 0.10514E+01_JPRB, 0.10483E+01_JPRB, 0.10606E+01_JPRB, 0.10747E+01_JPRB, 0.10990E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.10981E+01_JPRB, 0.10747E+01_JPRB, 0.10812E+01_JPRB, 0.10849E+01_JPRB, 0.10866E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11049E+01_JPRB, 0.11109E+01_JPRB, 0.11054E+01_JPRB, 0.10812E+01_JPRB, 0.10434E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.12351E+01_JPRB, 0.14237E+01_JPRB, 0.15580E+01_JPRB, 0.18852E+01_JPRB, 0.19878E+01_JPRB, 0.22669E+01_JPRB ,&
&  0.29593E+01_JPRB, 0.24730E+01_JPRB, 0.23359E+01_JPRB, 0.19285E+01_JPRB, 0.18802E+01_JPRB, 0.18290E+01_JPRB, 0.18184E+01_JPRB ,&
&  0.18800E+01_JPRB, 0.20169E+01_JPRB, 0.21775E+01_JPRB, 0.23588E+01_JPRB, 0.25110E+01_JPRB, 0.25720E+01_JPRB, 0.26039E+01_JPRB ,&
&  0.26358E+01_JPRB, 0.28841E+01_JPRB, 0.35572E+01_JPRB, 0.41161E+01_JPRB, 0.47626E+01_JPRB, 0.50683E+01_JPRB, 0.51154E+01_JPRB ,&
&  0.50547E+01_JPRB, 0.50657E+01_JPRB, 0.44141E+01_JPRB, 0.39115E+01_JPRB, 0.38022E+01_JPRB, 0.34453E+01_JPRB, 0.33476E+01_JPRB ,&
&  0.34972E+01_JPRB, 0.33692E+01_JPRB, 0.27116E+01_JPRB, 0.29722E+01_JPRB, 0.32681E+01_JPRB, 0.32034E+01_JPRB, 0.37198E+01_JPRB ,&
&  0.32202E+01_JPRB, 0.28742E+01_JPRB, 0.27260E+01_JPRB, 0.26429E+01_JPRB, 0.25255E+01_JPRB, 0.24188E+01_JPRB, 0.24859E+01_JPRB ,&
&  0.19095E+01_JPRB, 0.15417E+01_JPRB, 0.14138E+01_JPRB, 0.14656E+01_JPRB, 0.18700E+01_JPRB, 0.26031E+01_JPRB, 0.20254E+01_JPRB ,&
&  0.19063E+01_JPRB, 0.18666E+01_JPRB, 0.25636E+01_JPRB, 0.36544E+01_JPRB, 0.16561E+01_JPRB, 0.10141E+01_JPRB, 0.71541E+00_JPRB ,&
&  0.69674E+00_JPRB, 0.73060E+00_JPRB, 0.76373E+00_JPRB, 0.80665E+00_JPRB, 0.83389E+00_JPRB, 0.84845E+00_JPRB, 0.87004E+00_JPRB ,&
&  0.91356E+00_JPRB, 0.95338E+00_JPRB, 0.10082E+01_JPRB, 0.10188E+01_JPRB, 0.10070E+01_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.11011E+01_JPRB, 0.11330E+01_JPRB, 0.11430E+01_JPRB, 0.11706E+01_JPRB, 0.12065E+01_JPRB, 0.12256E+01_JPRB, 0.12356E+01_JPRB ,&
&  0.12286E+01_JPRB, 0.12306E+01_JPRB, 0.12301E+01_JPRB, 0.12504E+01_JPRB, 0.12579E+01_JPRB, 0.12572E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.13085E+01_JPRB, 0.13557E+01_JPRB, 0.13566E+01_JPRB, 0.13481E+01_JPRB, 0.13530E+01_JPRB, 0.13874E+01_JPRB ,&
&  0.14180E+01_JPRB, 0.14427E+01_JPRB, 0.14843E+01_JPRB, 0.15234E+01_JPRB, 0.16740E+01_JPRB, 0.18663E+01_JPRB, 0.22034E+01_JPRB ,&
&  0.24089E+01_JPRB, 0.25385E+01_JPRB, 0.21309E+01_JPRB, 0.17854E+01_JPRB, 0.17493E+01_JPRB, 0.17256E+01_JPRB, 0.16206E+01_JPRB ,&
&  0.15180E+01_JPRB, 0.14717E+01_JPRB, 0.14394E+01_JPRB, 0.14318E+01_JPRB, 0.14427E+01_JPRB, 0.14075E+01_JPRB, 0.13876E+01_JPRB ,&
&  0.13310E+01_JPRB, 0.13619E+01_JPRB, 0.17341E+01_JPRB, 0.22384E+01_JPRB, 0.27437E+01_JPRB, 0.31613E+01_JPRB, 0.28487E+01_JPRB ,&
&  0.27224E+01_JPRB, 0.28975E+01_JPRB, 0.26913E+01_JPRB, 0.25073E+01_JPRB, 0.22792E+01_JPRB, 0.20696E+01_JPRB, 0.19801E+01_JPRB ,&
&  0.21961E+01_JPRB, 0.24261E+01_JPRB, 0.23125E+01_JPRB, 0.26478E+01_JPRB, 0.28147E+01_JPRB, 0.28182E+01_JPRB, 0.25314E+01_JPRB ,&
&  0.21594E+01_JPRB, 0.19898E+01_JPRB, 0.19493E+01_JPRB, 0.19501E+01_JPRB, 0.19751E+01_JPRB, 0.18583E+01_JPRB, 0.16688E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.13748E+01_JPRB, 0.13471E+01_JPRB, 0.13234E+01_JPRB, 0.18240E+01_JPRB, 0.23319E+01_JPRB, 0.29872E+01_JPRB ,&
&  0.20881E+01_JPRB, 0.19346E+01_JPRB, 0.16778E+01_JPRB, 0.26287E+01_JPRB, 0.13729E+01_JPRB, 0.75891E+00_JPRB, 0.70345E+00_JPRB ,&
&  0.77869E+00_JPRB, 0.82736E+00_JPRB, 0.85487E+00_JPRB, 0.89387E+00_JPRB, 0.91541E+00_JPRB, 0.92283E+00_JPRB, 0.94319E+00_JPRB ,&
&  0.99378E+00_JPRB, 0.10149E+01_JPRB, 0.10361E+01_JPRB, 0.10446E+01_JPRB, 0.10710E+01_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.11392E+01_JPRB, 0.11836E+01_JPRB, 0.12135E+01_JPRB, 0.12445E+01_JPRB, 0.12832E+01_JPRB, 0.13107E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.13441E+01_JPRB, 0.13456E+01_JPRB, 0.13616E+01_JPRB, 0.13821E+01_JPRB, 0.13907E+01_JPRB, 0.13847E+01_JPRB, 0.13923E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.14613E+01_JPRB, 0.15114E+01_JPRB, 0.15080E+01_JPRB, 0.15084E+01_JPRB, 0.15413E+01_JPRB, 0.15868E+01_JPRB ,&
&  0.15898E+01_JPRB, 0.15754E+01_JPRB, 0.15879E+01_JPRB, 0.16132E+01_JPRB, 0.16607E+01_JPRB, 0.16916E+01_JPRB, 0.18372E+01_JPRB ,&
&  0.19902E+01_JPRB, 0.27085E+01_JPRB, 0.23694E+01_JPRB, 0.20083E+01_JPRB, 0.19652E+01_JPRB, 0.17845E+01_JPRB, 0.17377E+01_JPRB ,&
&  0.17024E+01_JPRB, 0.16157E+01_JPRB, 0.15641E+01_JPRB, 0.15157E+01_JPRB, 0.15116E+01_JPRB, 0.15259E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.15033E+01_JPRB, 0.15347E+01_JPRB, 0.16567E+01_JPRB, 0.18169E+01_JPRB, 0.19695E+01_JPRB, 0.20143E+01_JPRB, 0.18607E+01_JPRB ,&
&  0.17568E+01_JPRB, 0.15921E+01_JPRB, 0.13302E+01_JPRB, 0.13503E+01_JPRB, 0.13179E+01_JPRB, 0.11053E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.16458E+01_JPRB, 0.21134E+01_JPRB, 0.22543E+01_JPRB, 0.22465E+01_JPRB, 0.23191E+01_JPRB, 0.19493E+01_JPRB ,&
&  0.16995E+01_JPRB, 0.15616E+01_JPRB, 0.15614E+01_JPRB, 0.15221E+01_JPRB, 0.15536E+01_JPRB, 0.15302E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.12789E+01_JPRB, 0.13214E+01_JPRB, 0.14345E+01_JPRB, 0.16787E+01_JPRB, 0.32119E+01_JPRB, 0.48064E+01_JPRB ,&
&  0.24582E+01_JPRB, 0.17447E+01_JPRB, 0.99919E+00_JPRB, 0.12426E+01_JPRB, 0.91716E+00_JPRB, 0.72924E+00_JPRB, 0.75967E+00_JPRB ,&
&  0.88638E+00_JPRB, 0.94798E+00_JPRB, 0.96495E+00_JPRB, 0.96630E+00_JPRB, 0.95251E+00_JPRB, 0.94235E+00_JPRB, 0.94193E+00_JPRB ,&
&  0.97121E+00_JPRB, 0.10010E+01_JPRB, 0.10350E+01_JPRB, 0.10608E+01_JPRB, 0.10948E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.11509E+01_JPRB, 0.11953E+01_JPRB, 0.12333E+01_JPRB, 0.12771E+01_JPRB, 0.13244E+01_JPRB, 0.13683E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14427E+01_JPRB, 0.14719E+01_JPRB, 0.15002E+01_JPRB, 0.15276E+01_JPRB, 0.15568E+01_JPRB, 0.15792E+01_JPRB, 0.16085E+01_JPRB ,&
&  0.16528E+01_JPRB, 0.16938E+01_JPRB, 0.17457E+01_JPRB, 0.17565E+01_JPRB, 0.17673E+01_JPRB, 0.18215E+01_JPRB, 0.18693E+01_JPRB ,&
&  0.18810E+01_JPRB, 0.19044E+01_JPRB, 0.19442E+01_JPRB, 0.20021E+01_JPRB, 0.20847E+01_JPRB, 0.22828E+01_JPRB, 0.22729E+01_JPRB ,&
&  0.26090E+01_JPRB, 0.28335E+01_JPRB, 0.26047E+01_JPRB, 0.22896E+01_JPRB, 0.21570E+01_JPRB, 0.20925E+01_JPRB, 0.20855E+01_JPRB ,&
&  0.20998E+01_JPRB, 0.20330E+01_JPRB, 0.19954E+01_JPRB, 0.19479E+01_JPRB, 0.19295E+01_JPRB, 0.19575E+01_JPRB, 0.19902E+01_JPRB ,&
&  0.20415E+01_JPRB, 0.21079E+01_JPRB, 0.22041E+01_JPRB, 0.23331E+01_JPRB, 0.24725E+01_JPRB, 0.24597E+01_JPRB, 0.23795E+01_JPRB ,&
&  0.22068E+01_JPRB, 0.20233E+01_JPRB, 0.15771E+01_JPRB, 0.14684E+01_JPRB, 0.13711E+01_JPRB, 0.11927E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.13273E+01_JPRB, 0.14365E+01_JPRB, 0.16590E+01_JPRB, 0.19824E+01_JPRB, 0.19185E+01_JPRB, 0.18148E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.14218E+01_JPRB, 0.13095E+01_JPRB, 0.12782E+01_JPRB, 0.12347E+01_JPRB, 0.12385E+01_JPRB, 0.12558E+01_JPRB, 0.11674E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.11795E+01_JPRB, 0.12402E+01_JPRB, 0.13684E+01_JPRB, 0.16698E+01_JPRB, 0.19469E+01_JPRB, 0.30366E+01_JPRB ,&
&  0.21809E+01_JPRB, 0.12573E+01_JPRB, 0.11924E+01_JPRB, 0.10173E+01_JPRB, 0.88324E+00_JPRB, 0.77367E+00_JPRB, 0.83398E+00_JPRB ,&
&  0.99155E+00_JPRB, 0.11230E+01_JPRB, 0.11574E+01_JPRB, 0.11094E+01_JPRB, 0.10300E+01_JPRB, 0.99046E+00_JPRB, 0.97292E+00_JPRB ,&
&  0.99874E+00_JPRB, 0.10231E+01_JPRB, 0.10515E+01_JPRB, 0.10724E+01_JPRB, 0.11045E+01_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.11321E+01_JPRB, 0.11811E+01_JPRB, 0.12253E+01_JPRB, 0.12706E+01_JPRB, 0.13173E+01_JPRB, 0.13652E+01_JPRB, 0.14139E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.15064E+01_JPRB, 0.15628E+01_JPRB, 0.16181E+01_JPRB, 0.16732E+01_JPRB, 0.17294E+01_JPRB, 0.17806E+01_JPRB ,&
&  0.18412E+01_JPRB, 0.18905E+01_JPRB, 0.19530E+01_JPRB, 0.19829E+01_JPRB, 0.20210E+01_JPRB, 0.20787E+01_JPRB, 0.21382E+01_JPRB ,&
&  0.21642E+01_JPRB, 0.21995E+01_JPRB, 0.22413E+01_JPRB, 0.23040E+01_JPRB, 0.23422E+01_JPRB, 0.25294E+01_JPRB, 0.24116E+01_JPRB ,&
&  0.29270E+01_JPRB, 0.31653E+01_JPRB, 0.29403E+01_JPRB, 0.26341E+01_JPRB, 0.24833E+01_JPRB, 0.24777E+01_JPRB, 0.24908E+01_JPRB ,&
&  0.25286E+01_JPRB, 0.26155E+01_JPRB, 0.24602E+01_JPRB, 0.23798E+01_JPRB, 0.23571E+01_JPRB, 0.24045E+01_JPRB, 0.25073E+01_JPRB ,&
&  0.26198E+01_JPRB, 0.27432E+01_JPRB, 0.28731E+01_JPRB, 0.30002E+01_JPRB, 0.31191E+01_JPRB, 0.31900E+01_JPRB, 0.31923E+01_JPRB ,&
&  0.30970E+01_JPRB, 0.29585E+01_JPRB, 0.26397E+01_JPRB, 0.25799E+01_JPRB, 0.24210E+01_JPRB, 0.19949E+01_JPRB, 0.15905E+01_JPRB ,&
&  0.15134E+01_JPRB, 0.15304E+01_JPRB, 0.15321E+01_JPRB, 0.17368E+01_JPRB, 0.16783E+01_JPRB, 0.15571E+01_JPRB, 0.13877E+01_JPRB ,&
&  0.12370E+01_JPRB, 0.11214E+01_JPRB, 0.11094E+01_JPRB, 0.10617E+01_JPRB, 0.10231E+01_JPRB, 0.10056E+01_JPRB, 0.99121E+00_JPRB ,&
&  0.10215E+01_JPRB, 0.10754E+01_JPRB, 0.11547E+01_JPRB, 0.12627E+01_JPRB, 0.13918E+01_JPRB, 0.14709E+01_JPRB, 0.24158E+01_JPRB ,&
&  0.18288E+01_JPRB, 0.14255E+01_JPRB, 0.16435E+01_JPRB, 0.80955E+00_JPRB, 0.83749E+00_JPRB, 0.89641E+00_JPRB, 0.10322E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.15052E+01_JPRB, 0.19707E+01_JPRB, 0.15872E+01_JPRB, 0.13214E+01_JPRB, 0.10639E+01_JPRB, 0.98552E+00_JPRB ,&
&  0.99790E+00_JPRB, 0.10138E+01_JPRB, 0.10373E+01_JPRB, 0.10562E+01_JPRB, 0.10878E+01_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.10956E+01_JPRB, 0.11366E+01_JPRB, 0.11839E+01_JPRB, 0.12286E+01_JPRB, 0.12663E+01_JPRB, 0.13028E+01_JPRB, 0.13446E+01_JPRB ,&
&  0.13773E+01_JPRB, 0.14040E+01_JPRB, 0.14460E+01_JPRB, 0.15013E+01_JPRB, 0.15630E+01_JPRB, 0.16355E+01_JPRB, 0.17104E+01_JPRB ,&
&  0.18075E+01_JPRB, 0.19024E+01_JPRB, 0.20240E+01_JPRB, 0.21425E+01_JPRB, 0.22461E+01_JPRB, 0.23558E+01_JPRB, 0.24756E+01_JPRB ,&
&  0.26133E+01_JPRB, 0.27115E+01_JPRB, 0.27872E+01_JPRB, 0.29183E+01_JPRB, 0.30469E+01_JPRB, 0.32433E+01_JPRB, 0.27262E+01_JPRB ,&
&  0.32842E+01_JPRB, 0.36865E+01_JPRB, 0.33647E+01_JPRB, 0.29526E+01_JPRB, 0.27744E+01_JPRB, 0.27611E+01_JPRB, 0.27551E+01_JPRB ,&
&  0.27676E+01_JPRB, 0.27395E+01_JPRB, 0.25527E+01_JPRB, 0.26411E+01_JPRB, 0.27148E+01_JPRB, 0.27435E+01_JPRB, 0.29017E+01_JPRB ,&
&  0.30697E+01_JPRB, 0.32412E+01_JPRB, 0.33954E+01_JPRB, 0.35384E+01_JPRB, 0.36659E+01_JPRB, 0.38031E+01_JPRB, 0.38528E+01_JPRB ,&
&  0.38580E+01_JPRB, 0.38544E+01_JPRB, 0.37959E+01_JPRB, 0.34434E+01_JPRB, 0.32685E+01_JPRB, 0.30471E+01_JPRB, 0.24527E+01_JPRB ,&
&  0.19590E+01_JPRB, 0.15819E+01_JPRB, 0.14554E+01_JPRB, 0.14861E+01_JPRB, 0.13899E+01_JPRB, 0.12556E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.96046E+00_JPRB, 0.97098E+00_JPRB, 0.93823E+00_JPRB, 0.83874E+00_JPRB, 0.86664E+00_JPRB, 0.90114E+00_JPRB ,&
&  0.91567E+00_JPRB, 0.96281E+00_JPRB, 0.10321E+01_JPRB, 0.10999E+01_JPRB, 0.12631E+01_JPRB, 0.16057E+01_JPRB, 0.18407E+01_JPRB ,&
&  0.18850E+01_JPRB, 0.13026E+01_JPRB, 0.11853E+01_JPRB, 0.11344E+01_JPRB, 0.94566E+00_JPRB, 0.11730E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.21852E+01_JPRB, 0.22666E+01_JPRB, 0.28306E+01_JPRB, 0.43225E+01_JPRB, 0.33018E+01_JPRB, 0.22616E+01_JPRB, 0.98495E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.10077E+01_JPRB, 0.10396E+01_JPRB, 0.10537E+01_JPRB, 0.10779E+01_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.91685E+00_JPRB, 0.92798E+00_JPRB, 0.97171E+00_JPRB, 0.10218E+01_JPRB, 0.10709E+01_JPRB, 0.11189E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.12730E+01_JPRB, 0.13057E+01_JPRB, 0.13283E+01_JPRB, 0.13619E+01_JPRB, 0.14054E+01_JPRB, 0.14677E+01_JPRB ,&
&  0.15555E+01_JPRB, 0.16535E+01_JPRB, 0.17904E+01_JPRB, 0.19074E+01_JPRB, 0.20413E+01_JPRB, 0.22138E+01_JPRB, 0.23871E+01_JPRB ,&
&  0.25761E+01_JPRB, 0.28077E+01_JPRB, 0.29944E+01_JPRB, 0.31689E+01_JPRB, 0.33657E+01_JPRB, 0.33764E+01_JPRB, 0.24934E+01_JPRB ,&
&  0.35570E+01_JPRB, 0.38926E+01_JPRB, 0.36895E+01_JPRB, 0.32449E+01_JPRB, 0.30056E+01_JPRB, 0.29297E+01_JPRB, 0.29421E+01_JPRB ,&
&  0.29131E+01_JPRB, 0.27688E+01_JPRB, 0.25804E+01_JPRB, 0.25061E+01_JPRB, 0.28373E+01_JPRB, 0.28148E+01_JPRB, 0.29864E+01_JPRB ,&
&  0.31621E+01_JPRB, 0.33337E+01_JPRB, 0.34746E+01_JPRB, 0.35949E+01_JPRB, 0.36925E+01_JPRB, 0.38131E+01_JPRB, 0.39448E+01_JPRB ,&
&  0.40669E+01_JPRB, 0.41367E+01_JPRB, 0.42572E+01_JPRB, 0.38657E+01_JPRB, 0.36755E+01_JPRB, 0.39208E+01_JPRB, 0.34822E+01_JPRB ,&
&  0.22854E+01_JPRB, 0.16124E+01_JPRB, 0.15825E+01_JPRB, 0.14641E+01_JPRB, 0.12900E+01_JPRB, 0.11401E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.95387E+00_JPRB, 0.89653E+00_JPRB, 0.87301E+00_JPRB, 0.83788E+00_JPRB, 0.83080E+00_JPRB, 0.85157E+00_JPRB, 0.85362E+00_JPRB ,&
&  0.90018E+00_JPRB, 0.88619E+00_JPRB, 0.89973E+00_JPRB, 0.95071E+00_JPRB, 0.11127E+01_JPRB, 0.14666E+01_JPRB, 0.20768E+01_JPRB ,&
&  0.21937E+01_JPRB, 0.12743E+01_JPRB, 0.11412E+01_JPRB, 0.10694E+01_JPRB, 0.10777E+01_JPRB, 0.11976E+01_JPRB, 0.13994E+01_JPRB ,&
&  0.17782E+01_JPRB, 0.23004E+01_JPRB, 0.21551E+01_JPRB, 0.16851E+01_JPRB, 0.20374E+01_JPRB, 0.17476E+01_JPRB, 0.87502E+00_JPRB ,&
&  0.88645E+00_JPRB, 0.86161E+00_JPRB, 0.87762E+00_JPRB, 0.89590E+00_JPRB, 0.90900E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.74271E+00_JPRB, 0.73523E+00_JPRB, 0.75064E+00_JPRB, 0.78109E+00_JPRB, 0.81554E+00_JPRB, 0.87478E+00_JPRB, 0.94741E+00_JPRB ,&
&  0.10353E+01_JPRB, 0.11050E+01_JPRB, 0.11399E+01_JPRB, 0.11628E+01_JPRB, 0.11920E+01_JPRB, 0.12300E+01_JPRB, 0.12782E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.14157E+01_JPRB, 0.15119E+01_JPRB, 0.15926E+01_JPRB, 0.16922E+01_JPRB, 0.18602E+01_JPRB, 0.20629E+01_JPRB ,&
&  0.23047E+01_JPRB, 0.25647E+01_JPRB, 0.28563E+01_JPRB, 0.31599E+01_JPRB, 0.34781E+01_JPRB, 0.36338E+01_JPRB, 0.37396E+01_JPRB ,&
&  0.26827E+01_JPRB, 0.36988E+01_JPRB, 0.37752E+01_JPRB, 0.34723E+01_JPRB, 0.32250E+01_JPRB, 0.32098E+01_JPRB, 0.31640E+01_JPRB ,&
&  0.30736E+01_JPRB, 0.27895E+01_JPRB, 0.26324E+01_JPRB, 0.25143E+01_JPRB, 0.26493E+01_JPRB, 0.27216E+01_JPRB, 0.28683E+01_JPRB ,&
&  0.30119E+01_JPRB, 0.31302E+01_JPRB, 0.32272E+01_JPRB, 0.33309E+01_JPRB, 0.34724E+01_JPRB, 0.36735E+01_JPRB, 0.39357E+01_JPRB ,&
&  0.41760E+01_JPRB, 0.43632E+01_JPRB, 0.44917E+01_JPRB, 0.41827E+01_JPRB, 0.38918E+01_JPRB, 0.42956E+01_JPRB, 0.40885E+01_JPRB ,&
&  0.25920E+01_JPRB, 0.17260E+01_JPRB, 0.17476E+01_JPRB, 0.17070E+01_JPRB, 0.14351E+01_JPRB, 0.12664E+01_JPRB, 0.11309E+01_JPRB ,&
&  0.96081E+00_JPRB, 0.85236E+00_JPRB, 0.86326E+00_JPRB, 0.85796E+00_JPRB, 0.86853E+00_JPRB, 0.87082E+00_JPRB, 0.88893E+00_JPRB ,&
&  0.93426E+00_JPRB, 0.90813E+00_JPRB, 0.90745E+00_JPRB, 0.92402E+00_JPRB, 0.98345E+00_JPRB, 0.10569E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11957E+01_JPRB, 0.11028E+01_JPRB, 0.10713E+01_JPRB, 0.10836E+01_JPRB, 0.10849E+01_JPRB, 0.11100E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.12436E+01_JPRB, 0.12685E+01_JPRB, 0.12454E+01_JPRB, 0.11186E+01_JPRB, 0.10365E+01_JPRB, 0.79439E+00_JPRB ,&
&  0.78008E+00_JPRB, 0.75790E+00_JPRB, 0.75751E+00_JPRB, 0.75616E+00_JPRB, 0.75887E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.64478E+00_JPRB, 0.64190E+00_JPRB, 0.65513E+00_JPRB, 0.67306E+00_JPRB, 0.70487E+00_JPRB, 0.75792E+00_JPRB, 0.83337E+00_JPRB ,&
&  0.91790E+00_JPRB, 0.97284E+00_JPRB, 0.10222E+01_JPRB, 0.10288E+01_JPRB, 0.10419E+01_JPRB, 0.10785E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.12511E+01_JPRB, 0.12984E+01_JPRB, 0.13250E+01_JPRB, 0.13623E+01_JPRB, 0.14485E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.17344E+01_JPRB, 0.19734E+01_JPRB, 0.23016E+01_JPRB, 0.27415E+01_JPRB, 0.32249E+01_JPRB, 0.34732E+01_JPRB, 0.35566E+01_JPRB ,&
&  0.16765E+01_JPRB, 0.27936E+01_JPRB, 0.37647E+01_JPRB, 0.37457E+01_JPRB, 0.36229E+01_JPRB, 0.35560E+01_JPRB, 0.35171E+01_JPRB ,&
&  0.31200E+01_JPRB, 0.26729E+01_JPRB, 0.24541E+01_JPRB, 0.26335E+01_JPRB, 0.24625E+01_JPRB, 0.25551E+01_JPRB, 0.26616E+01_JPRB ,&
&  0.27910E+01_JPRB, 0.29016E+01_JPRB, 0.29619E+01_JPRB, 0.30132E+01_JPRB, 0.31240E+01_JPRB, 0.33122E+01_JPRB, 0.35557E+01_JPRB ,&
&  0.38772E+01_JPRB, 0.42788E+01_JPRB, 0.45579E+01_JPRB, 0.43287E+01_JPRB, 0.42229E+01_JPRB, 0.45129E+01_JPRB, 0.55563E+01_JPRB ,&
&  0.34868E+01_JPRB, 0.20637E+01_JPRB, 0.20398E+01_JPRB, 0.20345E+01_JPRB, 0.16725E+01_JPRB, 0.14882E+01_JPRB, 0.13345E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.10075E+01_JPRB, 0.94977E+00_JPRB, 0.91601E+00_JPRB, 0.92862E+00_JPRB, 0.96281E+00_JPRB, 0.10273E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10616E+01_JPRB, 0.10462E+01_JPRB, 0.10551E+01_JPRB, 0.10559E+01_JPRB, 0.10686E+01_JPRB, 0.10692E+01_JPRB ,&
&  0.10867E+01_JPRB, 0.10559E+01_JPRB, 0.10419E+01_JPRB, 0.10685E+01_JPRB, 0.11087E+01_JPRB, 0.11171E+01_JPRB, 0.11156E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10868E+01_JPRB, 0.10824E+01_JPRB, 0.10512E+01_JPRB, 0.99681E+00_JPRB, 0.93016E+00_JPRB, 0.82295E+00_JPRB ,&
&  0.76993E+00_JPRB, 0.70775E+00_JPRB, 0.66609E+00_JPRB, 0.64567E+00_JPRB, 0.65460E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.62122E+00_JPRB, 0.61706E+00_JPRB, 0.62233E+00_JPRB, 0.65562E+00_JPRB, 0.65610E+00_JPRB, 0.68303E+00_JPRB, 0.72278E+00_JPRB ,&
&  0.80420E+00_JPRB, 0.87852E+00_JPRB, 0.95177E+00_JPRB, 0.95827E+00_JPRB, 0.95395E+00_JPRB, 0.93653E+00_JPRB, 0.94806E+00_JPRB ,&
&  0.98144E+00_JPRB, 0.10295E+01_JPRB, 0.10822E+01_JPRB, 0.11156E+01_JPRB, 0.11454E+01_JPRB, 0.11781E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12771E+01_JPRB, 0.13714E+01_JPRB, 0.15640E+01_JPRB, 0.19429E+01_JPRB, 0.25809E+01_JPRB, 0.33116E+01_JPRB, 0.39021E+01_JPRB ,&
&  0.26782E+01_JPRB, 0.10178E+01_JPRB, 0.30692E+01_JPRB, 0.40057E+01_JPRB, 0.39556E+01_JPRB, 0.38481E+01_JPRB, 0.36703E+01_JPRB ,&
&  0.31699E+01_JPRB, 0.27944E+01_JPRB, 0.23742E+01_JPRB, 0.25006E+01_JPRB, 0.22413E+01_JPRB, 0.22399E+01_JPRB, 0.23364E+01_JPRB ,&
&  0.24734E+01_JPRB, 0.26075E+01_JPRB, 0.27383E+01_JPRB, 0.27756E+01_JPRB, 0.27792E+01_JPRB, 0.28434E+01_JPRB, 0.30023E+01_JPRB ,&
&  0.33014E+01_JPRB, 0.38071E+01_JPRB, 0.44654E+01_JPRB, 0.46997E+01_JPRB, 0.54305E+01_JPRB, 0.51414E+01_JPRB, 0.55573E+01_JPRB ,&
&  0.36781E+01_JPRB, 0.25233E+01_JPRB, 0.23628E+01_JPRB, 0.23688E+01_JPRB, 0.19630E+01_JPRB, 0.16340E+01_JPRB, 0.16055E+01_JPRB ,&
&  0.13076E+01_JPRB, 0.11645E+01_JPRB, 0.11050E+01_JPRB, 0.10546E+01_JPRB, 0.10195E+01_JPRB, 0.10593E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.11844E+01_JPRB, 0.12016E+01_JPRB, 0.11942E+01_JPRB, 0.11919E+01_JPRB, 0.11830E+01_JPRB, 0.11547E+01_JPRB, 0.11254E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.11391E+01_JPRB, 0.11488E+01_JPRB, 0.11594E+01_JPRB, 0.11691E+01_JPRB, 0.11442E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11263E+01_JPRB, 0.11348E+01_JPRB, 0.10987E+01_JPRB, 0.10971E+01_JPRB, 0.10576E+01_JPRB, 0.10015E+01_JPRB, 0.95507E+00_JPRB ,&
&  0.87952E+00_JPRB, 0.77179E+00_JPRB, 0.69105E+00_JPRB, 0.65759E+00_JPRB, 0.62993E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.68069E+00_JPRB, 0.65644E+00_JPRB, 0.66144E+00_JPRB, 0.67972E+00_JPRB, 0.69631E+00_JPRB, 0.71008E+00_JPRB, 0.69562E+00_JPRB ,&
&  0.72407E+00_JPRB, 0.78429E+00_JPRB, 0.82748E+00_JPRB, 0.87184E+00_JPRB, 0.86456E+00_JPRB, 0.84349E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82803E+00_JPRB, 0.83741E+00_JPRB, 0.86956E+00_JPRB, 0.88823E+00_JPRB, 0.90923E+00_JPRB, 0.93870E+00_JPRB, 0.95773E+00_JPRB ,&
&  0.97370E+00_JPRB, 0.10015E+01_JPRB, 0.10655E+01_JPRB, 0.12135E+01_JPRB, 0.15176E+01_JPRB, 0.21398E+01_JPRB, 0.31136E+01_JPRB ,&
&  0.49681E+01_JPRB, 0.35932E+01_JPRB, 0.14642E+01_JPRB, 0.33621E+01_JPRB, 0.43295E+01_JPRB, 0.44390E+01_JPRB, 0.37657E+01_JPRB ,&
&  0.35256E+01_JPRB, 0.29558E+01_JPRB, 0.24914E+01_JPRB, 0.25336E+01_JPRB, 0.21274E+01_JPRB, 0.19504E+01_JPRB, 0.19176E+01_JPRB ,&
&  0.19933E+01_JPRB, 0.21534E+01_JPRB, 0.23356E+01_JPRB, 0.24655E+01_JPRB, 0.25229E+01_JPRB, 0.25318E+01_JPRB, 0.25704E+01_JPRB ,&
&  0.27527E+01_JPRB, 0.30706E+01_JPRB, 0.36536E+01_JPRB, 0.46172E+01_JPRB, 0.66345E+01_JPRB, 0.62080E+01_JPRB, 0.61825E+01_JPRB ,&
&  0.39180E+01_JPRB, 0.33105E+01_JPRB, 0.30624E+01_JPRB, 0.27877E+01_JPRB, 0.22335E+01_JPRB, 0.19578E+01_JPRB, 0.19586E+01_JPRB ,&
&  0.15831E+01_JPRB, 0.13347E+01_JPRB, 0.12398E+01_JPRB, 0.11662E+01_JPRB, 0.11264E+01_JPRB, 0.11389E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.12545E+01_JPRB, 0.12389E+01_JPRB, 0.12287E+01_JPRB, 0.12209E+01_JPRB, 0.12094E+01_JPRB, 0.11909E+01_JPRB ,&
&  0.11820E+01_JPRB, 0.11938E+01_JPRB, 0.12206E+01_JPRB, 0.12102E+01_JPRB, 0.11723E+01_JPRB, 0.11433E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.11879E+01_JPRB, 0.11467E+01_JPRB, 0.11536E+01_JPRB, 0.11176E+01_JPRB, 0.10819E+01_JPRB, 0.10445E+01_JPRB ,&
&  0.99001E+00_JPRB, 0.88815E+00_JPRB, 0.78252E+00_JPRB, 0.69092E+00_JPRB, 0.68159E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.76438E+00_JPRB, 0.74360E+00_JPRB, 0.75008E+00_JPRB, 0.74502E+00_JPRB, 0.73545E+00_JPRB, 0.73622E+00_JPRB, 0.73135E+00_JPRB ,&
&  0.72182E+00_JPRB, 0.71964E+00_JPRB, 0.71706E+00_JPRB, 0.76108E+00_JPRB, 0.78499E+00_JPRB, 0.76793E+00_JPRB, 0.75014E+00_JPRB ,&
&  0.73399E+00_JPRB, 0.73430E+00_JPRB, 0.74740E+00_JPRB, 0.75756E+00_JPRB, 0.76690E+00_JPRB, 0.78645E+00_JPRB, 0.80206E+00_JPRB ,&
&  0.79903E+00_JPRB, 0.80202E+00_JPRB, 0.82288E+00_JPRB, 0.87536E+00_JPRB, 0.98702E+00_JPRB, 0.12128E+01_JPRB, 0.16724E+01_JPRB ,&
&  0.28956E+01_JPRB, 0.30604E+01_JPRB, 0.12809E+01_JPRB, 0.26350E+01_JPRB, 0.44518E+01_JPRB, 0.47753E+01_JPRB, 0.45642E+01_JPRB ,&
&  0.42104E+01_JPRB, 0.33046E+01_JPRB, 0.27954E+01_JPRB, 0.26625E+01_JPRB, 0.22290E+01_JPRB, 0.19080E+01_JPRB, 0.17090E+01_JPRB ,&
&  0.16256E+01_JPRB, 0.16565E+01_JPRB, 0.17933E+01_JPRB, 0.19595E+01_JPRB, 0.21103E+01_JPRB, 0.21874E+01_JPRB, 0.22165E+01_JPRB ,&
&  0.22700E+01_JPRB, 0.24288E+01_JPRB, 0.27986E+01_JPRB, 0.32690E+01_JPRB, 0.54360E+01_JPRB, 0.66101E+01_JPRB, 0.69618E+01_JPRB ,&
&  0.51623E+01_JPRB, 0.44888E+01_JPRB, 0.39170E+01_JPRB, 0.33394E+01_JPRB, 0.26258E+01_JPRB, 0.24129E+01_JPRB, 0.23750E+01_JPRB ,&
&  0.19616E+01_JPRB, 0.16043E+01_JPRB, 0.14348E+01_JPRB, 0.13193E+01_JPRB, 0.12388E+01_JPRB, 0.12073E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.12274E+01_JPRB, 0.12428E+01_JPRB, 0.12248E+01_JPRB, 0.12136E+01_JPRB, 0.12181E+01_JPRB, 0.12208E+01_JPRB, 0.12225E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.12229E+01_JPRB, 0.12441E+01_JPRB, 0.12340E+01_JPRB, 0.12368E+01_JPRB, 0.12521E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.12004E+01_JPRB, 0.11643E+01_JPRB, 0.11223E+01_JPRB, 0.11627E+01_JPRB, 0.11499E+01_JPRB, 0.11173E+01_JPRB ,&
&  0.11399E+01_JPRB, 0.12240E+01_JPRB, 0.11502E+01_JPRB, 0.83174E+00_JPRB, 0.77773E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.83531E+00_JPRB, 0.80622E+00_JPRB, 0.80228E+00_JPRB, 0.79509E+00_JPRB, 0.77982E+00_JPRB, 0.76864E+00_JPRB, 0.76679E+00_JPRB ,&
&  0.74876E+00_JPRB, 0.72015E+00_JPRB, 0.69268E+00_JPRB, 0.69367E+00_JPRB, 0.71143E+00_JPRB, 0.72630E+00_JPRB, 0.71116E+00_JPRB ,&
&  0.68967E+00_JPRB, 0.68435E+00_JPRB, 0.70214E+00_JPRB, 0.71196E+00_JPRB, 0.71278E+00_JPRB, 0.72026E+00_JPRB, 0.72359E+00_JPRB ,&
&  0.72165E+00_JPRB, 0.71597E+00_JPRB, 0.71795E+00_JPRB, 0.73875E+00_JPRB, 0.78654E+00_JPRB, 0.89084E+00_JPRB, 0.10681E+01_JPRB ,&
&  0.17198E+01_JPRB, 0.23062E+01_JPRB, 0.30176E+01_JPRB, 0.45509E+01_JPRB, 0.58138E+01_JPRB, 0.56812E+01_JPRB, 0.55289E+01_JPRB ,&
&  0.53201E+01_JPRB, 0.44370E+01_JPRB, 0.35596E+01_JPRB, 0.29271E+01_JPRB, 0.23771E+01_JPRB, 0.19794E+01_JPRB, 0.17193E+01_JPRB ,&
&  0.15802E+01_JPRB, 0.14994E+01_JPRB, 0.14929E+01_JPRB, 0.15479E+01_JPRB, 0.16382E+01_JPRB, 0.17337E+01_JPRB, 0.17921E+01_JPRB ,&
&  0.18401E+01_JPRB, 0.19514E+01_JPRB, 0.21949E+01_JPRB, 0.24883E+01_JPRB, 0.36712E+01_JPRB, 0.56891E+01_JPRB, 0.67441E+01_JPRB ,&
&  0.67095E+01_JPRB, 0.59808E+01_JPRB, 0.49601E+01_JPRB, 0.40352E+01_JPRB, 0.30098E+01_JPRB, 0.30511E+01_JPRB, 0.27260E+01_JPRB ,&
&  0.23162E+01_JPRB, 0.19769E+01_JPRB, 0.17163E+01_JPRB, 0.15343E+01_JPRB, 0.13966E+01_JPRB, 0.12934E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.12211E+01_JPRB, 0.12226E+01_JPRB, 0.12063E+01_JPRB, 0.11869E+01_JPRB, 0.11971E+01_JPRB, 0.12130E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.12223E+01_JPRB, 0.12288E+01_JPRB, 0.12004E+01_JPRB, 0.12226E+01_JPRB, 0.12531E+01_JPRB, 0.12420E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11693E+01_JPRB, 0.11694E+01_JPRB, 0.11531E+01_JPRB, 0.11542E+01_JPRB, 0.11627E+01_JPRB, 0.11427E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.11055E+01_JPRB, 0.10153E+01_JPRB, 0.91241E+00_JPRB, 0.86758E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.90714E+00_JPRB, 0.87725E+00_JPRB, 0.85787E+00_JPRB, 0.83946E+00_JPRB, 0.83082E+00_JPRB, 0.83076E+00_JPRB, 0.82550E+00_JPRB ,&
&  0.81023E+00_JPRB, 0.77969E+00_JPRB, 0.73938E+00_JPRB, 0.70340E+00_JPRB, 0.70188E+00_JPRB, 0.69832E+00_JPRB, 0.69595E+00_JPRB ,&
&  0.67844E+00_JPRB, 0.67220E+00_JPRB, 0.68029E+00_JPRB, 0.68343E+00_JPRB, 0.68275E+00_JPRB, 0.68142E+00_JPRB, 0.68116E+00_JPRB ,&
&  0.68653E+00_JPRB, 0.69068E+00_JPRB, 0.68985E+00_JPRB, 0.69222E+00_JPRB, 0.70979E+00_JPRB, 0.77015E+00_JPRB, 0.90107E+00_JPRB ,&
&  0.12745E+01_JPRB, 0.21119E+01_JPRB, 0.23440E+01_JPRB, 0.53497E+01_JPRB, 0.62880E+01_JPRB, 0.62708E+01_JPRB, 0.56837E+01_JPRB ,&
&  0.50414E+01_JPRB, 0.49055E+01_JPRB, 0.38832E+01_JPRB, 0.29957E+01_JPRB, 0.23257E+01_JPRB, 0.19569E+01_JPRB, 0.16876E+01_JPRB ,&
&  0.15070E+01_JPRB, 0.14136E+01_JPRB, 0.13810E+01_JPRB, 0.13710E+01_JPRB, 0.13686E+01_JPRB, 0.13993E+01_JPRB, 0.14327E+01_JPRB ,&
&  0.14755E+01_JPRB, 0.16070E+01_JPRB, 0.18602E+01_JPRB, 0.20802E+01_JPRB, 0.27860E+01_JPRB, 0.43553E+01_JPRB, 0.57110E+01_JPRB ,&
&  0.63390E+01_JPRB, 0.60018E+01_JPRB, 0.53150E+01_JPRB, 0.43823E+01_JPRB, 0.35599E+01_JPRB, 0.32505E+01_JPRB, 0.29065E+01_JPRB ,&
&  0.25568E+01_JPRB, 0.23134E+01_JPRB, 0.19843E+01_JPRB, 0.17448E+01_JPRB, 0.16165E+01_JPRB, 0.14711E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.12691E+01_JPRB, 0.12334E+01_JPRB, 0.12101E+01_JPRB, 0.11844E+01_JPRB, 0.11798E+01_JPRB, 0.11988E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.12181E+01_JPRB, 0.11667E+01_JPRB, 0.11629E+01_JPRB, 0.11958E+01_JPRB, 0.11957E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11437E+01_JPRB, 0.11452E+01_JPRB, 0.11469E+01_JPRB, 0.11321E+01_JPRB, 0.11037E+01_JPRB, 0.11748E+01_JPRB, 0.11558E+01_JPRB ,&
&  0.11437E+01_JPRB, 0.10580E+01_JPRB, 0.10087E+01_JPRB, 0.96210E+00_JPRB, 0.93580E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.94743E+00_JPRB, 0.92739E+00_JPRB, 0.91027E+00_JPRB, 0.88791E+00_JPRB, 0.87467E+00_JPRB, 0.86885E+00_JPRB, 0.86030E+00_JPRB ,&
&  0.84081E+00_JPRB, 0.81271E+00_JPRB, 0.77854E+00_JPRB, 0.73263E+00_JPRB, 0.70441E+00_JPRB, 0.69741E+00_JPRB, 0.70298E+00_JPRB ,&
&  0.68959E+00_JPRB, 0.66993E+00_JPRB, 0.67011E+00_JPRB, 0.67498E+00_JPRB, 0.67485E+00_JPRB, 0.66705E+00_JPRB, 0.65903E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.67401E+00_JPRB, 0.67970E+00_JPRB, 0.68116E+00_JPRB, 0.68543E+00_JPRB, 0.70895E+00_JPRB, 0.81123E+00_JPRB ,&
&  0.11459E+01_JPRB, 0.29109E+01_JPRB, 0.25266E+01_JPRB, 0.50185E+01_JPRB, 0.54375E+01_JPRB, 0.54081E+01_JPRB, 0.50184E+01_JPRB ,&
&  0.44926E+01_JPRB, 0.40820E+01_JPRB, 0.33110E+01_JPRB, 0.27308E+01_JPRB, 0.22139E+01_JPRB, 0.18313E+01_JPRB, 0.15838E+01_JPRB ,&
&  0.14281E+01_JPRB, 0.13169E+01_JPRB, 0.12524E+01_JPRB, 0.12303E+01_JPRB, 0.12230E+01_JPRB, 0.12218E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.13584E+01_JPRB, 0.16046E+01_JPRB, 0.21406E+01_JPRB, 0.25671E+01_JPRB, 0.36703E+01_JPRB, 0.52946E+01_JPRB ,&
&  0.60938E+01_JPRB, 0.59656E+01_JPRB, 0.50312E+01_JPRB, 0.41972E+01_JPRB, 0.35635E+01_JPRB, 0.31350E+01_JPRB, 0.27756E+01_JPRB ,&
&  0.25314E+01_JPRB, 0.23493E+01_JPRB, 0.21541E+01_JPRB, 0.19153E+01_JPRB, 0.17441E+01_JPRB, 0.16216E+01_JPRB, 0.15015E+01_JPRB ,&
&  0.13798E+01_JPRB, 0.12804E+01_JPRB, 0.12230E+01_JPRB, 0.11782E+01_JPRB, 0.11540E+01_JPRB, 0.11649E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.11790E+01_JPRB, 0.11779E+01_JPRB, 0.11282E+01_JPRB, 0.11162E+01_JPRB, 0.11443E+01_JPRB, 0.11499E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.11225E+01_JPRB, 0.11303E+01_JPRB, 0.11307E+01_JPRB, 0.11438E+01_JPRB, 0.11018E+01_JPRB, 0.11877E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.10655E+01_JPRB, 0.10163E+01_JPRB, 0.99948E+00_JPRB, 0.97745E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.96609E+00_JPRB, 0.94638E+00_JPRB, 0.93178E+00_JPRB, 0.92010E+00_JPRB, 0.91033E+00_JPRB, 0.89416E+00_JPRB, 0.87571E+00_JPRB ,&
&  0.85885E+00_JPRB, 0.83959E+00_JPRB, 0.80679E+00_JPRB, 0.75866E+00_JPRB, 0.71821E+00_JPRB, 0.69748E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.69831E+00_JPRB, 0.68608E+00_JPRB, 0.67560E+00_JPRB, 0.67397E+00_JPRB, 0.67281E+00_JPRB, 0.66418E+00_JPRB, 0.65339E+00_JPRB ,&
&  0.64710E+00_JPRB, 0.65036E+00_JPRB, 0.65612E+00_JPRB, 0.66238E+00_JPRB, 0.66721E+00_JPRB, 0.67617E+00_JPRB, 0.74382E+00_JPRB ,&
&  0.10924E+01_JPRB, 0.12081E+01_JPRB, 0.30863E+01_JPRB, 0.42774E+01_JPRB, 0.43302E+01_JPRB, 0.41808E+01_JPRB, 0.42047E+01_JPRB ,&
&  0.38639E+01_JPRB, 0.33507E+01_JPRB, 0.27372E+01_JPRB, 0.22395E+01_JPRB, 0.18702E+01_JPRB, 0.16276E+01_JPRB, 0.14487E+01_JPRB ,&
&  0.13117E+01_JPRB, 0.12109E+01_JPRB, 0.11387E+01_JPRB, 0.10980E+01_JPRB, 0.11019E+01_JPRB, 0.11167E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11584E+01_JPRB, 0.12020E+01_JPRB, 0.13318E+01_JPRB, 0.17521E+01_JPRB, 0.26836E+01_JPRB, 0.34973E+01_JPRB, 0.47876E+01_JPRB ,&
&  0.58279E+01_JPRB, 0.53672E+01_JPRB, 0.43807E+01_JPRB, 0.36810E+01_JPRB, 0.31651E+01_JPRB, 0.28006E+01_JPRB, 0.25243E+01_JPRB ,&
&  0.23155E+01_JPRB, 0.21655E+01_JPRB, 0.20389E+01_JPRB, 0.18973E+01_JPRB, 0.17770E+01_JPRB, 0.16853E+01_JPRB, 0.15988E+01_JPRB ,&
&  0.14967E+01_JPRB, 0.13945E+01_JPRB, 0.12978E+01_JPRB, 0.12107E+01_JPRB, 0.11420E+01_JPRB, 0.11265E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.11160E+01_JPRB, 0.10936E+01_JPRB, 0.10936E+01_JPRB, 0.11280E+01_JPRB, 0.11324E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.11126E+01_JPRB, 0.11057E+01_JPRB, 0.10969E+01_JPRB, 0.10378E+01_JPRB, 0.11273E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.10687E+01_JPRB, 0.10242E+01_JPRB, 0.10107E+01_JPRB, 0.99373E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.96514E+00_JPRB, 0.94266E+00_JPRB, 0.92879E+00_JPRB, 0.91914E+00_JPRB, 0.91045E+00_JPRB, 0.90153E+00_JPRB, 0.88109E+00_JPRB ,&
&  0.85868E+00_JPRB, 0.84594E+00_JPRB, 0.82795E+00_JPRB, 0.80284E+00_JPRB, 0.76381E+00_JPRB, 0.73196E+00_JPRB, 0.72161E+00_JPRB ,&
&  0.71615E+00_JPRB, 0.70703E+00_JPRB, 0.68976E+00_JPRB, 0.67817E+00_JPRB, 0.66772E+00_JPRB, 0.65615E+00_JPRB, 0.64888E+00_JPRB ,&
&  0.63699E+00_JPRB, 0.63266E+00_JPRB, 0.63650E+00_JPRB, 0.64449E+00_JPRB, 0.65147E+00_JPRB, 0.65973E+00_JPRB, 0.68121E+00_JPRB ,&
&  0.97371E+00_JPRB, 0.22938E+01_JPRB, 0.30669E+01_JPRB, 0.35131E+01_JPRB, 0.35270E+01_JPRB, 0.32532E+01_JPRB, 0.32493E+01_JPRB ,&
&  0.28925E+01_JPRB, 0.25221E+01_JPRB, 0.21480E+01_JPRB, 0.18038E+01_JPRB, 0.15283E+01_JPRB, 0.13782E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.11306E+01_JPRB, 0.10663E+01_JPRB, 0.10115E+01_JPRB, 0.99191E+00_JPRB, 0.10139E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.10690E+01_JPRB, 0.10864E+01_JPRB, 0.11447E+01_JPRB, 0.13929E+01_JPRB, 0.21619E+01_JPRB, 0.25761E+01_JPRB, 0.37751E+01_JPRB ,&
&  0.44763E+01_JPRB, 0.40395E+01_JPRB, 0.34011E+01_JPRB, 0.29276E+01_JPRB, 0.25361E+01_JPRB, 0.22353E+01_JPRB, 0.20179E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.18060E+01_JPRB, 0.17300E+01_JPRB, 0.16555E+01_JPRB, 0.15873E+01_JPRB, 0.15350E+01_JPRB, 0.14946E+01_JPRB ,&
&  0.14538E+01_JPRB, 0.13986E+01_JPRB, 0.13316E+01_JPRB, 0.12513E+01_JPRB, 0.11702E+01_JPRB, 0.11207E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10619E+01_JPRB, 0.10593E+01_JPRB, 0.10665E+01_JPRB, 0.11000E+01_JPRB, 0.11160E+01_JPRB, 0.11150E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.10643E+01_JPRB, 0.10485E+01_JPRB, 0.10137E+01_JPRB, 0.10194E+01_JPRB, 0.10692E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.10113E+01_JPRB, 0.10002E+01_JPRB, 0.98595E+00_JPRB, 0.97865E+00_JPRB, 0.99142E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.97371E+00_JPRB, 0.94258E+00_JPRB, 0.92078E+00_JPRB, 0.90625E+00_JPRB, 0.89733E+00_JPRB, 0.88739E+00_JPRB, 0.87039E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.83277E+00_JPRB, 0.81741E+00_JPRB, 0.80610E+00_JPRB, 0.78069E+00_JPRB, 0.73691E+00_JPRB, 0.70241E+00_JPRB ,&
&  0.69121E+00_JPRB, 0.68260E+00_JPRB, 0.67430E+00_JPRB, 0.66050E+00_JPRB, 0.64953E+00_JPRB, 0.64455E+00_JPRB, 0.63933E+00_JPRB ,&
&  0.63398E+00_JPRB, 0.62896E+00_JPRB, 0.63076E+00_JPRB, 0.63599E+00_JPRB, 0.64036E+00_JPRB, 0.64428E+00_JPRB, 0.65427E+00_JPRB ,&
&  0.80072E+00_JPRB, 0.16033E+01_JPRB, 0.19824E+01_JPRB, 0.25605E+01_JPRB, 0.27598E+01_JPRB, 0.25441E+01_JPRB, 0.24059E+01_JPRB ,&
&  0.22051E+01_JPRB, 0.19732E+01_JPRB, 0.17618E+01_JPRB, 0.15639E+01_JPRB, 0.13453E+01_JPRB, 0.12121E+01_JPRB, 0.11472E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.10499E+01_JPRB, 0.99922E+00_JPRB, 0.94884E+00_JPRB, 0.91617E+00_JPRB, 0.92363E+00_JPRB, 0.94577E+00_JPRB ,&
&  0.95039E+00_JPRB, 0.95126E+00_JPRB, 0.96308E+00_JPRB, 0.10594E+01_JPRB, 0.14464E+01_JPRB, 0.18334E+01_JPRB, 0.23713E+01_JPRB ,&
&  0.24212E+01_JPRB, 0.25432E+01_JPRB, 0.23430E+01_JPRB, 0.21095E+01_JPRB, 0.19042E+01_JPRB, 0.17443E+01_JPRB, 0.16236E+01_JPRB ,&
&  0.15427E+01_JPRB, 0.14772E+01_JPRB, 0.14266E+01_JPRB, 0.13854E+01_JPRB, 0.13528E+01_JPRB, 0.13150E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.12654E+01_JPRB, 0.12348E+01_JPRB, 0.11959E+01_JPRB, 0.11480E+01_JPRB, 0.11069E+01_JPRB, 0.10747E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.10154E+01_JPRB, 0.10064E+01_JPRB, 0.10179E+01_JPRB, 0.10300E+01_JPRB, 0.10388E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.10190E+01_JPRB, 0.99689E+00_JPRB, 0.96753E+00_JPRB, 0.97289E+00_JPRB, 0.99907E+00_JPRB, 0.97120E+00_JPRB ,&
&  0.94914E+00_JPRB, 0.93334E+00_JPRB, 0.93244E+00_JPRB, 0.94230E+00_JPRB, 0.10001E+01_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.99354E+00_JPRB, 0.95822E+00_JPRB, 0.91971E+00_JPRB, 0.89374E+00_JPRB, 0.87479E+00_JPRB, 0.85810E+00_JPRB, 0.83748E+00_JPRB ,&
&  0.81781E+00_JPRB, 0.79941E+00_JPRB, 0.78413E+00_JPRB, 0.77287E+00_JPRB, 0.76080E+00_JPRB, 0.73079E+00_JPRB, 0.68912E+00_JPRB ,&
&  0.66191E+00_JPRB, 0.65288E+00_JPRB, 0.64717E+00_JPRB, 0.63660E+00_JPRB, 0.62660E+00_JPRB, 0.62231E+00_JPRB, 0.61875E+00_JPRB ,&
&  0.61881E+00_JPRB, 0.62006E+00_JPRB, 0.62416E+00_JPRB, 0.62583E+00_JPRB, 0.62263E+00_JPRB, 0.62880E+00_JPRB, 0.63041E+00_JPRB ,&
&  0.73622E+00_JPRB, 0.10406E+01_JPRB, 0.13339E+01_JPRB, 0.16731E+01_JPRB, 0.19296E+01_JPRB, 0.19960E+01_JPRB, 0.19261E+01_JPRB ,&
&  0.18274E+01_JPRB, 0.17260E+01_JPRB, 0.15824E+01_JPRB, 0.14485E+01_JPRB, 0.12986E+01_JPRB, 0.11422E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.98034E+00_JPRB, 0.94705E+00_JPRB, 0.91300E+00_JPRB, 0.88461E+00_JPRB, 0.87312E+00_JPRB, 0.87296E+00_JPRB ,&
&  0.86692E+00_JPRB, 0.85658E+00_JPRB, 0.84502E+00_JPRB, 0.84981E+00_JPRB, 0.91679E+00_JPRB, 0.11103E+01_JPRB, 0.13643E+01_JPRB ,&
&  0.14724E+01_JPRB, 0.15454E+01_JPRB, 0.15821E+01_JPRB, 0.15645E+01_JPRB, 0.14943E+01_JPRB, 0.13990E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.12295E+01_JPRB, 0.11819E+01_JPRB, 0.11567E+01_JPRB, 0.11476E+01_JPRB, 0.11432E+01_JPRB, 0.11314E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.10869E+01_JPRB, 0.10865E+01_JPRB, 0.10790E+01_JPRB, 0.10671E+01_JPRB, 0.10520E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10356E+01_JPRB, 0.10258E+01_JPRB, 0.10015E+01_JPRB, 0.97844E+00_JPRB, 0.97613E+00_JPRB, 0.97372E+00_JPRB, 0.96574E+00_JPRB ,&
&  0.95805E+00_JPRB, 0.96207E+00_JPRB, 0.94750E+00_JPRB, 0.96616E+00_JPRB, 0.99184E+00_JPRB, 0.95288E+00_JPRB, 0.92708E+00_JPRB ,&
&  0.89611E+00_JPRB, 0.87952E+00_JPRB, 0.88422E+00_JPRB, 0.90650E+00_JPRB, 0.97838E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.85514E+00_JPRB, 0.84754E+00_JPRB, 0.83722E+00_JPRB, 0.82726E+00_JPRB, 0.81217E+00_JPRB, 0.79673E+00_JPRB, 0.77782E+00_JPRB ,&
&  0.75983E+00_JPRB, 0.74283E+00_JPRB, 0.72903E+00_JPRB, 0.72460E+00_JPRB, 0.71671E+00_JPRB, 0.70417E+00_JPRB, 0.67476E+00_JPRB ,&
&  0.65033E+00_JPRB, 0.63240E+00_JPRB, 0.62142E+00_JPRB, 0.61573E+00_JPRB, 0.61119E+00_JPRB, 0.60622E+00_JPRB, 0.60392E+00_JPRB ,&
&  0.60447E+00_JPRB, 0.60395E+00_JPRB, 0.60599E+00_JPRB, 0.60454E+00_JPRB, 0.60590E+00_JPRB, 0.60097E+00_JPRB, 0.59657E+00_JPRB ,&
&  0.65056E+00_JPRB, 0.78105E+00_JPRB, 0.95719E+00_JPRB, 0.11515E+01_JPRB, 0.13701E+01_JPRB, 0.15219E+01_JPRB, 0.16146E+01_JPRB ,&
&  0.16495E+01_JPRB, 0.15676E+01_JPRB, 0.14466E+01_JPRB, 0.13138E+01_JPRB, 0.12103E+01_JPRB, 0.11054E+01_JPRB, 0.10018E+01_JPRB ,&
&  0.94923E+00_JPRB, 0.91840E+00_JPRB, 0.89334E+00_JPRB, 0.87373E+00_JPRB, 0.85777E+00_JPRB, 0.84738E+00_JPRB, 0.83864E+00_JPRB ,&
&  0.82313E+00_JPRB, 0.80436E+00_JPRB, 0.78278E+00_JPRB, 0.77319E+00_JPRB, 0.77838E+00_JPRB, 0.82888E+00_JPRB, 0.95064E+00_JPRB ,&
&  0.10141E+01_JPRB, 0.10457E+01_JPRB, 0.10699E+01_JPRB, 0.11088E+01_JPRB, 0.11312E+01_JPRB, 0.11224E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.97445E+00_JPRB, 0.95301E+00_JPRB, 0.94569E+00_JPRB, 0.94859E+00_JPRB, 0.95202E+00_JPRB, 0.95343E+00_JPRB ,&
&  0.94281E+00_JPRB, 0.93110E+00_JPRB, 0.92698E+00_JPRB, 0.92773E+00_JPRB, 0.93264E+00_JPRB, 0.93670E+00_JPRB, 0.94450E+00_JPRB ,&
&  0.96114E+00_JPRB, 0.98398E+00_JPRB, 0.10021E+01_JPRB, 0.98652E+00_JPRB, 0.95462E+00_JPRB, 0.92992E+00_JPRB, 0.91360E+00_JPRB ,&
&  0.90375E+00_JPRB, 0.90604E+00_JPRB, 0.91214E+00_JPRB, 0.93717E+00_JPRB, 0.93894E+00_JPRB, 0.91477E+00_JPRB, 0.88111E+00_JPRB ,&
&  0.85253E+00_JPRB, 0.83563E+00_JPRB, 0.83531E+00_JPRB, 0.84416E+00_JPRB, 0.82866E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.76468E+00_JPRB, 0.75940E+00_JPRB, 0.75275E+00_JPRB, 0.74784E+00_JPRB, 0.73838E+00_JPRB, 0.72772E+00_JPRB, 0.71625E+00_JPRB ,&
&  0.70197E+00_JPRB, 0.68791E+00_JPRB, 0.67632E+00_JPRB, 0.67275E+00_JPRB, 0.67152E+00_JPRB, 0.67102E+00_JPRB, 0.66295E+00_JPRB ,&
&  0.64854E+00_JPRB, 0.63488E+00_JPRB, 0.62287E+00_JPRB, 0.61468E+00_JPRB, 0.61077E+00_JPRB, 0.60642E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.59683E+00_JPRB, 0.59545E+00_JPRB, 0.59292E+00_JPRB, 0.59014E+00_JPRB, 0.58624E+00_JPRB, 0.57892E+00_JPRB, 0.57672E+00_JPRB ,&
&  0.59509E+00_JPRB, 0.61213E+00_JPRB, 0.70752E+00_JPRB, 0.78521E+00_JPRB, 0.88241E+00_JPRB, 0.10352E+01_JPRB, 0.12065E+01_JPRB ,&
&  0.13407E+01_JPRB, 0.13439E+01_JPRB, 0.12884E+01_JPRB, 0.11941E+01_JPRB, 0.11260E+01_JPRB, 0.10772E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.94276E+00_JPRB, 0.89676E+00_JPRB, 0.86050E+00_JPRB, 0.83199E+00_JPRB, 0.81094E+00_JPRB, 0.79280E+00_JPRB, 0.77541E+00_JPRB ,&
&  0.75417E+00_JPRB, 0.73328E+00_JPRB, 0.71654E+00_JPRB, 0.70905E+00_JPRB, 0.71215E+00_JPRB, 0.72246E+00_JPRB, 0.75860E+00_JPRB ,&
&  0.79216E+00_JPRB, 0.80935E+00_JPRB, 0.80947E+00_JPRB, 0.82561E+00_JPRB, 0.85443E+00_JPRB, 0.87879E+00_JPRB, 0.88174E+00_JPRB ,&
&  0.85911E+00_JPRB, 0.82607E+00_JPRB, 0.79806E+00_JPRB, 0.77780E+00_JPRB, 0.76761E+00_JPRB, 0.76940E+00_JPRB, 0.77989E+00_JPRB ,&
&  0.78453E+00_JPRB, 0.78241E+00_JPRB, 0.78176E+00_JPRB, 0.78587E+00_JPRB, 0.79312E+00_JPRB, 0.80602E+00_JPRB, 0.82368E+00_JPRB ,&
&  0.84560E+00_JPRB, 0.87135E+00_JPRB, 0.90052E+00_JPRB, 0.92282E+00_JPRB, 0.92754E+00_JPRB, 0.92173E+00_JPRB, 0.90902E+00_JPRB ,&
&  0.89501E+00_JPRB, 0.88514E+00_JPRB, 0.87472E+00_JPRB, 0.86731E+00_JPRB, 0.88997E+00_JPRB, 0.88053E+00_JPRB, 0.85059E+00_JPRB ,&
&  0.82356E+00_JPRB, 0.79870E+00_JPRB, 0.78335E+00_JPRB, 0.72563E+00_JPRB, 0.76124E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.68804E+00_JPRB, 0.68303E+00_JPRB, 0.67561E+00_JPRB, 0.66931E+00_JPRB, 0.66508E+00_JPRB, 0.65904E+00_JPRB, 0.65416E+00_JPRB ,&
&  0.64743E+00_JPRB, 0.64190E+00_JPRB, 0.63295E+00_JPRB, 0.62706E+00_JPRB, 0.62455E+00_JPRB, 0.62518E+00_JPRB, 0.62582E+00_JPRB ,&
&  0.62175E+00_JPRB, 0.61396E+00_JPRB, 0.60571E+00_JPRB, 0.59897E+00_JPRB, 0.59311E+00_JPRB, 0.58639E+00_JPRB, 0.57946E+00_JPRB ,&
&  0.57299E+00_JPRB, 0.56738E+00_JPRB, 0.56245E+00_JPRB, 0.56063E+00_JPRB, 0.55890E+00_JPRB, 0.55615E+00_JPRB, 0.55205E+00_JPRB ,&
&  0.55717E+00_JPRB, 0.50957E+00_JPRB, 0.53575E+00_JPRB, 0.56541E+00_JPRB, 0.60575E+00_JPRB, 0.70550E+00_JPRB, 0.84859E+00_JPRB ,&
&  0.98670E+00_JPRB, 0.10751E+01_JPRB, 0.10958E+01_JPRB, 0.10539E+01_JPRB, 0.10074E+01_JPRB, 0.97980E+00_JPRB, 0.95687E+00_JPRB ,&
&  0.91220E+00_JPRB, 0.87146E+00_JPRB, 0.83034E+00_JPRB, 0.78853E+00_JPRB, 0.74988E+00_JPRB, 0.71541E+00_JPRB, 0.68801E+00_JPRB ,&
&  0.66990E+00_JPRB, 0.66018E+00_JPRB, 0.65533E+00_JPRB, 0.65353E+00_JPRB, 0.65499E+00_JPRB, 0.65689E+00_JPRB, 0.65839E+00_JPRB ,&
&  0.66455E+00_JPRB, 0.67738E+00_JPRB, 0.68114E+00_JPRB, 0.68471E+00_JPRB, 0.69672E+00_JPRB, 0.71554E+00_JPRB, 0.72395E+00_JPRB ,&
&  0.72105E+00_JPRB, 0.70982E+00_JPRB, 0.69531E+00_JPRB, 0.68046E+00_JPRB, 0.66789E+00_JPRB, 0.65733E+00_JPRB, 0.65348E+00_JPRB ,&
&  0.65188E+00_JPRB, 0.65344E+00_JPRB, 0.65680E+00_JPRB, 0.66499E+00_JPRB, 0.67553E+00_JPRB, 0.68662E+00_JPRB, 0.69751E+00_JPRB ,&
&  0.70835E+00_JPRB, 0.72037E+00_JPRB, 0.73811E+00_JPRB, 0.75905E+00_JPRB, 0.77726E+00_JPRB, 0.78731E+00_JPRB, 0.79047E+00_JPRB ,&
&  0.78878E+00_JPRB, 0.78587E+00_JPRB, 0.77812E+00_JPRB, 0.78159E+00_JPRB, 0.78055E+00_JPRB, 0.78108E+00_JPRB, 0.77077E+00_JPRB ,&
&  0.75622E+00_JPRB, 0.74034E+00_JPRB, 0.65442E+00_JPRB, 0.68593E+00_JPRB, 0.68807E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.63362E+00_JPRB, 0.62476E+00_JPRB, 0.61637E+00_JPRB, 0.60829E+00_JPRB, 0.60242E+00_JPRB, 0.59859E+00_JPRB, 0.59727E+00_JPRB ,&
&  0.59867E+00_JPRB, 0.60067E+00_JPRB, 0.59930E+00_JPRB, 0.59294E+00_JPRB, 0.58650E+00_JPRB, 0.58275E+00_JPRB, 0.58167E+00_JPRB ,&
&  0.57995E+00_JPRB, 0.57596E+00_JPRB, 0.57053E+00_JPRB, 0.56514E+00_JPRB, 0.55873E+00_JPRB, 0.55296E+00_JPRB, 0.54766E+00_JPRB ,&
&  0.54293E+00_JPRB, 0.53992E+00_JPRB, 0.53728E+00_JPRB, 0.53507E+00_JPRB, 0.53407E+00_JPRB, 0.53200E+00_JPRB, 0.53373E+00_JPRB ,&
&  0.50446E+00_JPRB, 0.47387E+00_JPRB, 0.48945E+00_JPRB, 0.50394E+00_JPRB, 0.51896E+00_JPRB, 0.56009E+00_JPRB, 0.63368E+00_JPRB ,&
&  0.73537E+00_JPRB, 0.82674E+00_JPRB, 0.88429E+00_JPRB, 0.88359E+00_JPRB, 0.86052E+00_JPRB, 0.84785E+00_JPRB, 0.84225E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.79195E+00_JPRB, 0.76601E+00_JPRB, 0.73533E+00_JPRB, 0.70264E+00_JPRB, 0.67408E+00_JPRB, 0.65190E+00_JPRB ,&
&  0.63542E+00_JPRB, 0.62581E+00_JPRB, 0.62154E+00_JPRB, 0.61802E+00_JPRB, 0.61418E+00_JPRB, 0.61158E+00_JPRB, 0.60758E+00_JPRB ,&
&  0.60537E+00_JPRB, 0.60520E+00_JPRB, 0.60853E+00_JPRB, 0.61021E+00_JPRB, 0.61351E+00_JPRB, 0.61592E+00_JPRB, 0.61535E+00_JPRB ,&
&  0.61112E+00_JPRB, 0.60691E+00_JPRB, 0.60487E+00_JPRB, 0.59968E+00_JPRB, 0.58911E+00_JPRB, 0.57849E+00_JPRB, 0.57329E+00_JPRB ,&
&  0.57222E+00_JPRB, 0.57230E+00_JPRB, 0.57313E+00_JPRB, 0.57658E+00_JPRB, 0.58235E+00_JPRB, 0.58941E+00_JPRB, 0.59693E+00_JPRB ,&
&  0.60471E+00_JPRB, 0.61177E+00_JPRB, 0.61879E+00_JPRB, 0.62783E+00_JPRB, 0.64106E+00_JPRB, 0.65426E+00_JPRB, 0.66557E+00_JPRB ,&
&  0.67429E+00_JPRB, 0.68068E+00_JPRB, 0.68564E+00_JPRB, 0.69267E+00_JPRB, 0.69980E+00_JPRB, 0.70355E+00_JPRB, 0.70021E+00_JPRB ,&
&  0.69031E+00_JPRB, 0.67513E+00_JPRB, 0.66538E+00_JPRB, 0.65217E+00_JPRB, 0.64130E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.56834E+00_JPRB, 0.56251E+00_JPRB, 0.55580E+00_JPRB, 0.54909E+00_JPRB, 0.54287E+00_JPRB, 0.54009E+00_JPRB, 0.53978E+00_JPRB ,&
&  0.54286E+00_JPRB, 0.54499E+00_JPRB, 0.54833E+00_JPRB, 0.54730E+00_JPRB, 0.54454E+00_JPRB, 0.54189E+00_JPRB, 0.53922E+00_JPRB ,&
&  0.53625E+00_JPRB, 0.53219E+00_JPRB, 0.52802E+00_JPRB, 0.52473E+00_JPRB, 0.52111E+00_JPRB, 0.51621E+00_JPRB, 0.51150E+00_JPRB ,&
&  0.50756E+00_JPRB, 0.50513E+00_JPRB, 0.50325E+00_JPRB, 0.50148E+00_JPRB, 0.49959E+00_JPRB, 0.49897E+00_JPRB, 0.49732E+00_JPRB ,&
&  0.49950E+00_JPRB, 0.47059E+00_JPRB, 0.48390E+00_JPRB, 0.48856E+00_JPRB, 0.49657E+00_JPRB, 0.51433E+00_JPRB, 0.53958E+00_JPRB ,&
&  0.58047E+00_JPRB, 0.63692E+00_JPRB, 0.68770E+00_JPRB, 0.70976E+00_JPRB, 0.70649E+00_JPRB, 0.70492E+00_JPRB, 0.71189E+00_JPRB ,&
&  0.71047E+00_JPRB, 0.70184E+00_JPRB, 0.69529E+00_JPRB, 0.68450E+00_JPRB, 0.66700E+00_JPRB, 0.64436E+00_JPRB, 0.62441E+00_JPRB ,&
&  0.61186E+00_JPRB, 0.60429E+00_JPRB, 0.59723E+00_JPRB, 0.58930E+00_JPRB, 0.57868E+00_JPRB, 0.56778E+00_JPRB, 0.55787E+00_JPRB ,&
&  0.54898E+00_JPRB, 0.53948E+00_JPRB, 0.53357E+00_JPRB, 0.53068E+00_JPRB, 0.52792E+00_JPRB, 0.52685E+00_JPRB, 0.52511E+00_JPRB ,&
&  0.52135E+00_JPRB, 0.51660E+00_JPRB, 0.51389E+00_JPRB, 0.51332E+00_JPRB, 0.51191E+00_JPRB, 0.51054E+00_JPRB, 0.50898E+00_JPRB ,&
&  0.50943E+00_JPRB, 0.51005E+00_JPRB, 0.50951E+00_JPRB, 0.50942E+00_JPRB, 0.51097E+00_JPRB, 0.51488E+00_JPRB, 0.51975E+00_JPRB ,&
&  0.52534E+00_JPRB, 0.53156E+00_JPRB, 0.53741E+00_JPRB, 0.54300E+00_JPRB, 0.55079E+00_JPRB, 0.55899E+00_JPRB, 0.56908E+00_JPRB ,&
&  0.58005E+00_JPRB, 0.58972E+00_JPRB, 0.59812E+00_JPRB, 0.60568E+00_JPRB, 0.61109E+00_JPRB, 0.61211E+00_JPRB, 0.61093E+00_JPRB ,&
&  0.60782E+00_JPRB, 0.59861E+00_JPRB, 0.59021E+00_JPRB, 0.58111E+00_JPRB, 0.57304E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.51783E+00_JPRB, 0.51200E+00_JPRB, 0.50578E+00_JPRB, 0.50089E+00_JPRB, 0.49682E+00_JPRB, 0.49300E+00_JPRB, 0.49165E+00_JPRB ,&
&  0.49274E+00_JPRB, 0.49560E+00_JPRB, 0.49976E+00_JPRB, 0.50259E+00_JPRB, 0.50411E+00_JPRB, 0.50543E+00_JPRB, 0.50523E+00_JPRB ,&
&  0.50415E+00_JPRB, 0.50208E+00_JPRB, 0.49912E+00_JPRB, 0.49517E+00_JPRB, 0.49071E+00_JPRB, 0.48646E+00_JPRB, 0.48115E+00_JPRB ,&
&  0.47709E+00_JPRB, 0.47424E+00_JPRB, 0.47223E+00_JPRB, 0.47096E+00_JPRB, 0.47098E+00_JPRB, 0.47115E+00_JPRB, 0.47088E+00_JPRB ,&
&  0.46806E+00_JPRB, 0.47124E+00_JPRB, 0.47883E+00_JPRB, 0.47925E+00_JPRB, 0.48467E+00_JPRB, 0.49691E+00_JPRB, 0.50838E+00_JPRB ,&
&  0.51838E+00_JPRB, 0.53948E+00_JPRB, 0.56973E+00_JPRB, 0.59288E+00_JPRB, 0.59847E+00_JPRB, 0.59978E+00_JPRB, 0.60938E+00_JPRB ,&
&  0.61545E+00_JPRB, 0.61474E+00_JPRB, 0.61627E+00_JPRB, 0.61636E+00_JPRB, 0.61287E+00_JPRB, 0.60720E+00_JPRB, 0.59742E+00_JPRB ,&
&  0.58648E+00_JPRB, 0.57658E+00_JPRB, 0.56583E+00_JPRB, 0.55472E+00_JPRB, 0.54097E+00_JPRB, 0.52632E+00_JPRB, 0.51123E+00_JPRB ,&
&  0.49645E+00_JPRB, 0.48568E+00_JPRB, 0.47852E+00_JPRB, 0.47264E+00_JPRB, 0.46747E+00_JPRB, 0.46447E+00_JPRB, 0.46122E+00_JPRB ,&
&  0.45911E+00_JPRB, 0.45839E+00_JPRB, 0.45642E+00_JPRB, 0.45543E+00_JPRB, 0.45645E+00_JPRB, 0.45763E+00_JPRB, 0.46041E+00_JPRB ,&
&  0.46168E+00_JPRB, 0.46261E+00_JPRB, 0.46336E+00_JPRB, 0.46321E+00_JPRB, 0.46222E+00_JPRB, 0.46337E+00_JPRB, 0.46624E+00_JPRB ,&
&  0.46955E+00_JPRB, 0.47459E+00_JPRB, 0.48018E+00_JPRB, 0.48507E+00_JPRB, 0.48969E+00_JPRB, 0.49673E+00_JPRB, 0.50575E+00_JPRB ,&
&  0.51584E+00_JPRB, 0.52679E+00_JPRB, 0.53648E+00_JPRB, 0.54394E+00_JPRB, 0.54661E+00_JPRB, 0.54672E+00_JPRB, 0.54620E+00_JPRB ,&
&  0.54376E+00_JPRB, 0.53684E+00_JPRB, 0.53225E+00_JPRB, 0.52824E+00_JPRB, 0.52316E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.45921E+00_JPRB, 0.45659E+00_JPRB, 0.45441E+00_JPRB, 0.45328E+00_JPRB, 0.45270E+00_JPRB, 0.45269E+00_JPRB, 0.45362E+00_JPRB ,&
&  0.45577E+00_JPRB, 0.45792E+00_JPRB, 0.46293E+00_JPRB, 0.46617E+00_JPRB, 0.46807E+00_JPRB, 0.47023E+00_JPRB, 0.47157E+00_JPRB ,&
&  0.47066E+00_JPRB, 0.46968E+00_JPRB, 0.46815E+00_JPRB, 0.46518E+00_JPRB, 0.46049E+00_JPRB, 0.45730E+00_JPRB, 0.45411E+00_JPRB ,&
&  0.44971E+00_JPRB, 0.44466E+00_JPRB, 0.44006E+00_JPRB, 0.43701E+00_JPRB, 0.43581E+00_JPRB, 0.43415E+00_JPRB, 0.43353E+00_JPRB ,&
&  0.43182E+00_JPRB, 0.43978E+00_JPRB, 0.42933E+00_JPRB, 0.43908E+00_JPRB, 0.44732E+00_JPRB, 0.45560E+00_JPRB, 0.46452E+00_JPRB ,&
&  0.47076E+00_JPRB, 0.47882E+00_JPRB, 0.49451E+00_JPRB, 0.50990E+00_JPRB, 0.51697E+00_JPRB, 0.51664E+00_JPRB, 0.51997E+00_JPRB ,&
&  0.52258E+00_JPRB, 0.51950E+00_JPRB, 0.51969E+00_JPRB, 0.52407E+00_JPRB, 0.52854E+00_JPRB, 0.53000E+00_JPRB, 0.52987E+00_JPRB ,&
&  0.52947E+00_JPRB, 0.52362E+00_JPRB, 0.51275E+00_JPRB, 0.50075E+00_JPRB, 0.48958E+00_JPRB, 0.47779E+00_JPRB, 0.46428E+00_JPRB ,&
&  0.44991E+00_JPRB, 0.43829E+00_JPRB, 0.42870E+00_JPRB, 0.42202E+00_JPRB, 0.41790E+00_JPRB, 0.41436E+00_JPRB, 0.41131E+00_JPRB ,&
&  0.40999E+00_JPRB, 0.40960E+00_JPRB, 0.40886E+00_JPRB, 0.40802E+00_JPRB, 0.40872E+00_JPRB, 0.41016E+00_JPRB, 0.41447E+00_JPRB ,&
&  0.41950E+00_JPRB, 0.42273E+00_JPRB, 0.42480E+00_JPRB, 0.42641E+00_JPRB, 0.42558E+00_JPRB, 0.42655E+00_JPRB, 0.42874E+00_JPRB ,&
&  0.43108E+00_JPRB, 0.43401E+00_JPRB, 0.43613E+00_JPRB, 0.43836E+00_JPRB, 0.44133E+00_JPRB, 0.44574E+00_JPRB, 0.45248E+00_JPRB ,&
&  0.45936E+00_JPRB, 0.46678E+00_JPRB, 0.47464E+00_JPRB, 0.48099E+00_JPRB, 0.48472E+00_JPRB, 0.48581E+00_JPRB, 0.48588E+00_JPRB ,&
&  0.48362E+00_JPRB, 0.47803E+00_JPRB, 0.47200E+00_JPRB, 0.46713E+00_JPRB, 0.46285E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.41515E+00_JPRB, 0.41516E+00_JPRB, 0.41621E+00_JPRB, 0.41641E+00_JPRB, 0.41625E+00_JPRB, 0.41728E+00_JPRB, 0.42072E+00_JPRB ,&
&  0.42400E+00_JPRB, 0.42655E+00_JPRB, 0.42927E+00_JPRB, 0.43143E+00_JPRB, 0.43316E+00_JPRB, 0.43610E+00_JPRB, 0.43840E+00_JPRB ,&
&  0.43925E+00_JPRB, 0.43844E+00_JPRB, 0.43708E+00_JPRB, 0.43502E+00_JPRB, 0.43203E+00_JPRB, 0.42861E+00_JPRB, 0.42550E+00_JPRB ,&
&  0.42152E+00_JPRB, 0.41697E+00_JPRB, 0.41240E+00_JPRB, 0.40956E+00_JPRB, 0.40796E+00_JPRB, 0.40676E+00_JPRB, 0.40666E+00_JPRB ,&
&  0.40645E+00_JPRB, 0.40903E+00_JPRB, 0.41217E+00_JPRB, 0.41609E+00_JPRB, 0.41976E+00_JPRB, 0.42444E+00_JPRB, 0.42945E+00_JPRB ,&
&  0.43411E+00_JPRB, 0.43685E+00_JPRB, 0.44100E+00_JPRB, 0.44352E+00_JPRB, 0.45121E+00_JPRB, 0.45729E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46075E+00_JPRB, 0.45363E+00_JPRB, 0.44882E+00_JPRB, 0.44971E+00_JPRB, 0.45306E+00_JPRB, 0.45665E+00_JPRB, 0.46014E+00_JPRB ,&
&  0.46168E+00_JPRB, 0.46025E+00_JPRB, 0.45488E+00_JPRB, 0.44698E+00_JPRB, 0.43842E+00_JPRB, 0.42908E+00_JPRB, 0.41741E+00_JPRB ,&
&  0.40669E+00_JPRB, 0.39796E+00_JPRB, 0.39080E+00_JPRB, 0.38450E+00_JPRB, 0.37991E+00_JPRB, 0.37525E+00_JPRB, 0.37338E+00_JPRB ,&
&  0.37347E+00_JPRB, 0.37553E+00_JPRB, 0.37660E+00_JPRB, 0.37577E+00_JPRB, 0.37450E+00_JPRB, 0.37471E+00_JPRB, 0.37806E+00_JPRB ,&
&  0.38400E+00_JPRB, 0.38878E+00_JPRB, 0.39104E+00_JPRB, 0.39181E+00_JPRB, 0.39182E+00_JPRB, 0.39373E+00_JPRB, 0.39679E+00_JPRB ,&
&  0.39858E+00_JPRB, 0.39955E+00_JPRB, 0.40075E+00_JPRB, 0.40057E+00_JPRB, 0.40248E+00_JPRB, 0.40586E+00_JPRB, 0.40984E+00_JPRB ,&
&  0.41389E+00_JPRB, 0.41896E+00_JPRB, 0.42398E+00_JPRB, 0.42793E+00_JPRB, 0.43118E+00_JPRB, 0.43130E+00_JPRB, 0.43119E+00_JPRB ,&
&  0.43053E+00_JPRB, 0.42804E+00_JPRB, 0.42522E+00_JPRB, 0.42166E+00_JPRB, 0.41767E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.37891E+00_JPRB, 0.38208E+00_JPRB, 0.38601E+00_JPRB, 0.38662E+00_JPRB, 0.38667E+00_JPRB, 0.38804E+00_JPRB, 0.39142E+00_JPRB ,&
&  0.39411E+00_JPRB, 0.39542E+00_JPRB, 0.39629E+00_JPRB, 0.39710E+00_JPRB, 0.39867E+00_JPRB, 0.40155E+00_JPRB, 0.40326E+00_JPRB ,&
&  0.40318E+00_JPRB, 0.40267E+00_JPRB, 0.40174E+00_JPRB, 0.40073E+00_JPRB, 0.39863E+00_JPRB, 0.39556E+00_JPRB, 0.39270E+00_JPRB ,&
&  0.38885E+00_JPRB, 0.38512E+00_JPRB, 0.38198E+00_JPRB, 0.37923E+00_JPRB, 0.37717E+00_JPRB, 0.37621E+00_JPRB, 0.37704E+00_JPRB ,&
&  0.37832E+00_JPRB, 0.38031E+00_JPRB, 0.38322E+00_JPRB, 0.38686E+00_JPRB, 0.39041E+00_JPRB, 0.39435E+00_JPRB, 0.39786E+00_JPRB ,&
&  0.40062E+00_JPRB, 0.40322E+00_JPRB, 0.40565E+00_JPRB, 0.40480E+00_JPRB, 0.40436E+00_JPRB, 0.40886E+00_JPRB, 0.41411E+00_JPRB ,&
&  0.41337E+00_JPRB, 0.40818E+00_JPRB, 0.40186E+00_JPRB, 0.39887E+00_JPRB, 0.39841E+00_JPRB, 0.39761E+00_JPRB, 0.39633E+00_JPRB ,&
&  0.39543E+00_JPRB, 0.39421E+00_JPRB, 0.39173E+00_JPRB, 0.38779E+00_JPRB, 0.38236E+00_JPRB, 0.37577E+00_JPRB, 0.36958E+00_JPRB ,&
&  0.36564E+00_JPRB, 0.36281E+00_JPRB, 0.36194E+00_JPRB, 0.35877E+00_JPRB, 0.35312E+00_JPRB, 0.34746E+00_JPRB, 0.34583E+00_JPRB ,&
&  0.34574E+00_JPRB, 0.34705E+00_JPRB, 0.34784E+00_JPRB, 0.34727E+00_JPRB, 0.34762E+00_JPRB, 0.34794E+00_JPRB, 0.34964E+00_JPRB ,&
&  0.35551E+00_JPRB, 0.35999E+00_JPRB, 0.36053E+00_JPRB, 0.35983E+00_JPRB, 0.35800E+00_JPRB, 0.35785E+00_JPRB, 0.35933E+00_JPRB ,&
&  0.36102E+00_JPRB, 0.36213E+00_JPRB, 0.36284E+00_JPRB, 0.36242E+00_JPRB, 0.36245E+00_JPRB, 0.36390E+00_JPRB, 0.36590E+00_JPRB ,&
&  0.36844E+00_JPRB, 0.37159E+00_JPRB, 0.37435E+00_JPRB, 0.37708E+00_JPRB, 0.37961E+00_JPRB, 0.38087E+00_JPRB, 0.38199E+00_JPRB ,&
&  0.38306E+00_JPRB, 0.38292E+00_JPRB, 0.38206E+00_JPRB, 0.38112E+00_JPRB, 0.37994E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.35087E+00_JPRB, 0.35560E+00_JPRB, 0.36129E+00_JPRB, 0.36217E+00_JPRB, 0.36373E+00_JPRB, 0.36583E+00_JPRB, 0.36930E+00_JPRB ,&
&  0.37202E+00_JPRB, 0.37297E+00_JPRB, 0.37201E+00_JPRB, 0.37092E+00_JPRB, 0.37068E+00_JPRB, 0.37119E+00_JPRB, 0.37139E+00_JPRB ,&
&  0.37168E+00_JPRB, 0.37206E+00_JPRB, 0.37198E+00_JPRB, 0.37152E+00_JPRB, 0.36975E+00_JPRB, 0.36721E+00_JPRB, 0.36422E+00_JPRB ,&
&  0.35976E+00_JPRB, 0.35649E+00_JPRB, 0.35484E+00_JPRB, 0.35364E+00_JPRB, 0.35199E+00_JPRB, 0.35191E+00_JPRB, 0.35338E+00_JPRB ,&
&  0.35704E+00_JPRB, 0.35985E+00_JPRB, 0.36356E+00_JPRB, 0.36730E+00_JPRB, 0.36945E+00_JPRB, 0.37111E+00_JPRB, 0.37301E+00_JPRB ,&
&  0.37422E+00_JPRB, 0.37456E+00_JPRB, 0.37490E+00_JPRB, 0.37551E+00_JPRB, 0.37436E+00_JPRB, 0.37708E+00_JPRB, 0.38151E+00_JPRB ,&
&  0.38359E+00_JPRB, 0.38063E+00_JPRB, 0.37554E+00_JPRB, 0.37180E+00_JPRB, 0.36818E+00_JPRB, 0.36531E+00_JPRB, 0.36279E+00_JPRB ,&
&  0.36075E+00_JPRB, 0.35927E+00_JPRB, 0.35832E+00_JPRB, 0.35559E+00_JPRB, 0.34899E+00_JPRB, 0.34266E+00_JPRB, 0.33915E+00_JPRB ,&
&  0.33746E+00_JPRB, 0.33680E+00_JPRB, 0.33794E+00_JPRB, 0.33606E+00_JPRB, 0.33060E+00_JPRB, 0.32584E+00_JPRB, 0.32390E+00_JPRB ,&
&  0.32223E+00_JPRB, 0.32191E+00_JPRB, 0.32330E+00_JPRB, 0.32426E+00_JPRB, 0.32484E+00_JPRB, 0.32474E+00_JPRB, 0.32658E+00_JPRB ,&
&  0.32925E+00_JPRB, 0.33191E+00_JPRB, 0.33210E+00_JPRB, 0.32919E+00_JPRB, 0.32503E+00_JPRB, 0.32237E+00_JPRB, 0.32264E+00_JPRB ,&
&  0.32530E+00_JPRB, 0.32778E+00_JPRB, 0.32902E+00_JPRB, 0.32981E+00_JPRB, 0.33125E+00_JPRB, 0.33399E+00_JPRB, 0.33741E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.34056E+00_JPRB, 0.34266E+00_JPRB, 0.34482E+00_JPRB, 0.34628E+00_JPRB, 0.34711E+00_JPRB, 0.34732E+00_JPRB ,&
&  0.34665E+00_JPRB, 0.34716E+00_JPRB, 0.34850E+00_JPRB, 0.35009E+00_JPRB, 0.35069E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.33059E+00_JPRB, 0.33365E+00_JPRB, 0.33641E+00_JPRB, 0.33763E+00_JPRB, 0.33906E+00_JPRB, 0.34100E+00_JPRB, 0.34461E+00_JPRB ,&
&  0.34907E+00_JPRB, 0.35090E+00_JPRB, 0.34927E+00_JPRB, 0.34629E+00_JPRB, 0.34459E+00_JPRB, 0.34418E+00_JPRB, 0.34447E+00_JPRB ,&
&  0.34472E+00_JPRB, 0.34486E+00_JPRB, 0.34410E+00_JPRB, 0.34206E+00_JPRB, 0.33901E+00_JPRB, 0.33574E+00_JPRB, 0.33296E+00_JPRB ,&
&  0.33185E+00_JPRB, 0.33266E+00_JPRB, 0.33427E+00_JPRB, 0.33378E+00_JPRB, 0.33359E+00_JPRB, 0.33496E+00_JPRB, 0.33741E+00_JPRB ,&
&  0.34166E+00_JPRB, 0.34340E+00_JPRB, 0.34462E+00_JPRB, 0.34112E+00_JPRB, 0.34476E+00_JPRB, 0.34620E+00_JPRB, 0.34645E+00_JPRB ,&
&  0.34750E+00_JPRB, 0.34841E+00_JPRB, 0.34973E+00_JPRB, 0.34948E+00_JPRB, 0.34609E+00_JPRB, 0.34675E+00_JPRB, 0.35128E+00_JPRB ,&
&  0.35489E+00_JPRB, 0.35350E+00_JPRB, 0.34976E+00_JPRB, 0.34639E+00_JPRB, 0.34351E+00_JPRB, 0.34117E+00_JPRB, 0.33916E+00_JPRB ,&
&  0.33814E+00_JPRB, 0.33683E+00_JPRB, 0.33393E+00_JPRB, 0.32755E+00_JPRB, 0.32065E+00_JPRB, 0.31590E+00_JPRB, 0.31227E+00_JPRB ,&
&  0.31165E+00_JPRB, 0.31335E+00_JPRB, 0.31521E+00_JPRB, 0.31504E+00_JPRB, 0.31270E+00_JPRB, 0.30874E+00_JPRB, 0.30392E+00_JPRB ,&
&  0.29856E+00_JPRB, 0.29710E+00_JPRB, 0.29746E+00_JPRB, 0.29718E+00_JPRB, 0.29791E+00_JPRB, 0.30011E+00_JPRB, 0.30280E+00_JPRB ,&
&  0.30616E+00_JPRB, 0.30595E+00_JPRB, 0.30411E+00_JPRB, 0.30125E+00_JPRB, 0.29837E+00_JPRB, 0.29822E+00_JPRB, 0.29950E+00_JPRB ,&
&  0.30092E+00_JPRB, 0.30239E+00_JPRB, 0.30428E+00_JPRB, 0.30653E+00_JPRB, 0.30907E+00_JPRB, 0.31207E+00_JPRB, 0.31432E+00_JPRB ,&
&  0.31420E+00_JPRB, 0.31299E+00_JPRB, 0.31435E+00_JPRB, 0.31691E+00_JPRB, 0.31835E+00_JPRB, 0.31913E+00_JPRB, 0.31957E+00_JPRB ,&
&  0.31884E+00_JPRB, 0.31819E+00_JPRB, 0.31972E+00_JPRB, 0.32409E+00_JPRB, 0.32797E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.31213E+00_JPRB, 0.31337E+00_JPRB, 0.31330E+00_JPRB, 0.31368E+00_JPRB, 0.31426E+00_JPRB, 0.31610E+00_JPRB, 0.32039E+00_JPRB ,&
&  0.32474E+00_JPRB, 0.32665E+00_JPRB, 0.32467E+00_JPRB, 0.32198E+00_JPRB, 0.32063E+00_JPRB, 0.32021E+00_JPRB, 0.32036E+00_JPRB ,&
&  0.32052E+00_JPRB, 0.31975E+00_JPRB, 0.31693E+00_JPRB, 0.31282E+00_JPRB, 0.30942E+00_JPRB, 0.30800E+00_JPRB, 0.30781E+00_JPRB ,&
&  0.31114E+00_JPRB, 0.31334E+00_JPRB, 0.31526E+00_JPRB, 0.31746E+00_JPRB, 0.31852E+00_JPRB, 0.31883E+00_JPRB, 0.32092E+00_JPRB ,&
&  0.32181E+00_JPRB, 0.31830E+00_JPRB, 0.31268E+00_JPRB, 0.31587E+00_JPRB, 0.31847E+00_JPRB, 0.32001E+00_JPRB, 0.31923E+00_JPRB ,&
&  0.31978E+00_JPRB, 0.32201E+00_JPRB, 0.32460E+00_JPRB, 0.32660E+00_JPRB, 0.32648E+00_JPRB, 0.32514E+00_JPRB, 0.32650E+00_JPRB ,&
&  0.32864E+00_JPRB, 0.32771E+00_JPRB, 0.32412E+00_JPRB, 0.32077E+00_JPRB, 0.31702E+00_JPRB, 0.31512E+00_JPRB, 0.31478E+00_JPRB ,&
&  0.31395E+00_JPRB, 0.31152E+00_JPRB, 0.30696E+00_JPRB, 0.30229E+00_JPRB, 0.29874E+00_JPRB, 0.29478E+00_JPRB, 0.29231E+00_JPRB ,&
&  0.29212E+00_JPRB, 0.29284E+00_JPRB, 0.29253E+00_JPRB, 0.28996E+00_JPRB, 0.28217E+00_JPRB, 0.27376E+00_JPRB, 0.26610E+00_JPRB ,&
&  0.26461E+00_JPRB, 0.26839E+00_JPRB, 0.27504E+00_JPRB, 0.27882E+00_JPRB, 0.28235E+00_JPRB, 0.28442E+00_JPRB, 0.28563E+00_JPRB ,&
&  0.28221E+00_JPRB, 0.27091E+00_JPRB, 0.26592E+00_JPRB, 0.26229E+00_JPRB, 0.25926E+00_JPRB, 0.25989E+00_JPRB, 0.26238E+00_JPRB ,&
&  0.26705E+00_JPRB, 0.27153E+00_JPRB, 0.27526E+00_JPRB, 0.27777E+00_JPRB, 0.27880E+00_JPRB, 0.27935E+00_JPRB, 0.27875E+00_JPRB ,&
&  0.27968E+00_JPRB, 0.28450E+00_JPRB, 0.29307E+00_JPRB, 0.29862E+00_JPRB, 0.29982E+00_JPRB, 0.30133E+00_JPRB, 0.30090E+00_JPRB ,&
&  0.30060E+00_JPRB, 0.30246E+00_JPRB, 0.30334E+00_JPRB, 0.30587E+00_JPRB, 0.30931E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.29736E+00_JPRB, 0.29843E+00_JPRB, 0.29828E+00_JPRB, 0.29695E+00_JPRB, 0.29536E+00_JPRB, 0.29514E+00_JPRB, 0.29628E+00_JPRB ,&
&  0.29912E+00_JPRB, 0.30104E+00_JPRB, 0.30128E+00_JPRB, 0.30117E+00_JPRB, 0.30031E+00_JPRB, 0.29855E+00_JPRB, 0.29728E+00_JPRB ,&
&  0.29640E+00_JPRB, 0.29608E+00_JPRB, 0.29578E+00_JPRB, 0.29582E+00_JPRB, 0.29580E+00_JPRB, 0.29634E+00_JPRB, 0.29668E+00_JPRB ,&
&  0.29980E+00_JPRB, 0.29925E+00_JPRB, 0.29850E+00_JPRB, 0.29840E+00_JPRB, 0.29845E+00_JPRB, 0.29885E+00_JPRB, 0.29878E+00_JPRB ,&
&  0.29475E+00_JPRB, 0.28735E+00_JPRB, 0.28190E+00_JPRB, 0.28932E+00_JPRB, 0.29883E+00_JPRB, 0.29900E+00_JPRB, 0.29964E+00_JPRB ,&
&  0.30119E+00_JPRB, 0.30405E+00_JPRB, 0.30654E+00_JPRB, 0.30765E+00_JPRB, 0.30688E+00_JPRB, 0.30619E+00_JPRB, 0.30665E+00_JPRB ,&
&  0.30765E+00_JPRB, 0.30583E+00_JPRB, 0.29956E+00_JPRB, 0.29204E+00_JPRB, 0.28444E+00_JPRB, 0.28169E+00_JPRB, 0.28027E+00_JPRB ,&
&  0.27739E+00_JPRB, 0.27554E+00_JPRB, 0.27502E+00_JPRB, 0.27460E+00_JPRB, 0.27509E+00_JPRB, 0.27214E+00_JPRB, 0.26713E+00_JPRB ,&
&  0.26496E+00_JPRB, 0.26023E+00_JPRB, 0.25529E+00_JPRB, 0.25004E+00_JPRB, 0.24574E+00_JPRB, 0.24027E+00_JPRB, 0.23643E+00_JPRB ,&
&  0.23602E+00_JPRB, 0.24135E+00_JPRB, 0.25262E+00_JPRB, 0.26209E+00_JPRB, 0.26395E+00_JPRB, 0.25745E+00_JPRB, 0.25230E+00_JPRB ,&
&  0.24474E+00_JPRB, 0.23743E+00_JPRB, 0.23327E+00_JPRB, 0.23268E+00_JPRB, 0.23311E+00_JPRB, 0.23419E+00_JPRB, 0.23661E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.24352E+00_JPRB, 0.24623E+00_JPRB, 0.24851E+00_JPRB, 0.25015E+00_JPRB, 0.25057E+00_JPRB, 0.25020E+00_JPRB ,&
&  0.24992E+00_JPRB, 0.25150E+00_JPRB, 0.25397E+00_JPRB, 0.25884E+00_JPRB, 0.26136E+00_JPRB, 0.26507E+00_JPRB, 0.26935E+00_JPRB ,&
&  0.27616E+00_JPRB, 0.28343E+00_JPRB, 0.28814E+00_JPRB, 0.29153E+00_JPRB, 0.29486E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.28213E+00_JPRB, 0.28713E+00_JPRB, 0.28906E+00_JPRB, 0.28805E+00_JPRB, 0.28728E+00_JPRB, 0.28736E+00_JPRB, 0.28761E+00_JPRB ,&
&  0.28834E+00_JPRB, 0.28764E+00_JPRB, 0.28598E+00_JPRB, 0.28471E+00_JPRB, 0.28335E+00_JPRB, 0.28163E+00_JPRB, 0.28085E+00_JPRB ,&
&  0.28037E+00_JPRB, 0.28119E+00_JPRB, 0.28185E+00_JPRB, 0.28356E+00_JPRB, 0.28433E+00_JPRB, 0.28575E+00_JPRB, 0.28341E+00_JPRB ,&
&  0.27896E+00_JPRB, 0.27613E+00_JPRB, 0.27869E+00_JPRB, 0.27898E+00_JPRB, 0.27955E+00_JPRB, 0.28017E+00_JPRB, 0.28187E+00_JPRB ,&
&  0.27873E+00_JPRB, 0.27630E+00_JPRB, 0.27531E+00_JPRB, 0.28330E+00_JPRB, 0.28934E+00_JPRB, 0.28665E+00_JPRB, 0.28689E+00_JPRB ,&
&  0.28900E+00_JPRB, 0.29164E+00_JPRB, 0.29308E+00_JPRB, 0.29313E+00_JPRB, 0.29195E+00_JPRB, 0.29192E+00_JPRB, 0.29074E+00_JPRB ,&
&  0.28713E+00_JPRB, 0.28415E+00_JPRB, 0.27300E+00_JPRB, 0.25429E+00_JPRB, 0.24881E+00_JPRB, 0.24678E+00_JPRB, 0.24289E+00_JPRB ,&
&  0.23719E+00_JPRB, 0.23575E+00_JPRB, 0.23613E+00_JPRB, 0.23341E+00_JPRB, 0.23343E+00_JPRB, 0.23321E+00_JPRB, 0.23150E+00_JPRB ,&
&  0.22949E+00_JPRB, 0.22827E+00_JPRB, 0.22661E+00_JPRB, 0.22372E+00_JPRB, 0.22146E+00_JPRB, 0.22110E+00_JPRB, 0.22225E+00_JPRB ,&
&  0.22454E+00_JPRB, 0.22752E+00_JPRB, 0.23711E+00_JPRB, 0.24256E+00_JPRB, 0.24072E+00_JPRB, 0.23807E+00_JPRB, 0.23094E+00_JPRB ,&
&  0.22334E+00_JPRB, 0.22116E+00_JPRB, 0.21966E+00_JPRB, 0.21829E+00_JPRB, 0.21877E+00_JPRB, 0.22002E+00_JPRB, 0.22123E+00_JPRB ,&
&  0.22292E+00_JPRB, 0.22510E+00_JPRB, 0.22647E+00_JPRB, 0.22728E+00_JPRB, 0.22804E+00_JPRB, 0.22836E+00_JPRB, 0.22892E+00_JPRB ,&
&  0.22876E+00_JPRB, 0.23034E+00_JPRB, 0.23139E+00_JPRB, 0.23408E+00_JPRB, 0.23577E+00_JPRB, 0.23824E+00_JPRB, 0.24417E+00_JPRB ,&
&  0.24816E+00_JPRB, 0.26106E+00_JPRB, 0.27696E+00_JPRB, 0.28079E+00_JPRB, 0.27636E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.27194E+00_JPRB, 0.27413E+00_JPRB, 0.27535E+00_JPRB, 0.27334E+00_JPRB, 0.27325E+00_JPRB, 0.27332E+00_JPRB, 0.27441E+00_JPRB ,&
&  0.27479E+00_JPRB, 0.27408E+00_JPRB, 0.27376E+00_JPRB, 0.27158E+00_JPRB, 0.26995E+00_JPRB, 0.26391E+00_JPRB, 0.26194E+00_JPRB ,&
&  0.26022E+00_JPRB, 0.26154E+00_JPRB, 0.26230E+00_JPRB, 0.26455E+00_JPRB, 0.26409E+00_JPRB, 0.26212E+00_JPRB, 0.25792E+00_JPRB ,&
&  0.25532E+00_JPRB, 0.25441E+00_JPRB, 0.25310E+00_JPRB, 0.25400E+00_JPRB, 0.25675E+00_JPRB, 0.25928E+00_JPRB, 0.26116E+00_JPRB ,&
&  0.26294E+00_JPRB, 0.26601E+00_JPRB, 0.26893E+00_JPRB, 0.27236E+00_JPRB, 0.27821E+00_JPRB, 0.27760E+00_JPRB, 0.27915E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.28103E+00_JPRB, 0.28136E+00_JPRB, 0.28043E+00_JPRB, 0.27830E+00_JPRB, 0.27141E+00_JPRB, 0.26947E+00_JPRB ,&
&  0.25760E+00_JPRB, 0.25047E+00_JPRB, 0.24447E+00_JPRB, 0.24033E+00_JPRB, 0.23331E+00_JPRB, 0.22967E+00_JPRB, 0.22552E+00_JPRB ,&
&  0.22199E+00_JPRB, 0.21811E+00_JPRB, 0.21532E+00_JPRB, 0.21433E+00_JPRB, 0.21403E+00_JPRB, 0.21369E+00_JPRB, 0.21267E+00_JPRB ,&
&  0.21284E+00_JPRB, 0.21230E+00_JPRB, 0.21190E+00_JPRB, 0.21149E+00_JPRB, 0.21120E+00_JPRB, 0.21131E+00_JPRB, 0.21227E+00_JPRB ,&
&  0.21468E+00_JPRB, 0.21667E+00_JPRB, 0.21896E+00_JPRB, 0.22156E+00_JPRB, 0.22127E+00_JPRB, 0.21918E+00_JPRB, 0.21599E+00_JPRB ,&
&  0.21305E+00_JPRB, 0.21110E+00_JPRB, 0.20903E+00_JPRB, 0.20842E+00_JPRB, 0.20916E+00_JPRB, 0.21004E+00_JPRB, 0.21174E+00_JPRB ,&
&  0.21322E+00_JPRB, 0.21449E+00_JPRB, 0.21535E+00_JPRB, 0.21583E+00_JPRB, 0.21663E+00_JPRB, 0.21730E+00_JPRB, 0.21853E+00_JPRB ,&
&  0.21990E+00_JPRB, 0.22154E+00_JPRB, 0.22294E+00_JPRB, 0.22493E+00_JPRB, 0.22819E+00_JPRB, 0.23267E+00_JPRB, 0.24070E+00_JPRB ,&
&  0.24737E+00_JPRB, 0.26472E+00_JPRB, 0.26704E+00_JPRB, 0.27063E+00_JPRB, 0.27278E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.26087E+00_JPRB, 0.26272E+00_JPRB, 0.26322E+00_JPRB, 0.26393E+00_JPRB, 0.26367E+00_JPRB, 0.26360E+00_JPRB, 0.26422E+00_JPRB ,&
&  0.25993E+00_JPRB, 0.25729E+00_JPRB, 0.25258E+00_JPRB, 0.24860E+00_JPRB, 0.24472E+00_JPRB, 0.24178E+00_JPRB, 0.24008E+00_JPRB ,&
&  0.23910E+00_JPRB, 0.23940E+00_JPRB, 0.23905E+00_JPRB, 0.23932E+00_JPRB, 0.23758E+00_JPRB, 0.23612E+00_JPRB, 0.23487E+00_JPRB ,&
&  0.23379E+00_JPRB, 0.23503E+00_JPRB, 0.23617E+00_JPRB, 0.24010E+00_JPRB, 0.24563E+00_JPRB, 0.25156E+00_JPRB, 0.25617E+00_JPRB ,&
&  0.25916E+00_JPRB, 0.26164E+00_JPRB, 0.26518E+00_JPRB, 0.26680E+00_JPRB, 0.26884E+00_JPRB, 0.27056E+00_JPRB, 0.27090E+00_JPRB ,&
&  0.27218E+00_JPRB, 0.27163E+00_JPRB, 0.27118E+00_JPRB, 0.27068E+00_JPRB, 0.26102E+00_JPRB, 0.25441E+00_JPRB, 0.24737E+00_JPRB ,&
&  0.24092E+00_JPRB, 0.23430E+00_JPRB, 0.22828E+00_JPRB, 0.22521E+00_JPRB, 0.22117E+00_JPRB, 0.21888E+00_JPRB, 0.21692E+00_JPRB ,&
&  0.21538E+00_JPRB, 0.21378E+00_JPRB, 0.21144E+00_JPRB, 0.20919E+00_JPRB, 0.20785E+00_JPRB, 0.20671E+00_JPRB, 0.20520E+00_JPRB ,&
&  0.20512E+00_JPRB, 0.20436E+00_JPRB, 0.20447E+00_JPRB, 0.20411E+00_JPRB, 0.20390E+00_JPRB, 0.20371E+00_JPRB, 0.20370E+00_JPRB ,&
&  0.20417E+00_JPRB, 0.20529E+00_JPRB, 0.20584E+00_JPRB, 0.20660E+00_JPRB, 0.20668E+00_JPRB, 0.20594E+00_JPRB, 0.20534E+00_JPRB ,&
&  0.20450E+00_JPRB, 0.20385E+00_JPRB, 0.20386E+00_JPRB, 0.20365E+00_JPRB, 0.20421E+00_JPRB, 0.20495E+00_JPRB, 0.20565E+00_JPRB ,&
&  0.20671E+00_JPRB, 0.20740E+00_JPRB, 0.20816E+00_JPRB, 0.20925E+00_JPRB, 0.21008E+00_JPRB, 0.21120E+00_JPRB, 0.21248E+00_JPRB ,&
&  0.21331E+00_JPRB, 0.21530E+00_JPRB, 0.21695E+00_JPRB, 0.21907E+00_JPRB, 0.22358E+00_JPRB, 0.22744E+00_JPRB, 0.23734E+00_JPRB ,&
&  0.24498E+00_JPRB, 0.25995E+00_JPRB, 0.26064E+00_JPRB, 0.25808E+00_JPRB, 0.25943E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.25469E+00_JPRB, 0.25606E+00_JPRB, 0.25619E+00_JPRB, 0.25680E+00_JPRB, 0.25673E+00_JPRB, 0.25803E+00_JPRB, 0.25765E+00_JPRB ,&
&  0.25775E+00_JPRB, 0.25427E+00_JPRB, 0.25218E+00_JPRB, 0.24941E+00_JPRB, 0.24699E+00_JPRB, 0.24389E+00_JPRB, 0.24090E+00_JPRB ,&
&  0.23860E+00_JPRB, 0.23648E+00_JPRB, 0.23413E+00_JPRB, 0.23104E+00_JPRB, 0.22788E+00_JPRB, 0.22576E+00_JPRB, 0.22456E+00_JPRB ,&
&  0.22341E+00_JPRB, 0.22501E+00_JPRB, 0.22742E+00_JPRB, 0.23167E+00_JPRB, 0.23700E+00_JPRB, 0.24101E+00_JPRB, 0.24660E+00_JPRB ,&
&  0.25241E+00_JPRB, 0.25433E+00_JPRB, 0.25812E+00_JPRB, 0.25965E+00_JPRB, 0.25942E+00_JPRB, 0.26043E+00_JPRB, 0.25888E+00_JPRB ,&
&  0.25887E+00_JPRB, 0.25427E+00_JPRB, 0.25885E+00_JPRB, 0.25406E+00_JPRB, 0.24601E+00_JPRB, 0.24306E+00_JPRB, 0.23634E+00_JPRB ,&
&  0.23231E+00_JPRB, 0.22753E+00_JPRB, 0.22460E+00_JPRB, 0.22310E+00_JPRB, 0.21906E+00_JPRB, 0.21698E+00_JPRB, 0.21357E+00_JPRB ,&
&  0.21041E+00_JPRB, 0.20959E+00_JPRB, 0.20778E+00_JPRB, 0.20724E+00_JPRB, 0.20636E+00_JPRB, 0.20498E+00_JPRB, 0.20347E+00_JPRB ,&
&  0.20250E+00_JPRB, 0.20170E+00_JPRB, 0.20137E+00_JPRB, 0.20081E+00_JPRB, 0.19997E+00_JPRB, 0.19912E+00_JPRB, 0.19806E+00_JPRB ,&
&  0.19744E+00_JPRB, 0.19724E+00_JPRB, 0.19716E+00_JPRB, 0.19736E+00_JPRB, 0.19754E+00_JPRB, 0.19743E+00_JPRB, 0.19771E+00_JPRB ,&
&  0.19810E+00_JPRB, 0.19885E+00_JPRB, 0.19936E+00_JPRB, 0.20007E+00_JPRB, 0.20062E+00_JPRB, 0.20135E+00_JPRB, 0.20177E+00_JPRB ,&
&  0.20234E+00_JPRB, 0.20335E+00_JPRB, 0.20424E+00_JPRB, 0.20533E+00_JPRB, 0.20648E+00_JPRB, 0.20739E+00_JPRB, 0.20902E+00_JPRB ,&
&  0.21065E+00_JPRB, 0.21228E+00_JPRB, 0.21473E+00_JPRB, 0.21706E+00_JPRB, 0.22009E+00_JPRB, 0.22485E+00_JPRB, 0.23010E+00_JPRB ,&
&  0.23672E+00_JPRB, 0.24612E+00_JPRB, 0.24634E+00_JPRB, 0.25093E+00_JPRB, 0.25305E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.25404E+00_JPRB, 0.25544E+00_JPRB, 0.25656E+00_JPRB, 0.25710E+00_JPRB, 0.25795E+00_JPRB, 0.25793E+00_JPRB, 0.25771E+00_JPRB ,&
&  0.25434E+00_JPRB, 0.25304E+00_JPRB, 0.25099E+00_JPRB, 0.24867E+00_JPRB, 0.24664E+00_JPRB, 0.24317E+00_JPRB, 0.24030E+00_JPRB ,&
&  0.23771E+00_JPRB, 0.23476E+00_JPRB, 0.23209E+00_JPRB, 0.22890E+00_JPRB, 0.22603E+00_JPRB, 0.22336E+00_JPRB, 0.22242E+00_JPRB ,&
&  0.22213E+00_JPRB, 0.22306E+00_JPRB, 0.22541E+00_JPRB, 0.22706E+00_JPRB, 0.22936E+00_JPRB, 0.23223E+00_JPRB, 0.23652E+00_JPRB ,&
&  0.23947E+00_JPRB, 0.24336E+00_JPRB, 0.24579E+00_JPRB, 0.24638E+00_JPRB, 0.24575E+00_JPRB, 0.24546E+00_JPRB, 0.24450E+00_JPRB ,&
&  0.24217E+00_JPRB, 0.23983E+00_JPRB, 0.23787E+00_JPRB, 0.23571E+00_JPRB, 0.23346E+00_JPRB, 0.23097E+00_JPRB, 0.22773E+00_JPRB ,&
&  0.22490E+00_JPRB, 0.22242E+00_JPRB, 0.21996E+00_JPRB, 0.21799E+00_JPRB, 0.21599E+00_JPRB, 0.21413E+00_JPRB, 0.21194E+00_JPRB ,&
&  0.21024E+00_JPRB, 0.20881E+00_JPRB, 0.20698E+00_JPRB, 0.20565E+00_JPRB, 0.20408E+00_JPRB, 0.20278E+00_JPRB, 0.20135E+00_JPRB ,&
&  0.20033E+00_JPRB, 0.19948E+00_JPRB, 0.19882E+00_JPRB, 0.19821E+00_JPRB, 0.19792E+00_JPRB, 0.19764E+00_JPRB, 0.19731E+00_JPRB ,&
&  0.19706E+00_JPRB, 0.19730E+00_JPRB, 0.19753E+00_JPRB, 0.19775E+00_JPRB, 0.19800E+00_JPRB, 0.19764E+00_JPRB, 0.19773E+00_JPRB ,&
&  0.19807E+00_JPRB, 0.19837E+00_JPRB, 0.19822E+00_JPRB, 0.19857E+00_JPRB, 0.19924E+00_JPRB, 0.20004E+00_JPRB, 0.20135E+00_JPRB ,&
&  0.20238E+00_JPRB, 0.20346E+00_JPRB, 0.20449E+00_JPRB, 0.20554E+00_JPRB, 0.20700E+00_JPRB, 0.20857E+00_JPRB, 0.21020E+00_JPRB ,&
&  0.21216E+00_JPRB, 0.21395E+00_JPRB, 0.21580E+00_JPRB, 0.21769E+00_JPRB, 0.22083E+00_JPRB, 0.22353E+00_JPRB, 0.22775E+00_JPRB ,&
&  0.23191E+00_JPRB, 0.23286E+00_JPRB, 0.24774E+00_JPRB, 0.24877E+00_JPRB, 0.25046E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.23249E+00_JPRB, 0.23459E+00_JPRB, 0.23565E+00_JPRB, 0.23663E+00_JPRB, 0.23179E+00_JPRB, 0.24382E+00_JPRB, 0.24133E+00_JPRB ,&
&  0.24080E+00_JPRB, 0.23649E+00_JPRB, 0.23764E+00_JPRB, 0.23737E+00_JPRB, 0.23612E+00_JPRB, 0.23465E+00_JPRB, 0.23284E+00_JPRB ,&
&  0.23101E+00_JPRB, 0.22859E+00_JPRB, 0.22676E+00_JPRB, 0.22523E+00_JPRB, 0.22383E+00_JPRB, 0.22263E+00_JPRB, 0.22144E+00_JPRB ,&
&  0.22152E+00_JPRB, 0.22248E+00_JPRB, 0.22384E+00_JPRB, 0.22275E+00_JPRB, 0.22306E+00_JPRB, 0.22395E+00_JPRB, 0.22525E+00_JPRB ,&
&  0.22626E+00_JPRB, 0.22788E+00_JPRB, 0.22897E+00_JPRB, 0.22937E+00_JPRB, 0.23043E+00_JPRB, 0.22974E+00_JPRB, 0.22924E+00_JPRB ,&
&  0.22838E+00_JPRB, 0.22716E+00_JPRB, 0.22563E+00_JPRB, 0.22429E+00_JPRB, 0.22283E+00_JPRB, 0.22101E+00_JPRB, 0.21913E+00_JPRB ,&
&  0.21745E+00_JPRB, 0.21599E+00_JPRB, 0.21433E+00_JPRB, 0.21298E+00_JPRB, 0.21169E+00_JPRB, 0.21040E+00_JPRB, 0.20917E+00_JPRB ,&
&  0.20819E+00_JPRB, 0.20732E+00_JPRB, 0.20638E+00_JPRB, 0.20546E+00_JPRB, 0.20446E+00_JPRB, 0.20351E+00_JPRB, 0.20260E+00_JPRB ,&
&  0.20227E+00_JPRB, 0.20201E+00_JPRB, 0.20171E+00_JPRB, 0.20139E+00_JPRB, 0.20141E+00_JPRB, 0.20139E+00_JPRB, 0.20131E+00_JPRB ,&
&  0.20118E+00_JPRB, 0.20092E+00_JPRB, 0.20086E+00_JPRB, 0.20078E+00_JPRB, 0.20070E+00_JPRB, 0.20062E+00_JPRB, 0.20074E+00_JPRB ,&
&  0.20093E+00_JPRB, 0.20121E+00_JPRB, 0.20182E+00_JPRB, 0.20246E+00_JPRB, 0.20312E+00_JPRB, 0.20372E+00_JPRB, 0.20459E+00_JPRB ,&
&  0.20557E+00_JPRB, 0.20650E+00_JPRB, 0.20727E+00_JPRB, 0.20790E+00_JPRB, 0.20908E+00_JPRB, 0.21020E+00_JPRB, 0.21130E+00_JPRB ,&
&  0.21251E+00_JPRB, 0.21406E+00_JPRB, 0.21567E+00_JPRB, 0.21734E+00_JPRB, 0.21933E+00_JPRB, 0.22087E+00_JPRB, 0.22246E+00_JPRB ,&
&  0.22435E+00_JPRB, 0.22799E+00_JPRB, 0.23018E+00_JPRB, 0.23151E+00_JPRB, 0.23277E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.21626E+00_JPRB, 0.21638E+00_JPRB, 0.21638E+00_JPRB, 0.21625E+00_JPRB, 0.21587E+00_JPRB, 0.21573E+00_JPRB, 0.21561E+00_JPRB ,&
&  0.21548E+00_JPRB, 0.21516E+00_JPRB, 0.21538E+00_JPRB, 0.21546E+00_JPRB, 0.21546E+00_JPRB, 0.21543E+00_JPRB, 0.21521E+00_JPRB ,&
&  0.21477E+00_JPRB, 0.21421E+00_JPRB, 0.21286E+00_JPRB, 0.21260E+00_JPRB, 0.21257E+00_JPRB, 0.21278E+00_JPRB, 0.21294E+00_JPRB ,&
&  0.21324E+00_JPRB, 0.21355E+00_JPRB, 0.21382E+00_JPRB, 0.21467E+00_JPRB, 0.21502E+00_JPRB, 0.21542E+00_JPRB, 0.21581E+00_JPRB ,&
&  0.21618E+00_JPRB, 0.21663E+00_JPRB, 0.21699E+00_JPRB, 0.21736E+00_JPRB, 0.21774E+00_JPRB, 0.21771E+00_JPRB, 0.21747E+00_JPRB ,&
&  0.21707E+00_JPRB, 0.21644E+00_JPRB, 0.21577E+00_JPRB, 0.21515E+00_JPRB, 0.21459E+00_JPRB, 0.21402E+00_JPRB, 0.21347E+00_JPRB ,&
&  0.21288E+00_JPRB, 0.21223E+00_JPRB, 0.21154E+00_JPRB, 0.21093E+00_JPRB, 0.21030E+00_JPRB, 0.20974E+00_JPRB, 0.20911E+00_JPRB ,&
&  0.20864E+00_JPRB, 0.20815E+00_JPRB, 0.20769E+00_JPRB, 0.20720E+00_JPRB, 0.20679E+00_JPRB, 0.20636E+00_JPRB, 0.20593E+00_JPRB ,&
&  0.20549E+00_JPRB, 0.20529E+00_JPRB, 0.20499E+00_JPRB, 0.20474E+00_JPRB, 0.20450E+00_JPRB, 0.20433E+00_JPRB, 0.20408E+00_JPRB ,&
&  0.20385E+00_JPRB, 0.20363E+00_JPRB, 0.20355E+00_JPRB, 0.20340E+00_JPRB, 0.20338E+00_JPRB, 0.20335E+00_JPRB, 0.20344E+00_JPRB ,&
&  0.20351E+00_JPRB, 0.20362E+00_JPRB, 0.20382E+00_JPRB, 0.20417E+00_JPRB, 0.20447E+00_JPRB, 0.20480E+00_JPRB, 0.20512E+00_JPRB ,&
&  0.20549E+00_JPRB, 0.20588E+00_JPRB, 0.20628E+00_JPRB, 0.20666E+00_JPRB, 0.20716E+00_JPRB, 0.20767E+00_JPRB, 0.20825E+00_JPRB ,&
&  0.20878E+00_JPRB, 0.20936E+00_JPRB, 0.20989E+00_JPRB, 0.21050E+00_JPRB, 0.21103E+00_JPRB, 0.21136E+00_JPRB, 0.21166E+00_JPRB ,&
&  0.21219E+00_JPRB, 0.21270E+00_JPRB, 0.21329E+00_JPRB, 0.21383E+00_JPRB, 0.21477E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB, 0.20762E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB, 0.84074E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.82856E+00_JPRB, 0.82897E+00_JPRB, 0.82932E+00_JPRB, 0.82967E+00_JPRB, 0.82997E+00_JPRB, 0.83028E+00_JPRB, 0.83062E+00_JPRB ,&
&  0.83085E+00_JPRB, 0.83134E+00_JPRB, 0.83195E+00_JPRB, 0.83265E+00_JPRB, 0.83330E+00_JPRB, 0.83396E+00_JPRB, 0.83462E+00_JPRB ,&
&  0.83524E+00_JPRB, 0.83567E+00_JPRB, 0.83612E+00_JPRB, 0.83679E+00_JPRB, 0.83765E+00_JPRB, 0.83869E+00_JPRB, 0.84013E+00_JPRB ,&
&  0.84074E+00_JPRB, 0.84053E+00_JPRB, 0.84043E+00_JPRB, 0.84127E+00_JPRB, 0.84111E+00_JPRB, 0.84102E+00_JPRB, 0.84102E+00_JPRB ,&
&  0.84094E+00_JPRB, 0.84080E+00_JPRB, 0.84050E+00_JPRB, 0.84021E+00_JPRB, 0.84013E+00_JPRB, 0.84031E+00_JPRB, 0.84059E+00_JPRB ,&
&  0.84101E+00_JPRB, 0.84132E+00_JPRB, 0.84217E+00_JPRB, 0.84305E+00_JPRB, 0.84408E+00_JPRB, 0.84476E+00_JPRB, 0.84556E+00_JPRB ,&
&  0.84606E+00_JPRB, 0.84662E+00_JPRB, 0.84749E+00_JPRB, 0.84870E+00_JPRB, 0.85001E+00_JPRB, 0.85130E+00_JPRB, 0.85221E+00_JPRB ,&
&  0.85363E+00_JPRB, 0.85553E+00_JPRB, 0.85770E+00_JPRB, 0.86003E+00_JPRB, 0.86171E+00_JPRB, 0.86346E+00_JPRB, 0.86540E+00_JPRB ,&
&  0.86706E+00_JPRB, 0.86793E+00_JPRB, 0.86899E+00_JPRB, 0.87009E+00_JPRB, 0.87082E+00_JPRB, 0.86988E+00_JPRB, 0.86912E+00_JPRB ,&
&  0.86846E+00_JPRB, 0.86745E+00_JPRB, 0.86497E+00_JPRB, 0.86290E+00_JPRB, 0.86103E+00_JPRB, 0.85911E+00_JPRB, 0.85695E+00_JPRB ,&
&  0.85501E+00_JPRB, 0.85321E+00_JPRB, 0.85128E+00_JPRB, 0.84922E+00_JPRB, 0.84739E+00_JPRB, 0.84565E+00_JPRB, 0.84396E+00_JPRB ,&
&  0.84236E+00_JPRB, 0.84101E+00_JPRB, 0.83970E+00_JPRB, 0.83853E+00_JPRB, 0.83725E+00_JPRB, 0.83603E+00_JPRB, 0.83475E+00_JPRB ,&
&  0.83360E+00_JPRB, 0.83240E+00_JPRB, 0.83130E+00_JPRB, 0.83021E+00_JPRB, 0.82948E+00_JPRB, 0.82902E+00_JPRB, 0.82863E+00_JPRB ,&
&  0.82832E+00_JPRB, 0.82819E+00_JPRB, 0.82827E+00_JPRB, 0.82828E+00_JPRB, 0.82832E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.82101E+00_JPRB, 0.82208E+00_JPRB, 0.82302E+00_JPRB, 0.82387E+00_JPRB, 0.82490E+00_JPRB, 0.82607E+00_JPRB, 0.82711E+00_JPRB ,&
&  0.82798E+00_JPRB, 0.82923E+00_JPRB, 0.83065E+00_JPRB, 0.83208E+00_JPRB, 0.83342E+00_JPRB, 0.83487E+00_JPRB, 0.83623E+00_JPRB ,&
&  0.83754E+00_JPRB, 0.83882E+00_JPRB, 0.84014E+00_JPRB, 0.84132E+00_JPRB, 0.84244E+00_JPRB, 0.84319E+00_JPRB, 0.84591E+00_JPRB ,&
&  0.84811E+00_JPRB, 0.84902E+00_JPRB, 0.84811E+00_JPRB, 0.84080E+00_JPRB, 0.83552E+00_JPRB, 0.83071E+00_JPRB, 0.82670E+00_JPRB ,&
&  0.82255E+00_JPRB, 0.82240E+00_JPRB, 0.82292E+00_JPRB, 0.82352E+00_JPRB, 0.82688E+00_JPRB, 0.82620E+00_JPRB, 0.82521E+00_JPRB ,&
&  0.82280E+00_JPRB, 0.81690E+00_JPRB, 0.81783E+00_JPRB, 0.81899E+00_JPRB, 0.82014E+00_JPRB, 0.82339E+00_JPRB, 0.82580E+00_JPRB ,&
&  0.82867E+00_JPRB, 0.83171E+00_JPRB, 0.83738E+00_JPRB, 0.84351E+00_JPRB, 0.84908E+00_JPRB, 0.85377E+00_JPRB, 0.85804E+00_JPRB ,&
&  0.86186E+00_JPRB, 0.86591E+00_JPRB, 0.87037E+00_JPRB, 0.87562E+00_JPRB, 0.87976E+00_JPRB, 0.88413E+00_JPRB, 0.88772E+00_JPRB ,&
&  0.88971E+00_JPRB, 0.89030E+00_JPRB, 0.89254E+00_JPRB, 0.89592E+00_JPRB, 0.89934E+00_JPRB, 0.89932E+00_JPRB, 0.89790E+00_JPRB ,&
&  0.89517E+00_JPRB, 0.89242E+00_JPRB, 0.88805E+00_JPRB, 0.88431E+00_JPRB, 0.88039E+00_JPRB, 0.87644E+00_JPRB, 0.87187E+00_JPRB ,&
&  0.86737E+00_JPRB, 0.86298E+00_JPRB, 0.85952E+00_JPRB, 0.85606E+00_JPRB, 0.85281E+00_JPRB, 0.84972E+00_JPRB, 0.84769E+00_JPRB ,&
&  0.84545E+00_JPRB, 0.84353E+00_JPRB, 0.84145E+00_JPRB, 0.83933E+00_JPRB, 0.83599E+00_JPRB, 0.83262E+00_JPRB, 0.82897E+00_JPRB ,&
&  0.82592E+00_JPRB, 0.82294E+00_JPRB, 0.82051E+00_JPRB, 0.81848E+00_JPRB, 0.81761E+00_JPRB, 0.81726E+00_JPRB, 0.81720E+00_JPRB ,&
&  0.81727E+00_JPRB, 0.81785E+00_JPRB, 0.81867E+00_JPRB, 0.81938E+00_JPRB, 0.82013E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.82525E+00_JPRB, 0.82679E+00_JPRB, 0.82824E+00_JPRB, 0.82914E+00_JPRB, 0.83076E+00_JPRB, 0.83230E+00_JPRB, 0.83381E+00_JPRB ,&
&  0.83531E+00_JPRB, 0.83635E+00_JPRB, 0.83731E+00_JPRB, 0.83839E+00_JPRB, 0.83938E+00_JPRB, 0.84130E+00_JPRB, 0.84311E+00_JPRB ,&
&  0.84476E+00_JPRB, 0.84588E+00_JPRB, 0.84848E+00_JPRB, 0.85083E+00_JPRB, 0.85216E+00_JPRB, 0.85337E+00_JPRB, 0.84781E+00_JPRB ,&
&  0.84432E+00_JPRB, 0.84199E+00_JPRB, 0.83890E+00_JPRB, 0.83157E+00_JPRB, 0.82792E+00_JPRB, 0.80971E+00_JPRB, 0.81657E+00_JPRB ,&
&  0.84259E+00_JPRB, 0.84096E+00_JPRB, 0.83498E+00_JPRB, 0.82486E+00_JPRB, 0.78912E+00_JPRB, 0.77442E+00_JPRB, 0.77201E+00_JPRB ,&
&  0.75350E+00_JPRB, 0.77307E+00_JPRB, 0.77892E+00_JPRB, 0.78151E+00_JPRB, 0.78594E+00_JPRB, 0.78257E+00_JPRB, 0.78277E+00_JPRB ,&
&  0.79488E+00_JPRB, 0.79504E+00_JPRB, 0.80194E+00_JPRB, 0.81060E+00_JPRB, 0.82012E+00_JPRB, 0.82916E+00_JPRB, 0.83957E+00_JPRB ,&
&  0.84791E+00_JPRB, 0.85411E+00_JPRB, 0.85858E+00_JPRB, 0.86029E+00_JPRB, 0.86232E+00_JPRB, 0.86478E+00_JPRB, 0.86963E+00_JPRB ,&
&  0.87699E+00_JPRB, 0.88468E+00_JPRB, 0.88855E+00_JPRB, 0.88695E+00_JPRB, 0.88832E+00_JPRB, 0.88923E+00_JPRB, 0.89022E+00_JPRB ,&
&  0.88958E+00_JPRB, 0.88782E+00_JPRB, 0.88337E+00_JPRB, 0.88219E+00_JPRB, 0.88147E+00_JPRB, 0.87950E+00_JPRB, 0.87638E+00_JPRB ,&
&  0.87242E+00_JPRB, 0.86673E+00_JPRB, 0.86219E+00_JPRB, 0.85859E+00_JPRB, 0.85543E+00_JPRB, 0.85264E+00_JPRB, 0.85168E+00_JPRB ,&
&  0.84937E+00_JPRB, 0.84754E+00_JPRB, 0.84410E+00_JPRB, 0.83916E+00_JPRB, 0.83351E+00_JPRB, 0.82856E+00_JPRB, 0.82334E+00_JPRB ,&
&  0.82021E+00_JPRB, 0.81827E+00_JPRB, 0.81736E+00_JPRB, 0.81613E+00_JPRB, 0.81635E+00_JPRB, 0.81674E+00_JPRB, 0.81774E+00_JPRB ,&
&  0.81873E+00_JPRB, 0.82008E+00_JPRB, 0.82119E+00_JPRB, 0.82261E+00_JPRB, 0.82351E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.82876E+00_JPRB, 0.82884E+00_JPRB, 0.82935E+00_JPRB, 0.82966E+00_JPRB, 0.83093E+00_JPRB, 0.83196E+00_JPRB, 0.83482E+00_JPRB ,&
&  0.83752E+00_JPRB, 0.84027E+00_JPRB, 0.84295E+00_JPRB, 0.84521E+00_JPRB, 0.84749E+00_JPRB, 0.84864E+00_JPRB, 0.84939E+00_JPRB ,&
&  0.85011E+00_JPRB, 0.85051E+00_JPRB, 0.85224E+00_JPRB, 0.85417E+00_JPRB, 0.85483E+00_JPRB, 0.85637E+00_JPRB, 0.85037E+00_JPRB ,&
&  0.84578E+00_JPRB, 0.84260E+00_JPRB, 0.82301E+00_JPRB, 0.83367E+00_JPRB, 0.83246E+00_JPRB, 0.83427E+00_JPRB, 0.83053E+00_JPRB ,&
&  0.80684E+00_JPRB, 0.79639E+00_JPRB, 0.77078E+00_JPRB, 0.75048E+00_JPRB, 0.75411E+00_JPRB, 0.74135E+00_JPRB, 0.74438E+00_JPRB ,&
&  0.74246E+00_JPRB, 0.73724E+00_JPRB, 0.73493E+00_JPRB, 0.73290E+00_JPRB, 0.73587E+00_JPRB, 0.73760E+00_JPRB, 0.73971E+00_JPRB ,&
&  0.74710E+00_JPRB, 0.75328E+00_JPRB, 0.76238E+00_JPRB, 0.76290E+00_JPRB, 0.77605E+00_JPRB, 0.79087E+00_JPRB, 0.80642E+00_JPRB ,&
&  0.81684E+00_JPRB, 0.81615E+00_JPRB, 0.82129E+00_JPRB, 0.82847E+00_JPRB, 0.83444E+00_JPRB, 0.84098E+00_JPRB, 0.85088E+00_JPRB ,&
&  0.86326E+00_JPRB, 0.87444E+00_JPRB, 0.87784E+00_JPRB, 0.87556E+00_JPRB, 0.87176E+00_JPRB, 0.86865E+00_JPRB, 0.86872E+00_JPRB ,&
&  0.86880E+00_JPRB, 0.87204E+00_JPRB, 0.87302E+00_JPRB, 0.87600E+00_JPRB, 0.87775E+00_JPRB, 0.88235E+00_JPRB, 0.88549E+00_JPRB ,&
&  0.88740E+00_JPRB, 0.88755E+00_JPRB, 0.89074E+00_JPRB, 0.88461E+00_JPRB, 0.88305E+00_JPRB, 0.88055E+00_JPRB, 0.87299E+00_JPRB ,&
&  0.86587E+00_JPRB, 0.86001E+00_JPRB, 0.85375E+00_JPRB, 0.84750E+00_JPRB, 0.84160E+00_JPRB, 0.83700E+00_JPRB, 0.83386E+00_JPRB ,&
&  0.83218E+00_JPRB, 0.83030E+00_JPRB, 0.82899E+00_JPRB, 0.82577E+00_JPRB, 0.82572E+00_JPRB, 0.82333E+00_JPRB, 0.82252E+00_JPRB ,&
&  0.82215E+00_JPRB, 0.82372E+00_JPRB, 0.82420E+00_JPRB, 0.82619E+00_JPRB, 0.82808E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.82129E+00_JPRB, 0.82453E+00_JPRB, 0.83054E+00_JPRB, 0.83564E+00_JPRB, 0.84066E+00_JPRB, 0.84472E+00_JPRB, 0.84687E+00_JPRB ,&
&  0.84858E+00_JPRB, 0.84919E+00_JPRB, 0.84992E+00_JPRB, 0.85096E+00_JPRB, 0.85274E+00_JPRB, 0.85423E+00_JPRB, 0.85622E+00_JPRB ,&
&  0.85709E+00_JPRB, 0.85772E+00_JPRB, 0.85808E+00_JPRB, 0.85844E+00_JPRB, 0.85929E+00_JPRB, 0.86159E+00_JPRB, 0.86056E+00_JPRB ,&
&  0.86093E+00_JPRB, 0.85522E+00_JPRB, 0.84092E+00_JPRB, 0.84694E+00_JPRB, 0.84937E+00_JPRB, 0.85096E+00_JPRB, 0.84117E+00_JPRB ,&
&  0.81262E+00_JPRB, 0.77169E+00_JPRB, 0.79667E+00_JPRB, 0.78362E+00_JPRB, 0.76929E+00_JPRB, 0.75418E+00_JPRB, 0.73009E+00_JPRB ,&
&  0.71550E+00_JPRB, 0.69614E+00_JPRB, 0.68154E+00_JPRB, 0.67620E+00_JPRB, 0.67514E+00_JPRB, 0.68154E+00_JPRB, 0.69067E+00_JPRB ,&
&  0.69406E+00_JPRB, 0.70530E+00_JPRB, 0.71242E+00_JPRB, 0.72235E+00_JPRB, 0.73538E+00_JPRB, 0.75251E+00_JPRB, 0.75950E+00_JPRB ,&
&  0.77158E+00_JPRB, 0.77622E+00_JPRB, 0.78789E+00_JPRB, 0.80404E+00_JPRB, 0.82550E+00_JPRB, 0.83889E+00_JPRB, 0.84772E+00_JPRB ,&
&  0.85768E+00_JPRB, 0.86299E+00_JPRB, 0.86594E+00_JPRB, 0.86730E+00_JPRB, 0.87097E+00_JPRB, 0.87497E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.88328E+00_JPRB, 0.88997E+00_JPRB, 0.89713E+00_JPRB, 0.90554E+00_JPRB, 0.91047E+00_JPRB, 0.91944E+00_JPRB, 0.92966E+00_JPRB ,&
&  0.93931E+00_JPRB, 0.94465E+00_JPRB, 0.95468E+00_JPRB, 0.94556E+00_JPRB, 0.94154E+00_JPRB, 0.92445E+00_JPRB, 0.91194E+00_JPRB ,&
&  0.90246E+00_JPRB, 0.89056E+00_JPRB, 0.88019E+00_JPRB, 0.87090E+00_JPRB, 0.86374E+00_JPRB, 0.85767E+00_JPRB, 0.85244E+00_JPRB ,&
&  0.84706E+00_JPRB, 0.84013E+00_JPRB, 0.83587E+00_JPRB, 0.83067E+00_JPRB, 0.82852E+00_JPRB, 0.82445E+00_JPRB, 0.82238E+00_JPRB ,&
&  0.82082E+00_JPRB, 0.82043E+00_JPRB, 0.81952E+00_JPRB, 0.82079E+00_JPRB, 0.82134E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.82405E+00_JPRB, 0.83030E+00_JPRB, 0.83743E+00_JPRB, 0.84336E+00_JPRB, 0.84620E+00_JPRB, 0.84855E+00_JPRB, 0.84674E+00_JPRB ,&
&  0.84568E+00_JPRB, 0.84377E+00_JPRB, 0.84235E+00_JPRB, 0.84263E+00_JPRB, 0.84410E+00_JPRB, 0.84698E+00_JPRB, 0.85046E+00_JPRB ,&
&  0.85369E+00_JPRB, 0.85768E+00_JPRB, 0.85949E+00_JPRB, 0.86167E+00_JPRB, 0.86445E+00_JPRB, 0.86990E+00_JPRB, 0.87237E+00_JPRB ,&
&  0.87532E+00_JPRB, 0.86916E+00_JPRB, 0.85565E+00_JPRB, 0.85280E+00_JPRB, 0.86074E+00_JPRB, 0.84544E+00_JPRB, 0.84509E+00_JPRB ,&
&  0.85511E+00_JPRB, 0.85657E+00_JPRB, 0.84076E+00_JPRB, 0.82984E+00_JPRB, 0.80110E+00_JPRB, 0.72140E+00_JPRB, 0.72157E+00_JPRB ,&
&  0.70741E+00_JPRB, 0.68036E+00_JPRB, 0.66595E+00_JPRB, 0.65311E+00_JPRB, 0.64693E+00_JPRB, 0.65689E+00_JPRB, 0.67288E+00_JPRB ,&
&  0.67583E+00_JPRB, 0.69255E+00_JPRB, 0.69891E+00_JPRB, 0.70986E+00_JPRB, 0.72210E+00_JPRB, 0.74021E+00_JPRB, 0.74991E+00_JPRB ,&
&  0.76053E+00_JPRB, 0.77167E+00_JPRB, 0.78927E+00_JPRB, 0.81213E+00_JPRB, 0.83083E+00_JPRB, 0.84073E+00_JPRB, 0.83998E+00_JPRB ,&
&  0.85145E+00_JPRB, 0.84676E+00_JPRB, 0.85298E+00_JPRB, 0.86061E+00_JPRB, 0.86635E+00_JPRB, 0.87516E+00_JPRB, 0.87914E+00_JPRB ,&
&  0.89126E+00_JPRB, 0.91185E+00_JPRB, 0.93312E+00_JPRB, 0.94452E+00_JPRB, 0.95374E+00_JPRB, 0.96445E+00_JPRB, 0.98068E+00_JPRB ,&
&  0.99571E+00_JPRB, 0.10134E+01_JPRB, 0.10288E+01_JPRB, 0.10109E+01_JPRB, 0.10021E+01_JPRB, 0.97949E+00_JPRB, 0.96278E+00_JPRB ,&
&  0.94300E+00_JPRB, 0.92879E+00_JPRB, 0.90621E+00_JPRB, 0.89411E+00_JPRB, 0.88213E+00_JPRB, 0.87269E+00_JPRB, 0.86326E+00_JPRB ,&
&  0.85513E+00_JPRB, 0.84727E+00_JPRB, 0.84258E+00_JPRB, 0.83982E+00_JPRB, 0.83699E+00_JPRB, 0.83168E+00_JPRB, 0.82744E+00_JPRB ,&
&  0.82107E+00_JPRB, 0.81730E+00_JPRB, 0.81431E+00_JPRB, 0.81493E+00_JPRB, 0.81749E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.81632E+00_JPRB, 0.82493E+00_JPRB, 0.83074E+00_JPRB, 0.83922E+00_JPRB, 0.83879E+00_JPRB, 0.84144E+00_JPRB, 0.83461E+00_JPRB ,&
&  0.82978E+00_JPRB, 0.82704E+00_JPRB, 0.82568E+00_JPRB, 0.82741E+00_JPRB, 0.83059E+00_JPRB, 0.83494E+00_JPRB, 0.84090E+00_JPRB ,&
&  0.84705E+00_JPRB, 0.84711E+00_JPRB, 0.85225E+00_JPRB, 0.86140E+00_JPRB, 0.86560E+00_JPRB, 0.87185E+00_JPRB, 0.87565E+00_JPRB ,&
&  0.87686E+00_JPRB, 0.87492E+00_JPRB, 0.87278E+00_JPRB, 0.86367E+00_JPRB, 0.86131E+00_JPRB, 0.86777E+00_JPRB, 0.87265E+00_JPRB ,&
&  0.84946E+00_JPRB, 0.83969E+00_JPRB, 0.86114E+00_JPRB, 0.86729E+00_JPRB, 0.81657E+00_JPRB, 0.77159E+00_JPRB, 0.75201E+00_JPRB ,&
&  0.72425E+00_JPRB, 0.68242E+00_JPRB, 0.66707E+00_JPRB, 0.65659E+00_JPRB, 0.65941E+00_JPRB, 0.66339E+00_JPRB, 0.67487E+00_JPRB ,&
&  0.68185E+00_JPRB, 0.69097E+00_JPRB, 0.70531E+00_JPRB, 0.72392E+00_JPRB, 0.73571E+00_JPRB, 0.74965E+00_JPRB, 0.76053E+00_JPRB ,&
&  0.77262E+00_JPRB, 0.78891E+00_JPRB, 0.81147E+00_JPRB, 0.83947E+00_JPRB, 0.86018E+00_JPRB, 0.87555E+00_JPRB, 0.88251E+00_JPRB ,&
&  0.91366E+00_JPRB, 0.89995E+00_JPRB, 0.90734E+00_JPRB, 0.91192E+00_JPRB, 0.91495E+00_JPRB, 0.92253E+00_JPRB, 0.92931E+00_JPRB ,&
&  0.94954E+00_JPRB, 0.97893E+00_JPRB, 0.98797E+00_JPRB, 0.99321E+00_JPRB, 0.99718E+00_JPRB, 0.10041E+01_JPRB, 0.10243E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10770E+01_JPRB, 0.11582E+01_JPRB, 0.11187E+01_JPRB, 0.10878E+01_JPRB, 0.10539E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.98685E+00_JPRB, 0.95803E+00_JPRB, 0.92965E+00_JPRB, 0.90443E+00_JPRB, 0.88735E+00_JPRB, 0.87029E+00_JPRB, 0.86541E+00_JPRB ,&
&  0.86168E+00_JPRB, 0.85633E+00_JPRB, 0.85304E+00_JPRB, 0.84983E+00_JPRB, 0.84207E+00_JPRB, 0.83377E+00_JPRB, 0.82764E+00_JPRB ,&
&  0.81734E+00_JPRB, 0.81195E+00_JPRB, 0.80747E+00_JPRB, 0.80810E+00_JPRB, 0.81034E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.80708E+00_JPRB, 0.82438E+00_JPRB, 0.83665E+00_JPRB, 0.84364E+00_JPRB, 0.83848E+00_JPRB, 0.84025E+00_JPRB, 0.83243E+00_JPRB ,&
&  0.82782E+00_JPRB, 0.82814E+00_JPRB, 0.82722E+00_JPRB, 0.82793E+00_JPRB, 0.82918E+00_JPRB, 0.83016E+00_JPRB, 0.83264E+00_JPRB ,&
&  0.83699E+00_JPRB, 0.84262E+00_JPRB, 0.84726E+00_JPRB, 0.85375E+00_JPRB, 0.86216E+00_JPRB, 0.87188E+00_JPRB, 0.87996E+00_JPRB ,&
&  0.88253E+00_JPRB, 0.88052E+00_JPRB, 0.88554E+00_JPRB, 0.88145E+00_JPRB, 0.87745E+00_JPRB, 0.87173E+00_JPRB, 0.86956E+00_JPRB ,&
&  0.87970E+00_JPRB, 0.89174E+00_JPRB, 0.87818E+00_JPRB, 0.87041E+00_JPRB, 0.86510E+00_JPRB, 0.82333E+00_JPRB, 0.78249E+00_JPRB ,&
&  0.74017E+00_JPRB, 0.70345E+00_JPRB, 0.68627E+00_JPRB, 0.67738E+00_JPRB, 0.68428E+00_JPRB, 0.70011E+00_JPRB, 0.71359E+00_JPRB ,&
&  0.70605E+00_JPRB, 0.72708E+00_JPRB, 0.74856E+00_JPRB, 0.76196E+00_JPRB, 0.77228E+00_JPRB, 0.78302E+00_JPRB, 0.79367E+00_JPRB ,&
&  0.81036E+00_JPRB, 0.83672E+00_JPRB, 0.88440E+00_JPRB, 0.93791E+00_JPRB, 0.97810E+00_JPRB, 0.99297E+00_JPRB, 0.10473E+01_JPRB ,&
&  0.11728E+01_JPRB, 0.10987E+01_JPRB, 0.10625E+01_JPRB, 0.10499E+01_JPRB, 0.10347E+01_JPRB, 0.10306E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10775E+01_JPRB, 0.10798E+01_JPRB, 0.10749E+01_JPRB, 0.10751E+01_JPRB, 0.10825E+01_JPRB, 0.11011E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.11789E+01_JPRB, 0.13212E+01_JPRB, 0.11829E+01_JPRB, 0.11090E+01_JPRB, 0.10709E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.10026E+01_JPRB, 0.97538E+00_JPRB, 0.94692E+00_JPRB, 0.91271E+00_JPRB, 0.88705E+00_JPRB, 0.86255E+00_JPRB, 0.85559E+00_JPRB ,&
&  0.85131E+00_JPRB, 0.84579E+00_JPRB, 0.84563E+00_JPRB, 0.84068E+00_JPRB, 0.83351E+00_JPRB, 0.83090E+00_JPRB, 0.82552E+00_JPRB ,&
&  0.81354E+00_JPRB, 0.80326E+00_JPRB, 0.79619E+00_JPRB, 0.79469E+00_JPRB, 0.79645E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.82803E+00_JPRB, 0.83768E+00_JPRB, 0.84798E+00_JPRB, 0.87027E+00_JPRB, 0.87628E+00_JPRB, 0.87156E+00_JPRB, 0.85259E+00_JPRB ,&
&  0.83995E+00_JPRB, 0.83534E+00_JPRB, 0.83864E+00_JPRB, 0.84302E+00_JPRB, 0.84534E+00_JPRB, 0.84560E+00_JPRB, 0.84339E+00_JPRB ,&
&  0.83951E+00_JPRB, 0.83967E+00_JPRB, 0.84353E+00_JPRB, 0.84955E+00_JPRB, 0.85749E+00_JPRB, 0.86643E+00_JPRB, 0.87697E+00_JPRB ,&
&  0.88547E+00_JPRB, 0.88765E+00_JPRB, 0.88977E+00_JPRB, 0.88870E+00_JPRB, 0.89140E+00_JPRB, 0.89730E+00_JPRB, 0.89535E+00_JPRB ,&
&  0.88437E+00_JPRB, 0.89422E+00_JPRB, 0.90526E+00_JPRB, 0.88880E+00_JPRB, 0.88041E+00_JPRB, 0.84950E+00_JPRB, 0.80953E+00_JPRB ,&
&  0.77554E+00_JPRB, 0.74160E+00_JPRB, 0.73273E+00_JPRB, 0.73460E+00_JPRB, 0.74255E+00_JPRB, 0.73798E+00_JPRB, 0.75997E+00_JPRB ,&
&  0.77781E+00_JPRB, 0.78947E+00_JPRB, 0.79872E+00_JPRB, 0.80568E+00_JPRB, 0.81325E+00_JPRB, 0.82729E+00_JPRB, 0.85712E+00_JPRB ,&
&  0.91140E+00_JPRB, 0.98184E+00_JPRB, 0.10577E+01_JPRB, 0.10369E+01_JPRB, 0.10498E+01_JPRB, 0.10842E+01_JPRB, 0.11450E+01_JPRB ,&
&  0.11874E+01_JPRB, 0.11839E+01_JPRB, 0.11832E+01_JPRB, 0.12288E+01_JPRB, 0.12570E+01_JPRB, 0.12856E+01_JPRB, 0.12788E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.12293E+01_JPRB, 0.12085E+01_JPRB, 0.12061E+01_JPRB, 0.12228E+01_JPRB, 0.12358E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11393E+01_JPRB, 0.10778E+01_JPRB, 0.10212E+01_JPRB, 0.10077E+01_JPRB, 0.10114E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.99463E+00_JPRB, 0.97168E+00_JPRB, 0.94786E+00_JPRB, 0.91459E+00_JPRB, 0.88390E+00_JPRB, 0.85063E+00_JPRB, 0.83725E+00_JPRB ,&
&  0.82965E+00_JPRB, 0.82516E+00_JPRB, 0.83058E+00_JPRB, 0.81621E+00_JPRB, 0.81490E+00_JPRB, 0.81595E+00_JPRB, 0.80355E+00_JPRB ,&
&  0.79382E+00_JPRB, 0.79075E+00_JPRB, 0.78377E+00_JPRB, 0.78981E+00_JPRB, 0.80242E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.85685E+00_JPRB, 0.88910E+00_JPRB, 0.91829E+00_JPRB, 0.93030E+00_JPRB, 0.93701E+00_JPRB, 0.93555E+00_JPRB, 0.91574E+00_JPRB ,&
&  0.90746E+00_JPRB, 0.87983E+00_JPRB, 0.85893E+00_JPRB, 0.83876E+00_JPRB, 0.82420E+00_JPRB, 0.81732E+00_JPRB, 0.82130E+00_JPRB ,&
&  0.82840E+00_JPRB, 0.83489E+00_JPRB, 0.83985E+00_JPRB, 0.84369E+00_JPRB, 0.84969E+00_JPRB, 0.85859E+00_JPRB, 0.87064E+00_JPRB ,&
&  0.88343E+00_JPRB, 0.89248E+00_JPRB, 0.89988E+00_JPRB, 0.90538E+00_JPRB, 0.90943E+00_JPRB, 0.91036E+00_JPRB, 0.90441E+00_JPRB ,&
&  0.90564E+00_JPRB, 0.90557E+00_JPRB, 0.90878E+00_JPRB, 0.89944E+00_JPRB, 0.89261E+00_JPRB, 0.86855E+00_JPRB, 0.82520E+00_JPRB ,&
&  0.80469E+00_JPRB, 0.79314E+00_JPRB, 0.79874E+00_JPRB, 0.79566E+00_JPRB, 0.80736E+00_JPRB, 0.82300E+00_JPRB, 0.82744E+00_JPRB ,&
&  0.82671E+00_JPRB, 0.82814E+00_JPRB, 0.84198E+00_JPRB, 0.85301E+00_JPRB, 0.86636E+00_JPRB, 0.88878E+00_JPRB, 0.91622E+00_JPRB ,&
&  0.96340E+00_JPRB, 0.10325E+01_JPRB, 0.11351E+01_JPRB, 0.11657E+01_JPRB, 0.12485E+01_JPRB, 0.14405E+01_JPRB, 0.13712E+01_JPRB ,&
&  0.13956E+01_JPRB, 0.14259E+01_JPRB, 0.14853E+01_JPRB, 0.15799E+01_JPRB, 0.15603E+01_JPRB, 0.15412E+01_JPRB, 0.15214E+01_JPRB ,&
&  0.14574E+01_JPRB, 0.14313E+01_JPRB, 0.14055E+01_JPRB, 0.14286E+01_JPRB, 0.14136E+01_JPRB, 0.13940E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.12216E+01_JPRB, 0.11430E+01_JPRB, 0.10712E+01_JPRB, 0.10352E+01_JPRB, 0.10150E+01_JPRB, 0.10023E+01_JPRB ,&
&  0.98582E+00_JPRB, 0.97250E+00_JPRB, 0.96016E+00_JPRB, 0.93917E+00_JPRB, 0.90290E+00_JPRB, 0.86215E+00_JPRB, 0.84994E+00_JPRB ,&
&  0.83932E+00_JPRB, 0.82826E+00_JPRB, 0.82277E+00_JPRB, 0.81523E+00_JPRB, 0.81549E+00_JPRB, 0.80906E+00_JPRB, 0.80009E+00_JPRB ,&
&  0.80298E+00_JPRB, 0.79564E+00_JPRB, 0.78988E+00_JPRB, 0.79594E+00_JPRB, 0.82624E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.91627E+00_JPRB, 0.95599E+00_JPRB, 0.97200E+00_JPRB, 0.98158E+00_JPRB, 0.99024E+00_JPRB, 0.98894E+00_JPRB, 0.95129E+00_JPRB ,&
&  0.93630E+00_JPRB, 0.87896E+00_JPRB, 0.85051E+00_JPRB, 0.83216E+00_JPRB, 0.82346E+00_JPRB, 0.81332E+00_JPRB, 0.80339E+00_JPRB ,&
&  0.80806E+00_JPRB, 0.82332E+00_JPRB, 0.83871E+00_JPRB, 0.84654E+00_JPRB, 0.85384E+00_JPRB, 0.85996E+00_JPRB, 0.86750E+00_JPRB ,&
&  0.87882E+00_JPRB, 0.89185E+00_JPRB, 0.90662E+00_JPRB, 0.91364E+00_JPRB, 0.91692E+00_JPRB, 0.91820E+00_JPRB, 0.91708E+00_JPRB ,&
&  0.91587E+00_JPRB, 0.90583E+00_JPRB, 0.90986E+00_JPRB, 0.90681E+00_JPRB, 0.89534E+00_JPRB, 0.89042E+00_JPRB, 0.86007E+00_JPRB ,&
&  0.83547E+00_JPRB, 0.85444E+00_JPRB, 0.85531E+00_JPRB, 0.87021E+00_JPRB, 0.87934E+00_JPRB, 0.88410E+00_JPRB, 0.88569E+00_JPRB ,&
&  0.88399E+00_JPRB, 0.88227E+00_JPRB, 0.88648E+00_JPRB, 0.89412E+00_JPRB, 0.90548E+00_JPRB, 0.92098E+00_JPRB, 0.93663E+00_JPRB ,&
&  0.95221E+00_JPRB, 0.93449E+00_JPRB, 0.10059E+01_JPRB, 0.11857E+01_JPRB, 0.15430E+01_JPRB, 0.15912E+01_JPRB, 0.16471E+01_JPRB ,&
&  0.17496E+01_JPRB, 0.17501E+01_JPRB, 0.17905E+01_JPRB, 0.18225E+01_JPRB, 0.18156E+01_JPRB, 0.17925E+01_JPRB, 0.17904E+01_JPRB ,&
&  0.17297E+01_JPRB, 0.17401E+01_JPRB, 0.17043E+01_JPRB, 0.16849E+01_JPRB, 0.16324E+01_JPRB, 0.15973E+01_JPRB, 0.15446E+01_JPRB ,&
&  0.14710E+01_JPRB, 0.14018E+01_JPRB, 0.13063E+01_JPRB, 0.12017E+01_JPRB, 0.11183E+01_JPRB, 0.10761E+01_JPRB, 0.10389E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.99949E+00_JPRB, 0.98178E+00_JPRB, 0.97176E+00_JPRB, 0.94673E+00_JPRB, 0.92005E+00_JPRB, 0.91552E+00_JPRB ,&
&  0.89094E+00_JPRB, 0.87156E+00_JPRB, 0.85566E+00_JPRB, 0.84717E+00_JPRB, 0.85378E+00_JPRB, 0.85738E+00_JPRB, 0.85742E+00_JPRB ,&
&  0.86624E+00_JPRB, 0.85911E+00_JPRB, 0.86064E+00_JPRB, 0.87087E+00_JPRB, 0.88860E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.97209E+00_JPRB, 0.10014E+01_JPRB, 0.10144E+01_JPRB, 0.10278E+01_JPRB, 0.10450E+01_JPRB, 0.10419E+01_JPRB, 0.10293E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.99978E+00_JPRB, 0.10299E+01_JPRB, 0.90273E+00_JPRB, 0.85260E+00_JPRB, 0.84058E+00_JPRB, 0.82598E+00_JPRB ,&
&  0.82571E+00_JPRB, 0.83449E+00_JPRB, 0.84461E+00_JPRB, 0.85528E+00_JPRB, 0.86669E+00_JPRB, 0.86972E+00_JPRB, 0.87292E+00_JPRB ,&
&  0.88018E+00_JPRB, 0.89386E+00_JPRB, 0.91503E+00_JPRB, 0.92602E+00_JPRB, 0.93714E+00_JPRB, 0.94757E+00_JPRB, 0.94264E+00_JPRB ,&
&  0.93082E+00_JPRB, 0.93317E+00_JPRB, 0.93165E+00_JPRB, 0.91701E+00_JPRB, 0.90183E+00_JPRB, 0.89789E+00_JPRB, 0.91318E+00_JPRB ,&
&  0.91165E+00_JPRB, 0.90288E+00_JPRB, 0.92061E+00_JPRB, 0.92742E+00_JPRB, 0.92800E+00_JPRB, 0.92546E+00_JPRB, 0.92266E+00_JPRB ,&
&  0.92129E+00_JPRB, 0.91836E+00_JPRB, 0.92148E+00_JPRB, 0.92519E+00_JPRB, 0.93372E+00_JPRB, 0.94774E+00_JPRB, 0.95604E+00_JPRB ,&
&  0.98175E+00_JPRB, 0.97628E+00_JPRB, 0.11297E+01_JPRB, 0.13056E+01_JPRB, 0.16174E+01_JPRB, 0.18053E+01_JPRB, 0.19031E+01_JPRB ,&
&  0.21460E+01_JPRB, 0.20834E+01_JPRB, 0.21747E+01_JPRB, 0.21555E+01_JPRB, 0.21209E+01_JPRB, 0.20680E+01_JPRB, 0.19987E+01_JPRB ,&
&  0.19749E+01_JPRB, 0.19385E+01_JPRB, 0.20062E+01_JPRB, 0.19549E+01_JPRB, 0.18511E+01_JPRB, 0.17955E+01_JPRB, 0.17471E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.16680E+01_JPRB, 0.15859E+01_JPRB, 0.13845E+01_JPRB, 0.12866E+01_JPRB, 0.12087E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.10622E+01_JPRB, 0.10137E+01_JPRB, 0.97785E+00_JPRB, 0.97133E+00_JPRB, 0.95330E+00_JPRB, 0.93598E+00_JPRB, 0.93605E+00_JPRB ,&
&  0.91823E+00_JPRB, 0.91046E+00_JPRB, 0.90235E+00_JPRB, 0.91038E+00_JPRB, 0.91973E+00_JPRB, 0.92526E+00_JPRB, 0.92550E+00_JPRB ,&
&  0.91648E+00_JPRB, 0.90580E+00_JPRB, 0.91774E+00_JPRB, 0.92425E+00_JPRB, 0.94355E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.10068E+01_JPRB, 0.10370E+01_JPRB, 0.10593E+01_JPRB, 0.10791E+01_JPRB, 0.11046E+01_JPRB, 0.11267E+01_JPRB, 0.11173E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.11165E+01_JPRB, 0.11142E+01_JPRB, 0.10944E+01_JPRB, 0.10575E+01_JPRB, 0.96658E+00_JPRB, 0.85900E+00_JPRB ,&
&  0.82932E+00_JPRB, 0.83996E+00_JPRB, 0.85629E+00_JPRB, 0.86443E+00_JPRB, 0.86926E+00_JPRB, 0.87556E+00_JPRB, 0.87746E+00_JPRB ,&
&  0.88514E+00_JPRB, 0.89722E+00_JPRB, 0.91860E+00_JPRB, 0.94882E+00_JPRB, 0.97718E+00_JPRB, 0.98595E+00_JPRB, 0.98269E+00_JPRB ,&
&  0.96730E+00_JPRB, 0.96062E+00_JPRB, 0.94347E+00_JPRB, 0.92448E+00_JPRB, 0.92468E+00_JPRB, 0.93189E+00_JPRB, 0.95434E+00_JPRB ,&
&  0.97568E+00_JPRB, 0.98519E+00_JPRB, 0.98588E+00_JPRB, 0.98708E+00_JPRB, 0.98510E+00_JPRB, 0.97711E+00_JPRB, 0.96604E+00_JPRB ,&
&  0.95655E+00_JPRB, 0.94996E+00_JPRB, 0.94615E+00_JPRB, 0.94927E+00_JPRB, 0.94864E+00_JPRB, 0.98751E+00_JPRB, 0.10257E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.11819E+01_JPRB, 0.13730E+01_JPRB, 0.14451E+01_JPRB, 0.17340E+01_JPRB, 0.19221E+01_JPRB, 0.21059E+01_JPRB ,&
&  0.22426E+01_JPRB, 0.21914E+01_JPRB, 0.23928E+01_JPRB, 0.24493E+01_JPRB, 0.24658E+01_JPRB, 0.24037E+01_JPRB, 0.22637E+01_JPRB ,&
&  0.21872E+01_JPRB, 0.22566E+01_JPRB, 0.23028E+01_JPRB, 0.22683E+01_JPRB, 0.21661E+01_JPRB, 0.21164E+01_JPRB, 0.20708E+01_JPRB ,&
&  0.20741E+01_JPRB, 0.20749E+01_JPRB, 0.19812E+01_JPRB, 0.18044E+01_JPRB, 0.15902E+01_JPRB, 0.14023E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.10159E+01_JPRB, 0.97494E+00_JPRB, 0.96542E+00_JPRB, 0.96501E+00_JPRB, 0.94436E+00_JPRB, 0.94392E+00_JPRB ,&
&  0.95446E+00_JPRB, 0.95887E+00_JPRB, 0.96262E+00_JPRB, 0.97290E+00_JPRB, 0.10050E+01_JPRB, 0.10201E+01_JPRB, 0.97323E+00_JPRB ,&
&  0.94100E+00_JPRB, 0.93595E+00_JPRB, 0.94158E+00_JPRB, 0.94996E+00_JPRB, 0.97534E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.10557E+01_JPRB, 0.10914E+01_JPRB, 0.11161E+01_JPRB, 0.11343E+01_JPRB, 0.11600E+01_JPRB, 0.11979E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11826E+01_JPRB, 0.11771E+01_JPRB, 0.11646E+01_JPRB, 0.11363E+01_JPRB, 0.10913E+01_JPRB, 0.99308E+00_JPRB ,&
&  0.88525E+00_JPRB, 0.89081E+00_JPRB, 0.87166E+00_JPRB, 0.88104E+00_JPRB, 0.88850E+00_JPRB, 0.88349E+00_JPRB, 0.87690E+00_JPRB ,&
&  0.88758E+00_JPRB, 0.90048E+00_JPRB, 0.92648E+00_JPRB, 0.95839E+00_JPRB, 0.96246E+00_JPRB, 0.96951E+00_JPRB, 0.99105E+00_JPRB ,&
&  0.98377E+00_JPRB, 0.96206E+00_JPRB, 0.94345E+00_JPRB, 0.93671E+00_JPRB, 0.95613E+00_JPRB, 0.98819E+00_JPRB, 0.10183E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.10327E+01_JPRB, 0.10292E+01_JPRB, 0.10259E+01_JPRB, 0.10219E+01_JPRB, 0.10121E+01_JPRB, 0.99660E+00_JPRB ,&
&  0.97957E+00_JPRB, 0.96938E+00_JPRB, 0.96486E+00_JPRB, 0.97249E+00_JPRB, 0.99281E+00_JPRB, 0.10713E+01_JPRB, 0.11581E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.13780E+01_JPRB, 0.15274E+01_JPRB, 0.17346E+01_JPRB, 0.19860E+01_JPRB, 0.21932E+01_JPRB, 0.23440E+01_JPRB ,&
&  0.25012E+01_JPRB, 0.24882E+01_JPRB, 0.27888E+01_JPRB, 0.27982E+01_JPRB, 0.26915E+01_JPRB, 0.26533E+01_JPRB, 0.24897E+01_JPRB ,&
&  0.23499E+01_JPRB, 0.22285E+01_JPRB, 0.23396E+01_JPRB, 0.23874E+01_JPRB, 0.24066E+01_JPRB, 0.24441E+01_JPRB, 0.24302E+01_JPRB ,&
&  0.24280E+01_JPRB, 0.23937E+01_JPRB, 0.20972E+01_JPRB, 0.17999E+01_JPRB, 0.14882E+01_JPRB, 0.14456E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.10093E+01_JPRB, 0.99626E+00_JPRB, 0.98789E+00_JPRB, 0.10050E+01_JPRB, 0.99941E+00_JPRB, 0.10089E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10570E+01_JPRB, 0.10623E+01_JPRB, 0.10878E+01_JPRB, 0.11288E+01_JPRB, 0.11301E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10088E+01_JPRB, 0.98204E+00_JPRB, 0.98423E+00_JPRB, 0.10123E+01_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.11258E+01_JPRB, 0.11540E+01_JPRB, 0.11737E+01_JPRB, 0.11951E+01_JPRB, 0.12257E+01_JPRB, 0.12594E+01_JPRB, 0.12599E+01_JPRB ,&
&  0.12456E+01_JPRB, 0.12345E+01_JPRB, 0.12248E+01_JPRB, 0.12069E+01_JPRB, 0.11828E+01_JPRB, 0.11372E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.95572E+00_JPRB, 0.93641E+00_JPRB, 0.87151E+00_JPRB, 0.87695E+00_JPRB, 0.95627E+00_JPRB, 0.92850E+00_JPRB, 0.92064E+00_JPRB ,&
&  0.90918E+00_JPRB, 0.92318E+00_JPRB, 0.94047E+00_JPRB, 0.95821E+00_JPRB, 0.94317E+00_JPRB, 0.96159E+00_JPRB, 0.99840E+00_JPRB ,&
&  0.99948E+00_JPRB, 0.98743E+00_JPRB, 0.98881E+00_JPRB, 0.99632E+00_JPRB, 0.10202E+01_JPRB, 0.10620E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.10980E+01_JPRB, 0.10882E+01_JPRB, 0.10796E+01_JPRB, 0.10696E+01_JPRB, 0.10550E+01_JPRB, 0.10367E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.10032E+01_JPRB, 0.10042E+01_JPRB, 0.10255E+01_JPRB, 0.10433E+01_JPRB, 0.11528E+01_JPRB, 0.13793E+01_JPRB ,&
&  0.16299E+01_JPRB, 0.17390E+01_JPRB, 0.19228E+01_JPRB, 0.23049E+01_JPRB, 0.25535E+01_JPRB, 0.27028E+01_JPRB, 0.27469E+01_JPRB ,&
&  0.27868E+01_JPRB, 0.27863E+01_JPRB, 0.27733E+01_JPRB, 0.28121E+01_JPRB, 0.27255E+01_JPRB, 0.27508E+01_JPRB, 0.26244E+01_JPRB ,&
&  0.24062E+01_JPRB, 0.23601E+01_JPRB, 0.24549E+01_JPRB, 0.25010E+01_JPRB, 0.25083E+01_JPRB, 0.25708E+01_JPRB, 0.26219E+01_JPRB ,&
&  0.25712E+01_JPRB, 0.22927E+01_JPRB, 0.17747E+01_JPRB, 0.16081E+01_JPRB, 0.12563E+01_JPRB, 0.12322E+01_JPRB, 0.13574E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.11986E+01_JPRB, 0.11030E+01_JPRB, 0.10630E+01_JPRB, 0.10730E+01_JPRB, 0.12079E+01_JPRB, 0.12439E+01_JPRB ,&
&  0.11678E+01_JPRB, 0.11889E+01_JPRB, 0.12330E+01_JPRB, 0.12929E+01_JPRB, 0.13385E+01_JPRB, 0.13338E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.12240E+01_JPRB, 0.11588E+01_JPRB, 0.11014E+01_JPRB, 0.10825E+01_JPRB, 0.10953E+01_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.12504E+01_JPRB, 0.12530E+01_JPRB, 0.12564E+01_JPRB, 0.12659E+01_JPRB, 0.12970E+01_JPRB, 0.13185E+01_JPRB, 0.13074E+01_JPRB ,&
&  0.12902E+01_JPRB, 0.12786E+01_JPRB, 0.12634E+01_JPRB, 0.12420E+01_JPRB, 0.12030E+01_JPRB, 0.11428E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.11125E+01_JPRB, 0.10505E+01_JPRB, 0.95939E+00_JPRB, 0.89268E+00_JPRB, 0.94080E+00_JPRB, 0.93494E+00_JPRB, 0.95513E+00_JPRB ,&
&  0.94635E+00_JPRB, 0.97919E+00_JPRB, 0.98913E+00_JPRB, 0.97933E+00_JPRB, 0.96392E+00_JPRB, 0.10099E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.11400E+01_JPRB, 0.11396E+01_JPRB, 0.11560E+01_JPRB, 0.11966E+01_JPRB, 0.11503E+01_JPRB, 0.11779E+01_JPRB ,&
&  0.11712E+01_JPRB, 0.11584E+01_JPRB, 0.11429E+01_JPRB, 0.11375E+01_JPRB, 0.11276E+01_JPRB, 0.11225E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11250E+01_JPRB, 0.11216E+01_JPRB, 0.10941E+01_JPRB, 0.10919E+01_JPRB, 0.11146E+01_JPRB, 0.12444E+01_JPRB, 0.13952E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.16949E+01_JPRB, 0.18395E+01_JPRB, 0.22533E+01_JPRB, 0.25593E+01_JPRB, 0.26835E+01_JPRB, 0.28092E+01_JPRB ,&
&  0.28749E+01_JPRB, 0.28970E+01_JPRB, 0.29384E+01_JPRB, 0.29393E+01_JPRB, 0.29511E+01_JPRB, 0.29314E+01_JPRB, 0.28559E+01_JPRB ,&
&  0.26734E+01_JPRB, 0.25989E+01_JPRB, 0.26559E+01_JPRB, 0.25768E+01_JPRB, 0.25253E+01_JPRB, 0.24092E+01_JPRB, 0.25581E+01_JPRB ,&
&  0.25614E+01_JPRB, 0.16536E+01_JPRB, 0.13683E+01_JPRB, 0.15356E+01_JPRB, 0.13063E+01_JPRB, 0.12631E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.12263E+01_JPRB, 0.12086E+01_JPRB, 0.11824E+01_JPRB, 0.12252E+01_JPRB, 0.15300E+01_JPRB, 0.15737E+01_JPRB ,&
&  0.15093E+01_JPRB, 0.14861E+01_JPRB, 0.15669E+01_JPRB, 0.15815E+01_JPRB, 0.15692E+01_JPRB, 0.15756E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.14179E+01_JPRB, 0.13383E+01_JPRB, 0.12752E+01_JPRB, 0.12491E+01_JPRB, 0.12478E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.14186E+01_JPRB, 0.13836E+01_JPRB, 0.13547E+01_JPRB, 0.13517E+01_JPRB, 0.13753E+01_JPRB, 0.13853E+01_JPRB, 0.13648E+01_JPRB ,&
&  0.13467E+01_JPRB, 0.13265E+01_JPRB, 0.13043E+01_JPRB, 0.12682E+01_JPRB, 0.12115E+01_JPRB, 0.11490E+01_JPRB, 0.10999E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.10792E+01_JPRB, 0.96816E+00_JPRB, 0.92539E+00_JPRB, 0.88748E+00_JPRB, 0.91571E+00_JPRB, 0.95086E+00_JPRB ,&
&  0.98803E+00_JPRB, 0.10051E+01_JPRB, 0.10229E+01_JPRB, 0.10320E+01_JPRB, 0.10448E+01_JPRB, 0.11258E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.13211E+01_JPRB, 0.13586E+01_JPRB, 0.13028E+01_JPRB, 0.13580E+01_JPRB, 0.13244E+01_JPRB, 0.12505E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.12491E+01_JPRB, 0.12223E+01_JPRB, 0.12204E+01_JPRB, 0.12199E+01_JPRB, 0.11956E+01_JPRB, 0.11684E+01_JPRB ,&
&  0.11344E+01_JPRB, 0.11138E+01_JPRB, 0.11090E+01_JPRB, 0.11023E+01_JPRB, 0.11202E+01_JPRB, 0.12096E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.13605E+01_JPRB, 0.12614E+01_JPRB, 0.12361E+01_JPRB, 0.16509E+01_JPRB, 0.22816E+01_JPRB, 0.25671E+01_JPRB, 0.25481E+01_JPRB ,&
&  0.27671E+01_JPRB, 0.29039E+01_JPRB, 0.30631E+01_JPRB, 0.31566E+01_JPRB, 0.31871E+01_JPRB, 0.31012E+01_JPRB, 0.29831E+01_JPRB ,&
&  0.28624E+01_JPRB, 0.28054E+01_JPRB, 0.27652E+01_JPRB, 0.27306E+01_JPRB, 0.26621E+01_JPRB, 0.27091E+01_JPRB, 0.28614E+01_JPRB ,&
&  0.26944E+01_JPRB, 0.23806E+01_JPRB, 0.16936E+01_JPRB, 0.16805E+01_JPRB, 0.13446E+01_JPRB, 0.12895E+01_JPRB, 0.13776E+01_JPRB ,&
&  0.13209E+01_JPRB, 0.13405E+01_JPRB, 0.13289E+01_JPRB, 0.13079E+01_JPRB, 0.15050E+01_JPRB, 0.18586E+01_JPRB, 0.19524E+01_JPRB ,&
&  0.20136E+01_JPRB, 0.18976E+01_JPRB, 0.20479E+01_JPRB, 0.20584E+01_JPRB, 0.20243E+01_JPRB, 0.19808E+01_JPRB, 0.19004E+01_JPRB ,&
&  0.17695E+01_JPRB, 0.16286E+01_JPRB, 0.15252E+01_JPRB, 0.14724E+01_JPRB, 0.14403E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.15215E+01_JPRB, 0.14558E+01_JPRB, 0.14128E+01_JPRB, 0.14022E+01_JPRB, 0.14179E+01_JPRB, 0.14216E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.13731E+01_JPRB, 0.13459E+01_JPRB, 0.13210E+01_JPRB, 0.12636E+01_JPRB, 0.12104E+01_JPRB, 0.11455E+01_JPRB, 0.10917E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.10297E+01_JPRB, 0.93478E+00_JPRB, 0.89536E+00_JPRB, 0.86214E+00_JPRB, 0.87603E+00_JPRB, 0.92344E+00_JPRB ,&
&  0.98145E+00_JPRB, 0.10282E+01_JPRB, 0.11149E+01_JPRB, 0.11506E+01_JPRB, 0.12030E+01_JPRB, 0.13274E+01_JPRB, 0.14844E+01_JPRB ,&
&  0.16909E+01_JPRB, 0.14963E+01_JPRB, 0.15077E+01_JPRB, 0.14628E+01_JPRB, 0.14342E+01_JPRB, 0.13835E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.12932E+01_JPRB, 0.12665E+01_JPRB, 0.12648E+01_JPRB, 0.12532E+01_JPRB, 0.12321E+01_JPRB, 0.12094E+01_JPRB, 0.11915E+01_JPRB ,&
&  0.11832E+01_JPRB, 0.11657E+01_JPRB, 0.11344E+01_JPRB, 0.11004E+01_JPRB, 0.11771E+01_JPRB, 0.12598E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.12256E+01_JPRB, 0.15753E+01_JPRB, 0.19269E+01_JPRB, 0.22770E+01_JPRB, 0.26292E+01_JPRB, 0.28609E+01_JPRB ,&
&  0.30550E+01_JPRB, 0.31468E+01_JPRB, 0.32271E+01_JPRB, 0.29112E+01_JPRB, 0.31730E+01_JPRB, 0.31244E+01_JPRB, 0.29884E+01_JPRB ,&
&  0.28419E+01_JPRB, 0.28283E+01_JPRB, 0.27342E+01_JPRB, 0.28128E+01_JPRB, 0.29913E+01_JPRB, 0.32744E+01_JPRB, 0.29318E+01_JPRB ,&
&  0.25257E+01_JPRB, 0.27071E+01_JPRB, 0.23603E+01_JPRB, 0.19235E+01_JPRB, 0.15667E+01_JPRB, 0.15271E+01_JPRB, 0.15084E+01_JPRB ,&
&  0.14718E+01_JPRB, 0.14506E+01_JPRB, 0.14252E+01_JPRB, 0.15556E+01_JPRB, 0.19510E+01_JPRB, 0.22981E+01_JPRB, 0.23055E+01_JPRB ,&
&  0.22586E+01_JPRB, 0.24613E+01_JPRB, 0.24971E+01_JPRB, 0.24741E+01_JPRB, 0.24551E+01_JPRB, 0.23897E+01_JPRB, 0.22613E+01_JPRB ,&
&  0.20893E+01_JPRB, 0.19289E+01_JPRB, 0.18023E+01_JPRB, 0.16912E+01_JPRB, 0.15975E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.15732E+01_JPRB, 0.15000E+01_JPRB, 0.14615E+01_JPRB, 0.14572E+01_JPRB, 0.14760E+01_JPRB, 0.14661E+01_JPRB, 0.14344E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.13433E+01_JPRB, 0.12858E+01_JPRB, 0.12332E+01_JPRB, 0.11947E+01_JPRB, 0.11389E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.10099E+01_JPRB, 0.94530E+00_JPRB, 0.95530E+00_JPRB, 0.94669E+00_JPRB, 0.89985E+00_JPRB, 0.92823E+00_JPRB ,&
&  0.98926E+00_JPRB, 0.10907E+01_JPRB, 0.12211E+01_JPRB, 0.13645E+01_JPRB, 0.14832E+01_JPRB, 0.16512E+01_JPRB, 0.17920E+01_JPRB ,&
&  0.17539E+01_JPRB, 0.17089E+01_JPRB, 0.16614E+01_JPRB, 0.15892E+01_JPRB, 0.14719E+01_JPRB, 0.13931E+01_JPRB, 0.13520E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.13100E+01_JPRB, 0.13048E+01_JPRB, 0.12893E+01_JPRB, 0.12835E+01_JPRB, 0.12763E+01_JPRB, 0.12735E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.11786E+01_JPRB, 0.11274E+01_JPRB, 0.11408E+01_JPRB, 0.11271E+01_JPRB, 0.11344E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.15086E+01_JPRB, 0.16563E+01_JPRB, 0.22128E+01_JPRB, 0.21762E+01_JPRB, 0.23435E+01_JPRB, 0.28314E+01_JPRB ,&
&  0.30834E+01_JPRB, 0.31638E+01_JPRB, 0.31731E+01_JPRB, 0.26233E+01_JPRB, 0.23479E+01_JPRB, 0.30514E+01_JPRB, 0.30597E+01_JPRB ,&
&  0.29182E+01_JPRB, 0.28851E+01_JPRB, 0.29362E+01_JPRB, 0.33888E+01_JPRB, 0.28601E+01_JPRB, 0.25830E+01_JPRB, 0.21118E+01_JPRB ,&
&  0.21988E+01_JPRB, 0.19527E+01_JPRB, 0.21958E+01_JPRB, 0.19585E+01_JPRB, 0.18970E+01_JPRB, 0.17731E+01_JPRB, 0.16165E+01_JPRB ,&
&  0.15606E+01_JPRB, 0.15526E+01_JPRB, 0.16266E+01_JPRB, 0.20519E+01_JPRB, 0.26552E+01_JPRB, 0.26828E+01_JPRB, 0.28329E+01_JPRB ,&
&  0.29568E+01_JPRB, 0.29999E+01_JPRB, 0.28161E+01_JPRB, 0.28148E+01_JPRB, 0.27273E+01_JPRB, 0.26073E+01_JPRB, 0.24615E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.21487E+01_JPRB, 0.19840E+01_JPRB, 0.18212E+01_JPRB, 0.16801E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.16426E+01_JPRB, 0.15822E+01_JPRB, 0.15585E+01_JPRB, 0.15557E+01_JPRB, 0.15441E+01_JPRB, 0.15066E+01_JPRB, 0.14550E+01_JPRB ,&
&  0.13914E+01_JPRB, 0.13120E+01_JPRB, 0.12422E+01_JPRB, 0.11913E+01_JPRB, 0.11658E+01_JPRB, 0.11371E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.10870E+01_JPRB, 0.10199E+01_JPRB, 0.99238E+00_JPRB, 0.10644E+01_JPRB, 0.10034E+01_JPRB, 0.94837E+00_JPRB ,&
&  0.10456E+01_JPRB, 0.11847E+01_JPRB, 0.13692E+01_JPRB, 0.15938E+01_JPRB, 0.17767E+01_JPRB, 0.19630E+01_JPRB, 0.20462E+01_JPRB ,&
&  0.20106E+01_JPRB, 0.18451E+01_JPRB, 0.16903E+01_JPRB, 0.15465E+01_JPRB, 0.14064E+01_JPRB, 0.13343E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.12965E+01_JPRB, 0.12841E+01_JPRB, 0.12414E+01_JPRB, 0.12302E+01_JPRB, 0.11822E+01_JPRB, 0.11471E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.11411E+01_JPRB, 0.11270E+01_JPRB, 0.11849E+01_JPRB, 0.11409E+01_JPRB, 0.11281E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.13131E+01_JPRB, 0.15306E+01_JPRB, 0.17249E+01_JPRB, 0.21326E+01_JPRB, 0.26860E+01_JPRB, 0.27829E+01_JPRB, 0.30671E+01_JPRB ,&
&  0.26788E+01_JPRB, 0.23072E+01_JPRB, 0.20436E+01_JPRB, 0.18167E+01_JPRB, 0.22253E+01_JPRB, 0.31973E+01_JPRB, 0.34935E+01_JPRB ,&
&  0.34108E+01_JPRB, 0.34294E+01_JPRB, 0.37415E+01_JPRB, 0.35900E+01_JPRB, 0.27477E+01_JPRB, 0.20807E+01_JPRB, 0.26355E+01_JPRB ,&
&  0.27936E+01_JPRB, 0.28399E+01_JPRB, 0.25904E+01_JPRB, 0.21350E+01_JPRB, 0.17931E+01_JPRB, 0.19973E+01_JPRB, 0.18703E+01_JPRB ,&
&  0.17437E+01_JPRB, 0.17282E+01_JPRB, 0.22825E+01_JPRB, 0.30817E+01_JPRB, 0.33696E+01_JPRB, 0.36465E+01_JPRB, 0.37236E+01_JPRB ,&
&  0.34913E+01_JPRB, 0.33400E+01_JPRB, 0.31481E+01_JPRB, 0.29276E+01_JPRB, 0.27278E+01_JPRB, 0.25255E+01_JPRB, 0.23638E+01_JPRB ,&
&  0.22285E+01_JPRB, 0.21041E+01_JPRB, 0.19692E+01_JPRB, 0.18383E+01_JPRB, 0.17266E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.17010E+01_JPRB, 0.16711E+01_JPRB, 0.16427E+01_JPRB, 0.16148E+01_JPRB, 0.15728E+01_JPRB, 0.15180E+01_JPRB, 0.14471E+01_JPRB ,&
&  0.13552E+01_JPRB, 0.12810E+01_JPRB, 0.12121E+01_JPRB, 0.11771E+01_JPRB, 0.11486E+01_JPRB, 0.11115E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.12091E+01_JPRB, 0.11132E+01_JPRB, 0.10894E+01_JPRB, 0.11354E+01_JPRB, 0.11564E+01_JPRB, 0.10039E+01_JPRB ,&
&  0.11562E+01_JPRB, 0.13157E+01_JPRB, 0.14759E+01_JPRB, 0.17246E+01_JPRB, 0.21184E+01_JPRB, 0.20657E+01_JPRB, 0.20624E+01_JPRB ,&
&  0.20088E+01_JPRB, 0.18429E+01_JPRB, 0.16987E+01_JPRB, 0.15260E+01_JPRB, 0.13892E+01_JPRB, 0.13376E+01_JPRB, 0.12962E+01_JPRB ,&
&  0.12937E+01_JPRB, 0.13060E+01_JPRB, 0.12875E+01_JPRB, 0.12403E+01_JPRB, 0.11964E+01_JPRB, 0.11627E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.11262E+01_JPRB, 0.11292E+01_JPRB, 0.12174E+01_JPRB, 0.12083E+01_JPRB, 0.11905E+01_JPRB, 0.13901E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.18944E+01_JPRB, 0.20238E+01_JPRB, 0.23269E+01_JPRB, 0.26640E+01_JPRB, 0.28123E+01_JPRB, 0.29598E+01_JPRB ,&
&  0.24068E+01_JPRB, 0.21633E+01_JPRB, 0.23986E+01_JPRB, 0.27016E+01_JPRB, 0.26680E+01_JPRB, 0.29358E+01_JPRB, 0.46582E+01_JPRB ,&
&  0.34651E+01_JPRB, 0.37646E+01_JPRB, 0.43087E+01_JPRB, 0.44361E+01_JPRB, 0.22129E+01_JPRB, 0.16360E+01_JPRB, 0.25147E+01_JPRB ,&
&  0.25806E+01_JPRB, 0.16403E+01_JPRB, 0.12788E+01_JPRB, 0.15486E+01_JPRB, 0.12680E+01_JPRB, 0.14564E+01_JPRB, 0.18273E+01_JPRB ,&
&  0.19983E+01_JPRB, 0.21227E+01_JPRB, 0.29985E+01_JPRB, 0.34504E+01_JPRB, 0.36420E+01_JPRB, 0.52291E+01_JPRB, 0.45347E+01_JPRB ,&
&  0.40947E+01_JPRB, 0.34049E+01_JPRB, 0.31981E+01_JPRB, 0.28396E+01_JPRB, 0.25876E+01_JPRB, 0.23600E+01_JPRB, 0.21740E+01_JPRB ,&
&  0.20354E+01_JPRB, 0.19211E+01_JPRB, 0.18381E+01_JPRB, 0.17731E+01_JPRB, 0.17367E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.17617E+01_JPRB, 0.17254E+01_JPRB, 0.16890E+01_JPRB, 0.16435E+01_JPRB, 0.15866E+01_JPRB, 0.15099E+01_JPRB, 0.14112E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.12433E+01_JPRB, 0.11884E+01_JPRB, 0.11581E+01_JPRB, 0.11228E+01_JPRB, 0.10924E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.12305E+01_JPRB, 0.12528E+01_JPRB, 0.12423E+01_JPRB, 0.12877E+01_JPRB, 0.13219E+01_JPRB, 0.12574E+01_JPRB ,&
&  0.14327E+01_JPRB, 0.15283E+01_JPRB, 0.16311E+01_JPRB, 0.17988E+01_JPRB, 0.19688E+01_JPRB, 0.19453E+01_JPRB, 0.19997E+01_JPRB ,&
&  0.19269E+01_JPRB, 0.18209E+01_JPRB, 0.16602E+01_JPRB, 0.15107E+01_JPRB, 0.14170E+01_JPRB, 0.13504E+01_JPRB, 0.13263E+01_JPRB ,&
&  0.13113E+01_JPRB, 0.13281E+01_JPRB, 0.13035E+01_JPRB, 0.12426E+01_JPRB, 0.12057E+01_JPRB, 0.11688E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.11499E+01_JPRB, 0.11878E+01_JPRB, 0.13202E+01_JPRB, 0.15734E+01_JPRB, 0.17014E+01_JPRB, 0.15981E+01_JPRB ,&
&  0.15416E+01_JPRB, 0.17603E+01_JPRB, 0.21901E+01_JPRB, 0.23230E+01_JPRB, 0.25260E+01_JPRB, 0.27255E+01_JPRB, 0.27208E+01_JPRB ,&
&  0.26942E+01_JPRB, 0.29559E+01_JPRB, 0.29687E+01_JPRB, 0.33773E+01_JPRB, 0.36663E+01_JPRB, 0.30745E+01_JPRB, 0.38284E+01_JPRB ,&
&  0.40004E+01_JPRB, 0.40176E+01_JPRB, 0.34440E+01_JPRB, 0.27188E+01_JPRB, 0.30523E+01_JPRB, 0.23512E+01_JPRB, 0.14752E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.11660E+01_JPRB, 0.10984E+01_JPRB, 0.11189E+01_JPRB, 0.11227E+01_JPRB, 0.14221E+01_JPRB, 0.18220E+01_JPRB ,&
&  0.25024E+01_JPRB, 0.29503E+01_JPRB, 0.33025E+01_JPRB, 0.35251E+01_JPRB, 0.35267E+01_JPRB, 0.49926E+01_JPRB, 0.44136E+01_JPRB ,&
&  0.35477E+01_JPRB, 0.31866E+01_JPRB, 0.29345E+01_JPRB, 0.26636E+01_JPRB, 0.24630E+01_JPRB, 0.23016E+01_JPRB, 0.21663E+01_JPRB ,&
&  0.20619E+01_JPRB, 0.19755E+01_JPRB, 0.19116E+01_JPRB, 0.18580E+01_JPRB, 0.18002E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.17639E+01_JPRB, 0.17203E+01_JPRB, 0.16772E+01_JPRB, 0.16123E+01_JPRB, 0.15384E+01_JPRB, 0.14432E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.12367E+01_JPRB, 0.11732E+01_JPRB, 0.11424E+01_JPRB, 0.11114E+01_JPRB, 0.10917E+01_JPRB, 0.10865E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.11787E+01_JPRB, 0.12564E+01_JPRB, 0.13669E+01_JPRB, 0.14505E+01_JPRB, 0.16894E+01_JPRB, 0.17306E+01_JPRB, 0.17979E+01_JPRB ,&
&  0.19111E+01_JPRB, 0.18968E+01_JPRB, 0.19370E+01_JPRB, 0.19645E+01_JPRB, 0.20289E+01_JPRB, 0.19902E+01_JPRB, 0.19582E+01_JPRB ,&
&  0.18698E+01_JPRB, 0.18106E+01_JPRB, 0.16650E+01_JPRB, 0.15294E+01_JPRB, 0.14244E+01_JPRB, 0.13671E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.13436E+01_JPRB, 0.13301E+01_JPRB, 0.12698E+01_JPRB, 0.12299E+01_JPRB, 0.12087E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.12119E+01_JPRB, 0.12658E+01_JPRB, 0.13416E+01_JPRB, 0.15183E+01_JPRB, 0.14931E+01_JPRB, 0.13294E+01_JPRB, 0.14947E+01_JPRB ,&
&  0.16310E+01_JPRB, 0.19735E+01_JPRB, 0.21008E+01_JPRB, 0.23684E+01_JPRB, 0.24595E+01_JPRB, 0.24277E+01_JPRB, 0.26301E+01_JPRB ,&
&  0.29114E+01_JPRB, 0.39404E+01_JPRB, 0.31566E+01_JPRB, 0.32687E+01_JPRB, 0.49981E+01_JPRB, 0.42201E+01_JPRB, 0.39961E+01_JPRB ,&
&  0.45762E+01_JPRB, 0.42499E+01_JPRB, 0.45250E+01_JPRB, 0.41291E+01_JPRB, 0.69550E+01_JPRB, 0.77270E+01_JPRB, 0.22299E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.13132E+01_JPRB, 0.11943E+01_JPRB, 0.10895E+01_JPRB, 0.11743E+01_JPRB, 0.15900E+01_JPRB, 0.23788E+01_JPRB ,&
&  0.27842E+01_JPRB, 0.34534E+01_JPRB, 0.36381E+01_JPRB, 0.39293E+01_JPRB, 0.36153E+01_JPRB, 0.40290E+01_JPRB, 0.32511E+01_JPRB ,&
&  0.30849E+01_JPRB, 0.29342E+01_JPRB, 0.26644E+01_JPRB, 0.24323E+01_JPRB, 0.22922E+01_JPRB, 0.21915E+01_JPRB, 0.20979E+01_JPRB ,&
&  0.20184E+01_JPRB, 0.19636E+01_JPRB, 0.19121E+01_JPRB, 0.18613E+01_JPRB, 0.18112E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.17008E+01_JPRB, 0.16588E+01_JPRB, 0.16035E+01_JPRB, 0.15198E+01_JPRB, 0.14465E+01_JPRB, 0.13596E+01_JPRB, 0.12557E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11621E+01_JPRB, 0.11251E+01_JPRB, 0.10901E+01_JPRB, 0.10853E+01_JPRB, 0.10996E+01_JPRB, 0.11476E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.13254E+01_JPRB, 0.14647E+01_JPRB, 0.17993E+01_JPRB, 0.21439E+01_JPRB, 0.22219E+01_JPRB, 0.27481E+01_JPRB ,&
&  0.26714E+01_JPRB, 0.24707E+01_JPRB, 0.23743E+01_JPRB, 0.22526E+01_JPRB, 0.21502E+01_JPRB, 0.20366E+01_JPRB, 0.19522E+01_JPRB ,&
&  0.18526E+01_JPRB, 0.18177E+01_JPRB, 0.16688E+01_JPRB, 0.15403E+01_JPRB, 0.14606E+01_JPRB, 0.13920E+01_JPRB, 0.13520E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.13834E+01_JPRB, 0.13782E+01_JPRB, 0.13266E+01_JPRB, 0.12872E+01_JPRB, 0.12714E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.13359E+01_JPRB, 0.14279E+01_JPRB, 0.15293E+01_JPRB, 0.16532E+01_JPRB, 0.14215E+01_JPRB, 0.16079E+01_JPRB, 0.16948E+01_JPRB ,&
&  0.18731E+01_JPRB, 0.20271E+01_JPRB, 0.20229E+01_JPRB, 0.21552E+01_JPRB, 0.23744E+01_JPRB, 0.24196E+01_JPRB, 0.26663E+01_JPRB ,&
&  0.33689E+01_JPRB, 0.33649E+01_JPRB, 0.30745E+01_JPRB, 0.34675E+01_JPRB, 0.46437E+01_JPRB, 0.67823E+01_JPRB, 0.38014E+01_JPRB ,&
&  0.32147E+01_JPRB, 0.48014E+01_JPRB, 0.55390E+01_JPRB, 0.56403E+01_JPRB, 0.84255E+01_JPRB, 0.11593E+02_JPRB, 0.69697E+01_JPRB ,&
&  0.19215E+01_JPRB, 0.13398E+01_JPRB, 0.15336E+01_JPRB, 0.13572E+01_JPRB, 0.12665E+01_JPRB, 0.17185E+01_JPRB, 0.42600E+01_JPRB ,&
&  0.34483E+01_JPRB, 0.40002E+01_JPRB, 0.41344E+01_JPRB, 0.40516E+01_JPRB, 0.36233E+01_JPRB, 0.34599E+01_JPRB, 0.29828E+01_JPRB ,&
&  0.25675E+01_JPRB, 0.23821E+01_JPRB, 0.21586E+01_JPRB, 0.20327E+01_JPRB, 0.19894E+01_JPRB, 0.19615E+01_JPRB, 0.18981E+01_JPRB ,&
&  0.18548E+01_JPRB, 0.18262E+01_JPRB, 0.17968E+01_JPRB, 0.17724E+01_JPRB, 0.17433E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.15955E+01_JPRB, 0.15561E+01_JPRB, 0.14822E+01_JPRB, 0.14181E+01_JPRB, 0.13914E+01_JPRB, 0.13198E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12149E+01_JPRB, 0.11900E+01_JPRB, 0.11595E+01_JPRB, 0.11624E+01_JPRB, 0.11403E+01_JPRB, 0.11665E+01_JPRB, 0.12259E+01_JPRB ,&
&  0.13178E+01_JPRB, 0.14298E+01_JPRB, 0.15793E+01_JPRB, 0.19522E+01_JPRB, 0.24676E+01_JPRB, 0.26126E+01_JPRB, 0.32635E+01_JPRB ,&
&  0.37872E+01_JPRB, 0.33990E+01_JPRB, 0.28659E+01_JPRB, 0.25444E+01_JPRB, 0.23177E+01_JPRB, 0.22097E+01_JPRB, 0.20276E+01_JPRB ,&
&  0.19116E+01_JPRB, 0.18828E+01_JPRB, 0.17025E+01_JPRB, 0.15625E+01_JPRB, 0.14577E+01_JPRB, 0.14084E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.14310E+01_JPRB, 0.14585E+01_JPRB, 0.14583E+01_JPRB, 0.14275E+01_JPRB, 0.13949E+01_JPRB, 0.13851E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.15583E+01_JPRB, 0.16749E+01_JPRB, 0.17527E+01_JPRB, 0.17453E+01_JPRB, 0.17725E+01_JPRB, 0.18097E+01_JPRB, 0.19689E+01_JPRB ,&
&  0.20894E+01_JPRB, 0.21595E+01_JPRB, 0.21923E+01_JPRB, 0.22109E+01_JPRB, 0.24006E+01_JPRB, 0.26486E+01_JPRB, 0.29834E+01_JPRB ,&
&  0.35217E+01_JPRB, 0.37265E+01_JPRB, 0.34640E+01_JPRB, 0.37763E+01_JPRB, 0.45661E+01_JPRB, 0.57294E+01_JPRB, 0.53792E+01_JPRB ,&
&  0.47066E+01_JPRB, 0.44890E+01_JPRB, 0.54569E+01_JPRB, 0.72274E+01_JPRB, 0.82697E+01_JPRB, 0.85226E+01_JPRB, 0.10342E+02_JPRB ,&
&  0.85425E+01_JPRB, 0.38301E+01_JPRB, 0.29224E+01_JPRB, 0.35213E+01_JPRB, 0.24298E+01_JPRB, 0.24645E+01_JPRB, 0.34265E+01_JPRB ,&
&  0.37004E+01_JPRB, 0.42833E+01_JPRB, 0.44336E+01_JPRB, 0.36510E+01_JPRB, 0.33556E+01_JPRB, 0.30933E+01_JPRB, 0.25680E+01_JPRB ,&
&  0.22001E+01_JPRB, 0.19739E+01_JPRB, 0.18414E+01_JPRB, 0.17560E+01_JPRB, 0.17053E+01_JPRB, 0.16566E+01_JPRB, 0.16292E+01_JPRB ,&
&  0.16008E+01_JPRB, 0.15905E+01_JPRB, 0.15941E+01_JPRB, 0.16008E+01_JPRB, 0.16082E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.15027E+01_JPRB, 0.14489E+01_JPRB, 0.14297E+01_JPRB, 0.14345E+01_JPRB, 0.14146E+01_JPRB, 0.13815E+01_JPRB, 0.13521E+01_JPRB ,&
&  0.12932E+01_JPRB, 0.13045E+01_JPRB, 0.13525E+01_JPRB, 0.13500E+01_JPRB, 0.13024E+01_JPRB, 0.13095E+01_JPRB, 0.13678E+01_JPRB ,&
&  0.14614E+01_JPRB, 0.16421E+01_JPRB, 0.19025E+01_JPRB, 0.24598E+01_JPRB, 0.31328E+01_JPRB, 0.43383E+01_JPRB, 0.33719E+01_JPRB ,&
&  0.37187E+01_JPRB, 0.38114E+01_JPRB, 0.30537E+01_JPRB, 0.27415E+01_JPRB, 0.25355E+01_JPRB, 0.24407E+01_JPRB, 0.21487E+01_JPRB ,&
&  0.20599E+01_JPRB, 0.19437E+01_JPRB, 0.17613E+01_JPRB, 0.15943E+01_JPRB, 0.14814E+01_JPRB, 0.15179E+01_JPRB, 0.15309E+01_JPRB ,&
&  0.15508E+01_JPRB, 0.15943E+01_JPRB, 0.15793E+01_JPRB, 0.15780E+01_JPRB, 0.15617E+01_JPRB, 0.15641E+01_JPRB, 0.16552E+01_JPRB ,&
&  0.17847E+01_JPRB, 0.18933E+01_JPRB, 0.19677E+01_JPRB, 0.19089E+01_JPRB, 0.19928E+01_JPRB, 0.20964E+01_JPRB, 0.22885E+01_JPRB ,&
&  0.23028E+01_JPRB, 0.21154E+01_JPRB, 0.21886E+01_JPRB, 0.24081E+01_JPRB, 0.26740E+01_JPRB, 0.28010E+01_JPRB, 0.31107E+01_JPRB ,&
&  0.38841E+01_JPRB, 0.45977E+01_JPRB, 0.49879E+01_JPRB, 0.43655E+01_JPRB, 0.44947E+01_JPRB, 0.54878E+01_JPRB, 0.65089E+01_JPRB ,&
&  0.69069E+01_JPRB, 0.68272E+01_JPRB, 0.73405E+01_JPRB, 0.72245E+01_JPRB, 0.69321E+01_JPRB, 0.65618E+01_JPRB, 0.69354E+01_JPRB ,&
&  0.79260E+01_JPRB, 0.85745E+01_JPRB, 0.62162E+01_JPRB, 0.47445E+01_JPRB, 0.44543E+01_JPRB, 0.38110E+01_JPRB, 0.35626E+01_JPRB ,&
&  0.40960E+01_JPRB, 0.42047E+01_JPRB, 0.40539E+01_JPRB, 0.42046E+01_JPRB, 0.32554E+01_JPRB, 0.26050E+01_JPRB, 0.19731E+01_JPRB ,&
&  0.15997E+01_JPRB, 0.14718E+01_JPRB, 0.14242E+01_JPRB, 0.14107E+01_JPRB, 0.14556E+01_JPRB, 0.14636E+01_JPRB, 0.14665E+01_JPRB ,&
&  0.14468E+01_JPRB, 0.14703E+01_JPRB, 0.15068E+01_JPRB, 0.15377E+01_JPRB, 0.15370E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.14360E+01_JPRB, 0.14286E+01_JPRB, 0.14530E+01_JPRB, 0.14761E+01_JPRB, 0.14735E+01_JPRB, 0.14949E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.14241E+01_JPRB, 0.14751E+01_JPRB, 0.14565E+01_JPRB, 0.14360E+01_JPRB, 0.14585E+01_JPRB, 0.15459E+01_JPRB ,&
&  0.16757E+01_JPRB, 0.19490E+01_JPRB, 0.24137E+01_JPRB, 0.30374E+01_JPRB, 0.35480E+01_JPRB, 0.41976E+01_JPRB, 0.45385E+01_JPRB ,&
&  0.32851E+01_JPRB, 0.39491E+01_JPRB, 0.29467E+01_JPRB, 0.27682E+01_JPRB, 0.27340E+01_JPRB, 0.29863E+01_JPRB, 0.27065E+01_JPRB ,&
&  0.22460E+01_JPRB, 0.19566E+01_JPRB, 0.18866E+01_JPRB, 0.17769E+01_JPRB, 0.16396E+01_JPRB, 0.16365E+01_JPRB, 0.16649E+01_JPRB ,&
&  0.16595E+01_JPRB, 0.16681E+01_JPRB, 0.17081E+01_JPRB, 0.17409E+01_JPRB, 0.17373E+01_JPRB, 0.17548E+01_JPRB, 0.18111E+01_JPRB ,&
&  0.19624E+01_JPRB, 0.20928E+01_JPRB, 0.21345E+01_JPRB, 0.21882E+01_JPRB, 0.22301E+01_JPRB, 0.24561E+01_JPRB, 0.26698E+01_JPRB ,&
&  0.26071E+01_JPRB, 0.23594E+01_JPRB, 0.24053E+01_JPRB, 0.25639E+01_JPRB, 0.27094E+01_JPRB, 0.30983E+01_JPRB, 0.33999E+01_JPRB ,&
&  0.43456E+01_JPRB, 0.45835E+01_JPRB, 0.55979E+01_JPRB, 0.45494E+01_JPRB, 0.45057E+01_JPRB, 0.54384E+01_JPRB, 0.56444E+01_JPRB ,&
&  0.54206E+01_JPRB, 0.64249E+01_JPRB, 0.67040E+01_JPRB, 0.68680E+01_JPRB, 0.66170E+01_JPRB, 0.59220E+01_JPRB, 0.55133E+01_JPRB ,&
&  0.56818E+01_JPRB, 0.60263E+01_JPRB, 0.57555E+01_JPRB, 0.47003E+01_JPRB, 0.43359E+01_JPRB, 0.38608E+01_JPRB, 0.40615E+01_JPRB ,&
&  0.46510E+01_JPRB, 0.48205E+01_JPRB, 0.45345E+01_JPRB, 0.39459E+01_JPRB, 0.23869E+01_JPRB, 0.17510E+01_JPRB, 0.13943E+01_JPRB ,&
&  0.12642E+01_JPRB, 0.12287E+01_JPRB, 0.12622E+01_JPRB, 0.13364E+01_JPRB, 0.13995E+01_JPRB, 0.14275E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.14478E+01_JPRB, 0.14616E+01_JPRB, 0.14619E+01_JPRB, 0.14531E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.14475E+01_JPRB, 0.14710E+01_JPRB, 0.15007E+01_JPRB, 0.15489E+01_JPRB, 0.15812E+01_JPRB, 0.16732E+01_JPRB, 0.17650E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.13442E+01_JPRB, 0.13948E+01_JPRB, 0.14448E+01_JPRB, 0.14966E+01_JPRB, 0.15657E+01_JPRB, 0.16861E+01_JPRB ,&
&  0.19364E+01_JPRB, 0.22706E+01_JPRB, 0.26788E+01_JPRB, 0.30937E+01_JPRB, 0.36068E+01_JPRB, 0.41931E+01_JPRB, 0.43243E+01_JPRB ,&
&  0.35054E+01_JPRB, 0.29523E+01_JPRB, 0.30327E+01_JPRB, 0.29607E+01_JPRB, 0.29565E+01_JPRB, 0.30376E+01_JPRB, 0.30676E+01_JPRB ,&
&  0.25059E+01_JPRB, 0.21776E+01_JPRB, 0.21453E+01_JPRB, 0.21728E+01_JPRB, 0.17974E+01_JPRB, 0.18427E+01_JPRB, 0.18860E+01_JPRB ,&
&  0.18917E+01_JPRB, 0.18896E+01_JPRB, 0.19472E+01_JPRB, 0.20050E+01_JPRB, 0.19877E+01_JPRB, 0.20032E+01_JPRB, 0.20874E+01_JPRB ,&
&  0.22807E+01_JPRB, 0.24252E+01_JPRB, 0.25046E+01_JPRB, 0.25095E+01_JPRB, 0.26454E+01_JPRB, 0.28887E+01_JPRB, 0.29685E+01_JPRB ,&
&  0.28309E+01_JPRB, 0.27828E+01_JPRB, 0.28400E+01_JPRB, 0.29734E+01_JPRB, 0.29461E+01_JPRB, 0.35182E+01_JPRB, 0.38643E+01_JPRB ,&
&  0.43826E+01_JPRB, 0.47030E+01_JPRB, 0.55999E+01_JPRB, 0.43424E+01_JPRB, 0.47758E+01_JPRB, 0.52375E+01_JPRB, 0.52515E+01_JPRB ,&
&  0.50169E+01_JPRB, 0.54465E+01_JPRB, 0.54861E+01_JPRB, 0.56320E+01_JPRB, 0.56339E+01_JPRB, 0.48976E+01_JPRB, 0.51491E+01_JPRB ,&
&  0.49133E+01_JPRB, 0.54906E+01_JPRB, 0.51726E+01_JPRB, 0.49606E+01_JPRB, 0.41761E+01_JPRB, 0.39748E+01_JPRB, 0.41173E+01_JPRB ,&
&  0.46475E+01_JPRB, 0.45674E+01_JPRB, 0.37241E+01_JPRB, 0.27982E+01_JPRB, 0.15570E+01_JPRB, 0.12144E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.12193E+01_JPRB, 0.12943E+01_JPRB, 0.13605E+01_JPRB, 0.13971E+01_JPRB, 0.14085E+01_JPRB, 0.14120E+01_JPRB ,&
&  0.14288E+01_JPRB, 0.14368E+01_JPRB, 0.14330E+01_JPRB, 0.14346E+01_JPRB, 0.14387E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.14475E+01_JPRB, 0.14446E+01_JPRB, 0.14456E+01_JPRB, 0.14146E+01_JPRB, 0.13761E+01_JPRB, 0.13328E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.12007E+01_JPRB, 0.12820E+01_JPRB, 0.13843E+01_JPRB, 0.14821E+01_JPRB, 0.16002E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.20156E+01_JPRB, 0.21936E+01_JPRB, 0.24039E+01_JPRB, 0.27368E+01_JPRB, 0.31493E+01_JPRB, 0.33430E+01_JPRB, 0.34924E+01_JPRB ,&
&  0.32060E+01_JPRB, 0.23537E+01_JPRB, 0.28866E+01_JPRB, 0.31808E+01_JPRB, 0.32825E+01_JPRB, 0.33421E+01_JPRB, 0.31157E+01_JPRB ,&
&  0.24881E+01_JPRB, 0.24336E+01_JPRB, 0.25658E+01_JPRB, 0.25842E+01_JPRB, 0.20954E+01_JPRB, 0.21282E+01_JPRB, 0.21723E+01_JPRB ,&
&  0.22190E+01_JPRB, 0.22475E+01_JPRB, 0.23427E+01_JPRB, 0.24047E+01_JPRB, 0.24202E+01_JPRB, 0.24938E+01_JPRB, 0.26202E+01_JPRB ,&
&  0.27495E+01_JPRB, 0.28484E+01_JPRB, 0.29868E+01_JPRB, 0.30787E+01_JPRB, 0.31805E+01_JPRB, 0.33219E+01_JPRB, 0.32904E+01_JPRB ,&
&  0.31700E+01_JPRB, 0.30631E+01_JPRB, 0.32522E+01_JPRB, 0.36846E+01_JPRB, 0.40512E+01_JPRB, 0.37478E+01_JPRB, 0.42192E+01_JPRB ,&
&  0.44220E+01_JPRB, 0.46714E+01_JPRB, 0.47150E+01_JPRB, 0.55617E+01_JPRB, 0.39987E+01_JPRB, 0.45446E+01_JPRB, 0.42498E+01_JPRB ,&
&  0.45783E+01_JPRB, 0.44384E+01_JPRB, 0.43206E+01_JPRB, 0.43753E+01_JPRB, 0.44777E+01_JPRB, 0.40425E+01_JPRB, 0.42876E+01_JPRB ,&
&  0.45822E+01_JPRB, 0.48343E+01_JPRB, 0.49247E+01_JPRB, 0.50332E+01_JPRB, 0.46830E+01_JPRB, 0.46033E+01_JPRB, 0.44626E+01_JPRB ,&
&  0.46612E+01_JPRB, 0.35992E+01_JPRB, 0.26599E+01_JPRB, 0.18581E+01_JPRB, 0.12281E+01_JPRB, 0.10372E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.12387E+01_JPRB, 0.12967E+01_JPRB, 0.13385E+01_JPRB, 0.13652E+01_JPRB, 0.13855E+01_JPRB, 0.14165E+01_JPRB ,&
&  0.14311E+01_JPRB, 0.14363E+01_JPRB, 0.14347E+01_JPRB, 0.14350E+01_JPRB, 0.14406E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.14035E+01_JPRB, 0.13730E+01_JPRB, 0.13155E+01_JPRB, 0.12277E+01_JPRB, 0.11355E+01_JPRB, 0.10470E+01_JPRB, 0.99734E+00_JPRB ,&
&  0.96383E+00_JPRB, 0.10035E+01_JPRB, 0.11059E+01_JPRB, 0.12429E+01_JPRB, 0.13979E+01_JPRB, 0.15528E+01_JPRB, 0.16938E+01_JPRB ,&
&  0.17817E+01_JPRB, 0.18355E+01_JPRB, 0.19547E+01_JPRB, 0.22326E+01_JPRB, 0.25541E+01_JPRB, 0.28129E+01_JPRB, 0.29999E+01_JPRB ,&
&  0.31663E+01_JPRB, 0.31949E+01_JPRB, 0.30004E+01_JPRB, 0.24465E+01_JPRB, 0.23411E+01_JPRB, 0.28975E+01_JPRB, 0.25190E+01_JPRB ,&
&  0.24105E+01_JPRB, 0.23278E+01_JPRB, 0.23469E+01_JPRB, 0.23903E+01_JPRB, 0.23589E+01_JPRB, 0.23581E+01_JPRB, 0.24460E+01_JPRB ,&
&  0.25183E+01_JPRB, 0.26156E+01_JPRB, 0.27128E+01_JPRB, 0.27832E+01_JPRB, 0.28765E+01_JPRB, 0.29953E+01_JPRB, 0.31084E+01_JPRB ,&
&  0.32104E+01_JPRB, 0.33637E+01_JPRB, 0.35415E+01_JPRB, 0.36966E+01_JPRB, 0.36740E+01_JPRB, 0.37551E+01_JPRB, 0.37137E+01_JPRB ,&
&  0.36456E+01_JPRB, 0.36039E+01_JPRB, 0.36742E+01_JPRB, 0.39855E+01_JPRB, 0.41602E+01_JPRB, 0.37454E+01_JPRB, 0.38621E+01_JPRB ,&
&  0.43052E+01_JPRB, 0.45143E+01_JPRB, 0.40651E+01_JPRB, 0.53165E+01_JPRB, 0.32861E+01_JPRB, 0.40321E+01_JPRB, 0.43420E+01_JPRB ,&
&  0.40107E+01_JPRB, 0.36683E+01_JPRB, 0.36159E+01_JPRB, 0.36795E+01_JPRB, 0.36402E+01_JPRB, 0.35409E+01_JPRB, 0.39247E+01_JPRB ,&
&  0.41879E+01_JPRB, 0.43692E+01_JPRB, 0.46932E+01_JPRB, 0.47285E+01_JPRB, 0.46840E+01_JPRB, 0.42945E+01_JPRB, 0.37068E+01_JPRB ,&
&  0.30648E+01_JPRB, 0.28476E+01_JPRB, 0.23533E+01_JPRB, 0.20190E+01_JPRB, 0.13067E+01_JPRB, 0.10051E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.11396E+01_JPRB, 0.11892E+01_JPRB, 0.12528E+01_JPRB, 0.13022E+01_JPRB, 0.13234E+01_JPRB, 0.13455E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.13855E+01_JPRB, 0.13911E+01_JPRB, 0.13944E+01_JPRB, 0.14045E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.11577E+01_JPRB, 0.11193E+01_JPRB, 0.10583E+01_JPRB, 0.10012E+01_JPRB, 0.94552E+00_JPRB, 0.88375E+00_JPRB, 0.87298E+00_JPRB ,&
&  0.85143E+00_JPRB, 0.86551E+00_JPRB, 0.91288E+00_JPRB, 0.97568E+00_JPRB, 0.10697E+01_JPRB, 0.11989E+01_JPRB, 0.13161E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.13846E+01_JPRB, 0.14560E+01_JPRB, 0.15631E+01_JPRB, 0.17835E+01_JPRB, 0.22015E+01_JPRB, 0.29023E+01_JPRB ,&
&  0.33137E+01_JPRB, 0.35143E+01_JPRB, 0.34190E+01_JPRB, 0.31751E+01_JPRB, 0.27985E+01_JPRB, 0.29800E+01_JPRB, 0.32141E+01_JPRB ,&
&  0.33550E+01_JPRB, 0.29673E+01_JPRB, 0.24915E+01_JPRB, 0.24701E+01_JPRB, 0.25058E+01_JPRB, 0.25545E+01_JPRB, 0.26520E+01_JPRB ,&
&  0.27568E+01_JPRB, 0.28399E+01_JPRB, 0.29277E+01_JPRB, 0.30409E+01_JPRB, 0.31413E+01_JPRB, 0.32408E+01_JPRB, 0.33655E+01_JPRB ,&
&  0.35000E+01_JPRB, 0.36255E+01_JPRB, 0.37295E+01_JPRB, 0.38625E+01_JPRB, 0.40701E+01_JPRB, 0.42729E+01_JPRB, 0.43394E+01_JPRB ,&
&  0.43335E+01_JPRB, 0.41398E+01_JPRB, 0.42861E+01_JPRB, 0.45634E+01_JPRB, 0.44858E+01_JPRB, 0.41658E+01_JPRB, 0.41349E+01_JPRB ,&
&  0.44040E+01_JPRB, 0.46013E+01_JPRB, 0.32933E+01_JPRB, 0.43113E+01_JPRB, 0.49973E+01_JPRB, 0.43692E+01_JPRB, 0.37732E+01_JPRB ,&
&  0.33074E+01_JPRB, 0.30543E+01_JPRB, 0.31810E+01_JPRB, 0.32773E+01_JPRB, 0.35590E+01_JPRB, 0.35119E+01_JPRB, 0.30535E+01_JPRB ,&
&  0.31696E+01_JPRB, 0.33969E+01_JPRB, 0.38720E+01_JPRB, 0.40629E+01_JPRB, 0.38666E+01_JPRB, 0.33868E+01_JPRB, 0.26780E+01_JPRB ,&
&  0.21401E+01_JPRB, 0.23099E+01_JPRB, 0.20273E+01_JPRB, 0.19463E+01_JPRB, 0.15837E+01_JPRB, 0.96745E+00_JPRB, 0.10332E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.11140E+01_JPRB, 0.11615E+01_JPRB, 0.11920E+01_JPRB, 0.12245E+01_JPRB, 0.12461E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12714E+01_JPRB, 0.12510E+01_JPRB, 0.12356E+01_JPRB, 0.12244E+01_JPRB, 0.11998E+01_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.10027E+01_JPRB, 0.98424E+00_JPRB, 0.96473E+00_JPRB, 0.95474E+00_JPRB, 0.95577E+00_JPRB, 0.94415E+00_JPRB, 0.92064E+00_JPRB ,&
&  0.88893E+00_JPRB, 0.87600E+00_JPRB, 0.84969E+00_JPRB, 0.82661E+00_JPRB, 0.84050E+00_JPRB, 0.91606E+00_JPRB, 0.99311E+00_JPRB ,&
&  0.10381E+01_JPRB, 0.10543E+01_JPRB, 0.10423E+01_JPRB, 0.10256E+01_JPRB, 0.10551E+01_JPRB, 0.11066E+01_JPRB, 0.14434E+01_JPRB ,&
&  0.20282E+01_JPRB, 0.28111E+01_JPRB, 0.33762E+01_JPRB, 0.36641E+01_JPRB, 0.36153E+01_JPRB, 0.33121E+01_JPRB, 0.41404E+01_JPRB ,&
&  0.36929E+01_JPRB, 0.27213E+01_JPRB, 0.24094E+01_JPRB, 0.22952E+01_JPRB, 0.23075E+01_JPRB, 0.23939E+01_JPRB, 0.25121E+01_JPRB ,&
&  0.26328E+01_JPRB, 0.27544E+01_JPRB, 0.28804E+01_JPRB, 0.30275E+01_JPRB, 0.31824E+01_JPRB, 0.33494E+01_JPRB, 0.35165E+01_JPRB ,&
&  0.37219E+01_JPRB, 0.38572E+01_JPRB, 0.40227E+01_JPRB, 0.39164E+01_JPRB, 0.43103E+01_JPRB, 0.46484E+01_JPRB, 0.48507E+01_JPRB ,&
&  0.48695E+01_JPRB, 0.47850E+01_JPRB, 0.50001E+01_JPRB, 0.50785E+01_JPRB, 0.51045E+01_JPRB, 0.49676E+01_JPRB, 0.47353E+01_JPRB ,&
&  0.48606E+01_JPRB, 0.45344E+01_JPRB, 0.32641E+01_JPRB, 0.35164E+01_JPRB, 0.39293E+01_JPRB, 0.31301E+01_JPRB, 0.33329E+01_JPRB ,&
&  0.28193E+01_JPRB, 0.24804E+01_JPRB, 0.22523E+01_JPRB, 0.23131E+01_JPRB, 0.27056E+01_JPRB, 0.30739E+01_JPRB, 0.26300E+01_JPRB ,&
&  0.21883E+01_JPRB, 0.23418E+01_JPRB, 0.26917E+01_JPRB, 0.28893E+01_JPRB, 0.27840E+01_JPRB, 0.24748E+01_JPRB, 0.21309E+01_JPRB ,&
&  0.19934E+01_JPRB, 0.18896E+01_JPRB, 0.17804E+01_JPRB, 0.16067E+01_JPRB, 0.11317E+01_JPRB, 0.88791E+00_JPRB, 0.92131E+00_JPRB ,&
&  0.94750E+00_JPRB, 0.95258E+00_JPRB, 0.97311E+00_JPRB, 0.99247E+00_JPRB, 0.10245E+01_JPRB, 0.10332E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.10690E+01_JPRB, 0.10651E+01_JPRB, 0.10667E+01_JPRB, 0.10372E+01_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.10741E+01_JPRB, 0.10795E+01_JPRB, 0.10950E+01_JPRB, 0.11126E+01_JPRB, 0.11352E+01_JPRB, 0.11173E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.10341E+01_JPRB, 0.10238E+01_JPRB, 0.10191E+01_JPRB, 0.10355E+01_JPRB, 0.10727E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.11624E+01_JPRB, 0.11446E+01_JPRB, 0.11216E+01_JPRB, 0.10641E+01_JPRB, 0.10118E+01_JPRB, 0.98067E+00_JPRB ,&
&  0.10320E+01_JPRB, 0.13341E+01_JPRB, 0.17202E+01_JPRB, 0.21004E+01_JPRB, 0.26114E+01_JPRB, 0.26100E+01_JPRB, 0.32369E+01_JPRB ,&
&  0.39302E+01_JPRB, 0.24922E+01_JPRB, 0.23435E+01_JPRB, 0.20534E+01_JPRB, 0.20471E+01_JPRB, 0.20566E+01_JPRB, 0.21619E+01_JPRB ,&
&  0.23050E+01_JPRB, 0.24072E+01_JPRB, 0.24990E+01_JPRB, 0.26010E+01_JPRB, 0.27249E+01_JPRB, 0.28393E+01_JPRB, 0.29046E+01_JPRB ,&
&  0.29593E+01_JPRB, 0.28779E+01_JPRB, 0.29600E+01_JPRB, 0.35362E+01_JPRB, 0.40891E+01_JPRB, 0.44020E+01_JPRB, 0.46458E+01_JPRB ,&
&  0.49677E+01_JPRB, 0.52051E+01_JPRB, 0.48874E+01_JPRB, 0.49010E+01_JPRB, 0.51909E+01_JPRB, 0.47740E+01_JPRB, 0.48482E+01_JPRB ,&
&  0.50190E+01_JPRB, 0.44203E+01_JPRB, 0.29256E+01_JPRB, 0.33384E+01_JPRB, 0.35540E+01_JPRB, 0.32005E+01_JPRB, 0.31165E+01_JPRB ,&
&  0.25994E+01_JPRB, 0.22295E+01_JPRB, 0.18284E+01_JPRB, 0.16784E+01_JPRB, 0.17510E+01_JPRB, 0.20360E+01_JPRB, 0.20975E+01_JPRB ,&
&  0.17332E+01_JPRB, 0.16983E+01_JPRB, 0.20084E+01_JPRB, 0.23841E+01_JPRB, 0.25403E+01_JPRB, 0.23111E+01_JPRB, 0.18803E+01_JPRB ,&
&  0.19654E+01_JPRB, 0.20701E+01_JPRB, 0.25801E+01_JPRB, 0.23571E+01_JPRB, 0.12247E+01_JPRB, 0.92459E+00_JPRB, 0.87327E+00_JPRB ,&
&  0.89409E+00_JPRB, 0.89617E+00_JPRB, 0.89322E+00_JPRB, 0.91216E+00_JPRB, 0.92595E+00_JPRB, 0.93146E+00_JPRB, 0.97478E+00_JPRB ,&
&  0.10260E+01_JPRB, 0.10327E+01_JPRB, 0.10457E+01_JPRB, 0.10516E+01_JPRB, 0.10615E+01_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.11908E+01_JPRB, 0.12127E+01_JPRB, 0.12427E+01_JPRB, 0.12528E+01_JPRB, 0.12615E+01_JPRB, 0.12527E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.12214E+01_JPRB, 0.12156E+01_JPRB, 0.12211E+01_JPRB, 0.12280E+01_JPRB, 0.12490E+01_JPRB, 0.12714E+01_JPRB, 0.13151E+01_JPRB ,&
&  0.13213E+01_JPRB, 0.13318E+01_JPRB, 0.13452E+01_JPRB, 0.13718E+01_JPRB, 0.13916E+01_JPRB, 0.13781E+01_JPRB, 0.13311E+01_JPRB ,&
&  0.13027E+01_JPRB, 0.12943E+01_JPRB, 0.13121E+01_JPRB, 0.13565E+01_JPRB, 0.15880E+01_JPRB, 0.19817E+01_JPRB, 0.26596E+01_JPRB ,&
&  0.30718E+01_JPRB, 0.24829E+01_JPRB, 0.20762E+01_JPRB, 0.18037E+01_JPRB, 0.17672E+01_JPRB, 0.17747E+01_JPRB, 0.16761E+01_JPRB ,&
&  0.16501E+01_JPRB, 0.16087E+01_JPRB, 0.16348E+01_JPRB, 0.17041E+01_JPRB, 0.17625E+01_JPRB, 0.17423E+01_JPRB, 0.17101E+01_JPRB ,&
&  0.15784E+01_JPRB, 0.15368E+01_JPRB, 0.16579E+01_JPRB, 0.19802E+01_JPRB, 0.26914E+01_JPRB, 0.31662E+01_JPRB, 0.30225E+01_JPRB ,&
&  0.33836E+01_JPRB, 0.40418E+01_JPRB, 0.43108E+01_JPRB, 0.43804E+01_JPRB, 0.42508E+01_JPRB, 0.40380E+01_JPRB, 0.39074E+01_JPRB ,&
&  0.38347E+01_JPRB, 0.34934E+01_JPRB, 0.28647E+01_JPRB, 0.31537E+01_JPRB, 0.31750E+01_JPRB, 0.31555E+01_JPRB, 0.27694E+01_JPRB ,&
&  0.23243E+01_JPRB, 0.20267E+01_JPRB, 0.18261E+01_JPRB, 0.16437E+01_JPRB, 0.15293E+01_JPRB, 0.15969E+01_JPRB, 0.15350E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.15029E+01_JPRB, 0.16331E+01_JPRB, 0.18608E+01_JPRB, 0.24325E+01_JPRB, 0.22693E+01_JPRB, 0.23317E+01_JPRB ,&
&  0.20874E+01_JPRB, 0.21854E+01_JPRB, 0.17628E+01_JPRB, 0.25409E+01_JPRB, 0.12059E+01_JPRB, 0.83632E+00_JPRB, 0.83767E+00_JPRB ,&
&  0.88477E+00_JPRB, 0.90914E+00_JPRB, 0.94606E+00_JPRB, 0.98378E+00_JPRB, 0.99876E+00_JPRB, 0.10155E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.11260E+01_JPRB, 0.11531E+01_JPRB, 0.11598E+01_JPRB, 0.11751E+01_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.12598E+01_JPRB, 0.12837E+01_JPRB, 0.13066E+01_JPRB, 0.13262E+01_JPRB, 0.13361E+01_JPRB, 0.13427E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.13479E+01_JPRB, 0.13474E+01_JPRB, 0.13502E+01_JPRB, 0.13699E+01_JPRB, 0.14029E+01_JPRB, 0.14221E+01_JPRB, 0.14625E+01_JPRB ,&
&  0.14619E+01_JPRB, 0.14890E+01_JPRB, 0.15286E+01_JPRB, 0.15652E+01_JPRB, 0.16157E+01_JPRB, 0.16187E+01_JPRB, 0.15952E+01_JPRB ,&
&  0.15828E+01_JPRB, 0.15654E+01_JPRB, 0.15754E+01_JPRB, 0.15899E+01_JPRB, 0.16144E+01_JPRB, 0.16657E+01_JPRB, 0.19629E+01_JPRB ,&
&  0.23382E+01_JPRB, 0.26045E+01_JPRB, 0.22835E+01_JPRB, 0.20476E+01_JPRB, 0.20299E+01_JPRB, 0.18359E+01_JPRB, 0.17207E+01_JPRB ,&
&  0.15661E+01_JPRB, 0.14208E+01_JPRB, 0.13797E+01_JPRB, 0.13700E+01_JPRB, 0.14357E+01_JPRB, 0.15089E+01_JPRB, 0.15705E+01_JPRB ,&
&  0.16823E+01_JPRB, 0.17781E+01_JPRB, 0.18994E+01_JPRB, 0.19851E+01_JPRB, 0.20599E+01_JPRB, 0.23085E+01_JPRB, 0.22013E+01_JPRB ,&
&  0.22857E+01_JPRB, 0.25738E+01_JPRB, 0.27566E+01_JPRB, 0.29115E+01_JPRB, 0.27569E+01_JPRB, 0.25721E+01_JPRB, 0.23046E+01_JPRB ,&
&  0.20269E+01_JPRB, 0.20064E+01_JPRB, 0.23310E+01_JPRB, 0.24730E+01_JPRB, 0.25754E+01_JPRB, 0.26341E+01_JPRB, 0.23649E+01_JPRB ,&
&  0.21013E+01_JPRB, 0.19147E+01_JPRB, 0.17338E+01_JPRB, 0.15721E+01_JPRB, 0.14800E+01_JPRB, 0.14678E+01_JPRB, 0.14227E+01_JPRB ,&
&  0.13815E+01_JPRB, 0.14076E+01_JPRB, 0.14934E+01_JPRB, 0.16511E+01_JPRB, 0.18955E+01_JPRB, 0.32701E+01_JPRB, 0.39097E+01_JPRB ,&
&  0.24909E+01_JPRB, 0.20524E+01_JPRB, 0.12114E+01_JPRB, 0.17619E+01_JPRB, 0.10249E+01_JPRB, 0.80918E+00_JPRB, 0.85405E+00_JPRB ,&
&  0.96011E+00_JPRB, 0.10450E+01_JPRB, 0.10941E+01_JPRB, 0.11539E+01_JPRB, 0.10843E+01_JPRB, 0.10842E+01_JPRB, 0.10974E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.11585E+01_JPRB, 0.11870E+01_JPRB, 0.12096E+01_JPRB, 0.12356E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.12809E+01_JPRB, 0.13068E+01_JPRB, 0.13328E+01_JPRB, 0.13627E+01_JPRB, 0.13885E+01_JPRB, 0.14185E+01_JPRB, 0.14378E+01_JPRB ,&
&  0.14537E+01_JPRB, 0.14662E+01_JPRB, 0.14737E+01_JPRB, 0.14920E+01_JPRB, 0.15135E+01_JPRB, 0.15518E+01_JPRB, 0.15994E+01_JPRB ,&
&  0.16541E+01_JPRB, 0.17175E+01_JPRB, 0.17910E+01_JPRB, 0.18469E+01_JPRB, 0.18981E+01_JPRB, 0.19421E+01_JPRB, 0.19809E+01_JPRB ,&
&  0.20021E+01_JPRB, 0.19917E+01_JPRB, 0.20226E+01_JPRB, 0.20940E+01_JPRB, 0.21689E+01_JPRB, 0.23413E+01_JPRB, 0.21812E+01_JPRB ,&
&  0.26471E+01_JPRB, 0.27125E+01_JPRB, 0.26177E+01_JPRB, 0.24163E+01_JPRB, 0.23682E+01_JPRB, 0.23328E+01_JPRB, 0.23451E+01_JPRB ,&
&  0.24030E+01_JPRB, 0.23063E+01_JPRB, 0.22016E+01_JPRB, 0.20273E+01_JPRB, 0.19267E+01_JPRB, 0.19596E+01_JPRB, 0.20747E+01_JPRB ,&
&  0.21393E+01_JPRB, 0.22208E+01_JPRB, 0.22449E+01_JPRB, 0.22461E+01_JPRB, 0.22577E+01_JPRB, 0.22537E+01_JPRB, 0.21305E+01_JPRB ,&
&  0.20303E+01_JPRB, 0.19989E+01_JPRB, 0.17433E+01_JPRB, 0.17585E+01_JPRB, 0.17602E+01_JPRB, 0.16181E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.12625E+01_JPRB, 0.13881E+01_JPRB, 0.18477E+01_JPRB, 0.20907E+01_JPRB, 0.21366E+01_JPRB, 0.20247E+01_JPRB ,&
&  0.18260E+01_JPRB, 0.16253E+01_JPRB, 0.14077E+01_JPRB, 0.12870E+01_JPRB, 0.12701E+01_JPRB, 0.13294E+01_JPRB, 0.13568E+01_JPRB ,&
&  0.13615E+01_JPRB, 0.13602E+01_JPRB, 0.13906E+01_JPRB, 0.14638E+01_JPRB, 0.17934E+01_JPRB, 0.24019E+01_JPRB, 0.36456E+01_JPRB ,&
&  0.26218E+01_JPRB, 0.15225E+01_JPRB, 0.14362E+01_JPRB, 0.14414E+01_JPRB, 0.99317E+00_JPRB, 0.76890E+00_JPRB, 0.84213E+00_JPRB ,&
&  0.10654E+01_JPRB, 0.12038E+01_JPRB, 0.12124E+01_JPRB, 0.12079E+01_JPRB, 0.11693E+01_JPRB, 0.11841E+01_JPRB, 0.11780E+01_JPRB ,&
&  0.11798E+01_JPRB, 0.11961E+01_JPRB, 0.12139E+01_JPRB, 0.12318E+01_JPRB, 0.12555E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.12834E+01_JPRB, 0.13169E+01_JPRB, 0.13493E+01_JPRB, 0.13818E+01_JPRB, 0.14090E+01_JPRB, 0.14430E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.14817E+01_JPRB, 0.14985E+01_JPRB, 0.15275E+01_JPRB, 0.15589E+01_JPRB, 0.15995E+01_JPRB, 0.16471E+01_JPRB, 0.17052E+01_JPRB ,&
&  0.17799E+01_JPRB, 0.18472E+01_JPRB, 0.19255E+01_JPRB, 0.19928E+01_JPRB, 0.20359E+01_JPRB, 0.20923E+01_JPRB, 0.21493E+01_JPRB ,&
&  0.21712E+01_JPRB, 0.21670E+01_JPRB, 0.22162E+01_JPRB, 0.22963E+01_JPRB, 0.23350E+01_JPRB, 0.24971E+01_JPRB, 0.21137E+01_JPRB ,&
&  0.27691E+01_JPRB, 0.29742E+01_JPRB, 0.30497E+01_JPRB, 0.28548E+01_JPRB, 0.27557E+01_JPRB, 0.28072E+01_JPRB, 0.28902E+01_JPRB ,&
&  0.30115E+01_JPRB, 0.31210E+01_JPRB, 0.30093E+01_JPRB, 0.28186E+01_JPRB, 0.26343E+01_JPRB, 0.26480E+01_JPRB, 0.27004E+01_JPRB ,&
&  0.27437E+01_JPRB, 0.27789E+01_JPRB, 0.27684E+01_JPRB, 0.27457E+01_JPRB, 0.27293E+01_JPRB, 0.26885E+01_JPRB, 0.25965E+01_JPRB ,&
&  0.25052E+01_JPRB, 0.24017E+01_JPRB, 0.21118E+01_JPRB, 0.22694E+01_JPRB, 0.21206E+01_JPRB, 0.17794E+01_JPRB, 0.14446E+01_JPRB ,&
&  0.12736E+01_JPRB, 0.12362E+01_JPRB, 0.14512E+01_JPRB, 0.15868E+01_JPRB, 0.16285E+01_JPRB, 0.16317E+01_JPRB, 0.15681E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.12348E+01_JPRB, 0.11423E+01_JPRB, 0.11278E+01_JPRB, 0.11605E+01_JPRB, 0.12152E+01_JPRB, 0.12478E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.12723E+01_JPRB, 0.12733E+01_JPRB, 0.12769E+01_JPRB, 0.14337E+01_JPRB, 0.17648E+01_JPRB, 0.29870E+01_JPRB ,&
&  0.23042E+01_JPRB, 0.16601E+01_JPRB, 0.22305E+01_JPRB, 0.83829E+00_JPRB, 0.84338E+00_JPRB, 0.72176E+00_JPRB, 0.79191E+00_JPRB ,&
&  0.93720E+00_JPRB, 0.12903E+01_JPRB, 0.16962E+01_JPRB, 0.14936E+01_JPRB, 0.13878E+01_JPRB, 0.12978E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.12029E+01_JPRB, 0.12153E+01_JPRB, 0.12259E+01_JPRB, 0.12508E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.12982E+01_JPRB, 0.13370E+01_JPRB, 0.13574E+01_JPRB, 0.13924E+01_JPRB, 0.13875E+01_JPRB, 0.14180E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.14660E+01_JPRB, 0.14969E+01_JPRB, 0.15387E+01_JPRB, 0.15820E+01_JPRB, 0.16378E+01_JPRB, 0.16928E+01_JPRB, 0.17556E+01_JPRB ,&
&  0.18279E+01_JPRB, 0.19104E+01_JPRB, 0.19816E+01_JPRB, 0.20778E+01_JPRB, 0.21364E+01_JPRB, 0.22287E+01_JPRB, 0.23172E+01_JPRB ,&
&  0.24434E+01_JPRB, 0.24677E+01_JPRB, 0.25053E+01_JPRB, 0.25635E+01_JPRB, 0.25820E+01_JPRB, 0.27506E+01_JPRB, 0.20477E+01_JPRB ,&
&  0.29066E+01_JPRB, 0.31960E+01_JPRB, 0.34039E+01_JPRB, 0.32837E+01_JPRB, 0.31158E+01_JPRB, 0.30902E+01_JPRB, 0.31272E+01_JPRB ,&
&  0.32273E+01_JPRB, 0.32947E+01_JPRB, 0.32021E+01_JPRB, 0.33168E+01_JPRB, 0.33616E+01_JPRB, 0.32820E+01_JPRB, 0.33783E+01_JPRB ,&
&  0.34497E+01_JPRB, 0.35144E+01_JPRB, 0.35585E+01_JPRB, 0.36077E+01_JPRB, 0.36505E+01_JPRB, 0.37395E+01_JPRB, 0.37945E+01_JPRB ,&
&  0.37680E+01_JPRB, 0.37614E+01_JPRB, 0.37245E+01_JPRB, 0.35221E+01_JPRB, 0.33026E+01_JPRB, 0.27678E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.15932E+01_JPRB, 0.13590E+01_JPRB, 0.14050E+01_JPRB, 0.14518E+01_JPRB, 0.13905E+01_JPRB, 0.13359E+01_JPRB, 0.12273E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10596E+01_JPRB, 0.10489E+01_JPRB, 0.10773E+01_JPRB, 0.11236E+01_JPRB, 0.11537E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.11965E+01_JPRB, 0.11563E+01_JPRB, 0.11327E+01_JPRB, 0.12521E+01_JPRB, 0.15966E+01_JPRB, 0.18095E+01_JPRB ,&
&  0.21132E+01_JPRB, 0.15678E+01_JPRB, 0.13347E+01_JPRB, 0.11864E+01_JPRB, 0.84760E+00_JPRB, 0.76427E+00_JPRB, 0.90802E+00_JPRB ,&
&  0.14381E+01_JPRB, 0.17848E+01_JPRB, 0.24672E+01_JPRB, 0.41181E+01_JPRB, 0.29354E+01_JPRB, 0.22358E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.12093E+01_JPRB, 0.12050E+01_JPRB, 0.12105E+01_JPRB, 0.12160E+01_JPRB, 0.12540E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.11633E+01_JPRB, 0.12094E+01_JPRB, 0.12581E+01_JPRB, 0.13076E+01_JPRB, 0.13434E+01_JPRB, 0.13644E+01_JPRB, 0.13772E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.14156E+01_JPRB, 0.14555E+01_JPRB, 0.14579E+01_JPRB, 0.14993E+01_JPRB, 0.15254E+01_JPRB, 0.15805E+01_JPRB ,&
&  0.16553E+01_JPRB, 0.17443E+01_JPRB, 0.18457E+01_JPRB, 0.19362E+01_JPRB, 0.20284E+01_JPRB, 0.21756E+01_JPRB, 0.23594E+01_JPRB ,&
&  0.25655E+01_JPRB, 0.27020E+01_JPRB, 0.28120E+01_JPRB, 0.28925E+01_JPRB, 0.28603E+01_JPRB, 0.28135E+01_JPRB, 0.20312E+01_JPRB ,&
&  0.28471E+01_JPRB, 0.33320E+01_JPRB, 0.37569E+01_JPRB, 0.37888E+01_JPRB, 0.35542E+01_JPRB, 0.33520E+01_JPRB, 0.33687E+01_JPRB ,&
&  0.33860E+01_JPRB, 0.33007E+01_JPRB, 0.31706E+01_JPRB, 0.31744E+01_JPRB, 0.35513E+01_JPRB, 0.34604E+01_JPRB, 0.35611E+01_JPRB ,&
&  0.36609E+01_JPRB, 0.37831E+01_JPRB, 0.39387E+01_JPRB, 0.41253E+01_JPRB, 0.43129E+01_JPRB, 0.44939E+01_JPRB, 0.46890E+01_JPRB ,&
&  0.48416E+01_JPRB, 0.48970E+01_JPRB, 0.49339E+01_JPRB, 0.43874E+01_JPRB, 0.40291E+01_JPRB, 0.40350E+01_JPRB, 0.30123E+01_JPRB ,&
&  0.17959E+01_JPRB, 0.14696E+01_JPRB, 0.14740E+01_JPRB, 0.14479E+01_JPRB, 0.13696E+01_JPRB, 0.12293E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10227E+01_JPRB, 0.10241E+01_JPRB, 0.10715E+01_JPRB, 0.11112E+01_JPRB, 0.11458E+01_JPRB, 0.11535E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.11602E+01_JPRB, 0.11175E+01_JPRB, 0.11144E+01_JPRB, 0.12477E+01_JPRB, 0.16011E+01_JPRB, 0.21512E+01_JPRB ,&
&  0.23185E+01_JPRB, 0.13505E+01_JPRB, 0.12242E+01_JPRB, 0.10227E+01_JPRB, 0.91182E+00_JPRB, 0.91202E+00_JPRB, 0.10785E+01_JPRB ,&
&  0.15635E+01_JPRB, 0.23539E+01_JPRB, 0.25501E+01_JPRB, 0.21696E+01_JPRB, 0.26122E+01_JPRB, 0.19275E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11268E+01_JPRB, 0.10994E+01_JPRB, 0.10981E+01_JPRB, 0.11181E+01_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.96440E+00_JPRB, 0.99745E+00_JPRB, 0.10329E+01_JPRB, 0.10826E+01_JPRB, 0.11398E+01_JPRB, 0.11836E+01_JPRB, 0.12285E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.12887E+01_JPRB, 0.13286E+01_JPRB, 0.13253E+01_JPRB, 0.13644E+01_JPRB, 0.13880E+01_JPRB, 0.14213E+01_JPRB ,&
&  0.14673E+01_JPRB, 0.15308E+01_JPRB, 0.16143E+01_JPRB, 0.17188E+01_JPRB, 0.18622E+01_JPRB, 0.20296E+01_JPRB, 0.22299E+01_JPRB ,&
&  0.24705E+01_JPRB, 0.26940E+01_JPRB, 0.29107E+01_JPRB, 0.30662E+01_JPRB, 0.31848E+01_JPRB, 0.32508E+01_JPRB, 0.32973E+01_JPRB ,&
&  0.18911E+01_JPRB, 0.31241E+01_JPRB, 0.35486E+01_JPRB, 0.38480E+01_JPRB, 0.40399E+01_JPRB, 0.38439E+01_JPRB, 0.38505E+01_JPRB ,&
&  0.36056E+01_JPRB, 0.32543E+01_JPRB, 0.31463E+01_JPRB, 0.30956E+01_JPRB, 0.32729E+01_JPRB, 0.32427E+01_JPRB, 0.32785E+01_JPRB ,&
&  0.33322E+01_JPRB, 0.34571E+01_JPRB, 0.36301E+01_JPRB, 0.38308E+01_JPRB, 0.40782E+01_JPRB, 0.43752E+01_JPRB, 0.47395E+01_JPRB ,&
&  0.51253E+01_JPRB, 0.54692E+01_JPRB, 0.57762E+01_JPRB, 0.51206E+01_JPRB, 0.45309E+01_JPRB, 0.46399E+01_JPRB, 0.38683E+01_JPRB ,&
&  0.22296E+01_JPRB, 0.15330E+01_JPRB, 0.15568E+01_JPRB, 0.16408E+01_JPRB, 0.15103E+01_JPRB, 0.13801E+01_JPRB, 0.12316E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.10797E+01_JPRB, 0.11247E+01_JPRB, 0.11155E+01_JPRB, 0.11695E+01_JPRB, 0.11818E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.12371E+01_JPRB, 0.12201E+01_JPRB, 0.12189E+01_JPRB, 0.12143E+01_JPRB, 0.12128E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.11621E+01_JPRB, 0.10968E+01_JPRB, 0.10820E+01_JPRB, 0.10468E+01_JPRB, 0.10210E+01_JPRB, 0.11033E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.14358E+01_JPRB, 0.17249E+01_JPRB, 0.17878E+01_JPRB, 0.16750E+01_JPRB, 0.12981E+01_JPRB, 0.99731E+00_JPRB ,&
&  0.10019E+01_JPRB, 0.96530E+00_JPRB, 0.93694E+00_JPRB, 0.92324E+00_JPRB, 0.93573E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.86872E+00_JPRB, 0.89630E+00_JPRB, 0.89473E+00_JPRB, 0.93045E+00_JPRB, 0.95622E+00_JPRB, 0.99321E+00_JPRB, 0.10376E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.11085E+01_JPRB, 0.11613E+01_JPRB, 0.11726E+01_JPRB, 0.12186E+01_JPRB, 0.12517E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.13510E+01_JPRB, 0.13975E+01_JPRB, 0.14549E+01_JPRB, 0.15439E+01_JPRB, 0.16865E+01_JPRB, 0.18477E+01_JPRB ,&
&  0.20451E+01_JPRB, 0.22849E+01_JPRB, 0.25985E+01_JPRB, 0.29206E+01_JPRB, 0.32203E+01_JPRB, 0.33899E+01_JPRB, 0.34197E+01_JPRB ,&
&  0.16034E+01_JPRB, 0.19265E+01_JPRB, 0.30801E+01_JPRB, 0.35478E+01_JPRB, 0.38824E+01_JPRB, 0.40025E+01_JPRB, 0.39521E+01_JPRB ,&
&  0.35860E+01_JPRB, 0.30219E+01_JPRB, 0.28315E+01_JPRB, 0.31116E+01_JPRB, 0.29914E+01_JPRB, 0.29916E+01_JPRB, 0.29850E+01_JPRB ,&
&  0.29994E+01_JPRB, 0.30607E+01_JPRB, 0.31705E+01_JPRB, 0.33183E+01_JPRB, 0.35615E+01_JPRB, 0.38893E+01_JPRB, 0.43285E+01_JPRB ,&
&  0.48702E+01_JPRB, 0.54331E+01_JPRB, 0.59341E+01_JPRB, 0.52309E+01_JPRB, 0.51210E+01_JPRB, 0.50892E+01_JPRB, 0.52637E+01_JPRB ,&
&  0.28550E+01_JPRB, 0.17320E+01_JPRB, 0.17770E+01_JPRB, 0.18798E+01_JPRB, 0.16858E+01_JPRB, 0.15914E+01_JPRB, 0.15014E+01_JPRB ,&
&  0.13789E+01_JPRB, 0.12888E+01_JPRB, 0.12716E+01_JPRB, 0.12463E+01_JPRB, 0.12477E+01_JPRB, 0.12757E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.13803E+01_JPRB, 0.13846E+01_JPRB, 0.13792E+01_JPRB, 0.13395E+01_JPRB, 0.12948E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.12304E+01_JPRB, 0.12175E+01_JPRB, 0.12439E+01_JPRB, 0.12593E+01_JPRB, 0.12518E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12091E+01_JPRB, 0.12036E+01_JPRB, 0.13058E+01_JPRB, 0.13274E+01_JPRB, 0.12812E+01_JPRB, 0.10888E+01_JPRB, 0.92127E+00_JPRB ,&
&  0.90622E+00_JPRB, 0.88109E+00_JPRB, 0.85933E+00_JPRB, 0.85679E+00_JPRB, 0.85635E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.79954E+00_JPRB, 0.78639E+00_JPRB, 0.77101E+00_JPRB, 0.78478E+00_JPRB, 0.79964E+00_JPRB, 0.81932E+00_JPRB, 0.87987E+00_JPRB ,&
&  0.93484E+00_JPRB, 0.95454E+00_JPRB, 0.98554E+00_JPRB, 0.10034E+01_JPRB, 0.10380E+01_JPRB, 0.10593E+01_JPRB, 0.10997E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.11598E+01_JPRB, 0.12086E+01_JPRB, 0.12565E+01_JPRB, 0.12979E+01_JPRB, 0.13589E+01_JPRB, 0.14492E+01_JPRB ,&
&  0.15457E+01_JPRB, 0.16692E+01_JPRB, 0.18708E+01_JPRB, 0.22215E+01_JPRB, 0.27756E+01_JPRB, 0.34452E+01_JPRB, 0.39919E+01_JPRB ,&
&  0.31168E+01_JPRB, 0.12072E+01_JPRB, 0.25245E+01_JPRB, 0.33634E+01_JPRB, 0.35131E+01_JPRB, 0.37113E+01_JPRB, 0.35521E+01_JPRB ,&
&  0.31964E+01_JPRB, 0.28299E+01_JPRB, 0.25804E+01_JPRB, 0.28275E+01_JPRB, 0.27966E+01_JPRB, 0.27833E+01_JPRB, 0.27709E+01_JPRB ,&
&  0.27419E+01_JPRB, 0.27465E+01_JPRB, 0.27745E+01_JPRB, 0.28538E+01_JPRB, 0.30300E+01_JPRB, 0.33257E+01_JPRB, 0.37324E+01_JPRB ,&
&  0.42950E+01_JPRB, 0.50697E+01_JPRB, 0.58422E+01_JPRB, 0.54002E+01_JPRB, 0.55704E+01_JPRB, 0.49656E+01_JPRB, 0.42879E+01_JPRB ,&
&  0.26582E+01_JPRB, 0.19246E+01_JPRB, 0.19378E+01_JPRB, 0.21043E+01_JPRB, 0.19065E+01_JPRB, 0.16779E+01_JPRB, 0.17474E+01_JPRB ,&
&  0.16140E+01_JPRB, 0.15289E+01_JPRB, 0.14702E+01_JPRB, 0.14250E+01_JPRB, 0.14138E+01_JPRB, 0.14127E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.14437E+01_JPRB, 0.14699E+01_JPRB, 0.14647E+01_JPRB, 0.14322E+01_JPRB, 0.14071E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.14148E+01_JPRB, 0.14066E+01_JPRB, 0.14202E+01_JPRB, 0.14355E+01_JPRB, 0.14334E+01_JPRB, 0.13836E+01_JPRB, 0.13674E+01_JPRB ,&
&  0.13332E+01_JPRB, 0.13091E+01_JPRB, 0.12630E+01_JPRB, 0.11937E+01_JPRB, 0.11395E+01_JPRB, 0.10439E+01_JPRB, 0.93930E+00_JPRB ,&
&  0.90399E+00_JPRB, 0.86593E+00_JPRB, 0.84236E+00_JPRB, 0.82890E+00_JPRB, 0.81582E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.86710E+00_JPRB, 0.84663E+00_JPRB, 0.81136E+00_JPRB, 0.75969E+00_JPRB, 0.75499E+00_JPRB, 0.75310E+00_JPRB, 0.76435E+00_JPRB ,&
&  0.81614E+00_JPRB, 0.87796E+00_JPRB, 0.89744E+00_JPRB, 0.89968E+00_JPRB, 0.87779E+00_JPRB, 0.88984E+00_JPRB, 0.91291E+00_JPRB ,&
&  0.94829E+00_JPRB, 0.98369E+00_JPRB, 0.10316E+01_JPRB, 0.10911E+01_JPRB, 0.11427E+01_JPRB, 0.11864E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.12828E+01_JPRB, 0.13264E+01_JPRB, 0.14142E+01_JPRB, 0.16198E+01_JPRB, 0.20913E+01_JPRB, 0.29332E+01_JPRB ,&
&  0.48261E+01_JPRB, 0.40712E+01_JPRB, 0.17437E+01_JPRB, 0.30646E+01_JPRB, 0.34447E+01_JPRB, 0.37371E+01_JPRB, 0.31494E+01_JPRB ,&
&  0.29919E+01_JPRB, 0.25984E+01_JPRB, 0.22844E+01_JPRB, 0.24495E+01_JPRB, 0.24300E+01_JPRB, 0.24221E+01_JPRB, 0.24103E+01_JPRB ,&
&  0.23450E+01_JPRB, 0.23522E+01_JPRB, 0.23775E+01_JPRB, 0.24150E+01_JPRB, 0.24903E+01_JPRB, 0.26949E+01_JPRB, 0.30935E+01_JPRB ,&
&  0.36197E+01_JPRB, 0.43405E+01_JPRB, 0.54264E+01_JPRB, 0.58946E+01_JPRB, 0.64323E+01_JPRB, 0.48492E+01_JPRB, 0.37784E+01_JPRB ,&
&  0.25176E+01_JPRB, 0.22844E+01_JPRB, 0.23687E+01_JPRB, 0.23567E+01_JPRB, 0.21488E+01_JPRB, 0.19674E+01_JPRB, 0.20371E+01_JPRB ,&
&  0.18663E+01_JPRB, 0.17500E+01_JPRB, 0.16851E+01_JPRB, 0.16197E+01_JPRB, 0.15740E+01_JPRB, 0.15506E+01_JPRB, 0.15319E+01_JPRB ,&
&  0.15143E+01_JPRB, 0.15240E+01_JPRB, 0.15405E+01_JPRB, 0.15521E+01_JPRB, 0.15485E+01_JPRB, 0.15274E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.15295E+01_JPRB, 0.15386E+01_JPRB, 0.15503E+01_JPRB, 0.15670E+01_JPRB, 0.15189E+01_JPRB, 0.14475E+01_JPRB, 0.14462E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.14182E+01_JPRB, 0.13677E+01_JPRB, 0.13432E+01_JPRB, 0.12352E+01_JPRB, 0.11172E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.98127E+00_JPRB, 0.95764E+00_JPRB, 0.90800E+00_JPRB, 0.86514E+00_JPRB, 0.85999E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.95387E+00_JPRB, 0.96481E+00_JPRB, 0.95789E+00_JPRB, 0.90504E+00_JPRB, 0.85723E+00_JPRB, 0.81145E+00_JPRB, 0.78081E+00_JPRB ,&
&  0.77407E+00_JPRB, 0.77700E+00_JPRB, 0.80771E+00_JPRB, 0.83684E+00_JPRB, 0.83000E+00_JPRB, 0.80317E+00_JPRB, 0.79654E+00_JPRB ,&
&  0.81149E+00_JPRB, 0.84741E+00_JPRB, 0.88526E+00_JPRB, 0.94286E+00_JPRB, 0.99558E+00_JPRB, 0.10340E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10782E+01_JPRB, 0.10894E+01_JPRB, 0.11118E+01_JPRB, 0.11591E+01_JPRB, 0.12886E+01_JPRB, 0.15095E+01_JPRB ,&
&  0.22200E+01_JPRB, 0.40947E+01_JPRB, 0.22151E+01_JPRB, 0.31687E+01_JPRB, 0.37430E+01_JPRB, 0.37935E+01_JPRB, 0.32929E+01_JPRB ,&
&  0.29590E+01_JPRB, 0.25272E+01_JPRB, 0.22599E+01_JPRB, 0.22222E+01_JPRB, 0.20221E+01_JPRB, 0.19906E+01_JPRB, 0.19677E+01_JPRB ,&
&  0.19748E+01_JPRB, 0.20080E+01_JPRB, 0.20364E+01_JPRB, 0.20610E+01_JPRB, 0.20602E+01_JPRB, 0.21058E+01_JPRB, 0.23180E+01_JPRB ,&
&  0.27729E+01_JPRB, 0.34423E+01_JPRB, 0.45960E+01_JPRB, 0.56225E+01_JPRB, 0.68295E+01_JPRB, 0.53408E+01_JPRB, 0.39791E+01_JPRB ,&
&  0.29391E+01_JPRB, 0.27984E+01_JPRB, 0.28086E+01_JPRB, 0.27636E+01_JPRB, 0.24821E+01_JPRB, 0.24561E+01_JPRB, 0.25092E+01_JPRB ,&
&  0.22508E+01_JPRB, 0.20386E+01_JPRB, 0.19260E+01_JPRB, 0.18443E+01_JPRB, 0.17567E+01_JPRB, 0.16894E+01_JPRB, 0.16450E+01_JPRB ,&
&  0.16139E+01_JPRB, 0.16028E+01_JPRB, 0.16169E+01_JPRB, 0.16318E+01_JPRB, 0.16477E+01_JPRB, 0.16435E+01_JPRB, 0.16320E+01_JPRB ,&
&  0.16037E+01_JPRB, 0.15909E+01_JPRB, 0.15887E+01_JPRB, 0.15832E+01_JPRB, 0.15494E+01_JPRB, 0.14992E+01_JPRB, 0.14880E+01_JPRB ,&
&  0.14857E+01_JPRB, 0.15063E+01_JPRB, 0.14665E+01_JPRB, 0.14085E+01_JPRB, 0.13979E+01_JPRB, 0.12900E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.12829E+01_JPRB, 0.13067E+01_JPRB, 0.95731E+00_JPRB, 0.94556E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.10243E+01_JPRB, 0.10236E+01_JPRB, 0.10095E+01_JPRB, 0.10039E+01_JPRB, 0.96604E+00_JPRB, 0.91742E+00_JPRB, 0.87091E+00_JPRB ,&
&  0.83768E+00_JPRB, 0.81582E+00_JPRB, 0.80961E+00_JPRB, 0.80331E+00_JPRB, 0.80293E+00_JPRB, 0.79451E+00_JPRB, 0.79026E+00_JPRB ,&
&  0.78074E+00_JPRB, 0.79166E+00_JPRB, 0.81555E+00_JPRB, 0.85850E+00_JPRB, 0.90742E+00_JPRB, 0.92221E+00_JPRB, 0.93634E+00_JPRB ,&
&  0.94793E+00_JPRB, 0.94961E+00_JPRB, 0.95301E+00_JPRB, 0.96662E+00_JPRB, 0.99390E+00_JPRB, 0.10415E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.14416E+01_JPRB, 0.33377E+01_JPRB, 0.50850E+01_JPRB, 0.57044E+01_JPRB, 0.50062E+01_JPRB, 0.42393E+01_JPRB, 0.37489E+01_JPRB ,&
&  0.36417E+01_JPRB, 0.32892E+01_JPRB, 0.28375E+01_JPRB, 0.22946E+01_JPRB, 0.19120E+01_JPRB, 0.17739E+01_JPRB, 0.17381E+01_JPRB ,&
&  0.18020E+01_JPRB, 0.18394E+01_JPRB, 0.18492E+01_JPRB, 0.18338E+01_JPRB, 0.17867E+01_JPRB, 0.17594E+01_JPRB, 0.17921E+01_JPRB ,&
&  0.19752E+01_JPRB, 0.24156E+01_JPRB, 0.32628E+01_JPRB, 0.47311E+01_JPRB, 0.63242E+01_JPRB, 0.56683E+01_JPRB, 0.45904E+01_JPRB ,&
&  0.42270E+01_JPRB, 0.38531E+01_JPRB, 0.35229E+01_JPRB, 0.33400E+01_JPRB, 0.27142E+01_JPRB, 0.32288E+01_JPRB, 0.30563E+01_JPRB ,&
&  0.28347E+01_JPRB, 0.26809E+01_JPRB, 0.24562E+01_JPRB, 0.22112E+01_JPRB, 0.20278E+01_JPRB, 0.18581E+01_JPRB, 0.17282E+01_JPRB ,&
&  0.16802E+01_JPRB, 0.16830E+01_JPRB, 0.17004E+01_JPRB, 0.17350E+01_JPRB, 0.17759E+01_JPRB, 0.17904E+01_JPRB, 0.17617E+01_JPRB ,&
&  0.16853E+01_JPRB, 0.16193E+01_JPRB, 0.15177E+01_JPRB, 0.15055E+01_JPRB, 0.14959E+01_JPRB, 0.14504E+01_JPRB, 0.14208E+01_JPRB ,&
&  0.14410E+01_JPRB, 0.14887E+01_JPRB, 0.14935E+01_JPRB, 0.14642E+01_JPRB, 0.14261E+01_JPRB, 0.13916E+01_JPRB, 0.12738E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12407E+01_JPRB, 0.11563E+01_JPRB, 0.10745E+01_JPRB, 0.10467E+01_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.10918E+01_JPRB, 0.10544E+01_JPRB, 0.10318E+01_JPRB, 0.10270E+01_JPRB, 0.10043E+01_JPRB, 0.98661E+00_JPRB, 0.97514E+00_JPRB ,&
&  0.92242E+00_JPRB, 0.87982E+00_JPRB, 0.84043E+00_JPRB, 0.82074E+00_JPRB, 0.81630E+00_JPRB, 0.81050E+00_JPRB, 0.81810E+00_JPRB ,&
&  0.81377E+00_JPRB, 0.80068E+00_JPRB, 0.80208E+00_JPRB, 0.82718E+00_JPRB, 0.85930E+00_JPRB, 0.86704E+00_JPRB, 0.88497E+00_JPRB ,&
&  0.90593E+00_JPRB, 0.90715E+00_JPRB, 0.89896E+00_JPRB, 0.89886E+00_JPRB, 0.90666E+00_JPRB, 0.91560E+00_JPRB, 0.97096E+00_JPRB ,&
&  0.12475E+01_JPRB, 0.33686E+01_JPRB, 0.33538E+01_JPRB, 0.65461E+01_JPRB, 0.56974E+01_JPRB, 0.48956E+01_JPRB, 0.38668E+01_JPRB ,&
&  0.39316E+01_JPRB, 0.43336E+01_JPRB, 0.32749E+01_JPRB, 0.25889E+01_JPRB, 0.21121E+01_JPRB, 0.17947E+01_JPRB, 0.16859E+01_JPRB ,&
&  0.17078E+01_JPRB, 0.17601E+01_JPRB, 0.17840E+01_JPRB, 0.17252E+01_JPRB, 0.16541E+01_JPRB, 0.16171E+01_JPRB, 0.16150E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.17639E+01_JPRB, 0.21733E+01_JPRB, 0.30339E+01_JPRB, 0.49985E+01_JPRB, 0.59082E+01_JPRB, 0.53646E+01_JPRB ,&
&  0.51268E+01_JPRB, 0.52706E+01_JPRB, 0.47986E+01_JPRB, 0.42699E+01_JPRB, 0.35630E+01_JPRB, 0.35644E+01_JPRB, 0.33848E+01_JPRB ,&
&  0.31896E+01_JPRB, 0.31337E+01_JPRB, 0.30305E+01_JPRB, 0.28077E+01_JPRB, 0.25338E+01_JPRB, 0.22485E+01_JPRB, 0.19867E+01_JPRB ,&
&  0.18030E+01_JPRB, 0.17180E+01_JPRB, 0.16829E+01_JPRB, 0.17235E+01_JPRB, 0.18080E+01_JPRB, 0.18680E+01_JPRB, 0.18805E+01_JPRB ,&
&  0.18193E+01_JPRB, 0.17208E+01_JPRB, 0.15345E+01_JPRB, 0.14432E+01_JPRB, 0.14336E+01_JPRB, 0.13899E+01_JPRB, 0.13670E+01_JPRB ,&
&  0.14166E+01_JPRB, 0.14599E+01_JPRB, 0.14598E+01_JPRB, 0.14353E+01_JPRB, 0.14201E+01_JPRB, 0.14433E+01_JPRB, 0.13449E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.12800E+01_JPRB, 0.12656E+01_JPRB, 0.11934E+01_JPRB, 0.11457E+01_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.11669E+01_JPRB, 0.11076E+01_JPRB, 0.10616E+01_JPRB, 0.10402E+01_JPRB, 0.10311E+01_JPRB, 0.10211E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.97672E+00_JPRB, 0.93583E+00_JPRB, 0.89425E+00_JPRB, 0.85856E+00_JPRB, 0.84963E+00_JPRB, 0.84246E+00_JPRB, 0.84187E+00_JPRB ,&
&  0.83625E+00_JPRB, 0.82325E+00_JPRB, 0.81309E+00_JPRB, 0.82141E+00_JPRB, 0.83122E+00_JPRB, 0.84174E+00_JPRB, 0.86193E+00_JPRB ,&
&  0.87032E+00_JPRB, 0.88089E+00_JPRB, 0.88400E+00_JPRB, 0.87988E+00_JPRB, 0.87161E+00_JPRB, 0.86439E+00_JPRB, 0.88000E+00_JPRB ,&
&  0.12157E+01_JPRB, 0.43029E+01_JPRB, 0.30643E+01_JPRB, 0.60168E+01_JPRB, 0.52478E+01_JPRB, 0.47550E+01_JPRB, 0.38139E+01_JPRB ,&
&  0.36215E+01_JPRB, 0.39722E+01_JPRB, 0.33624E+01_JPRB, 0.28050E+01_JPRB, 0.24197E+01_JPRB, 0.21016E+01_JPRB, 0.18768E+01_JPRB ,&
&  0.17891E+01_JPRB, 0.17533E+01_JPRB, 0.17127E+01_JPRB, 0.16495E+01_JPRB, 0.16056E+01_JPRB, 0.15755E+01_JPRB, 0.15717E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.15890E+01_JPRB, 0.17298E+01_JPRB, 0.22680E+01_JPRB, 0.34729E+01_JPRB, 0.50873E+01_JPRB, 0.61556E+01_JPRB ,&
&  0.62894E+01_JPRB, 0.68239E+01_JPRB, 0.61246E+01_JPRB, 0.52631E+01_JPRB, 0.45822E+01_JPRB, 0.41825E+01_JPRB, 0.38247E+01_JPRB ,&
&  0.34593E+01_JPRB, 0.32390E+01_JPRB, 0.31687E+01_JPRB, 0.31156E+01_JPRB, 0.30011E+01_JPRB, 0.27944E+01_JPRB, 0.25247E+01_JPRB ,&
&  0.22416E+01_JPRB, 0.20363E+01_JPRB, 0.18953E+01_JPRB, 0.18320E+01_JPRB, 0.18330E+01_JPRB, 0.18717E+01_JPRB, 0.18918E+01_JPRB ,&
&  0.18317E+01_JPRB, 0.17555E+01_JPRB, 0.15968E+01_JPRB, 0.14697E+01_JPRB, 0.14390E+01_JPRB, 0.13926E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.14434E+01_JPRB, 0.14274E+01_JPRB, 0.14130E+01_JPRB, 0.13822E+01_JPRB, 0.14575E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.12918E+01_JPRB, 0.12751E+01_JPRB, 0.12454E+01_JPRB, 0.12186E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.12167E+01_JPRB, 0.11722E+01_JPRB, 0.11254E+01_JPRB, 0.10880E+01_JPRB, 0.10590E+01_JPRB, 0.10439E+01_JPRB, 0.10311E+01_JPRB ,&
&  0.10031E+01_JPRB, 0.97110E+00_JPRB, 0.94157E+00_JPRB, 0.91649E+00_JPRB, 0.89765E+00_JPRB, 0.88717E+00_JPRB, 0.87381E+00_JPRB ,&
&  0.85769E+00_JPRB, 0.83010E+00_JPRB, 0.82989E+00_JPRB, 0.84109E+00_JPRB, 0.84295E+00_JPRB, 0.84615E+00_JPRB, 0.87008E+00_JPRB ,&
&  0.86674E+00_JPRB, 0.86714E+00_JPRB, 0.87082E+00_JPRB, 0.87279E+00_JPRB, 0.86198E+00_JPRB, 0.86159E+00_JPRB, 0.86190E+00_JPRB ,&
&  0.11554E+01_JPRB, 0.16337E+01_JPRB, 0.37241E+01_JPRB, 0.52261E+01_JPRB, 0.47479E+01_JPRB, 0.42540E+01_JPRB, 0.38474E+01_JPRB ,&
&  0.37166E+01_JPRB, 0.35795E+01_JPRB, 0.31250E+01_JPRB, 0.27848E+01_JPRB, 0.24836E+01_JPRB, 0.22509E+01_JPRB, 0.20742E+01_JPRB ,&
&  0.19370E+01_JPRB, 0.18107E+01_JPRB, 0.17142E+01_JPRB, 0.16244E+01_JPRB, 0.15624E+01_JPRB, 0.15302E+01_JPRB, 0.15234E+01_JPRB ,&
&  0.15176E+01_JPRB, 0.15097E+01_JPRB, 0.15451E+01_JPRB, 0.18224E+01_JPRB, 0.28291E+01_JPRB, 0.41930E+01_JPRB, 0.57306E+01_JPRB ,&
&  0.68963E+01_JPRB, 0.73760E+01_JPRB, 0.67492E+01_JPRB, 0.59727E+01_JPRB, 0.52120E+01_JPRB, 0.46303E+01_JPRB, 0.41797E+01_JPRB ,&
&  0.37694E+01_JPRB, 0.34141E+01_JPRB, 0.31357E+01_JPRB, 0.29855E+01_JPRB, 0.28940E+01_JPRB, 0.28215E+01_JPRB, 0.27316E+01_JPRB ,&
&  0.26002E+01_JPRB, 0.24116E+01_JPRB, 0.22117E+01_JPRB, 0.20499E+01_JPRB, 0.19503E+01_JPRB, 0.19119E+01_JPRB, 0.18843E+01_JPRB ,&
&  0.18024E+01_JPRB, 0.17202E+01_JPRB, 0.16000E+01_JPRB, 0.15155E+01_JPRB, 0.15056E+01_JPRB, 0.14752E+01_JPRB, 0.14428E+01_JPRB ,&
&  0.14431E+01_JPRB, 0.14485E+01_JPRB, 0.14234E+01_JPRB, 0.14168E+01_JPRB, 0.13703E+01_JPRB, 0.14445E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.13058E+01_JPRB, 0.12753E+01_JPRB, 0.12532E+01_JPRB, 0.12419E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.12502E+01_JPRB, 0.12246E+01_JPRB, 0.11739E+01_JPRB, 0.11245E+01_JPRB, 0.10886E+01_JPRB, 0.10633E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.10178E+01_JPRB, 0.98832E+00_JPRB, 0.96135E+00_JPRB, 0.94496E+00_JPRB, 0.93890E+00_JPRB, 0.93224E+00_JPRB, 0.91356E+00_JPRB ,&
&  0.88876E+00_JPRB, 0.86634E+00_JPRB, 0.86302E+00_JPRB, 0.86847E+00_JPRB, 0.87266E+00_JPRB, 0.85877E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.87558E+00_JPRB, 0.87405E+00_JPRB, 0.87153E+00_JPRB, 0.86742E+00_JPRB, 0.86137E+00_JPRB, 0.85716E+00_JPRB, 0.85312E+00_JPRB ,&
&  0.10565E+01_JPRB, 0.23926E+01_JPRB, 0.37757E+01_JPRB, 0.45746E+01_JPRB, 0.44680E+01_JPRB, 0.38672E+01_JPRB, 0.35078E+01_JPRB ,&
&  0.31964E+01_JPRB, 0.30527E+01_JPRB, 0.27524E+01_JPRB, 0.25321E+01_JPRB, 0.23230E+01_JPRB, 0.21396E+01_JPRB, 0.19779E+01_JPRB ,&
&  0.18574E+01_JPRB, 0.17605E+01_JPRB, 0.16715E+01_JPRB, 0.15957E+01_JPRB, 0.15391E+01_JPRB, 0.14947E+01_JPRB, 0.14710E+01_JPRB ,&
&  0.14589E+01_JPRB, 0.14449E+01_JPRB, 0.14481E+01_JPRB, 0.15615E+01_JPRB, 0.22691E+01_JPRB, 0.27909E+01_JPRB, 0.45828E+01_JPRB ,&
&  0.60406E+01_JPRB, 0.57446E+01_JPRB, 0.54902E+01_JPRB, 0.50632E+01_JPRB, 0.45255E+01_JPRB, 0.40748E+01_JPRB, 0.36004E+01_JPRB ,&
&  0.31900E+01_JPRB, 0.28668E+01_JPRB, 0.26043E+01_JPRB, 0.24236E+01_JPRB, 0.23108E+01_JPRB, 0.22512E+01_JPRB, 0.22292E+01_JPRB ,&
&  0.22171E+01_JPRB, 0.22003E+01_JPRB, 0.21633E+01_JPRB, 0.20961E+01_JPRB, 0.19994E+01_JPRB, 0.18980E+01_JPRB, 0.18006E+01_JPRB ,&
&  0.17007E+01_JPRB, 0.16360E+01_JPRB, 0.15585E+01_JPRB, 0.15040E+01_JPRB, 0.15281E+01_JPRB, 0.15639E+01_JPRB, 0.15387E+01_JPRB ,&
&  0.14788E+01_JPRB, 0.14578E+01_JPRB, 0.14423E+01_JPRB, 0.13945E+01_JPRB, 0.13588E+01_JPRB, 0.14497E+01_JPRB, 0.14071E+01_JPRB ,&
&  0.13555E+01_JPRB, 0.13222E+01_JPRB, 0.12934E+01_JPRB, 0.12620E+01_JPRB, 0.12625E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.12775E+01_JPRB, 0.12305E+01_JPRB, 0.11842E+01_JPRB, 0.11373E+01_JPRB, 0.11034E+01_JPRB, 0.10680E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.99191E+00_JPRB, 0.97801E+00_JPRB, 0.95801E+00_JPRB, 0.95417E+00_JPRB, 0.94648E+00_JPRB, 0.92849E+00_JPRB ,&
&  0.90117E+00_JPRB, 0.88075E+00_JPRB, 0.87168E+00_JPRB, 0.86994E+00_JPRB, 0.87674E+00_JPRB, 0.87629E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.88819E+00_JPRB, 0.87255E+00_JPRB, 0.86051E+00_JPRB, 0.85784E+00_JPRB, 0.85521E+00_JPRB, 0.85558E+00_JPRB, 0.85938E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.17713E+01_JPRB, 0.25800E+01_JPRB, 0.36193E+01_JPRB, 0.37698E+01_JPRB, 0.34211E+01_JPRB, 0.30894E+01_JPRB ,&
&  0.28220E+01_JPRB, 0.26267E+01_JPRB, 0.23893E+01_JPRB, 0.21141E+01_JPRB, 0.18970E+01_JPRB, 0.17467E+01_JPRB, 0.16373E+01_JPRB ,&
&  0.15694E+01_JPRB, 0.15175E+01_JPRB, 0.14700E+01_JPRB, 0.14272E+01_JPRB, 0.13892E+01_JPRB, 0.13565E+01_JPRB, 0.13374E+01_JPRB ,&
&  0.13259E+01_JPRB, 0.13175E+01_JPRB, 0.13172E+01_JPRB, 0.13482E+01_JPRB, 0.16606E+01_JPRB, 0.19829E+01_JPRB, 0.27789E+01_JPRB ,&
&  0.31286E+01_JPRB, 0.32814E+01_JPRB, 0.34190E+01_JPRB, 0.33724E+01_JPRB, 0.31738E+01_JPRB, 0.29713E+01_JPRB, 0.27558E+01_JPRB ,&
&  0.25220E+01_JPRB, 0.22895E+01_JPRB, 0.21305E+01_JPRB, 0.20158E+01_JPRB, 0.19096E+01_JPRB, 0.18303E+01_JPRB, 0.17834E+01_JPRB ,&
&  0.17744E+01_JPRB, 0.18041E+01_JPRB, 0.18398E+01_JPRB, 0.18622E+01_JPRB, 0.18428E+01_JPRB, 0.17962E+01_JPRB, 0.17218E+01_JPRB ,&
&  0.16220E+01_JPRB, 0.15418E+01_JPRB, 0.15014E+01_JPRB, 0.14923E+01_JPRB, 0.15064E+01_JPRB, 0.15146E+01_JPRB, 0.15208E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.15032E+01_JPRB, 0.14696E+01_JPRB, 0.14277E+01_JPRB, 0.13961E+01_JPRB, 0.14635E+01_JPRB, 0.14337E+01_JPRB ,&
&  0.13900E+01_JPRB, 0.13469E+01_JPRB, 0.13173E+01_JPRB, 0.12860E+01_JPRB, 0.13114E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.13416E+01_JPRB, 0.12647E+01_JPRB, 0.12042E+01_JPRB, 0.11589E+01_JPRB, 0.11279E+01_JPRB, 0.10904E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.10026E+01_JPRB, 0.98937E+00_JPRB, 0.96473E+00_JPRB, 0.94486E+00_JPRB, 0.92638E+00_JPRB, 0.91043E+00_JPRB ,&
&  0.88760E+00_JPRB, 0.87671E+00_JPRB, 0.86781E+00_JPRB, 0.86579E+00_JPRB, 0.86142E+00_JPRB, 0.86127E+00_JPRB, 0.86951E+00_JPRB ,&
&  0.87339E+00_JPRB, 0.85660E+00_JPRB, 0.85324E+00_JPRB, 0.84947E+00_JPRB, 0.84444E+00_JPRB, 0.84106E+00_JPRB, 0.84204E+00_JPRB ,&
&  0.99135E+00_JPRB, 0.13093E+01_JPRB, 0.17892E+01_JPRB, 0.23282E+01_JPRB, 0.25785E+01_JPRB, 0.26631E+01_JPRB, 0.25508E+01_JPRB ,&
&  0.23760E+01_JPRB, 0.21833E+01_JPRB, 0.19782E+01_JPRB, 0.17868E+01_JPRB, 0.16068E+01_JPRB, 0.14744E+01_JPRB, 0.13873E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.12987E+01_JPRB, 0.12728E+01_JPRB, 0.12513E+01_JPRB, 0.12299E+01_JPRB, 0.12048E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11727E+01_JPRB, 0.11701E+01_JPRB, 0.11902E+01_JPRB, 0.12485E+01_JPRB, 0.13817E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.19747E+01_JPRB, 0.21433E+01_JPRB, 0.23185E+01_JPRB, 0.23764E+01_JPRB, 0.23076E+01_JPRB, 0.21952E+01_JPRB, 0.20446E+01_JPRB ,&
&  0.19244E+01_JPRB, 0.18377E+01_JPRB, 0.17569E+01_JPRB, 0.16787E+01_JPRB, 0.16141E+01_JPRB, 0.15666E+01_JPRB, 0.15340E+01_JPRB ,&
&  0.15111E+01_JPRB, 0.15153E+01_JPRB, 0.15371E+01_JPRB, 0.15614E+01_JPRB, 0.15779E+01_JPRB, 0.15788E+01_JPRB, 0.15629E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.14455E+01_JPRB, 0.14050E+01_JPRB, 0.14085E+01_JPRB, 0.14409E+01_JPRB, 0.14592E+01_JPRB, 0.14659E+01_JPRB ,&
&  0.14739E+01_JPRB, 0.14804E+01_JPRB, 0.14444E+01_JPRB, 0.14350E+01_JPRB, 0.14983E+01_JPRB, 0.14667E+01_JPRB, 0.14260E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.13499E+01_JPRB, 0.13168E+01_JPRB, 0.12815E+01_JPRB, 0.13377E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.11700E+01_JPRB, 0.11594E+01_JPRB, 0.11298E+01_JPRB, 0.11049E+01_JPRB, 0.10912E+01_JPRB, 0.10771E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.10019E+01_JPRB, 0.98493E+00_JPRB, 0.96072E+00_JPRB, 0.93515E+00_JPRB, 0.91703E+00_JPRB, 0.90010E+00_JPRB ,&
&  0.87984E+00_JPRB, 0.86968E+00_JPRB, 0.86486E+00_JPRB, 0.85935E+00_JPRB, 0.85167E+00_JPRB, 0.84555E+00_JPRB, 0.84886E+00_JPRB ,&
&  0.84843E+00_JPRB, 0.84823E+00_JPRB, 0.84915E+00_JPRB, 0.84202E+00_JPRB, 0.83213E+00_JPRB, 0.82751E+00_JPRB, 0.81477E+00_JPRB ,&
&  0.87093E+00_JPRB, 0.97904E+00_JPRB, 0.12095E+01_JPRB, 0.15026E+01_JPRB, 0.17610E+01_JPRB, 0.19490E+01_JPRB, 0.20357E+01_JPRB ,&
&  0.19538E+01_JPRB, 0.17890E+01_JPRB, 0.16295E+01_JPRB, 0.15273E+01_JPRB, 0.14348E+01_JPRB, 0.13571E+01_JPRB, 0.12897E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.12133E+01_JPRB, 0.11874E+01_JPRB, 0.11624E+01_JPRB, 0.11358E+01_JPRB, 0.11118E+01_JPRB, 0.10923E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10665E+01_JPRB, 0.10648E+01_JPRB, 0.10611E+01_JPRB, 0.10742E+01_JPRB, 0.11461E+01_JPRB, 0.13040E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.15274E+01_JPRB, 0.16824E+01_JPRB, 0.17648E+01_JPRB, 0.17389E+01_JPRB, 0.16807E+01_JPRB, 0.16042E+01_JPRB ,&
&  0.15392E+01_JPRB, 0.14702E+01_JPRB, 0.14259E+01_JPRB, 0.13824E+01_JPRB, 0.13360E+01_JPRB, 0.12983E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.12713E+01_JPRB, 0.12704E+01_JPRB, 0.12821E+01_JPRB, 0.13004E+01_JPRB, 0.13141E+01_JPRB, 0.13269E+01_JPRB, 0.13413E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.13342E+01_JPRB, 0.13043E+01_JPRB, 0.13074E+01_JPRB, 0.13379E+01_JPRB, 0.13629E+01_JPRB, 0.13783E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.14090E+01_JPRB, 0.14170E+01_JPRB, 0.14377E+01_JPRB, 0.14248E+01_JPRB, 0.13814E+01_JPRB, 0.13397E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.12713E+01_JPRB, 0.12463E+01_JPRB, 0.12139E+01_JPRB, 0.11226E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.10952E+01_JPRB, 0.10791E+01_JPRB, 0.10544E+01_JPRB, 0.10321E+01_JPRB, 0.10215E+01_JPRB, 0.10177E+01_JPRB, 0.10157E+01_JPRB ,&
&  0.10072E+01_JPRB, 0.99464E+00_JPRB, 0.97994E+00_JPRB, 0.96201E+00_JPRB, 0.94148E+00_JPRB, 0.92246E+00_JPRB, 0.90536E+00_JPRB ,&
&  0.88171E+00_JPRB, 0.86637E+00_JPRB, 0.86258E+00_JPRB, 0.85882E+00_JPRB, 0.85552E+00_JPRB, 0.84919E+00_JPRB, 0.84622E+00_JPRB ,&
&  0.84706E+00_JPRB, 0.84803E+00_JPRB, 0.84820E+00_JPRB, 0.83946E+00_JPRB, 0.83105E+00_JPRB, 0.82816E+00_JPRB, 0.82016E+00_JPRB ,&
&  0.82933E+00_JPRB, 0.78210E+00_JPRB, 0.95601E+00_JPRB, 0.11356E+01_JPRB, 0.12617E+01_JPRB, 0.14185E+01_JPRB, 0.15652E+01_JPRB ,&
&  0.15877E+01_JPRB, 0.15229E+01_JPRB, 0.14244E+01_JPRB, 0.13434E+01_JPRB, 0.12737E+01_JPRB, 0.12243E+01_JPRB, 0.11865E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.11386E+01_JPRB, 0.11142E+01_JPRB, 0.10836E+01_JPRB, 0.10510E+01_JPRB, 0.10266E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.99326E+00_JPRB, 0.98534E+00_JPRB, 0.97849E+00_JPRB, 0.97978E+00_JPRB, 0.98121E+00_JPRB, 0.10036E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11948E+01_JPRB, 0.12912E+01_JPRB, 0.13331E+01_JPRB, 0.13271E+01_JPRB, 0.13075E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.11841E+01_JPRB, 0.11508E+01_JPRB, 0.11204E+01_JPRB, 0.10985E+01_JPRB, 0.10868E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.10962E+01_JPRB, 0.10999E+01_JPRB, 0.11082E+01_JPRB, 0.11143E+01_JPRB, 0.11190E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.12039E+01_JPRB, 0.12143E+01_JPRB, 0.12078E+01_JPRB, 0.12104E+01_JPRB, 0.12300E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.12777E+01_JPRB, 0.12917E+01_JPRB, 0.12644E+01_JPRB, 0.13133E+01_JPRB, 0.12882E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.12195E+01_JPRB, 0.11911E+01_JPRB, 0.11692E+01_JPRB, 0.10245E+01_JPRB, 0.10749E+01_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.10021E+01_JPRB, 0.98908E+00_JPRB, 0.96909E+00_JPRB, 0.95019E+00_JPRB, 0.93722E+00_JPRB, 0.93182E+00_JPRB, 0.92960E+00_JPRB ,&
&  0.93293E+00_JPRB, 0.93571E+00_JPRB, 0.93500E+00_JPRB, 0.93052E+00_JPRB, 0.92298E+00_JPRB, 0.91075E+00_JPRB, 0.89926E+00_JPRB ,&
&  0.88276E+00_JPRB, 0.86584E+00_JPRB, 0.85854E+00_JPRB, 0.85793E+00_JPRB, 0.85575E+00_JPRB, 0.84723E+00_JPRB, 0.84006E+00_JPRB ,&
&  0.83537E+00_JPRB, 0.83523E+00_JPRB, 0.82677E+00_JPRB, 0.82011E+00_JPRB, 0.81253E+00_JPRB, 0.81265E+00_JPRB, 0.81105E+00_JPRB ,&
&  0.80961E+00_JPRB, 0.70520E+00_JPRB, 0.86301E+00_JPRB, 0.97860E+00_JPRB, 0.10282E+01_JPRB, 0.11250E+01_JPRB, 0.12130E+01_JPRB ,&
&  0.12683E+01_JPRB, 0.12864E+01_JPRB, 0.12670E+01_JPRB, 0.12290E+01_JPRB, 0.11770E+01_JPRB, 0.11363E+01_JPRB, 0.11061E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.10469E+01_JPRB, 0.10128E+01_JPRB, 0.97775E+00_JPRB, 0.94950E+00_JPRB, 0.92144E+00_JPRB, 0.89736E+00_JPRB ,&
&  0.88368E+00_JPRB, 0.87944E+00_JPRB, 0.87256E+00_JPRB, 0.87122E+00_JPRB, 0.87570E+00_JPRB, 0.88107E+00_JPRB, 0.91037E+00_JPRB ,&
&  0.94087E+00_JPRB, 0.96875E+00_JPRB, 0.99970E+00_JPRB, 0.10211E+01_JPRB, 0.10259E+01_JPRB, 0.10314E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.10015E+01_JPRB, 0.98170E+00_JPRB, 0.96265E+00_JPRB, 0.94718E+00_JPRB, 0.93591E+00_JPRB, 0.93419E+00_JPRB ,&
&  0.93789E+00_JPRB, 0.93101E+00_JPRB, 0.92417E+00_JPRB, 0.92418E+00_JPRB, 0.92792E+00_JPRB, 0.93712E+00_JPRB, 0.95646E+00_JPRB ,&
&  0.98694E+00_JPRB, 0.10256E+01_JPRB, 0.10568E+01_JPRB, 0.10705E+01_JPRB, 0.10741E+01_JPRB, 0.10832E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.11244E+01_JPRB, 0.11341E+01_JPRB, 0.11662E+01_JPRB, 0.11689E+01_JPRB, 0.11551E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.10823E+01_JPRB, 0.90577E+00_JPRB, 0.97554E+00_JPRB, 0.99035E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.92189E+00_JPRB, 0.90893E+00_JPRB, 0.88968E+00_JPRB, 0.87358E+00_JPRB, 0.86032E+00_JPRB, 0.85440E+00_JPRB, 0.85077E+00_JPRB ,&
&  0.85457E+00_JPRB, 0.86281E+00_JPRB, 0.87136E+00_JPRB, 0.87677E+00_JPRB, 0.88102E+00_JPRB, 0.88092E+00_JPRB, 0.87587E+00_JPRB ,&
&  0.87128E+00_JPRB, 0.85883E+00_JPRB, 0.84827E+00_JPRB, 0.84668E+00_JPRB, 0.84280E+00_JPRB, 0.83771E+00_JPRB, 0.83013E+00_JPRB ,&
&  0.82212E+00_JPRB, 0.81429E+00_JPRB, 0.80082E+00_JPRB, 0.79332E+00_JPRB, 0.79073E+00_JPRB, 0.79281E+00_JPRB, 0.80075E+00_JPRB ,&
&  0.72302E+00_JPRB, 0.65188E+00_JPRB, 0.75813E+00_JPRB, 0.88143E+00_JPRB, 0.93505E+00_JPRB, 0.98380E+00_JPRB, 0.10265E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.11260E+01_JPRB, 0.11537E+01_JPRB, 0.11564E+01_JPRB, 0.11360E+01_JPRB, 0.11025E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.95613E+00_JPRB, 0.91469E+00_JPRB, 0.88928E+00_JPRB, 0.86896E+00_JPRB, 0.84897E+00_JPRB, 0.83229E+00_JPRB ,&
&  0.82056E+00_JPRB, 0.81203E+00_JPRB, 0.80542E+00_JPRB, 0.80091E+00_JPRB, 0.80158E+00_JPRB, 0.80480E+00_JPRB, 0.81295E+00_JPRB ,&
&  0.82662E+00_JPRB, 0.84027E+00_JPRB, 0.85157E+00_JPRB, 0.85728E+00_JPRB, 0.86268E+00_JPRB, 0.87446E+00_JPRB, 0.88255E+00_JPRB ,&
&  0.88916E+00_JPRB, 0.88776E+00_JPRB, 0.88195E+00_JPRB, 0.87289E+00_JPRB, 0.86045E+00_JPRB, 0.84793E+00_JPRB, 0.83878E+00_JPRB ,&
&  0.83364E+00_JPRB, 0.82582E+00_JPRB, 0.82323E+00_JPRB, 0.82630E+00_JPRB, 0.83250E+00_JPRB, 0.84032E+00_JPRB, 0.84943E+00_JPRB ,&
&  0.86541E+00_JPRB, 0.89135E+00_JPRB, 0.91373E+00_JPRB, 0.92873E+00_JPRB, 0.93860E+00_JPRB, 0.94619E+00_JPRB, 0.95710E+00_JPRB ,&
&  0.96834E+00_JPRB, 0.98011E+00_JPRB, 0.99498E+00_JPRB, 0.10137E+01_JPRB, 0.10197E+01_JPRB, 0.10157E+01_JPRB, 0.10042E+01_JPRB ,&
&  0.98563E+00_JPRB, 0.96127E+00_JPRB, 0.95048E+00_JPRB, 0.93745E+00_JPRB, 0.92847E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.81181E+00_JPRB, 0.80385E+00_JPRB, 0.79109E+00_JPRB, 0.78283E+00_JPRB, 0.77861E+00_JPRB, 0.78029E+00_JPRB, 0.78350E+00_JPRB ,&
&  0.78729E+00_JPRB, 0.78845E+00_JPRB, 0.79627E+00_JPRB, 0.80515E+00_JPRB, 0.81141E+00_JPRB, 0.81608E+00_JPRB, 0.81568E+00_JPRB ,&
&  0.81532E+00_JPRB, 0.81485E+00_JPRB, 0.80982E+00_JPRB, 0.80473E+00_JPRB, 0.79810E+00_JPRB, 0.79785E+00_JPRB, 0.79630E+00_JPRB ,&
&  0.79236E+00_JPRB, 0.78575E+00_JPRB, 0.77570E+00_JPRB, 0.76397E+00_JPRB, 0.75790E+00_JPRB, 0.75555E+00_JPRB, 0.75199E+00_JPRB ,&
&  0.75877E+00_JPRB, 0.68078E+00_JPRB, 0.73046E+00_JPRB, 0.77919E+00_JPRB, 0.84105E+00_JPRB, 0.88726E+00_JPRB, 0.92648E+00_JPRB ,&
&  0.96539E+00_JPRB, 0.99948E+00_JPRB, 0.10180E+01_JPRB, 0.10241E+01_JPRB, 0.10042E+01_JPRB, 0.96224E+00_JPRB, 0.91472E+00_JPRB ,&
&  0.87527E+00_JPRB, 0.84618E+00_JPRB, 0.81848E+00_JPRB, 0.80099E+00_JPRB, 0.78659E+00_JPRB, 0.76858E+00_JPRB, 0.75128E+00_JPRB ,&
&  0.73924E+00_JPRB, 0.73048E+00_JPRB, 0.72636E+00_JPRB, 0.72583E+00_JPRB, 0.72748E+00_JPRB, 0.72924E+00_JPRB, 0.72951E+00_JPRB ,&
&  0.73188E+00_JPRB, 0.73684E+00_JPRB, 0.74107E+00_JPRB, 0.74637E+00_JPRB, 0.75325E+00_JPRB, 0.76230E+00_JPRB, 0.76480E+00_JPRB ,&
&  0.76047E+00_JPRB, 0.76177E+00_JPRB, 0.76305E+00_JPRB, 0.76118E+00_JPRB, 0.75867E+00_JPRB, 0.75553E+00_JPRB, 0.75028E+00_JPRB ,&
&  0.74601E+00_JPRB, 0.74643E+00_JPRB, 0.74798E+00_JPRB, 0.75365E+00_JPRB, 0.75836E+00_JPRB, 0.76876E+00_JPRB, 0.77896E+00_JPRB ,&
&  0.79062E+00_JPRB, 0.80859E+00_JPRB, 0.82834E+00_JPRB, 0.84446E+00_JPRB, 0.85489E+00_JPRB, 0.86206E+00_JPRB, 0.86857E+00_JPRB ,&
&  0.87407E+00_JPRB, 0.87956E+00_JPRB, 0.88584E+00_JPRB, 0.88973E+00_JPRB, 0.88767E+00_JPRB, 0.88262E+00_JPRB, 0.87668E+00_JPRB ,&
&  0.86292E+00_JPRB, 0.84414E+00_JPRB, 0.83337E+00_JPRB, 0.82379E+00_JPRB, 0.81594E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.73624E+00_JPRB, 0.73056E+00_JPRB, 0.72477E+00_JPRB, 0.72352E+00_JPRB, 0.72211E+00_JPRB, 0.72337E+00_JPRB, 0.72533E+00_JPRB ,&
&  0.73154E+00_JPRB, 0.73564E+00_JPRB, 0.74521E+00_JPRB, 0.75348E+00_JPRB, 0.75800E+00_JPRB, 0.76322E+00_JPRB, 0.76649E+00_JPRB ,&
&  0.76795E+00_JPRB, 0.76973E+00_JPRB, 0.77064E+00_JPRB, 0.77498E+00_JPRB, 0.77547E+00_JPRB, 0.77465E+00_JPRB, 0.77319E+00_JPRB ,&
&  0.76993E+00_JPRB, 0.76555E+00_JPRB, 0.75759E+00_JPRB, 0.74752E+00_JPRB, 0.73548E+00_JPRB, 0.72890E+00_JPRB, 0.72085E+00_JPRB ,&
&  0.72029E+00_JPRB, 0.70370E+00_JPRB, 0.71843E+00_JPRB, 0.71743E+00_JPRB, 0.74753E+00_JPRB, 0.79399E+00_JPRB, 0.84142E+00_JPRB ,&
&  0.86856E+00_JPRB, 0.87822E+00_JPRB, 0.87266E+00_JPRB, 0.86128E+00_JPRB, 0.83967E+00_JPRB, 0.81891E+00_JPRB, 0.79560E+00_JPRB ,&
&  0.76959E+00_JPRB, 0.75395E+00_JPRB, 0.73671E+00_JPRB, 0.72395E+00_JPRB, 0.71791E+00_JPRB, 0.71306E+00_JPRB, 0.70471E+00_JPRB ,&
&  0.69209E+00_JPRB, 0.68087E+00_JPRB, 0.67860E+00_JPRB, 0.68080E+00_JPRB, 0.68391E+00_JPRB, 0.68585E+00_JPRB, 0.68432E+00_JPRB ,&
&  0.67821E+00_JPRB, 0.67446E+00_JPRB, 0.67548E+00_JPRB, 0.67821E+00_JPRB, 0.67741E+00_JPRB, 0.67292E+00_JPRB, 0.66766E+00_JPRB ,&
&  0.66715E+00_JPRB, 0.67004E+00_JPRB, 0.67337E+00_JPRB, 0.67773E+00_JPRB, 0.68046E+00_JPRB, 0.68369E+00_JPRB, 0.68483E+00_JPRB ,&
&  0.68893E+00_JPRB, 0.69305E+00_JPRB, 0.70188E+00_JPRB, 0.71061E+00_JPRB, 0.71276E+00_JPRB, 0.72204E+00_JPRB, 0.73227E+00_JPRB ,&
&  0.74375E+00_JPRB, 0.75774E+00_JPRB, 0.77053E+00_JPRB, 0.78152E+00_JPRB, 0.79223E+00_JPRB, 0.79880E+00_JPRB, 0.80619E+00_JPRB ,&
&  0.81491E+00_JPRB, 0.81728E+00_JPRB, 0.82335E+00_JPRB, 0.82376E+00_JPRB, 0.81880E+00_JPRB, 0.81042E+00_JPRB, 0.79813E+00_JPRB ,&
&  0.78320E+00_JPRB, 0.76370E+00_JPRB, 0.75162E+00_JPRB, 0.74375E+00_JPRB, 0.73899E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.65356E+00_JPRB, 0.65085E+00_JPRB, 0.65066E+00_JPRB, 0.65351E+00_JPRB, 0.65696E+00_JPRB, 0.65949E+00_JPRB, 0.66493E+00_JPRB ,&
&  0.67469E+00_JPRB, 0.68255E+00_JPRB, 0.69246E+00_JPRB, 0.70289E+00_JPRB, 0.71104E+00_JPRB, 0.71657E+00_JPRB, 0.71957E+00_JPRB ,&
&  0.71817E+00_JPRB, 0.71599E+00_JPRB, 0.71967E+00_JPRB, 0.72485E+00_JPRB, 0.72642E+00_JPRB, 0.72596E+00_JPRB, 0.72581E+00_JPRB ,&
&  0.72471E+00_JPRB, 0.72129E+00_JPRB, 0.71649E+00_JPRB, 0.70967E+00_JPRB, 0.70189E+00_JPRB, 0.69685E+00_JPRB, 0.69241E+00_JPRB ,&
&  0.68328E+00_JPRB, 0.68824E+00_JPRB, 0.63768E+00_JPRB, 0.65122E+00_JPRB, 0.66270E+00_JPRB, 0.68769E+00_JPRB, 0.71434E+00_JPRB ,&
&  0.73146E+00_JPRB, 0.73902E+00_JPRB, 0.74096E+00_JPRB, 0.73615E+00_JPRB, 0.72100E+00_JPRB, 0.71350E+00_JPRB, 0.70632E+00_JPRB ,&
&  0.69585E+00_JPRB, 0.68823E+00_JPRB, 0.67352E+00_JPRB, 0.65729E+00_JPRB, 0.65167E+00_JPRB, 0.65208E+00_JPRB, 0.65138E+00_JPRB ,&
&  0.64496E+00_JPRB, 0.63827E+00_JPRB, 0.63748E+00_JPRB, 0.63743E+00_JPRB, 0.63684E+00_JPRB, 0.63142E+00_JPRB, 0.62476E+00_JPRB ,&
&  0.61442E+00_JPRB, 0.60443E+00_JPRB, 0.59765E+00_JPRB, 0.59317E+00_JPRB, 0.58968E+00_JPRB, 0.58569E+00_JPRB, 0.58162E+00_JPRB ,&
&  0.58692E+00_JPRB, 0.60032E+00_JPRB, 0.61274E+00_JPRB, 0.62245E+00_JPRB, 0.62853E+00_JPRB, 0.63223E+00_JPRB, 0.63729E+00_JPRB ,&
&  0.64336E+00_JPRB, 0.65398E+00_JPRB, 0.66205E+00_JPRB, 0.66947E+00_JPRB, 0.67495E+00_JPRB, 0.68015E+00_JPRB, 0.68620E+00_JPRB ,&
&  0.69196E+00_JPRB, 0.70141E+00_JPRB, 0.70978E+00_JPRB, 0.71939E+00_JPRB, 0.72603E+00_JPRB, 0.73336E+00_JPRB, 0.74044E+00_JPRB ,&
&  0.74768E+00_JPRB, 0.75364E+00_JPRB, 0.75586E+00_JPRB, 0.75367E+00_JPRB, 0.74577E+00_JPRB, 0.73566E+00_JPRB, 0.72489E+00_JPRB ,&
&  0.71099E+00_JPRB, 0.69434E+00_JPRB, 0.67752E+00_JPRB, 0.66640E+00_JPRB, 0.65707E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.59033E+00_JPRB, 0.59043E+00_JPRB, 0.59465E+00_JPRB, 0.59877E+00_JPRB, 0.60192E+00_JPRB, 0.60669E+00_JPRB, 0.61644E+00_JPRB ,&
&  0.62636E+00_JPRB, 0.63594E+00_JPRB, 0.64249E+00_JPRB, 0.65045E+00_JPRB, 0.66190E+00_JPRB, 0.66984E+00_JPRB, 0.67616E+00_JPRB ,&
&  0.67901E+00_JPRB, 0.67996E+00_JPRB, 0.68174E+00_JPRB, 0.68545E+00_JPRB, 0.68534E+00_JPRB, 0.68182E+00_JPRB, 0.68052E+00_JPRB ,&
&  0.68018E+00_JPRB, 0.67796E+00_JPRB, 0.67686E+00_JPRB, 0.67657E+00_JPRB, 0.67241E+00_JPRB, 0.66655E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66160E+00_JPRB, 0.64731E+00_JPRB, 0.63883E+00_JPRB, 0.63078E+00_JPRB, 0.63315E+00_JPRB, 0.64322E+00_JPRB, 0.65486E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.66641E+00_JPRB, 0.66686E+00_JPRB, 0.66632E+00_JPRB, 0.66142E+00_JPRB, 0.65057E+00_JPRB, 0.64255E+00_JPRB ,&
&  0.63779E+00_JPRB, 0.62952E+00_JPRB, 0.60611E+00_JPRB, 0.59130E+00_JPRB, 0.58685E+00_JPRB, 0.58992E+00_JPRB, 0.59526E+00_JPRB ,&
&  0.59700E+00_JPRB, 0.59321E+00_JPRB, 0.58560E+00_JPRB, 0.57760E+00_JPRB, 0.56773E+00_JPRB, 0.55668E+00_JPRB, 0.54704E+00_JPRB ,&
&  0.54185E+00_JPRB, 0.53803E+00_JPRB, 0.53495E+00_JPRB, 0.53789E+00_JPRB, 0.53724E+00_JPRB, 0.53325E+00_JPRB, 0.53582E+00_JPRB ,&
&  0.54894E+00_JPRB, 0.56592E+00_JPRB, 0.58166E+00_JPRB, 0.59005E+00_JPRB, 0.59342E+00_JPRB, 0.59731E+00_JPRB, 0.60150E+00_JPRB ,&
&  0.60718E+00_JPRB, 0.61243E+00_JPRB, 0.61800E+00_JPRB, 0.62606E+00_JPRB, 0.63594E+00_JPRB, 0.64484E+00_JPRB, 0.64900E+00_JPRB ,&
&  0.65257E+00_JPRB, 0.65547E+00_JPRB, 0.65872E+00_JPRB, 0.66629E+00_JPRB, 0.67599E+00_JPRB, 0.68301E+00_JPRB, 0.68821E+00_JPRB ,&
&  0.69310E+00_JPRB, 0.69513E+00_JPRB, 0.69325E+00_JPRB, 0.68836E+00_JPRB, 0.68204E+00_JPRB, 0.67039E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.63905E+00_JPRB, 0.62326E+00_JPRB, 0.61111E+00_JPRB, 0.60059E+00_JPRB, 0.59307E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.54171E+00_JPRB, 0.54055E+00_JPRB, 0.54283E+00_JPRB, 0.54874E+00_JPRB, 0.55279E+00_JPRB, 0.55649E+00_JPRB, 0.56442E+00_JPRB ,&
&  0.57195E+00_JPRB, 0.57887E+00_JPRB, 0.58672E+00_JPRB, 0.59488E+00_JPRB, 0.60109E+00_JPRB, 0.61029E+00_JPRB, 0.61932E+00_JPRB ,&
&  0.62537E+00_JPRB, 0.62814E+00_JPRB, 0.63013E+00_JPRB, 0.63186E+00_JPRB, 0.63224E+00_JPRB, 0.63011E+00_JPRB, 0.62980E+00_JPRB ,&
&  0.62788E+00_JPRB, 0.62555E+00_JPRB, 0.62725E+00_JPRB, 0.62700E+00_JPRB, 0.62762E+00_JPRB, 0.62893E+00_JPRB, 0.62721E+00_JPRB ,&
&  0.62288E+00_JPRB, 0.60801E+00_JPRB, 0.60194E+00_JPRB, 0.59793E+00_JPRB, 0.60131E+00_JPRB, 0.60146E+00_JPRB, 0.59914E+00_JPRB ,&
&  0.59163E+00_JPRB, 0.58674E+00_JPRB, 0.58836E+00_JPRB, 0.59065E+00_JPRB, 0.58466E+00_JPRB, 0.57166E+00_JPRB, 0.56263E+00_JPRB ,&
&  0.55648E+00_JPRB, 0.54542E+00_JPRB, 0.52725E+00_JPRB, 0.51542E+00_JPRB, 0.51014E+00_JPRB, 0.50549E+00_JPRB, 0.50198E+00_JPRB ,&
&  0.50044E+00_JPRB, 0.49964E+00_JPRB, 0.50052E+00_JPRB, 0.49953E+00_JPRB, 0.49595E+00_JPRB, 0.49200E+00_JPRB, 0.49085E+00_JPRB ,&
&  0.49411E+00_JPRB, 0.49695E+00_JPRB, 0.50064E+00_JPRB, 0.50349E+00_JPRB, 0.50322E+00_JPRB, 0.50341E+00_JPRB, 0.50730E+00_JPRB ,&
&  0.51442E+00_JPRB, 0.52165E+00_JPRB, 0.52807E+00_JPRB, 0.53043E+00_JPRB, 0.53430E+00_JPRB, 0.54210E+00_JPRB, 0.54320E+00_JPRB ,&
&  0.54144E+00_JPRB, 0.54273E+00_JPRB, 0.54686E+00_JPRB, 0.55574E+00_JPRB, 0.56941E+00_JPRB, 0.57894E+00_JPRB, 0.58124E+00_JPRB ,&
&  0.58220E+00_JPRB, 0.58522E+00_JPRB, 0.59016E+00_JPRB, 0.60141E+00_JPRB, 0.61337E+00_JPRB, 0.62186E+00_JPRB, 0.62523E+00_JPRB ,&
&  0.62185E+00_JPRB, 0.61447E+00_JPRB, 0.60162E+00_JPRB, 0.58683E+00_JPRB, 0.57381E+00_JPRB, 0.56364E+00_JPRB, 0.55514E+00_JPRB ,&
&  0.54666E+00_JPRB, 0.54243E+00_JPRB, 0.54135E+00_JPRB, 0.54251E+00_JPRB, 0.54267E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.50285E+00_JPRB, 0.50415E+00_JPRB, 0.50619E+00_JPRB, 0.50905E+00_JPRB, 0.51201E+00_JPRB, 0.51361E+00_JPRB, 0.51709E+00_JPRB ,&
&  0.51946E+00_JPRB, 0.52660E+00_JPRB, 0.53802E+00_JPRB, 0.54915E+00_JPRB, 0.55703E+00_JPRB, 0.56358E+00_JPRB, 0.57029E+00_JPRB ,&
&  0.57599E+00_JPRB, 0.58343E+00_JPRB, 0.58480E+00_JPRB, 0.58288E+00_JPRB, 0.58068E+00_JPRB, 0.57905E+00_JPRB, 0.57749E+00_JPRB ,&
&  0.57494E+00_JPRB, 0.57462E+00_JPRB, 0.57805E+00_JPRB, 0.58179E+00_JPRB, 0.58542E+00_JPRB, 0.58560E+00_JPRB, 0.58297E+00_JPRB ,&
&  0.58493E+00_JPRB, 0.57364E+00_JPRB, 0.56586E+00_JPRB, 0.55733E+00_JPRB, 0.54992E+00_JPRB, 0.53779E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.51764E+00_JPRB, 0.50776E+00_JPRB, 0.50342E+00_JPRB, 0.49901E+00_JPRB, 0.48917E+00_JPRB, 0.47834E+00_JPRB, 0.47107E+00_JPRB ,&
&  0.46662E+00_JPRB, 0.46202E+00_JPRB, 0.45584E+00_JPRB, 0.45027E+00_JPRB, 0.44551E+00_JPRB, 0.44375E+00_JPRB, 0.44531E+00_JPRB ,&
&  0.45013E+00_JPRB, 0.45503E+00_JPRB, 0.45838E+00_JPRB, 0.45951E+00_JPRB, 0.45800E+00_JPRB, 0.45534E+00_JPRB, 0.45406E+00_JPRB ,&
&  0.45513E+00_JPRB, 0.45700E+00_JPRB, 0.45878E+00_JPRB, 0.46383E+00_JPRB, 0.46479E+00_JPRB, 0.46276E+00_JPRB, 0.45689E+00_JPRB ,&
&  0.45492E+00_JPRB, 0.45429E+00_JPRB, 0.45645E+00_JPRB, 0.46188E+00_JPRB, 0.47092E+00_JPRB, 0.47989E+00_JPRB, 0.48623E+00_JPRB ,&
&  0.48899E+00_JPRB, 0.49027E+00_JPRB, 0.49318E+00_JPRB, 0.49726E+00_JPRB, 0.49960E+00_JPRB, 0.49992E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.50288E+00_JPRB, 0.51127E+00_JPRB, 0.52014E+00_JPRB, 0.52644E+00_JPRB, 0.53017E+00_JPRB, 0.53008E+00_JPRB, 0.52368E+00_JPRB ,&
&  0.51107E+00_JPRB, 0.49621E+00_JPRB, 0.48188E+00_JPRB, 0.47001E+00_JPRB, 0.46180E+00_JPRB, 0.45777E+00_JPRB, 0.46042E+00_JPRB ,&
&  0.46786E+00_JPRB, 0.47749E+00_JPRB, 0.48596E+00_JPRB, 0.49194E+00_JPRB, 0.49855E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.46691E+00_JPRB, 0.46680E+00_JPRB, 0.46851E+00_JPRB, 0.46770E+00_JPRB, 0.46497E+00_JPRB, 0.46234E+00_JPRB, 0.46507E+00_JPRB ,&
&  0.47156E+00_JPRB, 0.48364E+00_JPRB, 0.49994E+00_JPRB, 0.51432E+00_JPRB, 0.52288E+00_JPRB, 0.52787E+00_JPRB, 0.53548E+00_JPRB ,&
&  0.54278E+00_JPRB, 0.55096E+00_JPRB, 0.55145E+00_JPRB, 0.54296E+00_JPRB, 0.53586E+00_JPRB, 0.53221E+00_JPRB, 0.53193E+00_JPRB ,&
&  0.53121E+00_JPRB, 0.53271E+00_JPRB, 0.53546E+00_JPRB, 0.53820E+00_JPRB, 0.53680E+00_JPRB, 0.53408E+00_JPRB, 0.53657E+00_JPRB ,&
&  0.53323E+00_JPRB, 0.52188E+00_JPRB, 0.50997E+00_JPRB, 0.48424E+00_JPRB, 0.48235E+00_JPRB, 0.47098E+00_JPRB, 0.46261E+00_JPRB ,&
&  0.45135E+00_JPRB, 0.43724E+00_JPRB, 0.42688E+00_JPRB, 0.42037E+00_JPRB, 0.41729E+00_JPRB, 0.41491E+00_JPRB, 0.41328E+00_JPRB ,&
&  0.41258E+00_JPRB, 0.41340E+00_JPRB, 0.41595E+00_JPRB, 0.41393E+00_JPRB, 0.41280E+00_JPRB, 0.41378E+00_JPRB, 0.41598E+00_JPRB ,&
&  0.41702E+00_JPRB, 0.41818E+00_JPRB, 0.41927E+00_JPRB, 0.41927E+00_JPRB, 0.41785E+00_JPRB, 0.41528E+00_JPRB, 0.41296E+00_JPRB ,&
&  0.41224E+00_JPRB, 0.41379E+00_JPRB, 0.41464E+00_JPRB, 0.41079E+00_JPRB, 0.40297E+00_JPRB, 0.39234E+00_JPRB, 0.38535E+00_JPRB ,&
&  0.38274E+00_JPRB, 0.38553E+00_JPRB, 0.39428E+00_JPRB, 0.40606E+00_JPRB, 0.41671E+00_JPRB, 0.42323E+00_JPRB, 0.42759E+00_JPRB ,&
&  0.43217E+00_JPRB, 0.42929E+00_JPRB, 0.42499E+00_JPRB, 0.42165E+00_JPRB, 0.41875E+00_JPRB, 0.41454E+00_JPRB, 0.41453E+00_JPRB ,&
&  0.41800E+00_JPRB, 0.42174E+00_JPRB, 0.42333E+00_JPRB, 0.42263E+00_JPRB, 0.42067E+00_JPRB, 0.41647E+00_JPRB, 0.41062E+00_JPRB ,&
&  0.40313E+00_JPRB, 0.39433E+00_JPRB, 0.39298E+00_JPRB, 0.39444E+00_JPRB, 0.39851E+00_JPRB, 0.40636E+00_JPRB, 0.41709E+00_JPRB ,&
&  0.42826E+00_JPRB, 0.43704E+00_JPRB, 0.44661E+00_JPRB, 0.45544E+00_JPRB, 0.46345E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.44364E+00_JPRB, 0.44495E+00_JPRB, 0.44224E+00_JPRB, 0.43956E+00_JPRB, 0.43946E+00_JPRB, 0.44086E+00_JPRB, 0.44604E+00_JPRB ,&
&  0.45572E+00_JPRB, 0.46904E+00_JPRB, 0.48196E+00_JPRB, 0.49258E+00_JPRB, 0.50186E+00_JPRB, 0.50585E+00_JPRB, 0.51165E+00_JPRB ,&
&  0.51801E+00_JPRB, 0.52385E+00_JPRB, 0.52460E+00_JPRB, 0.51431E+00_JPRB, 0.50443E+00_JPRB, 0.49920E+00_JPRB, 0.49922E+00_JPRB ,&
&  0.50081E+00_JPRB, 0.50303E+00_JPRB, 0.50866E+00_JPRB, 0.51026E+00_JPRB, 0.50359E+00_JPRB, 0.50129E+00_JPRB, 0.49836E+00_JPRB ,&
&  0.49176E+00_JPRB, 0.47812E+00_JPRB, 0.44465E+00_JPRB, 0.43741E+00_JPRB, 0.42831E+00_JPRB, 0.41885E+00_JPRB, 0.40995E+00_JPRB ,&
&  0.39956E+00_JPRB, 0.38881E+00_JPRB, 0.38149E+00_JPRB, 0.37703E+00_JPRB, 0.37739E+00_JPRB, 0.37873E+00_JPRB, 0.37865E+00_JPRB ,&
&  0.38124E+00_JPRB, 0.38382E+00_JPRB, 0.38645E+00_JPRB, 0.38736E+00_JPRB, 0.38764E+00_JPRB, 0.38766E+00_JPRB, 0.38436E+00_JPRB ,&
&  0.38145E+00_JPRB, 0.38030E+00_JPRB, 0.37946E+00_JPRB, 0.37872E+00_JPRB, 0.37900E+00_JPRB, 0.37982E+00_JPRB, 0.38011E+00_JPRB ,&
&  0.37747E+00_JPRB, 0.37384E+00_JPRB, 0.36970E+00_JPRB, 0.36201E+00_JPRB, 0.34730E+00_JPRB, 0.33343E+00_JPRB, 0.32388E+00_JPRB ,&
&  0.32297E+00_JPRB, 0.33338E+00_JPRB, 0.35327E+00_JPRB, 0.36550E+00_JPRB, 0.37330E+00_JPRB, 0.37680E+00_JPRB, 0.38034E+00_JPRB ,&
&  0.37237E+00_JPRB, 0.35088E+00_JPRB, 0.33850E+00_JPRB, 0.33079E+00_JPRB, 0.32583E+00_JPRB, 0.32275E+00_JPRB, 0.32233E+00_JPRB ,&
&  0.32606E+00_JPRB, 0.33172E+00_JPRB, 0.33835E+00_JPRB, 0.34148E+00_JPRB, 0.34044E+00_JPRB, 0.33756E+00_JPRB, 0.33404E+00_JPRB ,&
&  0.33221E+00_JPRB, 0.33784E+00_JPRB, 0.35287E+00_JPRB, 0.37137E+00_JPRB, 0.38004E+00_JPRB, 0.38704E+00_JPRB, 0.39336E+00_JPRB ,&
&  0.40031E+00_JPRB, 0.41078E+00_JPRB, 0.42288E+00_JPRB, 0.43206E+00_JPRB, 0.44003E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.41299E+00_JPRB, 0.41651E+00_JPRB, 0.41852E+00_JPRB, 0.42168E+00_JPRB, 0.42938E+00_JPRB, 0.43990E+00_JPRB, 0.45190E+00_JPRB ,&
&  0.46369E+00_JPRB, 0.47390E+00_JPRB, 0.47927E+00_JPRB, 0.48472E+00_JPRB, 0.48972E+00_JPRB, 0.49197E+00_JPRB, 0.49677E+00_JPRB ,&
&  0.49726E+00_JPRB, 0.49347E+00_JPRB, 0.48711E+00_JPRB, 0.47899E+00_JPRB, 0.47467E+00_JPRB, 0.47573E+00_JPRB, 0.47826E+00_JPRB ,&
&  0.47961E+00_JPRB, 0.48056E+00_JPRB, 0.48016E+00_JPRB, 0.47609E+00_JPRB, 0.47238E+00_JPRB, 0.47009E+00_JPRB, 0.46524E+00_JPRB ,&
&  0.45310E+00_JPRB, 0.40228E+00_JPRB, 0.36522E+00_JPRB, 0.36247E+00_JPRB, 0.37650E+00_JPRB, 0.37823E+00_JPRB, 0.37402E+00_JPRB ,&
&  0.36883E+00_JPRB, 0.36370E+00_JPRB, 0.36063E+00_JPRB, 0.35829E+00_JPRB, 0.35551E+00_JPRB, 0.35290E+00_JPRB, 0.35329E+00_JPRB ,&
&  0.35457E+00_JPRB, 0.35252E+00_JPRB, 0.34975E+00_JPRB, 0.34335E+00_JPRB, 0.33895E+00_JPRB, 0.33818E+00_JPRB, 0.33766E+00_JPRB ,&
&  0.33370E+00_JPRB, 0.32942E+00_JPRB, 0.32985E+00_JPRB, 0.33160E+00_JPRB, 0.33419E+00_JPRB, 0.33589E+00_JPRB, 0.33271E+00_JPRB ,&
&  0.32881E+00_JPRB, 0.32146E+00_JPRB, 0.31346E+00_JPRB, 0.30355E+00_JPRB, 0.29439E+00_JPRB, 0.28735E+00_JPRB, 0.28134E+00_JPRB ,&
&  0.28125E+00_JPRB, 0.28915E+00_JPRB, 0.30455E+00_JPRB, 0.31970E+00_JPRB, 0.33072E+00_JPRB, 0.32463E+00_JPRB, 0.31571E+00_JPRB ,&
&  0.30430E+00_JPRB, 0.29405E+00_JPRB, 0.28682E+00_JPRB, 0.28376E+00_JPRB, 0.28256E+00_JPRB, 0.28248E+00_JPRB, 0.28388E+00_JPRB ,&
&  0.28653E+00_JPRB, 0.28956E+00_JPRB, 0.29187E+00_JPRB, 0.29293E+00_JPRB, 0.29319E+00_JPRB, 0.29316E+00_JPRB, 0.29235E+00_JPRB ,&
&  0.29158E+00_JPRB, 0.29444E+00_JPRB, 0.30175E+00_JPRB, 0.31079E+00_JPRB, 0.31907E+00_JPRB, 0.32730E+00_JPRB, 0.34048E+00_JPRB ,&
&  0.35657E+00_JPRB, 0.37527E+00_JPRB, 0.38941E+00_JPRB, 0.40056E+00_JPRB, 0.40941E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.40054E+00_JPRB, 0.40966E+00_JPRB, 0.42099E+00_JPRB, 0.43119E+00_JPRB, 0.44074E+00_JPRB, 0.44956E+00_JPRB, 0.45775E+00_JPRB ,&
&  0.46282E+00_JPRB, 0.46577E+00_JPRB, 0.46706E+00_JPRB, 0.47092E+00_JPRB, 0.46763E+00_JPRB, 0.45976E+00_JPRB, 0.45384E+00_JPRB ,&
&  0.44875E+00_JPRB, 0.44488E+00_JPRB, 0.43941E+00_JPRB, 0.43653E+00_JPRB, 0.43612E+00_JPRB, 0.43595E+00_JPRB, 0.43005E+00_JPRB ,&
&  0.41879E+00_JPRB, 0.41659E+00_JPRB, 0.42062E+00_JPRB, 0.41430E+00_JPRB, 0.40803E+00_JPRB, 0.40181E+00_JPRB, 0.39549E+00_JPRB ,&
&  0.37805E+00_JPRB, 0.35696E+00_JPRB, 0.33099E+00_JPRB, 0.32954E+00_JPRB, 0.34519E+00_JPRB, 0.33726E+00_JPRB, 0.34170E+00_JPRB ,&
&  0.34314E+00_JPRB, 0.34300E+00_JPRB, 0.34155E+00_JPRB, 0.33963E+00_JPRB, 0.33627E+00_JPRB, 0.33565E+00_JPRB, 0.33378E+00_JPRB ,&
&  0.32670E+00_JPRB, 0.32140E+00_JPRB, 0.31252E+00_JPRB, 0.29353E+00_JPRB, 0.29007E+00_JPRB, 0.29032E+00_JPRB, 0.28653E+00_JPRB ,&
&  0.27916E+00_JPRB, 0.27525E+00_JPRB, 0.27461E+00_JPRB, 0.26918E+00_JPRB, 0.27150E+00_JPRB, 0.27414E+00_JPRB, 0.27388E+00_JPRB ,&
&  0.27363E+00_JPRB, 0.27302E+00_JPRB, 0.27068E+00_JPRB, 0.26660E+00_JPRB, 0.26317E+00_JPRB, 0.26233E+00_JPRB, 0.26378E+00_JPRB ,&
&  0.26655E+00_JPRB, 0.27084E+00_JPRB, 0.28273E+00_JPRB, 0.29159E+00_JPRB, 0.28978E+00_JPRB, 0.28622E+00_JPRB, 0.27666E+00_JPRB ,&
&  0.26677E+00_JPRB, 0.26332E+00_JPRB, 0.26023E+00_JPRB, 0.25844E+00_JPRB, 0.25864E+00_JPRB, 0.26017E+00_JPRB, 0.26073E+00_JPRB ,&
&  0.26201E+00_JPRB, 0.26352E+00_JPRB, 0.26453E+00_JPRB, 0.26496E+00_JPRB, 0.26565E+00_JPRB, 0.26647E+00_JPRB, 0.26817E+00_JPRB ,&
&  0.27069E+00_JPRB, 0.27526E+00_JPRB, 0.28122E+00_JPRB, 0.28765E+00_JPRB, 0.29244E+00_JPRB, 0.29495E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.30694E+00_JPRB, 0.32397E+00_JPRB, 0.35347E+00_JPRB, 0.39624E+00_JPRB, 0.39845E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.39660E+00_JPRB, 0.40897E+00_JPRB, 0.42406E+00_JPRB, 0.43380E+00_JPRB, 0.43994E+00_JPRB, 0.44322E+00_JPRB, 0.44273E+00_JPRB ,&
&  0.43943E+00_JPRB, 0.43857E+00_JPRB, 0.44931E+00_JPRB, 0.43006E+00_JPRB, 0.41290E+00_JPRB, 0.38074E+00_JPRB, 0.36407E+00_JPRB ,&
&  0.35087E+00_JPRB, 0.34836E+00_JPRB, 0.34757E+00_JPRB, 0.35194E+00_JPRB, 0.35517E+00_JPRB, 0.35763E+00_JPRB, 0.35303E+00_JPRB ,&
&  0.34696E+00_JPRB, 0.33744E+00_JPRB, 0.32946E+00_JPRB, 0.32223E+00_JPRB, 0.32275E+00_JPRB, 0.32154E+00_JPRB, 0.31888E+00_JPRB ,&
&  0.31618E+00_JPRB, 0.31475E+00_JPRB, 0.31513E+00_JPRB, 0.31648E+00_JPRB, 0.32804E+00_JPRB, 0.32633E+00_JPRB, 0.32756E+00_JPRB ,&
&  0.32769E+00_JPRB, 0.32765E+00_JPRB, 0.32655E+00_JPRB, 0.32481E+00_JPRB, 0.31984E+00_JPRB, 0.30264E+00_JPRB, 0.30066E+00_JPRB ,&
&  0.29310E+00_JPRB, 0.28295E+00_JPRB, 0.27276E+00_JPRB, 0.26569E+00_JPRB, 0.26138E+00_JPRB, 0.26021E+00_JPRB, 0.25558E+00_JPRB ,&
&  0.25069E+00_JPRB, 0.24450E+00_JPRB, 0.24190E+00_JPRB, 0.24218E+00_JPRB, 0.24310E+00_JPRB, 0.24377E+00_JPRB, 0.24427E+00_JPRB ,&
&  0.24583E+00_JPRB, 0.24656E+00_JPRB, 0.24699E+00_JPRB, 0.24790E+00_JPRB, 0.24877E+00_JPRB, 0.24997E+00_JPRB, 0.25199E+00_JPRB ,&
&  0.25558E+00_JPRB, 0.25930E+00_JPRB, 0.26291E+00_JPRB, 0.26666E+00_JPRB, 0.26568E+00_JPRB, 0.26126E+00_JPRB, 0.25568E+00_JPRB ,&
&  0.25150E+00_JPRB, 0.24995E+00_JPRB, 0.24846E+00_JPRB, 0.24811E+00_JPRB, 0.24804E+00_JPRB, 0.24834E+00_JPRB, 0.24891E+00_JPRB ,&
&  0.24984E+00_JPRB, 0.25061E+00_JPRB, 0.25154E+00_JPRB, 0.25278E+00_JPRB, 0.25441E+00_JPRB, 0.25592E+00_JPRB, 0.25799E+00_JPRB ,&
&  0.26110E+00_JPRB, 0.26452E+00_JPRB, 0.26923E+00_JPRB, 0.27347E+00_JPRB, 0.28010E+00_JPRB, 0.28871E+00_JPRB, 0.30285E+00_JPRB ,&
&  0.31740E+00_JPRB, 0.34972E+00_JPRB, 0.36361E+00_JPRB, 0.37792E+00_JPRB, 0.38651E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.39581E+00_JPRB, 0.40604E+00_JPRB, 0.42004E+00_JPRB, 0.42950E+00_JPRB, 0.43654E+00_JPRB, 0.43246E+00_JPRB, 0.42842E+00_JPRB ,&
&  0.42404E+00_JPRB, 0.41056E+00_JPRB, 0.39315E+00_JPRB, 0.36402E+00_JPRB, 0.33952E+00_JPRB, 0.32092E+00_JPRB, 0.30986E+00_JPRB ,&
&  0.30255E+00_JPRB, 0.29973E+00_JPRB, 0.29656E+00_JPRB, 0.29527E+00_JPRB, 0.29303E+00_JPRB, 0.29389E+00_JPRB, 0.29457E+00_JPRB ,&
&  0.29366E+00_JPRB, 0.29289E+00_JPRB, 0.29281E+00_JPRB, 0.29615E+00_JPRB, 0.30279E+00_JPRB, 0.30784E+00_JPRB, 0.31289E+00_JPRB ,&
&  0.31609E+00_JPRB, 0.31302E+00_JPRB, 0.31507E+00_JPRB, 0.31361E+00_JPRB, 0.31317E+00_JPRB, 0.31491E+00_JPRB, 0.31702E+00_JPRB ,&
&  0.31740E+00_JPRB, 0.31695E+00_JPRB, 0.31563E+00_JPRB, 0.31164E+00_JPRB, 0.29860E+00_JPRB, 0.29226E+00_JPRB, 0.28363E+00_JPRB ,&
&  0.27543E+00_JPRB, 0.26590E+00_JPRB, 0.25776E+00_JPRB, 0.25425E+00_JPRB, 0.24911E+00_JPRB, 0.24748E+00_JPRB, 0.24413E+00_JPRB ,&
&  0.24036E+00_JPRB, 0.23691E+00_JPRB, 0.23498E+00_JPRB, 0.23406E+00_JPRB, 0.23373E+00_JPRB, 0.23207E+00_JPRB, 0.23154E+00_JPRB ,&
&  0.23204E+00_JPRB, 0.23251E+00_JPRB, 0.23381E+00_JPRB, 0.23483E+00_JPRB, 0.23590E+00_JPRB, 0.23669E+00_JPRB, 0.23759E+00_JPRB ,&
&  0.23933E+00_JPRB, 0.24166E+00_JPRB, 0.24269E+00_JPRB, 0.24375E+00_JPRB, 0.24336E+00_JPRB, 0.24252E+00_JPRB, 0.24138E+00_JPRB ,&
&  0.24039E+00_JPRB, 0.24004E+00_JPRB, 0.24003E+00_JPRB, 0.23986E+00_JPRB, 0.24030E+00_JPRB, 0.24118E+00_JPRB, 0.24195E+00_JPRB ,&
&  0.24356E+00_JPRB, 0.24496E+00_JPRB, 0.24661E+00_JPRB, 0.24881E+00_JPRB, 0.25114E+00_JPRB, 0.25403E+00_JPRB, 0.25720E+00_JPRB ,&
&  0.25975E+00_JPRB, 0.26323E+00_JPRB, 0.26614E+00_JPRB, 0.26843E+00_JPRB, 0.27532E+00_JPRB, 0.28308E+00_JPRB, 0.29920E+00_JPRB ,&
&  0.31607E+00_JPRB, 0.35312E+00_JPRB, 0.39317E+00_JPRB, 0.38127E+00_JPRB, 0.38738E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.36808E+00_JPRB, 0.37955E+00_JPRB, 0.38813E+00_JPRB, 0.39357E+00_JPRB, 0.39863E+00_JPRB, 0.40030E+00_JPRB, 0.40718E+00_JPRB ,&
&  0.40170E+00_JPRB, 0.38415E+00_JPRB, 0.37302E+00_JPRB, 0.35621E+00_JPRB, 0.33583E+00_JPRB, 0.31880E+00_JPRB, 0.30972E+00_JPRB ,&
&  0.30417E+00_JPRB, 0.29943E+00_JPRB, 0.29396E+00_JPRB, 0.28809E+00_JPRB, 0.28231E+00_JPRB, 0.27793E+00_JPRB, 0.27542E+00_JPRB ,&
&  0.27308E+00_JPRB, 0.27449E+00_JPRB, 0.27805E+00_JPRB, 0.28326E+00_JPRB, 0.29077E+00_JPRB, 0.29500E+00_JPRB, 0.29858E+00_JPRB ,&
&  0.29919E+00_JPRB, 0.28835E+00_JPRB, 0.29216E+00_JPRB, 0.29573E+00_JPRB, 0.29510E+00_JPRB, 0.30029E+00_JPRB, 0.30910E+00_JPRB ,&
&  0.30514E+00_JPRB, 0.29405E+00_JPRB, 0.29572E+00_JPRB, 0.28794E+00_JPRB, 0.27907E+00_JPRB, 0.27857E+00_JPRB, 0.27266E+00_JPRB ,&
&  0.27162E+00_JPRB, 0.26462E+00_JPRB, 0.25949E+00_JPRB, 0.25619E+00_JPRB, 0.24883E+00_JPRB, 0.24599E+00_JPRB, 0.24188E+00_JPRB ,&
&  0.23796E+00_JPRB, 0.23562E+00_JPRB, 0.23267E+00_JPRB, 0.23162E+00_JPRB, 0.23060E+00_JPRB, 0.22907E+00_JPRB, 0.22779E+00_JPRB ,&
&  0.22699E+00_JPRB, 0.22628E+00_JPRB, 0.22626E+00_JPRB, 0.22618E+00_JPRB, 0.22597E+00_JPRB, 0.22621E+00_JPRB, 0.22621E+00_JPRB ,&
&  0.22687E+00_JPRB, 0.22779E+00_JPRB, 0.22852E+00_JPRB, 0.22983E+00_JPRB, 0.23027E+00_JPRB, 0.23094E+00_JPRB, 0.23134E+00_JPRB ,&
&  0.23171E+00_JPRB, 0.23235E+00_JPRB, 0.23244E+00_JPRB, 0.23301E+00_JPRB, 0.23360E+00_JPRB, 0.23510E+00_JPRB, 0.23656E+00_JPRB ,&
&  0.23840E+00_JPRB, 0.24071E+00_JPRB, 0.24309E+00_JPRB, 0.24565E+00_JPRB, 0.24883E+00_JPRB, 0.25160E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.25957E+00_JPRB, 0.26358E+00_JPRB, 0.26878E+00_JPRB, 0.27497E+00_JPRB, 0.28352E+00_JPRB, 0.29376E+00_JPRB, 0.30213E+00_JPRB ,&
&  0.31150E+00_JPRB, 0.33564E+00_JPRB, 0.34197E+00_JPRB, 0.34927E+00_JPRB, 0.35884E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.35678E+00_JPRB, 0.36222E+00_JPRB, 0.36640E+00_JPRB, 0.36913E+00_JPRB, 0.37193E+00_JPRB, 0.37367E+00_JPRB, 0.37434E+00_JPRB ,&
&  0.36766E+00_JPRB, 0.36277E+00_JPRB, 0.35551E+00_JPRB, 0.34660E+00_JPRB, 0.33837E+00_JPRB, 0.32944E+00_JPRB, 0.32045E+00_JPRB ,&
&  0.31302E+00_JPRB, 0.30604E+00_JPRB, 0.29927E+00_JPRB, 0.29200E+00_JPRB, 0.28453E+00_JPRB, 0.27699E+00_JPRB, 0.27447E+00_JPRB ,&
&  0.27444E+00_JPRB, 0.27666E+00_JPRB, 0.27994E+00_JPRB, 0.28151E+00_JPRB, 0.28244E+00_JPRB, 0.28277E+00_JPRB, 0.28151E+00_JPRB ,&
&  0.27740E+00_JPRB, 0.27859E+00_JPRB, 0.28035E+00_JPRB, 0.28147E+00_JPRB, 0.28353E+00_JPRB, 0.28539E+00_JPRB, 0.28542E+00_JPRB ,&
&  0.28328E+00_JPRB, 0.28056E+00_JPRB, 0.27682E+00_JPRB, 0.27327E+00_JPRB, 0.27237E+00_JPRB, 0.27044E+00_JPRB, 0.26825E+00_JPRB ,&
&  0.26718E+00_JPRB, 0.26338E+00_JPRB, 0.25894E+00_JPRB, 0.25581E+00_JPRB, 0.25118E+00_JPRB, 0.24805E+00_JPRB, 0.24387E+00_JPRB ,&
&  0.24066E+00_JPRB, 0.23790E+00_JPRB, 0.23484E+00_JPRB, 0.23259E+00_JPRB, 0.23054E+00_JPRB, 0.22902E+00_JPRB, 0.22746E+00_JPRB ,&
&  0.22658E+00_JPRB, 0.22567E+00_JPRB, 0.22505E+00_JPRB, 0.22432E+00_JPRB, 0.22380E+00_JPRB, 0.22330E+00_JPRB, 0.22291E+00_JPRB ,&
&  0.22269E+00_JPRB, 0.22307E+00_JPRB, 0.22331E+00_JPRB, 0.22375E+00_JPRB, 0.22426E+00_JPRB, 0.22446E+00_JPRB, 0.22500E+00_JPRB ,&
&  0.22613E+00_JPRB, 0.22737E+00_JPRB, 0.22800E+00_JPRB, 0.22893E+00_JPRB, 0.22999E+00_JPRB, 0.23125E+00_JPRB, 0.23312E+00_JPRB ,&
&  0.23505E+00_JPRB, 0.23722E+00_JPRB, 0.23973E+00_JPRB, 0.24213E+00_JPRB, 0.24540E+00_JPRB, 0.24878E+00_JPRB, 0.25239E+00_JPRB ,&
&  0.25688E+00_JPRB, 0.26158E+00_JPRB, 0.26657E+00_JPRB, 0.27208E+00_JPRB, 0.28017E+00_JPRB, 0.28603E+00_JPRB, 0.29316E+00_JPRB ,&
&  0.29956E+00_JPRB, 0.30225E+00_JPRB, 0.33581E+00_JPRB, 0.34056E+00_JPRB, 0.34759E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.28111E+00_JPRB, 0.28722E+00_JPRB, 0.29287E+00_JPRB, 0.30105E+00_JPRB, 0.30347E+00_JPRB, 0.33983E+00_JPRB, 0.33478E+00_JPRB ,&
&  0.33221E+00_JPRB, 0.32081E+00_JPRB, 0.32028E+00_JPRB, 0.31712E+00_JPRB, 0.31281E+00_JPRB, 0.30823E+00_JPRB, 0.30339E+00_JPRB ,&
&  0.29891E+00_JPRB, 0.29350E+00_JPRB, 0.28924E+00_JPRB, 0.28578E+00_JPRB, 0.28272E+00_JPRB, 0.27966E+00_JPRB, 0.27707E+00_JPRB ,&
&  0.27611E+00_JPRB, 0.27620E+00_JPRB, 0.27553E+00_JPRB, 0.27039E+00_JPRB, 0.26657E+00_JPRB, 0.26434E+00_JPRB, 0.26383E+00_JPRB ,&
&  0.26391E+00_JPRB, 0.26633E+00_JPRB, 0.26814E+00_JPRB, 0.26980E+00_JPRB, 0.27217E+00_JPRB, 0.27130E+00_JPRB, 0.27041E+00_JPRB ,&
&  0.26905E+00_JPRB, 0.26762E+00_JPRB, 0.26621E+00_JPRB, 0.26491E+00_JPRB, 0.26362E+00_JPRB, 0.26164E+00_JPRB, 0.26011E+00_JPRB ,&
&  0.25873E+00_JPRB, 0.25671E+00_JPRB, 0.25352E+00_JPRB, 0.25144E+00_JPRB, 0.24889E+00_JPRB, 0.24675E+00_JPRB, 0.24448E+00_JPRB ,&
&  0.24275E+00_JPRB, 0.24124E+00_JPRB, 0.23955E+00_JPRB, 0.23750E+00_JPRB, 0.23552E+00_JPRB, 0.23378E+00_JPRB, 0.23216E+00_JPRB ,&
&  0.23135E+00_JPRB, 0.23046E+00_JPRB, 0.22958E+00_JPRB, 0.22865E+00_JPRB, 0.22821E+00_JPRB, 0.22770E+00_JPRB, 0.22722E+00_JPRB ,&
&  0.22684E+00_JPRB, 0.22653E+00_JPRB, 0.22648E+00_JPRB, 0.22651E+00_JPRB, 0.22658E+00_JPRB, 0.22695E+00_JPRB, 0.22742E+00_JPRB ,&
&  0.22807E+00_JPRB, 0.22887E+00_JPRB, 0.23004E+00_JPRB, 0.23126E+00_JPRB, 0.23233E+00_JPRB, 0.23314E+00_JPRB, 0.23413E+00_JPRB ,&
&  0.23574E+00_JPRB, 0.23757E+00_JPRB, 0.23961E+00_JPRB, 0.24102E+00_JPRB, 0.24331E+00_JPRB, 0.24542E+00_JPRB, 0.24755E+00_JPRB ,&
&  0.24966E+00_JPRB, 0.25237E+00_JPRB, 0.25557E+00_JPRB, 0.25916E+00_JPRB, 0.26350E+00_JPRB, 0.26661E+00_JPRB, 0.26956E+00_JPRB ,&
&  0.27257E+00_JPRB, 0.27729E+00_JPRB, 0.27908E+00_JPRB, 0.27971E+00_JPRB, 0.28037E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.26100E+00_JPRB, 0.26085E+00_JPRB, 0.26105E+00_JPRB, 0.26122E+00_JPRB, 0.26126E+00_JPRB, 0.26169E+00_JPRB, 0.26160E+00_JPRB ,&
&  0.26131E+00_JPRB, 0.26095E+00_JPRB, 0.26128E+00_JPRB, 0.26164E+00_JPRB, 0.26184E+00_JPRB, 0.26185E+00_JPRB, 0.26143E+00_JPRB ,&
&  0.26064E+00_JPRB, 0.25966E+00_JPRB, 0.25786E+00_JPRB, 0.25710E+00_JPRB, 0.25655E+00_JPRB, 0.25610E+00_JPRB, 0.25528E+00_JPRB ,&
&  0.25452E+00_JPRB, 0.25326E+00_JPRB, 0.25159E+00_JPRB, 0.25062E+00_JPRB, 0.25068E+00_JPRB, 0.25133E+00_JPRB, 0.25232E+00_JPRB ,&
&  0.25309E+00_JPRB, 0.25364E+00_JPRB, 0.25393E+00_JPRB, 0.25430E+00_JPRB, 0.25478E+00_JPRB, 0.25471E+00_JPRB, 0.25451E+00_JPRB ,&
&  0.25420E+00_JPRB, 0.25363E+00_JPRB, 0.25308E+00_JPRB, 0.25259E+00_JPRB, 0.25216E+00_JPRB, 0.25160E+00_JPRB, 0.25096E+00_JPRB ,&
&  0.25021E+00_JPRB, 0.24928E+00_JPRB, 0.24812E+00_JPRB, 0.24720E+00_JPRB, 0.24621E+00_JPRB, 0.24543E+00_JPRB, 0.24442E+00_JPRB ,&
&  0.24357E+00_JPRB, 0.24279E+00_JPRB, 0.24201E+00_JPRB, 0.24128E+00_JPRB, 0.24061E+00_JPRB, 0.23997E+00_JPRB, 0.23937E+00_JPRB ,&
&  0.23878E+00_JPRB, 0.23829E+00_JPRB, 0.23775E+00_JPRB, 0.23725E+00_JPRB, 0.23690E+00_JPRB, 0.23660E+00_JPRB, 0.23636E+00_JPRB ,&
&  0.23616E+00_JPRB, 0.23612E+00_JPRB, 0.23612E+00_JPRB, 0.23611E+00_JPRB, 0.23612E+00_JPRB, 0.23624E+00_JPRB, 0.23637E+00_JPRB ,&
&  0.23653E+00_JPRB, 0.23664E+00_JPRB, 0.23675E+00_JPRB, 0.23691E+00_JPRB, 0.23709E+00_JPRB, 0.23731E+00_JPRB, 0.23761E+00_JPRB ,&
&  0.23797E+00_JPRB, 0.23840E+00_JPRB, 0.23887E+00_JPRB, 0.23908E+00_JPRB, 0.23976E+00_JPRB, 0.24056E+00_JPRB, 0.24152E+00_JPRB ,&
&  0.24259E+00_JPRB, 0.24390E+00_JPRB, 0.24542E+00_JPRB, 0.24730E+00_JPRB, 0.24931E+00_JPRB, 0.25049E+00_JPRB, 0.25170E+00_JPRB ,&
&  0.25294E+00_JPRB, 0.25445E+00_JPRB, 0.25587E+00_JPRB, 0.25730E+00_JPRB, 0.25895E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB, 0.24304E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB, 0.74343E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.75604E+00_JPRB, 0.75545E+00_JPRB, 0.75488E+00_JPRB, 0.75423E+00_JPRB, 0.75338E+00_JPRB, 0.75254E+00_JPRB, 0.75185E+00_JPRB ,&
&  0.75114E+00_JPRB, 0.75069E+00_JPRB, 0.74981E+00_JPRB, 0.74889E+00_JPRB, 0.74793E+00_JPRB, 0.74709E+00_JPRB, 0.74615E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.74420E+00_JPRB, 0.74347E+00_JPRB, 0.74259E+00_JPRB, 0.74163E+00_JPRB, 0.74075E+00_JPRB, 0.73996E+00_JPRB ,&
&  0.73924E+00_JPRB, 0.73839E+00_JPRB, 0.73752E+00_JPRB, 0.73646E+00_JPRB, 0.73579E+00_JPRB, 0.73481E+00_JPRB, 0.73403E+00_JPRB ,&
&  0.73312E+00_JPRB, 0.73251E+00_JPRB, 0.73181E+00_JPRB, 0.73115E+00_JPRB, 0.73033E+00_JPRB, 0.73028E+00_JPRB, 0.72981E+00_JPRB ,&
&  0.72962E+00_JPRB, 0.72940E+00_JPRB, 0.72946E+00_JPRB, 0.72941E+00_JPRB, 0.72946E+00_JPRB, 0.72958E+00_JPRB, 0.73018E+00_JPRB ,&
&  0.73068E+00_JPRB, 0.73128E+00_JPRB, 0.73218E+00_JPRB, 0.73300E+00_JPRB, 0.73375E+00_JPRB, 0.73453E+00_JPRB, 0.73539E+00_JPRB ,&
&  0.73656E+00_JPRB, 0.73771E+00_JPRB, 0.73879E+00_JPRB, 0.73990E+00_JPRB, 0.74100E+00_JPRB, 0.74189E+00_JPRB, 0.74263E+00_JPRB ,&
&  0.74307E+00_JPRB, 0.74362E+00_JPRB, 0.74412E+00_JPRB, 0.74465E+00_JPRB, 0.74474E+00_JPRB, 0.74508E+00_JPRB, 0.74528E+00_JPRB ,&
&  0.74547E+00_JPRB, 0.74511E+00_JPRB, 0.74516E+00_JPRB, 0.74499E+00_JPRB, 0.74506E+00_JPRB, 0.74499E+00_JPRB, 0.74539E+00_JPRB ,&
&  0.74567E+00_JPRB, 0.74630E+00_JPRB, 0.74696E+00_JPRB, 0.74790E+00_JPRB, 0.74860E+00_JPRB, 0.74939E+00_JPRB, 0.75008E+00_JPRB ,&
&  0.75087E+00_JPRB, 0.75167E+00_JPRB, 0.75250E+00_JPRB, 0.75301E+00_JPRB, 0.75348E+00_JPRB, 0.75406E+00_JPRB, 0.75468E+00_JPRB ,&
&  0.75551E+00_JPRB, 0.75598E+00_JPRB, 0.75653E+00_JPRB, 0.75702E+00_JPRB, 0.75720E+00_JPRB, 0.75735E+00_JPRB, 0.75735E+00_JPRB ,&
&  0.75724E+00_JPRB, 0.75706E+00_JPRB, 0.75682E+00_JPRB, 0.75663E+00_JPRB, 0.75643E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.76615E+00_JPRB, 0.76598E+00_JPRB, 0.76554E+00_JPRB, 0.76474E+00_JPRB, 0.76371E+00_JPRB, 0.76237E+00_JPRB, 0.76106E+00_JPRB ,&
&  0.75951E+00_JPRB, 0.75804E+00_JPRB, 0.75610E+00_JPRB, 0.75414E+00_JPRB, 0.75208E+00_JPRB, 0.75017E+00_JPRB, 0.74811E+00_JPRB ,&
&  0.74617E+00_JPRB, 0.74422E+00_JPRB, 0.74287E+00_JPRB, 0.74139E+00_JPRB, 0.73982E+00_JPRB, 0.73789E+00_JPRB, 0.73634E+00_JPRB ,&
&  0.73466E+00_JPRB, 0.73268E+00_JPRB, 0.73052E+00_JPRB, 0.72666E+00_JPRB, 0.72341E+00_JPRB, 0.72042E+00_JPRB, 0.71785E+00_JPRB ,&
&  0.71542E+00_JPRB, 0.71314E+00_JPRB, 0.71144E+00_JPRB, 0.71033E+00_JPRB, 0.71162E+00_JPRB, 0.71196E+00_JPRB, 0.71166E+00_JPRB ,&
&  0.71118E+00_JPRB, 0.70871E+00_JPRB, 0.70794E+00_JPRB, 0.70769E+00_JPRB, 0.70774E+00_JPRB, 0.70947E+00_JPRB, 0.71093E+00_JPRB ,&
&  0.71248E+00_JPRB, 0.71377E+00_JPRB, 0.71627E+00_JPRB, 0.71868E+00_JPRB, 0.72093E+00_JPRB, 0.72333E+00_JPRB, 0.72474E+00_JPRB ,&
&  0.72635E+00_JPRB, 0.72868E+00_JPRB, 0.73087E+00_JPRB, 0.73318E+00_JPRB, 0.73454E+00_JPRB, 0.73578E+00_JPRB, 0.73667E+00_JPRB ,&
&  0.73735E+00_JPRB, 0.73837E+00_JPRB, 0.73947E+00_JPRB, 0.74048E+00_JPRB, 0.74109E+00_JPRB, 0.74171E+00_JPRB, 0.74212E+00_JPRB ,&
&  0.74223E+00_JPRB, 0.74162E+00_JPRB, 0.74141E+00_JPRB, 0.74109E+00_JPRB, 0.74052E+00_JPRB, 0.73974E+00_JPRB, 0.73950E+00_JPRB ,&
&  0.73922E+00_JPRB, 0.73871E+00_JPRB, 0.73787E+00_JPRB, 0.73816E+00_JPRB, 0.73865E+00_JPRB, 0.73885E+00_JPRB, 0.73874E+00_JPRB ,&
&  0.74089E+00_JPRB, 0.74395E+00_JPRB, 0.74759E+00_JPRB, 0.75253E+00_JPRB, 0.75619E+00_JPRB, 0.75852E+00_JPRB, 0.76014E+00_JPRB ,&
&  0.76068E+00_JPRB, 0.76125E+00_JPRB, 0.76173E+00_JPRB, 0.76257E+00_JPRB, 0.76308E+00_JPRB, 0.76412E+00_JPRB, 0.76576E+00_JPRB ,&
&  0.76694E+00_JPRB, 0.76735E+00_JPRB, 0.76726E+00_JPRB, 0.76689E+00_JPRB, 0.76646E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.77025E+00_JPRB, 0.76922E+00_JPRB, 0.76822E+00_JPRB, 0.76635E+00_JPRB, 0.76544E+00_JPRB, 0.76373E+00_JPRB, 0.76228E+00_JPRB ,&
&  0.76038E+00_JPRB, 0.75814E+00_JPRB, 0.75638E+00_JPRB, 0.75489E+00_JPRB, 0.75329E+00_JPRB, 0.75172E+00_JPRB, 0.74999E+00_JPRB ,&
&  0.74818E+00_JPRB, 0.74610E+00_JPRB, 0.74528E+00_JPRB, 0.74389E+00_JPRB, 0.74231E+00_JPRB, 0.74051E+00_JPRB, 0.73599E+00_JPRB ,&
&  0.73172E+00_JPRB, 0.72663E+00_JPRB, 0.72143E+00_JPRB, 0.71702E+00_JPRB, 0.71436E+00_JPRB, 0.69457E+00_JPRB, 0.69138E+00_JPRB ,&
&  0.69997E+00_JPRB, 0.70494E+00_JPRB, 0.71031E+00_JPRB, 0.71569E+00_JPRB, 0.70323E+00_JPRB, 0.68914E+00_JPRB, 0.68078E+00_JPRB ,&
&  0.66176E+00_JPRB, 0.66954E+00_JPRB, 0.67357E+00_JPRB, 0.67739E+00_JPRB, 0.68292E+00_JPRB, 0.68291E+00_JPRB, 0.68617E+00_JPRB ,&
&  0.69831E+00_JPRB, 0.69838E+00_JPRB, 0.70000E+00_JPRB, 0.70183E+00_JPRB, 0.70362E+00_JPRB, 0.70545E+00_JPRB, 0.70906E+00_JPRB ,&
&  0.71204E+00_JPRB, 0.71549E+00_JPRB, 0.71932E+00_JPRB, 0.72113E+00_JPRB, 0.72290E+00_JPRB, 0.72526E+00_JPRB, 0.72740E+00_JPRB ,&
&  0.72834E+00_JPRB, 0.72968E+00_JPRB, 0.73122E+00_JPRB, 0.73277E+00_JPRB, 0.73472E+00_JPRB, 0.73612E+00_JPRB, 0.73728E+00_JPRB ,&
&  0.73820E+00_JPRB, 0.73800E+00_JPRB, 0.73798E+00_JPRB, 0.73825E+00_JPRB, 0.73821E+00_JPRB, 0.73789E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.73879E+00_JPRB, 0.73921E+00_JPRB, 0.74008E+00_JPRB, 0.74140E+00_JPRB, 0.74508E+00_JPRB, 0.75014E+00_JPRB, 0.76024E+00_JPRB ,&
&  0.76835E+00_JPRB, 0.77536E+00_JPRB, 0.77787E+00_JPRB, 0.78042E+00_JPRB, 0.78251E+00_JPRB, 0.78305E+00_JPRB, 0.78297E+00_JPRB ,&
&  0.78170E+00_JPRB, 0.77941E+00_JPRB, 0.77703E+00_JPRB, 0.77471E+00_JPRB, 0.77328E+00_JPRB, 0.77217E+00_JPRB, 0.77205E+00_JPRB ,&
&  0.77270E+00_JPRB, 0.77409E+00_JPRB, 0.77452E+00_JPRB, 0.77307E+00_JPRB, 0.77133E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.78175E+00_JPRB, 0.77885E+00_JPRB, 0.77800E+00_JPRB, 0.77542E+00_JPRB, 0.77318E+00_JPRB, 0.76979E+00_JPRB, 0.76694E+00_JPRB ,&
&  0.76350E+00_JPRB, 0.76048E+00_JPRB, 0.75775E+00_JPRB, 0.75524E+00_JPRB, 0.75397E+00_JPRB, 0.75254E+00_JPRB, 0.75161E+00_JPRB ,&
&  0.75043E+00_JPRB, 0.74872E+00_JPRB, 0.74786E+00_JPRB, 0.74655E+00_JPRB, 0.74536E+00_JPRB, 0.74397E+00_JPRB, 0.74024E+00_JPRB ,&
&  0.73392E+00_JPRB, 0.72555E+00_JPRB, 0.70729E+00_JPRB, 0.71107E+00_JPRB, 0.70938E+00_JPRB, 0.70737E+00_JPRB, 0.70866E+00_JPRB ,&
&  0.69957E+00_JPRB, 0.69702E+00_JPRB, 0.68587E+00_JPRB, 0.67569E+00_JPRB, 0.67166E+00_JPRB, 0.66172E+00_JPRB, 0.66163E+00_JPRB ,&
&  0.65837E+00_JPRB, 0.65675E+00_JPRB, 0.65796E+00_JPRB, 0.65714E+00_JPRB, 0.66247E+00_JPRB, 0.66474E+00_JPRB, 0.66859E+00_JPRB ,&
&  0.67590E+00_JPRB, 0.68174E+00_JPRB, 0.68513E+00_JPRB, 0.67840E+00_JPRB, 0.68152E+00_JPRB, 0.68469E+00_JPRB, 0.68879E+00_JPRB ,&
&  0.69277E+00_JPRB, 0.69563E+00_JPRB, 0.69875E+00_JPRB, 0.70101E+00_JPRB, 0.70343E+00_JPRB, 0.70722E+00_JPRB, 0.71105E+00_JPRB ,&
&  0.71568E+00_JPRB, 0.71903E+00_JPRB, 0.72120E+00_JPRB, 0.72353E+00_JPRB, 0.72586E+00_JPRB, 0.72817E+00_JPRB, 0.73104E+00_JPRB ,&
&  0.73453E+00_JPRB, 0.73876E+00_JPRB, 0.74324E+00_JPRB, 0.74787E+00_JPRB, 0.75217E+00_JPRB, 0.75767E+00_JPRB, 0.76400E+00_JPRB ,&
&  0.77322E+00_JPRB, 0.78154E+00_JPRB, 0.79458E+00_JPRB, 0.80227E+00_JPRB, 0.81900E+00_JPRB, 0.83469E+00_JPRB, 0.84361E+00_JPRB ,&
&  0.84130E+00_JPRB, 0.83617E+00_JPRB, 0.83142E+00_JPRB, 0.82642E+00_JPRB, 0.82213E+00_JPRB, 0.81661E+00_JPRB, 0.81271E+00_JPRB ,&
&  0.80877E+00_JPRB, 0.80492E+00_JPRB, 0.80120E+00_JPRB, 0.79743E+00_JPRB, 0.79428E+00_JPRB, 0.79067E+00_JPRB, 0.78844E+00_JPRB ,&
&  0.78759E+00_JPRB, 0.78832E+00_JPRB, 0.78846E+00_JPRB, 0.78748E+00_JPRB, 0.78485E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.79607E+00_JPRB, 0.79573E+00_JPRB, 0.79763E+00_JPRB, 0.79654E+00_JPRB, 0.79374E+00_JPRB, 0.79052E+00_JPRB, 0.78364E+00_JPRB ,&
&  0.77651E+00_JPRB, 0.76936E+00_JPRB, 0.76285E+00_JPRB, 0.75794E+00_JPRB, 0.75381E+00_JPRB, 0.75069E+00_JPRB, 0.74944E+00_JPRB ,&
&  0.74766E+00_JPRB, 0.74744E+00_JPRB, 0.74739E+00_JPRB, 0.74761E+00_JPRB, 0.74886E+00_JPRB, 0.74950E+00_JPRB, 0.74986E+00_JPRB ,&
&  0.74637E+00_JPRB, 0.73651E+00_JPRB, 0.72285E+00_JPRB, 0.72618E+00_JPRB, 0.72845E+00_JPRB, 0.73187E+00_JPRB, 0.73603E+00_JPRB ,&
&  0.73348E+00_JPRB, 0.70504E+00_JPRB, 0.71362E+00_JPRB, 0.70049E+00_JPRB, 0.69368E+00_JPRB, 0.68593E+00_JPRB, 0.66523E+00_JPRB ,&
&  0.65812E+00_JPRB, 0.64589E+00_JPRB, 0.63897E+00_JPRB, 0.63895E+00_JPRB, 0.64171E+00_JPRB, 0.64891E+00_JPRB, 0.65801E+00_JPRB ,&
&  0.65865E+00_JPRB, 0.67144E+00_JPRB, 0.67271E+00_JPRB, 0.67189E+00_JPRB, 0.67503E+00_JPRB, 0.67884E+00_JPRB, 0.67654E+00_JPRB ,&
&  0.67751E+00_JPRB, 0.67661E+00_JPRB, 0.67608E+00_JPRB, 0.68033E+00_JPRB, 0.68810E+00_JPRB, 0.69208E+00_JPRB, 0.69598E+00_JPRB ,&
&  0.70160E+00_JPRB, 0.70494E+00_JPRB, 0.70699E+00_JPRB, 0.70978E+00_JPRB, 0.71355E+00_JPRB, 0.71753E+00_JPRB, 0.72346E+00_JPRB ,&
&  0.73035E+00_JPRB, 0.74214E+00_JPRB, 0.75316E+00_JPRB, 0.76957E+00_JPRB, 0.78465E+00_JPRB, 0.80462E+00_JPRB, 0.82197E+00_JPRB ,&
&  0.84019E+00_JPRB, 0.85518E+00_JPRB, 0.87221E+00_JPRB, 0.88768E+00_JPRB, 0.89993E+00_JPRB, 0.89521E+00_JPRB, 0.87986E+00_JPRB ,&
&  0.86832E+00_JPRB, 0.85191E+00_JPRB, 0.84216E+00_JPRB, 0.83454E+00_JPRB, 0.82906E+00_JPRB, 0.82657E+00_JPRB, 0.82383E+00_JPRB ,&
&  0.82246E+00_JPRB, 0.81986E+00_JPRB, 0.81669E+00_JPRB, 0.81236E+00_JPRB, 0.80899E+00_JPRB, 0.80537E+00_JPRB, 0.80334E+00_JPRB ,&
&  0.80024E+00_JPRB, 0.79953E+00_JPRB, 0.79821E+00_JPRB, 0.79712E+00_JPRB, 0.79637E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.81132E+00_JPRB, 0.81539E+00_JPRB, 0.81382E+00_JPRB, 0.81157E+00_JPRB, 0.80383E+00_JPRB, 0.79707E+00_JPRB, 0.78666E+00_JPRB ,&
&  0.77618E+00_JPRB, 0.76570E+00_JPRB, 0.75612E+00_JPRB, 0.74938E+00_JPRB, 0.74399E+00_JPRB, 0.74030E+00_JPRB, 0.73892E+00_JPRB ,&
&  0.73861E+00_JPRB, 0.73980E+00_JPRB, 0.74093E+00_JPRB, 0.74244E+00_JPRB, 0.74585E+00_JPRB, 0.74974E+00_JPRB, 0.75386E+00_JPRB ,&
&  0.75527E+00_JPRB, 0.75151E+00_JPRB, 0.73938E+00_JPRB, 0.73939E+00_JPRB, 0.74686E+00_JPRB, 0.74288E+00_JPRB, 0.75720E+00_JPRB ,&
&  0.76786E+00_JPRB, 0.77314E+00_JPRB, 0.76302E+00_JPRB, 0.75595E+00_JPRB, 0.74207E+00_JPRB, 0.68222E+00_JPRB, 0.67805E+00_JPRB ,&
&  0.66821E+00_JPRB, 0.65257E+00_JPRB, 0.64572E+00_JPRB, 0.63976E+00_JPRB, 0.63838E+00_JPRB, 0.64733E+00_JPRB, 0.66038E+00_JPRB ,&
&  0.66261E+00_JPRB, 0.68206E+00_JPRB, 0.68138E+00_JPRB, 0.67983E+00_JPRB, 0.68085E+00_JPRB, 0.68459E+00_JPRB, 0.68153E+00_JPRB ,&
&  0.68223E+00_JPRB, 0.68233E+00_JPRB, 0.68263E+00_JPRB, 0.68816E+00_JPRB, 0.69390E+00_JPRB, 0.69841E+00_JPRB, 0.70495E+00_JPRB ,&
&  0.71865E+00_JPRB, 0.72808E+00_JPRB, 0.73357E+00_JPRB, 0.73852E+00_JPRB, 0.74621E+00_JPRB, 0.75536E+00_JPRB, 0.76779E+00_JPRB ,&
&  0.77689E+00_JPRB, 0.78824E+00_JPRB, 0.80122E+00_JPRB, 0.81455E+00_JPRB, 0.82782E+00_JPRB, 0.84265E+00_JPRB, 0.85850E+00_JPRB ,&
&  0.86667E+00_JPRB, 0.87813E+00_JPRB, 0.89055E+00_JPRB, 0.91178E+00_JPRB, 0.91128E+00_JPRB, 0.89572E+00_JPRB, 0.88367E+00_JPRB ,&
&  0.87452E+00_JPRB, 0.87462E+00_JPRB, 0.86570E+00_JPRB, 0.86042E+00_JPRB, 0.85428E+00_JPRB, 0.85211E+00_JPRB, 0.84773E+00_JPRB ,&
&  0.84454E+00_JPRB, 0.84004E+00_JPRB, 0.83388E+00_JPRB, 0.82666E+00_JPRB, 0.82150E+00_JPRB, 0.81619E+00_JPRB, 0.81284E+00_JPRB ,&
&  0.80881E+00_JPRB, 0.80741E+00_JPRB, 0.80675E+00_JPRB, 0.80731E+00_JPRB, 0.80720E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.82342E+00_JPRB, 0.82454E+00_JPRB, 0.81594E+00_JPRB, 0.81188E+00_JPRB, 0.79969E+00_JPRB, 0.78736E+00_JPRB, 0.77185E+00_JPRB ,&
&  0.75936E+00_JPRB, 0.74983E+00_JPRB, 0.74083E+00_JPRB, 0.73629E+00_JPRB, 0.73427E+00_JPRB, 0.73356E+00_JPRB, 0.73556E+00_JPRB ,&
&  0.73820E+00_JPRB, 0.73651E+00_JPRB, 0.74000E+00_JPRB, 0.74591E+00_JPRB, 0.75040E+00_JPRB, 0.75443E+00_JPRB, 0.75860E+00_JPRB ,&
&  0.75839E+00_JPRB, 0.75791E+00_JPRB, 0.75682E+00_JPRB, 0.75075E+00_JPRB, 0.74900E+00_JPRB, 0.76076E+00_JPRB, 0.77727E+00_JPRB ,&
&  0.77129E+00_JPRB, 0.77327E+00_JPRB, 0.78294E+00_JPRB, 0.78626E+00_JPRB, 0.76644E+00_JPRB, 0.73360E+00_JPRB, 0.70146E+00_JPRB ,&
&  0.67962E+00_JPRB, 0.65196E+00_JPRB, 0.63979E+00_JPRB, 0.63349E+00_JPRB, 0.63844E+00_JPRB, 0.64841E+00_JPRB, 0.66725E+00_JPRB ,&
&  0.67819E+00_JPRB, 0.68218E+00_JPRB, 0.68858E+00_JPRB, 0.69444E+00_JPRB, 0.69497E+00_JPRB, 0.69836E+00_JPRB, 0.69869E+00_JPRB ,&
&  0.70381E+00_JPRB, 0.70729E+00_JPRB, 0.71115E+00_JPRB, 0.71965E+00_JPRB, 0.73255E+00_JPRB, 0.74628E+00_JPRB, 0.76775E+00_JPRB ,&
&  0.81775E+00_JPRB, 0.83608E+00_JPRB, 0.84993E+00_JPRB, 0.85712E+00_JPRB, 0.86681E+00_JPRB, 0.87026E+00_JPRB, 0.86219E+00_JPRB ,&
&  0.86266E+00_JPRB, 0.87569E+00_JPRB, 0.87814E+00_JPRB, 0.88163E+00_JPRB, 0.88647E+00_JPRB, 0.89141E+00_JPRB, 0.89756E+00_JPRB ,&
&  0.89042E+00_JPRB, 0.90352E+00_JPRB, 0.97243E+00_JPRB, 0.99187E+00_JPRB, 0.97597E+00_JPRB, 0.95546E+00_JPRB, 0.93493E+00_JPRB ,&
&  0.91861E+00_JPRB, 0.90408E+00_JPRB, 0.89157E+00_JPRB, 0.88149E+00_JPRB, 0.87439E+00_JPRB, 0.87048E+00_JPRB, 0.86989E+00_JPRB ,&
&  0.86272E+00_JPRB, 0.85441E+00_JPRB, 0.84510E+00_JPRB, 0.83850E+00_JPRB, 0.83312E+00_JPRB, 0.82751E+00_JPRB, 0.82480E+00_JPRB ,&
&  0.82227E+00_JPRB, 0.82361E+00_JPRB, 0.82473E+00_JPRB, 0.82597E+00_JPRB, 0.82551E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.82433E+00_JPRB, 0.82617E+00_JPRB, 0.82383E+00_JPRB, 0.81183E+00_JPRB, 0.78795E+00_JPRB, 0.76544E+00_JPRB, 0.74887E+00_JPRB ,&
&  0.74099E+00_JPRB, 0.73650E+00_JPRB, 0.73165E+00_JPRB, 0.72941E+00_JPRB, 0.72920E+00_JPRB, 0.73077E+00_JPRB, 0.73406E+00_JPRB ,&
&  0.73912E+00_JPRB, 0.74397E+00_JPRB, 0.74660E+00_JPRB, 0.74809E+00_JPRB, 0.75226E+00_JPRB, 0.75941E+00_JPRB, 0.76729E+00_JPRB ,&
&  0.77187E+00_JPRB, 0.77219E+00_JPRB, 0.77527E+00_JPRB, 0.77482E+00_JPRB, 0.77479E+00_JPRB, 0.77391E+00_JPRB, 0.77515E+00_JPRB ,&
&  0.78575E+00_JPRB, 0.79548E+00_JPRB, 0.78912E+00_JPRB, 0.79378E+00_JPRB, 0.80193E+00_JPRB, 0.77215E+00_JPRB, 0.72842E+00_JPRB ,&
&  0.68412E+00_JPRB, 0.65174E+00_JPRB, 0.63784E+00_JPRB, 0.63435E+00_JPRB, 0.64377E+00_JPRB, 0.66520E+00_JPRB, 0.69078E+00_JPRB ,&
&  0.69187E+00_JPRB, 0.69838E+00_JPRB, 0.70966E+00_JPRB, 0.71583E+00_JPRB, 0.71644E+00_JPRB, 0.71562E+00_JPRB, 0.71634E+00_JPRB ,&
&  0.72116E+00_JPRB, 0.73422E+00_JPRB, 0.75671E+00_JPRB, 0.78831E+00_JPRB, 0.83432E+00_JPRB, 0.85760E+00_JPRB, 0.90345E+00_JPRB ,&
&  0.97394E+00_JPRB, 0.97660E+00_JPRB, 0.98681E+00_JPRB, 0.98647E+00_JPRB, 0.98449E+00_JPRB, 0.97803E+00_JPRB, 0.97640E+00_JPRB ,&
&  0.97620E+00_JPRB, 0.97237E+00_JPRB, 0.96171E+00_JPRB, 0.94996E+00_JPRB, 0.94901E+00_JPRB, 0.95659E+00_JPRB, 0.97571E+00_JPRB ,&
&  0.10067E+01_JPRB, 0.10568E+01_JPRB, 0.11879E+01_JPRB, 0.10980E+01_JPRB, 0.10514E+01_JPRB, 0.10308E+01_JPRB, 0.10019E+01_JPRB ,&
&  0.97737E+00_JPRB, 0.96078E+00_JPRB, 0.95009E+00_JPRB, 0.93736E+00_JPRB, 0.91404E+00_JPRB, 0.89744E+00_JPRB, 0.88508E+00_JPRB ,&
&  0.87437E+00_JPRB, 0.86384E+00_JPRB, 0.85588E+00_JPRB, 0.85036E+00_JPRB, 0.84486E+00_JPRB, 0.84151E+00_JPRB, 0.84142E+00_JPRB ,&
&  0.83943E+00_JPRB, 0.83804E+00_JPRB, 0.83617E+00_JPRB, 0.83140E+00_JPRB, 0.82739E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.82938E+00_JPRB, 0.83049E+00_JPRB, 0.81868E+00_JPRB, 0.80546E+00_JPRB, 0.77764E+00_JPRB, 0.75694E+00_JPRB, 0.74286E+00_JPRB ,&
&  0.73622E+00_JPRB, 0.73003E+00_JPRB, 0.72602E+00_JPRB, 0.72449E+00_JPRB, 0.72565E+00_JPRB, 0.72968E+00_JPRB, 0.73293E+00_JPRB ,&
&  0.73599E+00_JPRB, 0.74152E+00_JPRB, 0.74847E+00_JPRB, 0.75389E+00_JPRB, 0.75856E+00_JPRB, 0.76421E+00_JPRB, 0.77168E+00_JPRB ,&
&  0.78018E+00_JPRB, 0.78517E+00_JPRB, 0.78872E+00_JPRB, 0.79265E+00_JPRB, 0.80110E+00_JPRB, 0.81260E+00_JPRB, 0.81770E+00_JPRB ,&
&  0.80975E+00_JPRB, 0.80622E+00_JPRB, 0.81534E+00_JPRB, 0.81914E+00_JPRB, 0.82546E+00_JPRB, 0.80391E+00_JPRB, 0.75073E+00_JPRB ,&
&  0.69953E+00_JPRB, 0.66637E+00_JPRB, 0.66125E+00_JPRB, 0.67345E+00_JPRB, 0.69649E+00_JPRB, 0.70469E+00_JPRB, 0.71741E+00_JPRB ,&
&  0.72754E+00_JPRB, 0.73057E+00_JPRB, 0.73292E+00_JPRB, 0.73927E+00_JPRB, 0.74239E+00_JPRB, 0.74193E+00_JPRB, 0.74514E+00_JPRB ,&
&  0.75721E+00_JPRB, 0.77801E+00_JPRB, 0.79087E+00_JPRB, 0.79396E+00_JPRB, 0.83090E+00_JPRB, 0.92532E+00_JPRB, 0.10098E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10442E+01_JPRB, 0.10573E+01_JPRB, 0.10914E+01_JPRB, 0.11056E+01_JPRB, 0.11173E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11014E+01_JPRB, 0.10758E+01_JPRB, 0.10537E+01_JPRB, 0.10501E+01_JPRB, 0.10732E+01_JPRB, 0.11186E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.11847E+01_JPRB, 0.11549E+01_JPRB, 0.10677E+01_JPRB, 0.99015E+00_JPRB, 0.96644E+00_JPRB, 0.97270E+00_JPRB, 0.98482E+00_JPRB ,&
&  0.98050E+00_JPRB, 0.97302E+00_JPRB, 0.96467E+00_JPRB, 0.95606E+00_JPRB, 0.91790E+00_JPRB, 0.89389E+00_JPRB, 0.87474E+00_JPRB ,&
&  0.86130E+00_JPRB, 0.85155E+00_JPRB, 0.85287E+00_JPRB, 0.84863E+00_JPRB, 0.84615E+00_JPRB, 0.84306E+00_JPRB, 0.83763E+00_JPRB ,&
&  0.83568E+00_JPRB, 0.83360E+00_JPRB, 0.82838E+00_JPRB, 0.82505E+00_JPRB, 0.82416E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.84504E+00_JPRB, 0.83667E+00_JPRB, 0.82838E+00_JPRB, 0.81031E+00_JPRB, 0.79677E+00_JPRB, 0.78865E+00_JPRB, 0.78156E+00_JPRB ,&
&  0.78378E+00_JPRB, 0.76260E+00_JPRB, 0.74095E+00_JPRB, 0.72695E+00_JPRB, 0.71998E+00_JPRB, 0.71953E+00_JPRB, 0.72632E+00_JPRB ,&
&  0.73558E+00_JPRB, 0.74378E+00_JPRB, 0.75174E+00_JPRB, 0.76117E+00_JPRB, 0.77037E+00_JPRB, 0.77713E+00_JPRB, 0.78394E+00_JPRB ,&
&  0.79282E+00_JPRB, 0.80201E+00_JPRB, 0.81131E+00_JPRB, 0.82505E+00_JPRB, 0.83691E+00_JPRB, 0.84867E+00_JPRB, 0.85093E+00_JPRB ,&
&  0.84546E+00_JPRB, 0.83842E+00_JPRB, 0.83804E+00_JPRB, 0.84254E+00_JPRB, 0.84466E+00_JPRB, 0.82569E+00_JPRB, 0.76926E+00_JPRB ,&
&  0.72122E+00_JPRB, 0.70309E+00_JPRB, 0.71920E+00_JPRB, 0.73843E+00_JPRB, 0.74909E+00_JPRB, 0.75823E+00_JPRB, 0.75880E+00_JPRB ,&
&  0.75566E+00_JPRB, 0.75471E+00_JPRB, 0.76297E+00_JPRB, 0.77162E+00_JPRB, 0.77353E+00_JPRB, 0.77573E+00_JPRB, 0.78269E+00_JPRB ,&
&  0.80273E+00_JPRB, 0.82617E+00_JPRB, 0.84121E+00_JPRB, 0.83787E+00_JPRB, 0.92134E+00_JPRB, 0.11856E+01_JPRB, 0.12169E+01_JPRB ,&
&  0.12381E+01_JPRB, 0.12514E+01_JPRB, 0.12798E+01_JPRB, 0.13030E+01_JPRB, 0.12828E+01_JPRB, 0.12726E+01_JPRB, 0.12689E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.12216E+01_JPRB, 0.12206E+01_JPRB, 0.12625E+01_JPRB, 0.12896E+01_JPRB, 0.13020E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.12954E+01_JPRB, 0.12288E+01_JPRB, 0.11491E+01_JPRB, 0.10764E+01_JPRB, 0.10402E+01_JPRB, 0.10176E+01_JPRB, 0.10046E+01_JPRB ,&
&  0.98989E+00_JPRB, 0.98499E+00_JPRB, 0.98549E+00_JPRB, 0.98286E+00_JPRB, 0.94982E+00_JPRB, 0.90768E+00_JPRB, 0.88398E+00_JPRB ,&
&  0.86879E+00_JPRB, 0.85488E+00_JPRB, 0.84291E+00_JPRB, 0.83664E+00_JPRB, 0.84276E+00_JPRB, 0.84794E+00_JPRB, 0.84449E+00_JPRB ,&
&  0.84835E+00_JPRB, 0.84323E+00_JPRB, 0.84091E+00_JPRB, 0.84091E+00_JPRB, 0.84731E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.85159E+00_JPRB, 0.84650E+00_JPRB, 0.83783E+00_JPRB, 0.82753E+00_JPRB, 0.82361E+00_JPRB, 0.81866E+00_JPRB, 0.80395E+00_JPRB ,&
&  0.81028E+00_JPRB, 0.77138E+00_JPRB, 0.74870E+00_JPRB, 0.73984E+00_JPRB, 0.73922E+00_JPRB, 0.73405E+00_JPRB, 0.72793E+00_JPRB ,&
&  0.73146E+00_JPRB, 0.74421E+00_JPRB, 0.75842E+00_JPRB, 0.76955E+00_JPRB, 0.78079E+00_JPRB, 0.78919E+00_JPRB, 0.79674E+00_JPRB ,&
&  0.80658E+00_JPRB, 0.81908E+00_JPRB, 0.83562E+00_JPRB, 0.85040E+00_JPRB, 0.86190E+00_JPRB, 0.87496E+00_JPRB, 0.87822E+00_JPRB ,&
&  0.87824E+00_JPRB, 0.86992E+00_JPRB, 0.86043E+00_JPRB, 0.85631E+00_JPRB, 0.85169E+00_JPRB, 0.84946E+00_JPRB, 0.82355E+00_JPRB ,&
&  0.77057E+00_JPRB, 0.76772E+00_JPRB, 0.78430E+00_JPRB, 0.79478E+00_JPRB, 0.79427E+00_JPRB, 0.78875E+00_JPRB, 0.78713E+00_JPRB ,&
&  0.78836E+00_JPRB, 0.78958E+00_JPRB, 0.79479E+00_JPRB, 0.79617E+00_JPRB, 0.79999E+00_JPRB, 0.80500E+00_JPRB, 0.82522E+00_JPRB ,&
&  0.86716E+00_JPRB, 0.84877E+00_JPRB, 0.86239E+00_JPRB, 0.92147E+00_JPRB, 0.11373E+01_JPRB, 0.12586E+01_JPRB, 0.13474E+01_JPRB ,&
&  0.14811E+01_JPRB, 0.15096E+01_JPRB, 0.15039E+01_JPRB, 0.15028E+01_JPRB, 0.15062E+01_JPRB, 0.15091E+01_JPRB, 0.15167E+01_JPRB ,&
&  0.14792E+01_JPRB, 0.14796E+01_JPRB, 0.14908E+01_JPRB, 0.15210E+01_JPRB, 0.15159E+01_JPRB, 0.14986E+01_JPRB, 0.14683E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.13625E+01_JPRB, 0.12779E+01_JPRB, 0.11956E+01_JPRB, 0.11269E+01_JPRB, 0.10932E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.10513E+01_JPRB, 0.10411E+01_JPRB, 0.10302E+01_JPRB, 0.10173E+01_JPRB, 0.10008E+01_JPRB, 0.98348E+00_JPRB, 0.97878E+00_JPRB ,&
&  0.94032E+00_JPRB, 0.91225E+00_JPRB, 0.88379E+00_JPRB, 0.87074E+00_JPRB, 0.87318E+00_JPRB, 0.87973E+00_JPRB, 0.88395E+00_JPRB ,&
&  0.88963E+00_JPRB, 0.87747E+00_JPRB, 0.87454E+00_JPRB, 0.86946E+00_JPRB, 0.86453E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.85975E+00_JPRB, 0.86082E+00_JPRB, 0.85491E+00_JPRB, 0.85022E+00_JPRB, 0.85355E+00_JPRB, 0.85156E+00_JPRB, 0.84700E+00_JPRB ,&
&  0.88312E+00_JPRB, 0.85900E+00_JPRB, 0.86431E+00_JPRB, 0.79840E+00_JPRB, 0.77123E+00_JPRB, 0.75701E+00_JPRB, 0.73992E+00_JPRB ,&
&  0.73823E+00_JPRB, 0.74526E+00_JPRB, 0.75479E+00_JPRB, 0.76757E+00_JPRB, 0.78225E+00_JPRB, 0.79268E+00_JPRB, 0.80174E+00_JPRB ,&
&  0.81261E+00_JPRB, 0.82668E+00_JPRB, 0.84180E+00_JPRB, 0.85451E+00_JPRB, 0.86668E+00_JPRB, 0.88177E+00_JPRB, 0.88439E+00_JPRB ,&
&  0.86855E+00_JPRB, 0.86080E+00_JPRB, 0.85930E+00_JPRB, 0.85438E+00_JPRB, 0.85812E+00_JPRB, 0.85833E+00_JPRB, 0.86668E+00_JPRB ,&
&  0.86063E+00_JPRB, 0.84226E+00_JPRB, 0.84809E+00_JPRB, 0.84008E+00_JPRB, 0.83253E+00_JPRB, 0.82393E+00_JPRB, 0.81589E+00_JPRB ,&
&  0.81148E+00_JPRB, 0.81246E+00_JPRB, 0.81919E+00_JPRB, 0.82488E+00_JPRB, 0.83124E+00_JPRB, 0.84865E+00_JPRB, 0.88862E+00_JPRB ,&
&  0.94944E+00_JPRB, 0.90815E+00_JPRB, 0.10078E+01_JPRB, 0.10633E+01_JPRB, 0.12867E+01_JPRB, 0.15222E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.18343E+01_JPRB, 0.18478E+01_JPRB, 0.18157E+01_JPRB, 0.17882E+01_JPRB, 0.17942E+01_JPRB, 0.17564E+01_JPRB, 0.16929E+01_JPRB ,&
&  0.16674E+01_JPRB, 0.16236E+01_JPRB, 0.17005E+01_JPRB, 0.16914E+01_JPRB, 0.16402E+01_JPRB, 0.16038E+01_JPRB, 0.15407E+01_JPRB ,&
&  0.14901E+01_JPRB, 0.14385E+01_JPRB, 0.13887E+01_JPRB, 0.12832E+01_JPRB, 0.12453E+01_JPRB, 0.12075E+01_JPRB, 0.11342E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.10637E+01_JPRB, 0.10260E+01_JPRB, 0.10370E+01_JPRB, 0.10107E+01_JPRB, 0.99797E+00_JPRB, 0.99376E+00_JPRB ,&
&  0.96899E+00_JPRB, 0.94162E+00_JPRB, 0.94485E+00_JPRB, 0.94269E+00_JPRB, 0.93056E+00_JPRB, 0.92165E+00_JPRB, 0.92455E+00_JPRB ,&
&  0.90980E+00_JPRB, 0.90276E+00_JPRB, 0.90855E+00_JPRB, 0.88970E+00_JPRB, 0.87286E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.88989E+00_JPRB, 0.88798E+00_JPRB, 0.87857E+00_JPRB, 0.87835E+00_JPRB, 0.88686E+00_JPRB, 0.89320E+00_JPRB, 0.89984E+00_JPRB ,&
&  0.92124E+00_JPRB, 0.92812E+00_JPRB, 0.92512E+00_JPRB, 0.90749E+00_JPRB, 0.87932E+00_JPRB, 0.81944E+00_JPRB, 0.75182E+00_JPRB ,&
&  0.73824E+00_JPRB, 0.75291E+00_JPRB, 0.76940E+00_JPRB, 0.77830E+00_JPRB, 0.79117E+00_JPRB, 0.80494E+00_JPRB, 0.81698E+00_JPRB ,&
&  0.82872E+00_JPRB, 0.83816E+00_JPRB, 0.85062E+00_JPRB, 0.86396E+00_JPRB, 0.87466E+00_JPRB, 0.88901E+00_JPRB, 0.90000E+00_JPRB ,&
&  0.87335E+00_JPRB, 0.85794E+00_JPRB, 0.85060E+00_JPRB, 0.85132E+00_JPRB, 0.87878E+00_JPRB, 0.89061E+00_JPRB, 0.90117E+00_JPRB ,&
&  0.90808E+00_JPRB, 0.90725E+00_JPRB, 0.89738E+00_JPRB, 0.88834E+00_JPRB, 0.87783E+00_JPRB, 0.87032E+00_JPRB, 0.86077E+00_JPRB ,&
&  0.85464E+00_JPRB, 0.85361E+00_JPRB, 0.86094E+00_JPRB, 0.86913E+00_JPRB, 0.88253E+00_JPRB, 0.92611E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.10957E+01_JPRB, 0.11844E+01_JPRB, 0.12376E+01_JPRB, 0.12550E+01_JPRB, 0.14521E+01_JPRB, 0.16437E+01_JPRB, 0.17373E+01_JPRB ,&
&  0.18769E+01_JPRB, 0.18477E+01_JPRB, 0.19325E+01_JPRB, 0.20225E+01_JPRB, 0.20930E+01_JPRB, 0.20497E+01_JPRB, 0.19232E+01_JPRB ,&
&  0.18805E+01_JPRB, 0.19407E+01_JPRB, 0.19327E+01_JPRB, 0.19195E+01_JPRB, 0.18430E+01_JPRB, 0.17879E+01_JPRB, 0.17155E+01_JPRB ,&
&  0.16975E+01_JPRB, 0.16850E+01_JPRB, 0.16237E+01_JPRB, 0.15479E+01_JPRB, 0.14576E+01_JPRB, 0.13604E+01_JPRB, 0.12608E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.10584E+01_JPRB, 0.10178E+01_JPRB, 0.10167E+01_JPRB, 0.10271E+01_JPRB, 0.10041E+01_JPRB, 0.10003E+01_JPRB ,&
&  0.99726E+00_JPRB, 0.10001E+01_JPRB, 0.10200E+01_JPRB, 0.10023E+01_JPRB, 0.97549E+00_JPRB, 0.94850E+00_JPRB, 0.93328E+00_JPRB ,&
&  0.90954E+00_JPRB, 0.91814E+00_JPRB, 0.92180E+00_JPRB, 0.91088E+00_JPRB, 0.89565E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.91665E+00_JPRB, 0.90223E+00_JPRB, 0.89403E+00_JPRB, 0.90103E+00_JPRB, 0.91379E+00_JPRB, 0.92962E+00_JPRB, 0.95242E+00_JPRB ,&
&  0.97540E+00_JPRB, 0.98431E+00_JPRB, 0.96776E+00_JPRB, 0.94004E+00_JPRB, 0.91605E+00_JPRB, 0.89877E+00_JPRB, 0.85559E+00_JPRB ,&
&  0.79833E+00_JPRB, 0.80360E+00_JPRB, 0.79410E+00_JPRB, 0.80044E+00_JPRB, 0.82228E+00_JPRB, 0.83507E+00_JPRB, 0.84911E+00_JPRB ,&
&  0.86861E+00_JPRB, 0.87656E+00_JPRB, 0.88309E+00_JPRB, 0.88984E+00_JPRB, 0.90112E+00_JPRB, 0.93153E+00_JPRB, 0.95656E+00_JPRB ,&
&  0.92023E+00_JPRB, 0.89593E+00_JPRB, 0.89288E+00_JPRB, 0.90686E+00_JPRB, 0.93279E+00_JPRB, 0.95499E+00_JPRB, 0.96538E+00_JPRB ,&
&  0.96914E+00_JPRB, 0.96635E+00_JPRB, 0.95813E+00_JPRB, 0.94834E+00_JPRB, 0.93525E+00_JPRB, 0.92093E+00_JPRB, 0.91151E+00_JPRB ,&
&  0.90518E+00_JPRB, 0.90562E+00_JPRB, 0.91448E+00_JPRB, 0.92242E+00_JPRB, 0.93844E+00_JPRB, 0.10256E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.13840E+01_JPRB, 0.15450E+01_JPRB, 0.16039E+01_JPRB, 0.17096E+01_JPRB, 0.18185E+01_JPRB, 0.19320E+01_JPRB, 0.20130E+01_JPRB ,&
&  0.20836E+01_JPRB, 0.20060E+01_JPRB, 0.22003E+01_JPRB, 0.22888E+01_JPRB, 0.22795E+01_JPRB, 0.23022E+01_JPRB, 0.21719E+01_JPRB ,&
&  0.20777E+01_JPRB, 0.19932E+01_JPRB, 0.20524E+01_JPRB, 0.20542E+01_JPRB, 0.20112E+01_JPRB, 0.20286E+01_JPRB, 0.19685E+01_JPRB ,&
&  0.19327E+01_JPRB, 0.18764E+01_JPRB, 0.16674E+01_JPRB, 0.14998E+01_JPRB, 0.13241E+01_JPRB, 0.13670E+01_JPRB, 0.12913E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.10615E+01_JPRB, 0.10634E+01_JPRB, 0.10628E+01_JPRB, 0.10900E+01_JPRB, 0.11180E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.11198E+01_JPRB, 0.10949E+01_JPRB, 0.10721E+01_JPRB, 0.10449E+01_JPRB, 0.10043E+01_JPRB, 0.97594E+00_JPRB ,&
&  0.98170E+00_JPRB, 0.98528E+00_JPRB, 0.97048E+00_JPRB, 0.94620E+00_JPRB, 0.93001E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.96741E+00_JPRB, 0.95185E+00_JPRB, 0.94353E+00_JPRB, 0.94782E+00_JPRB, 0.95920E+00_JPRB, 0.98037E+00_JPRB, 0.10049E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10205E+01_JPRB, 0.99136E+00_JPRB, 0.96560E+00_JPRB, 0.94911E+00_JPRB, 0.93428E+00_JPRB, 0.93079E+00_JPRB ,&
&  0.85940E+00_JPRB, 0.84695E+00_JPRB, 0.80982E+00_JPRB, 0.81722E+00_JPRB, 0.85580E+00_JPRB, 0.86937E+00_JPRB, 0.88440E+00_JPRB ,&
&  0.89672E+00_JPRB, 0.91516E+00_JPRB, 0.92183E+00_JPRB, 0.93033E+00_JPRB, 0.94757E+00_JPRB, 0.99200E+00_JPRB, 0.10427E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.98895E+00_JPRB, 0.99858E+00_JPRB, 0.10215E+01_JPRB, 0.10299E+01_JPRB, 0.10562E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10521E+01_JPRB, 0.10255E+01_JPRB, 0.10034E+01_JPRB, 0.98686E+00_JPRB, 0.97321E+00_JPRB, 0.97001E+00_JPRB ,&
&  0.97148E+00_JPRB, 0.97480E+00_JPRB, 0.98074E+00_JPRB, 0.99207E+00_JPRB, 0.10044E+01_JPRB, 0.11298E+01_JPRB, 0.13763E+01_JPRB ,&
&  0.16811E+01_JPRB, 0.17593E+01_JPRB, 0.18937E+01_JPRB, 0.22783E+01_JPRB, 0.24333E+01_JPRB, 0.24839E+01_JPRB, 0.24382E+01_JPRB ,&
&  0.23787E+01_JPRB, 0.22525E+01_JPRB, 0.22287E+01_JPRB, 0.22657E+01_JPRB, 0.22396E+01_JPRB, 0.23288E+01_JPRB, 0.22591E+01_JPRB ,&
&  0.21155E+01_JPRB, 0.20706E+01_JPRB, 0.20682E+01_JPRB, 0.20472E+01_JPRB, 0.19943E+01_JPRB, 0.20125E+01_JPRB, 0.19753E+01_JPRB ,&
&  0.18994E+01_JPRB, 0.16944E+01_JPRB, 0.13762E+01_JPRB, 0.13150E+01_JPRB, 0.11334E+01_JPRB, 0.11789E+01_JPRB, 0.13554E+01_JPRB ,&
&  0.13655E+01_JPRB, 0.12946E+01_JPRB, 0.12243E+01_JPRB, 0.12018E+01_JPRB, 0.12163E+01_JPRB, 0.13303E+01_JPRB, 0.13131E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.12549E+01_JPRB, 0.12470E+01_JPRB, 0.12158E+01_JPRB, 0.11694E+01_JPRB, 0.11293E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.10896E+01_JPRB, 0.10835E+01_JPRB, 0.10660E+01_JPRB, 0.10178E+01_JPRB, 0.98294E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.10523E+01_JPRB, 0.10338E+01_JPRB, 0.10162E+01_JPRB, 0.10162E+01_JPRB, 0.10267E+01_JPRB, 0.10448E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10468E+01_JPRB, 0.10103E+01_JPRB, 0.99254E+00_JPRB, 0.97855E+00_JPRB, 0.95862E+00_JPRB, 0.95326E+00_JPRB ,&
&  0.96974E+00_JPRB, 0.93413E+00_JPRB, 0.88045E+00_JPRB, 0.83547E+00_JPRB, 0.85409E+00_JPRB, 0.86548E+00_JPRB, 0.89256E+00_JPRB ,&
&  0.91103E+00_JPRB, 0.94856E+00_JPRB, 0.96763E+00_JPRB, 0.98138E+00_JPRB, 0.10134E+01_JPRB, 0.10813E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.11594E+01_JPRB, 0.11729E+01_JPRB, 0.12025E+01_JPRB, 0.12414E+01_JPRB, 0.12531E+01_JPRB, 0.11912E+01_JPRB, 0.12039E+01_JPRB ,&
&  0.11728E+01_JPRB, 0.11324E+01_JPRB, 0.10871E+01_JPRB, 0.10646E+01_JPRB, 0.10555E+01_JPRB, 0.10573E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10774E+01_JPRB, 0.10532E+01_JPRB, 0.10482E+01_JPRB, 0.10707E+01_JPRB, 0.12244E+01_JPRB, 0.13961E+01_JPRB ,&
&  0.16725E+01_JPRB, 0.17242E+01_JPRB, 0.17797E+01_JPRB, 0.21259E+01_JPRB, 0.24242E+01_JPRB, 0.25634E+01_JPRB, 0.26130E+01_JPRB ,&
&  0.25688E+01_JPRB, 0.24829E+01_JPRB, 0.24117E+01_JPRB, 0.23043E+01_JPRB, 0.22992E+01_JPRB, 0.23406E+01_JPRB, 0.22895E+01_JPRB ,&
&  0.21473E+01_JPRB, 0.20409E+01_JPRB, 0.20083E+01_JPRB, 0.19342E+01_JPRB, 0.18716E+01_JPRB, 0.17562E+01_JPRB, 0.17877E+01_JPRB ,&
&  0.18179E+01_JPRB, 0.12550E+01_JPRB, 0.11059E+01_JPRB, 0.12356E+01_JPRB, 0.11187E+01_JPRB, 0.11540E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.12896E+01_JPRB, 0.12854E+01_JPRB, 0.13300E+01_JPRB, 0.13208E+01_JPRB, 0.13893E+01_JPRB, 0.15073E+01_JPRB, 0.15065E+01_JPRB ,&
&  0.14756E+01_JPRB, 0.14076E+01_JPRB, 0.14143E+01_JPRB, 0.13812E+01_JPRB, 0.13255E+01_JPRB, 0.12853E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.12109E+01_JPRB, 0.11828E+01_JPRB, 0.11694E+01_JPRB, 0.11202E+01_JPRB, 0.10749E+01_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.11684E+01_JPRB, 0.11430E+01_JPRB, 0.11246E+01_JPRB, 0.11187E+01_JPRB, 0.11195E+01_JPRB, 0.11285E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.10685E+01_JPRB, 0.10278E+01_JPRB, 0.10199E+01_JPRB, 0.10014E+01_JPRB, 0.97999E+00_JPRB, 0.98330E+00_JPRB ,&
&  0.10077E+01_JPRB, 0.10181E+01_JPRB, 0.91449E+00_JPRB, 0.86422E+00_JPRB, 0.82372E+00_JPRB, 0.84348E+00_JPRB, 0.86786E+00_JPRB ,&
&  0.91642E+00_JPRB, 0.94835E+00_JPRB, 0.97119E+00_JPRB, 0.10067E+01_JPRB, 0.10797E+01_JPRB, 0.11728E+01_JPRB, 0.12726E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.13858E+01_JPRB, 0.13720E+01_JPRB, 0.14535E+01_JPRB, 0.14294E+01_JPRB, 0.13346E+01_JPRB, 0.13436E+01_JPRB ,&
&  0.12883E+01_JPRB, 0.12371E+01_JPRB, 0.11888E+01_JPRB, 0.11692E+01_JPRB, 0.11577E+01_JPRB, 0.11350E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.10838E+01_JPRB, 0.10753E+01_JPRB, 0.10654E+01_JPRB, 0.11062E+01_JPRB, 0.12240E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.15282E+01_JPRB, 0.13823E+01_JPRB, 0.12672E+01_JPRB, 0.16419E+01_JPRB, 0.21120E+01_JPRB, 0.22455E+01_JPRB, 0.22132E+01_JPRB ,&
&  0.23848E+01_JPRB, 0.24159E+01_JPRB, 0.24376E+01_JPRB, 0.24859E+01_JPRB, 0.24767E+01_JPRB, 0.24444E+01_JPRB, 0.23559E+01_JPRB ,&
&  0.21635E+01_JPRB, 0.20368E+01_JPRB, 0.19646E+01_JPRB, 0.19846E+01_JPRB, 0.19059E+01_JPRB, 0.18580E+01_JPRB, 0.18309E+01_JPRB ,&
&  0.16693E+01_JPRB, 0.15482E+01_JPRB, 0.12396E+01_JPRB, 0.12637E+01_JPRB, 0.10907E+01_JPRB, 0.11001E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.12970E+01_JPRB, 0.13277E+01_JPRB, 0.13252E+01_JPRB, 0.15671E+01_JPRB, 0.17188E+01_JPRB, 0.17430E+01_JPRB ,&
&  0.17020E+01_JPRB, 0.15355E+01_JPRB, 0.15708E+01_JPRB, 0.16030E+01_JPRB, 0.15824E+01_JPRB, 0.15505E+01_JPRB, 0.15076E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.13746E+01_JPRB, 0.13289E+01_JPRB, 0.12642E+01_JPRB, 0.12068E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.12991E+01_JPRB, 0.12532E+01_JPRB, 0.12225E+01_JPRB, 0.12079E+01_JPRB, 0.12016E+01_JPRB, 0.11950E+01_JPRB, 0.11797E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.10888E+01_JPRB, 0.10598E+01_JPRB, 0.10474E+01_JPRB, 0.10204E+01_JPRB, 0.10015E+01_JPRB, 0.10149E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10149E+01_JPRB, 0.89500E+00_JPRB, 0.82730E+00_JPRB, 0.78929E+00_JPRB, 0.80141E+00_JPRB, 0.83501E+00_JPRB ,&
&  0.88161E+00_JPRB, 0.91905E+00_JPRB, 0.96784E+00_JPRB, 0.10368E+01_JPRB, 0.11354E+01_JPRB, 0.12419E+01_JPRB, 0.14178E+01_JPRB ,&
&  0.16630E+01_JPRB, 0.15477E+01_JPRB, 0.15911E+01_JPRB, 0.15594E+01_JPRB, 0.15523E+01_JPRB, 0.14791E+01_JPRB, 0.14007E+01_JPRB ,&
&  0.13205E+01_JPRB, 0.12665E+01_JPRB, 0.12129E+01_JPRB, 0.11630E+01_JPRB, 0.11218E+01_JPRB, 0.10948E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.11215E+01_JPRB, 0.11151E+01_JPRB, 0.11342E+01_JPRB, 0.12373E+01_JPRB, 0.12994E+01_JPRB, 0.13421E+01_JPRB ,&
&  0.13216E+01_JPRB, 0.13775E+01_JPRB, 0.17302E+01_JPRB, 0.19286E+01_JPRB, 0.20753E+01_JPRB, 0.22165E+01_JPRB, 0.22680E+01_JPRB ,&
&  0.23747E+01_JPRB, 0.23017E+01_JPRB, 0.23207E+01_JPRB, 0.21177E+01_JPRB, 0.23595E+01_JPRB, 0.23468E+01_JPRB, 0.22794E+01_JPRB ,&
&  0.21634E+01_JPRB, 0.21244E+01_JPRB, 0.20357E+01_JPRB, 0.20712E+01_JPRB, 0.20625E+01_JPRB, 0.20863E+01_JPRB, 0.17186E+01_JPRB ,&
&  0.14841E+01_JPRB, 0.16075E+01_JPRB, 0.14800E+01_JPRB, 0.13406E+01_JPRB, 0.12131E+01_JPRB, 0.12313E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.12767E+01_JPRB, 0.12919E+01_JPRB, 0.12841E+01_JPRB, 0.13627E+01_JPRB, 0.16669E+01_JPRB, 0.19792E+01_JPRB, 0.20165E+01_JPRB ,&
&  0.18483E+01_JPRB, 0.18982E+01_JPRB, 0.19344E+01_JPRB, 0.19365E+01_JPRB, 0.19612E+01_JPRB, 0.19177E+01_JPRB, 0.18392E+01_JPRB ,&
&  0.17425E+01_JPRB, 0.16464E+01_JPRB, 0.15563E+01_JPRB, 0.14613E+01_JPRB, 0.13655E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.14381E+01_JPRB, 0.13877E+01_JPRB, 0.13427E+01_JPRB, 0.13107E+01_JPRB, 0.12819E+01_JPRB, 0.12606E+01_JPRB, 0.12327E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.11113E+01_JPRB, 0.10993E+01_JPRB, 0.10803E+01_JPRB, 0.10560E+01_JPRB, 0.10420E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10147E+01_JPRB, 0.89284E+00_JPRB, 0.88250E+00_JPRB, 0.85660E+00_JPRB, 0.81131E+00_JPRB, 0.82314E+00_JPRB ,&
&  0.85914E+00_JPRB, 0.90303E+00_JPRB, 0.95485E+00_JPRB, 0.10754E+01_JPRB, 0.12256E+01_JPRB, 0.14070E+01_JPRB, 0.16124E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.17352E+01_JPRB, 0.17996E+01_JPRB, 0.17030E+01_JPRB, 0.15803E+01_JPRB, 0.14581E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.12965E+01_JPRB, 0.12347E+01_JPRB, 0.11755E+01_JPRB, 0.11325E+01_JPRB, 0.11124E+01_JPRB, 0.11014E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.11098E+01_JPRB, 0.10996E+01_JPRB, 0.11897E+01_JPRB, 0.11520E+01_JPRB, 0.11616E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.14849E+01_JPRB, 0.16787E+01_JPRB, 0.18007E+01_JPRB, 0.21084E+01_JPRB, 0.18335E+01_JPRB, 0.19716E+01_JPRB, 0.23548E+01_JPRB ,&
&  0.24367E+01_JPRB, 0.24158E+01_JPRB, 0.25288E+01_JPRB, 0.21095E+01_JPRB, 0.19184E+01_JPRB, 0.24900E+01_JPRB, 0.24787E+01_JPRB ,&
&  0.23613E+01_JPRB, 0.23135E+01_JPRB, 0.22691E+01_JPRB, 0.24791E+01_JPRB, 0.18253E+01_JPRB, 0.15996E+01_JPRB, 0.13323E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.13234E+01_JPRB, 0.14876E+01_JPRB, 0.14323E+01_JPRB, 0.14489E+01_JPRB, 0.14123E+01_JPRB, 0.13154E+01_JPRB ,&
&  0.12948E+01_JPRB, 0.12920E+01_JPRB, 0.13268E+01_JPRB, 0.15851E+01_JPRB, 0.21452E+01_JPRB, 0.21506E+01_JPRB, 0.23391E+01_JPRB ,&
&  0.23788E+01_JPRB, 0.25212E+01_JPRB, 0.23814E+01_JPRB, 0.24001E+01_JPRB, 0.23501E+01_JPRB, 0.22487E+01_JPRB, 0.20956E+01_JPRB ,&
&  0.19755E+01_JPRB, 0.18590E+01_JPRB, 0.17304E+01_JPRB, 0.16117E+01_JPRB, 0.15132E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.15426E+01_JPRB, 0.14748E+01_JPRB, 0.14150E+01_JPRB, 0.13622E+01_JPRB, 0.13304E+01_JPRB, 0.13008E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.11111E+01_JPRB, 0.11094E+01_JPRB, 0.10693E+01_JPRB, 0.10488E+01_JPRB, 0.10515E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.11288E+01_JPRB, 0.11318E+01_JPRB, 0.98832E+00_JPRB, 0.91702E+00_JPRB, 0.93471E+00_JPRB, 0.86784E+00_JPRB, 0.83905E+00_JPRB ,&
&  0.88002E+00_JPRB, 0.93187E+00_JPRB, 0.10378E+01_JPRB, 0.12473E+01_JPRB, 0.14698E+01_JPRB, 0.16887E+01_JPRB, 0.18820E+01_JPRB ,&
&  0.19342E+01_JPRB, 0.19176E+01_JPRB, 0.18189E+01_JPRB, 0.16839E+01_JPRB, 0.15084E+01_JPRB, 0.13745E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.11821E+01_JPRB, 0.11596E+01_JPRB, 0.11288E+01_JPRB, 0.11180E+01_JPRB, 0.10792E+01_JPRB, 0.10691E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.10973E+01_JPRB, 0.11225E+01_JPRB, 0.12494E+01_JPRB, 0.12258E+01_JPRB, 0.11857E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.14311E+01_JPRB, 0.15592E+01_JPRB, 0.17444E+01_JPRB, 0.19816E+01_JPRB, 0.23445E+01_JPRB, 0.24464E+01_JPRB, 0.26432E+01_JPRB ,&
&  0.21826E+01_JPRB, 0.19607E+01_JPRB, 0.19682E+01_JPRB, 0.18750E+01_JPRB, 0.23246E+01_JPRB, 0.31723E+01_JPRB, 0.32068E+01_JPRB ,&
&  0.30394E+01_JPRB, 0.28945E+01_JPRB, 0.28695E+01_JPRB, 0.23792E+01_JPRB, 0.18061E+01_JPRB, 0.14855E+01_JPRB, 0.17202E+01_JPRB ,&
&  0.18203E+01_JPRB, 0.19519E+01_JPRB, 0.18130E+01_JPRB, 0.15822E+01_JPRB, 0.14526E+01_JPRB, 0.17099E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.15374E+01_JPRB, 0.14729E+01_JPRB, 0.19104E+01_JPRB, 0.25275E+01_JPRB, 0.28225E+01_JPRB, 0.28579E+01_JPRB, 0.32771E+01_JPRB ,&
&  0.33747E+01_JPRB, 0.34565E+01_JPRB, 0.33059E+01_JPRB, 0.30929E+01_JPRB, 0.28345E+01_JPRB, 0.25532E+01_JPRB, 0.23340E+01_JPRB ,&
&  0.21639E+01_JPRB, 0.19993E+01_JPRB, 0.18462E+01_JPRB, 0.17150E+01_JPRB, 0.16191E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.16260E+01_JPRB, 0.15385E+01_JPRB, 0.14630E+01_JPRB, 0.14135E+01_JPRB, 0.13684E+01_JPRB, 0.13153E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.11182E+01_JPRB, 0.11059E+01_JPRB, 0.10631E+01_JPRB, 0.10541E+01_JPRB, 0.10683E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.12718E+01_JPRB, 0.11075E+01_JPRB, 0.10384E+01_JPRB, 0.10279E+01_JPRB, 0.10031E+01_JPRB, 0.89902E+00_JPRB ,&
&  0.93866E+00_JPRB, 0.10138E+01_JPRB, 0.11705E+01_JPRB, 0.14351E+01_JPRB, 0.17767E+01_JPRB, 0.18680E+01_JPRB, 0.19382E+01_JPRB ,&
&  0.19750E+01_JPRB, 0.18566E+01_JPRB, 0.17273E+01_JPRB, 0.15943E+01_JPRB, 0.14338E+01_JPRB, 0.13347E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.12410E+01_JPRB, 0.12362E+01_JPRB, 0.12206E+01_JPRB, 0.12202E+01_JPRB, 0.11746E+01_JPRB, 0.11264E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.11114E+01_JPRB, 0.11487E+01_JPRB, 0.12835E+01_JPRB, 0.13788E+01_JPRB, 0.13346E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.15872E+01_JPRB, 0.17577E+01_JPRB, 0.18454E+01_JPRB, 0.20952E+01_JPRB, 0.23217E+01_JPRB, 0.24156E+01_JPRB, 0.25950E+01_JPRB ,&
&  0.24122E+01_JPRB, 0.25477E+01_JPRB, 0.30761E+01_JPRB, 0.36937E+01_JPRB, 0.32321E+01_JPRB, 0.32509E+01_JPRB, 0.43816E+01_JPRB ,&
&  0.32451E+01_JPRB, 0.32119E+01_JPRB, 0.33394E+01_JPRB, 0.32896E+01_JPRB, 0.15485E+01_JPRB, 0.13320E+01_JPRB, 0.18893E+01_JPRB ,&
&  0.19930E+01_JPRB, 0.13954E+01_JPRB, 0.11344E+01_JPRB, 0.12982E+01_JPRB, 0.11161E+01_JPRB, 0.13468E+01_JPRB, 0.19178E+01_JPRB ,&
&  0.20196E+01_JPRB, 0.20828E+01_JPRB, 0.29895E+01_JPRB, 0.34093E+01_JPRB, 0.33198E+01_JPRB, 0.43494E+01_JPRB, 0.44116E+01_JPRB ,&
&  0.43895E+01_JPRB, 0.39446E+01_JPRB, 0.39168E+01_JPRB, 0.35730E+01_JPRB, 0.31846E+01_JPRB, 0.28285E+01_JPRB, 0.25377E+01_JPRB ,&
&  0.22683E+01_JPRB, 0.20737E+01_JPRB, 0.19402E+01_JPRB, 0.18072E+01_JPRB, 0.17027E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.15833E+01_JPRB, 0.14961E+01_JPRB, 0.14070E+01_JPRB, 0.13500E+01_JPRB, 0.13159E+01_JPRB, 0.12699E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.11367E+01_JPRB, 0.10974E+01_JPRB, 0.10662E+01_JPRB, 0.10482E+01_JPRB, 0.10618E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.11527E+01_JPRB, 0.12281E+01_JPRB, 0.12403E+01_JPRB, 0.11749E+01_JPRB, 0.11521E+01_JPRB, 0.11057E+01_JPRB, 0.10397E+01_JPRB ,&
&  0.10871E+01_JPRB, 0.12150E+01_JPRB, 0.14045E+01_JPRB, 0.16573E+01_JPRB, 0.18066E+01_JPRB, 0.18537E+01_JPRB, 0.19623E+01_JPRB ,&
&  0.18761E+01_JPRB, 0.17332E+01_JPRB, 0.16344E+01_JPRB, 0.15184E+01_JPRB, 0.14012E+01_JPRB, 0.13342E+01_JPRB, 0.12857E+01_JPRB ,&
&  0.12618E+01_JPRB, 0.12500E+01_JPRB, 0.12442E+01_JPRB, 0.11911E+01_JPRB, 0.11409E+01_JPRB, 0.11112E+01_JPRB, 0.10889E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.11353E+01_JPRB, 0.12099E+01_JPRB, 0.13206E+01_JPRB, 0.15227E+01_JPRB, 0.15986E+01_JPRB, 0.15052E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.15617E+01_JPRB, 0.19593E+01_JPRB, 0.19952E+01_JPRB, 0.19809E+01_JPRB, 0.20825E+01_JPRB, 0.21308E+01_JPRB ,&
&  0.23817E+01_JPRB, 0.29511E+01_JPRB, 0.30932E+01_JPRB, 0.38214E+01_JPRB, 0.40566E+01_JPRB, 0.30758E+01_JPRB, 0.34647E+01_JPRB ,&
&  0.34380E+01_JPRB, 0.30320E+01_JPRB, 0.24034E+01_JPRB, 0.18747E+01_JPRB, 0.21115E+01_JPRB, 0.19836E+01_JPRB, 0.13160E+01_JPRB ,&
&  0.11953E+01_JPRB, 0.11328E+01_JPRB, 0.10905E+01_JPRB, 0.10863E+01_JPRB, 0.10792E+01_JPRB, 0.13822E+01_JPRB, 0.18748E+01_JPRB ,&
&  0.26376E+01_JPRB, 0.32695E+01_JPRB, 0.38617E+01_JPRB, 0.39868E+01_JPRB, 0.36170E+01_JPRB, 0.47133E+01_JPRB, 0.50207E+01_JPRB ,&
&  0.41226E+01_JPRB, 0.37210E+01_JPRB, 0.36147E+01_JPRB, 0.32853E+01_JPRB, 0.29642E+01_JPRB, 0.26869E+01_JPRB, 0.24215E+01_JPRB ,&
&  0.21957E+01_JPRB, 0.20341E+01_JPRB, 0.19115E+01_JPRB, 0.17935E+01_JPRB, 0.16817E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.14793E+01_JPRB, 0.14190E+01_JPRB, 0.13778E+01_JPRB, 0.13432E+01_JPRB, 0.12994E+01_JPRB, 0.12582E+01_JPRB, 0.11746E+01_JPRB ,&
&  0.11381E+01_JPRB, 0.11369E+01_JPRB, 0.10803E+01_JPRB, 0.10411E+01_JPRB, 0.10238E+01_JPRB, 0.10431E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.12058E+01_JPRB, 0.13726E+01_JPRB, 0.13926E+01_JPRB, 0.15249E+01_JPRB, 0.13936E+01_JPRB, 0.14066E+01_JPRB ,&
&  0.14198E+01_JPRB, 0.15613E+01_JPRB, 0.17634E+01_JPRB, 0.18736E+01_JPRB, 0.18872E+01_JPRB, 0.19540E+01_JPRB, 0.18959E+01_JPRB ,&
&  0.17645E+01_JPRB, 0.16830E+01_JPRB, 0.15584E+01_JPRB, 0.14414E+01_JPRB, 0.13677E+01_JPRB, 0.13007E+01_JPRB, 0.12618E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.12434E+01_JPRB, 0.11920E+01_JPRB, 0.11688E+01_JPRB, 0.11611E+01_JPRB, 0.11457E+01_JPRB, 0.11300E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.12201E+01_JPRB, 0.13086E+01_JPRB, 0.14051E+01_JPRB, 0.13985E+01_JPRB, 0.12570E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.15481E+01_JPRB, 0.17906E+01_JPRB, 0.18541E+01_JPRB, 0.20844E+01_JPRB, 0.19794E+01_JPRB, 0.19228E+01_JPRB, 0.21108E+01_JPRB ,&
&  0.25112E+01_JPRB, 0.33836E+01_JPRB, 0.30863E+01_JPRB, 0.36129E+01_JPRB, 0.47330E+01_JPRB, 0.35904E+01_JPRB, 0.30428E+01_JPRB ,&
&  0.32677E+01_JPRB, 0.27939E+01_JPRB, 0.24358E+01_JPRB, 0.22838E+01_JPRB, 0.34328E+01_JPRB, 0.44287E+01_JPRB, 0.18737E+01_JPRB ,&
&  0.14436E+01_JPRB, 0.13687E+01_JPRB, 0.13166E+01_JPRB, 0.12213E+01_JPRB, 0.12271E+01_JPRB, 0.15987E+01_JPRB, 0.25143E+01_JPRB ,&
&  0.30500E+01_JPRB, 0.41784E+01_JPRB, 0.46344E+01_JPRB, 0.45471E+01_JPRB, 0.37446E+01_JPRB, 0.41756E+01_JPRB, 0.38301E+01_JPRB ,&
&  0.40139E+01_JPRB, 0.38184E+01_JPRB, 0.34945E+01_JPRB, 0.31604E+01_JPRB, 0.28376E+01_JPRB, 0.25427E+01_JPRB, 0.22807E+01_JPRB ,&
&  0.20559E+01_JPRB, 0.18875E+01_JPRB, 0.17592E+01_JPRB, 0.16579E+01_JPRB, 0.15586E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.14698E+01_JPRB, 0.14169E+01_JPRB, 0.13694E+01_JPRB, 0.13177E+01_JPRB, 0.12845E+01_JPRB, 0.12351E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.11230E+01_JPRB, 0.10629E+01_JPRB, 0.10291E+01_JPRB, 0.10226E+01_JPRB, 0.10380E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.11985E+01_JPRB, 0.13782E+01_JPRB, 0.17791E+01_JPRB, 0.20006E+01_JPRB, 0.18637E+01_JPRB, 0.20159E+01_JPRB ,&
&  0.19366E+01_JPRB, 0.20014E+01_JPRB, 0.21252E+01_JPRB, 0.20623E+01_JPRB, 0.20065E+01_JPRB, 0.20239E+01_JPRB, 0.18365E+01_JPRB ,&
&  0.16881E+01_JPRB, 0.16216E+01_JPRB, 0.14966E+01_JPRB, 0.14082E+01_JPRB, 0.13385E+01_JPRB, 0.13120E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.12279E+01_JPRB, 0.12064E+01_JPRB, 0.12116E+01_JPRB, 0.12220E+01_JPRB, 0.12207E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.13540E+01_JPRB, 0.14411E+01_JPRB, 0.15105E+01_JPRB, 0.12945E+01_JPRB, 0.14567E+01_JPRB, 0.15116E+01_JPRB ,&
&  0.15692E+01_JPRB, 0.16454E+01_JPRB, 0.16272E+01_JPRB, 0.17909E+01_JPRB, 0.19406E+01_JPRB, 0.19313E+01_JPRB, 0.20873E+01_JPRB ,&
&  0.27847E+01_JPRB, 0.28653E+01_JPRB, 0.30215E+01_JPRB, 0.36598E+01_JPRB, 0.41187E+01_JPRB, 0.52433E+01_JPRB, 0.28125E+01_JPRB ,&
&  0.22126E+01_JPRB, 0.26513E+01_JPRB, 0.27413E+01_JPRB, 0.30085E+01_JPRB, 0.40020E+01_JPRB, 0.64243E+01_JPRB, 0.53886E+01_JPRB ,&
&  0.18729E+01_JPRB, 0.14407E+01_JPRB, 0.16730E+01_JPRB, 0.16831E+01_JPRB, 0.15213E+01_JPRB, 0.20132E+01_JPRB, 0.46513E+01_JPRB ,&
&  0.43469E+01_JPRB, 0.50604E+01_JPRB, 0.49185E+01_JPRB, 0.41918E+01_JPRB, 0.35833E+01_JPRB, 0.37545E+01_JPRB, 0.39257E+01_JPRB ,&
&  0.37619E+01_JPRB, 0.34329E+01_JPRB, 0.31305E+01_JPRB, 0.28663E+01_JPRB, 0.25530E+01_JPRB, 0.22390E+01_JPRB, 0.19678E+01_JPRB ,&
&  0.17769E+01_JPRB, 0.16700E+01_JPRB, 0.16134E+01_JPRB, 0.15680E+01_JPRB, 0.15195E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.13997E+01_JPRB, 0.13529E+01_JPRB, 0.13169E+01_JPRB, 0.12897E+01_JPRB, 0.12794E+01_JPRB, 0.12315E+01_JPRB, 0.11733E+01_JPRB ,&
&  0.12035E+01_JPRB, 0.11287E+01_JPRB, 0.10740E+01_JPRB, 0.10329E+01_JPRB, 0.10361E+01_JPRB, 0.10612E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.12040E+01_JPRB, 0.14235E+01_JPRB, 0.19313E+01_JPRB, 0.22819E+01_JPRB, 0.21729E+01_JPRB, 0.23195E+01_JPRB ,&
&  0.25603E+01_JPRB, 0.24656E+01_JPRB, 0.22115E+01_JPRB, 0.20395E+01_JPRB, 0.20758E+01_JPRB, 0.20609E+01_JPRB, 0.18231E+01_JPRB ,&
&  0.16854E+01_JPRB, 0.16136E+01_JPRB, 0.15071E+01_JPRB, 0.14305E+01_JPRB, 0.13906E+01_JPRB, 0.13619E+01_JPRB, 0.13416E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.12895E+01_JPRB, 0.12944E+01_JPRB, 0.13023E+01_JPRB, 0.13305E+01_JPRB, 0.13256E+01_JPRB, 0.13737E+01_JPRB ,&
&  0.14540E+01_JPRB, 0.15118E+01_JPRB, 0.15714E+01_JPRB, 0.15104E+01_JPRB, 0.15205E+01_JPRB, 0.15337E+01_JPRB, 0.15931E+01_JPRB ,&
&  0.15964E+01_JPRB, 0.16405E+01_JPRB, 0.17174E+01_JPRB, 0.18015E+01_JPRB, 0.19770E+01_JPRB, 0.21217E+01_JPRB, 0.23565E+01_JPRB ,&
&  0.28473E+01_JPRB, 0.31330E+01_JPRB, 0.31273E+01_JPRB, 0.34261E+01_JPRB, 0.38723E+01_JPRB, 0.45591E+01_JPRB, 0.35832E+01_JPRB ,&
&  0.28577E+01_JPRB, 0.25364E+01_JPRB, 0.31106E+01_JPRB, 0.44138E+01_JPRB, 0.53190E+01_JPRB, 0.72444E+01_JPRB, 0.98814E+01_JPRB ,&
&  0.10681E+02_JPRB, 0.54826E+01_JPRB, 0.42725E+01_JPRB, 0.52339E+01_JPRB, 0.38531E+01_JPRB, 0.32464E+01_JPRB, 0.43217E+01_JPRB ,&
&  0.49113E+01_JPRB, 0.51723E+01_JPRB, 0.47848E+01_JPRB, 0.37480E+01_JPRB, 0.35179E+01_JPRB, 0.37673E+01_JPRB, 0.36334E+01_JPRB ,&
&  0.32535E+01_JPRB, 0.29417E+01_JPRB, 0.26177E+01_JPRB, 0.22703E+01_JPRB, 0.19783E+01_JPRB, 0.17678E+01_JPRB, 0.16432E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.15521E+01_JPRB, 0.15325E+01_JPRB, 0.14969E+01_JPRB, 0.14495E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.13241E+01_JPRB, 0.13073E+01_JPRB, 0.13113E+01_JPRB, 0.13118E+01_JPRB, 0.12921E+01_JPRB, 0.12385E+01_JPRB, 0.12318E+01_JPRB ,&
&  0.12651E+01_JPRB, 0.11716E+01_JPRB, 0.10949E+01_JPRB, 0.10614E+01_JPRB, 0.10758E+01_JPRB, 0.11132E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.13103E+01_JPRB, 0.16697E+01_JPRB, 0.21415E+01_JPRB, 0.26173E+01_JPRB, 0.34478E+01_JPRB, 0.24081E+01_JPRB ,&
&  0.25305E+01_JPRB, 0.24841E+01_JPRB, 0.21142E+01_JPRB, 0.20340E+01_JPRB, 0.20855E+01_JPRB, 0.20981E+01_JPRB, 0.18776E+01_JPRB ,&
&  0.17402E+01_JPRB, 0.16673E+01_JPRB, 0.16013E+01_JPRB, 0.15351E+01_JPRB, 0.14849E+01_JPRB, 0.14773E+01_JPRB, 0.14419E+01_JPRB ,&
&  0.14492E+01_JPRB, 0.14519E+01_JPRB, 0.14275E+01_JPRB, 0.14530E+01_JPRB, 0.14795E+01_JPRB, 0.14853E+01_JPRB, 0.15515E+01_JPRB ,&
&  0.16123E+01_JPRB, 0.16444E+01_JPRB, 0.16571E+01_JPRB, 0.15880E+01_JPRB, 0.16292E+01_JPRB, 0.16649E+01_JPRB, 0.17209E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.15488E+01_JPRB, 0.17307E+01_JPRB, 0.19838E+01_JPRB, 0.21946E+01_JPRB, 0.22661E+01_JPRB, 0.25380E+01_JPRB ,&
&  0.31374E+01_JPRB, 0.36156E+01_JPRB, 0.38572E+01_JPRB, 0.33309E+01_JPRB, 0.35873E+01_JPRB, 0.41305E+01_JPRB, 0.44528E+01_JPRB ,&
&  0.46317E+01_JPRB, 0.43835E+01_JPRB, 0.49691E+01_JPRB, 0.55441E+01_JPRB, 0.60465E+01_JPRB, 0.76477E+01_JPRB, 0.79440E+01_JPRB ,&
&  0.95543E+01_JPRB, 0.11850E+02_JPRB, 0.97895E+01_JPRB, 0.74131E+01_JPRB, 0.61152E+01_JPRB, 0.51383E+01_JPRB, 0.50130E+01_JPRB ,&
&  0.54374E+01_JPRB, 0.49442E+01_JPRB, 0.42368E+01_JPRB, 0.42226E+01_JPRB, 0.35225E+01_JPRB, 0.33835E+01_JPRB, 0.30098E+01_JPRB ,&
&  0.24996E+01_JPRB, 0.21215E+01_JPRB, 0.17730E+01_JPRB, 0.15568E+01_JPRB, 0.14414E+01_JPRB, 0.14127E+01_JPRB, 0.14094E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.14183E+01_JPRB, 0.14122E+01_JPRB, 0.13956E+01_JPRB, 0.13604E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.13125E+01_JPRB, 0.13142E+01_JPRB, 0.13357E+01_JPRB, 0.13431E+01_JPRB, 0.13227E+01_JPRB, 0.13044E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.13290E+01_JPRB, 0.11864E+01_JPRB, 0.11608E+01_JPRB, 0.11642E+01_JPRB, 0.11917E+01_JPRB, 0.12100E+01_JPRB, 0.12324E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.15537E+01_JPRB, 0.19449E+01_JPRB, 0.23361E+01_JPRB, 0.27309E+01_JPRB, 0.32441E+01_JPRB, 0.31750E+01_JPRB ,&
&  0.23421E+01_JPRB, 0.26752E+01_JPRB, 0.21145E+01_JPRB, 0.21834E+01_JPRB, 0.22993E+01_JPRB, 0.24880E+01_JPRB, 0.22127E+01_JPRB ,&
&  0.19501E+01_JPRB, 0.18598E+01_JPRB, 0.19050E+01_JPRB, 0.18089E+01_JPRB, 0.16430E+01_JPRB, 0.15838E+01_JPRB, 0.16144E+01_JPRB ,&
&  0.16352E+01_JPRB, 0.16247E+01_JPRB, 0.16191E+01_JPRB, 0.16559E+01_JPRB, 0.16791E+01_JPRB, 0.16771E+01_JPRB, 0.16945E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.17206E+01_JPRB, 0.16961E+01_JPRB, 0.17188E+01_JPRB, 0.17471E+01_JPRB, 0.18312E+01_JPRB, 0.18969E+01_JPRB ,&
&  0.18539E+01_JPRB, 0.17726E+01_JPRB, 0.19085E+01_JPRB, 0.20848E+01_JPRB, 0.22525E+01_JPRB, 0.25610E+01_JPRB, 0.27836E+01_JPRB ,&
&  0.33787E+01_JPRB, 0.35269E+01_JPRB, 0.42345E+01_JPRB, 0.32331E+01_JPRB, 0.33894E+01_JPRB, 0.38941E+01_JPRB, 0.40311E+01_JPRB ,&
&  0.37091E+01_JPRB, 0.43732E+01_JPRB, 0.49673E+01_JPRB, 0.58204E+01_JPRB, 0.64525E+01_JPRB, 0.71031E+01_JPRB, 0.65646E+01_JPRB ,&
&  0.67381E+01_JPRB, 0.73147E+01_JPRB, 0.69859E+01_JPRB, 0.56903E+01_JPRB, 0.52397E+01_JPRB, 0.46001E+01_JPRB, 0.50426E+01_JPRB ,&
&  0.52797E+01_JPRB, 0.48179E+01_JPRB, 0.45599E+01_JPRB, 0.42278E+01_JPRB, 0.30333E+01_JPRB, 0.25880E+01_JPRB, 0.21142E+01_JPRB ,&
&  0.17010E+01_JPRB, 0.14472E+01_JPRB, 0.12437E+01_JPRB, 0.12094E+01_JPRB, 0.12264E+01_JPRB, 0.12533E+01_JPRB, 0.13041E+01_JPRB ,&
&  0.13150E+01_JPRB, 0.13226E+01_JPRB, 0.13352E+01_JPRB, 0.13353E+01_JPRB, 0.13223E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.13299E+01_JPRB, 0.13326E+01_JPRB, 0.13597E+01_JPRB, 0.13929E+01_JPRB, 0.14175E+01_JPRB, 0.14535E+01_JPRB, 0.14204E+01_JPRB ,&
&  0.13939E+01_JPRB, 0.12478E+01_JPRB, 0.12829E+01_JPRB, 0.13056E+01_JPRB, 0.13295E+01_JPRB, 0.13261E+01_JPRB, 0.13834E+01_JPRB ,&
&  0.15579E+01_JPRB, 0.18241E+01_JPRB, 0.21900E+01_JPRB, 0.26224E+01_JPRB, 0.30554E+01_JPRB, 0.35608E+01_JPRB, 0.36759E+01_JPRB ,&
&  0.28107E+01_JPRB, 0.24154E+01_JPRB, 0.22705E+01_JPRB, 0.23834E+01_JPRB, 0.24559E+01_JPRB, 0.26941E+01_JPRB, 0.27636E+01_JPRB ,&
&  0.25894E+01_JPRB, 0.23654E+01_JPRB, 0.24217E+01_JPRB, 0.21262E+01_JPRB, 0.17073E+01_JPRB, 0.17341E+01_JPRB, 0.17857E+01_JPRB ,&
&  0.18161E+01_JPRB, 0.18256E+01_JPRB, 0.18455E+01_JPRB, 0.18866E+01_JPRB, 0.19079E+01_JPRB, 0.18809E+01_JPRB, 0.18463E+01_JPRB ,&
&  0.18432E+01_JPRB, 0.18647E+01_JPRB, 0.18651E+01_JPRB, 0.18458E+01_JPRB, 0.19167E+01_JPRB, 0.20046E+01_JPRB, 0.20627E+01_JPRB ,&
&  0.21014E+01_JPRB, 0.21815E+01_JPRB, 0.22816E+01_JPRB, 0.24351E+01_JPRB, 0.24706E+01_JPRB, 0.28796E+01_JPRB, 0.31200E+01_JPRB ,&
&  0.34645E+01_JPRB, 0.36793E+01_JPRB, 0.41510E+01_JPRB, 0.30934E+01_JPRB, 0.35261E+01_JPRB, 0.38154E+01_JPRB, 0.40682E+01_JPRB ,&
&  0.40765E+01_JPRB, 0.46102E+01_JPRB, 0.49839E+01_JPRB, 0.54348E+01_JPRB, 0.58361E+01_JPRB, 0.54990E+01_JPRB, 0.56005E+01_JPRB ,&
&  0.52205E+01_JPRB, 0.58614E+01_JPRB, 0.55451E+01_JPRB, 0.51762E+01_JPRB, 0.44709E+01_JPRB, 0.41861E+01_JPRB, 0.43976E+01_JPRB ,&
&  0.46827E+01_JPRB, 0.42640E+01_JPRB, 0.38093E+01_JPRB, 0.31272E+01_JPRB, 0.21524E+01_JPRB, 0.17490E+01_JPRB, 0.14169E+01_JPRB ,&
&  0.12651E+01_JPRB, 0.11419E+01_JPRB, 0.11037E+01_JPRB, 0.11599E+01_JPRB, 0.11951E+01_JPRB, 0.12704E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.12746E+01_JPRB, 0.12997E+01_JPRB, 0.13303E+01_JPRB, 0.13460E+01_JPRB, 0.13426E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.12984E+01_JPRB, 0.12901E+01_JPRB, 0.12826E+01_JPRB, 0.12909E+01_JPRB, 0.13162E+01_JPRB, 0.13287E+01_JPRB, 0.13009E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.13484E+01_JPRB, 0.14145E+01_JPRB, 0.14511E+01_JPRB, 0.14692E+01_JPRB, 0.14969E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.17817E+01_JPRB, 0.20590E+01_JPRB, 0.23944E+01_JPRB, 0.26503E+01_JPRB, 0.28731E+01_JPRB, 0.31275E+01_JPRB, 0.33699E+01_JPRB ,&
&  0.28605E+01_JPRB, 0.20954E+01_JPRB, 0.23883E+01_JPRB, 0.25447E+01_JPRB, 0.26393E+01_JPRB, 0.30640E+01_JPRB, 0.35011E+01_JPRB ,&
&  0.27000E+01_JPRB, 0.24312E+01_JPRB, 0.24518E+01_JPRB, 0.22505E+01_JPRB, 0.18418E+01_JPRB, 0.18897E+01_JPRB, 0.19570E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.20282E+01_JPRB, 0.20844E+01_JPRB, 0.21391E+01_JPRB, 0.21565E+01_JPRB, 0.21248E+01_JPRB, 0.20802E+01_JPRB ,&
&  0.21002E+01_JPRB, 0.21396E+01_JPRB, 0.21868E+01_JPRB, 0.21883E+01_JPRB, 0.22331E+01_JPRB, 0.23247E+01_JPRB, 0.24087E+01_JPRB ,&
&  0.24883E+01_JPRB, 0.25163E+01_JPRB, 0.26718E+01_JPRB, 0.29793E+01_JPRB, 0.32651E+01_JPRB, 0.30062E+01_JPRB, 0.33350E+01_JPRB ,&
&  0.34071E+01_JPRB, 0.34878E+01_JPRB, 0.34413E+01_JPRB, 0.40558E+01_JPRB, 0.28519E+01_JPRB, 0.33975E+01_JPRB, 0.35236E+01_JPRB ,&
&  0.44072E+01_JPRB, 0.44970E+01_JPRB, 0.47325E+01_JPRB, 0.49981E+01_JPRB, 0.52337E+01_JPRB, 0.47135E+01_JPRB, 0.46353E+01_JPRB ,&
&  0.47652E+01_JPRB, 0.48312E+01_JPRB, 0.47894E+01_JPRB, 0.47579E+01_JPRB, 0.43542E+01_JPRB, 0.42405E+01_JPRB, 0.40527E+01_JPRB ,&
&  0.40132E+01_JPRB, 0.33296E+01_JPRB, 0.27213E+01_JPRB, 0.20678E+01_JPRB, 0.14515E+01_JPRB, 0.12533E+01_JPRB, 0.11776E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.11902E+01_JPRB, 0.11857E+01_JPRB, 0.11950E+01_JPRB, 0.12287E+01_JPRB, 0.12861E+01_JPRB, 0.12797E+01_JPRB ,&
&  0.12981E+01_JPRB, 0.13107E+01_JPRB, 0.13347E+01_JPRB, 0.13423E+01_JPRB, 0.13290E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.12837E+01_JPRB, 0.12691E+01_JPRB, 0.12418E+01_JPRB, 0.12537E+01_JPRB, 0.12635E+01_JPRB, 0.12709E+01_JPRB, 0.12783E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.14010E+01_JPRB, 0.14879E+01_JPRB, 0.15507E+01_JPRB, 0.15786E+01_JPRB, 0.16165E+01_JPRB, 0.17186E+01_JPRB ,&
&  0.18638E+01_JPRB, 0.20797E+01_JPRB, 0.23469E+01_JPRB, 0.25377E+01_JPRB, 0.26771E+01_JPRB, 0.27284E+01_JPRB, 0.28185E+01_JPRB ,&
&  0.27510E+01_JPRB, 0.27249E+01_JPRB, 0.25747E+01_JPRB, 0.21583E+01_JPRB, 0.21578E+01_JPRB, 0.27435E+01_JPRB, 0.25780E+01_JPRB ,&
&  0.22899E+01_JPRB, 0.21596E+01_JPRB, 0.21050E+01_JPRB, 0.20543E+01_JPRB, 0.20119E+01_JPRB, 0.20630E+01_JPRB, 0.21436E+01_JPRB ,&
&  0.21870E+01_JPRB, 0.22577E+01_JPRB, 0.23263E+01_JPRB, 0.23755E+01_JPRB, 0.24021E+01_JPRB, 0.24489E+01_JPRB, 0.24671E+01_JPRB ,&
&  0.25472E+01_JPRB, 0.26378E+01_JPRB, 0.27351E+01_JPRB, 0.27777E+01_JPRB, 0.27487E+01_JPRB, 0.28320E+01_JPRB, 0.29080E+01_JPRB ,&
&  0.29952E+01_JPRB, 0.30121E+01_JPRB, 0.30183E+01_JPRB, 0.31893E+01_JPRB, 0.32908E+01_JPRB, 0.29058E+01_JPRB, 0.29647E+01_JPRB ,&
&  0.32765E+01_JPRB, 0.33621E+01_JPRB, 0.30504E+01_JPRB, 0.38603E+01_JPRB, 0.24061E+01_JPRB, 0.32331E+01_JPRB, 0.40450E+01_JPRB ,&
&  0.41019E+01_JPRB, 0.41856E+01_JPRB, 0.43200E+01_JPRB, 0.44279E+01_JPRB, 0.44048E+01_JPRB, 0.40267E+01_JPRB, 0.42415E+01_JPRB ,&
&  0.44287E+01_JPRB, 0.43549E+01_JPRB, 0.42934E+01_JPRB, 0.42207E+01_JPRB, 0.40603E+01_JPRB, 0.36520E+01_JPRB, 0.30668E+01_JPRB ,&
&  0.24840E+01_JPRB, 0.26007E+01_JPRB, 0.21364E+01_JPRB, 0.17713E+01_JPRB, 0.12616E+01_JPRB, 0.10870E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.11519E+01_JPRB, 0.11842E+01_JPRB, 0.11932E+01_JPRB, 0.12251E+01_JPRB, 0.12474E+01_JPRB, 0.12696E+01_JPRB, 0.12667E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.13011E+01_JPRB, 0.13161E+01_JPRB, 0.13135E+01_JPRB, 0.13012E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.12162E+01_JPRB, 0.11980E+01_JPRB, 0.11764E+01_JPRB, 0.12176E+01_JPRB, 0.12155E+01_JPRB, 0.12114E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.12587E+01_JPRB, 0.13071E+01_JPRB, 0.13725E+01_JPRB, 0.14454E+01_JPRB, 0.15302E+01_JPRB, 0.16391E+01_JPRB, 0.17332E+01_JPRB ,&
&  0.18391E+01_JPRB, 0.19397E+01_JPRB, 0.21078E+01_JPRB, 0.23199E+01_JPRB, 0.25936E+01_JPRB, 0.27402E+01_JPRB, 0.27559E+01_JPRB ,&
&  0.26224E+01_JPRB, 0.26914E+01_JPRB, 0.27896E+01_JPRB, 0.28865E+01_JPRB, 0.25664E+01_JPRB, 0.28862E+01_JPRB, 0.30346E+01_JPRB ,&
&  0.29110E+01_JPRB, 0.24973E+01_JPRB, 0.21713E+01_JPRB, 0.21348E+01_JPRB, 0.21383E+01_JPRB, 0.22116E+01_JPRB, 0.23114E+01_JPRB ,&
&  0.23867E+01_JPRB, 0.24736E+01_JPRB, 0.25342E+01_JPRB, 0.25995E+01_JPRB, 0.26973E+01_JPRB, 0.28505E+01_JPRB, 0.29297E+01_JPRB ,&
&  0.29990E+01_JPRB, 0.31338E+01_JPRB, 0.31958E+01_JPRB, 0.31598E+01_JPRB, 0.32538E+01_JPRB, 0.33275E+01_JPRB, 0.33955E+01_JPRB ,&
&  0.35165E+01_JPRB, 0.33788E+01_JPRB, 0.34604E+01_JPRB, 0.35934E+01_JPRB, 0.34830E+01_JPRB, 0.31645E+01_JPRB, 0.31415E+01_JPRB ,&
&  0.33229E+01_JPRB, 0.34110E+01_JPRB, 0.25169E+01_JPRB, 0.35674E+01_JPRB, 0.43995E+01_JPRB, 0.39162E+01_JPRB, 0.39049E+01_JPRB ,&
&  0.38537E+01_JPRB, 0.39447E+01_JPRB, 0.40953E+01_JPRB, 0.42422E+01_JPRB, 0.42880E+01_JPRB, 0.42645E+01_JPRB, 0.39041E+01_JPRB ,&
&  0.42974E+01_JPRB, 0.41675E+01_JPRB, 0.39327E+01_JPRB, 0.36750E+01_JPRB, 0.33350E+01_JPRB, 0.27877E+01_JPRB, 0.22346E+01_JPRB ,&
&  0.18884E+01_JPRB, 0.21810E+01_JPRB, 0.18909E+01_JPRB, 0.17615E+01_JPRB, 0.15068E+01_JPRB, 0.10456E+01_JPRB, 0.11074E+01_JPRB ,&
&  0.11430E+01_JPRB, 0.11675E+01_JPRB, 0.11850E+01_JPRB, 0.11800E+01_JPRB, 0.11943E+01_JPRB, 0.12110E+01_JPRB, 0.12289E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.12344E+01_JPRB, 0.12413E+01_JPRB, 0.12346E+01_JPRB, 0.12302E+01_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.11746E+01_JPRB, 0.11704E+01_JPRB, 0.11564E+01_JPRB, 0.11720E+01_JPRB, 0.11612E+01_JPRB, 0.11874E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.11592E+01_JPRB, 0.11798E+01_JPRB, 0.12047E+01_JPRB, 0.12539E+01_JPRB, 0.13393E+01_JPRB, 0.14356E+01_JPRB ,&
&  0.15270E+01_JPRB, 0.15349E+01_JPRB, 0.15695E+01_JPRB, 0.17020E+01_JPRB, 0.20273E+01_JPRB, 0.23792E+01_JPRB, 0.25230E+01_JPRB ,&
&  0.26917E+01_JPRB, 0.29228E+01_JPRB, 0.31154E+01_JPRB, 0.33569E+01_JPRB, 0.34802E+01_JPRB, 0.36538E+01_JPRB, 0.40237E+01_JPRB ,&
&  0.33263E+01_JPRB, 0.25484E+01_JPRB, 0.22885E+01_JPRB, 0.21910E+01_JPRB, 0.22211E+01_JPRB, 0.23040E+01_JPRB, 0.23889E+01_JPRB ,&
&  0.24699E+01_JPRB, 0.25657E+01_JPRB, 0.26513E+01_JPRB, 0.27768E+01_JPRB, 0.29567E+01_JPRB, 0.31638E+01_JPRB, 0.32897E+01_JPRB ,&
&  0.33606E+01_JPRB, 0.34561E+01_JPRB, 0.35670E+01_JPRB, 0.34500E+01_JPRB, 0.38522E+01_JPRB, 0.40108E+01_JPRB, 0.40956E+01_JPRB ,&
&  0.40673E+01_JPRB, 0.39177E+01_JPRB, 0.39981E+01_JPRB, 0.40309E+01_JPRB, 0.40299E+01_JPRB, 0.37422E+01_JPRB, 0.35453E+01_JPRB ,&
&  0.35463E+01_JPRB, 0.30863E+01_JPRB, 0.23253E+01_JPRB, 0.26566E+01_JPRB, 0.30561E+01_JPRB, 0.28094E+01_JPRB, 0.36986E+01_JPRB ,&
&  0.36624E+01_JPRB, 0.36143E+01_JPRB, 0.36917E+01_JPRB, 0.38143E+01_JPRB, 0.40162E+01_JPRB, 0.39240E+01_JPRB, 0.37550E+01_JPRB ,&
&  0.37788E+01_JPRB, 0.35930E+01_JPRB, 0.32792E+01_JPRB, 0.28740E+01_JPRB, 0.24620E+01_JPRB, 0.20998E+01_JPRB, 0.18866E+01_JPRB ,&
&  0.17856E+01_JPRB, 0.17891E+01_JPRB, 0.17241E+01_JPRB, 0.15146E+01_JPRB, 0.11812E+01_JPRB, 0.97442E+00_JPRB, 0.10013E+01_JPRB ,&
&  0.10673E+01_JPRB, 0.10838E+01_JPRB, 0.11005E+01_JPRB, 0.10976E+01_JPRB, 0.11023E+01_JPRB, 0.11042E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.11691E+01_JPRB, 0.11771E+01_JPRB, 0.11803E+01_JPRB, 0.11835E+01_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.11089E+01_JPRB, 0.11054E+01_JPRB, 0.11044E+01_JPRB, 0.11204E+01_JPRB, 0.11323E+01_JPRB, 0.11133E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.10992E+01_JPRB, 0.10795E+01_JPRB, 0.10920E+01_JPRB, 0.10995E+01_JPRB, 0.11228E+01_JPRB, 0.11512E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11553E+01_JPRB, 0.10943E+01_JPRB, 0.10913E+01_JPRB, 0.12042E+01_JPRB, 0.14208E+01_JPRB, 0.18396E+01_JPRB ,&
&  0.23123E+01_JPRB, 0.25684E+01_JPRB, 0.27537E+01_JPRB, 0.28515E+01_JPRB, 0.28673E+01_JPRB, 0.28495E+01_JPRB, 0.38668E+01_JPRB ,&
&  0.41478E+01_JPRB, 0.24660E+01_JPRB, 0.23245E+01_JPRB, 0.21345E+01_JPRB, 0.21806E+01_JPRB, 0.22135E+01_JPRB, 0.22790E+01_JPRB ,&
&  0.23369E+01_JPRB, 0.24408E+01_JPRB, 0.25869E+01_JPRB, 0.27914E+01_JPRB, 0.29936E+01_JPRB, 0.32034E+01_JPRB, 0.33607E+01_JPRB ,&
&  0.34607E+01_JPRB, 0.35472E+01_JPRB, 0.37153E+01_JPRB, 0.37624E+01_JPRB, 0.38213E+01_JPRB, 0.40323E+01_JPRB, 0.42416E+01_JPRB ,&
&  0.44044E+01_JPRB, 0.47182E+01_JPRB, 0.42704E+01_JPRB, 0.42683E+01_JPRB, 0.44175E+01_JPRB, 0.38727E+01_JPRB, 0.37474E+01_JPRB ,&
&  0.36845E+01_JPRB, 0.29930E+01_JPRB, 0.21170E+01_JPRB, 0.23269E+01_JPRB, 0.26060E+01_JPRB, 0.28252E+01_JPRB, 0.31621E+01_JPRB ,&
&  0.30671E+01_JPRB, 0.29237E+01_JPRB, 0.29689E+01_JPRB, 0.31710E+01_JPRB, 0.32078E+01_JPRB, 0.31186E+01_JPRB, 0.29587E+01_JPRB ,&
&  0.28358E+01_JPRB, 0.26357E+01_JPRB, 0.24979E+01_JPRB, 0.22695E+01_JPRB, 0.20944E+01_JPRB, 0.18987E+01_JPRB, 0.16638E+01_JPRB ,&
&  0.17798E+01_JPRB, 0.20773E+01_JPRB, 0.26147E+01_JPRB, 0.21337E+01_JPRB, 0.12446E+01_JPRB, 0.99675E+00_JPRB, 0.94585E+00_JPRB ,&
&  0.96896E+00_JPRB, 0.98403E+00_JPRB, 0.10164E+01_JPRB, 0.10255E+01_JPRB, 0.10198E+01_JPRB, 0.10129E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10855E+01_JPRB, 0.11088E+01_JPRB, 0.11059E+01_JPRB, 0.11020E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.11515E+01_JPRB, 0.11593E+01_JPRB, 0.11888E+01_JPRB, 0.11999E+01_JPRB, 0.11981E+01_JPRB, 0.12040E+01_JPRB, 0.12025E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11986E+01_JPRB, 0.12068E+01_JPRB, 0.12189E+01_JPRB, 0.12381E+01_JPRB, 0.12577E+01_JPRB, 0.12737E+01_JPRB ,&
&  0.12688E+01_JPRB, 0.12660E+01_JPRB, 0.13044E+01_JPRB, 0.13294E+01_JPRB, 0.13230E+01_JPRB, 0.13176E+01_JPRB, 0.13558E+01_JPRB ,&
&  0.14464E+01_JPRB, 0.15039E+01_JPRB, 0.15252E+01_JPRB, 0.15140E+01_JPRB, 0.15839E+01_JPRB, 0.18393E+01_JPRB, 0.28086E+01_JPRB ,&
&  0.33607E+01_JPRB, 0.24502E+01_JPRB, 0.20698E+01_JPRB, 0.18780E+01_JPRB, 0.19132E+01_JPRB, 0.20078E+01_JPRB, 0.19954E+01_JPRB ,&
&  0.19766E+01_JPRB, 0.20320E+01_JPRB, 0.20881E+01_JPRB, 0.21464E+01_JPRB, 0.22316E+01_JPRB, 0.22956E+01_JPRB, 0.23232E+01_JPRB ,&
&  0.22729E+01_JPRB, 0.22498E+01_JPRB, 0.23099E+01_JPRB, 0.24222E+01_JPRB, 0.28014E+01_JPRB, 0.33491E+01_JPRB, 0.32218E+01_JPRB ,&
&  0.32400E+01_JPRB, 0.35947E+01_JPRB, 0.39081E+01_JPRB, 0.40761E+01_JPRB, 0.39270E+01_JPRB, 0.35931E+01_JPRB, 0.33263E+01_JPRB ,&
&  0.30583E+01_JPRB, 0.26442E+01_JPRB, 0.22663E+01_JPRB, 0.26278E+01_JPRB, 0.27326E+01_JPRB, 0.28465E+01_JPRB, 0.26468E+01_JPRB ,&
&  0.24879E+01_JPRB, 0.22160E+01_JPRB, 0.21945E+01_JPRB, 0.23213E+01_JPRB, 0.23920E+01_JPRB, 0.23093E+01_JPRB, 0.21721E+01_JPRB ,&
&  0.20181E+01_JPRB, 0.19838E+01_JPRB, 0.20881E+01_JPRB, 0.21090E+01_JPRB, 0.21361E+01_JPRB, 0.17557E+01_JPRB, 0.16077E+01_JPRB ,&
&  0.17571E+01_JPRB, 0.20632E+01_JPRB, 0.17522E+01_JPRB, 0.21655E+01_JPRB, 0.11070E+01_JPRB, 0.88466E+00_JPRB, 0.87481E+00_JPRB ,&
&  0.90102E+00_JPRB, 0.93877E+00_JPRB, 0.95318E+00_JPRB, 0.10566E+01_JPRB, 0.10505E+01_JPRB, 0.10545E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10820E+01_JPRB, 0.10943E+01_JPRB, 0.11195E+01_JPRB, 0.11374E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.12199E+01_JPRB, 0.12282E+01_JPRB, 0.12423E+01_JPRB, 0.12628E+01_JPRB, 0.12753E+01_JPRB, 0.12848E+01_JPRB, 0.12866E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.12798E+01_JPRB, 0.12952E+01_JPRB, 0.13176E+01_JPRB, 0.13418E+01_JPRB, 0.13627E+01_JPRB, 0.13911E+01_JPRB ,&
&  0.13982E+01_JPRB, 0.14127E+01_JPRB, 0.14528E+01_JPRB, 0.14932E+01_JPRB, 0.15117E+01_JPRB, 0.15128E+01_JPRB, 0.15212E+01_JPRB ,&
&  0.15117E+01_JPRB, 0.15256E+01_JPRB, 0.15729E+01_JPRB, 0.16060E+01_JPRB, 0.16539E+01_JPRB, 0.17832E+01_JPRB, 0.21899E+01_JPRB ,&
&  0.25512E+01_JPRB, 0.26173E+01_JPRB, 0.21571E+01_JPRB, 0.19320E+01_JPRB, 0.19186E+01_JPRB, 0.18278E+01_JPRB, 0.17454E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.16314E+01_JPRB, 0.15691E+01_JPRB, 0.15747E+01_JPRB, 0.15528E+01_JPRB, 0.15831E+01_JPRB, 0.15830E+01_JPRB ,&
&  0.16485E+01_JPRB, 0.17261E+01_JPRB, 0.18861E+01_JPRB, 0.20250E+01_JPRB, 0.22095E+01_JPRB, 0.23325E+01_JPRB, 0.21177E+01_JPRB ,&
&  0.20906E+01_JPRB, 0.23467E+01_JPRB, 0.27329E+01_JPRB, 0.30325E+01_JPRB, 0.28857E+01_JPRB, 0.26331E+01_JPRB, 0.24363E+01_JPRB ,&
&  0.22506E+01_JPRB, 0.21866E+01_JPRB, 0.26298E+01_JPRB, 0.26778E+01_JPRB, 0.26848E+01_JPRB, 0.24945E+01_JPRB, 0.22153E+01_JPRB ,&
&  0.19054E+01_JPRB, 0.15577E+01_JPRB, 0.15262E+01_JPRB, 0.15237E+01_JPRB, 0.15386E+01_JPRB, 0.14890E+01_JPRB, 0.14721E+01_JPRB ,&
&  0.14246E+01_JPRB, 0.14407E+01_JPRB, 0.17557E+01_JPRB, 0.21447E+01_JPRB, 0.24233E+01_JPRB, 0.30087E+01_JPRB, 0.26080E+01_JPRB ,&
&  0.17789E+01_JPRB, 0.17394E+01_JPRB, 0.10225E+01_JPRB, 0.13930E+01_JPRB, 0.88577E+00_JPRB, 0.81586E+00_JPRB, 0.89113E+00_JPRB ,&
&  0.10081E+01_JPRB, 0.11132E+01_JPRB, 0.11940E+01_JPRB, 0.11728E+01_JPRB, 0.11147E+01_JPRB, 0.11174E+01_JPRB, 0.11265E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.11658E+01_JPRB, 0.11782E+01_JPRB, 0.11910E+01_JPRB, 0.12004E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.12404E+01_JPRB, 0.12597E+01_JPRB, 0.12842E+01_JPRB, 0.13120E+01_JPRB, 0.13367E+01_JPRB, 0.13598E+01_JPRB, 0.13787E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.14167E+01_JPRB, 0.14536E+01_JPRB, 0.14943E+01_JPRB, 0.15180E+01_JPRB, 0.15430E+01_JPRB, 0.15572E+01_JPRB ,&
&  0.15983E+01_JPRB, 0.16490E+01_JPRB, 0.16826E+01_JPRB, 0.17229E+01_JPRB, 0.17481E+01_JPRB, 0.17736E+01_JPRB, 0.18078E+01_JPRB ,&
&  0.18284E+01_JPRB, 0.18867E+01_JPRB, 0.19850E+01_JPRB, 0.20599E+01_JPRB, 0.21814E+01_JPRB, 0.24980E+01_JPRB, 0.23299E+01_JPRB ,&
&  0.27850E+01_JPRB, 0.25986E+01_JPRB, 0.23246E+01_JPRB, 0.20513E+01_JPRB, 0.19697E+01_JPRB, 0.18947E+01_JPRB, 0.18285E+01_JPRB ,&
&  0.17794E+01_JPRB, 0.16884E+01_JPRB, 0.17077E+01_JPRB, 0.16924E+01_JPRB, 0.16733E+01_JPRB, 0.17698E+01_JPRB, 0.18559E+01_JPRB ,&
&  0.19012E+01_JPRB, 0.19524E+01_JPRB, 0.20260E+01_JPRB, 0.20673E+01_JPRB, 0.21388E+01_JPRB, 0.21695E+01_JPRB, 0.20649E+01_JPRB ,&
&  0.19265E+01_JPRB, 0.18655E+01_JPRB, 0.18129E+01_JPRB, 0.18701E+01_JPRB, 0.18485E+01_JPRB, 0.16440E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.16258E+01_JPRB, 0.17158E+01_JPRB, 0.19098E+01_JPRB, 0.27494E+01_JPRB, 0.25946E+01_JPRB, 0.22942E+01_JPRB, 0.18835E+01_JPRB ,&
&  0.15645E+01_JPRB, 0.11991E+01_JPRB, 0.11018E+01_JPRB, 0.11214E+01_JPRB, 0.10887E+01_JPRB, 0.10812E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.11261E+01_JPRB, 0.14333E+01_JPRB, 0.18878E+01_JPRB, 0.24113E+01_JPRB, 0.26073E+01_JPRB, 0.33211E+01_JPRB ,&
&  0.21582E+01_JPRB, 0.12367E+01_JPRB, 0.13223E+01_JPRB, 0.11286E+01_JPRB, 0.95384E+00_JPRB, 0.79004E+00_JPRB, 0.89832E+00_JPRB ,&
&  0.11141E+01_JPRB, 0.11873E+01_JPRB, 0.12179E+01_JPRB, 0.11818E+01_JPRB, 0.11824E+01_JPRB, 0.11967E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.12107E+01_JPRB, 0.12199E+01_JPRB, 0.12215E+01_JPRB, 0.12265E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.12670E+01_JPRB, 0.12951E+01_JPRB, 0.13320E+01_JPRB, 0.13751E+01_JPRB, 0.14046E+01_JPRB, 0.14434E+01_JPRB, 0.14667E+01_JPRB ,&
&  0.14941E+01_JPRB, 0.15421E+01_JPRB, 0.15901E+01_JPRB, 0.16414E+01_JPRB, 0.16948E+01_JPRB, 0.17443E+01_JPRB, 0.17950E+01_JPRB ,&
&  0.18390E+01_JPRB, 0.18895E+01_JPRB, 0.19251E+01_JPRB, 0.19587E+01_JPRB, 0.19692E+01_JPRB, 0.19812E+01_JPRB, 0.19791E+01_JPRB ,&
&  0.19708E+01_JPRB, 0.19860E+01_JPRB, 0.20256E+01_JPRB, 0.20619E+01_JPRB, 0.21447E+01_JPRB, 0.24691E+01_JPRB, 0.20243E+01_JPRB ,&
&  0.26646E+01_JPRB, 0.25748E+01_JPRB, 0.24749E+01_JPRB, 0.22762E+01_JPRB, 0.21912E+01_JPRB, 0.22104E+01_JPRB, 0.22755E+01_JPRB ,&
&  0.23806E+01_JPRB, 0.25121E+01_JPRB, 0.26150E+01_JPRB, 0.24689E+01_JPRB, 0.22808E+01_JPRB, 0.23144E+01_JPRB, 0.23834E+01_JPRB ,&
&  0.24216E+01_JPRB, 0.24319E+01_JPRB, 0.24485E+01_JPRB, 0.24235E+01_JPRB, 0.24022E+01_JPRB, 0.23704E+01_JPRB, 0.22958E+01_JPRB ,&
&  0.21810E+01_JPRB, 0.20265E+01_JPRB, 0.18228E+01_JPRB, 0.19447E+01_JPRB, 0.19516E+01_JPRB, 0.16479E+01_JPRB, 0.11969E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.14385E+01_JPRB, 0.18602E+01_JPRB, 0.26644E+01_JPRB, 0.23766E+01_JPRB, 0.20189E+01_JPRB, 0.17566E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.10683E+01_JPRB, 0.93524E+00_JPRB, 0.94518E+00_JPRB, 0.94142E+00_JPRB, 0.94863E+00_JPRB, 0.93072E+00_JPRB ,&
&  0.91998E+00_JPRB, 0.97534E+00_JPRB, 0.11763E+01_JPRB, 0.14568E+01_JPRB, 0.18547E+01_JPRB, 0.19979E+01_JPRB, 0.31672E+01_JPRB ,&
&  0.19631E+01_JPRB, 0.15494E+01_JPRB, 0.20848E+01_JPRB, 0.82952E+00_JPRB, 0.90390E+00_JPRB, 0.97296E+00_JPRB, 0.10775E+01_JPRB ,&
&  0.10917E+01_JPRB, 0.14092E+01_JPRB, 0.16871E+01_JPRB, 0.15051E+01_JPRB, 0.13820E+01_JPRB, 0.12846E+01_JPRB, 0.12506E+01_JPRB ,&
&  0.12482E+01_JPRB, 0.12466E+01_JPRB, 0.12543E+01_JPRB, 0.12570E+01_JPRB, 0.12559E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.13280E+01_JPRB, 0.13445E+01_JPRB, 0.13513E+01_JPRB, 0.13984E+01_JPRB, 0.14126E+01_JPRB, 0.14443E+01_JPRB, 0.14596E+01_JPRB ,&
&  0.14976E+01_JPRB, 0.15356E+01_JPRB, 0.15834E+01_JPRB, 0.16500E+01_JPRB, 0.17019E+01_JPRB, 0.17497E+01_JPRB, 0.18130E+01_JPRB ,&
&  0.18767E+01_JPRB, 0.19515E+01_JPRB, 0.19806E+01_JPRB, 0.20030E+01_JPRB, 0.20123E+01_JPRB, 0.20435E+01_JPRB, 0.20618E+01_JPRB ,&
&  0.20890E+01_JPRB, 0.21015E+01_JPRB, 0.21487E+01_JPRB, 0.22183E+01_JPRB, 0.23233E+01_JPRB, 0.25647E+01_JPRB, 0.18279E+01_JPRB ,&
&  0.25595E+01_JPRB, 0.27023E+01_JPRB, 0.25989E+01_JPRB, 0.25745E+01_JPRB, 0.24621E+01_JPRB, 0.25319E+01_JPRB, 0.26697E+01_JPRB ,&
&  0.28504E+01_JPRB, 0.30270E+01_JPRB, 0.30207E+01_JPRB, 0.28921E+01_JPRB, 0.28381E+01_JPRB, 0.29132E+01_JPRB, 0.29952E+01_JPRB ,&
&  0.30493E+01_JPRB, 0.30835E+01_JPRB, 0.31268E+01_JPRB, 0.31500E+01_JPRB, 0.31834E+01_JPRB, 0.32290E+01_JPRB, 0.32177E+01_JPRB ,&
&  0.31476E+01_JPRB, 0.30789E+01_JPRB, 0.30939E+01_JPRB, 0.32570E+01_JPRB, 0.32160E+01_JPRB, 0.27524E+01_JPRB, 0.20949E+01_JPRB ,&
&  0.13359E+01_JPRB, 0.11558E+01_JPRB, 0.16866E+01_JPRB, 0.24142E+01_JPRB, 0.21312E+01_JPRB, 0.18119E+01_JPRB, 0.15003E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.10273E+01_JPRB, 0.87944E+00_JPRB, 0.88436E+00_JPRB, 0.90800E+00_JPRB, 0.96882E+00_JPRB, 0.94526E+00_JPRB ,&
&  0.98175E+00_JPRB, 0.10334E+01_JPRB, 0.11072E+01_JPRB, 0.12807E+01_JPRB, 0.16187E+01_JPRB, 0.18898E+01_JPRB, 0.18037E+01_JPRB ,&
&  0.20673E+01_JPRB, 0.14605E+01_JPRB, 0.13161E+01_JPRB, 0.13879E+01_JPRB, 0.11894E+01_JPRB, 0.13713E+01_JPRB, 0.16676E+01_JPRB ,&
&  0.21432E+01_JPRB, 0.20279E+01_JPRB, 0.26097E+01_JPRB, 0.39242E+01_JPRB, 0.30920E+01_JPRB, 0.22657E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.13122E+01_JPRB, 0.12996E+01_JPRB, 0.13234E+01_JPRB, 0.13221E+01_JPRB, 0.13226E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.12384E+01_JPRB, 0.12318E+01_JPRB, 0.12454E+01_JPRB, 0.12568E+01_JPRB, 0.12600E+01_JPRB, 0.12795E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.13370E+01_JPRB, 0.13914E+01_JPRB, 0.14787E+01_JPRB, 0.15534E+01_JPRB, 0.16519E+01_JPRB, 0.17079E+01_JPRB, 0.17247E+01_JPRB ,&
&  0.17628E+01_JPRB, 0.18489E+01_JPRB, 0.19307E+01_JPRB, 0.19915E+01_JPRB, 0.19998E+01_JPRB, 0.20468E+01_JPRB, 0.21602E+01_JPRB ,&
&  0.23101E+01_JPRB, 0.24215E+01_JPRB, 0.25226E+01_JPRB, 0.26538E+01_JPRB, 0.27316E+01_JPRB, 0.26691E+01_JPRB, 0.18913E+01_JPRB ,&
&  0.24866E+01_JPRB, 0.27306E+01_JPRB, 0.28314E+01_JPRB, 0.28275E+01_JPRB, 0.28521E+01_JPRB, 0.27742E+01_JPRB, 0.29091E+01_JPRB ,&
&  0.31014E+01_JPRB, 0.32488E+01_JPRB, 0.31042E+01_JPRB, 0.28658E+01_JPRB, 0.31864E+01_JPRB, 0.32589E+01_JPRB, 0.34062E+01_JPRB ,&
&  0.35261E+01_JPRB, 0.36393E+01_JPRB, 0.37514E+01_JPRB, 0.38892E+01_JPRB, 0.40041E+01_JPRB, 0.41074E+01_JPRB, 0.41865E+01_JPRB ,&
&  0.42015E+01_JPRB, 0.41890E+01_JPRB, 0.42722E+01_JPRB, 0.40917E+01_JPRB, 0.39809E+01_JPRB, 0.41030E+01_JPRB, 0.34593E+01_JPRB ,&
&  0.19568E+01_JPRB, 0.13017E+01_JPRB, 0.17349E+01_JPRB, 0.21107E+01_JPRB, 0.18183E+01_JPRB, 0.15570E+01_JPRB, 0.13700E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.97406E+00_JPRB, 0.92682E+00_JPRB, 0.98283E+00_JPRB, 0.10772E+01_JPRB, 0.11687E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.11840E+01_JPRB, 0.12333E+01_JPRB, 0.13411E+01_JPRB, 0.15609E+01_JPRB, 0.19737E+01_JPRB, 0.25270E+01_JPRB ,&
&  0.23764E+01_JPRB, 0.15240E+01_JPRB, 0.14607E+01_JPRB, 0.14535E+01_JPRB, 0.14886E+01_JPRB, 0.16147E+01_JPRB, 0.19183E+01_JPRB ,&
&  0.24060E+01_JPRB, 0.29121E+01_JPRB, 0.27665E+01_JPRB, 0.23014E+01_JPRB, 0.28367E+01_JPRB, 0.19580E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.11896E+01_JPRB, 0.12228E+01_JPRB, 0.12338E+01_JPRB, 0.12397E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.10629E+01_JPRB, 0.10486E+01_JPRB, 0.10417E+01_JPRB, 0.10362E+01_JPRB, 0.10522E+01_JPRB, 0.10970E+01_JPRB, 0.11520E+01_JPRB ,&
&  0.12183E+01_JPRB, 0.12944E+01_JPRB, 0.13913E+01_JPRB, 0.14619E+01_JPRB, 0.15445E+01_JPRB, 0.15889E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.16512E+01_JPRB, 0.17157E+01_JPRB, 0.18043E+01_JPRB, 0.18935E+01_JPRB, 0.19771E+01_JPRB, 0.21025E+01_JPRB, 0.22598E+01_JPRB ,&
&  0.24540E+01_JPRB, 0.26277E+01_JPRB, 0.28157E+01_JPRB, 0.30757E+01_JPRB, 0.32328E+01_JPRB, 0.33489E+01_JPRB, 0.35004E+01_JPRB ,&
&  0.18626E+01_JPRB, 0.25837E+01_JPRB, 0.27730E+01_JPRB, 0.28284E+01_JPRB, 0.31170E+01_JPRB, 0.32345E+01_JPRB, 0.33134E+01_JPRB ,&
&  0.34609E+01_JPRB, 0.33663E+01_JPRB, 0.31915E+01_JPRB, 0.30137E+01_JPRB, 0.32190E+01_JPRB, 0.33349E+01_JPRB, 0.34615E+01_JPRB ,&
&  0.35196E+01_JPRB, 0.35796E+01_JPRB, 0.37226E+01_JPRB, 0.38937E+01_JPRB, 0.40670E+01_JPRB, 0.42395E+01_JPRB, 0.44569E+01_JPRB ,&
&  0.46114E+01_JPRB, 0.47379E+01_JPRB, 0.49839E+01_JPRB, 0.45484E+01_JPRB, 0.42430E+01_JPRB, 0.45134E+01_JPRB, 0.41278E+01_JPRB ,&
&  0.28071E+01_JPRB, 0.15914E+01_JPRB, 0.16757E+01_JPRB, 0.19836E+01_JPRB, 0.16274E+01_JPRB, 0.13829E+01_JPRB, 0.12615E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.99023E+00_JPRB, 0.10635E+01_JPRB, 0.11617E+01_JPRB, 0.12759E+01_JPRB, 0.13760E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.13368E+01_JPRB, 0.13152E+01_JPRB, 0.12699E+01_JPRB, 0.12809E+01_JPRB, 0.13227E+01_JPRB, 0.13761E+01_JPRB, 0.14049E+01_JPRB ,&
&  0.14144E+01_JPRB, 0.13738E+01_JPRB, 0.13764E+01_JPRB, 0.14099E+01_JPRB, 0.14154E+01_JPRB, 0.13799E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.17306E+01_JPRB, 0.19404E+01_JPRB, 0.21797E+01_JPRB, 0.20912E+01_JPRB, 0.18584E+01_JPRB, 0.13760E+01_JPRB, 0.97698E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10371E+01_JPRB, 0.10491E+01_JPRB, 0.10734E+01_JPRB, 0.10699E+01_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.95281E+00_JPRB, 0.94636E+00_JPRB, 0.94131E+00_JPRB, 0.95276E+00_JPRB, 0.96808E+00_JPRB, 0.10286E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.12360E+01_JPRB, 0.13116E+01_JPRB, 0.13807E+01_JPRB, 0.14356E+01_JPRB, 0.14739E+01_JPRB, 0.14837E+01_JPRB, 0.15154E+01_JPRB ,&
&  0.15577E+01_JPRB, 0.16012E+01_JPRB, 0.16788E+01_JPRB, 0.17847E+01_JPRB, 0.18962E+01_JPRB, 0.20234E+01_JPRB, 0.21767E+01_JPRB ,&
&  0.23663E+01_JPRB, 0.26153E+01_JPRB, 0.28950E+01_JPRB, 0.32696E+01_JPRB, 0.36286E+01_JPRB, 0.39406E+01_JPRB, 0.41048E+01_JPRB ,&
&  0.18913E+01_JPRB, 0.19966E+01_JPRB, 0.27601E+01_JPRB, 0.27255E+01_JPRB, 0.30160E+01_JPRB, 0.33790E+01_JPRB, 0.37580E+01_JPRB ,&
&  0.35579E+01_JPRB, 0.30333E+01_JPRB, 0.28951E+01_JPRB, 0.31352E+01_JPRB, 0.30735E+01_JPRB, 0.31792E+01_JPRB, 0.32732E+01_JPRB ,&
&  0.32936E+01_JPRB, 0.32861E+01_JPRB, 0.33307E+01_JPRB, 0.34570E+01_JPRB, 0.36451E+01_JPRB, 0.38468E+01_JPRB, 0.40907E+01_JPRB ,&
&  0.43438E+01_JPRB, 0.46336E+01_JPRB, 0.49216E+01_JPRB, 0.41769E+01_JPRB, 0.44585E+01_JPRB, 0.48626E+01_JPRB, 0.52135E+01_JPRB ,&
&  0.39520E+01_JPRB, 0.23255E+01_JPRB, 0.19380E+01_JPRB, 0.21032E+01_JPRB, 0.16894E+01_JPRB, 0.15679E+01_JPRB, 0.13565E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.11982E+01_JPRB, 0.12598E+01_JPRB, 0.13593E+01_JPRB, 0.14668E+01_JPRB, 0.15551E+01_JPRB, 0.15778E+01_JPRB ,&
&  0.14996E+01_JPRB, 0.14492E+01_JPRB, 0.14241E+01_JPRB, 0.14294E+01_JPRB, 0.14500E+01_JPRB, 0.14649E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.14504E+01_JPRB, 0.14412E+01_JPRB, 0.14711E+01_JPRB, 0.14866E+01_JPRB, 0.15005E+01_JPRB, 0.15082E+01_JPRB, 0.15146E+01_JPRB ,&
&  0.14361E+01_JPRB, 0.13717E+01_JPRB, 0.14293E+01_JPRB, 0.13025E+01_JPRB, 0.11424E+01_JPRB, 0.99208E+00_JPRB, 0.85898E+00_JPRB ,&
&  0.88787E+00_JPRB, 0.90396E+00_JPRB, 0.90872E+00_JPRB, 0.92398E+00_JPRB, 0.94486E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.84938E+00_JPRB, 0.85714E+00_JPRB, 0.85187E+00_JPRB, 0.87742E+00_JPRB, 0.88443E+00_JPRB, 0.97058E+00_JPRB, 0.10548E+01_JPRB ,&
&  0.11905E+01_JPRB, 0.12965E+01_JPRB, 0.13717E+01_JPRB, 0.14105E+01_JPRB, 0.14085E+01_JPRB, 0.13890E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.14115E+01_JPRB, 0.14422E+01_JPRB, 0.14816E+01_JPRB, 0.15392E+01_JPRB, 0.16080E+01_JPRB, 0.17020E+01_JPRB, 0.18074E+01_JPRB ,&
&  0.19669E+01_JPRB, 0.22021E+01_JPRB, 0.25180E+01_JPRB, 0.29686E+01_JPRB, 0.35747E+01_JPRB, 0.43976E+01_JPRB, 0.52774E+01_JPRB ,&
&  0.42847E+01_JPRB, 0.20506E+01_JPRB, 0.30491E+01_JPRB, 0.27597E+01_JPRB, 0.25434E+01_JPRB, 0.30737E+01_JPRB, 0.36266E+01_JPRB ,&
&  0.33445E+01_JPRB, 0.29161E+01_JPRB, 0.26669E+01_JPRB, 0.29190E+01_JPRB, 0.29018E+01_JPRB, 0.29841E+01_JPRB, 0.30450E+01_JPRB ,&
&  0.30860E+01_JPRB, 0.30854E+01_JPRB, 0.30881E+01_JPRB, 0.31908E+01_JPRB, 0.33664E+01_JPRB, 0.35924E+01_JPRB, 0.38342E+01_JPRB ,&
&  0.41181E+01_JPRB, 0.44433E+01_JPRB, 0.48384E+01_JPRB, 0.41707E+01_JPRB, 0.46666E+01_JPRB, 0.47088E+01_JPRB, 0.41386E+01_JPRB ,&
&  0.38634E+01_JPRB, 0.26557E+01_JPRB, 0.21010E+01_JPRB, 0.22445E+01_JPRB, 0.19205E+01_JPRB, 0.14912E+01_JPRB, 0.14786E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.14050E+01_JPRB, 0.14102E+01_JPRB, 0.15201E+01_JPRB, 0.15888E+01_JPRB, 0.16404E+01_JPRB, 0.16299E+01_JPRB ,&
&  0.16206E+01_JPRB, 0.15698E+01_JPRB, 0.15510E+01_JPRB, 0.15568E+01_JPRB, 0.15633E+01_JPRB, 0.15908E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.15854E+01_JPRB, 0.16029E+01_JPRB, 0.16556E+01_JPRB, 0.17076E+01_JPRB, 0.17192E+01_JPRB, 0.16209E+01_JPRB, 0.15322E+01_JPRB ,&
&  0.14660E+01_JPRB, 0.14017E+01_JPRB, 0.12543E+01_JPRB, 0.10963E+01_JPRB, 0.10205E+01_JPRB, 0.92488E+00_JPRB, 0.84942E+00_JPRB ,&
&  0.85957E+00_JPRB, 0.85770E+00_JPRB, 0.84708E+00_JPRB, 0.85027E+00_JPRB, 0.85143E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.92536E+00_JPRB, 0.90587E+00_JPRB, 0.87056E+00_JPRB, 0.85460E+00_JPRB, 0.85551E+00_JPRB, 0.89429E+00_JPRB, 0.94265E+00_JPRB ,&
&  0.10179E+01_JPRB, 0.11242E+01_JPRB, 0.12168E+01_JPRB, 0.12719E+01_JPRB, 0.12729E+01_JPRB, 0.12901E+01_JPRB, 0.12805E+01_JPRB ,&
&  0.12806E+01_JPRB, 0.13066E+01_JPRB, 0.13361E+01_JPRB, 0.13531E+01_JPRB, 0.13827E+01_JPRB, 0.14238E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.14969E+01_JPRB, 0.15416E+01_JPRB, 0.16183E+01_JPRB, 0.18632E+01_JPRB, 0.23011E+01_JPRB, 0.29873E+01_JPRB, 0.42368E+01_JPRB ,&
&  0.67671E+01_JPRB, 0.70371E+01_JPRB, 0.33446E+01_JPRB, 0.35814E+01_JPRB, 0.30384E+01_JPRB, 0.32810E+01_JPRB, 0.31232E+01_JPRB ,&
&  0.32337E+01_JPRB, 0.28568E+01_JPRB, 0.25368E+01_JPRB, 0.26611E+01_JPRB, 0.26486E+01_JPRB, 0.27179E+01_JPRB, 0.27745E+01_JPRB ,&
&  0.28715E+01_JPRB, 0.29154E+01_JPRB, 0.28892E+01_JPRB, 0.29144E+01_JPRB, 0.30552E+01_JPRB, 0.32617E+01_JPRB, 0.35084E+01_JPRB ,&
&  0.38204E+01_JPRB, 0.41163E+01_JPRB, 0.45211E+01_JPRB, 0.41845E+01_JPRB, 0.46429E+01_JPRB, 0.42959E+01_JPRB, 0.34622E+01_JPRB ,&
&  0.29230E+01_JPRB, 0.26168E+01_JPRB, 0.24665E+01_JPRB, 0.24326E+01_JPRB, 0.21606E+01_JPRB, 0.16724E+01_JPRB, 0.16337E+01_JPRB ,&
&  0.15584E+01_JPRB, 0.15264E+01_JPRB, 0.15222E+01_JPRB, 0.15628E+01_JPRB, 0.16304E+01_JPRB, 0.16592E+01_JPRB, 0.16734E+01_JPRB ,&
&  0.16625E+01_JPRB, 0.16290E+01_JPRB, 0.16168E+01_JPRB, 0.16169E+01_JPRB, 0.16230E+01_JPRB, 0.16519E+01_JPRB, 0.16664E+01_JPRB ,&
&  0.16814E+01_JPRB, 0.17177E+01_JPRB, 0.17753E+01_JPRB, 0.18292E+01_JPRB, 0.17904E+01_JPRB, 0.16848E+01_JPRB, 0.15655E+01_JPRB ,&
&  0.15201E+01_JPRB, 0.15107E+01_JPRB, 0.14001E+01_JPRB, 0.13125E+01_JPRB, 0.12082E+01_JPRB, 0.11546E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10692E+01_JPRB, 0.10259E+01_JPRB, 0.91910E+00_JPRB, 0.91982E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.10439E+01_JPRB, 0.99295E+00_JPRB, 0.99078E+00_JPRB, 0.98960E+00_JPRB, 0.98155E+00_JPRB, 0.98907E+00_JPRB, 0.93788E+00_JPRB ,&
&  0.92130E+00_JPRB, 0.96722E+00_JPRB, 0.10398E+01_JPRB, 0.10864E+01_JPRB, 0.11172E+01_JPRB, 0.11258E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11767E+01_JPRB, 0.12101E+01_JPRB, 0.12334E+01_JPRB, 0.12650E+01_JPRB, 0.12908E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.13479E+01_JPRB, 0.13175E+01_JPRB, 0.12659E+01_JPRB, 0.12683E+01_JPRB, 0.14274E+01_JPRB, 0.17076E+01_JPRB, 0.21562E+01_JPRB ,&
&  0.38013E+01_JPRB, 0.86104E+01_JPRB, 0.39142E+01_JPRB, 0.39259E+01_JPRB, 0.37991E+01_JPRB, 0.36022E+01_JPRB, 0.33482E+01_JPRB ,&
&  0.33600E+01_JPRB, 0.28178E+01_JPRB, 0.23684E+01_JPRB, 0.23590E+01_JPRB, 0.23294E+01_JPRB, 0.23461E+01_JPRB, 0.24061E+01_JPRB ,&
&  0.24872E+01_JPRB, 0.25552E+01_JPRB, 0.26007E+01_JPRB, 0.25771E+01_JPRB, 0.26713E+01_JPRB, 0.28145E+01_JPRB, 0.30264E+01_JPRB ,&
&  0.32893E+01_JPRB, 0.36398E+01_JPRB, 0.41123E+01_JPRB, 0.40545E+01_JPRB, 0.44002E+01_JPRB, 0.41376E+01_JPRB, 0.33998E+01_JPRB ,&
&  0.29628E+01_JPRB, 0.28838E+01_JPRB, 0.26615E+01_JPRB, 0.25249E+01_JPRB, 0.22919E+01_JPRB, 0.19533E+01_JPRB, 0.19230E+01_JPRB ,&
&  0.17233E+01_JPRB, 0.16465E+01_JPRB, 0.16392E+01_JPRB, 0.16358E+01_JPRB, 0.16616E+01_JPRB, 0.16721E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.16709E+01_JPRB, 0.16413E+01_JPRB, 0.16305E+01_JPRB, 0.16458E+01_JPRB, 0.16553E+01_JPRB, 0.16565E+01_JPRB, 0.16781E+01_JPRB ,&
&  0.16984E+01_JPRB, 0.17443E+01_JPRB, 0.18183E+01_JPRB, 0.18261E+01_JPRB, 0.17786E+01_JPRB, 0.16669E+01_JPRB, 0.15545E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.15326E+01_JPRB, 0.14607E+01_JPRB, 0.13936E+01_JPRB, 0.13705E+01_JPRB, 0.13141E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.13939E+01_JPRB, 0.14765E+01_JPRB, 0.15310E+01_JPRB, 0.10731E+01_JPRB, 0.10695E+01_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.11223E+01_JPRB, 0.10714E+01_JPRB, 0.10748E+01_JPRB, 0.11118E+01_JPRB, 0.11181E+01_JPRB, 0.10951E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.96405E+00_JPRB, 0.94922E+00_JPRB, 0.96549E+00_JPRB, 0.99097E+00_JPRB, 0.10151E+01_JPRB, 0.10386E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.10626E+01_JPRB, 0.10872E+01_JPRB, 0.11248E+01_JPRB, 0.11613E+01_JPRB, 0.11957E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.12440E+01_JPRB, 0.12294E+01_JPRB, 0.11993E+01_JPRB, 0.11906E+01_JPRB, 0.12158E+01_JPRB, 0.13050E+01_JPRB, 0.15163E+01_JPRB ,&
&  0.23931E+01_JPRB, 0.72254E+01_JPRB, 0.65040E+01_JPRB, 0.49656E+01_JPRB, 0.39941E+01_JPRB, 0.35021E+01_JPRB, 0.33070E+01_JPRB ,&
&  0.36504E+01_JPRB, 0.36468E+01_JPRB, 0.30525E+01_JPRB, 0.23992E+01_JPRB, 0.21558E+01_JPRB, 0.21175E+01_JPRB, 0.21324E+01_JPRB ,&
&  0.21279E+01_JPRB, 0.21524E+01_JPRB, 0.21989E+01_JPRB, 0.22045E+01_JPRB, 0.22231E+01_JPRB, 0.23453E+01_JPRB, 0.25020E+01_JPRB ,&
&  0.26642E+01_JPRB, 0.29803E+01_JPRB, 0.35613E+01_JPRB, 0.41369E+01_JPRB, 0.41528E+01_JPRB, 0.42509E+01_JPRB, 0.40311E+01_JPRB ,&
&  0.41788E+01_JPRB, 0.37044E+01_JPRB, 0.30123E+01_JPRB, 0.25150E+01_JPRB, 0.21922E+01_JPRB, 0.22638E+01_JPRB, 0.21431E+01_JPRB ,&
&  0.18583E+01_JPRB, 0.17274E+01_JPRB, 0.16775E+01_JPRB, 0.16728E+01_JPRB, 0.16798E+01_JPRB, 0.16832E+01_JPRB, 0.16530E+01_JPRB ,&
&  0.16310E+01_JPRB, 0.16217E+01_JPRB, 0.16266E+01_JPRB, 0.16485E+01_JPRB, 0.16559E+01_JPRB, 0.16568E+01_JPRB, 0.16725E+01_JPRB ,&
&  0.16733E+01_JPRB, 0.16989E+01_JPRB, 0.17134E+01_JPRB, 0.17672E+01_JPRB, 0.17262E+01_JPRB, 0.16104E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.14669E+01_JPRB, 0.14933E+01_JPRB, 0.14861E+01_JPRB, 0.14592E+01_JPRB, 0.14307E+01_JPRB, 0.13946E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.13396E+01_JPRB, 0.12418E+01_JPRB, 0.11620E+01_JPRB, 0.11639E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.12119E+01_JPRB, 0.11795E+01_JPRB, 0.11582E+01_JPRB, 0.11691E+01_JPRB, 0.11792E+01_JPRB, 0.11615E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10369E+01_JPRB, 0.10362E+01_JPRB, 0.10255E+01_JPRB, 0.10180E+01_JPRB, 0.10193E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.10398E+01_JPRB, 0.10577E+01_JPRB, 0.10525E+01_JPRB, 0.10576E+01_JPRB, 0.10821E+01_JPRB, 0.11160E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.11583E+01_JPRB, 0.11478E+01_JPRB, 0.11453E+01_JPRB, 0.11438E+01_JPRB, 0.11843E+01_JPRB, 0.13053E+01_JPRB ,&
&  0.18751E+01_JPRB, 0.57226E+01_JPRB, 0.36853E+01_JPRB, 0.51143E+01_JPRB, 0.39341E+01_JPRB, 0.34229E+01_JPRB, 0.28706E+01_JPRB ,&
&  0.31260E+01_JPRB, 0.37610E+01_JPRB, 0.28862E+01_JPRB, 0.23462E+01_JPRB, 0.20772E+01_JPRB, 0.19890E+01_JPRB, 0.19495E+01_JPRB ,&
&  0.19246E+01_JPRB, 0.19350E+01_JPRB, 0.19396E+01_JPRB, 0.19763E+01_JPRB, 0.19683E+01_JPRB, 0.20269E+01_JPRB, 0.21200E+01_JPRB ,&
&  0.21969E+01_JPRB, 0.23453E+01_JPRB, 0.27394E+01_JPRB, 0.32378E+01_JPRB, 0.42080E+01_JPRB, 0.47363E+01_JPRB, 0.49819E+01_JPRB ,&
&  0.51619E+01_JPRB, 0.44786E+01_JPRB, 0.36083E+01_JPRB, 0.28531E+01_JPRB, 0.23911E+01_JPRB, 0.23185E+01_JPRB, 0.22844E+01_JPRB ,&
&  0.20434E+01_JPRB, 0.18651E+01_JPRB, 0.17635E+01_JPRB, 0.17173E+01_JPRB, 0.17228E+01_JPRB, 0.17240E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.16235E+01_JPRB, 0.16194E+01_JPRB, 0.16385E+01_JPRB, 0.16612E+01_JPRB, 0.16578E+01_JPRB, 0.16340E+01_JPRB, 0.16336E+01_JPRB ,&
&  0.16402E+01_JPRB, 0.16473E+01_JPRB, 0.16452E+01_JPRB, 0.17345E+01_JPRB, 0.16865E+01_JPRB, 0.15996E+01_JPRB, 0.15069E+01_JPRB ,&
&  0.14832E+01_JPRB, 0.14793E+01_JPRB, 0.14678E+01_JPRB, 0.14444E+01_JPRB, 0.14664E+01_JPRB, 0.14895E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.13539E+01_JPRB, 0.13301E+01_JPRB, 0.12936E+01_JPRB, 0.12402E+01_JPRB, 0.12376E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.12454E+01_JPRB, 0.12231E+01_JPRB, 0.12276E+01_JPRB, 0.12280E+01_JPRB, 0.12175E+01_JPRB, 0.11877E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.11353E+01_JPRB, 0.11135E+01_JPRB, 0.10844E+01_JPRB, 0.10680E+01_JPRB, 0.10521E+01_JPRB, 0.10498E+01_JPRB ,&
&  0.10598E+01_JPRB, 0.10663E+01_JPRB, 0.10591E+01_JPRB, 0.10584E+01_JPRB, 0.10491E+01_JPRB, 0.10596E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11061E+01_JPRB, 0.11161E+01_JPRB, 0.11164E+01_JPRB, 0.11118E+01_JPRB, 0.11195E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.16854E+01_JPRB, 0.61001E+01_JPRB, 0.33288E+01_JPRB, 0.51425E+01_JPRB, 0.38507E+01_JPRB, 0.33809E+01_JPRB, 0.27483E+01_JPRB ,&
&  0.29110E+01_JPRB, 0.34090E+01_JPRB, 0.27682E+01_JPRB, 0.22440E+01_JPRB, 0.20556E+01_JPRB, 0.19445E+01_JPRB, 0.18351E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.18078E+01_JPRB, 0.18121E+01_JPRB, 0.18398E+01_JPRB, 0.18637E+01_JPRB, 0.18612E+01_JPRB, 0.18901E+01_JPRB ,&
&  0.18995E+01_JPRB, 0.19395E+01_JPRB, 0.21686E+01_JPRB, 0.30207E+01_JPRB, 0.37864E+01_JPRB, 0.50396E+01_JPRB, 0.63602E+01_JPRB ,&
&  0.61507E+01_JPRB, 0.53234E+01_JPRB, 0.38854E+01_JPRB, 0.32043E+01_JPRB, 0.27134E+01_JPRB, 0.24952E+01_JPRB, 0.23711E+01_JPRB ,&
&  0.22355E+01_JPRB, 0.21037E+01_JPRB, 0.19301E+01_JPRB, 0.18458E+01_JPRB, 0.18218E+01_JPRB, 0.18031E+01_JPRB, 0.17621E+01_JPRB ,&
&  0.17129E+01_JPRB, 0.17051E+01_JPRB, 0.17151E+01_JPRB, 0.17193E+01_JPRB, 0.17091E+01_JPRB, 0.16685E+01_JPRB, 0.16458E+01_JPRB ,&
&  0.16569E+01_JPRB, 0.16500E+01_JPRB, 0.16543E+01_JPRB, 0.17780E+01_JPRB, 0.17261E+01_JPRB, 0.16123E+01_JPRB, 0.15345E+01_JPRB ,&
&  0.15291E+01_JPRB, 0.14910E+01_JPRB, 0.14779E+01_JPRB, 0.14819E+01_JPRB, 0.15047E+01_JPRB, 0.15918E+01_JPRB, 0.14727E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.13447E+01_JPRB, 0.13072E+01_JPRB, 0.12774E+01_JPRB, 0.12619E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.13043E+01_JPRB, 0.12842E+01_JPRB, 0.12839E+01_JPRB, 0.12998E+01_JPRB, 0.12777E+01_JPRB, 0.12401E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.12093E+01_JPRB, 0.11915E+01_JPRB, 0.11672E+01_JPRB, 0.11269E+01_JPRB, 0.11092E+01_JPRB, 0.10982E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.10944E+01_JPRB, 0.10737E+01_JPRB, 0.10618E+01_JPRB, 0.10567E+01_JPRB, 0.10494E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.10652E+01_JPRB, 0.10881E+01_JPRB, 0.10921E+01_JPRB, 0.10974E+01_JPRB, 0.10881E+01_JPRB, 0.11415E+01_JPRB ,&
&  0.14696E+01_JPRB, 0.21592E+01_JPRB, 0.40048E+01_JPRB, 0.47057E+01_JPRB, 0.38093E+01_JPRB, 0.33825E+01_JPRB, 0.30908E+01_JPRB ,&
&  0.30396E+01_JPRB, 0.30271E+01_JPRB, 0.26255E+01_JPRB, 0.22827E+01_JPRB, 0.20894E+01_JPRB, 0.19218E+01_JPRB, 0.17743E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.16381E+01_JPRB, 0.16623E+01_JPRB, 0.17156E+01_JPRB, 0.17514E+01_JPRB, 0.17594E+01_JPRB, 0.17259E+01_JPRB ,&
&  0.16903E+01_JPRB, 0.16699E+01_JPRB, 0.17568E+01_JPRB, 0.23256E+01_JPRB, 0.36465E+01_JPRB, 0.52058E+01_JPRB, 0.62635E+01_JPRB ,&
&  0.63909E+01_JPRB, 0.53445E+01_JPRB, 0.39499E+01_JPRB, 0.33721E+01_JPRB, 0.29604E+01_JPRB, 0.26478E+01_JPRB, 0.23914E+01_JPRB ,&
&  0.22411E+01_JPRB, 0.21799E+01_JPRB, 0.20945E+01_JPRB, 0.19862E+01_JPRB, 0.18882E+01_JPRB, 0.18336E+01_JPRB, 0.17941E+01_JPRB ,&
&  0.17600E+01_JPRB, 0.17692E+01_JPRB, 0.17945E+01_JPRB, 0.17679E+01_JPRB, 0.17314E+01_JPRB, 0.16831E+01_JPRB, 0.16567E+01_JPRB ,&
&  0.16700E+01_JPRB, 0.16889E+01_JPRB, 0.17056E+01_JPRB, 0.17746E+01_JPRB, 0.17366E+01_JPRB, 0.16737E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.15423E+01_JPRB, 0.14906E+01_JPRB, 0.14670E+01_JPRB, 0.15220E+01_JPRB, 0.15248E+01_JPRB, 0.16235E+01_JPRB, 0.15627E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.14021E+01_JPRB, 0.13633E+01_JPRB, 0.13394E+01_JPRB, 0.13286E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.13679E+01_JPRB, 0.13541E+01_JPRB, 0.13619E+01_JPRB, 0.13737E+01_JPRB, 0.13594E+01_JPRB, 0.13193E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.12460E+01_JPRB, 0.12136E+01_JPRB, 0.11762E+01_JPRB, 0.11340E+01_JPRB, 0.11195E+01_JPRB, 0.11270E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11306E+01_JPRB, 0.11051E+01_JPRB, 0.10776E+01_JPRB, 0.10678E+01_JPRB, 0.10624E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.10565E+01_JPRB, 0.10637E+01_JPRB, 0.10714E+01_JPRB, 0.10792E+01_JPRB, 0.10725E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.30226E+01_JPRB, 0.36457E+01_JPRB, 0.38030E+01_JPRB, 0.35972E+01_JPRB, 0.32475E+01_JPRB, 0.30787E+01_JPRB ,&
&  0.27901E+01_JPRB, 0.26767E+01_JPRB, 0.25318E+01_JPRB, 0.23627E+01_JPRB, 0.21684E+01_JPRB, 0.19711E+01_JPRB, 0.18014E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.16140E+01_JPRB, 0.15909E+01_JPRB, 0.16042E+01_JPRB, 0.16294E+01_JPRB, 0.16532E+01_JPRB, 0.16257E+01_JPRB ,&
&  0.15812E+01_JPRB, 0.15407E+01_JPRB, 0.15363E+01_JPRB, 0.18148E+01_JPRB, 0.28587E+01_JPRB, 0.35273E+01_JPRB, 0.50699E+01_JPRB ,&
&  0.55993E+01_JPRB, 0.42505E+01_JPRB, 0.34180E+01_JPRB, 0.30268E+01_JPRB, 0.27438E+01_JPRB, 0.24617E+01_JPRB, 0.22509E+01_JPRB ,&
&  0.21506E+01_JPRB, 0.20878E+01_JPRB, 0.20653E+01_JPRB, 0.19982E+01_JPRB, 0.18713E+01_JPRB, 0.17534E+01_JPRB, 0.16975E+01_JPRB ,&
&  0.16667E+01_JPRB, 0.16423E+01_JPRB, 0.16439E+01_JPRB, 0.16597E+01_JPRB, 0.16553E+01_JPRB, 0.16085E+01_JPRB, 0.15825E+01_JPRB ,&
&  0.15853E+01_JPRB, 0.16234E+01_JPRB, 0.16776E+01_JPRB, 0.16919E+01_JPRB, 0.17021E+01_JPRB, 0.17152E+01_JPRB, 0.16443E+01_JPRB ,&
&  0.15355E+01_JPRB, 0.14787E+01_JPRB, 0.14628E+01_JPRB, 0.15015E+01_JPRB, 0.15757E+01_JPRB, 0.16796E+01_JPRB, 0.16079E+01_JPRB ,&
&  0.15145E+01_JPRB, 0.14413E+01_JPRB, 0.14047E+01_JPRB, 0.13804E+01_JPRB, 0.13905E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.14003E+01_JPRB, 0.13845E+01_JPRB, 0.13892E+01_JPRB, 0.13933E+01_JPRB, 0.13805E+01_JPRB, 0.13414E+01_JPRB, 0.13058E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.12310E+01_JPRB, 0.11808E+01_JPRB, 0.11433E+01_JPRB, 0.11393E+01_JPRB, 0.11470E+01_JPRB, 0.11520E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.11387E+01_JPRB, 0.11179E+01_JPRB, 0.10954E+01_JPRB, 0.10818E+01_JPRB, 0.10684E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10412E+01_JPRB, 0.10460E+01_JPRB, 0.10566E+01_JPRB, 0.10560E+01_JPRB, 0.10605E+01_JPRB, 0.10698E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.19388E+01_JPRB, 0.22654E+01_JPRB, 0.28699E+01_JPRB, 0.31233E+01_JPRB, 0.29821E+01_JPRB, 0.28129E+01_JPRB ,&
&  0.25898E+01_JPRB, 0.24345E+01_JPRB, 0.23500E+01_JPRB, 0.22529E+01_JPRB, 0.20983E+01_JPRB, 0.19292E+01_JPRB, 0.17778E+01_JPRB ,&
&  0.16712E+01_JPRB, 0.16107E+01_JPRB, 0.15905E+01_JPRB, 0.15679E+01_JPRB, 0.15467E+01_JPRB, 0.15500E+01_JPRB, 0.15543E+01_JPRB ,&
&  0.14985E+01_JPRB, 0.14538E+01_JPRB, 0.14245E+01_JPRB, 0.14770E+01_JPRB, 0.19290E+01_JPRB, 0.25308E+01_JPRB, 0.33563E+01_JPRB ,&
&  0.31802E+01_JPRB, 0.28216E+01_JPRB, 0.25873E+01_JPRB, 0.24079E+01_JPRB, 0.22764E+01_JPRB, 0.21049E+01_JPRB, 0.19557E+01_JPRB ,&
&  0.18255E+01_JPRB, 0.17638E+01_JPRB, 0.17098E+01_JPRB, 0.16762E+01_JPRB, 0.16342E+01_JPRB, 0.16041E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.14720E+01_JPRB, 0.14657E+01_JPRB, 0.14892E+01_JPRB, 0.15291E+01_JPRB, 0.15402E+01_JPRB, 0.15418E+01_JPRB ,&
&  0.15559E+01_JPRB, 0.15700E+01_JPRB, 0.16176E+01_JPRB, 0.16725E+01_JPRB, 0.16486E+01_JPRB, 0.16063E+01_JPRB, 0.15748E+01_JPRB ,&
&  0.15463E+01_JPRB, 0.15039E+01_JPRB, 0.14562E+01_JPRB, 0.15048E+01_JPRB, 0.15487E+01_JPRB, 0.16430E+01_JPRB, 0.16069E+01_JPRB ,&
&  0.15344E+01_JPRB, 0.14477E+01_JPRB, 0.14080E+01_JPRB, 0.13870E+01_JPRB, 0.14036E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.14090E+01_JPRB, 0.14125E+01_JPRB, 0.14049E+01_JPRB, 0.13970E+01_JPRB, 0.13836E+01_JPRB, 0.13505E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.12446E+01_JPRB, 0.11932E+01_JPRB, 0.11613E+01_JPRB, 0.11486E+01_JPRB, 0.11438E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.11296E+01_JPRB, 0.11182E+01_JPRB, 0.11124E+01_JPRB, 0.10988E+01_JPRB, 0.10862E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10245E+01_JPRB, 0.10210E+01_JPRB, 0.10292E+01_JPRB, 0.10375E+01_JPRB, 0.10435E+01_JPRB, 0.10589E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.12614E+01_JPRB, 0.14944E+01_JPRB, 0.19241E+01_JPRB, 0.22488E+01_JPRB, 0.24199E+01_JPRB, 0.23555E+01_JPRB ,&
&  0.22294E+01_JPRB, 0.21075E+01_JPRB, 0.20298E+01_JPRB, 0.19384E+01_JPRB, 0.18313E+01_JPRB, 0.17169E+01_JPRB, 0.16293E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.15441E+01_JPRB, 0.15095E+01_JPRB, 0.14830E+01_JPRB, 0.14645E+01_JPRB, 0.14540E+01_JPRB, 0.14625E+01_JPRB ,&
&  0.14501E+01_JPRB, 0.14080E+01_JPRB, 0.13625E+01_JPRB, 0.13356E+01_JPRB, 0.14092E+01_JPRB, 0.16166E+01_JPRB, 0.19891E+01_JPRB ,&
&  0.21868E+01_JPRB, 0.20685E+01_JPRB, 0.19688E+01_JPRB, 0.18801E+01_JPRB, 0.18056E+01_JPRB, 0.17564E+01_JPRB, 0.17069E+01_JPRB ,&
&  0.16694E+01_JPRB, 0.16422E+01_JPRB, 0.16248E+01_JPRB, 0.16025E+01_JPRB, 0.15766E+01_JPRB, 0.15486E+01_JPRB, 0.15125E+01_JPRB ,&
&  0.14810E+01_JPRB, 0.14626E+01_JPRB, 0.14465E+01_JPRB, 0.14558E+01_JPRB, 0.14710E+01_JPRB, 0.14835E+01_JPRB, 0.14942E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.15541E+01_JPRB, 0.15832E+01_JPRB, 0.16103E+01_JPRB, 0.15942E+01_JPRB, 0.15578E+01_JPRB, 0.15341E+01_JPRB ,&
&  0.15049E+01_JPRB, 0.14790E+01_JPRB, 0.14172E+01_JPRB, 0.14521E+01_JPRB, 0.15666E+01_JPRB, 0.15558E+01_JPRB, 0.15364E+01_JPRB ,&
&  0.15221E+01_JPRB, 0.14804E+01_JPRB, 0.14153E+01_JPRB, 0.13525E+01_JPRB, 0.13684E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.12808E+01_JPRB, 0.13123E+01_JPRB, 0.13095E+01_JPRB, 0.13052E+01_JPRB, 0.13007E+01_JPRB, 0.12858E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.12235E+01_JPRB, 0.12006E+01_JPRB, 0.11707E+01_JPRB, 0.11490E+01_JPRB, 0.11333E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.10995E+01_JPRB, 0.10913E+01_JPRB, 0.10820E+01_JPRB, 0.10673E+01_JPRB, 0.10545E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10179E+01_JPRB, 0.10164E+01_JPRB, 0.10202E+01_JPRB, 0.10287E+01_JPRB, 0.10390E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.10946E+01_JPRB, 0.10645E+01_JPRB, 0.11446E+01_JPRB, 0.13670E+01_JPRB, 0.16545E+01_JPRB, 0.18974E+01_JPRB, 0.19649E+01_JPRB ,&
&  0.19161E+01_JPRB, 0.18435E+01_JPRB, 0.17580E+01_JPRB, 0.16771E+01_JPRB, 0.16092E+01_JPRB, 0.15576E+01_JPRB, 0.14883E+01_JPRB ,&
&  0.14432E+01_JPRB, 0.14145E+01_JPRB, 0.14025E+01_JPRB, 0.13857E+01_JPRB, 0.13663E+01_JPRB, 0.13524E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.13241E+01_JPRB, 0.12935E+01_JPRB, 0.12756E+01_JPRB, 0.12913E+01_JPRB, 0.13875E+01_JPRB, 0.15461E+01_JPRB ,&
&  0.16878E+01_JPRB, 0.16657E+01_JPRB, 0.16080E+01_JPRB, 0.15491E+01_JPRB, 0.15161E+01_JPRB, 0.15014E+01_JPRB, 0.14947E+01_JPRB ,&
&  0.14832E+01_JPRB, 0.14741E+01_JPRB, 0.14863E+01_JPRB, 0.14918E+01_JPRB, 0.14984E+01_JPRB, 0.14961E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.14733E+01_JPRB, 0.14507E+01_JPRB, 0.14384E+01_JPRB, 0.14355E+01_JPRB, 0.14452E+01_JPRB, 0.14497E+01_JPRB, 0.14434E+01_JPRB ,&
&  0.14338E+01_JPRB, 0.14479E+01_JPRB, 0.14934E+01_JPRB, 0.15302E+01_JPRB, 0.15179E+01_JPRB, 0.14940E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.14407E+01_JPRB, 0.14271E+01_JPRB, 0.14683E+01_JPRB, 0.14697E+01_JPRB, 0.14586E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.14340E+01_JPRB, 0.14095E+01_JPRB, 0.13828E+01_JPRB, 0.13319E+01_JPRB, 0.12207E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.11672E+01_JPRB, 0.11991E+01_JPRB, 0.12154E+01_JPRB, 0.12182E+01_JPRB, 0.12251E+01_JPRB, 0.12300E+01_JPRB, 0.12290E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.12210E+01_JPRB, 0.12057E+01_JPRB, 0.11808E+01_JPRB, 0.11550E+01_JPRB, 0.11267E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10816E+01_JPRB, 0.10624E+01_JPRB, 0.10449E+01_JPRB, 0.10293E+01_JPRB, 0.10166E+01_JPRB, 0.10101E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.99865E+00_JPRB, 0.10025E+01_JPRB, 0.10085E+01_JPRB, 0.10176E+01_JPRB, 0.10282E+01_JPRB, 0.10420E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.91099E+00_JPRB, 0.96145E+00_JPRB, 0.11043E+01_JPRB, 0.12667E+01_JPRB, 0.14432E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.16270E+01_JPRB, 0.16333E+01_JPRB, 0.15887E+01_JPRB, 0.15371E+01_JPRB, 0.15030E+01_JPRB, 0.14545E+01_JPRB, 0.13826E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13087E+01_JPRB, 0.13112E+01_JPRB, 0.13079E+01_JPRB, 0.12917E+01_JPRB, 0.12744E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.12649E+01_JPRB, 0.12644E+01_JPRB, 0.12635E+01_JPRB, 0.12823E+01_JPRB, 0.13255E+01_JPRB, 0.13907E+01_JPRB ,&
&  0.14207E+01_JPRB, 0.14174E+01_JPRB, 0.13951E+01_JPRB, 0.13627E+01_JPRB, 0.13434E+01_JPRB, 0.13474E+01_JPRB, 0.13505E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.13444E+01_JPRB, 0.13526E+01_JPRB, 0.13614E+01_JPRB, 0.13776E+01_JPRB, 0.13944E+01_JPRB, 0.14199E+01_JPRB ,&
&  0.14382E+01_JPRB, 0.14375E+01_JPRB, 0.14320E+01_JPRB, 0.14149E+01_JPRB, 0.13880E+01_JPRB, 0.13695E+01_JPRB, 0.13617E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.13648E+01_JPRB, 0.13892E+01_JPRB, 0.14155E+01_JPRB, 0.14174E+01_JPRB, 0.14139E+01_JPRB, 0.14103E+01_JPRB ,&
&  0.14110E+01_JPRB, 0.14091E+01_JPRB, 0.14084E+01_JPRB, 0.13544E+01_JPRB, 0.13942E+01_JPRB, 0.13829E+01_JPRB, 0.13538E+01_JPRB ,&
&  0.13187E+01_JPRB, 0.12817E+01_JPRB, 0.12562E+01_JPRB, 0.10701E+01_JPRB, 0.11425E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.10872E+01_JPRB, 0.11067E+01_JPRB, 0.11113E+01_JPRB, 0.11258E+01_JPRB, 0.11361E+01_JPRB, 0.11439E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.11577E+01_JPRB, 0.11495E+01_JPRB, 0.11328E+01_JPRB, 0.11056E+01_JPRB, 0.10767E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.10274E+01_JPRB, 0.10159E+01_JPRB, 0.10023E+01_JPRB, 0.99359E+00_JPRB, 0.98837E+00_JPRB, 0.98867E+00_JPRB ,&
&  0.99505E+00_JPRB, 0.99798E+00_JPRB, 0.99170E+00_JPRB, 0.98911E+00_JPRB, 0.99669E+00_JPRB, 0.10114E+01_JPRB, 0.10172E+01_JPRB ,&
&  0.10196E+01_JPRB, 0.83023E+00_JPRB, 0.89200E+00_JPRB, 0.97212E+00_JPRB, 0.10664E+01_JPRB, 0.11928E+01_JPRB, 0.13127E+01_JPRB ,&
&  0.13891E+01_JPRB, 0.14281E+01_JPRB, 0.14163E+01_JPRB, 0.13879E+01_JPRB, 0.13566E+01_JPRB, 0.13287E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.12510E+01_JPRB, 0.12281E+01_JPRB, 0.12215E+01_JPRB, 0.12204E+01_JPRB, 0.12186E+01_JPRB, 0.12204E+01_JPRB, 0.12123E+01_JPRB ,&
&  0.12101E+01_JPRB, 0.12079E+01_JPRB, 0.12080E+01_JPRB, 0.12100E+01_JPRB, 0.12099E+01_JPRB, 0.12113E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12199E+01_JPRB, 0.12209E+01_JPRB, 0.12150E+01_JPRB, 0.12007E+01_JPRB, 0.11950E+01_JPRB, 0.12002E+01_JPRB, 0.12134E+01_JPRB ,&
&  0.12331E+01_JPRB, 0.12376E+01_JPRB, 0.12457E+01_JPRB, 0.12573E+01_JPRB, 0.12748E+01_JPRB, 0.12956E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.13375E+01_JPRB, 0.13384E+01_JPRB, 0.13291E+01_JPRB, 0.13244E+01_JPRB, 0.13016E+01_JPRB, 0.12758E+01_JPRB ,&
&  0.12693E+01_JPRB, 0.12692E+01_JPRB, 0.12725E+01_JPRB, 0.12867E+01_JPRB, 0.12931E+01_JPRB, 0.12894E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.13002E+01_JPRB, 0.12870E+01_JPRB, 0.12771E+01_JPRB, 0.12564E+01_JPRB, 0.12396E+01_JPRB, 0.12122E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11534E+01_JPRB, 0.94091E+00_JPRB, 0.10371E+01_JPRB, 0.10597E+01_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.10568E+01_JPRB, 0.10626E+01_JPRB, 0.10658E+01_JPRB, 0.10711E+01_JPRB, 0.10729E+01_JPRB, 0.10726E+01_JPRB, 0.10782E+01_JPRB ,&
&  0.10843E+01_JPRB, 0.10803E+01_JPRB, 0.10718E+01_JPRB, 0.10680E+01_JPRB, 0.10566E+01_JPRB, 0.10402E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.10009E+01_JPRB, 0.99571E+00_JPRB, 0.99024E+00_JPRB, 0.98184E+00_JPRB, 0.97189E+00_JPRB, 0.97448E+00_JPRB ,&
&  0.98087E+00_JPRB, 0.98792E+00_JPRB, 0.98914E+00_JPRB, 0.99190E+00_JPRB, 0.99665E+00_JPRB, 0.10021E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.89734E+00_JPRB, 0.77301E+00_JPRB, 0.83302E+00_JPRB, 0.89847E+00_JPRB, 0.96809E+00_JPRB, 0.10778E+01_JPRB, 0.11680E+01_JPRB ,&
&  0.12360E+01_JPRB, 0.12790E+01_JPRB, 0.12871E+01_JPRB, 0.12571E+01_JPRB, 0.12258E+01_JPRB, 0.12193E+01_JPRB, 0.12205E+01_JPRB ,&
&  0.12136E+01_JPRB, 0.12038E+01_JPRB, 0.11982E+01_JPRB, 0.11961E+01_JPRB, 0.11980E+01_JPRB, 0.12035E+01_JPRB, 0.12060E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.11889E+01_JPRB, 0.11706E+01_JPRB, 0.11438E+01_JPRB, 0.11182E+01_JPRB, 0.11046E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10798E+01_JPRB, 0.10839E+01_JPRB, 0.10871E+01_JPRB, 0.10960E+01_JPRB, 0.11093E+01_JPRB, 0.11294E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.11753E+01_JPRB, 0.11919E+01_JPRB, 0.12092E+01_JPRB, 0.12176E+01_JPRB, 0.12279E+01_JPRB, 0.12463E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12505E+01_JPRB, 0.12398E+01_JPRB, 0.12303E+01_JPRB, 0.12288E+01_JPRB, 0.12291E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12032E+01_JPRB, 0.11995E+01_JPRB, 0.12040E+01_JPRB, 0.12004E+01_JPRB, 0.11906E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.11578E+01_JPRB, 0.11389E+01_JPRB, 0.11255E+01_JPRB, 0.11093E+01_JPRB, 0.10975E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10603E+01_JPRB, 0.10631E+01_JPRB, 0.10559E+01_JPRB, 0.10522E+01_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.10137E+01_JPRB, 0.10177E+01_JPRB, 0.10220E+01_JPRB, 0.10236E+01_JPRB, 0.10211E+01_JPRB, 0.10199E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.10187E+01_JPRB, 0.10115E+01_JPRB, 0.10056E+01_JPRB, 0.10025E+01_JPRB, 0.99205E+00_JPRB, 0.97642E+00_JPRB ,&
&  0.96501E+00_JPRB, 0.96264E+00_JPRB, 0.96683E+00_JPRB, 0.96863E+00_JPRB, 0.95868E+00_JPRB, 0.94848E+00_JPRB, 0.94457E+00_JPRB ,&
&  0.94518E+00_JPRB, 0.95076E+00_JPRB, 0.96032E+00_JPRB, 0.96504E+00_JPRB, 0.97253E+00_JPRB, 0.98566E+00_JPRB, 0.98479E+00_JPRB ,&
&  0.98241E+00_JPRB, 0.84523E+00_JPRB, 0.88738E+00_JPRB, 0.90510E+00_JPRB, 0.94162E+00_JPRB, 0.10017E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11501E+01_JPRB, 0.11650E+01_JPRB, 0.11576E+01_JPRB, 0.11374E+01_JPRB, 0.11192E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.11342E+01_JPRB, 0.11478E+01_JPRB, 0.11556E+01_JPRB, 0.11573E+01_JPRB, 0.11577E+01_JPRB, 0.11609E+01_JPRB ,&
&  0.11573E+01_JPRB, 0.11381E+01_JPRB, 0.11087E+01_JPRB, 0.10714E+01_JPRB, 0.10390E+01_JPRB, 0.10190E+01_JPRB, 0.10002E+01_JPRB ,&
&  0.97849E+00_JPRB, 0.96821E+00_JPRB, 0.97036E+00_JPRB, 0.97746E+00_JPRB, 0.99074E+00_JPRB, 0.10063E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.10520E+01_JPRB, 0.10737E+01_JPRB, 0.10943E+01_JPRB, 0.11122E+01_JPRB, 0.11266E+01_JPRB, 0.11357E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.11368E+01_JPRB, 0.11244E+01_JPRB, 0.11161E+01_JPRB, 0.11145E+01_JPRB, 0.11217E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.11108E+01_JPRB, 0.11057E+01_JPRB, 0.10927E+01_JPRB, 0.10774E+01_JPRB, 0.10619E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.10256E+01_JPRB, 0.10189E+01_JPRB, 0.10098E+01_JPRB, 0.99726E+00_JPRB, 0.98959E+00_JPRB, 0.98304E+00_JPRB ,&
&  0.97774E+00_JPRB, 0.97263E+00_JPRB, 0.97427E+00_JPRB, 0.98579E+00_JPRB, 0.10024E+01_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.95853E+00_JPRB, 0.96534E+00_JPRB, 0.97570E+00_JPRB, 0.98766E+00_JPRB, 0.99276E+00_JPRB, 0.99367E+00_JPRB, 0.99231E+00_JPRB ,&
&  0.99398E+00_JPRB, 0.98663E+00_JPRB, 0.98116E+00_JPRB, 0.97763E+00_JPRB, 0.97655E+00_JPRB, 0.97211E+00_JPRB, 0.96145E+00_JPRB ,&
&  0.95236E+00_JPRB, 0.94819E+00_JPRB, 0.95153E+00_JPRB, 0.95184E+00_JPRB, 0.93524E+00_JPRB, 0.92925E+00_JPRB, 0.92033E+00_JPRB ,&
&  0.91828E+00_JPRB, 0.91704E+00_JPRB, 0.91861E+00_JPRB, 0.92314E+00_JPRB, 0.93799E+00_JPRB, 0.94831E+00_JPRB, 0.95320E+00_JPRB ,&
&  0.94953E+00_JPRB, 0.88291E+00_JPRB, 0.91266E+00_JPRB, 0.92175E+00_JPRB, 0.94251E+00_JPRB, 0.96643E+00_JPRB, 0.99294E+00_JPRB ,&
&  0.10294E+01_JPRB, 0.10520E+01_JPRB, 0.10648E+01_JPRB, 0.10707E+01_JPRB, 0.10732E+01_JPRB, 0.10708E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10686E+01_JPRB, 0.10904E+01_JPRB, 0.11028E+01_JPRB, 0.11098E+01_JPRB, 0.11139E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.10877E+01_JPRB, 0.10584E+01_JPRB, 0.10217E+01_JPRB, 0.97978E+00_JPRB, 0.94819E+00_JPRB, 0.92896E+00_JPRB ,&
&  0.91806E+00_JPRB, 0.91814E+00_JPRB, 0.92173E+00_JPRB, 0.92647E+00_JPRB, 0.93300E+00_JPRB, 0.94465E+00_JPRB, 0.95138E+00_JPRB ,&
&  0.96038E+00_JPRB, 0.96822E+00_JPRB, 0.98062E+00_JPRB, 0.10008E+01_JPRB, 0.10207E+01_JPRB, 0.10318E+01_JPRB, 0.10329E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.10442E+01_JPRB, 0.10416E+01_JPRB, 0.10419E+01_JPRB, 0.10384E+01_JPRB, 0.10404E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.10306E+01_JPRB, 0.10238E+01_JPRB, 0.10150E+01_JPRB, 0.99805E+00_JPRB, 0.98218E+00_JPRB, 0.96948E+00_JPRB, 0.95846E+00_JPRB ,&
&  0.95501E+00_JPRB, 0.95724E+00_JPRB, 0.96145E+00_JPRB, 0.95522E+00_JPRB, 0.93964E+00_JPRB, 0.92910E+00_JPRB, 0.91716E+00_JPRB ,&
&  0.90925E+00_JPRB, 0.91120E+00_JPRB, 0.91136E+00_JPRB, 0.91953E+00_JPRB, 0.94010E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.90185E+00_JPRB, 0.91560E+00_JPRB, 0.93294E+00_JPRB, 0.95100E+00_JPRB, 0.96306E+00_JPRB, 0.97082E+00_JPRB, 0.96976E+00_JPRB ,&
&  0.95955E+00_JPRB, 0.94943E+00_JPRB, 0.95183E+00_JPRB, 0.95217E+00_JPRB, 0.95106E+00_JPRB, 0.94877E+00_JPRB, 0.94133E+00_JPRB ,&
&  0.93067E+00_JPRB, 0.92129E+00_JPRB, 0.91679E+00_JPRB, 0.91300E+00_JPRB, 0.90218E+00_JPRB, 0.89350E+00_JPRB, 0.88399E+00_JPRB ,&
&  0.88242E+00_JPRB, 0.88362E+00_JPRB, 0.88496E+00_JPRB, 0.88346E+00_JPRB, 0.88968E+00_JPRB, 0.89815E+00_JPRB, 0.90141E+00_JPRB ,&
&  0.89467E+00_JPRB, 0.90936E+00_JPRB, 0.82234E+00_JPRB, 0.89109E+00_JPRB, 0.91398E+00_JPRB, 0.92787E+00_JPRB, 0.94598E+00_JPRB ,&
&  0.96090E+00_JPRB, 0.97620E+00_JPRB, 0.98799E+00_JPRB, 0.99734E+00_JPRB, 0.10197E+01_JPRB, 0.10345E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10351E+01_JPRB, 0.10373E+01_JPRB, 0.10399E+01_JPRB, 0.10476E+01_JPRB, 0.10509E+01_JPRB, 0.10474E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.10281E+01_JPRB, 0.10081E+01_JPRB, 0.98391E+00_JPRB, 0.94627E+00_JPRB, 0.91348E+00_JPRB, 0.89204E+00_JPRB ,&
&  0.87547E+00_JPRB, 0.87039E+00_JPRB, 0.86523E+00_JPRB, 0.86045E+00_JPRB, 0.85943E+00_JPRB, 0.86193E+00_JPRB, 0.86144E+00_JPRB ,&
&  0.86823E+00_JPRB, 0.87684E+00_JPRB, 0.88102E+00_JPRB, 0.88914E+00_JPRB, 0.90024E+00_JPRB, 0.91600E+00_JPRB, 0.93128E+00_JPRB ,&
&  0.94589E+00_JPRB, 0.95673E+00_JPRB, 0.95993E+00_JPRB, 0.96126E+00_JPRB, 0.95928E+00_JPRB, 0.95820E+00_JPRB, 0.95120E+00_JPRB ,&
&  0.94571E+00_JPRB, 0.94287E+00_JPRB, 0.93619E+00_JPRB, 0.92328E+00_JPRB, 0.91433E+00_JPRB, 0.90644E+00_JPRB, 0.90355E+00_JPRB ,&
&  0.90320E+00_JPRB, 0.90384E+00_JPRB, 0.90109E+00_JPRB, 0.90156E+00_JPRB, 0.89457E+00_JPRB, 0.87301E+00_JPRB, 0.85596E+00_JPRB ,&
&  0.85443E+00_JPRB, 0.85943E+00_JPRB, 0.86404E+00_JPRB, 0.87301E+00_JPRB, 0.88871E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.91200E+00_JPRB, 0.91878E+00_JPRB, 0.92924E+00_JPRB, 0.94439E+00_JPRB, 0.96035E+00_JPRB, 0.97119E+00_JPRB, 0.97201E+00_JPRB ,&
&  0.96173E+00_JPRB, 0.94874E+00_JPRB, 0.94461E+00_JPRB, 0.94416E+00_JPRB, 0.94125E+00_JPRB, 0.93971E+00_JPRB, 0.93468E+00_JPRB ,&
&  0.92250E+00_JPRB, 0.90418E+00_JPRB, 0.89507E+00_JPRB, 0.88025E+00_JPRB, 0.86618E+00_JPRB, 0.85425E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.86072E+00_JPRB, 0.86996E+00_JPRB, 0.88151E+00_JPRB, 0.89422E+00_JPRB, 0.89981E+00_JPRB, 0.89655E+00_JPRB, 0.89188E+00_JPRB ,&
&  0.88782E+00_JPRB, 0.88493E+00_JPRB, 0.89418E+00_JPRB, 0.90208E+00_JPRB, 0.90699E+00_JPRB, 0.91752E+00_JPRB, 0.92753E+00_JPRB ,&
&  0.93756E+00_JPRB, 0.95103E+00_JPRB, 0.97176E+00_JPRB, 0.98774E+00_JPRB, 0.10009E+01_JPRB, 0.10137E+01_JPRB, 0.10157E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10307E+01_JPRB, 0.10331E+01_JPRB, 0.10245E+01_JPRB, 0.10113E+01_JPRB, 0.10040E+01_JPRB, 0.10014E+01_JPRB ,&
&  0.99413E+00_JPRB, 0.97551E+00_JPRB, 0.96062E+00_JPRB, 0.94778E+00_JPRB, 0.92646E+00_JPRB, 0.90383E+00_JPRB, 0.87790E+00_JPRB ,&
&  0.84887E+00_JPRB, 0.82727E+00_JPRB, 0.81033E+00_JPRB, 0.79492E+00_JPRB, 0.79025E+00_JPRB, 0.78047E+00_JPRB, 0.77362E+00_JPRB ,&
&  0.77834E+00_JPRB, 0.79769E+00_JPRB, 0.81275E+00_JPRB, 0.81905E+00_JPRB, 0.82314E+00_JPRB, 0.84058E+00_JPRB, 0.86160E+00_JPRB ,&
&  0.87845E+00_JPRB, 0.88665E+00_JPRB, 0.88438E+00_JPRB, 0.88820E+00_JPRB, 0.89443E+00_JPRB, 0.89915E+00_JPRB, 0.89289E+00_JPRB ,&
&  0.88406E+00_JPRB, 0.87853E+00_JPRB, 0.87721E+00_JPRB, 0.87321E+00_JPRB, 0.87429E+00_JPRB, 0.87383E+00_JPRB, 0.87068E+00_JPRB ,&
&  0.86479E+00_JPRB, 0.85569E+00_JPRB, 0.84363E+00_JPRB, 0.83554E+00_JPRB, 0.82385E+00_JPRB, 0.80608E+00_JPRB, 0.80545E+00_JPRB ,&
&  0.81619E+00_JPRB, 0.83422E+00_JPRB, 0.85209E+00_JPRB, 0.87393E+00_JPRB, 0.89973E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.93411E+00_JPRB, 0.94268E+00_JPRB, 0.94865E+00_JPRB, 0.95331E+00_JPRB, 0.96325E+00_JPRB, 0.97573E+00_JPRB, 0.97664E+00_JPRB ,&
&  0.96254E+00_JPRB, 0.94884E+00_JPRB, 0.93911E+00_JPRB, 0.93653E+00_JPRB, 0.93833E+00_JPRB, 0.94064E+00_JPRB, 0.93445E+00_JPRB ,&
&  0.91174E+00_JPRB, 0.88739E+00_JPRB, 0.87215E+00_JPRB, 0.85717E+00_JPRB, 0.84753E+00_JPRB, 0.84101E+00_JPRB, 0.84685E+00_JPRB ,&
&  0.85589E+00_JPRB, 0.87076E+00_JPRB, 0.88254E+00_JPRB, 0.89078E+00_JPRB, 0.89346E+00_JPRB, 0.88084E+00_JPRB, 0.88103E+00_JPRB ,&
&  0.88681E+00_JPRB, 0.88796E+00_JPRB, 0.90199E+00_JPRB, 0.90064E+00_JPRB, 0.88612E+00_JPRB, 0.87398E+00_JPRB, 0.86246E+00_JPRB ,&
&  0.85970E+00_JPRB, 0.87134E+00_JPRB, 0.89760E+00_JPRB, 0.93569E+00_JPRB, 0.95938E+00_JPRB, 0.96885E+00_JPRB, 0.96457E+00_JPRB ,&
&  0.96543E+00_JPRB, 0.97664E+00_JPRB, 0.98334E+00_JPRB, 0.97549E+00_JPRB, 0.95802E+00_JPRB, 0.94400E+00_JPRB, 0.93225E+00_JPRB ,&
&  0.92472E+00_JPRB, 0.91511E+00_JPRB, 0.90815E+00_JPRB, 0.89858E+00_JPRB, 0.87566E+00_JPRB, 0.84836E+00_JPRB, 0.82425E+00_JPRB ,&
&  0.80599E+00_JPRB, 0.78792E+00_JPRB, 0.76617E+00_JPRB, 0.75833E+00_JPRB, 0.76108E+00_JPRB, 0.73767E+00_JPRB, 0.71299E+00_JPRB ,&
&  0.71686E+00_JPRB, 0.74894E+00_JPRB, 0.77457E+00_JPRB, 0.78578E+00_JPRB, 0.78797E+00_JPRB, 0.79348E+00_JPRB, 0.79817E+00_JPRB ,&
&  0.80628E+00_JPRB, 0.81654E+00_JPRB, 0.82464E+00_JPRB, 0.83697E+00_JPRB, 0.84944E+00_JPRB, 0.84799E+00_JPRB, 0.83320E+00_JPRB ,&
&  0.81744E+00_JPRB, 0.80887E+00_JPRB, 0.80580E+00_JPRB, 0.80856E+00_JPRB, 0.81224E+00_JPRB, 0.81040E+00_JPRB, 0.80465E+00_JPRB ,&
&  0.79279E+00_JPRB, 0.77440E+00_JPRB, 0.75849E+00_JPRB, 0.74878E+00_JPRB, 0.74727E+00_JPRB, 0.75289E+00_JPRB, 0.76879E+00_JPRB ,&
&  0.80009E+00_JPRB, 0.84800E+00_JPRB, 0.88477E+00_JPRB, 0.90709E+00_JPRB, 0.91522E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.95810E+00_JPRB, 0.96704E+00_JPRB, 0.97724E+00_JPRB, 0.99110E+00_JPRB, 0.10042E+01_JPRB, 0.10017E+01_JPRB, 0.98936E+00_JPRB ,&
&  0.96583E+00_JPRB, 0.95654E+00_JPRB, 0.96083E+00_JPRB, 0.96298E+00_JPRB, 0.96449E+00_JPRB, 0.96199E+00_JPRB, 0.94511E+00_JPRB ,&
&  0.90568E+00_JPRB, 0.87403E+00_JPRB, 0.85849E+00_JPRB, 0.85005E+00_JPRB, 0.84687E+00_JPRB, 0.85275E+00_JPRB, 0.85950E+00_JPRB ,&
&  0.87428E+00_JPRB, 0.88256E+00_JPRB, 0.89370E+00_JPRB, 0.90279E+00_JPRB, 0.90538E+00_JPRB, 0.90893E+00_JPRB, 0.90917E+00_JPRB ,&
&  0.89380E+00_JPRB, 0.89314E+00_JPRB, 0.89317E+00_JPRB, 0.88229E+00_JPRB, 0.85884E+00_JPRB, 0.83526E+00_JPRB, 0.80698E+00_JPRB ,&
&  0.78796E+00_JPRB, 0.77859E+00_JPRB, 0.80440E+00_JPRB, 0.86745E+00_JPRB, 0.91266E+00_JPRB, 0.92108E+00_JPRB, 0.91169E+00_JPRB ,&
&  0.91199E+00_JPRB, 0.91889E+00_JPRB, 0.92275E+00_JPRB, 0.91172E+00_JPRB, 0.89240E+00_JPRB, 0.87554E+00_JPRB, 0.85999E+00_JPRB ,&
&  0.85610E+00_JPRB, 0.85724E+00_JPRB, 0.84175E+00_JPRB, 0.81046E+00_JPRB, 0.78227E+00_JPRB, 0.76200E+00_JPRB, 0.75101E+00_JPRB ,&
&  0.74377E+00_JPRB, 0.73303E+00_JPRB, 0.72560E+00_JPRB, 0.72912E+00_JPRB, 0.70836E+00_JPRB, 0.69022E+00_JPRB, 0.69397E+00_JPRB ,&
&  0.71049E+00_JPRB, 0.71680E+00_JPRB, 0.72695E+00_JPRB, 0.73124E+00_JPRB, 0.72954E+00_JPRB, 0.72710E+00_JPRB, 0.72950E+00_JPRB ,&
&  0.73959E+00_JPRB, 0.75110E+00_JPRB, 0.75976E+00_JPRB, 0.76535E+00_JPRB, 0.77076E+00_JPRB, 0.75855E+00_JPRB, 0.73719E+00_JPRB ,&
&  0.72025E+00_JPRB, 0.71783E+00_JPRB, 0.72301E+00_JPRB, 0.72289E+00_JPRB, 0.72163E+00_JPRB, 0.72213E+00_JPRB, 0.72139E+00_JPRB ,&
&  0.71920E+00_JPRB, 0.71243E+00_JPRB, 0.70395E+00_JPRB, 0.70352E+00_JPRB, 0.71421E+00_JPRB, 0.73746E+00_JPRB, 0.77358E+00_JPRB ,&
&  0.82615E+00_JPRB, 0.86560E+00_JPRB, 0.90499E+00_JPRB, 0.92284E+00_JPRB, 0.92885E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.98840E+00_JPRB, 0.10116E+01_JPRB, 0.10312E+01_JPRB, 0.10446E+01_JPRB, 0.10469E+01_JPRB, 0.10402E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.99358E+00_JPRB, 0.97616E+00_JPRB, 0.98489E+00_JPRB, 0.99935E+00_JPRB, 0.10078E+01_JPRB, 0.10072E+01_JPRB, 0.97477E+00_JPRB ,&
&  0.91836E+00_JPRB, 0.89644E+00_JPRB, 0.89240E+00_JPRB, 0.90098E+00_JPRB, 0.91788E+00_JPRB, 0.92310E+00_JPRB, 0.90914E+00_JPRB ,&
&  0.89537E+00_JPRB, 0.89587E+00_JPRB, 0.89794E+00_JPRB, 0.90472E+00_JPRB, 0.89449E+00_JPRB, 0.87322E+00_JPRB, 0.86308E+00_JPRB ,&
&  0.85910E+00_JPRB, 0.84959E+00_JPRB, 0.83935E+00_JPRB, 0.79034E+00_JPRB, 0.80956E+00_JPRB, 0.79676E+00_JPRB, 0.77667E+00_JPRB ,&
&  0.75770E+00_JPRB, 0.75944E+00_JPRB, 0.80215E+00_JPRB, 0.85005E+00_JPRB, 0.86222E+00_JPRB, 0.86435E+00_JPRB, 0.85776E+00_JPRB ,&
&  0.85328E+00_JPRB, 0.85345E+00_JPRB, 0.85500E+00_JPRB, 0.84695E+00_JPRB, 0.82688E+00_JPRB, 0.80091E+00_JPRB, 0.77273E+00_JPRB ,&
&  0.75640E+00_JPRB, 0.73287E+00_JPRB, 0.69022E+00_JPRB, 0.67057E+00_JPRB, 0.66944E+00_JPRB, 0.67795E+00_JPRB, 0.68187E+00_JPRB ,&
&  0.67863E+00_JPRB, 0.67042E+00_JPRB, 0.67214E+00_JPRB, 0.66739E+00_JPRB, 0.64486E+00_JPRB, 0.63048E+00_JPRB, 0.62390E+00_JPRB ,&
&  0.61301E+00_JPRB, 0.61348E+00_JPRB, 0.61998E+00_JPRB, 0.63294E+00_JPRB, 0.64425E+00_JPRB, 0.65997E+00_JPRB, 0.66730E+00_JPRB ,&
&  0.67030E+00_JPRB, 0.66608E+00_JPRB, 0.65934E+00_JPRB, 0.64682E+00_JPRB, 0.62779E+00_JPRB, 0.62046E+00_JPRB, 0.62245E+00_JPRB ,&
&  0.63243E+00_JPRB, 0.63992E+00_JPRB, 0.64064E+00_JPRB, 0.64034E+00_JPRB, 0.62919E+00_JPRB, 0.61674E+00_JPRB, 0.60768E+00_JPRB ,&
&  0.60404E+00_JPRB, 0.60821E+00_JPRB, 0.62558E+00_JPRB, 0.65238E+00_JPRB, 0.68590E+00_JPRB, 0.72651E+00_JPRB, 0.77222E+00_JPRB ,&
&  0.80402E+00_JPRB, 0.82208E+00_JPRB, 0.84671E+00_JPRB, 0.89611E+00_JPRB, 0.94498E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.98553E+00_JPRB, 0.10140E+01_JPRB, 0.10441E+01_JPRB, 0.10614E+01_JPRB, 0.10626E+01_JPRB, 0.10597E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.98913E+00_JPRB, 0.98516E+00_JPRB, 0.98912E+00_JPRB, 0.10004E+01_JPRB, 0.10182E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.98915E+00_JPRB, 0.96802E+00_JPRB, 0.95701E+00_JPRB, 0.94841E+00_JPRB, 0.92981E+00_JPRB, 0.90376E+00_JPRB, 0.88657E+00_JPRB ,&
&  0.88118E+00_JPRB, 0.88026E+00_JPRB, 0.87092E+00_JPRB, 0.85710E+00_JPRB, 0.82548E+00_JPRB, 0.78624E+00_JPRB, 0.75423E+00_JPRB ,&
&  0.74162E+00_JPRB, 0.72894E+00_JPRB, 0.68912E+00_JPRB, 0.70082E+00_JPRB, 0.70490E+00_JPRB, 0.70481E+00_JPRB, 0.70065E+00_JPRB ,&
&  0.71066E+00_JPRB, 0.74389E+00_JPRB, 0.77764E+00_JPRB, 0.78737E+00_JPRB, 0.79610E+00_JPRB, 0.79730E+00_JPRB, 0.79801E+00_JPRB ,&
&  0.80206E+00_JPRB, 0.79415E+00_JPRB, 0.76465E+00_JPRB, 0.72066E+00_JPRB, 0.67739E+00_JPRB, 0.64385E+00_JPRB, 0.61557E+00_JPRB ,&
&  0.59662E+00_JPRB, 0.58508E+00_JPRB, 0.57583E+00_JPRB, 0.57288E+00_JPRB, 0.58124E+00_JPRB, 0.59673E+00_JPRB, 0.59135E+00_JPRB ,&
&  0.57180E+00_JPRB, 0.55292E+00_JPRB, 0.52905E+00_JPRB, 0.49705E+00_JPRB, 0.46342E+00_JPRB, 0.42685E+00_JPRB, 0.40727E+00_JPRB ,&
&  0.41269E+00_JPRB, 0.45277E+00_JPRB, 0.50464E+00_JPRB, 0.52694E+00_JPRB, 0.55574E+00_JPRB, 0.58411E+00_JPRB, 0.57318E+00_JPRB ,&
&  0.52180E+00_JPRB, 0.45769E+00_JPRB, 0.43524E+00_JPRB, 0.42095E+00_JPRB, 0.41399E+00_JPRB, 0.40986E+00_JPRB, 0.40829E+00_JPRB ,&
&  0.41338E+00_JPRB, 0.42457E+00_JPRB, 0.43542E+00_JPRB, 0.43349E+00_JPRB, 0.42396E+00_JPRB, 0.41975E+00_JPRB, 0.42190E+00_JPRB ,&
&  0.43485E+00_JPRB, 0.46548E+00_JPRB, 0.52799E+00_JPRB, 0.63885E+00_JPRB, 0.67948E+00_JPRB, 0.71521E+00_JPRB, 0.75532E+00_JPRB ,&
&  0.79426E+00_JPRB, 0.82971E+00_JPRB, 0.85933E+00_JPRB, 0.89537E+00_JPRB, 0.94372E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.91952E+00_JPRB, 0.94004E+00_JPRB, 0.96306E+00_JPRB, 0.97524E+00_JPRB, 0.97686E+00_JPRB, 0.97425E+00_JPRB, 0.96576E+00_JPRB ,&
&  0.95098E+00_JPRB, 0.93307E+00_JPRB, 0.91962E+00_JPRB, 0.91005E+00_JPRB, 0.90493E+00_JPRB, 0.90645E+00_JPRB, 0.91004E+00_JPRB ,&
&  0.90727E+00_JPRB, 0.90165E+00_JPRB, 0.88809E+00_JPRB, 0.87070E+00_JPRB, 0.85243E+00_JPRB, 0.83843E+00_JPRB, 0.82792E+00_JPRB ,&
&  0.81777E+00_JPRB, 0.80294E+00_JPRB, 0.78007E+00_JPRB, 0.74451E+00_JPRB, 0.70240E+00_JPRB, 0.66388E+00_JPRB, 0.63188E+00_JPRB ,&
&  0.60128E+00_JPRB, 0.53639E+00_JPRB, 0.50073E+00_JPRB, 0.50810E+00_JPRB, 0.59216E+00_JPRB, 0.62745E+00_JPRB, 0.65339E+00_JPRB ,&
&  0.68831E+00_JPRB, 0.72589E+00_JPRB, 0.74399E+00_JPRB, 0.74386E+00_JPRB, 0.73783E+00_JPRB, 0.72478E+00_JPRB, 0.70459E+00_JPRB ,&
&  0.67913E+00_JPRB, 0.64834E+00_JPRB, 0.62191E+00_JPRB, 0.59168E+00_JPRB, 0.56113E+00_JPRB, 0.53902E+00_JPRB, 0.52380E+00_JPRB ,&
&  0.51341E+00_JPRB, 0.50330E+00_JPRB, 0.49500E+00_JPRB, 0.48477E+00_JPRB, 0.47547E+00_JPRB, 0.45401E+00_JPRB, 0.43000E+00_JPRB ,&
&  0.40884E+00_JPRB, 0.38961E+00_JPRB, 0.37091E+00_JPRB, 0.35224E+00_JPRB, 0.33701E+00_JPRB, 0.32678E+00_JPRB, 0.31891E+00_JPRB ,&
&  0.31735E+00_JPRB, 0.32592E+00_JPRB, 0.34754E+00_JPRB, 0.38025E+00_JPRB, 0.41160E+00_JPRB, 0.39241E+00_JPRB, 0.37453E+00_JPRB ,&
&  0.35144E+00_JPRB, 0.33813E+00_JPRB, 0.33310E+00_JPRB, 0.33244E+00_JPRB, 0.33356E+00_JPRB, 0.33573E+00_JPRB, 0.33837E+00_JPRB ,&
&  0.34256E+00_JPRB, 0.34672E+00_JPRB, 0.35082E+00_JPRB, 0.35418E+00_JPRB, 0.35642E+00_JPRB, 0.35975E+00_JPRB, 0.36500E+00_JPRB ,&
&  0.37388E+00_JPRB, 0.38788E+00_JPRB, 0.40894E+00_JPRB, 0.43630E+00_JPRB, 0.46565E+00_JPRB, 0.49206E+00_JPRB, 0.54545E+00_JPRB ,&
&  0.63297E+00_JPRB, 0.74280E+00_JPRB, 0.80480E+00_JPRB, 0.86527E+00_JPRB, 0.89566E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.84787E+00_JPRB, 0.86306E+00_JPRB, 0.87521E+00_JPRB, 0.87903E+00_JPRB, 0.87874E+00_JPRB, 0.87385E+00_JPRB, 0.86160E+00_JPRB ,&
&  0.84184E+00_JPRB, 0.81541E+00_JPRB, 0.78948E+00_JPRB, 0.76890E+00_JPRB, 0.75564E+00_JPRB, 0.75724E+00_JPRB, 0.76467E+00_JPRB ,&
&  0.78050E+00_JPRB, 0.80462E+00_JPRB, 0.80865E+00_JPRB, 0.80067E+00_JPRB, 0.76413E+00_JPRB, 0.72595E+00_JPRB, 0.69148E+00_JPRB ,&
&  0.66029E+00_JPRB, 0.62979E+00_JPRB, 0.61848E+00_JPRB, 0.59889E+00_JPRB, 0.58524E+00_JPRB, 0.56808E+00_JPRB, 0.54248E+00_JPRB ,&
&  0.50068E+00_JPRB, 0.47095E+00_JPRB, 0.45770E+00_JPRB, 0.46492E+00_JPRB, 0.54636E+00_JPRB, 0.60782E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.65494E+00_JPRB, 0.67765E+00_JPRB, 0.67173E+00_JPRB, 0.65077E+00_JPRB, 0.62841E+00_JPRB, 0.60466E+00_JPRB, 0.58400E+00_JPRB ,&
&  0.56354E+00_JPRB, 0.53705E+00_JPRB, 0.48611E+00_JPRB, 0.41378E+00_JPRB, 0.37889E+00_JPRB, 0.36578E+00_JPRB, 0.35135E+00_JPRB ,&
&  0.34063E+00_JPRB, 0.33343E+00_JPRB, 0.33027E+00_JPRB, 0.32555E+00_JPRB, 0.32512E+00_JPRB, 0.32137E+00_JPRB, 0.31757E+00_JPRB ,&
&  0.31165E+00_JPRB, 0.30718E+00_JPRB, 0.30164E+00_JPRB, 0.29628E+00_JPRB, 0.29166E+00_JPRB, 0.29098E+00_JPRB, 0.29105E+00_JPRB ,&
&  0.29250E+00_JPRB, 0.29709E+00_JPRB, 0.31099E+00_JPRB, 0.31839E+00_JPRB, 0.31914E+00_JPRB, 0.31634E+00_JPRB, 0.30908E+00_JPRB ,&
&  0.30119E+00_JPRB, 0.29997E+00_JPRB, 0.29871E+00_JPRB, 0.29965E+00_JPRB, 0.30319E+00_JPRB, 0.30745E+00_JPRB, 0.31057E+00_JPRB ,&
&  0.31311E+00_JPRB, 0.31632E+00_JPRB, 0.31914E+00_JPRB, 0.32222E+00_JPRB, 0.32609E+00_JPRB, 0.33055E+00_JPRB, 0.33661E+00_JPRB ,&
&  0.34269E+00_JPRB, 0.35139E+00_JPRB, 0.36277E+00_JPRB, 0.37944E+00_JPRB, 0.39514E+00_JPRB, 0.40589E+00_JPRB, 0.42169E+00_JPRB ,&
&  0.43916E+00_JPRB, 0.48913E+00_JPRB, 0.62586E+00_JPRB, 0.84478E+00_JPRB, 0.87701E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.76478E+00_JPRB, 0.77889E+00_JPRB, 0.79418E+00_JPRB, 0.79881E+00_JPRB, 0.80948E+00_JPRB, 0.80819E+00_JPRB, 0.79031E+00_JPRB ,&
&  0.76228E+00_JPRB, 0.73500E+00_JPRB, 0.72031E+00_JPRB, 0.69897E+00_JPRB, 0.66978E+00_JPRB, 0.61089E+00_JPRB, 0.58857E+00_JPRB ,&
&  0.56928E+00_JPRB, 0.56831E+00_JPRB, 0.55811E+00_JPRB, 0.55750E+00_JPRB, 0.54559E+00_JPRB, 0.53366E+00_JPRB, 0.51373E+00_JPRB ,&
&  0.49575E+00_JPRB, 0.47939E+00_JPRB, 0.46767E+00_JPRB, 0.46311E+00_JPRB, 0.46062E+00_JPRB, 0.45466E+00_JPRB, 0.44027E+00_JPRB ,&
&  0.43041E+00_JPRB, 0.42819E+00_JPRB, 0.42996E+00_JPRB, 0.43511E+00_JPRB, 0.47437E+00_JPRB, 0.53229E+00_JPRB, 0.53447E+00_JPRB ,&
&  0.55546E+00_JPRB, 0.56730E+00_JPRB, 0.57914E+00_JPRB, 0.56679E+00_JPRB, 0.53268E+00_JPRB, 0.49019E+00_JPRB, 0.43782E+00_JPRB ,&
&  0.38810E+00_JPRB, 0.35814E+00_JPRB, 0.33364E+00_JPRB, 0.31567E+00_JPRB, 0.30263E+00_JPRB, 0.29775E+00_JPRB, 0.29136E+00_JPRB ,&
&  0.28742E+00_JPRB, 0.28434E+00_JPRB, 0.28318E+00_JPRB, 0.28291E+00_JPRB, 0.28205E+00_JPRB, 0.27961E+00_JPRB, 0.27894E+00_JPRB ,&
&  0.27972E+00_JPRB, 0.27971E+00_JPRB, 0.27916E+00_JPRB, 0.28016E+00_JPRB, 0.28120E+00_JPRB, 0.28295E+00_JPRB, 0.28474E+00_JPRB ,&
&  0.28754E+00_JPRB, 0.28911E+00_JPRB, 0.29147E+00_JPRB, 0.29418E+00_JPRB, 0.29309E+00_JPRB, 0.29216E+00_JPRB, 0.28948E+00_JPRB ,&
&  0.28615E+00_JPRB, 0.28642E+00_JPRB, 0.28624E+00_JPRB, 0.28722E+00_JPRB, 0.28953E+00_JPRB, 0.29235E+00_JPRB, 0.29590E+00_JPRB ,&
&  0.29920E+00_JPRB, 0.30264E+00_JPRB, 0.30528E+00_JPRB, 0.30793E+00_JPRB, 0.31240E+00_JPRB, 0.31694E+00_JPRB, 0.32274E+00_JPRB ,&
&  0.32873E+00_JPRB, 0.33562E+00_JPRB, 0.34487E+00_JPRB, 0.35686E+00_JPRB, 0.37495E+00_JPRB, 0.39428E+00_JPRB, 0.42468E+00_JPRB ,&
&  0.44859E+00_JPRB, 0.54301E+00_JPRB, 0.77578E+00_JPRB, 0.71127E+00_JPRB, 0.73489E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.72571E+00_JPRB, 0.74429E+00_JPRB, 0.76845E+00_JPRB, 0.76193E+00_JPRB, 0.78155E+00_JPRB, 0.76825E+00_JPRB, 0.74425E+00_JPRB ,&
&  0.69696E+00_JPRB, 0.64207E+00_JPRB, 0.59489E+00_JPRB, 0.54303E+00_JPRB, 0.50130E+00_JPRB, 0.47174E+00_JPRB, 0.45662E+00_JPRB ,&
&  0.44790E+00_JPRB, 0.44484E+00_JPRB, 0.44151E+00_JPRB, 0.43971E+00_JPRB, 0.43305E+00_JPRB, 0.42858E+00_JPRB, 0.42183E+00_JPRB ,&
&  0.41592E+00_JPRB, 0.41313E+00_JPRB, 0.41009E+00_JPRB, 0.41510E+00_JPRB, 0.42079E+00_JPRB, 0.42627E+00_JPRB, 0.42828E+00_JPRB ,&
&  0.42935E+00_JPRB, 0.43090E+00_JPRB, 0.43418E+00_JPRB, 0.43173E+00_JPRB, 0.44347E+00_JPRB, 0.46401E+00_JPRB, 0.47996E+00_JPRB ,&
&  0.49552E+00_JPRB, 0.50798E+00_JPRB, 0.50275E+00_JPRB, 0.47763E+00_JPRB, 0.41055E+00_JPRB, 0.37648E+00_JPRB, 0.34504E+00_JPRB ,&
&  0.32444E+00_JPRB, 0.30906E+00_JPRB, 0.29829E+00_JPRB, 0.29232E+00_JPRB, 0.28692E+00_JPRB, 0.28444E+00_JPRB, 0.28178E+00_JPRB ,&
&  0.27935E+00_JPRB, 0.27823E+00_JPRB, 0.27654E+00_JPRB, 0.27562E+00_JPRB, 0.27433E+00_JPRB, 0.27259E+00_JPRB, 0.27161E+00_JPRB ,&
&  0.27215E+00_JPRB, 0.27216E+00_JPRB, 0.27369E+00_JPRB, 0.27485E+00_JPRB, 0.27698E+00_JPRB, 0.27866E+00_JPRB, 0.28121E+00_JPRB ,&
&  0.28365E+00_JPRB, 0.28578E+00_JPRB, 0.28576E+00_JPRB, 0.28561E+00_JPRB, 0.28482E+00_JPRB, 0.28457E+00_JPRB, 0.28395E+00_JPRB ,&
&  0.28284E+00_JPRB, 0.28338E+00_JPRB, 0.28419E+00_JPRB, 0.28496E+00_JPRB, 0.28674E+00_JPRB, 0.28896E+00_JPRB, 0.29196E+00_JPRB ,&
&  0.29591E+00_JPRB, 0.29993E+00_JPRB, 0.30344E+00_JPRB, 0.30733E+00_JPRB, 0.31207E+00_JPRB, 0.31721E+00_JPRB, 0.32341E+00_JPRB ,&
&  0.32817E+00_JPRB, 0.33743E+00_JPRB, 0.35074E+00_JPRB, 0.36499E+00_JPRB, 0.38983E+00_JPRB, 0.40980E+00_JPRB, 0.44342E+00_JPRB ,&
&  0.48860E+00_JPRB, 0.64871E+00_JPRB, 0.92881E+00_JPRB, 0.71241E+00_JPRB, 0.70773E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.64154E+00_JPRB, 0.65184E+00_JPRB, 0.66600E+00_JPRB, 0.67765E+00_JPRB, 0.70091E+00_JPRB, 0.71404E+00_JPRB, 0.75741E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.63452E+00_JPRB, 0.60227E+00_JPRB, 0.56634E+00_JPRB, 0.52625E+00_JPRB, 0.49693E+00_JPRB, 0.47640E+00_JPRB ,&
&  0.46358E+00_JPRB, 0.45250E+00_JPRB, 0.44154E+00_JPRB, 0.42830E+00_JPRB, 0.41493E+00_JPRB, 0.40071E+00_JPRB, 0.39067E+00_JPRB ,&
&  0.38499E+00_JPRB, 0.38431E+00_JPRB, 0.38373E+00_JPRB, 0.38728E+00_JPRB, 0.39194E+00_JPRB, 0.39775E+00_JPRB, 0.40514E+00_JPRB ,&
&  0.42390E+00_JPRB, 0.43334E+00_JPRB, 0.43712E+00_JPRB, 0.43642E+00_JPRB, 0.43675E+00_JPRB, 0.44002E+00_JPRB, 0.43074E+00_JPRB ,&
&  0.43372E+00_JPRB, 0.44994E+00_JPRB, 0.45483E+00_JPRB, 0.43275E+00_JPRB, 0.37794E+00_JPRB, 0.34629E+00_JPRB, 0.32306E+00_JPRB ,&
&  0.31552E+00_JPRB, 0.30448E+00_JPRB, 0.29760E+00_JPRB, 0.29067E+00_JPRB, 0.28545E+00_JPRB, 0.28350E+00_JPRB, 0.28139E+00_JPRB ,&
&  0.28020E+00_JPRB, 0.27983E+00_JPRB, 0.27779E+00_JPRB, 0.27704E+00_JPRB, 0.27554E+00_JPRB, 0.27463E+00_JPRB, 0.27424E+00_JPRB ,&
&  0.27402E+00_JPRB, 0.27372E+00_JPRB, 0.27403E+00_JPRB, 0.27462E+00_JPRB, 0.27560E+00_JPRB, 0.27638E+00_JPRB, 0.27712E+00_JPRB ,&
&  0.27831E+00_JPRB, 0.27945E+00_JPRB, 0.27988E+00_JPRB, 0.28075E+00_JPRB, 0.28146E+00_JPRB, 0.28311E+00_JPRB, 0.28397E+00_JPRB ,&
&  0.28484E+00_JPRB, 0.28567E+00_JPRB, 0.28587E+00_JPRB, 0.28692E+00_JPRB, 0.28820E+00_JPRB, 0.29060E+00_JPRB, 0.29321E+00_JPRB ,&
&  0.29650E+00_JPRB, 0.30050E+00_JPRB, 0.30474E+00_JPRB, 0.30902E+00_JPRB, 0.31431E+00_JPRB, 0.31955E+00_JPRB, 0.32677E+00_JPRB ,&
&  0.33445E+00_JPRB, 0.34525E+00_JPRB, 0.36072E+00_JPRB, 0.37872E+00_JPRB, 0.40128E+00_JPRB, 0.42002E+00_JPRB, 0.43832E+00_JPRB ,&
&  0.47019E+00_JPRB, 0.56362E+00_JPRB, 0.62345E+00_JPRB, 0.61883E+00_JPRB, 0.63314E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.61572E+00_JPRB, 0.61929E+00_JPRB, 0.62300E+00_JPRB, 0.62809E+00_JPRB, 0.63369E+00_JPRB, 0.63866E+00_JPRB, 0.64243E+00_JPRB ,&
&  0.62358E+00_JPRB, 0.60044E+00_JPRB, 0.58115E+00_JPRB, 0.55805E+00_JPRB, 0.53691E+00_JPRB, 0.51818E+00_JPRB, 0.49776E+00_JPRB ,&
&  0.47713E+00_JPRB, 0.45668E+00_JPRB, 0.43384E+00_JPRB, 0.41498E+00_JPRB, 0.39654E+00_JPRB, 0.38164E+00_JPRB, 0.37332E+00_JPRB ,&
&  0.36826E+00_JPRB, 0.36662E+00_JPRB, 0.36761E+00_JPRB, 0.36908E+00_JPRB, 0.37085E+00_JPRB, 0.37491E+00_JPRB, 0.38103E+00_JPRB ,&
&  0.38957E+00_JPRB, 0.40593E+00_JPRB, 0.42030E+00_JPRB, 0.42502E+00_JPRB, 0.42036E+00_JPRB, 0.41088E+00_JPRB, 0.40251E+00_JPRB ,&
&  0.39274E+00_JPRB, 0.38086E+00_JPRB, 0.36544E+00_JPRB, 0.35001E+00_JPRB, 0.33565E+00_JPRB, 0.32522E+00_JPRB, 0.31597E+00_JPRB ,&
&  0.30791E+00_JPRB, 0.30066E+00_JPRB, 0.29490E+00_JPRB, 0.29111E+00_JPRB, 0.28784E+00_JPRB, 0.28535E+00_JPRB, 0.28282E+00_JPRB ,&
&  0.28176E+00_JPRB, 0.28070E+00_JPRB, 0.28006E+00_JPRB, 0.27970E+00_JPRB, 0.27906E+00_JPRB, 0.27878E+00_JPRB, 0.27860E+00_JPRB ,&
&  0.27816E+00_JPRB, 0.27744E+00_JPRB, 0.27705E+00_JPRB, 0.27682E+00_JPRB, 0.27742E+00_JPRB, 0.27798E+00_JPRB, 0.27879E+00_JPRB ,&
&  0.27994E+00_JPRB, 0.28216E+00_JPRB, 0.28346E+00_JPRB, 0.28472E+00_JPRB, 0.28538E+00_JPRB, 0.28605E+00_JPRB, 0.28693E+00_JPRB ,&
&  0.28830E+00_JPRB, 0.28987E+00_JPRB, 0.29121E+00_JPRB, 0.29302E+00_JPRB, 0.29486E+00_JPRB, 0.29741E+00_JPRB, 0.30022E+00_JPRB ,&
&  0.30323E+00_JPRB, 0.30696E+00_JPRB, 0.31147E+00_JPRB, 0.31542E+00_JPRB, 0.32191E+00_JPRB, 0.32849E+00_JPRB, 0.33600E+00_JPRB ,&
&  0.34401E+00_JPRB, 0.35273E+00_JPRB, 0.36454E+00_JPRB, 0.37869E+00_JPRB, 0.39693E+00_JPRB, 0.40411E+00_JPRB, 0.41341E+00_JPRB ,&
&  0.42804E+00_JPRB, 0.45838E+00_JPRB, 0.59589E+00_JPRB, 0.60947E+00_JPRB, 0.61509E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.39407E+00_JPRB, 0.39982E+00_JPRB, 0.40348E+00_JPRB, 0.41222E+00_JPRB, 0.42129E+00_JPRB, 0.52173E+00_JPRB, 0.51687E+00_JPRB ,&
&  0.51650E+00_JPRB, 0.47305E+00_JPRB, 0.46568E+00_JPRB, 0.45376E+00_JPRB, 0.43852E+00_JPRB, 0.41973E+00_JPRB, 0.40819E+00_JPRB ,&
&  0.39863E+00_JPRB, 0.38724E+00_JPRB, 0.37679E+00_JPRB, 0.37048E+00_JPRB, 0.36548E+00_JPRB, 0.36083E+00_JPRB, 0.35782E+00_JPRB ,&
&  0.35607E+00_JPRB, 0.35575E+00_JPRB, 0.35575E+00_JPRB, 0.35531E+00_JPRB, 0.35815E+00_JPRB, 0.36149E+00_JPRB, 0.36495E+00_JPRB ,&
&  0.36602E+00_JPRB, 0.37087E+00_JPRB, 0.37224E+00_JPRB, 0.36933E+00_JPRB, 0.36483E+00_JPRB, 0.35727E+00_JPRB, 0.34968E+00_JPRB ,&
&  0.34188E+00_JPRB, 0.33488E+00_JPRB, 0.32895E+00_JPRB, 0.32364E+00_JPRB, 0.31743E+00_JPRB, 0.31122E+00_JPRB, 0.30703E+00_JPRB ,&
&  0.30308E+00_JPRB, 0.29897E+00_JPRB, 0.29561E+00_JPRB, 0.29397E+00_JPRB, 0.29214E+00_JPRB, 0.29087E+00_JPRB, 0.28986E+00_JPRB ,&
&  0.28956E+00_JPRB, 0.28937E+00_JPRB, 0.28940E+00_JPRB, 0.28963E+00_JPRB, 0.28957E+00_JPRB, 0.28945E+00_JPRB, 0.28938E+00_JPRB ,&
&  0.29002E+00_JPRB, 0.29010E+00_JPRB, 0.29009E+00_JPRB, 0.28995E+00_JPRB, 0.29087E+00_JPRB, 0.29131E+00_JPRB, 0.29190E+00_JPRB ,&
&  0.29265E+00_JPRB, 0.29362E+00_JPRB, 0.29432E+00_JPRB, 0.29508E+00_JPRB, 0.29583E+00_JPRB, 0.29678E+00_JPRB, 0.29790E+00_JPRB ,&
&  0.29906E+00_JPRB, 0.30076E+00_JPRB, 0.30253E+00_JPRB, 0.30405E+00_JPRB, 0.30544E+00_JPRB, 0.30730E+00_JPRB, 0.30942E+00_JPRB ,&
&  0.31243E+00_JPRB, 0.31532E+00_JPRB, 0.31826E+00_JPRB, 0.31974E+00_JPRB, 0.32244E+00_JPRB, 0.32477E+00_JPRB, 0.32741E+00_JPRB ,&
&  0.33068E+00_JPRB, 0.33620E+00_JPRB, 0.34455E+00_JPRB, 0.35393E+00_JPRB, 0.36480E+00_JPRB, 0.37234E+00_JPRB, 0.37708E+00_JPRB ,&
&  0.38085E+00_JPRB, 0.38507E+00_JPRB, 0.38943E+00_JPRB, 0.39164E+00_JPRB, 0.39325E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.36018E+00_JPRB, 0.36004E+00_JPRB, 0.36009E+00_JPRB, 0.35969E+00_JPRB, 0.35787E+00_JPRB, 0.35634E+00_JPRB, 0.35490E+00_JPRB ,&
&  0.35394E+00_JPRB, 0.35278E+00_JPRB, 0.35220E+00_JPRB, 0.35183E+00_JPRB, 0.35122E+00_JPRB, 0.35042E+00_JPRB, 0.34938E+00_JPRB ,&
&  0.34801E+00_JPRB, 0.34625E+00_JPRB, 0.34419E+00_JPRB, 0.34349E+00_JPRB, 0.34315E+00_JPRB, 0.34280E+00_JPRB, 0.34237E+00_JPRB ,&
&  0.34178E+00_JPRB, 0.34130E+00_JPRB, 0.34046E+00_JPRB, 0.34060E+00_JPRB, 0.34051E+00_JPRB, 0.34043E+00_JPRB, 0.34046E+00_JPRB ,&
&  0.34056E+00_JPRB, 0.34085E+00_JPRB, 0.34076E+00_JPRB, 0.34034E+00_JPRB, 0.33956E+00_JPRB, 0.33786E+00_JPRB, 0.33513E+00_JPRB ,&
&  0.33201E+00_JPRB, 0.32891E+00_JPRB, 0.32641E+00_JPRB, 0.32412E+00_JPRB, 0.32202E+00_JPRB, 0.32012E+00_JPRB, 0.31856E+00_JPRB ,&
&  0.31663E+00_JPRB, 0.31471E+00_JPRB, 0.31292E+00_JPRB, 0.31158E+00_JPRB, 0.31007E+00_JPRB, 0.30873E+00_JPRB, 0.30741E+00_JPRB ,&
&  0.30647E+00_JPRB, 0.30566E+00_JPRB, 0.30497E+00_JPRB, 0.30460E+00_JPRB, 0.30431E+00_JPRB, 0.30407E+00_JPRB, 0.30379E+00_JPRB ,&
&  0.30354E+00_JPRB, 0.30342E+00_JPRB, 0.30335E+00_JPRB, 0.30338E+00_JPRB, 0.30356E+00_JPRB, 0.30353E+00_JPRB, 0.30346E+00_JPRB ,&
&  0.30349E+00_JPRB, 0.30391E+00_JPRB, 0.30415E+00_JPRB, 0.30447E+00_JPRB, 0.30497E+00_JPRB, 0.30566E+00_JPRB, 0.30641E+00_JPRB ,&
&  0.30716E+00_JPRB, 0.30803E+00_JPRB, 0.30896E+00_JPRB, 0.31014E+00_JPRB, 0.31142E+00_JPRB, 0.31270E+00_JPRB, 0.31392E+00_JPRB ,&
&  0.31524E+00_JPRB, 0.31646E+00_JPRB, 0.31763E+00_JPRB, 0.31841E+00_JPRB, 0.31966E+00_JPRB, 0.32097E+00_JPRB, 0.32252E+00_JPRB ,&
&  0.32430E+00_JPRB, 0.32670E+00_JPRB, 0.33018E+00_JPRB, 0.33434E+00_JPRB, 0.33980E+00_JPRB, 0.34364E+00_JPRB, 0.34736E+00_JPRB ,&
&  0.34991E+00_JPRB, 0.35197E+00_JPRB, 0.35359E+00_JPRB, 0.35556E+00_JPRB, 0.35775E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB, 0.32288E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2030:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2030
