!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2040 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2040* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB, 0.54346E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.56441E+00_JPRB, 0.56352E+00_JPRB, 0.56268E+00_JPRB, 0.56193E+00_JPRB, 0.56123E+00_JPRB, 0.56070E+00_JPRB, 0.56024E+00_JPRB ,&
&  0.55963E+00_JPRB, 0.55907E+00_JPRB, 0.55837E+00_JPRB, 0.55769E+00_JPRB, 0.55704E+00_JPRB, 0.55632E+00_JPRB, 0.55543E+00_JPRB ,&
&  0.55434E+00_JPRB, 0.55334E+00_JPRB, 0.55227E+00_JPRB, 0.55100E+00_JPRB, 0.54974E+00_JPRB, 0.54837E+00_JPRB, 0.54674E+00_JPRB ,&
&  0.54487E+00_JPRB, 0.54303E+00_JPRB, 0.54109E+00_JPRB, 0.53908E+00_JPRB, 0.53686E+00_JPRB, 0.53439E+00_JPRB, 0.53214E+00_JPRB ,&
&  0.53048E+00_JPRB, 0.52868E+00_JPRB, 0.52699E+00_JPRB, 0.52530E+00_JPRB, 0.52395E+00_JPRB, 0.52292E+00_JPRB, 0.52183E+00_JPRB ,&
&  0.52088E+00_JPRB, 0.52010E+00_JPRB, 0.52005E+00_JPRB, 0.52027E+00_JPRB, 0.52066E+00_JPRB, 0.52172E+00_JPRB, 0.52343E+00_JPRB ,&
&  0.52487E+00_JPRB, 0.52610E+00_JPRB, 0.52761E+00_JPRB, 0.52933E+00_JPRB, 0.53112E+00_JPRB, 0.53293E+00_JPRB, 0.53461E+00_JPRB ,&
&  0.53591E+00_JPRB, 0.53696E+00_JPRB, 0.53809E+00_JPRB, 0.53906E+00_JPRB, 0.53962E+00_JPRB, 0.54014E+00_JPRB, 0.54072E+00_JPRB ,&
&  0.54118E+00_JPRB, 0.54152E+00_JPRB, 0.54198E+00_JPRB, 0.54275E+00_JPRB, 0.54344E+00_JPRB, 0.54423E+00_JPRB, 0.54529E+00_JPRB ,&
&  0.54668E+00_JPRB, 0.54776E+00_JPRB, 0.54907E+00_JPRB, 0.55071E+00_JPRB, 0.55280E+00_JPRB, 0.55524E+00_JPRB, 0.55723E+00_JPRB ,&
&  0.55931E+00_JPRB, 0.56150E+00_JPRB, 0.56383E+00_JPRB, 0.56555E+00_JPRB, 0.56739E+00_JPRB, 0.56912E+00_JPRB, 0.57078E+00_JPRB ,&
&  0.57139E+00_JPRB, 0.57226E+00_JPRB, 0.57334E+00_JPRB, 0.57438E+00_JPRB, 0.57401E+00_JPRB, 0.57378E+00_JPRB, 0.57363E+00_JPRB ,&
&  0.57422E+00_JPRB, 0.57312E+00_JPRB, 0.57262E+00_JPRB, 0.57190E+00_JPRB, 0.57153E+00_JPRB, 0.57035E+00_JPRB, 0.56949E+00_JPRB ,&
&  0.56859E+00_JPRB, 0.56789E+00_JPRB, 0.56699E+00_JPRB, 0.56607E+00_JPRB, 0.56516E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.58555E+00_JPRB, 0.58395E+00_JPRB, 0.58225E+00_JPRB, 0.58050E+00_JPRB, 0.57851E+00_JPRB, 0.57694E+00_JPRB, 0.57542E+00_JPRB ,&
&  0.57385E+00_JPRB, 0.57248E+00_JPRB, 0.57099E+00_JPRB, 0.56973E+00_JPRB, 0.56859E+00_JPRB, 0.56800E+00_JPRB, 0.56693E+00_JPRB ,&
&  0.56571E+00_JPRB, 0.56403E+00_JPRB, 0.56252E+00_JPRB, 0.56012E+00_JPRB, 0.55743E+00_JPRB, 0.55384E+00_JPRB, 0.55051E+00_JPRB ,&
&  0.54669E+00_JPRB, 0.54222E+00_JPRB, 0.53828E+00_JPRB, 0.53434E+00_JPRB, 0.53050E+00_JPRB, 0.52647E+00_JPRB, 0.52357E+00_JPRB ,&
&  0.52165E+00_JPRB, 0.51895E+00_JPRB, 0.51660E+00_JPRB, 0.51467E+00_JPRB, 0.51269E+00_JPRB, 0.51115E+00_JPRB, 0.50968E+00_JPRB ,&
&  0.50696E+00_JPRB, 0.50402E+00_JPRB, 0.50397E+00_JPRB, 0.50435E+00_JPRB, 0.50558E+00_JPRB, 0.50834E+00_JPRB, 0.51335E+00_JPRB ,&
&  0.51999E+00_JPRB, 0.52629E+00_JPRB, 0.53318E+00_JPRB, 0.53930E+00_JPRB, 0.54413E+00_JPRB, 0.54794E+00_JPRB, 0.55104E+00_JPRB ,&
&  0.55306E+00_JPRB, 0.55455E+00_JPRB, 0.55530E+00_JPRB, 0.55519E+00_JPRB, 0.55427E+00_JPRB, 0.55390E+00_JPRB, 0.55396E+00_JPRB ,&
&  0.55431E+00_JPRB, 0.55276E+00_JPRB, 0.55243E+00_JPRB, 0.55310E+00_JPRB, 0.55460E+00_JPRB, 0.55614E+00_JPRB, 0.55899E+00_JPRB ,&
&  0.56289E+00_JPRB, 0.56763E+00_JPRB, 0.57208E+00_JPRB, 0.57790E+00_JPRB, 0.58465E+00_JPRB, 0.59237E+00_JPRB, 0.59712E+00_JPRB ,&
&  0.60287E+00_JPRB, 0.60896E+00_JPRB, 0.61535E+00_JPRB, 0.61711E+00_JPRB, 0.61911E+00_JPRB, 0.62076E+00_JPRB, 0.62276E+00_JPRB ,&
&  0.62074E+00_JPRB, 0.62033E+00_JPRB, 0.62034E+00_JPRB, 0.62187E+00_JPRB, 0.61944E+00_JPRB, 0.61851E+00_JPRB, 0.61748E+00_JPRB ,&
&  0.61854E+00_JPRB, 0.61416E+00_JPRB, 0.61171E+00_JPRB, 0.60847E+00_JPRB, 0.60653E+00_JPRB, 0.60193E+00_JPRB, 0.59835E+00_JPRB ,&
&  0.59534E+00_JPRB, 0.59349E+00_JPRB, 0.59108E+00_JPRB, 0.58903E+00_JPRB, 0.58737E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.60941E+00_JPRB, 0.60582E+00_JPRB, 0.60052E+00_JPRB, 0.59381E+00_JPRB, 0.58746E+00_JPRB, 0.58261E+00_JPRB, 0.57742E+00_JPRB ,&
&  0.57317E+00_JPRB, 0.56925E+00_JPRB, 0.56643E+00_JPRB, 0.56425E+00_JPRB, 0.56211E+00_JPRB, 0.56241E+00_JPRB, 0.56274E+00_JPRB ,&
&  0.56307E+00_JPRB, 0.56239E+00_JPRB, 0.56357E+00_JPRB, 0.56242E+00_JPRB, 0.56037E+00_JPRB, 0.55617E+00_JPRB, 0.55033E+00_JPRB ,&
&  0.54399E+00_JPRB, 0.53687E+00_JPRB, 0.52889E+00_JPRB, 0.51980E+00_JPRB, 0.51122E+00_JPRB, 0.48184E+00_JPRB, 0.47485E+00_JPRB ,&
&  0.47751E+00_JPRB, 0.47277E+00_JPRB, 0.46873E+00_JPRB, 0.46627E+00_JPRB, 0.46079E+00_JPRB, 0.45660E+00_JPRB, 0.45291E+00_JPRB ,&
&  0.43034E+00_JPRB, 0.43000E+00_JPRB, 0.42787E+00_JPRB, 0.42663E+00_JPRB, 0.43039E+00_JPRB, 0.44261E+00_JPRB, 0.46200E+00_JPRB ,&
&  0.49702E+00_JPRB, 0.51201E+00_JPRB, 0.52486E+00_JPRB, 0.53989E+00_JPRB, 0.55167E+00_JPRB, 0.56048E+00_JPRB, 0.56926E+00_JPRB ,&
&  0.57656E+00_JPRB, 0.58362E+00_JPRB, 0.58876E+00_JPRB, 0.59130E+00_JPRB, 0.59148E+00_JPRB, 0.59128E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.59009E+00_JPRB, 0.58940E+00_JPRB, 0.58960E+00_JPRB, 0.58980E+00_JPRB, 0.58967E+00_JPRB, 0.59094E+00_JPRB, 0.59564E+00_JPRB ,&
&  0.60246E+00_JPRB, 0.61249E+00_JPRB, 0.62128E+00_JPRB, 0.63340E+00_JPRB, 0.64458E+00_JPRB, 0.65854E+00_JPRB, 0.66427E+00_JPRB ,&
&  0.67117E+00_JPRB, 0.67566E+00_JPRB, 0.68169E+00_JPRB, 0.68193E+00_JPRB, 0.68209E+00_JPRB, 0.68125E+00_JPRB, 0.68181E+00_JPRB ,&
&  0.67855E+00_JPRB, 0.67642E+00_JPRB, 0.67356E+00_JPRB, 0.67364E+00_JPRB, 0.66987E+00_JPRB, 0.66950E+00_JPRB, 0.66748E+00_JPRB ,&
&  0.66953E+00_JPRB, 0.66470E+00_JPRB, 0.66297E+00_JPRB, 0.65688E+00_JPRB, 0.65204E+00_JPRB, 0.64305E+00_JPRB, 0.63524E+00_JPRB ,&
&  0.62854E+00_JPRB, 0.62346E+00_JPRB, 0.61919E+00_JPRB, 0.61557E+00_JPRB, 0.61228E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.62819E+00_JPRB, 0.61811E+00_JPRB, 0.60689E+00_JPRB, 0.59740E+00_JPRB, 0.58852E+00_JPRB, 0.58192E+00_JPRB, 0.57557E+00_JPRB ,&
&  0.57067E+00_JPRB, 0.56591E+00_JPRB, 0.56227E+00_JPRB, 0.55883E+00_JPRB, 0.55612E+00_JPRB, 0.55424E+00_JPRB, 0.55246E+00_JPRB ,&
&  0.55165E+00_JPRB, 0.55055E+00_JPRB, 0.55064E+00_JPRB, 0.54997E+00_JPRB, 0.54987E+00_JPRB, 0.54686E+00_JPRB, 0.54075E+00_JPRB ,&
&  0.53376E+00_JPRB, 0.52633E+00_JPRB, 0.50208E+00_JPRB, 0.49397E+00_JPRB, 0.47392E+00_JPRB, 0.45925E+00_JPRB, 0.45057E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.42567E+00_JPRB, 0.40926E+00_JPRB, 0.40030E+00_JPRB, 0.39909E+00_JPRB, 0.38937E+00_JPRB, 0.38588E+00_JPRB ,&
&  0.37902E+00_JPRB, 0.36597E+00_JPRB, 0.36107E+00_JPRB, 0.35887E+00_JPRB, 0.36284E+00_JPRB, 0.37346E+00_JPRB, 0.39318E+00_JPRB ,&
&  0.43054E+00_JPRB, 0.47227E+00_JPRB, 0.52927E+00_JPRB, 0.54578E+00_JPRB, 0.54903E+00_JPRB, 0.56245E+00_JPRB, 0.57336E+00_JPRB ,&
&  0.58012E+00_JPRB, 0.59485E+00_JPRB, 0.61044E+00_JPRB, 0.61701E+00_JPRB, 0.62130E+00_JPRB, 0.62286E+00_JPRB, 0.62417E+00_JPRB ,&
&  0.62533E+00_JPRB, 0.63132E+00_JPRB, 0.63496E+00_JPRB, 0.63891E+00_JPRB, 0.64320E+00_JPRB, 0.64815E+00_JPRB, 0.65817E+00_JPRB ,&
&  0.66695E+00_JPRB, 0.68481E+00_JPRB, 0.69941E+00_JPRB, 0.72165E+00_JPRB, 0.73105E+00_JPRB, 0.74217E+00_JPRB, 0.74523E+00_JPRB ,&
&  0.75046E+00_JPRB, 0.75349E+00_JPRB, 0.75584E+00_JPRB, 0.74964E+00_JPRB, 0.74829E+00_JPRB, 0.75283E+00_JPRB, 0.75136E+00_JPRB ,&
&  0.74899E+00_JPRB, 0.74774E+00_JPRB, 0.74568E+00_JPRB, 0.74432E+00_JPRB, 0.74351E+00_JPRB, 0.74535E+00_JPRB, 0.74516E+00_JPRB ,&
&  0.74924E+00_JPRB, 0.74636E+00_JPRB, 0.74359E+00_JPRB, 0.73641E+00_JPRB, 0.72697E+00_JPRB, 0.71325E+00_JPRB, 0.69569E+00_JPRB ,&
&  0.68058E+00_JPRB, 0.66878E+00_JPRB, 0.65656E+00_JPRB, 0.64726E+00_JPRB, 0.63750E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.60364E+00_JPRB, 0.59068E+00_JPRB, 0.57979E+00_JPRB, 0.57202E+00_JPRB, 0.56531E+00_JPRB, 0.56097E+00_JPRB, 0.55678E+00_JPRB ,&
&  0.55453E+00_JPRB, 0.55180E+00_JPRB, 0.55055E+00_JPRB, 0.54895E+00_JPRB, 0.54845E+00_JPRB, 0.54831E+00_JPRB, 0.54884E+00_JPRB ,&
&  0.54963E+00_JPRB, 0.54950E+00_JPRB, 0.54969E+00_JPRB, 0.54696E+00_JPRB, 0.54450E+00_JPRB, 0.54066E+00_JPRB, 0.53688E+00_JPRB ,&
&  0.52902E+00_JPRB, 0.51973E+00_JPRB, 0.50199E+00_JPRB, 0.49272E+00_JPRB, 0.47458E+00_JPRB, 0.46151E+00_JPRB, 0.44819E+00_JPRB ,&
&  0.43560E+00_JPRB, 0.41260E+00_JPRB, 0.42043E+00_JPRB, 0.40601E+00_JPRB, 0.39237E+00_JPRB, 0.37774E+00_JPRB, 0.35987E+00_JPRB ,&
&  0.35317E+00_JPRB, 0.33580E+00_JPRB, 0.32946E+00_JPRB, 0.32966E+00_JPRB, 0.33216E+00_JPRB, 0.34364E+00_JPRB, 0.36176E+00_JPRB ,&
&  0.39759E+00_JPRB, 0.48894E+00_JPRB, 0.52185E+00_JPRB, 0.53668E+00_JPRB, 0.53381E+00_JPRB, 0.53580E+00_JPRB, 0.53138E+00_JPRB ,&
&  0.54754E+00_JPRB, 0.56904E+00_JPRB, 0.59122E+00_JPRB, 0.60633E+00_JPRB, 0.62463E+00_JPRB, 0.63466E+00_JPRB, 0.64360E+00_JPRB ,&
&  0.65420E+00_JPRB, 0.67092E+00_JPRB, 0.68623E+00_JPRB, 0.70184E+00_JPRB, 0.72125E+00_JPRB, 0.74029E+00_JPRB, 0.76258E+00_JPRB ,&
&  0.77628E+00_JPRB, 0.80387E+00_JPRB, 0.81367E+00_JPRB, 0.82667E+00_JPRB, 0.82861E+00_JPRB, 0.82711E+00_JPRB, 0.82811E+00_JPRB ,&
&  0.83180E+00_JPRB, 0.83628E+00_JPRB, 0.83978E+00_JPRB, 0.83812E+00_JPRB, 0.83743E+00_JPRB, 0.82677E+00_JPRB, 0.82300E+00_JPRB ,&
&  0.82343E+00_JPRB, 0.82238E+00_JPRB, 0.82010E+00_JPRB, 0.81499E+00_JPRB, 0.80766E+00_JPRB, 0.79968E+00_JPRB, 0.79543E+00_JPRB ,&
&  0.78968E+00_JPRB, 0.78040E+00_JPRB, 0.77011E+00_JPRB, 0.75895E+00_JPRB, 0.74728E+00_JPRB, 0.73500E+00_JPRB, 0.71476E+00_JPRB ,&
&  0.69000E+00_JPRB, 0.66879E+00_JPRB, 0.64895E+00_JPRB, 0.63376E+00_JPRB, 0.61785E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.58456E+00_JPRB, 0.57225E+00_JPRB, 0.56185E+00_JPRB, 0.55606E+00_JPRB, 0.55036E+00_JPRB, 0.54794E+00_JPRB, 0.54448E+00_JPRB ,&
&  0.54270E+00_JPRB, 0.54100E+00_JPRB, 0.54041E+00_JPRB, 0.53991E+00_JPRB, 0.54020E+00_JPRB, 0.54070E+00_JPRB, 0.54202E+00_JPRB ,&
&  0.54341E+00_JPRB, 0.54557E+00_JPRB, 0.54763E+00_JPRB, 0.54663E+00_JPRB, 0.54530E+00_JPRB, 0.54264E+00_JPRB, 0.53930E+00_JPRB ,&
&  0.53249E+00_JPRB, 0.52370E+00_JPRB, 0.50642E+00_JPRB, 0.49461E+00_JPRB, 0.48407E+00_JPRB, 0.46851E+00_JPRB, 0.46483E+00_JPRB ,&
&  0.46006E+00_JPRB, 0.44726E+00_JPRB, 0.42943E+00_JPRB, 0.42135E+00_JPRB, 0.40961E+00_JPRB, 0.37250E+00_JPRB, 0.37398E+00_JPRB ,&
&  0.36421E+00_JPRB, 0.33924E+00_JPRB, 0.33201E+00_JPRB, 0.32686E+00_JPRB, 0.32859E+00_JPRB, 0.33924E+00_JPRB, 0.35545E+00_JPRB ,&
&  0.38939E+00_JPRB, 0.49492E+00_JPRB, 0.52158E+00_JPRB, 0.53121E+00_JPRB, 0.52129E+00_JPRB, 0.52534E+00_JPRB, 0.52572E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.54817E+00_JPRB, 0.56189E+00_JPRB, 0.57384E+00_JPRB, 0.58626E+00_JPRB, 0.59269E+00_JPRB, 0.60599E+00_JPRB ,&
&  0.63086E+00_JPRB, 0.66232E+00_JPRB, 0.69133E+00_JPRB, 0.72780E+00_JPRB, 0.77250E+00_JPRB, 0.85010E+00_JPRB, 0.92876E+00_JPRB ,&
&  0.93535E+00_JPRB, 0.92562E+00_JPRB, 0.92243E+00_JPRB, 0.91505E+00_JPRB, 0.91429E+00_JPRB, 0.91400E+00_JPRB, 0.91906E+00_JPRB ,&
&  0.92994E+00_JPRB, 0.93927E+00_JPRB, 0.92273E+00_JPRB, 0.89208E+00_JPRB, 0.87879E+00_JPRB, 0.85722E+00_JPRB, 0.85483E+00_JPRB ,&
&  0.85099E+00_JPRB, 0.86399E+00_JPRB, 0.85820E+00_JPRB, 0.84681E+00_JPRB, 0.83574E+00_JPRB, 0.82326E+00_JPRB, 0.81594E+00_JPRB ,&
&  0.80693E+00_JPRB, 0.79321E+00_JPRB, 0.77809E+00_JPRB, 0.76527E+00_JPRB, 0.75365E+00_JPRB, 0.74428E+00_JPRB, 0.73019E+00_JPRB ,&
&  0.70297E+00_JPRB, 0.66920E+00_JPRB, 0.64334E+00_JPRB, 0.61924E+00_JPRB, 0.59966E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.57492E+00_JPRB, 0.56693E+00_JPRB, 0.56193E+00_JPRB, 0.56119E+00_JPRB, 0.55967E+00_JPRB, 0.55929E+00_JPRB, 0.55568E+00_JPRB ,&
&  0.55213E+00_JPRB, 0.54842E+00_JPRB, 0.54509E+00_JPRB, 0.54212E+00_JPRB, 0.54037E+00_JPRB, 0.53858E+00_JPRB, 0.53756E+00_JPRB ,&
&  0.53676E+00_JPRB, 0.53056E+00_JPRB, 0.52872E+00_JPRB, 0.53100E+00_JPRB, 0.53350E+00_JPRB, 0.53504E+00_JPRB, 0.53735E+00_JPRB ,&
&  0.53334E+00_JPRB, 0.52564E+00_JPRB, 0.51561E+00_JPRB, 0.50253E+00_JPRB, 0.49289E+00_JPRB, 0.49013E+00_JPRB, 0.48285E+00_JPRB ,&
&  0.46580E+00_JPRB, 0.45143E+00_JPRB, 0.45681E+00_JPRB, 0.45173E+00_JPRB, 0.42474E+00_JPRB, 0.41044E+00_JPRB, 0.39696E+00_JPRB ,&
&  0.37576E+00_JPRB, 0.34483E+00_JPRB, 0.33790E+00_JPRB, 0.33502E+00_JPRB, 0.34161E+00_JPRB, 0.35231E+00_JPRB, 0.37600E+00_JPRB ,&
&  0.41779E+00_JPRB, 0.50116E+00_JPRB, 0.52404E+00_JPRB, 0.53585E+00_JPRB, 0.54122E+00_JPRB, 0.54056E+00_JPRB, 0.53998E+00_JPRB ,&
&  0.54750E+00_JPRB, 0.55481E+00_JPRB, 0.56660E+00_JPRB, 0.57984E+00_JPRB, 0.59727E+00_JPRB, 0.61245E+00_JPRB, 0.62434E+00_JPRB ,&
&  0.66020E+00_JPRB, 0.68100E+00_JPRB, 0.70172E+00_JPRB, 0.76470E+00_JPRB, 0.87873E+00_JPRB, 0.99885E+00_JPRB, 0.10472E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10259E+01_JPRB, 0.99083E+00_JPRB, 0.97038E+00_JPRB, 0.96161E+00_JPRB, 0.95932E+00_JPRB, 0.96165E+00_JPRB ,&
&  0.96891E+00_JPRB, 0.98658E+00_JPRB, 0.10452E+01_JPRB, 0.99721E+00_JPRB, 0.96851E+00_JPRB, 0.93916E+00_JPRB, 0.90566E+00_JPRB ,&
&  0.88013E+00_JPRB, 0.86644E+00_JPRB, 0.85772E+00_JPRB, 0.84593E+00_JPRB, 0.83519E+00_JPRB, 0.82669E+00_JPRB, 0.82758E+00_JPRB ,&
&  0.82164E+00_JPRB, 0.80920E+00_JPRB, 0.78768E+00_JPRB, 0.77513E+00_JPRB, 0.76298E+00_JPRB, 0.75119E+00_JPRB, 0.73878E+00_JPRB ,&
&  0.70986E+00_JPRB, 0.67128E+00_JPRB, 0.64076E+00_JPRB, 0.61361E+00_JPRB, 0.59115E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.58132E+00_JPRB, 0.57786E+00_JPRB, 0.57991E+00_JPRB, 0.58670E+00_JPRB, 0.59101E+00_JPRB, 0.58381E+00_JPRB, 0.57452E+00_JPRB ,&
&  0.56765E+00_JPRB, 0.56464E+00_JPRB, 0.56237E+00_JPRB, 0.55763E+00_JPRB, 0.55401E+00_JPRB, 0.55051E+00_JPRB, 0.54714E+00_JPRB ,&
&  0.54382E+00_JPRB, 0.53993E+00_JPRB, 0.53506E+00_JPRB, 0.52804E+00_JPRB, 0.52403E+00_JPRB, 0.52496E+00_JPRB, 0.52928E+00_JPRB ,&
&  0.53323E+00_JPRB, 0.53219E+00_JPRB, 0.52856E+00_JPRB, 0.51760E+00_JPRB, 0.50598E+00_JPRB, 0.49669E+00_JPRB, 0.49684E+00_JPRB ,&
&  0.48974E+00_JPRB, 0.47457E+00_JPRB, 0.45697E+00_JPRB, 0.46229E+00_JPRB, 0.46343E+00_JPRB, 0.43578E+00_JPRB, 0.41179E+00_JPRB ,&
&  0.38829E+00_JPRB, 0.36721E+00_JPRB, 0.35779E+00_JPRB, 0.35728E+00_JPRB, 0.36900E+00_JPRB, 0.39035E+00_JPRB, 0.41909E+00_JPRB ,&
&  0.46000E+00_JPRB, 0.50173E+00_JPRB, 0.52164E+00_JPRB, 0.53714E+00_JPRB, 0.53984E+00_JPRB, 0.53752E+00_JPRB, 0.53496E+00_JPRB ,&
&  0.53683E+00_JPRB, 0.54172E+00_JPRB, 0.55044E+00_JPRB, 0.55708E+00_JPRB, 0.57047E+00_JPRB, 0.59953E+00_JPRB, 0.66324E+00_JPRB ,&
&  0.82107E+00_JPRB, 0.83431E+00_JPRB, 0.77071E+00_JPRB, 0.84350E+00_JPRB, 0.96376E+00_JPRB, 0.10704E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.11489E+01_JPRB, 0.10936E+01_JPRB, 0.10481E+01_JPRB, 0.10226E+01_JPRB, 0.10048E+01_JPRB, 0.98971E+00_JPRB, 0.98314E+00_JPRB ,&
&  0.98635E+00_JPRB, 0.10053E+01_JPRB, 0.10964E+01_JPRB, 0.97050E+00_JPRB, 0.90629E+00_JPRB, 0.88508E+00_JPRB, 0.86086E+00_JPRB ,&
&  0.83894E+00_JPRB, 0.82835E+00_JPRB, 0.83194E+00_JPRB, 0.84155E+00_JPRB, 0.81252E+00_JPRB, 0.78675E+00_JPRB, 0.77067E+00_JPRB ,&
&  0.76923E+00_JPRB, 0.76149E+00_JPRB, 0.75617E+00_JPRB, 0.76084E+00_JPRB, 0.75812E+00_JPRB, 0.73925E+00_JPRB, 0.72696E+00_JPRB ,&
&  0.70564E+00_JPRB, 0.67049E+00_JPRB, 0.64585E+00_JPRB, 0.62182E+00_JPRB, 0.59680E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.58240E+00_JPRB, 0.59541E+00_JPRB, 0.61301E+00_JPRB, 0.62472E+00_JPRB, 0.62686E+00_JPRB, 0.60120E+00_JPRB, 0.57500E+00_JPRB ,&
&  0.55967E+00_JPRB, 0.55396E+00_JPRB, 0.55470E+00_JPRB, 0.55478E+00_JPRB, 0.55392E+00_JPRB, 0.55351E+00_JPRB, 0.55197E+00_JPRB ,&
&  0.54643E+00_JPRB, 0.53946E+00_JPRB, 0.53313E+00_JPRB, 0.52707E+00_JPRB, 0.52240E+00_JPRB, 0.52114E+00_JPRB, 0.52275E+00_JPRB ,&
&  0.52640E+00_JPRB, 0.52848E+00_JPRB, 0.52843E+00_JPRB, 0.52285E+00_JPRB, 0.51940E+00_JPRB, 0.51623E+00_JPRB, 0.50680E+00_JPRB ,&
&  0.48999E+00_JPRB, 0.48711E+00_JPRB, 0.48178E+00_JPRB, 0.47477E+00_JPRB, 0.47658E+00_JPRB, 0.45852E+00_JPRB, 0.43277E+00_JPRB ,&
&  0.40851E+00_JPRB, 0.39080E+00_JPRB, 0.38838E+00_JPRB, 0.39827E+00_JPRB, 0.42213E+00_JPRB, 0.45161E+00_JPRB, 0.47052E+00_JPRB ,&
&  0.48807E+00_JPRB, 0.50337E+00_JPRB, 0.51449E+00_JPRB, 0.52440E+00_JPRB, 0.53144E+00_JPRB, 0.53420E+00_JPRB, 0.53406E+00_JPRB ,&
&  0.53210E+00_JPRB, 0.53165E+00_JPRB, 0.53314E+00_JPRB, 0.51664E+00_JPRB, 0.52954E+00_JPRB, 0.61199E+00_JPRB, 0.69973E+00_JPRB ,&
&  0.80447E+00_JPRB, 0.83875E+00_JPRB, 0.86930E+00_JPRB, 0.95546E+00_JPRB, 0.10344E+01_JPRB, 0.11019E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.11084E+01_JPRB, 0.10542E+01_JPRB, 0.10353E+01_JPRB, 0.10265E+01_JPRB, 0.10165E+01_JPRB, 0.97942E+00_JPRB ,&
&  0.95206E+00_JPRB, 0.90749E+00_JPRB, 0.84843E+00_JPRB, 0.79734E+00_JPRB, 0.78923E+00_JPRB, 0.80988E+00_JPRB, 0.84246E+00_JPRB ,&
&  0.84805E+00_JPRB, 0.84785E+00_JPRB, 0.84954E+00_JPRB, 0.86083E+00_JPRB, 0.79417E+00_JPRB, 0.75191E+00_JPRB, 0.72565E+00_JPRB ,&
&  0.71826E+00_JPRB, 0.71496E+00_JPRB, 0.71394E+00_JPRB, 0.72854E+00_JPRB, 0.73189E+00_JPRB, 0.72456E+00_JPRB, 0.69652E+00_JPRB ,&
&  0.65866E+00_JPRB, 0.62770E+00_JPRB, 0.60481E+00_JPRB, 0.58911E+00_JPRB, 0.58303E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.64561E+00_JPRB, 0.65615E+00_JPRB, 0.66071E+00_JPRB, 0.66265E+00_JPRB, 0.65372E+00_JPRB, 0.63929E+00_JPRB, 0.62443E+00_JPRB ,&
&  0.61948E+00_JPRB, 0.60643E+00_JPRB, 0.59283E+00_JPRB, 0.57454E+00_JPRB, 0.55843E+00_JPRB, 0.54488E+00_JPRB, 0.54189E+00_JPRB ,&
&  0.54397E+00_JPRB, 0.54614E+00_JPRB, 0.54309E+00_JPRB, 0.53526E+00_JPRB, 0.52776E+00_JPRB, 0.52366E+00_JPRB, 0.52386E+00_JPRB ,&
&  0.52607E+00_JPRB, 0.52941E+00_JPRB, 0.53112E+00_JPRB, 0.53031E+00_JPRB, 0.52929E+00_JPRB, 0.52696E+00_JPRB, 0.52219E+00_JPRB ,&
&  0.51106E+00_JPRB, 0.50319E+00_JPRB, 0.49860E+00_JPRB, 0.49579E+00_JPRB, 0.49822E+00_JPRB, 0.48102E+00_JPRB, 0.45567E+00_JPRB ,&
&  0.43220E+00_JPRB, 0.42437E+00_JPRB, 0.43662E+00_JPRB, 0.46681E+00_JPRB, 0.49048E+00_JPRB, 0.51038E+00_JPRB, 0.52371E+00_JPRB ,&
&  0.52298E+00_JPRB, 0.52099E+00_JPRB, 0.52841E+00_JPRB, 0.54557E+00_JPRB, 0.54829E+00_JPRB, 0.54891E+00_JPRB, 0.54592E+00_JPRB ,&
&  0.54126E+00_JPRB, 0.54026E+00_JPRB, 0.53960E+00_JPRB, 0.52870E+00_JPRB, 0.59387E+00_JPRB, 0.72662E+00_JPRB, 0.77600E+00_JPRB ,&
&  0.87140E+00_JPRB, 0.94147E+00_JPRB, 0.10325E+01_JPRB, 0.10574E+01_JPRB, 0.10457E+01_JPRB, 0.10779E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.11680E+01_JPRB, 0.11408E+01_JPRB, 0.11061E+01_JPRB, 0.10937E+01_JPRB, 0.10522E+01_JPRB, 0.10074E+01_JPRB, 0.96596E+00_JPRB ,&
&  0.95681E+00_JPRB, 0.93507E+00_JPRB, 0.88994E+00_JPRB, 0.83757E+00_JPRB, 0.81825E+00_JPRB, 0.81597E+00_JPRB, 0.83072E+00_JPRB ,&
&  0.83853E+00_JPRB, 0.85338E+00_JPRB, 0.86377E+00_JPRB, 0.87495E+00_JPRB, 0.81722E+00_JPRB, 0.74804E+00_JPRB, 0.70786E+00_JPRB ,&
&  0.68983E+00_JPRB, 0.68026E+00_JPRB, 0.66938E+00_JPRB, 0.68023E+00_JPRB, 0.69530E+00_JPRB, 0.69166E+00_JPRB, 0.65731E+00_JPRB ,&
&  0.62899E+00_JPRB, 0.61255E+00_JPRB, 0.60561E+00_JPRB, 0.61925E+00_JPRB, 0.63535E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.69064E+00_JPRB, 0.70285E+00_JPRB, 0.70007E+00_JPRB, 0.69350E+00_JPRB, 0.69058E+00_JPRB, 0.67826E+00_JPRB, 0.65388E+00_JPRB ,&
&  0.64286E+00_JPRB, 0.61446E+00_JPRB, 0.59885E+00_JPRB, 0.58645E+00_JPRB, 0.57604E+00_JPRB, 0.56175E+00_JPRB, 0.54185E+00_JPRB ,&
&  0.53481E+00_JPRB, 0.53858E+00_JPRB, 0.54630E+00_JPRB, 0.54579E+00_JPRB, 0.54134E+00_JPRB, 0.53571E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53389E+00_JPRB, 0.53687E+00_JPRB, 0.54195E+00_JPRB, 0.54039E+00_JPRB, 0.53598E+00_JPRB, 0.53268E+00_JPRB, 0.53382E+00_JPRB ,&
&  0.52826E+00_JPRB, 0.51821E+00_JPRB, 0.51844E+00_JPRB, 0.51772E+00_JPRB, 0.52078E+00_JPRB, 0.52174E+00_JPRB, 0.49905E+00_JPRB ,&
&  0.47278E+00_JPRB, 0.48095E+00_JPRB, 0.51219E+00_JPRB, 0.53621E+00_JPRB, 0.55676E+00_JPRB, 0.56439E+00_JPRB, 0.56929E+00_JPRB ,&
&  0.57248E+00_JPRB, 0.57653E+00_JPRB, 0.57754E+00_JPRB, 0.57947E+00_JPRB, 0.57943E+00_JPRB, 0.57713E+00_JPRB, 0.57295E+00_JPRB ,&
&  0.56996E+00_JPRB, 0.53769E+00_JPRB, 0.53231E+00_JPRB, 0.55909E+00_JPRB, 0.71076E+00_JPRB, 0.75460E+00_JPRB, 0.80320E+00_JPRB ,&
&  0.90745E+00_JPRB, 0.98031E+00_JPRB, 0.10683E+01_JPRB, 0.11230E+01_JPRB, 0.11315E+01_JPRB, 0.11400E+01_JPRB, 0.12035E+01_JPRB ,&
&  0.12432E+01_JPRB, 0.12327E+01_JPRB, 0.12085E+01_JPRB, 0.11661E+01_JPRB, 0.10945E+01_JPRB, 0.10631E+01_JPRB, 0.10422E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10438E+01_JPRB, 0.99670E+00_JPRB, 0.93741E+00_JPRB, 0.89540E+00_JPRB, 0.89568E+00_JPRB, 0.90233E+00_JPRB ,&
&  0.90578E+00_JPRB, 0.92032E+00_JPRB, 0.91563E+00_JPRB, 0.90238E+00_JPRB, 0.86125E+00_JPRB, 0.82490E+00_JPRB, 0.81049E+00_JPRB ,&
&  0.77900E+00_JPRB, 0.73853E+00_JPRB, 0.68863E+00_JPRB, 0.67876E+00_JPRB, 0.68797E+00_JPRB, 0.68268E+00_JPRB, 0.65978E+00_JPRB ,&
&  0.65033E+00_JPRB, 0.64149E+00_JPRB, 0.63881E+00_JPRB, 0.65675E+00_JPRB, 0.67108E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.70430E+00_JPRB, 0.71311E+00_JPRB, 0.70909E+00_JPRB, 0.70094E+00_JPRB, 0.70849E+00_JPRB, 0.69879E+00_JPRB, 0.69605E+00_JPRB ,&
&  0.70830E+00_JPRB, 0.69643E+00_JPRB, 0.71235E+00_JPRB, 0.64179E+00_JPRB, 0.59487E+00_JPRB, 0.57650E+00_JPRB, 0.55631E+00_JPRB ,&
&  0.54922E+00_JPRB, 0.55409E+00_JPRB, 0.55615E+00_JPRB, 0.55598E+00_JPRB, 0.55519E+00_JPRB, 0.54738E+00_JPRB, 0.54436E+00_JPRB ,&
&  0.54622E+00_JPRB, 0.55146E+00_JPRB, 0.55734E+00_JPRB, 0.55520E+00_JPRB, 0.55119E+00_JPRB, 0.54842E+00_JPRB, 0.54503E+00_JPRB ,&
&  0.53904E+00_JPRB, 0.53947E+00_JPRB, 0.54142E+00_JPRB, 0.53952E+00_JPRB, 0.54113E+00_JPRB, 0.54567E+00_JPRB, 0.54414E+00_JPRB ,&
&  0.54869E+00_JPRB, 0.54665E+00_JPRB, 0.57961E+00_JPRB, 0.59957E+00_JPRB, 0.60957E+00_JPRB, 0.60974E+00_JPRB, 0.60992E+00_JPRB ,&
&  0.61471E+00_JPRB, 0.62256E+00_JPRB, 0.63169E+00_JPRB, 0.63894E+00_JPRB, 0.63810E+00_JPRB, 0.63608E+00_JPRB, 0.63197E+00_JPRB ,&
&  0.62933E+00_JPRB, 0.57284E+00_JPRB, 0.63133E+00_JPRB, 0.66610E+00_JPRB, 0.77895E+00_JPRB, 0.91122E+00_JPRB, 0.93445E+00_JPRB ,&
&  0.11451E+01_JPRB, 0.11340E+01_JPRB, 0.11871E+01_JPRB, 0.11980E+01_JPRB, 0.12072E+01_JPRB, 0.12436E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.13425E+01_JPRB, 0.13597E+01_JPRB, 0.12663E+01_JPRB, 0.12020E+01_JPRB, 0.11971E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.11896E+01_JPRB, 0.11677E+01_JPRB, 0.10505E+01_JPRB, 0.10199E+01_JPRB, 0.10074E+01_JPRB, 0.95264E+00_JPRB ,&
&  0.96997E+00_JPRB, 0.96395E+00_JPRB, 0.92275E+00_JPRB, 0.96132E+00_JPRB, 0.89207E+00_JPRB, 0.85253E+00_JPRB, 0.85642E+00_JPRB ,&
&  0.87950E+00_JPRB, 0.80570E+00_JPRB, 0.75062E+00_JPRB, 0.72798E+00_JPRB, 0.70676E+00_JPRB, 0.68857E+00_JPRB, 0.68648E+00_JPRB ,&
&  0.66756E+00_JPRB, 0.67033E+00_JPRB, 0.69474E+00_JPRB, 0.69532E+00_JPRB, 0.69338E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.71807E+00_JPRB, 0.71478E+00_JPRB, 0.70863E+00_JPRB, 0.70559E+00_JPRB, 0.72942E+00_JPRB, 0.73921E+00_JPRB, 0.74009E+00_JPRB ,&
&  0.75621E+00_JPRB, 0.75089E+00_JPRB, 0.74639E+00_JPRB, 0.73095E+00_JPRB, 0.71103E+00_JPRB, 0.65876E+00_JPRB, 0.58234E+00_JPRB ,&
&  0.55547E+00_JPRB, 0.55823E+00_JPRB, 0.56743E+00_JPRB, 0.56882E+00_JPRB, 0.56311E+00_JPRB, 0.55608E+00_JPRB, 0.55370E+00_JPRB ,&
&  0.55922E+00_JPRB, 0.56569E+00_JPRB, 0.57296E+00_JPRB, 0.57498E+00_JPRB, 0.57185E+00_JPRB, 0.57055E+00_JPRB, 0.56842E+00_JPRB ,&
&  0.55999E+00_JPRB, 0.56047E+00_JPRB, 0.56210E+00_JPRB, 0.55665E+00_JPRB, 0.57484E+00_JPRB, 0.58707E+00_JPRB, 0.58810E+00_JPRB ,&
&  0.60755E+00_JPRB, 0.62991E+00_JPRB, 0.64374E+00_JPRB, 0.65702E+00_JPRB, 0.65669E+00_JPRB, 0.65542E+00_JPRB, 0.66251E+00_JPRB ,&
&  0.67735E+00_JPRB, 0.69147E+00_JPRB, 0.70495E+00_JPRB, 0.71484E+00_JPRB, 0.71730E+00_JPRB, 0.71084E+00_JPRB, 0.73483E+00_JPRB ,&
&  0.74088E+00_JPRB, 0.76972E+00_JPRB, 0.83799E+00_JPRB, 0.84636E+00_JPRB, 0.94654E+00_JPRB, 0.10368E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.11573E+01_JPRB, 0.11552E+01_JPRB, 0.12916E+01_JPRB, 0.13926E+01_JPRB, 0.14833E+01_JPRB, 0.15253E+01_JPRB, 0.15785E+01_JPRB ,&
&  0.17020E+01_JPRB, 0.17744E+01_JPRB, 0.15822E+01_JPRB, 0.14834E+01_JPRB, 0.14015E+01_JPRB, 0.14011E+01_JPRB, 0.13524E+01_JPRB ,&
&  0.13597E+01_JPRB, 0.13892E+01_JPRB, 0.13692E+01_JPRB, 0.13327E+01_JPRB, 0.12557E+01_JPRB, 0.11701E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10397E+01_JPRB, 0.94649E+00_JPRB, 0.89977E+00_JPRB, 0.91473E+00_JPRB, 0.91570E+00_JPRB, 0.84287E+00_JPRB, 0.87742E+00_JPRB ,&
&  0.94597E+00_JPRB, 0.89630E+00_JPRB, 0.82828E+00_JPRB, 0.77480E+00_JPRB, 0.75291E+00_JPRB, 0.73107E+00_JPRB, 0.69118E+00_JPRB ,&
&  0.66275E+00_JPRB, 0.69574E+00_JPRB, 0.71860E+00_JPRB, 0.71886E+00_JPRB, 0.71674E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.73587E+00_JPRB, 0.72320E+00_JPRB, 0.70815E+00_JPRB, 0.71321E+00_JPRB, 0.74196E+00_JPRB, 0.75417E+00_JPRB, 0.77279E+00_JPRB ,&
&  0.78293E+00_JPRB, 0.78404E+00_JPRB, 0.77715E+00_JPRB, 0.75182E+00_JPRB, 0.73709E+00_JPRB, 0.72840E+00_JPRB, 0.68366E+00_JPRB ,&
&  0.62288E+00_JPRB, 0.60407E+00_JPRB, 0.58640E+00_JPRB, 0.58229E+00_JPRB, 0.58077E+00_JPRB, 0.57175E+00_JPRB, 0.56757E+00_JPRB ,&
&  0.57491E+00_JPRB, 0.57843E+00_JPRB, 0.58243E+00_JPRB, 0.58583E+00_JPRB, 0.58752E+00_JPRB, 0.59133E+00_JPRB, 0.59122E+00_JPRB ,&
&  0.58160E+00_JPRB, 0.58412E+00_JPRB, 0.59133E+00_JPRB, 0.58802E+00_JPRB, 0.61582E+00_JPRB, 0.64451E+00_JPRB, 0.66212E+00_JPRB ,&
&  0.68524E+00_JPRB, 0.69978E+00_JPRB, 0.70456E+00_JPRB, 0.70747E+00_JPRB, 0.70371E+00_JPRB, 0.71251E+00_JPRB, 0.73251E+00_JPRB ,&
&  0.75884E+00_JPRB, 0.77643E+00_JPRB, 0.78973E+00_JPRB, 0.80454E+00_JPRB, 0.79972E+00_JPRB, 0.82314E+00_JPRB, 0.89067E+00_JPRB ,&
&  0.96312E+00_JPRB, 0.10374E+01_JPRB, 0.10756E+01_JPRB, 0.11312E+01_JPRB, 0.12329E+01_JPRB, 0.12779E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.13126E+01_JPRB, 0.12905E+01_JPRB, 0.16656E+01_JPRB, 0.17260E+01_JPRB, 0.17224E+01_JPRB, 0.17977E+01_JPRB, 0.17766E+01_JPRB ,&
&  0.17859E+01_JPRB, 0.17010E+01_JPRB, 0.16325E+01_JPRB, 0.15903E+01_JPRB, 0.15628E+01_JPRB, 0.16326E+01_JPRB, 0.15727E+01_JPRB ,&
&  0.15917E+01_JPRB, 0.15854E+01_JPRB, 0.13965E+01_JPRB, 0.12411E+01_JPRB, 0.10799E+01_JPRB, 0.11239E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10233E+01_JPRB, 0.89070E+00_JPRB, 0.93005E+00_JPRB, 0.96366E+00_JPRB, 0.98313E+00_JPRB, 0.10260E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.10855E+01_JPRB, 0.96950E+00_JPRB, 0.87928E+00_JPRB, 0.83341E+00_JPRB, 0.79466E+00_JPRB, 0.72408E+00_JPRB ,&
&  0.70521E+00_JPRB, 0.73709E+00_JPRB, 0.75931E+00_JPRB, 0.76049E+00_JPRB, 0.74861E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.76762E+00_JPRB, 0.75046E+00_JPRB, 0.72671E+00_JPRB, 0.73676E+00_JPRB, 0.75988E+00_JPRB, 0.76761E+00_JPRB, 0.78879E+00_JPRB ,&
&  0.81039E+00_JPRB, 0.81035E+00_JPRB, 0.78333E+00_JPRB, 0.76184E+00_JPRB, 0.75006E+00_JPRB, 0.74680E+00_JPRB, 0.74947E+00_JPRB ,&
&  0.68416E+00_JPRB, 0.64387E+00_JPRB, 0.60123E+00_JPRB, 0.59472E+00_JPRB, 0.62418E+00_JPRB, 0.60676E+00_JPRB, 0.60296E+00_JPRB ,&
&  0.59768E+00_JPRB, 0.60068E+00_JPRB, 0.59595E+00_JPRB, 0.59454E+00_JPRB, 0.59657E+00_JPRB, 0.60345E+00_JPRB, 0.61210E+00_JPRB ,&
&  0.61002E+00_JPRB, 0.62623E+00_JPRB, 0.64730E+00_JPRB, 0.64971E+00_JPRB, 0.68153E+00_JPRB, 0.70989E+00_JPRB, 0.74252E+00_JPRB ,&
&  0.76387E+00_JPRB, 0.75971E+00_JPRB, 0.75107E+00_JPRB, 0.75476E+00_JPRB, 0.76442E+00_JPRB, 0.79250E+00_JPRB, 0.82973E+00_JPRB ,&
&  0.86994E+00_JPRB, 0.90077E+00_JPRB, 0.92107E+00_JPRB, 0.93331E+00_JPRB, 0.92233E+00_JPRB, 0.96899E+00_JPRB, 0.10778E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.13173E+01_JPRB, 0.13530E+01_JPRB, 0.16315E+01_JPRB, 0.17885E+01_JPRB, 0.17748E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.15439E+01_JPRB, 0.14683E+01_JPRB, 0.16126E+01_JPRB, 0.17407E+01_JPRB, 0.17253E+01_JPRB, 0.18724E+01_JPRB, 0.18225E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.15714E+01_JPRB, 0.15153E+01_JPRB, 0.15677E+01_JPRB, 0.15657E+01_JPRB, 0.16663E+01_JPRB, 0.16752E+01_JPRB ,&
&  0.16836E+01_JPRB, 0.14816E+01_JPRB, 0.11406E+01_JPRB, 0.10597E+01_JPRB, 0.86246E+00_JPRB, 0.87388E+00_JPRB, 0.10477E+01_JPRB ,&
&  0.10456E+01_JPRB, 0.10450E+01_JPRB, 0.10467E+01_JPRB, 0.10956E+01_JPRB, 0.11244E+01_JPRB, 0.13254E+01_JPRB, 0.14071E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12582E+01_JPRB, 0.12066E+01_JPRB, 0.11276E+01_JPRB, 0.10401E+01_JPRB, 0.93251E+00_JPRB, 0.84746E+00_JPRB ,&
&  0.82034E+00_JPRB, 0.81049E+00_JPRB, 0.81659E+00_JPRB, 0.80368E+00_JPRB, 0.78018E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.81965E+00_JPRB, 0.80347E+00_JPRB, 0.76897E+00_JPRB, 0.77255E+00_JPRB, 0.78083E+00_JPRB, 0.79049E+00_JPRB, 0.81558E+00_JPRB ,&
&  0.83786E+00_JPRB, 0.82018E+00_JPRB, 0.78572E+00_JPRB, 0.77272E+00_JPRB, 0.76754E+00_JPRB, 0.76208E+00_JPRB, 0.76725E+00_JPRB ,&
&  0.78014E+00_JPRB, 0.73978E+00_JPRB, 0.67211E+00_JPRB, 0.61857E+00_JPRB, 0.63288E+00_JPRB, 0.62938E+00_JPRB, 0.64218E+00_JPRB ,&
&  0.63491E+00_JPRB, 0.65128E+00_JPRB, 0.63770E+00_JPRB, 0.63163E+00_JPRB, 0.62919E+00_JPRB, 0.63920E+00_JPRB, 0.66168E+00_JPRB ,&
&  0.69105E+00_JPRB, 0.74166E+00_JPRB, 0.79104E+00_JPRB, 0.79913E+00_JPRB, 0.82890E+00_JPRB, 0.80121E+00_JPRB, 0.82593E+00_JPRB ,&
&  0.83597E+00_JPRB, 0.82239E+00_JPRB, 0.81085E+00_JPRB, 0.82308E+00_JPRB, 0.85556E+00_JPRB, 0.90192E+00_JPRB, 0.96486E+00_JPRB ,&
&  0.10303E+01_JPRB, 0.10604E+01_JPRB, 0.10440E+01_JPRB, 0.10324E+01_JPRB, 0.10640E+01_JPRB, 0.11393E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.13637E+01_JPRB, 0.13425E+01_JPRB, 0.17134E+01_JPRB, 0.20238E+01_JPRB, 0.20773E+01_JPRB, 0.19810E+01_JPRB ,&
&  0.18781E+01_JPRB, 0.17788E+01_JPRB, 0.18660E+01_JPRB, 0.19114E+01_JPRB, 0.19237E+01_JPRB, 0.19274E+01_JPRB, 0.18253E+01_JPRB ,&
&  0.16294E+01_JPRB, 0.15273E+01_JPRB, 0.15349E+01_JPRB, 0.15345E+01_JPRB, 0.15620E+01_JPRB, 0.15229E+01_JPRB, 0.15840E+01_JPRB ,&
&  0.16463E+01_JPRB, 0.10156E+01_JPRB, 0.85157E+00_JPRB, 0.96819E+00_JPRB, 0.84437E+00_JPRB, 0.86210E+00_JPRB, 0.96071E+00_JPRB ,&
&  0.95078E+00_JPRB, 0.95855E+00_JPRB, 0.10813E+01_JPRB, 0.12190E+01_JPRB, 0.13857E+01_JPRB, 0.16753E+01_JPRB, 0.16883E+01_JPRB ,&
&  0.15810E+01_JPRB, 0.14501E+01_JPRB, 0.14804E+01_JPRB, 0.14479E+01_JPRB, 0.13289E+01_JPRB, 0.11507E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.93416E+00_JPRB, 0.90793E+00_JPRB, 0.89618E+00_JPRB, 0.87007E+00_JPRB, 0.83297E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.89631E+00_JPRB, 0.87228E+00_JPRB, 0.81258E+00_JPRB, 0.80000E+00_JPRB, 0.81365E+00_JPRB, 0.84076E+00_JPRB, 0.86677E+00_JPRB ,&
&  0.86706E+00_JPRB, 0.83189E+00_JPRB, 0.80568E+00_JPRB, 0.79663E+00_JPRB, 0.78645E+00_JPRB, 0.77329E+00_JPRB, 0.76856E+00_JPRB ,&
&  0.76950E+00_JPRB, 0.76807E+00_JPRB, 0.70242E+00_JPRB, 0.66973E+00_JPRB, 0.63190E+00_JPRB, 0.64163E+00_JPRB, 0.65819E+00_JPRB ,&
&  0.68026E+00_JPRB, 0.67809E+00_JPRB, 0.66964E+00_JPRB, 0.67429E+00_JPRB, 0.67969E+00_JPRB, 0.69878E+00_JPRB, 0.77134E+00_JPRB ,&
&  0.85654E+00_JPRB, 0.95013E+00_JPRB, 0.96535E+00_JPRB, 0.99780E+00_JPRB, 0.95925E+00_JPRB, 0.89860E+00_JPRB, 0.93051E+00_JPRB ,&
&  0.92230E+00_JPRB, 0.89879E+00_JPRB, 0.88465E+00_JPRB, 0.90606E+00_JPRB, 0.95027E+00_JPRB, 0.98878E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.11143E+01_JPRB, 0.11013E+01_JPRB, 0.10741E+01_JPRB, 0.11230E+01_JPRB, 0.11636E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.11873E+01_JPRB, 0.10983E+01_JPRB, 0.94539E+00_JPRB, 0.13571E+01_JPRB, 0.20695E+01_JPRB, 0.21825E+01_JPRB, 0.19080E+01_JPRB ,&
&  0.19701E+01_JPRB, 0.19571E+01_JPRB, 0.20771E+01_JPRB, 0.21924E+01_JPRB, 0.21575E+01_JPRB, 0.20201E+01_JPRB, 0.18608E+01_JPRB ,&
&  0.16826E+01_JPRB, 0.16118E+01_JPRB, 0.16404E+01_JPRB, 0.18500E+01_JPRB, 0.18827E+01_JPRB, 0.18760E+01_JPRB, 0.18280E+01_JPRB ,&
&  0.15753E+01_JPRB, 0.13694E+01_JPRB, 0.10092E+01_JPRB, 0.10291E+01_JPRB, 0.83947E+00_JPRB, 0.84486E+00_JPRB, 0.94341E+00_JPRB ,&
&  0.92530E+00_JPRB, 0.98871E+00_JPRB, 0.11261E+01_JPRB, 0.12886E+01_JPRB, 0.17447E+01_JPRB, 0.20674E+01_JPRB, 0.21669E+01_JPRB ,&
&  0.20709E+01_JPRB, 0.18188E+01_JPRB, 0.18626E+01_JPRB, 0.17300E+01_JPRB, 0.16165E+01_JPRB, 0.14303E+01_JPRB, 0.12326E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.10535E+01_JPRB, 0.10206E+01_JPRB, 0.97401E+00_JPRB, 0.91875E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.94524E+00_JPRB, 0.89522E+00_JPRB, 0.81239E+00_JPRB, 0.80156E+00_JPRB, 0.82561E+00_JPRB, 0.88146E+00_JPRB, 0.91023E+00_JPRB ,&
&  0.88792E+00_JPRB, 0.84858E+00_JPRB, 0.82378E+00_JPRB, 0.81186E+00_JPRB, 0.79276E+00_JPRB, 0.76966E+00_JPRB, 0.75973E+00_JPRB ,&
&  0.75451E+00_JPRB, 0.73888E+00_JPRB, 0.68469E+00_JPRB, 0.65111E+00_JPRB, 0.62062E+00_JPRB, 0.62088E+00_JPRB, 0.64208E+00_JPRB ,&
&  0.66380E+00_JPRB, 0.68271E+00_JPRB, 0.71094E+00_JPRB, 0.73104E+00_JPRB, 0.74708E+00_JPRB, 0.80907E+00_JPRB, 0.93723E+00_JPRB ,&
&  0.11625E+01_JPRB, 0.11187E+01_JPRB, 0.11426E+01_JPRB, 0.10858E+01_JPRB, 0.10397E+01_JPRB, 0.10046E+01_JPRB, 0.96899E+00_JPRB ,&
&  0.91930E+00_JPRB, 0.88255E+00_JPRB, 0.87945E+00_JPRB, 0.90534E+00_JPRB, 0.98233E+00_JPRB, 0.10599E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.11896E+01_JPRB, 0.11304E+01_JPRB, 0.11197E+01_JPRB, 0.11856E+01_JPRB, 0.11915E+01_JPRB, 0.11596E+01_JPRB ,&
&  0.10296E+01_JPRB, 0.94220E+00_JPRB, 0.10950E+01_JPRB, 0.13805E+01_JPRB, 0.18548E+01_JPRB, 0.22017E+01_JPRB, 0.21192E+01_JPRB ,&
&  0.21405E+01_JPRB, 0.21073E+01_JPRB, 0.21367E+01_JPRB, 0.17953E+01_JPRB, 0.19883E+01_JPRB, 0.19025E+01_JPRB, 0.17893E+01_JPRB ,&
&  0.16942E+01_JPRB, 0.16992E+01_JPRB, 0.18376E+01_JPRB, 0.20936E+01_JPRB, 0.22598E+01_JPRB, 0.23586E+01_JPRB, 0.18324E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.15934E+01_JPRB, 0.14557E+01_JPRB, 0.12323E+01_JPRB, 0.10599E+01_JPRB, 0.10796E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.11029E+01_JPRB, 0.12562E+01_JPRB, 0.16181E+01_JPRB, 0.22418E+01_JPRB, 0.26923E+01_JPRB, 0.27039E+01_JPRB ,&
&  0.24166E+01_JPRB, 0.24677E+01_JPRB, 0.22757E+01_JPRB, 0.20016E+01_JPRB, 0.18105E+01_JPRB, 0.16465E+01_JPRB, 0.14780E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.12326E+01_JPRB, 0.11665E+01_JPRB, 0.10850E+01_JPRB, 0.99379E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.98588E+00_JPRB, 0.93188E+00_JPRB, 0.85015E+00_JPRB, 0.83992E+00_JPRB, 0.88433E+00_JPRB, 0.94190E+00_JPRB, 0.94829E+00_JPRB ,&
&  0.88809E+00_JPRB, 0.84948E+00_JPRB, 0.82419E+00_JPRB, 0.82113E+00_JPRB, 0.80773E+00_JPRB, 0.78012E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.76074E+00_JPRB, 0.72458E+00_JPRB, 0.66921E+00_JPRB, 0.66564E+00_JPRB, 0.65490E+00_JPRB, 0.61572E+00_JPRB, 0.62484E+00_JPRB ,&
&  0.65083E+00_JPRB, 0.68655E+00_JPRB, 0.73038E+00_JPRB, 0.79488E+00_JPRB, 0.88399E+00_JPRB, 0.10155E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.12359E+01_JPRB, 0.12506E+01_JPRB, 0.11799E+01_JPRB, 0.10621E+01_JPRB, 0.98639E+00_JPRB, 0.94636E+00_JPRB ,&
&  0.90233E+00_JPRB, 0.87417E+00_JPRB, 0.90368E+00_JPRB, 0.98710E+00_JPRB, 0.11126E+01_JPRB, 0.12020E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.12428E+01_JPRB, 0.12340E+01_JPRB, 0.12205E+01_JPRB, 0.13095E+01_JPRB, 0.10542E+01_JPRB, 0.10145E+01_JPRB, 0.98195E+00_JPRB ,&
&  0.11254E+01_JPRB, 0.11009E+01_JPRB, 0.11297E+01_JPRB, 0.14350E+01_JPRB, 0.14424E+01_JPRB, 0.17282E+01_JPRB, 0.20798E+01_JPRB ,&
&  0.22600E+01_JPRB, 0.22888E+01_JPRB, 0.22838E+01_JPRB, 0.14844E+01_JPRB, 0.13707E+01_JPRB, 0.17708E+01_JPRB, 0.18410E+01_JPRB ,&
&  0.18243E+01_JPRB, 0.19908E+01_JPRB, 0.22046E+01_JPRB, 0.25702E+01_JPRB, 0.19786E+01_JPRB, 0.17258E+01_JPRB, 0.13780E+01_JPRB ,&
&  0.13734E+01_JPRB, 0.12961E+01_JPRB, 0.15248E+01_JPRB, 0.14056E+01_JPRB, 0.14390E+01_JPRB, 0.13847E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.14237E+01_JPRB, 0.16741E+01_JPRB, 0.22773E+01_JPRB, 0.30190E+01_JPRB, 0.29937E+01_JPRB, 0.33156E+01_JPRB ,&
&  0.32822E+01_JPRB, 0.29466E+01_JPRB, 0.24371E+01_JPRB, 0.22063E+01_JPRB, 0.20234E+01_JPRB, 0.18737E+01_JPRB, 0.17092E+01_JPRB ,&
&  0.15352E+01_JPRB, 0.14005E+01_JPRB, 0.12813E+01_JPRB, 0.11609E+01_JPRB, 0.10515E+01_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.10676E+01_JPRB, 0.10105E+01_JPRB, 0.91039E+00_JPRB, 0.90876E+00_JPRB, 0.96217E+00_JPRB, 0.98815E+00_JPRB, 0.94254E+00_JPRB ,&
&  0.89276E+00_JPRB, 0.84885E+00_JPRB, 0.83423E+00_JPRB, 0.82609E+00_JPRB, 0.81028E+00_JPRB, 0.80126E+00_JPRB, 0.78044E+00_JPRB ,&
&  0.79527E+00_JPRB, 0.78467E+00_JPRB, 0.71168E+00_JPRB, 0.67364E+00_JPRB, 0.68128E+00_JPRB, 0.63881E+00_JPRB, 0.61313E+00_JPRB ,&
&  0.66285E+00_JPRB, 0.72630E+00_JPRB, 0.81402E+00_JPRB, 0.92692E+00_JPRB, 0.10622E+01_JPRB, 0.12975E+01_JPRB, 0.14226E+01_JPRB ,&
&  0.13820E+01_JPRB, 0.13347E+01_JPRB, 0.12038E+01_JPRB, 0.10770E+01_JPRB, 0.97050E+00_JPRB, 0.91219E+00_JPRB, 0.87862E+00_JPRB ,&
&  0.86311E+00_JPRB, 0.87444E+00_JPRB, 0.95045E+00_JPRB, 0.10697E+01_JPRB, 0.11981E+01_JPRB, 0.12556E+01_JPRB, 0.12615E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.13782E+01_JPRB, 0.13905E+01_JPRB, 0.13278E+01_JPRB, 0.12097E+01_JPRB, 0.10412E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.11223E+01_JPRB, 0.10625E+01_JPRB, 0.11305E+01_JPRB, 0.14032E+01_JPRB, 0.16358E+01_JPRB, 0.17352E+01_JPRB, 0.20976E+01_JPRB ,&
&  0.17604E+01_JPRB, 0.14629E+01_JPRB, 0.12525E+01_JPRB, 0.10386E+01_JPRB, 0.14276E+01_JPRB, 0.20034E+01_JPRB, 0.21970E+01_JPRB ,&
&  0.22743E+01_JPRB, 0.24590E+01_JPRB, 0.27187E+01_JPRB, 0.24655E+01_JPRB, 0.18248E+01_JPRB, 0.15282E+01_JPRB, 0.18838E+01_JPRB ,&
&  0.20074E+01_JPRB, 0.21461E+01_JPRB, 0.21071E+01_JPRB, 0.18307E+01_JPRB, 0.15549E+01_JPRB, 0.17629E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.18450E+01_JPRB, 0.19648E+01_JPRB, 0.27091E+01_JPRB, 0.35235E+01_JPRB, 0.38188E+01_JPRB, 0.40995E+01_JPRB, 0.42953E+01_JPRB ,&
&  0.37819E+01_JPRB, 0.33597E+01_JPRB, 0.28472E+01_JPRB, 0.24439E+01_JPRB, 0.21788E+01_JPRB, 0.20055E+01_JPRB, 0.18687E+01_JPRB ,&
&  0.17018E+01_JPRB, 0.15792E+01_JPRB, 0.14388E+01_JPRB, 0.12650E+01_JPRB, 0.11358E+01_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.11751E+01_JPRB, 0.10909E+01_JPRB, 0.97626E+00_JPRB, 0.97171E+00_JPRB, 0.10249E+01_JPRB, 0.10036E+01_JPRB, 0.93242E+00_JPRB ,&
&  0.90499E+00_JPRB, 0.86395E+00_JPRB, 0.84961E+00_JPRB, 0.83290E+00_JPRB, 0.81323E+00_JPRB, 0.80000E+00_JPRB, 0.80234E+00_JPRB ,&
&  0.84255E+00_JPRB, 0.90554E+00_JPRB, 0.81832E+00_JPRB, 0.75714E+00_JPRB, 0.73054E+00_JPRB, 0.70476E+00_JPRB, 0.64041E+00_JPRB ,&
&  0.71585E+00_JPRB, 0.83415E+00_JPRB, 0.95270E+00_JPRB, 0.11038E+01_JPRB, 0.13145E+01_JPRB, 0.13781E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.13630E+01_JPRB, 0.12558E+01_JPRB, 0.11220E+01_JPRB, 0.99950E+00_JPRB, 0.92704E+00_JPRB, 0.89534E+00_JPRB, 0.87671E+00_JPRB ,&
&  0.88917E+00_JPRB, 0.95599E+00_JPRB, 0.10471E+01_JPRB, 0.11575E+01_JPRB, 0.12884E+01_JPRB, 0.13543E+01_JPRB, 0.13874E+01_JPRB ,&
&  0.14565E+01_JPRB, 0.15294E+01_JPRB, 0.15392E+01_JPRB, 0.14910E+01_JPRB, 0.14499E+01_JPRB, 0.12607E+01_JPRB, 0.14471E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.14522E+01_JPRB, 0.13910E+01_JPRB, 0.16821E+01_JPRB, 0.16740E+01_JPRB, 0.16728E+01_JPRB, 0.18870E+01_JPRB ,&
&  0.14869E+01_JPRB, 0.13796E+01_JPRB, 0.15538E+01_JPRB, 0.17411E+01_JPRB, 0.18301E+01_JPRB, 0.20439E+01_JPRB, 0.29578E+01_JPRB ,&
&  0.24922E+01_JPRB, 0.28376E+01_JPRB, 0.31006E+01_JPRB, 0.30212E+01_JPRB, 0.14200E+01_JPRB, 0.11540E+01_JPRB, 0.18649E+01_JPRB ,&
&  0.21052E+01_JPRB, 0.14629E+01_JPRB, 0.11279E+01_JPRB, 0.14082E+01_JPRB, 0.11185E+01_JPRB, 0.13462E+01_JPRB, 0.19367E+01_JPRB ,&
&  0.23117E+01_JPRB, 0.28142E+01_JPRB, 0.42521E+01_JPRB, 0.45549E+01_JPRB, 0.43700E+01_JPRB, 0.53764E+01_JPRB, 0.44761E+01_JPRB ,&
&  0.39589E+01_JPRB, 0.30781E+01_JPRB, 0.28795E+01_JPRB, 0.25485E+01_JPRB, 0.23029E+01_JPRB, 0.21558E+01_JPRB, 0.20213E+01_JPRB ,&
&  0.18615E+01_JPRB, 0.17157E+01_JPRB, 0.15638E+01_JPRB, 0.14010E+01_JPRB, 0.12731E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.12499E+01_JPRB, 0.11525E+01_JPRB, 0.10520E+01_JPRB, 0.10329E+01_JPRB, 0.10609E+01_JPRB, 0.98750E+00_JPRB, 0.93324E+00_JPRB ,&
&  0.89077E+00_JPRB, 0.86963E+00_JPRB, 0.84945E+00_JPRB, 0.82421E+00_JPRB, 0.80272E+00_JPRB, 0.80660E+00_JPRB, 0.82682E+00_JPRB ,&
&  0.86957E+00_JPRB, 0.96993E+00_JPRB, 0.10361E+01_JPRB, 0.98224E+00_JPRB, 0.94327E+00_JPRB, 0.88319E+00_JPRB, 0.82513E+00_JPRB ,&
&  0.89547E+00_JPRB, 0.10666E+01_JPRB, 0.11653E+01_JPRB, 0.12224E+01_JPRB, 0.12642E+01_JPRB, 0.12374E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.12539E+01_JPRB, 0.11449E+01_JPRB, 0.10507E+01_JPRB, 0.97031E+00_JPRB, 0.92945E+00_JPRB, 0.91137E+00_JPRB, 0.93307E+00_JPRB ,&
&  0.98138E+00_JPRB, 0.10385E+01_JPRB, 0.11059E+01_JPRB, 0.12518E+01_JPRB, 0.14070E+01_JPRB, 0.14741E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.15907E+01_JPRB, 0.16189E+01_JPRB, 0.16303E+01_JPRB, 0.16565E+01_JPRB, 0.17778E+01_JPRB, 0.16130E+01_JPRB, 0.13855E+01_JPRB ,&
&  0.12778E+01_JPRB, 0.14398E+01_JPRB, 0.19513E+01_JPRB, 0.18655E+01_JPRB, 0.16646E+01_JPRB, 0.17198E+01_JPRB, 0.17144E+01_JPRB ,&
&  0.16754E+01_JPRB, 0.19251E+01_JPRB, 0.21096E+01_JPRB, 0.23653E+01_JPRB, 0.25267E+01_JPRB, 0.20483E+01_JPRB, 0.28311E+01_JPRB ,&
&  0.30503E+01_JPRB, 0.29890E+01_JPRB, 0.22763E+01_JPRB, 0.16093E+01_JPRB, 0.16814E+01_JPRB, 0.13513E+01_JPRB, 0.97967E+00_JPRB ,&
&  0.93327E+00_JPRB, 0.89973E+00_JPRB, 0.83613E+00_JPRB, 0.88458E+00_JPRB, 0.90707E+00_JPRB, 0.12803E+01_JPRB, 0.19854E+01_JPRB ,&
&  0.31250E+01_JPRB, 0.42453E+01_JPRB, 0.50056E+01_JPRB, 0.49967E+01_JPRB, 0.47029E+01_JPRB, 0.54696E+01_JPRB, 0.46449E+01_JPRB ,&
&  0.33238E+01_JPRB, 0.28105E+01_JPRB, 0.26523E+01_JPRB, 0.25448E+01_JPRB, 0.24479E+01_JPRB, 0.23461E+01_JPRB, 0.21799E+01_JPRB ,&
&  0.19812E+01_JPRB, 0.18322E+01_JPRB, 0.16919E+01_JPRB, 0.15123E+01_JPRB, 0.13711E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.13091E+01_JPRB, 0.12146E+01_JPRB, 0.11581E+01_JPRB, 0.11134E+01_JPRB, 0.10829E+01_JPRB, 0.97623E+00_JPRB, 0.93215E+00_JPRB ,&
&  0.87600E+00_JPRB, 0.85983E+00_JPRB, 0.82631E+00_JPRB, 0.80790E+00_JPRB, 0.79708E+00_JPRB, 0.82022E+00_JPRB, 0.86557E+00_JPRB ,&
&  0.92899E+00_JPRB, 0.10891E+01_JPRB, 0.13006E+01_JPRB, 0.13964E+01_JPRB, 0.14967E+01_JPRB, 0.14478E+01_JPRB, 0.13596E+01_JPRB ,&
&  0.13008E+01_JPRB, 0.14445E+01_JPRB, 0.13849E+01_JPRB, 0.13202E+01_JPRB, 0.12876E+01_JPRB, 0.12376E+01_JPRB, 0.11955E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.10718E+01_JPRB, 0.99025E+00_JPRB, 0.94800E+00_JPRB, 0.94012E+00_JPRB, 0.95488E+00_JPRB, 0.98702E+00_JPRB ,&
&  0.10539E+01_JPRB, 0.10843E+01_JPRB, 0.11828E+01_JPRB, 0.13571E+01_JPRB, 0.14995E+01_JPRB, 0.15546E+01_JPRB, 0.16351E+01_JPRB ,&
&  0.17075E+01_JPRB, 0.17069E+01_JPRB, 0.16853E+01_JPRB, 0.16424E+01_JPRB, 0.14384E+01_JPRB, 0.12214E+01_JPRB, 0.14427E+01_JPRB ,&
&  0.16353E+01_JPRB, 0.19843E+01_JPRB, 0.21689E+01_JPRB, 0.22599E+01_JPRB, 0.18206E+01_JPRB, 0.17202E+01_JPRB, 0.17425E+01_JPRB ,&
&  0.19316E+01_JPRB, 0.28718E+01_JPRB, 0.25907E+01_JPRB, 0.24126E+01_JPRB, 0.29928E+01_JPRB, 0.25144E+01_JPRB, 0.26386E+01_JPRB ,&
&  0.31540E+01_JPRB, 0.29835E+01_JPRB, 0.23607E+01_JPRB, 0.18638E+01_JPRB, 0.24714E+01_JPRB, 0.26835E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.96676E+00_JPRB, 0.91712E+00_JPRB, 0.88155E+00_JPRB, 0.91192E+00_JPRB, 0.13636E+01_JPRB, 0.26497E+01_JPRB ,&
&  0.37610E+01_JPRB, 0.52989E+01_JPRB, 0.55425E+01_JPRB, 0.53509E+01_JPRB, 0.48559E+01_JPRB, 0.48822E+01_JPRB, 0.39116E+01_JPRB ,&
&  0.33278E+01_JPRB, 0.29471E+01_JPRB, 0.27288E+01_JPRB, 0.26190E+01_JPRB, 0.25137E+01_JPRB, 0.24050E+01_JPRB, 0.22549E+01_JPRB ,&
&  0.20708E+01_JPRB, 0.19151E+01_JPRB, 0.17680E+01_JPRB, 0.15794E+01_JPRB, 0.14251E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.13454E+01_JPRB, 0.12778E+01_JPRB, 0.12470E+01_JPRB, 0.12046E+01_JPRB, 0.11138E+01_JPRB, 0.10037E+01_JPRB, 0.91410E+00_JPRB ,&
&  0.86829E+00_JPRB, 0.83981E+00_JPRB, 0.81696E+00_JPRB, 0.80829E+00_JPRB, 0.81356E+00_JPRB, 0.84888E+00_JPRB, 0.90498E+00_JPRB ,&
&  0.10251E+01_JPRB, 0.12280E+01_JPRB, 0.14487E+01_JPRB, 0.18538E+01_JPRB, 0.20131E+01_JPRB, 0.18785E+01_JPRB, 0.20528E+01_JPRB ,&
&  0.20011E+01_JPRB, 0.19180E+01_JPRB, 0.16924E+01_JPRB, 0.14841E+01_JPRB, 0.13521E+01_JPRB, 0.12741E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.11075E+01_JPRB, 0.10460E+01_JPRB, 0.98654E+00_JPRB, 0.99301E+00_JPRB, 0.10070E+01_JPRB, 0.10206E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.11069E+01_JPRB, 0.12489E+01_JPRB, 0.14179E+01_JPRB, 0.15395E+01_JPRB, 0.16327E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.17789E+01_JPRB, 0.17692E+01_JPRB, 0.17177E+01_JPRB, 0.16433E+01_JPRB, 0.13648E+01_JPRB, 0.15332E+01_JPRB, 0.15910E+01_JPRB ,&
&  0.17417E+01_JPRB, 0.19373E+01_JPRB, 0.20004E+01_JPRB, 0.21254E+01_JPRB, 0.19206E+01_JPRB, 0.17626E+01_JPRB, 0.17444E+01_JPRB ,&
&  0.22946E+01_JPRB, 0.24252E+01_JPRB, 0.21947E+01_JPRB, 0.24106E+01_JPRB, 0.29825E+01_JPRB, 0.37592E+01_JPRB, 0.24489E+01_JPRB ,&
&  0.21826E+01_JPRB, 0.28267E+01_JPRB, 0.24317E+01_JPRB, 0.21946E+01_JPRB, 0.27512E+01_JPRB, 0.37934E+01_JPRB, 0.27544E+01_JPRB ,&
&  0.12919E+01_JPRB, 0.10406E+01_JPRB, 0.10939E+01_JPRB, 0.11888E+01_JPRB, 0.11988E+01_JPRB, 0.18479E+01_JPRB, 0.50226E+01_JPRB ,&
&  0.53293E+01_JPRB, 0.63218E+01_JPRB, 0.58483E+01_JPRB, 0.52271E+01_JPRB, 0.48858E+01_JPRB, 0.45802E+01_JPRB, 0.38876E+01_JPRB ,&
&  0.33533E+01_JPRB, 0.30308E+01_JPRB, 0.27831E+01_JPRB, 0.26031E+01_JPRB, 0.24240E+01_JPRB, 0.22453E+01_JPRB, 0.20812E+01_JPRB ,&
&  0.19641E+01_JPRB, 0.18437E+01_JPRB, 0.17321E+01_JPRB, 0.15792E+01_JPRB, 0.14427E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.13380E+01_JPRB, 0.12864E+01_JPRB, 0.12646E+01_JPRB, 0.12175E+01_JPRB, 0.11384E+01_JPRB, 0.10641E+01_JPRB, 0.95660E+00_JPRB ,&
&  0.88098E+00_JPRB, 0.83631E+00_JPRB, 0.81850E+00_JPRB, 0.82223E+00_JPRB, 0.84779E+00_JPRB, 0.89360E+00_JPRB, 0.94769E+00_JPRB ,&
&  0.11276E+01_JPRB, 0.13054E+01_JPRB, 0.15879E+01_JPRB, 0.19882E+01_JPRB, 0.22301E+01_JPRB, 0.20694E+01_JPRB, 0.22743E+01_JPRB ,&
&  0.27133E+01_JPRB, 0.23269E+01_JPRB, 0.19357E+01_JPRB, 0.17311E+01_JPRB, 0.15640E+01_JPRB, 0.14543E+01_JPRB, 0.13091E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.11765E+01_JPRB, 0.11220E+01_JPRB, 0.11250E+01_JPRB, 0.11046E+01_JPRB, 0.11275E+01_JPRB, 0.10974E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.11772E+01_JPRB, 0.13166E+01_JPRB, 0.14535E+01_JPRB, 0.15762E+01_JPRB, 0.16879E+01_JPRB, 0.18222E+01_JPRB ,&
&  0.18880E+01_JPRB, 0.18565E+01_JPRB, 0.17707E+01_JPRB, 0.16164E+01_JPRB, 0.15782E+01_JPRB, 0.15975E+01_JPRB, 0.17037E+01_JPRB ,&
&  0.17593E+01_JPRB, 0.18835E+01_JPRB, 0.19390E+01_JPRB, 0.19349E+01_JPRB, 0.18469E+01_JPRB, 0.17735E+01_JPRB, 0.17902E+01_JPRB ,&
&  0.21698E+01_JPRB, 0.22096E+01_JPRB, 0.20336E+01_JPRB, 0.25478E+01_JPRB, 0.31926E+01_JPRB, 0.36742E+01_JPRB, 0.34309E+01_JPRB ,&
&  0.29499E+01_JPRB, 0.24648E+01_JPRB, 0.23468E+01_JPRB, 0.27645E+01_JPRB, 0.31791E+01_JPRB, 0.38458E+01_JPRB, 0.51484E+01_JPRB ,&
&  0.54337E+01_JPRB, 0.31317E+01_JPRB, 0.26073E+01_JPRB, 0.35251E+01_JPRB, 0.30046E+01_JPRB, 0.30807E+01_JPRB, 0.45448E+01_JPRB ,&
&  0.56418E+01_JPRB, 0.63291E+01_JPRB, 0.59978E+01_JPRB, 0.52408E+01_JPRB, 0.48464E+01_JPRB, 0.42586E+01_JPRB, 0.37335E+01_JPRB ,&
&  0.32098E+01_JPRB, 0.27916E+01_JPRB, 0.24947E+01_JPRB, 0.22642E+01_JPRB, 0.20491E+01_JPRB, 0.18840E+01_JPRB, 0.18077E+01_JPRB ,&
&  0.17549E+01_JPRB, 0.16892E+01_JPRB, 0.16224E+01_JPRB, 0.15089E+01_JPRB, 0.14035E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.13044E+01_JPRB, 0.12690E+01_JPRB, 0.12452E+01_JPRB, 0.12004E+01_JPRB, 0.11839E+01_JPRB, 0.11560E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.97155E+00_JPRB, 0.85322E+00_JPRB, 0.84146E+00_JPRB, 0.86335E+00_JPRB, 0.90117E+00_JPRB, 0.95452E+00_JPRB, 0.10617E+01_JPRB ,&
&  0.12350E+01_JPRB, 0.14108E+01_JPRB, 0.17626E+01_JPRB, 0.22082E+01_JPRB, 0.26417E+01_JPRB, 0.33660E+01_JPRB, 0.23663E+01_JPRB ,&
&  0.23872E+01_JPRB, 0.22357E+01_JPRB, 0.19367E+01_JPRB, 0.18682E+01_JPRB, 0.17620E+01_JPRB, 0.17209E+01_JPRB, 0.14879E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.13175E+01_JPRB, 0.12563E+01_JPRB, 0.11979E+01_JPRB, 0.11794E+01_JPRB, 0.11690E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.12593E+01_JPRB, 0.13798E+01_JPRB, 0.14961E+01_JPRB, 0.16200E+01_JPRB, 0.17575E+01_JPRB, 0.18583E+01_JPRB ,&
&  0.19624E+01_JPRB, 0.19581E+01_JPRB, 0.18683E+01_JPRB, 0.16884E+01_JPRB, 0.16678E+01_JPRB, 0.16714E+01_JPRB, 0.16847E+01_JPRB ,&
&  0.15960E+01_JPRB, 0.15361E+01_JPRB, 0.17204E+01_JPRB, 0.18241E+01_JPRB, 0.17764E+01_JPRB, 0.16719E+01_JPRB, 0.17020E+01_JPRB ,&
&  0.21002E+01_JPRB, 0.23941E+01_JPRB, 0.27554E+01_JPRB, 0.29829E+01_JPRB, 0.34346E+01_JPRB, 0.37294E+01_JPRB, 0.38885E+01_JPRB ,&
&  0.39798E+01_JPRB, 0.35234E+01_JPRB, 0.33773E+01_JPRB, 0.35935E+01_JPRB, 0.36871E+01_JPRB, 0.41532E+01_JPRB, 0.49100E+01_JPRB ,&
&  0.59170E+01_JPRB, 0.74154E+01_JPRB, 0.76127E+01_JPRB, 0.61718E+01_JPRB, 0.54270E+01_JPRB, 0.46389E+01_JPRB, 0.49934E+01_JPRB ,&
&  0.57940E+01_JPRB, 0.56096E+01_JPRB, 0.53523E+01_JPRB, 0.55335E+01_JPRB, 0.42226E+01_JPRB, 0.36057E+01_JPRB, 0.31854E+01_JPRB ,&
&  0.28677E+01_JPRB, 0.24807E+01_JPRB, 0.21699E+01_JPRB, 0.19238E+01_JPRB, 0.16968E+01_JPRB, 0.15940E+01_JPRB, 0.15844E+01_JPRB ,&
&  0.15638E+01_JPRB, 0.15404E+01_JPRB, 0.15002E+01_JPRB, 0.14248E+01_JPRB, 0.13590E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.12729E+01_JPRB, 0.12317E+01_JPRB, 0.12075E+01_JPRB, 0.11814E+01_JPRB, 0.12148E+01_JPRB, 0.12150E+01_JPRB, 0.11927E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.91176E+00_JPRB, 0.89070E+00_JPRB, 0.91593E+00_JPRB, 0.96685E+00_JPRB, 0.10415E+01_JPRB, 0.11733E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.15072E+01_JPRB, 0.18990E+01_JPRB, 0.24015E+01_JPRB, 0.28498E+01_JPRB, 0.31981E+01_JPRB, 0.32147E+01_JPRB ,&
&  0.20167E+01_JPRB, 0.23113E+01_JPRB, 0.19666E+01_JPRB, 0.20531E+01_JPRB, 0.20687E+01_JPRB, 0.20168E+01_JPRB, 0.16842E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.14119E+01_JPRB, 0.13707E+01_JPRB, 0.13713E+01_JPRB, 0.13106E+01_JPRB, 0.12341E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12796E+01_JPRB, 0.13938E+01_JPRB, 0.15195E+01_JPRB, 0.16209E+01_JPRB, 0.17537E+01_JPRB, 0.18797E+01_JPRB, 0.19658E+01_JPRB ,&
&  0.20607E+01_JPRB, 0.20678E+01_JPRB, 0.19153E+01_JPRB, 0.18240E+01_JPRB, 0.17723E+01_JPRB, 0.17732E+01_JPRB, 0.17489E+01_JPRB ,&
&  0.16331E+01_JPRB, 0.15413E+01_JPRB, 0.16435E+01_JPRB, 0.16435E+01_JPRB, 0.15927E+01_JPRB, 0.16812E+01_JPRB, 0.18045E+01_JPRB ,&
&  0.23014E+01_JPRB, 0.25872E+01_JPRB, 0.35253E+01_JPRB, 0.32852E+01_JPRB, 0.33575E+01_JPRB, 0.37287E+01_JPRB, 0.36601E+01_JPRB ,&
&  0.33416E+01_JPRB, 0.35270E+01_JPRB, 0.35792E+01_JPRB, 0.40590E+01_JPRB, 0.44531E+01_JPRB, 0.49727E+01_JPRB, 0.52650E+01_JPRB ,&
&  0.58048E+01_JPRB, 0.64946E+01_JPRB, 0.72472E+01_JPRB, 0.58698E+01_JPRB, 0.52559E+01_JPRB, 0.48802E+01_JPRB, 0.51976E+01_JPRB ,&
&  0.54797E+01_JPRB, 0.52074E+01_JPRB, 0.52019E+01_JPRB, 0.47433E+01_JPRB, 0.35340E+01_JPRB, 0.30784E+01_JPRB, 0.27061E+01_JPRB ,&
&  0.24158E+01_JPRB, 0.21033E+01_JPRB, 0.18436E+01_JPRB, 0.16301E+01_JPRB, 0.15010E+01_JPRB, 0.14691E+01_JPRB, 0.14737E+01_JPRB ,&
&  0.14673E+01_JPRB, 0.14550E+01_JPRB, 0.14328E+01_JPRB, 0.13745E+01_JPRB, 0.13190E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.12578E+01_JPRB, 0.12345E+01_JPRB, 0.12206E+01_JPRB, 0.11769E+01_JPRB, 0.12134E+01_JPRB, 0.13118E+01_JPRB, 0.13831E+01_JPRB ,&
&  0.13302E+01_JPRB, 0.95492E+00_JPRB, 0.94351E+00_JPRB, 0.97141E+00_JPRB, 0.10214E+01_JPRB, 0.11225E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.13886E+01_JPRB, 0.16312E+01_JPRB, 0.20608E+01_JPRB, 0.25690E+01_JPRB, 0.29877E+01_JPRB, 0.34818E+01_JPRB, 0.36167E+01_JPRB ,&
&  0.25258E+01_JPRB, 0.19950E+01_JPRB, 0.19517E+01_JPRB, 0.21014E+01_JPRB, 0.20523E+01_JPRB, 0.21591E+01_JPRB, 0.21461E+01_JPRB ,&
&  0.19296E+01_JPRB, 0.16841E+01_JPRB, 0.16874E+01_JPRB, 0.16768E+01_JPRB, 0.13884E+01_JPRB, 0.13719E+01_JPRB, 0.13910E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.15746E+01_JPRB, 0.16879E+01_JPRB, 0.17825E+01_JPRB, 0.19213E+01_JPRB, 0.20260E+01_JPRB, 0.21194E+01_JPRB ,&
&  0.21563E+01_JPRB, 0.21773E+01_JPRB, 0.20550E+01_JPRB, 0.19151E+01_JPRB, 0.18655E+01_JPRB, 0.18292E+01_JPRB, 0.17606E+01_JPRB ,&
&  0.16959E+01_JPRB, 0.17168E+01_JPRB, 0.17303E+01_JPRB, 0.16890E+01_JPRB, 0.16681E+01_JPRB, 0.19337E+01_JPRB, 0.22081E+01_JPRB ,&
&  0.25795E+01_JPRB, 0.30713E+01_JPRB, 0.40844E+01_JPRB, 0.32397E+01_JPRB, 0.33047E+01_JPRB, 0.35373E+01_JPRB, 0.35547E+01_JPRB ,&
&  0.34257E+01_JPRB, 0.36475E+01_JPRB, 0.39370E+01_JPRB, 0.43813E+01_JPRB, 0.48977E+01_JPRB, 0.50346E+01_JPRB, 0.62642E+01_JPRB ,&
&  0.59218E+01_JPRB, 0.68108E+01_JPRB, 0.61465E+01_JPRB, 0.57804E+01_JPRB, 0.48499E+01_JPRB, 0.47496E+01_JPRB, 0.45823E+01_JPRB ,&
&  0.47197E+01_JPRB, 0.44785E+01_JPRB, 0.40709E+01_JPRB, 0.36273E+01_JPRB, 0.28296E+01_JPRB, 0.25117E+01_JPRB, 0.21906E+01_JPRB ,&
&  0.19014E+01_JPRB, 0.16968E+01_JPRB, 0.15982E+01_JPRB, 0.15228E+01_JPRB, 0.14936E+01_JPRB, 0.14793E+01_JPRB, 0.14674E+01_JPRB ,&
&  0.14447E+01_JPRB, 0.14229E+01_JPRB, 0.13959E+01_JPRB, 0.13402E+01_JPRB, 0.12934E+01_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.12408E+01_JPRB, 0.12133E+01_JPRB, 0.11774E+01_JPRB, 0.10934E+01_JPRB, 0.10429E+01_JPRB, 0.10425E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10091E+01_JPRB, 0.97981E+00_JPRB, 0.99529E+00_JPRB, 0.10410E+01_JPRB, 0.11035E+01_JPRB, 0.12082E+01_JPRB, 0.13465E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.17358E+01_JPRB, 0.20602E+01_JPRB, 0.24168E+01_JPRB, 0.27249E+01_JPRB, 0.29677E+01_JPRB, 0.31007E+01_JPRB ,&
&  0.25066E+01_JPRB, 0.17722E+01_JPRB, 0.21255E+01_JPRB, 0.23072E+01_JPRB, 0.23104E+01_JPRB, 0.25013E+01_JPRB, 0.24587E+01_JPRB ,&
&  0.20724E+01_JPRB, 0.19539E+01_JPRB, 0.20059E+01_JPRB, 0.18487E+01_JPRB, 0.15032E+01_JPRB, 0.15314E+01_JPRB, 0.15955E+01_JPRB ,&
&  0.16891E+01_JPRB, 0.17663E+01_JPRB, 0.18651E+01_JPRB, 0.19678E+01_JPRB, 0.20785E+01_JPRB, 0.22150E+01_JPRB, 0.23055E+01_JPRB ,&
&  0.23211E+01_JPRB, 0.23068E+01_JPRB, 0.22512E+01_JPRB, 0.20847E+01_JPRB, 0.19710E+01_JPRB, 0.19122E+01_JPRB, 0.18429E+01_JPRB ,&
&  0.18246E+01_JPRB, 0.18100E+01_JPRB, 0.18873E+01_JPRB, 0.20754E+01_JPRB, 0.23777E+01_JPRB, 0.23767E+01_JPRB, 0.27113E+01_JPRB ,&
&  0.29775E+01_JPRB, 0.35685E+01_JPRB, 0.37577E+01_JPRB, 0.42587E+01_JPRB, 0.27033E+01_JPRB, 0.31168E+01_JPRB, 0.29804E+01_JPRB ,&
&  0.37040E+01_JPRB, 0.37878E+01_JPRB, 0.42240E+01_JPRB, 0.48088E+01_JPRB, 0.56774E+01_JPRB, 0.58107E+01_JPRB, 0.62051E+01_JPRB ,&
&  0.59989E+01_JPRB, 0.56814E+01_JPRB, 0.50344E+01_JPRB, 0.49389E+01_JPRB, 0.46637E+01_JPRB, 0.49383E+01_JPRB, 0.41226E+01_JPRB ,&
&  0.40183E+01_JPRB, 0.37151E+01_JPRB, 0.33924E+01_JPRB, 0.30684E+01_JPRB, 0.23700E+01_JPRB, 0.19482E+01_JPRB, 0.16931E+01_JPRB ,&
&  0.15784E+01_JPRB, 0.15266E+01_JPRB, 0.14860E+01_JPRB, 0.14631E+01_JPRB, 0.14655E+01_JPRB, 0.14665E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.14092E+01_JPRB, 0.13750E+01_JPRB, 0.13251E+01_JPRB, 0.12825E+01_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.12059E+01_JPRB, 0.11698E+01_JPRB, 0.11182E+01_JPRB, 0.10539E+01_JPRB, 0.10208E+01_JPRB, 0.99966E+00_JPRB, 0.99672E+00_JPRB ,&
&  0.99936E+00_JPRB, 0.10234E+01_JPRB, 0.10453E+01_JPRB, 0.10833E+01_JPRB, 0.11338E+01_JPRB, 0.12239E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.15773E+01_JPRB, 0.18631E+01_JPRB, 0.22196E+01_JPRB, 0.25158E+01_JPRB, 0.26787E+01_JPRB, 0.25454E+01_JPRB ,&
&  0.24250E+01_JPRB, 0.23774E+01_JPRB, 0.22216E+01_JPRB, 0.18374E+01_JPRB, 0.18416E+01_JPRB, 0.23337E+01_JPRB, 0.21350E+01_JPRB ,&
&  0.18889E+01_JPRB, 0.18311E+01_JPRB, 0.17777E+01_JPRB, 0.17163E+01_JPRB, 0.16583E+01_JPRB, 0.16910E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.18376E+01_JPRB, 0.19174E+01_JPRB, 0.20359E+01_JPRB, 0.21545E+01_JPRB, 0.22805E+01_JPRB, 0.24557E+01_JPRB, 0.25478E+01_JPRB ,&
&  0.25372E+01_JPRB, 0.25170E+01_JPRB, 0.24492E+01_JPRB, 0.22939E+01_JPRB, 0.21167E+01_JPRB, 0.20858E+01_JPRB, 0.21063E+01_JPRB ,&
&  0.21500E+01_JPRB, 0.22237E+01_JPRB, 0.23227E+01_JPRB, 0.26118E+01_JPRB, 0.28812E+01_JPRB, 0.27406E+01_JPRB, 0.28004E+01_JPRB ,&
&  0.32535E+01_JPRB, 0.37961E+01_JPRB, 0.34162E+01_JPRB, 0.40313E+01_JPRB, 0.23409E+01_JPRB, 0.28975E+01_JPRB, 0.35211E+01_JPRB ,&
&  0.36002E+01_JPRB, 0.39448E+01_JPRB, 0.45335E+01_JPRB, 0.52750E+01_JPRB, 0.61071E+01_JPRB, 0.58035E+01_JPRB, 0.57175E+01_JPRB ,&
&  0.51283E+01_JPRB, 0.43667E+01_JPRB, 0.41524E+01_JPRB, 0.40394E+01_JPRB, 0.42308E+01_JPRB, 0.42963E+01_JPRB, 0.32775E+01_JPRB ,&
&  0.27722E+01_JPRB, 0.30717E+01_JPRB, 0.30875E+01_JPRB, 0.28375E+01_JPRB, 0.20034E+01_JPRB, 0.15574E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.14575E+01_JPRB, 0.14277E+01_JPRB, 0.14192E+01_JPRB, 0.14276E+01_JPRB, 0.14454E+01_JPRB, 0.14500E+01_JPRB, 0.14451E+01_JPRB ,&
&  0.14304E+01_JPRB, 0.13945E+01_JPRB, 0.13552E+01_JPRB, 0.13052E+01_JPRB, 0.12538E+01_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.11572E+01_JPRB, 0.11186E+01_JPRB, 0.10623E+01_JPRB, 0.10092E+01_JPRB, 0.94506E+00_JPRB, 0.91270E+00_JPRB, 0.90978E+00_JPRB ,&
&  0.93416E+00_JPRB, 0.97077E+00_JPRB, 0.99732E+00_JPRB, 0.10057E+01_JPRB, 0.10377E+01_JPRB, 0.10581E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.12906E+01_JPRB, 0.15283E+01_JPRB, 0.19091E+01_JPRB, 0.22847E+01_JPRB, 0.26393E+01_JPRB, 0.25329E+01_JPRB ,&
&  0.24082E+01_JPRB, 0.23236E+01_JPRB, 0.22934E+01_JPRB, 0.22999E+01_JPRB, 0.20653E+01_JPRB, 0.23005E+01_JPRB, 0.23250E+01_JPRB ,&
&  0.23337E+01_JPRB, 0.21738E+01_JPRB, 0.18882E+01_JPRB, 0.18296E+01_JPRB, 0.18072E+01_JPRB, 0.18346E+01_JPRB, 0.18997E+01_JPRB ,&
&  0.19799E+01_JPRB, 0.20650E+01_JPRB, 0.22037E+01_JPRB, 0.23748E+01_JPRB, 0.25798E+01_JPRB, 0.27672E+01_JPRB, 0.28410E+01_JPRB ,&
&  0.28303E+01_JPRB, 0.28122E+01_JPRB, 0.27612E+01_JPRB, 0.25969E+01_JPRB, 0.25205E+01_JPRB, 0.25595E+01_JPRB, 0.26564E+01_JPRB ,&
&  0.27865E+01_JPRB, 0.27525E+01_JPRB, 0.28960E+01_JPRB, 0.32270E+01_JPRB, 0.33254E+01_JPRB, 0.32444E+01_JPRB, 0.32559E+01_JPRB ,&
&  0.35518E+01_JPRB, 0.39668E+01_JPRB, 0.27596E+01_JPRB, 0.33456E+01_JPRB, 0.40575E+01_JPRB, 0.35264E+01_JPRB, 0.34712E+01_JPRB ,&
&  0.37705E+01_JPRB, 0.41598E+01_JPRB, 0.47360E+01_JPRB, 0.53976E+01_JPRB, 0.61562E+01_JPRB, 0.67420E+01_JPRB, 0.47229E+01_JPRB ,&
&  0.41275E+01_JPRB, 0.34484E+01_JPRB, 0.33180E+01_JPRB, 0.33694E+01_JPRB, 0.35090E+01_JPRB, 0.33525E+01_JPRB, 0.25305E+01_JPRB ,&
&  0.23125E+01_JPRB, 0.25157E+01_JPRB, 0.24216E+01_JPRB, 0.23370E+01_JPRB, 0.20697E+01_JPRB, 0.13624E+01_JPRB, 0.13842E+01_JPRB ,&
&  0.13867E+01_JPRB, 0.13716E+01_JPRB, 0.13748E+01_JPRB, 0.13876E+01_JPRB, 0.14038E+01_JPRB, 0.14094E+01_JPRB, 0.14067E+01_JPRB ,&
&  0.13829E+01_JPRB, 0.13403E+01_JPRB, 0.12942E+01_JPRB, 0.12456E+01_JPRB, 0.11989E+01_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.11150E+01_JPRB, 0.10669E+01_JPRB, 0.10027E+01_JPRB, 0.93180E+00_JPRB, 0.86823E+00_JPRB, 0.84344E+00_JPRB, 0.83392E+00_JPRB ,&
&  0.84309E+00_JPRB, 0.87497E+00_JPRB, 0.89535E+00_JPRB, 0.89375E+00_JPRB, 0.88911E+00_JPRB, 0.90469E+00_JPRB, 0.95814E+00_JPRB ,&
&  0.10252E+01_JPRB, 0.11080E+01_JPRB, 0.12766E+01_JPRB, 0.16413E+01_JPRB, 0.21174E+01_JPRB, 0.25235E+01_JPRB, 0.25345E+01_JPRB ,&
&  0.24181E+01_JPRB, 0.24404E+01_JPRB, 0.24738E+01_JPRB, 0.25576E+01_JPRB, 0.26330E+01_JPRB, 0.27848E+01_JPRB, 0.29573E+01_JPRB ,&
&  0.26927E+01_JPRB, 0.21871E+01_JPRB, 0.19865E+01_JPRB, 0.18812E+01_JPRB, 0.18960E+01_JPRB, 0.19587E+01_JPRB, 0.20305E+01_JPRB ,&
&  0.21189E+01_JPRB, 0.22484E+01_JPRB, 0.24288E+01_JPRB, 0.26998E+01_JPRB, 0.29817E+01_JPRB, 0.31486E+01_JPRB, 0.32311E+01_JPRB ,&
&  0.32795E+01_JPRB, 0.33111E+01_JPRB, 0.33881E+01_JPRB, 0.30738E+01_JPRB, 0.31897E+01_JPRB, 0.32899E+01_JPRB, 0.33870E+01_JPRB ,&
&  0.33664E+01_JPRB, 0.32723E+01_JPRB, 0.35284E+01_JPRB, 0.37298E+01_JPRB, 0.39311E+01_JPRB, 0.39329E+01_JPRB, 0.38466E+01_JPRB ,&
&  0.40204E+01_JPRB, 0.37004E+01_JPRB, 0.24890E+01_JPRB, 0.24852E+01_JPRB, 0.27514E+01_JPRB, 0.26378E+01_JPRB, 0.34311E+01_JPRB ,&
&  0.36107E+01_JPRB, 0.38843E+01_JPRB, 0.42166E+01_JPRB, 0.45541E+01_JPRB, 0.49090E+01_JPRB, 0.50850E+01_JPRB, 0.38881E+01_JPRB ,&
&  0.31340E+01_JPRB, 0.27688E+01_JPRB, 0.26916E+01_JPRB, 0.26256E+01_JPRB, 0.25616E+01_JPRB, 0.23496E+01_JPRB, 0.22102E+01_JPRB ,&
&  0.21672E+01_JPRB, 0.21159E+01_JPRB, 0.19946E+01_JPRB, 0.17495E+01_JPRB, 0.15053E+01_JPRB, 0.12985E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.13363E+01_JPRB, 0.13175E+01_JPRB, 0.13216E+01_JPRB, 0.13251E+01_JPRB, 0.13339E+01_JPRB, 0.13315E+01_JPRB, 0.13172E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.12543E+01_JPRB, 0.12206E+01_JPRB, 0.11830E+01_JPRB, 0.11459E+01_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.10709E+01_JPRB, 0.10107E+01_JPRB, 0.93481E+00_JPRB, 0.87153E+00_JPRB, 0.86135E+00_JPRB, 0.86486E+00_JPRB, 0.84866E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.83949E+00_JPRB, 0.86234E+00_JPRB, 0.86885E+00_JPRB, 0.87546E+00_JPRB, 0.89563E+00_JPRB, 0.92175E+00_JPRB ,&
&  0.94173E+00_JPRB, 0.95705E+00_JPRB, 0.10395E+01_JPRB, 0.12237E+01_JPRB, 0.15802E+01_JPRB, 0.20642E+01_JPRB, 0.24420E+01_JPRB ,&
&  0.26832E+01_JPRB, 0.28914E+01_JPRB, 0.30959E+01_JPRB, 0.32848E+01_JPRB, 0.31891E+01_JPRB, 0.33806E+01_JPRB, 0.38165E+01_JPRB ,&
&  0.28709E+01_JPRB, 0.20027E+01_JPRB, 0.20513E+01_JPRB, 0.18646E+01_JPRB, 0.19089E+01_JPRB, 0.20364E+01_JPRB, 0.21452E+01_JPRB ,&
&  0.22890E+01_JPRB, 0.24562E+01_JPRB, 0.26561E+01_JPRB, 0.29267E+01_JPRB, 0.32087E+01_JPRB, 0.34095E+01_JPRB, 0.35671E+01_JPRB ,&
&  0.36985E+01_JPRB, 0.38383E+01_JPRB, 0.40369E+01_JPRB, 0.40182E+01_JPRB, 0.39436E+01_JPRB, 0.40873E+01_JPRB, 0.41632E+01_JPRB ,&
&  0.42680E+01_JPRB, 0.42855E+01_JPRB, 0.37778E+01_JPRB, 0.39337E+01_JPRB, 0.44395E+01_JPRB, 0.42222E+01_JPRB, 0.41832E+01_JPRB ,&
&  0.44841E+01_JPRB, 0.37914E+01_JPRB, 0.22986E+01_JPRB, 0.22339E+01_JPRB, 0.25159E+01_JPRB, 0.28629E+01_JPRB, 0.32387E+01_JPRB ,&
&  0.34766E+01_JPRB, 0.36701E+01_JPRB, 0.37142E+01_JPRB, 0.37548E+01_JPRB, 0.35755E+01_JPRB, 0.33132E+01_JPRB, 0.30071E+01_JPRB ,&
&  0.25383E+01_JPRB, 0.23975E+01_JPRB, 0.22869E+01_JPRB, 0.22419E+01_JPRB, 0.22670E+01_JPRB, 0.21598E+01_JPRB, 0.20217E+01_JPRB ,&
&  0.21947E+01_JPRB, 0.23287E+01_JPRB, 0.25767E+01_JPRB, 0.19514E+01_JPRB, 0.14264E+01_JPRB, 0.12790E+01_JPRB, 0.12682E+01_JPRB ,&
&  0.13072E+01_JPRB, 0.12978E+01_JPRB, 0.12937E+01_JPRB, 0.12874E+01_JPRB, 0.12830E+01_JPRB, 0.12618E+01_JPRB, 0.12276E+01_JPRB ,&
&  0.11922E+01_JPRB, 0.11668E+01_JPRB, 0.11523E+01_JPRB, 0.11317E+01_JPRB, 0.11000E+01_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.99470E+00_JPRB, 0.91155E+00_JPRB, 0.86331E+00_JPRB, 0.90603E+00_JPRB, 0.87955E+00_JPRB, 0.90947E+00_JPRB, 0.90179E+00_JPRB ,&
&  0.90697E+00_JPRB, 0.91490E+00_JPRB, 0.94152E+00_JPRB, 0.97807E+00_JPRB, 0.10163E+01_JPRB, 0.10462E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.10897E+01_JPRB, 0.11081E+01_JPRB, 0.11485E+01_JPRB, 0.12339E+01_JPRB, 0.13863E+01_JPRB, 0.16830E+01_JPRB ,&
&  0.20001E+01_JPRB, 0.21971E+01_JPRB, 0.22988E+01_JPRB, 0.23512E+01_JPRB, 0.26584E+01_JPRB, 0.33262E+01_JPRB, 0.38767E+01_JPRB ,&
&  0.23613E+01_JPRB, 0.21063E+01_JPRB, 0.18491E+01_JPRB, 0.16790E+01_JPRB, 0.17655E+01_JPRB, 0.20062E+01_JPRB, 0.21459E+01_JPRB ,&
&  0.22447E+01_JPRB, 0.24006E+01_JPRB, 0.25896E+01_JPRB, 0.28213E+01_JPRB, 0.30263E+01_JPRB, 0.32338E+01_JPRB, 0.34516E+01_JPRB ,&
&  0.36869E+01_JPRB, 0.39872E+01_JPRB, 0.42979E+01_JPRB, 0.46142E+01_JPRB, 0.46057E+01_JPRB, 0.51552E+01_JPRB, 0.52722E+01_JPRB ,&
&  0.53502E+01_JPRB, 0.51132E+01_JPRB, 0.44482E+01_JPRB, 0.45519E+01_JPRB, 0.48731E+01_JPRB, 0.48326E+01_JPRB, 0.45088E+01_JPRB ,&
&  0.45330E+01_JPRB, 0.35570E+01_JPRB, 0.25044E+01_JPRB, 0.25264E+01_JPRB, 0.26184E+01_JPRB, 0.28123E+01_JPRB, 0.27939E+01_JPRB ,&
&  0.29988E+01_JPRB, 0.30392E+01_JPRB, 0.29658E+01_JPRB, 0.27958E+01_JPRB, 0.25864E+01_JPRB, 0.24690E+01_JPRB, 0.24326E+01_JPRB ,&
&  0.22254E+01_JPRB, 0.21611E+01_JPRB, 0.21287E+01_JPRB, 0.22188E+01_JPRB, 0.23587E+01_JPRB, 0.19841E+01_JPRB, 0.19677E+01_JPRB ,&
&  0.21049E+01_JPRB, 0.22768E+01_JPRB, 0.19814E+01_JPRB, 0.21340E+01_JPRB, 0.13243E+01_JPRB, 0.11743E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.12367E+01_JPRB, 0.12119E+01_JPRB, 0.12132E+01_JPRB, 0.12051E+01_JPRB, 0.12077E+01_JPRB, 0.11917E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.10760E+01_JPRB, 0.10756E+01_JPRB, 0.10662E+01_JPRB, 0.10494E+01_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.89897E+00_JPRB, 0.84954E+00_JPRB, 0.90321E+00_JPRB, 0.87867E+00_JPRB, 0.88328E+00_JPRB, 0.90754E+00_JPRB, 0.93197E+00_JPRB ,&
&  0.94908E+00_JPRB, 0.97658E+00_JPRB, 0.10204E+01_JPRB, 0.10812E+01_JPRB, 0.11374E+01_JPRB, 0.11808E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.12908E+01_JPRB, 0.13050E+01_JPRB, 0.13177E+01_JPRB, 0.13549E+01_JPRB, 0.13999E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.15880E+01_JPRB, 0.16313E+01_JPRB, 0.16509E+01_JPRB, 0.17219E+01_JPRB, 0.18101E+01_JPRB, 0.20388E+01_JPRB, 0.25726E+01_JPRB ,&
&  0.20489E+01_JPRB, 0.20624E+01_JPRB, 0.19053E+01_JPRB, 0.18054E+01_JPRB, 0.18709E+01_JPRB, 0.18392E+01_JPRB, 0.19182E+01_JPRB ,&
&  0.20877E+01_JPRB, 0.21420E+01_JPRB, 0.22468E+01_JPRB, 0.23417E+01_JPRB, 0.24707E+01_JPRB, 0.26430E+01_JPRB, 0.28198E+01_JPRB ,&
&  0.30470E+01_JPRB, 0.33157E+01_JPRB, 0.35243E+01_JPRB, 0.38577E+01_JPRB, 0.41917E+01_JPRB, 0.45488E+01_JPRB, 0.47292E+01_JPRB ,&
&  0.48415E+01_JPRB, 0.49119E+01_JPRB, 0.45591E+01_JPRB, 0.46088E+01_JPRB, 0.46245E+01_JPRB, 0.44068E+01_JPRB, 0.38896E+01_JPRB ,&
&  0.33842E+01_JPRB, 0.30034E+01_JPRB, 0.27633E+01_JPRB, 0.26096E+01_JPRB, 0.25676E+01_JPRB, 0.24807E+01_JPRB, 0.24737E+01_JPRB ,&
&  0.25272E+01_JPRB, 0.24178E+01_JPRB, 0.22684E+01_JPRB, 0.20687E+01_JPRB, 0.19693E+01_JPRB, 0.19445E+01_JPRB, 0.19833E+01_JPRB ,&
&  0.19446E+01_JPRB, 0.19239E+01_JPRB, 0.20379E+01_JPRB, 0.23388E+01_JPRB, 0.27241E+01_JPRB, 0.35381E+01_JPRB, 0.26122E+01_JPRB ,&
&  0.19099E+01_JPRB, 0.17696E+01_JPRB, 0.11761E+01_JPRB, 0.14743E+01_JPRB, 0.10941E+01_JPRB, 0.10285E+01_JPRB, 0.10714E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.10959E+01_JPRB, 0.11467E+01_JPRB, 0.11160E+01_JPRB, 0.10861E+01_JPRB, 0.10716E+01_JPRB, 0.10760E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.97470E+00_JPRB, 0.91485E+00_JPRB, 0.92868E+00_JPRB, 0.87703E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.76052E+00_JPRB, 0.78591E+00_JPRB, 0.89560E+00_JPRB, 0.84578E+00_JPRB, 0.85452E+00_JPRB, 0.88414E+00_JPRB, 0.92518E+00_JPRB ,&
&  0.96953E+00_JPRB, 0.10235E+01_JPRB, 0.10806E+01_JPRB, 0.11557E+01_JPRB, 0.12231E+01_JPRB, 0.12809E+01_JPRB, 0.13359E+01_JPRB ,&
&  0.13859E+01_JPRB, 0.14301E+01_JPRB, 0.14728E+01_JPRB, 0.14997E+01_JPRB, 0.15149E+01_JPRB, 0.15576E+01_JPRB, 0.16112E+01_JPRB ,&
&  0.17210E+01_JPRB, 0.17819E+01_JPRB, 0.18543E+01_JPRB, 0.19671E+01_JPRB, 0.20622E+01_JPRB, 0.22110E+01_JPRB, 0.20530E+01_JPRB ,&
&  0.23620E+01_JPRB, 0.20435E+01_JPRB, 0.19382E+01_JPRB, 0.19158E+01_JPRB, 0.18630E+01_JPRB, 0.18173E+01_JPRB, 0.18218E+01_JPRB ,&
&  0.18990E+01_JPRB, 0.18537E+01_JPRB, 0.18805E+01_JPRB, 0.18759E+01_JPRB, 0.19519E+01_JPRB, 0.21007E+01_JPRB, 0.22087E+01_JPRB ,&
&  0.23965E+01_JPRB, 0.26248E+01_JPRB, 0.28750E+01_JPRB, 0.30685E+01_JPRB, 0.32397E+01_JPRB, 0.32532E+01_JPRB, 0.32545E+01_JPRB ,&
&  0.33437E+01_JPRB, 0.35448E+01_JPRB, 0.35002E+01_JPRB, 0.36069E+01_JPRB, 0.36017E+01_JPRB, 0.33100E+01_JPRB, 0.26227E+01_JPRB ,&
&  0.23132E+01_JPRB, 0.24861E+01_JPRB, 0.23224E+01_JPRB, 0.27337E+01_JPRB, 0.24567E+01_JPRB, 0.22002E+01_JPRB, 0.21962E+01_JPRB ,&
&  0.21094E+01_JPRB, 0.19312E+01_JPRB, 0.17853E+01_JPRB, 0.15785E+01_JPRB, 0.15389E+01_JPRB, 0.15526E+01_JPRB, 0.16079E+01_JPRB ,&
&  0.15809E+01_JPRB, 0.15833E+01_JPRB, 0.16643E+01_JPRB, 0.19998E+01_JPRB, 0.29369E+01_JPRB, 0.30276E+01_JPRB, 0.32179E+01_JPRB ,&
&  0.18375E+01_JPRB, 0.11043E+01_JPRB, 0.11509E+01_JPRB, 0.10638E+01_JPRB, 0.94282E+00_JPRB, 0.86067E+00_JPRB, 0.91861E+00_JPRB ,&
&  0.99908E+00_JPRB, 0.10025E+01_JPRB, 0.98109E+00_JPRB, 0.98993E+00_JPRB, 0.98916E+00_JPRB, 0.99622E+00_JPRB, 0.92026E+00_JPRB ,&
&  0.90238E+00_JPRB, 0.84368E+00_JPRB, 0.80823E+00_JPRB, 0.79241E+00_JPRB, 0.75072E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.70080E+00_JPRB, 0.77876E+00_JPRB, 0.86188E+00_JPRB, 0.81391E+00_JPRB, 0.84070E+00_JPRB, 0.88269E+00_JPRB, 0.93656E+00_JPRB ,&
&  0.99550E+00_JPRB, 0.10612E+01_JPRB, 0.11288E+01_JPRB, 0.12065E+01_JPRB, 0.12701E+01_JPRB, 0.13301E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.14557E+01_JPRB, 0.14933E+01_JPRB, 0.15502E+01_JPRB, 0.15827E+01_JPRB, 0.16228E+01_JPRB, 0.16555E+01_JPRB, 0.16951E+01_JPRB ,&
&  0.18161E+01_JPRB, 0.19085E+01_JPRB, 0.20159E+01_JPRB, 0.21287E+01_JPRB, 0.22063E+01_JPRB, 0.24642E+01_JPRB, 0.20908E+01_JPRB ,&
&  0.23901E+01_JPRB, 0.20572E+01_JPRB, 0.19275E+01_JPRB, 0.19473E+01_JPRB, 0.19033E+01_JPRB, 0.18899E+01_JPRB, 0.18862E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.19038E+01_JPRB, 0.19093E+01_JPRB, 0.18827E+01_JPRB, 0.17409E+01_JPRB, 0.18324E+01_JPRB, 0.19376E+01_JPRB ,&
&  0.20648E+01_JPRB, 0.22252E+01_JPRB, 0.23485E+01_JPRB, 0.24245E+01_JPRB, 0.24632E+01_JPRB, 0.24107E+01_JPRB, 0.23572E+01_JPRB ,&
&  0.22930E+01_JPRB, 0.23241E+01_JPRB, 0.22105E+01_JPRB, 0.23887E+01_JPRB, 0.26503E+01_JPRB, 0.22529E+01_JPRB, 0.17256E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.19169E+01_JPRB, 0.23220E+01_JPRB, 0.26570E+01_JPRB, 0.22629E+01_JPRB, 0.20468E+01_JPRB, 0.19352E+01_JPRB ,&
&  0.17741E+01_JPRB, 0.15848E+01_JPRB, 0.14185E+01_JPRB, 0.12525E+01_JPRB, 0.11925E+01_JPRB, 0.11836E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.12074E+01_JPRB, 0.12118E+01_JPRB, 0.12739E+01_JPRB, 0.14195E+01_JPRB, 0.18811E+01_JPRB, 0.22215E+01_JPRB, 0.39263E+01_JPRB ,&
&  0.19826E+01_JPRB, 0.13505E+01_JPRB, 0.20398E+01_JPRB, 0.67696E+00_JPRB, 0.73131E+00_JPRB, 0.75201E+00_JPRB, 0.78624E+00_JPRB ,&
&  0.73432E+00_JPRB, 0.92935E+00_JPRB, 0.12707E+01_JPRB, 0.12772E+01_JPRB, 0.13289E+01_JPRB, 0.14026E+01_JPRB, 0.88733E+00_JPRB ,&
&  0.74861E+00_JPRB, 0.74666E+00_JPRB, 0.75510E+00_JPRB, 0.70332E+00_JPRB, 0.69364E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.65197E+00_JPRB, 0.71664E+00_JPRB, 0.80097E+00_JPRB, 0.87035E+00_JPRB, 0.87568E+00_JPRB, 0.89812E+00_JPRB, 0.93640E+00_JPRB ,&
&  0.97777E+00_JPRB, 0.10245E+01_JPRB, 0.10697E+01_JPRB, 0.11260E+01_JPRB, 0.11672E+01_JPRB, 0.12197E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.14578E+01_JPRB, 0.15173E+01_JPRB, 0.15856E+01_JPRB, 0.16514E+01_JPRB, 0.17148E+01_JPRB, 0.17860E+01_JPRB ,&
&  0.18393E+01_JPRB, 0.19264E+01_JPRB, 0.20128E+01_JPRB, 0.21896E+01_JPRB, 0.23565E+01_JPRB, 0.27978E+01_JPRB, 0.21019E+01_JPRB ,&
&  0.23096E+01_JPRB, 0.20307E+01_JPRB, 0.19143E+01_JPRB, 0.19490E+01_JPRB, 0.19319E+01_JPRB, 0.19073E+01_JPRB, 0.19138E+01_JPRB ,&
&  0.20076E+01_JPRB, 0.20677E+01_JPRB, 0.20615E+01_JPRB, 0.21471E+01_JPRB, 0.20314E+01_JPRB, 0.20157E+01_JPRB, 0.22112E+01_JPRB ,&
&  0.23980E+01_JPRB, 0.25649E+01_JPRB, 0.26305E+01_JPRB, 0.27283E+01_JPRB, 0.27334E+01_JPRB, 0.26744E+01_JPRB, 0.26013E+01_JPRB ,&
&  0.25083E+01_JPRB, 0.24880E+01_JPRB, 0.25609E+01_JPRB, 0.28050E+01_JPRB, 0.29077E+01_JPRB, 0.25824E+01_JPRB, 0.21095E+01_JPRB ,&
&  0.15566E+01_JPRB, 0.14049E+01_JPRB, 0.20077E+01_JPRB, 0.23862E+01_JPRB, 0.20579E+01_JPRB, 0.18765E+01_JPRB, 0.16559E+01_JPRB ,&
&  0.15467E+01_JPRB, 0.13743E+01_JPRB, 0.12357E+01_JPRB, 0.10892E+01_JPRB, 0.10285E+01_JPRB, 0.10080E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.10591E+01_JPRB, 0.10753E+01_JPRB, 0.11501E+01_JPRB, 0.15628E+01_JPRB, 0.22239E+01_JPRB, 0.23291E+01_JPRB ,&
&  0.22954E+01_JPRB, 0.14925E+01_JPRB, 0.12078E+01_JPRB, 0.11773E+01_JPRB, 0.72482E+00_JPRB, 0.73595E+00_JPRB, 0.77073E+00_JPRB ,&
&  0.96294E+00_JPRB, 0.10373E+01_JPRB, 0.13624E+01_JPRB, 0.35845E+01_JPRB, 0.40586E+01_JPRB, 0.50411E+01_JPRB, 0.20641E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.70487E+00_JPRB, 0.64406E+00_JPRB, 0.64961E+00_JPRB, 0.64060E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.61603E+00_JPRB, 0.66847E+00_JPRB, 0.71640E+00_JPRB, 0.78657E+00_JPRB, 0.82158E+00_JPRB, 0.84258E+00_JPRB, 0.85559E+00_JPRB ,&
&  0.85386E+00_JPRB, 0.86506E+00_JPRB, 0.89568E+00_JPRB, 0.96886E+00_JPRB, 0.10487E+01_JPRB, 0.11331E+01_JPRB, 0.12317E+01_JPRB ,&
&  0.13587E+01_JPRB, 0.14828E+01_JPRB, 0.15159E+01_JPRB, 0.15748E+01_JPRB, 0.16631E+01_JPRB, 0.17428E+01_JPRB, 0.17988E+01_JPRB ,&
&  0.18605E+01_JPRB, 0.19690E+01_JPRB, 0.20983E+01_JPRB, 0.22666E+01_JPRB, 0.24797E+01_JPRB, 0.27169E+01_JPRB, 0.20294E+01_JPRB ,&
&  0.22709E+01_JPRB, 0.20316E+01_JPRB, 0.20121E+01_JPRB, 0.20408E+01_JPRB, 0.19559E+01_JPRB, 0.18741E+01_JPRB, 0.19225E+01_JPRB ,&
&  0.20800E+01_JPRB, 0.22723E+01_JPRB, 0.23527E+01_JPRB, 0.23362E+01_JPRB, 0.24975E+01_JPRB, 0.24360E+01_JPRB, 0.27998E+01_JPRB ,&
&  0.31057E+01_JPRB, 0.33030E+01_JPRB, 0.33974E+01_JPRB, 0.34876E+01_JPRB, 0.35972E+01_JPRB, 0.37437E+01_JPRB, 0.39166E+01_JPRB ,&
&  0.40175E+01_JPRB, 0.40664E+01_JPRB, 0.41629E+01_JPRB, 0.42647E+01_JPRB, 0.38234E+01_JPRB, 0.40687E+01_JPRB, 0.32666E+01_JPRB ,&
&  0.21592E+01_JPRB, 0.17241E+01_JPRB, 0.20697E+01_JPRB, 0.21551E+01_JPRB, 0.19057E+01_JPRB, 0.17031E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.13541E+01_JPRB, 0.12290E+01_JPRB, 0.11182E+01_JPRB, 0.10084E+01_JPRB, 0.92784E+00_JPRB, 0.88904E+00_JPRB, 0.88455E+00_JPRB ,&
&  0.89481E+00_JPRB, 0.89632E+00_JPRB, 0.91050E+00_JPRB, 0.99993E+00_JPRB, 0.12785E+01_JPRB, 0.19163E+01_JPRB, 0.28304E+01_JPRB ,&
&  0.32152E+01_JPRB, 0.15540E+01_JPRB, 0.11319E+01_JPRB, 0.96266E+00_JPRB, 0.87977E+00_JPRB, 0.89439E+00_JPRB, 0.92249E+00_JPRB ,&
&  0.10601E+01_JPRB, 0.13815E+01_JPRB, 0.12654E+01_JPRB, 0.14183E+01_JPRB, 0.26639E+01_JPRB, 0.28500E+01_JPRB, 0.16741E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.79241E+00_JPRB, 0.65487E+00_JPRB, 0.61562E+00_JPRB, 0.60821E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.61857E+00_JPRB, 0.63657E+00_JPRB, 0.66364E+00_JPRB, 0.69756E+00_JPRB, 0.70256E+00_JPRB, 0.71183E+00_JPRB, 0.70515E+00_JPRB ,&
&  0.70857E+00_JPRB, 0.73543E+00_JPRB, 0.79897E+00_JPRB, 0.90675E+00_JPRB, 0.99173E+00_JPRB, 0.11062E+01_JPRB, 0.12682E+01_JPRB ,&
&  0.13702E+01_JPRB, 0.14034E+01_JPRB, 0.14414E+01_JPRB, 0.15000E+01_JPRB, 0.15652E+01_JPRB, 0.16433E+01_JPRB, 0.17217E+01_JPRB ,&
&  0.18199E+01_JPRB, 0.19997E+01_JPRB, 0.21726E+01_JPRB, 0.23834E+01_JPRB, 0.25916E+01_JPRB, 0.27710E+01_JPRB, 0.31539E+01_JPRB ,&
&  0.17337E+01_JPRB, 0.21938E+01_JPRB, 0.22139E+01_JPRB, 0.21681E+01_JPRB, 0.18616E+01_JPRB, 0.17389E+01_JPRB, 0.18977E+01_JPRB ,&
&  0.23190E+01_JPRB, 0.26741E+01_JPRB, 0.30444E+01_JPRB, 0.25687E+01_JPRB, 0.27968E+01_JPRB, 0.29596E+01_JPRB, 0.33934E+01_JPRB ,&
&  0.35704E+01_JPRB, 0.37375E+01_JPRB, 0.39494E+01_JPRB, 0.41902E+01_JPRB, 0.44482E+01_JPRB, 0.47524E+01_JPRB, 0.50334E+01_JPRB ,&
&  0.52496E+01_JPRB, 0.53500E+01_JPRB, 0.55762E+01_JPRB, 0.50108E+01_JPRB, 0.43698E+01_JPRB, 0.46145E+01_JPRB, 0.43996E+01_JPRB ,&
&  0.35942E+01_JPRB, 0.24556E+01_JPRB, 0.23101E+01_JPRB, 0.22857E+01_JPRB, 0.19653E+01_JPRB, 0.17370E+01_JPRB, 0.15284E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.10840E+01_JPRB, 0.99963E+00_JPRB, 0.91961E+00_JPRB, 0.85767E+00_JPRB, 0.83264E+00_JPRB, 0.84772E+00_JPRB ,&
&  0.87248E+00_JPRB, 0.88273E+00_JPRB, 0.87649E+00_JPRB, 0.90154E+00_JPRB, 0.10024E+01_JPRB, 0.11015E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.11079E+01_JPRB, 0.10573E+01_JPRB, 0.10529E+01_JPRB, 0.10381E+01_JPRB, 0.99998E+00_JPRB, 0.10135E+01_JPRB ,&
&  0.10086E+01_JPRB, 0.99207E+00_JPRB, 0.10846E+01_JPRB, 0.13158E+01_JPRB, 0.13209E+01_JPRB, 0.13690E+01_JPRB, 0.11402E+01_JPRB ,&
&  0.93013E+00_JPRB, 0.80081E+00_JPRB, 0.71099E+00_JPRB, 0.65305E+00_JPRB, 0.60669E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.61158E+00_JPRB, 0.61000E+00_JPRB, 0.61151E+00_JPRB, 0.63121E+00_JPRB, 0.63699E+00_JPRB, 0.64436E+00_JPRB, 0.64468E+00_JPRB ,&
&  0.67844E+00_JPRB, 0.74308E+00_JPRB, 0.84260E+00_JPRB, 0.95933E+00_JPRB, 0.10533E+01_JPRB, 0.11658E+01_JPRB, 0.12433E+01_JPRB ,&
&  0.12823E+01_JPRB, 0.13030E+01_JPRB, 0.13327E+01_JPRB, 0.13954E+01_JPRB, 0.14640E+01_JPRB, 0.15567E+01_JPRB, 0.16586E+01_JPRB ,&
&  0.17746E+01_JPRB, 0.19293E+01_JPRB, 0.21408E+01_JPRB, 0.23950E+01_JPRB, 0.26704E+01_JPRB, 0.29263E+01_JPRB, 0.32791E+01_JPRB ,&
&  0.13167E+01_JPRB, 0.16975E+01_JPRB, 0.22942E+01_JPRB, 0.24104E+01_JPRB, 0.18615E+01_JPRB, 0.16810E+01_JPRB, 0.16121E+01_JPRB ,&
&  0.19945E+01_JPRB, 0.24651E+01_JPRB, 0.26721E+01_JPRB, 0.28900E+01_JPRB, 0.29344E+01_JPRB, 0.32914E+01_JPRB, 0.35625E+01_JPRB ,&
&  0.36619E+01_JPRB, 0.37977E+01_JPRB, 0.40410E+01_JPRB, 0.43418E+01_JPRB, 0.46406E+01_JPRB, 0.49114E+01_JPRB, 0.52003E+01_JPRB ,&
&  0.55027E+01_JPRB, 0.57171E+01_JPRB, 0.59553E+01_JPRB, 0.50419E+01_JPRB, 0.52499E+01_JPRB, 0.55450E+01_JPRB, 0.61751E+01_JPRB ,&
&  0.50044E+01_JPRB, 0.30417E+01_JPRB, 0.23151E+01_JPRB, 0.23516E+01_JPRB, 0.20449E+01_JPRB, 0.18374E+01_JPRB, 0.15436E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.10686E+01_JPRB, 0.98834E+00_JPRB, 0.96546E+00_JPRB, 0.94029E+00_JPRB, 0.92871E+00_JPRB, 0.93196E+00_JPRB ,&
&  0.94561E+00_JPRB, 0.96356E+00_JPRB, 0.95097E+00_JPRB, 0.95044E+00_JPRB, 0.10186E+01_JPRB, 0.10833E+01_JPRB, 0.11267E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11220E+01_JPRB, 0.11290E+01_JPRB, 0.11479E+01_JPRB, 0.11746E+01_JPRB, 0.11716E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.97672E+00_JPRB, 0.94933E+00_JPRB, 0.92205E+00_JPRB, 0.89164E+00_JPRB, 0.91215E+00_JPRB, 0.85842E+00_JPRB ,&
&  0.82703E+00_JPRB, 0.81600E+00_JPRB, 0.78272E+00_JPRB, 0.70886E+00_JPRB, 0.63059E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.64114E+00_JPRB, 0.63390E+00_JPRB, 0.62593E+00_JPRB, 0.63421E+00_JPRB, 0.62863E+00_JPRB, 0.64073E+00_JPRB, 0.65950E+00_JPRB ,&
&  0.69669E+00_JPRB, 0.76727E+00_JPRB, 0.86001E+00_JPRB, 0.94599E+00_JPRB, 0.10453E+01_JPRB, 0.11310E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.11786E+01_JPRB, 0.12035E+01_JPRB, 0.12332E+01_JPRB, 0.12975E+01_JPRB, 0.13654E+01_JPRB, 0.14581E+01_JPRB, 0.15824E+01_JPRB ,&
&  0.17341E+01_JPRB, 0.19319E+01_JPRB, 0.21841E+01_JPRB, 0.24898E+01_JPRB, 0.28922E+01_JPRB, 0.33705E+01_JPRB, 0.39484E+01_JPRB ,&
&  0.29551E+01_JPRB, 0.15800E+01_JPRB, 0.26945E+01_JPRB, 0.28341E+01_JPRB, 0.21582E+01_JPRB, 0.19360E+01_JPRB, 0.16600E+01_JPRB ,&
&  0.16290E+01_JPRB, 0.22079E+01_JPRB, 0.24321E+01_JPRB, 0.26858E+01_JPRB, 0.28648E+01_JPRB, 0.31927E+01_JPRB, 0.34361E+01_JPRB ,&
&  0.35694E+01_JPRB, 0.37115E+01_JPRB, 0.38821E+01_JPRB, 0.40771E+01_JPRB, 0.43189E+01_JPRB, 0.45679E+01_JPRB, 0.48690E+01_JPRB ,&
&  0.52221E+01_JPRB, 0.55044E+01_JPRB, 0.57120E+01_JPRB, 0.49223E+01_JPRB, 0.53026E+01_JPRB, 0.50582E+01_JPRB, 0.47855E+01_JPRB ,&
&  0.41332E+01_JPRB, 0.25601E+01_JPRB, 0.19074E+01_JPRB, 0.22353E+01_JPRB, 0.20470E+01_JPRB, 0.15465E+01_JPRB, 0.15764E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.11433E+01_JPRB, 0.11081E+01_JPRB, 0.11275E+01_JPRB, 0.10737E+01_JPRB, 0.10305E+01_JPRB, 0.10014E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.10517E+01_JPRB, 0.10386E+01_JPRB, 0.10534E+01_JPRB, 0.10930E+01_JPRB, 0.11518E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.12274E+01_JPRB, 0.12453E+01_JPRB, 0.12561E+01_JPRB, 0.12868E+01_JPRB, 0.13065E+01_JPRB, 0.12296E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10729E+01_JPRB, 0.10444E+01_JPRB, 0.10155E+01_JPRB, 0.95651E+00_JPRB, 0.94898E+00_JPRB, 0.97045E+00_JPRB ,&
&  0.99811E+00_JPRB, 0.99881E+00_JPRB, 0.91785E+00_JPRB, 0.77014E+00_JPRB, 0.67352E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.66638E+00_JPRB, 0.63353E+00_JPRB, 0.62731E+00_JPRB, 0.64873E+00_JPRB, 0.64553E+00_JPRB, 0.65256E+00_JPRB, 0.67611E+00_JPRB ,&
&  0.71332E+00_JPRB, 0.76809E+00_JPRB, 0.81795E+00_JPRB, 0.88199E+00_JPRB, 0.96476E+00_JPRB, 0.10501E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10923E+01_JPRB, 0.11072E+01_JPRB, 0.11382E+01_JPRB, 0.11825E+01_JPRB, 0.12581E+01_JPRB, 0.13418E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.15902E+01_JPRB, 0.17992E+01_JPRB, 0.20294E+01_JPRB, 0.22831E+01_JPRB, 0.26365E+01_JPRB, 0.32018E+01_JPRB, 0.40811E+01_JPRB ,&
&  0.64833E+01_JPRB, 0.69876E+01_JPRB, 0.23418E+01_JPRB, 0.29360E+01_JPRB, 0.26357E+01_JPRB, 0.22937E+01_JPRB, 0.18157E+01_JPRB ,&
&  0.17185E+01_JPRB, 0.20418E+01_JPRB, 0.22323E+01_JPRB, 0.25204E+01_JPRB, 0.25484E+01_JPRB, 0.28133E+01_JPRB, 0.30421E+01_JPRB ,&
&  0.33124E+01_JPRB, 0.34892E+01_JPRB, 0.35821E+01_JPRB, 0.36304E+01_JPRB, 0.37441E+01_JPRB, 0.39204E+01_JPRB, 0.42259E+01_JPRB ,&
&  0.46919E+01_JPRB, 0.52116E+01_JPRB, 0.56172E+01_JPRB, 0.52179E+01_JPRB, 0.54774E+01_JPRB, 0.47706E+01_JPRB, 0.41825E+01_JPRB ,&
&  0.31654E+01_JPRB, 0.26173E+01_JPRB, 0.23592E+01_JPRB, 0.23122E+01_JPRB, 0.19795E+01_JPRB, 0.17373E+01_JPRB, 0.18113E+01_JPRB ,&
&  0.16516E+01_JPRB, 0.14392E+01_JPRB, 0.13016E+01_JPRB, 0.12267E+01_JPRB, 0.11874E+01_JPRB, 0.11296E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.11504E+01_JPRB, 0.11634E+01_JPRB, 0.11420E+01_JPRB, 0.11716E+01_JPRB, 0.12011E+01_JPRB, 0.12454E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.13151E+01_JPRB, 0.13483E+01_JPRB, 0.13644E+01_JPRB, 0.13651E+01_JPRB, 0.12861E+01_JPRB, 0.12072E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11562E+01_JPRB, 0.11719E+01_JPRB, 0.11812E+01_JPRB, 0.11440E+01_JPRB, 0.11450E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.11575E+01_JPRB, 0.11747E+01_JPRB, 0.10580E+01_JPRB, 0.82179E+00_JPRB, 0.71371E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.72403E+00_JPRB, 0.69224E+00_JPRB, 0.68970E+00_JPRB, 0.70136E+00_JPRB, 0.70452E+00_JPRB, 0.68633E+00_JPRB, 0.68707E+00_JPRB ,&
&  0.70372E+00_JPRB, 0.73003E+00_JPRB, 0.73519E+00_JPRB, 0.77703E+00_JPRB, 0.86969E+00_JPRB, 0.96503E+00_JPRB, 0.99538E+00_JPRB ,&
&  0.99968E+00_JPRB, 0.99828E+00_JPRB, 0.10023E+01_JPRB, 0.10316E+01_JPRB, 0.10786E+01_JPRB, 0.11738E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.14077E+01_JPRB, 0.15538E+01_JPRB, 0.16700E+01_JPRB, 0.17533E+01_JPRB, 0.19393E+01_JPRB, 0.23131E+01_JPRB, 0.33112E+01_JPRB ,&
&  0.51673E+01_JPRB, 0.10288E+02_JPRB, 0.27131E+01_JPRB, 0.28465E+01_JPRB, 0.29058E+01_JPRB, 0.26530E+01_JPRB, 0.23330E+01_JPRB ,&
&  0.22894E+01_JPRB, 0.20940E+01_JPRB, 0.21607E+01_JPRB, 0.23393E+01_JPRB, 0.22314E+01_JPRB, 0.23200E+01_JPRB, 0.24638E+01_JPRB ,&
&  0.28102E+01_JPRB, 0.30587E+01_JPRB, 0.32168E+01_JPRB, 0.32081E+01_JPRB, 0.31744E+01_JPRB, 0.31450E+01_JPRB, 0.32207E+01_JPRB ,&
&  0.36018E+01_JPRB, 0.43359E+01_JPRB, 0.51374E+01_JPRB, 0.53161E+01_JPRB, 0.57473E+01_JPRB, 0.50232E+01_JPRB, 0.45291E+01_JPRB ,&
&  0.36713E+01_JPRB, 0.32535E+01_JPRB, 0.31323E+01_JPRB, 0.28155E+01_JPRB, 0.22945E+01_JPRB, 0.20522E+01_JPRB, 0.20415E+01_JPRB ,&
&  0.19207E+01_JPRB, 0.17098E+01_JPRB, 0.14815E+01_JPRB, 0.13175E+01_JPRB, 0.12797E+01_JPRB, 0.12892E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.13331E+01_JPRB, 0.13091E+01_JPRB, 0.13609E+01_JPRB, 0.13794E+01_JPRB, 0.13829E+01_JPRB, 0.13856E+01_JPRB ,&
&  0.13868E+01_JPRB, 0.14184E+01_JPRB, 0.14278E+01_JPRB, 0.13690E+01_JPRB, 0.12518E+01_JPRB, 0.11779E+01_JPRB, 0.11782E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.12264E+01_JPRB, 0.12370E+01_JPRB, 0.12060E+01_JPRB, 0.12469E+01_JPRB, 0.12347E+01_JPRB, 0.12226E+01_JPRB ,&
&  0.12309E+01_JPRB, 0.12688E+01_JPRB, 0.12746E+01_JPRB, 0.94452E+00_JPRB, 0.80007E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.83131E+00_JPRB, 0.79205E+00_JPRB, 0.76297E+00_JPRB, 0.73904E+00_JPRB, 0.72194E+00_JPRB, 0.72237E+00_JPRB, 0.72724E+00_JPRB ,&
&  0.72779E+00_JPRB, 0.73713E+00_JPRB, 0.73637E+00_JPRB, 0.74319E+00_JPRB, 0.76637E+00_JPRB, 0.81699E+00_JPRB, 0.88139E+00_JPRB ,&
&  0.89852E+00_JPRB, 0.87866E+00_JPRB, 0.87308E+00_JPRB, 0.86967E+00_JPRB, 0.89804E+00_JPRB, 0.94824E+00_JPRB, 0.10476E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.13358E+01_JPRB, 0.13944E+01_JPRB, 0.13893E+01_JPRB, 0.14297E+01_JPRB, 0.15585E+01_JPRB, 0.23909E+01_JPRB ,&
&  0.41999E+01_JPRB, 0.92197E+01_JPRB, 0.56096E+01_JPRB, 0.43658E+01_JPRB, 0.32280E+01_JPRB, 0.27895E+01_JPRB, 0.26965E+01_JPRB ,&
&  0.30350E+01_JPRB, 0.28665E+01_JPRB, 0.26163E+01_JPRB, 0.21929E+01_JPRB, 0.19846E+01_JPRB, 0.18757E+01_JPRB, 0.19688E+01_JPRB ,&
&  0.22487E+01_JPRB, 0.25155E+01_JPRB, 0.27061E+01_JPRB, 0.27568E+01_JPRB, 0.26992E+01_JPRB, 0.26351E+01_JPRB, 0.25839E+01_JPRB ,&
&  0.25655E+01_JPRB, 0.28612E+01_JPRB, 0.39642E+01_JPRB, 0.52042E+01_JPRB, 0.59121E+01_JPRB, 0.54493E+01_JPRB, 0.51268E+01_JPRB ,&
&  0.49332E+01_JPRB, 0.43060E+01_JPRB, 0.38999E+01_JPRB, 0.32942E+01_JPRB, 0.25909E+01_JPRB, 0.26169E+01_JPRB, 0.22191E+01_JPRB ,&
&  0.20373E+01_JPRB, 0.18737E+01_JPRB, 0.16732E+01_JPRB, 0.14599E+01_JPRB, 0.14349E+01_JPRB, 0.14266E+01_JPRB, 0.13771E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.14567E+01_JPRB, 0.14348E+01_JPRB, 0.14465E+01_JPRB, 0.14497E+01_JPRB, 0.14585E+01_JPRB, 0.14558E+01_JPRB ,&
&  0.14498E+01_JPRB, 0.14750E+01_JPRB, 0.13930E+01_JPRB, 0.13036E+01_JPRB, 0.12444E+01_JPRB, 0.11771E+01_JPRB, 0.11563E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.12916E+01_JPRB, 0.12971E+01_JPRB, 0.13014E+01_JPRB, 0.12751E+01_JPRB, 0.12776E+01_JPRB, 0.12268E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.11661E+01_JPRB, 0.11346E+01_JPRB, 0.10363E+01_JPRB, 0.90943E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.90434E+00_JPRB, 0.86761E+00_JPRB, 0.79345E+00_JPRB, 0.73277E+00_JPRB, 0.71602E+00_JPRB, 0.72310E+00_JPRB, 0.74126E+00_JPRB ,&
&  0.75423E+00_JPRB, 0.76210E+00_JPRB, 0.76236E+00_JPRB, 0.75472E+00_JPRB, 0.75680E+00_JPRB, 0.72911E+00_JPRB, 0.75055E+00_JPRB ,&
&  0.79344E+00_JPRB, 0.81279E+00_JPRB, 0.83012E+00_JPRB, 0.79981E+00_JPRB, 0.79648E+00_JPRB, 0.79408E+00_JPRB, 0.84146E+00_JPRB ,&
&  0.95137E+00_JPRB, 0.11121E+01_JPRB, 0.12137E+01_JPRB, 0.12344E+01_JPRB, 0.12471E+01_JPRB, 0.13563E+01_JPRB, 0.18912E+01_JPRB ,&
&  0.34110E+01_JPRB, 0.77851E+01_JPRB, 0.32921E+01_JPRB, 0.55318E+01_JPRB, 0.38706E+01_JPRB, 0.30091E+01_JPRB, 0.23786E+01_JPRB ,&
&  0.25347E+01_JPRB, 0.30643E+01_JPRB, 0.24194E+01_JPRB, 0.19816E+01_JPRB, 0.17737E+01_JPRB, 0.17041E+01_JPRB, 0.16921E+01_JPRB ,&
&  0.17836E+01_JPRB, 0.20081E+01_JPRB, 0.22261E+01_JPRB, 0.23268E+01_JPRB, 0.22761E+01_JPRB, 0.21554E+01_JPRB, 0.21157E+01_JPRB ,&
&  0.20896E+01_JPRB, 0.21103E+01_JPRB, 0.25777E+01_JPRB, 0.33490E+01_JPRB, 0.51676E+01_JPRB, 0.58075E+01_JPRB, 0.59656E+01_JPRB ,&
&  0.60788E+01_JPRB, 0.55106E+01_JPRB, 0.46147E+01_JPRB, 0.37954E+01_JPRB, 0.30823E+01_JPRB, 0.29415E+01_JPRB, 0.24787E+01_JPRB ,&
&  0.21843E+01_JPRB, 0.19828E+01_JPRB, 0.17973E+01_JPRB, 0.16280E+01_JPRB, 0.15411E+01_JPRB, 0.15080E+01_JPRB, 0.14657E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.15854E+01_JPRB, 0.15633E+01_JPRB, 0.15097E+01_JPRB, 0.14847E+01_JPRB, 0.14807E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.14709E+01_JPRB, 0.15007E+01_JPRB, 0.14160E+01_JPRB, 0.12882E+01_JPRB, 0.12468E+01_JPRB, 0.12302E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.13709E+01_JPRB, 0.13661E+01_JPRB, 0.13157E+01_JPRB, 0.13044E+01_JPRB, 0.13456E+01_JPRB, 0.12599E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11400E+01_JPRB, 0.10917E+01_JPRB, 0.10432E+01_JPRB, 0.97873E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.91863E+00_JPRB, 0.85146E+00_JPRB, 0.77614E+00_JPRB, 0.73722E+00_JPRB, 0.73492E+00_JPRB, 0.72392E+00_JPRB, 0.73877E+00_JPRB ,&
&  0.77000E+00_JPRB, 0.77718E+00_JPRB, 0.77866E+00_JPRB, 0.76663E+00_JPRB, 0.75477E+00_JPRB, 0.72578E+00_JPRB, 0.69499E+00_JPRB ,&
&  0.72146E+00_JPRB, 0.79873E+00_JPRB, 0.83545E+00_JPRB, 0.81973E+00_JPRB, 0.79398E+00_JPRB, 0.76841E+00_JPRB, 0.76689E+00_JPRB ,&
&  0.80602E+00_JPRB, 0.90239E+00_JPRB, 0.10139E+01_JPRB, 0.10753E+01_JPRB, 0.11009E+01_JPRB, 0.11478E+01_JPRB, 0.14758E+01_JPRB ,&
&  0.25811E+01_JPRB, 0.81004E+01_JPRB, 0.32195E+01_JPRB, 0.60440E+01_JPRB, 0.44109E+01_JPRB, 0.34210E+01_JPRB, 0.24861E+01_JPRB ,&
&  0.23122E+01_JPRB, 0.24943E+01_JPRB, 0.21929E+01_JPRB, 0.19086E+01_JPRB, 0.17063E+01_JPRB, 0.16332E+01_JPRB, 0.16285E+01_JPRB ,&
&  0.16321E+01_JPRB, 0.16491E+01_JPRB, 0.17391E+01_JPRB, 0.18254E+01_JPRB, 0.18353E+01_JPRB, 0.17718E+01_JPRB, 0.17095E+01_JPRB ,&
&  0.16788E+01_JPRB, 0.16883E+01_JPRB, 0.19155E+01_JPRB, 0.24919E+01_JPRB, 0.37658E+01_JPRB, 0.54249E+01_JPRB, 0.71508E+01_JPRB ,&
&  0.76087E+01_JPRB, 0.69787E+01_JPRB, 0.56364E+01_JPRB, 0.46261E+01_JPRB, 0.38754E+01_JPRB, 0.35602E+01_JPRB, 0.30151E+01_JPRB ,&
&  0.25764E+01_JPRB, 0.22743E+01_JPRB, 0.20330E+01_JPRB, 0.18747E+01_JPRB, 0.17331E+01_JPRB, 0.16871E+01_JPRB, 0.16452E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.17607E+01_JPRB, 0.17591E+01_JPRB, 0.16743E+01_JPRB, 0.15636E+01_JPRB, 0.15129E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.14742E+01_JPRB, 0.14439E+01_JPRB, 0.13506E+01_JPRB, 0.12951E+01_JPRB, 0.12825E+01_JPRB, 0.12827E+01_JPRB ,&
&  0.13670E+01_JPRB, 0.14370E+01_JPRB, 0.14286E+01_JPRB, 0.13962E+01_JPRB, 0.13524E+01_JPRB, 0.14001E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.12119E+01_JPRB, 0.11547E+01_JPRB, 0.10992E+01_JPRB, 0.10266E+01_JPRB, 0.98847E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.94224E+00_JPRB, 0.85748E+00_JPRB, 0.79695E+00_JPRB, 0.75666E+00_JPRB, 0.75135E+00_JPRB, 0.74361E+00_JPRB, 0.74384E+00_JPRB ,&
&  0.76673E+00_JPRB, 0.78031E+00_JPRB, 0.79329E+00_JPRB, 0.78201E+00_JPRB, 0.76074E+00_JPRB, 0.75694E+00_JPRB, 0.73501E+00_JPRB ,&
&  0.72779E+00_JPRB, 0.78095E+00_JPRB, 0.83388E+00_JPRB, 0.84355E+00_JPRB, 0.82021E+00_JPRB, 0.77670E+00_JPRB, 0.75679E+00_JPRB ,&
&  0.76393E+00_JPRB, 0.79169E+00_JPRB, 0.84877E+00_JPRB, 0.91187E+00_JPRB, 0.95529E+00_JPRB, 0.98709E+00_JPRB, 0.10812E+01_JPRB ,&
&  0.15912E+01_JPRB, 0.26352E+01_JPRB, 0.47102E+01_JPRB, 0.59215E+01_JPRB, 0.47016E+01_JPRB, 0.35933E+01_JPRB, 0.27833E+01_JPRB ,&
&  0.24177E+01_JPRB, 0.22532E+01_JPRB, 0.20671E+01_JPRB, 0.18500E+01_JPRB, 0.16732E+01_JPRB, 0.15458E+01_JPRB, 0.15142E+01_JPRB ,&
&  0.14738E+01_JPRB, 0.14398E+01_JPRB, 0.14121E+01_JPRB, 0.14111E+01_JPRB, 0.14117E+01_JPRB, 0.14003E+01_JPRB, 0.13808E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.13945E+01_JPRB, 0.14914E+01_JPRB, 0.19038E+01_JPRB, 0.31815E+01_JPRB, 0.48183E+01_JPRB, 0.71862E+01_JPRB ,&
&  0.85324E+01_JPRB, 0.77595E+01_JPRB, 0.64719E+01_JPRB, 0.53893E+01_JPRB, 0.46053E+01_JPRB, 0.41674E+01_JPRB, 0.37075E+01_JPRB ,&
&  0.32697E+01_JPRB, 0.28350E+01_JPRB, 0.24738E+01_JPRB, 0.22458E+01_JPRB, 0.20010E+01_JPRB, 0.18879E+01_JPRB, 0.18912E+01_JPRB ,&
&  0.18432E+01_JPRB, 0.18436E+01_JPRB, 0.18617E+01_JPRB, 0.19288E+01_JPRB, 0.18151E+01_JPRB, 0.16792E+01_JPRB, 0.15531E+01_JPRB ,&
&  0.14563E+01_JPRB, 0.14512E+01_JPRB, 0.14215E+01_JPRB, 0.14030E+01_JPRB, 0.13951E+01_JPRB, 0.13829E+01_JPRB, 0.13747E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.14452E+01_JPRB, 0.14596E+01_JPRB, 0.14484E+01_JPRB, 0.13475E+01_JPRB, 0.14135E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.11730E+01_JPRB, 0.11275E+01_JPRB, 0.10705E+01_JPRB, 0.10295E+01_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.10399E+01_JPRB, 0.95738E+00_JPRB, 0.87325E+00_JPRB, 0.81900E+00_JPRB, 0.78000E+00_JPRB, 0.76300E+00_JPRB, 0.75310E+00_JPRB ,&
&  0.76029E+00_JPRB, 0.76961E+00_JPRB, 0.78531E+00_JPRB, 0.78359E+00_JPRB, 0.76617E+00_JPRB, 0.77770E+00_JPRB, 0.80030E+00_JPRB ,&
&  0.79733E+00_JPRB, 0.81102E+00_JPRB, 0.83466E+00_JPRB, 0.82342E+00_JPRB, 0.79699E+00_JPRB, 0.76925E+00_JPRB, 0.74753E+00_JPRB ,&
&  0.74910E+00_JPRB, 0.75481E+00_JPRB, 0.77156E+00_JPRB, 0.80626E+00_JPRB, 0.85543E+00_JPRB, 0.89642E+00_JPRB, 0.93122E+00_JPRB ,&
&  0.11171E+01_JPRB, 0.29617E+01_JPRB, 0.43656E+01_JPRB, 0.51298E+01_JPRB, 0.46283E+01_JPRB, 0.35640E+01_JPRB, 0.29453E+01_JPRB ,&
&  0.23706E+01_JPRB, 0.20961E+01_JPRB, 0.19177E+01_JPRB, 0.17621E+01_JPRB, 0.15943E+01_JPRB, 0.14906E+01_JPRB, 0.14400E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.13168E+01_JPRB, 0.12335E+01_JPRB, 0.11645E+01_JPRB, 0.11439E+01_JPRB, 0.11422E+01_JPRB, 0.11489E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11835E+01_JPRB, 0.12436E+01_JPRB, 0.14151E+01_JPRB, 0.23499E+01_JPRB, 0.29295E+01_JPRB, 0.53227E+01_JPRB ,&
&  0.65628E+01_JPRB, 0.56994E+01_JPRB, 0.51357E+01_JPRB, 0.47237E+01_JPRB, 0.42702E+01_JPRB, 0.38643E+01_JPRB, 0.36022E+01_JPRB ,&
&  0.33365E+01_JPRB, 0.30377E+01_JPRB, 0.27861E+01_JPRB, 0.25986E+01_JPRB, 0.24161E+01_JPRB, 0.22755E+01_JPRB, 0.21091E+01_JPRB ,&
&  0.20458E+01_JPRB, 0.19595E+01_JPRB, 0.19284E+01_JPRB, 0.19840E+01_JPRB, 0.20201E+01_JPRB, 0.19085E+01_JPRB, 0.17622E+01_JPRB ,&
&  0.16085E+01_JPRB, 0.15194E+01_JPRB, 0.14599E+01_JPRB, 0.14337E+01_JPRB, 0.14477E+01_JPRB, 0.14606E+01_JPRB, 0.14545E+01_JPRB ,&
&  0.14233E+01_JPRB, 0.14247E+01_JPRB, 0.14595E+01_JPRB, 0.14352E+01_JPRB, 0.13511E+01_JPRB, 0.14209E+01_JPRB, 0.13376E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.11646E+01_JPRB, 0.11145E+01_JPRB, 0.10687E+01_JPRB, 0.10799E+01_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.10868E+01_JPRB, 0.10353E+01_JPRB, 0.96809E+00_JPRB, 0.91204E+00_JPRB, 0.86367E+00_JPRB, 0.83129E+00_JPRB, 0.82097E+00_JPRB ,&
&  0.81218E+00_JPRB, 0.80006E+00_JPRB, 0.80519E+00_JPRB, 0.80383E+00_JPRB, 0.80431E+00_JPRB, 0.80966E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.84862E+00_JPRB, 0.84941E+00_JPRB, 0.84422E+00_JPRB, 0.82732E+00_JPRB, 0.80573E+00_JPRB, 0.78090E+00_JPRB, 0.76395E+00_JPRB ,&
&  0.75456E+00_JPRB, 0.75422E+00_JPRB, 0.76029E+00_JPRB, 0.77507E+00_JPRB, 0.80800E+00_JPRB, 0.85308E+00_JPRB, 0.89591E+00_JPRB ,&
&  0.94966E+00_JPRB, 0.17632E+01_JPRB, 0.25236E+01_JPRB, 0.38900E+01_JPRB, 0.42685E+01_JPRB, 0.35527E+01_JPRB, 0.29069E+01_JPRB ,&
&  0.23944E+01_JPRB, 0.20505E+01_JPRB, 0.18285E+01_JPRB, 0.16691E+01_JPRB, 0.15098E+01_JPRB, 0.14168E+01_JPRB, 0.13489E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.12180E+01_JPRB, 0.11379E+01_JPRB, 0.10751E+01_JPRB, 0.10520E+01_JPRB, 0.10535E+01_JPRB, 0.10500E+01_JPRB ,&
&  0.10437E+01_JPRB, 0.10529E+01_JPRB, 0.10690E+01_JPRB, 0.11177E+01_JPRB, 0.14940E+01_JPRB, 0.19205E+01_JPRB, 0.29133E+01_JPRB ,&
&  0.29902E+01_JPRB, 0.28516E+01_JPRB, 0.28306E+01_JPRB, 0.28013E+01_JPRB, 0.28393E+01_JPRB, 0.28580E+01_JPRB, 0.29055E+01_JPRB ,&
&  0.29349E+01_JPRB, 0.29022E+01_JPRB, 0.28137E+01_JPRB, 0.27413E+01_JPRB, 0.26673E+01_JPRB, 0.25797E+01_JPRB, 0.24194E+01_JPRB ,&
&  0.22919E+01_JPRB, 0.21863E+01_JPRB, 0.21153E+01_JPRB, 0.20494E+01_JPRB, 0.20429E+01_JPRB, 0.19916E+01_JPRB, 0.18841E+01_JPRB ,&
&  0.17407E+01_JPRB, 0.16019E+01_JPRB, 0.15382E+01_JPRB, 0.15087E+01_JPRB, 0.14854E+01_JPRB, 0.14550E+01_JPRB, 0.14504E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.14594E+01_JPRB, 0.14476E+01_JPRB, 0.14211E+01_JPRB, 0.13478E+01_JPRB, 0.14003E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.12610E+01_JPRB, 0.11758E+01_JPRB, 0.11036E+01_JPRB, 0.10639E+01_JPRB, 0.11041E+01_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.11719E+01_JPRB, 0.11207E+01_JPRB, 0.10626E+01_JPRB, 0.98579E+00_JPRB, 0.92237E+00_JPRB, 0.89231E+00_JPRB, 0.87340E+00_JPRB ,&
&  0.85084E+00_JPRB, 0.83014E+00_JPRB, 0.82757E+00_JPRB, 0.83572E+00_JPRB, 0.84519E+00_JPRB, 0.85246E+00_JPRB, 0.85124E+00_JPRB ,&
&  0.85154E+00_JPRB, 0.85374E+00_JPRB, 0.84261E+00_JPRB, 0.82480E+00_JPRB, 0.80556E+00_JPRB, 0.78913E+00_JPRB, 0.77652E+00_JPRB ,&
&  0.76877E+00_JPRB, 0.77032E+00_JPRB, 0.77206E+00_JPRB, 0.78295E+00_JPRB, 0.80479E+00_JPRB, 0.83561E+00_JPRB, 0.87868E+00_JPRB ,&
&  0.92930E+00_JPRB, 0.10371E+01_JPRB, 0.15530E+01_JPRB, 0.22974E+01_JPRB, 0.29343E+01_JPRB, 0.29786E+01_JPRB, 0.26580E+01_JPRB ,&
&  0.23568E+01_JPRB, 0.20589E+01_JPRB, 0.18282E+01_JPRB, 0.16366E+01_JPRB, 0.14800E+01_JPRB, 0.13612E+01_JPRB, 0.12751E+01_JPRB ,&
&  0.12007E+01_JPRB, 0.11402E+01_JPRB, 0.10849E+01_JPRB, 0.10358E+01_JPRB, 0.10120E+01_JPRB, 0.99643E+00_JPRB, 0.98313E+00_JPRB ,&
&  0.97134E+00_JPRB, 0.96452E+00_JPRB, 0.96230E+00_JPRB, 0.96812E+00_JPRB, 0.10183E+01_JPRB, 0.11366E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.15733E+01_JPRB, 0.16647E+01_JPRB, 0.16906E+01_JPRB, 0.16905E+01_JPRB, 0.17276E+01_JPRB, 0.18199E+01_JPRB, 0.19650E+01_JPRB ,&
&  0.21420E+01_JPRB, 0.23054E+01_JPRB, 0.24339E+01_JPRB, 0.25133E+01_JPRB, 0.25297E+01_JPRB, 0.24923E+01_JPRB, 0.24198E+01_JPRB ,&
&  0.23390E+01_JPRB, 0.22178E+01_JPRB, 0.20982E+01_JPRB, 0.19986E+01_JPRB, 0.19443E+01_JPRB, 0.19078E+01_JPRB, 0.18427E+01_JPRB ,&
&  0.17465E+01_JPRB, 0.16341E+01_JPRB, 0.15497E+01_JPRB, 0.15032E+01_JPRB, 0.14758E+01_JPRB, 0.14447E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.14270E+01_JPRB, 0.14256E+01_JPRB, 0.13753E+01_JPRB, 0.13733E+01_JPRB, 0.14032E+01_JPRB, 0.13482E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.11744E+01_JPRB, 0.11188E+01_JPRB, 0.10754E+01_JPRB, 0.11630E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.10881E+01_JPRB, 0.10790E+01_JPRB, 0.10384E+01_JPRB, 0.98159E+00_JPRB, 0.92603E+00_JPRB, 0.88067E+00_JPRB, 0.85601E+00_JPRB ,&
&  0.83876E+00_JPRB, 0.82824E+00_JPRB, 0.83493E+00_JPRB, 0.85185E+00_JPRB, 0.87180E+00_JPRB, 0.87931E+00_JPRB, 0.87536E+00_JPRB ,&
&  0.86139E+00_JPRB, 0.83822E+00_JPRB, 0.82056E+00_JPRB, 0.80861E+00_JPRB, 0.79477E+00_JPRB, 0.78701E+00_JPRB, 0.78248E+00_JPRB ,&
&  0.78381E+00_JPRB, 0.78792E+00_JPRB, 0.79028E+00_JPRB, 0.79971E+00_JPRB, 0.81657E+00_JPRB, 0.84397E+00_JPRB, 0.87698E+00_JPRB ,&
&  0.90175E+00_JPRB, 0.83674E+00_JPRB, 0.10382E+01_JPRB, 0.14779E+01_JPRB, 0.19828E+01_JPRB, 0.22464E+01_JPRB, 0.22327E+01_JPRB ,&
&  0.20801E+01_JPRB, 0.19263E+01_JPRB, 0.17646E+01_JPRB, 0.15848E+01_JPRB, 0.14282E+01_JPRB, 0.13035E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.11147E+01_JPRB, 0.10564E+01_JPRB, 0.10210E+01_JPRB, 0.99499E+00_JPRB, 0.97071E+00_JPRB, 0.93558E+00_JPRB, 0.91177E+00_JPRB ,&
&  0.90287E+00_JPRB, 0.89789E+00_JPRB, 0.89630E+00_JPRB, 0.89801E+00_JPRB, 0.91924E+00_JPRB, 0.95677E+00_JPRB, 0.10088E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.11164E+01_JPRB, 0.11448E+01_JPRB, 0.11590E+01_JPRB, 0.11670E+01_JPRB, 0.12336E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.14085E+01_JPRB, 0.15810E+01_JPRB, 0.17843E+01_JPRB, 0.19166E+01_JPRB, 0.19486E+01_JPRB, 0.19044E+01_JPRB, 0.18245E+01_JPRB ,&
&  0.17631E+01_JPRB, 0.16965E+01_JPRB, 0.16281E+01_JPRB, 0.15689E+01_JPRB, 0.15376E+01_JPRB, 0.15447E+01_JPRB, 0.15549E+01_JPRB ,&
&  0.15542E+01_JPRB, 0.15272E+01_JPRB, 0.14919E+01_JPRB, 0.14762E+01_JPRB, 0.14680E+01_JPRB, 0.14446E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.13486E+01_JPRB, 0.13204E+01_JPRB, 0.13216E+01_JPRB, 0.12813E+01_JPRB, 0.12491E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.11318E+01_JPRB, 0.10826E+01_JPRB, 0.10382E+01_JPRB, 0.10214E+01_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.96579E+00_JPRB, 0.10013E+01_JPRB, 0.10055E+01_JPRB, 0.97377E+00_JPRB, 0.93404E+00_JPRB, 0.90407E+00_JPRB, 0.88312E+00_JPRB ,&
&  0.87497E+00_JPRB, 0.86805E+00_JPRB, 0.87205E+00_JPRB, 0.88582E+00_JPRB, 0.90047E+00_JPRB, 0.90296E+00_JPRB, 0.88340E+00_JPRB ,&
&  0.85617E+00_JPRB, 0.83210E+00_JPRB, 0.81650E+00_JPRB, 0.80774E+00_JPRB, 0.79927E+00_JPRB, 0.79451E+00_JPRB, 0.80119E+00_JPRB ,&
&  0.80996E+00_JPRB, 0.82124E+00_JPRB, 0.83461E+00_JPRB, 0.84685E+00_JPRB, 0.85759E+00_JPRB, 0.86888E+00_JPRB, 0.89395E+00_JPRB ,&
&  0.91626E+00_JPRB, 0.75735E+00_JPRB, 0.82423E+00_JPRB, 0.10612E+01_JPRB, 0.13608E+01_JPRB, 0.17008E+01_JPRB, 0.18398E+01_JPRB ,&
&  0.17957E+01_JPRB, 0.16945E+01_JPRB, 0.15741E+01_JPRB, 0.14455E+01_JPRB, 0.13201E+01_JPRB, 0.12153E+01_JPRB, 0.11245E+01_JPRB ,&
&  0.10593E+01_JPRB, 0.10096E+01_JPRB, 0.97567E+00_JPRB, 0.94372E+00_JPRB, 0.92026E+00_JPRB, 0.89906E+00_JPRB, 0.88087E+00_JPRB ,&
&  0.86550E+00_JPRB, 0.85119E+00_JPRB, 0.84568E+00_JPRB, 0.84667E+00_JPRB, 0.84443E+00_JPRB, 0.85146E+00_JPRB, 0.84952E+00_JPRB ,&
&  0.87367E+00_JPRB, 0.90414E+00_JPRB, 0.91951E+00_JPRB, 0.92531E+00_JPRB, 0.93579E+00_JPRB, 0.95828E+00_JPRB, 0.99166E+00_JPRB ,&
&  0.10359E+01_JPRB, 0.11237E+01_JPRB, 0.12235E+01_JPRB, 0.13157E+01_JPRB, 0.13716E+01_JPRB, 0.13845E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.13753E+01_JPRB, 0.13670E+01_JPRB, 0.13470E+01_JPRB, 0.13173E+01_JPRB, 0.12793E+01_JPRB, 0.12621E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.13387E+01_JPRB, 0.13583E+01_JPRB, 0.13773E+01_JPRB, 0.13724E+01_JPRB, 0.13432E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.12766E+01_JPRB, 0.12326E+01_JPRB, 0.11915E+01_JPRB, 0.11171E+01_JPRB, 0.11647E+01_JPRB, 0.11543E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10597E+01_JPRB, 0.10265E+01_JPRB, 0.85187E+00_JPRB, 0.92794E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.90516E+00_JPRB, 0.93793E+00_JPRB, 0.95106E+00_JPRB, 0.94487E+00_JPRB, 0.92484E+00_JPRB, 0.91127E+00_JPRB, 0.90035E+00_JPRB ,&
&  0.89252E+00_JPRB, 0.88642E+00_JPRB, 0.89447E+00_JPRB, 0.90019E+00_JPRB, 0.90035E+00_JPRB, 0.88872E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.85600E+00_JPRB, 0.83375E+00_JPRB, 0.81664E+00_JPRB, 0.80192E+00_JPRB, 0.79801E+00_JPRB, 0.80426E+00_JPRB, 0.81876E+00_JPRB ,&
&  0.83856E+00_JPRB, 0.85277E+00_JPRB, 0.86525E+00_JPRB, 0.87950E+00_JPRB, 0.88452E+00_JPRB, 0.88769E+00_JPRB, 0.89430E+00_JPRB ,&
&  0.90568E+00_JPRB, 0.72486E+00_JPRB, 0.78500E+00_JPRB, 0.87962E+00_JPRB, 0.99917E+00_JPRB, 0.11861E+01_JPRB, 0.13390E+01_JPRB ,&
&  0.13918E+01_JPRB, 0.13274E+01_JPRB, 0.12639E+01_JPRB, 0.11950E+01_JPRB, 0.11275E+01_JPRB, 0.10689E+01_JPRB, 0.10116E+01_JPRB ,&
&  0.96614E+00_JPRB, 0.93824E+00_JPRB, 0.92378E+00_JPRB, 0.90912E+00_JPRB, 0.89212E+00_JPRB, 0.87175E+00_JPRB, 0.84936E+00_JPRB ,&
&  0.82673E+00_JPRB, 0.81124E+00_JPRB, 0.79890E+00_JPRB, 0.79002E+00_JPRB, 0.78843E+00_JPRB, 0.79079E+00_JPRB, 0.79161E+00_JPRB ,&
&  0.80076E+00_JPRB, 0.81001E+00_JPRB, 0.81554E+00_JPRB, 0.81557E+00_JPRB, 0.81579E+00_JPRB, 0.82031E+00_JPRB, 0.84138E+00_JPRB ,&
&  0.86389E+00_JPRB, 0.90118E+00_JPRB, 0.95066E+00_JPRB, 0.10157E+01_JPRB, 0.10685E+01_JPRB, 0.10960E+01_JPRB, 0.11105E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.11078E+01_JPRB, 0.10853E+01_JPRB, 0.10629E+01_JPRB, 0.10441E+01_JPRB, 0.10370E+01_JPRB, 0.10365E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10694E+01_JPRB, 0.10894E+01_JPRB, 0.11136E+01_JPRB, 0.11365E+01_JPRB, 0.11337E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10511E+01_JPRB, 0.10368E+01_JPRB, 0.10482E+01_JPRB, 0.10454E+01_JPRB, 0.10337E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.98658E+00_JPRB, 0.96808E+00_JPRB, 0.75914E+00_JPRB, 0.85070E+00_JPRB, 0.87171E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.88813E+00_JPRB, 0.88474E+00_JPRB, 0.88012E+00_JPRB, 0.88638E+00_JPRB, 0.89232E+00_JPRB, 0.89253E+00_JPRB, 0.89164E+00_JPRB ,&
&  0.88816E+00_JPRB, 0.88313E+00_JPRB, 0.88970E+00_JPRB, 0.89546E+00_JPRB, 0.89432E+00_JPRB, 0.88484E+00_JPRB, 0.87442E+00_JPRB ,&
&  0.86588E+00_JPRB, 0.85438E+00_JPRB, 0.84316E+00_JPRB, 0.83744E+00_JPRB, 0.83841E+00_JPRB, 0.85087E+00_JPRB, 0.87070E+00_JPRB ,&
&  0.88543E+00_JPRB, 0.88406E+00_JPRB, 0.88771E+00_JPRB, 0.89968E+00_JPRB, 0.90383E+00_JPRB, 0.90750E+00_JPRB, 0.91600E+00_JPRB ,&
&  0.79809E+00_JPRB, 0.67833E+00_JPRB, 0.75199E+00_JPRB, 0.82337E+00_JPRB, 0.88713E+00_JPRB, 0.96693E+00_JPRB, 0.10717E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.10971E+01_JPRB, 0.10713E+01_JPRB, 0.10356E+01_JPRB, 0.99580E+00_JPRB, 0.96650E+00_JPRB, 0.94319E+00_JPRB ,&
&  0.93530E+00_JPRB, 0.92210E+00_JPRB, 0.90214E+00_JPRB, 0.88532E+00_JPRB, 0.86873E+00_JPRB, 0.84724E+00_JPRB, 0.82636E+00_JPRB ,&
&  0.80492E+00_JPRB, 0.78618E+00_JPRB, 0.77748E+00_JPRB, 0.77110E+00_JPRB, 0.77014E+00_JPRB, 0.76760E+00_JPRB, 0.76065E+00_JPRB ,&
&  0.75622E+00_JPRB, 0.75680E+00_JPRB, 0.76512E+00_JPRB, 0.77331E+00_JPRB, 0.77629E+00_JPRB, 0.77588E+00_JPRB, 0.78683E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.81882E+00_JPRB, 0.84434E+00_JPRB, 0.88178E+00_JPRB, 0.91273E+00_JPRB, 0.92971E+00_JPRB, 0.94180E+00_JPRB ,&
&  0.95469E+00_JPRB, 0.95170E+00_JPRB, 0.93464E+00_JPRB, 0.92093E+00_JPRB, 0.91381E+00_JPRB, 0.90471E+00_JPRB, 0.89943E+00_JPRB ,&
&  0.90486E+00_JPRB, 0.91262E+00_JPRB, 0.91518E+00_JPRB, 0.92487E+00_JPRB, 0.93818E+00_JPRB, 0.94917E+00_JPRB, 0.95448E+00_JPRB ,&
&  0.93476E+00_JPRB, 0.93492E+00_JPRB, 0.93344E+00_JPRB, 0.94275E+00_JPRB, 0.95299E+00_JPRB, 0.93972E+00_JPRB, 0.92420E+00_JPRB ,&
&  0.90943E+00_JPRB, 0.88286E+00_JPRB, 0.87893E+00_JPRB, 0.87711E+00_JPRB, 0.87896E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.83180E+00_JPRB, 0.82324E+00_JPRB, 0.81828E+00_JPRB, 0.82854E+00_JPRB, 0.84589E+00_JPRB, 0.85950E+00_JPRB, 0.86131E+00_JPRB ,&
&  0.86212E+00_JPRB, 0.86697E+00_JPRB, 0.87498E+00_JPRB, 0.87630E+00_JPRB, 0.86783E+00_JPRB, 0.84925E+00_JPRB, 0.83864E+00_JPRB ,&
&  0.83437E+00_JPRB, 0.83334E+00_JPRB, 0.83164E+00_JPRB, 0.83462E+00_JPRB, 0.85273E+00_JPRB, 0.88206E+00_JPRB, 0.90989E+00_JPRB ,&
&  0.92336E+00_JPRB, 0.91756E+00_JPRB, 0.91123E+00_JPRB, 0.90650E+00_JPRB, 0.90589E+00_JPRB, 0.91155E+00_JPRB, 0.91301E+00_JPRB ,&
&  0.90698E+00_JPRB, 0.76354E+00_JPRB, 0.82584E+00_JPRB, 0.85265E+00_JPRB, 0.88501E+00_JPRB, 0.92333E+00_JPRB, 0.98401E+00_JPRB ,&
&  0.10075E+01_JPRB, 0.99352E+00_JPRB, 0.96332E+00_JPRB, 0.94053E+00_JPRB, 0.92051E+00_JPRB, 0.91018E+00_JPRB, 0.90235E+00_JPRB ,&
&  0.89867E+00_JPRB, 0.89467E+00_JPRB, 0.87180E+00_JPRB, 0.83553E+00_JPRB, 0.81380E+00_JPRB, 0.79748E+00_JPRB, 0.78251E+00_JPRB ,&
&  0.76816E+00_JPRB, 0.75106E+00_JPRB, 0.73488E+00_JPRB, 0.72172E+00_JPRB, 0.72061E+00_JPRB, 0.71650E+00_JPRB, 0.70196E+00_JPRB ,&
&  0.69509E+00_JPRB, 0.69179E+00_JPRB, 0.69873E+00_JPRB, 0.70945E+00_JPRB, 0.71446E+00_JPRB, 0.71124E+00_JPRB, 0.72253E+00_JPRB ,&
&  0.74756E+00_JPRB, 0.76525E+00_JPRB, 0.78053E+00_JPRB, 0.79932E+00_JPRB, 0.81083E+00_JPRB, 0.81463E+00_JPRB, 0.82036E+00_JPRB ,&
&  0.83168E+00_JPRB, 0.83317E+00_JPRB, 0.82565E+00_JPRB, 0.81993E+00_JPRB, 0.82174E+00_JPRB, 0.82315E+00_JPRB, 0.83233E+00_JPRB ,&
&  0.84098E+00_JPRB, 0.84421E+00_JPRB, 0.84046E+00_JPRB, 0.84300E+00_JPRB, 0.84945E+00_JPRB, 0.86081E+00_JPRB, 0.86800E+00_JPRB ,&
&  0.87562E+00_JPRB, 0.87016E+00_JPRB, 0.85626E+00_JPRB, 0.85681E+00_JPRB, 0.86289E+00_JPRB, 0.86070E+00_JPRB, 0.84653E+00_JPRB ,&
&  0.83310E+00_JPRB, 0.82258E+00_JPRB, 0.82371E+00_JPRB, 0.82402E+00_JPRB, 0.82848E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.81101E+00_JPRB, 0.80694E+00_JPRB, 0.80568E+00_JPRB, 0.81696E+00_JPRB, 0.83511E+00_JPRB, 0.84033E+00_JPRB, 0.84307E+00_JPRB ,&
&  0.85674E+00_JPRB, 0.86717E+00_JPRB, 0.86726E+00_JPRB, 0.86559E+00_JPRB, 0.85909E+00_JPRB, 0.84421E+00_JPRB, 0.83240E+00_JPRB ,&
&  0.83651E+00_JPRB, 0.84098E+00_JPRB, 0.84172E+00_JPRB, 0.86433E+00_JPRB, 0.89478E+00_JPRB, 0.93139E+00_JPRB, 0.95608E+00_JPRB ,&
&  0.96464E+00_JPRB, 0.95706E+00_JPRB, 0.95334E+00_JPRB, 0.94831E+00_JPRB, 0.94314E+00_JPRB, 0.94518E+00_JPRB, 0.93978E+00_JPRB ,&
&  0.92986E+00_JPRB, 0.84859E+00_JPRB, 0.88222E+00_JPRB, 0.89307E+00_JPRB, 0.91086E+00_JPRB, 0.93902E+00_JPRB, 0.98089E+00_JPRB ,&
&  0.98975E+00_JPRB, 0.98185E+00_JPRB, 0.96383E+00_JPRB, 0.93433E+00_JPRB, 0.91488E+00_JPRB, 0.90457E+00_JPRB, 0.88628E+00_JPRB ,&
&  0.85802E+00_JPRB, 0.82056E+00_JPRB, 0.78705E+00_JPRB, 0.76399E+00_JPRB, 0.75269E+00_JPRB, 0.74796E+00_JPRB, 0.74041E+00_JPRB ,&
&  0.73531E+00_JPRB, 0.73079E+00_JPRB, 0.71769E+00_JPRB, 0.70584E+00_JPRB, 0.69323E+00_JPRB, 0.68372E+00_JPRB, 0.66875E+00_JPRB ,&
&  0.65996E+00_JPRB, 0.64830E+00_JPRB, 0.64688E+00_JPRB, 0.66667E+00_JPRB, 0.68854E+00_JPRB, 0.69479E+00_JPRB, 0.70975E+00_JPRB ,&
&  0.73040E+00_JPRB, 0.74058E+00_JPRB, 0.74365E+00_JPRB, 0.74170E+00_JPRB, 0.73482E+00_JPRB, 0.73462E+00_JPRB, 0.74360E+00_JPRB ,&
&  0.75240E+00_JPRB, 0.76385E+00_JPRB, 0.76891E+00_JPRB, 0.77438E+00_JPRB, 0.78159E+00_JPRB, 0.78720E+00_JPRB, 0.79802E+00_JPRB ,&
&  0.80368E+00_JPRB, 0.80749E+00_JPRB, 0.81833E+00_JPRB, 0.82971E+00_JPRB, 0.83522E+00_JPRB, 0.83839E+00_JPRB, 0.84874E+00_JPRB ,&
&  0.85367E+00_JPRB, 0.85264E+00_JPRB, 0.84366E+00_JPRB, 0.83103E+00_JPRB, 0.82259E+00_JPRB, 0.81002E+00_JPRB, 0.78962E+00_JPRB ,&
&  0.77012E+00_JPRB, 0.76691E+00_JPRB, 0.77566E+00_JPRB, 0.78544E+00_JPRB, 0.80220E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.79036E+00_JPRB, 0.80177E+00_JPRB, 0.81486E+00_JPRB, 0.82754E+00_JPRB, 0.84062E+00_JPRB, 0.84009E+00_JPRB, 0.84228E+00_JPRB ,&
&  0.86320E+00_JPRB, 0.86784E+00_JPRB, 0.86831E+00_JPRB, 0.86022E+00_JPRB, 0.85411E+00_JPRB, 0.85374E+00_JPRB, 0.84424E+00_JPRB ,&
&  0.83906E+00_JPRB, 0.84448E+00_JPRB, 0.86158E+00_JPRB, 0.88829E+00_JPRB, 0.91442E+00_JPRB, 0.94947E+00_JPRB, 0.97549E+00_JPRB ,&
&  0.97917E+00_JPRB, 0.96787E+00_JPRB, 0.96811E+00_JPRB, 0.97161E+00_JPRB, 0.97172E+00_JPRB, 0.95488E+00_JPRB, 0.95105E+00_JPRB ,&
&  0.93066E+00_JPRB, 0.94599E+00_JPRB, 0.81541E+00_JPRB, 0.88708E+00_JPRB, 0.93267E+00_JPRB, 0.97134E+00_JPRB, 0.99265E+00_JPRB ,&
&  0.98773E+00_JPRB, 0.97401E+00_JPRB, 0.94441E+00_JPRB, 0.91171E+00_JPRB, 0.90044E+00_JPRB, 0.88072E+00_JPRB, 0.85714E+00_JPRB ,&
&  0.80780E+00_JPRB, 0.74460E+00_JPRB, 0.71231E+00_JPRB, 0.70017E+00_JPRB, 0.69367E+00_JPRB, 0.69338E+00_JPRB, 0.69674E+00_JPRB ,&
&  0.69859E+00_JPRB, 0.69751E+00_JPRB, 0.68483E+00_JPRB, 0.66649E+00_JPRB, 0.64347E+00_JPRB, 0.63078E+00_JPRB, 0.62812E+00_JPRB ,&
&  0.62487E+00_JPRB, 0.61088E+00_JPRB, 0.61384E+00_JPRB, 0.64391E+00_JPRB, 0.66901E+00_JPRB, 0.68847E+00_JPRB, 0.69891E+00_JPRB ,&
&  0.70715E+00_JPRB, 0.70295E+00_JPRB, 0.70052E+00_JPRB, 0.69145E+00_JPRB, 0.66725E+00_JPRB, 0.66033E+00_JPRB, 0.68138E+00_JPRB ,&
&  0.70542E+00_JPRB, 0.71940E+00_JPRB, 0.72801E+00_JPRB, 0.74095E+00_JPRB, 0.75495E+00_JPRB, 0.76305E+00_JPRB, 0.76926E+00_JPRB ,&
&  0.77338E+00_JPRB, 0.77747E+00_JPRB, 0.79105E+00_JPRB, 0.80666E+00_JPRB, 0.81422E+00_JPRB, 0.81608E+00_JPRB, 0.81669E+00_JPRB ,&
&  0.81450E+00_JPRB, 0.80730E+00_JPRB, 0.79879E+00_JPRB, 0.79096E+00_JPRB, 0.77135E+00_JPRB, 0.74760E+00_JPRB, 0.72841E+00_JPRB ,&
&  0.71759E+00_JPRB, 0.72677E+00_JPRB, 0.73416E+00_JPRB, 0.74667E+00_JPRB, 0.77341E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.82317E+00_JPRB, 0.83690E+00_JPRB, 0.84825E+00_JPRB, 0.86860E+00_JPRB, 0.88679E+00_JPRB, 0.88378E+00_JPRB, 0.87735E+00_JPRB ,&
&  0.87365E+00_JPRB, 0.87035E+00_JPRB, 0.86427E+00_JPRB, 0.85545E+00_JPRB, 0.84954E+00_JPRB, 0.84923E+00_JPRB, 0.84955E+00_JPRB ,&
&  0.85060E+00_JPRB, 0.87004E+00_JPRB, 0.90222E+00_JPRB, 0.92721E+00_JPRB, 0.95464E+00_JPRB, 0.99701E+00_JPRB, 0.10166E+01_JPRB ,&
&  0.99885E+00_JPRB, 0.99711E+00_JPRB, 0.98622E+00_JPRB, 0.97192E+00_JPRB, 0.97413E+00_JPRB, 0.96556E+00_JPRB, 0.93829E+00_JPRB ,&
&  0.93275E+00_JPRB, 0.92988E+00_JPRB, 0.92597E+00_JPRB, 0.95658E+00_JPRB, 0.98264E+00_JPRB, 0.98743E+00_JPRB, 0.98948E+00_JPRB ,&
&  0.96664E+00_JPRB, 0.92963E+00_JPRB, 0.88981E+00_JPRB, 0.86829E+00_JPRB, 0.86516E+00_JPRB, 0.84535E+00_JPRB, 0.81744E+00_JPRB ,&
&  0.75969E+00_JPRB, 0.71193E+00_JPRB, 0.68517E+00_JPRB, 0.66874E+00_JPRB, 0.66281E+00_JPRB, 0.65908E+00_JPRB, 0.65349E+00_JPRB ,&
&  0.65781E+00_JPRB, 0.65132E+00_JPRB, 0.62125E+00_JPRB, 0.58901E+00_JPRB, 0.58190E+00_JPRB, 0.59050E+00_JPRB, 0.61151E+00_JPRB ,&
&  0.62657E+00_JPRB, 0.62102E+00_JPRB, 0.62748E+00_JPRB, 0.63555E+00_JPRB, 0.65384E+00_JPRB, 0.67361E+00_JPRB, 0.67664E+00_JPRB ,&
&  0.67856E+00_JPRB, 0.67469E+00_JPRB, 0.66237E+00_JPRB, 0.65346E+00_JPRB, 0.64168E+00_JPRB, 0.63632E+00_JPRB, 0.64694E+00_JPRB ,&
&  0.66412E+00_JPRB, 0.67380E+00_JPRB, 0.68419E+00_JPRB, 0.69796E+00_JPRB, 0.71016E+00_JPRB, 0.71926E+00_JPRB, 0.72587E+00_JPRB ,&
&  0.73601E+00_JPRB, 0.75101E+00_JPRB, 0.76644E+00_JPRB, 0.78012E+00_JPRB, 0.78061E+00_JPRB, 0.76634E+00_JPRB, 0.74075E+00_JPRB ,&
&  0.72377E+00_JPRB, 0.71067E+00_JPRB, 0.70814E+00_JPRB, 0.71486E+00_JPRB, 0.71029E+00_JPRB, 0.69060E+00_JPRB, 0.68033E+00_JPRB ,&
&  0.69065E+00_JPRB, 0.71451E+00_JPRB, 0.73708E+00_JPRB, 0.76782E+00_JPRB, 0.79997E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.85628E+00_JPRB, 0.88739E+00_JPRB, 0.91072E+00_JPRB, 0.91807E+00_JPRB, 0.91507E+00_JPRB, 0.90630E+00_JPRB, 0.88429E+00_JPRB ,&
&  0.86027E+00_JPRB, 0.84369E+00_JPRB, 0.82607E+00_JPRB, 0.80002E+00_JPRB, 0.77845E+00_JPRB, 0.77428E+00_JPRB, 0.78699E+00_JPRB ,&
&  0.83330E+00_JPRB, 0.87426E+00_JPRB, 0.89056E+00_JPRB, 0.90376E+00_JPRB, 0.95288E+00_JPRB, 0.99396E+00_JPRB, 0.10022E+01_JPRB ,&
&  0.99558E+00_JPRB, 0.99627E+00_JPRB, 0.95968E+00_JPRB, 0.92691E+00_JPRB, 0.92782E+00_JPRB, 0.93426E+00_JPRB, 0.91732E+00_JPRB ,&
&  0.94408E+00_JPRB, 0.96342E+00_JPRB, 0.96031E+00_JPRB, 0.94124E+00_JPRB, 0.93774E+00_JPRB, 0.92775E+00_JPRB, 0.91494E+00_JPRB ,&
&  0.89259E+00_JPRB, 0.84432E+00_JPRB, 0.81003E+00_JPRB, 0.80521E+00_JPRB, 0.79790E+00_JPRB, 0.79144E+00_JPRB, 0.77881E+00_JPRB ,&
&  0.73858E+00_JPRB, 0.70207E+00_JPRB, 0.67407E+00_JPRB, 0.65472E+00_JPRB, 0.64222E+00_JPRB, 0.63206E+00_JPRB, 0.62099E+00_JPRB ,&
&  0.61107E+00_JPRB, 0.58111E+00_JPRB, 0.55027E+00_JPRB, 0.54178E+00_JPRB, 0.54604E+00_JPRB, 0.55356E+00_JPRB, 0.58217E+00_JPRB ,&
&  0.60732E+00_JPRB, 0.60577E+00_JPRB, 0.60235E+00_JPRB, 0.60561E+00_JPRB, 0.61577E+00_JPRB, 0.61808E+00_JPRB, 0.60949E+00_JPRB ,&
&  0.60566E+00_JPRB, 0.60416E+00_JPRB, 0.59485E+00_JPRB, 0.58782E+00_JPRB, 0.58737E+00_JPRB, 0.59107E+00_JPRB, 0.60480E+00_JPRB ,&
&  0.61305E+00_JPRB, 0.61378E+00_JPRB, 0.62685E+00_JPRB, 0.64375E+00_JPRB, 0.65457E+00_JPRB, 0.66340E+00_JPRB, 0.67005E+00_JPRB ,&
&  0.68429E+00_JPRB, 0.70436E+00_JPRB, 0.71961E+00_JPRB, 0.72196E+00_JPRB, 0.69937E+00_JPRB, 0.67539E+00_JPRB, 0.65667E+00_JPRB ,&
&  0.64838E+00_JPRB, 0.65274E+00_JPRB, 0.66065E+00_JPRB, 0.66844E+00_JPRB, 0.67541E+00_JPRB, 0.68026E+00_JPRB, 0.67991E+00_JPRB ,&
&  0.69980E+00_JPRB, 0.73082E+00_JPRB, 0.75567E+00_JPRB, 0.78520E+00_JPRB, 0.81628E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.91945E+00_JPRB, 0.93661E+00_JPRB, 0.94770E+00_JPRB, 0.94756E+00_JPRB, 0.93134E+00_JPRB, 0.90439E+00_JPRB, 0.88160E+00_JPRB ,&
&  0.85576E+00_JPRB, 0.83058E+00_JPRB, 0.80667E+00_JPRB, 0.77368E+00_JPRB, 0.75589E+00_JPRB, 0.75922E+00_JPRB, 0.80808E+00_JPRB ,&
&  0.87353E+00_JPRB, 0.87860E+00_JPRB, 0.87258E+00_JPRB, 0.89742E+00_JPRB, 0.94907E+00_JPRB, 0.98019E+00_JPRB, 0.10003E+01_JPRB ,&
&  0.10013E+01_JPRB, 0.96720E+00_JPRB, 0.93449E+00_JPRB, 0.92235E+00_JPRB, 0.94727E+00_JPRB, 0.96632E+00_JPRB, 0.97207E+00_JPRB ,&
&  0.96845E+00_JPRB, 0.94920E+00_JPRB, 0.93314E+00_JPRB, 0.91981E+00_JPRB, 0.92553E+00_JPRB, 0.90084E+00_JPRB, 0.84817E+00_JPRB ,&
&  0.78167E+00_JPRB, 0.75460E+00_JPRB, 0.74064E+00_JPRB, 0.72513E+00_JPRB, 0.71438E+00_JPRB, 0.71512E+00_JPRB, 0.70675E+00_JPRB ,&
&  0.69317E+00_JPRB, 0.67648E+00_JPRB, 0.65998E+00_JPRB, 0.64099E+00_JPRB, 0.62582E+00_JPRB, 0.60445E+00_JPRB, 0.58443E+00_JPRB ,&
&  0.56588E+00_JPRB, 0.54266E+00_JPRB, 0.52094E+00_JPRB, 0.50512E+00_JPRB, 0.49747E+00_JPRB, 0.50826E+00_JPRB, 0.54948E+00_JPRB ,&
&  0.58150E+00_JPRB, 0.57867E+00_JPRB, 0.57679E+00_JPRB, 0.58203E+00_JPRB, 0.57651E+00_JPRB, 0.56770E+00_JPRB, 0.56304E+00_JPRB ,&
&  0.55853E+00_JPRB, 0.55516E+00_JPRB, 0.55428E+00_JPRB, 0.55696E+00_JPRB, 0.55961E+00_JPRB, 0.55770E+00_JPRB, 0.55909E+00_JPRB ,&
&  0.56238E+00_JPRB, 0.56697E+00_JPRB, 0.58387E+00_JPRB, 0.60827E+00_JPRB, 0.62972E+00_JPRB, 0.64583E+00_JPRB, 0.66078E+00_JPRB ,&
&  0.68292E+00_JPRB, 0.69656E+00_JPRB, 0.68433E+00_JPRB, 0.67297E+00_JPRB, 0.66764E+00_JPRB, 0.67348E+00_JPRB, 0.68429E+00_JPRB ,&
&  0.70464E+00_JPRB, 0.71697E+00_JPRB, 0.72315E+00_JPRB, 0.72445E+00_JPRB, 0.72736E+00_JPRB, 0.72805E+00_JPRB, 0.74083E+00_JPRB ,&
&  0.77504E+00_JPRB, 0.80438E+00_JPRB, 0.81131E+00_JPRB, 0.84343E+00_JPRB, 0.88477E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.95599E+00_JPRB, 0.95959E+00_JPRB, 0.94917E+00_JPRB, 0.94441E+00_JPRB, 0.93356E+00_JPRB, 0.91966E+00_JPRB, 0.89741E+00_JPRB ,&
&  0.87141E+00_JPRB, 0.84338E+00_JPRB, 0.81638E+00_JPRB, 0.79405E+00_JPRB, 0.79723E+00_JPRB, 0.82915E+00_JPRB, 0.85632E+00_JPRB ,&
&  0.84547E+00_JPRB, 0.84152E+00_JPRB, 0.85823E+00_JPRB, 0.88834E+00_JPRB, 0.92092E+00_JPRB, 0.95088E+00_JPRB, 0.97177E+00_JPRB ,&
&  0.98835E+00_JPRB, 0.97006E+00_JPRB, 0.97512E+00_JPRB, 0.97866E+00_JPRB, 0.97372E+00_JPRB, 0.95634E+00_JPRB, 0.92787E+00_JPRB ,&
&  0.89977E+00_JPRB, 0.87248E+00_JPRB, 0.83898E+00_JPRB, 0.78219E+00_JPRB, 0.85590E+00_JPRB, 0.78310E+00_JPRB, 0.75578E+00_JPRB ,&
&  0.72825E+00_JPRB, 0.70937E+00_JPRB, 0.68753E+00_JPRB, 0.66464E+00_JPRB, 0.65687E+00_JPRB, 0.65231E+00_JPRB, 0.64411E+00_JPRB ,&
&  0.63700E+00_JPRB, 0.62864E+00_JPRB, 0.61616E+00_JPRB, 0.60111E+00_JPRB, 0.58734E+00_JPRB, 0.57301E+00_JPRB, 0.55980E+00_JPRB ,&
&  0.54018E+00_JPRB, 0.51136E+00_JPRB, 0.48103E+00_JPRB, 0.45839E+00_JPRB, 0.44626E+00_JPRB, 0.45587E+00_JPRB, 0.49030E+00_JPRB ,&
&  0.52547E+00_JPRB, 0.53176E+00_JPRB, 0.54222E+00_JPRB, 0.54635E+00_JPRB, 0.53854E+00_JPRB, 0.52809E+00_JPRB, 0.51041E+00_JPRB ,&
&  0.49778E+00_JPRB, 0.49685E+00_JPRB, 0.49907E+00_JPRB, 0.50436E+00_JPRB, 0.50930E+00_JPRB, 0.51582E+00_JPRB, 0.52487E+00_JPRB ,&
&  0.53296E+00_JPRB, 0.53580E+00_JPRB, 0.54815E+00_JPRB, 0.57496E+00_JPRB, 0.60656E+00_JPRB, 0.63904E+00_JPRB, 0.66742E+00_JPRB ,&
&  0.68736E+00_JPRB, 0.70172E+00_JPRB, 0.71055E+00_JPRB, 0.72587E+00_JPRB, 0.74168E+00_JPRB, 0.75712E+00_JPRB, 0.76486E+00_JPRB ,&
&  0.76194E+00_JPRB, 0.76609E+00_JPRB, 0.78448E+00_JPRB, 0.81353E+00_JPRB, 0.83347E+00_JPRB, 0.86737E+00_JPRB, 0.88862E+00_JPRB ,&
&  0.91089E+00_JPRB, 0.91641E+00_JPRB, 0.91078E+00_JPRB, 0.92304E+00_JPRB, 0.93930E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.93843E+00_JPRB, 0.94868E+00_JPRB, 0.94630E+00_JPRB, 0.94492E+00_JPRB, 0.94623E+00_JPRB, 0.93888E+00_JPRB, 0.92514E+00_JPRB ,&
&  0.91121E+00_JPRB, 0.88449E+00_JPRB, 0.85327E+00_JPRB, 0.83879E+00_JPRB, 0.84525E+00_JPRB, 0.84435E+00_JPRB, 0.82564E+00_JPRB ,&
&  0.82393E+00_JPRB, 0.83731E+00_JPRB, 0.85983E+00_JPRB, 0.88484E+00_JPRB, 0.91842E+00_JPRB, 0.95329E+00_JPRB, 0.97912E+00_JPRB ,&
&  0.98952E+00_JPRB, 0.98840E+00_JPRB, 0.98136E+00_JPRB, 0.95489E+00_JPRB, 0.91920E+00_JPRB, 0.89212E+00_JPRB, 0.87148E+00_JPRB ,&
&  0.84551E+00_JPRB, 0.79559E+00_JPRB, 0.72785E+00_JPRB, 0.72234E+00_JPRB, 0.66184E+00_JPRB, 0.64461E+00_JPRB, 0.66779E+00_JPRB ,&
&  0.68332E+00_JPRB, 0.66550E+00_JPRB, 0.64083E+00_JPRB, 0.62029E+00_JPRB, 0.61578E+00_JPRB, 0.61377E+00_JPRB, 0.60801E+00_JPRB ,&
&  0.60351E+00_JPRB, 0.59623E+00_JPRB, 0.58161E+00_JPRB, 0.56837E+00_JPRB, 0.55585E+00_JPRB, 0.54332E+00_JPRB, 0.53429E+00_JPRB ,&
&  0.50446E+00_JPRB, 0.45949E+00_JPRB, 0.42585E+00_JPRB, 0.41054E+00_JPRB, 0.40198E+00_JPRB, 0.40599E+00_JPRB, 0.41998E+00_JPRB ,&
&  0.43206E+00_JPRB, 0.43413E+00_JPRB, 0.42936E+00_JPRB, 0.41598E+00_JPRB, 0.39290E+00_JPRB, 0.35125E+00_JPRB, 0.32638E+00_JPRB ,&
&  0.34424E+00_JPRB, 0.37519E+00_JPRB, 0.40795E+00_JPRB, 0.42257E+00_JPRB, 0.43147E+00_JPRB, 0.43229E+00_JPRB, 0.41593E+00_JPRB ,&
&  0.38546E+00_JPRB, 0.36026E+00_JPRB, 0.35671E+00_JPRB, 0.35898E+00_JPRB, 0.36554E+00_JPRB, 0.37674E+00_JPRB, 0.39356E+00_JPRB ,&
&  0.41883E+00_JPRB, 0.45550E+00_JPRB, 0.47822E+00_JPRB, 0.48093E+00_JPRB, 0.47354E+00_JPRB, 0.46263E+00_JPRB, 0.45985E+00_JPRB ,&
&  0.47481E+00_JPRB, 0.53432E+00_JPRB, 0.65450E+00_JPRB, 0.79807E+00_JPRB, 0.80190E+00_JPRB, 0.84699E+00_JPRB, 0.88162E+00_JPRB ,&
&  0.91172E+00_JPRB, 0.92547E+00_JPRB, 0.91245E+00_JPRB, 0.91298E+00_JPRB, 0.91887E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.89495E+00_JPRB, 0.91475E+00_JPRB, 0.92540E+00_JPRB, 0.93912E+00_JPRB, 0.93921E+00_JPRB, 0.90905E+00_JPRB, 0.89470E+00_JPRB ,&
&  0.89917E+00_JPRB, 0.89374E+00_JPRB, 0.88185E+00_JPRB, 0.87399E+00_JPRB, 0.86455E+00_JPRB, 0.85463E+00_JPRB, 0.84663E+00_JPRB ,&
&  0.83833E+00_JPRB, 0.83291E+00_JPRB, 0.83934E+00_JPRB, 0.85272E+00_JPRB, 0.86357E+00_JPRB, 0.86932E+00_JPRB, 0.87433E+00_JPRB ,&
&  0.87409E+00_JPRB, 0.84142E+00_JPRB, 0.79596E+00_JPRB, 0.76277E+00_JPRB, 0.74681E+00_JPRB, 0.73333E+00_JPRB, 0.69953E+00_JPRB ,&
&  0.62847E+00_JPRB, 0.50751E+00_JPRB, 0.45414E+00_JPRB, 0.49709E+00_JPRB, 0.61048E+00_JPRB, 0.57887E+00_JPRB, 0.60074E+00_JPRB ,&
&  0.63215E+00_JPRB, 0.64856E+00_JPRB, 0.64436E+00_JPRB, 0.63364E+00_JPRB, 0.62976E+00_JPRB, 0.63318E+00_JPRB, 0.63010E+00_JPRB ,&
&  0.62691E+00_JPRB, 0.61097E+00_JPRB, 0.60123E+00_JPRB, 0.58886E+00_JPRB, 0.56999E+00_JPRB, 0.54357E+00_JPRB, 0.52692E+00_JPRB ,&
&  0.48682E+00_JPRB, 0.44751E+00_JPRB, 0.43524E+00_JPRB, 0.43443E+00_JPRB, 0.40310E+00_JPRB, 0.39036E+00_JPRB, 0.37442E+00_JPRB ,&
&  0.33700E+00_JPRB, 0.31622E+00_JPRB, 0.28577E+00_JPRB, 0.26701E+00_JPRB, 0.25409E+00_JPRB, 0.23902E+00_JPRB, 0.23674E+00_JPRB ,&
&  0.24284E+00_JPRB, 0.25708E+00_JPRB, 0.27757E+00_JPRB, 0.30746E+00_JPRB, 0.33339E+00_JPRB, 0.29867E+00_JPRB, 0.28335E+00_JPRB ,&
&  0.26824E+00_JPRB, 0.26472E+00_JPRB, 0.27193E+00_JPRB, 0.28000E+00_JPRB, 0.28788E+00_JPRB, 0.29706E+00_JPRB, 0.30644E+00_JPRB ,&
&  0.31329E+00_JPRB, 0.31633E+00_JPRB, 0.31652E+00_JPRB, 0.31557E+00_JPRB, 0.31363E+00_JPRB, 0.31380E+00_JPRB, 0.31709E+00_JPRB ,&
&  0.32427E+00_JPRB, 0.33730E+00_JPRB, 0.36151E+00_JPRB, 0.39722E+00_JPRB, 0.44298E+00_JPRB, 0.50359E+00_JPRB, 0.59496E+00_JPRB ,&
&  0.71904E+00_JPRB, 0.88182E+00_JPRB, 0.92946E+00_JPRB, 0.93881E+00_JPRB, 0.90686E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.89805E+00_JPRB, 0.89480E+00_JPRB, 0.90137E+00_JPRB, 0.89099E+00_JPRB, 0.87206E+00_JPRB, 0.86324E+00_JPRB, 0.88024E+00_JPRB ,&
&  0.89987E+00_JPRB, 0.86994E+00_JPRB, 0.84773E+00_JPRB, 0.82827E+00_JPRB, 0.81380E+00_JPRB, 0.79946E+00_JPRB, 0.77770E+00_JPRB ,&
&  0.75759E+00_JPRB, 0.74998E+00_JPRB, 0.74768E+00_JPRB, 0.74237E+00_JPRB, 0.73200E+00_JPRB, 0.72193E+00_JPRB, 0.70433E+00_JPRB ,&
&  0.67061E+00_JPRB, 0.63494E+00_JPRB, 0.61675E+00_JPRB, 0.59290E+00_JPRB, 0.57620E+00_JPRB, 0.55676E+00_JPRB, 0.52918E+00_JPRB ,&
&  0.48283E+00_JPRB, 0.42498E+00_JPRB, 0.39257E+00_JPRB, 0.43118E+00_JPRB, 0.52249E+00_JPRB, 0.51485E+00_JPRB, 0.53591E+00_JPRB ,&
&  0.57138E+00_JPRB, 0.59277E+00_JPRB, 0.60366E+00_JPRB, 0.60722E+00_JPRB, 0.61664E+00_JPRB, 0.62321E+00_JPRB, 0.61962E+00_JPRB ,&
&  0.60372E+00_JPRB, 0.55111E+00_JPRB, 0.50657E+00_JPRB, 0.40123E+00_JPRB, 0.33143E+00_JPRB, 0.29429E+00_JPRB, 0.26345E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.23131E+00_JPRB, 0.22888E+00_JPRB, 0.22668E+00_JPRB, 0.23247E+00_JPRB, 0.24041E+00_JPRB, 0.24088E+00_JPRB ,&
&  0.23620E+00_JPRB, 0.22727E+00_JPRB, 0.21554E+00_JPRB, 0.20925E+00_JPRB, 0.20571E+00_JPRB, 0.20604E+00_JPRB, 0.20832E+00_JPRB ,&
&  0.20923E+00_JPRB, 0.21211E+00_JPRB, 0.22484E+00_JPRB, 0.23523E+00_JPRB, 0.24882E+00_JPRB, 0.23323E+00_JPRB, 0.21946E+00_JPRB ,&
&  0.21377E+00_JPRB, 0.21952E+00_JPRB, 0.22779E+00_JPRB, 0.23385E+00_JPRB, 0.24016E+00_JPRB, 0.24513E+00_JPRB, 0.24997E+00_JPRB ,&
&  0.25529E+00_JPRB, 0.25917E+00_JPRB, 0.26078E+00_JPRB, 0.26255E+00_JPRB, 0.26408E+00_JPRB, 0.26651E+00_JPRB, 0.26932E+00_JPRB ,&
&  0.27414E+00_JPRB, 0.28408E+00_JPRB, 0.29687E+00_JPRB, 0.31814E+00_JPRB, 0.33811E+00_JPRB, 0.35780E+00_JPRB, 0.38144E+00_JPRB ,&
&  0.40146E+00_JPRB, 0.48314E+00_JPRB, 0.67558E+00_JPRB, 0.99275E+00_JPRB, 0.98637E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.84286E+00_JPRB, 0.85478E+00_JPRB, 0.87121E+00_JPRB, 0.84777E+00_JPRB, 0.83272E+00_JPRB, 0.83593E+00_JPRB, 0.83660E+00_JPRB ,&
&  0.81295E+00_JPRB, 0.77653E+00_JPRB, 0.72625E+00_JPRB, 0.68366E+00_JPRB, 0.62007E+00_JPRB, 0.53790E+00_JPRB, 0.49987E+00_JPRB ,&
&  0.46451E+00_JPRB, 0.46041E+00_JPRB, 0.46205E+00_JPRB, 0.47808E+00_JPRB, 0.48649E+00_JPRB, 0.48295E+00_JPRB, 0.46038E+00_JPRB ,&
&  0.43302E+00_JPRB, 0.40643E+00_JPRB, 0.38872E+00_JPRB, 0.37535E+00_JPRB, 0.37538E+00_JPRB, 0.37337E+00_JPRB, 0.36682E+00_JPRB ,&
&  0.35928E+00_JPRB, 0.36487E+00_JPRB, 0.38305E+00_JPRB, 0.40982E+00_JPRB, 0.45656E+00_JPRB, 0.43627E+00_JPRB, 0.44526E+00_JPRB ,&
&  0.47610E+00_JPRB, 0.49835E+00_JPRB, 0.50725E+00_JPRB, 0.51053E+00_JPRB, 0.50282E+00_JPRB, 0.52457E+00_JPRB, 0.46784E+00_JPRB ,&
&  0.36171E+00_JPRB, 0.30238E+00_JPRB, 0.25969E+00_JPRB, 0.23098E+00_JPRB, 0.21071E+00_JPRB, 0.19854E+00_JPRB, 0.18464E+00_JPRB ,&
&  0.17799E+00_JPRB, 0.17280E+00_JPRB, 0.17186E+00_JPRB, 0.17446E+00_JPRB, 0.17790E+00_JPRB, 0.18225E+00_JPRB, 0.18390E+00_JPRB ,&
&  0.18651E+00_JPRB, 0.18668E+00_JPRB, 0.18605E+00_JPRB, 0.18623E+00_JPRB, 0.18512E+00_JPRB, 0.18591E+00_JPRB, 0.18575E+00_JPRB ,&
&  0.18522E+00_JPRB, 0.18458E+00_JPRB, 0.18637E+00_JPRB, 0.18763E+00_JPRB, 0.18853E+00_JPRB, 0.18543E+00_JPRB, 0.18430E+00_JPRB ,&
&  0.18514E+00_JPRB, 0.18785E+00_JPRB, 0.19188E+00_JPRB, 0.19631E+00_JPRB, 0.20243E+00_JPRB, 0.20704E+00_JPRB, 0.21243E+00_JPRB ,&
&  0.21703E+00_JPRB, 0.22164E+00_JPRB, 0.22610E+00_JPRB, 0.23067E+00_JPRB, 0.23573E+00_JPRB, 0.24076E+00_JPRB, 0.24590E+00_JPRB ,&
&  0.25136E+00_JPRB, 0.26032E+00_JPRB, 0.27120E+00_JPRB, 0.29006E+00_JPRB, 0.31528E+00_JPRB, 0.34054E+00_JPRB, 0.38589E+00_JPRB ,&
&  0.44130E+00_JPRB, 0.66784E+00_JPRB, 0.97304E+00_JPRB, 0.80896E+00_JPRB, 0.82598E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.83156E+00_JPRB, 0.84968E+00_JPRB, 0.86518E+00_JPRB, 0.86241E+00_JPRB, 0.84496E+00_JPRB, 0.78738E+00_JPRB, 0.72391E+00_JPRB ,&
&  0.64528E+00_JPRB, 0.57991E+00_JPRB, 0.51027E+00_JPRB, 0.45056E+00_JPRB, 0.39992E+00_JPRB, 0.36068E+00_JPRB, 0.33572E+00_JPRB ,&
&  0.31762E+00_JPRB, 0.30957E+00_JPRB, 0.30558E+00_JPRB, 0.31000E+00_JPRB, 0.31175E+00_JPRB, 0.31715E+00_JPRB, 0.31140E+00_JPRB ,&
&  0.30430E+00_JPRB, 0.29681E+00_JPRB, 0.29463E+00_JPRB, 0.30315E+00_JPRB, 0.31684E+00_JPRB, 0.33001E+00_JPRB, 0.34732E+00_JPRB ,&
&  0.36568E+00_JPRB, 0.37158E+00_JPRB, 0.38663E+00_JPRB, 0.39014E+00_JPRB, 0.37603E+00_JPRB, 0.37819E+00_JPRB, 0.38429E+00_JPRB ,&
&  0.39778E+00_JPRB, 0.41636E+00_JPRB, 0.42694E+00_JPRB, 0.44632E+00_JPRB, 0.39104E+00_JPRB, 0.33106E+00_JPRB, 0.27824E+00_JPRB ,&
&  0.23869E+00_JPRB, 0.21576E+00_JPRB, 0.19870E+00_JPRB, 0.19008E+00_JPRB, 0.18107E+00_JPRB, 0.17619E+00_JPRB, 0.17118E+00_JPRB ,&
&  0.16901E+00_JPRB, 0.16929E+00_JPRB, 0.16908E+00_JPRB, 0.16919E+00_JPRB, 0.17014E+00_JPRB, 0.17148E+00_JPRB, 0.17122E+00_JPRB ,&
&  0.17081E+00_JPRB, 0.17074E+00_JPRB, 0.17038E+00_JPRB, 0.17002E+00_JPRB, 0.16911E+00_JPRB, 0.16899E+00_JPRB, 0.16693E+00_JPRB ,&
&  0.16585E+00_JPRB, 0.16416E+00_JPRB, 0.16323E+00_JPRB, 0.16332E+00_JPRB, 0.16393E+00_JPRB, 0.16450E+00_JPRB, 0.16545E+00_JPRB ,&
&  0.16712E+00_JPRB, 0.16860E+00_JPRB, 0.17104E+00_JPRB, 0.17318E+00_JPRB, 0.17671E+00_JPRB, 0.17964E+00_JPRB, 0.18372E+00_JPRB ,&
&  0.18809E+00_JPRB, 0.19337E+00_JPRB, 0.19962E+00_JPRB, 0.20672E+00_JPRB, 0.21247E+00_JPRB, 0.21802E+00_JPRB, 0.22395E+00_JPRB ,&
&  0.22849E+00_JPRB, 0.23671E+00_JPRB, 0.24585E+00_JPRB, 0.26118E+00_JPRB, 0.29336E+00_JPRB, 0.31613E+00_JPRB, 0.36365E+00_JPRB ,&
&  0.43993E+00_JPRB, 0.69029E+00_JPRB, 0.92471E+00_JPRB, 0.78154E+00_JPRB, 0.81325E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.70846E+00_JPRB, 0.73955E+00_JPRB, 0.76288E+00_JPRB, 0.76438E+00_JPRB, 0.76239E+00_JPRB, 0.72467E+00_JPRB, 0.66582E+00_JPRB ,&
&  0.56013E+00_JPRB, 0.49087E+00_JPRB, 0.44615E+00_JPRB, 0.40387E+00_JPRB, 0.36583E+00_JPRB, 0.33953E+00_JPRB, 0.32187E+00_JPRB ,&
&  0.30858E+00_JPRB, 0.30120E+00_JPRB, 0.29566E+00_JPRB, 0.29103E+00_JPRB, 0.28588E+00_JPRB, 0.28095E+00_JPRB, 0.27235E+00_JPRB ,&
&  0.26694E+00_JPRB, 0.26389E+00_JPRB, 0.26216E+00_JPRB, 0.26454E+00_JPRB, 0.27147E+00_JPRB, 0.27764E+00_JPRB, 0.28792E+00_JPRB ,&
&  0.29901E+00_JPRB, 0.31355E+00_JPRB, 0.32388E+00_JPRB, 0.32794E+00_JPRB, 0.33126E+00_JPRB, 0.33016E+00_JPRB, 0.30864E+00_JPRB ,&
&  0.31334E+00_JPRB, 0.33402E+00_JPRB, 0.35211E+00_JPRB, 0.35959E+00_JPRB, 0.31301E+00_JPRB, 0.27163E+00_JPRB, 0.23077E+00_JPRB ,&
&  0.21217E+00_JPRB, 0.19596E+00_JPRB, 0.18720E+00_JPRB, 0.18032E+00_JPRB, 0.17465E+00_JPRB, 0.17195E+00_JPRB, 0.16960E+00_JPRB ,&
&  0.16779E+00_JPRB, 0.16744E+00_JPRB, 0.16631E+00_JPRB, 0.16618E+00_JPRB, 0.16552E+00_JPRB, 0.16570E+00_JPRB, 0.16458E+00_JPRB ,&
&  0.16339E+00_JPRB, 0.16239E+00_JPRB, 0.16174E+00_JPRB, 0.16141E+00_JPRB, 0.16040E+00_JPRB, 0.15939E+00_JPRB, 0.15700E+00_JPRB ,&
&  0.15540E+00_JPRB, 0.15386E+00_JPRB, 0.15276E+00_JPRB, 0.15307E+00_JPRB, 0.15395E+00_JPRB, 0.15651E+00_JPRB, 0.15791E+00_JPRB ,&
&  0.15817E+00_JPRB, 0.15826E+00_JPRB, 0.15819E+00_JPRB, 0.15906E+00_JPRB, 0.16033E+00_JPRB, 0.16255E+00_JPRB, 0.16567E+00_JPRB ,&
&  0.16944E+00_JPRB, 0.17531E+00_JPRB, 0.18101E+00_JPRB, 0.18721E+00_JPRB, 0.19283E+00_JPRB, 0.19797E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.21231E+00_JPRB, 0.22116E+00_JPRB, 0.23463E+00_JPRB, 0.24853E+00_JPRB, 0.27391E+00_JPRB, 0.29326E+00_JPRB, 0.32256E+00_JPRB ,&
&  0.39057E+00_JPRB, 0.53484E+00_JPRB, 0.62129E+00_JPRB, 0.63624E+00_JPRB, 0.67015E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.52312E+00_JPRB, 0.53498E+00_JPRB, 0.55019E+00_JPRB, 0.55941E+00_JPRB, 0.57034E+00_JPRB, 0.53798E+00_JPRB, 0.51234E+00_JPRB ,&
&  0.45942E+00_JPRB, 0.42885E+00_JPRB, 0.40336E+00_JPRB, 0.37996E+00_JPRB, 0.35904E+00_JPRB, 0.33960E+00_JPRB, 0.32214E+00_JPRB ,&
&  0.31101E+00_JPRB, 0.30451E+00_JPRB, 0.29578E+00_JPRB, 0.28564E+00_JPRB, 0.27470E+00_JPRB, 0.26512E+00_JPRB, 0.26064E+00_JPRB ,&
&  0.25569E+00_JPRB, 0.25220E+00_JPRB, 0.25073E+00_JPRB, 0.25029E+00_JPRB, 0.25196E+00_JPRB, 0.25175E+00_JPRB, 0.25204E+00_JPRB ,&
&  0.24789E+00_JPRB, 0.25533E+00_JPRB, 0.26530E+00_JPRB, 0.26988E+00_JPRB, 0.27871E+00_JPRB, 0.27772E+00_JPRB, 0.27319E+00_JPRB ,&
&  0.26543E+00_JPRB, 0.25684E+00_JPRB, 0.24671E+00_JPRB, 0.23416E+00_JPRB, 0.22380E+00_JPRB, 0.21642E+00_JPRB, 0.20901E+00_JPRB ,&
&  0.20476E+00_JPRB, 0.19899E+00_JPRB, 0.19162E+00_JPRB, 0.18511E+00_JPRB, 0.17946E+00_JPRB, 0.17498E+00_JPRB, 0.17080E+00_JPRB ,&
&  0.16801E+00_JPRB, 0.16656E+00_JPRB, 0.16518E+00_JPRB, 0.16522E+00_JPRB, 0.16428E+00_JPRB, 0.16320E+00_JPRB, 0.16208E+00_JPRB ,&
&  0.16155E+00_JPRB, 0.16019E+00_JPRB, 0.15905E+00_JPRB, 0.15803E+00_JPRB, 0.15642E+00_JPRB, 0.15503E+00_JPRB, 0.15345E+00_JPRB ,&
&  0.15213E+00_JPRB, 0.15154E+00_JPRB, 0.15120E+00_JPRB, 0.15122E+00_JPRB, 0.15166E+00_JPRB, 0.15166E+00_JPRB, 0.15205E+00_JPRB ,&
&  0.15248E+00_JPRB, 0.15309E+00_JPRB, 0.15306E+00_JPRB, 0.15350E+00_JPRB, 0.15455E+00_JPRB, 0.15598E+00_JPRB, 0.15950E+00_JPRB ,&
&  0.16347E+00_JPRB, 0.16850E+00_JPRB, 0.17385E+00_JPRB, 0.17965E+00_JPRB, 0.18542E+00_JPRB, 0.19094E+00_JPRB, 0.19708E+00_JPRB ,&
&  0.20507E+00_JPRB, 0.21237E+00_JPRB, 0.22115E+00_JPRB, 0.23471E+00_JPRB, 0.26576E+00_JPRB, 0.28159E+00_JPRB, 0.29514E+00_JPRB ,&
&  0.31940E+00_JPRB, 0.35004E+00_JPRB, 0.45396E+00_JPRB, 0.48592E+00_JPRB, 0.50335E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.26823E+00_JPRB, 0.26776E+00_JPRB, 0.26777E+00_JPRB, 0.27326E+00_JPRB, 0.28042E+00_JPRB, 0.35815E+00_JPRB, 0.34817E+00_JPRB ,&
&  0.34012E+00_JPRB, 0.31263E+00_JPRB, 0.30978E+00_JPRB, 0.30316E+00_JPRB, 0.29395E+00_JPRB, 0.28704E+00_JPRB, 0.28322E+00_JPRB ,&
&  0.27885E+00_JPRB, 0.27384E+00_JPRB, 0.27151E+00_JPRB, 0.26857E+00_JPRB, 0.26558E+00_JPRB, 0.26194E+00_JPRB, 0.25908E+00_JPRB ,&
&  0.25616E+00_JPRB, 0.25407E+00_JPRB, 0.25185E+00_JPRB, 0.24712E+00_JPRB, 0.24514E+00_JPRB, 0.24146E+00_JPRB, 0.23606E+00_JPRB ,&
&  0.22893E+00_JPRB, 0.22538E+00_JPRB, 0.22147E+00_JPRB, 0.21805E+00_JPRB, 0.22194E+00_JPRB, 0.22139E+00_JPRB, 0.21947E+00_JPRB ,&
&  0.21578E+00_JPRB, 0.21197E+00_JPRB, 0.20689E+00_JPRB, 0.20198E+00_JPRB, 0.19802E+00_JPRB, 0.19479E+00_JPRB, 0.19207E+00_JPRB ,&
&  0.18995E+00_JPRB, 0.18819E+00_JPRB, 0.18548E+00_JPRB, 0.18303E+00_JPRB, 0.18042E+00_JPRB, 0.17815E+00_JPRB, 0.17569E+00_JPRB ,&
&  0.17378E+00_JPRB, 0.17199E+00_JPRB, 0.17012E+00_JPRB, 0.16823E+00_JPRB, 0.16674E+00_JPRB, 0.16478E+00_JPRB, 0.16332E+00_JPRB ,&
&  0.16207E+00_JPRB, 0.16041E+00_JPRB, 0.15918E+00_JPRB, 0.15769E+00_JPRB, 0.15659E+00_JPRB, 0.15567E+00_JPRB, 0.15478E+00_JPRB ,&
&  0.15360E+00_JPRB, 0.15281E+00_JPRB, 0.15254E+00_JPRB, 0.15235E+00_JPRB, 0.15205E+00_JPRB, 0.15216E+00_JPRB, 0.15259E+00_JPRB ,&
&  0.15318E+00_JPRB, 0.15389E+00_JPRB, 0.15499E+00_JPRB, 0.15623E+00_JPRB, 0.15807E+00_JPRB, 0.16054E+00_JPRB, 0.16440E+00_JPRB ,&
&  0.16855E+00_JPRB, 0.17310E+00_JPRB, 0.17760E+00_JPRB, 0.18224E+00_JPRB, 0.18707E+00_JPRB, 0.19186E+00_JPRB, 0.19740E+00_JPRB ,&
&  0.20338E+00_JPRB, 0.20815E+00_JPRB, 0.21276E+00_JPRB, 0.21769E+00_JPRB, 0.22875E+00_JPRB, 0.23685E+00_JPRB, 0.24281E+00_JPRB ,&
&  0.24952E+00_JPRB, 0.25891E+00_JPRB, 0.26338E+00_JPRB, 0.26608E+00_JPRB, 0.26732E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.23902E+00_JPRB, 0.24204E+00_JPRB, 0.24409E+00_JPRB, 0.24538E+00_JPRB, 0.24503E+00_JPRB, 0.24595E+00_JPRB, 0.24686E+00_JPRB ,&
&  0.24810E+00_JPRB, 0.24974E+00_JPRB, 0.25096E+00_JPRB, 0.25203E+00_JPRB, 0.25250E+00_JPRB, 0.25253E+00_JPRB, 0.25257E+00_JPRB ,&
&  0.25233E+00_JPRB, 0.25206E+00_JPRB, 0.24764E+00_JPRB, 0.24368E+00_JPRB, 0.24075E+00_JPRB, 0.23896E+00_JPRB, 0.23746E+00_JPRB ,&
&  0.23627E+00_JPRB, 0.23488E+00_JPRB, 0.23338E+00_JPRB, 0.23126E+00_JPRB, 0.22783E+00_JPRB, 0.22446E+00_JPRB, 0.22097E+00_JPRB ,&
&  0.21781E+00_JPRB, 0.21536E+00_JPRB, 0.21308E+00_JPRB, 0.21084E+00_JPRB, 0.20964E+00_JPRB, 0.20816E+00_JPRB, 0.20660E+00_JPRB ,&
&  0.20449E+00_JPRB, 0.20217E+00_JPRB, 0.19998E+00_JPRB, 0.19773E+00_JPRB, 0.19557E+00_JPRB, 0.19344E+00_JPRB, 0.19187E+00_JPRB ,&
&  0.19026E+00_JPRB, 0.18907E+00_JPRB, 0.18775E+00_JPRB, 0.18655E+00_JPRB, 0.18528E+00_JPRB, 0.18407E+00_JPRB, 0.18258E+00_JPRB ,&
&  0.18156E+00_JPRB, 0.18049E+00_JPRB, 0.17941E+00_JPRB, 0.17841E+00_JPRB, 0.17742E+00_JPRB, 0.17645E+00_JPRB, 0.17536E+00_JPRB ,&
&  0.17434E+00_JPRB, 0.17369E+00_JPRB, 0.17301E+00_JPRB, 0.17225E+00_JPRB, 0.17156E+00_JPRB, 0.17109E+00_JPRB, 0.17069E+00_JPRB ,&
&  0.17033E+00_JPRB, 0.17032E+00_JPRB, 0.17052E+00_JPRB, 0.17071E+00_JPRB, 0.17092E+00_JPRB, 0.17136E+00_JPRB, 0.17202E+00_JPRB ,&
&  0.17279E+00_JPRB, 0.17362E+00_JPRB, 0.17467E+00_JPRB, 0.17616E+00_JPRB, 0.17782E+00_JPRB, 0.17971E+00_JPRB, 0.18177E+00_JPRB ,&
&  0.18399E+00_JPRB, 0.18635E+00_JPRB, 0.18880E+00_JPRB, 0.19108E+00_JPRB, 0.19364E+00_JPRB, 0.19614E+00_JPRB, 0.19858E+00_JPRB ,&
&  0.20115E+00_JPRB, 0.20373E+00_JPRB, 0.20647E+00_JPRB, 0.20900E+00_JPRB, 0.21309E+00_JPRB, 0.21577E+00_JPRB, 0.21850E+00_JPRB ,&
&  0.22093E+00_JPRB, 0.22376E+00_JPRB, 0.22662E+00_JPRB, 0.23012E+00_JPRB, 0.23362E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB, 0.20025E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB, 0.59417E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.63700E+00_JPRB, 0.63478E+00_JPRB, 0.63227E+00_JPRB, 0.63003E+00_JPRB, 0.62729E+00_JPRB, 0.62499E+00_JPRB, 0.62264E+00_JPRB ,&
&  0.62046E+00_JPRB, 0.61804E+00_JPRB, 0.61569E+00_JPRB, 0.61307E+00_JPRB, 0.61056E+00_JPRB, 0.60813E+00_JPRB, 0.60583E+00_JPRB ,&
&  0.60351E+00_JPRB, 0.60134E+00_JPRB, 0.59912E+00_JPRB, 0.59668E+00_JPRB, 0.59458E+00_JPRB, 0.59216E+00_JPRB, 0.58914E+00_JPRB ,&
&  0.58458E+00_JPRB, 0.57978E+00_JPRB, 0.57551E+00_JPRB, 0.57250E+00_JPRB, 0.57017E+00_JPRB, 0.56822E+00_JPRB, 0.56648E+00_JPRB ,&
&  0.56521E+00_JPRB, 0.56441E+00_JPRB, 0.56384E+00_JPRB, 0.56302E+00_JPRB, 0.56274E+00_JPRB, 0.56361E+00_JPRB, 0.56398E+00_JPRB ,&
&  0.56398E+00_JPRB, 0.56405E+00_JPRB, 0.56491E+00_JPRB, 0.56556E+00_JPRB, 0.56617E+00_JPRB, 0.56705E+00_JPRB, 0.56888E+00_JPRB ,&
&  0.57069E+00_JPRB, 0.57232E+00_JPRB, 0.57422E+00_JPRB, 0.57626E+00_JPRB, 0.57833E+00_JPRB, 0.58042E+00_JPRB, 0.58325E+00_JPRB ,&
&  0.58642E+00_JPRB, 0.58922E+00_JPRB, 0.59170E+00_JPRB, 0.59383E+00_JPRB, 0.59566E+00_JPRB, 0.59670E+00_JPRB, 0.59743E+00_JPRB ,&
&  0.59753E+00_JPRB, 0.59852E+00_JPRB, 0.59884E+00_JPRB, 0.59925E+00_JPRB, 0.59910E+00_JPRB, 0.60013E+00_JPRB, 0.60079E+00_JPRB ,&
&  0.60234E+00_JPRB, 0.60337E+00_JPRB, 0.60518E+00_JPRB, 0.60692E+00_JPRB, 0.60959E+00_JPRB, 0.61258E+00_JPRB, 0.61586E+00_JPRB ,&
&  0.61947E+00_JPRB, 0.62351E+00_JPRB, 0.62810E+00_JPRB, 0.63096E+00_JPRB, 0.63380E+00_JPRB, 0.63664E+00_JPRB, 0.63965E+00_JPRB ,&
&  0.64193E+00_JPRB, 0.64408E+00_JPRB, 0.64631E+00_JPRB, 0.64811E+00_JPRB, 0.64917E+00_JPRB, 0.65000E+00_JPRB, 0.65082E+00_JPRB ,&
&  0.65121E+00_JPRB, 0.65151E+00_JPRB, 0.65161E+00_JPRB, 0.65177E+00_JPRB, 0.65099E+00_JPRB, 0.64968E+00_JPRB, 0.64813E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64498E+00_JPRB, 0.64318E+00_JPRB, 0.64124E+00_JPRB, 0.63936E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.71583E+00_JPRB, 0.71206E+00_JPRB, 0.70810E+00_JPRB, 0.70452E+00_JPRB, 0.69929E+00_JPRB, 0.69401E+00_JPRB, 0.68855E+00_JPRB ,&
&  0.68281E+00_JPRB, 0.67556E+00_JPRB, 0.66763E+00_JPRB, 0.65920E+00_JPRB, 0.65073E+00_JPRB, 0.64309E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63081E+00_JPRB, 0.62482E+00_JPRB, 0.61963E+00_JPRB, 0.61301E+00_JPRB, 0.60636E+00_JPRB, 0.60042E+00_JPRB, 0.59621E+00_JPRB ,&
&  0.58971E+00_JPRB, 0.58226E+00_JPRB, 0.57437E+00_JPRB, 0.56566E+00_JPRB, 0.55755E+00_JPRB, 0.54951E+00_JPRB, 0.54157E+00_JPRB ,&
&  0.53406E+00_JPRB, 0.52828E+00_JPRB, 0.52318E+00_JPRB, 0.51804E+00_JPRB, 0.51305E+00_JPRB, 0.51151E+00_JPRB, 0.50985E+00_JPRB ,&
&  0.50835E+00_JPRB, 0.50792E+00_JPRB, 0.51121E+00_JPRB, 0.51415E+00_JPRB, 0.51725E+00_JPRB, 0.52047E+00_JPRB, 0.52600E+00_JPRB ,&
&  0.53114E+00_JPRB, 0.53722E+00_JPRB, 0.54504E+00_JPRB, 0.55278E+00_JPRB, 0.55905E+00_JPRB, 0.56367E+00_JPRB, 0.56988E+00_JPRB ,&
&  0.57421E+00_JPRB, 0.57997E+00_JPRB, 0.58398E+00_JPRB, 0.58585E+00_JPRB, 0.58551E+00_JPRB, 0.58537E+00_JPRB, 0.58628E+00_JPRB ,&
&  0.58650E+00_JPRB, 0.58973E+00_JPRB, 0.59298E+00_JPRB, 0.59641E+00_JPRB, 0.59809E+00_JPRB, 0.60279E+00_JPRB, 0.60656E+00_JPRB ,&
&  0.61295E+00_JPRB, 0.62178E+00_JPRB, 0.63084E+00_JPRB, 0.64072E+00_JPRB, 0.65276E+00_JPRB, 0.66655E+00_JPRB, 0.67777E+00_JPRB ,&
&  0.68838E+00_JPRB, 0.69754E+00_JPRB, 0.70518E+00_JPRB, 0.70966E+00_JPRB, 0.71387E+00_JPRB, 0.71793E+00_JPRB, 0.72226E+00_JPRB ,&
&  0.72435E+00_JPRB, 0.72682E+00_JPRB, 0.72945E+00_JPRB, 0.73154E+00_JPRB, 0.73134E+00_JPRB, 0.73096E+00_JPRB, 0.73131E+00_JPRB ,&
&  0.73158E+00_JPRB, 0.73144E+00_JPRB, 0.73149E+00_JPRB, 0.73221E+00_JPRB, 0.73222E+00_JPRB, 0.73145E+00_JPRB, 0.73055E+00_JPRB ,&
&  0.72975E+00_JPRB, 0.72768E+00_JPRB, 0.72489E+00_JPRB, 0.72207E+00_JPRB, 0.71959E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.75048E+00_JPRB, 0.74930E+00_JPRB, 0.74797E+00_JPRB, 0.74648E+00_JPRB, 0.74318E+00_JPRB, 0.73945E+00_JPRB, 0.73461E+00_JPRB ,&
&  0.72889E+00_JPRB, 0.72056E+00_JPRB, 0.71175E+00_JPRB, 0.70208E+00_JPRB, 0.69278E+00_JPRB, 0.68443E+00_JPRB, 0.67641E+00_JPRB ,&
&  0.66814E+00_JPRB, 0.65776E+00_JPRB, 0.64617E+00_JPRB, 0.63077E+00_JPRB, 0.61660E+00_JPRB, 0.60483E+00_JPRB, 0.59189E+00_JPRB ,&
&  0.57377E+00_JPRB, 0.55383E+00_JPRB, 0.53609E+00_JPRB, 0.51853E+00_JPRB, 0.50584E+00_JPRB, 0.47396E+00_JPRB, 0.46365E+00_JPRB ,&
&  0.45682E+00_JPRB, 0.44865E+00_JPRB, 0.44433E+00_JPRB, 0.44050E+00_JPRB, 0.43872E+00_JPRB, 0.43274E+00_JPRB, 0.42957E+00_JPRB ,&
&  0.41559E+00_JPRB, 0.41901E+00_JPRB, 0.42317E+00_JPRB, 0.42800E+00_JPRB, 0.43743E+00_JPRB, 0.44760E+00_JPRB, 0.46367E+00_JPRB ,&
&  0.49275E+00_JPRB, 0.50533E+00_JPRB, 0.52457E+00_JPRB, 0.53969E+00_JPRB, 0.54916E+00_JPRB, 0.55452E+00_JPRB, 0.56104E+00_JPRB ,&
&  0.56783E+00_JPRB, 0.57802E+00_JPRB, 0.58426E+00_JPRB, 0.58846E+00_JPRB, 0.58844E+00_JPRB, 0.58708E+00_JPRB, 0.58675E+00_JPRB ,&
&  0.58574E+00_JPRB, 0.58876E+00_JPRB, 0.59433E+00_JPRB, 0.60557E+00_JPRB, 0.62135E+00_JPRB, 0.64168E+00_JPRB, 0.66273E+00_JPRB ,&
&  0.68360E+00_JPRB, 0.70600E+00_JPRB, 0.72475E+00_JPRB, 0.74249E+00_JPRB, 0.76136E+00_JPRB, 0.77655E+00_JPRB, 0.78953E+00_JPRB ,&
&  0.79828E+00_JPRB, 0.80189E+00_JPRB, 0.79964E+00_JPRB, 0.79636E+00_JPRB, 0.79134E+00_JPRB, 0.78758E+00_JPRB, 0.78724E+00_JPRB ,&
&  0.78733E+00_JPRB, 0.78860E+00_JPRB, 0.79051E+00_JPRB, 0.78914E+00_JPRB, 0.78718E+00_JPRB, 0.78464E+00_JPRB, 0.78236E+00_JPRB ,&
&  0.77929E+00_JPRB, 0.77679E+00_JPRB, 0.77443E+00_JPRB, 0.77202E+00_JPRB, 0.76975E+00_JPRB, 0.76601E+00_JPRB, 0.76275E+00_JPRB ,&
&  0.75937E+00_JPRB, 0.75802E+00_JPRB, 0.75554E+00_JPRB, 0.75359E+00_JPRB, 0.75165E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.70909E+00_JPRB, 0.70902E+00_JPRB, 0.71022E+00_JPRB, 0.71066E+00_JPRB, 0.71248E+00_JPRB, 0.71355E+00_JPRB, 0.71523E+00_JPRB ,&
&  0.71601E+00_JPRB, 0.71667E+00_JPRB, 0.71565E+00_JPRB, 0.71354E+00_JPRB, 0.70980E+00_JPRB, 0.70577E+00_JPRB, 0.70106E+00_JPRB ,&
&  0.69472E+00_JPRB, 0.68253E+00_JPRB, 0.66569E+00_JPRB, 0.64656E+00_JPRB, 0.62444E+00_JPRB, 0.60044E+00_JPRB, 0.57351E+00_JPRB ,&
&  0.54657E+00_JPRB, 0.52184E+00_JPRB, 0.49170E+00_JPRB, 0.47935E+00_JPRB, 0.46027E+00_JPRB, 0.44519E+00_JPRB, 0.43371E+00_JPRB ,&
&  0.41952E+00_JPRB, 0.40965E+00_JPRB, 0.39943E+00_JPRB, 0.38978E+00_JPRB, 0.38493E+00_JPRB, 0.37584E+00_JPRB, 0.37210E+00_JPRB ,&
&  0.36953E+00_JPRB, 0.36833E+00_JPRB, 0.36885E+00_JPRB, 0.37140E+00_JPRB, 0.37726E+00_JPRB, 0.38461E+00_JPRB, 0.39839E+00_JPRB ,&
&  0.42895E+00_JPRB, 0.47178E+00_JPRB, 0.53714E+00_JPRB, 0.55028E+00_JPRB, 0.55052E+00_JPRB, 0.56069E+00_JPRB, 0.57211E+00_JPRB ,&
&  0.58336E+00_JPRB, 0.59385E+00_JPRB, 0.60257E+00_JPRB, 0.60672E+00_JPRB, 0.61176E+00_JPRB, 0.61613E+00_JPRB, 0.62102E+00_JPRB ,&
&  0.62616E+00_JPRB, 0.63809E+00_JPRB, 0.65528E+00_JPRB, 0.67589E+00_JPRB, 0.69834E+00_JPRB, 0.71790E+00_JPRB, 0.73504E+00_JPRB ,&
&  0.74898E+00_JPRB, 0.76380E+00_JPRB, 0.78433E+00_JPRB, 0.80597E+00_JPRB, 0.82834E+00_JPRB, 0.84088E+00_JPRB, 0.85167E+00_JPRB ,&
&  0.85825E+00_JPRB, 0.85876E+00_JPRB, 0.85190E+00_JPRB, 0.84310E+00_JPRB, 0.83656E+00_JPRB, 0.84131E+00_JPRB, 0.84020E+00_JPRB ,&
&  0.83938E+00_JPRB, 0.83269E+00_JPRB, 0.82181E+00_JPRB, 0.80901E+00_JPRB, 0.80032E+00_JPRB, 0.79262E+00_JPRB, 0.78805E+00_JPRB ,&
&  0.78310E+00_JPRB, 0.77755E+00_JPRB, 0.77019E+00_JPRB, 0.76050E+00_JPRB, 0.75029E+00_JPRB, 0.73924E+00_JPRB, 0.73046E+00_JPRB ,&
&  0.72255E+00_JPRB, 0.71779E+00_JPRB, 0.71365E+00_JPRB, 0.71142E+00_JPRB, 0.70927E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.68285E+00_JPRB, 0.68861E+00_JPRB, 0.69479E+00_JPRB, 0.69909E+00_JPRB, 0.70253E+00_JPRB, 0.70495E+00_JPRB, 0.70581E+00_JPRB ,&
&  0.70597E+00_JPRB, 0.70597E+00_JPRB, 0.70622E+00_JPRB, 0.70610E+00_JPRB, 0.70619E+00_JPRB, 0.70661E+00_JPRB, 0.70637E+00_JPRB ,&
&  0.70541E+00_JPRB, 0.69622E+00_JPRB, 0.67871E+00_JPRB, 0.65891E+00_JPRB, 0.63728E+00_JPRB, 0.60946E+00_JPRB, 0.57556E+00_JPRB ,&
&  0.54524E+00_JPRB, 0.52321E+00_JPRB, 0.50045E+00_JPRB, 0.48569E+00_JPRB, 0.46938E+00_JPRB, 0.45939E+00_JPRB, 0.44527E+00_JPRB ,&
&  0.43838E+00_JPRB, 0.40874E+00_JPRB, 0.40596E+00_JPRB, 0.38724E+00_JPRB, 0.37361E+00_JPRB, 0.36227E+00_JPRB, 0.35376E+00_JPRB ,&
&  0.35178E+00_JPRB, 0.34843E+00_JPRB, 0.34838E+00_JPRB, 0.35254E+00_JPRB, 0.35760E+00_JPRB, 0.36856E+00_JPRB, 0.38077E+00_JPRB ,&
&  0.40703E+00_JPRB, 0.49449E+00_JPRB, 0.52801E+00_JPRB, 0.53905E+00_JPRB, 0.54625E+00_JPRB, 0.56270E+00_JPRB, 0.57824E+00_JPRB ,&
&  0.59959E+00_JPRB, 0.61386E+00_JPRB, 0.62801E+00_JPRB, 0.63688E+00_JPRB, 0.65798E+00_JPRB, 0.67629E+00_JPRB, 0.69217E+00_JPRB ,&
&  0.70891E+00_JPRB, 0.72528E+00_JPRB, 0.74397E+00_JPRB, 0.75613E+00_JPRB, 0.75775E+00_JPRB, 0.75728E+00_JPRB, 0.75639E+00_JPRB ,&
&  0.76165E+00_JPRB, 0.76890E+00_JPRB, 0.79394E+00_JPRB, 0.82702E+00_JPRB, 0.86366E+00_JPRB, 0.88844E+00_JPRB, 0.90158E+00_JPRB ,&
&  0.91127E+00_JPRB, 0.91677E+00_JPRB, 0.92526E+00_JPRB, 0.93758E+00_JPRB, 0.92971E+00_JPRB, 0.91283E+00_JPRB, 0.90371E+00_JPRB ,&
&  0.89292E+00_JPRB, 0.86891E+00_JPRB, 0.85032E+00_JPRB, 0.83548E+00_JPRB, 0.82395E+00_JPRB, 0.81425E+00_JPRB, 0.80333E+00_JPRB ,&
&  0.78864E+00_JPRB, 0.77272E+00_JPRB, 0.75461E+00_JPRB, 0.73269E+00_JPRB, 0.71381E+00_JPRB, 0.69446E+00_JPRB, 0.68305E+00_JPRB ,&
&  0.67387E+00_JPRB, 0.67173E+00_JPRB, 0.66977E+00_JPRB, 0.67333E+00_JPRB, 0.67593E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.66421E+00_JPRB, 0.67755E+00_JPRB, 0.68769E+00_JPRB, 0.69314E+00_JPRB, 0.69682E+00_JPRB, 0.70017E+00_JPRB, 0.70052E+00_JPRB ,&
&  0.70093E+00_JPRB, 0.69991E+00_JPRB, 0.70020E+00_JPRB, 0.69991E+00_JPRB, 0.70106E+00_JPRB, 0.70075E+00_JPRB, 0.69972E+00_JPRB ,&
&  0.69798E+00_JPRB, 0.69459E+00_JPRB, 0.68500E+00_JPRB, 0.66811E+00_JPRB, 0.64681E+00_JPRB, 0.62304E+00_JPRB, 0.58865E+00_JPRB ,&
&  0.55351E+00_JPRB, 0.52981E+00_JPRB, 0.50951E+00_JPRB, 0.49405E+00_JPRB, 0.48487E+00_JPRB, 0.47224E+00_JPRB, 0.46909E+00_JPRB ,&
&  0.46628E+00_JPRB, 0.45464E+00_JPRB, 0.43954E+00_JPRB, 0.42375E+00_JPRB, 0.40737E+00_JPRB, 0.36872E+00_JPRB, 0.36511E+00_JPRB ,&
&  0.35590E+00_JPRB, 0.34682E+00_JPRB, 0.34463E+00_JPRB, 0.34608E+00_JPRB, 0.34994E+00_JPRB, 0.36249E+00_JPRB, 0.37782E+00_JPRB ,&
&  0.39827E+00_JPRB, 0.49980E+00_JPRB, 0.52397E+00_JPRB, 0.53718E+00_JPRB, 0.55898E+00_JPRB, 0.58210E+00_JPRB, 0.60089E+00_JPRB ,&
&  0.61896E+00_JPRB, 0.63508E+00_JPRB, 0.65715E+00_JPRB, 0.67852E+00_JPRB, 0.70190E+00_JPRB, 0.72297E+00_JPRB, 0.73786E+00_JPRB ,&
&  0.75167E+00_JPRB, 0.75592E+00_JPRB, 0.78359E+00_JPRB, 0.79331E+00_JPRB, 0.78659E+00_JPRB, 0.80722E+00_JPRB, 0.81939E+00_JPRB ,&
&  0.81783E+00_JPRB, 0.81002E+00_JPRB, 0.82861E+00_JPRB, 0.87294E+00_JPRB, 0.93587E+00_JPRB, 0.98508E+00_JPRB, 0.10000E+01_JPRB ,&
&  0.10062E+01_JPRB, 0.10005E+01_JPRB, 0.99233E+00_JPRB, 0.99072E+00_JPRB, 0.96560E+00_JPRB, 0.93094E+00_JPRB, 0.90702E+00_JPRB ,&
&  0.88674E+00_JPRB, 0.88558E+00_JPRB, 0.87656E+00_JPRB, 0.86831E+00_JPRB, 0.85445E+00_JPRB, 0.83978E+00_JPRB, 0.81609E+00_JPRB ,&
&  0.78095E+00_JPRB, 0.74241E+00_JPRB, 0.71197E+00_JPRB, 0.68601E+00_JPRB, 0.67088E+00_JPRB, 0.65446E+00_JPRB, 0.64649E+00_JPRB ,&
&  0.63900E+00_JPRB, 0.63813E+00_JPRB, 0.63760E+00_JPRB, 0.64325E+00_JPRB, 0.65105E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.66081E+00_JPRB, 0.67911E+00_JPRB, 0.69565E+00_JPRB, 0.70589E+00_JPRB, 0.70777E+00_JPRB, 0.70852E+00_JPRB, 0.70672E+00_JPRB ,&
&  0.70607E+00_JPRB, 0.70574E+00_JPRB, 0.70464E+00_JPRB, 0.70089E+00_JPRB, 0.69707E+00_JPRB, 0.69190E+00_JPRB, 0.68883E+00_JPRB ,&
&  0.68471E+00_JPRB, 0.67378E+00_JPRB, 0.66615E+00_JPRB, 0.65805E+00_JPRB, 0.64405E+00_JPRB, 0.62540E+00_JPRB, 0.59719E+00_JPRB ,&
&  0.56376E+00_JPRB, 0.53752E+00_JPRB, 0.51921E+00_JPRB, 0.50275E+00_JPRB, 0.49037E+00_JPRB, 0.48332E+00_JPRB, 0.47902E+00_JPRB ,&
&  0.47667E+00_JPRB, 0.47004E+00_JPRB, 0.46593E+00_JPRB, 0.44513E+00_JPRB, 0.41826E+00_JPRB, 0.40958E+00_JPRB, 0.38938E+00_JPRB ,&
&  0.36849E+00_JPRB, 0.35177E+00_JPRB, 0.34800E+00_JPRB, 0.34849E+00_JPRB, 0.35480E+00_JPRB, 0.36808E+00_JPRB, 0.39736E+00_JPRB ,&
&  0.44456E+00_JPRB, 0.52662E+00_JPRB, 0.54571E+00_JPRB, 0.56732E+00_JPRB, 0.58953E+00_JPRB, 0.61089E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.63840E+00_JPRB, 0.65167E+00_JPRB, 0.67622E+00_JPRB, 0.70167E+00_JPRB, 0.74251E+00_JPRB, 0.78154E+00_JPRB, 0.81503E+00_JPRB ,&
&  0.84577E+00_JPRB, 0.81804E+00_JPRB, 0.82544E+00_JPRB, 0.83360E+00_JPRB, 0.86628E+00_JPRB, 0.89600E+00_JPRB, 0.89101E+00_JPRB ,&
&  0.89156E+00_JPRB, 0.92701E+00_JPRB, 0.93245E+00_JPRB, 0.95721E+00_JPRB, 0.10130E+01_JPRB, 0.10746E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10778E+01_JPRB, 0.11263E+01_JPRB, 0.10788E+01_JPRB, 0.10495E+01_JPRB, 0.10030E+01_JPRB, 0.94679E+00_JPRB ,&
&  0.91658E+00_JPRB, 0.90156E+00_JPRB, 0.89754E+00_JPRB, 0.89304E+00_JPRB, 0.87833E+00_JPRB, 0.85349E+00_JPRB, 0.80818E+00_JPRB ,&
&  0.74881E+00_JPRB, 0.70639E+00_JPRB, 0.67694E+00_JPRB, 0.65843E+00_JPRB, 0.64507E+00_JPRB, 0.63093E+00_JPRB, 0.61922E+00_JPRB ,&
&  0.60958E+00_JPRB, 0.60861E+00_JPRB, 0.61586E+00_JPRB, 0.63180E+00_JPRB, 0.64757E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.68255E+00_JPRB, 0.70351E+00_JPRB, 0.71581E+00_JPRB, 0.71345E+00_JPRB, 0.69969E+00_JPRB, 0.67997E+00_JPRB, 0.66524E+00_JPRB ,&
&  0.66006E+00_JPRB, 0.66106E+00_JPRB, 0.66402E+00_JPRB, 0.66159E+00_JPRB, 0.66054E+00_JPRB, 0.66045E+00_JPRB, 0.66223E+00_JPRB ,&
&  0.66321E+00_JPRB, 0.66362E+00_JPRB, 0.66264E+00_JPRB, 0.65394E+00_JPRB, 0.64087E+00_JPRB, 0.62308E+00_JPRB, 0.60071E+00_JPRB ,&
&  0.57461E+00_JPRB, 0.55264E+00_JPRB, 0.53432E+00_JPRB, 0.51972E+00_JPRB, 0.50543E+00_JPRB, 0.49802E+00_JPRB, 0.49441E+00_JPRB ,&
&  0.49221E+00_JPRB, 0.49187E+00_JPRB, 0.48431E+00_JPRB, 0.47420E+00_JPRB, 0.45288E+00_JPRB, 0.42706E+00_JPRB, 0.40269E+00_JPRB ,&
&  0.38282E+00_JPRB, 0.37094E+00_JPRB, 0.36459E+00_JPRB, 0.36363E+00_JPRB, 0.37357E+00_JPRB, 0.39476E+00_JPRB, 0.43911E+00_JPRB ,&
&  0.52004E+00_JPRB, 0.55181E+00_JPRB, 0.56831E+00_JPRB, 0.58643E+00_JPRB, 0.60289E+00_JPRB, 0.61568E+00_JPRB, 0.62739E+00_JPRB ,&
&  0.63791E+00_JPRB, 0.64815E+00_JPRB, 0.66518E+00_JPRB, 0.68025E+00_JPRB, 0.70803E+00_JPRB, 0.73199E+00_JPRB, 0.81302E+00_JPRB ,&
&  0.96739E+00_JPRB, 0.97684E+00_JPRB, 0.90635E+00_JPRB, 0.91797E+00_JPRB, 0.95816E+00_JPRB, 0.99760E+00_JPRB, 0.10378E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10768E+01_JPRB, 0.10818E+01_JPRB, 0.10859E+01_JPRB, 0.11086E+01_JPRB, 0.11331E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.11550E+01_JPRB, 0.12111E+01_JPRB, 0.10365E+01_JPRB, 0.95905E+00_JPRB, 0.93921E+00_JPRB, 0.90797E+00_JPRB ,&
&  0.88852E+00_JPRB, 0.88705E+00_JPRB, 0.90327E+00_JPRB, 0.91184E+00_JPRB, 0.86722E+00_JPRB, 0.80351E+00_JPRB, 0.73386E+00_JPRB ,&
&  0.69337E+00_JPRB, 0.67223E+00_JPRB, 0.65275E+00_JPRB, 0.64184E+00_JPRB, 0.63262E+00_JPRB, 0.61970E+00_JPRB, 0.60002E+00_JPRB ,&
&  0.59749E+00_JPRB, 0.60803E+00_JPRB, 0.62322E+00_JPRB, 0.65016E+00_JPRB, 0.66851E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.67813E+00_JPRB, 0.70935E+00_JPRB, 0.71369E+00_JPRB, 0.69367E+00_JPRB, 0.67154E+00_JPRB, 0.63394E+00_JPRB, 0.60068E+00_JPRB ,&
&  0.58613E+00_JPRB, 0.57955E+00_JPRB, 0.57916E+00_JPRB, 0.58016E+00_JPRB, 0.58436E+00_JPRB, 0.59216E+00_JPRB, 0.60034E+00_JPRB ,&
&  0.60496E+00_JPRB, 0.60872E+00_JPRB, 0.61297E+00_JPRB, 0.61803E+00_JPRB, 0.61641E+00_JPRB, 0.60411E+00_JPRB, 0.58851E+00_JPRB ,&
&  0.56604E+00_JPRB, 0.54897E+00_JPRB, 0.53612E+00_JPRB, 0.52584E+00_JPRB, 0.51909E+00_JPRB, 0.51713E+00_JPRB, 0.51842E+00_JPRB ,&
&  0.51670E+00_JPRB, 0.51573E+00_JPRB, 0.51471E+00_JPRB, 0.50643E+00_JPRB, 0.48681E+00_JPRB, 0.45740E+00_JPRB, 0.42502E+00_JPRB ,&
&  0.40346E+00_JPRB, 0.39574E+00_JPRB, 0.39920E+00_JPRB, 0.41155E+00_JPRB, 0.44690E+00_JPRB, 0.50960E+00_JPRB, 0.53892E+00_JPRB ,&
&  0.56358E+00_JPRB, 0.57835E+00_JPRB, 0.59155E+00_JPRB, 0.60785E+00_JPRB, 0.62123E+00_JPRB, 0.63138E+00_JPRB, 0.64137E+00_JPRB ,&
&  0.65767E+00_JPRB, 0.67537E+00_JPRB, 0.69907E+00_JPRB, 0.65911E+00_JPRB, 0.67560E+00_JPRB, 0.78058E+00_JPRB, 0.86064E+00_JPRB ,&
&  0.95731E+00_JPRB, 0.98582E+00_JPRB, 0.99838E+00_JPRB, 0.10781E+01_JPRB, 0.11176E+01_JPRB, 0.11718E+01_JPRB, 0.11852E+01_JPRB ,&
&  0.11782E+01_JPRB, 0.11574E+01_JPRB, 0.11353E+01_JPRB, 0.11205E+01_JPRB, 0.11382E+01_JPRB, 0.11761E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.11452E+01_JPRB, 0.11292E+01_JPRB, 0.10437E+01_JPRB, 0.92668E+00_JPRB, 0.88469E+00_JPRB, 0.87807E+00_JPRB, 0.88771E+00_JPRB ,&
&  0.88405E+00_JPRB, 0.89068E+00_JPRB, 0.90188E+00_JPRB, 0.90471E+00_JPRB, 0.82799E+00_JPRB, 0.74781E+00_JPRB, 0.67624E+00_JPRB ,&
&  0.63898E+00_JPRB, 0.63122E+00_JPRB, 0.62212E+00_JPRB, 0.62598E+00_JPRB, 0.62009E+00_JPRB, 0.60889E+00_JPRB, 0.59647E+00_JPRB ,&
&  0.59542E+00_JPRB, 0.60156E+00_JPRB, 0.62319E+00_JPRB, 0.64794E+00_JPRB, 0.66261E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.71804E+00_JPRB, 0.72557E+00_JPRB, 0.70106E+00_JPRB, 0.68284E+00_JPRB, 0.66266E+00_JPRB, 0.63958E+00_JPRB, 0.61393E+00_JPRB ,&
&  0.60663E+00_JPRB, 0.58114E+00_JPRB, 0.56097E+00_JPRB, 0.54422E+00_JPRB, 0.53561E+00_JPRB, 0.53280E+00_JPRB, 0.54119E+00_JPRB ,&
&  0.55840E+00_JPRB, 0.58002E+00_JPRB, 0.59355E+00_JPRB, 0.59467E+00_JPRB, 0.59012E+00_JPRB, 0.57649E+00_JPRB, 0.56642E+00_JPRB ,&
&  0.54964E+00_JPRB, 0.54030E+00_JPRB, 0.54026E+00_JPRB, 0.54431E+00_JPRB, 0.54891E+00_JPRB, 0.54767E+00_JPRB, 0.54050E+00_JPRB ,&
&  0.53102E+00_JPRB, 0.52626E+00_JPRB, 0.52894E+00_JPRB, 0.53007E+00_JPRB, 0.51840E+00_JPRB, 0.48287E+00_JPRB, 0.44866E+00_JPRB ,&
&  0.42692E+00_JPRB, 0.42542E+00_JPRB, 0.45474E+00_JPRB, 0.51890E+00_JPRB, 0.54882E+00_JPRB, 0.56829E+00_JPRB, 0.57914E+00_JPRB ,&
&  0.57394E+00_JPRB, 0.57868E+00_JPRB, 0.60475E+00_JPRB, 0.65305E+00_JPRB, 0.66200E+00_JPRB, 0.67171E+00_JPRB, 0.68229E+00_JPRB ,&
&  0.70560E+00_JPRB, 0.70614E+00_JPRB, 0.72310E+00_JPRB, 0.71592E+00_JPRB, 0.83281E+00_JPRB, 0.10805E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11398E+01_JPRB, 0.12281E+01_JPRB, 0.12668E+01_JPRB, 0.12719E+01_JPRB, 0.12826E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.12208E+01_JPRB, 0.11940E+01_JPRB, 0.11587E+01_JPRB, 0.11622E+01_JPRB, 0.11643E+01_JPRB, 0.11784E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.11682E+01_JPRB, 0.11468E+01_JPRB, 0.10730E+01_JPRB, 0.97863E+00_JPRB, 0.92969E+00_JPRB, 0.88782E+00_JPRB, 0.86106E+00_JPRB ,&
&  0.84555E+00_JPRB, 0.86157E+00_JPRB, 0.89009E+00_JPRB, 0.90614E+00_JPRB, 0.84542E+00_JPRB, 0.74978E+00_JPRB, 0.66236E+00_JPRB ,&
&  0.61496E+00_JPRB, 0.60115E+00_JPRB, 0.58225E+00_JPRB, 0.57734E+00_JPRB, 0.58386E+00_JPRB, 0.59008E+00_JPRB, 0.59469E+00_JPRB ,&
&  0.60073E+00_JPRB, 0.61309E+00_JPRB, 0.64748E+00_JPRB, 0.68357E+00_JPRB, 0.70400E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.71571E+00_JPRB, 0.72601E+00_JPRB, 0.71406E+00_JPRB, 0.70885E+00_JPRB, 0.69992E+00_JPRB, 0.67377E+00_JPRB, 0.63404E+00_JPRB ,&
&  0.62839E+00_JPRB, 0.57718E+00_JPRB, 0.55726E+00_JPRB, 0.54719E+00_JPRB, 0.54566E+00_JPRB, 0.53924E+00_JPRB, 0.52945E+00_JPRB ,&
&  0.53390E+00_JPRB, 0.55163E+00_JPRB, 0.57871E+00_JPRB, 0.59128E+00_JPRB, 0.59134E+00_JPRB, 0.57913E+00_JPRB, 0.55960E+00_JPRB ,&
&  0.54016E+00_JPRB, 0.53508E+00_JPRB, 0.54488E+00_JPRB, 0.56025E+00_JPRB, 0.56994E+00_JPRB, 0.56796E+00_JPRB, 0.56146E+00_JPRB ,&
&  0.55281E+00_JPRB, 0.54418E+00_JPRB, 0.54157E+00_JPRB, 0.54198E+00_JPRB, 0.53945E+00_JPRB, 0.52743E+00_JPRB, 0.49195E+00_JPRB ,&
&  0.46994E+00_JPRB, 0.47848E+00_JPRB, 0.54062E+00_JPRB, 0.57998E+00_JPRB, 0.60138E+00_JPRB, 0.61012E+00_JPRB, 0.61953E+00_JPRB ,&
&  0.63340E+00_JPRB, 0.65125E+00_JPRB, 0.67408E+00_JPRB, 0.70275E+00_JPRB, 0.72197E+00_JPRB, 0.73972E+00_JPRB, 0.75793E+00_JPRB ,&
&  0.80701E+00_JPRB, 0.71106E+00_JPRB, 0.68898E+00_JPRB, 0.73959E+00_JPRB, 0.10190E+01_JPRB, 0.11354E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.12555E+01_JPRB, 0.13013E+01_JPRB, 0.13718E+01_JPRB, 0.14130E+01_JPRB, 0.14217E+01_JPRB, 0.14087E+01_JPRB ,&
&  0.13792E+01_JPRB, 0.13413E+01_JPRB, 0.12780E+01_JPRB, 0.12401E+01_JPRB, 0.12096E+01_JPRB, 0.12236E+01_JPRB, 0.12094E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11639E+01_JPRB, 0.10947E+01_JPRB, 0.10173E+01_JPRB, 0.95782E+00_JPRB, 0.92014E+00_JPRB, 0.89003E+00_JPRB ,&
&  0.86955E+00_JPRB, 0.87433E+00_JPRB, 0.89305E+00_JPRB, 0.92489E+00_JPRB, 0.90183E+00_JPRB, 0.82273E+00_JPRB, 0.74178E+00_JPRB ,&
&  0.66829E+00_JPRB, 0.63128E+00_JPRB, 0.59485E+00_JPRB, 0.58637E+00_JPRB, 0.59765E+00_JPRB, 0.61375E+00_JPRB, 0.63182E+00_JPRB ,&
&  0.65350E+00_JPRB, 0.66896E+00_JPRB, 0.68401E+00_JPRB, 0.70469E+00_JPRB, 0.70906E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.71767E+00_JPRB, 0.73614E+00_JPRB, 0.73374E+00_JPRB, 0.73769E+00_JPRB, 0.73440E+00_JPRB, 0.69705E+00_JPRB, 0.66969E+00_JPRB ,&
&  0.69638E+00_JPRB, 0.64790E+00_JPRB, 0.66935E+00_JPRB, 0.59973E+00_JPRB, 0.57194E+00_JPRB, 0.56378E+00_JPRB, 0.54805E+00_JPRB ,&
&  0.54612E+00_JPRB, 0.55834E+00_JPRB, 0.57331E+00_JPRB, 0.58604E+00_JPRB, 0.59326E+00_JPRB, 0.57944E+00_JPRB, 0.56240E+00_JPRB ,&
&  0.54122E+00_JPRB, 0.53603E+00_JPRB, 0.54736E+00_JPRB, 0.56146E+00_JPRB, 0.57407E+00_JPRB, 0.57772E+00_JPRB, 0.57348E+00_JPRB ,&
&  0.56273E+00_JPRB, 0.55738E+00_JPRB, 0.55798E+00_JPRB, 0.55576E+00_JPRB, 0.55527E+00_JPRB, 0.54928E+00_JPRB, 0.54140E+00_JPRB ,&
&  0.53630E+00_JPRB, 0.54558E+00_JPRB, 0.58948E+00_JPRB, 0.62049E+00_JPRB, 0.63763E+00_JPRB, 0.64369E+00_JPRB, 0.65646E+00_JPRB ,&
&  0.66880E+00_JPRB, 0.68591E+00_JPRB, 0.71660E+00_JPRB, 0.75689E+00_JPRB, 0.79176E+00_JPRB, 0.82161E+00_JPRB, 0.85713E+00_JPRB ,&
&  0.91462E+00_JPRB, 0.77232E+00_JPRB, 0.87112E+00_JPRB, 0.89798E+00_JPRB, 0.10696E+01_JPRB, 0.12736E+01_JPRB, 0.12827E+01_JPRB ,&
&  0.14711E+01_JPRB, 0.14360E+01_JPRB, 0.14832E+01_JPRB, 0.15101E+01_JPRB, 0.15473E+01_JPRB, 0.15707E+01_JPRB, 0.15746E+01_JPRB ,&
&  0.16162E+01_JPRB, 0.14618E+01_JPRB, 0.14575E+01_JPRB, 0.13818E+01_JPRB, 0.13251E+01_JPRB, 0.13175E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.12257E+01_JPRB, 0.12018E+01_JPRB, 0.11587E+01_JPRB, 0.10358E+01_JPRB, 0.99962E+00_JPRB, 0.96963E+00_JPRB, 0.90050E+00_JPRB ,&
&  0.87556E+00_JPRB, 0.84767E+00_JPRB, 0.82554E+00_JPRB, 0.89980E+00_JPRB, 0.90306E+00_JPRB, 0.83354E+00_JPRB, 0.74558E+00_JPRB ,&
&  0.68630E+00_JPRB, 0.64693E+00_JPRB, 0.66331E+00_JPRB, 0.66625E+00_JPRB, 0.65447E+00_JPRB, 0.65687E+00_JPRB, 0.69883E+00_JPRB ,&
&  0.69425E+00_JPRB, 0.70535E+00_JPRB, 0.73292E+00_JPRB, 0.73345E+00_JPRB, 0.72030E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.73393E+00_JPRB, 0.75954E+00_JPRB, 0.75144E+00_JPRB, 0.76128E+00_JPRB, 0.76886E+00_JPRB, 0.73371E+00_JPRB, 0.69492E+00_JPRB ,&
&  0.69887E+00_JPRB, 0.69705E+00_JPRB, 0.69798E+00_JPRB, 0.69687E+00_JPRB, 0.69908E+00_JPRB, 0.64791E+00_JPRB, 0.56248E+00_JPRB ,&
&  0.53517E+00_JPRB, 0.54456E+00_JPRB, 0.56722E+00_JPRB, 0.58408E+00_JPRB, 0.58640E+00_JPRB, 0.57861E+00_JPRB, 0.57079E+00_JPRB ,&
&  0.56096E+00_JPRB, 0.55445E+00_JPRB, 0.56061E+00_JPRB, 0.56663E+00_JPRB, 0.57145E+00_JPRB, 0.58006E+00_JPRB, 0.58420E+00_JPRB ,&
&  0.57593E+00_JPRB, 0.57563E+00_JPRB, 0.57332E+00_JPRB, 0.56252E+00_JPRB, 0.57452E+00_JPRB, 0.56897E+00_JPRB, 0.56329E+00_JPRB ,&
&  0.57536E+00_JPRB, 0.59906E+00_JPRB, 0.62319E+00_JPRB, 0.65183E+00_JPRB, 0.67196E+00_JPRB, 0.68227E+00_JPRB, 0.69572E+00_JPRB ,&
&  0.71819E+00_JPRB, 0.75067E+00_JPRB, 0.78968E+00_JPRB, 0.83710E+00_JPRB, 0.89172E+00_JPRB, 0.89710E+00_JPRB, 0.98797E+00_JPRB ,&
&  0.99965E+00_JPRB, 0.10385E+01_JPRB, 0.10651E+01_JPRB, 0.10133E+01_JPRB, 0.11840E+01_JPRB, 0.13628E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.14766E+01_JPRB, 0.14226E+01_JPRB, 0.15523E+01_JPRB, 0.17021E+01_JPRB, 0.18222E+01_JPRB, 0.18106E+01_JPRB, 0.17801E+01_JPRB ,&
&  0.18434E+01_JPRB, 0.18220E+01_JPRB, 0.16268E+01_JPRB, 0.15482E+01_JPRB, 0.14673E+01_JPRB, 0.14425E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.12938E+01_JPRB, 0.12482E+01_JPRB, 0.11889E+01_JPRB, 0.11174E+01_JPRB, 0.10520E+01_JPRB, 0.98980E+00_JPRB ,&
&  0.91971E+00_JPRB, 0.82594E+00_JPRB, 0.79211E+00_JPRB, 0.83550E+00_JPRB, 0.88663E+00_JPRB, 0.80426E+00_JPRB, 0.71048E+00_JPRB ,&
&  0.67776E+00_JPRB, 0.68489E+00_JPRB, 0.73974E+00_JPRB, 0.74270E+00_JPRB, 0.72920E+00_JPRB, 0.71954E+00_JPRB, 0.69752E+00_JPRB ,&
&  0.67681E+00_JPRB, 0.72669E+00_JPRB, 0.74314E+00_JPRB, 0.73714E+00_JPRB, 0.72648E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.76530E+00_JPRB, 0.80305E+00_JPRB, 0.79092E+00_JPRB, 0.79770E+00_JPRB, 0.81211E+00_JPRB, 0.76653E+00_JPRB, 0.74606E+00_JPRB ,&
&  0.74271E+00_JPRB, 0.73643E+00_JPRB, 0.72503E+00_JPRB, 0.71924E+00_JPRB, 0.71702E+00_JPRB, 0.70492E+00_JPRB, 0.64262E+00_JPRB ,&
&  0.57396E+00_JPRB, 0.57261E+00_JPRB, 0.56841E+00_JPRB, 0.58359E+00_JPRB, 0.59999E+00_JPRB, 0.59195E+00_JPRB, 0.58575E+00_JPRB ,&
&  0.58666E+00_JPRB, 0.57962E+00_JPRB, 0.57549E+00_JPRB, 0.57772E+00_JPRB, 0.58557E+00_JPRB, 0.59995E+00_JPRB, 0.60704E+00_JPRB ,&
&  0.59397E+00_JPRB, 0.58698E+00_JPRB, 0.58156E+00_JPRB, 0.57943E+00_JPRB, 0.59116E+00_JPRB, 0.59470E+00_JPRB, 0.59637E+00_JPRB ,&
&  0.60855E+00_JPRB, 0.63019E+00_JPRB, 0.65269E+00_JPRB, 0.68542E+00_JPRB, 0.71444E+00_JPRB, 0.73735E+00_JPRB, 0.76942E+00_JPRB ,&
&  0.83287E+00_JPRB, 0.89499E+00_JPRB, 0.92795E+00_JPRB, 0.97497E+00_JPRB, 0.10172E+01_JPRB, 0.10869E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.11840E+01_JPRB, 0.11600E+01_JPRB, 0.12076E+01_JPRB, 0.13851E+01_JPRB, 0.15071E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.16843E+01_JPRB, 0.16276E+01_JPRB, 0.19730E+01_JPRB, 0.20909E+01_JPRB, 0.20475E+01_JPRB, 0.20670E+01_JPRB, 0.19594E+01_JPRB ,&
&  0.19325E+01_JPRB, 0.18196E+01_JPRB, 0.17371E+01_JPRB, 0.16620E+01_JPRB, 0.15905E+01_JPRB, 0.15950E+01_JPRB, 0.15099E+01_JPRB ,&
&  0.14935E+01_JPRB, 0.14535E+01_JPRB, 0.12741E+01_JPRB, 0.11414E+01_JPRB, 0.10037E+01_JPRB, 0.10514E+01_JPRB, 0.10138E+01_JPRB ,&
&  0.93297E+00_JPRB, 0.80597E+00_JPRB, 0.84582E+00_JPRB, 0.90679E+00_JPRB, 0.97962E+00_JPRB, 0.97405E+00_JPRB, 0.82579E+00_JPRB ,&
&  0.75221E+00_JPRB, 0.77099E+00_JPRB, 0.78114E+00_JPRB, 0.79436E+00_JPRB, 0.79812E+00_JPRB, 0.79547E+00_JPRB, 0.73605E+00_JPRB ,&
&  0.70206E+00_JPRB, 0.73990E+00_JPRB, 0.75457E+00_JPRB, 0.75036E+00_JPRB, 0.75053E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.80843E+00_JPRB, 0.85540E+00_JPRB, 0.86030E+00_JPRB, 0.84896E+00_JPRB, 0.85581E+00_JPRB, 0.80843E+00_JPRB, 0.78680E+00_JPRB ,&
&  0.78850E+00_JPRB, 0.78050E+00_JPRB, 0.75513E+00_JPRB, 0.74620E+00_JPRB, 0.74022E+00_JPRB, 0.72702E+00_JPRB, 0.70480E+00_JPRB ,&
&  0.61150E+00_JPRB, 0.59434E+00_JPRB, 0.56680E+00_JPRB, 0.57470E+00_JPRB, 0.63016E+00_JPRB, 0.62048E+00_JPRB, 0.61639E+00_JPRB ,&
&  0.61227E+00_JPRB, 0.60965E+00_JPRB, 0.59787E+00_JPRB, 0.59576E+00_JPRB, 0.60368E+00_JPRB, 0.62609E+00_JPRB, 0.64865E+00_JPRB ,&
&  0.63817E+00_JPRB, 0.62016E+00_JPRB, 0.60890E+00_JPRB, 0.60601E+00_JPRB, 0.61413E+00_JPRB, 0.62621E+00_JPRB, 0.64287E+00_JPRB ,&
&  0.65904E+00_JPRB, 0.67423E+00_JPRB, 0.70389E+00_JPRB, 0.74780E+00_JPRB, 0.78486E+00_JPRB, 0.81975E+00_JPRB, 0.89451E+00_JPRB ,&
&  0.10087E+01_JPRB, 0.10960E+01_JPRB, 0.11390E+01_JPRB, 0.11770E+01_JPRB, 0.11867E+01_JPRB, 0.12512E+01_JPRB, 0.12891E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.13493E+01_JPRB, 0.13458E+01_JPRB, 0.15764E+01_JPRB, 0.17891E+01_JPRB, 0.18655E+01_JPRB, 0.18596E+01_JPRB ,&
&  0.19089E+01_JPRB, 0.18661E+01_JPRB, 0.19999E+01_JPRB, 0.21450E+01_JPRB, 0.21002E+01_JPRB, 0.21840E+01_JPRB, 0.20435E+01_JPRB ,&
&  0.18333E+01_JPRB, 0.17184E+01_JPRB, 0.16760E+01_JPRB, 0.16641E+01_JPRB, 0.16062E+01_JPRB, 0.16756E+01_JPRB, 0.16456E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.14854E+01_JPRB, 0.11456E+01_JPRB, 0.10631E+01_JPRB, 0.86987E+00_JPRB, 0.88967E+00_JPRB, 0.10724E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.96892E+00_JPRB, 0.98846E+00_JPRB, 0.10665E+01_JPRB, 0.11748E+01_JPRB, 0.13383E+01_JPRB, 0.11514E+01_JPRB ,&
&  0.95111E+00_JPRB, 0.91128E+00_JPRB, 0.90983E+00_JPRB, 0.91121E+00_JPRB, 0.90506E+00_JPRB, 0.91435E+00_JPRB, 0.88975E+00_JPRB ,&
&  0.85537E+00_JPRB, 0.82512E+00_JPRB, 0.81290E+00_JPRB, 0.80010E+00_JPRB, 0.80130E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.85725E+00_JPRB, 0.89715E+00_JPRB, 0.91540E+00_JPRB, 0.89874E+00_JPRB, 0.89718E+00_JPRB, 0.86293E+00_JPRB, 0.83025E+00_JPRB ,&
&  0.82678E+00_JPRB, 0.81521E+00_JPRB, 0.78615E+00_JPRB, 0.77645E+00_JPRB, 0.76575E+00_JPRB, 0.74444E+00_JPRB, 0.72391E+00_JPRB ,&
&  0.71860E+00_JPRB, 0.66702E+00_JPRB, 0.61346E+00_JPRB, 0.58233E+00_JPRB, 0.62292E+00_JPRB, 0.62239E+00_JPRB, 0.63472E+00_JPRB ,&
&  0.62952E+00_JPRB, 0.64475E+00_JPRB, 0.64050E+00_JPRB, 0.63764E+00_JPRB, 0.65403E+00_JPRB, 0.68328E+00_JPRB, 0.70835E+00_JPRB ,&
&  0.70932E+00_JPRB, 0.70304E+00_JPRB, 0.68834E+00_JPRB, 0.68659E+00_JPRB, 0.70837E+00_JPRB, 0.69963E+00_JPRB, 0.72367E+00_JPRB ,&
&  0.73588E+00_JPRB, 0.74586E+00_JPRB, 0.76959E+00_JPRB, 0.81723E+00_JPRB, 0.86594E+00_JPRB, 0.92289E+00_JPRB, 0.10336E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.12016E+01_JPRB, 0.12031E+01_JPRB, 0.12283E+01_JPRB, 0.13084E+01_JPRB, 0.14105E+01_JPRB, 0.14082E+01_JPRB ,&
&  0.14255E+01_JPRB, 0.13567E+01_JPRB, 0.12934E+01_JPRB, 0.16350E+01_JPRB, 0.19321E+01_JPRB, 0.19725E+01_JPRB, 0.19736E+01_JPRB ,&
&  0.20453E+01_JPRB, 0.20864E+01_JPRB, 0.22312E+01_JPRB, 0.22978E+01_JPRB, 0.23098E+01_JPRB, 0.22703E+01_JPRB, 0.20682E+01_JPRB ,&
&  0.18167E+01_JPRB, 0.16965E+01_JPRB, 0.17198E+01_JPRB, 0.17158E+01_JPRB, 0.17132E+01_JPRB, 0.16453E+01_JPRB, 0.17534E+01_JPRB ,&
&  0.18650E+01_JPRB, 0.11073E+01_JPRB, 0.90777E+00_JPRB, 0.10435E+01_JPRB, 0.91387E+00_JPRB, 0.92995E+00_JPRB, 0.10315E+01_JPRB ,&
&  0.99237E+00_JPRB, 0.97377E+00_JPRB, 0.11119E+01_JPRB, 0.12094E+01_JPRB, 0.13726E+01_JPRB, 0.16095E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.12494E+01_JPRB, 0.11522E+01_JPRB, 0.11536E+01_JPRB, 0.11258E+01_JPRB, 0.11389E+01_JPRB, 0.11202E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10290E+01_JPRB, 0.98500E+00_JPRB, 0.93542E+00_JPRB, 0.89682E+00_JPRB, 0.87063E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.95052E+00_JPRB, 0.97158E+00_JPRB, 0.97241E+00_JPRB, 0.95494E+00_JPRB, 0.94262E+00_JPRB, 0.93006E+00_JPRB, 0.88929E+00_JPRB ,&
&  0.86480E+00_JPRB, 0.84438E+00_JPRB, 0.81592E+00_JPRB, 0.80494E+00_JPRB, 0.78321E+00_JPRB, 0.75895E+00_JPRB, 0.73967E+00_JPRB ,&
&  0.73575E+00_JPRB, 0.72000E+00_JPRB, 0.63724E+00_JPRB, 0.60538E+00_JPRB, 0.58128E+00_JPRB, 0.60322E+00_JPRB, 0.62374E+00_JPRB ,&
&  0.65181E+00_JPRB, 0.66600E+00_JPRB, 0.67761E+00_JPRB, 0.70188E+00_JPRB, 0.74284E+00_JPRB, 0.76758E+00_JPRB, 0.80767E+00_JPRB ,&
&  0.81692E+00_JPRB, 0.82749E+00_JPRB, 0.78829E+00_JPRB, 0.80111E+00_JPRB, 0.82283E+00_JPRB, 0.80733E+00_JPRB, 0.84612E+00_JPRB ,&
&  0.85578E+00_JPRB, 0.87507E+00_JPRB, 0.89815E+00_JPRB, 0.91843E+00_JPRB, 0.95215E+00_JPRB, 0.98756E+00_JPRB, 0.10846E+01_JPRB ,&
&  0.11591E+01_JPRB, 0.11835E+01_JPRB, 0.11617E+01_JPRB, 0.11657E+01_JPRB, 0.12211E+01_JPRB, 0.12955E+01_JPRB, 0.13128E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.11877E+01_JPRB, 0.93549E+00_JPRB, 0.13869E+01_JPRB, 0.19518E+01_JPRB, 0.19292E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.19074E+01_JPRB, 0.20538E+01_JPRB, 0.22789E+01_JPRB, 0.23703E+01_JPRB, 0.23709E+01_JPRB, 0.22816E+01_JPRB, 0.21350E+01_JPRB ,&
&  0.19639E+01_JPRB, 0.19563E+01_JPRB, 0.20680E+01_JPRB, 0.22633E+01_JPRB, 0.22744E+01_JPRB, 0.22488E+01_JPRB, 0.21953E+01_JPRB ,&
&  0.19217E+01_JPRB, 0.16462E+01_JPRB, 0.11676E+01_JPRB, 0.12031E+01_JPRB, 0.96355E+00_JPRB, 0.96150E+00_JPRB, 0.10578E+01_JPRB ,&
&  0.10192E+01_JPRB, 0.10779E+01_JPRB, 0.12351E+01_JPRB, 0.13706E+01_JPRB, 0.17486E+01_JPRB, 0.18789E+01_JPRB, 0.17954E+01_JPRB ,&
&  0.15698E+01_JPRB, 0.13674E+01_JPRB, 0.14002E+01_JPRB, 0.13886E+01_JPRB, 0.14033E+01_JPRB, 0.13581E+01_JPRB, 0.12960E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11450E+01_JPRB, 0.10961E+01_JPRB, 0.10285E+01_JPRB, 0.96673E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.10658E+01_JPRB, 0.10522E+01_JPRB, 0.10185E+01_JPRB, 0.97658E+00_JPRB, 0.95073E+00_JPRB, 0.94000E+00_JPRB, 0.92057E+00_JPRB ,&
&  0.90189E+00_JPRB, 0.87013E+00_JPRB, 0.84558E+00_JPRB, 0.82332E+00_JPRB, 0.80197E+00_JPRB, 0.77672E+00_JPRB, 0.76169E+00_JPRB ,&
&  0.75717E+00_JPRB, 0.70439E+00_JPRB, 0.61042E+00_JPRB, 0.57089E+00_JPRB, 0.55084E+00_JPRB, 0.56872E+00_JPRB, 0.60238E+00_JPRB ,&
&  0.64264E+00_JPRB, 0.67140E+00_JPRB, 0.72160E+00_JPRB, 0.79108E+00_JPRB, 0.86481E+00_JPRB, 0.92031E+00_JPRB, 0.95661E+00_JPRB ,&
&  0.10757E+01_JPRB, 0.98492E+00_JPRB, 0.99133E+00_JPRB, 0.96101E+00_JPRB, 0.98118E+00_JPRB, 0.97365E+00_JPRB, 0.95728E+00_JPRB ,&
&  0.94435E+00_JPRB, 0.93376E+00_JPRB, 0.92808E+00_JPRB, 0.92167E+00_JPRB, 0.93560E+00_JPRB, 0.98788E+00_JPRB, 0.11049E+01_JPRB ,&
&  0.11961E+01_JPRB, 0.11801E+01_JPRB, 0.11297E+01_JPRB, 0.11690E+01_JPRB, 0.12513E+01_JPRB, 0.13082E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.12872E+01_JPRB, 0.12156E+01_JPRB, 0.14576E+01_JPRB, 0.16724E+01_JPRB, 0.19534E+01_JPRB, 0.20299E+01_JPRB, 0.19427E+01_JPRB ,&
&  0.20638E+01_JPRB, 0.21286E+01_JPRB, 0.22522E+01_JPRB, 0.18779E+01_JPRB, 0.21391E+01_JPRB, 0.21411E+01_JPRB, 0.20972E+01_JPRB ,&
&  0.20783E+01_JPRB, 0.22160E+01_JPRB, 0.23587E+01_JPRB, 0.26411E+01_JPRB, 0.28454E+01_JPRB, 0.29998E+01_JPRB, 0.23208E+01_JPRB ,&
&  0.18510E+01_JPRB, 0.19742E+01_JPRB, 0.17967E+01_JPRB, 0.14894E+01_JPRB, 0.12477E+01_JPRB, 0.12566E+01_JPRB, 0.12407E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.13119E+01_JPRB, 0.14578E+01_JPRB, 0.18258E+01_JPRB, 0.22491E+01_JPRB, 0.24081E+01_JPRB, 0.22198E+01_JPRB ,&
&  0.18617E+01_JPRB, 0.18774E+01_JPRB, 0.17962E+01_JPRB, 0.17217E+01_JPRB, 0.16958E+01_JPRB, 0.16225E+01_JPRB, 0.15056E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.12892E+01_JPRB, 0.12158E+01_JPRB, 0.11413E+01_JPRB, 0.10853E+01_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.11425E+01_JPRB, 0.11154E+01_JPRB, 0.10696E+01_JPRB, 0.10231E+01_JPRB, 0.98697E+00_JPRB, 0.96838E+00_JPRB, 0.95079E+00_JPRB ,&
&  0.94728E+00_JPRB, 0.88754E+00_JPRB, 0.86307E+00_JPRB, 0.83973E+00_JPRB, 0.82008E+00_JPRB, 0.80162E+00_JPRB, 0.79363E+00_JPRB ,&
&  0.78748E+00_JPRB, 0.68854E+00_JPRB, 0.59078E+00_JPRB, 0.58385E+00_JPRB, 0.58380E+00_JPRB, 0.57373E+00_JPRB, 0.60013E+00_JPRB ,&
&  0.64029E+00_JPRB, 0.69269E+00_JPRB, 0.77398E+00_JPRB, 0.91398E+00_JPRB, 0.10383E+01_JPRB, 0.11346E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.12202E+01_JPRB, 0.12175E+01_JPRB, 0.11433E+01_JPRB, 0.10738E+01_JPRB, 0.10314E+01_JPRB, 0.10113E+01_JPRB ,&
&  0.99397E+00_JPRB, 0.96721E+00_JPRB, 0.95900E+00_JPRB, 0.96456E+00_JPRB, 0.98997E+00_JPRB, 0.10617E+01_JPRB, 0.11680E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.11819E+01_JPRB, 0.11602E+01_JPRB, 0.12546E+01_JPRB, 0.11963E+01_JPRB, 0.11654E+01_JPRB, 0.11336E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.15947E+01_JPRB, 0.16150E+01_JPRB, 0.17828E+01_JPRB, 0.16498E+01_JPRB, 0.17744E+01_JPRB, 0.20575E+01_JPRB ,&
&  0.22205E+01_JPRB, 0.22440E+01_JPRB, 0.23126E+01_JPRB, 0.16429E+01_JPRB, 0.15272E+01_JPRB, 0.20805E+01_JPRB, 0.21988E+01_JPRB ,&
&  0.22264E+01_JPRB, 0.25127E+01_JPRB, 0.28634E+01_JPRB, 0.33195E+01_JPRB, 0.25390E+01_JPRB, 0.22368E+01_JPRB, 0.17357E+01_JPRB ,&
&  0.16941E+01_JPRB, 0.15654E+01_JPRB, 0.17831E+01_JPRB, 0.16551E+01_JPRB, 0.16331E+01_JPRB, 0.15840E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.14980E+01_JPRB, 0.15633E+01_JPRB, 0.18536E+01_JPRB, 0.24818E+01_JPRB, 0.30428E+01_JPRB, 0.28446E+01_JPRB, 0.28034E+01_JPRB ,&
&  0.26789E+01_JPRB, 0.25115E+01_JPRB, 0.22299E+01_JPRB, 0.21566E+01_JPRB, 0.20210E+01_JPRB, 0.18382E+01_JPRB, 0.16316E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.13716E+01_JPRB, 0.12852E+01_JPRB, 0.12132E+01_JPRB, 0.11680E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.12169E+01_JPRB, 0.11772E+01_JPRB, 0.11318E+01_JPRB, 0.10834E+01_JPRB, 0.10451E+01_JPRB, 0.10164E+01_JPRB, 0.99566E+00_JPRB ,&
&  0.99439E+00_JPRB, 0.92723E+00_JPRB, 0.87193E+00_JPRB, 0.84679E+00_JPRB, 0.83655E+00_JPRB, 0.83082E+00_JPRB, 0.82956E+00_JPRB ,&
&  0.82832E+00_JPRB, 0.76484E+00_JPRB, 0.63002E+00_JPRB, 0.59333E+00_JPRB, 0.61859E+00_JPRB, 0.60475E+00_JPRB, 0.60480E+00_JPRB ,&
&  0.66612E+00_JPRB, 0.76502E+00_JPRB, 0.91338E+00_JPRB, 0.11276E+01_JPRB, 0.12882E+01_JPRB, 0.14288E+01_JPRB, 0.14977E+01_JPRB ,&
&  0.15027E+01_JPRB, 0.13922E+01_JPRB, 0.12459E+01_JPRB, 0.11187E+01_JPRB, 0.10431E+01_JPRB, 0.10007E+01_JPRB, 0.98412E+00_JPRB ,&
&  0.98335E+00_JPRB, 0.98046E+00_JPRB, 0.98656E+00_JPRB, 0.98999E+00_JPRB, 0.10363E+01_JPRB, 0.10895E+01_JPRB, 0.11781E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.11932E+01_JPRB, 0.12262E+01_JPRB, 0.13144E+01_JPRB, 0.13259E+01_JPRB, 0.12535E+01_JPRB, 0.12322E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.15087E+01_JPRB, 0.16432E+01_JPRB, 0.17610E+01_JPRB, 0.19012E+01_JPRB, 0.19846E+01_JPRB, 0.22416E+01_JPRB ,&
&  0.19238E+01_JPRB, 0.15784E+01_JPRB, 0.13447E+01_JPRB, 0.11723E+01_JPRB, 0.16103E+01_JPRB, 0.23363E+01_JPRB, 0.26488E+01_JPRB ,&
&  0.28965E+01_JPRB, 0.31983E+01_JPRB, 0.35268E+01_JPRB, 0.31710E+01_JPRB, 0.24537E+01_JPRB, 0.19659E+01_JPRB, 0.23573E+01_JPRB ,&
&  0.24655E+01_JPRB, 0.24890E+01_JPRB, 0.23339E+01_JPRB, 0.20493E+01_JPRB, 0.16640E+01_JPRB, 0.18967E+01_JPRB, 0.18388E+01_JPRB ,&
&  0.17976E+01_JPRB, 0.18510E+01_JPRB, 0.27205E+01_JPRB, 0.35576E+01_JPRB, 0.36881E+01_JPRB, 0.35901E+01_JPRB, 0.36144E+01_JPRB ,&
&  0.32801E+01_JPRB, 0.30803E+01_JPRB, 0.27414E+01_JPRB, 0.24037E+01_JPRB, 0.21100E+01_JPRB, 0.18646E+01_JPRB, 0.16665E+01_JPRB ,&
&  0.15349E+01_JPRB, 0.14374E+01_JPRB, 0.13375E+01_JPRB, 0.12849E+01_JPRB, 0.12474E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.13167E+01_JPRB, 0.12681E+01_JPRB, 0.12220E+01_JPRB, 0.11754E+01_JPRB, 0.11330E+01_JPRB, 0.10775E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10367E+01_JPRB, 0.96668E+00_JPRB, 0.90200E+00_JPRB, 0.87552E+00_JPRB, 0.88402E+00_JPRB, 0.88965E+00_JPRB, 0.88488E+00_JPRB ,&
&  0.87782E+00_JPRB, 0.89277E+00_JPRB, 0.72497E+00_JPRB, 0.65399E+00_JPRB, 0.65409E+00_JPRB, 0.64902E+00_JPRB, 0.62700E+00_JPRB ,&
&  0.74134E+00_JPRB, 0.92961E+00_JPRB, 0.11374E+01_JPRB, 0.13724E+01_JPRB, 0.16180E+01_JPRB, 0.15805E+01_JPRB, 0.15776E+01_JPRB ,&
&  0.15743E+01_JPRB, 0.14550E+01_JPRB, 0.13096E+01_JPRB, 0.11917E+01_JPRB, 0.11107E+01_JPRB, 0.10687E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.10667E+01_JPRB, 0.10703E+01_JPRB, 0.10689E+01_JPRB, 0.10845E+01_JPRB, 0.10922E+01_JPRB, 0.11343E+01_JPRB, 0.12137E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.12068E+01_JPRB, 0.12486E+01_JPRB, 0.13771E+01_JPRB, 0.14997E+01_JPRB, 0.13962E+01_JPRB, 0.16297E+01_JPRB ,&
&  0.15252E+01_JPRB, 0.16976E+01_JPRB, 0.18120E+01_JPRB, 0.20804E+01_JPRB, 0.21502E+01_JPRB, 0.20619E+01_JPRB, 0.21709E+01_JPRB ,&
&  0.17958E+01_JPRB, 0.16264E+01_JPRB, 0.18043E+01_JPRB, 0.20277E+01_JPRB, 0.21462E+01_JPRB, 0.23277E+01_JPRB, 0.35689E+01_JPRB ,&
&  0.30711E+01_JPRB, 0.34671E+01_JPRB, 0.37603E+01_JPRB, 0.37135E+01_JPRB, 0.18445E+01_JPRB, 0.14580E+01_JPRB, 0.21649E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.16662E+01_JPRB, 0.12176E+01_JPRB, 0.14639E+01_JPRB, 0.11257E+01_JPRB, 0.13622E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.20497E+01_JPRB, 0.23255E+01_JPRB, 0.35588E+01_JPRB, 0.41514E+01_JPRB, 0.42239E+01_JPRB, 0.48499E+01_JPRB, 0.41139E+01_JPRB ,&
&  0.37493E+01_JPRB, 0.31144E+01_JPRB, 0.29538E+01_JPRB, 0.25677E+01_JPRB, 0.22362E+01_JPRB, 0.20038E+01_JPRB, 0.18185E+01_JPRB ,&
&  0.16738E+01_JPRB, 0.15499E+01_JPRB, 0.14566E+01_JPRB, 0.14026E+01_JPRB, 0.13621E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.14644E+01_JPRB, 0.13952E+01_JPRB, 0.13309E+01_JPRB, 0.12711E+01_JPRB, 0.12055E+01_JPRB, 0.11520E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10058E+01_JPRB, 0.95321E+00_JPRB, 0.91414E+00_JPRB, 0.91580E+00_JPRB, 0.92347E+00_JPRB, 0.90982E+00_JPRB ,&
&  0.88777E+00_JPRB, 0.88199E+00_JPRB, 0.82855E+00_JPRB, 0.77579E+00_JPRB, 0.74529E+00_JPRB, 0.73855E+00_JPRB, 0.77532E+00_JPRB ,&
&  0.97761E+00_JPRB, 0.13162E+01_JPRB, 0.15179E+01_JPRB, 0.16730E+01_JPRB, 0.17478E+01_JPRB, 0.16690E+01_JPRB, 0.17539E+01_JPRB ,&
&  0.17529E+01_JPRB, 0.16444E+01_JPRB, 0.14938E+01_JPRB, 0.13461E+01_JPRB, 0.12493E+01_JPRB, 0.11908E+01_JPRB, 0.11581E+01_JPRB ,&
&  0.11519E+01_JPRB, 0.11571E+01_JPRB, 0.11508E+01_JPRB, 0.11279E+01_JPRB, 0.11182E+01_JPRB, 0.11594E+01_JPRB, 0.12348E+01_JPRB ,&
&  0.12496E+01_JPRB, 0.12221E+01_JPRB, 0.12856E+01_JPRB, 0.13893E+01_JPRB, 0.16560E+01_JPRB, 0.17277E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.14123E+01_JPRB, 0.19186E+01_JPRB, 0.20630E+01_JPRB, 0.20540E+01_JPRB, 0.20410E+01_JPRB, 0.19527E+01_JPRB ,&
&  0.19571E+01_JPRB, 0.22910E+01_JPRB, 0.24632E+01_JPRB, 0.28031E+01_JPRB, 0.29225E+01_JPRB, 0.23128E+01_JPRB, 0.31514E+01_JPRB ,&
&  0.34588E+01_JPRB, 0.34658E+01_JPRB, 0.28331E+01_JPRB, 0.20217E+01_JPRB, 0.21317E+01_JPRB, 0.16749E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.10357E+01_JPRB, 0.94310E+00_JPRB, 0.94979E+00_JPRB, 0.92826E+00_JPRB, 0.12710E+01_JPRB, 0.18451E+01_JPRB ,&
&  0.25969E+01_JPRB, 0.33978E+01_JPRB, 0.42586E+01_JPRB, 0.47137E+01_JPRB, 0.45081E+01_JPRB, 0.49734E+01_JPRB, 0.44329E+01_JPRB ,&
&  0.34828E+01_JPRB, 0.30333E+01_JPRB, 0.28281E+01_JPRB, 0.25168E+01_JPRB, 0.22733E+01_JPRB, 0.21073E+01_JPRB, 0.19697E+01_JPRB ,&
&  0.18292E+01_JPRB, 0.17049E+01_JPRB, 0.16186E+01_JPRB, 0.15679E+01_JPRB, 0.15243E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.15703E+01_JPRB, 0.14963E+01_JPRB, 0.14286E+01_JPRB, 0.13694E+01_JPRB, 0.13063E+01_JPRB, 0.12499E+01_JPRB, 0.12031E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.10404E+01_JPRB, 0.98410E+00_JPRB, 0.95024E+00_JPRB, 0.94240E+00_JPRB, 0.94610E+00_JPRB, 0.93867E+00_JPRB ,&
&  0.91979E+00_JPRB, 0.90754E+00_JPRB, 0.90472E+00_JPRB, 0.91967E+00_JPRB, 0.95935E+00_JPRB, 0.10320E+01_JPRB, 0.11807E+01_JPRB ,&
&  0.15258E+01_JPRB, 0.19343E+01_JPRB, 0.20425E+01_JPRB, 0.21096E+01_JPRB, 0.21482E+01_JPRB, 0.21066E+01_JPRB, 0.20534E+01_JPRB ,&
&  0.19581E+01_JPRB, 0.18272E+01_JPRB, 0.16567E+01_JPRB, 0.14770E+01_JPRB, 0.13471E+01_JPRB, 0.12748E+01_JPRB, 0.12417E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.12401E+01_JPRB, 0.12202E+01_JPRB, 0.11776E+01_JPRB, 0.11455E+01_JPRB, 0.11989E+01_JPRB, 0.12648E+01_JPRB ,&
&  0.12763E+01_JPRB, 0.13058E+01_JPRB, 0.13943E+01_JPRB, 0.14442E+01_JPRB, 0.14005E+01_JPRB, 0.11953E+01_JPRB, 0.13024E+01_JPRB ,&
&  0.14476E+01_JPRB, 0.17588E+01_JPRB, 0.18249E+01_JPRB, 0.22138E+01_JPRB, 0.22057E+01_JPRB, 0.20399E+01_JPRB, 0.20561E+01_JPRB ,&
&  0.22480E+01_JPRB, 0.29902E+01_JPRB, 0.26303E+01_JPRB, 0.26685E+01_JPRB, 0.32924E+01_JPRB, 0.27035E+01_JPRB, 0.29085E+01_JPRB ,&
&  0.34348E+01_JPRB, 0.32312E+01_JPRB, 0.27243E+01_JPRB, 0.20707E+01_JPRB, 0.27243E+01_JPRB, 0.28848E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.11576E+01_JPRB, 0.10776E+01_JPRB, 0.10178E+01_JPRB, 0.97149E+00_JPRB, 0.96854E+00_JPRB, 0.14364E+01_JPRB, 0.25864E+01_JPRB ,&
&  0.31233E+01_JPRB, 0.44712E+01_JPRB, 0.51164E+01_JPRB, 0.52607E+01_JPRB, 0.47132E+01_JPRB, 0.41977E+01_JPRB, 0.34934E+01_JPRB ,&
&  0.32396E+01_JPRB, 0.31462E+01_JPRB, 0.28848E+01_JPRB, 0.26809E+01_JPRB, 0.24786E+01_JPRB, 0.22995E+01_JPRB, 0.21345E+01_JPRB ,&
&  0.19689E+01_JPRB, 0.18474E+01_JPRB, 0.17558E+01_JPRB, 0.17149E+01_JPRB, 0.16471E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.16588E+01_JPRB, 0.15970E+01_JPRB, 0.15066E+01_JPRB, 0.14382E+01_JPRB, 0.13702E+01_JPRB, 0.12967E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.10897E+01_JPRB, 0.10251E+01_JPRB, 0.99833E+00_JPRB, 0.98405E+00_JPRB, 0.96790E+00_JPRB, 0.95863E+00_JPRB ,&
&  0.94400E+00_JPRB, 0.94065E+00_JPRB, 0.97903E+00_JPRB, 0.11841E+01_JPRB, 0.13317E+01_JPRB, 0.14579E+01_JPRB, 0.16578E+01_JPRB ,&
&  0.22619E+01_JPRB, 0.24669E+01_JPRB, 0.23717E+01_JPRB, 0.23344E+01_JPRB, 0.22103E+01_JPRB, 0.21068E+01_JPRB, 0.19644E+01_JPRB ,&
&  0.18394E+01_JPRB, 0.17354E+01_JPRB, 0.16150E+01_JPRB, 0.14874E+01_JPRB, 0.13841E+01_JPRB, 0.13356E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.13050E+01_JPRB, 0.12600E+01_JPRB, 0.11838E+01_JPRB, 0.11585E+01_JPRB, 0.12275E+01_JPRB, 0.12958E+01_JPRB ,&
&  0.13236E+01_JPRB, 0.14154E+01_JPRB, 0.14781E+01_JPRB, 0.14538E+01_JPRB, 0.12219E+01_JPRB, 0.13538E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.14708E+01_JPRB, 0.15452E+01_JPRB, 0.15318E+01_JPRB, 0.18448E+01_JPRB, 0.20974E+01_JPRB, 0.19980E+01_JPRB, 0.19670E+01_JPRB ,&
&  0.25473E+01_JPRB, 0.25845E+01_JPRB, 0.24105E+01_JPRB, 0.25602E+01_JPRB, 0.28800E+01_JPRB, 0.36629E+01_JPRB, 0.24943E+01_JPRB ,&
&  0.22932E+01_JPRB, 0.27557E+01_JPRB, 0.24287E+01_JPRB, 0.19452E+01_JPRB, 0.25075E+01_JPRB, 0.36189E+01_JPRB, 0.29622E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.10960E+01_JPRB, 0.11877E+01_JPRB, 0.13292E+01_JPRB, 0.12280E+01_JPRB, 0.18640E+01_JPRB, 0.47297E+01_JPRB ,&
&  0.45546E+01_JPRB, 0.56506E+01_JPRB, 0.55888E+01_JPRB, 0.52053E+01_JPRB, 0.46198E+01_JPRB, 0.39541E+01_JPRB, 0.35707E+01_JPRB ,&
&  0.32919E+01_JPRB, 0.31582E+01_JPRB, 0.30359E+01_JPRB, 0.28953E+01_JPRB, 0.27092E+01_JPRB, 0.24610E+01_JPRB, 0.22372E+01_JPRB ,&
&  0.20645E+01_JPRB, 0.19421E+01_JPRB, 0.18394E+01_JPRB, 0.17694E+01_JPRB, 0.17057E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.17581E+01_JPRB, 0.16531E+01_JPRB, 0.15458E+01_JPRB, 0.14751E+01_JPRB, 0.13939E+01_JPRB, 0.13155E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.11555E+01_JPRB, 0.10763E+01_JPRB, 0.10414E+01_JPRB, 0.10182E+01_JPRB, 0.99420E+00_JPRB, 0.98440E+00_JPRB ,&
&  0.97841E+00_JPRB, 0.98950E+00_JPRB, 0.10222E+01_JPRB, 0.12049E+01_JPRB, 0.14084E+01_JPRB, 0.14006E+01_JPRB, 0.17154E+01_JPRB ,&
&  0.25718E+01_JPRB, 0.26883E+01_JPRB, 0.24978E+01_JPRB, 0.23753E+01_JPRB, 0.22152E+01_JPRB, 0.21360E+01_JPRB, 0.19510E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.15949E+01_JPRB, 0.14766E+01_JPRB, 0.14356E+01_JPRB, 0.14099E+01_JPRB, 0.13808E+01_JPRB, 0.13640E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.12781E+01_JPRB, 0.12167E+01_JPRB, 0.11464E+01_JPRB, 0.11739E+01_JPRB, 0.12693E+01_JPRB, 0.13559E+01_JPRB ,&
&  0.14280E+01_JPRB, 0.15202E+01_JPRB, 0.15244E+01_JPRB, 0.14031E+01_JPRB, 0.13618E+01_JPRB, 0.13613E+01_JPRB, 0.14095E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.14696E+01_JPRB, 0.15831E+01_JPRB, 0.17227E+01_JPRB, 0.19050E+01_JPRB, 0.19405E+01_JPRB, 0.19783E+01_JPRB ,&
&  0.23228E+01_JPRB, 0.23363E+01_JPRB, 0.21904E+01_JPRB, 0.24439E+01_JPRB, 0.27211E+01_JPRB, 0.33877E+01_JPRB, 0.33782E+01_JPRB ,&
&  0.27798E+01_JPRB, 0.21987E+01_JPRB, 0.20244E+01_JPRB, 0.21031E+01_JPRB, 0.24568E+01_JPRB, 0.33878E+01_JPRB, 0.51329E+01_JPRB ,&
&  0.52114E+01_JPRB, 0.34137E+01_JPRB, 0.28083E+01_JPRB, 0.37976E+01_JPRB, 0.32511E+01_JPRB, 0.30728E+01_JPRB, 0.41458E+01_JPRB ,&
&  0.51737E+01_JPRB, 0.59410E+01_JPRB, 0.57678E+01_JPRB, 0.52342E+01_JPRB, 0.47169E+01_JPRB, 0.40347E+01_JPRB, 0.37193E+01_JPRB ,&
&  0.34962E+01_JPRB, 0.32886E+01_JPRB, 0.32272E+01_JPRB, 0.30898E+01_JPRB, 0.28335E+01_JPRB, 0.25180E+01_JPRB, 0.22396E+01_JPRB ,&
&  0.20724E+01_JPRB, 0.19723E+01_JPRB, 0.19022E+01_JPRB, 0.18580E+01_JPRB, 0.18167E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.17757E+01_JPRB, 0.16388E+01_JPRB, 0.15396E+01_JPRB, 0.14604E+01_JPRB, 0.13714E+01_JPRB, 0.13182E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.13304E+01_JPRB, 0.12092E+01_JPRB, 0.11358E+01_JPRB, 0.10756E+01_JPRB, 0.10347E+01_JPRB, 0.10173E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.10065E+01_JPRB, 0.10199E+01_JPRB, 0.10549E+01_JPRB, 0.12521E+01_JPRB, 0.16047E+01_JPRB, 0.21563E+01_JPRB, 0.17236E+01_JPRB ,&
&  0.25560E+01_JPRB, 0.30525E+01_JPRB, 0.27352E+01_JPRB, 0.24208E+01_JPRB, 0.21695E+01_JPRB, 0.21831E+01_JPRB, 0.19392E+01_JPRB ,&
&  0.16381E+01_JPRB, 0.14717E+01_JPRB, 0.13859E+01_JPRB, 0.13530E+01_JPRB, 0.13514E+01_JPRB, 0.13315E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.12057E+01_JPRB, 0.11652E+01_JPRB, 0.11591E+01_JPRB, 0.12186E+01_JPRB, 0.13194E+01_JPRB, 0.14354E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.16115E+01_JPRB, 0.15739E+01_JPRB, 0.14296E+01_JPRB, 0.14086E+01_JPRB, 0.13919E+01_JPRB, 0.13896E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.12987E+01_JPRB, 0.14934E+01_JPRB, 0.17087E+01_JPRB, 0.18266E+01_JPRB, 0.17914E+01_JPRB, 0.18120E+01_JPRB ,&
&  0.22041E+01_JPRB, 0.25345E+01_JPRB, 0.27987E+01_JPRB, 0.27283E+01_JPRB, 0.30330E+01_JPRB, 0.34127E+01_JPRB, 0.37169E+01_JPRB ,&
&  0.35729E+01_JPRB, 0.27558E+01_JPRB, 0.24286E+01_JPRB, 0.24013E+01_JPRB, 0.24669E+01_JPRB, 0.32793E+01_JPRB, 0.46693E+01_JPRB ,&
&  0.61997E+01_JPRB, 0.84295E+01_JPRB, 0.84109E+01_JPRB, 0.65980E+01_JPRB, 0.56893E+01_JPRB, 0.50661E+01_JPRB, 0.46874E+01_JPRB ,&
&  0.59239E+01_JPRB, 0.61307E+01_JPRB, 0.58299E+01_JPRB, 0.64497E+01_JPRB, 0.50022E+01_JPRB, 0.41967E+01_JPRB, 0.37888E+01_JPRB ,&
&  0.36340E+01_JPRB, 0.34302E+01_JPRB, 0.32322E+01_JPRB, 0.30084E+01_JPRB, 0.26685E+01_JPRB, 0.23611E+01_JPRB, 0.21603E+01_JPRB ,&
&  0.20441E+01_JPRB, 0.19786E+01_JPRB, 0.19300E+01_JPRB, 0.19046E+01_JPRB, 0.18706E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.17023E+01_JPRB, 0.15755E+01_JPRB, 0.14463E+01_JPRB, 0.13816E+01_JPRB, 0.13250E+01_JPRB, 0.13143E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.12521E+01_JPRB, 0.11484E+01_JPRB, 0.10705E+01_JPRB, 0.10371E+01_JPRB, 0.10353E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.10430E+01_JPRB, 0.10726E+01_JPRB, 0.11677E+01_JPRB, 0.13248E+01_JPRB, 0.16798E+01_JPRB, 0.21092E+01_JPRB, 0.22491E+01_JPRB ,&
&  0.27821E+01_JPRB, 0.39699E+01_JPRB, 0.29858E+01_JPRB, 0.24318E+01_JPRB, 0.22847E+01_JPRB, 0.25665E+01_JPRB, 0.21368E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.15072E+01_JPRB, 0.14635E+01_JPRB, 0.13450E+01_JPRB, 0.13065E+01_JPRB, 0.12689E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.11950E+01_JPRB, 0.11705E+01_JPRB, 0.12041E+01_JPRB, 0.12888E+01_JPRB, 0.14030E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.17211E+01_JPRB, 0.16741E+01_JPRB, 0.15482E+01_JPRB, 0.14773E+01_JPRB, 0.14461E+01_JPRB, 0.14479E+01_JPRB, 0.14499E+01_JPRB ,&
&  0.14133E+01_JPRB, 0.13667E+01_JPRB, 0.14825E+01_JPRB, 0.15462E+01_JPRB, 0.15841E+01_JPRB, 0.17196E+01_JPRB, 0.17719E+01_JPRB ,&
&  0.22106E+01_JPRB, 0.23834E+01_JPRB, 0.30667E+01_JPRB, 0.28470E+01_JPRB, 0.31967E+01_JPRB, 0.34993E+01_JPRB, 0.33144E+01_JPRB ,&
&  0.27902E+01_JPRB, 0.25480E+01_JPRB, 0.22955E+01_JPRB, 0.23884E+01_JPRB, 0.28828E+01_JPRB, 0.39642E+01_JPRB, 0.50510E+01_JPRB ,&
&  0.62556E+01_JPRB, 0.74394E+01_JPRB, 0.83364E+01_JPRB, 0.66863E+01_JPRB, 0.59811E+01_JPRB, 0.55971E+01_JPRB, 0.56080E+01_JPRB ,&
&  0.63987E+01_JPRB, 0.64768E+01_JPRB, 0.65133E+01_JPRB, 0.61236E+01_JPRB, 0.46686E+01_JPRB, 0.39401E+01_JPRB, 0.35854E+01_JPRB ,&
&  0.33788E+01_JPRB, 0.31724E+01_JPRB, 0.28343E+01_JPRB, 0.25604E+01_JPRB, 0.23583E+01_JPRB, 0.22009E+01_JPRB, 0.20515E+01_JPRB ,&
&  0.19630E+01_JPRB, 0.18946E+01_JPRB, 0.18634E+01_JPRB, 0.18529E+01_JPRB, 0.18043E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.15532E+01_JPRB, 0.14580E+01_JPRB, 0.13903E+01_JPRB, 0.13634E+01_JPRB, 0.13254E+01_JPRB, 0.13426E+01_JPRB, 0.13952E+01_JPRB ,&
&  0.15293E+01_JPRB, 0.12155E+01_JPRB, 0.11076E+01_JPRB, 0.10666E+01_JPRB, 0.10529E+01_JPRB, 0.10462E+01_JPRB, 0.10579E+01_JPRB ,&
&  0.10824E+01_JPRB, 0.11628E+01_JPRB, 0.13097E+01_JPRB, 0.14976E+01_JPRB, 0.18643E+01_JPRB, 0.24130E+01_JPRB, 0.29893E+01_JPRB ,&
&  0.35373E+01_JPRB, 0.34352E+01_JPRB, 0.29013E+01_JPRB, 0.23832E+01_JPRB, 0.22233E+01_JPRB, 0.24282E+01_JPRB, 0.23407E+01_JPRB ,&
&  0.19691E+01_JPRB, 0.16549E+01_JPRB, 0.15201E+01_JPRB, 0.13769E+01_JPRB, 0.12827E+01_JPRB, 0.12656E+01_JPRB, 0.12395E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.11726E+01_JPRB, 0.11750E+01_JPRB, 0.12530E+01_JPRB, 0.13650E+01_JPRB, 0.15342E+01_JPRB, 0.17833E+01_JPRB ,&
&  0.18254E+01_JPRB, 0.17121E+01_JPRB, 0.16082E+01_JPRB, 0.15003E+01_JPRB, 0.14885E+01_JPRB, 0.14921E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.14747E+01_JPRB, 0.15089E+01_JPRB, 0.15518E+01_JPRB, 0.15652E+01_JPRB, 0.15420E+01_JPRB, 0.17588E+01_JPRB, 0.18912E+01_JPRB ,&
&  0.22201E+01_JPRB, 0.25770E+01_JPRB, 0.33400E+01_JPRB, 0.27660E+01_JPRB, 0.33062E+01_JPRB, 0.33845E+01_JPRB, 0.30721E+01_JPRB ,&
&  0.26224E+01_JPRB, 0.24216E+01_JPRB, 0.23985E+01_JPRB, 0.26530E+01_JPRB, 0.35525E+01_JPRB, 0.47155E+01_JPRB, 0.67725E+01_JPRB ,&
&  0.67985E+01_JPRB, 0.76170E+01_JPRB, 0.71574E+01_JPRB, 0.70081E+01_JPRB, 0.61523E+01_JPRB, 0.58343E+01_JPRB, 0.57149E+01_JPRB ,&
&  0.61222E+01_JPRB, 0.59900E+01_JPRB, 0.54574E+01_JPRB, 0.48317E+01_JPRB, 0.38743E+01_JPRB, 0.33002E+01_JPRB, 0.30257E+01_JPRB ,&
&  0.27479E+01_JPRB, 0.24744E+01_JPRB, 0.22646E+01_JPRB, 0.21765E+01_JPRB, 0.20763E+01_JPRB, 0.19692E+01_JPRB, 0.18882E+01_JPRB ,&
&  0.18413E+01_JPRB, 0.17988E+01_JPRB, 0.17656E+01_JPRB, 0.17233E+01_JPRB, 0.16587E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.14163E+01_JPRB, 0.13591E+01_JPRB, 0.13274E+01_JPRB, 0.12826E+01_JPRB, 0.12388E+01_JPRB, 0.12288E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.11099E+01_JPRB, 0.10902E+01_JPRB, 0.10860E+01_JPRB, 0.10709E+01_JPRB, 0.10559E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.12491E+01_JPRB, 0.14141E+01_JPRB, 0.16791E+01_JPRB, 0.20256E+01_JPRB, 0.25531E+01_JPRB, 0.31384E+01_JPRB ,&
&  0.32240E+01_JPRB, 0.22830E+01_JPRB, 0.24912E+01_JPRB, 0.23735E+01_JPRB, 0.21528E+01_JPRB, 0.22528E+01_JPRB, 0.22130E+01_JPRB ,&
&  0.17818E+01_JPRB, 0.16356E+01_JPRB, 0.16026E+01_JPRB, 0.15110E+01_JPRB, 0.12813E+01_JPRB, 0.12562E+01_JPRB, 0.12200E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.11867E+01_JPRB, 0.12275E+01_JPRB, 0.13643E+01_JPRB, 0.15447E+01_JPRB, 0.18125E+01_JPRB, 0.19551E+01_JPRB ,&
&  0.18753E+01_JPRB, 0.18091E+01_JPRB, 0.17670E+01_JPRB, 0.16489E+01_JPRB, 0.16220E+01_JPRB, 0.16313E+01_JPRB, 0.16300E+01_JPRB ,&
&  0.16216E+01_JPRB, 0.16154E+01_JPRB, 0.16796E+01_JPRB, 0.18027E+01_JPRB, 0.19201E+01_JPRB, 0.18670E+01_JPRB, 0.21195E+01_JPRB ,&
&  0.23944E+01_JPRB, 0.29092E+01_JPRB, 0.32776E+01_JPRB, 0.38035E+01_JPRB, 0.27871E+01_JPRB, 0.30233E+01_JPRB, 0.25453E+01_JPRB ,&
&  0.26095E+01_JPRB, 0.26429E+01_JPRB, 0.30024E+01_JPRB, 0.35802E+01_JPRB, 0.48657E+01_JPRB, 0.60751E+01_JPRB, 0.79188E+01_JPRB ,&
&  0.69262E+01_JPRB, 0.65702E+01_JPRB, 0.62408E+01_JPRB, 0.66933E+01_JPRB, 0.62421E+01_JPRB, 0.63252E+01_JPRB, 0.56737E+01_JPRB ,&
&  0.55167E+01_JPRB, 0.48713E+01_JPRB, 0.42433E+01_JPRB, 0.38384E+01_JPRB, 0.30870E+01_JPRB, 0.24953E+01_JPRB, 0.21632E+01_JPRB ,&
&  0.19770E+01_JPRB, 0.19214E+01_JPRB, 0.18895E+01_JPRB, 0.18648E+01_JPRB, 0.18136E+01_JPRB, 0.17650E+01_JPRB, 0.17300E+01_JPRB ,&
&  0.16973E+01_JPRB, 0.16534E+01_JPRB, 0.16126E+01_JPRB, 0.15660E+01_JPRB, 0.15016E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.13200E+01_JPRB, 0.12746E+01_JPRB, 0.12479E+01_JPRB, 0.12096E+01_JPRB, 0.11801E+01_JPRB, 0.11672E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11161E+01_JPRB, 0.11107E+01_JPRB, 0.10927E+01_JPRB, 0.10631E+01_JPRB, 0.10610E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.13989E+01_JPRB, 0.16017E+01_JPRB, 0.18182E+01_JPRB, 0.21525E+01_JPRB, 0.24347E+01_JPRB, 0.25673E+01_JPRB ,&
&  0.27522E+01_JPRB, 0.28016E+01_JPRB, 0.23684E+01_JPRB, 0.17174E+01_JPRB, 0.16020E+01_JPRB, 0.19269E+01_JPRB, 0.17260E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.14676E+01_JPRB, 0.14259E+01_JPRB, 0.13768E+01_JPRB, 0.12970E+01_JPRB, 0.12634E+01_JPRB, 0.12466E+01_JPRB ,&
&  0.12465E+01_JPRB, 0.12887E+01_JPRB, 0.14181E+01_JPRB, 0.16662E+01_JPRB, 0.19123E+01_JPRB, 0.20845E+01_JPRB, 0.20809E+01_JPRB ,&
&  0.20594E+01_JPRB, 0.20891E+01_JPRB, 0.20786E+01_JPRB, 0.19616E+01_JPRB, 0.18738E+01_JPRB, 0.18888E+01_JPRB, 0.19057E+01_JPRB ,&
&  0.19160E+01_JPRB, 0.19225E+01_JPRB, 0.19388E+01_JPRB, 0.20518E+01_JPRB, 0.21434E+01_JPRB, 0.20443E+01_JPRB, 0.21899E+01_JPRB ,&
&  0.26607E+01_JPRB, 0.33582E+01_JPRB, 0.34002E+01_JPRB, 0.40673E+01_JPRB, 0.24395E+01_JPRB, 0.27203E+01_JPRB, 0.28919E+01_JPRB ,&
&  0.28217E+01_JPRB, 0.33145E+01_JPRB, 0.38587E+01_JPRB, 0.46268E+01_JPRB, 0.58341E+01_JPRB, 0.65970E+01_JPRB, 0.72864E+01_JPRB ,&
&  0.60945E+01_JPRB, 0.54245E+01_JPRB, 0.53624E+01_JPRB, 0.56849E+01_JPRB, 0.57850E+01_JPRB, 0.56951E+01_JPRB, 0.46881E+01_JPRB ,&
&  0.38848E+01_JPRB, 0.38862E+01_JPRB, 0.33716E+01_JPRB, 0.31870E+01_JPRB, 0.23067E+01_JPRB, 0.18176E+01_JPRB, 0.17438E+01_JPRB ,&
&  0.16854E+01_JPRB, 0.16401E+01_JPRB, 0.16314E+01_JPRB, 0.16172E+01_JPRB, 0.15989E+01_JPRB, 0.15810E+01_JPRB, 0.15505E+01_JPRB ,&
&  0.15237E+01_JPRB, 0.15035E+01_JPRB, 0.14761E+01_JPRB, 0.14372E+01_JPRB, 0.13837E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.12454E+01_JPRB, 0.12133E+01_JPRB, 0.12004E+01_JPRB, 0.11549E+01_JPRB, 0.11143E+01_JPRB, 0.11039E+01_JPRB, 0.11136E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.10932E+01_JPRB, 0.10750E+01_JPRB, 0.10607E+01_JPRB, 0.10743E+01_JPRB, 0.11292E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.13522E+01_JPRB, 0.15252E+01_JPRB, 0.17311E+01_JPRB, 0.19375E+01_JPRB, 0.22095E+01_JPRB, 0.23258E+01_JPRB, 0.23187E+01_JPRB ,&
&  0.23941E+01_JPRB, 0.24960E+01_JPRB, 0.24551E+01_JPRB, 0.24186E+01_JPRB, 0.20206E+01_JPRB, 0.21961E+01_JPRB, 0.22551E+01_JPRB ,&
&  0.21452E+01_JPRB, 0.17340E+01_JPRB, 0.14557E+01_JPRB, 0.14015E+01_JPRB, 0.13518E+01_JPRB, 0.13454E+01_JPRB, 0.13823E+01_JPRB ,&
&  0.14601E+01_JPRB, 0.16013E+01_JPRB, 0.18258E+01_JPRB, 0.20731E+01_JPRB, 0.22390E+01_JPRB, 0.23133E+01_JPRB, 0.23391E+01_JPRB ,&
&  0.23807E+01_JPRB, 0.24385E+01_JPRB, 0.24972E+01_JPRB, 0.24224E+01_JPRB, 0.23964E+01_JPRB, 0.23904E+01_JPRB, 0.24596E+01_JPRB ,&
&  0.25299E+01_JPRB, 0.24305E+01_JPRB, 0.24402E+01_JPRB, 0.25979E+01_JPRB, 0.26289E+01_JPRB, 0.26069E+01_JPRB, 0.27252E+01_JPRB ,&
&  0.32029E+01_JPRB, 0.39946E+01_JPRB, 0.32175E+01_JPRB, 0.33923E+01_JPRB, 0.39544E+01_JPRB, 0.32779E+01_JPRB, 0.29588E+01_JPRB ,&
&  0.32569E+01_JPRB, 0.37387E+01_JPRB, 0.42523E+01_JPRB, 0.49502E+01_JPRB, 0.60134E+01_JPRB, 0.73659E+01_JPRB, 0.60960E+01_JPRB ,&
&  0.50958E+01_JPRB, 0.44721E+01_JPRB, 0.45100E+01_JPRB, 0.46129E+01_JPRB, 0.45842E+01_JPRB, 0.44041E+01_JPRB, 0.36734E+01_JPRB ,&
&  0.31824E+01_JPRB, 0.31809E+01_JPRB, 0.28915E+01_JPRB, 0.25611E+01_JPRB, 0.22870E+01_JPRB, 0.16555E+01_JPRB, 0.15629E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.14612E+01_JPRB, 0.14739E+01_JPRB, 0.14746E+01_JPRB, 0.14622E+01_JPRB, 0.14352E+01_JPRB, 0.14121E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.13787E+01_JPRB, 0.13602E+01_JPRB, 0.13356E+01_JPRB, 0.12900E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.11882E+01_JPRB, 0.11676E+01_JPRB, 0.11496E+01_JPRB, 0.11114E+01_JPRB, 0.10697E+01_JPRB, 0.10595E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10041E+01_JPRB, 0.97372E+00_JPRB, 0.98966E+00_JPRB, 0.10324E+01_JPRB, 0.11153E+01_JPRB, 0.12103E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.14722E+01_JPRB, 0.16868E+01_JPRB, 0.18658E+01_JPRB, 0.21199E+01_JPRB, 0.22661E+01_JPRB, 0.23599E+01_JPRB ,&
&  0.24696E+01_JPRB, 0.25957E+01_JPRB, 0.26994E+01_JPRB, 0.27755E+01_JPRB, 0.27982E+01_JPRB, 0.28853E+01_JPRB, 0.30813E+01_JPRB ,&
&  0.25944E+01_JPRB, 0.18864E+01_JPRB, 0.16467E+01_JPRB, 0.15181E+01_JPRB, 0.15051E+01_JPRB, 0.15484E+01_JPRB, 0.16473E+01_JPRB ,&
&  0.18038E+01_JPRB, 0.19842E+01_JPRB, 0.22114E+01_JPRB, 0.23902E+01_JPRB, 0.25344E+01_JPRB, 0.26402E+01_JPRB, 0.27150E+01_JPRB ,&
&  0.28391E+01_JPRB, 0.29719E+01_JPRB, 0.31607E+01_JPRB, 0.30463E+01_JPRB, 0.33328E+01_JPRB, 0.33251E+01_JPRB, 0.33488E+01_JPRB ,&
&  0.32618E+01_JPRB, 0.30490E+01_JPRB, 0.31491E+01_JPRB, 0.33098E+01_JPRB, 0.34737E+01_JPRB, 0.35114E+01_JPRB, 0.35030E+01_JPRB ,&
&  0.39721E+01_JPRB, 0.41312E+01_JPRB, 0.31291E+01_JPRB, 0.27773E+01_JPRB, 0.28931E+01_JPRB, 0.25226E+01_JPRB, 0.30712E+01_JPRB ,&
&  0.32810E+01_JPRB, 0.36591E+01_JPRB, 0.40480E+01_JPRB, 0.46551E+01_JPRB, 0.53626E+01_JPRB, 0.58748E+01_JPRB, 0.50903E+01_JPRB ,&
&  0.39409E+01_JPRB, 0.35020E+01_JPRB, 0.34704E+01_JPRB, 0.34078E+01_JPRB, 0.33616E+01_JPRB, 0.32959E+01_JPRB, 0.30966E+01_JPRB ,&
&  0.28641E+01_JPRB, 0.26046E+01_JPRB, 0.24053E+01_JPRB, 0.19169E+01_JPRB, 0.16318E+01_JPRB, 0.14422E+01_JPRB, 0.13339E+01_JPRB ,&
&  0.13155E+01_JPRB, 0.13544E+01_JPRB, 0.13724E+01_JPRB, 0.13756E+01_JPRB, 0.13683E+01_JPRB, 0.13402E+01_JPRB, 0.13223E+01_JPRB ,&
&  0.13133E+01_JPRB, 0.12954E+01_JPRB, 0.12765E+01_JPRB, 0.12591E+01_JPRB, 0.12218E+01_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.10974E+01_JPRB, 0.10669E+01_JPRB, 0.10446E+01_JPRB, 0.10198E+01_JPRB, 0.98064E+00_JPRB, 0.95328E+00_JPRB, 0.94302E+00_JPRB ,&
&  0.89978E+00_JPRB, 0.89862E+00_JPRB, 0.87982E+00_JPRB, 0.89410E+00_JPRB, 0.93861E+00_JPRB, 0.98302E+00_JPRB, 0.10463E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.11710E+01_JPRB, 0.12967E+01_JPRB, 0.15621E+01_JPRB, 0.18676E+01_JPRB, 0.21123E+01_JPRB, 0.24037E+01_JPRB ,&
&  0.25521E+01_JPRB, 0.26886E+01_JPRB, 0.29237E+01_JPRB, 0.32490E+01_JPRB, 0.35118E+01_JPRB, 0.37625E+01_JPRB, 0.37629E+01_JPRB ,&
&  0.28439E+01_JPRB, 0.17792E+01_JPRB, 0.17139E+01_JPRB, 0.15808E+01_JPRB, 0.16449E+01_JPRB, 0.17471E+01_JPRB, 0.18598E+01_JPRB ,&
&  0.20368E+01_JPRB, 0.22210E+01_JPRB, 0.24134E+01_JPRB, 0.25730E+01_JPRB, 0.27523E+01_JPRB, 0.29424E+01_JPRB, 0.30552E+01_JPRB ,&
&  0.31685E+01_JPRB, 0.32816E+01_JPRB, 0.33710E+01_JPRB, 0.33053E+01_JPRB, 0.34934E+01_JPRB, 0.37364E+01_JPRB, 0.39370E+01_JPRB ,&
&  0.41313E+01_JPRB, 0.41597E+01_JPRB, 0.35255E+01_JPRB, 0.36662E+01_JPRB, 0.41810E+01_JPRB, 0.39325E+01_JPRB, 0.40653E+01_JPRB ,&
&  0.47508E+01_JPRB, 0.49175E+01_JPRB, 0.28933E+01_JPRB, 0.25302E+01_JPRB, 0.26307E+01_JPRB, 0.26893E+01_JPRB, 0.28646E+01_JPRB ,&
&  0.30294E+01_JPRB, 0.33572E+01_JPRB, 0.37175E+01_JPRB, 0.40267E+01_JPRB, 0.41814E+01_JPRB, 0.39513E+01_JPRB, 0.36643E+01_JPRB ,&
&  0.29935E+01_JPRB, 0.26874E+01_JPRB, 0.26849E+01_JPRB, 0.27211E+01_JPRB, 0.28300E+01_JPRB, 0.29459E+01_JPRB, 0.26324E+01_JPRB ,&
&  0.25363E+01_JPRB, 0.25213E+01_JPRB, 0.27624E+01_JPRB, 0.22093E+01_JPRB, 0.15237E+01_JPRB, 0.12936E+01_JPRB, 0.11906E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.12888E+01_JPRB, 0.13107E+01_JPRB, 0.13259E+01_JPRB, 0.13224E+01_JPRB, 0.13078E+01_JPRB, 0.12884E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.12341E+01_JPRB, 0.12021E+01_JPRB, 0.11681E+01_JPRB, 0.11355E+01_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.94932E+00_JPRB, 0.92033E+00_JPRB, 0.90760E+00_JPRB, 0.85518E+00_JPRB, 0.86713E+00_JPRB, 0.82903E+00_JPRB, 0.86767E+00_JPRB ,&
&  0.84173E+00_JPRB, 0.84198E+00_JPRB, 0.84540E+00_JPRB, 0.86607E+00_JPRB, 0.88623E+00_JPRB, 0.91007E+00_JPRB, 0.93092E+00_JPRB ,&
&  0.95660E+00_JPRB, 0.10137E+01_JPRB, 0.10993E+01_JPRB, 0.12044E+01_JPRB, 0.13430E+01_JPRB, 0.14887E+01_JPRB, 0.16250E+01_JPRB ,&
&  0.18056E+01_JPRB, 0.19708E+01_JPRB, 0.20894E+01_JPRB, 0.23483E+01_JPRB, 0.29749E+01_JPRB, 0.37454E+01_JPRB, 0.38853E+01_JPRB ,&
&  0.25139E+01_JPRB, 0.18362E+01_JPRB, 0.16049E+01_JPRB, 0.14943E+01_JPRB, 0.15984E+01_JPRB, 0.18182E+01_JPRB, 0.19213E+01_JPRB ,&
&  0.20379E+01_JPRB, 0.21800E+01_JPRB, 0.23431E+01_JPRB, 0.25208E+01_JPRB, 0.27351E+01_JPRB, 0.29399E+01_JPRB, 0.30545E+01_JPRB ,&
&  0.30828E+01_JPRB, 0.31527E+01_JPRB, 0.31768E+01_JPRB, 0.33931E+01_JPRB, 0.38236E+01_JPRB, 0.43298E+01_JPRB, 0.46989E+01_JPRB ,&
&  0.48962E+01_JPRB, 0.48256E+01_JPRB, 0.41268E+01_JPRB, 0.42097E+01_JPRB, 0.44544E+01_JPRB, 0.43936E+01_JPRB, 0.42631E+01_JPRB ,&
&  0.47231E+01_JPRB, 0.45235E+01_JPRB, 0.30224E+01_JPRB, 0.27822E+01_JPRB, 0.26993E+01_JPRB, 0.26543E+01_JPRB, 0.25312E+01_JPRB ,&
&  0.27681E+01_JPRB, 0.30366E+01_JPRB, 0.32157E+01_JPRB, 0.32387E+01_JPRB, 0.31059E+01_JPRB, 0.28485E+01_JPRB, 0.27854E+01_JPRB ,&
&  0.25217E+01_JPRB, 0.24125E+01_JPRB, 0.24371E+01_JPRB, 0.25286E+01_JPRB, 0.29496E+01_JPRB, 0.26717E+01_JPRB, 0.25813E+01_JPRB ,&
&  0.22432E+01_JPRB, 0.22284E+01_JPRB, 0.18600E+01_JPRB, 0.21220E+01_JPRB, 0.13773E+01_JPRB, 0.10758E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.11537E+01_JPRB, 0.11568E+01_JPRB, 0.11774E+01_JPRB, 0.11839E+01_JPRB, 0.11690E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.11212E+01_JPRB, 0.10760E+01_JPRB, 0.10393E+01_JPRB, 0.99956E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.90190E+00_JPRB, 0.89814E+00_JPRB, 0.87052E+00_JPRB, 0.82480E+00_JPRB, 0.79627E+00_JPRB, 0.77371E+00_JPRB, 0.78346E+00_JPRB ,&
&  0.79494E+00_JPRB, 0.82222E+00_JPRB, 0.84715E+00_JPRB, 0.87157E+00_JPRB, 0.88280E+00_JPRB, 0.89065E+00_JPRB, 0.91298E+00_JPRB ,&
&  0.94280E+00_JPRB, 0.97049E+00_JPRB, 0.10085E+01_JPRB, 0.10599E+01_JPRB, 0.11143E+01_JPRB, 0.11847E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.12904E+01_JPRB, 0.13451E+01_JPRB, 0.14488E+01_JPRB, 0.18063E+01_JPRB, 0.22051E+01_JPRB, 0.27570E+01_JPRB ,&
&  0.21289E+01_JPRB, 0.19911E+01_JPRB, 0.17197E+01_JPRB, 0.15960E+01_JPRB, 0.16514E+01_JPRB, 0.16435E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.17738E+01_JPRB, 0.18284E+01_JPRB, 0.18812E+01_JPRB, 0.19403E+01_JPRB, 0.20029E+01_JPRB, 0.21266E+01_JPRB, 0.20872E+01_JPRB ,&
&  0.20790E+01_JPRB, 0.21742E+01_JPRB, 0.22966E+01_JPRB, 0.26491E+01_JPRB, 0.31703E+01_JPRB, 0.36750E+01_JPRB, 0.37298E+01_JPRB ,&
&  0.37854E+01_JPRB, 0.40655E+01_JPRB, 0.39962E+01_JPRB, 0.40541E+01_JPRB, 0.40153E+01_JPRB, 0.38950E+01_JPRB, 0.39281E+01_JPRB ,&
&  0.37779E+01_JPRB, 0.34899E+01_JPRB, 0.31932E+01_JPRB, 0.28055E+01_JPRB, 0.25584E+01_JPRB, 0.23338E+01_JPRB, 0.23035E+01_JPRB ,&
&  0.24057E+01_JPRB, 0.25615E+01_JPRB, 0.26333E+01_JPRB, 0.25158E+01_JPRB, 0.23891E+01_JPRB, 0.22240E+01_JPRB, 0.22481E+01_JPRB ,&
&  0.22225E+01_JPRB, 0.20724E+01_JPRB, 0.21026E+01_JPRB, 0.23359E+01_JPRB, 0.26336E+01_JPRB, 0.43693E+01_JPRB, 0.37708E+01_JPRB ,&
&  0.21546E+01_JPRB, 0.17510E+01_JPRB, 0.11373E+01_JPRB, 0.16209E+01_JPRB, 0.10399E+01_JPRB, 0.92931E+00_JPRB, 0.95509E+00_JPRB ,&
&  0.97026E+00_JPRB, 0.98799E+00_JPRB, 0.99954E+00_JPRB, 0.10069E+01_JPRB, 0.10098E+01_JPRB, 0.10133E+01_JPRB, 0.99997E+00_JPRB ,&
&  0.98892E+00_JPRB, 0.97053E+00_JPRB, 0.96618E+00_JPRB, 0.92958E+00_JPRB, 0.91280E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.85473E+00_JPRB, 0.85328E+00_JPRB, 0.81655E+00_JPRB, 0.76290E+00_JPRB, 0.73944E+00_JPRB, 0.74281E+00_JPRB, 0.76132E+00_JPRB ,&
&  0.78087E+00_JPRB, 0.80607E+00_JPRB, 0.83579E+00_JPRB, 0.86540E+00_JPRB, 0.89270E+00_JPRB, 0.92264E+00_JPRB, 0.95850E+00_JPRB ,&
&  0.99282E+00_JPRB, 0.10344E+01_JPRB, 0.10781E+01_JPRB, 0.11250E+01_JPRB, 0.11695E+01_JPRB, 0.12440E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.14724E+01_JPRB, 0.15795E+01_JPRB, 0.16965E+01_JPRB, 0.18508E+01_JPRB, 0.20022E+01_JPRB, 0.19115E+01_JPRB ,&
&  0.22756E+01_JPRB, 0.20116E+01_JPRB, 0.17809E+01_JPRB, 0.16491E+01_JPRB, 0.15646E+01_JPRB, 0.15020E+01_JPRB, 0.14398E+01_JPRB ,&
&  0.14081E+01_JPRB, 0.13638E+01_JPRB, 0.13177E+01_JPRB, 0.12790E+01_JPRB, 0.12598E+01_JPRB, 0.13526E+01_JPRB, 0.13783E+01_JPRB ,&
&  0.15057E+01_JPRB, 0.17289E+01_JPRB, 0.19314E+01_JPRB, 0.22670E+01_JPRB, 0.25467E+01_JPRB, 0.27541E+01_JPRB, 0.26449E+01_JPRB ,&
&  0.26874E+01_JPRB, 0.29644E+01_JPRB, 0.29498E+01_JPRB, 0.30553E+01_JPRB, 0.30448E+01_JPRB, 0.29133E+01_JPRB, 0.28410E+01_JPRB ,&
&  0.25753E+01_JPRB, 0.25484E+01_JPRB, 0.26508E+01_JPRB, 0.28777E+01_JPRB, 0.24644E+01_JPRB, 0.21873E+01_JPRB, 0.21908E+01_JPRB ,&
&  0.21410E+01_JPRB, 0.21833E+01_JPRB, 0.21888E+01_JPRB, 0.20536E+01_JPRB, 0.19400E+01_JPRB, 0.18487E+01_JPRB, 0.18947E+01_JPRB ,&
&  0.19203E+01_JPRB, 0.17807E+01_JPRB, 0.18845E+01_JPRB, 0.22371E+01_JPRB, 0.27538E+01_JPRB, 0.30227E+01_JPRB, 0.36446E+01_JPRB ,&
&  0.20954E+01_JPRB, 0.11962E+01_JPRB, 0.11661E+01_JPRB, 0.11721E+01_JPRB, 0.91112E+00_JPRB, 0.78142E+00_JPRB, 0.83100E+00_JPRB ,&
&  0.86520E+00_JPRB, 0.88867E+00_JPRB, 0.87253E+00_JPRB, 0.89881E+00_JPRB, 0.95569E+00_JPRB, 0.95459E+00_JPRB, 0.90615E+00_JPRB ,&
&  0.91756E+00_JPRB, 0.88165E+00_JPRB, 0.86609E+00_JPRB, 0.86402E+00_JPRB, 0.86195E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.79767E+00_JPRB, 0.78431E+00_JPRB, 0.74259E+00_JPRB, 0.69514E+00_JPRB, 0.67697E+00_JPRB, 0.69258E+00_JPRB, 0.72036E+00_JPRB ,&
&  0.74693E+00_JPRB, 0.77802E+00_JPRB, 0.80942E+00_JPRB, 0.84455E+00_JPRB, 0.88130E+00_JPRB, 0.91727E+00_JPRB, 0.94581E+00_JPRB ,&
&  0.96935E+00_JPRB, 0.10002E+01_JPRB, 0.10304E+01_JPRB, 0.10608E+01_JPRB, 0.10863E+01_JPRB, 0.11027E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.11948E+01_JPRB, 0.12751E+01_JPRB, 0.13773E+01_JPRB, 0.14921E+01_JPRB, 0.16024E+01_JPRB, 0.17538E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.20162E+01_JPRB, 0.19406E+01_JPRB, 0.17924E+01_JPRB, 0.16542E+01_JPRB, 0.15314E+01_JPRB, 0.14378E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.12957E+01_JPRB, 0.13531E+01_JPRB, 0.12978E+01_JPRB, 0.13119E+01_JPRB, 0.11491E+01_JPRB, 0.12246E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.15537E+01_JPRB, 0.17981E+01_JPRB, 0.20112E+01_JPRB, 0.22072E+01_JPRB, 0.22977E+01_JPRB, 0.23704E+01_JPRB, 0.22555E+01_JPRB ,&
&  0.22674E+01_JPRB, 0.22776E+01_JPRB, 0.21521E+01_JPRB, 0.22539E+01_JPRB, 0.23553E+01_JPRB, 0.20456E+01_JPRB, 0.18328E+01_JPRB ,&
&  0.16913E+01_JPRB, 0.19487E+01_JPRB, 0.25629E+01_JPRB, 0.28647E+01_JPRB, 0.24246E+01_JPRB, 0.22472E+01_JPRB, 0.21132E+01_JPRB ,&
&  0.19518E+01_JPRB, 0.19244E+01_JPRB, 0.18779E+01_JPRB, 0.17733E+01_JPRB, 0.16129E+01_JPRB, 0.15236E+01_JPRB, 0.15698E+01_JPRB ,&
&  0.16083E+01_JPRB, 0.14987E+01_JPRB, 0.15038E+01_JPRB, 0.18274E+01_JPRB, 0.20911E+01_JPRB, 0.20330E+01_JPRB, 0.34880E+01_JPRB ,&
&  0.20663E+01_JPRB, 0.14652E+01_JPRB, 0.20130E+01_JPRB, 0.76606E+00_JPRB, 0.77836E+00_JPRB, 0.69149E+00_JPRB, 0.69974E+00_JPRB ,&
&  0.59303E+00_JPRB, 0.70132E+00_JPRB, 0.10759E+01_JPRB, 0.13552E+01_JPRB, 0.15469E+01_JPRB, 0.15501E+01_JPRB, 0.95396E+00_JPRB ,&
&  0.86186E+00_JPRB, 0.84530E+00_JPRB, 0.83475E+00_JPRB, 0.81785E+00_JPRB, 0.79810E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.72675E+00_JPRB, 0.68737E+00_JPRB, 0.66459E+00_JPRB, 0.64534E+00_JPRB, 0.63477E+00_JPRB, 0.65625E+00_JPRB, 0.67618E+00_JPRB ,&
&  0.70673E+00_JPRB, 0.74184E+00_JPRB, 0.78313E+00_JPRB, 0.82085E+00_JPRB, 0.86912E+00_JPRB, 0.91879E+00_JPRB, 0.96390E+00_JPRB ,&
&  0.10062E+01_JPRB, 0.10472E+01_JPRB, 0.10907E+01_JPRB, 0.11215E+01_JPRB, 0.11524E+01_JPRB, 0.11812E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.12347E+01_JPRB, 0.13131E+01_JPRB, 0.14344E+01_JPRB, 0.15717E+01_JPRB, 0.16961E+01_JPRB, 0.18931E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.17923E+01_JPRB, 0.17649E+01_JPRB, 0.16579E+01_JPRB, 0.15505E+01_JPRB, 0.14739E+01_JPRB, 0.14360E+01_JPRB ,&
&  0.14328E+01_JPRB, 0.15078E+01_JPRB, 0.14756E+01_JPRB, 0.15564E+01_JPRB, 0.15197E+01_JPRB, 0.14841E+01_JPRB, 0.16761E+01_JPRB ,&
&  0.18310E+01_JPRB, 0.20879E+01_JPRB, 0.22540E+01_JPRB, 0.23790E+01_JPRB, 0.24753E+01_JPRB, 0.25560E+01_JPRB, 0.25923E+01_JPRB ,&
&  0.26146E+01_JPRB, 0.25559E+01_JPRB, 0.24751E+01_JPRB, 0.26212E+01_JPRB, 0.26986E+01_JPRB, 0.23253E+01_JPRB, 0.21305E+01_JPRB ,&
&  0.15821E+01_JPRB, 0.14706E+01_JPRB, 0.22250E+01_JPRB, 0.27446E+01_JPRB, 0.24671E+01_JPRB, 0.22995E+01_JPRB, 0.20467E+01_JPRB ,&
&  0.18003E+01_JPRB, 0.16735E+01_JPRB, 0.15949E+01_JPRB, 0.14980E+01_JPRB, 0.13755E+01_JPRB, 0.12964E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.12724E+01_JPRB, 0.11552E+01_JPRB, 0.11273E+01_JPRB, 0.12901E+01_JPRB, 0.14367E+01_JPRB, 0.17681E+01_JPRB, 0.18963E+01_JPRB ,&
&  0.21737E+01_JPRB, 0.19335E+01_JPRB, 0.14183E+01_JPRB, 0.12384E+01_JPRB, 0.80365E+00_JPRB, 0.66498E+00_JPRB, 0.61889E+00_JPRB ,&
&  0.62925E+00_JPRB, 0.56785E+00_JPRB, 0.10683E+01_JPRB, 0.38832E+01_JPRB, 0.37989E+01_JPRB, 0.47842E+01_JPRB, 0.25517E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.83840E+00_JPRB, 0.75490E+00_JPRB, 0.75285E+00_JPRB, 0.73902E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.63276E+00_JPRB, 0.60750E+00_JPRB, 0.58064E+00_JPRB, 0.58643E+00_JPRB, 0.59420E+00_JPRB, 0.60520E+00_JPRB, 0.63837E+00_JPRB ,&
&  0.70514E+00_JPRB, 0.72429E+00_JPRB, 0.75675E+00_JPRB, 0.81146E+00_JPRB, 0.89244E+00_JPRB, 0.96847E+00_JPRB, 0.10431E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11898E+01_JPRB, 0.12384E+01_JPRB, 0.12729E+01_JPRB, 0.13213E+01_JPRB, 0.13749E+01_JPRB, 0.14403E+01_JPRB ,&
&  0.14862E+01_JPRB, 0.15687E+01_JPRB, 0.17069E+01_JPRB, 0.18537E+01_JPRB, 0.19931E+01_JPRB, 0.20505E+01_JPRB, 0.13199E+01_JPRB ,&
&  0.15135E+01_JPRB, 0.16065E+01_JPRB, 0.16652E+01_JPRB, 0.16495E+01_JPRB, 0.15604E+01_JPRB, 0.14856E+01_JPRB, 0.15307E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.17787E+01_JPRB, 0.17861E+01_JPRB, 0.17617E+01_JPRB, 0.19806E+01_JPRB, 0.19197E+01_JPRB, 0.22821E+01_JPRB ,&
&  0.24731E+01_JPRB, 0.26204E+01_JPRB, 0.28244E+01_JPRB, 0.29453E+01_JPRB, 0.30969E+01_JPRB, 0.32772E+01_JPRB, 0.34480E+01_JPRB ,&
&  0.35335E+01_JPRB, 0.35362E+01_JPRB, 0.36581E+01_JPRB, 0.40603E+01_JPRB, 0.37649E+01_JPRB, 0.42102E+01_JPRB, 0.34337E+01_JPRB ,&
&  0.22924E+01_JPRB, 0.19154E+01_JPRB, 0.23142E+01_JPRB, 0.25969E+01_JPRB, 0.24397E+01_JPRB, 0.23119E+01_JPRB, 0.20697E+01_JPRB ,&
&  0.17266E+01_JPRB, 0.15136E+01_JPRB, 0.14270E+01_JPRB, 0.13462E+01_JPRB, 0.12493E+01_JPRB, 0.11636E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.10273E+01_JPRB, 0.94771E+00_JPRB, 0.97925E+00_JPRB, 0.10162E+01_JPRB, 0.11182E+01_JPRB, 0.15142E+01_JPRB, 0.23358E+01_JPRB ,&
&  0.36695E+01_JPRB, 0.20472E+01_JPRB, 0.14893E+01_JPRB, 0.11826E+01_JPRB, 0.88206E+00_JPRB, 0.73704E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.62443E+00_JPRB, 0.76179E+00_JPRB, 0.89538E+00_JPRB, 0.15638E+01_JPRB, 0.26766E+01_JPRB, 0.27141E+01_JPRB, 0.18607E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.83923E+00_JPRB, 0.68205E+00_JPRB, 0.65472E+00_JPRB, 0.63472E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.57885E+00_JPRB, 0.56413E+00_JPRB, 0.56090E+00_JPRB, 0.55777E+00_JPRB, 0.59745E+00_JPRB, 0.62465E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.69372E+00_JPRB, 0.72344E+00_JPRB, 0.76379E+00_JPRB, 0.84126E+00_JPRB, 0.93230E+00_JPRB, 0.10182E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.11637E+01_JPRB, 0.12021E+01_JPRB, 0.12415E+01_JPRB, 0.13180E+01_JPRB, 0.13914E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.16282E+01_JPRB, 0.17689E+01_JPRB, 0.19335E+01_JPRB, 0.20854E+01_JPRB, 0.22571E+01_JPRB, 0.24187E+01_JPRB, 0.25589E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.14993E+01_JPRB, 0.15712E+01_JPRB, 0.15737E+01_JPRB, 0.15054E+01_JPRB, 0.15296E+01_JPRB, 0.17343E+01_JPRB ,&
&  0.19810E+01_JPRB, 0.20452E+01_JPRB, 0.22590E+01_JPRB, 0.20028E+01_JPRB, 0.23464E+01_JPRB, 0.24489E+01_JPRB, 0.27611E+01_JPRB ,&
&  0.29838E+01_JPRB, 0.31540E+01_JPRB, 0.33720E+01_JPRB, 0.36082E+01_JPRB, 0.38691E+01_JPRB, 0.41577E+01_JPRB, 0.45101E+01_JPRB ,&
&  0.48509E+01_JPRB, 0.52806E+01_JPRB, 0.58356E+01_JPRB, 0.58540E+01_JPRB, 0.54544E+01_JPRB, 0.58600E+01_JPRB, 0.55002E+01_JPRB ,&
&  0.45945E+01_JPRB, 0.32467E+01_JPRB, 0.30821E+01_JPRB, 0.27956E+01_JPRB, 0.24967E+01_JPRB, 0.23412E+01_JPRB, 0.20625E+01_JPRB ,&
&  0.17123E+01_JPRB, 0.15045E+01_JPRB, 0.13647E+01_JPRB, 0.12780E+01_JPRB, 0.11770E+01_JPRB, 0.10958E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.98523E+00_JPRB, 0.99096E+00_JPRB, 0.10078E+01_JPRB, 0.93831E+00_JPRB, 0.91832E+00_JPRB, 0.10539E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.15908E+01_JPRB, 0.13835E+01_JPRB, 0.11652E+01_JPRB, 0.11445E+01_JPRB, 0.10320E+01_JPRB, 0.90900E+00_JPRB, 0.84881E+00_JPRB ,&
&  0.82405E+00_JPRB, 0.79172E+00_JPRB, 0.89365E+00_JPRB, 0.13337E+01_JPRB, 0.16427E+01_JPRB, 0.18502E+01_JPRB, 0.15387E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.85538E+00_JPRB, 0.69491E+00_JPRB, 0.65314E+00_JPRB, 0.63065E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.62224E+00_JPRB, 0.60774E+00_JPRB, 0.60789E+00_JPRB, 0.60391E+00_JPRB, 0.64081E+00_JPRB, 0.68397E+00_JPRB, 0.69748E+00_JPRB ,&
&  0.69861E+00_JPRB, 0.72460E+00_JPRB, 0.75159E+00_JPRB, 0.79728E+00_JPRB, 0.88372E+00_JPRB, 0.94217E+00_JPRB, 0.10093E+01_JPRB ,&
&  0.10756E+01_JPRB, 0.11015E+01_JPRB, 0.11098E+01_JPRB, 0.11501E+01_JPRB, 0.12427E+01_JPRB, 0.13473E+01_JPRB, 0.14801E+01_JPRB ,&
&  0.16461E+01_JPRB, 0.18538E+01_JPRB, 0.21112E+01_JPRB, 0.23957E+01_JPRB, 0.27136E+01_JPRB, 0.30284E+01_JPRB, 0.32070E+01_JPRB ,&
&  0.13295E+01_JPRB, 0.12967E+01_JPRB, 0.15172E+01_JPRB, 0.15056E+01_JPRB, 0.14666E+01_JPRB, 0.15726E+01_JPRB, 0.17542E+01_JPRB ,&
&  0.18524E+01_JPRB, 0.17226E+01_JPRB, 0.18631E+01_JPRB, 0.23960E+01_JPRB, 0.25805E+01_JPRB, 0.28048E+01_JPRB, 0.30734E+01_JPRB ,&
&  0.32823E+01_JPRB, 0.35108E+01_JPRB, 0.37130E+01_JPRB, 0.38925E+01_JPRB, 0.41182E+01_JPRB, 0.44124E+01_JPRB, 0.48041E+01_JPRB ,&
&  0.53060E+01_JPRB, 0.60363E+01_JPRB, 0.67470E+01_JPRB, 0.62491E+01_JPRB, 0.64261E+01_JPRB, 0.67991E+01_JPRB, 0.76795E+01_JPRB ,&
&  0.68017E+01_JPRB, 0.45932E+01_JPRB, 0.34679E+01_JPRB, 0.29089E+01_JPRB, 0.23003E+01_JPRB, 0.20694E+01_JPRB, 0.18172E+01_JPRB ,&
&  0.16188E+01_JPRB, 0.14736E+01_JPRB, 0.13487E+01_JPRB, 0.12538E+01_JPRB, 0.11395E+01_JPRB, 0.10746E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10984E+01_JPRB, 0.10495E+01_JPRB, 0.98092E+00_JPRB, 0.96303E+00_JPRB, 0.10242E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.11604E+01_JPRB, 0.11622E+01_JPRB, 0.11743E+01_JPRB, 0.11711E+01_JPRB, 0.11618E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.99147E+00_JPRB, 0.97251E+00_JPRB, 0.94972E+00_JPRB, 0.10928E+01_JPRB, 0.13794E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.94630E+00_JPRB, 0.78115E+00_JPRB, 0.73022E+00_JPRB, 0.70193E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.74769E+00_JPRB, 0.69914E+00_JPRB, 0.68787E+00_JPRB, 0.67148E+00_JPRB, 0.68843E+00_JPRB, 0.72025E+00_JPRB, 0.74214E+00_JPRB ,&
&  0.75183E+00_JPRB, 0.75313E+00_JPRB, 0.75964E+00_JPRB, 0.78947E+00_JPRB, 0.82795E+00_JPRB, 0.87124E+00_JPRB, 0.92452E+00_JPRB ,&
&  0.96356E+00_JPRB, 0.98740E+00_JPRB, 0.97955E+00_JPRB, 0.97344E+00_JPRB, 0.10301E+01_JPRB, 0.11162E+01_JPRB, 0.12313E+01_JPRB ,&
&  0.13764E+01_JPRB, 0.15536E+01_JPRB, 0.18173E+01_JPRB, 0.22233E+01_JPRB, 0.28309E+01_JPRB, 0.35696E+01_JPRB, 0.43431E+01_JPRB ,&
&  0.35240E+01_JPRB, 0.15904E+01_JPRB, 0.17247E+01_JPRB, 0.17914E+01_JPRB, 0.16175E+01_JPRB, 0.17314E+01_JPRB, 0.17959E+01_JPRB ,&
&  0.17712E+01_JPRB, 0.17495E+01_JPRB, 0.16692E+01_JPRB, 0.18763E+01_JPRB, 0.23908E+01_JPRB, 0.27291E+01_JPRB, 0.30131E+01_JPRB ,&
&  0.32391E+01_JPRB, 0.34045E+01_JPRB, 0.35068E+01_JPRB, 0.36129E+01_JPRB, 0.38040E+01_JPRB, 0.40997E+01_JPRB, 0.44941E+01_JPRB ,&
&  0.50286E+01_JPRB, 0.57213E+01_JPRB, 0.65455E+01_JPRB, 0.60380E+01_JPRB, 0.68381E+01_JPRB, 0.68337E+01_JPRB, 0.68987E+01_JPRB ,&
&  0.62466E+01_JPRB, 0.41348E+01_JPRB, 0.30624E+01_JPRB, 0.27386E+01_JPRB, 0.21047E+01_JPRB, 0.15511E+01_JPRB, 0.15789E+01_JPRB ,&
&  0.14949E+01_JPRB, 0.14175E+01_JPRB, 0.13669E+01_JPRB, 0.12494E+01_JPRB, 0.11429E+01_JPRB, 0.10963E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.12173E+01_JPRB, 0.12625E+01_JPRB, 0.11804E+01_JPRB, 0.11566E+01_JPRB, 0.11213E+01_JPRB, 0.11581E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.12786E+01_JPRB, 0.13030E+01_JPRB, 0.13368E+01_JPRB, 0.13716E+01_JPRB, 0.13090E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.11950E+01_JPRB, 0.11219E+01_JPRB, 0.10727E+01_JPRB, 0.10134E+01_JPRB, 0.10278E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.10956E+01_JPRB, 0.96943E+00_JPRB, 0.84657E+00_JPRB, 0.81748E+00_JPRB, 0.82198E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.87970E+00_JPRB, 0.77006E+00_JPRB, 0.71423E+00_JPRB, 0.70972E+00_JPRB, 0.69637E+00_JPRB, 0.70465E+00_JPRB, 0.71642E+00_JPRB ,&
&  0.71405E+00_JPRB, 0.73596E+00_JPRB, 0.73369E+00_JPRB, 0.75470E+00_JPRB, 0.77794E+00_JPRB, 0.79329E+00_JPRB, 0.83138E+00_JPRB ,&
&  0.86544E+00_JPRB, 0.89877E+00_JPRB, 0.93188E+00_JPRB, 0.91065E+00_JPRB, 0.93087E+00_JPRB, 0.96948E+00_JPRB, 0.10398E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.13226E+01_JPRB, 0.15104E+01_JPRB, 0.17374E+01_JPRB, 0.20683E+01_JPRB, 0.26630E+01_JPRB, 0.37526E+01_JPRB ,&
&  0.64227E+01_JPRB, 0.54818E+01_JPRB, 0.16232E+01_JPRB, 0.20778E+01_JPRB, 0.20134E+01_JPRB, 0.20358E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.19102E+01_JPRB, 0.20206E+01_JPRB, 0.17164E+01_JPRB, 0.15536E+01_JPRB, 0.17864E+01_JPRB, 0.22534E+01_JPRB, 0.26114E+01_JPRB ,&
&  0.28420E+01_JPRB, 0.30165E+01_JPRB, 0.30901E+01_JPRB, 0.31599E+01_JPRB, 0.32652E+01_JPRB, 0.34717E+01_JPRB, 0.38001E+01_JPRB ,&
&  0.42819E+01_JPRB, 0.49664E+01_JPRB, 0.58856E+01_JPRB, 0.58531E+01_JPRB, 0.72347E+01_JPRB, 0.69197E+01_JPRB, 0.64057E+01_JPRB ,&
&  0.51994E+01_JPRB, 0.41044E+01_JPRB, 0.33611E+01_JPRB, 0.26066E+01_JPRB, 0.18401E+01_JPRB, 0.14542E+01_JPRB, 0.14096E+01_JPRB ,&
&  0.13729E+01_JPRB, 0.12294E+01_JPRB, 0.11928E+01_JPRB, 0.11781E+01_JPRB, 0.11231E+01_JPRB, 0.11994E+01_JPRB, 0.13134E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.13853E+01_JPRB, 0.13315E+01_JPRB, 0.13129E+01_JPRB, 0.12961E+01_JPRB, 0.13006E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.13505E+01_JPRB, 0.13759E+01_JPRB, 0.13837E+01_JPRB, 0.14130E+01_JPRB, 0.14426E+01_JPRB, 0.13726E+01_JPRB, 0.12826E+01_JPRB ,&
&  0.12688E+01_JPRB, 0.12871E+01_JPRB, 0.12512E+01_JPRB, 0.12195E+01_JPRB, 0.11352E+01_JPRB, 0.10655E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.10299E+01_JPRB, 0.96476E+00_JPRB, 0.89631E+00_JPRB, 0.91763E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.94550E+00_JPRB, 0.87772E+00_JPRB, 0.77282E+00_JPRB, 0.73960E+00_JPRB, 0.73129E+00_JPRB, 0.71375E+00_JPRB, 0.69134E+00_JPRB ,&
&  0.66671E+00_JPRB, 0.65723E+00_JPRB, 0.68663E+00_JPRB, 0.72217E+00_JPRB, 0.74975E+00_JPRB, 0.76124E+00_JPRB, 0.78592E+00_JPRB ,&
&  0.80047E+00_JPRB, 0.82797E+00_JPRB, 0.87834E+00_JPRB, 0.89978E+00_JPRB, 0.90311E+00_JPRB, 0.91427E+00_JPRB, 0.93717E+00_JPRB ,&
&  0.97878E+00_JPRB, 0.10515E+01_JPRB, 0.11775E+01_JPRB, 0.13219E+01_JPRB, 0.15123E+01_JPRB, 0.18503E+01_JPRB, 0.25571E+01_JPRB ,&
&  0.39778E+01_JPRB, 0.73561E+01_JPRB, 0.17456E+01_JPRB, 0.24511E+01_JPRB, 0.24360E+01_JPRB, 0.22807E+01_JPRB, 0.21286E+01_JPRB ,&
&  0.21375E+01_JPRB, 0.21062E+01_JPRB, 0.19157E+01_JPRB, 0.15896E+01_JPRB, 0.13325E+01_JPRB, 0.15567E+01_JPRB, 0.19491E+01_JPRB ,&
&  0.23117E+01_JPRB, 0.24980E+01_JPRB, 0.25727E+01_JPRB, 0.25618E+01_JPRB, 0.25972E+01_JPRB, 0.26846E+01_JPRB, 0.28733E+01_JPRB ,&
&  0.32116E+01_JPRB, 0.37536E+01_JPRB, 0.45740E+01_JPRB, 0.49523E+01_JPRB, 0.66404E+01_JPRB, 0.70513E+01_JPRB, 0.67121E+01_JPRB ,&
&  0.53573E+01_JPRB, 0.47386E+01_JPRB, 0.40905E+01_JPRB, 0.28770E+01_JPRB, 0.20047E+01_JPRB, 0.15687E+01_JPRB, 0.14456E+01_JPRB ,&
&  0.13762E+01_JPRB, 0.12303E+01_JPRB, 0.12120E+01_JPRB, 0.11833E+01_JPRB, 0.12044E+01_JPRB, 0.12925E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.14324E+01_JPRB, 0.14287E+01_JPRB, 0.13843E+01_JPRB, 0.13743E+01_JPRB, 0.13748E+01_JPRB, 0.13788E+01_JPRB, 0.14144E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.14481E+01_JPRB, 0.14444E+01_JPRB, 0.14086E+01_JPRB, 0.14204E+01_JPRB, 0.13765E+01_JPRB, 0.13142E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.13575E+01_JPRB, 0.13344E+01_JPRB, 0.12787E+01_JPRB, 0.12864E+01_JPRB, 0.12271E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.11772E+01_JPRB, 0.11335E+01_JPRB, 0.99758E+00_JPRB, 0.99878E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.10074E+01_JPRB, 0.95015E+00_JPRB, 0.87129E+00_JPRB, 0.79261E+00_JPRB, 0.78679E+00_JPRB, 0.78958E+00_JPRB, 0.74811E+00_JPRB ,&
&  0.68846E+00_JPRB, 0.68278E+00_JPRB, 0.68925E+00_JPRB, 0.71388E+00_JPRB, 0.73252E+00_JPRB, 0.74866E+00_JPRB, 0.79339E+00_JPRB ,&
&  0.79237E+00_JPRB, 0.78854E+00_JPRB, 0.81443E+00_JPRB, 0.85530E+00_JPRB, 0.87075E+00_JPRB, 0.88279E+00_JPRB, 0.90887E+00_JPRB ,&
&  0.92149E+00_JPRB, 0.91779E+00_JPRB, 0.94768E+00_JPRB, 0.10155E+01_JPRB, 0.11411E+01_JPRB, 0.13669E+01_JPRB, 0.18144E+01_JPRB ,&
&  0.31225E+01_JPRB, 0.76659E+01_JPRB, 0.44780E+01_JPRB, 0.43270E+01_JPRB, 0.32053E+01_JPRB, 0.26212E+01_JPRB, 0.24004E+01_JPRB ,&
&  0.26288E+01_JPRB, 0.24476E+01_JPRB, 0.22108E+01_JPRB, 0.16653E+01_JPRB, 0.13434E+01_JPRB, 0.12288E+01_JPRB, 0.14132E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.18557E+01_JPRB, 0.20042E+01_JPRB, 0.20159E+01_JPRB, 0.20304E+01_JPRB, 0.20936E+01_JPRB, 0.21795E+01_JPRB ,&
&  0.23608E+01_JPRB, 0.27200E+01_JPRB, 0.33539E+01_JPRB, 0.41882E+01_JPRB, 0.51917E+01_JPRB, 0.62064E+01_JPRB, 0.71921E+01_JPRB ,&
&  0.68254E+01_JPRB, 0.59033E+01_JPRB, 0.47574E+01_JPRB, 0.32079E+01_JPRB, 0.21297E+01_JPRB, 0.19149E+01_JPRB, 0.15071E+01_JPRB ,&
&  0.14036E+01_JPRB, 0.12948E+01_JPRB, 0.12849E+01_JPRB, 0.13065E+01_JPRB, 0.13457E+01_JPRB, 0.14169E+01_JPRB, 0.14757E+01_JPRB ,&
&  0.14865E+01_JPRB, 0.14587E+01_JPRB, 0.14346E+01_JPRB, 0.14225E+01_JPRB, 0.14143E+01_JPRB, 0.14119E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.14340E+01_JPRB, 0.14615E+01_JPRB, 0.14069E+01_JPRB, 0.13922E+01_JPRB, 0.13924E+01_JPRB, 0.13662E+01_JPRB, 0.13001E+01_JPRB ,&
&  0.13227E+01_JPRB, 0.13836E+01_JPRB, 0.13999E+01_JPRB, 0.13773E+01_JPRB, 0.13689E+01_JPRB, 0.13696E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.12956E+01_JPRB, 0.12613E+01_JPRB, 0.11598E+01_JPRB, 0.10995E+01_JPRB, 0.10774E+01_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.10727E+01_JPRB, 0.10260E+01_JPRB, 0.96422E+00_JPRB, 0.88731E+00_JPRB, 0.84745E+00_JPRB, 0.84620E+00_JPRB, 0.84972E+00_JPRB ,&
&  0.81154E+00_JPRB, 0.78530E+00_JPRB, 0.77356E+00_JPRB, 0.78806E+00_JPRB, 0.81696E+00_JPRB, 0.83612E+00_JPRB, 0.84993E+00_JPRB ,&
&  0.84476E+00_JPRB, 0.84019E+00_JPRB, 0.82427E+00_JPRB, 0.82621E+00_JPRB, 0.87801E+00_JPRB, 0.89093E+00_JPRB, 0.88704E+00_JPRB ,&
&  0.90721E+00_JPRB, 0.89752E+00_JPRB, 0.88254E+00_JPRB, 0.89700E+00_JPRB, 0.95442E+00_JPRB, 0.11079E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.28490E+01_JPRB, 0.69935E+01_JPRB, 0.30027E+01_JPRB, 0.50567E+01_JPRB, 0.35535E+01_JPRB, 0.29069E+01_JPRB, 0.23302E+01_JPRB ,&
&  0.23555E+01_JPRB, 0.27136E+01_JPRB, 0.21332E+01_JPRB, 0.17454E+01_JPRB, 0.14420E+01_JPRB, 0.12796E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.14571E+01_JPRB, 0.15183E+01_JPRB, 0.15751E+01_JPRB, 0.16114E+01_JPRB, 0.15861E+01_JPRB, 0.15981E+01_JPRB, 0.16559E+01_JPRB ,&
&  0.17375E+01_JPRB, 0.19287E+01_JPRB, 0.23358E+01_JPRB, 0.29770E+01_JPRB, 0.44130E+01_JPRB, 0.59731E+01_JPRB, 0.76582E+01_JPRB ,&
&  0.76709E+01_JPRB, 0.74589E+01_JPRB, 0.55392E+01_JPRB, 0.36004E+01_JPRB, 0.24288E+01_JPRB, 0.20290E+01_JPRB, 0.16719E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.14295E+01_JPRB, 0.13657E+01_JPRB, 0.13685E+01_JPRB, 0.14683E+01_JPRB, 0.15521E+01_JPRB, 0.15466E+01_JPRB ,&
&  0.15305E+01_JPRB, 0.14887E+01_JPRB, 0.14592E+01_JPRB, 0.14349E+01_JPRB, 0.14393E+01_JPRB, 0.14386E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.14207E+01_JPRB, 0.14386E+01_JPRB, 0.13964E+01_JPRB, 0.14000E+01_JPRB, 0.13990E+01_JPRB, 0.13799E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.13422E+01_JPRB, 0.13791E+01_JPRB, 0.13853E+01_JPRB, 0.13655E+01_JPRB, 0.13660E+01_JPRB, 0.14275E+01_JPRB, 0.13759E+01_JPRB ,&
&  0.13555E+01_JPRB, 0.13099E+01_JPRB, 0.12266E+01_JPRB, 0.11747E+01_JPRB, 0.11426E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.11328E+01_JPRB, 0.10741E+01_JPRB, 0.10272E+01_JPRB, 0.96462E+00_JPRB, 0.91940E+00_JPRB, 0.89589E+00_JPRB, 0.89276E+00_JPRB ,&
&  0.87647E+00_JPRB, 0.84449E+00_JPRB, 0.82541E+00_JPRB, 0.82595E+00_JPRB, 0.83006E+00_JPRB, 0.84640E+00_JPRB, 0.85957E+00_JPRB ,&
&  0.85876E+00_JPRB, 0.84885E+00_JPRB, 0.84396E+00_JPRB, 0.84170E+00_JPRB, 0.86570E+00_JPRB, 0.91021E+00_JPRB, 0.92823E+00_JPRB ,&
&  0.91639E+00_JPRB, 0.90331E+00_JPRB, 0.88070E+00_JPRB, 0.86689E+00_JPRB, 0.88380E+00_JPRB, 0.98609E+00_JPRB, 0.12923E+01_JPRB ,&
&  0.27217E+01_JPRB, 0.82533E+01_JPRB, 0.32207E+01_JPRB, 0.55804E+01_JPRB, 0.38350E+01_JPRB, 0.30536E+01_JPRB, 0.23952E+01_JPRB ,&
&  0.23497E+01_JPRB, 0.25471E+01_JPRB, 0.21775E+01_JPRB, 0.18660E+01_JPRB, 0.16193E+01_JPRB, 0.14474E+01_JPRB, 0.13707E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.14467E+01_JPRB, 0.14250E+01_JPRB, 0.14246E+01_JPRB, 0.13930E+01_JPRB, 0.13770E+01_JPRB, 0.13915E+01_JPRB ,&
&  0.13948E+01_JPRB, 0.14281E+01_JPRB, 0.16204E+01_JPRB, 0.23859E+01_JPRB, 0.35255E+01_JPRB, 0.56864E+01_JPRB, 0.87562E+01_JPRB ,&
&  0.83755E+01_JPRB, 0.87251E+01_JPRB, 0.62060E+01_JPRB, 0.40541E+01_JPRB, 0.26808E+01_JPRB, 0.21363E+01_JPRB, 0.18986E+01_JPRB ,&
&  0.16961E+01_JPRB, 0.15533E+01_JPRB, 0.14577E+01_JPRB, 0.14364E+01_JPRB, 0.15041E+01_JPRB, 0.15473E+01_JPRB, 0.15777E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.14919E+01_JPRB, 0.14502E+01_JPRB, 0.14343E+01_JPRB, 0.14453E+01_JPRB, 0.14441E+01_JPRB, 0.14295E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.14148E+01_JPRB, 0.13866E+01_JPRB, 0.14023E+01_JPRB, 0.14156E+01_JPRB, 0.14003E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.13441E+01_JPRB, 0.13662E+01_JPRB, 0.13580E+01_JPRB, 0.13676E+01_JPRB, 0.13704E+01_JPRB, 0.14485E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.13616E+01_JPRB, 0.13246E+01_JPRB, 0.12594E+01_JPRB, 0.12166E+01_JPRB, 0.11886E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.11773E+01_JPRB, 0.11282E+01_JPRB, 0.10915E+01_JPRB, 0.10449E+01_JPRB, 0.98393E+00_JPRB, 0.95348E+00_JPRB, 0.93277E+00_JPRB ,&
&  0.91951E+00_JPRB, 0.88819E+00_JPRB, 0.86506E+00_JPRB, 0.86215E+00_JPRB, 0.86776E+00_JPRB, 0.86580E+00_JPRB, 0.86935E+00_JPRB ,&
&  0.87993E+00_JPRB, 0.88511E+00_JPRB, 0.87521E+00_JPRB, 0.86201E+00_JPRB, 0.85555E+00_JPRB, 0.88966E+00_JPRB, 0.91221E+00_JPRB ,&
&  0.92180E+00_JPRB, 0.90564E+00_JPRB, 0.87817E+00_JPRB, 0.84901E+00_JPRB, 0.84737E+00_JPRB, 0.90065E+00_JPRB, 0.11397E+01_JPRB ,&
&  0.23293E+01_JPRB, 0.32247E+01_JPRB, 0.57909E+01_JPRB, 0.59524E+01_JPRB, 0.40866E+01_JPRB, 0.32184E+01_JPRB, 0.27669E+01_JPRB ,&
&  0.26521E+01_JPRB, 0.24885E+01_JPRB, 0.21814E+01_JPRB, 0.19498E+01_JPRB, 0.17524E+01_JPRB, 0.15874E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.13968E+01_JPRB, 0.13819E+01_JPRB, 0.13770E+01_JPRB, 0.13690E+01_JPRB, 0.13587E+01_JPRB, 0.13537E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.13422E+01_JPRB, 0.13727E+01_JPRB, 0.16920E+01_JPRB, 0.30173E+01_JPRB, 0.50319E+01_JPRB, 0.77950E+01_JPRB ,&
&  0.88374E+01_JPRB, 0.83034E+01_JPRB, 0.58325E+01_JPRB, 0.41449E+01_JPRB, 0.28494E+01_JPRB, 0.21679E+01_JPRB, 0.19493E+01_JPRB ,&
&  0.18420E+01_JPRB, 0.17344E+01_JPRB, 0.15911E+01_JPRB, 0.15332E+01_JPRB, 0.15203E+01_JPRB, 0.15147E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.15072E+01_JPRB, 0.14961E+01_JPRB, 0.14704E+01_JPRB, 0.14483E+01_JPRB, 0.14295E+01_JPRB, 0.14039E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.13731E+01_JPRB, 0.13584E+01_JPRB, 0.13442E+01_JPRB, 0.13745E+01_JPRB, 0.14396E+01_JPRB, 0.14471E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.13641E+01_JPRB, 0.13411E+01_JPRB, 0.13338E+01_JPRB, 0.13611E+01_JPRB, 0.13392E+01_JPRB, 0.14485E+01_JPRB, 0.14019E+01_JPRB ,&
&  0.13739E+01_JPRB, 0.13419E+01_JPRB, 0.12909E+01_JPRB, 0.12386E+01_JPRB, 0.11978E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.11888E+01_JPRB, 0.11353E+01_JPRB, 0.10963E+01_JPRB, 0.10696E+01_JPRB, 0.10303E+01_JPRB, 0.99100E+00_JPRB, 0.97972E+00_JPRB ,&
&  0.97575E+00_JPRB, 0.95397E+00_JPRB, 0.92668E+00_JPRB, 0.89257E+00_JPRB, 0.88213E+00_JPRB, 0.88739E+00_JPRB, 0.89505E+00_JPRB ,&
&  0.89749E+00_JPRB, 0.90071E+00_JPRB, 0.89966E+00_JPRB, 0.89531E+00_JPRB, 0.88923E+00_JPRB, 0.90753E+00_JPRB, 0.91678E+00_JPRB ,&
&  0.91008E+00_JPRB, 0.90917E+00_JPRB, 0.88506E+00_JPRB, 0.86038E+00_JPRB, 0.85088E+00_JPRB, 0.84984E+00_JPRB, 0.90958E+00_JPRB ,&
&  0.17258E+01_JPRB, 0.40340E+01_JPRB, 0.57938E+01_JPRB, 0.52627E+01_JPRB, 0.39879E+01_JPRB, 0.30813E+01_JPRB, 0.28234E+01_JPRB ,&
&  0.26113E+01_JPRB, 0.24193E+01_JPRB, 0.21672E+01_JPRB, 0.19709E+01_JPRB, 0.18130E+01_JPRB, 0.16753E+01_JPRB, 0.15355E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.14157E+01_JPRB, 0.13897E+01_JPRB, 0.13780E+01_JPRB, 0.13693E+01_JPRB, 0.13771E+01_JPRB, 0.13870E+01_JPRB ,&
&  0.13900E+01_JPRB, 0.13818E+01_JPRB, 0.13739E+01_JPRB, 0.14719E+01_JPRB, 0.22217E+01_JPRB, 0.27768E+01_JPRB, 0.51051E+01_JPRB ,&
&  0.71115E+01_JPRB, 0.57812E+01_JPRB, 0.44039E+01_JPRB, 0.35779E+01_JPRB, 0.28441E+01_JPRB, 0.22124E+01_JPRB, 0.19060E+01_JPRB ,&
&  0.17877E+01_JPRB, 0.17095E+01_JPRB, 0.16115E+01_JPRB, 0.15402E+01_JPRB, 0.14847E+01_JPRB, 0.14612E+01_JPRB, 0.14489E+01_JPRB ,&
&  0.14230E+01_JPRB, 0.14201E+01_JPRB, 0.14122E+01_JPRB, 0.14047E+01_JPRB, 0.13821E+01_JPRB, 0.13561E+01_JPRB, 0.13331E+01_JPRB ,&
&  0.13161E+01_JPRB, 0.12949E+01_JPRB, 0.12838E+01_JPRB, 0.13105E+01_JPRB, 0.14045E+01_JPRB, 0.14763E+01_JPRB, 0.14747E+01_JPRB ,&
&  0.14090E+01_JPRB, 0.13541E+01_JPRB, 0.13542E+01_JPRB, 0.13541E+01_JPRB, 0.13703E+01_JPRB, 0.14936E+01_JPRB, 0.14294E+01_JPRB ,&
&  0.13859E+01_JPRB, 0.13601E+01_JPRB, 0.13210E+01_JPRB, 0.12722E+01_JPRB, 0.12319E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.12322E+01_JPRB, 0.11737E+01_JPRB, 0.11217E+01_JPRB, 0.10773E+01_JPRB, 0.10606E+01_JPRB, 0.10336E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.98775E+00_JPRB, 0.98629E+00_JPRB, 0.97746E+00_JPRB, 0.95683E+00_JPRB, 0.93797E+00_JPRB, 0.93010E+00_JPRB, 0.92946E+00_JPRB ,&
&  0.92857E+00_JPRB, 0.92827E+00_JPRB, 0.92577E+00_JPRB, 0.91634E+00_JPRB, 0.91688E+00_JPRB, 0.92396E+00_JPRB, 0.93133E+00_JPRB ,&
&  0.91184E+00_JPRB, 0.89502E+00_JPRB, 0.88217E+00_JPRB, 0.86856E+00_JPRB, 0.86456E+00_JPRB, 0.84895E+00_JPRB, 0.84191E+00_JPRB ,&
&  0.12048E+01_JPRB, 0.25775E+01_JPRB, 0.36960E+01_JPRB, 0.45846E+01_JPRB, 0.39181E+01_JPRB, 0.31772E+01_JPRB, 0.28451E+01_JPRB ,&
&  0.25814E+01_JPRB, 0.23015E+01_JPRB, 0.20656E+01_JPRB, 0.19159E+01_JPRB, 0.18197E+01_JPRB, 0.17039E+01_JPRB, 0.15635E+01_JPRB ,&
&  0.14608E+01_JPRB, 0.13987E+01_JPRB, 0.13857E+01_JPRB, 0.13886E+01_JPRB, 0.13880E+01_JPRB, 0.14008E+01_JPRB, 0.14085E+01_JPRB ,&
&  0.14035E+01_JPRB, 0.13853E+01_JPRB, 0.13550E+01_JPRB, 0.13444E+01_JPRB, 0.15827E+01_JPRB, 0.18185E+01_JPRB, 0.25972E+01_JPRB ,&
&  0.29880E+01_JPRB, 0.31792E+01_JPRB, 0.30357E+01_JPRB, 0.28962E+01_JPRB, 0.26443E+01_JPRB, 0.22148E+01_JPRB, 0.18718E+01_JPRB ,&
&  0.16849E+01_JPRB, 0.16054E+01_JPRB, 0.15368E+01_JPRB, 0.14992E+01_JPRB, 0.14598E+01_JPRB, 0.14268E+01_JPRB, 0.14104E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.13780E+01_JPRB, 0.13608E+01_JPRB, 0.13588E+01_JPRB, 0.13357E+01_JPRB, 0.12938E+01_JPRB, 0.12595E+01_JPRB ,&
&  0.12408E+01_JPRB, 0.12234E+01_JPRB, 0.12348E+01_JPRB, 0.12756E+01_JPRB, 0.13229E+01_JPRB, 0.13604E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.14240E+01_JPRB, 0.14126E+01_JPRB, 0.13813E+01_JPRB, 0.13894E+01_JPRB, 0.14061E+01_JPRB, 0.15045E+01_JPRB, 0.14421E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.13711E+01_JPRB, 0.13358E+01_JPRB, 0.12755E+01_JPRB, 0.12841E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.12290E+01_JPRB, 0.11952E+01_JPRB, 0.11560E+01_JPRB, 0.11059E+01_JPRB, 0.10839E+01_JPRB, 0.10646E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10232E+01_JPRB, 0.10154E+01_JPRB, 0.99837E+00_JPRB, 0.98035E+00_JPRB, 0.96829E+00_JPRB, 0.95480E+00_JPRB ,&
&  0.95412E+00_JPRB, 0.95356E+00_JPRB, 0.94493E+00_JPRB, 0.93529E+00_JPRB, 0.92970E+00_JPRB, 0.92499E+00_JPRB, 0.92756E+00_JPRB ,&
&  0.91875E+00_JPRB, 0.90646E+00_JPRB, 0.90464E+00_JPRB, 0.89538E+00_JPRB, 0.88031E+00_JPRB, 0.86836E+00_JPRB, 0.83944E+00_JPRB ,&
&  0.93222E+00_JPRB, 0.14141E+01_JPRB, 0.22468E+01_JPRB, 0.31841E+01_JPRB, 0.31155E+01_JPRB, 0.29361E+01_JPRB, 0.26310E+01_JPRB ,&
&  0.23475E+01_JPRB, 0.21316E+01_JPRB, 0.18929E+01_JPRB, 0.17292E+01_JPRB, 0.16665E+01_JPRB, 0.16037E+01_JPRB, 0.15358E+01_JPRB ,&
&  0.14637E+01_JPRB, 0.14024E+01_JPRB, 0.13723E+01_JPRB, 0.13771E+01_JPRB, 0.13688E+01_JPRB, 0.13623E+01_JPRB, 0.13562E+01_JPRB ,&
&  0.13367E+01_JPRB, 0.13125E+01_JPRB, 0.12736E+01_JPRB, 0.12351E+01_JPRB, 0.12057E+01_JPRB, 0.12120E+01_JPRB, 0.13999E+01_JPRB ,&
&  0.16859E+01_JPRB, 0.18331E+01_JPRB, 0.19117E+01_JPRB, 0.20108E+01_JPRB, 0.20626E+01_JPRB, 0.20016E+01_JPRB, 0.18207E+01_JPRB ,&
&  0.16705E+01_JPRB, 0.15750E+01_JPRB, 0.14869E+01_JPRB, 0.14778E+01_JPRB, 0.14496E+01_JPRB, 0.14115E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.13136E+01_JPRB, 0.12843E+01_JPRB, 0.12615E+01_JPRB, 0.12371E+01_JPRB, 0.12051E+01_JPRB, 0.11714E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.11407E+01_JPRB, 0.11504E+01_JPRB, 0.11858E+01_JPRB, 0.12328E+01_JPRB, 0.12692E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.13380E+01_JPRB, 0.13713E+01_JPRB, 0.13420E+01_JPRB, 0.13468E+01_JPRB, 0.14704E+01_JPRB, 0.14651E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.13395E+01_JPRB, 0.13149E+01_JPRB, 0.12445E+01_JPRB, 0.12181E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.10981E+01_JPRB, 0.11154E+01_JPRB, 0.10991E+01_JPRB, 0.10633E+01_JPRB, 0.10431E+01_JPRB, 0.10353E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.10304E+01_JPRB, 0.10200E+01_JPRB, 0.10242E+01_JPRB, 0.10285E+01_JPRB, 0.10223E+01_JPRB, 0.10059E+01_JPRB, 0.99321E+00_JPRB ,&
&  0.98780E+00_JPRB, 0.98948E+00_JPRB, 0.99285E+00_JPRB, 0.98124E+00_JPRB, 0.95786E+00_JPRB, 0.94141E+00_JPRB, 0.92763E+00_JPRB ,&
&  0.92609E+00_JPRB, 0.92594E+00_JPRB, 0.92332E+00_JPRB, 0.92352E+00_JPRB, 0.89988E+00_JPRB, 0.87302E+00_JPRB, 0.85171E+00_JPRB ,&
&  0.85494E+00_JPRB, 0.90710E+00_JPRB, 0.13196E+01_JPRB, 0.18858E+01_JPRB, 0.21494E+01_JPRB, 0.21061E+01_JPRB, 0.18823E+01_JPRB ,&
&  0.17381E+01_JPRB, 0.16505E+01_JPRB, 0.15809E+01_JPRB, 0.14943E+01_JPRB, 0.14354E+01_JPRB, 0.13966E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.12982E+01_JPRB, 0.12709E+01_JPRB, 0.12393E+01_JPRB, 0.12131E+01_JPRB, 0.12011E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.11848E+01_JPRB, 0.11687E+01_JPRB, 0.11554E+01_JPRB, 0.11384E+01_JPRB, 0.10988E+01_JPRB, 0.10793E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.11735E+01_JPRB, 0.12434E+01_JPRB, 0.12923E+01_JPRB, 0.14024E+01_JPRB, 0.15069E+01_JPRB, 0.15955E+01_JPRB, 0.15918E+01_JPRB ,&
&  0.15431E+01_JPRB, 0.14992E+01_JPRB, 0.14497E+01_JPRB, 0.14280E+01_JPRB, 0.14014E+01_JPRB, 0.13488E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.12568E+01_JPRB, 0.12290E+01_JPRB, 0.11942E+01_JPRB, 0.11643E+01_JPRB, 0.11310E+01_JPRB, 0.10915E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10238E+01_JPRB, 0.10293E+01_JPRB, 0.10541E+01_JPRB, 0.10910E+01_JPRB, 0.11355E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.12539E+01_JPRB, 0.12902E+01_JPRB, 0.13499E+01_JPRB, 0.13563E+01_JPRB, 0.13502E+01_JPRB, 0.13508E+01_JPRB ,&
&  0.13265E+01_JPRB, 0.12789E+01_JPRB, 0.12287E+01_JPRB, 0.11496E+01_JPRB, 0.10504E+01_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.10366E+01_JPRB, 0.10487E+01_JPRB, 0.10432E+01_JPRB, 0.10288E+01_JPRB, 0.10173E+01_JPRB, 0.10104E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.10074E+01_JPRB, 0.10096E+01_JPRB, 0.10098E+01_JPRB, 0.10087E+01_JPRB, 0.10022E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.10002E+01_JPRB, 0.10018E+01_JPRB, 0.99395E+00_JPRB, 0.97244E+00_JPRB, 0.95683E+00_JPRB, 0.94188E+00_JPRB ,&
&  0.94061E+00_JPRB, 0.94004E+00_JPRB, 0.94031E+00_JPRB, 0.93306E+00_JPRB, 0.91060E+00_JPRB, 0.89025E+00_JPRB, 0.87710E+00_JPRB ,&
&  0.87862E+00_JPRB, 0.77532E+00_JPRB, 0.95346E+00_JPRB, 0.13591E+01_JPRB, 0.15619E+01_JPRB, 0.14534E+01_JPRB, 0.13947E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.13649E+01_JPRB, 0.13450E+01_JPRB, 0.13020E+01_JPRB, 0.12290E+01_JPRB, 0.11944E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11184E+01_JPRB, 0.11050E+01_JPRB, 0.10968E+01_JPRB, 0.10762E+01_JPRB, 0.10556E+01_JPRB, 0.10483E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10459E+01_JPRB, 0.10465E+01_JPRB, 0.10442E+01_JPRB, 0.10291E+01_JPRB, 0.10337E+01_JPRB, 0.10352E+01_JPRB ,&
&  0.10496E+01_JPRB, 0.10645E+01_JPRB, 0.10781E+01_JPRB, 0.11118E+01_JPRB, 0.11727E+01_JPRB, 0.12780E+01_JPRB, 0.13431E+01_JPRB ,&
&  0.13463E+01_JPRB, 0.13305E+01_JPRB, 0.13270E+01_JPRB, 0.13210E+01_JPRB, 0.13021E+01_JPRB, 0.12612E+01_JPRB, 0.12162E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.11463E+01_JPRB, 0.11123E+01_JPRB, 0.10765E+01_JPRB, 0.10406E+01_JPRB, 0.10065E+01_JPRB, 0.97777E+00_JPRB ,&
&  0.95849E+00_JPRB, 0.95028E+00_JPRB, 0.95927E+00_JPRB, 0.98389E+00_JPRB, 0.10176E+01_JPRB, 0.10566E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.11430E+01_JPRB, 0.11882E+01_JPRB, 0.12316E+01_JPRB, 0.11995E+01_JPRB, 0.12691E+01_JPRB, 0.12806E+01_JPRB, 0.12808E+01_JPRB ,&
&  0.12624E+01_JPRB, 0.12126E+01_JPRB, 0.11372E+01_JPRB, 0.91934E+00_JPRB, 0.10013E+01_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.10058E+01_JPRB, 0.10094E+01_JPRB, 0.10110E+01_JPRB, 0.99690E+00_JPRB, 0.97899E+00_JPRB, 0.97151E+00_JPRB, 0.97741E+00_JPRB ,&
&  0.97235E+00_JPRB, 0.95859E+00_JPRB, 0.96286E+00_JPRB, 0.96467E+00_JPRB, 0.96938E+00_JPRB, 0.96998E+00_JPRB, 0.97069E+00_JPRB ,&
&  0.97167E+00_JPRB, 0.97335E+00_JPRB, 0.97041E+00_JPRB, 0.97804E+00_JPRB, 0.97011E+00_JPRB, 0.96405E+00_JPRB, 0.96080E+00_JPRB ,&
&  0.95235E+00_JPRB, 0.94382E+00_JPRB, 0.93629E+00_JPRB, 0.92524E+00_JPRB, 0.91559E+00_JPRB, 0.90353E+00_JPRB, 0.88603E+00_JPRB ,&
&  0.89689E+00_JPRB, 0.76367E+00_JPRB, 0.90655E+00_JPRB, 0.11626E+01_JPRB, 0.11599E+01_JPRB, 0.11168E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.11533E+01_JPRB, 0.11613E+01_JPRB, 0.11545E+01_JPRB, 0.11223E+01_JPRB, 0.10529E+01_JPRB, 0.10212E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.98839E+00_JPRB, 0.97465E+00_JPRB, 0.96487E+00_JPRB, 0.96528E+00_JPRB, 0.96583E+00_JPRB, 0.96246E+00_JPRB, 0.95618E+00_JPRB ,&
&  0.95596E+00_JPRB, 0.96152E+00_JPRB, 0.96174E+00_JPRB, 0.96228E+00_JPRB, 0.96240E+00_JPRB, 0.95154E+00_JPRB, 0.94281E+00_JPRB ,&
&  0.93558E+00_JPRB, 0.93478E+00_JPRB, 0.93481E+00_JPRB, 0.94840E+00_JPRB, 0.97822E+00_JPRB, 0.10459E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11373E+01_JPRB, 0.11455E+01_JPRB, 0.11487E+01_JPRB, 0.11496E+01_JPRB, 0.11528E+01_JPRB, 0.11449E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.10565E+01_JPRB, 0.10279E+01_JPRB, 0.10052E+01_JPRB, 0.98154E+00_JPRB, 0.96321E+00_JPRB, 0.95427E+00_JPRB ,&
&  0.94279E+00_JPRB, 0.92876E+00_JPRB, 0.93157E+00_JPRB, 0.94871E+00_JPRB, 0.96762E+00_JPRB, 0.98495E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.11021E+01_JPRB, 0.11378E+01_JPRB, 0.11752E+01_JPRB, 0.11911E+01_JPRB, 0.11954E+01_JPRB, 0.11867E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11387E+01_JPRB, 0.86508E+00_JPRB, 0.97178E+00_JPRB, 0.99558E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.99473E+00_JPRB, 0.98474E+00_JPRB, 0.97276E+00_JPRB, 0.95546E+00_JPRB, 0.93746E+00_JPRB, 0.92581E+00_JPRB, 0.92997E+00_JPRB ,&
&  0.92734E+00_JPRB, 0.91442E+00_JPRB, 0.92435E+00_JPRB, 0.93673E+00_JPRB, 0.94604E+00_JPRB, 0.95396E+00_JPRB, 0.95499E+00_JPRB ,&
&  0.95636E+00_JPRB, 0.95175E+00_JPRB, 0.95420E+00_JPRB, 0.96641E+00_JPRB, 0.98553E+00_JPRB, 0.98749E+00_JPRB, 0.98452E+00_JPRB ,&
&  0.96241E+00_JPRB, 0.95123E+00_JPRB, 0.94086E+00_JPRB, 0.92808E+00_JPRB, 0.91573E+00_JPRB, 0.90975E+00_JPRB, 0.90958E+00_JPRB ,&
&  0.79487E+00_JPRB, 0.71221E+00_JPRB, 0.87133E+00_JPRB, 0.99053E+00_JPRB, 0.97248E+00_JPRB, 0.95070E+00_JPRB, 0.96822E+00_JPRB ,&
&  0.97458E+00_JPRB, 0.96818E+00_JPRB, 0.97044E+00_JPRB, 0.97101E+00_JPRB, 0.94262E+00_JPRB, 0.90563E+00_JPRB, 0.88944E+00_JPRB ,&
&  0.88067E+00_JPRB, 0.88646E+00_JPRB, 0.88943E+00_JPRB, 0.89772E+00_JPRB, 0.89818E+00_JPRB, 0.89050E+00_JPRB, 0.88685E+00_JPRB ,&
&  0.88422E+00_JPRB, 0.89034E+00_JPRB, 0.89508E+00_JPRB, 0.89711E+00_JPRB, 0.89544E+00_JPRB, 0.89245E+00_JPRB, 0.88503E+00_JPRB ,&
&  0.87829E+00_JPRB, 0.87343E+00_JPRB, 0.86933E+00_JPRB, 0.87421E+00_JPRB, 0.89025E+00_JPRB, 0.92336E+00_JPRB, 0.97161E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10179E+01_JPRB, 0.10256E+01_JPRB, 0.10247E+01_JPRB, 0.10333E+01_JPRB, 0.10349E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.99502E+00_JPRB, 0.97629E+00_JPRB, 0.96767E+00_JPRB, 0.95831E+00_JPRB, 0.95140E+00_JPRB, 0.94336E+00_JPRB, 0.93236E+00_JPRB ,&
&  0.92136E+00_JPRB, 0.90713E+00_JPRB, 0.89682E+00_JPRB, 0.89823E+00_JPRB, 0.91185E+00_JPRB, 0.92463E+00_JPRB, 0.94909E+00_JPRB ,&
&  0.98891E+00_JPRB, 0.10349E+01_JPRB, 0.10722E+01_JPRB, 0.11106E+01_JPRB, 0.11320E+01_JPRB, 0.11401E+01_JPRB, 0.11306E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.10670E+01_JPRB, 0.10486E+01_JPRB, 0.10213E+01_JPRB, 0.10066E+01_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.94105E+00_JPRB, 0.93371E+00_JPRB, 0.91927E+00_JPRB, 0.89323E+00_JPRB, 0.85928E+00_JPRB, 0.86655E+00_JPRB, 0.87626E+00_JPRB ,&
&  0.86835E+00_JPRB, 0.85808E+00_JPRB, 0.86861E+00_JPRB, 0.89392E+00_JPRB, 0.91791E+00_JPRB, 0.93100E+00_JPRB, 0.93184E+00_JPRB ,&
&  0.92528E+00_JPRB, 0.92596E+00_JPRB, 0.93370E+00_JPRB, 0.95111E+00_JPRB, 0.95082E+00_JPRB, 0.93327E+00_JPRB, 0.92255E+00_JPRB ,&
&  0.91996E+00_JPRB, 0.91822E+00_JPRB, 0.91575E+00_JPRB, 0.89885E+00_JPRB, 0.88772E+00_JPRB, 0.88427E+00_JPRB, 0.88377E+00_JPRB ,&
&  0.88311E+00_JPRB, 0.74949E+00_JPRB, 0.88722E+00_JPRB, 0.88973E+00_JPRB, 0.88719E+00_JPRB, 0.86403E+00_JPRB, 0.85912E+00_JPRB ,&
&  0.86101E+00_JPRB, 0.87553E+00_JPRB, 0.87312E+00_JPRB, 0.85751E+00_JPRB, 0.83956E+00_JPRB, 0.81511E+00_JPRB, 0.79838E+00_JPRB ,&
&  0.78980E+00_JPRB, 0.79405E+00_JPRB, 0.80528E+00_JPRB, 0.81785E+00_JPRB, 0.83051E+00_JPRB, 0.83261E+00_JPRB, 0.83572E+00_JPRB ,&
&  0.83642E+00_JPRB, 0.83861E+00_JPRB, 0.83755E+00_JPRB, 0.83885E+00_JPRB, 0.83720E+00_JPRB, 0.83157E+00_JPRB, 0.82179E+00_JPRB ,&
&  0.80803E+00_JPRB, 0.79956E+00_JPRB, 0.79838E+00_JPRB, 0.80165E+00_JPRB, 0.81489E+00_JPRB, 0.84071E+00_JPRB, 0.87307E+00_JPRB ,&
&  0.90587E+00_JPRB, 0.92754E+00_JPRB, 0.94000E+00_JPRB, 0.94333E+00_JPRB, 0.95057E+00_JPRB, 0.95709E+00_JPRB, 0.95088E+00_JPRB ,&
&  0.93528E+00_JPRB, 0.91791E+00_JPRB, 0.90601E+00_JPRB, 0.89360E+00_JPRB, 0.88609E+00_JPRB, 0.87989E+00_JPRB, 0.87112E+00_JPRB ,&
&  0.86131E+00_JPRB, 0.85985E+00_JPRB, 0.86191E+00_JPRB, 0.85239E+00_JPRB, 0.85611E+00_JPRB, 0.86898E+00_JPRB, 0.89157E+00_JPRB ,&
&  0.91990E+00_JPRB, 0.95649E+00_JPRB, 0.99164E+00_JPRB, 0.10194E+01_JPRB, 0.10357E+01_JPRB, 0.10400E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.10146E+01_JPRB, 0.98913E+00_JPRB, 0.96945E+00_JPRB, 0.95814E+00_JPRB, 0.95722E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.88733E+00_JPRB, 0.88766E+00_JPRB, 0.87728E+00_JPRB, 0.83893E+00_JPRB, 0.80944E+00_JPRB, 0.81776E+00_JPRB, 0.82514E+00_JPRB ,&
&  0.82450E+00_JPRB, 0.84006E+00_JPRB, 0.86526E+00_JPRB, 0.89052E+00_JPRB, 0.90726E+00_JPRB, 0.90647E+00_JPRB, 0.90948E+00_JPRB ,&
&  0.90582E+00_JPRB, 0.90302E+00_JPRB, 0.91260E+00_JPRB, 0.91760E+00_JPRB, 0.90696E+00_JPRB, 0.88631E+00_JPRB, 0.88468E+00_JPRB ,&
&  0.89127E+00_JPRB, 0.88688E+00_JPRB, 0.86860E+00_JPRB, 0.85624E+00_JPRB, 0.84623E+00_JPRB, 0.85207E+00_JPRB, 0.86073E+00_JPRB ,&
&  0.85788E+00_JPRB, 0.82913E+00_JPRB, 0.87024E+00_JPRB, 0.87514E+00_JPRB, 0.87062E+00_JPRB, 0.84504E+00_JPRB, 0.84554E+00_JPRB ,&
&  0.85399E+00_JPRB, 0.85890E+00_JPRB, 0.84154E+00_JPRB, 0.82166E+00_JPRB, 0.80558E+00_JPRB, 0.79090E+00_JPRB, 0.76525E+00_JPRB ,&
&  0.75650E+00_JPRB, 0.75429E+00_JPRB, 0.76872E+00_JPRB, 0.78430E+00_JPRB, 0.80239E+00_JPRB, 0.81033E+00_JPRB, 0.82096E+00_JPRB ,&
&  0.82207E+00_JPRB, 0.81468E+00_JPRB, 0.80691E+00_JPRB, 0.79950E+00_JPRB, 0.79594E+00_JPRB, 0.78847E+00_JPRB, 0.77748E+00_JPRB ,&
&  0.76738E+00_JPRB, 0.76505E+00_JPRB, 0.76982E+00_JPRB, 0.77946E+00_JPRB, 0.79428E+00_JPRB, 0.82073E+00_JPRB, 0.84506E+00_JPRB ,&
&  0.87324E+00_JPRB, 0.88575E+00_JPRB, 0.89547E+00_JPRB, 0.91006E+00_JPRB, 0.91739E+00_JPRB, 0.91522E+00_JPRB, 0.91060E+00_JPRB ,&
&  0.90225E+00_JPRB, 0.89008E+00_JPRB, 0.88892E+00_JPRB, 0.88239E+00_JPRB, 0.87777E+00_JPRB, 0.86480E+00_JPRB, 0.84612E+00_JPRB ,&
&  0.83845E+00_JPRB, 0.83348E+00_JPRB, 0.82554E+00_JPRB, 0.82305E+00_JPRB, 0.82076E+00_JPRB, 0.83993E+00_JPRB, 0.87383E+00_JPRB ,&
&  0.89384E+00_JPRB, 0.92031E+00_JPRB, 0.95665E+00_JPRB, 0.97954E+00_JPRB, 0.97966E+00_JPRB, 0.97811E+00_JPRB, 0.96572E+00_JPRB ,&
&  0.94638E+00_JPRB, 0.93232E+00_JPRB, 0.91883E+00_JPRB, 0.91335E+00_JPRB, 0.90853E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.83503E+00_JPRB, 0.85713E+00_JPRB, 0.82491E+00_JPRB, 0.80708E+00_JPRB, 0.78557E+00_JPRB, 0.77813E+00_JPRB, 0.76829E+00_JPRB ,&
&  0.78005E+00_JPRB, 0.79826E+00_JPRB, 0.82274E+00_JPRB, 0.83083E+00_JPRB, 0.83743E+00_JPRB, 0.83985E+00_JPRB, 0.85574E+00_JPRB ,&
&  0.84999E+00_JPRB, 0.84112E+00_JPRB, 0.84383E+00_JPRB, 0.83322E+00_JPRB, 0.82445E+00_JPRB, 0.83009E+00_JPRB, 0.84031E+00_JPRB ,&
&  0.85126E+00_JPRB, 0.84248E+00_JPRB, 0.82472E+00_JPRB, 0.79804E+00_JPRB, 0.78540E+00_JPRB, 0.78261E+00_JPRB, 0.79081E+00_JPRB ,&
&  0.79836E+00_JPRB, 0.83607E+00_JPRB, 0.76715E+00_JPRB, 0.82554E+00_JPRB, 0.82645E+00_JPRB, 0.81601E+00_JPRB, 0.81106E+00_JPRB ,&
&  0.81375E+00_JPRB, 0.80270E+00_JPRB, 0.79540E+00_JPRB, 0.78898E+00_JPRB, 0.76649E+00_JPRB, 0.74886E+00_JPRB, 0.73102E+00_JPRB ,&
&  0.71542E+00_JPRB, 0.70570E+00_JPRB, 0.71195E+00_JPRB, 0.73108E+00_JPRB, 0.75030E+00_JPRB, 0.76093E+00_JPRB, 0.76719E+00_JPRB ,&
&  0.77036E+00_JPRB, 0.76276E+00_JPRB, 0.75172E+00_JPRB, 0.75026E+00_JPRB, 0.75471E+00_JPRB, 0.75791E+00_JPRB, 0.75881E+00_JPRB ,&
&  0.75676E+00_JPRB, 0.73413E+00_JPRB, 0.72127E+00_JPRB, 0.73830E+00_JPRB, 0.75887E+00_JPRB, 0.78357E+00_JPRB, 0.81553E+00_JPRB ,&
&  0.84093E+00_JPRB, 0.85155E+00_JPRB, 0.85307E+00_JPRB, 0.86014E+00_JPRB, 0.86579E+00_JPRB, 0.87404E+00_JPRB, 0.87978E+00_JPRB ,&
&  0.86518E+00_JPRB, 0.85961E+00_JPRB, 0.85855E+00_JPRB, 0.85647E+00_JPRB, 0.84437E+00_JPRB, 0.81817E+00_JPRB, 0.78335E+00_JPRB ,&
&  0.76685E+00_JPRB, 0.75130E+00_JPRB, 0.74034E+00_JPRB, 0.73592E+00_JPRB, 0.73590E+00_JPRB, 0.76305E+00_JPRB, 0.80313E+00_JPRB ,&
&  0.84335E+00_JPRB, 0.87579E+00_JPRB, 0.90379E+00_JPRB, 0.92913E+00_JPRB, 0.92867E+00_JPRB, 0.90707E+00_JPRB, 0.89737E+00_JPRB ,&
&  0.88192E+00_JPRB, 0.86463E+00_JPRB, 0.85219E+00_JPRB, 0.84273E+00_JPRB, 0.83326E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.83167E+00_JPRB, 0.83364E+00_JPRB, 0.81939E+00_JPRB, 0.80934E+00_JPRB, 0.79098E+00_JPRB, 0.76686E+00_JPRB, 0.74655E+00_JPRB ,&
&  0.74915E+00_JPRB, 0.76603E+00_JPRB, 0.78194E+00_JPRB, 0.79318E+00_JPRB, 0.80860E+00_JPRB, 0.81146E+00_JPRB, 0.81705E+00_JPRB ,&
&  0.81147E+00_JPRB, 0.77368E+00_JPRB, 0.76746E+00_JPRB, 0.77484E+00_JPRB, 0.77665E+00_JPRB, 0.78336E+00_JPRB, 0.79524E+00_JPRB ,&
&  0.80598E+00_JPRB, 0.80160E+00_JPRB, 0.79317E+00_JPRB, 0.76309E+00_JPRB, 0.75612E+00_JPRB, 0.76097E+00_JPRB, 0.76617E+00_JPRB ,&
&  0.77697E+00_JPRB, 0.79792E+00_JPRB, 0.79880E+00_JPRB, 0.80904E+00_JPRB, 0.81430E+00_JPRB, 0.81484E+00_JPRB, 0.80961E+00_JPRB ,&
&  0.78213E+00_JPRB, 0.75516E+00_JPRB, 0.74446E+00_JPRB, 0.72786E+00_JPRB, 0.70405E+00_JPRB, 0.68773E+00_JPRB, 0.68112E+00_JPRB ,&
&  0.67867E+00_JPRB, 0.68491E+00_JPRB, 0.69157E+00_JPRB, 0.70581E+00_JPRB, 0.71120E+00_JPRB, 0.71556E+00_JPRB, 0.72152E+00_JPRB ,&
&  0.72213E+00_JPRB, 0.71918E+00_JPRB, 0.71976E+00_JPRB, 0.72948E+00_JPRB, 0.73413E+00_JPRB, 0.73021E+00_JPRB, 0.72405E+00_JPRB ,&
&  0.70946E+00_JPRB, 0.69605E+00_JPRB, 0.70356E+00_JPRB, 0.72012E+00_JPRB, 0.72956E+00_JPRB, 0.75861E+00_JPRB, 0.79551E+00_JPRB ,&
&  0.82709E+00_JPRB, 0.84703E+00_JPRB, 0.84740E+00_JPRB, 0.84204E+00_JPRB, 0.83866E+00_JPRB, 0.84528E+00_JPRB, 0.85641E+00_JPRB ,&
&  0.83923E+00_JPRB, 0.82938E+00_JPRB, 0.82654E+00_JPRB, 0.80587E+00_JPRB, 0.76751E+00_JPRB, 0.73350E+00_JPRB, 0.70871E+00_JPRB ,&
&  0.68931E+00_JPRB, 0.67859E+00_JPRB, 0.68249E+00_JPRB, 0.68494E+00_JPRB, 0.69631E+00_JPRB, 0.73214E+00_JPRB, 0.77059E+00_JPRB ,&
&  0.81286E+00_JPRB, 0.83510E+00_JPRB, 0.84317E+00_JPRB, 0.85252E+00_JPRB, 0.85962E+00_JPRB, 0.84909E+00_JPRB, 0.83981E+00_JPRB ,&
&  0.83695E+00_JPRB, 0.82802E+00_JPRB, 0.81853E+00_JPRB, 0.80924E+00_JPRB, 0.81754E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.82276E+00_JPRB, 0.81594E+00_JPRB, 0.80281E+00_JPRB, 0.77500E+00_JPRB, 0.75606E+00_JPRB, 0.72867E+00_JPRB, 0.71363E+00_JPRB ,&
&  0.71664E+00_JPRB, 0.71897E+00_JPRB, 0.72466E+00_JPRB, 0.73377E+00_JPRB, 0.73131E+00_JPRB, 0.72620E+00_JPRB, 0.72082E+00_JPRB ,&
&  0.72294E+00_JPRB, 0.72311E+00_JPRB, 0.73155E+00_JPRB, 0.74672E+00_JPRB, 0.76268E+00_JPRB, 0.77223E+00_JPRB, 0.78302E+00_JPRB ,&
&  0.79134E+00_JPRB, 0.78553E+00_JPRB, 0.78071E+00_JPRB, 0.74907E+00_JPRB, 0.73061E+00_JPRB, 0.73543E+00_JPRB, 0.74316E+00_JPRB ,&
&  0.75363E+00_JPRB, 0.76537E+00_JPRB, 0.77330E+00_JPRB, 0.78128E+00_JPRB, 0.76730E+00_JPRB, 0.76215E+00_JPRB, 0.75027E+00_JPRB ,&
&  0.70521E+00_JPRB, 0.66149E+00_JPRB, 0.63781E+00_JPRB, 0.62874E+00_JPRB, 0.62106E+00_JPRB, 0.62641E+00_JPRB, 0.63530E+00_JPRB ,&
&  0.65049E+00_JPRB, 0.66404E+00_JPRB, 0.66722E+00_JPRB, 0.66974E+00_JPRB, 0.67113E+00_JPRB, 0.67084E+00_JPRB, 0.67267E+00_JPRB ,&
&  0.67178E+00_JPRB, 0.67218E+00_JPRB, 0.68005E+00_JPRB, 0.68028E+00_JPRB, 0.67031E+00_JPRB, 0.66612E+00_JPRB, 0.66240E+00_JPRB ,&
&  0.65939E+00_JPRB, 0.65686E+00_JPRB, 0.65362E+00_JPRB, 0.66461E+00_JPRB, 0.69176E+00_JPRB, 0.72253E+00_JPRB, 0.74868E+00_JPRB ,&
&  0.76446E+00_JPRB, 0.77774E+00_JPRB, 0.78120E+00_JPRB, 0.78266E+00_JPRB, 0.78202E+00_JPRB, 0.77275E+00_JPRB, 0.76878E+00_JPRB ,&
&  0.76088E+00_JPRB, 0.74117E+00_JPRB, 0.71458E+00_JPRB, 0.67381E+00_JPRB, 0.63404E+00_JPRB, 0.61090E+00_JPRB, 0.60994E+00_JPRB ,&
&  0.62185E+00_JPRB, 0.63442E+00_JPRB, 0.64060E+00_JPRB, 0.64805E+00_JPRB, 0.66175E+00_JPRB, 0.67845E+00_JPRB, 0.69677E+00_JPRB ,&
&  0.70839E+00_JPRB, 0.70895E+00_JPRB, 0.70760E+00_JPRB, 0.71698E+00_JPRB, 0.72924E+00_JPRB, 0.74851E+00_JPRB, 0.76943E+00_JPRB ,&
&  0.78747E+00_JPRB, 0.78778E+00_JPRB, 0.78869E+00_JPRB, 0.79063E+00_JPRB, 0.81135E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.81405E+00_JPRB, 0.81271E+00_JPRB, 0.78341E+00_JPRB, 0.75082E+00_JPRB, 0.73230E+00_JPRB, 0.71236E+00_JPRB, 0.69653E+00_JPRB ,&
&  0.69507E+00_JPRB, 0.69213E+00_JPRB, 0.68937E+00_JPRB, 0.68580E+00_JPRB, 0.68349E+00_JPRB, 0.68040E+00_JPRB, 0.66845E+00_JPRB ,&
&  0.66377E+00_JPRB, 0.66986E+00_JPRB, 0.69184E+00_JPRB, 0.71633E+00_JPRB, 0.73589E+00_JPRB, 0.76193E+00_JPRB, 0.77369E+00_JPRB ,&
&  0.78652E+00_JPRB, 0.79964E+00_JPRB, 0.80141E+00_JPRB, 0.76691E+00_JPRB, 0.73862E+00_JPRB, 0.73946E+00_JPRB, 0.75103E+00_JPRB ,&
&  0.75191E+00_JPRB, 0.75556E+00_JPRB, 0.75752E+00_JPRB, 0.75553E+00_JPRB, 0.73536E+00_JPRB, 0.69724E+00_JPRB, 0.66603E+00_JPRB ,&
&  0.63908E+00_JPRB, 0.61790E+00_JPRB, 0.60989E+00_JPRB, 0.59589E+00_JPRB, 0.59195E+00_JPRB, 0.60181E+00_JPRB, 0.61710E+00_JPRB ,&
&  0.63334E+00_JPRB, 0.64138E+00_JPRB, 0.64858E+00_JPRB, 0.64429E+00_JPRB, 0.64016E+00_JPRB, 0.64381E+00_JPRB, 0.64455E+00_JPRB ,&
&  0.64517E+00_JPRB, 0.63656E+00_JPRB, 0.63325E+00_JPRB, 0.62565E+00_JPRB, 0.62116E+00_JPRB, 0.62031E+00_JPRB, 0.62225E+00_JPRB ,&
&  0.62305E+00_JPRB, 0.62870E+00_JPRB, 0.63061E+00_JPRB, 0.63331E+00_JPRB, 0.64323E+00_JPRB, 0.66030E+00_JPRB, 0.67718E+00_JPRB ,&
&  0.68983E+00_JPRB, 0.69978E+00_JPRB, 0.70719E+00_JPRB, 0.71321E+00_JPRB, 0.71329E+00_JPRB, 0.70843E+00_JPRB, 0.69422E+00_JPRB ,&
&  0.66370E+00_JPRB, 0.63107E+00_JPRB, 0.60611E+00_JPRB, 0.58625E+00_JPRB, 0.58328E+00_JPRB, 0.59503E+00_JPRB, 0.61331E+00_JPRB ,&
&  0.62700E+00_JPRB, 0.63018E+00_JPRB, 0.62293E+00_JPRB, 0.61983E+00_JPRB, 0.62296E+00_JPRB, 0.63122E+00_JPRB, 0.63772E+00_JPRB ,&
&  0.63634E+00_JPRB, 0.63340E+00_JPRB, 0.63835E+00_JPRB, 0.64741E+00_JPRB, 0.66375E+00_JPRB, 0.68368E+00_JPRB, 0.70486E+00_JPRB ,&
&  0.72552E+00_JPRB, 0.74126E+00_JPRB, 0.76098E+00_JPRB, 0.78080E+00_JPRB, 0.80603E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.79377E+00_JPRB, 0.76517E+00_JPRB, 0.73504E+00_JPRB, 0.70968E+00_JPRB, 0.69828E+00_JPRB, 0.68135E+00_JPRB, 0.66697E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.67370E+00_JPRB, 0.66841E+00_JPRB, 0.66171E+00_JPRB, 0.65231E+00_JPRB, 0.64496E+00_JPRB, 0.63636E+00_JPRB ,&
&  0.63224E+00_JPRB, 0.64291E+00_JPRB, 0.66794E+00_JPRB, 0.67993E+00_JPRB, 0.68551E+00_JPRB, 0.71560E+00_JPRB, 0.74723E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.77192E+00_JPRB, 0.77559E+00_JPRB, 0.74880E+00_JPRB, 0.73121E+00_JPRB, 0.72440E+00_JPRB, 0.72819E+00_JPRB ,&
&  0.72397E+00_JPRB, 0.72843E+00_JPRB, 0.71941E+00_JPRB, 0.65183E+00_JPRB, 0.65072E+00_JPRB, 0.62051E+00_JPRB, 0.60940E+00_JPRB ,&
&  0.60287E+00_JPRB, 0.60651E+00_JPRB, 0.60071E+00_JPRB, 0.58929E+00_JPRB, 0.58461E+00_JPRB, 0.58807E+00_JPRB, 0.59094E+00_JPRB ,&
&  0.58838E+00_JPRB, 0.57885E+00_JPRB, 0.58372E+00_JPRB, 0.59008E+00_JPRB, 0.59360E+00_JPRB, 0.60101E+00_JPRB, 0.61198E+00_JPRB ,&
&  0.60207E+00_JPRB, 0.59215E+00_JPRB, 0.58342E+00_JPRB, 0.58417E+00_JPRB, 0.59112E+00_JPRB, 0.60468E+00_JPRB, 0.60827E+00_JPRB ,&
&  0.60563E+00_JPRB, 0.60451E+00_JPRB, 0.60614E+00_JPRB, 0.61311E+00_JPRB, 0.62759E+00_JPRB, 0.64169E+00_JPRB, 0.64273E+00_JPRB ,&
&  0.64415E+00_JPRB, 0.64928E+00_JPRB, 0.65548E+00_JPRB, 0.65636E+00_JPRB, 0.65154E+00_JPRB, 0.63601E+00_JPRB, 0.61828E+00_JPRB ,&
&  0.60163E+00_JPRB, 0.58306E+00_JPRB, 0.56811E+00_JPRB, 0.56448E+00_JPRB, 0.57943E+00_JPRB, 0.61033E+00_JPRB, 0.62789E+00_JPRB ,&
&  0.63259E+00_JPRB, 0.63477E+00_JPRB, 0.63729E+00_JPRB, 0.63698E+00_JPRB, 0.63766E+00_JPRB, 0.63451E+00_JPRB, 0.62642E+00_JPRB ,&
&  0.61415E+00_JPRB, 0.60357E+00_JPRB, 0.60285E+00_JPRB, 0.61014E+00_JPRB, 0.61810E+00_JPRB, 0.63330E+00_JPRB, 0.65243E+00_JPRB ,&
&  0.67622E+00_JPRB, 0.71524E+00_JPRB, 0.74755E+00_JPRB, 0.77219E+00_JPRB, 0.78920E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.77040E+00_JPRB, 0.74268E+00_JPRB, 0.72396E+00_JPRB, 0.70968E+00_JPRB, 0.69128E+00_JPRB, 0.67722E+00_JPRB, 0.67152E+00_JPRB ,&
&  0.67960E+00_JPRB, 0.67791E+00_JPRB, 0.66802E+00_JPRB, 0.65994E+00_JPRB, 0.64437E+00_JPRB, 0.63052E+00_JPRB, 0.62492E+00_JPRB ,&
&  0.63314E+00_JPRB, 0.64885E+00_JPRB, 0.66830E+00_JPRB, 0.67531E+00_JPRB, 0.67382E+00_JPRB, 0.68212E+00_JPRB, 0.70150E+00_JPRB ,&
&  0.73039E+00_JPRB, 0.75066E+00_JPRB, 0.75696E+00_JPRB, 0.74243E+00_JPRB, 0.72384E+00_JPRB, 0.71525E+00_JPRB, 0.70125E+00_JPRB ,&
&  0.68472E+00_JPRB, 0.68704E+00_JPRB, 0.62089E+00_JPRB, 0.61310E+00_JPRB, 0.59064E+00_JPRB, 0.57571E+00_JPRB, 0.57138E+00_JPRB ,&
&  0.58391E+00_JPRB, 0.58672E+00_JPRB, 0.57693E+00_JPRB, 0.56340E+00_JPRB, 0.55485E+00_JPRB, 0.54400E+00_JPRB, 0.53454E+00_JPRB ,&
&  0.52879E+00_JPRB, 0.53206E+00_JPRB, 0.54116E+00_JPRB, 0.55060E+00_JPRB, 0.56134E+00_JPRB, 0.57190E+00_JPRB, 0.56872E+00_JPRB ,&
&  0.55710E+00_JPRB, 0.55535E+00_JPRB, 0.56334E+00_JPRB, 0.57934E+00_JPRB, 0.59665E+00_JPRB, 0.60426E+00_JPRB, 0.60637E+00_JPRB ,&
&  0.59631E+00_JPRB, 0.57832E+00_JPRB, 0.56322E+00_JPRB, 0.55102E+00_JPRB, 0.52226E+00_JPRB, 0.46055E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.45080E+00_JPRB, 0.50819E+00_JPRB, 0.57519E+00_JPRB, 0.56865E+00_JPRB, 0.56725E+00_JPRB, 0.55259E+00_JPRB, 0.51405E+00_JPRB ,&
&  0.46673E+00_JPRB, 0.40642E+00_JPRB, 0.38776E+00_JPRB, 0.38537E+00_JPRB, 0.38585E+00_JPRB, 0.39425E+00_JPRB, 0.40777E+00_JPRB ,&
&  0.42528E+00_JPRB, 0.44984E+00_JPRB, 0.46596E+00_JPRB, 0.47149E+00_JPRB, 0.46876E+00_JPRB, 0.45243E+00_JPRB, 0.43787E+00_JPRB ,&
&  0.42916E+00_JPRB, 0.45578E+00_JPRB, 0.51708E+00_JPRB, 0.59907E+00_JPRB, 0.60214E+00_JPRB, 0.63074E+00_JPRB, 0.65785E+00_JPRB ,&
&  0.68343E+00_JPRB, 0.72482E+00_JPRB, 0.75086E+00_JPRB, 0.75587E+00_JPRB, 0.77352E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.72928E+00_JPRB, 0.71574E+00_JPRB, 0.70623E+00_JPRB, 0.69949E+00_JPRB, 0.69194E+00_JPRB, 0.69007E+00_JPRB, 0.68902E+00_JPRB ,&
&  0.68305E+00_JPRB, 0.67473E+00_JPRB, 0.67614E+00_JPRB, 0.65968E+00_JPRB, 0.63579E+00_JPRB, 0.61981E+00_JPRB, 0.61768E+00_JPRB ,&
&  0.62141E+00_JPRB, 0.62845E+00_JPRB, 0.63723E+00_JPRB, 0.64484E+00_JPRB, 0.64930E+00_JPRB, 0.66081E+00_JPRB, 0.68139E+00_JPRB ,&
&  0.70521E+00_JPRB, 0.71855E+00_JPRB, 0.72217E+00_JPRB, 0.71337E+00_JPRB, 0.70067E+00_JPRB, 0.68786E+00_JPRB, 0.67636E+00_JPRB ,&
&  0.66422E+00_JPRB, 0.56261E+00_JPRB, 0.49747E+00_JPRB, 0.50584E+00_JPRB, 0.56981E+00_JPRB, 0.55319E+00_JPRB, 0.55302E+00_JPRB ,&
&  0.56555E+00_JPRB, 0.56412E+00_JPRB, 0.54988E+00_JPRB, 0.53224E+00_JPRB, 0.51599E+00_JPRB, 0.50445E+00_JPRB, 0.49688E+00_JPRB ,&
&  0.49403E+00_JPRB, 0.50151E+00_JPRB, 0.50907E+00_JPRB, 0.52063E+00_JPRB, 0.52280E+00_JPRB, 0.50915E+00_JPRB, 0.49264E+00_JPRB ,&
&  0.47707E+00_JPRB, 0.47131E+00_JPRB, 0.47415E+00_JPRB, 0.48210E+00_JPRB, 0.49252E+00_JPRB, 0.50058E+00_JPRB, 0.49767E+00_JPRB ,&
&  0.47015E+00_JPRB, 0.44648E+00_JPRB, 0.41814E+00_JPRB, 0.39130E+00_JPRB, 0.36826E+00_JPRB, 0.35300E+00_JPRB, 0.34471E+00_JPRB ,&
&  0.34319E+00_JPRB, 0.35242E+00_JPRB, 0.37492E+00_JPRB, 0.42413E+00_JPRB, 0.48420E+00_JPRB, 0.40447E+00_JPRB, 0.36642E+00_JPRB ,&
&  0.34613E+00_JPRB, 0.33268E+00_JPRB, 0.32749E+00_JPRB, 0.32887E+00_JPRB, 0.33413E+00_JPRB, 0.34135E+00_JPRB, 0.34705E+00_JPRB ,&
&  0.35174E+00_JPRB, 0.35509E+00_JPRB, 0.35585E+00_JPRB, 0.35598E+00_JPRB, 0.35699E+00_JPRB, 0.35805E+00_JPRB, 0.35677E+00_JPRB ,&
&  0.35590E+00_JPRB, 0.36390E+00_JPRB, 0.38383E+00_JPRB, 0.41451E+00_JPRB, 0.44899E+00_JPRB, 0.48712E+00_JPRB, 0.54064E+00_JPRB ,&
&  0.60614E+00_JPRB, 0.67736E+00_JPRB, 0.69792E+00_JPRB, 0.71320E+00_JPRB, 0.72403E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.66815E+00_JPRB, 0.67106E+00_JPRB, 0.67404E+00_JPRB, 0.67544E+00_JPRB, 0.66949E+00_JPRB, 0.66171E+00_JPRB, 0.65708E+00_JPRB ,&
&  0.66247E+00_JPRB, 0.66311E+00_JPRB, 0.64043E+00_JPRB, 0.62313E+00_JPRB, 0.61346E+00_JPRB, 0.60995E+00_JPRB, 0.60740E+00_JPRB ,&
&  0.60113E+00_JPRB, 0.59883E+00_JPRB, 0.59949E+00_JPRB, 0.60997E+00_JPRB, 0.61872E+00_JPRB, 0.62624E+00_JPRB, 0.63329E+00_JPRB ,&
&  0.62741E+00_JPRB, 0.63846E+00_JPRB, 0.65941E+00_JPRB, 0.65769E+00_JPRB, 0.65422E+00_JPRB, 0.64636E+00_JPRB, 0.63122E+00_JPRB ,&
&  0.58109E+00_JPRB, 0.51659E+00_JPRB, 0.45883E+00_JPRB, 0.47241E+00_JPRB, 0.54524E+00_JPRB, 0.55097E+00_JPRB, 0.53844E+00_JPRB ,&
&  0.54949E+00_JPRB, 0.54390E+00_JPRB, 0.52558E+00_JPRB, 0.51159E+00_JPRB, 0.49574E+00_JPRB, 0.48548E+00_JPRB, 0.47783E+00_JPRB ,&
&  0.47461E+00_JPRB, 0.47449E+00_JPRB, 0.45767E+00_JPRB, 0.40674E+00_JPRB, 0.37926E+00_JPRB, 0.36167E+00_JPRB, 0.34321E+00_JPRB ,&
&  0.32822E+00_JPRB, 0.31982E+00_JPRB, 0.31925E+00_JPRB, 0.31973E+00_JPRB, 0.32578E+00_JPRB, 0.32951E+00_JPRB, 0.33283E+00_JPRB ,&
&  0.33362E+00_JPRB, 0.33276E+00_JPRB, 0.32890E+00_JPRB, 0.32452E+00_JPRB, 0.32135E+00_JPRB, 0.32126E+00_JPRB, 0.32078E+00_JPRB ,&
&  0.32368E+00_JPRB, 0.32740E+00_JPRB, 0.34480E+00_JPRB, 0.36277E+00_JPRB, 0.35792E+00_JPRB, 0.34285E+00_JPRB, 0.32942E+00_JPRB ,&
&  0.31843E+00_JPRB, 0.31294E+00_JPRB, 0.30932E+00_JPRB, 0.30880E+00_JPRB, 0.31244E+00_JPRB, 0.31775E+00_JPRB, 0.31898E+00_JPRB ,&
&  0.31854E+00_JPRB, 0.31906E+00_JPRB, 0.31894E+00_JPRB, 0.31921E+00_JPRB, 0.31965E+00_JPRB, 0.32253E+00_JPRB, 0.32811E+00_JPRB ,&
&  0.33463E+00_JPRB, 0.34641E+00_JPRB, 0.35942E+00_JPRB, 0.37676E+00_JPRB, 0.38952E+00_JPRB, 0.39954E+00_JPRB, 0.41424E+00_JPRB ,&
&  0.43342E+00_JPRB, 0.47448E+00_JPRB, 0.54376E+00_JPRB, 0.66625E+00_JPRB, 0.65889E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.63422E+00_JPRB, 0.64501E+00_JPRB, 0.65636E+00_JPRB, 0.64756E+00_JPRB, 0.63423E+00_JPRB, 0.62671E+00_JPRB, 0.62678E+00_JPRB ,&
&  0.62712E+00_JPRB, 0.62452E+00_JPRB, 0.61294E+00_JPRB, 0.59866E+00_JPRB, 0.58027E+00_JPRB, 0.53306E+00_JPRB, 0.51669E+00_JPRB ,&
&  0.49951E+00_JPRB, 0.50234E+00_JPRB, 0.50595E+00_JPRB, 0.51151E+00_JPRB, 0.52424E+00_JPRB, 0.53429E+00_JPRB, 0.53506E+00_JPRB ,&
&  0.52392E+00_JPRB, 0.50507E+00_JPRB, 0.48653E+00_JPRB, 0.47255E+00_JPRB, 0.47198E+00_JPRB, 0.47129E+00_JPRB, 0.46359E+00_JPRB ,&
&  0.45014E+00_JPRB, 0.44471E+00_JPRB, 0.44477E+00_JPRB, 0.46436E+00_JPRB, 0.51392E+00_JPRB, 0.53154E+00_JPRB, 0.50840E+00_JPRB ,&
&  0.51297E+00_JPRB, 0.50728E+00_JPRB, 0.50286E+00_JPRB, 0.50341E+00_JPRB, 0.49228E+00_JPRB, 0.46908E+00_JPRB, 0.43494E+00_JPRB ,&
&  0.39380E+00_JPRB, 0.37254E+00_JPRB, 0.35139E+00_JPRB, 0.33017E+00_JPRB, 0.31251E+00_JPRB, 0.30250E+00_JPRB, 0.29411E+00_JPRB ,&
&  0.28875E+00_JPRB, 0.28405E+00_JPRB, 0.28114E+00_JPRB, 0.28157E+00_JPRB, 0.28402E+00_JPRB, 0.28539E+00_JPRB, 0.28693E+00_JPRB ,&
&  0.28906E+00_JPRB, 0.29051E+00_JPRB, 0.29147E+00_JPRB, 0.29345E+00_JPRB, 0.29540E+00_JPRB, 0.29876E+00_JPRB, 0.30294E+00_JPRB ,&
&  0.30816E+00_JPRB, 0.31191E+00_JPRB, 0.31538E+00_JPRB, 0.32249E+00_JPRB, 0.32318E+00_JPRB, 0.32163E+00_JPRB, 0.31318E+00_JPRB ,&
&  0.30684E+00_JPRB, 0.30416E+00_JPRB, 0.30131E+00_JPRB, 0.30059E+00_JPRB, 0.30203E+00_JPRB, 0.30443E+00_JPRB, 0.30689E+00_JPRB ,&
&  0.30864E+00_JPRB, 0.30865E+00_JPRB, 0.30754E+00_JPRB, 0.30719E+00_JPRB, 0.30762E+00_JPRB, 0.30766E+00_JPRB, 0.31180E+00_JPRB ,&
&  0.32115E+00_JPRB, 0.33463E+00_JPRB, 0.34951E+00_JPRB, 0.36924E+00_JPRB, 0.38821E+00_JPRB, 0.40536E+00_JPRB, 0.43391E+00_JPRB ,&
&  0.47192E+00_JPRB, 0.57631E+00_JPRB, 0.64349E+00_JPRB, 0.58841E+00_JPRB, 0.61410E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.60764E+00_JPRB, 0.61507E+00_JPRB, 0.62519E+00_JPRB, 0.62940E+00_JPRB, 0.63695E+00_JPRB, 0.62518E+00_JPRB, 0.61762E+00_JPRB ,&
&  0.58731E+00_JPRB, 0.56681E+00_JPRB, 0.53640E+00_JPRB, 0.49966E+00_JPRB, 0.47487E+00_JPRB, 0.45603E+00_JPRB, 0.44847E+00_JPRB ,&
&  0.43709E+00_JPRB, 0.43048E+00_JPRB, 0.42280E+00_JPRB, 0.41941E+00_JPRB, 0.41615E+00_JPRB, 0.41618E+00_JPRB, 0.41482E+00_JPRB ,&
&  0.41047E+00_JPRB, 0.40466E+00_JPRB, 0.39760E+00_JPRB, 0.39959E+00_JPRB, 0.41224E+00_JPRB, 0.42663E+00_JPRB, 0.44405E+00_JPRB ,&
&  0.46445E+00_JPRB, 0.47451E+00_JPRB, 0.48492E+00_JPRB, 0.48884E+00_JPRB, 0.47334E+00_JPRB, 0.47614E+00_JPRB, 0.48042E+00_JPRB ,&
&  0.48825E+00_JPRB, 0.49306E+00_JPRB, 0.48907E+00_JPRB, 0.48267E+00_JPRB, 0.43036E+00_JPRB, 0.39852E+00_JPRB, 0.36972E+00_JPRB ,&
&  0.34848E+00_JPRB, 0.33514E+00_JPRB, 0.32256E+00_JPRB, 0.31166E+00_JPRB, 0.29939E+00_JPRB, 0.29180E+00_JPRB, 0.28618E+00_JPRB ,&
&  0.28183E+00_JPRB, 0.28025E+00_JPRB, 0.27887E+00_JPRB, 0.27873E+00_JPRB, 0.27972E+00_JPRB, 0.27925E+00_JPRB, 0.27957E+00_JPRB ,&
&  0.27984E+00_JPRB, 0.27961E+00_JPRB, 0.28034E+00_JPRB, 0.28231E+00_JPRB, 0.28482E+00_JPRB, 0.28868E+00_JPRB, 0.28951E+00_JPRB ,&
&  0.29075E+00_JPRB, 0.29184E+00_JPRB, 0.29202E+00_JPRB, 0.29443E+00_JPRB, 0.29581E+00_JPRB, 0.29595E+00_JPRB, 0.29434E+00_JPRB ,&
&  0.29256E+00_JPRB, 0.29115E+00_JPRB, 0.29140E+00_JPRB, 0.29223E+00_JPRB, 0.29425E+00_JPRB, 0.29714E+00_JPRB, 0.29761E+00_JPRB ,&
&  0.29856E+00_JPRB, 0.29784E+00_JPRB, 0.29705E+00_JPRB, 0.29970E+00_JPRB, 0.30269E+00_JPRB, 0.30583E+00_JPRB, 0.30987E+00_JPRB ,&
&  0.31140E+00_JPRB, 0.31944E+00_JPRB, 0.33459E+00_JPRB, 0.35815E+00_JPRB, 0.39788E+00_JPRB, 0.41523E+00_JPRB, 0.45559E+00_JPRB ,&
&  0.50917E+00_JPRB, 0.64281E+00_JPRB, 0.70679E+00_JPRB, 0.57366E+00_JPRB, 0.59170E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.54842E+00_JPRB, 0.56082E+00_JPRB, 0.57690E+00_JPRB, 0.58689E+00_JPRB, 0.59983E+00_JPRB, 0.60184E+00_JPRB, 0.59426E+00_JPRB ,&
&  0.57694E+00_JPRB, 0.54985E+00_JPRB, 0.53374E+00_JPRB, 0.51211E+00_JPRB, 0.49098E+00_JPRB, 0.46897E+00_JPRB, 0.45899E+00_JPRB ,&
&  0.44695E+00_JPRB, 0.43255E+00_JPRB, 0.41811E+00_JPRB, 0.40531E+00_JPRB, 0.39289E+00_JPRB, 0.38165E+00_JPRB, 0.37539E+00_JPRB ,&
&  0.36899E+00_JPRB, 0.36888E+00_JPRB, 0.36833E+00_JPRB, 0.37220E+00_JPRB, 0.38385E+00_JPRB, 0.40216E+00_JPRB, 0.42806E+00_JPRB ,&
&  0.46626E+00_JPRB, 0.46914E+00_JPRB, 0.45484E+00_JPRB, 0.44880E+00_JPRB, 0.44124E+00_JPRB, 0.43523E+00_JPRB, 0.42548E+00_JPRB ,&
&  0.43312E+00_JPRB, 0.44121E+00_JPRB, 0.45593E+00_JPRB, 0.44113E+00_JPRB, 0.39434E+00_JPRB, 0.37538E+00_JPRB, 0.35159E+00_JPRB ,&
&  0.34124E+00_JPRB, 0.33156E+00_JPRB, 0.32354E+00_JPRB, 0.31508E+00_JPRB, 0.30435E+00_JPRB, 0.29829E+00_JPRB, 0.29162E+00_JPRB ,&
&  0.28749E+00_JPRB, 0.28641E+00_JPRB, 0.28415E+00_JPRB, 0.28489E+00_JPRB, 0.28524E+00_JPRB, 0.28473E+00_JPRB, 0.28449E+00_JPRB ,&
&  0.28366E+00_JPRB, 0.28276E+00_JPRB, 0.28260E+00_JPRB, 0.28349E+00_JPRB, 0.28432E+00_JPRB, 0.28508E+00_JPRB, 0.28234E+00_JPRB ,&
&  0.28109E+00_JPRB, 0.27989E+00_JPRB, 0.27992E+00_JPRB, 0.28202E+00_JPRB, 0.28279E+00_JPRB, 0.28422E+00_JPRB, 0.28468E+00_JPRB ,&
&  0.28505E+00_JPRB, 0.28589E+00_JPRB, 0.28548E+00_JPRB, 0.28641E+00_JPRB, 0.28665E+00_JPRB, 0.28791E+00_JPRB, 0.28857E+00_JPRB ,&
&  0.28930E+00_JPRB, 0.29172E+00_JPRB, 0.29423E+00_JPRB, 0.29812E+00_JPRB, 0.30330E+00_JPRB, 0.30611E+00_JPRB, 0.31093E+00_JPRB ,&
&  0.31623E+00_JPRB, 0.32613E+00_JPRB, 0.34640E+00_JPRB, 0.36897E+00_JPRB, 0.39766E+00_JPRB, 0.40838E+00_JPRB, 0.41354E+00_JPRB ,&
&  0.42232E+00_JPRB, 0.45072E+00_JPRB, 0.50134E+00_JPRB, 0.52998E+00_JPRB, 0.53919E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.53598E+00_JPRB, 0.53925E+00_JPRB, 0.54173E+00_JPRB, 0.53960E+00_JPRB, 0.54004E+00_JPRB, 0.54043E+00_JPRB, 0.53786E+00_JPRB ,&
&  0.52973E+00_JPRB, 0.52794E+00_JPRB, 0.52266E+00_JPRB, 0.51445E+00_JPRB, 0.50605E+00_JPRB, 0.49197E+00_JPRB, 0.47520E+00_JPRB ,&
&  0.45291E+00_JPRB, 0.43285E+00_JPRB, 0.41824E+00_JPRB, 0.40396E+00_JPRB, 0.39006E+00_JPRB, 0.37650E+00_JPRB, 0.37219E+00_JPRB ,&
&  0.36793E+00_JPRB, 0.36638E+00_JPRB, 0.36623E+00_JPRB, 0.36923E+00_JPRB, 0.37654E+00_JPRB, 0.38575E+00_JPRB, 0.39602E+00_JPRB ,&
&  0.40690E+00_JPRB, 0.41727E+00_JPRB, 0.41809E+00_JPRB, 0.40991E+00_JPRB, 0.40290E+00_JPRB, 0.39888E+00_JPRB, 0.39553E+00_JPRB ,&
&  0.39161E+00_JPRB, 0.38366E+00_JPRB, 0.37392E+00_JPRB, 0.36340E+00_JPRB, 0.35289E+00_JPRB, 0.34522E+00_JPRB, 0.33624E+00_JPRB ,&
&  0.32960E+00_JPRB, 0.32331E+00_JPRB, 0.31622E+00_JPRB, 0.31007E+00_JPRB, 0.30586E+00_JPRB, 0.30248E+00_JPRB, 0.29890E+00_JPRB ,&
&  0.29747E+00_JPRB, 0.29603E+00_JPRB, 0.29446E+00_JPRB, 0.29310E+00_JPRB, 0.29120E+00_JPRB, 0.28990E+00_JPRB, 0.28875E+00_JPRB ,&
&  0.28794E+00_JPRB, 0.28716E+00_JPRB, 0.28674E+00_JPRB, 0.28622E+00_JPRB, 0.28574E+00_JPRB, 0.28453E+00_JPRB, 0.28314E+00_JPRB ,&
&  0.28245E+00_JPRB, 0.28262E+00_JPRB, 0.28187E+00_JPRB, 0.28159E+00_JPRB, 0.28131E+00_JPRB, 0.28041E+00_JPRB, 0.28110E+00_JPRB ,&
&  0.28268E+00_JPRB, 0.28439E+00_JPRB, 0.28444E+00_JPRB, 0.28551E+00_JPRB, 0.28678E+00_JPRB, 0.28842E+00_JPRB, 0.29133E+00_JPRB ,&
&  0.29345E+00_JPRB, 0.29570E+00_JPRB, 0.29861E+00_JPRB, 0.30092E+00_JPRB, 0.30477E+00_JPRB, 0.30956E+00_JPRB, 0.31552E+00_JPRB ,&
&  0.32373E+00_JPRB, 0.33368E+00_JPRB, 0.34820E+00_JPRB, 0.36615E+00_JPRB, 0.39294E+00_JPRB, 0.40048E+00_JPRB, 0.40686E+00_JPRB ,&
&  0.41070E+00_JPRB, 0.41728E+00_JPRB, 0.50018E+00_JPRB, 0.51407E+00_JPRB, 0.52902E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.41446E+00_JPRB, 0.42385E+00_JPRB, 0.43111E+00_JPRB, 0.43761E+00_JPRB, 0.43106E+00_JPRB, 0.49555E+00_JPRB, 0.47382E+00_JPRB ,&
&  0.46344E+00_JPRB, 0.43769E+00_JPRB, 0.44061E+00_JPRB, 0.43755E+00_JPRB, 0.43195E+00_JPRB, 0.42644E+00_JPRB, 0.42160E+00_JPRB ,&
&  0.41649E+00_JPRB, 0.40924E+00_JPRB, 0.40539E+00_JPRB, 0.40000E+00_JPRB, 0.39469E+00_JPRB, 0.38917E+00_JPRB, 0.38577E+00_JPRB ,&
&  0.38326E+00_JPRB, 0.38177E+00_JPRB, 0.38063E+00_JPRB, 0.37776E+00_JPRB, 0.37913E+00_JPRB, 0.37945E+00_JPRB, 0.37676E+00_JPRB ,&
&  0.37186E+00_JPRB, 0.36753E+00_JPRB, 0.36252E+00_JPRB, 0.35745E+00_JPRB, 0.35552E+00_JPRB, 0.35356E+00_JPRB, 0.35139E+00_JPRB ,&
&  0.34813E+00_JPRB, 0.34441E+00_JPRB, 0.34028E+00_JPRB, 0.33678E+00_JPRB, 0.33326E+00_JPRB, 0.32946E+00_JPRB, 0.32575E+00_JPRB ,&
&  0.32172E+00_JPRB, 0.31768E+00_JPRB, 0.31360E+00_JPRB, 0.31107E+00_JPRB, 0.30896E+00_JPRB, 0.30705E+00_JPRB, 0.30529E+00_JPRB ,&
&  0.30471E+00_JPRB, 0.30396E+00_JPRB, 0.30299E+00_JPRB, 0.30174E+00_JPRB, 0.30069E+00_JPRB, 0.29955E+00_JPRB, 0.29869E+00_JPRB ,&
&  0.29871E+00_JPRB, 0.29813E+00_JPRB, 0.29753E+00_JPRB, 0.29671E+00_JPRB, 0.29638E+00_JPRB, 0.29543E+00_JPRB, 0.29424E+00_JPRB ,&
&  0.29319E+00_JPRB, 0.29223E+00_JPRB, 0.29212E+00_JPRB, 0.29197E+00_JPRB, 0.29180E+00_JPRB, 0.29162E+00_JPRB, 0.29214E+00_JPRB ,&
&  0.29279E+00_JPRB, 0.29362E+00_JPRB, 0.29493E+00_JPRB, 0.29638E+00_JPRB, 0.29766E+00_JPRB, 0.29895E+00_JPRB, 0.30078E+00_JPRB ,&
&  0.30299E+00_JPRB, 0.30523E+00_JPRB, 0.30776E+00_JPRB, 0.30978E+00_JPRB, 0.31365E+00_JPRB, 0.31785E+00_JPRB, 0.32257E+00_JPRB ,&
&  0.32816E+00_JPRB, 0.33598E+00_JPRB, 0.34535E+00_JPRB, 0.35397E+00_JPRB, 0.36422E+00_JPRB, 0.37354E+00_JPRB, 0.38115E+00_JPRB ,&
&  0.38767E+00_JPRB, 0.39449E+00_JPRB, 0.39871E+00_JPRB, 0.40375E+00_JPRB, 0.40907E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.37330E+00_JPRB, 0.37666E+00_JPRB, 0.37918E+00_JPRB, 0.38068E+00_JPRB, 0.37994E+00_JPRB, 0.38092E+00_JPRB, 0.38057E+00_JPRB ,&
&  0.37995E+00_JPRB, 0.37920E+00_JPRB, 0.37985E+00_JPRB, 0.38061E+00_JPRB, 0.38105E+00_JPRB, 0.38138E+00_JPRB, 0.38143E+00_JPRB ,&
&  0.38077E+00_JPRB, 0.37962E+00_JPRB, 0.37426E+00_JPRB, 0.37123E+00_JPRB, 0.36928E+00_JPRB, 0.36814E+00_JPRB, 0.36740E+00_JPRB ,&
&  0.36608E+00_JPRB, 0.36466E+00_JPRB, 0.36260E+00_JPRB, 0.36246E+00_JPRB, 0.36166E+00_JPRB, 0.36057E+00_JPRB, 0.35896E+00_JPRB ,&
&  0.35734E+00_JPRB, 0.35609E+00_JPRB, 0.35460E+00_JPRB, 0.35306E+00_JPRB, 0.35134E+00_JPRB, 0.34918E+00_JPRB, 0.34626E+00_JPRB ,&
&  0.34329E+00_JPRB, 0.34031E+00_JPRB, 0.33783E+00_JPRB, 0.33546E+00_JPRB, 0.33340E+00_JPRB, 0.33169E+00_JPRB, 0.33035E+00_JPRB ,&
&  0.32885E+00_JPRB, 0.32712E+00_JPRB, 0.32535E+00_JPRB, 0.32386E+00_JPRB, 0.32239E+00_JPRB, 0.32111E+00_JPRB, 0.32039E+00_JPRB ,&
&  0.31999E+00_JPRB, 0.31950E+00_JPRB, 0.31891E+00_JPRB, 0.31832E+00_JPRB, 0.31762E+00_JPRB, 0.31697E+00_JPRB, 0.31631E+00_JPRB ,&
&  0.31574E+00_JPRB, 0.31547E+00_JPRB, 0.31505E+00_JPRB, 0.31462E+00_JPRB, 0.31435E+00_JPRB, 0.31389E+00_JPRB, 0.31340E+00_JPRB ,&
&  0.31285E+00_JPRB, 0.31264E+00_JPRB, 0.31237E+00_JPRB, 0.31201E+00_JPRB, 0.31166E+00_JPRB, 0.31142E+00_JPRB, 0.31146E+00_JPRB ,&
&  0.31146E+00_JPRB, 0.31137E+00_JPRB, 0.31146E+00_JPRB, 0.31236E+00_JPRB, 0.31332E+00_JPRB, 0.31448E+00_JPRB, 0.31577E+00_JPRB ,&
&  0.31741E+00_JPRB, 0.31914E+00_JPRB, 0.32110E+00_JPRB, 0.32298E+00_JPRB, 0.32557E+00_JPRB, 0.32813E+00_JPRB, 0.33070E+00_JPRB ,&
&  0.33318E+00_JPRB, 0.33578E+00_JPRB, 0.33846E+00_JPRB, 0.34127E+00_JPRB, 0.34512E+00_JPRB, 0.34805E+00_JPRB, 0.35135E+00_JPRB ,&
&  0.35453E+00_JPRB, 0.35823E+00_JPRB, 0.36181E+00_JPRB, 0.36502E+00_JPRB, 0.36810E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB, 0.33839E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB, 0.58848E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.59706E+00_JPRB, 0.59701E+00_JPRB, 0.59702E+00_JPRB, 0.59720E+00_JPRB, 0.59699E+00_JPRB, 0.59652E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.59490E+00_JPRB, 0.59395E+00_JPRB, 0.59265E+00_JPRB, 0.59130E+00_JPRB, 0.58998E+00_JPRB, 0.58864E+00_JPRB, 0.58689E+00_JPRB ,&
&  0.58497E+00_JPRB, 0.58276E+00_JPRB, 0.58044E+00_JPRB, 0.57818E+00_JPRB, 0.57653E+00_JPRB, 0.57520E+00_JPRB, 0.57432E+00_JPRB ,&
&  0.57342E+00_JPRB, 0.57265E+00_JPRB, 0.57194E+00_JPRB, 0.57144E+00_JPRB, 0.57126E+00_JPRB, 0.57093E+00_JPRB, 0.57068E+00_JPRB ,&
&  0.57047E+00_JPRB, 0.57048E+00_JPRB, 0.57028E+00_JPRB, 0.57007E+00_JPRB, 0.56983E+00_JPRB, 0.57005E+00_JPRB, 0.57011E+00_JPRB ,&
&  0.57039E+00_JPRB, 0.57056E+00_JPRB, 0.57067E+00_JPRB, 0.57065E+00_JPRB, 0.57077E+00_JPRB, 0.57088E+00_JPRB, 0.57093E+00_JPRB ,&
&  0.57108E+00_JPRB, 0.57155E+00_JPRB, 0.57265E+00_JPRB, 0.57411E+00_JPRB, 0.57555E+00_JPRB, 0.57714E+00_JPRB, 0.57878E+00_JPRB ,&
&  0.58046E+00_JPRB, 0.58193E+00_JPRB, 0.58369E+00_JPRB, 0.58541E+00_JPRB, 0.58740E+00_JPRB, 0.58924E+00_JPRB, 0.59121E+00_JPRB ,&
&  0.59306E+00_JPRB, 0.59508E+00_JPRB, 0.59740E+00_JPRB, 0.59980E+00_JPRB, 0.60208E+00_JPRB, 0.60408E+00_JPRB, 0.60622E+00_JPRB ,&
&  0.60848E+00_JPRB, 0.61063E+00_JPRB, 0.61225E+00_JPRB, 0.61400E+00_JPRB, 0.61582E+00_JPRB, 0.61763E+00_JPRB, 0.61894E+00_JPRB ,&
&  0.62030E+00_JPRB, 0.62180E+00_JPRB, 0.62328E+00_JPRB, 0.62372E+00_JPRB, 0.62404E+00_JPRB, 0.62418E+00_JPRB, 0.62394E+00_JPRB ,&
&  0.62294E+00_JPRB, 0.62181E+00_JPRB, 0.62065E+00_JPRB, 0.61932E+00_JPRB, 0.61761E+00_JPRB, 0.61569E+00_JPRB, 0.61351E+00_JPRB ,&
&  0.61123E+00_JPRB, 0.60850E+00_JPRB, 0.60591E+00_JPRB, 0.60402E+00_JPRB, 0.60278E+00_JPRB, 0.60137E+00_JPRB, 0.59992E+00_JPRB ,&
&  0.59857E+00_JPRB, 0.59807E+00_JPRB, 0.59778E+00_JPRB, 0.59747E+00_JPRB, 0.59732E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.59997E+00_JPRB, 0.60040E+00_JPRB, 0.60106E+00_JPRB, 0.60131E+00_JPRB, 0.60015E+00_JPRB, 0.59905E+00_JPRB, 0.59780E+00_JPRB ,&
&  0.59690E+00_JPRB, 0.59597E+00_JPRB, 0.59453E+00_JPRB, 0.59272E+00_JPRB, 0.59092E+00_JPRB, 0.58914E+00_JPRB, 0.58732E+00_JPRB ,&
&  0.58562E+00_JPRB, 0.58378E+00_JPRB, 0.58164E+00_JPRB, 0.57775E+00_JPRB, 0.57401E+00_JPRB, 0.57093E+00_JPRB, 0.56958E+00_JPRB ,&
&  0.56805E+00_JPRB, 0.56653E+00_JPRB, 0.56436E+00_JPRB, 0.56175E+00_JPRB, 0.55914E+00_JPRB, 0.55680E+00_JPRB, 0.55440E+00_JPRB ,&
&  0.55237E+00_JPRB, 0.55060E+00_JPRB, 0.54922E+00_JPRB, 0.54758E+00_JPRB, 0.54827E+00_JPRB, 0.54901E+00_JPRB, 0.54962E+00_JPRB ,&
&  0.55020E+00_JPRB, 0.55019E+00_JPRB, 0.55141E+00_JPRB, 0.55269E+00_JPRB, 0.55389E+00_JPRB, 0.55594E+00_JPRB, 0.55715E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.55793E+00_JPRB, 0.56087E+00_JPRB, 0.56315E+00_JPRB, 0.56500E+00_JPRB, 0.56666E+00_JPRB, 0.56937E+00_JPRB ,&
&  0.57210E+00_JPRB, 0.57487E+00_JPRB, 0.57791E+00_JPRB, 0.58180E+00_JPRB, 0.58557E+00_JPRB, 0.58984E+00_JPRB, 0.59420E+00_JPRB ,&
&  0.59832E+00_JPRB, 0.60265E+00_JPRB, 0.60703E+00_JPRB, 0.61130E+00_JPRB, 0.61519E+00_JPRB, 0.61854E+00_JPRB, 0.62239E+00_JPRB ,&
&  0.62647E+00_JPRB, 0.63112E+00_JPRB, 0.63406E+00_JPRB, 0.63720E+00_JPRB, 0.64067E+00_JPRB, 0.64537E+00_JPRB, 0.64735E+00_JPRB ,&
&  0.64919E+00_JPRB, 0.65093E+00_JPRB, 0.65281E+00_JPRB, 0.65370E+00_JPRB, 0.65460E+00_JPRB, 0.65547E+00_JPRB, 0.65609E+00_JPRB ,&
&  0.65595E+00_JPRB, 0.65492E+00_JPRB, 0.65293E+00_JPRB, 0.64990E+00_JPRB, 0.64627E+00_JPRB, 0.64257E+00_JPRB, 0.63807E+00_JPRB ,&
&  0.63397E+00_JPRB, 0.62894E+00_JPRB, 0.62412E+00_JPRB, 0.62063E+00_JPRB, 0.61826E+00_JPRB, 0.61435E+00_JPRB, 0.61020E+00_JPRB ,&
&  0.60653E+00_JPRB, 0.60371E+00_JPRB, 0.60235E+00_JPRB, 0.60115E+00_JPRB, 0.60055E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.60440E+00_JPRB, 0.60494E+00_JPRB, 0.60525E+00_JPRB, 0.60498E+00_JPRB, 0.60323E+00_JPRB, 0.60166E+00_JPRB, 0.59993E+00_JPRB ,&
&  0.59855E+00_JPRB, 0.59671E+00_JPRB, 0.59520E+00_JPRB, 0.59386E+00_JPRB, 0.59268E+00_JPRB, 0.59189E+00_JPRB, 0.59109E+00_JPRB ,&
&  0.59062E+00_JPRB, 0.58974E+00_JPRB, 0.58910E+00_JPRB, 0.58743E+00_JPRB, 0.58409E+00_JPRB, 0.57875E+00_JPRB, 0.57279E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.56187E+00_JPRB, 0.55627E+00_JPRB, 0.55080E+00_JPRB, 0.54613E+00_JPRB, 0.51678E+00_JPRB, 0.50837E+00_JPRB ,&
&  0.51017E+00_JPRB, 0.51183E+00_JPRB, 0.51595E+00_JPRB, 0.52120E+00_JPRB, 0.52308E+00_JPRB, 0.51518E+00_JPRB, 0.51131E+00_JPRB ,&
&  0.49039E+00_JPRB, 0.49308E+00_JPRB, 0.49562E+00_JPRB, 0.49861E+00_JPRB, 0.50474E+00_JPRB, 0.50986E+00_JPRB, 0.51894E+00_JPRB ,&
&  0.53933E+00_JPRB, 0.54306E+00_JPRB, 0.54747E+00_JPRB, 0.55256E+00_JPRB, 0.55679E+00_JPRB, 0.56057E+00_JPRB, 0.56503E+00_JPRB ,&
&  0.56858E+00_JPRB, 0.57241E+00_JPRB, 0.57634E+00_JPRB, 0.58081E+00_JPRB, 0.58611E+00_JPRB, 0.59178E+00_JPRB, 0.59769E+00_JPRB ,&
&  0.60329E+00_JPRB, 0.60995E+00_JPRB, 0.61687E+00_JPRB, 0.62380E+00_JPRB, 0.63049E+00_JPRB, 0.63703E+00_JPRB, 0.64416E+00_JPRB ,&
&  0.65078E+00_JPRB, 0.65777E+00_JPRB, 0.66273E+00_JPRB, 0.66807E+00_JPRB, 0.67174E+00_JPRB, 0.67591E+00_JPRB, 0.67701E+00_JPRB ,&
&  0.67857E+00_JPRB, 0.67682E+00_JPRB, 0.67674E+00_JPRB, 0.67558E+00_JPRB, 0.67465E+00_JPRB, 0.67426E+00_JPRB, 0.67343E+00_JPRB ,&
&  0.67327E+00_JPRB, 0.67209E+00_JPRB, 0.67089E+00_JPRB, 0.66808E+00_JPRB, 0.66379E+00_JPRB, 0.65831E+00_JPRB, 0.65244E+00_JPRB ,&
&  0.64808E+00_JPRB, 0.64488E+00_JPRB, 0.64206E+00_JPRB, 0.63978E+00_JPRB, 0.63579E+00_JPRB, 0.63042E+00_JPRB, 0.62438E+00_JPRB ,&
&  0.61927E+00_JPRB, 0.61429E+00_JPRB, 0.61035E+00_JPRB, 0.60709E+00_JPRB, 0.60530E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.61200E+00_JPRB, 0.61337E+00_JPRB, 0.61416E+00_JPRB, 0.61457E+00_JPRB, 0.61313E+00_JPRB, 0.61172E+00_JPRB, 0.61025E+00_JPRB ,&
&  0.60862E+00_JPRB, 0.60706E+00_JPRB, 0.60540E+00_JPRB, 0.60408E+00_JPRB, 0.60247E+00_JPRB, 0.60119E+00_JPRB, 0.59916E+00_JPRB ,&
&  0.59827E+00_JPRB, 0.59643E+00_JPRB, 0.59453E+00_JPRB, 0.59044E+00_JPRB, 0.58627E+00_JPRB, 0.58126E+00_JPRB, 0.57213E+00_JPRB ,&
&  0.56205E+00_JPRB, 0.55275E+00_JPRB, 0.52859E+00_JPRB, 0.52273E+00_JPRB, 0.51079E+00_JPRB, 0.50410E+00_JPRB, 0.50098E+00_JPRB ,&
&  0.49564E+00_JPRB, 0.49180E+00_JPRB, 0.48143E+00_JPRB, 0.47241E+00_JPRB, 0.46612E+00_JPRB, 0.45530E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.44332E+00_JPRB, 0.44014E+00_JPRB, 0.44157E+00_JPRB, 0.44547E+00_JPRB, 0.45586E+00_JPRB, 0.46766E+00_JPRB, 0.48366E+00_JPRB ,&
&  0.50620E+00_JPRB, 0.52656E+00_JPRB, 0.54576E+00_JPRB, 0.54775E+00_JPRB, 0.55494E+00_JPRB, 0.56118E+00_JPRB, 0.56569E+00_JPRB ,&
&  0.56276E+00_JPRB, 0.56281E+00_JPRB, 0.57363E+00_JPRB, 0.57801E+00_JPRB, 0.58330E+00_JPRB, 0.58945E+00_JPRB, 0.59578E+00_JPRB ,&
&  0.60260E+00_JPRB, 0.61022E+00_JPRB, 0.61817E+00_JPRB, 0.62769E+00_JPRB, 0.63922E+00_JPRB, 0.65080E+00_JPRB, 0.66062E+00_JPRB ,&
&  0.66944E+00_JPRB, 0.68021E+00_JPRB, 0.68811E+00_JPRB, 0.69561E+00_JPRB, 0.70288E+00_JPRB, 0.71312E+00_JPRB, 0.72125E+00_JPRB ,&
&  0.72596E+00_JPRB, 0.72348E+00_JPRB, 0.71683E+00_JPRB, 0.70361E+00_JPRB, 0.69701E+00_JPRB, 0.70239E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.69882E+00_JPRB, 0.69554E+00_JPRB, 0.69305E+00_JPRB, 0.68882E+00_JPRB, 0.68418E+00_JPRB, 0.67732E+00_JPRB, 0.66891E+00_JPRB ,&
&  0.66194E+00_JPRB, 0.65853E+00_JPRB, 0.65548E+00_JPRB, 0.65242E+00_JPRB, 0.64605E+00_JPRB, 0.63949E+00_JPRB, 0.63303E+00_JPRB ,&
&  0.62736E+00_JPRB, 0.62217E+00_JPRB, 0.61721E+00_JPRB, 0.61392E+00_JPRB, 0.61224E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.62792E+00_JPRB, 0.62952E+00_JPRB, 0.62915E+00_JPRB, 0.62880E+00_JPRB, 0.62688E+00_JPRB, 0.62520E+00_JPRB, 0.62264E+00_JPRB ,&
&  0.62025E+00_JPRB, 0.61785E+00_JPRB, 0.61558E+00_JPRB, 0.61352E+00_JPRB, 0.61141E+00_JPRB, 0.60929E+00_JPRB, 0.60789E+00_JPRB ,&
&  0.60640E+00_JPRB, 0.60384E+00_JPRB, 0.59977E+00_JPRB, 0.59464E+00_JPRB, 0.59016E+00_JPRB, 0.58516E+00_JPRB, 0.57882E+00_JPRB ,&
&  0.57201E+00_JPRB, 0.56220E+00_JPRB, 0.54620E+00_JPRB, 0.53964E+00_JPRB, 0.53016E+00_JPRB, 0.52671E+00_JPRB, 0.52061E+00_JPRB ,&
&  0.51696E+00_JPRB, 0.48234E+00_JPRB, 0.48247E+00_JPRB, 0.46970E+00_JPRB, 0.46064E+00_JPRB, 0.45370E+00_JPRB, 0.43824E+00_JPRB ,&
&  0.43146E+00_JPRB, 0.42252E+00_JPRB, 0.41885E+00_JPRB, 0.42255E+00_JPRB, 0.42960E+00_JPRB, 0.44409E+00_JPRB, 0.46292E+00_JPRB ,&
&  0.48535E+00_JPRB, 0.53256E+00_JPRB, 0.54469E+00_JPRB, 0.55328E+00_JPRB, 0.56198E+00_JPRB, 0.56858E+00_JPRB, 0.56067E+00_JPRB ,&
&  0.55925E+00_JPRB, 0.55758E+00_JPRB, 0.56141E+00_JPRB, 0.56955E+00_JPRB, 0.57960E+00_JPRB, 0.58343E+00_JPRB, 0.58906E+00_JPRB ,&
&  0.59433E+00_JPRB, 0.60302E+00_JPRB, 0.61227E+00_JPRB, 0.62577E+00_JPRB, 0.64512E+00_JPRB, 0.66542E+00_JPRB, 0.68298E+00_JPRB ,&
&  0.69713E+00_JPRB, 0.70522E+00_JPRB, 0.70900E+00_JPRB, 0.71761E+00_JPRB, 0.73029E+00_JPRB, 0.74867E+00_JPRB, 0.76052E+00_JPRB ,&
&  0.76789E+00_JPRB, 0.77448E+00_JPRB, 0.77277E+00_JPRB, 0.76675E+00_JPRB, 0.75279E+00_JPRB, 0.73913E+00_JPRB, 0.73236E+00_JPRB ,&
&  0.73503E+00_JPRB, 0.73019E+00_JPRB, 0.72246E+00_JPRB, 0.71082E+00_JPRB, 0.70325E+00_JPRB, 0.69342E+00_JPRB, 0.68323E+00_JPRB ,&
&  0.67451E+00_JPRB, 0.66956E+00_JPRB, 0.66354E+00_JPRB, 0.65867E+00_JPRB, 0.65054E+00_JPRB, 0.64421E+00_JPRB, 0.63940E+00_JPRB ,&
&  0.63457E+00_JPRB, 0.63108E+00_JPRB, 0.62717E+00_JPRB, 0.62587E+00_JPRB, 0.62694E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.64430E+00_JPRB, 0.64465E+00_JPRB, 0.64399E+00_JPRB, 0.64326E+00_JPRB, 0.64189E+00_JPRB, 0.64141E+00_JPRB, 0.64148E+00_JPRB ,&
&  0.64062E+00_JPRB, 0.63981E+00_JPRB, 0.63875E+00_JPRB, 0.63748E+00_JPRB, 0.63591E+00_JPRB, 0.63450E+00_JPRB, 0.63288E+00_JPRB ,&
&  0.62950E+00_JPRB, 0.62471E+00_JPRB, 0.61799E+00_JPRB, 0.61137E+00_JPRB, 0.60532E+00_JPRB, 0.60165E+00_JPRB, 0.59595E+00_JPRB ,&
&  0.59019E+00_JPRB, 0.58176E+00_JPRB, 0.56883E+00_JPRB, 0.55992E+00_JPRB, 0.55762E+00_JPRB, 0.55014E+00_JPRB, 0.54967E+00_JPRB ,&
&  0.54655E+00_JPRB, 0.54105E+00_JPRB, 0.52542E+00_JPRB, 0.51928E+00_JPRB, 0.50893E+00_JPRB, 0.46114E+00_JPRB, 0.45093E+00_JPRB ,&
&  0.43676E+00_JPRB, 0.42006E+00_JPRB, 0.41362E+00_JPRB, 0.41147E+00_JPRB, 0.41484E+00_JPRB, 0.43081E+00_JPRB, 0.45182E+00_JPRB ,&
&  0.47302E+00_JPRB, 0.53201E+00_JPRB, 0.54763E+00_JPRB, 0.56061E+00_JPRB, 0.56981E+00_JPRB, 0.57478E+00_JPRB, 0.57099E+00_JPRB ,&
&  0.56870E+00_JPRB, 0.56824E+00_JPRB, 0.57399E+00_JPRB, 0.58415E+00_JPRB, 0.59148E+00_JPRB, 0.58981E+00_JPRB, 0.58952E+00_JPRB ,&
&  0.58522E+00_JPRB, 0.59090E+00_JPRB, 0.60291E+00_JPRB, 0.61979E+00_JPRB, 0.64272E+00_JPRB, 0.66065E+00_JPRB, 0.68520E+00_JPRB ,&
&  0.70597E+00_JPRB, 0.71944E+00_JPRB, 0.74223E+00_JPRB, 0.76584E+00_JPRB, 0.78260E+00_JPRB, 0.79331E+00_JPRB, 0.79636E+00_JPRB ,&
&  0.79450E+00_JPRB, 0.81306E+00_JPRB, 0.82312E+00_JPRB, 0.82008E+00_JPRB, 0.81076E+00_JPRB, 0.79270E+00_JPRB, 0.77395E+00_JPRB ,&
&  0.76515E+00_JPRB, 0.76830E+00_JPRB, 0.75996E+00_JPRB, 0.73808E+00_JPRB, 0.72082E+00_JPRB, 0.70665E+00_JPRB, 0.69526E+00_JPRB ,&
&  0.68659E+00_JPRB, 0.68134E+00_JPRB, 0.67388E+00_JPRB, 0.66745E+00_JPRB, 0.66015E+00_JPRB, 0.65388E+00_JPRB, 0.65041E+00_JPRB ,&
&  0.64770E+00_JPRB, 0.64736E+00_JPRB, 0.64627E+00_JPRB, 0.64526E+00_JPRB, 0.64566E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.66093E+00_JPRB, 0.66266E+00_JPRB, 0.66325E+00_JPRB, 0.66312E+00_JPRB, 0.66363E+00_JPRB, 0.66615E+00_JPRB, 0.66558E+00_JPRB ,&
&  0.66315E+00_JPRB, 0.66103E+00_JPRB, 0.65712E+00_JPRB, 0.65605E+00_JPRB, 0.65446E+00_JPRB, 0.65242E+00_JPRB, 0.65057E+00_JPRB ,&
&  0.64707E+00_JPRB, 0.63495E+00_JPRB, 0.62715E+00_JPRB, 0.62406E+00_JPRB, 0.62042E+00_JPRB, 0.61843E+00_JPRB, 0.61645E+00_JPRB ,&
&  0.61016E+00_JPRB, 0.60130E+00_JPRB, 0.59241E+00_JPRB, 0.57784E+00_JPRB, 0.56744E+00_JPRB, 0.56862E+00_JPRB, 0.57393E+00_JPRB ,&
&  0.56747E+00_JPRB, 0.55110E+00_JPRB, 0.55251E+00_JPRB, 0.54934E+00_JPRB, 0.52647E+00_JPRB, 0.51743E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.45398E+00_JPRB, 0.42991E+00_JPRB, 0.42142E+00_JPRB, 0.41585E+00_JPRB, 0.42091E+00_JPRB, 0.43396E+00_JPRB, 0.46494E+00_JPRB ,&
&  0.49748E+00_JPRB, 0.53217E+00_JPRB, 0.54522E+00_JPRB, 0.55938E+00_JPRB, 0.56765E+00_JPRB, 0.58062E+00_JPRB, 0.59104E+00_JPRB ,&
&  0.59695E+00_JPRB, 0.60157E+00_JPRB, 0.60957E+00_JPRB, 0.61622E+00_JPRB, 0.63104E+00_JPRB, 0.64324E+00_JPRB, 0.64865E+00_JPRB ,&
&  0.65836E+00_JPRB, 0.66043E+00_JPRB, 0.67032E+00_JPRB, 0.68232E+00_JPRB, 0.69779E+00_JPRB, 0.70629E+00_JPRB, 0.71643E+00_JPRB ,&
&  0.72824E+00_JPRB, 0.75599E+00_JPRB, 0.79741E+00_JPRB, 0.84948E+00_JPRB, 0.86474E+00_JPRB, 0.86111E+00_JPRB, 0.84075E+00_JPRB ,&
&  0.81608E+00_JPRB, 0.83976E+00_JPRB, 0.94630E+00_JPRB, 0.93282E+00_JPRB, 0.92060E+00_JPRB, 0.90288E+00_JPRB, 0.86120E+00_JPRB ,&
&  0.81240E+00_JPRB, 0.79263E+00_JPRB, 0.77442E+00_JPRB, 0.74318E+00_JPRB, 0.71838E+00_JPRB, 0.70168E+00_JPRB, 0.69249E+00_JPRB ,&
&  0.68655E+00_JPRB, 0.68558E+00_JPRB, 0.68044E+00_JPRB, 0.67689E+00_JPRB, 0.67019E+00_JPRB, 0.66328E+00_JPRB, 0.66064E+00_JPRB ,&
&  0.65852E+00_JPRB, 0.65935E+00_JPRB, 0.66115E+00_JPRB, 0.66223E+00_JPRB, 0.66225E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.67699E+00_JPRB, 0.68371E+00_JPRB, 0.68899E+00_JPRB, 0.69845E+00_JPRB, 0.70713E+00_JPRB, 0.71003E+00_JPRB, 0.70575E+00_JPRB ,&
&  0.69380E+00_JPRB, 0.67875E+00_JPRB, 0.66753E+00_JPRB, 0.65937E+00_JPRB, 0.65455E+00_JPRB, 0.65172E+00_JPRB, 0.64893E+00_JPRB ,&
&  0.64684E+00_JPRB, 0.64446E+00_JPRB, 0.63812E+00_JPRB, 0.62913E+00_JPRB, 0.62418E+00_JPRB, 0.62383E+00_JPRB, 0.62369E+00_JPRB ,&
&  0.62083E+00_JPRB, 0.61410E+00_JPRB, 0.60559E+00_JPRB, 0.59153E+00_JPRB, 0.57968E+00_JPRB, 0.58203E+00_JPRB, 0.57663E+00_JPRB ,&
&  0.57175E+00_JPRB, 0.56898E+00_JPRB, 0.56510E+00_JPRB, 0.55991E+00_JPRB, 0.55383E+00_JPRB, 0.53613E+00_JPRB, 0.50437E+00_JPRB ,&
&  0.47158E+00_JPRB, 0.44565E+00_JPRB, 0.43023E+00_JPRB, 0.42558E+00_JPRB, 0.43462E+00_JPRB, 0.45826E+00_JPRB, 0.49116E+00_JPRB ,&
&  0.52804E+00_JPRB, 0.54547E+00_JPRB, 0.56192E+00_JPRB, 0.57919E+00_JPRB, 0.59297E+00_JPRB, 0.61020E+00_JPRB, 0.62577E+00_JPRB ,&
&  0.64240E+00_JPRB, 0.66613E+00_JPRB, 0.69834E+00_JPRB, 0.74068E+00_JPRB, 0.79167E+00_JPRB, 0.78111E+00_JPRB, 0.82191E+00_JPRB ,&
&  0.94020E+00_JPRB, 0.94150E+00_JPRB, 0.90276E+00_JPRB, 0.85844E+00_JPRB, 0.83060E+00_JPRB, 0.82125E+00_JPRB, 0.82542E+00_JPRB ,&
&  0.83393E+00_JPRB, 0.85210E+00_JPRB, 0.88363E+00_JPRB, 0.92229E+00_JPRB, 0.93468E+00_JPRB, 0.92946E+00_JPRB, 0.90012E+00_JPRB ,&
&  0.87381E+00_JPRB, 0.89930E+00_JPRB, 0.11163E+01_JPRB, 0.10256E+01_JPRB, 0.94344E+00_JPRB, 0.90361E+00_JPRB, 0.85540E+00_JPRB ,&
&  0.81993E+00_JPRB, 0.79196E+00_JPRB, 0.76216E+00_JPRB, 0.73830E+00_JPRB, 0.71031E+00_JPRB, 0.68480E+00_JPRB, 0.66259E+00_JPRB ,&
&  0.65535E+00_JPRB, 0.65440E+00_JPRB, 0.65516E+00_JPRB, 0.65642E+00_JPRB, 0.65361E+00_JPRB, 0.65395E+00_JPRB, 0.65654E+00_JPRB ,&
&  0.65386E+00_JPRB, 0.65440E+00_JPRB, 0.66016E+00_JPRB, 0.66617E+00_JPRB, 0.67139E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.66090E+00_JPRB, 0.68102E+00_JPRB, 0.71284E+00_JPRB, 0.73458E+00_JPRB, 0.74361E+00_JPRB, 0.72964E+00_JPRB, 0.71039E+00_JPRB ,&
&  0.68870E+00_JPRB, 0.66159E+00_JPRB, 0.64564E+00_JPRB, 0.63626E+00_JPRB, 0.63092E+00_JPRB, 0.62704E+00_JPRB, 0.62384E+00_JPRB ,&
&  0.62150E+00_JPRB, 0.61973E+00_JPRB, 0.62044E+00_JPRB, 0.62378E+00_JPRB, 0.62585E+00_JPRB, 0.62376E+00_JPRB, 0.62045E+00_JPRB ,&
&  0.61776E+00_JPRB, 0.61302E+00_JPRB, 0.60228E+00_JPRB, 0.58788E+00_JPRB, 0.58315E+00_JPRB, 0.59161E+00_JPRB, 0.58985E+00_JPRB ,&
&  0.58373E+00_JPRB, 0.57661E+00_JPRB, 0.57574E+00_JPRB, 0.57622E+00_JPRB, 0.57103E+00_JPRB, 0.55687E+00_JPRB, 0.52808E+00_JPRB ,&
&  0.49211E+00_JPRB, 0.46330E+00_JPRB, 0.45480E+00_JPRB, 0.46879E+00_JPRB, 0.49670E+00_JPRB, 0.53490E+00_JPRB, 0.55207E+00_JPRB ,&
&  0.56540E+00_JPRB, 0.57740E+00_JPRB, 0.58935E+00_JPRB, 0.60910E+00_JPRB, 0.63191E+00_JPRB, 0.65201E+00_JPRB, 0.67165E+00_JPRB ,&
&  0.70216E+00_JPRB, 0.73687E+00_JPRB, 0.78030E+00_JPRB, 0.75228E+00_JPRB, 0.77652E+00_JPRB, 0.84027E+00_JPRB, 0.87519E+00_JPRB ,&
&  0.89281E+00_JPRB, 0.93056E+00_JPRB, 0.95332E+00_JPRB, 0.98394E+00_JPRB, 0.97109E+00_JPRB, 0.95254E+00_JPRB, 0.93258E+00_JPRB ,&
&  0.93040E+00_JPRB, 0.92880E+00_JPRB, 0.93798E+00_JPRB, 0.97748E+00_JPRB, 0.10153E+01_JPRB, 0.10181E+01_JPRB, 0.99185E+00_JPRB ,&
&  0.94892E+00_JPRB, 0.88253E+00_JPRB, 0.82088E+00_JPRB, 0.78050E+00_JPRB, 0.77916E+00_JPRB, 0.79562E+00_JPRB, 0.80394E+00_JPRB ,&
&  0.80070E+00_JPRB, 0.79413E+00_JPRB, 0.77632E+00_JPRB, 0.76107E+00_JPRB, 0.70988E+00_JPRB, 0.67339E+00_JPRB, 0.64480E+00_JPRB ,&
&  0.63395E+00_JPRB, 0.62852E+00_JPRB, 0.62762E+00_JPRB, 0.63572E+00_JPRB, 0.63387E+00_JPRB, 0.63687E+00_JPRB, 0.64006E+00_JPRB ,&
&  0.63917E+00_JPRB, 0.63716E+00_JPRB, 0.64026E+00_JPRB, 0.64477E+00_JPRB, 0.65093E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.73058E+00_JPRB, 0.74795E+00_JPRB, 0.76351E+00_JPRB, 0.77100E+00_JPRB, 0.76183E+00_JPRB, 0.75307E+00_JPRB, 0.74352E+00_JPRB ,&
&  0.73774E+00_JPRB, 0.68684E+00_JPRB, 0.65251E+00_JPRB, 0.62781E+00_JPRB, 0.60966E+00_JPRB, 0.59347E+00_JPRB, 0.58892E+00_JPRB ,&
&  0.59068E+00_JPRB, 0.59479E+00_JPRB, 0.59863E+00_JPRB, 0.60258E+00_JPRB, 0.60635E+00_JPRB, 0.60762E+00_JPRB, 0.61048E+00_JPRB ,&
&  0.61498E+00_JPRB, 0.61622E+00_JPRB, 0.60794E+00_JPRB, 0.60097E+00_JPRB, 0.60001E+00_JPRB, 0.60316E+00_JPRB, 0.59941E+00_JPRB ,&
&  0.59452E+00_JPRB, 0.58940E+00_JPRB, 0.58694E+00_JPRB, 0.58837E+00_JPRB, 0.58586E+00_JPRB, 0.57295E+00_JPRB, 0.54042E+00_JPRB ,&
&  0.50148E+00_JPRB, 0.48756E+00_JPRB, 0.50623E+00_JPRB, 0.55231E+00_JPRB, 0.57059E+00_JPRB, 0.58358E+00_JPRB, 0.59743E+00_JPRB ,&
&  0.59242E+00_JPRB, 0.59236E+00_JPRB, 0.61068E+00_JPRB, 0.65453E+00_JPRB, 0.67390E+00_JPRB, 0.69134E+00_JPRB, 0.71265E+00_JPRB ,&
&  0.75183E+00_JPRB, 0.78402E+00_JPRB, 0.82160E+00_JPRB, 0.78999E+00_JPRB, 0.85973E+00_JPRB, 0.10218E+01_JPRB, 0.99447E+00_JPRB ,&
&  0.99478E+00_JPRB, 0.10035E+01_JPRB, 0.10521E+01_JPRB, 0.10835E+01_JPRB, 0.10828E+01_JPRB, 0.10654E+01_JPRB, 0.10567E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.10392E+01_JPRB, 0.10534E+01_JPRB, 0.11083E+01_JPRB, 0.11095E+01_JPRB, 0.10920E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.10158E+01_JPRB, 0.94571E+00_JPRB, 0.88102E+00_JPRB, 0.83351E+00_JPRB, 0.82212E+00_JPRB, 0.82622E+00_JPRB, 0.82280E+00_JPRB ,&
&  0.80719E+00_JPRB, 0.80465E+00_JPRB, 0.80640E+00_JPRB, 0.80347E+00_JPRB, 0.75083E+00_JPRB, 0.69165E+00_JPRB, 0.64783E+00_JPRB ,&
&  0.61625E+00_JPRB, 0.59806E+00_JPRB, 0.58520E+00_JPRB, 0.59085E+00_JPRB, 0.60572E+00_JPRB, 0.62435E+00_JPRB, 0.62635E+00_JPRB ,&
&  0.63097E+00_JPRB, 0.63429E+00_JPRB, 0.64759E+00_JPRB, 0.67643E+00_JPRB, 0.70580E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.77171E+00_JPRB, 0.79091E+00_JPRB, 0.80594E+00_JPRB, 0.79924E+00_JPRB, 0.78363E+00_JPRB, 0.77220E+00_JPRB, 0.75866E+00_JPRB ,&
&  0.75095E+00_JPRB, 0.67319E+00_JPRB, 0.63862E+00_JPRB, 0.62067E+00_JPRB, 0.60813E+00_JPRB, 0.58487E+00_JPRB, 0.56664E+00_JPRB ,&
&  0.56462E+00_JPRB, 0.57488E+00_JPRB, 0.58999E+00_JPRB, 0.60049E+00_JPRB, 0.61001E+00_JPRB, 0.61682E+00_JPRB, 0.62016E+00_JPRB ,&
&  0.62402E+00_JPRB, 0.62798E+00_JPRB, 0.62498E+00_JPRB, 0.62054E+00_JPRB, 0.61814E+00_JPRB, 0.61948E+00_JPRB, 0.62152E+00_JPRB ,&
&  0.62035E+00_JPRB, 0.61578E+00_JPRB, 0.60563E+00_JPRB, 0.60157E+00_JPRB, 0.59870E+00_JPRB, 0.59967E+00_JPRB, 0.59229E+00_JPRB ,&
&  0.52506E+00_JPRB, 0.52937E+00_JPRB, 0.57584E+00_JPRB, 0.58402E+00_JPRB, 0.59812E+00_JPRB, 0.60471E+00_JPRB, 0.60951E+00_JPRB ,&
&  0.61890E+00_JPRB, 0.63799E+00_JPRB, 0.66605E+00_JPRB, 0.69839E+00_JPRB, 0.72382E+00_JPRB, 0.74232E+00_JPRB, 0.76047E+00_JPRB ,&
&  0.80916E+00_JPRB, 0.75755E+00_JPRB, 0.74313E+00_JPRB, 0.80643E+00_JPRB, 0.94898E+00_JPRB, 0.10333E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.11636E+01_JPRB, 0.11669E+01_JPRB, 0.11905E+01_JPRB, 0.11974E+01_JPRB, 0.11939E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.12183E+01_JPRB, 0.12367E+01_JPRB, 0.12648E+01_JPRB, 0.12878E+01_JPRB, 0.12423E+01_JPRB, 0.12064E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.10505E+01_JPRB, 0.98039E+00_JPRB, 0.92477E+00_JPRB, 0.88646E+00_JPRB, 0.88450E+00_JPRB, 0.87960E+00_JPRB ,&
&  0.86255E+00_JPRB, 0.85261E+00_JPRB, 0.83836E+00_JPRB, 0.82438E+00_JPRB, 0.78808E+00_JPRB, 0.75333E+00_JPRB, 0.73519E+00_JPRB ,&
&  0.68209E+00_JPRB, 0.64573E+00_JPRB, 0.60385E+00_JPRB, 0.60390E+00_JPRB, 0.63409E+00_JPRB, 0.65454E+00_JPRB, 0.66377E+00_JPRB ,&
&  0.68768E+00_JPRB, 0.70321E+00_JPRB, 0.71097E+00_JPRB, 0.72752E+00_JPRB, 0.74963E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.79606E+00_JPRB, 0.80371E+00_JPRB, 0.80586E+00_JPRB, 0.80068E+00_JPRB, 0.80300E+00_JPRB, 0.79896E+00_JPRB, 0.80056E+00_JPRB ,&
&  0.82429E+00_JPRB, 0.77552E+00_JPRB, 0.81073E+00_JPRB, 0.69547E+00_JPRB, 0.63589E+00_JPRB, 0.60332E+00_JPRB, 0.58056E+00_JPRB ,&
&  0.57689E+00_JPRB, 0.58582E+00_JPRB, 0.59485E+00_JPRB, 0.60587E+00_JPRB, 0.62352E+00_JPRB, 0.62664E+00_JPRB, 0.62810E+00_JPRB ,&
&  0.63412E+00_JPRB, 0.64063E+00_JPRB, 0.64921E+00_JPRB, 0.64047E+00_JPRB, 0.63837E+00_JPRB, 0.63999E+00_JPRB, 0.65153E+00_JPRB ,&
&  0.63144E+00_JPRB, 0.62689E+00_JPRB, 0.63664E+00_JPRB, 0.63388E+00_JPRB, 0.63223E+00_JPRB, 0.62876E+00_JPRB, 0.62848E+00_JPRB ,&
&  0.63166E+00_JPRB, 0.60017E+00_JPRB, 0.62440E+00_JPRB, 0.63246E+00_JPRB, 0.63780E+00_JPRB, 0.64347E+00_JPRB, 0.64615E+00_JPRB ,&
&  0.65445E+00_JPRB, 0.67149E+00_JPRB, 0.69766E+00_JPRB, 0.72558E+00_JPRB, 0.75877E+00_JPRB, 0.78875E+00_JPRB, 0.81335E+00_JPRB ,&
&  0.87302E+00_JPRB, 0.79049E+00_JPRB, 0.90471E+00_JPRB, 0.96695E+00_JPRB, 0.10909E+01_JPRB, 0.12501E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.15082E+01_JPRB, 0.14725E+01_JPRB, 0.14642E+01_JPRB, 0.14419E+01_JPRB, 0.14076E+01_JPRB, 0.13923E+01_JPRB, 0.13957E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.14665E+01_JPRB, 0.15692E+01_JPRB, 0.15304E+01_JPRB, 0.14218E+01_JPRB, 0.13522E+01_JPRB, 0.12781E+01_JPRB ,&
&  0.12168E+01_JPRB, 0.11773E+01_JPRB, 0.11363E+01_JPRB, 0.10362E+01_JPRB, 0.10118E+01_JPRB, 0.99679E+00_JPRB, 0.93849E+00_JPRB ,&
&  0.91051E+00_JPRB, 0.86626E+00_JPRB, 0.81437E+00_JPRB, 0.82136E+00_JPRB, 0.78293E+00_JPRB, 0.75409E+00_JPRB, 0.74870E+00_JPRB ,&
&  0.72352E+00_JPRB, 0.67441E+00_JPRB, 0.67563E+00_JPRB, 0.69546E+00_JPRB, 0.69341E+00_JPRB, 0.69083E+00_JPRB, 0.71792E+00_JPRB ,&
&  0.73170E+00_JPRB, 0.74651E+00_JPRB, 0.78143E+00_JPRB, 0.78719E+00_JPRB, 0.79615E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.81076E+00_JPRB, 0.79925E+00_JPRB, 0.79363E+00_JPRB, 0.79726E+00_JPRB, 0.81634E+00_JPRB, 0.84751E+00_JPRB, 0.83739E+00_JPRB ,&
&  0.85672E+00_JPRB, 0.86259E+00_JPRB, 0.84987E+00_JPRB, 0.82692E+00_JPRB, 0.79085E+00_JPRB, 0.70184E+00_JPRB, 0.60307E+00_JPRB ,&
&  0.57509E+00_JPRB, 0.58563E+00_JPRB, 0.60291E+00_JPRB, 0.61141E+00_JPRB, 0.62471E+00_JPRB, 0.62700E+00_JPRB, 0.63061E+00_JPRB ,&
&  0.63883E+00_JPRB, 0.64784E+00_JPRB, 0.65622E+00_JPRB, 0.66048E+00_JPRB, 0.65456E+00_JPRB, 0.66595E+00_JPRB, 0.68807E+00_JPRB ,&
&  0.65461E+00_JPRB, 0.63585E+00_JPRB, 0.63060E+00_JPRB, 0.62079E+00_JPRB, 0.64475E+00_JPRB, 0.65523E+00_JPRB, 0.66273E+00_JPRB ,&
&  0.66993E+00_JPRB, 0.67499E+00_JPRB, 0.67283E+00_JPRB, 0.67506E+00_JPRB, 0.68037E+00_JPRB, 0.69341E+00_JPRB, 0.70839E+00_JPRB ,&
&  0.72910E+00_JPRB, 0.75091E+00_JPRB, 0.77815E+00_JPRB, 0.81528E+00_JPRB, 0.86152E+00_JPRB, 0.87041E+00_JPRB, 0.92978E+00_JPRB ,&
&  0.98782E+00_JPRB, 0.10991E+01_JPRB, 0.12295E+01_JPRB, 0.12070E+01_JPRB, 0.13445E+01_JPRB, 0.14741E+01_JPRB, 0.15026E+01_JPRB ,&
&  0.16023E+01_JPRB, 0.15738E+01_JPRB, 0.16428E+01_JPRB, 0.17065E+01_JPRB, 0.17562E+01_JPRB, 0.17519E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.16812E+01_JPRB, 0.18472E+01_JPRB, 0.18733E+01_JPRB, 0.17840E+01_JPRB, 0.16274E+01_JPRB, 0.15263E+01_JPRB, 0.14123E+01_JPRB ,&
&  0.13741E+01_JPRB, 0.13805E+01_JPRB, 0.13367E+01_JPRB, 0.12882E+01_JPRB, 0.12008E+01_JPRB, 0.11064E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.95093E+00_JPRB, 0.84208E+00_JPRB, 0.78976E+00_JPRB, 0.77848E+00_JPRB, 0.77260E+00_JPRB, 0.73716E+00_JPRB, 0.73502E+00_JPRB ,&
&  0.73927E+00_JPRB, 0.73237E+00_JPRB, 0.76507E+00_JPRB, 0.78081E+00_JPRB, 0.77259E+00_JPRB, 0.75678E+00_JPRB, 0.73189E+00_JPRB ,&
&  0.71813E+00_JPRB, 0.75825E+00_JPRB, 0.79643E+00_JPRB, 0.81524E+00_JPRB, 0.81783E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.82381E+00_JPRB, 0.80811E+00_JPRB, 0.78645E+00_JPRB, 0.79136E+00_JPRB, 0.82787E+00_JPRB, 0.86304E+00_JPRB, 0.88305E+00_JPRB ,&
&  0.87744E+00_JPRB, 0.87632E+00_JPRB, 0.86390E+00_JPRB, 0.84798E+00_JPRB, 0.82137E+00_JPRB, 0.78959E+00_JPRB, 0.71454E+00_JPRB ,&
&  0.62549E+00_JPRB, 0.62261E+00_JPRB, 0.60149E+00_JPRB, 0.60797E+00_JPRB, 0.63363E+00_JPRB, 0.63515E+00_JPRB, 0.63895E+00_JPRB ,&
&  0.64942E+00_JPRB, 0.65802E+00_JPRB, 0.66678E+00_JPRB, 0.66733E+00_JPRB, 0.66791E+00_JPRB, 0.70863E+00_JPRB, 0.70291E+00_JPRB ,&
&  0.65631E+00_JPRB, 0.62938E+00_JPRB, 0.62672E+00_JPRB, 0.63543E+00_JPRB, 0.65510E+00_JPRB, 0.67726E+00_JPRB, 0.68585E+00_JPRB ,&
&  0.69102E+00_JPRB, 0.69435E+00_JPRB, 0.69141E+00_JPRB, 0.69905E+00_JPRB, 0.71515E+00_JPRB, 0.73950E+00_JPRB, 0.76269E+00_JPRB ,&
&  0.78187E+00_JPRB, 0.80787E+00_JPRB, 0.84540E+00_JPRB, 0.89837E+00_JPRB, 0.94074E+00_JPRB, 0.10063E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.11705E+01_JPRB, 0.12721E+01_JPRB, 0.13559E+01_JPRB, 0.14775E+01_JPRB, 0.16518E+01_JPRB, 0.17137E+01_JPRB, 0.17398E+01_JPRB ,&
&  0.17949E+01_JPRB, 0.17649E+01_JPRB, 0.20381E+01_JPRB, 0.20959E+01_JPRB, 0.20748E+01_JPRB, 0.20908E+01_JPRB, 0.19801E+01_JPRB ,&
&  0.18968E+01_JPRB, 0.18527E+01_JPRB, 0.18248E+01_JPRB, 0.17746E+01_JPRB, 0.17129E+01_JPRB, 0.17157E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.16147E+01_JPRB, 0.15814E+01_JPRB, 0.13641E+01_JPRB, 0.11994E+01_JPRB, 0.10452E+01_JPRB, 0.10590E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.94745E+00_JPRB, 0.82047E+00_JPRB, 0.81181E+00_JPRB, 0.79748E+00_JPRB, 0.80440E+00_JPRB, 0.82490E+00_JPRB, 0.81815E+00_JPRB ,&
&  0.82986E+00_JPRB, 0.86198E+00_JPRB, 0.87260E+00_JPRB, 0.91276E+00_JPRB, 0.89389E+00_JPRB, 0.85216E+00_JPRB, 0.75635E+00_JPRB ,&
&  0.72997E+00_JPRB, 0.76114E+00_JPRB, 0.79188E+00_JPRB, 0.81672E+00_JPRB, 0.82779E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.85381E+00_JPRB, 0.84145E+00_JPRB, 0.82112E+00_JPRB, 0.82258E+00_JPRB, 0.85517E+00_JPRB, 0.89040E+00_JPRB, 0.90950E+00_JPRB ,&
&  0.90264E+00_JPRB, 0.90196E+00_JPRB, 0.88234E+00_JPRB, 0.86517E+00_JPRB, 0.84585E+00_JPRB, 0.81432E+00_JPRB, 0.80360E+00_JPRB ,&
&  0.67783E+00_JPRB, 0.65092E+00_JPRB, 0.59385E+00_JPRB, 0.59094E+00_JPRB, 0.65022E+00_JPRB, 0.65202E+00_JPRB, 0.66116E+00_JPRB ,&
&  0.66301E+00_JPRB, 0.67732E+00_JPRB, 0.67775E+00_JPRB, 0.67717E+00_JPRB, 0.70213E+00_JPRB, 0.73858E+00_JPRB, 0.71714E+00_JPRB ,&
&  0.68162E+00_JPRB, 0.65776E+00_JPRB, 0.65860E+00_JPRB, 0.66226E+00_JPRB, 0.67244E+00_JPRB, 0.69120E+00_JPRB, 0.70266E+00_JPRB ,&
&  0.70810E+00_JPRB, 0.72706E+00_JPRB, 0.74148E+00_JPRB, 0.76479E+00_JPRB, 0.79413E+00_JPRB, 0.83627E+00_JPRB, 0.87084E+00_JPRB ,&
&  0.90910E+00_JPRB, 0.95617E+00_JPRB, 0.10060E+01_JPRB, 0.10694E+01_JPRB, 0.11050E+01_JPRB, 0.12028E+01_JPRB, 0.13312E+01_JPRB ,&
&  0.15324E+01_JPRB, 0.16024E+01_JPRB, 0.16660E+01_JPRB, 0.20035E+01_JPRB, 0.21511E+01_JPRB, 0.21447E+01_JPRB, 0.20272E+01_JPRB ,&
&  0.19918E+01_JPRB, 0.19601E+01_JPRB, 0.20161E+01_JPRB, 0.21068E+01_JPRB, 0.21147E+01_JPRB, 0.21879E+01_JPRB, 0.20989E+01_JPRB ,&
&  0.19002E+01_JPRB, 0.18118E+01_JPRB, 0.17692E+01_JPRB, 0.17758E+01_JPRB, 0.17060E+01_JPRB, 0.17328E+01_JPRB, 0.16723E+01_JPRB ,&
&  0.16905E+01_JPRB, 0.14886E+01_JPRB, 0.11445E+01_JPRB, 0.10552E+01_JPRB, 0.87958E+00_JPRB, 0.90029E+00_JPRB, 0.10830E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.95489E+00_JPRB, 0.88703E+00_JPRB, 0.85334E+00_JPRB, 0.86228E+00_JPRB, 0.10275E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.97571E+00_JPRB, 0.10157E+01_JPRB, 0.10754E+01_JPRB, 0.11185E+01_JPRB, 0.10900E+01_JPRB, 0.10360E+01_JPRB, 0.93279E+00_JPRB ,&
&  0.89923E+00_JPRB, 0.88140E+00_JPRB, 0.87893E+00_JPRB, 0.87421E+00_JPRB, 0.86491E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.93015E+00_JPRB, 0.91051E+00_JPRB, 0.88910E+00_JPRB, 0.88825E+00_JPRB, 0.90981E+00_JPRB, 0.93990E+00_JPRB, 0.95684E+00_JPRB ,&
&  0.94530E+00_JPRB, 0.93721E+00_JPRB, 0.91249E+00_JPRB, 0.89061E+00_JPRB, 0.86936E+00_JPRB, 0.84122E+00_JPRB, 0.82264E+00_JPRB ,&
&  0.81693E+00_JPRB, 0.72967E+00_JPRB, 0.63890E+00_JPRB, 0.59014E+00_JPRB, 0.62553E+00_JPRB, 0.64185E+00_JPRB, 0.66758E+00_JPRB ,&
&  0.66852E+00_JPRB, 0.69965E+00_JPRB, 0.70150E+00_JPRB, 0.70728E+00_JPRB, 0.76047E+00_JPRB, 0.77133E+00_JPRB, 0.75645E+00_JPRB ,&
&  0.75017E+00_JPRB, 0.72554E+00_JPRB, 0.71801E+00_JPRB, 0.72772E+00_JPRB, 0.74347E+00_JPRB, 0.72034E+00_JPRB, 0.74382E+00_JPRB ,&
&  0.77061E+00_JPRB, 0.80009E+00_JPRB, 0.82433E+00_JPRB, 0.86605E+00_JPRB, 0.92233E+00_JPRB, 0.97290E+00_JPRB, 0.10294E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.11232E+01_JPRB, 0.11407E+01_JPRB, 0.11889E+01_JPRB, 0.12703E+01_JPRB, 0.14180E+01_JPRB, 0.14599E+01_JPRB ,&
&  0.15715E+01_JPRB, 0.16584E+01_JPRB, 0.16564E+01_JPRB, 0.20533E+01_JPRB, 0.23206E+01_JPRB, 0.23262E+01_JPRB, 0.22665E+01_JPRB ,&
&  0.22396E+01_JPRB, 0.22038E+01_JPRB, 0.22992E+01_JPRB, 0.23334E+01_JPRB, 0.23526E+01_JPRB, 0.23297E+01_JPRB, 0.21705E+01_JPRB ,&
&  0.19672E+01_JPRB, 0.18303E+01_JPRB, 0.17620E+01_JPRB, 0.16900E+01_JPRB, 0.16258E+01_JPRB, 0.15673E+01_JPRB, 0.16211E+01_JPRB ,&
&  0.17433E+01_JPRB, 0.10670E+01_JPRB, 0.88430E+00_JPRB, 0.99733E+00_JPRB, 0.87131E+00_JPRB, 0.89632E+00_JPRB, 0.99245E+00_JPRB ,&
&  0.97687E+00_JPRB, 0.96325E+00_JPRB, 0.99157E+00_JPRB, 0.99375E+00_JPRB, 0.10645E+01_JPRB, 0.13399E+01_JPRB, 0.14021E+01_JPRB ,&
&  0.13255E+01_JPRB, 0.12459E+01_JPRB, 0.13213E+01_JPRB, 0.13220E+01_JPRB, 0.13022E+01_JPRB, 0.12662E+01_JPRB, 0.11892E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.10757E+01_JPRB, 0.10278E+01_JPRB, 0.98403E+00_JPRB, 0.95262E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.10602E+01_JPRB, 0.10364E+01_JPRB, 0.10174E+01_JPRB, 0.10049E+01_JPRB, 0.10090E+01_JPRB, 0.10302E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.99773E+00_JPRB, 0.96184E+00_JPRB, 0.93062E+00_JPRB, 0.90329E+00_JPRB, 0.88100E+00_JPRB, 0.86097E+00_JPRB ,&
&  0.83776E+00_JPRB, 0.79442E+00_JPRB, 0.67266E+00_JPRB, 0.61829E+00_JPRB, 0.58859E+00_JPRB, 0.61935E+00_JPRB, 0.65217E+00_JPRB ,&
&  0.67441E+00_JPRB, 0.70077E+00_JPRB, 0.72055E+00_JPRB, 0.78917E+00_JPRB, 0.87217E+00_JPRB, 0.84785E+00_JPRB, 0.87344E+00_JPRB ,&
&  0.87538E+00_JPRB, 0.84941E+00_JPRB, 0.79790E+00_JPRB, 0.81523E+00_JPRB, 0.79410E+00_JPRB, 0.77685E+00_JPRB, 0.85215E+00_JPRB ,&
&  0.89534E+00_JPRB, 0.93077E+00_JPRB, 0.96666E+00_JPRB, 0.10058E+01_JPRB, 0.10519E+01_JPRB, 0.10746E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.11734E+01_JPRB, 0.12151E+01_JPRB, 0.12573E+01_JPRB, 0.12819E+01_JPRB, 0.13450E+01_JPRB, 0.14369E+01_JPRB, 0.14490E+01_JPRB ,&
&  0.15266E+01_JPRB, 0.14197E+01_JPRB, 0.11789E+01_JPRB, 0.15708E+01_JPRB, 0.22952E+01_JPRB, 0.24278E+01_JPRB, 0.22884E+01_JPRB ,&
&  0.24455E+01_JPRB, 0.24860E+01_JPRB, 0.26406E+01_JPRB, 0.27765E+01_JPRB, 0.27163E+01_JPRB, 0.25762E+01_JPRB, 0.23861E+01_JPRB ,&
&  0.21461E+01_JPRB, 0.19798E+01_JPRB, 0.18978E+01_JPRB, 0.19838E+01_JPRB, 0.19184E+01_JPRB, 0.19220E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.17070E+01_JPRB, 0.14582E+01_JPRB, 0.10767E+01_JPRB, 0.11051E+01_JPRB, 0.90965E+00_JPRB, 0.92626E+00_JPRB, 0.10247E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.10706E+01_JPRB, 0.11360E+01_JPRB, 0.11682E+01_JPRB, 0.14306E+01_JPRB, 0.16834E+01_JPRB, 0.19186E+01_JPRB ,&
&  0.18850E+01_JPRB, 0.16132E+01_JPRB, 0.16764E+01_JPRB, 0.16276E+01_JPRB, 0.16231E+01_JPRB, 0.15805E+01_JPRB, 0.15098E+01_JPRB ,&
&  0.14175E+01_JPRB, 0.13127E+01_JPRB, 0.12203E+01_JPRB, 0.11441E+01_JPRB, 0.10932E+01_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.11641E+01_JPRB, 0.11256E+01_JPRB, 0.11007E+01_JPRB, 0.10887E+01_JPRB, 0.10929E+01_JPRB, 0.11107E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10461E+01_JPRB, 0.10027E+01_JPRB, 0.96411E+00_JPRB, 0.93360E+00_JPRB, 0.91256E+00_JPRB, 0.89456E+00_JPRB ,&
&  0.86171E+00_JPRB, 0.77104E+00_JPRB, 0.64643E+00_JPRB, 0.58988E+00_JPRB, 0.56716E+00_JPRB, 0.58199E+00_JPRB, 0.63215E+00_JPRB ,&
&  0.68317E+00_JPRB, 0.72465E+00_JPRB, 0.78906E+00_JPRB, 0.89377E+00_JPRB, 0.97923E+00_JPRB, 0.99948E+00_JPRB, 0.10299E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.95773E+00_JPRB, 0.93533E+00_JPRB, 0.91181E+00_JPRB, 0.94927E+00_JPRB, 0.98417E+00_JPRB, 0.97354E+00_JPRB ,&
&  0.96065E+00_JPRB, 0.98525E+00_JPRB, 0.10132E+01_JPRB, 0.10596E+01_JPRB, 0.11075E+01_JPRB, 0.11763E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.13963E+01_JPRB, 0.14199E+01_JPRB, 0.13634E+01_JPRB, 0.13293E+01_JPRB, 0.14148E+01_JPRB, 0.14894E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.13625E+01_JPRB, 0.13072E+01_JPRB, 0.15178E+01_JPRB, 0.18550E+01_JPRB, 0.22529E+01_JPRB, 0.25530E+01_JPRB, 0.26252E+01_JPRB ,&
&  0.27296E+01_JPRB, 0.26993E+01_JPRB, 0.27698E+01_JPRB, 0.25432E+01_JPRB, 0.27725E+01_JPRB, 0.27123E+01_JPRB, 0.26020E+01_JPRB ,&
&  0.25280E+01_JPRB, 0.24928E+01_JPRB, 0.23945E+01_JPRB, 0.24459E+01_JPRB, 0.25939E+01_JPRB, 0.26724E+01_JPRB, 0.21420E+01_JPRB ,&
&  0.17206E+01_JPRB, 0.17699E+01_JPRB, 0.15206E+01_JPRB, 0.13315E+01_JPRB, 0.11853E+01_JPRB, 0.12037E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.12286E+01_JPRB, 0.12811E+01_JPRB, 0.12906E+01_JPRB, 0.14426E+01_JPRB, 0.17349E+01_JPRB, 0.20285E+01_JPRB, 0.22471E+01_JPRB ,&
&  0.20770E+01_JPRB, 0.21917E+01_JPRB, 0.21235E+01_JPRB, 0.20385E+01_JPRB, 0.20335E+01_JPRB, 0.19763E+01_JPRB, 0.18628E+01_JPRB ,&
&  0.17029E+01_JPRB, 0.15513E+01_JPRB, 0.14227E+01_JPRB, 0.13130E+01_JPRB, 0.12258E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.13813E+01_JPRB, 0.13229E+01_JPRB, 0.12811E+01_JPRB, 0.12572E+01_JPRB, 0.12552E+01_JPRB, 0.12624E+01_JPRB, 0.12333E+01_JPRB ,&
&  0.11705E+01_JPRB, 0.11142E+01_JPRB, 0.10645E+01_JPRB, 0.10208E+01_JPRB, 0.97863E+00_JPRB, 0.94754E+00_JPRB, 0.92545E+00_JPRB ,&
&  0.90416E+00_JPRB, 0.76278E+00_JPRB, 0.62606E+00_JPRB, 0.60158E+00_JPRB, 0.59115E+00_JPRB, 0.57275E+00_JPRB, 0.61147E+00_JPRB ,&
&  0.68260E+00_JPRB, 0.77506E+00_JPRB, 0.87053E+00_JPRB, 0.10360E+01_JPRB, 0.11280E+01_JPRB, 0.11912E+01_JPRB, 0.12238E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.11383E+01_JPRB, 0.11298E+01_JPRB, 0.10925E+01_JPRB, 0.10471E+01_JPRB, 0.10439E+01_JPRB, 0.10253E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.10399E+01_JPRB, 0.10881E+01_JPRB, 0.11691E+01_JPRB, 0.12642E+01_JPRB, 0.13842E+01_JPRB, 0.15291E+01_JPRB ,&
&  0.15453E+01_JPRB, 0.14854E+01_JPRB, 0.13540E+01_JPRB, 0.13545E+01_JPRB, 0.12982E+01_JPRB, 0.13134E+01_JPRB, 0.12956E+01_JPRB ,&
&  0.15753E+01_JPRB, 0.16674E+01_JPRB, 0.16412E+01_JPRB, 0.19916E+01_JPRB, 0.19757E+01_JPRB, 0.21979E+01_JPRB, 0.27572E+01_JPRB ,&
&  0.28788E+01_JPRB, 0.28802E+01_JPRB, 0.29556E+01_JPRB, 0.23756E+01_JPRB, 0.21044E+01_JPRB, 0.29306E+01_JPRB, 0.30994E+01_JPRB ,&
&  0.32060E+01_JPRB, 0.33656E+01_JPRB, 0.33627E+01_JPRB, 0.35333E+01_JPRB, 0.29505E+01_JPRB, 0.24414E+01_JPRB, 0.18384E+01_JPRB ,&
&  0.17388E+01_JPRB, 0.16330E+01_JPRB, 0.17731E+01_JPRB, 0.16064E+01_JPRB, 0.16100E+01_JPRB, 0.14833E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.15084E+01_JPRB, 0.16170E+01_JPRB, 0.18645E+01_JPRB, 0.22138E+01_JPRB, 0.22093E+01_JPRB, 0.25210E+01_JPRB ,&
&  0.26342E+01_JPRB, 0.25847E+01_JPRB, 0.22865E+01_JPRB, 0.23043E+01_JPRB, 0.22184E+01_JPRB, 0.21332E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.18869E+01_JPRB, 0.17542E+01_JPRB, 0.16439E+01_JPRB, 0.15428E+01_JPRB, 0.14560E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.15922E+01_JPRB, 0.15358E+01_JPRB, 0.14893E+01_JPRB, 0.14592E+01_JPRB, 0.14392E+01_JPRB, 0.14024E+01_JPRB, 0.13234E+01_JPRB ,&
&  0.12311E+01_JPRB, 0.11635E+01_JPRB, 0.11007E+01_JPRB, 0.10497E+01_JPRB, 0.99564E+00_JPRB, 0.96895E+00_JPRB, 0.96139E+00_JPRB ,&
&  0.94445E+00_JPRB, 0.85012E+00_JPRB, 0.67731E+00_JPRB, 0.62524E+00_JPRB, 0.62885E+00_JPRB, 0.60483E+00_JPRB, 0.61337E+00_JPRB ,&
&  0.72707E+00_JPRB, 0.87399E+00_JPRB, 0.10200E+01_JPRB, 0.12333E+01_JPRB, 0.13266E+01_JPRB, 0.14202E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.14571E+01_JPRB, 0.12922E+01_JPRB, 0.11767E+01_JPRB, 0.10896E+01_JPRB, 0.10315E+01_JPRB, 0.10044E+01_JPRB, 0.10185E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.11122E+01_JPRB, 0.12084E+01_JPRB, 0.12653E+01_JPRB, 0.13645E+01_JPRB, 0.14601E+01_JPRB, 0.15396E+01_JPRB ,&
&  0.15323E+01_JPRB, 0.14783E+01_JPRB, 0.14405E+01_JPRB, 0.15215E+01_JPRB, 0.15304E+01_JPRB, 0.14362E+01_JPRB, 0.14535E+01_JPRB ,&
&  0.16085E+01_JPRB, 0.16330E+01_JPRB, 0.17095E+01_JPRB, 0.19332E+01_JPRB, 0.22551E+01_JPRB, 0.23527E+01_JPRB, 0.27154E+01_JPRB ,&
&  0.22605E+01_JPRB, 0.20116E+01_JPRB, 0.17674E+01_JPRB, 0.15747E+01_JPRB, 0.20768E+01_JPRB, 0.34981E+01_JPRB, 0.37747E+01_JPRB ,&
&  0.38184E+01_JPRB, 0.40718E+01_JPRB, 0.41632E+01_JPRB, 0.36353E+01_JPRB, 0.29384E+01_JPRB, 0.23583E+01_JPRB, 0.28453E+01_JPRB ,&
&  0.28928E+01_JPRB, 0.32051E+01_JPRB, 0.28686E+01_JPRB, 0.22263E+01_JPRB, 0.18396E+01_JPRB, 0.20851E+01_JPRB, 0.20426E+01_JPRB ,&
&  0.20214E+01_JPRB, 0.20600E+01_JPRB, 0.26499E+01_JPRB, 0.27152E+01_JPRB, 0.26855E+01_JPRB, 0.28177E+01_JPRB, 0.30941E+01_JPRB ,&
&  0.28787E+01_JPRB, 0.27916E+01_JPRB, 0.26575E+01_JPRB, 0.24843E+01_JPRB, 0.23489E+01_JPRB, 0.22196E+01_JPRB, 0.20950E+01_JPRB ,&
&  0.19922E+01_JPRB, 0.18958E+01_JPRB, 0.17941E+01_JPRB, 0.17185E+01_JPRB, 0.16506E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.17365E+01_JPRB, 0.16824E+01_JPRB, 0.16389E+01_JPRB, 0.15909E+01_JPRB, 0.15346E+01_JPRB, 0.14563E+01_JPRB, 0.13594E+01_JPRB ,&
&  0.12695E+01_JPRB, 0.11946E+01_JPRB, 0.11329E+01_JPRB, 0.10825E+01_JPRB, 0.10284E+01_JPRB, 0.99570E+00_JPRB, 0.98238E+00_JPRB ,&
&  0.97204E+00_JPRB, 0.98275E+00_JPRB, 0.78021E+00_JPRB, 0.68601E+00_JPRB, 0.67899E+00_JPRB, 0.68349E+00_JPRB, 0.66849E+00_JPRB ,&
&  0.83000E+00_JPRB, 0.10204E+01_JPRB, 0.11489E+01_JPRB, 0.13844E+01_JPRB, 0.16218E+01_JPRB, 0.15257E+01_JPRB, 0.14658E+01_JPRB ,&
&  0.14489E+01_JPRB, 0.13154E+01_JPRB, 0.11994E+01_JPRB, 0.11360E+01_JPRB, 0.10979E+01_JPRB, 0.11003E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.12003E+01_JPRB, 0.12922E+01_JPRB, 0.13195E+01_JPRB, 0.13668E+01_JPRB, 0.14335E+01_JPRB, 0.15431E+01_JPRB, 0.16169E+01_JPRB ,&
&  0.15705E+01_JPRB, 0.15207E+01_JPRB, 0.15528E+01_JPRB, 0.16988E+01_JPRB, 0.17296E+01_JPRB, 0.16417E+01_JPRB, 0.18013E+01_JPRB ,&
&  0.17910E+01_JPRB, 0.19406E+01_JPRB, 0.19100E+01_JPRB, 0.20931E+01_JPRB, 0.22534E+01_JPRB, 0.22695E+01_JPRB, 0.24447E+01_JPRB ,&
&  0.19387E+01_JPRB, 0.17516E+01_JPRB, 0.18516E+01_JPRB, 0.20919E+01_JPRB, 0.23853E+01_JPRB, 0.30595E+01_JPRB, 0.46950E+01_JPRB ,&
&  0.39306E+01_JPRB, 0.42043E+01_JPRB, 0.42138E+01_JPRB, 0.40377E+01_JPRB, 0.22387E+01_JPRB, 0.18670E+01_JPRB, 0.31410E+01_JPRB ,&
&  0.35512E+01_JPRB, 0.22557E+01_JPRB, 0.15665E+01_JPRB, 0.18145E+01_JPRB, 0.14052E+01_JPRB, 0.17444E+01_JPRB, 0.24274E+01_JPRB ,&
&  0.26765E+01_JPRB, 0.28466E+01_JPRB, 0.37033E+01_JPRB, 0.33495E+01_JPRB, 0.31894E+01_JPRB, 0.41162E+01_JPRB, 0.35285E+01_JPRB ,&
&  0.32472E+01_JPRB, 0.27869E+01_JPRB, 0.27522E+01_JPRB, 0.24721E+01_JPRB, 0.22980E+01_JPRB, 0.21904E+01_JPRB, 0.21258E+01_JPRB ,&
&  0.20843E+01_JPRB, 0.20332E+01_JPRB, 0.19651E+01_JPRB, 0.18844E+01_JPRB, 0.17997E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.17915E+01_JPRB, 0.17498E+01_JPRB, 0.17143E+01_JPRB, 0.16599E+01_JPRB, 0.15844E+01_JPRB, 0.14897E+01_JPRB, 0.13914E+01_JPRB ,&
&  0.13190E+01_JPRB, 0.12404E+01_JPRB, 0.11755E+01_JPRB, 0.11225E+01_JPRB, 0.10689E+01_JPRB, 0.10272E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.97798E+00_JPRB, 0.96705E+00_JPRB, 0.91528E+00_JPRB, 0.83610E+00_JPRB, 0.82563E+00_JPRB, 0.87099E+00_JPRB, 0.89411E+00_JPRB ,&
&  0.10730E+01_JPRB, 0.12518E+01_JPRB, 0.13938E+01_JPRB, 0.15598E+01_JPRB, 0.16281E+01_JPRB, 0.15225E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.14813E+01_JPRB, 0.13746E+01_JPRB, 0.12853E+01_JPRB, 0.12556E+01_JPRB, 0.12549E+01_JPRB, 0.12762E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.13776E+01_JPRB, 0.13689E+01_JPRB, 0.13766E+01_JPRB, 0.14784E+01_JPRB, 0.16118E+01_JPRB, 0.16701E+01_JPRB ,&
&  0.16208E+01_JPRB, 0.16189E+01_JPRB, 0.16688E+01_JPRB, 0.18109E+01_JPRB, 0.20268E+01_JPRB, 0.20216E+01_JPRB, 0.17406E+01_JPRB ,&
&  0.15595E+01_JPRB, 0.16775E+01_JPRB, 0.20906E+01_JPRB, 0.20476E+01_JPRB, 0.20949E+01_JPRB, 0.21891E+01_JPRB, 0.22751E+01_JPRB ,&
&  0.22169E+01_JPRB, 0.22573E+01_JPRB, 0.23313E+01_JPRB, 0.27201E+01_JPRB, 0.31570E+01_JPRB, 0.32250E+01_JPRB, 0.45786E+01_JPRB ,&
&  0.46334E+01_JPRB, 0.42328E+01_JPRB, 0.32479E+01_JPRB, 0.24883E+01_JPRB, 0.25618E+01_JPRB, 0.21487E+01_JPRB, 0.14970E+01_JPRB ,&
&  0.14427E+01_JPRB, 0.13704E+01_JPRB, 0.12121E+01_JPRB, 0.11882E+01_JPRB, 0.11189E+01_JPRB, 0.15609E+01_JPRB, 0.24627E+01_JPRB ,&
&  0.35283E+01_JPRB, 0.41055E+01_JPRB, 0.43959E+01_JPRB, 0.40565E+01_JPRB, 0.36749E+01_JPRB, 0.45635E+01_JPRB, 0.39304E+01_JPRB ,&
&  0.29811E+01_JPRB, 0.26632E+01_JPRB, 0.25963E+01_JPRB, 0.24121E+01_JPRB, 0.23220E+01_JPRB, 0.22431E+01_JPRB, 0.21737E+01_JPRB ,&
&  0.21041E+01_JPRB, 0.20459E+01_JPRB, 0.19836E+01_JPRB, 0.19113E+01_JPRB, 0.18476E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.18129E+01_JPRB, 0.17797E+01_JPRB, 0.17364E+01_JPRB, 0.16738E+01_JPRB, 0.15906E+01_JPRB, 0.15054E+01_JPRB, 0.14315E+01_JPRB ,&
&  0.13889E+01_JPRB, 0.13230E+01_JPRB, 0.12504E+01_JPRB, 0.11847E+01_JPRB, 0.11208E+01_JPRB, 0.10666E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.99443E+00_JPRB, 0.10211E+01_JPRB, 0.10145E+01_JPRB, 0.11201E+01_JPRB, 0.12892E+01_JPRB, 0.14014E+01_JPRB ,&
&  0.15803E+01_JPRB, 0.16998E+01_JPRB, 0.18755E+01_JPRB, 0.19093E+01_JPRB, 0.19180E+01_JPRB, 0.18735E+01_JPRB, 0.17681E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.15956E+01_JPRB, 0.15459E+01_JPRB, 0.15094E+01_JPRB, 0.14685E+01_JPRB, 0.14307E+01_JPRB, 0.14091E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.14337E+01_JPRB, 0.13976E+01_JPRB, 0.14285E+01_JPRB, 0.15364E+01_JPRB, 0.16476E+01_JPRB, 0.17186E+01_JPRB ,&
&  0.17368E+01_JPRB, 0.17447E+01_JPRB, 0.18127E+01_JPRB, 0.19335E+01_JPRB, 0.18253E+01_JPRB, 0.15546E+01_JPRB, 0.16878E+01_JPRB ,&
&  0.18009E+01_JPRB, 0.20393E+01_JPRB, 0.20006E+01_JPRB, 0.22822E+01_JPRB, 0.22250E+01_JPRB, 0.21371E+01_JPRB, 0.23383E+01_JPRB ,&
&  0.25488E+01_JPRB, 0.31969E+01_JPRB, 0.28003E+01_JPRB, 0.28499E+01_JPRB, 0.41279E+01_JPRB, 0.39538E+01_JPRB, 0.42829E+01_JPRB ,&
&  0.46542E+01_JPRB, 0.38875E+01_JPRB, 0.32371E+01_JPRB, 0.29406E+01_JPRB, 0.42301E+01_JPRB, 0.43800E+01_JPRB, 0.20625E+01_JPRB ,&
&  0.16337E+01_JPRB, 0.14286E+01_JPRB, 0.12956E+01_JPRB, 0.11689E+01_JPRB, 0.11197E+01_JPRB, 0.16672E+01_JPRB, 0.34524E+01_JPRB ,&
&  0.37999E+01_JPRB, 0.54312E+01_JPRB, 0.54929E+01_JPRB, 0.49690E+01_JPRB, 0.41943E+01_JPRB, 0.43145E+01_JPRB, 0.34027E+01_JPRB ,&
&  0.29591E+01_JPRB, 0.27619E+01_JPRB, 0.25618E+01_JPRB, 0.23906E+01_JPRB, 0.22130E+01_JPRB, 0.21085E+01_JPRB, 0.20617E+01_JPRB ,&
&  0.20190E+01_JPRB, 0.19759E+01_JPRB, 0.19368E+01_JPRB, 0.18918E+01_JPRB, 0.18520E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.17507E+01_JPRB, 0.17068E+01_JPRB, 0.16619E+01_JPRB, 0.16000E+01_JPRB, 0.15415E+01_JPRB, 0.14958E+01_JPRB, 0.14564E+01_JPRB ,&
&  0.14374E+01_JPRB, 0.13807E+01_JPRB, 0.13086E+01_JPRB, 0.12288E+01_JPRB, 0.11550E+01_JPRB, 0.10956E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.10386E+01_JPRB, 0.10633E+01_JPRB, 0.12149E+01_JPRB, 0.14527E+01_JPRB, 0.16282E+01_JPRB, 0.19334E+01_JPRB ,&
&  0.22799E+01_JPRB, 0.24473E+01_JPRB, 0.26632E+01_JPRB, 0.26345E+01_JPRB, 0.25103E+01_JPRB, 0.23522E+01_JPRB, 0.21657E+01_JPRB ,&
&  0.20275E+01_JPRB, 0.19511E+01_JPRB, 0.18454E+01_JPRB, 0.17334E+01_JPRB, 0.16270E+01_JPRB, 0.15486E+01_JPRB, 0.15005E+01_JPRB ,&
&  0.15207E+01_JPRB, 0.14797E+01_JPRB, 0.14396E+01_JPRB, 0.14724E+01_JPRB, 0.15677E+01_JPRB, 0.17006E+01_JPRB, 0.17935E+01_JPRB ,&
&  0.18192E+01_JPRB, 0.18107E+01_JPRB, 0.18597E+01_JPRB, 0.19431E+01_JPRB, 0.16205E+01_JPRB, 0.18114E+01_JPRB, 0.18189E+01_JPRB ,&
&  0.18376E+01_JPRB, 0.19109E+01_JPRB, 0.18771E+01_JPRB, 0.21310E+01_JPRB, 0.22341E+01_JPRB, 0.21482E+01_JPRB, 0.23153E+01_JPRB ,&
&  0.29583E+01_JPRB, 0.28564E+01_JPRB, 0.27611E+01_JPRB, 0.32662E+01_JPRB, 0.41313E+01_JPRB, 0.58692E+01_JPRB, 0.38852E+01_JPRB ,&
&  0.33216E+01_JPRB, 0.37823E+01_JPRB, 0.34795E+01_JPRB, 0.32906E+01_JPRB, 0.42716E+01_JPRB, 0.53331E+01_JPRB, 0.45595E+01_JPRB ,&
&  0.19261E+01_JPRB, 0.13934E+01_JPRB, 0.14673E+01_JPRB, 0.15302E+01_JPRB, 0.14724E+01_JPRB, 0.21052E+01_JPRB, 0.55585E+01_JPRB ,&
&  0.49298E+01_JPRB, 0.62736E+01_JPRB, 0.58170E+01_JPRB, 0.49988E+01_JPRB, 0.43222E+01_JPRB, 0.40466E+01_JPRB, 0.34912E+01_JPRB ,&
&  0.29304E+01_JPRB, 0.26105E+01_JPRB, 0.23516E+01_JPRB, 0.21707E+01_JPRB, 0.20658E+01_JPRB, 0.19851E+01_JPRB, 0.19335E+01_JPRB ,&
&  0.18892E+01_JPRB, 0.18547E+01_JPRB, 0.18322E+01_JPRB, 0.18149E+01_JPRB, 0.17900E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.16606E+01_JPRB, 0.16072E+01_JPRB, 0.15528E+01_JPRB, 0.15196E+01_JPRB, 0.14969E+01_JPRB, 0.14644E+01_JPRB, 0.14392E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.13618E+01_JPRB, 0.13008E+01_JPRB, 0.12274E+01_JPRB, 0.11578E+01_JPRB, 0.11080E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.10750E+01_JPRB, 0.10969E+01_JPRB, 0.12811E+01_JPRB, 0.16467E+01_JPRB, 0.17617E+01_JPRB, 0.21379E+01_JPRB ,&
&  0.33476E+01_JPRB, 0.37187E+01_JPRB, 0.34763E+01_JPRB, 0.33033E+01_JPRB, 0.31423E+01_JPRB, 0.30023E+01_JPRB, 0.27518E+01_JPRB ,&
&  0.24689E+01_JPRB, 0.22178E+01_JPRB, 0.19933E+01_JPRB, 0.18221E+01_JPRB, 0.16906E+01_JPRB, 0.16055E+01_JPRB, 0.15756E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.15018E+01_JPRB, 0.14624E+01_JPRB, 0.14850E+01_JPRB, 0.16031E+01_JPRB, 0.17316E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.18527E+01_JPRB, 0.18426E+01_JPRB, 0.18463E+01_JPRB, 0.17957E+01_JPRB, 0.17990E+01_JPRB, 0.18223E+01_JPRB, 0.18702E+01_JPRB ,&
&  0.18581E+01_JPRB, 0.19518E+01_JPRB, 0.20581E+01_JPRB, 0.20961E+01_JPRB, 0.21525E+01_JPRB, 0.22253E+01_JPRB, 0.24397E+01_JPRB ,&
&  0.28738E+01_JPRB, 0.29300E+01_JPRB, 0.30993E+01_JPRB, 0.39377E+01_JPRB, 0.44989E+01_JPRB, 0.51626E+01_JPRB, 0.48875E+01_JPRB ,&
&  0.40530E+01_JPRB, 0.31462E+01_JPRB, 0.30350E+01_JPRB, 0.35365E+01_JPRB, 0.40823E+01_JPRB, 0.46033E+01_JPRB, 0.69129E+01_JPRB ,&
&  0.68713E+01_JPRB, 0.45302E+01_JPRB, 0.37424E+01_JPRB, 0.48963E+01_JPRB, 0.41421E+01_JPRB, 0.37080E+01_JPRB, 0.46600E+01_JPRB ,&
&  0.55247E+01_JPRB, 0.65445E+01_JPRB, 0.58902E+01_JPRB, 0.47764E+01_JPRB, 0.43899E+01_JPRB, 0.39221E+01_JPRB, 0.32981E+01_JPRB ,&
&  0.27473E+01_JPRB, 0.24579E+01_JPRB, 0.22617E+01_JPRB, 0.20710E+01_JPRB, 0.19184E+01_JPRB, 0.18308E+01_JPRB, 0.17874E+01_JPRB ,&
&  0.17524E+01_JPRB, 0.17358E+01_JPRB, 0.17302E+01_JPRB, 0.17277E+01_JPRB, 0.17004E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.15353E+01_JPRB, 0.14959E+01_JPRB, 0.14756E+01_JPRB, 0.14370E+01_JPRB, 0.13955E+01_JPRB, 0.13622E+01_JPRB, 0.13519E+01_JPRB ,&
&  0.13394E+01_JPRB, 0.12953E+01_JPRB, 0.12445E+01_JPRB, 0.11879E+01_JPRB, 0.11281E+01_JPRB, 0.10941E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.11017E+01_JPRB, 0.11826E+01_JPRB, 0.14229E+01_JPRB, 0.18132E+01_JPRB, 0.24485E+01_JPRB, 0.22880E+01_JPRB ,&
&  0.34803E+01_JPRB, 0.46009E+01_JPRB, 0.43102E+01_JPRB, 0.40297E+01_JPRB, 0.37452E+01_JPRB, 0.35692E+01_JPRB, 0.30894E+01_JPRB ,&
&  0.26359E+01_JPRB, 0.23233E+01_JPRB, 0.20334E+01_JPRB, 0.18305E+01_JPRB, 0.17078E+01_JPRB, 0.16410E+01_JPRB, 0.15831E+01_JPRB ,&
&  0.15467E+01_JPRB, 0.14928E+01_JPRB, 0.14573E+01_JPRB, 0.14886E+01_JPRB, 0.15807E+01_JPRB, 0.17249E+01_JPRB, 0.18183E+01_JPRB ,&
&  0.18580E+01_JPRB, 0.18508E+01_JPRB, 0.18288E+01_JPRB, 0.17239E+01_JPRB, 0.17614E+01_JPRB, 0.18051E+01_JPRB, 0.18666E+01_JPRB ,&
&  0.18207E+01_JPRB, 0.17409E+01_JPRB, 0.19287E+01_JPRB, 0.21184E+01_JPRB, 0.22035E+01_JPRB, 0.22074E+01_JPRB, 0.23896E+01_JPRB ,&
&  0.29471E+01_JPRB, 0.34558E+01_JPRB, 0.43160E+01_JPRB, 0.46067E+01_JPRB, 0.48144E+01_JPRB, 0.49550E+01_JPRB, 0.49796E+01_JPRB ,&
&  0.49873E+01_JPRB, 0.41119E+01_JPRB, 0.37780E+01_JPRB, 0.38036E+01_JPRB, 0.38235E+01_JPRB, 0.42969E+01_JPRB, 0.60660E+01_JPRB ,&
&  0.77881E+01_JPRB, 0.88740E+01_JPRB, 0.91081E+01_JPRB, 0.76416E+01_JPRB, 0.64420E+01_JPRB, 0.54498E+01_JPRB, 0.50959E+01_JPRB ,&
&  0.61807E+01_JPRB, 0.63064E+01_JPRB, 0.55314E+01_JPRB, 0.56899E+01_JPRB, 0.42965E+01_JPRB, 0.38218E+01_JPRB, 0.31956E+01_JPRB ,&
&  0.27361E+01_JPRB, 0.23990E+01_JPRB, 0.21384E+01_JPRB, 0.19466E+01_JPRB, 0.18139E+01_JPRB, 0.17036E+01_JPRB, 0.16350E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.16095E+01_JPRB, 0.16224E+01_JPRB, 0.16183E+01_JPRB, 0.15797E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.14595E+01_JPRB, 0.14390E+01_JPRB, 0.14113E+01_JPRB, 0.13608E+01_JPRB, 0.13234E+01_JPRB, 0.13001E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.12257E+01_JPRB, 0.11828E+01_JPRB, 0.11338E+01_JPRB, 0.10945E+01_JPRB, 0.10696E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.12419E+01_JPRB, 0.14515E+01_JPRB, 0.17013E+01_JPRB, 0.21225E+01_JPRB, 0.27748E+01_JPRB, 0.32196E+01_JPRB ,&
&  0.36962E+01_JPRB, 0.56428E+01_JPRB, 0.46777E+01_JPRB, 0.42426E+01_JPRB, 0.39386E+01_JPRB, 0.36615E+01_JPRB, 0.30726E+01_JPRB ,&
&  0.24316E+01_JPRB, 0.21564E+01_JPRB, 0.19821E+01_JPRB, 0.18388E+01_JPRB, 0.17392E+01_JPRB, 0.16493E+01_JPRB, 0.16057E+01_JPRB ,&
&  0.15554E+01_JPRB, 0.15203E+01_JPRB, 0.14895E+01_JPRB, 0.15301E+01_JPRB, 0.16347E+01_JPRB, 0.17469E+01_JPRB, 0.18450E+01_JPRB ,&
&  0.18818E+01_JPRB, 0.18605E+01_JPRB, 0.17810E+01_JPRB, 0.17363E+01_JPRB, 0.17258E+01_JPRB, 0.17894E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.18856E+01_JPRB, 0.18210E+01_JPRB, 0.19302E+01_JPRB, 0.19942E+01_JPRB, 0.20540E+01_JPRB, 0.22809E+01_JPRB, 0.24532E+01_JPRB ,&
&  0.30499E+01_JPRB, 0.31812E+01_JPRB, 0.45816E+01_JPRB, 0.42961E+01_JPRB, 0.42991E+01_JPRB, 0.45254E+01_JPRB, 0.42880E+01_JPRB ,&
&  0.37295E+01_JPRB, 0.35999E+01_JPRB, 0.34173E+01_JPRB, 0.35419E+01_JPRB, 0.39641E+01_JPRB, 0.47734E+01_JPRB, 0.63022E+01_JPRB ,&
&  0.69026E+01_JPRB, 0.66993E+01_JPRB, 0.67367E+01_JPRB, 0.56080E+01_JPRB, 0.53597E+01_JPRB, 0.50094E+01_JPRB, 0.49285E+01_JPRB ,&
&  0.59821E+01_JPRB, 0.58997E+01_JPRB, 0.54824E+01_JPRB, 0.50070E+01_JPRB, 0.37791E+01_JPRB, 0.32479E+01_JPRB, 0.27109E+01_JPRB ,&
&  0.23638E+01_JPRB, 0.20793E+01_JPRB, 0.19218E+01_JPRB, 0.17931E+01_JPRB, 0.16943E+01_JPRB, 0.15914E+01_JPRB, 0.15438E+01_JPRB ,&
&  0.15240E+01_JPRB, 0.15245E+01_JPRB, 0.15298E+01_JPRB, 0.15134E+01_JPRB, 0.14842E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.14474E+01_JPRB, 0.14473E+01_JPRB, 0.14411E+01_JPRB, 0.14569E+01_JPRB, 0.14972E+01_JPRB, 0.15615E+01_JPRB, 0.15940E+01_JPRB ,&
&  0.14481E+01_JPRB, 0.11600E+01_JPRB, 0.11333E+01_JPRB, 0.11019E+01_JPRB, 0.10692E+01_JPRB, 0.10724E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.14286E+01_JPRB, 0.16192E+01_JPRB, 0.19475E+01_JPRB, 0.24422E+01_JPRB, 0.32709E+01_JPRB, 0.42004E+01_JPRB ,&
&  0.44856E+01_JPRB, 0.48613E+01_JPRB, 0.47137E+01_JPRB, 0.39956E+01_JPRB, 0.36067E+01_JPRB, 0.34071E+01_JPRB, 0.31532E+01_JPRB ,&
&  0.25361E+01_JPRB, 0.22030E+01_JPRB, 0.21278E+01_JPRB, 0.21139E+01_JPRB, 0.16970E+01_JPRB, 0.16415E+01_JPRB, 0.16158E+01_JPRB ,&
&  0.16027E+01_JPRB, 0.16086E+01_JPRB, 0.16179E+01_JPRB, 0.16740E+01_JPRB, 0.17637E+01_JPRB, 0.18580E+01_JPRB, 0.19298E+01_JPRB ,&
&  0.19556E+01_JPRB, 0.19357E+01_JPRB, 0.18792E+01_JPRB, 0.17604E+01_JPRB, 0.17499E+01_JPRB, 0.18063E+01_JPRB, 0.18821E+01_JPRB ,&
&  0.19285E+01_JPRB, 0.19885E+01_JPRB, 0.20329E+01_JPRB, 0.20720E+01_JPRB, 0.20741E+01_JPRB, 0.24339E+01_JPRB, 0.26779E+01_JPRB ,&
&  0.30698E+01_JPRB, 0.35502E+01_JPRB, 0.49328E+01_JPRB, 0.36657E+01_JPRB, 0.38657E+01_JPRB, 0.40127E+01_JPRB, 0.38150E+01_JPRB ,&
&  0.33653E+01_JPRB, 0.32547E+01_JPRB, 0.31022E+01_JPRB, 0.32601E+01_JPRB, 0.40558E+01_JPRB, 0.50700E+01_JPRB, 0.68949E+01_JPRB ,&
&  0.57943E+01_JPRB, 0.57946E+01_JPRB, 0.49360E+01_JPRB, 0.49636E+01_JPRB, 0.48655E+01_JPRB, 0.45952E+01_JPRB, 0.42827E+01_JPRB ,&
&  0.50748E+01_JPRB, 0.46532E+01_JPRB, 0.40552E+01_JPRB, 0.34732E+01_JPRB, 0.26246E+01_JPRB, 0.22314E+01_JPRB, 0.18868E+01_JPRB ,&
&  0.17368E+01_JPRB, 0.16608E+01_JPRB, 0.16484E+01_JPRB, 0.15905E+01_JPRB, 0.15507E+01_JPRB, 0.15131E+01_JPRB, 0.15049E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.14783E+01_JPRB, 0.14678E+01_JPRB, 0.14564E+01_JPRB, 0.14464E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.14774E+01_JPRB, 0.14586E+01_JPRB, 0.14389E+01_JPRB, 0.14259E+01_JPRB, 0.14152E+01_JPRB, 0.13831E+01_JPRB, 0.13148E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.11500E+01_JPRB, 0.11290E+01_JPRB, 0.11127E+01_JPRB, 0.11151E+01_JPRB, 0.11611E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.13553E+01_JPRB, 0.15408E+01_JPRB, 0.17967E+01_JPRB, 0.21519E+01_JPRB, 0.26299E+01_JPRB, 0.34009E+01_JPRB, 0.43269E+01_JPRB ,&
&  0.45448E+01_JPRB, 0.33941E+01_JPRB, 0.37952E+01_JPRB, 0.36435E+01_JPRB, 0.34197E+01_JPRB, 0.31674E+01_JPRB, 0.27070E+01_JPRB ,&
&  0.22323E+01_JPRB, 0.20942E+01_JPRB, 0.20443E+01_JPRB, 0.19220E+01_JPRB, 0.16729E+01_JPRB, 0.16955E+01_JPRB, 0.17349E+01_JPRB ,&
&  0.17785E+01_JPRB, 0.18136E+01_JPRB, 0.18192E+01_JPRB, 0.18784E+01_JPRB, 0.19361E+01_JPRB, 0.20213E+01_JPRB, 0.21141E+01_JPRB ,&
&  0.21497E+01_JPRB, 0.21540E+01_JPRB, 0.21292E+01_JPRB, 0.19915E+01_JPRB, 0.19520E+01_JPRB, 0.20261E+01_JPRB, 0.20964E+01_JPRB ,&
&  0.21388E+01_JPRB, 0.21185E+01_JPRB, 0.22169E+01_JPRB, 0.24671E+01_JPRB, 0.27768E+01_JPRB, 0.26645E+01_JPRB, 0.31050E+01_JPRB ,&
&  0.34210E+01_JPRB, 0.40132E+01_JPRB, 0.39309E+01_JPRB, 0.45065E+01_JPRB, 0.30244E+01_JPRB, 0.33027E+01_JPRB, 0.30105E+01_JPRB ,&
&  0.31562E+01_JPRB, 0.29633E+01_JPRB, 0.30651E+01_JPRB, 0.35910E+01_JPRB, 0.46441E+01_JPRB, 0.59569E+01_JPRB, 0.62367E+01_JPRB ,&
&  0.44075E+01_JPRB, 0.39833E+01_JPRB, 0.41954E+01_JPRB, 0.47439E+01_JPRB, 0.47622E+01_JPRB, 0.45490E+01_JPRB, 0.38008E+01_JPRB ,&
&  0.37665E+01_JPRB, 0.32523E+01_JPRB, 0.28738E+01_JPRB, 0.24235E+01_JPRB, 0.17257E+01_JPRB, 0.15097E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.14839E+01_JPRB, 0.15114E+01_JPRB, 0.15235E+01_JPRB, 0.15330E+01_JPRB, 0.15137E+01_JPRB, 0.14893E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.14644E+01_JPRB, 0.14657E+01_JPRB, 0.14672E+01_JPRB, 0.14648E+01_JPRB, 0.14716E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.14143E+01_JPRB, 0.13817E+01_JPRB, 0.13514E+01_JPRB, 0.13252E+01_JPRB, 0.13082E+01_JPRB, 0.12640E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.11498E+01_JPRB, 0.11487E+01_JPRB, 0.11613E+01_JPRB, 0.11988E+01_JPRB, 0.12419E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.14588E+01_JPRB, 0.16598E+01_JPRB, 0.19317E+01_JPRB, 0.22690E+01_JPRB, 0.27060E+01_JPRB, 0.32308E+01_JPRB, 0.36694E+01_JPRB ,&
&  0.41166E+01_JPRB, 0.42654E+01_JPRB, 0.35504E+01_JPRB, 0.25603E+01_JPRB, 0.24474E+01_JPRB, 0.24604E+01_JPRB, 0.21213E+01_JPRB ,&
&  0.20387E+01_JPRB, 0.19466E+01_JPRB, 0.18658E+01_JPRB, 0.18389E+01_JPRB, 0.18371E+01_JPRB, 0.18932E+01_JPRB, 0.19512E+01_JPRB ,&
&  0.19928E+01_JPRB, 0.20255E+01_JPRB, 0.20711E+01_JPRB, 0.21636E+01_JPRB, 0.22476E+01_JPRB, 0.23438E+01_JPRB, 0.24177E+01_JPRB ,&
&  0.24631E+01_JPRB, 0.25348E+01_JPRB, 0.25294E+01_JPRB, 0.24520E+01_JPRB, 0.23669E+01_JPRB, 0.24402E+01_JPRB, 0.25353E+01_JPRB ,&
&  0.26169E+01_JPRB, 0.26480E+01_JPRB, 0.27257E+01_JPRB, 0.30540E+01_JPRB, 0.33443E+01_JPRB, 0.31168E+01_JPRB, 0.32677E+01_JPRB ,&
&  0.38173E+01_JPRB, 0.42217E+01_JPRB, 0.32562E+01_JPRB, 0.38531E+01_JPRB, 0.23295E+01_JPRB, 0.29002E+01_JPRB, 0.31472E+01_JPRB ,&
&  0.30051E+01_JPRB, 0.31321E+01_JPRB, 0.33761E+01_JPRB, 0.40785E+01_JPRB, 0.50820E+01_JPRB, 0.57811E+01_JPRB, 0.50157E+01_JPRB ,&
&  0.35059E+01_JPRB, 0.33981E+01_JPRB, 0.36764E+01_JPRB, 0.39770E+01_JPRB, 0.40386E+01_JPRB, 0.37272E+01_JPRB, 0.27089E+01_JPRB ,&
&  0.20322E+01_JPRB, 0.23387E+01_JPRB, 0.23122E+01_JPRB, 0.21602E+01_JPRB, 0.14873E+01_JPRB, 0.12926E+01_JPRB, 0.13594E+01_JPRB ,&
&  0.14434E+01_JPRB, 0.15063E+01_JPRB, 0.15395E+01_JPRB, 0.15358E+01_JPRB, 0.15091E+01_JPRB, 0.14843E+01_JPRB, 0.14777E+01_JPRB ,&
&  0.14759E+01_JPRB, 0.14650E+01_JPRB, 0.14603E+01_JPRB, 0.14595E+01_JPRB, 0.14452E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.12908E+01_JPRB, 0.12482E+01_JPRB, 0.12229E+01_JPRB, 0.12057E+01_JPRB, 0.12023E+01_JPRB, 0.11754E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.10998E+01_JPRB, 0.11188E+01_JPRB, 0.11647E+01_JPRB, 0.12259E+01_JPRB, 0.12865E+01_JPRB, 0.14018E+01_JPRB ,&
&  0.15518E+01_JPRB, 0.17665E+01_JPRB, 0.20436E+01_JPRB, 0.24550E+01_JPRB, 0.28091E+01_JPRB, 0.31585E+01_JPRB, 0.32823E+01_JPRB ,&
&  0.34534E+01_JPRB, 0.35015E+01_JPRB, 0.32937E+01_JPRB, 0.30845E+01_JPRB, 0.24686E+01_JPRB, 0.24820E+01_JPRB, 0.26351E+01_JPRB ,&
&  0.27572E+01_JPRB, 0.23342E+01_JPRB, 0.20167E+01_JPRB, 0.20272E+01_JPRB, 0.20432E+01_JPRB, 0.21166E+01_JPRB, 0.21911E+01_JPRB ,&
&  0.22518E+01_JPRB, 0.23148E+01_JPRB, 0.23803E+01_JPRB, 0.24777E+01_JPRB, 0.25834E+01_JPRB, 0.26446E+01_JPRB, 0.27042E+01_JPRB ,&
&  0.27416E+01_JPRB, 0.27904E+01_JPRB, 0.28340E+01_JPRB, 0.28424E+01_JPRB, 0.29170E+01_JPRB, 0.30346E+01_JPRB, 0.32405E+01_JPRB ,&
&  0.34537E+01_JPRB, 0.34469E+01_JPRB, 0.36971E+01_JPRB, 0.41062E+01_JPRB, 0.41751E+01_JPRB, 0.40387E+01_JPRB, 0.40375E+01_JPRB ,&
&  0.44524E+01_JPRB, 0.48392E+01_JPRB, 0.27434E+01_JPRB, 0.30427E+01_JPRB, 0.35658E+01_JPRB, 0.31361E+01_JPRB, 0.29382E+01_JPRB ,&
&  0.30773E+01_JPRB, 0.33324E+01_JPRB, 0.37485E+01_JPRB, 0.42614E+01_JPRB, 0.50839E+01_JPRB, 0.65073E+01_JPRB, 0.44242E+01_JPRB ,&
&  0.28857E+01_JPRB, 0.27458E+01_JPRB, 0.29249E+01_JPRB, 0.30805E+01_JPRB, 0.30861E+01_JPRB, 0.26397E+01_JPRB, 0.17987E+01_JPRB ,&
&  0.15519E+01_JPRB, 0.20844E+01_JPRB, 0.21676E+01_JPRB, 0.21355E+01_JPRB, 0.17898E+01_JPRB, 0.12292E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.14340E+01_JPRB, 0.14824E+01_JPRB, 0.14745E+01_JPRB, 0.14733E+01_JPRB, 0.14709E+01_JPRB, 0.14609E+01_JPRB, 0.14582E+01_JPRB ,&
&  0.14553E+01_JPRB, 0.14503E+01_JPRB, 0.14460E+01_JPRB, 0.14078E+01_JPRB, 0.13417E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.11544E+01_JPRB, 0.11221E+01_JPRB, 0.11045E+01_JPRB, 0.10792E+01_JPRB, 0.10626E+01_JPRB, 0.10401E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.10401E+01_JPRB, 0.10700E+01_JPRB, 0.11149E+01_JPRB, 0.11788E+01_JPRB, 0.12661E+01_JPRB, 0.14201E+01_JPRB ,&
&  0.15791E+01_JPRB, 0.18258E+01_JPRB, 0.21333E+01_JPRB, 0.25012E+01_JPRB, 0.28081E+01_JPRB, 0.29514E+01_JPRB, 0.31304E+01_JPRB ,&
&  0.32589E+01_JPRB, 0.33092E+01_JPRB, 0.32735E+01_JPRB, 0.32150E+01_JPRB, 0.31263E+01_JPRB, 0.31859E+01_JPRB, 0.33951E+01_JPRB ,&
&  0.31626E+01_JPRB, 0.24344E+01_JPRB, 0.22310E+01_JPRB, 0.21902E+01_JPRB, 0.22477E+01_JPRB, 0.23062E+01_JPRB, 0.24017E+01_JPRB ,&
&  0.24807E+01_JPRB, 0.25418E+01_JPRB, 0.26119E+01_JPRB, 0.26939E+01_JPRB, 0.28215E+01_JPRB, 0.29095E+01_JPRB, 0.30143E+01_JPRB ,&
&  0.31141E+01_JPRB, 0.32188E+01_JPRB, 0.33906E+01_JPRB, 0.32492E+01_JPRB, 0.35833E+01_JPRB, 0.38390E+01_JPRB, 0.40829E+01_JPRB ,&
&  0.41841E+01_JPRB, 0.41797E+01_JPRB, 0.45353E+01_JPRB, 0.48124E+01_JPRB, 0.50192E+01_JPRB, 0.50909E+01_JPRB, 0.49719E+01_JPRB ,&
&  0.51880E+01_JPRB, 0.48545E+01_JPRB, 0.27310E+01_JPRB, 0.24266E+01_JPRB, 0.26619E+01_JPRB, 0.24538E+01_JPRB, 0.29455E+01_JPRB ,&
&  0.30995E+01_JPRB, 0.33963E+01_JPRB, 0.37465E+01_JPRB, 0.42134E+01_JPRB, 0.47762E+01_JPRB, 0.59063E+01_JPRB, 0.39424E+01_JPRB ,&
&  0.21838E+01_JPRB, 0.20426E+01_JPRB, 0.20933E+01_JPRB, 0.21611E+01_JPRB, 0.22329E+01_JPRB, 0.20079E+01_JPRB, 0.17082E+01_JPRB ,&
&  0.17075E+01_JPRB, 0.18203E+01_JPRB, 0.18906E+01_JPRB, 0.16634E+01_JPRB, 0.13082E+01_JPRB, 0.11543E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.12927E+01_JPRB, 0.13385E+01_JPRB, 0.13420E+01_JPRB, 0.13573E+01_JPRB, 0.13553E+01_JPRB, 0.13503E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.13354E+01_JPRB, 0.13253E+01_JPRB, 0.12975E+01_JPRB, 0.12397E+01_JPRB, 0.11885E+01_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.96989E+00_JPRB, 0.96952E+00_JPRB, 0.95106E+00_JPRB, 0.92785E+00_JPRB, 0.91568E+00_JPRB, 0.89430E+00_JPRB, 0.92123E+00_JPRB ,&
&  0.92435E+00_JPRB, 0.95622E+00_JPRB, 0.98924E+00_JPRB, 0.10269E+01_JPRB, 0.10912E+01_JPRB, 0.11611E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.14357E+01_JPRB, 0.17074E+01_JPRB, 0.20610E+01_JPRB, 0.25064E+01_JPRB, 0.27094E+01_JPRB, 0.28594E+01_JPRB, 0.30545E+01_JPRB ,&
&  0.32822E+01_JPRB, 0.34776E+01_JPRB, 0.36074E+01_JPRB, 0.37675E+01_JPRB, 0.39568E+01_JPRB, 0.39716E+01_JPRB, 0.38885E+01_JPRB ,&
&  0.32233E+01_JPRB, 0.22898E+01_JPRB, 0.23869E+01_JPRB, 0.22225E+01_JPRB, 0.23531E+01_JPRB, 0.24784E+01_JPRB, 0.25742E+01_JPRB ,&
&  0.26727E+01_JPRB, 0.27451E+01_JPRB, 0.28460E+01_JPRB, 0.29669E+01_JPRB, 0.31282E+01_JPRB, 0.32874E+01_JPRB, 0.34543E+01_JPRB ,&
&  0.35921E+01_JPRB, 0.37258E+01_JPRB, 0.38203E+01_JPRB, 0.37705E+01_JPRB, 0.40349E+01_JPRB, 0.43946E+01_JPRB, 0.47286E+01_JPRB ,&
&  0.49976E+01_JPRB, 0.49981E+01_JPRB, 0.45145E+01_JPRB, 0.47442E+01_JPRB, 0.51047E+01_JPRB, 0.49107E+01_JPRB, 0.51232E+01_JPRB ,&
&  0.56883E+01_JPRB, 0.50747E+01_JPRB, 0.25573E+01_JPRB, 0.22494E+01_JPRB, 0.24688E+01_JPRB, 0.26368E+01_JPRB, 0.27989E+01_JPRB ,&
&  0.29508E+01_JPRB, 0.31087E+01_JPRB, 0.33282E+01_JPRB, 0.35835E+01_JPRB, 0.37236E+01_JPRB, 0.38680E+01_JPRB, 0.31141E+01_JPRB ,&
&  0.17734E+01_JPRB, 0.16745E+01_JPRB, 0.17825E+01_JPRB, 0.19230E+01_JPRB, 0.22654E+01_JPRB, 0.22090E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.18355E+01_JPRB, 0.21950E+01_JPRB, 0.26618E+01_JPRB, 0.21681E+01_JPRB, 0.12838E+01_JPRB, 0.10596E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11103E+01_JPRB, 0.11371E+01_JPRB, 0.11571E+01_JPRB, 0.11822E+01_JPRB, 0.11683E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.11020E+01_JPRB, 0.10694E+01_JPRB, 0.10231E+01_JPRB, 0.98911E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.80867E+00_JPRB, 0.76973E+00_JPRB, 0.72383E+00_JPRB, 0.71630E+00_JPRB, 0.71858E+00_JPRB, 0.70281E+00_JPRB, 0.74539E+00_JPRB ,&
&  0.72164E+00_JPRB, 0.75937E+00_JPRB, 0.78192E+00_JPRB, 0.79299E+00_JPRB, 0.85512E+00_JPRB, 0.91821E+00_JPRB, 0.99825E+00_JPRB ,&
&  0.11068E+01_JPRB, 0.12740E+01_JPRB, 0.14897E+01_JPRB, 0.17646E+01_JPRB, 0.21037E+01_JPRB, 0.22909E+01_JPRB, 0.24751E+01_JPRB ,&
&  0.27244E+01_JPRB, 0.30494E+01_JPRB, 0.35020E+01_JPRB, 0.40248E+01_JPRB, 0.46356E+01_JPRB, 0.45135E+01_JPRB, 0.39018E+01_JPRB ,&
&  0.28743E+01_JPRB, 0.22755E+01_JPRB, 0.21157E+01_JPRB, 0.19845E+01_JPRB, 0.21479E+01_JPRB, 0.24608E+01_JPRB, 0.26224E+01_JPRB ,&
&  0.27398E+01_JPRB, 0.28734E+01_JPRB, 0.30188E+01_JPRB, 0.31804E+01_JPRB, 0.33596E+01_JPRB, 0.35637E+01_JPRB, 0.36823E+01_JPRB ,&
&  0.37649E+01_JPRB, 0.38492E+01_JPRB, 0.39234E+01_JPRB, 0.41499E+01_JPRB, 0.44772E+01_JPRB, 0.49037E+01_JPRB, 0.50275E+01_JPRB ,&
&  0.50639E+01_JPRB, 0.49663E+01_JPRB, 0.43369E+01_JPRB, 0.43165E+01_JPRB, 0.44600E+01_JPRB, 0.45268E+01_JPRB, 0.46484E+01_JPRB ,&
&  0.51370E+01_JPRB, 0.41384E+01_JPRB, 0.25783E+01_JPRB, 0.25390E+01_JPRB, 0.26383E+01_JPRB, 0.27872E+01_JPRB, 0.25542E+01_JPRB ,&
&  0.25994E+01_JPRB, 0.26542E+01_JPRB, 0.27582E+01_JPRB, 0.27698E+01_JPRB, 0.27834E+01_JPRB, 0.26920E+01_JPRB, 0.23368E+01_JPRB ,&
&  0.17906E+01_JPRB, 0.16484E+01_JPRB, 0.16288E+01_JPRB, 0.17388E+01_JPRB, 0.22557E+01_JPRB, 0.22554E+01_JPRB, 0.20004E+01_JPRB ,&
&  0.17994E+01_JPRB, 0.20319E+01_JPRB, 0.17769E+01_JPRB, 0.20932E+01_JPRB, 0.10952E+01_JPRB, 0.90237E+00_JPRB, 0.87567E+00_JPRB ,&
&  0.90686E+00_JPRB, 0.91530E+00_JPRB, 0.93508E+00_JPRB, 0.92379E+00_JPRB, 0.93826E+00_JPRB, 0.97598E+00_JPRB, 0.98554E+00_JPRB ,&
&  0.93566E+00_JPRB, 0.91991E+00_JPRB, 0.88009E+00_JPRB, 0.83571E+00_JPRB, 0.82402E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.67055E+00_JPRB, 0.65884E+00_JPRB, 0.65266E+00_JPRB, 0.64609E+00_JPRB, 0.65687E+00_JPRB, 0.66314E+00_JPRB, 0.67168E+00_JPRB ,&
&  0.67895E+00_JPRB, 0.70255E+00_JPRB, 0.72357E+00_JPRB, 0.75109E+00_JPRB, 0.77995E+00_JPRB, 0.83139E+00_JPRB, 0.88847E+00_JPRB ,&
&  0.95543E+00_JPRB, 0.10297E+01_JPRB, 0.11135E+01_JPRB, 0.12364E+01_JPRB, 0.14113E+01_JPRB, 0.15467E+01_JPRB, 0.16954E+01_JPRB ,&
&  0.18932E+01_JPRB, 0.21885E+01_JPRB, 0.25325E+01_JPRB, 0.29982E+01_JPRB, 0.35435E+01_JPRB, 0.37463E+01_JPRB, 0.35630E+01_JPRB ,&
&  0.26329E+01_JPRB, 0.24250E+01_JPRB, 0.22275E+01_JPRB, 0.20435E+01_JPRB, 0.21035E+01_JPRB, 0.20705E+01_JPRB, 0.21473E+01_JPRB ,&
&  0.22608E+01_JPRB, 0.23059E+01_JPRB, 0.23303E+01_JPRB, 0.23927E+01_JPRB, 0.24456E+01_JPRB, 0.24931E+01_JPRB, 0.24831E+01_JPRB ,&
&  0.24940E+01_JPRB, 0.26173E+01_JPRB, 0.30648E+01_JPRB, 0.36255E+01_JPRB, 0.40379E+01_JPRB, 0.42501E+01_JPRB, 0.40541E+01_JPRB ,&
&  0.37561E+01_JPRB, 0.35682E+01_JPRB, 0.33761E+01_JPRB, 0.33477E+01_JPRB, 0.34704E+01_JPRB, 0.35463E+01_JPRB, 0.38297E+01_JPRB ,&
&  0.36652E+01_JPRB, 0.30638E+01_JPRB, 0.28165E+01_JPRB, 0.26816E+01_JPRB, 0.26702E+01_JPRB, 0.26050E+01_JPRB, 0.24168E+01_JPRB ,&
&  0.24650E+01_JPRB, 0.23829E+01_JPRB, 0.23578E+01_JPRB, 0.22182E+01_JPRB, 0.20288E+01_JPRB, 0.18484E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.15899E+01_JPRB, 0.14814E+01_JPRB, 0.15200E+01_JPRB, 0.17838E+01_JPRB, 0.22571E+01_JPRB, 0.37156E+01_JPRB, 0.31878E+01_JPRB ,&
&  0.17946E+01_JPRB, 0.15157E+01_JPRB, 0.98994E+00_JPRB, 0.12812E+01_JPRB, 0.86523E+00_JPRB, 0.77054E+00_JPRB, 0.76015E+00_JPRB ,&
&  0.76529E+00_JPRB, 0.75908E+00_JPRB, 0.73908E+00_JPRB, 0.73513E+00_JPRB, 0.74775E+00_JPRB, 0.76019E+00_JPRB, 0.75276E+00_JPRB ,&
&  0.71436E+00_JPRB, 0.69798E+00_JPRB, 0.73091E+00_JPRB, 0.68227E+00_JPRB, 0.67103E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.61500E+00_JPRB, 0.61660E+00_JPRB, 0.63467E+00_JPRB, 0.65168E+00_JPRB, 0.66364E+00_JPRB, 0.67677E+00_JPRB, 0.69557E+00_JPRB ,&
&  0.71143E+00_JPRB, 0.73014E+00_JPRB, 0.74965E+00_JPRB, 0.77880E+00_JPRB, 0.81747E+00_JPRB, 0.87376E+00_JPRB, 0.93833E+00_JPRB ,&
&  0.10185E+01_JPRB, 0.11056E+01_JPRB, 0.12051E+01_JPRB, 0.13150E+01_JPRB, 0.14509E+01_JPRB, 0.16396E+01_JPRB, 0.18987E+01_JPRB ,&
&  0.20560E+01_JPRB, 0.22593E+01_JPRB, 0.25187E+01_JPRB, 0.27650E+01_JPRB, 0.28460E+01_JPRB, 0.28254E+01_JPRB, 0.23953E+01_JPRB ,&
&  0.28144E+01_JPRB, 0.24334E+01_JPRB, 0.21790E+01_JPRB, 0.19794E+01_JPRB, 0.18456E+01_JPRB, 0.17359E+01_JPRB, 0.16483E+01_JPRB ,&
&  0.16224E+01_JPRB, 0.15493E+01_JPRB, 0.14693E+01_JPRB, 0.14205E+01_JPRB, 0.13899E+01_JPRB, 0.14073E+01_JPRB, 0.14008E+01_JPRB ,&
&  0.15480E+01_JPRB, 0.18963E+01_JPRB, 0.23576E+01_JPRB, 0.29280E+01_JPRB, 0.32584E+01_JPRB, 0.32347E+01_JPRB, 0.27025E+01_JPRB ,&
&  0.23915E+01_JPRB, 0.22776E+01_JPRB, 0.22679E+01_JPRB, 0.24179E+01_JPRB, 0.24545E+01_JPRB, 0.25965E+01_JPRB, 0.26969E+01_JPRB ,&
&  0.26900E+01_JPRB, 0.24017E+01_JPRB, 0.23942E+01_JPRB, 0.28593E+01_JPRB, 0.27337E+01_JPRB, 0.25017E+01_JPRB, 0.24115E+01_JPRB ,&
&  0.22884E+01_JPRB, 0.22079E+01_JPRB, 0.21540E+01_JPRB, 0.20115E+01_JPRB, 0.18115E+01_JPRB, 0.16069E+01_JPRB, 0.15374E+01_JPRB ,&
&  0.14406E+01_JPRB, 0.13310E+01_JPRB, 0.13123E+01_JPRB, 0.14944E+01_JPRB, 0.19776E+01_JPRB, 0.25449E+01_JPRB, 0.35917E+01_JPRB ,&
&  0.18995E+01_JPRB, 0.10835E+01_JPRB, 0.10409E+01_JPRB, 0.95696E+00_JPRB, 0.81368E+00_JPRB, 0.70353E+00_JPRB, 0.74341E+00_JPRB ,&
&  0.80546E+00_JPRB, 0.87202E+00_JPRB, 0.86118E+00_JPRB, 0.75063E+00_JPRB, 0.70887E+00_JPRB, 0.69188E+00_JPRB, 0.66868E+00_JPRB ,&
&  0.64905E+00_JPRB, 0.61203E+00_JPRB, 0.62934E+00_JPRB, 0.61323E+00_JPRB, 0.61438E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.57535E+00_JPRB, 0.57541E+00_JPRB, 0.58659E+00_JPRB, 0.59751E+00_JPRB, 0.60252E+00_JPRB, 0.60876E+00_JPRB, 0.61564E+00_JPRB ,&
&  0.62319E+00_JPRB, 0.62699E+00_JPRB, 0.63240E+00_JPRB, 0.64276E+00_JPRB, 0.66324E+00_JPRB, 0.70778E+00_JPRB, 0.76234E+00_JPRB ,&
&  0.81373E+00_JPRB, 0.86593E+00_JPRB, 0.92201E+00_JPRB, 0.97288E+00_JPRB, 0.10272E+01_JPRB, 0.10697E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.12872E+01_JPRB, 0.14351E+01_JPRB, 0.15930E+01_JPRB, 0.17270E+01_JPRB, 0.18722E+01_JPRB, 0.19350E+01_JPRB, 0.16956E+01_JPRB ,&
&  0.23774E+01_JPRB, 0.22284E+01_JPRB, 0.20246E+01_JPRB, 0.18269E+01_JPRB, 0.16855E+01_JPRB, 0.15575E+01_JPRB, 0.14359E+01_JPRB ,&
&  0.13303E+01_JPRB, 0.14227E+01_JPRB, 0.13600E+01_JPRB, 0.14444E+01_JPRB, 0.12090E+01_JPRB, 0.12567E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.17209E+01_JPRB, 0.19656E+01_JPRB, 0.22703E+01_JPRB, 0.25735E+01_JPRB, 0.27560E+01_JPRB, 0.27122E+01_JPRB, 0.24303E+01_JPRB ,&
&  0.22693E+01_JPRB, 0.22256E+01_JPRB, 0.21625E+01_JPRB, 0.23743E+01_JPRB, 0.22974E+01_JPRB, 0.21688E+01_JPRB, 0.22179E+01_JPRB ,&
&  0.19756E+01_JPRB, 0.20050E+01_JPRB, 0.22822E+01_JPRB, 0.27901E+01_JPRB, 0.26389E+01_JPRB, 0.25131E+01_JPRB, 0.23974E+01_JPRB ,&
&  0.22500E+01_JPRB, 0.21344E+01_JPRB, 0.20400E+01_JPRB, 0.18871E+01_JPRB, 0.17044E+01_JPRB, 0.15178E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.12192E+01_JPRB, 0.11651E+01_JPRB, 0.11990E+01_JPRB, 0.13878E+01_JPRB, 0.17031E+01_JPRB, 0.30042E+01_JPRB ,&
&  0.18514E+01_JPRB, 0.14340E+01_JPRB, 0.15429E+01_JPRB, 0.60077E+00_JPRB, 0.67136E+00_JPRB, 0.66864E+00_JPRB, 0.77036E+00_JPRB ,&
&  0.92039E+00_JPRB, 0.14323E+01_JPRB, 0.21194E+01_JPRB, 0.18872E+01_JPRB, 0.14532E+01_JPRB, 0.95827E+00_JPRB, 0.60560E+00_JPRB ,&
&  0.58683E+00_JPRB, 0.58254E+00_JPRB, 0.58887E+00_JPRB, 0.58395E+00_JPRB, 0.58235E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.51869E+00_JPRB, 0.51050E+00_JPRB, 0.52157E+00_JPRB, 0.53376E+00_JPRB, 0.53140E+00_JPRB, 0.53276E+00_JPRB, 0.52819E+00_JPRB ,&
&  0.51943E+00_JPRB, 0.51267E+00_JPRB, 0.53742E+00_JPRB, 0.58430E+00_JPRB, 0.65170E+00_JPRB, 0.74107E+00_JPRB, 0.83476E+00_JPRB ,&
&  0.90900E+00_JPRB, 0.10014E+01_JPRB, 0.10907E+01_JPRB, 0.11676E+01_JPRB, 0.11994E+01_JPRB, 0.12265E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.12441E+01_JPRB, 0.13013E+01_JPRB, 0.14191E+01_JPRB, 0.15524E+01_JPRB, 0.17634E+01_JPRB, 0.20950E+01_JPRB, 0.13520E+01_JPRB ,&
&  0.19325E+01_JPRB, 0.19152E+01_JPRB, 0.18459E+01_JPRB, 0.17001E+01_JPRB, 0.16249E+01_JPRB, 0.15024E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.14664E+01_JPRB, 0.15863E+01_JPRB, 0.15852E+01_JPRB, 0.16900E+01_JPRB, 0.17785E+01_JPRB, 0.18438E+01_JPRB, 0.21516E+01_JPRB ,&
&  0.24284E+01_JPRB, 0.27865E+01_JPRB, 0.31748E+01_JPRB, 0.34380E+01_JPRB, 0.36525E+01_JPRB, 0.37502E+01_JPRB, 0.37496E+01_JPRB ,&
&  0.35141E+01_JPRB, 0.33553E+01_JPRB, 0.34900E+01_JPRB, 0.37867E+01_JPRB, 0.33105E+01_JPRB, 0.30994E+01_JPRB, 0.26077E+01_JPRB ,&
&  0.19468E+01_JPRB, 0.17755E+01_JPRB, 0.20329E+01_JPRB, 0.24854E+01_JPRB, 0.25201E+01_JPRB, 0.23937E+01_JPRB, 0.21739E+01_JPRB ,&
&  0.20176E+01_JPRB, 0.18632E+01_JPRB, 0.17492E+01_JPRB, 0.16198E+01_JPRB, 0.14854E+01_JPRB, 0.13455E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.11424E+01_JPRB, 0.10804E+01_JPRB, 0.10140E+01_JPRB, 0.10513E+01_JPRB, 0.11980E+01_JPRB, 0.16050E+01_JPRB, 0.17966E+01_JPRB ,&
&  0.19436E+01_JPRB, 0.13208E+01_JPRB, 0.10715E+01_JPRB, 0.97720E+00_JPRB, 0.67037E+00_JPRB, 0.66179E+00_JPRB, 0.81809E+00_JPRB ,&
&  0.12397E+01_JPRB, 0.17611E+01_JPRB, 0.28871E+01_JPRB, 0.51075E+01_JPRB, 0.28405E+01_JPRB, 0.22055E+01_JPRB, 0.58423E+00_JPRB ,&
&  0.56365E+00_JPRB, 0.56383E+00_JPRB, 0.56195E+00_JPRB, 0.54690E+00_JPRB, 0.54222E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.49740E+00_JPRB, 0.49027E+00_JPRB, 0.50184E+00_JPRB, 0.50047E+00_JPRB, 0.50444E+00_JPRB, 0.51240E+00_JPRB, 0.51685E+00_JPRB ,&
&  0.52196E+00_JPRB, 0.54014E+00_JPRB, 0.63959E+00_JPRB, 0.78042E+00_JPRB, 0.91981E+00_JPRB, 0.10316E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.11720E+01_JPRB, 0.12380E+01_JPRB, 0.12820E+01_JPRB, 0.13207E+01_JPRB, 0.13974E+01_JPRB, 0.14999E+01_JPRB, 0.15893E+01_JPRB ,&
&  0.16808E+01_JPRB, 0.17770E+01_JPRB, 0.18902E+01_JPRB, 0.20614E+01_JPRB, 0.22288E+01_JPRB, 0.22295E+01_JPRB, 0.13675E+01_JPRB ,&
&  0.17169E+01_JPRB, 0.18017E+01_JPRB, 0.18419E+01_JPRB, 0.17384E+01_JPRB, 0.16240E+01_JPRB, 0.15792E+01_JPRB, 0.17217E+01_JPRB ,&
&  0.18885E+01_JPRB, 0.19765E+01_JPRB, 0.19555E+01_JPRB, 0.20061E+01_JPRB, 0.26899E+01_JPRB, 0.28282E+01_JPRB, 0.34084E+01_JPRB ,&
&  0.38147E+01_JPRB, 0.42492E+01_JPRB, 0.45591E+01_JPRB, 0.48950E+01_JPRB, 0.51755E+01_JPRB, 0.53805E+01_JPRB, 0.56685E+01_JPRB ,&
&  0.57749E+01_JPRB, 0.59077E+01_JPRB, 0.59353E+01_JPRB, 0.57930E+01_JPRB, 0.47028E+01_JPRB, 0.49482E+01_JPRB, 0.33319E+01_JPRB ,&
&  0.23298E+01_JPRB, 0.19908E+01_JPRB, 0.21871E+01_JPRB, 0.22469E+01_JPRB, 0.21222E+01_JPRB, 0.18870E+01_JPRB, 0.15953E+01_JPRB ,&
&  0.15015E+01_JPRB, 0.14775E+01_JPRB, 0.14557E+01_JPRB, 0.13763E+01_JPRB, 0.13148E+01_JPRB, 0.11969E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10036E+01_JPRB, 0.93695E+00_JPRB, 0.98490E+00_JPRB, 0.11813E+01_JPRB, 0.17048E+01_JPRB, 0.25448E+01_JPRB ,&
&  0.26972E+01_JPRB, 0.14095E+01_JPRB, 0.11221E+01_JPRB, 0.91665E+00_JPRB, 0.80599E+00_JPRB, 0.79319E+00_JPRB, 0.85560E+00_JPRB ,&
&  0.10788E+01_JPRB, 0.18951E+01_JPRB, 0.27499E+01_JPRB, 0.20267E+01_JPRB, 0.22717E+01_JPRB, 0.15774E+01_JPRB, 0.68112E+00_JPRB ,&
&  0.54998E+00_JPRB, 0.52980E+00_JPRB, 0.53436E+00_JPRB, 0.52626E+00_JPRB, 0.51362E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.54560E+00_JPRB, 0.53445E+00_JPRB, 0.53302E+00_JPRB, 0.54357E+00_JPRB, 0.55034E+00_JPRB, 0.57678E+00_JPRB, 0.60917E+00_JPRB ,&
&  0.62843E+00_JPRB, 0.70669E+00_JPRB, 0.88283E+00_JPRB, 0.10379E+01_JPRB, 0.11162E+01_JPRB, 0.11765E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11900E+01_JPRB, 0.12027E+01_JPRB, 0.12029E+01_JPRB, 0.12541E+01_JPRB, 0.13402E+01_JPRB, 0.14585E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.17845E+01_JPRB, 0.19551E+01_JPRB, 0.21143E+01_JPRB, 0.22626E+01_JPRB, 0.24652E+01_JPRB, 0.25094E+01_JPRB, 0.24774E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.18109E+01_JPRB, 0.18526E+01_JPRB, 0.17783E+01_JPRB, 0.16795E+01_JPRB, 0.17264E+01_JPRB, 0.20290E+01_JPRB ,&
&  0.23034E+01_JPRB, 0.23707E+01_JPRB, 0.27632E+01_JPRB, 0.26515E+01_JPRB, 0.33350E+01_JPRB, 0.37575E+01_JPRB, 0.41972E+01_JPRB ,&
&  0.46194E+01_JPRB, 0.49311E+01_JPRB, 0.52631E+01_JPRB, 0.55851E+01_JPRB, 0.59029E+01_JPRB, 0.61864E+01_JPRB, 0.65070E+01_JPRB ,&
&  0.68869E+01_JPRB, 0.74015E+01_JPRB, 0.79487E+01_JPRB, 0.76631E+01_JPRB, 0.64728E+01_JPRB, 0.60154E+01_JPRB, 0.46584E+01_JPRB ,&
&  0.30767E+01_JPRB, 0.19071E+01_JPRB, 0.18701E+01_JPRB, 0.19243E+01_JPRB, 0.17313E+01_JPRB, 0.15369E+01_JPRB, 0.13539E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.12345E+01_JPRB, 0.11950E+01_JPRB, 0.12170E+01_JPRB, 0.12331E+01_JPRB, 0.11743E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.10319E+01_JPRB, 0.97579E+00_JPRB, 0.96636E+00_JPRB, 0.10568E+01_JPRB, 0.12438E+01_JPRB, 0.14047E+01_JPRB ,&
&  0.14305E+01_JPRB, 0.12234E+01_JPRB, 0.10617E+01_JPRB, 0.10162E+01_JPRB, 0.95281E+00_JPRB, 0.90883E+00_JPRB, 0.87821E+00_JPRB ,&
&  0.91536E+00_JPRB, 0.10864E+01_JPRB, 0.14946E+01_JPRB, 0.18840E+01_JPRB, 0.14707E+01_JPRB, 0.11666E+01_JPRB, 0.69149E+00_JPRB ,&
&  0.56670E+00_JPRB, 0.54730E+00_JPRB, 0.56424E+00_JPRB, 0.56836E+00_JPRB, 0.56911E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.65818E+00_JPRB, 0.64396E+00_JPRB, 0.62330E+00_JPRB, 0.63872E+00_JPRB, 0.67243E+00_JPRB, 0.71747E+00_JPRB, 0.73880E+00_JPRB ,&
&  0.76816E+00_JPRB, 0.88143E+00_JPRB, 0.10308E+01_JPRB, 0.11653E+01_JPRB, 0.11998E+01_JPRB, 0.11812E+01_JPRB, 0.11067E+01_JPRB ,&
&  0.11070E+01_JPRB, 0.11073E+01_JPRB, 0.11214E+01_JPRB, 0.11463E+01_JPRB, 0.12099E+01_JPRB, 0.13359E+01_JPRB, 0.15216E+01_JPRB ,&
&  0.17635E+01_JPRB, 0.20316E+01_JPRB, 0.23324E+01_JPRB, 0.26633E+01_JPRB, 0.29793E+01_JPRB, 0.31682E+01_JPRB, 0.30585E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.14027E+01_JPRB, 0.19481E+01_JPRB, 0.19566E+01_JPRB, 0.18901E+01_JPRB, 0.19182E+01_JPRB, 0.21183E+01_JPRB ,&
&  0.23210E+01_JPRB, 0.23796E+01_JPRB, 0.27001E+01_JPRB, 0.32315E+01_JPRB, 0.35079E+01_JPRB, 0.40142E+01_JPRB, 0.43104E+01_JPRB ,&
&  0.46149E+01_JPRB, 0.49242E+01_JPRB, 0.52604E+01_JPRB, 0.56048E+01_JPRB, 0.59453E+01_JPRB, 0.62658E+01_JPRB, 0.66697E+01_JPRB ,&
&  0.72101E+01_JPRB, 0.78123E+01_JPRB, 0.83124E+01_JPRB, 0.74888E+01_JPRB, 0.78984E+01_JPRB, 0.68921E+01_JPRB, 0.59183E+01_JPRB ,&
&  0.34215E+01_JPRB, 0.19522E+01_JPRB, 0.19042E+01_JPRB, 0.19516E+01_JPRB, 0.16706E+01_JPRB, 0.14308E+01_JPRB, 0.11797E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.10168E+01_JPRB, 0.10750E+01_JPRB, 0.11531E+01_JPRB, 0.11387E+01_JPRB, 0.11771E+01_JPRB, 0.11121E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11079E+01_JPRB, 0.10783E+01_JPRB, 0.10784E+01_JPRB, 0.11081E+01_JPRB, 0.11547E+01_JPRB, 0.11657E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11302E+01_JPRB, 0.11028E+01_JPRB, 0.10836E+01_JPRB, 0.10508E+01_JPRB, 0.10149E+01_JPRB, 0.95588E+00_JPRB ,&
&  0.89158E+00_JPRB, 0.89176E+00_JPRB, 0.98761E+00_JPRB, 0.11155E+01_JPRB, 0.10373E+01_JPRB, 0.89956E+00_JPRB, 0.72474E+00_JPRB ,&
&  0.71115E+00_JPRB, 0.73177E+00_JPRB, 0.73034E+00_JPRB, 0.71089E+00_JPRB, 0.68848E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.78544E+00_JPRB, 0.75549E+00_JPRB, 0.71518E+00_JPRB, 0.70261E+00_JPRB, 0.75286E+00_JPRB, 0.80333E+00_JPRB, 0.80731E+00_JPRB ,&
&  0.82141E+00_JPRB, 0.89473E+00_JPRB, 0.10251E+01_JPRB, 0.11279E+01_JPRB, 0.11651E+01_JPRB, 0.11622E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.10616E+01_JPRB, 0.10501E+01_JPRB, 0.10588E+01_JPRB, 0.10853E+01_JPRB, 0.11422E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.14058E+01_JPRB, 0.16650E+01_JPRB, 0.20358E+01_JPRB, 0.25690E+01_JPRB, 0.33227E+01_JPRB, 0.42148E+01_JPRB, 0.49971E+01_JPRB ,&
&  0.37824E+01_JPRB, 0.14614E+01_JPRB, 0.17584E+01_JPRB, 0.22692E+01_JPRB, 0.21223E+01_JPRB, 0.21795E+01_JPRB, 0.21610E+01_JPRB ,&
&  0.22660E+01_JPRB, 0.24223E+01_JPRB, 0.25977E+01_JPRB, 0.30714E+01_JPRB, 0.34603E+01_JPRB, 0.38924E+01_JPRB, 0.40966E+01_JPRB ,&
&  0.42841E+01_JPRB, 0.45157E+01_JPRB, 0.47573E+01_JPRB, 0.50088E+01_JPRB, 0.52891E+01_JPRB, 0.55971E+01_JPRB, 0.60558E+01_JPRB ,&
&  0.66406E+01_JPRB, 0.72209E+01_JPRB, 0.76178E+01_JPRB, 0.65534E+01_JPRB, 0.68985E+01_JPRB, 0.54474E+01_JPRB, 0.40438E+01_JPRB ,&
&  0.30610E+01_JPRB, 0.23700E+01_JPRB, 0.21088E+01_JPRB, 0.20620E+01_JPRB, 0.17121E+01_JPRB, 0.12802E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10010E+01_JPRB, 0.10475E+01_JPRB, 0.10231E+01_JPRB, 0.10913E+01_JPRB, 0.11233E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.11398E+01_JPRB, 0.11627E+01_JPRB, 0.11843E+01_JPRB, 0.12251E+01_JPRB, 0.12188E+01_JPRB, 0.12243E+01_JPRB, 0.11973E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.12064E+01_JPRB, 0.12060E+01_JPRB, 0.12080E+01_JPRB, 0.11992E+01_JPRB, 0.11026E+01_JPRB, 0.10403E+01_JPRB ,&
&  0.99835E+00_JPRB, 0.97485E+00_JPRB, 0.92756E+00_JPRB, 0.95792E+00_JPRB, 0.94522E+00_JPRB, 0.94432E+00_JPRB, 0.97448E+00_JPRB ,&
&  0.98257E+00_JPRB, 0.98293E+00_JPRB, 0.92445E+00_JPRB, 0.86400E+00_JPRB, 0.82734E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.87354E+00_JPRB, 0.80974E+00_JPRB, 0.77421E+00_JPRB, 0.78221E+00_JPRB, 0.83284E+00_JPRB, 0.88433E+00_JPRB, 0.88018E+00_JPRB ,&
&  0.86486E+00_JPRB, 0.88981E+00_JPRB, 0.96790E+00_JPRB, 0.10375E+01_JPRB, 0.10499E+01_JPRB, 0.10571E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10297E+01_JPRB, 0.10226E+01_JPRB, 0.10065E+01_JPRB, 0.10135E+01_JPRB, 0.10209E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.12767E+01_JPRB, 0.15348E+01_JPRB, 0.19235E+01_JPRB, 0.24959E+01_JPRB, 0.35511E+01_JPRB, 0.51917E+01_JPRB ,&
&  0.77088E+01_JPRB, 0.50506E+01_JPRB, 0.16792E+01_JPRB, 0.22423E+01_JPRB, 0.24071E+01_JPRB, 0.24809E+01_JPRB, 0.21525E+01_JPRB ,&
&  0.23238E+01_JPRB, 0.24592E+01_JPRB, 0.23751E+01_JPRB, 0.27824E+01_JPRB, 0.31794E+01_JPRB, 0.34828E+01_JPRB, 0.36820E+01_JPRB ,&
&  0.37592E+01_JPRB, 0.38903E+01_JPRB, 0.40104E+01_JPRB, 0.41159E+01_JPRB, 0.42470E+01_JPRB, 0.44550E+01_JPRB, 0.47991E+01_JPRB ,&
&  0.54428E+01_JPRB, 0.62463E+01_JPRB, 0.69202E+01_JPRB, 0.60379E+01_JPRB, 0.60406E+01_JPRB, 0.46198E+01_JPRB, 0.37763E+01_JPRB ,&
&  0.32094E+01_JPRB, 0.30674E+01_JPRB, 0.28221E+01_JPRB, 0.22764E+01_JPRB, 0.17855E+01_JPRB, 0.14815E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.10545E+01_JPRB, 0.10525E+01_JPRB, 0.11022E+01_JPRB, 0.11138E+01_JPRB, 0.11317E+01_JPRB, 0.11514E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12119E+01_JPRB, 0.12664E+01_JPRB, 0.12990E+01_JPRB, 0.13246E+01_JPRB, 0.13662E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.13376E+01_JPRB, 0.13243E+01_JPRB, 0.13222E+01_JPRB, 0.13233E+01_JPRB, 0.12554E+01_JPRB, 0.11599E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.11334E+01_JPRB, 0.11168E+01_JPRB, 0.10636E+01_JPRB, 0.10615E+01_JPRB, 0.10594E+01_JPRB, 0.11116E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.12017E+01_JPRB, 0.11625E+01_JPRB, 0.10793E+01_JPRB, 0.94265E+00_JPRB, 0.92628E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.94888E+00_JPRB, 0.92259E+00_JPRB, 0.88674E+00_JPRB, 0.86186E+00_JPRB, 0.88646E+00_JPRB, 0.92399E+00_JPRB, 0.92348E+00_JPRB ,&
&  0.89583E+00_JPRB, 0.87769E+00_JPRB, 0.91735E+00_JPRB, 0.96064E+00_JPRB, 0.95973E+00_JPRB, 0.98367E+00_JPRB, 0.99786E+00_JPRB ,&
&  0.10385E+01_JPRB, 0.10382E+01_JPRB, 0.10214E+01_JPRB, 0.99861E+00_JPRB, 0.99759E+00_JPRB, 0.98506E+00_JPRB, 0.98977E+00_JPRB ,&
&  0.10184E+01_JPRB, 0.10838E+01_JPRB, 0.12221E+01_JPRB, 0.14375E+01_JPRB, 0.17092E+01_JPRB, 0.22107E+01_JPRB, 0.32210E+01_JPRB ,&
&  0.43556E+01_JPRB, 0.55785E+01_JPRB, 0.18045E+01_JPRB, 0.24494E+01_JPRB, 0.30313E+01_JPRB, 0.29336E+01_JPRB, 0.26341E+01_JPRB ,&
&  0.26221E+01_JPRB, 0.25723E+01_JPRB, 0.24386E+01_JPRB, 0.23905E+01_JPRB, 0.26297E+01_JPRB, 0.28693E+01_JPRB, 0.28636E+01_JPRB ,&
&  0.29475E+01_JPRB, 0.31135E+01_JPRB, 0.33754E+01_JPRB, 0.34406E+01_JPRB, 0.34388E+01_JPRB, 0.34132E+01_JPRB, 0.35548E+01_JPRB ,&
&  0.40037E+01_JPRB, 0.47921E+01_JPRB, 0.59601E+01_JPRB, 0.55744E+01_JPRB, 0.55620E+01_JPRB, 0.48273E+01_JPRB, 0.42247E+01_JPRB ,&
&  0.37373E+01_JPRB, 0.36500E+01_JPRB, 0.33649E+01_JPRB, 0.26640E+01_JPRB, 0.20221E+01_JPRB, 0.17382E+01_JPRB, 0.16347E+01_JPRB ,&
&  0.13935E+01_JPRB, 0.12322E+01_JPRB, 0.12176E+01_JPRB, 0.12178E+01_JPRB, 0.12513E+01_JPRB, 0.12361E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.12373E+01_JPRB, 0.12744E+01_JPRB, 0.13254E+01_JPRB, 0.13759E+01_JPRB, 0.14371E+01_JPRB, 0.14617E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.14431E+01_JPRB, 0.14211E+01_JPRB, 0.13539E+01_JPRB, 0.12659E+01_JPRB, 0.12228E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.12251E+01_JPRB, 0.12300E+01_JPRB, 0.11785E+01_JPRB, 0.11220E+01_JPRB, 0.11541E+01_JPRB, 0.11575E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.13376E+01_JPRB, 0.13914E+01_JPRB, 0.13214E+01_JPRB, 0.10380E+01_JPRB, 0.10149E+01_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.10801E+01_JPRB, 0.10285E+01_JPRB, 0.99331E+00_JPRB, 0.95916E+00_JPRB, 0.93217E+00_JPRB, 0.96612E+00_JPRB, 0.96117E+00_JPRB ,&
&  0.93851E+00_JPRB, 0.90039E+00_JPRB, 0.91588E+00_JPRB, 0.96899E+00_JPRB, 0.97516E+00_JPRB, 0.96030E+00_JPRB, 0.95048E+00_JPRB ,&
&  0.95169E+00_JPRB, 0.96839E+00_JPRB, 0.97236E+00_JPRB, 0.98888E+00_JPRB, 0.99377E+00_JPRB, 0.98599E+00_JPRB, 0.98316E+00_JPRB ,&
&  0.10033E+01_JPRB, 0.10336E+01_JPRB, 0.10908E+01_JPRB, 0.11649E+01_JPRB, 0.12680E+01_JPRB, 0.14264E+01_JPRB, 0.19370E+01_JPRB ,&
&  0.26156E+01_JPRB, 0.43930E+01_JPRB, 0.35091E+01_JPRB, 0.42013E+01_JPRB, 0.39835E+01_JPRB, 0.36609E+01_JPRB, 0.32746E+01_JPRB ,&
&  0.35049E+01_JPRB, 0.31936E+01_JPRB, 0.30069E+01_JPRB, 0.23368E+01_JPRB, 0.21123E+01_JPRB, 0.21536E+01_JPRB, 0.20902E+01_JPRB ,&
&  0.20186E+01_JPRB, 0.20149E+01_JPRB, 0.23614E+01_JPRB, 0.25347E+01_JPRB, 0.26091E+01_JPRB, 0.26326E+01_JPRB, 0.26733E+01_JPRB ,&
&  0.29113E+01_JPRB, 0.33415E+01_JPRB, 0.44521E+01_JPRB, 0.52990E+01_JPRB, 0.51516E+01_JPRB, 0.50230E+01_JPRB, 0.48922E+01_JPRB ,&
&  0.49137E+01_JPRB, 0.43716E+01_JPRB, 0.38340E+01_JPRB, 0.30577E+01_JPRB, 0.23373E+01_JPRB, 0.21969E+01_JPRB, 0.18413E+01_JPRB ,&
&  0.15814E+01_JPRB, 0.14093E+01_JPRB, 0.13743E+01_JPRB, 0.13578E+01_JPRB, 0.13688E+01_JPRB, 0.13295E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.12713E+01_JPRB, 0.12882E+01_JPRB, 0.12781E+01_JPRB, 0.13202E+01_JPRB, 0.13574E+01_JPRB, 0.14003E+01_JPRB, 0.14345E+01_JPRB ,&
&  0.14694E+01_JPRB, 0.14887E+01_JPRB, 0.14304E+01_JPRB, 0.13710E+01_JPRB, 0.13018E+01_JPRB, 0.12610E+01_JPRB, 0.12293E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12737E+01_JPRB, 0.12640E+01_JPRB, 0.12125E+01_JPRB, 0.11802E+01_JPRB, 0.11616E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.11743E+01_JPRB, 0.12269E+01_JPRB, 0.11919E+01_JPRB, 0.11344E+01_JPRB, 0.11144E+01_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.11697E+01_JPRB, 0.11393E+01_JPRB, 0.10780E+01_JPRB, 0.10555E+01_JPRB, 0.10080E+01_JPRB, 0.10266E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.10002E+01_JPRB, 0.96673E+00_JPRB, 0.93868E+00_JPRB, 0.95974E+00_JPRB, 0.98431E+00_JPRB, 0.97596E+00_JPRB, 0.94146E+00_JPRB ,&
&  0.92965E+00_JPRB, 0.92531E+00_JPRB, 0.92272E+00_JPRB, 0.92818E+00_JPRB, 0.94749E+00_JPRB, 0.97579E+00_JPRB, 0.97745E+00_JPRB ,&
&  0.97561E+00_JPRB, 0.10187E+01_JPRB, 0.10728E+01_JPRB, 0.10943E+01_JPRB, 0.11247E+01_JPRB, 0.11869E+01_JPRB, 0.14286E+01_JPRB ,&
&  0.18128E+01_JPRB, 0.43382E+01_JPRB, 0.27118E+01_JPRB, 0.53961E+01_JPRB, 0.43068E+01_JPRB, 0.38613E+01_JPRB, 0.31606E+01_JPRB ,&
&  0.29320E+01_JPRB, 0.30715E+01_JPRB, 0.25080E+01_JPRB, 0.21147E+01_JPRB, 0.18285E+01_JPRB, 0.17067E+01_JPRB, 0.15839E+01_JPRB ,&
&  0.14066E+01_JPRB, 0.14079E+01_JPRB, 0.15530E+01_JPRB, 0.17339E+01_JPRB, 0.18538E+01_JPRB, 0.19225E+01_JPRB, 0.20220E+01_JPRB ,&
&  0.21882E+01_JPRB, 0.24002E+01_JPRB, 0.31338E+01_JPRB, 0.41011E+01_JPRB, 0.49848E+01_JPRB, 0.52675E+01_JPRB, 0.56792E+01_JPRB ,&
&  0.58173E+01_JPRB, 0.52911E+01_JPRB, 0.43529E+01_JPRB, 0.34296E+01_JPRB, 0.27809E+01_JPRB, 0.25450E+01_JPRB, 0.22313E+01_JPRB ,&
&  0.19429E+01_JPRB, 0.16807E+01_JPRB, 0.16066E+01_JPRB, 0.15344E+01_JPRB, 0.15223E+01_JPRB, 0.14705E+01_JPRB, 0.13738E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13015E+01_JPRB, 0.13293E+01_JPRB, 0.13561E+01_JPRB, 0.13923E+01_JPRB, 0.13995E+01_JPRB, 0.14020E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.14711E+01_JPRB, 0.14355E+01_JPRB, 0.14205E+01_JPRB, 0.13641E+01_JPRB, 0.13212E+01_JPRB, 0.13032E+01_JPRB ,&
&  0.13414E+01_JPRB, 0.13360E+01_JPRB, 0.12869E+01_JPRB, 0.12264E+01_JPRB, 0.11914E+01_JPRB, 0.11942E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.11358E+01_JPRB, 0.11700E+01_JPRB, 0.12076E+01_JPRB, 0.11728E+01_JPRB, 0.11702E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.12088E+01_JPRB, 0.12005E+01_JPRB, 0.11234E+01_JPRB, 0.10937E+01_JPRB, 0.10509E+01_JPRB, 0.10312E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10017E+01_JPRB, 0.97013E+00_JPRB, 0.95521E+00_JPRB, 0.96900E+00_JPRB, 0.98450E+00_JPRB, 0.95351E+00_JPRB ,&
&  0.91766E+00_JPRB, 0.90283E+00_JPRB, 0.89243E+00_JPRB, 0.89443E+00_JPRB, 0.90850E+00_JPRB, 0.92605E+00_JPRB, 0.93868E+00_JPRB ,&
&  0.93667E+00_JPRB, 0.98395E+00_JPRB, 0.10509E+01_JPRB, 0.10912E+01_JPRB, 0.11028E+01_JPRB, 0.11590E+01_JPRB, 0.13449E+01_JPRB ,&
&  0.18163E+01_JPRB, 0.60877E+01_JPRB, 0.31091E+01_JPRB, 0.56552E+01_JPRB, 0.44577E+01_JPRB, 0.38428E+01_JPRB, 0.30523E+01_JPRB ,&
&  0.27717E+01_JPRB, 0.25989E+01_JPRB, 0.22525E+01_JPRB, 0.19814E+01_JPRB, 0.17801E+01_JPRB, 0.16156E+01_JPRB, 0.14776E+01_JPRB ,&
&  0.13617E+01_JPRB, 0.13051E+01_JPRB, 0.13269E+01_JPRB, 0.14301E+01_JPRB, 0.14963E+01_JPRB, 0.15335E+01_JPRB, 0.15901E+01_JPRB ,&
&  0.16989E+01_JPRB, 0.18495E+01_JPRB, 0.22877E+01_JPRB, 0.35040E+01_JPRB, 0.44743E+01_JPRB, 0.53352E+01_JPRB, 0.76551E+01_JPRB ,&
&  0.72932E+01_JPRB, 0.66714E+01_JPRB, 0.48982E+01_JPRB, 0.38476E+01_JPRB, 0.31605E+01_JPRB, 0.27695E+01_JPRB, 0.25677E+01_JPRB ,&
&  0.23576E+01_JPRB, 0.20645E+01_JPRB, 0.19249E+01_JPRB, 0.18263E+01_JPRB, 0.17197E+01_JPRB, 0.16139E+01_JPRB, 0.15140E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.13981E+01_JPRB, 0.14147E+01_JPRB, 0.14303E+01_JPRB, 0.14427E+01_JPRB, 0.14293E+01_JPRB, 0.14159E+01_JPRB ,&
&  0.14233E+01_JPRB, 0.14483E+01_JPRB, 0.14243E+01_JPRB, 0.14293E+01_JPRB, 0.14090E+01_JPRB, 0.13740E+01_JPRB, 0.13541E+01_JPRB ,&
&  0.14070E+01_JPRB, 0.14065E+01_JPRB, 0.13301E+01_JPRB, 0.12804E+01_JPRB, 0.12231E+01_JPRB, 0.12443E+01_JPRB, 0.11731E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.11267E+01_JPRB, 0.11832E+01_JPRB, 0.11898E+01_JPRB, 0.12099E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.12256E+01_JPRB, 0.12087E+01_JPRB, 0.11673E+01_JPRB, 0.11041E+01_JPRB, 0.10759E+01_JPRB, 0.10467E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.10063E+01_JPRB, 0.99118E+00_JPRB, 0.96859E+00_JPRB, 0.95893E+00_JPRB, 0.97135E+00_JPRB, 0.96605E+00_JPRB ,&
&  0.94806E+00_JPRB, 0.92402E+00_JPRB, 0.91430E+00_JPRB, 0.90415E+00_JPRB, 0.89278E+00_JPRB, 0.89231E+00_JPRB, 0.90442E+00_JPRB ,&
&  0.91359E+00_JPRB, 0.93570E+00_JPRB, 0.97657E+00_JPRB, 0.10162E+01_JPRB, 0.10300E+01_JPRB, 0.10700E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.19607E+01_JPRB, 0.21229E+01_JPRB, 0.46576E+01_JPRB, 0.55334E+01_JPRB, 0.45676E+01_JPRB, 0.39642E+01_JPRB, 0.33262E+01_JPRB ,&
&  0.29085E+01_JPRB, 0.24196E+01_JPRB, 0.20236E+01_JPRB, 0.17756E+01_JPRB, 0.16413E+01_JPRB, 0.14873E+01_JPRB, 0.14011E+01_JPRB ,&
&  0.13498E+01_JPRB, 0.13143E+01_JPRB, 0.13071E+01_JPRB, 0.13262E+01_JPRB, 0.13295E+01_JPRB, 0.13331E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.14383E+01_JPRB, 0.15422E+01_JPRB, 0.17731E+01_JPRB, 0.25566E+01_JPRB, 0.40805E+01_JPRB, 0.55605E+01_JPRB, 0.76090E+01_JPRB ,&
&  0.85851E+01_JPRB, 0.77719E+01_JPRB, 0.54453E+01_JPRB, 0.41673E+01_JPRB, 0.34789E+01_JPRB, 0.30201E+01_JPRB, 0.27877E+01_JPRB ,&
&  0.26213E+01_JPRB, 0.23951E+01_JPRB, 0.22123E+01_JPRB, 0.21604E+01_JPRB, 0.19938E+01_JPRB, 0.18117E+01_JPRB, 0.16628E+01_JPRB ,&
&  0.15886E+01_JPRB, 0.15390E+01_JPRB, 0.14843E+01_JPRB, 0.14448E+01_JPRB, 0.14456E+01_JPRB, 0.14337E+01_JPRB, 0.14139E+01_JPRB ,&
&  0.14085E+01_JPRB, 0.14163E+01_JPRB, 0.14000E+01_JPRB, 0.13925E+01_JPRB, 0.14149E+01_JPRB, 0.14191E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.14175E+01_JPRB, 0.14263E+01_JPRB, 0.13776E+01_JPRB, 0.13404E+01_JPRB, 0.12534E+01_JPRB, 0.12878E+01_JPRB, 0.12146E+01_JPRB ,&
&  0.11632E+01_JPRB, 0.11254E+01_JPRB, 0.11271E+01_JPRB, 0.11753E+01_JPRB, 0.12189E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.12010E+01_JPRB, 0.11898E+01_JPRB, 0.11646E+01_JPRB, 0.11265E+01_JPRB, 0.10942E+01_JPRB, 0.10751E+01_JPRB, 0.10555E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.10491E+01_JPRB, 0.10377E+01_JPRB, 0.10091E+01_JPRB, 0.98426E+00_JPRB, 0.97704E+00_JPRB, 0.97957E+00_JPRB ,&
&  0.97175E+00_JPRB, 0.95988E+00_JPRB, 0.96191E+00_JPRB, 0.94378E+00_JPRB, 0.92176E+00_JPRB, 0.90755E+00_JPRB, 0.90419E+00_JPRB ,&
&  0.91078E+00_JPRB, 0.92254E+00_JPRB, 0.95230E+00_JPRB, 0.98327E+00_JPRB, 0.99119E+00_JPRB, 0.99462E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.17101E+01_JPRB, 0.35817E+01_JPRB, 0.54380E+01_JPRB, 0.50835E+01_JPRB, 0.43558E+01_JPRB, 0.36516E+01_JPRB, 0.32715E+01_JPRB ,&
&  0.27047E+01_JPRB, 0.23520E+01_JPRB, 0.20006E+01_JPRB, 0.17286E+01_JPRB, 0.15430E+01_JPRB, 0.14289E+01_JPRB, 0.13603E+01_JPRB ,&
&  0.13057E+01_JPRB, 0.12759E+01_JPRB, 0.12670E+01_JPRB, 0.12593E+01_JPRB, 0.12483E+01_JPRB, 0.12297E+01_JPRB, 0.12386E+01_JPRB ,&
&  0.12953E+01_JPRB, 0.13843E+01_JPRB, 0.15139E+01_JPRB, 0.19213E+01_JPRB, 0.33875E+01_JPRB, 0.42355E+01_JPRB, 0.65311E+01_JPRB ,&
&  0.88834E+01_JPRB, 0.73538E+01_JPRB, 0.55861E+01_JPRB, 0.44152E+01_JPRB, 0.37030E+01_JPRB, 0.32501E+01_JPRB, 0.29777E+01_JPRB ,&
&  0.27025E+01_JPRB, 0.25248E+01_JPRB, 0.23557E+01_JPRB, 0.21895E+01_JPRB, 0.20720E+01_JPRB, 0.19163E+01_JPRB, 0.17734E+01_JPRB ,&
&  0.17128E+01_JPRB, 0.16408E+01_JPRB, 0.15626E+01_JPRB, 0.14752E+01_JPRB, 0.14397E+01_JPRB, 0.14182E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.13885E+01_JPRB, 0.13748E+01_JPRB, 0.13680E+01_JPRB, 0.14030E+01_JPRB, 0.14371E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.13807E+01_JPRB, 0.14016E+01_JPRB, 0.13542E+01_JPRB, 0.13092E+01_JPRB, 0.13709E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.11478E+01_JPRB, 0.11169E+01_JPRB, 0.11229E+01_JPRB, 0.11902E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.11910E+01_JPRB, 0.11848E+01_JPRB, 0.11628E+01_JPRB, 0.11406E+01_JPRB, 0.11276E+01_JPRB, 0.11069E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.10676E+01_JPRB, 0.10572E+01_JPRB, 0.10415E+01_JPRB, 0.10196E+01_JPRB, 0.99915E+00_JPRB, 0.98958E+00_JPRB ,&
&  0.98646E+00_JPRB, 0.97543E+00_JPRB, 0.97813E+00_JPRB, 0.97838E+00_JPRB, 0.97457E+00_JPRB, 0.96324E+00_JPRB, 0.95220E+00_JPRB ,&
&  0.94441E+00_JPRB, 0.92923E+00_JPRB, 0.92673E+00_JPRB, 0.95201E+00_JPRB, 0.97494E+00_JPRB, 0.98362E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.27625E+01_JPRB, 0.39482E+01_JPRB, 0.42306E+01_JPRB, 0.39544E+01_JPRB, 0.34347E+01_JPRB, 0.29315E+01_JPRB ,&
&  0.24889E+01_JPRB, 0.21446E+01_JPRB, 0.19198E+01_JPRB, 0.17440E+01_JPRB, 0.15889E+01_JPRB, 0.14612E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.12967E+01_JPRB, 0.12561E+01_JPRB, 0.12291E+01_JPRB, 0.12134E+01_JPRB, 0.12012E+01_JPRB, 0.11912E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.12866E+01_JPRB, 0.13604E+01_JPRB, 0.14904E+01_JPRB, 0.21377E+01_JPRB, 0.32668E+01_JPRB, 0.47655E+01_JPRB ,&
&  0.52198E+01_JPRB, 0.53031E+01_JPRB, 0.48009E+01_JPRB, 0.40774E+01_JPRB, 0.34872E+01_JPRB, 0.30622E+01_JPRB, 0.27903E+01_JPRB ,&
&  0.25326E+01_JPRB, 0.23436E+01_JPRB, 0.22026E+01_JPRB, 0.20274E+01_JPRB, 0.18582E+01_JPRB, 0.17536E+01_JPRB, 0.17454E+01_JPRB ,&
&  0.17601E+01_JPRB, 0.17102E+01_JPRB, 0.16177E+01_JPRB, 0.15057E+01_JPRB, 0.13984E+01_JPRB, 0.13492E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.13151E+01_JPRB, 0.13186E+01_JPRB, 0.13403E+01_JPRB, 0.13651E+01_JPRB, 0.13806E+01_JPRB, 0.13856E+01_JPRB, 0.13820E+01_JPRB ,&
&  0.13935E+01_JPRB, 0.13887E+01_JPRB, 0.13735E+01_JPRB, 0.13708E+01_JPRB, 0.13389E+01_JPRB, 0.13929E+01_JPRB, 0.13258E+01_JPRB ,&
&  0.12365E+01_JPRB, 0.11745E+01_JPRB, 0.11420E+01_JPRB, 0.11244E+01_JPRB, 0.11703E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11922E+01_JPRB, 0.11834E+01_JPRB, 0.11641E+01_JPRB, 0.11473E+01_JPRB, 0.11242E+01_JPRB, 0.11026E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.10550E+01_JPRB, 0.10447E+01_JPRB, 0.10348E+01_JPRB, 0.10287E+01_JPRB, 0.10159E+01_JPRB, 0.99714E+00_JPRB ,&
&  0.97813E+00_JPRB, 0.96691E+00_JPRB, 0.96237E+00_JPRB, 0.96510E+00_JPRB, 0.97432E+00_JPRB, 0.98083E+00_JPRB, 0.97617E+00_JPRB ,&
&  0.96620E+00_JPRB, 0.94963E+00_JPRB, 0.93084E+00_JPRB, 0.94888E+00_JPRB, 0.96978E+00_JPRB, 0.97858E+00_JPRB, 0.97866E+00_JPRB ,&
&  0.11733E+01_JPRB, 0.18246E+01_JPRB, 0.26053E+01_JPRB, 0.28231E+01_JPRB, 0.28197E+01_JPRB, 0.28159E+01_JPRB, 0.25585E+01_JPRB ,&
&  0.22842E+01_JPRB, 0.19979E+01_JPRB, 0.17469E+01_JPRB, 0.16057E+01_JPRB, 0.15144E+01_JPRB, 0.14227E+01_JPRB, 0.13484E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.12200E+01_JPRB, 0.11888E+01_JPRB, 0.11760E+01_JPRB, 0.11705E+01_JPRB, 0.11705E+01_JPRB, 0.11769E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.11999E+01_JPRB, 0.12359E+01_JPRB, 0.12955E+01_JPRB, 0.14385E+01_JPRB, 0.18988E+01_JPRB, 0.26292E+01_JPRB ,&
&  0.31297E+01_JPRB, 0.35092E+01_JPRB, 0.37591E+01_JPRB, 0.37192E+01_JPRB, 0.33768E+01_JPRB, 0.29763E+01_JPRB, 0.26769E+01_JPRB ,&
&  0.24293E+01_JPRB, 0.21864E+01_JPRB, 0.19800E+01_JPRB, 0.18061E+01_JPRB, 0.16551E+01_JPRB, 0.15897E+01_JPRB, 0.16403E+01_JPRB ,&
&  0.16694E+01_JPRB, 0.16402E+01_JPRB, 0.15588E+01_JPRB, 0.14632E+01_JPRB, 0.13554E+01_JPRB, 0.12914E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.12747E+01_JPRB, 0.12904E+01_JPRB, 0.13133E+01_JPRB, 0.13341E+01_JPRB, 0.13460E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.13547E+01_JPRB, 0.13577E+01_JPRB, 0.12963E+01_JPRB, 0.12883E+01_JPRB, 0.13772E+01_JPRB, 0.13541E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.11921E+01_JPRB, 0.11404E+01_JPRB, 0.11044E+01_JPRB, 0.11498E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.10444E+01_JPRB, 0.10685E+01_JPRB, 0.10766E+01_JPRB, 0.10766E+01_JPRB, 0.10639E+01_JPRB, 0.10432E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.10031E+01_JPRB, 0.99574E+00_JPRB, 0.99178E+00_JPRB, 0.99357E+00_JPRB, 0.99867E+00_JPRB, 0.99573E+00_JPRB ,&
&  0.98306E+00_JPRB, 0.96572E+00_JPRB, 0.95379E+00_JPRB, 0.95181E+00_JPRB, 0.96071E+00_JPRB, 0.97119E+00_JPRB, 0.97785E+00_JPRB ,&
&  0.97989E+00_JPRB, 0.97167E+00_JPRB, 0.96630E+00_JPRB, 0.97797E+00_JPRB, 0.99024E+00_JPRB, 0.98507E+00_JPRB, 0.97119E+00_JPRB ,&
&  0.10542E+01_JPRB, 0.12305E+01_JPRB, 0.15263E+01_JPRB, 0.17840E+01_JPRB, 0.20121E+01_JPRB, 0.20943E+01_JPRB, 0.20289E+01_JPRB ,&
&  0.18847E+01_JPRB, 0.17537E+01_JPRB, 0.16017E+01_JPRB, 0.14645E+01_JPRB, 0.13636E+01_JPRB, 0.13077E+01_JPRB, 0.12729E+01_JPRB ,&
&  0.12251E+01_JPRB, 0.11765E+01_JPRB, 0.11359E+01_JPRB, 0.11124E+01_JPRB, 0.11031E+01_JPRB, 0.10947E+01_JPRB, 0.10859E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.10704E+01_JPRB, 0.10881E+01_JPRB, 0.11280E+01_JPRB, 0.11929E+01_JPRB, 0.13634E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.19956E+01_JPRB, 0.23194E+01_JPRB, 0.26381E+01_JPRB, 0.28852E+01_JPRB, 0.28821E+01_JPRB, 0.27253E+01_JPRB, 0.25096E+01_JPRB ,&
&  0.22591E+01_JPRB, 0.19602E+01_JPRB, 0.16884E+01_JPRB, 0.15295E+01_JPRB, 0.14754E+01_JPRB, 0.14861E+01_JPRB, 0.15391E+01_JPRB ,&
&  0.15582E+01_JPRB, 0.15264E+01_JPRB, 0.14384E+01_JPRB, 0.13495E+01_JPRB, 0.12821E+01_JPRB, 0.12282E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.12037E+01_JPRB, 0.12185E+01_JPRB, 0.12358E+01_JPRB, 0.12600E+01_JPRB, 0.12758E+01_JPRB, 0.12760E+01_JPRB ,&
&  0.12704E+01_JPRB, 0.12614E+01_JPRB, 0.12522E+01_JPRB, 0.12707E+01_JPRB, 0.12572E+01_JPRB, 0.12469E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.11500E+01_JPRB, 0.10969E+01_JPRB, 0.10629E+01_JPRB, 0.98972E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.97385E+00_JPRB, 0.99553E+00_JPRB, 0.10041E+01_JPRB, 0.10046E+01_JPRB, 0.10017E+01_JPRB, 0.98821E+00_JPRB, 0.97138E+00_JPRB ,&
&  0.95275E+00_JPRB, 0.94555E+00_JPRB, 0.94338E+00_JPRB, 0.94773E+00_JPRB, 0.95276E+00_JPRB, 0.96034E+00_JPRB, 0.97116E+00_JPRB ,&
&  0.98123E+00_JPRB, 0.98468E+00_JPRB, 0.98692E+00_JPRB, 0.98346E+00_JPRB, 0.98045E+00_JPRB, 0.98171E+00_JPRB, 0.98491E+00_JPRB ,&
&  0.99270E+00_JPRB, 0.99406E+00_JPRB, 0.98840E+00_JPRB, 0.99737E+00_JPRB, 0.10147E+01_JPRB, 0.10067E+01_JPRB, 0.99118E+00_JPRB ,&
&  0.10179E+01_JPRB, 0.97467E+00_JPRB, 0.10445E+01_JPRB, 0.12674E+01_JPRB, 0.14888E+01_JPRB, 0.16596E+01_JPRB, 0.16867E+01_JPRB ,&
&  0.16253E+01_JPRB, 0.15254E+01_JPRB, 0.14311E+01_JPRB, 0.13372E+01_JPRB, 0.12690E+01_JPRB, 0.12085E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.11107E+01_JPRB, 0.10706E+01_JPRB, 0.10346E+01_JPRB, 0.10042E+01_JPRB, 0.97545E+00_JPRB, 0.95729E+00_JPRB ,&
&  0.95046E+00_JPRB, 0.95802E+00_JPRB, 0.97449E+00_JPRB, 0.10046E+01_JPRB, 0.10592E+01_JPRB, 0.11414E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.14568E+01_JPRB, 0.16476E+01_JPRB, 0.18315E+01_JPRB, 0.19936E+01_JPRB, 0.21292E+01_JPRB, 0.21473E+01_JPRB, 0.20373E+01_JPRB ,&
&  0.18411E+01_JPRB, 0.16718E+01_JPRB, 0.15383E+01_JPRB, 0.14472E+01_JPRB, 0.14252E+01_JPRB, 0.14428E+01_JPRB, 0.14546E+01_JPRB ,&
&  0.14305E+01_JPRB, 0.13687E+01_JPRB, 0.12879E+01_JPRB, 0.12128E+01_JPRB, 0.11654E+01_JPRB, 0.11331E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.10746E+01_JPRB, 0.10790E+01_JPRB, 0.10986E+01_JPRB, 0.11273E+01_JPRB, 0.11574E+01_JPRB, 0.11706E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.11780E+01_JPRB, 0.11868E+01_JPRB, 0.11325E+01_JPRB, 0.11697E+01_JPRB, 0.11745E+01_JPRB, 0.11697E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.11131E+01_JPRB, 0.10653E+01_JPRB, 0.89912E+00_JPRB, 0.95080E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.94078E+00_JPRB, 0.94493E+00_JPRB, 0.94833E+00_JPRB, 0.94367E+00_JPRB, 0.93923E+00_JPRB, 0.93143E+00_JPRB, 0.91531E+00_JPRB ,&
&  0.90010E+00_JPRB, 0.89477E+00_JPRB, 0.89564E+00_JPRB, 0.89756E+00_JPRB, 0.90083E+00_JPRB, 0.90684E+00_JPRB, 0.91769E+00_JPRB ,&
&  0.93074E+00_JPRB, 0.94398E+00_JPRB, 0.95942E+00_JPRB, 0.97301E+00_JPRB, 0.97434E+00_JPRB, 0.96903E+00_JPRB, 0.96701E+00_JPRB ,&
&  0.97130E+00_JPRB, 0.96887E+00_JPRB, 0.95990E+00_JPRB, 0.96073E+00_JPRB, 0.97260E+00_JPRB, 0.96698E+00_JPRB, 0.95092E+00_JPRB ,&
&  0.95578E+00_JPRB, 0.84924E+00_JPRB, 0.91260E+00_JPRB, 0.10207E+01_JPRB, 0.11781E+01_JPRB, 0.13083E+01_JPRB, 0.13650E+01_JPRB ,&
&  0.13523E+01_JPRB, 0.12836E+01_JPRB, 0.12210E+01_JPRB, 0.11506E+01_JPRB, 0.11084E+01_JPRB, 0.10819E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.98024E+00_JPRB, 0.93623E+00_JPRB, 0.89506E+00_JPRB, 0.86273E+00_JPRB, 0.84408E+00_JPRB, 0.83497E+00_JPRB ,&
&  0.83146E+00_JPRB, 0.84098E+00_JPRB, 0.84862E+00_JPRB, 0.86500E+00_JPRB, 0.89535E+00_JPRB, 0.94552E+00_JPRB, 0.10186E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.12479E+01_JPRB, 0.13704E+01_JPRB, 0.14573E+01_JPRB, 0.15120E+01_JPRB, 0.15513E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.14847E+01_JPRB, 0.14198E+01_JPRB, 0.14029E+01_JPRB, 0.13921E+01_JPRB, 0.13665E+01_JPRB, 0.13372E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.11641E+01_JPRB, 0.11016E+01_JPRB, 0.10453E+01_JPRB, 0.10085E+01_JPRB, 0.98675E+00_JPRB, 0.95837E+00_JPRB ,&
&  0.94016E+00_JPRB, 0.93630E+00_JPRB, 0.94560E+00_JPRB, 0.96917E+00_JPRB, 0.10017E+01_JPRB, 0.10328E+01_JPRB, 0.10498E+01_JPRB ,&
&  0.10540E+01_JPRB, 0.10578E+01_JPRB, 0.10594E+01_JPRB, 0.10816E+01_JPRB, 0.10850E+01_JPRB, 0.10852E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.10668E+01_JPRB, 0.84803E+00_JPRB, 0.91440E+00_JPRB, 0.92540E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.95019E+00_JPRB, 0.92894E+00_JPRB, 0.91710E+00_JPRB, 0.90992E+00_JPRB, 0.90467E+00_JPRB, 0.89752E+00_JPRB, 0.88363E+00_JPRB ,&
&  0.86907E+00_JPRB, 0.86136E+00_JPRB, 0.85486E+00_JPRB, 0.84690E+00_JPRB, 0.84931E+00_JPRB, 0.85625E+00_JPRB, 0.86815E+00_JPRB ,&
&  0.88066E+00_JPRB, 0.89606E+00_JPRB, 0.91034E+00_JPRB, 0.92776E+00_JPRB, 0.93794E+00_JPRB, 0.94403E+00_JPRB, 0.94616E+00_JPRB ,&
&  0.94715E+00_JPRB, 0.94621E+00_JPRB, 0.93668E+00_JPRB, 0.92444E+00_JPRB, 0.92244E+00_JPRB, 0.92826E+00_JPRB, 0.92446E+00_JPRB ,&
&  0.82812E+00_JPRB, 0.75755E+00_JPRB, 0.83096E+00_JPRB, 0.91762E+00_JPRB, 0.10134E+01_JPRB, 0.10982E+01_JPRB, 0.11385E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.10756E+01_JPRB, 0.10333E+01_JPRB, 0.98485E+00_JPRB, 0.96009E+00_JPRB, 0.94757E+00_JPRB, 0.93385E+00_JPRB ,&
&  0.92195E+00_JPRB, 0.90691E+00_JPRB, 0.88477E+00_JPRB, 0.85001E+00_JPRB, 0.82325E+00_JPRB, 0.80358E+00_JPRB, 0.79657E+00_JPRB ,&
&  0.79588E+00_JPRB, 0.79824E+00_JPRB, 0.80162E+00_JPRB, 0.80574E+00_JPRB, 0.81629E+00_JPRB, 0.83628E+00_JPRB, 0.86491E+00_JPRB ,&
&  0.91088E+00_JPRB, 0.96965E+00_JPRB, 0.10418E+01_JPRB, 0.10931E+01_JPRB, 0.11230E+01_JPRB, 0.11493E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.12007E+01_JPRB, 0.11670E+01_JPRB, 0.11493E+01_JPRB, 0.11417E+01_JPRB, 0.11282E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.99884E+00_JPRB, 0.94112E+00_JPRB, 0.90405E+00_JPRB, 0.89197E+00_JPRB, 0.88693E+00_JPRB, 0.87806E+00_JPRB ,&
&  0.87001E+00_JPRB, 0.86819E+00_JPRB, 0.87432E+00_JPRB, 0.88734E+00_JPRB, 0.90503E+00_JPRB, 0.92606E+00_JPRB, 0.94247E+00_JPRB ,&
&  0.95659E+00_JPRB, 0.96903E+00_JPRB, 0.97780E+00_JPRB, 0.99318E+00_JPRB, 0.10092E+01_JPRB, 0.10160E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.99732E+00_JPRB, 0.98541E+00_JPRB, 0.97862E+00_JPRB, 0.96804E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.90801E+00_JPRB, 0.88772E+00_JPRB, 0.87562E+00_JPRB, 0.87027E+00_JPRB, 0.86069E+00_JPRB, 0.84094E+00_JPRB, 0.82459E+00_JPRB ,&
&  0.80902E+00_JPRB, 0.79973E+00_JPRB, 0.79522E+00_JPRB, 0.79415E+00_JPRB, 0.80049E+00_JPRB, 0.80697E+00_JPRB, 0.81509E+00_JPRB ,&
&  0.82762E+00_JPRB, 0.84708E+00_JPRB, 0.86310E+00_JPRB, 0.87677E+00_JPRB, 0.88819E+00_JPRB, 0.89904E+00_JPRB, 0.90042E+00_JPRB ,&
&  0.90122E+00_JPRB, 0.90731E+00_JPRB, 0.90710E+00_JPRB, 0.89204E+00_JPRB, 0.87723E+00_JPRB, 0.88256E+00_JPRB, 0.88172E+00_JPRB ,&
&  0.87377E+00_JPRB, 0.76117E+00_JPRB, 0.82721E+00_JPRB, 0.86555E+00_JPRB, 0.91033E+00_JPRB, 0.95841E+00_JPRB, 0.98438E+00_JPRB ,&
&  0.97754E+00_JPRB, 0.94916E+00_JPRB, 0.91924E+00_JPRB, 0.88632E+00_JPRB, 0.87606E+00_JPRB, 0.86672E+00_JPRB, 0.85341E+00_JPRB ,&
&  0.84432E+00_JPRB, 0.83580E+00_JPRB, 0.82364E+00_JPRB, 0.80796E+00_JPRB, 0.78891E+00_JPRB, 0.77438E+00_JPRB, 0.76121E+00_JPRB ,&
&  0.75569E+00_JPRB, 0.74600E+00_JPRB, 0.73800E+00_JPRB, 0.73875E+00_JPRB, 0.75179E+00_JPRB, 0.76079E+00_JPRB, 0.76742E+00_JPRB ,&
&  0.77859E+00_JPRB, 0.79631E+00_JPRB, 0.82231E+00_JPRB, 0.85331E+00_JPRB, 0.87869E+00_JPRB, 0.89812E+00_JPRB, 0.92359E+00_JPRB ,&
&  0.96099E+00_JPRB, 0.98616E+00_JPRB, 0.98635E+00_JPRB, 0.96597E+00_JPRB, 0.94880E+00_JPRB, 0.94003E+00_JPRB, 0.91843E+00_JPRB ,&
&  0.88416E+00_JPRB, 0.85570E+00_JPRB, 0.84020E+00_JPRB, 0.83139E+00_JPRB, 0.82309E+00_JPRB, 0.81543E+00_JPRB, 0.81425E+00_JPRB ,&
&  0.81529E+00_JPRB, 0.81457E+00_JPRB, 0.81331E+00_JPRB, 0.81630E+00_JPRB, 0.82295E+00_JPRB, 0.83157E+00_JPRB, 0.84309E+00_JPRB ,&
&  0.85963E+00_JPRB, 0.87427E+00_JPRB, 0.88137E+00_JPRB, 0.89081E+00_JPRB, 0.90285E+00_JPRB, 0.90792E+00_JPRB, 0.92170E+00_JPRB ,&
&  0.92139E+00_JPRB, 0.91724E+00_JPRB, 0.91554E+00_JPRB, 0.92038E+00_JPRB, 0.92140E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.86379E+00_JPRB, 0.84268E+00_JPRB, 0.82973E+00_JPRB, 0.81755E+00_JPRB, 0.80911E+00_JPRB, 0.79081E+00_JPRB, 0.78019E+00_JPRB ,&
&  0.77285E+00_JPRB, 0.76780E+00_JPRB, 0.76576E+00_JPRB, 0.76550E+00_JPRB, 0.77612E+00_JPRB, 0.78340E+00_JPRB, 0.78672E+00_JPRB ,&
&  0.79515E+00_JPRB, 0.81381E+00_JPRB, 0.83129E+00_JPRB, 0.84548E+00_JPRB, 0.85595E+00_JPRB, 0.86272E+00_JPRB, 0.86118E+00_JPRB ,&
&  0.86349E+00_JPRB, 0.87028E+00_JPRB, 0.87308E+00_JPRB, 0.86097E+00_JPRB, 0.84606E+00_JPRB, 0.83829E+00_JPRB, 0.84013E+00_JPRB ,&
&  0.82931E+00_JPRB, 0.79921E+00_JPRB, 0.81907E+00_JPRB, 0.83354E+00_JPRB, 0.85135E+00_JPRB, 0.86845E+00_JPRB, 0.88593E+00_JPRB ,&
&  0.89234E+00_JPRB, 0.87268E+00_JPRB, 0.84441E+00_JPRB, 0.82444E+00_JPRB, 0.82841E+00_JPRB, 0.82450E+00_JPRB, 0.81103E+00_JPRB ,&
&  0.80183E+00_JPRB, 0.79349E+00_JPRB, 0.78293E+00_JPRB, 0.76794E+00_JPRB, 0.75472E+00_JPRB, 0.74483E+00_JPRB, 0.73723E+00_JPRB ,&
&  0.72335E+00_JPRB, 0.71204E+00_JPRB, 0.70600E+00_JPRB, 0.69965E+00_JPRB, 0.69512E+00_JPRB, 0.70080E+00_JPRB, 0.70739E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.71978E+00_JPRB, 0.73418E+00_JPRB, 0.75526E+00_JPRB, 0.77483E+00_JPRB, 0.79542E+00_JPRB, 0.81786E+00_JPRB ,&
&  0.84143E+00_JPRB, 0.86296E+00_JPRB, 0.87173E+00_JPRB, 0.87613E+00_JPRB, 0.86808E+00_JPRB, 0.85448E+00_JPRB, 0.82894E+00_JPRB ,&
&  0.80027E+00_JPRB, 0.78960E+00_JPRB, 0.78604E+00_JPRB, 0.78826E+00_JPRB, 0.78404E+00_JPRB, 0.78040E+00_JPRB, 0.77928E+00_JPRB ,&
&  0.77588E+00_JPRB, 0.77228E+00_JPRB, 0.76680E+00_JPRB, 0.76184E+00_JPRB, 0.76511E+00_JPRB, 0.77099E+00_JPRB, 0.78452E+00_JPRB ,&
&  0.80311E+00_JPRB, 0.82292E+00_JPRB, 0.84238E+00_JPRB, 0.85440E+00_JPRB, 0.85746E+00_JPRB, 0.85905E+00_JPRB, 0.86174E+00_JPRB ,&
&  0.86237E+00_JPRB, 0.86002E+00_JPRB, 0.86567E+00_JPRB, 0.86997E+00_JPRB, 0.87455E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.80118E+00_JPRB, 0.77573E+00_JPRB, 0.75724E+00_JPRB, 0.75044E+00_JPRB, 0.74674E+00_JPRB, 0.73410E+00_JPRB, 0.72801E+00_JPRB ,&
&  0.73330E+00_JPRB, 0.73461E+00_JPRB, 0.74120E+00_JPRB, 0.74039E+00_JPRB, 0.73959E+00_JPRB, 0.74233E+00_JPRB, 0.73476E+00_JPRB ,&
&  0.73498E+00_JPRB, 0.74453E+00_JPRB, 0.75597E+00_JPRB, 0.77138E+00_JPRB, 0.78432E+00_JPRB, 0.79398E+00_JPRB, 0.80174E+00_JPRB ,&
&  0.80453E+00_JPRB, 0.81063E+00_JPRB, 0.81378E+00_JPRB, 0.80869E+00_JPRB, 0.79615E+00_JPRB, 0.78603E+00_JPRB, 0.78372E+00_JPRB ,&
&  0.78162E+00_JPRB, 0.78317E+00_JPRB, 0.73941E+00_JPRB, 0.76575E+00_JPRB, 0.77425E+00_JPRB, 0.78207E+00_JPRB, 0.78969E+00_JPRB ,&
&  0.78918E+00_JPRB, 0.77729E+00_JPRB, 0.76981E+00_JPRB, 0.75413E+00_JPRB, 0.75170E+00_JPRB, 0.75758E+00_JPRB, 0.75285E+00_JPRB ,&
&  0.74395E+00_JPRB, 0.72918E+00_JPRB, 0.71219E+00_JPRB, 0.69999E+00_JPRB, 0.69651E+00_JPRB, 0.69278E+00_JPRB, 0.68238E+00_JPRB ,&
&  0.67594E+00_JPRB, 0.67511E+00_JPRB, 0.67342E+00_JPRB, 0.66694E+00_JPRB, 0.65133E+00_JPRB, 0.63776E+00_JPRB, 0.63670E+00_JPRB ,&
&  0.64327E+00_JPRB, 0.65432E+00_JPRB, 0.67333E+00_JPRB, 0.69654E+00_JPRB, 0.71984E+00_JPRB, 0.74109E+00_JPRB, 0.75961E+00_JPRB ,&
&  0.76712E+00_JPRB, 0.77023E+00_JPRB, 0.77406E+00_JPRB, 0.78074E+00_JPRB, 0.78272E+00_JPRB, 0.77389E+00_JPRB, 0.75006E+00_JPRB ,&
&  0.72479E+00_JPRB, 0.71654E+00_JPRB, 0.71920E+00_JPRB, 0.72737E+00_JPRB, 0.73106E+00_JPRB, 0.73054E+00_JPRB, 0.72465E+00_JPRB ,&
&  0.72145E+00_JPRB, 0.71340E+00_JPRB, 0.70397E+00_JPRB, 0.69900E+00_JPRB, 0.69814E+00_JPRB, 0.70392E+00_JPRB, 0.72181E+00_JPRB ,&
&  0.74394E+00_JPRB, 0.76848E+00_JPRB, 0.78921E+00_JPRB, 0.79598E+00_JPRB, 0.79940E+00_JPRB, 0.79713E+00_JPRB, 0.78687E+00_JPRB ,&
&  0.79070E+00_JPRB, 0.80977E+00_JPRB, 0.81933E+00_JPRB, 0.82044E+00_JPRB, 0.81730E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.74331E+00_JPRB, 0.71456E+00_JPRB, 0.70324E+00_JPRB, 0.70548E+00_JPRB, 0.70882E+00_JPRB, 0.70407E+00_JPRB, 0.71083E+00_JPRB ,&
&  0.72160E+00_JPRB, 0.72992E+00_JPRB, 0.73445E+00_JPRB, 0.73033E+00_JPRB, 0.72755E+00_JPRB, 0.72424E+00_JPRB, 0.70888E+00_JPRB ,&
&  0.70352E+00_JPRB, 0.70168E+00_JPRB, 0.70628E+00_JPRB, 0.71432E+00_JPRB, 0.72336E+00_JPRB, 0.73677E+00_JPRB, 0.74669E+00_JPRB ,&
&  0.75297E+00_JPRB, 0.75670E+00_JPRB, 0.76079E+00_JPRB, 0.76074E+00_JPRB, 0.75661E+00_JPRB, 0.74643E+00_JPRB, 0.73852E+00_JPRB ,&
&  0.73361E+00_JPRB, 0.72376E+00_JPRB, 0.71281E+00_JPRB, 0.71162E+00_JPRB, 0.71499E+00_JPRB, 0.71860E+00_JPRB, 0.72027E+00_JPRB ,&
&  0.71090E+00_JPRB, 0.69685E+00_JPRB, 0.69712E+00_JPRB, 0.69602E+00_JPRB, 0.69992E+00_JPRB, 0.71163E+00_JPRB, 0.71423E+00_JPRB ,&
&  0.70884E+00_JPRB, 0.69213E+00_JPRB, 0.67077E+00_JPRB, 0.66595E+00_JPRB, 0.66716E+00_JPRB, 0.65751E+00_JPRB, 0.64870E+00_JPRB ,&
&  0.64730E+00_JPRB, 0.64460E+00_JPRB, 0.63877E+00_JPRB, 0.63053E+00_JPRB, 0.61714E+00_JPRB, 0.60002E+00_JPRB, 0.59192E+00_JPRB ,&
&  0.59324E+00_JPRB, 0.61270E+00_JPRB, 0.64568E+00_JPRB, 0.68121E+00_JPRB, 0.71301E+00_JPRB, 0.72745E+00_JPRB, 0.72576E+00_JPRB ,&
&  0.71076E+00_JPRB, 0.70501E+00_JPRB, 0.70917E+00_JPRB, 0.71426E+00_JPRB, 0.71650E+00_JPRB, 0.70582E+00_JPRB, 0.68648E+00_JPRB ,&
&  0.67184E+00_JPRB, 0.66192E+00_JPRB, 0.66108E+00_JPRB, 0.66606E+00_JPRB, 0.67075E+00_JPRB, 0.67197E+00_JPRB, 0.66669E+00_JPRB ,&
&  0.66187E+00_JPRB, 0.65609E+00_JPRB, 0.65174E+00_JPRB, 0.65308E+00_JPRB, 0.65642E+00_JPRB, 0.66057E+00_JPRB, 0.67500E+00_JPRB ,&
&  0.69620E+00_JPRB, 0.71662E+00_JPRB, 0.72091E+00_JPRB, 0.71612E+00_JPRB, 0.71491E+00_JPRB, 0.71143E+00_JPRB, 0.70810E+00_JPRB ,&
&  0.71779E+00_JPRB, 0.74033E+00_JPRB, 0.75638E+00_JPRB, 0.76466E+00_JPRB, 0.76420E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.66187E+00_JPRB, 0.64518E+00_JPRB, 0.64634E+00_JPRB, 0.65453E+00_JPRB, 0.65909E+00_JPRB, 0.66694E+00_JPRB, 0.68626E+00_JPRB ,&
&  0.69690E+00_JPRB, 0.70347E+00_JPRB, 0.70588E+00_JPRB, 0.70074E+00_JPRB, 0.69989E+00_JPRB, 0.69261E+00_JPRB, 0.67955E+00_JPRB ,&
&  0.67396E+00_JPRB, 0.67281E+00_JPRB, 0.67454E+00_JPRB, 0.67876E+00_JPRB, 0.68158E+00_JPRB, 0.68535E+00_JPRB, 0.68953E+00_JPRB ,&
&  0.69190E+00_JPRB, 0.69204E+00_JPRB, 0.69104E+00_JPRB, 0.69106E+00_JPRB, 0.68976E+00_JPRB, 0.68148E+00_JPRB, 0.67195E+00_JPRB ,&
&  0.66196E+00_JPRB, 0.65307E+00_JPRB, 0.64997E+00_JPRB, 0.65368E+00_JPRB, 0.65814E+00_JPRB, 0.66057E+00_JPRB, 0.65811E+00_JPRB ,&
&  0.65450E+00_JPRB, 0.65476E+00_JPRB, 0.65986E+00_JPRB, 0.66698E+00_JPRB, 0.67498E+00_JPRB, 0.68424E+00_JPRB, 0.68909E+00_JPRB ,&
&  0.68271E+00_JPRB, 0.66365E+00_JPRB, 0.65176E+00_JPRB, 0.65050E+00_JPRB, 0.64464E+00_JPRB, 0.62675E+00_JPRB, 0.61487E+00_JPRB ,&
&  0.60986E+00_JPRB, 0.60061E+00_JPRB, 0.59089E+00_JPRB, 0.57822E+00_JPRB, 0.56689E+00_JPRB, 0.56246E+00_JPRB, 0.56161E+00_JPRB ,&
&  0.56586E+00_JPRB, 0.58810E+00_JPRB, 0.62694E+00_JPRB, 0.66283E+00_JPRB, 0.67735E+00_JPRB, 0.66955E+00_JPRB, 0.65023E+00_JPRB ,&
&  0.63975E+00_JPRB, 0.63433E+00_JPRB, 0.63484E+00_JPRB, 0.63522E+00_JPRB, 0.63267E+00_JPRB, 0.62387E+00_JPRB, 0.61573E+00_JPRB ,&
&  0.60449E+00_JPRB, 0.59257E+00_JPRB, 0.58810E+00_JPRB, 0.58566E+00_JPRB, 0.58302E+00_JPRB, 0.58370E+00_JPRB, 0.58384E+00_JPRB ,&
&  0.58557E+00_JPRB, 0.58993E+00_JPRB, 0.60106E+00_JPRB, 0.60561E+00_JPRB, 0.60424E+00_JPRB, 0.59356E+00_JPRB, 0.59575E+00_JPRB ,&
&  0.60003E+00_JPRB, 0.59844E+00_JPRB, 0.59376E+00_JPRB, 0.59355E+00_JPRB, 0.59361E+00_JPRB, 0.59584E+00_JPRB, 0.60854E+00_JPRB ,&
&  0.63617E+00_JPRB, 0.65961E+00_JPRB, 0.67870E+00_JPRB, 0.68364E+00_JPRB, 0.67933E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.60198E+00_JPRB, 0.60758E+00_JPRB, 0.62142E+00_JPRB, 0.63275E+00_JPRB, 0.63643E+00_JPRB, 0.64642E+00_JPRB, 0.65747E+00_JPRB ,&
&  0.66557E+00_JPRB, 0.66644E+00_JPRB, 0.66235E+00_JPRB, 0.65249E+00_JPRB, 0.64096E+00_JPRB, 0.63137E+00_JPRB, 0.62779E+00_JPRB ,&
&  0.63007E+00_JPRB, 0.63303E+00_JPRB, 0.63854E+00_JPRB, 0.64177E+00_JPRB, 0.64592E+00_JPRB, 0.64529E+00_JPRB, 0.64645E+00_JPRB ,&
&  0.64896E+00_JPRB, 0.65044E+00_JPRB, 0.65102E+00_JPRB, 0.64703E+00_JPRB, 0.64365E+00_JPRB, 0.64052E+00_JPRB, 0.63092E+00_JPRB ,&
&  0.62058E+00_JPRB, 0.61414E+00_JPRB, 0.61795E+00_JPRB, 0.62878E+00_JPRB, 0.63538E+00_JPRB, 0.63164E+00_JPRB, 0.62947E+00_JPRB ,&
&  0.62972E+00_JPRB, 0.63592E+00_JPRB, 0.64595E+00_JPRB, 0.65940E+00_JPRB, 0.66748E+00_JPRB, 0.67782E+00_JPRB, 0.67829E+00_JPRB ,&
&  0.67174E+00_JPRB, 0.66215E+00_JPRB, 0.65883E+00_JPRB, 0.65667E+00_JPRB, 0.62457E+00_JPRB, 0.58945E+00_JPRB, 0.57240E+00_JPRB ,&
&  0.56316E+00_JPRB, 0.55201E+00_JPRB, 0.53605E+00_JPRB, 0.52522E+00_JPRB, 0.52460E+00_JPRB, 0.53014E+00_JPRB, 0.54206E+00_JPRB ,&
&  0.55150E+00_JPRB, 0.56556E+00_JPRB, 0.58271E+00_JPRB, 0.59420E+00_JPRB, 0.59449E+00_JPRB, 0.58129E+00_JPRB, 0.56831E+00_JPRB ,&
&  0.56008E+00_JPRB, 0.55711E+00_JPRB, 0.55503E+00_JPRB, 0.55302E+00_JPRB, 0.55143E+00_JPRB, 0.55089E+00_JPRB, 0.55008E+00_JPRB ,&
&  0.54602E+00_JPRB, 0.53903E+00_JPRB, 0.53062E+00_JPRB, 0.52635E+00_JPRB, 0.52670E+00_JPRB, 0.52799E+00_JPRB, 0.53058E+00_JPRB ,&
&  0.53956E+00_JPRB, 0.55098E+00_JPRB, 0.55950E+00_JPRB, 0.56188E+00_JPRB, 0.55717E+00_JPRB, 0.54798E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.53923E+00_JPRB, 0.53689E+00_JPRB, 0.53357E+00_JPRB, 0.53243E+00_JPRB, 0.53294E+00_JPRB, 0.54175E+00_JPRB, 0.55931E+00_JPRB ,&
&  0.58002E+00_JPRB, 0.60605E+00_JPRB, 0.61736E+00_JPRB, 0.61542E+00_JPRB, 0.60819E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.56923E+00_JPRB, 0.58277E+00_JPRB, 0.59956E+00_JPRB, 0.61101E+00_JPRB, 0.61565E+00_JPRB, 0.62063E+00_JPRB, 0.62267E+00_JPRB ,&
&  0.61632E+00_JPRB, 0.61331E+00_JPRB, 0.60770E+00_JPRB, 0.60014E+00_JPRB, 0.59365E+00_JPRB, 0.59061E+00_JPRB, 0.59133E+00_JPRB ,&
&  0.59356E+00_JPRB, 0.59546E+00_JPRB, 0.59224E+00_JPRB, 0.59085E+00_JPRB, 0.59544E+00_JPRB, 0.60157E+00_JPRB, 0.60576E+00_JPRB ,&
&  0.60797E+00_JPRB, 0.61013E+00_JPRB, 0.61539E+00_JPRB, 0.61861E+00_JPRB, 0.61857E+00_JPRB, 0.61149E+00_JPRB, 0.60588E+00_JPRB ,&
&  0.60024E+00_JPRB, 0.59805E+00_JPRB, 0.60324E+00_JPRB, 0.59592E+00_JPRB, 0.60984E+00_JPRB, 0.61075E+00_JPRB, 0.61528E+00_JPRB ,&
&  0.62274E+00_JPRB, 0.63661E+00_JPRB, 0.64835E+00_JPRB, 0.65767E+00_JPRB, 0.66020E+00_JPRB, 0.66286E+00_JPRB, 0.65239E+00_JPRB ,&
&  0.63834E+00_JPRB, 0.61964E+00_JPRB, 0.60057E+00_JPRB, 0.57872E+00_JPRB, 0.55620E+00_JPRB, 0.53894E+00_JPRB, 0.53114E+00_JPRB ,&
&  0.52157E+00_JPRB, 0.50731E+00_JPRB, 0.49393E+00_JPRB, 0.48903E+00_JPRB, 0.49392E+00_JPRB, 0.50413E+00_JPRB, 0.51961E+00_JPRB ,&
&  0.52971E+00_JPRB, 0.53242E+00_JPRB, 0.52953E+00_JPRB, 0.52595E+00_JPRB, 0.52203E+00_JPRB, 0.51371E+00_JPRB, 0.50862E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.49921E+00_JPRB, 0.49660E+00_JPRB, 0.49778E+00_JPRB, 0.50041E+00_JPRB, 0.50253E+00_JPRB, 0.50194E+00_JPRB ,&
&  0.50006E+00_JPRB, 0.49522E+00_JPRB, 0.48727E+00_JPRB, 0.48408E+00_JPRB, 0.48581E+00_JPRB, 0.49097E+00_JPRB, 0.49843E+00_JPRB ,&
&  0.50078E+00_JPRB, 0.50738E+00_JPRB, 0.51334E+00_JPRB, 0.51491E+00_JPRB, 0.51689E+00_JPRB, 0.51800E+00_JPRB, 0.51744E+00_JPRB ,&
&  0.51508E+00_JPRB, 0.50972E+00_JPRB, 0.50307E+00_JPRB, 0.50139E+00_JPRB, 0.50386E+00_JPRB, 0.51212E+00_JPRB, 0.52224E+00_JPRB ,&
&  0.53185E+00_JPRB, 0.54696E+00_JPRB, 0.55811E+00_JPRB, 0.56041E+00_JPRB, 0.56262E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.54483E+00_JPRB, 0.55718E+00_JPRB, 0.57142E+00_JPRB, 0.58079E+00_JPRB, 0.57931E+00_JPRB, 0.57869E+00_JPRB, 0.58410E+00_JPRB ,&
&  0.57644E+00_JPRB, 0.56828E+00_JPRB, 0.56213E+00_JPRB, 0.55641E+00_JPRB, 0.55502E+00_JPRB, 0.55448E+00_JPRB, 0.55814E+00_JPRB ,&
&  0.56088E+00_JPRB, 0.56030E+00_JPRB, 0.55926E+00_JPRB, 0.56078E+00_JPRB, 0.56830E+00_JPRB, 0.57747E+00_JPRB, 0.58515E+00_JPRB ,&
&  0.58724E+00_JPRB, 0.59159E+00_JPRB, 0.59818E+00_JPRB, 0.60377E+00_JPRB, 0.60641E+00_JPRB, 0.60768E+00_JPRB, 0.60595E+00_JPRB ,&
&  0.60266E+00_JPRB, 0.59805E+00_JPRB, 0.58066E+00_JPRB, 0.59191E+00_JPRB, 0.59651E+00_JPRB, 0.59677E+00_JPRB, 0.60040E+00_JPRB ,&
&  0.60750E+00_JPRB, 0.61695E+00_JPRB, 0.62195E+00_JPRB, 0.62383E+00_JPRB, 0.61884E+00_JPRB, 0.60884E+00_JPRB, 0.58359E+00_JPRB ,&
&  0.56245E+00_JPRB, 0.54085E+00_JPRB, 0.52725E+00_JPRB, 0.51845E+00_JPRB, 0.51105E+00_JPRB, 0.50500E+00_JPRB, 0.50016E+00_JPRB ,&
&  0.49342E+00_JPRB, 0.48751E+00_JPRB, 0.48383E+00_JPRB, 0.48341E+00_JPRB, 0.48702E+00_JPRB, 0.49750E+00_JPRB, 0.51040E+00_JPRB ,&
&  0.51570E+00_JPRB, 0.50682E+00_JPRB, 0.49826E+00_JPRB, 0.48833E+00_JPRB, 0.47078E+00_JPRB, 0.44547E+00_JPRB, 0.42891E+00_JPRB ,&
&  0.42571E+00_JPRB, 0.43833E+00_JPRB, 0.45818E+00_JPRB, 0.46504E+00_JPRB, 0.47152E+00_JPRB, 0.46880E+00_JPRB, 0.46296E+00_JPRB ,&
&  0.45171E+00_JPRB, 0.43002E+00_JPRB, 0.42010E+00_JPRB, 0.41788E+00_JPRB, 0.41657E+00_JPRB, 0.41850E+00_JPRB, 0.42174E+00_JPRB ,&
&  0.42703E+00_JPRB, 0.43334E+00_JPRB, 0.43774E+00_JPRB, 0.43929E+00_JPRB, 0.44183E+00_JPRB, 0.44217E+00_JPRB, 0.44101E+00_JPRB ,&
&  0.43798E+00_JPRB, 0.44614E+00_JPRB, 0.46616E+00_JPRB, 0.49267E+00_JPRB, 0.49956E+00_JPRB, 0.50835E+00_JPRB, 0.51715E+00_JPRB ,&
&  0.52284E+00_JPRB, 0.52918E+00_JPRB, 0.53048E+00_JPRB, 0.53098E+00_JPRB, 0.53585E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.51615E+00_JPRB, 0.52337E+00_JPRB, 0.53252E+00_JPRB, 0.54417E+00_JPRB, 0.54645E+00_JPRB, 0.54786E+00_JPRB, 0.55653E+00_JPRB ,&
&  0.55599E+00_JPRB, 0.55240E+00_JPRB, 0.54935E+00_JPRB, 0.55030E+00_JPRB, 0.55344E+00_JPRB, 0.55682E+00_JPRB, 0.55757E+00_JPRB ,&
&  0.54921E+00_JPRB, 0.54466E+00_JPRB, 0.54826E+00_JPRB, 0.55219E+00_JPRB, 0.55741E+00_JPRB, 0.56399E+00_JPRB, 0.56998E+00_JPRB ,&
&  0.57364E+00_JPRB, 0.57796E+00_JPRB, 0.58145E+00_JPRB, 0.58839E+00_JPRB, 0.59835E+00_JPRB, 0.60540E+00_JPRB, 0.61066E+00_JPRB ,&
&  0.59220E+00_JPRB, 0.54355E+00_JPRB, 0.51405E+00_JPRB, 0.51940E+00_JPRB, 0.56352E+00_JPRB, 0.57477E+00_JPRB, 0.57548E+00_JPRB ,&
&  0.57805E+00_JPRB, 0.58527E+00_JPRB, 0.58501E+00_JPRB, 0.57961E+00_JPRB, 0.56733E+00_JPRB, 0.54747E+00_JPRB, 0.52359E+00_JPRB ,&
&  0.50800E+00_JPRB, 0.49826E+00_JPRB, 0.49131E+00_JPRB, 0.48378E+00_JPRB, 0.47996E+00_JPRB, 0.47500E+00_JPRB, 0.46697E+00_JPRB ,&
&  0.46357E+00_JPRB, 0.46292E+00_JPRB, 0.46443E+00_JPRB, 0.46514E+00_JPRB, 0.46619E+00_JPRB, 0.46611E+00_JPRB, 0.45914E+00_JPRB ,&
&  0.44740E+00_JPRB, 0.43051E+00_JPRB, 0.41475E+00_JPRB, 0.39753E+00_JPRB, 0.38338E+00_JPRB, 0.37537E+00_JPRB, 0.36848E+00_JPRB ,&
&  0.36632E+00_JPRB, 0.37437E+00_JPRB, 0.39529E+00_JPRB, 0.41774E+00_JPRB, 0.44388E+00_JPRB, 0.41674E+00_JPRB, 0.39356E+00_JPRB ,&
&  0.38366E+00_JPRB, 0.37865E+00_JPRB, 0.37586E+00_JPRB, 0.37645E+00_JPRB, 0.37862E+00_JPRB, 0.37919E+00_JPRB, 0.37947E+00_JPRB ,&
&  0.38154E+00_JPRB, 0.38487E+00_JPRB, 0.38693E+00_JPRB, 0.38882E+00_JPRB, 0.39165E+00_JPRB, 0.39479E+00_JPRB, 0.39638E+00_JPRB ,&
&  0.39749E+00_JPRB, 0.40351E+00_JPRB, 0.41451E+00_JPRB, 0.42741E+00_JPRB, 0.43960E+00_JPRB, 0.45223E+00_JPRB, 0.46920E+00_JPRB ,&
&  0.48909E+00_JPRB, 0.51064E+00_JPRB, 0.51182E+00_JPRB, 0.51415E+00_JPRB, 0.51410E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.49843E+00_JPRB, 0.51053E+00_JPRB, 0.51630E+00_JPRB, 0.52216E+00_JPRB, 0.52487E+00_JPRB, 0.52649E+00_JPRB, 0.53464E+00_JPRB ,&
&  0.53962E+00_JPRB, 0.54380E+00_JPRB, 0.54668E+00_JPRB, 0.54919E+00_JPRB, 0.54648E+00_JPRB, 0.54071E+00_JPRB, 0.53998E+00_JPRB ,&
&  0.53915E+00_JPRB, 0.53975E+00_JPRB, 0.53914E+00_JPRB, 0.53926E+00_JPRB, 0.53986E+00_JPRB, 0.54459E+00_JPRB, 0.55197E+00_JPRB ,&
&  0.55373E+00_JPRB, 0.55771E+00_JPRB, 0.57162E+00_JPRB, 0.57936E+00_JPRB, 0.58427E+00_JPRB, 0.58289E+00_JPRB, 0.57600E+00_JPRB ,&
&  0.54490E+00_JPRB, 0.51575E+00_JPRB, 0.49054E+00_JPRB, 0.49150E+00_JPRB, 0.54076E+00_JPRB, 0.54559E+00_JPRB, 0.54605E+00_JPRB ,&
&  0.54464E+00_JPRB, 0.54838E+00_JPRB, 0.54862E+00_JPRB, 0.53363E+00_JPRB, 0.52282E+00_JPRB, 0.50878E+00_JPRB, 0.49594E+00_JPRB ,&
&  0.48366E+00_JPRB, 0.47644E+00_JPRB, 0.46216E+00_JPRB, 0.43316E+00_JPRB, 0.41724E+00_JPRB, 0.40907E+00_JPRB, 0.39510E+00_JPRB ,&
&  0.38417E+00_JPRB, 0.38209E+00_JPRB, 0.38155E+00_JPRB, 0.37935E+00_JPRB, 0.38305E+00_JPRB, 0.38531E+00_JPRB, 0.38270E+00_JPRB ,&
&  0.37740E+00_JPRB, 0.37311E+00_JPRB, 0.36850E+00_JPRB, 0.36272E+00_JPRB, 0.35988E+00_JPRB, 0.35950E+00_JPRB, 0.36079E+00_JPRB ,&
&  0.36635E+00_JPRB, 0.37375E+00_JPRB, 0.38941E+00_JPRB, 0.39995E+00_JPRB, 0.39262E+00_JPRB, 0.37951E+00_JPRB, 0.36955E+00_JPRB ,&
&  0.36207E+00_JPRB, 0.35919E+00_JPRB, 0.35692E+00_JPRB, 0.35437E+00_JPRB, 0.35514E+00_JPRB, 0.35773E+00_JPRB, 0.35914E+00_JPRB ,&
&  0.35918E+00_JPRB, 0.36167E+00_JPRB, 0.36333E+00_JPRB, 0.36505E+00_JPRB, 0.36649E+00_JPRB, 0.36880E+00_JPRB, 0.37192E+00_JPRB ,&
&  0.37448E+00_JPRB, 0.38036E+00_JPRB, 0.38654E+00_JPRB, 0.39375E+00_JPRB, 0.39839E+00_JPRB, 0.40196E+00_JPRB, 0.41007E+00_JPRB ,&
&  0.41619E+00_JPRB, 0.43189E+00_JPRB, 0.46494E+00_JPRB, 0.49782E+00_JPRB, 0.48696E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.48570E+00_JPRB, 0.49485E+00_JPRB, 0.50270E+00_JPRB, 0.50894E+00_JPRB, 0.51334E+00_JPRB, 0.51460E+00_JPRB, 0.51784E+00_JPRB ,&
&  0.52599E+00_JPRB, 0.53099E+00_JPRB, 0.53543E+00_JPRB, 0.53416E+00_JPRB, 0.53362E+00_JPRB, 0.51635E+00_JPRB, 0.51353E+00_JPRB ,&
&  0.50698E+00_JPRB, 0.50235E+00_JPRB, 0.49537E+00_JPRB, 0.49495E+00_JPRB, 0.49452E+00_JPRB, 0.49702E+00_JPRB, 0.49804E+00_JPRB ,&
&  0.49951E+00_JPRB, 0.49931E+00_JPRB, 0.49678E+00_JPRB, 0.49532E+00_JPRB, 0.49628E+00_JPRB, 0.49697E+00_JPRB, 0.49346E+00_JPRB ,&
&  0.48860E+00_JPRB, 0.48705E+00_JPRB, 0.48979E+00_JPRB, 0.49540E+00_JPRB, 0.53337E+00_JPRB, 0.52661E+00_JPRB, 0.52537E+00_JPRB ,&
&  0.52197E+00_JPRB, 0.51746E+00_JPRB, 0.51642E+00_JPRB, 0.50912E+00_JPRB, 0.49895E+00_JPRB, 0.47401E+00_JPRB, 0.46039E+00_JPRB ,&
&  0.42692E+00_JPRB, 0.40932E+00_JPRB, 0.39407E+00_JPRB, 0.38426E+00_JPRB, 0.37259E+00_JPRB, 0.36573E+00_JPRB, 0.35997E+00_JPRB ,&
&  0.35553E+00_JPRB, 0.35179E+00_JPRB, 0.34845E+00_JPRB, 0.34942E+00_JPRB, 0.35157E+00_JPRB, 0.35238E+00_JPRB, 0.35334E+00_JPRB ,&
&  0.35388E+00_JPRB, 0.35445E+00_JPRB, 0.35507E+00_JPRB, 0.35680E+00_JPRB, 0.35911E+00_JPRB, 0.36220E+00_JPRB, 0.36618E+00_JPRB ,&
&  0.37247E+00_JPRB, 0.37712E+00_JPRB, 0.37842E+00_JPRB, 0.38174E+00_JPRB, 0.38136E+00_JPRB, 0.37402E+00_JPRB, 0.36632E+00_JPRB ,&
&  0.35801E+00_JPRB, 0.35370E+00_JPRB, 0.34934E+00_JPRB, 0.34615E+00_JPRB, 0.34533E+00_JPRB, 0.34512E+00_JPRB, 0.34736E+00_JPRB ,&
&  0.34987E+00_JPRB, 0.35226E+00_JPRB, 0.35399E+00_JPRB, 0.35578E+00_JPRB, 0.35807E+00_JPRB, 0.35962E+00_JPRB, 0.36166E+00_JPRB ,&
&  0.36435E+00_JPRB, 0.36794E+00_JPRB, 0.37326E+00_JPRB, 0.37955E+00_JPRB, 0.38840E+00_JPRB, 0.40056E+00_JPRB, 0.42064E+00_JPRB ,&
&  0.44505E+00_JPRB, 0.49725E+00_JPRB, 0.49878E+00_JPRB, 0.47229E+00_JPRB, 0.47926E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.46155E+00_JPRB, 0.46937E+00_JPRB, 0.47646E+00_JPRB, 0.48422E+00_JPRB, 0.49394E+00_JPRB, 0.49842E+00_JPRB, 0.50041E+00_JPRB ,&
&  0.49588E+00_JPRB, 0.49222E+00_JPRB, 0.48208E+00_JPRB, 0.47412E+00_JPRB, 0.46515E+00_JPRB, 0.45827E+00_JPRB, 0.45203E+00_JPRB ,&
&  0.44745E+00_JPRB, 0.44401E+00_JPRB, 0.43798E+00_JPRB, 0.43422E+00_JPRB, 0.43047E+00_JPRB, 0.43101E+00_JPRB, 0.43209E+00_JPRB ,&
&  0.43242E+00_JPRB, 0.43355E+00_JPRB, 0.43184E+00_JPRB, 0.43702E+00_JPRB, 0.44657E+00_JPRB, 0.45767E+00_JPRB, 0.47220E+00_JPRB ,&
&  0.49240E+00_JPRB, 0.50282E+00_JPRB, 0.51826E+00_JPRB, 0.51770E+00_JPRB, 0.51598E+00_JPRB, 0.51239E+00_JPRB, 0.50923E+00_JPRB ,&
&  0.50430E+00_JPRB, 0.49712E+00_JPRB, 0.49147E+00_JPRB, 0.48395E+00_JPRB, 0.45247E+00_JPRB, 0.42902E+00_JPRB, 0.41073E+00_JPRB ,&
&  0.39604E+00_JPRB, 0.38213E+00_JPRB, 0.36927E+00_JPRB, 0.36302E+00_JPRB, 0.35713E+00_JPRB, 0.35363E+00_JPRB, 0.34991E+00_JPRB ,&
&  0.34588E+00_JPRB, 0.34500E+00_JPRB, 0.34314E+00_JPRB, 0.34236E+00_JPRB, 0.34375E+00_JPRB, 0.34343E+00_JPRB, 0.34423E+00_JPRB ,&
&  0.34510E+00_JPRB, 0.34682E+00_JPRB, 0.35059E+00_JPRB, 0.35336E+00_JPRB, 0.35611E+00_JPRB, 0.35859E+00_JPRB, 0.35960E+00_JPRB ,&
&  0.36188E+00_JPRB, 0.36395E+00_JPRB, 0.36383E+00_JPRB, 0.36288E+00_JPRB, 0.36065E+00_JPRB, 0.35526E+00_JPRB, 0.35007E+00_JPRB ,&
&  0.34441E+00_JPRB, 0.34034E+00_JPRB, 0.33910E+00_JPRB, 0.33785E+00_JPRB, 0.33892E+00_JPRB, 0.33963E+00_JPRB, 0.34026E+00_JPRB ,&
&  0.34230E+00_JPRB, 0.34432E+00_JPRB, 0.34687E+00_JPRB, 0.35077E+00_JPRB, 0.35349E+00_JPRB, 0.35568E+00_JPRB, 0.35832E+00_JPRB ,&
&  0.35760E+00_JPRB, 0.36237E+00_JPRB, 0.36846E+00_JPRB, 0.37756E+00_JPRB, 0.39518E+00_JPRB, 0.40625E+00_JPRB, 0.43046E+00_JPRB ,&
&  0.46145E+00_JPRB, 0.51866E+00_JPRB, 0.50949E+00_JPRB, 0.44721E+00_JPRB, 0.45411E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.44706E+00_JPRB, 0.45140E+00_JPRB, 0.45480E+00_JPRB, 0.45919E+00_JPRB, 0.46443E+00_JPRB, 0.46947E+00_JPRB, 0.47279E+00_JPRB ,&
&  0.47530E+00_JPRB, 0.47421E+00_JPRB, 0.47605E+00_JPRB, 0.47045E+00_JPRB, 0.46492E+00_JPRB, 0.45754E+00_JPRB, 0.45093E+00_JPRB ,&
&  0.44469E+00_JPRB, 0.43541E+00_JPRB, 0.42562E+00_JPRB, 0.41713E+00_JPRB, 0.40980E+00_JPRB, 0.40454E+00_JPRB, 0.40074E+00_JPRB ,&
&  0.39808E+00_JPRB, 0.39938E+00_JPRB, 0.40145E+00_JPRB, 0.40763E+00_JPRB, 0.41794E+00_JPRB, 0.43002E+00_JPRB, 0.44897E+00_JPRB ,&
&  0.47493E+00_JPRB, 0.47701E+00_JPRB, 0.48429E+00_JPRB, 0.48359E+00_JPRB, 0.47951E+00_JPRB, 0.47831E+00_JPRB, 0.47612E+00_JPRB ,&
&  0.47697E+00_JPRB, 0.46613E+00_JPRB, 0.47552E+00_JPRB, 0.45565E+00_JPRB, 0.42543E+00_JPRB, 0.41000E+00_JPRB, 0.39112E+00_JPRB ,&
&  0.37873E+00_JPRB, 0.36808E+00_JPRB, 0.36026E+00_JPRB, 0.35784E+00_JPRB, 0.35327E+00_JPRB, 0.35125E+00_JPRB, 0.34657E+00_JPRB ,&
&  0.34201E+00_JPRB, 0.34034E+00_JPRB, 0.33694E+00_JPRB, 0.33722E+00_JPRB, 0.33764E+00_JPRB, 0.33697E+00_JPRB, 0.33713E+00_JPRB ,&
&  0.33769E+00_JPRB, 0.33931E+00_JPRB, 0.34225E+00_JPRB, 0.34485E+00_JPRB, 0.34642E+00_JPRB, 0.34730E+00_JPRB, 0.34707E+00_JPRB ,&
&  0.34773E+00_JPRB, 0.34736E+00_JPRB, 0.34631E+00_JPRB, 0.34473E+00_JPRB, 0.34214E+00_JPRB, 0.33938E+00_JPRB, 0.33666E+00_JPRB ,&
&  0.33433E+00_JPRB, 0.33324E+00_JPRB, 0.33217E+00_JPRB, 0.33208E+00_JPRB, 0.33243E+00_JPRB, 0.33434E+00_JPRB, 0.33611E+00_JPRB ,&
&  0.33820E+00_JPRB, 0.34139E+00_JPRB, 0.34409E+00_JPRB, 0.34746E+00_JPRB, 0.35034E+00_JPRB, 0.35154E+00_JPRB, 0.35610E+00_JPRB ,&
&  0.36212E+00_JPRB, 0.37081E+00_JPRB, 0.38110E+00_JPRB, 0.39041E+00_JPRB, 0.40204E+00_JPRB, 0.41722E+00_JPRB, 0.43697E+00_JPRB ,&
&  0.45588E+00_JPRB, 0.47260E+00_JPRB, 0.46365E+00_JPRB, 0.44044E+00_JPRB, 0.44216E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.44066E+00_JPRB, 0.44038E+00_JPRB, 0.44209E+00_JPRB, 0.44441E+00_JPRB, 0.45020E+00_JPRB, 0.45710E+00_JPRB, 0.46450E+00_JPRB ,&
&  0.46462E+00_JPRB, 0.46850E+00_JPRB, 0.46702E+00_JPRB, 0.46310E+00_JPRB, 0.45699E+00_JPRB, 0.45000E+00_JPRB, 0.44526E+00_JPRB ,&
&  0.43869E+00_JPRB, 0.43017E+00_JPRB, 0.42195E+00_JPRB, 0.41330E+00_JPRB, 0.40531E+00_JPRB, 0.39703E+00_JPRB, 0.39327E+00_JPRB ,&
&  0.38971E+00_JPRB, 0.38892E+00_JPRB, 0.39098E+00_JPRB, 0.39317E+00_JPRB, 0.39841E+00_JPRB, 0.40565E+00_JPRB, 0.41538E+00_JPRB ,&
&  0.42440E+00_JPRB, 0.43568E+00_JPRB, 0.44179E+00_JPRB, 0.44159E+00_JPRB, 0.44161E+00_JPRB, 0.43974E+00_JPRB, 0.43641E+00_JPRB ,&
&  0.42925E+00_JPRB, 0.42157E+00_JPRB, 0.41228E+00_JPRB, 0.40246E+00_JPRB, 0.39238E+00_JPRB, 0.38368E+00_JPRB, 0.37558E+00_JPRB ,&
&  0.36903E+00_JPRB, 0.36348E+00_JPRB, 0.35818E+00_JPRB, 0.35367E+00_JPRB, 0.35021E+00_JPRB, 0.34774E+00_JPRB, 0.34426E+00_JPRB ,&
&  0.34154E+00_JPRB, 0.33907E+00_JPRB, 0.33606E+00_JPRB, 0.33411E+00_JPRB, 0.33232E+00_JPRB, 0.33118E+00_JPRB, 0.33022E+00_JPRB ,&
&  0.33044E+00_JPRB, 0.33069E+00_JPRB, 0.33193E+00_JPRB, 0.33318E+00_JPRB, 0.33453E+00_JPRB, 0.33477E+00_JPRB, 0.33473E+00_JPRB ,&
&  0.33438E+00_JPRB, 0.33397E+00_JPRB, 0.33204E+00_JPRB, 0.33049E+00_JPRB, 0.32935E+00_JPRB, 0.32810E+00_JPRB, 0.32782E+00_JPRB ,&
&  0.32856E+00_JPRB, 0.32933E+00_JPRB, 0.32980E+00_JPRB, 0.33158E+00_JPRB, 0.33331E+00_JPRB, 0.33596E+00_JPRB, 0.33926E+00_JPRB ,&
&  0.34158E+00_JPRB, 0.34434E+00_JPRB, 0.34738E+00_JPRB, 0.35092E+00_JPRB, 0.35526E+00_JPRB, 0.35965E+00_JPRB, 0.36497E+00_JPRB ,&
&  0.37102E+00_JPRB, 0.37703E+00_JPRB, 0.38378E+00_JPRB, 0.39130E+00_JPRB, 0.40037E+00_JPRB, 0.40492E+00_JPRB, 0.41300E+00_JPRB ,&
&  0.41822E+00_JPRB, 0.41019E+00_JPRB, 0.45033E+00_JPRB, 0.44376E+00_JPRB, 0.44020E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.38608E+00_JPRB, 0.39242E+00_JPRB, 0.39604E+00_JPRB, 0.39915E+00_JPRB, 0.38662E+00_JPRB, 0.42393E+00_JPRB, 0.42170E+00_JPRB ,&
&  0.42328E+00_JPRB, 0.41442E+00_JPRB, 0.41795E+00_JPRB, 0.41840E+00_JPRB, 0.41673E+00_JPRB, 0.41389E+00_JPRB, 0.41111E+00_JPRB ,&
&  0.40857E+00_JPRB, 0.40439E+00_JPRB, 0.39917E+00_JPRB, 0.39453E+00_JPRB, 0.39101E+00_JPRB, 0.38802E+00_JPRB, 0.38538E+00_JPRB ,&
&  0.38473E+00_JPRB, 0.38613E+00_JPRB, 0.38869E+00_JPRB, 0.38589E+00_JPRB, 0.38585E+00_JPRB, 0.38692E+00_JPRB, 0.38865E+00_JPRB ,&
&  0.39011E+00_JPRB, 0.39167E+00_JPRB, 0.39157E+00_JPRB, 0.38958E+00_JPRB, 0.38939E+00_JPRB, 0.38761E+00_JPRB, 0.38580E+00_JPRB ,&
&  0.38329E+00_JPRB, 0.38056E+00_JPRB, 0.37724E+00_JPRB, 0.37460E+00_JPRB, 0.37159E+00_JPRB, 0.36790E+00_JPRB, 0.36440E+00_JPRB ,&
&  0.36138E+00_JPRB, 0.35863E+00_JPRB, 0.35529E+00_JPRB, 0.35221E+00_JPRB, 0.34923E+00_JPRB, 0.34621E+00_JPRB, 0.34337E+00_JPRB ,&
&  0.34118E+00_JPRB, 0.33934E+00_JPRB, 0.33755E+00_JPRB, 0.33565E+00_JPRB, 0.33386E+00_JPRB, 0.33214E+00_JPRB, 0.33083E+00_JPRB ,&
&  0.33091E+00_JPRB, 0.33024E+00_JPRB, 0.32961E+00_JPRB, 0.32945E+00_JPRB, 0.32982E+00_JPRB, 0.32936E+00_JPRB, 0.32923E+00_JPRB ,&
&  0.32924E+00_JPRB, 0.32933E+00_JPRB, 0.32959E+00_JPRB, 0.32991E+00_JPRB, 0.33037E+00_JPRB, 0.33098E+00_JPRB, 0.33199E+00_JPRB ,&
&  0.33318E+00_JPRB, 0.33465E+00_JPRB, 0.33624E+00_JPRB, 0.33785E+00_JPRB, 0.33950E+00_JPRB, 0.34124E+00_JPRB, 0.34323E+00_JPRB ,&
&  0.34519E+00_JPRB, 0.34708E+00_JPRB, 0.34888E+00_JPRB, 0.35065E+00_JPRB, 0.35384E+00_JPRB, 0.35686E+00_JPRB, 0.36007E+00_JPRB ,&
&  0.36356E+00_JPRB, 0.36767E+00_JPRB, 0.37200E+00_JPRB, 0.37680E+00_JPRB, 0.38237E+00_JPRB, 0.38278E+00_JPRB, 0.38298E+00_JPRB ,&
&  0.38263E+00_JPRB, 0.38579E+00_JPRB, 0.38621E+00_JPRB, 0.38444E+00_JPRB, 0.38305E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.36110E+00_JPRB, 0.36092E+00_JPRB, 0.36063E+00_JPRB, 0.36024E+00_JPRB, 0.35774E+00_JPRB, 0.35776E+00_JPRB, 0.35796E+00_JPRB ,&
&  0.35868E+00_JPRB, 0.36078E+00_JPRB, 0.36312E+00_JPRB, 0.36484E+00_JPRB, 0.36601E+00_JPRB, 0.36679E+00_JPRB, 0.36744E+00_JPRB ,&
&  0.36762E+00_JPRB, 0.36809E+00_JPRB, 0.36356E+00_JPRB, 0.36188E+00_JPRB, 0.36120E+00_JPRB, 0.36104E+00_JPRB, 0.36120E+00_JPRB ,&
&  0.36122E+00_JPRB, 0.36108E+00_JPRB, 0.36025E+00_JPRB, 0.36063E+00_JPRB, 0.36105E+00_JPRB, 0.36130E+00_JPRB, 0.36156E+00_JPRB ,&
&  0.36183E+00_JPRB, 0.36210E+00_JPRB, 0.36205E+00_JPRB, 0.36178E+00_JPRB, 0.36174E+00_JPRB, 0.36136E+00_JPRB, 0.36054E+00_JPRB ,&
&  0.35951E+00_JPRB, 0.35815E+00_JPRB, 0.35683E+00_JPRB, 0.35560E+00_JPRB, 0.35456E+00_JPRB, 0.35366E+00_JPRB, 0.35279E+00_JPRB ,&
&  0.35168E+00_JPRB, 0.35015E+00_JPRB, 0.34853E+00_JPRB, 0.34693E+00_JPRB, 0.34539E+00_JPRB, 0.34398E+00_JPRB, 0.34247E+00_JPRB ,&
&  0.34133E+00_JPRB, 0.34039E+00_JPRB, 0.33947E+00_JPRB, 0.33869E+00_JPRB, 0.33789E+00_JPRB, 0.33712E+00_JPRB, 0.33638E+00_JPRB ,&
&  0.33534E+00_JPRB, 0.33484E+00_JPRB, 0.33427E+00_JPRB, 0.33408E+00_JPRB, 0.33428E+00_JPRB, 0.33445E+00_JPRB, 0.33473E+00_JPRB ,&
&  0.33510E+00_JPRB, 0.33550E+00_JPRB, 0.33569E+00_JPRB, 0.33597E+00_JPRB, 0.33637E+00_JPRB, 0.33688E+00_JPRB, 0.33748E+00_JPRB ,&
&  0.33812E+00_JPRB, 0.33882E+00_JPRB, 0.33959E+00_JPRB, 0.34044E+00_JPRB, 0.34133E+00_JPRB, 0.34230E+00_JPRB, 0.34322E+00_JPRB ,&
&  0.34416E+00_JPRB, 0.34503E+00_JPRB, 0.34592E+00_JPRB, 0.34660E+00_JPRB, 0.34765E+00_JPRB, 0.34865E+00_JPRB, 0.34985E+00_JPRB ,&
&  0.35105E+00_JPRB, 0.35235E+00_JPRB, 0.35349E+00_JPRB, 0.35475E+00_JPRB, 0.35617E+00_JPRB, 0.35574E+00_JPRB, 0.35547E+00_JPRB ,&
&  0.35526E+00_JPRB, 0.35547E+00_JPRB, 0.35516E+00_JPRB, 0.35541E+00_JPRB, 0.35637E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB ,&
&  0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB, 0.34255E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB ,&
&  0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB, 0.71237E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.72293E+00_JPRB, 0.72327E+00_JPRB, 0.72350E+00_JPRB, 0.72352E+00_JPRB, 0.72348E+00_JPRB, 0.72364E+00_JPRB, 0.72371E+00_JPRB ,&
&  0.72376E+00_JPRB, 0.72378E+00_JPRB, 0.72415E+00_JPRB, 0.72409E+00_JPRB, 0.72402E+00_JPRB, 0.72371E+00_JPRB, 0.72340E+00_JPRB ,&
&  0.72300E+00_JPRB, 0.72262E+00_JPRB, 0.72216E+00_JPRB, 0.72182E+00_JPRB, 0.72158E+00_JPRB, 0.72152E+00_JPRB, 0.72141E+00_JPRB ,&
&  0.72106E+00_JPRB, 0.72037E+00_JPRB, 0.71983E+00_JPRB, 0.71898E+00_JPRB, 0.71793E+00_JPRB, 0.71640E+00_JPRB, 0.71455E+00_JPRB ,&
&  0.71235E+00_JPRB, 0.71029E+00_JPRB, 0.70860E+00_JPRB, 0.70704E+00_JPRB, 0.70584E+00_JPRB, 0.70490E+00_JPRB, 0.70421E+00_JPRB ,&
&  0.70335E+00_JPRB, 0.70282E+00_JPRB, 0.70232E+00_JPRB, 0.70194E+00_JPRB, 0.70112E+00_JPRB, 0.70047E+00_JPRB, 0.69971E+00_JPRB ,&
&  0.69895E+00_JPRB, 0.69793E+00_JPRB, 0.69724E+00_JPRB, 0.69648E+00_JPRB, 0.69562E+00_JPRB, 0.69406E+00_JPRB, 0.69270E+00_JPRB ,&
&  0.69143E+00_JPRB, 0.69014E+00_JPRB, 0.68868E+00_JPRB, 0.68739E+00_JPRB, 0.68647E+00_JPRB, 0.68560E+00_JPRB, 0.68458E+00_JPRB ,&
&  0.68437E+00_JPRB, 0.68493E+00_JPRB, 0.68548E+00_JPRB, 0.68577E+00_JPRB, 0.68655E+00_JPRB, 0.68747E+00_JPRB, 0.68848E+00_JPRB ,&
&  0.68941E+00_JPRB, 0.69064E+00_JPRB, 0.69242E+00_JPRB, 0.69405E+00_JPRB, 0.69580E+00_JPRB, 0.69761E+00_JPRB, 0.69966E+00_JPRB ,&
&  0.70154E+00_JPRB, 0.70339E+00_JPRB, 0.70505E+00_JPRB, 0.70690E+00_JPRB, 0.70815E+00_JPRB, 0.70936E+00_JPRB, 0.71019E+00_JPRB ,&
&  0.71107E+00_JPRB, 0.71172E+00_JPRB, 0.71244E+00_JPRB, 0.71304E+00_JPRB, 0.71390E+00_JPRB, 0.71461E+00_JPRB, 0.71536E+00_JPRB ,&
&  0.71607E+00_JPRB, 0.71694E+00_JPRB, 0.71764E+00_JPRB, 0.71830E+00_JPRB, 0.71883E+00_JPRB, 0.71962E+00_JPRB, 0.72024E+00_JPRB ,&
&  0.72078E+00_JPRB, 0.72135E+00_JPRB, 0.72193E+00_JPRB, 0.72245E+00_JPRB, 0.72277E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.71847E+00_JPRB, 0.72082E+00_JPRB, 0.72214E+00_JPRB, 0.72338E+00_JPRB, 0.72447E+00_JPRB, 0.72621E+00_JPRB, 0.72744E+00_JPRB ,&
&  0.72864E+00_JPRB, 0.73030E+00_JPRB, 0.73274E+00_JPRB, 0.73404E+00_JPRB, 0.73496E+00_JPRB, 0.73529E+00_JPRB, 0.73547E+00_JPRB ,&
&  0.73509E+00_JPRB, 0.73425E+00_JPRB, 0.73328E+00_JPRB, 0.73239E+00_JPRB, 0.73156E+00_JPRB, 0.73087E+00_JPRB, 0.72976E+00_JPRB ,&
&  0.72778E+00_JPRB, 0.72526E+00_JPRB, 0.72249E+00_JPRB, 0.71817E+00_JPRB, 0.71338E+00_JPRB, 0.70836E+00_JPRB, 0.70310E+00_JPRB ,&
&  0.69737E+00_JPRB, 0.69293E+00_JPRB, 0.68876E+00_JPRB, 0.68486E+00_JPRB, 0.68350E+00_JPRB, 0.68202E+00_JPRB, 0.68047E+00_JPRB ,&
&  0.67889E+00_JPRB, 0.67625E+00_JPRB, 0.67536E+00_JPRB, 0.67481E+00_JPRB, 0.67472E+00_JPRB, 0.67578E+00_JPRB, 0.67657E+00_JPRB ,&
&  0.67706E+00_JPRB, 0.67686E+00_JPRB, 0.67657E+00_JPRB, 0.67685E+00_JPRB, 0.67751E+00_JPRB, 0.67744E+00_JPRB, 0.67620E+00_JPRB ,&
&  0.67527E+00_JPRB, 0.67378E+00_JPRB, 0.67206E+00_JPRB, 0.67001E+00_JPRB, 0.66903E+00_JPRB, 0.66770E+00_JPRB, 0.66619E+00_JPRB ,&
&  0.66531E+00_JPRB, 0.66529E+00_JPRB, 0.66551E+00_JPRB, 0.66586E+00_JPRB, 0.66697E+00_JPRB, 0.66841E+00_JPRB, 0.66984E+00_JPRB ,&
&  0.67134E+00_JPRB, 0.67389E+00_JPRB, 0.67681E+00_JPRB, 0.67949E+00_JPRB, 0.68230E+00_JPRB, 0.68512E+00_JPRB, 0.68831E+00_JPRB ,&
&  0.69082E+00_JPRB, 0.69276E+00_JPRB, 0.69366E+00_JPRB, 0.69548E+00_JPRB, 0.69705E+00_JPRB, 0.69914E+00_JPRB, 0.70067E+00_JPRB ,&
&  0.70288E+00_JPRB, 0.70428E+00_JPRB, 0.70516E+00_JPRB, 0.70603E+00_JPRB, 0.70702E+00_JPRB, 0.70775E+00_JPRB, 0.70813E+00_JPRB ,&
&  0.70831E+00_JPRB, 0.70863E+00_JPRB, 0.70883E+00_JPRB, 0.70889E+00_JPRB, 0.70916E+00_JPRB, 0.70959E+00_JPRB, 0.71012E+00_JPRB ,&
&  0.71087E+00_JPRB, 0.71232E+00_JPRB, 0.71420E+00_JPRB, 0.71552E+00_JPRB, 0.71682E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.70865E+00_JPRB, 0.71054E+00_JPRB, 0.71197E+00_JPRB, 0.71431E+00_JPRB, 0.71852E+00_JPRB, 0.72293E+00_JPRB, 0.72552E+00_JPRB ,&
&  0.72906E+00_JPRB, 0.73324E+00_JPRB, 0.73796E+00_JPRB, 0.74087E+00_JPRB, 0.74309E+00_JPRB, 0.74426E+00_JPRB, 0.74426E+00_JPRB ,&
&  0.74304E+00_JPRB, 0.74110E+00_JPRB, 0.73986E+00_JPRB, 0.73821E+00_JPRB, 0.73621E+00_JPRB, 0.73395E+00_JPRB, 0.72815E+00_JPRB ,&
&  0.72319E+00_JPRB, 0.71963E+00_JPRB, 0.71275E+00_JPRB, 0.70494E+00_JPRB, 0.69873E+00_JPRB, 0.65558E+00_JPRB, 0.64124E+00_JPRB ,&
&  0.64036E+00_JPRB, 0.63657E+00_JPRB, 0.63753E+00_JPRB, 0.63914E+00_JPRB, 0.64377E+00_JPRB, 0.63637E+00_JPRB, 0.63146E+00_JPRB ,&
&  0.60496E+00_JPRB, 0.60603E+00_JPRB, 0.60188E+00_JPRB, 0.59969E+00_JPRB, 0.60240E+00_JPRB, 0.60415E+00_JPRB, 0.61277E+00_JPRB ,&
&  0.63631E+00_JPRB, 0.64277E+00_JPRB, 0.64605E+00_JPRB, 0.64891E+00_JPRB, 0.65238E+00_JPRB, 0.65453E+00_JPRB, 0.65634E+00_JPRB ,&
&  0.65703E+00_JPRB, 0.65683E+00_JPRB, 0.65700E+00_JPRB, 0.65633E+00_JPRB, 0.65569E+00_JPRB, 0.65435E+00_JPRB, 0.65347E+00_JPRB ,&
&  0.65290E+00_JPRB, 0.65380E+00_JPRB, 0.65551E+00_JPRB, 0.65724E+00_JPRB, 0.65972E+00_JPRB, 0.66145E+00_JPRB, 0.66305E+00_JPRB ,&
&  0.66468E+00_JPRB, 0.66626E+00_JPRB, 0.66818E+00_JPRB, 0.67018E+00_JPRB, 0.67267E+00_JPRB, 0.67422E+00_JPRB, 0.67653E+00_JPRB ,&
&  0.67834E+00_JPRB, 0.68069E+00_JPRB, 0.68264E+00_JPRB, 0.68671E+00_JPRB, 0.68975E+00_JPRB, 0.69418E+00_JPRB, 0.69801E+00_JPRB ,&
&  0.70259E+00_JPRB, 0.70557E+00_JPRB, 0.70826E+00_JPRB, 0.71010E+00_JPRB, 0.71223E+00_JPRB, 0.71383E+00_JPRB, 0.71447E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71363E+00_JPRB, 0.71245E+00_JPRB, 0.71064E+00_JPRB, 0.70913E+00_JPRB, 0.70711E+00_JPRB, 0.70544E+00_JPRB ,&
&  0.70331E+00_JPRB, 0.70295E+00_JPRB, 0.70375E+00_JPRB, 0.70544E+00_JPRB, 0.70699E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.71722E+00_JPRB, 0.71713E+00_JPRB, 0.71677E+00_JPRB, 0.71930E+00_JPRB, 0.72246E+00_JPRB, 0.72828E+00_JPRB, 0.72980E+00_JPRB ,&
&  0.73511E+00_JPRB, 0.74054E+00_JPRB, 0.74670E+00_JPRB, 0.74963E+00_JPRB, 0.75226E+00_JPRB, 0.75230E+00_JPRB, 0.75077E+00_JPRB ,&
&  0.74876E+00_JPRB, 0.74484E+00_JPRB, 0.74218E+00_JPRB, 0.73792E+00_JPRB, 0.73242E+00_JPRB, 0.72848E+00_JPRB, 0.72113E+00_JPRB ,&
&  0.71674E+00_JPRB, 0.71068E+00_JPRB, 0.66982E+00_JPRB, 0.65844E+00_JPRB, 0.63833E+00_JPRB, 0.62619E+00_JPRB, 0.61971E+00_JPRB ,&
&  0.61080E+00_JPRB, 0.60209E+00_JPRB, 0.59023E+00_JPRB, 0.57655E+00_JPRB, 0.56243E+00_JPRB, 0.54202E+00_JPRB, 0.53361E+00_JPRB ,&
&  0.52530E+00_JPRB, 0.52395E+00_JPRB, 0.52609E+00_JPRB, 0.53118E+00_JPRB, 0.54249E+00_JPRB, 0.55359E+00_JPRB, 0.56801E+00_JPRB ,&
&  0.58838E+00_JPRB, 0.60869E+00_JPRB, 0.61878E+00_JPRB, 0.61704E+00_JPRB, 0.63446E+00_JPRB, 0.64304E+00_JPRB, 0.64789E+00_JPRB ,&
&  0.65001E+00_JPRB, 0.64804E+00_JPRB, 0.65353E+00_JPRB, 0.65353E+00_JPRB, 0.65361E+00_JPRB, 0.65416E+00_JPRB, 0.65538E+00_JPRB ,&
&  0.65825E+00_JPRB, 0.66123E+00_JPRB, 0.66357E+00_JPRB, 0.66568E+00_JPRB, 0.66699E+00_JPRB, 0.66742E+00_JPRB, 0.66773E+00_JPRB ,&
&  0.66830E+00_JPRB, 0.66902E+00_JPRB, 0.67007E+00_JPRB, 0.67125E+00_JPRB, 0.67333E+00_JPRB, 0.67461E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.67751E+00_JPRB, 0.68051E+00_JPRB, 0.68389E+00_JPRB, 0.68340E+00_JPRB, 0.68729E+00_JPRB, 0.69716E+00_JPRB, 0.70256E+00_JPRB ,&
&  0.71111E+00_JPRB, 0.71754E+00_JPRB, 0.72491E+00_JPRB, 0.73065E+00_JPRB, 0.73570E+00_JPRB, 0.73802E+00_JPRB, 0.73798E+00_JPRB ,&
&  0.73725E+00_JPRB, 0.73530E+00_JPRB, 0.73388E+00_JPRB, 0.73063E+00_JPRB, 0.72776E+00_JPRB, 0.72360E+00_JPRB, 0.72061E+00_JPRB ,&
&  0.71687E+00_JPRB, 0.71542E+00_JPRB, 0.71229E+00_JPRB, 0.71376E+00_JPRB, 0.71509E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.73061E+00_JPRB, 0.72974E+00_JPRB, 0.73111E+00_JPRB, 0.73681E+00_JPRB, 0.74688E+00_JPRB, 0.75839E+00_JPRB, 0.75813E+00_JPRB ,&
&  0.75896E+00_JPRB, 0.75761E+00_JPRB, 0.76075E+00_JPRB, 0.76085E+00_JPRB, 0.76053E+00_JPRB, 0.75808E+00_JPRB, 0.75613E+00_JPRB ,&
&  0.75401E+00_JPRB, 0.74716E+00_JPRB, 0.73818E+00_JPRB, 0.73341E+00_JPRB, 0.73134E+00_JPRB, 0.72954E+00_JPRB, 0.72257E+00_JPRB ,&
&  0.71453E+00_JPRB, 0.69799E+00_JPRB, 0.66727E+00_JPRB, 0.64819E+00_JPRB, 0.62656E+00_JPRB, 0.61813E+00_JPRB, 0.60374E+00_JPRB ,&
&  0.60380E+00_JPRB, 0.55844E+00_JPRB, 0.56200E+00_JPRB, 0.54349E+00_JPRB, 0.53129E+00_JPRB, 0.51732E+00_JPRB, 0.50007E+00_JPRB ,&
&  0.49525E+00_JPRB, 0.48876E+00_JPRB, 0.48774E+00_JPRB, 0.49672E+00_JPRB, 0.50751E+00_JPRB, 0.52527E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.55491E+00_JPRB, 0.60169E+00_JPRB, 0.61658E+00_JPRB, 0.62233E+00_JPRB, 0.63808E+00_JPRB, 0.64882E+00_JPRB, 0.65018E+00_JPRB ,&
&  0.65125E+00_JPRB, 0.65070E+00_JPRB, 0.65809E+00_JPRB, 0.66277E+00_JPRB, 0.66916E+00_JPRB, 0.66882E+00_JPRB, 0.66908E+00_JPRB ,&
&  0.67225E+00_JPRB, 0.67615E+00_JPRB, 0.68070E+00_JPRB, 0.68469E+00_JPRB, 0.68831E+00_JPRB, 0.69065E+00_JPRB, 0.69153E+00_JPRB ,&
&  0.69171E+00_JPRB, 0.69080E+00_JPRB, 0.69064E+00_JPRB, 0.69196E+00_JPRB, 0.69345E+00_JPRB, 0.69661E+00_JPRB, 0.69856E+00_JPRB ,&
&  0.70071E+00_JPRB, 0.70148E+00_JPRB, 0.70083E+00_JPRB, 0.70210E+00_JPRB, 0.70672E+00_JPRB, 0.71164E+00_JPRB, 0.72009E+00_JPRB ,&
&  0.73183E+00_JPRB, 0.73931E+00_JPRB, 0.74769E+00_JPRB, 0.75494E+00_JPRB, 0.76111E+00_JPRB, 0.76241E+00_JPRB, 0.76109E+00_JPRB ,&
&  0.75973E+00_JPRB, 0.75785E+00_JPRB, 0.75550E+00_JPRB, 0.75113E+00_JPRB, 0.74641E+00_JPRB, 0.73986E+00_JPRB, 0.73419E+00_JPRB ,&
&  0.72853E+00_JPRB, 0.72696E+00_JPRB, 0.72295E+00_JPRB, 0.72449E+00_JPRB, 0.72744E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.74899E+00_JPRB, 0.75054E+00_JPRB, 0.75601E+00_JPRB, 0.76373E+00_JPRB, 0.77193E+00_JPRB, 0.78159E+00_JPRB, 0.78577E+00_JPRB ,&
&  0.78902E+00_JPRB, 0.78781E+00_JPRB, 0.78473E+00_JPRB, 0.77288E+00_JPRB, 0.76521E+00_JPRB, 0.76149E+00_JPRB, 0.75709E+00_JPRB ,&
&  0.75094E+00_JPRB, 0.74625E+00_JPRB, 0.74232E+00_JPRB, 0.73988E+00_JPRB, 0.73503E+00_JPRB, 0.73193E+00_JPRB, 0.72069E+00_JPRB ,&
&  0.70063E+00_JPRB, 0.68201E+00_JPRB, 0.65517E+00_JPRB, 0.63347E+00_JPRB, 0.61863E+00_JPRB, 0.60362E+00_JPRB, 0.60215E+00_JPRB ,&
&  0.59831E+00_JPRB, 0.59507E+00_JPRB, 0.59037E+00_JPRB, 0.58650E+00_JPRB, 0.58691E+00_JPRB, 0.51172E+00_JPRB, 0.49489E+00_JPRB ,&
&  0.48400E+00_JPRB, 0.47450E+00_JPRB, 0.47236E+00_JPRB, 0.47250E+00_JPRB, 0.47964E+00_JPRB, 0.50297E+00_JPRB, 0.53314E+00_JPRB ,&
&  0.55075E+00_JPRB, 0.61405E+00_JPRB, 0.62673E+00_JPRB, 0.63802E+00_JPRB, 0.65840E+00_JPRB, 0.67134E+00_JPRB, 0.67465E+00_JPRB ,&
&  0.68270E+00_JPRB, 0.69745E+00_JPRB, 0.71600E+00_JPRB, 0.73386E+00_JPRB, 0.73549E+00_JPRB, 0.74093E+00_JPRB, 0.74069E+00_JPRB ,&
&  0.73620E+00_JPRB, 0.72775E+00_JPRB, 0.72505E+00_JPRB, 0.72408E+00_JPRB, 0.72230E+00_JPRB, 0.72258E+00_JPRB, 0.71960E+00_JPRB ,&
&  0.72197E+00_JPRB, 0.72059E+00_JPRB, 0.72676E+00_JPRB, 0.72949E+00_JPRB, 0.73546E+00_JPRB, 0.74175E+00_JPRB, 0.74466E+00_JPRB ,&
&  0.74460E+00_JPRB, 0.74539E+00_JPRB, 0.74088E+00_JPRB, 0.73262E+00_JPRB, 0.73021E+00_JPRB, 0.72740E+00_JPRB, 0.73078E+00_JPRB ,&
&  0.73705E+00_JPRB, 0.74877E+00_JPRB, 0.75580E+00_JPRB, 0.76349E+00_JPRB, 0.77009E+00_JPRB, 0.77103E+00_JPRB, 0.77339E+00_JPRB ,&
&  0.77442E+00_JPRB, 0.77606E+00_JPRB, 0.77442E+00_JPRB, 0.77034E+00_JPRB, 0.76317E+00_JPRB, 0.75339E+00_JPRB, 0.74300E+00_JPRB ,&
&  0.73756E+00_JPRB, 0.73750E+00_JPRB, 0.73586E+00_JPRB, 0.73829E+00_JPRB, 0.74349E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.76610E+00_JPRB, 0.76844E+00_JPRB, 0.77735E+00_JPRB, 0.78593E+00_JPRB, 0.79658E+00_JPRB, 0.80664E+00_JPRB, 0.81070E+00_JPRB ,&
&  0.82088E+00_JPRB, 0.81810E+00_JPRB, 0.80426E+00_JPRB, 0.78699E+00_JPRB, 0.76814E+00_JPRB, 0.75520E+00_JPRB, 0.74956E+00_JPRB ,&
&  0.74933E+00_JPRB, 0.74108E+00_JPRB, 0.73563E+00_JPRB, 0.73170E+00_JPRB, 0.72505E+00_JPRB, 0.71955E+00_JPRB, 0.70786E+00_JPRB ,&
&  0.68504E+00_JPRB, 0.67062E+00_JPRB, 0.65528E+00_JPRB, 0.63278E+00_JPRB, 0.61403E+00_JPRB, 0.60395E+00_JPRB, 0.60529E+00_JPRB ,&
&  0.60229E+00_JPRB, 0.59220E+00_JPRB, 0.59256E+00_JPRB, 0.58737E+00_JPRB, 0.57344E+00_JPRB, 0.56819E+00_JPRB, 0.51449E+00_JPRB ,&
&  0.49711E+00_JPRB, 0.47609E+00_JPRB, 0.46870E+00_JPRB, 0.46825E+00_JPRB, 0.48332E+00_JPRB, 0.50724E+00_JPRB, 0.55066E+00_JPRB ,&
&  0.58911E+00_JPRB, 0.62406E+00_JPRB, 0.65152E+00_JPRB, 0.67116E+00_JPRB, 0.69104E+00_JPRB, 0.70921E+00_JPRB, 0.72154E+00_JPRB ,&
&  0.74369E+00_JPRB, 0.77362E+00_JPRB, 0.80317E+00_JPRB, 0.83168E+00_JPRB, 0.85064E+00_JPRB, 0.85691E+00_JPRB, 0.86175E+00_JPRB ,&
&  0.85460E+00_JPRB, 0.78689E+00_JPRB, 0.76042E+00_JPRB, 0.74488E+00_JPRB, 0.73431E+00_JPRB, 0.73415E+00_JPRB, 0.72891E+00_JPRB ,&
&  0.73140E+00_JPRB, 0.73643E+00_JPRB, 0.73874E+00_JPRB, 0.74241E+00_JPRB, 0.74792E+00_JPRB, 0.75033E+00_JPRB, 0.75742E+00_JPRB ,&
&  0.76975E+00_JPRB, 0.78647E+00_JPRB, 0.81574E+00_JPRB, 0.79427E+00_JPRB, 0.76821E+00_JPRB, 0.74346E+00_JPRB, 0.73087E+00_JPRB ,&
&  0.72927E+00_JPRB, 0.73519E+00_JPRB, 0.74604E+00_JPRB, 0.75176E+00_JPRB, 0.75800E+00_JPRB, 0.76348E+00_JPRB, 0.77761E+00_JPRB ,&
&  0.78320E+00_JPRB, 0.78820E+00_JPRB, 0.78962E+00_JPRB, 0.78767E+00_JPRB, 0.77953E+00_JPRB, 0.76852E+00_JPRB, 0.75768E+00_JPRB ,&
&  0.75368E+00_JPRB, 0.75499E+00_JPRB, 0.75602E+00_JPRB, 0.75889E+00_JPRB, 0.76262E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.77244E+00_JPRB, 0.78289E+00_JPRB, 0.80195E+00_JPRB, 0.81898E+00_JPRB, 0.83168E+00_JPRB, 0.82206E+00_JPRB, 0.80971E+00_JPRB ,&
&  0.80744E+00_JPRB, 0.80569E+00_JPRB, 0.79376E+00_JPRB, 0.78034E+00_JPRB, 0.76017E+00_JPRB, 0.74439E+00_JPRB, 0.73978E+00_JPRB ,&
&  0.73889E+00_JPRB, 0.73193E+00_JPRB, 0.72123E+00_JPRB, 0.70701E+00_JPRB, 0.69570E+00_JPRB, 0.69233E+00_JPRB, 0.68510E+00_JPRB ,&
&  0.67153E+00_JPRB, 0.66411E+00_JPRB, 0.65577E+00_JPRB, 0.63945E+00_JPRB, 0.62415E+00_JPRB, 0.61472E+00_JPRB, 0.61088E+00_JPRB ,&
&  0.60846E+00_JPRB, 0.60668E+00_JPRB, 0.60545E+00_JPRB, 0.60056E+00_JPRB, 0.59797E+00_JPRB, 0.59309E+00_JPRB, 0.55293E+00_JPRB ,&
&  0.51250E+00_JPRB, 0.48530E+00_JPRB, 0.46979E+00_JPRB, 0.46908E+00_JPRB, 0.49056E+00_JPRB, 0.53380E+00_JPRB, 0.58058E+00_JPRB ,&
&  0.61960E+00_JPRB, 0.66423E+00_JPRB, 0.69951E+00_JPRB, 0.71941E+00_JPRB, 0.73508E+00_JPRB, 0.75510E+00_JPRB, 0.77898E+00_JPRB ,&
&  0.81308E+00_JPRB, 0.85648E+00_JPRB, 0.89646E+00_JPRB, 0.93739E+00_JPRB, 0.96716E+00_JPRB, 0.94045E+00_JPRB, 0.10029E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.10032E+01_JPRB, 0.84343E+00_JPRB, 0.78472E+00_JPRB, 0.77282E+00_JPRB, 0.77684E+00_JPRB, 0.78270E+00_JPRB ,&
&  0.79206E+00_JPRB, 0.79330E+00_JPRB, 0.78797E+00_JPRB, 0.78422E+00_JPRB, 0.78910E+00_JPRB, 0.79930E+00_JPRB, 0.82199E+00_JPRB ,&
&  0.85875E+00_JPRB, 0.10197E+01_JPRB, 0.12695E+01_JPRB, 0.98175E+00_JPRB, 0.87587E+00_JPRB, 0.79871E+00_JPRB, 0.73571E+00_JPRB ,&
&  0.71056E+00_JPRB, 0.70528E+00_JPRB, 0.71291E+00_JPRB, 0.72039E+00_JPRB, 0.72366E+00_JPRB, 0.73047E+00_JPRB, 0.74156E+00_JPRB ,&
&  0.75773E+00_JPRB, 0.77418E+00_JPRB, 0.78482E+00_JPRB, 0.79054E+00_JPRB, 0.78782E+00_JPRB, 0.77700E+00_JPRB, 0.76042E+00_JPRB ,&
&  0.75929E+00_JPRB, 0.76730E+00_JPRB, 0.77308E+00_JPRB, 0.77588E+00_JPRB, 0.77326E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.77484E+00_JPRB, 0.80840E+00_JPRB, 0.85145E+00_JPRB, 0.86108E+00_JPRB, 0.85528E+00_JPRB, 0.81319E+00_JPRB, 0.77083E+00_JPRB ,&
&  0.74675E+00_JPRB, 0.73811E+00_JPRB, 0.73444E+00_JPRB, 0.72330E+00_JPRB, 0.71215E+00_JPRB, 0.70626E+00_JPRB, 0.70188E+00_JPRB ,&
&  0.69803E+00_JPRB, 0.69405E+00_JPRB, 0.68843E+00_JPRB, 0.67623E+00_JPRB, 0.66906E+00_JPRB, 0.67075E+00_JPRB, 0.66947E+00_JPRB ,&
&  0.66426E+00_JPRB, 0.66257E+00_JPRB, 0.65067E+00_JPRB, 0.63452E+00_JPRB, 0.62686E+00_JPRB, 0.62588E+00_JPRB, 0.62815E+00_JPRB ,&
&  0.61938E+00_JPRB, 0.61258E+00_JPRB, 0.61890E+00_JPRB, 0.62718E+00_JPRB, 0.62088E+00_JPRB, 0.61195E+00_JPRB, 0.56597E+00_JPRB ,&
&  0.52083E+00_JPRB, 0.49314E+00_JPRB, 0.48626E+00_JPRB, 0.51521E+00_JPRB, 0.56690E+00_JPRB, 0.62942E+00_JPRB, 0.67288E+00_JPRB ,&
&  0.69625E+00_JPRB, 0.72784E+00_JPRB, 0.75682E+00_JPRB, 0.78736E+00_JPRB, 0.82544E+00_JPRB, 0.86693E+00_JPRB, 0.91533E+00_JPRB ,&
&  0.95358E+00_JPRB, 0.97685E+00_JPRB, 0.97552E+00_JPRB, 0.89470E+00_JPRB, 0.88182E+00_JPRB, 0.89144E+00_JPRB, 0.97308E+00_JPRB ,&
&  0.10568E+01_JPRB, 0.97602E+00_JPRB, 0.91743E+00_JPRB, 0.91355E+00_JPRB, 0.90419E+00_JPRB, 0.91498E+00_JPRB, 0.91843E+00_JPRB ,&
&  0.90378E+00_JPRB, 0.87758E+00_JPRB, 0.84955E+00_JPRB, 0.82786E+00_JPRB, 0.83077E+00_JPRB, 0.85426E+00_JPRB, 0.87944E+00_JPRB ,&
&  0.92956E+00_JPRB, 0.10363E+01_JPRB, 0.10134E+01_JPRB, 0.80311E+00_JPRB, 0.77093E+00_JPRB, 0.75490E+00_JPRB, 0.73743E+00_JPRB ,&
&  0.71074E+00_JPRB, 0.69490E+00_JPRB, 0.69454E+00_JPRB, 0.69686E+00_JPRB, 0.69358E+00_JPRB, 0.69031E+00_JPRB, 0.69287E+00_JPRB ,&
&  0.70059E+00_JPRB, 0.71479E+00_JPRB, 0.73324E+00_JPRB, 0.76130E+00_JPRB, 0.77467E+00_JPRB, 0.77215E+00_JPRB, 0.75653E+00_JPRB ,&
&  0.74976E+00_JPRB, 0.75444E+00_JPRB, 0.75715E+00_JPRB, 0.75609E+00_JPRB, 0.76018E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.88489E+00_JPRB, 0.89510E+00_JPRB, 0.89489E+00_JPRB, 0.89374E+00_JPRB, 0.85338E+00_JPRB, 0.83033E+00_JPRB, 0.79208E+00_JPRB ,&
&  0.78175E+00_JPRB, 0.75721E+00_JPRB, 0.72165E+00_JPRB, 0.68168E+00_JPRB, 0.65943E+00_JPRB, 0.65042E+00_JPRB, 0.65161E+00_JPRB ,&
&  0.65835E+00_JPRB, 0.66645E+00_JPRB, 0.66811E+00_JPRB, 0.65709E+00_JPRB, 0.65847E+00_JPRB, 0.66169E+00_JPRB, 0.66091E+00_JPRB ,&
&  0.65955E+00_JPRB, 0.66136E+00_JPRB, 0.65526E+00_JPRB, 0.64495E+00_JPRB, 0.63922E+00_JPRB, 0.63704E+00_JPRB, 0.63170E+00_JPRB ,&
&  0.62096E+00_JPRB, 0.61599E+00_JPRB, 0.62605E+00_JPRB, 0.64871E+00_JPRB, 0.63099E+00_JPRB, 0.62138E+00_JPRB, 0.56693E+00_JPRB ,&
&  0.52639E+00_JPRB, 0.51912E+00_JPRB, 0.55151E+00_JPRB, 0.61694E+00_JPRB, 0.65211E+00_JPRB, 0.68768E+00_JPRB, 0.70899E+00_JPRB ,&
&  0.70956E+00_JPRB, 0.72846E+00_JPRB, 0.77657E+00_JPRB, 0.86496E+00_JPRB, 0.90616E+00_JPRB, 0.95966E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10825E+01_JPRB, 0.10960E+01_JPRB, 0.10465E+01_JPRB, 0.10724E+01_JPRB, 0.11685E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.10190E+01_JPRB, 0.10237E+01_JPRB, 0.10766E+01_JPRB, 0.10314E+01_JPRB, 0.10489E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.96233E+00_JPRB, 0.92400E+00_JPRB, 0.89595E+00_JPRB, 0.88791E+00_JPRB, 0.88978E+00_JPRB, 0.90919E+00_JPRB, 0.94271E+00_JPRB ,&
&  0.99293E+00_JPRB, 0.10124E+01_JPRB, 0.94583E+00_JPRB, 0.81663E+00_JPRB, 0.76696E+00_JPRB, 0.74503E+00_JPRB, 0.72841E+00_JPRB ,&
&  0.71037E+00_JPRB, 0.69850E+00_JPRB, 0.69948E+00_JPRB, 0.69935E+00_JPRB, 0.69507E+00_JPRB, 0.68348E+00_JPRB, 0.67338E+00_JPRB ,&
&  0.66967E+00_JPRB, 0.67389E+00_JPRB, 0.67758E+00_JPRB, 0.70632E+00_JPRB, 0.73661E+00_JPRB, 0.75762E+00_JPRB, 0.75464E+00_JPRB ,&
&  0.75310E+00_JPRB, 0.76627E+00_JPRB, 0.78578E+00_JPRB, 0.82383E+00_JPRB, 0.85128E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.91948E+00_JPRB, 0.93415E+00_JPRB, 0.93959E+00_JPRB, 0.92665E+00_JPRB, 0.89350E+00_JPRB, 0.85079E+00_JPRB, 0.76653E+00_JPRB ,&
&  0.76133E+00_JPRB, 0.71555E+00_JPRB, 0.68171E+00_JPRB, 0.65639E+00_JPRB, 0.64770E+00_JPRB, 0.63550E+00_JPRB, 0.63169E+00_JPRB ,&
&  0.63222E+00_JPRB, 0.64694E+00_JPRB, 0.65537E+00_JPRB, 0.64852E+00_JPRB, 0.65254E+00_JPRB, 0.65726E+00_JPRB, 0.65627E+00_JPRB ,&
&  0.66129E+00_JPRB, 0.66722E+00_JPRB, 0.66941E+00_JPRB, 0.66485E+00_JPRB, 0.65168E+00_JPRB, 0.64262E+00_JPRB, 0.63278E+00_JPRB ,&
&  0.62648E+00_JPRB, 0.62466E+00_JPRB, 0.63246E+00_JPRB, 0.65100E+00_JPRB, 0.63914E+00_JPRB, 0.62536E+00_JPRB, 0.59730E+00_JPRB ,&
&  0.55160E+00_JPRB, 0.56978E+00_JPRB, 0.63039E+00_JPRB, 0.66184E+00_JPRB, 0.69417E+00_JPRB, 0.72799E+00_JPRB, 0.77715E+00_JPRB ,&
&  0.81855E+00_JPRB, 0.86242E+00_JPRB, 0.91226E+00_JPRB, 0.97218E+00_JPRB, 0.10434E+01_JPRB, 0.11098E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.12074E+01_JPRB, 0.10237E+01_JPRB, 0.10150E+01_JPRB, 0.11712E+01_JPRB, 0.14187E+01_JPRB, 0.12473E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.11546E+01_JPRB, 0.11034E+01_JPRB, 0.10819E+01_JPRB, 0.11437E+01_JPRB, 0.11560E+01_JPRB, 0.12037E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.11394E+01_JPRB, 0.10791E+01_JPRB, 0.10239E+01_JPRB, 0.10137E+01_JPRB, 0.10230E+01_JPRB, 0.10397E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.10184E+01_JPRB, 0.96386E+00_JPRB, 0.88777E+00_JPRB, 0.83259E+00_JPRB, 0.79959E+00_JPRB, 0.77388E+00_JPRB ,&
&  0.75180E+00_JPRB, 0.72890E+00_JPRB, 0.71153E+00_JPRB, 0.70313E+00_JPRB, 0.70166E+00_JPRB, 0.70976E+00_JPRB, 0.73505E+00_JPRB ,&
&  0.72030E+00_JPRB, 0.71227E+00_JPRB, 0.69909E+00_JPRB, 0.71881E+00_JPRB, 0.75603E+00_JPRB, 0.78869E+00_JPRB, 0.79867E+00_JPRB ,&
&  0.81951E+00_JPRB, 0.82998E+00_JPRB, 0.82611E+00_JPRB, 0.85205E+00_JPRB, 0.87634E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.94959E+00_JPRB, 0.96977E+00_JPRB, 0.97373E+00_JPRB, 0.97350E+00_JPRB, 0.93416E+00_JPRB, 0.86461E+00_JPRB, 0.80048E+00_JPRB ,&
&  0.85930E+00_JPRB, 0.78646E+00_JPRB, 0.78265E+00_JPRB, 0.68597E+00_JPRB, 0.65694E+00_JPRB, 0.64392E+00_JPRB, 0.62406E+00_JPRB ,&
&  0.60177E+00_JPRB, 0.61168E+00_JPRB, 0.62743E+00_JPRB, 0.63452E+00_JPRB, 0.64118E+00_JPRB, 0.65010E+00_JPRB, 0.64872E+00_JPRB ,&
&  0.65453E+00_JPRB, 0.66661E+00_JPRB, 0.68228E+00_JPRB, 0.67483E+00_JPRB, 0.66002E+00_JPRB, 0.65501E+00_JPRB, 0.64726E+00_JPRB ,&
&  0.63404E+00_JPRB, 0.62869E+00_JPRB, 0.63711E+00_JPRB, 0.65202E+00_JPRB, 0.65562E+00_JPRB, 0.63416E+00_JPRB, 0.61918E+00_JPRB ,&
&  0.61581E+00_JPRB, 0.62359E+00_JPRB, 0.66981E+00_JPRB, 0.70219E+00_JPRB, 0.74301E+00_JPRB, 0.79213E+00_JPRB, 0.84081E+00_JPRB ,&
&  0.89665E+00_JPRB, 0.95820E+00_JPRB, 0.10271E+01_JPRB, 0.11120E+01_JPRB, 0.11943E+01_JPRB, 0.12809E+01_JPRB, 0.13520E+01_JPRB ,&
&  0.14368E+01_JPRB, 0.11302E+01_JPRB, 0.13595E+01_JPRB, 0.14686E+01_JPRB, 0.16753E+01_JPRB, 0.16618E+01_JPRB, 0.15798E+01_JPRB ,&
&  0.16287E+01_JPRB, 0.14521E+01_JPRB, 0.13406E+01_JPRB, 0.12562E+01_JPRB, 0.12537E+01_JPRB, 0.13344E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.14934E+01_JPRB, 0.14475E+01_JPRB, 0.13575E+01_JPRB, 0.13266E+01_JPRB, 0.13142E+01_JPRB, 0.13020E+01_JPRB, 0.12648E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.11415E+01_JPRB, 0.10723E+01_JPRB, 0.98242E+00_JPRB, 0.93569E+00_JPRB, 0.89372E+00_JPRB, 0.84769E+00_JPRB ,&
&  0.82018E+00_JPRB, 0.77246E+00_JPRB, 0.72653E+00_JPRB, 0.69844E+00_JPRB, 0.67950E+00_JPRB, 0.67748E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.71173E+00_JPRB, 0.71551E+00_JPRB, 0.76629E+00_JPRB, 0.81096E+00_JPRB, 0.83717E+00_JPRB, 0.86200E+00_JPRB, 0.90807E+00_JPRB ,&
&  0.86040E+00_JPRB, 0.87083E+00_JPRB, 0.91929E+00_JPRB, 0.93000E+00_JPRB, 0.93743E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.10053E+01_JPRB, 0.10316E+01_JPRB, 0.10248E+01_JPRB, 0.10215E+01_JPRB, 0.95649E+00_JPRB, 0.92776E+00_JPRB, 0.85086E+00_JPRB ,&
&  0.86895E+00_JPRB, 0.83757E+00_JPRB, 0.77714E+00_JPRB, 0.75651E+00_JPRB, 0.76238E+00_JPRB, 0.68828E+00_JPRB, 0.62949E+00_JPRB ,&
&  0.59515E+00_JPRB, 0.58848E+00_JPRB, 0.61000E+00_JPRB, 0.62504E+00_JPRB, 0.63013E+00_JPRB, 0.63738E+00_JPRB, 0.63157E+00_JPRB ,&
&  0.65475E+00_JPRB, 0.66587E+00_JPRB, 0.68114E+00_JPRB, 0.68435E+00_JPRB, 0.67841E+00_JPRB, 0.67951E+00_JPRB, 0.68373E+00_JPRB ,&
&  0.66716E+00_JPRB, 0.66156E+00_JPRB, 0.64825E+00_JPRB, 0.64029E+00_JPRB, 0.68196E+00_JPRB, 0.67068E+00_JPRB, 0.65523E+00_JPRB ,&
&  0.65794E+00_JPRB, 0.67814E+00_JPRB, 0.70948E+00_JPRB, 0.75339E+00_JPRB, 0.80197E+00_JPRB, 0.85156E+00_JPRB, 0.90648E+00_JPRB ,&
&  0.95878E+00_JPRB, 0.10288E+01_JPRB, 0.10863E+01_JPRB, 0.11943E+01_JPRB, 0.12790E+01_JPRB, 0.12417E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.15525E+01_JPRB, 0.17061E+01_JPRB, 0.18656E+01_JPRB, 0.17560E+01_JPRB, 0.19733E+01_JPRB, 0.19288E+01_JPRB, 0.17736E+01_JPRB ,&
&  0.18081E+01_JPRB, 0.16821E+01_JPRB, 0.16511E+01_JPRB, 0.16481E+01_JPRB, 0.17365E+01_JPRB, 0.16892E+01_JPRB, 0.16669E+01_JPRB ,&
&  0.17488E+01_JPRB, 0.19894E+01_JPRB, 0.19009E+01_JPRB, 0.19279E+01_JPRB, 0.18693E+01_JPRB, 0.17998E+01_JPRB, 0.16613E+01_JPRB ,&
&  0.15496E+01_JPRB, 0.14198E+01_JPRB, 0.12981E+01_JPRB, 0.11999E+01_JPRB, 0.10842E+01_JPRB, 0.99666E+00_JPRB, 0.92712E+00_JPRB ,&
&  0.85389E+00_JPRB, 0.76482E+00_JPRB, 0.72898E+00_JPRB, 0.70458E+00_JPRB, 0.69104E+00_JPRB, 0.66471E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.70310E+00_JPRB, 0.76586E+00_JPRB, 0.86105E+00_JPRB, 0.91786E+00_JPRB, 0.99176E+00_JPRB, 0.10640E+01_JPRB, 0.98491E+00_JPRB ,&
&  0.91296E+00_JPRB, 0.98531E+00_JPRB, 0.10003E+01_JPRB, 0.10034E+01_JPRB, 0.10010E+01_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.11362E+01_JPRB, 0.11465E+01_JPRB, 0.11243E+01_JPRB, 0.10858E+01_JPRB, 0.99605E+00_JPRB, 0.95271E+00_JPRB, 0.89354E+00_JPRB ,&
&  0.89194E+00_JPRB, 0.85535E+00_JPRB, 0.79663E+00_JPRB, 0.77445E+00_JPRB, 0.77365E+00_JPRB, 0.75581E+00_JPRB, 0.68401E+00_JPRB ,&
&  0.63087E+00_JPRB, 0.61845E+00_JPRB, 0.60565E+00_JPRB, 0.62118E+00_JPRB, 0.64181E+00_JPRB, 0.63945E+00_JPRB, 0.65188E+00_JPRB ,&
&  0.67939E+00_JPRB, 0.68731E+00_JPRB, 0.68714E+00_JPRB, 0.68673E+00_JPRB, 0.68916E+00_JPRB, 0.70061E+00_JPRB, 0.70516E+00_JPRB ,&
&  0.68456E+00_JPRB, 0.66739E+00_JPRB, 0.65036E+00_JPRB, 0.64925E+00_JPRB, 0.69433E+00_JPRB, 0.71217E+00_JPRB, 0.70682E+00_JPRB ,&
&  0.70782E+00_JPRB, 0.71680E+00_JPRB, 0.73633E+00_JPRB, 0.77997E+00_JPRB, 0.83468E+00_JPRB, 0.89394E+00_JPRB, 0.95980E+00_JPRB ,&
&  0.10289E+01_JPRB, 0.11075E+01_JPRB, 0.11742E+01_JPRB, 0.12556E+01_JPRB, 0.12488E+01_JPRB, 0.13416E+01_JPRB, 0.15513E+01_JPRB ,&
&  0.16768E+01_JPRB, 0.18311E+01_JPRB, 0.19505E+01_JPRB, 0.21759E+01_JPRB, 0.23571E+01_JPRB, 0.22439E+01_JPRB, 0.21075E+01_JPRB ,&
&  0.20744E+01_JPRB, 0.20109E+01_JPRB, 0.24984E+01_JPRB, 0.23140E+01_JPRB, 0.21367E+01_JPRB, 0.20970E+01_JPRB, 0.20314E+01_JPRB ,&
&  0.20896E+01_JPRB, 0.22214E+01_JPRB, 0.22984E+01_JPRB, 0.23582E+01_JPRB, 0.23630E+01_JPRB, 0.23615E+01_JPRB, 0.22231E+01_JPRB ,&
&  0.21254E+01_JPRB, 0.19393E+01_JPRB, 0.15547E+01_JPRB, 0.12672E+01_JPRB, 0.10488E+01_JPRB, 0.10354E+01_JPRB, 0.96197E+00_JPRB ,&
&  0.86629E+00_JPRB, 0.76862E+00_JPRB, 0.75011E+00_JPRB, 0.72746E+00_JPRB, 0.72283E+00_JPRB, 0.70845E+00_JPRB, 0.69118E+00_JPRB ,&
&  0.72051E+00_JPRB, 0.81733E+00_JPRB, 0.88985E+00_JPRB, 0.10105E+01_JPRB, 0.11485E+01_JPRB, 0.12670E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10883E+01_JPRB, 0.11817E+01_JPRB, 0.12003E+01_JPRB, 0.11933E+01_JPRB, 0.11622E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.13792E+01_JPRB, 0.13310E+01_JPRB, 0.12666E+01_JPRB, 0.11688E+01_JPRB, 0.10460E+01_JPRB, 0.97031E+00_JPRB, 0.90557E+00_JPRB ,&
&  0.89302E+00_JPRB, 0.85846E+00_JPRB, 0.80877E+00_JPRB, 0.78952E+00_JPRB, 0.78357E+00_JPRB, 0.77602E+00_JPRB, 0.76828E+00_JPRB ,&
&  0.68746E+00_JPRB, 0.67879E+00_JPRB, 0.64401E+00_JPRB, 0.65779E+00_JPRB, 0.72580E+00_JPRB, 0.67973E+00_JPRB, 0.68775E+00_JPRB ,&
&  0.69407E+00_JPRB, 0.70893E+00_JPRB, 0.69854E+00_JPRB, 0.69849E+00_JPRB, 0.71642E+00_JPRB, 0.73598E+00_JPRB, 0.73922E+00_JPRB ,&
&  0.71761E+00_JPRB, 0.69177E+00_JPRB, 0.68269E+00_JPRB, 0.68642E+00_JPRB, 0.73064E+00_JPRB, 0.76481E+00_JPRB, 0.78225E+00_JPRB ,&
&  0.78945E+00_JPRB, 0.79337E+00_JPRB, 0.79921E+00_JPRB, 0.82703E+00_JPRB, 0.87277E+00_JPRB, 0.92923E+00_JPRB, 0.10135E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.11891E+01_JPRB, 0.12782E+01_JPRB, 0.13684E+01_JPRB, 0.13765E+01_JPRB, 0.15369E+01_JPRB, 0.17427E+01_JPRB ,&
&  0.19209E+01_JPRB, 0.18899E+01_JPRB, 0.19402E+01_JPRB, 0.24317E+01_JPRB, 0.27819E+01_JPRB, 0.27692E+01_JPRB, 0.26017E+01_JPRB ,&
&  0.24305E+01_JPRB, 0.23031E+01_JPRB, 0.24254E+01_JPRB, 0.24110E+01_JPRB, 0.22497E+01_JPRB, 0.23268E+01_JPRB, 0.23251E+01_JPRB ,&
&  0.23612E+01_JPRB, 0.24864E+01_JPRB, 0.26531E+01_JPRB, 0.27695E+01_JPRB, 0.26935E+01_JPRB, 0.27342E+01_JPRB, 0.26458E+01_JPRB ,&
&  0.25363E+01_JPRB, 0.21579E+01_JPRB, 0.15646E+01_JPRB, 0.12578E+01_JPRB, 0.95441E+00_JPRB, 0.91578E+00_JPRB, 0.10413E+01_JPRB ,&
&  0.10076E+01_JPRB, 0.94911E+00_JPRB, 0.87871E+00_JPRB, 0.83194E+00_JPRB, 0.80361E+00_JPRB, 0.82889E+00_JPRB, 0.83846E+00_JPRB ,&
&  0.84888E+00_JPRB, 0.96893E+00_JPRB, 0.11699E+01_JPRB, 0.13933E+01_JPRB, 0.15231E+01_JPRB, 0.15695E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.15201E+01_JPRB, 0.14947E+01_JPRB, 0.14912E+01_JPRB, 0.14940E+01_JPRB, 0.14537E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.16227E+01_JPRB, 0.15398E+01_JPRB, 0.14125E+01_JPRB, 0.12514E+01_JPRB, 0.10771E+01_JPRB, 0.97017E+00_JPRB, 0.92348E+00_JPRB ,&
&  0.90892E+00_JPRB, 0.87800E+00_JPRB, 0.82956E+00_JPRB, 0.80463E+00_JPRB, 0.79484E+00_JPRB, 0.78537E+00_JPRB, 0.79096E+00_JPRB ,&
&  0.81978E+00_JPRB, 0.80116E+00_JPRB, 0.71799E+00_JPRB, 0.66302E+00_JPRB, 0.69433E+00_JPRB, 0.68934E+00_JPRB, 0.70270E+00_JPRB ,&
&  0.70716E+00_JPRB, 0.73911E+00_JPRB, 0.71502E+00_JPRB, 0.72857E+00_JPRB, 0.77905E+00_JPRB, 0.81345E+00_JPRB, 0.83277E+00_JPRB ,&
&  0.83000E+00_JPRB, 0.81236E+00_JPRB, 0.80459E+00_JPRB, 0.82922E+00_JPRB, 0.87817E+00_JPRB, 0.85654E+00_JPRB, 0.88505E+00_JPRB ,&
&  0.88519E+00_JPRB, 0.89039E+00_JPRB, 0.88210E+00_JPRB, 0.90974E+00_JPRB, 0.96890E+00_JPRB, 0.10658E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12671E+01_JPRB, 0.13029E+01_JPRB, 0.13990E+01_JPRB, 0.15379E+01_JPRB, 0.17149E+01_JPRB, 0.17249E+01_JPRB ,&
&  0.18082E+01_JPRB, 0.17924E+01_JPRB, 0.17758E+01_JPRB, 0.22158E+01_JPRB, 0.26684E+01_JPRB, 0.28057E+01_JPRB, 0.28272E+01_JPRB ,&
&  0.27885E+01_JPRB, 0.27229E+01_JPRB, 0.28233E+01_JPRB, 0.27302E+01_JPRB, 0.26579E+01_JPRB, 0.27028E+01_JPRB, 0.27771E+01_JPRB ,&
&  0.27501E+01_JPRB, 0.28606E+01_JPRB, 0.30401E+01_JPRB, 0.30408E+01_JPRB, 0.29966E+01_JPRB, 0.28803E+01_JPRB, 0.28888E+01_JPRB ,&
&  0.28212E+01_JPRB, 0.17438E+01_JPRB, 0.13692E+01_JPRB, 0.14160E+01_JPRB, 0.10769E+01_JPRB, 0.94051E+00_JPRB, 0.99385E+00_JPRB ,&
&  0.95557E+00_JPRB, 0.94197E+00_JPRB, 0.97516E+00_JPRB, 0.98496E+00_JPRB, 0.99168E+00_JPRB, 0.10955E+01_JPRB, 0.11922E+01_JPRB ,&
&  0.13200E+01_JPRB, 0.13720E+01_JPRB, 0.17006E+01_JPRB, 0.19107E+01_JPRB, 0.19588E+01_JPRB, 0.19440E+01_JPRB, 0.18995E+01_JPRB ,&
&  0.18432E+01_JPRB, 0.17841E+01_JPRB, 0.17450E+01_JPRB, 0.17263E+01_JPRB, 0.16929E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.18567E+01_JPRB, 0.17584E+01_JPRB, 0.15887E+01_JPRB, 0.13785E+01_JPRB, 0.11522E+01_JPRB, 0.10169E+01_JPRB, 0.99087E+00_JPRB ,&
&  0.97446E+00_JPRB, 0.94958E+00_JPRB, 0.90263E+00_JPRB, 0.86429E+00_JPRB, 0.84675E+00_JPRB, 0.83141E+00_JPRB, 0.83141E+00_JPRB ,&
&  0.83065E+00_JPRB, 0.84350E+00_JPRB, 0.73414E+00_JPRB, 0.65461E+00_JPRB, 0.61169E+00_JPRB, 0.63891E+00_JPRB, 0.66738E+00_JPRB ,&
&  0.70838E+00_JPRB, 0.75193E+00_JPRB, 0.76642E+00_JPRB, 0.85270E+00_JPRB, 0.93778E+00_JPRB, 0.98165E+00_JPRB, 0.10436E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.10322E+01_JPRB, 0.99204E+00_JPRB, 0.10353E+01_JPRB, 0.10319E+01_JPRB, 0.97310E+00_JPRB, 0.10218E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.10311E+01_JPRB, 0.10355E+01_JPRB, 0.10765E+01_JPRB, 0.11351E+01_JPRB, 0.11643E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.11787E+01_JPRB, 0.12369E+01_JPRB, 0.13256E+01_JPRB, 0.14141E+01_JPRB, 0.15250E+01_JPRB, 0.16415E+01_JPRB, 0.16323E+01_JPRB ,&
&  0.16977E+01_JPRB, 0.15471E+01_JPRB, 0.14043E+01_JPRB, 0.17816E+01_JPRB, 0.25247E+01_JPRB, 0.28639E+01_JPRB, 0.27777E+01_JPRB ,&
&  0.30136E+01_JPRB, 0.31054E+01_JPRB, 0.32215E+01_JPRB, 0.31642E+01_JPRB, 0.30663E+01_JPRB, 0.30733E+01_JPRB, 0.32189E+01_JPRB ,&
&  0.32609E+01_JPRB, 0.34123E+01_JPRB, 0.34963E+01_JPRB, 0.34783E+01_JPRB, 0.34046E+01_JPRB, 0.34263E+01_JPRB, 0.34631E+01_JPRB ,&
&  0.31743E+01_JPRB, 0.26672E+01_JPRB, 0.18683E+01_JPRB, 0.17768E+01_JPRB, 0.12619E+01_JPRB, 0.10430E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.97139E+00_JPRB, 0.99639E+00_JPRB, 0.10784E+01_JPRB, 0.11380E+01_JPRB, 0.13637E+01_JPRB, 0.15960E+01_JPRB, 0.18690E+01_JPRB ,&
&  0.20469E+01_JPRB, 0.19929E+01_JPRB, 0.23261E+01_JPRB, 0.23854E+01_JPRB, 0.23821E+01_JPRB, 0.23857E+01_JPRB, 0.23733E+01_JPRB ,&
&  0.22962E+01_JPRB, 0.21884E+01_JPRB, 0.20972E+01_JPRB, 0.20088E+01_JPRB, 0.19335E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.19821E+01_JPRB, 0.18557E+01_JPRB, 0.16533E+01_JPRB, 0.14322E+01_JPRB, 0.12077E+01_JPRB, 0.10551E+01_JPRB, 0.10354E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.10696E+01_JPRB, 0.10423E+01_JPRB, 0.98787E+00_JPRB, 0.95020E+00_JPRB, 0.92480E+00_JPRB, 0.89903E+00_JPRB ,&
&  0.87412E+00_JPRB, 0.80668E+00_JPRB, 0.68259E+00_JPRB, 0.61141E+00_JPRB, 0.59033E+00_JPRB, 0.58956E+00_JPRB, 0.63525E+00_JPRB ,&
&  0.70013E+00_JPRB, 0.81623E+00_JPRB, 0.93282E+00_JPRB, 0.10710E+01_JPRB, 0.12412E+01_JPRB, 0.14007E+01_JPRB, 0.14433E+01_JPRB ,&
&  0.14580E+01_JPRB, 0.12612E+01_JPRB, 0.12681E+01_JPRB, 0.12279E+01_JPRB, 0.12334E+01_JPRB, 0.12058E+01_JPRB, 0.11873E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.11477E+01_JPRB, 0.11495E+01_JPRB, 0.11595E+01_JPRB, 0.11661E+01_JPRB, 0.11709E+01_JPRB, 0.11781E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.13217E+01_JPRB, 0.13933E+01_JPRB, 0.14558E+01_JPRB, 0.15540E+01_JPRB, 0.16798E+01_JPRB, 0.17248E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.16360E+01_JPRB, 0.18734E+01_JPRB, 0.21149E+01_JPRB, 0.24118E+01_JPRB, 0.28256E+01_JPRB, 0.30646E+01_JPRB ,&
&  0.32159E+01_JPRB, 0.31343E+01_JPRB, 0.31765E+01_JPRB, 0.29985E+01_JPRB, 0.33944E+01_JPRB, 0.34474E+01_JPRB, 0.35822E+01_JPRB ,&
&  0.37160E+01_JPRB, 0.39629E+01_JPRB, 0.38571E+01_JPRB, 0.37623E+01_JPRB, 0.38643E+01_JPRB, 0.41316E+01_JPRB, 0.37241E+01_JPRB ,&
&  0.31511E+01_JPRB, 0.34054E+01_JPRB, 0.29851E+01_JPRB, 0.24535E+01_JPRB, 0.18730E+01_JPRB, 0.16045E+01_JPRB, 0.13881E+01_JPRB ,&
&  0.12386E+01_JPRB, 0.11774E+01_JPRB, 0.12464E+01_JPRB, 0.15331E+01_JPRB, 0.20050E+01_JPRB, 0.23920E+01_JPRB, 0.26309E+01_JPRB ,&
&  0.26911E+01_JPRB, 0.32264E+01_JPRB, 0.31733E+01_JPRB, 0.30245E+01_JPRB, 0.29537E+01_JPRB, 0.29276E+01_JPRB, 0.28411E+01_JPRB ,&
&  0.26386E+01_JPRB, 0.24712E+01_JPRB, 0.23307E+01_JPRB, 0.22064E+01_JPRB, 0.20834E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.20369E+01_JPRB, 0.19217E+01_JPRB, 0.17219E+01_JPRB, 0.14855E+01_JPRB, 0.12716E+01_JPRB, 0.11269E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.11486E+01_JPRB, 0.11609E+01_JPRB, 0.11559E+01_JPRB, 0.11308E+01_JPRB, 0.10817E+01_JPRB, 0.10244E+01_JPRB ,&
&  0.97827E+00_JPRB, 0.81582E+00_JPRB, 0.67654E+00_JPRB, 0.63055E+00_JPRB, 0.59870E+00_JPRB, 0.58535E+00_JPRB, 0.61602E+00_JPRB ,&
&  0.72269E+00_JPRB, 0.97200E+00_JPRB, 0.12451E+01_JPRB, 0.15366E+01_JPRB, 0.17792E+01_JPRB, 0.19235E+01_JPRB, 0.18538E+01_JPRB ,&
&  0.16918E+01_JPRB, 0.16433E+01_JPRB, 0.15808E+01_JPRB, 0.14860E+01_JPRB, 0.14295E+01_JPRB, 0.13635E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.12623E+01_JPRB, 0.12288E+01_JPRB, 0.12449E+01_JPRB, 0.12595E+01_JPRB, 0.12735E+01_JPRB, 0.12900E+01_JPRB, 0.13130E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.14013E+01_JPRB, 0.14497E+01_JPRB, 0.15555E+01_JPRB, 0.14221E+01_JPRB, 0.15065E+01_JPRB, 0.16648E+01_JPRB ,&
&  0.19909E+01_JPRB, 0.20347E+01_JPRB, 0.19880E+01_JPRB, 0.21807E+01_JPRB, 0.20882E+01_JPRB, 0.23167E+01_JPRB, 0.28739E+01_JPRB ,&
&  0.30656E+01_JPRB, 0.30828E+01_JPRB, 0.32565E+01_JPRB, 0.28370E+01_JPRB, 0.27949E+01_JPRB, 0.39958E+01_JPRB, 0.42986E+01_JPRB ,&
&  0.43727E+01_JPRB, 0.44084E+01_JPRB, 0.42447E+01_JPRB, 0.43294E+01_JPRB, 0.38388E+01_JPRB, 0.34054E+01_JPRB, 0.26294E+01_JPRB ,&
&  0.28222E+01_JPRB, 0.27405E+01_JPRB, 0.30157E+01_JPRB, 0.27374E+01_JPRB, 0.25485E+01_JPRB, 0.22485E+01_JPRB, 0.18429E+01_JPRB ,&
&  0.16048E+01_JPRB, 0.15088E+01_JPRB, 0.16382E+01_JPRB, 0.22959E+01_JPRB, 0.31110E+01_JPRB, 0.31441E+01_JPRB, 0.37691E+01_JPRB ,&
&  0.41045E+01_JPRB, 0.41692E+01_JPRB, 0.36249E+01_JPRB, 0.35227E+01_JPRB, 0.33112E+01_JPRB, 0.31114E+01_JPRB, 0.28739E+01_JPRB ,&
&  0.26781E+01_JPRB, 0.25323E+01_JPRB, 0.23967E+01_JPRB, 0.22607E+01_JPRB, 0.21318E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.20817E+01_JPRB, 0.19784E+01_JPRB, 0.17837E+01_JPRB, 0.15459E+01_JPRB, 0.13388E+01_JPRB, 0.12097E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.12099E+01_JPRB, 0.12361E+01_JPRB, 0.12470E+01_JPRB, 0.12378E+01_JPRB, 0.12166E+01_JPRB, 0.11921E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.92424E+00_JPRB, 0.72806E+00_JPRB, 0.65036E+00_JPRB, 0.65125E+00_JPRB, 0.62735E+00_JPRB, 0.64407E+00_JPRB ,&
&  0.86068E+00_JPRB, 0.12683E+01_JPRB, 0.17829E+01_JPRB, 0.21856E+01_JPRB, 0.24253E+01_JPRB, 0.23773E+01_JPRB, 0.21756E+01_JPRB ,&
&  0.21031E+01_JPRB, 0.19509E+01_JPRB, 0.17353E+01_JPRB, 0.15593E+01_JPRB, 0.14504E+01_JPRB, 0.13847E+01_JPRB, 0.13192E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12363E+01_JPRB, 0.12401E+01_JPRB, 0.12438E+01_JPRB, 0.12653E+01_JPRB, 0.12716E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.14016E+01_JPRB, 0.14954E+01_JPRB, 0.16235E+01_JPRB, 0.15572E+01_JPRB, 0.16348E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.20796E+01_JPRB, 0.21234E+01_JPRB, 0.21069E+01_JPRB, 0.21368E+01_JPRB, 0.22743E+01_JPRB, 0.23323E+01_JPRB, 0.27582E+01_JPRB ,&
&  0.26612E+01_JPRB, 0.25254E+01_JPRB, 0.25047E+01_JPRB, 0.24021E+01_JPRB, 0.30478E+01_JPRB, 0.43506E+01_JPRB, 0.49947E+01_JPRB ,&
&  0.49622E+01_JPRB, 0.48816E+01_JPRB, 0.48077E+01_JPRB, 0.43640E+01_JPRB, 0.35656E+01_JPRB, 0.27826E+01_JPRB, 0.33234E+01_JPRB ,&
&  0.36054E+01_JPRB, 0.37928E+01_JPRB, 0.35987E+01_JPRB, 0.29644E+01_JPRB, 0.25710E+01_JPRB, 0.28693E+01_JPRB, 0.25854E+01_JPRB ,&
&  0.21510E+01_JPRB, 0.20366E+01_JPRB, 0.27356E+01_JPRB, 0.36706E+01_JPRB, 0.41187E+01_JPRB, 0.41974E+01_JPRB, 0.47834E+01_JPRB ,&
&  0.46160E+01_JPRB, 0.44205E+01_JPRB, 0.40594E+01_JPRB, 0.36392E+01_JPRB, 0.32899E+01_JPRB, 0.30015E+01_JPRB, 0.27696E+01_JPRB ,&
&  0.26384E+01_JPRB, 0.25525E+01_JPRB, 0.24372E+01_JPRB, 0.23191E+01_JPRB, 0.21848E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.21250E+01_JPRB, 0.20154E+01_JPRB, 0.18116E+01_JPRB, 0.15702E+01_JPRB, 0.14031E+01_JPRB, 0.12997E+01_JPRB, 0.12589E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.12676E+01_JPRB, 0.12680E+01_JPRB, 0.12563E+01_JPRB, 0.12413E+01_JPRB, 0.12244E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.11148E+01_JPRB, 0.85816E+00_JPRB, 0.74594E+00_JPRB, 0.72561E+00_JPRB, 0.74313E+00_JPRB, 0.76790E+00_JPRB ,&
&  0.11442E+01_JPRB, 0.18581E+01_JPRB, 0.24635E+01_JPRB, 0.28348E+01_JPRB, 0.29260E+01_JPRB, 0.24790E+01_JPRB, 0.23134E+01_JPRB ,&
&  0.22717E+01_JPRB, 0.21175E+01_JPRB, 0.18932E+01_JPRB, 0.16848E+01_JPRB, 0.15105E+01_JPRB, 0.14150E+01_JPRB, 0.13594E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.12771E+01_JPRB, 0.12702E+01_JPRB, 0.12900E+01_JPRB, 0.13004E+01_JPRB, 0.13090E+01_JPRB, 0.13358E+01_JPRB ,&
&  0.13786E+01_JPRB, 0.14413E+01_JPRB, 0.15190E+01_JPRB, 0.16950E+01_JPRB, 0.17552E+01_JPRB, 0.19246E+01_JPRB, 0.23215E+01_JPRB ,&
&  0.23567E+01_JPRB, 0.24292E+01_JPRB, 0.23237E+01_JPRB, 0.22832E+01_JPRB, 0.23075E+01_JPRB, 0.23667E+01_JPRB, 0.27402E+01_JPRB ,&
&  0.26964E+01_JPRB, 0.27095E+01_JPRB, 0.29438E+01_JPRB, 0.31225E+01_JPRB, 0.33500E+01_JPRB, 0.40280E+01_JPRB, 0.57113E+01_JPRB ,&
&  0.50068E+01_JPRB, 0.50210E+01_JPRB, 0.50359E+01_JPRB, 0.48174E+01_JPRB, 0.26961E+01_JPRB, 0.21225E+01_JPRB, 0.32658E+01_JPRB ,&
&  0.34492E+01_JPRB, 0.21619E+01_JPRB, 0.15783E+01_JPRB, 0.21080E+01_JPRB, 0.16594E+01_JPRB, 0.19874E+01_JPRB, 0.27882E+01_JPRB ,&
&  0.29206E+01_JPRB, 0.27101E+01_JPRB, 0.35653E+01_JPRB, 0.40799E+01_JPRB, 0.42305E+01_JPRB, 0.52292E+01_JPRB, 0.52669E+01_JPRB ,&
&  0.50206E+01_JPRB, 0.41387E+01_JPRB, 0.40252E+01_JPRB, 0.35707E+01_JPRB, 0.32194E+01_JPRB, 0.30068E+01_JPRB, 0.28373E+01_JPRB ,&
&  0.26432E+01_JPRB, 0.24618E+01_JPRB, 0.23454E+01_JPRB, 0.22901E+01_JPRB, 0.22127E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.21217E+01_JPRB, 0.20094E+01_JPRB, 0.17977E+01_JPRB, 0.15632E+01_JPRB, 0.14296E+01_JPRB, 0.13386E+01_JPRB, 0.12877E+01_JPRB ,&
&  0.12784E+01_JPRB, 0.12800E+01_JPRB, 0.12732E+01_JPRB, 0.12615E+01_JPRB, 0.12511E+01_JPRB, 0.12486E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.11366E+01_JPRB, 0.10031E+01_JPRB, 0.90322E+00_JPRB, 0.88631E+00_JPRB, 0.90426E+00_JPRB, 0.10299E+01_JPRB ,&
&  0.14688E+01_JPRB, 0.23959E+01_JPRB, 0.30523E+01_JPRB, 0.32061E+01_JPRB, 0.28272E+01_JPRB, 0.24465E+01_JPRB, 0.24723E+01_JPRB ,&
&  0.24061E+01_JPRB, 0.22152E+01_JPRB, 0.19664E+01_JPRB, 0.17843E+01_JPRB, 0.16306E+01_JPRB, 0.15273E+01_JPRB, 0.14675E+01_JPRB ,&
&  0.13990E+01_JPRB, 0.13566E+01_JPRB, 0.13569E+01_JPRB, 0.13571E+01_JPRB, 0.13584E+01_JPRB, 0.13694E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.14374E+01_JPRB, 0.15074E+01_JPRB, 0.16139E+01_JPRB, 0.17967E+01_JPRB, 0.21918E+01_JPRB, 0.24527E+01_JPRB, 0.23827E+01_JPRB ,&
&  0.22511E+01_JPRB, 0.23011E+01_JPRB, 0.25413E+01_JPRB, 0.23570E+01_JPRB, 0.22846E+01_JPRB, 0.23518E+01_JPRB, 0.26511E+01_JPRB ,&
&  0.31150E+01_JPRB, 0.33784E+01_JPRB, 0.31881E+01_JPRB, 0.33710E+01_JPRB, 0.36695E+01_JPRB, 0.35082E+01_JPRB, 0.48155E+01_JPRB ,&
&  0.52935E+01_JPRB, 0.46461E+01_JPRB, 0.37199E+01_JPRB, 0.29929E+01_JPRB, 0.32183E+01_JPRB, 0.25889E+01_JPRB, 0.15868E+01_JPRB ,&
&  0.13769E+01_JPRB, 0.12692E+01_JPRB, 0.11512E+01_JPRB, 0.12483E+01_JPRB, 0.12441E+01_JPRB, 0.17232E+01_JPRB, 0.26884E+01_JPRB ,&
&  0.35563E+01_JPRB, 0.37070E+01_JPRB, 0.41396E+01_JPRB, 0.42659E+01_JPRB, 0.42157E+01_JPRB, 0.52712E+01_JPRB, 0.53931E+01_JPRB ,&
&  0.43232E+01_JPRB, 0.37836E+01_JPRB, 0.36999E+01_JPRB, 0.33102E+01_JPRB, 0.31150E+01_JPRB, 0.29592E+01_JPRB, 0.27474E+01_JPRB ,&
&  0.24863E+01_JPRB, 0.22544E+01_JPRB, 0.21584E+01_JPRB, 0.21666E+01_JPRB, 0.21874E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.20973E+01_JPRB, 0.19830E+01_JPRB, 0.17961E+01_JPRB, 0.15948E+01_JPRB, 0.14879E+01_JPRB, 0.14153E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.13369E+01_JPRB, 0.13155E+01_JPRB, 0.13037E+01_JPRB, 0.12993E+01_JPRB, 0.12979E+01_JPRB, 0.13005E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.11797E+01_JPRB, 0.11086E+01_JPRB, 0.10476E+01_JPRB, 0.10769E+01_JPRB, 0.11053E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.19239E+01_JPRB, 0.29428E+01_JPRB, 0.35053E+01_JPRB, 0.34044E+01_JPRB, 0.30404E+01_JPRB, 0.27504E+01_JPRB, 0.25820E+01_JPRB ,&
&  0.23762E+01_JPRB, 0.22147E+01_JPRB, 0.20328E+01_JPRB, 0.18779E+01_JPRB, 0.17516E+01_JPRB, 0.16522E+01_JPRB, 0.15702E+01_JPRB ,&
&  0.14969E+01_JPRB, 0.14773E+01_JPRB, 0.14486E+01_JPRB, 0.14378E+01_JPRB, 0.14282E+01_JPRB, 0.14264E+01_JPRB, 0.14462E+01_JPRB ,&
&  0.15066E+01_JPRB, 0.16165E+01_JPRB, 0.18146E+01_JPRB, 0.21473E+01_JPRB, 0.22777E+01_JPRB, 0.20860E+01_JPRB, 0.22378E+01_JPRB ,&
&  0.23308E+01_JPRB, 0.24902E+01_JPRB, 0.24212E+01_JPRB, 0.24885E+01_JPRB, 0.23505E+01_JPRB, 0.22960E+01_JPRB, 0.26190E+01_JPRB ,&
&  0.31675E+01_JPRB, 0.38101E+01_JPRB, 0.32956E+01_JPRB, 0.31885E+01_JPRB, 0.40898E+01_JPRB, 0.39524E+01_JPRB, 0.44306E+01_JPRB ,&
&  0.50298E+01_JPRB, 0.43443E+01_JPRB, 0.37855E+01_JPRB, 0.34454E+01_JPRB, 0.48037E+01_JPRB, 0.50641E+01_JPRB, 0.24820E+01_JPRB ,&
&  0.17283E+01_JPRB, 0.14059E+01_JPRB, 0.12408E+01_JPRB, 0.11690E+01_JPRB, 0.11950E+01_JPRB, 0.17573E+01_JPRB, 0.32300E+01_JPRB ,&
&  0.36799E+01_JPRB, 0.44383E+01_JPRB, 0.47688E+01_JPRB, 0.47384E+01_JPRB, 0.45823E+01_JPRB, 0.46987E+01_JPRB, 0.42085E+01_JPRB ,&
&  0.41769E+01_JPRB, 0.38290E+01_JPRB, 0.34125E+01_JPRB, 0.30638E+01_JPRB, 0.27907E+01_JPRB, 0.24823E+01_JPRB, 0.22254E+01_JPRB ,&
&  0.20520E+01_JPRB, 0.19936E+01_JPRB, 0.19859E+01_JPRB, 0.20466E+01_JPRB, 0.21461E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.20693E+01_JPRB, 0.20070E+01_JPRB, 0.18434E+01_JPRB, 0.16626E+01_JPRB, 0.15602E+01_JPRB, 0.14912E+01_JPRB, 0.14460E+01_JPRB ,&
&  0.13984E+01_JPRB, 0.13561E+01_JPRB, 0.13252E+01_JPRB, 0.13114E+01_JPRB, 0.13174E+01_JPRB, 0.13152E+01_JPRB, 0.13050E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.12087E+01_JPRB, 0.11384E+01_JPRB, 0.12039E+01_JPRB, 0.12678E+01_JPRB, 0.13944E+01_JPRB, 0.17110E+01_JPRB ,&
&  0.25657E+01_JPRB, 0.36746E+01_JPRB, 0.38613E+01_JPRB, 0.36419E+01_JPRB, 0.32611E+01_JPRB, 0.29219E+01_JPRB, 0.24051E+01_JPRB ,&
&  0.21220E+01_JPRB, 0.20715E+01_JPRB, 0.19504E+01_JPRB, 0.19096E+01_JPRB, 0.18517E+01_JPRB, 0.17558E+01_JPRB, 0.16611E+01_JPRB ,&
&  0.15926E+01_JPRB, 0.15340E+01_JPRB, 0.15066E+01_JPRB, 0.14912E+01_JPRB, 0.14771E+01_JPRB, 0.14715E+01_JPRB, 0.15261E+01_JPRB ,&
&  0.16392E+01_JPRB, 0.17885E+01_JPRB, 0.20582E+01_JPRB, 0.23151E+01_JPRB, 0.19764E+01_JPRB, 0.21543E+01_JPRB, 0.22172E+01_JPRB ,&
&  0.22410E+01_JPRB, 0.22746E+01_JPRB, 0.21572E+01_JPRB, 0.21980E+01_JPRB, 0.23012E+01_JPRB, 0.23072E+01_JPRB, 0.25448E+01_JPRB ,&
&  0.34128E+01_JPRB, 0.32017E+01_JPRB, 0.29536E+01_JPRB, 0.30783E+01_JPRB, 0.38182E+01_JPRB, 0.57569E+01_JPRB, 0.41172E+01_JPRB ,&
&  0.38104E+01_JPRB, 0.43805E+01_JPRB, 0.40906E+01_JPRB, 0.41502E+01_JPRB, 0.54094E+01_JPRB, 0.69771E+01_JPRB, 0.65631E+01_JPRB ,&
&  0.25545E+01_JPRB, 0.16424E+01_JPRB, 0.16022E+01_JPRB, 0.15652E+01_JPRB, 0.14649E+01_JPRB, 0.18773E+01_JPRB, 0.51519E+01_JPRB ,&
&  0.44493E+01_JPRB, 0.51622E+01_JPRB, 0.51759E+01_JPRB, 0.47811E+01_JPRB, 0.46909E+01_JPRB, 0.46084E+01_JPRB, 0.41824E+01_JPRB ,&
&  0.37085E+01_JPRB, 0.32753E+01_JPRB, 0.28575E+01_JPRB, 0.24438E+01_JPRB, 0.21163E+01_JPRB, 0.19018E+01_JPRB, 0.18110E+01_JPRB ,&
&  0.17913E+01_JPRB, 0.18179E+01_JPRB, 0.18671E+01_JPRB, 0.19484E+01_JPRB, 0.20530E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.20461E+01_JPRB, 0.20230E+01_JPRB, 0.18672E+01_JPRB, 0.17099E+01_JPRB, 0.16107E+01_JPRB, 0.15536E+01_JPRB, 0.14799E+01_JPRB ,&
&  0.14550E+01_JPRB, 0.13878E+01_JPRB, 0.13212E+01_JPRB, 0.12846E+01_JPRB, 0.12887E+01_JPRB, 0.12843E+01_JPRB, 0.12816E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.12202E+01_JPRB, 0.11712E+01_JPRB, 0.11977E+01_JPRB, 0.13641E+01_JPRB, 0.13265E+01_JPRB, 0.16564E+01_JPRB ,&
&  0.30405E+01_JPRB, 0.45889E+01_JPRB, 0.40631E+01_JPRB, 0.36800E+01_JPRB, 0.32378E+01_JPRB, 0.29277E+01_JPRB, 0.21788E+01_JPRB ,&
&  0.19680E+01_JPRB, 0.19539E+01_JPRB, 0.19065E+01_JPRB, 0.19073E+01_JPRB, 0.18642E+01_JPRB, 0.17884E+01_JPRB, 0.16992E+01_JPRB ,&
&  0.15972E+01_JPRB, 0.15444E+01_JPRB, 0.15477E+01_JPRB, 0.15439E+01_JPRB, 0.15287E+01_JPRB, 0.15634E+01_JPRB, 0.16544E+01_JPRB ,&
&  0.17629E+01_JPRB, 0.19437E+01_JPRB, 0.22310E+01_JPRB, 0.22212E+01_JPRB, 0.21276E+01_JPRB, 0.21252E+01_JPRB, 0.21805E+01_JPRB ,&
&  0.21494E+01_JPRB, 0.21633E+01_JPRB, 0.21491E+01_JPRB, 0.20975E+01_JPRB, 0.21892E+01_JPRB, 0.23542E+01_JPRB, 0.25664E+01_JPRB ,&
&  0.30024E+01_JPRB, 0.28310E+01_JPRB, 0.26605E+01_JPRB, 0.29845E+01_JPRB, 0.37473E+01_JPRB, 0.52423E+01_JPRB, 0.54043E+01_JPRB ,&
&  0.48749E+01_JPRB, 0.37832E+01_JPRB, 0.38784E+01_JPRB, 0.51279E+01_JPRB, 0.60263E+01_JPRB, 0.68848E+01_JPRB, 0.96321E+01_JPRB ,&
&  0.95106E+01_JPRB, 0.50562E+01_JPRB, 0.39292E+01_JPRB, 0.49870E+01_JPRB, 0.33198E+01_JPRB, 0.23455E+01_JPRB, 0.35731E+01_JPRB ,&
&  0.44048E+01_JPRB, 0.50482E+01_JPRB, 0.51691E+01_JPRB, 0.49013E+01_JPRB, 0.48637E+01_JPRB, 0.44566E+01_JPRB, 0.36464E+01_JPRB ,&
&  0.30281E+01_JPRB, 0.24822E+01_JPRB, 0.21037E+01_JPRB, 0.18982E+01_JPRB, 0.18150E+01_JPRB, 0.17707E+01_JPRB, 0.17474E+01_JPRB ,&
&  0.17451E+01_JPRB, 0.17709E+01_JPRB, 0.18305E+01_JPRB, 0.19360E+01_JPRB, 0.20287E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.20143E+01_JPRB, 0.19702E+01_JPRB, 0.18368E+01_JPRB, 0.17125E+01_JPRB, 0.16296E+01_JPRB, 0.15616E+01_JPRB, 0.14743E+01_JPRB ,&
&  0.15783E+01_JPRB, 0.14129E+01_JPRB, 0.13001E+01_JPRB, 0.12556E+01_JPRB, 0.12396E+01_JPRB, 0.12426E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.12390E+01_JPRB, 0.12214E+01_JPRB, 0.12097E+01_JPRB, 0.12505E+01_JPRB, 0.14049E+01_JPRB, 0.18886E+01_JPRB, 0.18678E+01_JPRB ,&
&  0.32072E+01_JPRB, 0.53681E+01_JPRB, 0.42139E+01_JPRB, 0.37070E+01_JPRB, 0.33272E+01_JPRB, 0.31151E+01_JPRB, 0.22089E+01_JPRB ,&
&  0.19466E+01_JPRB, 0.18972E+01_JPRB, 0.18620E+01_JPRB, 0.18407E+01_JPRB, 0.18274E+01_JPRB, 0.17732E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.15637E+01_JPRB, 0.15440E+01_JPRB, 0.15703E+01_JPRB, 0.15865E+01_JPRB, 0.16101E+01_JPRB, 0.16869E+01_JPRB, 0.17518E+01_JPRB ,&
&  0.18765E+01_JPRB, 0.21828E+01_JPRB, 0.23775E+01_JPRB, 0.21682E+01_JPRB, 0.21271E+01_JPRB, 0.21529E+01_JPRB, 0.21875E+01_JPRB ,&
&  0.20577E+01_JPRB, 0.19154E+01_JPRB, 0.20284E+01_JPRB, 0.21291E+01_JPRB, 0.22147E+01_JPRB, 0.22882E+01_JPRB, 0.23909E+01_JPRB ,&
&  0.27667E+01_JPRB, 0.29469E+01_JPRB, 0.33000E+01_JPRB, 0.33588E+01_JPRB, 0.38595E+01_JPRB, 0.49451E+01_JPRB, 0.54405E+01_JPRB ,&
&  0.58468E+01_JPRB, 0.51884E+01_JPRB, 0.55992E+01_JPRB, 0.63301E+01_JPRB, 0.65544E+01_JPRB, 0.66305E+01_JPRB, 0.76061E+01_JPRB ,&
&  0.86970E+01_JPRB, 0.10290E+02_JPRB, 0.90474E+01_JPRB, 0.69341E+01_JPRB, 0.49733E+01_JPRB, 0.33523E+01_JPRB, 0.37193E+01_JPRB ,&
&  0.46621E+01_JPRB, 0.47841E+01_JPRB, 0.47912E+01_JPRB, 0.62015E+01_JPRB, 0.46950E+01_JPRB, 0.35443E+01_JPRB, 0.26295E+01_JPRB ,&
&  0.21050E+01_JPRB, 0.19073E+01_JPRB, 0.18567E+01_JPRB, 0.18009E+01_JPRB, 0.17783E+01_JPRB, 0.17562E+01_JPRB, 0.17591E+01_JPRB ,&
&  0.17906E+01_JPRB, 0.18402E+01_JPRB, 0.19038E+01_JPRB, 0.19801E+01_JPRB, 0.20308E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.19233E+01_JPRB, 0.18469E+01_JPRB, 0.17513E+01_JPRB, 0.16609E+01_JPRB, 0.15760E+01_JPRB, 0.14993E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.13626E+01_JPRB, 0.12502E+01_JPRB, 0.12184E+01_JPRB, 0.12103E+01_JPRB, 0.12081E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.12504E+01_JPRB, 0.12980E+01_JPRB, 0.14105E+01_JPRB, 0.16948E+01_JPRB, 0.23884E+01_JPRB, 0.30249E+01_JPRB ,&
&  0.36096E+01_JPRB, 0.63651E+01_JPRB, 0.43320E+01_JPRB, 0.37031E+01_JPRB, 0.33824E+01_JPRB, 0.32222E+01_JPRB, 0.24813E+01_JPRB ,&
&  0.20048E+01_JPRB, 0.18762E+01_JPRB, 0.18230E+01_JPRB, 0.17701E+01_JPRB, 0.17316E+01_JPRB, 0.16558E+01_JPRB, 0.16015E+01_JPRB ,&
&  0.15791E+01_JPRB, 0.15940E+01_JPRB, 0.16204E+01_JPRB, 0.16511E+01_JPRB, 0.16861E+01_JPRB, 0.17826E+01_JPRB, 0.19160E+01_JPRB ,&
&  0.21734E+01_JPRB, 0.24604E+01_JPRB, 0.23348E+01_JPRB, 0.21916E+01_JPRB, 0.21133E+01_JPRB, 0.21604E+01_JPRB, 0.21869E+01_JPRB ,&
&  0.20813E+01_JPRB, 0.19584E+01_JPRB, 0.20484E+01_JPRB, 0.20805E+01_JPRB, 0.20914E+01_JPRB, 0.22579E+01_JPRB, 0.23258E+01_JPRB ,&
&  0.27917E+01_JPRB, 0.30788E+01_JPRB, 0.42288E+01_JPRB, 0.41632E+01_JPRB, 0.38445E+01_JPRB, 0.47260E+01_JPRB, 0.48359E+01_JPRB ,&
&  0.44342E+01_JPRB, 0.49415E+01_JPRB, 0.57982E+01_JPRB, 0.67890E+01_JPRB, 0.70958E+01_JPRB, 0.72454E+01_JPRB, 0.70551E+01_JPRB ,&
&  0.63452E+01_JPRB, 0.65804E+01_JPRB, 0.53953E+01_JPRB, 0.44128E+01_JPRB, 0.43508E+01_JPRB, 0.36829E+01_JPRB, 0.42259E+01_JPRB ,&
&  0.50774E+01_JPRB, 0.50997E+01_JPRB, 0.54184E+01_JPRB, 0.54035E+01_JPRB, 0.34415E+01_JPRB, 0.24335E+01_JPRB, 0.19201E+01_JPRB ,&
&  0.17882E+01_JPRB, 0.18017E+01_JPRB, 0.18243E+01_JPRB, 0.18051E+01_JPRB, 0.17951E+01_JPRB, 0.17965E+01_JPRB, 0.18280E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.19462E+01_JPRB, 0.19742E+01_JPRB, 0.19798E+01_JPRB, 0.19769E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.18341E+01_JPRB, 0.17730E+01_JPRB, 0.17043E+01_JPRB, 0.16424E+01_JPRB, 0.16067E+01_JPRB, 0.15375E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.17325E+01_JPRB, 0.13034E+01_JPRB, 0.12011E+01_JPRB, 0.11797E+01_JPRB, 0.11856E+01_JPRB, 0.12101E+01_JPRB, 0.12642E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.13826E+01_JPRB, 0.15087E+01_JPRB, 0.17848E+01_JPRB, 0.23501E+01_JPRB, 0.33712E+01_JPRB, 0.43411E+01_JPRB ,&
&  0.54612E+01_JPRB, 0.53593E+01_JPRB, 0.43632E+01_JPRB, 0.33743E+01_JPRB, 0.31559E+01_JPRB, 0.30898E+01_JPRB, 0.29458E+01_JPRB ,&
&  0.23004E+01_JPRB, 0.20257E+01_JPRB, 0.19137E+01_JPRB, 0.17980E+01_JPRB, 0.16606E+01_JPRB, 0.16407E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.16752E+01_JPRB, 0.17031E+01_JPRB, 0.17433E+01_JPRB, 0.17947E+01_JPRB, 0.19001E+01_JPRB, 0.20832E+01_JPRB, 0.22960E+01_JPRB ,&
&  0.25252E+01_JPRB, 0.26090E+01_JPRB, 0.24361E+01_JPRB, 0.22562E+01_JPRB, 0.21625E+01_JPRB, 0.22001E+01_JPRB, 0.22038E+01_JPRB ,&
&  0.21527E+01_JPRB, 0.21434E+01_JPRB, 0.21684E+01_JPRB, 0.21673E+01_JPRB, 0.20911E+01_JPRB, 0.23439E+01_JPRB, 0.25067E+01_JPRB ,&
&  0.29946E+01_JPRB, 0.35790E+01_JPRB, 0.48731E+01_JPRB, 0.40626E+01_JPRB, 0.42979E+01_JPRB, 0.45359E+01_JPRB, 0.46251E+01_JPRB ,&
&  0.44589E+01_JPRB, 0.52896E+01_JPRB, 0.62869E+01_JPRB, 0.68525E+01_JPRB, 0.75543E+01_JPRB, 0.72040E+01_JPRB, 0.74267E+01_JPRB ,&
&  0.54544E+01_JPRB, 0.50764E+01_JPRB, 0.36969E+01_JPRB, 0.38908E+01_JPRB, 0.40016E+01_JPRB, 0.38825E+01_JPRB, 0.41124E+01_JPRB ,&
&  0.47690E+01_JPRB, 0.46673E+01_JPRB, 0.42007E+01_JPRB, 0.34087E+01_JPRB, 0.21466E+01_JPRB, 0.17547E+01_JPRB, 0.16363E+01_JPRB ,&
&  0.16515E+01_JPRB, 0.17773E+01_JPRB, 0.18185E+01_JPRB, 0.18414E+01_JPRB, 0.18356E+01_JPRB, 0.18480E+01_JPRB, 0.18810E+01_JPRB ,&
&  0.19284E+01_JPRB, 0.19544E+01_JPRB, 0.19551E+01_JPRB, 0.19225E+01_JPRB, 0.18864E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.17357E+01_JPRB, 0.16823E+01_JPRB, 0.16310E+01_JPRB, 0.15545E+01_JPRB, 0.14311E+01_JPRB, 0.13427E+01_JPRB, 0.12903E+01_JPRB ,&
&  0.12476E+01_JPRB, 0.12109E+01_JPRB, 0.12213E+01_JPRB, 0.12350E+01_JPRB, 0.12642E+01_JPRB, 0.13213E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.15021E+01_JPRB, 0.16640E+01_JPRB, 0.19683E+01_JPRB, 0.24325E+01_JPRB, 0.30065E+01_JPRB, 0.38630E+01_JPRB, 0.50636E+01_JPRB ,&
&  0.53416E+01_JPRB, 0.34258E+01_JPRB, 0.35721E+01_JPRB, 0.32101E+01_JPRB, 0.30813E+01_JPRB, 0.27923E+01_JPRB, 0.25654E+01_JPRB ,&
&  0.23559E+01_JPRB, 0.22250E+01_JPRB, 0.21574E+01_JPRB, 0.20164E+01_JPRB, 0.18032E+01_JPRB, 0.18458E+01_JPRB, 0.18925E+01_JPRB ,&
&  0.19332E+01_JPRB, 0.19980E+01_JPRB, 0.20823E+01_JPRB, 0.22119E+01_JPRB, 0.23962E+01_JPRB, 0.25778E+01_JPRB, 0.27202E+01_JPRB ,&
&  0.27808E+01_JPRB, 0.28157E+01_JPRB, 0.27616E+01_JPRB, 0.26270E+01_JPRB, 0.24599E+01_JPRB, 0.24267E+01_JPRB, 0.23528E+01_JPRB ,&
&  0.22982E+01_JPRB, 0.22480E+01_JPRB, 0.23229E+01_JPRB, 0.24875E+01_JPRB, 0.26076E+01_JPRB, 0.25337E+01_JPRB, 0.29202E+01_JPRB ,&
&  0.33752E+01_JPRB, 0.40406E+01_JPRB, 0.44443E+01_JPRB, 0.55635E+01_JPRB, 0.38878E+01_JPRB, 0.41545E+01_JPRB, 0.40089E+01_JPRB ,&
&  0.50926E+01_JPRB, 0.56027E+01_JPRB, 0.62075E+01_JPRB, 0.67196E+01_JPRB, 0.76647E+01_JPRB, 0.70314E+01_JPRB, 0.64081E+01_JPRB ,&
&  0.49697E+01_JPRB, 0.37794E+01_JPRB, 0.30974E+01_JPRB, 0.34158E+01_JPRB, 0.38359E+01_JPRB, 0.43788E+01_JPRB, 0.40443E+01_JPRB ,&
&  0.40179E+01_JPRB, 0.34688E+01_JPRB, 0.28341E+01_JPRB, 0.25451E+01_JPRB, 0.18055E+01_JPRB, 0.16279E+01_JPRB, 0.16402E+01_JPRB ,&
&  0.16986E+01_JPRB, 0.18059E+01_JPRB, 0.18366E+01_JPRB, 0.18332E+01_JPRB, 0.18304E+01_JPRB, 0.18330E+01_JPRB, 0.18468E+01_JPRB ,&
&  0.18583E+01_JPRB, 0.18682E+01_JPRB, 0.18621E+01_JPRB, 0.18252E+01_JPRB, 0.17786E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.16360E+01_JPRB, 0.15730E+01_JPRB, 0.14631E+01_JPRB, 0.13353E+01_JPRB, 0.12429E+01_JPRB, 0.11915E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.12268E+01_JPRB, 0.12699E+01_JPRB, 0.13268E+01_JPRB, 0.13832E+01_JPRB, 0.14490E+01_JPRB, 0.15745E+01_JPRB ,&
&  0.17225E+01_JPRB, 0.19688E+01_JPRB, 0.23342E+01_JPRB, 0.27463E+01_JPRB, 0.31240E+01_JPRB, 0.35151E+01_JPRB, 0.39028E+01_JPRB ,&
&  0.43310E+01_JPRB, 0.43247E+01_JPRB, 0.37025E+01_JPRB, 0.25680E+01_JPRB, 0.23277E+01_JPRB, 0.24859E+01_JPRB, 0.24075E+01_JPRB ,&
&  0.23508E+01_JPRB, 0.22561E+01_JPRB, 0.21847E+01_JPRB, 0.21642E+01_JPRB, 0.21744E+01_JPRB, 0.22426E+01_JPRB, 0.23073E+01_JPRB ,&
&  0.24019E+01_JPRB, 0.25011E+01_JPRB, 0.26140E+01_JPRB, 0.27381E+01_JPRB, 0.28315E+01_JPRB, 0.29205E+01_JPRB, 0.30187E+01_JPRB ,&
&  0.31006E+01_JPRB, 0.32354E+01_JPRB, 0.33507E+01_JPRB, 0.33582E+01_JPRB, 0.31272E+01_JPRB, 0.30080E+01_JPRB, 0.28527E+01_JPRB ,&
&  0.27813E+01_JPRB, 0.27292E+01_JPRB, 0.27695E+01_JPRB, 0.29817E+01_JPRB, 0.31441E+01_JPRB, 0.29384E+01_JPRB, 0.31283E+01_JPRB ,&
&  0.38472E+01_JPRB, 0.46089E+01_JPRB, 0.42787E+01_JPRB, 0.53871E+01_JPRB, 0.32879E+01_JPRB, 0.39379E+01_JPRB, 0.46632E+01_JPRB ,&
&  0.47614E+01_JPRB, 0.52175E+01_JPRB, 0.58456E+01_JPRB, 0.63406E+01_JPRB, 0.64714E+01_JPRB, 0.59713E+01_JPRB, 0.54371E+01_JPRB ,&
&  0.43883E+01_JPRB, 0.32661E+01_JPRB, 0.27839E+01_JPRB, 0.28810E+01_JPRB, 0.33655E+01_JPRB, 0.39289E+01_JPRB, 0.30584E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.25047E+01_JPRB, 0.23865E+01_JPRB, 0.24474E+01_JPRB, 0.18691E+01_JPRB, 0.15859E+01_JPRB, 0.16693E+01_JPRB ,&
&  0.17730E+01_JPRB, 0.17917E+01_JPRB, 0.18147E+01_JPRB, 0.17957E+01_JPRB, 0.17930E+01_JPRB, 0.17859E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.17808E+01_JPRB, 0.17912E+01_JPRB, 0.17846E+01_JPRB, 0.17356E+01_JPRB, 0.16803E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.14753E+01_JPRB, 0.13503E+01_JPRB, 0.12187E+01_JPRB, 0.11274E+01_JPRB, 0.10858E+01_JPRB, 0.10806E+01_JPRB, 0.11016E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.11957E+01_JPRB, 0.12632E+01_JPRB, 0.13484E+01_JPRB, 0.14296E+01_JPRB, 0.15214E+01_JPRB, 0.16523E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.20258E+01_JPRB, 0.23137E+01_JPRB, 0.26324E+01_JPRB, 0.28942E+01_JPRB, 0.31450E+01_JPRB, 0.33874E+01_JPRB ,&
&  0.36196E+01_JPRB, 0.37064E+01_JPRB, 0.35964E+01_JPRB, 0.34162E+01_JPRB, 0.27866E+01_JPRB, 0.29535E+01_JPRB, 0.31210E+01_JPRB ,&
&  0.31967E+01_JPRB, 0.26959E+01_JPRB, 0.24987E+01_JPRB, 0.24838E+01_JPRB, 0.25311E+01_JPRB, 0.26245E+01_JPRB, 0.27056E+01_JPRB ,&
&  0.27908E+01_JPRB, 0.28666E+01_JPRB, 0.29385E+01_JPRB, 0.29986E+01_JPRB, 0.30586E+01_JPRB, 0.31702E+01_JPRB, 0.33617E+01_JPRB ,&
&  0.34646E+01_JPRB, 0.35714E+01_JPRB, 0.36520E+01_JPRB, 0.37171E+01_JPRB, 0.39397E+01_JPRB, 0.40856E+01_JPRB, 0.39811E+01_JPRB ,&
&  0.38625E+01_JPRB, 0.36154E+01_JPRB, 0.37239E+01_JPRB, 0.40409E+01_JPRB, 0.41049E+01_JPRB, 0.38919E+01_JPRB, 0.39680E+01_JPRB ,&
&  0.46111E+01_JPRB, 0.53339E+01_JPRB, 0.38435E+01_JPRB, 0.43606E+01_JPRB, 0.51009E+01_JPRB, 0.42996E+01_JPRB, 0.38934E+01_JPRB ,&
&  0.35766E+01_JPRB, 0.38854E+01_JPRB, 0.47630E+01_JPRB, 0.51603E+01_JPRB, 0.50970E+01_JPRB, 0.58793E+01_JPRB, 0.47443E+01_JPRB ,&
&  0.37813E+01_JPRB, 0.27867E+01_JPRB, 0.24202E+01_JPRB, 0.23737E+01_JPRB, 0.26212E+01_JPRB, 0.26887E+01_JPRB, 0.18463E+01_JPRB ,&
&  0.16319E+01_JPRB, 0.20556E+01_JPRB, 0.20847E+01_JPRB, 0.22579E+01_JPRB, 0.21297E+01_JPRB, 0.15073E+01_JPRB, 0.15910E+01_JPRB ,&
&  0.16668E+01_JPRB, 0.17191E+01_JPRB, 0.17278E+01_JPRB, 0.17210E+01_JPRB, 0.17125E+01_JPRB, 0.17057E+01_JPRB, 0.17050E+01_JPRB ,&
&  0.17076E+01_JPRB, 0.17034E+01_JPRB, 0.16864E+01_JPRB, 0.16362E+01_JPRB, 0.15631E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.11946E+01_JPRB, 0.10808E+01_JPRB, 0.10017E+01_JPRB, 0.97721E+00_JPRB, 0.98743E+00_JPRB, 0.10063E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.10981E+01_JPRB, 0.11665E+01_JPRB, 0.12289E+01_JPRB, 0.13043E+01_JPRB, 0.13816E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.16277E+01_JPRB, 0.17657E+01_JPRB, 0.19744E+01_JPRB, 0.22712E+01_JPRB, 0.25035E+01_JPRB, 0.27178E+01_JPRB, 0.30130E+01_JPRB ,&
&  0.33199E+01_JPRB, 0.35304E+01_JPRB, 0.37484E+01_JPRB, 0.39051E+01_JPRB, 0.38521E+01_JPRB, 0.37474E+01_JPRB, 0.38862E+01_JPRB ,&
&  0.32923E+01_JPRB, 0.26492E+01_JPRB, 0.26249E+01_JPRB, 0.26614E+01_JPRB, 0.27755E+01_JPRB, 0.28395E+01_JPRB, 0.29023E+01_JPRB ,&
&  0.29687E+01_JPRB, 0.30153E+01_JPRB, 0.30596E+01_JPRB, 0.31156E+01_JPRB, 0.32151E+01_JPRB, 0.33807E+01_JPRB, 0.36016E+01_JPRB ,&
&  0.37170E+01_JPRB, 0.38558E+01_JPRB, 0.40260E+01_JPRB, 0.37598E+01_JPRB, 0.41646E+01_JPRB, 0.47227E+01_JPRB, 0.50489E+01_JPRB ,&
&  0.50354E+01_JPRB, 0.47685E+01_JPRB, 0.49036E+01_JPRB, 0.50843E+01_JPRB, 0.51667E+01_JPRB, 0.50979E+01_JPRB, 0.50811E+01_JPRB ,&
&  0.54368E+01_JPRB, 0.52586E+01_JPRB, 0.38284E+01_JPRB, 0.35182E+01_JPRB, 0.36453E+01_JPRB, 0.30032E+01_JPRB, 0.30256E+01_JPRB ,&
&  0.26084E+01_JPRB, 0.27965E+01_JPRB, 0.35578E+01_JPRB, 0.41766E+01_JPRB, 0.45003E+01_JPRB, 0.51670E+01_JPRB, 0.43450E+01_JPRB ,&
&  0.29246E+01_JPRB, 0.21523E+01_JPRB, 0.17975E+01_JPRB, 0.16590E+01_JPRB, 0.17852E+01_JPRB, 0.18208E+01_JPRB, 0.16199E+01_JPRB ,&
&  0.16554E+01_JPRB, 0.16916E+01_JPRB, 0.17409E+01_JPRB, 0.16221E+01_JPRB, 0.14256E+01_JPRB, 0.13512E+01_JPRB, 0.14298E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.15126E+01_JPRB, 0.15346E+01_JPRB, 0.15355E+01_JPRB, 0.15418E+01_JPRB, 0.15436E+01_JPRB, 0.15475E+01_JPRB ,&
&  0.15518E+01_JPRB, 0.15382E+01_JPRB, 0.14979E+01_JPRB, 0.14351E+01_JPRB, 0.13287E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.94283E+00_JPRB, 0.87551E+00_JPRB, 0.85582E+00_JPRB, 0.86531E+00_JPRB, 0.87867E+00_JPRB, 0.89315E+00_JPRB, 0.90989E+00_JPRB ,&
&  0.92562E+00_JPRB, 0.93936E+00_JPRB, 0.96125E+00_JPRB, 0.97968E+00_JPRB, 0.10398E+01_JPRB, 0.11116E+01_JPRB, 0.12025E+01_JPRB ,&
&  0.13336E+01_JPRB, 0.14537E+01_JPRB, 0.15604E+01_JPRB, 0.17265E+01_JPRB, 0.18492E+01_JPRB, 0.19802E+01_JPRB, 0.21805E+01_JPRB ,&
&  0.24333E+01_JPRB, 0.27440E+01_JPRB, 0.30430E+01_JPRB, 0.33819E+01_JPRB, 0.38533E+01_JPRB, 0.43224E+01_JPRB, 0.44466E+01_JPRB ,&
&  0.34385E+01_JPRB, 0.24647E+01_JPRB, 0.26684E+01_JPRB, 0.25363E+01_JPRB, 0.26822E+01_JPRB, 0.28064E+01_JPRB, 0.28661E+01_JPRB ,&
&  0.29421E+01_JPRB, 0.29988E+01_JPRB, 0.30518E+01_JPRB, 0.31118E+01_JPRB, 0.32373E+01_JPRB, 0.33938E+01_JPRB, 0.35934E+01_JPRB ,&
&  0.37510E+01_JPRB, 0.38761E+01_JPRB, 0.39245E+01_JPRB, 0.39524E+01_JPRB, 0.43525E+01_JPRB, 0.49126E+01_JPRB, 0.53216E+01_JPRB ,&
&  0.55748E+01_JPRB, 0.58425E+01_JPRB, 0.54072E+01_JPRB, 0.54923E+01_JPRB, 0.54330E+01_JPRB, 0.49245E+01_JPRB, 0.50329E+01_JPRB ,&
&  0.53936E+01_JPRB, 0.50290E+01_JPRB, 0.32556E+01_JPRB, 0.30465E+01_JPRB, 0.29626E+01_JPRB, 0.26869E+01_JPRB, 0.25190E+01_JPRB ,&
&  0.20284E+01_JPRB, 0.20883E+01_JPRB, 0.24304E+01_JPRB, 0.32162E+01_JPRB, 0.36544E+01_JPRB, 0.38765E+01_JPRB, 0.33912E+01_JPRB ,&
&  0.22794E+01_JPRB, 0.17386E+01_JPRB, 0.14701E+01_JPRB, 0.14971E+01_JPRB, 0.18019E+01_JPRB, 0.21533E+01_JPRB, 0.17124E+01_JPRB ,&
&  0.17559E+01_JPRB, 0.20970E+01_JPRB, 0.26506E+01_JPRB, 0.19648E+01_JPRB, 0.12964E+01_JPRB, 0.11767E+01_JPRB, 0.11953E+01_JPRB ,&
&  0.12201E+01_JPRB, 0.12471E+01_JPRB, 0.12618E+01_JPRB, 0.12688E+01_JPRB, 0.12799E+01_JPRB, 0.13029E+01_JPRB, 0.13121E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12613E+01_JPRB, 0.12266E+01_JPRB, 0.11295E+01_JPRB, 0.10274E+01_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.77543E+00_JPRB, 0.76717E+00_JPRB, 0.76997E+00_JPRB, 0.78714E+00_JPRB, 0.78930E+00_JPRB, 0.79209E+00_JPRB, 0.82882E+00_JPRB ,&
&  0.82081E+00_JPRB, 0.81474E+00_JPRB, 0.82559E+00_JPRB, 0.80989E+00_JPRB, 0.85008E+00_JPRB, 0.89275E+00_JPRB, 0.95895E+00_JPRB ,&
&  0.10262E+01_JPRB, 0.10725E+01_JPRB, 0.11100E+01_JPRB, 0.11625E+01_JPRB, 0.12616E+01_JPRB, 0.13409E+01_JPRB, 0.14115E+01_JPRB ,&
&  0.15360E+01_JPRB, 0.17210E+01_JPRB, 0.19207E+01_JPRB, 0.23543E+01_JPRB, 0.33428E+01_JPRB, 0.41573E+01_JPRB, 0.41875E+01_JPRB ,&
&  0.31913E+01_JPRB, 0.23884E+01_JPRB, 0.22485E+01_JPRB, 0.20683E+01_JPRB, 0.22021E+01_JPRB, 0.24599E+01_JPRB, 0.25635E+01_JPRB ,&
&  0.25748E+01_JPRB, 0.26261E+01_JPRB, 0.26538E+01_JPRB, 0.26910E+01_JPRB, 0.27391E+01_JPRB, 0.27722E+01_JPRB, 0.27383E+01_JPRB ,&
&  0.26411E+01_JPRB, 0.25173E+01_JPRB, 0.26221E+01_JPRB, 0.31810E+01_JPRB, 0.38185E+01_JPRB, 0.46731E+01_JPRB, 0.45912E+01_JPRB ,&
&  0.44098E+01_JPRB, 0.44229E+01_JPRB, 0.44829E+01_JPRB, 0.44390E+01_JPRB, 0.42837E+01_JPRB, 0.40976E+01_JPRB, 0.42027E+01_JPRB ,&
&  0.44893E+01_JPRB, 0.42033E+01_JPRB, 0.28195E+01_JPRB, 0.27590E+01_JPRB, 0.25868E+01_JPRB, 0.25217E+01_JPRB, 0.21251E+01_JPRB ,&
&  0.16921E+01_JPRB, 0.16253E+01_JPRB, 0.17601E+01_JPRB, 0.20775E+01_JPRB, 0.26377E+01_JPRB, 0.28176E+01_JPRB, 0.24554E+01_JPRB ,&
&  0.18600E+01_JPRB, 0.14868E+01_JPRB, 0.13069E+01_JPRB, 0.13704E+01_JPRB, 0.18512E+01_JPRB, 0.22410E+01_JPRB, 0.24117E+01_JPRB ,&
&  0.16490E+01_JPRB, 0.18745E+01_JPRB, 0.16510E+01_JPRB, 0.20117E+01_JPRB, 0.10639E+01_JPRB, 0.91461E+00_JPRB, 0.90373E+00_JPRB ,&
&  0.91980E+00_JPRB, 0.91376E+00_JPRB, 0.90582E+00_JPRB, 0.88710E+00_JPRB, 0.87039E+00_JPRB, 0.94028E+00_JPRB, 0.10051E+01_JPRB ,&
&  0.95014E+00_JPRB, 0.98023E+00_JPRB, 0.93482E+00_JPRB, 0.85869E+00_JPRB, 0.81996E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.73359E+00_JPRB, 0.74584E+00_JPRB, 0.75157E+00_JPRB, 0.76451E+00_JPRB, 0.78942E+00_JPRB, 0.81088E+00_JPRB, 0.83845E+00_JPRB ,&
&  0.87018E+00_JPRB, 0.89138E+00_JPRB, 0.91424E+00_JPRB, 0.94250E+00_JPRB, 0.96698E+00_JPRB, 0.10076E+01_JPRB, 0.10463E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.11321E+01_JPRB, 0.11713E+01_JPRB, 0.12311E+01_JPRB, 0.13048E+01_JPRB, 0.14146E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.16959E+01_JPRB, 0.18864E+01_JPRB, 0.21150E+01_JPRB, 0.23100E+01_JPRB, 0.24217E+01_JPRB, 0.25128E+01_JPRB, 0.29043E+01_JPRB ,&
&  0.25189E+01_JPRB, 0.24622E+01_JPRB, 0.23293E+01_JPRB, 0.20924E+01_JPRB, 0.20788E+01_JPRB, 0.19190E+01_JPRB, 0.18779E+01_JPRB ,&
&  0.18988E+01_JPRB, 0.18451E+01_JPRB, 0.18139E+01_JPRB, 0.17857E+01_JPRB, 0.17424E+01_JPRB, 0.16455E+01_JPRB, 0.15305E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.16384E+01_JPRB, 0.19021E+01_JPRB, 0.24378E+01_JPRB, 0.29843E+01_JPRB, 0.32163E+01_JPRB, 0.29849E+01_JPRB ,&
&  0.28811E+01_JPRB, 0.29134E+01_JPRB, 0.28527E+01_JPRB, 0.27392E+01_JPRB, 0.28703E+01_JPRB, 0.29783E+01_JPRB, 0.31001E+01_JPRB ,&
&  0.30146E+01_JPRB, 0.26940E+01_JPRB, 0.24941E+01_JPRB, 0.22715E+01_JPRB, 0.22041E+01_JPRB, 0.21231E+01_JPRB, 0.18996E+01_JPRB ,&
&  0.15973E+01_JPRB, 0.14153E+01_JPRB, 0.13618E+01_JPRB, 0.14570E+01_JPRB, 0.17791E+01_JPRB, 0.18265E+01_JPRB, 0.16509E+01_JPRB ,&
&  0.13583E+01_JPRB, 0.12107E+01_JPRB, 0.11843E+01_JPRB, 0.12844E+01_JPRB, 0.16741E+01_JPRB, 0.37459E+01_JPRB, 0.48573E+01_JPRB ,&
&  0.18124E+01_JPRB, 0.14458E+01_JPRB, 0.85494E+00_JPRB, 0.12025E+01_JPRB, 0.80104E+00_JPRB, 0.73787E+00_JPRB, 0.73487E+00_JPRB ,&
&  0.72775E+00_JPRB, 0.72642E+00_JPRB, 0.72713E+00_JPRB, 0.71504E+00_JPRB, 0.71036E+00_JPRB, 0.72468E+00_JPRB, 0.77330E+00_JPRB ,&
&  0.77435E+00_JPRB, 0.78201E+00_JPRB, 0.75987E+00_JPRB, 0.75616E+00_JPRB, 0.74127E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.74201E+00_JPRB, 0.76321E+00_JPRB, 0.79076E+00_JPRB, 0.82482E+00_JPRB, 0.85237E+00_JPRB, 0.88604E+00_JPRB, 0.91663E+00_JPRB ,&
&  0.95398E+00_JPRB, 0.98767E+00_JPRB, 0.10192E+01_JPRB, 0.10534E+01_JPRB, 0.10919E+01_JPRB, 0.11338E+01_JPRB, 0.11852E+01_JPRB ,&
&  0.12401E+01_JPRB, 0.12957E+01_JPRB, 0.13569E+01_JPRB, 0.14266E+01_JPRB, 0.15083E+01_JPRB, 0.16085E+01_JPRB, 0.17072E+01_JPRB ,&
&  0.18033E+01_JPRB, 0.19065E+01_JPRB, 0.20219E+01_JPRB, 0.21144E+01_JPRB, 0.21594E+01_JPRB, 0.22739E+01_JPRB, 0.21258E+01_JPRB ,&
&  0.25173E+01_JPRB, 0.23171E+01_JPRB, 0.21633E+01_JPRB, 0.19695E+01_JPRB, 0.17882E+01_JPRB, 0.16226E+01_JPRB, 0.14928E+01_JPRB ,&
&  0.15268E+01_JPRB, 0.14678E+01_JPRB, 0.14456E+01_JPRB, 0.14465E+01_JPRB, 0.13200E+01_JPRB, 0.13030E+01_JPRB, 0.13973E+01_JPRB ,&
&  0.15600E+01_JPRB, 0.17922E+01_JPRB, 0.19548E+01_JPRB, 0.21456E+01_JPRB, 0.23746E+01_JPRB, 0.23010E+01_JPRB, 0.21589E+01_JPRB ,&
&  0.20197E+01_JPRB, 0.19442E+01_JPRB, 0.15900E+01_JPRB, 0.18122E+01_JPRB, 0.18880E+01_JPRB, 0.19748E+01_JPRB, 0.21223E+01_JPRB ,&
&  0.19948E+01_JPRB, 0.16309E+01_JPRB, 0.16829E+01_JPRB, 0.19617E+01_JPRB, 0.18977E+01_JPRB, 0.17809E+01_JPRB, 0.16774E+01_JPRB ,&
&  0.15261E+01_JPRB, 0.13998E+01_JPRB, 0.12883E+01_JPRB, 0.12457E+01_JPRB, 0.13310E+01_JPRB, 0.13222E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.10753E+01_JPRB, 0.11122E+01_JPRB, 0.12130E+01_JPRB, 0.16017E+01_JPRB, 0.20792E+01_JPRB, 0.33896E+01_JPRB ,&
&  0.19052E+01_JPRB, 0.99110E+00_JPRB, 0.10283E+01_JPRB, 0.95221E+00_JPRB, 0.80426E+00_JPRB, 0.70433E+00_JPRB, 0.71974E+00_JPRB ,&
&  0.74567E+00_JPRB, 0.75656E+00_JPRB, 0.71382E+00_JPRB, 0.71425E+00_JPRB, 0.71549E+00_JPRB, 0.71861E+00_JPRB, 0.72802E+00_JPRB ,&
&  0.72982E+00_JPRB, 0.73161E+00_JPRB, 0.73948E+00_JPRB, 0.73458E+00_JPRB, 0.73349E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.65339E+00_JPRB, 0.66750E+00_JPRB, 0.68848E+00_JPRB, 0.70776E+00_JPRB, 0.72161E+00_JPRB, 0.74172E+00_JPRB, 0.75877E+00_JPRB ,&
&  0.77813E+00_JPRB, 0.79067E+00_JPRB, 0.80780E+00_JPRB, 0.82456E+00_JPRB, 0.84246E+00_JPRB, 0.86586E+00_JPRB, 0.89096E+00_JPRB ,&
&  0.91421E+00_JPRB, 0.95095E+00_JPRB, 0.98907E+00_JPRB, 0.10199E+01_JPRB, 0.10839E+01_JPRB, 0.12006E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.14604E+01_JPRB, 0.16276E+01_JPRB, 0.17883E+01_JPRB, 0.18936E+01_JPRB, 0.19417E+01_JPRB, 0.21054E+01_JPRB, 0.18463E+01_JPRB ,&
&  0.22711E+01_JPRB, 0.22009E+01_JPRB, 0.20093E+01_JPRB, 0.18979E+01_JPRB, 0.17865E+01_JPRB, 0.16834E+01_JPRB, 0.16407E+01_JPRB ,&
&  0.17094E+01_JPRB, 0.20714E+01_JPRB, 0.20352E+01_JPRB, 0.20901E+01_JPRB, 0.18065E+01_JPRB, 0.17546E+01_JPRB, 0.19611E+01_JPRB ,&
&  0.22066E+01_JPRB, 0.23158E+01_JPRB, 0.23708E+01_JPRB, 0.25063E+01_JPRB, 0.26541E+01_JPRB, 0.27308E+01_JPRB, 0.27614E+01_JPRB ,&
&  0.26606E+01_JPRB, 0.25687E+01_JPRB, 0.24191E+01_JPRB, 0.28654E+01_JPRB, 0.25272E+01_JPRB, 0.21948E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.15369E+01_JPRB, 0.14078E+01_JPRB, 0.14786E+01_JPRB, 0.15431E+01_JPRB, 0.15258E+01_JPRB, 0.15282E+01_JPRB, 0.15015E+01_JPRB ,&
&  0.14366E+01_JPRB, 0.13847E+01_JPRB, 0.12980E+01_JPRB, 0.12696E+01_JPRB, 0.12774E+01_JPRB, 0.12155E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10193E+01_JPRB, 0.10481E+01_JPRB, 0.10593E+01_JPRB, 0.11133E+01_JPRB, 0.13215E+01_JPRB, 0.14225E+01_JPRB, 0.21950E+01_JPRB ,&
&  0.15275E+01_JPRB, 0.13142E+01_JPRB, 0.17311E+01_JPRB, 0.61130E+00_JPRB, 0.68998E+00_JPRB, 0.78096E+00_JPRB, 0.97426E+00_JPRB ,&
&  0.10062E+01_JPRB, 0.12778E+01_JPRB, 0.15992E+01_JPRB, 0.12731E+01_JPRB, 0.99362E+00_JPRB, 0.71016E+00_JPRB, 0.68127E+00_JPRB ,&
&  0.67485E+00_JPRB, 0.66568E+00_JPRB, 0.66732E+00_JPRB, 0.65875E+00_JPRB, 0.65434E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.55809E+00_JPRB, 0.56519E+00_JPRB, 0.57949E+00_JPRB, 0.59015E+00_JPRB, 0.60055E+00_JPRB, 0.61035E+00_JPRB, 0.61823E+00_JPRB ,&
&  0.63725E+00_JPRB, 0.65066E+00_JPRB, 0.68337E+00_JPRB, 0.71538E+00_JPRB, 0.74603E+00_JPRB, 0.80121E+00_JPRB, 0.85412E+00_JPRB ,&
&  0.93197E+00_JPRB, 0.10320E+01_JPRB, 0.10904E+01_JPRB, 0.11574E+01_JPRB, 0.12032E+01_JPRB, 0.12607E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.14713E+01_JPRB, 0.17697E+01_JPRB, 0.19902E+01_JPRB, 0.22241E+01_JPRB, 0.22654E+01_JPRB, 0.24850E+01_JPRB, 0.19050E+01_JPRB ,&
&  0.23258E+01_JPRB, 0.23786E+01_JPRB, 0.22598E+01_JPRB, 0.22137E+01_JPRB, 0.22014E+01_JPRB, 0.22095E+01_JPRB, 0.22214E+01_JPRB ,&
&  0.23537E+01_JPRB, 0.25207E+01_JPRB, 0.24578E+01_JPRB, 0.25284E+01_JPRB, 0.25831E+01_JPRB, 0.25687E+01_JPRB, 0.28873E+01_JPRB ,&
&  0.31821E+01_JPRB, 0.34215E+01_JPRB, 0.37137E+01_JPRB, 0.40660E+01_JPRB, 0.43120E+01_JPRB, 0.44640E+01_JPRB, 0.45282E+01_JPRB ,&
&  0.45594E+01_JPRB, 0.47091E+01_JPRB, 0.51117E+01_JPRB, 0.50843E+01_JPRB, 0.40007E+01_JPRB, 0.32338E+01_JPRB, 0.24425E+01_JPRB ,&
&  0.16068E+01_JPRB, 0.12780E+01_JPRB, 0.12389E+01_JPRB, 0.12573E+01_JPRB, 0.12376E+01_JPRB, 0.12398E+01_JPRB, 0.12351E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.12713E+01_JPRB, 0.12689E+01_JPRB, 0.12629E+01_JPRB, 0.12204E+01_JPRB, 0.11015E+01_JPRB, 0.97404E+00_JPRB ,&
&  0.95188E+00_JPRB, 0.95552E+00_JPRB, 0.98424E+00_JPRB, 0.10620E+01_JPRB, 0.12278E+01_JPRB, 0.15413E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.16271E+01_JPRB, 0.10295E+01_JPRB, 0.93778E+00_JPRB, 0.95787E+00_JPRB, 0.72973E+00_JPRB, 0.98068E+00_JPRB, 0.14000E+01_JPRB ,&
&  0.20324E+01_JPRB, 0.22396E+01_JPRB, 0.28892E+01_JPRB, 0.41949E+01_JPRB, 0.26544E+01_JPRB, 0.17335E+01_JPRB, 0.62977E+00_JPRB ,&
&  0.62119E+00_JPRB, 0.59813E+00_JPRB, 0.59074E+00_JPRB, 0.57392E+00_JPRB, 0.56515E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.45571E+00_JPRB, 0.44980E+00_JPRB, 0.46381E+00_JPRB, 0.48437E+00_JPRB, 0.50076E+00_JPRB, 0.53710E+00_JPRB, 0.58163E+00_JPRB ,&
&  0.63143E+00_JPRB, 0.69707E+00_JPRB, 0.76202E+00_JPRB, 0.82083E+00_JPRB, 0.91184E+00_JPRB, 0.10300E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.12328E+01_JPRB, 0.13345E+01_JPRB, 0.14134E+01_JPRB, 0.14777E+01_JPRB, 0.15591E+01_JPRB, 0.16650E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.19600E+01_JPRB, 0.21268E+01_JPRB, 0.23412E+01_JPRB, 0.24886E+01_JPRB, 0.25749E+01_JPRB, 0.26213E+01_JPRB, 0.20367E+01_JPRB ,&
&  0.24886E+01_JPRB, 0.27077E+01_JPRB, 0.27824E+01_JPRB, 0.27405E+01_JPRB, 0.26744E+01_JPRB, 0.26517E+01_JPRB, 0.27257E+01_JPRB ,&
&  0.28429E+01_JPRB, 0.28595E+01_JPRB, 0.27945E+01_JPRB, 0.27846E+01_JPRB, 0.34919E+01_JPRB, 0.36099E+01_JPRB, 0.39736E+01_JPRB ,&
&  0.43172E+01_JPRB, 0.46156E+01_JPRB, 0.50025E+01_JPRB, 0.54089E+01_JPRB, 0.58305E+01_JPRB, 0.61882E+01_JPRB, 0.65111E+01_JPRB ,&
&  0.68163E+01_JPRB, 0.71449E+01_JPRB, 0.74484E+01_JPRB, 0.63922E+01_JPRB, 0.50391E+01_JPRB, 0.40850E+01_JPRB, 0.24737E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.12732E+01_JPRB, 0.12519E+01_JPRB, 0.11765E+01_JPRB, 0.10632E+01_JPRB, 0.10267E+01_JPRB, 0.10049E+01_JPRB ,&
&  0.10341E+01_JPRB, 0.10989E+01_JPRB, 0.11375E+01_JPRB, 0.11384E+01_JPRB, 0.10782E+01_JPRB, 0.96890E+00_JPRB, 0.86995E+00_JPRB ,&
&  0.89280E+00_JPRB, 0.90731E+00_JPRB, 0.93287E+00_JPRB, 0.10591E+01_JPRB, 0.12600E+01_JPRB, 0.16773E+01_JPRB, 0.21389E+01_JPRB ,&
&  0.18952E+01_JPRB, 0.10265E+01_JPRB, 0.86979E+00_JPRB, 0.80886E+00_JPRB, 0.76363E+00_JPRB, 0.82787E+00_JPRB, 0.98918E+00_JPRB ,&
&  0.13756E+01_JPRB, 0.22028E+01_JPRB, 0.24029E+01_JPRB, 0.15804E+01_JPRB, 0.20697E+01_JPRB, 0.13815E+01_JPRB, 0.60740E+00_JPRB ,&
&  0.58265E+00_JPRB, 0.56082E+00_JPRB, 0.53234E+00_JPRB, 0.49905E+00_JPRB, 0.47349E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.45531E+00_JPRB, 0.43601E+00_JPRB, 0.44952E+00_JPRB, 0.47666E+00_JPRB, 0.50819E+00_JPRB, 0.55869E+00_JPRB, 0.62442E+00_JPRB ,&
&  0.68905E+00_JPRB, 0.75950E+00_JPRB, 0.87670E+00_JPRB, 0.92586E+00_JPRB, 0.98287E+00_JPRB, 0.10734E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.12683E+01_JPRB, 0.13804E+01_JPRB, 0.14920E+01_JPRB, 0.16011E+01_JPRB, 0.17304E+01_JPRB, 0.18676E+01_JPRB, 0.19888E+01_JPRB ,&
&  0.21472E+01_JPRB, 0.23095E+01_JPRB, 0.25218E+01_JPRB, 0.27474E+01_JPRB, 0.29570E+01_JPRB, 0.30134E+01_JPRB, 0.31583E+01_JPRB ,&
&  0.20560E+01_JPRB, 0.28864E+01_JPRB, 0.29619E+01_JPRB, 0.29564E+01_JPRB, 0.28695E+01_JPRB, 0.27954E+01_JPRB, 0.28646E+01_JPRB ,&
&  0.29710E+01_JPRB, 0.28861E+01_JPRB, 0.30229E+01_JPRB, 0.31999E+01_JPRB, 0.40074E+01_JPRB, 0.42272E+01_JPRB, 0.45155E+01_JPRB ,&
&  0.48180E+01_JPRB, 0.51062E+01_JPRB, 0.54404E+01_JPRB, 0.58441E+01_JPRB, 0.62506E+01_JPRB, 0.66319E+01_JPRB, 0.70180E+01_JPRB ,&
&  0.73745E+01_JPRB, 0.77057E+01_JPRB, 0.80430E+01_JPRB, 0.66392E+01_JPRB, 0.54712E+01_JPRB, 0.44179E+01_JPRB, 0.28202E+01_JPRB ,&
&  0.17120E+01_JPRB, 0.12602E+01_JPRB, 0.12441E+01_JPRB, 0.12307E+01_JPRB, 0.10631E+01_JPRB, 0.98009E+00_JPRB, 0.87818E+00_JPRB ,&
&  0.87645E+00_JPRB, 0.91101E+00_JPRB, 0.93454E+00_JPRB, 0.94087E+00_JPRB, 0.94182E+00_JPRB, 0.85504E+00_JPRB, 0.79952E+00_JPRB ,&
&  0.82817E+00_JPRB, 0.83584E+00_JPRB, 0.86928E+00_JPRB, 0.93694E+00_JPRB, 0.10422E+01_JPRB, 0.11141E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.10951E+01_JPRB, 0.95613E+00_JPRB, 0.89182E+00_JPRB, 0.89417E+00_JPRB, 0.84309E+00_JPRB, 0.79292E+00_JPRB, 0.80096E+00_JPRB ,&
&  0.88138E+00_JPRB, 0.99315E+00_JPRB, 0.10815E+01_JPRB, 0.10515E+01_JPRB, 0.10595E+01_JPRB, 0.85377E+00_JPRB, 0.66152E+00_JPRB ,&
&  0.62178E+00_JPRB, 0.61289E+00_JPRB, 0.56470E+00_JPRB, 0.52915E+00_JPRB, 0.48955E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.56555E+00_JPRB, 0.49680E+00_JPRB, 0.49313E+00_JPRB, 0.53992E+00_JPRB, 0.58681E+00_JPRB, 0.63735E+00_JPRB, 0.68883E+00_JPRB ,&
&  0.74959E+00_JPRB, 0.78985E+00_JPRB, 0.86171E+00_JPRB, 0.91090E+00_JPRB, 0.95735E+00_JPRB, 0.10030E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.11588E+01_JPRB, 0.12733E+01_JPRB, 0.13867E+01_JPRB, 0.15295E+01_JPRB, 0.16822E+01_JPRB, 0.18234E+01_JPRB, 0.19790E+01_JPRB ,&
&  0.21348E+01_JPRB, 0.23089E+01_JPRB, 0.24910E+01_JPRB, 0.27012E+01_JPRB, 0.28968E+01_JPRB, 0.29777E+01_JPRB, 0.29397E+01_JPRB ,&
&  0.14586E+01_JPRB, 0.23542E+01_JPRB, 0.30106E+01_JPRB, 0.29436E+01_JPRB, 0.28792E+01_JPRB, 0.27640E+01_JPRB, 0.27890E+01_JPRB ,&
&  0.26793E+01_JPRB, 0.24945E+01_JPRB, 0.26907E+01_JPRB, 0.36416E+01_JPRB, 0.41429E+01_JPRB, 0.44193E+01_JPRB, 0.45918E+01_JPRB ,&
&  0.48056E+01_JPRB, 0.49704E+01_JPRB, 0.51699E+01_JPRB, 0.53872E+01_JPRB, 0.56276E+01_JPRB, 0.58590E+01_JPRB, 0.60740E+01_JPRB ,&
&  0.62665E+01_JPRB, 0.63842E+01_JPRB, 0.63563E+01_JPRB, 0.50145E+01_JPRB, 0.49234E+01_JPRB, 0.41475E+01_JPRB, 0.34927E+01_JPRB ,&
&  0.20651E+01_JPRB, 0.13590E+01_JPRB, 0.12966E+01_JPRB, 0.12787E+01_JPRB, 0.11007E+01_JPRB, 0.99421E+00_JPRB, 0.85942E+00_JPRB ,&
&  0.84109E+00_JPRB, 0.83649E+00_JPRB, 0.85390E+00_JPRB, 0.85601E+00_JPRB, 0.85788E+00_JPRB, 0.78279E+00_JPRB, 0.78445E+00_JPRB ,&
&  0.81152E+00_JPRB, 0.82405E+00_JPRB, 0.85531E+00_JPRB, 0.90509E+00_JPRB, 0.96941E+00_JPRB, 0.10247E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.10082E+01_JPRB, 0.99126E+00_JPRB, 0.97720E+00_JPRB, 0.95791E+00_JPRB, 0.93550E+00_JPRB, 0.90795E+00_JPRB ,&
&  0.87222E+00_JPRB, 0.87023E+00_JPRB, 0.89019E+00_JPRB, 0.81202E+00_JPRB, 0.84939E+00_JPRB, 0.77449E+00_JPRB, 0.75997E+00_JPRB ,&
&  0.74121E+00_JPRB, 0.72592E+00_JPRB, 0.67944E+00_JPRB, 0.65458E+00_JPRB, 0.61550E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.69815E+00_JPRB, 0.60876E+00_JPRB, 0.54212E+00_JPRB, 0.55761E+00_JPRB, 0.62858E+00_JPRB, 0.68877E+00_JPRB, 0.72515E+00_JPRB ,&
&  0.77068E+00_JPRB, 0.79093E+00_JPRB, 0.80714E+00_JPRB, 0.82778E+00_JPRB, 0.87176E+00_JPRB, 0.90971E+00_JPRB, 0.97954E+00_JPRB ,&
&  0.10458E+01_JPRB, 0.11134E+01_JPRB, 0.11856E+01_JPRB, 0.12994E+01_JPRB, 0.14429E+01_JPRB, 0.15865E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.18818E+01_JPRB, 0.20786E+01_JPRB, 0.23433E+01_JPRB, 0.26588E+01_JPRB, 0.30980E+01_JPRB, 0.35830E+01_JPRB, 0.39527E+01_JPRB ,&
&  0.27385E+01_JPRB, 0.11921E+01_JPRB, 0.25052E+01_JPRB, 0.29748E+01_JPRB, 0.27942E+01_JPRB, 0.27025E+01_JPRB, 0.25595E+01_JPRB ,&
&  0.24345E+01_JPRB, 0.23392E+01_JPRB, 0.23725E+01_JPRB, 0.30723E+01_JPRB, 0.39257E+01_JPRB, 0.42339E+01_JPRB, 0.43473E+01_JPRB ,&
&  0.44584E+01_JPRB, 0.45035E+01_JPRB, 0.45767E+01_JPRB, 0.46860E+01_JPRB, 0.48229E+01_JPRB, 0.49078E+01_JPRB, 0.49721E+01_JPRB ,&
&  0.50197E+01_JPRB, 0.50053E+01_JPRB, 0.48172E+01_JPRB, 0.34694E+01_JPRB, 0.31008E+01_JPRB, 0.26658E+01_JPRB, 0.24071E+01_JPRB ,&
&  0.20807E+01_JPRB, 0.15611E+01_JPRB, 0.13971E+01_JPRB, 0.13298E+01_JPRB, 0.11723E+01_JPRB, 0.99963E+00_JPRB, 0.90498E+00_JPRB ,&
&  0.87556E+00_JPRB, 0.84599E+00_JPRB, 0.82465E+00_JPRB, 0.80138E+00_JPRB, 0.83644E+00_JPRB, 0.83303E+00_JPRB, 0.81991E+00_JPRB ,&
&  0.81681E+00_JPRB, 0.86873E+00_JPRB, 0.92279E+00_JPRB, 0.97010E+00_JPRB, 0.10189E+01_JPRB, 0.10736E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.10951E+01_JPRB, 0.10810E+01_JPRB, 0.10509E+01_JPRB, 0.10260E+01_JPRB, 0.98597E+00_JPRB, 0.97835E+00_JPRB ,&
&  0.97249E+00_JPRB, 0.97207E+00_JPRB, 0.94933E+00_JPRB, 0.92117E+00_JPRB, 0.93373E+00_JPRB, 0.91351E+00_JPRB, 0.91214E+00_JPRB ,&
&  0.90231E+00_JPRB, 0.87927E+00_JPRB, 0.82359E+00_JPRB, 0.77521E+00_JPRB, 0.75079E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.78884E+00_JPRB, 0.72213E+00_JPRB, 0.64311E+00_JPRB, 0.58648E+00_JPRB, 0.60309E+00_JPRB, 0.67507E+00_JPRB, 0.73577E+00_JPRB ,&
&  0.76124E+00_JPRB, 0.76717E+00_JPRB, 0.76279E+00_JPRB, 0.76813E+00_JPRB, 0.79304E+00_JPRB, 0.80438E+00_JPRB, 0.84314E+00_JPRB ,&
&  0.90320E+00_JPRB, 0.95687E+00_JPRB, 0.99932E+00_JPRB, 0.10709E+01_JPRB, 0.11507E+01_JPRB, 0.12799E+01_JPRB, 0.13991E+01_JPRB ,&
&  0.14996E+01_JPRB, 0.16092E+01_JPRB, 0.17798E+01_JPRB, 0.20187E+01_JPRB, 0.23603E+01_JPRB, 0.29621E+01_JPRB, 0.40065E+01_JPRB ,&
&  0.58979E+01_JPRB, 0.39309E+01_JPRB, 0.16134E+01_JPRB, 0.28065E+01_JPRB, 0.29634E+01_JPRB, 0.28293E+01_JPRB, 0.24435E+01_JPRB ,&
&  0.24317E+01_JPRB, 0.22033E+01_JPRB, 0.20753E+01_JPRB, 0.23656E+01_JPRB, 0.28466E+01_JPRB, 0.34383E+01_JPRB, 0.37933E+01_JPRB ,&
&  0.38872E+01_JPRB, 0.39336E+01_JPRB, 0.39714E+01_JPRB, 0.40582E+01_JPRB, 0.41878E+01_JPRB, 0.43019E+01_JPRB, 0.43265E+01_JPRB ,&
&  0.43631E+01_JPRB, 0.44456E+01_JPRB, 0.43403E+01_JPRB, 0.32025E+01_JPRB, 0.29380E+01_JPRB, 0.28133E+01_JPRB, 0.26540E+01_JPRB ,&
&  0.22967E+01_JPRB, 0.20117E+01_JPRB, 0.17730E+01_JPRB, 0.14737E+01_JPRB, 0.12571E+01_JPRB, 0.11342E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.93648E+00_JPRB, 0.90510E+00_JPRB, 0.87253E+00_JPRB, 0.88782E+00_JPRB, 0.92762E+00_JPRB, 0.93950E+00_JPRB ,&
&  0.97024E+00_JPRB, 0.10159E+01_JPRB, 0.10480E+01_JPRB, 0.10778E+01_JPRB, 0.11166E+01_JPRB, 0.11492E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.11921E+01_JPRB, 0.11866E+01_JPRB, 0.11670E+01_JPRB, 0.11401E+01_JPRB, 0.10888E+01_JPRB, 0.10444E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10426E+01_JPRB, 0.10191E+01_JPRB, 0.10206E+01_JPRB, 0.10066E+01_JPRB, 0.10182E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.10292E+01_JPRB, 0.93802E+00_JPRB, 0.84391E+00_JPRB, 0.81585E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.83825E+00_JPRB, 0.81418E+00_JPRB, 0.76309E+00_JPRB, 0.66642E+00_JPRB, 0.62093E+00_JPRB, 0.63168E+00_JPRB, 0.70072E+00_JPRB ,&
&  0.73392E+00_JPRB, 0.74818E+00_JPRB, 0.73691E+00_JPRB, 0.72638E+00_JPRB, 0.72655E+00_JPRB, 0.74015E+00_JPRB, 0.77458E+00_JPRB ,&
&  0.81867E+00_JPRB, 0.82662E+00_JPRB, 0.82043E+00_JPRB, 0.83138E+00_JPRB, 0.88521E+00_JPRB, 0.96132E+00_JPRB, 0.10885E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.12543E+01_JPRB, 0.13229E+01_JPRB, 0.14764E+01_JPRB, 0.16631E+01_JPRB, 0.17637E+01_JPRB, 0.20615E+01_JPRB ,&
&  0.33176E+01_JPRB, 0.51472E+01_JPRB, 0.19233E+01_JPRB, 0.29100E+01_JPRB, 0.33792E+01_JPRB, 0.30424E+01_JPRB, 0.27683E+01_JPRB ,&
&  0.26697E+01_JPRB, 0.22077E+01_JPRB, 0.17885E+01_JPRB, 0.18398E+01_JPRB, 0.20880E+01_JPRB, 0.23071E+01_JPRB, 0.26576E+01_JPRB ,&
&  0.29766E+01_JPRB, 0.31810E+01_JPRB, 0.33442E+01_JPRB, 0.34806E+01_JPRB, 0.36610E+01_JPRB, 0.39003E+01_JPRB, 0.41136E+01_JPRB ,&
&  0.42729E+01_JPRB, 0.45018E+01_JPRB, 0.46635E+01_JPRB, 0.36969E+01_JPRB, 0.35210E+01_JPRB, 0.37648E+01_JPRB, 0.35379E+01_JPRB ,&
&  0.30214E+01_JPRB, 0.26191E+01_JPRB, 0.21727E+01_JPRB, 0.17244E+01_JPRB, 0.14271E+01_JPRB, 0.13066E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.10526E+01_JPRB, 0.10335E+01_JPRB, 0.10243E+01_JPRB, 0.10256E+01_JPRB, 0.10410E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.11327E+01_JPRB, 0.11826E+01_JPRB, 0.12163E+01_JPRB, 0.12320E+01_JPRB, 0.12431E+01_JPRB, 0.12541E+01_JPRB, 0.12407E+01_JPRB ,&
&  0.12438E+01_JPRB, 0.12326E+01_JPRB, 0.12129E+01_JPRB, 0.11855E+01_JPRB, 0.11544E+01_JPRB, 0.11373E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.10961E+01_JPRB, 0.10512E+01_JPRB, 0.10253E+01_JPRB, 0.10484E+01_JPRB, 0.10179E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.11545E+01_JPRB, 0.10839E+01_JPRB, 0.87964E+00_JPRB, 0.84972E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.88295E+00_JPRB, 0.86892E+00_JPRB, 0.84804E+00_JPRB, 0.77854E+00_JPRB, 0.70602E+00_JPRB, 0.66715E+00_JPRB, 0.67947E+00_JPRB ,&
&  0.71945E+00_JPRB, 0.74838E+00_JPRB, 0.75349E+00_JPRB, 0.74755E+00_JPRB, 0.75138E+00_JPRB, 0.76881E+00_JPRB, 0.79215E+00_JPRB ,&
&  0.80897E+00_JPRB, 0.80673E+00_JPRB, 0.78880E+00_JPRB, 0.78958E+00_JPRB, 0.79626E+00_JPRB, 0.81648E+00_JPRB, 0.88076E+00_JPRB ,&
&  0.97030E+00_JPRB, 0.10231E+01_JPRB, 0.10401E+01_JPRB, 0.10928E+01_JPRB, 0.12369E+01_JPRB, 0.12943E+01_JPRB, 0.13753E+01_JPRB ,&
&  0.18612E+01_JPRB, 0.39210E+01_JPRB, 0.41383E+01_JPRB, 0.47118E+01_JPRB, 0.41940E+01_JPRB, 0.34870E+01_JPRB, 0.33202E+01_JPRB ,&
&  0.33220E+01_JPRB, 0.27060E+01_JPRB, 0.21524E+01_JPRB, 0.17468E+01_JPRB, 0.17624E+01_JPRB, 0.18780E+01_JPRB, 0.19174E+01_JPRB ,&
&  0.20667E+01_JPRB, 0.23965E+01_JPRB, 0.26034E+01_JPRB, 0.28036E+01_JPRB, 0.29942E+01_JPRB, 0.32166E+01_JPRB, 0.35331E+01_JPRB ,&
&  0.39113E+01_JPRB, 0.43940E+01_JPRB, 0.49925E+01_JPRB, 0.46903E+01_JPRB, 0.41092E+01_JPRB, 0.47414E+01_JPRB, 0.49191E+01_JPRB ,&
&  0.46499E+01_JPRB, 0.35371E+01_JPRB, 0.26926E+01_JPRB, 0.20775E+01_JPRB, 0.16696E+01_JPRB, 0.16667E+01_JPRB, 0.14322E+01_JPRB ,&
&  0.12592E+01_JPRB, 0.11925E+01_JPRB, 0.11605E+01_JPRB, 0.11661E+01_JPRB, 0.11761E+01_JPRB, 0.11650E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.12749E+01_JPRB, 0.12975E+01_JPRB, 0.13132E+01_JPRB, 0.13303E+01_JPRB, 0.13414E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.12847E+01_JPRB, 0.12586E+01_JPRB, 0.11852E+01_JPRB, 0.11754E+01_JPRB, 0.11738E+01_JPRB, 0.11804E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.11368E+01_JPRB, 0.10994E+01_JPRB, 0.10702E+01_JPRB, 0.10750E+01_JPRB, 0.10554E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10249E+01_JPRB, 0.10018E+01_JPRB, 0.95926E+00_JPRB, 0.92413E+00_JPRB, 0.89944E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.94240E+00_JPRB, 0.92840E+00_JPRB, 0.91904E+00_JPRB, 0.88284E+00_JPRB, 0.83547E+00_JPRB, 0.78998E+00_JPRB, 0.76261E+00_JPRB ,&
&  0.76191E+00_JPRB, 0.77583E+00_JPRB, 0.79012E+00_JPRB, 0.78291E+00_JPRB, 0.78353E+00_JPRB, 0.80424E+00_JPRB, 0.81971E+00_JPRB ,&
&  0.83234E+00_JPRB, 0.83354E+00_JPRB, 0.82660E+00_JPRB, 0.82504E+00_JPRB, 0.81972E+00_JPRB, 0.80433E+00_JPRB, 0.80538E+00_JPRB ,&
&  0.84140E+00_JPRB, 0.87833E+00_JPRB, 0.89599E+00_JPRB, 0.90835E+00_JPRB, 0.98925E+00_JPRB, 0.10566E+01_JPRB, 0.11432E+01_JPRB ,&
&  0.13861E+01_JPRB, 0.33430E+01_JPRB, 0.28400E+01_JPRB, 0.54538E+01_JPRB, 0.46205E+01_JPRB, 0.38601E+01_JPRB, 0.32788E+01_JPRB ,&
&  0.31483E+01_JPRB, 0.32429E+01_JPRB, 0.25304E+01_JPRB, 0.20394E+01_JPRB, 0.17782E+01_JPRB, 0.17299E+01_JPRB, 0.16760E+01_JPRB ,&
&  0.16263E+01_JPRB, 0.16817E+01_JPRB, 0.18048E+01_JPRB, 0.19946E+01_JPRB, 0.22287E+01_JPRB, 0.24364E+01_JPRB, 0.26810E+01_JPRB ,&
&  0.29746E+01_JPRB, 0.34631E+01_JPRB, 0.42477E+01_JPRB, 0.44331E+01_JPRB, 0.45537E+01_JPRB, 0.53450E+01_JPRB, 0.61901E+01_JPRB ,&
&  0.61046E+01_JPRB, 0.48487E+01_JPRB, 0.33982E+01_JPRB, 0.24701E+01_JPRB, 0.20270E+01_JPRB, 0.18957E+01_JPRB, 0.16383E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.13292E+01_JPRB, 0.12886E+01_JPRB, 0.12741E+01_JPRB, 0.12846E+01_JPRB, 0.12877E+01_JPRB, 0.12978E+01_JPRB ,&
&  0.13154E+01_JPRB, 0.13512E+01_JPRB, 0.13524E+01_JPRB, 0.13433E+01_JPRB, 0.13438E+01_JPRB, 0.13477E+01_JPRB, 0.13366E+01_JPRB ,&
&  0.13302E+01_JPRB, 0.13100E+01_JPRB, 0.12171E+01_JPRB, 0.11730E+01_JPRB, 0.11736E+01_JPRB, 0.11731E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.11774E+01_JPRB, 0.11363E+01_JPRB, 0.10809E+01_JPRB, 0.10821E+01_JPRB, 0.10920E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.99187E+00_JPRB, 0.98265E+00_JPRB, 0.97372E+00_JPRB, 0.95788E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.95732E+00_JPRB, 0.94349E+00_JPRB, 0.93889E+00_JPRB, 0.92172E+00_JPRB, 0.88947E+00_JPRB, 0.85430E+00_JPRB, 0.83102E+00_JPRB ,&
&  0.81399E+00_JPRB, 0.81272E+00_JPRB, 0.82322E+00_JPRB, 0.81604E+00_JPRB, 0.82155E+00_JPRB, 0.83758E+00_JPRB, 0.84789E+00_JPRB ,&
&  0.84682E+00_JPRB, 0.84502E+00_JPRB, 0.83557E+00_JPRB, 0.83007E+00_JPRB, 0.82636E+00_JPRB, 0.81170E+00_JPRB, 0.80237E+00_JPRB ,&
&  0.80538E+00_JPRB, 0.82119E+00_JPRB, 0.83594E+00_JPRB, 0.85008E+00_JPRB, 0.89234E+00_JPRB, 0.93952E+00_JPRB, 0.10078E+01_JPRB ,&
&  0.12556E+01_JPRB, 0.37397E+01_JPRB, 0.26650E+01_JPRB, 0.50962E+01_JPRB, 0.44460E+01_JPRB, 0.36356E+01_JPRB, 0.29974E+01_JPRB ,&
&  0.29164E+01_JPRB, 0.28776E+01_JPRB, 0.25416E+01_JPRB, 0.22807E+01_JPRB, 0.20441E+01_JPRB, 0.18208E+01_JPRB, 0.16635E+01_JPRB ,&
&  0.15289E+01_JPRB, 0.14575E+01_JPRB, 0.14508E+01_JPRB, 0.14982E+01_JPRB, 0.16296E+01_JPRB, 0.17980E+01_JPRB, 0.19634E+01_JPRB ,&
&  0.21433E+01_JPRB, 0.24308E+01_JPRB, 0.29787E+01_JPRB, 0.38812E+01_JPRB, 0.41694E+01_JPRB, 0.48493E+01_JPRB, 0.72884E+01_JPRB ,&
&  0.75143E+01_JPRB, 0.64112E+01_JPRB, 0.43512E+01_JPRB, 0.29042E+01_JPRB, 0.23262E+01_JPRB, 0.20916E+01_JPRB, 0.18658E+01_JPRB ,&
&  0.16494E+01_JPRB, 0.14793E+01_JPRB, 0.14164E+01_JPRB, 0.13520E+01_JPRB, 0.13233E+01_JPRB, 0.13540E+01_JPRB, 0.14033E+01_JPRB ,&
&  0.14281E+01_JPRB, 0.14144E+01_JPRB, 0.14076E+01_JPRB, 0.13644E+01_JPRB, 0.13311E+01_JPRB, 0.13047E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.13245E+01_JPRB, 0.12635E+01_JPRB, 0.12030E+01_JPRB, 0.11891E+01_JPRB, 0.11924E+01_JPRB, 0.11802E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.11990E+01_JPRB, 0.11541E+01_JPRB, 0.11169E+01_JPRB, 0.10829E+01_JPRB, 0.10952E+01_JPRB, 0.10175E+01_JPRB ,&
&  0.98604E+00_JPRB, 0.97327E+00_JPRB, 0.97259E+00_JPRB, 0.97929E+00_JPRB, 0.97645E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.96742E+00_JPRB, 0.94987E+00_JPRB, 0.94564E+00_JPRB, 0.93975E+00_JPRB, 0.92413E+00_JPRB, 0.90454E+00_JPRB, 0.89632E+00_JPRB ,&
&  0.88634E+00_JPRB, 0.87810E+00_JPRB, 0.86606E+00_JPRB, 0.84815E+00_JPRB, 0.84766E+00_JPRB, 0.84955E+00_JPRB, 0.84952E+00_JPRB ,&
&  0.84666E+00_JPRB, 0.84439E+00_JPRB, 0.83069E+00_JPRB, 0.82366E+00_JPRB, 0.82476E+00_JPRB, 0.81910E+00_JPRB, 0.80600E+00_JPRB ,&
&  0.79370E+00_JPRB, 0.79284E+00_JPRB, 0.80653E+00_JPRB, 0.82286E+00_JPRB, 0.85536E+00_JPRB, 0.87713E+00_JPRB, 0.93430E+00_JPRB ,&
&  0.12425E+01_JPRB, 0.12732E+01_JPRB, 0.32437E+01_JPRB, 0.43912E+01_JPRB, 0.42487E+01_JPRB, 0.35970E+01_JPRB, 0.31455E+01_JPRB ,&
&  0.29187E+01_JPRB, 0.26826E+01_JPRB, 0.24994E+01_JPRB, 0.22639E+01_JPRB, 0.20089E+01_JPRB, 0.17339E+01_JPRB, 0.15847E+01_JPRB ,&
&  0.14987E+01_JPRB, 0.14196E+01_JPRB, 0.13878E+01_JPRB, 0.13929E+01_JPRB, 0.14215E+01_JPRB, 0.14694E+01_JPRB, 0.15534E+01_JPRB ,&
&  0.16463E+01_JPRB, 0.17604E+01_JPRB, 0.19994E+01_JPRB, 0.26814E+01_JPRB, 0.37611E+01_JPRB, 0.46314E+01_JPRB, 0.64633E+01_JPRB ,&
&  0.80879E+01_JPRB, 0.74449E+01_JPRB, 0.51942E+01_JPRB, 0.34228E+01_JPRB, 0.26291E+01_JPRB, 0.23073E+01_JPRB, 0.20621E+01_JPRB ,&
&  0.18664E+01_JPRB, 0.16615E+01_JPRB, 0.15195E+01_JPRB, 0.14185E+01_JPRB, 0.13544E+01_JPRB, 0.13367E+01_JPRB, 0.14070E+01_JPRB ,&
&  0.14680E+01_JPRB, 0.14807E+01_JPRB, 0.14397E+01_JPRB, 0.13878E+01_JPRB, 0.13456E+01_JPRB, 0.13125E+01_JPRB, 0.13058E+01_JPRB ,&
&  0.13192E+01_JPRB, 0.13301E+01_JPRB, 0.12905E+01_JPRB, 0.12417E+01_JPRB, 0.12262E+01_JPRB, 0.12376E+01_JPRB, 0.12421E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.11983E+01_JPRB, 0.11472E+01_JPRB, 0.11210E+01_JPRB, 0.10605E+01_JPRB, 0.10771E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.97138E+00_JPRB, 0.96191E+00_JPRB, 0.96924E+00_JPRB, 0.97649E+00_JPRB, 0.99084E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.97614E+00_JPRB, 0.94882E+00_JPRB, 0.94054E+00_JPRB, 0.94529E+00_JPRB, 0.94280E+00_JPRB, 0.94701E+00_JPRB, 0.94771E+00_JPRB ,&
&  0.93591E+00_JPRB, 0.91016E+00_JPRB, 0.88925E+00_JPRB, 0.86828E+00_JPRB, 0.86113E+00_JPRB, 0.85411E+00_JPRB, 0.85325E+00_JPRB ,&
&  0.84468E+00_JPRB, 0.83800E+00_JPRB, 0.82653E+00_JPRB, 0.82312E+00_JPRB, 0.82255E+00_JPRB, 0.81537E+00_JPRB, 0.79088E+00_JPRB ,&
&  0.78116E+00_JPRB, 0.78221E+00_JPRB, 0.79832E+00_JPRB, 0.81773E+00_JPRB, 0.84146E+00_JPRB, 0.85355E+00_JPRB, 0.88743E+00_JPRB ,&
&  0.11783E+01_JPRB, 0.21394E+01_JPRB, 0.33872E+01_JPRB, 0.35646E+01_JPRB, 0.35314E+01_JPRB, 0.31820E+01_JPRB, 0.28782E+01_JPRB ,&
&  0.25917E+01_JPRB, 0.24674E+01_JPRB, 0.24468E+01_JPRB, 0.22044E+01_JPRB, 0.18371E+01_JPRB, 0.15470E+01_JPRB, 0.14318E+01_JPRB ,&
&  0.13697E+01_JPRB, 0.13225E+01_JPRB, 0.12998E+01_JPRB, 0.13023E+01_JPRB, 0.13080E+01_JPRB, 0.13175E+01_JPRB, 0.13444E+01_JPRB ,&
&  0.13781E+01_JPRB, 0.14253E+01_JPRB, 0.15128E+01_JPRB, 0.18475E+01_JPRB, 0.30898E+01_JPRB, 0.35528E+01_JPRB, 0.55347E+01_JPRB ,&
&  0.81003E+01_JPRB, 0.71895E+01_JPRB, 0.54715E+01_JPRB, 0.39335E+01_JPRB, 0.30654E+01_JPRB, 0.26347E+01_JPRB, 0.23093E+01_JPRB ,&
&  0.20545E+01_JPRB, 0.18411E+01_JPRB, 0.16577E+01_JPRB, 0.15378E+01_JPRB, 0.14363E+01_JPRB, 0.13679E+01_JPRB, 0.13723E+01_JPRB ,&
&  0.14234E+01_JPRB, 0.14524E+01_JPRB, 0.14283E+01_JPRB, 0.13863E+01_JPRB, 0.13505E+01_JPRB, 0.13399E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.13358E+01_JPRB, 0.13053E+01_JPRB, 0.12464E+01_JPRB, 0.12312E+01_JPRB, 0.12501E+01_JPRB, 0.12477E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.11508E+01_JPRB, 0.11456E+01_JPRB, 0.11048E+01_JPRB, 0.10751E+01_JPRB, 0.10846E+01_JPRB, 0.10071E+01_JPRB ,&
&  0.96157E+00_JPRB, 0.94974E+00_JPRB, 0.96798E+00_JPRB, 0.99035E+00_JPRB, 0.10217E+01_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.99925E+00_JPRB, 0.95509E+00_JPRB, 0.93184E+00_JPRB, 0.93399E+00_JPRB, 0.94343E+00_JPRB, 0.94319E+00_JPRB, 0.94796E+00_JPRB ,&
&  0.93722E+00_JPRB, 0.91511E+00_JPRB, 0.89068E+00_JPRB, 0.87014E+00_JPRB, 0.85919E+00_JPRB, 0.85178E+00_JPRB, 0.84603E+00_JPRB ,&
&  0.84020E+00_JPRB, 0.83340E+00_JPRB, 0.82359E+00_JPRB, 0.82561E+00_JPRB, 0.82752E+00_JPRB, 0.82453E+00_JPRB, 0.79834E+00_JPRB ,&
&  0.78257E+00_JPRB, 0.78465E+00_JPRB, 0.79716E+00_JPRB, 0.80805E+00_JPRB, 0.82391E+00_JPRB, 0.83929E+00_JPRB, 0.86730E+00_JPRB ,&
&  0.10298E+01_JPRB, 0.18783E+01_JPRB, 0.25867E+01_JPRB, 0.30420E+01_JPRB, 0.31744E+01_JPRB, 0.30141E+01_JPRB, 0.27055E+01_JPRB ,&
&  0.24702E+01_JPRB, 0.23372E+01_JPRB, 0.22144E+01_JPRB, 0.19148E+01_JPRB, 0.15959E+01_JPRB, 0.14188E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12686E+01_JPRB, 0.12524E+01_JPRB, 0.12409E+01_JPRB, 0.12300E+01_JPRB, 0.12328E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.12727E+01_JPRB, 0.13038E+01_JPRB, 0.13611E+01_JPRB, 0.18941E+01_JPRB, 0.27386E+01_JPRB, 0.40592E+01_JPRB ,&
&  0.47156E+01_JPRB, 0.48081E+01_JPRB, 0.43491E+01_JPRB, 0.37521E+01_JPRB, 0.32374E+01_JPRB, 0.28308E+01_JPRB, 0.24790E+01_JPRB ,&
&  0.21628E+01_JPRB, 0.18861E+01_JPRB, 0.16613E+01_JPRB, 0.15157E+01_JPRB, 0.14237E+01_JPRB, 0.13516E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.13563E+01_JPRB, 0.13626E+01_JPRB, 0.13582E+01_JPRB, 0.13359E+01_JPRB, 0.13115E+01_JPRB, 0.12954E+01_JPRB ,&
&  0.12938E+01_JPRB, 0.12889E+01_JPRB, 0.12870E+01_JPRB, 0.12543E+01_JPRB, 0.12193E+01_JPRB, 0.11961E+01_JPRB, 0.11839E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.11377E+01_JPRB, 0.11120E+01_JPRB, 0.10940E+01_JPRB, 0.10656E+01_JPRB, 0.10756E+01_JPRB, 0.10023E+01_JPRB ,&
&  0.96053E+00_JPRB, 0.94819E+00_JPRB, 0.95034E+00_JPRB, 0.96693E+00_JPRB, 0.10063E+01_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.10121E+01_JPRB, 0.98042E+00_JPRB, 0.94938E+00_JPRB, 0.93291E+00_JPRB, 0.92809E+00_JPRB, 0.92741E+00_JPRB, 0.92710E+00_JPRB ,&
&  0.92297E+00_JPRB, 0.91319E+00_JPRB, 0.88891E+00_JPRB, 0.86916E+00_JPRB, 0.85458E+00_JPRB, 0.84837E+00_JPRB, 0.84297E+00_JPRB ,&
&  0.83917E+00_JPRB, 0.83450E+00_JPRB, 0.82661E+00_JPRB, 0.82438E+00_JPRB, 0.82562E+00_JPRB, 0.81921E+00_JPRB, 0.79823E+00_JPRB ,&
&  0.77548E+00_JPRB, 0.77511E+00_JPRB, 0.78557E+00_JPRB, 0.79656E+00_JPRB, 0.81219E+00_JPRB, 0.82571E+00_JPRB, 0.85325E+00_JPRB ,&
&  0.99477E+00_JPRB, 0.14672E+01_JPRB, 0.21624E+01_JPRB, 0.25454E+01_JPRB, 0.27027E+01_JPRB, 0.27168E+01_JPRB, 0.25277E+01_JPRB ,&
&  0.22856E+01_JPRB, 0.20386E+01_JPRB, 0.18380E+01_JPRB, 0.16267E+01_JPRB, 0.14746E+01_JPRB, 0.13584E+01_JPRB, 0.12785E+01_JPRB ,&
&  0.12357E+01_JPRB, 0.12207E+01_JPRB, 0.12057E+01_JPRB, 0.11877E+01_JPRB, 0.11741E+01_JPRB, 0.11663E+01_JPRB, 0.11644E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.11846E+01_JPRB, 0.12057E+01_JPRB, 0.12274E+01_JPRB, 0.13015E+01_JPRB, 0.16075E+01_JPRB, 0.23085E+01_JPRB ,&
&  0.29403E+01_JPRB, 0.33030E+01_JPRB, 0.32519E+01_JPRB, 0.31218E+01_JPRB, 0.29987E+01_JPRB, 0.27379E+01_JPRB, 0.24611E+01_JPRB ,&
&  0.21876E+01_JPRB, 0.19359E+01_JPRB, 0.16816E+01_JPRB, 0.14783E+01_JPRB, 0.13479E+01_JPRB, 0.12724E+01_JPRB, 0.12326E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.12584E+01_JPRB, 0.12737E+01_JPRB, 0.12889E+01_JPRB, 0.12823E+01_JPRB, 0.12587E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.11685E+01_JPRB, 0.11550E+01_JPRB, 0.11357E+01_JPRB, 0.11135E+01_JPRB, 0.11023E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.10941E+01_JPRB, 0.10527E+01_JPRB, 0.10464E+01_JPRB, 0.11077E+01_JPRB, 0.10602E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.97128E+00_JPRB, 0.95194E+00_JPRB, 0.94535E+00_JPRB, 0.95835E+00_JPRB, 0.99966E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.90963E+00_JPRB, 0.90804E+00_JPRB, 0.90611E+00_JPRB, 0.90951E+00_JPRB, 0.91142E+00_JPRB, 0.90360E+00_JPRB, 0.89702E+00_JPRB ,&
&  0.89514E+00_JPRB, 0.88969E+00_JPRB, 0.88124E+00_JPRB, 0.86813E+00_JPRB, 0.85147E+00_JPRB, 0.83819E+00_JPRB, 0.83273E+00_JPRB ,&
&  0.82525E+00_JPRB, 0.82141E+00_JPRB, 0.81944E+00_JPRB, 0.81370E+00_JPRB, 0.80704E+00_JPRB, 0.79695E+00_JPRB, 0.77913E+00_JPRB ,&
&  0.75545E+00_JPRB, 0.74649E+00_JPRB, 0.75695E+00_JPRB, 0.76896E+00_JPRB, 0.77948E+00_JPRB, 0.78585E+00_JPRB, 0.80938E+00_JPRB ,&
&  0.95257E+00_JPRB, 0.12487E+01_JPRB, 0.15589E+01_JPRB, 0.19203E+01_JPRB, 0.21320E+01_JPRB, 0.20756E+01_JPRB, 0.20174E+01_JPRB ,&
&  0.19013E+01_JPRB, 0.17779E+01_JPRB, 0.16562E+01_JPRB, 0.15236E+01_JPRB, 0.14082E+01_JPRB, 0.13160E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12163E+01_JPRB, 0.11812E+01_JPRB, 0.11534E+01_JPRB, 0.11242E+01_JPRB, 0.11023E+01_JPRB, 0.10874E+01_JPRB, 0.10855E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.10994E+01_JPRB, 0.11138E+01_JPRB, 0.11295E+01_JPRB, 0.11546E+01_JPRB, 0.12401E+01_JPRB, 0.15428E+01_JPRB ,&
&  0.19476E+01_JPRB, 0.23297E+01_JPRB, 0.25087E+01_JPRB, 0.24698E+01_JPRB, 0.24089E+01_JPRB, 0.23212E+01_JPRB, 0.21190E+01_JPRB ,&
&  0.18737E+01_JPRB, 0.17093E+01_JPRB, 0.15828E+01_JPRB, 0.14728E+01_JPRB, 0.13900E+01_JPRB, 0.13202E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.12262E+01_JPRB, 0.12231E+01_JPRB, 0.12193E+01_JPRB, 0.12059E+01_JPRB, 0.11724E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.10649E+01_JPRB, 0.10345E+01_JPRB, 0.10279E+01_JPRB, 0.10126E+01_JPRB, 0.99814E+00_JPRB, 0.99785E+00_JPRB, 0.10082E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.10196E+01_JPRB, 0.10247E+01_JPRB, 0.10548E+01_JPRB, 0.10499E+01_JPRB, 0.10238E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.97868E+00_JPRB, 0.94871E+00_JPRB, 0.92572E+00_JPRB, 0.92000E+00_JPRB, 0.87177E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.89224E+00_JPRB, 0.88822E+00_JPRB, 0.88167E+00_JPRB, 0.88216E+00_JPRB, 0.88529E+00_JPRB, 0.87272E+00_JPRB, 0.86005E+00_JPRB ,&
&  0.85296E+00_JPRB, 0.85298E+00_JPRB, 0.85499E+00_JPRB, 0.85393E+00_JPRB, 0.84456E+00_JPRB, 0.82884E+00_JPRB, 0.81826E+00_JPRB ,&
&  0.80549E+00_JPRB, 0.79515E+00_JPRB, 0.79172E+00_JPRB, 0.78963E+00_JPRB, 0.78234E+00_JPRB, 0.77399E+00_JPRB, 0.76650E+00_JPRB ,&
&  0.74984E+00_JPRB, 0.73711E+00_JPRB, 0.73436E+00_JPRB, 0.73932E+00_JPRB, 0.74600E+00_JPRB, 0.74830E+00_JPRB, 0.78183E+00_JPRB ,&
&  0.94099E+00_JPRB, 0.10223E+01_JPRB, 0.10532E+01_JPRB, 0.13139E+01_JPRB, 0.15391E+01_JPRB, 0.15736E+01_JPRB, 0.16016E+01_JPRB ,&
&  0.16480E+01_JPRB, 0.16387E+01_JPRB, 0.15804E+01_JPRB, 0.14603E+01_JPRB, 0.13566E+01_JPRB, 0.12722E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.10639E+01_JPRB, 0.10240E+01_JPRB, 0.99354E+00_JPRB, 0.97722E+00_JPRB, 0.95757E+00_JPRB, 0.94865E+00_JPRB ,&
&  0.95049E+00_JPRB, 0.96307E+00_JPRB, 0.97924E+00_JPRB, 0.99467E+00_JPRB, 0.10197E+01_JPRB, 0.10639E+01_JPRB, 0.11824E+01_JPRB ,&
&  0.14009E+01_JPRB, 0.17024E+01_JPRB, 0.19210E+01_JPRB, 0.19896E+01_JPRB, 0.19559E+01_JPRB, 0.18632E+01_JPRB, 0.17289E+01_JPRB ,&
&  0.15838E+01_JPRB, 0.14659E+01_JPRB, 0.13807E+01_JPRB, 0.13331E+01_JPRB, 0.12964E+01_JPRB, 0.12656E+01_JPRB, 0.12338E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.11867E+01_JPRB, 0.11712E+01_JPRB, 0.11522E+01_JPRB, 0.11267E+01_JPRB, 0.10798E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.96362E+00_JPRB, 0.92885E+00_JPRB, 0.91504E+00_JPRB, 0.90443E+00_JPRB, 0.89880E+00_JPRB, 0.91074E+00_JPRB, 0.92872E+00_JPRB ,&
&  0.93893E+00_JPRB, 0.95078E+00_JPRB, 0.97168E+00_JPRB, 0.95302E+00_JPRB, 0.99268E+00_JPRB, 0.99045E+00_JPRB, 0.99045E+00_JPRB ,&
&  0.97800E+00_JPRB, 0.95725E+00_JPRB, 0.93408E+00_JPRB, 0.81310E+00_JPRB, 0.87497E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.87716E+00_JPRB, 0.87195E+00_JPRB, 0.85756E+00_JPRB, 0.84792E+00_JPRB, 0.84134E+00_JPRB, 0.83289E+00_JPRB, 0.81911E+00_JPRB ,&
&  0.81092E+00_JPRB, 0.80894E+00_JPRB, 0.80790E+00_JPRB, 0.80789E+00_JPRB, 0.80648E+00_JPRB, 0.79812E+00_JPRB, 0.78600E+00_JPRB ,&
&  0.77445E+00_JPRB, 0.76497E+00_JPRB, 0.76601E+00_JPRB, 0.76424E+00_JPRB, 0.75745E+00_JPRB, 0.75180E+00_JPRB, 0.74503E+00_JPRB ,&
&  0.72935E+00_JPRB, 0.71207E+00_JPRB, 0.69892E+00_JPRB, 0.70246E+00_JPRB, 0.70999E+00_JPRB, 0.70676E+00_JPRB, 0.73645E+00_JPRB ,&
&  0.88431E+00_JPRB, 0.86963E+00_JPRB, 0.91448E+00_JPRB, 0.10639E+01_JPRB, 0.11613E+01_JPRB, 0.12500E+01_JPRB, 0.13150E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.14066E+01_JPRB, 0.13244E+01_JPRB, 0.12195E+01_JPRB, 0.11150E+01_JPRB, 0.10249E+01_JPRB, 0.95882E+00_JPRB ,&
&  0.91395E+00_JPRB, 0.88803E+00_JPRB, 0.86998E+00_JPRB, 0.85042E+00_JPRB, 0.83506E+00_JPRB, 0.82166E+00_JPRB, 0.81444E+00_JPRB ,&
&  0.81183E+00_JPRB, 0.81624E+00_JPRB, 0.82739E+00_JPRB, 0.85032E+00_JPRB, 0.87595E+00_JPRB, 0.90495E+00_JPRB, 0.95102E+00_JPRB ,&
&  0.10411E+01_JPRB, 0.12048E+01_JPRB, 0.13851E+01_JPRB, 0.14830E+01_JPRB, 0.14872E+01_JPRB, 0.14308E+01_JPRB, 0.13417E+01_JPRB ,&
&  0.12543E+01_JPRB, 0.11895E+01_JPRB, 0.11488E+01_JPRB, 0.11207E+01_JPRB, 0.10943E+01_JPRB, 0.10720E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10221E+01_JPRB, 0.10104E+01_JPRB, 0.99317E+00_JPRB, 0.96553E+00_JPRB, 0.92744E+00_JPRB, 0.88961E+00_JPRB ,&
&  0.85376E+00_JPRB, 0.82558E+00_JPRB, 0.80929E+00_JPRB, 0.80747E+00_JPRB, 0.81018E+00_JPRB, 0.82376E+00_JPRB, 0.84531E+00_JPRB ,&
&  0.86249E+00_JPRB, 0.87572E+00_JPRB, 0.88802E+00_JPRB, 0.91536E+00_JPRB, 0.93313E+00_JPRB, 0.95056E+00_JPRB, 0.96082E+00_JPRB ,&
&  0.96263E+00_JPRB, 0.96122E+00_JPRB, 0.78502E+00_JPRB, 0.85356E+00_JPRB, 0.86958E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.87138E+00_JPRB, 0.85040E+00_JPRB, 0.82564E+00_JPRB, 0.80780E+00_JPRB, 0.79784E+00_JPRB, 0.79301E+00_JPRB, 0.78698E+00_JPRB ,&
&  0.77921E+00_JPRB, 0.77803E+00_JPRB, 0.77869E+00_JPRB, 0.77905E+00_JPRB, 0.77832E+00_JPRB, 0.77283E+00_JPRB, 0.76209E+00_JPRB ,&
&  0.75017E+00_JPRB, 0.73674E+00_JPRB, 0.73421E+00_JPRB, 0.73286E+00_JPRB, 0.72575E+00_JPRB, 0.72043E+00_JPRB, 0.71904E+00_JPRB ,&
&  0.71188E+00_JPRB, 0.70035E+00_JPRB, 0.68019E+00_JPRB, 0.67439E+00_JPRB, 0.67799E+00_JPRB, 0.67883E+00_JPRB, 0.69196E+00_JPRB ,&
&  0.71921E+00_JPRB, 0.74216E+00_JPRB, 0.82344E+00_JPRB, 0.88384E+00_JPRB, 0.92506E+00_JPRB, 0.96678E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.10055E+01_JPRB, 0.96449E+00_JPRB, 0.90091E+00_JPRB, 0.85413E+00_JPRB, 0.82020E+00_JPRB, 0.79658E+00_JPRB ,&
&  0.78602E+00_JPRB, 0.77792E+00_JPRB, 0.76908E+00_JPRB, 0.76081E+00_JPRB, 0.75595E+00_JPRB, 0.74829E+00_JPRB, 0.74393E+00_JPRB ,&
&  0.74502E+00_JPRB, 0.74983E+00_JPRB, 0.75335E+00_JPRB, 0.76059E+00_JPRB, 0.77489E+00_JPRB, 0.79678E+00_JPRB, 0.82461E+00_JPRB ,&
&  0.85863E+00_JPRB, 0.91472E+00_JPRB, 0.10060E+01_JPRB, 0.10830E+01_JPRB, 0.11081E+01_JPRB, 0.10826E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.99215E+00_JPRB, 0.96537E+00_JPRB, 0.94652E+00_JPRB, 0.92927E+00_JPRB, 0.90754E+00_JPRB, 0.88754E+00_JPRB, 0.86962E+00_JPRB ,&
&  0.86174E+00_JPRB, 0.85004E+00_JPRB, 0.83626E+00_JPRB, 0.81902E+00_JPRB, 0.80222E+00_JPRB, 0.79073E+00_JPRB, 0.78224E+00_JPRB ,&
&  0.77338E+00_JPRB, 0.76133E+00_JPRB, 0.75373E+00_JPRB, 0.75481E+00_JPRB, 0.75733E+00_JPRB, 0.76469E+00_JPRB, 0.77752E+00_JPRB ,&
&  0.79399E+00_JPRB, 0.80933E+00_JPRB, 0.82499E+00_JPRB, 0.85393E+00_JPRB, 0.88328E+00_JPRB, 0.90906E+00_JPRB, 0.92555E+00_JPRB ,&
&  0.93611E+00_JPRB, 0.93005E+00_JPRB, 0.92405E+00_JPRB, 0.90801E+00_JPRB, 0.88835E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.80351E+00_JPRB, 0.77896E+00_JPRB, 0.75917E+00_JPRB, 0.74559E+00_JPRB, 0.73732E+00_JPRB, 0.73476E+00_JPRB, 0.73073E+00_JPRB ,&
&  0.72733E+00_JPRB, 0.72250E+00_JPRB, 0.71959E+00_JPRB, 0.72217E+00_JPRB, 0.72343E+00_JPRB, 0.72352E+00_JPRB, 0.72343E+00_JPRB ,&
&  0.71605E+00_JPRB, 0.70119E+00_JPRB, 0.68949E+00_JPRB, 0.68568E+00_JPRB, 0.68568E+00_JPRB, 0.68074E+00_JPRB, 0.67868E+00_JPRB ,&
&  0.68164E+00_JPRB, 0.67333E+00_JPRB, 0.65913E+00_JPRB, 0.64991E+00_JPRB, 0.65082E+00_JPRB, 0.64872E+00_JPRB, 0.65001E+00_JPRB ,&
&  0.69894E+00_JPRB, 0.66124E+00_JPRB, 0.72715E+00_JPRB, 0.74175E+00_JPRB, 0.75615E+00_JPRB, 0.76777E+00_JPRB, 0.77408E+00_JPRB ,&
&  0.77128E+00_JPRB, 0.76104E+00_JPRB, 0.74885E+00_JPRB, 0.73420E+00_JPRB, 0.72059E+00_JPRB, 0.70669E+00_JPRB, 0.69579E+00_JPRB ,&
&  0.69090E+00_JPRB, 0.68569E+00_JPRB, 0.68479E+00_JPRB, 0.68715E+00_JPRB, 0.69331E+00_JPRB, 0.69496E+00_JPRB, 0.69655E+00_JPRB ,&
&  0.69897E+00_JPRB, 0.70054E+00_JPRB, 0.70033E+00_JPRB, 0.69936E+00_JPRB, 0.69822E+00_JPRB, 0.70326E+00_JPRB, 0.71233E+00_JPRB ,&
&  0.72466E+00_JPRB, 0.74968E+00_JPRB, 0.78951E+00_JPRB, 0.82670E+00_JPRB, 0.84242E+00_JPRB, 0.83611E+00_JPRB, 0.81717E+00_JPRB ,&
&  0.80659E+00_JPRB, 0.80673E+00_JPRB, 0.80843E+00_JPRB, 0.80237E+00_JPRB, 0.79222E+00_JPRB, 0.78251E+00_JPRB, 0.76701E+00_JPRB ,&
&  0.74726E+00_JPRB, 0.72992E+00_JPRB, 0.71746E+00_JPRB, 0.70944E+00_JPRB, 0.70538E+00_JPRB, 0.70060E+00_JPRB, 0.69573E+00_JPRB ,&
&  0.69393E+00_JPRB, 0.69153E+00_JPRB, 0.69189E+00_JPRB, 0.69724E+00_JPRB, 0.70234E+00_JPRB, 0.70898E+00_JPRB, 0.71729E+00_JPRB ,&
&  0.72886E+00_JPRB, 0.74167E+00_JPRB, 0.75423E+00_JPRB, 0.77436E+00_JPRB, 0.80324E+00_JPRB, 0.82428E+00_JPRB, 0.83811E+00_JPRB ,&
&  0.85078E+00_JPRB, 0.85456E+00_JPRB, 0.84840E+00_JPRB, 0.83782E+00_JPRB, 0.82237E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.74559E+00_JPRB, 0.73098E+00_JPRB, 0.71937E+00_JPRB, 0.71011E+00_JPRB, 0.70363E+00_JPRB, 0.70153E+00_JPRB, 0.69868E+00_JPRB ,&
&  0.69572E+00_JPRB, 0.69268E+00_JPRB, 0.68471E+00_JPRB, 0.67752E+00_JPRB, 0.67730E+00_JPRB, 0.68362E+00_JPRB, 0.68820E+00_JPRB ,&
&  0.68435E+00_JPRB, 0.67230E+00_JPRB, 0.66103E+00_JPRB, 0.65803E+00_JPRB, 0.66015E+00_JPRB, 0.66195E+00_JPRB, 0.66111E+00_JPRB ,&
&  0.66265E+00_JPRB, 0.66391E+00_JPRB, 0.65553E+00_JPRB, 0.63758E+00_JPRB, 0.62965E+00_JPRB, 0.62539E+00_JPRB, 0.63314E+00_JPRB ,&
&  0.65503E+00_JPRB, 0.67447E+00_JPRB, 0.68285E+00_JPRB, 0.67768E+00_JPRB, 0.67721E+00_JPRB, 0.67522E+00_JPRB, 0.66952E+00_JPRB ,&
&  0.66381E+00_JPRB, 0.65667E+00_JPRB, 0.65061E+00_JPRB, 0.64611E+00_JPRB, 0.64326E+00_JPRB, 0.64158E+00_JPRB, 0.64210E+00_JPRB ,&
&  0.63900E+00_JPRB, 0.63405E+00_JPRB, 0.63493E+00_JPRB, 0.64122E+00_JPRB, 0.64993E+00_JPRB, 0.65531E+00_JPRB, 0.66033E+00_JPRB ,&
&  0.66446E+00_JPRB, 0.66665E+00_JPRB, 0.66938E+00_JPRB, 0.66968E+00_JPRB, 0.66541E+00_JPRB, 0.66445E+00_JPRB, 0.66436E+00_JPRB ,&
&  0.66752E+00_JPRB, 0.67585E+00_JPRB, 0.69435E+00_JPRB, 0.71715E+00_JPRB, 0.72673E+00_JPRB, 0.72015E+00_JPRB, 0.71025E+00_JPRB ,&
&  0.70900E+00_JPRB, 0.71636E+00_JPRB, 0.72486E+00_JPRB, 0.73013E+00_JPRB, 0.72993E+00_JPRB, 0.72106E+00_JPRB, 0.70903E+00_JPRB ,&
&  0.69572E+00_JPRB, 0.68217E+00_JPRB, 0.66978E+00_JPRB, 0.66148E+00_JPRB, 0.65459E+00_JPRB, 0.64916E+00_JPRB, 0.64567E+00_JPRB ,&
&  0.64422E+00_JPRB, 0.64425E+00_JPRB, 0.64798E+00_JPRB, 0.65565E+00_JPRB, 0.66127E+00_JPRB, 0.67126E+00_JPRB, 0.68287E+00_JPRB ,&
&  0.69442E+00_JPRB, 0.70509E+00_JPRB, 0.71764E+00_JPRB, 0.73536E+00_JPRB, 0.75563E+00_JPRB, 0.77081E+00_JPRB, 0.78268E+00_JPRB ,&
&  0.79020E+00_JPRB, 0.78902E+00_JPRB, 0.78044E+00_JPRB, 0.77042E+00_JPRB, 0.75938E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.69306E+00_JPRB, 0.68490E+00_JPRB, 0.67805E+00_JPRB, 0.67188E+00_JPRB, 0.66651E+00_JPRB, 0.66422E+00_JPRB, 0.66210E+00_JPRB ,&
&  0.66016E+00_JPRB, 0.65710E+00_JPRB, 0.65473E+00_JPRB, 0.65014E+00_JPRB, 0.64676E+00_JPRB, 0.64992E+00_JPRB, 0.65595E+00_JPRB ,&
&  0.65438E+00_JPRB, 0.64672E+00_JPRB, 0.64064E+00_JPRB, 0.64017E+00_JPRB, 0.64322E+00_JPRB, 0.64536E+00_JPRB, 0.64767E+00_JPRB ,&
&  0.64699E+00_JPRB, 0.64589E+00_JPRB, 0.64338E+00_JPRB, 0.62644E+00_JPRB, 0.61109E+00_JPRB, 0.60367E+00_JPRB, 0.60027E+00_JPRB ,&
&  0.60435E+00_JPRB, 0.62453E+00_JPRB, 0.60371E+00_JPRB, 0.61358E+00_JPRB, 0.60922E+00_JPRB, 0.60225E+00_JPRB, 0.59767E+00_JPRB ,&
&  0.59527E+00_JPRB, 0.59296E+00_JPRB, 0.59079E+00_JPRB, 0.59170E+00_JPRB, 0.59303E+00_JPRB, 0.59253E+00_JPRB, 0.59292E+00_JPRB ,&
&  0.59162E+00_JPRB, 0.59344E+00_JPRB, 0.59555E+00_JPRB, 0.60078E+00_JPRB, 0.60457E+00_JPRB, 0.60703E+00_JPRB, 0.61043E+00_JPRB ,&
&  0.61535E+00_JPRB, 0.61993E+00_JPRB, 0.62419E+00_JPRB, 0.62780E+00_JPRB, 0.62925E+00_JPRB, 0.62890E+00_JPRB, 0.62453E+00_JPRB ,&
&  0.62106E+00_JPRB, 0.62118E+00_JPRB, 0.62711E+00_JPRB, 0.64083E+00_JPRB, 0.65244E+00_JPRB, 0.65282E+00_JPRB, 0.64898E+00_JPRB ,&
&  0.64848E+00_JPRB, 0.65327E+00_JPRB, 0.65578E+00_JPRB, 0.65611E+00_JPRB, 0.65507E+00_JPRB, 0.65278E+00_JPRB, 0.64796E+00_JPRB ,&
&  0.64078E+00_JPRB, 0.63409E+00_JPRB, 0.62700E+00_JPRB, 0.61422E+00_JPRB, 0.60488E+00_JPRB, 0.60392E+00_JPRB, 0.60574E+00_JPRB ,&
&  0.60537E+00_JPRB, 0.60394E+00_JPRB, 0.60547E+00_JPRB, 0.61162E+00_JPRB, 0.61323E+00_JPRB, 0.62243E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.65399E+00_JPRB, 0.66719E+00_JPRB, 0.67756E+00_JPRB, 0.68819E+00_JPRB, 0.69864E+00_JPRB, 0.70658E+00_JPRB, 0.71308E+00_JPRB ,&
&  0.71569E+00_JPRB, 0.71329E+00_JPRB, 0.70717E+00_JPRB, 0.70364E+00_JPRB, 0.70066E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.66181E+00_JPRB, 0.66006E+00_JPRB, 0.65797E+00_JPRB, 0.65512E+00_JPRB, 0.65112E+00_JPRB, 0.64756E+00_JPRB, 0.64240E+00_JPRB ,&
&  0.63745E+00_JPRB, 0.63368E+00_JPRB, 0.63207E+00_JPRB, 0.63129E+00_JPRB, 0.63240E+00_JPRB, 0.63679E+00_JPRB, 0.64405E+00_JPRB ,&
&  0.64395E+00_JPRB, 0.63639E+00_JPRB, 0.63249E+00_JPRB, 0.63221E+00_JPRB, 0.63437E+00_JPRB, 0.63584E+00_JPRB, 0.63806E+00_JPRB ,&
&  0.63927E+00_JPRB, 0.63930E+00_JPRB, 0.63749E+00_JPRB, 0.62881E+00_JPRB, 0.61078E+00_JPRB, 0.60024E+00_JPRB, 0.59158E+00_JPRB ,&
&  0.58745E+00_JPRB, 0.58659E+00_JPRB, 0.58220E+00_JPRB, 0.58103E+00_JPRB, 0.57962E+00_JPRB, 0.57765E+00_JPRB, 0.57756E+00_JPRB ,&
&  0.57588E+00_JPRB, 0.57364E+00_JPRB, 0.57270E+00_JPRB, 0.57350E+00_JPRB, 0.57544E+00_JPRB, 0.57698E+00_JPRB, 0.57653E+00_JPRB ,&
&  0.57705E+00_JPRB, 0.58028E+00_JPRB, 0.58064E+00_JPRB, 0.58013E+00_JPRB, 0.57936E+00_JPRB, 0.57837E+00_JPRB, 0.57813E+00_JPRB ,&
&  0.58051E+00_JPRB, 0.58606E+00_JPRB, 0.59114E+00_JPRB, 0.59653E+00_JPRB, 0.59925E+00_JPRB, 0.60055E+00_JPRB, 0.60050E+00_JPRB ,&
&  0.59694E+00_JPRB, 0.59239E+00_JPRB, 0.59068E+00_JPRB, 0.59406E+00_JPRB, 0.60111E+00_JPRB, 0.60729E+00_JPRB, 0.60944E+00_JPRB ,&
&  0.61092E+00_JPRB, 0.61187E+00_JPRB, 0.61010E+00_JPRB, 0.60963E+00_JPRB, 0.60770E+00_JPRB, 0.60376E+00_JPRB, 0.60159E+00_JPRB ,&
&  0.59932E+00_JPRB, 0.59678E+00_JPRB, 0.59393E+00_JPRB, 0.58436E+00_JPRB, 0.57683E+00_JPRB, 0.57299E+00_JPRB, 0.57081E+00_JPRB ,&
&  0.57050E+00_JPRB, 0.57002E+00_JPRB, 0.57006E+00_JPRB, 0.57125E+00_JPRB, 0.57374E+00_JPRB, 0.58219E+00_JPRB, 0.59797E+00_JPRB ,&
&  0.62007E+00_JPRB, 0.64006E+00_JPRB, 0.65057E+00_JPRB, 0.65759E+00_JPRB, 0.66176E+00_JPRB, 0.66360E+00_JPRB, 0.66551E+00_JPRB ,&
&  0.66873E+00_JPRB, 0.66978E+00_JPRB, 0.66767E+00_JPRB, 0.66511E+00_JPRB, 0.66383E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.62993E+00_JPRB, 0.63005E+00_JPRB, 0.62830E+00_JPRB, 0.62646E+00_JPRB, 0.62446E+00_JPRB, 0.62268E+00_JPRB, 0.62041E+00_JPRB ,&
&  0.61552E+00_JPRB, 0.60913E+00_JPRB, 0.60544E+00_JPRB, 0.60417E+00_JPRB, 0.61035E+00_JPRB, 0.61904E+00_JPRB, 0.62665E+00_JPRB ,&
&  0.62525E+00_JPRB, 0.61514E+00_JPRB, 0.61180E+00_JPRB, 0.61352E+00_JPRB, 0.61594E+00_JPRB, 0.61853E+00_JPRB, 0.62198E+00_JPRB ,&
&  0.62585E+00_JPRB, 0.62967E+00_JPRB, 0.63126E+00_JPRB, 0.62771E+00_JPRB, 0.61987E+00_JPRB, 0.60517E+00_JPRB, 0.59296E+00_JPRB ,&
&  0.58516E+00_JPRB, 0.57938E+00_JPRB, 0.57469E+00_JPRB, 0.57330E+00_JPRB, 0.57082E+00_JPRB, 0.56705E+00_JPRB, 0.56203E+00_JPRB ,&
&  0.55805E+00_JPRB, 0.55658E+00_JPRB, 0.55974E+00_JPRB, 0.56231E+00_JPRB, 0.56354E+00_JPRB, 0.56289E+00_JPRB, 0.55921E+00_JPRB ,&
&  0.55664E+00_JPRB, 0.55512E+00_JPRB, 0.55370E+00_JPRB, 0.55404E+00_JPRB, 0.55043E+00_JPRB, 0.54750E+00_JPRB, 0.54889E+00_JPRB ,&
&  0.55318E+00_JPRB, 0.55737E+00_JPRB, 0.56157E+00_JPRB, 0.56539E+00_JPRB, 0.56760E+00_JPRB, 0.57040E+00_JPRB, 0.57180E+00_JPRB ,&
&  0.57033E+00_JPRB, 0.56586E+00_JPRB, 0.56170E+00_JPRB, 0.56167E+00_JPRB, 0.56361E+00_JPRB, 0.56599E+00_JPRB, 0.56914E+00_JPRB ,&
&  0.56933E+00_JPRB, 0.56397E+00_JPRB, 0.55881E+00_JPRB, 0.55439E+00_JPRB, 0.55046E+00_JPRB, 0.54770E+00_JPRB, 0.54660E+00_JPRB ,&
&  0.54659E+00_JPRB, 0.54341E+00_JPRB, 0.53722E+00_JPRB, 0.53227E+00_JPRB, 0.53225E+00_JPRB, 0.53277E+00_JPRB, 0.53002E+00_JPRB ,&
&  0.53163E+00_JPRB, 0.53513E+00_JPRB, 0.53438E+00_JPRB, 0.53249E+00_JPRB, 0.53447E+00_JPRB, 0.54057E+00_JPRB, 0.55349E+00_JPRB ,&
&  0.57064E+00_JPRB, 0.58992E+00_JPRB, 0.60747E+00_JPRB, 0.62192E+00_JPRB, 0.63009E+00_JPRB, 0.63181E+00_JPRB, 0.62989E+00_JPRB ,&
&  0.62937E+00_JPRB, 0.63052E+00_JPRB, 0.63000E+00_JPRB, 0.62867E+00_JPRB, 0.62899E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.59791E+00_JPRB, 0.59771E+00_JPRB, 0.59772E+00_JPRB, 0.59872E+00_JPRB, 0.59938E+00_JPRB, 0.59819E+00_JPRB, 0.59772E+00_JPRB ,&
&  0.59438E+00_JPRB, 0.58963E+00_JPRB, 0.58712E+00_JPRB, 0.58948E+00_JPRB, 0.59971E+00_JPRB, 0.61197E+00_JPRB, 0.61884E+00_JPRB ,&
&  0.61333E+00_JPRB, 0.59970E+00_JPRB, 0.59623E+00_JPRB, 0.59847E+00_JPRB, 0.60077E+00_JPRB, 0.60486E+00_JPRB, 0.61078E+00_JPRB ,&
&  0.61624E+00_JPRB, 0.61990E+00_JPRB, 0.62396E+00_JPRB, 0.62549E+00_JPRB, 0.62508E+00_JPRB, 0.61723E+00_JPRB, 0.60563E+00_JPRB ,&
&  0.59457E+00_JPRB, 0.58636E+00_JPRB, 0.58012E+00_JPRB, 0.57612E+00_JPRB, 0.57088E+00_JPRB, 0.56603E+00_JPRB, 0.55987E+00_JPRB ,&
&  0.55741E+00_JPRB, 0.55789E+00_JPRB, 0.55954E+00_JPRB, 0.55791E+00_JPRB, 0.55511E+00_JPRB, 0.54898E+00_JPRB, 0.54338E+00_JPRB ,&
&  0.54041E+00_JPRB, 0.54035E+00_JPRB, 0.54018E+00_JPRB, 0.53774E+00_JPRB, 0.53279E+00_JPRB, 0.52894E+00_JPRB, 0.52934E+00_JPRB ,&
&  0.53473E+00_JPRB, 0.53427E+00_JPRB, 0.53323E+00_JPRB, 0.53474E+00_JPRB, 0.53773E+00_JPRB, 0.54016E+00_JPRB, 0.53945E+00_JPRB ,&
&  0.53835E+00_JPRB, 0.53753E+00_JPRB, 0.53697E+00_JPRB, 0.53813E+00_JPRB, 0.53777E+00_JPRB, 0.53517E+00_JPRB, 0.52909E+00_JPRB ,&
&  0.52013E+00_JPRB, 0.50918E+00_JPRB, 0.50481E+00_JPRB, 0.50164E+00_JPRB, 0.49863E+00_JPRB, 0.49837E+00_JPRB, 0.50030E+00_JPRB ,&
&  0.49922E+00_JPRB, 0.49311E+00_JPRB, 0.49059E+00_JPRB, 0.49189E+00_JPRB, 0.49238E+00_JPRB, 0.49622E+00_JPRB, 0.50182E+00_JPRB ,&
&  0.51414E+00_JPRB, 0.52143E+00_JPRB, 0.52366E+00_JPRB, 0.52134E+00_JPRB, 0.52138E+00_JPRB, 0.52313E+00_JPRB, 0.52796E+00_JPRB ,&
&  0.53521E+00_JPRB, 0.54321E+00_JPRB, 0.55175E+00_JPRB, 0.56120E+00_JPRB, 0.57240E+00_JPRB, 0.58291E+00_JPRB, 0.58720E+00_JPRB ,&
&  0.58794E+00_JPRB, 0.58897E+00_JPRB, 0.59096E+00_JPRB, 0.59313E+00_JPRB, 0.59648E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.55939E+00_JPRB, 0.56176E+00_JPRB, 0.56424E+00_JPRB, 0.56606E+00_JPRB, 0.56903E+00_JPRB, 0.57427E+00_JPRB, 0.57639E+00_JPRB ,&
&  0.57422E+00_JPRB, 0.57172E+00_JPRB, 0.57163E+00_JPRB, 0.57568E+00_JPRB, 0.58358E+00_JPRB, 0.59145E+00_JPRB, 0.59934E+00_JPRB ,&
&  0.59424E+00_JPRB, 0.58026E+00_JPRB, 0.57529E+00_JPRB, 0.57745E+00_JPRB, 0.58252E+00_JPRB, 0.58805E+00_JPRB, 0.59466E+00_JPRB ,&
&  0.60088E+00_JPRB, 0.60479E+00_JPRB, 0.60897E+00_JPRB, 0.61287E+00_JPRB, 0.61530E+00_JPRB, 0.61305E+00_JPRB, 0.60832E+00_JPRB ,&
&  0.60262E+00_JPRB, 0.59623E+00_JPRB, 0.58989E+00_JPRB, 0.57442E+00_JPRB, 0.57200E+00_JPRB, 0.56330E+00_JPRB, 0.55636E+00_JPRB ,&
&  0.55187E+00_JPRB, 0.54804E+00_JPRB, 0.54545E+00_JPRB, 0.54195E+00_JPRB, 0.53670E+00_JPRB, 0.52794E+00_JPRB, 0.52435E+00_JPRB ,&
&  0.52185E+00_JPRB, 0.52142E+00_JPRB, 0.51515E+00_JPRB, 0.51123E+00_JPRB, 0.50685E+00_JPRB, 0.50320E+00_JPRB, 0.50621E+00_JPRB ,&
&  0.51057E+00_JPRB, 0.50732E+00_JPRB, 0.50290E+00_JPRB, 0.50282E+00_JPRB, 0.50749E+00_JPRB, 0.50926E+00_JPRB, 0.51004E+00_JPRB ,&
&  0.51139E+00_JPRB, 0.51110E+00_JPRB, 0.50538E+00_JPRB, 0.49946E+00_JPRB, 0.49097E+00_JPRB, 0.47898E+00_JPRB, 0.47493E+00_JPRB ,&
&  0.47143E+00_JPRB, 0.46612E+00_JPRB, 0.46389E+00_JPRB, 0.46318E+00_JPRB, 0.46308E+00_JPRB, 0.46454E+00_JPRB, 0.46533E+00_JPRB ,&
&  0.46614E+00_JPRB, 0.46281E+00_JPRB, 0.45801E+00_JPRB, 0.45435E+00_JPRB, 0.44926E+00_JPRB, 0.45365E+00_JPRB, 0.46562E+00_JPRB ,&
&  0.47966E+00_JPRB, 0.48853E+00_JPRB, 0.49425E+00_JPRB, 0.49773E+00_JPRB, 0.50078E+00_JPRB, 0.50192E+00_JPRB, 0.50418E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.50854E+00_JPRB, 0.50756E+00_JPRB, 0.50934E+00_JPRB, 0.51417E+00_JPRB, 0.52074E+00_JPRB, 0.52822E+00_JPRB ,&
&  0.53531E+00_JPRB, 0.54123E+00_JPRB, 0.54612E+00_JPRB, 0.55204E+00_JPRB, 0.55596E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.52716E+00_JPRB, 0.53187E+00_JPRB, 0.53656E+00_JPRB, 0.54162E+00_JPRB, 0.54863E+00_JPRB, 0.55260E+00_JPRB, 0.55408E+00_JPRB ,&
&  0.55541E+00_JPRB, 0.55566E+00_JPRB, 0.55677E+00_JPRB, 0.56101E+00_JPRB, 0.56787E+00_JPRB, 0.57337E+00_JPRB, 0.57992E+00_JPRB ,&
&  0.57989E+00_JPRB, 0.56996E+00_JPRB, 0.56150E+00_JPRB, 0.56129E+00_JPRB, 0.56718E+00_JPRB, 0.57323E+00_JPRB, 0.58054E+00_JPRB ,&
&  0.58763E+00_JPRB, 0.59252E+00_JPRB, 0.59510E+00_JPRB, 0.59634E+00_JPRB, 0.59797E+00_JPRB, 0.59704E+00_JPRB, 0.59365E+00_JPRB ,&
&  0.58872E+00_JPRB, 0.57811E+00_JPRB, 0.55844E+00_JPRB, 0.55613E+00_JPRB, 0.54739E+00_JPRB, 0.53997E+00_JPRB, 0.53491E+00_JPRB ,&
&  0.53094E+00_JPRB, 0.52733E+00_JPRB, 0.52294E+00_JPRB, 0.51854E+00_JPRB, 0.51332E+00_JPRB, 0.50468E+00_JPRB, 0.49955E+00_JPRB ,&
&  0.49471E+00_JPRB, 0.49088E+00_JPRB, 0.48679E+00_JPRB, 0.48565E+00_JPRB, 0.48467E+00_JPRB, 0.48621E+00_JPRB, 0.48657E+00_JPRB ,&
&  0.48239E+00_JPRB, 0.47769E+00_JPRB, 0.47422E+00_JPRB, 0.47571E+00_JPRB, 0.47831E+00_JPRB, 0.48008E+00_JPRB, 0.48067E+00_JPRB ,&
&  0.47735E+00_JPRB, 0.47133E+00_JPRB, 0.46496E+00_JPRB, 0.45803E+00_JPRB, 0.44933E+00_JPRB, 0.42368E+00_JPRB, 0.41326E+00_JPRB ,&
&  0.41194E+00_JPRB, 0.42196E+00_JPRB, 0.43280E+00_JPRB, 0.43296E+00_JPRB, 0.43771E+00_JPRB, 0.43870E+00_JPRB, 0.43613E+00_JPRB ,&
&  0.42541E+00_JPRB, 0.41250E+00_JPRB, 0.40540E+00_JPRB, 0.40266E+00_JPRB, 0.40080E+00_JPRB, 0.40434E+00_JPRB, 0.41234E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.43668E+00_JPRB, 0.44564E+00_JPRB, 0.44957E+00_JPRB, 0.44793E+00_JPRB, 0.44484E+00_JPRB, 0.44077E+00_JPRB ,&
&  0.43725E+00_JPRB, 0.44486E+00_JPRB, 0.46127E+00_JPRB, 0.48070E+00_JPRB, 0.48350E+00_JPRB, 0.48991E+00_JPRB, 0.49440E+00_JPRB ,&
&  0.50061E+00_JPRB, 0.50841E+00_JPRB, 0.51470E+00_JPRB, 0.52077E+00_JPRB, 0.52428E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.50170E+00_JPRB, 0.50704E+00_JPRB, 0.51422E+00_JPRB, 0.52220E+00_JPRB, 0.52809E+00_JPRB, 0.53130E+00_JPRB, 0.53475E+00_JPRB ,&
&  0.53872E+00_JPRB, 0.54048E+00_JPRB, 0.54153E+00_JPRB, 0.54415E+00_JPRB, 0.54986E+00_JPRB, 0.55356E+00_JPRB, 0.55924E+00_JPRB ,&
&  0.56177E+00_JPRB, 0.55924E+00_JPRB, 0.55191E+00_JPRB, 0.55007E+00_JPRB, 0.55411E+00_JPRB, 0.55923E+00_JPRB, 0.56437E+00_JPRB ,&
&  0.56941E+00_JPRB, 0.57393E+00_JPRB, 0.57586E+00_JPRB, 0.57595E+00_JPRB, 0.57564E+00_JPRB, 0.57505E+00_JPRB, 0.57205E+00_JPRB ,&
&  0.56455E+00_JPRB, 0.52271E+00_JPRB, 0.49663E+00_JPRB, 0.50410E+00_JPRB, 0.51445E+00_JPRB, 0.50675E+00_JPRB, 0.50517E+00_JPRB ,&
&  0.50370E+00_JPRB, 0.49947E+00_JPRB, 0.49330E+00_JPRB, 0.48790E+00_JPRB, 0.48158E+00_JPRB, 0.47322E+00_JPRB, 0.46676E+00_JPRB ,&
&  0.46454E+00_JPRB, 0.47027E+00_JPRB, 0.47129E+00_JPRB, 0.47153E+00_JPRB, 0.46784E+00_JPRB, 0.46767E+00_JPRB, 0.46447E+00_JPRB ,&
&  0.45697E+00_JPRB, 0.44869E+00_JPRB, 0.44521E+00_JPRB, 0.44419E+00_JPRB, 0.44539E+00_JPRB, 0.44823E+00_JPRB, 0.44664E+00_JPRB ,&
&  0.43420E+00_JPRB, 0.42222E+00_JPRB, 0.40288E+00_JPRB, 0.38406E+00_JPRB, 0.37072E+00_JPRB, 0.36493E+00_JPRB, 0.36081E+00_JPRB ,&
&  0.36077E+00_JPRB, 0.36874E+00_JPRB, 0.38949E+00_JPRB, 0.41340E+00_JPRB, 0.42240E+00_JPRB, 0.40340E+00_JPRB, 0.38916E+00_JPRB ,&
&  0.38096E+00_JPRB, 0.37634E+00_JPRB, 0.37222E+00_JPRB, 0.37136E+00_JPRB, 0.37262E+00_JPRB, 0.37576E+00_JPRB, 0.37994E+00_JPRB ,&
&  0.38583E+00_JPRB, 0.39115E+00_JPRB, 0.39398E+00_JPRB, 0.39530E+00_JPRB, 0.39618E+00_JPRB, 0.39721E+00_JPRB, 0.39671E+00_JPRB ,&
&  0.39715E+00_JPRB, 0.40212E+00_JPRB, 0.41074E+00_JPRB, 0.42092E+00_JPRB, 0.42973E+00_JPRB, 0.44005E+00_JPRB, 0.45436E+00_JPRB ,&
&  0.46949E+00_JPRB, 0.47966E+00_JPRB, 0.48373E+00_JPRB, 0.49088E+00_JPRB, 0.49672E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.48853E+00_JPRB, 0.49653E+00_JPRB, 0.50127E+00_JPRB, 0.51093E+00_JPRB, 0.51427E+00_JPRB, 0.51818E+00_JPRB, 0.52189E+00_JPRB ,&
&  0.52936E+00_JPRB, 0.53333E+00_JPRB, 0.53144E+00_JPRB, 0.53121E+00_JPRB, 0.53037E+00_JPRB, 0.52837E+00_JPRB, 0.52987E+00_JPRB ,&
&  0.53056E+00_JPRB, 0.53313E+00_JPRB, 0.53042E+00_JPRB, 0.52783E+00_JPRB, 0.53001E+00_JPRB, 0.53519E+00_JPRB, 0.53856E+00_JPRB ,&
&  0.53267E+00_JPRB, 0.53024E+00_JPRB, 0.53836E+00_JPRB, 0.53952E+00_JPRB, 0.53982E+00_JPRB, 0.54113E+00_JPRB, 0.54033E+00_JPRB ,&
&  0.52511E+00_JPRB, 0.49749E+00_JPRB, 0.46995E+00_JPRB, 0.47481E+00_JPRB, 0.48986E+00_JPRB, 0.48067E+00_JPRB, 0.47862E+00_JPRB ,&
&  0.47578E+00_JPRB, 0.47084E+00_JPRB, 0.46635E+00_JPRB, 0.46366E+00_JPRB, 0.46086E+00_JPRB, 0.45533E+00_JPRB, 0.45389E+00_JPRB ,&
&  0.45205E+00_JPRB, 0.46264E+00_JPRB, 0.45505E+00_JPRB, 0.42869E+00_JPRB, 0.41116E+00_JPRB, 0.40122E+00_JPRB, 0.38808E+00_JPRB ,&
&  0.37231E+00_JPRB, 0.36441E+00_JPRB, 0.36388E+00_JPRB, 0.36094E+00_JPRB, 0.36155E+00_JPRB, 0.36276E+00_JPRB, 0.36385E+00_JPRB ,&
&  0.36405E+00_JPRB, 0.36318E+00_JPRB, 0.35794E+00_JPRB, 0.35127E+00_JPRB, 0.34771E+00_JPRB, 0.34790E+00_JPRB, 0.35163E+00_JPRB ,&
&  0.35921E+00_JPRB, 0.36638E+00_JPRB, 0.38276E+00_JPRB, 0.39075E+00_JPRB, 0.38027E+00_JPRB, 0.37694E+00_JPRB, 0.37322E+00_JPRB ,&
&  0.36056E+00_JPRB, 0.35611E+00_JPRB, 0.35243E+00_JPRB, 0.35050E+00_JPRB, 0.35170E+00_JPRB, 0.35591E+00_JPRB, 0.35751E+00_JPRB ,&
&  0.36050E+00_JPRB, 0.36417E+00_JPRB, 0.36573E+00_JPRB, 0.36686E+00_JPRB, 0.36786E+00_JPRB, 0.37042E+00_JPRB, 0.37304E+00_JPRB ,&
&  0.37496E+00_JPRB, 0.37857E+00_JPRB, 0.38284E+00_JPRB, 0.38835E+00_JPRB, 0.39319E+00_JPRB, 0.39587E+00_JPRB, 0.40529E+00_JPRB ,&
&  0.41554E+00_JPRB, 0.43611E+00_JPRB, 0.46871E+00_JPRB, 0.49167E+00_JPRB, 0.48260E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.48832E+00_JPRB, 0.49244E+00_JPRB, 0.49233E+00_JPRB, 0.49560E+00_JPRB, 0.50170E+00_JPRB, 0.50855E+00_JPRB, 0.51462E+00_JPRB ,&
&  0.52041E+00_JPRB, 0.52007E+00_JPRB, 0.51511E+00_JPRB, 0.50651E+00_JPRB, 0.49951E+00_JPRB, 0.48280E+00_JPRB, 0.47467E+00_JPRB ,&
&  0.46774E+00_JPRB, 0.46831E+00_JPRB, 0.46861E+00_JPRB, 0.47055E+00_JPRB, 0.47133E+00_JPRB, 0.47652E+00_JPRB, 0.47763E+00_JPRB ,&
&  0.47422E+00_JPRB, 0.46837E+00_JPRB, 0.46227E+00_JPRB, 0.46091E+00_JPRB, 0.46356E+00_JPRB, 0.46514E+00_JPRB, 0.46144E+00_JPRB ,&
&  0.45762E+00_JPRB, 0.45757E+00_JPRB, 0.45797E+00_JPRB, 0.45939E+00_JPRB, 0.46863E+00_JPRB, 0.46124E+00_JPRB, 0.45631E+00_JPRB ,&
&  0.44927E+00_JPRB, 0.44347E+00_JPRB, 0.43980E+00_JPRB, 0.44037E+00_JPRB, 0.44462E+00_JPRB, 0.44189E+00_JPRB, 0.44510E+00_JPRB ,&
&  0.43533E+00_JPRB, 0.42713E+00_JPRB, 0.41119E+00_JPRB, 0.39793E+00_JPRB, 0.38214E+00_JPRB, 0.37139E+00_JPRB, 0.36178E+00_JPRB ,&
&  0.35370E+00_JPRB, 0.34651E+00_JPRB, 0.33993E+00_JPRB, 0.33807E+00_JPRB, 0.33919E+00_JPRB, 0.33880E+00_JPRB, 0.33766E+00_JPRB ,&
&  0.33786E+00_JPRB, 0.33870E+00_JPRB, 0.33935E+00_JPRB, 0.34050E+00_JPRB, 0.34262E+00_JPRB, 0.34592E+00_JPRB, 0.35053E+00_JPRB ,&
&  0.35755E+00_JPRB, 0.36289E+00_JPRB, 0.36566E+00_JPRB, 0.36767E+00_JPRB, 0.36442E+00_JPRB, 0.35831E+00_JPRB, 0.35268E+00_JPRB ,&
&  0.34637E+00_JPRB, 0.34352E+00_JPRB, 0.34070E+00_JPRB, 0.33937E+00_JPRB, 0.33898E+00_JPRB, 0.33991E+00_JPRB, 0.34211E+00_JPRB ,&
&  0.34432E+00_JPRB, 0.34741E+00_JPRB, 0.34942E+00_JPRB, 0.35178E+00_JPRB, 0.35453E+00_JPRB, 0.35687E+00_JPRB, 0.35924E+00_JPRB ,&
&  0.36195E+00_JPRB, 0.36409E+00_JPRB, 0.36788E+00_JPRB, 0.37262E+00_JPRB, 0.38195E+00_JPRB, 0.39191E+00_JPRB, 0.40932E+00_JPRB ,&
&  0.42862E+00_JPRB, 0.45858E+00_JPRB, 0.46471E+00_JPRB, 0.47787E+00_JPRB, 0.48753E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.48219E+00_JPRB, 0.48480E+00_JPRB, 0.48828E+00_JPRB, 0.49418E+00_JPRB, 0.50177E+00_JPRB, 0.50913E+00_JPRB, 0.51119E+00_JPRB ,&
&  0.50090E+00_JPRB, 0.48810E+00_JPRB, 0.46930E+00_JPRB, 0.45491E+00_JPRB, 0.44032E+00_JPRB, 0.42970E+00_JPRB, 0.42139E+00_JPRB ,&
&  0.41535E+00_JPRB, 0.41530E+00_JPRB, 0.41515E+00_JPRB, 0.41632E+00_JPRB, 0.41456E+00_JPRB, 0.41519E+00_JPRB, 0.41486E+00_JPRB ,&
&  0.41292E+00_JPRB, 0.41262E+00_JPRB, 0.41253E+00_JPRB, 0.41580E+00_JPRB, 0.42271E+00_JPRB, 0.42883E+00_JPRB, 0.43483E+00_JPRB ,&
&  0.44204E+00_JPRB, 0.44317E+00_JPRB, 0.44698E+00_JPRB, 0.44497E+00_JPRB, 0.44151E+00_JPRB, 0.43712E+00_JPRB, 0.43082E+00_JPRB ,&
&  0.42729E+00_JPRB, 0.42390E+00_JPRB, 0.42340E+00_JPRB, 0.42374E+00_JPRB, 0.42370E+00_JPRB, 0.42898E+00_JPRB, 0.42475E+00_JPRB ,&
&  0.41805E+00_JPRB, 0.40674E+00_JPRB, 0.39223E+00_JPRB, 0.38107E+00_JPRB, 0.36953E+00_JPRB, 0.36129E+00_JPRB, 0.35396E+00_JPRB ,&
&  0.34729E+00_JPRB, 0.34289E+00_JPRB, 0.33818E+00_JPRB, 0.33465E+00_JPRB, 0.33348E+00_JPRB, 0.33094E+00_JPRB, 0.32941E+00_JPRB ,&
&  0.32871E+00_JPRB, 0.32910E+00_JPRB, 0.33139E+00_JPRB, 0.33337E+00_JPRB, 0.33616E+00_JPRB, 0.33914E+00_JPRB, 0.34154E+00_JPRB ,&
&  0.34486E+00_JPRB, 0.34759E+00_JPRB, 0.34796E+00_JPRB, 0.34741E+00_JPRB, 0.34497E+00_JPRB, 0.34160E+00_JPRB, 0.33889E+00_JPRB ,&
&  0.33584E+00_JPRB, 0.33376E+00_JPRB, 0.33314E+00_JPRB, 0.33203E+00_JPRB, 0.33262E+00_JPRB, 0.33399E+00_JPRB, 0.33485E+00_JPRB ,&
&  0.33693E+00_JPRB, 0.33862E+00_JPRB, 0.34073E+00_JPRB, 0.34460E+00_JPRB, 0.34794E+00_JPRB, 0.35088E+00_JPRB, 0.35372E+00_JPRB ,&
&  0.35345E+00_JPRB, 0.35535E+00_JPRB, 0.35818E+00_JPRB, 0.36383E+00_JPRB, 0.38096E+00_JPRB, 0.39134E+00_JPRB, 0.41539E+00_JPRB ,&
&  0.43261E+00_JPRB, 0.46376E+00_JPRB, 0.45951E+00_JPRB, 0.46942E+00_JPRB, 0.47789E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.47590E+00_JPRB, 0.48326E+00_JPRB, 0.48976E+00_JPRB, 0.49527E+00_JPRB, 0.49872E+00_JPRB, 0.50220E+00_JPRB, 0.49890E+00_JPRB ,&
&  0.49285E+00_JPRB, 0.47904E+00_JPRB, 0.46803E+00_JPRB, 0.45632E+00_JPRB, 0.44681E+00_JPRB, 0.43736E+00_JPRB, 0.42871E+00_JPRB ,&
&  0.42009E+00_JPRB, 0.41277E+00_JPRB, 0.40641E+00_JPRB, 0.40069E+00_JPRB, 0.39302E+00_JPRB, 0.38617E+00_JPRB, 0.38065E+00_JPRB ,&
&  0.37608E+00_JPRB, 0.37600E+00_JPRB, 0.37875E+00_JPRB, 0.38355E+00_JPRB, 0.39265E+00_JPRB, 0.40108E+00_JPRB, 0.41040E+00_JPRB ,&
&  0.42432E+00_JPRB, 0.42553E+00_JPRB, 0.42427E+00_JPRB, 0.42373E+00_JPRB, 0.41866E+00_JPRB, 0.41212E+00_JPRB, 0.40342E+00_JPRB ,&
&  0.40717E+00_JPRB, 0.40518E+00_JPRB, 0.41431E+00_JPRB, 0.41408E+00_JPRB, 0.41272E+00_JPRB, 0.41491E+00_JPRB, 0.40803E+00_JPRB ,&
&  0.40427E+00_JPRB, 0.39430E+00_JPRB, 0.38650E+00_JPRB, 0.37725E+00_JPRB, 0.36714E+00_JPRB, 0.36127E+00_JPRB, 0.35342E+00_JPRB ,&
&  0.34789E+00_JPRB, 0.34408E+00_JPRB, 0.33891E+00_JPRB, 0.33590E+00_JPRB, 0.33309E+00_JPRB, 0.33006E+00_JPRB, 0.32845E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32680E+00_JPRB, 0.32710E+00_JPRB, 0.32739E+00_JPRB, 0.32806E+00_JPRB, 0.32919E+00_JPRB, 0.32999E+00_JPRB ,&
&  0.33092E+00_JPRB, 0.33135E+00_JPRB, 0.33100E+00_JPRB, 0.33067E+00_JPRB, 0.32928E+00_JPRB, 0.32844E+00_JPRB, 0.32719E+00_JPRB ,&
&  0.32635E+00_JPRB, 0.32670E+00_JPRB, 0.32695E+00_JPRB, 0.32772E+00_JPRB, 0.32843E+00_JPRB, 0.33023E+00_JPRB, 0.33187E+00_JPRB ,&
&  0.33364E+00_JPRB, 0.33604E+00_JPRB, 0.33788E+00_JPRB, 0.34039E+00_JPRB, 0.34329E+00_JPRB, 0.34497E+00_JPRB, 0.34835E+00_JPRB ,&
&  0.35153E+00_JPRB, 0.35567E+00_JPRB, 0.36336E+00_JPRB, 0.37152E+00_JPRB, 0.38251E+00_JPRB, 0.39447E+00_JPRB, 0.40896E+00_JPRB ,&
&  0.42713E+00_JPRB, 0.45072E+00_JPRB, 0.45093E+00_JPRB, 0.46268E+00_JPRB, 0.46947E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.46097E+00_JPRB, 0.46712E+00_JPRB, 0.47225E+00_JPRB, 0.47687E+00_JPRB, 0.48122E+00_JPRB, 0.48284E+00_JPRB, 0.48195E+00_JPRB ,&
&  0.47243E+00_JPRB, 0.46577E+00_JPRB, 0.45730E+00_JPRB, 0.44928E+00_JPRB, 0.44184E+00_JPRB, 0.43170E+00_JPRB, 0.42388E+00_JPRB ,&
&  0.41584E+00_JPRB, 0.40772E+00_JPRB, 0.40033E+00_JPRB, 0.39150E+00_JPRB, 0.38239E+00_JPRB, 0.37359E+00_JPRB, 0.36862E+00_JPRB ,&
&  0.36586E+00_JPRB, 0.36600E+00_JPRB, 0.36969E+00_JPRB, 0.37379E+00_JPRB, 0.37900E+00_JPRB, 0.38561E+00_JPRB, 0.39280E+00_JPRB ,&
&  0.39722E+00_JPRB, 0.39947E+00_JPRB, 0.39862E+00_JPRB, 0.39624E+00_JPRB, 0.39370E+00_JPRB, 0.39194E+00_JPRB, 0.38914E+00_JPRB ,&
&  0.38550E+00_JPRB, 0.38377E+00_JPRB, 0.38451E+00_JPRB, 0.38757E+00_JPRB, 0.38885E+00_JPRB, 0.38793E+00_JPRB, 0.38398E+00_JPRB ,&
&  0.37902E+00_JPRB, 0.37383E+00_JPRB, 0.36815E+00_JPRB, 0.36274E+00_JPRB, 0.35831E+00_JPRB, 0.35347E+00_JPRB, 0.34810E+00_JPRB ,&
&  0.34481E+00_JPRB, 0.34142E+00_JPRB, 0.33766E+00_JPRB, 0.33422E+00_JPRB, 0.33131E+00_JPRB, 0.32909E+00_JPRB, 0.32719E+00_JPRB ,&
&  0.32586E+00_JPRB, 0.32486E+00_JPRB, 0.32412E+00_JPRB, 0.32358E+00_JPRB, 0.32306E+00_JPRB, 0.32209E+00_JPRB, 0.32114E+00_JPRB ,&
&  0.32020E+00_JPRB, 0.31991E+00_JPRB, 0.31930E+00_JPRB, 0.31935E+00_JPRB, 0.31968E+00_JPRB, 0.31984E+00_JPRB, 0.32042E+00_JPRB ,&
&  0.32170E+00_JPRB, 0.32305E+00_JPRB, 0.32352E+00_JPRB, 0.32475E+00_JPRB, 0.32593E+00_JPRB, 0.32753E+00_JPRB, 0.32975E+00_JPRB ,&
&  0.33102E+00_JPRB, 0.33242E+00_JPRB, 0.33409E+00_JPRB, 0.33588E+00_JPRB, 0.33918E+00_JPRB, 0.34275E+00_JPRB, 0.34672E+00_JPRB ,&
&  0.35184E+00_JPRB, 0.35667E+00_JPRB, 0.36228E+00_JPRB, 0.36899E+00_JPRB, 0.38182E+00_JPRB, 0.38877E+00_JPRB, 0.39694E+00_JPRB ,&
&  0.40420E+00_JPRB, 0.40448E+00_JPRB, 0.44178E+00_JPRB, 0.44519E+00_JPRB, 0.45172E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.38395E+00_JPRB, 0.39094E+00_JPRB, 0.39488E+00_JPRB, 0.39856E+00_JPRB, 0.39055E+00_JPRB, 0.43085E+00_JPRB, 0.43000E+00_JPRB ,&
&  0.42973E+00_JPRB, 0.41781E+00_JPRB, 0.41728E+00_JPRB, 0.41324E+00_JPRB, 0.40793E+00_JPRB, 0.40177E+00_JPRB, 0.39641E+00_JPRB ,&
&  0.39138E+00_JPRB, 0.38559E+00_JPRB, 0.38103E+00_JPRB, 0.37679E+00_JPRB, 0.37333E+00_JPRB, 0.37030E+00_JPRB, 0.36818E+00_JPRB ,&
&  0.36777E+00_JPRB, 0.36963E+00_JPRB, 0.37341E+00_JPRB, 0.37492E+00_JPRB, 0.37513E+00_JPRB, 0.37529E+00_JPRB, 0.37554E+00_JPRB ,&
&  0.37516E+00_JPRB, 0.37456E+00_JPRB, 0.37271E+00_JPRB, 0.36950E+00_JPRB, 0.36740E+00_JPRB, 0.36624E+00_JPRB, 0.36548E+00_JPRB ,&
&  0.36484E+00_JPRB, 0.36444E+00_JPRB, 0.36399E+00_JPRB, 0.36395E+00_JPRB, 0.36337E+00_JPRB, 0.36217E+00_JPRB, 0.35995E+00_JPRB ,&
&  0.35757E+00_JPRB, 0.35497E+00_JPRB, 0.35164E+00_JPRB, 0.34897E+00_JPRB, 0.34635E+00_JPRB, 0.34370E+00_JPRB, 0.34101E+00_JPRB ,&
&  0.33950E+00_JPRB, 0.33803E+00_JPRB, 0.33615E+00_JPRB, 0.33378E+00_JPRB, 0.33204E+00_JPRB, 0.33022E+00_JPRB, 0.32865E+00_JPRB ,&
&  0.32768E+00_JPRB, 0.32660E+00_JPRB, 0.32551E+00_JPRB, 0.32439E+00_JPRB, 0.32362E+00_JPRB, 0.32266E+00_JPRB, 0.32174E+00_JPRB ,&
&  0.32089E+00_JPRB, 0.31989E+00_JPRB, 0.31964E+00_JPRB, 0.31950E+00_JPRB, 0.31957E+00_JPRB, 0.31957E+00_JPRB, 0.31991E+00_JPRB ,&
&  0.32045E+00_JPRB, 0.32129E+00_JPRB, 0.32235E+00_JPRB, 0.32331E+00_JPRB, 0.32422E+00_JPRB, 0.32516E+00_JPRB, 0.32661E+00_JPRB ,&
&  0.32812E+00_JPRB, 0.32968E+00_JPRB, 0.33128E+00_JPRB, 0.33247E+00_JPRB, 0.33549E+00_JPRB, 0.33829E+00_JPRB, 0.34125E+00_JPRB ,&
&  0.34466E+00_JPRB, 0.34877E+00_JPRB, 0.35329E+00_JPRB, 0.35822E+00_JPRB, 0.36494E+00_JPRB, 0.36856E+00_JPRB, 0.37103E+00_JPRB ,&
&  0.37377E+00_JPRB, 0.37876E+00_JPRB, 0.37957E+00_JPRB, 0.38044E+00_JPRB, 0.38195E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.35648E+00_JPRB, 0.35672E+00_JPRB, 0.35661E+00_JPRB, 0.35618E+00_JPRB, 0.35250E+00_JPRB, 0.35203E+00_JPRB, 0.35152E+00_JPRB ,&
&  0.35135E+00_JPRB, 0.35142E+00_JPRB, 0.35223E+00_JPRB, 0.35322E+00_JPRB, 0.35424E+00_JPRB, 0.35525E+00_JPRB, 0.35620E+00_JPRB ,&
&  0.35683E+00_JPRB, 0.35760E+00_JPRB, 0.35590E+00_JPRB, 0.35476E+00_JPRB, 0.35454E+00_JPRB, 0.35454E+00_JPRB, 0.35443E+00_JPRB ,&
&  0.35422E+00_JPRB, 0.35398E+00_JPRB, 0.35327E+00_JPRB, 0.35282E+00_JPRB, 0.35262E+00_JPRB, 0.35235E+00_JPRB, 0.35193E+00_JPRB ,&
&  0.35149E+00_JPRB, 0.35113E+00_JPRB, 0.35073E+00_JPRB, 0.35021E+00_JPRB, 0.35019E+00_JPRB, 0.35023E+00_JPRB, 0.34973E+00_JPRB ,&
&  0.34912E+00_JPRB, 0.34810E+00_JPRB, 0.34717E+00_JPRB, 0.34630E+00_JPRB, 0.34561E+00_JPRB, 0.34534E+00_JPRB, 0.34501E+00_JPRB ,&
&  0.34451E+00_JPRB, 0.34381E+00_JPRB, 0.34315E+00_JPRB, 0.34225E+00_JPRB, 0.34143E+00_JPRB, 0.34067E+00_JPRB, 0.33977E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.33777E+00_JPRB, 0.33671E+00_JPRB, 0.33550E+00_JPRB, 0.33428E+00_JPRB, 0.33304E+00_JPRB, 0.33167E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.32979E+00_JPRB, 0.32916E+00_JPRB, 0.32864E+00_JPRB, 0.32823E+00_JPRB, 0.32793E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.32738E+00_JPRB, 0.32712E+00_JPRB, 0.32701E+00_JPRB, 0.32689E+00_JPRB, 0.32690E+00_JPRB, 0.32698E+00_JPRB, 0.32720E+00_JPRB ,&
&  0.32745E+00_JPRB, 0.32777E+00_JPRB, 0.32820E+00_JPRB, 0.32896E+00_JPRB, 0.32970E+00_JPRB, 0.33058E+00_JPRB, 0.33143E+00_JPRB ,&
&  0.33238E+00_JPRB, 0.33331E+00_JPRB, 0.33427E+00_JPRB, 0.33519E+00_JPRB, 0.33646E+00_JPRB, 0.33767E+00_JPRB, 0.33902E+00_JPRB ,&
&  0.34035E+00_JPRB, 0.34180E+00_JPRB, 0.34323E+00_JPRB, 0.34476E+00_JPRB, 0.34653E+00_JPRB, 0.34700E+00_JPRB, 0.34750E+00_JPRB ,&
&  0.34812E+00_JPRB, 0.34903E+00_JPRB, 0.34983E+00_JPRB, 0.35097E+00_JPRB, 0.35291E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB, 0.33901E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB, 0.71339E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.71279E+00_JPRB, 0.71359E+00_JPRB, 0.71431E+00_JPRB, 0.71472E+00_JPRB, 0.71493E+00_JPRB, 0.71505E+00_JPRB, 0.71508E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71512E+00_JPRB, 0.71550E+00_JPRB, 0.71590E+00_JPRB, 0.71624E+00_JPRB, 0.71647E+00_JPRB, 0.71643E+00_JPRB ,&
&  0.71686E+00_JPRB, 0.71904E+00_JPRB, 0.72036E+00_JPRB, 0.72094E+00_JPRB, 0.72143E+00_JPRB, 0.72182E+00_JPRB, 0.72204E+00_JPRB ,&
&  0.72232E+00_JPRB, 0.72262E+00_JPRB, 0.72292E+00_JPRB, 0.72320E+00_JPRB, 0.72351E+00_JPRB, 0.72383E+00_JPRB, 0.72420E+00_JPRB ,&
&  0.72480E+00_JPRB, 0.72603E+00_JPRB, 0.72768E+00_JPRB, 0.72955E+00_JPRB, 0.73185E+00_JPRB, 0.73481E+00_JPRB, 0.73724E+00_JPRB ,&
&  0.73899E+00_JPRB, 0.73985E+00_JPRB, 0.73881E+00_JPRB, 0.73635E+00_JPRB, 0.73301E+00_JPRB, 0.72942E+00_JPRB, 0.72589E+00_JPRB ,&
&  0.72189E+00_JPRB, 0.71825E+00_JPRB, 0.71501E+00_JPRB, 0.71217E+00_JPRB, 0.70953E+00_JPRB, 0.70722E+00_JPRB, 0.70499E+00_JPRB ,&
&  0.70325E+00_JPRB, 0.70153E+00_JPRB, 0.70005E+00_JPRB, 0.69861E+00_JPRB, 0.69710E+00_JPRB, 0.69524E+00_JPRB, 0.69329E+00_JPRB ,&
&  0.69184E+00_JPRB, 0.69094E+00_JPRB, 0.69034E+00_JPRB, 0.68974E+00_JPRB, 0.68948E+00_JPRB, 0.69003E+00_JPRB, 0.69089E+00_JPRB ,&
&  0.69165E+00_JPRB, 0.69269E+00_JPRB, 0.69394E+00_JPRB, 0.69480E+00_JPRB, 0.69540E+00_JPRB, 0.69564E+00_JPRB, 0.69566E+00_JPRB ,&
&  0.69556E+00_JPRB, 0.69592E+00_JPRB, 0.69642E+00_JPRB, 0.69735E+00_JPRB, 0.69828E+00_JPRB, 0.69939E+00_JPRB, 0.70065E+00_JPRB ,&
&  0.70161E+00_JPRB, 0.70270E+00_JPRB, 0.70365E+00_JPRB, 0.70487E+00_JPRB, 0.70594E+00_JPRB, 0.70675E+00_JPRB, 0.70728E+00_JPRB ,&
&  0.70772E+00_JPRB, 0.70798E+00_JPRB, 0.70827E+00_JPRB, 0.70841E+00_JPRB, 0.70870E+00_JPRB, 0.70904E+00_JPRB, 0.70938E+00_JPRB ,&
&  0.70970E+00_JPRB, 0.71021E+00_JPRB, 0.71061E+00_JPRB, 0.71140E+00_JPRB, 0.71210E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.71860E+00_JPRB, 0.71986E+00_JPRB, 0.72083E+00_JPRB, 0.72126E+00_JPRB, 0.72116E+00_JPRB, 0.72034E+00_JPRB, 0.71898E+00_JPRB ,&
&  0.71689E+00_JPRB, 0.71529E+00_JPRB, 0.71438E+00_JPRB, 0.71330E+00_JPRB, 0.71164E+00_JPRB, 0.71241E+00_JPRB, 0.71395E+00_JPRB ,&
&  0.71288E+00_JPRB, 0.71128E+00_JPRB, 0.70933E+00_JPRB, 0.70762E+00_JPRB, 0.70579E+00_JPRB, 0.70373E+00_JPRB, 0.70258E+00_JPRB ,&
&  0.70208E+00_JPRB, 0.70109E+00_JPRB, 0.69941E+00_JPRB, 0.69681E+00_JPRB, 0.69615E+00_JPRB, 0.69555E+00_JPRB, 0.69485E+00_JPRB ,&
&  0.69359E+00_JPRB, 0.69019E+00_JPRB, 0.68713E+00_JPRB, 0.68520E+00_JPRB, 0.68924E+00_JPRB, 0.69378E+00_JPRB, 0.69890E+00_JPRB ,&
&  0.70448E+00_JPRB, 0.70759E+00_JPRB, 0.71182E+00_JPRB, 0.71470E+00_JPRB, 0.71494E+00_JPRB, 0.71775E+00_JPRB, 0.71950E+00_JPRB ,&
&  0.71893E+00_JPRB, 0.71608E+00_JPRB, 0.71401E+00_JPRB, 0.71347E+00_JPRB, 0.71326E+00_JPRB, 0.71261E+00_JPRB, 0.71162E+00_JPRB ,&
&  0.71111E+00_JPRB, 0.71024E+00_JPRB, 0.70880E+00_JPRB, 0.70671E+00_JPRB, 0.70387E+00_JPRB, 0.70067E+00_JPRB, 0.69708E+00_JPRB ,&
&  0.69427E+00_JPRB, 0.69099E+00_JPRB, 0.68781E+00_JPRB, 0.68463E+00_JPRB, 0.68216E+00_JPRB, 0.67913E+00_JPRB, 0.67615E+00_JPRB ,&
&  0.67479E+00_JPRB, 0.67602E+00_JPRB, 0.67810E+00_JPRB, 0.67850E+00_JPRB, 0.67894E+00_JPRB, 0.67899E+00_JPRB, 0.68028E+00_JPRB ,&
&  0.68177E+00_JPRB, 0.68488E+00_JPRB, 0.68832E+00_JPRB, 0.69310E+00_JPRB, 0.69757E+00_JPRB, 0.70198E+00_JPRB, 0.70545E+00_JPRB ,&
&  0.70855E+00_JPRB, 0.71059E+00_JPRB, 0.71188E+00_JPRB, 0.71256E+00_JPRB, 0.71342E+00_JPRB, 0.71390E+00_JPRB, 0.71377E+00_JPRB ,&
&  0.71337E+00_JPRB, 0.71321E+00_JPRB, 0.71282E+00_JPRB, 0.71216E+00_JPRB, 0.71192E+00_JPRB, 0.71140E+00_JPRB, 0.71136E+00_JPRB ,&
&  0.71141E+00_JPRB, 0.71224E+00_JPRB, 0.71320E+00_JPRB, 0.71485E+00_JPRB, 0.71677E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.71063E+00_JPRB, 0.71388E+00_JPRB, 0.71645E+00_JPRB, 0.71744E+00_JPRB, 0.71771E+00_JPRB, 0.71787E+00_JPRB, 0.71751E+00_JPRB ,&
&  0.71694E+00_JPRB, 0.71599E+00_JPRB, 0.71587E+00_JPRB, 0.71547E+00_JPRB, 0.71493E+00_JPRB, 0.71853E+00_JPRB, 0.71907E+00_JPRB ,&
&  0.71792E+00_JPRB, 0.71567E+00_JPRB, 0.71367E+00_JPRB, 0.71045E+00_JPRB, 0.70616E+00_JPRB, 0.70047E+00_JPRB, 0.68989E+00_JPRB ,&
&  0.68345E+00_JPRB, 0.67685E+00_JPRB, 0.66941E+00_JPRB, 0.66788E+00_JPRB, 0.66623E+00_JPRB, 0.64234E+00_JPRB, 0.64274E+00_JPRB ,&
&  0.66768E+00_JPRB, 0.67763E+00_JPRB, 0.68043E+00_JPRB, 0.68445E+00_JPRB, 0.65391E+00_JPRB, 0.62138E+00_JPRB, 0.60061E+00_JPRB ,&
&  0.56827E+00_JPRB, 0.59813E+00_JPRB, 0.61580E+00_JPRB, 0.62090E+00_JPRB, 0.61876E+00_JPRB, 0.61340E+00_JPRB, 0.63390E+00_JPRB ,&
&  0.68325E+00_JPRB, 0.69210E+00_JPRB, 0.70409E+00_JPRB, 0.71727E+00_JPRB, 0.73021E+00_JPRB, 0.73876E+00_JPRB, 0.74871E+00_JPRB ,&
&  0.75371E+00_JPRB, 0.75684E+00_JPRB, 0.75479E+00_JPRB, 0.75049E+00_JPRB, 0.74072E+00_JPRB, 0.73284E+00_JPRB, 0.72519E+00_JPRB ,&
&  0.72098E+00_JPRB, 0.71646E+00_JPRB, 0.71250E+00_JPRB, 0.70652E+00_JPRB, 0.70005E+00_JPRB, 0.69273E+00_JPRB, 0.68537E+00_JPRB ,&
&  0.67795E+00_JPRB, 0.67006E+00_JPRB, 0.66522E+00_JPRB, 0.66155E+00_JPRB, 0.66274E+00_JPRB, 0.66292E+00_JPRB, 0.66619E+00_JPRB ,&
&  0.66923E+00_JPRB, 0.67406E+00_JPRB, 0.67953E+00_JPRB, 0.68686E+00_JPRB, 0.69285E+00_JPRB, 0.69750E+00_JPRB, 0.70040E+00_JPRB ,&
&  0.70270E+00_JPRB, 0.70381E+00_JPRB, 0.70426E+00_JPRB, 0.70388E+00_JPRB, 0.70402E+00_JPRB, 0.70437E+00_JPRB, 0.70426E+00_JPRB ,&
&  0.70467E+00_JPRB, 0.70510E+00_JPRB, 0.70579E+00_JPRB, 0.70599E+00_JPRB, 0.70586E+00_JPRB, 0.70412E+00_JPRB, 0.70216E+00_JPRB ,&
&  0.70017E+00_JPRB, 0.70022E+00_JPRB, 0.70109E+00_JPRB, 0.70364E+00_JPRB, 0.70650E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.72155E+00_JPRB, 0.72240E+00_JPRB, 0.72392E+00_JPRB, 0.72479E+00_JPRB, 0.72515E+00_JPRB, 0.72492E+00_JPRB, 0.72434E+00_JPRB ,&
&  0.72405E+00_JPRB, 0.72416E+00_JPRB, 0.72369E+00_JPRB, 0.72233E+00_JPRB, 0.72234E+00_JPRB, 0.72289E+00_JPRB, 0.72275E+00_JPRB ,&
&  0.72015E+00_JPRB, 0.71667E+00_JPRB, 0.71378E+00_JPRB, 0.70947E+00_JPRB, 0.70528E+00_JPRB, 0.70146E+00_JPRB, 0.69230E+00_JPRB ,&
&  0.68624E+00_JPRB, 0.67674E+00_JPRB, 0.64238E+00_JPRB, 0.64633E+00_JPRB, 0.64035E+00_JPRB, 0.63966E+00_JPRB, 0.64375E+00_JPRB ,&
&  0.63316E+00_JPRB, 0.63151E+00_JPRB, 0.61752E+00_JPRB, 0.59690E+00_JPRB, 0.58703E+00_JPRB, 0.56942E+00_JPRB, 0.57066E+00_JPRB ,&
&  0.56737E+00_JPRB, 0.57338E+00_JPRB, 0.58684E+00_JPRB, 0.59101E+00_JPRB, 0.58763E+00_JPRB, 0.58482E+00_JPRB, 0.59453E+00_JPRB ,&
&  0.62213E+00_JPRB, 0.66339E+00_JPRB, 0.71727E+00_JPRB, 0.74552E+00_JPRB, 0.77317E+00_JPRB, 0.79605E+00_JPRB, 0.81699E+00_JPRB ,&
&  0.83229E+00_JPRB, 0.84416E+00_JPRB, 0.84465E+00_JPRB, 0.84267E+00_JPRB, 0.83081E+00_JPRB, 0.81288E+00_JPRB, 0.79041E+00_JPRB ,&
&  0.76878E+00_JPRB, 0.75302E+00_JPRB, 0.74139E+00_JPRB, 0.73505E+00_JPRB, 0.73238E+00_JPRB, 0.73014E+00_JPRB, 0.72904E+00_JPRB ,&
&  0.72355E+00_JPRB, 0.70821E+00_JPRB, 0.69572E+00_JPRB, 0.68304E+00_JPRB, 0.67969E+00_JPRB, 0.67828E+00_JPRB, 0.68235E+00_JPRB ,&
&  0.68716E+00_JPRB, 0.69721E+00_JPRB, 0.70909E+00_JPRB, 0.71312E+00_JPRB, 0.71938E+00_JPRB, 0.72686E+00_JPRB, 0.72609E+00_JPRB ,&
&  0.72542E+00_JPRB, 0.72343E+00_JPRB, 0.71957E+00_JPRB, 0.71487E+00_JPRB, 0.70964E+00_JPRB, 0.70487E+00_JPRB, 0.70122E+00_JPRB ,&
&  0.70020E+00_JPRB, 0.70148E+00_JPRB, 0.70674E+00_JPRB, 0.71264E+00_JPRB, 0.71956E+00_JPRB, 0.72347E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.71954E+00_JPRB, 0.71755E+00_JPRB, 0.71614E+00_JPRB, 0.71824E+00_JPRB, 0.72014E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.74014E+00_JPRB, 0.74250E+00_JPRB, 0.74773E+00_JPRB, 0.75357E+00_JPRB, 0.75841E+00_JPRB, 0.75952E+00_JPRB, 0.75197E+00_JPRB ,&
&  0.74509E+00_JPRB, 0.73837E+00_JPRB, 0.73302E+00_JPRB, 0.72932E+00_JPRB, 0.72777E+00_JPRB, 0.72456E+00_JPRB, 0.72092E+00_JPRB ,&
&  0.71780E+00_JPRB, 0.71639E+00_JPRB, 0.71632E+00_JPRB, 0.71602E+00_JPRB, 0.71765E+00_JPRB, 0.71693E+00_JPRB, 0.71420E+00_JPRB ,&
&  0.71163E+00_JPRB, 0.69794E+00_JPRB, 0.67062E+00_JPRB, 0.66200E+00_JPRB, 0.65230E+00_JPRB, 0.65314E+00_JPRB, 0.65336E+00_JPRB ,&
&  0.64713E+00_JPRB, 0.58435E+00_JPRB, 0.59435E+00_JPRB, 0.57964E+00_JPRB, 0.57151E+00_JPRB, 0.57244E+00_JPRB, 0.55779E+00_JPRB ,&
&  0.54844E+00_JPRB, 0.53665E+00_JPRB, 0.53694E+00_JPRB, 0.54460E+00_JPRB, 0.54321E+00_JPRB, 0.55322E+00_JPRB, 0.56739E+00_JPRB ,&
&  0.59270E+00_JPRB, 0.70515E+00_JPRB, 0.75679E+00_JPRB, 0.78697E+00_JPRB, 0.81217E+00_JPRB, 0.84108E+00_JPRB, 0.86899E+00_JPRB ,&
&  0.89057E+00_JPRB, 0.91712E+00_JPRB, 0.92831E+00_JPRB, 0.92327E+00_JPRB, 0.93056E+00_JPRB, 0.92168E+00_JPRB, 0.89888E+00_JPRB ,&
&  0.86491E+00_JPRB, 0.82007E+00_JPRB, 0.78969E+00_JPRB, 0.77125E+00_JPRB, 0.75777E+00_JPRB, 0.74742E+00_JPRB, 0.74087E+00_JPRB ,&
&  0.74495E+00_JPRB, 0.73937E+00_JPRB, 0.73152E+00_JPRB, 0.71654E+00_JPRB, 0.70870E+00_JPRB, 0.70476E+00_JPRB, 0.70686E+00_JPRB ,&
&  0.71159E+00_JPRB, 0.72244E+00_JPRB, 0.73561E+00_JPRB, 0.74597E+00_JPRB, 0.75140E+00_JPRB, 0.75064E+00_JPRB, 0.75488E+00_JPRB ,&
&  0.75395E+00_JPRB, 0.75008E+00_JPRB, 0.74562E+00_JPRB, 0.74084E+00_JPRB, 0.73621E+00_JPRB, 0.73343E+00_JPRB, 0.73614E+00_JPRB ,&
&  0.73980E+00_JPRB, 0.74214E+00_JPRB, 0.74293E+00_JPRB, 0.74538E+00_JPRB, 0.74991E+00_JPRB, 0.75386E+00_JPRB, 0.75527E+00_JPRB ,&
&  0.74933E+00_JPRB, 0.74314E+00_JPRB, 0.73822E+00_JPRB, 0.73850E+00_JPRB, 0.73899E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.73374E+00_JPRB, 0.74407E+00_JPRB, 0.76255E+00_JPRB, 0.77793E+00_JPRB, 0.78830E+00_JPRB, 0.78744E+00_JPRB, 0.77556E+00_JPRB ,&
&  0.76384E+00_JPRB, 0.75530E+00_JPRB, 0.74691E+00_JPRB, 0.73938E+00_JPRB, 0.73083E+00_JPRB, 0.72428E+00_JPRB, 0.71954E+00_JPRB ,&
&  0.71694E+00_JPRB, 0.71716E+00_JPRB, 0.71752E+00_JPRB, 0.71780E+00_JPRB, 0.71938E+00_JPRB, 0.72181E+00_JPRB, 0.72175E+00_JPRB ,&
&  0.72116E+00_JPRB, 0.71038E+00_JPRB, 0.68651E+00_JPRB, 0.66856E+00_JPRB, 0.66232E+00_JPRB, 0.65114E+00_JPRB, 0.65527E+00_JPRB ,&
&  0.65501E+00_JPRB, 0.65352E+00_JPRB, 0.64539E+00_JPRB, 0.64135E+00_JPRB, 0.61861E+00_JPRB, 0.53923E+00_JPRB, 0.54423E+00_JPRB ,&
&  0.53136E+00_JPRB, 0.51370E+00_JPRB, 0.51071E+00_JPRB, 0.51056E+00_JPRB, 0.51226E+00_JPRB, 0.53747E+00_JPRB, 0.56471E+00_JPRB ,&
&  0.59084E+00_JPRB, 0.71648E+00_JPRB, 0.75110E+00_JPRB, 0.78837E+00_JPRB, 0.82164E+00_JPRB, 0.86222E+00_JPRB, 0.90442E+00_JPRB ,&
&  0.93096E+00_JPRB, 0.95965E+00_JPRB, 0.98178E+00_JPRB, 0.99764E+00_JPRB, 0.10031E+01_JPRB, 0.10068E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.99637E+00_JPRB, 0.91860E+00_JPRB, 0.86782E+00_JPRB, 0.83833E+00_JPRB, 0.81092E+00_JPRB, 0.79257E+00_JPRB, 0.77790E+00_JPRB ,&
&  0.77350E+00_JPRB, 0.75719E+00_JPRB, 0.75511E+00_JPRB, 0.74583E+00_JPRB, 0.74083E+00_JPRB, 0.73731E+00_JPRB, 0.73300E+00_JPRB ,&
&  0.72565E+00_JPRB, 0.72932E+00_JPRB, 0.74574E+00_JPRB, 0.75407E+00_JPRB, 0.76020E+00_JPRB, 0.75574E+00_JPRB, 0.74737E+00_JPRB ,&
&  0.73294E+00_JPRB, 0.73064E+00_JPRB, 0.72399E+00_JPRB, 0.72244E+00_JPRB, 0.72389E+00_JPRB, 0.73459E+00_JPRB, 0.74875E+00_JPRB ,&
&  0.75789E+00_JPRB, 0.76080E+00_JPRB, 0.76030E+00_JPRB, 0.76079E+00_JPRB, 0.75936E+00_JPRB, 0.75619E+00_JPRB, 0.75003E+00_JPRB ,&
&  0.74404E+00_JPRB, 0.74086E+00_JPRB, 0.73463E+00_JPRB, 0.73032E+00_JPRB, 0.72970E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.72563E+00_JPRB, 0.74970E+00_JPRB, 0.77836E+00_JPRB, 0.79760E+00_JPRB, 0.80573E+00_JPRB, 0.79306E+00_JPRB, 0.77479E+00_JPRB ,&
&  0.76030E+00_JPRB, 0.75006E+00_JPRB, 0.74191E+00_JPRB, 0.73648E+00_JPRB, 0.73019E+00_JPRB, 0.72394E+00_JPRB, 0.71875E+00_JPRB ,&
&  0.71506E+00_JPRB, 0.70391E+00_JPRB, 0.70073E+00_JPRB, 0.70451E+00_JPRB, 0.70677E+00_JPRB, 0.71203E+00_JPRB, 0.71626E+00_JPRB ,&
&  0.71297E+00_JPRB, 0.70347E+00_JPRB, 0.69510E+00_JPRB, 0.67638E+00_JPRB, 0.65832E+00_JPRB, 0.65355E+00_JPRB, 0.66042E+00_JPRB ,&
&  0.64474E+00_JPRB, 0.62032E+00_JPRB, 0.64301E+00_JPRB, 0.66105E+00_JPRB, 0.62677E+00_JPRB, 0.59324E+00_JPRB, 0.55019E+00_JPRB ,&
&  0.53682E+00_JPRB, 0.50894E+00_JPRB, 0.50241E+00_JPRB, 0.50206E+00_JPRB, 0.51178E+00_JPRB, 0.53287E+00_JPRB, 0.57594E+00_JPRB ,&
&  0.62719E+00_JPRB, 0.71644E+00_JPRB, 0.74702E+00_JPRB, 0.79329E+00_JPRB, 0.83199E+00_JPRB, 0.86626E+00_JPRB, 0.89054E+00_JPRB ,&
&  0.92941E+00_JPRB, 0.98466E+00_JPRB, 0.10207E+01_JPRB, 0.10312E+01_JPRB, 0.10477E+01_JPRB, 0.10750E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.10511E+01_JPRB, 0.96441E+00_JPRB, 0.90911E+00_JPRB, 0.87463E+00_JPRB, 0.86250E+00_JPRB, 0.81813E+00_JPRB ,&
&  0.80408E+00_JPRB, 0.80059E+00_JPRB, 0.79042E+00_JPRB, 0.78618E+00_JPRB, 0.78273E+00_JPRB, 0.77901E+00_JPRB, 0.77131E+00_JPRB ,&
&  0.76067E+00_JPRB, 0.76708E+00_JPRB, 0.83561E+00_JPRB, 0.85178E+00_JPRB, 0.83752E+00_JPRB, 0.79195E+00_JPRB, 0.74379E+00_JPRB ,&
&  0.71072E+00_JPRB, 0.69790E+00_JPRB, 0.69321E+00_JPRB, 0.69134E+00_JPRB, 0.69922E+00_JPRB, 0.71080E+00_JPRB, 0.72895E+00_JPRB ,&
&  0.72962E+00_JPRB, 0.73983E+00_JPRB, 0.74996E+00_JPRB, 0.75537E+00_JPRB, 0.75480E+00_JPRB, 0.74901E+00_JPRB, 0.74480E+00_JPRB ,&
&  0.74108E+00_JPRB, 0.73701E+00_JPRB, 0.72978E+00_JPRB, 0.72155E+00_JPRB, 0.71870E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.70668E+00_JPRB, 0.74576E+00_JPRB, 0.79315E+00_JPRB, 0.82331E+00_JPRB, 0.82334E+00_JPRB, 0.79633E+00_JPRB, 0.76705E+00_JPRB ,&
&  0.74770E+00_JPRB, 0.73853E+00_JPRB, 0.73276E+00_JPRB, 0.72743E+00_JPRB, 0.72265E+00_JPRB, 0.71762E+00_JPRB, 0.71253E+00_JPRB ,&
&  0.70862E+00_JPRB, 0.70491E+00_JPRB, 0.69837E+00_JPRB, 0.68985E+00_JPRB, 0.68545E+00_JPRB, 0.68710E+00_JPRB, 0.69202E+00_JPRB ,&
&  0.69660E+00_JPRB, 0.69429E+00_JPRB, 0.69287E+00_JPRB, 0.68202E+00_JPRB, 0.66849E+00_JPRB, 0.65273E+00_JPRB, 0.64195E+00_JPRB ,&
&  0.63837E+00_JPRB, 0.64232E+00_JPRB, 0.63444E+00_JPRB, 0.65687E+00_JPRB, 0.65180E+00_JPRB, 0.61388E+00_JPRB, 0.57399E+00_JPRB ,&
&  0.54815E+00_JPRB, 0.51722E+00_JPRB, 0.50567E+00_JPRB, 0.51120E+00_JPRB, 0.54330E+00_JPRB, 0.58121E+00_JPRB, 0.63439E+00_JPRB ,&
&  0.72546E+00_JPRB, 0.79002E+00_JPRB, 0.83400E+00_JPRB, 0.87148E+00_JPRB, 0.90085E+00_JPRB, 0.93342E+00_JPRB, 0.97158E+00_JPRB ,&
&  0.10212E+01_JPRB, 0.10671E+01_JPRB, 0.10929E+01_JPRB, 0.10722E+01_JPRB, 0.10662E+01_JPRB, 0.10159E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.12709E+01_JPRB, 0.11146E+01_JPRB, 0.10343E+01_JPRB, 0.98591E+00_JPRB, 0.94738E+00_JPRB, 0.91219E+00_JPRB ,&
&  0.87553E+00_JPRB, 0.85452E+00_JPRB, 0.84420E+00_JPRB, 0.81003E+00_JPRB, 0.77687E+00_JPRB, 0.77711E+00_JPRB, 0.82142E+00_JPRB ,&
&  0.93081E+00_JPRB, 0.10912E+01_JPRB, 0.12921E+01_JPRB, 0.93796E+00_JPRB, 0.82749E+00_JPRB, 0.74863E+00_JPRB, 0.71186E+00_JPRB ,&
&  0.70132E+00_JPRB, 0.70042E+00_JPRB, 0.70240E+00_JPRB, 0.69872E+00_JPRB, 0.69602E+00_JPRB, 0.69911E+00_JPRB, 0.70087E+00_JPRB ,&
&  0.69914E+00_JPRB, 0.70938E+00_JPRB, 0.72407E+00_JPRB, 0.73556E+00_JPRB, 0.74609E+00_JPRB, 0.74810E+00_JPRB, 0.74775E+00_JPRB ,&
&  0.74525E+00_JPRB, 0.73669E+00_JPRB, 0.72582E+00_JPRB, 0.70696E+00_JPRB, 0.69723E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.70226E+00_JPRB, 0.74657E+00_JPRB, 0.80824E+00_JPRB, 0.84448E+00_JPRB, 0.84152E+00_JPRB, 0.79982E+00_JPRB, 0.74770E+00_JPRB ,&
&  0.71211E+00_JPRB, 0.69041E+00_JPRB, 0.68365E+00_JPRB, 0.68056E+00_JPRB, 0.68026E+00_JPRB, 0.68052E+00_JPRB, 0.67932E+00_JPRB ,&
&  0.67685E+00_JPRB, 0.67502E+00_JPRB, 0.67252E+00_JPRB, 0.67022E+00_JPRB, 0.66634E+00_JPRB, 0.66375E+00_JPRB, 0.66469E+00_JPRB ,&
&  0.66799E+00_JPRB, 0.66824E+00_JPRB, 0.66754E+00_JPRB, 0.66589E+00_JPRB, 0.66783E+00_JPRB, 0.67280E+00_JPRB, 0.67074E+00_JPRB ,&
&  0.65288E+00_JPRB, 0.64018E+00_JPRB, 0.64784E+00_JPRB, 0.66485E+00_JPRB, 0.66367E+00_JPRB, 0.63419E+00_JPRB, 0.58882E+00_JPRB ,&
&  0.55602E+00_JPRB, 0.53409E+00_JPRB, 0.55227E+00_JPRB, 0.59775E+00_JPRB, 0.65327E+00_JPRB, 0.76947E+00_JPRB, 0.85845E+00_JPRB ,&
&  0.93183E+00_JPRB, 0.97221E+00_JPRB, 0.99727E+00_JPRB, 0.10234E+01_JPRB, 0.10750E+01_JPRB, 0.11285E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.11988E+01_JPRB, 0.12180E+01_JPRB, 0.12283E+01_JPRB, 0.98917E+00_JPRB, 0.96959E+00_JPRB, 0.10660E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.11863E+01_JPRB, 0.11484E+01_JPRB, 0.11650E+01_JPRB, 0.10972E+01_JPRB, 0.10508E+01_JPRB, 0.10171E+01_JPRB ,&
&  0.98285E+00_JPRB, 0.97356E+00_JPRB, 0.91432E+00_JPRB, 0.84964E+00_JPRB, 0.84529E+00_JPRB, 0.84839E+00_JPRB, 0.86085E+00_JPRB ,&
&  0.89863E+00_JPRB, 0.92556E+00_JPRB, 0.85859E+00_JPRB, 0.70790E+00_JPRB, 0.67030E+00_JPRB, 0.65698E+00_JPRB, 0.66036E+00_JPRB ,&
&  0.66764E+00_JPRB, 0.67509E+00_JPRB, 0.68511E+00_JPRB, 0.68731E+00_JPRB, 0.68726E+00_JPRB, 0.68296E+00_JPRB, 0.67644E+00_JPRB ,&
&  0.66835E+00_JPRB, 0.67020E+00_JPRB, 0.68088E+00_JPRB, 0.68367E+00_JPRB, 0.70069E+00_JPRB, 0.71602E+00_JPRB, 0.73075E+00_JPRB ,&
&  0.72802E+00_JPRB, 0.72141E+00_JPRB, 0.71334E+00_JPRB, 0.69561E+00_JPRB, 0.68385E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.74440E+00_JPRB, 0.78866E+00_JPRB, 0.82788E+00_JPRB, 0.85006E+00_JPRB, 0.84294E+00_JPRB, 0.81228E+00_JPRB, 0.74525E+00_JPRB ,&
&  0.72006E+00_JPRB, 0.68282E+00_JPRB, 0.66905E+00_JPRB, 0.65696E+00_JPRB, 0.64679E+00_JPRB, 0.63896E+00_JPRB, 0.63705E+00_JPRB ,&
&  0.64044E+00_JPRB, 0.64402E+00_JPRB, 0.64616E+00_JPRB, 0.64879E+00_JPRB, 0.64811E+00_JPRB, 0.64831E+00_JPRB, 0.64942E+00_JPRB ,&
&  0.65004E+00_JPRB, 0.65088E+00_JPRB, 0.65367E+00_JPRB, 0.65768E+00_JPRB, 0.66340E+00_JPRB, 0.66801E+00_JPRB, 0.66563E+00_JPRB ,&
&  0.65438E+00_JPRB, 0.64421E+00_JPRB, 0.65495E+00_JPRB, 0.67224E+00_JPRB, 0.67137E+00_JPRB, 0.65293E+00_JPRB, 0.60459E+00_JPRB ,&
&  0.57079E+00_JPRB, 0.58288E+00_JPRB, 0.67039E+00_JPRB, 0.81174E+00_JPRB, 0.87874E+00_JPRB, 0.94359E+00_JPRB, 0.98224E+00_JPRB ,&
&  0.94046E+00_JPRB, 0.95042E+00_JPRB, 0.10127E+01_JPRB, 0.11899E+01_JPRB, 0.12261E+01_JPRB, 0.12718E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.13958E+01_JPRB, 0.13999E+01_JPRB, 0.13766E+01_JPRB, 0.11918E+01_JPRB, 0.13450E+01_JPRB, 0.16254E+01_JPRB, 0.14656E+01_JPRB ,&
&  0.14391E+01_JPRB, 0.14024E+01_JPRB, 0.14341E+01_JPRB, 0.13476E+01_JPRB, 0.12202E+01_JPRB, 0.11498E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.10738E+01_JPRB, 0.96520E+00_JPRB, 0.93018E+00_JPRB, 0.93791E+00_JPRB, 0.90853E+00_JPRB, 0.88321E+00_JPRB ,&
&  0.85697E+00_JPRB, 0.80425E+00_JPRB, 0.76124E+00_JPRB, 0.71372E+00_JPRB, 0.68210E+00_JPRB, 0.67551E+00_JPRB, 0.67855E+00_JPRB ,&
&  0.68201E+00_JPRB, 0.68423E+00_JPRB, 0.69172E+00_JPRB, 0.70046E+00_JPRB, 0.70106E+00_JPRB, 0.68674E+00_JPRB, 0.68337E+00_JPRB ,&
&  0.67767E+00_JPRB, 0.66607E+00_JPRB, 0.65770E+00_JPRB, 0.66521E+00_JPRB, 0.67398E+00_JPRB, 0.68941E+00_JPRB, 0.70730E+00_JPRB ,&
&  0.72001E+00_JPRB, 0.72746E+00_JPRB, 0.72351E+00_JPRB, 0.72587E+00_JPRB, 0.72342E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.78876E+00_JPRB, 0.80854E+00_JPRB, 0.83576E+00_JPRB, 0.85657E+00_JPRB, 0.86748E+00_JPRB, 0.81068E+00_JPRB, 0.75668E+00_JPRB ,&
&  0.77249E+00_JPRB, 0.70284E+00_JPRB, 0.66838E+00_JPRB, 0.65611E+00_JPRB, 0.65177E+00_JPRB, 0.63446E+00_JPRB, 0.62249E+00_JPRB ,&
&  0.61846E+00_JPRB, 0.62453E+00_JPRB, 0.63297E+00_JPRB, 0.63786E+00_JPRB, 0.64054E+00_JPRB, 0.64280E+00_JPRB, 0.64364E+00_JPRB ,&
&  0.64507E+00_JPRB, 0.64761E+00_JPRB, 0.65108E+00_JPRB, 0.65432E+00_JPRB, 0.65586E+00_JPRB, 0.65886E+00_JPRB, 0.65854E+00_JPRB ,&
&  0.65078E+00_JPRB, 0.64687E+00_JPRB, 0.65909E+00_JPRB, 0.68027E+00_JPRB, 0.68959E+00_JPRB, 0.69101E+00_JPRB, 0.66843E+00_JPRB ,&
&  0.61174E+00_JPRB, 0.66657E+00_JPRB, 0.86392E+00_JPRB, 0.93502E+00_JPRB, 0.98339E+00_JPRB, 0.10454E+01_JPRB, 0.10994E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.11768E+01_JPRB, 0.12076E+01_JPRB, 0.12510E+01_JPRB, 0.13076E+01_JPRB, 0.13624E+01_JPRB, 0.14441E+01_JPRB ,&
&  0.16274E+01_JPRB, 0.13135E+01_JPRB, 0.12306E+01_JPRB, 0.13028E+01_JPRB, 0.17818E+01_JPRB, 0.18584E+01_JPRB, 0.16628E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.15840E+01_JPRB, 0.15267E+01_JPRB, 0.14137E+01_JPRB, 0.13283E+01_JPRB, 0.13017E+01_JPRB, 0.12867E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.11751E+01_JPRB, 0.10521E+01_JPRB, 0.10280E+01_JPRB, 0.10260E+01_JPRB, 0.97392E+00_JPRB, 0.90074E+00_JPRB ,&
&  0.84268E+00_JPRB, 0.79766E+00_JPRB, 0.76598E+00_JPRB, 0.74269E+00_JPRB, 0.72420E+00_JPRB, 0.71348E+00_JPRB, 0.70604E+00_JPRB ,&
&  0.70296E+00_JPRB, 0.70250E+00_JPRB, 0.69519E+00_JPRB, 0.70221E+00_JPRB, 0.70827E+00_JPRB, 0.70165E+00_JPRB, 0.70994E+00_JPRB ,&
&  0.71206E+00_JPRB, 0.70467E+00_JPRB, 0.68814E+00_JPRB, 0.69603E+00_JPRB, 0.71493E+00_JPRB, 0.74702E+00_JPRB, 0.77720E+00_JPRB ,&
&  0.81488E+00_JPRB, 0.80977E+00_JPRB, 0.77684E+00_JPRB, 0.76911E+00_JPRB, 0.76973E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.82272E+00_JPRB, 0.83783E+00_JPRB, 0.84915E+00_JPRB, 0.86744E+00_JPRB, 0.88227E+00_JPRB, 0.83497E+00_JPRB, 0.79928E+00_JPRB ,&
&  0.85792E+00_JPRB, 0.77141E+00_JPRB, 0.81598E+00_JPRB, 0.72768E+00_JPRB, 0.68336E+00_JPRB, 0.65900E+00_JPRB, 0.64088E+00_JPRB ,&
&  0.62520E+00_JPRB, 0.63113E+00_JPRB, 0.64052E+00_JPRB, 0.64773E+00_JPRB, 0.65920E+00_JPRB, 0.65466E+00_JPRB, 0.65045E+00_JPRB ,&
&  0.64762E+00_JPRB, 0.64823E+00_JPRB, 0.65597E+00_JPRB, 0.65650E+00_JPRB, 0.65498E+00_JPRB, 0.65352E+00_JPRB, 0.64520E+00_JPRB ,&
&  0.62893E+00_JPRB, 0.63118E+00_JPRB, 0.65513E+00_JPRB, 0.67814E+00_JPRB, 0.70205E+00_JPRB, 0.71476E+00_JPRB, 0.73121E+00_JPRB ,&
&  0.74983E+00_JPRB, 0.77647E+00_JPRB, 0.91170E+00_JPRB, 0.95657E+00_JPRB, 0.10054E+01_JPRB, 0.10412E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.11636E+01_JPRB, 0.11862E+01_JPRB, 0.12309E+01_JPRB, 0.13185E+01_JPRB, 0.13969E+01_JPRB, 0.15370E+01_JPRB ,&
&  0.17685E+01_JPRB, 0.13626E+01_JPRB, 0.15914E+01_JPRB, 0.15906E+01_JPRB, 0.18367E+01_JPRB, 0.20230E+01_JPRB, 0.20176E+01_JPRB ,&
&  0.22320E+01_JPRB, 0.19792E+01_JPRB, 0.17938E+01_JPRB, 0.15941E+01_JPRB, 0.14387E+01_JPRB, 0.14070E+01_JPRB, 0.13888E+01_JPRB ,&
&  0.14220E+01_JPRB, 0.13879E+01_JPRB, 0.12037E+01_JPRB, 0.11178E+01_JPRB, 0.10876E+01_JPRB, 0.10647E+01_JPRB, 0.98442E+00_JPRB ,&
&  0.94158E+00_JPRB, 0.91462E+00_JPRB, 0.87560E+00_JPRB, 0.83367E+00_JPRB, 0.80176E+00_JPRB, 0.78410E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75545E+00_JPRB, 0.74240E+00_JPRB, 0.70210E+00_JPRB, 0.69430E+00_JPRB, 0.69501E+00_JPRB, 0.67711E+00_JPRB, 0.68738E+00_JPRB ,&
&  0.70914E+00_JPRB, 0.71593E+00_JPRB, 0.76127E+00_JPRB, 0.82333E+00_JPRB, 0.84399E+00_JPRB, 0.85285E+00_JPRB, 0.93369E+00_JPRB ,&
&  0.87255E+00_JPRB, 0.87399E+00_JPRB, 0.86976E+00_JPRB, 0.83464E+00_JPRB, 0.81795E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.87009E+00_JPRB, 0.86614E+00_JPRB, 0.86453E+00_JPRB, 0.87912E+00_JPRB, 0.89482E+00_JPRB, 0.90020E+00_JPRB, 0.86789E+00_JPRB ,&
&  0.91710E+00_JPRB, 0.88681E+00_JPRB, 0.86875E+00_JPRB, 0.86083E+00_JPRB, 0.84306E+00_JPRB, 0.75170E+00_JPRB, 0.65791E+00_JPRB ,&
&  0.63349E+00_JPRB, 0.63070E+00_JPRB, 0.65115E+00_JPRB, 0.65551E+00_JPRB, 0.66493E+00_JPRB, 0.66538E+00_JPRB, 0.65836E+00_JPRB ,&
&  0.65644E+00_JPRB, 0.65652E+00_JPRB, 0.66718E+00_JPRB, 0.66923E+00_JPRB, 0.66215E+00_JPRB, 0.66079E+00_JPRB, 0.65668E+00_JPRB ,&
&  0.63077E+00_JPRB, 0.62075E+00_JPRB, 0.63110E+00_JPRB, 0.63942E+00_JPRB, 0.69987E+00_JPRB, 0.72914E+00_JPRB, 0.75626E+00_JPRB ,&
&  0.78511E+00_JPRB, 0.82485E+00_JPRB, 0.89129E+00_JPRB, 0.94601E+00_JPRB, 0.98322E+00_JPRB, 0.10113E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10845E+01_JPRB, 0.11289E+01_JPRB, 0.11813E+01_JPRB, 0.12693E+01_JPRB, 0.14010E+01_JPRB, 0.13362E+01_JPRB, 0.15969E+01_JPRB ,&
&  0.17215E+01_JPRB, 0.19062E+01_JPRB, 0.20829E+01_JPRB, 0.19666E+01_JPRB, 0.21664E+01_JPRB, 0.22945E+01_JPRB, 0.22286E+01_JPRB ,&
&  0.21839E+01_JPRB, 0.19587E+01_JPRB, 0.19228E+01_JPRB, 0.18939E+01_JPRB, 0.18954E+01_JPRB, 0.17708E+01_JPRB, 0.16480E+01_JPRB ,&
&  0.17367E+01_JPRB, 0.19697E+01_JPRB, 0.15752E+01_JPRB, 0.14391E+01_JPRB, 0.13219E+01_JPRB, 0.12827E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.11869E+01_JPRB, 0.11464E+01_JPRB, 0.10876E+01_JPRB, 0.97265E+00_JPRB, 0.92053E+00_JPRB, 0.85896E+00_JPRB ,&
&  0.82763E+00_JPRB, 0.78408E+00_JPRB, 0.72176E+00_JPRB, 0.69908E+00_JPRB, 0.68016E+00_JPRB, 0.66856E+00_JPRB, 0.68776E+00_JPRB ,&
&  0.75148E+00_JPRB, 0.82579E+00_JPRB, 0.98232E+00_JPRB, 0.10408E+01_JPRB, 0.10754E+01_JPRB, 0.11187E+01_JPRB, 0.97627E+00_JPRB ,&
&  0.88908E+00_JPRB, 0.96509E+00_JPRB, 0.93612E+00_JPRB, 0.90166E+00_JPRB, 0.87869E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.95530E+00_JPRB, 0.93569E+00_JPRB, 0.91607E+00_JPRB, 0.91568E+00_JPRB, 0.92696E+00_JPRB, 0.92495E+00_JPRB, 0.93022E+00_JPRB ,&
&  0.94694E+00_JPRB, 0.93833E+00_JPRB, 0.91155E+00_JPRB, 0.89608E+00_JPRB, 0.87620E+00_JPRB, 0.87144E+00_JPRB, 0.77493E+00_JPRB ,&
&  0.68830E+00_JPRB, 0.66685E+00_JPRB, 0.64778E+00_JPRB, 0.65464E+00_JPRB, 0.68223E+00_JPRB, 0.67880E+00_JPRB, 0.67630E+00_JPRB ,&
&  0.67479E+00_JPRB, 0.67483E+00_JPRB, 0.68040E+00_JPRB, 0.68227E+00_JPRB, 0.68768E+00_JPRB, 0.70766E+00_JPRB, 0.71463E+00_JPRB ,&
&  0.67147E+00_JPRB, 0.62461E+00_JPRB, 0.60927E+00_JPRB, 0.62435E+00_JPRB, 0.67975E+00_JPRB, 0.73639E+00_JPRB, 0.77397E+00_JPRB ,&
&  0.80509E+00_JPRB, 0.83894E+00_JPRB, 0.88258E+00_JPRB, 0.93875E+00_JPRB, 0.97232E+00_JPRB, 0.10051E+01_JPRB, 0.10427E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.11477E+01_JPRB, 0.12301E+01_JPRB, 0.13554E+01_JPRB, 0.13614E+01_JPRB, 0.14379E+01_JPRB, 0.16448E+01_JPRB ,&
&  0.17872E+01_JPRB, 0.19422E+01_JPRB, 0.21067E+01_JPRB, 0.24521E+01_JPRB, 0.26623E+01_JPRB, 0.25803E+01_JPRB, 0.24217E+01_JPRB ,&
&  0.23153E+01_JPRB, 0.20811E+01_JPRB, 0.23667E+01_JPRB, 0.21967E+01_JPRB, 0.20633E+01_JPRB, 0.20676E+01_JPRB, 0.19747E+01_JPRB ,&
&  0.19559E+01_JPRB, 0.20331E+01_JPRB, 0.19701E+01_JPRB, 0.19392E+01_JPRB, 0.17983E+01_JPRB, 0.17686E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.16560E+01_JPRB, 0.16432E+01_JPRB, 0.14212E+01_JPRB, 0.12248E+01_JPRB, 0.10425E+01_JPRB, 0.10034E+01_JPRB, 0.92554E+00_JPRB ,&
&  0.86835E+00_JPRB, 0.80660E+00_JPRB, 0.77392E+00_JPRB, 0.73551E+00_JPRB, 0.72869E+00_JPRB, 0.75769E+00_JPRB, 0.77685E+00_JPRB ,&
&  0.85542E+00_JPRB, 0.10328E+01_JPRB, 0.11352E+01_JPRB, 0.12010E+01_JPRB, 0.12511E+01_JPRB, 0.12953E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.96549E+00_JPRB, 0.10264E+01_JPRB, 0.10122E+01_JPRB, 0.98679E+00_JPRB, 0.96708E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.10830E+01_JPRB, 0.10437E+01_JPRB, 0.10073E+01_JPRB, 0.98519E+00_JPRB, 0.97094E+00_JPRB, 0.96289E+00_JPRB, 0.97614E+00_JPRB ,&
&  0.99026E+00_JPRB, 0.97619E+00_JPRB, 0.95444E+00_JPRB, 0.93640E+00_JPRB, 0.91642E+00_JPRB, 0.91498E+00_JPRB, 0.92580E+00_JPRB ,&
&  0.78990E+00_JPRB, 0.73881E+00_JPRB, 0.66553E+00_JPRB, 0.65846E+00_JPRB, 0.74229E+00_JPRB, 0.70863E+00_JPRB, 0.69519E+00_JPRB ,&
&  0.68431E+00_JPRB, 0.69559E+00_JPRB, 0.70247E+00_JPRB, 0.73967E+00_JPRB, 0.78067E+00_JPRB, 0.82692E+00_JPRB, 0.83098E+00_JPRB ,&
&  0.76409E+00_JPRB, 0.68567E+00_JPRB, 0.65249E+00_JPRB, 0.64901E+00_JPRB, 0.69010E+00_JPRB, 0.73158E+00_JPRB, 0.80420E+00_JPRB ,&
&  0.84891E+00_JPRB, 0.88598E+00_JPRB, 0.92261E+00_JPRB, 0.97274E+00_JPRB, 0.10096E+01_JPRB, 0.10529E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.12532E+01_JPRB, 0.13376E+01_JPRB, 0.14485E+01_JPRB, 0.14816E+01_JPRB, 0.15717E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.18446E+01_JPRB, 0.19382E+01_JPRB, 0.21568E+01_JPRB, 0.29194E+01_JPRB, 0.33923E+01_JPRB, 0.34082E+01_JPRB, 0.30600E+01_JPRB ,&
&  0.26874E+01_JPRB, 0.23263E+01_JPRB, 0.23003E+01_JPRB, 0.21862E+01_JPRB, 0.20578E+01_JPRB, 0.21664E+01_JPRB, 0.21139E+01_JPRB ,&
&  0.20336E+01_JPRB, 0.20611E+01_JPRB, 0.20715E+01_JPRB, 0.21415E+01_JPRB, 0.20564E+01_JPRB, 0.20793E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.20531E+01_JPRB, 0.19002E+01_JPRB, 0.14636E+01_JPRB, 0.12517E+01_JPRB, 0.10501E+01_JPRB, 0.98724E+00_JPRB, 0.11268E+01_JPRB ,&
&  0.10149E+01_JPRB, 0.96929E+00_JPRB, 0.94870E+00_JPRB, 0.93042E+00_JPRB, 0.97676E+00_JPRB, 0.11322E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.99591E+00_JPRB, 0.11417E+01_JPRB, 0.13842E+01_JPRB, 0.15634E+01_JPRB, 0.16406E+01_JPRB, 0.15933E+01_JPRB, 0.12971E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.12012E+01_JPRB, 0.11737E+01_JPRB, 0.11317E+01_JPRB, 0.11042E+01_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.12792E+01_JPRB, 0.12291E+01_JPRB, 0.11715E+01_JPRB, 0.11015E+01_JPRB, 0.10609E+01_JPRB, 0.10504E+01_JPRB, 0.10589E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10217E+01_JPRB, 0.99878E+00_JPRB, 0.97909E+00_JPRB, 0.97088E+00_JPRB, 0.97706E+00_JPRB, 0.98575E+00_JPRB ,&
&  0.10002E+01_JPRB, 0.90177E+00_JPRB, 0.76971E+00_JPRB, 0.68655E+00_JPRB, 0.71512E+00_JPRB, 0.69920E+00_JPRB, 0.70263E+00_JPRB ,&
&  0.70266E+00_JPRB, 0.75535E+00_JPRB, 0.79264E+00_JPRB, 0.87523E+00_JPRB, 0.96969E+00_JPRB, 0.10502E+01_JPRB, 0.10477E+01_JPRB ,&
&  0.96064E+00_JPRB, 0.84248E+00_JPRB, 0.77228E+00_JPRB, 0.75673E+00_JPRB, 0.79621E+00_JPRB, 0.78081E+00_JPRB, 0.84680E+00_JPRB ,&
&  0.89035E+00_JPRB, 0.93567E+00_JPRB, 0.98443E+00_JPRB, 0.10263E+01_JPRB, 0.10836E+01_JPRB, 0.11542E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.13109E+01_JPRB, 0.13762E+01_JPRB, 0.14058E+01_JPRB, 0.14583E+01_JPRB, 0.15802E+01_JPRB, 0.17183E+01_JPRB, 0.16976E+01_JPRB ,&
&  0.17967E+01_JPRB, 0.19001E+01_JPRB, 0.21093E+01_JPRB, 0.28521E+01_JPRB, 0.34775E+01_JPRB, 0.36520E+01_JPRB, 0.35126E+01_JPRB ,&
&  0.32129E+01_JPRB, 0.29026E+01_JPRB, 0.28507E+01_JPRB, 0.25926E+01_JPRB, 0.24492E+01_JPRB, 0.24548E+01_JPRB, 0.24236E+01_JPRB ,&
&  0.23295E+01_JPRB, 0.23165E+01_JPRB, 0.23552E+01_JPRB, 0.23036E+01_JPRB, 0.23088E+01_JPRB, 0.22998E+01_JPRB, 0.23753E+01_JPRB ,&
&  0.26182E+01_JPRB, 0.18298E+01_JPRB, 0.14691E+01_JPRB, 0.15197E+01_JPRB, 0.12633E+01_JPRB, 0.11171E+01_JPRB, 0.11421E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10677E+01_JPRB, 0.12245E+01_JPRB, 0.12283E+01_JPRB, 0.14391E+01_JPRB, 0.17223E+01_JPRB, 0.15562E+01_JPRB ,&
&  0.15037E+01_JPRB, 0.15157E+01_JPRB, 0.18865E+01_JPRB, 0.21683E+01_JPRB, 0.22130E+01_JPRB, 0.20300E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.14669E+01_JPRB, 0.14162E+01_JPRB, 0.13687E+01_JPRB, 0.13315E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.15088E+01_JPRB, 0.14379E+01_JPRB, 0.13777E+01_JPRB, 0.13130E+01_JPRB, 0.12663E+01_JPRB, 0.12296E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.10799E+01_JPRB, 0.10520E+01_JPRB, 0.10412E+01_JPRB, 0.10427E+01_JPRB, 0.10553E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.10371E+01_JPRB, 0.86820E+00_JPRB, 0.75121E+00_JPRB, 0.67252E+00_JPRB, 0.68135E+00_JPRB, 0.69373E+00_JPRB ,&
&  0.73673E+00_JPRB, 0.82522E+00_JPRB, 0.99323E+00_JPRB, 0.11950E+01_JPRB, 0.13112E+01_JPRB, 0.13356E+01_JPRB, 0.13204E+01_JPRB ,&
&  0.12038E+01_JPRB, 0.10890E+01_JPRB, 0.95501E+00_JPRB, 0.94239E+00_JPRB, 0.91707E+00_JPRB, 0.88446E+00_JPRB, 0.94271E+00_JPRB ,&
&  0.98020E+00_JPRB, 0.10308E+01_JPRB, 0.10745E+01_JPRB, 0.11384E+01_JPRB, 0.11933E+01_JPRB, 0.12364E+01_JPRB, 0.12776E+01_JPRB ,&
&  0.13150E+01_JPRB, 0.13631E+01_JPRB, 0.14052E+01_JPRB, 0.14424E+01_JPRB, 0.15273E+01_JPRB, 0.16484E+01_JPRB, 0.16857E+01_JPRB ,&
&  0.18605E+01_JPRB, 0.18665E+01_JPRB, 0.19357E+01_JPRB, 0.28137E+01_JPRB, 0.40825E+01_JPRB, 0.41327E+01_JPRB, 0.37730E+01_JPRB ,&
&  0.36873E+01_JPRB, 0.36271E+01_JPRB, 0.37290E+01_JPRB, 0.36616E+01_JPRB, 0.33760E+01_JPRB, 0.32746E+01_JPRB, 0.31321E+01_JPRB ,&
&  0.29973E+01_JPRB, 0.29126E+01_JPRB, 0.28312E+01_JPRB, 0.28894E+01_JPRB, 0.29230E+01_JPRB, 0.30249E+01_JPRB, 0.31418E+01_JPRB ,&
&  0.31401E+01_JPRB, 0.27934E+01_JPRB, 0.19802E+01_JPRB, 0.18800E+01_JPRB, 0.15072E+01_JPRB, 0.13503E+01_JPRB, 0.13523E+01_JPRB ,&
&  0.12100E+01_JPRB, 0.13244E+01_JPRB, 0.15023E+01_JPRB, 0.15254E+01_JPRB, 0.20845E+01_JPRB, 0.23913E+01_JPRB, 0.24036E+01_JPRB ,&
&  0.23563E+01_JPRB, 0.22769E+01_JPRB, 0.26893E+01_JPRB, 0.28411E+01_JPRB, 0.27996E+01_JPRB, 0.26014E+01_JPRB, 0.23213E+01_JPRB ,&
&  0.20236E+01_JPRB, 0.18393E+01_JPRB, 0.17458E+01_JPRB, 0.16757E+01_JPRB, 0.15885E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.17467E+01_JPRB, 0.16488E+01_JPRB, 0.15585E+01_JPRB, 0.14910E+01_JPRB, 0.14342E+01_JPRB, 0.13685E+01_JPRB, 0.12833E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.11603E+01_JPRB, 0.11423E+01_JPRB, 0.11235E+01_JPRB, 0.11139E+01_JPRB, 0.11175E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.10459E+01_JPRB, 0.86373E+00_JPRB, 0.73837E+00_JPRB, 0.65730E+00_JPRB, 0.63750E+00_JPRB, 0.67278E+00_JPRB ,&
&  0.75739E+00_JPRB, 0.10162E+01_JPRB, 0.13858E+01_JPRB, 0.16006E+01_JPRB, 0.17261E+01_JPRB, 0.17572E+01_JPRB, 0.16873E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.13866E+01_JPRB, 0.13050E+01_JPRB, 0.11671E+01_JPRB, 0.11193E+01_JPRB, 0.11169E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10855E+01_JPRB, 0.11095E+01_JPRB, 0.11580E+01_JPRB, 0.11905E+01_JPRB, 0.12384E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.14172E+01_JPRB, 0.14732E+01_JPRB, 0.14515E+01_JPRB, 0.14371E+01_JPRB, 0.16567E+01_JPRB, 0.19418E+01_JPRB, 0.21175E+01_JPRB ,&
&  0.21699E+01_JPRB, 0.23426E+01_JPRB, 0.34050E+01_JPRB, 0.40321E+01_JPRB, 0.43360E+01_JPRB, 0.47444E+01_JPRB, 0.46960E+01_JPRB ,&
&  0.42936E+01_JPRB, 0.39276E+01_JPRB, 0.37531E+01_JPRB, 0.33641E+01_JPRB, 0.38299E+01_JPRB, 0.39572E+01_JPRB, 0.39455E+01_JPRB ,&
&  0.39021E+01_JPRB, 0.37436E+01_JPRB, 0.36234E+01_JPRB, 0.36557E+01_JPRB, 0.37747E+01_JPRB, 0.39281E+01_JPRB, 0.37926E+01_JPRB ,&
&  0.33857E+01_JPRB, 0.35353E+01_JPRB, 0.30241E+01_JPRB, 0.24593E+01_JPRB, 0.20159E+01_JPRB, 0.18680E+01_JPRB, 0.17350E+01_JPRB ,&
&  0.15910E+01_JPRB, 0.15979E+01_JPRB, 0.18905E+01_JPRB, 0.22736E+01_JPRB, 0.28792E+01_JPRB, 0.33520E+01_JPRB, 0.33663E+01_JPRB ,&
&  0.31960E+01_JPRB, 0.35201E+01_JPRB, 0.33918E+01_JPRB, 0.31568E+01_JPRB, 0.29958E+01_JPRB, 0.27604E+01_JPRB, 0.25488E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.21736E+01_JPRB, 0.20692E+01_JPRB, 0.19707E+01_JPRB, 0.18591E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.18709E+01_JPRB, 0.17687E+01_JPRB, 0.16706E+01_JPRB, 0.15848E+01_JPRB, 0.14959E+01_JPRB, 0.14166E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.12727E+01_JPRB, 0.12404E+01_JPRB, 0.12056E+01_JPRB, 0.11781E+01_JPRB, 0.11643E+01_JPRB, 0.11502E+01_JPRB, 0.11480E+01_JPRB ,&
&  0.11394E+01_JPRB, 0.10096E+01_JPRB, 0.85186E+00_JPRB, 0.78530E+00_JPRB, 0.72090E+00_JPRB, 0.66562E+00_JPRB, 0.68796E+00_JPRB ,&
&  0.86155E+00_JPRB, 0.13542E+01_JPRB, 0.18055E+01_JPRB, 0.20581E+01_JPRB, 0.21462E+01_JPRB, 0.21120E+01_JPRB, 0.19930E+01_JPRB ,&
&  0.17930E+01_JPRB, 0.17867E+01_JPRB, 0.17052E+01_JPRB, 0.15766E+01_JPRB, 0.14330E+01_JPRB, 0.13420E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.12250E+01_JPRB, 0.12232E+01_JPRB, 0.12454E+01_JPRB, 0.12781E+01_JPRB, 0.13405E+01_JPRB, 0.14197E+01_JPRB, 0.15101E+01_JPRB ,&
&  0.15463E+01_JPRB, 0.15648E+01_JPRB, 0.15205E+01_JPRB, 0.15511E+01_JPRB, 0.16496E+01_JPRB, 0.22113E+01_JPRB, 0.25302E+01_JPRB ,&
&  0.31354E+01_JPRB, 0.36528E+01_JPRB, 0.37991E+01_JPRB, 0.44691E+01_JPRB, 0.38853E+01_JPRB, 0.44062E+01_JPRB, 0.49686E+01_JPRB ,&
&  0.44303E+01_JPRB, 0.39675E+01_JPRB, 0.38251E+01_JPRB, 0.32780E+01_JPRB, 0.31947E+01_JPRB, 0.46306E+01_JPRB, 0.49034E+01_JPRB ,&
&  0.49901E+01_JPRB, 0.52334E+01_JPRB, 0.49422E+01_JPRB, 0.46924E+01_JPRB, 0.44249E+01_JPRB, 0.40844E+01_JPRB, 0.31370E+01_JPRB ,&
&  0.31621E+01_JPRB, 0.29014E+01_JPRB, 0.29921E+01_JPRB, 0.26525E+01_JPRB, 0.24435E+01_JPRB, 0.22115E+01_JPRB, 0.19844E+01_JPRB ,&
&  0.18582E+01_JPRB, 0.18887E+01_JPRB, 0.27467E+01_JPRB, 0.39051E+01_JPRB, 0.46152E+01_JPRB, 0.41592E+01_JPRB, 0.47686E+01_JPRB ,&
&  0.46396E+01_JPRB, 0.39573E+01_JPRB, 0.31918E+01_JPRB, 0.29119E+01_JPRB, 0.26850E+01_JPRB, 0.24994E+01_JPRB, 0.23621E+01_JPRB ,&
&  0.22374E+01_JPRB, 0.21267E+01_JPRB, 0.20462E+01_JPRB, 0.20037E+01_JPRB, 0.19478E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.19015E+01_JPRB, 0.18071E+01_JPRB, 0.17041E+01_JPRB, 0.16175E+01_JPRB, 0.15330E+01_JPRB, 0.14549E+01_JPRB, 0.13925E+01_JPRB ,&
&  0.13520E+01_JPRB, 0.13047E+01_JPRB, 0.12381E+01_JPRB, 0.11937E+01_JPRB, 0.11717E+01_JPRB, 0.11723E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.10536E+01_JPRB, 0.89941E+00_JPRB, 0.83148E+00_JPRB, 0.82606E+00_JPRB, 0.77311E+00_JPRB, 0.77798E+00_JPRB ,&
&  0.11570E+01_JPRB, 0.19630E+01_JPRB, 0.23635E+01_JPRB, 0.24199E+01_JPRB, 0.24513E+01_JPRB, 0.23895E+01_JPRB, 0.22011E+01_JPRB ,&
&  0.21531E+01_JPRB, 0.20164E+01_JPRB, 0.18948E+01_JPRB, 0.17633E+01_JPRB, 0.16331E+01_JPRB, 0.15020E+01_JPRB, 0.13955E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.13298E+01_JPRB, 0.13465E+01_JPRB, 0.13469E+01_JPRB, 0.14229E+01_JPRB, 0.14882E+01_JPRB, 0.15336E+01_JPRB ,&
&  0.15590E+01_JPRB, 0.15905E+01_JPRB, 0.15859E+01_JPRB, 0.16323E+01_JPRB, 0.19528E+01_JPRB, 0.26442E+01_JPRB, 0.32370E+01_JPRB ,&
&  0.36167E+01_JPRB, 0.39495E+01_JPRB, 0.43110E+01_JPRB, 0.45389E+01_JPRB, 0.51819E+01_JPRB, 0.50914E+01_JPRB, 0.52158E+01_JPRB ,&
&  0.40614E+01_JPRB, 0.33382E+01_JPRB, 0.27872E+01_JPRB, 0.23299E+01_JPRB, 0.36279E+01_JPRB, 0.59745E+01_JPRB, 0.67333E+01_JPRB ,&
&  0.67430E+01_JPRB, 0.69887E+01_JPRB, 0.67051E+01_JPRB, 0.56337E+01_JPRB, 0.47493E+01_JPRB, 0.38569E+01_JPRB, 0.43506E+01_JPRB ,&
&  0.42106E+01_JPRB, 0.40871E+01_JPRB, 0.36003E+01_JPRB, 0.28954E+01_JPRB, 0.24065E+01_JPRB, 0.25364E+01_JPRB, 0.23151E+01_JPRB ,&
&  0.21719E+01_JPRB, 0.24686E+01_JPRB, 0.46163E+01_JPRB, 0.58070E+01_JPRB, 0.57322E+01_JPRB, 0.52714E+01_JPRB, 0.58998E+01_JPRB ,&
&  0.46923E+01_JPRB, 0.37259E+01_JPRB, 0.30775E+01_JPRB, 0.26309E+01_JPRB, 0.23928E+01_JPRB, 0.22566E+01_JPRB, 0.21943E+01_JPRB ,&
&  0.21568E+01_JPRB, 0.21313E+01_JPRB, 0.20722E+01_JPRB, 0.20081E+01_JPRB, 0.19546E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.19452E+01_JPRB, 0.18507E+01_JPRB, 0.17643E+01_JPRB, 0.16718E+01_JPRB, 0.15789E+01_JPRB, 0.15110E+01_JPRB, 0.14652E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.13438E+01_JPRB, 0.12879E+01_JPRB, 0.12484E+01_JPRB, 0.12298E+01_JPRB, 0.12186E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.11826E+01_JPRB, 0.11510E+01_JPRB, 0.99066E+00_JPRB, 0.92003E+00_JPRB, 0.91290E+00_JPRB, 0.90880E+00_JPRB, 0.10049E+01_JPRB ,&
&  0.17920E+01_JPRB, 0.27349E+01_JPRB, 0.27971E+01_JPRB, 0.25955E+01_JPRB, 0.25858E+01_JPRB, 0.21790E+01_JPRB, 0.20451E+01_JPRB ,&
&  0.21329E+01_JPRB, 0.20337E+01_JPRB, 0.19373E+01_JPRB, 0.18294E+01_JPRB, 0.17264E+01_JPRB, 0.16355E+01_JPRB, 0.15559E+01_JPRB ,&
&  0.14954E+01_JPRB, 0.14614E+01_JPRB, 0.14771E+01_JPRB, 0.15482E+01_JPRB, 0.15885E+01_JPRB, 0.16057E+01_JPRB, 0.16136E+01_JPRB ,&
&  0.16063E+01_JPRB, 0.15976E+01_JPRB, 0.16123E+01_JPRB, 0.18082E+01_JPRB, 0.22586E+01_JPRB, 0.30911E+01_JPRB, 0.44463E+01_JPRB ,&
&  0.44860E+01_JPRB, 0.49276E+01_JPRB, 0.48138E+01_JPRB, 0.56124E+01_JPRB, 0.60532E+01_JPRB, 0.56847E+01_JPRB, 0.53299E+01_JPRB ,&
&  0.39591E+01_JPRB, 0.32623E+01_JPRB, 0.30399E+01_JPRB, 0.33867E+01_JPRB, 0.45003E+01_JPRB, 0.58786E+01_JPRB, 0.75052E+01_JPRB ,&
&  0.66409E+01_JPRB, 0.65959E+01_JPRB, 0.65515E+01_JPRB, 0.60247E+01_JPRB, 0.37096E+01_JPRB, 0.29121E+01_JPRB, 0.42204E+01_JPRB ,&
&  0.42452E+01_JPRB, 0.25573E+01_JPRB, 0.18074E+01_JPRB, 0.21228E+01_JPRB, 0.16101E+01_JPRB, 0.18553E+01_JPRB, 0.25384E+01_JPRB ,&
&  0.28074E+01_JPRB, 0.35809E+01_JPRB, 0.58053E+01_JPRB, 0.56459E+01_JPRB, 0.57348E+01_JPRB, 0.67085E+01_JPRB, 0.54929E+01_JPRB ,&
&  0.42799E+01_JPRB, 0.32590E+01_JPRB, 0.30140E+01_JPRB, 0.26190E+01_JPRB, 0.24076E+01_JPRB, 0.22825E+01_JPRB, 0.22656E+01_JPRB ,&
&  0.23182E+01_JPRB, 0.23194E+01_JPRB, 0.22517E+01_JPRB, 0.21389E+01_JPRB, 0.20319E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.20185E+01_JPRB, 0.19310E+01_JPRB, 0.18425E+01_JPRB, 0.17570E+01_JPRB, 0.16776E+01_JPRB, 0.16102E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.14155E+01_JPRB, 0.13512E+01_JPRB, 0.13260E+01_JPRB, 0.12975E+01_JPRB, 0.12798E+01_JPRB, 0.12779E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12200E+01_JPRB, 0.11824E+01_JPRB, 0.11235E+01_JPRB, 0.10559E+01_JPRB, 0.10291E+01_JPRB, 0.10122E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.26877E+01_JPRB, 0.35455E+01_JPRB, 0.31952E+01_JPRB, 0.26824E+01_JPRB, 0.22967E+01_JPRB, 0.19906E+01_JPRB, 0.21115E+01_JPRB ,&
&  0.21229E+01_JPRB, 0.20676E+01_JPRB, 0.19600E+01_JPRB, 0.18660E+01_JPRB, 0.17817E+01_JPRB, 0.17025E+01_JPRB, 0.16537E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.15690E+01_JPRB, 0.16073E+01_JPRB, 0.16125E+01_JPRB, 0.16196E+01_JPRB, 0.16085E+01_JPRB, 0.16461E+01_JPRB ,&
&  0.16401E+01_JPRB, 0.16544E+01_JPRB, 0.17352E+01_JPRB, 0.20216E+01_JPRB, 0.28136E+01_JPRB, 0.39728E+01_JPRB, 0.43576E+01_JPRB ,&
&  0.44461E+01_JPRB, 0.47585E+01_JPRB, 0.56550E+01_JPRB, 0.58414E+01_JPRB, 0.60204E+01_JPRB, 0.58691E+01_JPRB, 0.50631E+01_JPRB ,&
&  0.44797E+01_JPRB, 0.40635E+01_JPRB, 0.35433E+01_JPRB, 0.39634E+01_JPRB, 0.55021E+01_JPRB, 0.58421E+01_JPRB, 0.69215E+01_JPRB ,&
&  0.69023E+01_JPRB, 0.62092E+01_JPRB, 0.49000E+01_JPRB, 0.38892E+01_JPRB, 0.42392E+01_JPRB, 0.31976E+01_JPRB, 0.20625E+01_JPRB ,&
&  0.18133E+01_JPRB, 0.15993E+01_JPRB, 0.13997E+01_JPRB, 0.13941E+01_JPRB, 0.13324E+01_JPRB, 0.17282E+01_JPRB, 0.26576E+01_JPRB ,&
&  0.37524E+01_JPRB, 0.46679E+01_JPRB, 0.54706E+01_JPRB, 0.51490E+01_JPRB, 0.49552E+01_JPRB, 0.59953E+01_JPRB, 0.53730E+01_JPRB ,&
&  0.34599E+01_JPRB, 0.28434E+01_JPRB, 0.27840E+01_JPRB, 0.25565E+01_JPRB, 0.23935E+01_JPRB, 0.23590E+01_JPRB, 0.24471E+01_JPRB ,&
&  0.25441E+01_JPRB, 0.25307E+01_JPRB, 0.24276E+01_JPRB, 0.22602E+01_JPRB, 0.21167E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.20235E+01_JPRB, 0.19102E+01_JPRB, 0.18348E+01_JPRB, 0.17893E+01_JPRB, 0.17403E+01_JPRB, 0.16483E+01_JPRB, 0.14981E+01_JPRB ,&
&  0.13871E+01_JPRB, 0.13422E+01_JPRB, 0.13338E+01_JPRB, 0.13159E+01_JPRB, 0.12969E+01_JPRB, 0.12866E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.11920E+01_JPRB, 0.11968E+01_JPRB, 0.11899E+01_JPRB, 0.12255E+01_JPRB, 0.12592E+01_JPRB, 0.19520E+01_JPRB ,&
&  0.36925E+01_JPRB, 0.42251E+01_JPRB, 0.33999E+01_JPRB, 0.25910E+01_JPRB, 0.22736E+01_JPRB, 0.21844E+01_JPRB, 0.22131E+01_JPRB ,&
&  0.20976E+01_JPRB, 0.19697E+01_JPRB, 0.17963E+01_JPRB, 0.17112E+01_JPRB, 0.16930E+01_JPRB, 0.16702E+01_JPRB, 0.16549E+01_JPRB ,&
&  0.16143E+01_JPRB, 0.16098E+01_JPRB, 0.15953E+01_JPRB, 0.15884E+01_JPRB, 0.16052E+01_JPRB, 0.16422E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.17566E+01_JPRB, 0.18161E+01_JPRB, 0.20010E+01_JPRB, 0.25426E+01_JPRB, 0.33729E+01_JPRB, 0.37822E+01_JPRB, 0.44573E+01_JPRB ,&
&  0.51586E+01_JPRB, 0.56957E+01_JPRB, 0.54209E+01_JPRB, 0.62718E+01_JPRB, 0.60696E+01_JPRB, 0.54364E+01_JPRB, 0.49137E+01_JPRB ,&
&  0.45480E+01_JPRB, 0.50035E+01_JPRB, 0.41144E+01_JPRB, 0.38124E+01_JPRB, 0.59849E+01_JPRB, 0.65406E+01_JPRB, 0.69881E+01_JPRB ,&
&  0.74665E+01_JPRB, 0.64099E+01_JPRB, 0.52503E+01_JPRB, 0.48632E+01_JPRB, 0.68594E+01_JPRB, 0.66435E+01_JPRB, 0.26302E+01_JPRB ,&
&  0.18912E+01_JPRB, 0.16510E+01_JPRB, 0.13960E+01_JPRB, 0.12794E+01_JPRB, 0.13070E+01_JPRB, 0.17630E+01_JPRB, 0.32955E+01_JPRB ,&
&  0.37541E+01_JPRB, 0.51157E+01_JPRB, 0.52598E+01_JPRB, 0.50625E+01_JPRB, 0.45408E+01_JPRB, 0.43976E+01_JPRB, 0.38394E+01_JPRB ,&
&  0.34115E+01_JPRB, 0.29660E+01_JPRB, 0.25389E+01_JPRB, 0.22863E+01_JPRB, 0.21926E+01_JPRB, 0.22597E+01_JPRB, 0.24360E+01_JPRB ,&
&  0.25293E+01_JPRB, 0.25336E+01_JPRB, 0.24451E+01_JPRB, 0.22960E+01_JPRB, 0.21413E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.19503E+01_JPRB, 0.18987E+01_JPRB, 0.18672E+01_JPRB, 0.18032E+01_JPRB, 0.17223E+01_JPRB, 0.16115E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.13998E+01_JPRB, 0.13666E+01_JPRB, 0.13603E+01_JPRB, 0.13298E+01_JPRB, 0.12908E+01_JPRB, 0.12654E+01_JPRB, 0.12509E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.11848E+01_JPRB, 0.11846E+01_JPRB, 0.13110E+01_JPRB, 0.14910E+01_JPRB, 0.18567E+01_JPRB, 0.27379E+01_JPRB ,&
&  0.45291E+01_JPRB, 0.46779E+01_JPRB, 0.36217E+01_JPRB, 0.29020E+01_JPRB, 0.25481E+01_JPRB, 0.24580E+01_JPRB, 0.22201E+01_JPRB ,&
&  0.19908E+01_JPRB, 0.18505E+01_JPRB, 0.16889E+01_JPRB, 0.16830E+01_JPRB, 0.16898E+01_JPRB, 0.17018E+01_JPRB, 0.16938E+01_JPRB ,&
&  0.17018E+01_JPRB, 0.16383E+01_JPRB, 0.16301E+01_JPRB, 0.16315E+01_JPRB, 0.16670E+01_JPRB, 0.17524E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.19395E+01_JPRB, 0.20762E+01_JPRB, 0.24180E+01_JPRB, 0.30153E+01_JPRB, 0.32321E+01_JPRB, 0.40836E+01_JPRB, 0.45642E+01_JPRB ,&
&  0.49897E+01_JPRB, 0.51232E+01_JPRB, 0.49397E+01_JPRB, 0.55406E+01_JPRB, 0.58547E+01_JPRB, 0.53443E+01_JPRB, 0.48956E+01_JPRB ,&
&  0.51885E+01_JPRB, 0.45803E+01_JPRB, 0.41315E+01_JPRB, 0.37976E+01_JPRB, 0.49834E+01_JPRB, 0.82470E+01_JPRB, 0.65074E+01_JPRB ,&
&  0.63229E+01_JPRB, 0.71852E+01_JPRB, 0.63104E+01_JPRB, 0.62005E+01_JPRB, 0.72717E+01_JPRB, 0.88246E+01_JPRB, 0.68359E+01_JPRB ,&
&  0.24620E+01_JPRB, 0.16666E+01_JPRB, 0.16804E+01_JPRB, 0.16361E+01_JPRB, 0.14373E+01_JPRB, 0.18988E+01_JPRB, 0.54983E+01_JPRB ,&
&  0.45735E+01_JPRB, 0.58922E+01_JPRB, 0.56786E+01_JPRB, 0.51144E+01_JPRB, 0.46737E+01_JPRB, 0.39378E+01_JPRB, 0.34794E+01_JPRB ,&
&  0.28453E+01_JPRB, 0.24420E+01_JPRB, 0.22359E+01_JPRB, 0.21279E+01_JPRB, 0.21182E+01_JPRB, 0.21602E+01_JPRB, 0.22846E+01_JPRB ,&
&  0.23681E+01_JPRB, 0.23843E+01_JPRB, 0.23022E+01_JPRB, 0.21731E+01_JPRB, 0.20466E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.18954E+01_JPRB, 0.18715E+01_JPRB, 0.18282E+01_JPRB, 0.17421E+01_JPRB, 0.16565E+01_JPRB, 0.15376E+01_JPRB, 0.14434E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.13938E+01_JPRB, 0.13683E+01_JPRB, 0.13325E+01_JPRB, 0.12880E+01_JPRB, 0.12522E+01_JPRB, 0.12355E+01_JPRB ,&
&  0.12225E+01_JPRB, 0.11867E+01_JPRB, 0.11803E+01_JPRB, 0.13259E+01_JPRB, 0.18071E+01_JPRB, 0.24078E+01_JPRB, 0.30484E+01_JPRB ,&
&  0.52246E+01_JPRB, 0.51437E+01_JPRB, 0.34365E+01_JPRB, 0.29455E+01_JPRB, 0.27740E+01_JPRB, 0.26620E+01_JPRB, 0.22703E+01_JPRB ,&
&  0.20110E+01_JPRB, 0.18716E+01_JPRB, 0.17193E+01_JPRB, 0.16780E+01_JPRB, 0.16915E+01_JPRB, 0.17459E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.17156E+01_JPRB, 0.17126E+01_JPRB, 0.17605E+01_JPRB, 0.17770E+01_JPRB, 0.18196E+01_JPRB, 0.19329E+01_JPRB, 0.20445E+01_JPRB ,&
&  0.21957E+01_JPRB, 0.24326E+01_JPRB, 0.28477E+01_JPRB, 0.31904E+01_JPRB, 0.36655E+01_JPRB, 0.40442E+01_JPRB, 0.45337E+01_JPRB ,&
&  0.45684E+01_JPRB, 0.46972E+01_JPRB, 0.48634E+01_JPRB, 0.50556E+01_JPRB, 0.51926E+01_JPRB, 0.51259E+01_JPRB, 0.50358E+01_JPRB ,&
&  0.52320E+01_JPRB, 0.45322E+01_JPRB, 0.39766E+01_JPRB, 0.38390E+01_JPRB, 0.47007E+01_JPRB, 0.74053E+01_JPRB, 0.82848E+01_JPRB ,&
&  0.84326E+01_JPRB, 0.62125E+01_JPRB, 0.59486E+01_JPRB, 0.72397E+01_JPRB, 0.71574E+01_JPRB, 0.79239E+01_JPRB, 0.10731E+02_JPRB ,&
&  0.94598E+01_JPRB, 0.46859E+01_JPRB, 0.36844E+01_JPRB, 0.47551E+01_JPRB, 0.27763E+01_JPRB, 0.21205E+01_JPRB, 0.35574E+01_JPRB ,&
&  0.45083E+01_JPRB, 0.53887E+01_JPRB, 0.53164E+01_JPRB, 0.49814E+01_JPRB, 0.45693E+01_JPRB, 0.35402E+01_JPRB, 0.28689E+01_JPRB ,&
&  0.23142E+01_JPRB, 0.21148E+01_JPRB, 0.20189E+01_JPRB, 0.19158E+01_JPRB, 0.18791E+01_JPRB, 0.19466E+01_JPRB, 0.20491E+01_JPRB ,&
&  0.21357E+01_JPRB, 0.21610E+01_JPRB, 0.21345E+01_JPRB, 0.20577E+01_JPRB, 0.19586E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.18202E+01_JPRB, 0.18077E+01_JPRB, 0.17828E+01_JPRB, 0.16893E+01_JPRB, 0.15917E+01_JPRB, 0.14956E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.14286E+01_JPRB, 0.13969E+01_JPRB, 0.13614E+01_JPRB, 0.13131E+01_JPRB, 0.12893E+01_JPRB, 0.12586E+01_JPRB, 0.12608E+01_JPRB ,&
&  0.12736E+01_JPRB, 0.12535E+01_JPRB, 0.13175E+01_JPRB, 0.16952E+01_JPRB, 0.24715E+01_JPRB, 0.41039E+01_JPRB, 0.35492E+01_JPRB ,&
&  0.52686E+01_JPRB, 0.55878E+01_JPRB, 0.35132E+01_JPRB, 0.31981E+01_JPRB, 0.30743E+01_JPRB, 0.30543E+01_JPRB, 0.25312E+01_JPRB ,&
&  0.21622E+01_JPRB, 0.19968E+01_JPRB, 0.18810E+01_JPRB, 0.17700E+01_JPRB, 0.17650E+01_JPRB, 0.18042E+01_JPRB, 0.17631E+01_JPRB ,&
&  0.17878E+01_JPRB, 0.18699E+01_JPRB, 0.19104E+01_JPRB, 0.19342E+01_JPRB, 0.20199E+01_JPRB, 0.21410E+01_JPRB, 0.22907E+01_JPRB ,&
&  0.25260E+01_JPRB, 0.28194E+01_JPRB, 0.32473E+01_JPRB, 0.34106E+01_JPRB, 0.37021E+01_JPRB, 0.40924E+01_JPRB, 0.42940E+01_JPRB ,&
&  0.41422E+01_JPRB, 0.39633E+01_JPRB, 0.43770E+01_JPRB, 0.47341E+01_JPRB, 0.48070E+01_JPRB, 0.48196E+01_JPRB, 0.50111E+01_JPRB ,&
&  0.54510E+01_JPRB, 0.52800E+01_JPRB, 0.53914E+01_JPRB, 0.47613E+01_JPRB, 0.56525E+01_JPRB, 0.75267E+01_JPRB, 0.77372E+01_JPRB ,&
&  0.94263E+01_JPRB, 0.83838E+01_JPRB, 0.81881E+01_JPRB, 0.77567E+01_JPRB, 0.71151E+01_JPRB, 0.73616E+01_JPRB, 0.82260E+01_JPRB ,&
&  0.87442E+01_JPRB, 0.96174E+01_JPRB, 0.90143E+01_JPRB, 0.54283E+01_JPRB, 0.33861E+01_JPRB, 0.25853E+01_JPRB, 0.30009E+01_JPRB ,&
&  0.40917E+01_JPRB, 0.43197E+01_JPRB, 0.42510E+01_JPRB, 0.60366E+01_JPRB, 0.38916E+01_JPRB, 0.28177E+01_JPRB, 0.22087E+01_JPRB ,&
&  0.19056E+01_JPRB, 0.18203E+01_JPRB, 0.17567E+01_JPRB, 0.16525E+01_JPRB, 0.16330E+01_JPRB, 0.16885E+01_JPRB, 0.17796E+01_JPRB ,&
&  0.18811E+01_JPRB, 0.19488E+01_JPRB, 0.19841E+01_JPRB, 0.19384E+01_JPRB, 0.18666E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.17063E+01_JPRB, 0.17068E+01_JPRB, 0.16563E+01_JPRB, 0.15837E+01_JPRB, 0.15256E+01_JPRB, 0.14766E+01_JPRB, 0.14703E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.13526E+01_JPRB, 0.13222E+01_JPRB, 0.12963E+01_JPRB, 0.13132E+01_JPRB, 0.13387E+01_JPRB, 0.13896E+01_JPRB ,&
&  0.14616E+01_JPRB, 0.15808E+01_JPRB, 0.19053E+01_JPRB, 0.26759E+01_JPRB, 0.37802E+01_JPRB, 0.53365E+01_JPRB, 0.55065E+01_JPRB ,&
&  0.51874E+01_JPRB, 0.61254E+01_JPRB, 0.35109E+01_JPRB, 0.35181E+01_JPRB, 0.35852E+01_JPRB, 0.40433E+01_JPRB, 0.33736E+01_JPRB ,&
&  0.25532E+01_JPRB, 0.23019E+01_JPRB, 0.21554E+01_JPRB, 0.19344E+01_JPRB, 0.18652E+01_JPRB, 0.18437E+01_JPRB, 0.18649E+01_JPRB ,&
&  0.19610E+01_JPRB, 0.20619E+01_JPRB, 0.21238E+01_JPRB, 0.21951E+01_JPRB, 0.22991E+01_JPRB, 0.24205E+01_JPRB, 0.26354E+01_JPRB ,&
&  0.29363E+01_JPRB, 0.32644E+01_JPRB, 0.34428E+01_JPRB, 0.35690E+01_JPRB, 0.36805E+01_JPRB, 0.39589E+01_JPRB, 0.41055E+01_JPRB ,&
&  0.40339E+01_JPRB, 0.38738E+01_JPRB, 0.41585E+01_JPRB, 0.42613E+01_JPRB, 0.43324E+01_JPRB, 0.48160E+01_JPRB, 0.49686E+01_JPRB ,&
&  0.56111E+01_JPRB, 0.53589E+01_JPRB, 0.62316E+01_JPRB, 0.58912E+01_JPRB, 0.62108E+01_JPRB, 0.69986E+01_JPRB, 0.63149E+01_JPRB ,&
&  0.64575E+01_JPRB, 0.73165E+01_JPRB, 0.75646E+01_JPRB, 0.75835E+01_JPRB, 0.74061E+01_JPRB, 0.72823E+01_JPRB, 0.69790E+01_JPRB ,&
&  0.61492E+01_JPRB, 0.60075E+01_JPRB, 0.52937E+01_JPRB, 0.34692E+01_JPRB, 0.26950E+01_JPRB, 0.24497E+01_JPRB, 0.27486E+01_JPRB ,&
&  0.35102E+01_JPRB, 0.37374E+01_JPRB, 0.42433E+01_JPRB, 0.41508E+01_JPRB, 0.26611E+01_JPRB, 0.19844E+01_JPRB, 0.17632E+01_JPRB ,&
&  0.16706E+01_JPRB, 0.16029E+01_JPRB, 0.15061E+01_JPRB, 0.14195E+01_JPRB, 0.14253E+01_JPRB, 0.14973E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.16825E+01_JPRB, 0.17465E+01_JPRB, 0.17490E+01_JPRB, 0.17176E+01_JPRB, 0.17103E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.16821E+01_JPRB, 0.16818E+01_JPRB, 0.16729E+01_JPRB, 0.16894E+01_JPRB, 0.17184E+01_JPRB, 0.17645E+01_JPRB, 0.17423E+01_JPRB ,&
&  0.15704E+01_JPRB, 0.13101E+01_JPRB, 0.13305E+01_JPRB, 0.13864E+01_JPRB, 0.15040E+01_JPRB, 0.16216E+01_JPRB, 0.17839E+01_JPRB ,&
&  0.20371E+01_JPRB, 0.23918E+01_JPRB, 0.30096E+01_JPRB, 0.39466E+01_JPRB, 0.51264E+01_JPRB, 0.65611E+01_JPRB, 0.69805E+01_JPRB ,&
&  0.66997E+01_JPRB, 0.45887E+01_JPRB, 0.37897E+01_JPRB, 0.37023E+01_JPRB, 0.35157E+01_JPRB, 0.39598E+01_JPRB, 0.37607E+01_JPRB ,&
&  0.29982E+01_JPRB, 0.26303E+01_JPRB, 0.24737E+01_JPRB, 0.22621E+01_JPRB, 0.20937E+01_JPRB, 0.21472E+01_JPRB, 0.22368E+01_JPRB ,&
&  0.23636E+01_JPRB, 0.24441E+01_JPRB, 0.24715E+01_JPRB, 0.25115E+01_JPRB, 0.26120E+01_JPRB, 0.27561E+01_JPRB, 0.30272E+01_JPRB ,&
&  0.33242E+01_JPRB, 0.35602E+01_JPRB, 0.36660E+01_JPRB, 0.36024E+01_JPRB, 0.37044E+01_JPRB, 0.38562E+01_JPRB, 0.39228E+01_JPRB ,&
&  0.39321E+01_JPRB, 0.40419E+01_JPRB, 0.41696E+01_JPRB, 0.43188E+01_JPRB, 0.42333E+01_JPRB, 0.48242E+01_JPRB, 0.49359E+01_JPRB ,&
&  0.52928E+01_JPRB, 0.52145E+01_JPRB, 0.60927E+01_JPRB, 0.55341E+01_JPRB, 0.61871E+01_JPRB, 0.61992E+01_JPRB, 0.58436E+01_JPRB ,&
&  0.60069E+01_JPRB, 0.69894E+01_JPRB, 0.73464E+01_JPRB, 0.73272E+01_JPRB, 0.72833E+01_JPRB, 0.64299E+01_JPRB, 0.67494E+01_JPRB ,&
&  0.52469E+01_JPRB, 0.52335E+01_JPRB, 0.34668E+01_JPRB, 0.25739E+01_JPRB, 0.21025E+01_JPRB, 0.24587E+01_JPRB, 0.28922E+01_JPRB ,&
&  0.34245E+01_JPRB, 0.33128E+01_JPRB, 0.28615E+01_JPRB, 0.26972E+01_JPRB, 0.20868E+01_JPRB, 0.16980E+01_JPRB, 0.15993E+01_JPRB ,&
&  0.15574E+01_JPRB, 0.14654E+01_JPRB, 0.13826E+01_JPRB, 0.13633E+01_JPRB, 0.13820E+01_JPRB, 0.14757E+01_JPRB, 0.15401E+01_JPRB ,&
&  0.16149E+01_JPRB, 0.16379E+01_JPRB, 0.16386E+01_JPRB, 0.16493E+01_JPRB, 0.16606E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.16619E+01_JPRB, 0.16294E+01_JPRB, 0.16159E+01_JPRB, 0.16005E+01_JPRB, 0.15767E+01_JPRB, 0.15030E+01_JPRB, 0.14486E+01_JPRB ,&
&  0.13922E+01_JPRB, 0.13616E+01_JPRB, 0.14864E+01_JPRB, 0.16611E+01_JPRB, 0.18822E+01_JPRB, 0.21258E+01_JPRB, 0.24171E+01_JPRB ,&
&  0.27959E+01_JPRB, 0.32828E+01_JPRB, 0.38992E+01_JPRB, 0.44657E+01_JPRB, 0.50670E+01_JPRB, 0.58522E+01_JPRB, 0.63884E+01_JPRB ,&
&  0.56748E+01_JPRB, 0.34876E+01_JPRB, 0.37017E+01_JPRB, 0.37782E+01_JPRB, 0.36720E+01_JPRB, 0.36882E+01_JPRB, 0.34168E+01_JPRB ,&
&  0.30094E+01_JPRB, 0.28466E+01_JPRB, 0.27778E+01_JPRB, 0.26779E+01_JPRB, 0.24993E+01_JPRB, 0.25765E+01_JPRB, 0.26495E+01_JPRB ,&
&  0.26807E+01_JPRB, 0.26593E+01_JPRB, 0.26533E+01_JPRB, 0.27686E+01_JPRB, 0.29913E+01_JPRB, 0.31446E+01_JPRB, 0.33553E+01_JPRB ,&
&  0.35354E+01_JPRB, 0.37574E+01_JPRB, 0.38669E+01_JPRB, 0.39232E+01_JPRB, 0.40764E+01_JPRB, 0.41819E+01_JPRB, 0.41830E+01_JPRB ,&
&  0.41343E+01_JPRB, 0.41163E+01_JPRB, 0.42882E+01_JPRB, 0.47857E+01_JPRB, 0.50870E+01_JPRB, 0.46771E+01_JPRB, 0.49547E+01_JPRB ,&
&  0.51778E+01_JPRB, 0.54448E+01_JPRB, 0.53785E+01_JPRB, 0.70093E+01_JPRB, 0.54049E+01_JPRB, 0.55102E+01_JPRB, 0.49903E+01_JPRB ,&
&  0.58033E+01_JPRB, 0.63243E+01_JPRB, 0.68347E+01_JPRB, 0.69603E+01_JPRB, 0.67034E+01_JPRB, 0.60617E+01_JPRB, 0.64535E+01_JPRB ,&
&  0.55622E+01_JPRB, 0.46638E+01_JPRB, 0.32181E+01_JPRB, 0.23145E+01_JPRB, 0.23147E+01_JPRB, 0.39250E+01_JPRB, 0.38633E+01_JPRB ,&
&  0.33402E+01_JPRB, 0.26029E+01_JPRB, 0.24572E+01_JPRB, 0.28127E+01_JPRB, 0.20665E+01_JPRB, 0.15853E+01_JPRB, 0.14546E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.14307E+01_JPRB, 0.13737E+01_JPRB, 0.13756E+01_JPRB, 0.14374E+01_JPRB, 0.15304E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.16232E+01_JPRB, 0.16080E+01_JPRB, 0.16152E+01_JPRB, 0.16305E+01_JPRB, 0.16719E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.15576E+01_JPRB, 0.14975E+01_JPRB, 0.14486E+01_JPRB, 0.14010E+01_JPRB, 0.13552E+01_JPRB, 0.13318E+01_JPRB, 0.13496E+01_JPRB ,&
&  0.13919E+01_JPRB, 0.14936E+01_JPRB, 0.16432E+01_JPRB, 0.18284E+01_JPRB, 0.20671E+01_JPRB, 0.23602E+01_JPRB, 0.26844E+01_JPRB ,&
&  0.31004E+01_JPRB, 0.34940E+01_JPRB, 0.37074E+01_JPRB, 0.38570E+01_JPRB, 0.40410E+01_JPRB, 0.43111E+01_JPRB, 0.45467E+01_JPRB ,&
&  0.47473E+01_JPRB, 0.45399E+01_JPRB, 0.39829E+01_JPRB, 0.29616E+01_JPRB, 0.27289E+01_JPRB, 0.31466E+01_JPRB, 0.30817E+01_JPRB ,&
&  0.30077E+01_JPRB, 0.28952E+01_JPRB, 0.28099E+01_JPRB, 0.27385E+01_JPRB, 0.27309E+01_JPRB, 0.27836E+01_JPRB, 0.28216E+01_JPRB ,&
&  0.28169E+01_JPRB, 0.28347E+01_JPRB, 0.29045E+01_JPRB, 0.30608E+01_JPRB, 0.32935E+01_JPRB, 0.34513E+01_JPRB, 0.36108E+01_JPRB ,&
&  0.37443E+01_JPRB, 0.39763E+01_JPRB, 0.41079E+01_JPRB, 0.42802E+01_JPRB, 0.43642E+01_JPRB, 0.45226E+01_JPRB, 0.47533E+01_JPRB ,&
&  0.48236E+01_JPRB, 0.49418E+01_JPRB, 0.51840E+01_JPRB, 0.55855E+01_JPRB, 0.57816E+01_JPRB, 0.51462E+01_JPRB, 0.50447E+01_JPRB ,&
&  0.54791E+01_JPRB, 0.59814E+01_JPRB, 0.56578E+01_JPRB, 0.73088E+01_JPRB, 0.47342E+01_JPRB, 0.51762E+01_JPRB, 0.51929E+01_JPRB ,&
&  0.51144E+01_JPRB, 0.54211E+01_JPRB, 0.58224E+01_JPRB, 0.60984E+01_JPRB, 0.57243E+01_JPRB, 0.52043E+01_JPRB, 0.61860E+01_JPRB ,&
&  0.56430E+01_JPRB, 0.40840E+01_JPRB, 0.29331E+01_JPRB, 0.20146E+01_JPRB, 0.25687E+01_JPRB, 0.45435E+01_JPRB, 0.33807E+01_JPRB ,&
&  0.22083E+01_JPRB, 0.22600E+01_JPRB, 0.24841E+01_JPRB, 0.27558E+01_JPRB, 0.20823E+01_JPRB, 0.14397E+01_JPRB, 0.14022E+01_JPRB ,&
&  0.14348E+01_JPRB, 0.14408E+01_JPRB, 0.14676E+01_JPRB, 0.14957E+01_JPRB, 0.15413E+01_JPRB, 0.16006E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.16512E+01_JPRB, 0.16395E+01_JPRB, 0.16656E+01_JPRB, 0.16583E+01_JPRB, 0.16184E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.13126E+01_JPRB, 0.12431E+01_JPRB, 0.11900E+01_JPRB, 0.11735E+01_JPRB, 0.11842E+01_JPRB, 0.12150E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.14954E+01_JPRB, 0.16214E+01_JPRB, 0.17536E+01_JPRB, 0.19213E+01_JPRB, 0.21016E+01_JPRB, 0.23142E+01_JPRB ,&
&  0.25461E+01_JPRB, 0.27858E+01_JPRB, 0.28841E+01_JPRB, 0.28757E+01_JPRB, 0.29006E+01_JPRB, 0.29399E+01_JPRB, 0.31567E+01_JPRB ,&
&  0.36266E+01_JPRB, 0.40275E+01_JPRB, 0.42259E+01_JPRB, 0.40093E+01_JPRB, 0.32305E+01_JPRB, 0.33013E+01_JPRB, 0.35923E+01_JPRB ,&
&  0.36350E+01_JPRB, 0.32337E+01_JPRB, 0.29121E+01_JPRB, 0.28747E+01_JPRB, 0.28769E+01_JPRB, 0.29251E+01_JPRB, 0.29595E+01_JPRB ,&
&  0.29645E+01_JPRB, 0.30039E+01_JPRB, 0.31073E+01_JPRB, 0.32307E+01_JPRB, 0.34094E+01_JPRB, 0.35982E+01_JPRB, 0.37980E+01_JPRB ,&
&  0.39166E+01_JPRB, 0.40523E+01_JPRB, 0.40534E+01_JPRB, 0.39728E+01_JPRB, 0.41029E+01_JPRB, 0.45518E+01_JPRB, 0.51769E+01_JPRB ,&
&  0.55740E+01_JPRB, 0.55976E+01_JPRB, 0.60331E+01_JPRB, 0.65242E+01_JPRB, 0.64741E+01_JPRB, 0.59892E+01_JPRB, 0.59328E+01_JPRB ,&
&  0.62061E+01_JPRB, 0.69437E+01_JPRB, 0.51967E+01_JPRB, 0.61598E+01_JPRB, 0.63356E+01_JPRB, 0.55110E+01_JPRB, 0.47261E+01_JPRB ,&
&  0.42852E+01_JPRB, 0.39176E+01_JPRB, 0.33808E+01_JPRB, 0.36052E+01_JPRB, 0.38441E+01_JPRB, 0.42292E+01_JPRB, 0.50952E+01_JPRB ,&
&  0.48391E+01_JPRB, 0.34982E+01_JPRB, 0.23925E+01_JPRB, 0.18520E+01_JPRB, 0.22417E+01_JPRB, 0.31827E+01_JPRB, 0.23219E+01_JPRB ,&
&  0.19000E+01_JPRB, 0.20996E+01_JPRB, 0.21981E+01_JPRB, 0.25218E+01_JPRB, 0.23099E+01_JPRB, 0.13401E+01_JPRB, 0.13492E+01_JPRB ,&
&  0.13935E+01_JPRB, 0.14484E+01_JPRB, 0.14687E+01_JPRB, 0.14889E+01_JPRB, 0.15026E+01_JPRB, 0.15381E+01_JPRB, 0.15658E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.15568E+01_JPRB, 0.15233E+01_JPRB, 0.14648E+01_JPRB, 0.13833E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.10294E+01_JPRB, 0.10197E+01_JPRB, 0.10100E+01_JPRB, 0.10373E+01_JPRB, 0.10691E+01_JPRB, 0.10855E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.12165E+01_JPRB, 0.12643E+01_JPRB, 0.13090E+01_JPRB, 0.14370E+01_JPRB, 0.15744E+01_JPRB, 0.17328E+01_JPRB ,&
&  0.18179E+01_JPRB, 0.18843E+01_JPRB, 0.18677E+01_JPRB, 0.17919E+01_JPRB, 0.17206E+01_JPRB, 0.15682E+01_JPRB, 0.14262E+01_JPRB ,&
&  0.14997E+01_JPRB, 0.19462E+01_JPRB, 0.26747E+01_JPRB, 0.34273E+01_JPRB, 0.41514E+01_JPRB, 0.37841E+01_JPRB, 0.42490E+01_JPRB ,&
&  0.36733E+01_JPRB, 0.28611E+01_JPRB, 0.27915E+01_JPRB, 0.28727E+01_JPRB, 0.29894E+01_JPRB, 0.29879E+01_JPRB, 0.30381E+01_JPRB ,&
&  0.30747E+01_JPRB, 0.31248E+01_JPRB, 0.32068E+01_JPRB, 0.33605E+01_JPRB, 0.34937E+01_JPRB, 0.35948E+01_JPRB, 0.37165E+01_JPRB ,&
&  0.38268E+01_JPRB, 0.38825E+01_JPRB, 0.39664E+01_JPRB, 0.37289E+01_JPRB, 0.40862E+01_JPRB, 0.47868E+01_JPRB, 0.53980E+01_JPRB ,&
&  0.57103E+01_JPRB, 0.59049E+01_JPRB, 0.64986E+01_JPRB, 0.67886E+01_JPRB, 0.69844E+01_JPRB, 0.69305E+01_JPRB, 0.69475E+01_JPRB ,&
&  0.70175E+01_JPRB, 0.64680E+01_JPRB, 0.47971E+01_JPRB, 0.52065E+01_JPRB, 0.46711E+01_JPRB, 0.38150E+01_JPRB, 0.38562E+01_JPRB ,&
&  0.34312E+01_JPRB, 0.29430E+01_JPRB, 0.23080E+01_JPRB, 0.22783E+01_JPRB, 0.26324E+01_JPRB, 0.30624E+01_JPRB, 0.40471E+01_JPRB ,&
&  0.33620E+01_JPRB, 0.25542E+01_JPRB, 0.18547E+01_JPRB, 0.17377E+01_JPRB, 0.18924E+01_JPRB, 0.21340E+01_JPRB, 0.20783E+01_JPRB ,&
&  0.20512E+01_JPRB, 0.19793E+01_JPRB, 0.21256E+01_JPRB, 0.24286E+01_JPRB, 0.15398E+01_JPRB, 0.11721E+01_JPRB, 0.12062E+01_JPRB ,&
&  0.12384E+01_JPRB, 0.12492E+01_JPRB, 0.12532E+01_JPRB, 0.12449E+01_JPRB, 0.12588E+01_JPRB, 0.12876E+01_JPRB, 0.12883E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.11864E+01_JPRB, 0.11179E+01_JPRB, 0.10727E+01_JPRB, 0.10405E+01_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.78335E+00_JPRB, 0.78772E+00_JPRB, 0.79668E+00_JPRB, 0.81286E+00_JPRB, 0.82130E+00_JPRB, 0.82722E+00_JPRB, 0.82014E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.80844E+00_JPRB, 0.81072E+00_JPRB, 0.82032E+00_JPRB, 0.86184E+00_JPRB, 0.93353E+00_JPRB, 0.10239E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10439E+01_JPRB, 0.98741E+00_JPRB, 0.93579E+00_JPRB, 0.94081E+00_JPRB, 0.92063E+00_JPRB, 0.87755E+00_JPRB ,&
&  0.85420E+00_JPRB, 0.95033E+00_JPRB, 0.12116E+01_JPRB, 0.16357E+01_JPRB, 0.23980E+01_JPRB, 0.29431E+01_JPRB, 0.39521E+01_JPRB ,&
&  0.40240E+01_JPRB, 0.25547E+01_JPRB, 0.25928E+01_JPRB, 0.24741E+01_JPRB, 0.26303E+01_JPRB, 0.27211E+01_JPRB, 0.27543E+01_JPRB ,&
&  0.28372E+01_JPRB, 0.29286E+01_JPRB, 0.30174E+01_JPRB, 0.31100E+01_JPRB, 0.31807E+01_JPRB, 0.31901E+01_JPRB, 0.32027E+01_JPRB ,&
&  0.32270E+01_JPRB, 0.32209E+01_JPRB, 0.33047E+01_JPRB, 0.35564E+01_JPRB, 0.42148E+01_JPRB, 0.49471E+01_JPRB, 0.50887E+01_JPRB ,&
&  0.51951E+01_JPRB, 0.56419E+01_JPRB, 0.56126E+01_JPRB, 0.60354E+01_JPRB, 0.62259E+01_JPRB, 0.60702E+01_JPRB, 0.64969E+01_JPRB ,&
&  0.67702E+01_JPRB, 0.56988E+01_JPRB, 0.38674E+01_JPRB, 0.40579E+01_JPRB, 0.37375E+01_JPRB, 0.33735E+01_JPRB, 0.34482E+01_JPRB ,&
&  0.31586E+01_JPRB, 0.28160E+01_JPRB, 0.24622E+01_JPRB, 0.22450E+01_JPRB, 0.22516E+01_JPRB, 0.21685E+01_JPRB, 0.22980E+01_JPRB ,&
&  0.21131E+01_JPRB, 0.20603E+01_JPRB, 0.19296E+01_JPRB, 0.19039E+01_JPRB, 0.22210E+01_JPRB, 0.27098E+01_JPRB, 0.22830E+01_JPRB ,&
&  0.21880E+01_JPRB, 0.22092E+01_JPRB, 0.25856E+01_JPRB, 0.27381E+01_JPRB, 0.13998E+01_JPRB, 0.97142E+00_JPRB, 0.89375E+00_JPRB ,&
&  0.90520E+00_JPRB, 0.89860E+00_JPRB, 0.88862E+00_JPRB, 0.87136E+00_JPRB, 0.90866E+00_JPRB, 0.97419E+00_JPRB, 0.94659E+00_JPRB ,&
&  0.93418E+00_JPRB, 0.86351E+00_JPRB, 0.82966E+00_JPRB, 0.81694E+00_JPRB, 0.80726E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.71884E+00_JPRB, 0.71175E+00_JPRB, 0.70518E+00_JPRB, 0.69841E+00_JPRB, 0.71238E+00_JPRB, 0.72288E+00_JPRB, 0.74432E+00_JPRB ,&
&  0.76812E+00_JPRB, 0.78034E+00_JPRB, 0.79855E+00_JPRB, 0.80906E+00_JPRB, 0.84031E+00_JPRB, 0.86728E+00_JPRB, 0.91122E+00_JPRB ,&
&  0.95686E+00_JPRB, 0.98423E+00_JPRB, 0.10131E+01_JPRB, 0.10430E+01_JPRB, 0.10583E+01_JPRB, 0.10778E+01_JPRB, 0.10673E+01_JPRB ,&
&  0.10484E+01_JPRB, 0.10491E+01_JPRB, 0.10901E+01_JPRB, 0.12735E+01_JPRB, 0.16227E+01_JPRB, 0.20714E+01_JPRB, 0.30017E+01_JPRB ,&
&  0.32699E+01_JPRB, 0.22968E+01_JPRB, 0.19448E+01_JPRB, 0.17251E+01_JPRB, 0.18085E+01_JPRB, 0.19313E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.19601E+01_JPRB, 0.19754E+01_JPRB, 0.19698E+01_JPRB, 0.19356E+01_JPRB, 0.18361E+01_JPRB, 0.17116E+01_JPRB, 0.16330E+01_JPRB ,&
&  0.16548E+01_JPRB, 0.16598E+01_JPRB, 0.19185E+01_JPRB, 0.25663E+01_JPRB, 0.31506E+01_JPRB, 0.40724E+01_JPRB, 0.38819E+01_JPRB ,&
&  0.39593E+01_JPRB, 0.41975E+01_JPRB, 0.41697E+01_JPRB, 0.44782E+01_JPRB, 0.46131E+01_JPRB, 0.48039E+01_JPRB, 0.48302E+01_JPRB ,&
&  0.51652E+01_JPRB, 0.45288E+01_JPRB, 0.32260E+01_JPRB, 0.34902E+01_JPRB, 0.32965E+01_JPRB, 0.32933E+01_JPRB, 0.31168E+01_JPRB ,&
&  0.29085E+01_JPRB, 0.26613E+01_JPRB, 0.24475E+01_JPRB, 0.22848E+01_JPRB, 0.22676E+01_JPRB, 0.21940E+01_JPRB, 0.20093E+01_JPRB ,&
&  0.19938E+01_JPRB, 0.20303E+01_JPRB, 0.19977E+01_JPRB, 0.19546E+01_JPRB, 0.24104E+01_JPRB, 0.26967E+01_JPRB, 0.33893E+01_JPRB ,&
&  0.22598E+01_JPRB, 0.20540E+01_JPRB, 0.15395E+01_JPRB, 0.19572E+01_JPRB, 0.10821E+01_JPRB, 0.75454E+00_JPRB, 0.73559E+00_JPRB ,&
&  0.71728E+00_JPRB, 0.69308E+00_JPRB, 0.67281E+00_JPRB, 0.66722E+00_JPRB, 0.68114E+00_JPRB, 0.72154E+00_JPRB, 0.71186E+00_JPRB ,&
&  0.71335E+00_JPRB, 0.71408E+00_JPRB, 0.72572E+00_JPRB, 0.74486E+00_JPRB, 0.73673E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.77082E+00_JPRB, 0.77790E+00_JPRB, 0.79535E+00_JPRB, 0.82185E+00_JPRB, 0.84033E+00_JPRB, 0.85774E+00_JPRB, 0.87468E+00_JPRB ,&
&  0.90773E+00_JPRB, 0.94388E+00_JPRB, 0.97681E+00_JPRB, 0.10093E+01_JPRB, 0.10437E+01_JPRB, 0.10716E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.12108E+01_JPRB, 0.12422E+01_JPRB, 0.12793E+01_JPRB, 0.13124E+01_JPRB, 0.13030E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.13409E+01_JPRB, 0.13925E+01_JPRB, 0.14374E+01_JPRB, 0.15263E+01_JPRB, 0.16418E+01_JPRB, 0.17039E+01_JPRB, 0.20477E+01_JPRB ,&
&  0.21424E+01_JPRB, 0.21977E+01_JPRB, 0.18442E+01_JPRB, 0.15803E+01_JPRB, 0.15082E+01_JPRB, 0.13487E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12892E+01_JPRB, 0.12769E+01_JPRB, 0.12396E+01_JPRB, 0.11780E+01_JPRB, 0.12560E+01_JPRB, 0.14484E+01_JPRB ,&
&  0.17543E+01_JPRB, 0.19358E+01_JPRB, 0.21727E+01_JPRB, 0.23755E+01_JPRB, 0.26519E+01_JPRB, 0.28411E+01_JPRB, 0.28202E+01_JPRB ,&
&  0.28416E+01_JPRB, 0.28072E+01_JPRB, 0.26822E+01_JPRB, 0.31047E+01_JPRB, 0.34331E+01_JPRB, 0.35844E+01_JPRB, 0.36284E+01_JPRB ,&
&  0.36450E+01_JPRB, 0.32584E+01_JPRB, 0.31456E+01_JPRB, 0.30223E+01_JPRB, 0.28622E+01_JPRB, 0.28879E+01_JPRB, 0.27378E+01_JPRB ,&
&  0.25304E+01_JPRB, 0.23860E+01_JPRB, 0.23420E+01_JPRB, 0.23477E+01_JPRB, 0.24084E+01_JPRB, 0.24199E+01_JPRB, 0.22452E+01_JPRB ,&
&  0.21157E+01_JPRB, 0.20449E+01_JPRB, 0.19922E+01_JPRB, 0.19807E+01_JPRB, 0.21930E+01_JPRB, 0.46621E+01_JPRB, 0.57281E+01_JPRB ,&
&  0.21887E+01_JPRB, 0.15080E+01_JPRB, 0.82894E+00_JPRB, 0.10055E+01_JPRB, 0.79310E+00_JPRB, 0.71701E+00_JPRB, 0.78382E+00_JPRB ,&
&  0.79697E+00_JPRB, 0.78088E+00_JPRB, 0.71241E+00_JPRB, 0.64739E+00_JPRB, 0.67237E+00_JPRB, 0.68957E+00_JPRB, 0.68990E+00_JPRB ,&
&  0.70909E+00_JPRB, 0.71780E+00_JPRB, 0.73285E+00_JPRB, 0.76017E+00_JPRB, 0.75928E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.82888E+00_JPRB, 0.85260E+00_JPRB, 0.88191E+00_JPRB, 0.91981E+00_JPRB, 0.95755E+00_JPRB, 0.98938E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.10491E+01_JPRB, 0.10947E+01_JPRB, 0.11256E+01_JPRB, 0.11431E+01_JPRB, 0.11792E+01_JPRB, 0.12099E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.13726E+01_JPRB, 0.14373E+01_JPRB, 0.14972E+01_JPRB, 0.15488E+01_JPRB, 0.16041E+01_JPRB, 0.16560E+01_JPRB ,&
&  0.16976E+01_JPRB, 0.17813E+01_JPRB, 0.18420E+01_JPRB, 0.18756E+01_JPRB, 0.19589E+01_JPRB, 0.21377E+01_JPRB, 0.18963E+01_JPRB ,&
&  0.23476E+01_JPRB, 0.20746E+01_JPRB, 0.18108E+01_JPRB, 0.16310E+01_JPRB, 0.15037E+01_JPRB, 0.14328E+01_JPRB, 0.14056E+01_JPRB ,&
&  0.14577E+01_JPRB, 0.16002E+01_JPRB, 0.17207E+01_JPRB, 0.17335E+01_JPRB, 0.16649E+01_JPRB, 0.18591E+01_JPRB, 0.22745E+01_JPRB ,&
&  0.24988E+01_JPRB, 0.26313E+01_JPRB, 0.27309E+01_JPRB, 0.27696E+01_JPRB, 0.29018E+01_JPRB, 0.30902E+01_JPRB, 0.31971E+01_JPRB ,&
&  0.31960E+01_JPRB, 0.31008E+01_JPRB, 0.27602E+01_JPRB, 0.33572E+01_JPRB, 0.33224E+01_JPRB, 0.31527E+01_JPRB, 0.28453E+01_JPRB ,&
&  0.25524E+01_JPRB, 0.22743E+01_JPRB, 0.23822E+01_JPRB, 0.25464E+01_JPRB, 0.24594E+01_JPRB, 0.23947E+01_JPRB, 0.22300E+01_JPRB ,&
&  0.20551E+01_JPRB, 0.20216E+01_JPRB, 0.20702E+01_JPRB, 0.21826E+01_JPRB, 0.22825E+01_JPRB, 0.22836E+01_JPRB, 0.21286E+01_JPRB ,&
&  0.19718E+01_JPRB, 0.18195E+01_JPRB, 0.17378E+01_JPRB, 0.17188E+01_JPRB, 0.18981E+01_JPRB, 0.23748E+01_JPRB, 0.35028E+01_JPRB ,&
&  0.18221E+01_JPRB, 0.99925E+00_JPRB, 0.97323E+00_JPRB, 0.83188E+00_JPRB, 0.82300E+00_JPRB, 0.73574E+00_JPRB, 0.88278E+00_JPRB ,&
&  0.10912E+01_JPRB, 0.12385E+01_JPRB, 0.12118E+01_JPRB, 0.94738E+00_JPRB, 0.79316E+00_JPRB, 0.75124E+00_JPRB, 0.74693E+00_JPRB ,&
&  0.76310E+00_JPRB, 0.76791E+00_JPRB, 0.77808E+00_JPRB, 0.79206E+00_JPRB, 0.81034E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.80762E+00_JPRB, 0.82544E+00_JPRB, 0.84723E+00_JPRB, 0.87070E+00_JPRB, 0.89910E+00_JPRB, 0.92881E+00_JPRB, 0.95069E+00_JPRB ,&
&  0.97645E+00_JPRB, 0.10061E+01_JPRB, 0.10342E+01_JPRB, 0.10551E+01_JPRB, 0.10821E+01_JPRB, 0.11280E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.13070E+01_JPRB, 0.13694E+01_JPRB, 0.14627E+01_JPRB, 0.15566E+01_JPRB, 0.16619E+01_JPRB, 0.17470E+01_JPRB ,&
&  0.18411E+01_JPRB, 0.19781E+01_JPRB, 0.20536E+01_JPRB, 0.20613E+01_JPRB, 0.21320E+01_JPRB, 0.24044E+01_JPRB, 0.20705E+01_JPRB ,&
&  0.23555E+01_JPRB, 0.22079E+01_JPRB, 0.19952E+01_JPRB, 0.19160E+01_JPRB, 0.18613E+01_JPRB, 0.18971E+01_JPRB, 0.19780E+01_JPRB ,&
&  0.21119E+01_JPRB, 0.23676E+01_JPRB, 0.23312E+01_JPRB, 0.23832E+01_JPRB, 0.24748E+01_JPRB, 0.28213E+01_JPRB, 0.31631E+01_JPRB ,&
&  0.33085E+01_JPRB, 0.35639E+01_JPRB, 0.37014E+01_JPRB, 0.38281E+01_JPRB, 0.40244E+01_JPRB, 0.42345E+01_JPRB, 0.43862E+01_JPRB ,&
&  0.43358E+01_JPRB, 0.42481E+01_JPRB, 0.39219E+01_JPRB, 0.42053E+01_JPRB, 0.39543E+01_JPRB, 0.31713E+01_JPRB, 0.24556E+01_JPRB ,&
&  0.18124E+01_JPRB, 0.18728E+01_JPRB, 0.19863E+01_JPRB, 0.20997E+01_JPRB, 0.19705E+01_JPRB, 0.18557E+01_JPRB, 0.15934E+01_JPRB ,&
&  0.14884E+01_JPRB, 0.14844E+01_JPRB, 0.15870E+01_JPRB, 0.17399E+01_JPRB, 0.18282E+01_JPRB, 0.18135E+01_JPRB, 0.16884E+01_JPRB ,&
&  0.15722E+01_JPRB, 0.14437E+01_JPRB, 0.14128E+01_JPRB, 0.14533E+01_JPRB, 0.14939E+01_JPRB, 0.15131E+01_JPRB, 0.20796E+01_JPRB ,&
&  0.14175E+01_JPRB, 0.12181E+01_JPRB, 0.13941E+01_JPRB, 0.60050E+00_JPRB, 0.75478E+00_JPRB, 0.94281E+00_JPRB, 0.12160E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.19313E+01_JPRB, 0.23455E+01_JPRB, 0.17871E+01_JPRB, 0.12768E+01_JPRB, 0.80416E+00_JPRB, 0.71491E+00_JPRB ,&
&  0.73341E+00_JPRB, 0.74304E+00_JPRB, 0.75742E+00_JPRB, 0.76368E+00_JPRB, 0.78492E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.73055E+00_JPRB, 0.72841E+00_JPRB, 0.73096E+00_JPRB, 0.72227E+00_JPRB, 0.71989E+00_JPRB, 0.72247E+00_JPRB, 0.71374E+00_JPRB ,&
&  0.72074E+00_JPRB, 0.74102E+00_JPRB, 0.74930E+00_JPRB, 0.77876E+00_JPRB, 0.80286E+00_JPRB, 0.87009E+00_JPRB, 0.97167E+00_JPRB ,&
&  0.10611E+01_JPRB, 0.12303E+01_JPRB, 0.13454E+01_JPRB, 0.14754E+01_JPRB, 0.15926E+01_JPRB, 0.17205E+01_JPRB, 0.18317E+01_JPRB ,&
&  0.19742E+01_JPRB, 0.21576E+01_JPRB, 0.23409E+01_JPRB, 0.24804E+01_JPRB, 0.25440E+01_JPRB, 0.29335E+01_JPRB, 0.23188E+01_JPRB ,&
&  0.25155E+01_JPRB, 0.25072E+01_JPRB, 0.23738E+01_JPRB, 0.23168E+01_JPRB, 0.22672E+01_JPRB, 0.22564E+01_JPRB, 0.22620E+01_JPRB ,&
&  0.23463E+01_JPRB, 0.24186E+01_JPRB, 0.24690E+01_JPRB, 0.27536E+01_JPRB, 0.30732E+01_JPRB, 0.33571E+01_JPRB, 0.36518E+01_JPRB ,&
&  0.39235E+01_JPRB, 0.42004E+01_JPRB, 0.45368E+01_JPRB, 0.48849E+01_JPRB, 0.52373E+01_JPRB, 0.54553E+01_JPRB, 0.57187E+01_JPRB ,&
&  0.58557E+01_JPRB, 0.59480E+01_JPRB, 0.55497E+01_JPRB, 0.48239E+01_JPRB, 0.40539E+01_JPRB, 0.31614E+01_JPRB, 0.22949E+01_JPRB ,&
&  0.17170E+01_JPRB, 0.16164E+01_JPRB, 0.16069E+01_JPRB, 0.15798E+01_JPRB, 0.13983E+01_JPRB, 0.12815E+01_JPRB, 0.12050E+01_JPRB ,&
&  0.11339E+01_JPRB, 0.10500E+01_JPRB, 0.11137E+01_JPRB, 0.12172E+01_JPRB, 0.12587E+01_JPRB, 0.12497E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.11193E+01_JPRB, 0.11512E+01_JPRB, 0.12075E+01_JPRB, 0.13217E+01_JPRB, 0.15125E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.14192E+01_JPRB, 0.94251E+00_JPRB, 0.87952E+00_JPRB, 0.85392E+00_JPRB, 0.79548E+00_JPRB, 0.10059E+01_JPRB, 0.13609E+01_JPRB ,&
&  0.19947E+01_JPRB, 0.21936E+01_JPRB, 0.23919E+01_JPRB, 0.29268E+01_JPRB, 0.25436E+01_JPRB, 0.19135E+01_JPRB, 0.64715E+00_JPRB ,&
&  0.65211E+00_JPRB, 0.66081E+00_JPRB, 0.69520E+00_JPRB, 0.70777E+00_JPRB, 0.71545E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.61107E+00_JPRB, 0.59169E+00_JPRB, 0.58314E+00_JPRB, 0.58037E+00_JPRB, 0.56449E+00_JPRB, 0.56759E+00_JPRB, 0.56786E+00_JPRB ,&
&  0.59043E+00_JPRB, 0.62294E+00_JPRB, 0.67812E+00_JPRB, 0.75000E+00_JPRB, 0.85096E+00_JPRB, 0.94374E+00_JPRB, 0.10635E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.12701E+01_JPRB, 0.13808E+01_JPRB, 0.14992E+01_JPRB, 0.16294E+01_JPRB, 0.17512E+01_JPRB, 0.19055E+01_JPRB ,&
&  0.20504E+01_JPRB, 0.22195E+01_JPRB, 0.24248E+01_JPRB, 0.26086E+01_JPRB, 0.27517E+01_JPRB, 0.28674E+01_JPRB, 0.22241E+01_JPRB ,&
&  0.26644E+01_JPRB, 0.27640E+01_JPRB, 0.27866E+01_JPRB, 0.26969E+01_JPRB, 0.25545E+01_JPRB, 0.24192E+01_JPRB, 0.24066E+01_JPRB ,&
&  0.24290E+01_JPRB, 0.24315E+01_JPRB, 0.24171E+01_JPRB, 0.25002E+01_JPRB, 0.31353E+01_JPRB, 0.34053E+01_JPRB, 0.37085E+01_JPRB ,&
&  0.40267E+01_JPRB, 0.43842E+01_JPRB, 0.47559E+01_JPRB, 0.51336E+01_JPRB, 0.53888E+01_JPRB, 0.56643E+01_JPRB, 0.59508E+01_JPRB ,&
&  0.61795E+01_JPRB, 0.63752E+01_JPRB, 0.60942E+01_JPRB, 0.47365E+01_JPRB, 0.38652E+01_JPRB, 0.31045E+01_JPRB, 0.19843E+01_JPRB ,&
&  0.15054E+01_JPRB, 0.13813E+01_JPRB, 0.13681E+01_JPRB, 0.13046E+01_JPRB, 0.11474E+01_JPRB, 0.10388E+01_JPRB, 0.95827E+00_JPRB ,&
&  0.84182E+00_JPRB, 0.81104E+00_JPRB, 0.81198E+00_JPRB, 0.88941E+00_JPRB, 0.93585E+00_JPRB, 0.89453E+00_JPRB, 0.87530E+00_JPRB ,&
&  0.83526E+00_JPRB, 0.80885E+00_JPRB, 0.88825E+00_JPRB, 0.10173E+01_JPRB, 0.12890E+01_JPRB, 0.15905E+01_JPRB, 0.18082E+01_JPRB ,&
&  0.17593E+01_JPRB, 0.95538E+00_JPRB, 0.83215E+00_JPRB, 0.78691E+00_JPRB, 0.78659E+00_JPRB, 0.83938E+00_JPRB, 0.93458E+00_JPRB ,&
&  0.11613E+01_JPRB, 0.15621E+01_JPRB, 0.13518E+01_JPRB, 0.10401E+01_JPRB, 0.10675E+01_JPRB, 0.99038E+00_JPRB, 0.59553E+00_JPRB ,&
&  0.60089E+00_JPRB, 0.59606E+00_JPRB, 0.61488E+00_JPRB, 0.62174E+00_JPRB, 0.62589E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.53941E+00_JPRB, 0.51499E+00_JPRB, 0.49188E+00_JPRB, 0.48323E+00_JPRB, 0.47783E+00_JPRB, 0.47230E+00_JPRB, 0.49038E+00_JPRB ,&
&  0.53273E+00_JPRB, 0.57437E+00_JPRB, 0.64319E+00_JPRB, 0.74954E+00_JPRB, 0.86792E+00_JPRB, 0.95644E+00_JPRB, 0.10491E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.12235E+01_JPRB, 0.12987E+01_JPRB, 0.13764E+01_JPRB, 0.14771E+01_JPRB, 0.16050E+01_JPRB, 0.17800E+01_JPRB ,&
&  0.19855E+01_JPRB, 0.21894E+01_JPRB, 0.24357E+01_JPRB, 0.27246E+01_JPRB, 0.30052E+01_JPRB, 0.31399E+01_JPRB, 0.33861E+01_JPRB ,&
&  0.23087E+01_JPRB, 0.29308E+01_JPRB, 0.29450E+01_JPRB, 0.28646E+01_JPRB, 0.27492E+01_JPRB, 0.26361E+01_JPRB, 0.25756E+01_JPRB ,&
&  0.25372E+01_JPRB, 0.23540E+01_JPRB, 0.23323E+01_JPRB, 0.23959E+01_JPRB, 0.27786E+01_JPRB, 0.30819E+01_JPRB, 0.33804E+01_JPRB ,&
&  0.36973E+01_JPRB, 0.40558E+01_JPRB, 0.44305E+01_JPRB, 0.47892E+01_JPRB, 0.50729E+01_JPRB, 0.53800E+01_JPRB, 0.56154E+01_JPRB ,&
&  0.59034E+01_JPRB, 0.61490E+01_JPRB, 0.62225E+01_JPRB, 0.50496E+01_JPRB, 0.39624E+01_JPRB, 0.33614E+01_JPRB, 0.23665E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.12155E+01_JPRB, 0.12285E+01_JPRB, 0.12500E+01_JPRB, 0.11365E+01_JPRB, 0.10446E+01_JPRB, 0.93328E+00_JPRB ,&
&  0.84201E+00_JPRB, 0.74399E+00_JPRB, 0.70635E+00_JPRB, 0.73375E+00_JPRB, 0.80016E+00_JPRB, 0.75391E+00_JPRB, 0.69304E+00_JPRB ,&
&  0.65502E+00_JPRB, 0.64796E+00_JPRB, 0.72974E+00_JPRB, 0.82774E+00_JPRB, 0.96621E+00_JPRB, 0.10440E+01_JPRB, 0.10970E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.89227E+00_JPRB, 0.84475E+00_JPRB, 0.84291E+00_JPRB, 0.82714E+00_JPRB, 0.82716E+00_JPRB, 0.84357E+00_JPRB ,&
&  0.87945E+00_JPRB, 0.94876E+00_JPRB, 0.96069E+00_JPRB, 0.89480E+00_JPRB, 0.79568E+00_JPRB, 0.73583E+00_JPRB, 0.68728E+00_JPRB ,&
&  0.69019E+00_JPRB, 0.67542E+00_JPRB, 0.65813E+00_JPRB, 0.62408E+00_JPRB, 0.58373E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.55652E+00_JPRB, 0.51612E+00_JPRB, 0.46577E+00_JPRB, 0.42648E+00_JPRB, 0.45716E+00_JPRB, 0.46503E+00_JPRB, 0.49077E+00_JPRB ,&
&  0.50604E+00_JPRB, 0.54609E+00_JPRB, 0.59195E+00_JPRB, 0.65908E+00_JPRB, 0.75558E+00_JPRB, 0.85641E+00_JPRB, 0.94642E+00_JPRB ,&
&  0.10292E+01_JPRB, 0.11080E+01_JPRB, 0.11680E+01_JPRB, 0.11994E+01_JPRB, 0.12486E+01_JPRB, 0.13279E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.17604E+01_JPRB, 0.20054E+01_JPRB, 0.23137E+01_JPRB, 0.26669E+01_JPRB, 0.28860E+01_JPRB, 0.30655E+01_JPRB ,&
&  0.16307E+01_JPRB, 0.25917E+01_JPRB, 0.31349E+01_JPRB, 0.29461E+01_JPRB, 0.28548E+01_JPRB, 0.27604E+01_JPRB, 0.26791E+01_JPRB ,&
&  0.24570E+01_JPRB, 0.21998E+01_JPRB, 0.21392E+01_JPRB, 0.24267E+01_JPRB, 0.23679E+01_JPRB, 0.26404E+01_JPRB, 0.28575E+01_JPRB ,&
&  0.30933E+01_JPRB, 0.33476E+01_JPRB, 0.36448E+01_JPRB, 0.40188E+01_JPRB, 0.44034E+01_JPRB, 0.47217E+01_JPRB, 0.49936E+01_JPRB ,&
&  0.53635E+01_JPRB, 0.57118E+01_JPRB, 0.58383E+01_JPRB, 0.44785E+01_JPRB, 0.40251E+01_JPRB, 0.37408E+01_JPRB, 0.32829E+01_JPRB ,&
&  0.18733E+01_JPRB, 0.12399E+01_JPRB, 0.12447E+01_JPRB, 0.12902E+01_JPRB, 0.12063E+01_JPRB, 0.11240E+01_JPRB, 0.10350E+01_JPRB ,&
&  0.91431E+00_JPRB, 0.82575E+00_JPRB, 0.75716E+00_JPRB, 0.72913E+00_JPRB, 0.75964E+00_JPRB, 0.75927E+00_JPRB, 0.71747E+00_JPRB ,&
&  0.67325E+00_JPRB, 0.67787E+00_JPRB, 0.73330E+00_JPRB, 0.78354E+00_JPRB, 0.86208E+00_JPRB, 0.90733E+00_JPRB, 0.97021E+00_JPRB ,&
&  0.97323E+00_JPRB, 0.86933E+00_JPRB, 0.85504E+00_JPRB, 0.86746E+00_JPRB, 0.87480E+00_JPRB, 0.88226E+00_JPRB, 0.88522E+00_JPRB ,&
&  0.88341E+00_JPRB, 0.90423E+00_JPRB, 0.90885E+00_JPRB, 0.91178E+00_JPRB, 0.87829E+00_JPRB, 0.84271E+00_JPRB, 0.81688E+00_JPRB ,&
&  0.78500E+00_JPRB, 0.74834E+00_JPRB, 0.71817E+00_JPRB, 0.66377E+00_JPRB, 0.60653E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.58827E+00_JPRB, 0.53944E+00_JPRB, 0.49099E+00_JPRB, 0.47453E+00_JPRB, 0.50891E+00_JPRB, 0.54181E+00_JPRB, 0.54261E+00_JPRB ,&
&  0.53983E+00_JPRB, 0.53115E+00_JPRB, 0.53238E+00_JPRB, 0.55421E+00_JPRB, 0.61862E+00_JPRB, 0.71855E+00_JPRB, 0.80745E+00_JPRB ,&
&  0.88479E+00_JPRB, 0.94243E+00_JPRB, 0.98639E+00_JPRB, 0.10001E+01_JPRB, 0.98960E+00_JPRB, 0.10170E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.12904E+01_JPRB, 0.14577E+01_JPRB, 0.17057E+01_JPRB, 0.20759E+01_JPRB, 0.25319E+01_JPRB, 0.30005E+01_JPRB ,&
&  0.21174E+01_JPRB, 0.89552E+00_JPRB, 0.27755E+01_JPRB, 0.33611E+01_JPRB, 0.31390E+01_JPRB, 0.30680E+01_JPRB, 0.28024E+01_JPRB ,&
&  0.25132E+01_JPRB, 0.24424E+01_JPRB, 0.22387E+01_JPRB, 0.23461E+01_JPRB, 0.22056E+01_JPRB, 0.22593E+01_JPRB, 0.23187E+01_JPRB ,&
&  0.24204E+01_JPRB, 0.25488E+01_JPRB, 0.27577E+01_JPRB, 0.30166E+01_JPRB, 0.33098E+01_JPRB, 0.37643E+01_JPRB, 0.40961E+01_JPRB ,&
&  0.43863E+01_JPRB, 0.46876E+01_JPRB, 0.47756E+01_JPRB, 0.34367E+01_JPRB, 0.27324E+01_JPRB, 0.23684E+01_JPRB, 0.21103E+01_JPRB ,&
&  0.17629E+01_JPRB, 0.13604E+01_JPRB, 0.13137E+01_JPRB, 0.13567E+01_JPRB, 0.13059E+01_JPRB, 0.11855E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.10226E+01_JPRB, 0.92843E+00_JPRB, 0.84981E+00_JPRB, 0.78745E+00_JPRB, 0.78786E+00_JPRB, 0.79932E+00_JPRB, 0.80294E+00_JPRB ,&
&  0.77540E+00_JPRB, 0.76859E+00_JPRB, 0.79240E+00_JPRB, 0.79478E+00_JPRB, 0.80130E+00_JPRB, 0.80939E+00_JPRB, 0.82430E+00_JPRB ,&
&  0.87131E+00_JPRB, 0.85696E+00_JPRB, 0.86147E+00_JPRB, 0.88309E+00_JPRB, 0.89647E+00_JPRB, 0.88581E+00_JPRB, 0.89860E+00_JPRB ,&
&  0.91845E+00_JPRB, 0.93756E+00_JPRB, 0.92801E+00_JPRB, 0.93947E+00_JPRB, 0.92372E+00_JPRB, 0.91065E+00_JPRB, 0.90583E+00_JPRB ,&
&  0.89666E+00_JPRB, 0.87142E+00_JPRB, 0.82002E+00_JPRB, 0.71952E+00_JPRB, 0.65128E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.64452E+00_JPRB, 0.56368E+00_JPRB, 0.52044E+00_JPRB, 0.52719E+00_JPRB, 0.56048E+00_JPRB, 0.57545E+00_JPRB, 0.58147E+00_JPRB ,&
&  0.57304E+00_JPRB, 0.55925E+00_JPRB, 0.53723E+00_JPRB, 0.52825E+00_JPRB, 0.54042E+00_JPRB, 0.56925E+00_JPRB, 0.63720E+00_JPRB ,&
&  0.70480E+00_JPRB, 0.76995E+00_JPRB, 0.81982E+00_JPRB, 0.81700E+00_JPRB, 0.77342E+00_JPRB, 0.76426E+00_JPRB, 0.78696E+00_JPRB ,&
&  0.82762E+00_JPRB, 0.90107E+00_JPRB, 0.10039E+01_JPRB, 0.11472E+01_JPRB, 0.13784E+01_JPRB, 0.17808E+01_JPRB, 0.24157E+01_JPRB ,&
&  0.35758E+01_JPRB, 0.24104E+01_JPRB, 0.14609E+01_JPRB, 0.30135E+01_JPRB, 0.36530E+01_JPRB, 0.37917E+01_JPRB, 0.31022E+01_JPRB ,&
&  0.29726E+01_JPRB, 0.27572E+01_JPRB, 0.25036E+01_JPRB, 0.24713E+01_JPRB, 0.22170E+01_JPRB, 0.21292E+01_JPRB, 0.20513E+01_JPRB ,&
&  0.19837E+01_JPRB, 0.20607E+01_JPRB, 0.21896E+01_JPRB, 0.23157E+01_JPRB, 0.24499E+01_JPRB, 0.27517E+01_JPRB, 0.31453E+01_JPRB ,&
&  0.35339E+01_JPRB, 0.37849E+01_JPRB, 0.38771E+01_JPRB, 0.28826E+01_JPRB, 0.23638E+01_JPRB, 0.21151E+01_JPRB, 0.19862E+01_JPRB ,&
&  0.17357E+01_JPRB, 0.15779E+01_JPRB, 0.15328E+01_JPRB, 0.14631E+01_JPRB, 0.13812E+01_JPRB, 0.12929E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.10554E+01_JPRB, 0.97908E+00_JPRB, 0.92956E+00_JPRB, 0.90846E+00_JPRB, 0.90297E+00_JPRB, 0.89098E+00_JPRB ,&
&  0.89287E+00_JPRB, 0.89212E+00_JPRB, 0.89279E+00_JPRB, 0.86429E+00_JPRB, 0.83277E+00_JPRB, 0.81232E+00_JPRB, 0.78954E+00_JPRB ,&
&  0.81370E+00_JPRB, 0.84272E+00_JPRB, 0.86054E+00_JPRB, 0.89002E+00_JPRB, 0.89275E+00_JPRB, 0.88829E+00_JPRB, 0.91629E+00_JPRB ,&
&  0.92848E+00_JPRB, 0.94976E+00_JPRB, 0.94172E+00_JPRB, 0.94850E+00_JPRB, 0.93129E+00_JPRB, 0.91718E+00_JPRB, 0.91899E+00_JPRB ,&
&  0.95178E+00_JPRB, 0.98330E+00_JPRB, 0.94350E+00_JPRB, 0.76557E+00_JPRB, 0.69090E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.62885E+00_JPRB, 0.59037E+00_JPRB, 0.54802E+00_JPRB, 0.55853E+00_JPRB, 0.56883E+00_JPRB, 0.58343E+00_JPRB, 0.59218E+00_JPRB ,&
&  0.59840E+00_JPRB, 0.60074E+00_JPRB, 0.60376E+00_JPRB, 0.59009E+00_JPRB, 0.55612E+00_JPRB, 0.54260E+00_JPRB, 0.54476E+00_JPRB ,&
&  0.56432E+00_JPRB, 0.61026E+00_JPRB, 0.66271E+00_JPRB, 0.67302E+00_JPRB, 0.66195E+00_JPRB, 0.64931E+00_JPRB, 0.64496E+00_JPRB ,&
&  0.65493E+00_JPRB, 0.67757E+00_JPRB, 0.71226E+00_JPRB, 0.78369E+00_JPRB, 0.91187E+00_JPRB, 0.11191E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.24575E+01_JPRB, 0.26660E+01_JPRB, 0.11297E+01_JPRB, 0.23943E+01_JPRB, 0.37345E+01_JPRB, 0.40074E+01_JPRB, 0.38093E+01_JPRB ,&
&  0.35840E+01_JPRB, 0.31701E+01_JPRB, 0.28390E+01_JPRB, 0.25113E+01_JPRB, 0.21594E+01_JPRB, 0.19712E+01_JPRB, 0.18127E+01_JPRB ,&
&  0.16971E+01_JPRB, 0.17240E+01_JPRB, 0.18501E+01_JPRB, 0.19447E+01_JPRB, 0.19750E+01_JPRB, 0.20655E+01_JPRB, 0.23488E+01_JPRB ,&
&  0.27791E+01_JPRB, 0.32195E+01_JPRB, 0.34268E+01_JPRB, 0.27068E+01_JPRB, 0.23612E+01_JPRB, 0.23567E+01_JPRB, 0.22569E+01_JPRB ,&
&  0.20694E+01_JPRB, 0.18271E+01_JPRB, 0.16684E+01_JPRB, 0.15707E+01_JPRB, 0.14624E+01_JPRB, 0.14144E+01_JPRB, 0.14247E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.11962E+01_JPRB, 0.11097E+01_JPRB, 0.10524E+01_JPRB, 0.10137E+01_JPRB, 0.99146E+00_JPRB, 0.97331E+00_JPRB ,&
&  0.96133E+00_JPRB, 0.96969E+00_JPRB, 0.98463E+00_JPRB, 0.96701E+00_JPRB, 0.92674E+00_JPRB, 0.88149E+00_JPRB, 0.83720E+00_JPRB ,&
&  0.82535E+00_JPRB, 0.83169E+00_JPRB, 0.86148E+00_JPRB, 0.88062E+00_JPRB, 0.89387E+00_JPRB, 0.90789E+00_JPRB, 0.92652E+00_JPRB ,&
&  0.94838E+00_JPRB, 0.96031E+00_JPRB, 0.93794E+00_JPRB, 0.91563E+00_JPRB, 0.92757E+00_JPRB, 0.90615E+00_JPRB, 0.88714E+00_JPRB ,&
&  0.92050E+00_JPRB, 0.98371E+00_JPRB, 0.10595E+01_JPRB, 0.77034E+00_JPRB, 0.68875E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.62836E+00_JPRB, 0.58380E+00_JPRB, 0.57825E+00_JPRB, 0.58164E+00_JPRB, 0.59185E+00_JPRB, 0.60837E+00_JPRB, 0.62695E+00_JPRB ,&
&  0.63444E+00_JPRB, 0.63627E+00_JPRB, 0.63587E+00_JPRB, 0.62970E+00_JPRB, 0.61102E+00_JPRB, 0.59079E+00_JPRB, 0.57000E+00_JPRB ,&
&  0.56115E+00_JPRB, 0.56570E+00_JPRB, 0.58638E+00_JPRB, 0.59682E+00_JPRB, 0.59714E+00_JPRB, 0.59311E+00_JPRB, 0.59439E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60465E+00_JPRB, 0.60759E+00_JPRB, 0.63403E+00_JPRB, 0.69272E+00_JPRB, 0.80907E+00_JPRB, 0.10373E+01_JPRB ,&
&  0.17010E+01_JPRB, 0.22646E+01_JPRB, 0.21590E+01_JPRB, 0.35959E+01_JPRB, 0.44800E+01_JPRB, 0.44411E+01_JPRB, 0.43702E+01_JPRB ,&
&  0.41007E+01_JPRB, 0.36291E+01_JPRB, 0.33699E+01_JPRB, 0.26829E+01_JPRB, 0.22134E+01_JPRB, 0.18772E+01_JPRB, 0.16317E+01_JPRB ,&
&  0.14411E+01_JPRB, 0.13764E+01_JPRB, 0.14487E+01_JPRB, 0.15934E+01_JPRB, 0.16915E+01_JPRB, 0.17291E+01_JPRB, 0.18407E+01_JPRB ,&
&  0.21373E+01_JPRB, 0.26842E+01_JPRB, 0.31368E+01_JPRB, 0.29079E+01_JPRB, 0.23642E+01_JPRB, 0.26010E+01_JPRB, 0.28614E+01_JPRB ,&
&  0.32243E+01_JPRB, 0.23383E+01_JPRB, 0.18601E+01_JPRB, 0.16792E+01_JPRB, 0.15366E+01_JPRB, 0.16406E+01_JPRB, 0.15613E+01_JPRB ,&
&  0.14500E+01_JPRB, 0.13335E+01_JPRB, 0.12299E+01_JPRB, 0.11605E+01_JPRB, 0.11073E+01_JPRB, 0.10671E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10185E+01_JPRB, 0.10284E+01_JPRB, 0.10286E+01_JPRB, 0.10098E+01_JPRB, 0.97055E+00_JPRB, 0.91725E+00_JPRB ,&
&  0.87253E+00_JPRB, 0.85565E+00_JPRB, 0.83788E+00_JPRB, 0.85512E+00_JPRB, 0.88234E+00_JPRB, 0.89843E+00_JPRB, 0.90238E+00_JPRB ,&
&  0.92601E+00_JPRB, 0.94548E+00_JPRB, 0.93536E+00_JPRB, 0.91239E+00_JPRB, 0.90379E+00_JPRB, 0.89812E+00_JPRB, 0.86247E+00_JPRB ,&
&  0.84230E+00_JPRB, 0.82914E+00_JPRB, 0.77796E+00_JPRB, 0.73290E+00_JPRB, 0.67552E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.64128E+00_JPRB, 0.61392E+00_JPRB, 0.60794E+00_JPRB, 0.61522E+00_JPRB, 0.62536E+00_JPRB, 0.63751E+00_JPRB, 0.65506E+00_JPRB ,&
&  0.66411E+00_JPRB, 0.66539E+00_JPRB, 0.66350E+00_JPRB, 0.65940E+00_JPRB, 0.64608E+00_JPRB, 0.62975E+00_JPRB, 0.61151E+00_JPRB ,&
&  0.59412E+00_JPRB, 0.56573E+00_JPRB, 0.55124E+00_JPRB, 0.55724E+00_JPRB, 0.55978E+00_JPRB, 0.56112E+00_JPRB, 0.56427E+00_JPRB ,&
&  0.56558E+00_JPRB, 0.57223E+00_JPRB, 0.57271E+00_JPRB, 0.58370E+00_JPRB, 0.61122E+00_JPRB, 0.68477E+00_JPRB, 0.82457E+00_JPRB ,&
&  0.12590E+01_JPRB, 0.20389E+01_JPRB, 0.17542E+01_JPRB, 0.41113E+01_JPRB, 0.46794E+01_JPRB, 0.46308E+01_JPRB, 0.41871E+01_JPRB ,&
&  0.38613E+01_JPRB, 0.39809E+01_JPRB, 0.30609E+01_JPRB, 0.24684E+01_JPRB, 0.21437E+01_JPRB, 0.18502E+01_JPRB, 0.15380E+01_JPRB ,&
&  0.12990E+01_JPRB, 0.11776E+01_JPRB, 0.11404E+01_JPRB, 0.12030E+01_JPRB, 0.13363E+01_JPRB, 0.14568E+01_JPRB, 0.15509E+01_JPRB ,&
&  0.17223E+01_JPRB, 0.21232E+01_JPRB, 0.27068E+01_JPRB, 0.26791E+01_JPRB, 0.25169E+01_JPRB, 0.28773E+01_JPRB, 0.35859E+01_JPRB ,&
&  0.43250E+01_JPRB, 0.34014E+01_JPRB, 0.22679E+01_JPRB, 0.18491E+01_JPRB, 0.17339E+01_JPRB, 0.17844E+01_JPRB, 0.17423E+01_JPRB ,&
&  0.16126E+01_JPRB, 0.14975E+01_JPRB, 0.13641E+01_JPRB, 0.12732E+01_JPRB, 0.12087E+01_JPRB, 0.11381E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.10483E+01_JPRB, 0.10391E+01_JPRB, 0.10262E+01_JPRB, 0.10204E+01_JPRB, 0.10117E+01_JPRB, 0.98822E+00_JPRB ,&
&  0.94521E+00_JPRB, 0.89614E+00_JPRB, 0.84269E+00_JPRB, 0.83041E+00_JPRB, 0.85302E+00_JPRB, 0.87340E+00_JPRB, 0.88466E+00_JPRB ,&
&  0.92576E+00_JPRB, 0.93781E+00_JPRB, 0.91664E+00_JPRB, 0.88096E+00_JPRB, 0.87636E+00_JPRB, 0.90216E+00_JPRB, 0.86263E+00_JPRB ,&
&  0.84655E+00_JPRB, 0.82522E+00_JPRB, 0.79276E+00_JPRB, 0.75362E+00_JPRB, 0.69758E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.68717E+00_JPRB, 0.66582E+00_JPRB, 0.65417E+00_JPRB, 0.65311E+00_JPRB, 0.65097E+00_JPRB, 0.65556E+00_JPRB, 0.66867E+00_JPRB ,&
&  0.68668E+00_JPRB, 0.69166E+00_JPRB, 0.68855E+00_JPRB, 0.68424E+00_JPRB, 0.67509E+00_JPRB, 0.65983E+00_JPRB, 0.64236E+00_JPRB ,&
&  0.62684E+00_JPRB, 0.60495E+00_JPRB, 0.57501E+00_JPRB, 0.55140E+00_JPRB, 0.55174E+00_JPRB, 0.55671E+00_JPRB, 0.55986E+00_JPRB ,&
&  0.56085E+00_JPRB, 0.56871E+00_JPRB, 0.57220E+00_JPRB, 0.58323E+00_JPRB, 0.59359E+00_JPRB, 0.63481E+00_JPRB, 0.76347E+00_JPRB ,&
&  0.10992E+01_JPRB, 0.24558E+01_JPRB, 0.18886E+01_JPRB, 0.38164E+01_JPRB, 0.43638E+01_JPRB, 0.42539E+01_JPRB, 0.37809E+01_JPRB ,&
&  0.34932E+01_JPRB, 0.32730E+01_JPRB, 0.26884E+01_JPRB, 0.22001E+01_JPRB, 0.18941E+01_JPRB, 0.16602E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.12986E+01_JPRB, 0.11991E+01_JPRB, 0.11373E+01_JPRB, 0.11011E+01_JPRB, 0.11319E+01_JPRB, 0.12136E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.14387E+01_JPRB, 0.16562E+01_JPRB, 0.21514E+01_JPRB, 0.26521E+01_JPRB, 0.25019E+01_JPRB, 0.28975E+01_JPRB, 0.42863E+01_JPRB ,&
&  0.50120E+01_JPRB, 0.48683E+01_JPRB, 0.31526E+01_JPRB, 0.22260E+01_JPRB, 0.19357E+01_JPRB, 0.19585E+01_JPRB, 0.19393E+01_JPRB ,&
&  0.18149E+01_JPRB, 0.17140E+01_JPRB, 0.15621E+01_JPRB, 0.14012E+01_JPRB, 0.12835E+01_JPRB, 0.12002E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10431E+01_JPRB, 0.10222E+01_JPRB, 0.99901E+00_JPRB, 0.98642E+00_JPRB, 0.98759E+00_JPRB, 0.99080E+00_JPRB ,&
&  0.98418E+00_JPRB, 0.95379E+00_JPRB, 0.88700E+00_JPRB, 0.84204E+00_JPRB, 0.83941E+00_JPRB, 0.85410E+00_JPRB, 0.86412E+00_JPRB ,&
&  0.91438E+00_JPRB, 0.92425E+00_JPRB, 0.89314E+00_JPRB, 0.87354E+00_JPRB, 0.85981E+00_JPRB, 0.90873E+00_JPRB, 0.86532E+00_JPRB ,&
&  0.84472E+00_JPRB, 0.82444E+00_JPRB, 0.81292E+00_JPRB, 0.78653E+00_JPRB, 0.73592E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.72490E+00_JPRB, 0.69988E+00_JPRB, 0.68814E+00_JPRB, 0.67456E+00_JPRB, 0.66504E+00_JPRB, 0.66388E+00_JPRB, 0.67424E+00_JPRB ,&
&  0.69444E+00_JPRB, 0.70844E+00_JPRB, 0.70068E+00_JPRB, 0.69549E+00_JPRB, 0.69136E+00_JPRB, 0.67920E+00_JPRB, 0.66338E+00_JPRB ,&
&  0.64738E+00_JPRB, 0.63244E+00_JPRB, 0.61224E+00_JPRB, 0.58746E+00_JPRB, 0.57019E+00_JPRB, 0.56846E+00_JPRB, 0.56586E+00_JPRB ,&
&  0.56464E+00_JPRB, 0.56999E+00_JPRB, 0.57416E+00_JPRB, 0.58162E+00_JPRB, 0.59031E+00_JPRB, 0.61411E+00_JPRB, 0.73184E+00_JPRB ,&
&  0.11262E+01_JPRB, 0.87105E+00_JPRB, 0.23866E+01_JPRB, 0.33409E+01_JPRB, 0.35222E+01_JPRB, 0.32959E+01_JPRB, 0.31746E+01_JPRB ,&
&  0.30042E+01_JPRB, 0.26848E+01_JPRB, 0.23836E+01_JPRB, 0.21153E+01_JPRB, 0.17804E+01_JPRB, 0.15578E+01_JPRB, 0.14227E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.12343E+01_JPRB, 0.11629E+01_JPRB, 0.10912E+01_JPRB, 0.10430E+01_JPRB, 0.10462E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.13095E+01_JPRB, 0.16581E+01_JPRB, 0.21527E+01_JPRB, 0.24634E+01_JPRB, 0.28163E+01_JPRB, 0.35243E+01_JPRB ,&
&  0.46217E+01_JPRB, 0.50934E+01_JPRB, 0.38638E+01_JPRB, 0.28483E+01_JPRB, 0.22903E+01_JPRB, 0.21588E+01_JPRB, 0.21542E+01_JPRB ,&
&  0.20384E+01_JPRB, 0.18918E+01_JPRB, 0.17197E+01_JPRB, 0.15283E+01_JPRB, 0.13663E+01_JPRB, 0.12563E+01_JPRB, 0.11746E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.10488E+01_JPRB, 0.10125E+01_JPRB, 0.98604E+00_JPRB, 0.96836E+00_JPRB, 0.96197E+00_JPRB, 0.96661E+00_JPRB ,&
&  0.97980E+00_JPRB, 0.97853E+00_JPRB, 0.94118E+00_JPRB, 0.89918E+00_JPRB, 0.88776E+00_JPRB, 0.89541E+00_JPRB, 0.90859E+00_JPRB ,&
&  0.93591E+00_JPRB, 0.93342E+00_JPRB, 0.89710E+00_JPRB, 0.87782E+00_JPRB, 0.84007E+00_JPRB, 0.86871E+00_JPRB, 0.84956E+00_JPRB ,&
&  0.83138E+00_JPRB, 0.82051E+00_JPRB, 0.82079E+00_JPRB, 0.80472E+00_JPRB, 0.76615E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.75192E+00_JPRB, 0.72672E+00_JPRB, 0.70940E+00_JPRB, 0.68605E+00_JPRB, 0.67455E+00_JPRB, 0.67373E+00_JPRB, 0.68409E+00_JPRB ,&
&  0.69316E+00_JPRB, 0.69845E+00_JPRB, 0.69105E+00_JPRB, 0.68338E+00_JPRB, 0.67399E+00_JPRB, 0.66625E+00_JPRB, 0.65606E+00_JPRB ,&
&  0.64981E+00_JPRB, 0.64441E+00_JPRB, 0.63425E+00_JPRB, 0.61905E+00_JPRB, 0.60013E+00_JPRB, 0.58981E+00_JPRB, 0.57991E+00_JPRB ,&
&  0.57149E+00_JPRB, 0.56758E+00_JPRB, 0.56770E+00_JPRB, 0.57646E+00_JPRB, 0.58935E+00_JPRB, 0.61003E+00_JPRB, 0.68679E+00_JPRB ,&
&  0.11483E+01_JPRB, 0.17372E+01_JPRB, 0.27922E+01_JPRB, 0.29331E+01_JPRB, 0.26574E+01_JPRB, 0.23092E+01_JPRB, 0.22648E+01_JPRB ,&
&  0.21802E+01_JPRB, 0.20635E+01_JPRB, 0.19122E+01_JPRB, 0.18043E+01_JPRB, 0.16887E+01_JPRB, 0.16094E+01_JPRB, 0.14877E+01_JPRB ,&
&  0.13757E+01_JPRB, 0.12564E+01_JPRB, 0.11309E+01_JPRB, 0.10284E+01_JPRB, 0.95990E+00_JPRB, 0.93136E+00_JPRB, 0.94602E+00_JPRB ,&
&  0.98220E+00_JPRB, 0.10364E+01_JPRB, 0.12376E+01_JPRB, 0.16600E+01_JPRB, 0.23124E+01_JPRB, 0.24573E+01_JPRB, 0.33135E+01_JPRB ,&
&  0.44580E+01_JPRB, 0.44177E+01_JPRB, 0.39198E+01_JPRB, 0.33749E+01_JPRB, 0.28184E+01_JPRB, 0.25996E+01_JPRB, 0.25443E+01_JPRB ,&
&  0.23615E+01_JPRB, 0.20682E+01_JPRB, 0.17767E+01_JPRB, 0.15587E+01_JPRB, 0.14094E+01_JPRB, 0.13171E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.11426E+01_JPRB, 0.10818E+01_JPRB, 0.10330E+01_JPRB, 0.10021E+01_JPRB, 0.98793E+00_JPRB, 0.98328E+00_JPRB ,&
&  0.98652E+00_JPRB, 0.98576E+00_JPRB, 0.97619E+00_JPRB, 0.95515E+00_JPRB, 0.97044E+00_JPRB, 0.10043E+01_JPRB, 0.10169E+01_JPRB ,&
&  0.99793E+00_JPRB, 0.96970E+00_JPRB, 0.92400E+00_JPRB, 0.88285E+00_JPRB, 0.85248E+00_JPRB, 0.86571E+00_JPRB, 0.83935E+00_JPRB ,&
&  0.80907E+00_JPRB, 0.80203E+00_JPRB, 0.81616E+00_JPRB, 0.81237E+00_JPRB, 0.79510E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.77413E+00_JPRB, 0.74683E+00_JPRB, 0.72238E+00_JPRB, 0.70044E+00_JPRB, 0.68366E+00_JPRB, 0.67882E+00_JPRB, 0.68237E+00_JPRB ,&
&  0.68564E+00_JPRB, 0.68142E+00_JPRB, 0.67284E+00_JPRB, 0.66199E+00_JPRB, 0.65386E+00_JPRB, 0.64623E+00_JPRB, 0.63951E+00_JPRB ,&
&  0.63097E+00_JPRB, 0.63195E+00_JPRB, 0.62794E+00_JPRB, 0.61886E+00_JPRB, 0.60780E+00_JPRB, 0.59430E+00_JPRB, 0.58133E+00_JPRB ,&
&  0.57213E+00_JPRB, 0.56760E+00_JPRB, 0.56635E+00_JPRB, 0.57060E+00_JPRB, 0.58474E+00_JPRB, 0.60304E+00_JPRB, 0.64357E+00_JPRB ,&
&  0.98242E+00_JPRB, 0.16562E+01_JPRB, 0.21148E+01_JPRB, 0.24324E+01_JPRB, 0.21710E+01_JPRB, 0.18444E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.15732E+01_JPRB, 0.15625E+01_JPRB, 0.14917E+01_JPRB, 0.14004E+01_JPRB, 0.12895E+01_JPRB, 0.12327E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.10358E+01_JPRB, 0.97828E+00_JPRB, 0.94364E+00_JPRB, 0.91263E+00_JPRB, 0.87969E+00_JPRB, 0.86095E+00_JPRB ,&
&  0.86399E+00_JPRB, 0.88021E+00_JPRB, 0.94817E+00_JPRB, 0.12033E+01_JPRB, 0.17529E+01_JPRB, 0.21896E+01_JPRB, 0.27375E+01_JPRB ,&
&  0.28932E+01_JPRB, 0.30418E+01_JPRB, 0.31379E+01_JPRB, 0.30567E+01_JPRB, 0.28298E+01_JPRB, 0.26903E+01_JPRB, 0.26663E+01_JPRB ,&
&  0.25067E+01_JPRB, 0.21178E+01_JPRB, 0.17502E+01_JPRB, 0.15251E+01_JPRB, 0.14133E+01_JPRB, 0.13580E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.12783E+01_JPRB, 0.12394E+01_JPRB, 0.11861E+01_JPRB, 0.11265E+01_JPRB, 0.10732E+01_JPRB, 0.10289E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.98225E+00_JPRB, 0.97173E+00_JPRB, 0.98096E+00_JPRB, 0.10043E+01_JPRB, 0.10291E+01_JPRB, 0.10497E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10604E+01_JPRB, 0.10288E+01_JPRB, 0.94737E+00_JPRB, 0.89353E+00_JPRB, 0.85386E+00_JPRB, 0.84615E+00_JPRB, 0.80932E+00_JPRB ,&
&  0.78402E+00_JPRB, 0.77460E+00_JPRB, 0.78596E+00_JPRB, 0.79419E+00_JPRB, 0.79637E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.84455E+00_JPRB, 0.79222E+00_JPRB, 0.74699E+00_JPRB, 0.71248E+00_JPRB, 0.69350E+00_JPRB, 0.68362E+00_JPRB, 0.67578E+00_JPRB ,&
&  0.67006E+00_JPRB, 0.66107E+00_JPRB, 0.64667E+00_JPRB, 0.63559E+00_JPRB, 0.62757E+00_JPRB, 0.62269E+00_JPRB, 0.61768E+00_JPRB ,&
&  0.60574E+00_JPRB, 0.60346E+00_JPRB, 0.60347E+00_JPRB, 0.59828E+00_JPRB, 0.59100E+00_JPRB, 0.58227E+00_JPRB, 0.57077E+00_JPRB ,&
&  0.56210E+00_JPRB, 0.55872E+00_JPRB, 0.55972E+00_JPRB, 0.56391E+00_JPRB, 0.56916E+00_JPRB, 0.58354E+00_JPRB, 0.62654E+00_JPRB ,&
&  0.91366E+00_JPRB, 0.12927E+01_JPRB, 0.16092E+01_JPRB, 0.17575E+01_JPRB, 0.15032E+01_JPRB, 0.13595E+01_JPRB, 0.12801E+01_JPRB ,&
&  0.12684E+01_JPRB, 0.12835E+01_JPRB, 0.12127E+01_JPRB, 0.10952E+01_JPRB, 0.98623E+00_JPRB, 0.92662E+00_JPRB, 0.88656E+00_JPRB ,&
&  0.85085E+00_JPRB, 0.82177E+00_JPRB, 0.79569E+00_JPRB, 0.79041E+00_JPRB, 0.78673E+00_JPRB, 0.77786E+00_JPRB, 0.76972E+00_JPRB ,&
&  0.76745E+00_JPRB, 0.77343E+00_JPRB, 0.79269E+00_JPRB, 0.88887E+00_JPRB, 0.11612E+01_JPRB, 0.14654E+01_JPRB, 0.17683E+01_JPRB ,&
&  0.19108E+01_JPRB, 0.20511E+01_JPRB, 0.22158E+01_JPRB, 0.23279E+01_JPRB, 0.23347E+01_JPRB, 0.23273E+01_JPRB, 0.23204E+01_JPRB ,&
&  0.22558E+01_JPRB, 0.20249E+01_JPRB, 0.17192E+01_JPRB, 0.14972E+01_JPRB, 0.13776E+01_JPRB, 0.13167E+01_JPRB, 0.12760E+01_JPRB ,&
&  0.12400E+01_JPRB, 0.12064E+01_JPRB, 0.11619E+01_JPRB, 0.11035E+01_JPRB, 0.10414E+01_JPRB, 0.99714E+00_JPRB, 0.96480E+00_JPRB ,&
&  0.94412E+00_JPRB, 0.94652E+00_JPRB, 0.97086E+00_JPRB, 0.10135E+01_JPRB, 0.10427E+01_JPRB, 0.10545E+01_JPRB, 0.10478E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10236E+01_JPRB, 0.93941E+00_JPRB, 0.89027E+00_JPRB, 0.90428E+00_JPRB, 0.83904E+00_JPRB, 0.78179E+00_JPRB ,&
&  0.75958E+00_JPRB, 0.74325E+00_JPRB, 0.74021E+00_JPRB, 0.75702E+00_JPRB, 0.86778E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.77718E+00_JPRB, 0.76179E+00_JPRB, 0.73395E+00_JPRB, 0.70776E+00_JPRB, 0.69160E+00_JPRB, 0.68206E+00_JPRB, 0.67563E+00_JPRB ,&
&  0.66645E+00_JPRB, 0.65324E+00_JPRB, 0.63773E+00_JPRB, 0.61948E+00_JPRB, 0.60751E+00_JPRB, 0.60098E+00_JPRB, 0.59608E+00_JPRB ,&
&  0.58885E+00_JPRB, 0.57756E+00_JPRB, 0.57377E+00_JPRB, 0.57038E+00_JPRB, 0.56868E+00_JPRB, 0.56600E+00_JPRB, 0.55971E+00_JPRB ,&
&  0.55212E+00_JPRB, 0.54830E+00_JPRB, 0.55075E+00_JPRB, 0.55191E+00_JPRB, 0.55421E+00_JPRB, 0.56232E+00_JPRB, 0.59190E+00_JPRB ,&
&  0.77526E+00_JPRB, 0.98049E+00_JPRB, 0.12010E+01_JPRB, 0.13818E+01_JPRB, 0.12756E+01_JPRB, 0.11765E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10000E+01_JPRB, 0.92890E+00_JPRB, 0.84825E+00_JPRB, 0.78046E+00_JPRB, 0.75290E+00_JPRB, 0.73698E+00_JPRB ,&
&  0.72400E+00_JPRB, 0.71272E+00_JPRB, 0.70347E+00_JPRB, 0.69845E+00_JPRB, 0.69803E+00_JPRB, 0.69538E+00_JPRB, 0.69257E+00_JPRB ,&
&  0.69326E+00_JPRB, 0.69956E+00_JPRB, 0.70965E+00_JPRB, 0.73539E+00_JPRB, 0.83899E+00_JPRB, 0.10296E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.14570E+01_JPRB, 0.16073E+01_JPRB, 0.17313E+01_JPRB, 0.18092E+01_JPRB, 0.18376E+01_JPRB, 0.18337E+01_JPRB ,&
&  0.18173E+01_JPRB, 0.17388E+01_JPRB, 0.15638E+01_JPRB, 0.13789E+01_JPRB, 0.12600E+01_JPRB, 0.11835E+01_JPRB, 0.11267E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.10327E+01_JPRB, 0.98536E+00_JPRB, 0.94520E+00_JPRB, 0.91772E+00_JPRB, 0.89237E+00_JPRB, 0.87359E+00_JPRB ,&
&  0.88037E+00_JPRB, 0.91208E+00_JPRB, 0.95794E+00_JPRB, 0.98747E+00_JPRB, 0.99482E+00_JPRB, 0.99339E+00_JPRB, 0.97997E+00_JPRB ,&
&  0.96563E+00_JPRB, 0.95775E+00_JPRB, 0.93382E+00_JPRB, 0.91213E+00_JPRB, 0.87026E+00_JPRB, 0.82600E+00_JPRB, 0.76547E+00_JPRB ,&
&  0.73350E+00_JPRB, 0.71639E+00_JPRB, 0.70169E+00_JPRB, 0.71080E+00_JPRB, 0.75983E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.70581E+00_JPRB, 0.70670E+00_JPRB, 0.70325E+00_JPRB, 0.69549E+00_JPRB, 0.69133E+00_JPRB, 0.68553E+00_JPRB, 0.67461E+00_JPRB ,&
&  0.66117E+00_JPRB, 0.64872E+00_JPRB, 0.63565E+00_JPRB, 0.61878E+00_JPRB, 0.60101E+00_JPRB, 0.59291E+00_JPRB, 0.58962E+00_JPRB ,&
&  0.58375E+00_JPRB, 0.57394E+00_JPRB, 0.56485E+00_JPRB, 0.56223E+00_JPRB, 0.56085E+00_JPRB, 0.55960E+00_JPRB, 0.55540E+00_JPRB ,&
&  0.54899E+00_JPRB, 0.54750E+00_JPRB, 0.55007E+00_JPRB, 0.54984E+00_JPRB, 0.54524E+00_JPRB, 0.54749E+00_JPRB, 0.57308E+00_JPRB ,&
&  0.70294E+00_JPRB, 0.72748E+00_JPRB, 0.86543E+00_JPRB, 0.10824E+01_JPRB, 0.10028E+01_JPRB, 0.96055E+00_JPRB, 0.88319E+00_JPRB ,&
&  0.80554E+00_JPRB, 0.76130E+00_JPRB, 0.73119E+00_JPRB, 0.69975E+00_JPRB, 0.67041E+00_JPRB, 0.65472E+00_JPRB, 0.65480E+00_JPRB ,&
&  0.66334E+00_JPRB, 0.66677E+00_JPRB, 0.66590E+00_JPRB, 0.65671E+00_JPRB, 0.64984E+00_JPRB, 0.64388E+00_JPRB, 0.63975E+00_JPRB ,&
&  0.63702E+00_JPRB, 0.64031E+00_JPRB, 0.64743E+00_JPRB, 0.65632E+00_JPRB, 0.68651E+00_JPRB, 0.77847E+00_JPRB, 0.92799E+00_JPRB ,&
&  0.10591E+01_JPRB, 0.11640E+01_JPRB, 0.12659E+01_JPRB, 0.13714E+01_JPRB, 0.14508E+01_JPRB, 0.15219E+01_JPRB, 0.15587E+01_JPRB ,&
&  0.15363E+01_JPRB, 0.14666E+01_JPRB, 0.13690E+01_JPRB, 0.12543E+01_JPRB, 0.11457E+01_JPRB, 0.10575E+01_JPRB, 0.98951E+00_JPRB ,&
&  0.93796E+00_JPRB, 0.90391E+00_JPRB, 0.87697E+00_JPRB, 0.84926E+00_JPRB, 0.83366E+00_JPRB, 0.83116E+00_JPRB, 0.84174E+00_JPRB ,&
&  0.86517E+00_JPRB, 0.89177E+00_JPRB, 0.90888E+00_JPRB, 0.91200E+00_JPRB, 0.90189E+00_JPRB, 0.89094E+00_JPRB, 0.87986E+00_JPRB ,&
&  0.87849E+00_JPRB, 0.88523E+00_JPRB, 0.88814E+00_JPRB, 0.86193E+00_JPRB, 0.86487E+00_JPRB, 0.84132E+00_JPRB, 0.79344E+00_JPRB ,&
&  0.74701E+00_JPRB, 0.71498E+00_JPRB, 0.69503E+00_JPRB, 0.62659E+00_JPRB, 0.70112E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.67050E+00_JPRB, 0.67327E+00_JPRB, 0.67488E+00_JPRB, 0.67541E+00_JPRB, 0.67406E+00_JPRB, 0.66768E+00_JPRB, 0.65560E+00_JPRB ,&
&  0.64263E+00_JPRB, 0.62989E+00_JPRB, 0.61943E+00_JPRB, 0.60839E+00_JPRB, 0.59228E+00_JPRB, 0.58089E+00_JPRB, 0.57736E+00_JPRB ,&
&  0.57423E+00_JPRB, 0.56746E+00_JPRB, 0.55988E+00_JPRB, 0.55808E+00_JPRB, 0.55525E+00_JPRB, 0.55120E+00_JPRB, 0.54568E+00_JPRB ,&
&  0.54023E+00_JPRB, 0.53817E+00_JPRB, 0.53787E+00_JPRB, 0.53806E+00_JPRB, 0.53424E+00_JPRB, 0.53663E+00_JPRB, 0.56351E+00_JPRB ,&
&  0.66718E+00_JPRB, 0.62122E+00_JPRB, 0.73329E+00_JPRB, 0.85282E+00_JPRB, 0.78078E+00_JPRB, 0.72489E+00_JPRB, 0.67937E+00_JPRB ,&
&  0.64387E+00_JPRB, 0.62285E+00_JPRB, 0.61122E+00_JPRB, 0.60073E+00_JPRB, 0.59271E+00_JPRB, 0.58583E+00_JPRB, 0.58628E+00_JPRB ,&
&  0.59083E+00_JPRB, 0.60232E+00_JPRB, 0.60951E+00_JPRB, 0.60949E+00_JPRB, 0.60333E+00_JPRB, 0.59613E+00_JPRB, 0.58949E+00_JPRB ,&
&  0.58512E+00_JPRB, 0.58472E+00_JPRB, 0.58758E+00_JPRB, 0.59439E+00_JPRB, 0.61051E+00_JPRB, 0.65094E+00_JPRB, 0.73416E+00_JPRB ,&
&  0.83856E+00_JPRB, 0.92222E+00_JPRB, 0.10031E+01_JPRB, 0.11046E+01_JPRB, 0.12047E+01_JPRB, 0.12773E+01_JPRB, 0.13031E+01_JPRB ,&
&  0.12820E+01_JPRB, 0.12201E+01_JPRB, 0.11613E+01_JPRB, 0.10976E+01_JPRB, 0.10340E+01_JPRB, 0.97862E+00_JPRB, 0.93915E+00_JPRB ,&
&  0.91252E+00_JPRB, 0.89474E+00_JPRB, 0.88494E+00_JPRB, 0.87917E+00_JPRB, 0.88005E+00_JPRB, 0.88331E+00_JPRB, 0.88885E+00_JPRB ,&
&  0.88502E+00_JPRB, 0.87607E+00_JPRB, 0.85135E+00_JPRB, 0.81575E+00_JPRB, 0.78993E+00_JPRB, 0.77608E+00_JPRB, 0.76986E+00_JPRB ,&
&  0.77134E+00_JPRB, 0.78095E+00_JPRB, 0.79379E+00_JPRB, 0.81773E+00_JPRB, 0.82042E+00_JPRB, 0.81992E+00_JPRB, 0.80119E+00_JPRB ,&
&  0.76369E+00_JPRB, 0.72351E+00_JPRB, 0.60716E+00_JPRB, 0.65921E+00_JPRB, 0.67238E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.67847E+00_JPRB, 0.67180E+00_JPRB, 0.66593E+00_JPRB, 0.65983E+00_JPRB, 0.65162E+00_JPRB, 0.64128E+00_JPRB, 0.62909E+00_JPRB ,&
&  0.61729E+00_JPRB, 0.60582E+00_JPRB, 0.59437E+00_JPRB, 0.58424E+00_JPRB, 0.57447E+00_JPRB, 0.56914E+00_JPRB, 0.56709E+00_JPRB ,&
&  0.56694E+00_JPRB, 0.56541E+00_JPRB, 0.56239E+00_JPRB, 0.56135E+00_JPRB, 0.56026E+00_JPRB, 0.55672E+00_JPRB, 0.55184E+00_JPRB ,&
&  0.54446E+00_JPRB, 0.53806E+00_JPRB, 0.53512E+00_JPRB, 0.53666E+00_JPRB, 0.53496E+00_JPRB, 0.53873E+00_JPRB, 0.55517E+00_JPRB ,&
&  0.56004E+00_JPRB, 0.55917E+00_JPRB, 0.62701E+00_JPRB, 0.71289E+00_JPRB, 0.66147E+00_JPRB, 0.61152E+00_JPRB, 0.59056E+00_JPRB ,&
&  0.57379E+00_JPRB, 0.56239E+00_JPRB, 0.55465E+00_JPRB, 0.55040E+00_JPRB, 0.55043E+00_JPRB, 0.55047E+00_JPRB, 0.55098E+00_JPRB ,&
&  0.55299E+00_JPRB, 0.55860E+00_JPRB, 0.56466E+00_JPRB, 0.56869E+00_JPRB, 0.56777E+00_JPRB, 0.56395E+00_JPRB, 0.55766E+00_JPRB ,&
&  0.55184E+00_JPRB, 0.54793E+00_JPRB, 0.54716E+00_JPRB, 0.55145E+00_JPRB, 0.56067E+00_JPRB, 0.57966E+00_JPRB, 0.61952E+00_JPRB ,&
&  0.68245E+00_JPRB, 0.74775E+00_JPRB, 0.80874E+00_JPRB, 0.88126E+00_JPRB, 0.97328E+00_JPRB, 0.10604E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.10696E+01_JPRB, 0.10077E+01_JPRB, 0.95275E+00_JPRB, 0.91046E+00_JPRB, 0.88311E+00_JPRB, 0.86692E+00_JPRB ,&
&  0.86094E+00_JPRB, 0.85819E+00_JPRB, 0.85769E+00_JPRB, 0.85799E+00_JPRB, 0.85708E+00_JPRB, 0.85085E+00_JPRB, 0.83892E+00_JPRB ,&
&  0.81790E+00_JPRB, 0.78471E+00_JPRB, 0.74652E+00_JPRB, 0.71524E+00_JPRB, 0.69369E+00_JPRB, 0.68299E+00_JPRB, 0.68086E+00_JPRB ,&
&  0.68619E+00_JPRB, 0.70037E+00_JPRB, 0.71952E+00_JPRB, 0.75331E+00_JPRB, 0.77754E+00_JPRB, 0.79272E+00_JPRB, 0.79719E+00_JPRB ,&
&  0.77974E+00_JPRB, 0.74778E+00_JPRB, 0.71588E+00_JPRB, 0.69810E+00_JPRB, 0.68938E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.67704E+00_JPRB, 0.65919E+00_JPRB, 0.64180E+00_JPRB, 0.62951E+00_JPRB, 0.61966E+00_JPRB, 0.61008E+00_JPRB, 0.59953E+00_JPRB ,&
&  0.58714E+00_JPRB, 0.57473E+00_JPRB, 0.56348E+00_JPRB, 0.55324E+00_JPRB, 0.54789E+00_JPRB, 0.54515E+00_JPRB, 0.54513E+00_JPRB ,&
&  0.54774E+00_JPRB, 0.55240E+00_JPRB, 0.55337E+00_JPRB, 0.55200E+00_JPRB, 0.55227E+00_JPRB, 0.55158E+00_JPRB, 0.54880E+00_JPRB ,&
&  0.54314E+00_JPRB, 0.53653E+00_JPRB, 0.53239E+00_JPRB, 0.53217E+00_JPRB, 0.53197E+00_JPRB, 0.53422E+00_JPRB, 0.53883E+00_JPRB ,&
&  0.55533E+00_JPRB, 0.52656E+00_JPRB, 0.57287E+00_JPRB, 0.62886E+00_JPRB, 0.61189E+00_JPRB, 0.58671E+00_JPRB, 0.56579E+00_JPRB ,&
&  0.54928E+00_JPRB, 0.54002E+00_JPRB, 0.53500E+00_JPRB, 0.53541E+00_JPRB, 0.53884E+00_JPRB, 0.54129E+00_JPRB, 0.54286E+00_JPRB ,&
&  0.54116E+00_JPRB, 0.54170E+00_JPRB, 0.54010E+00_JPRB, 0.53881E+00_JPRB, 0.53875E+00_JPRB, 0.53957E+00_JPRB, 0.53923E+00_JPRB ,&
&  0.53643E+00_JPRB, 0.53178E+00_JPRB, 0.52728E+00_JPRB, 0.52608E+00_JPRB, 0.52750E+00_JPRB, 0.53284E+00_JPRB, 0.54653E+00_JPRB ,&
&  0.57268E+00_JPRB, 0.60621E+00_JPRB, 0.64748E+00_JPRB, 0.70212E+00_JPRB, 0.77715E+00_JPRB, 0.85555E+00_JPRB, 0.91715E+00_JPRB ,&
&  0.94017E+00_JPRB, 0.92909E+00_JPRB, 0.89839E+00_JPRB, 0.85982E+00_JPRB, 0.82325E+00_JPRB, 0.78889E+00_JPRB, 0.75948E+00_JPRB ,&
&  0.73819E+00_JPRB, 0.72213E+00_JPRB, 0.71039E+00_JPRB, 0.70262E+00_JPRB, 0.69482E+00_JPRB, 0.68388E+00_JPRB, 0.67269E+00_JPRB ,&
&  0.65963E+00_JPRB, 0.64681E+00_JPRB, 0.63587E+00_JPRB, 0.62593E+00_JPRB, 0.61670E+00_JPRB, 0.60947E+00_JPRB, 0.60739E+00_JPRB ,&
&  0.61216E+00_JPRB, 0.62277E+00_JPRB, 0.64152E+00_JPRB, 0.66769E+00_JPRB, 0.69759E+00_JPRB, 0.72175E+00_JPRB, 0.73702E+00_JPRB ,&
&  0.73984E+00_JPRB, 0.73543E+00_JPRB, 0.72176E+00_JPRB, 0.70638E+00_JPRB, 0.69310E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.64854E+00_JPRB, 0.63379E+00_JPRB, 0.61895E+00_JPRB, 0.60675E+00_JPRB, 0.59669E+00_JPRB, 0.58580E+00_JPRB, 0.57371E+00_JPRB ,&
&  0.56349E+00_JPRB, 0.55311E+00_JPRB, 0.54548E+00_JPRB, 0.53918E+00_JPRB, 0.53474E+00_JPRB, 0.53321E+00_JPRB, 0.53381E+00_JPRB ,&
&  0.53658E+00_JPRB, 0.54252E+00_JPRB, 0.54484E+00_JPRB, 0.54241E+00_JPRB, 0.54370E+00_JPRB, 0.54570E+00_JPRB, 0.54556E+00_JPRB ,&
&  0.54195E+00_JPRB, 0.53743E+00_JPRB, 0.53266E+00_JPRB, 0.52959E+00_JPRB, 0.52919E+00_JPRB, 0.53126E+00_JPRB, 0.53029E+00_JPRB ,&
&  0.53075E+00_JPRB, 0.53920E+00_JPRB, 0.56164E+00_JPRB, 0.58221E+00_JPRB, 0.58228E+00_JPRB, 0.56888E+00_JPRB, 0.55278E+00_JPRB ,&
&  0.53626E+00_JPRB, 0.52740E+00_JPRB, 0.52616E+00_JPRB, 0.52831E+00_JPRB, 0.53175E+00_JPRB, 0.53413E+00_JPRB, 0.53721E+00_JPRB ,&
&  0.53478E+00_JPRB, 0.53302E+00_JPRB, 0.52927E+00_JPRB, 0.52553E+00_JPRB, 0.52496E+00_JPRB, 0.52581E+00_JPRB, 0.52684E+00_JPRB ,&
&  0.52626E+00_JPRB, 0.52687E+00_JPRB, 0.52697E+00_JPRB, 0.52520E+00_JPRB, 0.52594E+00_JPRB, 0.52796E+00_JPRB, 0.53189E+00_JPRB ,&
&  0.54023E+00_JPRB, 0.55416E+00_JPRB, 0.57229E+00_JPRB, 0.60045E+00_JPRB, 0.64711E+00_JPRB, 0.70159E+00_JPRB, 0.74606E+00_JPRB ,&
&  0.76994E+00_JPRB, 0.77377E+00_JPRB, 0.76427E+00_JPRB, 0.74637E+00_JPRB, 0.72833E+00_JPRB, 0.71020E+00_JPRB, 0.68995E+00_JPRB ,&
&  0.66843E+00_JPRB, 0.65166E+00_JPRB, 0.63753E+00_JPRB, 0.62666E+00_JPRB, 0.61726E+00_JPRB, 0.60894E+00_JPRB, 0.60298E+00_JPRB ,&
&  0.59930E+00_JPRB, 0.59601E+00_JPRB, 0.59103E+00_JPRB, 0.58549E+00_JPRB, 0.58148E+00_JPRB, 0.57636E+00_JPRB, 0.57346E+00_JPRB ,&
&  0.57495E+00_JPRB, 0.58097E+00_JPRB, 0.59204E+00_JPRB, 0.61006E+00_JPRB, 0.62946E+00_JPRB, 0.64710E+00_JPRB, 0.66189E+00_JPRB ,&
&  0.67317E+00_JPRB, 0.67886E+00_JPRB, 0.67795E+00_JPRB, 0.67246E+00_JPRB, 0.66263E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.59451E+00_JPRB, 0.58733E+00_JPRB, 0.57830E+00_JPRB, 0.56893E+00_JPRB, 0.55989E+00_JPRB, 0.55092E+00_JPRB, 0.54273E+00_JPRB ,&
&  0.53590E+00_JPRB, 0.53166E+00_JPRB, 0.52908E+00_JPRB, 0.52718E+00_JPRB, 0.52352E+00_JPRB, 0.52189E+00_JPRB, 0.52262E+00_JPRB ,&
&  0.52529E+00_JPRB, 0.52801E+00_JPRB, 0.52931E+00_JPRB, 0.52798E+00_JPRB, 0.52978E+00_JPRB, 0.53138E+00_JPRB, 0.53095E+00_JPRB ,&
&  0.52921E+00_JPRB, 0.52720E+00_JPRB, 0.52379E+00_JPRB, 0.52020E+00_JPRB, 0.51980E+00_JPRB, 0.52073E+00_JPRB, 0.51812E+00_JPRB ,&
&  0.51219E+00_JPRB, 0.52469E+00_JPRB, 0.51803E+00_JPRB, 0.54349E+00_JPRB, 0.55147E+00_JPRB, 0.54426E+00_JPRB, 0.53306E+00_JPRB ,&
&  0.52528E+00_JPRB, 0.52230E+00_JPRB, 0.52434E+00_JPRB, 0.52697E+00_JPRB, 0.52884E+00_JPRB, 0.53010E+00_JPRB, 0.53004E+00_JPRB ,&
&  0.52102E+00_JPRB, 0.51444E+00_JPRB, 0.51189E+00_JPRB, 0.51123E+00_JPRB, 0.51041E+00_JPRB, 0.51055E+00_JPRB, 0.51233E+00_JPRB ,&
&  0.51399E+00_JPRB, 0.51517E+00_JPRB, 0.51625E+00_JPRB, 0.51845E+00_JPRB, 0.51970E+00_JPRB, 0.52009E+00_JPRB, 0.52185E+00_JPRB ,&
&  0.52520E+00_JPRB, 0.53085E+00_JPRB, 0.53806E+00_JPRB, 0.54806E+00_JPRB, 0.56906E+00_JPRB, 0.60024E+00_JPRB, 0.62888E+00_JPRB ,&
&  0.64573E+00_JPRB, 0.65220E+00_JPRB, 0.65201E+00_JPRB, 0.64460E+00_JPRB, 0.63252E+00_JPRB, 0.62131E+00_JPRB, 0.61078E+00_JPRB ,&
&  0.59772E+00_JPRB, 0.58809E+00_JPRB, 0.57931E+00_JPRB, 0.56909E+00_JPRB, 0.55989E+00_JPRB, 0.55311E+00_JPRB, 0.54863E+00_JPRB ,&
&  0.54683E+00_JPRB, 0.54733E+00_JPRB, 0.54656E+00_JPRB, 0.54642E+00_JPRB, 0.54832E+00_JPRB, 0.55080E+00_JPRB, 0.54993E+00_JPRB ,&
&  0.54800E+00_JPRB, 0.54761E+00_JPRB, 0.55021E+00_JPRB, 0.55602E+00_JPRB, 0.56307E+00_JPRB, 0.57199E+00_JPRB, 0.58162E+00_JPRB ,&
&  0.59073E+00_JPRB, 0.59762E+00_JPRB, 0.60136E+00_JPRB, 0.60266E+00_JPRB, 0.59998E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.53950E+00_JPRB, 0.53747E+00_JPRB, 0.53451E+00_JPRB, 0.52990E+00_JPRB, 0.52482E+00_JPRB, 0.52031E+00_JPRB, 0.51671E+00_JPRB ,&
&  0.51442E+00_JPRB, 0.51373E+00_JPRB, 0.51423E+00_JPRB, 0.51447E+00_JPRB, 0.51442E+00_JPRB, 0.51540E+00_JPRB, 0.51591E+00_JPRB ,&
&  0.51766E+00_JPRB, 0.51790E+00_JPRB, 0.51481E+00_JPRB, 0.51364E+00_JPRB, 0.51583E+00_JPRB, 0.52036E+00_JPRB, 0.52152E+00_JPRB ,&
&  0.51851E+00_JPRB, 0.51741E+00_JPRB, 0.51623E+00_JPRB, 0.51316E+00_JPRB, 0.51102E+00_JPRB, 0.51031E+00_JPRB, 0.50714E+00_JPRB ,&
&  0.50200E+00_JPRB, 0.50661E+00_JPRB, 0.50865E+00_JPRB, 0.51861E+00_JPRB, 0.52382E+00_JPRB, 0.52620E+00_JPRB, 0.52738E+00_JPRB ,&
&  0.52501E+00_JPRB, 0.52326E+00_JPRB, 0.52188E+00_JPRB, 0.52148E+00_JPRB, 0.51884E+00_JPRB, 0.51464E+00_JPRB, 0.50684E+00_JPRB ,&
&  0.49367E+00_JPRB, 0.48770E+00_JPRB, 0.48866E+00_JPRB, 0.49283E+00_JPRB, 0.49180E+00_JPRB, 0.49186E+00_JPRB, 0.49487E+00_JPRB ,&
&  0.49895E+00_JPRB, 0.50206E+00_JPRB, 0.50396E+00_JPRB, 0.50504E+00_JPRB, 0.50648E+00_JPRB, 0.50785E+00_JPRB, 0.50850E+00_JPRB ,&
&  0.50947E+00_JPRB, 0.51153E+00_JPRB, 0.51494E+00_JPRB, 0.51991E+00_JPRB, 0.52788E+00_JPRB, 0.54174E+00_JPRB, 0.55645E+00_JPRB ,&
&  0.56634E+00_JPRB, 0.56975E+00_JPRB, 0.56985E+00_JPRB, 0.56608E+00_JPRB, 0.55854E+00_JPRB, 0.55074E+00_JPRB, 0.54299E+00_JPRB ,&
&  0.53284E+00_JPRB, 0.52405E+00_JPRB, 0.51707E+00_JPRB, 0.51293E+00_JPRB, 0.51056E+00_JPRB, 0.50965E+00_JPRB, 0.51002E+00_JPRB ,&
&  0.51074E+00_JPRB, 0.51354E+00_JPRB, 0.51989E+00_JPRB, 0.52339E+00_JPRB, 0.52478E+00_JPRB, 0.52784E+00_JPRB, 0.52935E+00_JPRB ,&
&  0.52739E+00_JPRB, 0.52565E+00_JPRB, 0.52478E+00_JPRB, 0.52730E+00_JPRB, 0.52931E+00_JPRB, 0.52937E+00_JPRB, 0.53011E+00_JPRB ,&
&  0.53361E+00_JPRB, 0.53754E+00_JPRB, 0.53895E+00_JPRB, 0.53976E+00_JPRB, 0.54060E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.49545E+00_JPRB, 0.49565E+00_JPRB, 0.49499E+00_JPRB, 0.49375E+00_JPRB, 0.49248E+00_JPRB, 0.49152E+00_JPRB, 0.49166E+00_JPRB ,&
&  0.49201E+00_JPRB, 0.49294E+00_JPRB, 0.49424E+00_JPRB, 0.49531E+00_JPRB, 0.49730E+00_JPRB, 0.49962E+00_JPRB, 0.50167E+00_JPRB ,&
&  0.50371E+00_JPRB, 0.50299E+00_JPRB, 0.49919E+00_JPRB, 0.49557E+00_JPRB, 0.49623E+00_JPRB, 0.50185E+00_JPRB, 0.50790E+00_JPRB ,&
&  0.50893E+00_JPRB, 0.50941E+00_JPRB, 0.50916E+00_JPRB, 0.50721E+00_JPRB, 0.50495E+00_JPRB, 0.50460E+00_JPRB, 0.50145E+00_JPRB ,&
&  0.49669E+00_JPRB, 0.49679E+00_JPRB, 0.49609E+00_JPRB, 0.49650E+00_JPRB, 0.50010E+00_JPRB, 0.50705E+00_JPRB, 0.51006E+00_JPRB ,&
&  0.51118E+00_JPRB, 0.50934E+00_JPRB, 0.50494E+00_JPRB, 0.49998E+00_JPRB, 0.49374E+00_JPRB, 0.48354E+00_JPRB, 0.47205E+00_JPRB ,&
&  0.46293E+00_JPRB, 0.46188E+00_JPRB, 0.46745E+00_JPRB, 0.47306E+00_JPRB, 0.47100E+00_JPRB, 0.46885E+00_JPRB, 0.47000E+00_JPRB ,&
&  0.47370E+00_JPRB, 0.47877E+00_JPRB, 0.48209E+00_JPRB, 0.48148E+00_JPRB, 0.48032E+00_JPRB, 0.48225E+00_JPRB, 0.48502E+00_JPRB ,&
&  0.48621E+00_JPRB, 0.48746E+00_JPRB, 0.48844E+00_JPRB, 0.49038E+00_JPRB, 0.49242E+00_JPRB, 0.49507E+00_JPRB, 0.50026E+00_JPRB ,&
&  0.50382E+00_JPRB, 0.50491E+00_JPRB, 0.50237E+00_JPRB, 0.49681E+00_JPRB, 0.49106E+00_JPRB, 0.48582E+00_JPRB, 0.48311E+00_JPRB ,&
&  0.47984E+00_JPRB, 0.47923E+00_JPRB, 0.47818E+00_JPRB, 0.47517E+00_JPRB, 0.47185E+00_JPRB, 0.47171E+00_JPRB, 0.47538E+00_JPRB ,&
&  0.48050E+00_JPRB, 0.48499E+00_JPRB, 0.49148E+00_JPRB, 0.49424E+00_JPRB, 0.49597E+00_JPRB, 0.49858E+00_JPRB, 0.49948E+00_JPRB ,&
&  0.49682E+00_JPRB, 0.49443E+00_JPRB, 0.49380E+00_JPRB, 0.49674E+00_JPRB, 0.49775E+00_JPRB, 0.49665E+00_JPRB, 0.49319E+00_JPRB ,&
&  0.49175E+00_JPRB, 0.49329E+00_JPRB, 0.49373E+00_JPRB, 0.49321E+00_JPRB, 0.49390E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.46601E+00_JPRB, 0.46837E+00_JPRB, 0.47094E+00_JPRB, 0.47327E+00_JPRB, 0.47471E+00_JPRB, 0.47567E+00_JPRB, 0.47482E+00_JPRB ,&
&  0.47478E+00_JPRB, 0.47664E+00_JPRB, 0.47906E+00_JPRB, 0.48113E+00_JPRB, 0.48253E+00_JPRB, 0.48499E+00_JPRB, 0.48763E+00_JPRB ,&
&  0.48946E+00_JPRB, 0.48839E+00_JPRB, 0.48384E+00_JPRB, 0.48110E+00_JPRB, 0.47977E+00_JPRB, 0.48531E+00_JPRB, 0.49444E+00_JPRB ,&
&  0.49843E+00_JPRB, 0.50066E+00_JPRB, 0.50326E+00_JPRB, 0.50341E+00_JPRB, 0.50326E+00_JPRB, 0.50318E+00_JPRB, 0.49883E+00_JPRB ,&
&  0.49362E+00_JPRB, 0.48948E+00_JPRB, 0.48583E+00_JPRB, 0.48296E+00_JPRB, 0.48474E+00_JPRB, 0.48503E+00_JPRB, 0.48571E+00_JPRB ,&
&  0.48606E+00_JPRB, 0.48362E+00_JPRB, 0.47873E+00_JPRB, 0.47095E+00_JPRB, 0.46161E+00_JPRB, 0.44960E+00_JPRB, 0.44087E+00_JPRB ,&
&  0.43707E+00_JPRB, 0.44096E+00_JPRB, 0.45013E+00_JPRB, 0.45360E+00_JPRB, 0.44902E+00_JPRB, 0.44697E+00_JPRB, 0.44974E+00_JPRB ,&
&  0.45479E+00_JPRB, 0.46082E+00_JPRB, 0.46348E+00_JPRB, 0.46281E+00_JPRB, 0.46063E+00_JPRB, 0.46229E+00_JPRB, 0.46695E+00_JPRB ,&
&  0.47084E+00_JPRB, 0.47264E+00_JPRB, 0.47148E+00_JPRB, 0.47030E+00_JPRB, 0.46791E+00_JPRB, 0.46333E+00_JPRB, 0.45749E+00_JPRB ,&
&  0.45208E+00_JPRB, 0.44874E+00_JPRB, 0.44607E+00_JPRB, 0.44310E+00_JPRB, 0.44172E+00_JPRB, 0.44174E+00_JPRB, 0.44372E+00_JPRB ,&
&  0.44430E+00_JPRB, 0.44592E+00_JPRB, 0.44592E+00_JPRB, 0.44171E+00_JPRB, 0.43848E+00_JPRB, 0.44037E+00_JPRB, 0.44240E+00_JPRB ,&
&  0.44557E+00_JPRB, 0.44879E+00_JPRB, 0.45183E+00_JPRB, 0.45278E+00_JPRB, 0.45424E+00_JPRB, 0.45727E+00_JPRB, 0.45925E+00_JPRB ,&
&  0.46030E+00_JPRB, 0.46098E+00_JPRB, 0.46296E+00_JPRB, 0.46647E+00_JPRB, 0.46996E+00_JPRB, 0.47285E+00_JPRB, 0.47244E+00_JPRB ,&
&  0.47086E+00_JPRB, 0.46868E+00_JPRB, 0.46845E+00_JPRB, 0.46811E+00_JPRB, 0.46613E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.45869E+00_JPRB, 0.45820E+00_JPRB, 0.46144E+00_JPRB, 0.46491E+00_JPRB, 0.46541E+00_JPRB, 0.46455E+00_JPRB, 0.46151E+00_JPRB ,&
&  0.45948E+00_JPRB, 0.46062E+00_JPRB, 0.46256E+00_JPRB, 0.46413E+00_JPRB, 0.46691E+00_JPRB, 0.47073E+00_JPRB, 0.47376E+00_JPRB ,&
&  0.47402E+00_JPRB, 0.47179E+00_JPRB, 0.46802E+00_JPRB, 0.46769E+00_JPRB, 0.46734E+00_JPRB, 0.46985E+00_JPRB, 0.47810E+00_JPRB ,&
&  0.48406E+00_JPRB, 0.48803E+00_JPRB, 0.49129E+00_JPRB, 0.49159E+00_JPRB, 0.49083E+00_JPRB, 0.49258E+00_JPRB, 0.48895E+00_JPRB ,&
&  0.48238E+00_JPRB, 0.47677E+00_JPRB, 0.47030E+00_JPRB, 0.46314E+00_JPRB, 0.46639E+00_JPRB, 0.46464E+00_JPRB, 0.46216E+00_JPRB ,&
&  0.45778E+00_JPRB, 0.45138E+00_JPRB, 0.44347E+00_JPRB, 0.43585E+00_JPRB, 0.42999E+00_JPRB, 0.42451E+00_JPRB, 0.41805E+00_JPRB ,&
&  0.41587E+00_JPRB, 0.41874E+00_JPRB, 0.42469E+00_JPRB, 0.42606E+00_JPRB, 0.42196E+00_JPRB, 0.42188E+00_JPRB, 0.42853E+00_JPRB ,&
&  0.43616E+00_JPRB, 0.44014E+00_JPRB, 0.44027E+00_JPRB, 0.44089E+00_JPRB, 0.44152E+00_JPRB, 0.44307E+00_JPRB, 0.44384E+00_JPRB ,&
&  0.44265E+00_JPRB, 0.44068E+00_JPRB, 0.43780E+00_JPRB, 0.43176E+00_JPRB, 0.42485E+00_JPRB, 0.41845E+00_JPRB, 0.41630E+00_JPRB ,&
&  0.41940E+00_JPRB, 0.42032E+00_JPRB, 0.42215E+00_JPRB, 0.42483E+00_JPRB, 0.42571E+00_JPRB, 0.42516E+00_JPRB, 0.42438E+00_JPRB ,&
&  0.42611E+00_JPRB, 0.42427E+00_JPRB, 0.42465E+00_JPRB, 0.42171E+00_JPRB, 0.41722E+00_JPRB, 0.41249E+00_JPRB, 0.40785E+00_JPRB ,&
&  0.40549E+00_JPRB, 0.40531E+00_JPRB, 0.40758E+00_JPRB, 0.40850E+00_JPRB, 0.41062E+00_JPRB, 0.41319E+00_JPRB, 0.41540E+00_JPRB ,&
&  0.41685E+00_JPRB, 0.41897E+00_JPRB, 0.42286E+00_JPRB, 0.42989E+00_JPRB, 0.43716E+00_JPRB, 0.44352E+00_JPRB, 0.44955E+00_JPRB ,&
&  0.45485E+00_JPRB, 0.45831E+00_JPRB, 0.46402E+00_JPRB, 0.46457E+00_JPRB, 0.46279E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.45640E+00_JPRB, 0.45291E+00_JPRB, 0.45229E+00_JPRB, 0.45250E+00_JPRB, 0.45096E+00_JPRB, 0.44905E+00_JPRB, 0.44557E+00_JPRB ,&
&  0.44270E+00_JPRB, 0.44296E+00_JPRB, 0.44447E+00_JPRB, 0.44740E+00_JPRB, 0.45254E+00_JPRB, 0.45798E+00_JPRB, 0.45786E+00_JPRB ,&
&  0.45494E+00_JPRB, 0.45389E+00_JPRB, 0.45632E+00_JPRB, 0.45789E+00_JPRB, 0.45774E+00_JPRB, 0.45847E+00_JPRB, 0.46257E+00_JPRB ,&
&  0.46754E+00_JPRB, 0.46990E+00_JPRB, 0.47305E+00_JPRB, 0.47260E+00_JPRB, 0.46856E+00_JPRB, 0.46904E+00_JPRB, 0.46715E+00_JPRB ,&
&  0.46210E+00_JPRB, 0.45537E+00_JPRB, 0.44544E+00_JPRB, 0.44585E+00_JPRB, 0.44206E+00_JPRB, 0.43881E+00_JPRB, 0.43370E+00_JPRB ,&
&  0.42787E+00_JPRB, 0.42251E+00_JPRB, 0.41667E+00_JPRB, 0.41204E+00_JPRB, 0.40994E+00_JPRB, 0.40979E+00_JPRB, 0.40750E+00_JPRB ,&
&  0.40642E+00_JPRB, 0.40935E+00_JPRB, 0.41054E+00_JPRB, 0.40536E+00_JPRB, 0.40335E+00_JPRB, 0.40897E+00_JPRB, 0.41446E+00_JPRB ,&
&  0.41728E+00_JPRB, 0.41765E+00_JPRB, 0.41469E+00_JPRB, 0.41386E+00_JPRB, 0.41284E+00_JPRB, 0.41084E+00_JPRB, 0.40710E+00_JPRB ,&
&  0.40116E+00_JPRB, 0.39814E+00_JPRB, 0.39607E+00_JPRB, 0.39202E+00_JPRB, 0.38384E+00_JPRB, 0.36838E+00_JPRB, 0.35994E+00_JPRB ,&
&  0.36207E+00_JPRB, 0.37174E+00_JPRB, 0.38528E+00_JPRB, 0.39408E+00_JPRB, 0.40303E+00_JPRB, 0.40890E+00_JPRB, 0.40888E+00_JPRB ,&
&  0.40700E+00_JPRB, 0.38776E+00_JPRB, 0.37695E+00_JPRB, 0.36682E+00_JPRB, 0.35786E+00_JPRB, 0.35286E+00_JPRB, 0.35081E+00_JPRB ,&
&  0.35387E+00_JPRB, 0.35953E+00_JPRB, 0.36573E+00_JPRB, 0.36926E+00_JPRB, 0.36956E+00_JPRB, 0.36748E+00_JPRB, 0.36449E+00_JPRB ,&
&  0.36571E+00_JPRB, 0.37468E+00_JPRB, 0.39043E+00_JPRB, 0.40302E+00_JPRB, 0.41347E+00_JPRB, 0.42489E+00_JPRB, 0.43412E+00_JPRB ,&
&  0.44065E+00_JPRB, 0.44683E+00_JPRB, 0.45390E+00_JPRB, 0.45628E+00_JPRB, 0.45690E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.43968E+00_JPRB, 0.43476E+00_JPRB, 0.43417E+00_JPRB, 0.43411E+00_JPRB, 0.43231E+00_JPRB, 0.43147E+00_JPRB, 0.43006E+00_JPRB ,&
&  0.42905E+00_JPRB, 0.42687E+00_JPRB, 0.42519E+00_JPRB, 0.42609E+00_JPRB, 0.42872E+00_JPRB, 0.43161E+00_JPRB, 0.43260E+00_JPRB ,&
&  0.43284E+00_JPRB, 0.43482E+00_JPRB, 0.43877E+00_JPRB, 0.44232E+00_JPRB, 0.44266E+00_JPRB, 0.44283E+00_JPRB, 0.44475E+00_JPRB ,&
&  0.44712E+00_JPRB, 0.44801E+00_JPRB, 0.45102E+00_JPRB, 0.44859E+00_JPRB, 0.44377E+00_JPRB, 0.44229E+00_JPRB, 0.44096E+00_JPRB ,&
&  0.43301E+00_JPRB, 0.40779E+00_JPRB, 0.39237E+00_JPRB, 0.39886E+00_JPRB, 0.41335E+00_JPRB, 0.41140E+00_JPRB, 0.40876E+00_JPRB ,&
&  0.40598E+00_JPRB, 0.40211E+00_JPRB, 0.39895E+00_JPRB, 0.39780E+00_JPRB, 0.39840E+00_JPRB, 0.39889E+00_JPRB, 0.39800E+00_JPRB ,&
&  0.39913E+00_JPRB, 0.40574E+00_JPRB, 0.41201E+00_JPRB, 0.40641E+00_JPRB, 0.39939E+00_JPRB, 0.39064E+00_JPRB, 0.38515E+00_JPRB ,&
&  0.38008E+00_JPRB, 0.37541E+00_JPRB, 0.37202E+00_JPRB, 0.37111E+00_JPRB, 0.37089E+00_JPRB, 0.36839E+00_JPRB, 0.36409E+00_JPRB ,&
&  0.35826E+00_JPRB, 0.34819E+00_JPRB, 0.33857E+00_JPRB, 0.32905E+00_JPRB, 0.32302E+00_JPRB, 0.31844E+00_JPRB, 0.31445E+00_JPRB ,&
&  0.31746E+00_JPRB, 0.33248E+00_JPRB, 0.35443E+00_JPRB, 0.37577E+00_JPRB, 0.38902E+00_JPRB, 0.36880E+00_JPRB, 0.35952E+00_JPRB ,&
&  0.34710E+00_JPRB, 0.33610E+00_JPRB, 0.32826E+00_JPRB, 0.32251E+00_JPRB, 0.31817E+00_JPRB, 0.31549E+00_JPRB, 0.31504E+00_JPRB ,&
&  0.31857E+00_JPRB, 0.32336E+00_JPRB, 0.32785E+00_JPRB, 0.32976E+00_JPRB, 0.32981E+00_JPRB, 0.32817E+00_JPRB, 0.32730E+00_JPRB ,&
&  0.32936E+00_JPRB, 0.33669E+00_JPRB, 0.34790E+00_JPRB, 0.36437E+00_JPRB, 0.37844E+00_JPRB, 0.39268E+00_JPRB, 0.40803E+00_JPRB ,&
&  0.42350E+00_JPRB, 0.43540E+00_JPRB, 0.43889E+00_JPRB, 0.44262E+00_JPRB, 0.44317E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.42707E+00_JPRB, 0.42474E+00_JPRB, 0.42256E+00_JPRB, 0.42273E+00_JPRB, 0.42270E+00_JPRB, 0.42467E+00_JPRB, 0.42288E+00_JPRB ,&
&  0.41793E+00_JPRB, 0.41323E+00_JPRB, 0.41212E+00_JPRB, 0.41256E+00_JPRB, 0.41162E+00_JPRB, 0.41001E+00_JPRB, 0.41114E+00_JPRB ,&
&  0.40967E+00_JPRB, 0.41240E+00_JPRB, 0.41608E+00_JPRB, 0.41806E+00_JPRB, 0.41866E+00_JPRB, 0.41866E+00_JPRB, 0.41906E+00_JPRB ,&
&  0.41258E+00_JPRB, 0.41019E+00_JPRB, 0.41614E+00_JPRB, 0.41286E+00_JPRB, 0.40843E+00_JPRB, 0.40462E+00_JPRB, 0.40324E+00_JPRB ,&
&  0.39334E+00_JPRB, 0.37985E+00_JPRB, 0.36764E+00_JPRB, 0.37630E+00_JPRB, 0.39317E+00_JPRB, 0.38824E+00_JPRB, 0.38584E+00_JPRB ,&
&  0.38574E+00_JPRB, 0.38501E+00_JPRB, 0.38611E+00_JPRB, 0.38688E+00_JPRB, 0.38865E+00_JPRB, 0.38940E+00_JPRB, 0.39162E+00_JPRB ,&
&  0.39376E+00_JPRB, 0.40191E+00_JPRB, 0.40459E+00_JPRB, 0.38135E+00_JPRB, 0.36291E+00_JPRB, 0.34889E+00_JPRB, 0.33515E+00_JPRB ,&
&  0.32291E+00_JPRB, 0.32229E+00_JPRB, 0.31806E+00_JPRB, 0.30902E+00_JPRB, 0.30928E+00_JPRB, 0.31032E+00_JPRB, 0.30921E+00_JPRB ,&
&  0.30629E+00_JPRB, 0.30459E+00_JPRB, 0.30313E+00_JPRB, 0.30058E+00_JPRB, 0.29982E+00_JPRB, 0.30163E+00_JPRB, 0.30650E+00_JPRB ,&
&  0.31373E+00_JPRB, 0.32097E+00_JPRB, 0.33750E+00_JPRB, 0.35092E+00_JPRB, 0.34489E+00_JPRB, 0.33948E+00_JPRB, 0.33290E+00_JPRB ,&
&  0.31931E+00_JPRB, 0.31294E+00_JPRB, 0.30646E+00_JPRB, 0.30114E+00_JPRB, 0.29863E+00_JPRB, 0.29861E+00_JPRB, 0.29915E+00_JPRB ,&
&  0.30026E+00_JPRB, 0.30274E+00_JPRB, 0.30371E+00_JPRB, 0.30348E+00_JPRB, 0.30277E+00_JPRB, 0.30383E+00_JPRB, 0.30624E+00_JPRB ,&
&  0.31027E+00_JPRB, 0.31699E+00_JPRB, 0.32484E+00_JPRB, 0.33470E+00_JPRB, 0.34352E+00_JPRB, 0.35181E+00_JPRB, 0.36515E+00_JPRB ,&
&  0.37388E+00_JPRB, 0.39551E+00_JPRB, 0.42260E+00_JPRB, 0.44169E+00_JPRB, 0.43148E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.40414E+00_JPRB, 0.40208E+00_JPRB, 0.39930E+00_JPRB, 0.39882E+00_JPRB, 0.39975E+00_JPRB, 0.40189E+00_JPRB, 0.39897E+00_JPRB ,&
&  0.39377E+00_JPRB, 0.39156E+00_JPRB, 0.39270E+00_JPRB, 0.39155E+00_JPRB, 0.39170E+00_JPRB, 0.38247E+00_JPRB, 0.37725E+00_JPRB ,&
&  0.37009E+00_JPRB, 0.36846E+00_JPRB, 0.36696E+00_JPRB, 0.36820E+00_JPRB, 0.37006E+00_JPRB, 0.37111E+00_JPRB, 0.36639E+00_JPRB ,&
&  0.36170E+00_JPRB, 0.35761E+00_JPRB, 0.35294E+00_JPRB, 0.35093E+00_JPRB, 0.35224E+00_JPRB, 0.35326E+00_JPRB, 0.35299E+00_JPRB ,&
&  0.35379E+00_JPRB, 0.35706E+00_JPRB, 0.36036E+00_JPRB, 0.36455E+00_JPRB, 0.37742E+00_JPRB, 0.37412E+00_JPRB, 0.37286E+00_JPRB ,&
&  0.37366E+00_JPRB, 0.37464E+00_JPRB, 0.37641E+00_JPRB, 0.37792E+00_JPRB, 0.38364E+00_JPRB, 0.38604E+00_JPRB, 0.38816E+00_JPRB ,&
&  0.37515E+00_JPRB, 0.36944E+00_JPRB, 0.36251E+00_JPRB, 0.35271E+00_JPRB, 0.33853E+00_JPRB, 0.32763E+00_JPRB, 0.31776E+00_JPRB ,&
&  0.30956E+00_JPRB, 0.30184E+00_JPRB, 0.29437E+00_JPRB, 0.29154E+00_JPRB, 0.29042E+00_JPRB, 0.28857E+00_JPRB, 0.28822E+00_JPRB ,&
&  0.28881E+00_JPRB, 0.28973E+00_JPRB, 0.29023E+00_JPRB, 0.29206E+00_JPRB, 0.29511E+00_JPRB, 0.30000E+00_JPRB, 0.30519E+00_JPRB ,&
&  0.31275E+00_JPRB, 0.31772E+00_JPRB, 0.32380E+00_JPRB, 0.32973E+00_JPRB, 0.32823E+00_JPRB, 0.32229E+00_JPRB, 0.31622E+00_JPRB ,&
&  0.30793E+00_JPRB, 0.30322E+00_JPRB, 0.29828E+00_JPRB, 0.29466E+00_JPRB, 0.29253E+00_JPRB, 0.29112E+00_JPRB, 0.29112E+00_JPRB ,&
&  0.29084E+00_JPRB, 0.29117E+00_JPRB, 0.29038E+00_JPRB, 0.29067E+00_JPRB, 0.29171E+00_JPRB, 0.29407E+00_JPRB, 0.29720E+00_JPRB ,&
&  0.30209E+00_JPRB, 0.30772E+00_JPRB, 0.31424E+00_JPRB, 0.32125E+00_JPRB, 0.33202E+00_JPRB, 0.34532E+00_JPRB, 0.36334E+00_JPRB ,&
&  0.37905E+00_JPRB, 0.40348E+00_JPRB, 0.41253E+00_JPRB, 0.41001E+00_JPRB, 0.40832E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.38613E+00_JPRB, 0.38782E+00_JPRB, 0.38856E+00_JPRB, 0.39070E+00_JPRB, 0.39322E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.38412E+00_JPRB, 0.37707E+00_JPRB, 0.37018E+00_JPRB, 0.36011E+00_JPRB, 0.35212E+00_JPRB, 0.34519E+00_JPRB, 0.34024E+00_JPRB ,&
&  0.33428E+00_JPRB, 0.33177E+00_JPRB, 0.32815E+00_JPRB, 0.32714E+00_JPRB, 0.32612E+00_JPRB, 0.32629E+00_JPRB, 0.32427E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32396E+00_JPRB, 0.32488E+00_JPRB, 0.32937E+00_JPRB, 0.33628E+00_JPRB, 0.34394E+00_JPRB, 0.35150E+00_JPRB ,&
&  0.36120E+00_JPRB, 0.36496E+00_JPRB, 0.36827E+00_JPRB, 0.36909E+00_JPRB, 0.36835E+00_JPRB, 0.36850E+00_JPRB, 0.36698E+00_JPRB ,&
&  0.36763E+00_JPRB, 0.36789E+00_JPRB, 0.37031E+00_JPRB, 0.37190E+00_JPRB, 0.37007E+00_JPRB, 0.36891E+00_JPRB, 0.36446E+00_JPRB ,&
&  0.35683E+00_JPRB, 0.35090E+00_JPRB, 0.34397E+00_JPRB, 0.33838E+00_JPRB, 0.32932E+00_JPRB, 0.32156E+00_JPRB, 0.31406E+00_JPRB ,&
&  0.30780E+00_JPRB, 0.30331E+00_JPRB, 0.29778E+00_JPRB, 0.29329E+00_JPRB, 0.29111E+00_JPRB, 0.28837E+00_JPRB, 0.28718E+00_JPRB ,&
&  0.28718E+00_JPRB, 0.28774E+00_JPRB, 0.29059E+00_JPRB, 0.29279E+00_JPRB, 0.29621E+00_JPRB, 0.30022E+00_JPRB, 0.30242E+00_JPRB ,&
&  0.30662E+00_JPRB, 0.31026E+00_JPRB, 0.31227E+00_JPRB, 0.31391E+00_JPRB, 0.31291E+00_JPRB, 0.30910E+00_JPRB, 0.30546E+00_JPRB ,&
&  0.30039E+00_JPRB, 0.29695E+00_JPRB, 0.29475E+00_JPRB, 0.29228E+00_JPRB, 0.29092E+00_JPRB, 0.28966E+00_JPRB, 0.28817E+00_JPRB ,&
&  0.28702E+00_JPRB, 0.28643E+00_JPRB, 0.28591E+00_JPRB, 0.28732E+00_JPRB, 0.28874E+00_JPRB, 0.29141E+00_JPRB, 0.29461E+00_JPRB ,&
&  0.29744E+00_JPRB, 0.30189E+00_JPRB, 0.30670E+00_JPRB, 0.31335E+00_JPRB, 0.32610E+00_JPRB, 0.33536E+00_JPRB, 0.35591E+00_JPRB ,&
&  0.36592E+00_JPRB, 0.38352E+00_JPRB, 0.39169E+00_JPRB, 0.38495E+00_JPRB, 0.38547E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.37407E+00_JPRB, 0.37652E+00_JPRB, 0.37688E+00_JPRB, 0.37872E+00_JPRB, 0.38008E+00_JPRB, 0.38035E+00_JPRB, 0.37931E+00_JPRB ,&
&  0.37738E+00_JPRB, 0.37261E+00_JPRB, 0.36833E+00_JPRB, 0.36150E+00_JPRB, 0.35551E+00_JPRB, 0.34859E+00_JPRB, 0.34271E+00_JPRB ,&
&  0.33600E+00_JPRB, 0.32980E+00_JPRB, 0.32327E+00_JPRB, 0.31767E+00_JPRB, 0.31334E+00_JPRB, 0.30974E+00_JPRB, 0.30633E+00_JPRB ,&
&  0.30342E+00_JPRB, 0.30523E+00_JPRB, 0.30852E+00_JPRB, 0.31442E+00_JPRB, 0.32338E+00_JPRB, 0.33109E+00_JPRB, 0.33962E+00_JPRB ,&
&  0.35335E+00_JPRB, 0.35531E+00_JPRB, 0.35564E+00_JPRB, 0.35841E+00_JPRB, 0.35826E+00_JPRB, 0.35899E+00_JPRB, 0.34974E+00_JPRB ,&
&  0.35470E+00_JPRB, 0.35719E+00_JPRB, 0.36660E+00_JPRB, 0.36925E+00_JPRB, 0.36290E+00_JPRB, 0.35917E+00_JPRB, 0.35145E+00_JPRB ,&
&  0.34627E+00_JPRB, 0.34174E+00_JPRB, 0.33873E+00_JPRB, 0.33703E+00_JPRB, 0.33079E+00_JPRB, 0.32617E+00_JPRB, 0.31841E+00_JPRB ,&
&  0.31230E+00_JPRB, 0.30857E+00_JPRB, 0.30378E+00_JPRB, 0.30100E+00_JPRB, 0.29866E+00_JPRB, 0.29624E+00_JPRB, 0.29438E+00_JPRB ,&
&  0.29336E+00_JPRB, 0.29250E+00_JPRB, 0.29338E+00_JPRB, 0.29433E+00_JPRB, 0.29619E+00_JPRB, 0.29768E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.29812E+00_JPRB, 0.29872E+00_JPRB, 0.29903E+00_JPRB, 0.30005E+00_JPRB, 0.29947E+00_JPRB, 0.29822E+00_JPRB, 0.29671E+00_JPRB ,&
&  0.29493E+00_JPRB, 0.29412E+00_JPRB, 0.29272E+00_JPRB, 0.29180E+00_JPRB, 0.28922E+00_JPRB, 0.28749E+00_JPRB, 0.28658E+00_JPRB ,&
&  0.28577E+00_JPRB, 0.28638E+00_JPRB, 0.28655E+00_JPRB, 0.28766E+00_JPRB, 0.28933E+00_JPRB, 0.29069E+00_JPRB, 0.29322E+00_JPRB ,&
&  0.29634E+00_JPRB, 0.29995E+00_JPRB, 0.30644E+00_JPRB, 0.31286E+00_JPRB, 0.31995E+00_JPRB, 0.32749E+00_JPRB, 0.33251E+00_JPRB ,&
&  0.34030E+00_JPRB, 0.35272E+00_JPRB, 0.35453E+00_JPRB, 0.36296E+00_JPRB, 0.36975E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.35506E+00_JPRB, 0.36044E+00_JPRB, 0.36383E+00_JPRB, 0.36604E+00_JPRB, 0.36835E+00_JPRB, 0.36753E+00_JPRB, 0.36564E+00_JPRB ,&
&  0.36093E+00_JPRB, 0.35937E+00_JPRB, 0.35639E+00_JPRB, 0.35288E+00_JPRB, 0.34892E+00_JPRB, 0.34349E+00_JPRB, 0.33939E+00_JPRB ,&
&  0.33463E+00_JPRB, 0.32933E+00_JPRB, 0.32329E+00_JPRB, 0.31625E+00_JPRB, 0.30983E+00_JPRB, 0.30372E+00_JPRB, 0.30090E+00_JPRB ,&
&  0.29973E+00_JPRB, 0.30108E+00_JPRB, 0.30480E+00_JPRB, 0.30888E+00_JPRB, 0.31377E+00_JPRB, 0.31924E+00_JPRB, 0.32603E+00_JPRB ,&
&  0.33160E+00_JPRB, 0.33810E+00_JPRB, 0.34198E+00_JPRB, 0.34347E+00_JPRB, 0.34206E+00_JPRB, 0.34225E+00_JPRB, 0.34150E+00_JPRB ,&
&  0.34053E+00_JPRB, 0.33929E+00_JPRB, 0.33945E+00_JPRB, 0.34057E+00_JPRB, 0.34064E+00_JPRB, 0.33951E+00_JPRB, 0.33724E+00_JPRB ,&
&  0.33481E+00_JPRB, 0.33273E+00_JPRB, 0.32957E+00_JPRB, 0.32686E+00_JPRB, 0.32488E+00_JPRB, 0.32217E+00_JPRB, 0.31832E+00_JPRB ,&
&  0.31498E+00_JPRB, 0.31171E+00_JPRB, 0.30792E+00_JPRB, 0.30489E+00_JPRB, 0.30189E+00_JPRB, 0.29976E+00_JPRB, 0.29782E+00_JPRB ,&
&  0.29712E+00_JPRB, 0.29629E+00_JPRB, 0.29612E+00_JPRB, 0.29606E+00_JPRB, 0.29666E+00_JPRB, 0.29626E+00_JPRB, 0.29564E+00_JPRB ,&
&  0.29493E+00_JPRB, 0.29474E+00_JPRB, 0.29401E+00_JPRB, 0.29373E+00_JPRB, 0.29343E+00_JPRB, 0.29238E+00_JPRB, 0.29176E+00_JPRB ,&
&  0.29156E+00_JPRB, 0.29119E+00_JPRB, 0.29018E+00_JPRB, 0.28989E+00_JPRB, 0.28921E+00_JPRB, 0.28833E+00_JPRB, 0.28862E+00_JPRB ,&
&  0.28834E+00_JPRB, 0.28857E+00_JPRB, 0.28895E+00_JPRB, 0.28950E+00_JPRB, 0.29082E+00_JPRB, 0.29244E+00_JPRB, 0.29463E+00_JPRB ,&
&  0.29808E+00_JPRB, 0.30113E+00_JPRB, 0.30412E+00_JPRB, 0.30751E+00_JPRB, 0.31456E+00_JPRB, 0.31645E+00_JPRB, 0.32035E+00_JPRB ,&
&  0.32328E+00_JPRB, 0.31687E+00_JPRB, 0.34109E+00_JPRB, 0.34259E+00_JPRB, 0.34640E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.30943E+00_JPRB, 0.31466E+00_JPRB, 0.31785E+00_JPRB, 0.31981E+00_JPRB, 0.31217E+00_JPRB, 0.33331E+00_JPRB, 0.33168E+00_JPRB ,&
&  0.33276E+00_JPRB, 0.32940E+00_JPRB, 0.33220E+00_JPRB, 0.33168E+00_JPRB, 0.32962E+00_JPRB, 0.32668E+00_JPRB, 0.32366E+00_JPRB ,&
&  0.32044E+00_JPRB, 0.31644E+00_JPRB, 0.31293E+00_JPRB, 0.30942E+00_JPRB, 0.30653E+00_JPRB, 0.30391E+00_JPRB, 0.30259E+00_JPRB ,&
&  0.30283E+00_JPRB, 0.30468E+00_JPRB, 0.30783E+00_JPRB, 0.30913E+00_JPRB, 0.31060E+00_JPRB, 0.31219E+00_JPRB, 0.31395E+00_JPRB ,&
&  0.31521E+00_JPRB, 0.31715E+00_JPRB, 0.31810E+00_JPRB, 0.31751E+00_JPRB, 0.31691E+00_JPRB, 0.31720E+00_JPRB, 0.31780E+00_JPRB ,&
&  0.31913E+00_JPRB, 0.32011E+00_JPRB, 0.32082E+00_JPRB, 0.32160E+00_JPRB, 0.32237E+00_JPRB, 0.32220E+00_JPRB, 0.32102E+00_JPRB ,&
&  0.32000E+00_JPRB, 0.31915E+00_JPRB, 0.31709E+00_JPRB, 0.31515E+00_JPRB, 0.31331E+00_JPRB, 0.31138E+00_JPRB, 0.30932E+00_JPRB ,&
&  0.30788E+00_JPRB, 0.30643E+00_JPRB, 0.30461E+00_JPRB, 0.30252E+00_JPRB, 0.30063E+00_JPRB, 0.29898E+00_JPRB, 0.29774E+00_JPRB ,&
&  0.29760E+00_JPRB, 0.29693E+00_JPRB, 0.29635E+00_JPRB, 0.29577E+00_JPRB, 0.29566E+00_JPRB, 0.29506E+00_JPRB, 0.29460E+00_JPRB ,&
&  0.29423E+00_JPRB, 0.29374E+00_JPRB, 0.29316E+00_JPRB, 0.29258E+00_JPRB, 0.29218E+00_JPRB, 0.29167E+00_JPRB, 0.29113E+00_JPRB ,&
&  0.29075E+00_JPRB, 0.29046E+00_JPRB, 0.29015E+00_JPRB, 0.28954E+00_JPRB, 0.28895E+00_JPRB, 0.28825E+00_JPRB, 0.28817E+00_JPRB ,&
&  0.28828E+00_JPRB, 0.28855E+00_JPRB, 0.28907E+00_JPRB, 0.28916E+00_JPRB, 0.29021E+00_JPRB, 0.29122E+00_JPRB, 0.29243E+00_JPRB ,&
&  0.29365E+00_JPRB, 0.29561E+00_JPRB, 0.29781E+00_JPRB, 0.30024E+00_JPRB, 0.30430E+00_JPRB, 0.30519E+00_JPRB, 0.30585E+00_JPRB ,&
&  0.30674E+00_JPRB, 0.31119E+00_JPRB, 0.31153E+00_JPRB, 0.31052E+00_JPRB, 0.31046E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.30031E+00_JPRB, 0.30055E+00_JPRB, 0.30039E+00_JPRB, 0.29998E+00_JPRB, 0.29700E+00_JPRB, 0.29564E+00_JPRB, 0.29448E+00_JPRB ,&
&  0.29359E+00_JPRB, 0.29317E+00_JPRB, 0.29369E+00_JPRB, 0.29439E+00_JPRB, 0.29512E+00_JPRB, 0.29591E+00_JPRB, 0.29664E+00_JPRB ,&
&  0.29705E+00_JPRB, 0.29750E+00_JPRB, 0.29612E+00_JPRB, 0.29543E+00_JPRB, 0.29519E+00_JPRB, 0.29521E+00_JPRB, 0.29518E+00_JPRB ,&
&  0.29526E+00_JPRB, 0.29531E+00_JPRB, 0.29525E+00_JPRB, 0.29571E+00_JPRB, 0.29649E+00_JPRB, 0.29711E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29841E+00_JPRB, 0.29937E+00_JPRB, 0.30026E+00_JPRB, 0.30112E+00_JPRB, 0.30234E+00_JPRB, 0.30360E+00_JPRB, 0.30442E+00_JPRB ,&
&  0.30510E+00_JPRB, 0.30545E+00_JPRB, 0.30584E+00_JPRB, 0.30616E+00_JPRB, 0.30661E+00_JPRB, 0.30714E+00_JPRB, 0.30731E+00_JPRB ,&
&  0.30731E+00_JPRB, 0.30721E+00_JPRB, 0.30699E+00_JPRB, 0.30633E+00_JPRB, 0.30562E+00_JPRB, 0.30493E+00_JPRB, 0.30423E+00_JPRB ,&
&  0.30363E+00_JPRB, 0.30293E+00_JPRB, 0.30215E+00_JPRB, 0.30134E+00_JPRB, 0.30048E+00_JPRB, 0.29965E+00_JPRB, 0.29880E+00_JPRB ,&
&  0.29793E+00_JPRB, 0.29739E+00_JPRB, 0.29678E+00_JPRB, 0.29625E+00_JPRB, 0.29581E+00_JPRB, 0.29536E+00_JPRB, 0.29492E+00_JPRB ,&
&  0.29449E+00_JPRB, 0.29398E+00_JPRB, 0.29351E+00_JPRB, 0.29299E+00_JPRB, 0.29253E+00_JPRB, 0.29201E+00_JPRB, 0.29158E+00_JPRB ,&
&  0.29117E+00_JPRB, 0.29080E+00_JPRB, 0.29049E+00_JPRB, 0.29034E+00_JPRB, 0.29024E+00_JPRB, 0.29026E+00_JPRB, 0.29028E+00_JPRB ,&
&  0.29032E+00_JPRB, 0.29034E+00_JPRB, 0.29034E+00_JPRB, 0.29012E+00_JPRB, 0.29038E+00_JPRB, 0.29064E+00_JPRB, 0.29103E+00_JPRB ,&
&  0.29139E+00_JPRB, 0.29202E+00_JPRB, 0.29267E+00_JPRB, 0.29342E+00_JPRB, 0.29464E+00_JPRB, 0.29507E+00_JPRB, 0.29543E+00_JPRB ,&
&  0.29579E+00_JPRB, 0.29659E+00_JPRB, 0.29721E+00_JPRB, 0.29753E+00_JPRB, 0.29826E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB, 0.29505E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB ,&
&  0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB, 0.87925E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.85450E+00_JPRB, 0.85088E+00_JPRB, 0.84704E+00_JPRB, 0.84303E+00_JPRB, 0.83867E+00_JPRB, 0.83368E+00_JPRB, 0.82811E+00_JPRB ,&
&  0.82306E+00_JPRB, 0.81841E+00_JPRB, 0.81422E+00_JPRB, 0.81048E+00_JPRB, 0.80730E+00_JPRB, 0.80485E+00_JPRB, 0.80403E+00_JPRB ,&
&  0.80408E+00_JPRB, 0.80428E+00_JPRB, 0.80479E+00_JPRB, 0.80517E+00_JPRB, 0.80513E+00_JPRB, 0.80489E+00_JPRB, 0.80449E+00_JPRB ,&
&  0.80430E+00_JPRB, 0.80423E+00_JPRB, 0.80424E+00_JPRB, 0.80469E+00_JPRB, 0.80522E+00_JPRB, 0.80580E+00_JPRB, 0.80628E+00_JPRB ,&
&  0.80726E+00_JPRB, 0.80850E+00_JPRB, 0.81024E+00_JPRB, 0.81245E+00_JPRB, 0.81701E+00_JPRB, 0.82184E+00_JPRB, 0.82688E+00_JPRB ,&
&  0.83161E+00_JPRB, 0.83631E+00_JPRB, 0.84045E+00_JPRB, 0.84410E+00_JPRB, 0.84750E+00_JPRB, 0.85175E+00_JPRB, 0.85663E+00_JPRB ,&
&  0.86190E+00_JPRB, 0.86768E+00_JPRB, 0.87428E+00_JPRB, 0.88083E+00_JPRB, 0.88680E+00_JPRB, 0.89289E+00_JPRB, 0.89934E+00_JPRB ,&
&  0.90562E+00_JPRB, 0.91187E+00_JPRB, 0.91808E+00_JPRB, 0.92444E+00_JPRB, 0.93066E+00_JPRB, 0.93789E+00_JPRB, 0.94639E+00_JPRB ,&
&  0.95584E+00_JPRB, 0.96332E+00_JPRB, 0.97186E+00_JPRB, 0.98104E+00_JPRB, 0.99438E+00_JPRB, 0.10009E+01_JPRB, 0.10072E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.10216E+01_JPRB, 0.10272E+01_JPRB, 0.10328E+01_JPRB, 0.10381E+01_JPRB, 0.10427E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.10509E+01_JPRB, 0.10513E+01_JPRB, 0.10506E+01_JPRB, 0.10486E+01_JPRB, 0.10461E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.10296E+01_JPRB, 0.10249E+01_JPRB, 0.10181E+01_JPRB, 0.10112E+01_JPRB, 0.10040E+01_JPRB, 0.99671E+00_JPRB ,&
&  0.98732E+00_JPRB, 0.97775E+00_JPRB, 0.96749E+00_JPRB, 0.95653E+00_JPRB, 0.94585E+00_JPRB, 0.93470E+00_JPRB, 0.92256E+00_JPRB ,&
&  0.90909E+00_JPRB, 0.89644E+00_JPRB, 0.88233E+00_JPRB, 0.86831E+00_JPRB, 0.85829E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.83485E+00_JPRB, 0.82292E+00_JPRB, 0.81789E+00_JPRB, 0.81463E+00_JPRB, 0.81185E+00_JPRB, 0.80818E+00_JPRB, 0.80390E+00_JPRB ,&
&  0.80063E+00_JPRB, 0.79684E+00_JPRB, 0.79392E+00_JPRB, 0.79354E+00_JPRB, 0.79213E+00_JPRB, 0.79143E+00_JPRB, 0.79067E+00_JPRB ,&
&  0.79034E+00_JPRB, 0.78995E+00_JPRB, 0.79025E+00_JPRB, 0.78905E+00_JPRB, 0.78714E+00_JPRB, 0.78443E+00_JPRB, 0.78225E+00_JPRB ,&
&  0.78014E+00_JPRB, 0.77866E+00_JPRB, 0.77645E+00_JPRB, 0.77461E+00_JPRB, 0.77549E+00_JPRB, 0.77574E+00_JPRB, 0.77527E+00_JPRB ,&
&  0.77828E+00_JPRB, 0.77993E+00_JPRB, 0.77908E+00_JPRB, 0.77602E+00_JPRB, 0.77241E+00_JPRB, 0.77044E+00_JPRB, 0.77256E+00_JPRB ,&
&  0.77667E+00_JPRB, 0.78866E+00_JPRB, 0.79842E+00_JPRB, 0.80764E+00_JPRB, 0.81632E+00_JPRB, 0.82689E+00_JPRB, 0.83941E+00_JPRB ,&
&  0.85246E+00_JPRB, 0.86274E+00_JPRB, 0.87436E+00_JPRB, 0.88852E+00_JPRB, 0.90386E+00_JPRB, 0.92013E+00_JPRB, 0.93804E+00_JPRB ,&
&  0.95327E+00_JPRB, 0.96884E+00_JPRB, 0.98697E+00_JPRB, 0.10067E+01_JPRB, 0.10236E+01_JPRB, 0.10457E+01_JPRB, 0.10721E+01_JPRB ,&
&  0.11056E+01_JPRB, 0.11226E+01_JPRB, 0.11417E+01_JPRB, 0.11606E+01_JPRB, 0.11849E+01_JPRB, 0.11989E+01_JPRB, 0.12119E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.12279E+01_JPRB, 0.12249E+01_JPRB, 0.12197E+01_JPRB, 0.12162E+01_JPRB, 0.12113E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.12009E+01_JPRB, 0.11938E+01_JPRB, 0.11862E+01_JPRB, 0.11758E+01_JPRB, 0.11653E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11276E+01_JPRB, 0.11138E+01_JPRB, 0.11046E+01_JPRB, 0.10925E+01_JPRB, 0.10807E+01_JPRB, 0.10698E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10368E+01_JPRB, 0.10239E+01_JPRB, 0.10099E+01_JPRB, 0.99840E+00_JPRB, 0.98214E+00_JPRB, 0.96364E+00_JPRB ,&
&  0.94201E+00_JPRB, 0.92323E+00_JPRB, 0.90122E+00_JPRB, 0.87541E+00_JPRB, 0.85020E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.85245E+00_JPRB, 0.82430E+00_JPRB, 0.80957E+00_JPRB, 0.80546E+00_JPRB, 0.80333E+00_JPRB, 0.80111E+00_JPRB, 0.79882E+00_JPRB ,&
&  0.79505E+00_JPRB, 0.79241E+00_JPRB, 0.79346E+00_JPRB, 0.79511E+00_JPRB, 0.79530E+00_JPRB, 0.79524E+00_JPRB, 0.79364E+00_JPRB ,&
&  0.79156E+00_JPRB, 0.78765E+00_JPRB, 0.78370E+00_JPRB, 0.77661E+00_JPRB, 0.76901E+00_JPRB, 0.75947E+00_JPRB, 0.74611E+00_JPRB ,&
&  0.73572E+00_JPRB, 0.72659E+00_JPRB, 0.71463E+00_JPRB, 0.71145E+00_JPRB, 0.71084E+00_JPRB, 0.68366E+00_JPRB, 0.67878E+00_JPRB ,&
&  0.69030E+00_JPRB, 0.69271E+00_JPRB, 0.69307E+00_JPRB, 0.69297E+00_JPRB, 0.68567E+00_JPRB, 0.67146E+00_JPRB, 0.66640E+00_JPRB ,&
&  0.64512E+00_JPRB, 0.66571E+00_JPRB, 0.67956E+00_JPRB, 0.69101E+00_JPRB, 0.70187E+00_JPRB, 0.71647E+00_JPRB, 0.74757E+00_JPRB ,&
&  0.80611E+00_JPRB, 0.83072E+00_JPRB, 0.85926E+00_JPRB, 0.88680E+00_JPRB, 0.92253E+00_JPRB, 0.95964E+00_JPRB, 0.99407E+00_JPRB ,&
&  0.10245E+01_JPRB, 0.10572E+01_JPRB, 0.10953E+01_JPRB, 0.11442E+01_JPRB, 0.11841E+01_JPRB, 0.12207E+01_JPRB, 0.12472E+01_JPRB ,&
&  0.12773E+01_JPRB, 0.12989E+01_JPRB, 0.13145E+01_JPRB, 0.13237E+01_JPRB, 0.13344E+01_JPRB, 0.13394E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.13490E+01_JPRB, 0.13491E+01_JPRB, 0.13390E+01_JPRB, 0.13199E+01_JPRB, 0.13076E+01_JPRB, 0.13033E+01_JPRB, 0.12954E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.12745E+01_JPRB, 0.12602E+01_JPRB, 0.12425E+01_JPRB, 0.12194E+01_JPRB, 0.11993E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.11369E+01_JPRB, 0.11231E+01_JPRB, 0.11068E+01_JPRB, 0.10898E+01_JPRB, 0.10731E+01_JPRB, 0.10546E+01_JPRB ,&
&  0.10382E+01_JPRB, 0.10237E+01_JPRB, 0.10122E+01_JPRB, 0.99991E+00_JPRB, 0.99138E+00_JPRB, 0.97952E+00_JPRB, 0.96883E+00_JPRB ,&
&  0.95667E+00_JPRB, 0.94581E+00_JPRB, 0.92881E+00_JPRB, 0.90617E+00_JPRB, 0.87691E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.88750E+00_JPRB, 0.85694E+00_JPRB, 0.82460E+00_JPRB, 0.80646E+00_JPRB, 0.80220E+00_JPRB, 0.79848E+00_JPRB, 0.79542E+00_JPRB ,&
&  0.79377E+00_JPRB, 0.79648E+00_JPRB, 0.79939E+00_JPRB, 0.80087E+00_JPRB, 0.79982E+00_JPRB, 0.79676E+00_JPRB, 0.79205E+00_JPRB ,&
&  0.78544E+00_JPRB, 0.77695E+00_JPRB, 0.77008E+00_JPRB, 0.76281E+00_JPRB, 0.75471E+00_JPRB, 0.74617E+00_JPRB, 0.73253E+00_JPRB ,&
&  0.71947E+00_JPRB, 0.70747E+00_JPRB, 0.66632E+00_JPRB, 0.66587E+00_JPRB, 0.65910E+00_JPRB, 0.65783E+00_JPRB, 0.66244E+00_JPRB ,&
&  0.65986E+00_JPRB, 0.64720E+00_JPRB, 0.63257E+00_JPRB, 0.61306E+00_JPRB, 0.59946E+00_JPRB, 0.58471E+00_JPRB, 0.57862E+00_JPRB ,&
&  0.57746E+00_JPRB, 0.58254E+00_JPRB, 0.59492E+00_JPRB, 0.60584E+00_JPRB, 0.61757E+00_JPRB, 0.62796E+00_JPRB, 0.64779E+00_JPRB ,&
&  0.69780E+00_JPRB, 0.76628E+00_JPRB, 0.84968E+00_JPRB, 0.89515E+00_JPRB, 0.95033E+00_JPRB, 0.10037E+01_JPRB, 0.10445E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.11248E+01_JPRB, 0.11694E+01_JPRB, 0.12181E+01_JPRB, 0.12656E+01_JPRB, 0.13047E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.13944E+01_JPRB, 0.14283E+01_JPRB, 0.14433E+01_JPRB, 0.14510E+01_JPRB, 0.14508E+01_JPRB, 0.14385E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.14159E+01_JPRB, 0.14040E+01_JPRB, 0.13879E+01_JPRB, 0.13510E+01_JPRB, 0.13329E+01_JPRB, 0.13276E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.12831E+01_JPRB, 0.12549E+01_JPRB, 0.12271E+01_JPRB, 0.11884E+01_JPRB, 0.11559E+01_JPRB, 0.11444E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11178E+01_JPRB, 0.11084E+01_JPRB, 0.10917E+01_JPRB, 0.10653E+01_JPRB, 0.10390E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.99787E+00_JPRB, 0.97879E+00_JPRB, 0.96190E+00_JPRB, 0.94499E+00_JPRB, 0.93451E+00_JPRB, 0.92632E+00_JPRB, 0.92407E+00_JPRB ,&
&  0.92520E+00_JPRB, 0.92752E+00_JPRB, 0.92721E+00_JPRB, 0.92301E+00_JPRB, 0.90890E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.97302E+00_JPRB, 0.96014E+00_JPRB, 0.93319E+00_JPRB, 0.89693E+00_JPRB, 0.86847E+00_JPRB, 0.84824E+00_JPRB, 0.83813E+00_JPRB ,&
&  0.83165E+00_JPRB, 0.82870E+00_JPRB, 0.82505E+00_JPRB, 0.82123E+00_JPRB, 0.81471E+00_JPRB, 0.80687E+00_JPRB, 0.79845E+00_JPRB ,&
&  0.78474E+00_JPRB, 0.77163E+00_JPRB, 0.75850E+00_JPRB, 0.75005E+00_JPRB, 0.74301E+00_JPRB, 0.73750E+00_JPRB, 0.73238E+00_JPRB ,&
&  0.72677E+00_JPRB, 0.71960E+00_JPRB, 0.69697E+00_JPRB, 0.68912E+00_JPRB, 0.68014E+00_JPRB, 0.67971E+00_JPRB, 0.67509E+00_JPRB ,&
&  0.68632E+00_JPRB, 0.62281E+00_JPRB, 0.61643E+00_JPRB, 0.60005E+00_JPRB, 0.58866E+00_JPRB, 0.57533E+00_JPRB, 0.56062E+00_JPRB ,&
&  0.55290E+00_JPRB, 0.54366E+00_JPRB, 0.54238E+00_JPRB, 0.55400E+00_JPRB, 0.56317E+00_JPRB, 0.58056E+00_JPRB, 0.59873E+00_JPRB ,&
&  0.63093E+00_JPRB, 0.77294E+00_JPRB, 0.84453E+00_JPRB, 0.89612E+00_JPRB, 0.94899E+00_JPRB, 0.99754E+00_JPRB, 0.10245E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10935E+01_JPRB, 0.11268E+01_JPRB, 0.11419E+01_JPRB, 0.12286E+01_JPRB, 0.12915E+01_JPRB, 0.13590E+01_JPRB ,&
&  0.14209E+01_JPRB, 0.14803E+01_JPRB, 0.15314E+01_JPRB, 0.15705E+01_JPRB, 0.16017E+01_JPRB, 0.16016E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.15663E+01_JPRB, 0.15541E+01_JPRB, 0.15468E+01_JPRB, 0.14984E+01_JPRB, 0.14679E+01_JPRB, 0.14488E+01_JPRB, 0.13983E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.12929E+01_JPRB, 0.12427E+01_JPRB, 0.11902E+01_JPRB, 0.11421E+01_JPRB, 0.11148E+01_JPRB, 0.11033E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.10933E+01_JPRB, 0.10764E+01_JPRB, 0.10490E+01_JPRB, 0.10191E+01_JPRB, 0.99932E+00_JPRB, 0.97657E+00_JPRB ,&
&  0.95406E+00_JPRB, 0.92820E+00_JPRB, 0.91205E+00_JPRB, 0.90102E+00_JPRB, 0.89784E+00_JPRB, 0.90063E+00_JPRB, 0.90844E+00_JPRB ,&
&  0.91811E+00_JPRB, 0.93060E+00_JPRB, 0.94570E+00_JPRB, 0.96021E+00_JPRB, 0.97054E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.96807E+00_JPRB, 0.97624E+00_JPRB, 0.97781E+00_JPRB, 0.97061E+00_JPRB, 0.95580E+00_JPRB, 0.93607E+00_JPRB, 0.91623E+00_JPRB ,&
&  0.89756E+00_JPRB, 0.88293E+00_JPRB, 0.86852E+00_JPRB, 0.85707E+00_JPRB, 0.84326E+00_JPRB, 0.83331E+00_JPRB, 0.82185E+00_JPRB ,&
&  0.81451E+00_JPRB, 0.81077E+00_JPRB, 0.80128E+00_JPRB, 0.79049E+00_JPRB, 0.77249E+00_JPRB, 0.75668E+00_JPRB, 0.74427E+00_JPRB ,&
&  0.73306E+00_JPRB, 0.72321E+00_JPRB, 0.70153E+00_JPRB, 0.68613E+00_JPRB, 0.67774E+00_JPRB, 0.66873E+00_JPRB, 0.67374E+00_JPRB ,&
&  0.67054E+00_JPRB, 0.67448E+00_JPRB, 0.67798E+00_JPRB, 0.67079E+00_JPRB, 0.66214E+00_JPRB, 0.55551E+00_JPRB, 0.52867E+00_JPRB ,&
&  0.52197E+00_JPRB, 0.51481E+00_JPRB, 0.51459E+00_JPRB, 0.52244E+00_JPRB, 0.53486E+00_JPRB, 0.56825E+00_JPRB, 0.59146E+00_JPRB ,&
&  0.60633E+00_JPRB, 0.75522E+00_JPRB, 0.81717E+00_JPRB, 0.87130E+00_JPRB, 0.91217E+00_JPRB, 0.94407E+00_JPRB, 0.96392E+00_JPRB ,&
&  0.98427E+00_JPRB, 0.10132E+01_JPRB, 0.10304E+01_JPRB, 0.10669E+01_JPRB, 0.11373E+01_JPRB, 0.12557E+01_JPRB, 0.13638E+01_JPRB ,&
&  0.14854E+01_JPRB, 0.15492E+01_JPRB, 0.16065E+01_JPRB, 0.16357E+01_JPRB, 0.16835E+01_JPRB, 0.17143E+01_JPRB, 0.17225E+01_JPRB ,&
&  0.17372E+01_JPRB, 0.17157E+01_JPRB, 0.17272E+01_JPRB, 0.17009E+01_JPRB, 0.16773E+01_JPRB, 0.16504E+01_JPRB, 0.15558E+01_JPRB ,&
&  0.14445E+01_JPRB, 0.13704E+01_JPRB, 0.13026E+01_JPRB, 0.12158E+01_JPRB, 0.11518E+01_JPRB, 0.11056E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.10742E+01_JPRB, 0.10480E+01_JPRB, 0.10161E+01_JPRB, 0.99364E+00_JPRB, 0.97896E+00_JPRB, 0.95382E+00_JPRB ,&
&  0.93194E+00_JPRB, 0.91090E+00_JPRB, 0.89902E+00_JPRB, 0.88779E+00_JPRB, 0.88036E+00_JPRB, 0.87852E+00_JPRB, 0.88152E+00_JPRB ,&
&  0.88973E+00_JPRB, 0.90180E+00_JPRB, 0.91909E+00_JPRB, 0.93896E+00_JPRB, 0.96013E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.95319E+00_JPRB, 0.96815E+00_JPRB, 0.97543E+00_JPRB, 0.97274E+00_JPRB, 0.95773E+00_JPRB, 0.93803E+00_JPRB, 0.92044E+00_JPRB ,&
&  0.90328E+00_JPRB, 0.88711E+00_JPRB, 0.87166E+00_JPRB, 0.85974E+00_JPRB, 0.84907E+00_JPRB, 0.84100E+00_JPRB, 0.83468E+00_JPRB ,&
&  0.83010E+00_JPRB, 0.80900E+00_JPRB, 0.79995E+00_JPRB, 0.79833E+00_JPRB, 0.79865E+00_JPRB, 0.79496E+00_JPRB, 0.78796E+00_JPRB ,&
&  0.76512E+00_JPRB, 0.73390E+00_JPRB, 0.70452E+00_JPRB, 0.67882E+00_JPRB, 0.65772E+00_JPRB, 0.64835E+00_JPRB, 0.65186E+00_JPRB ,&
&  0.65468E+00_JPRB, 0.64734E+00_JPRB, 0.64898E+00_JPRB, 0.66835E+00_JPRB, 0.67252E+00_JPRB, 0.60910E+00_JPRB, 0.52886E+00_JPRB ,&
&  0.51980E+00_JPRB, 0.50127E+00_JPRB, 0.50130E+00_JPRB, 0.50927E+00_JPRB, 0.52437E+00_JPRB, 0.54691E+00_JPRB, 0.58468E+00_JPRB ,&
&  0.63036E+00_JPRB, 0.73672E+00_JPRB, 0.79769E+00_JPRB, 0.84168E+00_JPRB, 0.87997E+00_JPRB, 0.91263E+00_JPRB, 0.92819E+00_JPRB ,&
&  0.93776E+00_JPRB, 0.95917E+00_JPRB, 0.99716E+00_JPRB, 0.10424E+01_JPRB, 0.11219E+01_JPRB, 0.12870E+01_JPRB, 0.14308E+01_JPRB ,&
&  0.16274E+01_JPRB, 0.17347E+01_JPRB, 0.17281E+01_JPRB, 0.16613E+01_JPRB, 0.17012E+01_JPRB, 0.17877E+01_JPRB, 0.18116E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.19764E+01_JPRB, 0.19619E+01_JPRB, 0.18771E+01_JPRB, 0.18240E+01_JPRB, 0.17945E+01_JPRB, 0.16617E+01_JPRB ,&
&  0.15148E+01_JPRB, 0.14616E+01_JPRB, 0.14720E+01_JPRB, 0.13265E+01_JPRB, 0.12270E+01_JPRB, 0.11330E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.10448E+01_JPRB, 0.10168E+01_JPRB, 0.97978E+00_JPRB, 0.95099E+00_JPRB, 0.93540E+00_JPRB, 0.92550E+00_JPRB, 0.91437E+00_JPRB ,&
&  0.90213E+00_JPRB, 0.89125E+00_JPRB, 0.88047E+00_JPRB, 0.87290E+00_JPRB, 0.86466E+00_JPRB, 0.86100E+00_JPRB, 0.86097E+00_JPRB ,&
&  0.86613E+00_JPRB, 0.87725E+00_JPRB, 0.89368E+00_JPRB, 0.91276E+00_JPRB, 0.93580E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.96324E+00_JPRB, 0.97438E+00_JPRB, 0.96570E+00_JPRB, 0.94055E+00_JPRB, 0.90452E+00_JPRB, 0.85987E+00_JPRB, 0.82750E+00_JPRB ,&
&  0.80894E+00_JPRB, 0.80055E+00_JPRB, 0.79339E+00_JPRB, 0.78527E+00_JPRB, 0.78198E+00_JPRB, 0.78111E+00_JPRB, 0.78241E+00_JPRB ,&
&  0.78588E+00_JPRB, 0.78930E+00_JPRB, 0.78872E+00_JPRB, 0.77920E+00_JPRB, 0.77704E+00_JPRB, 0.77742E+00_JPRB, 0.77990E+00_JPRB ,&
&  0.77204E+00_JPRB, 0.74559E+00_JPRB, 0.71350E+00_JPRB, 0.68923E+00_JPRB, 0.66550E+00_JPRB, 0.65690E+00_JPRB, 0.64372E+00_JPRB ,&
&  0.62186E+00_JPRB, 0.61670E+00_JPRB, 0.63526E+00_JPRB, 0.66653E+00_JPRB, 0.66142E+00_JPRB, 0.67886E+00_JPRB, 0.56379E+00_JPRB ,&
&  0.52784E+00_JPRB, 0.50805E+00_JPRB, 0.50380E+00_JPRB, 0.51671E+00_JPRB, 0.55113E+00_JPRB, 0.59231E+00_JPRB, 0.62708E+00_JPRB ,&
&  0.66593E+00_JPRB, 0.73230E+00_JPRB, 0.76998E+00_JPRB, 0.80162E+00_JPRB, 0.83990E+00_JPRB, 0.87938E+00_JPRB, 0.90845E+00_JPRB ,&
&  0.92788E+00_JPRB, 0.96225E+00_JPRB, 0.10208E+01_JPRB, 0.11136E+01_JPRB, 0.13139E+01_JPRB, 0.13048E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.17774E+01_JPRB, 0.19859E+01_JPRB, 0.18576E+01_JPRB, 0.17455E+01_JPRB, 0.17130E+01_JPRB, 0.17678E+01_JPRB, 0.18638E+01_JPRB ,&
&  0.20468E+01_JPRB, 0.22102E+01_JPRB, 0.22139E+01_JPRB, 0.20167E+01_JPRB, 0.19174E+01_JPRB, 0.18632E+01_JPRB, 0.16846E+01_JPRB ,&
&  0.14883E+01_JPRB, 0.15235E+01_JPRB, 0.15956E+01_JPRB, 0.13577E+01_JPRB, 0.11939E+01_JPRB, 0.10888E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10171E+01_JPRB, 0.99378E+00_JPRB, 0.96622E+00_JPRB, 0.93977E+00_JPRB, 0.90704E+00_JPRB, 0.87829E+00_JPRB, 0.85371E+00_JPRB ,&
&  0.83680E+00_JPRB, 0.82518E+00_JPRB, 0.82406E+00_JPRB, 0.82845E+00_JPRB, 0.83089E+00_JPRB, 0.83378E+00_JPRB, 0.84418E+00_JPRB ,&
&  0.85914E+00_JPRB, 0.86626E+00_JPRB, 0.88122E+00_JPRB, 0.90282E+00_JPRB, 0.93341E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.92089E+00_JPRB, 0.92331E+00_JPRB, 0.90860E+00_JPRB, 0.87979E+00_JPRB, 0.84289E+00_JPRB, 0.79367E+00_JPRB, 0.75437E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.71793E+00_JPRB, 0.71007E+00_JPRB, 0.70620E+00_JPRB, 0.70677E+00_JPRB, 0.71183E+00_JPRB, 0.71195E+00_JPRB ,&
&  0.71419E+00_JPRB, 0.71648E+00_JPRB, 0.72109E+00_JPRB, 0.72803E+00_JPRB, 0.73215E+00_JPRB, 0.73506E+00_JPRB, 0.73858E+00_JPRB ,&
&  0.74165E+00_JPRB, 0.73074E+00_JPRB, 0.70455E+00_JPRB, 0.68467E+00_JPRB, 0.68649E+00_JPRB, 0.70241E+00_JPRB, 0.68386E+00_JPRB ,&
&  0.63944E+00_JPRB, 0.61825E+00_JPRB, 0.61761E+00_JPRB, 0.67413E+00_JPRB, 0.71377E+00_JPRB, 0.72727E+00_JPRB, 0.61127E+00_JPRB ,&
&  0.54937E+00_JPRB, 0.52595E+00_JPRB, 0.53088E+00_JPRB, 0.56411E+00_JPRB, 0.61817E+00_JPRB, 0.67313E+00_JPRB, 0.74385E+00_JPRB ,&
&  0.79182E+00_JPRB, 0.83173E+00_JPRB, 0.84321E+00_JPRB, 0.86705E+00_JPRB, 0.90700E+00_JPRB, 0.94358E+00_JPRB, 0.98368E+00_JPRB ,&
&  0.10106E+01_JPRB, 0.10529E+01_JPRB, 0.11702E+01_JPRB, 0.10560E+01_JPRB, 0.11145E+01_JPRB, 0.11862E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.15944E+01_JPRB, 0.16975E+01_JPRB, 0.17240E+01_JPRB, 0.18541E+01_JPRB, 0.17717E+01_JPRB, 0.17210E+01_JPRB, 0.17769E+01_JPRB ,&
&  0.20640E+01_JPRB, 0.23283E+01_JPRB, 0.22916E+01_JPRB, 0.20995E+01_JPRB, 0.20326E+01_JPRB, 0.19943E+01_JPRB, 0.17774E+01_JPRB ,&
&  0.15047E+01_JPRB, 0.13532E+01_JPRB, 0.12235E+01_JPRB, 0.10696E+01_JPRB, 0.10084E+01_JPRB, 0.97251E+00_JPRB, 0.95113E+00_JPRB ,&
&  0.93725E+00_JPRB, 0.91904E+00_JPRB, 0.90566E+00_JPRB, 0.85170E+00_JPRB, 0.80305E+00_JPRB, 0.75307E+00_JPRB, 0.72086E+00_JPRB ,&
&  0.70123E+00_JPRB, 0.70138E+00_JPRB, 0.71472E+00_JPRB, 0.73894E+00_JPRB, 0.78455E+00_JPRB, 0.82660E+00_JPRB, 0.85585E+00_JPRB ,&
&  0.86697E+00_JPRB, 0.85238E+00_JPRB, 0.84925E+00_JPRB, 0.85981E+00_JPRB, 0.89703E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.88065E+00_JPRB, 0.84514E+00_JPRB, 0.82293E+00_JPRB, 0.82727E+00_JPRB, 0.78224E+00_JPRB, 0.73565E+00_JPRB, 0.70820E+00_JPRB ,&
&  0.70781E+00_JPRB, 0.70315E+00_JPRB, 0.70232E+00_JPRB, 0.67588E+00_JPRB, 0.65233E+00_JPRB, 0.64006E+00_JPRB, 0.63532E+00_JPRB ,&
&  0.64623E+00_JPRB, 0.66045E+00_JPRB, 0.67264E+00_JPRB, 0.67266E+00_JPRB, 0.67104E+00_JPRB, 0.67389E+00_JPRB, 0.68348E+00_JPRB ,&
&  0.69575E+00_JPRB, 0.70194E+00_JPRB, 0.69492E+00_JPRB, 0.68793E+00_JPRB, 0.70814E+00_JPRB, 0.73243E+00_JPRB, 0.70468E+00_JPRB ,&
&  0.67225E+00_JPRB, 0.64530E+00_JPRB, 0.63919E+00_JPRB, 0.68052E+00_JPRB, 0.75016E+00_JPRB, 0.77593E+00_JPRB, 0.66073E+00_JPRB ,&
&  0.58056E+00_JPRB, 0.56876E+00_JPRB, 0.61541E+00_JPRB, 0.73107E+00_JPRB, 0.78144E+00_JPRB, 0.83624E+00_JPRB, 0.90500E+00_JPRB ,&
&  0.85824E+00_JPRB, 0.86235E+00_JPRB, 0.89011E+00_JPRB, 0.10293E+01_JPRB, 0.10680E+01_JPRB, 0.10853E+01_JPRB, 0.10990E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11492E+01_JPRB, 0.12928E+01_JPRB, 0.11339E+01_JPRB, 0.12079E+01_JPRB, 0.14415E+01_JPRB, 0.14617E+01_JPRB ,&
&  0.15890E+01_JPRB, 0.16505E+01_JPRB, 0.18566E+01_JPRB, 0.18407E+01_JPRB, 0.17281E+01_JPRB, 0.17073E+01_JPRB, 0.18577E+01_JPRB ,&
&  0.22198E+01_JPRB, 0.25275E+01_JPRB, 0.24722E+01_JPRB, 0.23785E+01_JPRB, 0.23194E+01_JPRB, 0.22142E+01_JPRB, 0.19697E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.13615E+01_JPRB, 0.11688E+01_JPRB, 0.10033E+01_JPRB, 0.92498E+00_JPRB, 0.89553E+00_JPRB, 0.86461E+00_JPRB ,&
&  0.85575E+00_JPRB, 0.84576E+00_JPRB, 0.83560E+00_JPRB, 0.77893E+00_JPRB, 0.73151E+00_JPRB, 0.69909E+00_JPRB, 0.67481E+00_JPRB ,&
&  0.66737E+00_JPRB, 0.65588E+00_JPRB, 0.64860E+00_JPRB, 0.66970E+00_JPRB, 0.73121E+00_JPRB, 0.80431E+00_JPRB, 0.83979E+00_JPRB ,&
&  0.82784E+00_JPRB, 0.80474E+00_JPRB, 0.82744E+00_JPRB, 0.87147E+00_JPRB, 0.89438E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.89931E+00_JPRB, 0.89093E+00_JPRB, 0.88420E+00_JPRB, 0.87240E+00_JPRB, 0.84943E+00_JPRB, 0.79178E+00_JPRB, 0.75470E+00_JPRB ,&
&  0.76310E+00_JPRB, 0.73613E+00_JPRB, 0.72402E+00_JPRB, 0.68891E+00_JPRB, 0.65714E+00_JPRB, 0.62501E+00_JPRB, 0.60994E+00_JPRB ,&
&  0.59661E+00_JPRB, 0.60863E+00_JPRB, 0.63597E+00_JPRB, 0.64516E+00_JPRB, 0.64682E+00_JPRB, 0.64685E+00_JPRB, 0.65851E+00_JPRB ,&
&  0.67204E+00_JPRB, 0.68904E+00_JPRB, 0.69885E+00_JPRB, 0.70604E+00_JPRB, 0.72336E+00_JPRB, 0.74711E+00_JPRB, 0.74397E+00_JPRB ,&
&  0.73718E+00_JPRB, 0.71191E+00_JPRB, 0.69133E+00_JPRB, 0.71725E+00_JPRB, 0.78112E+00_JPRB, 0.84204E+00_JPRB, 0.80834E+00_JPRB ,&
&  0.65336E+00_JPRB, 0.66135E+00_JPRB, 0.82855E+00_JPRB, 0.89168E+00_JPRB, 0.95165E+00_JPRB, 0.98993E+00_JPRB, 0.10390E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.11265E+01_JPRB, 0.11583E+01_JPRB, 0.11838E+01_JPRB, 0.12016E+01_JPRB, 0.12233E+01_JPRB, 0.12194E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.10831E+01_JPRB, 0.10440E+01_JPRB, 0.11182E+01_JPRB, 0.14377E+01_JPRB, 0.15345E+01_JPRB, 0.15260E+01_JPRB ,&
&  0.16245E+01_JPRB, 0.16467E+01_JPRB, 0.17522E+01_JPRB, 0.19086E+01_JPRB, 0.19669E+01_JPRB, 0.20859E+01_JPRB, 0.22585E+01_JPRB ,&
&  0.25340E+01_JPRB, 0.28413E+01_JPRB, 0.28725E+01_JPRB, 0.28216E+01_JPRB, 0.26839E+01_JPRB, 0.24599E+01_JPRB, 0.21034E+01_JPRB ,&
&  0.16691E+01_JPRB, 0.13698E+01_JPRB, 0.11291E+01_JPRB, 0.96276E+00_JPRB, 0.86819E+00_JPRB, 0.82546E+00_JPRB, 0.79401E+00_JPRB ,&
&  0.79081E+00_JPRB, 0.78500E+00_JPRB, 0.77186E+00_JPRB, 0.72974E+00_JPRB, 0.70299E+00_JPRB, 0.71552E+00_JPRB, 0.72792E+00_JPRB ,&
&  0.73786E+00_JPRB, 0.73139E+00_JPRB, 0.69382E+00_JPRB, 0.71190E+00_JPRB, 0.77251E+00_JPRB, 0.83411E+00_JPRB, 0.85768E+00_JPRB ,&
&  0.85241E+00_JPRB, 0.83069E+00_JPRB, 0.84430E+00_JPRB, 0.87648E+00_JPRB, 0.88887E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.93073E+00_JPRB, 0.93530E+00_JPRB, 0.91883E+00_JPRB, 0.90553E+00_JPRB, 0.88564E+00_JPRB, 0.81467E+00_JPRB, 0.79367E+00_JPRB ,&
&  0.86327E+00_JPRB, 0.80440E+00_JPRB, 0.85466E+00_JPRB, 0.74765E+00_JPRB, 0.71992E+00_JPRB, 0.68487E+00_JPRB, 0.65825E+00_JPRB ,&
&  0.62341E+00_JPRB, 0.60981E+00_JPRB, 0.62365E+00_JPRB, 0.64024E+00_JPRB, 0.64132E+00_JPRB, 0.63676E+00_JPRB, 0.64082E+00_JPRB ,&
&  0.64896E+00_JPRB, 0.67459E+00_JPRB, 0.70895E+00_JPRB, 0.72580E+00_JPRB, 0.73995E+00_JPRB, 0.76564E+00_JPRB, 0.77749E+00_JPRB ,&
&  0.75487E+00_JPRB, 0.75333E+00_JPRB, 0.75364E+00_JPRB, 0.77115E+00_JPRB, 0.81161E+00_JPRB, 0.87356E+00_JPRB, 0.91481E+00_JPRB ,&
&  0.92870E+00_JPRB, 0.82061E+00_JPRB, 0.93239E+00_JPRB, 0.96401E+00_JPRB, 0.10276E+01_JPRB, 0.10647E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.11263E+01_JPRB, 0.11659E+01_JPRB, 0.12096E+01_JPRB, 0.12490E+01_JPRB, 0.12891E+01_JPRB, 0.13282E+01_JPRB, 0.13335E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.10786E+01_JPRB, 0.11468E+01_JPRB, 0.12082E+01_JPRB, 0.15833E+01_JPRB, 0.17686E+01_JPRB, 0.17535E+01_JPRB ,&
&  0.20679E+01_JPRB, 0.19977E+01_JPRB, 0.21725E+01_JPRB, 0.23349E+01_JPRB, 0.25300E+01_JPRB, 0.27761E+01_JPRB, 0.29234E+01_JPRB ,&
&  0.29863E+01_JPRB, 0.31208E+01_JPRB, 0.32942E+01_JPRB, 0.30808E+01_JPRB, 0.27451E+01_JPRB, 0.23668E+01_JPRB, 0.19218E+01_JPRB ,&
&  0.16194E+01_JPRB, 0.13593E+01_JPRB, 0.11406E+01_JPRB, 0.99203E+00_JPRB, 0.87732E+00_JPRB, 0.80256E+00_JPRB, 0.76263E+00_JPRB ,&
&  0.74042E+00_JPRB, 0.71903E+00_JPRB, 0.71276E+00_JPRB, 0.69481E+00_JPRB, 0.68869E+00_JPRB, 0.70658E+00_JPRB, 0.75024E+00_JPRB ,&
&  0.76535E+00_JPRB, 0.77706E+00_JPRB, 0.82796E+00_JPRB, 0.92106E+00_JPRB, 0.93967E+00_JPRB, 0.96214E+00_JPRB, 0.10519E+01_JPRB ,&
&  0.93947E+00_JPRB, 0.93987E+00_JPRB, 0.98527E+00_JPRB, 0.96715E+00_JPRB, 0.93470E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.10014E+01_JPRB, 0.99443E+00_JPRB, 0.96087E+00_JPRB, 0.93781E+00_JPRB, 0.92979E+00_JPRB, 0.89075E+00_JPRB, 0.85567E+00_JPRB ,&
&  0.91434E+00_JPRB, 0.90327E+00_JPRB, 0.90284E+00_JPRB, 0.89160E+00_JPRB, 0.87704E+00_JPRB, 0.80602E+00_JPRB, 0.75195E+00_JPRB ,&
&  0.70158E+00_JPRB, 0.67416E+00_JPRB, 0.68125E+00_JPRB, 0.68141E+00_JPRB, 0.67700E+00_JPRB, 0.66213E+00_JPRB, 0.65432E+00_JPRB ,&
&  0.64849E+00_JPRB, 0.66386E+00_JPRB, 0.70133E+00_JPRB, 0.73287E+00_JPRB, 0.74091E+00_JPRB, 0.75137E+00_JPRB, 0.77488E+00_JPRB ,&
&  0.75775E+00_JPRB, 0.76483E+00_JPRB, 0.77618E+00_JPRB, 0.74439E+00_JPRB, 0.83674E+00_JPRB, 0.88288E+00_JPRB, 0.90923E+00_JPRB ,&
&  0.95816E+00_JPRB, 0.10006E+01_JPRB, 0.10219E+01_JPRB, 0.10299E+01_JPRB, 0.10521E+01_JPRB, 0.10657E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.11519E+01_JPRB, 0.12015E+01_JPRB, 0.12619E+01_JPRB, 0.13782E+01_JPRB, 0.12999E+01_JPRB, 0.14133E+01_JPRB ,&
&  0.14817E+01_JPRB, 0.15136E+01_JPRB, 0.16007E+01_JPRB, 0.14744E+01_JPRB, 0.18461E+01_JPRB, 0.20818E+01_JPRB, 0.19811E+01_JPRB ,&
&  0.20799E+01_JPRB, 0.20755E+01_JPRB, 0.25350E+01_JPRB, 0.28975E+01_JPRB, 0.31992E+01_JPRB, 0.34686E+01_JPRB, 0.35779E+01_JPRB ,&
&  0.35645E+01_JPRB, 0.37899E+01_JPRB, 0.37465E+01_JPRB, 0.33915E+01_JPRB, 0.29042E+01_JPRB, 0.25137E+01_JPRB, 0.20315E+01_JPRB ,&
&  0.16363E+01_JPRB, 0.13204E+01_JPRB, 0.11275E+01_JPRB, 0.10322E+01_JPRB, 0.87308E+00_JPRB, 0.81933E+00_JPRB, 0.76270E+00_JPRB ,&
&  0.70845E+00_JPRB, 0.67486E+00_JPRB, 0.68340E+00_JPRB, 0.69979E+00_JPRB, 0.69449E+00_JPRB, 0.71386E+00_JPRB, 0.73452E+00_JPRB ,&
&  0.80610E+00_JPRB, 0.95096E+00_JPRB, 0.12240E+01_JPRB, 0.12655E+01_JPRB, 0.12458E+01_JPRB, 0.12284E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.10203E+01_JPRB, 0.11143E+01_JPRB, 0.10712E+01_JPRB, 0.10254E+01_JPRB, 0.99427E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.10570E+01_JPRB, 0.10390E+01_JPRB, 0.10152E+01_JPRB, 0.99157E+00_JPRB, 0.97350E+00_JPRB, 0.94801E+00_JPRB, 0.93092E+00_JPRB ,&
&  0.93923E+00_JPRB, 0.93199E+00_JPRB, 0.92971E+00_JPRB, 0.92238E+00_JPRB, 0.90415E+00_JPRB, 0.91471E+00_JPRB, 0.83529E+00_JPRB ,&
&  0.75913E+00_JPRB, 0.72789E+00_JPRB, 0.70438E+00_JPRB, 0.71068E+00_JPRB, 0.73179E+00_JPRB, 0.72272E+00_JPRB, 0.70704E+00_JPRB ,&
&  0.68862E+00_JPRB, 0.67927E+00_JPRB, 0.69072E+00_JPRB, 0.71459E+00_JPRB, 0.72983E+00_JPRB, 0.74365E+00_JPRB, 0.76020E+00_JPRB ,&
&  0.73728E+00_JPRB, 0.71264E+00_JPRB, 0.70465E+00_JPRB, 0.71483E+00_JPRB, 0.78411E+00_JPRB, 0.88182E+00_JPRB, 0.93801E+00_JPRB ,&
&  0.96050E+00_JPRB, 0.97839E+00_JPRB, 0.10178E+01_JPRB, 0.10352E+01_JPRB, 0.10450E+01_JPRB, 0.10678E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.11232E+01_JPRB, 0.11615E+01_JPRB, 0.12135E+01_JPRB, 0.12989E+01_JPRB, 0.13166E+01_JPRB, 0.13346E+01_JPRB, 0.14856E+01_JPRB ,&
&  0.15932E+01_JPRB, 0.17175E+01_JPRB, 0.17639E+01_JPRB, 0.19028E+01_JPRB, 0.21617E+01_JPRB, 0.21818E+01_JPRB, 0.21981E+01_JPRB ,&
&  0.23938E+01_JPRB, 0.24976E+01_JPRB, 0.31763E+01_JPRB, 0.34713E+01_JPRB, 0.36076E+01_JPRB, 0.38714E+01_JPRB, 0.38932E+01_JPRB ,&
&  0.39573E+01_JPRB, 0.39707E+01_JPRB, 0.39172E+01_JPRB, 0.35693E+01_JPRB, 0.30236E+01_JPRB, 0.25982E+01_JPRB, 0.20241E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.13390E+01_JPRB, 0.10353E+01_JPRB, 0.91066E+00_JPRB, 0.79904E+00_JPRB, 0.77454E+00_JPRB, 0.73713E+00_JPRB ,&
&  0.69008E+00_JPRB, 0.69006E+00_JPRB, 0.71522E+00_JPRB, 0.79240E+00_JPRB, 0.80874E+00_JPRB, 0.75876E+00_JPRB, 0.81674E+00_JPRB ,&
&  0.93382E+00_JPRB, 0.12700E+01_JPRB, 0.13848E+01_JPRB, 0.15151E+01_JPRB, 0.14578E+01_JPRB, 0.14365E+01_JPRB, 0.11826E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.12637E+01_JPRB, 0.11674E+01_JPRB, 0.11046E+01_JPRB, 0.10764E+01_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.10985E+01_JPRB, 0.10711E+01_JPRB, 0.10574E+01_JPRB, 0.10548E+01_JPRB, 0.10471E+01_JPRB, 0.10102E+01_JPRB, 0.10071E+01_JPRB ,&
&  0.99831E+00_JPRB, 0.97766E+00_JPRB, 0.96976E+00_JPRB, 0.96481E+00_JPRB, 0.94844E+00_JPRB, 0.95347E+00_JPRB, 0.97845E+00_JPRB ,&
&  0.87637E+00_JPRB, 0.83869E+00_JPRB, 0.76043E+00_JPRB, 0.74101E+00_JPRB, 0.81621E+00_JPRB, 0.78727E+00_JPRB, 0.77494E+00_JPRB ,&
&  0.74179E+00_JPRB, 0.71894E+00_JPRB, 0.69341E+00_JPRB, 0.69888E+00_JPRB, 0.71859E+00_JPRB, 0.74014E+00_JPRB, 0.75074E+00_JPRB ,&
&  0.71736E+00_JPRB, 0.68470E+00_JPRB, 0.68046E+00_JPRB, 0.69539E+00_JPRB, 0.74741E+00_JPRB, 0.82882E+00_JPRB, 0.92569E+00_JPRB ,&
&  0.96709E+00_JPRB, 0.98223E+00_JPRB, 0.10084E+01_JPRB, 0.10389E+01_JPRB, 0.10762E+01_JPRB, 0.11306E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.12129E+01_JPRB, 0.12611E+01_JPRB, 0.13379E+01_JPRB, 0.13012E+01_JPRB, 0.13453E+01_JPRB, 0.15818E+01_JPRB ,&
&  0.18044E+01_JPRB, 0.17616E+01_JPRB, 0.18645E+01_JPRB, 0.25514E+01_JPRB, 0.29506E+01_JPRB, 0.30351E+01_JPRB, 0.29677E+01_JPRB ,&
&  0.30757E+01_JPRB, 0.32421E+01_JPRB, 0.34650E+01_JPRB, 0.37506E+01_JPRB, 0.39336E+01_JPRB, 0.41255E+01_JPRB, 0.41125E+01_JPRB ,&
&  0.40870E+01_JPRB, 0.40744E+01_JPRB, 0.39319E+01_JPRB, 0.36113E+01_JPRB, 0.29589E+01_JPRB, 0.24676E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.16128E+01_JPRB, 0.12429E+01_JPRB, 0.88324E+00_JPRB, 0.74852E+00_JPRB, 0.69995E+00_JPRB, 0.75459E+00_JPRB, 0.96720E+00_JPRB ,&
&  0.75570E+00_JPRB, 0.80832E+00_JPRB, 0.88935E+00_JPRB, 0.10330E+01_JPRB, 0.11352E+01_JPRB, 0.11429E+01_JPRB, 0.98779E+00_JPRB ,&
&  0.99856E+00_JPRB, 0.12550E+01_JPRB, 0.16535E+01_JPRB, 0.18042E+01_JPRB, 0.16750E+01_JPRB, 0.16595E+01_JPRB, 0.15371E+01_JPRB ,&
&  0.15985E+01_JPRB, 0.14237E+01_JPRB, 0.12752E+01_JPRB, 0.11729E+01_JPRB, 0.11421E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.11127E+01_JPRB, 0.11079E+01_JPRB, 0.11159E+01_JPRB, 0.11336E+01_JPRB, 0.11434E+01_JPRB, 0.10905E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10369E+01_JPRB, 0.10172E+01_JPRB, 0.10086E+01_JPRB, 0.99503E+00_JPRB, 0.98597E+00_JPRB, 0.98955E+00_JPRB ,&
&  0.10161E+01_JPRB, 0.97152E+00_JPRB, 0.87534E+00_JPRB, 0.76568E+00_JPRB, 0.79022E+00_JPRB, 0.78051E+00_JPRB, 0.78090E+00_JPRB ,&
&  0.76226E+00_JPRB, 0.75972E+00_JPRB, 0.73358E+00_JPRB, 0.71630E+00_JPRB, 0.73458E+00_JPRB, 0.76243E+00_JPRB, 0.77323E+00_JPRB ,&
&  0.75313E+00_JPRB, 0.71808E+00_JPRB, 0.70729E+00_JPRB, 0.73454E+00_JPRB, 0.82160E+00_JPRB, 0.82266E+00_JPRB, 0.92616E+00_JPRB ,&
&  0.98195E+00_JPRB, 0.10198E+01_JPRB, 0.10556E+01_JPRB, 0.11096E+01_JPRB, 0.11683E+01_JPRB, 0.12419E+01_JPRB, 0.12869E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.13158E+01_JPRB, 0.12838E+01_JPRB, 0.13533E+01_JPRB, 0.14259E+01_JPRB, 0.14960E+01_JPRB, 0.16205E+01_JPRB ,&
&  0.17562E+01_JPRB, 0.18215E+01_JPRB, 0.21211E+01_JPRB, 0.29193E+01_JPRB, 0.33512E+01_JPRB, 0.33780E+01_JPRB, 0.35768E+01_JPRB ,&
&  0.38372E+01_JPRB, 0.40917E+01_JPRB, 0.43543E+01_JPRB, 0.44147E+01_JPRB, 0.44642E+01_JPRB, 0.43050E+01_JPRB, 0.42154E+01_JPRB ,&
&  0.40873E+01_JPRB, 0.40950E+01_JPRB, 0.40372E+01_JPRB, 0.35211E+01_JPRB, 0.28554E+01_JPRB, 0.23065E+01_JPRB, 0.20065E+01_JPRB ,&
&  0.19113E+01_JPRB, 0.10319E+01_JPRB, 0.74286E+00_JPRB, 0.62069E+00_JPRB, 0.60549E+00_JPRB, 0.64231E+00_JPRB, 0.78941E+00_JPRB ,&
&  0.73355E+00_JPRB, 0.89799E+00_JPRB, 0.12209E+01_JPRB, 0.14198E+01_JPRB, 0.17404E+01_JPRB, 0.21016E+01_JPRB, 0.16260E+01_JPRB ,&
&  0.13680E+01_JPRB, 0.14878E+01_JPRB, 0.20362E+01_JPRB, 0.21569E+01_JPRB, 0.19443E+01_JPRB, 0.18995E+01_JPRB, 0.19179E+01_JPRB ,&
&  0.17727E+01_JPRB, 0.16540E+01_JPRB, 0.14183E+01_JPRB, 0.12204E+01_JPRB, 0.11462E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.12150E+01_JPRB, 0.12041E+01_JPRB, 0.12271E+01_JPRB, 0.12482E+01_JPRB, 0.12375E+01_JPRB, 0.11897E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.10941E+01_JPRB, 0.10617E+01_JPRB, 0.10438E+01_JPRB, 0.10299E+01_JPRB, 0.10203E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.98818E+00_JPRB, 0.10057E+01_JPRB, 0.89209E+00_JPRB, 0.79756E+00_JPRB, 0.71183E+00_JPRB, 0.72930E+00_JPRB, 0.75753E+00_JPRB ,&
&  0.79776E+00_JPRB, 0.82163E+00_JPRB, 0.84617E+00_JPRB, 0.86754E+00_JPRB, 0.88943E+00_JPRB, 0.86080E+00_JPRB, 0.88275E+00_JPRB ,&
&  0.86239E+00_JPRB, 0.82556E+00_JPRB, 0.77623E+00_JPRB, 0.84099E+00_JPRB, 0.89916E+00_JPRB, 0.89108E+00_JPRB, 0.10328E+01_JPRB ,&
&  0.11022E+01_JPRB, 0.11543E+01_JPRB, 0.11969E+01_JPRB, 0.12478E+01_JPRB, 0.12984E+01_JPRB, 0.13012E+01_JPRB, 0.12737E+01_JPRB ,&
&  0.12239E+01_JPRB, 0.12272E+01_JPRB, 0.12833E+01_JPRB, 0.13268E+01_JPRB, 0.14967E+01_JPRB, 0.16061E+01_JPRB, 0.17578E+01_JPRB ,&
&  0.19577E+01_JPRB, 0.19859E+01_JPRB, 0.21400E+01_JPRB, 0.30185E+01_JPRB, 0.40698E+01_JPRB, 0.41300E+01_JPRB, 0.42646E+01_JPRB ,&
&  0.45692E+01_JPRB, 0.46656E+01_JPRB, 0.46197E+01_JPRB, 0.44216E+01_JPRB, 0.43854E+01_JPRB, 0.43772E+01_JPRB, 0.43789E+01_JPRB ,&
&  0.41784E+01_JPRB, 0.40877E+01_JPRB, 0.39477E+01_JPRB, 0.35541E+01_JPRB, 0.29500E+01_JPRB, 0.26078E+01_JPRB, 0.22897E+01_JPRB ,&
&  0.19905E+01_JPRB, 0.13063E+01_JPRB, 0.86184E+00_JPRB, 0.72014E+00_JPRB, 0.68388E+00_JPRB, 0.69795E+00_JPRB, 0.83425E+00_JPRB ,&
&  0.87226E+00_JPRB, 0.11367E+01_JPRB, 0.15802E+01_JPRB, 0.18099E+01_JPRB, 0.27780E+01_JPRB, 0.34538E+01_JPRB, 0.31822E+01_JPRB ,&
&  0.25251E+01_JPRB, 0.21467E+01_JPRB, 0.25592E+01_JPRB, 0.25647E+01_JPRB, 0.23411E+01_JPRB, 0.21286E+01_JPRB, 0.20745E+01_JPRB ,&
&  0.19365E+01_JPRB, 0.18218E+01_JPRB, 0.16486E+01_JPRB, 0.14054E+01_JPRB, 0.12675E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.14280E+01_JPRB, 0.13626E+01_JPRB, 0.13301E+01_JPRB, 0.13119E+01_JPRB, 0.12466E+01_JPRB, 0.12024E+01_JPRB, 0.12077E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.11493E+01_JPRB, 0.11121E+01_JPRB, 0.11006E+01_JPRB, 0.10938E+01_JPRB, 0.10874E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.98505E+00_JPRB, 0.95404E+00_JPRB, 0.83126E+00_JPRB, 0.73970E+00_JPRB, 0.70018E+00_JPRB, 0.73856E+00_JPRB, 0.81500E+00_JPRB ,&
&  0.93006E+00_JPRB, 0.10501E+01_JPRB, 0.11574E+01_JPRB, 0.11756E+01_JPRB, 0.11883E+01_JPRB, 0.11601E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.10017E+01_JPRB, 0.99250E+00_JPRB, 0.10018E+01_JPRB, 0.10678E+01_JPRB, 0.11103E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.11932E+01_JPRB, 0.12199E+01_JPRB, 0.12458E+01_JPRB, 0.12667E+01_JPRB, 0.12703E+01_JPRB, 0.12635E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.12834E+01_JPRB, 0.12883E+01_JPRB, 0.15403E+01_JPRB, 0.20937E+01_JPRB, 0.23999E+01_JPRB, 0.24808E+01_JPRB ,&
&  0.23979E+01_JPRB, 0.25261E+01_JPRB, 0.36864E+01_JPRB, 0.42024E+01_JPRB, 0.44929E+01_JPRB, 0.51249E+01_JPRB, 0.53588E+01_JPRB ,&
&  0.51227E+01_JPRB, 0.46491E+01_JPRB, 0.42326E+01_JPRB, 0.38383E+01_JPRB, 0.45759E+01_JPRB, 0.47892E+01_JPRB, 0.46866E+01_JPRB ,&
&  0.42010E+01_JPRB, 0.40783E+01_JPRB, 0.36915E+01_JPRB, 0.33261E+01_JPRB, 0.30644E+01_JPRB, 0.30798E+01_JPRB, 0.24682E+01_JPRB ,&
&  0.19235E+01_JPRB, 0.17961E+01_JPRB, 0.13546E+01_JPRB, 0.11376E+01_JPRB, 0.10591E+01_JPRB, 0.10618E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.12973E+01_JPRB, 0.15962E+01_JPRB, 0.20657E+01_JPRB, 0.25660E+01_JPRB, 0.37211E+01_JPRB, 0.46106E+01_JPRB, 0.46070E+01_JPRB ,&
&  0.36935E+01_JPRB, 0.35546E+01_JPRB, 0.31760E+01_JPRB, 0.29875E+01_JPRB, 0.27416E+01_JPRB, 0.24270E+01_JPRB, 0.22501E+01_JPRB ,&
&  0.20503E+01_JPRB, 0.19111E+01_JPRB, 0.18036E+01_JPRB, 0.16434E+01_JPRB, 0.15373E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.17145E+01_JPRB, 0.16120E+01_JPRB, 0.14682E+01_JPRB, 0.13647E+01_JPRB, 0.12739E+01_JPRB, 0.12201E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11574E+01_JPRB, 0.11633E+01_JPRB, 0.11780E+01_JPRB, 0.11749E+01_JPRB, 0.11468E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.98731E+00_JPRB, 0.88639E+00_JPRB, 0.78969E+00_JPRB, 0.78030E+00_JPRB, 0.84592E+00_JPRB, 0.90553E+00_JPRB, 0.99188E+00_JPRB ,&
&  0.11844E+01_JPRB, 0.14774E+01_JPRB, 0.15965E+01_JPRB, 0.16781E+01_JPRB, 0.16874E+01_JPRB, 0.16281E+01_JPRB, 0.15343E+01_JPRB ,&
&  0.13882E+01_JPRB, 0.13586E+01_JPRB, 0.13667E+01_JPRB, 0.13198E+01_JPRB, 0.12662E+01_JPRB, 0.12444E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.12095E+01_JPRB, 0.12415E+01_JPRB, 0.12623E+01_JPRB, 0.13202E+01_JPRB, 0.13627E+01_JPRB, 0.14074E+01_JPRB ,&
&  0.13879E+01_JPRB, 0.13882E+01_JPRB, 0.15791E+01_JPRB, 0.22875E+01_JPRB, 0.29609E+01_JPRB, 0.34650E+01_JPRB, 0.33751E+01_JPRB ,&
&  0.37315E+01_JPRB, 0.40355E+01_JPRB, 0.42338E+01_JPRB, 0.47535E+01_JPRB, 0.40025E+01_JPRB, 0.45292E+01_JPRB, 0.52334E+01_JPRB ,&
&  0.50290E+01_JPRB, 0.45588E+01_JPRB, 0.43232E+01_JPRB, 0.41793E+01_JPRB, 0.45928E+01_JPRB, 0.60529E+01_JPRB, 0.55429E+01_JPRB ,&
&  0.46730E+01_JPRB, 0.42102E+01_JPRB, 0.36735E+01_JPRB, 0.35615E+01_JPRB, 0.30782E+01_JPRB, 0.27061E+01_JPRB, 0.19358E+01_JPRB ,&
&  0.19264E+01_JPRB, 0.17247E+01_JPRB, 0.17283E+01_JPRB, 0.15914E+01_JPRB, 0.16249E+01_JPRB, 0.15691E+01_JPRB, 0.16063E+01_JPRB ,&
&  0.18928E+01_JPRB, 0.22373E+01_JPRB, 0.32081E+01_JPRB, 0.40241E+01_JPRB, 0.59880E+01_JPRB, 0.57656E+01_JPRB, 0.57315E+01_JPRB ,&
&  0.50064E+01_JPRB, 0.46032E+01_JPRB, 0.37442E+01_JPRB, 0.35949E+01_JPRB, 0.31950E+01_JPRB, 0.28246E+01_JPRB, 0.24890E+01_JPRB ,&
&  0.21784E+01_JPRB, 0.19730E+01_JPRB, 0.18422E+01_JPRB, 0.17612E+01_JPRB, 0.17562E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.16662E+01_JPRB, 0.15785E+01_JPRB, 0.14789E+01_JPRB, 0.13747E+01_JPRB, 0.12663E+01_JPRB, 0.12054E+01_JPRB, 0.11695E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.11688E+01_JPRB, 0.11830E+01_JPRB, 0.11869E+01_JPRB, 0.11875E+01_JPRB, 0.11822E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.99471E+00_JPRB, 0.90367E+00_JPRB, 0.79452E+00_JPRB, 0.78939E+00_JPRB, 0.98029E+00_JPRB, 0.11286E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.16539E+01_JPRB, 0.20743E+01_JPRB, 0.21188E+01_JPRB, 0.21577E+01_JPRB, 0.22230E+01_JPRB, 0.22152E+01_JPRB, 0.20116E+01_JPRB ,&
&  0.18953E+01_JPRB, 0.17733E+01_JPRB, 0.15841E+01_JPRB, 0.14366E+01_JPRB, 0.13261E+01_JPRB, 0.12562E+01_JPRB, 0.11967E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11613E+01_JPRB, 0.11734E+01_JPRB, 0.12053E+01_JPRB, 0.12821E+01_JPRB, 0.12866E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.14686E+01_JPRB, 0.15854E+01_JPRB, 0.20136E+01_JPRB, 0.29081E+01_JPRB, 0.40276E+01_JPRB, 0.47238E+01_JPRB, 0.48152E+01_JPRB ,&
&  0.45473E+01_JPRB, 0.45428E+01_JPRB, 0.47156E+01_JPRB, 0.46705E+01_JPRB, 0.46527E+01_JPRB, 0.48116E+01_JPRB, 0.52413E+01_JPRB ,&
&  0.41499E+01_JPRB, 0.34643E+01_JPRB, 0.30818E+01_JPRB, 0.36178E+01_JPRB, 0.56280E+01_JPRB, 0.72447E+01_JPRB, 0.71286E+01_JPRB ,&
&  0.60400E+01_JPRB, 0.50948E+01_JPRB, 0.43357E+01_JPRB, 0.37455E+01_JPRB, 0.36678E+01_JPRB, 0.25431E+01_JPRB, 0.27005E+01_JPRB ,&
&  0.27253E+01_JPRB, 0.27406E+01_JPRB, 0.25023E+01_JPRB, 0.21049E+01_JPRB, 0.19735E+01_JPRB, 0.22427E+01_JPRB, 0.25212E+01_JPRB ,&
&  0.30148E+01_JPRB, 0.35277E+01_JPRB, 0.58129E+01_JPRB, 0.68896E+01_JPRB, 0.79420E+01_JPRB, 0.68454E+01_JPRB, 0.60807E+01_JPRB ,&
&  0.48927E+01_JPRB, 0.45691E+01_JPRB, 0.44162E+01_JPRB, 0.40313E+01_JPRB, 0.35709E+01_JPRB, 0.32090E+01_JPRB, 0.27493E+01_JPRB ,&
&  0.23846E+01_JPRB, 0.20975E+01_JPRB, 0.18868E+01_JPRB, 0.17556E+01_JPRB, 0.17141E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.16518E+01_JPRB, 0.15799E+01_JPRB, 0.14715E+01_JPRB, 0.13131E+01_JPRB, 0.12222E+01_JPRB, 0.11893E+01_JPRB, 0.11963E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.12219E+01_JPRB, 0.12081E+01_JPRB, 0.11894E+01_JPRB, 0.11947E+01_JPRB, 0.11745E+01_JPRB, 0.11025E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.97437E+00_JPRB, 0.84613E+00_JPRB, 0.88984E+00_JPRB, 0.11815E+01_JPRB, 0.13889E+01_JPRB, 0.14993E+01_JPRB ,&
&  0.24137E+01_JPRB, 0.25112E+01_JPRB, 0.21806E+01_JPRB, 0.21755E+01_JPRB, 0.25875E+01_JPRB, 0.23828E+01_JPRB, 0.21464E+01_JPRB ,&
&  0.20473E+01_JPRB, 0.18393E+01_JPRB, 0.16048E+01_JPRB, 0.14256E+01_JPRB, 0.13060E+01_JPRB, 0.12315E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.11471E+01_JPRB, 0.11943E+01_JPRB, 0.12641E+01_JPRB, 0.12794E+01_JPRB, 0.13052E+01_JPRB, 0.14434E+01_JPRB ,&
&  0.15848E+01_JPRB, 0.17498E+01_JPRB, 0.22234E+01_JPRB, 0.30962E+01_JPRB, 0.43764E+01_JPRB, 0.54639E+01_JPRB, 0.66719E+01_JPRB ,&
&  0.59022E+01_JPRB, 0.59761E+01_JPRB, 0.53633E+01_JPRB, 0.57720E+01_JPRB, 0.52613E+01_JPRB, 0.51599E+01_JPRB, 0.51658E+01_JPRB ,&
&  0.38137E+01_JPRB, 0.32884E+01_JPRB, 0.37471E+01_JPRB, 0.54775E+01_JPRB, 0.59931E+01_JPRB, 0.67472E+01_JPRB, 0.82256E+01_JPRB ,&
&  0.65350E+01_JPRB, 0.54887E+01_JPRB, 0.49348E+01_JPRB, 0.48505E+01_JPRB, 0.31217E+01_JPRB, 0.24652E+01_JPRB, 0.36930E+01_JPRB ,&
&  0.34476E+01_JPRB, 0.20505E+01_JPRB, 0.15787E+01_JPRB, 0.18347E+01_JPRB, 0.14954E+01_JPRB, 0.18826E+01_JPRB, 0.30754E+01_JPRB ,&
&  0.38170E+01_JPRB, 0.46502E+01_JPRB, 0.75647E+01_JPRB, 0.71057E+01_JPRB, 0.74618E+01_JPRB, 0.76539E+01_JPRB, 0.60125E+01_JPRB ,&
&  0.52920E+01_JPRB, 0.41750E+01_JPRB, 0.41826E+01_JPRB, 0.37510E+01_JPRB, 0.34813E+01_JPRB, 0.32929E+01_JPRB, 0.29232E+01_JPRB ,&
&  0.25254E+01_JPRB, 0.21908E+01_JPRB, 0.19358E+01_JPRB, 0.17765E+01_JPRB, 0.17070E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.15679E+01_JPRB, 0.14798E+01_JPRB, 0.14286E+01_JPRB, 0.13346E+01_JPRB, 0.12617E+01_JPRB, 0.12411E+01_JPRB, 0.12767E+01_JPRB ,&
&  0.12681E+01_JPRB, 0.12148E+01_JPRB, 0.11590E+01_JPRB, 0.11447E+01_JPRB, 0.11613E+01_JPRB, 0.11543E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.99132E+00_JPRB, 0.98461E+00_JPRB, 0.11219E+01_JPRB, 0.16417E+01_JPRB, 0.17097E+01_JPRB, 0.23243E+01_JPRB ,&
&  0.31278E+01_JPRB, 0.26578E+01_JPRB, 0.22248E+01_JPRB, 0.22217E+01_JPRB, 0.21611E+01_JPRB, 0.20051E+01_JPRB, 0.20098E+01_JPRB ,&
&  0.18549E+01_JPRB, 0.16857E+01_JPRB, 0.15391E+01_JPRB, 0.14275E+01_JPRB, 0.13017E+01_JPRB, 0.12038E+01_JPRB, 0.11594E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.12112E+01_JPRB, 0.13107E+01_JPRB, 0.12873E+01_JPRB, 0.13160E+01_JPRB, 0.13764E+01_JPRB, 0.15301E+01_JPRB ,&
&  0.17012E+01_JPRB, 0.19124E+01_JPRB, 0.23409E+01_JPRB, 0.32907E+01_JPRB, 0.53231E+01_JPRB, 0.69610E+01_JPRB, 0.68553E+01_JPRB ,&
&  0.61436E+01_JPRB, 0.59092E+01_JPRB, 0.61452E+01_JPRB, 0.60377E+01_JPRB, 0.52613E+01_JPRB, 0.52406E+01_JPRB, 0.49859E+01_JPRB ,&
&  0.43891E+01_JPRB, 0.41259E+01_JPRB, 0.42548E+01_JPRB, 0.58533E+01_JPRB, 0.69944E+01_JPRB, 0.70930E+01_JPRB, 0.80429E+01_JPRB ,&
&  0.72455E+01_JPRB, 0.59231E+01_JPRB, 0.45235E+01_JPRB, 0.36644E+01_JPRB, 0.36746E+01_JPRB, 0.34889E+01_JPRB, 0.25916E+01_JPRB ,&
&  0.21641E+01_JPRB, 0.17918E+01_JPRB, 0.15286E+01_JPRB, 0.15268E+01_JPRB, 0.13608E+01_JPRB, 0.17668E+01_JPRB, 0.28337E+01_JPRB ,&
&  0.44732E+01_JPRB, 0.54413E+01_JPRB, 0.68950E+01_JPRB, 0.67524E+01_JPRB, 0.68468E+01_JPRB, 0.80578E+01_JPRB, 0.74715E+01_JPRB ,&
&  0.46796E+01_JPRB, 0.38172E+01_JPRB, 0.38166E+01_JPRB, 0.35866E+01_JPRB, 0.33741E+01_JPRB, 0.31732E+01_JPRB, 0.28299E+01_JPRB ,&
&  0.24211E+01_JPRB, 0.20480E+01_JPRB, 0.18562E+01_JPRB, 0.17832E+01_JPRB, 0.16989E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.13347E+01_JPRB, 0.13135E+01_JPRB, 0.13303E+01_JPRB, 0.13105E+01_JPRB, 0.12774E+01_JPRB, 0.12884E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.12109E+01_JPRB, 0.11281E+01_JPRB, 0.11050E+01_JPRB, 0.11253E+01_JPRB, 0.11370E+01_JPRB, 0.11383E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.10474E+01_JPRB, 0.10271E+01_JPRB, 0.10890E+01_JPRB, 0.14418E+01_JPRB, 0.23537E+01_JPRB, 0.23031E+01_JPRB, 0.33525E+01_JPRB ,&
&  0.36406E+01_JPRB, 0.26576E+01_JPRB, 0.22523E+01_JPRB, 0.20828E+01_JPRB, 0.20222E+01_JPRB, 0.18361E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.15050E+01_JPRB, 0.14930E+01_JPRB, 0.13905E+01_JPRB, 0.13105E+01_JPRB, 0.12256E+01_JPRB, 0.11844E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.13579E+01_JPRB, 0.13433E+01_JPRB, 0.13521E+01_JPRB, 0.14278E+01_JPRB, 0.15079E+01_JPRB, 0.16563E+01_JPRB ,&
&  0.18448E+01_JPRB, 0.20848E+01_JPRB, 0.25649E+01_JPRB, 0.40968E+01_JPRB, 0.56026E+01_JPRB, 0.59739E+01_JPRB, 0.66016E+01_JPRB ,&
&  0.67664E+01_JPRB, 0.68749E+01_JPRB, 0.59766E+01_JPRB, 0.63884E+01_JPRB, 0.53450E+01_JPRB, 0.50045E+01_JPRB, 0.48465E+01_JPRB ,&
&  0.44456E+01_JPRB, 0.48689E+01_JPRB, 0.48076E+01_JPRB, 0.51960E+01_JPRB, 0.70470E+01_JPRB, 0.80961E+01_JPRB, 0.79820E+01_JPRB ,&
&  0.76625E+01_JPRB, 0.63965E+01_JPRB, 0.51240E+01_JPRB, 0.46328E+01_JPRB, 0.60497E+01_JPRB, 0.71724E+01_JPRB, 0.33405E+01_JPRB ,&
&  0.22632E+01_JPRB, 0.20785E+01_JPRB, 0.16197E+01_JPRB, 0.14035E+01_JPRB, 0.14097E+01_JPRB, 0.18380E+01_JPRB, 0.34247E+01_JPRB ,&
&  0.39599E+01_JPRB, 0.64160E+01_JPRB, 0.69643E+01_JPRB, 0.73241E+01_JPRB, 0.70113E+01_JPRB, 0.69309E+01_JPRB, 0.54816E+01_JPRB ,&
&  0.54768E+01_JPRB, 0.47895E+01_JPRB, 0.40254E+01_JPRB, 0.36017E+01_JPRB, 0.31730E+01_JPRB, 0.27737E+01_JPRB, 0.24887E+01_JPRB ,&
&  0.22590E+01_JPRB, 0.19785E+01_JPRB, 0.18022E+01_JPRB, 0.16730E+01_JPRB, 0.14781E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.11261E+01_JPRB, 0.11676E+01_JPRB, 0.12350E+01_JPRB, 0.12507E+01_JPRB, 0.12338E+01_JPRB, 0.12504E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.10928E+01_JPRB, 0.11018E+01_JPRB, 0.11361E+01_JPRB, 0.11859E+01_JPRB, 0.12220E+01_JPRB, 0.11794E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.11061E+01_JPRB, 0.12418E+01_JPRB, 0.20875E+01_JPRB, 0.29682E+01_JPRB, 0.37966E+01_JPRB, 0.43901E+01_JPRB ,&
&  0.40057E+01_JPRB, 0.28329E+01_JPRB, 0.22986E+01_JPRB, 0.21934E+01_JPRB, 0.20981E+01_JPRB, 0.19946E+01_JPRB, 0.16546E+01_JPRB ,&
&  0.14402E+01_JPRB, 0.13981E+01_JPRB, 0.12754E+01_JPRB, 0.12417E+01_JPRB, 0.12183E+01_JPRB, 0.12343E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.14064E+01_JPRB, 0.14077E+01_JPRB, 0.14514E+01_JPRB, 0.15271E+01_JPRB, 0.16282E+01_JPRB, 0.16997E+01_JPRB, 0.18200E+01_JPRB ,&
&  0.20335E+01_JPRB, 0.23742E+01_JPRB, 0.30984E+01_JPRB, 0.47916E+01_JPRB, 0.50457E+01_JPRB, 0.64331E+01_JPRB, 0.66069E+01_JPRB ,&
&  0.65416E+01_JPRB, 0.63921E+01_JPRB, 0.57151E+01_JPRB, 0.58130E+01_JPRB, 0.53045E+01_JPRB, 0.49608E+01_JPRB, 0.46710E+01_JPRB ,&
&  0.50092E+01_JPRB, 0.49014E+01_JPRB, 0.59600E+01_JPRB, 0.58088E+01_JPRB, 0.65397E+01_JPRB, 0.10360E+02_JPRB, 0.81269E+01_JPRB ,&
&  0.68511E+01_JPRB, 0.70880E+01_JPRB, 0.59293E+01_JPRB, 0.57594E+01_JPRB, 0.72691E+01_JPRB, 0.92782E+01_JPRB, 0.79370E+01_JPRB ,&
&  0.29883E+01_JPRB, 0.20216E+01_JPRB, 0.18334E+01_JPRB, 0.16438E+01_JPRB, 0.14276E+01_JPRB, 0.18499E+01_JPRB, 0.65252E+01_JPRB ,&
&  0.47350E+01_JPRB, 0.65887E+01_JPRB, 0.67303E+01_JPRB, 0.67332E+01_JPRB, 0.69633E+01_JPRB, 0.62384E+01_JPRB, 0.54765E+01_JPRB ,&
&  0.44888E+01_JPRB, 0.36562E+01_JPRB, 0.30394E+01_JPRB, 0.25837E+01_JPRB, 0.22879E+01_JPRB, 0.21336E+01_JPRB, 0.20938E+01_JPRB ,&
&  0.20150E+01_JPRB, 0.18080E+01_JPRB, 0.16111E+01_JPRB, 0.13648E+01_JPRB, 0.11610E+01_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.11455E+01_JPRB, 0.11703E+01_JPRB, 0.12080E+01_JPRB, 0.12621E+01_JPRB, 0.12988E+01_JPRB, 0.12792E+01_JPRB, 0.12189E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.12504E+01_JPRB, 0.13285E+01_JPRB, 0.13809E+01_JPRB, 0.14409E+01_JPRB, 0.14269E+01_JPRB, 0.13555E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.13099E+01_JPRB, 0.17613E+01_JPRB, 0.27839E+01_JPRB, 0.34386E+01_JPRB, 0.40503E+01_JPRB, 0.49269E+01_JPRB ,&
&  0.48393E+01_JPRB, 0.30790E+01_JPRB, 0.22788E+01_JPRB, 0.22967E+01_JPRB, 0.22822E+01_JPRB, 0.23192E+01_JPRB, 0.17950E+01_JPRB ,&
&  0.15053E+01_JPRB, 0.13956E+01_JPRB, 0.12866E+01_JPRB, 0.12840E+01_JPRB, 0.13034E+01_JPRB, 0.13441E+01_JPRB, 0.14185E+01_JPRB ,&
&  0.14850E+01_JPRB, 0.15938E+01_JPRB, 0.17211E+01_JPRB, 0.18276E+01_JPRB, 0.19038E+01_JPRB, 0.19617E+01_JPRB, 0.20783E+01_JPRB ,&
&  0.23430E+01_JPRB, 0.27945E+01_JPRB, 0.35926E+01_JPRB, 0.46862E+01_JPRB, 0.56265E+01_JPRB, 0.59175E+01_JPRB, 0.62678E+01_JPRB ,&
&  0.62144E+01_JPRB, 0.61557E+01_JPRB, 0.58302E+01_JPRB, 0.53922E+01_JPRB, 0.49534E+01_JPRB, 0.49311E+01_JPRB, 0.47413E+01_JPRB ,&
&  0.53070E+01_JPRB, 0.59197E+01_JPRB, 0.68531E+01_JPRB, 0.66782E+01_JPRB, 0.70089E+01_JPRB, 0.98039E+01_JPRB, 0.10378E+02_JPRB ,&
&  0.85086E+01_JPRB, 0.63998E+01_JPRB, 0.57929E+01_JPRB, 0.64990E+01_JPRB, 0.74379E+01_JPRB, 0.85007E+01_JPRB, 0.12547E+02_JPRB ,&
&  0.10266E+02_JPRB, 0.44458E+01_JPRB, 0.25150E+01_JPRB, 0.26096E+01_JPRB, 0.15087E+01_JPRB, 0.13704E+01_JPRB, 0.27822E+01_JPRB ,&
&  0.38740E+01_JPRB, 0.49023E+01_JPRB, 0.50274E+01_JPRB, 0.53744E+01_JPRB, 0.55047E+01_JPRB, 0.45524E+01_JPRB, 0.34125E+01_JPRB ,&
&  0.25145E+01_JPRB, 0.20203E+01_JPRB, 0.17553E+01_JPRB, 0.16279E+01_JPRB, 0.16583E+01_JPRB, 0.17410E+01_JPRB, 0.18102E+01_JPRB ,&
&  0.17774E+01_JPRB, 0.15981E+01_JPRB, 0.13337E+01_JPRB, 0.11353E+01_JPRB, 0.10979E+01_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.13312E+01_JPRB, 0.13263E+01_JPRB, 0.13474E+01_JPRB, 0.14051E+01_JPRB, 0.14324E+01_JPRB, 0.14523E+01_JPRB, 0.15762E+01_JPRB ,&
&  0.16797E+01_JPRB, 0.17516E+01_JPRB, 0.18004E+01_JPRB, 0.18326E+01_JPRB, 0.18097E+01_JPRB, 0.16961E+01_JPRB, 0.16055E+01_JPRB ,&
&  0.16276E+01_JPRB, 0.19992E+01_JPRB, 0.27924E+01_JPRB, 0.38659E+01_JPRB, 0.43658E+01_JPRB, 0.57154E+01_JPRB, 0.47436E+01_JPRB ,&
&  0.46861E+01_JPRB, 0.39009E+01_JPRB, 0.25675E+01_JPRB, 0.26550E+01_JPRB, 0.26496E+01_JPRB, 0.27516E+01_JPRB, 0.19456E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.15057E+01_JPRB, 0.13986E+01_JPRB, 0.13959E+01_JPRB, 0.14358E+01_JPRB, 0.15189E+01_JPRB, 0.16111E+01_JPRB ,&
&  0.17577E+01_JPRB, 0.19523E+01_JPRB, 0.21181E+01_JPRB, 0.21999E+01_JPRB, 0.22433E+01_JPRB, 0.22999E+01_JPRB, 0.24710E+01_JPRB ,&
&  0.27654E+01_JPRB, 0.32574E+01_JPRB, 0.40569E+01_JPRB, 0.49060E+01_JPRB, 0.54700E+01_JPRB, 0.54555E+01_JPRB, 0.57193E+01_JPRB ,&
&  0.54829E+01_JPRB, 0.51346E+01_JPRB, 0.52186E+01_JPRB, 0.50821E+01_JPRB, 0.48021E+01_JPRB, 0.46792E+01_JPRB, 0.45934E+01_JPRB ,&
&  0.57597E+01_JPRB, 0.75261E+01_JPRB, 0.95558E+01_JPRB, 0.83744E+01_JPRB, 0.82856E+01_JPRB, 0.10085E+02_JPRB, 0.10541E+02_JPRB ,&
&  0.93528E+01_JPRB, 0.72313E+01_JPRB, 0.67303E+01_JPRB, 0.63310E+01_JPRB, 0.65564E+01_JPRB, 0.72510E+01_JPRB, 0.92268E+01_JPRB ,&
&  0.10053E+02_JPRB, 0.10155E+02_JPRB, 0.52417E+01_JPRB, 0.22129E+01_JPRB, 0.15244E+01_JPRB, 0.16784E+01_JPRB, 0.21497E+01_JPRB ,&
&  0.29330E+01_JPRB, 0.33275E+01_JPRB, 0.34398E+01_JPRB, 0.44868E+01_JPRB, 0.39891E+01_JPRB, 0.30989E+01_JPRB, 0.23628E+01_JPRB ,&
&  0.18783E+01_JPRB, 0.15248E+01_JPRB, 0.13500E+01_JPRB, 0.13406E+01_JPRB, 0.14635E+01_JPRB, 0.15850E+01_JPRB, 0.16389E+01_JPRB ,&
&  0.15167E+01_JPRB, 0.13069E+01_JPRB, 0.12127E+01_JPRB, 0.12123E+01_JPRB, 0.12712E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.15683E+01_JPRB, 0.16161E+01_JPRB, 0.16966E+01_JPRB, 0.18113E+01_JPRB, 0.19243E+01_JPRB, 0.20672E+01_JPRB, 0.22369E+01_JPRB ,&
&  0.23470E+01_JPRB, 0.23605E+01_JPRB, 0.23404E+01_JPRB, 0.22716E+01_JPRB, 0.22336E+01_JPRB, 0.22275E+01_JPRB, 0.23914E+01_JPRB ,&
&  0.28773E+01_JPRB, 0.36374E+01_JPRB, 0.44139E+01_JPRB, 0.49472E+01_JPRB, 0.51694E+01_JPRB, 0.59127E+01_JPRB, 0.60326E+01_JPRB ,&
&  0.44030E+01_JPRB, 0.51508E+01_JPRB, 0.29410E+01_JPRB, 0.31037E+01_JPRB, 0.32662E+01_JPRB, 0.36221E+01_JPRB, 0.25671E+01_JPRB ,&
&  0.19004E+01_JPRB, 0.17683E+01_JPRB, 0.16990E+01_JPRB, 0.17009E+01_JPRB, 0.17241E+01_JPRB, 0.18530E+01_JPRB, 0.20340E+01_JPRB ,&
&  0.22168E+01_JPRB, 0.23626E+01_JPRB, 0.24688E+01_JPRB, 0.25292E+01_JPRB, 0.25987E+01_JPRB, 0.26947E+01_JPRB, 0.29489E+01_JPRB ,&
&  0.32997E+01_JPRB, 0.38377E+01_JPRB, 0.44855E+01_JPRB, 0.51581E+01_JPRB, 0.51452E+01_JPRB, 0.50709E+01_JPRB, 0.52165E+01_JPRB ,&
&  0.50567E+01_JPRB, 0.47416E+01_JPRB, 0.47971E+01_JPRB, 0.45598E+01_JPRB, 0.43262E+01_JPRB, 0.45981E+01_JPRB, 0.47972E+01_JPRB ,&
&  0.66702E+01_JPRB, 0.80307E+01_JPRB, 0.10860E+02_JPRB, 0.10112E+02_JPRB, 0.89175E+01_JPRB, 0.94191E+01_JPRB, 0.84021E+01_JPRB ,&
&  0.68625E+01_JPRB, 0.63613E+01_JPRB, 0.60495E+01_JPRB, 0.56812E+01_JPRB, 0.57247E+01_JPRB, 0.62204E+01_JPRB, 0.72188E+01_JPRB ,&
&  0.77164E+01_JPRB, 0.73099E+01_JPRB, 0.49628E+01_JPRB, 0.22809E+01_JPRB, 0.16948E+01_JPRB, 0.16576E+01_JPRB, 0.19962E+01_JPRB ,&
&  0.27223E+01_JPRB, 0.29366E+01_JPRB, 0.29084E+01_JPRB, 0.28311E+01_JPRB, 0.22725E+01_JPRB, 0.17929E+01_JPRB, 0.14612E+01_JPRB ,&
&  0.13513E+01_JPRB, 0.12982E+01_JPRB, 0.12849E+01_JPRB, 0.13841E+01_JPRB, 0.14774E+01_JPRB, 0.15011E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.12572E+01_JPRB, 0.13465E+01_JPRB, 0.14382E+01_JPRB, 0.15155E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.18230E+01_JPRB, 0.19373E+01_JPRB, 0.20913E+01_JPRB, 0.23130E+01_JPRB, 0.26074E+01_JPRB, 0.29452E+01_JPRB, 0.30074E+01_JPRB ,&
&  0.29992E+01_JPRB, 0.29435E+01_JPRB, 0.30653E+01_JPRB, 0.31962E+01_JPRB, 0.33556E+01_JPRB, 0.36053E+01_JPRB, 0.39787E+01_JPRB ,&
&  0.44457E+01_JPRB, 0.49103E+01_JPRB, 0.53524E+01_JPRB, 0.57975E+01_JPRB, 0.62608E+01_JPRB, 0.71804E+01_JPRB, 0.75824E+01_JPRB ,&
&  0.69823E+01_JPRB, 0.41555E+01_JPRB, 0.35897E+01_JPRB, 0.31845E+01_JPRB, 0.32667E+01_JPRB, 0.36465E+01_JPRB, 0.33620E+01_JPRB ,&
&  0.26223E+01_JPRB, 0.23184E+01_JPRB, 0.22193E+01_JPRB, 0.21853E+01_JPRB, 0.21277E+01_JPRB, 0.22869E+01_JPRB, 0.24395E+01_JPRB ,&
&  0.25587E+01_JPRB, 0.26388E+01_JPRB, 0.27279E+01_JPRB, 0.27999E+01_JPRB, 0.28916E+01_JPRB, 0.30611E+01_JPRB, 0.33808E+01_JPRB ,&
&  0.37356E+01_JPRB, 0.41775E+01_JPRB, 0.46321E+01_JPRB, 0.48729E+01_JPRB, 0.48065E+01_JPRB, 0.47390E+01_JPRB, 0.47658E+01_JPRB ,&
&  0.47243E+01_JPRB, 0.46947E+01_JPRB, 0.46071E+01_JPRB, 0.44605E+01_JPRB, 0.42825E+01_JPRB, 0.49608E+01_JPRB, 0.57938E+01_JPRB ,&
&  0.74421E+01_JPRB, 0.85829E+01_JPRB, 0.10958E+02_JPRB, 0.97199E+01_JPRB, 0.85330E+01_JPRB, 0.78361E+01_JPRB, 0.68804E+01_JPRB ,&
&  0.56425E+01_JPRB, 0.53870E+01_JPRB, 0.49803E+01_JPRB, 0.42598E+01_JPRB, 0.40011E+01_JPRB, 0.40813E+01_JPRB, 0.53420E+01_JPRB ,&
&  0.59479E+01_JPRB, 0.58518E+01_JPRB, 0.34590E+01_JPRB, 0.21715E+01_JPRB, 0.17640E+01_JPRB, 0.17075E+01_JPRB, 0.20911E+01_JPRB ,&
&  0.27376E+01_JPRB, 0.26136E+01_JPRB, 0.22400E+01_JPRB, 0.21371E+01_JPRB, 0.17574E+01_JPRB, 0.14040E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11883E+01_JPRB, 0.11786E+01_JPRB, 0.12075E+01_JPRB, 0.12880E+01_JPRB, 0.13139E+01_JPRB, 0.13664E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.14216E+01_JPRB, 0.15035E+01_JPRB, 0.15827E+01_JPRB, 0.16599E+01_JPRB, 0.17286E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.19159E+01_JPRB, 0.20557E+01_JPRB, 0.22267E+01_JPRB, 0.24287E+01_JPRB, 0.26506E+01_JPRB, 0.28453E+01_JPRB, 0.29920E+01_JPRB ,&
&  0.32007E+01_JPRB, 0.34316E+01_JPRB, 0.36714E+01_JPRB, 0.38727E+01_JPRB, 0.40311E+01_JPRB, 0.42312E+01_JPRB, 0.44566E+01_JPRB ,&
&  0.46692E+01_JPRB, 0.48915E+01_JPRB, 0.51667E+01_JPRB, 0.55054E+01_JPRB, 0.59638E+01_JPRB, 0.65771E+01_JPRB, 0.69256E+01_JPRB ,&
&  0.56417E+01_JPRB, 0.33857E+01_JPRB, 0.37120E+01_JPRB, 0.33890E+01_JPRB, 0.33328E+01_JPRB, 0.34022E+01_JPRB, 0.31601E+01_JPRB ,&
&  0.28198E+01_JPRB, 0.27113E+01_JPRB, 0.27374E+01_JPRB, 0.26646E+01_JPRB, 0.25057E+01_JPRB, 0.25995E+01_JPRB, 0.26902E+01_JPRB ,&
&  0.27693E+01_JPRB, 0.28441E+01_JPRB, 0.29164E+01_JPRB, 0.29690E+01_JPRB, 0.30792E+01_JPRB, 0.32719E+01_JPRB, 0.35718E+01_JPRB ,&
&  0.39217E+01_JPRB, 0.42229E+01_JPRB, 0.44891E+01_JPRB, 0.46660E+01_JPRB, 0.46249E+01_JPRB, 0.45908E+01_JPRB, 0.46590E+01_JPRB ,&
&  0.47492E+01_JPRB, 0.47016E+01_JPRB, 0.48345E+01_JPRB, 0.51795E+01_JPRB, 0.56362E+01_JPRB, 0.57931E+01_JPRB, 0.70931E+01_JPRB ,&
&  0.82549E+01_JPRB, 0.88097E+01_JPRB, 0.95198E+01_JPRB, 0.11208E+02_JPRB, 0.72926E+01_JPRB, 0.65482E+01_JPRB, 0.54789E+01_JPRB ,&
&  0.54321E+01_JPRB, 0.49645E+01_JPRB, 0.41971E+01_JPRB, 0.31562E+01_JPRB, 0.27479E+01_JPRB, 0.26157E+01_JPRB, 0.34290E+01_JPRB ,&
&  0.44667E+01_JPRB, 0.43542E+01_JPRB, 0.29677E+01_JPRB, 0.22974E+01_JPRB, 0.20567E+01_JPRB, 0.23396E+01_JPRB, 0.27276E+01_JPRB ,&
&  0.28276E+01_JPRB, 0.23023E+01_JPRB, 0.20780E+01_JPRB, 0.23107E+01_JPRB, 0.17091E+01_JPRB, 0.11535E+01_JPRB, 0.92536E+00_JPRB ,&
&  0.92162E+00_JPRB, 0.96947E+00_JPRB, 0.10521E+01_JPRB, 0.11644E+01_JPRB, 0.12211E+01_JPRB, 0.12915E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.14763E+01_JPRB, 0.15768E+01_JPRB, 0.16620E+01_JPRB, 0.17185E+01_JPRB, 0.18111E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.17848E+01_JPRB, 0.19164E+01_JPRB, 0.20329E+01_JPRB, 0.21596E+01_JPRB, 0.22762E+01_JPRB, 0.23882E+01_JPRB, 0.25456E+01_JPRB ,&
&  0.27299E+01_JPRB, 0.29421E+01_JPRB, 0.31453E+01_JPRB, 0.33045E+01_JPRB, 0.34764E+01_JPRB, 0.35947E+01_JPRB, 0.37008E+01_JPRB ,&
&  0.37494E+01_JPRB, 0.38191E+01_JPRB, 0.38733E+01_JPRB, 0.39722E+01_JPRB, 0.41320E+01_JPRB, 0.43187E+01_JPRB, 0.44992E+01_JPRB ,&
&  0.45846E+01_JPRB, 0.43204E+01_JPRB, 0.35918E+01_JPRB, 0.25815E+01_JPRB, 0.23638E+01_JPRB, 0.27227E+01_JPRB, 0.27274E+01_JPRB ,&
&  0.27057E+01_JPRB, 0.26388E+01_JPRB, 0.25804E+01_JPRB, 0.26432E+01_JPRB, 0.26751E+01_JPRB, 0.27188E+01_JPRB, 0.27480E+01_JPRB ,&
&  0.27895E+01_JPRB, 0.28636E+01_JPRB, 0.29057E+01_JPRB, 0.29535E+01_JPRB, 0.30487E+01_JPRB, 0.32365E+01_JPRB, 0.35065E+01_JPRB ,&
&  0.38303E+01_JPRB, 0.41262E+01_JPRB, 0.43118E+01_JPRB, 0.44538E+01_JPRB, 0.44328E+01_JPRB, 0.45315E+01_JPRB, 0.47796E+01_JPRB ,&
&  0.51169E+01_JPRB, 0.53176E+01_JPRB, 0.56796E+01_JPRB, 0.63037E+01_JPRB, 0.67538E+01_JPRB, 0.65965E+01_JPRB, 0.74087E+01_JPRB ,&
&  0.83113E+01_JPRB, 0.86237E+01_JPRB, 0.85334E+01_JPRB, 0.10826E+02_JPRB, 0.68790E+01_JPRB, 0.67283E+01_JPRB, 0.61918E+01_JPRB ,&
&  0.55276E+01_JPRB, 0.52804E+01_JPRB, 0.37792E+01_JPRB, 0.25612E+01_JPRB, 0.21351E+01_JPRB, 0.20290E+01_JPRB, 0.27120E+01_JPRB ,&
&  0.35377E+01_JPRB, 0.30576E+01_JPRB, 0.23552E+01_JPRB, 0.21108E+01_JPRB, 0.21409E+01_JPRB, 0.28400E+01_JPRB, 0.31794E+01_JPRB ,&
&  0.24922E+01_JPRB, 0.20837E+01_JPRB, 0.20073E+01_JPRB, 0.25891E+01_JPRB, 0.20313E+01_JPRB, 0.10463E+01_JPRB, 0.81339E+00_JPRB ,&
&  0.81574E+00_JPRB, 0.87543E+00_JPRB, 0.94520E+00_JPRB, 0.10101E+01_JPRB, 0.10659E+01_JPRB, 0.11346E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.14218E+01_JPRB, 0.15162E+01_JPRB, 0.16061E+01_JPRB, 0.16870E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.13379E+01_JPRB, 0.14524E+01_JPRB, 0.15609E+01_JPRB, 0.16369E+01_JPRB, 0.16768E+01_JPRB, 0.17085E+01_JPRB, 0.17197E+01_JPRB ,&
&  0.17390E+01_JPRB, 0.17409E+01_JPRB, 0.18035E+01_JPRB, 0.19481E+01_JPRB, 0.21981E+01_JPRB, 0.23525E+01_JPRB, 0.25198E+01_JPRB ,&
&  0.25218E+01_JPRB, 0.25265E+01_JPRB, 0.25263E+01_JPRB, 0.25713E+01_JPRB, 0.26588E+01_JPRB, 0.27826E+01_JPRB, 0.30866E+01_JPRB ,&
&  0.36983E+01_JPRB, 0.40812E+01_JPRB, 0.39785E+01_JPRB, 0.35451E+01_JPRB, 0.28722E+01_JPRB, 0.27966E+01_JPRB, 0.32175E+01_JPRB ,&
&  0.32347E+01_JPRB, 0.28726E+01_JPRB, 0.26326E+01_JPRB, 0.25993E+01_JPRB, 0.26079E+01_JPRB, 0.26228E+01_JPRB, 0.26371E+01_JPRB ,&
&  0.26753E+01_JPRB, 0.27279E+01_JPRB, 0.27652E+01_JPRB, 0.28154E+01_JPRB, 0.29086E+01_JPRB, 0.30481E+01_JPRB, 0.32255E+01_JPRB ,&
&  0.34635E+01_JPRB, 0.36723E+01_JPRB, 0.37989E+01_JPRB, 0.38695E+01_JPRB, 0.40269E+01_JPRB, 0.44056E+01_JPRB, 0.49728E+01_JPRB ,&
&  0.54980E+01_JPRB, 0.57688E+01_JPRB, 0.63572E+01_JPRB, 0.69693E+01_JPRB, 0.70558E+01_JPRB, 0.70156E+01_JPRB, 0.74027E+01_JPRB ,&
&  0.76772E+01_JPRB, 0.77567E+01_JPRB, 0.67616E+01_JPRB, 0.93251E+01_JPRB, 0.95488E+01_JPRB, 0.80300E+01_JPRB, 0.63952E+01_JPRB ,&
&  0.56612E+01_JPRB, 0.51183E+01_JPRB, 0.40371E+01_JPRB, 0.28885E+01_JPRB, 0.23689E+01_JPRB, 0.25512E+01_JPRB, 0.29536E+01_JPRB ,&
&  0.31138E+01_JPRB, 0.24224E+01_JPRB, 0.21492E+01_JPRB, 0.21411E+01_JPRB, 0.20792E+01_JPRB, 0.28583E+01_JPRB, 0.26440E+01_JPRB ,&
&  0.19173E+01_JPRB, 0.17450E+01_JPRB, 0.18405E+01_JPRB, 0.24244E+01_JPRB, 0.30625E+01_JPRB, 0.98012E+00_JPRB, 0.71610E+00_JPRB ,&
&  0.72540E+00_JPRB, 0.76125E+00_JPRB, 0.80710E+00_JPRB, 0.82935E+00_JPRB, 0.83354E+00_JPRB, 0.87690E+00_JPRB, 0.94504E+00_JPRB ,&
&  0.98288E+00_JPRB, 0.10160E+01_JPRB, 0.10581E+01_JPRB, 0.11623E+01_JPRB, 0.12412E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.83677E+00_JPRB, 0.89353E+00_JPRB, 0.93577E+00_JPRB, 0.97631E+00_JPRB, 0.99446E+00_JPRB, 0.98514E+00_JPRB, 0.93655E+00_JPRB ,&
&  0.89904E+00_JPRB, 0.86781E+00_JPRB, 0.87004E+00_JPRB, 0.86266E+00_JPRB, 0.91654E+00_JPRB, 0.10152E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.11386E+01_JPRB, 0.11217E+01_JPRB, 0.11460E+01_JPRB, 0.11420E+01_JPRB, 0.11902E+01_JPRB, 0.13838E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.20218E+01_JPRB, 0.25173E+01_JPRB, 0.33595E+01_JPRB, 0.37162E+01_JPRB, 0.31819E+01_JPRB, 0.36349E+01_JPRB ,&
&  0.32438E+01_JPRB, 0.25954E+01_JPRB, 0.24517E+01_JPRB, 0.23210E+01_JPRB, 0.23419E+01_JPRB, 0.23593E+01_JPRB, 0.24200E+01_JPRB ,&
&  0.24895E+01_JPRB, 0.25636E+01_JPRB, 0.26274E+01_JPRB, 0.26883E+01_JPRB, 0.27603E+01_JPRB, 0.28437E+01_JPRB, 0.29573E+01_JPRB ,&
&  0.30471E+01_JPRB, 0.31228E+01_JPRB, 0.32625E+01_JPRB, 0.33433E+01_JPRB, 0.37718E+01_JPRB, 0.43889E+01_JPRB, 0.50096E+01_JPRB ,&
&  0.54414E+01_JPRB, 0.57471E+01_JPRB, 0.62830E+01_JPRB, 0.64615E+01_JPRB, 0.65541E+01_JPRB, 0.66451E+01_JPRB, 0.65826E+01_JPRB ,&
&  0.62882E+01_JPRB, 0.51894E+01_JPRB, 0.47694E+01_JPRB, 0.73585E+01_JPRB, 0.71273E+01_JPRB, 0.56584E+01_JPRB, 0.56551E+01_JPRB ,&
&  0.51979E+01_JPRB, 0.48531E+01_JPRB, 0.41090E+01_JPRB, 0.32600E+01_JPRB, 0.26600E+01_JPRB, 0.24386E+01_JPRB, 0.28200E+01_JPRB ,&
&  0.24238E+01_JPRB, 0.19538E+01_JPRB, 0.20556E+01_JPRB, 0.22288E+01_JPRB, 0.22159E+01_JPRB, 0.24937E+01_JPRB, 0.22428E+01_JPRB ,&
&  0.18612E+01_JPRB, 0.15986E+01_JPRB, 0.19156E+01_JPRB, 0.25871E+01_JPRB, 0.17406E+01_JPRB, 0.81407E+00_JPRB, 0.62471E+00_JPRB ,&
&  0.63082E+00_JPRB, 0.64374E+00_JPRB, 0.65551E+00_JPRB, 0.66201E+00_JPRB, 0.66775E+00_JPRB, 0.68727E+00_JPRB, 0.70011E+00_JPRB ,&
&  0.71204E+00_JPRB, 0.74311E+00_JPRB, 0.75733E+00_JPRB, 0.77843E+00_JPRB, 0.81234E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.70263E+00_JPRB, 0.70734E+00_JPRB, 0.70386E+00_JPRB, 0.70872E+00_JPRB, 0.71786E+00_JPRB, 0.73979E+00_JPRB, 0.75489E+00_JPRB ,&
&  0.75908E+00_JPRB, 0.77073E+00_JPRB, 0.75450E+00_JPRB, 0.76335E+00_JPRB, 0.79798E+00_JPRB, 0.82842E+00_JPRB, 0.86095E+00_JPRB ,&
&  0.93663E+00_JPRB, 0.95330E+00_JPRB, 0.95157E+00_JPRB, 0.96130E+00_JPRB, 0.95058E+00_JPRB, 0.10216E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.98759E+00_JPRB, 0.10121E+01_JPRB, 0.11956E+01_JPRB, 0.16889E+01_JPRB, 0.22025E+01_JPRB, 0.23454E+01_JPRB, 0.28797E+01_JPRB ,&
&  0.29867E+01_JPRB, 0.20591E+01_JPRB, 0.20213E+01_JPRB, 0.17252E+01_JPRB, 0.17316E+01_JPRB, 0.18117E+01_JPRB, 0.18469E+01_JPRB ,&
&  0.18968E+01_JPRB, 0.18947E+01_JPRB, 0.19169E+01_JPRB, 0.19641E+01_JPRB, 0.19929E+01_JPRB, 0.20302E+01_JPRB, 0.20738E+01_JPRB ,&
&  0.20110E+01_JPRB, 0.21009E+01_JPRB, 0.23751E+01_JPRB, 0.28466E+01_JPRB, 0.33940E+01_JPRB, 0.41480E+01_JPRB, 0.44998E+01_JPRB ,&
&  0.47565E+01_JPRB, 0.50471E+01_JPRB, 0.47950E+01_JPRB, 0.49584E+01_JPRB, 0.53802E+01_JPRB, 0.50568E+01_JPRB, 0.48451E+01_JPRB ,&
&  0.46164E+01_JPRB, 0.36483E+01_JPRB, 0.35269E+01_JPRB, 0.51147E+01_JPRB, 0.53248E+01_JPRB, 0.49516E+01_JPRB, 0.49587E+01_JPRB ,&
&  0.46991E+01_JPRB, 0.42706E+01_JPRB, 0.35774E+01_JPRB, 0.30380E+01_JPRB, 0.25640E+01_JPRB, 0.21466E+01_JPRB, 0.21809E+01_JPRB ,&
&  0.17995E+01_JPRB, 0.18213E+01_JPRB, 0.21881E+01_JPRB, 0.24510E+01_JPRB, 0.26631E+01_JPRB, 0.31314E+01_JPRB, 0.23599E+01_JPRB ,&
&  0.17102E+01_JPRB, 0.16919E+01_JPRB, 0.24921E+01_JPRB, 0.32504E+01_JPRB, 0.14852E+01_JPRB, 0.83196E+00_JPRB, 0.54610E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.58028E+00_JPRB, 0.58539E+00_JPRB, 0.59090E+00_JPRB, 0.60560E+00_JPRB, 0.61755E+00_JPRB, 0.64013E+00_JPRB ,&
&  0.65304E+00_JPRB, 0.66314E+00_JPRB, 0.67411E+00_JPRB, 0.69381E+00_JPRB, 0.69354E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.76860E+00_JPRB, 0.78420E+00_JPRB, 0.79668E+00_JPRB, 0.82295E+00_JPRB, 0.86033E+00_JPRB, 0.87991E+00_JPRB, 0.90934E+00_JPRB ,&
&  0.93128E+00_JPRB, 0.96251E+00_JPRB, 0.99386E+00_JPRB, 0.10309E+01_JPRB, 0.10647E+01_JPRB, 0.10865E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.11727E+01_JPRB, 0.11749E+01_JPRB, 0.11993E+01_JPRB, 0.12460E+01_JPRB, 0.12557E+01_JPRB, 0.13100E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.12990E+01_JPRB, 0.13175E+01_JPRB, 0.14297E+01_JPRB, 0.15661E+01_JPRB, 0.18165E+01_JPRB, 0.23490E+01_JPRB ,&
&  0.22685E+01_JPRB, 0.17320E+01_JPRB, 0.14855E+01_JPRB, 0.12572E+01_JPRB, 0.12386E+01_JPRB, 0.12535E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.11918E+01_JPRB, 0.11879E+01_JPRB, 0.12095E+01_JPRB, 0.13045E+01_JPRB, 0.14966E+01_JPRB, 0.16505E+01_JPRB ,&
&  0.17091E+01_JPRB, 0.19890E+01_JPRB, 0.23292E+01_JPRB, 0.27750E+01_JPRB, 0.30981E+01_JPRB, 0.39292E+01_JPRB, 0.40118E+01_JPRB ,&
&  0.39239E+01_JPRB, 0.38324E+01_JPRB, 0.32916E+01_JPRB, 0.34662E+01_JPRB, 0.38484E+01_JPRB, 0.37159E+01_JPRB, 0.33082E+01_JPRB ,&
&  0.32858E+01_JPRB, 0.30951E+01_JPRB, 0.32008E+01_JPRB, 0.41943E+01_JPRB, 0.44911E+01_JPRB, 0.45345E+01_JPRB, 0.42912E+01_JPRB ,&
&  0.39613E+01_JPRB, 0.34665E+01_JPRB, 0.30215E+01_JPRB, 0.27851E+01_JPRB, 0.26321E+01_JPRB, 0.22639E+01_JPRB, 0.19526E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.19753E+01_JPRB, 0.23054E+01_JPRB, 0.25437E+01_JPRB, 0.31708E+01_JPRB, 0.28892E+01_JPRB, 0.31440E+01_JPRB ,&
&  0.16561E+01_JPRB, 0.16383E+01_JPRB, 0.12494E+01_JPRB, 0.16883E+01_JPRB, 0.99430E+00_JPRB, 0.62939E+00_JPRB, 0.57801E+00_JPRB ,&
&  0.59286E+00_JPRB, 0.60254E+00_JPRB, 0.61800E+00_JPRB, 0.62952E+00_JPRB, 0.64381E+00_JPRB, 0.66935E+00_JPRB, 0.69661E+00_JPRB ,&
&  0.72495E+00_JPRB, 0.72071E+00_JPRB, 0.72610E+00_JPRB, 0.73760E+00_JPRB, 0.75858E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.83573E+00_JPRB, 0.86780E+00_JPRB, 0.90014E+00_JPRB, 0.93297E+00_JPRB, 0.96309E+00_JPRB, 0.99250E+00_JPRB, 0.10225E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.10853E+01_JPRB, 0.11208E+01_JPRB, 0.11605E+01_JPRB, 0.11980E+01_JPRB, 0.12248E+01_JPRB, 0.12593E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.13200E+01_JPRB, 0.13270E+01_JPRB, 0.13819E+01_JPRB, 0.14346E+01_JPRB, 0.14656E+01_JPRB, 0.15111E+01_JPRB ,&
&  0.15533E+01_JPRB, 0.15103E+01_JPRB, 0.14914E+01_JPRB, 0.15333E+01_JPRB, 0.15755E+01_JPRB, 0.15330E+01_JPRB, 0.17875E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.15416E+01_JPRB, 0.13739E+01_JPRB, 0.11927E+01_JPRB, 0.11397E+01_JPRB, 0.10025E+01_JPRB, 0.93224E+00_JPRB ,&
&  0.10042E+01_JPRB, 0.10405E+01_JPRB, 0.10896E+01_JPRB, 0.11819E+01_JPRB, 0.13707E+01_JPRB, 0.16868E+01_JPRB, 0.19455E+01_JPRB ,&
&  0.22588E+01_JPRB, 0.25707E+01_JPRB, 0.29278E+01_JPRB, 0.33222E+01_JPRB, 0.37616E+01_JPRB, 0.40187E+01_JPRB, 0.39813E+01_JPRB ,&
&  0.37991E+01_JPRB, 0.35635E+01_JPRB, 0.27003E+01_JPRB, 0.30541E+01_JPRB, 0.36110E+01_JPRB, 0.35029E+01_JPRB, 0.29600E+01_JPRB ,&
&  0.26661E+01_JPRB, 0.26325E+01_JPRB, 0.29491E+01_JPRB, 0.33151E+01_JPRB, 0.35521E+01_JPRB, 0.36369E+01_JPRB, 0.34458E+01_JPRB ,&
&  0.32052E+01_JPRB, 0.27585E+01_JPRB, 0.25581E+01_JPRB, 0.25947E+01_JPRB, 0.25360E+01_JPRB, 0.23383E+01_JPRB, 0.19807E+01_JPRB ,&
&  0.18992E+01_JPRB, 0.20624E+01_JPRB, 0.22848E+01_JPRB, 0.25048E+01_JPRB, 0.28344E+01_JPRB, 0.42807E+01_JPRB, 0.41272E+01_JPRB ,&
&  0.16906E+01_JPRB, 0.13181E+01_JPRB, 0.71310E+00_JPRB, 0.80240E+00_JPRB, 0.80983E+00_JPRB, 0.69952E+00_JPRB, 0.73800E+00_JPRB ,&
&  0.77427E+00_JPRB, 0.74511E+00_JPRB, 0.73769E+00_JPRB, 0.71620E+00_JPRB, 0.72052E+00_JPRB, 0.72045E+00_JPRB, 0.73927E+00_JPRB ,&
&  0.75249E+00_JPRB, 0.75957E+00_JPRB, 0.77540E+00_JPRB, 0.79513E+00_JPRB, 0.81178E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.87316E+00_JPRB, 0.91013E+00_JPRB, 0.94720E+00_JPRB, 0.98140E+00_JPRB, 0.10151E+01_JPRB, 0.10480E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.11159E+01_JPRB, 0.11524E+01_JPRB, 0.11886E+01_JPRB, 0.12329E+01_JPRB, 0.12758E+01_JPRB, 0.13042E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.14637E+01_JPRB, 0.15151E+01_JPRB, 0.15962E+01_JPRB, 0.16652E+01_JPRB, 0.17172E+01_JPRB, 0.17915E+01_JPRB ,&
&  0.18236E+01_JPRB, 0.18300E+01_JPRB, 0.18377E+01_JPRB, 0.18950E+01_JPRB, 0.19407E+01_JPRB, 0.21568E+01_JPRB, 0.18698E+01_JPRB ,&
&  0.18508E+01_JPRB, 0.16247E+01_JPRB, 0.14819E+01_JPRB, 0.13787E+01_JPRB, 0.12927E+01_JPRB, 0.12205E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.12345E+01_JPRB, 0.12986E+01_JPRB, 0.14055E+01_JPRB, 0.16061E+01_JPRB, 0.19532E+01_JPRB, 0.22414E+01_JPRB ,&
&  0.25609E+01_JPRB, 0.29334E+01_JPRB, 0.32851E+01_JPRB, 0.36551E+01_JPRB, 0.40687E+01_JPRB, 0.44946E+01_JPRB, 0.48444E+01_JPRB ,&
&  0.48917E+01_JPRB, 0.47954E+01_JPRB, 0.40634E+01_JPRB, 0.45020E+01_JPRB, 0.47962E+01_JPRB, 0.42665E+01_JPRB, 0.32054E+01_JPRB ,&
&  0.25106E+01_JPRB, 0.20956E+01_JPRB, 0.21871E+01_JPRB, 0.25327E+01_JPRB, 0.26781E+01_JPRB, 0.27084E+01_JPRB, 0.25775E+01_JPRB ,&
&  0.23965E+01_JPRB, 0.21373E+01_JPRB, 0.20435E+01_JPRB, 0.20913E+01_JPRB, 0.22015E+01_JPRB, 0.22489E+01_JPRB, 0.20932E+01_JPRB ,&
&  0.20452E+01_JPRB, 0.20190E+01_JPRB, 0.20109E+01_JPRB, 0.20081E+01_JPRB, 0.21957E+01_JPRB, 0.21316E+01_JPRB, 0.23219E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.96903E+00_JPRB, 0.93084E+00_JPRB, 0.77535E+00_JPRB, 0.84631E+00_JPRB, 0.83480E+00_JPRB, 0.99867E+00_JPRB ,&
&  0.11303E+01_JPRB, 0.11829E+01_JPRB, 0.11131E+01_JPRB, 0.93457E+00_JPRB, 0.79899E+00_JPRB, 0.74783E+00_JPRB, 0.75418E+00_JPRB ,&
&  0.76324E+00_JPRB, 0.76982E+00_JPRB, 0.78898E+00_JPRB, 0.81630E+00_JPRB, 0.84173E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.84027E+00_JPRB, 0.87622E+00_JPRB, 0.91530E+00_JPRB, 0.94828E+00_JPRB, 0.98800E+00_JPRB, 0.10429E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.11859E+01_JPRB, 0.12200E+01_JPRB, 0.12650E+01_JPRB, 0.13221E+01_JPRB, 0.14031E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.15737E+01_JPRB, 0.16684E+01_JPRB, 0.17473E+01_JPRB, 0.18689E+01_JPRB, 0.19547E+01_JPRB, 0.20886E+01_JPRB, 0.21987E+01_JPRB ,&
&  0.22575E+01_JPRB, 0.22883E+01_JPRB, 0.23279E+01_JPRB, 0.24019E+01_JPRB, 0.24095E+01_JPRB, 0.25458E+01_JPRB, 0.20699E+01_JPRB ,&
&  0.20762E+01_JPRB, 0.19628E+01_JPRB, 0.17982E+01_JPRB, 0.17221E+01_JPRB, 0.16567E+01_JPRB, 0.16177E+01_JPRB, 0.16108E+01_JPRB ,&
&  0.16214E+01_JPRB, 0.15853E+01_JPRB, 0.14870E+01_JPRB, 0.15223E+01_JPRB, 0.16848E+01_JPRB, 0.20221E+01_JPRB, 0.23308E+01_JPRB ,&
&  0.26703E+01_JPRB, 0.30196E+01_JPRB, 0.33841E+01_JPRB, 0.38522E+01_JPRB, 0.44589E+01_JPRB, 0.51035E+01_JPRB, 0.57433E+01_JPRB ,&
&  0.59941E+01_JPRB, 0.61087E+01_JPRB, 0.55825E+01_JPRB, 0.56425E+01_JPRB, 0.57415E+01_JPRB, 0.46478E+01_JPRB, 0.31965E+01_JPRB ,&
&  0.19779E+01_JPRB, 0.18075E+01_JPRB, 0.19678E+01_JPRB, 0.20818E+01_JPRB, 0.19433E+01_JPRB, 0.18325E+01_JPRB, 0.16809E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.13908E+01_JPRB, 0.14107E+01_JPRB, 0.15424E+01_JPRB, 0.17050E+01_JPRB, 0.19033E+01_JPRB, 0.19587E+01_JPRB ,&
&  0.19247E+01_JPRB, 0.17957E+01_JPRB, 0.16923E+01_JPRB, 0.16577E+01_JPRB, 0.16226E+01_JPRB, 0.14773E+01_JPRB, 0.16175E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.10768E+01_JPRB, 0.10432E+01_JPRB, 0.67583E+00_JPRB, 0.83520E+00_JPRB, 0.10380E+01_JPRB, 0.13058E+01_JPRB ,&
&  0.14231E+01_JPRB, 0.18721E+01_JPRB, 0.22498E+01_JPRB, 0.16125E+01_JPRB, 0.12161E+01_JPRB, 0.80395E+00_JPRB, 0.73983E+00_JPRB ,&
&  0.75008E+00_JPRB, 0.74932E+00_JPRB, 0.76185E+00_JPRB, 0.78512E+00_JPRB, 0.80937E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.76606E+00_JPRB, 0.79365E+00_JPRB, 0.82820E+00_JPRB, 0.84266E+00_JPRB, 0.85908E+00_JPRB, 0.88460E+00_JPRB, 0.91226E+00_JPRB ,&
&  0.93758E+00_JPRB, 0.96566E+00_JPRB, 0.98858E+00_JPRB, 0.10218E+01_JPRB, 0.10651E+01_JPRB, 0.11659E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.14492E+01_JPRB, 0.15895E+01_JPRB, 0.17612E+01_JPRB, 0.18730E+01_JPRB, 0.20288E+01_JPRB, 0.21984E+01_JPRB, 0.23286E+01_JPRB ,&
&  0.24677E+01_JPRB, 0.25414E+01_JPRB, 0.26101E+01_JPRB, 0.27717E+01_JPRB, 0.27293E+01_JPRB, 0.28028E+01_JPRB, 0.21451E+01_JPRB ,&
&  0.22068E+01_JPRB, 0.21807E+01_JPRB, 0.20801E+01_JPRB, 0.20302E+01_JPRB, 0.19544E+01_JPRB, 0.18437E+01_JPRB, 0.17662E+01_JPRB ,&
&  0.17149E+01_JPRB, 0.15766E+01_JPRB, 0.14237E+01_JPRB, 0.14638E+01_JPRB, 0.15731E+01_JPRB, 0.18120E+01_JPRB, 0.21808E+01_JPRB ,&
&  0.25523E+01_JPRB, 0.28985E+01_JPRB, 0.32609E+01_JPRB, 0.37823E+01_JPRB, 0.44690E+01_JPRB, 0.51990E+01_JPRB, 0.59684E+01_JPRB ,&
&  0.65000E+01_JPRB, 0.69033E+01_JPRB, 0.70021E+01_JPRB, 0.59277E+01_JPRB, 0.55879E+01_JPRB, 0.42097E+01_JPRB, 0.27105E+01_JPRB ,&
&  0.19648E+01_JPRB, 0.17477E+01_JPRB, 0.17557E+01_JPRB, 0.17162E+01_JPRB, 0.14776E+01_JPRB, 0.13326E+01_JPRB, 0.11705E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10146E+01_JPRB, 0.10672E+01_JPRB, 0.12166E+01_JPRB, 0.13703E+01_JPRB, 0.15636E+01_JPRB, 0.16262E+01_JPRB ,&
&  0.15698E+01_JPRB, 0.14439E+01_JPRB, 0.13829E+01_JPRB, 0.13667E+01_JPRB, 0.14044E+01_JPRB, 0.14839E+01_JPRB, 0.12808E+01_JPRB ,&
&  0.11056E+01_JPRB, 0.87093E+00_JPRB, 0.82133E+00_JPRB, 0.87083E+00_JPRB, 0.88289E+00_JPRB, 0.11379E+01_JPRB, 0.15757E+01_JPRB ,&
&  0.22101E+01_JPRB, 0.22984E+01_JPRB, 0.27239E+01_JPRB, 0.35769E+01_JPRB, 0.26263E+01_JPRB, 0.18908E+01_JPRB, 0.66113E+00_JPRB ,&
&  0.68399E+00_JPRB, 0.68859E+00_JPRB, 0.69836E+00_JPRB, 0.70902E+00_JPRB, 0.73127E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.63204E+00_JPRB, 0.66732E+00_JPRB, 0.66259E+00_JPRB, 0.67005E+00_JPRB, 0.68016E+00_JPRB, 0.69726E+00_JPRB, 0.73141E+00_JPRB ,&
&  0.74667E+00_JPRB, 0.80424E+00_JPRB, 0.87075E+00_JPRB, 0.93086E+00_JPRB, 0.10047E+01_JPRB, 0.11072E+01_JPRB, 0.12435E+01_JPRB ,&
&  0.14015E+01_JPRB, 0.15037E+01_JPRB, 0.16137E+01_JPRB, 0.17080E+01_JPRB, 0.18512E+01_JPRB, 0.20274E+01_JPRB, 0.21607E+01_JPRB ,&
&  0.23479E+01_JPRB, 0.24896E+01_JPRB, 0.26452E+01_JPRB, 0.28595E+01_JPRB, 0.27503E+01_JPRB, 0.27050E+01_JPRB, 0.18989E+01_JPRB ,&
&  0.21870E+01_JPRB, 0.22380E+01_JPRB, 0.22079E+01_JPRB, 0.21609E+01_JPRB, 0.20514E+01_JPRB, 0.18745E+01_JPRB, 0.17610E+01_JPRB ,&
&  0.16543E+01_JPRB, 0.15356E+01_JPRB, 0.13933E+01_JPRB, 0.13166E+01_JPRB, 0.14501E+01_JPRB, 0.15530E+01_JPRB, 0.18807E+01_JPRB ,&
&  0.22620E+01_JPRB, 0.26891E+01_JPRB, 0.31018E+01_JPRB, 0.36044E+01_JPRB, 0.42637E+01_JPRB, 0.50209E+01_JPRB, 0.57921E+01_JPRB ,&
&  0.63729E+01_JPRB, 0.68574E+01_JPRB, 0.67896E+01_JPRB, 0.56319E+01_JPRB, 0.49626E+01_JPRB, 0.36846E+01_JPRB, 0.23511E+01_JPRB ,&
&  0.18503E+01_JPRB, 0.17027E+01_JPRB, 0.16960E+01_JPRB, 0.15673E+01_JPRB, 0.12986E+01_JPRB, 0.11192E+01_JPRB, 0.95953E+00_JPRB ,&
&  0.83084E+00_JPRB, 0.82429E+00_JPRB, 0.92200E+00_JPRB, 0.10626E+01_JPRB, 0.11789E+01_JPRB, 0.13008E+01_JPRB, 0.13000E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.11296E+01_JPRB, 0.10924E+01_JPRB, 0.11675E+01_JPRB, 0.13826E+01_JPRB, 0.16071E+01_JPRB, 0.17257E+01_JPRB ,&
&  0.11715E+01_JPRB, 0.87402E+00_JPRB, 0.82241E+00_JPRB, 0.81957E+00_JPRB, 0.84818E+00_JPRB, 0.92284E+00_JPRB, 0.10487E+01_JPRB ,&
&  0.12359E+01_JPRB, 0.16416E+01_JPRB, 0.15555E+01_JPRB, 0.12547E+01_JPRB, 0.17282E+01_JPRB, 0.14592E+01_JPRB, 0.60156E+00_JPRB ,&
&  0.58500E+00_JPRB, 0.57324E+00_JPRB, 0.58786E+00_JPRB, 0.59877E+00_JPRB, 0.62127E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.59014E+00_JPRB, 0.57990E+00_JPRB, 0.56210E+00_JPRB, 0.57582E+00_JPRB, 0.58046E+00_JPRB, 0.60672E+00_JPRB, 0.63786E+00_JPRB ,&
&  0.68252E+00_JPRB, 0.76583E+00_JPRB, 0.85820E+00_JPRB, 0.94211E+00_JPRB, 0.10089E+01_JPRB, 0.10842E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.12756E+01_JPRB, 0.13566E+01_JPRB, 0.14598E+01_JPRB, 0.15934E+01_JPRB, 0.17504E+01_JPRB, 0.19394E+01_JPRB ,&
&  0.21503E+01_JPRB, 0.23712E+01_JPRB, 0.26440E+01_JPRB, 0.27880E+01_JPRB, 0.28839E+01_JPRB, 0.29717E+01_JPRB, 0.29792E+01_JPRB ,&
&  0.19420E+01_JPRB, 0.23957E+01_JPRB, 0.23026E+01_JPRB, 0.21873E+01_JPRB, 0.20868E+01_JPRB, 0.19222E+01_JPRB, 0.17787E+01_JPRB ,&
&  0.16757E+01_JPRB, 0.15124E+01_JPRB, 0.14322E+01_JPRB, 0.12623E+01_JPRB, 0.12830E+01_JPRB, 0.13733E+01_JPRB, 0.15618E+01_JPRB ,&
&  0.18737E+01_JPRB, 0.23273E+01_JPRB, 0.28763E+01_JPRB, 0.34008E+01_JPRB, 0.39223E+01_JPRB, 0.45033E+01_JPRB, 0.53168E+01_JPRB ,&
&  0.62070E+01_JPRB, 0.68262E+01_JPRB, 0.72031E+01_JPRB, 0.62275E+01_JPRB, 0.49103E+01_JPRB, 0.41998E+01_JPRB, 0.30484E+01_JPRB ,&
&  0.19378E+01_JPRB, 0.15585E+01_JPRB, 0.15825E+01_JPRB, 0.15401E+01_JPRB, 0.12956E+01_JPRB, 0.11243E+01_JPRB, 0.95758E+00_JPRB ,&
&  0.81065E+00_JPRB, 0.73754E+00_JPRB, 0.76689E+00_JPRB, 0.91085E+00_JPRB, 0.98873E+00_JPRB, 0.10273E+01_JPRB, 0.10021E+01_JPRB ,&
&  0.97240E+00_JPRB, 0.90354E+00_JPRB, 0.83420E+00_JPRB, 0.84048E+00_JPRB, 0.89874E+00_JPRB, 0.94446E+00_JPRB, 0.93171E+00_JPRB ,&
&  0.89424E+00_JPRB, 0.87515E+00_JPRB, 0.86692E+00_JPRB, 0.87427E+00_JPRB, 0.87189E+00_JPRB, 0.88017E+00_JPRB, 0.89687E+00_JPRB ,&
&  0.91626E+00_JPRB, 0.94867E+00_JPRB, 0.91651E+00_JPRB, 0.87612E+00_JPRB, 0.91930E+00_JPRB, 0.96562E+00_JPRB, 0.65579E+00_JPRB ,&
&  0.57480E+00_JPRB, 0.54384E+00_JPRB, 0.55471E+00_JPRB, 0.55161E+00_JPRB, 0.60116E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.60737E+00_JPRB, 0.58554E+00_JPRB, 0.57427E+00_JPRB, 0.56668E+00_JPRB, 0.55076E+00_JPRB, 0.58423E+00_JPRB, 0.59876E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.69949E+00_JPRB, 0.77864E+00_JPRB, 0.83973E+00_JPRB, 0.87425E+00_JPRB, 0.92972E+00_JPRB, 0.97865E+00_JPRB ,&
&  0.10168E+01_JPRB, 0.10476E+01_JPRB, 0.10889E+01_JPRB, 0.11417E+01_JPRB, 0.12290E+01_JPRB, 0.13608E+01_JPRB, 0.15568E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.20379E+01_JPRB, 0.23119E+01_JPRB, 0.25742E+01_JPRB, 0.27748E+01_JPRB, 0.29312E+01_JPRB, 0.28818E+01_JPRB ,&
&  0.15541E+01_JPRB, 0.22317E+01_JPRB, 0.25773E+01_JPRB, 0.23513E+01_JPRB, 0.21598E+01_JPRB, 0.19693E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.16083E+01_JPRB, 0.14471E+01_JPRB, 0.13731E+01_JPRB, 0.13444E+01_JPRB, 0.11977E+01_JPRB, 0.12343E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.15161E+01_JPRB, 0.18463E+01_JPRB, 0.24500E+01_JPRB, 0.31042E+01_JPRB, 0.34702E+01_JPRB, 0.38356E+01_JPRB, 0.45082E+01_JPRB ,&
&  0.55427E+01_JPRB, 0.63512E+01_JPRB, 0.69389E+01_JPRB, 0.52247E+01_JPRB, 0.44164E+01_JPRB, 0.42639E+01_JPRB, 0.38210E+01_JPRB ,&
&  0.22723E+01_JPRB, 0.15847E+01_JPRB, 0.16023E+01_JPRB, 0.15961E+01_JPRB, 0.13660E+01_JPRB, 0.12269E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.88972E+00_JPRB, 0.77525E+00_JPRB, 0.69475E+00_JPRB, 0.73579E+00_JPRB, 0.77917E+00_JPRB, 0.80567E+00_JPRB, 0.76057E+00_JPRB ,&
&  0.80057E+00_JPRB, 0.77957E+00_JPRB, 0.76350E+00_JPRB, 0.76778E+00_JPRB, 0.84300E+00_JPRB, 0.87939E+00_JPRB, 0.89552E+00_JPRB ,&
&  0.91855E+00_JPRB, 0.93653E+00_JPRB, 0.93615E+00_JPRB, 0.94006E+00_JPRB, 0.94206E+00_JPRB, 0.93307E+00_JPRB, 0.92139E+00_JPRB ,&
&  0.89692E+00_JPRB, 0.87927E+00_JPRB, 0.83497E+00_JPRB, 0.78641E+00_JPRB, 0.72189E+00_JPRB, 0.71949E+00_JPRB, 0.65575E+00_JPRB ,&
&  0.61428E+00_JPRB, 0.58846E+00_JPRB, 0.57327E+00_JPRB, 0.57142E+00_JPRB, 0.58408E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.63553E+00_JPRB, 0.63073E+00_JPRB, 0.62428E+00_JPRB, 0.61167E+00_JPRB, 0.59457E+00_JPRB, 0.59428E+00_JPRB, 0.58137E+00_JPRB ,&
&  0.58488E+00_JPRB, 0.61861E+00_JPRB, 0.67132E+00_JPRB, 0.71362E+00_JPRB, 0.75411E+00_JPRB, 0.78732E+00_JPRB, 0.82701E+00_JPRB ,&
&  0.84629E+00_JPRB, 0.85208E+00_JPRB, 0.85129E+00_JPRB, 0.85462E+00_JPRB, 0.87745E+00_JPRB, 0.94960E+00_JPRB, 0.10724E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.14850E+01_JPRB, 0.17840E+01_JPRB, 0.21206E+01_JPRB, 0.24770E+01_JPRB, 0.28360E+01_JPRB, 0.30391E+01_JPRB ,&
&  0.19229E+01_JPRB, 0.91256E+00_JPRB, 0.24136E+01_JPRB, 0.26615E+01_JPRB, 0.23993E+01_JPRB, 0.22279E+01_JPRB, 0.19386E+01_JPRB ,&
&  0.16740E+01_JPRB, 0.15628E+01_JPRB, 0.14164E+01_JPRB, 0.13119E+01_JPRB, 0.11797E+01_JPRB, 0.11701E+01_JPRB, 0.11830E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.14776E+01_JPRB, 0.19009E+01_JPRB, 0.26153E+01_JPRB, 0.30977E+01_JPRB, 0.32892E+01_JPRB, 0.37208E+01_JPRB ,&
&  0.45960E+01_JPRB, 0.53639E+01_JPRB, 0.56866E+01_JPRB, 0.38257E+01_JPRB, 0.26388E+01_JPRB, 0.23395E+01_JPRB, 0.21634E+01_JPRB ,&
&  0.20431E+01_JPRB, 0.16712E+01_JPRB, 0.16383E+01_JPRB, 0.16635E+01_JPRB, 0.14956E+01_JPRB, 0.13018E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.10838E+01_JPRB, 0.94429E+00_JPRB, 0.82434E+00_JPRB, 0.76503E+00_JPRB, 0.72837E+00_JPRB, 0.73411E+00_JPRB, 0.72680E+00_JPRB ,&
&  0.72579E+00_JPRB, 0.74816E+00_JPRB, 0.75786E+00_JPRB, 0.77716E+00_JPRB, 0.83121E+00_JPRB, 0.88442E+00_JPRB, 0.91781E+00_JPRB ,&
&  0.94627E+00_JPRB, 0.97513E+00_JPRB, 0.99037E+00_JPRB, 0.99310E+00_JPRB, 0.99012E+00_JPRB, 0.96726E+00_JPRB, 0.96878E+00_JPRB ,&
&  0.96867E+00_JPRB, 0.96212E+00_JPRB, 0.91731E+00_JPRB, 0.88029E+00_JPRB, 0.80372E+00_JPRB, 0.74724E+00_JPRB, 0.67439E+00_JPRB ,&
&  0.64482E+00_JPRB, 0.60341E+00_JPRB, 0.57701E+00_JPRB, 0.57857E+00_JPRB, 0.59743E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.65515E+00_JPRB, 0.65567E+00_JPRB, 0.65953E+00_JPRB, 0.66324E+00_JPRB, 0.65984E+00_JPRB, 0.66026E+00_JPRB, 0.63550E+00_JPRB ,&
&  0.60107E+00_JPRB, 0.58971E+00_JPRB, 0.61784E+00_JPRB, 0.62306E+00_JPRB, 0.62577E+00_JPRB, 0.65628E+00_JPRB, 0.69063E+00_JPRB ,&
&  0.71271E+00_JPRB, 0.72451E+00_JPRB, 0.72518E+00_JPRB, 0.72483E+00_JPRB, 0.71189E+00_JPRB, 0.71631E+00_JPRB, 0.75845E+00_JPRB ,&
&  0.83664E+00_JPRB, 0.97904E+00_JPRB, 0.11949E+01_JPRB, 0.14702E+01_JPRB, 0.18032E+01_JPRB, 0.22855E+01_JPRB, 0.27625E+01_JPRB ,&
&  0.36395E+01_JPRB, 0.24188E+01_JPRB, 0.13366E+01_JPRB, 0.24332E+01_JPRB, 0.27262E+01_JPRB, 0.26801E+01_JPRB, 0.21944E+01_JPRB ,&
&  0.20309E+01_JPRB, 0.19049E+01_JPRB, 0.17530E+01_JPRB, 0.15568E+01_JPRB, 0.12977E+01_JPRB, 0.11990E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.11811E+01_JPRB, 0.14149E+01_JPRB, 0.19060E+01_JPRB, 0.25396E+01_JPRB, 0.28698E+01_JPRB, 0.30607E+01_JPRB ,&
&  0.36550E+01_JPRB, 0.44335E+01_JPRB, 0.46255E+01_JPRB, 0.30676E+01_JPRB, 0.22358E+01_JPRB, 0.20238E+01_JPRB, 0.19743E+01_JPRB ,&
&  0.18815E+01_JPRB, 0.18784E+01_JPRB, 0.19052E+01_JPRB, 0.17509E+01_JPRB, 0.15597E+01_JPRB, 0.14182E+01_JPRB, 0.14125E+01_JPRB ,&
&  0.12561E+01_JPRB, 0.11207E+01_JPRB, 0.10011E+01_JPRB, 0.87871E+00_JPRB, 0.77668E+00_JPRB, 0.74976E+00_JPRB, 0.73795E+00_JPRB ,&
&  0.74631E+00_JPRB, 0.77382E+00_JPRB, 0.78027E+00_JPRB, 0.80887E+00_JPRB, 0.83924E+00_JPRB, 0.86989E+00_JPRB, 0.89935E+00_JPRB ,&
&  0.92696E+00_JPRB, 0.95771E+00_JPRB, 0.98612E+00_JPRB, 0.99168E+00_JPRB, 0.98140E+00_JPRB, 0.96566E+00_JPRB, 0.98072E+00_JPRB ,&
&  0.98870E+00_JPRB, 0.10012E+01_JPRB, 0.98278E+00_JPRB, 0.97006E+00_JPRB, 0.90695E+00_JPRB, 0.84538E+00_JPRB, 0.77907E+00_JPRB ,&
&  0.75610E+00_JPRB, 0.70590E+00_JPRB, 0.67084E+00_JPRB, 0.63309E+00_JPRB, 0.64173E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.68203E+00_JPRB, 0.67447E+00_JPRB, 0.67679E+00_JPRB, 0.68197E+00_JPRB, 0.68381E+00_JPRB, 0.68753E+00_JPRB, 0.68726E+00_JPRB ,&
&  0.65048E+00_JPRB, 0.61608E+00_JPRB, 0.59787E+00_JPRB, 0.58442E+00_JPRB, 0.57773E+00_JPRB, 0.59457E+00_JPRB, 0.60548E+00_JPRB ,&
&  0.60687E+00_JPRB, 0.60953E+00_JPRB, 0.63956E+00_JPRB, 0.64983E+00_JPRB, 0.64753E+00_JPRB, 0.63829E+00_JPRB, 0.63760E+00_JPRB ,&
&  0.65877E+00_JPRB, 0.71316E+00_JPRB, 0.82371E+00_JPRB, 0.97671E+00_JPRB, 0.11519E+01_JPRB, 0.14667E+01_JPRB, 0.18803E+01_JPRB ,&
&  0.25866E+01_JPRB, 0.26754E+01_JPRB, 0.11642E+01_JPRB, 0.20909E+01_JPRB, 0.28720E+01_JPRB, 0.29438E+01_JPRB, 0.26991E+01_JPRB ,&
&  0.24772E+01_JPRB, 0.22791E+01_JPRB, 0.23079E+01_JPRB, 0.20688E+01_JPRB, 0.15974E+01_JPRB, 0.13157E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10061E+01_JPRB, 0.10874E+01_JPRB, 0.13281E+01_JPRB, 0.18232E+01_JPRB, 0.22848E+01_JPRB, 0.25141E+01_JPRB ,&
&  0.27958E+01_JPRB, 0.35212E+01_JPRB, 0.40293E+01_JPRB, 0.27847E+01_JPRB, 0.22301E+01_JPRB, 0.22721E+01_JPRB, 0.22447E+01_JPRB ,&
&  0.21306E+01_JPRB, 0.20725E+01_JPRB, 0.20151E+01_JPRB, 0.18553E+01_JPRB, 0.16473E+01_JPRB, 0.15536E+01_JPRB, 0.15509E+01_JPRB ,&
&  0.14070E+01_JPRB, 0.12767E+01_JPRB, 0.11555E+01_JPRB, 0.10351E+01_JPRB, 0.90464E+00_JPRB, 0.80728E+00_JPRB, 0.79196E+00_JPRB ,&
&  0.79238E+00_JPRB, 0.81869E+00_JPRB, 0.82243E+00_JPRB, 0.84053E+00_JPRB, 0.86001E+00_JPRB, 0.87912E+00_JPRB, 0.89359E+00_JPRB ,&
&  0.91039E+00_JPRB, 0.93355E+00_JPRB, 0.95855E+00_JPRB, 0.96233E+00_JPRB, 0.96076E+00_JPRB, 0.95630E+00_JPRB, 0.96035E+00_JPRB ,&
&  0.97203E+00_JPRB, 0.98582E+00_JPRB, 0.98163E+00_JPRB, 0.97863E+00_JPRB, 0.98325E+00_JPRB, 0.95051E+00_JPRB, 0.90762E+00_JPRB ,&
&  0.93007E+00_JPRB, 0.95331E+00_JPRB, 0.88113E+00_JPRB, 0.73333E+00_JPRB, 0.69996E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.73493E+00_JPRB, 0.71965E+00_JPRB, 0.70961E+00_JPRB, 0.70845E+00_JPRB, 0.70633E+00_JPRB, 0.70728E+00_JPRB, 0.71696E+00_JPRB ,&
&  0.70312E+00_JPRB, 0.67735E+00_JPRB, 0.64780E+00_JPRB, 0.61545E+00_JPRB, 0.59137E+00_JPRB, 0.59294E+00_JPRB, 0.60080E+00_JPRB ,&
&  0.60332E+00_JPRB, 0.59824E+00_JPRB, 0.59879E+00_JPRB, 0.60904E+00_JPRB, 0.60847E+00_JPRB, 0.60178E+00_JPRB, 0.59459E+00_JPRB ,&
&  0.59380E+00_JPRB, 0.60645E+00_JPRB, 0.64761E+00_JPRB, 0.72110E+00_JPRB, 0.81466E+00_JPRB, 0.97362E+00_JPRB, 0.12123E+01_JPRB ,&
&  0.17593E+01_JPRB, 0.21273E+01_JPRB, 0.22702E+01_JPRB, 0.34188E+01_JPRB, 0.35896E+01_JPRB, 0.33787E+01_JPRB, 0.33184E+01_JPRB ,&
&  0.31830E+01_JPRB, 0.29496E+01_JPRB, 0.31051E+01_JPRB, 0.24422E+01_JPRB, 0.19147E+01_JPRB, 0.14846E+01_JPRB, 0.12080E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.96241E+00_JPRB, 0.94475E+00_JPRB, 0.10038E+01_JPRB, 0.12476E+01_JPRB, 0.16785E+01_JPRB, 0.20223E+01_JPRB ,&
&  0.22719E+01_JPRB, 0.27542E+01_JPRB, 0.34730E+01_JPRB, 0.29280E+01_JPRB, 0.23720E+01_JPRB, 0.27198E+01_JPRB, 0.30649E+01_JPRB ,&
&  0.34520E+01_JPRB, 0.25579E+01_JPRB, 0.22117E+01_JPRB, 0.19996E+01_JPRB, 0.17384E+01_JPRB, 0.18676E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.15218E+01_JPRB, 0.14117E+01_JPRB, 0.12961E+01_JPRB, 0.11831E+01_JPRB, 0.10672E+01_JPRB, 0.95647E+00_JPRB, 0.89630E+00_JPRB ,&
&  0.88438E+00_JPRB, 0.89030E+00_JPRB, 0.88621E+00_JPRB, 0.89584E+00_JPRB, 0.89867E+00_JPRB, 0.91071E+00_JPRB, 0.92077E+00_JPRB ,&
&  0.93102E+00_JPRB, 0.93667E+00_JPRB, 0.92211E+00_JPRB, 0.92834E+00_JPRB, 0.93238E+00_JPRB, 0.92857E+00_JPRB, 0.92024E+00_JPRB ,&
&  0.92735E+00_JPRB, 0.94449E+00_JPRB, 0.95440E+00_JPRB, 0.96069E+00_JPRB, 0.97141E+00_JPRB, 0.95486E+00_JPRB, 0.91770E+00_JPRB ,&
&  0.89633E+00_JPRB, 0.89095E+00_JPRB, 0.86790E+00_JPRB, 0.79755E+00_JPRB, 0.76163E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.76724E+00_JPRB, 0.75240E+00_JPRB, 0.74221E+00_JPRB, 0.73112E+00_JPRB, 0.71783E+00_JPRB, 0.71547E+00_JPRB, 0.72441E+00_JPRB ,&
&  0.72266E+00_JPRB, 0.70683E+00_JPRB, 0.68418E+00_JPRB, 0.66096E+00_JPRB, 0.63259E+00_JPRB, 0.61709E+00_JPRB, 0.60730E+00_JPRB ,&
&  0.60070E+00_JPRB, 0.59761E+00_JPRB, 0.59111E+00_JPRB, 0.59010E+00_JPRB, 0.58911E+00_JPRB, 0.57633E+00_JPRB, 0.56612E+00_JPRB ,&
&  0.56124E+00_JPRB, 0.56660E+00_JPRB, 0.58283E+00_JPRB, 0.62023E+00_JPRB, 0.67839E+00_JPRB, 0.77973E+00_JPRB, 0.91322E+00_JPRB ,&
&  0.12550E+01_JPRB, 0.17874E+01_JPRB, 0.16257E+01_JPRB, 0.35722E+01_JPRB, 0.35431E+01_JPRB, 0.34796E+01_JPRB, 0.32864E+01_JPRB ,&
&  0.32653E+01_JPRB, 0.36216E+01_JPRB, 0.31454E+01_JPRB, 0.25856E+01_JPRB, 0.20140E+01_JPRB, 0.16029E+01_JPRB, 0.12980E+01_JPRB ,&
&  0.11069E+01_JPRB, 0.99511E+00_JPRB, 0.94158E+00_JPRB, 0.91750E+00_JPRB, 0.94267E+00_JPRB, 0.11805E+01_JPRB, 0.15629E+01_JPRB ,&
&  0.18521E+01_JPRB, 0.22006E+01_JPRB, 0.28613E+01_JPRB, 0.27404E+01_JPRB, 0.25609E+01_JPRB, 0.30412E+01_JPRB, 0.39652E+01_JPRB ,&
&  0.47663E+01_JPRB, 0.36165E+01_JPRB, 0.26531E+01_JPRB, 0.22543E+01_JPRB, 0.20341E+01_JPRB, 0.19798E+01_JPRB, 0.17986E+01_JPRB ,&
&  0.16033E+01_JPRB, 0.14959E+01_JPRB, 0.14053E+01_JPRB, 0.13150E+01_JPRB, 0.12148E+01_JPRB, 0.11190E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.98336E+00_JPRB, 0.96706E+00_JPRB, 0.95975E+00_JPRB, 0.96779E+00_JPRB, 0.97293E+00_JPRB, 0.98575E+00_JPRB, 0.98718E+00_JPRB ,&
&  0.99036E+00_JPRB, 0.97536E+00_JPRB, 0.92341E+00_JPRB, 0.90857E+00_JPRB, 0.90847E+00_JPRB, 0.90141E+00_JPRB, 0.88862E+00_JPRB ,&
&  0.89974E+00_JPRB, 0.91362E+00_JPRB, 0.91503E+00_JPRB, 0.91584E+00_JPRB, 0.93903E+00_JPRB, 0.94623E+00_JPRB, 0.92387E+00_JPRB ,&
&  0.88966E+00_JPRB, 0.85959E+00_JPRB, 0.83252E+00_JPRB, 0.80093E+00_JPRB, 0.78609E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.77891E+00_JPRB, 0.76752E+00_JPRB, 0.75663E+00_JPRB, 0.74226E+00_JPRB, 0.72250E+00_JPRB, 0.71350E+00_JPRB, 0.71827E+00_JPRB ,&
&  0.72444E+00_JPRB, 0.71492E+00_JPRB, 0.69825E+00_JPRB, 0.67618E+00_JPRB, 0.64880E+00_JPRB, 0.63289E+00_JPRB, 0.62152E+00_JPRB ,&
&  0.60539E+00_JPRB, 0.59185E+00_JPRB, 0.57500E+00_JPRB, 0.56874E+00_JPRB, 0.56895E+00_JPRB, 0.55984E+00_JPRB, 0.54148E+00_JPRB ,&
&  0.54199E+00_JPRB, 0.55212E+00_JPRB, 0.56506E+00_JPRB, 0.58372E+00_JPRB, 0.62457E+00_JPRB, 0.69990E+00_JPRB, 0.81798E+00_JPRB ,&
&  0.10653E+01_JPRB, 0.21103E+01_JPRB, 0.14965E+01_JPRB, 0.27582E+01_JPRB, 0.28510E+01_JPRB, 0.30508E+01_JPRB, 0.31093E+01_JPRB ,&
&  0.31957E+01_JPRB, 0.32498E+01_JPRB, 0.29867E+01_JPRB, 0.24384E+01_JPRB, 0.19012E+01_JPRB, 0.15524E+01_JPRB, 0.13305E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.10695E+01_JPRB, 0.98916E+00_JPRB, 0.93557E+00_JPRB, 0.90134E+00_JPRB, 0.92540E+00_JPRB, 0.11202E+01_JPRB ,&
&  0.14535E+01_JPRB, 0.17576E+01_JPRB, 0.22536E+01_JPRB, 0.27817E+01_JPRB, 0.25685E+01_JPRB, 0.29039E+01_JPRB, 0.42773E+01_JPRB ,&
&  0.51463E+01_JPRB, 0.51484E+01_JPRB, 0.34488E+01_JPRB, 0.26903E+01_JPRB, 0.23695E+01_JPRB, 0.21830E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.16493E+01_JPRB, 0.15196E+01_JPRB, 0.14502E+01_JPRB, 0.13916E+01_JPRB, 0.13176E+01_JPRB, 0.12143E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.10278E+01_JPRB, 0.10017E+01_JPRB, 0.99786E+00_JPRB, 0.10049E+01_JPRB, 0.10226E+01_JPRB, 0.10426E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.10506E+01_JPRB, 0.10340E+01_JPRB, 0.95982E+00_JPRB, 0.92138E+00_JPRB, 0.90908E+00_JPRB, 0.89677E+00_JPRB, 0.88144E+00_JPRB ,&
&  0.88727E+00_JPRB, 0.89224E+00_JPRB, 0.88972E+00_JPRB, 0.89766E+00_JPRB, 0.91675E+00_JPRB, 0.94152E+00_JPRB, 0.89439E+00_JPRB ,&
&  0.88847E+00_JPRB, 0.86594E+00_JPRB, 0.83889E+00_JPRB, 0.80931E+00_JPRB, 0.79448E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.77536E+00_JPRB, 0.76542E+00_JPRB, 0.75434E+00_JPRB, 0.74137E+00_JPRB, 0.72654E+00_JPRB, 0.71607E+00_JPRB, 0.71327E+00_JPRB ,&
&  0.71251E+00_JPRB, 0.71193E+00_JPRB, 0.70173E+00_JPRB, 0.67838E+00_JPRB, 0.65395E+00_JPRB, 0.63206E+00_JPRB, 0.61891E+00_JPRB ,&
&  0.60754E+00_JPRB, 0.59325E+00_JPRB, 0.57640E+00_JPRB, 0.56070E+00_JPRB, 0.55590E+00_JPRB, 0.55505E+00_JPRB, 0.54438E+00_JPRB ,&
&  0.54969E+00_JPRB, 0.55884E+00_JPRB, 0.56670E+00_JPRB, 0.57546E+00_JPRB, 0.60667E+00_JPRB, 0.66989E+00_JPRB, 0.77804E+00_JPRB ,&
&  0.99056E+00_JPRB, 0.86053E+00_JPRB, 0.16841E+01_JPRB, 0.20678E+01_JPRB, 0.22716E+01_JPRB, 0.24477E+01_JPRB, 0.28213E+01_JPRB ,&
&  0.31011E+01_JPRB, 0.29971E+01_JPRB, 0.26729E+01_JPRB, 0.22303E+01_JPRB, 0.18269E+01_JPRB, 0.15313E+01_JPRB, 0.13339E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.11128E+01_JPRB, 0.10273E+01_JPRB, 0.95371E+00_JPRB, 0.90499E+00_JPRB, 0.87961E+00_JPRB, 0.90269E+00_JPRB ,&
&  0.10345E+01_JPRB, 0.12806E+01_JPRB, 0.16210E+01_JPRB, 0.22214E+01_JPRB, 0.25393E+01_JPRB, 0.27589E+01_JPRB, 0.32810E+01_JPRB ,&
&  0.43903E+01_JPRB, 0.50797E+01_JPRB, 0.40082E+01_JPRB, 0.31456E+01_JPRB, 0.26748E+01_JPRB, 0.22874E+01_JPRB, 0.19535E+01_JPRB ,&
&  0.16907E+01_JPRB, 0.15288E+01_JPRB, 0.14330E+01_JPRB, 0.13767E+01_JPRB, 0.13404E+01_JPRB, 0.12872E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.10944E+01_JPRB, 0.10749E+01_JPRB, 0.10714E+01_JPRB, 0.10748E+01_JPRB, 0.10904E+01_JPRB, 0.10924E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10436E+01_JPRB, 0.99062E+00_JPRB, 0.94525E+00_JPRB, 0.92700E+00_JPRB, 0.90709E+00_JPRB, 0.88353E+00_JPRB ,&
&  0.87357E+00_JPRB, 0.87371E+00_JPRB, 0.87663E+00_JPRB, 0.89168E+00_JPRB, 0.89562E+00_JPRB, 0.91960E+00_JPRB, 0.87501E+00_JPRB ,&
&  0.86279E+00_JPRB, 0.84096E+00_JPRB, 0.81879E+00_JPRB, 0.79778E+00_JPRB, 0.78928E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.77187E+00_JPRB, 0.76120E+00_JPRB, 0.74784E+00_JPRB, 0.73335E+00_JPRB, 0.71779E+00_JPRB, 0.70610E+00_JPRB, 0.70062E+00_JPRB ,&
&  0.69722E+00_JPRB, 0.69572E+00_JPRB, 0.69466E+00_JPRB, 0.67829E+00_JPRB, 0.65754E+00_JPRB, 0.63806E+00_JPRB, 0.62592E+00_JPRB ,&
&  0.61743E+00_JPRB, 0.60754E+00_JPRB, 0.59483E+00_JPRB, 0.57899E+00_JPRB, 0.57054E+00_JPRB, 0.56433E+00_JPRB, 0.55932E+00_JPRB ,&
&  0.56429E+00_JPRB, 0.56813E+00_JPRB, 0.57332E+00_JPRB, 0.58210E+00_JPRB, 0.60717E+00_JPRB, 0.66374E+00_JPRB, 0.76216E+00_JPRB ,&
&  0.98605E+00_JPRB, 0.15009E+01_JPRB, 0.19448E+01_JPRB, 0.18338E+01_JPRB, 0.18471E+01_JPRB, 0.19057E+01_JPRB, 0.22439E+01_JPRB ,&
&  0.24679E+01_JPRB, 0.25330E+01_JPRB, 0.23102E+01_JPRB, 0.19996E+01_JPRB, 0.17206E+01_JPRB, 0.15041E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.10683E+01_JPRB, 0.97265E+00_JPRB, 0.90545E+00_JPRB, 0.86234E+00_JPRB, 0.83524E+00_JPRB, 0.82539E+00_JPRB ,&
&  0.84551E+00_JPRB, 0.93028E+00_JPRB, 0.11279E+01_JPRB, 0.15338E+01_JPRB, 0.24154E+01_JPRB, 0.24341E+01_JPRB, 0.30371E+01_JPRB ,&
&  0.41514E+01_JPRB, 0.40596E+01_JPRB, 0.36389E+01_JPRB, 0.31035E+01_JPRB, 0.26549E+01_JPRB, 0.22894E+01_JPRB, 0.19922E+01_JPRB ,&
&  0.17711E+01_JPRB, 0.16105E+01_JPRB, 0.14758E+01_JPRB, 0.13763E+01_JPRB, 0.13119E+01_JPRB, 0.12840E+01_JPRB, 0.12587E+01_JPRB ,&
&  0.12257E+01_JPRB, 0.11862E+01_JPRB, 0.11573E+01_JPRB, 0.11423E+01_JPRB, 0.11274E+01_JPRB, 0.11156E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.10057E+01_JPRB, 0.97304E+00_JPRB, 0.93443E+00_JPRB, 0.92170E+00_JPRB, 0.90522E+00_JPRB, 0.87683E+00_JPRB ,&
&  0.85144E+00_JPRB, 0.84765E+00_JPRB, 0.86302E+00_JPRB, 0.87314E+00_JPRB, 0.89305E+00_JPRB, 0.91511E+00_JPRB, 0.86958E+00_JPRB ,&
&  0.84794E+00_JPRB, 0.81690E+00_JPRB, 0.79117E+00_JPRB, 0.78021E+00_JPRB, 0.78366E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.75532E+00_JPRB, 0.74112E+00_JPRB, 0.72749E+00_JPRB, 0.71320E+00_JPRB, 0.69720E+00_JPRB, 0.68606E+00_JPRB, 0.67692E+00_JPRB ,&
&  0.67058E+00_JPRB, 0.66718E+00_JPRB, 0.66805E+00_JPRB, 0.66854E+00_JPRB, 0.66433E+00_JPRB, 0.65320E+00_JPRB, 0.63513E+00_JPRB ,&
&  0.62330E+00_JPRB, 0.61218E+00_JPRB, 0.59821E+00_JPRB, 0.58114E+00_JPRB, 0.56467E+00_JPRB, 0.55614E+00_JPRB, 0.55385E+00_JPRB ,&
&  0.55766E+00_JPRB, 0.56196E+00_JPRB, 0.56515E+00_JPRB, 0.57698E+00_JPRB, 0.60056E+00_JPRB, 0.65432E+00_JPRB, 0.73486E+00_JPRB ,&
&  0.89552E+00_JPRB, 0.14026E+01_JPRB, 0.15446E+01_JPRB, 0.15632E+01_JPRB, 0.16132E+01_JPRB, 0.16257E+01_JPRB, 0.17844E+01_JPRB ,&
&  0.19359E+01_JPRB, 0.20296E+01_JPRB, 0.19685E+01_JPRB, 0.17485E+01_JPRB, 0.15529E+01_JPRB, 0.14181E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.10452E+01_JPRB, 0.95910E+00_JPRB, 0.90067E+00_JPRB, 0.85563E+00_JPRB, 0.82142E+00_JPRB, 0.80113E+00_JPRB ,&
&  0.79525E+00_JPRB, 0.80647E+00_JPRB, 0.86277E+00_JPRB, 0.10020E+01_JPRB, 0.15631E+01_JPRB, 0.20718E+01_JPRB, 0.25176E+01_JPRB ,&
&  0.27016E+01_JPRB, 0.27817E+01_JPRB, 0.28054E+01_JPRB, 0.26118E+01_JPRB, 0.23658E+01_JPRB, 0.20880E+01_JPRB, 0.18884E+01_JPRB ,&
&  0.17517E+01_JPRB, 0.16022E+01_JPRB, 0.14680E+01_JPRB, 0.13467E+01_JPRB, 0.12596E+01_JPRB, 0.12095E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11245E+01_JPRB, 0.11030E+01_JPRB, 0.10844E+01_JPRB, 0.10634E+01_JPRB, 0.10373E+01_JPRB, 0.10042E+01_JPRB ,&
&  0.96573E+00_JPRB, 0.92770E+00_JPRB, 0.90202E+00_JPRB, 0.88416E+00_JPRB, 0.87072E+00_JPRB, 0.85228E+00_JPRB, 0.83064E+00_JPRB ,&
&  0.82370E+00_JPRB, 0.83134E+00_JPRB, 0.83965E+00_JPRB, 0.84990E+00_JPRB, 0.85935E+00_JPRB, 0.87781E+00_JPRB, 0.84498E+00_JPRB ,&
&  0.81440E+00_JPRB, 0.78460E+00_JPRB, 0.76317E+00_JPRB, 0.75191E+00_JPRB, 0.76441E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.73883E+00_JPRB, 0.71968E+00_JPRB, 0.70003E+00_JPRB, 0.67836E+00_JPRB, 0.66089E+00_JPRB, 0.65078E+00_JPRB, 0.64385E+00_JPRB ,&
&  0.64029E+00_JPRB, 0.63925E+00_JPRB, 0.63818E+00_JPRB, 0.63617E+00_JPRB, 0.63281E+00_JPRB, 0.62817E+00_JPRB, 0.61892E+00_JPRB ,&
&  0.60735E+00_JPRB, 0.59300E+00_JPRB, 0.58078E+00_JPRB, 0.57049E+00_JPRB, 0.55269E+00_JPRB, 0.54080E+00_JPRB, 0.54073E+00_JPRB ,&
&  0.54504E+00_JPRB, 0.54965E+00_JPRB, 0.55387E+00_JPRB, 0.56428E+00_JPRB, 0.59120E+00_JPRB, 0.64583E+00_JPRB, 0.70967E+00_JPRB ,&
&  0.84442E+00_JPRB, 0.10234E+01_JPRB, 0.11869E+01_JPRB, 0.12249E+01_JPRB, 0.12537E+01_JPRB, 0.13205E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.14702E+01_JPRB, 0.15940E+01_JPRB, 0.15702E+01_JPRB, 0.14280E+01_JPRB, 0.12749E+01_JPRB, 0.11626E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.98425E+00_JPRB, 0.94067E+00_JPRB, 0.90832E+00_JPRB, 0.87743E+00_JPRB, 0.84597E+00_JPRB, 0.81545E+00_JPRB ,&
&  0.79432E+00_JPRB, 0.78085E+00_JPRB, 0.78032E+00_JPRB, 0.81056E+00_JPRB, 0.94060E+00_JPRB, 0.11961E+01_JPRB, 0.15459E+01_JPRB ,&
&  0.18296E+01_JPRB, 0.19945E+01_JPRB, 0.21248E+01_JPRB, 0.21206E+01_JPRB, 0.20291E+01_JPRB, 0.19050E+01_JPRB, 0.17774E+01_JPRB ,&
&  0.16188E+01_JPRB, 0.14343E+01_JPRB, 0.12943E+01_JPRB, 0.11927E+01_JPRB, 0.11200E+01_JPRB, 0.10714E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10175E+01_JPRB, 0.10106E+01_JPRB, 0.10308E+01_JPRB, 0.10506E+01_JPRB, 0.10475E+01_JPRB, 0.10099E+01_JPRB, 0.95622E+00_JPRB ,&
&  0.90763E+00_JPRB, 0.86996E+00_JPRB, 0.83941E+00_JPRB, 0.81555E+00_JPRB, 0.79652E+00_JPRB, 0.78180E+00_JPRB, 0.76622E+00_JPRB ,&
&  0.75638E+00_JPRB, 0.76767E+00_JPRB, 0.76912E+00_JPRB, 0.79365E+00_JPRB, 0.83169E+00_JPRB, 0.81105E+00_JPRB, 0.77946E+00_JPRB ,&
&  0.75467E+00_JPRB, 0.73877E+00_JPRB, 0.72419E+00_JPRB, 0.71239E+00_JPRB, 0.75697E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.65033E+00_JPRB, 0.64500E+00_JPRB, 0.63309E+00_JPRB, 0.62158E+00_JPRB, 0.61307E+00_JPRB, 0.60947E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60445E+00_JPRB, 0.60367E+00_JPRB, 0.60326E+00_JPRB, 0.60045E+00_JPRB, 0.59480E+00_JPRB, 0.58645E+00_JPRB, 0.57610E+00_JPRB ,&
&  0.56646E+00_JPRB, 0.55678E+00_JPRB, 0.54802E+00_JPRB, 0.53904E+00_JPRB, 0.52768E+00_JPRB, 0.52071E+00_JPRB, 0.52262E+00_JPRB ,&
&  0.52882E+00_JPRB, 0.53587E+00_JPRB, 0.54243E+00_JPRB, 0.55184E+00_JPRB, 0.58457E+00_JPRB, 0.63423E+00_JPRB, 0.68021E+00_JPRB ,&
&  0.79674E+00_JPRB, 0.84599E+00_JPRB, 0.92201E+00_JPRB, 0.99963E+00_JPRB, 0.10841E+01_JPRB, 0.11134E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.12143E+01_JPRB, 0.12146E+01_JPRB, 0.11600E+01_JPRB, 0.10505E+01_JPRB, 0.94267E+00_JPRB, 0.87536E+00_JPRB ,&
&  0.84170E+00_JPRB, 0.82545E+00_JPRB, 0.81051E+00_JPRB, 0.79613E+00_JPRB, 0.78376E+00_JPRB, 0.77142E+00_JPRB, 0.75883E+00_JPRB ,&
&  0.74687E+00_JPRB, 0.73741E+00_JPRB, 0.73174E+00_JPRB, 0.73113E+00_JPRB, 0.75140E+00_JPRB, 0.82769E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.14754E+01_JPRB, 0.16025E+01_JPRB, 0.16776E+01_JPRB, 0.17130E+01_JPRB, 0.16399E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.11684E+01_JPRB, 0.10619E+01_JPRB, 0.98226E+00_JPRB, 0.92501E+00_JPRB, 0.88679E+00_JPRB, 0.86513E+00_JPRB ,&
&  0.85681E+00_JPRB, 0.85692E+00_JPRB, 0.86564E+00_JPRB, 0.87415E+00_JPRB, 0.87738E+00_JPRB, 0.86847E+00_JPRB, 0.84861E+00_JPRB ,&
&  0.82046E+00_JPRB, 0.79953E+00_JPRB, 0.77870E+00_JPRB, 0.75526E+00_JPRB, 0.73412E+00_JPRB, 0.71434E+00_JPRB, 0.69856E+00_JPRB ,&
&  0.68840E+00_JPRB, 0.69109E+00_JPRB, 0.70287E+00_JPRB, 0.73175E+00_JPRB, 0.72035E+00_JPRB, 0.70251E+00_JPRB, 0.68556E+00_JPRB ,&
&  0.67719E+00_JPRB, 0.67362E+00_JPRB, 0.66264E+00_JPRB, 0.65248E+00_JPRB, 0.64897E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.60182E+00_JPRB, 0.59679E+00_JPRB, 0.58702E+00_JPRB, 0.57862E+00_JPRB, 0.57559E+00_JPRB, 0.57578E+00_JPRB, 0.57712E+00_JPRB ,&
&  0.57618E+00_JPRB, 0.57271E+00_JPRB, 0.56771E+00_JPRB, 0.56393E+00_JPRB, 0.55864E+00_JPRB, 0.55143E+00_JPRB, 0.54250E+00_JPRB ,&
&  0.53375E+00_JPRB, 0.52579E+00_JPRB, 0.51946E+00_JPRB, 0.51287E+00_JPRB, 0.50788E+00_JPRB, 0.50720E+00_JPRB, 0.50883E+00_JPRB ,&
&  0.51555E+00_JPRB, 0.52434E+00_JPRB, 0.53193E+00_JPRB, 0.54971E+00_JPRB, 0.57388E+00_JPRB, 0.61269E+00_JPRB, 0.65671E+00_JPRB ,&
&  0.75280E+00_JPRB, 0.70598E+00_JPRB, 0.76161E+00_JPRB, 0.87514E+00_JPRB, 0.95154E+00_JPRB, 0.97666E+00_JPRB, 0.94656E+00_JPRB ,&
&  0.93006E+00_JPRB, 0.93562E+00_JPRB, 0.93888E+00_JPRB, 0.93438E+00_JPRB, 0.90075E+00_JPRB, 0.82601E+00_JPRB, 0.74967E+00_JPRB ,&
&  0.69685E+00_JPRB, 0.67714E+00_JPRB, 0.66782E+00_JPRB, 0.65500E+00_JPRB, 0.64196E+00_JPRB, 0.63432E+00_JPRB, 0.63236E+00_JPRB ,&
&  0.63081E+00_JPRB, 0.62867E+00_JPRB, 0.62515E+00_JPRB, 0.62108E+00_JPRB, 0.62064E+00_JPRB, 0.63798E+00_JPRB, 0.70262E+00_JPRB ,&
&  0.83384E+00_JPRB, 0.98983E+00_JPRB, 0.11370E+01_JPRB, 0.12501E+01_JPRB, 0.12751E+01_JPRB, 0.12417E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.10103E+01_JPRB, 0.94149E+00_JPRB, 0.88494E+00_JPRB, 0.83899E+00_JPRB, 0.80259E+00_JPRB, 0.77564E+00_JPRB ,&
&  0.76097E+00_JPRB, 0.75127E+00_JPRB, 0.74412E+00_JPRB, 0.73472E+00_JPRB, 0.72514E+00_JPRB, 0.71871E+00_JPRB, 0.71555E+00_JPRB ,&
&  0.70910E+00_JPRB, 0.70135E+00_JPRB, 0.69452E+00_JPRB, 0.68667E+00_JPRB, 0.67791E+00_JPRB, 0.66903E+00_JPRB, 0.66079E+00_JPRB ,&
&  0.65510E+00_JPRB, 0.65172E+00_JPRB, 0.64991E+00_JPRB, 0.65316E+00_JPRB, 0.66150E+00_JPRB, 0.64610E+00_JPRB, 0.63370E+00_JPRB ,&
&  0.62777E+00_JPRB, 0.62008E+00_JPRB, 0.61506E+00_JPRB, 0.57354E+00_JPRB, 0.60008E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.55788E+00_JPRB, 0.55916E+00_JPRB, 0.55698E+00_JPRB, 0.55056E+00_JPRB, 0.54622E+00_JPRB, 0.54382E+00_JPRB, 0.54329E+00_JPRB ,&
&  0.54346E+00_JPRB, 0.54236E+00_JPRB, 0.53891E+00_JPRB, 0.53268E+00_JPRB, 0.52715E+00_JPRB, 0.52037E+00_JPRB, 0.51329E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.50179E+00_JPRB, 0.49850E+00_JPRB, 0.49461E+00_JPRB, 0.49329E+00_JPRB, 0.49489E+00_JPRB, 0.49700E+00_JPRB ,&
&  0.50040E+00_JPRB, 0.50366E+00_JPRB, 0.51416E+00_JPRB, 0.53142E+00_JPRB, 0.56214E+00_JPRB, 0.58372E+00_JPRB, 0.62393E+00_JPRB ,&
&  0.71253E+00_JPRB, 0.62643E+00_JPRB, 0.69407E+00_JPRB, 0.78008E+00_JPRB, 0.83689E+00_JPRB, 0.86418E+00_JPRB, 0.81328E+00_JPRB ,&
&  0.77289E+00_JPRB, 0.75076E+00_JPRB, 0.74269E+00_JPRB, 0.74103E+00_JPRB, 0.73943E+00_JPRB, 0.71891E+00_JPRB, 0.69196E+00_JPRB ,&
&  0.66263E+00_JPRB, 0.63820E+00_JPRB, 0.61700E+00_JPRB, 0.59468E+00_JPRB, 0.57451E+00_JPRB, 0.56129E+00_JPRB, 0.55469E+00_JPRB ,&
&  0.55273E+00_JPRB, 0.55247E+00_JPRB, 0.55189E+00_JPRB, 0.55217E+00_JPRB, 0.55402E+00_JPRB, 0.55849E+00_JPRB, 0.56958E+00_JPRB ,&
&  0.60751E+00_JPRB, 0.67823E+00_JPRB, 0.76119E+00_JPRB, 0.84173E+00_JPRB, 0.89850E+00_JPRB, 0.92547E+00_JPRB, 0.93700E+00_JPRB ,&
&  0.92457E+00_JPRB, 0.89338E+00_JPRB, 0.85329E+00_JPRB, 0.81244E+00_JPRB, 0.77547E+00_JPRB, 0.74564E+00_JPRB, 0.71942E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.67281E+00_JPRB, 0.65305E+00_JPRB, 0.63775E+00_JPRB, 0.62379E+00_JPRB, 0.61162E+00_JPRB, 0.60465E+00_JPRB ,&
&  0.60166E+00_JPRB, 0.60114E+00_JPRB, 0.60040E+00_JPRB, 0.60166E+00_JPRB, 0.60348E+00_JPRB, 0.60488E+00_JPRB, 0.60396E+00_JPRB ,&
&  0.59958E+00_JPRB, 0.59697E+00_JPRB, 0.59330E+00_JPRB, 0.60016E+00_JPRB, 0.59709E+00_JPRB, 0.59279E+00_JPRB, 0.58681E+00_JPRB ,&
&  0.58254E+00_JPRB, 0.57322E+00_JPRB, 0.51784E+00_JPRB, 0.54290E+00_JPRB, 0.55211E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.52925E+00_JPRB, 0.52926E+00_JPRB, 0.53057E+00_JPRB, 0.53028E+00_JPRB, 0.52912E+00_JPRB, 0.52714E+00_JPRB, 0.52372E+00_JPRB ,&
&  0.51870E+00_JPRB, 0.51522E+00_JPRB, 0.51225E+00_JPRB, 0.50817E+00_JPRB, 0.50362E+00_JPRB, 0.49982E+00_JPRB, 0.49649E+00_JPRB ,&
&  0.49296E+00_JPRB, 0.48905E+00_JPRB, 0.48615E+00_JPRB, 0.48406E+00_JPRB, 0.48193E+00_JPRB, 0.48110E+00_JPRB, 0.48210E+00_JPRB ,&
&  0.48190E+00_JPRB, 0.48517E+00_JPRB, 0.49852E+00_JPRB, 0.51708E+00_JPRB, 0.54314E+00_JPRB, 0.56785E+00_JPRB, 0.59202E+00_JPRB ,&
&  0.59192E+00_JPRB, 0.54044E+00_JPRB, 0.59003E+00_JPRB, 0.65497E+00_JPRB, 0.72017E+00_JPRB, 0.76248E+00_JPRB, 0.72969E+00_JPRB ,&
&  0.68191E+00_JPRB, 0.64260E+00_JPRB, 0.62057E+00_JPRB, 0.60991E+00_JPRB, 0.61130E+00_JPRB, 0.61181E+00_JPRB, 0.61179E+00_JPRB ,&
&  0.60562E+00_JPRB, 0.59089E+00_JPRB, 0.57334E+00_JPRB, 0.55626E+00_JPRB, 0.54335E+00_JPRB, 0.53322E+00_JPRB, 0.52489E+00_JPRB ,&
&  0.52016E+00_JPRB, 0.51833E+00_JPRB, 0.51749E+00_JPRB, 0.51728E+00_JPRB, 0.51796E+00_JPRB, 0.52088E+00_JPRB, 0.52436E+00_JPRB ,&
&  0.52921E+00_JPRB, 0.54457E+00_JPRB, 0.57880E+00_JPRB, 0.62273E+00_JPRB, 0.66779E+00_JPRB, 0.70270E+00_JPRB, 0.72787E+00_JPRB ,&
&  0.74649E+00_JPRB, 0.75561E+00_JPRB, 0.75077E+00_JPRB, 0.73169E+00_JPRB, 0.70625E+00_JPRB, 0.68137E+00_JPRB, 0.65734E+00_JPRB ,&
&  0.63319E+00_JPRB, 0.60592E+00_JPRB, 0.57907E+00_JPRB, 0.56144E+00_JPRB, 0.55114E+00_JPRB, 0.54169E+00_JPRB, 0.53424E+00_JPRB ,&
&  0.53114E+00_JPRB, 0.53143E+00_JPRB, 0.53345E+00_JPRB, 0.53542E+00_JPRB, 0.53749E+00_JPRB, 0.54060E+00_JPRB, 0.54293E+00_JPRB ,&
&  0.54405E+00_JPRB, 0.54656E+00_JPRB, 0.55074E+00_JPRB, 0.56104E+00_JPRB, 0.56733E+00_JPRB, 0.56887E+00_JPRB, 0.56566E+00_JPRB ,&
&  0.55688E+00_JPRB, 0.54553E+00_JPRB, 0.53410E+00_JPRB, 0.52769E+00_JPRB, 0.52739E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.50890E+00_JPRB, 0.50822E+00_JPRB, 0.50798E+00_JPRB, 0.50642E+00_JPRB, 0.50484E+00_JPRB, 0.50314E+00_JPRB, 0.49894E+00_JPRB ,&
&  0.49702E+00_JPRB, 0.49265E+00_JPRB, 0.48855E+00_JPRB, 0.48632E+00_JPRB, 0.48365E+00_JPRB, 0.48059E+00_JPRB, 0.47845E+00_JPRB ,&
&  0.47612E+00_JPRB, 0.47321E+00_JPRB, 0.47084E+00_JPRB, 0.46781E+00_JPRB, 0.46348E+00_JPRB, 0.46040E+00_JPRB, 0.46069E+00_JPRB ,&
&  0.46112E+00_JPRB, 0.46402E+00_JPRB, 0.47545E+00_JPRB, 0.48740E+00_JPRB, 0.50337E+00_JPRB, 0.52089E+00_JPRB, 0.53564E+00_JPRB ,&
&  0.56234E+00_JPRB, 0.50095E+00_JPRB, 0.54746E+00_JPRB, 0.58597E+00_JPRB, 0.64131E+00_JPRB, 0.68261E+00_JPRB, 0.66786E+00_JPRB ,&
&  0.62690E+00_JPRB, 0.58320E+00_JPRB, 0.55660E+00_JPRB, 0.54447E+00_JPRB, 0.54966E+00_JPRB, 0.55627E+00_JPRB, 0.55535E+00_JPRB ,&
&  0.54824E+00_JPRB, 0.53303E+00_JPRB, 0.51566E+00_JPRB, 0.50477E+00_JPRB, 0.49879E+00_JPRB, 0.49391E+00_JPRB, 0.48913E+00_JPRB ,&
&  0.48551E+00_JPRB, 0.48371E+00_JPRB, 0.48268E+00_JPRB, 0.48211E+00_JPRB, 0.48256E+00_JPRB, 0.48422E+00_JPRB, 0.48653E+00_JPRB ,&
&  0.48802E+00_JPRB, 0.48820E+00_JPRB, 0.49238E+00_JPRB, 0.50642E+00_JPRB, 0.52669E+00_JPRB, 0.54745E+00_JPRB, 0.56490E+00_JPRB ,&
&  0.57976E+00_JPRB, 0.59347E+00_JPRB, 0.60271E+00_JPRB, 0.60315E+00_JPRB, 0.59491E+00_JPRB, 0.58153E+00_JPRB, 0.56660E+00_JPRB ,&
&  0.55033E+00_JPRB, 0.53301E+00_JPRB, 0.51378E+00_JPRB, 0.50143E+00_JPRB, 0.49506E+00_JPRB, 0.49122E+00_JPRB, 0.48853E+00_JPRB ,&
&  0.48689E+00_JPRB, 0.48620E+00_JPRB, 0.48734E+00_JPRB, 0.48904E+00_JPRB, 0.49001E+00_JPRB, 0.49095E+00_JPRB, 0.49292E+00_JPRB ,&
&  0.49669E+00_JPRB, 0.50288E+00_JPRB, 0.51081E+00_JPRB, 0.52207E+00_JPRB, 0.53178E+00_JPRB, 0.53754E+00_JPRB, 0.53992E+00_JPRB ,&
&  0.53672E+00_JPRB, 0.52963E+00_JPRB, 0.52015E+00_JPRB, 0.51187E+00_JPRB, 0.50989E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.49345E+00_JPRB, 0.48893E+00_JPRB, 0.48578E+00_JPRB, 0.48104E+00_JPRB, 0.47716E+00_JPRB, 0.47415E+00_JPRB, 0.47116E+00_JPRB ,&
&  0.46914E+00_JPRB, 0.46924E+00_JPRB, 0.46774E+00_JPRB, 0.46671E+00_JPRB, 0.46513E+00_JPRB, 0.46169E+00_JPRB, 0.45775E+00_JPRB ,&
&  0.45562E+00_JPRB, 0.45374E+00_JPRB, 0.45203E+00_JPRB, 0.44997E+00_JPRB, 0.44757E+00_JPRB, 0.44436E+00_JPRB, 0.44385E+00_JPRB ,&
&  0.44408E+00_JPRB, 0.44683E+00_JPRB, 0.45201E+00_JPRB, 0.45805E+00_JPRB, 0.46866E+00_JPRB, 0.48382E+00_JPRB, 0.49852E+00_JPRB ,&
&  0.51703E+00_JPRB, 0.50429E+00_JPRB, 0.53437E+00_JPRB, 0.55256E+00_JPRB, 0.58409E+00_JPRB, 0.61112E+00_JPRB, 0.61270E+00_JPRB ,&
&  0.59720E+00_JPRB, 0.56864E+00_JPRB, 0.54605E+00_JPRB, 0.53448E+00_JPRB, 0.53437E+00_JPRB, 0.53247E+00_JPRB, 0.52157E+00_JPRB ,&
&  0.50499E+00_JPRB, 0.48594E+00_JPRB, 0.47176E+00_JPRB, 0.46410E+00_JPRB, 0.46077E+00_JPRB, 0.45897E+00_JPRB, 0.45601E+00_JPRB ,&
&  0.45275E+00_JPRB, 0.44971E+00_JPRB, 0.44800E+00_JPRB, 0.44709E+00_JPRB, 0.44632E+00_JPRB, 0.44623E+00_JPRB, 0.44681E+00_JPRB ,&
&  0.44740E+00_JPRB, 0.44746E+00_JPRB, 0.44711E+00_JPRB, 0.44865E+00_JPRB, 0.45441E+00_JPRB, 0.46350E+00_JPRB, 0.47324E+00_JPRB ,&
&  0.48041E+00_JPRB, 0.48748E+00_JPRB, 0.49412E+00_JPRB, 0.49888E+00_JPRB, 0.49869E+00_JPRB, 0.49277E+00_JPRB, 0.48394E+00_JPRB ,&
&  0.47588E+00_JPRB, 0.46829E+00_JPRB, 0.46059E+00_JPRB, 0.45442E+00_JPRB, 0.45085E+00_JPRB, 0.44992E+00_JPRB, 0.44998E+00_JPRB ,&
&  0.45031E+00_JPRB, 0.45090E+00_JPRB, 0.45221E+00_JPRB, 0.45440E+00_JPRB, 0.45585E+00_JPRB, 0.45721E+00_JPRB, 0.45856E+00_JPRB ,&
&  0.46258E+00_JPRB, 0.46770E+00_JPRB, 0.47506E+00_JPRB, 0.48604E+00_JPRB, 0.49639E+00_JPRB, 0.50388E+00_JPRB, 0.50995E+00_JPRB ,&
&  0.51251E+00_JPRB, 0.51163E+00_JPRB, 0.50833E+00_JPRB, 0.50395E+00_JPRB, 0.49910E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.46449E+00_JPRB, 0.45781E+00_JPRB, 0.45302E+00_JPRB, 0.44951E+00_JPRB, 0.44872E+00_JPRB, 0.44672E+00_JPRB, 0.44274E+00_JPRB ,&
&  0.43961E+00_JPRB, 0.44087E+00_JPRB, 0.44398E+00_JPRB, 0.44538E+00_JPRB, 0.44451E+00_JPRB, 0.44254E+00_JPRB, 0.43897E+00_JPRB ,&
&  0.43434E+00_JPRB, 0.43199E+00_JPRB, 0.43181E+00_JPRB, 0.43295E+00_JPRB, 0.43246E+00_JPRB, 0.42984E+00_JPRB, 0.42748E+00_JPRB ,&
&  0.42741E+00_JPRB, 0.42898E+00_JPRB, 0.43041E+00_JPRB, 0.43315E+00_JPRB, 0.44079E+00_JPRB, 0.45527E+00_JPRB, 0.46826E+00_JPRB ,&
&  0.47786E+00_JPRB, 0.48267E+00_JPRB, 0.47788E+00_JPRB, 0.49960E+00_JPRB, 0.51753E+00_JPRB, 0.54061E+00_JPRB, 0.55726E+00_JPRB ,&
&  0.56211E+00_JPRB, 0.55491E+00_JPRB, 0.53863E+00_JPRB, 0.52487E+00_JPRB, 0.51470E+00_JPRB, 0.50064E+00_JPRB, 0.47970E+00_JPRB ,&
&  0.45831E+00_JPRB, 0.44137E+00_JPRB, 0.43114E+00_JPRB, 0.42618E+00_JPRB, 0.42506E+00_JPRB, 0.42483E+00_JPRB, 0.42216E+00_JPRB ,&
&  0.41898E+00_JPRB, 0.41602E+00_JPRB, 0.41465E+00_JPRB, 0.41352E+00_JPRB, 0.41217E+00_JPRB, 0.41107E+00_JPRB, 0.41060E+00_JPRB ,&
&  0.41054E+00_JPRB, 0.41043E+00_JPRB, 0.41055E+00_JPRB, 0.41098E+00_JPRB, 0.41130E+00_JPRB, 0.41289E+00_JPRB, 0.41669E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.42417E+00_JPRB, 0.42719E+00_JPRB, 0.42949E+00_JPRB, 0.43050E+00_JPRB, 0.42972E+00_JPRB, 0.42631E+00_JPRB ,&
&  0.42208E+00_JPRB, 0.41970E+00_JPRB, 0.41796E+00_JPRB, 0.41539E+00_JPRB, 0.41292E+00_JPRB, 0.41243E+00_JPRB, 0.41357E+00_JPRB ,&
&  0.41507E+00_JPRB, 0.41728E+00_JPRB, 0.41933E+00_JPRB, 0.42141E+00_JPRB, 0.42261E+00_JPRB, 0.42419E+00_JPRB, 0.42585E+00_JPRB ,&
&  0.42822E+00_JPRB, 0.43175E+00_JPRB, 0.43669E+00_JPRB, 0.44322E+00_JPRB, 0.45033E+00_JPRB, 0.45620E+00_JPRB, 0.46256E+00_JPRB ,&
&  0.46851E+00_JPRB, 0.47208E+00_JPRB, 0.47309E+00_JPRB, 0.47320E+00_JPRB, 0.47057E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.42955E+00_JPRB, 0.42526E+00_JPRB, 0.42206E+00_JPRB, 0.42109E+00_JPRB, 0.42235E+00_JPRB, 0.42169E+00_JPRB, 0.41854E+00_JPRB ,&
&  0.41579E+00_JPRB, 0.41699E+00_JPRB, 0.42091E+00_JPRB, 0.42359E+00_JPRB, 0.42496E+00_JPRB, 0.42592E+00_JPRB, 0.42417E+00_JPRB ,&
&  0.41954E+00_JPRB, 0.41531E+00_JPRB, 0.41581E+00_JPRB, 0.41804E+00_JPRB, 0.41884E+00_JPRB, 0.41646E+00_JPRB, 0.41376E+00_JPRB ,&
&  0.41341E+00_JPRB, 0.41351E+00_JPRB, 0.41578E+00_JPRB, 0.41804E+00_JPRB, 0.42431E+00_JPRB, 0.43623E+00_JPRB, 0.44686E+00_JPRB ,&
&  0.45394E+00_JPRB, 0.45339E+00_JPRB, 0.45254E+00_JPRB, 0.45850E+00_JPRB, 0.46615E+00_JPRB, 0.48271E+00_JPRB, 0.50123E+00_JPRB ,&
&  0.51161E+00_JPRB, 0.51119E+00_JPRB, 0.49768E+00_JPRB, 0.48192E+00_JPRB, 0.46739E+00_JPRB, 0.45109E+00_JPRB, 0.43487E+00_JPRB ,&
&  0.42005E+00_JPRB, 0.40975E+00_JPRB, 0.40383E+00_JPRB, 0.40108E+00_JPRB, 0.39986E+00_JPRB, 0.39842E+00_JPRB, 0.39506E+00_JPRB ,&
&  0.39099E+00_JPRB, 0.38777E+00_JPRB, 0.38544E+00_JPRB, 0.38384E+00_JPRB, 0.38242E+00_JPRB, 0.38170E+00_JPRB, 0.38082E+00_JPRB ,&
&  0.38038E+00_JPRB, 0.38069E+00_JPRB, 0.38137E+00_JPRB, 0.38107E+00_JPRB, 0.38047E+00_JPRB, 0.37945E+00_JPRB, 0.37969E+00_JPRB ,&
&  0.38172E+00_JPRB, 0.38253E+00_JPRB, 0.38326E+00_JPRB, 0.38507E+00_JPRB, 0.38639E+00_JPRB, 0.38738E+00_JPRB, 0.38703E+00_JPRB ,&
&  0.38623E+00_JPRB, 0.38601E+00_JPRB, 0.38597E+00_JPRB, 0.38524E+00_JPRB, 0.38433E+00_JPRB, 0.38392E+00_JPRB, 0.38421E+00_JPRB ,&
&  0.38602E+00_JPRB, 0.38836E+00_JPRB, 0.39025E+00_JPRB, 0.39212E+00_JPRB, 0.39417E+00_JPRB, 0.39693E+00_JPRB, 0.39810E+00_JPRB ,&
&  0.39878E+00_JPRB, 0.40040E+00_JPRB, 0.40386E+00_JPRB, 0.40837E+00_JPRB, 0.41283E+00_JPRB, 0.41621E+00_JPRB, 0.41999E+00_JPRB ,&
&  0.42377E+00_JPRB, 0.42711E+00_JPRB, 0.42937E+00_JPRB, 0.43022E+00_JPRB, 0.43060E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.39700E+00_JPRB, 0.39557E+00_JPRB, 0.39503E+00_JPRB, 0.39541E+00_JPRB, 0.39708E+00_JPRB, 0.39654E+00_JPRB, 0.39557E+00_JPRB ,&
&  0.39492E+00_JPRB, 0.39718E+00_JPRB, 0.40154E+00_JPRB, 0.40446E+00_JPRB, 0.40551E+00_JPRB, 0.40589E+00_JPRB, 0.40525E+00_JPRB ,&
&  0.40419E+00_JPRB, 0.40334E+00_JPRB, 0.40225E+00_JPRB, 0.40319E+00_JPRB, 0.40267E+00_JPRB, 0.40075E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.40046E+00_JPRB, 0.40089E+00_JPRB, 0.40369E+00_JPRB, 0.40640E+00_JPRB, 0.41241E+00_JPRB, 0.42340E+00_JPRB, 0.43277E+00_JPRB ,&
&  0.43723E+00_JPRB, 0.43224E+00_JPRB, 0.42666E+00_JPRB, 0.42704E+00_JPRB, 0.43060E+00_JPRB, 0.43851E+00_JPRB, 0.44842E+00_JPRB ,&
&  0.45430E+00_JPRB, 0.44974E+00_JPRB, 0.43777E+00_JPRB, 0.42567E+00_JPRB, 0.41515E+00_JPRB, 0.40603E+00_JPRB, 0.39803E+00_JPRB ,&
&  0.39105E+00_JPRB, 0.38635E+00_JPRB, 0.38420E+00_JPRB, 0.38356E+00_JPRB, 0.38252E+00_JPRB, 0.37862E+00_JPRB, 0.37391E+00_JPRB ,&
&  0.36835E+00_JPRB, 0.36436E+00_JPRB, 0.36117E+00_JPRB, 0.35995E+00_JPRB, 0.35986E+00_JPRB, 0.35931E+00_JPRB, 0.35787E+00_JPRB ,&
&  0.35618E+00_JPRB, 0.35550E+00_JPRB, 0.35488E+00_JPRB, 0.35332E+00_JPRB, 0.35136E+00_JPRB, 0.35040E+00_JPRB, 0.34939E+00_JPRB ,&
&  0.34847E+00_JPRB, 0.34918E+00_JPRB, 0.35104E+00_JPRB, 0.35350E+00_JPRB, 0.35411E+00_JPRB, 0.35559E+00_JPRB, 0.35763E+00_JPRB ,&
&  0.35703E+00_JPRB, 0.35425E+00_JPRB, 0.35223E+00_JPRB, 0.35205E+00_JPRB, 0.35355E+00_JPRB, 0.35608E+00_JPRB, 0.35729E+00_JPRB ,&
&  0.35838E+00_JPRB, 0.35857E+00_JPRB, 0.35982E+00_JPRB, 0.36171E+00_JPRB, 0.36442E+00_JPRB, 0.36706E+00_JPRB, 0.36870E+00_JPRB ,&
&  0.36959E+00_JPRB, 0.37033E+00_JPRB, 0.37288E+00_JPRB, 0.37668E+00_JPRB, 0.38005E+00_JPRB, 0.38377E+00_JPRB, 0.38670E+00_JPRB ,&
&  0.38886E+00_JPRB, 0.39119E+00_JPRB, 0.39288E+00_JPRB, 0.39409E+00_JPRB, 0.39565E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.37560E+00_JPRB, 0.37644E+00_JPRB, 0.37796E+00_JPRB, 0.37860E+00_JPRB, 0.37973E+00_JPRB, 0.38001E+00_JPRB, 0.38102E+00_JPRB ,&
&  0.38157E+00_JPRB, 0.38356E+00_JPRB, 0.38629E+00_JPRB, 0.38654E+00_JPRB, 0.38636E+00_JPRB, 0.38635E+00_JPRB, 0.38721E+00_JPRB ,&
&  0.38967E+00_JPRB, 0.39052E+00_JPRB, 0.39007E+00_JPRB, 0.39040E+00_JPRB, 0.38878E+00_JPRB, 0.38708E+00_JPRB, 0.38851E+00_JPRB ,&
&  0.38864E+00_JPRB, 0.38930E+00_JPRB, 0.39185E+00_JPRB, 0.39458E+00_JPRB, 0.40036E+00_JPRB, 0.40821E+00_JPRB, 0.41677E+00_JPRB ,&
&  0.42198E+00_JPRB, 0.41891E+00_JPRB, 0.41066E+00_JPRB, 0.40471E+00_JPRB, 0.40335E+00_JPRB, 0.40457E+00_JPRB, 0.40770E+00_JPRB ,&
&  0.40742E+00_JPRB, 0.40120E+00_JPRB, 0.39242E+00_JPRB, 0.38697E+00_JPRB, 0.38397E+00_JPRB, 0.37945E+00_JPRB, 0.37333E+00_JPRB ,&
&  0.36861E+00_JPRB, 0.36679E+00_JPRB, 0.36681E+00_JPRB, 0.36581E+00_JPRB, 0.36330E+00_JPRB, 0.35899E+00_JPRB, 0.35368E+00_JPRB ,&
&  0.34884E+00_JPRB, 0.34417E+00_JPRB, 0.34110E+00_JPRB, 0.33919E+00_JPRB, 0.33891E+00_JPRB, 0.33872E+00_JPRB, 0.33731E+00_JPRB ,&
&  0.33470E+00_JPRB, 0.33288E+00_JPRB, 0.33065E+00_JPRB, 0.32697E+00_JPRB, 0.32437E+00_JPRB, 0.32354E+00_JPRB, 0.32254E+00_JPRB ,&
&  0.32127E+00_JPRB, 0.32146E+00_JPRB, 0.32203E+00_JPRB, 0.32220E+00_JPRB, 0.32380E+00_JPRB, 0.32619E+00_JPRB, 0.32393E+00_JPRB ,&
&  0.32158E+00_JPRB, 0.31997E+00_JPRB, 0.32006E+00_JPRB, 0.32206E+00_JPRB, 0.32534E+00_JPRB, 0.32880E+00_JPRB, 0.33156E+00_JPRB ,&
&  0.33269E+00_JPRB, 0.33295E+00_JPRB, 0.33332E+00_JPRB, 0.33384E+00_JPRB, 0.33591E+00_JPRB, 0.33960E+00_JPRB, 0.34367E+00_JPRB ,&
&  0.34540E+00_JPRB, 0.34817E+00_JPRB, 0.35142E+00_JPRB, 0.35558E+00_JPRB, 0.35818E+00_JPRB, 0.36075E+00_JPRB, 0.36309E+00_JPRB ,&
&  0.36477E+00_JPRB, 0.36684E+00_JPRB, 0.36953E+00_JPRB, 0.37163E+00_JPRB, 0.37375E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.35965E+00_JPRB, 0.36139E+00_JPRB, 0.36245E+00_JPRB, 0.36435E+00_JPRB, 0.36760E+00_JPRB, 0.36917E+00_JPRB, 0.36885E+00_JPRB ,&
&  0.36897E+00_JPRB, 0.37078E+00_JPRB, 0.37234E+00_JPRB, 0.37310E+00_JPRB, 0.37086E+00_JPRB, 0.37056E+00_JPRB, 0.37259E+00_JPRB ,&
&  0.37527E+00_JPRB, 0.37725E+00_JPRB, 0.37751E+00_JPRB, 0.37772E+00_JPRB, 0.37638E+00_JPRB, 0.37446E+00_JPRB, 0.37565E+00_JPRB ,&
&  0.37669E+00_JPRB, 0.37710E+00_JPRB, 0.37879E+00_JPRB, 0.38102E+00_JPRB, 0.38554E+00_JPRB, 0.39094E+00_JPRB, 0.39615E+00_JPRB ,&
&  0.40087E+00_JPRB, 0.40059E+00_JPRB, 0.39215E+00_JPRB, 0.38317E+00_JPRB, 0.38265E+00_JPRB, 0.38310E+00_JPRB, 0.38263E+00_JPRB ,&
&  0.38025E+00_JPRB, 0.37697E+00_JPRB, 0.37479E+00_JPRB, 0.37296E+00_JPRB, 0.37029E+00_JPRB, 0.36486E+00_JPRB, 0.35879E+00_JPRB ,&
&  0.35377E+00_JPRB, 0.35194E+00_JPRB, 0.35048E+00_JPRB, 0.34754E+00_JPRB, 0.34304E+00_JPRB, 0.33911E+00_JPRB, 0.33498E+00_JPRB ,&
&  0.32832E+00_JPRB, 0.32204E+00_JPRB, 0.31849E+00_JPRB, 0.31724E+00_JPRB, 0.31714E+00_JPRB, 0.31624E+00_JPRB, 0.31376E+00_JPRB ,&
&  0.31096E+00_JPRB, 0.30779E+00_JPRB, 0.30585E+00_JPRB, 0.30535E+00_JPRB, 0.30672E+00_JPRB, 0.30469E+00_JPRB, 0.29940E+00_JPRB ,&
&  0.29670E+00_JPRB, 0.29571E+00_JPRB, 0.29688E+00_JPRB, 0.29615E+00_JPRB, 0.29557E+00_JPRB, 0.29457E+00_JPRB, 0.29515E+00_JPRB ,&
&  0.29653E+00_JPRB, 0.29651E+00_JPRB, 0.29952E+00_JPRB, 0.30500E+00_JPRB, 0.30979E+00_JPRB, 0.31260E+00_JPRB, 0.31434E+00_JPRB ,&
&  0.31503E+00_JPRB, 0.31483E+00_JPRB, 0.31460E+00_JPRB, 0.31569E+00_JPRB, 0.31870E+00_JPRB, 0.32280E+00_JPRB, 0.32640E+00_JPRB ,&
&  0.32701E+00_JPRB, 0.32840E+00_JPRB, 0.33206E+00_JPRB, 0.33755E+00_JPRB, 0.34070E+00_JPRB, 0.34305E+00_JPRB, 0.34485E+00_JPRB ,&
&  0.34554E+00_JPRB, 0.34721E+00_JPRB, 0.35115E+00_JPRB, 0.35377E+00_JPRB, 0.35644E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.34332E+00_JPRB, 0.34426E+00_JPRB, 0.34639E+00_JPRB, 0.34909E+00_JPRB, 0.35055E+00_JPRB, 0.35012E+00_JPRB, 0.34896E+00_JPRB ,&
&  0.34942E+00_JPRB, 0.35220E+00_JPRB, 0.35518E+00_JPRB, 0.35637E+00_JPRB, 0.35706E+00_JPRB, 0.35774E+00_JPRB, 0.35985E+00_JPRB ,&
&  0.36273E+00_JPRB, 0.36609E+00_JPRB, 0.36622E+00_JPRB, 0.36622E+00_JPRB, 0.36531E+00_JPRB, 0.36383E+00_JPRB, 0.36492E+00_JPRB ,&
&  0.36801E+00_JPRB, 0.36797E+00_JPRB, 0.36764E+00_JPRB, 0.36711E+00_JPRB, 0.36961E+00_JPRB, 0.37412E+00_JPRB, 0.37764E+00_JPRB ,&
&  0.37853E+00_JPRB, 0.37234E+00_JPRB, 0.36350E+00_JPRB, 0.36319E+00_JPRB, 0.36487E+00_JPRB, 0.36640E+00_JPRB, 0.36628E+00_JPRB ,&
&  0.36569E+00_JPRB, 0.36458E+00_JPRB, 0.36321E+00_JPRB, 0.36160E+00_JPRB, 0.35916E+00_JPRB, 0.35336E+00_JPRB, 0.34672E+00_JPRB ,&
&  0.34143E+00_JPRB, 0.33732E+00_JPRB, 0.33348E+00_JPRB, 0.32788E+00_JPRB, 0.32225E+00_JPRB, 0.31813E+00_JPRB, 0.31311E+00_JPRB ,&
&  0.30711E+00_JPRB, 0.30315E+00_JPRB, 0.30180E+00_JPRB, 0.30136E+00_JPRB, 0.30104E+00_JPRB, 0.29980E+00_JPRB, 0.29819E+00_JPRB ,&
&  0.29554E+00_JPRB, 0.29352E+00_JPRB, 0.29166E+00_JPRB, 0.29030E+00_JPRB, 0.28265E+00_JPRB, 0.27511E+00_JPRB, 0.26928E+00_JPRB ,&
&  0.26918E+00_JPRB, 0.27169E+00_JPRB, 0.27415E+00_JPRB, 0.27570E+00_JPRB, 0.27908E+00_JPRB, 0.28022E+00_JPRB, 0.27949E+00_JPRB ,&
&  0.27528E+00_JPRB, 0.27345E+00_JPRB, 0.27561E+00_JPRB, 0.27742E+00_JPRB, 0.27832E+00_JPRB, 0.28026E+00_JPRB, 0.28179E+00_JPRB ,&
&  0.28506E+00_JPRB, 0.28704E+00_JPRB, 0.28963E+00_JPRB, 0.29269E+00_JPRB, 0.29515E+00_JPRB, 0.29617E+00_JPRB, 0.29403E+00_JPRB ,&
&  0.29455E+00_JPRB, 0.30140E+00_JPRB, 0.31254E+00_JPRB, 0.32172E+00_JPRB, 0.32506E+00_JPRB, 0.32798E+00_JPRB, 0.33036E+00_JPRB ,&
&  0.33176E+00_JPRB, 0.33601E+00_JPRB, 0.33951E+00_JPRB, 0.34131E+00_JPRB, 0.34225E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.33120E+00_JPRB, 0.33037E+00_JPRB, 0.32890E+00_JPRB, 0.32774E+00_JPRB, 0.32607E+00_JPRB, 0.32458E+00_JPRB, 0.32543E+00_JPRB ,&
&  0.32865E+00_JPRB, 0.33287E+00_JPRB, 0.33694E+00_JPRB, 0.34005E+00_JPRB, 0.34239E+00_JPRB, 0.34359E+00_JPRB, 0.34537E+00_JPRB ,&
&  0.34738E+00_JPRB, 0.35109E+00_JPRB, 0.35302E+00_JPRB, 0.35463E+00_JPRB, 0.35373E+00_JPRB, 0.35271E+00_JPRB, 0.35371E+00_JPRB ,&
&  0.35699E+00_JPRB, 0.35824E+00_JPRB, 0.35715E+00_JPRB, 0.35478E+00_JPRB, 0.35556E+00_JPRB, 0.35848E+00_JPRB, 0.35906E+00_JPRB ,&
&  0.35204E+00_JPRB, 0.33699E+00_JPRB, 0.32752E+00_JPRB, 0.33241E+00_JPRB, 0.34148E+00_JPRB, 0.34558E+00_JPRB, 0.34900E+00_JPRB ,&
&  0.35110E+00_JPRB, 0.35109E+00_JPRB, 0.34970E+00_JPRB, 0.34724E+00_JPRB, 0.34345E+00_JPRB, 0.33716E+00_JPRB, 0.33001E+00_JPRB ,&
&  0.32391E+00_JPRB, 0.32117E+00_JPRB, 0.31733E+00_JPRB, 0.31037E+00_JPRB, 0.30391E+00_JPRB, 0.30005E+00_JPRB, 0.29471E+00_JPRB ,&
&  0.28907E+00_JPRB, 0.28531E+00_JPRB, 0.28453E+00_JPRB, 0.28375E+00_JPRB, 0.28315E+00_JPRB, 0.28175E+00_JPRB, 0.27813E+00_JPRB ,&
&  0.27526E+00_JPRB, 0.27030E+00_JPRB, 0.26443E+00_JPRB, 0.25892E+00_JPRB, 0.25540E+00_JPRB, 0.25139E+00_JPRB, 0.24609E+00_JPRB ,&
&  0.24351E+00_JPRB, 0.24683E+00_JPRB, 0.25781E+00_JPRB, 0.26889E+00_JPRB, 0.26917E+00_JPRB, 0.25755E+00_JPRB, 0.25496E+00_JPRB ,&
&  0.25559E+00_JPRB, 0.25583E+00_JPRB, 0.25645E+00_JPRB, 0.25834E+00_JPRB, 0.25920E+00_JPRB, 0.25860E+00_JPRB, 0.25925E+00_JPRB ,&
&  0.26205E+00_JPRB, 0.26485E+00_JPRB, 0.26623E+00_JPRB, 0.26805E+00_JPRB, 0.26924E+00_JPRB, 0.26914E+00_JPRB, 0.26869E+00_JPRB ,&
&  0.26965E+00_JPRB, 0.27370E+00_JPRB, 0.27995E+00_JPRB, 0.28861E+00_JPRB, 0.29421E+00_JPRB, 0.29929E+00_JPRB, 0.30534E+00_JPRB ,&
&  0.31319E+00_JPRB, 0.31963E+00_JPRB, 0.32495E+00_JPRB, 0.32954E+00_JPRB, 0.33100E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.31950E+00_JPRB, 0.32056E+00_JPRB, 0.31590E+00_JPRB, 0.31205E+00_JPRB, 0.31005E+00_JPRB, 0.30962E+00_JPRB, 0.31145E+00_JPRB ,&
&  0.31555E+00_JPRB, 0.31896E+00_JPRB, 0.32220E+00_JPRB, 0.32509E+00_JPRB, 0.32634E+00_JPRB, 0.32598E+00_JPRB, 0.32719E+00_JPRB ,&
&  0.32844E+00_JPRB, 0.33185E+00_JPRB, 0.33456E+00_JPRB, 0.33841E+00_JPRB, 0.33875E+00_JPRB, 0.33950E+00_JPRB, 0.34086E+00_JPRB ,&
&  0.34027E+00_JPRB, 0.34019E+00_JPRB, 0.34144E+00_JPRB, 0.33859E+00_JPRB, 0.33692E+00_JPRB, 0.33450E+00_JPRB, 0.33301E+00_JPRB ,&
&  0.32561E+00_JPRB, 0.31769E+00_JPRB, 0.31138E+00_JPRB, 0.31671E+00_JPRB, 0.32449E+00_JPRB, 0.32538E+00_JPRB, 0.33040E+00_JPRB ,&
&  0.33234E+00_JPRB, 0.33030E+00_JPRB, 0.32839E+00_JPRB, 0.32446E+00_JPRB, 0.32087E+00_JPRB, 0.31664E+00_JPRB, 0.31318E+00_JPRB ,&
&  0.30912E+00_JPRB, 0.30992E+00_JPRB, 0.30435E+00_JPRB, 0.28862E+00_JPRB, 0.28157E+00_JPRB, 0.27706E+00_JPRB, 0.26875E+00_JPRB ,&
&  0.25991E+00_JPRB, 0.25821E+00_JPRB, 0.25802E+00_JPRB, 0.25455E+00_JPRB, 0.25399E+00_JPRB, 0.25431E+00_JPRB, 0.25407E+00_JPRB ,&
&  0.25221E+00_JPRB, 0.25187E+00_JPRB, 0.25035E+00_JPRB, 0.24802E+00_JPRB, 0.24559E+00_JPRB, 0.24454E+00_JPRB, 0.24497E+00_JPRB ,&
&  0.24668E+00_JPRB, 0.24810E+00_JPRB, 0.25438E+00_JPRB, 0.25778E+00_JPRB, 0.25076E+00_JPRB, 0.24914E+00_JPRB, 0.25138E+00_JPRB ,&
&  0.24639E+00_JPRB, 0.24560E+00_JPRB, 0.24565E+00_JPRB, 0.24566E+00_JPRB, 0.24614E+00_JPRB, 0.24716E+00_JPRB, 0.24675E+00_JPRB ,&
&  0.24752E+00_JPRB, 0.24944E+00_JPRB, 0.25004E+00_JPRB, 0.25014E+00_JPRB, 0.24973E+00_JPRB, 0.25030E+00_JPRB, 0.25188E+00_JPRB ,&
&  0.25359E+00_JPRB, 0.25702E+00_JPRB, 0.26055E+00_JPRB, 0.26583E+00_JPRB, 0.27002E+00_JPRB, 0.27342E+00_JPRB, 0.27989E+00_JPRB ,&
&  0.28485E+00_JPRB, 0.29613E+00_JPRB, 0.31435E+00_JPRB, 0.32169E+00_JPRB, 0.31417E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.30773E+00_JPRB, 0.30622E+00_JPRB, 0.30403E+00_JPRB, 0.30223E+00_JPRB, 0.30336E+00_JPRB, 0.30476E+00_JPRB, 0.30690E+00_JPRB ,&
&  0.30924E+00_JPRB, 0.31005E+00_JPRB, 0.31022E+00_JPRB, 0.30737E+00_JPRB, 0.30606E+00_JPRB, 0.30079E+00_JPRB, 0.29913E+00_JPRB ,&
&  0.29795E+00_JPRB, 0.30081E+00_JPRB, 0.30433E+00_JPRB, 0.30953E+00_JPRB, 0.31139E+00_JPRB, 0.31336E+00_JPRB, 0.31201E+00_JPRB ,&
&  0.30946E+00_JPRB, 0.30654E+00_JPRB, 0.30352E+00_JPRB, 0.30118E+00_JPRB, 0.30043E+00_JPRB, 0.29810E+00_JPRB, 0.29584E+00_JPRB ,&
&  0.29598E+00_JPRB, 0.29778E+00_JPRB, 0.29843E+00_JPRB, 0.29913E+00_JPRB, 0.30485E+00_JPRB, 0.30470E+00_JPRB, 0.30741E+00_JPRB ,&
&  0.30851E+00_JPRB, 0.30981E+00_JPRB, 0.30973E+00_JPRB, 0.30853E+00_JPRB, 0.30739E+00_JPRB, 0.30009E+00_JPRB, 0.29734E+00_JPRB ,&
&  0.29051E+00_JPRB, 0.28672E+00_JPRB, 0.27932E+00_JPRB, 0.27221E+00_JPRB, 0.26362E+00_JPRB, 0.25716E+00_JPRB, 0.25165E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24391E+00_JPRB, 0.24128E+00_JPRB, 0.24097E+00_JPRB, 0.24097E+00_JPRB, 0.24033E+00_JPRB, 0.24051E+00_JPRB ,&
&  0.24174E+00_JPRB, 0.24304E+00_JPRB, 0.24289E+00_JPRB, 0.24357E+00_JPRB, 0.24410E+00_JPRB, 0.24489E+00_JPRB, 0.24658E+00_JPRB ,&
&  0.24915E+00_JPRB, 0.25107E+00_JPRB, 0.25135E+00_JPRB, 0.25391E+00_JPRB, 0.25319E+00_JPRB, 0.25014E+00_JPRB, 0.24729E+00_JPRB ,&
&  0.24370E+00_JPRB, 0.24217E+00_JPRB, 0.24027E+00_JPRB, 0.23915E+00_JPRB, 0.23892E+00_JPRB, 0.23902E+00_JPRB, 0.23992E+00_JPRB ,&
&  0.24091E+00_JPRB, 0.24200E+00_JPRB, 0.24235E+00_JPRB, 0.24281E+00_JPRB, 0.24316E+00_JPRB, 0.24417E+00_JPRB, 0.24624E+00_JPRB ,&
&  0.24914E+00_JPRB, 0.25249E+00_JPRB, 0.25576E+00_JPRB, 0.25969E+00_JPRB, 0.26610E+00_JPRB, 0.27428E+00_JPRB, 0.28653E+00_JPRB ,&
&  0.29504E+00_JPRB, 0.30505E+00_JPRB, 0.30206E+00_JPRB, 0.30718E+00_JPRB, 0.30870E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.29868E+00_JPRB, 0.29856E+00_JPRB, 0.29732E+00_JPRB, 0.29670E+00_JPRB, 0.29723E+00_JPRB, 0.29818E+00_JPRB, 0.29971E+00_JPRB ,&
&  0.29634E+00_JPRB, 0.29441E+00_JPRB, 0.29011E+00_JPRB, 0.28539E+00_JPRB, 0.28105E+00_JPRB, 0.27820E+00_JPRB, 0.27666E+00_JPRB ,&
&  0.27661E+00_JPRB, 0.27942E+00_JPRB, 0.28119E+00_JPRB, 0.28379E+00_JPRB, 0.28380E+00_JPRB, 0.28395E+00_JPRB, 0.28203E+00_JPRB ,&
&  0.27999E+00_JPRB, 0.27923E+00_JPRB, 0.27945E+00_JPRB, 0.28073E+00_JPRB, 0.28331E+00_JPRB, 0.28463E+00_JPRB, 0.28646E+00_JPRB ,&
&  0.28904E+00_JPRB, 0.28763E+00_JPRB, 0.28817E+00_JPRB, 0.28737E+00_JPRB, 0.28822E+00_JPRB, 0.28988E+00_JPRB, 0.29091E+00_JPRB ,&
&  0.29286E+00_JPRB, 0.29465E+00_JPRB, 0.29512E+00_JPRB, 0.29432E+00_JPRB, 0.28955E+00_JPRB, 0.28775E+00_JPRB, 0.28313E+00_JPRB ,&
&  0.27729E+00_JPRB, 0.27182E+00_JPRB, 0.26484E+00_JPRB, 0.25980E+00_JPRB, 0.25344E+00_JPRB, 0.24974E+00_JPRB, 0.24687E+00_JPRB ,&
&  0.24420E+00_JPRB, 0.24329E+00_JPRB, 0.24192E+00_JPRB, 0.24031E+00_JPRB, 0.23989E+00_JPRB, 0.23818E+00_JPRB, 0.23811E+00_JPRB ,&
&  0.23864E+00_JPRB, 0.23910E+00_JPRB, 0.24046E+00_JPRB, 0.24095E+00_JPRB, 0.24182E+00_JPRB, 0.24253E+00_JPRB, 0.24286E+00_JPRB ,&
&  0.24402E+00_JPRB, 0.24581E+00_JPRB, 0.24566E+00_JPRB, 0.24622E+00_JPRB, 0.24530E+00_JPRB, 0.24301E+00_JPRB, 0.24062E+00_JPRB ,&
&  0.23791E+00_JPRB, 0.23584E+00_JPRB, 0.23524E+00_JPRB, 0.23430E+00_JPRB, 0.23441E+00_JPRB, 0.23483E+00_JPRB, 0.23488E+00_JPRB ,&
&  0.23558E+00_JPRB, 0.23622E+00_JPRB, 0.23725E+00_JPRB, 0.23973E+00_JPRB, 0.24133E+00_JPRB, 0.24348E+00_JPRB, 0.24579E+00_JPRB ,&
&  0.24758E+00_JPRB, 0.25098E+00_JPRB, 0.25395E+00_JPRB, 0.25804E+00_JPRB, 0.26771E+00_JPRB, 0.27533E+00_JPRB, 0.28903E+00_JPRB ,&
&  0.29246E+00_JPRB, 0.29896E+00_JPRB, 0.29448E+00_JPRB, 0.29770E+00_JPRB, 0.29920E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.29295E+00_JPRB, 0.29393E+00_JPRB, 0.29354E+00_JPRB, 0.29458E+00_JPRB, 0.29525E+00_JPRB, 0.29641E+00_JPRB, 0.29605E+00_JPRB ,&
&  0.29586E+00_JPRB, 0.29241E+00_JPRB, 0.29060E+00_JPRB, 0.28704E+00_JPRB, 0.28390E+00_JPRB, 0.28036E+00_JPRB, 0.27754E+00_JPRB ,&
&  0.27579E+00_JPRB, 0.27440E+00_JPRB, 0.27314E+00_JPRB, 0.27135E+00_JPRB, 0.26899E+00_JPRB, 0.26657E+00_JPRB, 0.26439E+00_JPRB ,&
&  0.26235E+00_JPRB, 0.26261E+00_JPRB, 0.26425E+00_JPRB, 0.26674E+00_JPRB, 0.27139E+00_JPRB, 0.27434E+00_JPRB, 0.27818E+00_JPRB ,&
&  0.28350E+00_JPRB, 0.27833E+00_JPRB, 0.27863E+00_JPRB, 0.27940E+00_JPRB, 0.27948E+00_JPRB, 0.27967E+00_JPRB, 0.27808E+00_JPRB ,&
&  0.28423E+00_JPRB, 0.28464E+00_JPRB, 0.28904E+00_JPRB, 0.28737E+00_JPRB, 0.28175E+00_JPRB, 0.27833E+00_JPRB, 0.27296E+00_JPRB ,&
&  0.26971E+00_JPRB, 0.26458E+00_JPRB, 0.26113E+00_JPRB, 0.25770E+00_JPRB, 0.25311E+00_JPRB, 0.25126E+00_JPRB, 0.24806E+00_JPRB ,&
&  0.24622E+00_JPRB, 0.24589E+00_JPRB, 0.24467E+00_JPRB, 0.24474E+00_JPRB, 0.24363E+00_JPRB, 0.24206E+00_JPRB, 0.24112E+00_JPRB ,&
&  0.24032E+00_JPRB, 0.23943E+00_JPRB, 0.23917E+00_JPRB, 0.23882E+00_JPRB, 0.23895E+00_JPRB, 0.23885E+00_JPRB, 0.23873E+00_JPRB ,&
&  0.23884E+00_JPRB, 0.23896E+00_JPRB, 0.23835E+00_JPRB, 0.23825E+00_JPRB, 0.23710E+00_JPRB, 0.23608E+00_JPRB, 0.23443E+00_JPRB ,&
&  0.23336E+00_JPRB, 0.23264E+00_JPRB, 0.23145E+00_JPRB, 0.23068E+00_JPRB, 0.23035E+00_JPRB, 0.23086E+00_JPRB, 0.23156E+00_JPRB ,&
&  0.23240E+00_JPRB, 0.23413E+00_JPRB, 0.23551E+00_JPRB, 0.23740E+00_JPRB, 0.23946E+00_JPRB, 0.24100E+00_JPRB, 0.24331E+00_JPRB ,&
&  0.24556E+00_JPRB, 0.24790E+00_JPRB, 0.25205E+00_JPRB, 0.25650E+00_JPRB, 0.26232E+00_JPRB, 0.26774E+00_JPRB, 0.26940E+00_JPRB ,&
&  0.27097E+00_JPRB, 0.27604E+00_JPRB, 0.27887E+00_JPRB, 0.29009E+00_JPRB, 0.29192E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.28056E+00_JPRB, 0.28257E+00_JPRB, 0.28453E+00_JPRB, 0.28640E+00_JPRB, 0.28881E+00_JPRB, 0.28986E+00_JPRB, 0.29002E+00_JPRB ,&
&  0.28754E+00_JPRB, 0.28728E+00_JPRB, 0.28580E+00_JPRB, 0.28369E+00_JPRB, 0.28198E+00_JPRB, 0.27778E+00_JPRB, 0.27443E+00_JPRB ,&
&  0.27120E+00_JPRB, 0.26812E+00_JPRB, 0.26557E+00_JPRB, 0.26197E+00_JPRB, 0.25860E+00_JPRB, 0.25515E+00_JPRB, 0.25464E+00_JPRB ,&
&  0.25463E+00_JPRB, 0.25561E+00_JPRB, 0.25802E+00_JPRB, 0.25963E+00_JPRB, 0.26202E+00_JPRB, 0.26449E+00_JPRB, 0.26816E+00_JPRB ,&
&  0.27134E+00_JPRB, 0.27357E+00_JPRB, 0.27369E+00_JPRB, 0.27216E+00_JPRB, 0.27133E+00_JPRB, 0.27141E+00_JPRB, 0.27138E+00_JPRB ,&
&  0.27102E+00_JPRB, 0.27126E+00_JPRB, 0.27108E+00_JPRB, 0.27104E+00_JPRB, 0.26978E+00_JPRB, 0.26706E+00_JPRB, 0.26415E+00_JPRB ,&
&  0.26138E+00_JPRB, 0.25890E+00_JPRB, 0.25638E+00_JPRB, 0.25413E+00_JPRB, 0.25283E+00_JPRB, 0.25145E+00_JPRB, 0.24959E+00_JPRB ,&
&  0.24854E+00_JPRB, 0.24742E+00_JPRB, 0.24621E+00_JPRB, 0.24534E+00_JPRB, 0.24443E+00_JPRB, 0.24381E+00_JPRB, 0.24308E+00_JPRB ,&
&  0.24260E+00_JPRB, 0.24173E+00_JPRB, 0.24099E+00_JPRB, 0.24016E+00_JPRB, 0.23982E+00_JPRB, 0.23879E+00_JPRB, 0.23780E+00_JPRB ,&
&  0.23653E+00_JPRB, 0.23574E+00_JPRB, 0.23425E+00_JPRB, 0.23336E+00_JPRB, 0.23229E+00_JPRB, 0.23128E+00_JPRB, 0.23051E+00_JPRB ,&
&  0.23038E+00_JPRB, 0.23012E+00_JPRB, 0.22919E+00_JPRB, 0.22880E+00_JPRB, 0.22861E+00_JPRB, 0.22904E+00_JPRB, 0.23073E+00_JPRB ,&
&  0.23180E+00_JPRB, 0.23297E+00_JPRB, 0.23405E+00_JPRB, 0.23484E+00_JPRB, 0.23631E+00_JPRB, 0.23768E+00_JPRB, 0.23909E+00_JPRB ,&
&  0.24080E+00_JPRB, 0.24220E+00_JPRB, 0.24385E+00_JPRB, 0.24573E+00_JPRB, 0.25084E+00_JPRB, 0.25375E+00_JPRB, 0.25650E+00_JPRB ,&
&  0.25787E+00_JPRB, 0.25429E+00_JPRB, 0.27023E+00_JPRB, 0.27315E+00_JPRB, 0.27548E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.25474E+00_JPRB, 0.25782E+00_JPRB, 0.25930E+00_JPRB, 0.25998E+00_JPRB, 0.24937E+00_JPRB, 0.26126E+00_JPRB, 0.25768E+00_JPRB ,&
&  0.25734E+00_JPRB, 0.25785E+00_JPRB, 0.26081E+00_JPRB, 0.26165E+00_JPRB, 0.26152E+00_JPRB, 0.26105E+00_JPRB, 0.26017E+00_JPRB ,&
&  0.25927E+00_JPRB, 0.25794E+00_JPRB, 0.25727E+00_JPRB, 0.25648E+00_JPRB, 0.25586E+00_JPRB, 0.25518E+00_JPRB, 0.25472E+00_JPRB ,&
&  0.25492E+00_JPRB, 0.25608E+00_JPRB, 0.25807E+00_JPRB, 0.25715E+00_JPRB, 0.25695E+00_JPRB, 0.25723E+00_JPRB, 0.25782E+00_JPRB ,&
&  0.25796E+00_JPRB, 0.25790E+00_JPRB, 0.25741E+00_JPRB, 0.25595E+00_JPRB, 0.25632E+00_JPRB, 0.25679E+00_JPRB, 0.25732E+00_JPRB ,&
&  0.25760E+00_JPRB, 0.25757E+00_JPRB, 0.25673E+00_JPRB, 0.25594E+00_JPRB, 0.25512E+00_JPRB, 0.25405E+00_JPRB, 0.25266E+00_JPRB ,&
&  0.25153E+00_JPRB, 0.25068E+00_JPRB, 0.24955E+00_JPRB, 0.24841E+00_JPRB, 0.24743E+00_JPRB, 0.24646E+00_JPRB, 0.24549E+00_JPRB ,&
&  0.24488E+00_JPRB, 0.24444E+00_JPRB, 0.24394E+00_JPRB, 0.24321E+00_JPRB, 0.24243E+00_JPRB, 0.24168E+00_JPRB, 0.24102E+00_JPRB ,&
&  0.24061E+00_JPRB, 0.23986E+00_JPRB, 0.23911E+00_JPRB, 0.23828E+00_JPRB, 0.23766E+00_JPRB, 0.23660E+00_JPRB, 0.23567E+00_JPRB ,&
&  0.23476E+00_JPRB, 0.23392E+00_JPRB, 0.23325E+00_JPRB, 0.23274E+00_JPRB, 0.23226E+00_JPRB, 0.23189E+00_JPRB, 0.23157E+00_JPRB ,&
&  0.23130E+00_JPRB, 0.23104E+00_JPRB, 0.23088E+00_JPRB, 0.23064E+00_JPRB, 0.23036E+00_JPRB, 0.23021E+00_JPRB, 0.23048E+00_JPRB ,&
&  0.23070E+00_JPRB, 0.23082E+00_JPRB, 0.23084E+00_JPRB, 0.23032E+00_JPRB, 0.23078E+00_JPRB, 0.23119E+00_JPRB, 0.23168E+00_JPRB ,&
&  0.23210E+00_JPRB, 0.23308E+00_JPRB, 0.23428E+00_JPRB, 0.23577E+00_JPRB, 0.23875E+00_JPRB, 0.24126E+00_JPRB, 0.24399E+00_JPRB ,&
&  0.24703E+00_JPRB, 0.25154E+00_JPRB, 0.25311E+00_JPRB, 0.25401E+00_JPRB, 0.25450E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.24285E+00_JPRB, 0.24262E+00_JPRB, 0.24219E+00_JPRB, 0.24092E+00_JPRB, 0.23745E+00_JPRB, 0.23676E+00_JPRB, 0.23666E+00_JPRB ,&
&  0.23684E+00_JPRB, 0.23695E+00_JPRB, 0.23819E+00_JPRB, 0.23952E+00_JPRB, 0.24086E+00_JPRB, 0.24190E+00_JPRB, 0.24282E+00_JPRB ,&
&  0.24333E+00_JPRB, 0.24384E+00_JPRB, 0.24224E+00_JPRB, 0.24145E+00_JPRB, 0.24118E+00_JPRB, 0.24114E+00_JPRB, 0.24112E+00_JPRB ,&
&  0.24103E+00_JPRB, 0.24100E+00_JPRB, 0.24085E+00_JPRB, 0.24116E+00_JPRB, 0.24162E+00_JPRB, 0.24199E+00_JPRB, 0.24242E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24307E+00_JPRB, 0.24328E+00_JPRB, 0.24346E+00_JPRB, 0.24397E+00_JPRB, 0.24429E+00_JPRB, 0.24429E+00_JPRB ,&
&  0.24417E+00_JPRB, 0.24381E+00_JPRB, 0.24340E+00_JPRB, 0.24295E+00_JPRB, 0.24265E+00_JPRB, 0.24234E+00_JPRB, 0.24202E+00_JPRB ,&
&  0.24169E+00_JPRB, 0.24135E+00_JPRB, 0.24098E+00_JPRB, 0.24055E+00_JPRB, 0.24013E+00_JPRB, 0.23983E+00_JPRB, 0.23950E+00_JPRB ,&
&  0.23922E+00_JPRB, 0.23894E+00_JPRB, 0.23864E+00_JPRB, 0.23823E+00_JPRB, 0.23780E+00_JPRB, 0.23736E+00_JPRB, 0.23688E+00_JPRB ,&
&  0.23625E+00_JPRB, 0.23580E+00_JPRB, 0.23536E+00_JPRB, 0.23497E+00_JPRB, 0.23463E+00_JPRB, 0.23426E+00_JPRB, 0.23394E+00_JPRB ,&
&  0.23362E+00_JPRB, 0.23330E+00_JPRB, 0.23292E+00_JPRB, 0.23256E+00_JPRB, 0.23223E+00_JPRB, 0.23193E+00_JPRB, 0.23167E+00_JPRB ,&
&  0.23142E+00_JPRB, 0.23116E+00_JPRB, 0.23092E+00_JPRB, 0.23088E+00_JPRB, 0.23079E+00_JPRB, 0.23077E+00_JPRB, 0.23075E+00_JPRB ,&
&  0.23075E+00_JPRB, 0.23073E+00_JPRB, 0.23069E+00_JPRB, 0.23049E+00_JPRB, 0.23066E+00_JPRB, 0.23091E+00_JPRB, 0.23131E+00_JPRB ,&
&  0.23179E+00_JPRB, 0.23243E+00_JPRB, 0.23311E+00_JPRB, 0.23384E+00_JPRB, 0.23471E+00_JPRB, 0.23543E+00_JPRB, 0.23624E+00_JPRB ,&
&  0.23714E+00_JPRB, 0.23805E+00_JPRB, 0.23870E+00_JPRB, 0.23947E+00_JPRB, 0.24066E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB, 0.23494E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB, 0.89717E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.90872E+00_JPRB, 0.90730E+00_JPRB, 0.90608E+00_JPRB, 0.90539E+00_JPRB, 0.90501E+00_JPRB, 0.90468E+00_JPRB, 0.90320E+00_JPRB ,&
&  0.90140E+00_JPRB, 0.89920E+00_JPRB, 0.89692E+00_JPRB, 0.89413E+00_JPRB, 0.89067E+00_JPRB, 0.88658E+00_JPRB, 0.88288E+00_JPRB ,&
&  0.87941E+00_JPRB, 0.87656E+00_JPRB, 0.87355E+00_JPRB, 0.87146E+00_JPRB, 0.86911E+00_JPRB, 0.86671E+00_JPRB, 0.86420E+00_JPRB ,&
&  0.86221E+00_JPRB, 0.86035E+00_JPRB, 0.85858E+00_JPRB, 0.85708E+00_JPRB, 0.85650E+00_JPRB, 0.85569E+00_JPRB, 0.85481E+00_JPRB ,&
&  0.85406E+00_JPRB, 0.85367E+00_JPRB, 0.85340E+00_JPRB, 0.85309E+00_JPRB, 0.85289E+00_JPRB, 0.85318E+00_JPRB, 0.85347E+00_JPRB ,&
&  0.85399E+00_JPRB, 0.85524E+00_JPRB, 0.85706E+00_JPRB, 0.85874E+00_JPRB, 0.86061E+00_JPRB, 0.86319E+00_JPRB, 0.86638E+00_JPRB ,&
&  0.86923E+00_JPRB, 0.87198E+00_JPRB, 0.87510E+00_JPRB, 0.87870E+00_JPRB, 0.88222E+00_JPRB, 0.88606E+00_JPRB, 0.88973E+00_JPRB ,&
&  0.89339E+00_JPRB, 0.89635E+00_JPRB, 0.89921E+00_JPRB, 0.90062E+00_JPRB, 0.90263E+00_JPRB, 0.90432E+00_JPRB, 0.90625E+00_JPRB ,&
&  0.90675E+00_JPRB, 0.90705E+00_JPRB, 0.90749E+00_JPRB, 0.90826E+00_JPRB, 0.90871E+00_JPRB, 0.90932E+00_JPRB, 0.91006E+00_JPRB ,&
&  0.91114E+00_JPRB, 0.91170E+00_JPRB, 0.91260E+00_JPRB, 0.91322E+00_JPRB, 0.91403E+00_JPRB, 0.91418E+00_JPRB, 0.91474E+00_JPRB ,&
&  0.91534E+00_JPRB, 0.91630E+00_JPRB, 0.91640E+00_JPRB, 0.91667E+00_JPRB, 0.91679E+00_JPRB, 0.91709E+00_JPRB, 0.91734E+00_JPRB ,&
&  0.91772E+00_JPRB, 0.91796E+00_JPRB, 0.91815E+00_JPRB, 0.91771E+00_JPRB, 0.91686E+00_JPRB, 0.91607E+00_JPRB, 0.91544E+00_JPRB ,&
&  0.91505E+00_JPRB, 0.91461E+00_JPRB, 0.91456E+00_JPRB, 0.91481E+00_JPRB, 0.91547E+00_JPRB, 0.91534E+00_JPRB, 0.91519E+00_JPRB ,&
&  0.91486E+00_JPRB, 0.91443E+00_JPRB, 0.91323E+00_JPRB, 0.91182E+00_JPRB, 0.91026E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.91014E+00_JPRB, 0.90774E+00_JPRB, 0.90613E+00_JPRB, 0.90551E+00_JPRB, 0.90535E+00_JPRB, 0.90497E+00_JPRB, 0.90434E+00_JPRB ,&
&  0.90210E+00_JPRB, 0.89719E+00_JPRB, 0.89156E+00_JPRB, 0.88528E+00_JPRB, 0.87759E+00_JPRB, 0.87060E+00_JPRB, 0.86600E+00_JPRB ,&
&  0.86123E+00_JPRB, 0.85646E+00_JPRB, 0.85184E+00_JPRB, 0.84806E+00_JPRB, 0.84377E+00_JPRB, 0.83919E+00_JPRB, 0.83485E+00_JPRB ,&
&  0.83139E+00_JPRB, 0.82743E+00_JPRB, 0.82296E+00_JPRB, 0.81715E+00_JPRB, 0.81344E+00_JPRB, 0.80947E+00_JPRB, 0.80594E+00_JPRB ,&
&  0.80121E+00_JPRB, 0.79917E+00_JPRB, 0.79831E+00_JPRB, 0.79877E+00_JPRB, 0.80352E+00_JPRB, 0.80686E+00_JPRB, 0.80754E+00_JPRB ,&
&  0.80655E+00_JPRB, 0.80161E+00_JPRB, 0.80211E+00_JPRB, 0.80340E+00_JPRB, 0.80616E+00_JPRB, 0.81210E+00_JPRB, 0.81957E+00_JPRB ,&
&  0.83026E+00_JPRB, 0.84320E+00_JPRB, 0.85674E+00_JPRB, 0.86713E+00_JPRB, 0.87724E+00_JPRB, 0.88645E+00_JPRB, 0.89328E+00_JPRB ,&
&  0.90260E+00_JPRB, 0.91059E+00_JPRB, 0.91868E+00_JPRB, 0.92579E+00_JPRB, 0.93390E+00_JPRB, 0.94029E+00_JPRB, 0.94493E+00_JPRB ,&
&  0.94604E+00_JPRB, 0.94561E+00_JPRB, 0.94505E+00_JPRB, 0.94472E+00_JPRB, 0.94278E+00_JPRB, 0.94184E+00_JPRB, 0.94197E+00_JPRB ,&
&  0.94292E+00_JPRB, 0.94312E+00_JPRB, 0.94487E+00_JPRB, 0.94678E+00_JPRB, 0.94904E+00_JPRB, 0.94880E+00_JPRB, 0.94926E+00_JPRB ,&
&  0.94933E+00_JPRB, 0.94972E+00_JPRB, 0.94837E+00_JPRB, 0.94551E+00_JPRB, 0.94322E+00_JPRB, 0.94232E+00_JPRB, 0.94050E+00_JPRB ,&
&  0.93863E+00_JPRB, 0.93719E+00_JPRB, 0.93664E+00_JPRB, 0.93503E+00_JPRB, 0.93258E+00_JPRB, 0.93039E+00_JPRB, 0.92937E+00_JPRB ,&
&  0.92807E+00_JPRB, 0.92627E+00_JPRB, 0.92566E+00_JPRB, 0.92533E+00_JPRB, 0.92717E+00_JPRB, 0.92694E+00_JPRB, 0.92636E+00_JPRB ,&
&  0.92472E+00_JPRB, 0.92356E+00_JPRB, 0.92103E+00_JPRB, 0.91747E+00_JPRB, 0.91325E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.92387E+00_JPRB, 0.92312E+00_JPRB, 0.92454E+00_JPRB, 0.92329E+00_JPRB, 0.92076E+00_JPRB, 0.91622E+00_JPRB, 0.91328E+00_JPRB ,&
&  0.90786E+00_JPRB, 0.89921E+00_JPRB, 0.88646E+00_JPRB, 0.87561E+00_JPRB, 0.86488E+00_JPRB, 0.85789E+00_JPRB, 0.85134E+00_JPRB ,&
&  0.84480E+00_JPRB, 0.83936E+00_JPRB, 0.83365E+00_JPRB, 0.82639E+00_JPRB, 0.81727E+00_JPRB, 0.80664E+00_JPRB, 0.79242E+00_JPRB ,&
&  0.78239E+00_JPRB, 0.77283E+00_JPRB, 0.76494E+00_JPRB, 0.75998E+00_JPRB, 0.75968E+00_JPRB, 0.73610E+00_JPRB, 0.73312E+00_JPRB ,&
&  0.74769E+00_JPRB, 0.73358E+00_JPRB, 0.71928E+00_JPRB, 0.69597E+00_JPRB, 0.68662E+00_JPRB, 0.69892E+00_JPRB, 0.70877E+00_JPRB ,&
&  0.70215E+00_JPRB, 0.74070E+00_JPRB, 0.74410E+00_JPRB, 0.74529E+00_JPRB, 0.74205E+00_JPRB, 0.73440E+00_JPRB, 0.74549E+00_JPRB ,&
&  0.77316E+00_JPRB, 0.78500E+00_JPRB, 0.81030E+00_JPRB, 0.83834E+00_JPRB, 0.85803E+00_JPRB, 0.87802E+00_JPRB, 0.89822E+00_JPRB ,&
&  0.91805E+00_JPRB, 0.93308E+00_JPRB, 0.94594E+00_JPRB, 0.95740E+00_JPRB, 0.97422E+00_JPRB, 0.99067E+00_JPRB, 0.10050E+01_JPRB ,&
&  0.10147E+01_JPRB, 0.10207E+01_JPRB, 0.10277E+01_JPRB, 0.10347E+01_JPRB, 0.10447E+01_JPRB, 0.10535E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.10753E+01_JPRB, 0.10792E+01_JPRB, 0.10809E+01_JPRB, 0.10814E+01_JPRB, 0.10780E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.10640E+01_JPRB, 0.10543E+01_JPRB, 0.10413E+01_JPRB, 0.10316E+01_JPRB, 0.10239E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.99842E+00_JPRB, 0.99080E+00_JPRB, 0.98127E+00_JPRB, 0.97230E+00_JPRB, 0.96318E+00_JPRB, 0.95705E+00_JPRB ,&
&  0.95241E+00_JPRB, 0.94870E+00_JPRB, 0.94700E+00_JPRB, 0.94481E+00_JPRB, 0.94466E+00_JPRB, 0.94346E+00_JPRB, 0.94426E+00_JPRB ,&
&  0.94340E+00_JPRB, 0.93895E+00_JPRB, 0.93249E+00_JPRB, 0.92778E+00_JPRB, 0.92417E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.92808E+00_JPRB, 0.93056E+00_JPRB, 0.93725E+00_JPRB, 0.93546E+00_JPRB, 0.93168E+00_JPRB, 0.92167E+00_JPRB, 0.91658E+00_JPRB ,&
&  0.90687E+00_JPRB, 0.89724E+00_JPRB, 0.87755E+00_JPRB, 0.86472E+00_JPRB, 0.85293E+00_JPRB, 0.84548E+00_JPRB, 0.83951E+00_JPRB ,&
&  0.83472E+00_JPRB, 0.82845E+00_JPRB, 0.82089E+00_JPRB, 0.80957E+00_JPRB, 0.79532E+00_JPRB, 0.77910E+00_JPRB, 0.75583E+00_JPRB ,&
&  0.73665E+00_JPRB, 0.72324E+00_JPRB, 0.70610E+00_JPRB, 0.71437E+00_JPRB, 0.70582E+00_JPRB, 0.69913E+00_JPRB, 0.69440E+00_JPRB ,&
&  0.68104E+00_JPRB, 0.66552E+00_JPRB, 0.64323E+00_JPRB, 0.63292E+00_JPRB, 0.62653E+00_JPRB, 0.61813E+00_JPRB, 0.63536E+00_JPRB ,&
&  0.64821E+00_JPRB, 0.68130E+00_JPRB, 0.69477E+00_JPRB, 0.69754E+00_JPRB, 0.70192E+00_JPRB, 0.69965E+00_JPRB, 0.70833E+00_JPRB ,&
&  0.71944E+00_JPRB, 0.73345E+00_JPRB, 0.78089E+00_JPRB, 0.81215E+00_JPRB, 0.83476E+00_JPRB, 0.85510E+00_JPRB, 0.87000E+00_JPRB ,&
&  0.88911E+00_JPRB, 0.90151E+00_JPRB, 0.91485E+00_JPRB, 0.93137E+00_JPRB, 0.95587E+00_JPRB, 0.98440E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.10468E+01_JPRB, 0.10651E+01_JPRB, 0.10864E+01_JPRB, 0.11151E+01_JPRB, 0.11473E+01_JPRB, 0.11812E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.12515E+01_JPRB, 0.12806E+01_JPRB, 0.13065E+01_JPRB, 0.13207E+01_JPRB, 0.13242E+01_JPRB, 0.13225E+01_JPRB ,&
&  0.13048E+01_JPRB, 0.12838E+01_JPRB, 0.12548E+01_JPRB, 0.12098E+01_JPRB, 0.11756E+01_JPRB, 0.11573E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.10691E+01_JPRB, 0.10456E+01_JPRB, 0.10203E+01_JPRB, 0.99766E+00_JPRB, 0.97650E+00_JPRB, 0.96226E+00_JPRB ,&
&  0.95088E+00_JPRB, 0.94403E+00_JPRB, 0.93989E+00_JPRB, 0.93751E+00_JPRB, 0.93800E+00_JPRB, 0.93742E+00_JPRB, 0.94077E+00_JPRB ,&
&  0.94101E+00_JPRB, 0.93785E+00_JPRB, 0.93081E+00_JPRB, 0.92742E+00_JPRB, 0.92539E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.93453E+00_JPRB, 0.93535E+00_JPRB, 0.93322E+00_JPRB, 0.92219E+00_JPRB, 0.91538E+00_JPRB, 0.90763E+00_JPRB, 0.90559E+00_JPRB ,&
&  0.89634E+00_JPRB, 0.88767E+00_JPRB, 0.86849E+00_JPRB, 0.85651E+00_JPRB, 0.84285E+00_JPRB, 0.83463E+00_JPRB, 0.82680E+00_JPRB ,&
&  0.82089E+00_JPRB, 0.81364E+00_JPRB, 0.80839E+00_JPRB, 0.79706E+00_JPRB, 0.78608E+00_JPRB, 0.76909E+00_JPRB, 0.75031E+00_JPRB ,&
&  0.73057E+00_JPRB, 0.71047E+00_JPRB, 0.69520E+00_JPRB, 0.69866E+00_JPRB, 0.68974E+00_JPRB, 0.68291E+00_JPRB, 0.67440E+00_JPRB ,&
&  0.66687E+00_JPRB, 0.62925E+00_JPRB, 0.64378E+00_JPRB, 0.62923E+00_JPRB, 0.61346E+00_JPRB, 0.59256E+00_JPRB, 0.57503E+00_JPRB ,&
&  0.57969E+00_JPRB, 0.58319E+00_JPRB, 0.59748E+00_JPRB, 0.61895E+00_JPRB, 0.62852E+00_JPRB, 0.64595E+00_JPRB, 0.65733E+00_JPRB ,&
&  0.65114E+00_JPRB, 0.73371E+00_JPRB, 0.77682E+00_JPRB, 0.80162E+00_JPRB, 0.82620E+00_JPRB, 0.84562E+00_JPRB, 0.85771E+00_JPRB ,&
&  0.88103E+00_JPRB, 0.90697E+00_JPRB, 0.93664E+00_JPRB, 0.96693E+00_JPRB, 0.10099E+01_JPRB, 0.10401E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.11041E+01_JPRB, 0.11239E+01_JPRB, 0.11543E+01_JPRB, 0.11930E+01_JPRB, 0.12365E+01_JPRB, 0.12847E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13923E+01_JPRB, 0.14483E+01_JPRB, 0.14971E+01_JPRB, 0.15356E+01_JPRB, 0.15601E+01_JPRB, 0.15862E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.15458E+01_JPRB, 0.15050E+01_JPRB, 0.14472E+01_JPRB, 0.13753E+01_JPRB, 0.12955E+01_JPRB, 0.12154E+01_JPRB ,&
&  0.11731E+01_JPRB, 0.11282E+01_JPRB, 0.10872E+01_JPRB, 0.10483E+01_JPRB, 0.10139E+01_JPRB, 0.98555E+00_JPRB, 0.96928E+00_JPRB ,&
&  0.95526E+00_JPRB, 0.94623E+00_JPRB, 0.94041E+00_JPRB, 0.93543E+00_JPRB, 0.93278E+00_JPRB, 0.92730E+00_JPRB, 0.92574E+00_JPRB ,&
&  0.92558E+00_JPRB, 0.92827E+00_JPRB, 0.92737E+00_JPRB, 0.92859E+00_JPRB, 0.93080E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.94551E+00_JPRB, 0.94798E+00_JPRB, 0.94222E+00_JPRB, 0.93458E+00_JPRB, 0.92934E+00_JPRB, 0.91659E+00_JPRB, 0.90968E+00_JPRB ,&
&  0.90448E+00_JPRB, 0.89930E+00_JPRB, 0.88713E+00_JPRB, 0.87424E+00_JPRB, 0.85692E+00_JPRB, 0.84371E+00_JPRB, 0.82915E+00_JPRB ,&
&  0.81681E+00_JPRB, 0.80254E+00_JPRB, 0.78648E+00_JPRB, 0.77159E+00_JPRB, 0.76027E+00_JPRB, 0.74594E+00_JPRB, 0.73546E+00_JPRB ,&
&  0.72365E+00_JPRB, 0.70850E+00_JPRB, 0.69815E+00_JPRB, 0.69849E+00_JPRB, 0.68832E+00_JPRB, 0.67794E+00_JPRB, 0.67878E+00_JPRB ,&
&  0.68415E+00_JPRB, 0.67791E+00_JPRB, 0.66900E+00_JPRB, 0.65260E+00_JPRB, 0.62084E+00_JPRB, 0.55944E+00_JPRB, 0.57415E+00_JPRB ,&
&  0.56865E+00_JPRB, 0.55792E+00_JPRB, 0.56248E+00_JPRB, 0.57358E+00_JPRB, 0.58545E+00_JPRB, 0.61745E+00_JPRB, 0.62015E+00_JPRB ,&
&  0.60757E+00_JPRB, 0.72455E+00_JPRB, 0.75048E+00_JPRB, 0.78098E+00_JPRB, 0.80988E+00_JPRB, 0.83165E+00_JPRB, 0.85287E+00_JPRB ,&
&  0.87268E+00_JPRB, 0.90225E+00_JPRB, 0.93051E+00_JPRB, 0.97269E+00_JPRB, 0.10150E+01_JPRB, 0.10666E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.11576E+01_JPRB, 0.11823E+01_JPRB, 0.12021E+01_JPRB, 0.12260E+01_JPRB, 0.12688E+01_JPRB, 0.13263E+01_JPRB ,&
&  0.14063E+01_JPRB, 0.14564E+01_JPRB, 0.15640E+01_JPRB, 0.16459E+01_JPRB, 0.16891E+01_JPRB, 0.17486E+01_JPRB, 0.18147E+01_JPRB ,&
&  0.18007E+01_JPRB, 0.17943E+01_JPRB, 0.17643E+01_JPRB, 0.16404E+01_JPRB, 0.15066E+01_JPRB, 0.14153E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.12018E+01_JPRB, 0.11344E+01_JPRB, 0.10835E+01_JPRB, 0.10439E+01_JPRB, 0.10157E+01_JPRB, 0.10009E+01_JPRB ,&
&  0.98405E+00_JPRB, 0.97020E+00_JPRB, 0.95655E+00_JPRB, 0.95089E+00_JPRB, 0.93771E+00_JPRB, 0.92669E+00_JPRB, 0.91470E+00_JPRB ,&
&  0.91220E+00_JPRB, 0.91267E+00_JPRB, 0.92056E+00_JPRB, 0.92869E+00_JPRB, 0.93382E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.95884E+00_JPRB, 0.97661E+00_JPRB, 0.98251E+00_JPRB, 0.98373E+00_JPRB, 0.98421E+00_JPRB, 0.96483E+00_JPRB, 0.94707E+00_JPRB ,&
&  0.93384E+00_JPRB, 0.92216E+00_JPRB, 0.90987E+00_JPRB, 0.89688E+00_JPRB, 0.88221E+00_JPRB, 0.86999E+00_JPRB, 0.85846E+00_JPRB ,&
&  0.84799E+00_JPRB, 0.80329E+00_JPRB, 0.78126E+00_JPRB, 0.76385E+00_JPRB, 0.74686E+00_JPRB, 0.73786E+00_JPRB, 0.73346E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.71912E+00_JPRB, 0.71570E+00_JPRB, 0.70812E+00_JPRB, 0.69725E+00_JPRB, 0.68462E+00_JPRB, 0.67956E+00_JPRB ,&
&  0.68353E+00_JPRB, 0.66987E+00_JPRB, 0.65972E+00_JPRB, 0.64186E+00_JPRB, 0.60576E+00_JPRB, 0.58547E+00_JPRB, 0.58538E+00_JPRB ,&
&  0.58034E+00_JPRB, 0.55401E+00_JPRB, 0.55619E+00_JPRB, 0.56026E+00_JPRB, 0.56255E+00_JPRB, 0.57647E+00_JPRB, 0.60423E+00_JPRB ,&
&  0.63064E+00_JPRB, 0.72145E+00_JPRB, 0.75904E+00_JPRB, 0.80182E+00_JPRB, 0.83532E+00_JPRB, 0.86351E+00_JPRB, 0.88816E+00_JPRB ,&
&  0.91167E+00_JPRB, 0.94584E+00_JPRB, 0.98382E+00_JPRB, 0.10334E+01_JPRB, 0.10836E+01_JPRB, 0.11237E+01_JPRB, 0.11679E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.12123E+01_JPRB, 0.12317E+01_JPRB, 0.12341E+01_JPRB, 0.12534E+01_JPRB, 0.13096E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.13935E+01_JPRB, 0.14993E+01_JPRB, 0.16691E+01_JPRB, 0.18400E+01_JPRB, 0.19206E+01_JPRB, 0.20103E+01_JPRB, 0.20509E+01_JPRB ,&
&  0.19616E+01_JPRB, 0.19112E+01_JPRB, 0.19645E+01_JPRB, 0.18159E+01_JPRB, 0.16548E+01_JPRB, 0.15183E+01_JPRB, 0.14240E+01_JPRB ,&
&  0.13290E+01_JPRB, 0.12352E+01_JPRB, 0.11756E+01_JPRB, 0.11302E+01_JPRB, 0.10849E+01_JPRB, 0.10457E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.10271E+01_JPRB, 0.10145E+01_JPRB, 0.99991E+00_JPRB, 0.97120E+00_JPRB, 0.94113E+00_JPRB, 0.91538E+00_JPRB ,&
&  0.90854E+00_JPRB, 0.90685E+00_JPRB, 0.91637E+00_JPRB, 0.92507E+00_JPRB, 0.93909E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.96579E+00_JPRB, 0.98058E+00_JPRB, 0.98585E+00_JPRB, 0.99269E+00_JPRB, 0.10046E+01_JPRB, 0.95552E+00_JPRB, 0.91944E+00_JPRB ,&
&  0.89835E+00_JPRB, 0.89942E+00_JPRB, 0.89840E+00_JPRB, 0.89176E+00_JPRB, 0.87997E+00_JPRB, 0.86704E+00_JPRB, 0.85648E+00_JPRB ,&
&  0.84723E+00_JPRB, 0.82540E+00_JPRB, 0.78808E+00_JPRB, 0.73747E+00_JPRB, 0.72358E+00_JPRB, 0.71909E+00_JPRB, 0.72089E+00_JPRB ,&
&  0.72001E+00_JPRB, 0.71541E+00_JPRB, 0.71751E+00_JPRB, 0.70536E+00_JPRB, 0.69695E+00_JPRB, 0.67359E+00_JPRB, 0.65596E+00_JPRB ,&
&  0.65740E+00_JPRB, 0.65257E+00_JPRB, 0.65966E+00_JPRB, 0.64907E+00_JPRB, 0.63713E+00_JPRB, 0.62782E+00_JPRB, 0.57187E+00_JPRB ,&
&  0.56260E+00_JPRB, 0.54904E+00_JPRB, 0.54480E+00_JPRB, 0.54469E+00_JPRB, 0.56586E+00_JPRB, 0.58430E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.62122E+00_JPRB, 0.72690E+00_JPRB, 0.76068E+00_JPRB, 0.79061E+00_JPRB, 0.81968E+00_JPRB, 0.85570E+00_JPRB, 0.90321E+00_JPRB ,&
&  0.94950E+00_JPRB, 0.99570E+00_JPRB, 0.10331E+01_JPRB, 0.10844E+01_JPRB, 0.11354E+01_JPRB, 0.11209E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.15460E+01_JPRB, 0.14599E+01_JPRB, 0.13824E+01_JPRB, 0.13442E+01_JPRB, 0.13486E+01_JPRB, 0.13688E+01_JPRB, 0.14074E+01_JPRB ,&
&  0.15220E+01_JPRB, 0.17157E+01_JPRB, 0.20089E+01_JPRB, 0.22214E+01_JPRB, 0.22356E+01_JPRB, 0.22746E+01_JPRB, 0.22544E+01_JPRB ,&
&  0.21074E+01_JPRB, 0.20150E+01_JPRB, 0.21130E+01_JPRB, 0.17620E+01_JPRB, 0.15163E+01_JPRB, 0.13882E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.12065E+01_JPRB, 0.11685E+01_JPRB, 0.11233E+01_JPRB, 0.10663E+01_JPRB, 0.10332E+01_JPRB, 0.10288E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.10155E+01_JPRB, 0.10100E+01_JPRB, 0.99980E+00_JPRB, 0.97903E+00_JPRB, 0.95573E+00_JPRB, 0.94743E+00_JPRB ,&
&  0.94522E+00_JPRB, 0.94040E+00_JPRB, 0.93656E+00_JPRB, 0.93721E+00_JPRB, 0.94643E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.96525E+00_JPRB, 0.97785E+00_JPRB, 0.98229E+00_JPRB, 0.97191E+00_JPRB, 0.95704E+00_JPRB, 0.88785E+00_JPRB, 0.81310E+00_JPRB ,&
&  0.75927E+00_JPRB, 0.76223E+00_JPRB, 0.78434E+00_JPRB, 0.80193E+00_JPRB, 0.80104E+00_JPRB, 0.78471E+00_JPRB, 0.77707E+00_JPRB ,&
&  0.78237E+00_JPRB, 0.78030E+00_JPRB, 0.75466E+00_JPRB, 0.73755E+00_JPRB, 0.70899E+00_JPRB, 0.70326E+00_JPRB, 0.70504E+00_JPRB ,&
&  0.70772E+00_JPRB, 0.70565E+00_JPRB, 0.70573E+00_JPRB, 0.70309E+00_JPRB, 0.70241E+00_JPRB, 0.69534E+00_JPRB, 0.68124E+00_JPRB ,&
&  0.66155E+00_JPRB, 0.65807E+00_JPRB, 0.66262E+00_JPRB, 0.67112E+00_JPRB, 0.66635E+00_JPRB, 0.65711E+00_JPRB, 0.58701E+00_JPRB ,&
&  0.56004E+00_JPRB, 0.54201E+00_JPRB, 0.54476E+00_JPRB, 0.56010E+00_JPRB, 0.59353E+00_JPRB, 0.59744E+00_JPRB, 0.67726E+00_JPRB ,&
&  0.73728E+00_JPRB, 0.77135E+00_JPRB, 0.78693E+00_JPRB, 0.80130E+00_JPRB, 0.83112E+00_JPRB, 0.89276E+00_JPRB, 0.96931E+00_JPRB ,&
&  0.10197E+01_JPRB, 0.10547E+01_JPRB, 0.10510E+01_JPRB, 0.92360E+00_JPRB, 0.92982E+00_JPRB, 0.10119E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.13232E+01_JPRB, 0.13294E+01_JPRB, 0.13901E+01_JPRB, 0.13950E+01_JPRB, 0.13995E+01_JPRB, 0.14017E+01_JPRB ,&
&  0.15634E+01_JPRB, 0.18132E+01_JPRB, 0.22037E+01_JPRB, 0.24124E+01_JPRB, 0.23628E+01_JPRB, 0.23984E+01_JPRB, 0.23182E+01_JPRB ,&
&  0.22348E+01_JPRB, 0.20108E+01_JPRB, 0.18150E+01_JPRB, 0.15733E+01_JPRB, 0.13827E+01_JPRB, 0.12754E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.11580E+01_JPRB, 0.11070E+01_JPRB, 0.11018E+01_JPRB, 0.10747E+01_JPRB, 0.10196E+01_JPRB, 0.96885E+00_JPRB, 0.93023E+00_JPRB ,&
&  0.90320E+00_JPRB, 0.89020E+00_JPRB, 0.89032E+00_JPRB, 0.87285E+00_JPRB, 0.87478E+00_JPRB, 0.88949E+00_JPRB, 0.89692E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.89531E+00_JPRB, 0.90614E+00_JPRB, 0.92204E+00_JPRB, 0.93842E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.10144E+01_JPRB, 0.98707E+00_JPRB, 0.94992E+00_JPRB, 0.94332E+00_JPRB, 0.91780E+00_JPRB, 0.89907E+00_JPRB, 0.82214E+00_JPRB ,&
&  0.72502E+00_JPRB, 0.69646E+00_JPRB, 0.72006E+00_JPRB, 0.72787E+00_JPRB, 0.71062E+00_JPRB, 0.70467E+00_JPRB, 0.71002E+00_JPRB ,&
&  0.71669E+00_JPRB, 0.74123E+00_JPRB, 0.73873E+00_JPRB, 0.72199E+00_JPRB, 0.71132E+00_JPRB, 0.70602E+00_JPRB, 0.70898E+00_JPRB ,&
&  0.70970E+00_JPRB, 0.70913E+00_JPRB, 0.70185E+00_JPRB, 0.68927E+00_JPRB, 0.68656E+00_JPRB, 0.68430E+00_JPRB, 0.68100E+00_JPRB ,&
&  0.68041E+00_JPRB, 0.67629E+00_JPRB, 0.68446E+00_JPRB, 0.68929E+00_JPRB, 0.68198E+00_JPRB, 0.65715E+00_JPRB, 0.59499E+00_JPRB ,&
&  0.56219E+00_JPRB, 0.54944E+00_JPRB, 0.57964E+00_JPRB, 0.64655E+00_JPRB, 0.69793E+00_JPRB, 0.74575E+00_JPRB, 0.78134E+00_JPRB ,&
&  0.75830E+00_JPRB, 0.76617E+00_JPRB, 0.80595E+00_JPRB, 0.92107E+00_JPRB, 0.97250E+00_JPRB, 0.10383E+01_JPRB, 0.10917E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.11740E+01_JPRB, 0.11787E+01_JPRB, 0.10260E+01_JPRB, 0.10532E+01_JPRB, 0.12761E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.13388E+01_JPRB, 0.14330E+01_JPRB, 0.15141E+01_JPRB, 0.14987E+01_JPRB, 0.14150E+01_JPRB, 0.14284E+01_JPRB, 0.15329E+01_JPRB ,&
&  0.17070E+01_JPRB, 0.19849E+01_JPRB, 0.25105E+01_JPRB, 0.28078E+01_JPRB, 0.27686E+01_JPRB, 0.27452E+01_JPRB, 0.26918E+01_JPRB ,&
&  0.26433E+01_JPRB, 0.23984E+01_JPRB, 0.20798E+01_JPRB, 0.17828E+01_JPRB, 0.15276E+01_JPRB, 0.13279E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.11627E+01_JPRB, 0.11312E+01_JPRB, 0.11309E+01_JPRB, 0.11051E+01_JPRB, 0.10393E+01_JPRB, 0.96332E+00_JPRB, 0.90069E+00_JPRB ,&
&  0.86127E+00_JPRB, 0.82524E+00_JPRB, 0.78794E+00_JPRB, 0.77591E+00_JPRB, 0.81723E+00_JPRB, 0.87383E+00_JPRB, 0.89466E+00_JPRB ,&
&  0.88045E+00_JPRB, 0.87788E+00_JPRB, 0.91981E+00_JPRB, 0.96371E+00_JPRB, 0.98042E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.10548E+01_JPRB, 0.10147E+01_JPRB, 0.97991E+00_JPRB, 0.94879E+00_JPRB, 0.92239E+00_JPRB, 0.87542E+00_JPRB, 0.79158E+00_JPRB ,&
&  0.70883E+00_JPRB, 0.67277E+00_JPRB, 0.68244E+00_JPRB, 0.66749E+00_JPRB, 0.64527E+00_JPRB, 0.63203E+00_JPRB, 0.63427E+00_JPRB ,&
&  0.64552E+00_JPRB, 0.67645E+00_JPRB, 0.69837E+00_JPRB, 0.70457E+00_JPRB, 0.70146E+00_JPRB, 0.69730E+00_JPRB, 0.70458E+00_JPRB ,&
&  0.71514E+00_JPRB, 0.72631E+00_JPRB, 0.73148E+00_JPRB, 0.72534E+00_JPRB, 0.71625E+00_JPRB, 0.71206E+00_JPRB, 0.71063E+00_JPRB ,&
&  0.71347E+00_JPRB, 0.71310E+00_JPRB, 0.70036E+00_JPRB, 0.70085E+00_JPRB, 0.68014E+00_JPRB, 0.67743E+00_JPRB, 0.64329E+00_JPRB ,&
&  0.59489E+00_JPRB, 0.60410E+00_JPRB, 0.72293E+00_JPRB, 0.77128E+00_JPRB, 0.78750E+00_JPRB, 0.82444E+00_JPRB, 0.87559E+00_JPRB ,&
&  0.92270E+00_JPRB, 0.96689E+00_JPRB, 0.10079E+01_JPRB, 0.10536E+01_JPRB, 0.10974E+01_JPRB, 0.11603E+01_JPRB, 0.12259E+01_JPRB ,&
&  0.13212E+01_JPRB, 0.11380E+01_JPRB, 0.10887E+01_JPRB, 0.11580E+01_JPRB, 0.14779E+01_JPRB, 0.15670E+01_JPRB, 0.15868E+01_JPRB ,&
&  0.16585E+01_JPRB, 0.16470E+01_JPRB, 0.16307E+01_JPRB, 0.16526E+01_JPRB, 0.17290E+01_JPRB, 0.18585E+01_JPRB, 0.20678E+01_JPRB ,&
&  0.22609E+01_JPRB, 0.26291E+01_JPRB, 0.31774E+01_JPRB, 0.33187E+01_JPRB, 0.32799E+01_JPRB, 0.32605E+01_JPRB, 0.31584E+01_JPRB ,&
&  0.30254E+01_JPRB, 0.27622E+01_JPRB, 0.23295E+01_JPRB, 0.19658E+01_JPRB, 0.16641E+01_JPRB, 0.14349E+01_JPRB, 0.12527E+01_JPRB ,&
&  0.11626E+01_JPRB, 0.11404E+01_JPRB, 0.11340E+01_JPRB, 0.11135E+01_JPRB, 0.10723E+01_JPRB, 0.10125E+01_JPRB, 0.94337E+00_JPRB ,&
&  0.86388E+00_JPRB, 0.82250E+00_JPRB, 0.77971E+00_JPRB, 0.80432E+00_JPRB, 0.86861E+00_JPRB, 0.93354E+00_JPRB, 0.95358E+00_JPRB ,&
&  0.96639E+00_JPRB, 0.95804E+00_JPRB, 0.96319E+00_JPRB, 0.99485E+00_JPRB, 0.10350E+01_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.10715E+01_JPRB, 0.10242E+01_JPRB, 0.97082E+00_JPRB, 0.91232E+00_JPRB, 0.86283E+00_JPRB, 0.82318E+00_JPRB, 0.77608E+00_JPRB ,&
&  0.81280E+00_JPRB, 0.73106E+00_JPRB, 0.80201E+00_JPRB, 0.69615E+00_JPRB, 0.66663E+00_JPRB, 0.65794E+00_JPRB, 0.63880E+00_JPRB ,&
&  0.62691E+00_JPRB, 0.62997E+00_JPRB, 0.64040E+00_JPRB, 0.65406E+00_JPRB, 0.65474E+00_JPRB, 0.65461E+00_JPRB, 0.66429E+00_JPRB ,&
&  0.68251E+00_JPRB, 0.70923E+00_JPRB, 0.73842E+00_JPRB, 0.75412E+00_JPRB, 0.75887E+00_JPRB, 0.75465E+00_JPRB, 0.75664E+00_JPRB ,&
&  0.72169E+00_JPRB, 0.71072E+00_JPRB, 0.71346E+00_JPRB, 0.71110E+00_JPRB, 0.70628E+00_JPRB, 0.69230E+00_JPRB, 0.70643E+00_JPRB ,&
&  0.69456E+00_JPRB, 0.67565E+00_JPRB, 0.78449E+00_JPRB, 0.80499E+00_JPRB, 0.83666E+00_JPRB, 0.87237E+00_JPRB, 0.91200E+00_JPRB ,&
&  0.95329E+00_JPRB, 0.10017E+01_JPRB, 0.10563E+01_JPRB, 0.10981E+01_JPRB, 0.11659E+01_JPRB, 0.12556E+01_JPRB, 0.13397E+01_JPRB ,&
&  0.14347E+01_JPRB, 0.11812E+01_JPRB, 0.12910E+01_JPRB, 0.13684E+01_JPRB, 0.16339E+01_JPRB, 0.19306E+01_JPRB, 0.19812E+01_JPRB ,&
&  0.22097E+01_JPRB, 0.21943E+01_JPRB, 0.21659E+01_JPRB, 0.21369E+01_JPRB, 0.22777E+01_JPRB, 0.24732E+01_JPRB, 0.26944E+01_JPRB ,&
&  0.29873E+01_JPRB, 0.33882E+01_JPRB, 0.39713E+01_JPRB, 0.39972E+01_JPRB, 0.39253E+01_JPRB, 0.38175E+01_JPRB, 0.36140E+01_JPRB ,&
&  0.33344E+01_JPRB, 0.29940E+01_JPRB, 0.25653E+01_JPRB, 0.20998E+01_JPRB, 0.17828E+01_JPRB, 0.15161E+01_JPRB, 0.12985E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.10708E+01_JPRB, 0.10679E+01_JPRB, 0.10491E+01_JPRB, 0.10351E+01_JPRB, 0.99330E+00_JPRB, 0.92558E+00_JPRB ,&
&  0.85316E+00_JPRB, 0.87761E+00_JPRB, 0.93475E+00_JPRB, 0.10447E+01_JPRB, 0.10885E+01_JPRB, 0.10951E+01_JPRB, 0.11447E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10831E+01_JPRB, 0.11466E+01_JPRB, 0.11252E+01_JPRB, 0.11062E+01_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.11109E+01_JPRB, 0.10344E+01_JPRB, 0.92452E+00_JPRB, 0.86197E+00_JPRB, 0.85389E+00_JPRB, 0.84545E+00_JPRB, 0.81538E+00_JPRB ,&
&  0.84280E+00_JPRB, 0.82662E+00_JPRB, 0.85134E+00_JPRB, 0.88703E+00_JPRB, 0.91902E+00_JPRB, 0.82787E+00_JPRB, 0.69996E+00_JPRB ,&
&  0.65916E+00_JPRB, 0.66572E+00_JPRB, 0.67194E+00_JPRB, 0.67055E+00_JPRB, 0.67248E+00_JPRB, 0.66899E+00_JPRB, 0.66982E+00_JPRB ,&
&  0.67628E+00_JPRB, 0.69199E+00_JPRB, 0.72969E+00_JPRB, 0.74641E+00_JPRB, 0.75356E+00_JPRB, 0.75651E+00_JPRB, 0.76649E+00_JPRB ,&
&  0.73339E+00_JPRB, 0.72009E+00_JPRB, 0.71163E+00_JPRB, 0.68745E+00_JPRB, 0.72135E+00_JPRB, 0.73479E+00_JPRB, 0.74497E+00_JPRB ,&
&  0.75758E+00_JPRB, 0.78138E+00_JPRB, 0.81414E+00_JPRB, 0.84572E+00_JPRB, 0.87687E+00_JPRB, 0.91000E+00_JPRB, 0.94446E+00_JPRB ,&
&  0.97953E+00_JPRB, 0.10226E+01_JPRB, 0.10727E+01_JPRB, 0.11359E+01_JPRB, 0.12774E+01_JPRB, 0.12901E+01_JPRB, 0.14115E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.16283E+01_JPRB, 0.17934E+01_JPRB, 0.17451E+01_JPRB, 0.18913E+01_JPRB, 0.21105E+01_JPRB, 0.21017E+01_JPRB ,&
&  0.21610E+01_JPRB, 0.21527E+01_JPRB, 0.23027E+01_JPRB, 0.25131E+01_JPRB, 0.27999E+01_JPRB, 0.30458E+01_JPRB, 0.32786E+01_JPRB ,&
&  0.36654E+01_JPRB, 0.43776E+01_JPRB, 0.46702E+01_JPRB, 0.45381E+01_JPRB, 0.43647E+01_JPRB, 0.41648E+01_JPRB, 0.38496E+01_JPRB ,&
&  0.35026E+01_JPRB, 0.31027E+01_JPRB, 0.26516E+01_JPRB, 0.21943E+01_JPRB, 0.17988E+01_JPRB, 0.15825E+01_JPRB, 0.13533E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.99974E+00_JPRB, 0.10095E+01_JPRB, 0.10269E+01_JPRB, 0.10165E+01_JPRB, 0.95604E+00_JPRB, 0.90349E+00_JPRB ,&
&  0.94065E+00_JPRB, 0.10614E+01_JPRB, 0.13825E+01_JPRB, 0.14781E+01_JPRB, 0.14873E+01_JPRB, 0.14124E+01_JPRB, 0.11610E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.11858E+01_JPRB, 0.11706E+01_JPRB, 0.11744E+01_JPRB, 0.11570E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.11634E+01_JPRB, 0.10471E+01_JPRB, 0.89644E+00_JPRB, 0.84640E+00_JPRB, 0.86177E+00_JPRB, 0.85961E+00_JPRB, 0.85435E+00_JPRB ,&
&  0.86680E+00_JPRB, 0.87302E+00_JPRB, 0.88339E+00_JPRB, 0.90007E+00_JPRB, 0.91260E+00_JPRB, 0.94565E+00_JPRB, 0.86554E+00_JPRB ,&
&  0.76701E+00_JPRB, 0.73822E+00_JPRB, 0.71226E+00_JPRB, 0.70932E+00_JPRB, 0.74251E+00_JPRB, 0.74746E+00_JPRB, 0.74762E+00_JPRB ,&
&  0.75741E+00_JPRB, 0.75689E+00_JPRB, 0.77039E+00_JPRB, 0.76531E+00_JPRB, 0.75695E+00_JPRB, 0.76146E+00_JPRB, 0.76837E+00_JPRB ,&
&  0.72780E+00_JPRB, 0.69633E+00_JPRB, 0.67665E+00_JPRB, 0.66685E+00_JPRB, 0.69805E+00_JPRB, 0.74491E+00_JPRB, 0.77293E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.81168E+00_JPRB, 0.83746E+00_JPRB, 0.87019E+00_JPRB, 0.90597E+00_JPRB, 0.94188E+00_JPRB, 0.96926E+00_JPRB ,&
&  0.99877E+00_JPRB, 0.10328E+01_JPRB, 0.10803E+01_JPRB, 0.11651E+01_JPRB, 0.12262E+01_JPRB, 0.13260E+01_JPRB, 0.15130E+01_JPRB ,&
&  0.16404E+01_JPRB, 0.17939E+01_JPRB, 0.19686E+01_JPRB, 0.22594E+01_JPRB, 0.24700E+01_JPRB, 0.24389E+01_JPRB, 0.23912E+01_JPRB ,&
&  0.24374E+01_JPRB, 0.24658E+01_JPRB, 0.28727E+01_JPRB, 0.31609E+01_JPRB, 0.33665E+01_JPRB, 0.36442E+01_JPRB, 0.38755E+01_JPRB ,&
&  0.42816E+01_JPRB, 0.47109E+01_JPRB, 0.48119E+01_JPRB, 0.46674E+01_JPRB, 0.44695E+01_JPRB, 0.42838E+01_JPRB, 0.39352E+01_JPRB ,&
&  0.35498E+01_JPRB, 0.30838E+01_JPRB, 0.25379E+01_JPRB, 0.20838E+01_JPRB, 0.16789E+01_JPRB, 0.15616E+01_JPRB, 0.14157E+01_JPRB ,&
&  0.11559E+01_JPRB, 0.10411E+01_JPRB, 0.10606E+01_JPRB, 0.11455E+01_JPRB, 0.11386E+01_JPRB, 0.10480E+01_JPRB, 0.98971E+00_JPRB ,&
&  0.10527E+01_JPRB, 0.12992E+01_JPRB, 0.15211E+01_JPRB, 0.16777E+01_JPRB, 0.16849E+01_JPRB, 0.15537E+01_JPRB, 0.11474E+01_JPRB ,&
&  0.11102E+01_JPRB, 0.12326E+01_JPRB, 0.12349E+01_JPRB, 0.12291E+01_JPRB, 0.12123E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.12386E+01_JPRB, 0.10906E+01_JPRB, 0.95106E+00_JPRB, 0.87438E+00_JPRB, 0.88215E+00_JPRB, 0.89127E+00_JPRB, 0.89503E+00_JPRB ,&
&  0.91225E+00_JPRB, 0.92705E+00_JPRB, 0.94101E+00_JPRB, 0.95338E+00_JPRB, 0.95843E+00_JPRB, 0.95665E+00_JPRB, 0.99829E+00_JPRB ,&
&  0.85384E+00_JPRB, 0.80263E+00_JPRB, 0.75977E+00_JPRB, 0.76555E+00_JPRB, 0.87632E+00_JPRB, 0.90171E+00_JPRB, 0.93596E+00_JPRB ,&
&  0.93091E+00_JPRB, 0.93802E+00_JPRB, 0.92198E+00_JPRB, 0.89033E+00_JPRB, 0.85844E+00_JPRB, 0.84300E+00_JPRB, 0.81581E+00_JPRB ,&
&  0.75515E+00_JPRB, 0.71235E+00_JPRB, 0.70000E+00_JPRB, 0.70099E+00_JPRB, 0.73504E+00_JPRB, 0.76433E+00_JPRB, 0.81682E+00_JPRB ,&
&  0.84467E+00_JPRB, 0.87174E+00_JPRB, 0.90123E+00_JPRB, 0.93274E+00_JPRB, 0.95926E+00_JPRB, 0.99110E+00_JPRB, 0.10275E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.11071E+01_JPRB, 0.11785E+01_JPRB, 0.12713E+01_JPRB, 0.12832E+01_JPRB, 0.13985E+01_JPRB, 0.16692E+01_JPRB ,&
&  0.19120E+01_JPRB, 0.20217E+01_JPRB, 0.23903E+01_JPRB, 0.33860E+01_JPRB, 0.36439E+01_JPRB, 0.35204E+01_JPRB, 0.33466E+01_JPRB ,&
&  0.32737E+01_JPRB, 0.32240E+01_JPRB, 0.33813E+01_JPRB, 0.36026E+01_JPRB, 0.38105E+01_JPRB, 0.41619E+01_JPRB, 0.44649E+01_JPRB ,&
&  0.46449E+01_JPRB, 0.47113E+01_JPRB, 0.46519E+01_JPRB, 0.45038E+01_JPRB, 0.42199E+01_JPRB, 0.40106E+01_JPRB, 0.37637E+01_JPRB ,&
&  0.34417E+01_JPRB, 0.30281E+01_JPRB, 0.23892E+01_JPRB, 0.19815E+01_JPRB, 0.15191E+01_JPRB, 0.13932E+01_JPRB, 0.16229E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.11460E+01_JPRB, 0.12182E+01_JPRB, 0.13667E+01_JPRB, 0.14039E+01_JPRB, 0.13713E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.12703E+01_JPRB, 0.17807E+01_JPRB, 0.19187E+01_JPRB, 0.18798E+01_JPRB, 0.17071E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.13880E+01_JPRB, 0.13282E+01_JPRB, 0.13218E+01_JPRB, 0.13168E+01_JPRB, 0.13012E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.12997E+01_JPRB, 0.11457E+01_JPRB, 0.10190E+01_JPRB, 0.93137E+00_JPRB, 0.92557E+00_JPRB, 0.94551E+00_JPRB, 0.96107E+00_JPRB ,&
&  0.97810E+00_JPRB, 0.99264E+00_JPRB, 0.10043E+01_JPRB, 0.10150E+01_JPRB, 0.10162E+01_JPRB, 0.98889E+00_JPRB, 0.96560E+00_JPRB ,&
&  0.95180E+00_JPRB, 0.86417E+00_JPRB, 0.83051E+00_JPRB, 0.81664E+00_JPRB, 0.91781E+00_JPRB, 0.10066E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.12729E+01_JPRB, 0.12466E+01_JPRB, 0.11707E+01_JPRB, 0.10909E+01_JPRB, 0.10243E+01_JPRB, 0.93177E+00_JPRB ,&
&  0.83799E+00_JPRB, 0.79198E+00_JPRB, 0.78218E+00_JPRB, 0.81553E+00_JPRB, 0.87604E+00_JPRB, 0.82934E+00_JPRB, 0.89595E+00_JPRB ,&
&  0.93262E+00_JPRB, 0.97178E+00_JPRB, 0.99048E+00_JPRB, 0.10150E+01_JPRB, 0.10751E+01_JPRB, 0.11495E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.13010E+01_JPRB, 0.12943E+01_JPRB, 0.13702E+01_JPRB, 0.14511E+01_JPRB, 0.15401E+01_JPRB, 0.17425E+01_JPRB ,&
&  0.20771E+01_JPRB, 0.22789E+01_JPRB, 0.27718E+01_JPRB, 0.39947E+01_JPRB, 0.43550E+01_JPRB, 0.42316E+01_JPRB, 0.42614E+01_JPRB ,&
&  0.42640E+01_JPRB, 0.41727E+01_JPRB, 0.41314E+01_JPRB, 0.40712E+01_JPRB, 0.43195E+01_JPRB, 0.47162E+01_JPRB, 0.50031E+01_JPRB ,&
&  0.48378E+01_JPRB, 0.45045E+01_JPRB, 0.43607E+01_JPRB, 0.40781E+01_JPRB, 0.38004E+01_JPRB, 0.34837E+01_JPRB, 0.35167E+01_JPRB ,&
&  0.35546E+01_JPRB, 0.25825E+01_JPRB, 0.21238E+01_JPRB, 0.19963E+01_JPRB, 0.15159E+01_JPRB, 0.13198E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.11190E+01_JPRB, 0.13075E+01_JPRB, 0.15116E+01_JPRB, 0.17479E+01_JPRB, 0.18565E+01_JPRB, 0.17690E+01_JPRB ,&
&  0.16236E+01_JPRB, 0.17160E+01_JPRB, 0.23200E+01_JPRB, 0.23750E+01_JPRB, 0.21660E+01_JPRB, 0.19064E+01_JPRB, 0.16727E+01_JPRB ,&
&  0.15372E+01_JPRB, 0.15174E+01_JPRB, 0.14851E+01_JPRB, 0.14549E+01_JPRB, 0.14180E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.14323E+01_JPRB, 0.12743E+01_JPRB, 0.11261E+01_JPRB, 0.10388E+01_JPRB, 0.10321E+01_JPRB, 0.10467E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.10400E+01_JPRB, 0.10559E+01_JPRB, 0.10603E+01_JPRB, 0.10545E+01_JPRB, 0.10220E+01_JPRB, 0.96896E+00_JPRB ,&
&  0.93840E+00_JPRB, 0.92520E+00_JPRB, 0.87323E+00_JPRB, 0.89136E+00_JPRB, 0.94671E+00_JPRB, 0.11219E+01_JPRB, 0.12885E+01_JPRB ,&
&  0.14688E+01_JPRB, 0.16024E+01_JPRB, 0.16312E+01_JPRB, 0.16074E+01_JPRB, 0.15163E+01_JPRB, 0.13732E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.10587E+01_JPRB, 0.99454E+00_JPRB, 0.10709E+01_JPRB, 0.10881E+01_JPRB, 0.10265E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.11899E+01_JPRB, 0.12064E+01_JPRB, 0.12671E+01_JPRB, 0.13325E+01_JPRB, 0.13555E+01_JPRB, 0.13475E+01_JPRB ,&
&  0.13358E+01_JPRB, 0.13506E+01_JPRB, 0.13942E+01_JPRB, 0.13867E+01_JPRB, 0.14616E+01_JPRB, 0.16898E+01_JPRB, 0.20813E+01_JPRB ,&
&  0.25440E+01_JPRB, 0.25740E+01_JPRB, 0.26937E+01_JPRB, 0.39185E+01_JPRB, 0.50290E+01_JPRB, 0.50344E+01_JPRB, 0.49362E+01_JPRB ,&
&  0.49729E+01_JPRB, 0.47068E+01_JPRB, 0.44495E+01_JPRB, 0.43797E+01_JPRB, 0.49769E+01_JPRB, 0.53746E+01_JPRB, 0.54700E+01_JPRB ,&
&  0.51131E+01_JPRB, 0.46110E+01_JPRB, 0.43623E+01_JPRB, 0.40718E+01_JPRB, 0.36404E+01_JPRB, 0.34886E+01_JPRB, 0.35761E+01_JPRB ,&
&  0.35274E+01_JPRB, 0.31428E+01_JPRB, 0.24338E+01_JPRB, 0.21932E+01_JPRB, 0.17786E+01_JPRB, 0.15856E+01_JPRB, 0.16517E+01_JPRB ,&
&  0.14432E+01_JPRB, 0.14315E+01_JPRB, 0.15279E+01_JPRB, 0.16321E+01_JPRB, 0.25536E+01_JPRB, 0.28122E+01_JPRB, 0.30381E+01_JPRB ,&
&  0.30020E+01_JPRB, 0.28349E+01_JPRB, 0.34416E+01_JPRB, 0.32581E+01_JPRB, 0.27446E+01_JPRB, 0.22381E+01_JPRB, 0.18688E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.17235E+01_JPRB, 0.16691E+01_JPRB, 0.16142E+01_JPRB, 0.15699E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.15462E+01_JPRB, 0.13787E+01_JPRB, 0.12299E+01_JPRB, 0.11327E+01_JPRB, 0.10995E+01_JPRB, 0.11068E+01_JPRB, 0.11068E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10809E+01_JPRB, 0.10833E+01_JPRB, 0.10654E+01_JPRB, 0.10402E+01_JPRB, 0.10085E+01_JPRB, 0.96356E+00_JPRB ,&
&  0.93051E+00_JPRB, 0.92387E+00_JPRB, 0.88960E+00_JPRB, 0.92372E+00_JPRB, 0.10284E+01_JPRB, 0.12815E+01_JPRB, 0.16042E+01_JPRB ,&
&  0.19110E+01_JPRB, 0.20664E+01_JPRB, 0.21143E+01_JPRB, 0.20988E+01_JPRB, 0.20840E+01_JPRB, 0.19822E+01_JPRB, 0.18129E+01_JPRB ,&
&  0.17646E+01_JPRB, 0.14931E+01_JPRB, 0.15327E+01_JPRB, 0.14744E+01_JPRB, 0.15026E+01_JPRB, 0.14781E+01_JPRB, 0.14331E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.13878E+01_JPRB, 0.13740E+01_JPRB, 0.13893E+01_JPRB, 0.14179E+01_JPRB, 0.14359E+01_JPRB, 0.14617E+01_JPRB ,&
&  0.14901E+01_JPRB, 0.14531E+01_JPRB, 0.13866E+01_JPRB, 0.13953E+01_JPRB, 0.18096E+01_JPRB, 0.25132E+01_JPRB, 0.31677E+01_JPRB ,&
&  0.31741E+01_JPRB, 0.29485E+01_JPRB, 0.43234E+01_JPRB, 0.50427E+01_JPRB, 0.50529E+01_JPRB, 0.56093E+01_JPRB, 0.55909E+01_JPRB ,&
&  0.50869E+01_JPRB, 0.44799E+01_JPRB, 0.40311E+01_JPRB, 0.42516E+01_JPRB, 0.58926E+01_JPRB, 0.62987E+01_JPRB, 0.62672E+01_JPRB ,&
&  0.56521E+01_JPRB, 0.50063E+01_JPRB, 0.44158E+01_JPRB, 0.40160E+01_JPRB, 0.38005E+01_JPRB, 0.39786E+01_JPRB, 0.37734E+01_JPRB ,&
&  0.35514E+01_JPRB, 0.37925E+01_JPRB, 0.31809E+01_JPRB, 0.25298E+01_JPRB, 0.21796E+01_JPRB, 0.20662E+01_JPRB, 0.20176E+01_JPRB ,&
&  0.18895E+01_JPRB, 0.18819E+01_JPRB, 0.19543E+01_JPRB, 0.24557E+01_JPRB, 0.38289E+01_JPRB, 0.40049E+01_JPRB, 0.40237E+01_JPRB ,&
&  0.41908E+01_JPRB, 0.50556E+01_JPRB, 0.44968E+01_JPRB, 0.38361E+01_JPRB, 0.31577E+01_JPRB, 0.25841E+01_JPRB, 0.21640E+01_JPRB ,&
&  0.19341E+01_JPRB, 0.18987E+01_JPRB, 0.18534E+01_JPRB, 0.17718E+01_JPRB, 0.17036E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.17031E+01_JPRB, 0.15006E+01_JPRB, 0.13953E+01_JPRB, 0.12903E+01_JPRB, 0.12371E+01_JPRB, 0.11977E+01_JPRB, 0.11245E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.10821E+01_JPRB, 0.10641E+01_JPRB, 0.10464E+01_JPRB, 0.10262E+01_JPRB, 0.98758E+00_JPRB, 0.94802E+00_JPRB ,&
&  0.93533E+00_JPRB, 0.92160E+00_JPRB, 0.94294E+00_JPRB, 0.10677E+01_JPRB, 0.12208E+01_JPRB, 0.14436E+01_JPRB, 0.18469E+01_JPRB ,&
&  0.22437E+01_JPRB, 0.24907E+01_JPRB, 0.25357E+01_JPRB, 0.26514E+01_JPRB, 0.27195E+01_JPRB, 0.27304E+01_JPRB, 0.25668E+01_JPRB ,&
&  0.22697E+01_JPRB, 0.21979E+01_JPRB, 0.22039E+01_JPRB, 0.20741E+01_JPRB, 0.19766E+01_JPRB, 0.18777E+01_JPRB, 0.17703E+01_JPRB ,&
&  0.16570E+01_JPRB, 0.15598E+01_JPRB, 0.15153E+01_JPRB, 0.15137E+01_JPRB, 0.15326E+01_JPRB, 0.15920E+01_JPRB, 0.15849E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.14733E+01_JPRB, 0.14407E+01_JPRB, 0.17676E+01_JPRB, 0.25938E+01_JPRB, 0.39416E+01_JPRB, 0.44129E+01_JPRB ,&
&  0.46472E+01_JPRB, 0.42050E+01_JPRB, 0.44238E+01_JPRB, 0.51650E+01_JPRB, 0.44292E+01_JPRB, 0.49212E+01_JPRB, 0.51293E+01_JPRB ,&
&  0.45951E+01_JPRB, 0.43700E+01_JPRB, 0.49216E+01_JPRB, 0.58676E+01_JPRB, 0.70597E+01_JPRB, 0.85200E+01_JPRB, 0.83227E+01_JPRB ,&
&  0.69217E+01_JPRB, 0.57277E+01_JPRB, 0.46082E+01_JPRB, 0.42829E+01_JPRB, 0.38384E+01_JPRB, 0.37909E+01_JPRB, 0.31143E+01_JPRB ,&
&  0.32237E+01_JPRB, 0.28059E+01_JPRB, 0.29814E+01_JPRB, 0.26632E+01_JPRB, 0.25635E+01_JPRB, 0.23855E+01_JPRB, 0.22574E+01_JPRB ,&
&  0.22709E+01_JPRB, 0.23727E+01_JPRB, 0.27499E+01_JPRB, 0.39604E+01_JPRB, 0.55733E+01_JPRB, 0.48178E+01_JPRB, 0.57866E+01_JPRB ,&
&  0.74202E+01_JPRB, 0.70568E+01_JPRB, 0.49328E+01_JPRB, 0.41031E+01_JPRB, 0.33415E+01_JPRB, 0.28071E+01_JPRB, 0.25055E+01_JPRB ,&
&  0.22510E+01_JPRB, 0.21288E+01_JPRB, 0.20547E+01_JPRB, 0.19723E+01_JPRB, 0.18806E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.18000E+01_JPRB, 0.16002E+01_JPRB, 0.15119E+01_JPRB, 0.14676E+01_JPRB, 0.13856E+01_JPRB, 0.12316E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.11161E+01_JPRB, 0.10484E+01_JPRB, 0.10043E+01_JPRB, 0.99269E+00_JPRB, 0.98943E+00_JPRB, 0.96013E+00_JPRB ,&
&  0.97704E+00_JPRB, 0.10532E+01_JPRB, 0.11553E+01_JPRB, 0.12888E+01_JPRB, 0.14899E+01_JPRB, 0.16773E+01_JPRB, 0.18905E+01_JPRB ,&
&  0.27673E+01_JPRB, 0.29938E+01_JPRB, 0.28373E+01_JPRB, 0.29125E+01_JPRB, 0.31019E+01_JPRB, 0.32511E+01_JPRB, 0.31564E+01_JPRB ,&
&  0.31402E+01_JPRB, 0.30475E+01_JPRB, 0.25737E+01_JPRB, 0.22381E+01_JPRB, 0.20228E+01_JPRB, 0.19021E+01_JPRB, 0.17700E+01_JPRB ,&
&  0.16092E+01_JPRB, 0.14865E+01_JPRB, 0.14563E+01_JPRB, 0.14467E+01_JPRB, 0.14791E+01_JPRB, 0.14158E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.14282E+01_JPRB, 0.14985E+01_JPRB, 0.16418E+01_JPRB, 0.22524E+01_JPRB, 0.35912E+01_JPRB, 0.52821E+01_JPRB, 0.57449E+01_JPRB ,&
&  0.50725E+01_JPRB, 0.43402E+01_JPRB, 0.45988E+01_JPRB, 0.51003E+01_JPRB, 0.50075E+01_JPRB, 0.47722E+01_JPRB, 0.44807E+01_JPRB ,&
&  0.37866E+01_JPRB, 0.39271E+01_JPRB, 0.48820E+01_JPRB, 0.64700E+01_JPRB, 0.87304E+01_JPRB, 0.11009E+02_JPRB, 0.10693E+02_JPRB ,&
&  0.85322E+01_JPRB, 0.67714E+01_JPRB, 0.54814E+01_JPRB, 0.44281E+01_JPRB, 0.43663E+01_JPRB, 0.34111E+01_JPRB, 0.38666E+01_JPRB ,&
&  0.39461E+01_JPRB, 0.38190E+01_JPRB, 0.31841E+01_JPRB, 0.26449E+01_JPRB, 0.22610E+01_JPRB, 0.25431E+01_JPRB, 0.25486E+01_JPRB ,&
&  0.28137E+01_JPRB, 0.31155E+01_JPRB, 0.53509E+01_JPRB, 0.63926E+01_JPRB, 0.69106E+01_JPRB, 0.68043E+01_JPRB, 0.88507E+01_JPRB ,&
&  0.82062E+01_JPRB, 0.69902E+01_JPRB, 0.54463E+01_JPRB, 0.44004E+01_JPRB, 0.36003E+01_JPRB, 0.31397E+01_JPRB, 0.28525E+01_JPRB ,&
&  0.25546E+01_JPRB, 0.23936E+01_JPRB, 0.22737E+01_JPRB, 0.21531E+01_JPRB, 0.20087E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.17752E+01_JPRB, 0.16692E+01_JPRB, 0.16379E+01_JPRB, 0.15132E+01_JPRB, 0.13149E+01_JPRB, 0.11836E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11483E+01_JPRB, 0.11012E+01_JPRB, 0.10372E+01_JPRB, 0.99303E+00_JPRB, 0.99739E+00_JPRB, 0.10084E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.13322E+01_JPRB, 0.15120E+01_JPRB, 0.16897E+01_JPRB, 0.18286E+01_JPRB, 0.19289E+01_JPRB, 0.22218E+01_JPRB ,&
&  0.33091E+01_JPRB, 0.31021E+01_JPRB, 0.25546E+01_JPRB, 0.26171E+01_JPRB, 0.31267E+01_JPRB, 0.31353E+01_JPRB, 0.31246E+01_JPRB ,&
&  0.31072E+01_JPRB, 0.28259E+01_JPRB, 0.24123E+01_JPRB, 0.20603E+01_JPRB, 0.18415E+01_JPRB, 0.17076E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.14461E+01_JPRB, 0.14339E+01_JPRB, 0.14698E+01_JPRB, 0.14998E+01_JPRB, 0.14763E+01_JPRB, 0.14554E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.15078E+01_JPRB, 0.16081E+01_JPRB, 0.18679E+01_JPRB, 0.27112E+01_JPRB, 0.41908E+01_JPRB, 0.60694E+01_JPRB, 0.69331E+01_JPRB ,&
&  0.55512E+01_JPRB, 0.51063E+01_JPRB, 0.51386E+01_JPRB, 0.62638E+01_JPRB, 0.49159E+01_JPRB, 0.46062E+01_JPRB, 0.44679E+01_JPRB ,&
&  0.42212E+01_JPRB, 0.49677E+01_JPRB, 0.66987E+01_JPRB, 0.86420E+01_JPRB, 0.91354E+01_JPRB, 0.10650E+02_JPRB, 0.12265E+02_JPRB ,&
&  0.90531E+01_JPRB, 0.70460E+01_JPRB, 0.59913E+01_JPRB, 0.56200E+01_JPRB, 0.37636E+01_JPRB, 0.27311E+01_JPRB, 0.39973E+01_JPRB ,&
&  0.39587E+01_JPRB, 0.23006E+01_JPRB, 0.17188E+01_JPRB, 0.19516E+01_JPRB, 0.14841E+01_JPRB, 0.17501E+01_JPRB, 0.26702E+01_JPRB ,&
&  0.34346E+01_JPRB, 0.42157E+01_JPRB, 0.84823E+01_JPRB, 0.78617E+01_JPRB, 0.84304E+01_JPRB, 0.99947E+01_JPRB, 0.10377E+02_JPRB ,&
&  0.90480E+01_JPRB, 0.56587E+01_JPRB, 0.50353E+01_JPRB, 0.43680E+01_JPRB, 0.35507E+01_JPRB, 0.29585E+01_JPRB, 0.25405E+01_JPRB ,&
&  0.22186E+01_JPRB, 0.20326E+01_JPRB, 0.19258E+01_JPRB, 0.18775E+01_JPRB, 0.18759E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.15261E+01_JPRB, 0.14931E+01_JPRB, 0.14334E+01_JPRB, 0.12829E+01_JPRB, 0.11731E+01_JPRB, 0.11409E+01_JPRB, 0.11316E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.10430E+01_JPRB, 0.10034E+01_JPRB, 0.10126E+01_JPRB, 0.10313E+01_JPRB, 0.10554E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.14316E+01_JPRB, 0.19194E+01_JPRB, 0.22714E+01_JPRB, 0.24580E+01_JPRB, 0.20899E+01_JPRB, 0.27092E+01_JPRB ,&
&  0.34500E+01_JPRB, 0.27644E+01_JPRB, 0.22664E+01_JPRB, 0.24884E+01_JPRB, 0.27227E+01_JPRB, 0.26245E+01_JPRB, 0.26068E+01_JPRB ,&
&  0.25615E+01_JPRB, 0.23612E+01_JPRB, 0.21371E+01_JPRB, 0.20010E+01_JPRB, 0.17950E+01_JPRB, 0.15786E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.15937E+01_JPRB, 0.16706E+01_JPRB, 0.16380E+01_JPRB, 0.16390E+01_JPRB, 0.16273E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.17030E+01_JPRB, 0.18605E+01_JPRB, 0.22875E+01_JPRB, 0.34766E+01_JPRB, 0.55715E+01_JPRB, 0.80578E+01_JPRB, 0.67025E+01_JPRB ,&
&  0.50391E+01_JPRB, 0.45490E+01_JPRB, 0.56885E+01_JPRB, 0.59441E+01_JPRB, 0.47336E+01_JPRB, 0.47269E+01_JPRB, 0.46522E+01_JPRB ,&
&  0.53251E+01_JPRB, 0.63370E+01_JPRB, 0.69372E+01_JPRB, 0.86100E+01_JPRB, 0.10626E+02_JPRB, 0.10902E+02_JPRB, 0.11118E+02_JPRB ,&
&  0.91123E+01_JPRB, 0.69825E+01_JPRB, 0.51192E+01_JPRB, 0.41650E+01_JPRB, 0.48709E+01_JPRB, 0.37394E+01_JPRB, 0.24265E+01_JPRB ,&
&  0.19002E+01_JPRB, 0.15082E+01_JPRB, 0.12210E+01_JPRB, 0.11955E+01_JPRB, 0.10813E+01_JPRB, 0.15222E+01_JPRB, 0.25681E+01_JPRB ,&
&  0.50153E+01_JPRB, 0.70589E+01_JPRB, 0.93346E+01_JPRB, 0.94917E+01_JPRB, 0.88808E+01_JPRB, 0.10489E+02_JPRB, 0.11278E+02_JPRB ,&
&  0.69607E+01_JPRB, 0.49087E+01_JPRB, 0.45615E+01_JPRB, 0.36576E+01_JPRB, 0.27194E+01_JPRB, 0.21860E+01_JPRB, 0.18670E+01_JPRB ,&
&  0.16960E+01_JPRB, 0.15944E+01_JPRB, 0.15330E+01_JPRB, 0.15345E+01_JPRB, 0.15615E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.13038E+01_JPRB, 0.12782E+01_JPRB, 0.12519E+01_JPRB, 0.11751E+01_JPRB, 0.11097E+01_JPRB, 0.11186E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.10315E+01_JPRB, 0.10295E+01_JPRB, 0.10605E+01_JPRB, 0.10997E+01_JPRB, 0.11704E+01_JPRB, 0.12582E+01_JPRB ,&
&  0.13743E+01_JPRB, 0.16894E+01_JPRB, 0.23568E+01_JPRB, 0.30017E+01_JPRB, 0.33725E+01_JPRB, 0.26354E+01_JPRB, 0.32191E+01_JPRB ,&
&  0.33632E+01_JPRB, 0.26043E+01_JPRB, 0.23242E+01_JPRB, 0.24995E+01_JPRB, 0.25884E+01_JPRB, 0.22834E+01_JPRB, 0.22498E+01_JPRB ,&
&  0.22006E+01_JPRB, 0.20984E+01_JPRB, 0.19876E+01_JPRB, 0.19253E+01_JPRB, 0.17615E+01_JPRB, 0.17712E+01_JPRB, 0.18607E+01_JPRB ,&
&  0.19082E+01_JPRB, 0.19854E+01_JPRB, 0.19536E+01_JPRB, 0.19155E+01_JPRB, 0.19021E+01_JPRB, 0.19034E+01_JPRB, 0.19475E+01_JPRB ,&
&  0.20660E+01_JPRB, 0.23767E+01_JPRB, 0.32037E+01_JPRB, 0.50570E+01_JPRB, 0.67208E+01_JPRB, 0.67763E+01_JPRB, 0.61011E+01_JPRB ,&
&  0.55130E+01_JPRB, 0.55379E+01_JPRB, 0.55882E+01_JPRB, 0.62006E+01_JPRB, 0.49129E+01_JPRB, 0.47504E+01_JPRB, 0.48213E+01_JPRB ,&
&  0.55725E+01_JPRB, 0.74154E+01_JPRB, 0.80600E+01_JPRB, 0.91540E+01_JPRB, 0.12985E+02_JPRB, 0.13352E+02_JPRB, 0.10422E+02_JPRB ,&
&  0.89553E+01_JPRB, 0.68317E+01_JPRB, 0.55748E+01_JPRB, 0.49926E+01_JPRB, 0.75627E+01_JPRB, 0.81575E+01_JPRB, 0.28002E+01_JPRB ,&
&  0.16668E+01_JPRB, 0.13081E+01_JPRB, 0.97053E+00_JPRB, 0.94074E+00_JPRB, 0.10332E+01_JPRB, 0.15574E+01_JPRB, 0.39605E+01_JPRB ,&
&  0.47260E+01_JPRB, 0.77003E+01_JPRB, 0.86789E+01_JPRB, 0.93879E+01_JPRB, 0.88501E+01_JPRB, 0.89606E+01_JPRB, 0.70397E+01_JPRB ,&
&  0.57353E+01_JPRB, 0.44272E+01_JPRB, 0.33246E+01_JPRB, 0.25601E+01_JPRB, 0.20120E+01_JPRB, 0.17193E+01_JPRB, 0.15701E+01_JPRB ,&
&  0.15034E+01_JPRB, 0.14655E+01_JPRB, 0.14153E+01_JPRB, 0.13670E+01_JPRB, 0.13117E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.11165E+01_JPRB, 0.11431E+01_JPRB, 0.11756E+01_JPRB, 0.11714E+01_JPRB, 0.11681E+01_JPRB, 0.12044E+01_JPRB, 0.11875E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.11052E+01_JPRB, 0.11236E+01_JPRB, 0.11672E+01_JPRB, 0.12374E+01_JPRB, 0.13414E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.16944E+01_JPRB, 0.22179E+01_JPRB, 0.29808E+01_JPRB, 0.39417E+01_JPRB, 0.42236E+01_JPRB, 0.37848E+01_JPRB, 0.38305E+01_JPRB ,&
&  0.34416E+01_JPRB, 0.27306E+01_JPRB, 0.23945E+01_JPRB, 0.25347E+01_JPRB, 0.25124E+01_JPRB, 0.23056E+01_JPRB, 0.21290E+01_JPRB ,&
&  0.21394E+01_JPRB, 0.21645E+01_JPRB, 0.21444E+01_JPRB, 0.21856E+01_JPRB, 0.22746E+01_JPRB, 0.23828E+01_JPRB, 0.24718E+01_JPRB ,&
&  0.25137E+01_JPRB, 0.24515E+01_JPRB, 0.23425E+01_JPRB, 0.22773E+01_JPRB, 0.22673E+01_JPRB, 0.23158E+01_JPRB, 0.24366E+01_JPRB ,&
&  0.26879E+01_JPRB, 0.33037E+01_JPRB, 0.46289E+01_JPRB, 0.63081E+01_JPRB, 0.61712E+01_JPRB, 0.65994E+01_JPRB, 0.60520E+01_JPRB ,&
&  0.55035E+01_JPRB, 0.54605E+01_JPRB, 0.51970E+01_JPRB, 0.55820E+01_JPRB, 0.52221E+01_JPRB, 0.51695E+01_JPRB, 0.52913E+01_JPRB ,&
&  0.71845E+01_JPRB, 0.85562E+01_JPRB, 0.10252E+02_JPRB, 0.10976E+02_JPRB, 0.13390E+02_JPRB, 0.16987E+02_JPRB, 0.10710E+02_JPRB ,&
&  0.78288E+01_JPRB, 0.72883E+01_JPRB, 0.61240E+01_JPRB, 0.56294E+01_JPRB, 0.71130E+01_JPRB, 0.10001E+02_JPRB, 0.74835E+01_JPRB ,&
&  0.23273E+01_JPRB, 0.14385E+01_JPRB, 0.12624E+01_JPRB, 0.12723E+01_JPRB, 0.10543E+01_JPRB, 0.16353E+01_JPRB, 0.57588E+01_JPRB ,&
&  0.50708E+01_JPRB, 0.67497E+01_JPRB, 0.75351E+01_JPRB, 0.82060E+01_JPRB, 0.82107E+01_JPRB, 0.61539E+01_JPRB, 0.44915E+01_JPRB ,&
&  0.29559E+01_JPRB, 0.21456E+01_JPRB, 0.18250E+01_JPRB, 0.16104E+01_JPRB, 0.13719E+01_JPRB, 0.12534E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.11640E+01_JPRB, 0.11371E+01_JPRB, 0.11026E+01_JPRB, 0.11022E+01_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.10781E+01_JPRB, 0.11542E+01_JPRB, 0.12223E+01_JPRB, 0.12924E+01_JPRB, 0.13415E+01_JPRB, 0.13447E+01_JPRB, 0.13179E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.12855E+01_JPRB, 0.13132E+01_JPRB, 0.13685E+01_JPRB, 0.14890E+01_JPRB, 0.16725E+01_JPRB, 0.19567E+01_JPRB ,&
&  0.24118E+01_JPRB, 0.30623E+01_JPRB, 0.38454E+01_JPRB, 0.45887E+01_JPRB, 0.46664E+01_JPRB, 0.40180E+01_JPRB, 0.39441E+01_JPRB ,&
&  0.36436E+01_JPRB, 0.30185E+01_JPRB, 0.25503E+01_JPRB, 0.27418E+01_JPRB, 0.28243E+01_JPRB, 0.27316E+01_JPRB, 0.25433E+01_JPRB ,&
&  0.25602E+01_JPRB, 0.26076E+01_JPRB, 0.26676E+01_JPRB, 0.27850E+01_JPRB, 0.28973E+01_JPRB, 0.29785E+01_JPRB, 0.30051E+01_JPRB ,&
&  0.29155E+01_JPRB, 0.27860E+01_JPRB, 0.27226E+01_JPRB, 0.27371E+01_JPRB, 0.28064E+01_JPRB, 0.29070E+01_JPRB, 0.31343E+01_JPRB ,&
&  0.36329E+01_JPRB, 0.45326E+01_JPRB, 0.57227E+01_JPRB, 0.62294E+01_JPRB, 0.60908E+01_JPRB, 0.59118E+01_JPRB, 0.55649E+01_JPRB ,&
&  0.52852E+01_JPRB, 0.53602E+01_JPRB, 0.53302E+01_JPRB, 0.53831E+01_JPRB, 0.54036E+01_JPRB, 0.57691E+01_JPRB, 0.63582E+01_JPRB ,&
&  0.87343E+01_JPRB, 0.10622E+02_JPRB, 0.11027E+02_JPRB, 0.11055E+02_JPRB, 0.11652E+02_JPRB, 0.13364E+02_JPRB, 0.12263E+02_JPRB ,&
&  0.91732E+01_JPRB, 0.67755E+01_JPRB, 0.58992E+01_JPRB, 0.61900E+01_JPRB, 0.65254E+01_JPRB, 0.76602E+01_JPRB, 0.94637E+01_JPRB ,&
&  0.84404E+01_JPRB, 0.32115E+01_JPRB, 0.20544E+01_JPRB, 0.29791E+01_JPRB, 0.17703E+01_JPRB, 0.17495E+01_JPRB, 0.30315E+01_JPRB ,&
&  0.39383E+01_JPRB, 0.48212E+01_JPRB, 0.54354E+01_JPRB, 0.64074E+01_JPRB, 0.54858E+01_JPRB, 0.39166E+01_JPRB, 0.26317E+01_JPRB ,&
&  0.18019E+01_JPRB, 0.14805E+01_JPRB, 0.14009E+01_JPRB, 0.12276E+01_JPRB, 0.11076E+01_JPRB, 0.10781E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.99898E+00_JPRB, 0.10053E+01_JPRB, 0.10241E+01_JPRB, 0.10250E+01_JPRB, 0.10237E+01_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.11720E+01_JPRB, 0.12656E+01_JPRB, 0.13963E+01_JPRB, 0.14776E+01_JPRB, 0.15049E+01_JPRB, 0.15177E+01_JPRB, 0.15525E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.16382E+01_JPRB, 0.17296E+01_JPRB, 0.18775E+01_JPRB, 0.21305E+01_JPRB, 0.24524E+01_JPRB, 0.28688E+01_JPRB ,&
&  0.33840E+01_JPRB, 0.39535E+01_JPRB, 0.45459E+01_JPRB, 0.51154E+01_JPRB, 0.54843E+01_JPRB, 0.58070E+01_JPRB, 0.38244E+01_JPRB ,&
&  0.31927E+01_JPRB, 0.34985E+01_JPRB, 0.28035E+01_JPRB, 0.30922E+01_JPRB, 0.32286E+01_JPRB, 0.32415E+01_JPRB, 0.29604E+01_JPRB ,&
&  0.29397E+01_JPRB, 0.30113E+01_JPRB, 0.31016E+01_JPRB, 0.31549E+01_JPRB, 0.31846E+01_JPRB, 0.32066E+01_JPRB, 0.31807E+01_JPRB ,&
&  0.31327E+01_JPRB, 0.31196E+01_JPRB, 0.31729E+01_JPRB, 0.32646E+01_JPRB, 0.33997E+01_JPRB, 0.36213E+01_JPRB, 0.39985E+01_JPRB ,&
&  0.45512E+01_JPRB, 0.52172E+01_JPRB, 0.58095E+01_JPRB, 0.56417E+01_JPRB, 0.54906E+01_JPRB, 0.53285E+01_JPRB, 0.53045E+01_JPRB ,&
&  0.51177E+01_JPRB, 0.48468E+01_JPRB, 0.52195E+01_JPRB, 0.56367E+01_JPRB, 0.58244E+01_JPRB, 0.62575E+01_JPRB, 0.72855E+01_JPRB ,&
&  0.97713E+01_JPRB, 0.11215E+02_JPRB, 0.11766E+02_JPRB, 0.10129E+02_JPRB, 0.95526E+01_JPRB, 0.10492E+02_JPRB, 0.10785E+02_JPRB ,&
&  0.92516E+01_JPRB, 0.72679E+01_JPRB, 0.63088E+01_JPRB, 0.58290E+01_JPRB, 0.55281E+01_JPRB, 0.59843E+01_JPRB, 0.66617E+01_JPRB ,&
&  0.71477E+01_JPRB, 0.78926E+01_JPRB, 0.63031E+01_JPRB, 0.36769E+01_JPRB, 0.25330E+01_JPRB, 0.23947E+01_JPRB, 0.29132E+01_JPRB ,&
&  0.33602E+01_JPRB, 0.35192E+01_JPRB, 0.34808E+01_JPRB, 0.52406E+01_JPRB, 0.29896E+01_JPRB, 0.20549E+01_JPRB, 0.14606E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.12110E+01_JPRB, 0.12025E+01_JPRB, 0.11109E+01_JPRB, 0.10366E+01_JPRB, 0.10574E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.10688E+01_JPRB, 0.10781E+01_JPRB, 0.10896E+01_JPRB, 0.11294E+01_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.13892E+01_JPRB, 0.14793E+01_JPRB, 0.15926E+01_JPRB, 0.16684E+01_JPRB, 0.17572E+01_JPRB, 0.18382E+01_JPRB, 0.19603E+01_JPRB ,&
&  0.21006E+01_JPRB, 0.22658E+01_JPRB, 0.24543E+01_JPRB, 0.26973E+01_JPRB, 0.30183E+01_JPRB, 0.33709E+01_JPRB, 0.37628E+01_JPRB ,&
&  0.41330E+01_JPRB, 0.44610E+01_JPRB, 0.47771E+01_JPRB, 0.51112E+01_JPRB, 0.53790E+01_JPRB, 0.57327E+01_JPRB, 0.49733E+01_JPRB ,&
&  0.29800E+01_JPRB, 0.42158E+01_JPRB, 0.30215E+01_JPRB, 0.34185E+01_JPRB, 0.35115E+01_JPRB, 0.36674E+01_JPRB, 0.32871E+01_JPRB ,&
&  0.31091E+01_JPRB, 0.31422E+01_JPRB, 0.31675E+01_JPRB, 0.32119E+01_JPRB, 0.32420E+01_JPRB, 0.32780E+01_JPRB, 0.33098E+01_JPRB ,&
&  0.33655E+01_JPRB, 0.34683E+01_JPRB, 0.36028E+01_JPRB, 0.37502E+01_JPRB, 0.39784E+01_JPRB, 0.42948E+01_JPRB, 0.46088E+01_JPRB ,&
&  0.49318E+01_JPRB, 0.52512E+01_JPRB, 0.53371E+01_JPRB, 0.51670E+01_JPRB, 0.50807E+01_JPRB, 0.52016E+01_JPRB, 0.54664E+01_JPRB ,&
&  0.53726E+01_JPRB, 0.51823E+01_JPRB, 0.55129E+01_JPRB, 0.57357E+01_JPRB, 0.59160E+01_JPRB, 0.68710E+01_JPRB, 0.77834E+01_JPRB ,&
&  0.96020E+01_JPRB, 0.99643E+01_JPRB, 0.11446E+02_JPRB, 0.95735E+01_JPRB, 0.80582E+01_JPRB, 0.84053E+01_JPRB, 0.77484E+01_JPRB ,&
&  0.64851E+01_JPRB, 0.60170E+01_JPRB, 0.52750E+01_JPRB, 0.44757E+01_JPRB, 0.44928E+01_JPRB, 0.47162E+01_JPRB, 0.52826E+01_JPRB ,&
&  0.58120E+01_JPRB, 0.59070E+01_JPRB, 0.43547E+01_JPRB, 0.26912E+01_JPRB, 0.24032E+01_JPRB, 0.24882E+01_JPRB, 0.28841E+01_JPRB ,&
&  0.29014E+01_JPRB, 0.28606E+01_JPRB, 0.30364E+01_JPRB, 0.28527E+01_JPRB, 0.16654E+01_JPRB, 0.12415E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.11346E+01_JPRB, 0.11361E+01_JPRB, 0.11091E+01_JPRB, 0.10413E+01_JPRB, 0.10897E+01_JPRB, 0.11007E+01_JPRB ,&
&  0.11685E+01_JPRB, 0.11961E+01_JPRB, 0.12425E+01_JPRB, 0.12886E+01_JPRB, 0.13301E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.16564E+01_JPRB, 0.17835E+01_JPRB, 0.19760E+01_JPRB, 0.21497E+01_JPRB, 0.22962E+01_JPRB, 0.24212E+01_JPRB, 0.25248E+01_JPRB ,&
&  0.26220E+01_JPRB, 0.27498E+01_JPRB, 0.29879E+01_JPRB, 0.32353E+01_JPRB, 0.35423E+01_JPRB, 0.38853E+01_JPRB, 0.42110E+01_JPRB ,&
&  0.44865E+01_JPRB, 0.47422E+01_JPRB, 0.50022E+01_JPRB, 0.53596E+01_JPRB, 0.57967E+01_JPRB, 0.65497E+01_JPRB, 0.63080E+01_JPRB ,&
&  0.44881E+01_JPRB, 0.33419E+01_JPRB, 0.32315E+01_JPRB, 0.33312E+01_JPRB, 0.32718E+01_JPRB, 0.35799E+01_JPRB, 0.36730E+01_JPRB ,&
&  0.33392E+01_JPRB, 0.32342E+01_JPRB, 0.32328E+01_JPRB, 0.32579E+01_JPRB, 0.32245E+01_JPRB, 0.33136E+01_JPRB, 0.34168E+01_JPRB ,&
&  0.35684E+01_JPRB, 0.37441E+01_JPRB, 0.39234E+01_JPRB, 0.41363E+01_JPRB, 0.43690E+01_JPRB, 0.46225E+01_JPRB, 0.48424E+01_JPRB ,&
&  0.50190E+01_JPRB, 0.51646E+01_JPRB, 0.51460E+01_JPRB, 0.49831E+01_JPRB, 0.50957E+01_JPRB, 0.53453E+01_JPRB, 0.54795E+01_JPRB ,&
&  0.55146E+01_JPRB, 0.56793E+01_JPRB, 0.60389E+01_JPRB, 0.62625E+01_JPRB, 0.62740E+01_JPRB, 0.72140E+01_JPRB, 0.78758E+01_JPRB ,&
&  0.87032E+01_JPRB, 0.88812E+01_JPRB, 0.10363E+02_JPRB, 0.88016E+01_JPRB, 0.75474E+01_JPRB, 0.70675E+01_JPRB, 0.62071E+01_JPRB ,&
&  0.53368E+01_JPRB, 0.50207E+01_JPRB, 0.44946E+01_JPRB, 0.33676E+01_JPRB, 0.35884E+01_JPRB, 0.34938E+01_JPRB, 0.46108E+01_JPRB ,&
&  0.53657E+01_JPRB, 0.58932E+01_JPRB, 0.35403E+01_JPRB, 0.26627E+01_JPRB, 0.23446E+01_JPRB, 0.25867E+01_JPRB, 0.26380E+01_JPRB ,&
&  0.24489E+01_JPRB, 0.21604E+01_JPRB, 0.19984E+01_JPRB, 0.18858E+01_JPRB, 0.11962E+01_JPRB, 0.10191E+01_JPRB, 0.97366E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.10982E+01_JPRB, 0.11109E+01_JPRB, 0.11006E+01_JPRB, 0.11177E+01_JPRB, 0.11999E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.13276E+01_JPRB, 0.13838E+01_JPRB, 0.14485E+01_JPRB, 0.14912E+01_JPRB, 0.15510E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.17222E+01_JPRB, 0.18511E+01_JPRB, 0.20500E+01_JPRB, 0.22397E+01_JPRB, 0.23979E+01_JPRB, 0.24995E+01_JPRB, 0.25579E+01_JPRB ,&
&  0.26569E+01_JPRB, 0.28015E+01_JPRB, 0.30074E+01_JPRB, 0.32004E+01_JPRB, 0.34288E+01_JPRB, 0.36620E+01_JPRB, 0.38926E+01_JPRB ,&
&  0.41055E+01_JPRB, 0.43010E+01_JPRB, 0.45492E+01_JPRB, 0.48630E+01_JPRB, 0.52352E+01_JPRB, 0.58102E+01_JPRB, 0.58456E+01_JPRB ,&
&  0.45577E+01_JPRB, 0.27038E+01_JPRB, 0.29551E+01_JPRB, 0.29948E+01_JPRB, 0.31007E+01_JPRB, 0.34784E+01_JPRB, 0.35812E+01_JPRB ,&
&  0.31358E+01_JPRB, 0.30960E+01_JPRB, 0.32089E+01_JPRB, 0.32207E+01_JPRB, 0.31682E+01_JPRB, 0.33586E+01_JPRB, 0.35255E+01_JPRB ,&
&  0.37134E+01_JPRB, 0.38778E+01_JPRB, 0.40350E+01_JPRB, 0.42070E+01_JPRB, 0.43667E+01_JPRB, 0.45442E+01_JPRB, 0.47463E+01_JPRB ,&
&  0.49581E+01_JPRB, 0.51211E+01_JPRB, 0.50904E+01_JPRB, 0.50656E+01_JPRB, 0.51809E+01_JPRB, 0.54637E+01_JPRB, 0.57287E+01_JPRB ,&
&  0.59066E+01_JPRB, 0.59540E+01_JPRB, 0.64403E+01_JPRB, 0.70656E+01_JPRB, 0.73612E+01_JPRB, 0.72009E+01_JPRB, 0.77015E+01_JPRB ,&
&  0.80454E+01_JPRB, 0.79617E+01_JPRB, 0.84802E+01_JPRB, 0.93906E+01_JPRB, 0.65434E+01_JPRB, 0.62790E+01_JPRB, 0.50679E+01_JPRB ,&
&  0.51211E+01_JPRB, 0.46713E+01_JPRB, 0.40674E+01_JPRB, 0.33566E+01_JPRB, 0.29255E+01_JPRB, 0.28579E+01_JPRB, 0.37263E+01_JPRB ,&
&  0.47599E+01_JPRB, 0.41796E+01_JPRB, 0.28647E+01_JPRB, 0.24096E+01_JPRB, 0.24090E+01_JPRB, 0.30068E+01_JPRB, 0.29495E+01_JPRB ,&
&  0.24420E+01_JPRB, 0.20524E+01_JPRB, 0.19371E+01_JPRB, 0.24677E+01_JPRB, 0.14574E+01_JPRB, 0.99188E+00_JPRB, 0.90195E+00_JPRB ,&
&  0.97426E+00_JPRB, 0.10489E+01_JPRB, 0.11171E+01_JPRB, 0.11688E+01_JPRB, 0.12175E+01_JPRB, 0.12875E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.14106E+01_JPRB, 0.14381E+01_JPRB, 0.15242E+01_JPRB, 0.15766E+01_JPRB, 0.16423E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.17097E+01_JPRB, 0.18083E+01_JPRB, 0.19136E+01_JPRB, 0.20360E+01_JPRB, 0.21669E+01_JPRB, 0.22715E+01_JPRB, 0.23411E+01_JPRB ,&
&  0.24267E+01_JPRB, 0.25412E+01_JPRB, 0.26765E+01_JPRB, 0.27924E+01_JPRB, 0.29632E+01_JPRB, 0.31235E+01_JPRB, 0.32832E+01_JPRB ,&
&  0.34413E+01_JPRB, 0.36214E+01_JPRB, 0.38507E+01_JPRB, 0.40471E+01_JPRB, 0.41586E+01_JPRB, 0.42520E+01_JPRB, 0.43496E+01_JPRB ,&
&  0.44311E+01_JPRB, 0.37337E+01_JPRB, 0.32343E+01_JPRB, 0.24858E+01_JPRB, 0.20754E+01_JPRB, 0.26310E+01_JPRB, 0.26087E+01_JPRB ,&
&  0.26160E+01_JPRB, 0.25959E+01_JPRB, 0.26385E+01_JPRB, 0.27505E+01_JPRB, 0.29394E+01_JPRB, 0.31385E+01_JPRB, 0.33108E+01_JPRB ,&
&  0.34541E+01_JPRB, 0.35684E+01_JPRB, 0.36632E+01_JPRB, 0.37726E+01_JPRB, 0.39042E+01_JPRB, 0.40305E+01_JPRB, 0.41641E+01_JPRB ,&
&  0.43408E+01_JPRB, 0.45703E+01_JPRB, 0.47283E+01_JPRB, 0.49167E+01_JPRB, 0.50758E+01_JPRB, 0.54649E+01_JPRB, 0.58810E+01_JPRB ,&
&  0.61541E+01_JPRB, 0.63115E+01_JPRB, 0.67723E+01_JPRB, 0.73050E+01_JPRB, 0.73895E+01_JPRB, 0.66194E+01_JPRB, 0.69122E+01_JPRB ,&
&  0.73040E+01_JPRB, 0.70886E+01_JPRB, 0.69315E+01_JPRB, 0.88890E+01_JPRB, 0.55303E+01_JPRB, 0.58359E+01_JPRB, 0.56238E+01_JPRB ,&
&  0.47885E+01_JPRB, 0.44511E+01_JPRB, 0.37904E+01_JPRB, 0.28548E+01_JPRB, 0.24185E+01_JPRB, 0.24039E+01_JPRB, 0.31203E+01_JPRB ,&
&  0.36357E+01_JPRB, 0.29522E+01_JPRB, 0.21394E+01_JPRB, 0.18788E+01_JPRB, 0.21263E+01_JPRB, 0.30166E+01_JPRB, 0.31184E+01_JPRB ,&
&  0.23305E+01_JPRB, 0.20744E+01_JPRB, 0.18818E+01_JPRB, 0.30408E+01_JPRB, 0.21615E+01_JPRB, 0.99424E+00_JPRB, 0.79267E+00_JPRB ,&
&  0.82407E+00_JPRB, 0.90252E+00_JPRB, 0.10169E+01_JPRB, 0.11004E+01_JPRB, 0.11446E+01_JPRB, 0.11985E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.13907E+01_JPRB, 0.14905E+01_JPRB, 0.15637E+01_JPRB, 0.16353E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.13425E+01_JPRB, 0.14180E+01_JPRB, 0.14789E+01_JPRB, 0.15150E+01_JPRB, 0.15375E+01_JPRB, 0.15392E+01_JPRB, 0.15084E+01_JPRB ,&
&  0.14736E+01_JPRB, 0.15184E+01_JPRB, 0.16446E+01_JPRB, 0.19217E+01_JPRB, 0.21627E+01_JPRB, 0.23212E+01_JPRB, 0.24696E+01_JPRB ,&
&  0.26135E+01_JPRB, 0.26678E+01_JPRB, 0.25929E+01_JPRB, 0.25601E+01_JPRB, 0.24488E+01_JPRB, 0.23143E+01_JPRB, 0.25121E+01_JPRB ,&
&  0.28966E+01_JPRB, 0.33204E+01_JPRB, 0.35446E+01_JPRB, 0.37642E+01_JPRB, 0.30219E+01_JPRB, 0.25744E+01_JPRB, 0.28903E+01_JPRB ,&
&  0.32396E+01_JPRB, 0.28149E+01_JPRB, 0.22752E+01_JPRB, 0.22780E+01_JPRB, 0.23325E+01_JPRB, 0.24589E+01_JPRB, 0.26563E+01_JPRB ,&
&  0.27930E+01_JPRB, 0.28893E+01_JPRB, 0.29763E+01_JPRB, 0.30530E+01_JPRB, 0.31285E+01_JPRB, 0.32039E+01_JPRB, 0.32973E+01_JPRB ,&
&  0.33757E+01_JPRB, 0.34548E+01_JPRB, 0.36906E+01_JPRB, 0.40307E+01_JPRB, 0.44300E+01_JPRB, 0.49656E+01_JPRB, 0.54292E+01_JPRB ,&
&  0.57169E+01_JPRB, 0.59494E+01_JPRB, 0.66587E+01_JPRB, 0.71200E+01_JPRB, 0.67520E+01_JPRB, 0.61382E+01_JPRB, 0.63922E+01_JPRB ,&
&  0.64320E+01_JPRB, 0.60175E+01_JPRB, 0.47997E+01_JPRB, 0.69314E+01_JPRB, 0.69643E+01_JPRB, 0.59384E+01_JPRB, 0.48338E+01_JPRB ,&
&  0.42453E+01_JPRB, 0.39852E+01_JPRB, 0.34931E+01_JPRB, 0.28383E+01_JPRB, 0.22847E+01_JPRB, 0.27307E+01_JPRB, 0.30193E+01_JPRB ,&
&  0.30784E+01_JPRB, 0.23880E+01_JPRB, 0.17178E+01_JPRB, 0.15266E+01_JPRB, 0.16355E+01_JPRB, 0.25877E+01_JPRB, 0.25482E+01_JPRB ,&
&  0.19373E+01_JPRB, 0.17350E+01_JPRB, 0.16130E+01_JPRB, 0.23894E+01_JPRB, 0.35546E+01_JPRB, 0.10613E+01_JPRB, 0.69897E+00_JPRB ,&
&  0.70031E+00_JPRB, 0.73995E+00_JPRB, 0.78349E+00_JPRB, 0.80073E+00_JPRB, 0.80801E+00_JPRB, 0.85133E+00_JPRB, 0.92145E+00_JPRB ,&
&  0.10075E+01_JPRB, 0.10862E+01_JPRB, 0.11415E+01_JPRB, 0.12162E+01_JPRB, 0.12785E+01_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.92101E+00_JPRB, 0.95784E+00_JPRB, 0.96653E+00_JPRB, 0.98222E+00_JPRB, 0.97867E+00_JPRB, 0.97809E+00_JPRB, 0.94461E+00_JPRB ,&
&  0.92545E+00_JPRB, 0.95688E+00_JPRB, 0.10464E+01_JPRB, 0.12108E+01_JPRB, 0.12733E+01_JPRB, 0.13226E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.14078E+01_JPRB, 0.13341E+01_JPRB, 0.12778E+01_JPRB, 0.11803E+01_JPRB, 0.11353E+01_JPRB, 0.11521E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.14461E+01_JPRB, 0.16931E+01_JPRB, 0.19678E+01_JPRB, 0.23010E+01_JPRB, 0.30925E+01_JPRB, 0.28619E+01_JPRB, 0.29396E+01_JPRB ,&
&  0.35172E+01_JPRB, 0.26207E+01_JPRB, 0.21462E+01_JPRB, 0.18695E+01_JPRB, 0.17421E+01_JPRB, 0.16922E+01_JPRB, 0.17734E+01_JPRB ,&
&  0.18205E+01_JPRB, 0.18986E+01_JPRB, 0.19916E+01_JPRB, 0.20317E+01_JPRB, 0.20729E+01_JPRB, 0.20485E+01_JPRB, 0.20081E+01_JPRB ,&
&  0.19610E+01_JPRB, 0.20929E+01_JPRB, 0.23752E+01_JPRB, 0.25910E+01_JPRB, 0.31255E+01_JPRB, 0.36032E+01_JPRB, 0.40137E+01_JPRB ,&
&  0.42502E+01_JPRB, 0.45653E+01_JPRB, 0.51936E+01_JPRB, 0.53755E+01_JPRB, 0.54834E+01_JPRB, 0.52326E+01_JPRB, 0.50864E+01_JPRB ,&
&  0.47947E+01_JPRB, 0.38715E+01_JPRB, 0.32217E+01_JPRB, 0.48401E+01_JPRB, 0.49703E+01_JPRB, 0.39225E+01_JPRB, 0.40377E+01_JPRB ,&
&  0.37020E+01_JPRB, 0.34683E+01_JPRB, 0.32796E+01_JPRB, 0.27116E+01_JPRB, 0.21084E+01_JPRB, 0.22958E+01_JPRB, 0.28337E+01_JPRB ,&
&  0.24745E+01_JPRB, 0.17668E+01_JPRB, 0.13468E+01_JPRB, 0.12966E+01_JPRB, 0.14473E+01_JPRB, 0.20926E+01_JPRB, 0.22516E+01_JPRB ,&
&  0.20391E+01_JPRB, 0.15216E+01_JPRB, 0.17803E+01_JPRB, 0.28414E+01_JPRB, 0.20103E+01_JPRB, 0.96093E+00_JPRB, 0.60745E+00_JPRB ,&
&  0.57896E+00_JPRB, 0.59527E+00_JPRB, 0.61744E+00_JPRB, 0.62795E+00_JPRB, 0.63367E+00_JPRB, 0.64737E+00_JPRB, 0.69384E+00_JPRB ,&
&  0.73591E+00_JPRB, 0.77978E+00_JPRB, 0.81640E+00_JPRB, 0.86574E+00_JPRB, 0.89742E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.85964E+00_JPRB, 0.88282E+00_JPRB, 0.91120E+00_JPRB, 0.93012E+00_JPRB, 0.95480E+00_JPRB, 0.97714E+00_JPRB, 0.10282E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.11233E+01_JPRB, 0.11822E+01_JPRB, 0.12453E+01_JPRB, 0.12657E+01_JPRB, 0.12849E+01_JPRB, 0.13016E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.13415E+01_JPRB, 0.13213E+01_JPRB, 0.13124E+01_JPRB, 0.13008E+01_JPRB, 0.13878E+01_JPRB, 0.13732E+01_JPRB ,&
&  0.13524E+01_JPRB, 0.13728E+01_JPRB, 0.13833E+01_JPRB, 0.14593E+01_JPRB, 0.17014E+01_JPRB, 0.19629E+01_JPRB, 0.21007E+01_JPRB ,&
&  0.27127E+01_JPRB, 0.21574E+01_JPRB, 0.18857E+01_JPRB, 0.14159E+01_JPRB, 0.13174E+01_JPRB, 0.13295E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.12868E+01_JPRB, 0.13378E+01_JPRB, 0.14301E+01_JPRB, 0.14433E+01_JPRB, 0.15251E+01_JPRB, 0.15906E+01_JPRB, 0.16363E+01_JPRB ,&
&  0.17244E+01_JPRB, 0.18321E+01_JPRB, 0.19510E+01_JPRB, 0.21385E+01_JPRB, 0.23147E+01_JPRB, 0.27596E+01_JPRB, 0.29609E+01_JPRB ,&
&  0.30454E+01_JPRB, 0.32074E+01_JPRB, 0.30076E+01_JPRB, 0.30671E+01_JPRB, 0.34874E+01_JPRB, 0.34900E+01_JPRB, 0.33999E+01_JPRB ,&
&  0.34893E+01_JPRB, 0.31060E+01_JPRB, 0.23253E+01_JPRB, 0.34429E+01_JPRB, 0.38082E+01_JPRB, 0.34618E+01_JPRB, 0.33822E+01_JPRB ,&
&  0.31387E+01_JPRB, 0.27803E+01_JPRB, 0.25070E+01_JPRB, 0.23343E+01_JPRB, 0.19569E+01_JPRB, 0.17920E+01_JPRB, 0.19778E+01_JPRB ,&
&  0.17214E+01_JPRB, 0.14152E+01_JPRB, 0.12745E+01_JPRB, 0.13838E+01_JPRB, 0.18875E+01_JPRB, 0.31817E+01_JPRB, 0.28411E+01_JPRB ,&
&  0.19129E+01_JPRB, 0.15333E+01_JPRB, 0.23034E+01_JPRB, 0.34148E+01_JPRB, 0.17922E+01_JPRB, 0.10324E+01_JPRB, 0.60438E+00_JPRB ,&
&  0.54260E+00_JPRB, 0.56608E+00_JPRB, 0.59523E+00_JPRB, 0.61990E+00_JPRB, 0.65021E+00_JPRB, 0.67842E+00_JPRB, 0.70231E+00_JPRB ,&
&  0.72533E+00_JPRB, 0.75487E+00_JPRB, 0.78212E+00_JPRB, 0.81559E+00_JPRB, 0.84575E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.10217E+01_JPRB, 0.10771E+01_JPRB, 0.11196E+01_JPRB, 0.11555E+01_JPRB, 0.11883E+01_JPRB, 0.12216E+01_JPRB, 0.12610E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.13365E+01_JPRB, 0.13704E+01_JPRB, 0.14130E+01_JPRB, 0.14662E+01_JPRB, 0.15155E+01_JPRB, 0.15372E+01_JPRB ,&
&  0.15431E+01_JPRB, 0.15943E+01_JPRB, 0.16181E+01_JPRB, 0.16843E+01_JPRB, 0.17068E+01_JPRB, 0.17529E+01_JPRB, 0.17557E+01_JPRB ,&
&  0.17419E+01_JPRB, 0.17347E+01_JPRB, 0.17377E+01_JPRB, 0.17507E+01_JPRB, 0.17561E+01_JPRB, 0.18854E+01_JPRB, 0.20841E+01_JPRB ,&
&  0.21340E+01_JPRB, 0.21404E+01_JPRB, 0.15566E+01_JPRB, 0.12730E+01_JPRB, 0.12704E+01_JPRB, 0.13010E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.13537E+01_JPRB, 0.14162E+01_JPRB, 0.14477E+01_JPRB, 0.15666E+01_JPRB, 0.17737E+01_JPRB, 0.18672E+01_JPRB ,&
&  0.20083E+01_JPRB, 0.21277E+01_JPRB, 0.21964E+01_JPRB, 0.23965E+01_JPRB, 0.24266E+01_JPRB, 0.29424E+01_JPRB, 0.31998E+01_JPRB ,&
&  0.32658E+01_JPRB, 0.31059E+01_JPRB, 0.24034E+01_JPRB, 0.24063E+01_JPRB, 0.26445E+01_JPRB, 0.25711E+01_JPRB, 0.24862E+01_JPRB ,&
&  0.28905E+01_JPRB, 0.28049E+01_JPRB, 0.24515E+01_JPRB, 0.30729E+01_JPRB, 0.32412E+01_JPRB, 0.31399E+01_JPRB, 0.27530E+01_JPRB ,&
&  0.23945E+01_JPRB, 0.18509E+01_JPRB, 0.18059E+01_JPRB, 0.18076E+01_JPRB, 0.16471E+01_JPRB, 0.14865E+01_JPRB, 0.14074E+01_JPRB ,&
&  0.13226E+01_JPRB, 0.12570E+01_JPRB, 0.13074E+01_JPRB, 0.15492E+01_JPRB, 0.25346E+01_JPRB, 0.30916E+01_JPRB, 0.36279E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.15576E+01_JPRB, 0.12166E+01_JPRB, 0.15827E+01_JPRB, 0.12367E+01_JPRB, 0.74771E+00_JPRB, 0.58895E+00_JPRB ,&
&  0.60911E+00_JPRB, 0.65810E+00_JPRB, 0.70150E+00_JPRB, 0.73602E+00_JPRB, 0.76894E+00_JPRB, 0.80391E+00_JPRB, 0.82515E+00_JPRB ,&
&  0.85091E+00_JPRB, 0.88039E+00_JPRB, 0.90911E+00_JPRB, 0.95033E+00_JPRB, 0.98080E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.11204E+01_JPRB, 0.11671E+01_JPRB, 0.12086E+01_JPRB, 0.12486E+01_JPRB, 0.12930E+01_JPRB, 0.13410E+01_JPRB, 0.13853E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.14633E+01_JPRB, 0.14979E+01_JPRB, 0.15316E+01_JPRB, 0.15754E+01_JPRB, 0.16190E+01_JPRB, 0.16478E+01_JPRB ,&
&  0.16777E+01_JPRB, 0.17288E+01_JPRB, 0.17806E+01_JPRB, 0.18315E+01_JPRB, 0.18591E+01_JPRB, 0.18882E+01_JPRB, 0.18875E+01_JPRB ,&
&  0.18719E+01_JPRB, 0.18657E+01_JPRB, 0.18861E+01_JPRB, 0.19190E+01_JPRB, 0.19690E+01_JPRB, 0.19764E+01_JPRB, 0.21438E+01_JPRB ,&
&  0.19931E+01_JPRB, 0.22195E+01_JPRB, 0.17448E+01_JPRB, 0.15308E+01_JPRB, 0.15522E+01_JPRB, 0.14705E+01_JPRB, 0.14752E+01_JPRB ,&
&  0.15340E+01_JPRB, 0.15422E+01_JPRB, 0.15863E+01_JPRB, 0.16256E+01_JPRB, 0.17366E+01_JPRB, 0.19072E+01_JPRB, 0.20362E+01_JPRB ,&
&  0.22297E+01_JPRB, 0.24477E+01_JPRB, 0.26214E+01_JPRB, 0.28186E+01_JPRB, 0.32485E+01_JPRB, 0.36206E+01_JPRB, 0.38053E+01_JPRB ,&
&  0.37629E+01_JPRB, 0.35776E+01_JPRB, 0.27044E+01_JPRB, 0.27895E+01_JPRB, 0.29510E+01_JPRB, 0.27257E+01_JPRB, 0.24800E+01_JPRB ,&
&  0.25864E+01_JPRB, 0.21663E+01_JPRB, 0.24720E+01_JPRB, 0.25163E+01_JPRB, 0.23971E+01_JPRB, 0.21110E+01_JPRB, 0.17321E+01_JPRB ,&
&  0.13955E+01_JPRB, 0.11396E+01_JPRB, 0.11620E+01_JPRB, 0.13351E+01_JPRB, 0.12759E+01_JPRB, 0.10663E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.10096E+01_JPRB, 0.10992E+01_JPRB, 0.14530E+01_JPRB, 0.21798E+01_JPRB, 0.37874E+01_JPRB, 0.33822E+01_JPRB ,&
&  0.14999E+01_JPRB, 0.13337E+01_JPRB, 0.75035E+00_JPRB, 0.77981E+00_JPRB, 0.79981E+00_JPRB, 0.64773E+00_JPRB, 0.63754E+00_JPRB ,&
&  0.74075E+00_JPRB, 0.84813E+00_JPRB, 0.87416E+00_JPRB, 0.83245E+00_JPRB, 0.82917E+00_JPRB, 0.85317E+00_JPRB, 0.88732E+00_JPRB ,&
&  0.92259E+00_JPRB, 0.95675E+00_JPRB, 0.99255E+00_JPRB, 0.10323E+01_JPRB, 0.10739E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.11658E+01_JPRB, 0.12136E+01_JPRB, 0.12567E+01_JPRB, 0.12965E+01_JPRB, 0.13443E+01_JPRB, 0.13964E+01_JPRB, 0.14468E+01_JPRB ,&
&  0.14928E+01_JPRB, 0.15356E+01_JPRB, 0.15819E+01_JPRB, 0.16267E+01_JPRB, 0.16708E+01_JPRB, 0.17180E+01_JPRB, 0.17752E+01_JPRB ,&
&  0.18294E+01_JPRB, 0.18714E+01_JPRB, 0.19369E+01_JPRB, 0.20137E+01_JPRB, 0.20680E+01_JPRB, 0.21058E+01_JPRB, 0.21264E+01_JPRB ,&
&  0.21491E+01_JPRB, 0.22063E+01_JPRB, 0.22442E+01_JPRB, 0.22737E+01_JPRB, 0.23276E+01_JPRB, 0.25406E+01_JPRB, 0.23063E+01_JPRB ,&
&  0.25495E+01_JPRB, 0.22140E+01_JPRB, 0.18699E+01_JPRB, 0.17618E+01_JPRB, 0.17381E+01_JPRB, 0.17308E+01_JPRB, 0.17361E+01_JPRB ,&
&  0.17743E+01_JPRB, 0.17195E+01_JPRB, 0.17201E+01_JPRB, 0.17431E+01_JPRB, 0.18374E+01_JPRB, 0.19883E+01_JPRB, 0.21505E+01_JPRB ,&
&  0.24058E+01_JPRB, 0.27286E+01_JPRB, 0.31022E+01_JPRB, 0.35066E+01_JPRB, 0.39876E+01_JPRB, 0.43968E+01_JPRB, 0.46501E+01_JPRB ,&
&  0.46984E+01_JPRB, 0.46380E+01_JPRB, 0.39222E+01_JPRB, 0.41366E+01_JPRB, 0.40694E+01_JPRB, 0.33499E+01_JPRB, 0.27347E+01_JPRB ,&
&  0.22568E+01_JPRB, 0.17417E+01_JPRB, 0.18192E+01_JPRB, 0.18212E+01_JPRB, 0.16419E+01_JPRB, 0.13728E+01_JPRB, 0.11222E+01_JPRB ,&
&  0.84817E+00_JPRB, 0.75201E+00_JPRB, 0.75534E+00_JPRB, 0.90759E+00_JPRB, 0.91234E+00_JPRB, 0.80434E+00_JPRB, 0.82283E+00_JPRB ,&
&  0.84504E+00_JPRB, 0.86017E+00_JPRB, 0.97565E+00_JPRB, 0.12886E+01_JPRB, 0.17631E+01_JPRB, 0.17792E+01_JPRB, 0.18196E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.10217E+01_JPRB, 0.93275E+00_JPRB, 0.75649E+00_JPRB, 0.78291E+00_JPRB, 0.76974E+00_JPRB, 0.86279E+00_JPRB ,&
&  0.10226E+01_JPRB, 0.12402E+01_JPRB, 0.12131E+01_JPRB, 0.10164E+01_JPRB, 0.87785E+00_JPRB, 0.86277E+00_JPRB, 0.89775E+00_JPRB ,&
&  0.93641E+00_JPRB, 0.97276E+00_JPRB, 0.10170E+01_JPRB, 0.10650E+01_JPRB, 0.11145E+01_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.11733E+01_JPRB, 0.12190E+01_JPRB, 0.12586E+01_JPRB, 0.12943E+01_JPRB, 0.13421E+01_JPRB, 0.13924E+01_JPRB, 0.14447E+01_JPRB ,&
&  0.14958E+01_JPRB, 0.15473E+01_JPRB, 0.16024E+01_JPRB, 0.16562E+01_JPRB, 0.17163E+01_JPRB, 0.17787E+01_JPRB, 0.18465E+01_JPRB ,&
&  0.19116E+01_JPRB, 0.19745E+01_JPRB, 0.20337E+01_JPRB, 0.21392E+01_JPRB, 0.22579E+01_JPRB, 0.23478E+01_JPRB, 0.24130E+01_JPRB ,&
&  0.24475E+01_JPRB, 0.25193E+01_JPRB, 0.26052E+01_JPRB, 0.26757E+01_JPRB, 0.27015E+01_JPRB, 0.28806E+01_JPRB, 0.26173E+01_JPRB ,&
&  0.25666E+01_JPRB, 0.22890E+01_JPRB, 0.20298E+01_JPRB, 0.19804E+01_JPRB, 0.19441E+01_JPRB, 0.19387E+01_JPRB, 0.19269E+01_JPRB ,&
&  0.19056E+01_JPRB, 0.18910E+01_JPRB, 0.18449E+01_JPRB, 0.18392E+01_JPRB, 0.18910E+01_JPRB, 0.20319E+01_JPRB, 0.22233E+01_JPRB ,&
&  0.24834E+01_JPRB, 0.28351E+01_JPRB, 0.32590E+01_JPRB, 0.37224E+01_JPRB, 0.42510E+01_JPRB, 0.47286E+01_JPRB, 0.51725E+01_JPRB ,&
&  0.55059E+01_JPRB, 0.56966E+01_JPRB, 0.52160E+01_JPRB, 0.51659E+01_JPRB, 0.48564E+01_JPRB, 0.33308E+01_JPRB, 0.23954E+01_JPRB ,&
&  0.16947E+01_JPRB, 0.15948E+01_JPRB, 0.16213E+01_JPRB, 0.14786E+01_JPRB, 0.11640E+01_JPRB, 0.98498E+00_JPRB, 0.79145E+00_JPRB ,&
&  0.63914E+00_JPRB, 0.58806E+00_JPRB, 0.60410E+00_JPRB, 0.68581E+00_JPRB, 0.68400E+00_JPRB, 0.64367E+00_JPRB, 0.69149E+00_JPRB ,&
&  0.72525E+00_JPRB, 0.80640E+00_JPRB, 0.94030E+00_JPRB, 0.11527E+01_JPRB, 0.13644E+01_JPRB, 0.13692E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.11234E+01_JPRB, 0.96989E+00_JPRB, 0.69902E+00_JPRB, 0.85653E+00_JPRB, 0.97160E+00_JPRB, 0.11078E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.16405E+01_JPRB, 0.18513E+01_JPRB, 0.14637E+01_JPRB, 0.11725E+01_JPRB, 0.91014E+00_JPRB, 0.89608E+00_JPRB ,&
&  0.93952E+00_JPRB, 0.96479E+00_JPRB, 0.10092E+01_JPRB, 0.10625E+01_JPRB, 0.11197E+01_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.11140E+01_JPRB, 0.11633E+01_JPRB, 0.11977E+01_JPRB, 0.12145E+01_JPRB, 0.12548E+01_JPRB, 0.13160E+01_JPRB, 0.13639E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14629E+01_JPRB, 0.15216E+01_JPRB, 0.15772E+01_JPRB, 0.16363E+01_JPRB, 0.16977E+01_JPRB, 0.17684E+01_JPRB ,&
&  0.18351E+01_JPRB, 0.19377E+01_JPRB, 0.20114E+01_JPRB, 0.21323E+01_JPRB, 0.22636E+01_JPRB, 0.24224E+01_JPRB, 0.25218E+01_JPRB ,&
&  0.25598E+01_JPRB, 0.26466E+01_JPRB, 0.27500E+01_JPRB, 0.28666E+01_JPRB, 0.28768E+01_JPRB, 0.29119E+01_JPRB, 0.22560E+01_JPRB ,&
&  0.24178E+01_JPRB, 0.23156E+01_JPRB, 0.21982E+01_JPRB, 0.21493E+01_JPRB, 0.20854E+01_JPRB, 0.20248E+01_JPRB, 0.19608E+01_JPRB ,&
&  0.19446E+01_JPRB, 0.18957E+01_JPRB, 0.17862E+01_JPRB, 0.18560E+01_JPRB, 0.19073E+01_JPRB, 0.20064E+01_JPRB, 0.21994E+01_JPRB ,&
&  0.24127E+01_JPRB, 0.27227E+01_JPRB, 0.31249E+01_JPRB, 0.36046E+01_JPRB, 0.41312E+01_JPRB, 0.46582E+01_JPRB, 0.51774E+01_JPRB ,&
&  0.56622E+01_JPRB, 0.60470E+01_JPRB, 0.61068E+01_JPRB, 0.50049E+01_JPRB, 0.41635E+01_JPRB, 0.29061E+01_JPRB, 0.21726E+01_JPRB ,&
&  0.17707E+01_JPRB, 0.15914E+01_JPRB, 0.15033E+01_JPRB, 0.13286E+01_JPRB, 0.10007E+01_JPRB, 0.82421E+00_JPRB, 0.66297E+00_JPRB ,&
&  0.60085E+00_JPRB, 0.56283E+00_JPRB, 0.56829E+00_JPRB, 0.57746E+00_JPRB, 0.58602E+00_JPRB, 0.58358E+00_JPRB, 0.62248E+00_JPRB ,&
&  0.66562E+00_JPRB, 0.75387E+00_JPRB, 0.91341E+00_JPRB, 0.11445E+01_JPRB, 0.13621E+01_JPRB, 0.14652E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.97191E+00_JPRB, 0.90663E+00_JPRB, 0.92365E+00_JPRB, 0.94342E+00_JPRB, 0.11265E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.15990E+01_JPRB, 0.16099E+01_JPRB, 0.18803E+01_JPRB, 0.24334E+01_JPRB, 0.21912E+01_JPRB, 0.19128E+01_JPRB, 0.81073E+00_JPRB ,&
&  0.87240E+00_JPRB, 0.90035E+00_JPRB, 0.95435E+00_JPRB, 0.10092E+01_JPRB, 0.10765E+01_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.88693E+00_JPRB, 0.93213E+00_JPRB, 0.97399E+00_JPRB, 0.99571E+00_JPRB, 0.10526E+01_JPRB, 0.11173E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.12135E+01_JPRB, 0.12772E+01_JPRB, 0.13327E+01_JPRB, 0.14003E+01_JPRB, 0.14595E+01_JPRB, 0.15177E+01_JPRB, 0.15801E+01_JPRB ,&
&  0.16410E+01_JPRB, 0.17294E+01_JPRB, 0.18200E+01_JPRB, 0.19458E+01_JPRB, 0.20786E+01_JPRB, 0.22419E+01_JPRB, 0.23935E+01_JPRB ,&
&  0.25117E+01_JPRB, 0.26296E+01_JPRB, 0.27676E+01_JPRB, 0.28631E+01_JPRB, 0.28383E+01_JPRB, 0.27794E+01_JPRB, 0.20284E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.23431E+01_JPRB, 0.23308E+01_JPRB, 0.22745E+01_JPRB, 0.21826E+01_JPRB, 0.20736E+01_JPRB, 0.20069E+01_JPRB ,&
&  0.19491E+01_JPRB, 0.18543E+01_JPRB, 0.17471E+01_JPRB, 0.16815E+01_JPRB, 0.18744E+01_JPRB, 0.19608E+01_JPRB, 0.21235E+01_JPRB ,&
&  0.23413E+01_JPRB, 0.26123E+01_JPRB, 0.29440E+01_JPRB, 0.34175E+01_JPRB, 0.39403E+01_JPRB, 0.44599E+01_JPRB, 0.49955E+01_JPRB ,&
&  0.54155E+01_JPRB, 0.56309E+01_JPRB, 0.56822E+01_JPRB, 0.45222E+01_JPRB, 0.37115E+01_JPRB, 0.28812E+01_JPRB, 0.20333E+01_JPRB ,&
&  0.16768E+01_JPRB, 0.15387E+01_JPRB, 0.14646E+01_JPRB, 0.12640E+01_JPRB, 0.97885E+00_JPRB, 0.77581E+00_JPRB, 0.63441E+00_JPRB ,&
&  0.58089E+00_JPRB, 0.54216E+00_JPRB, 0.54519E+00_JPRB, 0.55677E+00_JPRB, 0.57678E+00_JPRB, 0.59325E+00_JPRB, 0.62572E+00_JPRB ,&
&  0.67228E+00_JPRB, 0.74251E+00_JPRB, 0.85547E+00_JPRB, 0.10421E+01_JPRB, 0.13078E+01_JPRB, 0.15991E+01_JPRB, 0.15369E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.98589E+00_JPRB, 0.88956E+00_JPRB, 0.83990E+00_JPRB, 0.84354E+00_JPRB, 0.89731E+00_JPRB, 0.96781E+00_JPRB ,&
&  0.11015E+01_JPRB, 0.11749E+01_JPRB, 0.10486E+01_JPRB, 0.87471E+00_JPRB, 0.11235E+01_JPRB, 0.11228E+01_JPRB, 0.65740E+00_JPRB ,&
&  0.72162E+00_JPRB, 0.71086E+00_JPRB, 0.75006E+00_JPRB, 0.79742E+00_JPRB, 0.85485E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.67304E+00_JPRB, 0.74099E+00_JPRB, 0.77654E+00_JPRB, 0.82124E+00_JPRB, 0.86131E+00_JPRB, 0.91904E+00_JPRB, 0.97619E+00_JPRB ,&
&  0.10348E+01_JPRB, 0.11004E+01_JPRB, 0.11547E+01_JPRB, 0.12197E+01_JPRB, 0.12783E+01_JPRB, 0.13321E+01_JPRB, 0.13864E+01_JPRB ,&
&  0.14379E+01_JPRB, 0.15132E+01_JPRB, 0.15901E+01_JPRB, 0.16790E+01_JPRB, 0.17753E+01_JPRB, 0.19011E+01_JPRB, 0.20753E+01_JPRB ,&
&  0.22866E+01_JPRB, 0.24934E+01_JPRB, 0.26811E+01_JPRB, 0.28394E+01_JPRB, 0.29287E+01_JPRB, 0.30499E+01_JPRB, 0.30847E+01_JPRB ,&
&  0.20525E+01_JPRB, 0.25181E+01_JPRB, 0.24228E+01_JPRB, 0.22986E+01_JPRB, 0.22001E+01_JPRB, 0.21036E+01_JPRB, 0.20575E+01_JPRB ,&
&  0.19938E+01_JPRB, 0.18133E+01_JPRB, 0.17462E+01_JPRB, 0.16657E+01_JPRB, 0.17486E+01_JPRB, 0.18511E+01_JPRB, 0.20061E+01_JPRB ,&
&  0.22233E+01_JPRB, 0.25059E+01_JPRB, 0.28126E+01_JPRB, 0.31429E+01_JPRB, 0.35898E+01_JPRB, 0.41393E+01_JPRB, 0.46427E+01_JPRB ,&
&  0.51714E+01_JPRB, 0.57071E+01_JPRB, 0.60322E+01_JPRB, 0.50701E+01_JPRB, 0.41373E+01_JPRB, 0.36585E+01_JPRB, 0.26371E+01_JPRB ,&
&  0.17331E+01_JPRB, 0.13917E+01_JPRB, 0.13641E+01_JPRB, 0.12697E+01_JPRB, 0.99293E+00_JPRB, 0.80931E+00_JPRB, 0.68488E+00_JPRB ,&
&  0.62661E+00_JPRB, 0.57737E+00_JPRB, 0.56609E+00_JPRB, 0.57727E+00_JPRB, 0.59658E+00_JPRB, 0.62086E+00_JPRB, 0.66887E+00_JPRB ,&
&  0.70638E+00_JPRB, 0.74174E+00_JPRB, 0.78410E+00_JPRB, 0.82556E+00_JPRB, 0.90563E+00_JPRB, 0.10375E+01_JPRB, 0.10533E+01_JPRB ,&
&  0.99865E+00_JPRB, 0.94953E+00_JPRB, 0.89605E+00_JPRB, 0.87291E+00_JPRB, 0.82960E+00_JPRB, 0.80708E+00_JPRB, 0.81361E+00_JPRB ,&
&  0.82949E+00_JPRB, 0.83683E+00_JPRB, 0.78573E+00_JPRB, 0.74460E+00_JPRB, 0.72319E+00_JPRB, 0.68969E+00_JPRB, 0.63682E+00_JPRB ,&
&  0.61817E+00_JPRB, 0.60204E+00_JPRB, 0.58341E+00_JPRB, 0.59592E+00_JPRB, 0.62991E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.57369E+00_JPRB, 0.59863E+00_JPRB, 0.63057E+00_JPRB, 0.66519E+00_JPRB, 0.69846E+00_JPRB, 0.73255E+00_JPRB, 0.78347E+00_JPRB ,&
&  0.84338E+00_JPRB, 0.89253E+00_JPRB, 0.92395E+00_JPRB, 0.97068E+00_JPRB, 0.10265E+01_JPRB, 0.10945E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.12287E+01_JPRB, 0.13122E+01_JPRB, 0.13817E+01_JPRB, 0.14531E+01_JPRB, 0.15274E+01_JPRB, 0.16099E+01_JPRB, 0.17337E+01_JPRB ,&
&  0.18943E+01_JPRB, 0.21050E+01_JPRB, 0.23691E+01_JPRB, 0.26291E+01_JPRB, 0.28695E+01_JPRB, 0.31046E+01_JPRB, 0.31929E+01_JPRB ,&
&  0.18249E+01_JPRB, 0.24133E+01_JPRB, 0.27692E+01_JPRB, 0.24544E+01_JPRB, 0.22157E+01_JPRB, 0.20693E+01_JPRB, 0.19873E+01_JPRB ,&
&  0.18572E+01_JPRB, 0.16850E+01_JPRB, 0.16195E+01_JPRB, 0.17059E+01_JPRB, 0.16335E+01_JPRB, 0.17022E+01_JPRB, 0.18002E+01_JPRB ,&
&  0.19655E+01_JPRB, 0.21833E+01_JPRB, 0.23911E+01_JPRB, 0.26375E+01_JPRB, 0.29950E+01_JPRB, 0.34434E+01_JPRB, 0.39919E+01_JPRB ,&
&  0.46858E+01_JPRB, 0.54938E+01_JPRB, 0.62458E+01_JPRB, 0.49945E+01_JPRB, 0.45161E+01_JPRB, 0.42425E+01_JPRB, 0.36013E+01_JPRB ,&
&  0.20955E+01_JPRB, 0.14073E+01_JPRB, 0.13717E+01_JPRB, 0.12979E+01_JPRB, 0.10806E+01_JPRB, 0.92687E+00_JPRB, 0.82277E+00_JPRB ,&
&  0.74653E+00_JPRB, 0.68837E+00_JPRB, 0.65862E+00_JPRB, 0.65291E+00_JPRB, 0.66061E+00_JPRB, 0.69142E+00_JPRB, 0.73108E+00_JPRB ,&
&  0.76564E+00_JPRB, 0.77366E+00_JPRB, 0.81316E+00_JPRB, 0.83135E+00_JPRB, 0.88264E+00_JPRB, 0.92640E+00_JPRB, 0.95665E+00_JPRB ,&
&  0.96697E+00_JPRB, 0.95510E+00_JPRB, 0.94265E+00_JPRB, 0.93051E+00_JPRB, 0.92000E+00_JPRB, 0.90636E+00_JPRB, 0.89045E+00_JPRB ,&
&  0.86864E+00_JPRB, 0.86736E+00_JPRB, 0.84082E+00_JPRB, 0.81825E+00_JPRB, 0.81047E+00_JPRB, 0.76651E+00_JPRB, 0.74320E+00_JPRB ,&
&  0.72216E+00_JPRB, 0.66790E+00_JPRB, 0.63206E+00_JPRB, 0.60387E+00_JPRB, 0.58012E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.61463E+00_JPRB, 0.57638E+00_JPRB, 0.58084E+00_JPRB, 0.58998E+00_JPRB, 0.61519E+00_JPRB, 0.64184E+00_JPRB, 0.63925E+00_JPRB ,&
&  0.65428E+00_JPRB, 0.67813E+00_JPRB, 0.71139E+00_JPRB, 0.74373E+00_JPRB, 0.78181E+00_JPRB, 0.82546E+00_JPRB, 0.87559E+00_JPRB ,&
&  0.94019E+00_JPRB, 0.10186E+01_JPRB, 0.11030E+01_JPRB, 0.11805E+01_JPRB, 0.12577E+01_JPRB, 0.13215E+01_JPRB, 0.13867E+01_JPRB ,&
&  0.14756E+01_JPRB, 0.16159E+01_JPRB, 0.18274E+01_JPRB, 0.20966E+01_JPRB, 0.24469E+01_JPRB, 0.28666E+01_JPRB, 0.31652E+01_JPRB ,&
&  0.21926E+01_JPRB, 0.10614E+01_JPRB, 0.25727E+01_JPRB, 0.28425E+01_JPRB, 0.24189E+01_JPRB, 0.22250E+01_JPRB, 0.19920E+01_JPRB ,&
&  0.18370E+01_JPRB, 0.17887E+01_JPRB, 0.16422E+01_JPRB, 0.16236E+01_JPRB, 0.15487E+01_JPRB, 0.15688E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.16551E+01_JPRB, 0.17767E+01_JPRB, 0.19853E+01_JPRB, 0.21149E+01_JPRB, 0.23085E+01_JPRB, 0.26438E+01_JPRB, 0.31439E+01_JPRB ,&
&  0.38475E+01_JPRB, 0.47108E+01_JPRB, 0.55810E+01_JPRB, 0.43185E+01_JPRB, 0.33433E+01_JPRB, 0.26472E+01_JPRB, 0.21601E+01_JPRB ,&
&  0.18587E+01_JPRB, 0.14546E+01_JPRB, 0.13656E+01_JPRB, 0.13107E+01_JPRB, 0.11725E+01_JPRB, 0.10004E+01_JPRB, 0.96456E+00_JPRB ,&
&  0.88919E+00_JPRB, 0.83373E+00_JPRB, 0.80004E+00_JPRB, 0.77756E+00_JPRB, 0.77320E+00_JPRB, 0.79261E+00_JPRB, 0.81471E+00_JPRB ,&
&  0.84243E+00_JPRB, 0.86238E+00_JPRB, 0.89356E+00_JPRB, 0.92988E+00_JPRB, 0.93973E+00_JPRB, 0.94209E+00_JPRB, 0.95895E+00_JPRB ,&
&  0.95867E+00_JPRB, 0.96863E+00_JPRB, 0.98048E+00_JPRB, 0.98510E+00_JPRB, 0.98572E+00_JPRB, 0.97039E+00_JPRB, 0.97471E+00_JPRB ,&
&  0.96702E+00_JPRB, 0.95805E+00_JPRB, 0.93329E+00_JPRB, 0.93054E+00_JPRB, 0.92755E+00_JPRB, 0.91421E+00_JPRB, 0.92062E+00_JPRB ,&
&  0.90451E+00_JPRB, 0.86456E+00_JPRB, 0.79213E+00_JPRB, 0.71496E+00_JPRB, 0.66946E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.75815E+00_JPRB, 0.69551E+00_JPRB, 0.65005E+00_JPRB, 0.62606E+00_JPRB, 0.61160E+00_JPRB, 0.60729E+00_JPRB, 0.60672E+00_JPRB ,&
&  0.61267E+00_JPRB, 0.61928E+00_JPRB, 0.61118E+00_JPRB, 0.62129E+00_JPRB, 0.64269E+00_JPRB, 0.67111E+00_JPRB, 0.70755E+00_JPRB ,&
&  0.73732E+00_JPRB, 0.77098E+00_JPRB, 0.80846E+00_JPRB, 0.85868E+00_JPRB, 0.92094E+00_JPRB, 0.99068E+00_JPRB, 0.10596E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.12143E+01_JPRB, 0.13359E+01_JPRB, 0.15127E+01_JPRB, 0.17624E+01_JPRB, 0.21391E+01_JPRB, 0.25782E+01_JPRB ,&
&  0.32116E+01_JPRB, 0.22873E+01_JPRB, 0.15082E+01_JPRB, 0.25870E+01_JPRB, 0.27094E+01_JPRB, 0.25463E+01_JPRB, 0.21080E+01_JPRB ,&
&  0.20827E+01_JPRB, 0.20758E+01_JPRB, 0.18346E+01_JPRB, 0.16776E+01_JPRB, 0.14938E+01_JPRB, 0.14675E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.14384E+01_JPRB, 0.15659E+01_JPRB, 0.16655E+01_JPRB, 0.17485E+01_JPRB, 0.19611E+01_JPRB, 0.24012E+01_JPRB ,&
&  0.30084E+01_JPRB, 0.38050E+01_JPRB, 0.47061E+01_JPRB, 0.39348E+01_JPRB, 0.29338E+01_JPRB, 0.21966E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.17402E+01_JPRB, 0.16179E+01_JPRB, 0.14662E+01_JPRB, 0.13352E+01_JPRB, 0.11996E+01_JPRB, 0.10715E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.10097E+01_JPRB, 0.94682E+00_JPRB, 0.90134E+00_JPRB, 0.86211E+00_JPRB, 0.85270E+00_JPRB, 0.86365E+00_JPRB, 0.87767E+00_JPRB ,&
&  0.89530E+00_JPRB, 0.91763E+00_JPRB, 0.95204E+00_JPRB, 0.99204E+00_JPRB, 0.10027E+01_JPRB, 0.99258E+00_JPRB, 0.98693E+00_JPRB ,&
&  0.97956E+00_JPRB, 0.97711E+00_JPRB, 0.99354E+00_JPRB, 0.10015E+01_JPRB, 0.99702E+00_JPRB, 0.97948E+00_JPRB, 0.99385E+00_JPRB ,&
&  0.99332E+00_JPRB, 0.10010E+01_JPRB, 0.98879E+00_JPRB, 0.99598E+00_JPRB, 0.99508E+00_JPRB, 0.10146E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10316E+01_JPRB, 0.98049E+00_JPRB, 0.85854E+00_JPRB, 0.80550E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.82473E+00_JPRB, 0.78461E+00_JPRB, 0.75404E+00_JPRB, 0.72684E+00_JPRB, 0.69980E+00_JPRB, 0.67432E+00_JPRB, 0.65411E+00_JPRB ,&
&  0.63092E+00_JPRB, 0.62807E+00_JPRB, 0.62563E+00_JPRB, 0.61972E+00_JPRB, 0.61224E+00_JPRB, 0.61671E+00_JPRB, 0.63014E+00_JPRB ,&
&  0.64043E+00_JPRB, 0.65038E+00_JPRB, 0.66574E+00_JPRB, 0.67541E+00_JPRB, 0.68855E+00_JPRB, 0.71460E+00_JPRB, 0.75554E+00_JPRB ,&
&  0.81279E+00_JPRB, 0.87899E+00_JPRB, 0.96668E+00_JPRB, 0.10775E+01_JPRB, 0.11931E+01_JPRB, 0.13759E+01_JPRB, 0.16039E+01_JPRB ,&
&  0.21640E+01_JPRB, 0.22427E+01_JPRB, 0.12445E+01_JPRB, 0.22512E+01_JPRB, 0.28673E+01_JPRB, 0.27119E+01_JPRB, 0.25052E+01_JPRB ,&
&  0.25039E+01_JPRB, 0.23450E+01_JPRB, 0.20906E+01_JPRB, 0.17614E+01_JPRB, 0.14850E+01_JPRB, 0.14058E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.13058E+01_JPRB, 0.12604E+01_JPRB, 0.12796E+01_JPRB, 0.13075E+01_JPRB, 0.13440E+01_JPRB, 0.14342E+01_JPRB, 0.17357E+01_JPRB ,&
&  0.22262E+01_JPRB, 0.29767E+01_JPRB, 0.38277E+01_JPRB, 0.36085E+01_JPRB, 0.27127E+01_JPRB, 0.22915E+01_JPRB, 0.20830E+01_JPRB ,&
&  0.20013E+01_JPRB, 0.18484E+01_JPRB, 0.15361E+01_JPRB, 0.13477E+01_JPRB, 0.12062E+01_JPRB, 0.10992E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10165E+01_JPRB, 0.98211E+00_JPRB, 0.93573E+00_JPRB, 0.91026E+00_JPRB, 0.90098E+00_JPRB, 0.90675E+00_JPRB ,&
&  0.91889E+00_JPRB, 0.93284E+00_JPRB, 0.95357E+00_JPRB, 0.98110E+00_JPRB, 0.10000E+01_JPRB, 0.10077E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.99809E+00_JPRB, 0.99865E+00_JPRB, 0.10035E+01_JPRB, 0.10047E+01_JPRB, 0.10006E+01_JPRB, 0.99353E+00_JPRB, 0.98442E+00_JPRB ,&
&  0.98026E+00_JPRB, 0.98010E+00_JPRB, 0.96651E+00_JPRB, 0.95662E+00_JPRB, 0.98954E+00_JPRB, 0.99922E+00_JPRB, 0.10077E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10704E+01_JPRB, 0.10484E+01_JPRB, 0.92552E+00_JPRB, 0.86841E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.85643E+00_JPRB, 0.82356E+00_JPRB, 0.79812E+00_JPRB, 0.77857E+00_JPRB, 0.75530E+00_JPRB, 0.73254E+00_JPRB, 0.71287E+00_JPRB ,&
&  0.68768E+00_JPRB, 0.67254E+00_JPRB, 0.66294E+00_JPRB, 0.65251E+00_JPRB, 0.64175E+00_JPRB, 0.63189E+00_JPRB, 0.62403E+00_JPRB ,&
&  0.61617E+00_JPRB, 0.60961E+00_JPRB, 0.60629E+00_JPRB, 0.59966E+00_JPRB, 0.59425E+00_JPRB, 0.59530E+00_JPRB, 0.60537E+00_JPRB ,&
&  0.63529E+00_JPRB, 0.68184E+00_JPRB, 0.74145E+00_JPRB, 0.80939E+00_JPRB, 0.86940E+00_JPRB, 0.95042E+00_JPRB, 0.10501E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.17454E+01_JPRB, 0.20343E+01_JPRB, 0.33279E+01_JPRB, 0.34065E+01_JPRB, 0.30439E+01_JPRB, 0.30179E+01_JPRB ,&
&  0.30940E+01_JPRB, 0.28827E+01_JPRB, 0.26156E+01_JPRB, 0.20307E+01_JPRB, 0.16201E+01_JPRB, 0.14062E+01_JPRB, 0.12921E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.11295E+01_JPRB, 0.11192E+01_JPRB, 0.11076E+01_JPRB, 0.11200E+01_JPRB, 0.12319E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.16983E+01_JPRB, 0.22479E+01_JPRB, 0.30173E+01_JPRB, 0.35095E+01_JPRB, 0.27321E+01_JPRB, 0.25223E+01_JPRB, 0.25427E+01_JPRB ,&
&  0.30339E+01_JPRB, 0.23478E+01_JPRB, 0.17329E+01_JPRB, 0.13977E+01_JPRB, 0.12091E+01_JPRB, 0.11350E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.10292E+01_JPRB, 0.10053E+01_JPRB, 0.97858E+00_JPRB, 0.95908E+00_JPRB, 0.93993E+00_JPRB, 0.92973E+00_JPRB ,&
&  0.92753E+00_JPRB, 0.93147E+00_JPRB, 0.93718E+00_JPRB, 0.94969E+00_JPRB, 0.97104E+00_JPRB, 0.98820E+00_JPRB, 0.99504E+00_JPRB ,&
&  0.99923E+00_JPRB, 0.10053E+01_JPRB, 0.98081E+00_JPRB, 0.97594E+00_JPRB, 0.98332E+00_JPRB, 0.97723E+00_JPRB, 0.94591E+00_JPRB ,&
&  0.93309E+00_JPRB, 0.94727E+00_JPRB, 0.94725E+00_JPRB, 0.93713E+00_JPRB, 0.94776E+00_JPRB, 0.95703E+00_JPRB, 0.93557E+00_JPRB ,&
&  0.93976E+00_JPRB, 0.95055E+00_JPRB, 0.94140E+00_JPRB, 0.91732E+00_JPRB, 0.89021E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.87144E+00_JPRB, 0.84491E+00_JPRB, 0.81862E+00_JPRB, 0.79411E+00_JPRB, 0.76717E+00_JPRB, 0.74531E+00_JPRB, 0.73238E+00_JPRB ,&
&  0.71356E+00_JPRB, 0.69752E+00_JPRB, 0.68621E+00_JPRB, 0.68047E+00_JPRB, 0.67565E+00_JPRB, 0.66331E+00_JPRB, 0.64062E+00_JPRB ,&
&  0.61780E+00_JPRB, 0.60188E+00_JPRB, 0.59198E+00_JPRB, 0.58212E+00_JPRB, 0.57253E+00_JPRB, 0.56784E+00_JPRB, 0.57147E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.60247E+00_JPRB, 0.64084E+00_JPRB, 0.68601E+00_JPRB, 0.72729E+00_JPRB, 0.76858E+00_JPRB, 0.80059E+00_JPRB ,&
&  0.10367E+01_JPRB, 0.14752E+01_JPRB, 0.14590E+01_JPRB, 0.28212E+01_JPRB, 0.30291E+01_JPRB, 0.29756E+01_JPRB, 0.30011E+01_JPRB ,&
&  0.30361E+01_JPRB, 0.33580E+01_JPRB, 0.28534E+01_JPRB, 0.23106E+01_JPRB, 0.18472E+01_JPRB, 0.15441E+01_JPRB, 0.13326E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.10975E+01_JPRB, 0.10470E+01_JPRB, 0.10296E+01_JPRB, 0.10271E+01_JPRB, 0.10827E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.18238E+01_JPRB, 0.23816E+01_JPRB, 0.27696E+01_JPRB, 0.26482E+01_JPRB, 0.26750E+01_JPRB, 0.29678E+01_JPRB ,&
&  0.37228E+01_JPRB, 0.31781E+01_JPRB, 0.20893E+01_JPRB, 0.15128E+01_JPRB, 0.12558E+01_JPRB, 0.11515E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10170E+01_JPRB, 0.99408E+00_JPRB, 0.97956E+00_JPRB, 0.96875E+00_JPRB, 0.96384E+00_JPRB, 0.95577E+00_JPRB ,&
&  0.94457E+00_JPRB, 0.93773E+00_JPRB, 0.93378E+00_JPRB, 0.93234E+00_JPRB, 0.93719E+00_JPRB, 0.95099E+00_JPRB, 0.96686E+00_JPRB ,&
&  0.98088E+00_JPRB, 0.99230E+00_JPRB, 0.97131E+00_JPRB, 0.95184E+00_JPRB, 0.95216E+00_JPRB, 0.94393E+00_JPRB, 0.91819E+00_JPRB ,&
&  0.92182E+00_JPRB, 0.93559E+00_JPRB, 0.93220E+00_JPRB, 0.91129E+00_JPRB, 0.91768E+00_JPRB, 0.93818E+00_JPRB, 0.90668E+00_JPRB ,&
&  0.89986E+00_JPRB, 0.90481E+00_JPRB, 0.91080E+00_JPRB, 0.90078E+00_JPRB, 0.88781E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.85139E+00_JPRB, 0.83550E+00_JPRB, 0.81429E+00_JPRB, 0.79078E+00_JPRB, 0.76680E+00_JPRB, 0.74611E+00_JPRB, 0.73420E+00_JPRB ,&
&  0.71988E+00_JPRB, 0.70536E+00_JPRB, 0.69261E+00_JPRB, 0.68487E+00_JPRB, 0.68117E+00_JPRB, 0.66904E+00_JPRB, 0.64733E+00_JPRB ,&
&  0.62441E+00_JPRB, 0.60421E+00_JPRB, 0.58704E+00_JPRB, 0.57592E+00_JPRB, 0.56446E+00_JPRB, 0.55944E+00_JPRB, 0.55961E+00_JPRB ,&
&  0.56265E+00_JPRB, 0.57297E+00_JPRB, 0.59571E+00_JPRB, 0.63035E+00_JPRB, 0.66565E+00_JPRB, 0.69623E+00_JPRB, 0.71680E+00_JPRB ,&
&  0.85147E+00_JPRB, 0.17457E+01_JPRB, 0.13573E+01_JPRB, 0.21671E+01_JPRB, 0.23707E+01_JPRB, 0.25217E+01_JPRB, 0.27245E+01_JPRB ,&
&  0.28131E+01_JPRB, 0.29481E+01_JPRB, 0.27601E+01_JPRB, 0.24099E+01_JPRB, 0.20089E+01_JPRB, 0.17265E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.13407E+01_JPRB, 0.12263E+01_JPRB, 0.11418E+01_JPRB, 0.10780E+01_JPRB, 0.10380E+01_JPRB, 0.10454E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.17090E+01_JPRB, 0.20579E+01_JPRB, 0.24524E+01_JPRB, 0.23566E+01_JPRB, 0.26467E+01_JPRB, 0.35146E+01_JPRB ,&
&  0.40682E+01_JPRB, 0.40218E+01_JPRB, 0.26721E+01_JPRB, 0.17672E+01_JPRB, 0.13698E+01_JPRB, 0.11795E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.97855E+00_JPRB, 0.96054E+00_JPRB, 0.95553E+00_JPRB, 0.95920E+00_JPRB, 0.96951E+00_JPRB, 0.96976E+00_JPRB ,&
&  0.96743E+00_JPRB, 0.96306E+00_JPRB, 0.95781E+00_JPRB, 0.94727E+00_JPRB, 0.93276E+00_JPRB, 0.92795E+00_JPRB, 0.93228E+00_JPRB ,&
&  0.94532E+00_JPRB, 0.96110E+00_JPRB, 0.95162E+00_JPRB, 0.93790E+00_JPRB, 0.93064E+00_JPRB, 0.92158E+00_JPRB, 0.90651E+00_JPRB ,&
&  0.91464E+00_JPRB, 0.91934E+00_JPRB, 0.91516E+00_JPRB, 0.91329E+00_JPRB, 0.90533E+00_JPRB, 0.93179E+00_JPRB, 0.89276E+00_JPRB ,&
&  0.88191E+00_JPRB, 0.88375E+00_JPRB, 0.88466E+00_JPRB, 0.87102E+00_JPRB, 0.86071E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.80822E+00_JPRB, 0.79546E+00_JPRB, 0.77687E+00_JPRB, 0.75774E+00_JPRB, 0.74469E+00_JPRB, 0.72996E+00_JPRB, 0.71822E+00_JPRB ,&
&  0.70747E+00_JPRB, 0.69749E+00_JPRB, 0.68928E+00_JPRB, 0.68027E+00_JPRB, 0.66877E+00_JPRB, 0.65305E+00_JPRB, 0.63739E+00_JPRB ,&
&  0.62137E+00_JPRB, 0.60610E+00_JPRB, 0.59015E+00_JPRB, 0.57358E+00_JPRB, 0.55995E+00_JPRB, 0.55104E+00_JPRB, 0.54523E+00_JPRB ,&
&  0.54467E+00_JPRB, 0.54881E+00_JPRB, 0.56230E+00_JPRB, 0.58845E+00_JPRB, 0.62240E+00_JPRB, 0.65548E+00_JPRB, 0.68833E+00_JPRB ,&
&  0.82587E+00_JPRB, 0.77194E+00_JPRB, 0.15429E+01_JPRB, 0.18500E+01_JPRB, 0.18272E+01_JPRB, 0.18840E+01_JPRB, 0.22627E+01_JPRB ,&
&  0.24953E+01_JPRB, 0.25901E+01_JPRB, 0.25902E+01_JPRB, 0.24352E+01_JPRB, 0.21318E+01_JPRB, 0.19062E+01_JPRB, 0.17299E+01_JPRB ,&
&  0.15977E+01_JPRB, 0.14899E+01_JPRB, 0.13641E+01_JPRB, 0.12417E+01_JPRB, 0.11525E+01_JPRB, 0.11333E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.14562E+01_JPRB, 0.16855E+01_JPRB, 0.18687E+01_JPRB, 0.21074E+01_JPRB, 0.21714E+01_JPRB, 0.25430E+01_JPRB, 0.30051E+01_JPRB ,&
&  0.39790E+01_JPRB, 0.40947E+01_JPRB, 0.30152E+01_JPRB, 0.21396E+01_JPRB, 0.16153E+01_JPRB, 0.13359E+01_JPRB, 0.11461E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.97309E+00_JPRB, 0.93372E+00_JPRB, 0.92530E+00_JPRB, 0.94173E+00_JPRB, 0.96953E+00_JPRB, 0.99305E+00_JPRB ,&
&  0.10185E+01_JPRB, 0.10362E+01_JPRB, 0.10375E+01_JPRB, 0.10259E+01_JPRB, 0.98792E+00_JPRB, 0.94274E+00_JPRB, 0.91318E+00_JPRB ,&
&  0.90768E+00_JPRB, 0.91665E+00_JPRB, 0.91756E+00_JPRB, 0.90940E+00_JPRB, 0.90596E+00_JPRB, 0.90160E+00_JPRB, 0.89658E+00_JPRB ,&
&  0.90014E+00_JPRB, 0.90565E+00_JPRB, 0.90436E+00_JPRB, 0.91105E+00_JPRB, 0.88965E+00_JPRB, 0.91869E+00_JPRB, 0.88944E+00_JPRB ,&
&  0.87150E+00_JPRB, 0.86082E+00_JPRB, 0.85074E+00_JPRB, 0.83212E+00_JPRB, 0.81549E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.76074E+00_JPRB, 0.74808E+00_JPRB, 0.73015E+00_JPRB, 0.71316E+00_JPRB, 0.70369E+00_JPRB, 0.69714E+00_JPRB, 0.69240E+00_JPRB ,&
&  0.68904E+00_JPRB, 0.67958E+00_JPRB, 0.66954E+00_JPRB, 0.65556E+00_JPRB, 0.63749E+00_JPRB, 0.62198E+00_JPRB, 0.61206E+00_JPRB ,&
&  0.60053E+00_JPRB, 0.59450E+00_JPRB, 0.58652E+00_JPRB, 0.57171E+00_JPRB, 0.55529E+00_JPRB, 0.54170E+00_JPRB, 0.53307E+00_JPRB ,&
&  0.52887E+00_JPRB, 0.53155E+00_JPRB, 0.54004E+00_JPRB, 0.55924E+00_JPRB, 0.58051E+00_JPRB, 0.60597E+00_JPRB, 0.65123E+00_JPRB ,&
&  0.82697E+00_JPRB, 0.14548E+01_JPRB, 0.18787E+01_JPRB, 0.16847E+01_JPRB, 0.15506E+01_JPRB, 0.14478E+01_JPRB, 0.16854E+01_JPRB ,&
&  0.18876E+01_JPRB, 0.20967E+01_JPRB, 0.22208E+01_JPRB, 0.22612E+01_JPRB, 0.21089E+01_JPRB, 0.19964E+01_JPRB, 0.18843E+01_JPRB ,&
&  0.17693E+01_JPRB, 0.16381E+01_JPRB, 0.14600E+01_JPRB, 0.12874E+01_JPRB, 0.11920E+01_JPRB, 0.11837E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.14452E+01_JPRB, 0.16283E+01_JPRB, 0.17191E+01_JPRB, 0.19122E+01_JPRB, 0.22317E+01_JPRB, 0.22575E+01_JPRB, 0.29299E+01_JPRB ,&
&  0.37209E+01_JPRB, 0.33512E+01_JPRB, 0.27836E+01_JPRB, 0.22187E+01_JPRB, 0.17906E+01_JPRB, 0.14460E+01_JPRB, 0.12444E+01_JPRB ,&
&  0.11041E+01_JPRB, 0.10107E+01_JPRB, 0.94512E+00_JPRB, 0.91235E+00_JPRB, 0.92718E+00_JPRB, 0.97734E+00_JPRB, 0.10224E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.11489E+01_JPRB, 0.11895E+01_JPRB, 0.11892E+01_JPRB, 0.11418E+01_JPRB, 0.10626E+01_JPRB, 0.98480E+00_JPRB ,&
&  0.93314E+00_JPRB, 0.90171E+00_JPRB, 0.89469E+00_JPRB, 0.88452E+00_JPRB, 0.88934E+00_JPRB, 0.90144E+00_JPRB, 0.89773E+00_JPRB ,&
&  0.89376E+00_JPRB, 0.89929E+00_JPRB, 0.91548E+00_JPRB, 0.90515E+00_JPRB, 0.88670E+00_JPRB, 0.92168E+00_JPRB, 0.89169E+00_JPRB ,&
&  0.86349E+00_JPRB, 0.83713E+00_JPRB, 0.81597E+00_JPRB, 0.78539E+00_JPRB, 0.76785E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.73289E+00_JPRB, 0.71458E+00_JPRB, 0.68925E+00_JPRB, 0.66834E+00_JPRB, 0.65698E+00_JPRB, 0.65136E+00_JPRB, 0.65059E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.64303E+00_JPRB, 0.63223E+00_JPRB, 0.61579E+00_JPRB, 0.60114E+00_JPRB, 0.58760E+00_JPRB, 0.57620E+00_JPRB ,&
&  0.56689E+00_JPRB, 0.56185E+00_JPRB, 0.55657E+00_JPRB, 0.55048E+00_JPRB, 0.54154E+00_JPRB, 0.52882E+00_JPRB, 0.52036E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.51427E+00_JPRB, 0.51867E+00_JPRB, 0.53128E+00_JPRB, 0.54783E+00_JPRB, 0.56924E+00_JPRB, 0.60231E+00_JPRB ,&
&  0.71700E+00_JPRB, 0.12189E+01_JPRB, 0.13199E+01_JPRB, 0.12890E+01_JPRB, 0.13161E+01_JPRB, 0.12862E+01_JPRB, 0.13623E+01_JPRB ,&
&  0.15121E+01_JPRB, 0.16542E+01_JPRB, 0.17832E+01_JPRB, 0.18338E+01_JPRB, 0.17625E+01_JPRB, 0.16922E+01_JPRB, 0.15980E+01_JPRB ,&
&  0.14943E+01_JPRB, 0.13794E+01_JPRB, 0.12621E+01_JPRB, 0.11539E+01_JPRB, 0.10763E+01_JPRB, 0.10758E+01_JPRB, 0.11341E+01_JPRB ,&
&  0.12487E+01_JPRB, 0.13416E+01_JPRB, 0.14167E+01_JPRB, 0.15786E+01_JPRB, 0.19224E+01_JPRB, 0.21362E+01_JPRB, 0.24238E+01_JPRB ,&
&  0.24907E+01_JPRB, 0.24045E+01_JPRB, 0.22480E+01_JPRB, 0.20372E+01_JPRB, 0.17621E+01_JPRB, 0.15037E+01_JPRB, 0.12952E+01_JPRB ,&
&  0.11378E+01_JPRB, 0.10292E+01_JPRB, 0.96069E+00_JPRB, 0.93011E+00_JPRB, 0.94060E+00_JPRB, 0.98721E+00_JPRB, 0.10517E+01_JPRB ,&
&  0.11341E+01_JPRB, 0.12190E+01_JPRB, 0.12770E+01_JPRB, 0.13029E+01_JPRB, 0.12867E+01_JPRB, 0.12376E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.10119E+01_JPRB, 0.96087E+00_JPRB, 0.94007E+00_JPRB, 0.93749E+00_JPRB, 0.93745E+00_JPRB, 0.93526E+00_JPRB ,&
&  0.93819E+00_JPRB, 0.94119E+00_JPRB, 0.93646E+00_JPRB, 0.91275E+00_JPRB, 0.88430E+00_JPRB, 0.90286E+00_JPRB, 0.87295E+00_JPRB ,&
&  0.84238E+00_JPRB, 0.80841E+00_JPRB, 0.77929E+00_JPRB, 0.74584E+00_JPRB, 0.74135E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.75354E+00_JPRB, 0.71888E+00_JPRB, 0.67747E+00_JPRB, 0.64710E+00_JPRB, 0.62730E+00_JPRB, 0.61447E+00_JPRB, 0.60906E+00_JPRB ,&
&  0.60847E+00_JPRB, 0.59825E+00_JPRB, 0.58004E+00_JPRB, 0.56401E+00_JPRB, 0.55530E+00_JPRB, 0.54853E+00_JPRB, 0.53998E+00_JPRB ,&
&  0.53338E+00_JPRB, 0.52744E+00_JPRB, 0.52268E+00_JPRB, 0.52096E+00_JPRB, 0.52016E+00_JPRB, 0.51768E+00_JPRB, 0.51092E+00_JPRB ,&
&  0.50130E+00_JPRB, 0.49717E+00_JPRB, 0.49759E+00_JPRB, 0.50601E+00_JPRB, 0.52001E+00_JPRB, 0.53828E+00_JPRB, 0.56696E+00_JPRB ,&
&  0.68169E+00_JPRB, 0.79481E+00_JPRB, 0.88846E+00_JPRB, 0.91010E+00_JPRB, 0.95140E+00_JPRB, 0.10410E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.12925E+01_JPRB, 0.14231E+01_JPRB, 0.14876E+01_JPRB, 0.14399E+01_JPRB, 0.13495E+01_JPRB, 0.12689E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11385E+01_JPRB, 0.10848E+01_JPRB, 0.10350E+01_JPRB, 0.10102E+01_JPRB, 0.10196E+01_JPRB, 0.10500E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10659E+01_JPRB, 0.11477E+01_JPRB, 0.12814E+01_JPRB, 0.14781E+01_JPRB, 0.16195E+01_JPRB, 0.17586E+01_JPRB ,&
&  0.18695E+01_JPRB, 0.19127E+01_JPRB, 0.19206E+01_JPRB, 0.18694E+01_JPRB, 0.17391E+01_JPRB, 0.15834E+01_JPRB, 0.14182E+01_JPRB ,&
&  0.12580E+01_JPRB, 0.11442E+01_JPRB, 0.10797E+01_JPRB, 0.10461E+01_JPRB, 0.10390E+01_JPRB, 0.10495E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.11714E+01_JPRB, 0.12100E+01_JPRB, 0.12289E+01_JPRB, 0.12252E+01_JPRB, 0.12182E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.11595E+01_JPRB, 0.11180E+01_JPRB, 0.10716E+01_JPRB, 0.10301E+01_JPRB, 0.10001E+01_JPRB, 0.97590E+00_JPRB ,&
&  0.95095E+00_JPRB, 0.93337E+00_JPRB, 0.89704E+00_JPRB, 0.87895E+00_JPRB, 0.89648E+00_JPRB, 0.87047E+00_JPRB, 0.83786E+00_JPRB ,&
&  0.79938E+00_JPRB, 0.76516E+00_JPRB, 0.73392E+00_JPRB, 0.70182E+00_JPRB, 0.74456E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.65050E+00_JPRB, 0.64395E+00_JPRB, 0.62723E+00_JPRB, 0.60601E+00_JPRB, 0.59122E+00_JPRB, 0.57837E+00_JPRB, 0.56832E+00_JPRB ,&
&  0.56202E+00_JPRB, 0.55281E+00_JPRB, 0.53928E+00_JPRB, 0.52863E+00_JPRB, 0.52671E+00_JPRB, 0.52356E+00_JPRB, 0.51790E+00_JPRB ,&
&  0.50826E+00_JPRB, 0.49886E+00_JPRB, 0.49242E+00_JPRB, 0.49054E+00_JPRB, 0.49189E+00_JPRB, 0.49447E+00_JPRB, 0.49498E+00_JPRB ,&
&  0.49087E+00_JPRB, 0.48422E+00_JPRB, 0.48221E+00_JPRB, 0.48406E+00_JPRB, 0.49107E+00_JPRB, 0.50358E+00_JPRB, 0.52952E+00_JPRB ,&
&  0.61180E+00_JPRB, 0.62570E+00_JPRB, 0.66838E+00_JPRB, 0.69451E+00_JPRB, 0.73935E+00_JPRB, 0.82051E+00_JPRB, 0.91252E+00_JPRB ,&
&  0.96519E+00_JPRB, 0.10204E+01_JPRB, 0.11090E+01_JPRB, 0.11854E+01_JPRB, 0.11888E+01_JPRB, 0.11436E+01_JPRB, 0.10755E+01_JPRB ,&
&  0.10056E+01_JPRB, 0.95339E+00_JPRB, 0.90979E+00_JPRB, 0.87577E+00_JPRB, 0.86019E+00_JPRB, 0.85997E+00_JPRB, 0.86027E+00_JPRB ,&
&  0.84153E+00_JPRB, 0.84688E+00_JPRB, 0.90849E+00_JPRB, 0.10059E+01_JPRB, 0.11293E+01_JPRB, 0.12549E+01_JPRB, 0.13845E+01_JPRB ,&
&  0.14912E+01_JPRB, 0.15534E+01_JPRB, 0.16015E+01_JPRB, 0.16372E+01_JPRB, 0.16178E+01_JPRB, 0.15851E+01_JPRB, 0.14904E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.12720E+01_JPRB, 0.12025E+01_JPRB, 0.11472E+01_JPRB, 0.11067E+01_JPRB, 0.10807E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.10729E+01_JPRB, 0.10742E+01_JPRB, 0.10690E+01_JPRB, 0.10618E+01_JPRB, 0.10575E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.10513E+01_JPRB, 0.10402E+01_JPRB, 0.10146E+01_JPRB, 0.97734E+00_JPRB, 0.93961E+00_JPRB, 0.90277E+00_JPRB ,&
&  0.87137E+00_JPRB, 0.85463E+00_JPRB, 0.83466E+00_JPRB, 0.82750E+00_JPRB, 0.80531E+00_JPRB, 0.78261E+00_JPRB, 0.75865E+00_JPRB ,&
&  0.73254E+00_JPRB, 0.70648E+00_JPRB, 0.67444E+00_JPRB, 0.64364E+00_JPRB, 0.63320E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.58392E+00_JPRB, 0.58739E+00_JPRB, 0.58508E+00_JPRB, 0.57135E+00_JPRB, 0.55667E+00_JPRB, 0.54832E+00_JPRB, 0.53911E+00_JPRB ,&
&  0.52886E+00_JPRB, 0.52073E+00_JPRB, 0.50905E+00_JPRB, 0.50278E+00_JPRB, 0.50031E+00_JPRB, 0.49968E+00_JPRB, 0.49690E+00_JPRB ,&
&  0.48989E+00_JPRB, 0.47948E+00_JPRB, 0.47173E+00_JPRB, 0.46870E+00_JPRB, 0.47000E+00_JPRB, 0.47283E+00_JPRB, 0.47493E+00_JPRB ,&
&  0.47271E+00_JPRB, 0.47035E+00_JPRB, 0.46766E+00_JPRB, 0.46584E+00_JPRB, 0.46654E+00_JPRB, 0.47593E+00_JPRB, 0.49506E+00_JPRB ,&
&  0.55554E+00_JPRB, 0.52650E+00_JPRB, 0.54744E+00_JPRB, 0.58699E+00_JPRB, 0.62984E+00_JPRB, 0.69652E+00_JPRB, 0.77114E+00_JPRB ,&
&  0.82107E+00_JPRB, 0.83666E+00_JPRB, 0.87163E+00_JPRB, 0.93039E+00_JPRB, 0.94146E+00_JPRB, 0.93677E+00_JPRB, 0.91125E+00_JPRB ,&
&  0.86696E+00_JPRB, 0.81084E+00_JPRB, 0.75646E+00_JPRB, 0.71886E+00_JPRB, 0.70643E+00_JPRB, 0.70594E+00_JPRB, 0.70070E+00_JPRB ,&
&  0.67648E+00_JPRB, 0.66933E+00_JPRB, 0.69830E+00_JPRB, 0.74157E+00_JPRB, 0.79372E+00_JPRB, 0.87581E+00_JPRB, 0.98118E+00_JPRB ,&
&  0.10752E+01_JPRB, 0.11706E+01_JPRB, 0.12690E+01_JPRB, 0.13434E+01_JPRB, 0.14010E+01_JPRB, 0.14167E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.13236E+01_JPRB, 0.12516E+01_JPRB, 0.11826E+01_JPRB, 0.11157E+01_JPRB, 0.10587E+01_JPRB, 0.10216E+01_JPRB, 0.99750E+00_JPRB ,&
&  0.98081E+00_JPRB, 0.96713E+00_JPRB, 0.94692E+00_JPRB, 0.91903E+00_JPRB, 0.88993E+00_JPRB, 0.86517E+00_JPRB, 0.84345E+00_JPRB ,&
&  0.82498E+00_JPRB, 0.80573E+00_JPRB, 0.79051E+00_JPRB, 0.77941E+00_JPRB, 0.76740E+00_JPRB, 0.75631E+00_JPRB, 0.74615E+00_JPRB ,&
&  0.73469E+00_JPRB, 0.72960E+00_JPRB, 0.71578E+00_JPRB, 0.70782E+00_JPRB, 0.71310E+00_JPRB, 0.69970E+00_JPRB, 0.68693E+00_JPRB ,&
&  0.66760E+00_JPRB, 0.64621E+00_JPRB, 0.62224E+00_JPRB, 0.56761E+00_JPRB, 0.58275E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.53558E+00_JPRB, 0.53921E+00_JPRB, 0.53791E+00_JPRB, 0.53131E+00_JPRB, 0.52171E+00_JPRB, 0.51505E+00_JPRB, 0.50553E+00_JPRB ,&
&  0.49508E+00_JPRB, 0.49006E+00_JPRB, 0.48389E+00_JPRB, 0.48086E+00_JPRB, 0.48117E+00_JPRB, 0.47710E+00_JPRB, 0.47313E+00_JPRB ,&
&  0.46931E+00_JPRB, 0.46240E+00_JPRB, 0.45778E+00_JPRB, 0.45423E+00_JPRB, 0.45408E+00_JPRB, 0.45472E+00_JPRB, 0.45450E+00_JPRB ,&
&  0.45153E+00_JPRB, 0.44691E+00_JPRB, 0.44291E+00_JPRB, 0.44000E+00_JPRB, 0.44083E+00_JPRB, 0.44775E+00_JPRB, 0.46210E+00_JPRB ,&
&  0.50394E+00_JPRB, 0.46172E+00_JPRB, 0.48598E+00_JPRB, 0.51288E+00_JPRB, 0.55296E+00_JPRB, 0.59393E+00_JPRB, 0.62174E+00_JPRB ,&
&  0.65416E+00_JPRB, 0.68363E+00_JPRB, 0.71371E+00_JPRB, 0.74559E+00_JPRB, 0.75160E+00_JPRB, 0.74389E+00_JPRB, 0.73346E+00_JPRB ,&
&  0.71437E+00_JPRB, 0.68816E+00_JPRB, 0.66075E+00_JPRB, 0.63327E+00_JPRB, 0.61819E+00_JPRB, 0.61346E+00_JPRB, 0.60303E+00_JPRB ,&
&  0.58279E+00_JPRB, 0.56627E+00_JPRB, 0.55813E+00_JPRB, 0.56221E+00_JPRB, 0.57962E+00_JPRB, 0.61260E+00_JPRB, 0.67615E+00_JPRB ,&
&  0.75009E+00_JPRB, 0.83778E+00_JPRB, 0.93516E+00_JPRB, 0.10155E+01_JPRB, 0.10756E+01_JPRB, 0.11061E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10837E+01_JPRB, 0.10571E+01_JPRB, 0.10210E+01_JPRB, 0.97279E+00_JPRB, 0.92520E+00_JPRB, 0.88619E+00_JPRB ,&
&  0.85558E+00_JPRB, 0.82458E+00_JPRB, 0.79171E+00_JPRB, 0.75598E+00_JPRB, 0.71782E+00_JPRB, 0.68210E+00_JPRB, 0.65211E+00_JPRB ,&
&  0.62680E+00_JPRB, 0.60536E+00_JPRB, 0.59217E+00_JPRB, 0.58678E+00_JPRB, 0.59049E+00_JPRB, 0.59699E+00_JPRB, 0.60109E+00_JPRB ,&
&  0.60441E+00_JPRB, 0.61008E+00_JPRB, 0.61180E+00_JPRB, 0.61955E+00_JPRB, 0.61725E+00_JPRB, 0.61259E+00_JPRB, 0.60852E+00_JPRB ,&
&  0.59996E+00_JPRB, 0.58303E+00_JPRB, 0.51631E+00_JPRB, 0.53519E+00_JPRB, 0.53460E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.49868E+00_JPRB, 0.49317E+00_JPRB, 0.48877E+00_JPRB, 0.49021E+00_JPRB, 0.48919E+00_JPRB, 0.48530E+00_JPRB, 0.47874E+00_JPRB ,&
&  0.46811E+00_JPRB, 0.46137E+00_JPRB, 0.45698E+00_JPRB, 0.45634E+00_JPRB, 0.45970E+00_JPRB, 0.45780E+00_JPRB, 0.45248E+00_JPRB ,&
&  0.44838E+00_JPRB, 0.44370E+00_JPRB, 0.44014E+00_JPRB, 0.43611E+00_JPRB, 0.43264E+00_JPRB, 0.43079E+00_JPRB, 0.42956E+00_JPRB ,&
&  0.42719E+00_JPRB, 0.42283E+00_JPRB, 0.41860E+00_JPRB, 0.41578E+00_JPRB, 0.41712E+00_JPRB, 0.42288E+00_JPRB, 0.43196E+00_JPRB ,&
&  0.43342E+00_JPRB, 0.41877E+00_JPRB, 0.44212E+00_JPRB, 0.47166E+00_JPRB, 0.49396E+00_JPRB, 0.51834E+00_JPRB, 0.54076E+00_JPRB ,&
&  0.56240E+00_JPRB, 0.57077E+00_JPRB, 0.58130E+00_JPRB, 0.59339E+00_JPRB, 0.60251E+00_JPRB, 0.60215E+00_JPRB, 0.59898E+00_JPRB ,&
&  0.59484E+00_JPRB, 0.58931E+00_JPRB, 0.58229E+00_JPRB, 0.57101E+00_JPRB, 0.55926E+00_JPRB, 0.54787E+00_JPRB, 0.53624E+00_JPRB ,&
&  0.52400E+00_JPRB, 0.51265E+00_JPRB, 0.50520E+00_JPRB, 0.50294E+00_JPRB, 0.50684E+00_JPRB, 0.51471E+00_JPRB, 0.52944E+00_JPRB ,&
&  0.54811E+00_JPRB, 0.58235E+00_JPRB, 0.64563E+00_JPRB, 0.71494E+00_JPRB, 0.77204E+00_JPRB, 0.81236E+00_JPRB, 0.84419E+00_JPRB ,&
&  0.85596E+00_JPRB, 0.85526E+00_JPRB, 0.84954E+00_JPRB, 0.83624E+00_JPRB, 0.81512E+00_JPRB, 0.78545E+00_JPRB, 0.74954E+00_JPRB ,&
&  0.71685E+00_JPRB, 0.68799E+00_JPRB, 0.65959E+00_JPRB, 0.63142E+00_JPRB, 0.60332E+00_JPRB, 0.57720E+00_JPRB, 0.55477E+00_JPRB ,&
&  0.53544E+00_JPRB, 0.51837E+00_JPRB, 0.50555E+00_JPRB, 0.49842E+00_JPRB, 0.49880E+00_JPRB, 0.50466E+00_JPRB, 0.51308E+00_JPRB ,&
&  0.52000E+00_JPRB, 0.52937E+00_JPRB, 0.53703E+00_JPRB, 0.54442E+00_JPRB, 0.54970E+00_JPRB, 0.54805E+00_JPRB, 0.54588E+00_JPRB ,&
&  0.54155E+00_JPRB, 0.53127E+00_JPRB, 0.52155E+00_JPRB, 0.51084E+00_JPRB, 0.50254E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.45995E+00_JPRB, 0.45273E+00_JPRB, 0.45180E+00_JPRB, 0.45550E+00_JPRB, 0.46002E+00_JPRB, 0.46013E+00_JPRB, 0.45608E+00_JPRB ,&
&  0.44594E+00_JPRB, 0.43705E+00_JPRB, 0.43047E+00_JPRB, 0.42820E+00_JPRB, 0.43121E+00_JPRB, 0.43221E+00_JPRB, 0.42930E+00_JPRB ,&
&  0.42573E+00_JPRB, 0.41897E+00_JPRB, 0.41297E+00_JPRB, 0.40701E+00_JPRB, 0.40265E+00_JPRB, 0.40053E+00_JPRB, 0.39948E+00_JPRB ,&
&  0.39772E+00_JPRB, 0.39536E+00_JPRB, 0.39413E+00_JPRB, 0.39319E+00_JPRB, 0.39419E+00_JPRB, 0.39805E+00_JPRB, 0.40406E+00_JPRB ,&
&  0.41674E+00_JPRB, 0.40480E+00_JPRB, 0.42832E+00_JPRB, 0.43438E+00_JPRB, 0.44437E+00_JPRB, 0.45810E+00_JPRB, 0.47709E+00_JPRB ,&
&  0.49433E+00_JPRB, 0.50093E+00_JPRB, 0.50405E+00_JPRB, 0.50651E+00_JPRB, 0.50802E+00_JPRB, 0.51196E+00_JPRB, 0.51121E+00_JPRB ,&
&  0.50722E+00_JPRB, 0.50512E+00_JPRB, 0.50406E+00_JPRB, 0.50169E+00_JPRB, 0.49937E+00_JPRB, 0.49492E+00_JPRB, 0.48924E+00_JPRB ,&
&  0.48353E+00_JPRB, 0.47748E+00_JPRB, 0.47202E+00_JPRB, 0.46630E+00_JPRB, 0.46182E+00_JPRB, 0.46052E+00_JPRB, 0.46298E+00_JPRB ,&
&  0.46663E+00_JPRB, 0.47440E+00_JPRB, 0.49295E+00_JPRB, 0.52195E+00_JPRB, 0.55723E+00_JPRB, 0.59227E+00_JPRB, 0.62038E+00_JPRB ,&
&  0.63765E+00_JPRB, 0.64630E+00_JPRB, 0.64726E+00_JPRB, 0.64352E+00_JPRB, 0.63428E+00_JPRB, 0.62183E+00_JPRB, 0.60495E+00_JPRB ,&
&  0.58805E+00_JPRB, 0.57175E+00_JPRB, 0.55502E+00_JPRB, 0.53822E+00_JPRB, 0.52017E+00_JPRB, 0.50155E+00_JPRB, 0.48462E+00_JPRB ,&
&  0.47094E+00_JPRB, 0.46095E+00_JPRB, 0.45363E+00_JPRB, 0.44908E+00_JPRB, 0.44660E+00_JPRB, 0.44766E+00_JPRB, 0.45084E+00_JPRB ,&
&  0.45578E+00_JPRB, 0.46227E+00_JPRB, 0.46949E+00_JPRB, 0.47556E+00_JPRB, 0.48166E+00_JPRB, 0.48386E+00_JPRB, 0.48519E+00_JPRB ,&
&  0.48392E+00_JPRB, 0.47900E+00_JPRB, 0.47501E+00_JPRB, 0.47051E+00_JPRB, 0.46622E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.42345E+00_JPRB, 0.42016E+00_JPRB, 0.42272E+00_JPRB, 0.42730E+00_JPRB, 0.43456E+00_JPRB, 0.43465E+00_JPRB, 0.43276E+00_JPRB ,&
&  0.42690E+00_JPRB, 0.41887E+00_JPRB, 0.41119E+00_JPRB, 0.40426E+00_JPRB, 0.40195E+00_JPRB, 0.40316E+00_JPRB, 0.40332E+00_JPRB ,&
&  0.40180E+00_JPRB, 0.39662E+00_JPRB, 0.38976E+00_JPRB, 0.38328E+00_JPRB, 0.37860E+00_JPRB, 0.37597E+00_JPRB, 0.37391E+00_JPRB ,&
&  0.37284E+00_JPRB, 0.37094E+00_JPRB, 0.37170E+00_JPRB, 0.37166E+00_JPRB, 0.37216E+00_JPRB, 0.37557E+00_JPRB, 0.37968E+00_JPRB ,&
&  0.38384E+00_JPRB, 0.40481E+00_JPRB, 0.39969E+00_JPRB, 0.39782E+00_JPRB, 0.40255E+00_JPRB, 0.41264E+00_JPRB, 0.42855E+00_JPRB ,&
&  0.44436E+00_JPRB, 0.45199E+00_JPRB, 0.45428E+00_JPRB, 0.45123E+00_JPRB, 0.44795E+00_JPRB, 0.44720E+00_JPRB, 0.44891E+00_JPRB ,&
&  0.44933E+00_JPRB, 0.44589E+00_JPRB, 0.44083E+00_JPRB, 0.43636E+00_JPRB, 0.43480E+00_JPRB, 0.43387E+00_JPRB, 0.43230E+00_JPRB ,&
&  0.43011E+00_JPRB, 0.42716E+00_JPRB, 0.42204E+00_JPRB, 0.41556E+00_JPRB, 0.40932E+00_JPRB, 0.40608E+00_JPRB, 0.40660E+00_JPRB ,&
&  0.41000E+00_JPRB, 0.41569E+00_JPRB, 0.42479E+00_JPRB, 0.43671E+00_JPRB, 0.44981E+00_JPRB, 0.46619E+00_JPRB, 0.48184E+00_JPRB ,&
&  0.49339E+00_JPRB, 0.49904E+00_JPRB, 0.49984E+00_JPRB, 0.49795E+00_JPRB, 0.49389E+00_JPRB, 0.48846E+00_JPRB, 0.48279E+00_JPRB ,&
&  0.47724E+00_JPRB, 0.47104E+00_JPRB, 0.46403E+00_JPRB, 0.45595E+00_JPRB, 0.44564E+00_JPRB, 0.43486E+00_JPRB, 0.42603E+00_JPRB ,&
&  0.41995E+00_JPRB, 0.41571E+00_JPRB, 0.41266E+00_JPRB, 0.41006E+00_JPRB, 0.40855E+00_JPRB, 0.40848E+00_JPRB, 0.41018E+00_JPRB ,&
&  0.41335E+00_JPRB, 0.41759E+00_JPRB, 0.42282E+00_JPRB, 0.42784E+00_JPRB, 0.43125E+00_JPRB, 0.43404E+00_JPRB, 0.43679E+00_JPRB ,&
&  0.43751E+00_JPRB, 0.43512E+00_JPRB, 0.43327E+00_JPRB, 0.43185E+00_JPRB, 0.42848E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.39247E+00_JPRB, 0.39210E+00_JPRB, 0.39708E+00_JPRB, 0.40221E+00_JPRB, 0.40809E+00_JPRB, 0.41062E+00_JPRB, 0.40854E+00_JPRB ,&
&  0.40558E+00_JPRB, 0.40145E+00_JPRB, 0.39513E+00_JPRB, 0.38718E+00_JPRB, 0.38001E+00_JPRB, 0.37830E+00_JPRB, 0.37877E+00_JPRB ,&
&  0.37906E+00_JPRB, 0.37631E+00_JPRB, 0.37159E+00_JPRB, 0.36651E+00_JPRB, 0.36094E+00_JPRB, 0.35606E+00_JPRB, 0.35239E+00_JPRB ,&
&  0.35133E+00_JPRB, 0.35078E+00_JPRB, 0.35165E+00_JPRB, 0.35266E+00_JPRB, 0.35375E+00_JPRB, 0.35618E+00_JPRB, 0.36276E+00_JPRB ,&
&  0.36264E+00_JPRB, 0.37163E+00_JPRB, 0.36106E+00_JPRB, 0.36446E+00_JPRB, 0.36829E+00_JPRB, 0.37408E+00_JPRB, 0.38267E+00_JPRB ,&
&  0.39322E+00_JPRB, 0.40045E+00_JPRB, 0.40470E+00_JPRB, 0.40428E+00_JPRB, 0.40181E+00_JPRB, 0.40200E+00_JPRB, 0.40300E+00_JPRB ,&
&  0.40357E+00_JPRB, 0.40341E+00_JPRB, 0.40117E+00_JPRB, 0.39600E+00_JPRB, 0.38900E+00_JPRB, 0.38316E+00_JPRB, 0.37913E+00_JPRB ,&
&  0.37843E+00_JPRB, 0.37802E+00_JPRB, 0.37610E+00_JPRB, 0.37337E+00_JPRB, 0.37032E+00_JPRB, 0.36862E+00_JPRB, 0.36949E+00_JPRB ,&
&  0.37207E+00_JPRB, 0.37489E+00_JPRB, 0.37761E+00_JPRB, 0.38292E+00_JPRB, 0.39046E+00_JPRB, 0.39761E+00_JPRB, 0.40438E+00_JPRB ,&
&  0.41009E+00_JPRB, 0.41191E+00_JPRB, 0.41068E+00_JPRB, 0.40792E+00_JPRB, 0.40573E+00_JPRB, 0.40407E+00_JPRB, 0.40353E+00_JPRB ,&
&  0.40256E+00_JPRB, 0.40078E+00_JPRB, 0.39770E+00_JPRB, 0.39345E+00_JPRB, 0.38914E+00_JPRB, 0.38499E+00_JPRB, 0.38195E+00_JPRB ,&
&  0.38017E+00_JPRB, 0.37897E+00_JPRB, 0.37762E+00_JPRB, 0.37630E+00_JPRB, 0.37623E+00_JPRB, 0.37660E+00_JPRB, 0.37767E+00_JPRB ,&
&  0.37938E+00_JPRB, 0.38183E+00_JPRB, 0.38520E+00_JPRB, 0.38863E+00_JPRB, 0.38856E+00_JPRB, 0.38863E+00_JPRB, 0.39086E+00_JPRB ,&
&  0.39325E+00_JPRB, 0.39415E+00_JPRB, 0.39401E+00_JPRB, 0.39389E+00_JPRB, 0.39402E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.36686E+00_JPRB, 0.36874E+00_JPRB, 0.37293E+00_JPRB, 0.37931E+00_JPRB, 0.38292E+00_JPRB, 0.38736E+00_JPRB, 0.38752E+00_JPRB ,&
&  0.38481E+00_JPRB, 0.38110E+00_JPRB, 0.37659E+00_JPRB, 0.37069E+00_JPRB, 0.36427E+00_JPRB, 0.35949E+00_JPRB, 0.35790E+00_JPRB ,&
&  0.35777E+00_JPRB, 0.35617E+00_JPRB, 0.35265E+00_JPRB, 0.34816E+00_JPRB, 0.34329E+00_JPRB, 0.33929E+00_JPRB, 0.33630E+00_JPRB ,&
&  0.33561E+00_JPRB, 0.33635E+00_JPRB, 0.33791E+00_JPRB, 0.33897E+00_JPRB, 0.33917E+00_JPRB, 0.34152E+00_JPRB, 0.34653E+00_JPRB ,&
&  0.34557E+00_JPRB, 0.34441E+00_JPRB, 0.33638E+00_JPRB, 0.33575E+00_JPRB, 0.33787E+00_JPRB, 0.34077E+00_JPRB, 0.34473E+00_JPRB ,&
&  0.34954E+00_JPRB, 0.35397E+00_JPRB, 0.35865E+00_JPRB, 0.36238E+00_JPRB, 0.36478E+00_JPRB, 0.36667E+00_JPRB, 0.36792E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.36791E+00_JPRB, 0.36561E+00_JPRB, 0.36132E+00_JPRB, 0.35587E+00_JPRB, 0.35206E+00_JPRB, 0.35054E+00_JPRB ,&
&  0.34925E+00_JPRB, 0.34922E+00_JPRB, 0.34900E+00_JPRB, 0.34780E+00_JPRB, 0.34604E+00_JPRB, 0.34540E+00_JPRB, 0.34608E+00_JPRB ,&
&  0.34699E+00_JPRB, 0.34706E+00_JPRB, 0.34641E+00_JPRB, 0.34725E+00_JPRB, 0.35060E+00_JPRB, 0.35473E+00_JPRB, 0.35790E+00_JPRB ,&
&  0.35989E+00_JPRB, 0.35951E+00_JPRB, 0.35756E+00_JPRB, 0.35446E+00_JPRB, 0.35204E+00_JPRB, 0.35167E+00_JPRB, 0.35315E+00_JPRB ,&
&  0.35443E+00_JPRB, 0.35456E+00_JPRB, 0.35456E+00_JPRB, 0.35261E+00_JPRB, 0.35042E+00_JPRB, 0.34946E+00_JPRB, 0.34887E+00_JPRB ,&
&  0.34783E+00_JPRB, 0.34663E+00_JPRB, 0.34566E+00_JPRB, 0.34534E+00_JPRB, 0.34576E+00_JPRB, 0.34563E+00_JPRB, 0.34620E+00_JPRB ,&
&  0.34876E+00_JPRB, 0.35200E+00_JPRB, 0.35483E+00_JPRB, 0.35653E+00_JPRB, 0.35694E+00_JPRB, 0.35633E+00_JPRB, 0.35812E+00_JPRB ,&
&  0.36139E+00_JPRB, 0.36396E+00_JPRB, 0.36412E+00_JPRB, 0.36365E+00_JPRB, 0.36509E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.34489E+00_JPRB, 0.34752E+00_JPRB, 0.35197E+00_JPRB, 0.35713E+00_JPRB, 0.36038E+00_JPRB, 0.36535E+00_JPRB, 0.36633E+00_JPRB ,&
&  0.36572E+00_JPRB, 0.36391E+00_JPRB, 0.35948E+00_JPRB, 0.35399E+00_JPRB, 0.34850E+00_JPRB, 0.34378E+00_JPRB, 0.34051E+00_JPRB ,&
&  0.33870E+00_JPRB, 0.33707E+00_JPRB, 0.33508E+00_JPRB, 0.33214E+00_JPRB, 0.32874E+00_JPRB, 0.32575E+00_JPRB, 0.32353E+00_JPRB ,&
&  0.32301E+00_JPRB, 0.32332E+00_JPRB, 0.32400E+00_JPRB, 0.32420E+00_JPRB, 0.32383E+00_JPRB, 0.32501E+00_JPRB, 0.32713E+00_JPRB ,&
&  0.32635E+00_JPRB, 0.32203E+00_JPRB, 0.31843E+00_JPRB, 0.31804E+00_JPRB, 0.31799E+00_JPRB, 0.31835E+00_JPRB, 0.31940E+00_JPRB ,&
&  0.32041E+00_JPRB, 0.32124E+00_JPRB, 0.32355E+00_JPRB, 0.32647E+00_JPRB, 0.32930E+00_JPRB, 0.33293E+00_JPRB, 0.33485E+00_JPRB ,&
&  0.33525E+00_JPRB, 0.33443E+00_JPRB, 0.33279E+00_JPRB, 0.33018E+00_JPRB, 0.32852E+00_JPRB, 0.32828E+00_JPRB, 0.32768E+00_JPRB ,&
&  0.32852E+00_JPRB, 0.32934E+00_JPRB, 0.33000E+00_JPRB, 0.32852E+00_JPRB, 0.32603E+00_JPRB, 0.32549E+00_JPRB, 0.32594E+00_JPRB ,&
&  0.32610E+00_JPRB, 0.32510E+00_JPRB, 0.32308E+00_JPRB, 0.32203E+00_JPRB, 0.32183E+00_JPRB, 0.32266E+00_JPRB, 0.32292E+00_JPRB ,&
&  0.32228E+00_JPRB, 0.32041E+00_JPRB, 0.31897E+00_JPRB, 0.31818E+00_JPRB, 0.31780E+00_JPRB, 0.31808E+00_JPRB, 0.31794E+00_JPRB ,&
&  0.31902E+00_JPRB, 0.32001E+00_JPRB, 0.31916E+00_JPRB, 0.31829E+00_JPRB, 0.31745E+00_JPRB, 0.31724E+00_JPRB, 0.31684E+00_JPRB ,&
&  0.31598E+00_JPRB, 0.31505E+00_JPRB, 0.31459E+00_JPRB, 0.31476E+00_JPRB, 0.31378E+00_JPRB, 0.31241E+00_JPRB, 0.31309E+00_JPRB ,&
&  0.31660E+00_JPRB, 0.32049E+00_JPRB, 0.32382E+00_JPRB, 0.32588E+00_JPRB, 0.32647E+00_JPRB, 0.32783E+00_JPRB, 0.33069E+00_JPRB ,&
&  0.33308E+00_JPRB, 0.33414E+00_JPRB, 0.33673E+00_JPRB, 0.33901E+00_JPRB, 0.34132E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.32584E+00_JPRB, 0.32972E+00_JPRB, 0.33235E+00_JPRB, 0.33582E+00_JPRB, 0.33883E+00_JPRB, 0.34420E+00_JPRB, 0.34634E+00_JPRB ,&
&  0.34630E+00_JPRB, 0.34623E+00_JPRB, 0.34487E+00_JPRB, 0.34215E+00_JPRB, 0.33779E+00_JPRB, 0.33317E+00_JPRB, 0.32875E+00_JPRB ,&
&  0.32529E+00_JPRB, 0.32251E+00_JPRB, 0.32039E+00_JPRB, 0.31852E+00_JPRB, 0.31597E+00_JPRB, 0.31358E+00_JPRB, 0.31184E+00_JPRB ,&
&  0.31123E+00_JPRB, 0.31085E+00_JPRB, 0.31080E+00_JPRB, 0.31102E+00_JPRB, 0.31066E+00_JPRB, 0.30997E+00_JPRB, 0.30989E+00_JPRB ,&
&  0.30850E+00_JPRB, 0.30443E+00_JPRB, 0.30191E+00_JPRB, 0.30100E+00_JPRB, 0.30007E+00_JPRB, 0.30048E+00_JPRB, 0.30007E+00_JPRB ,&
&  0.29987E+00_JPRB, 0.29996E+00_JPRB, 0.30178E+00_JPRB, 0.30443E+00_JPRB, 0.30544E+00_JPRB, 0.30701E+00_JPRB, 0.30924E+00_JPRB ,&
&  0.31025E+00_JPRB, 0.30941E+00_JPRB, 0.30739E+00_JPRB, 0.30711E+00_JPRB, 0.30845E+00_JPRB, 0.30891E+00_JPRB, 0.30950E+00_JPRB ,&
&  0.31022E+00_JPRB, 0.31143E+00_JPRB, 0.31202E+00_JPRB, 0.30977E+00_JPRB, 0.30747E+00_JPRB, 0.30664E+00_JPRB, 0.30655E+00_JPRB ,&
&  0.30627E+00_JPRB, 0.30517E+00_JPRB, 0.30275E+00_JPRB, 0.30040E+00_JPRB, 0.29721E+00_JPRB, 0.29468E+00_JPRB, 0.29336E+00_JPRB ,&
&  0.29368E+00_JPRB, 0.29407E+00_JPRB, 0.29551E+00_JPRB, 0.29674E+00_JPRB, 0.29734E+00_JPRB, 0.29727E+00_JPRB, 0.29563E+00_JPRB ,&
&  0.29386E+00_JPRB, 0.29324E+00_JPRB, 0.29197E+00_JPRB, 0.29058E+00_JPRB, 0.29073E+00_JPRB, 0.29072E+00_JPRB, 0.29022E+00_JPRB ,&
&  0.28976E+00_JPRB, 0.29036E+00_JPRB, 0.29085E+00_JPRB, 0.28987E+00_JPRB, 0.28847E+00_JPRB, 0.28704E+00_JPRB, 0.28752E+00_JPRB ,&
&  0.28929E+00_JPRB, 0.29196E+00_JPRB, 0.29498E+00_JPRB, 0.29819E+00_JPRB, 0.30135E+00_JPRB, 0.30413E+00_JPRB, 0.30701E+00_JPRB ,&
&  0.31049E+00_JPRB, 0.31146E+00_JPRB, 0.31383E+00_JPRB, 0.31750E+00_JPRB, 0.32098E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.30388E+00_JPRB, 0.30951E+00_JPRB, 0.31237E+00_JPRB, 0.31450E+00_JPRB, 0.31705E+00_JPRB, 0.32200E+00_JPRB, 0.32552E+00_JPRB ,&
&  0.32637E+00_JPRB, 0.32666E+00_JPRB, 0.32658E+00_JPRB, 0.32571E+00_JPRB, 0.32402E+00_JPRB, 0.32229E+00_JPRB, 0.31928E+00_JPRB ,&
&  0.31544E+00_JPRB, 0.31189E+00_JPRB, 0.30872E+00_JPRB, 0.30576E+00_JPRB, 0.30301E+00_JPRB, 0.30052E+00_JPRB, 0.29874E+00_JPRB ,&
&  0.29855E+00_JPRB, 0.29888E+00_JPRB, 0.29839E+00_JPRB, 0.29849E+00_JPRB, 0.29839E+00_JPRB, 0.29710E+00_JPRB, 0.29513E+00_JPRB ,&
&  0.29261E+00_JPRB, 0.28863E+00_JPRB, 0.28459E+00_JPRB, 0.28379E+00_JPRB, 0.28339E+00_JPRB, 0.28235E+00_JPRB, 0.28084E+00_JPRB ,&
&  0.28204E+00_JPRB, 0.28417E+00_JPRB, 0.28684E+00_JPRB, 0.28844E+00_JPRB, 0.28962E+00_JPRB, 0.29163E+00_JPRB, 0.29408E+00_JPRB ,&
&  0.29469E+00_JPRB, 0.29185E+00_JPRB, 0.28919E+00_JPRB, 0.28902E+00_JPRB, 0.28933E+00_JPRB, 0.28920E+00_JPRB, 0.28987E+00_JPRB ,&
&  0.29071E+00_JPRB, 0.29252E+00_JPRB, 0.29364E+00_JPRB, 0.29223E+00_JPRB, 0.28969E+00_JPRB, 0.28785E+00_JPRB, 0.28668E+00_JPRB ,&
&  0.28614E+00_JPRB, 0.28557E+00_JPRB, 0.28452E+00_JPRB, 0.28275E+00_JPRB, 0.28067E+00_JPRB, 0.27852E+00_JPRB, 0.27624E+00_JPRB ,&
&  0.27424E+00_JPRB, 0.27442E+00_JPRB, 0.27747E+00_JPRB, 0.27902E+00_JPRB, 0.27820E+00_JPRB, 0.27660E+00_JPRB, 0.27514E+00_JPRB ,&
&  0.27438E+00_JPRB, 0.27141E+00_JPRB, 0.26868E+00_JPRB, 0.26664E+00_JPRB, 0.26546E+00_JPRB, 0.26378E+00_JPRB, 0.26333E+00_JPRB ,&
&  0.26411E+00_JPRB, 0.26636E+00_JPRB, 0.26677E+00_JPRB, 0.26632E+00_JPRB, 0.26551E+00_JPRB, 0.26490E+00_JPRB, 0.26597E+00_JPRB ,&
&  0.26751E+00_JPRB, 0.27020E+00_JPRB, 0.27328E+00_JPRB, 0.27736E+00_JPRB, 0.28078E+00_JPRB, 0.28472E+00_JPRB, 0.28784E+00_JPRB ,&
&  0.28980E+00_JPRB, 0.29035E+00_JPRB, 0.29165E+00_JPRB, 0.29393E+00_JPRB, 0.29776E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.28089E+00_JPRB, 0.28717E+00_JPRB, 0.29234E+00_JPRB, 0.29486E+00_JPRB, 0.29624E+00_JPRB, 0.29865E+00_JPRB, 0.30133E+00_JPRB ,&
&  0.30348E+00_JPRB, 0.30496E+00_JPRB, 0.30587E+00_JPRB, 0.30610E+00_JPRB, 0.30546E+00_JPRB, 0.30412E+00_JPRB, 0.30241E+00_JPRB ,&
&  0.30041E+00_JPRB, 0.29742E+00_JPRB, 0.29459E+00_JPRB, 0.29251E+00_JPRB, 0.29088E+00_JPRB, 0.28905E+00_JPRB, 0.28764E+00_JPRB ,&
&  0.28637E+00_JPRB, 0.28635E+00_JPRB, 0.28596E+00_JPRB, 0.28513E+00_JPRB, 0.28412E+00_JPRB, 0.28267E+00_JPRB, 0.28154E+00_JPRB ,&
&  0.27820E+00_JPRB, 0.27286E+00_JPRB, 0.27121E+00_JPRB, 0.26959E+00_JPRB, 0.26977E+00_JPRB, 0.26890E+00_JPRB, 0.26768E+00_JPRB ,&
&  0.26924E+00_JPRB, 0.27241E+00_JPRB, 0.27497E+00_JPRB, 0.27631E+00_JPRB, 0.27732E+00_JPRB, 0.27903E+00_JPRB, 0.28086E+00_JPRB ,&
&  0.27859E+00_JPRB, 0.27549E+00_JPRB, 0.27323E+00_JPRB, 0.27236E+00_JPRB, 0.27255E+00_JPRB, 0.27382E+00_JPRB, 0.27604E+00_JPRB ,&
&  0.27800E+00_JPRB, 0.27916E+00_JPRB, 0.27883E+00_JPRB, 0.27710E+00_JPRB, 0.27487E+00_JPRB, 0.27384E+00_JPRB, 0.27336E+00_JPRB ,&
&  0.27312E+00_JPRB, 0.27256E+00_JPRB, 0.27188E+00_JPRB, 0.27023E+00_JPRB, 0.26238E+00_JPRB, 0.25948E+00_JPRB, 0.25680E+00_JPRB ,&
&  0.25783E+00_JPRB, 0.25630E+00_JPRB, 0.25175E+00_JPRB, 0.25964E+00_JPRB, 0.26769E+00_JPRB, 0.26126E+00_JPRB, 0.25419E+00_JPRB ,&
&  0.24443E+00_JPRB, 0.24832E+00_JPRB, 0.24550E+00_JPRB, 0.23947E+00_JPRB, 0.23832E+00_JPRB, 0.23894E+00_JPRB, 0.23835E+00_JPRB ,&
&  0.23922E+00_JPRB, 0.23864E+00_JPRB, 0.23975E+00_JPRB, 0.24021E+00_JPRB, 0.24051E+00_JPRB, 0.24191E+00_JPRB, 0.24323E+00_JPRB ,&
&  0.24657E+00_JPRB, 0.25169E+00_JPRB, 0.25679E+00_JPRB, 0.25681E+00_JPRB, 0.26063E+00_JPRB, 0.26362E+00_JPRB, 0.26535E+00_JPRB ,&
&  0.26675E+00_JPRB, 0.26902E+00_JPRB, 0.27110E+00_JPRB, 0.27291E+00_JPRB, 0.27584E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.26514E+00_JPRB, 0.26792E+00_JPRB, 0.27122E+00_JPRB, 0.27442E+00_JPRB, 0.27665E+00_JPRB, 0.27889E+00_JPRB, 0.28091E+00_JPRB ,&
&  0.28272E+00_JPRB, 0.28403E+00_JPRB, 0.28498E+00_JPRB, 0.28563E+00_JPRB, 0.28612E+00_JPRB, 0.28604E+00_JPRB, 0.28553E+00_JPRB ,&
&  0.28499E+00_JPRB, 0.28388E+00_JPRB, 0.28226E+00_JPRB, 0.28096E+00_JPRB, 0.28019E+00_JPRB, 0.27901E+00_JPRB, 0.27725E+00_JPRB ,&
&  0.27540E+00_JPRB, 0.27362E+00_JPRB, 0.27201E+00_JPRB, 0.27020E+00_JPRB, 0.26986E+00_JPRB, 0.27048E+00_JPRB, 0.27106E+00_JPRB ,&
&  0.26444E+00_JPRB, 0.25877E+00_JPRB, 0.26150E+00_JPRB, 0.26259E+00_JPRB, 0.25769E+00_JPRB, 0.25907E+00_JPRB, 0.25979E+00_JPRB ,&
&  0.26071E+00_JPRB, 0.26218E+00_JPRB, 0.26379E+00_JPRB, 0.26449E+00_JPRB, 0.26501E+00_JPRB, 0.26594E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26344E+00_JPRB, 0.26167E+00_JPRB, 0.26006E+00_JPRB, 0.25838E+00_JPRB, 0.25806E+00_JPRB, 0.26040E+00_JPRB, 0.26194E+00_JPRB ,&
&  0.26088E+00_JPRB, 0.25908E+00_JPRB, 0.25734E+00_JPRB, 0.25792E+00_JPRB, 0.25899E+00_JPRB, 0.25956E+00_JPRB, 0.25675E+00_JPRB ,&
&  0.25617E+00_JPRB, 0.25290E+00_JPRB, 0.25009E+00_JPRB, 0.24725E+00_JPRB, 0.24625E+00_JPRB, 0.24196E+00_JPRB, 0.23717E+00_JPRB ,&
&  0.23639E+00_JPRB, 0.24169E+00_JPRB, 0.24900E+00_JPRB, 0.24978E+00_JPRB, 0.23148E+00_JPRB, 0.23617E+00_JPRB, 0.23834E+00_JPRB ,&
&  0.23337E+00_JPRB, 0.22679E+00_JPRB, 0.22187E+00_JPRB, 0.21977E+00_JPRB, 0.21899E+00_JPRB, 0.21822E+00_JPRB, 0.21879E+00_JPRB ,&
&  0.21988E+00_JPRB, 0.22028E+00_JPRB, 0.22089E+00_JPRB, 0.22247E+00_JPRB, 0.22404E+00_JPRB, 0.22569E+00_JPRB, 0.22722E+00_JPRB ,&
&  0.23073E+00_JPRB, 0.23485E+00_JPRB, 0.23771E+00_JPRB, 0.24122E+00_JPRB, 0.24494E+00_JPRB, 0.24849E+00_JPRB, 0.25150E+00_JPRB ,&
&  0.25475E+00_JPRB, 0.25569E+00_JPRB, 0.25770E+00_JPRB, 0.26153E+00_JPRB, 0.26339E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.25749E+00_JPRB, 0.26090E+00_JPRB, 0.26303E+00_JPRB, 0.26514E+00_JPRB, 0.26679E+00_JPRB, 0.26847E+00_JPRB, 0.26926E+00_JPRB ,&
&  0.27002E+00_JPRB, 0.27006E+00_JPRB, 0.27064E+00_JPRB, 0.27072E+00_JPRB, 0.27104E+00_JPRB, 0.26977E+00_JPRB, 0.26938E+00_JPRB ,&
&  0.26897E+00_JPRB, 0.26968E+00_JPRB, 0.26993E+00_JPRB, 0.26999E+00_JPRB, 0.26922E+00_JPRB, 0.26812E+00_JPRB, 0.26625E+00_JPRB ,&
&  0.26236E+00_JPRB, 0.25919E+00_JPRB, 0.25907E+00_JPRB, 0.25816E+00_JPRB, 0.25854E+00_JPRB, 0.25835E+00_JPRB, 0.25887E+00_JPRB ,&
&  0.25325E+00_JPRB, 0.25186E+00_JPRB, 0.25338E+00_JPRB, 0.25182E+00_JPRB, 0.25260E+00_JPRB, 0.25308E+00_JPRB, 0.25516E+00_JPRB ,&
&  0.25545E+00_JPRB, 0.25556E+00_JPRB, 0.25650E+00_JPRB, 0.25574E+00_JPRB, 0.25604E+00_JPRB, 0.25681E+00_JPRB, 0.25563E+00_JPRB ,&
&  0.25179E+00_JPRB, 0.24909E+00_JPRB, 0.24502E+00_JPRB, 0.23567E+00_JPRB, 0.23461E+00_JPRB, 0.23598E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23193E+00_JPRB, 0.23324E+00_JPRB, 0.23275E+00_JPRB, 0.23057E+00_JPRB, 0.23322E+00_JPRB, 0.23709E+00_JPRB, 0.23721E+00_JPRB ,&
&  0.23540E+00_JPRB, 0.23490E+00_JPRB, 0.23405E+00_JPRB, 0.23240E+00_JPRB, 0.23041E+00_JPRB, 0.22965E+00_JPRB, 0.22893E+00_JPRB ,&
&  0.22845E+00_JPRB, 0.22629E+00_JPRB, 0.23008E+00_JPRB, 0.23020E+00_JPRB, 0.22349E+00_JPRB, 0.22808E+00_JPRB, 0.22584E+00_JPRB ,&
&  0.21655E+00_JPRB, 0.21255E+00_JPRB, 0.21016E+00_JPRB, 0.20807E+00_JPRB, 0.20759E+00_JPRB, 0.20751E+00_JPRB, 0.20707E+00_JPRB ,&
&  0.20807E+00_JPRB, 0.20947E+00_JPRB, 0.21008E+00_JPRB, 0.21100E+00_JPRB, 0.21232E+00_JPRB, 0.21433E+00_JPRB, 0.21700E+00_JPRB ,&
&  0.21937E+00_JPRB, 0.22255E+00_JPRB, 0.22404E+00_JPRB, 0.22648E+00_JPRB, 0.22826E+00_JPRB, 0.23033E+00_JPRB, 0.23547E+00_JPRB ,&
&  0.23929E+00_JPRB, 0.24682E+00_JPRB, 0.25664E+00_JPRB, 0.25835E+00_JPRB, 0.25403E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.25227E+00_JPRB, 0.25484E+00_JPRB, 0.25700E+00_JPRB, 0.25884E+00_JPRB, 0.26083E+00_JPRB, 0.26245E+00_JPRB, 0.26328E+00_JPRB ,&
&  0.26342E+00_JPRB, 0.26255E+00_JPRB, 0.26252E+00_JPRB, 0.26050E+00_JPRB, 0.25856E+00_JPRB, 0.25409E+00_JPRB, 0.25411E+00_JPRB ,&
&  0.25325E+00_JPRB, 0.25434E+00_JPRB, 0.25383E+00_JPRB, 0.25422E+00_JPRB, 0.25316E+00_JPRB, 0.25251E+00_JPRB, 0.24999E+00_JPRB ,&
&  0.24642E+00_JPRB, 0.24397E+00_JPRB, 0.24171E+00_JPRB, 0.24127E+00_JPRB, 0.24234E+00_JPRB, 0.24297E+00_JPRB, 0.24244E+00_JPRB ,&
&  0.24252E+00_JPRB, 0.24271E+00_JPRB, 0.24304E+00_JPRB, 0.24268E+00_JPRB, 0.24856E+00_JPRB, 0.25122E+00_JPRB, 0.25254E+00_JPRB ,&
&  0.25206E+00_JPRB, 0.25203E+00_JPRB, 0.25195E+00_JPRB, 0.25056E+00_JPRB, 0.24924E+00_JPRB, 0.24347E+00_JPRB, 0.24005E+00_JPRB ,&
&  0.23173E+00_JPRB, 0.22558E+00_JPRB, 0.22102E+00_JPRB, 0.21820E+00_JPRB, 0.21710E+00_JPRB, 0.21735E+00_JPRB, 0.21708E+00_JPRB ,&
&  0.21524E+00_JPRB, 0.21347E+00_JPRB, 0.21234E+00_JPRB, 0.21336E+00_JPRB, 0.21502E+00_JPRB, 0.21679E+00_JPRB, 0.21883E+00_JPRB ,&
&  0.22054E+00_JPRB, 0.22099E+00_JPRB, 0.22091E+00_JPRB, 0.22153E+00_JPRB, 0.22202E+00_JPRB, 0.22276E+00_JPRB, 0.22321E+00_JPRB ,&
&  0.22450E+00_JPRB, 0.22314E+00_JPRB, 0.22395E+00_JPRB, 0.22571E+00_JPRB, 0.22251E+00_JPRB, 0.21756E+00_JPRB, 0.21276E+00_JPRB ,&
&  0.20803E+00_JPRB, 0.20539E+00_JPRB, 0.20316E+00_JPRB, 0.20167E+00_JPRB, 0.20127E+00_JPRB, 0.20123E+00_JPRB, 0.20180E+00_JPRB ,&
&  0.20257E+00_JPRB, 0.20377E+00_JPRB, 0.20451E+00_JPRB, 0.20562E+00_JPRB, 0.20700E+00_JPRB, 0.20861E+00_JPRB, 0.21052E+00_JPRB ,&
&  0.21292E+00_JPRB, 0.21512E+00_JPRB, 0.21671E+00_JPRB, 0.21886E+00_JPRB, 0.22290E+00_JPRB, 0.22806E+00_JPRB, 0.23571E+00_JPRB ,&
&  0.24374E+00_JPRB, 0.24860E+00_JPRB, 0.24324E+00_JPRB, 0.24760E+00_JPRB, 0.25085E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.25221E+00_JPRB, 0.25446E+00_JPRB, 0.25613E+00_JPRB, 0.25790E+00_JPRB, 0.25927E+00_JPRB, 0.26049E+00_JPRB, 0.26128E+00_JPRB ,&
&  0.25799E+00_JPRB, 0.25637E+00_JPRB, 0.25326E+00_JPRB, 0.25088E+00_JPRB, 0.24779E+00_JPRB, 0.24419E+00_JPRB, 0.24320E+00_JPRB ,&
&  0.24116E+00_JPRB, 0.24082E+00_JPRB, 0.23895E+00_JPRB, 0.23812E+00_JPRB, 0.23646E+00_JPRB, 0.23532E+00_JPRB, 0.23339E+00_JPRB ,&
&  0.23011E+00_JPRB, 0.22856E+00_JPRB, 0.22710E+00_JPRB, 0.22734E+00_JPRB, 0.22906E+00_JPRB, 0.23082E+00_JPRB, 0.23275E+00_JPRB ,&
&  0.23561E+00_JPRB, 0.23655E+00_JPRB, 0.23930E+00_JPRB, 0.24144E+00_JPRB, 0.24557E+00_JPRB, 0.24909E+00_JPRB, 0.24999E+00_JPRB ,&
&  0.24992E+00_JPRB, 0.24680E+00_JPRB, 0.24548E+00_JPRB, 0.24219E+00_JPRB, 0.23495E+00_JPRB, 0.23139E+00_JPRB, 0.22587E+00_JPRB ,&
&  0.21973E+00_JPRB, 0.21517E+00_JPRB, 0.21194E+00_JPRB, 0.21105E+00_JPRB, 0.20992E+00_JPRB, 0.20967E+00_JPRB, 0.20966E+00_JPRB ,&
&  0.20879E+00_JPRB, 0.20775E+00_JPRB, 0.20645E+00_JPRB, 0.20579E+00_JPRB, 0.20646E+00_JPRB, 0.20762E+00_JPRB, 0.21007E+00_JPRB ,&
&  0.21246E+00_JPRB, 0.21386E+00_JPRB, 0.21536E+00_JPRB, 0.21587E+00_JPRB, 0.21602E+00_JPRB, 0.21633E+00_JPRB, 0.21637E+00_JPRB ,&
&  0.21730E+00_JPRB, 0.21750E+00_JPRB, 0.21682E+00_JPRB, 0.21570E+00_JPRB, 0.21272E+00_JPRB, 0.20873E+00_JPRB, 0.20572E+00_JPRB ,&
&  0.20310E+00_JPRB, 0.20138E+00_JPRB, 0.20081E+00_JPRB, 0.20014E+00_JPRB, 0.20045E+00_JPRB, 0.20091E+00_JPRB, 0.20128E+00_JPRB ,&
&  0.20197E+00_JPRB, 0.20259E+00_JPRB, 0.20320E+00_JPRB, 0.20478E+00_JPRB, 0.20607E+00_JPRB, 0.20806E+00_JPRB, 0.21013E+00_JPRB ,&
&  0.21156E+00_JPRB, 0.21371E+00_JPRB, 0.21555E+00_JPRB, 0.21757E+00_JPRB, 0.22381E+00_JPRB, 0.22851E+00_JPRB, 0.23814E+00_JPRB ,&
&  0.24363E+00_JPRB, 0.24338E+00_JPRB, 0.24112E+00_JPRB, 0.24896E+00_JPRB, 0.25073E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.25178E+00_JPRB, 0.25292E+00_JPRB, 0.25303E+00_JPRB, 0.25422E+00_JPRB, 0.25487E+00_JPRB, 0.25559E+00_JPRB, 0.25403E+00_JPRB ,&
&  0.25278E+00_JPRB, 0.25154E+00_JPRB, 0.25052E+00_JPRB, 0.24698E+00_JPRB, 0.24427E+00_JPRB, 0.24078E+00_JPRB, 0.23776E+00_JPRB ,&
&  0.23504E+00_JPRB, 0.23241E+00_JPRB, 0.22993E+00_JPRB, 0.22778E+00_JPRB, 0.22571E+00_JPRB, 0.22379E+00_JPRB, 0.22185E+00_JPRB ,&
&  0.21897E+00_JPRB, 0.21815E+00_JPRB, 0.21806E+00_JPRB, 0.21958E+00_JPRB, 0.22284E+00_JPRB, 0.22468E+00_JPRB, 0.22696E+00_JPRB ,&
&  0.22900E+00_JPRB, 0.22756E+00_JPRB, 0.23153E+00_JPRB, 0.23528E+00_JPRB, 0.23841E+00_JPRB, 0.24118E+00_JPRB, 0.24391E+00_JPRB ,&
&  0.24452E+00_JPRB, 0.23704E+00_JPRB, 0.23784E+00_JPRB, 0.23160E+00_JPRB, 0.22595E+00_JPRB, 0.22406E+00_JPRB, 0.21965E+00_JPRB ,&
&  0.21836E+00_JPRB, 0.21484E+00_JPRB, 0.21258E+00_JPRB, 0.21049E+00_JPRB, 0.20774E+00_JPRB, 0.20719E+00_JPRB, 0.20622E+00_JPRB ,&
&  0.20577E+00_JPRB, 0.20611E+00_JPRB, 0.20523E+00_JPRB, 0.20508E+00_JPRB, 0.20463E+00_JPRB, 0.20427E+00_JPRB, 0.20506E+00_JPRB ,&
&  0.20617E+00_JPRB, 0.20738E+00_JPRB, 0.20848E+00_JPRB, 0.20867E+00_JPRB, 0.20854E+00_JPRB, 0.20851E+00_JPRB, 0.20839E+00_JPRB ,&
&  0.20880E+00_JPRB, 0.20886E+00_JPRB, 0.20836E+00_JPRB, 0.20778E+00_JPRB, 0.20635E+00_JPRB, 0.20489E+00_JPRB, 0.20346E+00_JPRB ,&
&  0.20216E+00_JPRB, 0.20145E+00_JPRB, 0.20083E+00_JPRB, 0.20062E+00_JPRB, 0.20059E+00_JPRB, 0.20097E+00_JPRB, 0.20136E+00_JPRB ,&
&  0.20172E+00_JPRB, 0.20269E+00_JPRB, 0.20338E+00_JPRB, 0.20456E+00_JPRB, 0.20598E+00_JPRB, 0.20723E+00_JPRB, 0.20915E+00_JPRB ,&
&  0.21125E+00_JPRB, 0.21352E+00_JPRB, 0.21684E+00_JPRB, 0.21962E+00_JPRB, 0.22319E+00_JPRB, 0.22763E+00_JPRB, 0.23069E+00_JPRB ,&
&  0.23392E+00_JPRB, 0.23632E+00_JPRB, 0.23562E+00_JPRB, 0.24470E+00_JPRB, 0.24974E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.24111E+00_JPRB, 0.24224E+00_JPRB, 0.24281E+00_JPRB, 0.24324E+00_JPRB, 0.24455E+00_JPRB, 0.24450E+00_JPRB, 0.24420E+00_JPRB ,&
&  0.24156E+00_JPRB, 0.24070E+00_JPRB, 0.23920E+00_JPRB, 0.23697E+00_JPRB, 0.23463E+00_JPRB, 0.23047E+00_JPRB, 0.22812E+00_JPRB ,&
&  0.22591E+00_JPRB, 0.22354E+00_JPRB, 0.22211E+00_JPRB, 0.22007E+00_JPRB, 0.21831E+00_JPRB, 0.21620E+00_JPRB, 0.21556E+00_JPRB ,&
&  0.21442E+00_JPRB, 0.21429E+00_JPRB, 0.21529E+00_JPRB, 0.21594E+00_JPRB, 0.21733E+00_JPRB, 0.21877E+00_JPRB, 0.22067E+00_JPRB ,&
&  0.22081E+00_JPRB, 0.22151E+00_JPRB, 0.22231E+00_JPRB, 0.22292E+00_JPRB, 0.22531E+00_JPRB, 0.22699E+00_JPRB, 0.22765E+00_JPRB ,&
&  0.22715E+00_JPRB, 0.22600E+00_JPRB, 0.22380E+00_JPRB, 0.22125E+00_JPRB, 0.21846E+00_JPRB, 0.21551E+00_JPRB, 0.21268E+00_JPRB ,&
&  0.21057E+00_JPRB, 0.20857E+00_JPRB, 0.20706E+00_JPRB, 0.20583E+00_JPRB, 0.20512E+00_JPRB, 0.20457E+00_JPRB, 0.20399E+00_JPRB ,&
&  0.20373E+00_JPRB, 0.20354E+00_JPRB, 0.20319E+00_JPRB, 0.20333E+00_JPRB, 0.20333E+00_JPRB, 0.20352E+00_JPRB, 0.20375E+00_JPRB ,&
&  0.20416E+00_JPRB, 0.20433E+00_JPRB, 0.20436E+00_JPRB, 0.20408E+00_JPRB, 0.20379E+00_JPRB, 0.20343E+00_JPRB, 0.20307E+00_JPRB ,&
&  0.20268E+00_JPRB, 0.20270E+00_JPRB, 0.20235E+00_JPRB, 0.20210E+00_JPRB, 0.20176E+00_JPRB, 0.20111E+00_JPRB, 0.20090E+00_JPRB ,&
&  0.20096E+00_JPRB, 0.20109E+00_JPRB, 0.20081E+00_JPRB, 0.20088E+00_JPRB, 0.20108E+00_JPRB, 0.20142E+00_JPRB, 0.20246E+00_JPRB ,&
&  0.20308E+00_JPRB, 0.20372E+00_JPRB, 0.20430E+00_JPRB, 0.20515E+00_JPRB, 0.20668E+00_JPRB, 0.20821E+00_JPRB, 0.20993E+00_JPRB ,&
&  0.21200E+00_JPRB, 0.21391E+00_JPRB, 0.21582E+00_JPRB, 0.21770E+00_JPRB, 0.22088E+00_JPRB, 0.22298E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.22801E+00_JPRB, 0.22563E+00_JPRB, 0.23520E+00_JPRB, 0.23568E+00_JPRB, 0.23690E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.22293E+00_JPRB, 0.22260E+00_JPRB, 0.22111E+00_JPRB, 0.21943E+00_JPRB, 0.20997E+00_JPRB, 0.21700E+00_JPRB, 0.21639E+00_JPRB ,&
&  0.21732E+00_JPRB, 0.21845E+00_JPRB, 0.22060E+00_JPRB, 0.22121E+00_JPRB, 0.22117E+00_JPRB, 0.22134E+00_JPRB, 0.22126E+00_JPRB ,&
&  0.22106E+00_JPRB, 0.22023E+00_JPRB, 0.21992E+00_JPRB, 0.21930E+00_JPRB, 0.21863E+00_JPRB, 0.21741E+00_JPRB, 0.21607E+00_JPRB ,&
&  0.21545E+00_JPRB, 0.21550E+00_JPRB, 0.21595E+00_JPRB, 0.21356E+00_JPRB, 0.21240E+00_JPRB, 0.21160E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21049E+00_JPRB, 0.21026E+00_JPRB, 0.21024E+00_JPRB, 0.21018E+00_JPRB, 0.21204E+00_JPRB, 0.21265E+00_JPRB, 0.21312E+00_JPRB ,&
&  0.21326E+00_JPRB, 0.21315E+00_JPRB, 0.21237E+00_JPRB, 0.21163E+00_JPRB, 0.21088E+00_JPRB, 0.20949E+00_JPRB, 0.20806E+00_JPRB ,&
&  0.20685E+00_JPRB, 0.20594E+00_JPRB, 0.20487E+00_JPRB, 0.20406E+00_JPRB, 0.20341E+00_JPRB, 0.20279E+00_JPRB, 0.20240E+00_JPRB ,&
&  0.20233E+00_JPRB, 0.20238E+00_JPRB, 0.20242E+00_JPRB, 0.20245E+00_JPRB, 0.20246E+00_JPRB, 0.20252E+00_JPRB, 0.20261E+00_JPRB ,&
&  0.20318E+00_JPRB, 0.20324E+00_JPRB, 0.20308E+00_JPRB, 0.20275E+00_JPRB, 0.20269E+00_JPRB, 0.20234E+00_JPRB, 0.20210E+00_JPRB ,&
&  0.20191E+00_JPRB, 0.20166E+00_JPRB, 0.20155E+00_JPRB, 0.20153E+00_JPRB, 0.20156E+00_JPRB, 0.20155E+00_JPRB, 0.20168E+00_JPRB ,&
&  0.20187E+00_JPRB, 0.20212E+00_JPRB, 0.20259E+00_JPRB, 0.20294E+00_JPRB, 0.20327E+00_JPRB, 0.20355E+00_JPRB, 0.20417E+00_JPRB ,&
&  0.20482E+00_JPRB, 0.20545E+00_JPRB, 0.20607E+00_JPRB, 0.20646E+00_JPRB, 0.20740E+00_JPRB, 0.20835E+00_JPRB, 0.20939E+00_JPRB ,&
&  0.21053E+00_JPRB, 0.21195E+00_JPRB, 0.21345E+00_JPRB, 0.21497E+00_JPRB, 0.21701E+00_JPRB, 0.21860E+00_JPRB, 0.22016E+00_JPRB ,&
&  0.22191E+00_JPRB, 0.22520E+00_JPRB, 0.22588E+00_JPRB, 0.22578E+00_JPRB, 0.22472E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.21798E+00_JPRB, 0.21788E+00_JPRB, 0.21764E+00_JPRB, 0.21704E+00_JPRB, 0.21600E+00_JPRB, 0.21574E+00_JPRB, 0.21540E+00_JPRB ,&
&  0.21494E+00_JPRB, 0.21402E+00_JPRB, 0.21401E+00_JPRB, 0.21389E+00_JPRB, 0.21375E+00_JPRB, 0.21358E+00_JPRB, 0.21322E+00_JPRB ,&
&  0.21252E+00_JPRB, 0.21145E+00_JPRB, 0.20833E+00_JPRB, 0.20682E+00_JPRB, 0.20583E+00_JPRB, 0.20528E+00_JPRB, 0.20481E+00_JPRB ,&
&  0.20453E+00_JPRB, 0.20437E+00_JPRB, 0.20422E+00_JPRB, 0.20478E+00_JPRB, 0.20503E+00_JPRB, 0.20512E+00_JPRB, 0.20518E+00_JPRB ,&
&  0.20518E+00_JPRB, 0.20530E+00_JPRB, 0.20532E+00_JPRB, 0.20530E+00_JPRB, 0.20547E+00_JPRB, 0.20549E+00_JPRB, 0.20526E+00_JPRB ,&
&  0.20500E+00_JPRB, 0.20469E+00_JPRB, 0.20440E+00_JPRB, 0.20402E+00_JPRB, 0.20376E+00_JPRB, 0.20354E+00_JPRB, 0.20347E+00_JPRB ,&
&  0.20328E+00_JPRB, 0.20315E+00_JPRB, 0.20300E+00_JPRB, 0.20295E+00_JPRB, 0.20284E+00_JPRB, 0.20280E+00_JPRB, 0.20265E+00_JPRB ,&
&  0.20279E+00_JPRB, 0.20284E+00_JPRB, 0.20296E+00_JPRB, 0.20303E+00_JPRB, 0.20309E+00_JPRB, 0.20310E+00_JPRB, 0.20304E+00_JPRB ,&
&  0.20286E+00_JPRB, 0.20276E+00_JPRB, 0.20262E+00_JPRB, 0.20252E+00_JPRB, 0.20251E+00_JPRB, 0.20249E+00_JPRB, 0.20251E+00_JPRB ,&
&  0.20254E+00_JPRB, 0.20269E+00_JPRB, 0.20274E+00_JPRB, 0.20282E+00_JPRB, 0.20291E+00_JPRB, 0.20301E+00_JPRB, 0.20308E+00_JPRB ,&
&  0.20320E+00_JPRB, 0.20331E+00_JPRB, 0.20345E+00_JPRB, 0.20363E+00_JPRB, 0.20385E+00_JPRB, 0.20417E+00_JPRB, 0.20454E+00_JPRB ,&
&  0.20491E+00_JPRB, 0.20530E+00_JPRB, 0.20568E+00_JPRB, 0.20605E+00_JPRB, 0.20665E+00_JPRB, 0.20726E+00_JPRB, 0.20803E+00_JPRB ,&
&  0.20876E+00_JPRB, 0.20948E+00_JPRB, 0.21013E+00_JPRB, 0.21085E+00_JPRB, 0.21187E+00_JPRB, 0.21236E+00_JPRB, 0.21277E+00_JPRB ,&
&  0.21345E+00_JPRB, 0.21417E+00_JPRB, 0.21485E+00_JPRB, 0.21543E+00_JPRB, 0.21636E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB, 0.20513E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB, 0.83378E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.83799E+00_JPRB, 0.83562E+00_JPRB, 0.83351E+00_JPRB, 0.83125E+00_JPRB, 0.82887E+00_JPRB, 0.82695E+00_JPRB, 0.82516E+00_JPRB ,&
&  0.82359E+00_JPRB, 0.82278E+00_JPRB, 0.82237E+00_JPRB, 0.82215E+00_JPRB, 0.82219E+00_JPRB, 0.82214E+00_JPRB, 0.82219E+00_JPRB ,&
&  0.82212E+00_JPRB, 0.82220E+00_JPRB, 0.82219E+00_JPRB, 0.82181E+00_JPRB, 0.82111E+00_JPRB, 0.82019E+00_JPRB, 0.81899E+00_JPRB ,&
&  0.81728E+00_JPRB, 0.81542E+00_JPRB, 0.81364E+00_JPRB, 0.81171E+00_JPRB, 0.80997E+00_JPRB, 0.80800E+00_JPRB, 0.80590E+00_JPRB ,&
&  0.80402E+00_JPRB, 0.80292E+00_JPRB, 0.80203E+00_JPRB, 0.80114E+00_JPRB, 0.80069E+00_JPRB, 0.80154E+00_JPRB, 0.80227E+00_JPRB ,&
&  0.80307E+00_JPRB, 0.80404E+00_JPRB, 0.80615E+00_JPRB, 0.80825E+00_JPRB, 0.81022E+00_JPRB, 0.81226E+00_JPRB, 0.81433E+00_JPRB ,&
&  0.81536E+00_JPRB, 0.81589E+00_JPRB, 0.81609E+00_JPRB, 0.81634E+00_JPRB, 0.81609E+00_JPRB, 0.81608E+00_JPRB, 0.81609E+00_JPRB ,&
&  0.81598E+00_JPRB, 0.81590E+00_JPRB, 0.81559E+00_JPRB, 0.81454E+00_JPRB, 0.81183E+00_JPRB, 0.80991E+00_JPRB, 0.80915E+00_JPRB ,&
&  0.80929E+00_JPRB, 0.80963E+00_JPRB, 0.81041E+00_JPRB, 0.81158E+00_JPRB, 0.81353E+00_JPRB, 0.81563E+00_JPRB, 0.81810E+00_JPRB ,&
&  0.82077E+00_JPRB, 0.82405E+00_JPRB, 0.82825E+00_JPRB, 0.83265E+00_JPRB, 0.83753E+00_JPRB, 0.84327E+00_JPRB, 0.84851E+00_JPRB ,&
&  0.85395E+00_JPRB, 0.85969E+00_JPRB, 0.86457E+00_JPRB, 0.86926E+00_JPRB, 0.87386E+00_JPRB, 0.87859E+00_JPRB, 0.88215E+00_JPRB ,&
&  0.88388E+00_JPRB, 0.88514E+00_JPRB, 0.88606E+00_JPRB, 0.88540E+00_JPRB, 0.88364E+00_JPRB, 0.88223E+00_JPRB, 0.88073E+00_JPRB ,&
&  0.87883E+00_JPRB, 0.87639E+00_JPRB, 0.87407E+00_JPRB, 0.87147E+00_JPRB, 0.86784E+00_JPRB, 0.86384E+00_JPRB, 0.86004E+00_JPRB ,&
&  0.85628E+00_JPRB, 0.85214E+00_JPRB, 0.84833E+00_JPRB, 0.84468E+00_JPRB, 0.84127E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.86493E+00_JPRB, 0.85597E+00_JPRB, 0.84782E+00_JPRB, 0.84038E+00_JPRB, 0.83429E+00_JPRB, 0.82961E+00_JPRB, 0.82567E+00_JPRB ,&
&  0.82229E+00_JPRB, 0.82068E+00_JPRB, 0.81903E+00_JPRB, 0.81749E+00_JPRB, 0.81601E+00_JPRB, 0.81407E+00_JPRB, 0.81171E+00_JPRB ,&
&  0.80956E+00_JPRB, 0.80815E+00_JPRB, 0.80672E+00_JPRB, 0.80485E+00_JPRB, 0.80152E+00_JPRB, 0.79651E+00_JPRB, 0.79065E+00_JPRB ,&
&  0.78382E+00_JPRB, 0.77561E+00_JPRB, 0.76624E+00_JPRB, 0.75646E+00_JPRB, 0.75142E+00_JPRB, 0.74558E+00_JPRB, 0.73997E+00_JPRB ,&
&  0.73506E+00_JPRB, 0.73352E+00_JPRB, 0.73239E+00_JPRB, 0.73164E+00_JPRB, 0.73170E+00_JPRB, 0.73340E+00_JPRB, 0.73432E+00_JPRB ,&
&  0.73499E+00_JPRB, 0.73716E+00_JPRB, 0.74371E+00_JPRB, 0.75072E+00_JPRB, 0.75887E+00_JPRB, 0.76880E+00_JPRB, 0.78011E+00_JPRB ,&
&  0.78865E+00_JPRB, 0.79378E+00_JPRB, 0.79878E+00_JPRB, 0.80346E+00_JPRB, 0.80648E+00_JPRB, 0.80850E+00_JPRB, 0.80974E+00_JPRB ,&
&  0.81259E+00_JPRB, 0.81448E+00_JPRB, 0.81626E+00_JPRB, 0.81753E+00_JPRB, 0.81679E+00_JPRB, 0.81596E+00_JPRB, 0.81488E+00_JPRB ,&
&  0.81277E+00_JPRB, 0.81246E+00_JPRB, 0.81389E+00_JPRB, 0.81550E+00_JPRB, 0.81897E+00_JPRB, 0.82430E+00_JPRB, 0.83082E+00_JPRB ,&
&  0.83996E+00_JPRB, 0.85259E+00_JPRB, 0.86810E+00_JPRB, 0.88539E+00_JPRB, 0.90542E+00_JPRB, 0.93043E+00_JPRB, 0.95063E+00_JPRB ,&
&  0.96900E+00_JPRB, 0.98469E+00_JPRB, 0.99870E+00_JPRB, 0.10088E+01_JPRB, 0.10161E+01_JPRB, 0.10231E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.10195E+01_JPRB, 0.10178E+01_JPRB, 0.10129E+01_JPRB, 0.10067E+01_JPRB, 0.10003E+01_JPRB, 0.99420E+00_JPRB ,&
&  0.98502E+00_JPRB, 0.97558E+00_JPRB, 0.96628E+00_JPRB, 0.95719E+00_JPRB, 0.94684E+00_JPRB, 0.93675E+00_JPRB, 0.92676E+00_JPRB ,&
&  0.91671E+00_JPRB, 0.90590E+00_JPRB, 0.89554E+00_JPRB, 0.88546E+00_JPRB, 0.87532E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.89782E+00_JPRB, 0.88252E+00_JPRB, 0.86858E+00_JPRB, 0.85625E+00_JPRB, 0.84554E+00_JPRB, 0.83707E+00_JPRB, 0.83024E+00_JPRB ,&
&  0.82470E+00_JPRB, 0.82148E+00_JPRB, 0.81830E+00_JPRB, 0.81481E+00_JPRB, 0.81124E+00_JPRB, 0.81071E+00_JPRB, 0.80817E+00_JPRB ,&
&  0.80440E+00_JPRB, 0.79945E+00_JPRB, 0.79467E+00_JPRB, 0.78749E+00_JPRB, 0.77721E+00_JPRB, 0.76504E+00_JPRB, 0.75008E+00_JPRB ,&
&  0.74061E+00_JPRB, 0.72839E+00_JPRB, 0.71545E+00_JPRB, 0.70215E+00_JPRB, 0.69287E+00_JPRB, 0.67164E+00_JPRB, 0.66427E+00_JPRB ,&
&  0.65943E+00_JPRB, 0.64406E+00_JPRB, 0.63596E+00_JPRB, 0.63211E+00_JPRB, 0.64613E+00_JPRB, 0.66475E+00_JPRB, 0.67511E+00_JPRB ,&
&  0.66429E+00_JPRB, 0.67286E+00_JPRB, 0.66991E+00_JPRB, 0.66708E+00_JPRB, 0.66827E+00_JPRB, 0.67291E+00_JPRB, 0.69711E+00_JPRB ,&
&  0.74082E+00_JPRB, 0.75791E+00_JPRB, 0.76803E+00_JPRB, 0.77783E+00_JPRB, 0.78434E+00_JPRB, 0.79066E+00_JPRB, 0.79528E+00_JPRB ,&
&  0.80035E+00_JPRB, 0.80470E+00_JPRB, 0.81083E+00_JPRB, 0.81370E+00_JPRB, 0.81846E+00_JPRB, 0.82328E+00_JPRB, 0.82775E+00_JPRB ,&
&  0.82510E+00_JPRB, 0.82423E+00_JPRB, 0.82542E+00_JPRB, 0.82940E+00_JPRB, 0.83639E+00_JPRB, 0.85009E+00_JPRB, 0.86645E+00_JPRB ,&
&  0.88441E+00_JPRB, 0.89874E+00_JPRB, 0.92177E+00_JPRB, 0.95262E+00_JPRB, 0.98993E+00_JPRB, 0.10357E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.11270E+01_JPRB, 0.11645E+01_JPRB, 0.12007E+01_JPRB, 0.12193E+01_JPRB, 0.12294E+01_JPRB, 0.12364E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.12280E+01_JPRB, 0.12212E+01_JPRB, 0.12115E+01_JPRB, 0.11981E+01_JPRB, 0.11802E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11121E+01_JPRB, 0.10888E+01_JPRB, 0.10660E+01_JPRB, 0.10431E+01_JPRB, 0.10221E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.98512E+00_JPRB, 0.96658E+00_JPRB, 0.94921E+00_JPRB, 0.93228E+00_JPRB, 0.91521E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.92984E+00_JPRB, 0.91303E+00_JPRB, 0.89517E+00_JPRB, 0.87890E+00_JPRB, 0.86516E+00_JPRB, 0.85459E+00_JPRB, 0.84704E+00_JPRB ,&
&  0.84055E+00_JPRB, 0.83269E+00_JPRB, 0.82440E+00_JPRB, 0.81557E+00_JPRB, 0.80787E+00_JPRB, 0.80457E+00_JPRB, 0.80194E+00_JPRB ,&
&  0.80000E+00_JPRB, 0.79376E+00_JPRB, 0.78539E+00_JPRB, 0.77207E+00_JPRB, 0.75767E+00_JPRB, 0.74439E+00_JPRB, 0.72863E+00_JPRB ,&
&  0.71637E+00_JPRB, 0.70380E+00_JPRB, 0.68481E+00_JPRB, 0.67380E+00_JPRB, 0.64758E+00_JPRB, 0.63000E+00_JPRB, 0.61878E+00_JPRB ,&
&  0.61059E+00_JPRB, 0.61193E+00_JPRB, 0.61657E+00_JPRB, 0.62532E+00_JPRB, 0.62878E+00_JPRB, 0.62571E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.61873E+00_JPRB, 0.61389E+00_JPRB, 0.61055E+00_JPRB, 0.60983E+00_JPRB, 0.61282E+00_JPRB, 0.62053E+00_JPRB, 0.63687E+00_JPRB ,&
&  0.67206E+00_JPRB, 0.71705E+00_JPRB, 0.75794E+00_JPRB, 0.77855E+00_JPRB, 0.77874E+00_JPRB, 0.78530E+00_JPRB, 0.78571E+00_JPRB ,&
&  0.78887E+00_JPRB, 0.78954E+00_JPRB, 0.79486E+00_JPRB, 0.80329E+00_JPRB, 0.81321E+00_JPRB, 0.82250E+00_JPRB, 0.83283E+00_JPRB ,&
&  0.83740E+00_JPRB, 0.84198E+00_JPRB, 0.84019E+00_JPRB, 0.84654E+00_JPRB, 0.86170E+00_JPRB, 0.88015E+00_JPRB, 0.88965E+00_JPRB ,&
&  0.90508E+00_JPRB, 0.92559E+00_JPRB, 0.96338E+00_JPRB, 0.10202E+01_JPRB, 0.10817E+01_JPRB, 0.11385E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.12441E+01_JPRB, 0.13069E+01_JPRB, 0.13677E+01_JPRB, 0.13931E+01_JPRB, 0.13933E+01_JPRB, 0.13997E+01_JPRB, 0.13900E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.13716E+01_JPRB, 0.13658E+01_JPRB, 0.13602E+01_JPRB, 0.13416E+01_JPRB, 0.13123E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12470E+01_JPRB, 0.12099E+01_JPRB, 0.11747E+01_JPRB, 0.11390E+01_JPRB, 0.11052E+01_JPRB, 0.10723E+01_JPRB, 0.10445E+01_JPRB ,&
&  0.10175E+01_JPRB, 0.99636E+00_JPRB, 0.97672E+00_JPRB, 0.96069E+00_JPRB, 0.94546E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.96005E+00_JPRB, 0.94898E+00_JPRB, 0.93771E+00_JPRB, 0.92291E+00_JPRB, 0.90771E+00_JPRB, 0.89375E+00_JPRB, 0.88252E+00_JPRB ,&
&  0.87300E+00_JPRB, 0.86268E+00_JPRB, 0.85245E+00_JPRB, 0.84013E+00_JPRB, 0.82847E+00_JPRB, 0.81799E+00_JPRB, 0.80962E+00_JPRB ,&
&  0.80281E+00_JPRB, 0.79201E+00_JPRB, 0.78157E+00_JPRB, 0.76808E+00_JPRB, 0.75519E+00_JPRB, 0.74008E+00_JPRB, 0.72273E+00_JPRB ,&
&  0.70969E+00_JPRB, 0.69060E+00_JPRB, 0.67420E+00_JPRB, 0.66459E+00_JPRB, 0.63955E+00_JPRB, 0.62682E+00_JPRB, 0.62001E+00_JPRB ,&
&  0.63492E+00_JPRB, 0.63227E+00_JPRB, 0.64613E+00_JPRB, 0.62960E+00_JPRB, 0.61734E+00_JPRB, 0.60599E+00_JPRB, 0.59409E+00_JPRB ,&
&  0.58653E+00_JPRB, 0.57848E+00_JPRB, 0.57933E+00_JPRB, 0.58446E+00_JPRB, 0.58788E+00_JPRB, 0.59944E+00_JPRB, 0.61513E+00_JPRB ,&
&  0.65113E+00_JPRB, 0.74848E+00_JPRB, 0.76745E+00_JPRB, 0.78812E+00_JPRB, 0.79092E+00_JPRB, 0.79590E+00_JPRB, 0.79904E+00_JPRB ,&
&  0.80252E+00_JPRB, 0.80352E+00_JPRB, 0.80391E+00_JPRB, 0.80647E+00_JPRB, 0.82272E+00_JPRB, 0.83428E+00_JPRB, 0.84877E+00_JPRB ,&
&  0.85506E+00_JPRB, 0.86319E+00_JPRB, 0.86635E+00_JPRB, 0.87481E+00_JPRB, 0.89716E+00_JPRB, 0.91040E+00_JPRB, 0.91375E+00_JPRB ,&
&  0.93724E+00_JPRB, 0.97005E+00_JPRB, 0.10213E+01_JPRB, 0.10838E+01_JPRB, 0.11309E+01_JPRB, 0.11709E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12727E+01_JPRB, 0.13417E+01_JPRB, 0.14155E+01_JPRB, 0.14494E+01_JPRB, 0.14553E+01_JPRB, 0.14468E+01_JPRB, 0.14345E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.14113E+01_JPRB, 0.13996E+01_JPRB, 0.13936E+01_JPRB, 0.13703E+01_JPRB, 0.13413E+01_JPRB, 0.13105E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.12379E+01_JPRB, 0.12097E+01_JPRB, 0.11773E+01_JPRB, 0.11432E+01_JPRB, 0.11099E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10209E+01_JPRB, 0.99737E+00_JPRB, 0.98211E+00_JPRB, 0.96853E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.98401E+00_JPRB, 0.97341E+00_JPRB, 0.96350E+00_JPRB, 0.95377E+00_JPRB, 0.94268E+00_JPRB, 0.93135E+00_JPRB, 0.91958E+00_JPRB ,&
&  0.90921E+00_JPRB, 0.89935E+00_JPRB, 0.88868E+00_JPRB, 0.87497E+00_JPRB, 0.86064E+00_JPRB, 0.84461E+00_JPRB, 0.83171E+00_JPRB ,&
&  0.81835E+00_JPRB, 0.80416E+00_JPRB, 0.78411E+00_JPRB, 0.76541E+00_JPRB, 0.75254E+00_JPRB, 0.74009E+00_JPRB, 0.72450E+00_JPRB ,&
&  0.71077E+00_JPRB, 0.69244E+00_JPRB, 0.67092E+00_JPRB, 0.65282E+00_JPRB, 0.62997E+00_JPRB, 0.62531E+00_JPRB, 0.65799E+00_JPRB ,&
&  0.67437E+00_JPRB, 0.67468E+00_JPRB, 0.67046E+00_JPRB, 0.66113E+00_JPRB, 0.65144E+00_JPRB, 0.60473E+00_JPRB, 0.61059E+00_JPRB ,&
&  0.59210E+00_JPRB, 0.57661E+00_JPRB, 0.57517E+00_JPRB, 0.57643E+00_JPRB, 0.58347E+00_JPRB, 0.60019E+00_JPRB, 0.61902E+00_JPRB ,&
&  0.65411E+00_JPRB, 0.76525E+00_JPRB, 0.78605E+00_JPRB, 0.80568E+00_JPRB, 0.81123E+00_JPRB, 0.81613E+00_JPRB, 0.82159E+00_JPRB ,&
&  0.82460E+00_JPRB, 0.83057E+00_JPRB, 0.83696E+00_JPRB, 0.84655E+00_JPRB, 0.85816E+00_JPRB, 0.86351E+00_JPRB, 0.88030E+00_JPRB ,&
&  0.89283E+00_JPRB, 0.90480E+00_JPRB, 0.91185E+00_JPRB, 0.91133E+00_JPRB, 0.92276E+00_JPRB, 0.91990E+00_JPRB, 0.92081E+00_JPRB ,&
&  0.96652E+00_JPRB, 0.10334E+01_JPRB, 0.10921E+01_JPRB, 0.11460E+01_JPRB, 0.11938E+01_JPRB, 0.12469E+01_JPRB, 0.13116E+01_JPRB ,&
&  0.13570E+01_JPRB, 0.14216E+01_JPRB, 0.14997E+01_JPRB, 0.15264E+01_JPRB, 0.15259E+01_JPRB, 0.15219E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.15313E+01_JPRB, 0.15394E+01_JPRB, 0.14982E+01_JPRB, 0.14738E+01_JPRB, 0.14214E+01_JPRB, 0.13540E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.12308E+01_JPRB, 0.11719E+01_JPRB, 0.11361E+01_JPRB, 0.11102E+01_JPRB, 0.10909E+01_JPRB, 0.10777E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10398E+01_JPRB, 0.10240E+01_JPRB, 0.10083E+01_JPRB, 0.99485E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.99088E+00_JPRB, 0.98357E+00_JPRB, 0.97296E+00_JPRB, 0.96370E+00_JPRB, 0.95147E+00_JPRB, 0.94248E+00_JPRB, 0.93348E+00_JPRB ,&
&  0.92779E+00_JPRB, 0.92426E+00_JPRB, 0.91994E+00_JPRB, 0.91539E+00_JPRB, 0.90762E+00_JPRB, 0.89774E+00_JPRB, 0.88451E+00_JPRB ,&
&  0.86524E+00_JPRB, 0.82127E+00_JPRB, 0.78964E+00_JPRB, 0.77487E+00_JPRB, 0.76006E+00_JPRB, 0.74764E+00_JPRB, 0.73544E+00_JPRB ,&
&  0.71753E+00_JPRB, 0.70155E+00_JPRB, 0.68250E+00_JPRB, 0.65667E+00_JPRB, 0.64402E+00_JPRB, 0.65388E+00_JPRB, 0.65942E+00_JPRB ,&
&  0.66695E+00_JPRB, 0.68395E+00_JPRB, 0.68165E+00_JPRB, 0.67145E+00_JPRB, 0.67337E+00_JPRB, 0.67439E+00_JPRB, 0.63358E+00_JPRB ,&
&  0.60321E+00_JPRB, 0.57965E+00_JPRB, 0.57566E+00_JPRB, 0.57596E+00_JPRB, 0.58398E+00_JPRB, 0.60207E+00_JPRB, 0.64765E+00_JPRB ,&
&  0.72665E+00_JPRB, 0.82140E+00_JPRB, 0.81964E+00_JPRB, 0.82167E+00_JPRB, 0.82950E+00_JPRB, 0.83658E+00_JPRB, 0.84467E+00_JPRB ,&
&  0.85299E+00_JPRB, 0.86698E+00_JPRB, 0.88498E+00_JPRB, 0.90288E+00_JPRB, 0.92500E+00_JPRB, 0.94114E+00_JPRB, 0.95384E+00_JPRB ,&
&  0.97770E+00_JPRB, 0.96522E+00_JPRB, 0.96435E+00_JPRB, 0.95555E+00_JPRB, 0.94904E+00_JPRB, 0.93361E+00_JPRB, 0.95774E+00_JPRB ,&
&  0.10158E+01_JPRB, 0.10747E+01_JPRB, 0.11381E+01_JPRB, 0.11905E+01_JPRB, 0.12461E+01_JPRB, 0.13204E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.14766E+01_JPRB, 0.15746E+01_JPRB, 0.17131E+01_JPRB, 0.16961E+01_JPRB, 0.16696E+01_JPRB, 0.16547E+01_JPRB, 0.16608E+01_JPRB ,&
&  0.16614E+01_JPRB, 0.16805E+01_JPRB, 0.16078E+01_JPRB, 0.14998E+01_JPRB, 0.14031E+01_JPRB, 0.12909E+01_JPRB, 0.12244E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.11063E+01_JPRB, 0.10713E+01_JPRB, 0.10520E+01_JPRB, 0.10396E+01_JPRB, 0.10252E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10072E+01_JPRB, 0.10044E+01_JPRB, 0.10042E+01_JPRB, 0.10003E+01_JPRB, 0.99616E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.97567E+00_JPRB, 0.97907E+00_JPRB, 0.97760E+00_JPRB, 0.96947E+00_JPRB, 0.94535E+00_JPRB, 0.90979E+00_JPRB, 0.88660E+00_JPRB ,&
&  0.87181E+00_JPRB, 0.86845E+00_JPRB, 0.86425E+00_JPRB, 0.86006E+00_JPRB, 0.86156E+00_JPRB, 0.86877E+00_JPRB, 0.87570E+00_JPRB ,&
&  0.87636E+00_JPRB, 0.86464E+00_JPRB, 0.84260E+00_JPRB, 0.80855E+00_JPRB, 0.78128E+00_JPRB, 0.76098E+00_JPRB, 0.74902E+00_JPRB ,&
&  0.73990E+00_JPRB, 0.72712E+00_JPRB, 0.71137E+00_JPRB, 0.68548E+00_JPRB, 0.66767E+00_JPRB, 0.65379E+00_JPRB, 0.66088E+00_JPRB ,&
&  0.68027E+00_JPRB, 0.68585E+00_JPRB, 0.68151E+00_JPRB, 0.69451E+00_JPRB, 0.69864E+00_JPRB, 0.70877E+00_JPRB, 0.65737E+00_JPRB ,&
&  0.62005E+00_JPRB, 0.60378E+00_JPRB, 0.59177E+00_JPRB, 0.58880E+00_JPRB, 0.60101E+00_JPRB, 0.62745E+00_JPRB, 0.68170E+00_JPRB ,&
&  0.79699E+00_JPRB, 0.82617E+00_JPRB, 0.83717E+00_JPRB, 0.85255E+00_JPRB, 0.86453E+00_JPRB, 0.86986E+00_JPRB, 0.88070E+00_JPRB ,&
&  0.90236E+00_JPRB, 0.93827E+00_JPRB, 0.97507E+00_JPRB, 0.98101E+00_JPRB, 0.98319E+00_JPRB, 0.98715E+00_JPRB, 0.10996E+01_JPRB ,&
&  0.13987E+01_JPRB, 0.12759E+01_JPRB, 0.11246E+01_JPRB, 0.10339E+01_JPRB, 0.99115E+00_JPRB, 0.97421E+00_JPRB, 0.10065E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.11199E+01_JPRB, 0.12021E+01_JPRB, 0.12665E+01_JPRB, 0.13337E+01_JPRB, 0.14258E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.16837E+01_JPRB, 0.18559E+01_JPRB, 0.22250E+01_JPRB, 0.19831E+01_JPRB, 0.18425E+01_JPRB, 0.17937E+01_JPRB, 0.17775E+01_JPRB ,&
&  0.17242E+01_JPRB, 0.17267E+01_JPRB, 0.16133E+01_JPRB, 0.14577E+01_JPRB, 0.13764E+01_JPRB, 0.12864E+01_JPRB, 0.12197E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.10963E+01_JPRB, 0.10634E+01_JPRB, 0.10381E+01_JPRB, 0.10304E+01_JPRB, 0.10220E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.99643E+00_JPRB, 0.98431E+00_JPRB, 0.97607E+00_JPRB, 0.97035E+00_JPRB, 0.96941E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.94374E+00_JPRB, 0.96583E+00_JPRB, 0.98365E+00_JPRB, 0.96352E+00_JPRB, 0.92984E+00_JPRB, 0.86715E+00_JPRB, 0.81863E+00_JPRB ,&
&  0.78391E+00_JPRB, 0.77364E+00_JPRB, 0.76885E+00_JPRB, 0.76995E+00_JPRB, 0.77729E+00_JPRB, 0.79065E+00_JPRB, 0.79967E+00_JPRB ,&
&  0.80944E+00_JPRB, 0.80931E+00_JPRB, 0.80865E+00_JPRB, 0.79793E+00_JPRB, 0.77860E+00_JPRB, 0.75603E+00_JPRB, 0.74052E+00_JPRB ,&
&  0.73130E+00_JPRB, 0.72518E+00_JPRB, 0.71639E+00_JPRB, 0.70251E+00_JPRB, 0.69290E+00_JPRB, 0.67968E+00_JPRB, 0.68210E+00_JPRB ,&
&  0.67393E+00_JPRB, 0.67884E+00_JPRB, 0.69547E+00_JPRB, 0.70369E+00_JPRB, 0.71790E+00_JPRB, 0.73032E+00_JPRB, 0.69165E+00_JPRB ,&
&  0.64891E+00_JPRB, 0.62899E+00_JPRB, 0.62387E+00_JPRB, 0.62754E+00_JPRB, 0.67230E+00_JPRB, 0.77791E+00_JPRB, 0.82671E+00_JPRB ,&
&  0.85318E+00_JPRB, 0.87004E+00_JPRB, 0.88869E+00_JPRB, 0.91194E+00_JPRB, 0.91968E+00_JPRB, 0.92672E+00_JPRB, 0.94865E+00_JPRB ,&
&  0.98679E+00_JPRB, 0.10479E+01_JPRB, 0.10970E+01_JPRB, 0.98804E+00_JPRB, 0.94430E+00_JPRB, 0.98396E+00_JPRB, 0.10487E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.11230E+01_JPRB, 0.10967E+01_JPRB, 0.11222E+01_JPRB, 0.11275E+01_JPRB, 0.11333E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.12144E+01_JPRB, 0.13447E+01_JPRB, 0.14146E+01_JPRB, 0.15220E+01_JPRB, 0.16781E+01_JPRB, 0.18162E+01_JPRB ,&
&  0.19734E+01_JPRB, 0.20898E+01_JPRB, 0.21459E+01_JPRB, 0.20721E+01_JPRB, 0.20156E+01_JPRB, 0.20145E+01_JPRB, 0.19786E+01_JPRB ,&
&  0.18376E+01_JPRB, 0.17358E+01_JPRB, 0.16499E+01_JPRB, 0.15607E+01_JPRB, 0.14348E+01_JPRB, 0.13301E+01_JPRB, 0.12226E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.10425E+01_JPRB, 0.10035E+01_JPRB, 0.97718E+00_JPRB, 0.96885E+00_JPRB, 0.96580E+00_JPRB, 0.95349E+00_JPRB ,&
&  0.93462E+00_JPRB, 0.91427E+00_JPRB, 0.90469E+00_JPRB, 0.91126E+00_JPRB, 0.93427E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.99273E+00_JPRB, 0.98691E+00_JPRB, 0.95105E+00_JPRB, 0.95471E+00_JPRB, 0.92287E+00_JPRB, 0.89105E+00_JPRB, 0.84441E+00_JPRB ,&
&  0.81296E+00_JPRB, 0.77487E+00_JPRB, 0.74794E+00_JPRB, 0.71350E+00_JPRB, 0.69303E+00_JPRB, 0.68398E+00_JPRB, 0.70169E+00_JPRB ,&
&  0.72642E+00_JPRB, 0.74443E+00_JPRB, 0.75538E+00_JPRB, 0.75781E+00_JPRB, 0.75467E+00_JPRB, 0.74431E+00_JPRB, 0.73316E+00_JPRB ,&
&  0.72347E+00_JPRB, 0.72210E+00_JPRB, 0.71627E+00_JPRB, 0.70942E+00_JPRB, 0.70160E+00_JPRB, 0.69420E+00_JPRB, 0.69128E+00_JPRB ,&
&  0.68131E+00_JPRB, 0.67693E+00_JPRB, 0.68432E+00_JPRB, 0.70407E+00_JPRB, 0.72619E+00_JPRB, 0.74058E+00_JPRB, 0.70901E+00_JPRB ,&
&  0.67435E+00_JPRB, 0.66746E+00_JPRB, 0.68189E+00_JPRB, 0.77710E+00_JPRB, 0.81098E+00_JPRB, 0.82843E+00_JPRB, 0.85740E+00_JPRB ,&
&  0.84137E+00_JPRB, 0.84100E+00_JPRB, 0.86217E+00_JPRB, 0.93405E+00_JPRB, 0.95849E+00_JPRB, 0.97430E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.11219E+01_JPRB, 0.11896E+01_JPRB, 0.10946E+01_JPRB, 0.11535E+01_JPRB, 0.13362E+01_JPRB, 0.12393E+01_JPRB ,&
&  0.12163E+01_JPRB, 0.11914E+01_JPRB, 0.12277E+01_JPRB, 0.12634E+01_JPRB, 0.12593E+01_JPRB, 0.12955E+01_JPRB, 0.13299E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.13901E+01_JPRB, 0.15468E+01_JPRB, 0.17021E+01_JPRB, 0.18464E+01_JPRB, 0.20347E+01_JPRB, 0.22297E+01_JPRB ,&
&  0.24665E+01_JPRB, 0.26773E+01_JPRB, 0.28282E+01_JPRB, 0.28058E+01_JPRB, 0.27379E+01_JPRB, 0.26085E+01_JPRB, 0.24201E+01_JPRB ,&
&  0.21579E+01_JPRB, 0.19681E+01_JPRB, 0.18399E+01_JPRB, 0.17365E+01_JPRB, 0.16076E+01_JPRB, 0.14396E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.96043E+00_JPRB, 0.91875E+00_JPRB, 0.90714E+00_JPRB, 0.91198E+00_JPRB, 0.91656E+00_JPRB, 0.90972E+00_JPRB ,&
&  0.89888E+00_JPRB, 0.89138E+00_JPRB, 0.88754E+00_JPRB, 0.90157E+00_JPRB, 0.96177E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.10281E+01_JPRB, 0.10065E+01_JPRB, 0.98248E+00_JPRB, 0.97529E+00_JPRB, 0.96555E+00_JPRB, 0.91786E+00_JPRB, 0.84078E+00_JPRB ,&
&  0.84744E+00_JPRB, 0.80243E+00_JPRB, 0.79125E+00_JPRB, 0.77643E+00_JPRB, 0.75412E+00_JPRB, 0.71163E+00_JPRB, 0.67899E+00_JPRB ,&
&  0.67844E+00_JPRB, 0.69726E+00_JPRB, 0.71733E+00_JPRB, 0.73455E+00_JPRB, 0.74111E+00_JPRB, 0.73699E+00_JPRB, 0.72961E+00_JPRB ,&
&  0.72319E+00_JPRB, 0.72169E+00_JPRB, 0.72350E+00_JPRB, 0.72568E+00_JPRB, 0.72302E+00_JPRB, 0.71907E+00_JPRB, 0.71952E+00_JPRB ,&
&  0.71730E+00_JPRB, 0.71286E+00_JPRB, 0.70716E+00_JPRB, 0.71697E+00_JPRB, 0.72688E+00_JPRB, 0.74922E+00_JPRB, 0.76673E+00_JPRB ,&
&  0.70768E+00_JPRB, 0.72092E+00_JPRB, 0.80898E+00_JPRB, 0.82717E+00_JPRB, 0.84738E+00_JPRB, 0.86550E+00_JPRB, 0.88066E+00_JPRB ,&
&  0.89884E+00_JPRB, 0.92551E+00_JPRB, 0.95814E+00_JPRB, 0.98810E+00_JPRB, 0.10121E+01_JPRB, 0.10443E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.10830E+01_JPRB, 0.10510E+01_JPRB, 0.11239E+01_JPRB, 0.14530E+01_JPRB, 0.14853E+01_JPRB, 0.14769E+01_JPRB ,&
&  0.15055E+01_JPRB, 0.14581E+01_JPRB, 0.14538E+01_JPRB, 0.15012E+01_JPRB, 0.15291E+01_JPRB, 0.16072E+01_JPRB, 0.17225E+01_JPRB ,&
&  0.17347E+01_JPRB, 0.18561E+01_JPRB, 0.20123E+01_JPRB, 0.21506E+01_JPRB, 0.23044E+01_JPRB, 0.25329E+01_JPRB, 0.27897E+01_JPRB ,&
&  0.30597E+01_JPRB, 0.32908E+01_JPRB, 0.33950E+01_JPRB, 0.33463E+01_JPRB, 0.31962E+01_JPRB, 0.30001E+01_JPRB, 0.27534E+01_JPRB ,&
&  0.24335E+01_JPRB, 0.22066E+01_JPRB, 0.20533E+01_JPRB, 0.19032E+01_JPRB, 0.17655E+01_JPRB, 0.15838E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.10514E+01_JPRB, 0.93615E+00_JPRB, 0.91340E+00_JPRB, 0.91015E+00_JPRB, 0.91710E+00_JPRB, 0.93201E+00_JPRB, 0.94533E+00_JPRB ,&
&  0.95125E+00_JPRB, 0.93489E+00_JPRB, 0.93275E+00_JPRB, 0.96582E+00_JPRB, 0.10061E+01_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.99624E+00_JPRB, 0.98658E+00_JPRB, 0.98915E+00_JPRB, 0.10054E+01_JPRB, 0.99064E+00_JPRB, 0.94222E+00_JPRB, 0.89112E+00_JPRB ,&
&  0.10041E+01_JPRB, 0.98864E+00_JPRB, 0.10418E+01_JPRB, 0.88594E+00_JPRB, 0.83024E+00_JPRB, 0.80295E+00_JPRB, 0.76289E+00_JPRB ,&
&  0.73105E+00_JPRB, 0.71179E+00_JPRB, 0.70449E+00_JPRB, 0.71751E+00_JPRB, 0.72396E+00_JPRB, 0.72040E+00_JPRB, 0.71614E+00_JPRB ,&
&  0.71313E+00_JPRB, 0.71587E+00_JPRB, 0.72507E+00_JPRB, 0.72917E+00_JPRB, 0.73183E+00_JPRB, 0.73484E+00_JPRB, 0.74087E+00_JPRB ,&
&  0.72382E+00_JPRB, 0.71973E+00_JPRB, 0.72412E+00_JPRB, 0.72617E+00_JPRB, 0.73918E+00_JPRB, 0.74893E+00_JPRB, 0.77166E+00_JPRB ,&
&  0.81687E+00_JPRB, 0.79311E+00_JPRB, 0.87772E+00_JPRB, 0.88967E+00_JPRB, 0.90487E+00_JPRB, 0.91906E+00_JPRB, 0.93088E+00_JPRB ,&
&  0.94134E+00_JPRB, 0.96080E+00_JPRB, 0.98947E+00_JPRB, 0.10143E+01_JPRB, 0.10489E+01_JPRB, 0.10972E+01_JPRB, 0.11406E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.11024E+01_JPRB, 0.11900E+01_JPRB, 0.12462E+01_JPRB, 0.14818E+01_JPRB, 0.17499E+01_JPRB, 0.17315E+01_JPRB ,&
&  0.20187E+01_JPRB, 0.19116E+01_JPRB, 0.18309E+01_JPRB, 0.17605E+01_JPRB, 0.18175E+01_JPRB, 0.19244E+01_JPRB, 0.20543E+01_JPRB ,&
&  0.21761E+01_JPRB, 0.23004E+01_JPRB, 0.25065E+01_JPRB, 0.26847E+01_JPRB, 0.28720E+01_JPRB, 0.30904E+01_JPRB, 0.32966E+01_JPRB ,&
&  0.34800E+01_JPRB, 0.35957E+01_JPRB, 0.36626E+01_JPRB, 0.34661E+01_JPRB, 0.32915E+01_JPRB, 0.31049E+01_JPRB, 0.27346E+01_JPRB ,&
&  0.24494E+01_JPRB, 0.22335E+01_JPRB, 0.20543E+01_JPRB, 0.19213E+01_JPRB, 0.16871E+01_JPRB, 0.14927E+01_JPRB, 0.13074E+01_JPRB ,&
&  0.10656E+01_JPRB, 0.98104E+00_JPRB, 0.98646E+00_JPRB, 0.99926E+00_JPRB, 0.99743E+00_JPRB, 0.10056E+01_JPRB, 0.10099E+01_JPRB ,&
&  0.99240E+00_JPRB, 0.98749E+00_JPRB, 0.10178E+01_JPRB, 0.10168E+01_JPRB, 0.10074E+01_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.10521E+01_JPRB, 0.10665E+01_JPRB, 0.10735E+01_JPRB, 0.10641E+01_JPRB, 0.10403E+01_JPRB, 0.10545E+01_JPRB, 0.10121E+01_JPRB ,&
&  0.10932E+01_JPRB, 0.11225E+01_JPRB, 0.11158E+01_JPRB, 0.11188E+01_JPRB, 0.11346E+01_JPRB, 0.10484E+01_JPRB, 0.88983E+00_JPRB ,&
&  0.81250E+00_JPRB, 0.79897E+00_JPRB, 0.79241E+00_JPRB, 0.78523E+00_JPRB, 0.78253E+00_JPRB, 0.77603E+00_JPRB, 0.77220E+00_JPRB ,&
&  0.76889E+00_JPRB, 0.76343E+00_JPRB, 0.76651E+00_JPRB, 0.76849E+00_JPRB, 0.76559E+00_JPRB, 0.76460E+00_JPRB, 0.77059E+00_JPRB ,&
&  0.74635E+00_JPRB, 0.73176E+00_JPRB, 0.72224E+00_JPRB, 0.71113E+00_JPRB, 0.73485E+00_JPRB, 0.75574E+00_JPRB, 0.78934E+00_JPRB ,&
&  0.83702E+00_JPRB, 0.88502E+00_JPRB, 0.91395E+00_JPRB, 0.93886E+00_JPRB, 0.95891E+00_JPRB, 0.97746E+00_JPRB, 0.99506E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.10303E+01_JPRB, 0.10576E+01_JPRB, 0.10783E+01_JPRB, 0.11362E+01_JPRB, 0.11518E+01_JPRB, 0.12858E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.15150E+01_JPRB, 0.16168E+01_JPRB, 0.14904E+01_JPRB, 0.16762E+01_JPRB, 0.19130E+01_JPRB, 0.19428E+01_JPRB ,&
&  0.20343E+01_JPRB, 0.20464E+01_JPRB, 0.21247E+01_JPRB, 0.22298E+01_JPRB, 0.24925E+01_JPRB, 0.26876E+01_JPRB, 0.27841E+01_JPRB ,&
&  0.28761E+01_JPRB, 0.31867E+01_JPRB, 0.34051E+01_JPRB, 0.35386E+01_JPRB, 0.35968E+01_JPRB, 0.36503E+01_JPRB, 0.37358E+01_JPRB ,&
&  0.37989E+01_JPRB, 0.38593E+01_JPRB, 0.38450E+01_JPRB, 0.37222E+01_JPRB, 0.35318E+01_JPRB, 0.32704E+01_JPRB, 0.29146E+01_JPRB ,&
&  0.25981E+01_JPRB, 0.22064E+01_JPRB, 0.18867E+01_JPRB, 0.16912E+01_JPRB, 0.15090E+01_JPRB, 0.13306E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.10779E+01_JPRB, 0.11579E+01_JPRB, 0.11631E+01_JPRB, 0.11621E+01_JPRB, 0.11394E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.10651E+01_JPRB, 0.10794E+01_JPRB, 0.10690E+01_JPRB, 0.10548E+01_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.12069E+01_JPRB, 0.12007E+01_JPRB, 0.11886E+01_JPRB, 0.11683E+01_JPRB, 0.11637E+01_JPRB, 0.11677E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.12047E+01_JPRB, 0.12018E+01_JPRB, 0.11788E+01_JPRB, 0.11535E+01_JPRB, 0.11404E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.91275E+00_JPRB, 0.87117E+00_JPRB, 0.85933E+00_JPRB, 0.86783E+00_JPRB, 0.88680E+00_JPRB, 0.89370E+00_JPRB, 0.89672E+00_JPRB ,&
&  0.90109E+00_JPRB, 0.88668E+00_JPRB, 0.88777E+00_JPRB, 0.88433E+00_JPRB, 0.87202E+00_JPRB, 0.86098E+00_JPRB, 0.85025E+00_JPRB ,&
&  0.80228E+00_JPRB, 0.76966E+00_JPRB, 0.75292E+00_JPRB, 0.74353E+00_JPRB, 0.76186E+00_JPRB, 0.79597E+00_JPRB, 0.84288E+00_JPRB ,&
&  0.89809E+00_JPRB, 0.94167E+00_JPRB, 0.98130E+00_JPRB, 0.10179E+01_JPRB, 0.10439E+01_JPRB, 0.10691E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.11310E+01_JPRB, 0.11433E+01_JPRB, 0.11647E+01_JPRB, 0.11758E+01_JPRB, 0.13309E+01_JPRB, 0.15950E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.17697E+01_JPRB, 0.18815E+01_JPRB, 0.21223E+01_JPRB, 0.22625E+01_JPRB, 0.22356E+01_JPRB, 0.22986E+01_JPRB ,&
&  0.25181E+01_JPRB, 0.27701E+01_JPRB, 0.32859E+01_JPRB, 0.33311E+01_JPRB, 0.34280E+01_JPRB, 0.35567E+01_JPRB, 0.35431E+01_JPRB ,&
&  0.36532E+01_JPRB, 0.37812E+01_JPRB, 0.39737E+01_JPRB, 0.39832E+01_JPRB, 0.39223E+01_JPRB, 0.39546E+01_JPRB, 0.39483E+01_JPRB ,&
&  0.39944E+01_JPRB, 0.40393E+01_JPRB, 0.38647E+01_JPRB, 0.37037E+01_JPRB, 0.33842E+01_JPRB, 0.33791E+01_JPRB, 0.31368E+01_JPRB ,&
&  0.26448E+01_JPRB, 0.19893E+01_JPRB, 0.18080E+01_JPRB, 0.16090E+01_JPRB, 0.14754E+01_JPRB, 0.13296E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.12611E+01_JPRB, 0.12815E+01_JPRB, 0.12969E+01_JPRB, 0.13119E+01_JPRB, 0.12869E+01_JPRB, 0.11670E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11992E+01_JPRB, 0.12151E+01_JPRB, 0.12131E+01_JPRB, 0.12092E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.14285E+01_JPRB, 0.13913E+01_JPRB, 0.13598E+01_JPRB, 0.13408E+01_JPRB, 0.13180E+01_JPRB, 0.12852E+01_JPRB, 0.12512E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.12844E+01_JPRB, 0.12858E+01_JPRB, 0.12447E+01_JPRB, 0.11989E+01_JPRB, 0.11669E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.10146E+01_JPRB, 0.10165E+01_JPRB, 0.10569E+01_JPRB, 0.11650E+01_JPRB, 0.11877E+01_JPRB, 0.12279E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.11831E+01_JPRB, 0.11663E+01_JPRB, 0.11395E+01_JPRB, 0.10958E+01_JPRB, 0.10611E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.92763E+00_JPRB, 0.86175E+00_JPRB, 0.83766E+00_JPRB, 0.82614E+00_JPRB, 0.85305E+00_JPRB, 0.88689E+00_JPRB, 0.94780E+00_JPRB ,&
&  0.10035E+01_JPRB, 0.10571E+01_JPRB, 0.11065E+01_JPRB, 0.11583E+01_JPRB, 0.11959E+01_JPRB, 0.12315E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.12700E+01_JPRB, 0.12772E+01_JPRB, 0.12937E+01_JPRB, 0.12875E+01_JPRB, 0.13272E+01_JPRB, 0.16791E+01_JPRB, 0.21042E+01_JPRB ,&
&  0.22198E+01_JPRB, 0.23623E+01_JPRB, 0.30227E+01_JPRB, 0.39134E+01_JPRB, 0.36828E+01_JPRB, 0.33341E+01_JPRB, 0.32396E+01_JPRB ,&
&  0.34780E+01_JPRB, 0.38156E+01_JPRB, 0.41793E+01_JPRB, 0.43114E+01_JPRB, 0.42925E+01_JPRB, 0.44065E+01_JPRB, 0.44342E+01_JPRB ,&
&  0.44033E+01_JPRB, 0.44363E+01_JPRB, 0.43674E+01_JPRB, 0.42440E+01_JPRB, 0.41284E+01_JPRB, 0.42097E+01_JPRB, 0.42020E+01_JPRB ,&
&  0.41549E+01_JPRB, 0.40021E+01_JPRB, 0.37146E+01_JPRB, 0.35550E+01_JPRB, 0.29986E+01_JPRB, 0.29088E+01_JPRB, 0.32470E+01_JPRB ,&
&  0.26965E+01_JPRB, 0.21425E+01_JPRB, 0.18445E+01_JPRB, 0.16710E+01_JPRB, 0.16079E+01_JPRB, 0.16374E+01_JPRB, 0.15917E+01_JPRB ,&
&  0.14168E+01_JPRB, 0.14562E+01_JPRB, 0.15869E+01_JPRB, 0.16559E+01_JPRB, 0.16746E+01_JPRB, 0.16581E+01_JPRB, 0.15141E+01_JPRB ,&
&  0.15207E+01_JPRB, 0.14762E+01_JPRB, 0.14775E+01_JPRB, 0.14764E+01_JPRB, 0.14598E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.16599E+01_JPRB, 0.15810E+01_JPRB, 0.15348E+01_JPRB, 0.14722E+01_JPRB, 0.14006E+01_JPRB, 0.13367E+01_JPRB, 0.13022E+01_JPRB ,&
&  0.13294E+01_JPRB, 0.13446E+01_JPRB, 0.13282E+01_JPRB, 0.12774E+01_JPRB, 0.12376E+01_JPRB, 0.12103E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.11678E+01_JPRB, 0.11572E+01_JPRB, 0.11652E+01_JPRB, 0.13686E+01_JPRB, 0.15623E+01_JPRB, 0.17307E+01_JPRB ,&
&  0.17851E+01_JPRB, 0.18088E+01_JPRB, 0.17767E+01_JPRB, 0.16651E+01_JPRB, 0.15453E+01_JPRB, 0.14354E+01_JPRB, 0.13190E+01_JPRB ,&
&  0.11961E+01_JPRB, 0.10833E+01_JPRB, 0.10250E+01_JPRB, 0.10168E+01_JPRB, 0.10621E+01_JPRB, 0.10271E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.11581E+01_JPRB, 0.12275E+01_JPRB, 0.12789E+01_JPRB, 0.13382E+01_JPRB, 0.13746E+01_JPRB, 0.14031E+01_JPRB, 0.14094E+01_JPRB ,&
&  0.14321E+01_JPRB, 0.14118E+01_JPRB, 0.13115E+01_JPRB, 0.13699E+01_JPRB, 0.15637E+01_JPRB, 0.22036E+01_JPRB, 0.28803E+01_JPRB ,&
&  0.33059E+01_JPRB, 0.40119E+01_JPRB, 0.49416E+01_JPRB, 0.55216E+01_JPRB, 0.49611E+01_JPRB, 0.45266E+01_JPRB, 0.45559E+01_JPRB ,&
&  0.48465E+01_JPRB, 0.51733E+01_JPRB, 0.53085E+01_JPRB, 0.50884E+01_JPRB, 0.51418E+01_JPRB, 0.52449E+01_JPRB, 0.53075E+01_JPRB ,&
&  0.52428E+01_JPRB, 0.50333E+01_JPRB, 0.47793E+01_JPRB, 0.45439E+01_JPRB, 0.43887E+01_JPRB, 0.40818E+01_JPRB, 0.40044E+01_JPRB ,&
&  0.40966E+01_JPRB, 0.35016E+01_JPRB, 0.31746E+01_JPRB, 0.33112E+01_JPRB, 0.27492E+01_JPRB, 0.26896E+01_JPRB, 0.28874E+01_JPRB ,&
&  0.24748E+01_JPRB, 0.20872E+01_JPRB, 0.18989E+01_JPRB, 0.17459E+01_JPRB, 0.17534E+01_JPRB, 0.20397E+01_JPRB, 0.22511E+01_JPRB ,&
&  0.21372E+01_JPRB, 0.20284E+01_JPRB, 0.23219E+01_JPRB, 0.25161E+01_JPRB, 0.26266E+01_JPRB, 0.25145E+01_JPRB, 0.23212E+01_JPRB ,&
&  0.21626E+01_JPRB, 0.20558E+01_JPRB, 0.19696E+01_JPRB, 0.18735E+01_JPRB, 0.17727E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.18248E+01_JPRB, 0.17815E+01_JPRB, 0.17008E+01_JPRB, 0.15880E+01_JPRB, 0.14919E+01_JPRB, 0.14233E+01_JPRB, 0.13658E+01_JPRB ,&
&  0.13384E+01_JPRB, 0.13067E+01_JPRB, 0.12855E+01_JPRB, 0.12482E+01_JPRB, 0.12341E+01_JPRB, 0.12362E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.12421E+01_JPRB, 0.12547E+01_JPRB, 0.12100E+01_JPRB, 0.12325E+01_JPRB, 0.14472E+01_JPRB, 0.20618E+01_JPRB, 0.25143E+01_JPRB ,&
&  0.26812E+01_JPRB, 0.26885E+01_JPRB, 0.25842E+01_JPRB, 0.24837E+01_JPRB, 0.23073E+01_JPRB, 0.20510E+01_JPRB, 0.18833E+01_JPRB ,&
&  0.17217E+01_JPRB, 0.15507E+01_JPRB, 0.13655E+01_JPRB, 0.13535E+01_JPRB, 0.13435E+01_JPRB, 0.12734E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.14330E+01_JPRB, 0.14634E+01_JPRB, 0.14828E+01_JPRB, 0.15008E+01_JPRB, 0.15055E+01_JPRB, 0.14333E+01_JPRB, 0.13547E+01_JPRB ,&
&  0.13346E+01_JPRB, 0.13751E+01_JPRB, 0.14332E+01_JPRB, 0.16674E+01_JPRB, 0.22774E+01_JPRB, 0.33111E+01_JPRB, 0.42067E+01_JPRB ,&
&  0.48199E+01_JPRB, 0.51381E+01_JPRB, 0.48727E+01_JPRB, 0.53669E+01_JPRB, 0.61465E+01_JPRB, 0.59510E+01_JPRB, 0.56563E+01_JPRB ,&
&  0.56642E+01_JPRB, 0.56493E+01_JPRB, 0.55521E+01_JPRB, 0.56201E+01_JPRB, 0.60312E+01_JPRB, 0.62226E+01_JPRB, 0.61818E+01_JPRB ,&
&  0.54099E+01_JPRB, 0.51224E+01_JPRB, 0.49842E+01_JPRB, 0.47928E+01_JPRB, 0.42816E+01_JPRB, 0.40859E+01_JPRB, 0.41730E+01_JPRB ,&
&  0.40801E+01_JPRB, 0.40258E+01_JPRB, 0.32291E+01_JPRB, 0.30506E+01_JPRB, 0.23855E+01_JPRB, 0.23067E+01_JPRB, 0.25874E+01_JPRB ,&
&  0.23777E+01_JPRB, 0.21917E+01_JPRB, 0.19616E+01_JPRB, 0.19265E+01_JPRB, 0.24265E+01_JPRB, 0.30081E+01_JPRB, 0.37450E+01_JPRB ,&
&  0.37139E+01_JPRB, 0.31659E+01_JPRB, 0.34100E+01_JPRB, 0.35559E+01_JPRB, 0.37241E+01_JPRB, 0.35089E+01_JPRB, 0.31066E+01_JPRB ,&
&  0.27751E+01_JPRB, 0.25173E+01_JPRB, 0.22692E+01_JPRB, 0.20442E+01_JPRB, 0.19017E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.18988E+01_JPRB, 0.18420E+01_JPRB, 0.17369E+01_JPRB, 0.16086E+01_JPRB, 0.15140E+01_JPRB, 0.14417E+01_JPRB, 0.13833E+01_JPRB ,&
&  0.13235E+01_JPRB, 0.12720E+01_JPRB, 0.12433E+01_JPRB, 0.12252E+01_JPRB, 0.12199E+01_JPRB, 0.12223E+01_JPRB, 0.12288E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.12747E+01_JPRB, 0.12484E+01_JPRB, 0.12947E+01_JPRB, 0.16413E+01_JPRB, 0.24106E+01_JPRB, 0.31592E+01_JPRB ,&
&  0.34905E+01_JPRB, 0.34075E+01_JPRB, 0.32623E+01_JPRB, 0.31245E+01_JPRB, 0.30273E+01_JPRB, 0.28657E+01_JPRB, 0.26517E+01_JPRB ,&
&  0.24153E+01_JPRB, 0.19452E+01_JPRB, 0.17965E+01_JPRB, 0.16768E+01_JPRB, 0.16843E+01_JPRB, 0.16844E+01_JPRB, 0.16453E+01_JPRB ,&
&  0.15866E+01_JPRB, 0.15355E+01_JPRB, 0.14995E+01_JPRB, 0.14416E+01_JPRB, 0.14076E+01_JPRB, 0.13472E+01_JPRB, 0.13672E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.15954E+01_JPRB, 0.19909E+01_JPRB, 0.26717E+01_JPRB, 0.38435E+01_JPRB, 0.53649E+01_JPRB, 0.55900E+01_JPRB ,&
&  0.46910E+01_JPRB, 0.45988E+01_JPRB, 0.64042E+01_JPRB, 0.61280E+01_JPRB, 0.63453E+01_JPRB, 0.70800E+01_JPRB, 0.66638E+01_JPRB ,&
&  0.60100E+01_JPRB, 0.55753E+01_JPRB, 0.54360E+01_JPRB, 0.57614E+01_JPRB, 0.75278E+01_JPRB, 0.77857E+01_JPRB, 0.72356E+01_JPRB ,&
&  0.60739E+01_JPRB, 0.57693E+01_JPRB, 0.53065E+01_JPRB, 0.47982E+01_JPRB, 0.44970E+01_JPRB, 0.47212E+01_JPRB, 0.44746E+01_JPRB ,&
&  0.41797E+01_JPRB, 0.45259E+01_JPRB, 0.36266E+01_JPRB, 0.28064E+01_JPRB, 0.23564E+01_JPRB, 0.24157E+01_JPRB, 0.25731E+01_JPRB ,&
&  0.25553E+01_JPRB, 0.24675E+01_JPRB, 0.24499E+01_JPRB, 0.30335E+01_JPRB, 0.43423E+01_JPRB, 0.54039E+01_JPRB, 0.53236E+01_JPRB ,&
&  0.51149E+01_JPRB, 0.54954E+01_JPRB, 0.51579E+01_JPRB, 0.51165E+01_JPRB, 0.50315E+01_JPRB, 0.43901E+01_JPRB, 0.35571E+01_JPRB ,&
&  0.28543E+01_JPRB, 0.24085E+01_JPRB, 0.21673E+01_JPRB, 0.20279E+01_JPRB, 0.19466E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.20047E+01_JPRB, 0.18860E+01_JPRB, 0.17620E+01_JPRB, 0.16410E+01_JPRB, 0.15306E+01_JPRB, 0.14399E+01_JPRB, 0.13629E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.12355E+01_JPRB, 0.12227E+01_JPRB, 0.12313E+01_JPRB, 0.12437E+01_JPRB, 0.12289E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.13089E+01_JPRB, 0.12983E+01_JPRB, 0.14412E+01_JPRB, 0.17973E+01_JPRB, 0.23561E+01_JPRB, 0.31761E+01_JPRB ,&
&  0.37455E+01_JPRB, 0.35645E+01_JPRB, 0.32561E+01_JPRB, 0.33508E+01_JPRB, 0.35454E+01_JPRB, 0.35939E+01_JPRB, 0.34262E+01_JPRB ,&
&  0.29412E+01_JPRB, 0.25789E+01_JPRB, 0.23812E+01_JPRB, 0.21394E+01_JPRB, 0.20016E+01_JPRB, 0.18551E+01_JPRB, 0.16736E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.14391E+01_JPRB, 0.13994E+01_JPRB, 0.13851E+01_JPRB, 0.14245E+01_JPRB, 0.14848E+01_JPRB, 0.15891E+01_JPRB ,&
&  0.17883E+01_JPRB, 0.21518E+01_JPRB, 0.27930E+01_JPRB, 0.37634E+01_JPRB, 0.45013E+01_JPRB, 0.64864E+01_JPRB, 0.61429E+01_JPRB ,&
&  0.58186E+01_JPRB, 0.58578E+01_JPRB, 0.58495E+01_JPRB, 0.64037E+01_JPRB, 0.56687E+01_JPRB, 0.62214E+01_JPRB, 0.64810E+01_JPRB ,&
&  0.61865E+01_JPRB, 0.60254E+01_JPRB, 0.63604E+01_JPRB, 0.65254E+01_JPRB, 0.77996E+01_JPRB, 0.98410E+01_JPRB, 0.89604E+01_JPRB ,&
&  0.73666E+01_JPRB, 0.65305E+01_JPRB, 0.56388E+01_JPRB, 0.51547E+01_JPRB, 0.43786E+01_JPRB, 0.43670E+01_JPRB, 0.38454E+01_JPRB ,&
&  0.35772E+01_JPRB, 0.29384E+01_JPRB, 0.31786E+01_JPRB, 0.27175E+01_JPRB, 0.25780E+01_JPRB, 0.25451E+01_JPRB, 0.25945E+01_JPRB ,&
&  0.27946E+01_JPRB, 0.30400E+01_JPRB, 0.38055E+01_JPRB, 0.52682E+01_JPRB, 0.72006E+01_JPRB, 0.64972E+01_JPRB, 0.68458E+01_JPRB ,&
&  0.69387E+01_JPRB, 0.68519E+01_JPRB, 0.57855E+01_JPRB, 0.53976E+01_JPRB, 0.47915E+01_JPRB, 0.42733E+01_JPRB, 0.37173E+01_JPRB ,&
&  0.31449E+01_JPRB, 0.26699E+01_JPRB, 0.24053E+01_JPRB, 0.22449E+01_JPRB, 0.21200E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.19479E+01_JPRB, 0.18039E+01_JPRB, 0.16901E+01_JPRB, 0.15832E+01_JPRB, 0.14836E+01_JPRB, 0.13796E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.11681E+01_JPRB, 0.11797E+01_JPRB, 0.12033E+01_JPRB, 0.12263E+01_JPRB, 0.12602E+01_JPRB, 0.12916E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.14032E+01_JPRB, 0.14038E+01_JPRB, 0.15057E+01_JPRB, 0.18449E+01_JPRB, 0.22801E+01_JPRB, 0.27897E+01_JPRB ,&
&  0.37422E+01_JPRB, 0.31615E+01_JPRB, 0.27578E+01_JPRB, 0.30450E+01_JPRB, 0.38163E+01_JPRB, 0.42861E+01_JPRB, 0.40889E+01_JPRB ,&
&  0.39325E+01_JPRB, 0.34306E+01_JPRB, 0.27733E+01_JPRB, 0.22774E+01_JPRB, 0.18841E+01_JPRB, 0.16047E+01_JPRB, 0.13941E+01_JPRB ,&
&  0.12932E+01_JPRB, 0.12594E+01_JPRB, 0.12710E+01_JPRB, 0.12995E+01_JPRB, 0.13820E+01_JPRB, 0.14477E+01_JPRB, 0.17371E+01_JPRB ,&
&  0.20605E+01_JPRB, 0.24713E+01_JPRB, 0.30253E+01_JPRB, 0.35899E+01_JPRB, 0.44182E+01_JPRB, 0.69173E+01_JPRB, 0.79335E+01_JPRB ,&
&  0.62178E+01_JPRB, 0.58268E+01_JPRB, 0.62442E+01_JPRB, 0.65067E+01_JPRB, 0.66612E+01_JPRB, 0.61496E+01_JPRB, 0.60086E+01_JPRB ,&
&  0.48096E+01_JPRB, 0.43552E+01_JPRB, 0.45984E+01_JPRB, 0.57928E+01_JPRB, 0.92146E+01_JPRB, 0.12184E+02_JPRB, 0.11219E+02_JPRB ,&
&  0.93138E+01_JPRB, 0.78668E+01_JPRB, 0.66414E+01_JPRB, 0.50838E+01_JPRB, 0.45192E+01_JPRB, 0.39203E+01_JPRB, 0.45569E+01_JPRB ,&
&  0.43441E+01_JPRB, 0.39814E+01_JPRB, 0.33162E+01_JPRB, 0.25874E+01_JPRB, 0.21525E+01_JPRB, 0.25809E+01_JPRB, 0.28394E+01_JPRB ,&
&  0.33066E+01_JPRB, 0.39052E+01_JPRB, 0.66261E+01_JPRB, 0.77092E+01_JPRB, 0.81617E+01_JPRB, 0.71538E+01_JPRB, 0.83453E+01_JPRB ,&
&  0.75724E+01_JPRB, 0.73099E+01_JPRB, 0.61455E+01_JPRB, 0.46980E+01_JPRB, 0.37512E+01_JPRB, 0.32200E+01_JPRB, 0.29455E+01_JPRB ,&
&  0.28365E+01_JPRB, 0.27255E+01_JPRB, 0.25375E+01_JPRB, 0.23305E+01_JPRB, 0.21481E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.16648E+01_JPRB, 0.15299E+01_JPRB, 0.14415E+01_JPRB, 0.13695E+01_JPRB, 0.12898E+01_JPRB, 0.12134E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.11099E+01_JPRB, 0.11389E+01_JPRB, 0.11935E+01_JPRB, 0.12491E+01_JPRB, 0.13086E+01_JPRB, 0.13552E+01_JPRB ,&
&  0.14217E+01_JPRB, 0.15385E+01_JPRB, 0.15925E+01_JPRB, 0.17210E+01_JPRB, 0.20361E+01_JPRB, 0.22583E+01_JPRB, 0.27732E+01_JPRB ,&
&  0.35788E+01_JPRB, 0.27955E+01_JPRB, 0.24796E+01_JPRB, 0.28476E+01_JPRB, 0.40283E+01_JPRB, 0.42638E+01_JPRB, 0.39453E+01_JPRB ,&
&  0.39668E+01_JPRB, 0.32643E+01_JPRB, 0.23756E+01_JPRB, 0.18917E+01_JPRB, 0.15842E+01_JPRB, 0.13543E+01_JPRB, 0.12485E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.12645E+01_JPRB, 0.13139E+01_JPRB, 0.13687E+01_JPRB, 0.14358E+01_JPRB, 0.16173E+01_JPRB, 0.19609E+01_JPRB ,&
&  0.22954E+01_JPRB, 0.26432E+01_JPRB, 0.29312E+01_JPRB, 0.32875E+01_JPRB, 0.42698E+01_JPRB, 0.68719E+01_JPRB, 0.98014E+01_JPRB ,&
&  0.81029E+01_JPRB, 0.75693E+01_JPRB, 0.72894E+01_JPRB, 0.73614E+01_JPRB, 0.64655E+01_JPRB, 0.59271E+01_JPRB, 0.56389E+01_JPRB ,&
&  0.45581E+01_JPRB, 0.45563E+01_JPRB, 0.58680E+01_JPRB, 0.80721E+01_JPRB, 0.98167E+01_JPRB, 0.11808E+02_JPRB, 0.13763E+02_JPRB ,&
&  0.10275E+02_JPRB, 0.84305E+01_JPRB, 0.72761E+01_JPRB, 0.63127E+01_JPRB, 0.37318E+01_JPRB, 0.27530E+01_JPRB, 0.38419E+01_JPRB ,&
&  0.35530E+01_JPRB, 0.20952E+01_JPRB, 0.16772E+01_JPRB, 0.20988E+01_JPRB, 0.15665E+01_JPRB, 0.19004E+01_JPRB, 0.31206E+01_JPRB ,&
&  0.39902E+01_JPRB, 0.50564E+01_JPRB, 0.93019E+01_JPRB, 0.84521E+01_JPRB, 0.87556E+01_JPRB, 0.10435E+02_JPRB, 0.10051E+02_JPRB ,&
&  0.91278E+01_JPRB, 0.56941E+01_JPRB, 0.39925E+01_JPRB, 0.30326E+01_JPRB, 0.25310E+01_JPRB, 0.24530E+01_JPRB, 0.23962E+01_JPRB ,&
&  0.23476E+01_JPRB, 0.22328E+01_JPRB, 0.21180E+01_JPRB, 0.19507E+01_JPRB, 0.18186E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.13093E+01_JPRB, 0.12192E+01_JPRB, 0.11930E+01_JPRB, 0.11673E+01_JPRB, 0.11099E+01_JPRB, 0.10738E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10960E+01_JPRB, 0.11555E+01_JPRB, 0.12369E+01_JPRB, 0.13360E+01_JPRB, 0.14266E+01_JPRB, 0.14657E+01_JPRB ,&
&  0.15149E+01_JPRB, 0.16583E+01_JPRB, 0.19018E+01_JPRB, 0.22067E+01_JPRB, 0.26179E+01_JPRB, 0.24263E+01_JPRB, 0.33910E+01_JPRB ,&
&  0.34256E+01_JPRB, 0.25609E+01_JPRB, 0.24709E+01_JPRB, 0.29392E+01_JPRB, 0.34889E+01_JPRB, 0.33898E+01_JPRB, 0.31427E+01_JPRB ,&
&  0.28646E+01_JPRB, 0.23041E+01_JPRB, 0.18597E+01_JPRB, 0.15795E+01_JPRB, 0.13957E+01_JPRB, 0.13012E+01_JPRB, 0.13244E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.14312E+01_JPRB, 0.15178E+01_JPRB, 0.15031E+01_JPRB, 0.16434E+01_JPRB, 0.18440E+01_JPRB, 0.21416E+01_JPRB ,&
&  0.24135E+01_JPRB, 0.26652E+01_JPRB, 0.29223E+01_JPRB, 0.34841E+01_JPRB, 0.53909E+01_JPRB, 0.81148E+01_JPRB, 0.88298E+01_JPRB ,&
&  0.79857E+01_JPRB, 0.74442E+01_JPRB, 0.82373E+01_JPRB, 0.74842E+01_JPRB, 0.65267E+01_JPRB, 0.64753E+01_JPRB, 0.59965E+01_JPRB ,&
&  0.60699E+01_JPRB, 0.67386E+01_JPRB, 0.76293E+01_JPRB, 0.94734E+01_JPRB, 0.12027E+02_JPRB, 0.12462E+02_JPRB, 0.12699E+02_JPRB ,&
&  0.11203E+02_JPRB, 0.86312E+01_JPRB, 0.59684E+01_JPRB, 0.42722E+01_JPRB, 0.41154E+01_JPRB, 0.35597E+01_JPRB, 0.24044E+01_JPRB ,&
&  0.19789E+01_JPRB, 0.17068E+01_JPRB, 0.12950E+01_JPRB, 0.14099E+01_JPRB, 0.12995E+01_JPRB, 0.16868E+01_JPRB, 0.30416E+01_JPRB ,&
&  0.54898E+01_JPRB, 0.72613E+01_JPRB, 0.93788E+01_JPRB, 0.85496E+01_JPRB, 0.87159E+01_JPRB, 0.11019E+02_JPRB, 0.10777E+02_JPRB ,&
&  0.50041E+01_JPRB, 0.30818E+01_JPRB, 0.24625E+01_JPRB, 0.22451E+01_JPRB, 0.21488E+01_JPRB, 0.20796E+01_JPRB, 0.20065E+01_JPRB ,&
&  0.18856E+01_JPRB, 0.17773E+01_JPRB, 0.17237E+01_JPRB, 0.16341E+01_JPRB, 0.14686E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.11374E+01_JPRB, 0.11171E+01_JPRB, 0.11210E+01_JPRB, 0.10970E+01_JPRB, 0.10384E+01_JPRB, 0.10464E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.11709E+01_JPRB, 0.12603E+01_JPRB, 0.13876E+01_JPRB, 0.15628E+01_JPRB, 0.16511E+01_JPRB, 0.16676E+01_JPRB ,&
&  0.17200E+01_JPRB, 0.19460E+01_JPRB, 0.23549E+01_JPRB, 0.29466E+01_JPRB, 0.34251E+01_JPRB, 0.28760E+01_JPRB, 0.40492E+01_JPRB ,&
&  0.32675E+01_JPRB, 0.25352E+01_JPRB, 0.24924E+01_JPRB, 0.27591E+01_JPRB, 0.28902E+01_JPRB, 0.26918E+01_JPRB, 0.23757E+01_JPRB ,&
&  0.20408E+01_JPRB, 0.18424E+01_JPRB, 0.16168E+01_JPRB, 0.14756E+01_JPRB, 0.13616E+01_JPRB, 0.13761E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.15044E+01_JPRB, 0.16239E+01_JPRB, 0.16560E+01_JPRB, 0.17936E+01_JPRB, 0.20577E+01_JPRB, 0.22877E+01_JPRB, 0.25003E+01_JPRB ,&
&  0.26518E+01_JPRB, 0.29144E+01_JPRB, 0.34379E+01_JPRB, 0.48976E+01_JPRB, 0.64034E+01_JPRB, 0.69233E+01_JPRB, 0.79524E+01_JPRB ,&
&  0.85782E+01_JPRB, 0.87307E+01_JPRB, 0.77872E+01_JPRB, 0.82191E+01_JPRB, 0.72697E+01_JPRB, 0.66363E+01_JPRB, 0.66612E+01_JPRB ,&
&  0.72228E+01_JPRB, 0.91093E+01_JPRB, 0.98968E+01_JPRB, 0.10751E+02_JPRB, 0.15020E+02_JPRB, 0.16215E+02_JPRB, 0.13007E+02_JPRB ,&
&  0.11567E+02_JPRB, 0.83980E+01_JPRB, 0.59948E+01_JPRB, 0.45173E+01_JPRB, 0.57100E+01_JPRB, 0.66492E+01_JPRB, 0.33892E+01_JPRB ,&
&  0.22795E+01_JPRB, 0.17850E+01_JPRB, 0.13434E+01_JPRB, 0.13099E+01_JPRB, 0.13368E+01_JPRB, 0.17517E+01_JPRB, 0.41789E+01_JPRB ,&
&  0.51510E+01_JPRB, 0.84475E+01_JPRB, 0.97802E+01_JPRB, 0.93721E+01_JPRB, 0.90178E+01_JPRB, 0.82067E+01_JPRB, 0.48745E+01_JPRB ,&
&  0.28585E+01_JPRB, 0.19662E+01_JPRB, 0.18662E+01_JPRB, 0.20188E+01_JPRB, 0.20537E+01_JPRB, 0.19304E+01_JPRB, 0.17618E+01_JPRB ,&
&  0.16503E+01_JPRB, 0.16096E+01_JPRB, 0.15229E+01_JPRB, 0.14356E+01_JPRB, 0.12829E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.11384E+01_JPRB, 0.11650E+01_JPRB, 0.11958E+01_JPRB, 0.11285E+01_JPRB, 0.11130E+01_JPRB, 0.11345E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.13202E+01_JPRB, 0.14470E+01_JPRB, 0.16699E+01_JPRB, 0.18945E+01_JPRB, 0.19372E+01_JPRB, 0.19443E+01_JPRB ,&
&  0.20578E+01_JPRB, 0.24395E+01_JPRB, 0.30610E+01_JPRB, 0.38563E+01_JPRB, 0.39906E+01_JPRB, 0.37349E+01_JPRB, 0.43748E+01_JPRB ,&
&  0.33552E+01_JPRB, 0.26886E+01_JPRB, 0.24505E+01_JPRB, 0.26268E+01_JPRB, 0.26459E+01_JPRB, 0.25144E+01_JPRB, 0.21455E+01_JPRB ,&
&  0.18483E+01_JPRB, 0.18125E+01_JPRB, 0.15676E+01_JPRB, 0.16154E+01_JPRB, 0.15481E+01_JPRB, 0.16193E+01_JPRB, 0.16734E+01_JPRB ,&
&  0.17733E+01_JPRB, 0.18610E+01_JPRB, 0.19664E+01_JPRB, 0.21957E+01_JPRB, 0.25007E+01_JPRB, 0.27695E+01_JPRB, 0.30128E+01_JPRB ,&
&  0.32051E+01_JPRB, 0.37048E+01_JPRB, 0.46650E+01_JPRB, 0.61538E+01_JPRB, 0.59811E+01_JPRB, 0.77418E+01_JPRB, 0.80263E+01_JPRB ,&
&  0.83642E+01_JPRB, 0.82742E+01_JPRB, 0.74050E+01_JPRB, 0.75553E+01_JPRB, 0.76263E+01_JPRB, 0.73555E+01_JPRB, 0.75671E+01_JPRB ,&
&  0.96057E+01_JPRB, 0.10828E+02_JPRB, 0.12137E+02_JPRB, 0.13032E+02_JPRB, 0.16849E+02_JPRB, 0.22640E+02_JPRB, 0.13569E+02_JPRB ,&
&  0.95412E+01_JPRB, 0.85250E+01_JPRB, 0.62271E+01_JPRB, 0.51294E+01_JPRB, 0.66015E+01_JPRB, 0.81848E+01_JPRB, 0.80419E+01_JPRB ,&
&  0.33240E+01_JPRB, 0.19321E+01_JPRB, 0.17614E+01_JPRB, 0.17616E+01_JPRB, 0.14688E+01_JPRB, 0.19503E+01_JPRB, 0.69431E+01_JPRB ,&
&  0.55374E+01_JPRB, 0.86822E+01_JPRB, 0.92569E+01_JPRB, 0.88310E+01_JPRB, 0.89619E+01_JPRB, 0.61922E+01_JPRB, 0.34447E+01_JPRB ,&
&  0.16915E+01_JPRB, 0.16300E+01_JPRB, 0.18218E+01_JPRB, 0.17588E+01_JPRB, 0.17236E+01_JPRB, 0.16165E+01_JPRB, 0.14371E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.13920E+01_JPRB, 0.12967E+01_JPRB, 0.12131E+01_JPRB, 0.11602E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.12012E+01_JPRB, 0.12423E+01_JPRB, 0.12460E+01_JPRB, 0.12465E+01_JPRB, 0.12655E+01_JPRB, 0.12854E+01_JPRB, 0.12903E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.15650E+01_JPRB, 0.18181E+01_JPRB, 0.21481E+01_JPRB, 0.23543E+01_JPRB, 0.23557E+01_JPRB, 0.24016E+01_JPRB ,&
&  0.26769E+01_JPRB, 0.32070E+01_JPRB, 0.38691E+01_JPRB, 0.43487E+01_JPRB, 0.42025E+01_JPRB, 0.38291E+01_JPRB, 0.41845E+01_JPRB ,&
&  0.37154E+01_JPRB, 0.29456E+01_JPRB, 0.24122E+01_JPRB, 0.25729E+01_JPRB, 0.27596E+01_JPRB, 0.27921E+01_JPRB, 0.23835E+01_JPRB ,&
&  0.21793E+01_JPRB, 0.21281E+01_JPRB, 0.19906E+01_JPRB, 0.20021E+01_JPRB, 0.20262E+01_JPRB, 0.21190E+01_JPRB, 0.21880E+01_JPRB ,&
&  0.21731E+01_JPRB, 0.22676E+01_JPRB, 0.24676E+01_JPRB, 0.27526E+01_JPRB, 0.30854E+01_JPRB, 0.33970E+01_JPRB, 0.37461E+01_JPRB ,&
&  0.41992E+01_JPRB, 0.49477E+01_JPRB, 0.59060E+01_JPRB, 0.62586E+01_JPRB, 0.67319E+01_JPRB, 0.73578E+01_JPRB, 0.78582E+01_JPRB ,&
&  0.81189E+01_JPRB, 0.82250E+01_JPRB, 0.79271E+01_JPRB, 0.76246E+01_JPRB, 0.78426E+01_JPRB, 0.83231E+01_JPRB, 0.94509E+01_JPRB ,&
&  0.11800E+02_JPRB, 0.13271E+02_JPRB, 0.13543E+02_JPRB, 0.14247E+02_JPRB, 0.15666E+02_JPRB, 0.16750E+02_JPRB, 0.13933E+02_JPRB ,&
&  0.10307E+02_JPRB, 0.74002E+01_JPRB, 0.60368E+01_JPRB, 0.65443E+01_JPRB, 0.82969E+01_JPRB, 0.88684E+01_JPRB, 0.12011E+02_JPRB ,&
&  0.12038E+02_JPRB, 0.47101E+01_JPRB, 0.31639E+01_JPRB, 0.42655E+01_JPRB, 0.21365E+01_JPRB, 0.18928E+01_JPRB, 0.35854E+01_JPRB ,&
&  0.58308E+01_JPRB, 0.78584E+01_JPRB, 0.78190E+01_JPRB, 0.74693E+01_JPRB, 0.76557E+01_JPRB, 0.43475E+01_JPRB, 0.21027E+01_JPRB ,&
&  0.14965E+01_JPRB, 0.15755E+01_JPRB, 0.15777E+01_JPRB, 0.14163E+01_JPRB, 0.13599E+01_JPRB, 0.14060E+01_JPRB, 0.12554E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12624E+01_JPRB, 0.12704E+01_JPRB, 0.12368E+01_JPRB, 0.11902E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.13443E+01_JPRB, 0.13834E+01_JPRB, 0.14555E+01_JPRB, 0.15086E+01_JPRB, 0.15250E+01_JPRB, 0.15714E+01_JPRB, 0.16473E+01_JPRB ,&
&  0.18361E+01_JPRB, 0.21104E+01_JPRB, 0.24263E+01_JPRB, 0.27366E+01_JPRB, 0.28432E+01_JPRB, 0.28761E+01_JPRB, 0.30652E+01_JPRB ,&
&  0.34375E+01_JPRB, 0.39537E+01_JPRB, 0.44554E+01_JPRB, 0.48409E+01_JPRB, 0.48791E+01_JPRB, 0.50494E+01_JPRB, 0.42042E+01_JPRB ,&
&  0.35714E+01_JPRB, 0.33976E+01_JPRB, 0.26292E+01_JPRB, 0.28488E+01_JPRB, 0.30426E+01_JPRB, 0.31300E+01_JPRB, 0.28218E+01_JPRB ,&
&  0.26505E+01_JPRB, 0.25960E+01_JPRB, 0.25055E+01_JPRB, 0.24942E+01_JPRB, 0.25872E+01_JPRB, 0.26885E+01_JPRB, 0.27792E+01_JPRB ,&
&  0.28833E+01_JPRB, 0.29801E+01_JPRB, 0.31917E+01_JPRB, 0.34745E+01_JPRB, 0.37850E+01_JPRB, 0.41461E+01_JPRB, 0.46367E+01_JPRB ,&
&  0.51786E+01_JPRB, 0.56689E+01_JPRB, 0.61565E+01_JPRB, 0.61547E+01_JPRB, 0.65738E+01_JPRB, 0.71008E+01_JPRB, 0.75160E+01_JPRB ,&
&  0.73560E+01_JPRB, 0.71085E+01_JPRB, 0.76994E+01_JPRB, 0.85290E+01_JPRB, 0.91717E+01_JPRB, 0.96682E+01_JPRB, 0.10808E+02_JPRB ,&
&  0.12954E+02_JPRB, 0.14327E+02_JPRB, 0.14730E+02_JPRB, 0.12900E+02_JPRB, 0.11906E+02_JPRB, 0.11282E+02_JPRB, 0.11137E+02_JPRB ,&
&  0.95998E+01_JPRB, 0.77954E+01_JPRB, 0.71963E+01_JPRB, 0.79648E+01_JPRB, 0.89826E+01_JPRB, 0.93309E+01_JPRB, 0.10933E+02_JPRB ,&
&  0.12556E+02_JPRB, 0.11115E+02_JPRB, 0.87536E+01_JPRB, 0.54387E+01_JPRB, 0.33148E+01_JPRB, 0.27452E+01_JPRB, 0.35612E+01_JPRB ,&
&  0.55595E+01_JPRB, 0.60054E+01_JPRB, 0.58434E+01_JPRB, 0.74932E+01_JPRB, 0.54169E+01_JPRB, 0.28803E+01_JPRB, 0.16089E+01_JPRB ,&
&  0.13150E+01_JPRB, 0.13489E+01_JPRB, 0.13488E+01_JPRB, 0.12749E+01_JPRB, 0.12948E+01_JPRB, 0.13278E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.13537E+01_JPRB, 0.13142E+01_JPRB, 0.12814E+01_JPRB, 0.13093E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.16193E+01_JPRB, 0.17064E+01_JPRB, 0.17854E+01_JPRB, 0.18376E+01_JPRB, 0.19322E+01_JPRB, 0.19972E+01_JPRB, 0.21512E+01_JPRB ,&
&  0.24160E+01_JPRB, 0.26999E+01_JPRB, 0.29685E+01_JPRB, 0.31324E+01_JPRB, 0.32078E+01_JPRB, 0.33376E+01_JPRB, 0.36050E+01_JPRB ,&
&  0.39109E+01_JPRB, 0.42807E+01_JPRB, 0.46119E+01_JPRB, 0.49735E+01_JPRB, 0.52410E+01_JPRB, 0.59094E+01_JPRB, 0.53263E+01_JPRB ,&
&  0.34276E+01_JPRB, 0.43956E+01_JPRB, 0.29833E+01_JPRB, 0.32846E+01_JPRB, 0.34128E+01_JPRB, 0.35937E+01_JPRB, 0.32073E+01_JPRB ,&
&  0.29611E+01_JPRB, 0.29684E+01_JPRB, 0.29637E+01_JPRB, 0.29143E+01_JPRB, 0.29216E+01_JPRB, 0.31128E+01_JPRB, 0.33023E+01_JPRB ,&
&  0.34665E+01_JPRB, 0.36007E+01_JPRB, 0.37948E+01_JPRB, 0.40455E+01_JPRB, 0.43341E+01_JPRB, 0.46831E+01_JPRB, 0.50901E+01_JPRB ,&
&  0.54286E+01_JPRB, 0.57525E+01_JPRB, 0.58662E+01_JPRB, 0.59700E+01_JPRB, 0.61620E+01_JPRB, 0.65057E+01_JPRB, 0.69003E+01_JPRB ,&
&  0.70513E+01_JPRB, 0.71624E+01_JPRB, 0.79114E+01_JPRB, 0.84363E+01_JPRB, 0.88136E+01_JPRB, 0.97490E+01_JPRB, 0.10357E+02_JPRB ,&
&  0.12109E+02_JPRB, 0.12126E+02_JPRB, 0.12342E+02_JPRB, 0.10348E+02_JPRB, 0.84287E+01_JPRB, 0.76467E+01_JPRB, 0.74578E+01_JPRB ,&
&  0.67955E+01_JPRB, 0.69617E+01_JPRB, 0.71790E+01_JPRB, 0.77995E+01_JPRB, 0.86947E+01_JPRB, 0.95321E+01_JPRB, 0.93903E+01_JPRB ,&
&  0.92293E+01_JPRB, 0.65956E+01_JPRB, 0.48900E+01_JPRB, 0.42030E+01_JPRB, 0.31386E+01_JPRB, 0.28351E+01_JPRB, 0.38384E+01_JPRB ,&
&  0.51053E+01_JPRB, 0.52635E+01_JPRB, 0.55972E+01_JPRB, 0.54660E+01_JPRB, 0.31506E+01_JPRB, 0.18474E+01_JPRB, 0.12999E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.12303E+01_JPRB, 0.12341E+01_JPRB, 0.12298E+01_JPRB, 0.12841E+01_JPRB, 0.12351E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.13289E+01_JPRB, 0.13840E+01_JPRB, 0.14273E+01_JPRB, 0.14447E+01_JPRB, 0.15135E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.19352E+01_JPRB, 0.21282E+01_JPRB, 0.23283E+01_JPRB, 0.24840E+01_JPRB, 0.25057E+01_JPRB, 0.25431E+01_JPRB, 0.26467E+01_JPRB ,&
&  0.27803E+01_JPRB, 0.29508E+01_JPRB, 0.31366E+01_JPRB, 0.32984E+01_JPRB, 0.34371E+01_JPRB, 0.36608E+01_JPRB, 0.39833E+01_JPRB ,&
&  0.43500E+01_JPRB, 0.47373E+01_JPRB, 0.51246E+01_JPRB, 0.55970E+01_JPRB, 0.61977E+01_JPRB, 0.71435E+01_JPRB, 0.70412E+01_JPRB ,&
&  0.49412E+01_JPRB, 0.37019E+01_JPRB, 0.35820E+01_JPRB, 0.34094E+01_JPRB, 0.33067E+01_JPRB, 0.36002E+01_JPRB, 0.36088E+01_JPRB ,&
&  0.33309E+01_JPRB, 0.32243E+01_JPRB, 0.32372E+01_JPRB, 0.31957E+01_JPRB, 0.31298E+01_JPRB, 0.33503E+01_JPRB, 0.35545E+01_JPRB ,&
&  0.37578E+01_JPRB, 0.38992E+01_JPRB, 0.40553E+01_JPRB, 0.42507E+01_JPRB, 0.44737E+01_JPRB, 0.47203E+01_JPRB, 0.49979E+01_JPRB ,&
&  0.51783E+01_JPRB, 0.53846E+01_JPRB, 0.55053E+01_JPRB, 0.56033E+01_JPRB, 0.57550E+01_JPRB, 0.58779E+01_JPRB, 0.60510E+01_JPRB ,&
&  0.64113E+01_JPRB, 0.69526E+01_JPRB, 0.75122E+01_JPRB, 0.78715E+01_JPRB, 0.79609E+01_JPRB, 0.88239E+01_JPRB, 0.92778E+01_JPRB ,&
&  0.96934E+01_JPRB, 0.91536E+01_JPRB, 0.92594E+01_JPRB, 0.79448E+01_JPRB, 0.69222E+01_JPRB, 0.60241E+01_JPRB, 0.57517E+01_JPRB ,&
&  0.55023E+01_JPRB, 0.54904E+01_JPRB, 0.53572E+01_JPRB, 0.57672E+01_JPRB, 0.68861E+01_JPRB, 0.75817E+01_JPRB, 0.78457E+01_JPRB ,&
&  0.65886E+01_JPRB, 0.49559E+01_JPRB, 0.34930E+01_JPRB, 0.35910E+01_JPRB, 0.29689E+01_JPRB, 0.33554E+01_JPRB, 0.39297E+01_JPRB ,&
&  0.42519E+01_JPRB, 0.36241E+01_JPRB, 0.37582E+01_JPRB, 0.35485E+01_JPRB, 0.20778E+01_JPRB, 0.13532E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.97600E+00_JPRB, 0.10294E+01_JPRB, 0.10579E+01_JPRB, 0.10607E+01_JPRB, 0.11337E+01_JPRB, 0.11969E+01_JPRB, 0.12963E+01_JPRB ,&
&  0.13386E+01_JPRB, 0.14157E+01_JPRB, 0.15070E+01_JPRB, 0.16008E+01_JPRB, 0.17484E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.19583E+01_JPRB, 0.21150E+01_JPRB, 0.22855E+01_JPRB, 0.23386E+01_JPRB, 0.23974E+01_JPRB, 0.24663E+01_JPRB, 0.25617E+01_JPRB ,&
&  0.27430E+01_JPRB, 0.29146E+01_JPRB, 0.30988E+01_JPRB, 0.32602E+01_JPRB, 0.34041E+01_JPRB, 0.36304E+01_JPRB, 0.39050E+01_JPRB ,&
&  0.41479E+01_JPRB, 0.43825E+01_JPRB, 0.46208E+01_JPRB, 0.49382E+01_JPRB, 0.52859E+01_JPRB, 0.57675E+01_JPRB, 0.59989E+01_JPRB ,&
&  0.46917E+01_JPRB, 0.27945E+01_JPRB, 0.29989E+01_JPRB, 0.30522E+01_JPRB, 0.31903E+01_JPRB, 0.35419E+01_JPRB, 0.36381E+01_JPRB ,&
&  0.32616E+01_JPRB, 0.31905E+01_JPRB, 0.32796E+01_JPRB, 0.32032E+01_JPRB, 0.29946E+01_JPRB, 0.32247E+01_JPRB, 0.34545E+01_JPRB ,&
&  0.36786E+01_JPRB, 0.38705E+01_JPRB, 0.40221E+01_JPRB, 0.41892E+01_JPRB, 0.43270E+01_JPRB, 0.44569E+01_JPRB, 0.45772E+01_JPRB ,&
&  0.46979E+01_JPRB, 0.47680E+01_JPRB, 0.48858E+01_JPRB, 0.51356E+01_JPRB, 0.52172E+01_JPRB, 0.52542E+01_JPRB, 0.52840E+01_JPRB ,&
&  0.55225E+01_JPRB, 0.58814E+01_JPRB, 0.64739E+01_JPRB, 0.70029E+01_JPRB, 0.74149E+01_JPRB, 0.73591E+01_JPRB, 0.74917E+01_JPRB ,&
&  0.71594E+01_JPRB, 0.65086E+01_JPRB, 0.67237E+01_JPRB, 0.70779E+01_JPRB, 0.52594E+01_JPRB, 0.48209E+01_JPRB, 0.43221E+01_JPRB ,&
&  0.43633E+01_JPRB, 0.39023E+01_JPRB, 0.36024E+01_JPRB, 0.38548E+01_JPRB, 0.50639E+01_JPRB, 0.54206E+01_JPRB, 0.56438E+01_JPRB ,&
&  0.47826E+01_JPRB, 0.30908E+01_JPRB, 0.27806E+01_JPRB, 0.33378E+01_JPRB, 0.34257E+01_JPRB, 0.48671E+01_JPRB, 0.40844E+01_JPRB ,&
&  0.28080E+01_JPRB, 0.23413E+01_JPRB, 0.27061E+01_JPRB, 0.27815E+01_JPRB, 0.16935E+01_JPRB, 0.11574E+01_JPRB, 0.96600E+00_JPRB ,&
&  0.89659E+00_JPRB, 0.88445E+00_JPRB, 0.93134E+00_JPRB, 0.92376E+00_JPRB, 0.99025E+00_JPRB, 0.10516E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.12965E+01_JPRB, 0.14040E+01_JPRB, 0.14987E+01_JPRB, 0.16255E+01_JPRB, 0.17728E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.13921E+01_JPRB, 0.14894E+01_JPRB, 0.15336E+01_JPRB, 0.17248E+01_JPRB, 0.19451E+01_JPRB, 0.21220E+01_JPRB, 0.22828E+01_JPRB ,&
&  0.24895E+01_JPRB, 0.27468E+01_JPRB, 0.29535E+01_JPRB, 0.31046E+01_JPRB, 0.32629E+01_JPRB, 0.34669E+01_JPRB, 0.35309E+01_JPRB ,&
&  0.35331E+01_JPRB, 0.35331E+01_JPRB, 0.35548E+01_JPRB, 0.35588E+01_JPRB, 0.35890E+01_JPRB, 0.36096E+01_JPRB, 0.38927E+01_JPRB ,&
&  0.38176E+01_JPRB, 0.34655E+01_JPRB, 0.31003E+01_JPRB, 0.22902E+01_JPRB, 0.19046E+01_JPRB, 0.24170E+01_JPRB, 0.24220E+01_JPRB ,&
&  0.26779E+01_JPRB, 0.27058E+01_JPRB, 0.26199E+01_JPRB, 0.27313E+01_JPRB, 0.28196E+01_JPRB, 0.29701E+01_JPRB, 0.31798E+01_JPRB ,&
&  0.33420E+01_JPRB, 0.35163E+01_JPRB, 0.36749E+01_JPRB, 0.38165E+01_JPRB, 0.39185E+01_JPRB, 0.39827E+01_JPRB, 0.39993E+01_JPRB ,&
&  0.40255E+01_JPRB, 0.41032E+01_JPRB, 0.42373E+01_JPRB, 0.45558E+01_JPRB, 0.46706E+01_JPRB, 0.47916E+01_JPRB, 0.47380E+01_JPRB ,&
&  0.48236E+01_JPRB, 0.49904E+01_JPRB, 0.53932E+01_JPRB, 0.57211E+01_JPRB, 0.57967E+01_JPRB, 0.54588E+01_JPRB, 0.53113E+01_JPRB ,&
&  0.50779E+01_JPRB, 0.47479E+01_JPRB, 0.48605E+01_JPRB, 0.54100E+01_JPRB, 0.38169E+01_JPRB, 0.39345E+01_JPRB, 0.39216E+01_JPRB ,&
&  0.33141E+01_JPRB, 0.29035E+01_JPRB, 0.26412E+01_JPRB, 0.24720E+01_JPRB, 0.31774E+01_JPRB, 0.38653E+01_JPRB, 0.45700E+01_JPRB ,&
&  0.36458E+01_JPRB, 0.23761E+01_JPRB, 0.23057E+01_JPRB, 0.30290E+01_JPRB, 0.36000E+01_JPRB, 0.48273E+01_JPRB, 0.33910E+01_JPRB ,&
&  0.20263E+01_JPRB, 0.21198E+01_JPRB, 0.25239E+01_JPRB, 0.29620E+01_JPRB, 0.19327E+01_JPRB, 0.10863E+01_JPRB, 0.87558E+00_JPRB ,&
&  0.83121E+00_JPRB, 0.83068E+00_JPRB, 0.84096E+00_JPRB, 0.83999E+00_JPRB, 0.88268E+00_JPRB, 0.92835E+00_JPRB, 0.94945E+00_JPRB ,&
&  0.97724E+00_JPRB, 0.99910E+00_JPRB, 0.10547E+01_JPRB, 0.11460E+01_JPRB, 0.12822E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.90511E+00_JPRB, 0.91975E+00_JPRB, 0.91239E+00_JPRB, 0.95646E+00_JPRB, 0.10395E+01_JPRB, 0.12068E+01_JPRB, 0.13767E+01_JPRB ,&
&  0.15700E+01_JPRB, 0.18055E+01_JPRB, 0.21430E+01_JPRB, 0.24672E+01_JPRB, 0.26123E+01_JPRB, 0.25915E+01_JPRB, 0.23488E+01_JPRB ,&
&  0.20181E+01_JPRB, 0.16605E+01_JPRB, 0.14342E+01_JPRB, 0.13336E+01_JPRB, 0.13862E+01_JPRB, 0.14055E+01_JPRB, 0.14881E+01_JPRB ,&
&  0.16549E+01_JPRB, 0.19368E+01_JPRB, 0.21661E+01_JPRB, 0.27144E+01_JPRB, 0.28507E+01_JPRB, 0.22767E+01_JPRB, 0.23409E+01_JPRB ,&
&  0.30751E+01_JPRB, 0.30811E+01_JPRB, 0.21110E+01_JPRB, 0.20232E+01_JPRB, 0.22145E+01_JPRB, 0.23723E+01_JPRB, 0.26062E+01_JPRB ,&
&  0.27238E+01_JPRB, 0.28467E+01_JPRB, 0.29528E+01_JPRB, 0.30109E+01_JPRB, 0.30307E+01_JPRB, 0.29754E+01_JPRB, 0.29206E+01_JPRB ,&
&  0.28893E+01_JPRB, 0.28720E+01_JPRB, 0.30762E+01_JPRB, 0.33533E+01_JPRB, 0.36517E+01_JPRB, 0.38881E+01_JPRB, 0.39665E+01_JPRB ,&
&  0.40269E+01_JPRB, 0.41204E+01_JPRB, 0.45417E+01_JPRB, 0.46501E+01_JPRB, 0.43822E+01_JPRB, 0.41158E+01_JPRB, 0.37783E+01_JPRB ,&
&  0.34157E+01_JPRB, 0.29030E+01_JPRB, 0.23150E+01_JPRB, 0.35639E+01_JPRB, 0.40720E+01_JPRB, 0.34853E+01_JPRB, 0.28472E+01_JPRB ,&
&  0.25029E+01_JPRB, 0.22556E+01_JPRB, 0.20165E+01_JPRB, 0.19492E+01_JPRB, 0.22780E+01_JPRB, 0.35339E+01_JPRB, 0.36562E+01_JPRB ,&
&  0.30323E+01_JPRB, 0.18788E+01_JPRB, 0.16355E+01_JPRB, 0.19979E+01_JPRB, 0.24554E+01_JPRB, 0.31689E+01_JPRB, 0.21448E+01_JPRB ,&
&  0.15914E+01_JPRB, 0.19738E+01_JPRB, 0.22892E+01_JPRB, 0.27904E+01_JPRB, 0.27861E+01_JPRB, 0.87790E+00_JPRB, 0.79715E+00_JPRB ,&
&  0.77409E+00_JPRB, 0.76510E+00_JPRB, 0.78374E+00_JPRB, 0.77548E+00_JPRB, 0.78323E+00_JPRB, 0.77092E+00_JPRB, 0.75466E+00_JPRB ,&
&  0.75653E+00_JPRB, 0.77889E+00_JPRB, 0.80843E+00_JPRB, 0.82978E+00_JPRB, 0.87212E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.82406E+00_JPRB, 0.82643E+00_JPRB, 0.83265E+00_JPRB, 0.84329E+00_JPRB, 0.85228E+00_JPRB, 0.86147E+00_JPRB, 0.89862E+00_JPRB ,&
&  0.96149E+00_JPRB, 0.10553E+01_JPRB, 0.11887E+01_JPRB, 0.12877E+01_JPRB, 0.12692E+01_JPRB, 0.11620E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10291E+01_JPRB, 0.10623E+01_JPRB, 0.11200E+01_JPRB, 0.11912E+01_JPRB, 0.12053E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.12683E+01_JPRB, 0.13254E+01_JPRB, 0.14496E+01_JPRB, 0.15545E+01_JPRB, 0.23089E+01_JPRB, 0.22483E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.25177E+01_JPRB, 0.25942E+01_JPRB, 0.22028E+01_JPRB, 0.17178E+01_JPRB, 0.15785E+01_JPRB, 0.15512E+01_JPRB, 0.15988E+01_JPRB ,&
&  0.15921E+01_JPRB, 0.17186E+01_JPRB, 0.18266E+01_JPRB, 0.19457E+01_JPRB, 0.20306E+01_JPRB, 0.20589E+01_JPRB, 0.21340E+01_JPRB ,&
&  0.22081E+01_JPRB, 0.22900E+01_JPRB, 0.23966E+01_JPRB, 0.22174E+01_JPRB, 0.24512E+01_JPRB, 0.27235E+01_JPRB, 0.29238E+01_JPRB ,&
&  0.30231E+01_JPRB, 0.30840E+01_JPRB, 0.33573E+01_JPRB, 0.32779E+01_JPRB, 0.31378E+01_JPRB, 0.28644E+01_JPRB, 0.26238E+01_JPRB ,&
&  0.24229E+01_JPRB, 0.17982E+01_JPRB, 0.12155E+01_JPRB, 0.18036E+01_JPRB, 0.21888E+01_JPRB, 0.18730E+01_JPRB, 0.19521E+01_JPRB ,&
&  0.18167E+01_JPRB, 0.16893E+01_JPRB, 0.15680E+01_JPRB, 0.15690E+01_JPRB, 0.15978E+01_JPRB, 0.21827E+01_JPRB, 0.26576E+01_JPRB ,&
&  0.21729E+01_JPRB, 0.14656E+01_JPRB, 0.12395E+01_JPRB, 0.13355E+01_JPRB, 0.15947E+01_JPRB, 0.19589E+01_JPRB, 0.20745E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.16456E+01_JPRB, 0.21024E+01_JPRB, 0.26583E+01_JPRB, 0.14326E+01_JPRB, 0.70891E+00_JPRB, 0.64841E+00_JPRB ,&
&  0.64617E+00_JPRB, 0.65686E+00_JPRB, 0.68032E+00_JPRB, 0.69669E+00_JPRB, 0.70851E+00_JPRB, 0.71722E+00_JPRB, 0.73706E+00_JPRB ,&
&  0.75419E+00_JPRB, 0.77389E+00_JPRB, 0.78562E+00_JPRB, 0.79964E+00_JPRB, 0.81479E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.10502E+01_JPRB, 0.10735E+01_JPRB, 0.11073E+01_JPRB, 0.11418E+01_JPRB, 0.11676E+01_JPRB, 0.12032E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.12958E+01_JPRB, 0.13664E+01_JPRB, 0.14175E+01_JPRB, 0.14371E+01_JPRB, 0.14503E+01_JPRB, 0.14664E+01_JPRB, 0.14622E+01_JPRB ,&
&  0.14968E+01_JPRB, 0.15169E+01_JPRB, 0.16016E+01_JPRB, 0.16375E+01_JPRB, 0.16803E+01_JPRB, 0.17328E+01_JPRB, 0.17365E+01_JPRB ,&
&  0.17132E+01_JPRB, 0.16977E+01_JPRB, 0.17065E+01_JPRB, 0.16905E+01_JPRB, 0.17705E+01_JPRB, 0.17943E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.19061E+01_JPRB, 0.19462E+01_JPRB, 0.20464E+01_JPRB, 0.15384E+01_JPRB, 0.14679E+01_JPRB, 0.14861E+01_JPRB, 0.13677E+01_JPRB ,&
&  0.13551E+01_JPRB, 0.14311E+01_JPRB, 0.15227E+01_JPRB, 0.16171E+01_JPRB, 0.17504E+01_JPRB, 0.18226E+01_JPRB, 0.18820E+01_JPRB ,&
&  0.19523E+01_JPRB, 0.20287E+01_JPRB, 0.20757E+01_JPRB, 0.20504E+01_JPRB, 0.20741E+01_JPRB, 0.24370E+01_JPRB, 0.25982E+01_JPRB ,&
&  0.26666E+01_JPRB, 0.27436E+01_JPRB, 0.23227E+01_JPRB, 0.21744E+01_JPRB, 0.22867E+01_JPRB, 0.20424E+01_JPRB, 0.19452E+01_JPRB ,&
&  0.21145E+01_JPRB, 0.18181E+01_JPRB, 0.99195E+00_JPRB, 0.12829E+01_JPRB, 0.14841E+01_JPRB, 0.15283E+01_JPRB, 0.15359E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.12588E+01_JPRB, 0.11808E+01_JPRB, 0.12583E+01_JPRB, 0.13247E+01_JPRB, 0.13226E+01_JPRB, 0.14654E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.11853E+01_JPRB, 0.11105E+01_JPRB, 0.13329E+01_JPRB, 0.20537E+01_JPRB, 0.33876E+01_JPRB, 0.29790E+01_JPRB ,&
&  0.20287E+01_JPRB, 0.18490E+01_JPRB, 0.30419E+01_JPRB, 0.40409E+01_JPRB, 0.15149E+01_JPRB, 0.89302E+00_JPRB, 0.62989E+00_JPRB ,&
&  0.66392E+00_JPRB, 0.68064E+00_JPRB, 0.69292E+00_JPRB, 0.72179E+00_JPRB, 0.76689E+00_JPRB, 0.79256E+00_JPRB, 0.80526E+00_JPRB ,&
&  0.83892E+00_JPRB, 0.88649E+00_JPRB, 0.92322E+00_JPRB, 0.96781E+00_JPRB, 0.10083E+01_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.12698E+01_JPRB, 0.13071E+01_JPRB, 0.13369E+01_JPRB, 0.13694E+01_JPRB, 0.14088E+01_JPRB, 0.14475E+01_JPRB, 0.14762E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.15407E+01_JPRB, 0.15547E+01_JPRB, 0.15788E+01_JPRB, 0.15871E+01_JPRB, 0.16609E+01_JPRB, 0.16907E+01_JPRB ,&
&  0.17664E+01_JPRB, 0.18137E+01_JPRB, 0.18343E+01_JPRB, 0.18427E+01_JPRB, 0.18642E+01_JPRB, 0.18489E+01_JPRB, 0.18330E+01_JPRB ,&
&  0.18632E+01_JPRB, 0.19275E+01_JPRB, 0.19850E+01_JPRB, 0.20365E+01_JPRB, 0.20477E+01_JPRB, 0.21267E+01_JPRB, 0.22412E+01_JPRB ,&
&  0.18771E+01_JPRB, 0.21691E+01_JPRB, 0.20129E+01_JPRB, 0.16947E+01_JPRB, 0.16963E+01_JPRB, 0.17276E+01_JPRB, 0.16606E+01_JPRB ,&
&  0.16106E+01_JPRB, 0.16535E+01_JPRB, 0.16845E+01_JPRB, 0.17209E+01_JPRB, 0.18014E+01_JPRB, 0.18902E+01_JPRB, 0.19554E+01_JPRB ,&
&  0.20580E+01_JPRB, 0.21375E+01_JPRB, 0.22002E+01_JPRB, 0.23022E+01_JPRB, 0.22641E+01_JPRB, 0.26664E+01_JPRB, 0.27996E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.27334E+01_JPRB, 0.21174E+01_JPRB, 0.20272E+01_JPRB, 0.21080E+01_JPRB, 0.20117E+01_JPRB, 0.18988E+01_JPRB ,&
&  0.20689E+01_JPRB, 0.19458E+01_JPRB, 0.13471E+01_JPRB, 0.14177E+01_JPRB, 0.14220E+01_JPRB, 0.14085E+01_JPRB, 0.12861E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.10074E+01_JPRB, 0.95704E+00_JPRB, 0.98861E+00_JPRB, 0.10058E+01_JPRB, 0.98987E+00_JPRB, 0.10341E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.10762E+01_JPRB, 0.11285E+01_JPRB, 0.12757E+01_JPRB, 0.22076E+01_JPRB, 0.29002E+01_JPRB, 0.39427E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.17514E+01_JPRB, 0.15318E+01_JPRB, 0.21123E+01_JPRB, 0.12078E+01_JPRB, 0.71949E+00_JPRB, 0.66805E+00_JPRB ,&
&  0.74096E+00_JPRB, 0.77738E+00_JPRB, 0.79058E+00_JPRB, 0.81698E+00_JPRB, 0.87472E+00_JPRB, 0.91455E+00_JPRB, 0.95442E+00_JPRB ,&
&  0.97983E+00_JPRB, 0.10422E+01_JPRB, 0.11219E+01_JPRB, 0.11836E+01_JPRB, 0.12198E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.13793E+01_JPRB, 0.14282E+01_JPRB, 0.14726E+01_JPRB, 0.15173E+01_JPRB, 0.15609E+01_JPRB, 0.16047E+01_JPRB, 0.16422E+01_JPRB ,&
&  0.16732E+01_JPRB, 0.16953E+01_JPRB, 0.17074E+01_JPRB, 0.17242E+01_JPRB, 0.17342E+01_JPRB, 0.17972E+01_JPRB, 0.18180E+01_JPRB ,&
&  0.18896E+01_JPRB, 0.19231E+01_JPRB, 0.19194E+01_JPRB, 0.19656E+01_JPRB, 0.19446E+01_JPRB, 0.19087E+01_JPRB, 0.19031E+01_JPRB ,&
&  0.19221E+01_JPRB, 0.19287E+01_JPRB, 0.19349E+01_JPRB, 0.19544E+01_JPRB, 0.19688E+01_JPRB, 0.19405E+01_JPRB, 0.21170E+01_JPRB ,&
&  0.17348E+01_JPRB, 0.23996E+01_JPRB, 0.23552E+01_JPRB, 0.20716E+01_JPRB, 0.19652E+01_JPRB, 0.18198E+01_JPRB, 0.17974E+01_JPRB ,&
&  0.18042E+01_JPRB, 0.17760E+01_JPRB, 0.17819E+01_JPRB, 0.18181E+01_JPRB, 0.18919E+01_JPRB, 0.19909E+01_JPRB, 0.20803E+01_JPRB ,&
&  0.21965E+01_JPRB, 0.23035E+01_JPRB, 0.23892E+01_JPRB, 0.24704E+01_JPRB, 0.26282E+01_JPRB, 0.27476E+01_JPRB, 0.27918E+01_JPRB ,&
&  0.27890E+01_JPRB, 0.28054E+01_JPRB, 0.22122E+01_JPRB, 0.21785E+01_JPRB, 0.23556E+01_JPRB, 0.22603E+01_JPRB, 0.20173E+01_JPRB ,&
&  0.20400E+01_JPRB, 0.18634E+01_JPRB, 0.16675E+01_JPRB, 0.14884E+01_JPRB, 0.13702E+01_JPRB, 0.12660E+01_JPRB, 0.11415E+01_JPRB ,&
&  0.93615E+00_JPRB, 0.79682E+00_JPRB, 0.76641E+00_JPRB, 0.81201E+00_JPRB, 0.83309E+00_JPRB, 0.84377E+00_JPRB, 0.84352E+00_JPRB ,&
&  0.88691E+00_JPRB, 0.96240E+00_JPRB, 0.10994E+01_JPRB, 0.14011E+01_JPRB, 0.18865E+01_JPRB, 0.36509E+01_JPRB, 0.36863E+01_JPRB ,&
&  0.16680E+01_JPRB, 0.15405E+01_JPRB, 0.89207E+00_JPRB, 0.99375E+00_JPRB, 0.99559E+00_JPRB, 0.80313E+00_JPRB, 0.80351E+00_JPRB ,&
&  0.91981E+00_JPRB, 0.99075E+00_JPRB, 0.10228E+01_JPRB, 0.97523E+00_JPRB, 0.10092E+01_JPRB, 0.10115E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.11330E+01_JPRB, 0.12025E+01_JPRB, 0.12680E+01_JPRB, 0.13253E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.14264E+01_JPRB, 0.14793E+01_JPRB, 0.15350E+01_JPRB, 0.15963E+01_JPRB, 0.16589E+01_JPRB, 0.17097E+01_JPRB, 0.17584E+01_JPRB ,&
&  0.18049E+01_JPRB, 0.18388E+01_JPRB, 0.18516E+01_JPRB, 0.18605E+01_JPRB, 0.18684E+01_JPRB, 0.19266E+01_JPRB, 0.19735E+01_JPRB ,&
&  0.19776E+01_JPRB, 0.20113E+01_JPRB, 0.20337E+01_JPRB, 0.20659E+01_JPRB, 0.20582E+01_JPRB, 0.20141E+01_JPRB, 0.20355E+01_JPRB ,&
&  0.20364E+01_JPRB, 0.20513E+01_JPRB, 0.20682E+01_JPRB, 0.21257E+01_JPRB, 0.22038E+01_JPRB, 0.24156E+01_JPRB, 0.21602E+01_JPRB ,&
&  0.24071E+01_JPRB, 0.25745E+01_JPRB, 0.24887E+01_JPRB, 0.22714E+01_JPRB, 0.20560E+01_JPRB, 0.20026E+01_JPRB, 0.19782E+01_JPRB ,&
&  0.19832E+01_JPRB, 0.19191E+01_JPRB, 0.19008E+01_JPRB, 0.18975E+01_JPRB, 0.19305E+01_JPRB, 0.20236E+01_JPRB, 0.20997E+01_JPRB ,&
&  0.22123E+01_JPRB, 0.23876E+01_JPRB, 0.25596E+01_JPRB, 0.27142E+01_JPRB, 0.29140E+01_JPRB, 0.31204E+01_JPRB, 0.32426E+01_JPRB ,&
&  0.32965E+01_JPRB, 0.33569E+01_JPRB, 0.30556E+01_JPRB, 0.30957E+01_JPRB, 0.32483E+01_JPRB, 0.29343E+01_JPRB, 0.24570E+01_JPRB ,&
&  0.22353E+01_JPRB, 0.17519E+01_JPRB, 0.14795E+01_JPRB, 0.15611E+01_JPRB, 0.13754E+01_JPRB, 0.11939E+01_JPRB, 0.10033E+01_JPRB ,&
&  0.79957E+00_JPRB, 0.70185E+00_JPRB, 0.70163E+00_JPRB, 0.75163E+00_JPRB, 0.75574E+00_JPRB, 0.78162E+00_JPRB, 0.84144E+00_JPRB ,&
&  0.93449E+00_JPRB, 0.10440E+01_JPRB, 0.12050E+01_JPRB, 0.14800E+01_JPRB, 0.18505E+01_JPRB, 0.19065E+01_JPRB, 0.20631E+01_JPRB ,&
&  0.14386E+01_JPRB, 0.11782E+01_JPRB, 0.11654E+01_JPRB, 0.10575E+01_JPRB, 0.10728E+01_JPRB, 0.10078E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.12501E+01_JPRB, 0.13981E+01_JPRB, 0.14150E+01_JPRB, 0.12131E+01_JPRB, 0.11257E+01_JPRB, 0.10953E+01_JPRB, 0.11087E+01_JPRB ,&
&  0.11390E+01_JPRB, 0.11814E+01_JPRB, 0.12431E+01_JPRB, 0.13125E+01_JPRB, 0.13734E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.14209E+01_JPRB, 0.14768E+01_JPRB, 0.15362E+01_JPRB, 0.15954E+01_JPRB, 0.16463E+01_JPRB, 0.16899E+01_JPRB, 0.17270E+01_JPRB ,&
&  0.17737E+01_JPRB, 0.18199E+01_JPRB, 0.18558E+01_JPRB, 0.18787E+01_JPRB, 0.19032E+01_JPRB, 0.19473E+01_JPRB, 0.19988E+01_JPRB ,&
&  0.20599E+01_JPRB, 0.21112E+01_JPRB, 0.21920E+01_JPRB, 0.22138E+01_JPRB, 0.22357E+01_JPRB, 0.22727E+01_JPRB, 0.23547E+01_JPRB ,&
&  0.24028E+01_JPRB, 0.24131E+01_JPRB, 0.24369E+01_JPRB, 0.25204E+01_JPRB, 0.26045E+01_JPRB, 0.27681E+01_JPRB, 0.25265E+01_JPRB ,&
&  0.26800E+01_JPRB, 0.27971E+01_JPRB, 0.26692E+01_JPRB, 0.25199E+01_JPRB, 0.22935E+01_JPRB, 0.22132E+01_JPRB, 0.21736E+01_JPRB ,&
&  0.21323E+01_JPRB, 0.20893E+01_JPRB, 0.20292E+01_JPRB, 0.19996E+01_JPRB, 0.19995E+01_JPRB, 0.20378E+01_JPRB, 0.21271E+01_JPRB ,&
&  0.22198E+01_JPRB, 0.23561E+01_JPRB, 0.25696E+01_JPRB, 0.27846E+01_JPRB, 0.30747E+01_JPRB, 0.33597E+01_JPRB, 0.35760E+01_JPRB ,&
&  0.37869E+01_JPRB, 0.39949E+01_JPRB, 0.37657E+01_JPRB, 0.38884E+01_JPRB, 0.41169E+01_JPRB, 0.36303E+01_JPRB, 0.27888E+01_JPRB ,&
&  0.19344E+01_JPRB, 0.16295E+01_JPRB, 0.16028E+01_JPRB, 0.16130E+01_JPRB, 0.13684E+01_JPRB, 0.11655E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.84230E+00_JPRB, 0.72577E+00_JPRB, 0.73820E+00_JPRB, 0.73744E+00_JPRB, 0.73827E+00_JPRB, 0.79166E+00_JPRB, 0.86996E+00_JPRB ,&
&  0.97578E+00_JPRB, 0.11392E+01_JPRB, 0.13249E+01_JPRB, 0.15369E+01_JPRB, 0.16898E+01_JPRB, 0.16452E+01_JPRB, 0.16748E+01_JPRB ,&
&  0.14535E+01_JPRB, 0.13926E+01_JPRB, 0.13381E+01_JPRB, 0.99316E+00_JPRB, 0.12313E+01_JPRB, 0.13074E+01_JPRB, 0.14325E+01_JPRB ,&
&  0.13901E+01_JPRB, 0.18161E+01_JPRB, 0.21660E+01_JPRB, 0.17169E+01_JPRB, 0.14207E+01_JPRB, 0.11396E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.11381E+01_JPRB, 0.11757E+01_JPRB, 0.12308E+01_JPRB, 0.12973E+01_JPRB, 0.13605E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.13612E+01_JPRB, 0.14221E+01_JPRB, 0.14864E+01_JPRB, 0.15413E+01_JPRB, 0.15839E+01_JPRB, 0.15932E+01_JPRB, 0.16003E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.16560E+01_JPRB, 0.16992E+01_JPRB, 0.17458E+01_JPRB, 0.17851E+01_JPRB, 0.18268E+01_JPRB, 0.18810E+01_JPRB ,&
&  0.19457E+01_JPRB, 0.20224E+01_JPRB, 0.21255E+01_JPRB, 0.22259E+01_JPRB, 0.23467E+01_JPRB, 0.24648E+01_JPRB, 0.25944E+01_JPRB ,&
&  0.27132E+01_JPRB, 0.28356E+01_JPRB, 0.28959E+01_JPRB, 0.29836E+01_JPRB, 0.30389E+01_JPRB, 0.31662E+01_JPRB, 0.25560E+01_JPRB ,&
&  0.28077E+01_JPRB, 0.28843E+01_JPRB, 0.28409E+01_JPRB, 0.26668E+01_JPRB, 0.24301E+01_JPRB, 0.23104E+01_JPRB, 0.22194E+01_JPRB ,&
&  0.21700E+01_JPRB, 0.20824E+01_JPRB, 0.19333E+01_JPRB, 0.19767E+01_JPRB, 0.20243E+01_JPRB, 0.19931E+01_JPRB, 0.20949E+01_JPRB ,&
&  0.21971E+01_JPRB, 0.23444E+01_JPRB, 0.25249E+01_JPRB, 0.27930E+01_JPRB, 0.31296E+01_JPRB, 0.34488E+01_JPRB, 0.37742E+01_JPRB ,&
&  0.41118E+01_JPRB, 0.44662E+01_JPRB, 0.47330E+01_JPRB, 0.43222E+01_JPRB, 0.43157E+01_JPRB, 0.38005E+01_JPRB, 0.27992E+01_JPRB ,&
&  0.18542E+01_JPRB, 0.15932E+01_JPRB, 0.16115E+01_JPRB, 0.16333E+01_JPRB, 0.14005E+01_JPRB, 0.11714E+01_JPRB, 0.10086E+01_JPRB ,&
&  0.85619E+00_JPRB, 0.78259E+00_JPRB, 0.77156E+00_JPRB, 0.75009E+00_JPRB, 0.75477E+00_JPRB, 0.80944E+00_JPRB, 0.88735E+00_JPRB ,&
&  0.98585E+00_JPRB, 0.11266E+01_JPRB, 0.13065E+01_JPRB, 0.15069E+01_JPRB, 0.17032E+01_JPRB, 0.17719E+01_JPRB, 0.15686E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.13228E+01_JPRB, 0.13354E+01_JPRB, 0.14197E+01_JPRB, 0.14091E+01_JPRB, 0.16416E+01_JPRB, 0.18802E+01_JPRB ,&
&  0.21049E+01_JPRB, 0.19289E+01_JPRB, 0.22218E+01_JPRB, 0.30398E+01_JPRB, 0.27919E+01_JPRB, 0.23335E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.10652E+01_JPRB, 0.11263E+01_JPRB, 0.12051E+01_JPRB, 0.12936E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.10805E+01_JPRB, 0.11584E+01_JPRB, 0.12396E+01_JPRB, 0.13485E+01_JPRB, 0.14485E+01_JPRB, 0.14819E+01_JPRB, 0.14782E+01_JPRB ,&
&  0.14795E+01_JPRB, 0.14937E+01_JPRB, 0.15275E+01_JPRB, 0.15738E+01_JPRB, 0.16179E+01_JPRB, 0.16574E+01_JPRB, 0.17044E+01_JPRB ,&
&  0.17662E+01_JPRB, 0.18371E+01_JPRB, 0.19222E+01_JPRB, 0.20156E+01_JPRB, 0.21149E+01_JPRB, 0.22549E+01_JPRB, 0.24243E+01_JPRB ,&
&  0.26095E+01_JPRB, 0.27907E+01_JPRB, 0.29334E+01_JPRB, 0.30730E+01_JPRB, 0.31930E+01_JPRB, 0.31841E+01_JPRB, 0.22877E+01_JPRB ,&
&  0.28142E+01_JPRB, 0.28714E+01_JPRB, 0.29303E+01_JPRB, 0.27678E+01_JPRB, 0.24870E+01_JPRB, 0.23238E+01_JPRB, 0.22488E+01_JPRB ,&
&  0.21658E+01_JPRB, 0.20470E+01_JPRB, 0.19100E+01_JPRB, 0.18167E+01_JPRB, 0.19558E+01_JPRB, 0.19125E+01_JPRB, 0.20293E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.22888E+01_JPRB, 0.24581E+01_JPRB, 0.27237E+01_JPRB, 0.30745E+01_JPRB, 0.34360E+01_JPRB, 0.38314E+01_JPRB ,&
&  0.42307E+01_JPRB, 0.45859E+01_JPRB, 0.47860E+01_JPRB, 0.41966E+01_JPRB, 0.39821E+01_JPRB, 0.36684E+01_JPRB, 0.24237E+01_JPRB ,&
&  0.18744E+01_JPRB, 0.17577E+01_JPRB, 0.18184E+01_JPRB, 0.17225E+01_JPRB, 0.14911E+01_JPRB, 0.12958E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.89183E+00_JPRB, 0.79623E+00_JPRB, 0.78378E+00_JPRB, 0.77934E+00_JPRB, 0.79262E+00_JPRB, 0.85344E+00_JPRB, 0.92888E+00_JPRB ,&
&  0.10086E+01_JPRB, 0.11080E+01_JPRB, 0.12006E+01_JPRB, 0.12922E+01_JPRB, 0.13937E+01_JPRB, 0.15627E+01_JPRB, 0.16687E+01_JPRB ,&
&  0.14603E+01_JPRB, 0.12987E+01_JPRB, 0.12596E+01_JPRB, 0.12555E+01_JPRB, 0.12706E+01_JPRB, 0.13332E+01_JPRB, 0.14023E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.16054E+01_JPRB, 0.13532E+01_JPRB, 0.11275E+01_JPRB, 0.14110E+01_JPRB, 0.15216E+01_JPRB, 0.87448E+00_JPRB ,&
&  0.85411E+00_JPRB, 0.88104E+00_JPRB, 0.91300E+00_JPRB, 0.96789E+00_JPRB, 0.10228E+01_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.85412E+00_JPRB, 0.88324E+00_JPRB, 0.98664E+00_JPRB, 0.10283E+01_JPRB, 0.11388E+01_JPRB, 0.12001E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.13252E+01_JPRB, 0.13451E+01_JPRB, 0.13719E+01_JPRB, 0.14054E+01_JPRB, 0.14487E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.15393E+01_JPRB, 0.15987E+01_JPRB, 0.16675E+01_JPRB, 0.17416E+01_JPRB, 0.18380E+01_JPRB, 0.19520E+01_JPRB, 0.21105E+01_JPRB ,&
&  0.23123E+01_JPRB, 0.25500E+01_JPRB, 0.27958E+01_JPRB, 0.30422E+01_JPRB, 0.32982E+01_JPRB, 0.34969E+01_JPRB, 0.35978E+01_JPRB ,&
&  0.23826E+01_JPRB, 0.29508E+01_JPRB, 0.29130E+01_JPRB, 0.27483E+01_JPRB, 0.25340E+01_JPRB, 0.23944E+01_JPRB, 0.23201E+01_JPRB ,&
&  0.22423E+01_JPRB, 0.20528E+01_JPRB, 0.19684E+01_JPRB, 0.18417E+01_JPRB, 0.18189E+01_JPRB, 0.18119E+01_JPRB, 0.19187E+01_JPRB ,&
&  0.20219E+01_JPRB, 0.21733E+01_JPRB, 0.23750E+01_JPRB, 0.25808E+01_JPRB, 0.29090E+01_JPRB, 0.33480E+01_JPRB, 0.38057E+01_JPRB ,&
&  0.42886E+01_JPRB, 0.47954E+01_JPRB, 0.50942E+01_JPRB, 0.45184E+01_JPRB, 0.39483E+01_JPRB, 0.37159E+01_JPRB, 0.26865E+01_JPRB ,&
&  0.21436E+01_JPRB, 0.18468E+01_JPRB, 0.19624E+01_JPRB, 0.19352E+01_JPRB, 0.16608E+01_JPRB, 0.14846E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.89551E+00_JPRB, 0.83889E+00_JPRB, 0.84211E+00_JPRB, 0.86591E+00_JPRB, 0.91389E+00_JPRB, 0.95525E+00_JPRB ,&
&  0.10024E+01_JPRB, 0.10542E+01_JPRB, 0.11003E+01_JPRB, 0.11509E+01_JPRB, 0.11745E+01_JPRB, 0.11957E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.12269E+01_JPRB, 0.12149E+01_JPRB, 0.12143E+01_JPRB, 0.11832E+01_JPRB, 0.11761E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.12035E+01_JPRB, 0.12103E+01_JPRB, 0.11857E+01_JPRB, 0.11375E+01_JPRB, 0.10494E+01_JPRB, 0.96550E+00_JPRB, 0.80898E+00_JPRB ,&
&  0.79029E+00_JPRB, 0.77442E+00_JPRB, 0.79545E+00_JPRB, 0.81260E+00_JPRB, 0.83834E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.73782E+00_JPRB, 0.76488E+00_JPRB, 0.79245E+00_JPRB, 0.82419E+00_JPRB, 0.87514E+00_JPRB, 0.93767E+00_JPRB, 0.10206E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.11422E+01_JPRB, 0.11548E+01_JPRB, 0.11643E+01_JPRB, 0.11839E+01_JPRB, 0.12232E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.13176E+01_JPRB, 0.13609E+01_JPRB, 0.14026E+01_JPRB, 0.14520E+01_JPRB, 0.15130E+01_JPRB, 0.16031E+01_JPRB, 0.17139E+01_JPRB ,&
&  0.18572E+01_JPRB, 0.20843E+01_JPRB, 0.23826E+01_JPRB, 0.27300E+01_JPRB, 0.31072E+01_JPRB, 0.34178E+01_JPRB, 0.35917E+01_JPRB ,&
&  0.19551E+01_JPRB, 0.25402E+01_JPRB, 0.30665E+01_JPRB, 0.28599E+01_JPRB, 0.26393E+01_JPRB, 0.25133E+01_JPRB, 0.24388E+01_JPRB ,&
&  0.22683E+01_JPRB, 0.20500E+01_JPRB, 0.19458E+01_JPRB, 0.19512E+01_JPRB, 0.17520E+01_JPRB, 0.17308E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.18611E+01_JPRB, 0.19452E+01_JPRB, 0.20936E+01_JPRB, 0.23000E+01_JPRB, 0.25957E+01_JPRB, 0.30499E+01_JPRB, 0.35715E+01_JPRB ,&
&  0.41407E+01_JPRB, 0.48040E+01_JPRB, 0.54010E+01_JPRB, 0.49078E+01_JPRB, 0.48489E+01_JPRB, 0.49941E+01_JPRB, 0.45181E+01_JPRB ,&
&  0.29539E+01_JPRB, 0.22065E+01_JPRB, 0.22700E+01_JPRB, 0.22448E+01_JPRB, 0.18925E+01_JPRB, 0.17138E+01_JPRB, 0.15441E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.11704E+01_JPRB, 0.97409E+00_JPRB, 0.92806E+00_JPRB, 0.94418E+00_JPRB, 0.98218E+00_JPRB, 0.10168E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.10441E+01_JPRB, 0.10640E+01_JPRB, 0.10906E+01_JPRB, 0.11341E+01_JPRB, 0.11525E+01_JPRB, 0.11521E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11819E+01_JPRB, 0.11949E+01_JPRB, 0.12167E+01_JPRB, 0.12375E+01_JPRB, 0.12431E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.11961E+01_JPRB, 0.11951E+01_JPRB, 0.12052E+01_JPRB, 0.11859E+01_JPRB, 0.11301E+01_JPRB, 0.10182E+01_JPRB, 0.92808E+00_JPRB ,&
&  0.83777E+00_JPRB, 0.82533E+00_JPRB, 0.79646E+00_JPRB, 0.76970E+00_JPRB, 0.74474E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.72908E+00_JPRB, 0.72557E+00_JPRB, 0.73589E+00_JPRB, 0.74417E+00_JPRB, 0.76195E+00_JPRB, 0.79765E+00_JPRB, 0.84366E+00_JPRB ,&
&  0.88386E+00_JPRB, 0.95101E+00_JPRB, 0.99969E+00_JPRB, 0.10024E+01_JPRB, 0.99713E+00_JPRB, 0.10162E+01_JPRB, 0.10558E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.11525E+01_JPRB, 0.11865E+01_JPRB, 0.12172E+01_JPRB, 0.12553E+01_JPRB, 0.12953E+01_JPRB, 0.13471E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.15585E+01_JPRB, 0.17720E+01_JPRB, 0.21094E+01_JPRB, 0.25757E+01_JPRB, 0.31236E+01_JPRB, 0.35088E+01_JPRB ,&
&  0.24925E+01_JPRB, 0.11836E+01_JPRB, 0.26816E+01_JPRB, 0.31785E+01_JPRB, 0.28557E+01_JPRB, 0.27662E+01_JPRB, 0.25846E+01_JPRB ,&
&  0.23879E+01_JPRB, 0.22862E+01_JPRB, 0.21126E+01_JPRB, 0.19885E+01_JPRB, 0.17442E+01_JPRB, 0.16717E+01_JPRB, 0.16488E+01_JPRB ,&
&  0.16716E+01_JPRB, 0.17158E+01_JPRB, 0.18115E+01_JPRB, 0.19605E+01_JPRB, 0.21797E+01_JPRB, 0.25429E+01_JPRB, 0.31119E+01_JPRB ,&
&  0.37327E+01_JPRB, 0.44856E+01_JPRB, 0.54729E+01_JPRB, 0.53579E+01_JPRB, 0.56036E+01_JPRB, 0.51261E+01_JPRB, 0.42784E+01_JPRB ,&
&  0.33185E+01_JPRB, 0.26430E+01_JPRB, 0.26385E+01_JPRB, 0.26430E+01_JPRB, 0.22211E+01_JPRB, 0.18671E+01_JPRB, 0.18404E+01_JPRB ,&
&  0.15801E+01_JPRB, 0.14609E+01_JPRB, 0.12412E+01_JPRB, 0.10902E+01_JPRB, 0.10474E+01_JPRB, 0.10724E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10957E+01_JPRB, 0.10992E+01_JPRB, 0.11152E+01_JPRB, 0.11434E+01_JPRB, 0.11622E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.11875E+01_JPRB, 0.12098E+01_JPRB, 0.12320E+01_JPRB, 0.12539E+01_JPRB, 0.12689E+01_JPRB, 0.12468E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.12487E+01_JPRB, 0.12426E+01_JPRB, 0.12261E+01_JPRB, 0.12251E+01_JPRB, 0.11930E+01_JPRB, 0.11219E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.10107E+01_JPRB, 0.93546E+00_JPRB, 0.84372E+00_JPRB, 0.78506E+00_JPRB, 0.76173E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.85516E+00_JPRB, 0.80989E+00_JPRB, 0.76283E+00_JPRB, 0.75035E+00_JPRB, 0.74969E+00_JPRB, 0.74630E+00_JPRB, 0.76781E+00_JPRB ,&
&  0.78735E+00_JPRB, 0.81780E+00_JPRB, 0.83118E+00_JPRB, 0.86047E+00_JPRB, 0.87215E+00_JPRB, 0.87880E+00_JPRB, 0.90123E+00_JPRB ,&
&  0.92331E+00_JPRB, 0.95022E+00_JPRB, 0.97927E+00_JPRB, 0.10122E+01_JPRB, 0.10453E+01_JPRB, 0.10606E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.11787E+01_JPRB, 0.12697E+01_JPRB, 0.14598E+01_JPRB, 0.18002E+01_JPRB, 0.22936E+01_JPRB, 0.28838E+01_JPRB ,&
&  0.34716E+01_JPRB, 0.24503E+01_JPRB, 0.16634E+01_JPRB, 0.29710E+01_JPRB, 0.32638E+01_JPRB, 0.32152E+01_JPRB, 0.27265E+01_JPRB ,&
&  0.27448E+01_JPRB, 0.25942E+01_JPRB, 0.24046E+01_JPRB, 0.21697E+01_JPRB, 0.18451E+01_JPRB, 0.16847E+01_JPRB, 0.15718E+01_JPRB ,&
&  0.15143E+01_JPRB, 0.14927E+01_JPRB, 0.15407E+01_JPRB, 0.16810E+01_JPRB, 0.18451E+01_JPRB, 0.20800E+01_JPRB, 0.25393E+01_JPRB ,&
&  0.31398E+01_JPRB, 0.38935E+01_JPRB, 0.50779E+01_JPRB, 0.56915E+01_JPRB, 0.65415E+01_JPRB, 0.55918E+01_JPRB, 0.45587E+01_JPRB ,&
&  0.35470E+01_JPRB, 0.33589E+01_JPRB, 0.34501E+01_JPRB, 0.30744E+01_JPRB, 0.24863E+01_JPRB, 0.22110E+01_JPRB, 0.21880E+01_JPRB ,&
&  0.18741E+01_JPRB, 0.17109E+01_JPRB, 0.15696E+01_JPRB, 0.13722E+01_JPRB, 0.12466E+01_JPRB, 0.12062E+01_JPRB, 0.12004E+01_JPRB ,&
&  0.11893E+01_JPRB, 0.11640E+01_JPRB, 0.11495E+01_JPRB, 0.11537E+01_JPRB, 0.11652E+01_JPRB, 0.11737E+01_JPRB, 0.11880E+01_JPRB ,&
&  0.12035E+01_JPRB, 0.12222E+01_JPRB, 0.12445E+01_JPRB, 0.12568E+01_JPRB, 0.12427E+01_JPRB, 0.12197E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12746E+01_JPRB, 0.12561E+01_JPRB, 0.12540E+01_JPRB, 0.12316E+01_JPRB, 0.12157E+01_JPRB, 0.11994E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11452E+01_JPRB, 0.10498E+01_JPRB, 0.93329E+00_JPRB, 0.89857E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.97151E+00_JPRB, 0.93493E+00_JPRB, 0.89061E+00_JPRB, 0.84589E+00_JPRB, 0.82207E+00_JPRB, 0.79786E+00_JPRB, 0.76782E+00_JPRB ,&
&  0.75072E+00_JPRB, 0.74264E+00_JPRB, 0.72955E+00_JPRB, 0.74109E+00_JPRB, 0.77120E+00_JPRB, 0.78767E+00_JPRB, 0.80297E+00_JPRB ,&
&  0.82070E+00_JPRB, 0.83866E+00_JPRB, 0.83893E+00_JPRB, 0.84500E+00_JPRB, 0.86574E+00_JPRB, 0.88201E+00_JPRB, 0.88526E+00_JPRB ,&
&  0.89726E+00_JPRB, 0.92584E+00_JPRB, 0.96741E+00_JPRB, 0.10476E+01_JPRB, 0.12129E+01_JPRB, 0.14865E+01_JPRB, 0.18836E+01_JPRB ,&
&  0.25107E+01_JPRB, 0.25776E+01_JPRB, 0.13541E+01_JPRB, 0.25121E+01_JPRB, 0.34831E+01_JPRB, 0.35354E+01_JPRB, 0.32425E+01_JPRB ,&
&  0.32139E+01_JPRB, 0.30967E+01_JPRB, 0.29040E+01_JPRB, 0.24265E+01_JPRB, 0.21004E+01_JPRB, 0.18541E+01_JPRB, 0.16510E+01_JPRB ,&
&  0.14879E+01_JPRB, 0.13761E+01_JPRB, 0.13344E+01_JPRB, 0.14007E+01_JPRB, 0.15382E+01_JPRB, 0.16711E+01_JPRB, 0.19333E+01_JPRB ,&
&  0.24023E+01_JPRB, 0.30211E+01_JPRB, 0.40495E+01_JPRB, 0.51157E+01_JPRB, 0.67443E+01_JPRB, 0.67651E+01_JPRB, 0.54764E+01_JPRB ,&
&  0.42832E+01_JPRB, 0.41951E+01_JPRB, 0.41444E+01_JPRB, 0.36280E+01_JPRB, 0.27992E+01_JPRB, 0.25442E+01_JPRB, 0.24907E+01_JPRB ,&
&  0.21790E+01_JPRB, 0.19347E+01_JPRB, 0.18207E+01_JPRB, 0.16937E+01_JPRB, 0.15159E+01_JPRB, 0.13650E+01_JPRB, 0.12962E+01_JPRB ,&
&  0.12715E+01_JPRB, 0.12460E+01_JPRB, 0.12288E+01_JPRB, 0.12117E+01_JPRB, 0.12016E+01_JPRB, 0.11972E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12304E+01_JPRB, 0.12464E+01_JPRB, 0.12424E+01_JPRB, 0.12371E+01_JPRB, 0.12336E+01_JPRB, 0.12338E+01_JPRB ,&
&  0.12330E+01_JPRB, 0.12372E+01_JPRB, 0.12262E+01_JPRB, 0.11917E+01_JPRB, 0.12312E+01_JPRB, 0.12241E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.12544E+01_JPRB, 0.11786E+01_JPRB, 0.10416E+01_JPRB, 0.10084E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.10181E+01_JPRB, 0.98948E+00_JPRB, 0.96111E+00_JPRB, 0.93679E+00_JPRB, 0.91589E+00_JPRB, 0.89440E+00_JPRB, 0.86092E+00_JPRB ,&
&  0.81919E+00_JPRB, 0.79125E+00_JPRB, 0.75158E+00_JPRB, 0.73605E+00_JPRB, 0.73907E+00_JPRB, 0.74320E+00_JPRB, 0.75585E+00_JPRB ,&
&  0.76676E+00_JPRB, 0.77915E+00_JPRB, 0.78723E+00_JPRB, 0.79042E+00_JPRB, 0.79428E+00_JPRB, 0.79854E+00_JPRB, 0.79766E+00_JPRB ,&
&  0.79135E+00_JPRB, 0.79734E+00_JPRB, 0.81544E+00_JPRB, 0.84636E+00_JPRB, 0.91512E+00_JPRB, 0.10471E+01_JPRB, 0.12504E+01_JPRB ,&
&  0.17469E+01_JPRB, 0.20761E+01_JPRB, 0.22611E+01_JPRB, 0.34754E+01_JPRB, 0.41786E+01_JPRB, 0.40311E+01_JPRB, 0.40185E+01_JPRB ,&
&  0.41104E+01_JPRB, 0.40386E+01_JPRB, 0.36644E+01_JPRB, 0.27600E+01_JPRB, 0.22028E+01_JPRB, 0.19195E+01_JPRB, 0.17049E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.13710E+01_JPRB, 0.12608E+01_JPRB, 0.12232E+01_JPRB, 0.12690E+01_JPRB, 0.13809E+01_JPRB, 0.15151E+01_JPRB ,&
&  0.17794E+01_JPRB, 0.22402E+01_JPRB, 0.29339E+01_JPRB, 0.38406E+01_JPRB, 0.54589E+01_JPRB, 0.66611E+01_JPRB, 0.63452E+01_JPRB ,&
&  0.59151E+01_JPRB, 0.53688E+01_JPRB, 0.47330E+01_JPRB, 0.42124E+01_JPRB, 0.31585E+01_JPRB, 0.30494E+01_JPRB, 0.26970E+01_JPRB ,&
&  0.23466E+01_JPRB, 0.20940E+01_JPRB, 0.19499E+01_JPRB, 0.18638E+01_JPRB, 0.17386E+01_JPRB, 0.15640E+01_JPRB, 0.14203E+01_JPRB ,&
&  0.13420E+01_JPRB, 0.13100E+01_JPRB, 0.12974E+01_JPRB, 0.12797E+01_JPRB, 0.12637E+01_JPRB, 0.12531E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.12327E+01_JPRB, 0.12110E+01_JPRB, 0.12170E+01_JPRB, 0.12104E+01_JPRB, 0.11975E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.12138E+01_JPRB, 0.12147E+01_JPRB, 0.12039E+01_JPRB, 0.11957E+01_JPRB, 0.11886E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11352E+01_JPRB, 0.10746E+01_JPRB, 0.10538E+01_JPRB, 0.10380E+01_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.10272E+01_JPRB, 0.10028E+01_JPRB, 0.98545E+00_JPRB, 0.96517E+00_JPRB, 0.94449E+00_JPRB, 0.93065E+00_JPRB, 0.91793E+00_JPRB ,&
&  0.89672E+00_JPRB, 0.85203E+00_JPRB, 0.80559E+00_JPRB, 0.77927E+00_JPRB, 0.76717E+00_JPRB, 0.74812E+00_JPRB, 0.74600E+00_JPRB ,&
&  0.74858E+00_JPRB, 0.75754E+00_JPRB, 0.75773E+00_JPRB, 0.75829E+00_JPRB, 0.75905E+00_JPRB, 0.76087E+00_JPRB, 0.75844E+00_JPRB ,&
&  0.74763E+00_JPRB, 0.74220E+00_JPRB, 0.74732E+00_JPRB, 0.75948E+00_JPRB, 0.78746E+00_JPRB, 0.85042E+00_JPRB, 0.93821E+00_JPRB ,&
&  0.12362E+01_JPRB, 0.16554E+01_JPRB, 0.17392E+01_JPRB, 0.34036E+01_JPRB, 0.40444E+01_JPRB, 0.43440E+01_JPRB, 0.41590E+01_JPRB ,&
&  0.42606E+01_JPRB, 0.47599E+01_JPRB, 0.39745E+01_JPRB, 0.28647E+01_JPRB, 0.21290E+01_JPRB, 0.17885E+01_JPRB, 0.15938E+01_JPRB ,&
&  0.14349E+01_JPRB, 0.12986E+01_JPRB, 0.12044E+01_JPRB, 0.11500E+01_JPRB, 0.11482E+01_JPRB, 0.11781E+01_JPRB, 0.12469E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.16788E+01_JPRB, 0.21521E+01_JPRB, 0.26189E+01_JPRB, 0.38078E+01_JPRB, 0.51747E+01_JPRB, 0.59596E+01_JPRB ,&
&  0.63057E+01_JPRB, 0.62168E+01_JPRB, 0.51167E+01_JPRB, 0.44362E+01_JPRB, 0.36215E+01_JPRB, 0.31865E+01_JPRB, 0.27464E+01_JPRB ,&
&  0.24384E+01_JPRB, 0.21972E+01_JPRB, 0.20221E+01_JPRB, 0.19191E+01_JPRB, 0.18169E+01_JPRB, 0.16937E+01_JPRB, 0.15418E+01_JPRB ,&
&  0.14128E+01_JPRB, 0.13488E+01_JPRB, 0.13209E+01_JPRB, 0.13057E+01_JPRB, 0.13021E+01_JPRB, 0.12940E+01_JPRB, 0.12801E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.12455E+01_JPRB, 0.12078E+01_JPRB, 0.12025E+01_JPRB, 0.11789E+01_JPRB, 0.11384E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.12036E+01_JPRB, 0.12018E+01_JPRB, 0.11715E+01_JPRB, 0.11800E+01_JPRB, 0.11948E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.10966E+01_JPRB, 0.10686E+01_JPRB, 0.10533E+01_JPRB, 0.10432E+01_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.10173E+01_JPRB, 0.98559E+00_JPRB, 0.96268E+00_JPRB, 0.94328E+00_JPRB, 0.92456E+00_JPRB, 0.90729E+00_JPRB, 0.90107E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.87950E+00_JPRB, 0.84851E+00_JPRB, 0.81215E+00_JPRB, 0.79197E+00_JPRB, 0.78060E+00_JPRB, 0.76062E+00_JPRB ,&
&  0.75043E+00_JPRB, 0.74973E+00_JPRB, 0.74936E+00_JPRB, 0.74285E+00_JPRB, 0.73576E+00_JPRB, 0.72979E+00_JPRB, 0.72447E+00_JPRB ,&
&  0.70919E+00_JPRB, 0.70206E+00_JPRB, 0.70190E+00_JPRB, 0.70278E+00_JPRB, 0.71115E+00_JPRB, 0.75772E+00_JPRB, 0.84884E+00_JPRB ,&
&  0.10253E+01_JPRB, 0.18678E+01_JPRB, 0.17237E+01_JPRB, 0.27548E+01_JPRB, 0.31586E+01_JPRB, 0.36357E+01_JPRB, 0.39325E+01_JPRB ,&
&  0.42640E+01_JPRB, 0.46303E+01_JPRB, 0.37625E+01_JPRB, 0.27616E+01_JPRB, 0.19466E+01_JPRB, 0.15165E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.11631E+01_JPRB, 0.11034E+01_JPRB, 0.10804E+01_JPRB, 0.10802E+01_JPRB, 0.10884E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.13503E+01_JPRB, 0.17569E+01_JPRB, 0.23306E+01_JPRB, 0.26913E+01_JPRB, 0.36161E+01_JPRB, 0.49873E+01_JPRB ,&
&  0.58700E+01_JPRB, 0.64441E+01_JPRB, 0.53711E+01_JPRB, 0.44834E+01_JPRB, 0.38047E+01_JPRB, 0.32815E+01_JPRB, 0.28371E+01_JPRB ,&
&  0.24949E+01_JPRB, 0.22427E+01_JPRB, 0.20587E+01_JPRB, 0.19282E+01_JPRB, 0.18352E+01_JPRB, 0.17625E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.15418E+01_JPRB, 0.14513E+01_JPRB, 0.13928E+01_JPRB, 0.13603E+01_JPRB, 0.13499E+01_JPRB, 0.13305E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.12599E+01_JPRB, 0.12452E+01_JPRB, 0.12104E+01_JPRB, 0.11997E+01_JPRB, 0.11795E+01_JPRB, 0.11323E+01_JPRB, 0.11131E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.11778E+01_JPRB, 0.11825E+01_JPRB, 0.11799E+01_JPRB, 0.11825E+01_JPRB, 0.12313E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.10784E+01_JPRB, 0.10575E+01_JPRB, 0.10488E+01_JPRB, 0.10367E+01_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.99271E+00_JPRB, 0.96034E+00_JPRB, 0.93348E+00_JPRB, 0.90685E+00_JPRB, 0.88633E+00_JPRB, 0.86459E+00_JPRB, 0.85003E+00_JPRB ,&
&  0.84286E+00_JPRB, 0.84203E+00_JPRB, 0.83819E+00_JPRB, 0.82345E+00_JPRB, 0.79917E+00_JPRB, 0.77889E+00_JPRB, 0.76355E+00_JPRB ,&
&  0.74865E+00_JPRB, 0.74303E+00_JPRB, 0.73988E+00_JPRB, 0.72875E+00_JPRB, 0.71634E+00_JPRB, 0.70430E+00_JPRB, 0.69173E+00_JPRB ,&
&  0.68018E+00_JPRB, 0.67435E+00_JPRB, 0.67035E+00_JPRB, 0.66671E+00_JPRB, 0.66491E+00_JPRB, 0.69082E+00_JPRB, 0.81890E+00_JPRB ,&
&  0.10970E+01_JPRB, 0.97081E+00_JPRB, 0.21612E+01_JPRB, 0.24965E+01_JPRB, 0.26144E+01_JPRB, 0.28179E+01_JPRB, 0.33402E+01_JPRB ,&
&  0.38756E+01_JPRB, 0.38507E+01_JPRB, 0.33311E+01_JPRB, 0.25860E+01_JPRB, 0.19228E+01_JPRB, 0.14939E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.11229E+01_JPRB, 0.10404E+01_JPRB, 0.10007E+01_JPRB, 0.97944E+00_JPRB, 0.97843E+00_JPRB, 0.99310E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.10541E+01_JPRB, 0.11650E+01_JPRB, 0.14544E+01_JPRB, 0.20149E+01_JPRB, 0.26077E+01_JPRB, 0.30561E+01_JPRB, 0.39235E+01_JPRB ,&
&  0.51161E+01_JPRB, 0.55417E+01_JPRB, 0.46298E+01_JPRB, 0.39396E+01_JPRB, 0.34930E+01_JPRB, 0.31692E+01_JPRB, 0.28434E+01_JPRB ,&
&  0.25291E+01_JPRB, 0.22271E+01_JPRB, 0.20072E+01_JPRB, 0.18523E+01_JPRB, 0.17647E+01_JPRB, 0.17320E+01_JPRB, 0.17019E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.15944E+01_JPRB, 0.15286E+01_JPRB, 0.14680E+01_JPRB, 0.14204E+01_JPRB, 0.13756E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.12279E+01_JPRB, 0.11923E+01_JPRB, 0.11642E+01_JPRB, 0.11528E+01_JPRB, 0.11372E+01_JPRB, 0.11296E+01_JPRB ,&
&  0.11418E+01_JPRB, 0.11455E+01_JPRB, 0.11468E+01_JPRB, 0.11551E+01_JPRB, 0.11382E+01_JPRB, 0.11865E+01_JPRB, 0.11497E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.10883E+01_JPRB, 0.10518E+01_JPRB, 0.10315E+01_JPRB, 0.10166E+01_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.96265E+00_JPRB, 0.93642E+00_JPRB, 0.90129E+00_JPRB, 0.87038E+00_JPRB, 0.84577E+00_JPRB, 0.81984E+00_JPRB, 0.80133E+00_JPRB ,&
&  0.78793E+00_JPRB, 0.78937E+00_JPRB, 0.79552E+00_JPRB, 0.79816E+00_JPRB, 0.78962E+00_JPRB, 0.76870E+00_JPRB, 0.74740E+00_JPRB ,&
&  0.73137E+00_JPRB, 0.72445E+00_JPRB, 0.72121E+00_JPRB, 0.71392E+00_JPRB, 0.70331E+00_JPRB, 0.68974E+00_JPRB, 0.67728E+00_JPRB ,&
&  0.66541E+00_JPRB, 0.65723E+00_JPRB, 0.64939E+00_JPRB, 0.64309E+00_JPRB, 0.63619E+00_JPRB, 0.64928E+00_JPRB, 0.77043E+00_JPRB ,&
&  0.11521E+01_JPRB, 0.16417E+01_JPRB, 0.21172E+01_JPRB, 0.19965E+01_JPRB, 0.19888E+01_JPRB, 0.20946E+01_JPRB, 0.24803E+01_JPRB ,&
&  0.26979E+01_JPRB, 0.27939E+01_JPRB, 0.26436E+01_JPRB, 0.22702E+01_JPRB, 0.18190E+01_JPRB, 0.15029E+01_JPRB, 0.12737E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.97379E+00_JPRB, 0.91150E+00_JPRB, 0.89250E+00_JPRB, 0.89785E+00_JPRB, 0.91454E+00_JPRB, 0.93010E+00_JPRB ,&
&  0.95299E+00_JPRB, 0.10146E+01_JPRB, 0.12282E+01_JPRB, 0.17318E+01_JPRB, 0.25894E+01_JPRB, 0.26999E+01_JPRB, 0.36405E+01_JPRB ,&
&  0.45996E+01_JPRB, 0.41877E+01_JPRB, 0.38004E+01_JPRB, 0.34661E+01_JPRB, 0.32024E+01_JPRB, 0.29693E+01_JPRB, 0.26032E+01_JPRB ,&
&  0.23091E+01_JPRB, 0.20346E+01_JPRB, 0.18278E+01_JPRB, 0.17078E+01_JPRB, 0.16341E+01_JPRB, 0.16053E+01_JPRB, 0.16007E+01_JPRB ,&
&  0.15922E+01_JPRB, 0.15715E+01_JPRB, 0.15504E+01_JPRB, 0.15057E+01_JPRB, 0.14441E+01_JPRB, 0.13820E+01_JPRB, 0.13178E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.12055E+01_JPRB, 0.11557E+01_JPRB, 0.11117E+01_JPRB, 0.11031E+01_JPRB, 0.11054E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.10767E+01_JPRB, 0.10922E+01_JPRB, 0.10906E+01_JPRB, 0.10941E+01_JPRB, 0.11473E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.10690E+01_JPRB, 0.10320E+01_JPRB, 0.10020E+01_JPRB, 0.98332E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.93686E+00_JPRB, 0.90423E+00_JPRB, 0.86875E+00_JPRB, 0.83635E+00_JPRB, 0.80689E+00_JPRB, 0.78416E+00_JPRB, 0.76957E+00_JPRB ,&
&  0.75684E+00_JPRB, 0.74992E+00_JPRB, 0.75371E+00_JPRB, 0.76093E+00_JPRB, 0.76024E+00_JPRB, 0.74818E+00_JPRB, 0.73013E+00_JPRB ,&
&  0.71128E+00_JPRB, 0.69639E+00_JPRB, 0.68547E+00_JPRB, 0.68039E+00_JPRB, 0.67377E+00_JPRB, 0.66615E+00_JPRB, 0.65387E+00_JPRB ,&
&  0.64357E+00_JPRB, 0.63545E+00_JPRB, 0.62415E+00_JPRB, 0.61236E+00_JPRB, 0.60341E+00_JPRB, 0.60844E+00_JPRB, 0.69902E+00_JPRB ,&
&  0.98609E+00_JPRB, 0.14046E+01_JPRB, 0.14572E+01_JPRB, 0.15069E+01_JPRB, 0.16527E+01_JPRB, 0.16916E+01_JPRB, 0.18398E+01_JPRB ,&
&  0.19123E+01_JPRB, 0.19587E+01_JPRB, 0.19813E+01_JPRB, 0.18634E+01_JPRB, 0.16208E+01_JPRB, 0.14278E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.97950E+00_JPRB, 0.90427E+00_JPRB, 0.87168E+00_JPRB, 0.86894E+00_JPRB, 0.87963E+00_JPRB, 0.88579E+00_JPRB ,&
&  0.89067E+00_JPRB, 0.89905E+00_JPRB, 0.97304E+00_JPRB, 0.11668E+01_JPRB, 0.16608E+01_JPRB, 0.20258E+01_JPRB, 0.25714E+01_JPRB ,&
&  0.26877E+01_JPRB, 0.26565E+01_JPRB, 0.25901E+01_JPRB, 0.25185E+01_JPRB, 0.24214E+01_JPRB, 0.22645E+01_JPRB, 0.21239E+01_JPRB ,&
&  0.19489E+01_JPRB, 0.17662E+01_JPRB, 0.16219E+01_JPRB, 0.15241E+01_JPRB, 0.14614E+01_JPRB, 0.14232E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.13857E+01_JPRB, 0.13775E+01_JPRB, 0.13552E+01_JPRB, 0.13358E+01_JPRB, 0.13209E+01_JPRB, 0.12829E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.11610E+01_JPRB, 0.11062E+01_JPRB, 0.10637E+01_JPRB, 0.10381E+01_JPRB, 0.10238E+01_JPRB, 0.10127E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.10103E+01_JPRB, 0.10149E+01_JPRB, 0.10108E+01_JPRB, 0.10064E+01_JPRB, 0.10465E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.10116E+01_JPRB, 0.98914E+00_JPRB, 0.96192E+00_JPRB, 0.95766E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.92247E+00_JPRB, 0.89392E+00_JPRB, 0.85688E+00_JPRB, 0.82084E+00_JPRB, 0.78771E+00_JPRB, 0.76158E+00_JPRB, 0.74593E+00_JPRB ,&
&  0.73467E+00_JPRB, 0.72738E+00_JPRB, 0.72407E+00_JPRB, 0.72228E+00_JPRB, 0.71913E+00_JPRB, 0.71498E+00_JPRB, 0.70748E+00_JPRB ,&
&  0.69712E+00_JPRB, 0.68410E+00_JPRB, 0.67217E+00_JPRB, 0.66352E+00_JPRB, 0.65700E+00_JPRB, 0.65238E+00_JPRB, 0.64538E+00_JPRB ,&
&  0.63334E+00_JPRB, 0.61969E+00_JPRB, 0.60316E+00_JPRB, 0.58672E+00_JPRB, 0.57558E+00_JPRB, 0.57654E+00_JPRB, 0.63366E+00_JPRB ,&
&  0.83833E+00_JPRB, 0.96968E+00_JPRB, 0.10862E+01_JPRB, 0.11688E+01_JPRB, 0.12863E+01_JPRB, 0.13568E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.14231E+01_JPRB, 0.14568E+01_JPRB, 0.15207E+01_JPRB, 0.15116E+01_JPRB, 0.13946E+01_JPRB, 0.12761E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.10652E+01_JPRB, 0.97917E+00_JPRB, 0.90928E+00_JPRB, 0.87589E+00_JPRB, 0.86458E+00_JPRB, 0.86614E+00_JPRB, 0.86513E+00_JPRB ,&
&  0.86110E+00_JPRB, 0.84636E+00_JPRB, 0.84205E+00_JPRB, 0.86554E+00_JPRB, 0.94271E+00_JPRB, 0.11025E+01_JPRB, 0.13639E+01_JPRB ,&
&  0.14846E+01_JPRB, 0.15294E+01_JPRB, 0.15711E+01_JPRB, 0.16123E+01_JPRB, 0.15877E+01_JPRB, 0.15454E+01_JPRB, 0.15045E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.14093E+01_JPRB, 0.13633E+01_JPRB, 0.13051E+01_JPRB, 0.12544E+01_JPRB, 0.12177E+01_JPRB, 0.11909E+01_JPRB ,&
&  0.11708E+01_JPRB, 0.11618E+01_JPRB, 0.11459E+01_JPRB, 0.11143E+01_JPRB, 0.10904E+01_JPRB, 0.10655E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.97653E+00_JPRB, 0.94294E+00_JPRB, 0.92169E+00_JPRB, 0.91625E+00_JPRB, 0.91719E+00_JPRB, 0.91994E+00_JPRB, 0.91653E+00_JPRB ,&
&  0.91716E+00_JPRB, 0.93028E+00_JPRB, 0.92103E+00_JPRB, 0.91780E+00_JPRB, 0.95680E+00_JPRB, 0.95544E+00_JPRB, 0.95408E+00_JPRB ,&
&  0.94943E+00_JPRB, 0.93908E+00_JPRB, 0.92147E+00_JPRB, 0.89407E+00_JPRB, 0.92468E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.80058E+00_JPRB, 0.79889E+00_JPRB, 0.77952E+00_JPRB, 0.75740E+00_JPRB, 0.73756E+00_JPRB, 0.72274E+00_JPRB, 0.71077E+00_JPRB ,&
&  0.70196E+00_JPRB, 0.69499E+00_JPRB, 0.69058E+00_JPRB, 0.68774E+00_JPRB, 0.68357E+00_JPRB, 0.67727E+00_JPRB, 0.67024E+00_JPRB ,&
&  0.66382E+00_JPRB, 0.65585E+00_JPRB, 0.64399E+00_JPRB, 0.63538E+00_JPRB, 0.63021E+00_JPRB, 0.62335E+00_JPRB, 0.61573E+00_JPRB ,&
&  0.60630E+00_JPRB, 0.58921E+00_JPRB, 0.57198E+00_JPRB, 0.55979E+00_JPRB, 0.54990E+00_JPRB, 0.54768E+00_JPRB, 0.58554E+00_JPRB ,&
&  0.71192E+00_JPRB, 0.72991E+00_JPRB, 0.82554E+00_JPRB, 0.91858E+00_JPRB, 0.10356E+01_JPRB, 0.11011E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.11746E+01_JPRB, 0.12069E+01_JPRB, 0.12511E+01_JPRB, 0.12061E+01_JPRB, 0.11244E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.98719E+00_JPRB, 0.93215E+00_JPRB, 0.88679E+00_JPRB, 0.85568E+00_JPRB, 0.84249E+00_JPRB, 0.83687E+00_JPRB, 0.82570E+00_JPRB ,&
&  0.81501E+00_JPRB, 0.79699E+00_JPRB, 0.78220E+00_JPRB, 0.77937E+00_JPRB, 0.77018E+00_JPRB, 0.80612E+00_JPRB, 0.86749E+00_JPRB ,&
&  0.92629E+00_JPRB, 0.10086E+01_JPRB, 0.10543E+01_JPRB, 0.10850E+01_JPRB, 0.10872E+01_JPRB, 0.10870E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10826E+01_JPRB, 0.10772E+01_JPRB, 0.10663E+01_JPRB, 0.10450E+01_JPRB, 0.10178E+01_JPRB, 0.99243E+00_JPRB, 0.97015E+00_JPRB ,&
&  0.95141E+00_JPRB, 0.93599E+00_JPRB, 0.92474E+00_JPRB, 0.90115E+00_JPRB, 0.86523E+00_JPRB, 0.83403E+00_JPRB, 0.80967E+00_JPRB ,&
&  0.78840E+00_JPRB, 0.77874E+00_JPRB, 0.77682E+00_JPRB, 0.78493E+00_JPRB, 0.79318E+00_JPRB, 0.80130E+00_JPRB, 0.80534E+00_JPRB ,&
&  0.80805E+00_JPRB, 0.81871E+00_JPRB, 0.83734E+00_JPRB, 0.86143E+00_JPRB, 0.87316E+00_JPRB, 0.87992E+00_JPRB, 0.88086E+00_JPRB ,&
&  0.87668E+00_JPRB, 0.86926E+00_JPRB, 0.85083E+00_JPRB, 0.82738E+00_JPRB, 0.79088E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.72556E+00_JPRB, 0.72448E+00_JPRB, 0.71383E+00_JPRB, 0.69854E+00_JPRB, 0.68591E+00_JPRB, 0.67521E+00_JPRB, 0.66694E+00_JPRB ,&
&  0.65459E+00_JPRB, 0.64361E+00_JPRB, 0.63956E+00_JPRB, 0.63932E+00_JPRB, 0.63826E+00_JPRB, 0.63349E+00_JPRB, 0.62706E+00_JPRB ,&
&  0.61949E+00_JPRB, 0.61093E+00_JPRB, 0.60083E+00_JPRB, 0.59067E+00_JPRB, 0.58230E+00_JPRB, 0.57247E+00_JPRB, 0.56397E+00_JPRB ,&
&  0.55558E+00_JPRB, 0.54753E+00_JPRB, 0.53574E+00_JPRB, 0.52209E+00_JPRB, 0.51533E+00_JPRB, 0.51005E+00_JPRB, 0.54248E+00_JPRB ,&
&  0.64549E+00_JPRB, 0.57459E+00_JPRB, 0.63708E+00_JPRB, 0.73561E+00_JPRB, 0.84471E+00_JPRB, 0.94728E+00_JPRB, 0.97840E+00_JPRB ,&
&  0.98947E+00_JPRB, 0.10054E+01_JPRB, 0.10172E+01_JPRB, 0.10372E+01_JPRB, 0.10444E+01_JPRB, 0.10018E+01_JPRB, 0.94198E+00_JPRB ,&
&  0.89345E+00_JPRB, 0.85196E+00_JPRB, 0.82560E+00_JPRB, 0.80108E+00_JPRB, 0.78768E+00_JPRB, 0.78022E+00_JPRB, 0.76399E+00_JPRB ,&
&  0.74575E+00_JPRB, 0.73748E+00_JPRB, 0.73369E+00_JPRB, 0.73703E+00_JPRB, 0.74093E+00_JPRB, 0.74447E+00_JPRB, 0.73282E+00_JPRB ,&
&  0.72154E+00_JPRB, 0.74333E+00_JPRB, 0.78490E+00_JPRB, 0.80957E+00_JPRB, 0.82560E+00_JPRB, 0.82682E+00_JPRB, 0.82059E+00_JPRB ,&
&  0.81824E+00_JPRB, 0.81749E+00_JPRB, 0.82072E+00_JPRB, 0.81639E+00_JPRB, 0.80722E+00_JPRB, 0.79033E+00_JPRB, 0.76969E+00_JPRB ,&
&  0.74989E+00_JPRB, 0.73248E+00_JPRB, 0.71843E+00_JPRB, 0.70542E+00_JPRB, 0.69013E+00_JPRB, 0.67084E+00_JPRB, 0.65939E+00_JPRB ,&
&  0.65315E+00_JPRB, 0.65236E+00_JPRB, 0.65604E+00_JPRB, 0.66370E+00_JPRB, 0.67516E+00_JPRB, 0.68581E+00_JPRB, 0.69897E+00_JPRB ,&
&  0.71005E+00_JPRB, 0.72126E+00_JPRB, 0.73181E+00_JPRB, 0.74393E+00_JPRB, 0.78066E+00_JPRB, 0.79124E+00_JPRB, 0.80055E+00_JPRB ,&
&  0.80156E+00_JPRB, 0.79927E+00_JPRB, 0.79076E+00_JPRB, 0.70182E+00_JPRB, 0.72432E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.66081E+00_JPRB, 0.65769E+00_JPRB, 0.64950E+00_JPRB, 0.64036E+00_JPRB, 0.63297E+00_JPRB, 0.62564E+00_JPRB, 0.61979E+00_JPRB ,&
&  0.61130E+00_JPRB, 0.60058E+00_JPRB, 0.59273E+00_JPRB, 0.58959E+00_JPRB, 0.58829E+00_JPRB, 0.58499E+00_JPRB, 0.57892E+00_JPRB ,&
&  0.57102E+00_JPRB, 0.56223E+00_JPRB, 0.55358E+00_JPRB, 0.54302E+00_JPRB, 0.53320E+00_JPRB, 0.52393E+00_JPRB, 0.51575E+00_JPRB ,&
&  0.50753E+00_JPRB, 0.49876E+00_JPRB, 0.49017E+00_JPRB, 0.48377E+00_JPRB, 0.47901E+00_JPRB, 0.47764E+00_JPRB, 0.50461E+00_JPRB ,&
&  0.58576E+00_JPRB, 0.50639E+00_JPRB, 0.54937E+00_JPRB, 0.60890E+00_JPRB, 0.70403E+00_JPRB, 0.81967E+00_JPRB, 0.87093E+00_JPRB ,&
&  0.88902E+00_JPRB, 0.89793E+00_JPRB, 0.89641E+00_JPRB, 0.89388E+00_JPRB, 0.90308E+00_JPRB, 0.89706E+00_JPRB, 0.86527E+00_JPRB ,&
&  0.82770E+00_JPRB, 0.79303E+00_JPRB, 0.76931E+00_JPRB, 0.75189E+00_JPRB, 0.73440E+00_JPRB, 0.71806E+00_JPRB, 0.69724E+00_JPRB ,&
&  0.67320E+00_JPRB, 0.66171E+00_JPRB, 0.65739E+00_JPRB, 0.65357E+00_JPRB, 0.65209E+00_JPRB, 0.64587E+00_JPRB, 0.62514E+00_JPRB ,&
&  0.60487E+00_JPRB, 0.59121E+00_JPRB, 0.58760E+00_JPRB, 0.59530E+00_JPRB, 0.60586E+00_JPRB, 0.61341E+00_JPRB, 0.61323E+00_JPRB ,&
&  0.60912E+00_JPRB, 0.60124E+00_JPRB, 0.59778E+00_JPRB, 0.59668E+00_JPRB, 0.59525E+00_JPRB, 0.59383E+00_JPRB, 0.58897E+00_JPRB ,&
&  0.58215E+00_JPRB, 0.57853E+00_JPRB, 0.57880E+00_JPRB, 0.57980E+00_JPRB, 0.57874E+00_JPRB, 0.57538E+00_JPRB, 0.57138E+00_JPRB ,&
&  0.56599E+00_JPRB, 0.56215E+00_JPRB, 0.56279E+00_JPRB, 0.56883E+00_JPRB, 0.57804E+00_JPRB, 0.58829E+00_JPRB, 0.59959E+00_JPRB ,&
&  0.61192E+00_JPRB, 0.62453E+00_JPRB, 0.63661E+00_JPRB, 0.66016E+00_JPRB, 0.67310E+00_JPRB, 0.68855E+00_JPRB, 0.70375E+00_JPRB ,&
&  0.70722E+00_JPRB, 0.70357E+00_JPRB, 0.60896E+00_JPRB, 0.64774E+00_JPRB, 0.65596E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.61618E+00_JPRB, 0.60501E+00_JPRB, 0.59207E+00_JPRB, 0.58347E+00_JPRB, 0.57993E+00_JPRB, 0.57616E+00_JPRB, 0.57255E+00_JPRB ,&
&  0.56674E+00_JPRB, 0.56020E+00_JPRB, 0.55361E+00_JPRB, 0.54770E+00_JPRB, 0.54285E+00_JPRB, 0.53952E+00_JPRB, 0.53505E+00_JPRB ,&
&  0.52822E+00_JPRB, 0.52009E+00_JPRB, 0.51131E+00_JPRB, 0.50124E+00_JPRB, 0.48831E+00_JPRB, 0.47771E+00_JPRB, 0.46926E+00_JPRB ,&
&  0.46268E+00_JPRB, 0.45746E+00_JPRB, 0.45495E+00_JPRB, 0.45217E+00_JPRB, 0.44929E+00_JPRB, 0.44930E+00_JPRB, 0.46860E+00_JPRB ,&
&  0.49081E+00_JPRB, 0.46417E+00_JPRB, 0.49453E+00_JPRB, 0.53749E+00_JPRB, 0.60836E+00_JPRB, 0.71946E+00_JPRB, 0.79008E+00_JPRB ,&
&  0.82499E+00_JPRB, 0.83916E+00_JPRB, 0.84096E+00_JPRB, 0.82864E+00_JPRB, 0.81761E+00_JPRB, 0.81992E+00_JPRB, 0.81441E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.75879E+00_JPRB, 0.72967E+00_JPRB, 0.70777E+00_JPRB, 0.68190E+00_JPRB, 0.65846E+00_JPRB, 0.63434E+00_JPRB ,&
&  0.61323E+00_JPRB, 0.60019E+00_JPRB, 0.59350E+00_JPRB, 0.58840E+00_JPRB, 0.58267E+00_JPRB, 0.57048E+00_JPRB, 0.55341E+00_JPRB ,&
&  0.53895E+00_JPRB, 0.52588E+00_JPRB, 0.51381E+00_JPRB, 0.50401E+00_JPRB, 0.49751E+00_JPRB, 0.49352E+00_JPRB, 0.49067E+00_JPRB ,&
&  0.49065E+00_JPRB, 0.49246E+00_JPRB, 0.49608E+00_JPRB, 0.49964E+00_JPRB, 0.50124E+00_JPRB, 0.50037E+00_JPRB, 0.49920E+00_JPRB ,&
&  0.49797E+00_JPRB, 0.49403E+00_JPRB, 0.49351E+00_JPRB, 0.49893E+00_JPRB, 0.50609E+00_JPRB, 0.51044E+00_JPRB, 0.51174E+00_JPRB ,&
&  0.50963E+00_JPRB, 0.50733E+00_JPRB, 0.50694E+00_JPRB, 0.50930E+00_JPRB, 0.51327E+00_JPRB, 0.51954E+00_JPRB, 0.52720E+00_JPRB ,&
&  0.53500E+00_JPRB, 0.54503E+00_JPRB, 0.55827E+00_JPRB, 0.57727E+00_JPRB, 0.59202E+00_JPRB, 0.60420E+00_JPRB, 0.61533E+00_JPRB ,&
&  0.61886E+00_JPRB, 0.62041E+00_JPRB, 0.62501E+00_JPRB, 0.62394E+00_JPRB, 0.62186E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.54825E+00_JPRB, 0.54186E+00_JPRB, 0.53398E+00_JPRB, 0.52709E+00_JPRB, 0.52333E+00_JPRB, 0.52039E+00_JPRB, 0.51582E+00_JPRB ,&
&  0.51208E+00_JPRB, 0.50885E+00_JPRB, 0.50547E+00_JPRB, 0.50251E+00_JPRB, 0.49985E+00_JPRB, 0.49772E+00_JPRB, 0.49397E+00_JPRB ,&
&  0.48752E+00_JPRB, 0.47922E+00_JPRB, 0.46975E+00_JPRB, 0.46150E+00_JPRB, 0.45150E+00_JPRB, 0.44110E+00_JPRB, 0.43314E+00_JPRB ,&
&  0.42658E+00_JPRB, 0.42202E+00_JPRB, 0.42002E+00_JPRB, 0.41755E+00_JPRB, 0.41796E+00_JPRB, 0.42080E+00_JPRB, 0.43261E+00_JPRB ,&
&  0.46359E+00_JPRB, 0.44863E+00_JPRB, 0.47877E+00_JPRB, 0.50724E+00_JPRB, 0.56501E+00_JPRB, 0.65337E+00_JPRB, 0.71215E+00_JPRB ,&
&  0.74703E+00_JPRB, 0.77492E+00_JPRB, 0.79397E+00_JPRB, 0.79416E+00_JPRB, 0.77926E+00_JPRB, 0.77266E+00_JPRB, 0.76978E+00_JPRB ,&
&  0.75639E+00_JPRB, 0.73114E+00_JPRB, 0.69593E+00_JPRB, 0.65821E+00_JPRB, 0.62486E+00_JPRB, 0.59970E+00_JPRB, 0.58091E+00_JPRB ,&
&  0.56675E+00_JPRB, 0.55529E+00_JPRB, 0.54636E+00_JPRB, 0.53312E+00_JPRB, 0.51771E+00_JPRB, 0.50197E+00_JPRB, 0.48630E+00_JPRB ,&
&  0.47055E+00_JPRB, 0.45550E+00_JPRB, 0.44274E+00_JPRB, 0.43248E+00_JPRB, 0.42629E+00_JPRB, 0.42269E+00_JPRB, 0.42063E+00_JPRB ,&
&  0.42011E+00_JPRB, 0.42184E+00_JPRB, 0.42514E+00_JPRB, 0.42927E+00_JPRB, 0.43276E+00_JPRB, 0.43368E+00_JPRB, 0.43133E+00_JPRB ,&
&  0.42991E+00_JPRB, 0.42906E+00_JPRB, 0.42977E+00_JPRB, 0.43306E+00_JPRB, 0.43552E+00_JPRB, 0.43843E+00_JPRB, 0.44292E+00_JPRB ,&
&  0.44654E+00_JPRB, 0.44810E+00_JPRB, 0.44892E+00_JPRB, 0.45008E+00_JPRB, 0.45175E+00_JPRB, 0.45494E+00_JPRB, 0.46008E+00_JPRB ,&
&  0.46625E+00_JPRB, 0.47329E+00_JPRB, 0.48371E+00_JPRB, 0.49735E+00_JPRB, 0.51021E+00_JPRB, 0.52124E+00_JPRB, 0.53078E+00_JPRB ,&
&  0.53539E+00_JPRB, 0.53919E+00_JPRB, 0.54547E+00_JPRB, 0.54872E+00_JPRB, 0.55090E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.49255E+00_JPRB, 0.48960E+00_JPRB, 0.48586E+00_JPRB, 0.48232E+00_JPRB, 0.47938E+00_JPRB, 0.47459E+00_JPRB, 0.46920E+00_JPRB ,&
&  0.46605E+00_JPRB, 0.46349E+00_JPRB, 0.46190E+00_JPRB, 0.45898E+00_JPRB, 0.45757E+00_JPRB, 0.45679E+00_JPRB, 0.45252E+00_JPRB ,&
&  0.44714E+00_JPRB, 0.43915E+00_JPRB, 0.43116E+00_JPRB, 0.42371E+00_JPRB, 0.41649E+00_JPRB, 0.40961E+00_JPRB, 0.40202E+00_JPRB ,&
&  0.39489E+00_JPRB, 0.38949E+00_JPRB, 0.38624E+00_JPRB, 0.38355E+00_JPRB, 0.38467E+00_JPRB, 0.39070E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.41481E+00_JPRB, 0.43648E+00_JPRB, 0.45729E+00_JPRB, 0.47925E+00_JPRB, 0.52267E+00_JPRB, 0.58759E+00_JPRB, 0.63590E+00_JPRB ,&
&  0.65944E+00_JPRB, 0.68590E+00_JPRB, 0.72081E+00_JPRB, 0.74439E+00_JPRB, 0.74660E+00_JPRB, 0.73526E+00_JPRB, 0.72653E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.69079E+00_JPRB, 0.66404E+00_JPRB, 0.63345E+00_JPRB, 0.60309E+00_JPRB, 0.57831E+00_JPRB, 0.55930E+00_JPRB ,&
&  0.54326E+00_JPRB, 0.52898E+00_JPRB, 0.51519E+00_JPRB, 0.50016E+00_JPRB, 0.48366E+00_JPRB, 0.46520E+00_JPRB, 0.44543E+00_JPRB ,&
&  0.42664E+00_JPRB, 0.41120E+00_JPRB, 0.39969E+00_JPRB, 0.39301E+00_JPRB, 0.38870E+00_JPRB, 0.38484E+00_JPRB, 0.38193E+00_JPRB ,&
&  0.38113E+00_JPRB, 0.38197E+00_JPRB, 0.38414E+00_JPRB, 0.38725E+00_JPRB, 0.39032E+00_JPRB, 0.39170E+00_JPRB, 0.38924E+00_JPRB ,&
&  0.38634E+00_JPRB, 0.38624E+00_JPRB, 0.38711E+00_JPRB, 0.38479E+00_JPRB, 0.38188E+00_JPRB, 0.38297E+00_JPRB, 0.38664E+00_JPRB ,&
&  0.39253E+00_JPRB, 0.39774E+00_JPRB, 0.40158E+00_JPRB, 0.40450E+00_JPRB, 0.40650E+00_JPRB, 0.41001E+00_JPRB, 0.41447E+00_JPRB ,&
&  0.42021E+00_JPRB, 0.42742E+00_JPRB, 0.43744E+00_JPRB, 0.44984E+00_JPRB, 0.46105E+00_JPRB, 0.47046E+00_JPRB, 0.47618E+00_JPRB ,&
&  0.47982E+00_JPRB, 0.48239E+00_JPRB, 0.48492E+00_JPRB, 0.48849E+00_JPRB, 0.49181E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.43914E+00_JPRB, 0.43914E+00_JPRB, 0.43965E+00_JPRB, 0.43923E+00_JPRB, 0.43538E+00_JPRB, 0.43146E+00_JPRB, 0.42722E+00_JPRB ,&
&  0.42512E+00_JPRB, 0.42377E+00_JPRB, 0.42351E+00_JPRB, 0.42064E+00_JPRB, 0.41859E+00_JPRB, 0.41756E+00_JPRB, 0.41374E+00_JPRB ,&
&  0.40947E+00_JPRB, 0.40041E+00_JPRB, 0.39087E+00_JPRB, 0.38398E+00_JPRB, 0.37963E+00_JPRB, 0.37634E+00_JPRB, 0.36997E+00_JPRB ,&
&  0.36497E+00_JPRB, 0.36093E+00_JPRB, 0.35872E+00_JPRB, 0.35849E+00_JPRB, 0.36074E+00_JPRB, 0.36705E+00_JPRB, 0.37716E+00_JPRB ,&
&  0.38570E+00_JPRB, 0.40537E+00_JPRB, 0.41009E+00_JPRB, 0.43290E+00_JPRB, 0.46060E+00_JPRB, 0.50289E+00_JPRB, 0.54577E+00_JPRB ,&
&  0.56954E+00_JPRB, 0.58376E+00_JPRB, 0.60802E+00_JPRB, 0.63808E+00_JPRB, 0.65301E+00_JPRB, 0.65620E+00_JPRB, 0.65055E+00_JPRB ,&
&  0.64166E+00_JPRB, 0.63209E+00_JPRB, 0.62115E+00_JPRB, 0.60617E+00_JPRB, 0.58539E+00_JPRB, 0.56268E+00_JPRB, 0.54124E+00_JPRB ,&
&  0.52195E+00_JPRB, 0.50335E+00_JPRB, 0.48475E+00_JPRB, 0.46437E+00_JPRB, 0.44320E+00_JPRB, 0.42276E+00_JPRB, 0.40304E+00_JPRB ,&
&  0.38723E+00_JPRB, 0.37537E+00_JPRB, 0.36671E+00_JPRB, 0.36020E+00_JPRB, 0.35502E+00_JPRB, 0.35016E+00_JPRB, 0.34733E+00_JPRB ,&
&  0.34645E+00_JPRB, 0.34763E+00_JPRB, 0.35036E+00_JPRB, 0.35286E+00_JPRB, 0.35590E+00_JPRB, 0.35638E+00_JPRB, 0.35245E+00_JPRB ,&
&  0.34879E+00_JPRB, 0.34816E+00_JPRB, 0.34384E+00_JPRB, 0.34127E+00_JPRB, 0.34075E+00_JPRB, 0.34156E+00_JPRB, 0.34420E+00_JPRB ,&
&  0.34914E+00_JPRB, 0.35561E+00_JPRB, 0.36157E+00_JPRB, 0.36685E+00_JPRB, 0.37163E+00_JPRB, 0.37625E+00_JPRB, 0.38139E+00_JPRB ,&
&  0.38672E+00_JPRB, 0.39184E+00_JPRB, 0.39955E+00_JPRB, 0.40817E+00_JPRB, 0.41559E+00_JPRB, 0.42366E+00_JPRB, 0.42862E+00_JPRB ,&
&  0.43064E+00_JPRB, 0.43135E+00_JPRB, 0.43344E+00_JPRB, 0.43672E+00_JPRB, 0.43886E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.39421E+00_JPRB, 0.39591E+00_JPRB, 0.39795E+00_JPRB, 0.39795E+00_JPRB, 0.39681E+00_JPRB, 0.39625E+00_JPRB, 0.39471E+00_JPRB ,&
&  0.39188E+00_JPRB, 0.39012E+00_JPRB, 0.39012E+00_JPRB, 0.38851E+00_JPRB, 0.38547E+00_JPRB, 0.38180E+00_JPRB, 0.37777E+00_JPRB ,&
&  0.37432E+00_JPRB, 0.37065E+00_JPRB, 0.36440E+00_JPRB, 0.35648E+00_JPRB, 0.34974E+00_JPRB, 0.34528E+00_JPRB, 0.34266E+00_JPRB ,&
&  0.34127E+00_JPRB, 0.34079E+00_JPRB, 0.34011E+00_JPRB, 0.34059E+00_JPRB, 0.34282E+00_JPRB, 0.34808E+00_JPRB, 0.35583E+00_JPRB ,&
&  0.36336E+00_JPRB, 0.37234E+00_JPRB, 0.38167E+00_JPRB, 0.39253E+00_JPRB, 0.40647E+00_JPRB, 0.42673E+00_JPRB, 0.45081E+00_JPRB ,&
&  0.47699E+00_JPRB, 0.48661E+00_JPRB, 0.48936E+00_JPRB, 0.49884E+00_JPRB, 0.51645E+00_JPRB, 0.53084E+00_JPRB, 0.54087E+00_JPRB ,&
&  0.54667E+00_JPRB, 0.54709E+00_JPRB, 0.54452E+00_JPRB, 0.53646E+00_JPRB, 0.52364E+00_JPRB, 0.50708E+00_JPRB, 0.48898E+00_JPRB ,&
&  0.47128E+00_JPRB, 0.45306E+00_JPRB, 0.43380E+00_JPRB, 0.41483E+00_JPRB, 0.39737E+00_JPRB, 0.38132E+00_JPRB, 0.36652E+00_JPRB ,&
&  0.35395E+00_JPRB, 0.34428E+00_JPRB, 0.33672E+00_JPRB, 0.33118E+00_JPRB, 0.32653E+00_JPRB, 0.32282E+00_JPRB, 0.32111E+00_JPRB ,&
&  0.32086E+00_JPRB, 0.32179E+00_JPRB, 0.32225E+00_JPRB, 0.32382E+00_JPRB, 0.32666E+00_JPRB, 0.32745E+00_JPRB, 0.32536E+00_JPRB ,&
&  0.32053E+00_JPRB, 0.31613E+00_JPRB, 0.31180E+00_JPRB, 0.31132E+00_JPRB, 0.31157E+00_JPRB, 0.31350E+00_JPRB, 0.31626E+00_JPRB ,&
&  0.32106E+00_JPRB, 0.32718E+00_JPRB, 0.33290E+00_JPRB, 0.33740E+00_JPRB, 0.34246E+00_JPRB, 0.34699E+00_JPRB, 0.35127E+00_JPRB ,&
&  0.35573E+00_JPRB, 0.36024E+00_JPRB, 0.36604E+00_JPRB, 0.37189E+00_JPRB, 0.37727E+00_JPRB, 0.38155E+00_JPRB, 0.38461E+00_JPRB ,&
&  0.38665E+00_JPRB, 0.38731E+00_JPRB, 0.38779E+00_JPRB, 0.38965E+00_JPRB, 0.39225E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.35584E+00_JPRB, 0.35813E+00_JPRB, 0.36093E+00_JPRB, 0.36288E+00_JPRB, 0.36330E+00_JPRB, 0.36251E+00_JPRB, 0.36151E+00_JPRB ,&
&  0.36086E+00_JPRB, 0.35997E+00_JPRB, 0.35789E+00_JPRB, 0.35493E+00_JPRB, 0.35209E+00_JPRB, 0.34915E+00_JPRB, 0.34719E+00_JPRB ,&
&  0.34537E+00_JPRB, 0.34315E+00_JPRB, 0.33884E+00_JPRB, 0.33234E+00_JPRB, 0.32560E+00_JPRB, 0.32182E+00_JPRB, 0.32023E+00_JPRB ,&
&  0.32043E+00_JPRB, 0.32126E+00_JPRB, 0.32387E+00_JPRB, 0.32556E+00_JPRB, 0.32790E+00_JPRB, 0.33200E+00_JPRB, 0.33789E+00_JPRB ,&
&  0.34326E+00_JPRB, 0.34855E+00_JPRB, 0.35446E+00_JPRB, 0.36006E+00_JPRB, 0.36578E+00_JPRB, 0.37401E+00_JPRB, 0.38484E+00_JPRB ,&
&  0.40077E+00_JPRB, 0.41385E+00_JPRB, 0.41435E+00_JPRB, 0.41441E+00_JPRB, 0.42038E+00_JPRB, 0.42937E+00_JPRB, 0.43743E+00_JPRB ,&
&  0.44402E+00_JPRB, 0.44560E+00_JPRB, 0.44465E+00_JPRB, 0.44062E+00_JPRB, 0.43496E+00_JPRB, 0.42765E+00_JPRB, 0.41758E+00_JPRB ,&
&  0.40541E+00_JPRB, 0.39200E+00_JPRB, 0.37878E+00_JPRB, 0.36568E+00_JPRB, 0.35282E+00_JPRB, 0.34118E+00_JPRB, 0.33117E+00_JPRB ,&
&  0.32250E+00_JPRB, 0.31573E+00_JPRB, 0.30943E+00_JPRB, 0.30514E+00_JPRB, 0.30206E+00_JPRB, 0.29887E+00_JPRB, 0.29777E+00_JPRB ,&
&  0.29623E+00_JPRB, 0.29509E+00_JPRB, 0.29469E+00_JPRB, 0.29596E+00_JPRB, 0.29890E+00_JPRB, 0.30067E+00_JPRB, 0.29905E+00_JPRB ,&
&  0.29546E+00_JPRB, 0.29334E+00_JPRB, 0.29206E+00_JPRB, 0.29127E+00_JPRB, 0.28903E+00_JPRB, 0.28920E+00_JPRB, 0.29198E+00_JPRB ,&
&  0.29761E+00_JPRB, 0.30338E+00_JPRB, 0.30794E+00_JPRB, 0.31084E+00_JPRB, 0.31411E+00_JPRB, 0.31719E+00_JPRB, 0.31987E+00_JPRB ,&
&  0.32247E+00_JPRB, 0.32427E+00_JPRB, 0.32552E+00_JPRB, 0.32712E+00_JPRB, 0.32967E+00_JPRB, 0.33212E+00_JPRB, 0.33450E+00_JPRB ,&
&  0.33729E+00_JPRB, 0.34183E+00_JPRB, 0.34644E+00_JPRB, 0.34986E+00_JPRB, 0.35361E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.32928E+00_JPRB, 0.33198E+00_JPRB, 0.33406E+00_JPRB, 0.33349E+00_JPRB, 0.33085E+00_JPRB, 0.32931E+00_JPRB, 0.32926E+00_JPRB ,&
&  0.32939E+00_JPRB, 0.32869E+00_JPRB, 0.32661E+00_JPRB, 0.32610E+00_JPRB, 0.32597E+00_JPRB, 0.32511E+00_JPRB, 0.32476E+00_JPRB ,&
&  0.32366E+00_JPRB, 0.32045E+00_JPRB, 0.31557E+00_JPRB, 0.30961E+00_JPRB, 0.30539E+00_JPRB, 0.30357E+00_JPRB, 0.30393E+00_JPRB ,&
&  0.30549E+00_JPRB, 0.30657E+00_JPRB, 0.30930E+00_JPRB, 0.31123E+00_JPRB, 0.31339E+00_JPRB, 0.31654E+00_JPRB, 0.32006E+00_JPRB ,&
&  0.32240E+00_JPRB, 0.32501E+00_JPRB, 0.32801E+00_JPRB, 0.33111E+00_JPRB, 0.33526E+00_JPRB, 0.34189E+00_JPRB, 0.34826E+00_JPRB ,&
&  0.35855E+00_JPRB, 0.36746E+00_JPRB, 0.36444E+00_JPRB, 0.35941E+00_JPRB, 0.35905E+00_JPRB, 0.36258E+00_JPRB, 0.36760E+00_JPRB ,&
&  0.37330E+00_JPRB, 0.37874E+00_JPRB, 0.38301E+00_JPRB, 0.38427E+00_JPRB, 0.38211E+00_JPRB, 0.37693E+00_JPRB, 0.36895E+00_JPRB ,&
&  0.35951E+00_JPRB, 0.34946E+00_JPRB, 0.33922E+00_JPRB, 0.32862E+00_JPRB, 0.31936E+00_JPRB, 0.31096E+00_JPRB, 0.30365E+00_JPRB ,&
&  0.29701E+00_JPRB, 0.29084E+00_JPRB, 0.28613E+00_JPRB, 0.28283E+00_JPRB, 0.28082E+00_JPRB, 0.27863E+00_JPRB, 0.27806E+00_JPRB ,&
&  0.27680E+00_JPRB, 0.27605E+00_JPRB, 0.27556E+00_JPRB, 0.27587E+00_JPRB, 0.27709E+00_JPRB, 0.27805E+00_JPRB, 0.27737E+00_JPRB ,&
&  0.27705E+00_JPRB, 0.27871E+00_JPRB, 0.27856E+00_JPRB, 0.27582E+00_JPRB, 0.27238E+00_JPRB, 0.27176E+00_JPRB, 0.27372E+00_JPRB ,&
&  0.27732E+00_JPRB, 0.28126E+00_JPRB, 0.28470E+00_JPRB, 0.28755E+00_JPRB, 0.28994E+00_JPRB, 0.29289E+00_JPRB, 0.29556E+00_JPRB ,&
&  0.29789E+00_JPRB, 0.29939E+00_JPRB, 0.30148E+00_JPRB, 0.30370E+00_JPRB, 0.30589E+00_JPRB, 0.30736E+00_JPRB, 0.30714E+00_JPRB ,&
&  0.30654E+00_JPRB, 0.30841E+00_JPRB, 0.31385E+00_JPRB, 0.32147E+00_JPRB, 0.32622E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.30575E+00_JPRB, 0.30787E+00_JPRB, 0.30870E+00_JPRB, 0.30775E+00_JPRB, 0.30633E+00_JPRB, 0.30651E+00_JPRB, 0.30686E+00_JPRB ,&
&  0.30758E+00_JPRB, 0.30733E+00_JPRB, 0.30530E+00_JPRB, 0.30342E+00_JPRB, 0.30481E+00_JPRB, 0.30451E+00_JPRB, 0.30421E+00_JPRB ,&
&  0.30273E+00_JPRB, 0.30074E+00_JPRB, 0.29784E+00_JPRB, 0.29460E+00_JPRB, 0.29092E+00_JPRB, 0.28934E+00_JPRB, 0.29067E+00_JPRB ,&
&  0.29306E+00_JPRB, 0.29400E+00_JPRB, 0.29471E+00_JPRB, 0.29527E+00_JPRB, 0.29675E+00_JPRB, 0.29876E+00_JPRB, 0.30017E+00_JPRB ,&
&  0.30109E+00_JPRB, 0.30385E+00_JPRB, 0.30556E+00_JPRB, 0.30550E+00_JPRB, 0.30974E+00_JPRB, 0.31489E+00_JPRB, 0.32062E+00_JPRB ,&
&  0.32697E+00_JPRB, 0.33133E+00_JPRB, 0.33167E+00_JPRB, 0.32999E+00_JPRB, 0.32867E+00_JPRB, 0.32992E+00_JPRB, 0.33113E+00_JPRB ,&
&  0.33307E+00_JPRB, 0.33809E+00_JPRB, 0.34396E+00_JPRB, 0.34736E+00_JPRB, 0.34757E+00_JPRB, 0.34368E+00_JPRB, 0.33585E+00_JPRB ,&
&  0.32467E+00_JPRB, 0.31230E+00_JPRB, 0.30173E+00_JPRB, 0.29316E+00_JPRB, 0.28669E+00_JPRB, 0.28187E+00_JPRB, 0.27799E+00_JPRB ,&
&  0.27373E+00_JPRB, 0.27047E+00_JPRB, 0.26997E+00_JPRB, 0.27020E+00_JPRB, 0.26935E+00_JPRB, 0.26615E+00_JPRB, 0.26395E+00_JPRB ,&
&  0.26206E+00_JPRB, 0.26244E+00_JPRB, 0.26374E+00_JPRB, 0.26424E+00_JPRB, 0.26447E+00_JPRB, 0.26455E+00_JPRB, 0.26455E+00_JPRB ,&
&  0.26570E+00_JPRB, 0.26518E+00_JPRB, 0.26359E+00_JPRB, 0.26118E+00_JPRB, 0.25906E+00_JPRB, 0.25881E+00_JPRB, 0.25970E+00_JPRB ,&
&  0.26110E+00_JPRB, 0.26347E+00_JPRB, 0.26625E+00_JPRB, 0.26900E+00_JPRB, 0.27114E+00_JPRB, 0.27224E+00_JPRB, 0.27288E+00_JPRB ,&
&  0.27343E+00_JPRB, 0.27389E+00_JPRB, 0.27604E+00_JPRB, 0.28086E+00_JPRB, 0.28514E+00_JPRB, 0.28814E+00_JPRB, 0.28941E+00_JPRB ,&
&  0.28848E+00_JPRB, 0.28881E+00_JPRB, 0.29193E+00_JPRB, 0.29788E+00_JPRB, 0.30224E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.28716E+00_JPRB, 0.28918E+00_JPRB, 0.28938E+00_JPRB, 0.28843E+00_JPRB, 0.28738E+00_JPRB, 0.28797E+00_JPRB, 0.28909E+00_JPRB ,&
&  0.28967E+00_JPRB, 0.28972E+00_JPRB, 0.28834E+00_JPRB, 0.28569E+00_JPRB, 0.28348E+00_JPRB, 0.28331E+00_JPRB, 0.28181E+00_JPRB ,&
&  0.28089E+00_JPRB, 0.28211E+00_JPRB, 0.28192E+00_JPRB, 0.28140E+00_JPRB, 0.28010E+00_JPRB, 0.27986E+00_JPRB, 0.28140E+00_JPRB ,&
&  0.28392E+00_JPRB, 0.28437E+00_JPRB, 0.28473E+00_JPRB, 0.28520E+00_JPRB, 0.28625E+00_JPRB, 0.28776E+00_JPRB, 0.28831E+00_JPRB ,&
&  0.28824E+00_JPRB, 0.28611E+00_JPRB, 0.28491E+00_JPRB, 0.28707E+00_JPRB, 0.29136E+00_JPRB, 0.29738E+00_JPRB, 0.30186E+00_JPRB ,&
&  0.30565E+00_JPRB, 0.30810E+00_JPRB, 0.30899E+00_JPRB, 0.30859E+00_JPRB, 0.30861E+00_JPRB, 0.31102E+00_JPRB, 0.31326E+00_JPRB ,&
&  0.31450E+00_JPRB, 0.31538E+00_JPRB, 0.31501E+00_JPRB, 0.31486E+00_JPRB, 0.31225E+00_JPRB, 0.30732E+00_JPRB, 0.29836E+00_JPRB ,&
&  0.28840E+00_JPRB, 0.27996E+00_JPRB, 0.27353E+00_JPRB, 0.26855E+00_JPRB, 0.26535E+00_JPRB, 0.26527E+00_JPRB, 0.26510E+00_JPRB ,&
&  0.26431E+00_JPRB, 0.26352E+00_JPRB, 0.26468E+00_JPRB, 0.26458E+00_JPRB, 0.25786E+00_JPRB, 0.25348E+00_JPRB, 0.24805E+00_JPRB ,&
&  0.24647E+00_JPRB, 0.24603E+00_JPRB, 0.24523E+00_JPRB, 0.24889E+00_JPRB, 0.25402E+00_JPRB, 0.25517E+00_JPRB, 0.25343E+00_JPRB ,&
&  0.24616E+00_JPRB, 0.24193E+00_JPRB, 0.23890E+00_JPRB, 0.23586E+00_JPRB, 0.23431E+00_JPRB, 0.23510E+00_JPRB, 0.23622E+00_JPRB ,&
&  0.23906E+00_JPRB, 0.24048E+00_JPRB, 0.24347E+00_JPRB, 0.24530E+00_JPRB, 0.24538E+00_JPRB, 0.24580E+00_JPRB, 0.24521E+00_JPRB ,&
&  0.24770E+00_JPRB, 0.25366E+00_JPRB, 0.26191E+00_JPRB, 0.26351E+00_JPRB, 0.26915E+00_JPRB, 0.27309E+00_JPRB, 0.27452E+00_JPRB ,&
&  0.27515E+00_JPRB, 0.27498E+00_JPRB, 0.27422E+00_JPRB, 0.27787E+00_JPRB, 0.28359E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.27208E+00_JPRB, 0.27333E+00_JPRB, 0.27358E+00_JPRB, 0.27315E+00_JPRB, 0.27253E+00_JPRB, 0.27260E+00_JPRB, 0.27284E+00_JPRB ,&
&  0.27254E+00_JPRB, 0.27154E+00_JPRB, 0.27008E+00_JPRB, 0.26752E+00_JPRB, 0.26431E+00_JPRB, 0.26237E+00_JPRB, 0.26347E+00_JPRB ,&
&  0.26553E+00_JPRB, 0.26737E+00_JPRB, 0.26940E+00_JPRB, 0.27070E+00_JPRB, 0.27151E+00_JPRB, 0.27273E+00_JPRB, 0.27443E+00_JPRB ,&
&  0.27564E+00_JPRB, 0.27602E+00_JPRB, 0.27716E+00_JPRB, 0.27794E+00_JPRB, 0.27992E+00_JPRB, 0.28164E+00_JPRB, 0.28155E+00_JPRB ,&
&  0.27682E+00_JPRB, 0.26874E+00_JPRB, 0.26421E+00_JPRB, 0.26717E+00_JPRB, 0.27399E+00_JPRB, 0.28062E+00_JPRB, 0.28521E+00_JPRB ,&
&  0.28802E+00_JPRB, 0.28896E+00_JPRB, 0.28897E+00_JPRB, 0.29029E+00_JPRB, 0.29272E+00_JPRB, 0.29540E+00_JPRB, 0.29532E+00_JPRB ,&
&  0.29355E+00_JPRB, 0.29219E+00_JPRB, 0.29018E+00_JPRB, 0.28608E+00_JPRB, 0.27995E+00_JPRB, 0.27365E+00_JPRB, 0.26780E+00_JPRB ,&
&  0.26224E+00_JPRB, 0.25695E+00_JPRB, 0.25287E+00_JPRB, 0.25030E+00_JPRB, 0.24976E+00_JPRB, 0.25161E+00_JPRB, 0.25085E+00_JPRB ,&
&  0.25192E+00_JPRB, 0.24976E+00_JPRB, 0.24741E+00_JPRB, 0.24422E+00_JPRB, 0.24143E+00_JPRB, 0.23609E+00_JPRB, 0.22913E+00_JPRB ,&
&  0.22581E+00_JPRB, 0.22937E+00_JPRB, 0.23806E+00_JPRB, 0.24386E+00_JPRB, 0.23760E+00_JPRB, 0.23253E+00_JPRB, 0.23018E+00_JPRB ,&
&  0.22625E+00_JPRB, 0.22132E+00_JPRB, 0.21794E+00_JPRB, 0.21710E+00_JPRB, 0.21728E+00_JPRB, 0.21748E+00_JPRB, 0.21877E+00_JPRB ,&
&  0.22089E+00_JPRB, 0.22270E+00_JPRB, 0.22365E+00_JPRB, 0.22498E+00_JPRB, 0.22557E+00_JPRB, 0.22568E+00_JPRB, 0.22579E+00_JPRB ,&
&  0.22815E+00_JPRB, 0.23290E+00_JPRB, 0.23883E+00_JPRB, 0.24521E+00_JPRB, 0.24982E+00_JPRB, 0.25404E+00_JPRB, 0.25700E+00_JPRB ,&
&  0.25934E+00_JPRB, 0.25876E+00_JPRB, 0.26184E+00_JPRB, 0.26652E+00_JPRB, 0.26982E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.25528E+00_JPRB, 0.25928E+00_JPRB, 0.26044E+00_JPRB, 0.26017E+00_JPRB, 0.25875E+00_JPRB, 0.25737E+00_JPRB, 0.25677E+00_JPRB ,&
&  0.25623E+00_JPRB, 0.25487E+00_JPRB, 0.25387E+00_JPRB, 0.25327E+00_JPRB, 0.25263E+00_JPRB, 0.25167E+00_JPRB, 0.25209E+00_JPRB ,&
&  0.25396E+00_JPRB, 0.25690E+00_JPRB, 0.25951E+00_JPRB, 0.26148E+00_JPRB, 0.26181E+00_JPRB, 0.26267E+00_JPRB, 0.26382E+00_JPRB ,&
&  0.26390E+00_JPRB, 0.26354E+00_JPRB, 0.26665E+00_JPRB, 0.26629E+00_JPRB, 0.26699E+00_JPRB, 0.26651E+00_JPRB, 0.26636E+00_JPRB ,&
&  0.26016E+00_JPRB, 0.25419E+00_JPRB, 0.25274E+00_JPRB, 0.25560E+00_JPRB, 0.26342E+00_JPRB, 0.26467E+00_JPRB, 0.26918E+00_JPRB ,&
&  0.27149E+00_JPRB, 0.27221E+00_JPRB, 0.27419E+00_JPRB, 0.27570E+00_JPRB, 0.27744E+00_JPRB, 0.27706E+00_JPRB, 0.27494E+00_JPRB ,&
&  0.27161E+00_JPRB, 0.27074E+00_JPRB, 0.26745E+00_JPRB, 0.25167E+00_JPRB, 0.24262E+00_JPRB, 0.23739E+00_JPRB, 0.23171E+00_JPRB ,&
&  0.22638E+00_JPRB, 0.22407E+00_JPRB, 0.22392E+00_JPRB, 0.22207E+00_JPRB, 0.22313E+00_JPRB, 0.22614E+00_JPRB, 0.22734E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.22755E+00_JPRB, 0.22722E+00_JPRB, 0.22590E+00_JPRB, 0.22319E+00_JPRB, 0.22151E+00_JPRB, 0.22057E+00_JPRB ,&
&  0.22127E+00_JPRB, 0.22068E+00_JPRB, 0.22534E+00_JPRB, 0.22607E+00_JPRB, 0.21863E+00_JPRB, 0.21869E+00_JPRB, 0.21794E+00_JPRB ,&
&  0.21136E+00_JPRB, 0.20778E+00_JPRB, 0.20567E+00_JPRB, 0.20482E+00_JPRB, 0.20612E+00_JPRB, 0.20792E+00_JPRB, 0.20862E+00_JPRB ,&
&  0.20873E+00_JPRB, 0.21016E+00_JPRB, 0.21081E+00_JPRB, 0.21151E+00_JPRB, 0.21195E+00_JPRB, 0.21306E+00_JPRB, 0.21473E+00_JPRB ,&
&  0.21677E+00_JPRB, 0.22060E+00_JPRB, 0.22423E+00_JPRB, 0.22824E+00_JPRB, 0.23044E+00_JPRB, 0.23205E+00_JPRB, 0.23619E+00_JPRB ,&
&  0.23776E+00_JPRB, 0.24598E+00_JPRB, 0.25751E+00_JPRB, 0.25734E+00_JPRB, 0.24980E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.24638E+00_JPRB, 0.24797E+00_JPRB, 0.24890E+00_JPRB, 0.24773E+00_JPRB, 0.24786E+00_JPRB, 0.24750E+00_JPRB, 0.24738E+00_JPRB ,&
&  0.24709E+00_JPRB, 0.24625E+00_JPRB, 0.24656E+00_JPRB, 0.24397E+00_JPRB, 0.24253E+00_JPRB, 0.23968E+00_JPRB, 0.23960E+00_JPRB ,&
&  0.23843E+00_JPRB, 0.23902E+00_JPRB, 0.23996E+00_JPRB, 0.24208E+00_JPRB, 0.24270E+00_JPRB, 0.24455E+00_JPRB, 0.24344E+00_JPRB ,&
&  0.24399E+00_JPRB, 0.24459E+00_JPRB, 0.24309E+00_JPRB, 0.24265E+00_JPRB, 0.24277E+00_JPRB, 0.24267E+00_JPRB, 0.24282E+00_JPRB ,&
&  0.24548E+00_JPRB, 0.24791E+00_JPRB, 0.25028E+00_JPRB, 0.25126E+00_JPRB, 0.25584E+00_JPRB, 0.25715E+00_JPRB, 0.25991E+00_JPRB ,&
&  0.26094E+00_JPRB, 0.26204E+00_JPRB, 0.26325E+00_JPRB, 0.26240E+00_JPRB, 0.26101E+00_JPRB, 0.25617E+00_JPRB, 0.25464E+00_JPRB ,&
&  0.24462E+00_JPRB, 0.23753E+00_JPRB, 0.23057E+00_JPRB, 0.22398E+00_JPRB, 0.21875E+00_JPRB, 0.21520E+00_JPRB, 0.21238E+00_JPRB ,&
&  0.21012E+00_JPRB, 0.20781E+00_JPRB, 0.20516E+00_JPRB, 0.20550E+00_JPRB, 0.20652E+00_JPRB, 0.20812E+00_JPRB, 0.20982E+00_JPRB ,&
&  0.21180E+00_JPRB, 0.21292E+00_JPRB, 0.21336E+00_JPRB, 0.21424E+00_JPRB, 0.21428E+00_JPRB, 0.21455E+00_JPRB, 0.21472E+00_JPRB ,&
&  0.21635E+00_JPRB, 0.21672E+00_JPRB, 0.21666E+00_JPRB, 0.21887E+00_JPRB, 0.21650E+00_JPRB, 0.21309E+00_JPRB, 0.20969E+00_JPRB ,&
&  0.20590E+00_JPRB, 0.20357E+00_JPRB, 0.20150E+00_JPRB, 0.20070E+00_JPRB, 0.20123E+00_JPRB, 0.20222E+00_JPRB, 0.20339E+00_JPRB ,&
&  0.20349E+00_JPRB, 0.20426E+00_JPRB, 0.20482E+00_JPRB, 0.20593E+00_JPRB, 0.20709E+00_JPRB, 0.20847E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.21242E+00_JPRB, 0.21518E+00_JPRB, 0.21787E+00_JPRB, 0.22058E+00_JPRB, 0.22498E+00_JPRB, 0.22985E+00_JPRB, 0.23742E+00_JPRB ,&
&  0.24279E+00_JPRB, 0.24705E+00_JPRB, 0.24282E+00_JPRB, 0.24793E+00_JPRB, 0.24848E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.24021E+00_JPRB, 0.24084E+00_JPRB, 0.23827E+00_JPRB, 0.23826E+00_JPRB, 0.23881E+00_JPRB, 0.23902E+00_JPRB, 0.23948E+00_JPRB ,&
&  0.23667E+00_JPRB, 0.23616E+00_JPRB, 0.23390E+00_JPRB, 0.23235E+00_JPRB, 0.23007E+00_JPRB, 0.22793E+00_JPRB, 0.22641E+00_JPRB ,&
&  0.22446E+00_JPRB, 0.22413E+00_JPRB, 0.22329E+00_JPRB, 0.22483E+00_JPRB, 0.22505E+00_JPRB, 0.22729E+00_JPRB, 0.22903E+00_JPRB ,&
&  0.22981E+00_JPRB, 0.23107E+00_JPRB, 0.22915E+00_JPRB, 0.22820E+00_JPRB, 0.22982E+00_JPRB, 0.23100E+00_JPRB, 0.23364E+00_JPRB ,&
&  0.23620E+00_JPRB, 0.23973E+00_JPRB, 0.24574E+00_JPRB, 0.24772E+00_JPRB, 0.25051E+00_JPRB, 0.25210E+00_JPRB, 0.25184E+00_JPRB ,&
&  0.25145E+00_JPRB, 0.25063E+00_JPRB, 0.25131E+00_JPRB, 0.25095E+00_JPRB, 0.24338E+00_JPRB, 0.23788E+00_JPRB, 0.23192E+00_JPRB ,&
&  0.22671E+00_JPRB, 0.22153E+00_JPRB, 0.21675E+00_JPRB, 0.21355E+00_JPRB, 0.20972E+00_JPRB, 0.20713E+00_JPRB, 0.20507E+00_JPRB ,&
&  0.20308E+00_JPRB, 0.20208E+00_JPRB, 0.20058E+00_JPRB, 0.20016E+00_JPRB, 0.20065E+00_JPRB, 0.20176E+00_JPRB, 0.20338E+00_JPRB ,&
&  0.20528E+00_JPRB, 0.20656E+00_JPRB, 0.20814E+00_JPRB, 0.20957E+00_JPRB, 0.21059E+00_JPRB, 0.21112E+00_JPRB, 0.21061E+00_JPRB ,&
&  0.21080E+00_JPRB, 0.21117E+00_JPRB, 0.21064E+00_JPRB, 0.21075E+00_JPRB, 0.20936E+00_JPRB, 0.20771E+00_JPRB, 0.20603E+00_JPRB ,&
&  0.20360E+00_JPRB, 0.20174E+00_JPRB, 0.20111E+00_JPRB, 0.20078E+00_JPRB, 0.20174E+00_JPRB, 0.20315E+00_JPRB, 0.20401E+00_JPRB ,&
&  0.20434E+00_JPRB, 0.20412E+00_JPRB, 0.20402E+00_JPRB, 0.20552E+00_JPRB, 0.20708E+00_JPRB, 0.20908E+00_JPRB, 0.21119E+00_JPRB ,&
&  0.21177E+00_JPRB, 0.21344E+00_JPRB, 0.21484E+00_JPRB, 0.21671E+00_JPRB, 0.22332E+00_JPRB, 0.22800E+00_JPRB, 0.23698E+00_JPRB ,&
&  0.23960E+00_JPRB, 0.23594E+00_JPRB, 0.23187E+00_JPRB, 0.23837E+00_JPRB, 0.23987E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.23868E+00_JPRB, 0.24134E+00_JPRB, 0.23944E+00_JPRB, 0.23892E+00_JPRB, 0.23779E+00_JPRB, 0.23725E+00_JPRB, 0.23548E+00_JPRB ,&
&  0.23522E+00_JPRB, 0.23371E+00_JPRB, 0.23364E+00_JPRB, 0.23200E+00_JPRB, 0.23038E+00_JPRB, 0.22793E+00_JPRB, 0.22549E+00_JPRB ,&
&  0.22319E+00_JPRB, 0.22122E+00_JPRB, 0.21999E+00_JPRB, 0.21974E+00_JPRB, 0.21879E+00_JPRB, 0.21936E+00_JPRB, 0.22265E+00_JPRB ,&
&  0.22342E+00_JPRB, 0.22486E+00_JPRB, 0.22477E+00_JPRB, 0.22600E+00_JPRB, 0.22765E+00_JPRB, 0.22763E+00_JPRB, 0.22976E+00_JPRB ,&
&  0.22968E+00_JPRB, 0.23134E+00_JPRB, 0.23915E+00_JPRB, 0.24304E+00_JPRB, 0.24486E+00_JPRB, 0.24812E+00_JPRB, 0.24921E+00_JPRB ,&
&  0.24719E+00_JPRB, 0.24047E+00_JPRB, 0.24334E+00_JPRB, 0.23938E+00_JPRB, 0.23208E+00_JPRB, 0.22953E+00_JPRB, 0.22379E+00_JPRB ,&
&  0.22125E+00_JPRB, 0.21745E+00_JPRB, 0.21466E+00_JPRB, 0.21249E+00_JPRB, 0.20867E+00_JPRB, 0.20634E+00_JPRB, 0.20366E+00_JPRB ,&
&  0.20126E+00_JPRB, 0.20042E+00_JPRB, 0.19884E+00_JPRB, 0.19889E+00_JPRB, 0.19929E+00_JPRB, 0.20001E+00_JPRB, 0.20115E+00_JPRB ,&
&  0.20254E+00_JPRB, 0.20380E+00_JPRB, 0.20496E+00_JPRB, 0.20592E+00_JPRB, 0.20673E+00_JPRB, 0.20715E+00_JPRB, 0.20690E+00_JPRB ,&
&  0.20699E+00_JPRB, 0.20684E+00_JPRB, 0.20624E+00_JPRB, 0.20606E+00_JPRB, 0.20491E+00_JPRB, 0.20448E+00_JPRB, 0.20400E+00_JPRB ,&
&  0.20380E+00_JPRB, 0.20384E+00_JPRB, 0.20371E+00_JPRB, 0.20394E+00_JPRB, 0.20423E+00_JPRB, 0.20507E+00_JPRB, 0.20568E+00_JPRB ,&
&  0.20617E+00_JPRB, 0.20728E+00_JPRB, 0.20794E+00_JPRB, 0.20888E+00_JPRB, 0.21021E+00_JPRB, 0.21070E+00_JPRB, 0.21221E+00_JPRB ,&
&  0.21345E+00_JPRB, 0.21496E+00_JPRB, 0.21785E+00_JPRB, 0.22068E+00_JPRB, 0.22484E+00_JPRB, 0.22968E+00_JPRB, 0.23073E+00_JPRB ,&
&  0.23119E+00_JPRB, 0.23007E+00_JPRB, 0.22668E+00_JPRB, 0.23290E+00_JPRB, 0.23587E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.23075E+00_JPRB, 0.23284E+00_JPRB, 0.23421E+00_JPRB, 0.23514E+00_JPRB, 0.23685E+00_JPRB, 0.23765E+00_JPRB, 0.23832E+00_JPRB ,&
&  0.23622E+00_JPRB, 0.23576E+00_JPRB, 0.23478E+00_JPRB, 0.23351E+00_JPRB, 0.23272E+00_JPRB, 0.22987E+00_JPRB, 0.22722E+00_JPRB ,&
&  0.22441E+00_JPRB, 0.22197E+00_JPRB, 0.22079E+00_JPRB, 0.21964E+00_JPRB, 0.21837E+00_JPRB, 0.21732E+00_JPRB, 0.22045E+00_JPRB ,&
&  0.22238E+00_JPRB, 0.22352E+00_JPRB, 0.22522E+00_JPRB, 0.22451E+00_JPRB, 0.22593E+00_JPRB, 0.22819E+00_JPRB, 0.23128E+00_JPRB ,&
&  0.23270E+00_JPRB, 0.23581E+00_JPRB, 0.23758E+00_JPRB, 0.23847E+00_JPRB, 0.23983E+00_JPRB, 0.24090E+00_JPRB, 0.24066E+00_JPRB ,&
&  0.23913E+00_JPRB, 0.23711E+00_JPRB, 0.23460E+00_JPRB, 0.23129E+00_JPRB, 0.22790E+00_JPRB, 0.22472E+00_JPRB, 0.22151E+00_JPRB ,&
&  0.21931E+00_JPRB, 0.21728E+00_JPRB, 0.21503E+00_JPRB, 0.21294E+00_JPRB, 0.21093E+00_JPRB, 0.20911E+00_JPRB, 0.20727E+00_JPRB ,&
&  0.20585E+00_JPRB, 0.20471E+00_JPRB, 0.20326E+00_JPRB, 0.20263E+00_JPRB, 0.20199E+00_JPRB, 0.20178E+00_JPRB, 0.20174E+00_JPRB ,&
&  0.20241E+00_JPRB, 0.20318E+00_JPRB, 0.20394E+00_JPRB, 0.20438E+00_JPRB, 0.20479E+00_JPRB, 0.20499E+00_JPRB, 0.20537E+00_JPRB ,&
&  0.20574E+00_JPRB, 0.20615E+00_JPRB, 0.20561E+00_JPRB, 0.20513E+00_JPRB, 0.20448E+00_JPRB, 0.20373E+00_JPRB, 0.20351E+00_JPRB ,&
&  0.20386E+00_JPRB, 0.20415E+00_JPRB, 0.20385E+00_JPRB, 0.20418E+00_JPRB, 0.20480E+00_JPRB, 0.20543E+00_JPRB, 0.20712E+00_JPRB ,&
&  0.20802E+00_JPRB, 0.20887E+00_JPRB, 0.20967E+00_JPRB, 0.21028E+00_JPRB, 0.21126E+00_JPRB, 0.21203E+00_JPRB, 0.21302E+00_JPRB ,&
&  0.21421E+00_JPRB, 0.21533E+00_JPRB, 0.21685E+00_JPRB, 0.21885E+00_JPRB, 0.22383E+00_JPRB, 0.22675E+00_JPRB, 0.22895E+00_JPRB ,&
&  0.22934E+00_JPRB, 0.22324E+00_JPRB, 0.23040E+00_JPRB, 0.22848E+00_JPRB, 0.22772E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.22507E+00_JPRB, 0.22681E+00_JPRB, 0.22799E+00_JPRB, 0.22973E+00_JPRB, 0.22286E+00_JPRB, 0.22959E+00_JPRB, 0.22568E+00_JPRB ,&
&  0.22169E+00_JPRB, 0.21667E+00_JPRB, 0.21737E+00_JPRB, 0.21690E+00_JPRB, 0.21600E+00_JPRB, 0.21589E+00_JPRB, 0.21629E+00_JPRB ,&
&  0.21683E+00_JPRB, 0.21703E+00_JPRB, 0.21805E+00_JPRB, 0.21884E+00_JPRB, 0.21947E+00_JPRB, 0.21999E+00_JPRB, 0.22064E+00_JPRB ,&
&  0.22154E+00_JPRB, 0.22289E+00_JPRB, 0.22429E+00_JPRB, 0.22166E+00_JPRB, 0.22126E+00_JPRB, 0.22140E+00_JPRB, 0.22197E+00_JPRB ,&
&  0.22212E+00_JPRB, 0.22333E+00_JPRB, 0.22435E+00_JPRB, 0.22515E+00_JPRB, 0.22763E+00_JPRB, 0.22791E+00_JPRB, 0.22784E+00_JPRB ,&
&  0.22742E+00_JPRB, 0.22643E+00_JPRB, 0.22506E+00_JPRB, 0.22377E+00_JPRB, 0.22244E+00_JPRB, 0.22065E+00_JPRB, 0.21894E+00_JPRB ,&
&  0.21755E+00_JPRB, 0.21643E+00_JPRB, 0.21492E+00_JPRB, 0.21346E+00_JPRB, 0.21216E+00_JPRB, 0.21092E+00_JPRB, 0.20986E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.20845E+00_JPRB, 0.20775E+00_JPRB, 0.20724E+00_JPRB, 0.20667E+00_JPRB, 0.20614E+00_JPRB, 0.20564E+00_JPRB ,&
&  0.20589E+00_JPRB, 0.20606E+00_JPRB, 0.20624E+00_JPRB, 0.20644E+00_JPRB, 0.20705E+00_JPRB, 0.20720E+00_JPRB, 0.20738E+00_JPRB ,&
&  0.20751E+00_JPRB, 0.20742E+00_JPRB, 0.20726E+00_JPRB, 0.20698E+00_JPRB, 0.20665E+00_JPRB, 0.20624E+00_JPRB, 0.20605E+00_JPRB ,&
&  0.20602E+00_JPRB, 0.20615E+00_JPRB, 0.20659E+00_JPRB, 0.20689E+00_JPRB, 0.20719E+00_JPRB, 0.20746E+00_JPRB, 0.20807E+00_JPRB ,&
&  0.20867E+00_JPRB, 0.20927E+00_JPRB, 0.20983E+00_JPRB, 0.20964E+00_JPRB, 0.20993E+00_JPRB, 0.21020E+00_JPRB, 0.21046E+00_JPRB ,&
&  0.21060E+00_JPRB, 0.21117E+00_JPRB, 0.21211E+00_JPRB, 0.21344E+00_JPRB, 0.21595E+00_JPRB, 0.21796E+00_JPRB, 0.21983E+00_JPRB ,&
&  0.22181E+00_JPRB, 0.22593E+00_JPRB, 0.22722E+00_JPRB, 0.22734E+00_JPRB, 0.22686E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.21464E+00_JPRB, 0.21569E+00_JPRB, 0.21639E+00_JPRB, 0.21691E+00_JPRB, 0.21657E+00_JPRB, 0.21656E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21636E+00_JPRB, 0.21621E+00_JPRB, 0.21654E+00_JPRB, 0.21680E+00_JPRB, 0.21713E+00_JPRB, 0.21750E+00_JPRB, 0.21800E+00_JPRB ,&
&  0.21818E+00_JPRB, 0.21862E+00_JPRB, 0.21693E+00_JPRB, 0.21552E+00_JPRB, 0.21455E+00_JPRB, 0.21392E+00_JPRB, 0.21355E+00_JPRB ,&
&  0.21328E+00_JPRB, 0.21311E+00_JPRB, 0.21293E+00_JPRB, 0.21415E+00_JPRB, 0.21493E+00_JPRB, 0.21560E+00_JPRB, 0.21612E+00_JPRB ,&
&  0.21631E+00_JPRB, 0.21662E+00_JPRB, 0.21689E+00_JPRB, 0.21717E+00_JPRB, 0.21750E+00_JPRB, 0.21751E+00_JPRB, 0.21741E+00_JPRB ,&
&  0.21721E+00_JPRB, 0.21684E+00_JPRB, 0.21641E+00_JPRB, 0.21600E+00_JPRB, 0.21566E+00_JPRB, 0.21525E+00_JPRB, 0.21480E+00_JPRB ,&
&  0.21438E+00_JPRB, 0.21388E+00_JPRB, 0.21341E+00_JPRB, 0.21296E+00_JPRB, 0.21253E+00_JPRB, 0.21216E+00_JPRB, 0.21177E+00_JPRB ,&
&  0.21148E+00_JPRB, 0.21124E+00_JPRB, 0.21097E+00_JPRB, 0.21078E+00_JPRB, 0.21046E+00_JPRB, 0.21036E+00_JPRB, 0.21009E+00_JPRB ,&
&  0.21000E+00_JPRB, 0.20990E+00_JPRB, 0.20985E+00_JPRB, 0.20977E+00_JPRB, 0.20979E+00_JPRB, 0.20978E+00_JPRB, 0.20979E+00_JPRB ,&
&  0.20979E+00_JPRB, 0.20999E+00_JPRB, 0.20986E+00_JPRB, 0.20968E+00_JPRB, 0.20948E+00_JPRB, 0.20928E+00_JPRB, 0.20910E+00_JPRB ,&
&  0.20893E+00_JPRB, 0.20875E+00_JPRB, 0.20864E+00_JPRB, 0.20869E+00_JPRB, 0.20877E+00_JPRB, 0.20887E+00_JPRB, 0.20899E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.20925E+00_JPRB, 0.20938E+00_JPRB, 0.20932E+00_JPRB, 0.20933E+00_JPRB, 0.20936E+00_JPRB, 0.20938E+00_JPRB ,&
&  0.20939E+00_JPRB, 0.20945E+00_JPRB, 0.20956E+00_JPRB, 0.20972E+00_JPRB, 0.21009E+00_JPRB, 0.21022E+00_JPRB, 0.21043E+00_JPRB ,&
&  0.21071E+00_JPRB, 0.21122E+00_JPRB, 0.21166E+00_JPRB, 0.21214E+00_JPRB, 0.21267E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB, 0.21117E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB, 0.82620E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.82276E+00_JPRB, 0.82173E+00_JPRB, 0.82082E+00_JPRB, 0.81995E+00_JPRB, 0.81899E+00_JPRB, 0.81834E+00_JPRB, 0.81784E+00_JPRB ,&
&  0.81731E+00_JPRB, 0.81685E+00_JPRB, 0.81634E+00_JPRB, 0.81597E+00_JPRB, 0.81575E+00_JPRB, 0.81540E+00_JPRB, 0.81504E+00_JPRB ,&
&  0.81454E+00_JPRB, 0.81407E+00_JPRB, 0.81376E+00_JPRB, 0.81305E+00_JPRB, 0.81237E+00_JPRB, 0.81176E+00_JPRB, 0.81070E+00_JPRB ,&
&  0.80924E+00_JPRB, 0.80782E+00_JPRB, 0.80628E+00_JPRB, 0.80499E+00_JPRB, 0.80400E+00_JPRB, 0.80256E+00_JPRB, 0.80088E+00_JPRB ,&
&  0.79942E+00_JPRB, 0.79845E+00_JPRB, 0.79780E+00_JPRB, 0.79738E+00_JPRB, 0.79713E+00_JPRB, 0.79704E+00_JPRB, 0.79691E+00_JPRB ,&
&  0.79691E+00_JPRB, 0.79730E+00_JPRB, 0.79812E+00_JPRB, 0.79926E+00_JPRB, 0.80069E+00_JPRB, 0.80242E+00_JPRB, 0.80488E+00_JPRB ,&
&  0.80756E+00_JPRB, 0.81032E+00_JPRB, 0.81321E+00_JPRB, 0.81582E+00_JPRB, 0.81819E+00_JPRB, 0.82027E+00_JPRB, 0.82186E+00_JPRB ,&
&  0.82359E+00_JPRB, 0.82538E+00_JPRB, 0.82724E+00_JPRB, 0.82864E+00_JPRB, 0.82988E+00_JPRB, 0.83103E+00_JPRB, 0.83228E+00_JPRB ,&
&  0.83297E+00_JPRB, 0.83348E+00_JPRB, 0.83408E+00_JPRB, 0.83488E+00_JPRB, 0.83544E+00_JPRB, 0.83583E+00_JPRB, 0.83624E+00_JPRB ,&
&  0.83687E+00_JPRB, 0.83735E+00_JPRB, 0.83760E+00_JPRB, 0.83787E+00_JPRB, 0.83836E+00_JPRB, 0.83906E+00_JPRB, 0.83977E+00_JPRB ,&
&  0.84039E+00_JPRB, 0.84097E+00_JPRB, 0.84137E+00_JPRB, 0.84155E+00_JPRB, 0.84146E+00_JPRB, 0.84100E+00_JPRB, 0.83987E+00_JPRB ,&
&  0.83852E+00_JPRB, 0.83700E+00_JPRB, 0.83572E+00_JPRB, 0.83489E+00_JPRB, 0.83400E+00_JPRB, 0.83324E+00_JPRB, 0.83268E+00_JPRB ,&
&  0.83247E+00_JPRB, 0.83234E+00_JPRB, 0.83220E+00_JPRB, 0.83218E+00_JPRB, 0.83204E+00_JPRB, 0.83162E+00_JPRB, 0.83099E+00_JPRB ,&
&  0.83014E+00_JPRB, 0.82897E+00_JPRB, 0.82751E+00_JPRB, 0.82590E+00_JPRB, 0.82424E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.81285E+00_JPRB, 0.81035E+00_JPRB, 0.80856E+00_JPRB, 0.80737E+00_JPRB, 0.80629E+00_JPRB, 0.80531E+00_JPRB, 0.80427E+00_JPRB ,&
&  0.80326E+00_JPRB, 0.80331E+00_JPRB, 0.80293E+00_JPRB, 0.80268E+00_JPRB, 0.80238E+00_JPRB, 0.80276E+00_JPRB, 0.80280E+00_JPRB ,&
&  0.80242E+00_JPRB, 0.80096E+00_JPRB, 0.79985E+00_JPRB, 0.79796E+00_JPRB, 0.79776E+00_JPRB, 0.79753E+00_JPRB, 0.79886E+00_JPRB ,&
&  0.79808E+00_JPRB, 0.79698E+00_JPRB, 0.79483E+00_JPRB, 0.78881E+00_JPRB, 0.78376E+00_JPRB, 0.77879E+00_JPRB, 0.77450E+00_JPRB ,&
&  0.77038E+00_JPRB, 0.76729E+00_JPRB, 0.76602E+00_JPRB, 0.76585E+00_JPRB, 0.76960E+00_JPRB, 0.77067E+00_JPRB, 0.77108E+00_JPRB ,&
&  0.77012E+00_JPRB, 0.76548E+00_JPRB, 0.76668E+00_JPRB, 0.76821E+00_JPRB, 0.77048E+00_JPRB, 0.77564E+00_JPRB, 0.78016E+00_JPRB ,&
&  0.78541E+00_JPRB, 0.79131E+00_JPRB, 0.80083E+00_JPRB, 0.80762E+00_JPRB, 0.81433E+00_JPRB, 0.82007E+00_JPRB, 0.82309E+00_JPRB ,&
&  0.82614E+00_JPRB, 0.82884E+00_JPRB, 0.83158E+00_JPRB, 0.83375E+00_JPRB, 0.83542E+00_JPRB, 0.83724E+00_JPRB, 0.83938E+00_JPRB ,&
&  0.84080E+00_JPRB, 0.84197E+00_JPRB, 0.84324E+00_JPRB, 0.84452E+00_JPRB, 0.84535E+00_JPRB, 0.84620E+00_JPRB, 0.84725E+00_JPRB ,&
&  0.84834E+00_JPRB, 0.84822E+00_JPRB, 0.84811E+00_JPRB, 0.84885E+00_JPRB, 0.84964E+00_JPRB, 0.85043E+00_JPRB, 0.85033E+00_JPRB ,&
&  0.85045E+00_JPRB, 0.85016E+00_JPRB, 0.85020E+00_JPRB, 0.85039E+00_JPRB, 0.85019E+00_JPRB, 0.84998E+00_JPRB, 0.84905E+00_JPRB ,&
&  0.84771E+00_JPRB, 0.84567E+00_JPRB, 0.84387E+00_JPRB, 0.84299E+00_JPRB, 0.84196E+00_JPRB, 0.84174E+00_JPRB, 0.84191E+00_JPRB ,&
&  0.84258E+00_JPRB, 0.84144E+00_JPRB, 0.83994E+00_JPRB, 0.83785E+00_JPRB, 0.83574E+00_JPRB, 0.83337E+00_JPRB, 0.83111E+00_JPRB ,&
&  0.82882E+00_JPRB, 0.82614E+00_JPRB, 0.82291E+00_JPRB, 0.81944E+00_JPRB, 0.81605E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.81464E+00_JPRB, 0.81015E+00_JPRB, 0.80702E+00_JPRB, 0.80507E+00_JPRB, 0.80322E+00_JPRB, 0.80138E+00_JPRB, 0.80007E+00_JPRB ,&
&  0.79868E+00_JPRB, 0.79625E+00_JPRB, 0.79493E+00_JPRB, 0.79396E+00_JPRB, 0.79254E+00_JPRB, 0.79307E+00_JPRB, 0.79313E+00_JPRB ,&
&  0.79379E+00_JPRB, 0.79316E+00_JPRB, 0.79533E+00_JPRB, 0.79373E+00_JPRB, 0.79241E+00_JPRB, 0.79068E+00_JPRB, 0.78630E+00_JPRB ,&
&  0.78502E+00_JPRB, 0.78362E+00_JPRB, 0.77881E+00_JPRB, 0.77279E+00_JPRB, 0.76937E+00_JPRB, 0.75094E+00_JPRB, 0.75248E+00_JPRB ,&
&  0.76151E+00_JPRB, 0.74806E+00_JPRB, 0.74095E+00_JPRB, 0.73361E+00_JPRB, 0.72254E+00_JPRB, 0.72255E+00_JPRB, 0.72758E+00_JPRB ,&
&  0.72216E+00_JPRB, 0.74145E+00_JPRB, 0.74435E+00_JPRB, 0.74620E+00_JPRB, 0.74833E+00_JPRB, 0.74514E+00_JPRB, 0.75038E+00_JPRB ,&
&  0.77133E+00_JPRB, 0.77503E+00_JPRB, 0.78434E+00_JPRB, 0.79270E+00_JPRB, 0.80331E+00_JPRB, 0.81317E+00_JPRB, 0.82420E+00_JPRB ,&
&  0.83243E+00_JPRB, 0.83825E+00_JPRB, 0.84241E+00_JPRB, 0.84332E+00_JPRB, 0.84427E+00_JPRB, 0.84447E+00_JPRB, 0.84592E+00_JPRB ,&
&  0.84590E+00_JPRB, 0.84734E+00_JPRB, 0.84980E+00_JPRB, 0.85269E+00_JPRB, 0.85659E+00_JPRB, 0.85903E+00_JPRB, 0.86133E+00_JPRB ,&
&  0.86283E+00_JPRB, 0.86314E+00_JPRB, 0.86277E+00_JPRB, 0.86261E+00_JPRB, 0.86130E+00_JPRB, 0.86059E+00_JPRB, 0.85988E+00_JPRB ,&
&  0.86037E+00_JPRB, 0.86009E+00_JPRB, 0.86068E+00_JPRB, 0.86104E+00_JPRB, 0.86229E+00_JPRB, 0.86463E+00_JPRB, 0.86651E+00_JPRB ,&
&  0.86927E+00_JPRB, 0.87054E+00_JPRB, 0.87109E+00_JPRB, 0.87087E+00_JPRB, 0.87033E+00_JPRB, 0.87074E+00_JPRB, 0.87092E+00_JPRB ,&
&  0.87175E+00_JPRB, 0.86814E+00_JPRB, 0.86375E+00_JPRB, 0.85902E+00_JPRB, 0.85490E+00_JPRB, 0.85115E+00_JPRB, 0.84714E+00_JPRB ,&
&  0.84342E+00_JPRB, 0.83835E+00_JPRB, 0.83269E+00_JPRB, 0.82625E+00_JPRB, 0.82000E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.83177E+00_JPRB, 0.82488E+00_JPRB, 0.81870E+00_JPRB, 0.81427E+00_JPRB, 0.81007E+00_JPRB, 0.80654E+00_JPRB, 0.80350E+00_JPRB ,&
&  0.80079E+00_JPRB, 0.79842E+00_JPRB, 0.79698E+00_JPRB, 0.79543E+00_JPRB, 0.79436E+00_JPRB, 0.79287E+00_JPRB, 0.79211E+00_JPRB ,&
&  0.79403E+00_JPRB, 0.79385E+00_JPRB, 0.79619E+00_JPRB, 0.79394E+00_JPRB, 0.79197E+00_JPRB, 0.78734E+00_JPRB, 0.77632E+00_JPRB ,&
&  0.77106E+00_JPRB, 0.76663E+00_JPRB, 0.75159E+00_JPRB, 0.76526E+00_JPRB, 0.75914E+00_JPRB, 0.75502E+00_JPRB, 0.74821E+00_JPRB ,&
&  0.73276E+00_JPRB, 0.72519E+00_JPRB, 0.71566E+00_JPRB, 0.71538E+00_JPRB, 0.72175E+00_JPRB, 0.71958E+00_JPRB, 0.72469E+00_JPRB ,&
&  0.72038E+00_JPRB, 0.71888E+00_JPRB, 0.71997E+00_JPRB, 0.72001E+00_JPRB, 0.72549E+00_JPRB, 0.72616E+00_JPRB, 0.73046E+00_JPRB ,&
&  0.74027E+00_JPRB, 0.75359E+00_JPRB, 0.77259E+00_JPRB, 0.77945E+00_JPRB, 0.79626E+00_JPRB, 0.81000E+00_JPRB, 0.82281E+00_JPRB ,&
&  0.83122E+00_JPRB, 0.83634E+00_JPRB, 0.84002E+00_JPRB, 0.84164E+00_JPRB, 0.84160E+00_JPRB, 0.83826E+00_JPRB, 0.83834E+00_JPRB ,&
&  0.83915E+00_JPRB, 0.84132E+00_JPRB, 0.84418E+00_JPRB, 0.84690E+00_JPRB, 0.85077E+00_JPRB, 0.85431E+00_JPRB, 0.85893E+00_JPRB ,&
&  0.86306E+00_JPRB, 0.86723E+00_JPRB, 0.86820E+00_JPRB, 0.87006E+00_JPRB, 0.87123E+00_JPRB, 0.87232E+00_JPRB, 0.87345E+00_JPRB ,&
&  0.87479E+00_JPRB, 0.87679E+00_JPRB, 0.88059E+00_JPRB, 0.87798E+00_JPRB, 0.88055E+00_JPRB, 0.89276E+00_JPRB, 0.89600E+00_JPRB ,&
&  0.90024E+00_JPRB, 0.90099E+00_JPRB, 0.89910E+00_JPRB, 0.89693E+00_JPRB, 0.89476E+00_JPRB, 0.89411E+00_JPRB, 0.89375E+00_JPRB ,&
&  0.89320E+00_JPRB, 0.88777E+00_JPRB, 0.88219E+00_JPRB, 0.87722E+00_JPRB, 0.87186E+00_JPRB, 0.86776E+00_JPRB, 0.86358E+00_JPRB ,&
&  0.86063E+00_JPRB, 0.85804E+00_JPRB, 0.85469E+00_JPRB, 0.84863E+00_JPRB, 0.84078E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.85292E+00_JPRB, 0.84486E+00_JPRB, 0.83835E+00_JPRB, 0.83285E+00_JPRB, 0.82860E+00_JPRB, 0.82500E+00_JPRB, 0.82141E+00_JPRB ,&
&  0.81791E+00_JPRB, 0.81371E+00_JPRB, 0.80942E+00_JPRB, 0.80626E+00_JPRB, 0.80384E+00_JPRB, 0.80152E+00_JPRB, 0.79902E+00_JPRB ,&
&  0.79781E+00_JPRB, 0.79625E+00_JPRB, 0.79760E+00_JPRB, 0.79495E+00_JPRB, 0.79611E+00_JPRB, 0.79103E+00_JPRB, 0.77862E+00_JPRB ,&
&  0.76659E+00_JPRB, 0.75414E+00_JPRB, 0.74265E+00_JPRB, 0.75502E+00_JPRB, 0.75422E+00_JPRB, 0.75443E+00_JPRB, 0.73786E+00_JPRB ,&
&  0.72193E+00_JPRB, 0.70868E+00_JPRB, 0.73272E+00_JPRB, 0.72470E+00_JPRB, 0.72052E+00_JPRB, 0.71222E+00_JPRB, 0.69348E+00_JPRB ,&
&  0.69042E+00_JPRB, 0.67939E+00_JPRB, 0.67865E+00_JPRB, 0.68364E+00_JPRB, 0.69017E+00_JPRB, 0.69960E+00_JPRB, 0.70890E+00_JPRB ,&
&  0.70797E+00_JPRB, 0.74089E+00_JPRB, 0.76372E+00_JPRB, 0.78082E+00_JPRB, 0.79777E+00_JPRB, 0.81180E+00_JPRB, 0.82143E+00_JPRB ,&
&  0.83018E+00_JPRB, 0.83667E+00_JPRB, 0.84290E+00_JPRB, 0.84522E+00_JPRB, 0.85539E+00_JPRB, 0.85774E+00_JPRB, 0.86123E+00_JPRB ,&
&  0.86534E+00_JPRB, 0.86538E+00_JPRB, 0.86424E+00_JPRB, 0.86103E+00_JPRB, 0.85978E+00_JPRB, 0.85753E+00_JPRB, 0.85899E+00_JPRB ,&
&  0.85968E+00_JPRB, 0.86164E+00_JPRB, 0.86693E+00_JPRB, 0.87187E+00_JPRB, 0.87647E+00_JPRB, 0.88082E+00_JPRB, 0.88554E+00_JPRB ,&
&  0.88999E+00_JPRB, 0.89824E+00_JPRB, 0.90618E+00_JPRB, 0.92050E+00_JPRB, 0.93304E+00_JPRB, 0.93955E+00_JPRB, 0.94431E+00_JPRB ,&
&  0.94989E+00_JPRB, 0.94401E+00_JPRB, 0.93827E+00_JPRB, 0.93443E+00_JPRB, 0.93219E+00_JPRB, 0.93111E+00_JPRB, 0.93140E+00_JPRB ,&
&  0.93029E+00_JPRB, 0.92524E+00_JPRB, 0.91910E+00_JPRB, 0.91184E+00_JPRB, 0.90472E+00_JPRB, 0.89910E+00_JPRB, 0.89432E+00_JPRB ,&
&  0.89003E+00_JPRB, 0.88556E+00_JPRB, 0.87844E+00_JPRB, 0.86987E+00_JPRB, 0.86118E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.88214E+00_JPRB, 0.87374E+00_JPRB, 0.86712E+00_JPRB, 0.86129E+00_JPRB, 0.85710E+00_JPRB, 0.85359E+00_JPRB, 0.85043E+00_JPRB ,&
&  0.84790E+00_JPRB, 0.84256E+00_JPRB, 0.83634E+00_JPRB, 0.82872E+00_JPRB, 0.82339E+00_JPRB, 0.81862E+00_JPRB, 0.81335E+00_JPRB ,&
&  0.80605E+00_JPRB, 0.79826E+00_JPRB, 0.78997E+00_JPRB, 0.78340E+00_JPRB, 0.78508E+00_JPRB, 0.78122E+00_JPRB, 0.77544E+00_JPRB ,&
&  0.76412E+00_JPRB, 0.75067E+00_JPRB, 0.74076E+00_JPRB, 0.74530E+00_JPRB, 0.74869E+00_JPRB, 0.73817E+00_JPRB, 0.73554E+00_JPRB ,&
&  0.74937E+00_JPRB, 0.75107E+00_JPRB, 0.74271E+00_JPRB, 0.73881E+00_JPRB, 0.70836E+00_JPRB, 0.66782E+00_JPRB, 0.69349E+00_JPRB ,&
&  0.69396E+00_JPRB, 0.67496E+00_JPRB, 0.67166E+00_JPRB, 0.66942E+00_JPRB, 0.67724E+00_JPRB, 0.69442E+00_JPRB, 0.71017E+00_JPRB ,&
&  0.71328E+00_JPRB, 0.76996E+00_JPRB, 0.79466E+00_JPRB, 0.81218E+00_JPRB, 0.82392E+00_JPRB, 0.83375E+00_JPRB, 0.84132E+00_JPRB ,&
&  0.84858E+00_JPRB, 0.85986E+00_JPRB, 0.87481E+00_JPRB, 0.88987E+00_JPRB, 0.90454E+00_JPRB, 0.91532E+00_JPRB, 0.92685E+00_JPRB ,&
&  0.94243E+00_JPRB, 0.94506E+00_JPRB, 0.93224E+00_JPRB, 0.91807E+00_JPRB, 0.90541E+00_JPRB, 0.89377E+00_JPRB, 0.88367E+00_JPRB ,&
&  0.87962E+00_JPRB, 0.87737E+00_JPRB, 0.88411E+00_JPRB, 0.88608E+00_JPRB, 0.89050E+00_JPRB, 0.89624E+00_JPRB, 0.90977E+00_JPRB ,&
&  0.91876E+00_JPRB, 0.93200E+00_JPRB, 0.94771E+00_JPRB, 0.96845E+00_JPRB, 0.98496E+00_JPRB, 0.99277E+00_JPRB, 0.99708E+00_JPRB ,&
&  0.99802E+00_JPRB, 0.10043E+01_JPRB, 0.10052E+01_JPRB, 0.10101E+01_JPRB, 0.10192E+01_JPRB, 0.10235E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.10308E+01_JPRB, 0.10300E+01_JPRB, 0.10243E+01_JPRB, 0.10146E+01_JPRB, 0.10010E+01_JPRB, 0.98566E+00_JPRB, 0.96743E+00_JPRB ,&
&  0.94830E+00_JPRB, 0.92775E+00_JPRB, 0.91042E+00_JPRB, 0.89922E+00_JPRB, 0.88981E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.93398E+00_JPRB, 0.91742E+00_JPRB, 0.90545E+00_JPRB, 0.89837E+00_JPRB, 0.89252E+00_JPRB, 0.88765E+00_JPRB, 0.88289E+00_JPRB ,&
&  0.87808E+00_JPRB, 0.87278E+00_JPRB, 0.86564E+00_JPRB, 0.85780E+00_JPRB, 0.85122E+00_JPRB, 0.84477E+00_JPRB, 0.83950E+00_JPRB ,&
&  0.83051E+00_JPRB, 0.81014E+00_JPRB, 0.80659E+00_JPRB, 0.79591E+00_JPRB, 0.77851E+00_JPRB, 0.76842E+00_JPRB, 0.76054E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.74785E+00_JPRB, 0.74905E+00_JPRB, 0.74367E+00_JPRB, 0.74175E+00_JPRB, 0.75456E+00_JPRB, 0.75782E+00_JPRB ,&
&  0.72931E+00_JPRB, 0.72883E+00_JPRB, 0.76552E+00_JPRB, 0.76655E+00_JPRB, 0.72384E+00_JPRB, 0.71050E+00_JPRB, 0.71768E+00_JPRB ,&
&  0.71642E+00_JPRB, 0.68753E+00_JPRB, 0.68277E+00_JPRB, 0.68138E+00_JPRB, 0.69413E+00_JPRB, 0.70686E+00_JPRB, 0.73779E+00_JPRB ,&
&  0.76118E+00_JPRB, 0.81338E+00_JPRB, 0.83819E+00_JPRB, 0.85517E+00_JPRB, 0.86792E+00_JPRB, 0.88137E+00_JPRB, 0.89619E+00_JPRB ,&
&  0.91714E+00_JPRB, 0.94338E+00_JPRB, 0.97781E+00_JPRB, 0.10089E+01_JPRB, 0.10389E+01_JPRB, 0.10530E+01_JPRB, 0.10550E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10815E+01_JPRB, 0.10482E+01_JPRB, 0.10213E+01_JPRB, 0.99255E+00_JPRB, 0.96523E+00_JPRB, 0.93700E+00_JPRB ,&
&  0.93036E+00_JPRB, 0.93857E+00_JPRB, 0.93123E+00_JPRB, 0.92597E+00_JPRB, 0.92548E+00_JPRB, 0.92775E+00_JPRB, 0.93527E+00_JPRB ,&
&  0.94068E+00_JPRB, 0.97652E+00_JPRB, 0.10874E+01_JPRB, 0.11167E+01_JPRB, 0.11282E+01_JPRB, 0.11307E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.11375E+01_JPRB, 0.11500E+01_JPRB, 0.11655E+01_JPRB, 0.11729E+01_JPRB, 0.11811E+01_JPRB, 0.11731E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.11724E+01_JPRB, 0.11626E+01_JPRB, 0.11503E+01_JPRB, 0.11408E+01_JPRB, 0.11291E+01_JPRB, 0.11178E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.10484E+01_JPRB, 0.10195E+01_JPRB, 0.98867E+00_JPRB, 0.95879E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.10098E+01_JPRB, 0.97162E+00_JPRB, 0.94572E+00_JPRB, 0.93431E+00_JPRB, 0.92677E+00_JPRB, 0.90893E+00_JPRB, 0.89515E+00_JPRB ,&
&  0.88505E+00_JPRB, 0.88088E+00_JPRB, 0.87304E+00_JPRB, 0.86328E+00_JPRB, 0.85702E+00_JPRB, 0.85139E+00_JPRB, 0.84836E+00_JPRB ,&
&  0.84438E+00_JPRB, 0.83472E+00_JPRB, 0.81442E+00_JPRB, 0.79253E+00_JPRB, 0.77726E+00_JPRB, 0.76474E+00_JPRB, 0.75448E+00_JPRB ,&
&  0.74553E+00_JPRB, 0.74386E+00_JPRB, 0.75391E+00_JPRB, 0.75353E+00_JPRB, 0.75311E+00_JPRB, 0.74862E+00_JPRB, 0.74497E+00_JPRB ,&
&  0.74941E+00_JPRB, 0.76413E+00_JPRB, 0.74318E+00_JPRB, 0.74782E+00_JPRB, 0.76393E+00_JPRB, 0.74733E+00_JPRB, 0.73268E+00_JPRB ,&
&  0.72024E+00_JPRB, 0.70957E+00_JPRB, 0.70468E+00_JPRB, 0.70522E+00_JPRB, 0.72619E+00_JPRB, 0.75129E+00_JPRB, 0.79139E+00_JPRB ,&
&  0.83453E+00_JPRB, 0.86547E+00_JPRB, 0.88388E+00_JPRB, 0.89998E+00_JPRB, 0.91625E+00_JPRB, 0.93472E+00_JPRB, 0.96578E+00_JPRB ,&
&  0.10167E+01_JPRB, 0.10717E+01_JPRB, 0.11258E+01_JPRB, 0.11619E+01_JPRB, 0.11990E+01_JPRB, 0.11730E+01_JPRB, 0.11770E+01_JPRB ,&
&  0.13034E+01_JPRB, 0.13466E+01_JPRB, 0.12772E+01_JPRB, 0.12181E+01_JPRB, 0.11565E+01_JPRB, 0.11032E+01_JPRB, 0.10605E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.10176E+01_JPRB, 0.10062E+01_JPRB, 0.98913E+00_JPRB, 0.97875E+00_JPRB, 0.98703E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.12428E+01_JPRB, 0.15737E+01_JPRB, 0.13853E+01_JPRB, 0.13162E+01_JPRB, 0.13046E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.13353E+01_JPRB, 0.13565E+01_JPRB, 0.13663E+01_JPRB, 0.13430E+01_JPRB, 0.13024E+01_JPRB, 0.12540E+01_JPRB, 0.12222E+01_JPRB ,&
&  0.11953E+01_JPRB, 0.11686E+01_JPRB, 0.11513E+01_JPRB, 0.11460E+01_JPRB, 0.11340E+01_JPRB, 0.11216E+01_JPRB, 0.11098E+01_JPRB ,&
&  0.10955E+01_JPRB, 0.10863E+01_JPRB, 0.10821E+01_JPRB, 0.10702E+01_JPRB, 0.10503E+01_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.10324E+01_JPRB, 0.10378E+01_JPRB, 0.10260E+01_JPRB, 0.10082E+01_JPRB, 0.98798E+00_JPRB, 0.94131E+00_JPRB, 0.90514E+00_JPRB ,&
&  0.88616E+00_JPRB, 0.87881E+00_JPRB, 0.87744E+00_JPRB, 0.87352E+00_JPRB, 0.86992E+00_JPRB, 0.86434E+00_JPRB, 0.85444E+00_JPRB ,&
&  0.84100E+00_JPRB, 0.83111E+00_JPRB, 0.82239E+00_JPRB, 0.80775E+00_JPRB, 0.78856E+00_JPRB, 0.76846E+00_JPRB, 0.75256E+00_JPRB ,&
&  0.74140E+00_JPRB, 0.73716E+00_JPRB, 0.74566E+00_JPRB, 0.75463E+00_JPRB, 0.76420E+00_JPRB, 0.76793E+00_JPRB, 0.76476E+00_JPRB ,&
&  0.74997E+00_JPRB, 0.74709E+00_JPRB, 0.75916E+00_JPRB, 0.75951E+00_JPRB, 0.76619E+00_JPRB, 0.76863E+00_JPRB, 0.75635E+00_JPRB ,&
&  0.74508E+00_JPRB, 0.73673E+00_JPRB, 0.74169E+00_JPRB, 0.75782E+00_JPRB, 0.80394E+00_JPRB, 0.85457E+00_JPRB, 0.87864E+00_JPRB ,&
&  0.90157E+00_JPRB, 0.91540E+00_JPRB, 0.92890E+00_JPRB, 0.95032E+00_JPRB, 0.98055E+00_JPRB, 0.10187E+01_JPRB, 0.10900E+01_JPRB ,&
&  0.11876E+01_JPRB, 0.12593E+01_JPRB, 0.13245E+01_JPRB, 0.12466E+01_JPRB, 0.12180E+01_JPRB, 0.12486E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.13612E+01_JPRB, 0.13965E+01_JPRB, 0.13892E+01_JPRB, 0.13963E+01_JPRB, 0.13405E+01_JPRB, 0.12712E+01_JPRB, 0.11796E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.10704E+01_JPRB, 0.10574E+01_JPRB, 0.10467E+01_JPRB, 0.10635E+01_JPRB, 0.10998E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.12615E+01_JPRB, 0.13812E+01_JPRB, 0.14465E+01_JPRB, 0.14076E+01_JPRB, 0.14358E+01_JPRB, 0.14968E+01_JPRB, 0.15420E+01_JPRB ,&
&  0.15409E+01_JPRB, 0.15340E+01_JPRB, 0.15391E+01_JPRB, 0.15239E+01_JPRB, 0.14300E+01_JPRB, 0.13447E+01_JPRB, 0.12492E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.10746E+01_JPRB, 0.10329E+01_JPRB, 0.10212E+01_JPRB, 0.10128E+01_JPRB, 0.10025E+01_JPRB, 0.98230E+00_JPRB ,&
&  0.96111E+00_JPRB, 0.95771E+00_JPRB, 0.97124E+00_JPRB, 0.99030E+00_JPRB, 0.10160E+01_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.10782E+01_JPRB, 0.10991E+01_JPRB, 0.11095E+01_JPRB, 0.11100E+01_JPRB, 0.10651E+01_JPRB, 0.10197E+01_JPRB, 0.97491E+00_JPRB ,&
&  0.94892E+00_JPRB, 0.92064E+00_JPRB, 0.90150E+00_JPRB, 0.87931E+00_JPRB, 0.85968E+00_JPRB, 0.84917E+00_JPRB, 0.84659E+00_JPRB ,&
&  0.84553E+00_JPRB, 0.83566E+00_JPRB, 0.82261E+00_JPRB, 0.80535E+00_JPRB, 0.79066E+00_JPRB, 0.77715E+00_JPRB, 0.76474E+00_JPRB ,&
&  0.75461E+00_JPRB, 0.75286E+00_JPRB, 0.75708E+00_JPRB, 0.76746E+00_JPRB, 0.77536E+00_JPRB, 0.77949E+00_JPRB, 0.78042E+00_JPRB ,&
&  0.77451E+00_JPRB, 0.76508E+00_JPRB, 0.76955E+00_JPRB, 0.77978E+00_JPRB, 0.78763E+00_JPRB, 0.79605E+00_JPRB, 0.77992E+00_JPRB ,&
&  0.77218E+00_JPRB, 0.77885E+00_JPRB, 0.80452E+00_JPRB, 0.86549E+00_JPRB, 0.89931E+00_JPRB, 0.91931E+00_JPRB, 0.93219E+00_JPRB ,&
&  0.92207E+00_JPRB, 0.92266E+00_JPRB, 0.94542E+00_JPRB, 0.99594E+00_JPRB, 0.10555E+01_JPRB, 0.11594E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.14265E+01_JPRB, 0.14505E+01_JPRB, 0.13676E+01_JPRB, 0.13722E+01_JPRB, 0.15354E+01_JPRB, 0.15787E+01_JPRB ,&
&  0.16244E+01_JPRB, 0.16417E+01_JPRB, 0.16832E+01_JPRB, 0.16346E+01_JPRB, 0.14902E+01_JPRB, 0.13836E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.12305E+01_JPRB, 0.11890E+01_JPRB, 0.11884E+01_JPRB, 0.12244E+01_JPRB, 0.13004E+01_JPRB, 0.14469E+01_JPRB, 0.16303E+01_JPRB ,&
&  0.18048E+01_JPRB, 0.19081E+01_JPRB, 0.19711E+01_JPRB, 0.19581E+01_JPRB, 0.19321E+01_JPRB, 0.18932E+01_JPRB, 0.18639E+01_JPRB ,&
&  0.18275E+01_JPRB, 0.18158E+01_JPRB, 0.17992E+01_JPRB, 0.17807E+01_JPRB, 0.16621E+01_JPRB, 0.14642E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.10501E+01_JPRB, 0.99614E+00_JPRB, 0.98401E+00_JPRB, 0.98758E+00_JPRB, 0.99043E+00_JPRB, 0.96532E+00_JPRB ,&
&  0.95372E+00_JPRB, 0.96605E+00_JPRB, 0.98708E+00_JPRB, 0.10081E+01_JPRB, 0.10414E+01_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.11723E+01_JPRB, 0.11925E+01_JPRB, 0.12007E+01_JPRB, 0.11886E+01_JPRB, 0.11529E+01_JPRB, 0.10917E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.99039E+00_JPRB, 0.95186E+00_JPRB, 0.93346E+00_JPRB, 0.91215E+00_JPRB, 0.89267E+00_JPRB, 0.86625E+00_JPRB, 0.84054E+00_JPRB ,&
&  0.83267E+00_JPRB, 0.83575E+00_JPRB, 0.83379E+00_JPRB, 0.82404E+00_JPRB, 0.81161E+00_JPRB, 0.79800E+00_JPRB, 0.78626E+00_JPRB ,&
&  0.77840E+00_JPRB, 0.77826E+00_JPRB, 0.78826E+00_JPRB, 0.80119E+00_JPRB, 0.80751E+00_JPRB, 0.81092E+00_JPRB, 0.81726E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.80772E+00_JPRB, 0.80570E+00_JPRB, 0.81064E+00_JPRB, 0.81491E+00_JPRB, 0.83640E+00_JPRB, 0.83641E+00_JPRB ,&
&  0.81078E+00_JPRB, 0.83704E+00_JPRB, 0.89225E+00_JPRB, 0.92359E+00_JPRB, 0.94393E+00_JPRB, 0.95847E+00_JPRB, 0.97047E+00_JPRB ,&
&  0.98088E+00_JPRB, 0.99866E+00_JPRB, 0.10200E+01_JPRB, 0.10451E+01_JPRB, 0.11479E+01_JPRB, 0.13836E+01_JPRB, 0.14862E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.14284E+01_JPRB, 0.13593E+01_JPRB, 0.13794E+01_JPRB, 0.15681E+01_JPRB, 0.16801E+01_JPRB, 0.17561E+01_JPRB ,&
&  0.18628E+01_JPRB, 0.18728E+01_JPRB, 0.18489E+01_JPRB, 0.17770E+01_JPRB, 0.16530E+01_JPRB, 0.15613E+01_JPRB, 0.15370E+01_JPRB ,&
&  0.14975E+01_JPRB, 0.15029E+01_JPRB, 0.15889E+01_JPRB, 0.17696E+01_JPRB, 0.19785E+01_JPRB, 0.21531E+01_JPRB, 0.22797E+01_JPRB ,&
&  0.23998E+01_JPRB, 0.24556E+01_JPRB, 0.24280E+01_JPRB, 0.23739E+01_JPRB, 0.22777E+01_JPRB, 0.22210E+01_JPRB, 0.21978E+01_JPRB ,&
&  0.21680E+01_JPRB, 0.21425E+01_JPRB, 0.20897E+01_JPRB, 0.20153E+01_JPRB, 0.18905E+01_JPRB, 0.17673E+01_JPRB, 0.15948E+01_JPRB ,&
&  0.13245E+01_JPRB, 0.11734E+01_JPRB, 0.10566E+01_JPRB, 0.10253E+01_JPRB, 0.10292E+01_JPRB, 0.10441E+01_JPRB, 0.10389E+01_JPRB ,&
&  0.10558E+01_JPRB, 0.10680E+01_JPRB, 0.10736E+01_JPRB, 0.11029E+01_JPRB, 0.11407E+01_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.12716E+01_JPRB, 0.12790E+01_JPRB, 0.12708E+01_JPRB, 0.12581E+01_JPRB, 0.12312E+01_JPRB, 0.11746E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.11103E+01_JPRB, 0.11116E+01_JPRB, 0.99351E+00_JPRB, 0.94982E+00_JPRB, 0.93159E+00_JPRB, 0.89700E+00_JPRB ,&
&  0.87633E+00_JPRB, 0.86767E+00_JPRB, 0.85626E+00_JPRB, 0.85149E+00_JPRB, 0.84369E+00_JPRB, 0.82855E+00_JPRB, 0.81723E+00_JPRB ,&
&  0.81354E+00_JPRB, 0.81981E+00_JPRB, 0.83298E+00_JPRB, 0.84404E+00_JPRB, 0.85441E+00_JPRB, 0.86292E+00_JPRB, 0.86834E+00_JPRB ,&
&  0.85422E+00_JPRB, 0.84848E+00_JPRB, 0.84775E+00_JPRB, 0.84201E+00_JPRB, 0.84767E+00_JPRB, 0.86334E+00_JPRB, 0.89047E+00_JPRB ,&
&  0.90857E+00_JPRB, 0.90123E+00_JPRB, 0.94961E+00_JPRB, 0.97447E+00_JPRB, 0.99146E+00_JPRB, 0.10037E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10374E+01_JPRB, 0.10609E+01_JPRB, 0.10928E+01_JPRB, 0.12822E+01_JPRB, 0.16577E+01_JPRB, 0.16939E+01_JPRB ,&
&  0.16827E+01_JPRB, 0.14648E+01_JPRB, 0.16082E+01_JPRB, 0.16800E+01_JPRB, 0.18247E+01_JPRB, 0.19629E+01_JPRB, 0.19817E+01_JPRB ,&
&  0.22078E+01_JPRB, 0.23080E+01_JPRB, 0.22377E+01_JPRB, 0.20803E+01_JPRB, 0.19376E+01_JPRB, 0.18725E+01_JPRB, 0.18545E+01_JPRB ,&
&  0.18902E+01_JPRB, 0.18699E+01_JPRB, 0.20981E+01_JPRB, 0.22960E+01_JPRB, 0.24025E+01_JPRB, 0.25029E+01_JPRB, 0.26195E+01_JPRB ,&
&  0.26976E+01_JPRB, 0.27065E+01_JPRB, 0.26982E+01_JPRB, 0.25601E+01_JPRB, 0.24889E+01_JPRB, 0.24329E+01_JPRB, 0.23314E+01_JPRB ,&
&  0.22661E+01_JPRB, 0.22142E+01_JPRB, 0.21416E+01_JPRB, 0.21564E+01_JPRB, 0.19802E+01_JPRB, 0.18513E+01_JPRB, 0.17712E+01_JPRB ,&
&  0.15569E+01_JPRB, 0.13293E+01_JPRB, 0.12405E+01_JPRB, 0.12072E+01_JPRB, 0.11798E+01_JPRB, 0.11762E+01_JPRB, 0.11731E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11630E+01_JPRB, 0.12167E+01_JPRB, 0.12380E+01_JPRB, 0.12513E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.13626E+01_JPRB, 0.13707E+01_JPRB, 0.13636E+01_JPRB, 0.13507E+01_JPRB, 0.13344E+01_JPRB, 0.13050E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.12460E+01_JPRB, 0.12099E+01_JPRB, 0.11669E+01_JPRB, 0.11381E+01_JPRB, 0.11311E+01_JPRB, 0.10602E+01_JPRB, 0.95060E+00_JPRB ,&
&  0.90765E+00_JPRB, 0.90457E+00_JPRB, 0.90918E+00_JPRB, 0.90523E+00_JPRB, 0.89898E+00_JPRB, 0.88997E+00_JPRB, 0.88531E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.89290E+00_JPRB, 0.90823E+00_JPRB, 0.92702E+00_JPRB, 0.93215E+00_JPRB, 0.93719E+00_JPRB, 0.94479E+00_JPRB ,&
&  0.92025E+00_JPRB, 0.89699E+00_JPRB, 0.88190E+00_JPRB, 0.86361E+00_JPRB, 0.89341E+00_JPRB, 0.91694E+00_JPRB, 0.94791E+00_JPRB ,&
&  0.97757E+00_JPRB, 0.10034E+01_JPRB, 0.10307E+01_JPRB, 0.10499E+01_JPRB, 0.10584E+01_JPRB, 0.10653E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.11006E+01_JPRB, 0.11294E+01_JPRB, 0.11662E+01_JPRB, 0.15347E+01_JPRB, 0.19420E+01_JPRB, 0.20565E+01_JPRB ,&
&  0.19955E+01_JPRB, 0.21520E+01_JPRB, 0.22705E+01_JPRB, 0.21814E+01_JPRB, 0.23030E+01_JPRB, 0.23867E+01_JPRB, 0.23376E+01_JPRB ,&
&  0.24607E+01_JPRB, 0.24832E+01_JPRB, 0.24592E+01_JPRB, 0.24247E+01_JPRB, 0.24074E+01_JPRB, 0.23570E+01_JPRB, 0.23231E+01_JPRB ,&
&  0.23798E+01_JPRB, 0.25788E+01_JPRB, 0.26291E+01_JPRB, 0.27028E+01_JPRB, 0.27207E+01_JPRB, 0.27672E+01_JPRB, 0.28149E+01_JPRB ,&
&  0.28486E+01_JPRB, 0.28953E+01_JPRB, 0.29047E+01_JPRB, 0.28307E+01_JPRB, 0.27261E+01_JPRB, 0.26140E+01_JPRB, 0.24758E+01_JPRB ,&
&  0.23677E+01_JPRB, 0.21471E+01_JPRB, 0.19740E+01_JPRB, 0.19220E+01_JPRB, 0.18734E+01_JPRB, 0.17291E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.16334E+01_JPRB, 0.15445E+01_JPRB, 0.16123E+01_JPRB, 0.15214E+01_JPRB, 0.14365E+01_JPRB, 0.13374E+01_JPRB, 0.12164E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.12539E+01_JPRB, 0.13198E+01_JPRB, 0.13555E+01_JPRB, 0.13601E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.14388E+01_JPRB, 0.14512E+01_JPRB, 0.14553E+01_JPRB, 0.14502E+01_JPRB, 0.14276E+01_JPRB, 0.13923E+01_JPRB, 0.13503E+01_JPRB ,&
&  0.13125E+01_JPRB, 0.12678E+01_JPRB, 0.12284E+01_JPRB, 0.11981E+01_JPRB, 0.11849E+01_JPRB, 0.11663E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.99621E+00_JPRB, 0.96767E+00_JPRB, 0.94342E+00_JPRB, 0.93934E+00_JPRB, 0.95366E+00_JPRB, 0.96117E+00_JPRB, 0.97033E+00_JPRB ,&
&  0.99649E+00_JPRB, 0.10150E+01_JPRB, 0.10260E+01_JPRB, 0.10364E+01_JPRB, 0.10427E+01_JPRB, 0.10554E+01_JPRB, 0.10676E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.97605E+00_JPRB, 0.94814E+00_JPRB, 0.93770E+00_JPRB, 0.96913E+00_JPRB, 0.10111E+01_JPRB, 0.10542E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.11131E+01_JPRB, 0.11356E+01_JPRB, 0.11505E+01_JPRB, 0.11529E+01_JPRB, 0.11559E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.12019E+01_JPRB, 0.12182E+01_JPRB, 0.12763E+01_JPRB, 0.18638E+01_JPRB, 0.25480E+01_JPRB, 0.28579E+01_JPRB ,&
&  0.28118E+01_JPRB, 0.28438E+01_JPRB, 0.29234E+01_JPRB, 0.31145E+01_JPRB, 0.31103E+01_JPRB, 0.29238E+01_JPRB, 0.27620E+01_JPRB ,&
&  0.27987E+01_JPRB, 0.26986E+01_JPRB, 0.29139E+01_JPRB, 0.29105E+01_JPRB, 0.29513E+01_JPRB, 0.30209E+01_JPRB, 0.29976E+01_JPRB ,&
&  0.30625E+01_JPRB, 0.30890E+01_JPRB, 0.30660E+01_JPRB, 0.29753E+01_JPRB, 0.29357E+01_JPRB, 0.29770E+01_JPRB, 0.30089E+01_JPRB ,&
&  0.31048E+01_JPRB, 0.31807E+01_JPRB, 0.30492E+01_JPRB, 0.29421E+01_JPRB, 0.28041E+01_JPRB, 0.29285E+01_JPRB, 0.27076E+01_JPRB ,&
&  0.23460E+01_JPRB, 0.19123E+01_JPRB, 0.19039E+01_JPRB, 0.19044E+01_JPRB, 0.19294E+01_JPRB, 0.18761E+01_JPRB, 0.17586E+01_JPRB ,&
&  0.17550E+01_JPRB, 0.19075E+01_JPRB, 0.19543E+01_JPRB, 0.17596E+01_JPRB, 0.16406E+01_JPRB, 0.15014E+01_JPRB, 0.13175E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.13987E+01_JPRB, 0.14302E+01_JPRB, 0.14246E+01_JPRB, 0.14192E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.15780E+01_JPRB, 0.15399E+01_JPRB, 0.15221E+01_JPRB, 0.15127E+01_JPRB, 0.14868E+01_JPRB, 0.14553E+01_JPRB, 0.14173E+01_JPRB ,&
&  0.13850E+01_JPRB, 0.13543E+01_JPRB, 0.13233E+01_JPRB, 0.12965E+01_JPRB, 0.12795E+01_JPRB, 0.12523E+01_JPRB, 0.12237E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.10553E+01_JPRB, 0.10246E+01_JPRB, 0.10230E+01_JPRB, 0.10807E+01_JPRB, 0.10769E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.11235E+01_JPRB, 0.11183E+01_JPRB, 0.11217E+01_JPRB, 0.11408E+01_JPRB, 0.11915E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.11384E+01_JPRB, 0.11067E+01_JPRB, 0.10957E+01_JPRB, 0.11362E+01_JPRB, 0.11711E+01_JPRB, 0.12355E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.12647E+01_JPRB, 0.12651E+01_JPRB, 0.12632E+01_JPRB, 0.12664E+01_JPRB, 0.12830E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.13400E+01_JPRB, 0.13303E+01_JPRB, 0.13366E+01_JPRB, 0.14767E+01_JPRB, 0.23501E+01_JPRB, 0.32878E+01_JPRB, 0.38271E+01_JPRB ,&
&  0.40431E+01_JPRB, 0.39099E+01_JPRB, 0.39726E+01_JPRB, 0.45416E+01_JPRB, 0.44005E+01_JPRB, 0.39429E+01_JPRB, 0.35413E+01_JPRB ,&
&  0.33759E+01_JPRB, 0.32346E+01_JPRB, 0.33796E+01_JPRB, 0.35059E+01_JPRB, 0.34245E+01_JPRB, 0.34301E+01_JPRB, 0.34169E+01_JPRB ,&
&  0.34196E+01_JPRB, 0.34409E+01_JPRB, 0.33159E+01_JPRB, 0.31867E+01_JPRB, 0.30857E+01_JPRB, 0.31792E+01_JPRB, 0.32345E+01_JPRB ,&
&  0.34134E+01_JPRB, 0.33731E+01_JPRB, 0.30266E+01_JPRB, 0.30126E+01_JPRB, 0.24717E+01_JPRB, 0.23505E+01_JPRB, 0.26513E+01_JPRB ,&
&  0.22845E+01_JPRB, 0.20039E+01_JPRB, 0.19500E+01_JPRB, 0.20273E+01_JPRB, 0.20944E+01_JPRB, 0.22254E+01_JPRB, 0.21639E+01_JPRB ,&
&  0.19053E+01_JPRB, 0.19927E+01_JPRB, 0.22921E+01_JPRB, 0.20656E+01_JPRB, 0.18890E+01_JPRB, 0.17684E+01_JPRB, 0.16036E+01_JPRB ,&
&  0.16248E+01_JPRB, 0.16085E+01_JPRB, 0.15916E+01_JPRB, 0.15717E+01_JPRB, 0.15744E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.16829E+01_JPRB, 0.15593E+01_JPRB, 0.15512E+01_JPRB, 0.15194E+01_JPRB, 0.14929E+01_JPRB, 0.14657E+01_JPRB, 0.14512E+01_JPRB ,&
&  0.14451E+01_JPRB, 0.14339E+01_JPRB, 0.14123E+01_JPRB, 0.13859E+01_JPRB, 0.13647E+01_JPRB, 0.13346E+01_JPRB, 0.12998E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.12069E+01_JPRB, 0.11855E+01_JPRB, 0.11415E+01_JPRB, 0.11896E+01_JPRB, 0.11906E+01_JPRB, 0.12105E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.12778E+01_JPRB, 0.12803E+01_JPRB, 0.12902E+01_JPRB, 0.13260E+01_JPRB, 0.14178E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.15447E+01_JPRB, 0.14677E+01_JPRB, 0.14242E+01_JPRB, 0.14432E+01_JPRB, 0.15192E+01_JPRB, 0.14265E+01_JPRB, 0.14395E+01_JPRB ,&
&  0.14190E+01_JPRB, 0.13920E+01_JPRB, 0.13726E+01_JPRB, 0.13715E+01_JPRB, 0.14046E+01_JPRB, 0.14564E+01_JPRB, 0.14828E+01_JPRB ,&
&  0.14860E+01_JPRB, 0.14206E+01_JPRB, 0.14193E+01_JPRB, 0.18442E+01_JPRB, 0.29099E+01_JPRB, 0.40990E+01_JPRB, 0.46427E+01_JPRB ,&
&  0.47861E+01_JPRB, 0.47285E+01_JPRB, 0.47374E+01_JPRB, 0.52558E+01_JPRB, 0.50426E+01_JPRB, 0.45454E+01_JPRB, 0.42711E+01_JPRB ,&
&  0.40970E+01_JPRB, 0.40025E+01_JPRB, 0.40728E+01_JPRB, 0.38575E+01_JPRB, 0.36083E+01_JPRB, 0.34600E+01_JPRB, 0.33418E+01_JPRB ,&
&  0.32033E+01_JPRB, 0.31845E+01_JPRB, 0.31653E+01_JPRB, 0.29982E+01_JPRB, 0.29229E+01_JPRB, 0.28686E+01_JPRB, 0.31393E+01_JPRB ,&
&  0.35208E+01_JPRB, 0.27731E+01_JPRB, 0.23513E+01_JPRB, 0.25704E+01_JPRB, 0.20192E+01_JPRB, 0.19515E+01_JPRB, 0.21542E+01_JPRB ,&
&  0.20263E+01_JPRB, 0.20277E+01_JPRB, 0.22203E+01_JPRB, 0.22057E+01_JPRB, 0.23412E+01_JPRB, 0.26101E+01_JPRB, 0.25934E+01_JPRB ,&
&  0.24124E+01_JPRB, 0.22889E+01_JPRB, 0.27724E+01_JPRB, 0.25034E+01_JPRB, 0.22490E+01_JPRB, 0.21284E+01_JPRB, 0.20182E+01_JPRB ,&
&  0.19493E+01_JPRB, 0.18829E+01_JPRB, 0.18062E+01_JPRB, 0.17555E+01_JPRB, 0.17430E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.16918E+01_JPRB, 0.15638E+01_JPRB, 0.16037E+01_JPRB, 0.15902E+01_JPRB, 0.15746E+01_JPRB, 0.15621E+01_JPRB, 0.15583E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.15257E+01_JPRB, 0.14969E+01_JPRB, 0.14445E+01_JPRB, 0.14065E+01_JPRB, 0.13744E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.13375E+01_JPRB, 0.12807E+01_JPRB, 0.12619E+01_JPRB, 0.12298E+01_JPRB, 0.12716E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.13956E+01_JPRB, 0.14849E+01_JPRB, 0.15862E+01_JPRB, 0.16976E+01_JPRB, 0.17921E+01_JPRB, 0.18624E+01_JPRB, 0.19895E+01_JPRB ,&
&  0.20191E+01_JPRB, 0.19149E+01_JPRB, 0.18041E+01_JPRB, 0.18238E+01_JPRB, 0.18122E+01_JPRB, 0.16770E+01_JPRB, 0.16523E+01_JPRB ,&
&  0.15916E+01_JPRB, 0.15518E+01_JPRB, 0.15309E+01_JPRB, 0.15525E+01_JPRB, 0.15509E+01_JPRB, 0.14842E+01_JPRB, 0.14167E+01_JPRB ,&
&  0.13822E+01_JPRB, 0.14266E+01_JPRB, 0.17430E+01_JPRB, 0.24853E+01_JPRB, 0.37155E+01_JPRB, 0.51096E+01_JPRB, 0.54912E+01_JPRB ,&
&  0.55474E+01_JPRB, 0.50510E+01_JPRB, 0.40236E+01_JPRB, 0.44844E+01_JPRB, 0.55237E+01_JPRB, 0.52224E+01_JPRB, 0.47179E+01_JPRB ,&
&  0.46792E+01_JPRB, 0.43738E+01_JPRB, 0.42467E+01_JPRB, 0.38657E+01_JPRB, 0.35597E+01_JPRB, 0.32913E+01_JPRB, 0.30612E+01_JPRB ,&
&  0.28796E+01_JPRB, 0.28237E+01_JPRB, 0.28202E+01_JPRB, 0.28587E+01_JPRB, 0.28476E+01_JPRB, 0.30514E+01_JPRB, 0.34435E+01_JPRB ,&
&  0.34669E+01_JPRB, 0.32273E+01_JPRB, 0.25353E+01_JPRB, 0.24579E+01_JPRB, 0.19139E+01_JPRB, 0.18500E+01_JPRB, 0.20280E+01_JPRB ,&
&  0.20126E+01_JPRB, 0.23360E+01_JPRB, 0.28218E+01_JPRB, 0.29296E+01_JPRB, 0.34149E+01_JPRB, 0.35549E+01_JPRB, 0.34540E+01_JPRB ,&
&  0.32572E+01_JPRB, 0.29379E+01_JPRB, 0.34328E+01_JPRB, 0.29938E+01_JPRB, 0.25912E+01_JPRB, 0.24219E+01_JPRB, 0.22984E+01_JPRB ,&
&  0.22096E+01_JPRB, 0.20871E+01_JPRB, 0.19562E+01_JPRB, 0.18719E+01_JPRB, 0.18053E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.17025E+01_JPRB, 0.16183E+01_JPRB, 0.16573E+01_JPRB, 0.16202E+01_JPRB, 0.16324E+01_JPRB, 0.16036E+01_JPRB, 0.16054E+01_JPRB ,&
&  0.15917E+01_JPRB, 0.15507E+01_JPRB, 0.14941E+01_JPRB, 0.14431E+01_JPRB, 0.14070E+01_JPRB, 0.13752E+01_JPRB, 0.13613E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.13298E+01_JPRB, 0.12394E+01_JPRB, 0.11925E+01_JPRB, 0.11744E+01_JPRB, 0.12414E+01_JPRB, 0.13724E+01_JPRB ,&
&  0.15530E+01_JPRB, 0.17408E+01_JPRB, 0.19961E+01_JPRB, 0.21603E+01_JPRB, 0.22789E+01_JPRB, 0.23707E+01_JPRB, 0.24905E+01_JPRB ,&
&  0.25893E+01_JPRB, 0.22510E+01_JPRB, 0.21446E+01_JPRB, 0.19771E+01_JPRB, 0.18915E+01_JPRB, 0.18098E+01_JPRB, 0.17245E+01_JPRB ,&
&  0.16580E+01_JPRB, 0.16260E+01_JPRB, 0.15922E+01_JPRB, 0.15242E+01_JPRB, 0.14508E+01_JPRB, 0.14035E+01_JPRB, 0.13976E+01_JPRB ,&
&  0.14583E+01_JPRB, 0.16988E+01_JPRB, 0.24284E+01_JPRB, 0.34914E+01_JPRB, 0.51124E+01_JPRB, 0.65111E+01_JPRB, 0.64797E+01_JPRB ,&
&  0.51043E+01_JPRB, 0.45507E+01_JPRB, 0.59795E+01_JPRB, 0.52692E+01_JPRB, 0.53839E+01_JPRB, 0.57517E+01_JPRB, 0.53377E+01_JPRB ,&
&  0.46745E+01_JPRB, 0.39511E+01_JPRB, 0.38516E+01_JPRB, 0.36380E+01_JPRB, 0.45037E+01_JPRB, 0.38688E+01_JPRB, 0.33993E+01_JPRB ,&
&  0.31173E+01_JPRB, 0.30766E+01_JPRB, 0.30145E+01_JPRB, 0.30950E+01_JPRB, 0.34022E+01_JPRB, 0.40064E+01_JPRB, 0.41657E+01_JPRB ,&
&  0.37781E+01_JPRB, 0.37944E+01_JPRB, 0.31837E+01_JPRB, 0.26224E+01_JPRB, 0.22557E+01_JPRB, 0.22260E+01_JPRB, 0.22735E+01_JPRB ,&
&  0.24596E+01_JPRB, 0.27501E+01_JPRB, 0.35397E+01_JPRB, 0.43779E+01_JPRB, 0.50518E+01_JPRB, 0.49854E+01_JPRB, 0.42807E+01_JPRB ,&
&  0.40407E+01_JPRB, 0.45713E+01_JPRB, 0.45079E+01_JPRB, 0.35056E+01_JPRB, 0.29008E+01_JPRB, 0.26963E+01_JPRB, 0.25387E+01_JPRB ,&
&  0.23279E+01_JPRB, 0.21156E+01_JPRB, 0.19618E+01_JPRB, 0.18771E+01_JPRB, 0.18282E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.18000E+01_JPRB, 0.17191E+01_JPRB, 0.16299E+01_JPRB, 0.15948E+01_JPRB, 0.16177E+01_JPRB, 0.15764E+01_JPRB, 0.15423E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.14425E+01_JPRB, 0.14299E+01_JPRB, 0.14274E+01_JPRB, 0.14116E+01_JPRB, 0.13841E+01_JPRB, 0.13700E+01_JPRB ,&
&  0.13519E+01_JPRB, 0.12791E+01_JPRB, 0.11970E+01_JPRB, 0.12057E+01_JPRB, 0.12073E+01_JPRB, 0.12276E+01_JPRB, 0.14140E+01_JPRB ,&
&  0.17276E+01_JPRB, 0.21177E+01_JPRB, 0.24244E+01_JPRB, 0.25632E+01_JPRB, 0.25901E+01_JPRB, 0.26339E+01_JPRB, 0.27027E+01_JPRB ,&
&  0.25213E+01_JPRB, 0.24618E+01_JPRB, 0.23551E+01_JPRB, 0.21512E+01_JPRB, 0.19856E+01_JPRB, 0.18383E+01_JPRB, 0.17546E+01_JPRB ,&
&  0.16982E+01_JPRB, 0.16773E+01_JPRB, 0.16004E+01_JPRB, 0.15083E+01_JPRB, 0.15037E+01_JPRB, 0.15097E+01_JPRB, 0.15187E+01_JPRB ,&
&  0.15950E+01_JPRB, 0.21125E+01_JPRB, 0.30386E+01_JPRB, 0.42368E+01_JPRB, 0.46160E+01_JPRB, 0.59036E+01_JPRB, 0.59013E+01_JPRB ,&
&  0.66938E+01_JPRB, 0.63372E+01_JPRB, 0.59825E+01_JPRB, 0.64636E+01_JPRB, 0.53971E+01_JPRB, 0.54678E+01_JPRB, 0.53190E+01_JPRB ,&
&  0.47043E+01_JPRB, 0.44447E+01_JPRB, 0.48584E+01_JPRB, 0.44247E+01_JPRB, 0.46961E+01_JPRB, 0.52350E+01_JPRB, 0.46490E+01_JPRB ,&
&  0.43801E+01_JPRB, 0.41567E+01_JPRB, 0.38944E+01_JPRB, 0.40976E+01_JPRB, 0.41044E+01_JPRB, 0.42382E+01_JPRB, 0.36565E+01_JPRB ,&
&  0.34972E+01_JPRB, 0.29708E+01_JPRB, 0.31845E+01_JPRB, 0.28746E+01_JPRB, 0.28073E+01_JPRB, 0.27198E+01_JPRB, 0.27213E+01_JPRB ,&
&  0.31126E+01_JPRB, 0.34737E+01_JPRB, 0.46722E+01_JPRB, 0.59252E+01_JPRB, 0.70361E+01_JPRB, 0.60860E+01_JPRB, 0.53983E+01_JPRB ,&
&  0.57055E+01_JPRB, 0.59001E+01_JPRB, 0.48658E+01_JPRB, 0.37809E+01_JPRB, 0.31787E+01_JPRB, 0.29408E+01_JPRB, 0.27664E+01_JPRB ,&
&  0.25572E+01_JPRB, 0.23501E+01_JPRB, 0.21850E+01_JPRB, 0.20624E+01_JPRB, 0.19629E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.16832E+01_JPRB, 0.16308E+01_JPRB, 0.15254E+01_JPRB, 0.15197E+01_JPRB, 0.15624E+01_JPRB, 0.15268E+01_JPRB, 0.14919E+01_JPRB ,&
&  0.14354E+01_JPRB, 0.13992E+01_JPRB, 0.14116E+01_JPRB, 0.14220E+01_JPRB, 0.14093E+01_JPRB, 0.13945E+01_JPRB, 0.13668E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.12944E+01_JPRB, 0.12224E+01_JPRB, 0.12219E+01_JPRB, 0.12821E+01_JPRB, 0.13362E+01_JPRB, 0.15052E+01_JPRB ,&
&  0.21237E+01_JPRB, 0.26169E+01_JPRB, 0.27670E+01_JPRB, 0.27428E+01_JPRB, 0.26584E+01_JPRB, 0.26177E+01_JPRB, 0.25385E+01_JPRB ,&
&  0.24362E+01_JPRB, 0.22508E+01_JPRB, 0.21257E+01_JPRB, 0.20254E+01_JPRB, 0.19275E+01_JPRB, 0.18322E+01_JPRB, 0.17494E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.15422E+01_JPRB, 0.14210E+01_JPRB, 0.13481E+01_JPRB, 0.13749E+01_JPRB, 0.13679E+01_JPRB, 0.14510E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.23593E+01_JPRB, 0.30812E+01_JPRB, 0.39824E+01_JPRB, 0.46368E+01_JPRB, 0.59400E+01_JPRB, 0.66949E+01_JPRB ,&
&  0.64443E+01_JPRB, 0.58116E+01_JPRB, 0.63963E+01_JPRB, 0.70311E+01_JPRB, 0.68241E+01_JPRB, 0.58941E+01_JPRB, 0.55555E+01_JPRB ,&
&  0.42249E+01_JPRB, 0.34786E+01_JPRB, 0.33240E+01_JPRB, 0.37862E+01_JPRB, 0.55852E+01_JPRB, 0.72455E+01_JPRB, 0.68918E+01_JPRB ,&
&  0.67131E+01_JPRB, 0.62898E+01_JPRB, 0.57821E+01_JPRB, 0.50333E+01_JPRB, 0.46108E+01_JPRB, 0.42680E+01_JPRB, 0.52918E+01_JPRB ,&
&  0.51611E+01_JPRB, 0.50984E+01_JPRB, 0.43047E+01_JPRB, 0.34830E+01_JPRB, 0.29297E+01_JPRB, 0.32418E+01_JPRB, 0.34376E+01_JPRB ,&
&  0.39025E+01_JPRB, 0.45353E+01_JPRB, 0.78067E+01_JPRB, 0.90407E+01_JPRB, 0.87680E+01_JPRB, 0.78520E+01_JPRB, 0.77095E+01_JPRB ,&
&  0.70288E+01_JPRB, 0.67744E+01_JPRB, 0.54618E+01_JPRB, 0.40116E+01_JPRB, 0.31623E+01_JPRB, 0.26943E+01_JPRB, 0.24358E+01_JPRB ,&
&  0.23463E+01_JPRB, 0.22585E+01_JPRB, 0.21216E+01_JPRB, 0.19547E+01_JPRB, 0.18306E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.15701E+01_JPRB, 0.15225E+01_JPRB, 0.14137E+01_JPRB, 0.13994E+01_JPRB, 0.14441E+01_JPRB, 0.14772E+01_JPRB, 0.14565E+01_JPRB ,&
&  0.14292E+01_JPRB, 0.14382E+01_JPRB, 0.14586E+01_JPRB, 0.14424E+01_JPRB, 0.14153E+01_JPRB, 0.14050E+01_JPRB, 0.13832E+01_JPRB ,&
&  0.13849E+01_JPRB, 0.14229E+01_JPRB, 0.13499E+01_JPRB, 0.14013E+01_JPRB, 0.15164E+01_JPRB, 0.16638E+01_JPRB, 0.18987E+01_JPRB ,&
&  0.27227E+01_JPRB, 0.29208E+01_JPRB, 0.26783E+01_JPRB, 0.25956E+01_JPRB, 0.26228E+01_JPRB, 0.23404E+01_JPRB, 0.22375E+01_JPRB ,&
&  0.21921E+01_JPRB, 0.20363E+01_JPRB, 0.19417E+01_JPRB, 0.18678E+01_JPRB, 0.18363E+01_JPRB, 0.18092E+01_JPRB, 0.17229E+01_JPRB ,&
&  0.15874E+01_JPRB, 0.14342E+01_JPRB, 0.13583E+01_JPRB, 0.13514E+01_JPRB, 0.12933E+01_JPRB, 0.13107E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.18443E+01_JPRB, 0.24175E+01_JPRB, 0.28434E+01_JPRB, 0.34248E+01_JPRB, 0.42017E+01_JPRB, 0.56493E+01_JPRB, 0.77825E+01_JPRB ,&
&  0.71285E+01_JPRB, 0.69623E+01_JPRB, 0.68393E+01_JPRB, 0.75341E+01_JPRB, 0.66453E+01_JPRB, 0.57829E+01_JPRB, 0.53212E+01_JPRB ,&
&  0.39977E+01_JPRB, 0.34565E+01_JPRB, 0.39523E+01_JPRB, 0.50612E+01_JPRB, 0.56448E+01_JPRB, 0.65729E+01_JPRB, 0.82550E+01_JPRB ,&
&  0.74017E+01_JPRB, 0.71530E+01_JPRB, 0.68559E+01_JPRB, 0.64841E+01_JPRB, 0.39468E+01_JPRB, 0.32855E+01_JPRB, 0.47811E+01_JPRB ,&
&  0.49949E+01_JPRB, 0.32816E+01_JPRB, 0.21968E+01_JPRB, 0.27321E+01_JPRB, 0.20467E+01_JPRB, 0.23568E+01_JPRB, 0.34695E+01_JPRB ,&
&  0.47406E+01_JPRB, 0.65444E+01_JPRB, 0.11424E+02_JPRB, 0.10430E+02_JPRB, 0.98268E+01_JPRB, 0.11165E+02_JPRB, 0.91913E+01_JPRB ,&
&  0.89272E+01_JPRB, 0.63225E+01_JPRB, 0.49698E+01_JPRB, 0.34451E+01_JPRB, 0.24393E+01_JPRB, 0.20073E+01_JPRB, 0.18692E+01_JPRB ,&
&  0.18226E+01_JPRB, 0.18333E+01_JPRB, 0.18310E+01_JPRB, 0.17531E+01_JPRB, 0.16553E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.16179E+01_JPRB, 0.14904E+01_JPRB, 0.13249E+01_JPRB, 0.13036E+01_JPRB, 0.13366E+01_JPRB, 0.13884E+01_JPRB, 0.13872E+01_JPRB ,&
&  0.14092E+01_JPRB, 0.14618E+01_JPRB, 0.15457E+01_JPRB, 0.15345E+01_JPRB, 0.14751E+01_JPRB, 0.14555E+01_JPRB, 0.14228E+01_JPRB ,&
&  0.14348E+01_JPRB, 0.15304E+01_JPRB, 0.16913E+01_JPRB, 0.18905E+01_JPRB, 0.21215E+01_JPRB, 0.22543E+01_JPRB, 0.27985E+01_JPRB ,&
&  0.32108E+01_JPRB, 0.28397E+01_JPRB, 0.25314E+01_JPRB, 0.24577E+01_JPRB, 0.23167E+01_JPRB, 0.20038E+01_JPRB, 0.20045E+01_JPRB ,&
&  0.19657E+01_JPRB, 0.18809E+01_JPRB, 0.18137E+01_JPRB, 0.17540E+01_JPRB, 0.17539E+01_JPRB, 0.17535E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.15422E+01_JPRB, 0.14567E+01_JPRB, 0.13973E+01_JPRB, 0.12994E+01_JPRB, 0.12494E+01_JPRB, 0.12859E+01_JPRB, 0.14512E+01_JPRB ,&
&  0.18396E+01_JPRB, 0.23026E+01_JPRB, 0.25843E+01_JPRB, 0.29961E+01_JPRB, 0.46073E+01_JPRB, 0.58434E+01_JPRB, 0.59814E+01_JPRB ,&
&  0.58009E+01_JPRB, 0.58570E+01_JPRB, 0.73898E+01_JPRB, 0.72775E+01_JPRB, 0.67611E+01_JPRB, 0.64920E+01_JPRB, 0.55814E+01_JPRB ,&
&  0.49307E+01_JPRB, 0.45582E+01_JPRB, 0.49494E+01_JPRB, 0.58536E+01_JPRB, 0.68801E+01_JPRB, 0.69415E+01_JPRB, 0.84729E+01_JPRB ,&
&  0.86165E+01_JPRB, 0.81176E+01_JPRB, 0.63973E+01_JPRB, 0.48984E+01_JPRB, 0.44608E+01_JPRB, 0.34624E+01_JPRB, 0.24602E+01_JPRB ,&
&  0.21866E+01_JPRB, 0.21100E+01_JPRB, 0.19104E+01_JPRB, 0.19380E+01_JPRB, 0.17667E+01_JPRB, 0.21292E+01_JPRB, 0.31887E+01_JPRB ,&
&  0.62431E+01_JPRB, 0.96611E+01_JPRB, 0.11499E+02_JPRB, 0.97854E+01_JPRB, 0.90575E+01_JPRB, 0.11185E+02_JPRB, 0.11247E+02_JPRB ,&
&  0.67735E+01_JPRB, 0.48850E+01_JPRB, 0.32950E+01_JPRB, 0.23003E+01_JPRB, 0.18694E+01_JPRB, 0.16579E+01_JPRB, 0.16024E+01_JPRB ,&
&  0.15968E+01_JPRB, 0.15837E+01_JPRB, 0.16052E+01_JPRB, 0.16365E+01_JPRB, 0.16124E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.15907E+01_JPRB, 0.14117E+01_JPRB, 0.12418E+01_JPRB, 0.12000E+01_JPRB, 0.12592E+01_JPRB, 0.12925E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.14102E+01_JPRB, 0.15322E+01_JPRB, 0.16234E+01_JPRB, 0.16063E+01_JPRB, 0.15538E+01_JPRB, 0.15109E+01_JPRB, 0.14997E+01_JPRB ,&
&  0.15837E+01_JPRB, 0.18203E+01_JPRB, 0.22577E+01_JPRB, 0.27188E+01_JPRB, 0.31705E+01_JPRB, 0.31065E+01_JPRB, 0.35611E+01_JPRB ,&
&  0.31694E+01_JPRB, 0.26313E+01_JPRB, 0.25310E+01_JPRB, 0.23920E+01_JPRB, 0.22026E+01_JPRB, 0.19398E+01_JPRB, 0.18823E+01_JPRB ,&
&  0.18024E+01_JPRB, 0.18099E+01_JPRB, 0.17504E+01_JPRB, 0.17603E+01_JPRB, 0.17608E+01_JPRB, 0.17223E+01_JPRB, 0.16196E+01_JPRB ,&
&  0.15064E+01_JPRB, 0.14876E+01_JPRB, 0.13491E+01_JPRB, 0.13365E+01_JPRB, 0.13382E+01_JPRB, 0.14219E+01_JPRB, 0.16054E+01_JPRB ,&
&  0.19345E+01_JPRB, 0.23007E+01_JPRB, 0.26328E+01_JPRB, 0.35052E+01_JPRB, 0.43338E+01_JPRB, 0.42319E+01_JPRB, 0.52877E+01_JPRB ,&
&  0.61653E+01_JPRB, 0.67780E+01_JPRB, 0.67671E+01_JPRB, 0.75934E+01_JPRB, 0.72466E+01_JPRB, 0.65744E+01_JPRB, 0.60087E+01_JPRB ,&
&  0.52794E+01_JPRB, 0.57084E+01_JPRB, 0.71914E+01_JPRB, 0.79413E+01_JPRB, 0.97294E+01_JPRB, 0.93047E+01_JPRB, 0.97937E+01_JPRB ,&
&  0.10141E+02_JPRB, 0.92740E+01_JPRB, 0.74979E+01_JPRB, 0.60321E+01_JPRB, 0.65677E+01_JPRB, 0.63220E+01_JPRB, 0.35718E+01_JPRB ,&
&  0.29950E+01_JPRB, 0.26306E+01_JPRB, 0.21925E+01_JPRB, 0.19120E+01_JPRB, 0.18679E+01_JPRB, 0.23154E+01_JPRB, 0.41729E+01_JPRB ,&
&  0.56305E+01_JPRB, 0.10157E+02_JPRB, 0.11967E+02_JPRB, 0.10230E+02_JPRB, 0.87068E+01_JPRB, 0.10211E+02_JPRB, 0.72354E+01_JPRB ,&
&  0.59080E+01_JPRB, 0.40425E+01_JPRB, 0.23390E+01_JPRB, 0.17992E+01_JPRB, 0.16866E+01_JPRB, 0.16091E+01_JPRB, 0.15280E+01_JPRB ,&
&  0.14375E+01_JPRB, 0.14301E+01_JPRB, 0.14743E+01_JPRB, 0.15126E+01_JPRB, 0.15156E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.13976E+01_JPRB, 0.12569E+01_JPRB, 0.11620E+01_JPRB, 0.11028E+01_JPRB, 0.11802E+01_JPRB, 0.12956E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.14730E+01_JPRB, 0.16406E+01_JPRB, 0.17300E+01_JPRB, 0.17237E+01_JPRB, 0.16356E+01_JPRB, 0.16229E+01_JPRB, 0.17277E+01_JPRB ,&
&  0.19900E+01_JPRB, 0.24389E+01_JPRB, 0.30766E+01_JPRB, 0.37572E+01_JPRB, 0.40531E+01_JPRB, 0.41833E+01_JPRB, 0.38818E+01_JPRB ,&
&  0.29840E+01_JPRB, 0.25467E+01_JPRB, 0.25318E+01_JPRB, 0.24536E+01_JPRB, 0.21543E+01_JPRB, 0.20035E+01_JPRB, 0.18096E+01_JPRB ,&
&  0.17782E+01_JPRB, 0.18078E+01_JPRB, 0.17941E+01_JPRB, 0.18074E+01_JPRB, 0.17678E+01_JPRB, 0.17030E+01_JPRB, 0.16394E+01_JPRB ,&
&  0.15619E+01_JPRB, 0.14350E+01_JPRB, 0.13711E+01_JPRB, 0.13837E+01_JPRB, 0.15237E+01_JPRB, 0.17501E+01_JPRB, 0.19990E+01_JPRB ,&
&  0.23059E+01_JPRB, 0.26624E+01_JPRB, 0.31671E+01_JPRB, 0.40841E+01_JPRB, 0.37185E+01_JPRB, 0.46869E+01_JPRB, 0.53770E+01_JPRB ,&
&  0.57085E+01_JPRB, 0.59832E+01_JPRB, 0.58552E+01_JPRB, 0.65827E+01_JPRB, 0.69992E+01_JPRB, 0.65225E+01_JPRB, 0.58895E+01_JPRB ,&
&  0.61629E+01_JPRB, 0.67142E+01_JPRB, 0.95439E+01_JPRB, 0.10998E+02_JPRB, 0.12113E+02_JPRB, 0.15412E+02_JPRB, 0.11457E+02_JPRB ,&
&  0.97391E+01_JPRB, 0.10129E+02_JPRB, 0.86075E+01_JPRB, 0.71511E+01_JPRB, 0.72516E+01_JPRB, 0.88111E+01_JPRB, 0.10098E+02_JPRB ,&
&  0.45597E+01_JPRB, 0.28205E+01_JPRB, 0.27978E+01_JPRB, 0.25313E+01_JPRB, 0.21502E+01_JPRB, 0.25450E+01_JPRB, 0.67305E+01_JPRB ,&
&  0.57077E+01_JPRB, 0.94505E+01_JPRB, 0.10786E+02_JPRB, 0.92570E+01_JPRB, 0.83460E+01_JPRB, 0.78464E+01_JPRB, 0.62510E+01_JPRB ,&
&  0.44306E+01_JPRB, 0.29658E+01_JPRB, 0.20161E+01_JPRB, 0.16705E+01_JPRB, 0.15984E+01_JPRB, 0.15559E+01_JPRB, 0.14842E+01_JPRB ,&
&  0.14164E+01_JPRB, 0.13726E+01_JPRB, 0.13995E+01_JPRB, 0.13625E+01_JPRB, 0.14223E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.12772E+01_JPRB, 0.12078E+01_JPRB, 0.12157E+01_JPRB, 0.12268E+01_JPRB, 0.12889E+01_JPRB, 0.13562E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.18286E+01_JPRB, 0.19752E+01_JPRB, 0.19735E+01_JPRB, 0.19095E+01_JPRB, 0.20374E+01_JPRB, 0.23435E+01_JPRB ,&
&  0.27668E+01_JPRB, 0.32886E+01_JPRB, 0.38840E+01_JPRB, 0.42836E+01_JPRB, 0.44967E+01_JPRB, 0.41126E+01_JPRB, 0.35977E+01_JPRB ,&
&  0.28697E+01_JPRB, 0.24723E+01_JPRB, 0.23072E+01_JPRB, 0.23134E+01_JPRB, 0.22560E+01_JPRB, 0.20576E+01_JPRB, 0.18005E+01_JPRB ,&
&  0.17636E+01_JPRB, 0.17641E+01_JPRB, 0.17480E+01_JPRB, 0.17651E+01_JPRB, 0.17804E+01_JPRB, 0.17869E+01_JPRB, 0.17327E+01_JPRB ,&
&  0.15960E+01_JPRB, 0.15314E+01_JPRB, 0.15113E+01_JPRB, 0.16362E+01_JPRB, 0.19462E+01_JPRB, 0.22916E+01_JPRB, 0.26213E+01_JPRB ,&
&  0.29644E+01_JPRB, 0.33132E+01_JPRB, 0.37621E+01_JPRB, 0.40571E+01_JPRB, 0.45293E+01_JPRB, 0.49370E+01_JPRB, 0.52263E+01_JPRB ,&
&  0.52573E+01_JPRB, 0.55943E+01_JPRB, 0.59099E+01_JPRB, 0.61747E+01_JPRB, 0.64495E+01_JPRB, 0.63243E+01_JPRB, 0.61515E+01_JPRB ,&
&  0.71440E+01_JPRB, 0.90605E+01_JPRB, 0.11392E+02_JPRB, 0.12392E+02_JPRB, 0.12959E+02_JPRB, 0.15601E+02_JPRB, 0.15385E+02_JPRB ,&
&  0.12119E+02_JPRB, 0.90436E+01_JPRB, 0.79971E+01_JPRB, 0.83705E+01_JPRB, 0.89225E+01_JPRB, 0.10774E+02_JPRB, 0.16605E+02_JPRB ,&
&  0.16714E+02_JPRB, 0.74421E+01_JPRB, 0.47679E+01_JPRB, 0.53408E+01_JPRB, 0.29880E+01_JPRB, 0.25161E+01_JPRB, 0.41909E+01_JPRB ,&
&  0.57403E+01_JPRB, 0.84672E+01_JPRB, 0.99154E+01_JPRB, 0.79008E+01_JPRB, 0.71298E+01_JPRB, 0.54889E+01_JPRB, 0.41203E+01_JPRB ,&
&  0.31125E+01_JPRB, 0.22098E+01_JPRB, 0.16903E+01_JPRB, 0.15410E+01_JPRB, 0.14471E+01_JPRB, 0.14564E+01_JPRB, 0.14925E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.13176E+01_JPRB, 0.13242E+01_JPRB, 0.13231E+01_JPRB, 0.12890E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.13091E+01_JPRB, 0.13072E+01_JPRB, 0.13648E+01_JPRB, 0.14367E+01_JPRB, 0.15133E+01_JPRB, 0.16084E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.19226E+01_JPRB, 0.21658E+01_JPRB, 0.23229E+01_JPRB, 0.24250E+01_JPRB, 0.25517E+01_JPRB, 0.28095E+01_JPRB, 0.31529E+01_JPRB ,&
&  0.35530E+01_JPRB, 0.39632E+01_JPRB, 0.43593E+01_JPRB, 0.47840E+01_JPRB, 0.53612E+01_JPRB, 0.56789E+01_JPRB, 0.33554E+01_JPRB ,&
&  0.27417E+01_JPRB, 0.27445E+01_JPRB, 0.23169E+01_JPRB, 0.24555E+01_JPRB, 0.25188E+01_JPRB, 0.24666E+01_JPRB, 0.19925E+01_JPRB ,&
&  0.17518E+01_JPRB, 0.17280E+01_JPRB, 0.17263E+01_JPRB, 0.17751E+01_JPRB, 0.18365E+01_JPRB, 0.19747E+01_JPRB, 0.20009E+01_JPRB ,&
&  0.19956E+01_JPRB, 0.20515E+01_JPRB, 0.21462E+01_JPRB, 0.24487E+01_JPRB, 0.28675E+01_JPRB, 0.32021E+01_JPRB, 0.34348E+01_JPRB ,&
&  0.36276E+01_JPRB, 0.38604E+01_JPRB, 0.42022E+01_JPRB, 0.44432E+01_JPRB, 0.49198E+01_JPRB, 0.51534E+01_JPRB, 0.53147E+01_JPRB ,&
&  0.51431E+01_JPRB, 0.49174E+01_JPRB, 0.54122E+01_JPRB, 0.58175E+01_JPRB, 0.60172E+01_JPRB, 0.61213E+01_JPRB, 0.66970E+01_JPRB ,&
&  0.89403E+01_JPRB, 0.11550E+02_JPRB, 0.14374E+02_JPRB, 0.12935E+02_JPRB, 0.12411E+02_JPRB, 0.13756E+02_JPRB, 0.14976E+02_JPRB ,&
&  0.12669E+02_JPRB, 0.10270E+02_JPRB, 0.99076E+01_JPRB, 0.99337E+01_JPRB, 0.10562E+02_JPRB, 0.11424E+02_JPRB, 0.14712E+02_JPRB ,&
&  0.17102E+02_JPRB, 0.13096E+02_JPRB, 0.75529E+01_JPRB, 0.40839E+01_JPRB, 0.28703E+01_JPRB, 0.29232E+01_JPRB, 0.37526E+01_JPRB ,&
&  0.60351E+01_JPRB, 0.74906E+01_JPRB, 0.88428E+01_JPRB, 0.89847E+01_JPRB, 0.49266E+01_JPRB, 0.32523E+01_JPRB, 0.25320E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.14485E+01_JPRB, 0.14173E+01_JPRB, 0.13741E+01_JPRB, 0.13091E+01_JPRB, 0.13726E+01_JPRB, 0.15031E+01_JPRB ,&
&  0.14336E+01_JPRB, 0.12753E+01_JPRB, 0.12832E+01_JPRB, 0.12773E+01_JPRB, 0.12776E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.13657E+01_JPRB, 0.14061E+01_JPRB, 0.14910E+01_JPRB, 0.16266E+01_JPRB, 0.17981E+01_JPRB, 0.19836E+01_JPRB, 0.20803E+01_JPRB ,&
&  0.22135E+01_JPRB, 0.24324E+01_JPRB, 0.26410E+01_JPRB, 0.28472E+01_JPRB, 0.30596E+01_JPRB, 0.33111E+01_JPRB, 0.35845E+01_JPRB ,&
&  0.38810E+01_JPRB, 0.41639E+01_JPRB, 0.44526E+01_JPRB, 0.48364E+01_JPRB, 0.53377E+01_JPRB, 0.56170E+01_JPRB, 0.41594E+01_JPRB ,&
&  0.27652E+01_JPRB, 0.38130E+01_JPRB, 0.25718E+01_JPRB, 0.28146E+01_JPRB, 0.30035E+01_JPRB, 0.32788E+01_JPRB, 0.25621E+01_JPRB ,&
&  0.19089E+01_JPRB, 0.18442E+01_JPRB, 0.18618E+01_JPRB, 0.18549E+01_JPRB, 0.18209E+01_JPRB, 0.20791E+01_JPRB, 0.23497E+01_JPRB ,&
&  0.24812E+01_JPRB, 0.26579E+01_JPRB, 0.29153E+01_JPRB, 0.32768E+01_JPRB, 0.35545E+01_JPRB, 0.37828E+01_JPRB, 0.39644E+01_JPRB ,&
&  0.41417E+01_JPRB, 0.43963E+01_JPRB, 0.46109E+01_JPRB, 0.49162E+01_JPRB, 0.50705E+01_JPRB, 0.53307E+01_JPRB, 0.54800E+01_JPRB ,&
&  0.53115E+01_JPRB, 0.50989E+01_JPRB, 0.53474E+01_JPRB, 0.54313E+01_JPRB, 0.58774E+01_JPRB, 0.71790E+01_JPRB, 0.82145E+01_JPRB ,&
&  0.10683E+02_JPRB, 0.12052E+02_JPRB, 0.14578E+02_JPRB, 0.11824E+02_JPRB, 0.10858E+02_JPRB, 0.11515E+02_JPRB, 0.11030E+02_JPRB ,&
&  0.97788E+01_JPRB, 0.10105E+02_JPRB, 0.10228E+02_JPRB, 0.10651E+02_JPRB, 0.11201E+02_JPRB, 0.11804E+02_JPRB, 0.13042E+02_JPRB ,&
&  0.11184E+02_JPRB, 0.59068E+01_JPRB, 0.37687E+01_JPRB, 0.28879E+01_JPRB, 0.24565E+01_JPRB, 0.27556E+01_JPRB, 0.43176E+01_JPRB ,&
&  0.67404E+01_JPRB, 0.74930E+01_JPRB, 0.83268E+01_JPRB, 0.59715E+01_JPRB, 0.27943E+01_JPRB, 0.19000E+01_JPRB, 0.14556E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.10623E+01_JPRB, 0.11640E+01_JPRB, 0.12001E+01_JPRB, 0.11666E+01_JPRB, 0.12227E+01_JPRB, 0.12899E+01_JPRB ,&
&  0.12679E+01_JPRB, 0.12487E+01_JPRB, 0.13191E+01_JPRB, 0.13074E+01_JPRB, 0.13255E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.13953E+01_JPRB, 0.14599E+01_JPRB, 0.16064E+01_JPRB, 0.18086E+01_JPRB, 0.21051E+01_JPRB, 0.24466E+01_JPRB, 0.26477E+01_JPRB ,&
&  0.25626E+01_JPRB, 0.24716E+01_JPRB, 0.27745E+01_JPRB, 0.30570E+01_JPRB, 0.32685E+01_JPRB, 0.35268E+01_JPRB, 0.37962E+01_JPRB ,&
&  0.41134E+01_JPRB, 0.44457E+01_JPRB, 0.48179E+01_JPRB, 0.52779E+01_JPRB, 0.58158E+01_JPRB, 0.63303E+01_JPRB, 0.61940E+01_JPRB ,&
&  0.43690E+01_JPRB, 0.31967E+01_JPRB, 0.31675E+01_JPRB, 0.29034E+01_JPRB, 0.28402E+01_JPRB, 0.31374E+01_JPRB, 0.31254E+01_JPRB ,&
&  0.25311E+01_JPRB, 0.21402E+01_JPRB, 0.21583E+01_JPRB, 0.21546E+01_JPRB, 0.16404E+01_JPRB, 0.18963E+01_JPRB, 0.23296E+01_JPRB ,&
&  0.25945E+01_JPRB, 0.28388E+01_JPRB, 0.31574E+01_JPRB, 0.35023E+01_JPRB, 0.37533E+01_JPRB, 0.40602E+01_JPRB, 0.43404E+01_JPRB ,&
&  0.45628E+01_JPRB, 0.48210E+01_JPRB, 0.49576E+01_JPRB, 0.50015E+01_JPRB, 0.52160E+01_JPRB, 0.54658E+01_JPRB, 0.55552E+01_JPRB ,&
&  0.55776E+01_JPRB, 0.58316E+01_JPRB, 0.61547E+01_JPRB, 0.65590E+01_JPRB, 0.69141E+01_JPRB, 0.83837E+01_JPRB, 0.95329E+01_JPRB ,&
&  0.10792E+02_JPRB, 0.11197E+02_JPRB, 0.12402E+02_JPRB, 0.99455E+01_JPRB, 0.93205E+01_JPRB, 0.91882E+01_JPRB, 0.90251E+01_JPRB ,&
&  0.87097E+01_JPRB, 0.92758E+01_JPRB, 0.94586E+01_JPRB, 0.10257E+02_JPRB, 0.11131E+02_JPRB, 0.10981E+02_JPRB, 0.11401E+02_JPRB ,&
&  0.69500E+01_JPRB, 0.40362E+01_JPRB, 0.24871E+01_JPRB, 0.24955E+01_JPRB, 0.25959E+01_JPRB, 0.30799E+01_JPRB, 0.40290E+01_JPRB ,&
&  0.54422E+01_JPRB, 0.46146E+01_JPRB, 0.43529E+01_JPRB, 0.37866E+01_JPRB, 0.19178E+01_JPRB, 0.12731E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.93372E+00_JPRB, 0.95589E+00_JPRB, 0.10281E+01_JPRB, 0.10409E+01_JPRB, 0.10125E+01_JPRB, 0.10637E+01_JPRB, 0.11517E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.12187E+01_JPRB, 0.12964E+01_JPRB, 0.13052E+01_JPRB, 0.13646E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.13100E+01_JPRB, 0.14276E+01_JPRB, 0.15321E+01_JPRB, 0.16580E+01_JPRB, 0.18002E+01_JPRB, 0.19988E+01_JPRB, 0.21063E+01_JPRB ,&
&  0.21249E+01_JPRB, 0.21581E+01_JPRB, 0.23641E+01_JPRB, 0.26078E+01_JPRB, 0.28190E+01_JPRB, 0.29482E+01_JPRB, 0.30975E+01_JPRB ,&
&  0.32518E+01_JPRB, 0.33996E+01_JPRB, 0.35574E+01_JPRB, 0.37093E+01_JPRB, 0.39533E+01_JPRB, 0.42999E+01_JPRB, 0.46221E+01_JPRB ,&
&  0.41057E+01_JPRB, 0.26022E+01_JPRB, 0.28894E+01_JPRB, 0.27826E+01_JPRB, 0.28026E+01_JPRB, 0.29403E+01_JPRB, 0.31133E+01_JPRB ,&
&  0.25355E+01_JPRB, 0.22802E+01_JPRB, 0.23061E+01_JPRB, 0.23624E+01_JPRB, 0.15591E+01_JPRB, 0.16873E+01_JPRB, 0.20576E+01_JPRB ,&
&  0.24002E+01_JPRB, 0.27413E+01_JPRB, 0.30813E+01_JPRB, 0.33711E+01_JPRB, 0.36673E+01_JPRB, 0.40410E+01_JPRB, 0.43888E+01_JPRB ,&
&  0.46754E+01_JPRB, 0.48972E+01_JPRB, 0.50736E+01_JPRB, 0.52523E+01_JPRB, 0.54423E+01_JPRB, 0.57251E+01_JPRB, 0.58340E+01_JPRB ,&
&  0.60469E+01_JPRB, 0.62171E+01_JPRB, 0.69297E+01_JPRB, 0.78043E+01_JPRB, 0.85149E+01_JPRB, 0.85207E+01_JPRB, 0.92063E+01_JPRB ,&
&  0.94590E+01_JPRB, 0.92911E+01_JPRB, 0.95667E+01_JPRB, 0.97248E+01_JPRB, 0.69927E+01_JPRB, 0.68698E+01_JPRB, 0.70709E+01_JPRB ,&
&  0.79717E+01_JPRB, 0.80222E+01_JPRB, 0.83386E+01_JPRB, 0.88389E+01_JPRB, 0.90451E+01_JPRB, 0.89895E+01_JPRB, 0.82836E+01_JPRB ,&
&  0.55183E+01_JPRB, 0.30947E+01_JPRB, 0.22268E+01_JPRB, 0.24354E+01_JPRB, 0.29961E+01_JPRB, 0.41990E+01_JPRB, 0.35133E+01_JPRB ,&
&  0.39291E+01_JPRB, 0.27938E+01_JPRB, 0.27255E+01_JPRB, 0.32588E+01_JPRB, 0.18788E+01_JPRB, 0.12337E+01_JPRB, 0.97684E+00_JPRB ,&
&  0.89421E+00_JPRB, 0.91705E+00_JPRB, 0.91995E+00_JPRB, 0.90843E+00_JPRB, 0.90178E+00_JPRB, 0.95652E+00_JPRB, 0.10294E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.11081E+01_JPRB, 0.11629E+01_JPRB, 0.12208E+01_JPRB, 0.12940E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.10483E+01_JPRB, 0.10774E+01_JPRB, 0.11427E+01_JPRB, 0.12548E+01_JPRB, 0.13634E+01_JPRB, 0.14739E+01_JPRB, 0.15614E+01_JPRB ,&
&  0.17003E+01_JPRB, 0.18216E+01_JPRB, 0.19517E+01_JPRB, 0.20498E+01_JPRB, 0.21306E+01_JPRB, 0.23034E+01_JPRB, 0.24147E+01_JPRB ,&
&  0.24408E+01_JPRB, 0.23994E+01_JPRB, 0.24074E+01_JPRB, 0.24645E+01_JPRB, 0.26574E+01_JPRB, 0.30023E+01_JPRB, 0.34153E+01_JPRB ,&
&  0.38407E+01_JPRB, 0.36059E+01_JPRB, 0.31687E+01_JPRB, 0.20329E+01_JPRB, 0.17181E+01_JPRB, 0.20377E+01_JPRB, 0.20647E+01_JPRB ,&
&  0.21700E+01_JPRB, 0.22156E+01_JPRB, 0.19292E+01_JPRB, 0.17548E+01_JPRB, 0.15932E+01_JPRB, 0.15734E+01_JPRB, 0.17680E+01_JPRB ,&
&  0.20260E+01_JPRB, 0.23310E+01_JPRB, 0.26775E+01_JPRB, 0.29515E+01_JPRB, 0.32453E+01_JPRB, 0.36291E+01_JPRB, 0.40387E+01_JPRB ,&
&  0.43845E+01_JPRB, 0.46873E+01_JPRB, 0.49508E+01_JPRB, 0.53619E+01_JPRB, 0.56522E+01_JPRB, 0.60435E+01_JPRB, 0.63341E+01_JPRB ,&
&  0.66697E+01_JPRB, 0.68556E+01_JPRB, 0.74620E+01_JPRB, 0.81351E+01_JPRB, 0.82968E+01_JPRB, 0.76308E+01_JPRB, 0.75247E+01_JPRB ,&
&  0.76119E+01_JPRB, 0.72878E+01_JPRB, 0.72876E+01_JPRB, 0.78457E+01_JPRB, 0.54849E+01_JPRB, 0.57717E+01_JPRB, 0.65174E+01_JPRB ,&
&  0.65091E+01_JPRB, 0.67116E+01_JPRB, 0.65011E+01_JPRB, 0.56720E+01_JPRB, 0.54092E+01_JPRB, 0.56378E+01_JPRB, 0.54870E+01_JPRB ,&
&  0.42789E+01_JPRB, 0.27534E+01_JPRB, 0.20381E+01_JPRB, 0.18431E+01_JPRB, 0.25147E+01_JPRB, 0.42606E+01_JPRB, 0.31622E+01_JPRB ,&
&  0.24858E+01_JPRB, 0.23300E+01_JPRB, 0.22017E+01_JPRB, 0.34274E+01_JPRB, 0.24493E+01_JPRB, 0.14783E+01_JPRB, 0.10600E+01_JPRB ,&
&  0.88757E+00_JPRB, 0.86080E+00_JPRB, 0.86419E+00_JPRB, 0.84710E+00_JPRB, 0.86508E+00_JPRB, 0.91259E+00_JPRB, 0.92412E+00_JPRB ,&
&  0.99097E+00_JPRB, 0.10081E+01_JPRB, 0.10449E+01_JPRB, 0.10744E+01_JPRB, 0.10434E+01_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.72419E+00_JPRB, 0.71794E+00_JPRB, 0.74923E+00_JPRB, 0.84853E+00_JPRB, 0.92441E+00_JPRB, 0.98785E+00_JPRB, 0.10837E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.13775E+01_JPRB, 0.15179E+01_JPRB, 0.15658E+01_JPRB, 0.15164E+01_JPRB, 0.14662E+01_JPRB, 0.14564E+01_JPRB ,&
&  0.14517E+01_JPRB, 0.13144E+01_JPRB, 0.11979E+01_JPRB, 0.11581E+01_JPRB, 0.12080E+01_JPRB, 0.13491E+01_JPRB, 0.15652E+01_JPRB ,&
&  0.19269E+01_JPRB, 0.22136E+01_JPRB, 0.24331E+01_JPRB, 0.30607E+01_JPRB, 0.29928E+01_JPRB, 0.18903E+01_JPRB, 0.22024E+01_JPRB ,&
&  0.30716E+01_JPRB, 0.31197E+01_JPRB, 0.20528E+01_JPRB, 0.17543E+01_JPRB, 0.15855E+01_JPRB, 0.14784E+01_JPRB, 0.15316E+01_JPRB ,&
&  0.17041E+01_JPRB, 0.18933E+01_JPRB, 0.21434E+01_JPRB, 0.24610E+01_JPRB, 0.27983E+01_JPRB, 0.31046E+01_JPRB, 0.34770E+01_JPRB ,&
&  0.38016E+01_JPRB, 0.40060E+01_JPRB, 0.42691E+01_JPRB, 0.46609E+01_JPRB, 0.51890E+01_JPRB, 0.58258E+01_JPRB, 0.64087E+01_JPRB ,&
&  0.68866E+01_JPRB, 0.71542E+01_JPRB, 0.77214E+01_JPRB, 0.80731E+01_JPRB, 0.74706E+01_JPRB, 0.68160E+01_JPRB, 0.62590E+01_JPRB ,&
&  0.57777E+01_JPRB, 0.50525E+01_JPRB, 0.45665E+01_JPRB, 0.57996E+01_JPRB, 0.60388E+01_JPRB, 0.53715E+01_JPRB, 0.52944E+01_JPRB ,&
&  0.53438E+01_JPRB, 0.53111E+01_JPRB, 0.47335E+01_JPRB, 0.36775E+01_JPRB, 0.29118E+01_JPRB, 0.33380E+01_JPRB, 0.36196E+01_JPRB ,&
&  0.30811E+01_JPRB, 0.21563E+01_JPRB, 0.17045E+01_JPRB, 0.15932E+01_JPRB, 0.17340E+01_JPRB, 0.29411E+01_JPRB, 0.22469E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.18216E+01_JPRB, 0.17893E+01_JPRB, 0.26542E+01_JPRB, 0.33195E+01_JPRB, 0.17986E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.84800E+00_JPRB, 0.82457E+00_JPRB, 0.82587E+00_JPRB, 0.82443E+00_JPRB, 0.82680E+00_JPRB, 0.84646E+00_JPRB, 0.84846E+00_JPRB ,&
&  0.86374E+00_JPRB, 0.84831E+00_JPRB, 0.84948E+00_JPRB, 0.80430E+00_JPRB, 0.73730E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.74445E+00_JPRB, 0.74162E+00_JPRB, 0.72915E+00_JPRB, 0.72629E+00_JPRB, 0.74015E+00_JPRB, 0.76254E+00_JPRB, 0.79477E+00_JPRB ,&
&  0.84457E+00_JPRB, 0.90143E+00_JPRB, 0.95407E+00_JPRB, 0.10116E+01_JPRB, 0.10179E+01_JPRB, 0.94263E+00_JPRB, 0.92720E+00_JPRB ,&
&  0.91654E+00_JPRB, 0.92312E+00_JPRB, 0.91425E+00_JPRB, 0.10076E+01_JPRB, 0.10258E+01_JPRB, 0.10320E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11670E+01_JPRB, 0.13188E+01_JPRB, 0.14213E+01_JPRB, 0.15598E+01_JPRB, 0.24635E+01_JPRB, 0.23575E+01_JPRB, 0.19054E+01_JPRB ,&
&  0.28351E+01_JPRB, 0.28658E+01_JPRB, 0.20877E+01_JPRB, 0.16164E+01_JPRB, 0.14643E+01_JPRB, 0.13371E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.15132E+01_JPRB, 0.17655E+01_JPRB, 0.20262E+01_JPRB, 0.22418E+01_JPRB, 0.24441E+01_JPRB, 0.26701E+01_JPRB ,&
&  0.28401E+01_JPRB, 0.30218E+01_JPRB, 0.31206E+01_JPRB, 0.33190E+01_JPRB, 0.39466E+01_JPRB, 0.45566E+01_JPRB, 0.50433E+01_JPRB ,&
&  0.54378E+01_JPRB, 0.58187E+01_JPRB, 0.64183E+01_JPRB, 0.62357E+01_JPRB, 0.57060E+01_JPRB, 0.53682E+01_JPRB, 0.45463E+01_JPRB ,&
&  0.37222E+01_JPRB, 0.29339E+01_JPRB, 0.26758E+01_JPRB, 0.34532E+01_JPRB, 0.36463E+01_JPRB, 0.32360E+01_JPRB, 0.37859E+01_JPRB ,&
&  0.39553E+01_JPRB, 0.40502E+01_JPRB, 0.33289E+01_JPRB, 0.28587E+01_JPRB, 0.23144E+01_JPRB, 0.23042E+01_JPRB, 0.25263E+01_JPRB ,&
&  0.20719E+01_JPRB, 0.15396E+01_JPRB, 0.13279E+01_JPRB, 0.12734E+01_JPRB, 0.13316E+01_JPRB, 0.18987E+01_JPRB, 0.21642E+01_JPRB ,&
&  0.20135E+01_JPRB, 0.17470E+01_JPRB, 0.17667E+01_JPRB, 0.26526E+01_JPRB, 0.21500E+01_JPRB, 0.12907E+01_JPRB, 0.84122E+00_JPRB ,&
&  0.73637E+00_JPRB, 0.73978E+00_JPRB, 0.73394E+00_JPRB, 0.72960E+00_JPRB, 0.74055E+00_JPRB, 0.76021E+00_JPRB, 0.74544E+00_JPRB ,&
&  0.74231E+00_JPRB, 0.72629E+00_JPRB, 0.71450E+00_JPRB, 0.70675E+00_JPRB, 0.71393E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.90397E+00_JPRB, 0.91445E+00_JPRB, 0.92625E+00_JPRB, 0.94061E+00_JPRB, 0.96919E+00_JPRB, 0.10019E+01_JPRB, 0.10313E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.11255E+01_JPRB, 0.11632E+01_JPRB, 0.11867E+01_JPRB, 0.12048E+01_JPRB, 0.12183E+01_JPRB, 0.12242E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12854E+01_JPRB, 0.12831E+01_JPRB, 0.12892E+01_JPRB, 0.13150E+01_JPRB, 0.13311E+01_JPRB, 0.13099E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.12603E+01_JPRB, 0.12891E+01_JPRB, 0.13361E+01_JPRB, 0.15643E+01_JPRB, 0.17408E+01_JPRB, 0.17517E+01_JPRB ,&
&  0.21163E+01_JPRB, 0.23321E+01_JPRB, 0.22177E+01_JPRB, 0.16249E+01_JPRB, 0.14739E+01_JPRB, 0.14001E+01_JPRB, 0.13289E+01_JPRB ,&
&  0.13388E+01_JPRB, 0.13920E+01_JPRB, 0.14714E+01_JPRB, 0.15585E+01_JPRB, 0.16619E+01_JPRB, 0.17023E+01_JPRB, 0.17772E+01_JPRB ,&
&  0.18056E+01_JPRB, 0.18725E+01_JPRB, 0.19805E+01_JPRB, 0.22346E+01_JPRB, 0.26987E+01_JPRB, 0.32643E+01_JPRB, 0.33537E+01_JPRB ,&
&  0.34141E+01_JPRB, 0.37495E+01_JPRB, 0.35366E+01_JPRB, 0.34374E+01_JPRB, 0.33206E+01_JPRB, 0.28619E+01_JPRB, 0.23945E+01_JPRB ,&
&  0.22913E+01_JPRB, 0.20890E+01_JPRB, 0.16651E+01_JPRB, 0.20835E+01_JPRB, 0.23852E+01_JPRB, 0.24650E+01_JPRB, 0.28307E+01_JPRB ,&
&  0.30520E+01_JPRB, 0.26530E+01_JPRB, 0.20874E+01_JPRB, 0.20370E+01_JPRB, 0.19809E+01_JPRB, 0.16308E+01_JPRB, 0.16194E+01_JPRB ,&
&  0.14218E+01_JPRB, 0.12730E+01_JPRB, 0.12358E+01_JPRB, 0.12735E+01_JPRB, 0.16235E+01_JPRB, 0.27827E+01_JPRB, 0.29796E+01_JPRB ,&
&  0.22634E+01_JPRB, 0.19158E+01_JPRB, 0.22781E+01_JPRB, 0.35374E+01_JPRB, 0.19164E+01_JPRB, 0.12211E+01_JPRB, 0.77441E+00_JPRB ,&
&  0.74376E+00_JPRB, 0.74901E+00_JPRB, 0.75553E+00_JPRB, 0.76206E+00_JPRB, 0.77167E+00_JPRB, 0.79243E+00_JPRB, 0.78436E+00_JPRB ,&
&  0.77608E+00_JPRB, 0.77062E+00_JPRB, 0.78589E+00_JPRB, 0.81137E+00_JPRB, 0.86274E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.10430E+01_JPRB, 0.10494E+01_JPRB, 0.10715E+01_JPRB, 0.10987E+01_JPRB, 0.11256E+01_JPRB, 0.11578E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.13025E+01_JPRB, 0.13428E+01_JPRB, 0.13734E+01_JPRB, 0.13833E+01_JPRB, 0.13999E+01_JPRB, 0.14188E+01_JPRB ,&
&  0.14387E+01_JPRB, 0.14306E+01_JPRB, 0.14549E+01_JPRB, 0.14953E+01_JPRB, 0.15465E+01_JPRB, 0.15687E+01_JPRB, 0.15938E+01_JPRB ,&
&  0.16176E+01_JPRB, 0.16745E+01_JPRB, 0.17362E+01_JPRB, 0.18110E+01_JPRB, 0.19279E+01_JPRB, 0.21219E+01_JPRB, 0.22958E+01_JPRB ,&
&  0.20212E+01_JPRB, 0.24826E+01_JPRB, 0.21216E+01_JPRB, 0.17016E+01_JPRB, 0.16012E+01_JPRB, 0.16376E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.15648E+01_JPRB, 0.15975E+01_JPRB, 0.16375E+01_JPRB, 0.16751E+01_JPRB, 0.17115E+01_JPRB, 0.17646E+01_JPRB, 0.17709E+01_JPRB ,&
&  0.18168E+01_JPRB, 0.18502E+01_JPRB, 0.18926E+01_JPRB, 0.19975E+01_JPRB, 0.20286E+01_JPRB, 0.25522E+01_JPRB, 0.24711E+01_JPRB ,&
&  0.23049E+01_JPRB, 0.23481E+01_JPRB, 0.18529E+01_JPRB, 0.19094E+01_JPRB, 0.19208E+01_JPRB, 0.16599E+01_JPRB, 0.14965E+01_JPRB ,&
&  0.19115E+01_JPRB, 0.20709E+01_JPRB, 0.16168E+01_JPRB, 0.18748E+01_JPRB, 0.20519E+01_JPRB, 0.21400E+01_JPRB, 0.22025E+01_JPRB ,&
&  0.21653E+01_JPRB, 0.18457E+01_JPRB, 0.15694E+01_JPRB, 0.14428E+01_JPRB, 0.13917E+01_JPRB, 0.13062E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.11670E+01_JPRB, 0.11508E+01_JPRB, 0.11573E+01_JPRB, 0.12121E+01_JPRB, 0.18905E+01_JPRB, 0.26593E+01_JPRB, 0.38936E+01_JPRB ,&
&  0.20930E+01_JPRB, 0.18371E+01_JPRB, 0.14423E+01_JPRB, 0.21423E+01_JPRB, 0.14546E+01_JPRB, 0.86110E+00_JPRB, 0.81418E+00_JPRB ,&
&  0.87027E+00_JPRB, 0.91199E+00_JPRB, 0.91405E+00_JPRB, 0.90055E+00_JPRB, 0.90166E+00_JPRB, 0.90720E+00_JPRB, 0.88273E+00_JPRB ,&
&  0.86964E+00_JPRB, 0.88886E+00_JPRB, 0.92852E+00_JPRB, 0.95025E+00_JPRB, 0.99781E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.11233E+01_JPRB, 0.11577E+01_JPRB, 0.11946E+01_JPRB, 0.12343E+01_JPRB, 0.12726E+01_JPRB, 0.13092E+01_JPRB, 0.13489E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.14318E+01_JPRB, 0.14657E+01_JPRB, 0.14856E+01_JPRB, 0.14960E+01_JPRB, 0.15181E+01_JPRB, 0.15266E+01_JPRB ,&
&  0.15317E+01_JPRB, 0.15377E+01_JPRB, 0.15657E+01_JPRB, 0.15857E+01_JPRB, 0.16034E+01_JPRB, 0.16114E+01_JPRB, 0.16464E+01_JPRB ,&
&  0.16542E+01_JPRB, 0.17152E+01_JPRB, 0.17614E+01_JPRB, 0.18210E+01_JPRB, 0.18731E+01_JPRB, 0.19055E+01_JPRB, 0.20730E+01_JPRB ,&
&  0.18744E+01_JPRB, 0.27778E+01_JPRB, 0.25514E+01_JPRB, 0.20013E+01_JPRB, 0.19207E+01_JPRB, 0.18030E+01_JPRB, 0.17911E+01_JPRB ,&
&  0.18075E+01_JPRB, 0.18299E+01_JPRB, 0.18828E+01_JPRB, 0.19652E+01_JPRB, 0.20396E+01_JPRB, 0.21114E+01_JPRB, 0.21466E+01_JPRB ,&
&  0.22032E+01_JPRB, 0.22607E+01_JPRB, 0.23232E+01_JPRB, 0.23598E+01_JPRB, 0.24296E+01_JPRB, 0.24916E+01_JPRB, 0.24299E+01_JPRB ,&
&  0.22478E+01_JPRB, 0.20717E+01_JPRB, 0.15164E+01_JPRB, 0.14852E+01_JPRB, 0.15886E+01_JPRB, 0.15112E+01_JPRB, 0.13586E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.19231E+01_JPRB, 0.17882E+01_JPRB, 0.17869E+01_JPRB, 0.17790E+01_JPRB, 0.16933E+01_JPRB, 0.16251E+01_JPRB ,&
&  0.14494E+01_JPRB, 0.12481E+01_JPRB, 0.11168E+01_JPRB, 0.10390E+01_JPRB, 0.10474E+01_JPRB, 0.10399E+01_JPRB, 0.10174E+01_JPRB ,&
&  0.98325E+00_JPRB, 0.10298E+01_JPRB, 0.11434E+01_JPRB, 0.13223E+01_JPRB, 0.17569E+01_JPRB, 0.35818E+01_JPRB, 0.37432E+01_JPRB ,&
&  0.16646E+01_JPRB, 0.14945E+01_JPRB, 0.94044E+00_JPRB, 0.10637E+01_JPRB, 0.10637E+01_JPRB, 0.96328E+00_JPRB, 0.10596E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11885E+01_JPRB, 0.11191E+01_JPRB, 0.10300E+01_JPRB, 0.99386E+00_JPRB, 0.98224E+00_JPRB, 0.95688E+00_JPRB ,&
&  0.95690E+00_JPRB, 0.99157E+00_JPRB, 0.10254E+01_JPRB, 0.10553E+01_JPRB, 0.10886E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.11953E+01_JPRB, 0.12287E+01_JPRB, 0.12625E+01_JPRB, 0.13102E+01_JPRB, 0.13606E+01_JPRB, 0.14082E+01_JPRB, 0.14476E+01_JPRB ,&
&  0.14888E+01_JPRB, 0.15201E+01_JPRB, 0.15553E+01_JPRB, 0.15826E+01_JPRB, 0.16039E+01_JPRB, 0.16226E+01_JPRB, 0.16321E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.16537E+01_JPRB, 0.16834E+01_JPRB, 0.17141E+01_JPRB, 0.17405E+01_JPRB, 0.17680E+01_JPRB, 0.18072E+01_JPRB ,&
&  0.18272E+01_JPRB, 0.18510E+01_JPRB, 0.19041E+01_JPRB, 0.19841E+01_JPRB, 0.20890E+01_JPRB, 0.23560E+01_JPRB, 0.22732E+01_JPRB ,&
&  0.26232E+01_JPRB, 0.29428E+01_JPRB, 0.28501E+01_JPRB, 0.22685E+01_JPRB, 0.20701E+01_JPRB, 0.19961E+01_JPRB, 0.20004E+01_JPRB ,&
&  0.20204E+01_JPRB, 0.20175E+01_JPRB, 0.20480E+01_JPRB, 0.21246E+01_JPRB, 0.22407E+01_JPRB, 0.23643E+01_JPRB, 0.24325E+01_JPRB ,&
&  0.25227E+01_JPRB, 0.26295E+01_JPRB, 0.27449E+01_JPRB, 0.28530E+01_JPRB, 0.30159E+01_JPRB, 0.31146E+01_JPRB, 0.30730E+01_JPRB ,&
&  0.29205E+01_JPRB, 0.27997E+01_JPRB, 0.23788E+01_JPRB, 0.22736E+01_JPRB, 0.23543E+01_JPRB, 0.20747E+01_JPRB, 0.18425E+01_JPRB ,&
&  0.20151E+01_JPRB, 0.17135E+01_JPRB, 0.14969E+01_JPRB, 0.16300E+01_JPRB, 0.15505E+01_JPRB, 0.13924E+01_JPRB, 0.13067E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.94209E+00_JPRB, 0.82564E+00_JPRB, 0.80190E+00_JPRB, 0.82389E+00_JPRB, 0.87292E+00_JPRB, 0.89767E+00_JPRB ,&
&  0.95778E+00_JPRB, 0.10627E+01_JPRB, 0.11908E+01_JPRB, 0.14042E+01_JPRB, 0.18652E+01_JPRB, 0.19207E+01_JPRB, 0.21920E+01_JPRB ,&
&  0.15019E+01_JPRB, 0.12276E+01_JPRB, 0.12445E+01_JPRB, 0.11418E+01_JPRB, 0.11853E+01_JPRB, 0.11863E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.15141E+01_JPRB, 0.16687E+01_JPRB, 0.15354E+01_JPRB, 0.13371E+01_JPRB, 0.11408E+01_JPRB, 0.10593E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.10306E+01_JPRB, 0.10730E+01_JPRB, 0.11072E+01_JPRB, 0.11311E+01_JPRB, 0.11617E+01_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.12059E+01_JPRB, 0.12406E+01_JPRB, 0.12704E+01_JPRB, 0.13076E+01_JPRB, 0.13521E+01_JPRB, 0.14011E+01_JPRB, 0.14406E+01_JPRB ,&
&  0.14828E+01_JPRB, 0.15191E+01_JPRB, 0.15566E+01_JPRB, 0.15977E+01_JPRB, 0.16348E+01_JPRB, 0.16800E+01_JPRB, 0.17115E+01_JPRB ,&
&  0.17544E+01_JPRB, 0.17738E+01_JPRB, 0.18077E+01_JPRB, 0.18534E+01_JPRB, 0.19064E+01_JPRB, 0.19670E+01_JPRB, 0.20345E+01_JPRB ,&
&  0.20775E+01_JPRB, 0.20943E+01_JPRB, 0.21176E+01_JPRB, 0.21504E+01_JPRB, 0.21668E+01_JPRB, 0.23633E+01_JPRB, 0.23511E+01_JPRB ,&
&  0.30136E+01_JPRB, 0.33140E+01_JPRB, 0.31179E+01_JPRB, 0.25468E+01_JPRB, 0.23428E+01_JPRB, 0.22531E+01_JPRB, 0.22397E+01_JPRB ,&
&  0.22264E+01_JPRB, 0.22177E+01_JPRB, 0.21909E+01_JPRB, 0.22277E+01_JPRB, 0.23364E+01_JPRB, 0.25203E+01_JPRB, 0.26811E+01_JPRB ,&
&  0.28049E+01_JPRB, 0.29670E+01_JPRB, 0.31684E+01_JPRB, 0.33995E+01_JPRB, 0.36768E+01_JPRB, 0.38387E+01_JPRB, 0.39001E+01_JPRB ,&
&  0.38849E+01_JPRB, 0.38290E+01_JPRB, 0.34778E+01_JPRB, 0.36280E+01_JPRB, 0.37204E+01_JPRB, 0.29867E+01_JPRB, 0.23947E+01_JPRB ,&
&  0.19475E+01_JPRB, 0.16282E+01_JPRB, 0.15868E+01_JPRB, 0.15485E+01_JPRB, 0.13841E+01_JPRB, 0.12775E+01_JPRB, 0.11571E+01_JPRB ,&
&  0.94052E+00_JPRB, 0.82786E+00_JPRB, 0.76470E+00_JPRB, 0.72105E+00_JPRB, 0.73274E+00_JPRB, 0.77684E+00_JPRB, 0.88214E+00_JPRB ,&
&  0.98301E+00_JPRB, 0.10980E+01_JPRB, 0.12538E+01_JPRB, 0.14209E+01_JPRB, 0.16617E+01_JPRB, 0.16939E+01_JPRB, 0.18772E+01_JPRB ,&
&  0.15678E+01_JPRB, 0.14641E+01_JPRB, 0.14187E+01_JPRB, 0.10374E+01_JPRB, 0.12386E+01_JPRB, 0.13361E+01_JPRB, 0.14928E+01_JPRB ,&
&  0.15351E+01_JPRB, 0.20105E+01_JPRB, 0.23485E+01_JPRB, 0.19811E+01_JPRB, 0.14832E+01_JPRB, 0.11102E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10830E+01_JPRB, 0.11217E+01_JPRB, 0.11469E+01_JPRB, 0.11757E+01_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.11455E+01_JPRB, 0.11777E+01_JPRB, 0.12085E+01_JPRB, 0.12334E+01_JPRB, 0.12510E+01_JPRB, 0.12808E+01_JPRB, 0.13253E+01_JPRB ,&
&  0.13671E+01_JPRB, 0.14102E+01_JPRB, 0.14547E+01_JPRB, 0.15005E+01_JPRB, 0.15481E+01_JPRB, 0.15997E+01_JPRB, 0.16670E+01_JPRB ,&
&  0.17574E+01_JPRB, 0.18472E+01_JPRB, 0.19223E+01_JPRB, 0.20294E+01_JPRB, 0.21320E+01_JPRB, 0.22288E+01_JPRB, 0.23800E+01_JPRB ,&
&  0.25183E+01_JPRB, 0.26621E+01_JPRB, 0.27770E+01_JPRB, 0.28476E+01_JPRB, 0.28424E+01_JPRB, 0.30490E+01_JPRB, 0.28176E+01_JPRB ,&
&  0.33500E+01_JPRB, 0.36480E+01_JPRB, 0.33732E+01_JPRB, 0.27952E+01_JPRB, 0.26097E+01_JPRB, 0.24842E+01_JPRB, 0.23864E+01_JPRB ,&
&  0.23557E+01_JPRB, 0.22842E+01_JPRB, 0.21419E+01_JPRB, 0.22274E+01_JPRB, 0.23401E+01_JPRB, 0.25373E+01_JPRB, 0.27855E+01_JPRB ,&
&  0.29902E+01_JPRB, 0.32188E+01_JPRB, 0.34624E+01_JPRB, 0.37290E+01_JPRB, 0.40613E+01_JPRB, 0.43333E+01_JPRB, 0.45500E+01_JPRB ,&
&  0.47180E+01_JPRB, 0.48280E+01_JPRB, 0.49084E+01_JPRB, 0.46365E+01_JPRB, 0.45996E+01_JPRB, 0.40212E+01_JPRB, 0.29666E+01_JPRB ,&
&  0.18777E+01_JPRB, 0.15622E+01_JPRB, 0.15572E+01_JPRB, 0.14929E+01_JPRB, 0.13513E+01_JPRB, 0.11587E+01_JPRB, 0.10646E+01_JPRB ,&
&  0.88320E+00_JPRB, 0.77891E+00_JPRB, 0.74432E+00_JPRB, 0.72122E+00_JPRB, 0.72239E+00_JPRB, 0.78707E+00_JPRB, 0.85596E+00_JPRB ,&
&  0.97189E+00_JPRB, 0.11233E+01_JPRB, 0.12783E+01_JPRB, 0.14530E+01_JPRB, 0.17375E+01_JPRB, 0.19917E+01_JPRB, 0.17509E+01_JPRB ,&
&  0.16504E+01_JPRB, 0.14113E+01_JPRB, 0.13753E+01_JPRB, 0.14815E+01_JPRB, 0.13655E+01_JPRB, 0.15748E+01_JPRB, 0.18020E+01_JPRB ,&
&  0.21402E+01_JPRB, 0.20289E+01_JPRB, 0.22811E+01_JPRB, 0.32924E+01_JPRB, 0.27868E+01_JPRB, 0.23964E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.10536E+01_JPRB, 0.11004E+01_JPRB, 0.11195E+01_JPRB, 0.11382E+01_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.98704E+00_JPRB, 0.10393E+01_JPRB, 0.10616E+01_JPRB, 0.11139E+01_JPRB, 0.11289E+01_JPRB, 0.11367E+01_JPRB, 0.11768E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.12503E+01_JPRB, 0.13008E+01_JPRB, 0.13524E+01_JPRB, 0.13981E+01_JPRB, 0.14326E+01_JPRB, 0.14851E+01_JPRB ,&
&  0.15627E+01_JPRB, 0.16591E+01_JPRB, 0.17827E+01_JPRB, 0.19459E+01_JPRB, 0.20784E+01_JPRB, 0.22206E+01_JPRB, 0.23992E+01_JPRB ,&
&  0.25618E+01_JPRB, 0.27973E+01_JPRB, 0.30273E+01_JPRB, 0.32451E+01_JPRB, 0.33923E+01_JPRB, 0.34400E+01_JPRB, 0.26580E+01_JPRB ,&
&  0.36269E+01_JPRB, 0.36025E+01_JPRB, 0.34380E+01_JPRB, 0.29891E+01_JPRB, 0.27899E+01_JPRB, 0.26317E+01_JPRB, 0.25100E+01_JPRB ,&
&  0.24200E+01_JPRB, 0.22919E+01_JPRB, 0.21397E+01_JPRB, 0.20452E+01_JPRB, 0.23095E+01_JPRB, 0.24314E+01_JPRB, 0.26705E+01_JPRB ,&
&  0.29189E+01_JPRB, 0.31581E+01_JPRB, 0.34006E+01_JPRB, 0.36637E+01_JPRB, 0.40091E+01_JPRB, 0.44001E+01_JPRB, 0.47737E+01_JPRB ,&
&  0.51132E+01_JPRB, 0.53079E+01_JPRB, 0.55775E+01_JPRB, 0.49993E+01_JPRB, 0.48476E+01_JPRB, 0.45410E+01_JPRB, 0.28448E+01_JPRB ,&
&  0.17311E+01_JPRB, 0.15897E+01_JPRB, 0.16447E+01_JPRB, 0.15570E+01_JPRB, 0.13585E+01_JPRB, 0.11481E+01_JPRB, 0.99436E+00_JPRB ,&
&  0.85162E+00_JPRB, 0.80554E+00_JPRB, 0.77090E+00_JPRB, 0.75448E+00_JPRB, 0.75765E+00_JPRB, 0.80898E+00_JPRB, 0.86489E+00_JPRB ,&
&  0.97899E+00_JPRB, 0.10773E+01_JPRB, 0.11898E+01_JPRB, 0.13075E+01_JPRB, 0.14770E+01_JPRB, 0.16804E+01_JPRB, 0.18722E+01_JPRB ,&
&  0.15575E+01_JPRB, 0.13526E+01_JPRB, 0.12955E+01_JPRB, 0.13279E+01_JPRB, 0.13404E+01_JPRB, 0.14200E+01_JPRB, 0.15227E+01_JPRB ,&
&  0.16687E+01_JPRB, 0.17657E+01_JPRB, 0.15916E+01_JPRB, 0.13158E+01_JPRB, 0.15970E+01_JPRB, 0.16254E+01_JPRB, 0.83589E+00_JPRB ,&
&  0.84143E+00_JPRB, 0.88741E+00_JPRB, 0.92238E+00_JPRB, 0.95945E+00_JPRB, 0.99322E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.80598E+00_JPRB, 0.82200E+00_JPRB, 0.82978E+00_JPRB, 0.88750E+00_JPRB, 0.91171E+00_JPRB, 0.95094E+00_JPRB, 0.98264E+00_JPRB ,&
&  0.10232E+01_JPRB, 0.10753E+01_JPRB, 0.11346E+01_JPRB, 0.11911E+01_JPRB, 0.12436E+01_JPRB, 0.12871E+01_JPRB, 0.13299E+01_JPRB ,&
&  0.13744E+01_JPRB, 0.14267E+01_JPRB, 0.14957E+01_JPRB, 0.15927E+01_JPRB, 0.17437E+01_JPRB, 0.19192E+01_JPRB, 0.21126E+01_JPRB ,&
&  0.23155E+01_JPRB, 0.25723E+01_JPRB, 0.28585E+01_JPRB, 0.31680E+01_JPRB, 0.34875E+01_JPRB, 0.37183E+01_JPRB, 0.38884E+01_JPRB ,&
&  0.28918E+01_JPRB, 0.34718E+01_JPRB, 0.33928E+01_JPRB, 0.31030E+01_JPRB, 0.29067E+01_JPRB, 0.27974E+01_JPRB, 0.26672E+01_JPRB ,&
&  0.25482E+01_JPRB, 0.23301E+01_JPRB, 0.22229E+01_JPRB, 0.21301E+01_JPRB, 0.22162E+01_JPRB, 0.22946E+01_JPRB, 0.24642E+01_JPRB ,&
&  0.27017E+01_JPRB, 0.28917E+01_JPRB, 0.31225E+01_JPRB, 0.33598E+01_JPRB, 0.36672E+01_JPRB, 0.41432E+01_JPRB, 0.46937E+01_JPRB ,&
&  0.52937E+01_JPRB, 0.57736E+01_JPRB, 0.61234E+01_JPRB, 0.54080E+01_JPRB, 0.49585E+01_JPRB, 0.44989E+01_JPRB, 0.27995E+01_JPRB ,&
&  0.18575E+01_JPRB, 0.15571E+01_JPRB, 0.16391E+01_JPRB, 0.16670E+01_JPRB, 0.14248E+01_JPRB, 0.12296E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.90673E+00_JPRB, 0.83509E+00_JPRB, 0.80840E+00_JPRB, 0.78570E+00_JPRB, 0.79373E+00_JPRB, 0.83245E+00_JPRB, 0.87275E+00_JPRB ,&
&  0.94674E+00_JPRB, 0.98631E+00_JPRB, 0.10701E+01_JPRB, 0.11407E+01_JPRB, 0.12127E+01_JPRB, 0.12609E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.13059E+01_JPRB, 0.13116E+01_JPRB, 0.13059E+01_JPRB, 0.13241E+01_JPRB, 0.13172E+01_JPRB, 0.13229E+01_JPRB, 0.13445E+01_JPRB ,&
&  0.13682E+01_JPRB, 0.13775E+01_JPRB, 0.13241E+01_JPRB, 0.12599E+01_JPRB, 0.11542E+01_JPRB, 0.10655E+01_JPRB, 0.83245E+00_JPRB ,&
&  0.78903E+00_JPRB, 0.74695E+00_JPRB, 0.77601E+00_JPRB, 0.81088E+00_JPRB, 0.80720E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.71440E+00_JPRB, 0.70504E+00_JPRB, 0.69453E+00_JPRB, 0.70930E+00_JPRB, 0.72567E+00_JPRB, 0.76421E+00_JPRB, 0.77178E+00_JPRB ,&
&  0.79913E+00_JPRB, 0.87127E+00_JPRB, 0.96520E+00_JPRB, 0.10329E+01_JPRB, 0.10876E+01_JPRB, 0.11418E+01_JPRB, 0.11949E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.12522E+01_JPRB, 0.12749E+01_JPRB, 0.13061E+01_JPRB, 0.13678E+01_JPRB, 0.14688E+01_JPRB, 0.16297E+01_JPRB ,&
&  0.18050E+01_JPRB, 0.20071E+01_JPRB, 0.22612E+01_JPRB, 0.25712E+01_JPRB, 0.29067E+01_JPRB, 0.31307E+01_JPRB, 0.33185E+01_JPRB ,&
&  0.19004E+01_JPRB, 0.29181E+01_JPRB, 0.35311E+01_JPRB, 0.33087E+01_JPRB, 0.30557E+01_JPRB, 0.29149E+01_JPRB, 0.27411E+01_JPRB ,&
&  0.24802E+01_JPRB, 0.22151E+01_JPRB, 0.21044E+01_JPRB, 0.22482E+01_JPRB, 0.21246E+01_JPRB, 0.21567E+01_JPRB, 0.22092E+01_JPRB ,&
&  0.23826E+01_JPRB, 0.25121E+01_JPRB, 0.25851E+01_JPRB, 0.26909E+01_JPRB, 0.28649E+01_JPRB, 0.32608E+01_JPRB, 0.39433E+01_JPRB ,&
&  0.48766E+01_JPRB, 0.58752E+01_JPRB, 0.64345E+01_JPRB, 0.55220E+01_JPRB, 0.56742E+01_JPRB, 0.51914E+01_JPRB, 0.42188E+01_JPRB ,&
&  0.24033E+01_JPRB, 0.16901E+01_JPRB, 0.17242E+01_JPRB, 0.17452E+01_JPRB, 0.15319E+01_JPRB, 0.13747E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.95097E+00_JPRB, 0.89595E+00_JPRB, 0.86288E+00_JPRB, 0.85234E+00_JPRB, 0.86528E+00_JPRB, 0.89545E+00_JPRB ,&
&  0.91858E+00_JPRB, 0.94527E+00_JPRB, 0.99157E+00_JPRB, 0.10449E+01_JPRB, 0.11059E+01_JPRB, 0.11575E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.12443E+01_JPRB, 0.12708E+01_JPRB, 0.12955E+01_JPRB, 0.13125E+01_JPRB, 0.13199E+01_JPRB, 0.13268E+01_JPRB, 0.13137E+01_JPRB ,&
&  0.12974E+01_JPRB, 0.13108E+01_JPRB, 0.13063E+01_JPRB, 0.12797E+01_JPRB, 0.12076E+01_JPRB, 0.11377E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.96911E+00_JPRB, 0.84408E+00_JPRB, 0.74700E+00_JPRB, 0.72753E+00_JPRB, 0.71764E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.68094E+00_JPRB, 0.65240E+00_JPRB, 0.64818E+00_JPRB, 0.65840E+00_JPRB, 0.68079E+00_JPRB, 0.69526E+00_JPRB, 0.69688E+00_JPRB ,&
&  0.70456E+00_JPRB, 0.72535E+00_JPRB, 0.74998E+00_JPRB, 0.82205E+00_JPRB, 0.90892E+00_JPRB, 0.98167E+00_JPRB, 0.10322E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.10838E+01_JPRB, 0.10777E+01_JPRB, 0.10786E+01_JPRB, 0.10844E+01_JPRB, 0.11129E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.13109E+01_JPRB, 0.14653E+01_JPRB, 0.16659E+01_JPRB, 0.19484E+01_JPRB, 0.23368E+01_JPRB, 0.27493E+01_JPRB, 0.30653E+01_JPRB ,&
&  0.22379E+01_JPRB, 0.10615E+01_JPRB, 0.31541E+01_JPRB, 0.37965E+01_JPRB, 0.33529E+01_JPRB, 0.31792E+01_JPRB, 0.28435E+01_JPRB ,&
&  0.24155E+01_JPRB, 0.22185E+01_JPRB, 0.20642E+01_JPRB, 0.21150E+01_JPRB, 0.20310E+01_JPRB, 0.20553E+01_JPRB, 0.20959E+01_JPRB ,&
&  0.21778E+01_JPRB, 0.23083E+01_JPRB, 0.23270E+01_JPRB, 0.22924E+01_JPRB, 0.22929E+01_JPRB, 0.24292E+01_JPRB, 0.28749E+01_JPRB ,&
&  0.37935E+01_JPRB, 0.51712E+01_JPRB, 0.64635E+01_JPRB, 0.57988E+01_JPRB, 0.59954E+01_JPRB, 0.47059E+01_JPRB, 0.33579E+01_JPRB ,&
&  0.24423E+01_JPRB, 0.19133E+01_JPRB, 0.18329E+01_JPRB, 0.18384E+01_JPRB, 0.16711E+01_JPRB, 0.14699E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.12123E+01_JPRB, 0.10887E+01_JPRB, 0.99897E+00_JPRB, 0.94871E+00_JPRB, 0.92470E+00_JPRB, 0.92308E+00_JPRB, 0.93328E+00_JPRB ,&
&  0.93592E+00_JPRB, 0.94612E+00_JPRB, 0.96478E+00_JPRB, 0.10057E+01_JPRB, 0.10598E+01_JPRB, 0.11277E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.12254E+01_JPRB, 0.12463E+01_JPRB, 0.12561E+01_JPRB, 0.12641E+01_JPRB, 0.12406E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.12511E+01_JPRB, 0.12742E+01_JPRB, 0.12945E+01_JPRB, 0.13253E+01_JPRB, 0.12781E+01_JPRB, 0.12310E+01_JPRB, 0.12295E+01_JPRB ,&
&  0.11873E+01_JPRB, 0.10717E+01_JPRB, 0.90764E+00_JPRB, 0.76981E+00_JPRB, 0.71497E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.77123E+00_JPRB, 0.73155E+00_JPRB, 0.69875E+00_JPRB, 0.69354E+00_JPRB, 0.71085E+00_JPRB, 0.70675E+00_JPRB, 0.70530E+00_JPRB ,&
&  0.70018E+00_JPRB, 0.69445E+00_JPRB, 0.67933E+00_JPRB, 0.69132E+00_JPRB, 0.73662E+00_JPRB, 0.81204E+00_JPRB, 0.86813E+00_JPRB ,&
&  0.91633E+00_JPRB, 0.93059E+00_JPRB, 0.92714E+00_JPRB, 0.92210E+00_JPRB, 0.91618E+00_JPRB, 0.91704E+00_JPRB, 0.93187E+00_JPRB ,&
&  0.97624E+00_JPRB, 0.10505E+01_JPRB, 0.11526E+01_JPRB, 0.12908E+01_JPRB, 0.15031E+01_JPRB, 0.18976E+01_JPRB, 0.25159E+01_JPRB ,&
&  0.35161E+01_JPRB, 0.25141E+01_JPRB, 0.14643E+01_JPRB, 0.33202E+01_JPRB, 0.37356E+01_JPRB, 0.36946E+01_JPRB, 0.30465E+01_JPRB ,&
&  0.27613E+01_JPRB, 0.24027E+01_JPRB, 0.21211E+01_JPRB, 0.21368E+01_JPRB, 0.19487E+01_JPRB, 0.19381E+01_JPRB, 0.20022E+01_JPRB ,&
&  0.20554E+01_JPRB, 0.21134E+01_JPRB, 0.21627E+01_JPRB, 0.21277E+01_JPRB, 0.20383E+01_JPRB, 0.19939E+01_JPRB, 0.21384E+01_JPRB ,&
&  0.27018E+01_JPRB, 0.38755E+01_JPRB, 0.57805E+01_JPRB, 0.59371E+01_JPRB, 0.61281E+01_JPRB, 0.42962E+01_JPRB, 0.30997E+01_JPRB ,&
&  0.25091E+01_JPRB, 0.23762E+01_JPRB, 0.22274E+01_JPRB, 0.19927E+01_JPRB, 0.17412E+01_JPRB, 0.15896E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.12748E+01_JPRB, 0.11575E+01_JPRB, 0.10847E+01_JPRB, 0.10345E+01_JPRB, 0.10042E+01_JPRB, 0.98773E+00_JPRB ,&
&  0.99169E+00_JPRB, 0.99059E+00_JPRB, 0.10010E+01_JPRB, 0.10191E+01_JPRB, 0.10452E+01_JPRB, 0.10870E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.11880E+01_JPRB, 0.11927E+01_JPRB, 0.11899E+01_JPRB, 0.11644E+01_JPRB, 0.11483E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.11769E+01_JPRB, 0.11986E+01_JPRB, 0.12076E+01_JPRB, 0.12478E+01_JPRB, 0.12688E+01_JPRB, 0.12746E+01_JPRB, 0.12911E+01_JPRB ,&
&  0.13109E+01_JPRB, 0.12570E+01_JPRB, 0.11081E+01_JPRB, 0.88105E+00_JPRB, 0.80931E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.87873E+00_JPRB, 0.84870E+00_JPRB, 0.83223E+00_JPRB, 0.83331E+00_JPRB, 0.83525E+00_JPRB, 0.81659E+00_JPRB, 0.79060E+00_JPRB ,&
&  0.75547E+00_JPRB, 0.72859E+00_JPRB, 0.69117E+00_JPRB, 0.65379E+00_JPRB, 0.64619E+00_JPRB, 0.66950E+00_JPRB, 0.71518E+00_JPRB ,&
&  0.75622E+00_JPRB, 0.77189E+00_JPRB, 0.78214E+00_JPRB, 0.79008E+00_JPRB, 0.79778E+00_JPRB, 0.80117E+00_JPRB, 0.80215E+00_JPRB ,&
&  0.81444E+00_JPRB, 0.84002E+00_JPRB, 0.87857E+00_JPRB, 0.92996E+00_JPRB, 0.10114E+01_JPRB, 0.11679E+01_JPRB, 0.14848E+01_JPRB ,&
&  0.21577E+01_JPRB, 0.25549E+01_JPRB, 0.11711E+01_JPRB, 0.25535E+01_JPRB, 0.37321E+01_JPRB, 0.37616E+01_JPRB, 0.35324E+01_JPRB ,&
&  0.31956E+01_JPRB, 0.26694E+01_JPRB, 0.24492E+01_JPRB, 0.21893E+01_JPRB, 0.19320E+01_JPRB, 0.18477E+01_JPRB, 0.18449E+01_JPRB ,&
&  0.18524E+01_JPRB, 0.18061E+01_JPRB, 0.17976E+01_JPRB, 0.18329E+01_JPRB, 0.17961E+01_JPRB, 0.17182E+01_JPRB, 0.17313E+01_JPRB ,&
&  0.19988E+01_JPRB, 0.28044E+01_JPRB, 0.45873E+01_JPRB, 0.56797E+01_JPRB, 0.60438E+01_JPRB, 0.46928E+01_JPRB, 0.34797E+01_JPRB ,&
&  0.29971E+01_JPRB, 0.30336E+01_JPRB, 0.27738E+01_JPRB, 0.22922E+01_JPRB, 0.18954E+01_JPRB, 0.17677E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.14334E+01_JPRB, 0.13391E+01_JPRB, 0.12602E+01_JPRB, 0.11976E+01_JPRB, 0.11341E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10613E+01_JPRB, 0.10594E+01_JPRB, 0.10582E+01_JPRB, 0.10563E+01_JPRB, 0.10652E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.11438E+01_JPRB, 0.11497E+01_JPRB, 0.11277E+01_JPRB, 0.11125E+01_JPRB, 0.11146E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.11588E+01_JPRB, 0.11551E+01_JPRB, 0.11639E+01_JPRB, 0.12242E+01_JPRB, 0.12443E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.13228E+01_JPRB, 0.13658E+01_JPRB, 0.13111E+01_JPRB, 0.10034E+01_JPRB, 0.93295E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.96953E+00_JPRB, 0.93441E+00_JPRB, 0.93029E+00_JPRB, 0.93297E+00_JPRB, 0.93242E+00_JPRB, 0.92209E+00_JPRB, 0.89767E+00_JPRB ,&
&  0.86556E+00_JPRB, 0.83856E+00_JPRB, 0.80706E+00_JPRB, 0.76573E+00_JPRB, 0.70687E+00_JPRB, 0.65638E+00_JPRB, 0.65134E+00_JPRB ,&
&  0.68119E+00_JPRB, 0.68787E+00_JPRB, 0.69304E+00_JPRB, 0.70184E+00_JPRB, 0.71411E+00_JPRB, 0.72828E+00_JPRB, 0.73868E+00_JPRB ,&
&  0.74484E+00_JPRB, 0.75625E+00_JPRB, 0.77248E+00_JPRB, 0.79281E+00_JPRB, 0.82740E+00_JPRB, 0.90058E+00_JPRB, 0.10491E+01_JPRB ,&
&  0.14526E+01_JPRB, 0.20783E+01_JPRB, 0.22388E+01_JPRB, 0.37622E+01_JPRB, 0.43282E+01_JPRB, 0.41311E+01_JPRB, 0.41747E+01_JPRB ,&
&  0.40894E+01_JPRB, 0.35820E+01_JPRB, 0.30327E+01_JPRB, 0.23566E+01_JPRB, 0.19994E+01_JPRB, 0.18349E+01_JPRB, 0.17453E+01_JPRB ,&
&  0.16901E+01_JPRB, 0.16199E+01_JPRB, 0.15290E+01_JPRB, 0.15004E+01_JPRB, 0.14851E+01_JPRB, 0.14533E+01_JPRB, 0.14570E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.20571E+01_JPRB, 0.33281E+01_JPRB, 0.53533E+01_JPRB, 0.60006E+01_JPRB, 0.53712E+01_JPRB, 0.43980E+01_JPRB ,&
&  0.43728E+01_JPRB, 0.40921E+01_JPRB, 0.36365E+01_JPRB, 0.28550E+01_JPRB, 0.22454E+01_JPRB, 0.21795E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.16808E+01_JPRB, 0.15503E+01_JPRB, 0.14702E+01_JPRB, 0.14117E+01_JPRB, 0.13597E+01_JPRB, 0.12903E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.11498E+01_JPRB, 0.11224E+01_JPRB, 0.11068E+01_JPRB, 0.10984E+01_JPRB, 0.10852E+01_JPRB, 0.10735E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.11041E+01_JPRB, 0.10713E+01_JPRB, 0.10656E+01_JPRB, 0.10758E+01_JPRB, 0.10800E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.11115E+01_JPRB, 0.11374E+01_JPRB, 0.11453E+01_JPRB, 0.11736E+01_JPRB, 0.12062E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.12306E+01_JPRB, 0.12142E+01_JPRB, 0.11579E+01_JPRB, 0.10662E+01_JPRB, 0.10181E+01_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.10161E+01_JPRB, 0.98469E+00_JPRB, 0.96910E+00_JPRB, 0.95947E+00_JPRB, 0.95546E+00_JPRB, 0.95149E+00_JPRB, 0.93245E+00_JPRB ,&
&  0.91023E+00_JPRB, 0.89535E+00_JPRB, 0.87046E+00_JPRB, 0.84377E+00_JPRB, 0.79244E+00_JPRB, 0.72581E+00_JPRB, 0.69433E+00_JPRB ,&
&  0.67853E+00_JPRB, 0.67320E+00_JPRB, 0.68124E+00_JPRB, 0.68651E+00_JPRB, 0.68970E+00_JPRB, 0.69726E+00_JPRB, 0.70495E+00_JPRB ,&
&  0.71450E+00_JPRB, 0.72033E+00_JPRB, 0.73018E+00_JPRB, 0.74075E+00_JPRB, 0.75795E+00_JPRB, 0.80935E+00_JPRB, 0.90023E+00_JPRB ,&
&  0.11509E+01_JPRB, 0.18823E+01_JPRB, 0.16290E+01_JPRB, 0.36558E+01_JPRB, 0.41291E+01_JPRB, 0.41801E+01_JPRB, 0.39677E+01_JPRB ,&
&  0.40170E+01_JPRB, 0.39945E+01_JPRB, 0.30444E+01_JPRB, 0.24324E+01_JPRB, 0.20284E+01_JPRB, 0.18245E+01_JPRB, 0.17043E+01_JPRB ,&
&  0.16190E+01_JPRB, 0.15322E+01_JPRB, 0.14231E+01_JPRB, 0.13357E+01_JPRB, 0.12951E+01_JPRB, 0.12675E+01_JPRB, 0.12680E+01_JPRB ,&
&  0.13127E+01_JPRB, 0.15648E+01_JPRB, 0.23832E+01_JPRB, 0.41715E+01_JPRB, 0.57628E+01_JPRB, 0.59529E+01_JPRB, 0.53181E+01_JPRB ,&
&  0.53472E+01_JPRB, 0.54085E+01_JPRB, 0.48635E+01_JPRB, 0.37329E+01_JPRB, 0.29324E+01_JPRB, 0.25388E+01_JPRB, 0.21345E+01_JPRB ,&
&  0.18754E+01_JPRB, 0.17161E+01_JPRB, 0.16187E+01_JPRB, 0.15366E+01_JPRB, 0.14676E+01_JPRB, 0.13932E+01_JPRB, 0.12992E+01_JPRB ,&
&  0.12159E+01_JPRB, 0.11538E+01_JPRB, 0.11252E+01_JPRB, 0.11119E+01_JPRB, 0.11069E+01_JPRB, 0.10987E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.10883E+01_JPRB, 0.10972E+01_JPRB, 0.10447E+01_JPRB, 0.10290E+01_JPRB, 0.10391E+01_JPRB, 0.10452E+01_JPRB, 0.10562E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.10974E+01_JPRB, 0.11080E+01_JPRB, 0.11029E+01_JPRB, 0.11273E+01_JPRB, 0.11840E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11821E+01_JPRB, 0.11543E+01_JPRB, 0.11224E+01_JPRB, 0.10811E+01_JPRB, 0.10474E+01_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.10287E+01_JPRB, 0.10062E+01_JPRB, 0.98007E+00_JPRB, 0.95708E+00_JPRB, 0.95427E+00_JPRB, 0.95417E+00_JPRB, 0.93938E+00_JPRB ,&
&  0.92380E+00_JPRB, 0.90296E+00_JPRB, 0.87062E+00_JPRB, 0.83460E+00_JPRB, 0.79438E+00_JPRB, 0.74584E+00_JPRB, 0.70214E+00_JPRB ,&
&  0.68275E+00_JPRB, 0.66660E+00_JPRB, 0.66741E+00_JPRB, 0.67434E+00_JPRB, 0.68377E+00_JPRB, 0.68948E+00_JPRB, 0.69686E+00_JPRB ,&
&  0.70443E+00_JPRB, 0.70960E+00_JPRB, 0.71542E+00_JPRB, 0.72081E+00_JPRB, 0.72809E+00_JPRB, 0.76001E+00_JPRB, 0.85657E+00_JPRB ,&
&  0.10658E+01_JPRB, 0.22502E+01_JPRB, 0.16593E+01_JPRB, 0.30016E+01_JPRB, 0.34425E+01_JPRB, 0.36899E+01_JPRB, 0.35198E+01_JPRB ,&
&  0.35847E+01_JPRB, 0.33746E+01_JPRB, 0.27934E+01_JPRB, 0.23482E+01_JPRB, 0.20269E+01_JPRB, 0.18211E+01_JPRB, 0.16799E+01_JPRB ,&
&  0.15807E+01_JPRB, 0.14705E+01_JPRB, 0.13529E+01_JPRB, 0.12465E+01_JPRB, 0.11888E+01_JPRB, 0.11582E+01_JPRB, 0.11472E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.12709E+01_JPRB, 0.17192E+01_JPRB, 0.31570E+01_JPRB, 0.48916E+01_JPRB, 0.59131E+01_JPRB, 0.61032E+01_JPRB ,&
&  0.62029E+01_JPRB, 0.66307E+01_JPRB, 0.60488E+01_JPRB, 0.47841E+01_JPRB, 0.37851E+01_JPRB, 0.30653E+01_JPRB, 0.25143E+01_JPRB ,&
&  0.21772E+01_JPRB, 0.19764E+01_JPRB, 0.18371E+01_JPRB, 0.17200E+01_JPRB, 0.16052E+01_JPRB, 0.14769E+01_JPRB, 0.13426E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.11693E+01_JPRB, 0.11279E+01_JPRB, 0.11174E+01_JPRB, 0.11180E+01_JPRB, 0.11177E+01_JPRB, 0.11172E+01_JPRB ,&
&  0.11156E+01_JPRB, 0.11161E+01_JPRB, 0.10635E+01_JPRB, 0.10399E+01_JPRB, 0.10374E+01_JPRB, 0.10455E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.10963E+01_JPRB, 0.10849E+01_JPRB, 0.10839E+01_JPRB, 0.11058E+01_JPRB, 0.11796E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.11637E+01_JPRB, 0.11269E+01_JPRB, 0.10966E+01_JPRB, 0.10776E+01_JPRB, 0.10546E+01_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.10314E+01_JPRB, 0.10080E+01_JPRB, 0.99330E+00_JPRB, 0.98243E+00_JPRB, 0.98362E+00_JPRB, 0.97858E+00_JPRB, 0.95699E+00_JPRB ,&
&  0.92292E+00_JPRB, 0.88251E+00_JPRB, 0.84086E+00_JPRB, 0.80668E+00_JPRB, 0.78009E+00_JPRB, 0.75131E+00_JPRB, 0.72705E+00_JPRB ,&
&  0.70695E+00_JPRB, 0.68504E+00_JPRB, 0.67112E+00_JPRB, 0.66442E+00_JPRB, 0.66605E+00_JPRB, 0.67128E+00_JPRB, 0.67961E+00_JPRB ,&
&  0.69010E+00_JPRB, 0.69702E+00_JPRB, 0.69849E+00_JPRB, 0.70110E+00_JPRB, 0.70155E+00_JPRB, 0.73016E+00_JPRB, 0.83016E+00_JPRB ,&
&  0.10784E+01_JPRB, 0.10391E+01_JPRB, 0.19064E+01_JPRB, 0.23576E+01_JPRB, 0.26758E+01_JPRB, 0.29968E+01_JPRB, 0.32313E+01_JPRB ,&
&  0.33453E+01_JPRB, 0.29788E+01_JPRB, 0.25077E+01_JPRB, 0.21811E+01_JPRB, 0.19198E+01_JPRB, 0.17338E+01_JPRB, 0.15933E+01_JPRB ,&
&  0.14578E+01_JPRB, 0.13565E+01_JPRB, 0.12742E+01_JPRB, 0.11744E+01_JPRB, 0.11106E+01_JPRB, 0.10834E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10995E+01_JPRB, 0.13165E+01_JPRB, 0.20738E+01_JPRB, 0.39675E+01_JPRB, 0.54573E+01_JPRB, 0.64004E+01_JPRB ,&
&  0.75055E+01_JPRB, 0.80244E+01_JPRB, 0.71802E+01_JPRB, 0.57990E+01_JPRB, 0.44490E+01_JPRB, 0.35405E+01_JPRB, 0.27941E+01_JPRB ,&
&  0.23625E+01_JPRB, 0.21219E+01_JPRB, 0.19809E+01_JPRB, 0.18668E+01_JPRB, 0.17476E+01_JPRB, 0.15831E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.11685E+01_JPRB, 0.11166E+01_JPRB, 0.10879E+01_JPRB, 0.10760E+01_JPRB, 0.10798E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.11246E+01_JPRB, 0.10798E+01_JPRB, 0.10471E+01_JPRB, 0.10418E+01_JPRB, 0.10569E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.10591E+01_JPRB, 0.10417E+01_JPRB, 0.10475E+01_JPRB, 0.10677E+01_JPRB, 0.11636E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.11571E+01_JPRB, 0.11238E+01_JPRB, 0.10831E+01_JPRB, 0.10690E+01_JPRB, 0.10587E+01_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.10300E+01_JPRB, 0.10105E+01_JPRB, 0.10103E+01_JPRB, 0.10312E+01_JPRB, 0.10507E+01_JPRB, 0.10358E+01_JPRB, 0.97716E+00_JPRB ,&
&  0.92030E+00_JPRB, 0.87120E+00_JPRB, 0.82643E+00_JPRB, 0.79160E+00_JPRB, 0.76830E+00_JPRB, 0.75093E+00_JPRB, 0.73379E+00_JPRB ,&
&  0.72039E+00_JPRB, 0.70791E+00_JPRB, 0.68619E+00_JPRB, 0.66638E+00_JPRB, 0.65706E+00_JPRB, 0.65522E+00_JPRB, 0.65961E+00_JPRB ,&
&  0.66523E+00_JPRB, 0.67146E+00_JPRB, 0.67544E+00_JPRB, 0.67822E+00_JPRB, 0.67963E+00_JPRB, 0.70433E+00_JPRB, 0.79068E+00_JPRB ,&
&  0.10745E+01_JPRB, 0.16372E+01_JPRB, 0.19670E+01_JPRB, 0.19980E+01_JPRB, 0.21761E+01_JPRB, 0.24962E+01_JPRB, 0.28946E+01_JPRB ,&
&  0.28918E+01_JPRB, 0.26193E+01_JPRB, 0.22128E+01_JPRB, 0.19283E+01_JPRB, 0.17118E+01_JPRB, 0.15577E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.12963E+01_JPRB, 0.12167E+01_JPRB, 0.11382E+01_JPRB, 0.10693E+01_JPRB, 0.10403E+01_JPRB, 0.10349E+01_JPRB, 0.10338E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10241E+01_JPRB, 0.11104E+01_JPRB, 0.15955E+01_JPRB, 0.31325E+01_JPRB, 0.41715E+01_JPRB, 0.59434E+01_JPRB ,&
&  0.77627E+01_JPRB, 0.77424E+01_JPRB, 0.70317E+01_JPRB, 0.58922E+01_JPRB, 0.46234E+01_JPRB, 0.35535E+01_JPRB, 0.28730E+01_JPRB ,&
&  0.24470E+01_JPRB, 0.21136E+01_JPRB, 0.19107E+01_JPRB, 0.17809E+01_JPRB, 0.16738E+01_JPRB, 0.15715E+01_JPRB, 0.14474E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.12098E+01_JPRB, 0.11332E+01_JPRB, 0.10812E+01_JPRB, 0.10541E+01_JPRB, 0.10476E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.10595E+01_JPRB, 0.10648E+01_JPRB, 0.10459E+01_JPRB, 0.10463E+01_JPRB, 0.10597E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10407E+01_JPRB, 0.10187E+01_JPRB, 0.10137E+01_JPRB, 0.10145E+01_JPRB, 0.10626E+01_JPRB, 0.11648E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11144E+01_JPRB, 0.10874E+01_JPRB, 0.10603E+01_JPRB, 0.10577E+01_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.10172E+01_JPRB, 0.99506E+00_JPRB, 0.98884E+00_JPRB, 0.10183E+01_JPRB, 0.10523E+01_JPRB, 0.10464E+01_JPRB, 0.99235E+00_JPRB ,&
&  0.91687E+00_JPRB, 0.85612E+00_JPRB, 0.81386E+00_JPRB, 0.78406E+00_JPRB, 0.76090E+00_JPRB, 0.74066E+00_JPRB, 0.71883E+00_JPRB ,&
&  0.70149E+00_JPRB, 0.69096E+00_JPRB, 0.68338E+00_JPRB, 0.67397E+00_JPRB, 0.65862E+00_JPRB, 0.64683E+00_JPRB, 0.64270E+00_JPRB ,&
&  0.64052E+00_JPRB, 0.64010E+00_JPRB, 0.64562E+00_JPRB, 0.65252E+00_JPRB, 0.65630E+00_JPRB, 0.67172E+00_JPRB, 0.72837E+00_JPRB ,&
&  0.92824E+00_JPRB, 0.14203E+01_JPRB, 0.14406E+01_JPRB, 0.15962E+01_JPRB, 0.18964E+01_JPRB, 0.21726E+01_JPRB, 0.24302E+01_JPRB ,&
&  0.24342E+01_JPRB, 0.22926E+01_JPRB, 0.20140E+01_JPRB, 0.17352E+01_JPRB, 0.15085E+01_JPRB, 0.13691E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11050E+01_JPRB, 0.10337E+01_JPRB, 0.98036E+00_JPRB, 0.94429E+00_JPRB, 0.92964E+00_JPRB, 0.93223E+00_JPRB ,&
&  0.93504E+00_JPRB, 0.93365E+00_JPRB, 0.94798E+00_JPRB, 0.11171E+01_JPRB, 0.19089E+01_JPRB, 0.30444E+01_JPRB, 0.42939E+01_JPRB ,&
&  0.47374E+01_JPRB, 0.49640E+01_JPRB, 0.49226E+01_JPRB, 0.46233E+01_JPRB, 0.40690E+01_JPRB, 0.35148E+01_JPRB, 0.29161E+01_JPRB ,&
&  0.24727E+01_JPRB, 0.21735E+01_JPRB, 0.19556E+01_JPRB, 0.17699E+01_JPRB, 0.16027E+01_JPRB, 0.14681E+01_JPRB, 0.13740E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.12429E+01_JPRB, 0.11843E+01_JPRB, 0.11307E+01_JPRB, 0.10903E+01_JPRB, 0.10678E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10336E+01_JPRB, 0.10260E+01_JPRB, 0.10248E+01_JPRB, 0.10291E+01_JPRB, 0.10271E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.10235E+01_JPRB, 0.10186E+01_JPRB, 0.10181E+01_JPRB, 0.10555E+01_JPRB, 0.11298E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.11358E+01_JPRB, 0.11085E+01_JPRB, 0.10845E+01_JPRB, 0.10667E+01_JPRB, 0.10527E+01_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.10950E+01_JPRB, 0.10094E+01_JPRB, 0.96101E+00_JPRB, 0.95099E+00_JPRB, 0.96123E+00_JPRB, 0.94994E+00_JPRB, 0.91953E+00_JPRB ,&
&  0.88462E+00_JPRB, 0.85201E+00_JPRB, 0.82232E+00_JPRB, 0.78976E+00_JPRB, 0.75785E+00_JPRB, 0.73012E+00_JPRB, 0.70392E+00_JPRB ,&
&  0.67933E+00_JPRB, 0.66460E+00_JPRB, 0.65475E+00_JPRB, 0.64788E+00_JPRB, 0.64286E+00_JPRB, 0.63416E+00_JPRB, 0.62527E+00_JPRB ,&
&  0.61846E+00_JPRB, 0.60852E+00_JPRB, 0.60487E+00_JPRB, 0.61212E+00_JPRB, 0.62108E+00_JPRB, 0.63428E+00_JPRB, 0.67663E+00_JPRB ,&
&  0.83037E+00_JPRB, 0.10489E+01_JPRB, 0.12067E+01_JPRB, 0.13502E+01_JPRB, 0.14974E+01_JPRB, 0.16678E+01_JPRB, 0.17216E+01_JPRB ,&
&  0.16357E+01_JPRB, 0.16431E+01_JPRB, 0.15990E+01_JPRB, 0.14577E+01_JPRB, 0.12987E+01_JPRB, 0.11915E+01_JPRB, 0.11153E+01_JPRB ,&
&  0.10508E+01_JPRB, 0.98571E+00_JPRB, 0.92135E+00_JPRB, 0.87027E+00_JPRB, 0.83943E+00_JPRB, 0.82712E+00_JPRB, 0.82694E+00_JPRB ,&
&  0.83191E+00_JPRB, 0.83983E+00_JPRB, 0.84533E+00_JPRB, 0.86948E+00_JPRB, 0.10493E+01_JPRB, 0.15457E+01_JPRB, 0.23183E+01_JPRB ,&
&  0.27204E+01_JPRB, 0.28782E+01_JPRB, 0.29985E+01_JPRB, 0.30243E+01_JPRB, 0.28384E+01_JPRB, 0.26265E+01_JPRB, 0.23873E+01_JPRB ,&
&  0.21478E+01_JPRB, 0.18955E+01_JPRB, 0.16792E+01_JPRB, 0.15090E+01_JPRB, 0.13764E+01_JPRB, 0.12850E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.12099E+01_JPRB, 0.11966E+01_JPRB, 0.11842E+01_JPRB, 0.11591E+01_JPRB, 0.11269E+01_JPRB, 0.11024E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10468E+01_JPRB, 0.10296E+01_JPRB, 0.10159E+01_JPRB, 0.10079E+01_JPRB, 0.99978E+00_JPRB, 0.99543E+00_JPRB ,&
&  0.99516E+00_JPRB, 0.10029E+01_JPRB, 0.98851E+00_JPRB, 0.99419E+00_JPRB, 0.10579E+01_JPRB, 0.10802E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.11216E+01_JPRB, 0.11073E+01_JPRB, 0.10727E+01_JPRB, 0.10601E+01_JPRB, 0.11550E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.99073E+00_JPRB, 0.95164E+00_JPRB, 0.90089E+00_JPRB, 0.87300E+00_JPRB, 0.85516E+00_JPRB, 0.82918E+00_JPRB, 0.80829E+00_JPRB ,&
&  0.79097E+00_JPRB, 0.77289E+00_JPRB, 0.75007E+00_JPRB, 0.72357E+00_JPRB, 0.69806E+00_JPRB, 0.67935E+00_JPRB, 0.66401E+00_JPRB ,&
&  0.64971E+00_JPRB, 0.63601E+00_JPRB, 0.62356E+00_JPRB, 0.61419E+00_JPRB, 0.60449E+00_JPRB, 0.59708E+00_JPRB, 0.59100E+00_JPRB ,&
&  0.58401E+00_JPRB, 0.57556E+00_JPRB, 0.56467E+00_JPRB, 0.56812E+00_JPRB, 0.58016E+00_JPRB, 0.59745E+00_JPRB, 0.62048E+00_JPRB ,&
&  0.71205E+00_JPRB, 0.81583E+00_JPRB, 0.95601E+00_JPRB, 0.11098E+01_JPRB, 0.12228E+01_JPRB, 0.13054E+01_JPRB, 0.12985E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.12339E+01_JPRB, 0.12032E+01_JPRB, 0.11418E+01_JPRB, 0.10972E+01_JPRB, 0.10058E+01_JPRB, 0.92246E+00_JPRB ,&
&  0.86365E+00_JPRB, 0.82546E+00_JPRB, 0.79426E+00_JPRB, 0.76392E+00_JPRB, 0.74603E+00_JPRB, 0.74223E+00_JPRB, 0.74906E+00_JPRB ,&
&  0.75647E+00_JPRB, 0.76617E+00_JPRB, 0.77492E+00_JPRB, 0.78616E+00_JPRB, 0.81381E+00_JPRB, 0.95777E+00_JPRB, 0.13336E+01_JPRB ,&
&  0.16312E+01_JPRB, 0.18242E+01_JPRB, 0.19528E+01_JPRB, 0.20065E+01_JPRB, 0.19558E+01_JPRB, 0.18491E+01_JPRB, 0.17125E+01_JPRB ,&
&  0.15642E+01_JPRB, 0.14059E+01_JPRB, 0.12612E+01_JPRB, 0.11481E+01_JPRB, 0.10612E+01_JPRB, 0.10040E+01_JPRB, 0.97444E+00_JPRB ,&
&  0.97185E+00_JPRB, 0.98874E+00_JPRB, 0.10142E+01_JPRB, 0.10381E+01_JPRB, 0.10503E+01_JPRB, 0.10485E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10250E+01_JPRB, 0.10051E+01_JPRB, 0.98258E+00_JPRB, 0.96308E+00_JPRB, 0.94703E+00_JPRB, 0.93497E+00_JPRB, 0.92713E+00_JPRB ,&
&  0.92241E+00_JPRB, 0.92240E+00_JPRB, 0.92813E+00_JPRB, 0.95058E+00_JPRB, 0.96182E+00_JPRB, 0.98086E+00_JPRB, 0.99260E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.10055E+01_JPRB, 0.98864E+00_JPRB, 0.95288E+00_JPRB, 0.95347E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.86253E+00_JPRB, 0.85033E+00_JPRB, 0.82306E+00_JPRB, 0.79407E+00_JPRB, 0.77012E+00_JPRB, 0.74752E+00_JPRB, 0.72697E+00_JPRB ,&
&  0.70857E+00_JPRB, 0.69088E+00_JPRB, 0.67161E+00_JPRB, 0.65352E+00_JPRB, 0.63616E+00_JPRB, 0.62212E+00_JPRB, 0.60940E+00_JPRB ,&
&  0.60031E+00_JPRB, 0.59337E+00_JPRB, 0.58624E+00_JPRB, 0.57823E+00_JPRB, 0.57035E+00_JPRB, 0.56282E+00_JPRB, 0.55504E+00_JPRB ,&
&  0.54885E+00_JPRB, 0.54527E+00_JPRB, 0.53576E+00_JPRB, 0.53282E+00_JPRB, 0.54189E+00_JPRB, 0.55710E+00_JPRB, 0.57246E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.64456E+00_JPRB, 0.67580E+00_JPRB, 0.81333E+00_JPRB, 0.92846E+00_JPRB, 0.10532E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.11153E+01_JPRB, 0.10759E+01_JPRB, 0.10240E+01_JPRB, 0.94519E+00_JPRB, 0.87995E+00_JPRB, 0.83189E+00_JPRB ,&
&  0.78962E+00_JPRB, 0.74985E+00_JPRB, 0.71912E+00_JPRB, 0.69564E+00_JPRB, 0.67889E+00_JPRB, 0.67254E+00_JPRB, 0.67551E+00_JPRB ,&
&  0.68065E+00_JPRB, 0.68577E+00_JPRB, 0.69250E+00_JPRB, 0.70493E+00_JPRB, 0.72164E+00_JPRB, 0.75535E+00_JPRB, 0.88071E+00_JPRB ,&
&  0.10812E+01_JPRB, 0.12406E+01_JPRB, 0.13825E+01_JPRB, 0.14647E+01_JPRB, 0.14625E+01_JPRB, 0.13955E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.10257E+01_JPRB, 0.95858E+00_JPRB, 0.91684E+00_JPRB, 0.89367E+00_JPRB, 0.87737E+00_JPRB, 0.86333E+00_JPRB ,&
&  0.85074E+00_JPRB, 0.84417E+00_JPRB, 0.84763E+00_JPRB, 0.86037E+00_JPRB, 0.87947E+00_JPRB, 0.89463E+00_JPRB, 0.90004E+00_JPRB ,&
&  0.89768E+00_JPRB, 0.89032E+00_JPRB, 0.88232E+00_JPRB, 0.87203E+00_JPRB, 0.85961E+00_JPRB, 0.84596E+00_JPRB, 0.83718E+00_JPRB ,&
&  0.83459E+00_JPRB, 0.83757E+00_JPRB, 0.84345E+00_JPRB, 0.84658E+00_JPRB, 0.87574E+00_JPRB, 0.88339E+00_JPRB, 0.88665E+00_JPRB ,&
&  0.88819E+00_JPRB, 0.88285E+00_JPRB, 0.87200E+00_JPRB, 0.78097E+00_JPRB, 0.84598E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.73861E+00_JPRB, 0.73571E+00_JPRB, 0.71791E+00_JPRB, 0.69778E+00_JPRB, 0.67730E+00_JPRB, 0.65754E+00_JPRB, 0.64195E+00_JPRB ,&
&  0.62936E+00_JPRB, 0.61422E+00_JPRB, 0.60112E+00_JPRB, 0.59189E+00_JPRB, 0.58243E+00_JPRB, 0.57376E+00_JPRB, 0.56403E+00_JPRB ,&
&  0.55610E+00_JPRB, 0.55072E+00_JPRB, 0.54655E+00_JPRB, 0.54187E+00_JPRB, 0.53680E+00_JPRB, 0.53105E+00_JPRB, 0.52557E+00_JPRB ,&
&  0.52072E+00_JPRB, 0.51751E+00_JPRB, 0.51161E+00_JPRB, 0.50760E+00_JPRB, 0.51160E+00_JPRB, 0.52254E+00_JPRB, 0.53298E+00_JPRB ,&
&  0.58372E+00_JPRB, 0.53886E+00_JPRB, 0.55491E+00_JPRB, 0.61736E+00_JPRB, 0.69413E+00_JPRB, 0.79387E+00_JPRB, 0.87612E+00_JPRB ,&
&  0.91001E+00_JPRB, 0.94043E+00_JPRB, 0.92146E+00_JPRB, 0.88700E+00_JPRB, 0.83010E+00_JPRB, 0.77516E+00_JPRB, 0.73434E+00_JPRB ,&
&  0.71057E+00_JPRB, 0.69232E+00_JPRB, 0.67144E+00_JPRB, 0.64868E+00_JPRB, 0.62825E+00_JPRB, 0.61434E+00_JPRB, 0.60857E+00_JPRB ,&
&  0.61000E+00_JPRB, 0.61286E+00_JPRB, 0.61364E+00_JPRB, 0.61795E+00_JPRB, 0.62919E+00_JPRB, 0.64721E+00_JPRB, 0.68335E+00_JPRB ,&
&  0.75917E+00_JPRB, 0.86107E+00_JPRB, 0.95194E+00_JPRB, 0.10374E+01_JPRB, 0.10910E+01_JPRB, 0.11044E+01_JPRB, 0.10500E+01_JPRB ,&
&  0.97443E+00_JPRB, 0.90379E+00_JPRB, 0.85671E+00_JPRB, 0.83082E+00_JPRB, 0.81503E+00_JPRB, 0.80334E+00_JPRB, 0.78689E+00_JPRB ,&
&  0.76636E+00_JPRB, 0.74313E+00_JPRB, 0.72324E+00_JPRB, 0.70392E+00_JPRB, 0.69086E+00_JPRB, 0.68759E+00_JPRB, 0.69176E+00_JPRB ,&
&  0.69783E+00_JPRB, 0.70198E+00_JPRB, 0.70526E+00_JPRB, 0.70888E+00_JPRB, 0.71110E+00_JPRB, 0.71300E+00_JPRB, 0.71442E+00_JPRB ,&
&  0.71891E+00_JPRB, 0.72573E+00_JPRB, 0.73250E+00_JPRB, 0.74782E+00_JPRB, 0.75868E+00_JPRB, 0.76458E+00_JPRB, 0.76755E+00_JPRB ,&
&  0.76689E+00_JPRB, 0.76976E+00_JPRB, 0.67298E+00_JPRB, 0.71317E+00_JPRB, 0.72951E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.64385E+00_JPRB, 0.64234E+00_JPRB, 0.63627E+00_JPRB, 0.62173E+00_JPRB, 0.60478E+00_JPRB, 0.58979E+00_JPRB, 0.57971E+00_JPRB ,&
&  0.57280E+00_JPRB, 0.56394E+00_JPRB, 0.55639E+00_JPRB, 0.55118E+00_JPRB, 0.54497E+00_JPRB, 0.53750E+00_JPRB, 0.52717E+00_JPRB ,&
&  0.51814E+00_JPRB, 0.51100E+00_JPRB, 0.50592E+00_JPRB, 0.50295E+00_JPRB, 0.50114E+00_JPRB, 0.49925E+00_JPRB, 0.49634E+00_JPRB ,&
&  0.49360E+00_JPRB, 0.49237E+00_JPRB, 0.49094E+00_JPRB, 0.48933E+00_JPRB, 0.49006E+00_JPRB, 0.49694E+00_JPRB, 0.50576E+00_JPRB ,&
&  0.50609E+00_JPRB, 0.48634E+00_JPRB, 0.50004E+00_JPRB, 0.52779E+00_JPRB, 0.56732E+00_JPRB, 0.63509E+00_JPRB, 0.70635E+00_JPRB ,&
&  0.74853E+00_JPRB, 0.77134E+00_JPRB, 0.75302E+00_JPRB, 0.72848E+00_JPRB, 0.70345E+00_JPRB, 0.67382E+00_JPRB, 0.64942E+00_JPRB ,&
&  0.62996E+00_JPRB, 0.61675E+00_JPRB, 0.60593E+00_JPRB, 0.59109E+00_JPRB, 0.57375E+00_JPRB, 0.55973E+00_JPRB, 0.55137E+00_JPRB ,&
&  0.54852E+00_JPRB, 0.55055E+00_JPRB, 0.55360E+00_JPRB, 0.55641E+00_JPRB, 0.56162E+00_JPRB, 0.57208E+00_JPRB, 0.58771E+00_JPRB ,&
&  0.60667E+00_JPRB, 0.64157E+00_JPRB, 0.68968E+00_JPRB, 0.73714E+00_JPRB, 0.78251E+00_JPRB, 0.81502E+00_JPRB, 0.83318E+00_JPRB ,&
&  0.83231E+00_JPRB, 0.81196E+00_JPRB, 0.78385E+00_JPRB, 0.75443E+00_JPRB, 0.72035E+00_JPRB, 0.68002E+00_JPRB, 0.63752E+00_JPRB ,&
&  0.60196E+00_JPRB, 0.57500E+00_JPRB, 0.55734E+00_JPRB, 0.54890E+00_JPRB, 0.54554E+00_JPRB, 0.54763E+00_JPRB, 0.55344E+00_JPRB ,&
&  0.56158E+00_JPRB, 0.57055E+00_JPRB, 0.57752E+00_JPRB, 0.58477E+00_JPRB, 0.59111E+00_JPRB, 0.59791E+00_JPRB, 0.60505E+00_JPRB ,&
&  0.61295E+00_JPRB, 0.62041E+00_JPRB, 0.62754E+00_JPRB, 0.63922E+00_JPRB, 0.64882E+00_JPRB, 0.65765E+00_JPRB, 0.66156E+00_JPRB ,&
&  0.65772E+00_JPRB, 0.65454E+00_JPRB, 0.65051E+00_JPRB, 0.64498E+00_JPRB, 0.64590E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.57697E+00_JPRB, 0.57500E+00_JPRB, 0.57260E+00_JPRB, 0.56860E+00_JPRB, 0.56154E+00_JPRB, 0.55174E+00_JPRB, 0.54368E+00_JPRB ,&
&  0.53826E+00_JPRB, 0.53257E+00_JPRB, 0.52752E+00_JPRB, 0.52154E+00_JPRB, 0.51442E+00_JPRB, 0.50568E+00_JPRB, 0.49452E+00_JPRB ,&
&  0.48420E+00_JPRB, 0.47485E+00_JPRB, 0.46862E+00_JPRB, 0.46560E+00_JPRB, 0.46531E+00_JPRB, 0.46508E+00_JPRB, 0.46286E+00_JPRB ,&
&  0.46137E+00_JPRB, 0.46119E+00_JPRB, 0.46140E+00_JPRB, 0.46089E+00_JPRB, 0.46099E+00_JPRB, 0.46596E+00_JPRB, 0.47133E+00_JPRB ,&
&  0.48890E+00_JPRB, 0.47076E+00_JPRB, 0.48761E+00_JPRB, 0.49399E+00_JPRB, 0.50875E+00_JPRB, 0.54526E+00_JPRB, 0.59568E+00_JPRB ,&
&  0.63707E+00_JPRB, 0.63978E+00_JPRB, 0.62077E+00_JPRB, 0.60024E+00_JPRB, 0.59112E+00_JPRB, 0.57960E+00_JPRB, 0.56613E+00_JPRB ,&
&  0.55337E+00_JPRB, 0.54414E+00_JPRB, 0.53645E+00_JPRB, 0.52916E+00_JPRB, 0.51971E+00_JPRB, 0.50808E+00_JPRB, 0.49738E+00_JPRB ,&
&  0.49029E+00_JPRB, 0.48815E+00_JPRB, 0.49006E+00_JPRB, 0.49393E+00_JPRB, 0.49824E+00_JPRB, 0.50554E+00_JPRB, 0.51529E+00_JPRB ,&
&  0.52587E+00_JPRB, 0.53211E+00_JPRB, 0.53907E+00_JPRB, 0.55442E+00_JPRB, 0.57314E+00_JPRB, 0.58798E+00_JPRB, 0.59729E+00_JPRB ,&
&  0.60116E+00_JPRB, 0.59636E+00_JPRB, 0.58546E+00_JPRB, 0.57001E+00_JPRB, 0.55220E+00_JPRB, 0.53388E+00_JPRB, 0.51761E+00_JPRB ,&
&  0.50447E+00_JPRB, 0.49361E+00_JPRB, 0.48457E+00_JPRB, 0.47831E+00_JPRB, 0.47527E+00_JPRB, 0.47464E+00_JPRB, 0.47625E+00_JPRB ,&
&  0.48119E+00_JPRB, 0.48790E+00_JPRB, 0.49406E+00_JPRB, 0.49996E+00_JPRB, 0.50517E+00_JPRB, 0.50948E+00_JPRB, 0.51491E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.52737E+00_JPRB, 0.53157E+00_JPRB, 0.53743E+00_JPRB, 0.54505E+00_JPRB, 0.55381E+00_JPRB, 0.56180E+00_JPRB ,&
&  0.56696E+00_JPRB, 0.56925E+00_JPRB, 0.57247E+00_JPRB, 0.57396E+00_JPRB, 0.57739E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.52982E+00_JPRB, 0.52900E+00_JPRB, 0.52705E+00_JPRB, 0.52561E+00_JPRB, 0.52284E+00_JPRB, 0.51822E+00_JPRB, 0.51267E+00_JPRB ,&
&  0.50799E+00_JPRB, 0.50361E+00_JPRB, 0.49879E+00_JPRB, 0.49246E+00_JPRB, 0.48399E+00_JPRB, 0.47556E+00_JPRB, 0.46610E+00_JPRB ,&
&  0.45715E+00_JPRB, 0.44938E+00_JPRB, 0.44382E+00_JPRB, 0.44039E+00_JPRB, 0.43766E+00_JPRB, 0.43513E+00_JPRB, 0.43222E+00_JPRB ,&
&  0.43035E+00_JPRB, 0.42950E+00_JPRB, 0.42965E+00_JPRB, 0.43039E+00_JPRB, 0.43115E+00_JPRB, 0.43460E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.44366E+00_JPRB, 0.46448E+00_JPRB, 0.47648E+00_JPRB, 0.48042E+00_JPRB, 0.48643E+00_JPRB, 0.49907E+00_JPRB, 0.51764E+00_JPRB ,&
&  0.53515E+00_JPRB, 0.54267E+00_JPRB, 0.53331E+00_JPRB, 0.51819E+00_JPRB, 0.50947E+00_JPRB, 0.50588E+00_JPRB, 0.49758E+00_JPRB ,&
&  0.48903E+00_JPRB, 0.48233E+00_JPRB, 0.47705E+00_JPRB, 0.47299E+00_JPRB, 0.46909E+00_JPRB, 0.46363E+00_JPRB, 0.45747E+00_JPRB ,&
&  0.45004E+00_JPRB, 0.44420E+00_JPRB, 0.44000E+00_JPRB, 0.43856E+00_JPRB, 0.43894E+00_JPRB, 0.44101E+00_JPRB, 0.44475E+00_JPRB ,&
&  0.45093E+00_JPRB, 0.45709E+00_JPRB, 0.45982E+00_JPRB, 0.46082E+00_JPRB, 0.46265E+00_JPRB, 0.46636E+00_JPRB, 0.46945E+00_JPRB ,&
&  0.47005E+00_JPRB, 0.46839E+00_JPRB, 0.46452E+00_JPRB, 0.45878E+00_JPRB, 0.45185E+00_JPRB, 0.44567E+00_JPRB, 0.44050E+00_JPRB ,&
&  0.43586E+00_JPRB, 0.43136E+00_JPRB, 0.42711E+00_JPRB, 0.42470E+00_JPRB, 0.42342E+00_JPRB, 0.42308E+00_JPRB, 0.42442E+00_JPRB ,&
&  0.42752E+00_JPRB, 0.43147E+00_JPRB, 0.43611E+00_JPRB, 0.44070E+00_JPRB, 0.44511E+00_JPRB, 0.44867E+00_JPRB, 0.45269E+00_JPRB ,&
&  0.45884E+00_JPRB, 0.46566E+00_JPRB, 0.47230E+00_JPRB, 0.47823E+00_JPRB, 0.48499E+00_JPRB, 0.49343E+00_JPRB, 0.50265E+00_JPRB ,&
&  0.50897E+00_JPRB, 0.51301E+00_JPRB, 0.51782E+00_JPRB, 0.52326E+00_JPRB, 0.52810E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.47793E+00_JPRB, 0.48025E+00_JPRB, 0.48151E+00_JPRB, 0.48307E+00_JPRB, 0.48349E+00_JPRB, 0.48207E+00_JPRB, 0.47823E+00_JPRB ,&
&  0.47357E+00_JPRB, 0.46825E+00_JPRB, 0.46280E+00_JPRB, 0.45596E+00_JPRB, 0.44873E+00_JPRB, 0.44259E+00_JPRB, 0.43592E+00_JPRB ,&
&  0.42912E+00_JPRB, 0.42282E+00_JPRB, 0.41813E+00_JPRB, 0.41436E+00_JPRB, 0.41065E+00_JPRB, 0.40685E+00_JPRB, 0.40221E+00_JPRB ,&
&  0.39816E+00_JPRB, 0.39460E+00_JPRB, 0.39475E+00_JPRB, 0.39766E+00_JPRB, 0.40018E+00_JPRB, 0.40202E+00_JPRB, 0.40378E+00_JPRB ,&
&  0.40512E+00_JPRB, 0.42715E+00_JPRB, 0.42320E+00_JPRB, 0.43902E+00_JPRB, 0.44470E+00_JPRB, 0.44543E+00_JPRB, 0.44691E+00_JPRB ,&
&  0.45187E+00_JPRB, 0.45829E+00_JPRB, 0.46287E+00_JPRB, 0.45707E+00_JPRB, 0.44811E+00_JPRB, 0.44137E+00_JPRB, 0.43469E+00_JPRB ,&
&  0.42598E+00_JPRB, 0.42199E+00_JPRB, 0.42127E+00_JPRB, 0.42114E+00_JPRB, 0.42109E+00_JPRB, 0.42046E+00_JPRB, 0.41883E+00_JPRB ,&
&  0.41605E+00_JPRB, 0.41208E+00_JPRB, 0.40752E+00_JPRB, 0.40317E+00_JPRB, 0.40033E+00_JPRB, 0.39767E+00_JPRB, 0.39577E+00_JPRB ,&
&  0.39582E+00_JPRB, 0.39796E+00_JPRB, 0.40013E+00_JPRB, 0.40082E+00_JPRB, 0.40000E+00_JPRB, 0.39912E+00_JPRB, 0.39878E+00_JPRB ,&
&  0.39811E+00_JPRB, 0.39693E+00_JPRB, 0.39516E+00_JPRB, 0.39266E+00_JPRB, 0.38986E+00_JPRB, 0.38783E+00_JPRB, 0.38661E+00_JPRB ,&
&  0.38517E+00_JPRB, 0.38269E+00_JPRB, 0.37952E+00_JPRB, 0.37882E+00_JPRB, 0.37893E+00_JPRB, 0.37941E+00_JPRB, 0.38011E+00_JPRB ,&
&  0.38166E+00_JPRB, 0.38453E+00_JPRB, 0.38799E+00_JPRB, 0.39161E+00_JPRB, 0.39537E+00_JPRB, 0.40034E+00_JPRB, 0.40578E+00_JPRB ,&
&  0.41158E+00_JPRB, 0.41770E+00_JPRB, 0.42371E+00_JPRB, 0.42942E+00_JPRB, 0.43439E+00_JPRB, 0.44010E+00_JPRB, 0.44743E+00_JPRB ,&
&  0.45370E+00_JPRB, 0.45860E+00_JPRB, 0.46270E+00_JPRB, 0.46792E+00_JPRB, 0.47363E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.43525E+00_JPRB, 0.44090E+00_JPRB, 0.44437E+00_JPRB, 0.44644E+00_JPRB, 0.44740E+00_JPRB, 0.44608E+00_JPRB, 0.44299E+00_JPRB ,&
&  0.43788E+00_JPRB, 0.43248E+00_JPRB, 0.42678E+00_JPRB, 0.42072E+00_JPRB, 0.41521E+00_JPRB, 0.41037E+00_JPRB, 0.40593E+00_JPRB ,&
&  0.40136E+00_JPRB, 0.39708E+00_JPRB, 0.39353E+00_JPRB, 0.39002E+00_JPRB, 0.38590E+00_JPRB, 0.38073E+00_JPRB, 0.37444E+00_JPRB ,&
&  0.37035E+00_JPRB, 0.36715E+00_JPRB, 0.36773E+00_JPRB, 0.37099E+00_JPRB, 0.37369E+00_JPRB, 0.37524E+00_JPRB, 0.37684E+00_JPRB ,&
&  0.37922E+00_JPRB, 0.38550E+00_JPRB, 0.39075E+00_JPRB, 0.39722E+00_JPRB, 0.40158E+00_JPRB, 0.40156E+00_JPRB, 0.39885E+00_JPRB ,&
&  0.39739E+00_JPRB, 0.39887E+00_JPRB, 0.40155E+00_JPRB, 0.40318E+00_JPRB, 0.40155E+00_JPRB, 0.39750E+00_JPRB, 0.39115E+00_JPRB ,&
&  0.38133E+00_JPRB, 0.37420E+00_JPRB, 0.37268E+00_JPRB, 0.37423E+00_JPRB, 0.37666E+00_JPRB, 0.37848E+00_JPRB, 0.37951E+00_JPRB ,&
&  0.37974E+00_JPRB, 0.37876E+00_JPRB, 0.37705E+00_JPRB, 0.37488E+00_JPRB, 0.37221E+00_JPRB, 0.36952E+00_JPRB, 0.36785E+00_JPRB ,&
&  0.36634E+00_JPRB, 0.36538E+00_JPRB, 0.36519E+00_JPRB, 0.36514E+00_JPRB, 0.36392E+00_JPRB, 0.36168E+00_JPRB, 0.36008E+00_JPRB ,&
&  0.35812E+00_JPRB, 0.35681E+00_JPRB, 0.35507E+00_JPRB, 0.35321E+00_JPRB, 0.35103E+00_JPRB, 0.34901E+00_JPRB, 0.34799E+00_JPRB ,&
&  0.34775E+00_JPRB, 0.34633E+00_JPRB, 0.34473E+00_JPRB, 0.34478E+00_JPRB, 0.34570E+00_JPRB, 0.34691E+00_JPRB, 0.34775E+00_JPRB ,&
&  0.34855E+00_JPRB, 0.34963E+00_JPRB, 0.35222E+00_JPRB, 0.35538E+00_JPRB, 0.35912E+00_JPRB, 0.36361E+00_JPRB, 0.36861E+00_JPRB ,&
&  0.37422E+00_JPRB, 0.38042E+00_JPRB, 0.38716E+00_JPRB, 0.39351E+00_JPRB, 0.39900E+00_JPRB, 0.40298E+00_JPRB, 0.40804E+00_JPRB ,&
&  0.41395E+00_JPRB, 0.41804E+00_JPRB, 0.42139E+00_JPRB, 0.42523E+00_JPRB, 0.42959E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.39410E+00_JPRB, 0.39991E+00_JPRB, 0.40435E+00_JPRB, 0.40703E+00_JPRB, 0.40751E+00_JPRB, 0.40632E+00_JPRB, 0.40327E+00_JPRB ,&
&  0.39853E+00_JPRB, 0.39358E+00_JPRB, 0.38878E+00_JPRB, 0.38491E+00_JPRB, 0.38230E+00_JPRB, 0.37999E+00_JPRB, 0.37778E+00_JPRB ,&
&  0.37490E+00_JPRB, 0.37166E+00_JPRB, 0.36802E+00_JPRB, 0.36481E+00_JPRB, 0.36084E+00_JPRB, 0.35554E+00_JPRB, 0.35138E+00_JPRB ,&
&  0.34833E+00_JPRB, 0.34565E+00_JPRB, 0.34474E+00_JPRB, 0.34613E+00_JPRB, 0.34878E+00_JPRB, 0.35130E+00_JPRB, 0.35426E+00_JPRB ,&
&  0.35659E+00_JPRB, 0.35920E+00_JPRB, 0.36173E+00_JPRB, 0.36227E+00_JPRB, 0.36226E+00_JPRB, 0.36206E+00_JPRB, 0.36148E+00_JPRB ,&
&  0.36086E+00_JPRB, 0.36027E+00_JPRB, 0.35926E+00_JPRB, 0.35920E+00_JPRB, 0.35895E+00_JPRB, 0.35739E+00_JPRB, 0.35460E+00_JPRB ,&
&  0.35006E+00_JPRB, 0.34400E+00_JPRB, 0.33946E+00_JPRB, 0.33749E+00_JPRB, 0.33712E+00_JPRB, 0.33765E+00_JPRB, 0.33869E+00_JPRB ,&
&  0.33992E+00_JPRB, 0.34169E+00_JPRB, 0.34341E+00_JPRB, 0.34428E+00_JPRB, 0.34407E+00_JPRB, 0.34376E+00_JPRB, 0.34223E+00_JPRB ,&
&  0.34077E+00_JPRB, 0.33975E+00_JPRB, 0.33915E+00_JPRB, 0.33852E+00_JPRB, 0.33607E+00_JPRB, 0.33289E+00_JPRB, 0.33175E+00_JPRB ,&
&  0.32960E+00_JPRB, 0.32818E+00_JPRB, 0.32624E+00_JPRB, 0.32490E+00_JPRB, 0.32344E+00_JPRB, 0.32159E+00_JPRB, 0.31978E+00_JPRB ,&
&  0.31850E+00_JPRB, 0.31781E+00_JPRB, 0.31749E+00_JPRB, 0.31680E+00_JPRB, 0.31736E+00_JPRB, 0.31837E+00_JPRB, 0.32005E+00_JPRB ,&
&  0.32136E+00_JPRB, 0.32249E+00_JPRB, 0.32430E+00_JPRB, 0.32650E+00_JPRB, 0.32977E+00_JPRB, 0.33358E+00_JPRB, 0.33792E+00_JPRB ,&
&  0.34174E+00_JPRB, 0.34664E+00_JPRB, 0.35206E+00_JPRB, 0.35721E+00_JPRB, 0.36134E+00_JPRB, 0.36443E+00_JPRB, 0.36874E+00_JPRB ,&
&  0.37526E+00_JPRB, 0.38035E+00_JPRB, 0.38291E+00_JPRB, 0.38595E+00_JPRB, 0.38912E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.36168E+00_JPRB, 0.36394E+00_JPRB, 0.36616E+00_JPRB, 0.36845E+00_JPRB, 0.37010E+00_JPRB, 0.37077E+00_JPRB, 0.36938E+00_JPRB ,&
&  0.36646E+00_JPRB, 0.36292E+00_JPRB, 0.35930E+00_JPRB, 0.35621E+00_JPRB, 0.35428E+00_JPRB, 0.35292E+00_JPRB, 0.35178E+00_JPRB ,&
&  0.35069E+00_JPRB, 0.34957E+00_JPRB, 0.34702E+00_JPRB, 0.34353E+00_JPRB, 0.33894E+00_JPRB, 0.33485E+00_JPRB, 0.33319E+00_JPRB ,&
&  0.33102E+00_JPRB, 0.32950E+00_JPRB, 0.32831E+00_JPRB, 0.32711E+00_JPRB, 0.32651E+00_JPRB, 0.32737E+00_JPRB, 0.32943E+00_JPRB ,&
&  0.33160E+00_JPRB, 0.33367E+00_JPRB, 0.33548E+00_JPRB, 0.33621E+00_JPRB, 0.33548E+00_JPRB, 0.33460E+00_JPRB, 0.33346E+00_JPRB ,&
&  0.33195E+00_JPRB, 0.33026E+00_JPRB, 0.32869E+00_JPRB, 0.32666E+00_JPRB, 0.32378E+00_JPRB, 0.32021E+00_JPRB, 0.31770E+00_JPRB ,&
&  0.31587E+00_JPRB, 0.31413E+00_JPRB, 0.31234E+00_JPRB, 0.31145E+00_JPRB, 0.31074E+00_JPRB, 0.31045E+00_JPRB, 0.31056E+00_JPRB ,&
&  0.31113E+00_JPRB, 0.31257E+00_JPRB, 0.31467E+00_JPRB, 0.31704E+00_JPRB, 0.31798E+00_JPRB, 0.31831E+00_JPRB, 0.31846E+00_JPRB ,&
&  0.31746E+00_JPRB, 0.31573E+00_JPRB, 0.31349E+00_JPRB, 0.31092E+00_JPRB, 0.30903E+00_JPRB, 0.30756E+00_JPRB, 0.30594E+00_JPRB ,&
&  0.30345E+00_JPRB, 0.30189E+00_JPRB, 0.30115E+00_JPRB, 0.30098E+00_JPRB, 0.30112E+00_JPRB, 0.30104E+00_JPRB, 0.30008E+00_JPRB ,&
&  0.29893E+00_JPRB, 0.29803E+00_JPRB, 0.29708E+00_JPRB, 0.29599E+00_JPRB, 0.29409E+00_JPRB, 0.29483E+00_JPRB, 0.29627E+00_JPRB ,&
&  0.29674E+00_JPRB, 0.29670E+00_JPRB, 0.29691E+00_JPRB, 0.29857E+00_JPRB, 0.30055E+00_JPRB, 0.30304E+00_JPRB, 0.30613E+00_JPRB ,&
&  0.30887E+00_JPRB, 0.31259E+00_JPRB, 0.31776E+00_JPRB, 0.32264E+00_JPRB, 0.32618E+00_JPRB, 0.32805E+00_JPRB, 0.33187E+00_JPRB ,&
&  0.33938E+00_JPRB, 0.34683E+00_JPRB, 0.35123E+00_JPRB, 0.35547E+00_JPRB, 0.35955E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.33293E+00_JPRB, 0.33418E+00_JPRB, 0.33519E+00_JPRB, 0.33639E+00_JPRB, 0.33801E+00_JPRB, 0.33908E+00_JPRB, 0.33992E+00_JPRB ,&
&  0.33926E+00_JPRB, 0.33722E+00_JPRB, 0.33476E+00_JPRB, 0.33221E+00_JPRB, 0.33096E+00_JPRB, 0.33055E+00_JPRB, 0.33040E+00_JPRB ,&
&  0.32964E+00_JPRB, 0.32919E+00_JPRB, 0.32697E+00_JPRB, 0.32337E+00_JPRB, 0.31899E+00_JPRB, 0.31580E+00_JPRB, 0.31472E+00_JPRB ,&
&  0.31391E+00_JPRB, 0.31270E+00_JPRB, 0.31186E+00_JPRB, 0.31059E+00_JPRB, 0.30732E+00_JPRB, 0.30574E+00_JPRB, 0.30613E+00_JPRB ,&
&  0.30702E+00_JPRB, 0.30870E+00_JPRB, 0.30875E+00_JPRB, 0.30673E+00_JPRB, 0.30894E+00_JPRB, 0.30886E+00_JPRB, 0.30694E+00_JPRB ,&
&  0.30443E+00_JPRB, 0.30182E+00_JPRB, 0.29943E+00_JPRB, 0.29690E+00_JPRB, 0.29419E+00_JPRB, 0.29212E+00_JPRB, 0.29080E+00_JPRB ,&
&  0.29054E+00_JPRB, 0.29103E+00_JPRB, 0.29185E+00_JPRB, 0.29300E+00_JPRB, 0.29372E+00_JPRB, 0.29413E+00_JPRB, 0.29428E+00_JPRB ,&
&  0.29455E+00_JPRB, 0.29491E+00_JPRB, 0.29548E+00_JPRB, 0.29582E+00_JPRB, 0.29617E+00_JPRB, 0.29633E+00_JPRB, 0.29588E+00_JPRB ,&
&  0.29426E+00_JPRB, 0.29180E+00_JPRB, 0.28870E+00_JPRB, 0.28727E+00_JPRB, 0.28693E+00_JPRB, 0.28465E+00_JPRB, 0.28157E+00_JPRB ,&
&  0.27898E+00_JPRB, 0.27881E+00_JPRB, 0.28064E+00_JPRB, 0.28223E+00_JPRB, 0.28350E+00_JPRB, 0.28399E+00_JPRB, 0.28373E+00_JPRB ,&
&  0.28263E+00_JPRB, 0.27968E+00_JPRB, 0.27654E+00_JPRB, 0.27390E+00_JPRB, 0.27215E+00_JPRB, 0.27202E+00_JPRB, 0.27319E+00_JPRB ,&
&  0.27350E+00_JPRB, 0.27386E+00_JPRB, 0.27525E+00_JPRB, 0.27661E+00_JPRB, 0.27779E+00_JPRB, 0.27899E+00_JPRB, 0.28075E+00_JPRB ,&
&  0.28307E+00_JPRB, 0.28711E+00_JPRB, 0.29218E+00_JPRB, 0.29810E+00_JPRB, 0.30218E+00_JPRB, 0.30164E+00_JPRB, 0.30332E+00_JPRB ,&
&  0.30962E+00_JPRB, 0.31661E+00_JPRB, 0.32194E+00_JPRB, 0.32601E+00_JPRB, 0.32986E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.31011E+00_JPRB, 0.31158E+00_JPRB, 0.31261E+00_JPRB, 0.31330E+00_JPRB, 0.31465E+00_JPRB, 0.31578E+00_JPRB, 0.31728E+00_JPRB ,&
&  0.31834E+00_JPRB, 0.31694E+00_JPRB, 0.31511E+00_JPRB, 0.31348E+00_JPRB, 0.31266E+00_JPRB, 0.31155E+00_JPRB, 0.31050E+00_JPRB ,&
&  0.31077E+00_JPRB, 0.30978E+00_JPRB, 0.30665E+00_JPRB, 0.30396E+00_JPRB, 0.29988E+00_JPRB, 0.29727E+00_JPRB, 0.29689E+00_JPRB ,&
&  0.29858E+00_JPRB, 0.29791E+00_JPRB, 0.29641E+00_JPRB, 0.29454E+00_JPRB, 0.29239E+00_JPRB, 0.29117E+00_JPRB, 0.28989E+00_JPRB ,&
&  0.28906E+00_JPRB, 0.28740E+00_JPRB, 0.28288E+00_JPRB, 0.28321E+00_JPRB, 0.28302E+00_JPRB, 0.28340E+00_JPRB, 0.28307E+00_JPRB ,&
&  0.28254E+00_JPRB, 0.28153E+00_JPRB, 0.27962E+00_JPRB, 0.27743E+00_JPRB, 0.27562E+00_JPRB, 0.27469E+00_JPRB, 0.27480E+00_JPRB ,&
&  0.27561E+00_JPRB, 0.27731E+00_JPRB, 0.27966E+00_JPRB, 0.28210E+00_JPRB, 0.28315E+00_JPRB, 0.28400E+00_JPRB, 0.28535E+00_JPRB ,&
&  0.28602E+00_JPRB, 0.28615E+00_JPRB, 0.28603E+00_JPRB, 0.28557E+00_JPRB, 0.28464E+00_JPRB, 0.28309E+00_JPRB, 0.28180E+00_JPRB ,&
&  0.27919E+00_JPRB, 0.27593E+00_JPRB, 0.27357E+00_JPRB, 0.27080E+00_JPRB, 0.26338E+00_JPRB, 0.25305E+00_JPRB, 0.24736E+00_JPRB ,&
&  0.24826E+00_JPRB, 0.25337E+00_JPRB, 0.25800E+00_JPRB, 0.26212E+00_JPRB, 0.26933E+00_JPRB, 0.26938E+00_JPRB, 0.26435E+00_JPRB ,&
&  0.25308E+00_JPRB, 0.24985E+00_JPRB, 0.24688E+00_JPRB, 0.24155E+00_JPRB, 0.23844E+00_JPRB, 0.23830E+00_JPRB, 0.23849E+00_JPRB ,&
&  0.24086E+00_JPRB, 0.24356E+00_JPRB, 0.24693E+00_JPRB, 0.24901E+00_JPRB, 0.24944E+00_JPRB, 0.24862E+00_JPRB, 0.24723E+00_JPRB ,&
&  0.24784E+00_JPRB, 0.25312E+00_JPRB, 0.26275E+00_JPRB, 0.27086E+00_JPRB, 0.27644E+00_JPRB, 0.27894E+00_JPRB, 0.28152E+00_JPRB ,&
&  0.28673E+00_JPRB, 0.29330E+00_JPRB, 0.29950E+00_JPRB, 0.30382E+00_JPRB, 0.30732E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.28720E+00_JPRB, 0.28857E+00_JPRB, 0.28977E+00_JPRB, 0.29085E+00_JPRB, 0.29194E+00_JPRB, 0.29317E+00_JPRB, 0.29428E+00_JPRB ,&
&  0.29569E+00_JPRB, 0.29659E+00_JPRB, 0.29669E+00_JPRB, 0.29542E+00_JPRB, 0.29443E+00_JPRB, 0.29367E+00_JPRB, 0.29343E+00_JPRB ,&
&  0.29080E+00_JPRB, 0.28745E+00_JPRB, 0.28584E+00_JPRB, 0.28528E+00_JPRB, 0.28434E+00_JPRB, 0.28400E+00_JPRB, 0.28496E+00_JPRB ,&
&  0.28588E+00_JPRB, 0.28493E+00_JPRB, 0.28412E+00_JPRB, 0.28215E+00_JPRB, 0.28062E+00_JPRB, 0.27947E+00_JPRB, 0.27805E+00_JPRB ,&
&  0.27305E+00_JPRB, 0.26252E+00_JPRB, 0.25674E+00_JPRB, 0.26192E+00_JPRB, 0.26768E+00_JPRB, 0.26667E+00_JPRB, 0.26705E+00_JPRB ,&
&  0.26691E+00_JPRB, 0.26565E+00_JPRB, 0.26420E+00_JPRB, 0.26291E+00_JPRB, 0.26265E+00_JPRB, 0.26344E+00_JPRB, 0.26505E+00_JPRB ,&
&  0.26706E+00_JPRB, 0.27063E+00_JPRB, 0.27403E+00_JPRB, 0.27524E+00_JPRB, 0.27660E+00_JPRB, 0.27740E+00_JPRB, 0.27758E+00_JPRB ,&
&  0.27654E+00_JPRB, 0.27472E+00_JPRB, 0.27299E+00_JPRB, 0.27124E+00_JPRB, 0.26947E+00_JPRB, 0.26723E+00_JPRB, 0.26151E+00_JPRB ,&
&  0.25624E+00_JPRB, 0.25021E+00_JPRB, 0.24303E+00_JPRB, 0.23581E+00_JPRB, 0.22981E+00_JPRB, 0.22546E+00_JPRB, 0.22146E+00_JPRB ,&
&  0.22135E+00_JPRB, 0.22677E+00_JPRB, 0.23658E+00_JPRB, 0.24713E+00_JPRB, 0.24845E+00_JPRB, 0.24033E+00_JPRB, 0.23475E+00_JPRB ,&
&  0.22840E+00_JPRB, 0.22292E+00_JPRB, 0.21958E+00_JPRB, 0.21775E+00_JPRB, 0.21631E+00_JPRB, 0.21608E+00_JPRB, 0.21675E+00_JPRB ,&
&  0.21858E+00_JPRB, 0.22048E+00_JPRB, 0.22219E+00_JPRB, 0.22346E+00_JPRB, 0.22402E+00_JPRB, 0.22392E+00_JPRB, 0.22350E+00_JPRB ,&
&  0.22452E+00_JPRB, 0.22782E+00_JPRB, 0.23294E+00_JPRB, 0.23872E+00_JPRB, 0.24382E+00_JPRB, 0.24887E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.26351E+00_JPRB, 0.26884E+00_JPRB, 0.27468E+00_JPRB, 0.28108E+00_JPRB, 0.28495E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.26808E+00_JPRB, 0.27100E+00_JPRB, 0.27181E+00_JPRB, 0.27329E+00_JPRB, 0.27348E+00_JPRB, 0.27450E+00_JPRB, 0.27499E+00_JPRB ,&
&  0.27710E+00_JPRB, 0.27909E+00_JPRB, 0.27947E+00_JPRB, 0.27743E+00_JPRB, 0.27564E+00_JPRB, 0.27138E+00_JPRB, 0.26941E+00_JPRB ,&
&  0.26840E+00_JPRB, 0.26915E+00_JPRB, 0.26920E+00_JPRB, 0.26915E+00_JPRB, 0.26898E+00_JPRB, 0.26885E+00_JPRB, 0.26954E+00_JPRB ,&
&  0.26714E+00_JPRB, 0.26524E+00_JPRB, 0.26720E+00_JPRB, 0.26576E+00_JPRB, 0.26460E+00_JPRB, 0.26271E+00_JPRB, 0.26165E+00_JPRB ,&
&  0.25669E+00_JPRB, 0.25013E+00_JPRB, 0.24428E+00_JPRB, 0.24887E+00_JPRB, 0.25612E+00_JPRB, 0.25253E+00_JPRB, 0.25214E+00_JPRB ,&
&  0.25213E+00_JPRB, 0.25141E+00_JPRB, 0.25158E+00_JPRB, 0.25164E+00_JPRB, 0.25319E+00_JPRB, 0.25472E+00_JPRB, 0.25768E+00_JPRB ,&
&  0.26156E+00_JPRB, 0.26672E+00_JPRB, 0.26678E+00_JPRB, 0.25914E+00_JPRB, 0.25641E+00_JPRB, 0.25573E+00_JPRB, 0.25185E+00_JPRB ,&
&  0.24697E+00_JPRB, 0.24593E+00_JPRB, 0.24435E+00_JPRB, 0.23998E+00_JPRB, 0.23814E+00_JPRB, 0.23742E+00_JPRB, 0.23374E+00_JPRB ,&
&  0.22925E+00_JPRB, 0.22595E+00_JPRB, 0.22173E+00_JPRB, 0.21756E+00_JPRB, 0.21474E+00_JPRB, 0.21350E+00_JPRB, 0.21383E+00_JPRB ,&
&  0.21640E+00_JPRB, 0.21857E+00_JPRB, 0.22536E+00_JPRB, 0.22917E+00_JPRB, 0.22473E+00_JPRB, 0.22248E+00_JPRB, 0.21910E+00_JPRB ,&
&  0.21213E+00_JPRB, 0.20916E+00_JPRB, 0.20666E+00_JPRB, 0.20474E+00_JPRB, 0.20452E+00_JPRB, 0.20528E+00_JPRB, 0.20567E+00_JPRB ,&
&  0.20649E+00_JPRB, 0.20798E+00_JPRB, 0.20871E+00_JPRB, 0.20914E+00_JPRB, 0.20927E+00_JPRB, 0.20973E+00_JPRB, 0.21067E+00_JPRB ,&
&  0.21174E+00_JPRB, 0.21427E+00_JPRB, 0.21746E+00_JPRB, 0.22074E+00_JPRB, 0.22325E+00_JPRB, 0.22536E+00_JPRB, 0.22993E+00_JPRB ,&
&  0.23473E+00_JPRB, 0.24594E+00_JPRB, 0.26129E+00_JPRB, 0.27033E+00_JPRB, 0.26550E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.26432E+00_JPRB, 0.26619E+00_JPRB, 0.26576E+00_JPRB, 0.26585E+00_JPRB, 0.26518E+00_JPRB, 0.26562E+00_JPRB, 0.26405E+00_JPRB ,&
&  0.26320E+00_JPRB, 0.26107E+00_JPRB, 0.25924E+00_JPRB, 0.25516E+00_JPRB, 0.25293E+00_JPRB, 0.24643E+00_JPRB, 0.24428E+00_JPRB ,&
&  0.24209E+00_JPRB, 0.24260E+00_JPRB, 0.24258E+00_JPRB, 0.24301E+00_JPRB, 0.24355E+00_JPRB, 0.24504E+00_JPRB, 0.24448E+00_JPRB ,&
&  0.24298E+00_JPRB, 0.24089E+00_JPRB, 0.23838E+00_JPRB, 0.23612E+00_JPRB, 0.23654E+00_JPRB, 0.23597E+00_JPRB, 0.23466E+00_JPRB ,&
&  0.23369E+00_JPRB, 0.23446E+00_JPRB, 0.23491E+00_JPRB, 0.23646E+00_JPRB, 0.24149E+00_JPRB, 0.23917E+00_JPRB, 0.23843E+00_JPRB ,&
&  0.23903E+00_JPRB, 0.23980E+00_JPRB, 0.24137E+00_JPRB, 0.24281E+00_JPRB, 0.24649E+00_JPRB, 0.24616E+00_JPRB, 0.25004E+00_JPRB ,&
&  0.24904E+00_JPRB, 0.24855E+00_JPRB, 0.24705E+00_JPRB, 0.24378E+00_JPRB, 0.23854E+00_JPRB, 0.23523E+00_JPRB, 0.23216E+00_JPRB ,&
&  0.22862E+00_JPRB, 0.22564E+00_JPRB, 0.22175E+00_JPRB, 0.22040E+00_JPRB, 0.21821E+00_JPRB, 0.21626E+00_JPRB, 0.21451E+00_JPRB ,&
&  0.21333E+00_JPRB, 0.21164E+00_JPRB, 0.21025E+00_JPRB, 0.20933E+00_JPRB, 0.20912E+00_JPRB, 0.20930E+00_JPRB, 0.21068E+00_JPRB ,&
&  0.21317E+00_JPRB, 0.21502E+00_JPRB, 0.21652E+00_JPRB, 0.21829E+00_JPRB, 0.21655E+00_JPRB, 0.21344E+00_JPRB, 0.21048E+00_JPRB ,&
&  0.20580E+00_JPRB, 0.20337E+00_JPRB, 0.20094E+00_JPRB, 0.19945E+00_JPRB, 0.19898E+00_JPRB, 0.19873E+00_JPRB, 0.19887E+00_JPRB ,&
&  0.19903E+00_JPRB, 0.19957E+00_JPRB, 0.19996E+00_JPRB, 0.20058E+00_JPRB, 0.20139E+00_JPRB, 0.20220E+00_JPRB, 0.20345E+00_JPRB ,&
&  0.20535E+00_JPRB, 0.20774E+00_JPRB, 0.21071E+00_JPRB, 0.21399E+00_JPRB, 0.21823E+00_JPRB, 0.22361E+00_JPRB, 0.23124E+00_JPRB ,&
&  0.23828E+00_JPRB, 0.24814E+00_JPRB, 0.24868E+00_JPRB, 0.25827E+00_JPRB, 0.26314E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.26145E+00_JPRB, 0.26387E+00_JPRB, 0.26438E+00_JPRB, 0.26489E+00_JPRB, 0.26353E+00_JPRB, 0.26014E+00_JPRB, 0.25653E+00_JPRB ,&
&  0.24957E+00_JPRB, 0.24420E+00_JPRB, 0.23774E+00_JPRB, 0.23189E+00_JPRB, 0.22623E+00_JPRB, 0.22280E+00_JPRB, 0.22057E+00_JPRB ,&
&  0.22000E+00_JPRB, 0.22114E+00_JPRB, 0.22189E+00_JPRB, 0.22237E+00_JPRB, 0.22236E+00_JPRB, 0.22315E+00_JPRB, 0.22274E+00_JPRB ,&
&  0.22196E+00_JPRB, 0.22090E+00_JPRB, 0.22016E+00_JPRB, 0.22084E+00_JPRB, 0.22305E+00_JPRB, 0.22527E+00_JPRB, 0.22714E+00_JPRB ,&
&  0.22855E+00_JPRB, 0.22737E+00_JPRB, 0.22732E+00_JPRB, 0.22648E+00_JPRB, 0.22650E+00_JPRB, 0.22655E+00_JPRB, 0.22725E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.23084E+00_JPRB, 0.23402E+00_JPRB, 0.23556E+00_JPRB, 0.23475E+00_JPRB, 0.23866E+00_JPRB, 0.24035E+00_JPRB ,&
&  0.24044E+00_JPRB, 0.23691E+00_JPRB, 0.23280E+00_JPRB, 0.22937E+00_JPRB, 0.22556E+00_JPRB, 0.22236E+00_JPRB, 0.21974E+00_JPRB ,&
&  0.21738E+00_JPRB, 0.21622E+00_JPRB, 0.21366E+00_JPRB, 0.21149E+00_JPRB, 0.21008E+00_JPRB, 0.20859E+00_JPRB, 0.20768E+00_JPRB ,&
&  0.20718E+00_JPRB, 0.20618E+00_JPRB, 0.20605E+00_JPRB, 0.20545E+00_JPRB, 0.20552E+00_JPRB, 0.20560E+00_JPRB, 0.20592E+00_JPRB ,&
&  0.20685E+00_JPRB, 0.20816E+00_JPRB, 0.20806E+00_JPRB, 0.20800E+00_JPRB, 0.20606E+00_JPRB, 0.20396E+00_JPRB, 0.20223E+00_JPRB ,&
&  0.19978E+00_JPRB, 0.19805E+00_JPRB, 0.19697E+00_JPRB, 0.19587E+00_JPRB, 0.19563E+00_JPRB, 0.19556E+00_JPRB, 0.19538E+00_JPRB ,&
&  0.19554E+00_JPRB, 0.19589E+00_JPRB, 0.19650E+00_JPRB, 0.19788E+00_JPRB, 0.19908E+00_JPRB, 0.20081E+00_JPRB, 0.20270E+00_JPRB ,&
&  0.20415E+00_JPRB, 0.20686E+00_JPRB, 0.20963E+00_JPRB, 0.21275E+00_JPRB, 0.21953E+00_JPRB, 0.22509E+00_JPRB, 0.23483E+00_JPRB ,&
&  0.24128E+00_JPRB, 0.24948E+00_JPRB, 0.24643E+00_JPRB, 0.25341E+00_JPRB, 0.25866E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.25965E+00_JPRB, 0.26096E+00_JPRB, 0.26026E+00_JPRB, 0.26013E+00_JPRB, 0.25804E+00_JPRB, 0.25525E+00_JPRB, 0.25105E+00_JPRB ,&
&  0.24739E+00_JPRB, 0.24098E+00_JPRB, 0.23617E+00_JPRB, 0.23075E+00_JPRB, 0.22622E+00_JPRB, 0.22213E+00_JPRB, 0.21915E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21689E+00_JPRB, 0.21640E+00_JPRB, 0.21526E+00_JPRB, 0.21323E+00_JPRB, 0.21191E+00_JPRB, 0.21027E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.20914E+00_JPRB, 0.21025E+00_JPRB, 0.21235E+00_JPRB, 0.21614E+00_JPRB, 0.21849E+00_JPRB, 0.22074E+00_JPRB ,&
&  0.22238E+00_JPRB, 0.21770E+00_JPRB, 0.21785E+00_JPRB, 0.21863E+00_JPRB, 0.21903E+00_JPRB, 0.21890E+00_JPRB, 0.21890E+00_JPRB ,&
&  0.22371E+00_JPRB, 0.22510E+00_JPRB, 0.23182E+00_JPRB, 0.23138E+00_JPRB, 0.23060E+00_JPRB, 0.23312E+00_JPRB, 0.23234E+00_JPRB ,&
&  0.23301E+00_JPRB, 0.22949E+00_JPRB, 0.22687E+00_JPRB, 0.22255E+00_JPRB, 0.21803E+00_JPRB, 0.21573E+00_JPRB, 0.21309E+00_JPRB ,&
&  0.21228E+00_JPRB, 0.21213E+00_JPRB, 0.21030E+00_JPRB, 0.20916E+00_JPRB, 0.20693E+00_JPRB, 0.20440E+00_JPRB, 0.20290E+00_JPRB ,&
&  0.20136E+00_JPRB, 0.20016E+00_JPRB, 0.19953E+00_JPRB, 0.19887E+00_JPRB, 0.19856E+00_JPRB, 0.19837E+00_JPRB, 0.19816E+00_JPRB ,&
&  0.19817E+00_JPRB, 0.19810E+00_JPRB, 0.19743E+00_JPRB, 0.19714E+00_JPRB, 0.19585E+00_JPRB, 0.19498E+00_JPRB, 0.19426E+00_JPRB ,&
&  0.19356E+00_JPRB, 0.19331E+00_JPRB, 0.19315E+00_JPRB, 0.19321E+00_JPRB, 0.19336E+00_JPRB, 0.19396E+00_JPRB, 0.19465E+00_JPRB ,&
&  0.19532E+00_JPRB, 0.19652E+00_JPRB, 0.19743E+00_JPRB, 0.19878E+00_JPRB, 0.20039E+00_JPRB, 0.20175E+00_JPRB, 0.20394E+00_JPRB ,&
&  0.20633E+00_JPRB, 0.20912E+00_JPRB, 0.21318E+00_JPRB, 0.21656E+00_JPRB, 0.22081E+00_JPRB, 0.22629E+00_JPRB, 0.23202E+00_JPRB ,&
&  0.23812E+00_JPRB, 0.24416E+00_JPRB, 0.24429E+00_JPRB, 0.25227E+00_JPRB, 0.25711E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.24797E+00_JPRB, 0.24873E+00_JPRB, 0.24950E+00_JPRB, 0.24970E+00_JPRB, 0.24970E+00_JPRB, 0.24851E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.24224E+00_JPRB, 0.23893E+00_JPRB, 0.23509E+00_JPRB, 0.23122E+00_JPRB, 0.22779E+00_JPRB, 0.22392E+00_JPRB, 0.22064E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21533E+00_JPRB, 0.21338E+00_JPRB, 0.21092E+00_JPRB, 0.20786E+00_JPRB, 0.20506E+00_JPRB, 0.20351E+00_JPRB ,&
&  0.20322E+00_JPRB, 0.20375E+00_JPRB, 0.20555E+00_JPRB, 0.20726E+00_JPRB, 0.20907E+00_JPRB, 0.21042E+00_JPRB, 0.21182E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21103E+00_JPRB, 0.21075E+00_JPRB, 0.21011E+00_JPRB, 0.21114E+00_JPRB, 0.21251E+00_JPRB, 0.21350E+00_JPRB ,&
&  0.21465E+00_JPRB, 0.21718E+00_JPRB, 0.21916E+00_JPRB, 0.22094E+00_JPRB, 0.22127E+00_JPRB, 0.22086E+00_JPRB, 0.21940E+00_JPRB ,&
&  0.21739E+00_JPRB, 0.21513E+00_JPRB, 0.21303E+00_JPRB, 0.21118E+00_JPRB, 0.20976E+00_JPRB, 0.20804E+00_JPRB, 0.20653E+00_JPRB ,&
&  0.20579E+00_JPRB, 0.20512E+00_JPRB, 0.20418E+00_JPRB, 0.20327E+00_JPRB, 0.20185E+00_JPRB, 0.20060E+00_JPRB, 0.19932E+00_JPRB ,&
&  0.19817E+00_JPRB, 0.19709E+00_JPRB, 0.19609E+00_JPRB, 0.19506E+00_JPRB, 0.19434E+00_JPRB, 0.19371E+00_JPRB, 0.19308E+00_JPRB ,&
&  0.19244E+00_JPRB, 0.19223E+00_JPRB, 0.19176E+00_JPRB, 0.19161E+00_JPRB, 0.19129E+00_JPRB, 0.19108E+00_JPRB, 0.19130E+00_JPRB ,&
&  0.19194E+00_JPRB, 0.19259E+00_JPRB, 0.19289E+00_JPRB, 0.19331E+00_JPRB, 0.19375E+00_JPRB, 0.19432E+00_JPRB, 0.19528E+00_JPRB ,&
&  0.19601E+00_JPRB, 0.19684E+00_JPRB, 0.19747E+00_JPRB, 0.19826E+00_JPRB, 0.19986E+00_JPRB, 0.20150E+00_JPRB, 0.20336E+00_JPRB ,&
&  0.20571E+00_JPRB, 0.20815E+00_JPRB, 0.21101E+00_JPRB, 0.21373E+00_JPRB, 0.21742E+00_JPRB, 0.21917E+00_JPRB, 0.22282E+00_JPRB ,&
&  0.22683E+00_JPRB, 0.22720E+00_JPRB, 0.24126E+00_JPRB, 0.24259E+00_JPRB, 0.24453E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.21770E+00_JPRB, 0.21966E+00_JPRB, 0.22072E+00_JPRB, 0.22189E+00_JPRB, 0.21848E+00_JPRB, 0.23105E+00_JPRB, 0.22862E+00_JPRB ,&
&  0.22725E+00_JPRB, 0.22308E+00_JPRB, 0.22312E+00_JPRB, 0.22143E+00_JPRB, 0.21919E+00_JPRB, 0.21663E+00_JPRB, 0.21426E+00_JPRB ,&
&  0.21204E+00_JPRB, 0.20955E+00_JPRB, 0.20785E+00_JPRB, 0.20656E+00_JPRB, 0.20537E+00_JPRB, 0.20429E+00_JPRB, 0.20350E+00_JPRB ,&
&  0.20360E+00_JPRB, 0.20433E+00_JPRB, 0.20568E+00_JPRB, 0.20553E+00_JPRB, 0.20511E+00_JPRB, 0.20459E+00_JPRB, 0.20429E+00_JPRB ,&
&  0.20360E+00_JPRB, 0.20311E+00_JPRB, 0.20266E+00_JPRB, 0.20201E+00_JPRB, 0.20312E+00_JPRB, 0.20432E+00_JPRB, 0.20548E+00_JPRB ,&
&  0.20659E+00_JPRB, 0.20768E+00_JPRB, 0.20793E+00_JPRB, 0.20795E+00_JPRB, 0.20763E+00_JPRB, 0.20717E+00_JPRB, 0.20630E+00_JPRB ,&
&  0.20548E+00_JPRB, 0.20458E+00_JPRB, 0.20342E+00_JPRB, 0.20268E+00_JPRB, 0.20201E+00_JPRB, 0.20139E+00_JPRB, 0.20083E+00_JPRB ,&
&  0.20054E+00_JPRB, 0.20035E+00_JPRB, 0.20010E+00_JPRB, 0.19971E+00_JPRB, 0.19926E+00_JPRB, 0.19879E+00_JPRB, 0.19838E+00_JPRB ,&
&  0.19807E+00_JPRB, 0.19744E+00_JPRB, 0.19688E+00_JPRB, 0.19629E+00_JPRB, 0.19585E+00_JPRB, 0.19524E+00_JPRB, 0.19472E+00_JPRB ,&
&  0.19422E+00_JPRB, 0.19384E+00_JPRB, 0.19369E+00_JPRB, 0.19357E+00_JPRB, 0.19347E+00_JPRB, 0.19350E+00_JPRB, 0.19362E+00_JPRB ,&
&  0.19381E+00_JPRB, 0.19402E+00_JPRB, 0.19436E+00_JPRB, 0.19461E+00_JPRB, 0.19485E+00_JPRB, 0.19503E+00_JPRB, 0.19552E+00_JPRB ,&
&  0.19611E+00_JPRB, 0.19671E+00_JPRB, 0.19723E+00_JPRB, 0.19734E+00_JPRB, 0.19826E+00_JPRB, 0.19927E+00_JPRB, 0.20039E+00_JPRB ,&
&  0.20158E+00_JPRB, 0.20313E+00_JPRB, 0.20483E+00_JPRB, 0.20669E+00_JPRB, 0.20894E+00_JPRB, 0.21023E+00_JPRB, 0.21180E+00_JPRB ,&
&  0.21396E+00_JPRB, 0.21760E+00_JPRB, 0.21856E+00_JPRB, 0.21871E+00_JPRB, 0.21849E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.20529E+00_JPRB, 0.20496E+00_JPRB, 0.20457E+00_JPRB, 0.20411E+00_JPRB, 0.20279E+00_JPRB, 0.20207E+00_JPRB, 0.20149E+00_JPRB ,&
&  0.20111E+00_JPRB, 0.20086E+00_JPRB, 0.20093E+00_JPRB, 0.20108E+00_JPRB, 0.20127E+00_JPRB, 0.20143E+00_JPRB, 0.20159E+00_JPRB ,&
&  0.20156E+00_JPRB, 0.20157E+00_JPRB, 0.20067E+00_JPRB, 0.19992E+00_JPRB, 0.19944E+00_JPRB, 0.19906E+00_JPRB, 0.19860E+00_JPRB ,&
&  0.19822E+00_JPRB, 0.19778E+00_JPRB, 0.19727E+00_JPRB, 0.19697E+00_JPRB, 0.19703E+00_JPRB, 0.19689E+00_JPRB, 0.19695E+00_JPRB ,&
&  0.19700E+00_JPRB, 0.19732E+00_JPRB, 0.19751E+00_JPRB, 0.19771E+00_JPRB, 0.19831E+00_JPRB, 0.19888E+00_JPRB, 0.19910E+00_JPRB ,&
&  0.19926E+00_JPRB, 0.19913E+00_JPRB, 0.19899E+00_JPRB, 0.19889E+00_JPRB, 0.19887E+00_JPRB, 0.19890E+00_JPRB, 0.19890E+00_JPRB ,&
&  0.19880E+00_JPRB, 0.19863E+00_JPRB, 0.19838E+00_JPRB, 0.19808E+00_JPRB, 0.19782E+00_JPRB, 0.19775E+00_JPRB, 0.19779E+00_JPRB ,&
&  0.19784E+00_JPRB, 0.19773E+00_JPRB, 0.19751E+00_JPRB, 0.19714E+00_JPRB, 0.19676E+00_JPRB, 0.19638E+00_JPRB, 0.19602E+00_JPRB ,&
&  0.19567E+00_JPRB, 0.19551E+00_JPRB, 0.19534E+00_JPRB, 0.19522E+00_JPRB, 0.19514E+00_JPRB, 0.19500E+00_JPRB, 0.19487E+00_JPRB ,&
&  0.19475E+00_JPRB, 0.19465E+00_JPRB, 0.19452E+00_JPRB, 0.19442E+00_JPRB, 0.19435E+00_JPRB, 0.19429E+00_JPRB, 0.19426E+00_JPRB ,&
&  0.19425E+00_JPRB, 0.19425E+00_JPRB, 0.19427E+00_JPRB, 0.19442E+00_JPRB, 0.19461E+00_JPRB, 0.19484E+00_JPRB, 0.19512E+00_JPRB ,&
&  0.19542E+00_JPRB, 0.19576E+00_JPRB, 0.19611E+00_JPRB, 0.19635E+00_JPRB, 0.19677E+00_JPRB, 0.19719E+00_JPRB, 0.19764E+00_JPRB ,&
&  0.19807E+00_JPRB, 0.19861E+00_JPRB, 0.19922E+00_JPRB, 0.19992E+00_JPRB, 0.20076E+00_JPRB, 0.20119E+00_JPRB, 0.20177E+00_JPRB ,&
&  0.20244E+00_JPRB, 0.20338E+00_JPRB, 0.20385E+00_JPRB, 0.20419E+00_JPRB, 0.20453E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB, 0.19688E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB ,&
&  0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB, 0.82922E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.83630E+00_JPRB, 0.83585E+00_JPRB, 0.83539E+00_JPRB, 0.83500E+00_JPRB, 0.83461E+00_JPRB, 0.83437E+00_JPRB, 0.83406E+00_JPRB ,&
&  0.83400E+00_JPRB, 0.83392E+00_JPRB, 0.83383E+00_JPRB, 0.83361E+00_JPRB, 0.83359E+00_JPRB, 0.83347E+00_JPRB, 0.83333E+00_JPRB ,&
&  0.83307E+00_JPRB, 0.83294E+00_JPRB, 0.83245E+00_JPRB, 0.83198E+00_JPRB, 0.83131E+00_JPRB, 0.83085E+00_JPRB, 0.83010E+00_JPRB ,&
&  0.82961E+00_JPRB, 0.82875E+00_JPRB, 0.82795E+00_JPRB, 0.82649E+00_JPRB, 0.82541E+00_JPRB, 0.82424E+00_JPRB, 0.82324E+00_JPRB ,&
&  0.82237E+00_JPRB, 0.82184E+00_JPRB, 0.82131E+00_JPRB, 0.82055E+00_JPRB, 0.81995E+00_JPRB, 0.82008E+00_JPRB, 0.82039E+00_JPRB ,&
&  0.82071E+00_JPRB, 0.82117E+00_JPRB, 0.82177E+00_JPRB, 0.82202E+00_JPRB, 0.82190E+00_JPRB, 0.82200E+00_JPRB, 0.82196E+00_JPRB ,&
&  0.82174E+00_JPRB, 0.82134E+00_JPRB, 0.82096E+00_JPRB, 0.82090E+00_JPRB, 0.82086E+00_JPRB, 0.82055E+00_JPRB, 0.82037E+00_JPRB ,&
&  0.82020E+00_JPRB, 0.81997E+00_JPRB, 0.81965E+00_JPRB, 0.81913E+00_JPRB, 0.81873E+00_JPRB, 0.81834E+00_JPRB, 0.81788E+00_JPRB ,&
&  0.81737E+00_JPRB, 0.81674E+00_JPRB, 0.81609E+00_JPRB, 0.81538E+00_JPRB, 0.81463E+00_JPRB, 0.81408E+00_JPRB, 0.81371E+00_JPRB ,&
&  0.81346E+00_JPRB, 0.81344E+00_JPRB, 0.81357E+00_JPRB, 0.81376E+00_JPRB, 0.81406E+00_JPRB, 0.81457E+00_JPRB, 0.81528E+00_JPRB ,&
&  0.81610E+00_JPRB, 0.81701E+00_JPRB, 0.81822E+00_JPRB, 0.81971E+00_JPRB, 0.82114E+00_JPRB, 0.82260E+00_JPRB, 0.82423E+00_JPRB ,&
&  0.82567E+00_JPRB, 0.82678E+00_JPRB, 0.82761E+00_JPRB, 0.82849E+00_JPRB, 0.82981E+00_JPRB, 0.83089E+00_JPRB, 0.83180E+00_JPRB ,&
&  0.83261E+00_JPRB, 0.83350E+00_JPRB, 0.83434E+00_JPRB, 0.83523E+00_JPRB, 0.83560E+00_JPRB, 0.83598E+00_JPRB, 0.83630E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.83667E+00_JPRB, 0.83670E+00_JPRB, 0.83668E+00_JPRB, 0.83654E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.83471E+00_JPRB, 0.83360E+00_JPRB, 0.83284E+00_JPRB, 0.83241E+00_JPRB, 0.83213E+00_JPRB, 0.83210E+00_JPRB, 0.83221E+00_JPRB ,&
&  0.83252E+00_JPRB, 0.83243E+00_JPRB, 0.83194E+00_JPRB, 0.83201E+00_JPRB, 0.83275E+00_JPRB, 0.83364E+00_JPRB, 0.83342E+00_JPRB ,&
&  0.83335E+00_JPRB, 0.83309E+00_JPRB, 0.83304E+00_JPRB, 0.83247E+00_JPRB, 0.83145E+00_JPRB, 0.83000E+00_JPRB, 0.82842E+00_JPRB ,&
&  0.82721E+00_JPRB, 0.82557E+00_JPRB, 0.82379E+00_JPRB, 0.81999E+00_JPRB, 0.81675E+00_JPRB, 0.81360E+00_JPRB, 0.81063E+00_JPRB ,&
&  0.80647E+00_JPRB, 0.80288E+00_JPRB, 0.80010E+00_JPRB, 0.79784E+00_JPRB, 0.80150E+00_JPRB, 0.80394E+00_JPRB, 0.80535E+00_JPRB ,&
&  0.80552E+00_JPRB, 0.80172E+00_JPRB, 0.80061E+00_JPRB, 0.80009E+00_JPRB, 0.79988E+00_JPRB, 0.80318E+00_JPRB, 0.80547E+00_JPRB ,&
&  0.80740E+00_JPRB, 0.80897E+00_JPRB, 0.81247E+00_JPRB, 0.81420E+00_JPRB, 0.81543E+00_JPRB, 0.81614E+00_JPRB, 0.81602E+00_JPRB ,&
&  0.81660E+00_JPRB, 0.81720E+00_JPRB, 0.81771E+00_JPRB, 0.81752E+00_JPRB, 0.81711E+00_JPRB, 0.81670E+00_JPRB, 0.81583E+00_JPRB ,&
&  0.81437E+00_JPRB, 0.81276E+00_JPRB, 0.81114E+00_JPRB, 0.80943E+00_JPRB, 0.80749E+00_JPRB, 0.80608E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.80370E+00_JPRB, 0.80277E+00_JPRB, 0.80219E+00_JPRB, 0.80179E+00_JPRB, 0.80164E+00_JPRB, 0.80213E+00_JPRB, 0.80348E+00_JPRB ,&
&  0.80554E+00_JPRB, 0.80819E+00_JPRB, 0.81134E+00_JPRB, 0.81471E+00_JPRB, 0.81761E+00_JPRB, 0.82017E+00_JPRB, 0.82327E+00_JPRB ,&
&  0.82661E+00_JPRB, 0.82923E+00_JPRB, 0.83138E+00_JPRB, 0.83332E+00_JPRB, 0.83572E+00_JPRB, 0.83713E+00_JPRB, 0.83787E+00_JPRB ,&
&  0.83841E+00_JPRB, 0.83964E+00_JPRB, 0.84095E+00_JPRB, 0.84248E+00_JPRB, 0.84326E+00_JPRB, 0.84326E+00_JPRB, 0.84266E+00_JPRB ,&
&  0.84181E+00_JPRB, 0.84097E+00_JPRB, 0.83969E+00_JPRB, 0.83801E+00_JPRB, 0.83630E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.82980E+00_JPRB, 0.83098E+00_JPRB, 0.83278E+00_JPRB, 0.83509E+00_JPRB, 0.83781E+00_JPRB, 0.83977E+00_JPRB, 0.84190E+00_JPRB ,&
&  0.84339E+00_JPRB, 0.84391E+00_JPRB, 0.84354E+00_JPRB, 0.84424E+00_JPRB, 0.84449E+00_JPRB, 0.84505E+00_JPRB, 0.84416E+00_JPRB ,&
&  0.84395E+00_JPRB, 0.84273E+00_JPRB, 0.84365E+00_JPRB, 0.84410E+00_JPRB, 0.84404E+00_JPRB, 0.84427E+00_JPRB, 0.83745E+00_JPRB ,&
&  0.83116E+00_JPRB, 0.82648E+00_JPRB, 0.81996E+00_JPRB, 0.81361E+00_JPRB, 0.80932E+00_JPRB, 0.78318E+00_JPRB, 0.77538E+00_JPRB ,&
&  0.78836E+00_JPRB, 0.79647E+00_JPRB, 0.80301E+00_JPRB, 0.80737E+00_JPRB, 0.79070E+00_JPRB, 0.77261E+00_JPRB, 0.76176E+00_JPRB ,&
&  0.73636E+00_JPRB, 0.74449E+00_JPRB, 0.75025E+00_JPRB, 0.75484E+00_JPRB, 0.76211E+00_JPRB, 0.76436E+00_JPRB, 0.77123E+00_JPRB ,&
&  0.78771E+00_JPRB, 0.79235E+00_JPRB, 0.79888E+00_JPRB, 0.80443E+00_JPRB, 0.80793E+00_JPRB, 0.81211E+00_JPRB, 0.82040E+00_JPRB ,&
&  0.82408E+00_JPRB, 0.82755E+00_JPRB, 0.82977E+00_JPRB, 0.82952E+00_JPRB, 0.82980E+00_JPRB, 0.82886E+00_JPRB, 0.82760E+00_JPRB ,&
&  0.82403E+00_JPRB, 0.82207E+00_JPRB, 0.82041E+00_JPRB, 0.81841E+00_JPRB, 0.81691E+00_JPRB, 0.81270E+00_JPRB, 0.80964E+00_JPRB ,&
&  0.80765E+00_JPRB, 0.80661E+00_JPRB, 0.80616E+00_JPRB, 0.80672E+00_JPRB, 0.80857E+00_JPRB, 0.81124E+00_JPRB, 0.81410E+00_JPRB ,&
&  0.81799E+00_JPRB, 0.82153E+00_JPRB, 0.82598E+00_JPRB, 0.83044E+00_JPRB, 0.83403E+00_JPRB, 0.83697E+00_JPRB, 0.83974E+00_JPRB ,&
&  0.84318E+00_JPRB, 0.84693E+00_JPRB, 0.84907E+00_JPRB, 0.84914E+00_JPRB, 0.85138E+00_JPRB, 0.85316E+00_JPRB, 0.85380E+00_JPRB ,&
&  0.85422E+00_JPRB, 0.85513E+00_JPRB, 0.85435E+00_JPRB, 0.85239E+00_JPRB, 0.84978E+00_JPRB, 0.84725E+00_JPRB, 0.84444E+00_JPRB ,&
&  0.84140E+00_JPRB, 0.83850E+00_JPRB, 0.83553E+00_JPRB, 0.83247E+00_JPRB, 0.83039E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.83502E+00_JPRB, 0.83952E+00_JPRB, 0.84513E+00_JPRB, 0.84851E+00_JPRB, 0.85275E+00_JPRB, 0.85447E+00_JPRB, 0.85703E+00_JPRB ,&
&  0.85776E+00_JPRB, 0.85898E+00_JPRB, 0.85979E+00_JPRB, 0.85991E+00_JPRB, 0.85972E+00_JPRB, 0.85857E+00_JPRB, 0.85759E+00_JPRB ,&
&  0.85711E+00_JPRB, 0.85617E+00_JPRB, 0.85675E+00_JPRB, 0.85692E+00_JPRB, 0.85637E+00_JPRB, 0.85593E+00_JPRB, 0.84810E+00_JPRB ,&
&  0.83945E+00_JPRB, 0.83114E+00_JPRB, 0.79809E+00_JPRB, 0.79760E+00_JPRB, 0.79384E+00_JPRB, 0.79201E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.78995E+00_JPRB, 0.78470E+00_JPRB, 0.76484E+00_JPRB, 0.74592E+00_JPRB, 0.73292E+00_JPRB, 0.71909E+00_JPRB, 0.71966E+00_JPRB ,&
&  0.71910E+00_JPRB, 0.72474E+00_JPRB, 0.72986E+00_JPRB, 0.73274E+00_JPRB, 0.74219E+00_JPRB, 0.74696E+00_JPRB, 0.75746E+00_JPRB ,&
&  0.77072E+00_JPRB, 0.77918E+00_JPRB, 0.77698E+00_JPRB, 0.77465E+00_JPRB, 0.79508E+00_JPRB, 0.80921E+00_JPRB, 0.82196E+00_JPRB ,&
&  0.83016E+00_JPRB, 0.83183E+00_JPRB, 0.83566E+00_JPRB, 0.83558E+00_JPRB, 0.83680E+00_JPRB, 0.84041E+00_JPRB, 0.84133E+00_JPRB ,&
&  0.84310E+00_JPRB, 0.84158E+00_JPRB, 0.83800E+00_JPRB, 0.83319E+00_JPRB, 0.82552E+00_JPRB, 0.82046E+00_JPRB, 0.81734E+00_JPRB ,&
&  0.81702E+00_JPRB, 0.81937E+00_JPRB, 0.82286E+00_JPRB, 0.82796E+00_JPRB, 0.83328E+00_JPRB, 0.83744E+00_JPRB, 0.84026E+00_JPRB ,&
&  0.84236E+00_JPRB, 0.84360E+00_JPRB, 0.84944E+00_JPRB, 0.85070E+00_JPRB, 0.85391E+00_JPRB, 0.86200E+00_JPRB, 0.86172E+00_JPRB ,&
&  0.86381E+00_JPRB, 0.86904E+00_JPRB, 0.87245E+00_JPRB, 0.87538E+00_JPRB, 0.87740E+00_JPRB, 0.87836E+00_JPRB, 0.87959E+00_JPRB ,&
&  0.87995E+00_JPRB, 0.87870E+00_JPRB, 0.87527E+00_JPRB, 0.87044E+00_JPRB, 0.86520E+00_JPRB, 0.85978E+00_JPRB, 0.85275E+00_JPRB ,&
&  0.84584E+00_JPRB, 0.83901E+00_JPRB, 0.83415E+00_JPRB, 0.83085E+00_JPRB, 0.83204E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.85106E+00_JPRB, 0.85550E+00_JPRB, 0.86248E+00_JPRB, 0.86492E+00_JPRB, 0.86891E+00_JPRB, 0.87033E+00_JPRB, 0.87172E+00_JPRB ,&
&  0.87234E+00_JPRB, 0.87297E+00_JPRB, 0.87272E+00_JPRB, 0.87269E+00_JPRB, 0.87207E+00_JPRB, 0.87163E+00_JPRB, 0.87098E+00_JPRB ,&
&  0.86979E+00_JPRB, 0.86859E+00_JPRB, 0.86741E+00_JPRB, 0.86637E+00_JPRB, 0.86623E+00_JPRB, 0.86612E+00_JPRB, 0.86206E+00_JPRB ,&
&  0.85554E+00_JPRB, 0.84161E+00_JPRB, 0.81617E+00_JPRB, 0.81415E+00_JPRB, 0.81166E+00_JPRB, 0.81386E+00_JPRB, 0.81262E+00_JPRB ,&
&  0.80018E+00_JPRB, 0.74987E+00_JPRB, 0.75517E+00_JPRB, 0.74490E+00_JPRB, 0.74190E+00_JPRB, 0.73826E+00_JPRB, 0.72461E+00_JPRB ,&
&  0.72008E+00_JPRB, 0.71447E+00_JPRB, 0.71339E+00_JPRB, 0.72081E+00_JPRB, 0.72750E+00_JPRB, 0.73758E+00_JPRB, 0.74953E+00_JPRB ,&
&  0.74665E+00_JPRB, 0.76355E+00_JPRB, 0.77535E+00_JPRB, 0.78296E+00_JPRB, 0.80319E+00_JPRB, 0.82036E+00_JPRB, 0.82472E+00_JPRB ,&
&  0.83002E+00_JPRB, 0.82784E+00_JPRB, 0.83094E+00_JPRB, 0.83907E+00_JPRB, 0.84775E+00_JPRB, 0.84726E+00_JPRB, 0.84735E+00_JPRB ,&
&  0.84571E+00_JPRB, 0.84387E+00_JPRB, 0.84155E+00_JPRB, 0.83843E+00_JPRB, 0.83694E+00_JPRB, 0.83892E+00_JPRB, 0.84214E+00_JPRB ,&
&  0.84883E+00_JPRB, 0.85574E+00_JPRB, 0.86366E+00_JPRB, 0.87099E+00_JPRB, 0.87684E+00_JPRB, 0.88246E+00_JPRB, 0.88780E+00_JPRB ,&
&  0.89323E+00_JPRB, 0.89712E+00_JPRB, 0.89821E+00_JPRB, 0.89896E+00_JPRB, 0.89822E+00_JPRB, 0.89512E+00_JPRB, 0.89570E+00_JPRB ,&
&  0.89958E+00_JPRB, 0.90214E+00_JPRB, 0.90330E+00_JPRB, 0.90466E+00_JPRB, 0.90705E+00_JPRB, 0.90821E+00_JPRB, 0.90869E+00_JPRB ,&
&  0.90846E+00_JPRB, 0.90621E+00_JPRB, 0.90231E+00_JPRB, 0.89356E+00_JPRB, 0.88441E+00_JPRB, 0.87846E+00_JPRB, 0.86848E+00_JPRB ,&
&  0.85860E+00_JPRB, 0.84771E+00_JPRB, 0.84295E+00_JPRB, 0.84018E+00_JPRB, 0.84488E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.87261E+00_JPRB, 0.87908E+00_JPRB, 0.88671E+00_JPRB, 0.89068E+00_JPRB, 0.89480E+00_JPRB, 0.89584E+00_JPRB, 0.89780E+00_JPRB ,&
&  0.89891E+00_JPRB, 0.90042E+00_JPRB, 0.90102E+00_JPRB, 0.90107E+00_JPRB, 0.90007E+00_JPRB, 0.89873E+00_JPRB, 0.89656E+00_JPRB ,&
&  0.89296E+00_JPRB, 0.88947E+00_JPRB, 0.88404E+00_JPRB, 0.87945E+00_JPRB, 0.87719E+00_JPRB, 0.87776E+00_JPRB, 0.87523E+00_JPRB ,&
&  0.86926E+00_JPRB, 0.85705E+00_JPRB, 0.83774E+00_JPRB, 0.83191E+00_JPRB, 0.83304E+00_JPRB, 0.82174E+00_JPRB, 0.81839E+00_JPRB ,&
&  0.81443E+00_JPRB, 0.81242E+00_JPRB, 0.80139E+00_JPRB, 0.79457E+00_JPRB, 0.77797E+00_JPRB, 0.71598E+00_JPRB, 0.71284E+00_JPRB ,&
&  0.71008E+00_JPRB, 0.70146E+00_JPRB, 0.70086E+00_JPRB, 0.70331E+00_JPRB, 0.71082E+00_JPRB, 0.73603E+00_JPRB, 0.75701E+00_JPRB ,&
&  0.75321E+00_JPRB, 0.77311E+00_JPRB, 0.78221E+00_JPRB, 0.79130E+00_JPRB, 0.80823E+00_JPRB, 0.82356E+00_JPRB, 0.82506E+00_JPRB ,&
&  0.82903E+00_JPRB, 0.83206E+00_JPRB, 0.83941E+00_JPRB, 0.84719E+00_JPRB, 0.85315E+00_JPRB, 0.85620E+00_JPRB, 0.86029E+00_JPRB ,&
&  0.86531E+00_JPRB, 0.86905E+00_JPRB, 0.87389E+00_JPRB, 0.87535E+00_JPRB, 0.87561E+00_JPRB, 0.87680E+00_JPRB, 0.88073E+00_JPRB ,&
&  0.88946E+00_JPRB, 0.89831E+00_JPRB, 0.91146E+00_JPRB, 0.91195E+00_JPRB, 0.91675E+00_JPRB, 0.92125E+00_JPRB, 0.93024E+00_JPRB ,&
&  0.93594E+00_JPRB, 0.95032E+00_JPRB, 0.96108E+00_JPRB, 0.96717E+00_JPRB, 0.95931E+00_JPRB, 0.95073E+00_JPRB, 0.94488E+00_JPRB ,&
&  0.94232E+00_JPRB, 0.94266E+00_JPRB, 0.93474E+00_JPRB, 0.93278E+00_JPRB, 0.93273E+00_JPRB, 0.93377E+00_JPRB, 0.93487E+00_JPRB ,&
&  0.93464E+00_JPRB, 0.93005E+00_JPRB, 0.92269E+00_JPRB, 0.91158E+00_JPRB, 0.90211E+00_JPRB, 0.89371E+00_JPRB, 0.87806E+00_JPRB ,&
&  0.86522E+00_JPRB, 0.85304E+00_JPRB, 0.85112E+00_JPRB, 0.85282E+00_JPRB, 0.86181E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.89571E+00_JPRB, 0.90706E+00_JPRB, 0.91777E+00_JPRB, 0.92666E+00_JPRB, 0.93024E+00_JPRB, 0.93071E+00_JPRB, 0.92826E+00_JPRB ,&
&  0.92538E+00_JPRB, 0.92448E+00_JPRB, 0.92301E+00_JPRB, 0.92190E+00_JPRB, 0.92041E+00_JPRB, 0.91729E+00_JPRB, 0.91345E+00_JPRB ,&
&  0.91005E+00_JPRB, 0.89427E+00_JPRB, 0.88622E+00_JPRB, 0.88531E+00_JPRB, 0.88254E+00_JPRB, 0.88147E+00_JPRB, 0.88048E+00_JPRB ,&
&  0.87319E+00_JPRB, 0.86334E+00_JPRB, 0.85605E+00_JPRB, 0.84575E+00_JPRB, 0.83540E+00_JPRB, 0.84157E+00_JPRB, 0.84043E+00_JPRB ,&
&  0.81697E+00_JPRB, 0.79921E+00_JPRB, 0.80634E+00_JPRB, 0.80822E+00_JPRB, 0.77520E+00_JPRB, 0.74153E+00_JPRB, 0.72300E+00_JPRB ,&
&  0.72795E+00_JPRB, 0.71243E+00_JPRB, 0.71007E+00_JPRB, 0.71252E+00_JPRB, 0.72342E+00_JPRB, 0.73437E+00_JPRB, 0.75551E+00_JPRB ,&
&  0.76063E+00_JPRB, 0.76421E+00_JPRB, 0.78304E+00_JPRB, 0.80211E+00_JPRB, 0.81311E+00_JPRB, 0.82429E+00_JPRB, 0.82914E+00_JPRB ,&
&  0.83893E+00_JPRB, 0.84856E+00_JPRB, 0.85705E+00_JPRB, 0.86502E+00_JPRB, 0.87470E+00_JPRB, 0.87827E+00_JPRB, 0.88431E+00_JPRB ,&
&  0.92379E+00_JPRB, 0.95528E+00_JPRB, 0.97058E+00_JPRB, 0.98177E+00_JPRB, 0.98387E+00_JPRB, 0.98097E+00_JPRB, 0.96886E+00_JPRB ,&
&  0.96512E+00_JPRB, 0.96970E+00_JPRB, 0.97359E+00_JPRB, 0.98101E+00_JPRB, 0.98604E+00_JPRB, 0.98650E+00_JPRB, 0.98019E+00_JPRB ,&
&  0.96797E+00_JPRB, 0.97304E+00_JPRB, 0.10583E+01_JPRB, 0.10794E+01_JPRB, 0.10669E+01_JPRB, 0.10390E+01_JPRB, 0.10084E+01_JPRB ,&
&  0.98606E+00_JPRB, 0.97238E+00_JPRB, 0.96425E+00_JPRB, 0.95801E+00_JPRB, 0.95347E+00_JPRB, 0.94639E+00_JPRB, 0.94713E+00_JPRB ,&
&  0.94401E+00_JPRB, 0.93561E+00_JPRB, 0.92680E+00_JPRB, 0.91687E+00_JPRB, 0.90769E+00_JPRB, 0.90022E+00_JPRB, 0.88887E+00_JPRB ,&
&  0.87795E+00_JPRB, 0.86742E+00_JPRB, 0.86763E+00_JPRB, 0.87321E+00_JPRB, 0.88450E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.92323E+00_JPRB, 0.93811E+00_JPRB, 0.95287E+00_JPRB, 0.96360E+00_JPRB, 0.96316E+00_JPRB, 0.94637E+00_JPRB, 0.93129E+00_JPRB ,&
&  0.92125E+00_JPRB, 0.91723E+00_JPRB, 0.91397E+00_JPRB, 0.90957E+00_JPRB, 0.90675E+00_JPRB, 0.90402E+00_JPRB, 0.90216E+00_JPRB ,&
&  0.90118E+00_JPRB, 0.89716E+00_JPRB, 0.88809E+00_JPRB, 0.87803E+00_JPRB, 0.87312E+00_JPRB, 0.87301E+00_JPRB, 0.87421E+00_JPRB ,&
&  0.87087E+00_JPRB, 0.86701E+00_JPRB, 0.86818E+00_JPRB, 0.85531E+00_JPRB, 0.84456E+00_JPRB, 0.82674E+00_JPRB, 0.81706E+00_JPRB ,&
&  0.81959E+00_JPRB, 0.82157E+00_JPRB, 0.80119E+00_JPRB, 0.80041E+00_JPRB, 0.80398E+00_JPRB, 0.77169E+00_JPRB, 0.74744E+00_JPRB ,&
&  0.73432E+00_JPRB, 0.72479E+00_JPRB, 0.72066E+00_JPRB, 0.72290E+00_JPRB, 0.74966E+00_JPRB, 0.78754E+00_JPRB, 0.79612E+00_JPRB ,&
&  0.75570E+00_JPRB, 0.77554E+00_JPRB, 0.80169E+00_JPRB, 0.81507E+00_JPRB, 0.82541E+00_JPRB, 0.84008E+00_JPRB, 0.85461E+00_JPRB ,&
&  0.86734E+00_JPRB, 0.87796E+00_JPRB, 0.88400E+00_JPRB, 0.89864E+00_JPRB, 0.90352E+00_JPRB, 0.89854E+00_JPRB, 0.91540E+00_JPRB ,&
&  0.10193E+01_JPRB, 0.10996E+01_JPRB, 0.11340E+01_JPRB, 0.11419E+01_JPRB, 0.11359E+01_JPRB, 0.11168E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.10792E+01_JPRB, 0.10707E+01_JPRB, 0.10643E+01_JPRB, 0.10577E+01_JPRB, 0.10472E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.10305E+01_JPRB, 0.10465E+01_JPRB, 0.12822E+01_JPRB, 0.12626E+01_JPRB, 0.12014E+01_JPRB, 0.11543E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.10429E+01_JPRB, 0.10278E+01_JPRB, 0.10109E+01_JPRB, 0.97569E+00_JPRB, 0.94957E+00_JPRB, 0.93810E+00_JPRB ,&
&  0.93169E+00_JPRB, 0.91993E+00_JPRB, 0.91096E+00_JPRB, 0.90339E+00_JPRB, 0.89779E+00_JPRB, 0.89621E+00_JPRB, 0.89182E+00_JPRB ,&
&  0.88669E+00_JPRB, 0.88808E+00_JPRB, 0.89369E+00_JPRB, 0.90426E+00_JPRB, 0.91356E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.94695E+00_JPRB, 0.97867E+00_JPRB, 0.10004E+01_JPRB, 0.10075E+01_JPRB, 0.99966E+00_JPRB, 0.95502E+00_JPRB, 0.91520E+00_JPRB ,&
&  0.89177E+00_JPRB, 0.88349E+00_JPRB, 0.88306E+00_JPRB, 0.88252E+00_JPRB, 0.88330E+00_JPRB, 0.88470E+00_JPRB, 0.88389E+00_JPRB ,&
&  0.88094E+00_JPRB, 0.88012E+00_JPRB, 0.87960E+00_JPRB, 0.87529E+00_JPRB, 0.86876E+00_JPRB, 0.86465E+00_JPRB, 0.86460E+00_JPRB ,&
&  0.86416E+00_JPRB, 0.86168E+00_JPRB, 0.86039E+00_JPRB, 0.85488E+00_JPRB, 0.85265E+00_JPRB, 0.84555E+00_JPRB, 0.83184E+00_JPRB ,&
&  0.81347E+00_JPRB, 0.81484E+00_JPRB, 0.82516E+00_JPRB, 0.82271E+00_JPRB, 0.82641E+00_JPRB, 0.80560E+00_JPRB, 0.77583E+00_JPRB ,&
&  0.75619E+00_JPRB, 0.74912E+00_JPRB, 0.76468E+00_JPRB, 0.80782E+00_JPRB, 0.83352E+00_JPRB, 0.79189E+00_JPRB, 0.80863E+00_JPRB ,&
&  0.81172E+00_JPRB, 0.81373E+00_JPRB, 0.81932E+00_JPRB, 0.83684E+00_JPRB, 0.85630E+00_JPRB, 0.87353E+00_JPRB, 0.88683E+00_JPRB ,&
&  0.89995E+00_JPRB, 0.91060E+00_JPRB, 0.91813E+00_JPRB, 0.91021E+00_JPRB, 0.91428E+00_JPRB, 0.95639E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.11304E+01_JPRB, 0.11521E+01_JPRB, 0.11957E+01_JPRB, 0.12232E+01_JPRB, 0.12285E+01_JPRB, 0.12213E+01_JPRB ,&
&  0.12121E+01_JPRB, 0.11850E+01_JPRB, 0.11600E+01_JPRB, 0.11468E+01_JPRB, 0.11457E+01_JPRB, 0.11512E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.11049E+01_JPRB, 0.10753E+01_JPRB, 0.10257E+01_JPRB, 0.10255E+01_JPRB, 0.10488E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.10540E+01_JPRB, 0.10460E+01_JPRB, 0.10295E+01_JPRB, 0.97231E+00_JPRB, 0.93328E+00_JPRB, 0.91046E+00_JPRB ,&
&  0.89666E+00_JPRB, 0.87853E+00_JPRB, 0.88018E+00_JPRB, 0.87695E+00_JPRB, 0.88323E+00_JPRB, 0.89093E+00_JPRB, 0.88796E+00_JPRB ,&
&  0.88611E+00_JPRB, 0.88744E+00_JPRB, 0.89598E+00_JPRB, 0.90686E+00_JPRB, 0.92473E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.10381E+01_JPRB, 0.10535E+01_JPRB, 0.10518E+01_JPRB, 0.10520E+01_JPRB, 0.10299E+01_JPRB, 0.10006E+01_JPRB, 0.96726E+00_JPRB ,&
&  0.94477E+00_JPRB, 0.91584E+00_JPRB, 0.88842E+00_JPRB, 0.86421E+00_JPRB, 0.85160E+00_JPRB, 0.84764E+00_JPRB, 0.85106E+00_JPRB ,&
&  0.85757E+00_JPRB, 0.86462E+00_JPRB, 0.86898E+00_JPRB, 0.86623E+00_JPRB, 0.86093E+00_JPRB, 0.85914E+00_JPRB, 0.86073E+00_JPRB ,&
&  0.86002E+00_JPRB, 0.85873E+00_JPRB, 0.85665E+00_JPRB, 0.85322E+00_JPRB, 0.84856E+00_JPRB, 0.84249E+00_JPRB, 0.84060E+00_JPRB ,&
&  0.84015E+00_JPRB, 0.84044E+00_JPRB, 0.84656E+00_JPRB, 0.84640E+00_JPRB, 0.85335E+00_JPRB, 0.83578E+00_JPRB, 0.81034E+00_JPRB ,&
&  0.78931E+00_JPRB, 0.79084E+00_JPRB, 0.84469E+00_JPRB, 0.81718E+00_JPRB, 0.84011E+00_JPRB, 0.85622E+00_JPRB, 0.85289E+00_JPRB ,&
&  0.85155E+00_JPRB, 0.85572E+00_JPRB, 0.87260E+00_JPRB, 0.88709E+00_JPRB, 0.90139E+00_JPRB, 0.91830E+00_JPRB, 0.93258E+00_JPRB ,&
&  0.95145E+00_JPRB, 0.97026E+00_JPRB, 0.97765E+00_JPRB, 0.96680E+00_JPRB, 0.10304E+01_JPRB, 0.12251E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.13105E+01_JPRB, 0.13391E+01_JPRB, 0.13863E+01_JPRB, 0.14237E+01_JPRB, 0.14027E+01_JPRB, 0.13857E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.13437E+01_JPRB, 0.13258E+01_JPRB, 0.13137E+01_JPRB, 0.13317E+01_JPRB, 0.13313E+01_JPRB, 0.13304E+01_JPRB, 0.13334E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.13031E+01_JPRB, 0.12691E+01_JPRB, 0.12153E+01_JPRB, 0.11868E+01_JPRB, 0.11682E+01_JPRB, 0.11585E+01_JPRB ,&
&  0.11417E+01_JPRB, 0.11319E+01_JPRB, 0.11195E+01_JPRB, 0.10928E+01_JPRB, 0.10243E+01_JPRB, 0.95767E+00_JPRB, 0.91801E+00_JPRB ,&
&  0.88981E+00_JPRB, 0.86655E+00_JPRB, 0.86714E+00_JPRB, 0.87514E+00_JPRB, 0.88922E+00_JPRB, 0.90303E+00_JPRB, 0.90019E+00_JPRB ,&
&  0.90696E+00_JPRB, 0.91756E+00_JPRB, 0.94135E+00_JPRB, 0.97372E+00_JPRB, 0.10076E+01_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.11351E+01_JPRB, 0.11406E+01_JPRB, 0.11254E+01_JPRB, 0.11124E+01_JPRB, 0.10898E+01_JPRB, 0.10464E+01_JPRB, 0.98714E+00_JPRB ,&
&  0.95952E+00_JPRB, 0.91740E+00_JPRB, 0.89223E+00_JPRB, 0.88141E+00_JPRB, 0.87269E+00_JPRB, 0.85866E+00_JPRB, 0.84394E+00_JPRB ,&
&  0.84339E+00_JPRB, 0.85555E+00_JPRB, 0.86692E+00_JPRB, 0.87097E+00_JPRB, 0.87292E+00_JPRB, 0.87021E+00_JPRB, 0.87003E+00_JPRB ,&
&  0.86944E+00_JPRB, 0.86936E+00_JPRB, 0.87145E+00_JPRB, 0.86388E+00_JPRB, 0.85893E+00_JPRB, 0.85698E+00_JPRB, 0.85513E+00_JPRB ,&
&  0.85854E+00_JPRB, 0.85785E+00_JPRB, 0.86574E+00_JPRB, 0.86407E+00_JPRB, 0.86186E+00_JPRB, 0.86516E+00_JPRB, 0.82090E+00_JPRB ,&
&  0.81246E+00_JPRB, 0.84227E+00_JPRB, 0.83103E+00_JPRB, 0.84224E+00_JPRB, 0.86258E+00_JPRB, 0.87500E+00_JPRB, 0.88436E+00_JPRB ,&
&  0.88998E+00_JPRB, 0.89743E+00_JPRB, 0.90811E+00_JPRB, 0.92310E+00_JPRB, 0.94251E+00_JPRB, 0.96156E+00_JPRB, 0.97841E+00_JPRB ,&
&  0.10119E+01_JPRB, 0.98955E+00_JPRB, 0.99847E+00_JPRB, 0.10730E+01_JPRB, 0.12930E+01_JPRB, 0.13851E+01_JPRB, 0.14569E+01_JPRB ,&
&  0.15413E+01_JPRB, 0.15889E+01_JPRB, 0.16046E+01_JPRB, 0.16275E+01_JPRB, 0.16152E+01_JPRB, 0.16008E+01_JPRB, 0.16150E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.16192E+01_JPRB, 0.16234E+01_JPRB, 0.16479E+01_JPRB, 0.16443E+01_JPRB, 0.16574E+01_JPRB, 0.16672E+01_JPRB ,&
&  0.16530E+01_JPRB, 0.16109E+01_JPRB, 0.15297E+01_JPRB, 0.14471E+01_JPRB, 0.13714E+01_JPRB, 0.13281E+01_JPRB, 0.12923E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.12253E+01_JPRB, 0.11918E+01_JPRB, 0.11452E+01_JPRB, 0.10825E+01_JPRB, 0.10312E+01_JPRB, 0.98938E+00_JPRB ,&
&  0.93161E+00_JPRB, 0.90961E+00_JPRB, 0.91445E+00_JPRB, 0.92281E+00_JPRB, 0.93756E+00_JPRB, 0.96378E+00_JPRB, 0.96221E+00_JPRB ,&
&  0.98895E+00_JPRB, 0.10094E+01_JPRB, 0.10225E+01_JPRB, 0.10502E+01_JPRB, 0.10899E+01_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.12293E+01_JPRB, 0.12266E+01_JPRB, 0.12068E+01_JPRB, 0.11863E+01_JPRB, 0.11552E+01_JPRB, 0.10912E+01_JPRB, 0.10407E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10121E+01_JPRB, 0.99063E+00_JPRB, 0.93671E+00_JPRB, 0.90885E+00_JPRB, 0.88712E+00_JPRB, 0.86633E+00_JPRB ,&
&  0.86023E+00_JPRB, 0.86717E+00_JPRB, 0.87558E+00_JPRB, 0.88461E+00_JPRB, 0.88635E+00_JPRB, 0.88436E+00_JPRB, 0.88104E+00_JPRB ,&
&  0.88004E+00_JPRB, 0.88523E+00_JPRB, 0.88816E+00_JPRB, 0.89086E+00_JPRB, 0.89224E+00_JPRB, 0.89160E+00_JPRB, 0.88342E+00_JPRB ,&
&  0.87838E+00_JPRB, 0.87817E+00_JPRB, 0.88295E+00_JPRB, 0.87997E+00_JPRB, 0.88078E+00_JPRB, 0.87867E+00_JPRB, 0.87359E+00_JPRB ,&
&  0.85072E+00_JPRB, 0.85306E+00_JPRB, 0.86125E+00_JPRB, 0.87447E+00_JPRB, 0.88925E+00_JPRB, 0.90114E+00_JPRB, 0.91165E+00_JPRB ,&
&  0.92464E+00_JPRB, 0.93990E+00_JPRB, 0.95768E+00_JPRB, 0.97593E+00_JPRB, 0.99661E+00_JPRB, 0.10228E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.11129E+01_JPRB, 0.10745E+01_JPRB, 0.12164E+01_JPRB, 0.13436E+01_JPRB, 0.15493E+01_JPRB, 0.17194E+01_JPRB, 0.17703E+01_JPRB ,&
&  0.19259E+01_JPRB, 0.19651E+01_JPRB, 0.19915E+01_JPRB, 0.19718E+01_JPRB, 0.19438E+01_JPRB, 0.19368E+01_JPRB, 0.19591E+01_JPRB ,&
&  0.19959E+01_JPRB, 0.19738E+01_JPRB, 0.20937E+01_JPRB, 0.20864E+01_JPRB, 0.20692E+01_JPRB, 0.20768E+01_JPRB, 0.20581E+01_JPRB ,&
&  0.20088E+01_JPRB, 0.19222E+01_JPRB, 0.18058E+01_JPRB, 0.16193E+01_JPRB, 0.15395E+01_JPRB, 0.14700E+01_JPRB, 0.13636E+01_JPRB ,&
&  0.13041E+01_JPRB, 0.12401E+01_JPRB, 0.11827E+01_JPRB, 0.11710E+01_JPRB, 0.10936E+01_JPRB, 0.10356E+01_JPRB, 0.99998E+00_JPRB ,&
&  0.95594E+00_JPRB, 0.95131E+00_JPRB, 0.97971E+00_JPRB, 0.10143E+01_JPRB, 0.10384E+01_JPRB, 0.10454E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10963E+01_JPRB, 0.11618E+01_JPRB, 0.11898E+01_JPRB, 0.12118E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.13645E+01_JPRB, 0.13497E+01_JPRB, 0.13224E+01_JPRB, 0.12882E+01_JPRB, 0.12257E+01_JPRB, 0.11480E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.10749E+01_JPRB, 0.10551E+01_JPRB, 0.10242E+01_JPRB, 0.98803E+00_JPRB, 0.94020E+00_JPRB, 0.88791E+00_JPRB ,&
&  0.87492E+00_JPRB, 0.89069E+00_JPRB, 0.90553E+00_JPRB, 0.91687E+00_JPRB, 0.91447E+00_JPRB, 0.91153E+00_JPRB, 0.91221E+00_JPRB ,&
&  0.91509E+00_JPRB, 0.92046E+00_JPRB, 0.92786E+00_JPRB, 0.93327E+00_JPRB, 0.93333E+00_JPRB, 0.93356E+00_JPRB, 0.93145E+00_JPRB ,&
&  0.90937E+00_JPRB, 0.89541E+00_JPRB, 0.88954E+00_JPRB, 0.88942E+00_JPRB, 0.91310E+00_JPRB, 0.92755E+00_JPRB, 0.93703E+00_JPRB ,&
&  0.93687E+00_JPRB, 0.93265E+00_JPRB, 0.93323E+00_JPRB, 0.94528E+00_JPRB, 0.95713E+00_JPRB, 0.96278E+00_JPRB, 0.97281E+00_JPRB ,&
&  0.98879E+00_JPRB, 0.10085E+01_JPRB, 0.10312E+01_JPRB, 0.10620E+01_JPRB, 0.10971E+01_JPRB, 0.11094E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.14356E+01_JPRB, 0.16205E+01_JPRB, 0.16723E+01_JPRB, 0.18739E+01_JPRB, 0.20375E+01_JPRB, 0.20863E+01_JPRB ,&
&  0.22160E+01_JPRB, 0.22556E+01_JPRB, 0.23787E+01_JPRB, 0.24155E+01_JPRB, 0.24543E+01_JPRB, 0.24832E+01_JPRB, 0.24927E+01_JPRB ,&
&  0.25117E+01_JPRB, 0.25681E+01_JPRB, 0.26347E+01_JPRB, 0.26444E+01_JPRB, 0.25866E+01_JPRB, 0.24994E+01_JPRB, 0.23791E+01_JPRB ,&
&  0.22623E+01_JPRB, 0.21296E+01_JPRB, 0.19970E+01_JPRB, 0.18915E+01_JPRB, 0.17640E+01_JPRB, 0.16205E+01_JPRB, 0.14786E+01_JPRB ,&
&  0.13424E+01_JPRB, 0.11885E+01_JPRB, 0.11220E+01_JPRB, 0.11176E+01_JPRB, 0.10973E+01_JPRB, 0.10272E+01_JPRB, 0.99195E+00_JPRB ,&
&  0.98789E+00_JPRB, 0.10270E+01_JPRB, 0.10828E+01_JPRB, 0.11208E+01_JPRB, 0.11870E+01_JPRB, 0.12347E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.12281E+01_JPRB, 0.12924E+01_JPRB, 0.13318E+01_JPRB, 0.13580E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.14877E+01_JPRB, 0.14616E+01_JPRB, 0.14339E+01_JPRB, 0.14010E+01_JPRB, 0.13353E+01_JPRB, 0.12453E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.11224E+01_JPRB, 0.10924E+01_JPRB, 0.10639E+01_JPRB, 0.10362E+01_JPRB, 0.10024E+01_JPRB, 0.96765E+00_JPRB ,&
&  0.94647E+00_JPRB, 0.95620E+00_JPRB, 0.94899E+00_JPRB, 0.94899E+00_JPRB, 0.95772E+00_JPRB, 0.95109E+00_JPRB, 0.95156E+00_JPRB ,&
&  0.96478E+00_JPRB, 0.97074E+00_JPRB, 0.97105E+00_JPRB, 0.96798E+00_JPRB, 0.96714E+00_JPRB, 0.97188E+00_JPRB, 0.97176E+00_JPRB ,&
&  0.94217E+00_JPRB, 0.91786E+00_JPRB, 0.90030E+00_JPRB, 0.89875E+00_JPRB, 0.92754E+00_JPRB, 0.97021E+00_JPRB, 0.10014E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.10373E+01_JPRB, 0.10420E+01_JPRB, 0.10458E+01_JPRB, 0.10444E+01_JPRB, 0.10415E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.10638E+01_JPRB, 0.10860E+01_JPRB, 0.11265E+01_JPRB, 0.11596E+01_JPRB, 0.12118E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.17486E+01_JPRB, 0.19021E+01_JPRB, 0.21484E+01_JPRB, 0.23513E+01_JPRB, 0.24335E+01_JPRB, 0.24705E+01_JPRB ,&
&  0.25754E+01_JPRB, 0.25668E+01_JPRB, 0.27675E+01_JPRB, 0.27651E+01_JPRB, 0.27941E+01_JPRB, 0.29160E+01_JPRB, 0.29191E+01_JPRB ,&
&  0.28896E+01_JPRB, 0.28386E+01_JPRB, 0.29554E+01_JPRB, 0.29594E+01_JPRB, 0.29102E+01_JPRB, 0.28262E+01_JPRB, 0.26706E+01_JPRB ,&
&  0.25143E+01_JPRB, 0.23462E+01_JPRB, 0.20342E+01_JPRB, 0.18042E+01_JPRB, 0.15587E+01_JPRB, 0.15364E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.11287E+01_JPRB, 0.11176E+01_JPRB, 0.11191E+01_JPRB, 0.11409E+01_JPRB, 0.11120E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.10699E+01_JPRB, 0.11339E+01_JPRB, 0.11675E+01_JPRB, 0.12240E+01_JPRB, 0.13267E+01_JPRB, 0.14294E+01_JPRB, 0.12614E+01_JPRB ,&
&  0.12764E+01_JPRB, 0.14080E+01_JPRB, 0.14668E+01_JPRB, 0.14961E+01_JPRB, 0.15015E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.15292E+01_JPRB, 0.14881E+01_JPRB, 0.14505E+01_JPRB, 0.14182E+01_JPRB, 0.13751E+01_JPRB, 0.13230E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.11467E+01_JPRB, 0.11164E+01_JPRB, 0.11004E+01_JPRB, 0.10846E+01_JPRB, 0.10517E+01_JPRB, 0.10495E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.10112E+01_JPRB, 0.96718E+00_JPRB, 0.97526E+00_JPRB, 0.10276E+01_JPRB, 0.99662E+00_JPRB, 0.99593E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.10115E+01_JPRB, 0.10026E+01_JPRB, 0.99150E+00_JPRB, 0.98724E+00_JPRB, 0.99067E+00_JPRB, 0.10009E+01_JPRB ,&
&  0.98002E+00_JPRB, 0.93179E+00_JPRB, 0.90293E+00_JPRB, 0.90185E+00_JPRB, 0.93984E+00_JPRB, 0.98931E+00_JPRB, 0.10424E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10923E+01_JPRB, 0.11074E+01_JPRB, 0.11152E+01_JPRB, 0.11146E+01_JPRB, 0.11145E+01_JPRB, 0.11181E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.11383E+01_JPRB, 0.11697E+01_JPRB, 0.12132E+01_JPRB, 0.12289E+01_JPRB, 0.13365E+01_JPRB, 0.16237E+01_JPRB ,&
&  0.20382E+01_JPRB, 0.21762E+01_JPRB, 0.23325E+01_JPRB, 0.28092E+01_JPRB, 0.30531E+01_JPRB, 0.31289E+01_JPRB, 0.31493E+01_JPRB ,&
&  0.31808E+01_JPRB, 0.31597E+01_JPRB, 0.31722E+01_JPRB, 0.32982E+01_JPRB, 0.32836E+01_JPRB, 0.32957E+01_JPRB, 0.32122E+01_JPRB ,&
&  0.30766E+01_JPRB, 0.30716E+01_JPRB, 0.31060E+01_JPRB, 0.30354E+01_JPRB, 0.29123E+01_JPRB, 0.28719E+01_JPRB, 0.27984E+01_JPRB ,&
&  0.26542E+01_JPRB, 0.23587E+01_JPRB, 0.18007E+01_JPRB, 0.16141E+01_JPRB, 0.12850E+01_JPRB, 0.12851E+01_JPRB, 0.14630E+01_JPRB ,&
&  0.14123E+01_JPRB, 0.13135E+01_JPRB, 0.12537E+01_JPRB, 0.12373E+01_JPRB, 0.12502E+01_JPRB, 0.13027E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12414E+01_JPRB, 0.13064E+01_JPRB, 0.14233E+01_JPRB, 0.15818E+01_JPRB, 0.16616E+01_JPRB, 0.15165E+01_JPRB ,&
&  0.15905E+01_JPRB, 0.16257E+01_JPRB, 0.16331E+01_JPRB, 0.16238E+01_JPRB, 0.15834E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.14748E+01_JPRB, 0.14335E+01_JPRB, 0.13899E+01_JPRB, 0.13756E+01_JPRB, 0.13775E+01_JPRB, 0.13629E+01_JPRB, 0.13109E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.11743E+01_JPRB, 0.11400E+01_JPRB, 0.11346E+01_JPRB, 0.11337E+01_JPRB, 0.11049E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.11078E+01_JPRB, 0.10307E+01_JPRB, 0.97825E+00_JPRB, 0.10121E+01_JPRB, 0.10035E+01_JPRB, 0.10128E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.10351E+01_JPRB, 0.10300E+01_JPRB, 0.10297E+01_JPRB, 0.10349E+01_JPRB, 0.10549E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.10288E+01_JPRB, 0.97768E+00_JPRB, 0.98293E+00_JPRB, 0.10437E+01_JPRB, 0.10336E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.11384E+01_JPRB, 0.11632E+01_JPRB, 0.11859E+01_JPRB, 0.12022E+01_JPRB, 0.12130E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.12671E+01_JPRB, 0.12680E+01_JPRB, 0.13015E+01_JPRB, 0.14112E+01_JPRB, 0.17097E+01_JPRB, 0.21115E+01_JPRB ,&
&  0.23923E+01_JPRB, 0.24531E+01_JPRB, 0.25208E+01_JPRB, 0.29634E+01_JPRB, 0.33408E+01_JPRB, 0.35136E+01_JPRB, 0.37044E+01_JPRB ,&
&  0.37617E+01_JPRB, 0.37683E+01_JPRB, 0.37487E+01_JPRB, 0.35128E+01_JPRB, 0.31530E+01_JPRB, 0.29148E+01_JPRB, 0.28200E+01_JPRB ,&
&  0.27343E+01_JPRB, 0.27559E+01_JPRB, 0.28680E+01_JPRB, 0.28193E+01_JPRB, 0.27990E+01_JPRB, 0.26841E+01_JPRB, 0.28116E+01_JPRB ,&
&  0.27460E+01_JPRB, 0.17504E+01_JPRB, 0.13969E+01_JPRB, 0.14738E+01_JPRB, 0.12517E+01_JPRB, 0.12628E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.13525E+01_JPRB, 0.14076E+01_JPRB, 0.14490E+01_JPRB, 0.14677E+01_JPRB, 0.15823E+01_JPRB, 0.16238E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.14995E+01_JPRB, 0.16012E+01_JPRB, 0.17689E+01_JPRB, 0.19312E+01_JPRB, 0.19406E+01_JPRB, 0.18698E+01_JPRB ,&
&  0.18261E+01_JPRB, 0.17906E+01_JPRB, 0.17410E+01_JPRB, 0.16474E+01_JPRB, 0.15418E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.14904E+01_JPRB, 0.14533E+01_JPRB, 0.14027E+01_JPRB, 0.14033E+01_JPRB, 0.14267E+01_JPRB, 0.14149E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.12429E+01_JPRB, 0.11817E+01_JPRB, 0.11535E+01_JPRB, 0.11492E+01_JPRB, 0.11600E+01_JPRB, 0.11585E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.11469E+01_JPRB, 0.10670E+01_JPRB, 0.10212E+01_JPRB, 0.96256E+00_JPRB, 0.98523E+00_JPRB, 0.10145E+01_JPRB ,&
&  0.10404E+01_JPRB, 0.10619E+01_JPRB, 0.11141E+01_JPRB, 0.11842E+01_JPRB, 0.12485E+01_JPRB, 0.13050E+01_JPRB, 0.14173E+01_JPRB ,&
&  0.13873E+01_JPRB, 0.12681E+01_JPRB, 0.11461E+01_JPRB, 0.11492E+01_JPRB, 0.11713E+01_JPRB, 0.11208E+01_JPRB, 0.11823E+01_JPRB ,&
&  0.12176E+01_JPRB, 0.12535E+01_JPRB, 0.12861E+01_JPRB, 0.13034E+01_JPRB, 0.13063E+01_JPRB, 0.12800E+01_JPRB, 0.12726E+01_JPRB ,&
&  0.12802E+01_JPRB, 0.13073E+01_JPRB, 0.13462E+01_JPRB, 0.13998E+01_JPRB, 0.17128E+01_JPRB, 0.21598E+01_JPRB, 0.25093E+01_JPRB ,&
&  0.27595E+01_JPRB, 0.23969E+01_JPRB, 0.21600E+01_JPRB, 0.27943E+01_JPRB, 0.39542E+01_JPRB, 0.43346E+01_JPRB, 0.41186E+01_JPRB ,&
&  0.43983E+01_JPRB, 0.40743E+01_JPRB, 0.35837E+01_JPRB, 0.32350E+01_JPRB, 0.29832E+01_JPRB, 0.27930E+01_JPRB, 0.27139E+01_JPRB ,&
&  0.25620E+01_JPRB, 0.25627E+01_JPRB, 0.26977E+01_JPRB, 0.27832E+01_JPRB, 0.27591E+01_JPRB, 0.27340E+01_JPRB, 0.28136E+01_JPRB ,&
&  0.26544E+01_JPRB, 0.21972E+01_JPRB, 0.15994E+01_JPRB, 0.14953E+01_JPRB, 0.12451E+01_JPRB, 0.12294E+01_JPRB, 0.13732E+01_JPRB ,&
&  0.13712E+01_JPRB, 0.14582E+01_JPRB, 0.15208E+01_JPRB, 0.15364E+01_JPRB, 0.17300E+01_JPRB, 0.18657E+01_JPRB, 0.20617E+01_JPRB ,&
&  0.21792E+01_JPRB, 0.19265E+01_JPRB, 0.20893E+01_JPRB, 0.22773E+01_JPRB, 0.23308E+01_JPRB, 0.21970E+01_JPRB, 0.20383E+01_JPRB ,&
&  0.19407E+01_JPRB, 0.18682E+01_JPRB, 0.17718E+01_JPRB, 0.16355E+01_JPRB, 0.15402E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.15269E+01_JPRB, 0.15142E+01_JPRB, 0.14617E+01_JPRB, 0.14596E+01_JPRB, 0.14536E+01_JPRB, 0.14098E+01_JPRB, 0.13479E+01_JPRB ,&
&  0.12856E+01_JPRB, 0.12527E+01_JPRB, 0.12252E+01_JPRB, 0.12089E+01_JPRB, 0.12079E+01_JPRB, 0.11993E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.11244E+01_JPRB, 0.10347E+01_JPRB, 0.97939E+00_JPRB, 0.94772E+00_JPRB, 0.96380E+00_JPRB, 0.10095E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.11979E+01_JPRB, 0.13812E+01_JPRB, 0.15472E+01_JPRB, 0.17668E+01_JPRB, 0.19337E+01_JPRB, 0.20326E+01_JPRB ,&
&  0.20132E+01_JPRB, 0.15955E+01_JPRB, 0.15061E+01_JPRB, 0.14021E+01_JPRB, 0.13790E+01_JPRB, 0.13324E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.13364E+01_JPRB, 0.13451E+01_JPRB, 0.13252E+01_JPRB, 0.13051E+01_JPRB, 0.12981E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.13919E+01_JPRB, 0.14065E+01_JPRB, 0.14644E+01_JPRB, 0.17211E+01_JPRB, 0.22557E+01_JPRB, 0.27517E+01_JPRB, 0.30650E+01_JPRB ,&
&  0.28989E+01_JPRB, 0.27360E+01_JPRB, 0.36976E+01_JPRB, 0.40920E+01_JPRB, 0.39195E+01_JPRB, 0.45259E+01_JPRB, 0.46783E+01_JPRB ,&
&  0.42829E+01_JPRB, 0.35056E+01_JPRB, 0.31229E+01_JPRB, 0.28529E+01_JPRB, 0.33019E+01_JPRB, 0.31042E+01_JPRB, 0.27779E+01_JPRB ,&
&  0.25039E+01_JPRB, 0.26518E+01_JPRB, 0.27089E+01_JPRB, 0.28243E+01_JPRB, 0.29936E+01_JPRB, 0.31018E+01_JPRB, 0.28327E+01_JPRB ,&
&  0.25137E+01_JPRB, 0.26967E+01_JPRB, 0.21820E+01_JPRB, 0.16924E+01_JPRB, 0.14570E+01_JPRB, 0.14538E+01_JPRB, 0.15197E+01_JPRB ,&
&  0.15899E+01_JPRB, 0.16513E+01_JPRB, 0.16853E+01_JPRB, 0.18233E+01_JPRB, 0.21486E+01_JPRB, 0.23484E+01_JPRB, 0.24612E+01_JPRB ,&
&  0.24510E+01_JPRB, 0.27061E+01_JPRB, 0.27573E+01_JPRB, 0.27249E+01_JPRB, 0.24909E+01_JPRB, 0.21883E+01_JPRB, 0.20101E+01_JPRB ,&
&  0.19324E+01_JPRB, 0.18678E+01_JPRB, 0.17372E+01_JPRB, 0.16029E+01_JPRB, 0.15263E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.15513E+01_JPRB, 0.15565E+01_JPRB, 0.15417E+01_JPRB, 0.15142E+01_JPRB, 0.14752E+01_JPRB, 0.14277E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.12987E+01_JPRB, 0.12888E+01_JPRB, 0.12903E+01_JPRB, 0.12846E+01_JPRB, 0.12600E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11188E+01_JPRB, 0.10141E+01_JPRB, 0.10127E+01_JPRB, 0.10108E+01_JPRB, 0.99285E+00_JPRB, 0.10549E+01_JPRB ,&
&  0.11848E+01_JPRB, 0.14654E+01_JPRB, 0.17874E+01_JPRB, 0.21207E+01_JPRB, 0.23180E+01_JPRB, 0.23670E+01_JPRB, 0.23603E+01_JPRB ,&
&  0.21535E+01_JPRB, 0.19942E+01_JPRB, 0.18659E+01_JPRB, 0.17296E+01_JPRB, 0.16412E+01_JPRB, 0.15794E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.14421E+01_JPRB, 0.14040E+01_JPRB, 0.13855E+01_JPRB, 0.13456E+01_JPRB, 0.13439E+01_JPRB, 0.13774E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.14891E+01_JPRB, 0.16610E+01_JPRB, 0.20919E+01_JPRB, 0.23221E+01_JPRB, 0.27152E+01_JPRB, 0.30764E+01_JPRB ,&
&  0.40427E+01_JPRB, 0.43304E+01_JPRB, 0.39954E+01_JPRB, 0.47679E+01_JPRB, 0.32513E+01_JPRB, 0.37771E+01_JPRB, 0.43769E+01_JPRB ,&
&  0.40479E+01_JPRB, 0.38311E+01_JPRB, 0.40125E+01_JPRB, 0.35623E+01_JPRB, 0.30606E+01_JPRB, 0.40857E+01_JPRB, 0.33911E+01_JPRB ,&
&  0.29121E+01_JPRB, 0.29182E+01_JPRB, 0.30710E+01_JPRB, 0.34472E+01_JPRB, 0.30401E+01_JPRB, 0.27507E+01_JPRB, 0.22598E+01_JPRB ,&
&  0.22693E+01_JPRB, 0.20840E+01_JPRB, 0.22450E+01_JPRB, 0.19814E+01_JPRB, 0.19804E+01_JPRB, 0.18487E+01_JPRB, 0.17653E+01_JPRB ,&
&  0.18585E+01_JPRB, 0.18850E+01_JPRB, 0.19145E+01_JPRB, 0.22708E+01_JPRB, 0.28984E+01_JPRB, 0.26960E+01_JPRB, 0.29383E+01_JPRB ,&
&  0.31265E+01_JPRB, 0.31811E+01_JPRB, 0.29247E+01_JPRB, 0.27053E+01_JPRB, 0.23161E+01_JPRB, 0.20796E+01_JPRB, 0.19811E+01_JPRB ,&
&  0.19091E+01_JPRB, 0.17794E+01_JPRB, 0.16317E+01_JPRB, 0.15700E+01_JPRB, 0.15178E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.15524E+01_JPRB, 0.15333E+01_JPRB, 0.15219E+01_JPRB, 0.14905E+01_JPRB, 0.14764E+01_JPRB, 0.14410E+01_JPRB, 0.14121E+01_JPRB ,&
&  0.13754E+01_JPRB, 0.13382E+01_JPRB, 0.13331E+01_JPRB, 0.13282E+01_JPRB, 0.13179E+01_JPRB, 0.13084E+01_JPRB, 0.12732E+01_JPRB ,&
&  0.12690E+01_JPRB, 0.12101E+01_JPRB, 0.10663E+01_JPRB, 0.10257E+01_JPRB, 0.10506E+01_JPRB, 0.10549E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.14794E+01_JPRB, 0.19972E+01_JPRB, 0.23216E+01_JPRB, 0.25383E+01_JPRB, 0.26025E+01_JPRB, 0.24675E+01_JPRB, 0.23253E+01_JPRB ,&
&  0.22648E+01_JPRB, 0.19908E+01_JPRB, 0.18294E+01_JPRB, 0.17636E+01_JPRB, 0.16786E+01_JPRB, 0.15949E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.14513E+01_JPRB, 0.13817E+01_JPRB, 0.13380E+01_JPRB, 0.12973E+01_JPRB, 0.13157E+01_JPRB, 0.13694E+01_JPRB, 0.14430E+01_JPRB ,&
&  0.14999E+01_JPRB, 0.16178E+01_JPRB, 0.19484E+01_JPRB, 0.23839E+01_JPRB, 0.27415E+01_JPRB, 0.31185E+01_JPRB, 0.39591E+01_JPRB ,&
&  0.43899E+01_JPRB, 0.42649E+01_JPRB, 0.43752E+01_JPRB, 0.46114E+01_JPRB, 0.44059E+01_JPRB, 0.38985E+01_JPRB, 0.42166E+01_JPRB ,&
&  0.33162E+01_JPRB, 0.29869E+01_JPRB, 0.29088E+01_JPRB, 0.29465E+01_JPRB, 0.36831E+01_JPRB, 0.54939E+01_JPRB, 0.49138E+01_JPRB ,&
&  0.44109E+01_JPRB, 0.38571E+01_JPRB, 0.39359E+01_JPRB, 0.36979E+01_JPRB, 0.32857E+01_JPRB, 0.25187E+01_JPRB, 0.30866E+01_JPRB ,&
&  0.32418E+01_JPRB, 0.33891E+01_JPRB, 0.30809E+01_JPRB, 0.25134E+01_JPRB, 0.21307E+01_JPRB, 0.23082E+01_JPRB, 0.23121E+01_JPRB ,&
&  0.23291E+01_JPRB, 0.22877E+01_JPRB, 0.28652E+01_JPRB, 0.32863E+01_JPRB, 0.35242E+01_JPRB, 0.34797E+01_JPRB, 0.37802E+01_JPRB ,&
&  0.35170E+01_JPRB, 0.33901E+01_JPRB, 0.30184E+01_JPRB, 0.25149E+01_JPRB, 0.21847E+01_JPRB, 0.20355E+01_JPRB, 0.19271E+01_JPRB ,&
&  0.18074E+01_JPRB, 0.16443E+01_JPRB, 0.15569E+01_JPRB, 0.15066E+01_JPRB, 0.14955E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.15219E+01_JPRB, 0.14952E+01_JPRB, 0.14537E+01_JPRB, 0.14330E+01_JPRB, 0.14148E+01_JPRB, 0.13792E+01_JPRB, 0.13748E+01_JPRB ,&
&  0.13974E+01_JPRB, 0.13653E+01_JPRB, 0.13240E+01_JPRB, 0.12958E+01_JPRB, 0.12846E+01_JPRB, 0.13267E+01_JPRB, 0.13656E+01_JPRB ,&
&  0.13788E+01_JPRB, 0.13680E+01_JPRB, 0.11743E+01_JPRB, 0.11265E+01_JPRB, 0.11453E+01_JPRB, 0.12088E+01_JPRB, 0.13121E+01_JPRB ,&
&  0.19219E+01_JPRB, 0.24950E+01_JPRB, 0.25942E+01_JPRB, 0.26228E+01_JPRB, 0.25839E+01_JPRB, 0.24122E+01_JPRB, 0.23156E+01_JPRB ,&
&  0.22517E+01_JPRB, 0.20452E+01_JPRB, 0.18537E+01_JPRB, 0.17363E+01_JPRB, 0.16293E+01_JPRB, 0.15552E+01_JPRB, 0.14927E+01_JPRB ,&
&  0.14146E+01_JPRB, 0.13538E+01_JPRB, 0.13109E+01_JPRB, 0.13018E+01_JPRB, 0.13029E+01_JPRB, 0.14078E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.16618E+01_JPRB, 0.19219E+01_JPRB, 0.22472E+01_JPRB, 0.26851E+01_JPRB, 0.31821E+01_JPRB, 0.36554E+01_JPRB, 0.50199E+01_JPRB ,&
&  0.48891E+01_JPRB, 0.48922E+01_JPRB, 0.47895E+01_JPRB, 0.54725E+01_JPRB, 0.42947E+01_JPRB, 0.37531E+01_JPRB, 0.39370E+01_JPRB ,&
&  0.30806E+01_JPRB, 0.28708E+01_JPRB, 0.33541E+01_JPRB, 0.42667E+01_JPRB, 0.44024E+01_JPRB, 0.48723E+01_JPRB, 0.64508E+01_JPRB ,&
&  0.56869E+01_JPRB, 0.49852E+01_JPRB, 0.47333E+01_JPRB, 0.47317E+01_JPRB, 0.25300E+01_JPRB, 0.21074E+01_JPRB, 0.32432E+01_JPRB ,&
&  0.34329E+01_JPRB, 0.23539E+01_JPRB, 0.18190E+01_JPRB, 0.21726E+01_JPRB, 0.17120E+01_JPRB, 0.19866E+01_JPRB, 0.26722E+01_JPRB ,&
&  0.28129E+01_JPRB, 0.30162E+01_JPRB, 0.44113E+01_JPRB, 0.43086E+01_JPRB, 0.42182E+01_JPRB, 0.54320E+01_JPRB, 0.45541E+01_JPRB ,&
&  0.39793E+01_JPRB, 0.30812E+01_JPRB, 0.27452E+01_JPRB, 0.23121E+01_JPRB, 0.20678E+01_JPRB, 0.18906E+01_JPRB, 0.17589E+01_JPRB ,&
&  0.16206E+01_JPRB, 0.15385E+01_JPRB, 0.15314E+01_JPRB, 0.15259E+01_JPRB, 0.15058E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.14995E+01_JPRB, 0.14429E+01_JPRB, 0.13795E+01_JPRB, 0.13682E+01_JPRB, 0.13406E+01_JPRB, 0.13099E+01_JPRB, 0.13938E+01_JPRB ,&
&  0.14547E+01_JPRB, 0.13578E+01_JPRB, 0.12723E+01_JPRB, 0.12523E+01_JPRB, 0.12771E+01_JPRB, 0.14118E+01_JPRB, 0.14930E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.15045E+01_JPRB, 0.14326E+01_JPRB, 0.14199E+01_JPRB, 0.14735E+01_JPRB, 0.15759E+01_JPRB, 0.18506E+01_JPRB ,&
&  0.26024E+01_JPRB, 0.27955E+01_JPRB, 0.25780E+01_JPRB, 0.24342E+01_JPRB, 0.23843E+01_JPRB, 0.22802E+01_JPRB, 0.23164E+01_JPRB ,&
&  0.21984E+01_JPRB, 0.19800E+01_JPRB, 0.18168E+01_JPRB, 0.17037E+01_JPRB, 0.16513E+01_JPRB, 0.15994E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.14359E+01_JPRB, 0.13570E+01_JPRB, 0.13362E+01_JPRB, 0.13155E+01_JPRB, 0.13691E+01_JPRB, 0.15060E+01_JPRB, 0.16741E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.22110E+01_JPRB, 0.25378E+01_JPRB, 0.30654E+01_JPRB, 0.39223E+01_JPRB, 0.43676E+01_JPRB, 0.40936E+01_JPRB ,&
&  0.37952E+01_JPRB, 0.37619E+01_JPRB, 0.48592E+01_JPRB, 0.49655E+01_JPRB, 0.43850E+01_JPRB, 0.40944E+01_JPRB, 0.38836E+01_JPRB ,&
&  0.34942E+01_JPRB, 0.34060E+01_JPRB, 0.38330E+01_JPRB, 0.49218E+01_JPRB, 0.55189E+01_JPRB, 0.52200E+01_JPRB, 0.66111E+01_JPRB ,&
&  0.67034E+01_JPRB, 0.58807E+01_JPRB, 0.41700E+01_JPRB, 0.31764E+01_JPRB, 0.29202E+01_JPRB, 0.26405E+01_JPRB, 0.19163E+01_JPRB ,&
&  0.17421E+01_JPRB, 0.17302E+01_JPRB, 0.16311E+01_JPRB, 0.16584E+01_JPRB, 0.16166E+01_JPRB, 0.20806E+01_JPRB, 0.27189E+01_JPRB ,&
&  0.36703E+01_JPRB, 0.43776E+01_JPRB, 0.50258E+01_JPRB, 0.49885E+01_JPRB, 0.47776E+01_JPRB, 0.64377E+01_JPRB, 0.49031E+01_JPRB ,&
&  0.32894E+01_JPRB, 0.26680E+01_JPRB, 0.23882E+01_JPRB, 0.21158E+01_JPRB, 0.19109E+01_JPRB, 0.17250E+01_JPRB, 0.15805E+01_JPRB ,&
&  0.15061E+01_JPRB, 0.15054E+01_JPRB, 0.15195E+01_JPRB, 0.15201E+01_JPRB, 0.14790E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.14769E+01_JPRB, 0.14229E+01_JPRB, 0.13645E+01_JPRB, 0.13177E+01_JPRB, 0.12631E+01_JPRB, 0.12901E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.15052E+01_JPRB, 0.13498E+01_JPRB, 0.12485E+01_JPRB, 0.12574E+01_JPRB, 0.13835E+01_JPRB, 0.15307E+01_JPRB, 0.15842E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.16569E+01_JPRB, 0.17507E+01_JPRB, 0.18931E+01_JPRB, 0.21653E+01_JPRB, 0.22778E+01_JPRB, 0.28144E+01_JPRB ,&
&  0.32504E+01_JPRB, 0.28390E+01_JPRB, 0.24388E+01_JPRB, 0.23014E+01_JPRB, 0.22933E+01_JPRB, 0.22718E+01_JPRB, 0.22487E+01_JPRB ,&
&  0.20585E+01_JPRB, 0.18884E+01_JPRB, 0.17505E+01_JPRB, 0.16798E+01_JPRB, 0.16387E+01_JPRB, 0.15424E+01_JPRB, 0.14728E+01_JPRB ,&
&  0.13991E+01_JPRB, 0.13704E+01_JPRB, 0.13523E+01_JPRB, 0.13872E+01_JPRB, 0.15209E+01_JPRB, 0.17083E+01_JPRB, 0.19293E+01_JPRB ,&
&  0.21888E+01_JPRB, 0.25530E+01_JPRB, 0.29460E+01_JPRB, 0.35688E+01_JPRB, 0.35740E+01_JPRB, 0.31322E+01_JPRB, 0.35724E+01_JPRB ,&
&  0.39249E+01_JPRB, 0.42528E+01_JPRB, 0.41649E+01_JPRB, 0.49731E+01_JPRB, 0.46181E+01_JPRB, 0.39611E+01_JPRB, 0.37361E+01_JPRB ,&
&  0.34182E+01_JPRB, 0.41084E+01_JPRB, 0.47668E+01_JPRB, 0.54733E+01_JPRB, 0.72049E+01_JPRB, 0.68866E+01_JPRB, 0.74140E+01_JPRB ,&
&  0.77087E+01_JPRB, 0.64351E+01_JPRB, 0.47871E+01_JPRB, 0.36884E+01_JPRB, 0.45313E+01_JPRB, 0.53976E+01_JPRB, 0.23799E+01_JPRB ,&
&  0.19472E+01_JPRB, 0.18091E+01_JPRB, 0.17744E+01_JPRB, 0.16646E+01_JPRB, 0.16678E+01_JPRB, 0.21672E+01_JPRB, 0.38308E+01_JPRB ,&
&  0.38636E+01_JPRB, 0.57354E+01_JPRB, 0.57019E+01_JPRB, 0.55728E+01_JPRB, 0.50968E+01_JPRB, 0.61315E+01_JPRB, 0.41368E+01_JPRB ,&
&  0.32783E+01_JPRB, 0.27449E+01_JPRB, 0.23390E+01_JPRB, 0.20972E+01_JPRB, 0.18502E+01_JPRB, 0.16307E+01_JPRB, 0.15367E+01_JPRB ,&
&  0.14886E+01_JPRB, 0.14560E+01_JPRB, 0.14493E+01_JPRB, 0.14485E+01_JPRB, 0.14515E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.14682E+01_JPRB, 0.14118E+01_JPRB, 0.13641E+01_JPRB, 0.13241E+01_JPRB, 0.12864E+01_JPRB, 0.13842E+01_JPRB, 0.14917E+01_JPRB ,&
&  0.15149E+01_JPRB, 0.14085E+01_JPRB, 0.13388E+01_JPRB, 0.13851E+01_JPRB, 0.15225E+01_JPRB, 0.16867E+01_JPRB, 0.17841E+01_JPRB ,&
&  0.18899E+01_JPRB, 0.19972E+01_JPRB, 0.22037E+01_JPRB, 0.27351E+01_JPRB, 0.31429E+01_JPRB, 0.33869E+01_JPRB, 0.38536E+01_JPRB ,&
&  0.34182E+01_JPRB, 0.26073E+01_JPRB, 0.23424E+01_JPRB, 0.23163E+01_JPRB, 0.22056E+01_JPRB, 0.21246E+01_JPRB, 0.20503E+01_JPRB ,&
&  0.19347E+01_JPRB, 0.18278E+01_JPRB, 0.17370E+01_JPRB, 0.17131E+01_JPRB, 0.16737E+01_JPRB, 0.15746E+01_JPRB, 0.15255E+01_JPRB ,&
&  0.14790E+01_JPRB, 0.14561E+01_JPRB, 0.14926E+01_JPRB, 0.15406E+01_JPRB, 0.17314E+01_JPRB, 0.19256E+01_JPRB, 0.21842E+01_JPRB ,&
&  0.25196E+01_JPRB, 0.28526E+01_JPRB, 0.32233E+01_JPRB, 0.36606E+01_JPRB, 0.31409E+01_JPRB, 0.37376E+01_JPRB, 0.39459E+01_JPRB ,&
&  0.41307E+01_JPRB, 0.39939E+01_JPRB, 0.36290E+01_JPRB, 0.41499E+01_JPRB, 0.42993E+01_JPRB, 0.37543E+01_JPRB, 0.34722E+01_JPRB ,&
&  0.39843E+01_JPRB, 0.45988E+01_JPRB, 0.57851E+01_JPRB, 0.67836E+01_JPRB, 0.78092E+01_JPRB, 0.10669E+02_JPRB, 0.76260E+01_JPRB ,&
&  0.64567E+01_JPRB, 0.64344E+01_JPRB, 0.55295E+01_JPRB, 0.47911E+01_JPRB, 0.55816E+01_JPRB, 0.87294E+01_JPRB, 0.63233E+01_JPRB ,&
&  0.23322E+01_JPRB, 0.17805E+01_JPRB, 0.20208E+01_JPRB, 0.20553E+01_JPRB, 0.18050E+01_JPRB, 0.22608E+01_JPRB, 0.62836E+01_JPRB ,&
&  0.49184E+01_JPRB, 0.67237E+01_JPRB, 0.59124E+01_JPRB, 0.54743E+01_JPRB, 0.53024E+01_JPRB, 0.55564E+01_JPRB, 0.43874E+01_JPRB ,&
&  0.32743E+01_JPRB, 0.27207E+01_JPRB, 0.23142E+01_JPRB, 0.20206E+01_JPRB, 0.17514E+01_JPRB, 0.15822E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.14526E+01_JPRB, 0.14389E+01_JPRB, 0.14354E+01_JPRB, 0.14320E+01_JPRB, 0.14501E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.14730E+01_JPRB, 0.13818E+01_JPRB, 0.13388E+01_JPRB, 0.13077E+01_JPRB, 0.13779E+01_JPRB, 0.14873E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.16215E+01_JPRB, 0.16463E+01_JPRB, 0.16453E+01_JPRB, 0.16452E+01_JPRB, 0.17605E+01_JPRB, 0.19771E+01_JPRB, 0.21539E+01_JPRB ,&
&  0.23778E+01_JPRB, 0.26309E+01_JPRB, 0.29800E+01_JPRB, 0.35437E+01_JPRB, 0.41286E+01_JPRB, 0.41146E+01_JPRB, 0.39690E+01_JPRB ,&
&  0.32797E+01_JPRB, 0.25477E+01_JPRB, 0.22089E+01_JPRB, 0.22774E+01_JPRB, 0.22203E+01_JPRB, 0.21667E+01_JPRB, 0.21509E+01_JPRB ,&
&  0.21504E+01_JPRB, 0.20562E+01_JPRB, 0.19486E+01_JPRB, 0.19120E+01_JPRB, 0.17802E+01_JPRB, 0.17327E+01_JPRB, 0.18113E+01_JPRB ,&
&  0.17725E+01_JPRB, 0.17718E+01_JPRB, 0.17998E+01_JPRB, 0.18670E+01_JPRB, 0.20735E+01_JPRB, 0.22756E+01_JPRB, 0.25479E+01_JPRB ,&
&  0.28531E+01_JPRB, 0.31000E+01_JPRB, 0.34164E+01_JPRB, 0.34818E+01_JPRB, 0.35225E+01_JPRB, 0.36791E+01_JPRB, 0.40036E+01_JPRB ,&
&  0.40820E+01_JPRB, 0.39195E+01_JPRB, 0.37097E+01_JPRB, 0.37279E+01_JPRB, 0.37391E+01_JPRB, 0.35718E+01_JPRB, 0.36483E+01_JPRB ,&
&  0.46716E+01_JPRB, 0.60773E+01_JPRB, 0.71361E+01_JPRB, 0.80912E+01_JPRB, 0.86928E+01_JPRB, 0.10811E+02_JPRB, 0.99361E+01_JPRB ,&
&  0.77132E+01_JPRB, 0.60764E+01_JPRB, 0.57911E+01_JPRB, 0.66701E+01_JPRB, 0.73823E+01_JPRB, 0.86659E+01_JPRB, 0.10163E+02_JPRB ,&
&  0.10039E+02_JPRB, 0.52786E+01_JPRB, 0.38699E+01_JPRB, 0.55175E+01_JPRB, 0.36330E+01_JPRB, 0.25989E+01_JPRB, 0.37665E+01_JPRB ,&
&  0.52785E+01_JPRB, 0.63795E+01_JPRB, 0.60402E+01_JPRB, 0.53215E+01_JPRB, 0.53405E+01_JPRB, 0.49165E+01_JPRB, 0.38290E+01_JPRB ,&
&  0.29489E+01_JPRB, 0.23898E+01_JPRB, 0.20461E+01_JPRB, 0.18109E+01_JPRB, 0.15970E+01_JPRB, 0.15083E+01_JPRB, 0.14980E+01_JPRB ,&
&  0.14582E+01_JPRB, 0.14779E+01_JPRB, 0.14801E+01_JPRB, 0.14638E+01_JPRB, 0.14794E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.14253E+01_JPRB, 0.13520E+01_JPRB, 0.13152E+01_JPRB, 0.13334E+01_JPRB, 0.14642E+01_JPRB, 0.16192E+01_JPRB, 0.17033E+01_JPRB ,&
&  0.17895E+01_JPRB, 0.18489E+01_JPRB, 0.19221E+01_JPRB, 0.19405E+01_JPRB, 0.21520E+01_JPRB, 0.25283E+01_JPRB, 0.28521E+01_JPRB ,&
&  0.31197E+01_JPRB, 0.34059E+01_JPRB, 0.37535E+01_JPRB, 0.43284E+01_JPRB, 0.51549E+01_JPRB, 0.62041E+01_JPRB, 0.36260E+01_JPRB ,&
&  0.29426E+01_JPRB, 0.26448E+01_JPRB, 0.22817E+01_JPRB, 0.24735E+01_JPRB, 0.24157E+01_JPRB, 0.24104E+01_JPRB, 0.23158E+01_JPRB ,&
&  0.24196E+01_JPRB, 0.23604E+01_JPRB, 0.22937E+01_JPRB, 0.23255E+01_JPRB, 0.22069E+01_JPRB, 0.23432E+01_JPRB, 0.24771E+01_JPRB ,&
&  0.24260E+01_JPRB, 0.24087E+01_JPRB, 0.25009E+01_JPRB, 0.25996E+01_JPRB, 0.27301E+01_JPRB, 0.28598E+01_JPRB, 0.30276E+01_JPRB ,&
&  0.32085E+01_JPRB, 0.34141E+01_JPRB, 0.37018E+01_JPRB, 0.35208E+01_JPRB, 0.34858E+01_JPRB, 0.35595E+01_JPRB, 0.37980E+01_JPRB ,&
&  0.36295E+01_JPRB, 0.32489E+01_JPRB, 0.33357E+01_JPRB, 0.35279E+01_JPRB, 0.35410E+01_JPRB, 0.35622E+01_JPRB, 0.43714E+01_JPRB ,&
&  0.60964E+01_JPRB, 0.78306E+01_JPRB, 0.97651E+01_JPRB, 0.89465E+01_JPRB, 0.90720E+01_JPRB, 0.97861E+01_JPRB, 0.95105E+01_JPRB ,&
&  0.88341E+01_JPRB, 0.75761E+01_JPRB, 0.72696E+01_JPRB, 0.76794E+01_JPRB, 0.75600E+01_JPRB, 0.75179E+01_JPRB, 0.76964E+01_JPRB ,&
&  0.94636E+01_JPRB, 0.11801E+02_JPRB, 0.99221E+01_JPRB, 0.62612E+01_JPRB, 0.44370E+01_JPRB, 0.31524E+01_JPRB, 0.36409E+01_JPRB ,&
&  0.56625E+01_JPRB, 0.55447E+01_JPRB, 0.52111E+01_JPRB, 0.57875E+01_JPRB, 0.45799E+01_JPRB, 0.38025E+01_JPRB, 0.29165E+01_JPRB ,&
&  0.22598E+01_JPRB, 0.17836E+01_JPRB, 0.16140E+01_JPRB, 0.14632E+01_JPRB, 0.14311E+01_JPRB, 0.14451E+01_JPRB, 0.14848E+01_JPRB ,&
&  0.15298E+01_JPRB, 0.15373E+01_JPRB, 0.15167E+01_JPRB, 0.14869E+01_JPRB, 0.14797E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.14207E+01_JPRB, 0.13712E+01_JPRB, 0.13429E+01_JPRB, 0.14024E+01_JPRB, 0.15567E+01_JPRB, 0.16967E+01_JPRB, 0.17586E+01_JPRB ,&
&  0.18146E+01_JPRB, 0.18890E+01_JPRB, 0.20616E+01_JPRB, 0.22438E+01_JPRB, 0.26183E+01_JPRB, 0.30241E+01_JPRB, 0.34006E+01_JPRB ,&
&  0.36901E+01_JPRB, 0.39238E+01_JPRB, 0.42466E+01_JPRB, 0.47352E+01_JPRB, 0.52819E+01_JPRB, 0.57593E+01_JPRB, 0.43577E+01_JPRB ,&
&  0.26185E+01_JPRB, 0.33517E+01_JPRB, 0.25589E+01_JPRB, 0.28439E+01_JPRB, 0.29738E+01_JPRB, 0.32545E+01_JPRB, 0.30080E+01_JPRB ,&
&  0.29924E+01_JPRB, 0.28443E+01_JPRB, 0.28334E+01_JPRB, 0.29086E+01_JPRB, 0.27633E+01_JPRB, 0.29309E+01_JPRB, 0.30888E+01_JPRB ,&
&  0.31758E+01_JPRB, 0.32804E+01_JPRB, 0.33211E+01_JPRB, 0.33131E+01_JPRB, 0.33324E+01_JPRB, 0.33736E+01_JPRB, 0.34521E+01_JPRB ,&
&  0.36301E+01_JPRB, 0.38592E+01_JPRB, 0.39161E+01_JPRB, 0.38545E+01_JPRB, 0.36180E+01_JPRB, 0.36662E+01_JPRB, 0.37653E+01_JPRB ,&
&  0.34773E+01_JPRB, 0.31033E+01_JPRB, 0.32568E+01_JPRB, 0.34270E+01_JPRB, 0.37131E+01_JPRB, 0.47891E+01_JPRB, 0.59432E+01_JPRB ,&
&  0.77277E+01_JPRB, 0.85944E+01_JPRB, 0.10945E+02_JPRB, 0.82835E+01_JPRB, 0.79738E+01_JPRB, 0.84087E+01_JPRB, 0.75665E+01_JPRB ,&
&  0.67650E+01_JPRB, 0.69358E+01_JPRB, 0.69829E+01_JPRB, 0.76884E+01_JPRB, 0.77515E+01_JPRB, 0.71127E+01_JPRB, 0.62775E+01_JPRB ,&
&  0.66429E+01_JPRB, 0.71424E+01_JPRB, 0.61231E+01_JPRB, 0.41195E+01_JPRB, 0.35761E+01_JPRB, 0.34824E+01_JPRB, 0.46899E+01_JPRB ,&
&  0.54462E+01_JPRB, 0.52644E+01_JPRB, 0.51089E+01_JPRB, 0.47389E+01_JPRB, 0.32357E+01_JPRB, 0.24945E+01_JPRB, 0.18183E+01_JPRB ,&
&  0.13798E+01_JPRB, 0.12116E+01_JPRB, 0.13529E+01_JPRB, 0.13209E+01_JPRB, 0.13570E+01_JPRB, 0.14158E+01_JPRB, 0.14345E+01_JPRB ,&
&  0.14731E+01_JPRB, 0.14788E+01_JPRB, 0.14872E+01_JPRB, 0.14702E+01_JPRB, 0.14583E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.13115E+01_JPRB, 0.12958E+01_JPRB, 0.12903E+01_JPRB, 0.14056E+01_JPRB, 0.16030E+01_JPRB, 0.17746E+01_JPRB, 0.18079E+01_JPRB ,&
&  0.18539E+01_JPRB, 0.17939E+01_JPRB, 0.20623E+01_JPRB, 0.23236E+01_JPRB, 0.26882E+01_JPRB, 0.31058E+01_JPRB, 0.35722E+01_JPRB ,&
&  0.40291E+01_JPRB, 0.44490E+01_JPRB, 0.48128E+01_JPRB, 0.52324E+01_JPRB, 0.57875E+01_JPRB, 0.63738E+01_JPRB, 0.57037E+01_JPRB ,&
&  0.40943E+01_JPRB, 0.29269E+01_JPRB, 0.28558E+01_JPRB, 0.29570E+01_JPRB, 0.30695E+01_JPRB, 0.33828E+01_JPRB, 0.35242E+01_JPRB ,&
&  0.35828E+01_JPRB, 0.31959E+01_JPRB, 0.31030E+01_JPRB, 0.32176E+01_JPRB, 0.29972E+01_JPRB, 0.31867E+01_JPRB, 0.34135E+01_JPRB ,&
&  0.35616E+01_JPRB, 0.36853E+01_JPRB, 0.37212E+01_JPRB, 0.37105E+01_JPRB, 0.37282E+01_JPRB, 0.38061E+01_JPRB, 0.39301E+01_JPRB ,&
&  0.41624E+01_JPRB, 0.44471E+01_JPRB, 0.46020E+01_JPRB, 0.43652E+01_JPRB, 0.42435E+01_JPRB, 0.43140E+01_JPRB, 0.42410E+01_JPRB ,&
&  0.39977E+01_JPRB, 0.39652E+01_JPRB, 0.42763E+01_JPRB, 0.48733E+01_JPRB, 0.52576E+01_JPRB, 0.65405E+01_JPRB, 0.73371E+01_JPRB ,&
&  0.82583E+01_JPRB, 0.87972E+01_JPRB, 0.98258E+01_JPRB, 0.71427E+01_JPRB, 0.71600E+01_JPRB, 0.71655E+01_JPRB, 0.67068E+01_JPRB ,&
&  0.64421E+01_JPRB, 0.67548E+01_JPRB, 0.70469E+01_JPRB, 0.72178E+01_JPRB, 0.72291E+01_JPRB, 0.61257E+01_JPRB, 0.57966E+01_JPRB ,&
&  0.54380E+01_JPRB, 0.58729E+01_JPRB, 0.46643E+01_JPRB, 0.40536E+01_JPRB, 0.35356E+01_JPRB, 0.37581E+01_JPRB, 0.41412E+01_JPRB ,&
&  0.46605E+01_JPRB, 0.44273E+01_JPRB, 0.38702E+01_JPRB, 0.29452E+01_JPRB, 0.17847E+01_JPRB, 0.13130E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.11427E+01_JPRB, 0.12096E+01_JPRB, 0.12381E+01_JPRB, 0.12666E+01_JPRB, 0.13086E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.12950E+01_JPRB, 0.12801E+01_JPRB, 0.13272E+01_JPRB, 0.13443E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.11217E+01_JPRB, 0.10837E+01_JPRB, 0.10802E+01_JPRB, 0.11995E+01_JPRB, 0.13459E+01_JPRB, 0.14353E+01_JPRB, 0.14429E+01_JPRB ,&
&  0.14609E+01_JPRB, 0.14997E+01_JPRB, 0.16948E+01_JPRB, 0.19472E+01_JPRB, 0.22387E+01_JPRB, 0.26190E+01_JPRB, 0.30259E+01_JPRB ,&
&  0.34484E+01_JPRB, 0.37699E+01_JPRB, 0.40023E+01_JPRB, 0.42659E+01_JPRB, 0.46224E+01_JPRB, 0.50276E+01_JPRB, 0.51951E+01_JPRB ,&
&  0.41278E+01_JPRB, 0.24606E+01_JPRB, 0.26950E+01_JPRB, 0.29515E+01_JPRB, 0.32705E+01_JPRB, 0.35304E+01_JPRB, 0.35679E+01_JPRB ,&
&  0.31173E+01_JPRB, 0.29782E+01_JPRB, 0.30796E+01_JPRB, 0.31804E+01_JPRB, 0.29486E+01_JPRB, 0.31495E+01_JPRB, 0.33814E+01_JPRB ,&
&  0.36252E+01_JPRB, 0.38091E+01_JPRB, 0.39192E+01_JPRB, 0.40166E+01_JPRB, 0.41165E+01_JPRB, 0.42669E+01_JPRB, 0.45190E+01_JPRB ,&
&  0.48560E+01_JPRB, 0.51630E+01_JPRB, 0.54109E+01_JPRB, 0.54188E+01_JPRB, 0.53916E+01_JPRB, 0.55408E+01_JPRB, 0.55185E+01_JPRB ,&
&  0.55365E+01_JPRB, 0.55590E+01_JPRB, 0.61959E+01_JPRB, 0.70841E+01_JPRB, 0.76172E+01_JPRB, 0.70059E+01_JPRB, 0.76925E+01_JPRB ,&
&  0.79401E+01_JPRB, 0.78607E+01_JPRB, 0.74239E+01_JPRB, 0.77502E+01_JPRB, 0.55722E+01_JPRB, 0.58951E+01_JPRB, 0.59034E+01_JPRB ,&
&  0.65783E+01_JPRB, 0.66524E+01_JPRB, 0.67378E+01_JPRB, 0.65034E+01_JPRB, 0.66225E+01_JPRB, 0.55486E+01_JPRB, 0.50078E+01_JPRB ,&
&  0.48922E+01_JPRB, 0.49035E+01_JPRB, 0.44283E+01_JPRB, 0.42252E+01_JPRB, 0.38245E+01_JPRB, 0.39726E+01_JPRB, 0.36583E+01_JPRB ,&
&  0.37467E+01_JPRB, 0.30202E+01_JPRB, 0.25079E+01_JPRB, 0.18795E+01_JPRB, 0.10790E+01_JPRB, 0.92765E+00_JPRB, 0.96059E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.10572E+01_JPRB, 0.10652E+01_JPRB, 0.11117E+01_JPRB, 0.11449E+01_JPRB, 0.11313E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11370E+01_JPRB, 0.11407E+01_JPRB, 0.11450E+01_JPRB, 0.11390E+01_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.88748E+00_JPRB, 0.88743E+00_JPRB, 0.90569E+00_JPRB, 0.96197E+00_JPRB, 0.10740E+01_JPRB, 0.11559E+01_JPRB, 0.12105E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.12907E+01_JPRB, 0.14050E+01_JPRB, 0.14938E+01_JPRB, 0.16602E+01_JPRB, 0.18952E+01_JPRB, 0.21304E+01_JPRB ,&
&  0.24346E+01_JPRB, 0.27024E+01_JPRB, 0.29025E+01_JPRB, 0.31701E+01_JPRB, 0.34196E+01_JPRB, 0.37771E+01_JPRB, 0.40095E+01_JPRB ,&
&  0.39632E+01_JPRB, 0.32264E+01_JPRB, 0.29359E+01_JPRB, 0.23935E+01_JPRB, 0.23160E+01_JPRB, 0.28047E+01_JPRB, 0.26223E+01_JPRB ,&
&  0.25922E+01_JPRB, 0.26295E+01_JPRB, 0.26001E+01_JPRB, 0.26856E+01_JPRB, 0.28297E+01_JPRB, 0.30164E+01_JPRB, 0.32857E+01_JPRB ,&
&  0.35858E+01_JPRB, 0.38571E+01_JPRB, 0.41008E+01_JPRB, 0.43255E+01_JPRB, 0.44984E+01_JPRB, 0.47214E+01_JPRB, 0.50376E+01_JPRB ,&
&  0.54055E+01_JPRB, 0.57118E+01_JPRB, 0.60703E+01_JPRB, 0.65527E+01_JPRB, 0.65642E+01_JPRB, 0.68206E+01_JPRB, 0.69598E+01_JPRB ,&
&  0.70613E+01_JPRB, 0.70204E+01_JPRB, 0.71181E+01_JPRB, 0.74260E+01_JPRB, 0.73979E+01_JPRB, 0.66110E+01_JPRB, 0.66163E+01_JPRB ,&
&  0.68960E+01_JPRB, 0.65156E+01_JPRB, 0.56988E+01_JPRB, 0.64358E+01_JPRB, 0.44843E+01_JPRB, 0.52268E+01_JPRB, 0.59282E+01_JPRB ,&
&  0.59135E+01_JPRB, 0.58243E+01_JPRB, 0.55655E+01_JPRB, 0.54858E+01_JPRB, 0.54628E+01_JPRB, 0.50375E+01_JPRB, 0.47919E+01_JPRB ,&
&  0.45311E+01_JPRB, 0.36800E+01_JPRB, 0.33951E+01_JPRB, 0.36464E+01_JPRB, 0.39047E+01_JPRB, 0.35109E+01_JPRB, 0.27496E+01_JPRB ,&
&  0.22524E+01_JPRB, 0.23913E+01_JPRB, 0.22431E+01_JPRB, 0.23774E+01_JPRB, 0.14835E+01_JPRB, 0.90143E+00_JPRB, 0.89346E+00_JPRB ,&
&  0.91482E+00_JPRB, 0.93840E+00_JPRB, 0.92851E+00_JPRB, 0.97881E+00_JPRB, 0.99287E+00_JPRB, 0.97074E+00_JPRB, 0.97279E+00_JPRB ,&
&  0.99768E+00_JPRB, 0.99461E+00_JPRB, 0.98828E+00_JPRB, 0.96006E+00_JPRB, 0.93560E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.84033E+00_JPRB, 0.82545E+00_JPRB, 0.83066E+00_JPRB, 0.83092E+00_JPRB, 0.84816E+00_JPRB, 0.85651E+00_JPRB, 0.88533E+00_JPRB ,&
&  0.93103E+00_JPRB, 0.98081E+00_JPRB, 0.98820E+00_JPRB, 0.97902E+00_JPRB, 0.99435E+00_JPRB, 0.10301E+01_JPRB, 0.11279E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.15498E+01_JPRB, 0.16861E+01_JPRB, 0.18228E+01_JPRB, 0.21005E+01_JPRB, 0.26262E+01_JPRB, 0.31334E+01_JPRB ,&
&  0.35183E+01_JPRB, 0.37515E+01_JPRB, 0.36494E+01_JPRB, 0.37223E+01_JPRB, 0.29523E+01_JPRB, 0.25890E+01_JPRB, 0.28887E+01_JPRB ,&
&  0.31761E+01_JPRB, 0.30539E+01_JPRB, 0.25112E+01_JPRB, 0.24691E+01_JPRB, 0.25126E+01_JPRB, 0.26743E+01_JPRB, 0.30395E+01_JPRB ,&
&  0.33991E+01_JPRB, 0.37872E+01_JPRB, 0.41102E+01_JPRB, 0.43900E+01_JPRB, 0.46280E+01_JPRB, 0.49023E+01_JPRB, 0.51726E+01_JPRB ,&
&  0.54630E+01_JPRB, 0.56941E+01_JPRB, 0.59409E+01_JPRB, 0.64367E+01_JPRB, 0.69261E+01_JPRB, 0.73526E+01_JPRB, 0.75919E+01_JPRB ,&
&  0.76569E+01_JPRB, 0.72509E+01_JPRB, 0.73067E+01_JPRB, 0.73860E+01_JPRB, 0.69617E+01_JPRB, 0.63566E+01_JPRB, 0.61508E+01_JPRB ,&
&  0.59442E+01_JPRB, 0.55713E+01_JPRB, 0.42926E+01_JPRB, 0.51994E+01_JPRB, 0.59302E+01_JPRB, 0.54273E+01_JPRB, 0.52322E+01_JPRB ,&
&  0.49352E+01_JPRB, 0.47368E+01_JPRB, 0.44638E+01_JPRB, 0.45318E+01_JPRB, 0.48646E+01_JPRB, 0.62400E+01_JPRB, 0.49163E+01_JPRB ,&
&  0.41334E+01_JPRB, 0.26463E+01_JPRB, 0.22739E+01_JPRB, 0.25872E+01_JPRB, 0.29095E+01_JPRB, 0.26619E+01_JPRB, 0.18672E+01_JPRB ,&
&  0.16663E+01_JPRB, 0.19721E+01_JPRB, 0.21237E+01_JPRB, 0.26912E+01_JPRB, 0.27482E+01_JPRB, 0.96660E+00_JPRB, 0.82002E+00_JPRB ,&
&  0.81425E+00_JPRB, 0.81388E+00_JPRB, 0.80544E+00_JPRB, 0.82953E+00_JPRB, 0.83010E+00_JPRB, 0.83632E+00_JPRB, 0.86347E+00_JPRB ,&
&  0.88294E+00_JPRB, 0.88342E+00_JPRB, 0.87613E+00_JPRB, 0.84541E+00_JPRB, 0.84848E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.90724E+00_JPRB, 0.90678E+00_JPRB, 0.91508E+00_JPRB, 0.90691E+00_JPRB, 0.89819E+00_JPRB, 0.89887E+00_JPRB, 0.90572E+00_JPRB ,&
&  0.91011E+00_JPRB, 0.89801E+00_JPRB, 0.88201E+00_JPRB, 0.87287E+00_JPRB, 0.86700E+00_JPRB, 0.82667E+00_JPRB, 0.77099E+00_JPRB ,&
&  0.79097E+00_JPRB, 0.82109E+00_JPRB, 0.84148E+00_JPRB, 0.88532E+00_JPRB, 0.10013E+01_JPRB, 0.11850E+01_JPRB, 0.14525E+01_JPRB ,&
&  0.19592E+01_JPRB, 0.25143E+01_JPRB, 0.29012E+01_JPRB, 0.29809E+01_JPRB, 0.33270E+01_JPRB, 0.27484E+01_JPRB, 0.29545E+01_JPRB ,&
&  0.34969E+01_JPRB, 0.28487E+01_JPRB, 0.24594E+01_JPRB, 0.22492E+01_JPRB, 0.22643E+01_JPRB, 0.24108E+01_JPRB, 0.26582E+01_JPRB ,&
&  0.30256E+01_JPRB, 0.34158E+01_JPRB, 0.37544E+01_JPRB, 0.40946E+01_JPRB, 0.44237E+01_JPRB, 0.47597E+01_JPRB, 0.50454E+01_JPRB ,&
&  0.52382E+01_JPRB, 0.54703E+01_JPRB, 0.57779E+01_JPRB, 0.59999E+01_JPRB, 0.66393E+01_JPRB, 0.70426E+01_JPRB, 0.72413E+01_JPRB ,&
&  0.70956E+01_JPRB, 0.67780E+01_JPRB, 0.68077E+01_JPRB, 0.64784E+01_JPRB, 0.61071E+01_JPRB, 0.57548E+01_JPRB, 0.53680E+01_JPRB ,&
&  0.50011E+01_JPRB, 0.44675E+01_JPRB, 0.35932E+01_JPRB, 0.40603E+01_JPRB, 0.43428E+01_JPRB, 0.39962E+01_JPRB, 0.46372E+01_JPRB ,&
&  0.43047E+01_JPRB, 0.41117E+01_JPRB, 0.39638E+01_JPRB, 0.38095E+01_JPRB, 0.36901E+01_JPRB, 0.43331E+01_JPRB, 0.43496E+01_JPRB ,&
&  0.29369E+01_JPRB, 0.18791E+01_JPRB, 0.15377E+01_JPRB, 0.16048E+01_JPRB, 0.18553E+01_JPRB, 0.19761E+01_JPRB, 0.18011E+01_JPRB ,&
&  0.17912E+01_JPRB, 0.17215E+01_JPRB, 0.19592E+01_JPRB, 0.25483E+01_JPRB, 0.16998E+01_JPRB, 0.95908E+00_JPRB, 0.75342E+00_JPRB ,&
&  0.70812E+00_JPRB, 0.70340E+00_JPRB, 0.71177E+00_JPRB, 0.74448E+00_JPRB, 0.77314E+00_JPRB, 0.80293E+00_JPRB, 0.82912E+00_JPRB ,&
&  0.84298E+00_JPRB, 0.86421E+00_JPRB, 0.88083E+00_JPRB, 0.90001E+00_JPRB, 0.91085E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.10181E+01_JPRB, 0.10397E+01_JPRB, 0.10358E+01_JPRB, 0.10472E+01_JPRB, 0.10603E+01_JPRB, 0.10835E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.10817E+01_JPRB, 0.10582E+01_JPRB, 0.10641E+01_JPRB, 0.10670E+01_JPRB, 0.10687E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.10864E+01_JPRB, 0.10925E+01_JPRB, 0.10876E+01_JPRB, 0.10643E+01_JPRB, 0.10275E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.12168E+01_JPRB, 0.14039E+01_JPRB, 0.15402E+01_JPRB, 0.18660E+01_JPRB, 0.19639E+01_JPRB, 0.22370E+01_JPRB ,&
&  0.29039E+01_JPRB, 0.24930E+01_JPRB, 0.24172E+01_JPRB, 0.20319E+01_JPRB, 0.20050E+01_JPRB, 0.19696E+01_JPRB, 0.19788E+01_JPRB ,&
&  0.20656E+01_JPRB, 0.22374E+01_JPRB, 0.24332E+01_JPRB, 0.26519E+01_JPRB, 0.28363E+01_JPRB, 0.29138E+01_JPRB, 0.29565E+01_JPRB ,&
&  0.30058E+01_JPRB, 0.33075E+01_JPRB, 0.41121E+01_JPRB, 0.47971E+01_JPRB, 0.55831E+01_JPRB, 0.59483E+01_JPRB, 0.60045E+01_JPRB ,&
&  0.59311E+01_JPRB, 0.59458E+01_JPRB, 0.51538E+01_JPRB, 0.45319E+01_JPRB, 0.43794E+01_JPRB, 0.39337E+01_JPRB, 0.37930E+01_JPRB ,&
&  0.39385E+01_JPRB, 0.37402E+01_JPRB, 0.29601E+01_JPRB, 0.32097E+01_JPRB, 0.34852E+01_JPRB, 0.33845E+01_JPRB, 0.39142E+01_JPRB ,&
&  0.33764E+01_JPRB, 0.30075E+01_JPRB, 0.28488E+01_JPRB, 0.27550E+01_JPRB, 0.26252E+01_JPRB, 0.25017E+01_JPRB, 0.25548E+01_JPRB ,&
&  0.19544E+01_JPRB, 0.15712E+01_JPRB, 0.14330E+01_JPRB, 0.14765E+01_JPRB, 0.18871E+01_JPRB, 0.26394E+01_JPRB, 0.20490E+01_JPRB ,&
&  0.19330E+01_JPRB, 0.18951E+01_JPRB, 0.26096E+01_JPRB, 0.37199E+01_JPRB, 0.16791E+01_JPRB, 0.10193E+01_JPRB, 0.71129E+00_JPRB ,&
&  0.69090E+00_JPRB, 0.72434E+00_JPRB, 0.75703E+00_JPRB, 0.79934E+00_JPRB, 0.82592E+00_JPRB, 0.83976E+00_JPRB, 0.86065E+00_JPRB ,&
&  0.90351E+00_JPRB, 0.94293E+00_JPRB, 0.99708E+00_JPRB, 0.10076E+01_JPRB, 0.99621E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.10904E+01_JPRB, 0.11215E+01_JPRB, 0.11305E+01_JPRB, 0.11568E+01_JPRB, 0.11913E+01_JPRB, 0.12091E+01_JPRB, 0.12181E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12118E+01_JPRB, 0.12106E+01_JPRB, 0.12306E+01_JPRB, 0.12375E+01_JPRB, 0.12374E+01_JPRB, 0.12405E+01_JPRB ,&
&  0.12673E+01_JPRB, 0.12877E+01_JPRB, 0.13339E+01_JPRB, 0.13353E+01_JPRB, 0.13272E+01_JPRB, 0.13324E+01_JPRB, 0.13660E+01_JPRB ,&
&  0.13966E+01_JPRB, 0.14224E+01_JPRB, 0.14646E+01_JPRB, 0.15040E+01_JPRB, 0.16534E+01_JPRB, 0.18454E+01_JPRB, 0.21835E+01_JPRB ,&
&  0.23801E+01_JPRB, 0.25602E+01_JPRB, 0.21933E+01_JPRB, 0.18542E+01_JPRB, 0.18285E+01_JPRB, 0.18258E+01_JPRB, 0.17275E+01_JPRB ,&
&  0.16306E+01_JPRB, 0.15923E+01_JPRB, 0.15636E+01_JPRB, 0.15604E+01_JPRB, 0.15760E+01_JPRB, 0.15350E+01_JPRB, 0.15156E+01_JPRB ,&
&  0.14543E+01_JPRB, 0.14955E+01_JPRB, 0.19380E+01_JPRB, 0.25399E+01_JPRB, 0.31595E+01_JPRB, 0.36684E+01_JPRB, 0.33012E+01_JPRB ,&
&  0.31531E+01_JPRB, 0.33794E+01_JPRB, 0.31415E+01_JPRB, 0.29138E+01_JPRB, 0.26351E+01_JPRB, 0.23753E+01_JPRB, 0.22546E+01_JPRB ,&
&  0.24954E+01_JPRB, 0.27226E+01_JPRB, 0.25375E+01_JPRB, 0.28704E+01_JPRB, 0.30039E+01_JPRB, 0.29800E+01_JPRB, 0.26621E+01_JPRB ,&
&  0.22661E+01_JPRB, 0.20875E+01_JPRB, 0.20428E+01_JPRB, 0.20428E+01_JPRB, 0.20656E+01_JPRB, 0.19351E+01_JPRB, 0.17263E+01_JPRB ,&
&  0.15509E+01_JPRB, 0.14045E+01_JPRB, 0.13709E+01_JPRB, 0.13434E+01_JPRB, 0.18537E+01_JPRB, 0.23774E+01_JPRB, 0.30443E+01_JPRB ,&
&  0.21262E+01_JPRB, 0.19698E+01_JPRB, 0.17058E+01_JPRB, 0.26770E+01_JPRB, 0.13904E+01_JPRB, 0.75847E+00_JPRB, 0.69923E+00_JPRB ,&
&  0.77357E+00_JPRB, 0.82171E+00_JPRB, 0.84867E+00_JPRB, 0.88712E+00_JPRB, 0.90813E+00_JPRB, 0.91493E+00_JPRB, 0.93475E+00_JPRB ,&
&  0.98484E+00_JPRB, 0.10056E+01_JPRB, 0.10263E+01_JPRB, 0.10347E+01_JPRB, 0.10607E+01_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.11297E+01_JPRB, 0.11734E+01_JPRB, 0.12020E+01_JPRB, 0.12312E+01_JPRB, 0.12683E+01_JPRB, 0.12944E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.13251E+01_JPRB, 0.13257E+01_JPRB, 0.13404E+01_JPRB, 0.13601E+01_JPRB, 0.13689E+01_JPRB, 0.13634E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.13979E+01_JPRB, 0.14383E+01_JPRB, 0.14869E+01_JPRB, 0.14837E+01_JPRB, 0.14842E+01_JPRB, 0.15164E+01_JPRB, 0.15608E+01_JPRB ,&
&  0.15647E+01_JPRB, 0.15520E+01_JPRB, 0.15650E+01_JPRB, 0.15899E+01_JPRB, 0.16364E+01_JPRB, 0.16664E+01_JPRB, 0.18094E+01_JPRB ,&
&  0.19689E+01_JPRB, 0.27244E+01_JPRB, 0.24314E+01_JPRB, 0.20795E+01_JPRB, 0.20469E+01_JPRB, 0.18731E+01_JPRB, 0.18382E+01_JPRB ,&
&  0.18186E+01_JPRB, 0.17386E+01_JPRB, 0.16940E+01_JPRB, 0.16489E+01_JPRB, 0.16512E+01_JPRB, 0.16706E+01_JPRB, 0.16316E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.16889E+01_JPRB, 0.18364E+01_JPRB, 0.20304E+01_JPRB, 0.22213E+01_JPRB, 0.22881E+01_JPRB, 0.21140E+01_JPRB ,&
&  0.19943E+01_JPRB, 0.18003E+01_JPRB, 0.15074E+01_JPRB, 0.15427E+01_JPRB, 0.15080E+01_JPRB, 0.12536E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.15930E+01_JPRB, 0.18585E+01_JPRB, 0.23552E+01_JPRB, 0.24387E+01_JPRB, 0.23957E+01_JPRB, 0.24526E+01_JPRB, 0.20505E+01_JPRB ,&
&  0.17861E+01_JPRB, 0.16409E+01_JPRB, 0.16395E+01_JPRB, 0.15979E+01_JPRB, 0.16292E+01_JPRB, 0.15995E+01_JPRB, 0.14505E+01_JPRB ,&
&  0.13301E+01_JPRB, 0.13119E+01_JPRB, 0.13514E+01_JPRB, 0.14645E+01_JPRB, 0.17140E+01_JPRB, 0.32833E+01_JPRB, 0.48998E+01_JPRB ,&
&  0.25095E+01_JPRB, 0.17780E+01_JPRB, 0.10110E+01_JPRB, 0.12577E+01_JPRB, 0.92227E+00_JPRB, 0.72811E+00_JPRB, 0.75633E+00_JPRB ,&
&  0.88231E+00_JPRB, 0.94331E+00_JPRB, 0.96006E+00_JPRB, 0.96141E+00_JPRB, 0.94742E+00_JPRB, 0.93694E+00_JPRB, 0.93621E+00_JPRB ,&
&  0.96505E+00_JPRB, 0.99440E+00_JPRB, 0.10278E+01_JPRB, 0.10529E+01_JPRB, 0.10860E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.11432E+01_JPRB, 0.11870E+01_JPRB, 0.12237E+01_JPRB, 0.12658E+01_JPRB, 0.13112E+01_JPRB, 0.13534E+01_JPRB, 0.13912E+01_JPRB ,&
&  0.14244E+01_JPRB, 0.14517E+01_JPRB, 0.14781E+01_JPRB, 0.15040E+01_JPRB, 0.15324E+01_JPRB, 0.15545E+01_JPRB, 0.15834E+01_JPRB ,&
&  0.16269E+01_JPRB, 0.16671E+01_JPRB, 0.17172E+01_JPRB, 0.17274E+01_JPRB, 0.17379E+01_JPRB, 0.17903E+01_JPRB, 0.18371E+01_JPRB ,&
&  0.18497E+01_JPRB, 0.18736E+01_JPRB, 0.19133E+01_JPRB, 0.19699E+01_JPRB, 0.20508E+01_JPRB, 0.22411E+01_JPRB, 0.22325E+01_JPRB ,&
&  0.25805E+01_JPRB, 0.28419E+01_JPRB, 0.26524E+01_JPRB, 0.23595E+01_JPRB, 0.22383E+01_JPRB, 0.21903E+01_JPRB, 0.22061E+01_JPRB ,&
&  0.22518E+01_JPRB, 0.22070E+01_JPRB, 0.21874E+01_JPRB, 0.21554E+01_JPRB, 0.21511E+01_JPRB, 0.21935E+01_JPRB, 0.22380E+01_JPRB ,&
&  0.23015E+01_JPRB, 0.23847E+01_JPRB, 0.25037E+01_JPRB, 0.26626E+01_JPRB, 0.28366E+01_JPRB, 0.28363E+01_JPRB, 0.27528E+01_JPRB ,&
&  0.25514E+01_JPRB, 0.23320E+01_JPRB, 0.18034E+01_JPRB, 0.16838E+01_JPRB, 0.15706E+01_JPRB, 0.13596E+01_JPRB, 0.11922E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.16332E+01_JPRB, 0.18395E+01_JPRB, 0.21492E+01_JPRB, 0.20516E+01_JPRB, 0.19236E+01_JPRB, 0.16913E+01_JPRB ,&
&  0.15004E+01_JPRB, 0.13798E+01_JPRB, 0.13440E+01_JPRB, 0.12960E+01_JPRB, 0.12961E+01_JPRB, 0.13083E+01_JPRB, 0.12094E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.12119E+01_JPRB, 0.12716E+01_JPRB, 0.14016E+01_JPRB, 0.17091E+01_JPRB, 0.19914E+01_JPRB, 0.31035E+01_JPRB ,&
&  0.22276E+01_JPRB, 0.12787E+01_JPRB, 0.12102E+01_JPRB, 0.10269E+01_JPRB, 0.88800E+00_JPRB, 0.77409E+00_JPRB, 0.83273E+00_JPRB ,&
&  0.98936E+00_JPRB, 0.11204E+01_JPRB, 0.11550E+01_JPRB, 0.11070E+01_JPRB, 0.10273E+01_JPRB, 0.98723E+00_JPRB, 0.96934E+00_JPRB ,&
&  0.99461E+00_JPRB, 0.10185E+01_JPRB, 0.10463E+01_JPRB, 0.10663E+01_JPRB, 0.10976E+01_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.11261E+01_JPRB, 0.11745E+01_JPRB, 0.12177E+01_JPRB, 0.12615E+01_JPRB, 0.13064E+01_JPRB, 0.13523E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.14389E+01_JPRB, 0.14866E+01_JPRB, 0.15399E+01_JPRB, 0.15923E+01_JPRB, 0.16443E+01_JPRB, 0.16982E+01_JPRB, 0.17474E+01_JPRB ,&
&  0.18062E+01_JPRB, 0.18537E+01_JPRB, 0.19132E+01_JPRB, 0.19409E+01_JPRB, 0.19766E+01_JPRB, 0.20315E+01_JPRB, 0.20880E+01_JPRB ,&
&  0.21128E+01_JPRB, 0.21466E+01_JPRB, 0.21870E+01_JPRB, 0.22475E+01_JPRB, 0.22853E+01_JPRB, 0.24625E+01_JPRB, 0.23437E+01_JPRB ,&
&  0.28863E+01_JPRB, 0.31571E+01_JPRB, 0.29706E+01_JPRB, 0.26957E+01_JPRB, 0.25615E+01_JPRB, 0.25822E+01_JPRB, 0.26239E+01_JPRB ,&
&  0.26927E+01_JPRB, 0.28124E+01_JPRB, 0.26971E+01_JPRB, 0.26434E+01_JPRB, 0.26456E+01_JPRB, 0.27224E+01_JPRB, 0.28553E+01_JPRB ,&
&  0.29955E+01_JPRB, 0.31489E+01_JPRB, 0.33120E+01_JPRB, 0.34723E+01_JPRB, 0.36264E+01_JPRB, 0.37258E+01_JPRB, 0.37448E+01_JPRB ,&
&  0.36386E+01_JPRB, 0.34732E+01_JPRB, 0.30909E+01_JPRB, 0.30261E+01_JPRB, 0.28367E+01_JPRB, 0.23234E+01_JPRB, 0.18371E+01_JPRB ,&
&  0.17439E+01_JPRB, 0.17350E+01_JPRB, 0.16939E+01_JPRB, 0.18853E+01_JPRB, 0.18027E+01_JPRB, 0.16638E+01_JPRB, 0.14769E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.11839E+01_JPRB, 0.11656E+01_JPRB, 0.11094E+01_JPRB, 0.10616E+01_JPRB, 0.10369E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.11038E+01_JPRB, 0.11844E+01_JPRB, 0.12946E+01_JPRB, 0.14248E+01_JPRB, 0.15034E+01_JPRB, 0.24701E+01_JPRB ,&
&  0.18658E+01_JPRB, 0.14509E+01_JPRB, 0.16727E+01_JPRB, 0.81649E+00_JPRB, 0.84174E+00_JPRB, 0.89973E+00_JPRB, 0.10351E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.15074E+01_JPRB, 0.19712E+01_JPRB, 0.15861E+01_JPRB, 0.13198E+01_JPRB, 0.10618E+01_JPRB, 0.98295E+00_JPRB ,&
&  0.99491E+00_JPRB, 0.10103E+01_JPRB, 0.10331E+01_JPRB, 0.10514E+01_JPRB, 0.10823E+01_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.10903E+01_JPRB, 0.11304E+01_JPRB, 0.11768E+01_JPRB, 0.12205E+01_JPRB, 0.12572E+01_JPRB, 0.12925E+01_JPRB, 0.13327E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.13882E+01_JPRB, 0.14273E+01_JPRB, 0.14792E+01_JPRB, 0.15375E+01_JPRB, 0.16062E+01_JPRB, 0.16775E+01_JPRB ,&
&  0.17708E+01_JPRB, 0.18618E+01_JPRB, 0.19782E+01_JPRB, 0.20902E+01_JPRB, 0.21877E+01_JPRB, 0.22911E+01_JPRB, 0.24036E+01_JPRB ,&
&  0.25340E+01_JPRB, 0.26269E+01_JPRB, 0.26999E+01_JPRB, 0.28258E+01_JPRB, 0.29479E+01_JPRB, 0.31315E+01_JPRB, 0.26426E+01_JPRB ,&
&  0.32289E+01_JPRB, 0.36566E+01_JPRB, 0.33750E+01_JPRB, 0.30036E+01_JPRB, 0.28420E+01_JPRB, 0.28471E+01_JPRB, 0.28580E+01_JPRB ,&
&  0.28958E+01_JPRB, 0.28999E+01_JPRB, 0.27600E+01_JPRB, 0.29173E+01_JPRB, 0.30428E+01_JPRB, 0.31118E+01_JPRB, 0.33174E+01_JPRB ,&
&  0.35317E+01_JPRB, 0.37512E+01_JPRB, 0.39477E+01_JPRB, 0.41296E+01_JPRB, 0.42949E+01_JPRB, 0.44697E+01_JPRB, 0.45437E+01_JPRB ,&
&  0.45571E+01_JPRB, 0.45566E+01_JPRB, 0.44846E+01_JPRB, 0.40659E+01_JPRB, 0.38587E+01_JPRB, 0.35934E+01_JPRB, 0.28708E+01_JPRB ,&
&  0.22561E+01_JPRB, 0.17690E+01_JPRB, 0.16011E+01_JPRB, 0.16121E+01_JPRB, 0.14919E+01_JPRB, 0.13417E+01_JPRB, 0.12067E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.10088E+01_JPRB, 0.10152E+01_JPRB, 0.97231E+00_JPRB, 0.85904E+00_JPRB, 0.88601E+00_JPRB, 0.92205E+00_JPRB ,&
&  0.93779E+00_JPRB, 0.98627E+00_JPRB, 0.10556E+01_JPRB, 0.11236E+01_JPRB, 0.12890E+01_JPRB, 0.16391E+01_JPRB, 0.18799E+01_JPRB ,&
&  0.19231E+01_JPRB, 0.13241E+01_JPRB, 0.12026E+01_JPRB, 0.11482E+01_JPRB, 0.95351E+00_JPRB, 0.11804E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.21946E+01_JPRB, 0.22726E+01_JPRB, 0.28331E+01_JPRB, 0.43244E+01_JPRB, 0.33016E+01_JPRB, 0.22609E+01_JPRB, 0.98384E+00_JPRB ,&
&  0.99948E+00_JPRB, 0.10052E+01_JPRB, 0.10364E+01_JPRB, 0.10497E+01_JPRB, 0.10732E+01_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.91316E+00_JPRB, 0.92375E+00_JPRB, 0.96697E+00_JPRB, 0.10167E+01_JPRB, 0.10651E+01_JPRB, 0.11124E+01_JPRB, 0.11537E+01_JPRB ,&
&  0.12097E+01_JPRB, 0.12618E+01_JPRB, 0.12926E+01_JPRB, 0.13132E+01_JPRB, 0.13445E+01_JPRB, 0.13851E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.15280E+01_JPRB, 0.16218E+01_JPRB, 0.17531E+01_JPRB, 0.18639E+01_JPRB, 0.19900E+01_JPRB, 0.21532E+01_JPRB, 0.23172E+01_JPRB ,&
&  0.24959E+01_JPRB, 0.27151E+01_JPRB, 0.28931E+01_JPRB, 0.30613E+01_JPRB, 0.32490E+01_JPRB, 0.32611E+01_JPRB, 0.24270E+01_JPRB ,&
&  0.34830E+01_JPRB, 0.38502E+01_JPRB, 0.36845E+01_JPRB, 0.32794E+01_JPRB, 0.30544E+01_JPRB, 0.29866E+01_JPRB, 0.30076E+01_JPRB ,&
&  0.29968E+01_JPRB, 0.28852E+01_JPRB, 0.27409E+01_JPRB, 0.27182E+01_JPRB, 0.31512E+01_JPRB, 0.31727E+01_JPRB, 0.33972E+01_JPRB ,&
&  0.36260E+01_JPRB, 0.38488E+01_JPRB, 0.40348E+01_JPRB, 0.41947E+01_JPRB, 0.43217E+01_JPRB, 0.44763E+01_JPRB, 0.46455E+01_JPRB ,&
&  0.48017E+01_JPRB, 0.48913E+01_JPRB, 0.50382E+01_JPRB, 0.45766E+01_JPRB, 0.43494E+01_JPRB, 0.46564E+01_JPRB, 0.41138E+01_JPRB ,&
&  0.26332E+01_JPRB, 0.18004E+01_JPRB, 0.17505E+01_JPRB, 0.15944E+01_JPRB, 0.13835E+01_JPRB, 0.12147E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.93607E+00_JPRB, 0.90713E+00_JPRB, 0.86298E+00_JPRB, 0.85177E+00_JPRB, 0.87323E+00_JPRB, 0.87476E+00_JPRB ,&
&  0.92096E+00_JPRB, 0.90269E+00_JPRB, 0.91438E+00_JPRB, 0.96642E+00_JPRB, 0.11325E+01_JPRB, 0.14948E+01_JPRB, 0.21190E+01_JPRB ,&
&  0.22352E+01_JPRB, 0.12990E+01_JPRB, 0.11600E+01_JPRB, 0.10843E+01_JPRB, 0.10904E+01_JPRB, 0.12100E+01_JPRB, 0.14116E+01_JPRB ,&
&  0.17911E+01_JPRB, 0.23150E+01_JPRB, 0.21651E+01_JPRB, 0.16910E+01_JPRB, 0.20400E+01_JPRB, 0.17495E+01_JPRB, 0.87619E+00_JPRB ,&
&  0.88670E+00_JPRB, 0.86065E+00_JPRB, 0.87596E+00_JPRB, 0.89350E+00_JPRB, 0.90589E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.74088E+00_JPRB, 0.73323E+00_JPRB, 0.74856E+00_JPRB, 0.77915E+00_JPRB, 0.81332E+00_JPRB, 0.87210E+00_JPRB, 0.94362E+00_JPRB ,&
&  0.10298E+01_JPRB, 0.10975E+01_JPRB, 0.11309E+01_JPRB, 0.11527E+01_JPRB, 0.11805E+01_JPRB, 0.12166E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.13271E+01_JPRB, 0.13934E+01_JPRB, 0.14853E+01_JPRB, 0.15624E+01_JPRB, 0.16563E+01_JPRB, 0.18156E+01_JPRB, 0.20072E+01_JPRB ,&
&  0.22360E+01_JPRB, 0.24822E+01_JPRB, 0.27593E+01_JPRB, 0.30485E+01_JPRB, 0.33517E+01_JPRB, 0.35006E+01_JPRB, 0.35992E+01_JPRB ,&
&  0.26132E+01_JPRB, 0.36458E+01_JPRB, 0.37453E+01_JPRB, 0.34733E+01_JPRB, 0.32409E+01_JPRB, 0.32235E+01_JPRB, 0.31787E+01_JPRB ,&
&  0.30975E+01_JPRB, 0.28298E+01_JPRB, 0.27221E+01_JPRB, 0.26863E+01_JPRB, 0.29105E+01_JPRB, 0.30321E+01_JPRB, 0.32286E+01_JPRB ,&
&  0.34189E+01_JPRB, 0.35797E+01_JPRB, 0.37172E+01_JPRB, 0.38601E+01_JPRB, 0.40456E+01_JPRB, 0.43001E+01_JPRB, 0.46273E+01_JPRB ,&
&  0.49284E+01_JPRB, 0.51597E+01_JPRB, 0.53148E+01_JPRB, 0.49602E+01_JPRB, 0.46092E+01_JPRB, 0.51086E+01_JPRB, 0.48513E+01_JPRB ,&
&  0.30040E+01_JPRB, 0.19332E+01_JPRB, 0.19479E+01_JPRB, 0.18860E+01_JPRB, 0.15590E+01_JPRB, 0.13660E+01_JPRB, 0.12053E+01_JPRB ,&
&  0.10058E+01_JPRB, 0.88464E+00_JPRB, 0.89490E+00_JPRB, 0.88693E+00_JPRB, 0.89677E+00_JPRB, 0.89841E+00_JPRB, 0.91580E+00_JPRB ,&
&  0.96187E+00_JPRB, 0.92997E+00_JPRB, 0.92635E+00_JPRB, 0.94182E+00_JPRB, 0.10026E+01_JPRB, 0.10781E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.12212E+01_JPRB, 0.11261E+01_JPRB, 0.10923E+01_JPRB, 0.11032E+01_JPRB, 0.11029E+01_JPRB, 0.11269E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.12605E+01_JPRB, 0.12819E+01_JPRB, 0.12559E+01_JPRB, 0.11254E+01_JPRB, 0.10430E+01_JPRB, 0.79949E+00_JPRB ,&
&  0.78350E+00_JPRB, 0.75931E+00_JPRB, 0.75748E+00_JPRB, 0.75525E+00_JPRB, 0.75741E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.64507E+00_JPRB, 0.64227E+00_JPRB, 0.65583E+00_JPRB, 0.67375E+00_JPRB, 0.70599E+00_JPRB, 0.75969E+00_JPRB, 0.83483E+00_JPRB ,&
&  0.91800E+00_JPRB, 0.97160E+00_JPRB, 0.10190E+01_JPRB, 0.10246E+01_JPRB, 0.10369E+01_JPRB, 0.10723E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.12372E+01_JPRB, 0.12824E+01_JPRB, 0.13074E+01_JPRB, 0.13428E+01_JPRB, 0.14247E+01_JPRB, 0.15409E+01_JPRB ,&
&  0.16951E+01_JPRB, 0.19202E+01_JPRB, 0.22294E+01_JPRB, 0.26441E+01_JPRB, 0.30998E+01_JPRB, 0.33322E+01_JPRB, 0.34087E+01_JPRB ,&
&  0.16215E+01_JPRB, 0.27289E+01_JPRB, 0.37027E+01_JPRB, 0.36971E+01_JPRB, 0.35908E+01_JPRB, 0.35226E+01_JPRB, 0.34816E+01_JPRB ,&
&  0.30984E+01_JPRB, 0.26569E+01_JPRB, 0.24679E+01_JPRB, 0.27333E+01_JPRB, 0.26405E+01_JPRB, 0.27974E+01_JPRB, 0.29506E+01_JPRB ,&
&  0.31270E+01_JPRB, 0.32781E+01_JPRB, 0.33701E+01_JPRB, 0.34524E+01_JPRB, 0.36044E+01_JPRB, 0.38453E+01_JPRB, 0.41507E+01_JPRB ,&
&  0.45501E+01_JPRB, 0.50441E+01_JPRB, 0.53885E+01_JPRB, 0.51524E+01_JPRB, 0.50238E+01_JPRB, 0.53781E+01_JPRB, 0.66770E+01_JPRB ,&
&  0.41158E+01_JPRB, 0.23366E+01_JPRB, 0.23004E+01_JPRB, 0.22820E+01_JPRB, 0.18440E+01_JPRB, 0.16296E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.10545E+01_JPRB, 0.99001E+00_JPRB, 0.95211E+00_JPRB, 0.96385E+00_JPRB, 0.10005E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.11022E+01_JPRB, 0.10821E+01_JPRB, 0.10897E+01_JPRB, 0.10867E+01_JPRB, 0.10987E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.10816E+01_JPRB, 0.10644E+01_JPRB, 0.10890E+01_JPRB, 0.11282E+01_JPRB, 0.11360E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.11113E+01_JPRB, 0.11072E+01_JPRB, 0.10723E+01_JPRB, 0.10122E+01_JPRB, 0.94326E+00_JPRB, 0.83421E+00_JPRB ,&
&  0.77927E+00_JPRB, 0.71371E+00_JPRB, 0.66858E+00_JPRB, 0.64680E+00_JPRB, 0.65502E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.62392E+00_JPRB, 0.62005E+00_JPRB, 0.62564E+00_JPRB, 0.66026E+00_JPRB, 0.66099E+00_JPRB, 0.68867E+00_JPRB, 0.72904E+00_JPRB ,&
&  0.80975E+00_JPRB, 0.88266E+00_JPRB, 0.95411E+00_JPRB, 0.96016E+00_JPRB, 0.95660E+00_JPRB, 0.93869E+00_JPRB, 0.94883E+00_JPRB ,&
&  0.98000E+00_JPRB, 0.10259E+01_JPRB, 0.10769E+01_JPRB, 0.11088E+01_JPRB, 0.11369E+01_JPRB, 0.11681E+01_JPRB, 0.12065E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.13523E+01_JPRB, 0.15342E+01_JPRB, 0.18911E+01_JPRB, 0.24902E+01_JPRB, 0.31733E+01_JPRB, 0.37226E+01_JPRB ,&
&  0.25587E+01_JPRB, 0.98806E+00_JPRB, 0.29684E+01_JPRB, 0.38976E+01_JPRB, 0.38649E+01_JPRB, 0.37597E+01_JPRB, 0.35922E+01_JPRB ,&
&  0.31024E+01_JPRB, 0.27429E+01_JPRB, 0.23355E+01_JPRB, 0.24897E+01_JPRB, 0.22866E+01_JPRB, 0.23496E+01_JPRB, 0.25107E+01_JPRB ,&
&  0.27065E+01_JPRB, 0.28937E+01_JPRB, 0.30719E+01_JPRB, 0.31353E+01_JPRB, 0.31581E+01_JPRB, 0.32507E+01_JPRB, 0.34549E+01_JPRB ,&
&  0.38255E+01_JPRB, 0.44508E+01_JPRB, 0.52712E+01_JPRB, 0.56090E+01_JPRB, 0.65211E+01_JPRB, 0.61599E+01_JPRB, 0.66833E+01_JPRB ,&
&  0.43425E+01_JPRB, 0.28942E+01_JPRB, 0.26920E+01_JPRB, 0.26858E+01_JPRB, 0.21928E+01_JPRB, 0.18085E+01_JPRB, 0.17723E+01_JPRB ,&
&  0.14083E+01_JPRB, 0.12365E+01_JPRB, 0.11650E+01_JPRB, 0.11076E+01_JPRB, 0.10688E+01_JPRB, 0.11123E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.12493E+01_JPRB, 0.12644E+01_JPRB, 0.12538E+01_JPRB, 0.12487E+01_JPRB, 0.12361E+01_JPRB, 0.12018E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.11685E+01_JPRB, 0.11740E+01_JPRB, 0.11799E+01_JPRB, 0.11866E+01_JPRB, 0.11918E+01_JPRB, 0.11625E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11586E+01_JPRB, 0.11273E+01_JPRB, 0.11277E+01_JPRB, 0.10830E+01_JPRB, 0.10226E+01_JPRB, 0.97321E+00_JPRB ,&
&  0.89416E+00_JPRB, 0.78309E+00_JPRB, 0.69753E+00_JPRB, 0.66145E+00_JPRB, 0.63257E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.68800E+00_JPRB, 0.66403E+00_JPRB, 0.66963E+00_JPRB, 0.68902E+00_JPRB, 0.70708E+00_JPRB, 0.72207E+00_JPRB, 0.70727E+00_JPRB ,&
&  0.73452E+00_JPRB, 0.79342E+00_JPRB, 0.83391E+00_JPRB, 0.87717E+00_JPRB, 0.87103E+00_JPRB, 0.85101E+00_JPRB, 0.83667E+00_JPRB ,&
&  0.83435E+00_JPRB, 0.84256E+00_JPRB, 0.87368E+00_JPRB, 0.89184E+00_JPRB, 0.91221E+00_JPRB, 0.94034E+00_JPRB, 0.95819E+00_JPRB ,&
&  0.97319E+00_JPRB, 0.99990E+00_JPRB, 0.10609E+01_JPRB, 0.12017E+01_JPRB, 0.14887E+01_JPRB, 0.20729E+01_JPRB, 0.29820E+01_JPRB ,&
&  0.47039E+01_JPRB, 0.34037E+01_JPRB, 0.14066E+01_JPRB, 0.32382E+01_JPRB, 0.41836E+01_JPRB, 0.42863E+01_JPRB, 0.36386E+01_JPRB ,&
&  0.34098E+01_JPRB, 0.28681E+01_JPRB, 0.24256E+01_JPRB, 0.24658E+01_JPRB, 0.20804E+01_JPRB, 0.19278E+01_JPRB, 0.19363E+01_JPRB ,&
&  0.20632E+01_JPRB, 0.22896E+01_JPRB, 0.25435E+01_JPRB, 0.27223E+01_JPRB, 0.28107E+01_JPRB, 0.28407E+01_JPRB, 0.29040E+01_JPRB ,&
&  0.31375E+01_JPRB, 0.35323E+01_JPRB, 0.42538E+01_JPRB, 0.54889E+01_JPRB, 0.80030E+01_JPRB, 0.74809E+01_JPRB, 0.74486E+01_JPRB ,&
&  0.46366E+01_JPRB, 0.38568E+01_JPRB, 0.35348E+01_JPRB, 0.31921E+01_JPRB, 0.25218E+01_JPRB, 0.21985E+01_JPRB, 0.21962E+01_JPRB ,&
&  0.17370E+01_JPRB, 0.14381E+01_JPRB, 0.13248E+01_JPRB, 0.12396E+01_JPRB, 0.11955E+01_JPRB, 0.12096E+01_JPRB, 0.12679E+01_JPRB ,&
&  0.13325E+01_JPRB, 0.13331E+01_JPRB, 0.13130E+01_JPRB, 0.12987E+01_JPRB, 0.12878E+01_JPRB, 0.12722E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.12395E+01_JPRB, 0.12627E+01_JPRB, 0.12481E+01_JPRB, 0.12041E+01_JPRB, 0.11677E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.12051E+01_JPRB, 0.11743E+01_JPRB, 0.11870E+01_JPRB, 0.11524E+01_JPRB, 0.11121E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.90278E+00_JPRB, 0.79252E+00_JPRB, 0.69784E+00_JPRB, 0.68821E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.77960E+00_JPRB, 0.75914E+00_JPRB, 0.76643E+00_JPRB, 0.76140E+00_JPRB, 0.75212E+00_JPRB, 0.75445E+00_JPRB, 0.75007E+00_JPRB ,&
&  0.73885E+00_JPRB, 0.73397E+00_JPRB, 0.72698E+00_JPRB, 0.76886E+00_JPRB, 0.79277E+00_JPRB, 0.77664E+00_JPRB, 0.76001E+00_JPRB ,&
&  0.74472E+00_JPRB, 0.74526E+00_JPRB, 0.75840E+00_JPRB, 0.76854E+00_JPRB, 0.77761E+00_JPRB, 0.79607E+00_JPRB, 0.81092E+00_JPRB ,&
&  0.80805E+00_JPRB, 0.81065E+00_JPRB, 0.83011E+00_JPRB, 0.87954E+00_JPRB, 0.98525E+00_JPRB, 0.11986E+01_JPRB, 0.16313E+01_JPRB ,&
&  0.27752E+01_JPRB, 0.29241E+01_JPRB, 0.12373E+01_JPRB, 0.25275E+01_JPRB, 0.42733E+01_JPRB, 0.45772E+01_JPRB, 0.43711E+01_JPRB ,&
&  0.40288E+01_JPRB, 0.31630E+01_JPRB, 0.26934E+01_JPRB, 0.25729E+01_JPRB, 0.21572E+01_JPRB, 0.18505E+01_JPRB, 0.16611E+01_JPRB ,&
&  0.15914E+01_JPRB, 0.16505E+01_JPRB, 0.18353E+01_JPRB, 0.20633E+01_JPRB, 0.22665E+01_JPRB, 0.23798E+01_JPRB, 0.24327E+01_JPRB ,&
&  0.25160E+01_JPRB, 0.27228E+01_JPRB, 0.31853E+01_JPRB, 0.38054E+01_JPRB, 0.65065E+01_JPRB, 0.79686E+01_JPRB, 0.84084E+01_JPRB ,&
&  0.61713E+01_JPRB, 0.53087E+01_JPRB, 0.45869E+01_JPRB, 0.38718E+01_JPRB, 0.30021E+01_JPRB, 0.27500E+01_JPRB, 0.27009E+01_JPRB ,&
&  0.21969E+01_JPRB, 0.17629E+01_JPRB, 0.15575E+01_JPRB, 0.14210E+01_JPRB, 0.13290E+01_JPRB, 0.12935E+01_JPRB, 0.12863E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.13293E+01_JPRB, 0.13063E+01_JPRB, 0.12917E+01_JPRB, 0.12948E+01_JPRB, 0.12955E+01_JPRB, 0.12922E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12796E+01_JPRB, 0.12958E+01_JPRB, 0.12822E+01_JPRB, 0.12809E+01_JPRB, 0.12890E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.12025E+01_JPRB, 0.12096E+01_JPRB, 0.11879E+01_JPRB, 0.11521E+01_JPRB, 0.11994E+01_JPRB, 0.11869E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.12446E+01_JPRB, 0.11667E+01_JPRB, 0.84619E+00_JPRB, 0.79200E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.86036E+00_JPRB, 0.83133E+00_JPRB, 0.82756E+00_JPRB, 0.81958E+00_JPRB, 0.80350E+00_JPRB, 0.79192E+00_JPRB, 0.79124E+00_JPRB ,&
&  0.77110E+00_JPRB, 0.73973E+00_JPRB, 0.70861E+00_JPRB, 0.70656E+00_JPRB, 0.72221E+00_JPRB, 0.73688E+00_JPRB, 0.72259E+00_JPRB ,&
&  0.70125E+00_JPRB, 0.69713E+00_JPRB, 0.71623E+00_JPRB, 0.72668E+00_JPRB, 0.72807E+00_JPRB, 0.73512E+00_JPRB, 0.73796E+00_JPRB ,&
&  0.73597E+00_JPRB, 0.72994E+00_JPRB, 0.73114E+00_JPRB, 0.75031E+00_JPRB, 0.79504E+00_JPRB, 0.89315E+00_JPRB, 0.10603E+01_JPRB ,&
&  0.16731E+01_JPRB, 0.22190E+01_JPRB, 0.28761E+01_JPRB, 0.43437E+01_JPRB, 0.55500E+01_JPRB, 0.54177E+01_JPRB, 0.52618E+01_JPRB ,&
&  0.50530E+01_JPRB, 0.42176E+01_JPRB, 0.34051E+01_JPRB, 0.28134E+01_JPRB, 0.22928E+01_JPRB, 0.19168E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.15331E+01_JPRB, 0.14573E+01_JPRB, 0.14608E+01_JPRB, 0.15382E+01_JPRB, 0.16638E+01_JPRB, 0.17970E+01_JPRB, 0.18863E+01_JPRB ,&
&  0.19608E+01_JPRB, 0.21062E+01_JPRB, 0.24106E+01_JPRB, 0.28017E+01_JPRB, 0.43045E+01_JPRB, 0.68106E+01_JPRB, 0.81264E+01_JPRB ,&
&  0.80723E+01_JPRB, 0.71559E+01_JPRB, 0.58831E+01_JPRB, 0.47393E+01_JPRB, 0.34820E+01_JPRB, 0.35268E+01_JPRB, 0.31323E+01_JPRB ,&
&  0.26334E+01_JPRB, 0.22193E+01_JPRB, 0.19013E+01_JPRB, 0.16814E+01_JPRB, 0.15201E+01_JPRB, 0.14008E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.13137E+01_JPRB, 0.13132E+01_JPRB, 0.12936E+01_JPRB, 0.12712E+01_JPRB, 0.12797E+01_JPRB, 0.12961E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.12940E+01_JPRB, 0.12940E+01_JPRB, 0.12587E+01_JPRB, 0.12784E+01_JPRB, 0.13061E+01_JPRB, 0.12874E+01_JPRB, 0.12426E+01_JPRB ,&
&  0.11911E+01_JPRB, 0.11968E+01_JPRB, 0.11970E+01_JPRB, 0.11826E+01_JPRB, 0.11873E+01_JPRB, 0.11991E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.11768E+01_JPRB, 0.11355E+01_JPRB, 0.10418E+01_JPRB, 0.93755E+00_JPRB, 0.89239E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.94150E+00_JPRB, 0.91144E+00_JPRB, 0.89172E+00_JPRB, 0.87166E+00_JPRB, 0.86219E+00_JPRB, 0.86226E+00_JPRB, 0.85729E+00_JPRB ,&
&  0.84025E+00_JPRB, 0.80668E+00_JPRB, 0.76235E+00_JPRB, 0.72223E+00_JPRB, 0.71786E+00_JPRB, 0.71253E+00_JPRB, 0.71042E+00_JPRB ,&
&  0.69276E+00_JPRB, 0.68681E+00_JPRB, 0.69626E+00_JPRB, 0.69994E+00_JPRB, 0.70000E+00_JPRB, 0.69864E+00_JPRB, 0.69771E+00_JPRB ,&
&  0.70261E+00_JPRB, 0.70644E+00_JPRB, 0.70534E+00_JPRB, 0.70730E+00_JPRB, 0.72392E+00_JPRB, 0.78070E+00_JPRB, 0.90344E+00_JPRB ,&
&  0.12545E+01_JPRB, 0.20372E+01_JPRB, 0.22586E+01_JPRB, 0.50986E+01_JPRB, 0.59911E+01_JPRB, 0.59658E+01_JPRB, 0.53999E+01_JPRB ,&
&  0.47864E+01_JPRB, 0.46599E+01_JPRB, 0.37050E+01_JPRB, 0.28741E+01_JPRB, 0.22414E+01_JPRB, 0.18935E+01_JPRB, 0.16386E+01_JPRB ,&
&  0.14661E+01_JPRB, 0.13766E+01_JPRB, 0.13458E+01_JPRB, 0.13400E+01_JPRB, 0.13471E+01_JPRB, 0.13947E+01_JPRB, 0.14465E+01_JPRB ,&
&  0.15085E+01_JPRB, 0.16725E+01_JPRB, 0.19819E+01_JPRB, 0.22697E+01_JPRB, 0.31771E+01_JPRB, 0.51291E+01_JPRB, 0.68210E+01_JPRB ,&
&  0.75940E+01_JPRB, 0.71757E+01_JPRB, 0.63257E+01_JPRB, 0.51758E+01_JPRB, 0.41617E+01_JPRB, 0.37791E+01_JPRB, 0.33582E+01_JPRB ,&
&  0.29319E+01_JPRB, 0.26378E+01_JPRB, 0.22414E+01_JPRB, 0.19503E+01_JPRB, 0.17910E+01_JPRB, 0.16180E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.13744E+01_JPRB, 0.13313E+01_JPRB, 0.13035E+01_JPRB, 0.12727E+01_JPRB, 0.12655E+01_JPRB, 0.12864E+01_JPRB, 0.13012E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.12876E+01_JPRB, 0.12270E+01_JPRB, 0.12200E+01_JPRB, 0.12526E+01_JPRB, 0.12456E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.11788E+01_JPRB, 0.11767E+01_JPRB, 0.11633E+01_JPRB, 0.11342E+01_JPRB, 0.12083E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.11839E+01_JPRB, 0.10961E+01_JPRB, 0.10445E+01_JPRB, 0.99694E+00_JPRB, 0.97050E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.98977E+00_JPRB, 0.96892E+00_JPRB, 0.95089E+00_JPRB, 0.92655E+00_JPRB, 0.91248E+00_JPRB, 0.90678E+00_JPRB, 0.89821E+00_JPRB ,&
&  0.87678E+00_JPRB, 0.84584E+00_JPRB, 0.80795E+00_JPRB, 0.75754E+00_JPRB, 0.72567E+00_JPRB, 0.71643E+00_JPRB, 0.72131E+00_JPRB ,&
&  0.70722E+00_JPRB, 0.68712E+00_JPRB, 0.68767E+00_JPRB, 0.69300E+00_JPRB, 0.69306E+00_JPRB, 0.68496E+00_JPRB, 0.67617E+00_JPRB ,&
&  0.68171E+00_JPRB, 0.69050E+00_JPRB, 0.69617E+00_JPRB, 0.69768E+00_JPRB, 0.70219E+00_JPRB, 0.72480E+00_JPRB, 0.82078E+00_JPRB ,&
&  0.11352E+01_JPRB, 0.27830E+01_JPRB, 0.24356E+01_JPRB, 0.47849E+01_JPRB, 0.51831E+01_JPRB, 0.51508E+01_JPRB, 0.47735E+01_JPRB ,&
&  0.42731E+01_JPRB, 0.38884E+01_JPRB, 0.31658E+01_JPRB, 0.26226E+01_JPRB, 0.21352E+01_JPRB, 0.17742E+01_JPRB, 0.15411E+01_JPRB ,&
&  0.13932E+01_JPRB, 0.12869E+01_JPRB, 0.12249E+01_JPRB, 0.12043E+01_JPRB, 0.11987E+01_JPRB, 0.12027E+01_JPRB, 0.12241E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.13733E+01_JPRB, 0.16708E+01_JPRB, 0.23275E+01_JPRB, 0.28880E+01_JPRB, 0.42600E+01_JPRB, 0.62682E+01_JPRB ,&
&  0.72019E+01_JPRB, 0.70955E+01_JPRB, 0.59692E+01_JPRB, 0.49527E+01_JPRB, 0.41769E+01_JPRB, 0.36492E+01_JPRB, 0.32081E+01_JPRB ,&
&  0.29086E+01_JPRB, 0.26876E+01_JPRB, 0.24533E+01_JPRB, 0.21670E+01_JPRB, 0.19570E+01_JPRB, 0.18049E+01_JPRB, 0.16585E+01_JPRB ,&
&  0.15114E+01_JPRB, 0.13911E+01_JPRB, 0.13221E+01_JPRB, 0.12687E+01_JPRB, 0.12390E+01_JPRB, 0.12501E+01_JPRB, 0.12683E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.12501E+01_JPRB, 0.11899E+01_JPRB, 0.11737E+01_JPRB, 0.12025E+01_JPRB, 0.12022E+01_JPRB, 0.11669E+01_JPRB ,&
&  0.11632E+01_JPRB, 0.11669E+01_JPRB, 0.11599E+01_JPRB, 0.11692E+01_JPRB, 0.11236E+01_JPRB, 0.12100E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.11765E+01_JPRB, 0.11100E+01_JPRB, 0.10608E+01_JPRB, 0.10440E+01_JPRB, 0.10210E+01_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.10134E+01_JPRB, 0.99196E+00_JPRB, 0.97650E+00_JPRB, 0.96358E+00_JPRB, 0.95310E+00_JPRB, 0.93615E+00_JPRB, 0.91644E+00_JPRB ,&
&  0.89841E+00_JPRB, 0.87742E+00_JPRB, 0.84127E+00_JPRB, 0.78891E+00_JPRB, 0.74438E+00_JPRB, 0.72068E+00_JPRB, 0.72060E+00_JPRB ,&
&  0.71990E+00_JPRB, 0.70701E+00_JPRB, 0.69606E+00_JPRB, 0.69422E+00_JPRB, 0.69259E+00_JPRB, 0.68310E+00_JPRB, 0.67143E+00_JPRB ,&
&  0.66443E+00_JPRB, 0.66736E+00_JPRB, 0.67335E+00_JPRB, 0.68003E+00_JPRB, 0.68532E+00_JPRB, 0.69455E+00_JPRB, 0.75845E+00_JPRB ,&
&  0.10847E+01_JPRB, 0.11886E+01_JPRB, 0.29666E+01_JPRB, 0.40863E+01_JPRB, 0.41366E+01_JPRB, 0.39939E+01_JPRB, 0.40103E+01_JPRB ,&
&  0.36867E+01_JPRB, 0.32030E+01_JPRB, 0.26261E+01_JPRB, 0.21584E+01_JPRB, 0.18103E+01_JPRB, 0.15815E+01_JPRB, 0.14131E+01_JPRB ,&
&  0.12835E+01_JPRB, 0.11877E+01_JPRB, 0.11187E+01_JPRB, 0.10800E+01_JPRB, 0.10839E+01_JPRB, 0.10994E+01_JPRB, 0.11196E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11981E+01_JPRB, 0.13561E+01_JPRB, 0.18702E+01_JPRB, 0.30319E+01_JPRB, 0.40358E+01_JPRB, 0.55790E+01_JPRB ,&
&  0.67580E+01_JPRB, 0.62912E+01_JPRB, 0.51384E+01_JPRB, 0.43131E+01_JPRB, 0.36947E+01_JPRB, 0.32513E+01_JPRB, 0.29148E+01_JPRB ,&
&  0.26595E+01_JPRB, 0.24772E+01_JPRB, 0.23225E+01_JPRB, 0.21519E+01_JPRB, 0.20046E+01_JPRB, 0.18898E+01_JPRB, 0.17822E+01_JPRB ,&
&  0.16565E+01_JPRB, 0.15310E+01_JPRB, 0.14146E+01_JPRB, 0.13109E+01_JPRB, 0.12288E+01_JPRB, 0.12090E+01_JPRB, 0.12119E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11866E+01_JPRB, 0.11554E+01_JPRB, 0.11527E+01_JPRB, 0.11882E+01_JPRB, 0.11880E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.11518E+01_JPRB, 0.11381E+01_JPRB, 0.11173E+01_JPRB, 0.10542E+01_JPRB, 0.11499E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11126E+01_JPRB, 0.10695E+01_JPRB, 0.10584E+01_JPRB, 0.10422E+01_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.10117E+01_JPRB, 0.98859E+00_JPRB, 0.97362E+00_JPRB, 0.96318E+00_JPRB, 0.95387E+00_JPRB, 0.94439E+00_JPRB, 0.92221E+00_JPRB ,&
&  0.89861E+00_JPRB, 0.88499E+00_JPRB, 0.86515E+00_JPRB, 0.83803E+00_JPRB, 0.79612E+00_JPRB, 0.76114E+00_JPRB, 0.74897E+00_JPRB ,&
&  0.74257E+00_JPRB, 0.73243E+00_JPRB, 0.71367E+00_JPRB, 0.70074E+00_JPRB, 0.68937E+00_JPRB, 0.67654E+00_JPRB, 0.66822E+00_JPRB ,&
&  0.65544E+00_JPRB, 0.65077E+00_JPRB, 0.65466E+00_JPRB, 0.66301E+00_JPRB, 0.67051E+00_JPRB, 0.67928E+00_JPRB, 0.70020E+00_JPRB ,&
&  0.97360E+00_JPRB, 0.21970E+01_JPRB, 0.29368E+01_JPRB, 0.33605E+01_JPRB, 0.33740E+01_JPRB, 0.31161E+01_JPRB, 0.31104E+01_JPRB ,&
&  0.27723E+01_JPRB, 0.24234E+01_JPRB, 0.20713E+01_JPRB, 0.17473E+01_JPRB, 0.14879E+01_JPRB, 0.13467E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.11120E+01_JPRB, 0.10505E+01_JPRB, 0.99802E+00_JPRB, 0.97936E+00_JPRB, 0.10006E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10565E+01_JPRB, 0.10772E+01_JPRB, 0.11508E+01_JPRB, 0.14551E+01_JPRB, 0.23995E+01_JPRB, 0.29068E+01_JPRB, 0.43150E+01_JPRB ,&
&  0.51724E+01_JPRB, 0.46813E+01_JPRB, 0.39470E+01_JPRB, 0.33959E+01_JPRB, 0.29289E+01_JPRB, 0.25657E+01_JPRB, 0.23017E+01_JPRB ,&
&  0.21498E+01_JPRB, 0.20446E+01_JPRB, 0.19515E+01_JPRB, 0.18603E+01_JPRB, 0.17764E+01_JPRB, 0.17107E+01_JPRB, 0.16595E+01_JPRB ,&
&  0.16078E+01_JPRB, 0.15390E+01_JPRB, 0.14573E+01_JPRB, 0.13623E+01_JPRB, 0.12674E+01_JPRB, 0.12081E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.11330E+01_JPRB, 0.11211E+01_JPRB, 0.11285E+01_JPRB, 0.11611E+01_JPRB, 0.11741E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11222E+01_JPRB, 0.11052E+01_JPRB, 0.10831E+01_JPRB, 0.10287E+01_JPRB, 0.10139E+01_JPRB, 0.10843E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10376E+01_JPRB, 0.10247E+01_JPRB, 0.10217E+01_JPRB, 0.10377E+01_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.10177E+01_JPRB, 0.98695E+00_JPRB, 0.96416E+00_JPRB, 0.94861E+00_JPRB, 0.93897E+00_JPRB, 0.92798E+00_JPRB, 0.90971E+00_JPRB ,&
&  0.88945E+00_JPRB, 0.87010E+00_JPRB, 0.85326E+00_JPRB, 0.84082E+00_JPRB, 0.81363E+00_JPRB, 0.76636E+00_JPRB, 0.72889E+00_JPRB ,&
&  0.71599E+00_JPRB, 0.70607E+00_JPRB, 0.69672E+00_JPRB, 0.68173E+00_JPRB, 0.66997E+00_JPRB, 0.66445E+00_JPRB, 0.65885E+00_JPRB ,&
&  0.65327E+00_JPRB, 0.64807E+00_JPRB, 0.64994E+00_JPRB, 0.65538E+00_JPRB, 0.65989E+00_JPRB, 0.66384E+00_JPRB, 0.67360E+00_JPRB ,&
&  0.81090E+00_JPRB, 0.15557E+01_JPRB, 0.19154E+01_JPRB, 0.24607E+01_JPRB, 0.26451E+01_JPRB, 0.24432E+01_JPRB, 0.23132E+01_JPRB ,&
&  0.21239E+01_JPRB, 0.19054E+01_JPRB, 0.17065E+01_JPRB, 0.15204E+01_JPRB, 0.13147E+01_JPRB, 0.11895E+01_JPRB, 0.11284E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.10361E+01_JPRB, 0.98705E+00_JPRB, 0.93825E+00_JPRB, 0.90658E+00_JPRB, 0.91376E+00_JPRB, 0.93530E+00_JPRB ,&
&  0.93944E+00_JPRB, 0.94120E+00_JPRB, 0.95871E+00_JPRB, 0.10768E+01_JPRB, 0.15437E+01_JPRB, 0.20202E+01_JPRB, 0.26663E+01_JPRB ,&
&  0.27307E+01_JPRB, 0.28806E+01_JPRB, 0.26606E+01_JPRB, 0.23943E+01_JPRB, 0.21525E+01_JPRB, 0.19637E+01_JPRB, 0.18209E+01_JPRB ,&
&  0.17235E+01_JPRB, 0.16428E+01_JPRB, 0.15808E+01_JPRB, 0.15312E+01_JPRB, 0.14925E+01_JPRB, 0.14470E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.14095E+01_JPRB, 0.13828E+01_JPRB, 0.13455E+01_JPRB, 0.12993E+01_JPRB, 0.12437E+01_JPRB, 0.11964E+01_JPRB, 0.11598E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.10909E+01_JPRB, 0.10730E+01_JPRB, 0.10798E+01_JPRB, 0.10905E+01_JPRB, 0.10977E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.10609E+01_JPRB, 0.10364E+01_JPRB, 0.98950E+00_JPRB, 0.97605E+00_JPRB, 0.10179E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.98182E+00_JPRB, 0.96714E+00_JPRB, 0.96923E+00_JPRB, 0.98284E+00_JPRB, 0.10425E+01_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.10317E+01_JPRB, 0.99699E+00_JPRB, 0.95815E+00_JPRB, 0.93150E+00_JPRB, 0.91183E+00_JPRB, 0.89415E+00_JPRB, 0.87225E+00_JPRB ,&
&  0.85125E+00_JPRB, 0.83184E+00_JPRB, 0.81537E+00_JPRB, 0.80288E+00_JPRB, 0.78967E+00_JPRB, 0.75752E+00_JPRB, 0.71300E+00_JPRB ,&
&  0.68367E+00_JPRB, 0.67338E+00_JPRB, 0.66689E+00_JPRB, 0.65540E+00_JPRB, 0.64460E+00_JPRB, 0.63987E+00_JPRB, 0.63613E+00_JPRB ,&
&  0.63629E+00_JPRB, 0.63787E+00_JPRB, 0.64224E+00_JPRB, 0.64408E+00_JPRB, 0.64075E+00_JPRB, 0.64672E+00_JPRB, 0.64821E+00_JPRB ,&
&  0.74769E+00_JPRB, 0.10284E+01_JPRB, 0.13052E+01_JPRB, 0.16236E+01_JPRB, 0.18630E+01_JPRB, 0.19248E+01_JPRB, 0.18596E+01_JPRB ,&
&  0.17668E+01_JPRB, 0.16712E+01_JPRB, 0.15365E+01_JPRB, 0.14105E+01_JPRB, 0.12693E+01_JPRB, 0.11218E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.96909E+00_JPRB, 0.93726E+00_JPRB, 0.90448E+00_JPRB, 0.87691E+00_JPRB, 0.86577E+00_JPRB, 0.86566E+00_JPRB ,&
&  0.85946E+00_JPRB, 0.84953E+00_JPRB, 0.83913E+00_JPRB, 0.84608E+00_JPRB, 0.92470E+00_JPRB, 0.11551E+01_JPRB, 0.14566E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.16810E+01_JPRB, 0.17397E+01_JPRB, 0.17326E+01_JPRB, 0.16559E+01_JPRB, 0.15455E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.13453E+01_JPRB, 0.12874E+01_JPRB, 0.12565E+01_JPRB, 0.12459E+01_JPRB, 0.12416E+01_JPRB, 0.12283E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.11732E+01_JPRB, 0.11728E+01_JPRB, 0.11647E+01_JPRB, 0.11516E+01_JPRB, 0.11353E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.11105E+01_JPRB, 0.10780E+01_JPRB, 0.10454E+01_JPRB, 0.10392E+01_JPRB, 0.10348E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.10130E+01_JPRB, 0.99483E+00_JPRB, 0.99742E+00_JPRB, 0.10140E+01_JPRB, 0.97970E+00_JPRB, 0.95862E+00_JPRB ,&
&  0.92965E+00_JPRB, 0.91372E+00_JPRB, 0.92027E+00_JPRB, 0.94339E+00_JPRB, 0.10149E+01_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.88542E+00_JPRB, 0.87732E+00_JPRB, 0.86675E+00_JPRB, 0.85658E+00_JPRB, 0.84114E+00_JPRB, 0.82488E+00_JPRB, 0.80496E+00_JPRB ,&
&  0.78587E+00_JPRB, 0.76783E+00_JPRB, 0.75330E+00_JPRB, 0.74851E+00_JPRB, 0.74011E+00_JPRB, 0.72705E+00_JPRB, 0.69575E+00_JPRB ,&
&  0.66980E+00_JPRB, 0.65063E+00_JPRB, 0.63876E+00_JPRB, 0.63251E+00_JPRB, 0.62753E+00_JPRB, 0.62227E+00_JPRB, 0.61980E+00_JPRB ,&
&  0.62034E+00_JPRB, 0.61984E+00_JPRB, 0.62198E+00_JPRB, 0.62052E+00_JPRB, 0.62170E+00_JPRB, 0.61666E+00_JPRB, 0.61222E+00_JPRB ,&
&  0.66355E+00_JPRB, 0.78225E+00_JPRB, 0.94852E+00_JPRB, 0.11311E+01_JPRB, 0.13355E+01_JPRB, 0.14771E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.15978E+01_JPRB, 0.15212E+01_JPRB, 0.14077E+01_JPRB, 0.12822E+01_JPRB, 0.11844E+01_JPRB, 0.10850E+01_JPRB, 0.98682E+00_JPRB ,&
&  0.93701E+00_JPRB, 0.90795E+00_JPRB, 0.88434E+00_JPRB, 0.86591E+00_JPRB, 0.85069E+00_JPRB, 0.84078E+00_JPRB, 0.83257E+00_JPRB ,&
&  0.81745E+00_JPRB, 0.79935E+00_JPRB, 0.77860E+00_JPRB, 0.76961E+00_JPRB, 0.77511E+00_JPRB, 0.83159E+00_JPRB, 0.97710E+00_JPRB ,&
&  0.10566E+01_JPRB, 0.10989E+01_JPRB, 0.11325E+01_JPRB, 0.11864E+01_JPRB, 0.12217E+01_JPRB, 0.12167E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.10457E+01_JPRB, 0.10199E+01_JPRB, 0.10107E+01_JPRB, 0.10137E+01_JPRB, 0.10179E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.10077E+01_JPRB, 0.99472E+00_JPRB, 0.99039E+00_JPRB, 0.99179E+00_JPRB, 0.99887E+00_JPRB, 0.10051E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10658E+01_JPRB, 0.10877E+01_JPRB, 0.10673E+01_JPRB, 0.10276E+01_JPRB, 0.99722E+00_JPRB, 0.97610E+00_JPRB ,&
&  0.96259E+00_JPRB, 0.96186E+00_JPRB, 0.96591E+00_JPRB, 0.98199E+00_JPRB, 0.97847E+00_JPRB, 0.95238E+00_JPRB, 0.91911E+00_JPRB ,&
&  0.89033E+00_JPRB, 0.87242E+00_JPRB, 0.87140E+00_JPRB, 0.87766E+00_JPRB, 0.85945E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.78756E+00_JPRB, 0.78109E+00_JPRB, 0.77359E+00_JPRB, 0.76830E+00_JPRB, 0.75856E+00_JPRB, 0.74751E+00_JPRB, 0.73547E+00_JPRB ,&
&  0.72049E+00_JPRB, 0.70572E+00_JPRB, 0.69370E+00_JPRB, 0.69020E+00_JPRB, 0.68930E+00_JPRB, 0.68926E+00_JPRB, 0.68110E+00_JPRB ,&
&  0.66625E+00_JPRB, 0.65206E+00_JPRB, 0.63951E+00_JPRB, 0.63086E+00_JPRB, 0.62655E+00_JPRB, 0.62184E+00_JPRB, 0.61625E+00_JPRB ,&
&  0.61162E+00_JPRB, 0.61007E+00_JPRB, 0.60732E+00_JPRB, 0.60433E+00_JPRB, 0.60012E+00_JPRB, 0.59257E+00_JPRB, 0.59025E+00_JPRB ,&
&  0.60807E+00_JPRB, 0.61932E+00_JPRB, 0.70928E+00_JPRB, 0.78399E+00_JPRB, 0.87538E+00_JPRB, 0.10186E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.13063E+01_JPRB, 0.13097E+01_JPRB, 0.12576E+01_JPRB, 0.11682E+01_JPRB, 0.11035E+01_JPRB, 0.10570E+01_JPRB, 0.99245E+00_JPRB ,&
&  0.92882E+00_JPRB, 0.88511E+00_JPRB, 0.85093E+00_JPRB, 0.82403E+00_JPRB, 0.80418E+00_JPRB, 0.78706E+00_JPRB, 0.77072E+00_JPRB ,&
&  0.75051E+00_JPRB, 0.73058E+00_JPRB, 0.71451E+00_JPRB, 0.70724E+00_JPRB, 0.71004E+00_JPRB, 0.72032E+00_JPRB, 0.76170E+00_JPRB ,&
&  0.80461E+00_JPRB, 0.82972E+00_JPRB, 0.83348E+00_JPRB, 0.85667E+00_JPRB, 0.89526E+00_JPRB, 0.92766E+00_JPRB, 0.93480E+00_JPRB ,&
&  0.91098E+00_JPRB, 0.87410E+00_JPRB, 0.84219E+00_JPRB, 0.81857E+00_JPRB, 0.80665E+00_JPRB, 0.80902E+00_JPRB, 0.82201E+00_JPRB ,&
&  0.82832E+00_JPRB, 0.82680E+00_JPRB, 0.82706E+00_JPRB, 0.83260E+00_JPRB, 0.84143E+00_JPRB, 0.85661E+00_JPRB, 0.87760E+00_JPRB ,&
&  0.90404E+00_JPRB, 0.93533E+00_JPRB, 0.97034E+00_JPRB, 0.99684E+00_JPRB, 0.10019E+01_JPRB, 0.99404E+00_JPRB, 0.97819E+00_JPRB ,&
&  0.96059E+00_JPRB, 0.94620E+00_JPRB, 0.93115E+00_JPRB, 0.90751E+00_JPRB, 0.92808E+00_JPRB, 0.91573E+00_JPRB, 0.88501E+00_JPRB ,&
&  0.85722E+00_JPRB, 0.83174E+00_JPRB, 0.81345E+00_JPRB, 0.75161E+00_JPRB, 0.78609E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.70553E+00_JPRB, 0.69915E+00_JPRB, 0.69068E+00_JPRB, 0.68385E+00_JPRB, 0.67938E+00_JPRB, 0.67316E+00_JPRB, 0.66816E+00_JPRB ,&
&  0.66131E+00_JPRB, 0.65560E+00_JPRB, 0.64630E+00_JPRB, 0.64024E+00_JPRB, 0.63796E+00_JPRB, 0.63901E+00_JPRB, 0.64014E+00_JPRB ,&
&  0.63630E+00_JPRB, 0.62859E+00_JPRB, 0.62032E+00_JPRB, 0.61350E+00_JPRB, 0.60746E+00_JPRB, 0.60046E+00_JPRB, 0.59320E+00_JPRB ,&
&  0.58640E+00_JPRB, 0.58054E+00_JPRB, 0.57538E+00_JPRB, 0.57330E+00_JPRB, 0.57126E+00_JPRB, 0.56818E+00_JPRB, 0.56376E+00_JPRB ,&
&  0.56854E+00_JPRB, 0.51919E+00_JPRB, 0.54478E+00_JPRB, 0.57386E+00_JPRB, 0.61270E+00_JPRB, 0.70746E+00_JPRB, 0.84272E+00_JPRB ,&
&  0.97239E+00_JPRB, 0.10555E+01_JPRB, 0.10747E+01_JPRB, 0.10348E+01_JPRB, 0.99030E+00_JPRB, 0.96365E+00_JPRB, 0.94151E+00_JPRB ,&
&  0.89876E+00_JPRB, 0.85994E+00_JPRB, 0.82089E+00_JPRB, 0.78129E+00_JPRB, 0.74468E+00_JPRB, 0.71206E+00_JPRB, 0.68608E+00_JPRB ,&
&  0.66889E+00_JPRB, 0.65973E+00_JPRB, 0.65507E+00_JPRB, 0.65322E+00_JPRB, 0.65455E+00_JPRB, 0.65648E+00_JPRB, 0.65844E+00_JPRB ,&
&  0.66682E+00_JPRB, 0.68426E+00_JPRB, 0.69109E+00_JPRB, 0.69742E+00_JPRB, 0.71389E+00_JPRB, 0.73776E+00_JPRB, 0.74988E+00_JPRB ,&
&  0.74910E+00_JPRB, 0.73863E+00_JPRB, 0.72393E+00_JPRB, 0.70812E+00_JPRB, 0.69451E+00_JPRB, 0.68324E+00_JPRB, 0.68001E+00_JPRB ,&
&  0.67948E+00_JPRB, 0.68222E+00_JPRB, 0.68682E+00_JPRB, 0.69664E+00_JPRB, 0.70914E+00_JPRB, 0.72199E+00_JPRB, 0.73442E+00_JPRB ,&
&  0.74671E+00_JPRB, 0.76076E+00_JPRB, 0.78154E+00_JPRB, 0.80598E+00_JPRB, 0.82689E+00_JPRB, 0.83798E+00_JPRB, 0.84097E+00_JPRB ,&
&  0.83784E+00_JPRB, 0.83233E+00_JPRB, 0.82143E+00_JPRB, 0.81931E+00_JPRB, 0.81341E+00_JPRB, 0.81014E+00_JPRB, 0.79768E+00_JPRB ,&
&  0.78188E+00_JPRB, 0.76479E+00_JPRB, 0.67595E+00_JPRB, 0.70705E+00_JPRB, 0.70717E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.64619E+00_JPRB, 0.63634E+00_JPRB, 0.62731E+00_JPRB, 0.61884E+00_JPRB, 0.61286E+00_JPRB, 0.60904E+00_JPRB, 0.60784E+00_JPRB ,&
&  0.60949E+00_JPRB, 0.61176E+00_JPRB, 0.61048E+00_JPRB, 0.60396E+00_JPRB, 0.59738E+00_JPRB, 0.59369E+00_JPRB, 0.59286E+00_JPRB ,&
&  0.59143E+00_JPRB, 0.58764E+00_JPRB, 0.58240E+00_JPRB, 0.57717E+00_JPRB, 0.57086E+00_JPRB, 0.56508E+00_JPRB, 0.55968E+00_JPRB ,&
&  0.55482E+00_JPRB, 0.55166E+00_JPRB, 0.54881E+00_JPRB, 0.54638E+00_JPRB, 0.54512E+00_JPRB, 0.54272E+00_JPRB, 0.54402E+00_JPRB ,&
&  0.51394E+00_JPRB, 0.48225E+00_JPRB, 0.49799E+00_JPRB, 0.51260E+00_JPRB, 0.52768E+00_JPRB, 0.56806E+00_JPRB, 0.63871E+00_JPRB ,&
&  0.73498E+00_JPRB, 0.82095E+00_JPRB, 0.87477E+00_JPRB, 0.87343E+00_JPRB, 0.85084E+00_JPRB, 0.83826E+00_JPRB, 0.83250E+00_JPRB ,&
&  0.81066E+00_JPRB, 0.78433E+00_JPRB, 0.75971E+00_JPRB, 0.73059E+00_JPRB, 0.69959E+00_JPRB, 0.67242E+00_JPRB, 0.65129E+00_JPRB ,&
&  0.63560E+00_JPRB, 0.62639E+00_JPRB, 0.62233E+00_JPRB, 0.61892E+00_JPRB, 0.61525E+00_JPRB, 0.61288E+00_JPRB, 0.60912E+00_JPRB ,&
&  0.60726E+00_JPRB, 0.60779E+00_JPRB, 0.61283E+00_JPRB, 0.61638E+00_JPRB, 0.62196E+00_JPRB, 0.62702E+00_JPRB, 0.62848E+00_JPRB ,&
&  0.62515E+00_JPRB, 0.62167E+00_JPRB, 0.62068E+00_JPRB, 0.61636E+00_JPRB, 0.60629E+00_JPRB, 0.59569E+00_JPRB, 0.59081E+00_JPRB ,&
&  0.59030E+00_JPRB, 0.59102E+00_JPRB, 0.59268E+00_JPRB, 0.59717E+00_JPRB, 0.60408E+00_JPRB, 0.61220E+00_JPRB, 0.62046E+00_JPRB ,&
&  0.62864E+00_JPRB, 0.63604E+00_JPRB, 0.64373E+00_JPRB, 0.65405E+00_JPRB, 0.66881E+00_JPRB, 0.68320E+00_JPRB, 0.69522E+00_JPRB ,&
&  0.70416E+00_JPRB, 0.71016E+00_JPRB, 0.71436E+00_JPRB, 0.71984E+00_JPRB, 0.72447E+00_JPRB, 0.72607E+00_JPRB, 0.72094E+00_JPRB ,&
&  0.70946E+00_JPRB, 0.69334E+00_JPRB, 0.68221E+00_JPRB, 0.66748E+00_JPRB, 0.65514E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.57765E+00_JPRB, 0.57126E+00_JPRB, 0.56416E+00_JPRB, 0.55723E+00_JPRB, 0.55091E+00_JPRB, 0.54820E+00_JPRB, 0.54811E+00_JPRB ,&
&  0.55157E+00_JPRB, 0.55424E+00_JPRB, 0.55802E+00_JPRB, 0.55722E+00_JPRB, 0.55447E+00_JPRB, 0.55179E+00_JPRB, 0.54916E+00_JPRB ,&
&  0.54626E+00_JPRB, 0.54223E+00_JPRB, 0.53808E+00_JPRB, 0.53487E+00_JPRB, 0.53133E+00_JPRB, 0.52648E+00_JPRB, 0.52178E+00_JPRB ,&
&  0.51776E+00_JPRB, 0.51521E+00_JPRB, 0.51317E+00_JPRB, 0.51121E+00_JPRB, 0.50907E+00_JPRB, 0.50817E+00_JPRB, 0.50623E+00_JPRB ,&
&  0.50789E+00_JPRB, 0.47799E+00_JPRB, 0.49121E+00_JPRB, 0.49577E+00_JPRB, 0.50381E+00_JPRB, 0.52159E+00_JPRB, 0.54661E+00_JPRB ,&
&  0.58611E+00_JPRB, 0.63958E+00_JPRB, 0.68713E+00_JPRB, 0.70737E+00_JPRB, 0.70363E+00_JPRB, 0.70171E+00_JPRB, 0.70798E+00_JPRB ,&
&  0.70643E+00_JPRB, 0.69809E+00_JPRB, 0.69172E+00_JPRB, 0.68147E+00_JPRB, 0.66491E+00_JPRB, 0.64339E+00_JPRB, 0.62433E+00_JPRB ,&
&  0.61240E+00_JPRB, 0.60518E+00_JPRB, 0.59843E+00_JPRB, 0.59084E+00_JPRB, 0.58062E+00_JPRB, 0.57015E+00_JPRB, 0.56065E+00_JPRB ,&
&  0.55205E+00_JPRB, 0.54276E+00_JPRB, 0.53714E+00_JPRB, 0.53498E+00_JPRB, 0.53334E+00_JPRB, 0.53368E+00_JPRB, 0.53328E+00_JPRB ,&
&  0.53023E+00_JPRB, 0.52571E+00_JPRB, 0.52327E+00_JPRB, 0.52327E+00_JPRB, 0.52258E+00_JPRB, 0.52176E+00_JPRB, 0.52074E+00_JPRB ,&
&  0.52180E+00_JPRB, 0.52301E+00_JPRB, 0.52294E+00_JPRB, 0.52326E+00_JPRB, 0.52527E+00_JPRB, 0.52963E+00_JPRB, 0.53480E+00_JPRB ,&
&  0.54059E+00_JPRB, 0.54691E+00_JPRB, 0.55286E+00_JPRB, 0.55872E+00_JPRB, 0.56703E+00_JPRB, 0.57594E+00_JPRB, 0.58673E+00_JPRB ,&
&  0.59835E+00_JPRB, 0.60848E+00_JPRB, 0.61711E+00_JPRB, 0.62435E+00_JPRB, 0.62873E+00_JPRB, 0.62867E+00_JPRB, 0.62613E+00_JPRB ,&
&  0.62162E+00_JPRB, 0.61124E+00_JPRB, 0.60186E+00_JPRB, 0.59186E+00_JPRB, 0.58301E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.52519E+00_JPRB, 0.51912E+00_JPRB, 0.51270E+00_JPRB, 0.50770E+00_JPRB, 0.50360E+00_JPRB, 0.49987E+00_JPRB, 0.49875E+00_JPRB ,&
&  0.50024E+00_JPRB, 0.50365E+00_JPRB, 0.50836E+00_JPRB, 0.51161E+00_JPRB, 0.51338E+00_JPRB, 0.51483E+00_JPRB, 0.51471E+00_JPRB ,&
&  0.51365E+00_JPRB, 0.51155E+00_JPRB, 0.50852E+00_JPRB, 0.50448E+00_JPRB, 0.49996E+00_JPRB, 0.49564E+00_JPRB, 0.49020E+00_JPRB ,&
&  0.48601E+00_JPRB, 0.48299E+00_JPRB, 0.48082E+00_JPRB, 0.47937E+00_JPRB, 0.47919E+00_JPRB, 0.47915E+00_JPRB, 0.47867E+00_JPRB ,&
&  0.47559E+00_JPRB, 0.47708E+00_JPRB, 0.48442E+00_JPRB, 0.48495E+00_JPRB, 0.49057E+00_JPRB, 0.50298E+00_JPRB, 0.51459E+00_JPRB ,&
&  0.52457E+00_JPRB, 0.54490E+00_JPRB, 0.57342E+00_JPRB, 0.59506E+00_JPRB, 0.59995E+00_JPRB, 0.60091E+00_JPRB, 0.60976E+00_JPRB ,&
&  0.61538E+00_JPRB, 0.61459E+00_JPRB, 0.61597E+00_JPRB, 0.61595E+00_JPRB, 0.61253E+00_JPRB, 0.60706E+00_JPRB, 0.59763E+00_JPRB ,&
&  0.58714E+00_JPRB, 0.57771E+00_JPRB, 0.56750E+00_JPRB, 0.55694E+00_JPRB, 0.54375E+00_JPRB, 0.52960E+00_JPRB, 0.51490E+00_JPRB ,&
&  0.50041E+00_JPRB, 0.48985E+00_JPRB, 0.48285E+00_JPRB, 0.47728E+00_JPRB, 0.47263E+00_JPRB, 0.47038E+00_JPRB, 0.46796E+00_JPRB ,&
&  0.46640E+00_JPRB, 0.46586E+00_JPRB, 0.46394E+00_JPRB, 0.46303E+00_JPRB, 0.46424E+00_JPRB, 0.46567E+00_JPRB, 0.46884E+00_JPRB ,&
&  0.47061E+00_JPRB, 0.47208E+00_JPRB, 0.47318E+00_JPRB, 0.47321E+00_JPRB, 0.47225E+00_JPRB, 0.47352E+00_JPRB, 0.47651E+00_JPRB ,&
&  0.47989E+00_JPRB, 0.48503E+00_JPRB, 0.49071E+00_JPRB, 0.49575E+00_JPRB, 0.50057E+00_JPRB, 0.50795E+00_JPRB, 0.51739E+00_JPRB ,&
&  0.52795E+00_JPRB, 0.53925E+00_JPRB, 0.54918E+00_JPRB, 0.55652E+00_JPRB, 0.55863E+00_JPRB, 0.55822E+00_JPRB, 0.55708E+00_JPRB ,&
&  0.55384E+00_JPRB, 0.54617E+00_JPRB, 0.54091E+00_JPRB, 0.53630E+00_JPRB, 0.53082E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.46545E+00_JPRB, 0.46281E+00_JPRB, 0.46061E+00_JPRB, 0.45946E+00_JPRB, 0.45895E+00_JPRB, 0.45908E+00_JPRB, 0.46022E+00_JPRB ,&
&  0.46268E+00_JPRB, 0.46524E+00_JPRB, 0.47073E+00_JPRB, 0.47441E+00_JPRB, 0.47663E+00_JPRB, 0.47902E+00_JPRB, 0.48050E+00_JPRB ,&
&  0.47968E+00_JPRB, 0.47874E+00_JPRB, 0.47716E+00_JPRB, 0.47409E+00_JPRB, 0.46926E+00_JPRB, 0.46592E+00_JPRB, 0.46254E+00_JPRB ,&
&  0.45792E+00_JPRB, 0.45263E+00_JPRB, 0.44777E+00_JPRB, 0.44446E+00_JPRB, 0.44303E+00_JPRB, 0.44117E+00_JPRB, 0.44036E+00_JPRB ,&
&  0.43851E+00_JPRB, 0.44572E+00_JPRB, 0.43484E+00_JPRB, 0.44457E+00_JPRB, 0.45284E+00_JPRB, 0.46117E+00_JPRB, 0.47017E+00_JPRB ,&
&  0.47652E+00_JPRB, 0.48458E+00_JPRB, 0.49972E+00_JPRB, 0.51409E+00_JPRB, 0.52053E+00_JPRB, 0.51996E+00_JPRB, 0.52291E+00_JPRB ,&
&  0.52520E+00_JPRB, 0.52209E+00_JPRB, 0.52218E+00_JPRB, 0.52629E+00_JPRB, 0.53051E+00_JPRB, 0.53180E+00_JPRB, 0.53155E+00_JPRB ,&
&  0.53108E+00_JPRB, 0.52550E+00_JPRB, 0.51515E+00_JPRB, 0.50369E+00_JPRB, 0.49295E+00_JPRB, 0.48150E+00_JPRB, 0.46827E+00_JPRB ,&
&  0.45417E+00_JPRB, 0.44278E+00_JPRB, 0.43338E+00_JPRB, 0.42690E+00_JPRB, 0.42304E+00_JPRB, 0.41997E+00_JPRB, 0.41742E+00_JPRB ,&
&  0.41654E+00_JPRB, 0.41633E+00_JPRB, 0.41562E+00_JPRB, 0.41478E+00_JPRB, 0.41549E+00_JPRB, 0.41695E+00_JPRB, 0.42132E+00_JPRB ,&
&  0.42648E+00_JPRB, 0.43003E+00_JPRB, 0.43247E+00_JPRB, 0.43435E+00_JPRB, 0.43357E+00_JPRB, 0.43464E+00_JPRB, 0.43697E+00_JPRB ,&
&  0.43940E+00_JPRB, 0.44235E+00_JPRB, 0.44444E+00_JPRB, 0.44669E+00_JPRB, 0.44975E+00_JPRB, 0.45426E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.46818E+00_JPRB, 0.47570E+00_JPRB, 0.48360E+00_JPRB, 0.48981E+00_JPRB, 0.49335E+00_JPRB, 0.49418E+00_JPRB, 0.49382E+00_JPRB ,&
&  0.49106E+00_JPRB, 0.48505E+00_JPRB, 0.47871E+00_JPRB, 0.47361E+00_JPRB, 0.46916E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.42108E+00_JPRB, 0.42121E+00_JPRB, 0.42235E+00_JPRB, 0.42255E+00_JPRB, 0.42242E+00_JPRB, 0.42353E+00_JPRB, 0.42711E+00_JPRB ,&
&  0.43058E+00_JPRB, 0.43337E+00_JPRB, 0.43643E+00_JPRB, 0.43892E+00_JPRB, 0.44094E+00_JPRB, 0.44412E+00_JPRB, 0.44661E+00_JPRB ,&
&  0.44759E+00_JPRB, 0.44682E+00_JPRB, 0.44544E+00_JPRB, 0.44329E+00_JPRB, 0.44016E+00_JPRB, 0.43656E+00_JPRB, 0.43324E+00_JPRB ,&
&  0.42903E+00_JPRB, 0.42423E+00_JPRB, 0.41938E+00_JPRB, 0.41629E+00_JPRB, 0.41446E+00_JPRB, 0.41308E+00_JPRB, 0.41283E+00_JPRB ,&
&  0.41250E+00_JPRB, 0.41489E+00_JPRB, 0.41784E+00_JPRB, 0.42162E+00_JPRB, 0.42520E+00_JPRB, 0.42981E+00_JPRB, 0.43479E+00_JPRB ,&
&  0.43949E+00_JPRB, 0.44227E+00_JPRB, 0.44635E+00_JPRB, 0.44860E+00_JPRB, 0.45575E+00_JPRB, 0.46142E+00_JPRB, 0.46471E+00_JPRB ,&
&  0.46450E+00_JPRB, 0.45746E+00_JPRB, 0.45272E+00_JPRB, 0.45358E+00_JPRB, 0.45685E+00_JPRB, 0.46030E+00_JPRB, 0.46361E+00_JPRB ,&
&  0.46502E+00_JPRB, 0.46360E+00_JPRB, 0.45838E+00_JPRB, 0.45071E+00_JPRB, 0.44240E+00_JPRB, 0.43331E+00_JPRB, 0.42191E+00_JPRB ,&
&  0.41146E+00_JPRB, 0.40295E+00_JPRB, 0.39594E+00_JPRB, 0.38973E+00_JPRB, 0.38523E+00_JPRB, 0.38072E+00_JPRB, 0.37913E+00_JPRB ,&
&  0.37953E+00_JPRB, 0.38177E+00_JPRB, 0.38288E+00_JPRB, 0.38197E+00_JPRB, 0.38065E+00_JPRB, 0.38083E+00_JPRB, 0.38417E+00_JPRB ,&
&  0.39011E+00_JPRB, 0.39496E+00_JPRB, 0.39732E+00_JPRB, 0.39822E+00_JPRB, 0.39833E+00_JPRB, 0.40040E+00_JPRB, 0.40368E+00_JPRB ,&
&  0.40564E+00_JPRB, 0.40668E+00_JPRB, 0.40786E+00_JPRB, 0.40764E+00_JPRB, 0.40953E+00_JPRB, 0.41293E+00_JPRB, 0.41692E+00_JPRB ,&
&  0.42097E+00_JPRB, 0.42605E+00_JPRB, 0.43103E+00_JPRB, 0.43486E+00_JPRB, 0.43794E+00_JPRB, 0.43789E+00_JPRB, 0.43758E+00_JPRB ,&
&  0.43674E+00_JPRB, 0.43410E+00_JPRB, 0.43118E+00_JPRB, 0.42758E+00_JPRB, 0.42358E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.38471E+00_JPRB, 0.38805E+00_JPRB, 0.39215E+00_JPRB, 0.39278E+00_JPRB, 0.39280E+00_JPRB, 0.39416E+00_JPRB, 0.39757E+00_JPRB ,&
&  0.40036E+00_JPRB, 0.40182E+00_JPRB, 0.40289E+00_JPRB, 0.40391E+00_JPRB, 0.40570E+00_JPRB, 0.40877E+00_JPRB, 0.41063E+00_JPRB ,&
&  0.41063E+00_JPRB, 0.41015E+00_JPRB, 0.40919E+00_JPRB, 0.40810E+00_JPRB, 0.40589E+00_JPRB, 0.40265E+00_JPRB, 0.39960E+00_JPRB ,&
&  0.39555E+00_JPRB, 0.39162E+00_JPRB, 0.38828E+00_JPRB, 0.38536E+00_JPRB, 0.38314E+00_JPRB, 0.38205E+00_JPRB, 0.38277E+00_JPRB ,&
&  0.38397E+00_JPRB, 0.38589E+00_JPRB, 0.38870E+00_JPRB, 0.39223E+00_JPRB, 0.39568E+00_JPRB, 0.39955E+00_JPRB, 0.40300E+00_JPRB ,&
&  0.40575E+00_JPRB, 0.40837E+00_JPRB, 0.41081E+00_JPRB, 0.40983E+00_JPRB, 0.40924E+00_JPRB, 0.41349E+00_JPRB, 0.41845E+00_JPRB ,&
&  0.41764E+00_JPRB, 0.41257E+00_JPRB, 0.40637E+00_JPRB, 0.40342E+00_JPRB, 0.40297E+00_JPRB, 0.40219E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.40000E+00_JPRB, 0.39879E+00_JPRB, 0.39633E+00_JPRB, 0.39244E+00_JPRB, 0.38708E+00_JPRB, 0.38059E+00_JPRB, 0.37452E+00_JPRB ,&
&  0.37069E+00_JPRB, 0.36793E+00_JPRB, 0.36711E+00_JPRB, 0.36396E+00_JPRB, 0.35830E+00_JPRB, 0.35262E+00_JPRB, 0.35112E+00_JPRB ,&
&  0.35137E+00_JPRB, 0.35296E+00_JPRB, 0.35386E+00_JPRB, 0.35321E+00_JPRB, 0.35339E+00_JPRB, 0.35354E+00_JPRB, 0.35515E+00_JPRB ,&
&  0.36099E+00_JPRB, 0.36554E+00_JPRB, 0.36614E+00_JPRB, 0.36545E+00_JPRB, 0.36363E+00_JPRB, 0.36353E+00_JPRB, 0.36514E+00_JPRB ,&
&  0.36700E+00_JPRB, 0.36824E+00_JPRB, 0.36900E+00_JPRB, 0.36860E+00_JPRB, 0.36862E+00_JPRB, 0.37006E+00_JPRB, 0.37204E+00_JPRB ,&
&  0.37458E+00_JPRB, 0.37772E+00_JPRB, 0.38045E+00_JPRB, 0.38312E+00_JPRB, 0.38556E+00_JPRB, 0.38671E+00_JPRB, 0.38775E+00_JPRB ,&
&  0.38878E+00_JPRB, 0.38861E+00_JPRB, 0.38774E+00_JPRB, 0.38678E+00_JPRB, 0.38565E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.35651E+00_JPRB, 0.36143E+00_JPRB, 0.36730E+00_JPRB, 0.36820E+00_JPRB, 0.36968E+00_JPRB, 0.37170E+00_JPRB, 0.37515E+00_JPRB ,&
&  0.37796E+00_JPRB, 0.37905E+00_JPRB, 0.37823E+00_JPRB, 0.37726E+00_JPRB, 0.37714E+00_JPRB, 0.37775E+00_JPRB, 0.37802E+00_JPRB ,&
&  0.37835E+00_JPRB, 0.37872E+00_JPRB, 0.37860E+00_JPRB, 0.37808E+00_JPRB, 0.37622E+00_JPRB, 0.37353E+00_JPRB, 0.37038E+00_JPRB ,&
&  0.36576E+00_JPRB, 0.36235E+00_JPRB, 0.36056E+00_JPRB, 0.35926E+00_JPRB, 0.35751E+00_JPRB, 0.35736E+00_JPRB, 0.35878E+00_JPRB ,&
&  0.36241E+00_JPRB, 0.36520E+00_JPRB, 0.36886E+00_JPRB, 0.37251E+00_JPRB, 0.37456E+00_JPRB, 0.37615E+00_JPRB, 0.37798E+00_JPRB ,&
&  0.37914E+00_JPRB, 0.37947E+00_JPRB, 0.37980E+00_JPRB, 0.38034E+00_JPRB, 0.37908E+00_JPRB, 0.38165E+00_JPRB, 0.38589E+00_JPRB ,&
&  0.38786E+00_JPRB, 0.38500E+00_JPRB, 0.38006E+00_JPRB, 0.37638E+00_JPRB, 0.37278E+00_JPRB, 0.36993E+00_JPRB, 0.36740E+00_JPRB ,&
&  0.36535E+00_JPRB, 0.36389E+00_JPRB, 0.36293E+00_JPRB, 0.36023E+00_JPRB, 0.35369E+00_JPRB, 0.34743E+00_JPRB, 0.34399E+00_JPRB ,&
&  0.34235E+00_JPRB, 0.34172E+00_JPRB, 0.34290E+00_JPRB, 0.34103E+00_JPRB, 0.33555E+00_JPRB, 0.33080E+00_JPRB, 0.32895E+00_JPRB ,&
&  0.32746E+00_JPRB, 0.32728E+00_JPRB, 0.32875E+00_JPRB, 0.32966E+00_JPRB, 0.33010E+00_JPRB, 0.32983E+00_JPRB, 0.33155E+00_JPRB ,&
&  0.33420E+00_JPRB, 0.33692E+00_JPRB, 0.33716E+00_JPRB, 0.33427E+00_JPRB, 0.33008E+00_JPRB, 0.32743E+00_JPRB, 0.32781E+00_JPRB ,&
&  0.33063E+00_JPRB, 0.33327E+00_JPRB, 0.33462E+00_JPRB, 0.33548E+00_JPRB, 0.33696E+00_JPRB, 0.33968E+00_JPRB, 0.34309E+00_JPRB ,&
&  0.34469E+00_JPRB, 0.34621E+00_JPRB, 0.34831E+00_JPRB, 0.35047E+00_JPRB, 0.35189E+00_JPRB, 0.35264E+00_JPRB, 0.35279E+00_JPRB ,&
&  0.35211E+00_JPRB, 0.35260E+00_JPRB, 0.35395E+00_JPRB, 0.35556E+00_JPRB, 0.35624E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.33596E+00_JPRB, 0.33913E+00_JPRB, 0.34200E+00_JPRB, 0.34326E+00_JPRB, 0.34469E+00_JPRB, 0.34661E+00_JPRB, 0.35025E+00_JPRB ,&
&  0.35478E+00_JPRB, 0.35670E+00_JPRB, 0.35513E+00_JPRB, 0.35221E+00_JPRB, 0.35057E+00_JPRB, 0.35021E+00_JPRB, 0.35054E+00_JPRB ,&
&  0.35082E+00_JPRB, 0.35094E+00_JPRB, 0.35013E+00_JPRB, 0.34799E+00_JPRB, 0.34483E+00_JPRB, 0.34142E+00_JPRB, 0.33851E+00_JPRB ,&
&  0.33728E+00_JPRB, 0.33799E+00_JPRB, 0.33954E+00_JPRB, 0.33899E+00_JPRB, 0.33878E+00_JPRB, 0.34013E+00_JPRB, 0.34259E+00_JPRB ,&
&  0.34684E+00_JPRB, 0.34853E+00_JPRB, 0.34963E+00_JPRB, 0.34613E+00_JPRB, 0.34964E+00_JPRB, 0.35103E+00_JPRB, 0.35122E+00_JPRB ,&
&  0.35221E+00_JPRB, 0.35310E+00_JPRB, 0.35442E+00_JPRB, 0.35413E+00_JPRB, 0.35065E+00_JPRB, 0.35118E+00_JPRB, 0.35553E+00_JPRB ,&
&  0.35900E+00_JPRB, 0.35765E+00_JPRB, 0.35400E+00_JPRB, 0.35067E+00_JPRB, 0.34780E+00_JPRB, 0.34545E+00_JPRB, 0.34346E+00_JPRB ,&
&  0.34250E+00_JPRB, 0.34126E+00_JPRB, 0.33841E+00_JPRB, 0.33207E+00_JPRB, 0.32522E+00_JPRB, 0.32051E+00_JPRB, 0.31690E+00_JPRB ,&
&  0.31628E+00_JPRB, 0.31801E+00_JPRB, 0.31993E+00_JPRB, 0.31982E+00_JPRB, 0.31752E+00_JPRB, 0.31355E+00_JPRB, 0.30871E+00_JPRB ,&
&  0.30339E+00_JPRB, 0.30201E+00_JPRB, 0.30245E+00_JPRB, 0.30219E+00_JPRB, 0.30286E+00_JPRB, 0.30500E+00_JPRB, 0.30762E+00_JPRB ,&
&  0.31094E+00_JPRB, 0.31064E+00_JPRB, 0.30877E+00_JPRB, 0.30587E+00_JPRB, 0.30299E+00_JPRB, 0.30286E+00_JPRB, 0.30424E+00_JPRB ,&
&  0.30578E+00_JPRB, 0.30737E+00_JPRB, 0.30937E+00_JPRB, 0.31170E+00_JPRB, 0.31431E+00_JPRB, 0.31735E+00_JPRB, 0.31960E+00_JPRB ,&
&  0.31945E+00_JPRB, 0.31821E+00_JPRB, 0.31957E+00_JPRB, 0.32217E+00_JPRB, 0.32362E+00_JPRB, 0.32438E+00_JPRB, 0.32481E+00_JPRB ,&
&  0.32406E+00_JPRB, 0.32341E+00_JPRB, 0.32496E+00_JPRB, 0.32933E+00_JPRB, 0.33327E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.31726E+00_JPRB, 0.31859E+00_JPRB, 0.31860E+00_JPRB, 0.31904E+00_JPRB, 0.31966E+00_JPRB, 0.32154E+00_JPRB, 0.32587E+00_JPRB ,&
&  0.33028E+00_JPRB, 0.33224E+00_JPRB, 0.33027E+00_JPRB, 0.32759E+00_JPRB, 0.32624E+00_JPRB, 0.32584E+00_JPRB, 0.32601E+00_JPRB ,&
&  0.32617E+00_JPRB, 0.32536E+00_JPRB, 0.32247E+00_JPRB, 0.31828E+00_JPRB, 0.31477E+00_JPRB, 0.31324E+00_JPRB, 0.31293E+00_JPRB ,&
&  0.31618E+00_JPRB, 0.31830E+00_JPRB, 0.32019E+00_JPRB, 0.32241E+00_JPRB, 0.32349E+00_JPRB, 0.32381E+00_JPRB, 0.32589E+00_JPRB ,&
&  0.32671E+00_JPRB, 0.32304E+00_JPRB, 0.31734E+00_JPRB, 0.32040E+00_JPRB, 0.32302E+00_JPRB, 0.32456E+00_JPRB, 0.32378E+00_JPRB ,&
&  0.32431E+00_JPRB, 0.32655E+00_JPRB, 0.32917E+00_JPRB, 0.33117E+00_JPRB, 0.33090E+00_JPRB, 0.32939E+00_JPRB, 0.33057E+00_JPRB ,&
&  0.33257E+00_JPRB, 0.33164E+00_JPRB, 0.32811E+00_JPRB, 0.32481E+00_JPRB, 0.32109E+00_JPRB, 0.31923E+00_JPRB, 0.31897E+00_JPRB ,&
&  0.31821E+00_JPRB, 0.31587E+00_JPRB, 0.31139E+00_JPRB, 0.30676E+00_JPRB, 0.30323E+00_JPRB, 0.29927E+00_JPRB, 0.29676E+00_JPRB ,&
&  0.29654E+00_JPRB, 0.29729E+00_JPRB, 0.29702E+00_JPRB, 0.29446E+00_JPRB, 0.28658E+00_JPRB, 0.27837E+00_JPRB, 0.27075E+00_JPRB ,&
&  0.26928E+00_JPRB, 0.27305E+00_JPRB, 0.27964E+00_JPRB, 0.28359E+00_JPRB, 0.28721E+00_JPRB, 0.28924E+00_JPRB, 0.29034E+00_JPRB ,&
&  0.28665E+00_JPRB, 0.27535E+00_JPRB, 0.27035E+00_JPRB, 0.26667E+00_JPRB, 0.26361E+00_JPRB, 0.26428E+00_JPRB, 0.26681E+00_JPRB ,&
&  0.27158E+00_JPRB, 0.27609E+00_JPRB, 0.27991E+00_JPRB, 0.28251E+00_JPRB, 0.28360E+00_JPRB, 0.28423E+00_JPRB, 0.28364E+00_JPRB ,&
&  0.28462E+00_JPRB, 0.28950E+00_JPRB, 0.29812E+00_JPRB, 0.30352E+00_JPRB, 0.30474E+00_JPRB, 0.30624E+00_JPRB, 0.30575E+00_JPRB ,&
&  0.30544E+00_JPRB, 0.30736E+00_JPRB, 0.30827E+00_JPRB, 0.31083E+00_JPRB, 0.31435E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.30239E+00_JPRB, 0.30354E+00_JPRB, 0.30348E+00_JPRB, 0.30218E+00_JPRB, 0.30059E+00_JPRB, 0.30036E+00_JPRB, 0.30151E+00_JPRB ,&
&  0.30436E+00_JPRB, 0.30628E+00_JPRB, 0.30650E+00_JPRB, 0.30635E+00_JPRB, 0.30546E+00_JPRB, 0.30367E+00_JPRB, 0.30240E+00_JPRB ,&
&  0.30153E+00_JPRB, 0.30125E+00_JPRB, 0.30097E+00_JPRB, 0.30099E+00_JPRB, 0.30093E+00_JPRB, 0.30138E+00_JPRB, 0.30161E+00_JPRB ,&
&  0.30468E+00_JPRB, 0.30406E+00_JPRB, 0.30328E+00_JPRB, 0.30314E+00_JPRB, 0.30317E+00_JPRB, 0.30353E+00_JPRB, 0.30341E+00_JPRB ,&
&  0.29922E+00_JPRB, 0.29183E+00_JPRB, 0.28647E+00_JPRB, 0.29401E+00_JPRB, 0.30324E+00_JPRB, 0.30339E+00_JPRB, 0.30406E+00_JPRB ,&
&  0.30563E+00_JPRB, 0.30849E+00_JPRB, 0.31098E+00_JPRB, 0.31203E+00_JPRB, 0.31114E+00_JPRB, 0.31029E+00_JPRB, 0.31059E+00_JPRB ,&
&  0.31150E+00_JPRB, 0.30973E+00_JPRB, 0.30351E+00_JPRB, 0.29600E+00_JPRB, 0.28839E+00_JPRB, 0.28568E+00_JPRB, 0.28434E+00_JPRB ,&
&  0.28148E+00_JPRB, 0.27965E+00_JPRB, 0.27916E+00_JPRB, 0.27877E+00_JPRB, 0.27930E+00_JPRB, 0.27631E+00_JPRB, 0.27117E+00_JPRB ,&
&  0.26911E+00_JPRB, 0.26438E+00_JPRB, 0.25949E+00_JPRB, 0.25430E+00_JPRB, 0.25007E+00_JPRB, 0.24465E+00_JPRB, 0.24084E+00_JPRB ,&
&  0.24051E+00_JPRB, 0.24600E+00_JPRB, 0.25743E+00_JPRB, 0.26687E+00_JPRB, 0.26827E+00_JPRB, 0.26204E+00_JPRB, 0.25696E+00_JPRB ,&
&  0.24930E+00_JPRB, 0.24181E+00_JPRB, 0.23752E+00_JPRB, 0.23689E+00_JPRB, 0.23728E+00_JPRB, 0.23834E+00_JPRB, 0.24080E+00_JPRB ,&
&  0.24404E+00_JPRB, 0.24785E+00_JPRB, 0.25063E+00_JPRB, 0.25299E+00_JPRB, 0.25471E+00_JPRB, 0.25515E+00_JPRB, 0.25478E+00_JPRB ,&
&  0.25452E+00_JPRB, 0.25613E+00_JPRB, 0.25861E+00_JPRB, 0.26352E+00_JPRB, 0.26602E+00_JPRB, 0.26971E+00_JPRB, 0.27396E+00_JPRB ,&
&  0.28077E+00_JPRB, 0.28803E+00_JPRB, 0.29288E+00_JPRB, 0.29638E+00_JPRB, 0.29979E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.28705E+00_JPRB, 0.29215E+00_JPRB, 0.29411E+00_JPRB, 0.29309E+00_JPRB, 0.29229E+00_JPRB, 0.29234E+00_JPRB, 0.29254E+00_JPRB ,&
&  0.29324E+00_JPRB, 0.29250E+00_JPRB, 0.29077E+00_JPRB, 0.28943E+00_JPRB, 0.28801E+00_JPRB, 0.28626E+00_JPRB, 0.28548E+00_JPRB ,&
&  0.28499E+00_JPRB, 0.28583E+00_JPRB, 0.28655E+00_JPRB, 0.28833E+00_JPRB, 0.28915E+00_JPRB, 0.29056E+00_JPRB, 0.28814E+00_JPRB ,&
&  0.28360E+00_JPRB, 0.28065E+00_JPRB, 0.28320E+00_JPRB, 0.28345E+00_JPRB, 0.28399E+00_JPRB, 0.28455E+00_JPRB, 0.28623E+00_JPRB ,&
&  0.28302E+00_JPRB, 0.28066E+00_JPRB, 0.27991E+00_JPRB, 0.28809E+00_JPRB, 0.29386E+00_JPRB, 0.29094E+00_JPRB, 0.29122E+00_JPRB ,&
&  0.29338E+00_JPRB, 0.29601E+00_JPRB, 0.29740E+00_JPRB, 0.29736E+00_JPRB, 0.29608E+00_JPRB, 0.29596E+00_JPRB, 0.29470E+00_JPRB ,&
&  0.29104E+00_JPRB, 0.28811E+00_JPRB, 0.27684E+00_JPRB, 0.25802E+00_JPRB, 0.25267E+00_JPRB, 0.25076E+00_JPRB, 0.24691E+00_JPRB ,&
&  0.24120E+00_JPRB, 0.23985E+00_JPRB, 0.24025E+00_JPRB, 0.23744E+00_JPRB, 0.23745E+00_JPRB, 0.23726E+00_JPRB, 0.23551E+00_JPRB ,&
&  0.23347E+00_JPRB, 0.23224E+00_JPRB, 0.23060E+00_JPRB, 0.22772E+00_JPRB, 0.22547E+00_JPRB, 0.22518E+00_JPRB, 0.22638E+00_JPRB ,&
&  0.22874E+00_JPRB, 0.23177E+00_JPRB, 0.24148E+00_JPRB, 0.24698E+00_JPRB, 0.24514E+00_JPRB, 0.24255E+00_JPRB, 0.23531E+00_JPRB ,&
&  0.22751E+00_JPRB, 0.22528E+00_JPRB, 0.22373E+00_JPRB, 0.22233E+00_JPRB, 0.22281E+00_JPRB, 0.22408E+00_JPRB, 0.22532E+00_JPRB ,&
&  0.22705E+00_JPRB, 0.22931E+00_JPRB, 0.23073E+00_JPRB, 0.23156E+00_JPRB, 0.23234E+00_JPRB, 0.23267E+00_JPRB, 0.23324E+00_JPRB ,&
&  0.23306E+00_JPRB, 0.23464E+00_JPRB, 0.23568E+00_JPRB, 0.23838E+00_JPRB, 0.24008E+00_JPRB, 0.24261E+00_JPRB, 0.24866E+00_JPRB ,&
&  0.25272E+00_JPRB, 0.26580E+00_JPRB, 0.28186E+00_JPRB, 0.28555E+00_JPRB, 0.28107E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.27675E+00_JPRB, 0.27898E+00_JPRB, 0.28013E+00_JPRB, 0.27806E+00_JPRB, 0.27791E+00_JPRB, 0.27796E+00_JPRB, 0.27901E+00_JPRB ,&
&  0.27936E+00_JPRB, 0.27858E+00_JPRB, 0.27822E+00_JPRB, 0.27594E+00_JPRB, 0.27429E+00_JPRB, 0.26827E+00_JPRB, 0.26631E+00_JPRB ,&
&  0.26459E+00_JPRB, 0.26593E+00_JPRB, 0.26667E+00_JPRB, 0.26890E+00_JPRB, 0.26838E+00_JPRB, 0.26638E+00_JPRB, 0.26215E+00_JPRB ,&
&  0.25958E+00_JPRB, 0.25872E+00_JPRB, 0.25744E+00_JPRB, 0.25839E+00_JPRB, 0.26116E+00_JPRB, 0.26371E+00_JPRB, 0.26563E+00_JPRB ,&
&  0.26750E+00_JPRB, 0.27065E+00_JPRB, 0.27365E+00_JPRB, 0.27712E+00_JPRB, 0.28281E+00_JPRB, 0.28200E+00_JPRB, 0.28357E+00_JPRB ,&
&  0.28460E+00_JPRB, 0.28539E+00_JPRB, 0.28566E+00_JPRB, 0.28462E+00_JPRB, 0.28238E+00_JPRB, 0.27530E+00_JPRB, 0.27329E+00_JPRB ,&
&  0.26155E+00_JPRB, 0.25450E+00_JPRB, 0.24862E+00_JPRB, 0.24456E+00_JPRB, 0.23741E+00_JPRB, 0.23371E+00_JPRB, 0.22949E+00_JPRB ,&
&  0.22589E+00_JPRB, 0.22194E+00_JPRB, 0.21910E+00_JPRB, 0.21810E+00_JPRB, 0.21779E+00_JPRB, 0.21746E+00_JPRB, 0.21641E+00_JPRB ,&
&  0.21661E+00_JPRB, 0.21608E+00_JPRB, 0.21569E+00_JPRB, 0.21530E+00_JPRB, 0.21502E+00_JPRB, 0.21516E+00_JPRB, 0.21617E+00_JPRB ,&
&  0.21864E+00_JPRB, 0.22065E+00_JPRB, 0.22299E+00_JPRB, 0.22568E+00_JPRB, 0.22541E+00_JPRB, 0.22327E+00_JPRB, 0.22003E+00_JPRB ,&
&  0.21704E+00_JPRB, 0.21506E+00_JPRB, 0.21297E+00_JPRB, 0.21236E+00_JPRB, 0.21312E+00_JPRB, 0.21402E+00_JPRB, 0.21575E+00_JPRB ,&
&  0.21727E+00_JPRB, 0.21855E+00_JPRB, 0.21942E+00_JPRB, 0.21991E+00_JPRB, 0.22071E+00_JPRB, 0.22139E+00_JPRB, 0.22263E+00_JPRB ,&
&  0.22401E+00_JPRB, 0.22566E+00_JPRB, 0.22707E+00_JPRB, 0.22908E+00_JPRB, 0.23239E+00_JPRB, 0.23692E+00_JPRB, 0.24504E+00_JPRB ,&
&  0.25175E+00_JPRB, 0.26907E+00_JPRB, 0.27144E+00_JPRB, 0.27538E+00_JPRB, 0.27765E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.26544E+00_JPRB, 0.26729E+00_JPRB, 0.26771E+00_JPRB, 0.26837E+00_JPRB, 0.26806E+00_JPRB, 0.26795E+00_JPRB, 0.26854E+00_JPRB ,&
&  0.26419E+00_JPRB, 0.26152E+00_JPRB, 0.25676E+00_JPRB, 0.25277E+00_JPRB, 0.24888E+00_JPRB, 0.24595E+00_JPRB, 0.24423E+00_JPRB ,&
&  0.24321E+00_JPRB, 0.24350E+00_JPRB, 0.24308E+00_JPRB, 0.24330E+00_JPRB, 0.24147E+00_JPRB, 0.23999E+00_JPRB, 0.23876E+00_JPRB ,&
&  0.23773E+00_JPRB, 0.23908E+00_JPRB, 0.24029E+00_JPRB, 0.24427E+00_JPRB, 0.24986E+00_JPRB, 0.25584E+00_JPRB, 0.26051E+00_JPRB ,&
&  0.26352E+00_JPRB, 0.26601E+00_JPRB, 0.26960E+00_JPRB, 0.27122E+00_JPRB, 0.27329E+00_JPRB, 0.27501E+00_JPRB, 0.27530E+00_JPRB ,&
&  0.27657E+00_JPRB, 0.27596E+00_JPRB, 0.27544E+00_JPRB, 0.27486E+00_JPRB, 0.26509E+00_JPRB, 0.25845E+00_JPRB, 0.25138E+00_JPRB ,&
&  0.24495E+00_JPRB, 0.23830E+00_JPRB, 0.23226E+00_JPRB, 0.22920E+00_JPRB, 0.22507E+00_JPRB, 0.22274E+00_JPRB, 0.22072E+00_JPRB ,&
&  0.21915E+00_JPRB, 0.21752E+00_JPRB, 0.21512E+00_JPRB, 0.21281E+00_JPRB, 0.21144E+00_JPRB, 0.21027E+00_JPRB, 0.20873E+00_JPRB ,&
&  0.20867E+00_JPRB, 0.20791E+00_JPRB, 0.20804E+00_JPRB, 0.20771E+00_JPRB, 0.20752E+00_JPRB, 0.20735E+00_JPRB, 0.20734E+00_JPRB ,&
&  0.20785E+00_JPRB, 0.20899E+00_JPRB, 0.20957E+00_JPRB, 0.21036E+00_JPRB, 0.21045E+00_JPRB, 0.20970E+00_JPRB, 0.20910E+00_JPRB ,&
&  0.20826E+00_JPRB, 0.20763E+00_JPRB, 0.20766E+00_JPRB, 0.20747E+00_JPRB, 0.20805E+00_JPRB, 0.20881E+00_JPRB, 0.20952E+00_JPRB ,&
&  0.21059E+00_JPRB, 0.21129E+00_JPRB, 0.21207E+00_JPRB, 0.21318E+00_JPRB, 0.21402E+00_JPRB, 0.21515E+00_JPRB, 0.21643E+00_JPRB ,&
&  0.21727E+00_JPRB, 0.21928E+00_JPRB, 0.22096E+00_JPRB, 0.22313E+00_JPRB, 0.22775E+00_JPRB, 0.23165E+00_JPRB, 0.24170E+00_JPRB ,&
&  0.24937E+00_JPRB, 0.26429E+00_JPRB, 0.26494E+00_JPRB, 0.26255E+00_JPRB, 0.26397E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.25907E+00_JPRB, 0.26043E+00_JPRB, 0.26049E+00_JPRB, 0.26107E+00_JPRB, 0.26094E+00_JPRB, 0.26220E+00_JPRB, 0.26176E+00_JPRB ,&
&  0.26183E+00_JPRB, 0.25835E+00_JPRB, 0.25626E+00_JPRB, 0.25347E+00_JPRB, 0.25103E+00_JPRB, 0.24787E+00_JPRB, 0.24483E+00_JPRB ,&
&  0.24250E+00_JPRB, 0.24035E+00_JPRB, 0.23796E+00_JPRB, 0.23482E+00_JPRB, 0.23161E+00_JPRB, 0.22948E+00_JPRB, 0.22829E+00_JPRB ,&
&  0.22717E+00_JPRB, 0.22885E+00_JPRB, 0.23135E+00_JPRB, 0.23571E+00_JPRB, 0.24114E+00_JPRB, 0.24519E+00_JPRB, 0.25084E+00_JPRB ,&
&  0.25662E+00_JPRB, 0.25851E+00_JPRB, 0.26238E+00_JPRB, 0.26392E+00_JPRB, 0.26367E+00_JPRB, 0.26468E+00_JPRB, 0.26317E+00_JPRB ,&
&  0.26315E+00_JPRB, 0.25841E+00_JPRB, 0.26298E+00_JPRB, 0.25805E+00_JPRB, 0.24989E+00_JPRB, 0.24696E+00_JPRB, 0.24020E+00_JPRB ,&
&  0.23621E+00_JPRB, 0.23142E+00_JPRB, 0.22850E+00_JPRB, 0.22701E+00_JPRB, 0.22289E+00_JPRB, 0.22077E+00_JPRB, 0.21729E+00_JPRB ,&
&  0.21406E+00_JPRB, 0.21323E+00_JPRB, 0.21138E+00_JPRB, 0.21082E+00_JPRB, 0.20992E+00_JPRB, 0.20852E+00_JPRB, 0.20698E+00_JPRB ,&
&  0.20599E+00_JPRB, 0.20519E+00_JPRB, 0.20487E+00_JPRB, 0.20432E+00_JPRB, 0.20348E+00_JPRB, 0.20263E+00_JPRB, 0.20157E+00_JPRB ,&
&  0.20094E+00_JPRB, 0.20076E+00_JPRB, 0.20069E+00_JPRB, 0.20092E+00_JPRB, 0.20112E+00_JPRB, 0.20102E+00_JPRB, 0.20132E+00_JPRB ,&
&  0.20171E+00_JPRB, 0.20249E+00_JPRB, 0.20301E+00_JPRB, 0.20374E+00_JPRB, 0.20430E+00_JPRB, 0.20505E+00_JPRB, 0.20546E+00_JPRB ,&
&  0.20604E+00_JPRB, 0.20708E+00_JPRB, 0.20798E+00_JPRB, 0.20909E+00_JPRB, 0.21025E+00_JPRB, 0.21118E+00_JPRB, 0.21283E+00_JPRB ,&
&  0.21449E+00_JPRB, 0.21614E+00_JPRB, 0.21863E+00_JPRB, 0.22100E+00_JPRB, 0.22407E+00_JPRB, 0.22889E+00_JPRB, 0.23417E+00_JPRB ,&
&  0.24085E+00_JPRB, 0.25028E+00_JPRB, 0.25050E+00_JPRB, 0.25522E+00_JPRB, 0.25739E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.25824E+00_JPRB, 0.25965E+00_JPRB, 0.26076E+00_JPRB, 0.26130E+00_JPRB, 0.26217E+00_JPRB, 0.26214E+00_JPRB, 0.26191E+00_JPRB ,&
&  0.25849E+00_JPRB, 0.25717E+00_JPRB, 0.25509E+00_JPRB, 0.25273E+00_JPRB, 0.25066E+00_JPRB, 0.24711E+00_JPRB, 0.24417E+00_JPRB ,&
&  0.24153E+00_JPRB, 0.23853E+00_JPRB, 0.23584E+00_JPRB, 0.23263E+00_JPRB, 0.22974E+00_JPRB, 0.22707E+00_JPRB, 0.22615E+00_JPRB ,&
&  0.22590E+00_JPRB, 0.22688E+00_JPRB, 0.22930E+00_JPRB, 0.23100E+00_JPRB, 0.23335E+00_JPRB, 0.23627E+00_JPRB, 0.24061E+00_JPRB ,&
&  0.24357E+00_JPRB, 0.24748E+00_JPRB, 0.24991E+00_JPRB, 0.25048E+00_JPRB, 0.24984E+00_JPRB, 0.24954E+00_JPRB, 0.24857E+00_JPRB ,&
&  0.24620E+00_JPRB, 0.24382E+00_JPRB, 0.24183E+00_JPRB, 0.23964E+00_JPRB, 0.23737E+00_JPRB, 0.23488E+00_JPRB, 0.23162E+00_JPRB ,&
&  0.22877E+00_JPRB, 0.22627E+00_JPRB, 0.22379E+00_JPRB, 0.22179E+00_JPRB, 0.21975E+00_JPRB, 0.21786E+00_JPRB, 0.21563E+00_JPRB ,&
&  0.21389E+00_JPRB, 0.21243E+00_JPRB, 0.21056E+00_JPRB, 0.20921E+00_JPRB, 0.20760E+00_JPRB, 0.20628E+00_JPRB, 0.20482E+00_JPRB ,&
&  0.20379E+00_JPRB, 0.20294E+00_JPRB, 0.20228E+00_JPRB, 0.20166E+00_JPRB, 0.20138E+00_JPRB, 0.20112E+00_JPRB, 0.20080E+00_JPRB ,&
&  0.20055E+00_JPRB, 0.20081E+00_JPRB, 0.20106E+00_JPRB, 0.20130E+00_JPRB, 0.20156E+00_JPRB, 0.20120E+00_JPRB, 0.20130E+00_JPRB ,&
&  0.20164E+00_JPRB, 0.20195E+00_JPRB, 0.20178E+00_JPRB, 0.20213E+00_JPRB, 0.20281E+00_JPRB, 0.20362E+00_JPRB, 0.20496E+00_JPRB ,&
&  0.20600E+00_JPRB, 0.20711E+00_JPRB, 0.20816E+00_JPRB, 0.20923E+00_JPRB, 0.21072E+00_JPRB, 0.21232E+00_JPRB, 0.21398E+00_JPRB ,&
&  0.21596E+00_JPRB, 0.21778E+00_JPRB, 0.21966E+00_JPRB, 0.22158E+00_JPRB, 0.22476E+00_JPRB, 0.22748E+00_JPRB, 0.23174E+00_JPRB ,&
&  0.23594E+00_JPRB, 0.23684E+00_JPRB, 0.25186E+00_JPRB, 0.25289E+00_JPRB, 0.25459E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.23656E+00_JPRB, 0.23864E+00_JPRB, 0.23966E+00_JPRB, 0.24058E+00_JPRB, 0.23553E+00_JPRB, 0.24760E+00_JPRB, 0.24508E+00_JPRB ,&
&  0.24454E+00_JPRB, 0.24028E+00_JPRB, 0.24148E+00_JPRB, 0.24124E+00_JPRB, 0.23999E+00_JPRB, 0.23851E+00_JPRB, 0.23668E+00_JPRB ,&
&  0.23485E+00_JPRB, 0.23241E+00_JPRB, 0.23058E+00_JPRB, 0.22906E+00_JPRB, 0.22766E+00_JPRB, 0.22647E+00_JPRB, 0.22529E+00_JPRB ,&
&  0.22539E+00_JPRB, 0.22637E+00_JPRB, 0.22777E+00_JPRB, 0.22665E+00_JPRB, 0.22696E+00_JPRB, 0.22786E+00_JPRB, 0.22916E+00_JPRB ,&
&  0.23018E+00_JPRB, 0.23181E+00_JPRB, 0.23291E+00_JPRB, 0.23331E+00_JPRB, 0.23441E+00_JPRB, 0.23372E+00_JPRB, 0.23321E+00_JPRB ,&
&  0.23233E+00_JPRB, 0.23108E+00_JPRB, 0.22951E+00_JPRB, 0.22815E+00_JPRB, 0.22666E+00_JPRB, 0.22482E+00_JPRB, 0.22291E+00_JPRB ,&
&  0.22122E+00_JPRB, 0.21974E+00_JPRB, 0.21804E+00_JPRB, 0.21666E+00_JPRB, 0.21535E+00_JPRB, 0.21404E+00_JPRB, 0.21279E+00_JPRB ,&
&  0.21179E+00_JPRB, 0.21090E+00_JPRB, 0.20994E+00_JPRB, 0.20900E+00_JPRB, 0.20799E+00_JPRB, 0.20703E+00_JPRB, 0.20610E+00_JPRB ,&
&  0.20578E+00_JPRB, 0.20552E+00_JPRB, 0.20523E+00_JPRB, 0.20492E+00_JPRB, 0.20495E+00_JPRB, 0.20493E+00_JPRB, 0.20485E+00_JPRB ,&
&  0.20473E+00_JPRB, 0.20447E+00_JPRB, 0.20441E+00_JPRB, 0.20434E+00_JPRB, 0.20426E+00_JPRB, 0.20418E+00_JPRB, 0.20430E+00_JPRB ,&
&  0.20450E+00_JPRB, 0.20478E+00_JPRB, 0.20540E+00_JPRB, 0.20605E+00_JPRB, 0.20672E+00_JPRB, 0.20733E+00_JPRB, 0.20822E+00_JPRB ,&
&  0.20922E+00_JPRB, 0.21016E+00_JPRB, 0.21095E+00_JPRB, 0.21160E+00_JPRB, 0.21280E+00_JPRB, 0.21394E+00_JPRB, 0.21507E+00_JPRB ,&
&  0.21630E+00_JPRB, 0.21788E+00_JPRB, 0.21953E+00_JPRB, 0.22123E+00_JPRB, 0.22326E+00_JPRB, 0.22483E+00_JPRB, 0.22645E+00_JPRB ,&
&  0.22837E+00_JPRB, 0.23208E+00_JPRB, 0.23430E+00_JPRB, 0.23563E+00_JPRB, 0.23689E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.22015E+00_JPRB, 0.22027E+00_JPRB, 0.22025E+00_JPRB, 0.22012E+00_JPRB, 0.21971E+00_JPRB, 0.21957E+00_JPRB, 0.21945E+00_JPRB ,&
&  0.21930E+00_JPRB, 0.21897E+00_JPRB, 0.21920E+00_JPRB, 0.21928E+00_JPRB, 0.21928E+00_JPRB, 0.21925E+00_JPRB, 0.21902E+00_JPRB ,&
&  0.21857E+00_JPRB, 0.21801E+00_JPRB, 0.21662E+00_JPRB, 0.21636E+00_JPRB, 0.21632E+00_JPRB, 0.21654E+00_JPRB, 0.21670E+00_JPRB ,&
&  0.21700E+00_JPRB, 0.21732E+00_JPRB, 0.21759E+00_JPRB, 0.21846E+00_JPRB, 0.21882E+00_JPRB, 0.21922E+00_JPRB, 0.21962E+00_JPRB ,&
&  0.22000E+00_JPRB, 0.22046E+00_JPRB, 0.22082E+00_JPRB, 0.22120E+00_JPRB, 0.22159E+00_JPRB, 0.22155E+00_JPRB, 0.22130E+00_JPRB ,&
&  0.22090E+00_JPRB, 0.22025E+00_JPRB, 0.21957E+00_JPRB, 0.21893E+00_JPRB, 0.21836E+00_JPRB, 0.21777E+00_JPRB, 0.21721E+00_JPRB ,&
&  0.21661E+00_JPRB, 0.21595E+00_JPRB, 0.21525E+00_JPRB, 0.21463E+00_JPRB, 0.21398E+00_JPRB, 0.21341E+00_JPRB, 0.21278E+00_JPRB ,&
&  0.21230E+00_JPRB, 0.21180E+00_JPRB, 0.21133E+00_JPRB, 0.21084E+00_JPRB, 0.21042E+00_JPRB, 0.20998E+00_JPRB, 0.20955E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.20890E+00_JPRB, 0.20860E+00_JPRB, 0.20835E+00_JPRB, 0.20811E+00_JPRB, 0.20794E+00_JPRB, 0.20768E+00_JPRB ,&
&  0.20745E+00_JPRB, 0.20722E+00_JPRB, 0.20715E+00_JPRB, 0.20700E+00_JPRB, 0.20698E+00_JPRB, 0.20696E+00_JPRB, 0.20705E+00_JPRB ,&
&  0.20711E+00_JPRB, 0.20723E+00_JPRB, 0.20743E+00_JPRB, 0.20779E+00_JPRB, 0.20810E+00_JPRB, 0.20843E+00_JPRB, 0.20875E+00_JPRB ,&
&  0.20914E+00_JPRB, 0.20953E+00_JPRB, 0.20994E+00_JPRB, 0.21033E+00_JPRB, 0.21084E+00_JPRB, 0.21136E+00_JPRB, 0.21195E+00_JPRB ,&
&  0.21249E+00_JPRB, 0.21309E+00_JPRB, 0.21363E+00_JPRB, 0.21426E+00_JPRB, 0.21481E+00_JPRB, 0.21514E+00_JPRB, 0.21545E+00_JPRB ,&
&  0.21600E+00_JPRB, 0.21652E+00_JPRB, 0.21713E+00_JPRB, 0.21768E+00_JPRB, 0.21863E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB, 0.21132E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB ,&
&  0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB, 0.81946E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.80776E+00_JPRB, 0.80813E+00_JPRB, 0.80844E+00_JPRB, 0.80875E+00_JPRB, 0.80902E+00_JPRB, 0.80929E+00_JPRB, 0.80960E+00_JPRB ,&
&  0.80979E+00_JPRB, 0.81024E+00_JPRB, 0.81081E+00_JPRB, 0.81147E+00_JPRB, 0.81208E+00_JPRB, 0.81269E+00_JPRB, 0.81330E+00_JPRB ,&
&  0.81389E+00_JPRB, 0.81428E+00_JPRB, 0.81470E+00_JPRB, 0.81535E+00_JPRB, 0.81619E+00_JPRB, 0.81718E+00_JPRB, 0.81860E+00_JPRB ,&
&  0.81918E+00_JPRB, 0.81899E+00_JPRB, 0.81890E+00_JPRB, 0.81972E+00_JPRB, 0.81956E+00_JPRB, 0.81948E+00_JPRB, 0.81950E+00_JPRB ,&
&  0.81944E+00_JPRB, 0.81930E+00_JPRB, 0.81900E+00_JPRB, 0.81870E+00_JPRB, 0.81860E+00_JPRB, 0.81879E+00_JPRB, 0.81907E+00_JPRB ,&
&  0.81950E+00_JPRB, 0.81984E+00_JPRB, 0.82072E+00_JPRB, 0.82165E+00_JPRB, 0.82271E+00_JPRB, 0.82342E+00_JPRB, 0.82426E+00_JPRB ,&
&  0.82481E+00_JPRB, 0.82543E+00_JPRB, 0.82636E+00_JPRB, 0.82762E+00_JPRB, 0.82897E+00_JPRB, 0.83026E+00_JPRB, 0.83122E+00_JPRB ,&
&  0.83265E+00_JPRB, 0.83457E+00_JPRB, 0.83673E+00_JPRB, 0.83904E+00_JPRB, 0.84074E+00_JPRB, 0.84253E+00_JPRB, 0.84451E+00_JPRB ,&
&  0.84617E+00_JPRB, 0.84705E+00_JPRB, 0.84811E+00_JPRB, 0.84921E+00_JPRB, 0.84994E+00_JPRB, 0.84902E+00_JPRB, 0.84829E+00_JPRB ,&
&  0.84765E+00_JPRB, 0.84665E+00_JPRB, 0.84419E+00_JPRB, 0.84213E+00_JPRB, 0.84025E+00_JPRB, 0.83834E+00_JPRB, 0.83618E+00_JPRB ,&
&  0.83423E+00_JPRB, 0.83243E+00_JPRB, 0.83051E+00_JPRB, 0.82844E+00_JPRB, 0.82662E+00_JPRB, 0.82487E+00_JPRB, 0.82320E+00_JPRB ,&
&  0.82162E+00_JPRB, 0.82028E+00_JPRB, 0.81899E+00_JPRB, 0.81782E+00_JPRB, 0.81652E+00_JPRB, 0.81529E+00_JPRB, 0.81403E+00_JPRB ,&
&  0.81289E+00_JPRB, 0.81171E+00_JPRB, 0.81062E+00_JPRB, 0.80955E+00_JPRB, 0.80884E+00_JPRB, 0.80838E+00_JPRB, 0.80798E+00_JPRB ,&
&  0.80767E+00_JPRB, 0.80753E+00_JPRB, 0.80757E+00_JPRB, 0.80756E+00_JPRB, 0.80757E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.80148E+00_JPRB, 0.80245E+00_JPRB, 0.80332E+00_JPRB, 0.80408E+00_JPRB, 0.80503E+00_JPRB, 0.80610E+00_JPRB, 0.80705E+00_JPRB ,&
&  0.80784E+00_JPRB, 0.80898E+00_JPRB, 0.81029E+00_JPRB, 0.81162E+00_JPRB, 0.81286E+00_JPRB, 0.81419E+00_JPRB, 0.81543E+00_JPRB ,&
&  0.81663E+00_JPRB, 0.81778E+00_JPRB, 0.81897E+00_JPRB, 0.82004E+00_JPRB, 0.82106E+00_JPRB, 0.82174E+00_JPRB, 0.82434E+00_JPRB ,&
&  0.82647E+00_JPRB, 0.82738E+00_JPRB, 0.82652E+00_JPRB, 0.81945E+00_JPRB, 0.81433E+00_JPRB, 0.80968E+00_JPRB, 0.80580E+00_JPRB ,&
&  0.80178E+00_JPRB, 0.80168E+00_JPRB, 0.80223E+00_JPRB, 0.80289E+00_JPRB, 0.80619E+00_JPRB, 0.80558E+00_JPRB, 0.80467E+00_JPRB ,&
&  0.80239E+00_JPRB, 0.79669E+00_JPRB, 0.79765E+00_JPRB, 0.79881E+00_JPRB, 0.80000E+00_JPRB, 0.80319E+00_JPRB, 0.80559E+00_JPRB ,&
&  0.80845E+00_JPRB, 0.81152E+00_JPRB, 0.81714E+00_JPRB, 0.82324E+00_JPRB, 0.82878E+00_JPRB, 0.83345E+00_JPRB, 0.83776E+00_JPRB ,&
&  0.84160E+00_JPRB, 0.84568E+00_JPRB, 0.85015E+00_JPRB, 0.85541E+00_JPRB, 0.85955E+00_JPRB, 0.86396E+00_JPRB, 0.86753E+00_JPRB ,&
&  0.86951E+00_JPRB, 0.87008E+00_JPRB, 0.87227E+00_JPRB, 0.87560E+00_JPRB, 0.87898E+00_JPRB, 0.87893E+00_JPRB, 0.87749E+00_JPRB ,&
&  0.87479E+00_JPRB, 0.87210E+00_JPRB, 0.86776E+00_JPRB, 0.86403E+00_JPRB, 0.86008E+00_JPRB, 0.85609E+00_JPRB, 0.85144E+00_JPRB ,&
&  0.84687E+00_JPRB, 0.84242E+00_JPRB, 0.83892E+00_JPRB, 0.83546E+00_JPRB, 0.83223E+00_JPRB, 0.82918E+00_JPRB, 0.82719E+00_JPRB ,&
&  0.82504E+00_JPRB, 0.82318E+00_JPRB, 0.82119E+00_JPRB, 0.81915E+00_JPRB, 0.81592E+00_JPRB, 0.81270E+00_JPRB, 0.80925E+00_JPRB ,&
&  0.80641E+00_JPRB, 0.80359E+00_JPRB, 0.80129E+00_JPRB, 0.79937E+00_JPRB, 0.79854E+00_JPRB, 0.79818E+00_JPRB, 0.79808E+00_JPRB ,&
&  0.79810E+00_JPRB, 0.79862E+00_JPRB, 0.79936E+00_JPRB, 0.79999E+00_JPRB, 0.80067E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.80615E+00_JPRB, 0.80761E+00_JPRB, 0.80899E+00_JPRB, 0.80982E+00_JPRB, 0.81140E+00_JPRB, 0.81288E+00_JPRB, 0.81434E+00_JPRB ,&
&  0.81579E+00_JPRB, 0.81683E+00_JPRB, 0.81777E+00_JPRB, 0.81882E+00_JPRB, 0.81977E+00_JPRB, 0.82159E+00_JPRB, 0.82329E+00_JPRB ,&
&  0.82481E+00_JPRB, 0.82581E+00_JPRB, 0.82818E+00_JPRB, 0.83034E+00_JPRB, 0.83152E+00_JPRB, 0.83252E+00_JPRB, 0.82704E+00_JPRB ,&
&  0.82360E+00_JPRB, 0.82128E+00_JPRB, 0.81826E+00_JPRB, 0.81116E+00_JPRB, 0.80758E+00_JPRB, 0.78984E+00_JPRB, 0.79652E+00_JPRB ,&
&  0.82192E+00_JPRB, 0.82050E+00_JPRB, 0.81479E+00_JPRB, 0.80504E+00_JPRB, 0.77036E+00_JPRB, 0.75609E+00_JPRB, 0.75376E+00_JPRB ,&
&  0.73577E+00_JPRB, 0.75480E+00_JPRB, 0.76052E+00_JPRB, 0.76307E+00_JPRB, 0.76743E+00_JPRB, 0.76421E+00_JPRB, 0.76446E+00_JPRB ,&
&  0.77634E+00_JPRB, 0.77659E+00_JPRB, 0.78339E+00_JPRB, 0.79193E+00_JPRB, 0.80128E+00_JPRB, 0.81013E+00_JPRB, 0.82028E+00_JPRB ,&
&  0.82840E+00_JPRB, 0.83445E+00_JPRB, 0.83880E+00_JPRB, 0.84046E+00_JPRB, 0.84248E+00_JPRB, 0.84493E+00_JPRB, 0.84981E+00_JPRB ,&
&  0.85720E+00_JPRB, 0.86488E+00_JPRB, 0.86875E+00_JPRB, 0.86718E+00_JPRB, 0.86850E+00_JPRB, 0.86928E+00_JPRB, 0.87009E+00_JPRB ,&
&  0.86944E+00_JPRB, 0.86779E+00_JPRB, 0.86348E+00_JPRB, 0.86232E+00_JPRB, 0.86153E+00_JPRB, 0.85943E+00_JPRB, 0.85609E+00_JPRB ,&
&  0.85195E+00_JPRB, 0.84609E+00_JPRB, 0.84141E+00_JPRB, 0.83777E+00_JPRB, 0.83466E+00_JPRB, 0.83199E+00_JPRB, 0.83112E+00_JPRB ,&
&  0.82898E+00_JPRB, 0.82733E+00_JPRB, 0.82412E+00_JPRB, 0.81952E+00_JPRB, 0.81431E+00_JPRB, 0.80987E+00_JPRB, 0.80515E+00_JPRB ,&
&  0.80237E+00_JPRB, 0.80060E+00_JPRB, 0.79972E+00_JPRB, 0.79837E+00_JPRB, 0.79843E+00_JPRB, 0.79860E+00_JPRB, 0.79942E+00_JPRB ,&
&  0.80027E+00_JPRB, 0.80154E+00_JPRB, 0.80250E+00_JPRB, 0.80375E+00_JPRB, 0.80450E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.80948E+00_JPRB, 0.80956E+00_JPRB, 0.81005E+00_JPRB, 0.81036E+00_JPRB, 0.81162E+00_JPRB, 0.81262E+00_JPRB, 0.81544E+00_JPRB ,&
&  0.81809E+00_JPRB, 0.82079E+00_JPRB, 0.82339E+00_JPRB, 0.82555E+00_JPRB, 0.82772E+00_JPRB, 0.82877E+00_JPRB, 0.82941E+00_JPRB ,&
&  0.83000E+00_JPRB, 0.83026E+00_JPRB, 0.83182E+00_JPRB, 0.83361E+00_JPRB, 0.83420E+00_JPRB, 0.83563E+00_JPRB, 0.82980E+00_JPRB ,&
&  0.82531E+00_JPRB, 0.82222E+00_JPRB, 0.80312E+00_JPRB, 0.81351E+00_JPRB, 0.81232E+00_JPRB, 0.81433E+00_JPRB, 0.81088E+00_JPRB ,&
&  0.78803E+00_JPRB, 0.77791E+00_JPRB, 0.75294E+00_JPRB, 0.73306E+00_JPRB, 0.73653E+00_JPRB, 0.72411E+00_JPRB, 0.72709E+00_JPRB ,&
&  0.72529E+00_JPRB, 0.72024E+00_JPRB, 0.71802E+00_JPRB, 0.71605E+00_JPRB, 0.71897E+00_JPRB, 0.72070E+00_JPRB, 0.72281E+00_JPRB ,&
&  0.73008E+00_JPRB, 0.73622E+00_JPRB, 0.74534E+00_JPRB, 0.74604E+00_JPRB, 0.75891E+00_JPRB, 0.77344E+00_JPRB, 0.78859E+00_JPRB ,&
&  0.79860E+00_JPRB, 0.79762E+00_JPRB, 0.80246E+00_JPRB, 0.80938E+00_JPRB, 0.81513E+00_JPRB, 0.82140E+00_JPRB, 0.83111E+00_JPRB ,&
&  0.84336E+00_JPRB, 0.85447E+00_JPRB, 0.85792E+00_JPRB, 0.85571E+00_JPRB, 0.85198E+00_JPRB, 0.84886E+00_JPRB, 0.84884E+00_JPRB ,&
&  0.84884E+00_JPRB, 0.85207E+00_JPRB, 0.85313E+00_JPRB, 0.85619E+00_JPRB, 0.85795E+00_JPRB, 0.86233E+00_JPRB, 0.86519E+00_JPRB ,&
&  0.86689E+00_JPRB, 0.86691E+00_JPRB, 0.86986E+00_JPRB, 0.86375E+00_JPRB, 0.86244E+00_JPRB, 0.86007E+00_JPRB, 0.85289E+00_JPRB ,&
&  0.84595E+00_JPRB, 0.84044E+00_JPRB, 0.83458E+00_JPRB, 0.82888E+00_JPRB, 0.82353E+00_JPRB, 0.81947E+00_JPRB, 0.81662E+00_JPRB ,&
&  0.81506E+00_JPRB, 0.81313E+00_JPRB, 0.81174E+00_JPRB, 0.80820E+00_JPRB, 0.80787E+00_JPRB, 0.80545E+00_JPRB, 0.80442E+00_JPRB ,&
&  0.80376E+00_JPRB, 0.80504E+00_JPRB, 0.80536E+00_JPRB, 0.80712E+00_JPRB, 0.80887E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.80241E+00_JPRB, 0.80556E+00_JPRB, 0.81145E+00_JPRB, 0.81651E+00_JPRB, 0.82153E+00_JPRB, 0.82555E+00_JPRB, 0.82773E+00_JPRB ,&
&  0.82944E+00_JPRB, 0.83004E+00_JPRB, 0.83074E+00_JPRB, 0.83174E+00_JPRB, 0.83345E+00_JPRB, 0.83486E+00_JPRB, 0.83673E+00_JPRB ,&
&  0.83748E+00_JPRB, 0.83800E+00_JPRB, 0.83827E+00_JPRB, 0.83855E+00_JPRB, 0.83933E+00_JPRB, 0.84158E+00_JPRB, 0.84060E+00_JPRB ,&
&  0.84098E+00_JPRB, 0.83545E+00_JPRB, 0.82140E+00_JPRB, 0.82713E+00_JPRB, 0.82942E+00_JPRB, 0.83131E+00_JPRB, 0.82195E+00_JPRB ,&
&  0.79420E+00_JPRB, 0.75410E+00_JPRB, 0.77839E+00_JPRB, 0.76556E+00_JPRB, 0.75162E+00_JPRB, 0.73688E+00_JPRB, 0.71332E+00_JPRB ,&
&  0.69905E+00_JPRB, 0.68014E+00_JPRB, 0.66594E+00_JPRB, 0.66080E+00_JPRB, 0.65984E+00_JPRB, 0.66611E+00_JPRB, 0.67505E+00_JPRB ,&
&  0.67840E+00_JPRB, 0.68946E+00_JPRB, 0.69664E+00_JPRB, 0.70663E+00_JPRB, 0.71951E+00_JPRB, 0.73637E+00_JPRB, 0.74317E+00_JPRB ,&
&  0.75492E+00_JPRB, 0.75930E+00_JPRB, 0.77064E+00_JPRB, 0.78630E+00_JPRB, 0.80726E+00_JPRB, 0.82028E+00_JPRB, 0.82881E+00_JPRB ,&
&  0.83861E+00_JPRB, 0.84374E+00_JPRB, 0.84662E+00_JPRB, 0.84791E+00_JPRB, 0.85138E+00_JPRB, 0.85525E+00_JPRB, 0.85942E+00_JPRB ,&
&  0.86331E+00_JPRB, 0.86985E+00_JPRB, 0.87688E+00_JPRB, 0.88506E+00_JPRB, 0.88983E+00_JPRB, 0.89849E+00_JPRB, 0.90854E+00_JPRB ,&
&  0.91821E+00_JPRB, 0.92375E+00_JPRB, 0.93420E+00_JPRB, 0.92526E+00_JPRB, 0.92168E+00_JPRB, 0.90492E+00_JPRB, 0.89284E+00_JPRB ,&
&  0.88362E+00_JPRB, 0.87214E+00_JPRB, 0.86222E+00_JPRB, 0.85346E+00_JPRB, 0.84666E+00_JPRB, 0.84083E+00_JPRB, 0.83535E+00_JPRB ,&
&  0.82983E+00_JPRB, 0.82284E+00_JPRB, 0.81874E+00_JPRB, 0.81354E+00_JPRB, 0.81110E+00_JPRB, 0.80695E+00_JPRB, 0.80456E+00_JPRB ,&
&  0.80263E+00_JPRB, 0.80201E+00_JPRB, 0.80107E+00_JPRB, 0.80212E+00_JPRB, 0.80255E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.80551E+00_JPRB, 0.81162E+00_JPRB, 0.81879E+00_JPRB, 0.82480E+00_JPRB, 0.82776E+00_JPRB, 0.83014E+00_JPRB, 0.82833E+00_JPRB ,&
&  0.82726E+00_JPRB, 0.82538E+00_JPRB, 0.82400E+00_JPRB, 0.82426E+00_JPRB, 0.82569E+00_JPRB, 0.82849E+00_JPRB, 0.83186E+00_JPRB ,&
&  0.83497E+00_JPRB, 0.83883E+00_JPRB, 0.84051E+00_JPRB, 0.84256E+00_JPRB, 0.84524E+00_JPRB, 0.85058E+00_JPRB, 0.85302E+00_JPRB ,&
&  0.85589E+00_JPRB, 0.84985E+00_JPRB, 0.83648E+00_JPRB, 0.83339E+00_JPRB, 0.84092E+00_JPRB, 0.82608E+00_JPRB, 0.82585E+00_JPRB ,&
&  0.83572E+00_JPRB, 0.83712E+00_JPRB, 0.82150E+00_JPRB, 0.81048E+00_JPRB, 0.78236E+00_JPRB, 0.70475E+00_JPRB, 0.70480E+00_JPRB ,&
&  0.69101E+00_JPRB, 0.66462E+00_JPRB, 0.65063E+00_JPRB, 0.63816E+00_JPRB, 0.63212E+00_JPRB, 0.64185E+00_JPRB, 0.65749E+00_JPRB ,&
&  0.66037E+00_JPRB, 0.67673E+00_JPRB, 0.68317E+00_JPRB, 0.69421E+00_JPRB, 0.70641E+00_JPRB, 0.72432E+00_JPRB, 0.73384E+00_JPRB ,&
&  0.74426E+00_JPRB, 0.75510E+00_JPRB, 0.77241E+00_JPRB, 0.79496E+00_JPRB, 0.81352E+00_JPRB, 0.82323E+00_JPRB, 0.82225E+00_JPRB ,&
&  0.83338E+00_JPRB, 0.82806E+00_JPRB, 0.83366E+00_JPRB, 0.84076E+00_JPRB, 0.84607E+00_JPRB, 0.85452E+00_JPRB, 0.85814E+00_JPRB ,&
&  0.86985E+00_JPRB, 0.88970E+00_JPRB, 0.91049E+00_JPRB, 0.92168E+00_JPRB, 0.93097E+00_JPRB, 0.94182E+00_JPRB, 0.95855E+00_JPRB ,&
&  0.97441E+00_JPRB, 0.99303E+00_JPRB, 0.10094E+01_JPRB, 0.99198E+00_JPRB, 0.98379E+00_JPRB, 0.96166E+00_JPRB, 0.94532E+00_JPRB ,&
&  0.92571E+00_JPRB, 0.91192E+00_JPRB, 0.88974E+00_JPRB, 0.87761E+00_JPRB, 0.86548E+00_JPRB, 0.85589E+00_JPRB, 0.84619E+00_JPRB ,&
&  0.83797E+00_JPRB, 0.83015E+00_JPRB, 0.82554E+00_JPRB, 0.82286E+00_JPRB, 0.81993E+00_JPRB, 0.81470E+00_JPRB, 0.81033E+00_JPRB ,&
&  0.80374E+00_JPRB, 0.79966E+00_JPRB, 0.79663E+00_JPRB, 0.79695E+00_JPRB, 0.79928E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.79880E+00_JPRB, 0.80720E+00_JPRB, 0.81341E+00_JPRB, 0.82202E+00_JPRB, 0.82170E+00_JPRB, 0.82420E+00_JPRB, 0.81739E+00_JPRB ,&
&  0.81268E+00_JPRB, 0.81003E+00_JPRB, 0.80875E+00_JPRB, 0.81043E+00_JPRB, 0.81350E+00_JPRB, 0.81765E+00_JPRB, 0.82337E+00_JPRB ,&
&  0.82926E+00_JPRB, 0.82919E+00_JPRB, 0.83408E+00_JPRB, 0.84290E+00_JPRB, 0.84689E+00_JPRB, 0.85288E+00_JPRB, 0.85660E+00_JPRB ,&
&  0.85778E+00_JPRB, 0.85586E+00_JPRB, 0.85368E+00_JPRB, 0.84454E+00_JPRB, 0.84192E+00_JPRB, 0.84809E+00_JPRB, 0.85289E+00_JPRB ,&
&  0.83039E+00_JPRB, 0.82066E+00_JPRB, 0.84127E+00_JPRB, 0.84684E+00_JPRB, 0.79728E+00_JPRB, 0.75340E+00_JPRB, 0.73440E+00_JPRB ,&
&  0.70757E+00_JPRB, 0.66684E+00_JPRB, 0.65199E+00_JPRB, 0.64178E+00_JPRB, 0.64451E+00_JPRB, 0.64839E+00_JPRB, 0.65956E+00_JPRB ,&
&  0.66629E+00_JPRB, 0.67523E+00_JPRB, 0.68925E+00_JPRB, 0.70749E+00_JPRB, 0.71906E+00_JPRB, 0.73271E+00_JPRB, 0.74334E+00_JPRB ,&
&  0.75507E+00_JPRB, 0.77104E+00_JPRB, 0.79343E+00_JPRB, 0.82142E+00_JPRB, 0.84228E+00_JPRB, 0.85743E+00_JPRB, 0.86387E+00_JPRB ,&
&  0.89419E+00_JPRB, 0.87916E+00_JPRB, 0.88547E+00_JPRB, 0.88948E+00_JPRB, 0.89198E+00_JPRB, 0.89895E+00_JPRB, 0.90480E+00_JPRB ,&
&  0.92445E+00_JPRB, 0.95344E+00_JPRB, 0.96260E+00_JPRB, 0.96815E+00_JPRB, 0.97280E+00_JPRB, 0.98062E+00_JPRB, 0.10017E+01_JPRB ,&
&  0.10216E+01_JPRB, 0.10574E+01_JPRB, 0.11407E+01_JPRB, 0.11024E+01_JPRB, 0.10725E+01_JPRB, 0.10393E+01_JPRB, 0.10044E+01_JPRB ,&
&  0.97213E+00_JPRB, 0.94298E+00_JPRB, 0.91425E+00_JPRB, 0.88859E+00_JPRB, 0.87089E+00_JPRB, 0.85379E+00_JPRB, 0.84888E+00_JPRB ,&
&  0.84492E+00_JPRB, 0.83941E+00_JPRB, 0.83591E+00_JPRB, 0.83259E+00_JPRB, 0.82514E+00_JPRB, 0.81752E+00_JPRB, 0.81182E+00_JPRB ,&
&  0.80167E+00_JPRB, 0.79613E+00_JPRB, 0.79141E+00_JPRB, 0.79161E+00_JPRB, 0.79326E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.79136E+00_JPRB, 0.80850E+00_JPRB, 0.82034E+00_JPRB, 0.82715E+00_JPRB, 0.82194E+00_JPRB, 0.82345E+00_JPRB, 0.81562E+00_JPRB ,&
&  0.81108E+00_JPRB, 0.81140E+00_JPRB, 0.81100E+00_JPRB, 0.81162E+00_JPRB, 0.81275E+00_JPRB, 0.81366E+00_JPRB, 0.81600E+00_JPRB ,&
&  0.82012E+00_JPRB, 0.82548E+00_JPRB, 0.82981E+00_JPRB, 0.83597E+00_JPRB, 0.84400E+00_JPRB, 0.85331E+00_JPRB, 0.86106E+00_JPRB ,&
&  0.86349E+00_JPRB, 0.86146E+00_JPRB, 0.86628E+00_JPRB, 0.86213E+00_JPRB, 0.85802E+00_JPRB, 0.85220E+00_JPRB, 0.85000E+00_JPRB ,&
&  0.85984E+00_JPRB, 0.87134E+00_JPRB, 0.85770E+00_JPRB, 0.84984E+00_JPRB, 0.84466E+00_JPRB, 0.80399E+00_JPRB, 0.76414E+00_JPRB ,&
&  0.72297E+00_JPRB, 0.68752E+00_JPRB, 0.67093E+00_JPRB, 0.66230E+00_JPRB, 0.66898E+00_JPRB, 0.68436E+00_JPRB, 0.69738E+00_JPRB ,&
&  0.68981E+00_JPRB, 0.71004E+00_JPRB, 0.73063E+00_JPRB, 0.74343E+00_JPRB, 0.75338E+00_JPRB, 0.76368E+00_JPRB, 0.77376E+00_JPRB ,&
&  0.78969E+00_JPRB, 0.81500E+00_JPRB, 0.86098E+00_JPRB, 0.91270E+00_JPRB, 0.95190E+00_JPRB, 0.96764E+00_JPRB, 0.10227E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.10738E+01_JPRB, 0.10355E+01_JPRB, 0.10224E+01_JPRB, 0.10070E+01_JPRB, 0.10027E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.10491E+01_JPRB, 0.10523E+01_JPRB, 0.10488E+01_JPRB, 0.10506E+01_JPRB, 0.10593E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.11621E+01_JPRB, 0.13068E+01_JPRB, 0.11690E+01_JPRB, 0.10958E+01_JPRB, 0.10585E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.99009E+00_JPRB, 0.96225E+00_JPRB, 0.93319E+00_JPRB, 0.89842E+00_JPRB, 0.87169E+00_JPRB, 0.84686E+00_JPRB, 0.83933E+00_JPRB ,&
&  0.83454E+00_JPRB, 0.82878E+00_JPRB, 0.82840E+00_JPRB, 0.82360E+00_JPRB, 0.81686E+00_JPRB, 0.81502E+00_JPRB, 0.81054E+00_JPRB ,&
&  0.79930E+00_JPRB, 0.78924E+00_JPRB, 0.78182E+00_JPRB, 0.77982E+00_JPRB, 0.78119E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.81283E+00_JPRB, 0.82204E+00_JPRB, 0.83164E+00_JPRB, 0.85369E+00_JPRB, 0.85995E+00_JPRB, 0.85538E+00_JPRB, 0.83676E+00_JPRB ,&
&  0.82455E+00_JPRB, 0.82011E+00_JPRB, 0.82356E+00_JPRB, 0.82801E+00_JPRB, 0.83031E+00_JPRB, 0.83047E+00_JPRB, 0.82810E+00_JPRB ,&
&  0.82391E+00_JPRB, 0.82370E+00_JPRB, 0.82709E+00_JPRB, 0.83258E+00_JPRB, 0.83989E+00_JPRB, 0.84828E+00_JPRB, 0.85831E+00_JPRB ,&
&  0.86643E+00_JPRB, 0.86845E+00_JPRB, 0.87037E+00_JPRB, 0.86920E+00_JPRB, 0.87167E+00_JPRB, 0.87730E+00_JPRB, 0.87531E+00_JPRB ,&
&  0.86433E+00_JPRB, 0.87383E+00_JPRB, 0.88436E+00_JPRB, 0.86795E+00_JPRB, 0.85969E+00_JPRB, 0.82955E+00_JPRB, 0.79059E+00_JPRB ,&
&  0.75763E+00_JPRB, 0.72482E+00_JPRB, 0.71638E+00_JPRB, 0.71814E+00_JPRB, 0.72569E+00_JPRB, 0.72057E+00_JPRB, 0.74149E+00_JPRB ,&
&  0.75848E+00_JPRB, 0.76930E+00_JPRB, 0.77789E+00_JPRB, 0.78474E+00_JPRB, 0.79208E+00_JPRB, 0.80472E+00_JPRB, 0.83096E+00_JPRB ,&
&  0.87899E+00_JPRB, 0.94206E+00_JPRB, 0.10099E+01_JPRB, 0.99233E+00_JPRB, 0.10066E+01_JPRB, 0.10428E+01_JPRB, 0.11033E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.11461E+01_JPRB, 0.11448E+01_JPRB, 0.11903E+01_JPRB, 0.12178E+01_JPRB, 0.12468E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12239E+01_JPRB, 0.11985E+01_JPRB, 0.11802E+01_JPRB, 0.11799E+01_JPRB, 0.11989E+01_JPRB, 0.12144E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.11246E+01_JPRB, 0.10648E+01_JPRB, 0.10092E+01_JPRB, 0.99633E+00_JPRB, 0.10007E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.98381E+00_JPRB, 0.96005E+00_JPRB, 0.93520E+00_JPRB, 0.90169E+00_JPRB, 0.86964E+00_JPRB, 0.83568E+00_JPRB, 0.82122E+00_JPRB ,&
&  0.81289E+00_JPRB, 0.80814E+00_JPRB, 0.81317E+00_JPRB, 0.79966E+00_JPRB, 0.79914E+00_JPRB, 0.80108E+00_JPRB, 0.78927E+00_JPRB ,&
&  0.77977E+00_JPRB, 0.77711E+00_JPRB, 0.77009E+00_JPRB, 0.77557E+00_JPRB, 0.78745E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.84175E+00_JPRB, 0.87292E+00_JPRB, 0.90112E+00_JPRB, 0.91318E+00_JPRB, 0.91991E+00_JPRB, 0.91860E+00_JPRB, 0.89927E+00_JPRB ,&
&  0.89167E+00_JPRB, 0.86452E+00_JPRB, 0.84436E+00_JPRB, 0.82461E+00_JPRB, 0.81022E+00_JPRB, 0.80335E+00_JPRB, 0.80716E+00_JPRB ,&
&  0.81386E+00_JPRB, 0.81990E+00_JPRB, 0.82425E+00_JPRB, 0.82743E+00_JPRB, 0.83284E+00_JPRB, 0.84115E+00_JPRB, 0.85257E+00_JPRB ,&
&  0.86477E+00_JPRB, 0.87345E+00_JPRB, 0.88045E+00_JPRB, 0.88555E+00_JPRB, 0.88916E+00_JPRB, 0.88991E+00_JPRB, 0.88412E+00_JPRB ,&
&  0.88528E+00_JPRB, 0.88522E+00_JPRB, 0.88786E+00_JPRB, 0.87823E+00_JPRB, 0.87129E+00_JPRB, 0.84785E+00_JPRB, 0.80601E+00_JPRB ,&
&  0.78635E+00_JPRB, 0.77534E+00_JPRB, 0.78077E+00_JPRB, 0.77686E+00_JPRB, 0.78765E+00_JPRB, 0.80234E+00_JPRB, 0.80576E+00_JPRB ,&
&  0.80438E+00_JPRB, 0.80552E+00_JPRB, 0.81788E+00_JPRB, 0.82613E+00_JPRB, 0.83772E+00_JPRB, 0.85660E+00_JPRB, 0.87887E+00_JPRB ,&
&  0.91947E+00_JPRB, 0.98072E+00_JPRB, 0.10738E+01_JPRB, 0.11060E+01_JPRB, 0.11872E+01_JPRB, 0.13710E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13731E+01_JPRB, 0.14326E+01_JPRB, 0.15273E+01_JPRB, 0.15110E+01_JPRB, 0.14951E+01_JPRB, 0.14795E+01_JPRB ,&
&  0.14209E+01_JPRB, 0.13991E+01_JPRB, 0.13764E+01_JPRB, 0.14027E+01_JPRB, 0.13905E+01_JPRB, 0.13739E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.12114E+01_JPRB, 0.11350E+01_JPRB, 0.10635E+01_JPRB, 0.10269E+01_JPRB, 0.10059E+01_JPRB, 0.99235E+00_JPRB ,&
&  0.97485E+00_JPRB, 0.96063E+00_JPRB, 0.94748E+00_JPRB, 0.92638E+00_JPRB, 0.88977E+00_JPRB, 0.84795E+00_JPRB, 0.83409E+00_JPRB ,&
&  0.82236E+00_JPRB, 0.81121E+00_JPRB, 0.80543E+00_JPRB, 0.79836E+00_JPRB, 0.79965E+00_JPRB, 0.79444E+00_JPRB, 0.78586E+00_JPRB ,&
&  0.78853E+00_JPRB, 0.78134E+00_JPRB, 0.77548E+00_JPRB, 0.78131E+00_JPRB, 0.81135E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.89993E+00_JPRB, 0.93857E+00_JPRB, 0.95381E+00_JPRB, 0.96346E+00_JPRB, 0.97208E+00_JPRB, 0.97110E+00_JPRB, 0.93451E+00_JPRB ,&
&  0.92064E+00_JPRB, 0.86386E+00_JPRB, 0.83662E+00_JPRB, 0.81863E+00_JPRB, 0.80991E+00_JPRB, 0.79970E+00_JPRB, 0.78974E+00_JPRB ,&
&  0.79411E+00_JPRB, 0.80873E+00_JPRB, 0.82324E+00_JPRB, 0.82997E+00_JPRB, 0.83639E+00_JPRB, 0.84184E+00_JPRB, 0.84889E+00_JPRB ,&
&  0.85977E+00_JPRB, 0.87246E+00_JPRB, 0.88677E+00_JPRB, 0.89332E+00_JPRB, 0.89607E+00_JPRB, 0.89699E+00_JPRB, 0.89583E+00_JPRB ,&
&  0.89477E+00_JPRB, 0.88498E+00_JPRB, 0.88783E+00_JPRB, 0.88423E+00_JPRB, 0.87308E+00_JPRB, 0.86858E+00_JPRB, 0.83909E+00_JPRB ,&
&  0.81612E+00_JPRB, 0.83495E+00_JPRB, 0.83507E+00_JPRB, 0.84916E+00_JPRB, 0.85785E+00_JPRB, 0.86210E+00_JPRB, 0.86301E+00_JPRB ,&
&  0.86022E+00_JPRB, 0.85716E+00_JPRB, 0.85941E+00_JPRB, 0.86487E+00_JPRB, 0.87320E+00_JPRB, 0.88528E+00_JPRB, 0.89745E+00_JPRB ,&
&  0.90920E+00_JPRB, 0.89294E+00_JPRB, 0.95716E+00_JPRB, 0.11270E+01_JPRB, 0.14667E+01_JPRB, 0.14999E+01_JPRB, 0.15575E+01_JPRB ,&
&  0.16674E+01_JPRB, 0.16743E+01_JPRB, 0.17205E+01_JPRB, 0.17597E+01_JPRB, 0.17606E+01_JPRB, 0.17425E+01_JPRB, 0.17444E+01_JPRB ,&
&  0.16890E+01_JPRB, 0.17045E+01_JPRB, 0.16731E+01_JPRB, 0.16580E+01_JPRB, 0.16098E+01_JPRB, 0.15791E+01_JPRB, 0.15313E+01_JPRB ,&
&  0.14622E+01_JPRB, 0.13987E+01_JPRB, 0.13051E+01_JPRB, 0.11999E+01_JPRB, 0.11140E+01_JPRB, 0.10689E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.10021E+01_JPRB, 0.98660E+00_JPRB, 0.96804E+00_JPRB, 0.95774E+00_JPRB, 0.93331E+00_JPRB, 0.90632E+00_JPRB, 0.90048E+00_JPRB ,&
&  0.87402E+00_JPRB, 0.85459E+00_JPRB, 0.83838E+00_JPRB, 0.82984E+00_JPRB, 0.83709E+00_JPRB, 0.84188E+00_JPRB, 0.84253E+00_JPRB ,&
&  0.85140E+00_JPRB, 0.84393E+00_JPRB, 0.84520E+00_JPRB, 0.85546E+00_JPRB, 0.87295E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.95407E+00_JPRB, 0.98252E+00_JPRB, 0.99502E+00_JPRB, 0.10082E+01_JPRB, 0.10252E+01_JPRB, 0.10229E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.98456E+00_JPRB, 0.10153E+01_JPRB, 0.88910E+00_JPRB, 0.83941E+00_JPRB, 0.82719E+00_JPRB, 0.81222E+00_JPRB ,&
&  0.81098E+00_JPRB, 0.81843E+00_JPRB, 0.82716E+00_JPRB, 0.83686E+00_JPRB, 0.84668E+00_JPRB, 0.84906E+00_JPRB, 0.85188E+00_JPRB ,&
&  0.85871E+00_JPRB, 0.87187E+00_JPRB, 0.89096E+00_JPRB, 0.90077E+00_JPRB, 0.90959E+00_JPRB, 0.91802E+00_JPRB, 0.91305E+00_JPRB ,&
&  0.90336E+00_JPRB, 0.90640E+00_JPRB, 0.90441E+00_JPRB, 0.89141E+00_JPRB, 0.87749E+00_JPRB, 0.87428E+00_JPRB, 0.88863E+00_JPRB ,&
&  0.88635E+00_JPRB, 0.87922E+00_JPRB, 0.89618E+00_JPRB, 0.90269E+00_JPRB, 0.90349E+00_JPRB, 0.90135E+00_JPRB, 0.89886E+00_JPRB ,&
&  0.89750E+00_JPRB, 0.89461E+00_JPRB, 0.89714E+00_JPRB, 0.89901E+00_JPRB, 0.90353E+00_JPRB, 0.91136E+00_JPRB, 0.91593E+00_JPRB ,&
&  0.93232E+00_JPRB, 0.92643E+00_JPRB, 0.10630E+01_JPRB, 0.12253E+01_JPRB, 0.15230E+01_JPRB, 0.17051E+01_JPRB, 0.17963E+01_JPRB ,&
&  0.20522E+01_JPRB, 0.19969E+01_JPRB, 0.20986E+01_JPRB, 0.20889E+01_JPRB, 0.20620E+01_JPRB, 0.20147E+01_JPRB, 0.19498E+01_JPRB ,&
&  0.19318E+01_JPRB, 0.19012E+01_JPRB, 0.19732E+01_JPRB, 0.19276E+01_JPRB, 0.18298E+01_JPRB, 0.17807E+01_JPRB, 0.17399E+01_JPRB ,&
&  0.17103E+01_JPRB, 0.16750E+01_JPRB, 0.15958E+01_JPRB, 0.13904E+01_JPRB, 0.12893E+01_JPRB, 0.12068E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.10005E+01_JPRB, 0.96282E+00_JPRB, 0.95641E+00_JPRB, 0.93886E+00_JPRB, 0.92170E+00_JPRB, 0.92042E+00_JPRB ,&
&  0.90082E+00_JPRB, 0.89195E+00_JPRB, 0.88433E+00_JPRB, 0.89118E+00_JPRB, 0.90023E+00_JPRB, 0.90698E+00_JPRB, 0.90889E+00_JPRB ,&
&  0.90057E+00_JPRB, 0.88980E+00_JPRB, 0.90134E+00_JPRB, 0.90731E+00_JPRB, 0.92630E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.98741E+00_JPRB, 0.10165E+01_JPRB, 0.10381E+01_JPRB, 0.10574E+01_JPRB, 0.10830E+01_JPRB, 0.11062E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.11004E+01_JPRB, 0.10984E+01_JPRB, 0.10792E+01_JPRB, 0.10429E+01_JPRB, 0.95197E+00_JPRB, 0.84442E+00_JPRB ,&
&  0.81358E+00_JPRB, 0.82127E+00_JPRB, 0.83468E+00_JPRB, 0.84130E+00_JPRB, 0.84591E+00_JPRB, 0.85145E+00_JPRB, 0.85359E+00_JPRB ,&
&  0.86065E+00_JPRB, 0.87156E+00_JPRB, 0.88978E+00_JPRB, 0.91368E+00_JPRB, 0.93584E+00_JPRB, 0.94560E+00_JPRB, 0.94260E+00_JPRB ,&
&  0.92922E+00_JPRB, 0.92453E+00_JPRB, 0.90979E+00_JPRB, 0.89487E+00_JPRB, 0.89517E+00_JPRB, 0.90238E+00_JPRB, 0.92259E+00_JPRB ,&
&  0.94116E+00_JPRB, 0.95064E+00_JPRB, 0.95267E+00_JPRB, 0.95515E+00_JPRB, 0.95473E+00_JPRB, 0.94838E+00_JPRB, 0.93888E+00_JPRB ,&
&  0.93069E+00_JPRB, 0.92498E+00_JPRB, 0.92167E+00_JPRB, 0.92421E+00_JPRB, 0.92222E+00_JPRB, 0.94663E+00_JPRB, 0.97413E+00_JPRB ,&
&  0.10031E+01_JPRB, 0.11054E+01_JPRB, 0.12733E+01_JPRB, 0.13397E+01_JPRB, 0.16207E+01_JPRB, 0.18072E+01_JPRB, 0.19895E+01_JPRB ,&
&  0.21314E+01_JPRB, 0.20936E+01_JPRB, 0.23078E+01_JPRB, 0.23749E+01_JPRB, 0.24000E+01_JPRB, 0.23458E+01_JPRB, 0.22122E+01_JPRB ,&
&  0.21424E+01_JPRB, 0.22197E+01_JPRB, 0.22715E+01_JPRB, 0.22445E+01_JPRB, 0.21514E+01_JPRB, 0.21118E+01_JPRB, 0.20779E+01_JPRB ,&
&  0.20913E+01_JPRB, 0.21006E+01_JPRB, 0.20091E+01_JPRB, 0.18274E+01_JPRB, 0.16063E+01_JPRB, 0.14097E+01_JPRB, 0.12504E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.10033E+01_JPRB, 0.95989E+00_JPRB, 0.94961E+00_JPRB, 0.94989E+00_JPRB, 0.92905E+00_JPRB, 0.92733E+00_JPRB ,&
&  0.93581E+00_JPRB, 0.93867E+00_JPRB, 0.94249E+00_JPRB, 0.95007E+00_JPRB, 0.97871E+00_JPRB, 0.99286E+00_JPRB, 0.95122E+00_JPRB ,&
&  0.92246E+00_JPRB, 0.91875E+00_JPRB, 0.92420E+00_JPRB, 0.93228E+00_JPRB, 0.95702E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.10343E+01_JPRB, 0.10684E+01_JPRB, 0.10923E+01_JPRB, 0.11105E+01_JPRB, 0.11371E+01_JPRB, 0.11765E+01_JPRB, 0.11841E+01_JPRB ,&
&  0.11750E+01_JPRB, 0.11660E+01_JPRB, 0.11607E+01_JPRB, 0.11488E+01_JPRB, 0.11214E+01_JPRB, 0.10753E+01_JPRB, 0.97287E+00_JPRB ,&
&  0.86387E+00_JPRB, 0.85811E+00_JPRB, 0.84089E+00_JPRB, 0.84730E+00_JPRB, 0.85468E+00_JPRB, 0.85302E+00_JPRB, 0.84933E+00_JPRB ,&
&  0.85799E+00_JPRB, 0.86924E+00_JPRB, 0.88982E+00_JPRB, 0.91517E+00_JPRB, 0.92330E+00_JPRB, 0.93239E+00_JPRB, 0.94701E+00_JPRB ,&
&  0.93748E+00_JPRB, 0.91983E+00_JPRB, 0.90583E+00_JPRB, 0.89879E+00_JPRB, 0.91616E+00_JPRB, 0.94392E+00_JPRB, 0.96948E+00_JPRB ,&
&  0.98329E+00_JPRB, 0.98668E+00_JPRB, 0.98751E+00_JPRB, 0.98796E+00_JPRB, 0.98687E+00_JPRB, 0.97987E+00_JPRB, 0.96673E+00_JPRB ,&
&  0.95188E+00_JPRB, 0.94331E+00_JPRB, 0.93934E+00_JPRB, 0.94548E+00_JPRB, 0.95845E+00_JPRB, 0.10149E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.12726E+01_JPRB, 0.13894E+01_JPRB, 0.15844E+01_JPRB, 0.18397E+01_JPRB, 0.20548E+01_JPRB, 0.22124E+01_JPRB ,&
&  0.23773E+01_JPRB, 0.23766E+01_JPRB, 0.26935E+01_JPRB, 0.27148E+01_JPRB, 0.26190E+01_JPRB, 0.25901E+01_JPRB, 0.24347E+01_JPRB ,&
&  0.23047E+01_JPRB, 0.21936E+01_JPRB, 0.23133E+01_JPRB, 0.23742E+01_JPRB, 0.24089E+01_JPRB, 0.24607E+01_JPRB, 0.24617E+01_JPRB ,&
&  0.24721E+01_JPRB, 0.24450E+01_JPRB, 0.21422E+01_JPRB, 0.18363E+01_JPRB, 0.15142E+01_JPRB, 0.14671E+01_JPRB, 0.13157E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.99900E+00_JPRB, 0.98239E+00_JPRB, 0.97233E+00_JPRB, 0.98953E+00_JPRB, 0.98260E+00_JPRB, 0.98868E+00_JPRB ,&
&  0.10099E+01_JPRB, 0.10334E+01_JPRB, 0.10359E+01_JPRB, 0.10555E+01_JPRB, 0.10910E+01_JPRB, 0.10931E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10025E+01_JPRB, 0.98663E+00_JPRB, 0.96185E+00_JPRB, 0.96472E+00_JPRB, 0.99236E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.11008E+01_JPRB, 0.11280E+01_JPRB, 0.11473E+01_JPRB, 0.11695E+01_JPRB, 0.12023E+01_JPRB, 0.12383E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.12280E+01_JPRB, 0.12175E+01_JPRB, 0.12082E+01_JPRB, 0.11909E+01_JPRB, 0.11674E+01_JPRB, 0.11214E+01_JPRB, 0.10690E+01_JPRB ,&
&  0.93045E+00_JPRB, 0.89542E+00_JPRB, 0.83806E+00_JPRB, 0.83912E+00_JPRB, 0.89908E+00_JPRB, 0.88322E+00_JPRB, 0.87993E+00_JPRB ,&
&  0.87282E+00_JPRB, 0.88467E+00_JPRB, 0.89669E+00_JPRB, 0.91186E+00_JPRB, 0.90578E+00_JPRB, 0.92126E+00_JPRB, 0.94524E+00_JPRB ,&
&  0.94127E+00_JPRB, 0.93128E+00_JPRB, 0.93155E+00_JPRB, 0.93844E+00_JPRB, 0.96134E+00_JPRB, 0.99576E+00_JPRB, 0.10305E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.10430E+01_JPRB, 0.10404E+01_JPRB, 0.10371E+01_JPRB, 0.10317E+01_JPRB, 0.10211E+01_JPRB, 0.10059E+01_JPRB ,&
&  0.98870E+00_JPRB, 0.97628E+00_JPRB, 0.97845E+00_JPRB, 0.99574E+00_JPRB, 0.10057E+01_JPRB, 0.10866E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.14493E+01_JPRB, 0.15364E+01_JPRB, 0.16815E+01_JPRB, 0.20749E+01_JPRB, 0.23617E+01_JPRB, 0.25392E+01_JPRB, 0.26010E+01_JPRB ,&
&  0.26533E+01_JPRB, 0.26661E+01_JPRB, 0.26740E+01_JPRB, 0.27284E+01_JPRB, 0.26546E+01_JPRB, 0.26889E+01_JPRB, 0.25730E+01_JPRB ,&
&  0.23695E+01_JPRB, 0.23364E+01_JPRB, 0.24488E+01_JPRB, 0.25146E+01_JPRB, 0.25401E+01_JPRB, 0.26195E+01_JPRB, 0.26895E+01_JPRB ,&
&  0.26499E+01_JPRB, 0.23648E+01_JPRB, 0.18276E+01_JPRB, 0.16544E+01_JPRB, 0.12825E+01_JPRB, 0.12531E+01_JPRB, 0.13762E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.11955E+01_JPRB, 0.10930E+01_JPRB, 0.10491E+01_JPRB, 0.10556E+01_JPRB, 0.11747E+01_JPRB, 0.12022E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.11533E+01_JPRB, 0.11912E+01_JPRB, 0.12421E+01_JPRB, 0.12852E+01_JPRB, 0.12834E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.11872E+01_JPRB, 0.11280E+01_JPRB, 0.10753E+01_JPRB, 0.10584E+01_JPRB, 0.10712E+01_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.12181E+01_JPRB, 0.12216E+01_JPRB, 0.12269E+01_JPRB, 0.12394E+01_JPRB, 0.12737E+01_JPRB, 0.12978E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.12724E+01_JPRB, 0.12614E+01_JPRB, 0.12467E+01_JPRB, 0.12258E+01_JPRB, 0.11874E+01_JPRB, 0.11271E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.98897E+00_JPRB, 0.90897E+00_JPRB, 0.85308E+00_JPRB, 0.88959E+00_JPRB, 0.88906E+00_JPRB, 0.90663E+00_JPRB ,&
&  0.90263E+00_JPRB, 0.92788E+00_JPRB, 0.93279E+00_JPRB, 0.92779E+00_JPRB, 0.91957E+00_JPRB, 0.95172E+00_JPRB, 0.99443E+00_JPRB ,&
&  0.10239E+01_JPRB, 0.10351E+01_JPRB, 0.10370E+01_JPRB, 0.10574E+01_JPRB, 0.10976E+01_JPRB, 0.10639E+01_JPRB, 0.10951E+01_JPRB ,&
&  0.10995E+01_JPRB, 0.10976E+01_JPRB, 0.10924E+01_JPRB, 0.10948E+01_JPRB, 0.10906E+01_JPRB, 0.10896E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.10953E+01_JPRB, 0.10687E+01_JPRB, 0.10650E+01_JPRB, 0.10839E+01_JPRB, 0.12017E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.14480E+01_JPRB, 0.14957E+01_JPRB, 0.16020E+01_JPRB, 0.20458E+01_JPRB, 0.24064E+01_JPRB, 0.25505E+01_JPRB, 0.26879E+01_JPRB ,&
&  0.27664E+01_JPRB, 0.28017E+01_JPRB, 0.28648E+01_JPRB, 0.28809E+01_JPRB, 0.29003E+01_JPRB, 0.28887E+01_JPRB, 0.28279E+01_JPRB ,&
&  0.26629E+01_JPRB, 0.26075E+01_JPRB, 0.26897E+01_JPRB, 0.26323E+01_JPRB, 0.25977E+01_JPRB, 0.24913E+01_JPRB, 0.26684E+01_JPRB ,&
&  0.26752E+01_JPRB, 0.17091E+01_JPRB, 0.14105E+01_JPRB, 0.15873E+01_JPRB, 0.13411E+01_JPRB, 0.12927E+01_JPRB, 0.13620E+01_JPRB ,&
&  0.12997E+01_JPRB, 0.12326E+01_JPRB, 0.12061E+01_JPRB, 0.11673E+01_JPRB, 0.11985E+01_JPRB, 0.14731E+01_JPRB, 0.15066E+01_JPRB ,&
&  0.14407E+01_JPRB, 0.14193E+01_JPRB, 0.14931E+01_JPRB, 0.15107E+01_JPRB, 0.15041E+01_JPRB, 0.15147E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.12977E+01_JPRB, 0.12388E+01_JPRB, 0.12148E+01_JPRB, 0.12145E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.13803E+01_JPRB, 0.13498E+01_JPRB, 0.13256E+01_JPRB, 0.13263E+01_JPRB, 0.13530E+01_JPRB, 0.13651E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.13285E+01_JPRB, 0.13089E+01_JPRB, 0.12873E+01_JPRB, 0.12520E+01_JPRB, 0.11964E+01_JPRB, 0.11339E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.10277E+01_JPRB, 0.92674E+00_JPRB, 0.88440E+00_JPRB, 0.85156E+00_JPRB, 0.87435E+00_JPRB, 0.90273E+00_JPRB ,&
&  0.93280E+00_JPRB, 0.94759E+00_JPRB, 0.95824E+00_JPRB, 0.96734E+00_JPRB, 0.97980E+00_JPRB, 0.10358E+01_JPRB, 0.11204E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.11955E+01_JPRB, 0.11610E+01_JPRB, 0.12137E+01_JPRB, 0.12000E+01_JPRB, 0.11485E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.11913E+01_JPRB, 0.11780E+01_JPRB, 0.11845E+01_JPRB, 0.11883E+01_JPRB, 0.11659E+01_JPRB, 0.11395E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.10870E+01_JPRB, 0.10833E+01_JPRB, 0.10770E+01_JPRB, 0.10918E+01_JPRB, 0.11675E+01_JPRB, 0.12017E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.11339E+01_JPRB, 0.11132E+01_JPRB, 0.15129E+01_JPRB, 0.21412E+01_JPRB, 0.24702E+01_JPRB, 0.24762E+01_JPRB ,&
&  0.27027E+01_JPRB, 0.28528E+01_JPRB, 0.30289E+01_JPRB, 0.31325E+01_JPRB, 0.31649E+01_JPRB, 0.30865E+01_JPRB, 0.29859E+01_JPRB ,&
&  0.28869E+01_JPRB, 0.28540E+01_JPRB, 0.28424E+01_JPRB, 0.28301E+01_JPRB, 0.27792E+01_JPRB, 0.28486E+01_JPRB, 0.30308E+01_JPRB ,&
&  0.28590E+01_JPRB, 0.25159E+01_JPRB, 0.17666E+01_JPRB, 0.17495E+01_JPRB, 0.13846E+01_JPRB, 0.13224E+01_JPRB, 0.14058E+01_JPRB ,&
&  0.13398E+01_JPRB, 0.13528E+01_JPRB, 0.13312E+01_JPRB, 0.12965E+01_JPRB, 0.14639E+01_JPRB, 0.17804E+01_JPRB, 0.18611E+01_JPRB ,&
&  0.19158E+01_JPRB, 0.18060E+01_JPRB, 0.19470E+01_JPRB, 0.19641E+01_JPRB, 0.19378E+01_JPRB, 0.19002E+01_JPRB, 0.18268E+01_JPRB ,&
&  0.17048E+01_JPRB, 0.15732E+01_JPRB, 0.14769E+01_JPRB, 0.14278E+01_JPRB, 0.13991E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.14848E+01_JPRB, 0.14245E+01_JPRB, 0.13862E+01_JPRB, 0.13792E+01_JPRB, 0.13971E+01_JPRB, 0.14019E+01_JPRB, 0.13762E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.13293E+01_JPRB, 0.13054E+01_JPRB, 0.12494E+01_JPRB, 0.11970E+01_JPRB, 0.11317E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.99162E+00_JPRB, 0.90353E+00_JPRB, 0.86605E+00_JPRB, 0.83566E+00_JPRB, 0.84323E+00_JPRB, 0.88058E+00_JPRB ,&
&  0.92757E+00_JPRB, 0.96583E+00_JPRB, 0.10292E+01_JPRB, 0.10572E+01_JPRB, 0.11016E+01_JPRB, 0.11913E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.14467E+01_JPRB, 0.13035E+01_JPRB, 0.13253E+01_JPRB, 0.13022E+01_JPRB, 0.12984E+01_JPRB, 0.12730E+01_JPRB, 0.12435E+01_JPRB ,&
&  0.12293E+01_JPRB, 0.12206E+01_JPRB, 0.12291E+01_JPRB, 0.12225E+01_JPRB, 0.12036E+01_JPRB, 0.11810E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.11546E+01_JPRB, 0.11385E+01_JPRB, 0.11085E+01_JPRB, 0.10756E+01_JPRB, 0.11449E+01_JPRB, 0.12089E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.10642E+01_JPRB, 0.11209E+01_JPRB, 0.14375E+01_JPRB, 0.18117E+01_JPRB, 0.21998E+01_JPRB, 0.25824E+01_JPRB, 0.28269E+01_JPRB ,&
&  0.30312E+01_JPRB, 0.31356E+01_JPRB, 0.32259E+01_JPRB, 0.29132E+01_JPRB, 0.31776E+01_JPRB, 0.31399E+01_JPRB, 0.30256E+01_JPRB ,&
&  0.29067E+01_JPRB, 0.29232E+01_JPRB, 0.28605E+01_JPRB, 0.29733E+01_JPRB, 0.31869E+01_JPRB, 0.35103E+01_JPRB, 0.31420E+01_JPRB ,&
&  0.26983E+01_JPRB, 0.28844E+01_JPRB, 0.25066E+01_JPRB, 0.20252E+01_JPRB, 0.16291E+01_JPRB, 0.15787E+01_JPRB, 0.15483E+01_JPRB ,&
&  0.15007E+01_JPRB, 0.14691E+01_JPRB, 0.14295E+01_JPRB, 0.15403E+01_JPRB, 0.18834E+01_JPRB, 0.21907E+01_JPRB, 0.21936E+01_JPRB ,&
&  0.21544E+01_JPRB, 0.23479E+01_JPRB, 0.23882E+01_JPRB, 0.23726E+01_JPRB, 0.23609E+01_JPRB, 0.23014E+01_JPRB, 0.21802E+01_JPRB ,&
&  0.20170E+01_JPRB, 0.18652E+01_JPRB, 0.17466E+01_JPRB, 0.16430E+01_JPRB, 0.15555E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.15425E+01_JPRB, 0.14739E+01_JPRB, 0.14389E+01_JPRB, 0.14367E+01_JPRB, 0.14561E+01_JPRB, 0.14468E+01_JPRB, 0.14160E+01_JPRB ,&
&  0.13740E+01_JPRB, 0.13281E+01_JPRB, 0.12725E+01_JPRB, 0.12213E+01_JPRB, 0.11827E+01_JPRB, 0.11259E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10460E+01_JPRB, 0.97960E+00_JPRB, 0.91724E+00_JPRB, 0.91743E+00_JPRB, 0.90223E+00_JPRB, 0.86131E+00_JPRB, 0.88316E+00_JPRB ,&
&  0.93674E+00_JPRB, 0.10198E+01_JPRB, 0.11155E+01_JPRB, 0.12229E+01_JPRB, 0.13147E+01_JPRB, 0.14380E+01_JPRB, 0.15328E+01_JPRB ,&
&  0.14971E+01_JPRB, 0.14699E+01_JPRB, 0.14565E+01_JPRB, 0.14210E+01_JPRB, 0.13426E+01_JPRB, 0.12963E+01_JPRB, 0.12807E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.12763E+01_JPRB, 0.12778E+01_JPRB, 0.12651E+01_JPRB, 0.12597E+01_JPRB, 0.12512E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.11509E+01_JPRB, 0.11011E+01_JPRB, 0.11156E+01_JPRB, 0.10715E+01_JPRB, 0.10590E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.14189E+01_JPRB, 0.15922E+01_JPRB, 0.21710E+01_JPRB, 0.21620E+01_JPRB, 0.23394E+01_JPRB, 0.28340E+01_JPRB ,&
&  0.30893E+01_JPRB, 0.31700E+01_JPRB, 0.31786E+01_JPRB, 0.26254E+01_JPRB, 0.23716E+01_JPRB, 0.31003E+01_JPRB, 0.31566E+01_JPRB ,&
&  0.30523E+01_JPRB, 0.30466E+01_JPRB, 0.31444E+01_JPRB, 0.36661E+01_JPRB, 0.30724E+01_JPRB, 0.27645E+01_JPRB, 0.22603E+01_JPRB ,&
&  0.23583E+01_JPRB, 0.20842E+01_JPRB, 0.23469E+01_JPRB, 0.20801E+01_JPRB, 0.19982E+01_JPRB, 0.18534E+01_JPRB, 0.16673E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.15632E+01_JPRB, 0.16143E+01_JPRB, 0.19880E+01_JPRB, 0.25027E+01_JPRB, 0.25576E+01_JPRB, 0.27024E+01_JPRB ,&
&  0.28233E+01_JPRB, 0.28752E+01_JPRB, 0.27118E+01_JPRB, 0.27165E+01_JPRB, 0.26360E+01_JPRB, 0.25245E+01_JPRB, 0.23867E+01_JPRB ,&
&  0.22360E+01_JPRB, 0.20897E+01_JPRB, 0.19325E+01_JPRB, 0.17774E+01_JPRB, 0.16434E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.16158E+01_JPRB, 0.15582E+01_JPRB, 0.15361E+01_JPRB, 0.15341E+01_JPRB, 0.15227E+01_JPRB, 0.14862E+01_JPRB, 0.14363E+01_JPRB ,&
&  0.13752E+01_JPRB, 0.12981E+01_JPRB, 0.12300E+01_JPRB, 0.11799E+01_JPRB, 0.11539E+01_JPRB, 0.11239E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.10515E+01_JPRB, 0.98557E+00_JPRB, 0.95626E+00_JPRB, 0.10098E+01_JPRB, 0.95437E+00_JPRB, 0.90272E+00_JPRB ,&
&  0.98580E+00_JPRB, 0.11006E+01_JPRB, 0.12386E+01_JPRB, 0.14038E+01_JPRB, 0.15420E+01_JPRB, 0.16796E+01_JPRB, 0.17464E+01_JPRB ,&
&  0.17028E+01_JPRB, 0.15868E+01_JPRB, 0.14845E+01_JPRB, 0.13864E+01_JPRB, 0.12905E+01_JPRB, 0.12529E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12778E+01_JPRB, 0.12673E+01_JPRB, 0.12216E+01_JPRB, 0.12068E+01_JPRB, 0.11564E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.11137E+01_JPRB, 0.11002E+01_JPRB, 0.11562E+01_JPRB, 0.10910E+01_JPRB, 0.10579E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.14753E+01_JPRB, 0.17043E+01_JPRB, 0.21304E+01_JPRB, 0.26949E+01_JPRB, 0.27954E+01_JPRB, 0.30910E+01_JPRB ,&
&  0.26918E+01_JPRB, 0.23072E+01_JPRB, 0.20547E+01_JPRB, 0.18499E+01_JPRB, 0.23216E+01_JPRB, 0.33720E+01_JPRB, 0.37466E+01_JPRB ,&
&  0.36944E+01_JPRB, 0.37288E+01_JPRB, 0.40979E+01_JPRB, 0.39103E+01_JPRB, 0.29427E+01_JPRB, 0.22034E+01_JPRB, 0.28208E+01_JPRB ,&
&  0.30098E+01_JPRB, 0.30635E+01_JPRB, 0.27897E+01_JPRB, 0.22822E+01_JPRB, 0.18729E+01_JPRB, 0.20819E+01_JPRB, 0.19326E+01_JPRB ,&
&  0.17661E+01_JPRB, 0.17202E+01_JPRB, 0.21873E+01_JPRB, 0.28726E+01_JPRB, 0.31356E+01_JPRB, 0.34661E+01_JPRB, 0.35634E+01_JPRB ,&
&  0.33642E+01_JPRB, 0.32283E+01_JPRB, 0.30502E+01_JPRB, 0.28444E+01_JPRB, 0.26558E+01_JPRB, 0.24627E+01_JPRB, 0.23096E+01_JPRB ,&
&  0.21808E+01_JPRB, 0.20620E+01_JPRB, 0.19317E+01_JPRB, 0.18049E+01_JPRB, 0.16969E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.16768E+01_JPRB, 0.16474E+01_JPRB, 0.16197E+01_JPRB, 0.15918E+01_JPRB, 0.15506E+01_JPRB, 0.14977E+01_JPRB, 0.14289E+01_JPRB ,&
&  0.13400E+01_JPRB, 0.12675E+01_JPRB, 0.11996E+01_JPRB, 0.11647E+01_JPRB, 0.11355E+01_JPRB, 0.10976E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11599E+01_JPRB, 0.10737E+01_JPRB, 0.10498E+01_JPRB, 0.10880E+01_JPRB, 0.11009E+01_JPRB, 0.96137E+00_JPRB ,&
&  0.10864E+01_JPRB, 0.12165E+01_JPRB, 0.13350E+01_JPRB, 0.15168E+01_JPRB, 0.18104E+01_JPRB, 0.17673E+01_JPRB, 0.17446E+01_JPRB ,&
&  0.17063E+01_JPRB, 0.15992E+01_JPRB, 0.15035E+01_JPRB, 0.13828E+01_JPRB, 0.12917E+01_JPRB, 0.12744E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.12805E+01_JPRB, 0.12976E+01_JPRB, 0.12730E+01_JPRB, 0.12178E+01_JPRB, 0.11703E+01_JPRB, 0.11353E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.10975E+01_JPRB, 0.10981E+01_JPRB, 0.11796E+01_JPRB, 0.11497E+01_JPRB, 0.11385E+01_JPRB, 0.13287E+01_JPRB ,&
&  0.15169E+01_JPRB, 0.19054E+01_JPRB, 0.20706E+01_JPRB, 0.23637E+01_JPRB, 0.26914E+01_JPRB, 0.28308E+01_JPRB, 0.29869E+01_JPRB ,&
&  0.24239E+01_JPRB, 0.21920E+01_JPRB, 0.24934E+01_JPRB, 0.28885E+01_JPRB, 0.29108E+01_JPRB, 0.32247E+01_JPRB, 0.52476E+01_JPRB ,&
&  0.38483E+01_JPRB, 0.41673E+01_JPRB, 0.47600E+01_JPRB, 0.48499E+01_JPRB, 0.23539E+01_JPRB, 0.17176E+01_JPRB, 0.26704E+01_JPRB ,&
&  0.27455E+01_JPRB, 0.17155E+01_JPRB, 0.13229E+01_JPRB, 0.16190E+01_JPRB, 0.13066E+01_JPRB, 0.14935E+01_JPRB, 0.18598E+01_JPRB ,&
&  0.19942E+01_JPRB, 0.20808E+01_JPRB, 0.28099E+01_JPRB, 0.31819E+01_JPRB, 0.33958E+01_JPRB, 0.50662E+01_JPRB, 0.43839E+01_JPRB ,&
&  0.39754E+01_JPRB, 0.33178E+01_JPRB, 0.31182E+01_JPRB, 0.27717E+01_JPRB, 0.25277E+01_JPRB, 0.23079E+01_JPRB, 0.21296E+01_JPRB ,&
&  0.19972E+01_JPRB, 0.18886E+01_JPRB, 0.18094E+01_JPRB, 0.17466E+01_JPRB, 0.17116E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.17357E+01_JPRB, 0.17006E+01_JPRB, 0.16646E+01_JPRB, 0.16194E+01_JPRB, 0.15639E+01_JPRB, 0.14898E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.12298E+01_JPRB, 0.11753E+01_JPRB, 0.11446E+01_JPRB, 0.11087E+01_JPRB, 0.10777E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.11843E+01_JPRB, 0.11952E+01_JPRB, 0.11914E+01_JPRB, 0.12355E+01_JPRB, 0.12644E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.13415E+01_JPRB, 0.14131E+01_JPRB, 0.14787E+01_JPRB, 0.15943E+01_JPRB, 0.17065E+01_JPRB, 0.16717E+01_JPRB, 0.17104E+01_JPRB ,&
&  0.16622E+01_JPRB, 0.16005E+01_JPRB, 0.14957E+01_JPRB, 0.13956E+01_JPRB, 0.13431E+01_JPRB, 0.13132E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.13236E+01_JPRB, 0.12890E+01_JPRB, 0.12204E+01_JPRB, 0.11794E+01_JPRB, 0.11400E+01_JPRB, 0.11143E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.11219E+01_JPRB, 0.11638E+01_JPRB, 0.13099E+01_JPRB, 0.15953E+01_JPRB, 0.17573E+01_JPRB, 0.16364E+01_JPRB ,&
&  0.15647E+01_JPRB, 0.18025E+01_JPRB, 0.22794E+01_JPRB, 0.23984E+01_JPRB, 0.25880E+01_JPRB, 0.27729E+01_JPRB, 0.27712E+01_JPRB ,&
&  0.27640E+01_JPRB, 0.31224E+01_JPRB, 0.32168E+01_JPRB, 0.37271E+01_JPRB, 0.41333E+01_JPRB, 0.34354E+01_JPRB, 0.43418E+01_JPRB ,&
&  0.44912E+01_JPRB, 0.44514E+01_JPRB, 0.37358E+01_JPRB, 0.28855E+01_JPRB, 0.32342E+01_JPRB, 0.24390E+01_JPRB, 0.15103E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.11864E+01_JPRB, 0.11159E+01_JPRB, 0.11406E+01_JPRB, 0.11415E+01_JPRB, 0.14324E+01_JPRB, 0.18253E+01_JPRB ,&
&  0.24423E+01_JPRB, 0.28073E+01_JPRB, 0.30716E+01_JPRB, 0.32484E+01_JPRB, 0.33012E+01_JPRB, 0.48698E+01_JPRB, 0.42965E+01_JPRB ,&
&  0.34585E+01_JPRB, 0.31052E+01_JPRB, 0.28584E+01_JPRB, 0.25953E+01_JPRB, 0.24015E+01_JPRB, 0.22460E+01_JPRB, 0.21173E+01_JPRB ,&
&  0.20191E+01_JPRB, 0.19385E+01_JPRB, 0.18790E+01_JPRB, 0.18284E+01_JPRB, 0.17731E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.17343E+01_JPRB, 0.16925E+01_JPRB, 0.16506E+01_JPRB, 0.15881E+01_JPRB, 0.15172E+01_JPRB, 0.14262E+01_JPRB, 0.13165E+01_JPRB ,&
&  0.12229E+01_JPRB, 0.11599E+01_JPRB, 0.11292E+01_JPRB, 0.10975E+01_JPRB, 0.10770E+01_JPRB, 0.10705E+01_JPRB, 0.10994E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.12129E+01_JPRB, 0.13024E+01_JPRB, 0.13848E+01_JPRB, 0.16109E+01_JPRB, 0.16550E+01_JPRB, 0.17050E+01_JPRB ,&
&  0.17910E+01_JPRB, 0.17558E+01_JPRB, 0.17558E+01_JPRB, 0.17528E+01_JPRB, 0.17779E+01_JPRB, 0.17326E+01_JPRB, 0.17023E+01_JPRB ,&
&  0.16407E+01_JPRB, 0.16235E+01_JPRB, 0.15323E+01_JPRB, 0.14449E+01_JPRB, 0.13786E+01_JPRB, 0.13530E+01_JPRB, 0.13239E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.13424E+01_JPRB, 0.13181E+01_JPRB, 0.12494E+01_JPRB, 0.12068E+01_JPRB, 0.11830E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11923E+01_JPRB, 0.12582E+01_JPRB, 0.13521E+01_JPRB, 0.15629E+01_JPRB, 0.15543E+01_JPRB, 0.13546E+01_JPRB, 0.15160E+01_JPRB ,&
&  0.16524E+01_JPRB, 0.20320E+01_JPRB, 0.21863E+01_JPRB, 0.24717E+01_JPRB, 0.25517E+01_JPRB, 0.25002E+01_JPRB, 0.27306E+01_JPRB ,&
&  0.31045E+01_JPRB, 0.43749E+01_JPRB, 0.34934E+01_JPRB, 0.36379E+01_JPRB, 0.57630E+01_JPRB, 0.48002E+01_JPRB, 0.45008E+01_JPRB ,&
&  0.51379E+01_JPRB, 0.46732E+01_JPRB, 0.48491E+01_JPRB, 0.43483E+01_JPRB, 0.72912E+01_JPRB, 0.79722E+01_JPRB, 0.22763E+01_JPRB ,&
&  0.15515E+01_JPRB, 0.13342E+01_JPRB, 0.12088E+01_JPRB, 0.10988E+01_JPRB, 0.11694E+01_JPRB, 0.15536E+01_JPRB, 0.23030E+01_JPRB ,&
&  0.26727E+01_JPRB, 0.32336E+01_JPRB, 0.33564E+01_JPRB, 0.35911E+01_JPRB, 0.33817E+01_JPRB, 0.38793E+01_JPRB, 0.31540E+01_JPRB ,&
&  0.29934E+01_JPRB, 0.28469E+01_JPRB, 0.25858E+01_JPRB, 0.23643E+01_JPRB, 0.22321E+01_JPRB, 0.21368E+01_JPRB, 0.20473E+01_JPRB ,&
&  0.19718E+01_JPRB, 0.19207E+01_JPRB, 0.18730E+01_JPRB, 0.18257E+01_JPRB, 0.17791E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.16681E+01_JPRB, 0.16292E+01_JPRB, 0.15773E+01_JPRB, 0.14981E+01_JPRB, 0.14291E+01_JPRB, 0.13448E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.11489E+01_JPRB, 0.11120E+01_JPRB, 0.10761E+01_JPRB, 0.10700E+01_JPRB, 0.10821E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.12814E+01_JPRB, 0.14064E+01_JPRB, 0.17226E+01_JPRB, 0.20567E+01_JPRB, 0.21359E+01_JPRB, 0.26124E+01_JPRB ,&
&  0.25076E+01_JPRB, 0.22918E+01_JPRB, 0.21543E+01_JPRB, 0.20158E+01_JPRB, 0.19102E+01_JPRB, 0.17985E+01_JPRB, 0.17310E+01_JPRB ,&
&  0.16610E+01_JPRB, 0.16687E+01_JPRB, 0.15723E+01_JPRB, 0.14890E+01_JPRB, 0.14418E+01_JPRB, 0.13959E+01_JPRB, 0.13666E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.13840E+01_JPRB, 0.13694E+01_JPRB, 0.13131E+01_JPRB, 0.12708E+01_JPRB, 0.12559E+01_JPRB, 0.12758E+01_JPRB ,&
&  0.13390E+01_JPRB, 0.14520E+01_JPRB, 0.15751E+01_JPRB, 0.17181E+01_JPRB, 0.14528E+01_JPRB, 0.16421E+01_JPRB, 0.17302E+01_JPRB ,&
&  0.19328E+01_JPRB, 0.21112E+01_JPRB, 0.21163E+01_JPRB, 0.22569E+01_JPRB, 0.24975E+01_JPRB, 0.25402E+01_JPRB, 0.28476E+01_JPRB ,&
&  0.37394E+01_JPRB, 0.37708E+01_JPRB, 0.34348E+01_JPRB, 0.38854E+01_JPRB, 0.52945E+01_JPRB, 0.78459E+01_JPRB, 0.42271E+01_JPRB ,&
&  0.35033E+01_JPRB, 0.51393E+01_JPRB, 0.57714E+01_JPRB, 0.58050E+01_JPRB, 0.86610E+01_JPRB, 0.11798E+02_JPRB, 0.70731E+01_JPRB ,&
&  0.19446E+01_JPRB, 0.13533E+01_JPRB, 0.15444E+01_JPRB, 0.13655E+01_JPRB, 0.12657E+01_JPRB, 0.16742E+01_JPRB, 0.39309E+01_JPRB ,&
&  0.32196E+01_JPRB, 0.36715E+01_JPRB, 0.37650E+01_JPRB, 0.37074E+01_JPRB, 0.34086E+01_JPRB, 0.33340E+01_JPRB, 0.28916E+01_JPRB ,&
&  0.24873E+01_JPRB, 0.23075E+01_JPRB, 0.20942E+01_JPRB, 0.19766E+01_JPRB, 0.19365E+01_JPRB, 0.19102E+01_JPRB, 0.18501E+01_JPRB ,&
&  0.18091E+01_JPRB, 0.17828E+01_JPRB, 0.17557E+01_JPRB, 0.17336E+01_JPRB, 0.17075E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.15664E+01_JPRB, 0.15298E+01_JPRB, 0.14607E+01_JPRB, 0.14005E+01_JPRB, 0.13754E+01_JPRB, 0.13050E+01_JPRB, 0.12479E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.11756E+01_JPRB, 0.11443E+01_JPRB, 0.11438E+01_JPRB, 0.11215E+01_JPRB, 0.11446E+01_JPRB, 0.11986E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.13831E+01_JPRB, 0.15236E+01_JPRB, 0.18817E+01_JPRB, 0.23775E+01_JPRB, 0.25078E+01_JPRB, 0.31091E+01_JPRB ,&
&  0.35671E+01_JPRB, 0.31528E+01_JPRB, 0.26266E+01_JPRB, 0.23106E+01_JPRB, 0.20872E+01_JPRB, 0.19673E+01_JPRB, 0.18345E+01_JPRB ,&
&  0.17598E+01_JPRB, 0.17712E+01_JPRB, 0.16412E+01_JPRB, 0.15382E+01_JPRB, 0.14541E+01_JPRB, 0.14202E+01_JPRB, 0.14397E+01_JPRB ,&
&  0.14422E+01_JPRB, 0.14656E+01_JPRB, 0.14613E+01_JPRB, 0.14311E+01_JPRB, 0.13980E+01_JPRB, 0.13941E+01_JPRB, 0.14708E+01_JPRB ,&
&  0.16064E+01_JPRB, 0.17491E+01_JPRB, 0.18397E+01_JPRB, 0.18235E+01_JPRB, 0.18428E+01_JPRB, 0.18730E+01_JPRB, 0.20571E+01_JPRB ,&
&  0.22121E+01_JPRB, 0.22999E+01_JPRB, 0.23366E+01_JPRB, 0.23552E+01_JPRB, 0.25703E+01_JPRB, 0.28609E+01_JPRB, 0.32816E+01_JPRB ,&
&  0.39664E+01_JPRB, 0.42245E+01_JPRB, 0.38874E+01_JPRB, 0.42255E+01_JPRB, 0.51342E+01_JPRB, 0.64310E+01_JPRB, 0.58939E+01_JPRB ,&
&  0.49979E+01_JPRB, 0.46219E+01_JPRB, 0.55462E+01_JPRB, 0.73174E+01_JPRB, 0.83614E+01_JPRB, 0.85397E+01_JPRB, 0.10338E+02_JPRB ,&
&  0.85311E+01_JPRB, 0.38472E+01_JPRB, 0.29260E+01_JPRB, 0.34372E+01_JPRB, 0.23504E+01_JPRB, 0.23458E+01_JPRB, 0.31843E+01_JPRB ,&
&  0.34182E+01_JPRB, 0.38926E+01_JPRB, 0.40174E+01_JPRB, 0.33819E+01_JPRB, 0.31865E+01_JPRB, 0.29836E+01_JPRB, 0.24817E+01_JPRB ,&
&  0.21284E+01_JPRB, 0.19142E+01_JPRB, 0.17881E+01_JPRB, 0.17083E+01_JPRB, 0.16622E+01_JPRB, 0.16174E+01_JPRB, 0.15924E+01_JPRB ,&
&  0.15659E+01_JPRB, 0.15570E+01_JPRB, 0.15611E+01_JPRB, 0.15684E+01_JPRB, 0.15769E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.14785E+01_JPRB, 0.14279E+01_JPRB, 0.14112E+01_JPRB, 0.14170E+01_JPRB, 0.13976E+01_JPRB, 0.13647E+01_JPRB, 0.13343E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12822E+01_JPRB, 0.13249E+01_JPRB, 0.13196E+01_JPRB, 0.12734E+01_JPRB, 0.12796E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.14191E+01_JPRB, 0.15878E+01_JPRB, 0.18396E+01_JPRB, 0.23763E+01_JPRB, 0.30137E+01_JPRB, 0.41292E+01_JPRB, 0.32079E+01_JPRB ,&
&  0.35366E+01_JPRB, 0.35461E+01_JPRB, 0.28180E+01_JPRB, 0.25177E+01_JPRB, 0.23262E+01_JPRB, 0.22409E+01_JPRB, 0.20089E+01_JPRB ,&
&  0.19529E+01_JPRB, 0.18749E+01_JPRB, 0.17286E+01_JPRB, 0.15789E+01_JPRB, 0.14835E+01_JPRB, 0.15384E+01_JPRB, 0.15541E+01_JPRB ,&
&  0.15711E+01_JPRB, 0.16162E+01_JPRB, 0.16045E+01_JPRB, 0.16104E+01_JPRB, 0.15975E+01_JPRB, 0.16091E+01_JPRB, 0.17193E+01_JPRB ,&
&  0.18753E+01_JPRB, 0.20062E+01_JPRB, 0.20916E+01_JPRB, 0.20194E+01_JPRB, 0.21088E+01_JPRB, 0.22304E+01_JPRB, 0.24678E+01_JPRB ,&
&  0.25073E+01_JPRB, 0.22924E+01_JPRB, 0.23645E+01_JPRB, 0.26155E+01_JPRB, 0.29318E+01_JPRB, 0.30855E+01_JPRB, 0.34793E+01_JPRB ,&
&  0.44201E+01_JPRB, 0.52467E+01_JPRB, 0.56448E+01_JPRB, 0.48743E+01_JPRB, 0.49792E+01_JPRB, 0.60233E+01_JPRB, 0.69350E+01_JPRB ,&
&  0.71375E+01_JPRB, 0.68749E+01_JPRB, 0.73340E+01_JPRB, 0.71891E+01_JPRB, 0.68688E+01_JPRB, 0.64486E+01_JPRB, 0.67865E+01_JPRB ,&
&  0.77053E+01_JPRB, 0.82915E+01_JPRB, 0.58566E+01_JPRB, 0.44245E+01_JPRB, 0.40841E+01_JPRB, 0.35127E+01_JPRB, 0.32979E+01_JPRB ,&
&  0.37451E+01_JPRB, 0.38445E+01_JPRB, 0.37596E+01_JPRB, 0.39846E+01_JPRB, 0.31247E+01_JPRB, 0.25146E+01_JPRB, 0.19078E+01_JPRB ,&
&  0.15568E+01_JPRB, 0.14404E+01_JPRB, 0.13965E+01_JPRB, 0.13844E+01_JPRB, 0.14289E+01_JPRB, 0.14369E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.14224E+01_JPRB, 0.14465E+01_JPRB, 0.14825E+01_JPRB, 0.15120E+01_JPRB, 0.15113E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.14168E+01_JPRB, 0.14104E+01_JPRB, 0.14351E+01_JPRB, 0.14583E+01_JPRB, 0.14544E+01_JPRB, 0.14740E+01_JPRB, 0.14680E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.13916E+01_JPRB, 0.14377E+01_JPRB, 0.14178E+01_JPRB, 0.13967E+01_JPRB, 0.14172E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.16247E+01_JPRB, 0.18895E+01_JPRB, 0.23376E+01_JPRB, 0.29310E+01_JPRB, 0.34005E+01_JPRB, 0.39899E+01_JPRB, 0.42951E+01_JPRB ,&
&  0.31247E+01_JPRB, 0.36875E+01_JPRB, 0.27469E+01_JPRB, 0.25798E+01_JPRB, 0.25599E+01_JPRB, 0.28143E+01_JPRB, 0.25797E+01_JPRB ,&
&  0.21666E+01_JPRB, 0.19075E+01_JPRB, 0.18599E+01_JPRB, 0.17662E+01_JPRB, 0.16589E+01_JPRB, 0.16703E+01_JPRB, 0.17001E+01_JPRB ,&
&  0.16914E+01_JPRB, 0.17055E+01_JPRB, 0.17615E+01_JPRB, 0.18084E+01_JPRB, 0.18082E+01_JPRB, 0.18332E+01_JPRB, 0.19018E+01_JPRB ,&
&  0.20845E+01_JPRB, 0.22422E+01_JPRB, 0.22934E+01_JPRB, 0.23570E+01_JPRB, 0.24065E+01_JPRB, 0.26831E+01_JPRB, 0.29508E+01_JPRB ,&
&  0.28905E+01_JPRB, 0.25968E+01_JPRB, 0.26419E+01_JPRB, 0.28336E+01_JPRB, 0.30102E+01_JPRB, 0.34741E+01_JPRB, 0.38419E+01_JPRB ,&
&  0.49332E+01_JPRB, 0.51365E+01_JPRB, 0.62581E+01_JPRB, 0.50190E+01_JPRB, 0.49081E+01_JPRB, 0.58620E+01_JPRB, 0.59003E+01_JPRB ,&
&  0.55005E+01_JPRB, 0.64058E+01_JPRB, 0.66221E+01_JPRB, 0.67358E+01_JPRB, 0.64473E+01_JPRB, 0.57223E+01_JPRB, 0.53145E+01_JPRB ,&
&  0.54431E+01_JPRB, 0.57284E+01_JPRB, 0.53418E+01_JPRB, 0.43335E+01_JPRB, 0.39704E+01_JPRB, 0.35459E+01_JPRB, 0.37155E+01_JPRB ,&
&  0.42410E+01_JPRB, 0.44698E+01_JPRB, 0.43185E+01_JPRB, 0.38411E+01_JPRB, 0.23055E+01_JPRB, 0.16988E+01_JPRB, 0.13656E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12146E+01_JPRB, 0.12485E+01_JPRB, 0.13209E+01_JPRB, 0.13818E+01_JPRB, 0.14089E+01_JPRB, 0.14007E+01_JPRB ,&
&  0.14055E+01_JPRB, 0.14293E+01_JPRB, 0.14430E+01_JPRB, 0.14428E+01_JPRB, 0.14336E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.14307E+01_JPRB, 0.14545E+01_JPRB, 0.14846E+01_JPRB, 0.15326E+01_JPRB, 0.15640E+01_JPRB, 0.16536E+01_JPRB, 0.17423E+01_JPRB ,&
&  0.16348E+01_JPRB, 0.13140E+01_JPRB, 0.13610E+01_JPRB, 0.14083E+01_JPRB, 0.14589E+01_JPRB, 0.15274E+01_JPRB, 0.16453E+01_JPRB ,&
&  0.18868E+01_JPRB, 0.22067E+01_JPRB, 0.25901E+01_JPRB, 0.29785E+01_JPRB, 0.34536E+01_JPRB, 0.39881E+01_JPRB, 0.40979E+01_JPRB ,&
&  0.33269E+01_JPRB, 0.28025E+01_JPRB, 0.28553E+01_JPRB, 0.27965E+01_JPRB, 0.28100E+01_JPRB, 0.29087E+01_JPRB, 0.29587E+01_JPRB ,&
&  0.24403E+01_JPRB, 0.21444E+01_JPRB, 0.21316E+01_JPRB, 0.21727E+01_JPRB, 0.18402E+01_JPRB, 0.19014E+01_JPRB, 0.19523E+01_JPRB ,&
&  0.19617E+01_JPRB, 0.19686E+01_JPRB, 0.20490E+01_JPRB, 0.21247E+01_JPRB, 0.21093E+01_JPRB, 0.21315E+01_JPRB, 0.22343E+01_JPRB ,&
&  0.24701E+01_JPRB, 0.26464E+01_JPRB, 0.27458E+01_JPRB, 0.27567E+01_JPRB, 0.29234E+01_JPRB, 0.32247E+01_JPRB, 0.33302E+01_JPRB ,&
&  0.31695E+01_JPRB, 0.31128E+01_JPRB, 0.31759E+01_JPRB, 0.33342E+01_JPRB, 0.32998E+01_JPRB, 0.39710E+01_JPRB, 0.43437E+01_JPRB ,&
&  0.48846E+01_JPRB, 0.51737E+01_JPRB, 0.61582E+01_JPRB, 0.46823E+01_JPRB, 0.51170E+01_JPRB, 0.55299E+01_JPRB, 0.53734E+01_JPRB ,&
&  0.50093E+01_JPRB, 0.53822E+01_JPRB, 0.53681E+01_JPRB, 0.54710E+01_JPRB, 0.54407E+01_JPRB, 0.46926E+01_JPRB, 0.48918E+01_JPRB ,&
&  0.46301E+01_JPRB, 0.51207E+01_JPRB, 0.47783E+01_JPRB, 0.45614E+01_JPRB, 0.38470E+01_JPRB, 0.36486E+01_JPRB, 0.37796E+01_JPRB ,&
&  0.43064E+01_JPRB, 0.43462E+01_JPRB, 0.36215E+01_JPRB, 0.27533E+01_JPRB, 0.15168E+01_JPRB, 0.11934E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.12101E+01_JPRB, 0.12838E+01_JPRB, 0.13485E+01_JPRB, 0.13841E+01_JPRB, 0.13945E+01_JPRB, 0.13974E+01_JPRB ,&
&  0.14139E+01_JPRB, 0.14220E+01_JPRB, 0.14178E+01_JPRB, 0.14189E+01_JPRB, 0.14224E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.14328E+01_JPRB, 0.14306E+01_JPRB, 0.14322E+01_JPRB, 0.14013E+01_JPRB, 0.13622E+01_JPRB, 0.13173E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.11774E+01_JPRB, 0.12549E+01_JPRB, 0.13536E+01_JPRB, 0.14484E+01_JPRB, 0.15623E+01_JPRB, 0.17415E+01_JPRB ,&
&  0.19602E+01_JPRB, 0.21292E+01_JPRB, 0.23273E+01_JPRB, 0.26439E+01_JPRB, 0.30343E+01_JPRB, 0.32096E+01_JPRB, 0.33422E+01_JPRB ,&
&  0.30544E+01_JPRB, 0.22523E+01_JPRB, 0.27540E+01_JPRB, 0.30458E+01_JPRB, 0.31553E+01_JPRB, 0.32347E+01_JPRB, 0.30456E+01_JPRB ,&
&  0.24641E+01_JPRB, 0.24352E+01_JPRB, 0.25840E+01_JPRB, 0.26245E+01_JPRB, 0.21835E+01_JPRB, 0.22356E+01_JPRB, 0.23000E+01_JPRB ,&
&  0.23641E+01_JPRB, 0.24109E+01_JPRB, 0.25341E+01_JPRB, 0.26133E+01_JPRB, 0.26380E+01_JPRB, 0.27272E+01_JPRB, 0.28828E+01_JPRB ,&
&  0.30456E+01_JPRB, 0.31703E+01_JPRB, 0.33435E+01_JPRB, 0.34578E+01_JPRB, 0.35842E+01_JPRB, 0.37603E+01_JPRB, 0.37247E+01_JPRB ,&
&  0.35791E+01_JPRB, 0.34477E+01_JPRB, 0.36599E+01_JPRB, 0.41541E+01_JPRB, 0.45650E+01_JPRB, 0.41874E+01_JPRB, 0.46937E+01_JPRB ,&
&  0.48521E+01_JPRB, 0.50561E+01_JPRB, 0.50590E+01_JPRB, 0.59244E+01_JPRB, 0.41909E+01_JPRB, 0.47145E+01_JPRB, 0.42798E+01_JPRB ,&
&  0.45401E+01_JPRB, 0.43712E+01_JPRB, 0.42283E+01_JPRB, 0.42603E+01_JPRB, 0.43369E+01_JPRB, 0.38840E+01_JPRB, 0.40761E+01_JPRB ,&
&  0.43104E+01_JPRB, 0.45031E+01_JPRB, 0.45687E+01_JPRB, 0.46576E+01_JPRB, 0.43364E+01_JPRB, 0.42577E+01_JPRB, 0.41479E+01_JPRB ,&
&  0.43815E+01_JPRB, 0.34797E+01_JPRB, 0.26232E+01_JPRB, 0.18461E+01_JPRB, 0.12119E+01_JPRB, 0.10286E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.12302E+01_JPRB, 0.12873E+01_JPRB, 0.13283E+01_JPRB, 0.13542E+01_JPRB, 0.13737E+01_JPRB, 0.14040E+01_JPRB ,&
&  0.14181E+01_JPRB, 0.14230E+01_JPRB, 0.14208E+01_JPRB, 0.14208E+01_JPRB, 0.14260E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.13922E+01_JPRB, 0.13619E+01_JPRB, 0.13045E+01_JPRB, 0.12166E+01_JPRB, 0.11240E+01_JPRB, 0.10345E+01_JPRB, 0.98319E+00_JPRB ,&
&  0.94777E+00_JPRB, 0.98505E+00_JPRB, 0.10828E+01_JPRB, 0.12143E+01_JPRB, 0.13638E+01_JPRB, 0.15132E+01_JPRB, 0.16503E+01_JPRB ,&
&  0.17340E+01_JPRB, 0.17854E+01_JPRB, 0.19005E+01_JPRB, 0.21713E+01_JPRB, 0.24844E+01_JPRB, 0.27324E+01_JPRB, 0.29070E+01_JPRB ,&
&  0.30567E+01_JPRB, 0.30815E+01_JPRB, 0.29014E+01_JPRB, 0.23832E+01_JPRB, 0.22872E+01_JPRB, 0.28621E+01_JPRB, 0.25366E+01_JPRB ,&
&  0.24515E+01_JPRB, 0.23939E+01_JPRB, 0.24366E+01_JPRB, 0.25105E+01_JPRB, 0.25064E+01_JPRB, 0.25338E+01_JPRB, 0.26513E+01_JPRB ,&
&  0.27508E+01_JPRB, 0.28749E+01_JPRB, 0.29977E+01_JPRB, 0.30902E+01_JPRB, 0.32041E+01_JPRB, 0.33463E+01_JPRB, 0.34850E+01_JPRB ,&
&  0.36167E+01_JPRB, 0.38103E+01_JPRB, 0.40356E+01_JPRB, 0.42193E+01_JPRB, 0.41905E+01_JPRB, 0.42883E+01_JPRB, 0.42314E+01_JPRB ,&
&  0.41345E+01_JPRB, 0.40679E+01_JPRB, 0.41260E+01_JPRB, 0.44621E+01_JPRB, 0.46386E+01_JPRB, 0.41312E+01_JPRB, 0.42266E+01_JPRB ,&
&  0.46534E+01_JPRB, 0.48254E+01_JPRB, 0.42933E+01_JPRB, 0.56097E+01_JPRB, 0.33870E+01_JPRB, 0.41152E+01_JPRB, 0.43438E+01_JPRB ,&
&  0.39800E+01_JPRB, 0.36216E+01_JPRB, 0.35550E+01_JPRB, 0.36066E+01_JPRB, 0.35508E+01_JPRB, 0.34302E+01_JPRB, 0.37512E+01_JPRB ,&
&  0.39397E+01_JPRB, 0.40968E+01_JPRB, 0.44066E+01_JPRB, 0.44413E+01_JPRB, 0.43940E+01_JPRB, 0.40336E+01_JPRB, 0.34762E+01_JPRB ,&
&  0.29060E+01_JPRB, 0.27992E+01_JPRB, 0.23536E+01_JPRB, 0.20261E+01_JPRB, 0.13030E+01_JPRB, 0.99977E+00_JPRB, 0.10708E+01_JPRB ,&
&  0.11330E+01_JPRB, 0.11818E+01_JPRB, 0.12446E+01_JPRB, 0.12932E+01_JPRB, 0.13136E+01_JPRB, 0.13350E+01_JPRB, 0.13683E+01_JPRB ,&
&  0.13743E+01_JPRB, 0.13740E+01_JPRB, 0.13796E+01_JPRB, 0.13829E+01_JPRB, 0.13929E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.11495E+01_JPRB, 0.11113E+01_JPRB, 0.10502E+01_JPRB, 0.99318E+00_JPRB, 0.93727E+00_JPRB, 0.87504E+00_JPRB, 0.86332E+00_JPRB ,&
&  0.84025E+00_JPRB, 0.85236E+00_JPRB, 0.89699E+00_JPRB, 0.95628E+00_JPRB, 0.10461E+01_JPRB, 0.11708E+01_JPRB, 0.12849E+01_JPRB ,&
&  0.13388E+01_JPRB, 0.13516E+01_JPRB, 0.14226E+01_JPRB, 0.15305E+01_JPRB, 0.17475E+01_JPRB, 0.21526E+01_JPRB, 0.28333E+01_JPRB ,&
&  0.32307E+01_JPRB, 0.34330E+01_JPRB, 0.33546E+01_JPRB, 0.31345E+01_JPRB, 0.27701E+01_JPRB, 0.29735E+01_JPRB, 0.32354E+01_JPRB ,&
&  0.34064E+01_JPRB, 0.30637E+01_JPRB, 0.26192E+01_JPRB, 0.26289E+01_JPRB, 0.26991E+01_JPRB, 0.27866E+01_JPRB, 0.29183E+01_JPRB ,&
&  0.30544E+01_JPRB, 0.31623E+01_JPRB, 0.32757E+01_JPRB, 0.34136E+01_JPRB, 0.35367E+01_JPRB, 0.36576E+01_JPRB, 0.38087E+01_JPRB ,&
&  0.39766E+01_JPRB, 0.41378E+01_JPRB, 0.42665E+01_JPRB, 0.44363E+01_JPRB, 0.46858E+01_JPRB, 0.49239E+01_JPRB, 0.49852E+01_JPRB ,&
&  0.49498E+01_JPRB, 0.46916E+01_JPRB, 0.48267E+01_JPRB, 0.51152E+01_JPRB, 0.49837E+01_JPRB, 0.45760E+01_JPRB, 0.44939E+01_JPRB ,&
&  0.47496E+01_JPRB, 0.49260E+01_JPRB, 0.34523E+01_JPRB, 0.44870E+01_JPRB, 0.51249E+01_JPRB, 0.44275E+01_JPRB, 0.37715E+01_JPRB ,&
&  0.32996E+01_JPRB, 0.30377E+01_JPRB, 0.31546E+01_JPRB, 0.32465E+01_JPRB, 0.35188E+01_JPRB, 0.34511E+01_JPRB, 0.29691E+01_JPRB ,&
&  0.29962E+01_JPRB, 0.32090E+01_JPRB, 0.36688E+01_JPRB, 0.38594E+01_JPRB, 0.36694E+01_JPRB, 0.32158E+01_JPRB, 0.25364E+01_JPRB ,&
&  0.20697E+01_JPRB, 0.23008E+01_JPRB, 0.20373E+01_JPRB, 0.19573E+01_JPRB, 0.15861E+01_JPRB, 0.96291E+00_JPRB, 0.10281E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.11080E+01_JPRB, 0.11550E+01_JPRB, 0.11849E+01_JPRB, 0.12168E+01_JPRB, 0.12377E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.12624E+01_JPRB, 0.12420E+01_JPRB, 0.12268E+01_JPRB, 0.12159E+01_JPRB, 0.11914E+01_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.99775E+00_JPRB, 0.97939E+00_JPRB, 0.96000E+00_JPRB, 0.95030E+00_JPRB, 0.95158E+00_JPRB, 0.93979E+00_JPRB, 0.91565E+00_JPRB ,&
&  0.88286E+00_JPRB, 0.86896E+00_JPRB, 0.84141E+00_JPRB, 0.81720E+00_JPRB, 0.82985E+00_JPRB, 0.90350E+00_JPRB, 0.97926E+00_JPRB ,&
&  0.10235E+01_JPRB, 0.10393E+01_JPRB, 0.10281E+01_JPRB, 0.10138E+01_JPRB, 0.10447E+01_JPRB, 0.10931E+01_JPRB, 0.14184E+01_JPRB ,&
&  0.19905E+01_JPRB, 0.27637E+01_JPRB, 0.33287E+01_JPRB, 0.36246E+01_JPRB, 0.35804E+01_JPRB, 0.32839E+01_JPRB, 0.40990E+01_JPRB ,&
&  0.37110E+01_JPRB, 0.28005E+01_JPRB, 0.25234E+01_JPRB, 0.24432E+01_JPRB, 0.24848E+01_JPRB, 0.26118E+01_JPRB, 0.27692E+01_JPRB ,&
&  0.29204E+01_JPRB, 0.30711E+01_JPRB, 0.32272E+01_JPRB, 0.34072E+01_JPRB, 0.35983E+01_JPRB, 0.38021E+01_JPRB, 0.40073E+01_JPRB ,&
&  0.42615E+01_JPRB, 0.44323E+01_JPRB, 0.46252E+01_JPRB, 0.45146E+01_JPRB, 0.49942E+01_JPRB, 0.53971E+01_JPRB, 0.56227E+01_JPRB ,&
&  0.56179E+01_JPRB, 0.54896E+01_JPRB, 0.56989E+01_JPRB, 0.57543E+01_JPRB, 0.57515E+01_JPRB, 0.55433E+01_JPRB, 0.52171E+01_JPRB ,&
&  0.53464E+01_JPRB, 0.49758E+01_JPRB, 0.34705E+01_JPRB, 0.36511E+01_JPRB, 0.40361E+01_JPRB, 0.31803E+01_JPRB, 0.33510E+01_JPRB ,&
&  0.28335E+01_JPRB, 0.24960E+01_JPRB, 0.22619E+01_JPRB, 0.23148E+01_JPRB, 0.27060E+01_JPRB, 0.30630E+01_JPRB, 0.25848E+01_JPRB ,&
&  0.20964E+01_JPRB, 0.22445E+01_JPRB, 0.25756E+01_JPRB, 0.27691E+01_JPRB, 0.26668E+01_JPRB, 0.23749E+01_JPRB, 0.20587E+01_JPRB ,&
&  0.19724E+01_JPRB, 0.19007E+01_JPRB, 0.18043E+01_JPRB, 0.16251E+01_JPRB, 0.11374E+01_JPRB, 0.88470E+00_JPRB, 0.91738E+00_JPRB ,&
&  0.94333E+00_JPRB, 0.94816E+00_JPRB, 0.96843E+00_JPRB, 0.98749E+00_JPRB, 0.10191E+01_JPRB, 0.10274E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10657E+01_JPRB, 0.10627E+01_JPRB, 0.10591E+01_JPRB, 0.10610E+01_JPRB, 0.10318E+01_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.10718E+01_JPRB, 0.10777E+01_JPRB, 0.10934E+01_JPRB, 0.11113E+01_JPRB, 0.11339E+01_JPRB, 0.11157E+01_JPRB, 0.10873E+01_JPRB ,&
&  0.10418E+01_JPRB, 0.10308E+01_JPRB, 0.10199E+01_JPRB, 0.10143E+01_JPRB, 0.10301E+01_JPRB, 0.10670E+01_JPRB, 0.11337E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.11571E+01_JPRB, 0.11391E+01_JPRB, 0.11156E+01_JPRB, 0.10576E+01_JPRB, 0.10045E+01_JPRB, 0.97198E+00_JPRB ,&
&  0.10208E+01_JPRB, 0.13173E+01_JPRB, 0.16998E+01_JPRB, 0.20795E+01_JPRB, 0.25854E+01_JPRB, 0.25842E+01_JPRB, 0.31952E+01_JPRB ,&
&  0.38801E+01_JPRB, 0.25745E+01_JPRB, 0.24794E+01_JPRB, 0.21813E+01_JPRB, 0.21894E+01_JPRB, 0.22140E+01_JPRB, 0.23583E+01_JPRB ,&
&  0.25358E+01_JPRB, 0.26687E+01_JPRB, 0.27898E+01_JPRB, 0.29210E+01_JPRB, 0.30737E+01_JPRB, 0.32103E+01_JPRB, 0.32917E+01_JPRB ,&
&  0.33630E+01_JPRB, 0.32732E+01_JPRB, 0.33766E+01_JPRB, 0.40756E+01_JPRB, 0.47409E+01_JPRB, 0.51092E+01_JPRB, 0.53899E+01_JPRB ,&
&  0.57609E+01_JPRB, 0.60320E+01_JPRB, 0.56301E+01_JPRB, 0.56208E+01_JPRB, 0.59434E+01_JPRB, 0.54176E+01_JPRB, 0.54706E+01_JPRB ,&
&  0.56651E+01_JPRB, 0.49505E+01_JPRB, 0.31431E+01_JPRB, 0.35073E+01_JPRB, 0.36807E+01_JPRB, 0.32850E+01_JPRB, 0.31736E+01_JPRB ,&
&  0.26465E+01_JPRB, 0.22763E+01_JPRB, 0.18706E+01_JPRB, 0.17084E+01_JPRB, 0.17699E+01_JPRB, 0.20450E+01_JPRB, 0.20890E+01_JPRB ,&
&  0.17011E+01_JPRB, 0.16636E+01_JPRB, 0.19670E+01_JPRB, 0.23501E+01_JPRB, 0.25077E+01_JPRB, 0.22775E+01_JPRB, 0.18577E+01_JPRB ,&
&  0.19748E+01_JPRB, 0.20988E+01_JPRB, 0.26325E+01_JPRB, 0.23940E+01_JPRB, 0.12356E+01_JPRB, 0.92428E+00_JPRB, 0.87035E+00_JPRB ,&
&  0.89115E+00_JPRB, 0.89311E+00_JPRB, 0.89008E+00_JPRB, 0.90887E+00_JPRB, 0.92243E+00_JPRB, 0.92779E+00_JPRB, 0.97095E+00_JPRB ,&
&  0.10222E+01_JPRB, 0.10289E+01_JPRB, 0.10421E+01_JPRB, 0.10484E+01_JPRB, 0.10587E+01_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.11910E+01_JPRB, 0.12133E+01_JPRB, 0.12436E+01_JPRB, 0.12538E+01_JPRB, 0.12624E+01_JPRB, 0.12534E+01_JPRB, 0.12426E+01_JPRB ,&
&  0.12213E+01_JPRB, 0.12152E+01_JPRB, 0.12204E+01_JPRB, 0.12269E+01_JPRB, 0.12473E+01_JPRB, 0.12695E+01_JPRB, 0.13130E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.13289E+01_JPRB, 0.13417E+01_JPRB, 0.13673E+01_JPRB, 0.13858E+01_JPRB, 0.13716E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.12957E+01_JPRB, 0.12869E+01_JPRB, 0.13039E+01_JPRB, 0.13475E+01_JPRB, 0.15772E+01_JPRB, 0.19713E+01_JPRB, 0.26498E+01_JPRB ,&
&  0.30550E+01_JPRB, 0.25863E+01_JPRB, 0.21970E+01_JPRB, 0.19094E+01_JPRB, 0.18723E+01_JPRB, 0.18876E+01_JPRB, 0.17890E+01_JPRB ,&
&  0.17699E+01_JPRB, 0.17386E+01_JPRB, 0.17777E+01_JPRB, 0.18630E+01_JPRB, 0.19334E+01_JPRB, 0.19118E+01_JPRB, 0.18758E+01_JPRB ,&
&  0.17253E+01_JPRB, 0.16781E+01_JPRB, 0.18266E+01_JPRB, 0.22193E+01_JPRB, 0.30751E+01_JPRB, 0.36499E+01_JPRB, 0.34810E+01_JPRB ,&
&  0.39039E+01_JPRB, 0.46866E+01_JPRB, 0.49979E+01_JPRB, 0.50678E+01_JPRB, 0.48927E+01_JPRB, 0.46227E+01_JPRB, 0.44569E+01_JPRB ,&
&  0.43735E+01_JPRB, 0.39523E+01_JPRB, 0.31276E+01_JPRB, 0.33701E+01_JPRB, 0.33117E+01_JPRB, 0.32665E+01_JPRB, 0.28595E+01_JPRB ,&
&  0.24041E+01_JPRB, 0.20998E+01_JPRB, 0.18889E+01_JPRB, 0.16958E+01_JPRB, 0.15689E+01_JPRB, 0.16266E+01_JPRB, 0.15498E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.15082E+01_JPRB, 0.16344E+01_JPRB, 0.18641E+01_JPRB, 0.24480E+01_JPRB, 0.22882E+01_JPRB, 0.23420E+01_JPRB ,&
&  0.21119E+01_JPRB, 0.22267E+01_JPRB, 0.17948E+01_JPRB, 0.25853E+01_JPRB, 0.12168E+01_JPRB, 0.83578E+00_JPRB, 0.83572E+00_JPRB ,&
&  0.88286E+00_JPRB, 0.90718E+00_JPRB, 0.94408E+00_JPRB, 0.98174E+00_JPRB, 0.99664E+00_JPRB, 0.10133E+01_JPRB, 0.10512E+01_JPRB ,&
&  0.10943E+01_JPRB, 0.11241E+01_JPRB, 0.11514E+01_JPRB, 0.11586E+01_JPRB, 0.11746E+01_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.12623E+01_JPRB, 0.12868E+01_JPRB, 0.13101E+01_JPRB, 0.13297E+01_JPRB, 0.13398E+01_JPRB, 0.13463E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.13502E+01_JPRB, 0.13527E+01_JPRB, 0.13722E+01_JPRB, 0.14050E+01_JPRB, 0.14237E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.14627E+01_JPRB, 0.14894E+01_JPRB, 0.15282E+01_JPRB, 0.15639E+01_JPRB, 0.16130E+01_JPRB, 0.16161E+01_JPRB, 0.15936E+01_JPRB ,&
&  0.15826E+01_JPRB, 0.15662E+01_JPRB, 0.15762E+01_JPRB, 0.15906E+01_JPRB, 0.16158E+01_JPRB, 0.16665E+01_JPRB, 0.19577E+01_JPRB ,&
&  0.23568E+01_JPRB, 0.27209E+01_JPRB, 0.24336E+01_JPRB, 0.21938E+01_JPRB, 0.21855E+01_JPRB, 0.19752E+01_JPRB, 0.18509E+01_JPRB ,&
&  0.16765E+01_JPRB, 0.15152E+01_JPRB, 0.14730E+01_JPRB, 0.14683E+01_JPRB, 0.15455E+01_JPRB, 0.16303E+01_JPRB, 0.17051E+01_JPRB ,&
&  0.18358E+01_JPRB, 0.19515E+01_JPRB, 0.21016E+01_JPRB, 0.22090E+01_JPRB, 0.23034E+01_JPRB, 0.26124E+01_JPRB, 0.24894E+01_JPRB ,&
&  0.25967E+01_JPRB, 0.29403E+01_JPRB, 0.31737E+01_JPRB, 0.33563E+01_JPRB, 0.31653E+01_JPRB, 0.29431E+01_JPRB, 0.26327E+01_JPRB ,&
&  0.23184E+01_JPRB, 0.22773E+01_JPRB, 0.25988E+01_JPRB, 0.26618E+01_JPRB, 0.27010E+01_JPRB, 0.27404E+01_JPRB, 0.24551E+01_JPRB ,&
&  0.21805E+01_JPRB, 0.19841E+01_JPRB, 0.17959E+01_JPRB, 0.16285E+01_JPRB, 0.15294E+01_JPRB, 0.15110E+01_JPRB, 0.14576E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.14332E+01_JPRB, 0.15191E+01_JPRB, 0.16783E+01_JPRB, 0.19268E+01_JPRB, 0.33274E+01_JPRB, 0.39692E+01_JPRB ,&
&  0.25370E+01_JPRB, 0.20968E+01_JPRB, 0.12317E+01_JPRB, 0.17919E+01_JPRB, 0.10335E+01_JPRB, 0.80957E+00_JPRB, 0.85299E+00_JPRB ,&
&  0.95909E+00_JPRB, 0.10439E+01_JPRB, 0.10930E+01_JPRB, 0.11527E+01_JPRB, 0.10832E+01_JPRB, 0.10831E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.11582E+01_JPRB, 0.11872E+01_JPRB, 0.12106E+01_JPRB, 0.12374E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.12866E+01_JPRB, 0.13132E+01_JPRB, 0.13397E+01_JPRB, 0.13698E+01_JPRB, 0.13958E+01_JPRB, 0.14260E+01_JPRB, 0.14451E+01_JPRB ,&
&  0.14606E+01_JPRB, 0.14725E+01_JPRB, 0.14795E+01_JPRB, 0.14975E+01_JPRB, 0.15190E+01_JPRB, 0.15570E+01_JPRB, 0.16040E+01_JPRB ,&
&  0.16581E+01_JPRB, 0.17208E+01_JPRB, 0.17932E+01_JPRB, 0.18481E+01_JPRB, 0.18979E+01_JPRB, 0.19417E+01_JPRB, 0.19811E+01_JPRB ,&
&  0.20041E+01_JPRB, 0.19966E+01_JPRB, 0.20289E+01_JPRB, 0.21006E+01_JPRB, 0.21773E+01_JPRB, 0.23447E+01_JPRB, 0.21845E+01_JPRB ,&
&  0.27025E+01_JPRB, 0.28445E+01_JPRB, 0.27927E+01_JPRB, 0.26056E+01_JPRB, 0.25747E+01_JPRB, 0.25545E+01_JPRB, 0.25848E+01_JPRB ,&
&  0.26630E+01_JPRB, 0.25591E+01_JPRB, 0.24380E+01_JPRB, 0.22380E+01_JPRB, 0.21257E+01_JPRB, 0.21686E+01_JPRB, 0.23062E+01_JPRB ,&
&  0.23861E+01_JPRB, 0.24850E+01_JPRB, 0.25182E+01_JPRB, 0.25269E+01_JPRB, 0.25470E+01_JPRB, 0.25525E+01_JPRB, 0.24116E+01_JPRB ,&
&  0.23008E+01_JPRB, 0.22664E+01_JPRB, 0.19749E+01_JPRB, 0.20032E+01_JPRB, 0.20073E+01_JPRB, 0.18426E+01_JPRB, 0.15761E+01_JPRB ,&
&  0.15009E+01_JPRB, 0.14276E+01_JPRB, 0.15410E+01_JPRB, 0.19952E+01_JPRB, 0.22103E+01_JPRB, 0.22381E+01_JPRB, 0.21144E+01_JPRB ,&
&  0.19009E+01_JPRB, 0.16852E+01_JPRB, 0.14546E+01_JPRB, 0.13271E+01_JPRB, 0.13094E+01_JPRB, 0.13695E+01_JPRB, 0.13955E+01_JPRB ,&
&  0.13967E+01_JPRB, 0.13929E+01_JPRB, 0.14224E+01_JPRB, 0.14960E+01_JPRB, 0.18333E+01_JPRB, 0.24529E+01_JPRB, 0.37152E+01_JPRB ,&
&  0.26764E+01_JPRB, 0.15544E+01_JPRB, 0.14644E+01_JPRB, 0.14659E+01_JPRB, 0.10039E+01_JPRB, 0.77065E+00_JPRB, 0.84229E+00_JPRB ,&
&  0.10657E+01_JPRB, 0.12040E+01_JPRB, 0.12126E+01_JPRB, 0.12082E+01_JPRB, 0.11698E+01_JPRB, 0.11849E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.11983E+01_JPRB, 0.12168E+01_JPRB, 0.12356E+01_JPRB, 0.12602E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.12916E+01_JPRB, 0.13258E+01_JPRB, 0.13586E+01_JPRB, 0.13911E+01_JPRB, 0.14182E+01_JPRB, 0.14521E+01_JPRB, 0.14698E+01_JPRB ,&
&  0.14900E+01_JPRB, 0.15062E+01_JPRB, 0.15344E+01_JPRB, 0.15654E+01_JPRB, 0.16055E+01_JPRB, 0.16527E+01_JPRB, 0.17100E+01_JPRB ,&
&  0.17835E+01_JPRB, 0.18500E+01_JPRB, 0.19268E+01_JPRB, 0.19928E+01_JPRB, 0.20344E+01_JPRB, 0.20890E+01_JPRB, 0.21449E+01_JPRB ,&
&  0.21676E+01_JPRB, 0.21654E+01_JPRB, 0.22155E+01_JPRB, 0.22954E+01_JPRB, 0.23367E+01_JPRB, 0.24893E+01_JPRB, 0.21059E+01_JPRB ,&
&  0.28510E+01_JPRB, 0.31278E+01_JPRB, 0.32441E+01_JPRB, 0.30688E+01_JPRB, 0.29896E+01_JPRB, 0.30735E+01_JPRB, 0.31859E+01_JPRB ,&
&  0.33359E+01_JPRB, 0.34707E+01_JPRB, 0.33704E+01_JPRB, 0.31712E+01_JPRB, 0.29651E+01_JPRB, 0.29921E+01_JPRB, 0.30618E+01_JPRB ,&
&  0.31195E+01_JPRB, 0.31647E+01_JPRB, 0.31564E+01_JPRB, 0.31384E+01_JPRB, 0.31295E+01_JPRB, 0.30896E+01_JPRB, 0.29901E+01_JPRB ,&
&  0.28889E+01_JPRB, 0.27657E+01_JPRB, 0.24258E+01_JPRB, 0.26327E+01_JPRB, 0.24603E+01_JPRB, 0.20549E+01_JPRB, 0.16556E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.13836E+01_JPRB, 0.16045E+01_JPRB, 0.16987E+01_JPRB, 0.17153E+01_JPRB, 0.17118E+01_JPRB, 0.16378E+01_JPRB ,&
&  0.14443E+01_JPRB, 0.12695E+01_JPRB, 0.11684E+01_JPRB, 0.11530E+01_JPRB, 0.11889E+01_JPRB, 0.12460E+01_JPRB, 0.12785E+01_JPRB ,&
&  0.12907E+01_JPRB, 0.13016E+01_JPRB, 0.13023E+01_JPRB, 0.13043E+01_JPRB, 0.14645E+01_JPRB, 0.18021E+01_JPRB, 0.30505E+01_JPRB ,&
&  0.23525E+01_JPRB, 0.16948E+01_JPRB, 0.22794E+01_JPRB, 0.85034E+00_JPRB, 0.85168E+00_JPRB, 0.72538E+00_JPRB, 0.79418E+00_JPRB ,&
&  0.93967E+00_JPRB, 0.12936E+01_JPRB, 0.16986E+01_JPRB, 0.14953E+01_JPRB, 0.13901E+01_JPRB, 0.13006E+01_JPRB, 0.12225E+01_JPRB ,&
&  0.12025E+01_JPRB, 0.12077E+01_JPRB, 0.12209E+01_JPRB, 0.12324E+01_JPRB, 0.12583E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.13063E+01_JPRB, 0.13458E+01_JPRB, 0.13664E+01_JPRB, 0.14014E+01_JPRB, 0.13960E+01_JPRB, 0.14261E+01_JPRB, 0.14397E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.15035E+01_JPRB, 0.15440E+01_JPRB, 0.15866E+01_JPRB, 0.16411E+01_JPRB, 0.16952E+01_JPRB, 0.17568E+01_JPRB ,&
&  0.18276E+01_JPRB, 0.19084E+01_JPRB, 0.19776E+01_JPRB, 0.20716E+01_JPRB, 0.21273E+01_JPRB, 0.22154E+01_JPRB, 0.22994E+01_JPRB ,&
&  0.24222E+01_JPRB, 0.24488E+01_JPRB, 0.24873E+01_JPRB, 0.25449E+01_JPRB, 0.25659E+01_JPRB, 0.27246E+01_JPRB, 0.20481E+01_JPRB ,&
&  0.30080E+01_JPRB, 0.33533E+01_JPRB, 0.36053E+01_JPRB, 0.35030E+01_JPRB, 0.33539E+01_JPRB, 0.33456E+01_JPRB, 0.34008E+01_JPRB ,&
&  0.35322E+01_JPRB, 0.36331E+01_JPRB, 0.35750E+01_JPRB, 0.37567E+01_JPRB, 0.38372E+01_JPRB, 0.37607E+01_JPRB, 0.38846E+01_JPRB ,&
&  0.39812E+01_JPRB, 0.40677E+01_JPRB, 0.41273E+01_JPRB, 0.41958E+01_JPRB, 0.42614E+01_JPRB, 0.43770E+01_JPRB, 0.44543E+01_JPRB ,&
&  0.44313E+01_JPRB, 0.44275E+01_JPRB, 0.43851E+01_JPRB, 0.41523E+01_JPRB, 0.39013E+01_JPRB, 0.32615E+01_JPRB, 0.26438E+01_JPRB ,&
&  0.18137E+01_JPRB, 0.14963E+01_JPRB, 0.15210E+01_JPRB, 0.15349E+01_JPRB, 0.14520E+01_JPRB, 0.13914E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.10767E+01_JPRB, 0.10655E+01_JPRB, 0.10962E+01_JPRB, 0.11470E+01_JPRB, 0.11780E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.12210E+01_JPRB, 0.11778E+01_JPRB, 0.11514E+01_JPRB, 0.12736E+01_JPRB, 0.16266E+01_JPRB, 0.18466E+01_JPRB ,&
&  0.21568E+01_JPRB, 0.15976E+01_JPRB, 0.13579E+01_JPRB, 0.12052E+01_JPRB, 0.85667E+00_JPRB, 0.76934E+00_JPRB, 0.91149E+00_JPRB ,&
&  0.14426E+01_JPRB, 0.17882E+01_JPRB, 0.24697E+01_JPRB, 0.41216E+01_JPRB, 0.29382E+01_JPRB, 0.22394E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.12107E+01_JPRB, 0.12169E+01_JPRB, 0.12230E+01_JPRB, 0.12617E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.11694E+01_JPRB, 0.12160E+01_JPRB, 0.12648E+01_JPRB, 0.13144E+01_JPRB, 0.13504E+01_JPRB, 0.13714E+01_JPRB, 0.13841E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.14219E+01_JPRB, 0.14608E+01_JPRB, 0.14626E+01_JPRB, 0.15023E+01_JPRB, 0.15276E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.16542E+01_JPRB, 0.17403E+01_JPRB, 0.18390E+01_JPRB, 0.19263E+01_JPRB, 0.20146E+01_JPRB, 0.21556E+01_JPRB, 0.23309E+01_JPRB ,&
&  0.25293E+01_JPRB, 0.26623E+01_JPRB, 0.27717E+01_JPRB, 0.28536E+01_JPRB, 0.28293E+01_JPRB, 0.27931E+01_JPRB, 0.20391E+01_JPRB ,&
&  0.29375E+01_JPRB, 0.34826E+01_JPRB, 0.39453E+01_JPRB, 0.39949E+01_JPRB, 0.37748E+01_JPRB, 0.35748E+01_JPRB, 0.36079E+01_JPRB ,&
&  0.36565E+01_JPRB, 0.36074E+01_JPRB, 0.35223E+01_JPRB, 0.35813E+01_JPRB, 0.40434E+01_JPRB, 0.39615E+01_JPRB, 0.40912E+01_JPRB ,&
&  0.42251E+01_JPRB, 0.43840E+01_JPRB, 0.45863E+01_JPRB, 0.48243E+01_JPRB, 0.50647E+01_JPRB, 0.52944E+01_JPRB, 0.55379E+01_JPRB ,&
&  0.57337E+01_JPRB, 0.58108E+01_JPRB, 0.58599E+01_JPRB, 0.52059E+01_JPRB, 0.47814E+01_JPRB, 0.48031E+01_JPRB, 0.35384E+01_JPRB ,&
&  0.20310E+01_JPRB, 0.16093E+01_JPRB, 0.15844E+01_JPRB, 0.15262E+01_JPRB, 0.14302E+01_JPRB, 0.12772E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.10374E+01_JPRB, 0.10404E+01_JPRB, 0.10907E+01_JPRB, 0.11330E+01_JPRB, 0.11687E+01_JPRB, 0.11764E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.11826E+01_JPRB, 0.11370E+01_JPRB, 0.11318E+01_JPRB, 0.12685E+01_JPRB, 0.16301E+01_JPRB, 0.21925E+01_JPRB ,&
&  0.23614E+01_JPRB, 0.13757E+01_JPRB, 0.12444E+01_JPRB, 0.10365E+01_JPRB, 0.92124E+00_JPRB, 0.91906E+00_JPRB, 0.10841E+01_JPRB ,&
&  0.15692E+01_JPRB, 0.23615E+01_JPRB, 0.25552E+01_JPRB, 0.21732E+01_JPRB, 0.26151E+01_JPRB, 0.19309E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11314E+01_JPRB, 0.11044E+01_JPRB, 0.11033E+01_JPRB, 0.11237E+01_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.96935E+00_JPRB, 0.10029E+01_JPRB, 0.10383E+01_JPRB, 0.10880E+01_JPRB, 0.11450E+01_JPRB, 0.11886E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.12539E+01_JPRB, 0.12934E+01_JPRB, 0.13331E+01_JPRB, 0.13304E+01_JPRB, 0.13687E+01_JPRB, 0.13915E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.15286E+01_JPRB, 0.16077E+01_JPRB, 0.17067E+01_JPRB, 0.18434E+01_JPRB, 0.20032E+01_JPRB, 0.21929E+01_JPRB ,&
&  0.24216E+01_JPRB, 0.26348E+01_JPRB, 0.28446E+01_JPRB, 0.29973E+01_JPRB, 0.31166E+01_JPRB, 0.31874E+01_JPRB, 0.32336E+01_JPRB ,&
&  0.19100E+01_JPRB, 0.32301E+01_JPRB, 0.36822E+01_JPRB, 0.39923E+01_JPRB, 0.42047E+01_JPRB, 0.40132E+01_JPRB, 0.40374E+01_JPRB ,&
&  0.38222E+01_JPRB, 0.34981E+01_JPRB, 0.34484E+01_JPRB, 0.34741E+01_JPRB, 0.37008E+01_JPRB, 0.36712E+01_JPRB, 0.37234E+01_JPRB ,&
&  0.38021E+01_JPRB, 0.39641E+01_JPRB, 0.41883E+01_JPRB, 0.44491E+01_JPRB, 0.47646E+01_JPRB, 0.51391E+01_JPRB, 0.55873E+01_JPRB ,&
&  0.60642E+01_JPRB, 0.64903E+01_JPRB, 0.68728E+01_JPRB, 0.60853E+01_JPRB, 0.53801E+01_JPRB, 0.55268E+01_JPRB, 0.45680E+01_JPRB ,&
&  0.25400E+01_JPRB, 0.16659E+01_JPRB, 0.16688E+01_JPRB, 0.17408E+01_JPRB, 0.15859E+01_JPRB, 0.14451E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.11575E+01_JPRB, 0.11004E+01_JPRB, 0.11470E+01_JPRB, 0.11394E+01_JPRB, 0.11951E+01_JPRB, 0.12088E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.12679E+01_JPRB, 0.12496E+01_JPRB, 0.12463E+01_JPRB, 0.12392E+01_JPRB, 0.12358E+01_JPRB, 0.12429E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.11819E+01_JPRB, 0.11153E+01_JPRB, 0.10990E+01_JPRB, 0.10615E+01_JPRB, 0.10335E+01_JPRB, 0.11142E+01_JPRB ,&
&  0.12612E+01_JPRB, 0.14434E+01_JPRB, 0.17316E+01_JPRB, 0.17933E+01_JPRB, 0.16778E+01_JPRB, 0.13010E+01_JPRB, 0.10004E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.96901E+00_JPRB, 0.94079E+00_JPRB, 0.92728E+00_JPRB, 0.94011E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.87364E+00_JPRB, 0.90199E+00_JPRB, 0.90015E+00_JPRB, 0.93592E+00_JPRB, 0.96113E+00_JPRB, 0.99752E+00_JPRB, 0.10421E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.11127E+01_JPRB, 0.11656E+01_JPRB, 0.11776E+01_JPRB, 0.12235E+01_JPRB, 0.12564E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.13518E+01_JPRB, 0.13952E+01_JPRB, 0.14483E+01_JPRB, 0.15313E+01_JPRB, 0.16658E+01_JPRB, 0.18172E+01_JPRB ,&
&  0.20027E+01_JPRB, 0.22289E+01_JPRB, 0.25256E+01_JPRB, 0.28288E+01_JPRB, 0.31109E+01_JPRB, 0.32676E+01_JPRB, 0.32881E+01_JPRB ,&
&  0.15600E+01_JPRB, 0.19371E+01_JPRB, 0.31492E+01_JPRB, 0.36383E+01_JPRB, 0.39700E+01_JPRB, 0.41071E+01_JPRB, 0.40804E+01_JPRB ,&
&  0.37425E+01_JPRB, 0.32137E+01_JPRB, 0.30871E+01_JPRB, 0.34490E+01_JPRB, 0.33459E+01_JPRB, 0.33481E+01_JPRB, 0.33506E+01_JPRB ,&
&  0.33800E+01_JPRB, 0.34638E+01_JPRB, 0.36082E+01_JPRB, 0.38053E+01_JPRB, 0.41178E+01_JPRB, 0.45326E+01_JPRB, 0.50760E+01_JPRB ,&
&  0.57411E+01_JPRB, 0.64327E+01_JPRB, 0.70492E+01_JPRB, 0.62202E+01_JPRB, 0.60877E+01_JPRB, 0.60619E+01_JPRB, 0.62836E+01_JPRB ,&
&  0.32985E+01_JPRB, 0.18875E+01_JPRB, 0.19217E+01_JPRB, 0.20188E+01_JPRB, 0.17878E+01_JPRB, 0.16844E+01_JPRB, 0.15793E+01_JPRB ,&
&  0.14306E+01_JPRB, 0.13308E+01_JPRB, 0.13105E+01_JPRB, 0.12823E+01_JPRB, 0.12831E+01_JPRB, 0.13132E+01_JPRB, 0.13646E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.14261E+01_JPRB, 0.14309E+01_JPRB, 0.14248E+01_JPRB, 0.13801E+01_JPRB, 0.13300E+01_JPRB, 0.13023E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.12574E+01_JPRB, 0.12438E+01_JPRB, 0.12702E+01_JPRB, 0.12842E+01_JPRB, 0.12750E+01_JPRB, 0.12680E+01_JPRB ,&
&  0.12290E+01_JPRB, 0.12207E+01_JPRB, 0.13183E+01_JPRB, 0.13361E+01_JPRB, 0.12861E+01_JPRB, 0.10924E+01_JPRB, 0.92430E+00_JPRB ,&
&  0.90927E+00_JPRB, 0.88422E+00_JPRB, 0.86256E+00_JPRB, 0.86030E+00_JPRB, 0.86045E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.80549E+00_JPRB, 0.79271E+00_JPRB, 0.77644E+00_JPRB, 0.79022E+00_JPRB, 0.80472E+00_JPRB, 0.82382E+00_JPRB, 0.88454E+00_JPRB ,&
&  0.93983E+00_JPRB, 0.95922E+00_JPRB, 0.99019E+00_JPRB, 0.10084E+01_JPRB, 0.10432E+01_JPRB, 0.10653E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11309E+01_JPRB, 0.11653E+01_JPRB, 0.12124E+01_JPRB, 0.12573E+01_JPRB, 0.12951E+01_JPRB, 0.13517E+01_JPRB, 0.14354E+01_JPRB ,&
&  0.15251E+01_JPRB, 0.16411E+01_JPRB, 0.18315E+01_JPRB, 0.21614E+01_JPRB, 0.26803E+01_JPRB, 0.33038E+01_JPRB, 0.38078E+01_JPRB ,&
&  0.29704E+01_JPRB, 0.11702E+01_JPRB, 0.24636E+01_JPRB, 0.33791E+01_JPRB, 0.35426E+01_JPRB, 0.37487E+01_JPRB, 0.36115E+01_JPRB ,&
&  0.32813E+01_JPRB, 0.29519E+01_JPRB, 0.27461E+01_JPRB, 0.30822E+01_JPRB, 0.30874E+01_JPRB, 0.30853E+01_JPRB, 0.30763E+01_JPRB ,&
&  0.30518E+01_JPRB, 0.30679E+01_JPRB, 0.31146E+01_JPRB, 0.32228E+01_JPRB, 0.34512E+01_JPRB, 0.38260E+01_JPRB, 0.43339E+01_JPRB ,&
&  0.50283E+01_JPRB, 0.59813E+01_JPRB, 0.69446E+01_JPRB, 0.64378E+01_JPRB, 0.66517E+01_JPRB, 0.59116E+01_JPRB, 0.50754E+01_JPRB ,&
&  0.30383E+01_JPRB, 0.21156E+01_JPRB, 0.21169E+01_JPRB, 0.22879E+01_JPRB, 0.20502E+01_JPRB, 0.17958E+01_JPRB, 0.18673E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.16076E+01_JPRB, 0.15387E+01_JPRB, 0.14879E+01_JPRB, 0.14729E+01_JPRB, 0.14692E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.14712E+01_JPRB, 0.14991E+01_JPRB, 0.15297E+01_JPRB, 0.15256E+01_JPRB, 0.14909E+01_JPRB, 0.14608E+01_JPRB, 0.14549E+01_JPRB ,&
&  0.14604E+01_JPRB, 0.14485E+01_JPRB, 0.14592E+01_JPRB, 0.14730E+01_JPRB, 0.14671E+01_JPRB, 0.14144E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.13392E+01_JPRB, 0.12870E+01_JPRB, 0.12098E+01_JPRB, 0.11501E+01_JPRB, 0.10513E+01_JPRB, 0.94424E+00_JPRB ,&
&  0.90796E+00_JPRB, 0.86909E+00_JPRB, 0.84543E+00_JPRB, 0.83270E+00_JPRB, 0.82063E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.87872E+00_JPRB, 0.85837E+00_JPRB, 0.82210E+00_JPRB, 0.76826E+00_JPRB, 0.76213E+00_JPRB, 0.75945E+00_JPRB, 0.77003E+00_JPRB ,&
&  0.82225E+00_JPRB, 0.88411E+00_JPRB, 0.90394E+00_JPRB, 0.90690E+00_JPRB, 0.88429E+00_JPRB, 0.89670E+00_JPRB, 0.92019E+00_JPRB ,&
&  0.95597E+00_JPRB, 0.99199E+00_JPRB, 0.10396E+01_JPRB, 0.10978E+01_JPRB, 0.11469E+01_JPRB, 0.11877E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.12755E+01_JPRB, 0.13165E+01_JPRB, 0.13995E+01_JPRB, 0.15935E+01_JPRB, 0.20350E+01_JPRB, 0.28176E+01_JPRB ,&
&  0.45795E+01_JPRB, 0.38579E+01_JPRB, 0.16730E+01_JPRB, 0.30032E+01_JPRB, 0.34212E+01_JPRB, 0.37304E+01_JPRB, 0.31612E+01_JPRB ,&
&  0.30200E+01_JPRB, 0.26412E+01_JPRB, 0.23535E+01_JPRB, 0.25694E+01_JPRB, 0.25994E+01_JPRB, 0.26255E+01_JPRB, 0.26250E+01_JPRB ,&
&  0.25541E+01_JPRB, 0.25705E+01_JPRB, 0.26105E+01_JPRB, 0.26658E+01_JPRB, 0.27698E+01_JPRB, 0.30343E+01_JPRB, 0.35327E+01_JPRB ,&
&  0.41851E+01_JPRB, 0.50805E+01_JPRB, 0.64395E+01_JPRB, 0.70594E+01_JPRB, 0.77323E+01_JPRB, 0.57754E+01_JPRB, 0.44445E+01_JPRB ,&
&  0.28694E+01_JPRB, 0.25540E+01_JPRB, 0.26324E+01_JPRB, 0.26033E+01_JPRB, 0.23565E+01_JPRB, 0.21490E+01_JPRB, 0.22164E+01_JPRB ,&
&  0.20078E+01_JPRB, 0.18690E+01_JPRB, 0.17918E+01_JPRB, 0.17180E+01_JPRB, 0.16664E+01_JPRB, 0.16381E+01_JPRB, 0.16144E+01_JPRB ,&
&  0.15913E+01_JPRB, 0.15997E+01_JPRB, 0.16179E+01_JPRB, 0.16320E+01_JPRB, 0.16281E+01_JPRB, 0.16027E+01_JPRB, 0.15966E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.15965E+01_JPRB, 0.16015E+01_JPRB, 0.16158E+01_JPRB, 0.15612E+01_JPRB, 0.14849E+01_JPRB, 0.14836E+01_JPRB ,&
&  0.14610E+01_JPRB, 0.14556E+01_JPRB, 0.14033E+01_JPRB, 0.13728E+01_JPRB, 0.12562E+01_JPRB, 0.11314E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.98775E+00_JPRB, 0.96297E+00_JPRB, 0.91333E+00_JPRB, 0.87231E+00_JPRB, 0.86941E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.97465E+00_JPRB, 0.98627E+00_JPRB, 0.97990E+00_JPRB, 0.92438E+00_JPRB, 0.87387E+00_JPRB, 0.82525E+00_JPRB, 0.79322E+00_JPRB ,&
&  0.78499E+00_JPRB, 0.78622E+00_JPRB, 0.81638E+00_JPRB, 0.84622E+00_JPRB, 0.83988E+00_JPRB, 0.81263E+00_JPRB, 0.80558E+00_JPRB ,&
&  0.82049E+00_JPRB, 0.85707E+00_JPRB, 0.89508E+00_JPRB, 0.95271E+00_JPRB, 0.10046E+01_JPRB, 0.10415E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.10824E+01_JPRB, 0.10930E+01_JPRB, 0.11138E+01_JPRB, 0.11587E+01_JPRB, 0.12812E+01_JPRB, 0.14870E+01_JPRB ,&
&  0.21488E+01_JPRB, 0.39062E+01_JPRB, 0.21295E+01_JPRB, 0.30569E+01_JPRB, 0.36594E+01_JPRB, 0.37370E+01_JPRB, 0.32687E+01_JPRB ,&
&  0.29497E+01_JPRB, 0.25189E+01_JPRB, 0.22777E+01_JPRB, 0.22595E+01_JPRB, 0.20705E+01_JPRB, 0.20621E+01_JPRB, 0.20590E+01_JPRB ,&
&  0.20772E+01_JPRB, 0.21235E+01_JPRB, 0.21651E+01_JPRB, 0.22009E+01_JPRB, 0.22108E+01_JPRB, 0.22813E+01_JPRB, 0.25583E+01_JPRB ,&
&  0.31261E+01_JPRB, 0.39636E+01_JPRB, 0.54058E+01_JPRB, 0.67298E+01_JPRB, 0.82428E+01_JPRB, 0.63950E+01_JPRB, 0.46994E+01_JPRB ,&
&  0.34039E+01_JPRB, 0.31962E+01_JPRB, 0.31845E+01_JPRB, 0.31219E+01_JPRB, 0.27904E+01_JPRB, 0.27634E+01_JPRB, 0.28070E+01_JPRB ,&
&  0.24860E+01_JPRB, 0.22248E+01_JPRB, 0.20870E+01_JPRB, 0.19906E+01_JPRB, 0.18903E+01_JPRB, 0.18142E+01_JPRB, 0.17633E+01_JPRB ,&
&  0.17260E+01_JPRB, 0.17097E+01_JPRB, 0.17239E+01_JPRB, 0.17405E+01_JPRB, 0.17566E+01_JPRB, 0.17479E+01_JPRB, 0.17288E+01_JPRB ,&
&  0.16883E+01_JPRB, 0.16653E+01_JPRB, 0.16539E+01_JPRB, 0.16434E+01_JPRB, 0.16015E+01_JPRB, 0.15440E+01_JPRB, 0.15299E+01_JPRB ,&
&  0.15256E+01_JPRB, 0.15448E+01_JPRB, 0.15132E+01_JPRB, 0.14482E+01_JPRB, 0.14311E+01_JPRB, 0.13148E+01_JPRB, 0.11915E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.12947E+01_JPRB, 0.13186E+01_JPRB, 0.97139E+00_JPRB, 0.96356E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.10571E+01_JPRB, 0.10558E+01_JPRB, 0.10411E+01_JPRB, 0.10353E+01_JPRB, 0.99685E+00_JPRB, 0.94525E+00_JPRB, 0.89479E+00_JPRB ,&
&  0.85782E+00_JPRB, 0.83256E+00_JPRB, 0.82333E+00_JPRB, 0.81565E+00_JPRB, 0.81502E+00_JPRB, 0.80651E+00_JPRB, 0.80277E+00_JPRB ,&
&  0.79341E+00_JPRB, 0.80445E+00_JPRB, 0.82794E+00_JPRB, 0.87040E+00_JPRB, 0.91941E+00_JPRB, 0.93442E+00_JPRB, 0.94810E+00_JPRB ,&
&  0.95905E+00_JPRB, 0.96030E+00_JPRB, 0.96295E+00_JPRB, 0.97569E+00_JPRB, 0.10019E+01_JPRB, 0.10473E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.14192E+01_JPRB, 0.31992E+01_JPRB, 0.48473E+01_JPRB, 0.54609E+01_JPRB, 0.48309E+01_JPRB, 0.41185E+01_JPRB, 0.36661E+01_JPRB ,&
&  0.35597E+01_JPRB, 0.32167E+01_JPRB, 0.28194E+01_JPRB, 0.23004E+01_JPRB, 0.19165E+01_JPRB, 0.17827E+01_JPRB, 0.17627E+01_JPRB ,&
&  0.18442E+01_JPRB, 0.18900E+01_JPRB, 0.19055E+01_JPRB, 0.18895E+01_JPRB, 0.18397E+01_JPRB, 0.18176E+01_JPRB, 0.18763E+01_JPRB ,&
&  0.21213E+01_JPRB, 0.26844E+01_JPRB, 0.37511E+01_JPRB, 0.56067E+01_JPRB, 0.76125E+01_JPRB, 0.68033E+01_JPRB, 0.54615E+01_JPRB ,&
&  0.49943E+01_JPRB, 0.45084E+01_JPRB, 0.40867E+01_JPRB, 0.38563E+01_JPRB, 0.30972E+01_JPRB, 0.37165E+01_JPRB, 0.34965E+01_JPRB ,&
&  0.32164E+01_JPRB, 0.30235E+01_JPRB, 0.27478E+01_JPRB, 0.24485E+01_JPRB, 0.22269E+01_JPRB, 0.20253E+01_JPRB, 0.18723E+01_JPRB ,&
&  0.18182E+01_JPRB, 0.18224E+01_JPRB, 0.18443E+01_JPRB, 0.18859E+01_JPRB, 0.19327E+01_JPRB, 0.19449E+01_JPRB, 0.19026E+01_JPRB ,&
&  0.18041E+01_JPRB, 0.17170E+01_JPRB, 0.15950E+01_JPRB, 0.15758E+01_JPRB, 0.15595E+01_JPRB, 0.15068E+01_JPRB, 0.14734E+01_JPRB ,&
&  0.14928E+01_JPRB, 0.15413E+01_JPRB, 0.15476E+01_JPRB, 0.15137E+01_JPRB, 0.14695E+01_JPRB, 0.14291E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.12688E+01_JPRB, 0.11838E+01_JPRB, 0.11048E+01_JPRB, 0.10797E+01_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.11335E+01_JPRB, 0.10931E+01_JPRB, 0.10687E+01_JPRB, 0.10626E+01_JPRB, 0.10399E+01_JPRB, 0.10219E+01_JPRB, 0.10088E+01_JPRB ,&
&  0.95197E+00_JPRB, 0.90566E+00_JPRB, 0.86187E+00_JPRB, 0.83908E+00_JPRB, 0.83309E+00_JPRB, 0.82678E+00_JPRB, 0.83534E+00_JPRB ,&
&  0.83160E+00_JPRB, 0.81750E+00_JPRB, 0.81774E+00_JPRB, 0.84150E+00_JPRB, 0.87376E+00_JPRB, 0.88260E+00_JPRB, 0.90047E+00_JPRB ,&
&  0.92094E+00_JPRB, 0.92182E+00_JPRB, 0.91285E+00_JPRB, 0.91215E+00_JPRB, 0.91925E+00_JPRB, 0.92675E+00_JPRB, 0.97786E+00_JPRB ,&
&  0.12349E+01_JPRB, 0.32325E+01_JPRB, 0.32284E+01_JPRB, 0.62569E+01_JPRB, 0.54670E+01_JPRB, 0.47158E+01_JPRB, 0.37322E+01_JPRB ,&
&  0.37898E+01_JPRB, 0.41979E+01_JPRB, 0.32227E+01_JPRB, 0.25821E+01_JPRB, 0.21085E+01_JPRB, 0.17858E+01_JPRB, 0.16834E+01_JPRB ,&
&  0.17188E+01_JPRB, 0.17798E+01_JPRB, 0.18046E+01_JPRB, 0.17388E+01_JPRB, 0.16595E+01_JPRB, 0.16214E+01_JPRB, 0.16277E+01_JPRB ,&
&  0.16753E+01_JPRB, 0.18456E+01_JPRB, 0.23715E+01_JPRB, 0.34757E+01_JPRB, 0.59457E+01_JPRB, 0.70797E+01_JPRB, 0.64162E+01_JPRB ,&
&  0.61116E+01_JPRB, 0.62519E+01_JPRB, 0.56694E+01_JPRB, 0.50164E+01_JPRB, 0.41373E+01_JPRB, 0.41357E+01_JPRB, 0.39095E+01_JPRB ,&
&  0.36604E+01_JPRB, 0.35867E+01_JPRB, 0.34609E+01_JPRB, 0.31890E+01_JPRB, 0.28525E+01_JPRB, 0.25053E+01_JPRB, 0.21887E+01_JPRB ,&
&  0.19670E+01_JPRB, 0.18665E+01_JPRB, 0.18284E+01_JPRB, 0.18819E+01_JPRB, 0.19861E+01_JPRB, 0.20582E+01_JPRB, 0.20684E+01_JPRB ,&
&  0.19879E+01_JPRB, 0.18580E+01_JPRB, 0.16340E+01_JPRB, 0.15243E+01_JPRB, 0.15044E+01_JPRB, 0.14519E+01_JPRB, 0.14253E+01_JPRB ,&
&  0.14774E+01_JPRB, 0.15227E+01_JPRB, 0.15206E+01_JPRB, 0.14911E+01_JPRB, 0.14715E+01_JPRB, 0.14910E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.13300E+01_JPRB, 0.13260E+01_JPRB, 0.13106E+01_JPRB, 0.12393E+01_JPRB, 0.11906E+01_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.12144E+01_JPRB, 0.11512E+01_JPRB, 0.11020E+01_JPRB, 0.10787E+01_JPRB, 0.10689E+01_JPRB, 0.10591E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.10120E+01_JPRB, 0.96789E+00_JPRB, 0.92206E+00_JPRB, 0.88314E+00_JPRB, 0.87263E+00_JPRB, 0.86447E+00_JPRB, 0.86447E+00_JPRB ,&
&  0.85910E+00_JPRB, 0.84419E+00_JPRB, 0.83218E+00_JPRB, 0.83915E+00_JPRB, 0.84797E+00_JPRB, 0.85894E+00_JPRB, 0.87943E+00_JPRB ,&
&  0.88706E+00_JPRB, 0.89710E+00_JPRB, 0.90007E+00_JPRB, 0.89579E+00_JPRB, 0.88700E+00_JPRB, 0.87855E+00_JPRB, 0.89058E+00_JPRB ,&
&  0.12044E+01_JPRB, 0.40988E+01_JPRB, 0.29468E+01_JPRB, 0.57434E+01_JPRB, 0.50231E+01_JPRB, 0.45615E+01_JPRB, 0.36599E+01_JPRB ,&
&  0.34740E+01_JPRB, 0.38242E+01_JPRB, 0.32779E+01_JPRB, 0.27703E+01_JPRB, 0.24034E+01_JPRB, 0.20870E+01_JPRB, 0.18647E+01_JPRB ,&
&  0.17829E+01_JPRB, 0.17542E+01_JPRB, 0.17171E+01_JPRB, 0.16516E+01_JPRB, 0.16005E+01_JPRB, 0.15648E+01_JPRB, 0.15602E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.16010E+01_JPRB, 0.17898E+01_JPRB, 0.24709E+01_JPRB, 0.40207E+01_JPRB, 0.60408E+01_JPRB, 0.73734E+01_JPRB ,&
&  0.75027E+01_JPRB, 0.81406E+01_JPRB, 0.72949E+01_JPRB, 0.62402E+01_JPRB, 0.53924E+01_JPRB, 0.48982E+01_JPRB, 0.44521E+01_JPRB ,&
&  0.39950E+01_JPRB, 0.37187E+01_JPRB, 0.36312E+01_JPRB, 0.35676E+01_JPRB, 0.34277E+01_JPRB, 0.31776E+01_JPRB, 0.28517E+01_JPRB ,&
&  0.25082E+01_JPRB, 0.22590E+01_JPRB, 0.20914E+01_JPRB, 0.20189E+01_JPRB, 0.20248E+01_JPRB, 0.20748E+01_JPRB, 0.20973E+01_JPRB ,&
&  0.20207E+01_JPRB, 0.19181E+01_JPRB, 0.17197E+01_JPRB, 0.15672E+01_JPRB, 0.15213E+01_JPRB, 0.14629E+01_JPRB, 0.14233E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.15147E+01_JPRB, 0.14971E+01_JPRB, 0.14736E+01_JPRB, 0.14361E+01_JPRB, 0.15070E+01_JPRB, 0.14161E+01_JPRB ,&
&  0.13675E+01_JPRB, 0.13459E+01_JPRB, 0.13281E+01_JPRB, 0.12976E+01_JPRB, 0.12693E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.12647E+01_JPRB, 0.12181E+01_JPRB, 0.11686E+01_JPRB, 0.11292E+01_JPRB, 0.10985E+01_JPRB, 0.10828E+01_JPRB, 0.10700E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.10065E+01_JPRB, 0.97354E+00_JPRB, 0.94604E+00_JPRB, 0.92536E+00_JPRB, 0.91358E+00_JPRB, 0.89998E+00_JPRB ,&
&  0.88289E+00_JPRB, 0.85313E+00_JPRB, 0.85156E+00_JPRB, 0.86201E+00_JPRB, 0.86214E+00_JPRB, 0.86434E+00_JPRB, 0.88893E+00_JPRB ,&
&  0.88518E+00_JPRB, 0.88489E+00_JPRB, 0.88827E+00_JPRB, 0.89029E+00_JPRB, 0.87908E+00_JPRB, 0.87775E+00_JPRB, 0.87488E+00_JPRB ,&
&  0.11484E+01_JPRB, 0.15868E+01_JPRB, 0.35601E+01_JPRB, 0.49853E+01_JPRB, 0.45400E+01_JPRB, 0.40736E+01_JPRB, 0.36828E+01_JPRB ,&
&  0.35636E+01_JPRB, 0.34380E+01_JPRB, 0.30219E+01_JPRB, 0.27156E+01_JPRB, 0.24364E+01_JPRB, 0.22173E+01_JPRB, 0.20490E+01_JPRB ,&
&  0.19197E+01_JPRB, 0.17994E+01_JPRB, 0.17069E+01_JPRB, 0.16179E+01_JPRB, 0.15538E+01_JPRB, 0.15185E+01_JPRB, 0.15094E+01_JPRB ,&
&  0.15026E+01_JPRB, 0.14982E+01_JPRB, 0.15497E+01_JPRB, 0.19010E+01_JPRB, 0.31843E+01_JPRB, 0.48896E+01_JPRB, 0.67863E+01_JPRB ,&
&  0.81636E+01_JPRB, 0.87694E+01_JPRB, 0.80293E+01_JPRB, 0.70964E+01_JPRB, 0.61636E+01_JPRB, 0.54436E+01_JPRB, 0.48850E+01_JPRB ,&
&  0.43789E+01_JPRB, 0.39390E+01_JPRB, 0.35950E+01_JPRB, 0.34097E+01_JPRB, 0.32998E+01_JPRB, 0.32147E+01_JPRB, 0.31095E+01_JPRB ,&
&  0.29532E+01_JPRB, 0.27251E+01_JPRB, 0.24835E+01_JPRB, 0.22897E+01_JPRB, 0.21744E+01_JPRB, 0.21301E+01_JPRB, 0.20961E+01_JPRB ,&
&  0.19958E+01_JPRB, 0.18896E+01_JPRB, 0.17337E+01_JPRB, 0.16280E+01_JPRB, 0.16063E+01_JPRB, 0.15640E+01_JPRB, 0.15219E+01_JPRB ,&
&  0.15170E+01_JPRB, 0.15221E+01_JPRB, 0.14970E+01_JPRB, 0.14819E+01_JPRB, 0.14264E+01_JPRB, 0.15012E+01_JPRB, 0.14400E+01_JPRB ,&
&  0.13845E+01_JPRB, 0.13608E+01_JPRB, 0.13280E+01_JPRB, 0.13038E+01_JPRB, 0.12910E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.12977E+01_JPRB, 0.12707E+01_JPRB, 0.12175E+01_JPRB, 0.11659E+01_JPRB, 0.11284E+01_JPRB, 0.11021E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10239E+01_JPRB, 0.99510E+00_JPRB, 0.97719E+00_JPRB, 0.97022E+00_JPRB, 0.96255E+00_JPRB, 0.94292E+00_JPRB ,&
&  0.91664E+00_JPRB, 0.89254E+00_JPRB, 0.88828E+00_JPRB, 0.89289E+00_JPRB, 0.89505E+00_JPRB, 0.87919E+00_JPRB, 0.88881E+00_JPRB ,&
&  0.89510E+00_JPRB, 0.89323E+00_JPRB, 0.89043E+00_JPRB, 0.88593E+00_JPRB, 0.87941E+00_JPRB, 0.87442E+00_JPRB, 0.86850E+00_JPRB ,&
&  0.10573E+01_JPRB, 0.22868E+01_JPRB, 0.35943E+01_JPRB, 0.43598E+01_JPRB, 0.42635E+01_JPRB, 0.36971E+01_JPRB, 0.33581E+01_JPRB ,&
&  0.30668E+01_JPRB, 0.29330E+01_JPRB, 0.26529E+01_JPRB, 0.24495E+01_JPRB, 0.22572E+01_JPRB, 0.20885E+01_JPRB, 0.19391E+01_JPRB ,&
&  0.18275E+01_JPRB, 0.17374E+01_JPRB, 0.16526E+01_JPRB, 0.15791E+01_JPRB, 0.15234E+01_JPRB, 0.14793E+01_JPRB, 0.14554E+01_JPRB ,&
&  0.14429E+01_JPRB, 0.14290E+01_JPRB, 0.14363E+01_JPRB, 0.15800E+01_JPRB, 0.24689E+01_JPRB, 0.31266E+01_JPRB, 0.53023E+01_JPRB ,&
&  0.70552E+01_JPRB, 0.67384E+01_JPRB, 0.64549E+01_JPRB, 0.59479E+01_JPRB, 0.52978E+01_JPRB, 0.47470E+01_JPRB, 0.41657E+01_JPRB ,&
&  0.36630E+01_JPRB, 0.32680E+01_JPRB, 0.29462E+01_JPRB, 0.27256E+01_JPRB, 0.25902E+01_JPRB, 0.25215E+01_JPRB, 0.24970E+01_JPRB ,&
&  0.24863E+01_JPRB, 0.24689E+01_JPRB, 0.24261E+01_JPRB, 0.23469E+01_JPRB, 0.22340E+01_JPRB, 0.21149E+01_JPRB, 0.19975E+01_JPRB ,&
&  0.18764E+01_JPRB, 0.17947E+01_JPRB, 0.16901E+01_JPRB, 0.16207E+01_JPRB, 0.16393E+01_JPRB, 0.16746E+01_JPRB, 0.16412E+01_JPRB ,&
&  0.15674E+01_JPRB, 0.15341E+01_JPRB, 0.15169E+01_JPRB, 0.14601E+01_JPRB, 0.14047E+01_JPRB, 0.15007E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.14114E+01_JPRB, 0.13763E+01_JPRB, 0.13454E+01_JPRB, 0.13121E+01_JPRB, 0.13111E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.13219E+01_JPRB, 0.12738E+01_JPRB, 0.12260E+01_JPRB, 0.11778E+01_JPRB, 0.11425E+01_JPRB, 0.11057E+01_JPRB, 0.10705E+01_JPRB ,&
&  0.10423E+01_JPRB, 0.10259E+01_JPRB, 0.10112E+01_JPRB, 0.99004E+00_JPRB, 0.98569E+00_JPRB, 0.97756E+00_JPRB, 0.95868E+00_JPRB ,&
&  0.92943E+00_JPRB, 0.90757E+00_JPRB, 0.89750E+00_JPRB, 0.89442E+00_JPRB, 0.89959E+00_JPRB, 0.89743E+00_JPRB, 0.90186E+00_JPRB ,&
&  0.90858E+00_JPRB, 0.89343E+00_JPRB, 0.88082E+00_JPRB, 0.87725E+00_JPRB, 0.87369E+00_JPRB, 0.87319E+00_JPRB, 0.87518E+00_JPRB ,&
&  0.10094E+01_JPRB, 0.17135E+01_JPRB, 0.24766E+01_JPRB, 0.34588E+01_JPRB, 0.35988E+01_JPRB, 0.32721E+01_JPRB, 0.29619E+01_JPRB ,&
&  0.27116E+01_JPRB, 0.25284E+01_JPRB, 0.23049E+01_JPRB, 0.20456E+01_JPRB, 0.18414E+01_JPRB, 0.17015E+01_JPRB, 0.16014E+01_JPRB ,&
&  0.15403E+01_JPRB, 0.14930E+01_JPRB, 0.14486E+01_JPRB, 0.14079E+01_JPRB, 0.13712E+01_JPRB, 0.13395E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.13099E+01_JPRB, 0.13016E+01_JPRB, 0.13026E+01_JPRB, 0.13403E+01_JPRB, 0.17146E+01_JPRB, 0.21122E+01_JPRB, 0.30811E+01_JPRB ,&
&  0.35152E+01_JPRB, 0.37258E+01_JPRB, 0.39139E+01_JPRB, 0.38706E+01_JPRB, 0.36355E+01_JPRB, 0.33908E+01_JPRB, 0.31291E+01_JPRB ,&
&  0.28467E+01_JPRB, 0.25679E+01_JPRB, 0.23763E+01_JPRB, 0.22375E+01_JPRB, 0.21091E+01_JPRB, 0.20139E+01_JPRB, 0.19588E+01_JPRB ,&
&  0.19493E+01_JPRB, 0.19853E+01_JPRB, 0.20303E+01_JPRB, 0.20606E+01_JPRB, 0.20412E+01_JPRB, 0.19884E+01_JPRB, 0.18991E+01_JPRB ,&
&  0.17791E+01_JPRB, 0.16816E+01_JPRB, 0.16282E+01_JPRB, 0.16106E+01_JPRB, 0.16207E+01_JPRB, 0.16287E+01_JPRB, 0.16339E+01_JPRB ,&
&  0.16267E+01_JPRB, 0.15930E+01_JPRB, 0.15515E+01_JPRB, 0.14958E+01_JPRB, 0.14459E+01_JPRB, 0.15137E+01_JPRB, 0.14857E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.14002E+01_JPRB, 0.13691E+01_JPRB, 0.13359E+01_JPRB, 0.13574E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.13807E+01_JPRB, 0.13021E+01_JPRB, 0.12406E+01_JPRB, 0.11943E+01_JPRB, 0.11629E+01_JPRB, 0.11247E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10348E+01_JPRB, 0.10208E+01_JPRB, 0.99502E+00_JPRB, 0.97400E+00_JPRB, 0.95491E+00_JPRB, 0.93822E+00_JPRB ,&
&  0.91393E+00_JPRB, 0.90173E+00_JPRB, 0.89189E+00_JPRB, 0.88846E+00_JPRB, 0.88260E+00_JPRB, 0.88119E+00_JPRB, 0.88851E+00_JPRB ,&
&  0.89236E+00_JPRB, 0.87604E+00_JPRB, 0.87275E+00_JPRB, 0.86813E+00_JPRB, 0.86202E+00_JPRB, 0.85763E+00_JPRB, 0.85724E+00_JPRB ,&
&  0.99649E+00_JPRB, 0.12862E+01_JPRB, 0.17399E+01_JPRB, 0.22454E+01_JPRB, 0.24800E+01_JPRB, 0.25608E+01_JPRB, 0.24565E+01_JPRB ,&
&  0.22942E+01_JPRB, 0.21131E+01_JPRB, 0.19194E+01_JPRB, 0.17381E+01_JPRB, 0.15673E+01_JPRB, 0.14422E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.12785E+01_JPRB, 0.12544E+01_JPRB, 0.12343E+01_JPRB, 0.12143E+01_JPRB, 0.11905E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.11612E+01_JPRB, 0.11591E+01_JPRB, 0.11805E+01_JPRB, 0.12479E+01_JPRB, 0.14105E+01_JPRB, 0.18486E+01_JPRB ,&
&  0.21334E+01_JPRB, 0.23530E+01_JPRB, 0.25808E+01_JPRB, 0.26655E+01_JPRB, 0.25918E+01_JPRB, 0.24588E+01_JPRB, 0.22778E+01_JPRB ,&
&  0.21325E+01_JPRB, 0.20291E+01_JPRB, 0.19339E+01_JPRB, 0.18406E+01_JPRB, 0.17629E+01_JPRB, 0.17046E+01_JPRB, 0.16646E+01_JPRB ,&
&  0.16369E+01_JPRB, 0.16407E+01_JPRB, 0.16665E+01_JPRB, 0.16968E+01_JPRB, 0.17195E+01_JPRB, 0.17219E+01_JPRB, 0.17023E+01_JPRB ,&
&  0.16425E+01_JPRB, 0.15614E+01_JPRB, 0.15118E+01_JPRB, 0.15131E+01_JPRB, 0.15463E+01_JPRB, 0.15662E+01_JPRB, 0.15742E+01_JPRB ,&
&  0.15808E+01_JPRB, 0.15825E+01_JPRB, 0.15360E+01_JPRB, 0.15002E+01_JPRB, 0.15589E+01_JPRB, 0.15244E+01_JPRB, 0.14811E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.14012E+01_JPRB, 0.13669E+01_JPRB, 0.13298E+01_JPRB, 0.13791E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.12058E+01_JPRB, 0.11924E+01_JPRB, 0.11607E+01_JPRB, 0.11345E+01_JPRB, 0.11202E+01_JPRB, 0.11059E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.10304E+01_JPRB, 0.10129E+01_JPRB, 0.98765E+00_JPRB, 0.96092E+00_JPRB, 0.94186E+00_JPRB, 0.92426E+00_JPRB ,&
&  0.90329E+00_JPRB, 0.89204E+00_JPRB, 0.88635E+00_JPRB, 0.88002E+00_JPRB, 0.87136E+00_JPRB, 0.86424E+00_JPRB, 0.86652E+00_JPRB ,&
&  0.86567E+00_JPRB, 0.86583E+00_JPRB, 0.86721E+00_JPRB, 0.85961E+00_JPRB, 0.84878E+00_JPRB, 0.84305E+00_JPRB, 0.82954E+00_JPRB ,&
&  0.88142E+00_JPRB, 0.97614E+00_JPRB, 0.11943E+01_JPRB, 0.14709E+01_JPRB, 0.17141E+01_JPRB, 0.18906E+01_JPRB, 0.19735E+01_JPRB ,&
&  0.18990E+01_JPRB, 0.17447E+01_JPRB, 0.15941E+01_JPRB, 0.14972E+01_JPRB, 0.14086E+01_JPRB, 0.13344E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.11966E+01_JPRB, 0.11717E+01_JPRB, 0.11480E+01_JPRB, 0.11228E+01_JPRB, 0.11003E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10592E+01_JPRB, 0.10583E+01_JPRB, 0.10555E+01_JPRB, 0.10711E+01_JPRB, 0.11558E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.14775E+01_JPRB, 0.16271E+01_JPRB, 0.18225E+01_JPRB, 0.19335E+01_JPRB, 0.19129E+01_JPRB, 0.18474E+01_JPRB, 0.17578E+01_JPRB ,&
&  0.16811E+01_JPRB, 0.15999E+01_JPRB, 0.15465E+01_JPRB, 0.14940E+01_JPRB, 0.14380E+01_JPRB, 0.13917E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.13562E+01_JPRB, 0.13542E+01_JPRB, 0.13670E+01_JPRB, 0.13879E+01_JPRB, 0.14052E+01_JPRB, 0.14215E+01_JPRB, 0.14381E+01_JPRB ,&
&  0.14479E+01_JPRB, 0.14270E+01_JPRB, 0.13910E+01_JPRB, 0.13916E+01_JPRB, 0.14247E+01_JPRB, 0.14529E+01_JPRB, 0.14708E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.15050E+01_JPRB, 0.15108E+01_JPRB, 0.15202E+01_JPRB, 0.14997E+01_JPRB, 0.14498E+01_JPRB, 0.14010E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.13208E+01_JPRB, 0.12938E+01_JPRB, 0.12571E+01_JPRB, 0.11604E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.11275E+01_JPRB, 0.11078E+01_JPRB, 0.10805E+01_JPRB, 0.10565E+01_JPRB, 0.10447E+01_JPRB, 0.10405E+01_JPRB, 0.10385E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.10176E+01_JPRB, 0.10028E+01_JPRB, 0.98442E+00_JPRB, 0.96318E+00_JPRB, 0.94332E+00_JPRB, 0.92567E+00_JPRB ,&
&  0.90155E+00_JPRB, 0.88554E+00_JPRB, 0.88128E+00_JPRB, 0.87725E+00_JPRB, 0.87359E+00_JPRB, 0.86668E+00_JPRB, 0.86291E+00_JPRB ,&
&  0.86328E+00_JPRB, 0.86435E+00_JPRB, 0.86468E+00_JPRB, 0.85562E+00_JPRB, 0.84645E+00_JPRB, 0.84263E+00_JPRB, 0.83395E+00_JPRB ,&
&  0.84191E+00_JPRB, 0.78852E+00_JPRB, 0.95331E+00_JPRB, 0.11256E+01_JPRB, 0.12444E+01_JPRB, 0.13927E+01_JPRB, 0.15331E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.14957E+01_JPRB, 0.14025E+01_JPRB, 0.13256E+01_JPRB, 0.12586E+01_JPRB, 0.12104E+01_JPRB, 0.11732E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.11261E+01_JPRB, 0.11027E+01_JPRB, 0.10735E+01_JPRB, 0.10426E+01_JPRB, 0.10197E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.98896E+00_JPRB, 0.98226E+00_JPRB, 0.97655E+00_JPRB, 0.97883E+00_JPRB, 0.98150E+00_JPRB, 0.10087E+01_JPRB, 0.10968E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.12443E+01_JPRB, 0.13638E+01_JPRB, 0.14209E+01_JPRB, 0.14208E+01_JPRB, 0.14019E+01_JPRB, 0.13665E+01_JPRB ,&
&  0.13226E+01_JPRB, 0.12638E+01_JPRB, 0.12242E+01_JPRB, 0.11869E+01_JPRB, 0.11602E+01_JPRB, 0.11457E+01_JPRB, 0.11481E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.11514E+01_JPRB, 0.11542E+01_JPRB, 0.11617E+01_JPRB, 0.11676E+01_JPRB, 0.11737E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.12724E+01_JPRB, 0.12828E+01_JPRB, 0.12736E+01_JPRB, 0.12746E+01_JPRB, 0.12954E+01_JPRB, 0.13179E+01_JPRB ,&
&  0.13359E+01_JPRB, 0.13511E+01_JPRB, 0.13678E+01_JPRB, 0.13286E+01_JPRB, 0.13747E+01_JPRB, 0.13464E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.12379E+01_JPRB, 0.12114E+01_JPRB, 0.10606E+01_JPRB, 0.11103E+01_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.10273E+01_JPRB, 0.10115E+01_JPRB, 0.98931E+00_JPRB, 0.96927E+00_JPRB, 0.95564E+00_JPRB, 0.94972E+00_JPRB, 0.94720E+00_JPRB ,&
&  0.95040E+00_JPRB, 0.95326E+00_JPRB, 0.95270E+00_JPRB, 0.94837E+00_JPRB, 0.94089E+00_JPRB, 0.92842E+00_JPRB, 0.91663E+00_JPRB ,&
&  0.89995E+00_JPRB, 0.88284E+00_JPRB, 0.87526E+00_JPRB, 0.87460E+00_JPRB, 0.87237E+00_JPRB, 0.86355E+00_JPRB, 0.85584E+00_JPRB ,&
&  0.85074E+00_JPRB, 0.85035E+00_JPRB, 0.84166E+00_JPRB, 0.83450E+00_JPRB, 0.82622E+00_JPRB, 0.82572E+00_JPRB, 0.82351E+00_JPRB ,&
&  0.82143E+00_JPRB, 0.71375E+00_JPRB, 0.86412E+00_JPRB, 0.97501E+00_JPRB, 0.10227E+01_JPRB, 0.11154E+01_JPRB, 0.12002E+01_JPRB ,&
&  0.12536E+01_JPRB, 0.12711E+01_JPRB, 0.12526E+01_JPRB, 0.12162E+01_JPRB, 0.11662E+01_JPRB, 0.11266E+01_JPRB, 0.10968E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10388E+01_JPRB, 0.10063E+01_JPRB, 0.97300E+00_JPRB, 0.94643E+00_JPRB, 0.92007E+00_JPRB, 0.89755E+00_JPRB ,&
&  0.88511E+00_JPRB, 0.88190E+00_JPRB, 0.87585E+00_JPRB, 0.87520E+00_JPRB, 0.88001E+00_JPRB, 0.88619E+00_JPRB, 0.92108E+00_JPRB ,&
&  0.95808E+00_JPRB, 0.99375E+00_JPRB, 0.10337E+01_JPRB, 0.10627E+01_JPRB, 0.10720E+01_JPRB, 0.10812E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.10709E+01_JPRB, 0.10523E+01_JPRB, 0.10302E+01_JPRB, 0.10075E+01_JPRB, 0.98828E+00_JPRB, 0.97443E+00_JPRB, 0.97100E+00_JPRB ,&
&  0.97300E+00_JPRB, 0.96347E+00_JPRB, 0.95370E+00_JPRB, 0.95152E+00_JPRB, 0.95434E+00_JPRB, 0.96452E+00_JPRB, 0.98703E+00_JPRB ,&
&  0.10222E+01_JPRB, 0.10661E+01_JPRB, 0.11009E+01_JPRB, 0.11158E+01_JPRB, 0.11190E+01_JPRB, 0.11280E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.11623E+01_JPRB, 0.11749E+01_JPRB, 0.11872E+01_JPRB, 0.12178E+01_JPRB, 0.12179E+01_JPRB, 0.12012E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.11401E+01_JPRB, 0.11194E+01_JPRB, 0.93648E+00_JPRB, 0.10060E+01_JPRB, 0.10186E+01_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.94003E+00_JPRB, 0.92556E+00_JPRB, 0.90517E+00_JPRB, 0.88831E+00_JPRB, 0.87456E+00_JPRB, 0.86823E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.86794E+00_JPRB, 0.87617E+00_JPRB, 0.88499E+00_JPRB, 0.89087E+00_JPRB, 0.89555E+00_JPRB, 0.89565E+00_JPRB, 0.89062E+00_JPRB ,&
&  0.88600E+00_JPRB, 0.87367E+00_JPRB, 0.86305E+00_JPRB, 0.86145E+00_JPRB, 0.85765E+00_JPRB, 0.85258E+00_JPRB, 0.84475E+00_JPRB ,&
&  0.83638E+00_JPRB, 0.82802E+00_JPRB, 0.81390E+00_JPRB, 0.80577E+00_JPRB, 0.80267E+00_JPRB, 0.80447E+00_JPRB, 0.81206E+00_JPRB ,&
&  0.73354E+00_JPRB, 0.66057E+00_JPRB, 0.76305E+00_JPRB, 0.88093E+00_JPRB, 0.93238E+00_JPRB, 0.98023E+00_JPRB, 0.10220E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.11187E+01_JPRB, 0.11451E+01_JPRB, 0.11468E+01_JPRB, 0.11266E+01_JPRB, 0.10941E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.95365E+00_JPRB, 0.91397E+00_JPRB, 0.88982E+00_JPRB, 0.87053E+00_JPRB, 0.85145E+00_JPRB, 0.83568E+00_JPRB ,&
&  0.82473E+00_JPRB, 0.81682E+00_JPRB, 0.81064E+00_JPRB, 0.80652E+00_JPRB, 0.80743E+00_JPRB, 0.81081E+00_JPRB, 0.82001E+00_JPRB ,&
&  0.83686E+00_JPRB, 0.85409E+00_JPRB, 0.86937E+00_JPRB, 0.87862E+00_JPRB, 0.88692E+00_JPRB, 0.90160E+00_JPRB, 0.91248E+00_JPRB ,&
&  0.92164E+00_JPRB, 0.92113E+00_JPRB, 0.91472E+00_JPRB, 0.90398E+00_JPRB, 0.88911E+00_JPRB, 0.87409E+00_JPRB, 0.86228E+00_JPRB ,&
&  0.85453E+00_JPRB, 0.84452E+00_JPRB, 0.84025E+00_JPRB, 0.84225E+00_JPRB, 0.84793E+00_JPRB, 0.85567E+00_JPRB, 0.86554E+00_JPRB ,&
&  0.88329E+00_JPRB, 0.91201E+00_JPRB, 0.93737E+00_JPRB, 0.95427E+00_JPRB, 0.96511E+00_JPRB, 0.97305E+00_JPRB, 0.98460E+00_JPRB ,&
&  0.99719E+00_JPRB, 0.10108E+01_JPRB, 0.10280E+01_JPRB, 0.10477E+01_JPRB, 0.10537E+01_JPRB, 0.10489E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.10151E+01_JPRB, 0.98857E+00_JPRB, 0.97515E+00_JPRB, 0.95948E+00_JPRB, 0.94845E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.82504E+00_JPRB, 0.81647E+00_JPRB, 0.80326E+00_JPRB, 0.79463E+00_JPRB, 0.79012E+00_JPRB, 0.79158E+00_JPRB, 0.79466E+00_JPRB ,&
&  0.79833E+00_JPRB, 0.79943E+00_JPRB, 0.80736E+00_JPRB, 0.81652E+00_JPRB, 0.82315E+00_JPRB, 0.82813E+00_JPRB, 0.82791E+00_JPRB ,&
&  0.82764E+00_JPRB, 0.82733E+00_JPRB, 0.82251E+00_JPRB, 0.81748E+00_JPRB, 0.81088E+00_JPRB, 0.81072E+00_JPRB, 0.80916E+00_JPRB ,&
&  0.80500E+00_JPRB, 0.79791E+00_JPRB, 0.78729E+00_JPRB, 0.77499E+00_JPRB, 0.76846E+00_JPRB, 0.76574E+00_JPRB, 0.76194E+00_JPRB ,&
&  0.76815E+00_JPRB, 0.68901E+00_JPRB, 0.73787E+00_JPRB, 0.78360E+00_JPRB, 0.84172E+00_JPRB, 0.88603E+00_JPRB, 0.92424E+00_JPRB ,&
&  0.96236E+00_JPRB, 0.99549E+00_JPRB, 0.10130E+01_JPRB, 0.10184E+01_JPRB, 0.99886E+00_JPRB, 0.95869E+00_JPRB, 0.91324E+00_JPRB ,&
&  0.87551E+00_JPRB, 0.84762E+00_JPRB, 0.82075E+00_JPRB, 0.80380E+00_JPRB, 0.78987E+00_JPRB, 0.77240E+00_JPRB, 0.75571E+00_JPRB ,&
&  0.74424E+00_JPRB, 0.73594E+00_JPRB, 0.73208E+00_JPRB, 0.73173E+00_JPRB, 0.73350E+00_JPRB, 0.73530E+00_JPRB, 0.73560E+00_JPRB ,&
&  0.73864E+00_JPRB, 0.74523E+00_JPRB, 0.75172E+00_JPRB, 0.75905E+00_JPRB, 0.76811E+00_JPRB, 0.77919E+00_JPRB, 0.78363E+00_JPRB ,&
&  0.78029E+00_JPRB, 0.78191E+00_JPRB, 0.78306E+00_JPRB, 0.78044E+00_JPRB, 0.77667E+00_JPRB, 0.77207E+00_JPRB, 0.76546E+00_JPRB ,&
&  0.76002E+00_JPRB, 0.75950E+00_JPRB, 0.76033E+00_JPRB, 0.76550E+00_JPRB, 0.76986E+00_JPRB, 0.78011E+00_JPRB, 0.79037E+00_JPRB ,&
&  0.80247E+00_JPRB, 0.82145E+00_JPRB, 0.84300E+00_JPRB, 0.86084E+00_JPRB, 0.87247E+00_JPRB, 0.88012E+00_JPRB, 0.88695E+00_JPRB ,&
&  0.89290E+00_JPRB, 0.89906E+00_JPRB, 0.90646E+00_JPRB, 0.91141E+00_JPRB, 0.91002E+00_JPRB, 0.90495E+00_JPRB, 0.89832E+00_JPRB ,&
&  0.88328E+00_JPRB, 0.86280E+00_JPRB, 0.85027E+00_JPRB, 0.83904E+00_JPRB, 0.82999E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.74686E+00_JPRB, 0.74089E+00_JPRB, 0.73488E+00_JPRB, 0.73349E+00_JPRB, 0.73196E+00_JPRB, 0.73308E+00_JPRB, 0.73488E+00_JPRB ,&
&  0.74101E+00_JPRB, 0.74511E+00_JPRB, 0.75477E+00_JPRB, 0.76319E+00_JPRB, 0.76790E+00_JPRB, 0.77330E+00_JPRB, 0.77672E+00_JPRB ,&
&  0.77833E+00_JPRB, 0.78026E+00_JPRB, 0.78137E+00_JPRB, 0.78588E+00_JPRB, 0.78651E+00_JPRB, 0.78581E+00_JPRB, 0.78434E+00_JPRB ,&
&  0.78096E+00_JPRB, 0.77627E+00_JPRB, 0.76786E+00_JPRB, 0.75733E+00_JPRB, 0.74486E+00_JPRB, 0.73793E+00_JPRB, 0.72965E+00_JPRB ,&
&  0.72896E+00_JPRB, 0.71064E+00_JPRB, 0.72505E+00_JPRB, 0.72381E+00_JPRB, 0.75241E+00_JPRB, 0.79661E+00_JPRB, 0.84248E+00_JPRB ,&
&  0.86907E+00_JPRB, 0.87879E+00_JPRB, 0.87365E+00_JPRB, 0.86274E+00_JPRB, 0.84148E+00_JPRB, 0.82104E+00_JPRB, 0.79824E+00_JPRB ,&
&  0.77283E+00_JPRB, 0.75766E+00_JPRB, 0.74080E+00_JPRB, 0.72831E+00_JPRB, 0.72245E+00_JPRB, 0.71776E+00_JPRB, 0.70963E+00_JPRB ,&
&  0.69735E+00_JPRB, 0.68640E+00_JPRB, 0.68427E+00_JPRB, 0.68650E+00_JPRB, 0.68961E+00_JPRB, 0.69156E+00_JPRB, 0.69002E+00_JPRB ,&
&  0.68395E+00_JPRB, 0.68056E+00_JPRB, 0.68266E+00_JPRB, 0.68645E+00_JPRB, 0.68690E+00_JPRB, 0.68344E+00_JPRB, 0.67919E+00_JPRB ,&
&  0.67925E+00_JPRB, 0.68226E+00_JPRB, 0.68543E+00_JPRB, 0.68968E+00_JPRB, 0.69218E+00_JPRB, 0.69508E+00_JPRB, 0.69594E+00_JPRB ,&
&  0.69986E+00_JPRB, 0.70385E+00_JPRB, 0.71247E+00_JPRB, 0.72097E+00_JPRB, 0.72284E+00_JPRB, 0.73193E+00_JPRB, 0.74211E+00_JPRB ,&
&  0.75383E+00_JPRB, 0.76829E+00_JPRB, 0.78187E+00_JPRB, 0.79385E+00_JPRB, 0.80549E+00_JPRB, 0.81262E+00_JPRB, 0.82028E+00_JPRB ,&
&  0.82921E+00_JPRB, 0.83174E+00_JPRB, 0.83803E+00_JPRB, 0.83867E+00_JPRB, 0.83408E+00_JPRB, 0.82596E+00_JPRB, 0.81342E+00_JPRB ,&
&  0.79773E+00_JPRB, 0.77727E+00_JPRB, 0.76418E+00_JPRB, 0.75545E+00_JPRB, 0.75004E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.66287E+00_JPRB, 0.65998E+00_JPRB, 0.65963E+00_JPRB, 0.66237E+00_JPRB, 0.66574E+00_JPRB, 0.66818E+00_JPRB, 0.67353E+00_JPRB ,&
&  0.68321E+00_JPRB, 0.69105E+00_JPRB, 0.70099E+00_JPRB, 0.71151E+00_JPRB, 0.71979E+00_JPRB, 0.72542E+00_JPRB, 0.72850E+00_JPRB ,&
&  0.72720E+00_JPRB, 0.72514E+00_JPRB, 0.72898E+00_JPRB, 0.73428E+00_JPRB, 0.73591E+00_JPRB, 0.73551E+00_JPRB, 0.73540E+00_JPRB ,&
&  0.73428E+00_JPRB, 0.73075E+00_JPRB, 0.72572E+00_JPRB, 0.71859E+00_JPRB, 0.71049E+00_JPRB, 0.70515E+00_JPRB, 0.70045E+00_JPRB ,&
&  0.69121E+00_JPRB, 0.69530E+00_JPRB, 0.64433E+00_JPRB, 0.65783E+00_JPRB, 0.66920E+00_JPRB, 0.69365E+00_JPRB, 0.71961E+00_JPRB ,&
&  0.73639E+00_JPRB, 0.74391E+00_JPRB, 0.74596E+00_JPRB, 0.74123E+00_JPRB, 0.72613E+00_JPRB, 0.71850E+00_JPRB, 0.71106E+00_JPRB ,&
&  0.70046E+00_JPRB, 0.69280E+00_JPRB, 0.67820E+00_JPRB, 0.66211E+00_JPRB, 0.65659E+00_JPRB, 0.65703E+00_JPRB, 0.65635E+00_JPRB ,&
&  0.65007E+00_JPRB, 0.64357E+00_JPRB, 0.64289E+00_JPRB, 0.64282E+00_JPRB, 0.64218E+00_JPRB, 0.63674E+00_JPRB, 0.63008E+00_JPRB ,&
&  0.61979E+00_JPRB, 0.60996E+00_JPRB, 0.60341E+00_JPRB, 0.59929E+00_JPRB, 0.59631E+00_JPRB, 0.59289E+00_JPRB, 0.58928E+00_JPRB ,&
&  0.59504E+00_JPRB, 0.60888E+00_JPRB, 0.62164E+00_JPRB, 0.63160E+00_JPRB, 0.63786E+00_JPRB, 0.64163E+00_JPRB, 0.64669E+00_JPRB ,&
&  0.65274E+00_JPRB, 0.66342E+00_JPRB, 0.67158E+00_JPRB, 0.67902E+00_JPRB, 0.68439E+00_JPRB, 0.68947E+00_JPRB, 0.69545E+00_JPRB ,&
&  0.70133E+00_JPRB, 0.71110E+00_JPRB, 0.71990E+00_JPRB, 0.73002E+00_JPRB, 0.73716E+00_JPRB, 0.74488E+00_JPRB, 0.75210E+00_JPRB ,&
&  0.75932E+00_JPRB, 0.76522E+00_JPRB, 0.76745E+00_JPRB, 0.76521E+00_JPRB, 0.75737E+00_JPRB, 0.74740E+00_JPRB, 0.73648E+00_JPRB ,&
&  0.72229E+00_JPRB, 0.70520E+00_JPRB, 0.68789E+00_JPRB, 0.67636E+00_JPRB, 0.66666E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.59882E+00_JPRB, 0.59875E+00_JPRB, 0.60283E+00_JPRB, 0.60679E+00_JPRB, 0.60986E+00_JPRB, 0.61456E+00_JPRB, 0.62428E+00_JPRB ,&
&  0.63419E+00_JPRB, 0.64380E+00_JPRB, 0.65035E+00_JPRB, 0.65831E+00_JPRB, 0.66978E+00_JPRB, 0.67778E+00_JPRB, 0.68414E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.68809E+00_JPRB, 0.68997E+00_JPRB, 0.69375E+00_JPRB, 0.69368E+00_JPRB, 0.69019E+00_JPRB, 0.68892E+00_JPRB ,&
&  0.68858E+00_JPRB, 0.68632E+00_JPRB, 0.68515E+00_JPRB, 0.68473E+00_JPRB, 0.68038E+00_JPRB, 0.67431E+00_JPRB, 0.67154E+00_JPRB ,&
&  0.66898E+00_JPRB, 0.65435E+00_JPRB, 0.64554E+00_JPRB, 0.63729E+00_JPRB, 0.63955E+00_JPRB, 0.64960E+00_JPRB, 0.66124E+00_JPRB ,&
&  0.66806E+00_JPRB, 0.67267E+00_JPRB, 0.67296E+00_JPRB, 0.67226E+00_JPRB, 0.66722E+00_JPRB, 0.65622E+00_JPRB, 0.64796E+00_JPRB ,&
&  0.64297E+00_JPRB, 0.63446E+00_JPRB, 0.61099E+00_JPRB, 0.59624E+00_JPRB, 0.59183E+00_JPRB, 0.59490E+00_JPRB, 0.60019E+00_JPRB ,&
&  0.60190E+00_JPRB, 0.59815E+00_JPRB, 0.59061E+00_JPRB, 0.58264E+00_JPRB, 0.57274E+00_JPRB, 0.56169E+00_JPRB, 0.55206E+00_JPRB ,&
&  0.54693E+00_JPRB, 0.54325E+00_JPRB, 0.54037E+00_JPRB, 0.54353E+00_JPRB, 0.54302E+00_JPRB, 0.53928E+00_JPRB, 0.54222E+00_JPRB ,&
&  0.55579E+00_JPRB, 0.57327E+00_JPRB, 0.58951E+00_JPRB, 0.59828E+00_JPRB, 0.60185E+00_JPRB, 0.60594E+00_JPRB, 0.61030E+00_JPRB ,&
&  0.61599E+00_JPRB, 0.62117E+00_JPRB, 0.62676E+00_JPRB, 0.63493E+00_JPRB, 0.64489E+00_JPRB, 0.65381E+00_JPRB, 0.65791E+00_JPRB ,&
&  0.66146E+00_JPRB, 0.66450E+00_JPRB, 0.66797E+00_JPRB, 0.67583E+00_JPRB, 0.68575E+00_JPRB, 0.69291E+00_JPRB, 0.69810E+00_JPRB ,&
&  0.70289E+00_JPRB, 0.70484E+00_JPRB, 0.70292E+00_JPRB, 0.69795E+00_JPRB, 0.69162E+00_JPRB, 0.68001E+00_JPRB, 0.66539E+00_JPRB ,&
&  0.64856E+00_JPRB, 0.63263E+00_JPRB, 0.62028E+00_JPRB, 0.60956E+00_JPRB, 0.60181E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.54961E+00_JPRB, 0.54824E+00_JPRB, 0.55036E+00_JPRB, 0.55617E+00_JPRB, 0.56016E+00_JPRB, 0.56382E+00_JPRB, 0.57174E+00_JPRB ,&
&  0.57929E+00_JPRB, 0.58627E+00_JPRB, 0.59414E+00_JPRB, 0.60226E+00_JPRB, 0.60840E+00_JPRB, 0.61758E+00_JPRB, 0.62662E+00_JPRB ,&
&  0.63269E+00_JPRB, 0.63548E+00_JPRB, 0.63752E+00_JPRB, 0.63929E+00_JPRB, 0.63971E+00_JPRB, 0.63762E+00_JPRB, 0.63733E+00_JPRB ,&
&  0.63541E+00_JPRB, 0.63307E+00_JPRB, 0.63477E+00_JPRB, 0.63449E+00_JPRB, 0.63503E+00_JPRB, 0.63625E+00_JPRB, 0.63440E+00_JPRB ,&
&  0.62993E+00_JPRB, 0.61485E+00_JPRB, 0.60858E+00_JPRB, 0.60442E+00_JPRB, 0.60769E+00_JPRB, 0.60776E+00_JPRB, 0.60540E+00_JPRB ,&
&  0.59785E+00_JPRB, 0.59288E+00_JPRB, 0.59440E+00_JPRB, 0.59657E+00_JPRB, 0.59043E+00_JPRB, 0.57728E+00_JPRB, 0.56806E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.55042E+00_JPRB, 0.53213E+00_JPRB, 0.52025E+00_JPRB, 0.51494E+00_JPRB, 0.51024E+00_JPRB, 0.50667E+00_JPRB ,&
&  0.50508E+00_JPRB, 0.50431E+00_JPRB, 0.50527E+00_JPRB, 0.50435E+00_JPRB, 0.50082E+00_JPRB, 0.49691E+00_JPRB, 0.49581E+00_JPRB ,&
&  0.49916E+00_JPRB, 0.50221E+00_JPRB, 0.50612E+00_JPRB, 0.50910E+00_JPRB, 0.50893E+00_JPRB, 0.50929E+00_JPRB, 0.51341E+00_JPRB ,&
&  0.52100E+00_JPRB, 0.52880E+00_JPRB, 0.53549E+00_JPRB, 0.53794E+00_JPRB, 0.54198E+00_JPRB, 0.54999E+00_JPRB, 0.55124E+00_JPRB ,&
&  0.54949E+00_JPRB, 0.55070E+00_JPRB, 0.55478E+00_JPRB, 0.56369E+00_JPRB, 0.57741E+00_JPRB, 0.58691E+00_JPRB, 0.58913E+00_JPRB ,&
&  0.59002E+00_JPRB, 0.59302E+00_JPRB, 0.59814E+00_JPRB, 0.60967E+00_JPRB, 0.62179E+00_JPRB, 0.63035E+00_JPRB, 0.63367E+00_JPRB ,&
&  0.63019E+00_JPRB, 0.62267E+00_JPRB, 0.60973E+00_JPRB, 0.59491E+00_JPRB, 0.58193E+00_JPRB, 0.57184E+00_JPRB, 0.56339E+00_JPRB ,&
&  0.55500E+00_JPRB, 0.55079E+00_JPRB, 0.54971E+00_JPRB, 0.55082E+00_JPRB, 0.55081E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.51019E+00_JPRB, 0.51139E+00_JPRB, 0.51335E+00_JPRB, 0.51614E+00_JPRB, 0.51900E+00_JPRB, 0.52054E+00_JPRB, 0.52397E+00_JPRB ,&
&  0.52633E+00_JPRB, 0.53351E+00_JPRB, 0.54498E+00_JPRB, 0.55611E+00_JPRB, 0.56396E+00_JPRB, 0.57045E+00_JPRB, 0.57714E+00_JPRB ,&
&  0.58286E+00_JPRB, 0.59030E+00_JPRB, 0.59168E+00_JPRB, 0.58978E+00_JPRB, 0.58758E+00_JPRB, 0.58597E+00_JPRB, 0.58442E+00_JPRB ,&
&  0.58188E+00_JPRB, 0.58156E+00_JPRB, 0.58499E+00_JPRB, 0.58872E+00_JPRB, 0.59233E+00_JPRB, 0.59249E+00_JPRB, 0.58982E+00_JPRB ,&
&  0.59169E+00_JPRB, 0.58027E+00_JPRB, 0.57236E+00_JPRB, 0.56370E+00_JPRB, 0.55615E+00_JPRB, 0.54394E+00_JPRB, 0.53471E+00_JPRB ,&
&  0.52363E+00_JPRB, 0.51364E+00_JPRB, 0.50921E+00_JPRB, 0.50469E+00_JPRB, 0.49469E+00_JPRB, 0.48371E+00_JPRB, 0.47629E+00_JPRB ,&
&  0.47168E+00_JPRB, 0.46692E+00_JPRB, 0.46060E+00_JPRB, 0.45492E+00_JPRB, 0.45013E+00_JPRB, 0.44836E+00_JPRB, 0.44989E+00_JPRB ,&
&  0.45466E+00_JPRB, 0.45960E+00_JPRB, 0.46301E+00_JPRB, 0.46422E+00_JPRB, 0.46279E+00_JPRB, 0.46024E+00_JPRB, 0.45907E+00_JPRB ,&
&  0.46033E+00_JPRB, 0.46246E+00_JPRB, 0.46432E+00_JPRB, 0.46941E+00_JPRB, 0.47051E+00_JPRB, 0.46866E+00_JPRB, 0.46302E+00_JPRB ,&
&  0.46133E+00_JPRB, 0.46097E+00_JPRB, 0.46335E+00_JPRB, 0.46893E+00_JPRB, 0.47805E+00_JPRB, 0.48706E+00_JPRB, 0.49338E+00_JPRB ,&
&  0.49608E+00_JPRB, 0.49727E+00_JPRB, 0.50012E+00_JPRB, 0.50417E+00_JPRB, 0.50652E+00_JPRB, 0.50685E+00_JPRB, 0.50619E+00_JPRB ,&
&  0.50976E+00_JPRB, 0.51821E+00_JPRB, 0.52721E+00_JPRB, 0.53378E+00_JPRB, 0.53768E+00_JPRB, 0.53764E+00_JPRB, 0.53119E+00_JPRB ,&
&  0.51852E+00_JPRB, 0.50359E+00_JPRB, 0.48921E+00_JPRB, 0.47734E+00_JPRB, 0.46915E+00_JPRB, 0.46516E+00_JPRB, 0.46783E+00_JPRB ,&
&  0.47529E+00_JPRB, 0.48496E+00_JPRB, 0.49344E+00_JPRB, 0.49941E+00_JPRB, 0.50597E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.47375E+00_JPRB, 0.47365E+00_JPRB, 0.47535E+00_JPRB, 0.47453E+00_JPRB, 0.47176E+00_JPRB, 0.46905E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.47814E+00_JPRB, 0.49023E+00_JPRB, 0.50659E+00_JPRB, 0.52102E+00_JPRB, 0.52960E+00_JPRB, 0.53455E+00_JPRB, 0.54214E+00_JPRB ,&
&  0.54944E+00_JPRB, 0.55761E+00_JPRB, 0.55810E+00_JPRB, 0.54961E+00_JPRB, 0.54250E+00_JPRB, 0.53884E+00_JPRB, 0.53855E+00_JPRB ,&
&  0.53782E+00_JPRB, 0.53930E+00_JPRB, 0.54203E+00_JPRB, 0.54475E+00_JPRB, 0.54333E+00_JPRB, 0.54059E+00_JPRB, 0.54305E+00_JPRB ,&
&  0.53965E+00_JPRB, 0.52819E+00_JPRB, 0.51612E+00_JPRB, 0.49034E+00_JPRB, 0.48834E+00_JPRB, 0.47687E+00_JPRB, 0.46845E+00_JPRB ,&
&  0.45714E+00_JPRB, 0.44295E+00_JPRB, 0.43252E+00_JPRB, 0.42593E+00_JPRB, 0.42278E+00_JPRB, 0.42037E+00_JPRB, 0.41864E+00_JPRB ,&
&  0.41778E+00_JPRB, 0.41838E+00_JPRB, 0.42078E+00_JPRB, 0.41863E+00_JPRB, 0.41743E+00_JPRB, 0.41835E+00_JPRB, 0.42051E+00_JPRB ,&
&  0.42156E+00_JPRB, 0.42279E+00_JPRB, 0.42397E+00_JPRB, 0.42406E+00_JPRB, 0.42274E+00_JPRB, 0.42028E+00_JPRB, 0.41807E+00_JPRB ,&
&  0.41747E+00_JPRB, 0.41921E+00_JPRB, 0.42020E+00_JPRB, 0.41648E+00_JPRB, 0.40878E+00_JPRB, 0.39822E+00_JPRB, 0.39134E+00_JPRB ,&
&  0.38892E+00_JPRB, 0.39193E+00_JPRB, 0.40087E+00_JPRB, 0.41280E+00_JPRB, 0.42347E+00_JPRB, 0.43009E+00_JPRB, 0.43436E+00_JPRB ,&
&  0.43878E+00_JPRB, 0.43573E+00_JPRB, 0.43129E+00_JPRB, 0.42784E+00_JPRB, 0.42490E+00_JPRB, 0.42067E+00_JPRB, 0.42067E+00_JPRB ,&
&  0.42418E+00_JPRB, 0.42800E+00_JPRB, 0.42967E+00_JPRB, 0.42909E+00_JPRB, 0.42721E+00_JPRB, 0.42304E+00_JPRB, 0.41723E+00_JPRB ,&
&  0.40975E+00_JPRB, 0.40097E+00_JPRB, 0.39971E+00_JPRB, 0.40122E+00_JPRB, 0.40530E+00_JPRB, 0.41312E+00_JPRB, 0.42383E+00_JPRB ,&
&  0.43499E+00_JPRB, 0.44376E+00_JPRB, 0.45335E+00_JPRB, 0.46221E+00_JPRB, 0.47027E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.45015E+00_JPRB, 0.45145E+00_JPRB, 0.44874E+00_JPRB, 0.44612E+00_JPRB, 0.44603E+00_JPRB, 0.44737E+00_JPRB, 0.45248E+00_JPRB ,&
&  0.46216E+00_JPRB, 0.47555E+00_JPRB, 0.48854E+00_JPRB, 0.49919E+00_JPRB, 0.50845E+00_JPRB, 0.51243E+00_JPRB, 0.51818E+00_JPRB ,&
&  0.52454E+00_JPRB, 0.53037E+00_JPRB, 0.53110E+00_JPRB, 0.52078E+00_JPRB, 0.51086E+00_JPRB, 0.50558E+00_JPRB, 0.50557E+00_JPRB ,&
&  0.50713E+00_JPRB, 0.50931E+00_JPRB, 0.51490E+00_JPRB, 0.51647E+00_JPRB, 0.50978E+00_JPRB, 0.50745E+00_JPRB, 0.50450E+00_JPRB ,&
&  0.49784E+00_JPRB, 0.48403E+00_JPRB, 0.45047E+00_JPRB, 0.44309E+00_JPRB, 0.43391E+00_JPRB, 0.42441E+00_JPRB, 0.41551E+00_JPRB ,&
&  0.40510E+00_JPRB, 0.39431E+00_JPRB, 0.38695E+00_JPRB, 0.38244E+00_JPRB, 0.38273E+00_JPRB, 0.38403E+00_JPRB, 0.38388E+00_JPRB ,&
&  0.38635E+00_JPRB, 0.38875E+00_JPRB, 0.39122E+00_JPRB, 0.39204E+00_JPRB, 0.39226E+00_JPRB, 0.39226E+00_JPRB, 0.38898E+00_JPRB ,&
&  0.38611E+00_JPRB, 0.38504E+00_JPRB, 0.38431E+00_JPRB, 0.38370E+00_JPRB, 0.38409E+00_JPRB, 0.38499E+00_JPRB, 0.38534E+00_JPRB ,&
&  0.38275E+00_JPRB, 0.37921E+00_JPRB, 0.37517E+00_JPRB, 0.36754E+00_JPRB, 0.35275E+00_JPRB, 0.33915E+00_JPRB, 0.32969E+00_JPRB ,&
&  0.32889E+00_JPRB, 0.33932E+00_JPRB, 0.35920E+00_JPRB, 0.37182E+00_JPRB, 0.37968E+00_JPRB, 0.38316E+00_JPRB, 0.38660E+00_JPRB ,&
&  0.37827E+00_JPRB, 0.35700E+00_JPRB, 0.34459E+00_JPRB, 0.33672E+00_JPRB, 0.33181E+00_JPRB, 0.32884E+00_JPRB, 0.32840E+00_JPRB ,&
&  0.33213E+00_JPRB, 0.33769E+00_JPRB, 0.34432E+00_JPRB, 0.34747E+00_JPRB, 0.34643E+00_JPRB, 0.34361E+00_JPRB, 0.34010E+00_JPRB ,&
&  0.33840E+00_JPRB, 0.34414E+00_JPRB, 0.35919E+00_JPRB, 0.37735E+00_JPRB, 0.38609E+00_JPRB, 0.39314E+00_JPRB, 0.39955E+00_JPRB ,&
&  0.40659E+00_JPRB, 0.41715E+00_JPRB, 0.42932E+00_JPRB, 0.43854E+00_JPRB, 0.44652E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.41923E+00_JPRB, 0.42273E+00_JPRB, 0.42473E+00_JPRB, 0.42791E+00_JPRB, 0.43563E+00_JPRB, 0.44611E+00_JPRB, 0.45807E+00_JPRB ,&
&  0.46992E+00_JPRB, 0.48022E+00_JPRB, 0.48565E+00_JPRB, 0.49112E+00_JPRB, 0.49611E+00_JPRB, 0.49834E+00_JPRB, 0.50310E+00_JPRB ,&
&  0.50358E+00_JPRB, 0.49977E+00_JPRB, 0.49335E+00_JPRB, 0.48519E+00_JPRB, 0.48083E+00_JPRB, 0.48184E+00_JPRB, 0.48431E+00_JPRB ,&
&  0.48561E+00_JPRB, 0.48653E+00_JPRB, 0.48611E+00_JPRB, 0.48202E+00_JPRB, 0.47829E+00_JPRB, 0.47599E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.45867E+00_JPRB, 0.40775E+00_JPRB, 0.37072E+00_JPRB, 0.36809E+00_JPRB, 0.38190E+00_JPRB, 0.38356E+00_JPRB, 0.37942E+00_JPRB ,&
&  0.37425E+00_JPRB, 0.36907E+00_JPRB, 0.36596E+00_JPRB, 0.36360E+00_JPRB, 0.36077E+00_JPRB, 0.35806E+00_JPRB, 0.35827E+00_JPRB ,&
&  0.35944E+00_JPRB, 0.35732E+00_JPRB, 0.35447E+00_JPRB, 0.34799E+00_JPRB, 0.34355E+00_JPRB, 0.34281E+00_JPRB, 0.34232E+00_JPRB ,&
&  0.33836E+00_JPRB, 0.33410E+00_JPRB, 0.33462E+00_JPRB, 0.33646E+00_JPRB, 0.33913E+00_JPRB, 0.34086E+00_JPRB, 0.33767E+00_JPRB ,&
&  0.33394E+00_JPRB, 0.32661E+00_JPRB, 0.31867E+00_JPRB, 0.30881E+00_JPRB, 0.29971E+00_JPRB, 0.29265E+00_JPRB, 0.28662E+00_JPRB ,&
&  0.28666E+00_JPRB, 0.29479E+00_JPRB, 0.31045E+00_JPRB, 0.32571E+00_JPRB, 0.33625E+00_JPRB, 0.33066E+00_JPRB, 0.32193E+00_JPRB ,&
&  0.31041E+00_JPRB, 0.29999E+00_JPRB, 0.29264E+00_JPRB, 0.28949E+00_JPRB, 0.28825E+00_JPRB, 0.28813E+00_JPRB, 0.28952E+00_JPRB ,&
&  0.29217E+00_JPRB, 0.29519E+00_JPRB, 0.29747E+00_JPRB, 0.29849E+00_JPRB, 0.29873E+00_JPRB, 0.29866E+00_JPRB, 0.29782E+00_JPRB ,&
&  0.29707E+00_JPRB, 0.29999E+00_JPRB, 0.30738E+00_JPRB, 0.31645E+00_JPRB, 0.32471E+00_JPRB, 0.33300E+00_JPRB, 0.34622E+00_JPRB ,&
&  0.36237E+00_JPRB, 0.38103E+00_JPRB, 0.39538E+00_JPRB, 0.40670E+00_JPRB, 0.41564E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.40620E+00_JPRB, 0.41537E+00_JPRB, 0.42669E+00_JPRB, 0.43690E+00_JPRB, 0.44652E+00_JPRB, 0.45537E+00_JPRB, 0.46365E+00_JPRB ,&
&  0.46881E+00_JPRB, 0.47182E+00_JPRB, 0.47313E+00_JPRB, 0.47700E+00_JPRB, 0.47367E+00_JPRB, 0.46571E+00_JPRB, 0.45977E+00_JPRB ,&
&  0.45465E+00_JPRB, 0.45075E+00_JPRB, 0.44525E+00_JPRB, 0.44234E+00_JPRB, 0.44190E+00_JPRB, 0.44170E+00_JPRB, 0.43577E+00_JPRB ,&
&  0.42444E+00_JPRB, 0.42213E+00_JPRB, 0.42611E+00_JPRB, 0.41972E+00_JPRB, 0.41341E+00_JPRB, 0.40710E+00_JPRB, 0.40072E+00_JPRB ,&
&  0.38313E+00_JPRB, 0.36193E+00_JPRB, 0.33604E+00_JPRB, 0.33482E+00_JPRB, 0.35045E+00_JPRB, 0.34235E+00_JPRB, 0.34688E+00_JPRB ,&
&  0.34838E+00_JPRB, 0.34820E+00_JPRB, 0.34675E+00_JPRB, 0.34480E+00_JPRB, 0.34139E+00_JPRB, 0.34071E+00_JPRB, 0.33867E+00_JPRB ,&
&  0.33139E+00_JPRB, 0.32599E+00_JPRB, 0.31700E+00_JPRB, 0.29789E+00_JPRB, 0.29451E+00_JPRB, 0.29488E+00_JPRB, 0.29113E+00_JPRB ,&
&  0.28376E+00_JPRB, 0.27996E+00_JPRB, 0.27946E+00_JPRB, 0.27394E+00_JPRB, 0.27629E+00_JPRB, 0.27900E+00_JPRB, 0.27875E+00_JPRB ,&
&  0.27850E+00_JPRB, 0.27788E+00_JPRB, 0.27554E+00_JPRB, 0.27146E+00_JPRB, 0.26801E+00_JPRB, 0.26721E+00_JPRB, 0.26878E+00_JPRB ,&
&  0.27165E+00_JPRB, 0.27600E+00_JPRB, 0.28806E+00_JPRB, 0.29705E+00_JPRB, 0.29524E+00_JPRB, 0.29180E+00_JPRB, 0.28220E+00_JPRB ,&
&  0.27210E+00_JPRB, 0.26856E+00_JPRB, 0.26538E+00_JPRB, 0.26352E+00_JPRB, 0.26368E+00_JPRB, 0.26521E+00_JPRB, 0.26574E+00_JPRB ,&
&  0.26702E+00_JPRB, 0.26854E+00_JPRB, 0.26954E+00_JPRB, 0.26994E+00_JPRB, 0.27061E+00_JPRB, 0.27142E+00_JPRB, 0.27312E+00_JPRB ,&
&  0.27563E+00_JPRB, 0.28024E+00_JPRB, 0.28622E+00_JPRB, 0.29271E+00_JPRB, 0.29756E+00_JPRB, 0.30013E+00_JPRB, 0.30582E+00_JPRB ,&
&  0.31234E+00_JPRB, 0.32956E+00_JPRB, 0.35918E+00_JPRB, 0.40180E+00_JPRB, 0.40394E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.40186E+00_JPRB, 0.41429E+00_JPRB, 0.42938E+00_JPRB, 0.43916E+00_JPRB, 0.44536E+00_JPRB, 0.44870E+00_JPRB, 0.44827E+00_JPRB ,&
&  0.44499E+00_JPRB, 0.44409E+00_JPRB, 0.45482E+00_JPRB, 0.43548E+00_JPRB, 0.41838E+00_JPRB, 0.38631E+00_JPRB, 0.36963E+00_JPRB ,&
&  0.35635E+00_JPRB, 0.35381E+00_JPRB, 0.35297E+00_JPRB, 0.35732E+00_JPRB, 0.36048E+00_JPRB, 0.36291E+00_JPRB, 0.35824E+00_JPRB ,&
&  0.35211E+00_JPRB, 0.34255E+00_JPRB, 0.33454E+00_JPRB, 0.32729E+00_JPRB, 0.32783E+00_JPRB, 0.32660E+00_JPRB, 0.32392E+00_JPRB ,&
&  0.32122E+00_JPRB, 0.31981E+00_JPRB, 0.32023E+00_JPRB, 0.32167E+00_JPRB, 0.33321E+00_JPRB, 0.33139E+00_JPRB, 0.33273E+00_JPRB ,&
&  0.33291E+00_JPRB, 0.33288E+00_JPRB, 0.33175E+00_JPRB, 0.32998E+00_JPRB, 0.32492E+00_JPRB, 0.30728E+00_JPRB, 0.30509E+00_JPRB ,&
&  0.29764E+00_JPRB, 0.28748E+00_JPRB, 0.27726E+00_JPRB, 0.27015E+00_JPRB, 0.26581E+00_JPRB, 0.26466E+00_JPRB, 0.26001E+00_JPRB ,&
&  0.25509E+00_JPRB, 0.24885E+00_JPRB, 0.24624E+00_JPRB, 0.24656E+00_JPRB, 0.24753E+00_JPRB, 0.24822E+00_JPRB, 0.24873E+00_JPRB ,&
&  0.25033E+00_JPRB, 0.25107E+00_JPRB, 0.25152E+00_JPRB, 0.25246E+00_JPRB, 0.25338E+00_JPRB, 0.25462E+00_JPRB, 0.25668E+00_JPRB ,&
&  0.26034E+00_JPRB, 0.26411E+00_JPRB, 0.26778E+00_JPRB, 0.27166E+00_JPRB, 0.27071E+00_JPRB, 0.26621E+00_JPRB, 0.26053E+00_JPRB ,&
&  0.25626E+00_JPRB, 0.25466E+00_JPRB, 0.25310E+00_JPRB, 0.25271E+00_JPRB, 0.25262E+00_JPRB, 0.25290E+00_JPRB, 0.25346E+00_JPRB ,&
&  0.25438E+00_JPRB, 0.25516E+00_JPRB, 0.25609E+00_JPRB, 0.25734E+00_JPRB, 0.25897E+00_JPRB, 0.26047E+00_JPRB, 0.26254E+00_JPRB ,&
&  0.26565E+00_JPRB, 0.26908E+00_JPRB, 0.27383E+00_JPRB, 0.27814E+00_JPRB, 0.28489E+00_JPRB, 0.29360E+00_JPRB, 0.30782E+00_JPRB ,&
&  0.32240E+00_JPRB, 0.35463E+00_JPRB, 0.36841E+00_JPRB, 0.38303E+00_JPRB, 0.39174E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.40095E+00_JPRB, 0.41125E+00_JPRB, 0.42525E+00_JPRB, 0.43475E+00_JPRB, 0.44178E+00_JPRB, 0.43769E+00_JPRB, 0.43365E+00_JPRB ,&
&  0.42924E+00_JPRB, 0.41584E+00_JPRB, 0.39839E+00_JPRB, 0.36929E+00_JPRB, 0.34481E+00_JPRB, 0.32624E+00_JPRB, 0.31513E+00_JPRB ,&
&  0.30772E+00_JPRB, 0.30485E+00_JPRB, 0.30159E+00_JPRB, 0.30025E+00_JPRB, 0.29793E+00_JPRB, 0.29873E+00_JPRB, 0.29937E+00_JPRB ,&
&  0.29841E+00_JPRB, 0.29767E+00_JPRB, 0.29761E+00_JPRB, 0.30096E+00_JPRB, 0.30763E+00_JPRB, 0.31269E+00_JPRB, 0.31776E+00_JPRB ,&
&  0.32094E+00_JPRB, 0.31782E+00_JPRB, 0.31992E+00_JPRB, 0.31848E+00_JPRB, 0.31817E+00_JPRB, 0.32005E+00_JPRB, 0.32229E+00_JPRB ,&
&  0.32267E+00_JPRB, 0.32213E+00_JPRB, 0.32073E+00_JPRB, 0.31661E+00_JPRB, 0.30338E+00_JPRB, 0.29695E+00_JPRB, 0.28818E+00_JPRB ,&
&  0.27996E+00_JPRB, 0.27040E+00_JPRB, 0.26219E+00_JPRB, 0.25865E+00_JPRB, 0.25342E+00_JPRB, 0.25177E+00_JPRB, 0.24836E+00_JPRB ,&
&  0.24455E+00_JPRB, 0.24107E+00_JPRB, 0.23915E+00_JPRB, 0.23823E+00_JPRB, 0.23793E+00_JPRB, 0.23626E+00_JPRB, 0.23573E+00_JPRB ,&
&  0.23627E+00_JPRB, 0.23675E+00_JPRB, 0.23808E+00_JPRB, 0.23913E+00_JPRB, 0.24019E+00_JPRB, 0.24099E+00_JPRB, 0.24188E+00_JPRB ,&
&  0.24364E+00_JPRB, 0.24600E+00_JPRB, 0.24704E+00_JPRB, 0.24813E+00_JPRB, 0.24774E+00_JPRB, 0.24688E+00_JPRB, 0.24571E+00_JPRB ,&
&  0.24470E+00_JPRB, 0.24433E+00_JPRB, 0.24432E+00_JPRB, 0.24414E+00_JPRB, 0.24457E+00_JPRB, 0.24544E+00_JPRB, 0.24620E+00_JPRB ,&
&  0.24782E+00_JPRB, 0.24922E+00_JPRB, 0.25089E+00_JPRB, 0.25311E+00_JPRB, 0.25545E+00_JPRB, 0.25836E+00_JPRB, 0.26156E+00_JPRB ,&
&  0.26412E+00_JPRB, 0.26764E+00_JPRB, 0.27059E+00_JPRB, 0.27295E+00_JPRB, 0.28003E+00_JPRB, 0.28790E+00_JPRB, 0.30422E+00_JPRB ,&
&  0.32115E+00_JPRB, 0.35803E+00_JPRB, 0.39792E+00_JPRB, 0.38626E+00_JPRB, 0.39247E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.37309E+00_JPRB, 0.38463E+00_JPRB, 0.39322E+00_JPRB, 0.39866E+00_JPRB, 0.40369E+00_JPRB, 0.40532E+00_JPRB, 0.41210E+00_JPRB ,&
&  0.40664E+00_JPRB, 0.38913E+00_JPRB, 0.37800E+00_JPRB, 0.36118E+00_JPRB, 0.34081E+00_JPRB, 0.32376E+00_JPRB, 0.31462E+00_JPRB ,&
&  0.30901E+00_JPRB, 0.30420E+00_JPRB, 0.29866E+00_JPRB, 0.29273E+00_JPRB, 0.28691E+00_JPRB, 0.28248E+00_JPRB, 0.27997E+00_JPRB ,&
&  0.27762E+00_JPRB, 0.27909E+00_JPRB, 0.28273E+00_JPRB, 0.28799E+00_JPRB, 0.29560E+00_JPRB, 0.29982E+00_JPRB, 0.30340E+00_JPRB ,&
&  0.30387E+00_JPRB, 0.29287E+00_JPRB, 0.29686E+00_JPRB, 0.30055E+00_JPRB, 0.30003E+00_JPRB, 0.30544E+00_JPRB, 0.31458E+00_JPRB ,&
&  0.31041E+00_JPRB, 0.29895E+00_JPRB, 0.30050E+00_JPRB, 0.29247E+00_JPRB, 0.28351E+00_JPRB, 0.28306E+00_JPRB, 0.27709E+00_JPRB ,&
&  0.27618E+00_JPRB, 0.26917E+00_JPRB, 0.26400E+00_JPRB, 0.26067E+00_JPRB, 0.25314E+00_JPRB, 0.25025E+00_JPRB, 0.24606E+00_JPRB ,&
&  0.24209E+00_JPRB, 0.23973E+00_JPRB, 0.23676E+00_JPRB, 0.23572E+00_JPRB, 0.23470E+00_JPRB, 0.23315E+00_JPRB, 0.23187E+00_JPRB ,&
&  0.23106E+00_JPRB, 0.23035E+00_JPRB, 0.23033E+00_JPRB, 0.23023E+00_JPRB, 0.23000E+00_JPRB, 0.23023E+00_JPRB, 0.23020E+00_JPRB ,&
&  0.23086E+00_JPRB, 0.23178E+00_JPRB, 0.23252E+00_JPRB, 0.23384E+00_JPRB, 0.23429E+00_JPRB, 0.23496E+00_JPRB, 0.23538E+00_JPRB ,&
&  0.23575E+00_JPRB, 0.23639E+00_JPRB, 0.23647E+00_JPRB, 0.23704E+00_JPRB, 0.23763E+00_JPRB, 0.23913E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24245E+00_JPRB, 0.24478E+00_JPRB, 0.24719E+00_JPRB, 0.24978E+00_JPRB, 0.25300E+00_JPRB, 0.25579E+00_JPRB, 0.25964E+00_JPRB ,&
&  0.26385E+00_JPRB, 0.26792E+00_JPRB, 0.27322E+00_JPRB, 0.27951E+00_JPRB, 0.28820E+00_JPRB, 0.29851E+00_JPRB, 0.30687E+00_JPRB ,&
&  0.31619E+00_JPRB, 0.34020E+00_JPRB, 0.34656E+00_JPRB, 0.35411E+00_JPRB, 0.36378E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.36148E+00_JPRB, 0.36698E+00_JPRB, 0.37122E+00_JPRB, 0.37398E+00_JPRB, 0.37684E+00_JPRB, 0.37859E+00_JPRB, 0.37928E+00_JPRB ,&
&  0.37258E+00_JPRB, 0.36771E+00_JPRB, 0.36044E+00_JPRB, 0.35149E+00_JPRB, 0.34321E+00_JPRB, 0.33413E+00_JPRB, 0.32507E+00_JPRB ,&
&  0.31757E+00_JPRB, 0.31053E+00_JPRB, 0.30372E+00_JPRB, 0.29641E+00_JPRB, 0.28889E+00_JPRB, 0.28132E+00_JPRB, 0.27887E+00_JPRB ,&
&  0.27892E+00_JPRB, 0.28122E+00_JPRB, 0.28461E+00_JPRB, 0.28617E+00_JPRB, 0.28709E+00_JPRB, 0.28738E+00_JPRB, 0.28610E+00_JPRB ,&
&  0.28196E+00_JPRB, 0.28320E+00_JPRB, 0.28505E+00_JPRB, 0.28623E+00_JPRB, 0.28836E+00_JPRB, 0.29025E+00_JPRB, 0.29028E+00_JPRB ,&
&  0.28807E+00_JPRB, 0.28531E+00_JPRB, 0.28149E+00_JPRB, 0.27788E+00_JPRB, 0.27696E+00_JPRB, 0.27503E+00_JPRB, 0.27282E+00_JPRB ,&
&  0.27180E+00_JPRB, 0.26798E+00_JPRB, 0.26347E+00_JPRB, 0.26029E+00_JPRB, 0.25556E+00_JPRB, 0.25237E+00_JPRB, 0.24811E+00_JPRB ,&
&  0.24483E+00_JPRB, 0.24203E+00_JPRB, 0.23892E+00_JPRB, 0.23664E+00_JPRB, 0.23456E+00_JPRB, 0.23302E+00_JPRB, 0.23144E+00_JPRB ,&
&  0.23056E+00_JPRB, 0.22963E+00_JPRB, 0.22899E+00_JPRB, 0.22824E+00_JPRB, 0.22770E+00_JPRB, 0.22719E+00_JPRB, 0.22679E+00_JPRB ,&
&  0.22656E+00_JPRB, 0.22695E+00_JPRB, 0.22717E+00_JPRB, 0.22762E+00_JPRB, 0.22813E+00_JPRB, 0.22832E+00_JPRB, 0.22885E+00_JPRB ,&
&  0.22999E+00_JPRB, 0.23125E+00_JPRB, 0.23188E+00_JPRB, 0.23282E+00_JPRB, 0.23389E+00_JPRB, 0.23517E+00_JPRB, 0.23707E+00_JPRB ,&
&  0.23902E+00_JPRB, 0.24122E+00_JPRB, 0.24376E+00_JPRB, 0.24620E+00_JPRB, 0.24952E+00_JPRB, 0.25294E+00_JPRB, 0.25661E+00_JPRB ,&
&  0.26117E+00_JPRB, 0.26594E+00_JPRB, 0.27100E+00_JPRB, 0.27659E+00_JPRB, 0.28479E+00_JPRB, 0.29064E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.30410E+00_JPRB, 0.30660E+00_JPRB, 0.34033E+00_JPRB, 0.34509E+00_JPRB, 0.35215E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.28550E+00_JPRB, 0.29158E+00_JPRB, 0.29716E+00_JPRB, 0.30524E+00_JPRB, 0.30737E+00_JPRB, 0.34386E+00_JPRB, 0.33883E+00_JPRB ,&
&  0.33635E+00_JPRB, 0.32511E+00_JPRB, 0.32465E+00_JPRB, 0.32153E+00_JPRB, 0.31724E+00_JPRB, 0.31267E+00_JPRB, 0.30783E+00_JPRB ,&
&  0.30335E+00_JPRB, 0.29794E+00_JPRB, 0.29371E+00_JPRB, 0.29026E+00_JPRB, 0.28722E+00_JPRB, 0.28417E+00_JPRB, 0.28159E+00_JPRB ,&
&  0.28065E+00_JPRB, 0.28075E+00_JPRB, 0.28010E+00_JPRB, 0.27487E+00_JPRB, 0.27101E+00_JPRB, 0.26877E+00_JPRB, 0.26827E+00_JPRB ,&
&  0.26837E+00_JPRB, 0.27087E+00_JPRB, 0.27275E+00_JPRB, 0.27448E+00_JPRB, 0.27693E+00_JPRB, 0.27603E+00_JPRB, 0.27512E+00_JPRB ,&
&  0.27373E+00_JPRB, 0.27228E+00_JPRB, 0.27085E+00_JPRB, 0.26953E+00_JPRB, 0.26821E+00_JPRB, 0.26621E+00_JPRB, 0.26466E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26122E+00_JPRB, 0.25797E+00_JPRB, 0.25585E+00_JPRB, 0.25325E+00_JPRB, 0.25105E+00_JPRB, 0.24873E+00_JPRB ,&
&  0.24696E+00_JPRB, 0.24541E+00_JPRB, 0.24368E+00_JPRB, 0.24160E+00_JPRB, 0.23958E+00_JPRB, 0.23781E+00_JPRB, 0.23616E+00_JPRB ,&
&  0.23533E+00_JPRB, 0.23443E+00_JPRB, 0.23354E+00_JPRB, 0.23258E+00_JPRB, 0.23213E+00_JPRB, 0.23161E+00_JPRB, 0.23112E+00_JPRB ,&
&  0.23073E+00_JPRB, 0.23041E+00_JPRB, 0.23035E+00_JPRB, 0.23038E+00_JPRB, 0.23045E+00_JPRB, 0.23082E+00_JPRB, 0.23129E+00_JPRB ,&
&  0.23195E+00_JPRB, 0.23276E+00_JPRB, 0.23396E+00_JPRB, 0.23519E+00_JPRB, 0.23628E+00_JPRB, 0.23711E+00_JPRB, 0.23812E+00_JPRB ,&
&  0.23975E+00_JPRB, 0.24160E+00_JPRB, 0.24367E+00_JPRB, 0.24508E+00_JPRB, 0.24741E+00_JPRB, 0.24954E+00_JPRB, 0.25170E+00_JPRB ,&
&  0.25384E+00_JPRB, 0.25660E+00_JPRB, 0.25987E+00_JPRB, 0.26353E+00_JPRB, 0.26796E+00_JPRB, 0.27110E+00_JPRB, 0.27408E+00_JPRB ,&
&  0.27712E+00_JPRB, 0.28191E+00_JPRB, 0.28369E+00_JPRB, 0.28427E+00_JPRB, 0.28487E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.26547E+00_JPRB, 0.26531E+00_JPRB, 0.26549E+00_JPRB, 0.26563E+00_JPRB, 0.26563E+00_JPRB, 0.26604E+00_JPRB, 0.26594E+00_JPRB ,&
&  0.26563E+00_JPRB, 0.26526E+00_JPRB, 0.26559E+00_JPRB, 0.26596E+00_JPRB, 0.26617E+00_JPRB, 0.26618E+00_JPRB, 0.26576E+00_JPRB ,&
&  0.26496E+00_JPRB, 0.26396E+00_JPRB, 0.26213E+00_JPRB, 0.26136E+00_JPRB, 0.26080E+00_JPRB, 0.26035E+00_JPRB, 0.25953E+00_JPRB ,&
&  0.25877E+00_JPRB, 0.25751E+00_JPRB, 0.25585E+00_JPRB, 0.25490E+00_JPRB, 0.25499E+00_JPRB, 0.25567E+00_JPRB, 0.25669E+00_JPRB ,&
&  0.25749E+00_JPRB, 0.25805E+00_JPRB, 0.25836E+00_JPRB, 0.25874E+00_JPRB, 0.25923E+00_JPRB, 0.25916E+00_JPRB, 0.25895E+00_JPRB ,&
&  0.25864E+00_JPRB, 0.25806E+00_JPRB, 0.25750E+00_JPRB, 0.25700E+00_JPRB, 0.25656E+00_JPRB, 0.25599E+00_JPRB, 0.25534E+00_JPRB ,&
&  0.25457E+00_JPRB, 0.25363E+00_JPRB, 0.25244E+00_JPRB, 0.25151E+00_JPRB, 0.25049E+00_JPRB, 0.24970E+00_JPRB, 0.24866E+00_JPRB ,&
&  0.24778E+00_JPRB, 0.24699E+00_JPRB, 0.24618E+00_JPRB, 0.24544E+00_JPRB, 0.24475E+00_JPRB, 0.24410E+00_JPRB, 0.24348E+00_JPRB ,&
&  0.24287E+00_JPRB, 0.24237E+00_JPRB, 0.24183E+00_JPRB, 0.24131E+00_JPRB, 0.24095E+00_JPRB, 0.24065E+00_JPRB, 0.24040E+00_JPRB ,&
&  0.24020E+00_JPRB, 0.24016E+00_JPRB, 0.24016E+00_JPRB, 0.24015E+00_JPRB, 0.24016E+00_JPRB, 0.24029E+00_JPRB, 0.24041E+00_JPRB ,&
&  0.24058E+00_JPRB, 0.24070E+00_JPRB, 0.24080E+00_JPRB, 0.24097E+00_JPRB, 0.24116E+00_JPRB, 0.24138E+00_JPRB, 0.24169E+00_JPRB ,&
&  0.24206E+00_JPRB, 0.24249E+00_JPRB, 0.24297E+00_JPRB, 0.24318E+00_JPRB, 0.24387E+00_JPRB, 0.24468E+00_JPRB, 0.24565E+00_JPRB ,&
&  0.24673E+00_JPRB, 0.24806E+00_JPRB, 0.24962E+00_JPRB, 0.25154E+00_JPRB, 0.25359E+00_JPRB, 0.25480E+00_JPRB, 0.25603E+00_JPRB ,&
&  0.25729E+00_JPRB, 0.25883E+00_JPRB, 0.26027E+00_JPRB, 0.26172E+00_JPRB, 0.26340E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB, 0.24725E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB, 0.72572E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.73869E+00_JPRB, 0.73807E+00_JPRB, 0.73746E+00_JPRB, 0.73678E+00_JPRB, 0.73590E+00_JPRB, 0.73503E+00_JPRB, 0.73430E+00_JPRB ,&
&  0.73357E+00_JPRB, 0.73309E+00_JPRB, 0.73218E+00_JPRB, 0.73125E+00_JPRB, 0.73028E+00_JPRB, 0.72943E+00_JPRB, 0.72848E+00_JPRB ,&
&  0.72751E+00_JPRB, 0.72653E+00_JPRB, 0.72579E+00_JPRB, 0.72489E+00_JPRB, 0.72392E+00_JPRB, 0.72304E+00_JPRB, 0.72224E+00_JPRB ,&
&  0.72149E+00_JPRB, 0.72064E+00_JPRB, 0.71977E+00_JPRB, 0.71870E+00_JPRB, 0.71803E+00_JPRB, 0.71706E+00_JPRB, 0.71627E+00_JPRB ,&
&  0.71537E+00_JPRB, 0.71476E+00_JPRB, 0.71406E+00_JPRB, 0.71340E+00_JPRB, 0.71261E+00_JPRB, 0.71255E+00_JPRB, 0.71210E+00_JPRB ,&
&  0.71190E+00_JPRB, 0.71168E+00_JPRB, 0.71174E+00_JPRB, 0.71169E+00_JPRB, 0.71175E+00_JPRB, 0.71188E+00_JPRB, 0.71248E+00_JPRB ,&
&  0.71298E+00_JPRB, 0.71358E+00_JPRB, 0.71447E+00_JPRB, 0.71529E+00_JPRB, 0.71604E+00_JPRB, 0.71682E+00_JPRB, 0.71769E+00_JPRB ,&
&  0.71886E+00_JPRB, 0.71999E+00_JPRB, 0.72105E+00_JPRB, 0.72213E+00_JPRB, 0.72320E+00_JPRB, 0.72409E+00_JPRB, 0.72482E+00_JPRB ,&
&  0.72527E+00_JPRB, 0.72582E+00_JPRB, 0.72632E+00_JPRB, 0.72687E+00_JPRB, 0.72699E+00_JPRB, 0.72735E+00_JPRB, 0.72757E+00_JPRB ,&
&  0.72777E+00_JPRB, 0.72743E+00_JPRB, 0.72749E+00_JPRB, 0.72736E+00_JPRB, 0.72745E+00_JPRB, 0.72740E+00_JPRB, 0.72781E+00_JPRB ,&
&  0.72810E+00_JPRB, 0.72875E+00_JPRB, 0.72943E+00_JPRB, 0.73038E+00_JPRB, 0.73109E+00_JPRB, 0.73189E+00_JPRB, 0.73259E+00_JPRB ,&
&  0.73339E+00_JPRB, 0.73422E+00_JPRB, 0.73509E+00_JPRB, 0.73564E+00_JPRB, 0.73615E+00_JPRB, 0.73675E+00_JPRB, 0.73739E+00_JPRB ,&
&  0.73824E+00_JPRB, 0.73872E+00_JPRB, 0.73928E+00_JPRB, 0.73978E+00_JPRB, 0.73997E+00_JPRB, 0.74013E+00_JPRB, 0.74011E+00_JPRB ,&
&  0.73999E+00_JPRB, 0.73980E+00_JPRB, 0.73953E+00_JPRB, 0.73932E+00_JPRB, 0.73910E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.74917E+00_JPRB, 0.74891E+00_JPRB, 0.74838E+00_JPRB, 0.74749E+00_JPRB, 0.74639E+00_JPRB, 0.74499E+00_JPRB, 0.74362E+00_JPRB ,&
&  0.74204E+00_JPRB, 0.74054E+00_JPRB, 0.73858E+00_JPRB, 0.73661E+00_JPRB, 0.73457E+00_JPRB, 0.73263E+00_JPRB, 0.73055E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.72663E+00_JPRB, 0.72526E+00_JPRB, 0.72375E+00_JPRB, 0.72216E+00_JPRB, 0.72024E+00_JPRB, 0.71870E+00_JPRB ,&
&  0.71703E+00_JPRB, 0.71506E+00_JPRB, 0.71293E+00_JPRB, 0.70914E+00_JPRB, 0.70591E+00_JPRB, 0.70295E+00_JPRB, 0.70039E+00_JPRB ,&
&  0.69802E+00_JPRB, 0.69579E+00_JPRB, 0.69411E+00_JPRB, 0.69301E+00_JPRB, 0.69424E+00_JPRB, 0.69456E+00_JPRB, 0.69424E+00_JPRB ,&
&  0.69374E+00_JPRB, 0.69132E+00_JPRB, 0.69059E+00_JPRB, 0.69038E+00_JPRB, 0.69047E+00_JPRB, 0.69219E+00_JPRB, 0.69364E+00_JPRB ,&
&  0.69518E+00_JPRB, 0.69645E+00_JPRB, 0.69892E+00_JPRB, 0.70128E+00_JPRB, 0.70349E+00_JPRB, 0.70585E+00_JPRB, 0.70723E+00_JPRB ,&
&  0.70881E+00_JPRB, 0.71108E+00_JPRB, 0.71322E+00_JPRB, 0.71546E+00_JPRB, 0.71680E+00_JPRB, 0.71803E+00_JPRB, 0.71891E+00_JPRB ,&
&  0.71961E+00_JPRB, 0.72062E+00_JPRB, 0.72172E+00_JPRB, 0.72274E+00_JPRB, 0.72338E+00_JPRB, 0.72402E+00_JPRB, 0.72444E+00_JPRB ,&
&  0.72456E+00_JPRB, 0.72399E+00_JPRB, 0.72381E+00_JPRB, 0.72350E+00_JPRB, 0.72295E+00_JPRB, 0.72220E+00_JPRB, 0.72196E+00_JPRB ,&
&  0.72169E+00_JPRB, 0.72117E+00_JPRB, 0.72034E+00_JPRB, 0.72064E+00_JPRB, 0.72114E+00_JPRB, 0.72137E+00_JPRB, 0.72133E+00_JPRB ,&
&  0.72358E+00_JPRB, 0.72672E+00_JPRB, 0.73040E+00_JPRB, 0.73540E+00_JPRB, 0.73910E+00_JPRB, 0.74144E+00_JPRB, 0.74303E+00_JPRB ,&
&  0.74361E+00_JPRB, 0.74420E+00_JPRB, 0.74472E+00_JPRB, 0.74560E+00_JPRB, 0.74619E+00_JPRB, 0.74731E+00_JPRB, 0.74896E+00_JPRB ,&
&  0.75013E+00_JPRB, 0.75053E+00_JPRB, 0.75042E+00_JPRB, 0.75003E+00_JPRB, 0.74955E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.75303E+00_JPRB, 0.75199E+00_JPRB, 0.75093E+00_JPRB, 0.74904E+00_JPRB, 0.74809E+00_JPRB, 0.74641E+00_JPRB, 0.74494E+00_JPRB ,&
&  0.74308E+00_JPRB, 0.74085E+00_JPRB, 0.73908E+00_JPRB, 0.73756E+00_JPRB, 0.73594E+00_JPRB, 0.73434E+00_JPRB, 0.73257E+00_JPRB ,&
&  0.73073E+00_JPRB, 0.72866E+00_JPRB, 0.72781E+00_JPRB, 0.72638E+00_JPRB, 0.72479E+00_JPRB, 0.72299E+00_JPRB, 0.71853E+00_JPRB ,&
&  0.71428E+00_JPRB, 0.70919E+00_JPRB, 0.70392E+00_JPRB, 0.69943E+00_JPRB, 0.69667E+00_JPRB, 0.67763E+00_JPRB, 0.67454E+00_JPRB ,&
&  0.68303E+00_JPRB, 0.68792E+00_JPRB, 0.69313E+00_JPRB, 0.69838E+00_JPRB, 0.68593E+00_JPRB, 0.67207E+00_JPRB, 0.66387E+00_JPRB ,&
&  0.64542E+00_JPRB, 0.65315E+00_JPRB, 0.65726E+00_JPRB, 0.66111E+00_JPRB, 0.66658E+00_JPRB, 0.66662E+00_JPRB, 0.66979E+00_JPRB ,&
&  0.68153E+00_JPRB, 0.68161E+00_JPRB, 0.68322E+00_JPRB, 0.68505E+00_JPRB, 0.68683E+00_JPRB, 0.68866E+00_JPRB, 0.69220E+00_JPRB ,&
&  0.69512E+00_JPRB, 0.69849E+00_JPRB, 0.70223E+00_JPRB, 0.70397E+00_JPRB, 0.70567E+00_JPRB, 0.70794E+00_JPRB, 0.71001E+00_JPRB ,&
&  0.71092E+00_JPRB, 0.71224E+00_JPRB, 0.71375E+00_JPRB, 0.71524E+00_JPRB, 0.71716E+00_JPRB, 0.71854E+00_JPRB, 0.71969E+00_JPRB ,&
&  0.72060E+00_JPRB, 0.72044E+00_JPRB, 0.72046E+00_JPRB, 0.72075E+00_JPRB, 0.72075E+00_JPRB, 0.72048E+00_JPRB, 0.72071E+00_JPRB ,&
&  0.72140E+00_JPRB, 0.72182E+00_JPRB, 0.72272E+00_JPRB, 0.72405E+00_JPRB, 0.72773E+00_JPRB, 0.73278E+00_JPRB, 0.74289E+00_JPRB ,&
&  0.75099E+00_JPRB, 0.75798E+00_JPRB, 0.76045E+00_JPRB, 0.76313E+00_JPRB, 0.76527E+00_JPRB, 0.76578E+00_JPRB, 0.76560E+00_JPRB ,&
&  0.76423E+00_JPRB, 0.76179E+00_JPRB, 0.75936E+00_JPRB, 0.75708E+00_JPRB, 0.75579E+00_JPRB, 0.75487E+00_JPRB, 0.75491E+00_JPRB ,&
&  0.75570E+00_JPRB, 0.75710E+00_JPRB, 0.75751E+00_JPRB, 0.75602E+00_JPRB, 0.75421E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.76411E+00_JPRB, 0.76129E+00_JPRB, 0.76047E+00_JPRB, 0.75794E+00_JPRB, 0.75569E+00_JPRB, 0.75242E+00_JPRB, 0.74967E+00_JPRB ,&
&  0.74637E+00_JPRB, 0.74339E+00_JPRB, 0.74072E+00_JPRB, 0.73823E+00_JPRB, 0.73692E+00_JPRB, 0.73543E+00_JPRB, 0.73444E+00_JPRB ,&
&  0.73321E+00_JPRB, 0.73147E+00_JPRB, 0.73057E+00_JPRB, 0.72921E+00_JPRB, 0.72794E+00_JPRB, 0.72648E+00_JPRB, 0.72282E+00_JPRB ,&
&  0.71647E+00_JPRB, 0.70789E+00_JPRB, 0.68986E+00_JPRB, 0.69354E+00_JPRB, 0.69207E+00_JPRB, 0.69013E+00_JPRB, 0.69123E+00_JPRB ,&
&  0.68199E+00_JPRB, 0.67934E+00_JPRB, 0.66829E+00_JPRB, 0.65851E+00_JPRB, 0.65511E+00_JPRB, 0.64599E+00_JPRB, 0.64640E+00_JPRB ,&
&  0.64352E+00_JPRB, 0.64207E+00_JPRB, 0.64336E+00_JPRB, 0.64252E+00_JPRB, 0.64767E+00_JPRB, 0.64971E+00_JPRB, 0.65329E+00_JPRB ,&
&  0.66027E+00_JPRB, 0.66588E+00_JPRB, 0.66906E+00_JPRB, 0.66243E+00_JPRB, 0.66550E+00_JPRB, 0.66864E+00_JPRB, 0.67267E+00_JPRB ,&
&  0.67661E+00_JPRB, 0.67941E+00_JPRB, 0.68248E+00_JPRB, 0.68466E+00_JPRB, 0.68694E+00_JPRB, 0.69056E+00_JPRB, 0.69423E+00_JPRB ,&
&  0.69873E+00_JPRB, 0.70198E+00_JPRB, 0.70407E+00_JPRB, 0.70628E+00_JPRB, 0.70851E+00_JPRB, 0.71074E+00_JPRB, 0.71354E+00_JPRB ,&
&  0.71700E+00_JPRB, 0.72124E+00_JPRB, 0.72572E+00_JPRB, 0.73036E+00_JPRB, 0.73464E+00_JPRB, 0.74019E+00_JPRB, 0.74648E+00_JPRB ,&
&  0.75569E+00_JPRB, 0.76394E+00_JPRB, 0.77692E+00_JPRB, 0.78467E+00_JPRB, 0.80152E+00_JPRB, 0.81718E+00_JPRB, 0.82622E+00_JPRB ,&
&  0.82393E+00_JPRB, 0.81890E+00_JPRB, 0.81425E+00_JPRB, 0.80942E+00_JPRB, 0.80518E+00_JPRB, 0.79963E+00_JPRB, 0.79565E+00_JPRB ,&
&  0.79159E+00_JPRB, 0.78763E+00_JPRB, 0.78373E+00_JPRB, 0.77978E+00_JPRB, 0.77650E+00_JPRB, 0.77284E+00_JPRB, 0.77065E+00_JPRB ,&
&  0.76992E+00_JPRB, 0.77073E+00_JPRB, 0.77092E+00_JPRB, 0.76994E+00_JPRB, 0.76726E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.77780E+00_JPRB, 0.77765E+00_JPRB, 0.77954E+00_JPRB, 0.77852E+00_JPRB, 0.77568E+00_JPRB, 0.77254E+00_JPRB, 0.76587E+00_JPRB ,&
&  0.75902E+00_JPRB, 0.75213E+00_JPRB, 0.74589E+00_JPRB, 0.74116E+00_JPRB, 0.73720E+00_JPRB, 0.73411E+00_JPRB, 0.73281E+00_JPRB ,&
&  0.73098E+00_JPRB, 0.73069E+00_JPRB, 0.73053E+00_JPRB, 0.73065E+00_JPRB, 0.73176E+00_JPRB, 0.73221E+00_JPRB, 0.73247E+00_JPRB ,&
&  0.72886E+00_JPRB, 0.71875E+00_JPRB, 0.70486E+00_JPRB, 0.70786E+00_JPRB, 0.71001E+00_JPRB, 0.71310E+00_JPRB, 0.71696E+00_JPRB ,&
&  0.71340E+00_JPRB, 0.68620E+00_JPRB, 0.69504E+00_JPRB, 0.68303E+00_JPRB, 0.67715E+00_JPRB, 0.67039E+00_JPRB, 0.65042E+00_JPRB ,&
&  0.64355E+00_JPRB, 0.63153E+00_JPRB, 0.62473E+00_JPRB, 0.62470E+00_JPRB, 0.62735E+00_JPRB, 0.63427E+00_JPRB, 0.64304E+00_JPRB ,&
&  0.64347E+00_JPRB, 0.65558E+00_JPRB, 0.65685E+00_JPRB, 0.65606E+00_JPRB, 0.65917E+00_JPRB, 0.66302E+00_JPRB, 0.66081E+00_JPRB ,&
&  0.66177E+00_JPRB, 0.66094E+00_JPRB, 0.66042E+00_JPRB, 0.66448E+00_JPRB, 0.67192E+00_JPRB, 0.67566E+00_JPRB, 0.67939E+00_JPRB ,&
&  0.68486E+00_JPRB, 0.68814E+00_JPRB, 0.69012E+00_JPRB, 0.69284E+00_JPRB, 0.69652E+00_JPRB, 0.70036E+00_JPRB, 0.70612E+00_JPRB ,&
&  0.71272E+00_JPRB, 0.72410E+00_JPRB, 0.73475E+00_JPRB, 0.75076E+00_JPRB, 0.76557E+00_JPRB, 0.78540E+00_JPRB, 0.80266E+00_JPRB ,&
&  0.82094E+00_JPRB, 0.83607E+00_JPRB, 0.85343E+00_JPRB, 0.86920E+00_JPRB, 0.88175E+00_JPRB, 0.87733E+00_JPRB, 0.86222E+00_JPRB ,&
&  0.85092E+00_JPRB, 0.83481E+00_JPRB, 0.82543E+00_JPRB, 0.81807E+00_JPRB, 0.81278E+00_JPRB, 0.81035E+00_JPRB, 0.80762E+00_JPRB ,&
&  0.80619E+00_JPRB, 0.80353E+00_JPRB, 0.80021E+00_JPRB, 0.79570E+00_JPRB, 0.79205E+00_JPRB, 0.78810E+00_JPRB, 0.78573E+00_JPRB ,&
&  0.78231E+00_JPRB, 0.78137E+00_JPRB, 0.77992E+00_JPRB, 0.77896E+00_JPRB, 0.77826E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.79228E+00_JPRB, 0.79655E+00_JPRB, 0.79523E+00_JPRB, 0.79320E+00_JPRB, 0.78575E+00_JPRB, 0.77932E+00_JPRB, 0.76936E+00_JPRB ,&
&  0.75943E+00_JPRB, 0.74941E+00_JPRB, 0.74022E+00_JPRB, 0.73375E+00_JPRB, 0.72863E+00_JPRB, 0.72508E+00_JPRB, 0.72370E+00_JPRB ,&
&  0.72333E+00_JPRB, 0.72444E+00_JPRB, 0.72546E+00_JPRB, 0.72679E+00_JPRB, 0.72991E+00_JPRB, 0.73348E+00_JPRB, 0.73726E+00_JPRB ,&
&  0.73838E+00_JPRB, 0.73429E+00_JPRB, 0.72177E+00_JPRB, 0.72129E+00_JPRB, 0.72814E+00_JPRB, 0.72368E+00_JPRB, 0.73701E+00_JPRB ,&
&  0.74730E+00_JPRB, 0.75179E+00_JPRB, 0.74147E+00_JPRB, 0.73414E+00_JPRB, 0.71994E+00_JPRB, 0.66426E+00_JPRB, 0.66098E+00_JPRB ,&
&  0.65197E+00_JPRB, 0.63713E+00_JPRB, 0.63072E+00_JPRB, 0.62508E+00_JPRB, 0.62385E+00_JPRB, 0.63267E+00_JPRB, 0.64540E+00_JPRB ,&
&  0.64742E+00_JPRB, 0.66592E+00_JPRB, 0.66536E+00_JPRB, 0.66396E+00_JPRB, 0.66503E+00_JPRB, 0.66884E+00_JPRB, 0.66592E+00_JPRB ,&
&  0.66658E+00_JPRB, 0.66663E+00_JPRB, 0.66684E+00_JPRB, 0.67205E+00_JPRB, 0.67741E+00_JPRB, 0.68159E+00_JPRB, 0.68780E+00_JPRB ,&
&  0.70127E+00_JPRB, 0.71035E+00_JPRB, 0.71550E+00_JPRB, 0.72007E+00_JPRB, 0.72709E+00_JPRB, 0.73535E+00_JPRB, 0.74688E+00_JPRB ,&
&  0.75527E+00_JPRB, 0.76603E+00_JPRB, 0.77852E+00_JPRB, 0.79177E+00_JPRB, 0.80510E+00_JPRB, 0.82027E+00_JPRB, 0.83648E+00_JPRB ,&
&  0.84521E+00_JPRB, 0.85727E+00_JPRB, 0.87058E+00_JPRB, 0.89263E+00_JPRB, 0.89273E+00_JPRB, 0.87768E+00_JPRB, 0.86619E+00_JPRB ,&
&  0.85750E+00_JPRB, 0.85812E+00_JPRB, 0.84969E+00_JPRB, 0.84465E+00_JPRB, 0.83872E+00_JPRB, 0.83656E+00_JPRB, 0.83221E+00_JPRB ,&
&  0.82894E+00_JPRB, 0.82429E+00_JPRB, 0.81794E+00_JPRB, 0.81047E+00_JPRB, 0.80503E+00_JPRB, 0.79937E+00_JPRB, 0.79563E+00_JPRB ,&
&  0.79112E+00_JPRB, 0.78930E+00_JPRB, 0.78827E+00_JPRB, 0.78860E+00_JPRB, 0.78837E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.80467E+00_JPRB, 0.80636E+00_JPRB, 0.79828E+00_JPRB, 0.79447E+00_JPRB, 0.78271E+00_JPRB, 0.77089E+00_JPRB, 0.75604E+00_JPRB ,&
&  0.74402E+00_JPRB, 0.73486E+00_JPRB, 0.72613E+00_JPRB, 0.72178E+00_JPRB, 0.71992E+00_JPRB, 0.71931E+00_JPRB, 0.72127E+00_JPRB ,&
&  0.72381E+00_JPRB, 0.72213E+00_JPRB, 0.72554E+00_JPRB, 0.73125E+00_JPRB, 0.73547E+00_JPRB, 0.73920E+00_JPRB, 0.74304E+00_JPRB ,&
&  0.74249E+00_JPRB, 0.74158E+00_JPRB, 0.73984E+00_JPRB, 0.73295E+00_JPRB, 0.73053E+00_JPRB, 0.74165E+00_JPRB, 0.75730E+00_JPRB ,&
&  0.75020E+00_JPRB, 0.75145E+00_JPRB, 0.75962E+00_JPRB, 0.76098E+00_JPRB, 0.74019E+00_JPRB, 0.70909E+00_JPRB, 0.68146E+00_JPRB ,&
&  0.66223E+00_JPRB, 0.63635E+00_JPRB, 0.62519E+00_JPRB, 0.61932E+00_JPRB, 0.62417E+00_JPRB, 0.63365E+00_JPRB, 0.65171E+00_JPRB ,&
&  0.66209E+00_JPRB, 0.66583E+00_JPRB, 0.67222E+00_JPRB, 0.67812E+00_JPRB, 0.67873E+00_JPRB, 0.68210E+00_JPRB, 0.68240E+00_JPRB ,&
&  0.68736E+00_JPRB, 0.69072E+00_JPRB, 0.69443E+00_JPRB, 0.70264E+00_JPRB, 0.71509E+00_JPRB, 0.72814E+00_JPRB, 0.74840E+00_JPRB ,&
&  0.79703E+00_JPRB, 0.81345E+00_JPRB, 0.82535E+00_JPRB, 0.83129E+00_JPRB, 0.83951E+00_JPRB, 0.84228E+00_JPRB, 0.83446E+00_JPRB ,&
&  0.83533E+00_JPRB, 0.84823E+00_JPRB, 0.85111E+00_JPRB, 0.85524E+00_JPRB, 0.86090E+00_JPRB, 0.86665E+00_JPRB, 0.87375E+00_JPRB ,&
&  0.86802E+00_JPRB, 0.88278E+00_JPRB, 0.95361E+00_JPRB, 0.97438E+00_JPRB, 0.95933E+00_JPRB, 0.93959E+00_JPRB, 0.91963E+00_JPRB ,&
&  0.90382E+00_JPRB, 0.88970E+00_JPRB, 0.87749E+00_JPRB, 0.86748E+00_JPRB, 0.86027E+00_JPRB, 0.85624E+00_JPRB, 0.85531E+00_JPRB ,&
&  0.84786E+00_JPRB, 0.83921E+00_JPRB, 0.82967E+00_JPRB, 0.82273E+00_JPRB, 0.81699E+00_JPRB, 0.81090E+00_JPRB, 0.80770E+00_JPRB ,&
&  0.80465E+00_JPRB, 0.80552E+00_JPRB, 0.80630E+00_JPRB, 0.80729E+00_JPRB, 0.80671E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.80655E+00_JPRB, 0.80868E+00_JPRB, 0.80703E+00_JPRB, 0.79568E+00_JPRB, 0.77215E+00_JPRB, 0.75002E+00_JPRB, 0.73369E+00_JPRB ,&
&  0.72601E+00_JPRB, 0.72177E+00_JPRB, 0.71776E+00_JPRB, 0.71583E+00_JPRB, 0.71584E+00_JPRB, 0.71755E+00_JPRB, 0.72085E+00_JPRB ,&
&  0.72580E+00_JPRB, 0.73049E+00_JPRB, 0.73293E+00_JPRB, 0.73426E+00_JPRB, 0.73819E+00_JPRB, 0.74500E+00_JPRB, 0.75242E+00_JPRB ,&
&  0.75626E+00_JPRB, 0.75582E+00_JPRB, 0.75805E+00_JPRB, 0.75621E+00_JPRB, 0.75499E+00_JPRB, 0.75228E+00_JPRB, 0.75319E+00_JPRB ,&
&  0.76384E+00_JPRB, 0.77246E+00_JPRB, 0.76280E+00_JPRB, 0.76488E+00_JPRB, 0.77209E+00_JPRB, 0.74444E+00_JPRB, 0.70604E+00_JPRB ,&
&  0.66599E+00_JPRB, 0.63639E+00_JPRB, 0.62369E+00_JPRB, 0.62049E+00_JPRB, 0.62959E+00_JPRB, 0.65021E+00_JPRB, 0.67470E+00_JPRB ,&
&  0.67480E+00_JPRB, 0.68136E+00_JPRB, 0.69258E+00_JPRB, 0.69873E+00_JPRB, 0.69941E+00_JPRB, 0.69867E+00_JPRB, 0.69931E+00_JPRB ,&
&  0.70369E+00_JPRB, 0.71613E+00_JPRB, 0.73765E+00_JPRB, 0.76840E+00_JPRB, 0.81350E+00_JPRB, 0.83464E+00_JPRB, 0.87805E+00_JPRB ,&
&  0.94611E+00_JPRB, 0.94611E+00_JPRB, 0.95426E+00_JPRB, 0.95266E+00_JPRB, 0.95014E+00_JPRB, 0.94414E+00_JPRB, 0.94325E+00_JPRB ,&
&  0.94418E+00_JPRB, 0.94160E+00_JPRB, 0.93256E+00_JPRB, 0.92274E+00_JPRB, 0.92353E+00_JPRB, 0.93270E+00_JPRB, 0.95408E+00_JPRB ,&
&  0.98800E+00_JPRB, 0.10413E+01_JPRB, 0.11751E+01_JPRB, 0.10855E+01_JPRB, 0.10395E+01_JPRB, 0.10195E+01_JPRB, 0.99075E+00_JPRB ,&
&  0.96633E+00_JPRB, 0.94981E+00_JPRB, 0.93912E+00_JPRB, 0.92608E+00_JPRB, 0.90204E+00_JPRB, 0.88473E+00_JPRB, 0.87180E+00_JPRB ,&
&  0.86075E+00_JPRB, 0.84989E+00_JPRB, 0.84157E+00_JPRB, 0.83576E+00_JPRB, 0.82977E+00_JPRB, 0.82562E+00_JPRB, 0.82454E+00_JPRB ,&
&  0.82165E+00_JPRB, 0.81976E+00_JPRB, 0.81789E+00_JPRB, 0.81327E+00_JPRB, 0.80942E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.81234E+00_JPRB, 0.81401E+00_JPRB, 0.80322E+00_JPRB, 0.79025E+00_JPRB, 0.76270E+00_JPRB, 0.74221E+00_JPRB, 0.72858E+00_JPRB ,&
&  0.72260E+00_JPRB, 0.71698E+00_JPRB, 0.71352E+00_JPRB, 0.71241E+00_JPRB, 0.71381E+00_JPRB, 0.71787E+00_JPRB, 0.72101E+00_JPRB ,&
&  0.72388E+00_JPRB, 0.72916E+00_JPRB, 0.73574E+00_JPRB, 0.74072E+00_JPRB, 0.74485E+00_JPRB, 0.74997E+00_JPRB, 0.75684E+00_JPRB ,&
&  0.76442E+00_JPRB, 0.76838E+00_JPRB, 0.77086E+00_JPRB, 0.77320E+00_JPRB, 0.77935E+00_JPRB, 0.78773E+00_JPRB, 0.79003E+00_JPRB ,&
&  0.78057E+00_JPRB, 0.77741E+00_JPRB, 0.78557E+00_JPRB, 0.78645E+00_JPRB, 0.79146E+00_JPRB, 0.77166E+00_JPRB, 0.72651E+00_JPRB ,&
&  0.68099E+00_JPRB, 0.65069E+00_JPRB, 0.64644E+00_JPRB, 0.65828E+00_JPRB, 0.68026E+00_JPRB, 0.68683E+00_JPRB, 0.69934E+00_JPRB ,&
&  0.70929E+00_JPRB, 0.71233E+00_JPRB, 0.71470E+00_JPRB, 0.72110E+00_JPRB, 0.72435E+00_JPRB, 0.72382E+00_JPRB, 0.72647E+00_JPRB ,&
&  0.73679E+00_JPRB, 0.75500E+00_JPRB, 0.76421E+00_JPRB, 0.76640E+00_JPRB, 0.80210E+00_JPRB, 0.89106E+00_JPRB, 0.97062E+00_JPRB ,&
&  0.98975E+00_JPRB, 0.10011E+01_JPRB, 0.10131E+01_JPRB, 0.10468E+01_JPRB, 0.10620E+01_JPRB, 0.10764E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.10439E+01_JPRB, 0.10246E+01_JPRB, 0.10236E+01_JPRB, 0.10495E+01_JPRB, 0.10989E+01_JPRB, 0.11472E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.11492E+01_JPRB, 0.10623E+01_JPRB, 0.98424E+00_JPRB, 0.96035E+00_JPRB, 0.96676E+00_JPRB, 0.97924E+00_JPRB ,&
&  0.97476E+00_JPRB, 0.96687E+00_JPRB, 0.95774E+00_JPRB, 0.94804E+00_JPRB, 0.90820E+00_JPRB, 0.88309E+00_JPRB, 0.86305E+00_JPRB ,&
&  0.84924E+00_JPRB, 0.83928E+00_JPRB, 0.84027E+00_JPRB, 0.83614E+00_JPRB, 0.83282E+00_JPRB, 0.82857E+00_JPRB, 0.82177E+00_JPRB ,&
&  0.81836E+00_JPRB, 0.81561E+00_JPRB, 0.81049E+00_JPRB, 0.80731E+00_JPRB, 0.80673E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.82768E+00_JPRB, 0.82046E+00_JPRB, 0.81287E+00_JPRB, 0.79529E+00_JPRB, 0.78178E+00_JPRB, 0.77395E+00_JPRB, 0.76738E+00_JPRB ,&
&  0.77043E+00_JPRB, 0.74995E+00_JPRB, 0.72918E+00_JPRB, 0.71567E+00_JPRB, 0.70881E+00_JPRB, 0.70832E+00_JPRB, 0.71494E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.73170E+00_JPRB, 0.73917E+00_JPRB, 0.74787E+00_JPRB, 0.75626E+00_JPRB, 0.76219E+00_JPRB, 0.76817E+00_JPRB ,&
&  0.77580E+00_JPRB, 0.78358E+00_JPRB, 0.79102E+00_JPRB, 0.80161E+00_JPRB, 0.80970E+00_JPRB, 0.81595E+00_JPRB, 0.81507E+00_JPRB ,&
&  0.80877E+00_JPRB, 0.80278E+00_JPRB, 0.80335E+00_JPRB, 0.80598E+00_JPRB, 0.80720E+00_JPRB, 0.78964E+00_JPRB, 0.74334E+00_JPRB ,&
&  0.70289E+00_JPRB, 0.68639E+00_JPRB, 0.70154E+00_JPRB, 0.71804E+00_JPRB, 0.72900E+00_JPRB, 0.73814E+00_JPRB, 0.73863E+00_JPRB ,&
&  0.73609E+00_JPRB, 0.73580E+00_JPRB, 0.74399E+00_JPRB, 0.75202E+00_JPRB, 0.75409E+00_JPRB, 0.75607E+00_JPRB, 0.76159E+00_JPRB ,&
&  0.77768E+00_JPRB, 0.79627E+00_JPRB, 0.80587E+00_JPRB, 0.80291E+00_JPRB, 0.88112E+00_JPRB, 0.11281E+01_JPRB, 0.11517E+01_JPRB ,&
&  0.11722E+01_JPRB, 0.11867E+01_JPRB, 0.12170E+01_JPRB, 0.12452E+01_JPRB, 0.12305E+01_JPRB, 0.12250E+01_JPRB, 0.12258E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.11887E+01_JPRB, 0.11932E+01_JPRB, 0.12410E+01_JPRB, 0.12757E+01_JPRB, 0.12959E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.12996E+01_JPRB, 0.12339E+01_JPRB, 0.11528E+01_JPRB, 0.10783E+01_JPRB, 0.10408E+01_JPRB, 0.10170E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.98721E+00_JPRB, 0.98177E+00_JPRB, 0.98153E+00_JPRB, 0.97763E+00_JPRB, 0.94208E+00_JPRB, 0.89795E+00_JPRB, 0.87283E+00_JPRB ,&
&  0.85721E+00_JPRB, 0.84339E+00_JPRB, 0.83181E+00_JPRB, 0.82520E+00_JPRB, 0.82981E+00_JPRB, 0.83360E+00_JPRB, 0.82855E+00_JPRB ,&
&  0.83080E+00_JPRB, 0.82485E+00_JPRB, 0.82249E+00_JPRB, 0.82281E+00_JPRB, 0.82938E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.83461E+00_JPRB, 0.83049E+00_JPRB, 0.82258E+00_JPRB, 0.81256E+00_JPRB, 0.80872E+00_JPRB, 0.80400E+00_JPRB, 0.79021E+00_JPRB ,&
&  0.79758E+00_JPRB, 0.75900E+00_JPRB, 0.73737E+00_JPRB, 0.72869E+00_JPRB, 0.72774E+00_JPRB, 0.72244E+00_JPRB, 0.71628E+00_JPRB ,&
&  0.71960E+00_JPRB, 0.73188E+00_JPRB, 0.74525E+00_JPRB, 0.75559E+00_JPRB, 0.76588E+00_JPRB, 0.77310E+00_JPRB, 0.77960E+00_JPRB ,&
&  0.78784E+00_JPRB, 0.79859E+00_JPRB, 0.81300E+00_JPRB, 0.82384E+00_JPRB, 0.83064E+00_JPRB, 0.83749E+00_JPRB, 0.83732E+00_JPRB ,&
&  0.83470E+00_JPRB, 0.82585E+00_JPRB, 0.81959E+00_JPRB, 0.81660E+00_JPRB, 0.81281E+00_JPRB, 0.81204E+00_JPRB, 0.78887E+00_JPRB ,&
&  0.74857E+00_JPRB, 0.74812E+00_JPRB, 0.76034E+00_JPRB, 0.77201E+00_JPRB, 0.77222E+00_JPRB, 0.76747E+00_JPRB, 0.76632E+00_JPRB ,&
&  0.76776E+00_JPRB, 0.76934E+00_JPRB, 0.77469E+00_JPRB, 0.77627E+00_JPRB, 0.77977E+00_JPRB, 0.78351E+00_JPRB, 0.79928E+00_JPRB ,&
&  0.83275E+00_JPRB, 0.81309E+00_JPRB, 0.82344E+00_JPRB, 0.87690E+00_JPRB, 0.10787E+01_JPRB, 0.11841E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.13983E+01_JPRB, 0.14300E+01_JPRB, 0.14307E+01_JPRB, 0.14367E+01_JPRB, 0.14472E+01_JPRB, 0.14559E+01_JPRB, 0.14703E+01_JPRB ,&
&  0.14407E+01_JPRB, 0.14497E+01_JPRB, 0.14710E+01_JPRB, 0.15107E+01_JPRB, 0.15147E+01_JPRB, 0.15040E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.14329E+01_JPRB, 0.13712E+01_JPRB, 0.12845E+01_JPRB, 0.12003E+01_JPRB, 0.11295E+01_JPRB, 0.10946E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10387E+01_JPRB, 0.10270E+01_JPRB, 0.10123E+01_JPRB, 0.99361E+00_JPRB, 0.97419E+00_JPRB, 0.96814E+00_JPRB ,&
&  0.92863E+00_JPRB, 0.90065E+00_JPRB, 0.87242E+00_JPRB, 0.85867E+00_JPRB, 0.85957E+00_JPRB, 0.86439E+00_JPRB, 0.86686E+00_JPRB ,&
&  0.87165E+00_JPRB, 0.85861E+00_JPRB, 0.85531E+00_JPRB, 0.85082E+00_JPRB, 0.84654E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.84259E+00_JPRB, 0.84411E+00_JPRB, 0.83873E+00_JPRB, 0.83435E+00_JPRB, 0.83776E+00_JPRB, 0.83648E+00_JPRB, 0.83286E+00_JPRB ,&
&  0.86998E+00_JPRB, 0.84599E+00_JPRB, 0.85134E+00_JPRB, 0.78643E+00_JPRB, 0.75931E+00_JPRB, 0.74481E+00_JPRB, 0.72749E+00_JPRB ,&
&  0.72521E+00_JPRB, 0.73126E+00_JPRB, 0.74011E+00_JPRB, 0.75251E+00_JPRB, 0.76583E+00_JPRB, 0.77490E+00_JPRB, 0.78320E+00_JPRB ,&
&  0.79303E+00_JPRB, 0.80564E+00_JPRB, 0.81807E+00_JPRB, 0.82774E+00_JPRB, 0.83530E+00_JPRB, 0.84421E+00_JPRB, 0.84254E+00_JPRB ,&
&  0.82837E+00_JPRB, 0.82218E+00_JPRB, 0.82021E+00_JPRB, 0.81654E+00_JPRB, 0.81981E+00_JPRB, 0.82092E+00_JPRB, 0.83053E+00_JPRB ,&
&  0.82682E+00_JPRB, 0.81535E+00_JPRB, 0.82076E+00_JPRB, 0.81481E+00_JPRB, 0.80883E+00_JPRB, 0.80140E+00_JPRB, 0.79418E+00_JPRB ,&
&  0.79027E+00_JPRB, 0.79116E+00_JPRB, 0.79746E+00_JPRB, 0.80253E+00_JPRB, 0.80760E+00_JPRB, 0.82022E+00_JPRB, 0.85260E+00_JPRB ,&
&  0.90236E+00_JPRB, 0.86340E+00_JPRB, 0.94920E+00_JPRB, 0.99705E+00_JPRB, 0.12060E+01_JPRB, 0.14324E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.17393E+01_JPRB, 0.17592E+01_JPRB, 0.17373E+01_JPRB, 0.17206E+01_JPRB, 0.17344E+01_JPRB, 0.17052E+01_JPRB, 0.16516E+01_JPRB ,&
&  0.16390E+01_JPRB, 0.16084E+01_JPRB, 0.16953E+01_JPRB, 0.16933E+01_JPRB, 0.16462E+01_JPRB, 0.16119E+01_JPRB, 0.15486E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.14439E+01_JPRB, 0.13925E+01_JPRB, 0.12849E+01_JPRB, 0.12466E+01_JPRB, 0.12083E+01_JPRB, 0.11337E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.10607E+01_JPRB, 0.10213E+01_JPRB, 0.10315E+01_JPRB, 0.10036E+01_JPRB, 0.98911E+00_JPRB, 0.98356E+00_JPRB ,&
&  0.95781E+00_JPRB, 0.92951E+00_JPRB, 0.93347E+00_JPRB, 0.92997E+00_JPRB, 0.91566E+00_JPRB, 0.90458E+00_JPRB, 0.90514E+00_JPRB ,&
&  0.89104E+00_JPRB, 0.88445E+00_JPRB, 0.89032E+00_JPRB, 0.87159E+00_JPRB, 0.85513E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.87056E+00_JPRB, 0.86906E+00_JPRB, 0.86037E+00_JPRB, 0.86088E+00_JPRB, 0.87002E+00_JPRB, 0.87719E+00_JPRB, 0.88491E+00_JPRB ,&
&  0.90690E+00_JPRB, 0.91415E+00_JPRB, 0.91105E+00_JPRB, 0.89361E+00_JPRB, 0.86555E+00_JPRB, 0.80629E+00_JPRB, 0.73966E+00_JPRB ,&
&  0.72519E+00_JPRB, 0.73732E+00_JPRB, 0.75126E+00_JPRB, 0.75901E+00_JPRB, 0.77024E+00_JPRB, 0.78240E+00_JPRB, 0.79341E+00_JPRB ,&
&  0.80470E+00_JPRB, 0.81372E+00_JPRB, 0.82416E+00_JPRB, 0.83381E+00_JPRB, 0.84040E+00_JPRB, 0.84993E+00_JPRB, 0.85520E+00_JPRB ,&
&  0.83252E+00_JPRB, 0.82016E+00_JPRB, 0.81356E+00_JPRB, 0.81764E+00_JPRB, 0.83853E+00_JPRB, 0.85002E+00_JPRB, 0.86191E+00_JPRB ,&
&  0.87088E+00_JPRB, 0.87340E+00_JPRB, 0.86732E+00_JPRB, 0.86084E+00_JPRB, 0.85225E+00_JPRB, 0.84596E+00_JPRB, 0.83783E+00_JPRB ,&
&  0.83222E+00_JPRB, 0.83045E+00_JPRB, 0.83584E+00_JPRB, 0.84123E+00_JPRB, 0.85021E+00_JPRB, 0.88114E+00_JPRB, 0.95382E+00_JPRB ,&
&  0.10284E+01_JPRB, 0.11091E+01_JPRB, 0.11561E+01_JPRB, 0.11707E+01_JPRB, 0.13604E+01_JPRB, 0.15452E+01_JPRB, 0.16388E+01_JPRB ,&
&  0.17797E+01_JPRB, 0.17606E+01_JPRB, 0.18557E+01_JPRB, 0.19567E+01_JPRB, 0.20366E+01_JPRB, 0.20051E+01_JPRB, 0.18939E+01_JPRB ,&
&  0.18665E+01_JPRB, 0.19379E+01_JPRB, 0.19368E+01_JPRB, 0.19280E+01_JPRB, 0.18543E+01_JPRB, 0.17999E+01_JPRB, 0.17269E+01_JPRB ,&
&  0.17077E+01_JPRB, 0.16940E+01_JPRB, 0.16307E+01_JPRB, 0.15530E+01_JPRB, 0.14612E+01_JPRB, 0.13629E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.10560E+01_JPRB, 0.10137E+01_JPRB, 0.10113E+01_JPRB, 0.10202E+01_JPRB, 0.99436E+00_JPRB, 0.98876E+00_JPRB ,&
&  0.98531E+00_JPRB, 0.98726E+00_JPRB, 0.10080E+01_JPRB, 0.98830E+00_JPRB, 0.95800E+00_JPRB, 0.92603E+00_JPRB, 0.90843E+00_JPRB ,&
&  0.88725E+00_JPRB, 0.89628E+00_JPRB, 0.90089E+00_JPRB, 0.89056E+00_JPRB, 0.87598E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.89522E+00_JPRB, 0.88215E+00_JPRB, 0.87514E+00_JPRB, 0.88284E+00_JPRB, 0.89629E+00_JPRB, 0.91292E+00_JPRB, 0.93655E+00_JPRB ,&
&  0.95989E+00_JPRB, 0.96903E+00_JPRB, 0.95269E+00_JPRB, 0.92539E+00_JPRB, 0.90174E+00_JPRB, 0.88481E+00_JPRB, 0.84134E+00_JPRB ,&
&  0.78306E+00_JPRB, 0.78122E+00_JPRB, 0.77141E+00_JPRB, 0.77596E+00_JPRB, 0.79321E+00_JPRB, 0.80509E+00_JPRB, 0.81785E+00_JPRB ,&
&  0.83478E+00_JPRB, 0.84248E+00_JPRB, 0.84780E+00_JPRB, 0.85234E+00_JPRB, 0.86020E+00_JPRB, 0.88180E+00_JPRB, 0.89776E+00_JPRB ,&
&  0.86651E+00_JPRB, 0.84611E+00_JPRB, 0.84472E+00_JPRB, 0.85621E+00_JPRB, 0.87828E+00_JPRB, 0.90027E+00_JPRB, 0.91376E+00_JPRB ,&
&  0.92216E+00_JPRB, 0.92459E+00_JPRB, 0.92098E+00_JPRB, 0.91460E+00_JPRB, 0.90520E+00_JPRB, 0.89440E+00_JPRB, 0.88724E+00_JPRB ,&
&  0.88175E+00_JPRB, 0.88148E+00_JPRB, 0.88794E+00_JPRB, 0.89375E+00_JPRB, 0.90369E+00_JPRB, 0.96873E+00_JPRB, 0.11151E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.14152E+01_JPRB, 0.14664E+01_JPRB, 0.15790E+01_JPRB, 0.16966E+01_JPRB, 0.18216E+01_JPRB, 0.19111E+01_JPRB ,&
&  0.19907E+01_JPRB, 0.19250E+01_JPRB, 0.21311E+01_JPRB, 0.22303E+01_JPRB, 0.22319E+01_JPRB, 0.22704E+01_JPRB, 0.21567E+01_JPRB ,&
&  0.20759E+01_JPRB, 0.19983E+01_JPRB, 0.20643E+01_JPRB, 0.20689E+01_JPRB, 0.20279E+01_JPRB, 0.20462E+01_JPRB, 0.19863E+01_JPRB ,&
&  0.19495E+01_JPRB, 0.18925E+01_JPRB, 0.16804E+01_JPRB, 0.15097E+01_JPRB, 0.13307E+01_JPRB, 0.13750E+01_JPRB, 0.12979E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.10625E+01_JPRB, 0.10636E+01_JPRB, 0.10606E+01_JPRB, 0.10842E+01_JPRB, 0.11039E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.10793E+01_JPRB, 0.11010E+01_JPRB, 0.10763E+01_JPRB, 0.10510E+01_JPRB, 0.10198E+01_JPRB, 0.97600E+00_JPRB, 0.94840E+00_JPRB ,&
&  0.95311E+00_JPRB, 0.95792E+00_JPRB, 0.94509E+00_JPRB, 0.92268E+00_JPRB, 0.90756E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.94400E+00_JPRB, 0.93021E+00_JPRB, 0.92342E+00_JPRB, 0.92856E+00_JPRB, 0.94069E+00_JPRB, 0.96254E+00_JPRB, 0.98772E+00_JPRB ,&
&  0.10059E+01_JPRB, 0.10041E+01_JPRB, 0.97578E+00_JPRB, 0.95095E+00_JPRB, 0.93506E+00_JPRB, 0.92072E+00_JPRB, 0.91615E+00_JPRB ,&
&  0.84388E+00_JPRB, 0.82006E+00_JPRB, 0.78560E+00_JPRB, 0.78821E+00_JPRB, 0.81949E+00_JPRB, 0.83108E+00_JPRB, 0.84437E+00_JPRB ,&
&  0.85644E+00_JPRB, 0.87262E+00_JPRB, 0.87848E+00_JPRB, 0.88504E+00_JPRB, 0.89739E+00_JPRB, 0.92885E+00_JPRB, 0.96288E+00_JPRB ,&
&  0.93990E+00_JPRB, 0.91635E+00_JPRB, 0.92255E+00_JPRB, 0.94109E+00_JPRB, 0.95343E+00_JPRB, 0.97757E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.99396E+00_JPRB, 0.97774E+00_JPRB, 0.96483E+00_JPRB, 0.95516E+00_JPRB, 0.94639E+00_JPRB, 0.94595E+00_JPRB ,&
&  0.94871E+00_JPRB, 0.95181E+00_JPRB, 0.95707E+00_JPRB, 0.96217E+00_JPRB, 0.96616E+00_JPRB, 0.10634E+01_JPRB, 0.12586E+01_JPRB ,&
&  0.15052E+01_JPRB, 0.15661E+01_JPRB, 0.16728E+01_JPRB, 0.20749E+01_JPRB, 0.22736E+01_JPRB, 0.23527E+01_JPRB, 0.23257E+01_JPRB ,&
&  0.22814E+01_JPRB, 0.21700E+01_JPRB, 0.21634E+01_JPRB, 0.22148E+01_JPRB, 0.22051E+01_JPRB, 0.23121E+01_JPRB, 0.22558E+01_JPRB ,&
&  0.21229E+01_JPRB, 0.20840E+01_JPRB, 0.20857E+01_JPRB, 0.20664E+01_JPRB, 0.20149E+01_JPRB, 0.20353E+01_JPRB, 0.20027E+01_JPRB ,&
&  0.19257E+01_JPRB, 0.17152E+01_JPRB, 0.13894E+01_JPRB, 0.13252E+01_JPRB, 0.11383E+01_JPRB, 0.11854E+01_JPRB, 0.13656E+01_JPRB ,&
&  0.13762E+01_JPRB, 0.13032E+01_JPRB, 0.12313E+01_JPRB, 0.12036E+01_JPRB, 0.12085E+01_JPRB, 0.13050E+01_JPRB, 0.12772E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.12226E+01_JPRB, 0.12151E+01_JPRB, 0.11820E+01_JPRB, 0.11341E+01_JPRB, 0.10927E+01_JPRB, 0.10646E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10519E+01_JPRB, 0.10357E+01_JPRB, 0.99036E+00_JPRB, 0.95805E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.10260E+01_JPRB, 0.10101E+01_JPRB, 0.99461E+00_JPRB, 0.99575E+00_JPRB, 0.10069E+01_JPRB, 0.10256E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10298E+01_JPRB, 0.99457E+00_JPRB, 0.97789E+00_JPRB, 0.96478E+00_JPRB, 0.94567E+00_JPRB, 0.93977E+00_JPRB ,&
&  0.95300E+00_JPRB, 0.89609E+00_JPRB, 0.84500E+00_JPRB, 0.80513E+00_JPRB, 0.82173E+00_JPRB, 0.83177E+00_JPRB, 0.85316E+00_JPRB ,&
&  0.86905E+00_JPRB, 0.90021E+00_JPRB, 0.91471E+00_JPRB, 0.92545E+00_JPRB, 0.94822E+00_JPRB, 0.99689E+00_JPRB, 0.10428E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10569E+01_JPRB, 0.10766E+01_JPRB, 0.11087E+01_JPRB, 0.11254E+01_JPRB, 0.10835E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10641E+01_JPRB, 0.10357E+01_JPRB, 0.10255E+01_JPRB, 0.10239E+01_JPRB, 0.10298E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10540E+01_JPRB, 0.10299E+01_JPRB, 0.10227E+01_JPRB, 0.10395E+01_JPRB, 0.11752E+01_JPRB, 0.13023E+01_JPRB ,&
&  0.15130E+01_JPRB, 0.15439E+01_JPRB, 0.15771E+01_JPRB, 0.19498E+01_JPRB, 0.23042E+01_JPRB, 0.24622E+01_JPRB, 0.25198E+01_JPRB ,&
&  0.24868E+01_JPRB, 0.24133E+01_JPRB, 0.23605E+01_JPRB, 0.22681E+01_JPRB, 0.22779E+01_JPRB, 0.23361E+01_JPRB, 0.22950E+01_JPRB ,&
&  0.21630E+01_JPRB, 0.20596E+01_JPRB, 0.20300E+01_JPRB, 0.19579E+01_JPRB, 0.18986E+01_JPRB, 0.17866E+01_JPRB, 0.18293E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.12691E+01_JPRB, 0.11146E+01_JPRB, 0.12454E+01_JPRB, 0.11242E+01_JPRB, 0.11606E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.12986E+01_JPRB, 0.13442E+01_JPRB, 0.13266E+01_JPRB, 0.13781E+01_JPRB, 0.14748E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.14314E+01_JPRB, 0.13673E+01_JPRB, 0.13731E+01_JPRB, 0.13372E+01_JPRB, 0.12801E+01_JPRB, 0.12410E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.11466E+01_JPRB, 0.11338E+01_JPRB, 0.10879E+01_JPRB, 0.10458E+01_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.11395E+01_JPRB, 0.11172E+01_JPRB, 0.11009E+01_JPRB, 0.10963E+01_JPRB, 0.10980E+01_JPRB, 0.11076E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.10514E+01_JPRB, 0.10124E+01_JPRB, 0.10054E+01_JPRB, 0.98792E+00_JPRB, 0.96716E+00_JPRB, 0.96966E+00_JPRB ,&
&  0.99192E+00_JPRB, 0.98759E+00_JPRB, 0.88830E+00_JPRB, 0.83982E+00_JPRB, 0.80125E+00_JPRB, 0.81748E+00_JPRB, 0.83717E+00_JPRB ,&
&  0.87524E+00_JPRB, 0.90229E+00_JPRB, 0.92075E+00_JPRB, 0.94824E+00_JPRB, 0.10024E+01_JPRB, 0.10710E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.11832E+01_JPRB, 0.12239E+01_JPRB, 0.12109E+01_JPRB, 0.12719E+01_JPRB, 0.12658E+01_JPRB, 0.12016E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.11879E+01_JPRB, 0.11598E+01_JPRB, 0.11306E+01_JPRB, 0.11230E+01_JPRB, 0.11189E+01_JPRB, 0.11011E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.10566E+01_JPRB, 0.10480E+01_JPRB, 0.10378E+01_JPRB, 0.10723E+01_JPRB, 0.11712E+01_JPRB, 0.12450E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.12671E+01_JPRB, 0.11708E+01_JPRB, 0.15374E+01_JPRB, 0.20092E+01_JPRB, 0.21681E+01_JPRB, 0.21496E+01_JPRB ,&
&  0.23217E+01_JPRB, 0.23630E+01_JPRB, 0.24011E+01_JPRB, 0.24629E+01_JPRB, 0.24678E+01_JPRB, 0.24518E+01_JPRB, 0.23757E+01_JPRB ,&
&  0.21929E+01_JPRB, 0.20692E+01_JPRB, 0.20036E+01_JPRB, 0.20297E+01_JPRB, 0.19546E+01_JPRB, 0.19121E+01_JPRB, 0.18901E+01_JPRB ,&
&  0.17184E+01_JPRB, 0.15763E+01_JPRB, 0.12524E+01_JPRB, 0.12748E+01_JPRB, 0.10944E+01_JPRB, 0.11047E+01_JPRB, 0.12305E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.13107E+01_JPRB, 0.13442E+01_JPRB, 0.13357E+01_JPRB, 0.15496E+01_JPRB, 0.16680E+01_JPRB, 0.16856E+01_JPRB ,&
&  0.16490E+01_JPRB, 0.14933E+01_JPRB, 0.15257E+01_JPRB, 0.15493E+01_JPRB, 0.15246E+01_JPRB, 0.14931E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.13919E+01_JPRB, 0.13291E+01_JPRB, 0.12866E+01_JPRB, 0.12265E+01_JPRB, 0.11737E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.12669E+01_JPRB, 0.12241E+01_JPRB, 0.11959E+01_JPRB, 0.11830E+01_JPRB, 0.11778E+01_JPRB, 0.11723E+01_JPRB, 0.11585E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.10722E+01_JPRB, 0.10444E+01_JPRB, 0.10329E+01_JPRB, 0.10069E+01_JPRB, 0.98791E+00_JPRB, 0.99958E+00_JPRB ,&
&  0.10274E+01_JPRB, 0.99055E+00_JPRB, 0.87404E+00_JPRB, 0.80955E+00_JPRB, 0.77285E+00_JPRB, 0.78005E+00_JPRB, 0.80831E+00_JPRB ,&
&  0.84704E+00_JPRB, 0.87818E+00_JPRB, 0.91676E+00_JPRB, 0.96922E+00_JPRB, 0.10493E+01_JPRB, 0.11327E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.14481E+01_JPRB, 0.13492E+01_JPRB, 0.13796E+01_JPRB, 0.13607E+01_JPRB, 0.13725E+01_JPRB, 0.13289E+01_JPRB, 0.12774E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.11835E+01_JPRB, 0.11464E+01_JPRB, 0.11098E+01_JPRB, 0.10784E+01_JPRB, 0.10590E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10904E+01_JPRB, 0.10786E+01_JPRB, 0.10930E+01_JPRB, 0.11835E+01_JPRB, 0.12249E+01_JPRB, 0.12473E+01_JPRB ,&
&  0.12255E+01_JPRB, 0.12899E+01_JPRB, 0.16191E+01_JPRB, 0.18471E+01_JPRB, 0.20134E+01_JPRB, 0.21684E+01_JPRB, 0.22237E+01_JPRB ,&
&  0.23333E+01_JPRB, 0.22704E+01_JPRB, 0.23058E+01_JPRB, 0.21161E+01_JPRB, 0.23693E+01_JPRB, 0.23731E+01_JPRB, 0.23255E+01_JPRB ,&
&  0.22209E+01_JPRB, 0.21893E+01_JPRB, 0.21081E+01_JPRB, 0.21559E+01_JPRB, 0.21476E+01_JPRB, 0.21750E+01_JPRB, 0.17852E+01_JPRB ,&
&  0.15291E+01_JPRB, 0.16446E+01_JPRB, 0.15107E+01_JPRB, 0.13618E+01_JPRB, 0.12240E+01_JPRB, 0.12404E+01_JPRB, 0.12631E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.13054E+01_JPRB, 0.12986E+01_JPRB, 0.13725E+01_JPRB, 0.16396E+01_JPRB, 0.19017E+01_JPRB, 0.19368E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.18354E+01_JPRB, 0.18631E+01_JPRB, 0.18586E+01_JPRB, 0.18824E+01_JPRB, 0.18428E+01_JPRB, 0.17687E+01_JPRB ,&
&  0.16784E+01_JPRB, 0.15891E+01_JPRB, 0.15057E+01_JPRB, 0.14175E+01_JPRB, 0.13285E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.14019E+01_JPRB, 0.13547E+01_JPRB, 0.13126E+01_JPRB, 0.12826E+01_JPRB, 0.12553E+01_JPRB, 0.12355E+01_JPRB, 0.12097E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.10939E+01_JPRB, 0.10824E+01_JPRB, 0.10647E+01_JPRB, 0.10409E+01_JPRB, 0.10260E+01_JPRB, 0.10405E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.99084E+00_JPRB, 0.87302E+00_JPRB, 0.86013E+00_JPRB, 0.82881E+00_JPRB, 0.78564E+00_JPRB, 0.79432E+00_JPRB ,&
&  0.82552E+00_JPRB, 0.86268E+00_JPRB, 0.90482E+00_JPRB, 0.99966E+00_JPRB, 0.11209E+01_JPRB, 0.12624E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.15084E+01_JPRB, 0.15649E+01_JPRB, 0.14947E+01_JPRB, 0.14090E+01_JPRB, 0.13231E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.12062E+01_JPRB, 0.11613E+01_JPRB, 0.11191E+01_JPRB, 0.10891E+01_JPRB, 0.10788E+01_JPRB, 0.10732E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10846E+01_JPRB, 0.10796E+01_JPRB, 0.10634E+01_JPRB, 0.11483E+01_JPRB, 0.10922E+01_JPRB, 0.10850E+01_JPRB, 0.11085E+01_JPRB ,&
&  0.13817E+01_JPRB, 0.15988E+01_JPRB, 0.17411E+01_JPRB, 0.20675E+01_JPRB, 0.18148E+01_JPRB, 0.19569E+01_JPRB, 0.23412E+01_JPRB ,&
&  0.24227E+01_JPRB, 0.24128E+01_JPRB, 0.25458E+01_JPRB, 0.21329E+01_JPRB, 0.19504E+01_JPRB, 0.25563E+01_JPRB, 0.25728E+01_JPRB ,&
&  0.24688E+01_JPRB, 0.24270E+01_JPRB, 0.23909E+01_JPRB, 0.26215E+01_JPRB, 0.19042E+01_JPRB, 0.16645E+01_JPRB, 0.13814E+01_JPRB ,&
&  0.13899E+01_JPRB, 0.13569E+01_JPRB, 0.15255E+01_JPRB, 0.14629E+01_JPRB, 0.14693E+01_JPRB, 0.14263E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13006E+01_JPRB, 0.13278E+01_JPRB, 0.15596E+01_JPRB, 0.20357E+01_JPRB, 0.20449E+01_JPRB, 0.22294E+01_JPRB ,&
&  0.22755E+01_JPRB, 0.24106E+01_JPRB, 0.22827E+01_JPRB, 0.23016E+01_JPRB, 0.22567E+01_JPRB, 0.21622E+01_JPRB, 0.20188E+01_JPRB ,&
&  0.19063E+01_JPRB, 0.17968E+01_JPRB, 0.16765E+01_JPRB, 0.15652E+01_JPRB, 0.14726E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.15028E+01_JPRB, 0.14382E+01_JPRB, 0.13808E+01_JPRB, 0.13304E+01_JPRB, 0.13006E+01_JPRB, 0.12732E+01_JPRB, 0.12325E+01_JPRB ,&
&  0.11608E+01_JPRB, 0.10938E+01_JPRB, 0.10921E+01_JPRB, 0.10538E+01_JPRB, 0.10329E+01_JPRB, 0.10344E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.10985E+01_JPRB, 0.95927E+00_JPRB, 0.89100E+00_JPRB, 0.89854E+00_JPRB, 0.83614E+00_JPRB, 0.80795E+00_JPRB ,&
&  0.84326E+00_JPRB, 0.88623E+00_JPRB, 0.97231E+00_JPRB, 0.11373E+01_JPRB, 0.13149E+01_JPRB, 0.14854E+01_JPRB, 0.16455E+01_JPRB ,&
&  0.16800E+01_JPRB, 0.16638E+01_JPRB, 0.15860E+01_JPRB, 0.14817E+01_JPRB, 0.13543E+01_JPRB, 0.12609E+01_JPRB, 0.11969E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.11272E+01_JPRB, 0.11153E+01_JPRB, 0.10947E+01_JPRB, 0.10917E+01_JPRB, 0.10559E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10675E+01_JPRB, 0.10892E+01_JPRB, 0.12177E+01_JPRB, 0.11857E+01_JPRB, 0.11316E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.15027E+01_JPRB, 0.17101E+01_JPRB, 0.19550E+01_JPRB, 0.23294E+01_JPRB, 0.24460E+01_JPRB, 0.26564E+01_JPRB ,&
&  0.21983E+01_JPRB, 0.19823E+01_JPRB, 0.20255E+01_JPRB, 0.19530E+01_JPRB, 0.24514E+01_JPRB, 0.33729E+01_JPRB, 0.34341E+01_JPRB ,&
&  0.32692E+01_JPRB, 0.31105E+01_JPRB, 0.30851E+01_JPRB, 0.25372E+01_JPRB, 0.18935E+01_JPRB, 0.15484E+01_JPRB, 0.17958E+01_JPRB ,&
&  0.18936E+01_JPRB, 0.20259E+01_JPRB, 0.18765E+01_JPRB, 0.16248E+01_JPRB, 0.14605E+01_JPRB, 0.17005E+01_JPRB, 0.16622E+01_JPRB ,&
&  0.15209E+01_JPRB, 0.14553E+01_JPRB, 0.18305E+01_JPRB, 0.23594E+01_JPRB, 0.26171E+01_JPRB, 0.26773E+01_JPRB, 0.30863E+01_JPRB ,&
&  0.32049E+01_JPRB, 0.32937E+01_JPRB, 0.31549E+01_JPRB, 0.29579E+01_JPRB, 0.27183E+01_JPRB, 0.24564E+01_JPRB, 0.22509E+01_JPRB ,&
&  0.20899E+01_JPRB, 0.19335E+01_JPRB, 0.17887E+01_JPRB, 0.16653E+01_JPRB, 0.15752E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.15814E+01_JPRB, 0.14971E+01_JPRB, 0.14242E+01_JPRB, 0.13773E+01_JPRB, 0.13361E+01_JPRB, 0.12881E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.11012E+01_JPRB, 0.10893E+01_JPRB, 0.10477E+01_JPRB, 0.10376E+01_JPRB, 0.10499E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.11315E+01_JPRB, 0.12250E+01_JPRB, 0.10683E+01_JPRB, 0.10034E+01_JPRB, 0.98979E+00_JPRB, 0.96355E+00_JPRB, 0.86701E+00_JPRB ,&
&  0.89966E+00_JPRB, 0.96212E+00_JPRB, 0.10953E+01_JPRB, 0.13111E+01_JPRB, 0.15760E+01_JPRB, 0.16450E+01_JPRB, 0.16882E+01_JPRB ,&
&  0.17214E+01_JPRB, 0.16345E+01_JPRB, 0.15343E+01_JPRB, 0.14325E+01_JPRB, 0.13124E+01_JPRB, 0.12445E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.11903E+01_JPRB, 0.11818E+01_JPRB, 0.11905E+01_JPRB, 0.11510E+01_JPRB, 0.11031E+01_JPRB, 0.10842E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.10841E+01_JPRB, 0.11242E+01_JPRB, 0.12671E+01_JPRB, 0.13586E+01_JPRB, 0.13125E+01_JPRB, 0.14604E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.17571E+01_JPRB, 0.18618E+01_JPRB, 0.20966E+01_JPRB, 0.23256E+01_JPRB, 0.24371E+01_JPRB, 0.26404E+01_JPRB ,&
&  0.24828E+01_JPRB, 0.26477E+01_JPRB, 0.32625E+01_JPRB, 0.39838E+01_JPRB, 0.34966E+01_JPRB, 0.35275E+01_JPRB, 0.48279E+01_JPRB ,&
&  0.35520E+01_JPRB, 0.35072E+01_JPRB, 0.36439E+01_JPRB, 0.35647E+01_JPRB, 0.16305E+01_JPRB, 0.13867E+01_JPRB, 0.19754E+01_JPRB ,&
&  0.20828E+01_JPRB, 0.14427E+01_JPRB, 0.11608E+01_JPRB, 0.13296E+01_JPRB, 0.11298E+01_JPRB, 0.13418E+01_JPRB, 0.18667E+01_JPRB ,&
&  0.19482E+01_JPRB, 0.19921E+01_JPRB, 0.27608E+01_JPRB, 0.31271E+01_JPRB, 0.30720E+01_JPRB, 0.41495E+01_JPRB, 0.42268E+01_JPRB ,&
&  0.42274E+01_JPRB, 0.38115E+01_JPRB, 0.37810E+01_JPRB, 0.34496E+01_JPRB, 0.30751E+01_JPRB, 0.27317E+01_JPRB, 0.24523E+01_JPRB ,&
&  0.21939E+01_JPRB, 0.20061E+01_JPRB, 0.18774E+01_JPRB, 0.17516E+01_JPRB, 0.16537E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.15424E+01_JPRB, 0.14602E+01_JPRB, 0.13781E+01_JPRB, 0.13260E+01_JPRB, 0.12946E+01_JPRB, 0.12516E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.11070E+01_JPRB, 0.11214E+01_JPRB, 0.10829E+01_JPRB, 0.10518E+01_JPRB, 0.10330E+01_JPRB, 0.10436E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.11888E+01_JPRB, 0.11918E+01_JPRB, 0.11322E+01_JPRB, 0.11110E+01_JPRB, 0.10671E+01_JPRB, 0.10016E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.11483E+01_JPRB, 0.13081E+01_JPRB, 0.15194E+01_JPRB, 0.16242E+01_JPRB, 0.16475E+01_JPRB, 0.17342E+01_JPRB ,&
&  0.16710E+01_JPRB, 0.15613E+01_JPRB, 0.14815E+01_JPRB, 0.13935E+01_JPRB, 0.13110E+01_JPRB, 0.12651E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.12174E+01_JPRB, 0.12108E+01_JPRB, 0.12109E+01_JPRB, 0.11673E+01_JPRB, 0.11219E+01_JPRB, 0.10914E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.11175E+01_JPRB, 0.12015E+01_JPRB, 0.13299E+01_JPRB, 0.15710E+01_JPRB, 0.16634E+01_JPRB, 0.15535E+01_JPRB ,&
&  0.14607E+01_JPRB, 0.15910E+01_JPRB, 0.20186E+01_JPRB, 0.20268E+01_JPRB, 0.19972E+01_JPRB, 0.21164E+01_JPRB, 0.21904E+01_JPRB ,&
&  0.24919E+01_JPRB, 0.31536E+01_JPRB, 0.33751E+01_JPRB, 0.42320E+01_JPRB, 0.45204E+01_JPRB, 0.33755E+01_JPRB, 0.38503E+01_JPRB ,&
&  0.38109E+01_JPRB, 0.33407E+01_JPRB, 0.26018E+01_JPRB, 0.19910E+01_JPRB, 0.22357E+01_JPRB, 0.20626E+01_JPRB, 0.13464E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.11532E+01_JPRB, 0.11089E+01_JPRB, 0.11058E+01_JPRB, 0.10933E+01_JPRB, 0.13724E+01_JPRB, 0.18294E+01_JPRB ,&
&  0.24959E+01_JPRB, 0.30189E+01_JPRB, 0.35189E+01_JPRB, 0.36388E+01_JPRB, 0.33491E+01_JPRB, 0.45396E+01_JPRB, 0.48751E+01_JPRB ,&
&  0.39968E+01_JPRB, 0.35929E+01_JPRB, 0.34853E+01_JPRB, 0.31705E+01_JPRB, 0.28627E+01_JPRB, 0.25943E+01_JPRB, 0.23397E+01_JPRB ,&
&  0.21234E+01_JPRB, 0.19686E+01_JPRB, 0.18514E+01_JPRB, 0.17397E+01_JPRB, 0.16351E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.14524E+01_JPRB, 0.13959E+01_JPRB, 0.13571E+01_JPRB, 0.13239E+01_JPRB, 0.12825E+01_JPRB, 0.12435E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11219E+01_JPRB, 0.10655E+01_JPRB, 0.10262E+01_JPRB, 0.10083E+01_JPRB, 0.10247E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.11702E+01_JPRB, 0.13173E+01_JPRB, 0.13365E+01_JPRB, 0.14580E+01_JPRB, 0.13366E+01_JPRB, 0.13440E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.14649E+01_JPRB, 0.16258E+01_JPRB, 0.17139E+01_JPRB, 0.17003E+01_JPRB, 0.17530E+01_JPRB, 0.17003E+01_JPRB ,&
&  0.15964E+01_JPRB, 0.15394E+01_JPRB, 0.14412E+01_JPRB, 0.13569E+01_JPRB, 0.13082E+01_JPRB, 0.12556E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.12150E+01_JPRB, 0.11712E+01_JPRB, 0.11550E+01_JPRB, 0.11492E+01_JPRB, 0.11313E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11436E+01_JPRB, 0.12227E+01_JPRB, 0.13259E+01_JPRB, 0.14458E+01_JPRB, 0.14614E+01_JPRB, 0.12901E+01_JPRB, 0.14473E+01_JPRB ,&
&  0.15844E+01_JPRB, 0.18488E+01_JPRB, 0.19169E+01_JPRB, 0.21449E+01_JPRB, 0.20244E+01_JPRB, 0.19753E+01_JPRB, 0.21954E+01_JPRB ,&
&  0.26781E+01_JPRB, 0.37368E+01_JPRB, 0.34213E+01_JPRB, 0.40396E+01_JPRB, 0.53669E+01_JPRB, 0.40220E+01_JPRB, 0.33857E+01_JPRB ,&
&  0.36425E+01_JPRB, 0.30833E+01_JPRB, 0.26380E+01_JPRB, 0.24361E+01_JPRB, 0.36530E+01_JPRB, 0.46252E+01_JPRB, 0.19237E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.13950E+01_JPRB, 0.13402E+01_JPRB, 0.12416E+01_JPRB, 0.12350E+01_JPRB, 0.15756E+01_JPRB, 0.24079E+01_JPRB ,&
&  0.28572E+01_JPRB, 0.38138E+01_JPRB, 0.42066E+01_JPRB, 0.41298E+01_JPRB, 0.34564E+01_JPRB, 0.39747E+01_JPRB, 0.36709E+01_JPRB ,&
&  0.38750E+01_JPRB, 0.36833E+01_JPRB, 0.33683E+01_JPRB, 0.30481E+01_JPRB, 0.27386E+01_JPRB, 0.24546E+01_JPRB, 0.22043E+01_JPRB ,&
&  0.19898E+01_JPRB, 0.18308E+01_JPRB, 0.17115E+01_JPRB, 0.16178E+01_JPRB, 0.15261E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.14459E+01_JPRB, 0.13964E+01_JPRB, 0.13511E+01_JPRB, 0.13013E+01_JPRB, 0.12697E+01_JPRB, 0.12223E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.11075E+01_JPRB, 0.10473E+01_JPRB, 0.10128E+01_JPRB, 0.10058E+01_JPRB, 0.10192E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.11648E+01_JPRB, 0.13289E+01_JPRB, 0.17030E+01_JPRB, 0.19110E+01_JPRB, 0.17842E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.18338E+01_JPRB, 0.18700E+01_JPRB, 0.19556E+01_JPRB, 0.18830E+01_JPRB, 0.18288E+01_JPRB, 0.18367E+01_JPRB, 0.16754E+01_JPRB ,&
&  0.15551E+01_JPRB, 0.15198E+01_JPRB, 0.14250E+01_JPRB, 0.13589E+01_JPRB, 0.13033E+01_JPRB, 0.12817E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.12453E+01_JPRB, 0.12149E+01_JPRB, 0.11982E+01_JPRB, 0.12051E+01_JPRB, 0.12167E+01_JPRB, 0.12157E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.13072E+01_JPRB, 0.13827E+01_JPRB, 0.14853E+01_JPRB, 0.15724E+01_JPRB, 0.13281E+01_JPRB, 0.15031E+01_JPRB, 0.15525E+01_JPRB ,&
&  0.16069E+01_JPRB, 0.16862E+01_JPRB, 0.16633E+01_JPRB, 0.18289E+01_JPRB, 0.20007E+01_JPRB, 0.19996E+01_JPRB, 0.21950E+01_JPRB ,&
&  0.30581E+01_JPRB, 0.31640E+01_JPRB, 0.33666E+01_JPRB, 0.41151E+01_JPRB, 0.46499E+01_JPRB, 0.60619E+01_JPRB, 0.31397E+01_JPRB ,&
&  0.24133E+01_JPRB, 0.29026E+01_JPRB, 0.29443E+01_JPRB, 0.31654E+01_JPRB, 0.41944E+01_JPRB, 0.66096E+01_JPRB, 0.55045E+01_JPRB ,&
&  0.19009E+01_JPRB, 0.14577E+01_JPRB, 0.16945E+01_JPRB, 0.17032E+01_JPRB, 0.15320E+01_JPRB, 0.19811E+01_JPRB, 0.42710E+01_JPRB ,&
&  0.40119E+01_JPRB, 0.46134E+01_JPRB, 0.45017E+01_JPRB, 0.38580E+01_JPRB, 0.33525E+01_JPRB, 0.35978E+01_JPRB, 0.37926E+01_JPRB ,&
&  0.36406E+01_JPRB, 0.33184E+01_JPRB, 0.30236E+01_JPRB, 0.27685E+01_JPRB, 0.24669E+01_JPRB, 0.21671E+01_JPRB, 0.19115E+01_JPRB ,&
&  0.17330E+01_JPRB, 0.16331E+01_JPRB, 0.15800E+01_JPRB, 0.15381E+01_JPRB, 0.14929E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.13790E+01_JPRB, 0.13359E+01_JPRB, 0.13017E+01_JPRB, 0.12760E+01_JPRB, 0.12665E+01_JPRB, 0.12200E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.11127E+01_JPRB, 0.10571E+01_JPRB, 0.10154E+01_JPRB, 0.10178E+01_JPRB, 0.10407E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.11710E+01_JPRB, 0.13747E+01_JPRB, 0.18503E+01_JPRB, 0.21809E+01_JPRB, 0.20815E+01_JPRB, 0.22170E+01_JPRB ,&
&  0.24205E+01_JPRB, 0.23018E+01_JPRB, 0.20548E+01_JPRB, 0.18851E+01_JPRB, 0.19103E+01_JPRB, 0.18831E+01_JPRB, 0.16951E+01_JPRB ,&
&  0.15909E+01_JPRB, 0.15504E+01_JPRB, 0.14652E+01_JPRB, 0.14022E+01_JPRB, 0.13674E+01_JPRB, 0.13454E+01_JPRB, 0.13336E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.12895E+01_JPRB, 0.12941E+01_JPRB, 0.13030E+01_JPRB, 0.13378E+01_JPRB, 0.13388E+01_JPRB, 0.14016E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.15682E+01_JPRB, 0.16369E+01_JPRB, 0.15667E+01_JPRB, 0.15745E+01_JPRB, 0.15811E+01_JPRB, 0.16404E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.16838E+01_JPRB, 0.17657E+01_JPRB, 0.18583E+01_JPRB, 0.20580E+01_JPRB, 0.22336E+01_JPRB, 0.25370E+01_JPRB ,&
&  0.31495E+01_JPRB, 0.34969E+01_JPRB, 0.35055E+01_JPRB, 0.38409E+01_JPRB, 0.43596E+01_JPRB, 0.52213E+01_JPRB, 0.40362E+01_JPRB ,&
&  0.31206E+01_JPRB, 0.27026E+01_JPRB, 0.32476E+01_JPRB, 0.45562E+01_JPRB, 0.54672E+01_JPRB, 0.73487E+01_JPRB, 0.99975E+01_JPRB ,&
&  0.10781E+02_JPRB, 0.55454E+01_JPRB, 0.43246E+01_JPRB, 0.52246E+01_JPRB, 0.38199E+01_JPRB, 0.31608E+01_JPRB, 0.40520E+01_JPRB ,&
&  0.45496E+01_JPRB, 0.47592E+01_JPRB, 0.44366E+01_JPRB, 0.35276E+01_JPRB, 0.33480E+01_JPRB, 0.36359E+01_JPRB, 0.35185E+01_JPRB ,&
&  0.31431E+01_JPRB, 0.28417E+01_JPRB, 0.25302E+01_JPRB, 0.21967E+01_JPRB, 0.19179E+01_JPRB, 0.17188E+01_JPRB, 0.16024E+01_JPRB ,&
&  0.15471E+01_JPRB, 0.15205E+01_JPRB, 0.15023E+01_JPRB, 0.14689E+01_JPRB, 0.14249E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.13077E+01_JPRB, 0.12932E+01_JPRB, 0.12985E+01_JPRB, 0.12997E+01_JPRB, 0.12803E+01_JPRB, 0.12270E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.11539E+01_JPRB, 0.10770E+01_JPRB, 0.10431E+01_JPRB, 0.10563E+01_JPRB, 0.10912E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.12742E+01_JPRB, 0.16123E+01_JPRB, 0.20534E+01_JPRB, 0.25018E+01_JPRB, 0.32781E+01_JPRB, 0.23064E+01_JPRB ,&
&  0.24195E+01_JPRB, 0.23304E+01_JPRB, 0.19724E+01_JPRB, 0.18898E+01_JPRB, 0.19395E+01_JPRB, 0.19580E+01_JPRB, 0.17822E+01_JPRB ,&
&  0.16778E+01_JPRB, 0.16283E+01_JPRB, 0.15743E+01_JPRB, 0.15191E+01_JPRB, 0.14775E+01_JPRB, 0.14815E+01_JPRB, 0.14532E+01_JPRB ,&
&  0.14672E+01_JPRB, 0.14704E+01_JPRB, 0.14444E+01_JPRB, 0.14763E+01_JPRB, 0.15126E+01_JPRB, 0.15288E+01_JPRB, 0.16123E+01_JPRB ,&
&  0.16846E+01_JPRB, 0.17204E+01_JPRB, 0.17329E+01_JPRB, 0.16517E+01_JPRB, 0.16953E+01_JPRB, 0.17310E+01_JPRB, 0.17913E+01_JPRB ,&
&  0.17150E+01_JPRB, 0.15977E+01_JPRB, 0.17969E+01_JPRB, 0.20852E+01_JPRB, 0.23330E+01_JPRB, 0.24323E+01_JPRB, 0.27728E+01_JPRB ,&
&  0.34997E+01_JPRB, 0.40855E+01_JPRB, 0.43717E+01_JPRB, 0.37127E+01_JPRB, 0.40200E+01_JPRB, 0.46714E+01_JPRB, 0.49547E+01_JPRB ,&
&  0.49974E+01_JPRB, 0.45700E+01_JPRB, 0.50934E+01_JPRB, 0.56333E+01_JPRB, 0.61034E+01_JPRB, 0.76487E+01_JPRB, 0.79186E+01_JPRB ,&
&  0.95224E+01_JPRB, 0.11847E+02_JPRB, 0.96834E+01_JPRB, 0.72721E+01_JPRB, 0.58855E+01_JPRB, 0.48713E+01_JPRB, 0.46973E+01_JPRB ,&
&  0.50526E+01_JPRB, 0.46237E+01_JPRB, 0.40189E+01_JPRB, 0.40502E+01_JPRB, 0.33921E+01_JPRB, 0.32791E+01_JPRB, 0.29188E+01_JPRB ,&
&  0.24252E+01_JPRB, 0.20610E+01_JPRB, 0.17254E+01_JPRB, 0.15182E+01_JPRB, 0.14072E+01_JPRB, 0.13805E+01_JPRB, 0.13786E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.13904E+01_JPRB, 0.13859E+01_JPRB, 0.13725E+01_JPRB, 0.13410E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.12989E+01_JPRB, 0.13021E+01_JPRB, 0.13238E+01_JPRB, 0.13306E+01_JPRB, 0.13102E+01_JPRB, 0.12913E+01_JPRB, 0.13023E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.11673E+01_JPRB, 0.11403E+01_JPRB, 0.11424E+01_JPRB, 0.11681E+01_JPRB, 0.11846E+01_JPRB, 0.12043E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.15057E+01_JPRB, 0.18733E+01_JPRB, 0.22383E+01_JPRB, 0.26064E+01_JPRB, 0.30829E+01_JPRB, 0.30172E+01_JPRB ,&
&  0.22419E+01_JPRB, 0.25013E+01_JPRB, 0.19823E+01_JPRB, 0.20486E+01_JPRB, 0.21660E+01_JPRB, 0.23596E+01_JPRB, 0.21246E+01_JPRB ,&
&  0.18996E+01_JPRB, 0.18311E+01_JPRB, 0.18862E+01_JPRB, 0.18017E+01_JPRB, 0.16572E+01_JPRB, 0.16100E+01_JPRB, 0.16530E+01_JPRB ,&
&  0.16826E+01_JPRB, 0.16731E+01_JPRB, 0.16697E+01_JPRB, 0.17159E+01_JPRB, 0.17474E+01_JPRB, 0.17527E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.17964E+01_JPRB, 0.18062E+01_JPRB, 0.17779E+01_JPRB, 0.18040E+01_JPRB, 0.18339E+01_JPRB, 0.19283E+01_JPRB, 0.20045E+01_JPRB ,&
&  0.19594E+01_JPRB, 0.18665E+01_JPRB, 0.20206E+01_JPRB, 0.22283E+01_JPRB, 0.24279E+01_JPRB, 0.27882E+01_JPRB, 0.30646E+01_JPRB ,&
&  0.37757E+01_JPRB, 0.39493E+01_JPRB, 0.47522E+01_JPRB, 0.35512E+01_JPRB, 0.37285E+01_JPRB, 0.43113E+01_JPRB, 0.43681E+01_JPRB ,&
&  0.38927E+01_JPRB, 0.44769E+01_JPRB, 0.50213E+01_JPRB, 0.58303E+01_JPRB, 0.64250E+01_JPRB, 0.70215E+01_JPRB, 0.64607E+01_JPRB ,&
&  0.66266E+01_JPRB, 0.72011E+01_JPRB, 0.67732E+01_JPRB, 0.54676E+01_JPRB, 0.49767E+01_JPRB, 0.43347E+01_JPRB, 0.47117E+01_JPRB ,&
&  0.49424E+01_JPRB, 0.45680E+01_JPRB, 0.43845E+01_JPRB, 0.41176E+01_JPRB, 0.29377E+01_JPRB, 0.25165E+01_JPRB, 0.20583E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.14176E+01_JPRB, 0.12213E+01_JPRB, 0.11891E+01_JPRB, 0.12056E+01_JPRB, 0.12315E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.13009E+01_JPRB, 0.13157E+01_JPRB, 0.13181E+01_JPRB, 0.13067E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.13178E+01_JPRB, 0.13210E+01_JPRB, 0.13476E+01_JPRB, 0.13802E+01_JPRB, 0.14045E+01_JPRB, 0.14391E+01_JPRB, 0.14042E+01_JPRB ,&
&  0.13752E+01_JPRB, 0.12255E+01_JPRB, 0.12571E+01_JPRB, 0.12776E+01_JPRB, 0.12991E+01_JPRB, 0.12934E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.15093E+01_JPRB, 0.17595E+01_JPRB, 0.21012E+01_JPRB, 0.25038E+01_JPRB, 0.29060E+01_JPRB, 0.33767E+01_JPRB, 0.34826E+01_JPRB ,&
&  0.26769E+01_JPRB, 0.23045E+01_JPRB, 0.21331E+01_JPRB, 0.22517E+01_JPRB, 0.23310E+01_JPRB, 0.25759E+01_JPRB, 0.26671E+01_JPRB ,&
&  0.25242E+01_JPRB, 0.23318E+01_JPRB, 0.24021E+01_JPRB, 0.21330E+01_JPRB, 0.17434E+01_JPRB, 0.17860E+01_JPRB, 0.18519E+01_JPRB ,&
&  0.18938E+01_JPRB, 0.19093E+01_JPRB, 0.19366E+01_JPRB, 0.19873E+01_JPRB, 0.20136E+01_JPRB, 0.19854E+01_JPRB, 0.19491E+01_JPRB ,&
&  0.19460E+01_JPRB, 0.19731E+01_JPRB, 0.19754E+01_JPRB, 0.19533E+01_JPRB, 0.20362E+01_JPRB, 0.21385E+01_JPRB, 0.22079E+01_JPRB ,&
&  0.22572E+01_JPRB, 0.23531E+01_JPRB, 0.24692E+01_JPRB, 0.26496E+01_JPRB, 0.26941E+01_JPRB, 0.31677E+01_JPRB, 0.34561E+01_JPRB ,&
&  0.38496E+01_JPRB, 0.40593E+01_JPRB, 0.45570E+01_JPRB, 0.33293E+01_JPRB, 0.38224E+01_JPRB, 0.41085E+01_JPRB, 0.42792E+01_JPRB ,&
&  0.41745E+01_JPRB, 0.46571E+01_JPRB, 0.49811E+01_JPRB, 0.53854E+01_JPRB, 0.57560E+01_JPRB, 0.53909E+01_JPRB, 0.54532E+01_JPRB ,&
&  0.50614E+01_JPRB, 0.56470E+01_JPRB, 0.53170E+01_JPRB, 0.49371E+01_JPRB, 0.42594E+01_JPRB, 0.39598E+01_JPRB, 0.41368E+01_JPRB ,&
&  0.44205E+01_JPRB, 0.40980E+01_JPRB, 0.37014E+01_JPRB, 0.30626E+01_JPRB, 0.20929E+01_JPRB, 0.17071E+01_JPRB, 0.13895E+01_JPRB ,&
&  0.12464E+01_JPRB, 0.11282E+01_JPRB, 0.10913E+01_JPRB, 0.11463E+01_JPRB, 0.11806E+01_JPRB, 0.12552E+01_JPRB, 0.12694E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.12842E+01_JPRB, 0.13155E+01_JPRB, 0.13316E+01_JPRB, 0.13290E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.12873E+01_JPRB, 0.12790E+01_JPRB, 0.12711E+01_JPRB, 0.12788E+01_JPRB, 0.13026E+01_JPRB, 0.13129E+01_JPRB, 0.12818E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.13194E+01_JPRB, 0.13811E+01_JPRB, 0.14141E+01_JPRB, 0.14286E+01_JPRB, 0.14522E+01_JPRB, 0.15489E+01_JPRB ,&
&  0.17179E+01_JPRB, 0.19783E+01_JPRB, 0.22908E+01_JPRB, 0.25278E+01_JPRB, 0.27331E+01_JPRB, 0.29697E+01_JPRB, 0.31969E+01_JPRB ,&
&  0.27231E+01_JPRB, 0.20100E+01_JPRB, 0.22728E+01_JPRB, 0.24279E+01_JPRB, 0.25245E+01_JPRB, 0.29495E+01_JPRB, 0.33979E+01_JPRB ,&
&  0.26552E+01_JPRB, 0.24187E+01_JPRB, 0.24502E+01_JPRB, 0.22702E+01_JPRB, 0.18972E+01_JPRB, 0.19648E+01_JPRB, 0.20514E+01_JPRB ,&
&  0.21128E+01_JPRB, 0.21503E+01_JPRB, 0.22198E+01_JPRB, 0.22862E+01_JPRB, 0.23055E+01_JPRB, 0.22695E+01_JPRB, 0.22201E+01_JPRB ,&
&  0.22467E+01_JPRB, 0.22980E+01_JPRB, 0.23570E+01_JPRB, 0.23598E+01_JPRB, 0.24151E+01_JPRB, 0.25240E+01_JPRB, 0.26260E+01_JPRB ,&
&  0.27221E+01_JPRB, 0.27577E+01_JPRB, 0.29377E+01_JPRB, 0.32914E+01_JPRB, 0.36149E+01_JPRB, 0.33145E+01_JPRB, 0.36838E+01_JPRB ,&
&  0.37285E+01_JPRB, 0.37577E+01_JPRB, 0.36699E+01_JPRB, 0.43421E+01_JPRB, 0.30308E+01_JPRB, 0.35847E+01_JPRB, 0.36192E+01_JPRB ,&
&  0.44495E+01_JPRB, 0.44949E+01_JPRB, 0.46897E+01_JPRB, 0.49234E+01_JPRB, 0.51443E+01_JPRB, 0.46088E+01_JPRB, 0.45041E+01_JPRB ,&
&  0.45992E+01_JPRB, 0.46455E+01_JPRB, 0.45884E+01_JPRB, 0.45529E+01_JPRB, 0.41622E+01_JPRB, 0.40331E+01_JPRB, 0.38420E+01_JPRB ,&
&  0.38266E+01_JPRB, 0.32342E+01_JPRB, 0.26685E+01_JPRB, 0.20348E+01_JPRB, 0.14206E+01_JPRB, 0.12337E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11519E+01_JPRB, 0.11806E+01_JPRB, 0.11753E+01_JPRB, 0.11839E+01_JPRB, 0.12169E+01_JPRB, 0.12738E+01_JPRB, 0.12672E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.12985E+01_JPRB, 0.13226E+01_JPRB, 0.13303E+01_JPRB, 0.13173E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.12731E+01_JPRB, 0.12583E+01_JPRB, 0.12309E+01_JPRB, 0.12420E+01_JPRB, 0.12498E+01_JPRB, 0.12541E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.13684E+01_JPRB, 0.14484E+01_JPRB, 0.15051E+01_JPRB, 0.15287E+01_JPRB, 0.15626E+01_JPRB, 0.16576E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.19966E+01_JPRB, 0.22479E+01_JPRB, 0.24255E+01_JPRB, 0.25567E+01_JPRB, 0.26041E+01_JPRB, 0.26925E+01_JPRB ,&
&  0.26296E+01_JPRB, 0.26093E+01_JPRB, 0.24744E+01_JPRB, 0.20899E+01_JPRB, 0.20902E+01_JPRB, 0.26704E+01_JPRB, 0.25492E+01_JPRB ,&
&  0.22948E+01_JPRB, 0.21828E+01_JPRB, 0.21423E+01_JPRB, 0.21116E+01_JPRB, 0.20915E+01_JPRB, 0.21649E+01_JPRB, 0.22669E+01_JPRB ,&
&  0.23313E+01_JPRB, 0.24214E+01_JPRB, 0.25071E+01_JPRB, 0.25677E+01_JPRB, 0.25984E+01_JPRB, 0.26516E+01_JPRB, 0.26744E+01_JPRB ,&
&  0.27726E+01_JPRB, 0.28910E+01_JPRB, 0.30130E+01_JPRB, 0.30643E+01_JPRB, 0.30321E+01_JPRB, 0.31308E+01_JPRB, 0.32233E+01_JPRB ,&
&  0.33225E+01_JPRB, 0.33376E+01_JPRB, 0.33370E+01_JPRB, 0.35211E+01_JPRB, 0.36235E+01_JPRB, 0.31730E+01_JPRB, 0.32175E+01_JPRB ,&
&  0.35179E+01_JPRB, 0.35480E+01_JPRB, 0.31881E+01_JPRB, 0.40527E+01_JPRB, 0.25064E+01_JPRB, 0.33364E+01_JPRB, 0.40951E+01_JPRB ,&
&  0.41026E+01_JPRB, 0.41475E+01_JPRB, 0.42610E+01_JPRB, 0.43555E+01_JPRB, 0.43181E+01_JPRB, 0.39310E+01_JPRB, 0.41161E+01_JPRB ,&
&  0.42708E+01_JPRB, 0.41927E+01_JPRB, 0.41309E+01_JPRB, 0.40574E+01_JPRB, 0.38971E+01_JPRB, 0.35020E+01_JPRB, 0.29327E+01_JPRB ,&
&  0.23974E+01_JPRB, 0.25510E+01_JPRB, 0.21190E+01_JPRB, 0.17618E+01_JPRB, 0.12483E+01_JPRB, 0.10766E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11755E+01_JPRB, 0.11841E+01_JPRB, 0.12157E+01_JPRB, 0.12374E+01_JPRB, 0.12593E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.12906E+01_JPRB, 0.13056E+01_JPRB, 0.13030E+01_JPRB, 0.12907E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.12075E+01_JPRB, 0.11892E+01_JPRB, 0.11673E+01_JPRB, 0.12074E+01_JPRB, 0.12036E+01_JPRB, 0.11971E+01_JPRB, 0.12110E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.12809E+01_JPRB, 0.13401E+01_JPRB, 0.14064E+01_JPRB, 0.14846E+01_JPRB, 0.15867E+01_JPRB, 0.16756E+01_JPRB ,&
&  0.17762E+01_JPRB, 0.18718E+01_JPRB, 0.20325E+01_JPRB, 0.22352E+01_JPRB, 0.24985E+01_JPRB, 0.26391E+01_JPRB, 0.26531E+01_JPRB ,&
&  0.25241E+01_JPRB, 0.25952E+01_JPRB, 0.26996E+01_JPRB, 0.28073E+01_JPRB, 0.25067E+01_JPRB, 0.28324E+01_JPRB, 0.30129E+01_JPRB ,&
&  0.29182E+01_JPRB, 0.25336E+01_JPRB, 0.22304E+01_JPRB, 0.22135E+01_JPRB, 0.22356E+01_JPRB, 0.23368E+01_JPRB, 0.24620E+01_JPRB ,&
&  0.25636E+01_JPRB, 0.26743E+01_JPRB, 0.27520E+01_JPRB, 0.28333E+01_JPRB, 0.29491E+01_JPRB, 0.31264E+01_JPRB, 0.32197E+01_JPRB ,&
&  0.33107E+01_JPRB, 0.34835E+01_JPRB, 0.35729E+01_JPRB, 0.35408E+01_JPRB, 0.36504E+01_JPRB, 0.37340E+01_JPRB, 0.38075E+01_JPRB ,&
&  0.39338E+01_JPRB, 0.37569E+01_JPRB, 0.38344E+01_JPRB, 0.39693E+01_JPRB, 0.38217E+01_JPRB, 0.34401E+01_JPRB, 0.33823E+01_JPRB ,&
&  0.35398E+01_JPRB, 0.35904E+01_JPRB, 0.26082E+01_JPRB, 0.36840E+01_JPRB, 0.45310E+01_JPRB, 0.39793E+01_JPRB, 0.39180E+01_JPRB ,&
&  0.38256E+01_JPRB, 0.38986E+01_JPRB, 0.40364E+01_JPRB, 0.41746E+01_JPRB, 0.42115E+01_JPRB, 0.41748E+01_JPRB, 0.38149E+01_JPRB ,&
&  0.41515E+01_JPRB, 0.40261E+01_JPRB, 0.37992E+01_JPRB, 0.35444E+01_JPRB, 0.32114E+01_JPRB, 0.26854E+01_JPRB, 0.21542E+01_JPRB ,&
&  0.18437E+01_JPRB, 0.21542E+01_JPRB, 0.18907E+01_JPRB, 0.17618E+01_JPRB, 0.15024E+01_JPRB, 0.10395E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.11603E+01_JPRB, 0.11772E+01_JPRB, 0.11718E+01_JPRB, 0.11859E+01_JPRB, 0.12026E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.12257E+01_JPRB, 0.12327E+01_JPRB, 0.12259E+01_JPRB, 0.12215E+01_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.11680E+01_JPRB, 0.11638E+01_JPRB, 0.11497E+01_JPRB, 0.11646E+01_JPRB, 0.11528E+01_JPRB, 0.11770E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.11380E+01_JPRB, 0.11416E+01_JPRB, 0.11590E+01_JPRB, 0.11804E+01_JPRB, 0.12258E+01_JPRB, 0.13069E+01_JPRB, 0.13994E+01_JPRB ,&
&  0.14871E+01_JPRB, 0.14948E+01_JPRB, 0.15283E+01_JPRB, 0.16564E+01_JPRB, 0.19708E+01_JPRB, 0.23095E+01_JPRB, 0.24454E+01_JPRB ,&
&  0.26097E+01_JPRB, 0.28387E+01_JPRB, 0.30352E+01_JPRB, 0.32835E+01_JPRB, 0.34082E+01_JPRB, 0.35800E+01_JPRB, 0.39519E+01_JPRB ,&
&  0.33100E+01_JPRB, 0.25689E+01_JPRB, 0.23376E+01_JPRB, 0.22687E+01_JPRB, 0.23235E+01_JPRB, 0.24380E+01_JPRB, 0.25523E+01_JPRB ,&
&  0.26604E+01_JPRB, 0.27824E+01_JPRB, 0.28936E+01_JPRB, 0.30466E+01_JPRB, 0.32608E+01_JPRB, 0.35050E+01_JPRB, 0.36619E+01_JPRB ,&
&  0.37614E+01_JPRB, 0.38923E+01_JPRB, 0.40413E+01_JPRB, 0.39216E+01_JPRB, 0.44073E+01_JPRB, 0.45836E+01_JPRB, 0.46727E+01_JPRB ,&
&  0.46224E+01_JPRB, 0.44151E+01_JPRB, 0.44741E+01_JPRB, 0.44920E+01_JPRB, 0.44737E+01_JPRB, 0.41078E+01_JPRB, 0.38503E+01_JPRB ,&
&  0.38252E+01_JPRB, 0.32844E+01_JPRB, 0.24199E+01_JPRB, 0.27326E+01_JPRB, 0.31268E+01_JPRB, 0.28353E+01_JPRB, 0.36846E+01_JPRB ,&
&  0.36194E+01_JPRB, 0.35669E+01_JPRB, 0.36377E+01_JPRB, 0.37553E+01_JPRB, 0.39521E+01_JPRB, 0.38437E+01_JPRB, 0.36606E+01_JPRB ,&
&  0.36506E+01_JPRB, 0.34704E+01_JPRB, 0.31663E+01_JPRB, 0.27748E+01_JPRB, 0.23805E+01_JPRB, 0.20370E+01_JPRB, 0.18419E+01_JPRB ,&
&  0.17656E+01_JPRB, 0.17862E+01_JPRB, 0.17404E+01_JPRB, 0.15268E+01_JPRB, 0.11859E+01_JPRB, 0.97079E+00_JPRB, 0.99713E+00_JPRB ,&
&  0.10624E+01_JPRB, 0.10786E+01_JPRB, 0.10947E+01_JPRB, 0.10913E+01_JPRB, 0.10959E+01_JPRB, 0.10978E+01_JPRB, 0.11040E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.11625E+01_JPRB, 0.11706E+01_JPRB, 0.11737E+01_JPRB, 0.11769E+01_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.11049E+01_JPRB, 0.11015E+01_JPRB, 0.11005E+01_JPRB, 0.11162E+01_JPRB, 0.11277E+01_JPRB, 0.11081E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.10705E+01_JPRB, 0.10820E+01_JPRB, 0.10887E+01_JPRB, 0.11112E+01_JPRB, 0.11385E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.11411E+01_JPRB, 0.10815E+01_JPRB, 0.10787E+01_JPRB, 0.11874E+01_JPRB, 0.13952E+01_JPRB, 0.17990E+01_JPRB ,&
&  0.22569E+01_JPRB, 0.25070E+01_JPRB, 0.26930E+01_JPRB, 0.27922E+01_JPRB, 0.28048E+01_JPRB, 0.27850E+01_JPRB, 0.37777E+01_JPRB ,&
&  0.40558E+01_JPRB, 0.25037E+01_JPRB, 0.24053E+01_JPRB, 0.22232E+01_JPRB, 0.22914E+01_JPRB, 0.23442E+01_JPRB, 0.24352E+01_JPRB ,&
&  0.25210E+01_JPRB, 0.26541E+01_JPRB, 0.28366E+01_JPRB, 0.30822E+01_JPRB, 0.33260E+01_JPRB, 0.35793E+01_JPRB, 0.37758E+01_JPRB ,&
&  0.39099E+01_JPRB, 0.40275E+01_JPRB, 0.42361E+01_JPRB, 0.43114E+01_JPRB, 0.43852E+01_JPRB, 0.46298E+01_JPRB, 0.48795E+01_JPRB ,&
&  0.50686E+01_JPRB, 0.54308E+01_JPRB, 0.48718E+01_JPRB, 0.48478E+01_JPRB, 0.50079E+01_JPRB, 0.43263E+01_JPRB, 0.41419E+01_JPRB ,&
&  0.40363E+01_JPRB, 0.31995E+01_JPRB, 0.21925E+01_JPRB, 0.23863E+01_JPRB, 0.26492E+01_JPRB, 0.28338E+01_JPRB, 0.31355E+01_JPRB ,&
&  0.30269E+01_JPRB, 0.28834E+01_JPRB, 0.29259E+01_JPRB, 0.31229E+01_JPRB, 0.31511E+01_JPRB, 0.30487E+01_JPRB, 0.28755E+01_JPRB ,&
&  0.27430E+01_JPRB, 0.25547E+01_JPRB, 0.24311E+01_JPRB, 0.22178E+01_JPRB, 0.20593E+01_JPRB, 0.18699E+01_JPRB, 0.16436E+01_JPRB ,&
&  0.17810E+01_JPRB, 0.20981E+01_JPRB, 0.26589E+01_JPRB, 0.21644E+01_JPRB, 0.12545E+01_JPRB, 0.99628E+00_JPRB, 0.94294E+00_JPRB ,&
&  0.96588E+00_JPRB, 0.98077E+00_JPRB, 0.10128E+01_JPRB, 0.10216E+01_JPRB, 0.10157E+01_JPRB, 0.10086E+01_JPRB, 0.10063E+01_JPRB ,&
&  0.10373E+01_JPRB, 0.10809E+01_JPRB, 0.11043E+01_JPRB, 0.11014E+01_JPRB, 0.10977E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.11487E+01_JPRB, 0.11566E+01_JPRB, 0.11859E+01_JPRB, 0.11970E+01_JPRB, 0.11951E+01_JPRB, 0.12008E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.11889E+01_JPRB, 0.11938E+01_JPRB, 0.12018E+01_JPRB, 0.12136E+01_JPRB, 0.12325E+01_JPRB, 0.12518E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.12595E+01_JPRB, 0.12971E+01_JPRB, 0.13216E+01_JPRB, 0.13146E+01_JPRB, 0.13080E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.14283E+01_JPRB, 0.14827E+01_JPRB, 0.15040E+01_JPRB, 0.14942E+01_JPRB, 0.15622E+01_JPRB, 0.18177E+01_JPRB, 0.27805E+01_JPRB ,&
&  0.33182E+01_JPRB, 0.25032E+01_JPRB, 0.21470E+01_JPRB, 0.19533E+01_JPRB, 0.20022E+01_JPRB, 0.21254E+01_JPRB, 0.21286E+01_JPRB ,&
&  0.21259E+01_JPRB, 0.21986E+01_JPRB, 0.22737E+01_JPRB, 0.23498E+01_JPRB, 0.24565E+01_JPRB, 0.25357E+01_JPRB, 0.25746E+01_JPRB ,&
&  0.25280E+01_JPRB, 0.25086E+01_JPRB, 0.25878E+01_JPRB, 0.27318E+01_JPRB, 0.31867E+01_JPRB, 0.38393E+01_JPRB, 0.36930E+01_JPRB ,&
&  0.37121E+01_JPRB, 0.41329E+01_JPRB, 0.44921E+01_JPRB, 0.46787E+01_JPRB, 0.44812E+01_JPRB, 0.40507E+01_JPRB, 0.36994E+01_JPRB ,&
&  0.33403E+01_JPRB, 0.28038E+01_JPRB, 0.23476E+01_JPRB, 0.26945E+01_JPRB, 0.27650E+01_JPRB, 0.28469E+01_JPRB, 0.26251E+01_JPRB ,&
&  0.24614E+01_JPRB, 0.21876E+01_JPRB, 0.21647E+01_JPRB, 0.22838E+01_JPRB, 0.23437E+01_JPRB, 0.22523E+01_JPRB, 0.21144E+01_JPRB ,&
&  0.19660E+01_JPRB, 0.19475E+01_JPRB, 0.20646E+01_JPRB, 0.21038E+01_JPRB, 0.21421E+01_JPRB, 0.17617E+01_JPRB, 0.16054E+01_JPRB ,&
&  0.17712E+01_JPRB, 0.20919E+01_JPRB, 0.17823E+01_JPRB, 0.22001E+01_JPRB, 0.11133E+01_JPRB, 0.88388E+00_JPRB, 0.87292E+00_JPRB ,&
&  0.89912E+00_JPRB, 0.93672E+00_JPRB, 0.95102E+00_JPRB, 0.10541E+01_JPRB, 0.10480E+01_JPRB, 0.10518E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10791E+01_JPRB, 0.10914E+01_JPRB, 0.11167E+01_JPRB, 0.11346E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.12183E+01_JPRB, 0.12267E+01_JPRB, 0.12409E+01_JPRB, 0.12616E+01_JPRB, 0.12743E+01_JPRB, 0.12839E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.12758E+01_JPRB, 0.12777E+01_JPRB, 0.12926E+01_JPRB, 0.13146E+01_JPRB, 0.13382E+01_JPRB, 0.13587E+01_JPRB, 0.13866E+01_JPRB ,&
&  0.13936E+01_JPRB, 0.14075E+01_JPRB, 0.14466E+01_JPRB, 0.14861E+01_JPRB, 0.15047E+01_JPRB, 0.15060E+01_JPRB, 0.15145E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.15200E+01_JPRB, 0.15672E+01_JPRB, 0.16008E+01_JPRB, 0.16487E+01_JPRB, 0.17761E+01_JPRB, 0.21700E+01_JPRB ,&
&  0.25379E+01_JPRB, 0.26730E+01_JPRB, 0.22588E+01_JPRB, 0.20338E+01_JPRB, 0.20277E+01_JPRB, 0.19344E+01_JPRB, 0.18538E+01_JPRB ,&
&  0.17707E+01_JPRB, 0.17433E+01_JPRB, 0.16810E+01_JPRB, 0.16887E+01_JPRB, 0.16682E+01_JPRB, 0.17015E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.17839E+01_JPRB, 0.18780E+01_JPRB, 0.20728E+01_JPRB, 0.22437E+01_JPRB, 0.24670E+01_JPRB, 0.26264E+01_JPRB, 0.23745E+01_JPRB ,&
&  0.23405E+01_JPRB, 0.26444E+01_JPRB, 0.31176E+01_JPRB, 0.34691E+01_JPRB, 0.32831E+01_JPRB, 0.29632E+01_JPRB, 0.27009E+01_JPRB ,&
&  0.24445E+01_JPRB, 0.23165E+01_JPRB, 0.27345E+01_JPRB, 0.27394E+01_JPRB, 0.27148E+01_JPRB, 0.25005E+01_JPRB, 0.22089E+01_JPRB ,&
&  0.18973E+01_JPRB, 0.15458E+01_JPRB, 0.15097E+01_JPRB, 0.15019E+01_JPRB, 0.15122E+01_JPRB, 0.14607E+01_JPRB, 0.14470E+01_JPRB ,&
&  0.14099E+01_JPRB, 0.14393E+01_JPRB, 0.17660E+01_JPRB, 0.21700E+01_JPRB, 0.24610E+01_JPRB, 0.30538E+01_JPRB, 0.26385E+01_JPRB ,&
&  0.18025E+01_JPRB, 0.17709E+01_JPRB, 0.10360E+01_JPRB, 0.14120E+01_JPRB, 0.88919E+00_JPRB, 0.81510E+00_JPRB, 0.88993E+00_JPRB ,&
&  0.10069E+01_JPRB, 0.11119E+01_JPRB, 0.11925E+01_JPRB, 0.11712E+01_JPRB, 0.11130E+01_JPRB, 0.11155E+01_JPRB, 0.11246E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.11639E+01_JPRB, 0.11764E+01_JPRB, 0.11893E+01_JPRB, 0.11987E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.12407E+01_JPRB, 0.12603E+01_JPRB, 0.12851E+01_JPRB, 0.13131E+01_JPRB, 0.13379E+01_JPRB, 0.13612E+01_JPRB, 0.13799E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.14168E+01_JPRB, 0.14530E+01_JPRB, 0.14929E+01_JPRB, 0.15160E+01_JPRB, 0.15404E+01_JPRB, 0.15542E+01_JPRB ,&
&  0.15945E+01_JPRB, 0.16439E+01_JPRB, 0.16764E+01_JPRB, 0.17155E+01_JPRB, 0.17409E+01_JPRB, 0.17667E+01_JPRB, 0.18013E+01_JPRB ,&
&  0.18234E+01_JPRB, 0.18823E+01_JPRB, 0.19799E+01_JPRB, 0.20552E+01_JPRB, 0.21755E+01_JPRB, 0.24794E+01_JPRB, 0.23090E+01_JPRB ,&
&  0.27809E+01_JPRB, 0.26573E+01_JPRB, 0.24304E+01_JPRB, 0.21661E+01_JPRB, 0.20844E+01_JPRB, 0.20062E+01_JPRB, 0.19392E+01_JPRB ,&
&  0.18899E+01_JPRB, 0.18010E+01_JPRB, 0.18230E+01_JPRB, 0.18123E+01_JPRB, 0.17987E+01_JPRB, 0.19167E+01_JPRB, 0.20305E+01_JPRB ,&
&  0.20880E+01_JPRB, 0.21514E+01_JPRB, 0.22435E+01_JPRB, 0.22975E+01_JPRB, 0.23877E+01_JPRB, 0.24333E+01_JPRB, 0.23147E+01_JPRB ,&
&  0.21539E+01_JPRB, 0.20886E+01_JPRB, 0.20382E+01_JPRB, 0.21146E+01_JPRB, 0.20844E+01_JPRB, 0.18402E+01_JPRB, 0.15727E+01_JPRB ,&
&  0.17958E+01_JPRB, 0.18293E+01_JPRB, 0.19891E+01_JPRB, 0.28150E+01_JPRB, 0.26276E+01_JPRB, 0.23118E+01_JPRB, 0.18911E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.12026E+01_JPRB, 0.10991E+01_JPRB, 0.11138E+01_JPRB, 0.10799E+01_JPRB, 0.10742E+01_JPRB, 0.10589E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.11358E+01_JPRB, 0.14517E+01_JPRB, 0.19198E+01_JPRB, 0.24561E+01_JPRB, 0.26553E+01_JPRB, 0.33769E+01_JPRB ,&
&  0.22005E+01_JPRB, 0.12598E+01_JPRB, 0.13458E+01_JPRB, 0.11431E+01_JPRB, 0.96237E+00_JPRB, 0.79170E+00_JPRB, 0.89860E+00_JPRB ,&
&  0.11141E+01_JPRB, 0.11872E+01_JPRB, 0.12173E+01_JPRB, 0.11813E+01_JPRB, 0.11818E+01_JPRB, 0.11961E+01_JPRB, 0.11973E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.12101E+01_JPRB, 0.12195E+01_JPRB, 0.12213E+01_JPRB, 0.12265E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.12690E+01_JPRB, 0.12971E+01_JPRB, 0.13342E+01_JPRB, 0.13774E+01_JPRB, 0.14069E+01_JPRB, 0.14455E+01_JPRB, 0.14684E+01_JPRB ,&
&  0.14950E+01_JPRB, 0.15420E+01_JPRB, 0.15891E+01_JPRB, 0.16391E+01_JPRB, 0.16912E+01_JPRB, 0.17394E+01_JPRB, 0.17888E+01_JPRB ,&
&  0.18315E+01_JPRB, 0.18800E+01_JPRB, 0.19137E+01_JPRB, 0.19449E+01_JPRB, 0.19544E+01_JPRB, 0.19659E+01_JPRB, 0.19640E+01_JPRB ,&
&  0.19560E+01_JPRB, 0.19715E+01_JPRB, 0.20100E+01_JPRB, 0.20454E+01_JPRB, 0.21240E+01_JPRB, 0.24281E+01_JPRB, 0.19866E+01_JPRB ,&
&  0.26713E+01_JPRB, 0.26369E+01_JPRB, 0.25765E+01_JPRB, 0.23915E+01_JPRB, 0.23098E+01_JPRB, 0.23375E+01_JPRB, 0.24149E+01_JPRB ,&
&  0.25351E+01_JPRB, 0.26892E+01_JPRB, 0.28210E+01_JPRB, 0.26916E+01_JPRB, 0.25124E+01_JPRB, 0.25657E+01_JPRB, 0.26632E+01_JPRB ,&
&  0.27158E+01_JPRB, 0.27346E+01_JPRB, 0.27628E+01_JPRB, 0.27385E+01_JPRB, 0.27188E+01_JPRB, 0.26877E+01_JPRB, 0.26070E+01_JPRB ,&
&  0.24769E+01_JPRB, 0.22986E+01_JPRB, 0.20665E+01_JPRB, 0.22340E+01_JPRB, 0.22489E+01_JPRB, 0.18852E+01_JPRB, 0.13486E+01_JPRB ,&
&  0.12466E+01_JPRB, 0.15585E+01_JPRB, 0.19616E+01_JPRB, 0.27441E+01_JPRB, 0.24166E+01_JPRB, 0.20420E+01_JPRB, 0.17761E+01_JPRB ,&
&  0.13741E+01_JPRB, 0.10794E+01_JPRB, 0.93943E+00_JPRB, 0.94618E+00_JPRB, 0.94347E+00_JPRB, 0.95368E+00_JPRB, 0.93680E+00_JPRB ,&
&  0.92762E+00_JPRB, 0.98516E+00_JPRB, 0.11917E+01_JPRB, 0.14805E+01_JPRB, 0.18900E+01_JPRB, 0.20377E+01_JPRB, 0.32323E+01_JPRB ,&
&  0.20016E+01_JPRB, 0.15788E+01_JPRB, 0.21252E+01_JPRB, 0.83798E+00_JPRB, 0.90996E+00_JPRB, 0.97782E+00_JPRB, 0.10813E+01_JPRB ,&
&  0.10944E+01_JPRB, 0.14120E+01_JPRB, 0.16884E+01_JPRB, 0.15056E+01_JPRB, 0.13827E+01_JPRB, 0.12854E+01_JPRB, 0.12513E+01_JPRB ,&
&  0.12490E+01_JPRB, 0.12475E+01_JPRB, 0.12554E+01_JPRB, 0.12585E+01_JPRB, 0.12576E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.13306E+01_JPRB, 0.13469E+01_JPRB, 0.13539E+01_JPRB, 0.14011E+01_JPRB, 0.14150E+01_JPRB, 0.14465E+01_JPRB, 0.14613E+01_JPRB ,&
&  0.14986E+01_JPRB, 0.15356E+01_JPRB, 0.15822E+01_JPRB, 0.16470E+01_JPRB, 0.16973E+01_JPRB, 0.17435E+01_JPRB, 0.18050E+01_JPRB ,&
&  0.18666E+01_JPRB, 0.19390E+01_JPRB, 0.19658E+01_JPRB, 0.19854E+01_JPRB, 0.19915E+01_JPRB, 0.20189E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.20588E+01_JPRB, 0.20707E+01_JPRB, 0.21155E+01_JPRB, 0.21812E+01_JPRB, 0.22793E+01_JPRB, 0.25031E+01_JPRB, 0.17936E+01_JPRB ,&
&  0.25780E+01_JPRB, 0.27647E+01_JPRB, 0.26924E+01_JPRB, 0.26841E+01_JPRB, 0.25777E+01_JPRB, 0.26595E+01_JPRB, 0.28149E+01_JPRB ,&
&  0.30198E+01_JPRB, 0.32235E+01_JPRB, 0.32580E+01_JPRB, 0.31883E+01_JPRB, 0.31633E+01_JPRB, 0.32732E+01_JPRB, 0.33898E+01_JPRB ,&
&  0.34678E+01_JPRB, 0.35192E+01_JPRB, 0.35819E+01_JPRB, 0.36204E+01_JPRB, 0.36708E+01_JPRB, 0.37333E+01_JPRB, 0.37323E+01_JPRB ,&
&  0.36613E+01_JPRB, 0.35933E+01_JPRB, 0.36258E+01_JPRB, 0.38411E+01_JPRB, 0.38036E+01_JPRB, 0.32512E+01_JPRB, 0.24625E+01_JPRB ,&
&  0.15468E+01_JPRB, 0.12740E+01_JPRB, 0.18038E+01_JPRB, 0.25130E+01_JPRB, 0.21815E+01_JPRB, 0.18458E+01_JPRB, 0.15297E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.10439E+01_JPRB, 0.89006E+00_JPRB, 0.89184E+00_JPRB, 0.91798E+00_JPRB, 0.98155E+00_JPRB, 0.95481E+00_JPRB ,&
&  0.99105E+00_JPRB, 0.10446E+01_JPRB, 0.11208E+01_JPRB, 0.12996E+01_JPRB, 0.16479E+01_JPRB, 0.19266E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.21086E+01_JPRB, 0.14842E+01_JPRB, 0.13348E+01_JPRB, 0.14059E+01_JPRB, 0.11994E+01_JPRB, 0.13788E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.21499E+01_JPRB, 0.20317E+01_JPRB, 0.26120E+01_JPRB, 0.39264E+01_JPRB, 0.30931E+01_JPRB, 0.22669E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.13136E+01_JPRB, 0.13011E+01_JPRB, 0.13252E+01_JPRB, 0.13245E+01_JPRB, 0.13252E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.12411E+01_JPRB, 0.12343E+01_JPRB, 0.12483E+01_JPRB, 0.12600E+01_JPRB, 0.12627E+01_JPRB, 0.12821E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.13386E+01_JPRB, 0.13921E+01_JPRB, 0.14779E+01_JPRB, 0.15509E+01_JPRB, 0.16472E+01_JPRB, 0.17012E+01_JPRB, 0.17156E+01_JPRB ,&
&  0.17511E+01_JPRB, 0.18339E+01_JPRB, 0.19122E+01_JPRB, 0.19692E+01_JPRB, 0.19739E+01_JPRB, 0.20153E+01_JPRB, 0.21210E+01_JPRB ,&
&  0.22622E+01_JPRB, 0.23683E+01_JPRB, 0.24650E+01_JPRB, 0.25907E+01_JPRB, 0.26638E+01_JPRB, 0.26025E+01_JPRB, 0.18572E+01_JPRB ,&
&  0.24902E+01_JPRB, 0.27846E+01_JPRB, 0.29179E+01_JPRB, 0.29245E+01_JPRB, 0.29620E+01_JPRB, 0.28875E+01_JPRB, 0.30386E+01_JPRB ,&
&  0.32528E+01_JPRB, 0.34362E+01_JPRB, 0.33408E+01_JPRB, 0.31448E+01_JPRB, 0.35566E+01_JPRB, 0.36741E+01_JPRB, 0.38715E+01_JPRB ,&
&  0.40369E+01_JPRB, 0.41884E+01_JPRB, 0.43360E+01_JPRB, 0.45111E+01_JPRB, 0.46649E+01_JPRB, 0.48007E+01_JPRB, 0.49083E+01_JPRB ,&
&  0.49455E+01_JPRB, 0.49515E+01_JPRB, 0.50684E+01_JPRB, 0.48677E+01_JPRB, 0.47439E+01_JPRB, 0.49114E+01_JPRB, 0.41394E+01_JPRB ,&
&  0.23086E+01_JPRB, 0.14791E+01_JPRB, 0.19016E+01_JPRB, 0.22182E+01_JPRB, 0.18839E+01_JPRB, 0.16024E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.99432E+00_JPRB, 0.94320E+00_JPRB, 0.99634E+00_JPRB, 0.10942E+01_JPRB, 0.11912E+01_JPRB, 0.12160E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.12053E+01_JPRB, 0.12553E+01_JPRB, 0.13642E+01_JPRB, 0.15896E+01_JPRB, 0.20133E+01_JPRB, 0.25789E+01_JPRB ,&
&  0.24250E+01_JPRB, 0.15553E+01_JPRB, 0.14857E+01_JPRB, 0.14749E+01_JPRB, 0.15061E+01_JPRB, 0.16283E+01_JPRB, 0.19295E+01_JPRB ,&
&  0.24161E+01_JPRB, 0.29217E+01_JPRB, 0.27722E+01_JPRB, 0.23051E+01_JPRB, 0.28384E+01_JPRB, 0.19595E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11911E+01_JPRB, 0.12248E+01_JPRB, 0.12365E+01_JPRB, 0.12425E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.10664E+01_JPRB, 0.10522E+01_JPRB, 0.10454E+01_JPRB, 0.10401E+01_JPRB, 0.10555E+01_JPRB, 0.11000E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.12956E+01_JPRB, 0.13910E+01_JPRB, 0.14599E+01_JPRB, 0.15398E+01_JPRB, 0.15818E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.16397E+01_JPRB, 0.17006E+01_JPRB, 0.17846E+01_JPRB, 0.18688E+01_JPRB, 0.19463E+01_JPRB, 0.20634E+01_JPRB, 0.22103E+01_JPRB ,&
&  0.23920E+01_JPRB, 0.25555E+01_JPRB, 0.27340E+01_JPRB, 0.29818E+01_JPRB, 0.31298E+01_JPRB, 0.32375E+01_JPRB, 0.33779E+01_JPRB ,&
&  0.18258E+01_JPRB, 0.26090E+01_JPRB, 0.28315E+01_JPRB, 0.28957E+01_JPRB, 0.32021E+01_JPRB, 0.33268E+01_JPRB, 0.34194E+01_JPRB ,&
&  0.35923E+01_JPRB, 0.35171E+01_JPRB, 0.33966E+01_JPRB, 0.33120E+01_JPRB, 0.35867E+01_JPRB, 0.37424E+01_JPRB, 0.39173E+01_JPRB ,&
&  0.40102E+01_JPRB, 0.41018E+01_JPRB, 0.42892E+01_JPRB, 0.45026E+01_JPRB, 0.47275E+01_JPRB, 0.49523E+01_JPRB, 0.52306E+01_JPRB ,&
&  0.54434E+01_JPRB, 0.56199E+01_JPRB, 0.59324E+01_JPRB, 0.54219E+01_JPRB, 0.50630E+01_JPRB, 0.54069E+01_JPRB, 0.49321E+01_JPRB ,&
&  0.33224E+01_JPRB, 0.18340E+01_JPRB, 0.18557E+01_JPRB, 0.21185E+01_JPRB, 0.17047E+01_JPRB, 0.14390E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.10141E+01_JPRB, 0.10852E+01_JPRB, 0.11861E+01_JPRB, 0.13102E+01_JPRB, 0.14203E+01_JPRB, 0.14639E+01_JPRB ,&
&  0.13820E+01_JPRB, 0.13601E+01_JPRB, 0.13082E+01_JPRB, 0.13158E+01_JPRB, 0.13553E+01_JPRB, 0.14097E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.14485E+01_JPRB, 0.14062E+01_JPRB, 0.14087E+01_JPRB, 0.14411E+01_JPRB, 0.14433E+01_JPRB, 0.14015E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.17448E+01_JPRB, 0.19521E+01_JPRB, 0.21885E+01_JPRB, 0.20971E+01_JPRB, 0.18611E+01_JPRB, 0.13781E+01_JPRB, 0.97892E+00_JPRB ,&
&  0.10100E+01_JPRB, 0.10394E+01_JPRB, 0.10520E+01_JPRB, 0.10771E+01_JPRB, 0.10736E+01_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.95786E+00_JPRB, 0.95145E+00_JPRB, 0.94623E+00_JPRB, 0.95777E+00_JPRB, 0.97272E+00_JPRB, 0.10329E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.13139E+01_JPRB, 0.13821E+01_JPRB, 0.14364E+01_JPRB, 0.14725E+01_JPRB, 0.14801E+01_JPRB, 0.15098E+01_JPRB ,&
&  0.15495E+01_JPRB, 0.15897E+01_JPRB, 0.16627E+01_JPRB, 0.17627E+01_JPRB, 0.18672E+01_JPRB, 0.19857E+01_JPRB, 0.21281E+01_JPRB ,&
&  0.23044E+01_JPRB, 0.25379E+01_JPRB, 0.27994E+01_JPRB, 0.31511E+01_JPRB, 0.34853E+01_JPRB, 0.37732E+01_JPRB, 0.39169E+01_JPRB ,&
&  0.18174E+01_JPRB, 0.19525E+01_JPRB, 0.27591E+01_JPRB, 0.27413E+01_JPRB, 0.30440E+01_JPRB, 0.34190E+01_JPRB, 0.38282E+01_JPRB ,&
&  0.36602E+01_JPRB, 0.31593E+01_JPRB, 0.30828E+01_JPRB, 0.34224E+01_JPRB, 0.34047E+01_JPRB, 0.35459E+01_JPRB, 0.36762E+01_JPRB ,&
&  0.37208E+01_JPRB, 0.37261E+01_JPRB, 0.37931E+01_JPRB, 0.39630E+01_JPRB, 0.42072E+01_JPRB, 0.44693E+01_JPRB, 0.47823E+01_JPRB ,&
&  0.51107E+01_JPRB, 0.54806E+01_JPRB, 0.58433E+01_JPRB, 0.49712E+01_JPRB, 0.53342E+01_JPRB, 0.58117E+01_JPRB, 0.61304E+01_JPRB ,&
&  0.46069E+01_JPRB, 0.26893E+01_JPRB, 0.21901E+01_JPRB, 0.22829E+01_JPRB, 0.17785E+01_JPRB, 0.16381E+01_JPRB, 0.14130E+01_JPRB ,&
&  0.13369E+01_JPRB, 0.12352E+01_JPRB, 0.13036E+01_JPRB, 0.14090E+01_JPRB, 0.15274E+01_JPRB, 0.16260E+01_JPRB, 0.16555E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.15155E+01_JPRB, 0.14858E+01_JPRB, 0.14871E+01_JPRB, 0.15036E+01_JPRB, 0.15158E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.14838E+01_JPRB, 0.15129E+01_JPRB, 0.15274E+01_JPRB, 0.15382E+01_JPRB, 0.15412E+01_JPRB, 0.15426E+01_JPRB ,&
&  0.14612E+01_JPRB, 0.13913E+01_JPRB, 0.14440E+01_JPRB, 0.13125E+01_JPRB, 0.11476E+01_JPRB, 0.99553E+00_JPRB, 0.86180E+00_JPRB ,&
&  0.89091E+00_JPRB, 0.90732E+00_JPRB, 0.91267E+00_JPRB, 0.92861E+00_JPRB, 0.94964E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.85531E+00_JPRB, 0.86351E+00_JPRB, 0.85810E+00_JPRB, 0.88412E+00_JPRB, 0.89088E+00_JPRB, 0.97784E+00_JPRB, 0.10602E+01_JPRB ,&
&  0.11947E+01_JPRB, 0.13001E+01_JPRB, 0.13757E+01_JPRB, 0.14160E+01_JPRB, 0.14140E+01_JPRB, 0.13942E+01_JPRB, 0.13802E+01_JPRB ,&
&  0.14122E+01_JPRB, 0.14402E+01_JPRB, 0.14767E+01_JPRB, 0.15309E+01_JPRB, 0.15957E+01_JPRB, 0.16844E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.19317E+01_JPRB, 0.21518E+01_JPRB, 0.24468E+01_JPRB, 0.28681E+01_JPRB, 0.34343E+01_JPRB, 0.42019E+01_JPRB, 0.50200E+01_JPRB ,&
&  0.40711E+01_JPRB, 0.19625E+01_JPRB, 0.29324E+01_JPRB, 0.27108E+01_JPRB, 0.25197E+01_JPRB, 0.30563E+01_JPRB, 0.36346E+01_JPRB ,&
&  0.33902E+01_JPRB, 0.30071E+01_JPRB, 0.28088E+01_JPRB, 0.31555E+01_JPRB, 0.31804E+01_JPRB, 0.32991E+01_JPRB, 0.33904E+01_JPRB ,&
&  0.34591E+01_JPRB, 0.34732E+01_JPRB, 0.34866E+01_JPRB, 0.36248E+01_JPRB, 0.38580E+01_JPRB, 0.41519E+01_JPRB, 0.44650E+01_JPRB ,&
&  0.48246E+01_JPRB, 0.52354E+01_JPRB, 0.57261E+01_JPRB, 0.49537E+01_JPRB, 0.55790E+01_JPRB, 0.56089E+01_JPRB, 0.48828E+01_JPRB ,&
&  0.45016E+01_JPRB, 0.30641E+01_JPRB, 0.23754E+01_JPRB, 0.24518E+01_JPRB, 0.20580E+01_JPRB, 0.15878E+01_JPRB, 0.15618E+01_JPRB ,&
&  0.14791E+01_JPRB, 0.14714E+01_JPRB, 0.14782E+01_JPRB, 0.15952E+01_JPRB, 0.16699E+01_JPRB, 0.17261E+01_JPRB, 0.17173E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.16547E+01_JPRB, 0.16319E+01_JPRB, 0.16343E+01_JPRB, 0.16370E+01_JPRB, 0.16612E+01_JPRB, 0.16587E+01_JPRB ,&
&  0.16467E+01_JPRB, 0.16592E+01_JPRB, 0.17092E+01_JPRB, 0.17600E+01_JPRB, 0.17686E+01_JPRB, 0.16628E+01_JPRB, 0.15674E+01_JPRB ,&
&  0.14990E+01_JPRB, 0.14332E+01_JPRB, 0.12804E+01_JPRB, 0.11165E+01_JPRB, 0.10343E+01_JPRB, 0.93363E+00_JPRB, 0.85519E+00_JPRB ,&
&  0.86441E+00_JPRB, 0.86240E+00_JPRB, 0.85242E+00_JPRB, 0.85639E+00_JPRB, 0.85735E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.93615E+00_JPRB, 0.91740E+00_JPRB, 0.88100E+00_JPRB, 0.86521E+00_JPRB, 0.86625E+00_JPRB, 0.90581E+00_JPRB, 0.95207E+00_JPRB ,&
&  0.10238E+01_JPRB, 0.11283E+01_JPRB, 0.12220E+01_JPRB, 0.12805E+01_JPRB, 0.12839E+01_JPRB, 0.13022E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.12903E+01_JPRB, 0.13128E+01_JPRB, 0.13397E+01_JPRB, 0.13547E+01_JPRB, 0.13828E+01_JPRB, 0.14222E+01_JPRB, 0.14649E+01_JPRB ,&
&  0.14910E+01_JPRB, 0.15324E+01_JPRB, 0.16031E+01_JPRB, 0.18323E+01_JPRB, 0.22414E+01_JPRB, 0.28826E+01_JPRB, 0.40501E+01_JPRB ,&
&  0.64220E+01_JPRB, 0.66599E+01_JPRB, 0.31899E+01_JPRB, 0.34305E+01_JPRB, 0.29437E+01_JPRB, 0.31990E+01_JPRB, 0.30787E+01_JPRB ,&
&  0.32199E+01_JPRB, 0.28700E+01_JPRB, 0.25942E+01_JPRB, 0.28118E+01_JPRB, 0.28506E+01_JPRB, 0.29599E+01_JPRB, 0.30502E+01_JPRB ,&
&  0.31833E+01_JPRB, 0.32507E+01_JPRB, 0.32247E+01_JPRB, 0.32689E+01_JPRB, 0.34634E+01_JPRB, 0.37365E+01_JPRB, 0.40581E+01_JPRB ,&
&  0.44512E+01_JPRB, 0.48292E+01_JPRB, 0.53311E+01_JPRB, 0.49563E+01_JPRB, 0.55376E+01_JPRB, 0.51030E+01_JPRB, 0.40805E+01_JPRB ,&
&  0.34089E+01_JPRB, 0.30239E+01_JPRB, 0.27754E+01_JPRB, 0.26812E+01_JPRB, 0.23618E+01_JPRB, 0.18093E+01_JPRB, 0.17480E+01_JPRB ,&
&  0.16653E+01_JPRB, 0.16220E+01_JPRB, 0.16119E+01_JPRB, 0.16526E+01_JPRB, 0.17242E+01_JPRB, 0.17563E+01_JPRB, 0.17710E+01_JPRB ,&
&  0.17623E+01_JPRB, 0.17252E+01_JPRB, 0.17094E+01_JPRB, 0.17075E+01_JPRB, 0.17113E+01_JPRB, 0.17370E+01_JPRB, 0.17478E+01_JPRB ,&
&  0.17578E+01_JPRB, 0.17896E+01_JPRB, 0.18450E+01_JPRB, 0.18964E+01_JPRB, 0.18508E+01_JPRB, 0.17355E+01_JPRB, 0.16087E+01_JPRB ,&
&  0.15584E+01_JPRB, 0.15488E+01_JPRB, 0.14373E+01_JPRB, 0.13457E+01_JPRB, 0.12369E+01_JPRB, 0.11760E+01_JPRB, 0.11153E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.10799E+01_JPRB, 0.10369E+01_JPRB, 0.92959E+00_JPRB, 0.92991E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.10631E+01_JPRB, 0.10114E+01_JPRB, 0.10102E+01_JPRB, 0.10096E+01_JPRB, 0.10025E+01_JPRB, 0.10117E+01_JPRB, 0.95602E+00_JPRB ,&
&  0.93271E+00_JPRB, 0.97434E+00_JPRB, 0.10470E+01_JPRB, 0.10974E+01_JPRB, 0.11311E+01_JPRB, 0.11414E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.11918E+01_JPRB, 0.12228E+01_JPRB, 0.12437E+01_JPRB, 0.12731E+01_JPRB, 0.12973E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.13512E+01_JPRB, 0.13208E+01_JPRB, 0.12709E+01_JPRB, 0.12713E+01_JPRB, 0.14201E+01_JPRB, 0.16825E+01_JPRB, 0.21004E+01_JPRB ,&
&  0.36446E+01_JPRB, 0.81796E+01_JPRB, 0.37536E+01_JPRB, 0.37486E+01_JPRB, 0.36404E+01_JPRB, 0.34694E+01_JPRB, 0.32460E+01_JPRB ,&
&  0.32794E+01_JPRB, 0.27599E+01_JPRB, 0.23488E+01_JPRB, 0.24008E+01_JPRB, 0.24329E+01_JPRB, 0.24944E+01_JPRB, 0.25913E+01_JPRB ,&
&  0.27015E+01_JPRB, 0.27901E+01_JPRB, 0.28471E+01_JPRB, 0.28268E+01_JPRB, 0.29614E+01_JPRB, 0.31585E+01_JPRB, 0.34393E+01_JPRB ,&
&  0.37774E+01_JPRB, 0.42217E+01_JPRB, 0.48153E+01_JPRB, 0.47812E+01_JPRB, 0.52362E+01_JPRB, 0.49079E+01_JPRB, 0.40026E+01_JPRB ,&
&  0.34703E+01_JPRB, 0.33309E+01_JPRB, 0.30015E+01_JPRB, 0.28014E+01_JPRB, 0.25293E+01_JPRB, 0.21450E+01_JPRB, 0.20882E+01_JPRB ,&
&  0.18592E+01_JPRB, 0.17628E+01_JPRB, 0.17475E+01_JPRB, 0.17392E+01_JPRB, 0.17644E+01_JPRB, 0.17765E+01_JPRB, 0.17837E+01_JPRB ,&
&  0.17768E+01_JPRB, 0.17434E+01_JPRB, 0.17301E+01_JPRB, 0.17477E+01_JPRB, 0.17590E+01_JPRB, 0.17561E+01_JPRB, 0.17726E+01_JPRB ,&
&  0.17878E+01_JPRB, 0.18295E+01_JPRB, 0.19031E+01_JPRB, 0.19061E+01_JPRB, 0.18481E+01_JPRB, 0.17252E+01_JPRB, 0.16034E+01_JPRB ,&
&  0.15575E+01_JPRB, 0.15630E+01_JPRB, 0.15014E+01_JPRB, 0.14317E+01_JPRB, 0.14087E+01_JPRB, 0.13496E+01_JPRB, 0.13538E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.15000E+01_JPRB, 0.15511E+01_JPRB, 0.10905E+01_JPRB, 0.10873E+01_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.11487E+01_JPRB, 0.10966E+01_JPRB, 0.11013E+01_JPRB, 0.11415E+01_JPRB, 0.11499E+01_JPRB, 0.11265E+01_JPRB, 0.10675E+01_JPRB ,&
&  0.98338E+00_JPRB, 0.96307E+00_JPRB, 0.97818E+00_JPRB, 0.10059E+01_JPRB, 0.10325E+01_JPRB, 0.10576E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.10863E+01_JPRB, 0.11098E+01_JPRB, 0.11453E+01_JPRB, 0.11793E+01_JPRB, 0.12111E+01_JPRB, 0.12450E+01_JPRB ,&
&  0.12548E+01_JPRB, 0.12395E+01_JPRB, 0.12096E+01_JPRB, 0.11993E+01_JPRB, 0.12209E+01_JPRB, 0.13035E+01_JPRB, 0.14999E+01_JPRB ,&
&  0.23198E+01_JPRB, 0.68811E+01_JPRB, 0.62172E+01_JPRB, 0.47397E+01_JPRB, 0.38189E+01_JPRB, 0.33563E+01_JPRB, 0.31776E+01_JPRB ,&
&  0.35108E+01_JPRB, 0.35083E+01_JPRB, 0.29750E+01_JPRB, 0.23813E+01_JPRB, 0.21810E+01_JPRB, 0.21855E+01_JPRB, 0.22323E+01_JPRB ,&
&  0.22453E+01_JPRB, 0.22773E+01_JPRB, 0.23347E+01_JPRB, 0.23434E+01_JPRB, 0.23778E+01_JPRB, 0.25449E+01_JPRB, 0.27558E+01_JPRB ,&
&  0.29728E+01_JPRB, 0.33767E+01_JPRB, 0.41060E+01_JPRB, 0.48496E+01_JPRB, 0.49102E+01_JPRB, 0.50334E+01_JPRB, 0.47601E+01_JPRB ,&
&  0.49344E+01_JPRB, 0.43019E+01_JPRB, 0.34130E+01_JPRB, 0.27930E+01_JPRB, 0.24165E+01_JPRB, 0.24933E+01_JPRB, 0.23528E+01_JPRB ,&
&  0.20105E+01_JPRB, 0.18563E+01_JPRB, 0.17936E+01_JPRB, 0.17832E+01_JPRB, 0.17889E+01_JPRB, 0.17926E+01_JPRB, 0.17613E+01_JPRB ,&
&  0.17387E+01_JPRB, 0.17267E+01_JPRB, 0.17318E+01_JPRB, 0.17566E+01_JPRB, 0.17643E+01_JPRB, 0.17645E+01_JPRB, 0.17765E+01_JPRB ,&
&  0.17705E+01_JPRB, 0.17904E+01_JPRB, 0.18037E+01_JPRB, 0.18559E+01_JPRB, 0.18055E+01_JPRB, 0.16771E+01_JPRB, 0.15610E+01_JPRB ,&
&  0.15109E+01_JPRB, 0.15301E+01_JPRB, 0.15257E+01_JPRB, 0.14973E+01_JPRB, 0.14710E+01_JPRB, 0.14401E+01_JPRB, 0.13881E+01_JPRB ,&
&  0.13878E+01_JPRB, 0.13789E+01_JPRB, 0.12734E+01_JPRB, 0.11894E+01_JPRB, 0.11899E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.12479E+01_JPRB, 0.12144E+01_JPRB, 0.11924E+01_JPRB, 0.12041E+01_JPRB, 0.12149E+01_JPRB, 0.11968E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.11108E+01_JPRB, 0.10594E+01_JPRB, 0.10575E+01_JPRB, 0.10478E+01_JPRB, 0.10407E+01_JPRB, 0.10437E+01_JPRB, 0.10538E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10861E+01_JPRB, 0.10816E+01_JPRB, 0.10854E+01_JPRB, 0.11078E+01_JPRB, 0.11391E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11747E+01_JPRB, 0.11631E+01_JPRB, 0.11585E+01_JPRB, 0.11538E+01_JPRB, 0.11901E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.18337E+01_JPRB, 0.54609E+01_JPRB, 0.35463E+01_JPRB, 0.48756E+01_JPRB, 0.37585E+01_JPRB, 0.32763E+01_JPRB, 0.27504E+01_JPRB ,&
&  0.29994E+01_JPRB, 0.36142E+01_JPRB, 0.28026E+01_JPRB, 0.23065E+01_JPRB, 0.20639E+01_JPRB, 0.20011E+01_JPRB, 0.19878E+01_JPRB ,&
&  0.19808E+01_JPRB, 0.20023E+01_JPRB, 0.20079E+01_JPRB, 0.20507E+01_JPRB, 0.20451E+01_JPRB, 0.21260E+01_JPRB, 0.22585E+01_JPRB ,&
&  0.23710E+01_JPRB, 0.25693E+01_JPRB, 0.30699E+01_JPRB, 0.37143E+01_JPRB, 0.49430E+01_JPRB, 0.56024E+01_JPRB, 0.59139E+01_JPRB ,&
&  0.61299E+01_JPRB, 0.52383E+01_JPRB, 0.41422E+01_JPRB, 0.32131E+01_JPRB, 0.26372E+01_JPRB, 0.25563E+01_JPRB, 0.25165E+01_JPRB ,&
&  0.22273E+01_JPRB, 0.20153E+01_JPRB, 0.18953E+01_JPRB, 0.18397E+01_JPRB, 0.18425E+01_JPRB, 0.18436E+01_JPRB, 0.17831E+01_JPRB ,&
&  0.17345E+01_JPRB, 0.17276E+01_JPRB, 0.17467E+01_JPRB, 0.17707E+01_JPRB, 0.17667E+01_JPRB, 0.17401E+01_JPRB, 0.17385E+01_JPRB ,&
&  0.17418E+01_JPRB, 0.17442E+01_JPRB, 0.17373E+01_JPRB, 0.18295E+01_JPRB, 0.17728E+01_JPRB, 0.16733E+01_JPRB, 0.15689E+01_JPRB ,&
&  0.15339E+01_JPRB, 0.15230E+01_JPRB, 0.15097E+01_JPRB, 0.14798E+01_JPRB, 0.15034E+01_JPRB, 0.15363E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.14055E+01_JPRB, 0.13797E+01_JPRB, 0.13387E+01_JPRB, 0.12801E+01_JPRB, 0.12745E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.12887E+01_JPRB, 0.12654E+01_JPRB, 0.12683E+01_JPRB, 0.12675E+01_JPRB, 0.12554E+01_JPRB, 0.12248E+01_JPRB, 0.12024E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.11641E+01_JPRB, 0.11398E+01_JPRB, 0.11107E+01_JPRB, 0.10950E+01_JPRB, 0.10803E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.10974E+01_JPRB, 0.10901E+01_JPRB, 0.10888E+01_JPRB, 0.10770E+01_JPRB, 0.10854E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.11266E+01_JPRB, 0.11350E+01_JPRB, 0.11332E+01_JPRB, 0.11257E+01_JPRB, 0.11303E+01_JPRB, 0.12087E+01_JPRB ,&
&  0.16570E+01_JPRB, 0.57995E+01_JPRB, 0.31939E+01_JPRB, 0.49002E+01_JPRB, 0.36787E+01_JPRB, 0.32343E+01_JPRB, 0.26352E+01_JPRB ,&
&  0.27969E+01_JPRB, 0.32816E+01_JPRB, 0.26870E+01_JPRB, 0.21973E+01_JPRB, 0.20251E+01_JPRB, 0.19275E+01_JPRB, 0.18360E+01_JPRB ,&
&  0.18090E+01_JPRB, 0.18366E+01_JPRB, 0.18514E+01_JPRB, 0.18853E+01_JPRB, 0.19131E+01_JPRB, 0.19109E+01_JPRB, 0.19550E+01_JPRB ,&
&  0.19847E+01_JPRB, 0.20488E+01_JPRB, 0.23482E+01_JPRB, 0.34084E+01_JPRB, 0.43989E+01_JPRB, 0.59525E+01_JPRB, 0.75685E+01_JPRB ,&
&  0.72751E+01_JPRB, 0.62561E+01_JPRB, 0.44856E+01_JPRB, 0.36489E+01_JPRB, 0.30334E+01_JPRB, 0.27688E+01_JPRB, 0.26197E+01_JPRB ,&
&  0.24589E+01_JPRB, 0.22972E+01_JPRB, 0.20941E+01_JPRB, 0.19952E+01_JPRB, 0.19649E+01_JPRB, 0.19412E+01_JPRB, 0.18957E+01_JPRB ,&
&  0.18416E+01_JPRB, 0.18311E+01_JPRB, 0.18388E+01_JPRB, 0.18418E+01_JPRB, 0.18303E+01_JPRB, 0.17847E+01_JPRB, 0.17568E+01_JPRB ,&
&  0.17648E+01_JPRB, 0.17526E+01_JPRB, 0.17503E+01_JPRB, 0.18785E+01_JPRB, 0.18188E+01_JPRB, 0.16922E+01_JPRB, 0.16037E+01_JPRB ,&
&  0.15901E+01_JPRB, 0.15435E+01_JPRB, 0.15259E+01_JPRB, 0.15168E+01_JPRB, 0.15333E+01_JPRB, 0.16281E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.14557E+01_JPRB, 0.13983E+01_JPRB, 0.13589E+01_JPRB, 0.13258E+01_JPRB, 0.13070E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.13530E+01_JPRB, 0.13306E+01_JPRB, 0.13284E+01_JPRB, 0.13438E+01_JPRB, 0.13203E+01_JPRB, 0.12809E+01_JPRB, 0.12459E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.12253E+01_JPRB, 0.11972E+01_JPRB, 0.11555E+01_JPRB, 0.11385E+01_JPRB, 0.11292E+01_JPRB, 0.11309E+01_JPRB ,&
&  0.11390E+01_JPRB, 0.11280E+01_JPRB, 0.11061E+01_JPRB, 0.10931E+01_JPRB, 0.10858E+01_JPRB, 0.10756E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.10875E+01_JPRB, 0.11091E+01_JPRB, 0.11113E+01_JPRB, 0.11145E+01_JPRB, 0.11029E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.14550E+01_JPRB, 0.20894E+01_JPRB, 0.38229E+01_JPRB, 0.44871E+01_JPRB, 0.36423E+01_JPRB, 0.32387E+01_JPRB, 0.29647E+01_JPRB ,&
&  0.29266E+01_JPRB, 0.29218E+01_JPRB, 0.25492E+01_JPRB, 0.22284E+01_JPRB, 0.20488E+01_JPRB, 0.18931E+01_JPRB, 0.17579E+01_JPRB ,&
&  0.16648E+01_JPRB, 0.16390E+01_JPRB, 0.16723E+01_JPRB, 0.17358E+01_JPRB, 0.17796E+01_JPRB, 0.17901E+01_JPRB, 0.17566E+01_JPRB ,&
&  0.17263E+01_JPRB, 0.17150E+01_JPRB, 0.18356E+01_JPRB, 0.25491E+01_JPRB, 0.42078E+01_JPRB, 0.61430E+01_JPRB, 0.73920E+01_JPRB ,&
&  0.75166E+01_JPRB, 0.62581E+01_JPRB, 0.45626E+01_JPRB, 0.38537E+01_JPRB, 0.33490E+01_JPRB, 0.29629E+01_JPRB, 0.26503E+01_JPRB ,&
&  0.24674E+01_JPRB, 0.23879E+01_JPRB, 0.22863E+01_JPRB, 0.21611E+01_JPRB, 0.20486E+01_JPRB, 0.19840E+01_JPRB, 0.19380E+01_JPRB ,&
&  0.18970E+01_JPRB, 0.19058E+01_JPRB, 0.19362E+01_JPRB, 0.19054E+01_JPRB, 0.18613E+01_JPRB, 0.18053E+01_JPRB, 0.17736E+01_JPRB ,&
&  0.17844E+01_JPRB, 0.17995E+01_JPRB, 0.18045E+01_JPRB, 0.18748E+01_JPRB, 0.18297E+01_JPRB, 0.17600E+01_JPRB, 0.16774E+01_JPRB ,&
&  0.16081E+01_JPRB, 0.15483E+01_JPRB, 0.15187E+01_JPRB, 0.15605E+01_JPRB, 0.15495E+01_JPRB, 0.16518E+01_JPRB, 0.16059E+01_JPRB ,&
&  0.15183E+01_JPRB, 0.14559E+01_JPRB, 0.14178E+01_JPRB, 0.13929E+01_JPRB, 0.13796E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.14172E+01_JPRB, 0.14014E+01_JPRB, 0.14076E+01_JPRB, 0.14192E+01_JPRB, 0.14053E+01_JPRB, 0.13646E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.12509E+01_JPRB, 0.12092E+01_JPRB, 0.11640E+01_JPRB, 0.11492E+01_JPRB, 0.11584E+01_JPRB, 0.11715E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11655E+01_JPRB, 0.11386E+01_JPRB, 0.11088E+01_JPRB, 0.10968E+01_JPRB, 0.10886E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.10781E+01_JPRB, 0.10844E+01_JPRB, 0.10913E+01_JPRB, 0.10981E+01_JPRB, 0.10900E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.13007E+01_JPRB, 0.28876E+01_JPRB, 0.34822E+01_JPRB, 0.36382E+01_JPRB, 0.34476E+01_JPRB, 0.31185E+01_JPRB, 0.29616E+01_JPRB ,&
&  0.26938E+01_JPRB, 0.25921E+01_JPRB, 0.24591E+01_JPRB, 0.23004E+01_JPRB, 0.21180E+01_JPRB, 0.19347E+01_JPRB, 0.17777E+01_JPRB ,&
&  0.16617E+01_JPRB, 0.16025E+01_JPRB, 0.15835E+01_JPRB, 0.16038E+01_JPRB, 0.16385E+01_JPRB, 0.16699E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.16024E+01_JPRB, 0.15617E+01_JPRB, 0.15676E+01_JPRB, 0.19193E+01_JPRB, 0.32188E+01_JPRB, 0.40652E+01_JPRB, 0.59336E+01_JPRB ,&
&  0.65536E+01_JPRB, 0.49265E+01_JPRB, 0.39147E+01_JPRB, 0.34382E+01_JPRB, 0.30917E+01_JPRB, 0.27451E+01_JPRB, 0.24847E+01_JPRB ,&
&  0.23596E+01_JPRB, 0.22804E+01_JPRB, 0.22483E+01_JPRB, 0.21660E+01_JPRB, 0.20170E+01_JPRB, 0.18832E+01_JPRB, 0.18187E+01_JPRB ,&
&  0.17827E+01_JPRB, 0.17546E+01_JPRB, 0.17595E+01_JPRB, 0.17799E+01_JPRB, 0.17745E+01_JPRB, 0.17210E+01_JPRB, 0.16912E+01_JPRB ,&
&  0.16925E+01_JPRB, 0.17299E+01_JPRB, 0.17746E+01_JPRB, 0.17874E+01_JPRB, 0.17923E+01_JPRB, 0.18046E+01_JPRB, 0.17291E+01_JPRB ,&
&  0.16103E+01_JPRB, 0.15381E+01_JPRB, 0.15164E+01_JPRB, 0.15369E+01_JPRB, 0.15831E+01_JPRB, 0.16925E+01_JPRB, 0.16350E+01_JPRB ,&
&  0.15524E+01_JPRB, 0.14867E+01_JPRB, 0.14546E+01_JPRB, 0.14317E+01_JPRB, 0.14410E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.14432E+01_JPRB, 0.14277E+01_JPRB, 0.14322E+01_JPRB, 0.14361E+01_JPRB, 0.14237E+01_JPRB, 0.13841E+01_JPRB, 0.13481E+01_JPRB ,&
&  0.13225E+01_JPRB, 0.12692E+01_JPRB, 0.12158E+01_JPRB, 0.11752E+01_JPRB, 0.11697E+01_JPRB, 0.11780E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11769E+01_JPRB, 0.11713E+01_JPRB, 0.11493E+01_JPRB, 0.11243E+01_JPRB, 0.11087E+01_JPRB, 0.10930E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.10612E+01_JPRB, 0.10657E+01_JPRB, 0.10762E+01_JPRB, 0.10754E+01_JPRB, 0.10796E+01_JPRB, 0.10873E+01_JPRB ,&
&  0.11738E+01_JPRB, 0.18833E+01_JPRB, 0.21939E+01_JPRB, 0.27649E+01_JPRB, 0.30023E+01_JPRB, 0.28719E+01_JPRB, 0.27142E+01_JPRB ,&
&  0.25064E+01_JPRB, 0.23632E+01_JPRB, 0.22852E+01_JPRB, 0.21939E+01_JPRB, 0.20488E+01_JPRB, 0.18916E+01_JPRB, 0.17508E+01_JPRB ,&
&  0.16518E+01_JPRB, 0.15945E+01_JPRB, 0.15761E+01_JPRB, 0.15573E+01_JPRB, 0.15433E+01_JPRB, 0.15544E+01_JPRB, 0.15648E+01_JPRB ,&
&  0.15127E+01_JPRB, 0.14686E+01_JPRB, 0.14399E+01_JPRB, 0.15119E+01_JPRB, 0.20556E+01_JPRB, 0.28063E+01_JPRB, 0.38122E+01_JPRB ,&
&  0.35920E+01_JPRB, 0.31627E+01_JPRB, 0.28870E+01_JPRB, 0.26746E+01_JPRB, 0.25145E+01_JPRB, 0.23087E+01_JPRB, 0.21260E+01_JPRB ,&
&  0.19680E+01_JPRB, 0.18905E+01_JPRB, 0.18242E+01_JPRB, 0.17830E+01_JPRB, 0.17335E+01_JPRB, 0.16986E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.15518E+01_JPRB, 0.15449E+01_JPRB, 0.15751E+01_JPRB, 0.16235E+01_JPRB, 0.16395E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.16599E+01_JPRB, 0.16740E+01_JPRB, 0.17195E+01_JPRB, 0.17702E+01_JPRB, 0.17394E+01_JPRB, 0.16925E+01_JPRB, 0.16565E+01_JPRB ,&
&  0.16226E+01_JPRB, 0.15658E+01_JPRB, 0.15137E+01_JPRB, 0.15374E+01_JPRB, 0.15590E+01_JPRB, 0.16555E+01_JPRB, 0.16298E+01_JPRB ,&
&  0.15628E+01_JPRB, 0.14808E+01_JPRB, 0.14461E+01_JPRB, 0.14279E+01_JPRB, 0.14453E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.14415E+01_JPRB, 0.14474E+01_JPRB, 0.14415E+01_JPRB, 0.14341E+01_JPRB, 0.14213E+01_JPRB, 0.13884E+01_JPRB, 0.13496E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.12798E+01_JPRB, 0.12264E+01_JPRB, 0.11926E+01_JPRB, 0.11787E+01_JPRB, 0.11732E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.11575E+01_JPRB, 0.11447E+01_JPRB, 0.11375E+01_JPRB, 0.11225E+01_JPRB, 0.11083E+01_JPRB, 0.10790E+01_JPRB ,&
&  0.10573E+01_JPRB, 0.10428E+01_JPRB, 0.10390E+01_JPRB, 0.10474E+01_JPRB, 0.10560E+01_JPRB, 0.10621E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.12539E+01_JPRB, 0.14739E+01_JPRB, 0.18780E+01_JPRB, 0.21825E+01_JPRB, 0.23454E+01_JPRB, 0.22851E+01_JPRB ,&
&  0.21680E+01_JPRB, 0.20552E+01_JPRB, 0.19841E+01_JPRB, 0.18979E+01_JPRB, 0.17964E+01_JPRB, 0.16884E+01_JPRB, 0.16062E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.15270E+01_JPRB, 0.14947E+01_JPRB, 0.14702E+01_JPRB, 0.14550E+01_JPRB, 0.14497E+01_JPRB, 0.14639E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14166E+01_JPRB, 0.13726E+01_JPRB, 0.13474E+01_JPRB, 0.14372E+01_JPRB, 0.16909E+01_JPRB, 0.21424E+01_JPRB ,&
&  0.23823E+01_JPRB, 0.22418E+01_JPRB, 0.21282E+01_JPRB, 0.20269E+01_JPRB, 0.19401E+01_JPRB, 0.18787E+01_JPRB, 0.18176E+01_JPRB ,&
&  0.17704E+01_JPRB, 0.17377E+01_JPRB, 0.17173E+01_JPRB, 0.16925E+01_JPRB, 0.16630E+01_JPRB, 0.16304E+01_JPRB, 0.15892E+01_JPRB ,&
&  0.15537E+01_JPRB, 0.15327E+01_JPRB, 0.15158E+01_JPRB, 0.15280E+01_JPRB, 0.15492E+01_JPRB, 0.15680E+01_JPRB, 0.15852E+01_JPRB ,&
&  0.16075E+01_JPRB, 0.16513E+01_JPRB, 0.16821E+01_JPRB, 0.17079E+01_JPRB, 0.16860E+01_JPRB, 0.16433E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.15789E+01_JPRB, 0.15475E+01_JPRB, 0.14778E+01_JPRB, 0.14872E+01_JPRB, 0.15983E+01_JPRB, 0.15860E+01_JPRB, 0.15661E+01_JPRB ,&
&  0.15507E+01_JPRB, 0.15072E+01_JPRB, 0.14436E+01_JPRB, 0.13828E+01_JPRB, 0.13980E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.13053E+01_JPRB, 0.13376E+01_JPRB, 0.13363E+01_JPRB, 0.13334E+01_JPRB, 0.13301E+01_JPRB, 0.13160E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.12699E+01_JPRB, 0.12537E+01_JPRB, 0.12302E+01_JPRB, 0.11997E+01_JPRB, 0.11772E+01_JPRB, 0.11603E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11227E+01_JPRB, 0.11132E+01_JPRB, 0.11027E+01_JPRB, 0.10869E+01_JPRB, 0.10734E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10348E+01_JPRB, 0.10332E+01_JPRB, 0.10369E+01_JPRB, 0.10455E+01_JPRB, 0.10560E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.10703E+01_JPRB, 0.11459E+01_JPRB, 0.13559E+01_JPRB, 0.16263E+01_JPRB, 0.18555E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.18746E+01_JPRB, 0.18076E+01_JPRB, 0.17281E+01_JPRB, 0.16519E+01_JPRB, 0.15871E+01_JPRB, 0.15382E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.14288E+01_JPRB, 0.14016E+01_JPRB, 0.13908E+01_JPRB, 0.13752E+01_JPRB, 0.13571E+01_JPRB, 0.13452E+01_JPRB, 0.13480E+01_JPRB ,&
&  0.13492E+01_JPRB, 0.13275E+01_JPRB, 0.12994E+01_JPRB, 0.12834E+01_JPRB, 0.13034E+01_JPRB, 0.14146E+01_JPRB, 0.16034E+01_JPRB ,&
&  0.17781E+01_JPRB, 0.17572E+01_JPRB, 0.16960E+01_JPRB, 0.16305E+01_JPRB, 0.15934E+01_JPRB, 0.15746E+01_JPRB, 0.15628E+01_JPRB ,&
&  0.15478E+01_JPRB, 0.15368E+01_JPRB, 0.15496E+01_JPRB, 0.15561E+01_JPRB, 0.15644E+01_JPRB, 0.15625E+01_JPRB, 0.15569E+01_JPRB ,&
&  0.15400E+01_JPRB, 0.15171E+01_JPRB, 0.15054E+01_JPRB, 0.15044E+01_JPRB, 0.15173E+01_JPRB, 0.15252E+01_JPRB, 0.15204E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.15272E+01_JPRB, 0.15748E+01_JPRB, 0.16127E+01_JPRB, 0.15973E+01_JPRB, 0.15696E+01_JPRB, 0.15415E+01_JPRB ,&
&  0.15263E+01_JPRB, 0.15033E+01_JPRB, 0.14868E+01_JPRB, 0.15178E+01_JPRB, 0.15163E+01_JPRB, 0.15007E+01_JPRB, 0.14830E+01_JPRB ,&
&  0.14676E+01_JPRB, 0.14407E+01_JPRB, 0.14125E+01_JPRB, 0.13587E+01_JPRB, 0.12447E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.11860E+01_JPRB, 0.12182E+01_JPRB, 0.12356E+01_JPRB, 0.12397E+01_JPRB, 0.12480E+01_JPRB, 0.12543E+01_JPRB, 0.12541E+01_JPRB ,&
&  0.12522E+01_JPRB, 0.12463E+01_JPRB, 0.12308E+01_JPRB, 0.12056E+01_JPRB, 0.11792E+01_JPRB, 0.11500E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.11013E+01_JPRB, 0.10808E+01_JPRB, 0.10624E+01_JPRB, 0.10459E+01_JPRB, 0.10326E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.10137E+01_JPRB, 0.10176E+01_JPRB, 0.10234E+01_JPRB, 0.10325E+01_JPRB, 0.10431E+01_JPRB, 0.10570E+01_JPRB ,&
&  0.10736E+01_JPRB, 0.92028E+00_JPRB, 0.96905E+00_JPRB, 0.11072E+01_JPRB, 0.12603E+01_JPRB, 0.14274E+01_JPRB, 0.15682E+01_JPRB ,&
&  0.16020E+01_JPRB, 0.16095E+01_JPRB, 0.15677E+01_JPRB, 0.15191E+01_JPRB, 0.14872E+01_JPRB, 0.14409E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.13173E+01_JPRB, 0.13007E+01_JPRB, 0.13038E+01_JPRB, 0.13010E+01_JPRB, 0.12857E+01_JPRB, 0.12692E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.12643E+01_JPRB, 0.12658E+01_JPRB, 0.12666E+01_JPRB, 0.12883E+01_JPRB, 0.13399E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.14648E+01_JPRB, 0.14696E+01_JPRB, 0.14479E+01_JPRB, 0.14120E+01_JPRB, 0.13902E+01_JPRB, 0.13933E+01_JPRB, 0.13943E+01_JPRB ,&
&  0.13899E+01_JPRB, 0.13862E+01_JPRB, 0.13953E+01_JPRB, 0.14061E+01_JPRB, 0.14243E+01_JPRB, 0.14431E+01_JPRB, 0.14717E+01_JPRB ,&
&  0.14935E+01_JPRB, 0.14962E+01_JPRB, 0.14935E+01_JPRB, 0.14775E+01_JPRB, 0.14501E+01_JPRB, 0.14305E+01_JPRB, 0.14221E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.14258E+01_JPRB, 0.14519E+01_JPRB, 0.14790E+01_JPRB, 0.14793E+01_JPRB, 0.14741E+01_JPRB, 0.14684E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.14624E+01_JPRB, 0.14605E+01_JPRB, 0.13938E+01_JPRB, 0.14303E+01_JPRB, 0.14161E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.13092E+01_JPRB, 0.12795E+01_JPRB, 0.10903E+01_JPRB, 0.11619E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.11024E+01_JPRB, 0.11217E+01_JPRB, 0.11266E+01_JPRB, 0.11419E+01_JPRB, 0.11531E+01_JPRB, 0.11619E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.11809E+01_JPRB, 0.11773E+01_JPRB, 0.11691E+01_JPRB, 0.11522E+01_JPRB, 0.11247E+01_JPRB, 0.10952E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.10510E+01_JPRB, 0.10438E+01_JPRB, 0.10315E+01_JPRB, 0.10172E+01_JPRB, 0.10080E+01_JPRB, 0.10024E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.10116E+01_JPRB, 0.10052E+01_JPRB, 0.10025E+01_JPRB, 0.10099E+01_JPRB, 0.10245E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.84081E+00_JPRB, 0.90158E+00_JPRB, 0.97942E+00_JPRB, 0.10690E+01_JPRB, 0.11896E+01_JPRB, 0.13045E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.14149E+01_JPRB, 0.14038E+01_JPRB, 0.13769E+01_JPRB, 0.13465E+01_JPRB, 0.13193E+01_JPRB, 0.12826E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.12234E+01_JPRB, 0.12178E+01_JPRB, 0.12170E+01_JPRB, 0.12155E+01_JPRB, 0.12173E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.12076E+01_JPRB, 0.12060E+01_JPRB, 0.12070E+01_JPRB, 0.12102E+01_JPRB, 0.12121E+01_JPRB, 0.12181E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12441E+01_JPRB, 0.12528E+01_JPRB, 0.12509E+01_JPRB, 0.12364E+01_JPRB, 0.12295E+01_JPRB, 0.12341E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.12667E+01_JPRB, 0.12717E+01_JPRB, 0.12802E+01_JPRB, 0.12926E+01_JPRB, 0.13110E+01_JPRB, 0.13332E+01_JPRB, 0.13592E+01_JPRB ,&
&  0.13767E+01_JPRB, 0.13810E+01_JPRB, 0.13836E+01_JPRB, 0.13751E+01_JPRB, 0.13702E+01_JPRB, 0.13462E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.13123E+01_JPRB, 0.13122E+01_JPRB, 0.13162E+01_JPRB, 0.13316E+01_JPRB, 0.13379E+01_JPRB, 0.13332E+01_JPRB, 0.13357E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.13395E+01_JPRB, 0.13240E+01_JPRB, 0.13104E+01_JPRB, 0.12857E+01_JPRB, 0.12652E+01_JPRB, 0.12351E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.11719E+01_JPRB, 0.95726E+00_JPRB, 0.10533E+01_JPRB, 0.10753E+01_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.10686E+01_JPRB, 0.10744E+01_JPRB, 0.10777E+01_JPRB, 0.10832E+01_JPRB, 0.10856E+01_JPRB, 0.10859E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.10954E+01_JPRB, 0.10871E+01_JPRB, 0.10833E+01_JPRB, 0.10717E+01_JPRB, 0.10551E+01_JPRB, 0.10404E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10142E+01_JPRB, 0.10086E+01_JPRB, 0.10027E+01_JPRB, 0.99396E+00_JPRB, 0.98381E+00_JPRB, 0.98632E+00_JPRB ,&
&  0.99268E+00_JPRB, 0.99975E+00_JPRB, 0.10009E+01_JPRB, 0.10035E+01_JPRB, 0.10082E+01_JPRB, 0.10134E+01_JPRB, 0.10230E+01_JPRB ,&
&  0.90802E+00_JPRB, 0.78258E+00_JPRB, 0.84257E+00_JPRB, 0.90727E+00_JPRB, 0.97487E+00_JPRB, 0.10810E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.12742E+01_JPRB, 0.12818E+01_JPRB, 0.12523E+01_JPRB, 0.12217E+01_JPRB, 0.12150E+01_JPRB, 0.12159E+01_JPRB ,&
&  0.12093E+01_JPRB, 0.12003E+01_JPRB, 0.11958E+01_JPRB, 0.11944E+01_JPRB, 0.11967E+01_JPRB, 0.12022E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.11880E+01_JPRB, 0.11702E+01_JPRB, 0.11442E+01_JPRB, 0.11198E+01_JPRB, 0.11084E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.10981E+01_JPRB, 0.11069E+01_JPRB, 0.11123E+01_JPRB, 0.11224E+01_JPRB, 0.11367E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.12060E+01_JPRB, 0.12242E+01_JPRB, 0.12426E+01_JPRB, 0.12512E+01_JPRB, 0.12613E+01_JPRB, 0.12791E+01_JPRB ,&
&  0.12936E+01_JPRB, 0.12816E+01_JPRB, 0.12695E+01_JPRB, 0.12592E+01_JPRB, 0.12576E+01_JPRB, 0.12582E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.12321E+01_JPRB, 0.12284E+01_JPRB, 0.12332E+01_JPRB, 0.12296E+01_JPRB, 0.12192E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.11822E+01_JPRB, 0.11619E+01_JPRB, 0.11470E+01_JPRB, 0.11288E+01_JPRB, 0.11148E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.10914E+01_JPRB, 0.10735E+01_JPRB, 0.10755E+01_JPRB, 0.10679E+01_JPRB, 0.10642E+01_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.10230E+01_JPRB, 0.10271E+01_JPRB, 0.10316E+01_JPRB, 0.10334E+01_JPRB, 0.10312E+01_JPRB, 0.10304E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10308E+01_JPRB, 0.10239E+01_JPRB, 0.10181E+01_JPRB, 0.10149E+01_JPRB, 0.10042E+01_JPRB, 0.98815E+00_JPRB ,&
&  0.97634E+00_JPRB, 0.97358E+00_JPRB, 0.97748E+00_JPRB, 0.97906E+00_JPRB, 0.96893E+00_JPRB, 0.95861E+00_JPRB, 0.95465E+00_JPRB ,&
&  0.95524E+00_JPRB, 0.96077E+00_JPRB, 0.97027E+00_JPRB, 0.97493E+00_JPRB, 0.98238E+00_JPRB, 0.99543E+00_JPRB, 0.99445E+00_JPRB ,&
&  0.99173E+00_JPRB, 0.85365E+00_JPRB, 0.89584E+00_JPRB, 0.91347E+00_JPRB, 0.94957E+00_JPRB, 0.10083E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.11510E+01_JPRB, 0.11650E+01_JPRB, 0.11576E+01_JPRB, 0.11377E+01_JPRB, 0.11197E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.11337E+01_JPRB, 0.11476E+01_JPRB, 0.11559E+01_JPRB, 0.11580E+01_JPRB, 0.11587E+01_JPRB, 0.11618E+01_JPRB ,&
&  0.11582E+01_JPRB, 0.11392E+01_JPRB, 0.11102E+01_JPRB, 0.10734E+01_JPRB, 0.10416E+01_JPRB, 0.10225E+01_JPRB, 0.10057E+01_JPRB ,&
&  0.98634E+00_JPRB, 0.97928E+00_JPRB, 0.98502E+00_JPRB, 0.99461E+00_JPRB, 0.10096E+01_JPRB, 0.10268E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.10771E+01_JPRB, 0.11002E+01_JPRB, 0.11217E+01_JPRB, 0.11397E+01_JPRB, 0.11531E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.11638E+01_JPRB, 0.11582E+01_JPRB, 0.11447E+01_JPRB, 0.11357E+01_JPRB, 0.11339E+01_JPRB, 0.11413E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.11375E+01_JPRB, 0.11301E+01_JPRB, 0.11245E+01_JPRB, 0.11110E+01_JPRB, 0.10951E+01_JPRB, 0.10790E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10413E+01_JPRB, 0.10343E+01_JPRB, 0.10247E+01_JPRB, 0.10114E+01_JPRB, 0.10027E+01_JPRB, 0.99521E+00_JPRB ,&
&  0.98911E+00_JPRB, 0.98328E+00_JPRB, 0.98436E+00_JPRB, 0.99552E+00_JPRB, 0.10118E+01_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.96668E+00_JPRB, 0.97345E+00_JPRB, 0.98388E+00_JPRB, 0.99601E+00_JPRB, 0.10014E+01_JPRB, 0.10026E+01_JPRB, 0.10016E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.99669E+00_JPRB, 0.99145E+00_JPRB, 0.98806E+00_JPRB, 0.98697E+00_JPRB, 0.98236E+00_JPRB, 0.97144E+00_JPRB ,&
&  0.96207E+00_JPRB, 0.95757E+00_JPRB, 0.96072E+00_JPRB, 0.96088E+00_JPRB, 0.94418E+00_JPRB, 0.93812E+00_JPRB, 0.92921E+00_JPRB ,&
&  0.92716E+00_JPRB, 0.92586E+00_JPRB, 0.92736E+00_JPRB, 0.93179E+00_JPRB, 0.94659E+00_JPRB, 0.95685E+00_JPRB, 0.96167E+00_JPRB ,&
&  0.95786E+00_JPRB, 0.88958E+00_JPRB, 0.91912E+00_JPRB, 0.92855E+00_JPRB, 0.94959E+00_JPRB, 0.97347E+00_JPRB, 0.99938E+00_JPRB ,&
&  0.10349E+01_JPRB, 0.10567E+01_JPRB, 0.10688E+01_JPRB, 0.10742E+01_JPRB, 0.10765E+01_JPRB, 0.10743E+01_JPRB, 0.10651E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.10720E+01_JPRB, 0.10936E+01_JPRB, 0.11059E+01_JPRB, 0.11130E+01_JPRB, 0.11170E+01_JPRB, 0.11148E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.10910E+01_JPRB, 0.10618E+01_JPRB, 0.10254E+01_JPRB, 0.98383E+00_JPRB, 0.95269E+00_JPRB, 0.93415E+00_JPRB ,&
&  0.92423E+00_JPRB, 0.92579E+00_JPRB, 0.93154E+00_JPRB, 0.93803E+00_JPRB, 0.94607E+00_JPRB, 0.95916E+00_JPRB, 0.96734E+00_JPRB ,&
&  0.97745E+00_JPRB, 0.98618E+00_JPRB, 0.99894E+00_JPRB, 0.10191E+01_JPRB, 0.10386E+01_JPRB, 0.10489E+01_JPRB, 0.10493E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.10599E+01_JPRB, 0.10570E+01_JPRB, 0.10569E+01_JPRB, 0.10528E+01_JPRB, 0.10545E+01_JPRB, 0.10529E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10371E+01_JPRB, 0.10279E+01_JPRB, 0.10105E+01_JPRB, 0.99428E+00_JPRB, 0.98130E+00_JPRB, 0.97010E+00_JPRB ,&
&  0.96654E+00_JPRB, 0.96873E+00_JPRB, 0.97291E+00_JPRB, 0.96655E+00_JPRB, 0.95062E+00_JPRB, 0.93959E+00_JPRB, 0.92721E+00_JPRB ,&
&  0.91898E+00_JPRB, 0.92058E+00_JPRB, 0.92035E+00_JPRB, 0.92813E+00_JPRB, 0.94840E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.90940E+00_JPRB, 0.92309E+00_JPRB, 0.94042E+00_JPRB, 0.95859E+00_JPRB, 0.97086E+00_JPRB, 0.97884E+00_JPRB, 0.97802E+00_JPRB ,&
&  0.96806E+00_JPRB, 0.95816E+00_JPRB, 0.96079E+00_JPRB, 0.96127E+00_JPRB, 0.96020E+00_JPRB, 0.95783E+00_JPRB, 0.95021E+00_JPRB ,&
&  0.93933E+00_JPRB, 0.92974E+00_JPRB, 0.92509E+00_JPRB, 0.92122E+00_JPRB, 0.91036E+00_JPRB, 0.90166E+00_JPRB, 0.89214E+00_JPRB ,&
&  0.89052E+00_JPRB, 0.89167E+00_JPRB, 0.89292E+00_JPRB, 0.89130E+00_JPRB, 0.89740E+00_JPRB, 0.90579E+00_JPRB, 0.90887E+00_JPRB ,&
&  0.90196E+00_JPRB, 0.91549E+00_JPRB, 0.82866E+00_JPRB, 0.89760E+00_JPRB, 0.92074E+00_JPRB, 0.93476E+00_JPRB, 0.95271E+00_JPRB ,&
&  0.96730E+00_JPRB, 0.98233E+00_JPRB, 0.99381E+00_JPRB, 0.10027E+01_JPRB, 0.10246E+01_JPRB, 0.10392E+01_JPRB, 0.10403E+01_JPRB ,&
&  0.10416E+01_JPRB, 0.10403E+01_JPRB, 0.10424E+01_JPRB, 0.10450E+01_JPRB, 0.10526E+01_JPRB, 0.10558E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.10453E+01_JPRB, 0.10328E+01_JPRB, 0.10129E+01_JPRB, 0.98871E+00_JPRB, 0.95123E+00_JPRB, 0.91855E+00_JPRB, 0.89727E+00_JPRB ,&
&  0.88103E+00_JPRB, 0.87644E+00_JPRB, 0.87203E+00_JPRB, 0.86810E+00_JPRB, 0.86797E+00_JPRB, 0.87130E+00_JPRB, 0.87166E+00_JPRB ,&
&  0.87914E+00_JPRB, 0.88809E+00_JPRB, 0.89230E+00_JPRB, 0.90043E+00_JPRB, 0.91163E+00_JPRB, 0.92751E+00_JPRB, 0.94286E+00_JPRB ,&
&  0.95750E+00_JPRB, 0.96840E+00_JPRB, 0.97162E+00_JPRB, 0.97281E+00_JPRB, 0.97048E+00_JPRB, 0.96895E+00_JPRB, 0.96167E+00_JPRB ,&
&  0.95597E+00_JPRB, 0.95297E+00_JPRB, 0.94615E+00_JPRB, 0.93307E+00_JPRB, 0.92391E+00_JPRB, 0.91586E+00_JPRB, 0.91292E+00_JPRB ,&
&  0.91256E+00_JPRB, 0.91324E+00_JPRB, 0.91055E+00_JPRB, 0.91098E+00_JPRB, 0.90379E+00_JPRB, 0.88190E+00_JPRB, 0.86454E+00_JPRB ,&
&  0.86277E+00_JPRB, 0.86762E+00_JPRB, 0.87209E+00_JPRB, 0.88083E+00_JPRB, 0.89639E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.91946E+00_JPRB, 0.92611E+00_JPRB, 0.93651E+00_JPRB, 0.95166E+00_JPRB, 0.96774E+00_JPRB, 0.97873E+00_JPRB, 0.97972E+00_JPRB ,&
&  0.96958E+00_JPRB, 0.95671E+00_JPRB, 0.95270E+00_JPRB, 0.95235E+00_JPRB, 0.94949E+00_JPRB, 0.94793E+00_JPRB, 0.94285E+00_JPRB ,&
&  0.93053E+00_JPRB, 0.91207E+00_JPRB, 0.90287E+00_JPRB, 0.88794E+00_JPRB, 0.87382E+00_JPRB, 0.86184E+00_JPRB, 0.86061E+00_JPRB ,&
&  0.86821E+00_JPRB, 0.87743E+00_JPRB, 0.88894E+00_JPRB, 0.90157E+00_JPRB, 0.90704E+00_JPRB, 0.90368E+00_JPRB, 0.89892E+00_JPRB ,&
&  0.89480E+00_JPRB, 0.89171E+00_JPRB, 0.90086E+00_JPRB, 0.90872E+00_JPRB, 0.91366E+00_JPRB, 0.92420E+00_JPRB, 0.93419E+00_JPRB ,&
&  0.94409E+00_JPRB, 0.95743E+00_JPRB, 0.97797E+00_JPRB, 0.99368E+00_JPRB, 0.10065E+01_JPRB, 0.10191E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.10365E+01_JPRB, 0.10389E+01_JPRB, 0.10303E+01_JPRB, 0.10170E+01_JPRB, 0.10097E+01_JPRB, 0.10069E+01_JPRB ,&
&  0.99957E+00_JPRB, 0.98085E+00_JPRB, 0.96589E+00_JPRB, 0.95302E+00_JPRB, 0.93174E+00_JPRB, 0.90918E+00_JPRB, 0.88329E+00_JPRB ,&
&  0.85440E+00_JPRB, 0.83290E+00_JPRB, 0.81618E+00_JPRB, 0.80097E+00_JPRB, 0.79653E+00_JPRB, 0.78702E+00_JPRB, 0.78049E+00_JPRB ,&
&  0.78555E+00_JPRB, 0.80535E+00_JPRB, 0.82082E+00_JPRB, 0.82754E+00_JPRB, 0.83189E+00_JPRB, 0.84954E+00_JPRB, 0.87071E+00_JPRB ,&
&  0.88766E+00_JPRB, 0.89593E+00_JPRB, 0.89370E+00_JPRB, 0.89749E+00_JPRB, 0.90366E+00_JPRB, 0.90826E+00_JPRB, 0.90175E+00_JPRB ,&
&  0.89266E+00_JPRB, 0.88697E+00_JPRB, 0.88557E+00_JPRB, 0.88147E+00_JPRB, 0.88254E+00_JPRB, 0.88207E+00_JPRB, 0.87897E+00_JPRB ,&
&  0.87314E+00_JPRB, 0.86414E+00_JPRB, 0.85210E+00_JPRB, 0.84387E+00_JPRB, 0.83189E+00_JPRB, 0.81381E+00_JPRB, 0.81293E+00_JPRB ,&
&  0.82359E+00_JPRB, 0.84163E+00_JPRB, 0.85955E+00_JPRB, 0.88145E+00_JPRB, 0.90729E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.94169E+00_JPRB, 0.95009E+00_JPRB, 0.95590E+00_JPRB, 0.96047E+00_JPRB, 0.97045E+00_JPRB, 0.98301E+00_JPRB, 0.98398E+00_JPRB ,&
&  0.96992E+00_JPRB, 0.95626E+00_JPRB, 0.94657E+00_JPRB, 0.94406E+00_JPRB, 0.94597E+00_JPRB, 0.94836E+00_JPRB, 0.94219E+00_JPRB ,&
&  0.91934E+00_JPRB, 0.89483E+00_JPRB, 0.87956E+00_JPRB, 0.86453E+00_JPRB, 0.85486E+00_JPRB, 0.84832E+00_JPRB, 0.85411E+00_JPRB ,&
&  0.86306E+00_JPRB, 0.87783E+00_JPRB, 0.88951E+00_JPRB, 0.89765E+00_JPRB, 0.90028E+00_JPRB, 0.88762E+00_JPRB, 0.88779E+00_JPRB ,&
&  0.89356E+00_JPRB, 0.89469E+00_JPRB, 0.90867E+00_JPRB, 0.90726E+00_JPRB, 0.89270E+00_JPRB, 0.88051E+00_JPRB, 0.86894E+00_JPRB ,&
&  0.86613E+00_JPRB, 0.87768E+00_JPRB, 0.90374E+00_JPRB, 0.94166E+00_JPRB, 0.96514E+00_JPRB, 0.97453E+00_JPRB, 0.97035E+00_JPRB ,&
&  0.97136E+00_JPRB, 0.98263E+00_JPRB, 0.98930E+00_JPRB, 0.98140E+00_JPRB, 0.96388E+00_JPRB, 0.94982E+00_JPRB, 0.93801E+00_JPRB ,&
&  0.93033E+00_JPRB, 0.92055E+00_JPRB, 0.91349E+00_JPRB, 0.90390E+00_JPRB, 0.88105E+00_JPRB, 0.85383E+00_JPRB, 0.82977E+00_JPRB ,&
&  0.81153E+00_JPRB, 0.79343E+00_JPRB, 0.77170E+00_JPRB, 0.76388E+00_JPRB, 0.76671E+00_JPRB, 0.74346E+00_JPRB, 0.71898E+00_JPRB ,&
&  0.72317E+00_JPRB, 0.75568E+00_JPRB, 0.78170E+00_JPRB, 0.79327E+00_JPRB, 0.79577E+00_JPRB, 0.80141E+00_JPRB, 0.80617E+00_JPRB ,&
&  0.81434E+00_JPRB, 0.82463E+00_JPRB, 0.83272E+00_JPRB, 0.84504E+00_JPRB, 0.85752E+00_JPRB, 0.85600E+00_JPRB, 0.84097E+00_JPRB ,&
&  0.82496E+00_JPRB, 0.81626E+00_JPRB, 0.81316E+00_JPRB, 0.81595E+00_JPRB, 0.81970E+00_JPRB, 0.81797E+00_JPRB, 0.81225E+00_JPRB ,&
&  0.80033E+00_JPRB, 0.78191E+00_JPRB, 0.76597E+00_JPRB, 0.75614E+00_JPRB, 0.75448E+00_JPRB, 0.75997E+00_JPRB, 0.77583E+00_JPRB ,&
&  0.80717E+00_JPRB, 0.85518E+00_JPRB, 0.89211E+00_JPRB, 0.91459E+00_JPRB, 0.92287E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.96545E+00_JPRB, 0.97424E+00_JPRB, 0.98439E+00_JPRB, 0.99824E+00_JPRB, 0.10113E+01_JPRB, 0.10089E+01_JPRB, 0.99650E+00_JPRB ,&
&  0.97297E+00_JPRB, 0.96365E+00_JPRB, 0.96794E+00_JPRB, 0.97013E+00_JPRB, 0.97171E+00_JPRB, 0.96926E+00_JPRB, 0.95241E+00_JPRB ,&
&  0.91281E+00_JPRB, 0.88113E+00_JPRB, 0.86563E+00_JPRB, 0.85721E+00_JPRB, 0.85406E+00_JPRB, 0.85992E+00_JPRB, 0.86659E+00_JPRB ,&
&  0.88119E+00_JPRB, 0.88930E+00_JPRB, 0.90033E+00_JPRB, 0.90938E+00_JPRB, 0.91198E+00_JPRB, 0.91556E+00_JPRB, 0.91580E+00_JPRB ,&
&  0.90038E+00_JPRB, 0.89970E+00_JPRB, 0.89968E+00_JPRB, 0.88876E+00_JPRB, 0.86525E+00_JPRB, 0.84162E+00_JPRB, 0.81330E+00_JPRB ,&
&  0.79421E+00_JPRB, 0.78469E+00_JPRB, 0.81040E+00_JPRB, 0.87340E+00_JPRB, 0.91854E+00_JPRB, 0.92690E+00_JPRB, 0.91756E+00_JPRB ,&
&  0.91789E+00_JPRB, 0.92478E+00_JPRB, 0.92864E+00_JPRB, 0.91759E+00_JPRB, 0.89826E+00_JPRB, 0.88136E+00_JPRB, 0.86571E+00_JPRB ,&
&  0.86174E+00_JPRB, 0.86275E+00_JPRB, 0.84715E+00_JPRB, 0.81582E+00_JPRB, 0.78769E+00_JPRB, 0.76749E+00_JPRB, 0.75654E+00_JPRB ,&
&  0.74932E+00_JPRB, 0.73852E+00_JPRB, 0.73104E+00_JPRB, 0.73458E+00_JPRB, 0.71386E+00_JPRB, 0.69584E+00_JPRB, 0.69977E+00_JPRB ,&
&  0.71650E+00_JPRB, 0.72311E+00_JPRB, 0.73351E+00_JPRB, 0.73807E+00_JPRB, 0.73661E+00_JPRB, 0.73439E+00_JPRB, 0.73690E+00_JPRB ,&
&  0.74708E+00_JPRB, 0.75869E+00_JPRB, 0.76736E+00_JPRB, 0.77295E+00_JPRB, 0.77829E+00_JPRB, 0.76581E+00_JPRB, 0.74427E+00_JPRB ,&
&  0.72727E+00_JPRB, 0.72480E+00_JPRB, 0.72990E+00_JPRB, 0.72972E+00_JPRB, 0.72847E+00_JPRB, 0.72902E+00_JPRB, 0.72836E+00_JPRB ,&
&  0.72621E+00_JPRB, 0.71948E+00_JPRB, 0.71107E+00_JPRB, 0.71061E+00_JPRB, 0.72121E+00_JPRB, 0.74451E+00_JPRB, 0.78071E+00_JPRB ,&
&  0.83330E+00_JPRB, 0.87276E+00_JPRB, 0.91228E+00_JPRB, 0.93024E+00_JPRB, 0.93628E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.99556E+00_JPRB, 0.10188E+01_JPRB, 0.10383E+01_JPRB, 0.10516E+01_JPRB, 0.10539E+01_JPRB, 0.10471E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.98298E+00_JPRB, 0.99173E+00_JPRB, 0.10062E+01_JPRB, 0.10147E+01_JPRB, 0.10142E+01_JPRB, 0.98176E+00_JPRB ,&
&  0.92528E+00_JPRB, 0.90340E+00_JPRB, 0.89943E+00_JPRB, 0.90805E+00_JPRB, 0.92490E+00_JPRB, 0.93002E+00_JPRB, 0.91592E+00_JPRB ,&
&  0.90201E+00_JPRB, 0.90240E+00_JPRB, 0.90438E+00_JPRB, 0.91114E+00_JPRB, 0.90097E+00_JPRB, 0.87973E+00_JPRB, 0.86955E+00_JPRB ,&
&  0.86547E+00_JPRB, 0.85585E+00_JPRB, 0.84550E+00_JPRB, 0.79656E+00_JPRB, 0.81563E+00_JPRB, 0.80292E+00_JPRB, 0.78285E+00_JPRB ,&
&  0.76376E+00_JPRB, 0.76538E+00_JPRB, 0.80810E+00_JPRB, 0.85603E+00_JPRB, 0.86817E+00_JPRB, 0.87026E+00_JPRB, 0.86364E+00_JPRB ,&
&  0.85918E+00_JPRB, 0.85935E+00_JPRB, 0.86090E+00_JPRB, 0.85284E+00_JPRB, 0.83273E+00_JPRB, 0.80669E+00_JPRB, 0.77843E+00_JPRB ,&
&  0.76200E+00_JPRB, 0.73834E+00_JPRB, 0.69559E+00_JPRB, 0.67587E+00_JPRB, 0.67472E+00_JPRB, 0.68325E+00_JPRB, 0.68720E+00_JPRB ,&
&  0.68403E+00_JPRB, 0.67589E+00_JPRB, 0.67768E+00_JPRB, 0.67305E+00_JPRB, 0.65059E+00_JPRB, 0.63618E+00_JPRB, 0.62966E+00_JPRB ,&
&  0.61894E+00_JPRB, 0.61967E+00_JPRB, 0.62638E+00_JPRB, 0.63950E+00_JPRB, 0.65097E+00_JPRB, 0.66695E+00_JPRB, 0.67441E+00_JPRB ,&
&  0.67749E+00_JPRB, 0.67317E+00_JPRB, 0.66627E+00_JPRB, 0.65352E+00_JPRB, 0.63425E+00_JPRB, 0.62683E+00_JPRB, 0.62879E+00_JPRB ,&
&  0.63874E+00_JPRB, 0.64625E+00_JPRB, 0.64698E+00_JPRB, 0.64667E+00_JPRB, 0.63550E+00_JPRB, 0.62304E+00_JPRB, 0.61402E+00_JPRB ,&
&  0.61048E+00_JPRB, 0.61479E+00_JPRB, 0.63231E+00_JPRB, 0.65923E+00_JPRB, 0.69288E+00_JPRB, 0.73367E+00_JPRB, 0.77942E+00_JPRB ,&
&  0.81118E+00_JPRB, 0.82914E+00_JPRB, 0.85372E+00_JPRB, 0.90312E+00_JPRB, 0.95209E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.99248E+00_JPRB, 0.10209E+01_JPRB, 0.10510E+01_JPRB, 0.10682E+01_JPRB, 0.10694E+01_JPRB, 0.10664E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.99587E+00_JPRB, 0.99195E+00_JPRB, 0.99600E+00_JPRB, 0.10073E+01_JPRB, 0.10252E+01_JPRB, 0.10251E+01_JPRB ,&
&  0.99606E+00_JPRB, 0.97493E+00_JPRB, 0.96390E+00_JPRB, 0.95522E+00_JPRB, 0.93651E+00_JPRB, 0.91034E+00_JPRB, 0.89306E+00_JPRB ,&
&  0.88758E+00_JPRB, 0.88659E+00_JPRB, 0.87723E+00_JPRB, 0.86343E+00_JPRB, 0.83179E+00_JPRB, 0.79253E+00_JPRB, 0.76041E+00_JPRB ,&
&  0.74764E+00_JPRB, 0.73481E+00_JPRB, 0.69511E+00_JPRB, 0.70665E+00_JPRB, 0.71083E+00_JPRB, 0.71087E+00_JPRB, 0.70674E+00_JPRB ,&
&  0.71661E+00_JPRB, 0.74975E+00_JPRB, 0.78349E+00_JPRB, 0.79318E+00_JPRB, 0.80191E+00_JPRB, 0.80313E+00_JPRB, 0.80385E+00_JPRB ,&
&  0.80790E+00_JPRB, 0.79997E+00_JPRB, 0.77043E+00_JPRB, 0.72635E+00_JPRB, 0.68301E+00_JPRB, 0.64940E+00_JPRB, 0.62103E+00_JPRB ,&
&  0.60201E+00_JPRB, 0.59034E+00_JPRB, 0.58098E+00_JPRB, 0.57799E+00_JPRB, 0.58639E+00_JPRB, 0.60199E+00_JPRB, 0.59672E+00_JPRB ,&
&  0.57729E+00_JPRB, 0.55847E+00_JPRB, 0.53466E+00_JPRB, 0.50263E+00_JPRB, 0.46889E+00_JPRB, 0.43246E+00_JPRB, 0.41284E+00_JPRB ,&
&  0.41834E+00_JPRB, 0.45860E+00_JPRB, 0.51063E+00_JPRB, 0.53318E+00_JPRB, 0.56206E+00_JPRB, 0.59058E+00_JPRB, 0.57963E+00_JPRB ,&
&  0.52796E+00_JPRB, 0.46409E+00_JPRB, 0.44157E+00_JPRB, 0.42709E+00_JPRB, 0.42008E+00_JPRB, 0.41591E+00_JPRB, 0.41420E+00_JPRB ,&
&  0.41926E+00_JPRB, 0.43037E+00_JPRB, 0.44126E+00_JPRB, 0.43934E+00_JPRB, 0.42978E+00_JPRB, 0.42563E+00_JPRB, 0.42782E+00_JPRB ,&
&  0.44102E+00_JPRB, 0.47182E+00_JPRB, 0.53421E+00_JPRB, 0.64488E+00_JPRB, 0.68586E+00_JPRB, 0.72183E+00_JPRB, 0.76211E+00_JPRB ,&
&  0.80119E+00_JPRB, 0.83668E+00_JPRB, 0.86625E+00_JPRB, 0.90222E+00_JPRB, 0.95058E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.92650E+00_JPRB, 0.94703E+00_JPRB, 0.97006E+00_JPRB, 0.98222E+00_JPRB, 0.98383E+00_JPRB, 0.98120E+00_JPRB, 0.97268E+00_JPRB ,&
&  0.95789E+00_JPRB, 0.93999E+00_JPRB, 0.92655E+00_JPRB, 0.91697E+00_JPRB, 0.91184E+00_JPRB, 0.91334E+00_JPRB, 0.91686E+00_JPRB ,&
&  0.91404E+00_JPRB, 0.90837E+00_JPRB, 0.89474E+00_JPRB, 0.87726E+00_JPRB, 0.85888E+00_JPRB, 0.84480E+00_JPRB, 0.83428E+00_JPRB ,&
&  0.82411E+00_JPRB, 0.80931E+00_JPRB, 0.78649E+00_JPRB, 0.75092E+00_JPRB, 0.70875E+00_JPRB, 0.67015E+00_JPRB, 0.63796E+00_JPRB ,&
&  0.60706E+00_JPRB, 0.54232E+00_JPRB, 0.50685E+00_JPRB, 0.51406E+00_JPRB, 0.59758E+00_JPRB, 0.63320E+00_JPRB, 0.65937E+00_JPRB ,&
&  0.69414E+00_JPRB, 0.73156E+00_JPRB, 0.74962E+00_JPRB, 0.74950E+00_JPRB, 0.74345E+00_JPRB, 0.73039E+00_JPRB, 0.71016E+00_JPRB ,&
&  0.68466E+00_JPRB, 0.65380E+00_JPRB, 0.62729E+00_JPRB, 0.59693E+00_JPRB, 0.56630E+00_JPRB, 0.54414E+00_JPRB, 0.52890E+00_JPRB ,&
&  0.51846E+00_JPRB, 0.50827E+00_JPRB, 0.49989E+00_JPRB, 0.48966E+00_JPRB, 0.48043E+00_JPRB, 0.45905E+00_JPRB, 0.43505E+00_JPRB ,&
&  0.41414E+00_JPRB, 0.39490E+00_JPRB, 0.37628E+00_JPRB, 0.35761E+00_JPRB, 0.34239E+00_JPRB, 0.33214E+00_JPRB, 0.32423E+00_JPRB ,&
&  0.32271E+00_JPRB, 0.33144E+00_JPRB, 0.35321E+00_JPRB, 0.38593E+00_JPRB, 0.41692E+00_JPRB, 0.39829E+00_JPRB, 0.38059E+00_JPRB ,&
&  0.35735E+00_JPRB, 0.34398E+00_JPRB, 0.33886E+00_JPRB, 0.33814E+00_JPRB, 0.33920E+00_JPRB, 0.34130E+00_JPRB, 0.34391E+00_JPRB ,&
&  0.34810E+00_JPRB, 0.35227E+00_JPRB, 0.35634E+00_JPRB, 0.35968E+00_JPRB, 0.36189E+00_JPRB, 0.36517E+00_JPRB, 0.37041E+00_JPRB ,&
&  0.37937E+00_JPRB, 0.39348E+00_JPRB, 0.41462E+00_JPRB, 0.44214E+00_JPRB, 0.47173E+00_JPRB, 0.49834E+00_JPRB, 0.55179E+00_JPRB ,&
&  0.63934E+00_JPRB, 0.74904E+00_JPRB, 0.81135E+00_JPRB, 0.87201E+00_JPRB, 0.90257E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.85451E+00_JPRB, 0.86980E+00_JPRB, 0.88202E+00_JPRB, 0.88587E+00_JPRB, 0.88558E+00_JPRB, 0.88064E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.84856E+00_JPRB, 0.82212E+00_JPRB, 0.79617E+00_JPRB, 0.77557E+00_JPRB, 0.76232E+00_JPRB, 0.76388E+00_JPRB, 0.77125E+00_JPRB ,&
&  0.78701E+00_JPRB, 0.81108E+00_JPRB, 0.81504E+00_JPRB, 0.80698E+00_JPRB, 0.77041E+00_JPRB, 0.73220E+00_JPRB, 0.69779E+00_JPRB ,&
&  0.66658E+00_JPRB, 0.63603E+00_JPRB, 0.62470E+00_JPRB, 0.60507E+00_JPRB, 0.59134E+00_JPRB, 0.57408E+00_JPRB, 0.54833E+00_JPRB ,&
&  0.50638E+00_JPRB, 0.47674E+00_JPRB, 0.46378E+00_JPRB, 0.47070E+00_JPRB, 0.55165E+00_JPRB, 0.61315E+00_JPRB, 0.62721E+00_JPRB ,&
&  0.66045E+00_JPRB, 0.68306E+00_JPRB, 0.67710E+00_JPRB, 0.65611E+00_JPRB, 0.63368E+00_JPRB, 0.60991E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.56858E+00_JPRB, 0.54207E+00_JPRB, 0.49105E+00_JPRB, 0.41864E+00_JPRB, 0.38384E+00_JPRB, 0.37077E+00_JPRB, 0.35637E+00_JPRB ,&
&  0.34569E+00_JPRB, 0.33858E+00_JPRB, 0.33543E+00_JPRB, 0.33062E+00_JPRB, 0.33023E+00_JPRB, 0.32655E+00_JPRB, 0.32277E+00_JPRB ,&
&  0.31679E+00_JPRB, 0.31226E+00_JPRB, 0.30669E+00_JPRB, 0.30132E+00_JPRB, 0.29664E+00_JPRB, 0.29598E+00_JPRB, 0.29608E+00_JPRB ,&
&  0.29756E+00_JPRB, 0.30218E+00_JPRB, 0.31622E+00_JPRB, 0.32367E+00_JPRB, 0.32442E+00_JPRB, 0.32171E+00_JPRB, 0.31442E+00_JPRB ,&
&  0.30646E+00_JPRB, 0.30529E+00_JPRB, 0.30401E+00_JPRB, 0.30492E+00_JPRB, 0.30847E+00_JPRB, 0.31273E+00_JPRB, 0.31581E+00_JPRB ,&
&  0.31831E+00_JPRB, 0.32151E+00_JPRB, 0.32428E+00_JPRB, 0.32734E+00_JPRB, 0.33121E+00_JPRB, 0.33568E+00_JPRB, 0.34179E+00_JPRB ,&
&  0.34792E+00_JPRB, 0.35672E+00_JPRB, 0.36818E+00_JPRB, 0.38500E+00_JPRB, 0.40091E+00_JPRB, 0.41185E+00_JPRB, 0.42794E+00_JPRB ,&
&  0.44552E+00_JPRB, 0.49568E+00_JPRB, 0.63254E+00_JPRB, 0.85104E+00_JPRB, 0.88335E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.77123E+00_JPRB, 0.78543E+00_JPRB, 0.80074E+00_JPRB, 0.80540E+00_JPRB, 0.81603E+00_JPRB, 0.81469E+00_JPRB, 0.79674E+00_JPRB ,&
&  0.76869E+00_JPRB, 0.74134E+00_JPRB, 0.72664E+00_JPRB, 0.70507E+00_JPRB, 0.67583E+00_JPRB, 0.61704E+00_JPRB, 0.59480E+00_JPRB ,&
&  0.57548E+00_JPRB, 0.57448E+00_JPRB, 0.56424E+00_JPRB, 0.56360E+00_JPRB, 0.55168E+00_JPRB, 0.53974E+00_JPRB, 0.51977E+00_JPRB ,&
&  0.50176E+00_JPRB, 0.48547E+00_JPRB, 0.47376E+00_JPRB, 0.46925E+00_JPRB, 0.46670E+00_JPRB, 0.46062E+00_JPRB, 0.44609E+00_JPRB ,&
&  0.43619E+00_JPRB, 0.43399E+00_JPRB, 0.43573E+00_JPRB, 0.44060E+00_JPRB, 0.47960E+00_JPRB, 0.53751E+00_JPRB, 0.53978E+00_JPRB ,&
&  0.56062E+00_JPRB, 0.57238E+00_JPRB, 0.58417E+00_JPRB, 0.57178E+00_JPRB, 0.53758E+00_JPRB, 0.49487E+00_JPRB, 0.44262E+00_JPRB ,&
&  0.39310E+00_JPRB, 0.36312E+00_JPRB, 0.33861E+00_JPRB, 0.32057E+00_JPRB, 0.30741E+00_JPRB, 0.30251E+00_JPRB, 0.29607E+00_JPRB ,&
&  0.29212E+00_JPRB, 0.28901E+00_JPRB, 0.28784E+00_JPRB, 0.28760E+00_JPRB, 0.28675E+00_JPRB, 0.28433E+00_JPRB, 0.28368E+00_JPRB ,&
&  0.28448E+00_JPRB, 0.28448E+00_JPRB, 0.28393E+00_JPRB, 0.28496E+00_JPRB, 0.28600E+00_JPRB, 0.28778E+00_JPRB, 0.28962E+00_JPRB ,&
&  0.29249E+00_JPRB, 0.29409E+00_JPRB, 0.29646E+00_JPRB, 0.29920E+00_JPRB, 0.29808E+00_JPRB, 0.29715E+00_JPRB, 0.29447E+00_JPRB ,&
&  0.29112E+00_JPRB, 0.29143E+00_JPRB, 0.29122E+00_JPRB, 0.29221E+00_JPRB, 0.29452E+00_JPRB, 0.29735E+00_JPRB, 0.30090E+00_JPRB ,&
&  0.30420E+00_JPRB, 0.30763E+00_JPRB, 0.31025E+00_JPRB, 0.31290E+00_JPRB, 0.31740E+00_JPRB, 0.32196E+00_JPRB, 0.32781E+00_JPRB ,&
&  0.33384E+00_JPRB, 0.34080E+00_JPRB, 0.35011E+00_JPRB, 0.36220E+00_JPRB, 0.38052E+00_JPRB, 0.40004E+00_JPRB, 0.43071E+00_JPRB ,&
&  0.45461E+00_JPRB, 0.54873E+00_JPRB, 0.78128E+00_JPRB, 0.71737E+00_JPRB, 0.74133E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.73225E+00_JPRB, 0.75089E+00_JPRB, 0.77493E+00_JPRB, 0.76839E+00_JPRB, 0.78790E+00_JPRB, 0.77456E+00_JPRB, 0.75046E+00_JPRB ,&
&  0.70314E+00_JPRB, 0.64835E+00_JPRB, 0.60110E+00_JPRB, 0.54924E+00_JPRB, 0.50744E+00_JPRB, 0.47793E+00_JPRB, 0.46280E+00_JPRB ,&
&  0.45399E+00_JPRB, 0.45090E+00_JPRB, 0.44747E+00_JPRB, 0.44565E+00_JPRB, 0.43888E+00_JPRB, 0.43436E+00_JPRB, 0.42755E+00_JPRB ,&
&  0.42165E+00_JPRB, 0.41900E+00_JPRB, 0.41596E+00_JPRB, 0.42097E+00_JPRB, 0.42662E+00_JPRB, 0.43200E+00_JPRB, 0.43391E+00_JPRB ,&
&  0.43487E+00_JPRB, 0.43641E+00_JPRB, 0.43973E+00_JPRB, 0.43721E+00_JPRB, 0.44895E+00_JPRB, 0.46938E+00_JPRB, 0.48516E+00_JPRB ,&
&  0.50055E+00_JPRB, 0.51289E+00_JPRB, 0.50761E+00_JPRB, 0.48243E+00_JPRB, 0.41535E+00_JPRB, 0.38134E+00_JPRB, 0.34986E+00_JPRB ,&
&  0.32924E+00_JPRB, 0.31382E+00_JPRB, 0.30299E+00_JPRB, 0.29699E+00_JPRB, 0.29151E+00_JPRB, 0.28901E+00_JPRB, 0.28633E+00_JPRB ,&
&  0.28388E+00_JPRB, 0.28277E+00_JPRB, 0.28108E+00_JPRB, 0.28017E+00_JPRB, 0.27889E+00_JPRB, 0.27715E+00_JPRB, 0.27619E+00_JPRB ,&
&  0.27677E+00_JPRB, 0.27680E+00_JPRB, 0.27836E+00_JPRB, 0.27954E+00_JPRB, 0.28168E+00_JPRB, 0.28338E+00_JPRB, 0.28596E+00_JPRB ,&
&  0.28845E+00_JPRB, 0.29062E+00_JPRB, 0.29060E+00_JPRB, 0.29045E+00_JPRB, 0.28966E+00_JPRB, 0.28941E+00_JPRB, 0.28881E+00_JPRB ,&
&  0.28769E+00_JPRB, 0.28824E+00_JPRB, 0.28908E+00_JPRB, 0.28987E+00_JPRB, 0.29166E+00_JPRB, 0.29388E+00_JPRB, 0.29689E+00_JPRB ,&
&  0.30085E+00_JPRB, 0.30488E+00_JPRB, 0.30839E+00_JPRB, 0.31231E+00_JPRB, 0.31709E+00_JPRB, 0.32227E+00_JPRB, 0.32852E+00_JPRB ,&
&  0.33327E+00_JPRB, 0.34259E+00_JPRB, 0.35602E+00_JPRB, 0.37044E+00_JPRB, 0.39573E+00_JPRB, 0.41585E+00_JPRB, 0.44976E+00_JPRB ,&
&  0.49468E+00_JPRB, 0.65413E+00_JPRB, 0.93405E+00_JPRB, 0.71833E+00_JPRB, 0.71411E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.64787E+00_JPRB, 0.65819E+00_JPRB, 0.67228E+00_JPRB, 0.68388E+00_JPRB, 0.70702E+00_JPRB, 0.72007E+00_JPRB, 0.76326E+00_JPRB ,&
&  0.71796E+00_JPRB, 0.64050E+00_JPRB, 0.60822E+00_JPRB, 0.57222E+00_JPRB, 0.53210E+00_JPRB, 0.50276E+00_JPRB, 0.48223E+00_JPRB ,&
&  0.46936E+00_JPRB, 0.45823E+00_JPRB, 0.44720E+00_JPRB, 0.43390E+00_JPRB, 0.42049E+00_JPRB, 0.40629E+00_JPRB, 0.39631E+00_JPRB ,&
&  0.39071E+00_JPRB, 0.39010E+00_JPRB, 0.38950E+00_JPRB, 0.39299E+00_JPRB, 0.39755E+00_JPRB, 0.40320E+00_JPRB, 0.41047E+00_JPRB ,&
&  0.42908E+00_JPRB, 0.43864E+00_JPRB, 0.44259E+00_JPRB, 0.44181E+00_JPRB, 0.44211E+00_JPRB, 0.44540E+00_JPRB, 0.43615E+00_JPRB ,&
&  0.43881E+00_JPRB, 0.45470E+00_JPRB, 0.45953E+00_JPRB, 0.43726E+00_JPRB, 0.38247E+00_JPRB, 0.35094E+00_JPRB, 0.32774E+00_JPRB ,&
&  0.32029E+00_JPRB, 0.30922E+00_JPRB, 0.30230E+00_JPRB, 0.29530E+00_JPRB, 0.28999E+00_JPRB, 0.28803E+00_JPRB, 0.28589E+00_JPRB ,&
&  0.28470E+00_JPRB, 0.28437E+00_JPRB, 0.28234E+00_JPRB, 0.28161E+00_JPRB, 0.28011E+00_JPRB, 0.27920E+00_JPRB, 0.27882E+00_JPRB ,&
&  0.27861E+00_JPRB, 0.27832E+00_JPRB, 0.27865E+00_JPRB, 0.27925E+00_JPRB, 0.28025E+00_JPRB, 0.28104E+00_JPRB, 0.28179E+00_JPRB ,&
&  0.28301E+00_JPRB, 0.28419E+00_JPRB, 0.28464E+00_JPRB, 0.28554E+00_JPRB, 0.28627E+00_JPRB, 0.28795E+00_JPRB, 0.28884E+00_JPRB ,&
&  0.28972E+00_JPRB, 0.29057E+00_JPRB, 0.29079E+00_JPRB, 0.29185E+00_JPRB, 0.29315E+00_JPRB, 0.29557E+00_JPRB, 0.29820E+00_JPRB ,&
&  0.30149E+00_JPRB, 0.30549E+00_JPRB, 0.30973E+00_JPRB, 0.31404E+00_JPRB, 0.31938E+00_JPRB, 0.32462E+00_JPRB, 0.33188E+00_JPRB ,&
&  0.33958E+00_JPRB, 0.35050E+00_JPRB, 0.36618E+00_JPRB, 0.38438E+00_JPRB, 0.40721E+00_JPRB, 0.42599E+00_JPRB, 0.44417E+00_JPRB ,&
&  0.47589E+00_JPRB, 0.56902E+00_JPRB, 0.62889E+00_JPRB, 0.62481E+00_JPRB, 0.63935E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.62148E+00_JPRB, 0.62511E+00_JPRB, 0.62887E+00_JPRB, 0.63400E+00_JPRB, 0.63966E+00_JPRB, 0.64467E+00_JPRB, 0.64844E+00_JPRB ,&
&  0.62954E+00_JPRB, 0.60640E+00_JPRB, 0.58708E+00_JPRB, 0.56394E+00_JPRB, 0.54274E+00_JPRB, 0.52387E+00_JPRB, 0.50341E+00_JPRB ,&
&  0.48273E+00_JPRB, 0.46225E+00_JPRB, 0.43939E+00_JPRB, 0.42050E+00_JPRB, 0.40205E+00_JPRB, 0.38713E+00_JPRB, 0.37888E+00_JPRB ,&
&  0.37382E+00_JPRB, 0.37216E+00_JPRB, 0.37309E+00_JPRB, 0.37442E+00_JPRB, 0.37608E+00_JPRB, 0.38008E+00_JPRB, 0.38615E+00_JPRB ,&
&  0.39463E+00_JPRB, 0.41104E+00_JPRB, 0.42544E+00_JPRB, 0.43014E+00_JPRB, 0.42541E+00_JPRB, 0.41588E+00_JPRB, 0.40745E+00_JPRB ,&
&  0.39764E+00_JPRB, 0.38576E+00_JPRB, 0.37031E+00_JPRB, 0.35486E+00_JPRB, 0.34048E+00_JPRB, 0.33005E+00_JPRB, 0.32078E+00_JPRB ,&
&  0.31269E+00_JPRB, 0.30540E+00_JPRB, 0.29958E+00_JPRB, 0.29576E+00_JPRB, 0.29246E+00_JPRB, 0.28996E+00_JPRB, 0.28740E+00_JPRB ,&
&  0.28635E+00_JPRB, 0.28529E+00_JPRB, 0.28466E+00_JPRB, 0.28432E+00_JPRB, 0.28368E+00_JPRB, 0.28341E+00_JPRB, 0.28324E+00_JPRB ,&
&  0.28281E+00_JPRB, 0.28209E+00_JPRB, 0.28172E+00_JPRB, 0.28150E+00_JPRB, 0.28211E+00_JPRB, 0.28268E+00_JPRB, 0.28352E+00_JPRB ,&
&  0.28471E+00_JPRB, 0.28697E+00_JPRB, 0.28829E+00_JPRB, 0.28957E+00_JPRB, 0.29024E+00_JPRB, 0.29092E+00_JPRB, 0.29181E+00_JPRB ,&
&  0.29321E+00_JPRB, 0.29480E+00_JPRB, 0.29615E+00_JPRB, 0.29798E+00_JPRB, 0.29983E+00_JPRB, 0.30239E+00_JPRB, 0.30523E+00_JPRB ,&
&  0.30825E+00_JPRB, 0.31197E+00_JPRB, 0.31648E+00_JPRB, 0.32043E+00_JPRB, 0.32695E+00_JPRB, 0.33357E+00_JPRB, 0.34114E+00_JPRB ,&
&  0.34922E+00_JPRB, 0.35802E+00_JPRB, 0.36996E+00_JPRB, 0.38431E+00_JPRB, 0.40282E+00_JPRB, 0.40991E+00_JPRB, 0.41909E+00_JPRB ,&
&  0.43356E+00_JPRB, 0.46354E+00_JPRB, 0.60132E+00_JPRB, 0.61496E+00_JPRB, 0.62067E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.39950E+00_JPRB, 0.40525E+00_JPRB, 0.40889E+00_JPRB, 0.41758E+00_JPRB, 0.42633E+00_JPRB, 0.52706E+00_JPRB, 0.52218E+00_JPRB ,&
&  0.52185E+00_JPRB, 0.47834E+00_JPRB, 0.47103E+00_JPRB, 0.45913E+00_JPRB, 0.44388E+00_JPRB, 0.42511E+00_JPRB, 0.41358E+00_JPRB ,&
&  0.40403E+00_JPRB, 0.39263E+00_JPRB, 0.38221E+00_JPRB, 0.37590E+00_JPRB, 0.37090E+00_JPRB, 0.36624E+00_JPRB, 0.36318E+00_JPRB ,&
&  0.36136E+00_JPRB, 0.36097E+00_JPRB, 0.36090E+00_JPRB, 0.36034E+00_JPRB, 0.36312E+00_JPRB, 0.36643E+00_JPRB, 0.36986E+00_JPRB ,&
&  0.37088E+00_JPRB, 0.37575E+00_JPRB, 0.37714E+00_JPRB, 0.37425E+00_JPRB, 0.36978E+00_JPRB, 0.36220E+00_JPRB, 0.35461E+00_JPRB ,&
&  0.34678E+00_JPRB, 0.33976E+00_JPRB, 0.33383E+00_JPRB, 0.32851E+00_JPRB, 0.32231E+00_JPRB, 0.31608E+00_JPRB, 0.31187E+00_JPRB ,&
&  0.30790E+00_JPRB, 0.30376E+00_JPRB, 0.30037E+00_JPRB, 0.29872E+00_JPRB, 0.29688E+00_JPRB, 0.29561E+00_JPRB, 0.29458E+00_JPRB ,&
&  0.29429E+00_JPRB, 0.29409E+00_JPRB, 0.29413E+00_JPRB, 0.29436E+00_JPRB, 0.29431E+00_JPRB, 0.29420E+00_JPRB, 0.29413E+00_JPRB ,&
&  0.29478E+00_JPRB, 0.29487E+00_JPRB, 0.29486E+00_JPRB, 0.29473E+00_JPRB, 0.29566E+00_JPRB, 0.29612E+00_JPRB, 0.29674E+00_JPRB ,&
&  0.29752E+00_JPRB, 0.29850E+00_JPRB, 0.29922E+00_JPRB, 0.29998E+00_JPRB, 0.30073E+00_JPRB, 0.30169E+00_JPRB, 0.30281E+00_JPRB ,&
&  0.30398E+00_JPRB, 0.30569E+00_JPRB, 0.30747E+00_JPRB, 0.30900E+00_JPRB, 0.31039E+00_JPRB, 0.31228E+00_JPRB, 0.31441E+00_JPRB ,&
&  0.31743E+00_JPRB, 0.32032E+00_JPRB, 0.32326E+00_JPRB, 0.32472E+00_JPRB, 0.32742E+00_JPRB, 0.32975E+00_JPRB, 0.33240E+00_JPRB ,&
&  0.33567E+00_JPRB, 0.34122E+00_JPRB, 0.34966E+00_JPRB, 0.35918E+00_JPRB, 0.37025E+00_JPRB, 0.37784E+00_JPRB, 0.38260E+00_JPRB ,&
&  0.38638E+00_JPRB, 0.39062E+00_JPRB, 0.39492E+00_JPRB, 0.39711E+00_JPRB, 0.39869E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.36562E+00_JPRB, 0.36548E+00_JPRB, 0.36553E+00_JPRB, 0.36512E+00_JPRB, 0.36328E+00_JPRB, 0.36172E+00_JPRB, 0.36024E+00_JPRB ,&
&  0.35925E+00_JPRB, 0.35806E+00_JPRB, 0.35748E+00_JPRB, 0.35709E+00_JPRB, 0.35647E+00_JPRB, 0.35566E+00_JPRB, 0.35460E+00_JPRB ,&
&  0.35321E+00_JPRB, 0.35142E+00_JPRB, 0.34931E+00_JPRB, 0.34857E+00_JPRB, 0.34819E+00_JPRB, 0.34780E+00_JPRB, 0.34732E+00_JPRB ,&
&  0.34670E+00_JPRB, 0.34618E+00_JPRB, 0.34532E+00_JPRB, 0.34545E+00_JPRB, 0.34536E+00_JPRB, 0.34527E+00_JPRB, 0.34530E+00_JPRB ,&
&  0.34538E+00_JPRB, 0.34567E+00_JPRB, 0.34558E+00_JPRB, 0.34515E+00_JPRB, 0.34438E+00_JPRB, 0.34268E+00_JPRB, 0.33993E+00_JPRB ,&
&  0.33681E+00_JPRB, 0.33369E+00_JPRB, 0.33119E+00_JPRB, 0.32889E+00_JPRB, 0.32679E+00_JPRB, 0.32489E+00_JPRB, 0.32333E+00_JPRB ,&
&  0.32141E+00_JPRB, 0.31950E+00_JPRB, 0.31772E+00_JPRB, 0.31638E+00_JPRB, 0.31488E+00_JPRB, 0.31353E+00_JPRB, 0.31221E+00_JPRB ,&
&  0.31127E+00_JPRB, 0.31045E+00_JPRB, 0.30976E+00_JPRB, 0.30939E+00_JPRB, 0.30911E+00_JPRB, 0.30887E+00_JPRB, 0.30859E+00_JPRB ,&
&  0.30834E+00_JPRB, 0.30823E+00_JPRB, 0.30817E+00_JPRB, 0.30821E+00_JPRB, 0.30840E+00_JPRB, 0.30838E+00_JPRB, 0.30833E+00_JPRB ,&
&  0.30836E+00_JPRB, 0.30880E+00_JPRB, 0.30905E+00_JPRB, 0.30938E+00_JPRB, 0.30988E+00_JPRB, 0.31056E+00_JPRB, 0.31132E+00_JPRB ,&
&  0.31207E+00_JPRB, 0.31294E+00_JPRB, 0.31387E+00_JPRB, 0.31505E+00_JPRB, 0.31633E+00_JPRB, 0.31761E+00_JPRB, 0.31884E+00_JPRB ,&
&  0.32016E+00_JPRB, 0.32138E+00_JPRB, 0.32255E+00_JPRB, 0.32333E+00_JPRB, 0.32458E+00_JPRB, 0.32590E+00_JPRB, 0.32745E+00_JPRB ,&
&  0.32925E+00_JPRB, 0.33167E+00_JPRB, 0.33519E+00_JPRB, 0.33940E+00_JPRB, 0.34496E+00_JPRB, 0.34886E+00_JPRB, 0.35262E+00_JPRB ,&
&  0.35520E+00_JPRB, 0.35729E+00_JPRB, 0.35893E+00_JPRB, 0.36093E+00_JPRB, 0.36314E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB ,&
&  0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB, 0.32778E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2040:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2040
