!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2050 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2050* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB ,&
&  0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB, 0.47872E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.49706E+00_JPRB, 0.49621E+00_JPRB, 0.49541E+00_JPRB, 0.49470E+00_JPRB, 0.49405E+00_JPRB, 0.49356E+00_JPRB, 0.49313E+00_JPRB ,&
&  0.49256E+00_JPRB, 0.49207E+00_JPRB, 0.49146E+00_JPRB, 0.49086E+00_JPRB, 0.49027E+00_JPRB, 0.48964E+00_JPRB, 0.48886E+00_JPRB ,&
&  0.48792E+00_JPRB, 0.48704E+00_JPRB, 0.48606E+00_JPRB, 0.48489E+00_JPRB, 0.48374E+00_JPRB, 0.48249E+00_JPRB, 0.48100E+00_JPRB ,&
&  0.47928E+00_JPRB, 0.47759E+00_JPRB, 0.47581E+00_JPRB, 0.47396E+00_JPRB, 0.47195E+00_JPRB, 0.46973E+00_JPRB, 0.46773E+00_JPRB ,&
&  0.46625E+00_JPRB, 0.46467E+00_JPRB, 0.46319E+00_JPRB, 0.46173E+00_JPRB, 0.46058E+00_JPRB, 0.45975E+00_JPRB, 0.45886E+00_JPRB ,&
&  0.45809E+00_JPRB, 0.45750E+00_JPRB, 0.45757E+00_JPRB, 0.45787E+00_JPRB, 0.45834E+00_JPRB, 0.45940E+00_JPRB, 0.46107E+00_JPRB ,&
&  0.46248E+00_JPRB, 0.46371E+00_JPRB, 0.46519E+00_JPRB, 0.46684E+00_JPRB, 0.46858E+00_JPRB, 0.47033E+00_JPRB, 0.47195E+00_JPRB ,&
&  0.47318E+00_JPRB, 0.47419E+00_JPRB, 0.47528E+00_JPRB, 0.47620E+00_JPRB, 0.47672E+00_JPRB, 0.47722E+00_JPRB, 0.47777E+00_JPRB ,&
&  0.47819E+00_JPRB, 0.47847E+00_JPRB, 0.47887E+00_JPRB, 0.47954E+00_JPRB, 0.48014E+00_JPRB, 0.48081E+00_JPRB, 0.48174E+00_JPRB ,&
&  0.48298E+00_JPRB, 0.48392E+00_JPRB, 0.48506E+00_JPRB, 0.48651E+00_JPRB, 0.48838E+00_JPRB, 0.49059E+00_JPRB, 0.49236E+00_JPRB ,&
&  0.49422E+00_JPRB, 0.49618E+00_JPRB, 0.49824E+00_JPRB, 0.49973E+00_JPRB, 0.50133E+00_JPRB, 0.50282E+00_JPRB, 0.50425E+00_JPRB ,&
&  0.50471E+00_JPRB, 0.50540E+00_JPRB, 0.50627E+00_JPRB, 0.50708E+00_JPRB, 0.50662E+00_JPRB, 0.50634E+00_JPRB, 0.50613E+00_JPRB ,&
&  0.50660E+00_JPRB, 0.50554E+00_JPRB, 0.50502E+00_JPRB, 0.50429E+00_JPRB, 0.50389E+00_JPRB, 0.50277E+00_JPRB, 0.50194E+00_JPRB ,&
&  0.50107E+00_JPRB, 0.50040E+00_JPRB, 0.49954E+00_JPRB, 0.49867E+00_JPRB, 0.49779E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.51556E+00_JPRB, 0.51404E+00_JPRB, 0.51244E+00_JPRB, 0.51078E+00_JPRB, 0.50899E+00_JPRB, 0.50756E+00_JPRB, 0.50617E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.50355E+00_JPRB, 0.50223E+00_JPRB, 0.50111E+00_JPRB, 0.50012E+00_JPRB, 0.49964E+00_JPRB, 0.49872E+00_JPRB ,&
&  0.49767E+00_JPRB, 0.49618E+00_JPRB, 0.49487E+00_JPRB, 0.49270E+00_JPRB, 0.49030E+00_JPRB, 0.48707E+00_JPRB, 0.48406E+00_JPRB ,&
&  0.48054E+00_JPRB, 0.47644E+00_JPRB, 0.47285E+00_JPRB, 0.46933E+00_JPRB, 0.46583E+00_JPRB, 0.46217E+00_JPRB, 0.45956E+00_JPRB ,&
&  0.45785E+00_JPRB, 0.45549E+00_JPRB, 0.45345E+00_JPRB, 0.45176E+00_JPRB, 0.44992E+00_JPRB, 0.44862E+00_JPRB, 0.44742E+00_JPRB ,&
&  0.44513E+00_JPRB, 0.44278E+00_JPRB, 0.44292E+00_JPRB, 0.44345E+00_JPRB, 0.44475E+00_JPRB, 0.44738E+00_JPRB, 0.45216E+00_JPRB ,&
&  0.45844E+00_JPRB, 0.46443E+00_JPRB, 0.47090E+00_JPRB, 0.47669E+00_JPRB, 0.48134E+00_JPRB, 0.48502E+00_JPRB, 0.48804E+00_JPRB ,&
&  0.49000E+00_JPRB, 0.49147E+00_JPRB, 0.49225E+00_JPRB, 0.49224E+00_JPRB, 0.49143E+00_JPRB, 0.49114E+00_JPRB, 0.49123E+00_JPRB ,&
&  0.49155E+00_JPRB, 0.49008E+00_JPRB, 0.48975E+00_JPRB, 0.49032E+00_JPRB, 0.49162E+00_JPRB, 0.49294E+00_JPRB, 0.49549E+00_JPRB ,&
&  0.49903E+00_JPRB, 0.50337E+00_JPRB, 0.50736E+00_JPRB, 0.51262E+00_JPRB, 0.51876E+00_JPRB, 0.52582E+00_JPRB, 0.53008E+00_JPRB ,&
&  0.53527E+00_JPRB, 0.54077E+00_JPRB, 0.54658E+00_JPRB, 0.54807E+00_JPRB, 0.54976E+00_JPRB, 0.55110E+00_JPRB, 0.55276E+00_JPRB ,&
&  0.55072E+00_JPRB, 0.55015E+00_JPRB, 0.54993E+00_JPRB, 0.55109E+00_JPRB, 0.54863E+00_JPRB, 0.54756E+00_JPRB, 0.54641E+00_JPRB ,&
&  0.54720E+00_JPRB, 0.54302E+00_JPRB, 0.54063E+00_JPRB, 0.53753E+00_JPRB, 0.53564E+00_JPRB, 0.53132E+00_JPRB, 0.52792E+00_JPRB ,&
&  0.52502E+00_JPRB, 0.52320E+00_JPRB, 0.52089E+00_JPRB, 0.51892E+00_JPRB, 0.51729E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.53564E+00_JPRB, 0.53225E+00_JPRB, 0.52732E+00_JPRB, 0.52112E+00_JPRB, 0.51533E+00_JPRB, 0.51096E+00_JPRB, 0.50632E+00_JPRB ,&
&  0.50255E+00_JPRB, 0.49915E+00_JPRB, 0.49675E+00_JPRB, 0.49491E+00_JPRB, 0.49309E+00_JPRB, 0.49344E+00_JPRB, 0.49389E+00_JPRB ,&
&  0.49434E+00_JPRB, 0.49385E+00_JPRB, 0.49503E+00_JPRB, 0.49409E+00_JPRB, 0.49236E+00_JPRB, 0.48870E+00_JPRB, 0.48362E+00_JPRB ,&
&  0.47795E+00_JPRB, 0.47156E+00_JPRB, 0.46428E+00_JPRB, 0.45598E+00_JPRB, 0.44811E+00_JPRB, 0.42147E+00_JPRB, 0.41495E+00_JPRB ,&
&  0.41675E+00_JPRB, 0.41241E+00_JPRB, 0.40887E+00_JPRB, 0.40690E+00_JPRB, 0.40326E+00_JPRB, 0.39993E+00_JPRB, 0.39674E+00_JPRB ,&
&  0.37645E+00_JPRB, 0.37556E+00_JPRB, 0.37339E+00_JPRB, 0.37220E+00_JPRB, 0.37566E+00_JPRB, 0.38732E+00_JPRB, 0.40546E+00_JPRB ,&
&  0.43781E+00_JPRB, 0.45183E+00_JPRB, 0.46370E+00_JPRB, 0.47777E+00_JPRB, 0.48873E+00_JPRB, 0.49692E+00_JPRB, 0.50502E+00_JPRB ,&
&  0.51181E+00_JPRB, 0.51840E+00_JPRB, 0.52325E+00_JPRB, 0.52579E+00_JPRB, 0.52609E+00_JPRB, 0.52602E+00_JPRB, 0.52568E+00_JPRB ,&
&  0.52517E+00_JPRB, 0.52462E+00_JPRB, 0.52487E+00_JPRB, 0.52502E+00_JPRB, 0.52481E+00_JPRB, 0.52587E+00_JPRB, 0.53015E+00_JPRB ,&
&  0.53641E+00_JPRB, 0.54572E+00_JPRB, 0.55381E+00_JPRB, 0.56495E+00_JPRB, 0.57511E+00_JPRB, 0.58787E+00_JPRB, 0.59291E+00_JPRB ,&
&  0.59904E+00_JPRB, 0.60296E+00_JPRB, 0.60827E+00_JPRB, 0.60829E+00_JPRB, 0.60821E+00_JPRB, 0.60718E+00_JPRB, 0.60743E+00_JPRB ,&
&  0.60416E+00_JPRB, 0.60193E+00_JPRB, 0.59899E+00_JPRB, 0.59876E+00_JPRB, 0.59500E+00_JPRB, 0.59431E+00_JPRB, 0.59204E+00_JPRB ,&
&  0.59356E+00_JPRB, 0.58879E+00_JPRB, 0.58691E+00_JPRB, 0.58104E+00_JPRB, 0.57634E+00_JPRB, 0.56781E+00_JPRB, 0.56038E+00_JPRB ,&
&  0.55401E+00_JPRB, 0.54915E+00_JPRB, 0.54506E+00_JPRB, 0.54157E+00_JPRB, 0.53840E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.55066E+00_JPRB, 0.54141E+00_JPRB, 0.53118E+00_JPRB, 0.52258E+00_JPRB, 0.51461E+00_JPRB, 0.50874E+00_JPRB, 0.50314E+00_JPRB ,&
&  0.49886E+00_JPRB, 0.49473E+00_JPRB, 0.49162E+00_JPRB, 0.48873E+00_JPRB, 0.48650E+00_JPRB, 0.48505E+00_JPRB, 0.48371E+00_JPRB ,&
&  0.48323E+00_JPRB, 0.48247E+00_JPRB, 0.48268E+00_JPRB, 0.48216E+00_JPRB, 0.48215E+00_JPRB, 0.47952E+00_JPRB, 0.47427E+00_JPRB ,&
&  0.46815E+00_JPRB, 0.46157E+00_JPRB, 0.43964E+00_JPRB, 0.43197E+00_JPRB, 0.41383E+00_JPRB, 0.40049E+00_JPRB, 0.39254E+00_JPRB ,&
&  0.37835E+00_JPRB, 0.37055E+00_JPRB, 0.35621E+00_JPRB, 0.34820E+00_JPRB, 0.34669E+00_JPRB, 0.33784E+00_JPRB, 0.33448E+00_JPRB ,&
&  0.32826E+00_JPRB, 0.31679E+00_JPRB, 0.31247E+00_JPRB, 0.31067E+00_JPRB, 0.31438E+00_JPRB, 0.32424E+00_JPRB, 0.34240E+00_JPRB ,&
&  0.37684E+00_JPRB, 0.41538E+00_JPRB, 0.46827E+00_JPRB, 0.48366E+00_JPRB, 0.48627E+00_JPRB, 0.49856E+00_JPRB, 0.50852E+00_JPRB ,&
&  0.51480E+00_JPRB, 0.52849E+00_JPRB, 0.54302E+00_JPRB, 0.54922E+00_JPRB, 0.55341E+00_JPRB, 0.55500E+00_JPRB, 0.55633E+00_JPRB ,&
&  0.55748E+00_JPRB, 0.56313E+00_JPRB, 0.56671E+00_JPRB, 0.57062E+00_JPRB, 0.57486E+00_JPRB, 0.57952E+00_JPRB, 0.58887E+00_JPRB ,&
&  0.59703E+00_JPRB, 0.61359E+00_JPRB, 0.62696E+00_JPRB, 0.64723E+00_JPRB, 0.65554E+00_JPRB, 0.66541E+00_JPRB, 0.66784E+00_JPRB ,&
&  0.67236E+00_JPRB, 0.67487E+00_JPRB, 0.67676E+00_JPRB, 0.67085E+00_JPRB, 0.66942E+00_JPRB, 0.67329E+00_JPRB, 0.67168E+00_JPRB ,&
&  0.66914E+00_JPRB, 0.66752E+00_JPRB, 0.66518E+00_JPRB, 0.66346E+00_JPRB, 0.66225E+00_JPRB, 0.66329E+00_JPRB, 0.66247E+00_JPRB ,&
&  0.66539E+00_JPRB, 0.66220E+00_JPRB, 0.65899E+00_JPRB, 0.65198E+00_JPRB, 0.64290E+00_JPRB, 0.62996E+00_JPRB, 0.61360E+00_JPRB ,&
&  0.59951E+00_JPRB, 0.58851E+00_JPRB, 0.57715E+00_JPRB, 0.56840E+00_JPRB, 0.55931E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.52802E+00_JPRB, 0.51630E+00_JPRB, 0.50654E+00_JPRB, 0.49963E+00_JPRB, 0.49367E+00_JPRB, 0.48985E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.48417E+00_JPRB, 0.48177E+00_JPRB, 0.48071E+00_JPRB, 0.47933E+00_JPRB, 0.47898E+00_JPRB, 0.47898E+00_JPRB, 0.47964E+00_JPRB ,&
&  0.48057E+00_JPRB, 0.48076E+00_JPRB, 0.48118E+00_JPRB, 0.47892E+00_JPRB, 0.47680E+00_JPRB, 0.47340E+00_JPRB, 0.47015E+00_JPRB ,&
&  0.46316E+00_JPRB, 0.45500E+00_JPRB, 0.43899E+00_JPRB, 0.43025E+00_JPRB, 0.41385E+00_JPRB, 0.40179E+00_JPRB, 0.38969E+00_JPRB ,&
&  0.37871E+00_JPRB, 0.35805E+00_JPRB, 0.36435E+00_JPRB, 0.35151E+00_JPRB, 0.33931E+00_JPRB, 0.32642E+00_JPRB, 0.31087E+00_JPRB ,&
&  0.30497E+00_JPRB, 0.28978E+00_JPRB, 0.28429E+00_JPRB, 0.28448E+00_JPRB, 0.28679E+00_JPRB, 0.29730E+00_JPRB, 0.31385E+00_JPRB ,&
&  0.34706E+00_JPRB, 0.43145E+00_JPRB, 0.46158E+00_JPRB, 0.47508E+00_JPRB, 0.47202E+00_JPRB, 0.47362E+00_JPRB, 0.46964E+00_JPRB ,&
&  0.48450E+00_JPRB, 0.50437E+00_JPRB, 0.52496E+00_JPRB, 0.53879E+00_JPRB, 0.55558E+00_JPRB, 0.56498E+00_JPRB, 0.57331E+00_JPRB ,&
&  0.58312E+00_JPRB, 0.59859E+00_JPRB, 0.61292E+00_JPRB, 0.62787E+00_JPRB, 0.64664E+00_JPRB, 0.66507E+00_JPRB, 0.68641E+00_JPRB ,&
&  0.69919E+00_JPRB, 0.72456E+00_JPRB, 0.73318E+00_JPRB, 0.74457E+00_JPRB, 0.74571E+00_JPRB, 0.74368E+00_JPRB, 0.74425E+00_JPRB ,&
&  0.74743E+00_JPRB, 0.75121E+00_JPRB, 0.75402E+00_JPRB, 0.75211E+00_JPRB, 0.75122E+00_JPRB, 0.74100E+00_JPRB, 0.73705E+00_JPRB ,&
&  0.73685E+00_JPRB, 0.73519E+00_JPRB, 0.73246E+00_JPRB, 0.72720E+00_JPRB, 0.71987E+00_JPRB, 0.71172E+00_JPRB, 0.70694E+00_JPRB ,&
&  0.70070E+00_JPRB, 0.69189E+00_JPRB, 0.68201E+00_JPRB, 0.67173E+00_JPRB, 0.66088E+00_JPRB, 0.64946E+00_JPRB, 0.63079E+00_JPRB ,&
&  0.60782E+00_JPRB, 0.58809E+00_JPRB, 0.56974E+00_JPRB, 0.55559E+00_JPRB, 0.54102E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.51026E+00_JPRB, 0.49923E+00_JPRB, 0.49005E+00_JPRB, 0.48498E+00_JPRB, 0.47992E+00_JPRB, 0.47779E+00_JPRB, 0.47465E+00_JPRB ,&
&  0.47304E+00_JPRB, 0.47148E+00_JPRB, 0.47093E+00_JPRB, 0.47043E+00_JPRB, 0.47068E+00_JPRB, 0.47112E+00_JPRB, 0.47239E+00_JPRB ,&
&  0.47375E+00_JPRB, 0.47593E+00_JPRB, 0.47808E+00_JPRB, 0.47754E+00_JPRB, 0.47666E+00_JPRB, 0.47444E+00_JPRB, 0.47156E+00_JPRB ,&
&  0.46548E+00_JPRB, 0.45777E+00_JPRB, 0.44215E+00_JPRB, 0.43110E+00_JPRB, 0.42132E+00_JPRB, 0.40720E+00_JPRB, 0.40356E+00_JPRB ,&
&  0.39893E+00_JPRB, 0.38753E+00_JPRB, 0.37165E+00_JPRB, 0.36458E+00_JPRB, 0.35450E+00_JPRB, 0.32203E+00_JPRB, 0.32319E+00_JPRB ,&
&  0.31444E+00_JPRB, 0.29250E+00_JPRB, 0.28615E+00_JPRB, 0.28164E+00_JPRB, 0.28325E+00_JPRB, 0.29273E+00_JPRB, 0.30740E+00_JPRB ,&
&  0.33892E+00_JPRB, 0.43640E+00_JPRB, 0.46078E+00_JPRB, 0.46958E+00_JPRB, 0.46021E+00_JPRB, 0.46391E+00_JPRB, 0.46432E+00_JPRB ,&
&  0.47494E+00_JPRB, 0.48519E+00_JPRB, 0.49801E+00_JPRB, 0.50896E+00_JPRB, 0.52038E+00_JPRB, 0.52615E+00_JPRB, 0.53831E+00_JPRB ,&
&  0.56118E+00_JPRB, 0.58992E+00_JPRB, 0.61657E+00_JPRB, 0.65062E+00_JPRB, 0.69308E+00_JPRB, 0.76638E+00_JPRB, 0.84072E+00_JPRB ,&
&  0.84680E+00_JPRB, 0.83722E+00_JPRB, 0.83341E+00_JPRB, 0.82589E+00_JPRB, 0.82468E+00_JPRB, 0.82398E+00_JPRB, 0.82873E+00_JPRB ,&
&  0.83893E+00_JPRB, 0.84747E+00_JPRB, 0.83161E+00_JPRB, 0.80242E+00_JPRB, 0.78963E+00_JPRB, 0.76901E+00_JPRB, 0.76588E+00_JPRB ,&
&  0.76148E+00_JPRB, 0.77277E+00_JPRB, 0.76693E+00_JPRB, 0.75577E+00_JPRB, 0.74480E+00_JPRB, 0.73229E+00_JPRB, 0.72420E+00_JPRB ,&
&  0.71464E+00_JPRB, 0.70156E+00_JPRB, 0.68743E+00_JPRB, 0.67553E+00_JPRB, 0.66486E+00_JPRB, 0.65622E+00_JPRB, 0.64333E+00_JPRB ,&
&  0.61837E+00_JPRB, 0.58745E+00_JPRB, 0.56377E+00_JPRB, 0.54172E+00_JPRB, 0.52388E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.50130E+00_JPRB, 0.49414E+00_JPRB, 0.48957E+00_JPRB, 0.48875E+00_JPRB, 0.48722E+00_JPRB, 0.48681E+00_JPRB, 0.48352E+00_JPRB ,&
&  0.48037E+00_JPRB, 0.47711E+00_JPRB, 0.47422E+00_JPRB, 0.47160E+00_JPRB, 0.47007E+00_JPRB, 0.46848E+00_JPRB, 0.46757E+00_JPRB ,&
&  0.46687E+00_JPRB, 0.46147E+00_JPRB, 0.45990E+00_JPRB, 0.46219E+00_JPRB, 0.46492E+00_JPRB, 0.46674E+00_JPRB, 0.46911E+00_JPRB ,&
&  0.46557E+00_JPRB, 0.45868E+00_JPRB, 0.44943E+00_JPRB, 0.43729E+00_JPRB, 0.42816E+00_JPRB, 0.42496E+00_JPRB, 0.41810E+00_JPRB ,&
&  0.40320E+00_JPRB, 0.39060E+00_JPRB, 0.39498E+00_JPRB, 0.39038E+00_JPRB, 0.36697E+00_JPRB, 0.35456E+00_JPRB, 0.34286E+00_JPRB ,&
&  0.32418E+00_JPRB, 0.29708E+00_JPRB, 0.29101E+00_JPRB, 0.28847E+00_JPRB, 0.29438E+00_JPRB, 0.30411E+00_JPRB, 0.32579E+00_JPRB ,&
&  0.36457E+00_JPRB, 0.44178E+00_JPRB, 0.46244E+00_JPRB, 0.47305E+00_JPRB, 0.47796E+00_JPRB, 0.47734E+00_JPRB, 0.47687E+00_JPRB ,&
&  0.48396E+00_JPRB, 0.49084E+00_JPRB, 0.50187E+00_JPRB, 0.51420E+00_JPRB, 0.53048E+00_JPRB, 0.54470E+00_JPRB, 0.55568E+00_JPRB ,&
&  0.58909E+00_JPRB, 0.60787E+00_JPRB, 0.62620E+00_JPRB, 0.68430E+00_JPRB, 0.79027E+00_JPRB, 0.90301E+00_JPRB, 0.94913E+00_JPRB ,&
&  0.94117E+00_JPRB, 0.93008E+00_JPRB, 0.89708E+00_JPRB, 0.87768E+00_JPRB, 0.86955E+00_JPRB, 0.86755E+00_JPRB, 0.86962E+00_JPRB ,&
&  0.87592E+00_JPRB, 0.89218E+00_JPRB, 0.94723E+00_JPRB, 0.90166E+00_JPRB, 0.87429E+00_JPRB, 0.84623E+00_JPRB, 0.81421E+00_JPRB ,&
&  0.78986E+00_JPRB, 0.77680E+00_JPRB, 0.76851E+00_JPRB, 0.75711E+00_JPRB, 0.74592E+00_JPRB, 0.73662E+00_JPRB, 0.73515E+00_JPRB ,&
&  0.72750E+00_JPRB, 0.71437E+00_JPRB, 0.69401E+00_JPRB, 0.68214E+00_JPRB, 0.67090E+00_JPRB, 0.66027E+00_JPRB, 0.64909E+00_JPRB ,&
&  0.62301E+00_JPRB, 0.58809E+00_JPRB, 0.56054E+00_JPRB, 0.53606E+00_JPRB, 0.51582E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.50633E+00_JPRB, 0.50296E+00_JPRB, 0.50449E+00_JPRB, 0.51030E+00_JPRB, 0.51420E+00_JPRB, 0.50763E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.49326E+00_JPRB, 0.49063E+00_JPRB, 0.48893E+00_JPRB, 0.48491E+00_JPRB, 0.48189E+00_JPRB, 0.47891E+00_JPRB, 0.47600E+00_JPRB ,&
&  0.47304E+00_JPRB, 0.46961E+00_JPRB, 0.46537E+00_JPRB, 0.45917E+00_JPRB, 0.45582E+00_JPRB, 0.45698E+00_JPRB, 0.46119E+00_JPRB ,&
&  0.46499E+00_JPRB, 0.46409E+00_JPRB, 0.46047E+00_JPRB, 0.45031E+00_JPRB, 0.43937E+00_JPRB, 0.43054E+00_JPRB, 0.43022E+00_JPRB ,&
&  0.42361E+00_JPRB, 0.41043E+00_JPRB, 0.39524E+00_JPRB, 0.39940E+00_JPRB, 0.39989E+00_JPRB, 0.37563E+00_JPRB, 0.35537E+00_JPRB ,&
&  0.33506E+00_JPRB, 0.31648E+00_JPRB, 0.30820E+00_JPRB, 0.30786E+00_JPRB, 0.31833E+00_JPRB, 0.33758E+00_JPRB, 0.36394E+00_JPRB ,&
&  0.40289E+00_JPRB, 0.44130E+00_JPRB, 0.45940E+00_JPRB, 0.47369E+00_JPRB, 0.47613E+00_JPRB, 0.47401E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.47347E+00_JPRB, 0.47794E+00_JPRB, 0.48589E+00_JPRB, 0.49197E+00_JPRB, 0.50480E+00_JPRB, 0.53184E+00_JPRB, 0.59084E+00_JPRB ,&
&  0.73960E+00_JPRB, 0.75096E+00_JPRB, 0.68908E+00_JPRB, 0.75543E+00_JPRB, 0.86698E+00_JPRB, 0.96747E+00_JPRB, 0.10382E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.99376E+00_JPRB, 0.95103E+00_JPRB, 0.92679E+00_JPRB, 0.91020E+00_JPRB, 0.89546E+00_JPRB, 0.88861E+00_JPRB ,&
&  0.89143E+00_JPRB, 0.90958E+00_JPRB, 0.99904E+00_JPRB, 0.88063E+00_JPRB, 0.81885E+00_JPRB, 0.79778E+00_JPRB, 0.77388E+00_JPRB ,&
&  0.75275E+00_JPRB, 0.74269E+00_JPRB, 0.74567E+00_JPRB, 0.75345E+00_JPRB, 0.72541E+00_JPRB, 0.70044E+00_JPRB, 0.68375E+00_JPRB ,&
&  0.68019E+00_JPRB, 0.67112E+00_JPRB, 0.66445E+00_JPRB, 0.66716E+00_JPRB, 0.66351E+00_JPRB, 0.64657E+00_JPRB, 0.63578E+00_JPRB ,&
&  0.61710E+00_JPRB, 0.58590E+00_JPRB, 0.56423E+00_JPRB, 0.54290E+00_JPRB, 0.52044E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.50589E+00_JPRB, 0.51714E+00_JPRB, 0.53270E+00_JPRB, 0.54313E+00_JPRB, 0.54514E+00_JPRB, 0.52250E+00_JPRB, 0.49937E+00_JPRB ,&
&  0.48598E+00_JPRB, 0.48083E+00_JPRB, 0.48150E+00_JPRB, 0.48165E+00_JPRB, 0.48108E+00_JPRB, 0.48096E+00_JPRB, 0.47995E+00_JPRB ,&
&  0.47533E+00_JPRB, 0.46927E+00_JPRB, 0.46366E+00_JPRB, 0.45826E+00_JPRB, 0.45416E+00_JPRB, 0.45326E+00_JPRB, 0.45492E+00_JPRB ,&
&  0.45838E+00_JPRB, 0.46032E+00_JPRB, 0.46000E+00_JPRB, 0.45451E+00_JPRB, 0.45079E+00_JPRB, 0.44742E+00_JPRB, 0.43906E+00_JPRB ,&
&  0.42425E+00_JPRB, 0.42146E+00_JPRB, 0.41649E+00_JPRB, 0.40996E+00_JPRB, 0.41079E+00_JPRB, 0.39467E+00_JPRB, 0.37289E+00_JPRB ,&
&  0.35209E+00_JPRB, 0.33660E+00_JPRB, 0.33452E+00_JPRB, 0.34373E+00_JPRB, 0.36570E+00_JPRB, 0.39418E+00_JPRB, 0.41135E+00_JPRB ,&
&  0.42724E+00_JPRB, 0.44136E+00_JPRB, 0.45160E+00_JPRB, 0.46082E+00_JPRB, 0.46740E+00_JPRB, 0.47007E+00_JPRB, 0.47007E+00_JPRB ,&
&  0.46833E+00_JPRB, 0.46798E+00_JPRB, 0.46968E+00_JPRB, 0.45306E+00_JPRB, 0.46464E+00_JPRB, 0.54101E+00_JPRB, 0.62145E+00_JPRB ,&
&  0.72035E+00_JPRB, 0.75191E+00_JPRB, 0.77828E+00_JPRB, 0.85717E+00_JPRB, 0.93140E+00_JPRB, 0.99638E+00_JPRB, 0.10480E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10082E+01_JPRB, 0.95654E+00_JPRB, 0.93811E+00_JPRB, 0.92944E+00_JPRB, 0.91920E+00_JPRB, 0.88406E+00_JPRB ,&
&  0.85856E+00_JPRB, 0.81698E+00_JPRB, 0.76227E+00_JPRB, 0.71477E+00_JPRB, 0.70717E+00_JPRB, 0.72620E+00_JPRB, 0.75621E+00_JPRB ,&
&  0.76117E+00_JPRB, 0.76062E+00_JPRB, 0.76152E+00_JPRB, 0.77062E+00_JPRB, 0.70817E+00_JPRB, 0.66826E+00_JPRB, 0.64258E+00_JPRB ,&
&  0.63340E+00_JPRB, 0.62782E+00_JPRB, 0.62432E+00_JPRB, 0.63626E+00_JPRB, 0.63780E+00_JPRB, 0.63052E+00_JPRB, 0.60574E+00_JPRB ,&
&  0.57283E+00_JPRB, 0.54608E+00_JPRB, 0.52639E+00_JPRB, 0.51260E+00_JPRB, 0.50689E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.56143E+00_JPRB, 0.57039E+00_JPRB, 0.57425E+00_JPRB, 0.57619E+00_JPRB, 0.56857E+00_JPRB, 0.55606E+00_JPRB, 0.54307E+00_JPRB ,&
&  0.53942E+00_JPRB, 0.52746E+00_JPRB, 0.51548E+00_JPRB, 0.49912E+00_JPRB, 0.48474E+00_JPRB, 0.47273E+00_JPRB, 0.47035E+00_JPRB ,&
&  0.47258E+00_JPRB, 0.47496E+00_JPRB, 0.47252E+00_JPRB, 0.46554E+00_JPRB, 0.45872E+00_JPRB, 0.45506E+00_JPRB, 0.45537E+00_JPRB ,&
&  0.45755E+00_JPRB, 0.46067E+00_JPRB, 0.46206E+00_JPRB, 0.46090E+00_JPRB, 0.45952E+00_JPRB, 0.45699E+00_JPRB, 0.45255E+00_JPRB ,&
&  0.44259E+00_JPRB, 0.43541E+00_JPRB, 0.43096E+00_JPRB, 0.42804E+00_JPRB, 0.42924E+00_JPRB, 0.41372E+00_JPRB, 0.39209E+00_JPRB ,&
&  0.37224E+00_JPRB, 0.36549E+00_JPRB, 0.37683E+00_JPRB, 0.40615E+00_JPRB, 0.42783E+00_JPRB, 0.44589E+00_JPRB, 0.45856E+00_JPRB ,&
&  0.45777E+00_JPRB, 0.45595E+00_JPRB, 0.46285E+00_JPRB, 0.47935E+00_JPRB, 0.48230E+00_JPRB, 0.48310E+00_JPRB, 0.48046E+00_JPRB ,&
&  0.47614E+00_JPRB, 0.47518E+00_JPRB, 0.47477E+00_JPRB, 0.46416E+00_JPRB, 0.52340E+00_JPRB, 0.64642E+00_JPRB, 0.68936E+00_JPRB ,&
&  0.77943E+00_JPRB, 0.84464E+00_JPRB, 0.92834E+00_JPRB, 0.95188E+00_JPRB, 0.94101E+00_JPRB, 0.97347E+00_JPRB, 0.10500E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10381E+01_JPRB, 0.10043E+01_JPRB, 0.99162E+00_JPRB, 0.95205E+00_JPRB, 0.90996E+00_JPRB, 0.87138E+00_JPRB ,&
&  0.86281E+00_JPRB, 0.84210E+00_JPRB, 0.80006E+00_JPRB, 0.75118E+00_JPRB, 0.73311E+00_JPRB, 0.73084E+00_JPRB, 0.74439E+00_JPRB ,&
&  0.75128E+00_JPRB, 0.76490E+00_JPRB, 0.77396E+00_JPRB, 0.78334E+00_JPRB, 0.72902E+00_JPRB, 0.66458E+00_JPRB, 0.62550E+00_JPRB ,&
&  0.60469E+00_JPRB, 0.59165E+00_JPRB, 0.57847E+00_JPRB, 0.58730E+00_JPRB, 0.60066E+00_JPRB, 0.59848E+00_JPRB, 0.56902E+00_JPRB ,&
&  0.54505E+00_JPRB, 0.53152E+00_JPRB, 0.52575E+00_JPRB, 0.53811E+00_JPRB, 0.55235E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.59959E+00_JPRB, 0.61017E+00_JPRB, 0.60789E+00_JPRB, 0.60241E+00_JPRB, 0.60051E+00_JPRB, 0.58964E+00_JPRB, 0.56823E+00_JPRB ,&
&  0.56004E+00_JPRB, 0.53406E+00_JPRB, 0.52051E+00_JPRB, 0.50915E+00_JPRB, 0.49975E+00_JPRB, 0.48730E+00_JPRB, 0.47008E+00_JPRB ,&
&  0.46437E+00_JPRB, 0.46818E+00_JPRB, 0.47547E+00_JPRB, 0.47505E+00_JPRB, 0.47075E+00_JPRB, 0.46547E+00_JPRB, 0.46332E+00_JPRB ,&
&  0.46399E+00_JPRB, 0.46682E+00_JPRB, 0.47133E+00_JPRB, 0.46967E+00_JPRB, 0.46533E+00_JPRB, 0.46193E+00_JPRB, 0.46260E+00_JPRB ,&
&  0.45767E+00_JPRB, 0.44867E+00_JPRB, 0.44814E+00_JPRB, 0.44716E+00_JPRB, 0.44882E+00_JPRB, 0.44865E+00_JPRB, 0.42872E+00_JPRB ,&
&  0.40692E+00_JPRB, 0.41444E+00_JPRB, 0.44388E+00_JPRB, 0.46642E+00_JPRB, 0.48611E+00_JPRB, 0.49424E+00_JPRB, 0.49969E+00_JPRB ,&
&  0.50361E+00_JPRB, 0.50829E+00_JPRB, 0.51008E+00_JPRB, 0.51247E+00_JPRB, 0.51276E+00_JPRB, 0.51067E+00_JPRB, 0.50652E+00_JPRB ,&
&  0.50327E+00_JPRB, 0.47260E+00_JPRB, 0.46652E+00_JPRB, 0.49137E+00_JPRB, 0.63797E+00_JPRB, 0.67205E+00_JPRB, 0.71341E+00_JPRB ,&
&  0.81092E+00_JPRB, 0.87811E+00_JPRB, 0.96140E+00_JPRB, 0.10130E+01_JPRB, 0.10215E+01_JPRB, 0.10313E+01_JPRB, 0.10941E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.11251E+01_JPRB, 0.11000E+01_JPRB, 0.10591E+01_JPRB, 0.99217E+00_JPRB, 0.96274E+00_JPRB, 0.94285E+00_JPRB ,&
&  0.94809E+00_JPRB, 0.94322E+00_JPRB, 0.89889E+00_JPRB, 0.84336E+00_JPRB, 0.80407E+00_JPRB, 0.80385E+00_JPRB, 0.80960E+00_JPRB ,&
&  0.81224E+00_JPRB, 0.82515E+00_JPRB, 0.82044E+00_JPRB, 0.80744E+00_JPRB, 0.76979E+00_JPRB, 0.73470E+00_JPRB, 0.71777E+00_JPRB ,&
&  0.67879E+00_JPRB, 0.63595E+00_JPRB, 0.59091E+00_JPRB, 0.58413E+00_JPRB, 0.59396E+00_JPRB, 0.59071E+00_JPRB, 0.57104E+00_JPRB ,&
&  0.56371E+00_JPRB, 0.55627E+00_JPRB, 0.55357E+00_JPRB, 0.56969E+00_JPRB, 0.58248E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.61054E+00_JPRB, 0.61861E+00_JPRB, 0.61565E+00_JPRB, 0.60920E+00_JPRB, 0.61596E+00_JPRB, 0.60733E+00_JPRB, 0.60530E+00_JPRB ,&
&  0.61927E+00_JPRB, 0.60824E+00_JPRB, 0.62425E+00_JPRB, 0.55895E+00_JPRB, 0.51732E+00_JPRB, 0.50123E+00_JPRB, 0.48352E+00_JPRB ,&
&  0.47761E+00_JPRB, 0.48232E+00_JPRB, 0.48422E+00_JPRB, 0.48360E+00_JPRB, 0.48257E+00_JPRB, 0.47545E+00_JPRB, 0.47270E+00_JPRB ,&
&  0.47446E+00_JPRB, 0.47926E+00_JPRB, 0.48459E+00_JPRB, 0.48246E+00_JPRB, 0.47848E+00_JPRB, 0.47561E+00_JPRB, 0.47230E+00_JPRB ,&
&  0.46680E+00_JPRB, 0.46635E+00_JPRB, 0.46696E+00_JPRB, 0.46552E+00_JPRB, 0.46616E+00_JPRB, 0.46849E+00_JPRB, 0.46674E+00_JPRB ,&
&  0.47096E+00_JPRB, 0.47136E+00_JPRB, 0.50259E+00_JPRB, 0.52264E+00_JPRB, 0.53344E+00_JPRB, 0.53535E+00_JPRB, 0.53734E+00_JPRB ,&
&  0.54359E+00_JPRB, 0.55255E+00_JPRB, 0.56253E+00_JPRB, 0.57035E+00_JPRB, 0.57011E+00_JPRB, 0.56716E+00_JPRB, 0.56146E+00_JPRB ,&
&  0.55682E+00_JPRB, 0.50127E+00_JPRB, 0.55436E+00_JPRB, 0.58643E+00_JPRB, 0.69573E+00_JPRB, 0.82357E+00_JPRB, 0.83753E+00_JPRB ,&
&  0.10358E+01_JPRB, 0.10229E+01_JPRB, 0.10719E+01_JPRB, 0.10828E+01_JPRB, 0.10918E+01_JPRB, 0.11271E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.12829E+01_JPRB, 0.12273E+01_JPRB, 0.12416E+01_JPRB, 0.11535E+01_JPRB, 0.10930E+01_JPRB, 0.10893E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10811E+01_JPRB, 0.10601E+01_JPRB, 0.95046E+00_JPRB, 0.92092E+00_JPRB, 0.90834E+00_JPRB, 0.85637E+00_JPRB ,&
&  0.87166E+00_JPRB, 0.86556E+00_JPRB, 0.82627E+00_JPRB, 0.85901E+00_JPRB, 0.79640E+00_JPRB, 0.75977E+00_JPRB, 0.75939E+00_JPRB ,&
&  0.76369E+00_JPRB, 0.69096E+00_JPRB, 0.64511E+00_JPRB, 0.62833E+00_JPRB, 0.61080E+00_JPRB, 0.59530E+00_JPRB, 0.59361E+00_JPRB ,&
&  0.57675E+00_JPRB, 0.57898E+00_JPRB, 0.59998E+00_JPRB, 0.60119E+00_JPRB, 0.60065E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.62255E+00_JPRB, 0.62035E+00_JPRB, 0.61614E+00_JPRB, 0.61438E+00_JPRB, 0.63488E+00_JPRB, 0.64333E+00_JPRB, 0.64388E+00_JPRB ,&
&  0.65971E+00_JPRB, 0.65579E+00_JPRB, 0.65233E+00_JPRB, 0.64003E+00_JPRB, 0.62345E+00_JPRB, 0.57589E+00_JPRB, 0.50680E+00_JPRB ,&
&  0.48260E+00_JPRB, 0.48473E+00_JPRB, 0.49236E+00_JPRB, 0.49306E+00_JPRB, 0.48828E+00_JPRB, 0.48239E+00_JPRB, 0.48034E+00_JPRB ,&
&  0.48522E+00_JPRB, 0.49129E+00_JPRB, 0.49780E+00_JPRB, 0.49945E+00_JPRB, 0.49641E+00_JPRB, 0.49494E+00_JPRB, 0.49245E+00_JPRB ,&
&  0.48430E+00_JPRB, 0.48313E+00_JPRB, 0.48252E+00_JPRB, 0.47889E+00_JPRB, 0.49385E+00_JPRB, 0.50198E+00_JPRB, 0.50207E+00_JPRB ,&
&  0.51949E+00_JPRB, 0.54296E+00_JPRB, 0.55944E+00_JPRB, 0.57483E+00_JPRB, 0.57731E+00_JPRB, 0.57871E+00_JPRB, 0.58799E+00_JPRB ,&
&  0.60491E+00_JPRB, 0.62085E+00_JPRB, 0.63583E+00_JPRB, 0.64689E+00_JPRB, 0.65040E+00_JPRB, 0.63698E+00_JPRB, 0.65441E+00_JPRB ,&
&  0.65574E+00_JPRB, 0.68034E+00_JPRB, 0.74269E+00_JPRB, 0.74961E+00_JPRB, 0.84930E+00_JPRB, 0.93648E+00_JPRB, 0.94842E+00_JPRB ,&
&  0.10465E+01_JPRB, 0.10417E+01_JPRB, 0.11703E+01_JPRB, 0.12655E+01_JPRB, 0.13525E+01_JPRB, 0.13931E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.15652E+01_JPRB, 0.16377E+01_JPRB, 0.14519E+01_JPRB, 0.13585E+01_JPRB, 0.12822E+01_JPRB, 0.12832E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.12449E+01_JPRB, 0.12723E+01_JPRB, 0.12515E+01_JPRB, 0.12163E+01_JPRB, 0.11423E+01_JPRB, 0.10614E+01_JPRB, 0.98265E+00_JPRB ,&
&  0.93821E+00_JPRB, 0.85144E+00_JPRB, 0.80704E+00_JPRB, 0.81529E+00_JPRB, 0.81378E+00_JPRB, 0.74804E+00_JPRB, 0.77356E+00_JPRB ,&
&  0.81942E+00_JPRB, 0.76961E+00_JPRB, 0.71204E+00_JPRB, 0.66675E+00_JPRB, 0.64932E+00_JPRB, 0.62971E+00_JPRB, 0.59457E+00_JPRB ,&
&  0.56994E+00_JPRB, 0.59900E+00_JPRB, 0.61923E+00_JPRB, 0.62083E+00_JPRB, 0.62082E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.63911E+00_JPRB, 0.62859E+00_JPRB, 0.61646E+00_JPRB, 0.62145E+00_JPRB, 0.64595E+00_JPRB, 0.65608E+00_JPRB, 0.67269E+00_JPRB ,&
&  0.68229E+00_JPRB, 0.68337E+00_JPRB, 0.67878E+00_JPRB, 0.65848E+00_JPRB, 0.64599E+00_JPRB, 0.63738E+00_JPRB, 0.59556E+00_JPRB ,&
&  0.54065E+00_JPRB, 0.52256E+00_JPRB, 0.50657E+00_JPRB, 0.50257E+00_JPRB, 0.50097E+00_JPRB, 0.49402E+00_JPRB, 0.49064E+00_JPRB ,&
&  0.49691E+00_JPRB, 0.50093E+00_JPRB, 0.50515E+00_JPRB, 0.50826E+00_JPRB, 0.50942E+00_JPRB, 0.51230E+00_JPRB, 0.51123E+00_JPRB ,&
&  0.50160E+00_JPRB, 0.50065E+00_JPRB, 0.50426E+00_JPRB, 0.50209E+00_JPRB, 0.52373E+00_JPRB, 0.54531E+00_JPRB, 0.55998E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.60357E+00_JPRB, 0.61405E+00_JPRB, 0.62199E+00_JPRB, 0.62254E+00_JPRB, 0.63457E+00_JPRB, 0.65750E+00_JPRB ,&
&  0.68689E+00_JPRB, 0.70708E+00_JPRB, 0.72243E+00_JPRB, 0.73842E+00_JPRB, 0.73299E+00_JPRB, 0.74471E+00_JPRB, 0.79442E+00_JPRB ,&
&  0.85081E+00_JPRB, 0.92084E+00_JPRB, 0.94910E+00_JPRB, 0.10028E+01_JPRB, 0.11065E+01_JPRB, 0.11536E+01_JPRB, 0.11284E+01_JPRB ,&
&  0.11895E+01_JPRB, 0.11672E+01_JPRB, 0.15272E+01_JPRB, 0.15836E+01_JPRB, 0.15778E+01_JPRB, 0.16494E+01_JPRB, 0.16298E+01_JPRB ,&
&  0.16406E+01_JPRB, 0.15638E+01_JPRB, 0.14971E+01_JPRB, 0.14600E+01_JPRB, 0.14372E+01_JPRB, 0.15082E+01_JPRB, 0.14526E+01_JPRB ,&
&  0.14713E+01_JPRB, 0.14641E+01_JPRB, 0.12817E+01_JPRB, 0.11319E+01_JPRB, 0.98045E+00_JPRB, 0.10204E+01_JPRB, 0.97494E+00_JPRB ,&
&  0.92585E+00_JPRB, 0.80278E+00_JPRB, 0.83651E+00_JPRB, 0.85735E+00_JPRB, 0.87244E+00_JPRB, 0.90660E+00_JPRB, 0.91770E+00_JPRB ,&
&  0.92840E+00_JPRB, 0.93467E+00_JPRB, 0.83082E+00_JPRB, 0.75256E+00_JPRB, 0.71452E+00_JPRB, 0.67898E+00_JPRB, 0.62030E+00_JPRB ,&
&  0.60399E+00_JPRB, 0.63361E+00_JPRB, 0.65443E+00_JPRB, 0.65730E+00_JPRB, 0.64940E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.66653E+00_JPRB, 0.65167E+00_JPRB, 0.63230E+00_JPRB, 0.64159E+00_JPRB, 0.66160E+00_JPRB, 0.66837E+00_JPRB, 0.68679E+00_JPRB ,&
&  0.70568E+00_JPRB, 0.70614E+00_JPRB, 0.68515E+00_JPRB, 0.66810E+00_JPRB, 0.65802E+00_JPRB, 0.65316E+00_JPRB, 0.65220E+00_JPRB ,&
&  0.59257E+00_JPRB, 0.55454E+00_JPRB, 0.51767E+00_JPRB, 0.51049E+00_JPRB, 0.53344E+00_JPRB, 0.52056E+00_JPRB, 0.51747E+00_JPRB ,&
&  0.51393E+00_JPRB, 0.51745E+00_JPRB, 0.51454E+00_JPRB, 0.51426E+00_JPRB, 0.51629E+00_JPRB, 0.52159E+00_JPRB, 0.52690E+00_JPRB ,&
&  0.52205E+00_JPRB, 0.52988E+00_JPRB, 0.54327E+00_JPRB, 0.54563E+00_JPRB, 0.57080E+00_JPRB, 0.59394E+00_JPRB, 0.62419E+00_JPRB ,&
&  0.65161E+00_JPRB, 0.65774E+00_JPRB, 0.65869E+00_JPRB, 0.66858E+00_JPRB, 0.68286E+00_JPRB, 0.71494E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.80027E+00_JPRB, 0.83343E+00_JPRB, 0.85515E+00_JPRB, 0.86640E+00_JPRB, 0.85236E+00_JPRB, 0.88566E+00_JPRB, 0.96599E+00_JPRB ,&
&  0.10993E+01_JPRB, 0.11465E+01_JPRB, 0.11548E+01_JPRB, 0.14310E+01_JPRB, 0.16090E+01_JPRB, 0.16083E+01_JPRB, 0.14721E+01_JPRB ,&
&  0.13997E+01_JPRB, 0.13296E+01_JPRB, 0.14702E+01_JPRB, 0.15896E+01_JPRB, 0.15743E+01_JPRB, 0.17131E+01_JPRB, 0.16649E+01_JPRB ,&
&  0.15301E+01_JPRB, 0.14332E+01_JPRB, 0.13846E+01_JPRB, 0.14425E+01_JPRB, 0.14469E+01_JPRB, 0.15483E+01_JPRB, 0.15597E+01_JPRB ,&
&  0.15676E+01_JPRB, 0.13707E+01_JPRB, 0.10443E+01_JPRB, 0.96600E+00_JPRB, 0.78078E+00_JPRB, 0.79083E+00_JPRB, 0.95237E+00_JPRB ,&
&  0.94910E+00_JPRB, 0.94784E+00_JPRB, 0.94306E+00_JPRB, 0.97280E+00_JPRB, 0.99205E+00_JPRB, 0.11617E+01_JPRB, 0.12224E+01_JPRB ,&
&  0.10883E+01_JPRB, 0.10830E+01_JPRB, 0.10319E+01_JPRB, 0.96000E+00_JPRB, 0.88393E+00_JPRB, 0.79326E+00_JPRB, 0.72338E+00_JPRB ,&
&  0.70196E+00_JPRB, 0.69653E+00_JPRB, 0.70374E+00_JPRB, 0.69463E+00_JPRB, 0.67672E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.71093E+00_JPRB, 0.69684E+00_JPRB, 0.66927E+00_JPRB, 0.67273E+00_JPRB, 0.68067E+00_JPRB, 0.68953E+00_JPRB, 0.71084E+00_JPRB ,&
&  0.73005E+00_JPRB, 0.71648E+00_JPRB, 0.68937E+00_JPRB, 0.67913E+00_JPRB, 0.67352E+00_JPRB, 0.66563E+00_JPRB, 0.66577E+00_JPRB ,&
&  0.67185E+00_JPRB, 0.63175E+00_JPRB, 0.57558E+00_JPRB, 0.53182E+00_JPRB, 0.54225E+00_JPRB, 0.53972E+00_JPRB, 0.54862E+00_JPRB ,&
&  0.54325E+00_JPRB, 0.55575E+00_JPRB, 0.54539E+00_JPRB, 0.54080E+00_JPRB, 0.53929E+00_JPRB, 0.54628E+00_JPRB, 0.56118E+00_JPRB ,&
&  0.57950E+00_JPRB, 0.61201E+00_JPRB, 0.64571E+00_JPRB, 0.65415E+00_JPRB, 0.68056E+00_JPRB, 0.66564E+00_JPRB, 0.69619E+00_JPRB ,&
&  0.71780E+00_JPRB, 0.71624E+00_JPRB, 0.71553E+00_JPRB, 0.73553E+00_JPRB, 0.77423E+00_JPRB, 0.82667E+00_JPRB, 0.89466E+00_JPRB ,&
&  0.96265E+00_JPRB, 0.99350E+00_JPRB, 0.97687E+00_JPRB, 0.96495E+00_JPRB, 0.99456E+00_JPRB, 0.10656E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.11885E+01_JPRB, 0.11454E+01_JPRB, 0.15162E+01_JPRB, 0.18463E+01_JPRB, 0.19005E+01_JPRB, 0.18067E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.16199E+01_JPRB, 0.17075E+01_JPRB, 0.17502E+01_JPRB, 0.17585E+01_JPRB, 0.17586E+01_JPRB, 0.16613E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.13915E+01_JPRB, 0.14084E+01_JPRB, 0.14202E+01_JPRB, 0.14552E+01_JPRB, 0.14236E+01_JPRB, 0.14857E+01_JPRB ,&
&  0.15427E+01_JPRB, 0.93197E+00_JPRB, 0.77429E+00_JPRB, 0.88246E+00_JPRB, 0.76588E+00_JPRB, 0.78194E+00_JPRB, 0.87318E+00_JPRB ,&
&  0.86239E+00_JPRB, 0.86746E+00_JPRB, 0.97232E+00_JPRB, 0.10802E+01_JPRB, 0.12179E+01_JPRB, 0.14569E+01_JPRB, 0.14607E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.12565E+01_JPRB, 0.12771E+01_JPRB, 0.12408E+01_JPRB, 0.11322E+01_JPRB, 0.97857E+00_JPRB, 0.85986E+00_JPRB ,&
&  0.79953E+00_JPRB, 0.78030E+00_JPRB, 0.77190E+00_JPRB, 0.75119E+00_JPRB, 0.72184E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.77815E+00_JPRB, 0.75798E+00_JPRB, 0.70986E+00_JPRB, 0.69937E+00_JPRB, 0.71164E+00_JPRB, 0.73513E+00_JPRB, 0.75686E+00_JPRB ,&
&  0.75685E+00_JPRB, 0.72959E+00_JPRB, 0.70892E+00_JPRB, 0.70120E+00_JPRB, 0.69042E+00_JPRB, 0.67548E+00_JPRB, 0.66881E+00_JPRB ,&
&  0.66698E+00_JPRB, 0.66087E+00_JPRB, 0.60590E+00_JPRB, 0.57723E+00_JPRB, 0.54573E+00_JPRB, 0.55241E+00_JPRB, 0.56389E+00_JPRB ,&
&  0.57925E+00_JPRB, 0.57721E+00_JPRB, 0.57039E+00_JPRB, 0.57341E+00_JPRB, 0.57662E+00_JPRB, 0.58905E+00_JPRB, 0.64159E+00_JPRB ,&
&  0.70200E+00_JPRB, 0.76621E+00_JPRB, 0.77603E+00_JPRB, 0.80259E+00_JPRB, 0.78489E+00_JPRB, 0.75079E+00_JPRB, 0.79371E+00_JPRB ,&
&  0.80154E+00_JPRB, 0.79333E+00_JPRB, 0.79139E+00_JPRB, 0.82158E+00_JPRB, 0.87330E+00_JPRB, 0.91775E+00_JPRB, 0.98332E+00_JPRB ,&
&  0.10301E+01_JPRB, 0.10464E+01_JPRB, 0.10330E+01_JPRB, 0.10048E+01_JPRB, 0.10504E+01_JPRB, 0.10826E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.96018E+00_JPRB, 0.81367E+00_JPRB, 0.11943E+01_JPRB, 0.18715E+01_JPRB, 0.19973E+01_JPRB, 0.17391E+01_JPRB ,&
&  0.17945E+01_JPRB, 0.17844E+01_JPRB, 0.19064E+01_JPRB, 0.20191E+01_JPRB, 0.19820E+01_JPRB, 0.18464E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.14763E+01_JPRB, 0.15200E+01_JPRB, 0.17399E+01_JPRB, 0.17822E+01_JPRB, 0.17813E+01_JPRB, 0.17370E+01_JPRB ,&
&  0.14865E+01_JPRB, 0.12752E+01_JPRB, 0.92648E+00_JPRB, 0.94543E+00_JPRB, 0.76511E+00_JPRB, 0.76961E+00_JPRB, 0.85883E+00_JPRB ,&
&  0.84137E+00_JPRB, 0.89848E+00_JPRB, 0.10149E+01_JPRB, 0.11455E+01_JPRB, 0.15318E+01_JPRB, 0.17935E+01_JPRB, 0.18705E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.15705E+01_JPRB, 0.16017E+01_JPRB, 0.14829E+01_JPRB, 0.13820E+01_JPRB, 0.12153E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.94682E+00_JPRB, 0.90430E+00_JPRB, 0.87805E+00_JPRB, 0.84078E+00_JPRB, 0.79664E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.81997E+00_JPRB, 0.77768E+00_JPRB, 0.70942E+00_JPRB, 0.70085E+00_JPRB, 0.72212E+00_JPRB, 0.76973E+00_JPRB, 0.79480E+00_JPRB ,&
&  0.77796E+00_JPRB, 0.74774E+00_JPRB, 0.72711E+00_JPRB, 0.71613E+00_JPRB, 0.69744E+00_JPRB, 0.67431E+00_JPRB, 0.66427E+00_JPRB ,&
&  0.65771E+00_JPRB, 0.64044E+00_JPRB, 0.59310E+00_JPRB, 0.56415E+00_JPRB, 0.53820E+00_JPRB, 0.53660E+00_JPRB, 0.55229E+00_JPRB ,&
&  0.56850E+00_JPRB, 0.58214E+00_JPRB, 0.60183E+00_JPRB, 0.61629E+00_JPRB, 0.62840E+00_JPRB, 0.67543E+00_JPRB, 0.77128E+00_JPRB ,&
&  0.94256E+00_JPRB, 0.89840E+00_JPRB, 0.91846E+00_JPRB, 0.88212E+00_JPRB, 0.86487E+00_JPRB, 0.85662E+00_JPRB, 0.84086E+00_JPRB ,&
&  0.80629E+00_JPRB, 0.78389E+00_JPRB, 0.79224E+00_JPRB, 0.82684E+00_JPRB, 0.91063E+00_JPRB, 0.99181E+00_JPRB, 0.10973E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.11217E+01_JPRB, 0.10609E+01_JPRB, 0.10469E+01_JPRB, 0.11066E+01_JPRB, 0.11040E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.92147E+00_JPRB, 0.82645E+00_JPRB, 0.94169E+00_JPRB, 0.12183E+01_JPRB, 0.16764E+01_JPRB, 0.20179E+01_JPRB, 0.19368E+01_JPRB ,&
&  0.19545E+01_JPRB, 0.19270E+01_JPRB, 0.19576E+01_JPRB, 0.16370E+01_JPRB, 0.18129E+01_JPRB, 0.17315E+01_JPRB, 0.16325E+01_JPRB ,&
&  0.15519E+01_JPRB, 0.15719E+01_JPRB, 0.17275E+01_JPRB, 0.20008E+01_JPRB, 0.21760E+01_JPRB, 0.22764E+01_JPRB, 0.17570E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.15039E+01_JPRB, 0.13672E+01_JPRB, 0.11476E+01_JPRB, 0.97919E+00_JPRB, 0.99733E+00_JPRB, 0.98731E+00_JPRB ,&
&  0.98596E+00_JPRB, 0.10095E+01_JPRB, 0.11339E+01_JPRB, 0.14389E+01_JPRB, 0.19622E+01_JPRB, 0.23409E+01_JPRB, 0.23422E+01_JPRB ,&
&  0.20911E+01_JPRB, 0.21228E+01_JPRB, 0.19442E+01_JPRB, 0.17018E+01_JPRB, 0.15439E+01_JPRB, 0.14028E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.10556E+01_JPRB, 0.10033E+01_JPRB, 0.93800E+00_JPRB, 0.86200E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.85629E+00_JPRB, 0.81048E+00_JPRB, 0.74353E+00_JPRB, 0.73630E+00_JPRB, 0.77470E+00_JPRB, 0.82324E+00_JPRB, 0.82930E+00_JPRB ,&
&  0.78167E+00_JPRB, 0.75073E+00_JPRB, 0.72923E+00_JPRB, 0.72635E+00_JPRB, 0.71363E+00_JPRB, 0.68641E+00_JPRB, 0.66599E+00_JPRB ,&
&  0.66463E+00_JPRB, 0.62977E+00_JPRB, 0.58100E+00_JPRB, 0.57570E+00_JPRB, 0.56503E+00_JPRB, 0.53242E+00_JPRB, 0.53855E+00_JPRB ,&
&  0.55883E+00_JPRB, 0.58573E+00_JPRB, 0.61926E+00_JPRB, 0.66866E+00_JPRB, 0.73686E+00_JPRB, 0.84067E+00_JPRB, 0.96832E+00_JPRB ,&
&  0.99444E+00_JPRB, 0.10014E+01_JPRB, 0.10239E+01_JPRB, 0.98202E+00_JPRB, 0.90247E+00_JPRB, 0.85439E+00_JPRB, 0.82987E+00_JPRB ,&
&  0.79926E+00_JPRB, 0.78347E+00_JPRB, 0.82204E+00_JPRB, 0.91178E+00_JPRB, 0.10430E+01_JPRB, 0.11342E+01_JPRB, 0.12022E+01_JPRB ,&
&  0.11748E+01_JPRB, 0.11656E+01_JPRB, 0.11495E+01_JPRB, 0.12339E+01_JPRB, 0.97620E+00_JPRB, 0.93046E+00_JPRB, 0.89561E+00_JPRB ,&
&  0.10054E+01_JPRB, 0.96718E+00_JPRB, 0.98676E+00_JPRB, 0.12690E+01_JPRB, 0.12961E+01_JPRB, 0.15759E+01_JPRB, 0.19020E+01_JPRB ,&
&  0.20700E+01_JPRB, 0.20973E+01_JPRB, 0.20895E+01_JPRB, 0.13384E+01_JPRB, 0.12468E+01_JPRB, 0.16213E+01_JPRB, 0.17043E+01_JPRB ,&
&  0.17026E+01_JPRB, 0.18825E+01_JPRB, 0.21187E+01_JPRB, 0.24993E+01_JPRB, 0.19119E+01_JPRB, 0.16591E+01_JPRB, 0.13152E+01_JPRB ,&
&  0.13061E+01_JPRB, 0.12249E+01_JPRB, 0.14454E+01_JPRB, 0.13266E+01_JPRB, 0.13469E+01_JPRB, 0.12959E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.12920E+01_JPRB, 0.14919E+01_JPRB, 0.20013E+01_JPRB, 0.26218E+01_JPRB, 0.26219E+01_JPRB, 0.28911E+01_JPRB ,&
&  0.28423E+01_JPRB, 0.25268E+01_JPRB, 0.20828E+01_JPRB, 0.18771E+01_JPRB, 0.17216E+01_JPRB, 0.15960E+01_JPRB, 0.14575E+01_JPRB ,&
&  0.13096E+01_JPRB, 0.11974E+01_JPRB, 0.11020E+01_JPRB, 0.10037E+01_JPRB, 0.91219E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.92652E+00_JPRB, 0.87816E+00_JPRB, 0.79745E+00_JPRB, 0.79839E+00_JPRB, 0.84489E+00_JPRB, 0.86649E+00_JPRB, 0.82897E+00_JPRB ,&
&  0.78957E+00_JPRB, 0.75268E+00_JPRB, 0.73980E+00_JPRB, 0.73147E+00_JPRB, 0.71657E+00_JPRB, 0.70866E+00_JPRB, 0.68730E+00_JPRB ,&
&  0.69430E+00_JPRB, 0.68045E+00_JPRB, 0.61652E+00_JPRB, 0.58353E+00_JPRB, 0.58714E+00_JPRB, 0.55183E+00_JPRB, 0.53008E+00_JPRB ,&
&  0.56873E+00_JPRB, 0.61699E+00_JPRB, 0.68530E+00_JPRB, 0.77321E+00_JPRB, 0.88037E+00_JPRB, 0.10665E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.10821E+01_JPRB, 0.99024E+00_JPRB, 0.90151E+00_JPRB, 0.82861E+00_JPRB, 0.79036E+00_JPRB, 0.77139E+00_JPRB ,&
&  0.76736E+00_JPRB, 0.78784E+00_JPRB, 0.87036E+00_JPRB, 0.99567E+00_JPRB, 0.11297E+01_JPRB, 0.11880E+01_JPRB, 0.11933E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.13079E+01_JPRB, 0.13182E+01_JPRB, 0.12528E+01_JPRB, 0.11299E+01_JPRB, 0.95887E+00_JPRB, 0.95543E+00_JPRB ,&
&  0.10058E+01_JPRB, 0.94272E+00_JPRB, 0.99802E+00_JPRB, 0.12582E+01_JPRB, 0.14757E+01_JPRB, 0.15819E+01_JPRB, 0.19217E+01_JPRB ,&
&  0.16020E+01_JPRB, 0.13165E+01_JPRB, 0.11252E+01_JPRB, 0.93959E+00_JPRB, 0.13377E+01_JPRB, 0.18944E+01_JPRB, 0.20968E+01_JPRB ,&
&  0.21871E+01_JPRB, 0.23866E+01_JPRB, 0.26632E+01_JPRB, 0.24134E+01_JPRB, 0.17659E+01_JPRB, 0.14628E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.19255E+01_JPRB, 0.20452E+01_JPRB, 0.20023E+01_JPRB, 0.17343E+01_JPRB, 0.14463E+01_JPRB, 0.16398E+01_JPRB, 0.16214E+01_JPRB ,&
&  0.16803E+01_JPRB, 0.17577E+01_JPRB, 0.23348E+01_JPRB, 0.30087E+01_JPRB, 0.32866E+01_JPRB, 0.35722E+01_JPRB, 0.37344E+01_JPRB ,&
&  0.32656E+01_JPRB, 0.28805E+01_JPRB, 0.24273E+01_JPRB, 0.20784E+01_JPRB, 0.18545E+01_JPRB, 0.17105E+01_JPRB, 0.15953E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.13489E+01_JPRB, 0.12342E+01_JPRB, 0.10917E+01_JPRB, 0.98448E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.10197E+01_JPRB, 0.94980E+00_JPRB, 0.85772E+00_JPRB, 0.85649E+00_JPRB, 0.90378E+00_JPRB, 0.88446E+00_JPRB, 0.82532E+00_JPRB ,&
&  0.80361E+00_JPRB, 0.76835E+00_JPRB, 0.75471E+00_JPRB, 0.73828E+00_JPRB, 0.71966E+00_JPRB, 0.70710E+00_JPRB, 0.70594E+00_JPRB ,&
&  0.73519E+00_JPRB, 0.78097E+00_JPRB, 0.70425E+00_JPRB, 0.65255E+00_JPRB, 0.62937E+00_JPRB, 0.60523E+00_JPRB, 0.55339E+00_JPRB ,&
&  0.61096E+00_JPRB, 0.70246E+00_JPRB, 0.79603E+00_JPRB, 0.91722E+00_JPRB, 0.10816E+01_JPRB, 0.11300E+01_JPRB, 0.11335E+01_JPRB ,&
&  0.11080E+01_JPRB, 0.10349E+01_JPRB, 0.93920E+00_JPRB, 0.85014E+00_JPRB, 0.80026E+00_JPRB, 0.78217E+00_JPRB, 0.77518E+00_JPRB ,&
&  0.79692E+00_JPRB, 0.87040E+00_JPRB, 0.96752E+00_JPRB, 0.10873E+01_JPRB, 0.12211E+01_JPRB, 0.12855E+01_JPRB, 0.13176E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.14568E+01_JPRB, 0.14664E+01_JPRB, 0.14162E+01_JPRB, 0.13702E+01_JPRB, 0.11761E+01_JPRB, 0.13345E+01_JPRB ,&
&  0.12856E+01_JPRB, 0.13461E+01_JPRB, 0.12768E+01_JPRB, 0.15491E+01_JPRB, 0.15256E+01_JPRB, 0.15249E+01_JPRB, 0.17245E+01_JPRB ,&
&  0.13503E+01_JPRB, 0.12523E+01_JPRB, 0.14363E+01_JPRB, 0.16433E+01_JPRB, 0.17683E+01_JPRB, 0.19867E+01_JPRB, 0.29093E+01_JPRB ,&
&  0.24505E+01_JPRB, 0.28046E+01_JPRB, 0.30720E+01_JPRB, 0.29802E+01_JPRB, 0.13649E+01_JPRB, 0.10837E+01_JPRB, 0.17578E+01_JPRB ,&
&  0.19811E+01_JPRB, 0.13647E+01_JPRB, 0.10417E+01_JPRB, 0.13113E+01_JPRB, 0.10303E+01_JPRB, 0.12358E+01_JPRB, 0.17629E+01_JPRB ,&
&  0.20673E+01_JPRB, 0.24581E+01_JPRB, 0.35863E+01_JPRB, 0.38492E+01_JPRB, 0.37271E+01_JPRB, 0.46130E+01_JPRB, 0.38787E+01_JPRB ,&
&  0.34157E+01_JPRB, 0.26652E+01_JPRB, 0.24808E+01_JPRB, 0.21868E+01_JPRB, 0.19719E+01_JPRB, 0.18435E+01_JPRB, 0.17259E+01_JPRB ,&
&  0.15894E+01_JPRB, 0.14667E+01_JPRB, 0.13423E+01_JPRB, 0.12076E+01_JPRB, 0.11013E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.10853E+01_JPRB, 0.10076E+01_JPRB, 0.92829E+00_JPRB, 0.91485E+00_JPRB, 0.94033E+00_JPRB, 0.87659E+00_JPRB, 0.83128E+00_JPRB ,&
&  0.79439E+00_JPRB, 0.77536E+00_JPRB, 0.75566E+00_JPRB, 0.73149E+00_JPRB, 0.71094E+00_JPRB, 0.71272E+00_JPRB, 0.72672E+00_JPRB ,&
&  0.75793E+00_JPRB, 0.83752E+00_JPRB, 0.88357E+00_JPRB, 0.83828E+00_JPRB, 0.80321E+00_JPRB, 0.74988E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.75311E+00_JPRB, 0.88660E+00_JPRB, 0.96517E+00_JPRB, 0.10124E+01_JPRB, 0.10438E+01_JPRB, 0.10183E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.96305E+00_JPRB, 0.89306E+00_JPRB, 0.83456E+00_JPRB, 0.80808E+00_JPRB, 0.80240E+00_JPRB, 0.83328E+00_JPRB ,&
&  0.88907E+00_JPRB, 0.95395E+00_JPRB, 0.10322E+01_JPRB, 0.11850E+01_JPRB, 0.13394E+01_JPRB, 0.14047E+01_JPRB, 0.14614E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.15459E+01_JPRB, 0.15590E+01_JPRB, 0.15873E+01_JPRB, 0.17120E+01_JPRB, 0.15431E+01_JPRB, 0.13078E+01_JPRB ,&
&  0.11938E+01_JPRB, 0.13560E+01_JPRB, 0.18569E+01_JPRB, 0.17420E+01_JPRB, 0.15265E+01_JPRB, 0.15738E+01_JPRB, 0.15664E+01_JPRB ,&
&  0.15315E+01_JPRB, 0.17940E+01_JPRB, 0.20230E+01_JPRB, 0.22915E+01_JPRB, 0.24814E+01_JPRB, 0.20056E+01_JPRB, 0.28182E+01_JPRB ,&
&  0.30424E+01_JPRB, 0.29779E+01_JPRB, 0.22392E+01_JPRB, 0.15543E+01_JPRB, 0.16028E+01_JPRB, 0.12511E+01_JPRB, 0.89510E+00_JPRB ,&
&  0.84947E+00_JPRB, 0.81786E+00_JPRB, 0.75749E+00_JPRB, 0.80465E+00_JPRB, 0.82456E+00_JPRB, 0.11552E+01_JPRB, 0.17825E+01_JPRB ,&
&  0.27257E+01_JPRB, 0.36031E+01_JPRB, 0.41888E+01_JPRB, 0.42080E+01_JPRB, 0.39987E+01_JPRB, 0.46490E+01_JPRB, 0.40060E+01_JPRB ,&
&  0.28584E+01_JPRB, 0.24281E+01_JPRB, 0.22768E+01_JPRB, 0.21824E+01_JPRB, 0.20930E+01_JPRB, 0.20004E+01_JPRB, 0.18553E+01_JPRB ,&
&  0.16872E+01_JPRB, 0.15643E+01_JPRB, 0.14502E+01_JPRB, 0.13016E+01_JPRB, 0.11836E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.11393E+01_JPRB, 0.10661E+01_JPRB, 0.10248E+01_JPRB, 0.98924E+00_JPRB, 0.96399E+00_JPRB, 0.87224E+00_JPRB, 0.83520E+00_JPRB ,&
&  0.78389E+00_JPRB, 0.76775E+00_JPRB, 0.73558E+00_JPRB, 0.71754E+00_JPRB, 0.70628E+00_JPRB, 0.72436E+00_JPRB, 0.75997E+00_JPRB ,&
&  0.80897E+00_JPRB, 0.93728E+00_JPRB, 0.11069E+01_JPRB, 0.11845E+01_JPRB, 0.12577E+01_JPRB, 0.12163E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.10786E+01_JPRB, 0.11856E+01_JPRB, 0.11372E+01_JPRB, 0.10886E+01_JPRB, 0.10625E+01_JPRB, 0.10226E+01_JPRB, 0.99208E+00_JPRB ,&
&  0.97213E+00_JPRB, 0.91630E+00_JPRB, 0.85195E+00_JPRB, 0.82352E+00_JPRB, 0.82455E+00_JPRB, 0.84823E+00_JPRB, 0.89030E+00_JPRB ,&
&  0.96536E+00_JPRB, 0.10072E+01_JPRB, 0.11155E+01_JPRB, 0.12924E+01_JPRB, 0.14330E+01_JPRB, 0.14857E+01_JPRB, 0.15630E+01_JPRB ,&
&  0.16333E+01_JPRB, 0.16323E+01_JPRB, 0.16111E+01_JPRB, 0.15686E+01_JPRB, 0.13692E+01_JPRB, 0.11487E+01_JPRB, 0.13667E+01_JPRB ,&
&  0.15573E+01_JPRB, 0.19058E+01_JPRB, 0.20869E+01_JPRB, 0.21504E+01_JPRB, 0.16933E+01_JPRB, 0.15879E+01_JPRB, 0.16098E+01_JPRB ,&
&  0.18175E+01_JPRB, 0.28036E+01_JPRB, 0.25456E+01_JPRB, 0.23590E+01_JPRB, 0.29676E+01_JPRB, 0.24901E+01_JPRB, 0.26313E+01_JPRB ,&
&  0.31610E+01_JPRB, 0.29730E+01_JPRB, 0.23130E+01_JPRB, 0.17994E+01_JPRB, 0.23329E+01_JPRB, 0.24238E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.93749E+00_JPRB, 0.87389E+00_JPRB, 0.82650E+00_JPRB, 0.79248E+00_JPRB, 0.81645E+00_JPRB, 0.12138E+01_JPRB, 0.23302E+01_JPRB ,&
&  0.32445E+01_JPRB, 0.44640E+01_JPRB, 0.46482E+01_JPRB, 0.44932E+01_JPRB, 0.41107E+01_JPRB, 0.41554E+01_JPRB, 0.33894E+01_JPRB ,&
&  0.28648E+01_JPRB, 0.25381E+01_JPRB, 0.23382E+01_JPRB, 0.22388E+01_JPRB, 0.21446E+01_JPRB, 0.20487E+01_JPRB, 0.19195E+01_JPRB ,&
&  0.17650E+01_JPRB, 0.16348E+01_JPRB, 0.15146E+01_JPRB, 0.13595E+01_JPRB, 0.12324E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.11754E+01_JPRB, 0.11240E+01_JPRB, 0.11039E+01_JPRB, 0.10710E+01_JPRB, 0.99570E+00_JPRB, 0.90284E+00_JPRB, 0.82402E+00_JPRB ,&
&  0.78009E+00_JPRB, 0.75121E+00_JPRB, 0.72853E+00_JPRB, 0.71881E+00_JPRB, 0.72128E+00_JPRB, 0.74976E+00_JPRB, 0.79443E+00_JPRB ,&
&  0.89126E+00_JPRB, 0.10573E+01_JPRB, 0.12419E+01_JPRB, 0.15796E+01_JPRB, 0.17041E+01_JPRB, 0.15975E+01_JPRB, 0.17198E+01_JPRB ,&
&  0.16625E+01_JPRB, 0.15769E+01_JPRB, 0.13935E+01_JPRB, 0.12285E+01_JPRB, 0.11250E+01_JPRB, 0.10659E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.94462E+00_JPRB, 0.90546E+00_JPRB, 0.85872E+00_JPRB, 0.87089E+00_JPRB, 0.89206E+00_JPRB, 0.91773E+00_JPRB, 0.97014E+00_JPRB ,&
&  0.99659E+00_JPRB, 0.10366E+01_JPRB, 0.11843E+01_JPRB, 0.13533E+01_JPRB, 0.14729E+01_JPRB, 0.15640E+01_JPRB, 0.16722E+01_JPRB ,&
&  0.17026E+01_JPRB, 0.16923E+01_JPRB, 0.16400E+01_JPRB, 0.15643E+01_JPRB, 0.12891E+01_JPRB, 0.14551E+01_JPRB, 0.15114E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.18553E+01_JPRB, 0.19177E+01_JPRB, 0.20247E+01_JPRB, 0.18056E+01_JPRB, 0.16391E+01_JPRB, 0.16275E+01_JPRB ,&
&  0.22138E+01_JPRB, 0.23662E+01_JPRB, 0.21407E+01_JPRB, 0.23665E+01_JPRB, 0.29607E+01_JPRB, 0.37851E+01_JPRB, 0.24274E+01_JPRB ,&
&  0.21472E+01_JPRB, 0.27850E+01_JPRB, 0.23544E+01_JPRB, 0.20946E+01_JPRB, 0.25594E+01_JPRB, 0.33323E+01_JPRB, 0.24277E+01_JPRB ,&
&  0.11656E+01_JPRB, 0.93683E+00_JPRB, 0.98013E+00_JPRB, 0.10623E+01_JPRB, 0.10687E+01_JPRB, 0.16308E+01_JPRB, 0.42783E+01_JPRB ,&
&  0.45370E+01_JPRB, 0.53267E+01_JPRB, 0.49435E+01_JPRB, 0.44206E+01_JPRB, 0.41258E+01_JPRB, 0.38989E+01_JPRB, 0.33504E+01_JPRB ,&
&  0.28807E+01_JPRB, 0.25996E+01_JPRB, 0.23824E+01_JPRB, 0.22255E+01_JPRB, 0.20733E+01_JPRB, 0.19235E+01_JPRB, 0.17845E+01_JPRB ,&
&  0.16845E+01_JPRB, 0.15816E+01_JPRB, 0.14895E+01_JPRB, 0.13642E+01_JPRB, 0.12524E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.11744E+01_JPRB, 0.11350E+01_JPRB, 0.11209E+01_JPRB, 0.10847E+01_JPRB, 0.10251E+01_JPRB, 0.96650E+00_JPRB, 0.87035E+00_JPRB ,&
&  0.79613E+00_JPRB, 0.75061E+00_JPRB, 0.73195E+00_JPRB, 0.73283E+00_JPRB, 0.75257E+00_JPRB, 0.79004E+00_JPRB, 0.83355E+00_JPRB ,&
&  0.98212E+00_JPRB, 0.11301E+01_JPRB, 0.13687E+01_JPRB, 0.17044E+01_JPRB, 0.19013E+01_JPRB, 0.17553E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.22513E+01_JPRB, 0.19266E+01_JPRB, 0.16152E+01_JPRB, 0.14555E+01_JPRB, 0.13246E+01_JPRB, 0.12309E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.10296E+01_JPRB, 0.98537E+00_JPRB, 0.99450E+00_JPRB, 0.98954E+00_JPRB, 0.10261E+01_JPRB, 0.10075E+01_JPRB ,&
&  0.10095E+01_JPRB, 0.11101E+01_JPRB, 0.12520E+01_JPRB, 0.13876E+01_JPRB, 0.15094E+01_JPRB, 0.16197E+01_JPRB, 0.17499E+01_JPRB ,&
&  0.18127E+01_JPRB, 0.17790E+01_JPRB, 0.16915E+01_JPRB, 0.15358E+01_JPRB, 0.14970E+01_JPRB, 0.15150E+01_JPRB, 0.16195E+01_JPRB ,&
&  0.16744E+01_JPRB, 0.17976E+01_JPRB, 0.18500E+01_JPRB, 0.18364E+01_JPRB, 0.17394E+01_JPRB, 0.16597E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.20977E+01_JPRB, 0.21511E+01_JPRB, 0.19814E+01_JPRB, 0.25183E+01_JPRB, 0.31821E+01_JPRB, 0.36813E+01_JPRB, 0.34216E+01_JPRB ,&
&  0.29053E+01_JPRB, 0.23938E+01_JPRB, 0.22286E+01_JPRB, 0.25722E+01_JPRB, 0.28923E+01_JPRB, 0.33599E+01_JPRB, 0.43800E+01_JPRB ,&
&  0.45802E+01_JPRB, 0.26965E+01_JPRB, 0.22607E+01_JPRB, 0.30098E+01_JPRB, 0.25700E+01_JPRB, 0.26422E+01_JPRB, 0.38315E+01_JPRB ,&
&  0.47441E+01_JPRB, 0.53378E+01_JPRB, 0.50772E+01_JPRB, 0.44353E+01_JPRB, 0.41065E+01_JPRB, 0.36446E+01_JPRB, 0.32167E+01_JPRB ,&
&  0.27615E+01_JPRB, 0.24003E+01_JPRB, 0.21466E+01_JPRB, 0.19538E+01_JPRB, 0.17743E+01_JPRB, 0.16348E+01_JPRB, 0.15665E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.14596E+01_JPRB, 0.14036E+01_JPRB, 0.13118E+01_JPRB, 0.12261E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.11512E+01_JPRB, 0.11254E+01_JPRB, 0.11091E+01_JPRB, 0.10770E+01_JPRB, 0.10772E+01_JPRB, 0.10592E+01_JPRB, 0.96911E+00_JPRB ,&
&  0.88778E+00_JPRB, 0.77047E+00_JPRB, 0.75604E+00_JPRB, 0.77225E+00_JPRB, 0.80241E+00_JPRB, 0.84633E+00_JPRB, 0.93658E+00_JPRB ,&
&  0.10815E+01_JPRB, 0.12288E+01_JPRB, 0.15251E+01_JPRB, 0.18970E+01_JPRB, 0.22653E+01_JPRB, 0.28328E+01_JPRB, 0.19975E+01_JPRB ,&
&  0.20105E+01_JPRB, 0.18750E+01_JPRB, 0.16402E+01_JPRB, 0.15921E+01_JPRB, 0.15140E+01_JPRB, 0.14811E+01_JPRB, 0.12967E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.11653E+01_JPRB, 0.11154E+01_JPRB, 0.10720E+01_JPRB, 0.10773E+01_JPRB, 0.10776E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.10892E+01_JPRB, 0.11905E+01_JPRB, 0.13126E+01_JPRB, 0.14281E+01_JPRB, 0.15514E+01_JPRB, 0.16883E+01_JPRB, 0.17859E+01_JPRB ,&
&  0.18876E+01_JPRB, 0.18802E+01_JPRB, 0.17873E+01_JPRB, 0.16053E+01_JPRB, 0.15823E+01_JPRB, 0.15850E+01_JPRB, 0.15979E+01_JPRB ,&
&  0.15103E+01_JPRB, 0.14513E+01_JPRB, 0.16306E+01_JPRB, 0.17260E+01_JPRB, 0.16746E+01_JPRB, 0.15709E+01_JPRB, 0.16145E+01_JPRB ,&
&  0.20360E+01_JPRB, 0.23478E+01_JPRB, 0.27262E+01_JPRB, 0.29675E+01_JPRB, 0.34320E+01_JPRB, 0.37250E+01_JPRB, 0.38595E+01_JPRB ,&
&  0.39215E+01_JPRB, 0.33936E+01_JPRB, 0.31528E+01_JPRB, 0.32761E+01_JPRB, 0.32892E+01_JPRB, 0.35992E+01_JPRB, 0.41857E+01_JPRB ,&
&  0.49786E+01_JPRB, 0.61683E+01_JPRB, 0.63251E+01_JPRB, 0.51542E+01_JPRB, 0.45426E+01_JPRB, 0.38813E+01_JPRB, 0.41563E+01_JPRB ,&
&  0.48240E+01_JPRB, 0.47135E+01_JPRB, 0.45199E+01_JPRB, 0.46699E+01_JPRB, 0.36075E+01_JPRB, 0.31116E+01_JPRB, 0.27565E+01_JPRB ,&
&  0.24773E+01_JPRB, 0.21468E+01_JPRB, 0.18883E+01_JPRB, 0.16834E+01_JPRB, 0.14916E+01_JPRB, 0.14001E+01_JPRB, 0.13857E+01_JPRB ,&
&  0.13627E+01_JPRB, 0.13407E+01_JPRB, 0.13067E+01_JPRB, 0.12466E+01_JPRB, 0.11943E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.11300E+01_JPRB, 0.10986E+01_JPRB, 0.10821E+01_JPRB, 0.10667E+01_JPRB, 0.11151E+01_JPRB, 0.11236E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.82756E+00_JPRB, 0.80241E+00_JPRB, 0.82151E+00_JPRB, 0.86393E+00_JPRB, 0.92724E+00_JPRB, 0.10400E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.13253E+01_JPRB, 0.16519E+01_JPRB, 0.20731E+01_JPRB, 0.24422E+01_JPRB, 0.27075E+01_JPRB, 0.26895E+01_JPRB ,&
&  0.17160E+01_JPRB, 0.19552E+01_JPRB, 0.16895E+01_JPRB, 0.17721E+01_JPRB, 0.17856E+01_JPRB, 0.17445E+01_JPRB, 0.14771E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.12672E+01_JPRB, 0.12387E+01_JPRB, 0.12533E+01_JPRB, 0.12194E+01_JPRB, 0.11491E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.12054E+01_JPRB, 0.13224E+01_JPRB, 0.14498E+01_JPRB, 0.15499E+01_JPRB, 0.16830E+01_JPRB, 0.18083E+01_JPRB, 0.18921E+01_JPRB ,&
&  0.19843E+01_JPRB, 0.19878E+01_JPRB, 0.18314E+01_JPRB, 0.17371E+01_JPRB, 0.16834E+01_JPRB, 0.16832E+01_JPRB, 0.16593E+01_JPRB ,&
&  0.15444E+01_JPRB, 0.14527E+01_JPRB, 0.15519E+01_JPRB, 0.15493E+01_JPRB, 0.14999E+01_JPRB, 0.15898E+01_JPRB, 0.17278E+01_JPRB ,&
&  0.22443E+01_JPRB, 0.25430E+01_JPRB, 0.35043E+01_JPRB, 0.32625E+01_JPRB, 0.33332E+01_JPRB, 0.37045E+01_JPRB, 0.36077E+01_JPRB ,&
&  0.32535E+01_JPRB, 0.33683E+01_JPRB, 0.33178E+01_JPRB, 0.36417E+01_JPRB, 0.38679E+01_JPRB, 0.42402E+01_JPRB, 0.44507E+01_JPRB ,&
&  0.48647E+01_JPRB, 0.54191E+01_JPRB, 0.60362E+01_JPRB, 0.49123E+01_JPRB, 0.43973E+01_JPRB, 0.40641E+01_JPRB, 0.43028E+01_JPRB ,&
&  0.45557E+01_JPRB, 0.43697E+01_JPRB, 0.44115E+01_JPRB, 0.40565E+01_JPRB, 0.30310E+01_JPRB, 0.26557E+01_JPRB, 0.23469E+01_JPRB ,&
&  0.21033E+01_JPRB, 0.18431E+01_JPRB, 0.16290E+01_JPRB, 0.14491E+01_JPRB, 0.13349E+01_JPRB, 0.13004E+01_JPRB, 0.12972E+01_JPRB ,&
&  0.12868E+01_JPRB, 0.12748E+01_JPRB, 0.12564E+01_JPRB, 0.12100E+01_JPRB, 0.11664E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.11241E+01_JPRB, 0.11072E+01_JPRB, 0.10999E+01_JPRB, 0.10699E+01_JPRB, 0.11212E+01_JPRB, 0.12241E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.86730E+00_JPRB, 0.85261E+00_JPRB, 0.87487E+00_JPRB, 0.91759E+00_JPRB, 0.10058E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.14506E+01_JPRB, 0.18060E+01_JPRB, 0.22288E+01_JPRB, 0.25657E+01_JPRB, 0.29553E+01_JPRB, 0.30409E+01_JPRB ,&
&  0.21468E+01_JPRB, 0.17203E+01_JPRB, 0.17004E+01_JPRB, 0.18289E+01_JPRB, 0.17892E+01_JPRB, 0.18727E+01_JPRB, 0.18642E+01_JPRB ,&
&  0.16962E+01_JPRB, 0.15103E+01_JPRB, 0.15230E+01_JPRB, 0.15270E+01_JPRB, 0.13007E+01_JPRB, 0.12881E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.15011E+01_JPRB, 0.16160E+01_JPRB, 0.17095E+01_JPRB, 0.18493E+01_JPRB, 0.19529E+01_JPRB, 0.20438E+01_JPRB ,&
&  0.20748E+01_JPRB, 0.20907E+01_JPRB, 0.19633E+01_JPRB, 0.18206E+01_JPRB, 0.17714E+01_JPRB, 0.17362E+01_JPRB, 0.16691E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.16248E+01_JPRB, 0.16367E+01_JPRB, 0.15967E+01_JPRB, 0.15809E+01_JPRB, 0.18531E+01_JPRB, 0.21421E+01_JPRB ,&
&  0.25255E+01_JPRB, 0.30248E+01_JPRB, 0.40613E+01_JPRB, 0.31895E+01_JPRB, 0.32616E+01_JPRB, 0.34892E+01_JPRB, 0.34755E+01_JPRB ,&
&  0.32913E+01_JPRB, 0.34370E+01_JPRB, 0.35727E+01_JPRB, 0.38448E+01_JPRB, 0.41841E+01_JPRB, 0.42489E+01_JPRB, 0.52058E+01_JPRB ,&
&  0.49231E+01_JPRB, 0.56594E+01_JPRB, 0.51457E+01_JPRB, 0.48320E+01_JPRB, 0.40615E+01_JPRB, 0.39545E+01_JPRB, 0.38163E+01_JPRB ,&
&  0.39576E+01_JPRB, 0.38009E+01_JPRB, 0.34889E+01_JPRB, 0.31347E+01_JPRB, 0.24571E+01_JPRB, 0.21952E+01_JPRB, 0.19290E+01_JPRB ,&
&  0.16870E+01_JPRB, 0.15149E+01_JPRB, 0.14299E+01_JPRB, 0.13620E+01_JPRB, 0.13321E+01_JPRB, 0.13130E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.12526E+01_JPRB, 0.12310E+01_JPRB, 0.11872E+01_JPRB, 0.11509E+01_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.11135E+01_JPRB, 0.10908E+01_JPRB, 0.10637E+01_JPRB, 0.99600E+00_JPRB, 0.95687E+00_JPRB, 0.95726E+00_JPRB, 0.94054E+00_JPRB ,&
&  0.91905E+00_JPRB, 0.88871E+00_JPRB, 0.90055E+00_JPRB, 0.94058E+00_JPRB, 0.99656E+00_JPRB, 0.10906E+01_JPRB, 0.12146E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.15538E+01_JPRB, 0.18306E+01_JPRB, 0.21273E+01_JPRB, 0.23759E+01_JPRB, 0.25689E+01_JPRB, 0.26617E+01_JPRB ,&
&  0.21497E+01_JPRB, 0.15480E+01_JPRB, 0.18516E+01_JPRB, 0.20155E+01_JPRB, 0.20116E+01_JPRB, 0.21643E+01_JPRB, 0.21366E+01_JPRB ,&
&  0.18319E+01_JPRB, 0.17501E+01_JPRB, 0.18015E+01_JPRB, 0.16820E+01_JPRB, 0.14150E+01_JPRB, 0.14478E+01_JPRB, 0.15146E+01_JPRB ,&
&  0.16113E+01_JPRB, 0.16917E+01_JPRB, 0.17924E+01_JPRB, 0.18950E+01_JPRB, 0.20069E+01_JPRB, 0.21415E+01_JPRB, 0.22282E+01_JPRB ,&
&  0.22365E+01_JPRB, 0.22162E+01_JPRB, 0.21570E+01_JPRB, 0.19867E+01_JPRB, 0.18746E+01_JPRB, 0.18185E+01_JPRB, 0.17504E+01_JPRB ,&
&  0.17326E+01_JPRB, 0.17186E+01_JPRB, 0.17973E+01_JPRB, 0.19921E+01_JPRB, 0.23052E+01_JPRB, 0.23132E+01_JPRB, 0.26557E+01_JPRB ,&
&  0.29214E+01_JPRB, 0.35108E+01_JPRB, 0.36949E+01_JPRB, 0.41938E+01_JPRB, 0.26239E+01_JPRB, 0.30420E+01_JPRB, 0.28711E+01_JPRB ,&
&  0.34938E+01_JPRB, 0.34463E+01_JPRB, 0.37078E+01_JPRB, 0.41106E+01_JPRB, 0.47607E+01_JPRB, 0.48268E+01_JPRB, 0.51240E+01_JPRB ,&
&  0.49798E+01_JPRB, 0.47544E+01_JPRB, 0.42374E+01_JPRB, 0.41460E+01_JPRB, 0.39060E+01_JPRB, 0.41049E+01_JPRB, 0.34608E+01_JPRB ,&
&  0.33972E+01_JPRB, 0.31733E+01_JPRB, 0.29255E+01_JPRB, 0.26755E+01_JPRB, 0.20926E+01_JPRB, 0.17298E+01_JPRB, 0.15158E+01_JPRB ,&
&  0.14208E+01_JPRB, 0.13740E+01_JPRB, 0.13338E+01_JPRB, 0.13092E+01_JPRB, 0.13061E+01_JPRB, 0.13021E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.12709E+01_JPRB, 0.12460E+01_JPRB, 0.12192E+01_JPRB, 0.11800E+01_JPRB, 0.11468E+01_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.10839E+01_JPRB, 0.10533E+01_JPRB, 0.10138E+01_JPRB, 0.96381E+00_JPRB, 0.93692E+00_JPRB, 0.91620E+00_JPRB, 0.91175E+00_JPRB ,&
&  0.91159E+00_JPRB, 0.93080E+00_JPRB, 0.94799E+00_JPRB, 0.98196E+00_JPRB, 0.10277E+01_JPRB, 0.11090E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.14212E+01_JPRB, 0.16751E+01_JPRB, 0.19909E+01_JPRB, 0.22461E+01_JPRB, 0.23826E+01_JPRB, 0.22564E+01_JPRB ,&
&  0.21359E+01_JPRB, 0.20911E+01_JPRB, 0.19649E+01_JPRB, 0.16363E+01_JPRB, 0.16251E+01_JPRB, 0.20522E+01_JPRB, 0.19104E+01_JPRB ,&
&  0.17262E+01_JPRB, 0.16878E+01_JPRB, 0.16505E+01_JPRB, 0.16081E+01_JPRB, 0.15688E+01_JPRB, 0.16085E+01_JPRB, 0.16790E+01_JPRB ,&
&  0.17614E+01_JPRB, 0.18431E+01_JPRB, 0.19638E+01_JPRB, 0.20840E+01_JPRB, 0.22109E+01_JPRB, 0.23865E+01_JPRB, 0.24751E+01_JPRB ,&
&  0.24591E+01_JPRB, 0.24374E+01_JPRB, 0.23679E+01_JPRB, 0.22059E+01_JPRB, 0.20256E+01_JPRB, 0.19950E+01_JPRB, 0.20162E+01_JPRB ,&
&  0.20624E+01_JPRB, 0.21416E+01_JPRB, 0.22464E+01_JPRB, 0.25471E+01_JPRB, 0.28251E+01_JPRB, 0.26857E+01_JPRB, 0.27377E+01_JPRB ,&
&  0.31819E+01_JPRB, 0.37077E+01_JPRB, 0.33071E+01_JPRB, 0.38918E+01_JPRB, 0.22324E+01_JPRB, 0.27703E+01_JPRB, 0.33029E+01_JPRB ,&
&  0.32612E+01_JPRB, 0.34536E+01_JPRB, 0.38762E+01_JPRB, 0.44434E+01_JPRB, 0.50814E+01_JPRB, 0.47991E+01_JPRB, 0.47287E+01_JPRB ,&
&  0.42881E+01_JPRB, 0.36807E+01_JPRB, 0.35018E+01_JPRB, 0.34089E+01_JPRB, 0.35543E+01_JPRB, 0.35939E+01_JPRB, 0.27859E+01_JPRB ,&
&  0.23799E+01_JPRB, 0.26481E+01_JPRB, 0.26775E+01_JPRB, 0.24910E+01_JPRB, 0.18024E+01_JPRB, 0.13997E+01_JPRB, 0.13239E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.12865E+01_JPRB, 0.12745E+01_JPRB, 0.12772E+01_JPRB, 0.12885E+01_JPRB, 0.12889E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.12691E+01_JPRB, 0.12391E+01_JPRB, 0.12074E+01_JPRB, 0.11668E+01_JPRB, 0.11241E+01_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.10411E+01_JPRB, 0.10109E+01_JPRB, 0.96892E+00_JPRB, 0.92790E+00_JPRB, 0.87183E+00_JPRB, 0.84211E+00_JPRB, 0.83798E+00_JPRB ,&
&  0.85780E+00_JPRB, 0.88858E+00_JPRB, 0.91105E+00_JPRB, 0.91879E+00_JPRB, 0.94865E+00_JPRB, 0.96752E+00_JPRB, 0.99956E+00_JPRB ,&
&  0.10677E+01_JPRB, 0.11818E+01_JPRB, 0.13989E+01_JPRB, 0.17468E+01_JPRB, 0.20851E+01_JPRB, 0.24010E+01_JPRB, 0.22861E+01_JPRB ,&
&  0.21585E+01_JPRB, 0.20773E+01_JPRB, 0.20555E+01_JPRB, 0.20661E+01_JPRB, 0.18543E+01_JPRB, 0.20611E+01_JPRB, 0.21115E+01_JPRB ,&
&  0.21306E+01_JPRB, 0.19993E+01_JPRB, 0.17757E+01_JPRB, 0.17346E+01_JPRB, 0.17199E+01_JPRB, 0.17526E+01_JPRB, 0.18196E+01_JPRB ,&
&  0.19024E+01_JPRB, 0.19902E+01_JPRB, 0.21322E+01_JPRB, 0.23073E+01_JPRB, 0.25166E+01_JPRB, 0.27078E+01_JPRB, 0.27818E+01_JPRB ,&
&  0.27711E+01_JPRB, 0.27541E+01_JPRB, 0.27022E+01_JPRB, 0.25297E+01_JPRB, 0.24467E+01_JPRB, 0.24851E+01_JPRB, 0.25860E+01_JPRB ,&
&  0.27208E+01_JPRB, 0.26905E+01_JPRB, 0.28375E+01_JPRB, 0.31766E+01_JPRB, 0.32756E+01_JPRB, 0.31919E+01_JPRB, 0.31864E+01_JPRB ,&
&  0.34660E+01_JPRB, 0.38524E+01_JPRB, 0.26247E+01_JPRB, 0.31672E+01_JPRB, 0.38306E+01_JPRB, 0.32797E+01_JPRB, 0.31506E+01_JPRB ,&
&  0.33002E+01_JPRB, 0.35600E+01_JPRB, 0.39989E+01_JPRB, 0.45135E+01_JPRB, 0.51009E+01_JPRB, 0.55400E+01_JPRB, 0.39297E+01_JPRB ,&
&  0.34768E+01_JPRB, 0.29294E+01_JPRB, 0.28234E+01_JPRB, 0.28635E+01_JPRB, 0.29713E+01_JPRB, 0.28429E+01_JPRB, 0.21923E+01_JPRB ,&
&  0.20160E+01_JPRB, 0.22035E+01_JPRB, 0.21423E+01_JPRB, 0.21150E+01_JPRB, 0.19087E+01_JPRB, 0.12390E+01_JPRB, 0.12599E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.12387E+01_JPRB, 0.12375E+01_JPRB, 0.12446E+01_JPRB, 0.12553E+01_JPRB, 0.12580E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.11974E+01_JPRB, 0.11581E+01_JPRB, 0.11167E+01_JPRB, 0.10764E+01_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.10059E+01_JPRB, 0.96966E+00_JPRB, 0.92217E+00_JPRB, 0.86381E+00_JPRB, 0.80839E+00_JPRB, 0.78593E+00_JPRB, 0.77638E+00_JPRB ,&
&  0.78313E+00_JPRB, 0.81048E+00_JPRB, 0.82762E+00_JPRB, 0.82550E+00_JPRB, 0.82113E+00_JPRB, 0.83667E+00_JPRB, 0.88738E+00_JPRB ,&
&  0.95100E+00_JPRB, 0.10294E+01_JPRB, 0.11864E+01_JPRB, 0.15232E+01_JPRB, 0.19600E+01_JPRB, 0.23247E+01_JPRB, 0.23171E+01_JPRB ,&
&  0.21971E+01_JPRB, 0.22152E+01_JPRB, 0.22461E+01_JPRB, 0.23213E+01_JPRB, 0.23845E+01_JPRB, 0.25161E+01_JPRB, 0.26581E+01_JPRB ,&
&  0.24089E+01_JPRB, 0.19759E+01_JPRB, 0.18389E+01_JPRB, 0.17814E+01_JPRB, 0.18121E+01_JPRB, 0.18760E+01_JPRB, 0.19505E+01_JPRB ,&
&  0.20427E+01_JPRB, 0.21759E+01_JPRB, 0.23616E+01_JPRB, 0.26413E+01_JPRB, 0.29334E+01_JPRB, 0.31073E+01_JPRB, 0.31954E+01_JPRB ,&
&  0.32495E+01_JPRB, 0.32839E+01_JPRB, 0.33647E+01_JPRB, 0.30410E+01_JPRB, 0.31546E+01_JPRB, 0.32516E+01_JPRB, 0.33490E+01_JPRB ,&
&  0.33247E+01_JPRB, 0.32294E+01_JPRB, 0.34862E+01_JPRB, 0.36896E+01_JPRB, 0.38919E+01_JPRB, 0.38847E+01_JPRB, 0.37791E+01_JPRB ,&
&  0.39409E+01_JPRB, 0.35977E+01_JPRB, 0.23591E+01_JPRB, 0.23280E+01_JPRB, 0.25571E+01_JPRB, 0.24022E+01_JPRB, 0.30412E+01_JPRB ,&
&  0.31181E+01_JPRB, 0.33120E+01_JPRB, 0.35587E+01_JPRB, 0.38206E+01_JPRB, 0.40931E+01_JPRB, 0.42184E+01_JPRB, 0.32641E+01_JPRB ,&
&  0.26727E+01_JPRB, 0.23732E+01_JPRB, 0.23104E+01_JPRB, 0.22532E+01_JPRB, 0.22003E+01_JPRB, 0.20336E+01_JPRB, 0.19246E+01_JPRB ,&
&  0.18927E+01_JPRB, 0.18505E+01_JPRB, 0.17500E+01_JPRB, 0.15669E+01_JPRB, 0.13627E+01_JPRB, 0.11914E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.12171E+01_JPRB, 0.11948E+01_JPRB, 0.11957E+01_JPRB, 0.11954E+01_JPRB, 0.12002E+01_JPRB, 0.11965E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.11565E+01_JPRB, 0.11269E+01_JPRB, 0.10971E+01_JPRB, 0.10633E+01_JPRB, 0.10310E+01_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.97255E+00_JPRB, 0.92773E+00_JPRB, 0.86977E+00_JPRB, 0.81784E+00_JPRB, 0.81082E+00_JPRB, 0.81482E+00_JPRB, 0.79895E+00_JPRB ,&
&  0.79182E+00_JPRB, 0.78739E+00_JPRB, 0.80777E+00_JPRB, 0.81321E+00_JPRB, 0.81938E+00_JPRB, 0.83839E+00_JPRB, 0.86225E+00_JPRB ,&
&  0.88178E+00_JPRB, 0.89675E+00_JPRB, 0.97417E+00_JPRB, 0.11445E+01_JPRB, 0.14724E+01_JPRB, 0.19152E+01_JPRB, 0.22540E+01_JPRB ,&
&  0.24671E+01_JPRB, 0.26534E+01_JPRB, 0.28339E+01_JPRB, 0.29966E+01_JPRB, 0.28907E+01_JPRB, 0.30397E+01_JPRB, 0.33941E+01_JPRB ,&
&  0.25527E+01_JPRB, 0.18256E+01_JPRB, 0.19185E+01_JPRB, 0.17662E+01_JPRB, 0.18238E+01_JPRB, 0.19558E+01_JPRB, 0.20696E+01_JPRB ,&
&  0.22201E+01_JPRB, 0.23935E+01_JPRB, 0.26016E+01_JPRB, 0.28835E+01_JPRB, 0.31798E+01_JPRB, 0.33916E+01_JPRB, 0.35600E+01_JPRB ,&
&  0.37001E+01_JPRB, 0.38461E+01_JPRB, 0.40518E+01_JPRB, 0.40351E+01_JPRB, 0.39531E+01_JPRB, 0.40944E+01_JPRB, 0.41683E+01_JPRB ,&
&  0.42741E+01_JPRB, 0.42867E+01_JPRB, 0.37554E+01_JPRB, 0.39157E+01_JPRB, 0.44302E+01_JPRB, 0.41996E+01_JPRB, 0.41442E+01_JPRB ,&
&  0.44406E+01_JPRB, 0.37010E+01_JPRB, 0.21614E+01_JPRB, 0.20734E+01_JPRB, 0.22987E+01_JPRB, 0.25627E+01_JPRB, 0.28358E+01_JPRB ,&
&  0.29899E+01_JPRB, 0.31223E+01_JPRB, 0.31423E+01_JPRB, 0.31647E+01_JPRB, 0.30131E+01_JPRB, 0.27990E+01_JPRB, 0.25574E+01_JPRB ,&
&  0.21792E+01_JPRB, 0.20654E+01_JPRB, 0.19730E+01_JPRB, 0.19282E+01_JPRB, 0.19410E+01_JPRB, 0.18542E+01_JPRB, 0.17514E+01_JPRB ,&
&  0.18967E+01_JPRB, 0.19968E+01_JPRB, 0.22061E+01_JPRB, 0.17194E+01_JPRB, 0.12901E+01_JPRB, 0.11823E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.11916E+01_JPRB, 0.11835E+01_JPRB, 0.11739E+01_JPRB, 0.11669E+01_JPRB, 0.11455E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.10544E+01_JPRB, 0.10401E+01_JPRB, 0.10212E+01_JPRB, 0.99398E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.91424E+00_JPRB, 0.84871E+00_JPRB, 0.81475E+00_JPRB, 0.85656E+00_JPRB, 0.83131E+00_JPRB, 0.85927E+00_JPRB, 0.85221E+00_JPRB ,&
&  0.85604E+00_JPRB, 0.86097E+00_JPRB, 0.88434E+00_JPRB, 0.91614E+00_JPRB, 0.95129E+00_JPRB, 0.97906E+00_JPRB, 0.99301E+00_JPRB ,&
&  0.10065E+01_JPRB, 0.10179E+01_JPRB, 0.10362E+01_JPRB, 0.10743E+01_JPRB, 0.11527E+01_JPRB, 0.12906E+01_JPRB, 0.15608E+01_JPRB ,&
&  0.18471E+01_JPRB, 0.20193E+01_JPRB, 0.21035E+01_JPRB, 0.21403E+01_JPRB, 0.24107E+01_JPRB, 0.30159E+01_JPRB, 0.35037E+01_JPRB ,&
&  0.21482E+01_JPRB, 0.19343E+01_JPRB, 0.17304E+01_JPRB, 0.15804E+01_JPRB, 0.16734E+01_JPRB, 0.19265E+01_JPRB, 0.20776E+01_JPRB ,&
&  0.21830E+01_JPRB, 0.23469E+01_JPRB, 0.25447E+01_JPRB, 0.27880E+01_JPRB, 0.30048E+01_JPRB, 0.32265E+01_JPRB, 0.34571E+01_JPRB ,&
&  0.37028E+01_JPRB, 0.40137E+01_JPRB, 0.43350E+01_JPRB, 0.46649E+01_JPRB, 0.46501E+01_JPRB, 0.52144E+01_JPRB, 0.53385E+01_JPRB ,&
&  0.54237E+01_JPRB, 0.51787E+01_JPRB, 0.44832E+01_JPRB, 0.45856E+01_JPRB, 0.49190E+01_JPRB, 0.48692E+01_JPRB, 0.45175E+01_JPRB ,&
&  0.45165E+01_JPRB, 0.34385E+01_JPRB, 0.23302E+01_JPRB, 0.23145E+01_JPRB, 0.23607E+01_JPRB, 0.24906E+01_JPRB, 0.24403E+01_JPRB ,&
&  0.25857E+01_JPRB, 0.26015E+01_JPRB, 0.25335E+01_JPRB, 0.23925E+01_JPRB, 0.22243E+01_JPRB, 0.21406E+01_JPRB, 0.21111E+01_JPRB ,&
&  0.19318E+01_JPRB, 0.18745E+01_JPRB, 0.18333E+01_JPRB, 0.18885E+01_JPRB, 0.19936E+01_JPRB, 0.16866E+01_JPRB, 0.16959E+01_JPRB ,&
&  0.18054E+01_JPRB, 0.19396E+01_JPRB, 0.17039E+01_JPRB, 0.18746E+01_JPRB, 0.12206E+01_JPRB, 0.10986E+01_JPRB, 0.11169E+01_JPRB ,&
&  0.11554E+01_JPRB, 0.11273E+01_JPRB, 0.11235E+01_JPRB, 0.11137E+01_JPRB, 0.11134E+01_JPRB, 0.10959E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.10048E+01_JPRB, 0.98301E+00_JPRB, 0.97991E+00_JPRB, 0.97041E+00_JPRB, 0.95734E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.83893E+00_JPRB, 0.80366E+00_JPRB, 0.85979E+00_JPRB, 0.83212E+00_JPRB, 0.83570E+00_JPRB, 0.85840E+00_JPRB, 0.88170E+00_JPRB ,&
&  0.89574E+00_JPRB, 0.91947E+00_JPRB, 0.95828E+00_JPRB, 0.10135E+01_JPRB, 0.10651E+01_JPRB, 0.11048E+01_JPRB, 0.11492E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.12040E+01_JPRB, 0.12161E+01_JPRB, 0.12274E+01_JPRB, 0.12619E+01_JPRB, 0.13033E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.14702E+01_JPRB, 0.15061E+01_JPRB, 0.15213E+01_JPRB, 0.15822E+01_JPRB, 0.16574E+01_JPRB, 0.18572E+01_JPRB, 0.23042E+01_JPRB ,&
&  0.18480E+01_JPRB, 0.19027E+01_JPRB, 0.17871E+01_JPRB, 0.17030E+01_JPRB, 0.17794E+01_JPRB, 0.17610E+01_JPRB, 0.18522E+01_JPRB ,&
&  0.20319E+01_JPRB, 0.20924E+01_JPRB, 0.22062E+01_JPRB, 0.23127E+01_JPRB, 0.24529E+01_JPRB, 0.26365E+01_JPRB, 0.28229E+01_JPRB ,&
&  0.30579E+01_JPRB, 0.33355E+01_JPRB, 0.35508E+01_JPRB, 0.38929E+01_JPRB, 0.42333E+01_JPRB, 0.45997E+01_JPRB, 0.47919E+01_JPRB ,&
&  0.49116E+01_JPRB, 0.49917E+01_JPRB, 0.46207E+01_JPRB, 0.46682E+01_JPRB, 0.46849E+01_JPRB, 0.44574E+01_JPRB, 0.39129E+01_JPRB ,&
&  0.33611E+01_JPRB, 0.28750E+01_JPRB, 0.25478E+01_JPRB, 0.23670E+01_JPRB, 0.23016E+01_JPRB, 0.21968E+01_JPRB, 0.21648E+01_JPRB ,&
&  0.21911E+01_JPRB, 0.20931E+01_JPRB, 0.19650E+01_JPRB, 0.18011E+01_JPRB, 0.17221E+01_JPRB, 0.17069E+01_JPRB, 0.17322E+01_JPRB ,&
&  0.16911E+01_JPRB, 0.16631E+01_JPRB, 0.17357E+01_JPRB, 0.19541E+01_JPRB, 0.22496E+01_JPRB, 0.29247E+01_JPRB, 0.22037E+01_JPRB ,&
&  0.16233E+01_JPRB, 0.15020E+01_JPRB, 0.10374E+01_JPRB, 0.13181E+01_JPRB, 0.10289E+01_JPRB, 0.97835E+00_JPRB, 0.10193E+01_JPRB ,&
&  0.10410E+01_JPRB, 0.10390E+01_JPRB, 0.10841E+01_JPRB, 0.10513E+01_JPRB, 0.10205E+01_JPRB, 0.10048E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.95749E+00_JPRB, 0.90393E+00_JPRB, 0.84696E+00_JPRB, 0.85846E+00_JPRB, 0.81150E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.71997E+00_JPRB, 0.75200E+00_JPRB, 0.85790E+00_JPRB, 0.80411E+00_JPRB, 0.81080E+00_JPRB, 0.83765E+00_JPRB, 0.87522E+00_JPRB ,&
&  0.91545E+00_JPRB, 0.96457E+00_JPRB, 0.10165E+01_JPRB, 0.10855E+01_JPRB, 0.11470E+01_JPRB, 0.11997E+01_JPRB, 0.12498E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.13348E+01_JPRB, 0.13730E+01_JPRB, 0.13961E+01_JPRB, 0.14103E+01_JPRB, 0.14482E+01_JPRB, 0.14958E+01_JPRB ,&
&  0.15921E+01_JPRB, 0.16452E+01_JPRB, 0.17088E+01_JPRB, 0.18074E+01_JPRB, 0.18889E+01_JPRB, 0.20061E+01_JPRB, 0.18435E+01_JPRB ,&
&  0.21274E+01_JPRB, 0.18887E+01_JPRB, 0.18131E+01_JPRB, 0.18030E+01_JPRB, 0.17666E+01_JPRB, 0.17367E+01_JPRB, 0.17526E+01_JPRB ,&
&  0.18386E+01_JPRB, 0.18070E+01_JPRB, 0.18417E+01_JPRB, 0.18452E+01_JPRB, 0.19317E+01_JPRB, 0.20891E+01_JPRB, 0.22040E+01_JPRB ,&
&  0.23968E+01_JPRB, 0.26293E+01_JPRB, 0.28860E+01_JPRB, 0.30865E+01_JPRB, 0.32624E+01_JPRB, 0.32775E+01_JPRB, 0.32821E+01_JPRB ,&
&  0.33776E+01_JPRB, 0.35875E+01_JPRB, 0.35430E+01_JPRB, 0.36539E+01_JPRB, 0.36497E+01_JPRB, 0.33501E+01_JPRB, 0.26396E+01_JPRB ,&
&  0.23024E+01_JPRB, 0.23974E+01_JPRB, 0.21437E+01_JPRB, 0.24705E+01_JPRB, 0.22001E+01_JPRB, 0.19558E+01_JPRB, 0.19321E+01_JPRB ,&
&  0.18430E+01_JPRB, 0.16923E+01_JPRB, 0.15675E+01_JPRB, 0.13927E+01_JPRB, 0.13632E+01_JPRB, 0.13742E+01_JPRB, 0.14134E+01_JPRB ,&
&  0.13796E+01_JPRB, 0.13672E+01_JPRB, 0.14160E+01_JPRB, 0.16661E+01_JPRB, 0.24034E+01_JPRB, 0.24931E+01_JPRB, 0.26941E+01_JPRB ,&
&  0.15369E+01_JPRB, 0.93672E+00_JPRB, 0.99813E+00_JPRB, 0.96226E+00_JPRB, 0.87785E+00_JPRB, 0.82405E+00_JPRB, 0.88366E+00_JPRB ,&
&  0.96179E+00_JPRB, 0.96416E+00_JPRB, 0.94156E+00_JPRB, 0.94798E+00_JPRB, 0.94495E+00_JPRB, 0.94967E+00_JPRB, 0.87253E+00_JPRB ,&
&  0.85205E+00_JPRB, 0.79432E+00_JPRB, 0.76014E+00_JPRB, 0.74468E+00_JPRB, 0.70640E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.67215E+00_JPRB, 0.75093E+00_JPRB, 0.83114E+00_JPRB, 0.77875E+00_JPRB, 0.80172E+00_JPRB, 0.83912E+00_JPRB, 0.88809E+00_JPRB ,&
&  0.94197E+00_JPRB, 0.10024E+01_JPRB, 0.10647E+01_JPRB, 0.11364E+01_JPRB, 0.11946E+01_JPRB, 0.12491E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.13955E+01_JPRB, 0.14462E+01_JPRB, 0.14721E+01_JPRB, 0.15083E+01_JPRB, 0.15339E+01_JPRB, 0.15684E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.17481E+01_JPRB, 0.18375E+01_JPRB, 0.19271E+01_JPRB, 0.19827E+01_JPRB, 0.21784E+01_JPRB, 0.18320E+01_JPRB ,&
&  0.21401E+01_JPRB, 0.19000E+01_JPRB, 0.17980E+01_JPRB, 0.18260E+01_JPRB, 0.17967E+01_JPRB, 0.17968E+01_JPRB, 0.17983E+01_JPRB ,&
&  0.17999E+01_JPRB, 0.18212E+01_JPRB, 0.18419E+01_JPRB, 0.18254E+01_JPRB, 0.17186E+01_JPRB, 0.18166E+01_JPRB, 0.19264E+01_JPRB ,&
&  0.20575E+01_JPRB, 0.22214E+01_JPRB, 0.23488E+01_JPRB, 0.24271E+01_JPRB, 0.24665E+01_JPRB, 0.24132E+01_JPRB, 0.23591E+01_JPRB ,&
&  0.22940E+01_JPRB, 0.23269E+01_JPRB, 0.22149E+01_JPRB, 0.24113E+01_JPRB, 0.26882E+01_JPRB, 0.22788E+01_JPRB, 0.17342E+01_JPRB ,&
&  0.15013E+01_JPRB, 0.18658E+01_JPRB, 0.21727E+01_JPRB, 0.24161E+01_JPRB, 0.20366E+01_JPRB, 0.18318E+01_JPRB, 0.17151E+01_JPRB ,&
&  0.15646E+01_JPRB, 0.14020E+01_JPRB, 0.12632E+01_JPRB, 0.11191E+01_JPRB, 0.10653E+01_JPRB, 0.10568E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10673E+01_JPRB, 0.10635E+01_JPRB, 0.11058E+01_JPRB, 0.12151E+01_JPRB, 0.15663E+01_JPRB, 0.18297E+01_JPRB, 0.32323E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.11336E+01_JPRB, 0.17180E+01_JPRB, 0.61428E+00_JPRB, 0.68437E+00_JPRB, 0.71354E+00_JPRB, 0.75064E+00_JPRB ,&
&  0.70284E+00_JPRB, 0.89038E+00_JPRB, 0.12340E+01_JPRB, 0.12475E+01_JPRB, 0.13011E+01_JPRB, 0.13752E+01_JPRB, 0.85865E+00_JPRB ,&
&  0.71755E+00_JPRB, 0.71368E+00_JPRB, 0.72096E+00_JPRB, 0.67010E+00_JPRB, 0.66160E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.63074E+00_JPRB, 0.69502E+00_JPRB, 0.77625E+00_JPRB, 0.84062E+00_JPRB, 0.84030E+00_JPRB, 0.85724E+00_JPRB, 0.89065E+00_JPRB ,&
&  0.92809E+00_JPRB, 0.97156E+00_JPRB, 0.10140E+01_JPRB, 0.10675E+01_JPRB, 0.11059E+01_JPRB, 0.11543E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.13187E+01_JPRB, 0.13657E+01_JPRB, 0.14155E+01_JPRB, 0.14727E+01_JPRB, 0.15279E+01_JPRB, 0.15819E+01_JPRB, 0.16410E+01_JPRB ,&
&  0.16833E+01_JPRB, 0.17526E+01_JPRB, 0.18194E+01_JPRB, 0.19591E+01_JPRB, 0.20851E+01_JPRB, 0.24350E+01_JPRB, 0.18354E+01_JPRB ,&
&  0.20708E+01_JPRB, 0.18730E+01_JPRB, 0.17739E+01_JPRB, 0.18130E+01_JPRB, 0.18067E+01_JPRB, 0.17895E+01_JPRB, 0.17924E+01_JPRB ,&
&  0.18770E+01_JPRB, 0.19282E+01_JPRB, 0.19426E+01_JPRB, 0.20729E+01_JPRB, 0.20075E+01_JPRB, 0.20091E+01_JPRB, 0.22095E+01_JPRB ,&
&  0.24031E+01_JPRB, 0.25757E+01_JPRB, 0.26448E+01_JPRB, 0.27466E+01_JPRB, 0.27514E+01_JPRB, 0.26926E+01_JPRB, 0.26187E+01_JPRB ,&
&  0.25247E+01_JPRB, 0.25082E+01_JPRB, 0.25959E+01_JPRB, 0.28735E+01_JPRB, 0.29872E+01_JPRB, 0.26524E+01_JPRB, 0.21614E+01_JPRB ,&
&  0.15740E+01_JPRB, 0.13821E+01_JPRB, 0.19126E+01_JPRB, 0.21971E+01_JPRB, 0.18767E+01_JPRB, 0.17040E+01_JPRB, 0.14862E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.12271E+01_JPRB, 0.11108E+01_JPRB, 0.98436E+00_JPRB, 0.92911E+00_JPRB, 0.91316E+00_JPRB, 0.95847E+00_JPRB ,&
&  0.98311E+00_JPRB, 0.95699E+00_JPRB, 0.96576E+00_JPRB, 0.10182E+01_JPRB, 0.13405E+01_JPRB, 0.18643E+01_JPRB, 0.19486E+01_JPRB ,&
&  0.19328E+01_JPRB, 0.12782E+01_JPRB, 0.10516E+01_JPRB, 0.10374E+01_JPRB, 0.67717E+00_JPRB, 0.69533E+00_JPRB, 0.73589E+00_JPRB ,&
&  0.92499E+00_JPRB, 0.10017E+01_JPRB, 0.13296E+01_JPRB, 0.35464E+01_JPRB, 0.40329E+01_JPRB, 0.50201E+01_JPRB, 0.20436E+01_JPRB ,&
&  0.99030E+00_JPRB, 0.68406E+00_JPRB, 0.62172E+00_JPRB, 0.62592E+00_JPRB, 0.61734E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.59705E+00_JPRB, 0.64869E+00_JPRB, 0.69490E+00_JPRB, 0.76196E+00_JPRB, 0.79363E+00_JPRB, 0.81124E+00_JPRB, 0.82126E+00_JPRB ,&
&  0.81757E+00_JPRB, 0.82759E+00_JPRB, 0.85596E+00_JPRB, 0.92453E+00_JPRB, 0.99855E+00_JPRB, 0.10754E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.12787E+01_JPRB, 0.13848E+01_JPRB, 0.14070E+01_JPRB, 0.14533E+01_JPRB, 0.15282E+01_JPRB, 0.15966E+01_JPRB, 0.16412E+01_JPRB ,&
&  0.16862E+01_JPRB, 0.17713E+01_JPRB, 0.18725E+01_JPRB, 0.20085E+01_JPRB, 0.21787E+01_JPRB, 0.23641E+01_JPRB, 0.17699E+01_JPRB ,&
&  0.20292E+01_JPRB, 0.18556E+01_JPRB, 0.18396E+01_JPRB, 0.18691E+01_JPRB, 0.17975E+01_JPRB, 0.17167E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.18959E+01_JPRB, 0.20765E+01_JPRB, 0.21928E+01_JPRB, 0.22470E+01_JPRB, 0.24469E+01_JPRB, 0.24421E+01_JPRB, 0.28168E+01_JPRB ,&
&  0.31358E+01_JPRB, 0.33442E+01_JPRB, 0.34441E+01_JPRB, 0.35400E+01_JPRB, 0.36573E+01_JPRB, 0.38143E+01_JPRB, 0.39978E+01_JPRB ,&
&  0.41086E+01_JPRB, 0.41677E+01_JPRB, 0.42841E+01_JPRB, 0.44087E+01_JPRB, 0.39568E+01_JPRB, 0.42238E+01_JPRB, 0.33894E+01_JPRB ,&
&  0.22216E+01_JPRB, 0.17519E+01_JPRB, 0.20389E+01_JPRB, 0.20297E+01_JPRB, 0.17784E+01_JPRB, 0.15813E+01_JPRB, 0.14025E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.11155E+01_JPRB, 0.10168E+01_JPRB, 0.92187E+00_JPRB, 0.85074E+00_JPRB, 0.82175E+00_JPRB, 0.82183E+00_JPRB ,&
&  0.83205E+00_JPRB, 0.83278E+00_JPRB, 0.84202E+00_JPRB, 0.91580E+00_JPRB, 0.11411E+01_JPRB, 0.16662E+01_JPRB, 0.24319E+01_JPRB ,&
&  0.27916E+01_JPRB, 0.13423E+01_JPRB, 0.10038E+01_JPRB, 0.87697E+00_JPRB, 0.82155E+00_JPRB, 0.84138E+00_JPRB, 0.87579E+00_JPRB ,&
&  0.10175E+01_JPRB, 0.13176E+01_JPRB, 0.12091E+01_JPRB, 0.13750E+01_JPRB, 0.26285E+01_JPRB, 0.28262E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.77338E+00_JPRB, 0.63676E+00_JPRB, 0.59686E+00_JPRB, 0.58858E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.60124E+00_JPRB, 0.61838E+00_JPRB, 0.64408E+00_JPRB, 0.67605E+00_JPRB, 0.67935E+00_JPRB, 0.68690E+00_JPRB, 0.67893E+00_JPRB ,&
&  0.68090E+00_JPRB, 0.70545E+00_JPRB, 0.76460E+00_JPRB, 0.86535E+00_JPRB, 0.94318E+00_JPRB, 0.10482E+01_JPRB, 0.11964E+01_JPRB ,&
&  0.12822E+01_JPRB, 0.13035E+01_JPRB, 0.13298E+01_JPRB, 0.13750E+01_JPRB, 0.14278E+01_JPRB, 0.14925E+01_JPRB, 0.15561E+01_JPRB ,&
&  0.16341E+01_JPRB, 0.17806E+01_JPRB, 0.19197E+01_JPRB, 0.20935E+01_JPRB, 0.22652E+01_JPRB, 0.24041E+01_JPRB, 0.26909E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.19739E+01_JPRB, 0.19775E+01_JPRB, 0.19362E+01_JPRB, 0.16733E+01_JPRB, 0.15534E+01_JPRB, 0.16813E+01_JPRB ,&
&  0.20495E+01_JPRB, 0.23860E+01_JPRB, 0.27715E+01_JPRB, 0.25039E+01_JPRB, 0.27798E+01_JPRB, 0.29810E+01_JPRB, 0.34295E+01_JPRB ,&
&  0.36175E+01_JPRB, 0.37965E+01_JPRB, 0.40208E+01_JPRB, 0.42753E+01_JPRB, 0.45493E+01_JPRB, 0.48705E+01_JPRB, 0.51677E+01_JPRB ,&
&  0.54009E+01_JPRB, 0.55189E+01_JPRB, 0.57723E+01_JPRB, 0.51937E+01_JPRB, 0.45304E+01_JPRB, 0.47949E+01_JPRB, 0.45632E+01_JPRB ,&
&  0.37137E+01_JPRB, 0.25231E+01_JPRB, 0.23417E+01_JPRB, 0.22434E+01_JPRB, 0.18909E+01_JPRB, 0.16620E+01_JPRB, 0.14540E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.10137E+01_JPRB, 0.92885E+00_JPRB, 0.85497E+00_JPRB, 0.80337E+00_JPRB, 0.78992E+00_JPRB, 0.81100E+00_JPRB ,&
&  0.83177E+00_JPRB, 0.83946E+00_JPRB, 0.83152E+00_JPRB, 0.85519E+00_JPRB, 0.94463E+00_JPRB, 0.10321E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10223E+01_JPRB, 0.98375E+00_JPRB, 0.98061E+00_JPRB, 0.97423E+00_JPRB, 0.94958E+00_JPRB, 0.96699E+00_JPRB ,&
&  0.96627E+00_JPRB, 0.95141E+00_JPRB, 0.10385E+01_JPRB, 0.12590E+01_JPRB, 0.12890E+01_JPRB, 0.13458E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.91011E+00_JPRB, 0.78083E+00_JPRB, 0.69272E+00_JPRB, 0.63575E+00_JPRB, 0.58984E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.59530E+00_JPRB, 0.59346E+00_JPRB, 0.59382E+00_JPRB, 0.61171E+00_JPRB, 0.61602E+00_JPRB, 0.62206E+00_JPRB, 0.62126E+00_JPRB ,&
&  0.65254E+00_JPRB, 0.71309E+00_JPRB, 0.80626E+00_JPRB, 0.91500E+00_JPRB, 0.10006E+01_JPRB, 0.11030E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.12096E+01_JPRB, 0.12308E+01_JPRB, 0.12822E+01_JPRB, 0.13348E+01_JPRB, 0.14081E+01_JPRB, 0.14896E+01_JPRB ,&
&  0.15813E+01_JPRB, 0.17056E+01_JPRB, 0.18747E+01_JPRB, 0.20787E+01_JPRB, 0.23036E+01_JPRB, 0.25072E+01_JPRB, 0.27623E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.14601E+01_JPRB, 0.19884E+01_JPRB, 0.20846E+01_JPRB, 0.16140E+01_JPRB, 0.14567E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.17375E+01_JPRB, 0.22088E+01_JPRB, 0.24969E+01_JPRB, 0.28424E+01_JPRB, 0.29418E+01_JPRB, 0.33207E+01_JPRB, 0.36031E+01_JPRB ,&
&  0.37089E+01_JPRB, 0.38550E+01_JPRB, 0.41127E+01_JPRB, 0.44315E+01_JPRB, 0.47482E+01_JPRB, 0.50339E+01_JPRB, 0.53398E+01_JPRB ,&
&  0.56647E+01_JPRB, 0.59031E+01_JPRB, 0.61659E+01_JPRB, 0.52256E+01_JPRB, 0.54482E+01_JPRB, 0.57532E+01_JPRB, 0.64025E+01_JPRB ,&
&  0.51732E+01_JPRB, 0.31234E+01_JPRB, 0.23591E+01_JPRB, 0.23581E+01_JPRB, 0.20146E+01_JPRB, 0.17969E+01_JPRB, 0.15055E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.10306E+01_JPRB, 0.94501E+00_JPRB, 0.91983E+00_JPRB, 0.90038E+00_JPRB, 0.89747E+00_JPRB, 0.90388E+00_JPRB ,&
&  0.91350E+00_JPRB, 0.92866E+00_JPRB, 0.91824E+00_JPRB, 0.91737E+00_JPRB, 0.98210E+00_JPRB, 0.10433E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.10729E+01_JPRB, 0.10770E+01_JPRB, 0.10930E+01_JPRB, 0.11181E+01_JPRB, 0.11171E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.99650E+00_JPRB, 0.93718E+00_JPRB, 0.91327E+00_JPRB, 0.88763E+00_JPRB, 0.86393E+00_JPRB, 0.88728E+00_JPRB, 0.83520E+00_JPRB ,&
&  0.80440E+00_JPRB, 0.79384E+00_JPRB, 0.76227E+00_JPRB, 0.69063E+00_JPRB, 0.61406E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.62342E+00_JPRB, 0.61663E+00_JPRB, 0.60774E+00_JPRB, 0.61438E+00_JPRB, 0.60815E+00_JPRB, 0.61905E+00_JPRB, 0.63655E+00_JPRB ,&
&  0.67193E+00_JPRB, 0.73883E+00_JPRB, 0.82592E+00_JPRB, 0.90549E+00_JPRB, 0.99679E+00_JPRB, 0.10739E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.11249E+01_JPRB, 0.11442E+01_JPRB, 0.11969E+01_JPRB, 0.12509E+01_JPRB, 0.13253E+01_JPRB, 0.14237E+01_JPRB ,&
&  0.15439E+01_JPRB, 0.17006E+01_JPRB, 0.19001E+01_JPRB, 0.21414E+01_JPRB, 0.24621E+01_JPRB, 0.28434E+01_JPRB, 0.32928E+01_JPRB ,&
&  0.24289E+01_JPRB, 0.13003E+01_JPRB, 0.22215E+01_JPRB, 0.23720E+01_JPRB, 0.18166E+01_JPRB, 0.16281E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.19649E+01_JPRB, 0.22981E+01_JPRB, 0.26551E+01_JPRB, 0.28673E+01_JPRB, 0.32132E+01_JPRB, 0.34678E+01_JPRB ,&
&  0.36057E+01_JPRB, 0.37579E+01_JPRB, 0.39403E+01_JPRB, 0.41490E+01_JPRB, 0.44058E+01_JPRB, 0.46687E+01_JPRB, 0.49879E+01_JPRB ,&
&  0.53662E+01_JPRB, 0.56769E+01_JPRB, 0.59066E+01_JPRB, 0.51000E+01_JPRB, 0.55021E+01_JPRB, 0.52432E+01_JPRB, 0.49544E+01_JPRB ,&
&  0.42558E+01_JPRB, 0.26148E+01_JPRB, 0.19316E+01_JPRB, 0.22511E+01_JPRB, 0.20410E+01_JPRB, 0.15300E+01_JPRB, 0.15588E+01_JPRB ,&
&  0.13262E+01_JPRB, 0.11201E+01_JPRB, 0.10791E+01_JPRB, 0.10924E+01_JPRB, 0.10404E+01_JPRB, 0.10054E+01_JPRB, 0.97816E+00_JPRB ,&
&  0.10178E+01_JPRB, 0.10234E+01_JPRB, 0.10105E+01_JPRB, 0.10246E+01_JPRB, 0.10628E+01_JPRB, 0.11182E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.12024E+01_JPRB, 0.12092E+01_JPRB, 0.12344E+01_JPRB, 0.12500E+01_JPRB, 0.11732E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.10282E+01_JPRB, 0.10026E+01_JPRB, 0.97690E+00_JPRB, 0.92352E+00_JPRB, 0.91773E+00_JPRB, 0.94025E+00_JPRB ,&
&  0.96859E+00_JPRB, 0.97051E+00_JPRB, 0.89264E+00_JPRB, 0.74892E+00_JPRB, 0.65481E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.64290E+00_JPRB, 0.61490E+00_JPRB, 0.60760E+00_JPRB, 0.62784E+00_JPRB, 0.62452E+00_JPRB, 0.63116E+00_JPRB, 0.65348E+00_JPRB ,&
&  0.68927E+00_JPRB, 0.74123E+00_JPRB, 0.78712E+00_JPRB, 0.84682E+00_JPRB, 0.92324E+00_JPRB, 0.99976E+00_JPRB, 0.10286E+01_JPRB ,&
&  0.10297E+01_JPRB, 0.10397E+01_JPRB, 0.10639E+01_JPRB, 0.10986E+01_JPRB, 0.11600E+01_JPRB, 0.12286E+01_JPRB, 0.13081E+01_JPRB ,&
&  0.14269E+01_JPRB, 0.15934E+01_JPRB, 0.17765E+01_JPRB, 0.19796E+01_JPRB, 0.22586E+01_JPRB, 0.27068E+01_JPRB, 0.33949E+01_JPRB ,&
&  0.52783E+01_JPRB, 0.56203E+01_JPRB, 0.19327E+01_JPRB, 0.24374E+01_JPRB, 0.21869E+01_JPRB, 0.18990E+01_JPRB, 0.15097E+01_JPRB ,&
&  0.14312E+01_JPRB, 0.17578E+01_JPRB, 0.20852E+01_JPRB, 0.24832E+01_JPRB, 0.25352E+01_JPRB, 0.28128E+01_JPRB, 0.30505E+01_JPRB ,&
&  0.33287E+01_JPRB, 0.35170E+01_JPRB, 0.36182E+01_JPRB, 0.36719E+01_JPRB, 0.37938E+01_JPRB, 0.39799E+01_JPRB, 0.43051E+01_JPRB ,&
&  0.48044E+01_JPRB, 0.53659E+01_JPRB, 0.58081E+01_JPRB, 0.54116E+01_JPRB, 0.56885E+01_JPRB, 0.49418E+01_JPRB, 0.43209E+01_JPRB ,&
&  0.32487E+01_JPRB, 0.26704E+01_JPRB, 0.23891E+01_JPRB, 0.23314E+01_JPRB, 0.19841E+01_JPRB, 0.17340E+01_JPRB, 0.18036E+01_JPRB ,&
&  0.16403E+01_JPRB, 0.14241E+01_JPRB, 0.12811E+01_JPRB, 0.11990E+01_JPRB, 0.11584E+01_JPRB, 0.11071E+01_JPRB, 0.10951E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.11370E+01_JPRB, 0.11153E+01_JPRB, 0.11446E+01_JPRB, 0.11737E+01_JPRB, 0.12154E+01_JPRB, 0.12472E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.13075E+01_JPRB, 0.13198E+01_JPRB, 0.13167E+01_JPRB, 0.12350E+01_JPRB, 0.11552E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.11091E+01_JPRB, 0.11263E+01_JPRB, 0.11376E+01_JPRB, 0.11037E+01_JPRB, 0.11056E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.11399E+01_JPRB, 0.10279E+01_JPRB, 0.79585E+00_JPRB, 0.68974E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.69937E+00_JPRB, 0.67072E+00_JPRB, 0.66735E+00_JPRB, 0.67815E+00_JPRB, 0.68121E+00_JPRB, 0.66372E+00_JPRB, 0.66430E+00_JPRB ,&
&  0.68011E+00_JPRB, 0.70503E+00_JPRB, 0.70875E+00_JPRB, 0.74762E+00_JPRB, 0.83511E+00_JPRB, 0.92409E+00_JPRB, 0.94728E+00_JPRB ,&
&  0.94780E+00_JPRB, 0.94359E+00_JPRB, 0.94507E+00_JPRB, 0.97116E+00_JPRB, 0.10093E+01_JPRB, 0.10893E+01_JPRB, 0.11796E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.13942E+01_JPRB, 0.14872E+01_JPRB, 0.15573E+01_JPRB, 0.17091E+01_JPRB, 0.20039E+01_JPRB, 0.27939E+01_JPRB ,&
&  0.42618E+01_JPRB, 0.83602E+01_JPRB, 0.22738E+01_JPRB, 0.23751E+01_JPRB, 0.23954E+01_JPRB, 0.21771E+01_JPRB, 0.19170E+01_JPRB ,&
&  0.18781E+01_JPRB, 0.17476E+01_JPRB, 0.19529E+01_JPRB, 0.22785E+01_JPRB, 0.22022E+01_JPRB, 0.22932E+01_JPRB, 0.24387E+01_JPRB ,&
&  0.27929E+01_JPRB, 0.30539E+01_JPRB, 0.32235E+01_JPRB, 0.32183E+01_JPRB, 0.31865E+01_JPRB, 0.31558E+01_JPRB, 0.32375E+01_JPRB ,&
&  0.36465E+01_JPRB, 0.44344E+01_JPRB, 0.52955E+01_JPRB, 0.55108E+01_JPRB, 0.59750E+01_JPRB, 0.52074E+01_JPRB, 0.46852E+01_JPRB ,&
&  0.37787E+01_JPRB, 0.33283E+01_JPRB, 0.31878E+01_JPRB, 0.28482E+01_JPRB, 0.23070E+01_JPRB, 0.20548E+01_JPRB, 0.20370E+01_JPRB ,&
&  0.19118E+01_JPRB, 0.16958E+01_JPRB, 0.14650E+01_JPRB, 0.12963E+01_JPRB, 0.12553E+01_JPRB, 0.12685E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.13074E+01_JPRB, 0.12819E+01_JPRB, 0.13330E+01_JPRB, 0.13508E+01_JPRB, 0.13525E+01_JPRB, 0.13524E+01_JPRB ,&
&  0.13506E+01_JPRB, 0.13783E+01_JPRB, 0.13842E+01_JPRB, 0.13238E+01_JPRB, 0.12057E+01_JPRB, 0.11309E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.11341E+01_JPRB, 0.11712E+01_JPRB, 0.11896E+01_JPRB, 0.11615E+01_JPRB, 0.12041E+01_JPRB, 0.11919E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.11882E+01_JPRB, 0.12289E+01_JPRB, 0.12412E+01_JPRB, 0.91487E+00_JPRB, 0.77386E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.80353E+00_JPRB, 0.76596E+00_JPRB, 0.73761E+00_JPRB, 0.71438E+00_JPRB, 0.69799E+00_JPRB, 0.69881E+00_JPRB, 0.70350E+00_JPRB ,&
&  0.70366E+00_JPRB, 0.71234E+00_JPRB, 0.71084E+00_JPRB, 0.71635E+00_JPRB, 0.73773E+00_JPRB, 0.78552E+00_JPRB, 0.84514E+00_JPRB ,&
&  0.85846E+00_JPRB, 0.83583E+00_JPRB, 0.83138E+00_JPRB, 0.82887E+00_JPRB, 0.85567E+00_JPRB, 0.89935E+00_JPRB, 0.98307E+00_JPRB ,&
&  0.11038E+01_JPRB, 0.12159E+01_JPRB, 0.12627E+01_JPRB, 0.12620E+01_JPRB, 0.12983E+01_JPRB, 0.14003E+01_JPRB, 0.20674E+01_JPRB ,&
&  0.35149E+01_JPRB, 0.76106E+01_JPRB, 0.48331E+01_JPRB, 0.36494E+01_JPRB, 0.26509E+01_JPRB, 0.22811E+01_JPRB, 0.22110E+01_JPRB ,&
&  0.24860E+01_JPRB, 0.23762E+01_JPRB, 0.22863E+01_JPRB, 0.20797E+01_JPRB, 0.19298E+01_JPRB, 0.18258E+01_JPRB, 0.19194E+01_JPRB ,&
&  0.22020E+01_JPRB, 0.24761E+01_JPRB, 0.26755E+01_JPRB, 0.27301E+01_JPRB, 0.26734E+01_JPRB, 0.26103E+01_JPRB, 0.25587E+01_JPRB ,&
&  0.25451E+01_JPRB, 0.28664E+01_JPRB, 0.40383E+01_JPRB, 0.53708E+01_JPRB, 0.61398E+01_JPRB, 0.56515E+01_JPRB, 0.53104E+01_JPRB ,&
&  0.50992E+01_JPRB, 0.44272E+01_JPRB, 0.39899E+01_JPRB, 0.33482E+01_JPRB, 0.26150E+01_JPRB, 0.26381E+01_JPRB, 0.22192E+01_JPRB ,&
&  0.20285E+01_JPRB, 0.18600E+01_JPRB, 0.16564E+01_JPRB, 0.14408E+01_JPRB, 0.14121E+01_JPRB, 0.14071E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.13882E+01_JPRB, 0.14336E+01_JPRB, 0.14096E+01_JPRB, 0.14194E+01_JPRB, 0.14207E+01_JPRB, 0.14272E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.14144E+01_JPRB, 0.14364E+01_JPRB, 0.13538E+01_JPRB, 0.12647E+01_JPRB, 0.12048E+01_JPRB, 0.11363E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.12424E+01_JPRB, 0.12502E+01_JPRB, 0.12549E+01_JPRB, 0.12319E+01_JPRB, 0.12342E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.11604E+01_JPRB, 0.11213E+01_JPRB, 0.10994E+01_JPRB, 0.10035E+01_JPRB, 0.88031E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.87414E+00_JPRB, 0.83869E+00_JPRB, 0.76706E+00_JPRB, 0.70837E+00_JPRB, 0.69275E+00_JPRB, 0.69960E+00_JPRB, 0.71686E+00_JPRB ,&
&  0.72915E+00_JPRB, 0.73659E+00_JPRB, 0.73645E+00_JPRB, 0.72818E+00_JPRB, 0.72940E+00_JPRB, 0.70243E+00_JPRB, 0.72284E+00_JPRB ,&
&  0.76236E+00_JPRB, 0.77788E+00_JPRB, 0.79627E+00_JPRB, 0.76954E+00_JPRB, 0.76801E+00_JPRB, 0.76506E+00_JPRB, 0.80613E+00_JPRB ,&
&  0.89989E+00_JPRB, 0.10326E+01_JPRB, 0.11149E+01_JPRB, 0.11323E+01_JPRB, 0.11455E+01_JPRB, 0.12362E+01_JPRB, 0.16661E+01_JPRB ,&
&  0.28813E+01_JPRB, 0.64682E+01_JPRB, 0.28414E+01_JPRB, 0.45753E+01_JPRB, 0.31739E+01_JPRB, 0.24609E+01_JPRB, 0.19675E+01_JPRB ,&
&  0.21017E+01_JPRB, 0.25471E+01_JPRB, 0.20891E+01_JPRB, 0.18214E+01_JPRB, 0.16867E+01_JPRB, 0.16319E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.17181E+01_JPRB, 0.19487E+01_JPRB, 0.21772E+01_JPRB, 0.22817E+01_JPRB, 0.22280E+01_JPRB, 0.21006E+01_JPRB, 0.20591E+01_JPRB ,&
&  0.20346E+01_JPRB, 0.20625E+01_JPRB, 0.25592E+01_JPRB, 0.33904E+01_JPRB, 0.53304E+01_JPRB, 0.60119E+01_JPRB, 0.61830E+01_JPRB ,&
&  0.63019E+01_JPRB, 0.56877E+01_JPRB, 0.47394E+01_JPRB, 0.38714E+01_JPRB, 0.31232E+01_JPRB, 0.29760E+01_JPRB, 0.24924E+01_JPRB ,&
&  0.21856E+01_JPRB, 0.19773E+01_JPRB, 0.17870E+01_JPRB, 0.16166E+01_JPRB, 0.15267E+01_JPRB, 0.14939E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.15028E+01_JPRB, 0.15733E+01_JPRB, 0.15496E+01_JPRB, 0.14905E+01_JPRB, 0.14607E+01_JPRB, 0.14531E+01_JPRB, 0.14383E+01_JPRB ,&
&  0.14366E+01_JPRB, 0.14634E+01_JPRB, 0.13791E+01_JPRB, 0.12543E+01_JPRB, 0.12127E+01_JPRB, 0.11933E+01_JPRB, 0.11752E+01_JPRB ,&
&  0.12477E+01_JPRB, 0.13255E+01_JPRB, 0.13202E+01_JPRB, 0.12702E+01_JPRB, 0.12603E+01_JPRB, 0.13012E+01_JPRB, 0.12194E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.11024E+01_JPRB, 0.10561E+01_JPRB, 0.10097E+01_JPRB, 0.94734E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.88858E+00_JPRB, 0.82324E+00_JPRB, 0.75049E+00_JPRB, 0.71279E+00_JPRB, 0.71113E+00_JPRB, 0.69971E+00_JPRB, 0.71375E+00_JPRB ,&
&  0.74394E+00_JPRB, 0.75087E+00_JPRB, 0.75232E+00_JPRB, 0.74022E+00_JPRB, 0.72797E+00_JPRB, 0.69949E+00_JPRB, 0.67108E+00_JPRB ,&
&  0.69678E+00_JPRB, 0.76986E+00_JPRB, 0.80497E+00_JPRB, 0.79116E+00_JPRB, 0.76839E+00_JPRB, 0.74398E+00_JPRB, 0.74127E+00_JPRB ,&
&  0.77563E+00_JPRB, 0.85764E+00_JPRB, 0.95039E+00_JPRB, 0.99987E+00_JPRB, 0.10222E+01_JPRB, 0.10619E+01_JPRB, 0.13280E+01_JPRB ,&
&  0.22069E+01_JPRB, 0.66412E+01_JPRB, 0.26897E+01_JPRB, 0.49525E+01_JPRB, 0.36107E+01_JPRB, 0.28011E+01_JPRB, 0.20606E+01_JPRB ,&
&  0.19362E+01_JPRB, 0.21005E+01_JPRB, 0.18854E+01_JPRB, 0.17096E+01_JPRB, 0.15738E+01_JPRB, 0.15277E+01_JPRB, 0.15353E+01_JPRB ,&
&  0.15491E+01_JPRB, 0.15737E+01_JPRB, 0.16728E+01_JPRB, 0.17646E+01_JPRB, 0.17762E+01_JPRB, 0.17069E+01_JPRB, 0.16370E+01_JPRB ,&
&  0.16009E+01_JPRB, 0.16106E+01_JPRB, 0.18492E+01_JPRB, 0.24623E+01_JPRB, 0.38280E+01_JPRB, 0.55893E+01_JPRB, 0.74179E+01_JPRB ,&
&  0.78855E+01_JPRB, 0.72216E+01_JPRB, 0.58107E+01_JPRB, 0.47461E+01_JPRB, 0.39558E+01_JPRB, 0.36264E+01_JPRB, 0.30561E+01_JPRB ,&
&  0.25972E+01_JPRB, 0.22817E+01_JPRB, 0.20332E+01_JPRB, 0.18732E+01_JPRB, 0.17309E+01_JPRB, 0.16854E+01_JPRB, 0.16439E+01_JPRB ,&
&  0.16604E+01_JPRB, 0.17654E+01_JPRB, 0.17644E+01_JPRB, 0.16727E+01_JPRB, 0.15528E+01_JPRB, 0.14950E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.14108E+01_JPRB, 0.14393E+01_JPRB, 0.14090E+01_JPRB, 0.13204E+01_JPRB, 0.12657E+01_JPRB, 0.12505E+01_JPRB, 0.12476E+01_JPRB ,&
&  0.13280E+01_JPRB, 0.13933E+01_JPRB, 0.13834E+01_JPRB, 0.13495E+01_JPRB, 0.13055E+01_JPRB, 0.13513E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.11728E+01_JPRB, 0.11166E+01_JPRB, 0.10628E+01_JPRB, 0.99307E+00_JPRB, 0.95715E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.91186E+00_JPRB, 0.82943E+00_JPRB, 0.77072E+00_JPRB, 0.73160E+00_JPRB, 0.72677E+00_JPRB, 0.71899E+00_JPRB, 0.71861E+00_JPRB ,&
&  0.74099E+00_JPRB, 0.75458E+00_JPRB, 0.76783E+00_JPRB, 0.75672E+00_JPRB, 0.73505E+00_JPRB, 0.73041E+00_JPRB, 0.71049E+00_JPRB ,&
&  0.70498E+00_JPRB, 0.75672E+00_JPRB, 0.80752E+00_JPRB, 0.81723E+00_JPRB, 0.79463E+00_JPRB, 0.75228E+00_JPRB, 0.73246E+00_JPRB ,&
&  0.73886E+00_JPRB, 0.76334E+00_JPRB, 0.81146E+00_JPRB, 0.86343E+00_JPRB, 0.90028E+00_JPRB, 0.92883E+00_JPRB, 0.10065E+01_JPRB ,&
&  0.14064E+01_JPRB, 0.22158E+01_JPRB, 0.38538E+01_JPRB, 0.48287E+01_JPRB, 0.38442E+01_JPRB, 0.29492E+01_JPRB, 0.23062E+01_JPRB ,&
&  0.20352E+01_JPRB, 0.19081E+01_JPRB, 0.17719E+01_JPRB, 0.16214E+01_JPRB, 0.14920E+01_JPRB, 0.14031E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.13440E+01_JPRB, 0.13239E+01_JPRB, 0.13291E+01_JPRB, 0.13360E+01_JPRB, 0.13255E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.13085E+01_JPRB, 0.14073E+01_JPRB, 0.18427E+01_JPRB, 0.32067E+01_JPRB, 0.49426E+01_JPRB, 0.74399E+01_JPRB ,&
&  0.88302E+01_JPRB, 0.80323E+01_JPRB, 0.66887E+01_JPRB, 0.55535E+01_JPRB, 0.47306E+01_JPRB, 0.42710E+01_JPRB, 0.37880E+01_JPRB ,&
&  0.33271E+01_JPRB, 0.28708E+01_JPRB, 0.24936E+01_JPRB, 0.22591E+01_JPRB, 0.20103E+01_JPRB, 0.18978E+01_JPRB, 0.19034E+01_JPRB ,&
&  0.18552E+01_JPRB, 0.18568E+01_JPRB, 0.18767E+01_JPRB, 0.19462E+01_JPRB, 0.18241E+01_JPRB, 0.16778E+01_JPRB, 0.15419E+01_JPRB ,&
&  0.14361E+01_JPRB, 0.14240E+01_JPRB, 0.13899E+01_JPRB, 0.13769E+01_JPRB, 0.13711E+01_JPRB, 0.13569E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.13662E+01_JPRB, 0.14052E+01_JPRB, 0.14160E+01_JPRB, 0.14003E+01_JPRB, 0.13008E+01_JPRB, 0.13663E+01_JPRB, 0.12786E+01_JPRB ,&
&  0.11923E+01_JPRB, 0.11351E+01_JPRB, 0.10901E+01_JPRB, 0.10349E+01_JPRB, 0.99635E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.10060E+01_JPRB, 0.92673E+00_JPRB, 0.84518E+00_JPRB, 0.79245E+00_JPRB, 0.75446E+00_JPRB, 0.73799E+00_JPRB, 0.72834E+00_JPRB ,&
&  0.73548E+00_JPRB, 0.74476E+00_JPRB, 0.76058E+00_JPRB, 0.75899E+00_JPRB, 0.74148E+00_JPRB, 0.75236E+00_JPRB, 0.77539E+00_JPRB ,&
&  0.77385E+00_JPRB, 0.78766E+00_JPRB, 0.81063E+00_JPRB, 0.79922E+00_JPRB, 0.77257E+00_JPRB, 0.74493E+00_JPRB, 0.72338E+00_JPRB ,&
&  0.72480E+00_JPRB, 0.73021E+00_JPRB, 0.74537E+00_JPRB, 0.77527E+00_JPRB, 0.81729E+00_JPRB, 0.85378E+00_JPRB, 0.88455E+00_JPRB ,&
&  0.10247E+01_JPRB, 0.24294E+01_JPRB, 0.35526E+01_JPRB, 0.41816E+01_JPRB, 0.37839E+01_JPRB, 0.29332E+01_JPRB, 0.24461E+01_JPRB ,&
&  0.20001E+01_JPRB, 0.17857E+01_JPRB, 0.16488E+01_JPRB, 0.15332E+01_JPRB, 0.14030E+01_JPRB, 0.13256E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.12546E+01_JPRB, 0.11938E+01_JPRB, 0.11249E+01_JPRB, 0.10695E+01_JPRB, 0.10571E+01_JPRB, 0.10608E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.11017E+01_JPRB, 0.11628E+01_JPRB, 0.13438E+01_JPRB, 0.23350E+01_JPRB, 0.29532E+01_JPRB, 0.54613E+01_JPRB ,&
&  0.67614E+01_JPRB, 0.58674E+01_JPRB, 0.52834E+01_JPRB, 0.48567E+01_JPRB, 0.43834E+01_JPRB, 0.39596E+01_JPRB, 0.36834E+01_JPRB ,&
&  0.34024E+01_JPRB, 0.30876E+01_JPRB, 0.28231E+01_JPRB, 0.26275E+01_JPRB, 0.24403E+01_JPRB, 0.22988E+01_JPRB, 0.21304E+01_JPRB ,&
&  0.20670E+01_JPRB, 0.19780E+01_JPRB, 0.19467E+01_JPRB, 0.20054E+01_JPRB, 0.20425E+01_JPRB, 0.19234E+01_JPRB, 0.17676E+01_JPRB ,&
&  0.16043E+01_JPRB, 0.15069E+01_JPRB, 0.14389E+01_JPRB, 0.14137E+01_JPRB, 0.14291E+01_JPRB, 0.14406E+01_JPRB, 0.14319E+01_JPRB ,&
&  0.13970E+01_JPRB, 0.13907E+01_JPRB, 0.14198E+01_JPRB, 0.13881E+01_JPRB, 0.12981E+01_JPRB, 0.13700E+01_JPRB, 0.12921E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.11256E+01_JPRB, 0.10776E+01_JPRB, 0.10330E+01_JPRB, 0.10445E+01_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.10524E+01_JPRB, 0.10023E+01_JPRB, 0.93726E+00_JPRB, 0.88288E+00_JPRB, 0.83601E+00_JPRB, 0.80469E+00_JPRB, 0.79488E+00_JPRB ,&
&  0.78629E+00_JPRB, 0.77451E+00_JPRB, 0.78006E+00_JPRB, 0.77914E+00_JPRB, 0.77998E+00_JPRB, 0.78556E+00_JPRB, 0.80885E+00_JPRB ,&
&  0.82529E+00_JPRB, 0.82651E+00_JPRB, 0.82116E+00_JPRB, 0.80377E+00_JPRB, 0.78187E+00_JPRB, 0.75689E+00_JPRB, 0.73994E+00_JPRB ,&
&  0.73059E+00_JPRB, 0.73019E+00_JPRB, 0.73606E+00_JPRB, 0.74998E+00_JPRB, 0.77976E+00_JPRB, 0.81990E+00_JPRB, 0.85803E+00_JPRB ,&
&  0.89384E+00_JPRB, 0.14982E+01_JPRB, 0.21119E+01_JPRB, 0.32007E+01_JPRB, 0.34974E+01_JPRB, 0.29293E+01_JPRB, 0.24214E+01_JPRB ,&
&  0.20229E+01_JPRB, 0.17543E+01_JPRB, 0.15797E+01_JPRB, 0.14558E+01_JPRB, 0.13320E+01_JPRB, 0.12608E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.10979E+01_JPRB, 0.10303E+01_JPRB, 0.97973E+00_JPRB, 0.96460E+00_JPRB, 0.97065E+00_JPRB, 0.97097E+00_JPRB ,&
&  0.96723E+00_JPRB, 0.97785E+00_JPRB, 0.99599E+00_JPRB, 0.10492E+01_JPRB, 0.14443E+01_JPRB, 0.18935E+01_JPRB, 0.29342E+01_JPRB ,&
&  0.30210E+01_JPRB, 0.28847E+01_JPRB, 0.28666E+01_JPRB, 0.28396E+01_JPRB, 0.28822E+01_JPRB, 0.29038E+01_JPRB, 0.29543E+01_JPRB ,&
&  0.29843E+01_JPRB, 0.29493E+01_JPRB, 0.28560E+01_JPRB, 0.27805E+01_JPRB, 0.27058E+01_JPRB, 0.26168E+01_JPRB, 0.24529E+01_JPRB ,&
&  0.23226E+01_JPRB, 0.22145E+01_JPRB, 0.21415E+01_JPRB, 0.20737E+01_JPRB, 0.20674E+01_JPRB, 0.20133E+01_JPRB, 0.18991E+01_JPRB ,&
&  0.17479E+01_JPRB, 0.16010E+01_JPRB, 0.15301E+01_JPRB, 0.14968E+01_JPRB, 0.14720E+01_JPRB, 0.14400E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.14377E+01_JPRB, 0.14324E+01_JPRB, 0.14147E+01_JPRB, 0.13761E+01_JPRB, 0.12967E+01_JPRB, 0.13529E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.11359E+01_JPRB, 0.10663E+01_JPRB, 0.10282E+01_JPRB, 0.10688E+01_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.11366E+01_JPRB, 0.10868E+01_JPRB, 0.10300E+01_JPRB, 0.95499E+00_JPRB, 0.89326E+00_JPRB, 0.86430E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.82463E+00_JPRB, 0.80459E+00_JPRB, 0.80260E+00_JPRB, 0.81134E+00_JPRB, 0.82142E+00_JPRB, 0.82925E+00_JPRB, 0.82843E+00_JPRB ,&
&  0.82893E+00_JPRB, 0.83123E+00_JPRB, 0.82012E+00_JPRB, 0.80220E+00_JPRB, 0.78288E+00_JPRB, 0.76632E+00_JPRB, 0.75356E+00_JPRB ,&
&  0.74575E+00_JPRB, 0.74719E+00_JPRB, 0.74877E+00_JPRB, 0.75938E+00_JPRB, 0.78046E+00_JPRB, 0.80882E+00_JPRB, 0.84750E+00_JPRB ,&
&  0.88350E+00_JPRB, 0.93965E+00_JPRB, 0.13538E+01_JPRB, 0.19470E+01_JPRB, 0.24497E+01_JPRB, 0.24865E+01_JPRB, 0.22357E+01_JPRB ,&
&  0.20027E+01_JPRB, 0.17678E+01_JPRB, 0.15854E+01_JPRB, 0.14345E+01_JPRB, 0.13102E+01_JPRB, 0.12151E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.10845E+01_JPRB, 0.10337E+01_JPRB, 0.98696E+00_JPRB, 0.94583E+00_JPRB, 0.92850E+00_JPRB, 0.91830E+00_JPRB, 0.90977E+00_JPRB ,&
&  0.90260E+00_JPRB, 0.89956E+00_JPRB, 0.90032E+00_JPRB, 0.90952E+00_JPRB, 0.96489E+00_JPRB, 0.10892E+01_JPRB, 0.13870E+01_JPRB ,&
&  0.15511E+01_JPRB, 0.16500E+01_JPRB, 0.16804E+01_JPRB, 0.16821E+01_JPRB, 0.17225E+01_JPRB, 0.18202E+01_JPRB, 0.19727E+01_JPRB ,&
&  0.21581E+01_JPRB, 0.23293E+01_JPRB, 0.24639E+01_JPRB, 0.25465E+01_JPRB, 0.25648E+01_JPRB, 0.25285E+01_JPRB, 0.24558E+01_JPRB ,&
&  0.23739E+01_JPRB, 0.22494E+01_JPRB, 0.21254E+01_JPRB, 0.20215E+01_JPRB, 0.19653E+01_JPRB, 0.19274E+01_JPRB, 0.18590E+01_JPRB ,&
&  0.17576E+01_JPRB, 0.16387E+01_JPRB, 0.15476E+01_JPRB, 0.14958E+01_JPRB, 0.14655E+01_JPRB, 0.14330E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.14129E+01_JPRB, 0.14085E+01_JPRB, 0.13517E+01_JPRB, 0.13321E+01_JPRB, 0.13617E+01_JPRB, 0.13080E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11371E+01_JPRB, 0.10822E+01_JPRB, 0.10391E+01_JPRB, 0.11266E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.10551E+01_JPRB, 0.10470E+01_JPRB, 0.10079E+01_JPRB, 0.95241E+00_JPRB, 0.89818E+00_JPRB, 0.85412E+00_JPRB, 0.83048E+00_JPRB ,&
&  0.81397E+00_JPRB, 0.80398E+00_JPRB, 0.81103E+00_JPRB, 0.82836E+00_JPRB, 0.84876E+00_JPRB, 0.85667E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.83924E+00_JPRB, 0.81614E+00_JPRB, 0.79859E+00_JPRB, 0.78677E+00_JPRB, 0.77302E+00_JPRB, 0.76516E+00_JPRB, 0.76046E+00_JPRB ,&
&  0.76160E+00_JPRB, 0.76552E+00_JPRB, 0.76767E+00_JPRB, 0.77683E+00_JPRB, 0.79335E+00_JPRB, 0.81986E+00_JPRB, 0.85053E+00_JPRB ,&
&  0.86898E+00_JPRB, 0.79211E+00_JPRB, 0.95372E+00_JPRB, 0.13079E+01_JPRB, 0.17084E+01_JPRB, 0.19176E+01_JPRB, 0.19103E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.16732E+01_JPRB, 0.15443E+01_JPRB, 0.14006E+01_JPRB, 0.12736E+01_JPRB, 0.11713E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.10146E+01_JPRB, 0.96633E+00_JPRB, 0.93719E+00_JPRB, 0.91611E+00_JPRB, 0.89695E+00_JPRB, 0.86808E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.84529E+00_JPRB, 0.84371E+00_JPRB, 0.84470E+00_JPRB, 0.84908E+00_JPRB, 0.87364E+00_JPRB, 0.91436E+00_JPRB, 0.97053E+00_JPRB ,&
&  0.10317E+01_JPRB, 0.10879E+01_JPRB, 0.11195E+01_JPRB, 0.11362E+01_JPRB, 0.11462E+01_JPRB, 0.12159E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.13988E+01_JPRB, 0.15790E+01_JPRB, 0.17911E+01_JPRB, 0.19290E+01_JPRB, 0.19621E+01_JPRB, 0.19165E+01_JPRB, 0.18349E+01_JPRB ,&
&  0.17725E+01_JPRB, 0.17047E+01_JPRB, 0.16344E+01_JPRB, 0.15731E+01_JPRB, 0.15408E+01_JPRB, 0.15489E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.15595E+01_JPRB, 0.15307E+01_JPRB, 0.14919E+01_JPRB, 0.14722E+01_JPRB, 0.14608E+01_JPRB, 0.14352E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.13341E+01_JPRB, 0.13029E+01_JPRB, 0.12957E+01_JPRB, 0.12524E+01_JPRB, 0.12177E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.11404E+01_JPRB, 0.10994E+01_JPRB, 0.10504E+01_JPRB, 0.10058E+01_JPRB, 0.98879E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.93685E+00_JPRB, 0.97266E+00_JPRB, 0.97745E+00_JPRB, 0.94665E+00_JPRB, 0.90775E+00_JPRB, 0.87850E+00_JPRB, 0.85819E+00_JPRB ,&
&  0.85058E+00_JPRB, 0.84416E+00_JPRB, 0.84851E+00_JPRB, 0.86264E+00_JPRB, 0.87767E+00_JPRB, 0.88046E+00_JPRB, 0.86117E+00_JPRB ,&
&  0.83419E+00_JPRB, 0.81032E+00_JPRB, 0.79500E+00_JPRB, 0.78644E+00_JPRB, 0.77810E+00_JPRB, 0.77334E+00_JPRB, 0.77991E+00_JPRB ,&
&  0.78852E+00_JPRB, 0.79961E+00_JPRB, 0.81275E+00_JPRB, 0.82471E+00_JPRB, 0.83511E+00_JPRB, 0.84599E+00_JPRB, 0.87022E+00_JPRB ,&
&  0.88980E+00_JPRB, 0.72639E+00_JPRB, 0.78203E+00_JPRB, 0.98293E+00_JPRB, 0.12231E+01_JPRB, 0.14970E+01_JPRB, 0.16115E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.14998E+01_JPRB, 0.14030E+01_JPRB, 0.12991E+01_JPRB, 0.11953E+01_JPRB, 0.11076E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.97565E+00_JPRB, 0.93402E+00_JPRB, 0.90611E+00_JPRB, 0.87888E+00_JPRB, 0.85958E+00_JPRB, 0.84292E+00_JPRB, 0.82864E+00_JPRB ,&
&  0.81716E+00_JPRB, 0.80619E+00_JPRB, 0.80345E+00_JPRB, 0.80678E+00_JPRB, 0.80718E+00_JPRB, 0.81625E+00_JPRB, 0.81539E+00_JPRB ,&
&  0.84157E+00_JPRB, 0.87386E+00_JPRB, 0.89094E+00_JPRB, 0.89836E+00_JPRB, 0.91063E+00_JPRB, 0.93483E+00_JPRB, 0.96997E+00_JPRB ,&
&  0.10166E+01_JPRB, 0.11080E+01_JPRB, 0.12117E+01_JPRB, 0.13068E+01_JPRB, 0.13637E+01_JPRB, 0.13762E+01_JPRB, 0.13645E+01_JPRB ,&
&  0.13664E+01_JPRB, 0.13588E+01_JPRB, 0.13393E+01_JPRB, 0.13097E+01_JPRB, 0.12712E+01_JPRB, 0.12543E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.13354E+01_JPRB, 0.13549E+01_JPRB, 0.13723E+01_JPRB, 0.13651E+01_JPRB, 0.13336E+01_JPRB, 0.13011E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.12159E+01_JPRB, 0.11725E+01_JPRB, 0.10889E+01_JPRB, 0.11337E+01_JPRB, 0.11226E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10299E+01_JPRB, 0.99717E+00_JPRB, 0.82427E+00_JPRB, 0.89904E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.87814E+00_JPRB, 0.91119E+00_JPRB, 0.92477E+00_JPRB, 0.91923E+00_JPRB, 0.89992E+00_JPRB, 0.88689E+00_JPRB, 0.87634E+00_JPRB ,&
&  0.86888E+00_JPRB, 0.86307E+00_JPRB, 0.87136E+00_JPRB, 0.87738E+00_JPRB, 0.87787E+00_JPRB, 0.86653E+00_JPRB, 0.85319E+00_JPRB ,&
&  0.83419E+00_JPRB, 0.81212E+00_JPRB, 0.79522E+00_JPRB, 0.78078E+00_JPRB, 0.77705E+00_JPRB, 0.78341E+00_JPRB, 0.79801E+00_JPRB ,&
&  0.81780E+00_JPRB, 0.83193E+00_JPRB, 0.84424E+00_JPRB, 0.85825E+00_JPRB, 0.86298E+00_JPRB, 0.86583E+00_JPRB, 0.87208E+00_JPRB ,&
&  0.88266E+00_JPRB, 0.70006E+00_JPRB, 0.75528E+00_JPRB, 0.83933E+00_JPRB, 0.93913E+00_JPRB, 0.10940E+01_JPRB, 0.12206E+01_JPRB ,&
&  0.12642E+01_JPRB, 0.12105E+01_JPRB, 0.11589E+01_JPRB, 0.11020E+01_JPRB, 0.10449E+01_JPRB, 0.99496E+00_JPRB, 0.94484E+00_JPRB ,&
&  0.90579E+00_JPRB, 0.88123E+00_JPRB, 0.86889E+00_JPRB, 0.85667E+00_JPRB, 0.84253E+00_JPRB, 0.82532E+00_JPRB, 0.80609E+00_JPRB ,&
&  0.78660E+00_JPRB, 0.77395E+00_JPRB, 0.76437E+00_JPRB, 0.75743E+00_JPRB, 0.75733E+00_JPRB, 0.76065E+00_JPRB, 0.76205E+00_JPRB ,&
&  0.77190E+00_JPRB, 0.78220E+00_JPRB, 0.78863E+00_JPRB, 0.78978E+00_JPRB, 0.79127E+00_JPRB, 0.79720E+00_JPRB, 0.81940E+00_JPRB ,&
&  0.84311E+00_JPRB, 0.88175E+00_JPRB, 0.93278E+00_JPRB, 0.99910E+00_JPRB, 0.10524E+01_JPRB, 0.10799E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.10906E+01_JPRB, 0.10680E+01_JPRB, 0.10459E+01_JPRB, 0.10273E+01_JPRB, 0.10205E+01_JPRB, 0.10210E+01_JPRB ,&
&  0.10341E+01_JPRB, 0.10566E+01_JPRB, 0.10776E+01_JPRB, 0.11021E+01_JPRB, 0.11244E+01_JPRB, 0.11201E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.10315E+01_JPRB, 0.10162E+01_JPRB, 0.10242E+01_JPRB, 0.10193E+01_JPRB, 0.10062E+01_JPRB, 0.97861E+00_JPRB ,&
&  0.95826E+00_JPRB, 0.93991E+00_JPRB, 0.73358E+00_JPRB, 0.82362E+00_JPRB, 0.84464E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.86346E+00_JPRB, 0.86031E+00_JPRB, 0.85600E+00_JPRB, 0.86249E+00_JPRB, 0.86871E+00_JPRB, 0.86929E+00_JPRB, 0.86873E+00_JPRB ,&
&  0.86559E+00_JPRB, 0.86089E+00_JPRB, 0.86769E+00_JPRB, 0.87357E+00_JPRB, 0.87247E+00_JPRB, 0.86307E+00_JPRB, 0.85273E+00_JPRB ,&
&  0.84429E+00_JPRB, 0.83293E+00_JPRB, 0.82190E+00_JPRB, 0.81644E+00_JPRB, 0.81763E+00_JPRB, 0.83030E+00_JPRB, 0.85029E+00_JPRB ,&
&  0.86515E+00_JPRB, 0.86386E+00_JPRB, 0.86749E+00_JPRB, 0.87934E+00_JPRB, 0.88327E+00_JPRB, 0.88668E+00_JPRB, 0.89499E+00_JPRB ,&
&  0.77703E+00_JPRB, 0.65543E+00_JPRB, 0.72670E+00_JPRB, 0.79407E+00_JPRB, 0.85198E+00_JPRB, 0.92273E+00_JPRB, 0.10128E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.10308E+01_JPRB, 0.10083E+01_JPRB, 0.97769E+00_JPRB, 0.94296E+00_JPRB, 0.91778E+00_JPRB, 0.89777E+00_JPRB ,&
&  0.89149E+00_JPRB, 0.87922E+00_JPRB, 0.86045E+00_JPRB, 0.84479E+00_JPRB, 0.82999E+00_JPRB, 0.81077E+00_JPRB, 0.79208E+00_JPRB ,&
&  0.77279E+00_JPRB, 0.75593E+00_JPRB, 0.74873E+00_JPRB, 0.74373E+00_JPRB, 0.74375E+00_JPRB, 0.74199E+00_JPRB, 0.73563E+00_JPRB ,&
&  0.73173E+00_JPRB, 0.73305E+00_JPRB, 0.74192E+00_JPRB, 0.75072E+00_JPRB, 0.75447E+00_JPRB, 0.75487E+00_JPRB, 0.76648E+00_JPRB ,&
&  0.77884E+00_JPRB, 0.79970E+00_JPRB, 0.82564E+00_JPRB, 0.86329E+00_JPRB, 0.89419E+00_JPRB, 0.91091E+00_JPRB, 0.92266E+00_JPRB ,&
&  0.93530E+00_JPRB, 0.93205E+00_JPRB, 0.91475E+00_JPRB, 0.90103E+00_JPRB, 0.89417E+00_JPRB, 0.88535E+00_JPRB, 0.88037E+00_JPRB ,&
&  0.88635E+00_JPRB, 0.89496E+00_JPRB, 0.89831E+00_JPRB, 0.90858E+00_JPRB, 0.92194E+00_JPRB, 0.93236E+00_JPRB, 0.93678E+00_JPRB ,&
&  0.91613E+00_JPRB, 0.91543E+00_JPRB, 0.91354E+00_JPRB, 0.92147E+00_JPRB, 0.93011E+00_JPRB, 0.91576E+00_JPRB, 0.89959E+00_JPRB ,&
&  0.88460E+00_JPRB, 0.85821E+00_JPRB, 0.85440E+00_JPRB, 0.85261E+00_JPRB, 0.85434E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.81014E+00_JPRB, 0.80151E+00_JPRB, 0.79652E+00_JPRB, 0.80677E+00_JPRB, 0.82415E+00_JPRB, 0.83794E+00_JPRB, 0.84001E+00_JPRB ,&
&  0.84106E+00_JPRB, 0.84610E+00_JPRB, 0.85422E+00_JPRB, 0.85556E+00_JPRB, 0.84708E+00_JPRB, 0.82849E+00_JPRB, 0.81787E+00_JPRB ,&
&  0.81362E+00_JPRB, 0.81269E+00_JPRB, 0.81112E+00_JPRB, 0.81427E+00_JPRB, 0.83259E+00_JPRB, 0.86211E+00_JPRB, 0.89013E+00_JPRB ,&
&  0.90374E+00_JPRB, 0.89799E+00_JPRB, 0.89168E+00_JPRB, 0.88689E+00_JPRB, 0.88617E+00_JPRB, 0.89169E+00_JPRB, 0.89315E+00_JPRB ,&
&  0.88656E+00_JPRB, 0.74195E+00_JPRB, 0.80258E+00_JPRB, 0.82773E+00_JPRB, 0.85766E+00_JPRB, 0.89293E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.96298E+00_JPRB, 0.94982E+00_JPRB, 0.92279E+00_JPRB, 0.90260E+00_JPRB, 0.88381E+00_JPRB, 0.87501E+00_JPRB, 0.86867E+00_JPRB ,&
&  0.86551E+00_JPRB, 0.86184E+00_JPRB, 0.83934E+00_JPRB, 0.80342E+00_JPRB, 0.78243E+00_JPRB, 0.76758E+00_JPRB, 0.75423E+00_JPRB ,&
&  0.74151E+00_JPRB, 0.72583E+00_JPRB, 0.71094E+00_JPRB, 0.69874E+00_JPRB, 0.69834E+00_JPRB, 0.69491E+00_JPRB, 0.68092E+00_JPRB ,&
&  0.67455E+00_JPRB, 0.67175E+00_JPRB, 0.67912E+00_JPRB, 0.69021E+00_JPRB, 0.69559E+00_JPRB, 0.69271E+00_JPRB, 0.70416E+00_JPRB ,&
&  0.72938E+00_JPRB, 0.74732E+00_JPRB, 0.76265E+00_JPRB, 0.78125E+00_JPRB, 0.79248E+00_JPRB, 0.79597E+00_JPRB, 0.80140E+00_JPRB ,&
&  0.81257E+00_JPRB, 0.81391E+00_JPRB, 0.80622E+00_JPRB, 0.80043E+00_JPRB, 0.80232E+00_JPRB, 0.80389E+00_JPRB, 0.81322E+00_JPRB ,&
&  0.82200E+00_JPRB, 0.82546E+00_JPRB, 0.82202E+00_JPRB, 0.82480E+00_JPRB, 0.83142E+00_JPRB, 0.84279E+00_JPRB, 0.84979E+00_JPRB ,&
&  0.85712E+00_JPRB, 0.85133E+00_JPRB, 0.83710E+00_JPRB, 0.83692E+00_JPRB, 0.84206E+00_JPRB, 0.83931E+00_JPRB, 0.82485E+00_JPRB ,&
&  0.81135E+00_JPRB, 0.80091E+00_JPRB, 0.80211E+00_JPRB, 0.80254E+00_JPRB, 0.80696E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.79157E+00_JPRB, 0.78744E+00_JPRB, 0.78610E+00_JPRB, 0.79728E+00_JPRB, 0.81540E+00_JPRB, 0.82074E+00_JPRB, 0.82362E+00_JPRB ,&
&  0.83737E+00_JPRB, 0.84779E+00_JPRB, 0.84782E+00_JPRB, 0.84611E+00_JPRB, 0.83953E+00_JPRB, 0.82463E+00_JPRB, 0.81281E+00_JPRB ,&
&  0.81694E+00_JPRB, 0.82148E+00_JPRB, 0.82237E+00_JPRB, 0.84518E+00_JPRB, 0.87583E+00_JPRB, 0.91262E+00_JPRB, 0.93747E+00_JPRB ,&
&  0.94614E+00_JPRB, 0.93860E+00_JPRB, 0.93484E+00_JPRB, 0.92971E+00_JPRB, 0.92445E+00_JPRB, 0.92641E+00_JPRB, 0.92103E+00_JPRB ,&
&  0.91116E+00_JPRB, 0.82596E+00_JPRB, 0.85840E+00_JPRB, 0.86896E+00_JPRB, 0.88625E+00_JPRB, 0.91315E+00_JPRB, 0.95035E+00_JPRB ,&
&  0.95620E+00_JPRB, 0.94892E+00_JPRB, 0.93337E+00_JPRB, 0.90656E+00_JPRB, 0.88757E+00_JPRB, 0.87731E+00_JPRB, 0.85941E+00_JPRB ,&
&  0.83143E+00_JPRB, 0.79400E+00_JPRB, 0.76071E+00_JPRB, 0.73792E+00_JPRB, 0.72697E+00_JPRB, 0.72285E+00_JPRB, 0.71628E+00_JPRB ,&
&  0.71218E+00_JPRB, 0.70865E+00_JPRB, 0.69639E+00_JPRB, 0.68534E+00_JPRB, 0.67365E+00_JPRB, 0.66502E+00_JPRB, 0.65066E+00_JPRB ,&
&  0.64215E+00_JPRB, 0.63080E+00_JPRB, 0.62969E+00_JPRB, 0.64978E+00_JPRB, 0.67184E+00_JPRB, 0.67820E+00_JPRB, 0.69310E+00_JPRB ,&
&  0.71358E+00_JPRB, 0.72358E+00_JPRB, 0.72647E+00_JPRB, 0.72436E+00_JPRB, 0.71724E+00_JPRB, 0.71687E+00_JPRB, 0.72576E+00_JPRB ,&
&  0.73452E+00_JPRB, 0.74592E+00_JPRB, 0.75093E+00_JPRB, 0.75632E+00_JPRB, 0.76353E+00_JPRB, 0.76915E+00_JPRB, 0.77999E+00_JPRB ,&
&  0.78564E+00_JPRB, 0.78943E+00_JPRB, 0.80022E+00_JPRB, 0.81163E+00_JPRB, 0.81715E+00_JPRB, 0.82023E+00_JPRB, 0.83045E+00_JPRB ,&
&  0.83526E+00_JPRB, 0.83412E+00_JPRB, 0.82500E+00_JPRB, 0.81217E+00_JPRB, 0.80345E+00_JPRB, 0.79069E+00_JPRB, 0.77013E+00_JPRB ,&
&  0.75064E+00_JPRB, 0.74748E+00_JPRB, 0.75628E+00_JPRB, 0.76609E+00_JPRB, 0.78282E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.77259E+00_JPRB, 0.78391E+00_JPRB, 0.79692E+00_JPRB, 0.80954E+00_JPRB, 0.82264E+00_JPRB, 0.82222E+00_JPRB, 0.82443E+00_JPRB ,&
&  0.84525E+00_JPRB, 0.84975E+00_JPRB, 0.85010E+00_JPRB, 0.84193E+00_JPRB, 0.83575E+00_JPRB, 0.83537E+00_JPRB, 0.82590E+00_JPRB ,&
&  0.82077E+00_JPRB, 0.82629E+00_JPRB, 0.84357E+00_JPRB, 0.87050E+00_JPRB, 0.89685E+00_JPRB, 0.93210E+00_JPRB, 0.95826E+00_JPRB ,&
&  0.96208E+00_JPRB, 0.95087E+00_JPRB, 0.95104E+00_JPRB, 0.95441E+00_JPRB, 0.95445E+00_JPRB, 0.93754E+00_JPRB, 0.93358E+00_JPRB ,&
&  0.91331E+00_JPRB, 0.92555E+00_JPRB, 0.79505E+00_JPRB, 0.86625E+00_JPRB, 0.91151E+00_JPRB, 0.94895E+00_JPRB, 0.96667E+00_JPRB ,&
&  0.96062E+00_JPRB, 0.94710E+00_JPRB, 0.91945E+00_JPRB, 0.88868E+00_JPRB, 0.87791E+00_JPRB, 0.85798E+00_JPRB, 0.83444E+00_JPRB ,&
&  0.78528E+00_JPRB, 0.72234E+00_JPRB, 0.69025E+00_JPRB, 0.67831E+00_JPRB, 0.67203E+00_JPRB, 0.67221E+00_JPRB, 0.67619E+00_JPRB ,&
&  0.67865E+00_JPRB, 0.67813E+00_JPRB, 0.66603E+00_JPRB, 0.64833E+00_JPRB, 0.62634E+00_JPRB, 0.61433E+00_JPRB, 0.61191E+00_JPRB ,&
&  0.60877E+00_JPRB, 0.59498E+00_JPRB, 0.59812E+00_JPRB, 0.62843E+00_JPRB, 0.65384E+00_JPRB, 0.67330E+00_JPRB, 0.68347E+00_JPRB ,&
&  0.69131E+00_JPRB, 0.68672E+00_JPRB, 0.68404E+00_JPRB, 0.67482E+00_JPRB, 0.65056E+00_JPRB, 0.64358E+00_JPRB, 0.66461E+00_JPRB ,&
&  0.68871E+00_JPRB, 0.70271E+00_JPRB, 0.71128E+00_JPRB, 0.72419E+00_JPRB, 0.73813E+00_JPRB, 0.74611E+00_JPRB, 0.75224E+00_JPRB ,&
&  0.75633E+00_JPRB, 0.76038E+00_JPRB, 0.77392E+00_JPRB, 0.78951E+00_JPRB, 0.79695E+00_JPRB, 0.79867E+00_JPRB, 0.79922E+00_JPRB ,&
&  0.79700E+00_JPRB, 0.78978E+00_JPRB, 0.78125E+00_JPRB, 0.77338E+00_JPRB, 0.75370E+00_JPRB, 0.73003E+00_JPRB, 0.71080E+00_JPRB ,&
&  0.69992E+00_JPRB, 0.70911E+00_JPRB, 0.71650E+00_JPRB, 0.72901E+00_JPRB, 0.75572E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.80655E+00_JPRB, 0.82024E+00_JPRB, 0.83155E+00_JPRB, 0.85196E+00_JPRB, 0.87028E+00_JPRB, 0.86735E+00_JPRB, 0.86089E+00_JPRB ,&
&  0.85707E+00_JPRB, 0.85363E+00_JPRB, 0.84743E+00_JPRB, 0.83850E+00_JPRB, 0.83253E+00_JPRB, 0.83221E+00_JPRB, 0.83256E+00_JPRB ,&
&  0.83367E+00_JPRB, 0.85322E+00_JPRB, 0.88554E+00_JPRB, 0.91072E+00_JPRB, 0.93836E+00_JPRB, 0.98097E+00_JPRB, 0.10008E+01_JPRB ,&
&  0.98332E+00_JPRB, 0.98173E+00_JPRB, 0.97082E+00_JPRB, 0.95641E+00_JPRB, 0.95846E+00_JPRB, 0.94967E+00_JPRB, 0.92209E+00_JPRB ,&
&  0.91619E+00_JPRB, 0.91180E+00_JPRB, 0.90773E+00_JPRB, 0.93776E+00_JPRB, 0.96289E+00_JPRB, 0.96617E+00_JPRB, 0.96644E+00_JPRB ,&
&  0.94369E+00_JPRB, 0.90668E+00_JPRB, 0.86841E+00_JPRB, 0.84837E+00_JPRB, 0.84560E+00_JPRB, 0.82567E+00_JPRB, 0.79764E+00_JPRB ,&
&  0.73991E+00_JPRB, 0.69232E+00_JPRB, 0.66578E+00_JPRB, 0.64965E+00_JPRB, 0.64397E+00_JPRB, 0.64056E+00_JPRB, 0.63536E+00_JPRB ,&
&  0.64006E+00_JPRB, 0.63404E+00_JPRB, 0.60470E+00_JPRB, 0.57340E+00_JPRB, 0.56687E+00_JPRB, 0.57568E+00_JPRB, 0.59679E+00_JPRB ,&
&  0.61197E+00_JPRB, 0.60665E+00_JPRB, 0.61335E+00_JPRB, 0.62163E+00_JPRB, 0.64000E+00_JPRB, 0.65952E+00_JPRB, 0.66209E+00_JPRB ,&
&  0.66349E+00_JPRB, 0.65923E+00_JPRB, 0.64661E+00_JPRB, 0.63761E+00_JPRB, 0.62580E+00_JPRB, 0.62044E+00_JPRB, 0.63111E+00_JPRB ,&
&  0.64834E+00_JPRB, 0.65812E+00_JPRB, 0.66860E+00_JPRB, 0.68239E+00_JPRB, 0.69443E+00_JPRB, 0.70334E+00_JPRB, 0.70981E+00_JPRB ,&
&  0.71986E+00_JPRB, 0.73479E+00_JPRB, 0.75019E+00_JPRB, 0.76387E+00_JPRB, 0.76429E+00_JPRB, 0.74997E+00_JPRB, 0.72435E+00_JPRB ,&
&  0.70736E+00_JPRB, 0.69427E+00_JPRB, 0.69171E+00_JPRB, 0.69840E+00_JPRB, 0.69390E+00_JPRB, 0.67425E+00_JPRB, 0.66393E+00_JPRB ,&
&  0.67419E+00_JPRB, 0.69802E+00_JPRB, 0.72056E+00_JPRB, 0.75128E+00_JPRB, 0.78342E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.84035E+00_JPRB, 0.87148E+00_JPRB, 0.89494E+00_JPRB, 0.90250E+00_JPRB, 0.89963E+00_JPRB, 0.89087E+00_JPRB, 0.86883E+00_JPRB ,&
&  0.84478E+00_JPRB, 0.82814E+00_JPRB, 0.81043E+00_JPRB, 0.78433E+00_JPRB, 0.76273E+00_JPRB, 0.75855E+00_JPRB, 0.77129E+00_JPRB ,&
&  0.81761E+00_JPRB, 0.85863E+00_JPRB, 0.87505E+00_JPRB, 0.88843E+00_JPRB, 0.93781E+00_JPRB, 0.97914E+00_JPRB, 0.98762E+00_JPRB ,&
&  0.98135E+00_JPRB, 0.98223E+00_JPRB, 0.94567E+00_JPRB, 0.91282E+00_JPRB, 0.91348E+00_JPRB, 0.91952E+00_JPRB, 0.90197E+00_JPRB ,&
&  0.92821E+00_JPRB, 0.94684E+00_JPRB, 0.94291E+00_JPRB, 0.92228E+00_JPRB, 0.91771E+00_JPRB, 0.90771E+00_JPRB, 0.89491E+00_JPRB ,&
&  0.87263E+00_JPRB, 0.82437E+00_JPRB, 0.79147E+00_JPRB, 0.78768E+00_JPRB, 0.78041E+00_JPRB, 0.77371E+00_JPRB, 0.76084E+00_JPRB ,&
&  0.72064E+00_JPRB, 0.68433E+00_JPRB, 0.65662E+00_JPRB, 0.63755E+00_JPRB, 0.62530E+00_JPRB, 0.61545E+00_JPRB, 0.60482E+00_JPRB ,&
&  0.59542E+00_JPRB, 0.56623E+00_JPRB, 0.53607E+00_JPRB, 0.52817E+00_JPRB, 0.53284E+00_JPRB, 0.54056E+00_JPRB, 0.56930E+00_JPRB ,&
&  0.59450E+00_JPRB, 0.59299E+00_JPRB, 0.58966E+00_JPRB, 0.59290E+00_JPRB, 0.60287E+00_JPRB, 0.60479E+00_JPRB, 0.59572E+00_JPRB ,&
&  0.59148E+00_JPRB, 0.58963E+00_JPRB, 0.58014E+00_JPRB, 0.57297E+00_JPRB, 0.57240E+00_JPRB, 0.57608E+00_JPRB, 0.58986E+00_JPRB ,&
&  0.59821E+00_JPRB, 0.59908E+00_JPRB, 0.61225E+00_JPRB, 0.62915E+00_JPRB, 0.63980E+00_JPRB, 0.64845E+00_JPRB, 0.65494E+00_JPRB ,&
&  0.66905E+00_JPRB, 0.68902E+00_JPRB, 0.70426E+00_JPRB, 0.70661E+00_JPRB, 0.68393E+00_JPRB, 0.65990E+00_JPRB, 0.64110E+00_JPRB ,&
&  0.63278E+00_JPRB, 0.63714E+00_JPRB, 0.64505E+00_JPRB, 0.65284E+00_JPRB, 0.65983E+00_JPRB, 0.66468E+00_JPRB, 0.66428E+00_JPRB ,&
&  0.68409E+00_JPRB, 0.71506E+00_JPRB, 0.73989E+00_JPRB, 0.76940E+00_JPRB, 0.80042E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.90410E+00_JPRB, 0.92129E+00_JPRB, 0.93247E+00_JPRB, 0.93243E+00_JPRB, 0.91632E+00_JPRB, 0.88942E+00_JPRB, 0.86666E+00_JPRB ,&
&  0.84080E+00_JPRB, 0.81560E+00_JPRB, 0.79171E+00_JPRB, 0.75879E+00_JPRB, 0.74109E+00_JPRB, 0.74452E+00_JPRB, 0.79352E+00_JPRB ,&
&  0.85903E+00_JPRB, 0.86414E+00_JPRB, 0.85825E+00_JPRB, 0.88331E+00_JPRB, 0.93524E+00_JPRB, 0.96658E+00_JPRB, 0.98691E+00_JPRB ,&
&  0.98821E+00_JPRB, 0.95430E+00_JPRB, 0.92155E+00_JPRB, 0.90917E+00_JPRB, 0.93371E+00_JPRB, 0.95221E+00_JPRB, 0.95746E+00_JPRB ,&
&  0.95342E+00_JPRB, 0.93365E+00_JPRB, 0.91686E+00_JPRB, 0.90278E+00_JPRB, 0.90807E+00_JPRB, 0.88326E+00_JPRB, 0.83063E+00_JPRB ,&
&  0.76410E+00_JPRB, 0.73716E+00_JPRB, 0.72408E+00_JPRB, 0.70900E+00_JPRB, 0.69826E+00_JPRB, 0.69894E+00_JPRB, 0.69044E+00_JPRB ,&
&  0.67692E+00_JPRB, 0.66049E+00_JPRB, 0.64442E+00_JPRB, 0.62583E+00_JPRB, 0.61104E+00_JPRB, 0.59012E+00_JPRB, 0.57051E+00_JPRB ,&
&  0.55231E+00_JPRB, 0.52951E+00_JPRB, 0.50831E+00_JPRB, 0.49302E+00_JPRB, 0.48581E+00_JPRB, 0.49676E+00_JPRB, 0.53806E+00_JPRB ,&
&  0.57006E+00_JPRB, 0.56718E+00_JPRB, 0.56519E+00_JPRB, 0.57015E+00_JPRB, 0.56423E+00_JPRB, 0.55492E+00_JPRB, 0.54986E+00_JPRB ,&
&  0.54499E+00_JPRB, 0.54133E+00_JPRB, 0.54012E+00_JPRB, 0.54251E+00_JPRB, 0.54501E+00_JPRB, 0.54312E+00_JPRB, 0.54472E+00_JPRB ,&
&  0.54827E+00_JPRB, 0.55303E+00_JPRB, 0.57000E+00_JPRB, 0.59440E+00_JPRB, 0.61569E+00_JPRB, 0.63154E+00_JPRB, 0.64621E+00_JPRB ,&
&  0.66814E+00_JPRB, 0.68165E+00_JPRB, 0.66936E+00_JPRB, 0.65790E+00_JPRB, 0.65244E+00_JPRB, 0.65820E+00_JPRB, 0.66898E+00_JPRB ,&
&  0.68940E+00_JPRB, 0.70179E+00_JPRB, 0.70796E+00_JPRB, 0.70927E+00_JPRB, 0.71224E+00_JPRB, 0.71297E+00_JPRB, 0.72568E+00_JPRB ,&
&  0.75978E+00_JPRB, 0.78904E+00_JPRB, 0.79597E+00_JPRB, 0.82809E+00_JPRB, 0.86942E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.94107E+00_JPRB, 0.94474E+00_JPRB, 0.93439E+00_JPRB, 0.92969E+00_JPRB, 0.91890E+00_JPRB, 0.90508E+00_JPRB, 0.88287E+00_JPRB ,&
&  0.85690E+00_JPRB, 0.82890E+00_JPRB, 0.80200E+00_JPRB, 0.77983E+00_JPRB, 0.78320E+00_JPRB, 0.81535E+00_JPRB, 0.84269E+00_JPRB ,&
&  0.83193E+00_JPRB, 0.82815E+00_JPRB, 0.84505E+00_JPRB, 0.87539E+00_JPRB, 0.90818E+00_JPRB, 0.93833E+00_JPRB, 0.95943E+00_JPRB ,&
&  0.97626E+00_JPRB, 0.95802E+00_JPRB, 0.96283E+00_JPRB, 0.96602E+00_JPRB, 0.96069E+00_JPRB, 0.94290E+00_JPRB, 0.91413E+00_JPRB ,&
&  0.88583E+00_JPRB, 0.85854E+00_JPRB, 0.82502E+00_JPRB, 0.76784E+00_JPRB, 0.84122E+00_JPRB, 0.76766E+00_JPRB, 0.74006E+00_JPRB ,&
&  0.71252E+00_JPRB, 0.69382E+00_JPRB, 0.67248E+00_JPRB, 0.64983E+00_JPRB, 0.64217E+00_JPRB, 0.63771E+00_JPRB, 0.62964E+00_JPRB ,&
&  0.62275E+00_JPRB, 0.61475E+00_JPRB, 0.60274E+00_JPRB, 0.58809E+00_JPRB, 0.57459E+00_JPRB, 0.56044E+00_JPRB, 0.54736E+00_JPRB ,&
&  0.52805E+00_JPRB, 0.49963E+00_JPRB, 0.46971E+00_JPRB, 0.44746E+00_JPRB, 0.43561E+00_JPRB, 0.44525E+00_JPRB, 0.47953E+00_JPRB ,&
&  0.51453E+00_JPRB, 0.52066E+00_JPRB, 0.53088E+00_JPRB, 0.53460E+00_JPRB, 0.52640E+00_JPRB, 0.51575E+00_JPRB, 0.49783E+00_JPRB ,&
&  0.48488E+00_JPRB, 0.48356E+00_JPRB, 0.48536E+00_JPRB, 0.49029E+00_JPRB, 0.49527E+00_JPRB, 0.50195E+00_JPRB, 0.51125E+00_JPRB ,&
&  0.51962E+00_JPRB, 0.52265E+00_JPRB, 0.53508E+00_JPRB, 0.56192E+00_JPRB, 0.59337E+00_JPRB, 0.62558E+00_JPRB, 0.65360E+00_JPRB ,&
&  0.67320E+00_JPRB, 0.68726E+00_JPRB, 0.69597E+00_JPRB, 0.71126E+00_JPRB, 0.72702E+00_JPRB, 0.74254E+00_JPRB, 0.75036E+00_JPRB ,&
&  0.74758E+00_JPRB, 0.75168E+00_JPRB, 0.76986E+00_JPRB, 0.79875E+00_JPRB, 0.81866E+00_JPRB, 0.85250E+00_JPRB, 0.87369E+00_JPRB ,&
&  0.89592E+00_JPRB, 0.90142E+00_JPRB, 0.89576E+00_JPRB, 0.90801E+00_JPRB, 0.92431E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.92405E+00_JPRB, 0.93433E+00_JPRB, 0.93198E+00_JPRB, 0.93068E+00_JPRB, 0.93213E+00_JPRB, 0.92493E+00_JPRB, 0.91127E+00_JPRB ,&
&  0.89740E+00_JPRB, 0.87078E+00_JPRB, 0.83976E+00_JPRB, 0.82547E+00_JPRB, 0.83214E+00_JPRB, 0.83144E+00_JPRB, 0.81284E+00_JPRB ,&
&  0.81126E+00_JPRB, 0.82483E+00_JPRB, 0.84753E+00_JPRB, 0.87273E+00_JPRB, 0.90651E+00_JPRB, 0.94155E+00_JPRB, 0.96753E+00_JPRB ,&
&  0.97796E+00_JPRB, 0.97670E+00_JPRB, 0.96948E+00_JPRB, 0.94293E+00_JPRB, 0.90716E+00_JPRB, 0.87983E+00_JPRB, 0.85904E+00_JPRB ,&
&  0.83308E+00_JPRB, 0.78348E+00_JPRB, 0.71545E+00_JPRB, 0.70978E+00_JPRB, 0.64861E+00_JPRB, 0.63092E+00_JPRB, 0.65392E+00_JPRB ,&
&  0.66941E+00_JPRB, 0.65160E+00_JPRB, 0.62717E+00_JPRB, 0.60688E+00_JPRB, 0.60260E+00_JPRB, 0.60084E+00_JPRB, 0.59532E+00_JPRB ,&
&  0.59106E+00_JPRB, 0.58399E+00_JPRB, 0.56956E+00_JPRB, 0.55649E+00_JPRB, 0.54415E+00_JPRB, 0.53178E+00_JPRB, 0.52295E+00_JPRB ,&
&  0.49345E+00_JPRB, 0.44882E+00_JPRB, 0.41543E+00_JPRB, 0.40035E+00_JPRB, 0.39191E+00_JPRB, 0.39580E+00_JPRB, 0.40950E+00_JPRB ,&
&  0.42122E+00_JPRB, 0.42296E+00_JPRB, 0.41796E+00_JPRB, 0.40457E+00_JPRB, 0.38189E+00_JPRB, 0.33936E+00_JPRB, 0.31409E+00_JPRB ,&
&  0.33143E+00_JPRB, 0.36278E+00_JPRB, 0.39589E+00_JPRB, 0.40951E+00_JPRB, 0.41849E+00_JPRB, 0.41963E+00_JPRB, 0.40364E+00_JPRB ,&
&  0.37366E+00_JPRB, 0.34762E+00_JPRB, 0.34368E+00_JPRB, 0.34598E+00_JPRB, 0.35219E+00_JPRB, 0.36281E+00_JPRB, 0.37937E+00_JPRB ,&
&  0.40433E+00_JPRB, 0.44127E+00_JPRB, 0.46389E+00_JPRB, 0.46661E+00_JPRB, 0.45927E+00_JPRB, 0.44825E+00_JPRB, 0.44560E+00_JPRB ,&
&  0.46031E+00_JPRB, 0.51996E+00_JPRB, 0.64041E+00_JPRB, 0.78471E+00_JPRB, 0.78783E+00_JPRB, 0.83276E+00_JPRB, 0.86739E+00_JPRB ,&
&  0.89739E+00_JPRB, 0.91102E+00_JPRB, 0.89795E+00_JPRB, 0.89855E+00_JPRB, 0.90448E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.88134E+00_JPRB, 0.90120E+00_JPRB, 0.91195E+00_JPRB, 0.92579E+00_JPRB, 0.92600E+00_JPRB, 0.89595E+00_JPRB, 0.88168E+00_JPRB ,&
&  0.88628E+00_JPRB, 0.88101E+00_JPRB, 0.86934E+00_JPRB, 0.86170E+00_JPRB, 0.85238E+00_JPRB, 0.84251E+00_JPRB, 0.83452E+00_JPRB ,&
&  0.82629E+00_JPRB, 0.82100E+00_JPRB, 0.82759E+00_JPRB, 0.84110E+00_JPRB, 0.85209E+00_JPRB, 0.85794E+00_JPRB, 0.86296E+00_JPRB ,&
&  0.86271E+00_JPRB, 0.82997E+00_JPRB, 0.78459E+00_JPRB, 0.75149E+00_JPRB, 0.73547E+00_JPRB, 0.72183E+00_JPRB, 0.68807E+00_JPRB ,&
&  0.61760E+00_JPRB, 0.49670E+00_JPRB, 0.44287E+00_JPRB, 0.48538E+00_JPRB, 0.59920E+00_JPRB, 0.56697E+00_JPRB, 0.58874E+00_JPRB ,&
&  0.62003E+00_JPRB, 0.63636E+00_JPRB, 0.63222E+00_JPRB, 0.62164E+00_JPRB, 0.61801E+00_JPRB, 0.62159E+00_JPRB, 0.61859E+00_JPRB ,&
&  0.61550E+00_JPRB, 0.59969E+00_JPRB, 0.59008E+00_JPRB, 0.57807E+00_JPRB, 0.55949E+00_JPRB, 0.53325E+00_JPRB, 0.51683E+00_JPRB ,&
&  0.47703E+00_JPRB, 0.43796E+00_JPRB, 0.42586E+00_JPRB, 0.42524E+00_JPRB, 0.39391E+00_JPRB, 0.38106E+00_JPRB, 0.36512E+00_JPRB ,&
&  0.32710E+00_JPRB, 0.30610E+00_JPRB, 0.27528E+00_JPRB, 0.25618E+00_JPRB, 0.24271E+00_JPRB, 0.22766E+00_JPRB, 0.22533E+00_JPRB ,&
&  0.23109E+00_JPRB, 0.24491E+00_JPRB, 0.26549E+00_JPRB, 0.29621E+00_JPRB, 0.32317E+00_JPRB, 0.28684E+00_JPRB, 0.27096E+00_JPRB ,&
&  0.25605E+00_JPRB, 0.25268E+00_JPRB, 0.25959E+00_JPRB, 0.26740E+00_JPRB, 0.27506E+00_JPRB, 0.28398E+00_JPRB, 0.29298E+00_JPRB ,&
&  0.29950E+00_JPRB, 0.30249E+00_JPRB, 0.30279E+00_JPRB, 0.30187E+00_JPRB, 0.30009E+00_JPRB, 0.30037E+00_JPRB, 0.30369E+00_JPRB ,&
&  0.31068E+00_JPRB, 0.32354E+00_JPRB, 0.34767E+00_JPRB, 0.38329E+00_JPRB, 0.42913E+00_JPRB, 0.48968E+00_JPRB, 0.58117E+00_JPRB ,&
&  0.70540E+00_JPRB, 0.86852E+00_JPRB, 0.91564E+00_JPRB, 0.92491E+00_JPRB, 0.89315E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.88518E+00_JPRB, 0.88212E+00_JPRB, 0.88896E+00_JPRB, 0.87868E+00_JPRB, 0.85982E+00_JPRB, 0.85104E+00_JPRB, 0.86812E+00_JPRB ,&
&  0.88782E+00_JPRB, 0.85802E+00_JPRB, 0.83593E+00_JPRB, 0.81653E+00_JPRB, 0.80222E+00_JPRB, 0.78799E+00_JPRB, 0.76622E+00_JPRB ,&
&  0.74617E+00_JPRB, 0.73866E+00_JPRB, 0.73649E+00_JPRB, 0.73127E+00_JPRB, 0.72096E+00_JPRB, 0.71094E+00_JPRB, 0.69336E+00_JPRB ,&
&  0.65983E+00_JPRB, 0.62445E+00_JPRB, 0.60637E+00_JPRB, 0.58262E+00_JPRB, 0.56594E+00_JPRB, 0.54652E+00_JPRB, 0.51895E+00_JPRB ,&
&  0.47285E+00_JPRB, 0.41502E+00_JPRB, 0.38204E+00_JPRB, 0.42018E+00_JPRB, 0.51192E+00_JPRB, 0.50444E+00_JPRB, 0.52520E+00_JPRB ,&
&  0.56050E+00_JPRB, 0.58186E+00_JPRB, 0.59275E+00_JPRB, 0.59641E+00_JPRB, 0.60595E+00_JPRB, 0.61250E+00_JPRB, 0.60906E+00_JPRB ,&
&  0.59347E+00_JPRB, 0.54103E+00_JPRB, 0.49690E+00_JPRB, 0.39196E+00_JPRB, 0.32189E+00_JPRB, 0.28468E+00_JPRB, 0.25408E+00_JPRB ,&
&  0.23362E+00_JPRB, 0.22213E+00_JPRB, 0.21963E+00_JPRB, 0.21767E+00_JPRB, 0.22332E+00_JPRB, 0.23073E+00_JPRB, 0.23095E+00_JPRB ,&
&  0.22630E+00_JPRB, 0.21747E+00_JPRB, 0.20572E+00_JPRB, 0.19921E+00_JPRB, 0.19558E+00_JPRB, 0.19578E+00_JPRB, 0.19790E+00_JPRB ,&
&  0.19883E+00_JPRB, 0.20189E+00_JPRB, 0.21457E+00_JPRB, 0.22507E+00_JPRB, 0.23862E+00_JPRB, 0.22249E+00_JPRB, 0.20850E+00_JPRB ,&
&  0.20293E+00_JPRB, 0.20843E+00_JPRB, 0.21639E+00_JPRB, 0.22226E+00_JPRB, 0.22839E+00_JPRB, 0.23328E+00_JPRB, 0.23804E+00_JPRB ,&
&  0.24316E+00_JPRB, 0.24683E+00_JPRB, 0.24838E+00_JPRB, 0.25011E+00_JPRB, 0.25163E+00_JPRB, 0.25401E+00_JPRB, 0.25683E+00_JPRB ,&
&  0.26168E+00_JPRB, 0.27150E+00_JPRB, 0.28419E+00_JPRB, 0.30529E+00_JPRB, 0.32508E+00_JPRB, 0.34462E+00_JPRB, 0.36789E+00_JPRB ,&
&  0.38806E+00_JPRB, 0.46953E+00_JPRB, 0.66226E+00_JPRB, 0.98023E+00_JPRB, 0.97390E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.83099E+00_JPRB, 0.84305E+00_JPRB, 0.85962E+00_JPRB, 0.83624E+00_JPRB, 0.82121E+00_JPRB, 0.82449E+00_JPRB, 0.82523E+00_JPRB ,&
&  0.80170E+00_JPRB, 0.76550E+00_JPRB, 0.71547E+00_JPRB, 0.67327E+00_JPRB, 0.60948E+00_JPRB, 0.52703E+00_JPRB, 0.48911E+00_JPRB ,&
&  0.45398E+00_JPRB, 0.45001E+00_JPRB, 0.45181E+00_JPRB, 0.46788E+00_JPRB, 0.47641E+00_JPRB, 0.47294E+00_JPRB, 0.45060E+00_JPRB ,&
&  0.42339E+00_JPRB, 0.39679E+00_JPRB, 0.37914E+00_JPRB, 0.36574E+00_JPRB, 0.36576E+00_JPRB, 0.36380E+00_JPRB, 0.35725E+00_JPRB ,&
&  0.34955E+00_JPRB, 0.35492E+00_JPRB, 0.37295E+00_JPRB, 0.39987E+00_JPRB, 0.44701E+00_JPRB, 0.42675E+00_JPRB, 0.43536E+00_JPRB ,&
&  0.46607E+00_JPRB, 0.48823E+00_JPRB, 0.49711E+00_JPRB, 0.50043E+00_JPRB, 0.49294E+00_JPRB, 0.51526E+00_JPRB, 0.45869E+00_JPRB ,&
&  0.35242E+00_JPRB, 0.29314E+00_JPRB, 0.25052E+00_JPRB, 0.22209E+00_JPRB, 0.20215E+00_JPRB, 0.19023E+00_JPRB, 0.17667E+00_JPRB ,&
&  0.17019E+00_JPRB, 0.16515E+00_JPRB, 0.16416E+00_JPRB, 0.16652E+00_JPRB, 0.16971E+00_JPRB, 0.17375E+00_JPRB, 0.17520E+00_JPRB ,&
&  0.17760E+00_JPRB, 0.17768E+00_JPRB, 0.17694E+00_JPRB, 0.17697E+00_JPRB, 0.17581E+00_JPRB, 0.17648E+00_JPRB, 0.17622E+00_JPRB ,&
&  0.17568E+00_JPRB, 0.17511E+00_JPRB, 0.17683E+00_JPRB, 0.17791E+00_JPRB, 0.17874E+00_JPRB, 0.17562E+00_JPRB, 0.17436E+00_JPRB ,&
&  0.17505E+00_JPRB, 0.17761E+00_JPRB, 0.18149E+00_JPRB, 0.18581E+00_JPRB, 0.19177E+00_JPRB, 0.19632E+00_JPRB, 0.20159E+00_JPRB ,&
&  0.20611E+00_JPRB, 0.21058E+00_JPRB, 0.21492E+00_JPRB, 0.21938E+00_JPRB, 0.22429E+00_JPRB, 0.22917E+00_JPRB, 0.23420E+00_JPRB ,&
&  0.23952E+00_JPRB, 0.24834E+00_JPRB, 0.25913E+00_JPRB, 0.27779E+00_JPRB, 0.30274E+00_JPRB, 0.32825E+00_JPRB, 0.37387E+00_JPRB ,&
&  0.43007E+00_JPRB, 0.65730E+00_JPRB, 0.96176E+00_JPRB, 0.79657E+00_JPRB, 0.81377E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.82035E+00_JPRB, 0.83856E+00_JPRB, 0.85423E+00_JPRB, 0.85153E+00_JPRB, 0.83417E+00_JPRB, 0.77663E+00_JPRB, 0.71323E+00_JPRB ,&
&  0.63466E+00_JPRB, 0.56923E+00_JPRB, 0.49968E+00_JPRB, 0.43996E+00_JPRB, 0.38938E+00_JPRB, 0.35026E+00_JPRB, 0.32562E+00_JPRB ,&
&  0.30784E+00_JPRB, 0.29995E+00_JPRB, 0.29621E+00_JPRB, 0.30069E+00_JPRB, 0.30259E+00_JPRB, 0.30804E+00_JPRB, 0.30236E+00_JPRB ,&
&  0.29533E+00_JPRB, 0.28779E+00_JPRB, 0.28564E+00_JPRB, 0.29419E+00_JPRB, 0.30783E+00_JPRB, 0.32096E+00_JPRB, 0.33819E+00_JPRB ,&
&  0.35654E+00_JPRB, 0.36246E+00_JPRB, 0.37739E+00_JPRB, 0.38093E+00_JPRB, 0.36675E+00_JPRB, 0.36885E+00_JPRB, 0.37493E+00_JPRB ,&
&  0.38840E+00_JPRB, 0.40699E+00_JPRB, 0.41761E+00_JPRB, 0.43715E+00_JPRB, 0.38229E+00_JPRB, 0.32232E+00_JPRB, 0.26968E+00_JPRB ,&
&  0.23028E+00_JPRB, 0.20752E+00_JPRB, 0.19073E+00_JPRB, 0.18224E+00_JPRB, 0.17346E+00_JPRB, 0.16869E+00_JPRB, 0.16376E+00_JPRB ,&
&  0.16159E+00_JPRB, 0.16176E+00_JPRB, 0.16145E+00_JPRB, 0.16141E+00_JPRB, 0.16218E+00_JPRB, 0.16330E+00_JPRB, 0.16291E+00_JPRB ,&
&  0.16237E+00_JPRB, 0.16220E+00_JPRB, 0.16177E+00_JPRB, 0.16135E+00_JPRB, 0.16040E+00_JPRB, 0.16019E+00_JPRB, 0.15814E+00_JPRB ,&
&  0.15701E+00_JPRB, 0.15531E+00_JPRB, 0.15436E+00_JPRB, 0.15438E+00_JPRB, 0.15489E+00_JPRB, 0.15538E+00_JPRB, 0.15624E+00_JPRB ,&
&  0.15780E+00_JPRB, 0.15918E+00_JPRB, 0.16147E+00_JPRB, 0.16353E+00_JPRB, 0.16691E+00_JPRB, 0.16976E+00_JPRB, 0.17374E+00_JPRB ,&
&  0.17800E+00_JPRB, 0.18314E+00_JPRB, 0.18919E+00_JPRB, 0.19605E+00_JPRB, 0.20166E+00_JPRB, 0.20711E+00_JPRB, 0.21295E+00_JPRB ,&
&  0.21745E+00_JPRB, 0.22548E+00_JPRB, 0.23446E+00_JPRB, 0.24945E+00_JPRB, 0.28086E+00_JPRB, 0.30379E+00_JPRB, 0.35153E+00_JPRB ,&
&  0.42885E+00_JPRB, 0.68008E+00_JPRB, 0.91415E+00_JPRB, 0.77027E+00_JPRB, 0.80197E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.69778E+00_JPRB, 0.72890E+00_JPRB, 0.75246E+00_JPRB, 0.75411E+00_JPRB, 0.75231E+00_JPRB, 0.71471E+00_JPRB, 0.65606E+00_JPRB ,&
&  0.55026E+00_JPRB, 0.48095E+00_JPRB, 0.43637E+00_JPRB, 0.39426E+00_JPRB, 0.35633E+00_JPRB, 0.33023E+00_JPRB, 0.31267E+00_JPRB ,&
&  0.29944E+00_JPRB, 0.29212E+00_JPRB, 0.28661E+00_JPRB, 0.28200E+00_JPRB, 0.27692E+00_JPRB, 0.27203E+00_JPRB, 0.26345E+00_JPRB ,&
&  0.25808E+00_JPRB, 0.25495E+00_JPRB, 0.25329E+00_JPRB, 0.25568E+00_JPRB, 0.26262E+00_JPRB, 0.26896E+00_JPRB, 0.27933E+00_JPRB ,&
&  0.29080E+00_JPRB, 0.30529E+00_JPRB, 0.31520E+00_JPRB, 0.31915E+00_JPRB, 0.32242E+00_JPRB, 0.32122E+00_JPRB, 0.29954E+00_JPRB ,&
&  0.30425E+00_JPRB, 0.32529E+00_JPRB, 0.34344E+00_JPRB, 0.35137E+00_JPRB, 0.30517E+00_JPRB, 0.26383E+00_JPRB, 0.22299E+00_JPRB ,&
&  0.20423E+00_JPRB, 0.18818E+00_JPRB, 0.17954E+00_JPRB, 0.17280E+00_JPRB, 0.16734E+00_JPRB, 0.16465E+00_JPRB, 0.16227E+00_JPRB ,&
&  0.16038E+00_JPRB, 0.15990E+00_JPRB, 0.15868E+00_JPRB, 0.15840E+00_JPRB, 0.15763E+00_JPRB, 0.15770E+00_JPRB, 0.15650E+00_JPRB ,&
&  0.15524E+00_JPRB, 0.15419E+00_JPRB, 0.15350E+00_JPRB, 0.15312E+00_JPRB, 0.15211E+00_JPRB, 0.15109E+00_JPRB, 0.14876E+00_JPRB ,&
&  0.14714E+00_JPRB, 0.14560E+00_JPRB, 0.14446E+00_JPRB, 0.14469E+00_JPRB, 0.14546E+00_JPRB, 0.14783E+00_JPRB, 0.14911E+00_JPRB ,&
&  0.14931E+00_JPRB, 0.14937E+00_JPRB, 0.14928E+00_JPRB, 0.15009E+00_JPRB, 0.15131E+00_JPRB, 0.15345E+00_JPRB, 0.15645E+00_JPRB ,&
&  0.16009E+00_JPRB, 0.16575E+00_JPRB, 0.17127E+00_JPRB, 0.17733E+00_JPRB, 0.18285E+00_JPRB, 0.18801E+00_JPRB, 0.19507E+00_JPRB ,&
&  0.20220E+00_JPRB, 0.21088E+00_JPRB, 0.22403E+00_JPRB, 0.23768E+00_JPRB, 0.26255E+00_JPRB, 0.28197E+00_JPRB, 0.31167E+00_JPRB ,&
&  0.38012E+00_JPRB, 0.52490E+00_JPRB, 0.61127E+00_JPRB, 0.62569E+00_JPRB, 0.65952E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.51362E+00_JPRB, 0.52545E+00_JPRB, 0.54068E+00_JPRB, 0.54992E+00_JPRB, 0.56073E+00_JPRB, 0.52835E+00_JPRB, 0.50272E+00_JPRB ,&
&  0.44987E+00_JPRB, 0.41940E+00_JPRB, 0.39400E+00_JPRB, 0.37069E+00_JPRB, 0.34986E+00_JPRB, 0.33069E+00_JPRB, 0.31331E+00_JPRB ,&
&  0.30223E+00_JPRB, 0.29573E+00_JPRB, 0.28696E+00_JPRB, 0.27684E+00_JPRB, 0.26595E+00_JPRB, 0.25641E+00_JPRB, 0.25186E+00_JPRB ,&
&  0.24698E+00_JPRB, 0.24354E+00_JPRB, 0.24210E+00_JPRB, 0.24180E+00_JPRB, 0.24353E+00_JPRB, 0.24342E+00_JPRB, 0.24381E+00_JPRB ,&
&  0.23989E+00_JPRB, 0.24730E+00_JPRB, 0.25719E+00_JPRB, 0.26173E+00_JPRB, 0.27041E+00_JPRB, 0.26935E+00_JPRB, 0.26478E+00_JPRB ,&
&  0.25713E+00_JPRB, 0.24857E+00_JPRB, 0.23857E+00_JPRB, 0.22617E+00_JPRB, 0.21580E+00_JPRB, 0.20837E+00_JPRB, 0.20091E+00_JPRB ,&
&  0.19660E+00_JPRB, 0.19086E+00_JPRB, 0.18364E+00_JPRB, 0.17729E+00_JPRB, 0.17177E+00_JPRB, 0.16737E+00_JPRB, 0.16327E+00_JPRB ,&
&  0.16048E+00_JPRB, 0.15897E+00_JPRB, 0.15753E+00_JPRB, 0.15745E+00_JPRB, 0.15643E+00_JPRB, 0.15531E+00_JPRB, 0.15416E+00_JPRB ,&
&  0.15357E+00_JPRB, 0.15219E+00_JPRB, 0.15106E+00_JPRB, 0.15003E+00_JPRB, 0.14844E+00_JPRB, 0.14706E+00_JPRB, 0.14552E+00_JPRB ,&
&  0.14422E+00_JPRB, 0.14359E+00_JPRB, 0.14322E+00_JPRB, 0.14318E+00_JPRB, 0.14356E+00_JPRB, 0.14353E+00_JPRB, 0.14385E+00_JPRB ,&
&  0.14423E+00_JPRB, 0.14478E+00_JPRB, 0.14473E+00_JPRB, 0.14512E+00_JPRB, 0.14611E+00_JPRB, 0.14747E+00_JPRB, 0.15084E+00_JPRB ,&
&  0.15467E+00_JPRB, 0.15953E+00_JPRB, 0.16473E+00_JPRB, 0.17045E+00_JPRB, 0.17615E+00_JPRB, 0.18166E+00_JPRB, 0.18775E+00_JPRB ,&
&  0.19568E+00_JPRB, 0.20291E+00_JPRB, 0.21155E+00_JPRB, 0.22486E+00_JPRB, 0.25497E+00_JPRB, 0.27051E+00_JPRB, 0.28430E+00_JPRB ,&
&  0.30893E+00_JPRB, 0.34065E+00_JPRB, 0.44455E+00_JPRB, 0.47665E+00_JPRB, 0.49414E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.25857E+00_JPRB, 0.25823E+00_JPRB, 0.25841E+00_JPRB, 0.26409E+00_JPRB, 0.27215E+00_JPRB, 0.34987E+00_JPRB, 0.34013E+00_JPRB ,&
&  0.33226E+00_JPRB, 0.30464E+00_JPRB, 0.30167E+00_JPRB, 0.29497E+00_JPRB, 0.28571E+00_JPRB, 0.27866E+00_JPRB, 0.27470E+00_JPRB ,&
&  0.27023E+00_JPRB, 0.26514E+00_JPRB, 0.26267E+00_JPRB, 0.25970E+00_JPRB, 0.25670E+00_JPRB, 0.25308E+00_JPRB, 0.25027E+00_JPRB ,&
&  0.24744E+00_JPRB, 0.24544E+00_JPRB, 0.24329E+00_JPRB, 0.23875E+00_JPRB, 0.23684E+00_JPRB, 0.23325E+00_JPRB, 0.22796E+00_JPRB ,&
&  0.22098E+00_JPRB, 0.21746E+00_JPRB, 0.21357E+00_JPRB, 0.21015E+00_JPRB, 0.21383E+00_JPRB, 0.21319E+00_JPRB, 0.21122E+00_JPRB ,&
&  0.20755E+00_JPRB, 0.20376E+00_JPRB, 0.19873E+00_JPRB, 0.19389E+00_JPRB, 0.18995E+00_JPRB, 0.18674E+00_JPRB, 0.18402E+00_JPRB ,&
&  0.18189E+00_JPRB, 0.18012E+00_JPRB, 0.17744E+00_JPRB, 0.17502E+00_JPRB, 0.17244E+00_JPRB, 0.17019E+00_JPRB, 0.16778E+00_JPRB ,&
&  0.16589E+00_JPRB, 0.16412E+00_JPRB, 0.16227E+00_JPRB, 0.16039E+00_JPRB, 0.15891E+00_JPRB, 0.15697E+00_JPRB, 0.15552E+00_JPRB ,&
&  0.15428E+00_JPRB, 0.15264E+00_JPRB, 0.15142E+00_JPRB, 0.14994E+00_JPRB, 0.14885E+00_JPRB, 0.14793E+00_JPRB, 0.14705E+00_JPRB ,&
&  0.14588E+00_JPRB, 0.14510E+00_JPRB, 0.14482E+00_JPRB, 0.14461E+00_JPRB, 0.14429E+00_JPRB, 0.14437E+00_JPRB, 0.14477E+00_JPRB ,&
&  0.14532E+00_JPRB, 0.14598E+00_JPRB, 0.14701E+00_JPRB, 0.14820E+00_JPRB, 0.14997E+00_JPRB, 0.15235E+00_JPRB, 0.15607E+00_JPRB ,&
&  0.16010E+00_JPRB, 0.16452E+00_JPRB, 0.16894E+00_JPRB, 0.17358E+00_JPRB, 0.17839E+00_JPRB, 0.18318E+00_JPRB, 0.18869E+00_JPRB ,&
&  0.19466E+00_JPRB, 0.19941E+00_JPRB, 0.20396E+00_JPRB, 0.20881E+00_JPRB, 0.21952E+00_JPRB, 0.22738E+00_JPRB, 0.23318E+00_JPRB ,&
&  0.23973E+00_JPRB, 0.24883E+00_JPRB, 0.25335E+00_JPRB, 0.25616E+00_JPRB, 0.25753E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.22998E+00_JPRB, 0.23293E+00_JPRB, 0.23493E+00_JPRB, 0.23619E+00_JPRB, 0.23586E+00_JPRB, 0.23678E+00_JPRB, 0.23770E+00_JPRB ,&
&  0.23894E+00_JPRB, 0.24058E+00_JPRB, 0.24179E+00_JPRB, 0.24286E+00_JPRB, 0.24335E+00_JPRB, 0.24338E+00_JPRB, 0.24341E+00_JPRB ,&
&  0.24319E+00_JPRB, 0.24293E+00_JPRB, 0.23872E+00_JPRB, 0.23494E+00_JPRB, 0.23217E+00_JPRB, 0.23050E+00_JPRB, 0.22909E+00_JPRB ,&
&  0.22796E+00_JPRB, 0.22660E+00_JPRB, 0.22512E+00_JPRB, 0.22299E+00_JPRB, 0.21959E+00_JPRB, 0.21625E+00_JPRB, 0.21279E+00_JPRB ,&
&  0.20966E+00_JPRB, 0.20721E+00_JPRB, 0.20493E+00_JPRB, 0.20267E+00_JPRB, 0.20144E+00_JPRB, 0.19994E+00_JPRB, 0.19836E+00_JPRB ,&
&  0.19626E+00_JPRB, 0.19396E+00_JPRB, 0.19178E+00_JPRB, 0.18955E+00_JPRB, 0.18741E+00_JPRB, 0.18529E+00_JPRB, 0.18374E+00_JPRB ,&
&  0.18214E+00_JPRB, 0.18095E+00_JPRB, 0.17962E+00_JPRB, 0.17842E+00_JPRB, 0.17715E+00_JPRB, 0.17595E+00_JPRB, 0.17448E+00_JPRB ,&
&  0.17347E+00_JPRB, 0.17242E+00_JPRB, 0.17135E+00_JPRB, 0.17037E+00_JPRB, 0.16939E+00_JPRB, 0.16843E+00_JPRB, 0.16735E+00_JPRB ,&
&  0.16635E+00_JPRB, 0.16570E+00_JPRB, 0.16503E+00_JPRB, 0.16428E+00_JPRB, 0.16360E+00_JPRB, 0.16313E+00_JPRB, 0.16273E+00_JPRB ,&
&  0.16236E+00_JPRB, 0.16234E+00_JPRB, 0.16252E+00_JPRB, 0.16270E+00_JPRB, 0.16290E+00_JPRB, 0.16333E+00_JPRB, 0.16397E+00_JPRB ,&
&  0.16472E+00_JPRB, 0.16553E+00_JPRB, 0.16655E+00_JPRB, 0.16802E+00_JPRB, 0.16964E+00_JPRB, 0.17150E+00_JPRB, 0.17352E+00_JPRB ,&
&  0.17572E+00_JPRB, 0.17804E+00_JPRB, 0.18047E+00_JPRB, 0.18277E+00_JPRB, 0.18532E+00_JPRB, 0.18782E+00_JPRB, 0.19026E+00_JPRB ,&
&  0.19282E+00_JPRB, 0.19539E+00_JPRB, 0.19809E+00_JPRB, 0.20058E+00_JPRB, 0.20458E+00_JPRB, 0.20720E+00_JPRB, 0.20986E+00_JPRB ,&
&  0.21225E+00_JPRB, 0.21503E+00_JPRB, 0.21784E+00_JPRB, 0.22128E+00_JPRB, 0.22473E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB, 0.19198E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB, 0.52897E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.56595E+00_JPRB, 0.56394E+00_JPRB, 0.56164E+00_JPRB, 0.55959E+00_JPRB, 0.55708E+00_JPRB, 0.55501E+00_JPRB, 0.55294E+00_JPRB ,&
&  0.55103E+00_JPRB, 0.54891E+00_JPRB, 0.54690E+00_JPRB, 0.54460E+00_JPRB, 0.54243E+00_JPRB, 0.54034E+00_JPRB, 0.53838E+00_JPRB ,&
&  0.53638E+00_JPRB, 0.53453E+00_JPRB, 0.53263E+00_JPRB, 0.53055E+00_JPRB, 0.52879E+00_JPRB, 0.52675E+00_JPRB, 0.52412E+00_JPRB ,&
&  0.52003E+00_JPRB, 0.51569E+00_JPRB, 0.51185E+00_JPRB, 0.50914E+00_JPRB, 0.50706E+00_JPRB, 0.50533E+00_JPRB, 0.50378E+00_JPRB ,&
&  0.50265E+00_JPRB, 0.50195E+00_JPRB, 0.50146E+00_JPRB, 0.50076E+00_JPRB, 0.50054E+00_JPRB, 0.50138E+00_JPRB, 0.50177E+00_JPRB ,&
&  0.50181E+00_JPRB, 0.50191E+00_JPRB, 0.50274E+00_JPRB, 0.50340E+00_JPRB, 0.50402E+00_JPRB, 0.50489E+00_JPRB, 0.50665E+00_JPRB ,&
&  0.50838E+00_JPRB, 0.50996E+00_JPRB, 0.51180E+00_JPRB, 0.51374E+00_JPRB, 0.51569E+00_JPRB, 0.51767E+00_JPRB, 0.52033E+00_JPRB ,&
&  0.52327E+00_JPRB, 0.52587E+00_JPRB, 0.52817E+00_JPRB, 0.53007E+00_JPRB, 0.53171E+00_JPRB, 0.53262E+00_JPRB, 0.53325E+00_JPRB ,&
&  0.53324E+00_JPRB, 0.53407E+00_JPRB, 0.53428E+00_JPRB, 0.53458E+00_JPRB, 0.53435E+00_JPRB, 0.53522E+00_JPRB, 0.53573E+00_JPRB ,&
&  0.53704E+00_JPRB, 0.53788E+00_JPRB, 0.53943E+00_JPRB, 0.54091E+00_JPRB, 0.54322E+00_JPRB, 0.54584E+00_JPRB, 0.54872E+00_JPRB ,&
&  0.55191E+00_JPRB, 0.55548E+00_JPRB, 0.55955E+00_JPRB, 0.56205E+00_JPRB, 0.56453E+00_JPRB, 0.56702E+00_JPRB, 0.56969E+00_JPRB ,&
&  0.57170E+00_JPRB, 0.57359E+00_JPRB, 0.57556E+00_JPRB, 0.57711E+00_JPRB, 0.57799E+00_JPRB, 0.57865E+00_JPRB, 0.57931E+00_JPRB ,&
&  0.57956E+00_JPRB, 0.57970E+00_JPRB, 0.57968E+00_JPRB, 0.57974E+00_JPRB, 0.57895E+00_JPRB, 0.57770E+00_JPRB, 0.57625E+00_JPRB ,&
&  0.57502E+00_JPRB, 0.57331E+00_JPRB, 0.57164E+00_JPRB, 0.56984E+00_JPRB, 0.56811E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.63551E+00_JPRB, 0.63199E+00_JPRB, 0.62829E+00_JPRB, 0.62493E+00_JPRB, 0.62010E+00_JPRB, 0.61524E+00_JPRB, 0.61020E+00_JPRB ,&
&  0.60502E+00_JPRB, 0.59868E+00_JPRB, 0.59183E+00_JPRB, 0.58458E+00_JPRB, 0.57731E+00_JPRB, 0.57068E+00_JPRB, 0.56514E+00_JPRB ,&
&  0.56007E+00_JPRB, 0.55484E+00_JPRB, 0.55030E+00_JPRB, 0.54448E+00_JPRB, 0.53860E+00_JPRB, 0.53337E+00_JPRB, 0.52967E+00_JPRB ,&
&  0.52388E+00_JPRB, 0.51716E+00_JPRB, 0.51004E+00_JPRB, 0.50220E+00_JPRB, 0.49484E+00_JPRB, 0.48753E+00_JPRB, 0.48027E+00_JPRB ,&
&  0.47338E+00_JPRB, 0.46809E+00_JPRB, 0.46340E+00_JPRB, 0.45870E+00_JPRB, 0.45411E+00_JPRB, 0.45275E+00_JPRB, 0.45125E+00_JPRB ,&
&  0.44991E+00_JPRB, 0.44962E+00_JPRB, 0.45275E+00_JPRB, 0.45558E+00_JPRB, 0.45855E+00_JPRB, 0.46164E+00_JPRB, 0.46689E+00_JPRB ,&
&  0.47178E+00_JPRB, 0.47757E+00_JPRB, 0.48496E+00_JPRB, 0.49224E+00_JPRB, 0.49811E+00_JPRB, 0.50243E+00_JPRB, 0.50821E+00_JPRB ,&
&  0.51224E+00_JPRB, 0.51758E+00_JPRB, 0.52124E+00_JPRB, 0.52292E+00_JPRB, 0.52260E+00_JPRB, 0.52245E+00_JPRB, 0.52322E+00_JPRB ,&
&  0.52327E+00_JPRB, 0.52610E+00_JPRB, 0.52890E+00_JPRB, 0.53184E+00_JPRB, 0.53315E+00_JPRB, 0.53728E+00_JPRB, 0.54064E+00_JPRB ,&
&  0.54644E+00_JPRB, 0.55436E+00_JPRB, 0.56260E+00_JPRB, 0.57162E+00_JPRB, 0.58262E+00_JPRB, 0.59531E+00_JPRB, 0.60560E+00_JPRB ,&
&  0.61534E+00_JPRB, 0.62370E+00_JPRB, 0.63066E+00_JPRB, 0.63463E+00_JPRB, 0.63840E+00_JPRB, 0.64203E+00_JPRB, 0.64602E+00_JPRB ,&
&  0.64791E+00_JPRB, 0.65014E+00_JPRB, 0.65248E+00_JPRB, 0.65435E+00_JPRB, 0.65402E+00_JPRB, 0.65349E+00_JPRB, 0.65357E+00_JPRB ,&
&  0.65345E+00_JPRB, 0.65293E+00_JPRB, 0.65255E+00_JPRB, 0.65274E+00_JPRB, 0.65229E+00_JPRB, 0.65123E+00_JPRB, 0.65009E+00_JPRB ,&
&  0.64903E+00_JPRB, 0.64691E+00_JPRB, 0.64420E+00_JPRB, 0.64148E+00_JPRB, 0.63904E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.66404E+00_JPRB, 0.66300E+00_JPRB, 0.66182E+00_JPRB, 0.66045E+00_JPRB, 0.65749E+00_JPRB, 0.65410E+00_JPRB, 0.64973E+00_JPRB ,&
&  0.64455E+00_JPRB, 0.63709E+00_JPRB, 0.62928E+00_JPRB, 0.62078E+00_JPRB, 0.61271E+00_JPRB, 0.60562E+00_JPRB, 0.59888E+00_JPRB ,&
&  0.59182E+00_JPRB, 0.58281E+00_JPRB, 0.57255E+00_JPRB, 0.55902E+00_JPRB, 0.54652E+00_JPRB, 0.53607E+00_JPRB, 0.52459E+00_JPRB ,&
&  0.50826E+00_JPRB, 0.49012E+00_JPRB, 0.47393E+00_JPRB, 0.45781E+00_JPRB, 0.44604E+00_JPRB, 0.41673E+00_JPRB, 0.40710E+00_JPRB ,&
&  0.40047E+00_JPRB, 0.39314E+00_JPRB, 0.38937E+00_JPRB, 0.38621E+00_JPRB, 0.38538E+00_JPRB, 0.38016E+00_JPRB, 0.37734E+00_JPRB ,&
&  0.36461E+00_JPRB, 0.36737E+00_JPRB, 0.37110E+00_JPRB, 0.37556E+00_JPRB, 0.38429E+00_JPRB, 0.39398E+00_JPRB, 0.40914E+00_JPRB ,&
&  0.43634E+00_JPRB, 0.44832E+00_JPRB, 0.46654E+00_JPRB, 0.48085E+00_JPRB, 0.48966E+00_JPRB, 0.49465E+00_JPRB, 0.50053E+00_JPRB ,&
&  0.50675E+00_JPRB, 0.51601E+00_JPRB, 0.52159E+00_JPRB, 0.52547E+00_JPRB, 0.52545E+00_JPRB, 0.52414E+00_JPRB, 0.52373E+00_JPRB ,&
&  0.52266E+00_JPRB, 0.52530E+00_JPRB, 0.53023E+00_JPRB, 0.54026E+00_JPRB, 0.55419E+00_JPRB, 0.57231E+00_JPRB, 0.59123E+00_JPRB ,&
&  0.61027E+00_JPRB, 0.63105E+00_JPRB, 0.64859E+00_JPRB, 0.66518E+00_JPRB, 0.68265E+00_JPRB, 0.69690E+00_JPRB, 0.70897E+00_JPRB ,&
&  0.71712E+00_JPRB, 0.72026E+00_JPRB, 0.71782E+00_JPRB, 0.71442E+00_JPRB, 0.70949E+00_JPRB, 0.70582E+00_JPRB, 0.70542E+00_JPRB ,&
&  0.70525E+00_JPRB, 0.70612E+00_JPRB, 0.70759E+00_JPRB, 0.70589E+00_JPRB, 0.70351E+00_JPRB, 0.70048E+00_JPRB, 0.69761E+00_JPRB ,&
&  0.69391E+00_JPRB, 0.69080E+00_JPRB, 0.68790E+00_JPRB, 0.68508E+00_JPRB, 0.68253E+00_JPRB, 0.67873E+00_JPRB, 0.67548E+00_JPRB ,&
&  0.67216E+00_JPRB, 0.67088E+00_JPRB, 0.66862E+00_JPRB, 0.66686E+00_JPRB, 0.66509E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.62547E+00_JPRB, 0.62541E+00_JPRB, 0.62640E+00_JPRB, 0.62678E+00_JPRB, 0.62837E+00_JPRB, 0.62932E+00_JPRB, 0.63084E+00_JPRB ,&
&  0.63150E+00_JPRB, 0.63214E+00_JPRB, 0.63127E+00_JPRB, 0.62951E+00_JPRB, 0.62631E+00_JPRB, 0.62302E+00_JPRB, 0.61921E+00_JPRB ,&
&  0.61387E+00_JPRB, 0.60326E+00_JPRB, 0.58839E+00_JPRB, 0.57154E+00_JPRB, 0.55211E+00_JPRB, 0.53096E+00_JPRB, 0.50699E+00_JPRB ,&
&  0.48251E+00_JPRB, 0.45970E+00_JPRB, 0.43212E+00_JPRB, 0.42052E+00_JPRB, 0.40311E+00_JPRB, 0.38934E+00_JPRB, 0.37899E+00_JPRB ,&
&  0.36647E+00_JPRB, 0.35775E+00_JPRB, 0.34890E+00_JPRB, 0.34046E+00_JPRB, 0.33596E+00_JPRB, 0.32767E+00_JPRB, 0.32411E+00_JPRB ,&
&  0.32165E+00_JPRB, 0.32054E+00_JPRB, 0.32099E+00_JPRB, 0.32336E+00_JPRB, 0.32868E+00_JPRB, 0.33537E+00_JPRB, 0.34810E+00_JPRB ,&
&  0.37668E+00_JPRB, 0.41708E+00_JPRB, 0.47888E+00_JPRB, 0.49139E+00_JPRB, 0.49128E+00_JPRB, 0.50065E+00_JPRB, 0.51109E+00_JPRB ,&
&  0.52136E+00_JPRB, 0.53080E+00_JPRB, 0.53851E+00_JPRB, 0.54231E+00_JPRB, 0.54692E+00_JPRB, 0.55087E+00_JPRB, 0.55529E+00_JPRB ,&
&  0.55992E+00_JPRB, 0.57081E+00_JPRB, 0.58650E+00_JPRB, 0.60537E+00_JPRB, 0.62586E+00_JPRB, 0.64345E+00_JPRB, 0.65891E+00_JPRB ,&
&  0.67150E+00_JPRB, 0.68519E+00_JPRB, 0.70415E+00_JPRB, 0.72428E+00_JPRB, 0.74491E+00_JPRB, 0.75649E+00_JPRB, 0.76645E+00_JPRB ,&
&  0.77251E+00_JPRB, 0.77254E+00_JPRB, 0.76579E+00_JPRB, 0.75723E+00_JPRB, 0.75076E+00_JPRB, 0.75480E+00_JPRB, 0.75359E+00_JPRB ,&
&  0.75265E+00_JPRB, 0.74599E+00_JPRB, 0.73532E+00_JPRB, 0.72227E+00_JPRB, 0.71299E+00_JPRB, 0.70468E+00_JPRB, 0.69960E+00_JPRB ,&
&  0.69468E+00_JPRB, 0.68952E+00_JPRB, 0.68251E+00_JPRB, 0.67329E+00_JPRB, 0.66361E+00_JPRB, 0.65326E+00_JPRB, 0.64506E+00_JPRB ,&
&  0.63775E+00_JPRB, 0.63333E+00_JPRB, 0.62954E+00_JPRB, 0.62754E+00_JPRB, 0.62563E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.60110E+00_JPRB, 0.60621E+00_JPRB, 0.61165E+00_JPRB, 0.61550E+00_JPRB, 0.61870E+00_JPRB, 0.62102E+00_JPRB, 0.62187E+00_JPRB ,&
&  0.62210E+00_JPRB, 0.62205E+00_JPRB, 0.62232E+00_JPRB, 0.62222E+00_JPRB, 0.62236E+00_JPRB, 0.62300E+00_JPRB, 0.62315E+00_JPRB ,&
&  0.62284E+00_JPRB, 0.61498E+00_JPRB, 0.59937E+00_JPRB, 0.58156E+00_JPRB, 0.56216E+00_JPRB, 0.53746E+00_JPRB, 0.50726E+00_JPRB ,&
&  0.47999E+00_JPRB, 0.46012E+00_JPRB, 0.43943E+00_JPRB, 0.42584E+00_JPRB, 0.41105E+00_JPRB, 0.40208E+00_JPRB, 0.38949E+00_JPRB ,&
&  0.38377E+00_JPRB, 0.35710E+00_JPRB, 0.35416E+00_JPRB, 0.33731E+00_JPRB, 0.32500E+00_JPRB, 0.31479E+00_JPRB, 0.30722E+00_JPRB ,&
&  0.30536E+00_JPRB, 0.30244E+00_JPRB, 0.30245E+00_JPRB, 0.30620E+00_JPRB, 0.31081E+00_JPRB, 0.32075E+00_JPRB, 0.33186E+00_JPRB ,&
&  0.35658E+00_JPRB, 0.43868E+00_JPRB, 0.47007E+00_JPRB, 0.48045E+00_JPRB, 0.48725E+00_JPRB, 0.50276E+00_JPRB, 0.51742E+00_JPRB ,&
&  0.53708E+00_JPRB, 0.55004E+00_JPRB, 0.56239E+00_JPRB, 0.56991E+00_JPRB, 0.58891E+00_JPRB, 0.60551E+00_JPRB, 0.62000E+00_JPRB ,&
&  0.63557E+00_JPRB, 0.65101E+00_JPRB, 0.66882E+00_JPRB, 0.68034E+00_JPRB, 0.68191E+00_JPRB, 0.68080E+00_JPRB, 0.67930E+00_JPRB ,&
&  0.68361E+00_JPRB, 0.69023E+00_JPRB, 0.71333E+00_JPRB, 0.74387E+00_JPRB, 0.77773E+00_JPRB, 0.80073E+00_JPRB, 0.81265E+00_JPRB ,&
&  0.82104E+00_JPRB, 0.82534E+00_JPRB, 0.83281E+00_JPRB, 0.84399E+00_JPRB, 0.83622E+00_JPRB, 0.82010E+00_JPRB, 0.81142E+00_JPRB ,&
&  0.80145E+00_JPRB, 0.77862E+00_JPRB, 0.76038E+00_JPRB, 0.74465E+00_JPRB, 0.73259E+00_JPRB, 0.72235E+00_JPRB, 0.71201E+00_JPRB ,&
&  0.69870E+00_JPRB, 0.68453E+00_JPRB, 0.66817E+00_JPRB, 0.64793E+00_JPRB, 0.63033E+00_JPRB, 0.61236E+00_JPRB, 0.60176E+00_JPRB ,&
&  0.59313E+00_JPRB, 0.59109E+00_JPRB, 0.58927E+00_JPRB, 0.59252E+00_JPRB, 0.59488E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.58357E+00_JPRB, 0.59564E+00_JPRB, 0.60528E+00_JPRB, 0.61108E+00_JPRB, 0.61489E+00_JPRB, 0.61791E+00_JPRB, 0.61795E+00_JPRB ,&
&  0.61791E+00_JPRB, 0.61660E+00_JPRB, 0.61660E+00_JPRB, 0.61611E+00_JPRB, 0.61702E+00_JPRB, 0.61665E+00_JPRB, 0.61600E+00_JPRB ,&
&  0.61514E+00_JPRB, 0.61271E+00_JPRB, 0.60448E+00_JPRB, 0.58927E+00_JPRB, 0.56998E+00_JPRB, 0.54854E+00_JPRB, 0.51756E+00_JPRB ,&
&  0.48627E+00_JPRB, 0.46564E+00_JPRB, 0.44763E+00_JPRB, 0.43348E+00_JPRB, 0.42502E+00_JPRB, 0.41394E+00_JPRB, 0.41118E+00_JPRB ,&
&  0.40835E+00_JPRB, 0.39789E+00_JPRB, 0.38457E+00_JPRB, 0.37066E+00_JPRB, 0.35635E+00_JPRB, 0.32132E+00_JPRB, 0.31753E+00_JPRB ,&
&  0.30899E+00_JPRB, 0.30086E+00_JPRB, 0.29888E+00_JPRB, 0.30018E+00_JPRB, 0.30367E+00_JPRB, 0.31492E+00_JPRB, 0.32881E+00_JPRB ,&
&  0.34806E+00_JPRB, 0.44273E+00_JPRB, 0.46504E+00_JPRB, 0.47769E+00_JPRB, 0.49840E+00_JPRB, 0.52023E+00_JPRB, 0.53819E+00_JPRB ,&
&  0.55527E+00_JPRB, 0.57003E+00_JPRB, 0.58972E+00_JPRB, 0.60865E+00_JPRB, 0.62949E+00_JPRB, 0.64856E+00_JPRB, 0.66251E+00_JPRB ,&
&  0.67533E+00_JPRB, 0.67901E+00_JPRB, 0.70437E+00_JPRB, 0.71377E+00_JPRB, 0.70779E+00_JPRB, 0.72610E+00_JPRB, 0.73653E+00_JPRB ,&
&  0.73486E+00_JPRB, 0.72762E+00_JPRB, 0.74469E+00_JPRB, 0.78587E+00_JPRB, 0.84410E+00_JPRB, 0.88962E+00_JPRB, 0.90302E+00_JPRB ,&
&  0.90841E+00_JPRB, 0.90298E+00_JPRB, 0.89540E+00_JPRB, 0.89396E+00_JPRB, 0.87046E+00_JPRB, 0.83788E+00_JPRB, 0.81486E+00_JPRB ,&
&  0.79551E+00_JPRB, 0.79356E+00_JPRB, 0.78372E+00_JPRB, 0.77310E+00_JPRB, 0.75899E+00_JPRB, 0.74484E+00_JPRB, 0.72280E+00_JPRB ,&
&  0.69060E+00_JPRB, 0.65540E+00_JPRB, 0.62751E+00_JPRB, 0.60341E+00_JPRB, 0.58940E+00_JPRB, 0.57398E+00_JPRB, 0.56657E+00_JPRB ,&
&  0.55954E+00_JPRB, 0.55894E+00_JPRB, 0.55872E+00_JPRB, 0.56425E+00_JPRB, 0.57142E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.58050E+00_JPRB, 0.59805E+00_JPRB, 0.61347E+00_JPRB, 0.62285E+00_JPRB, 0.62472E+00_JPRB, 0.62553E+00_JPRB, 0.62392E+00_JPRB ,&
&  0.62328E+00_JPRB, 0.62263E+00_JPRB, 0.62133E+00_JPRB, 0.61770E+00_JPRB, 0.61411E+00_JPRB, 0.60928E+00_JPRB, 0.60651E+00_JPRB ,&
&  0.60282E+00_JPRB, 0.59320E+00_JPRB, 0.58644E+00_JPRB, 0.57919E+00_JPRB, 0.56661E+00_JPRB, 0.54969E+00_JPRB, 0.52423E+00_JPRB ,&
&  0.49459E+00_JPRB, 0.47181E+00_JPRB, 0.45603E+00_JPRB, 0.44141E+00_JPRB, 0.43003E+00_JPRB, 0.42334E+00_JPRB, 0.41952E+00_JPRB ,&
&  0.41785E+00_JPRB, 0.41178E+00_JPRB, 0.40772E+00_JPRB, 0.38944E+00_JPRB, 0.36631E+00_JPRB, 0.35831E+00_JPRB, 0.33912E+00_JPRB ,&
&  0.32016E+00_JPRB, 0.30534E+00_JPRB, 0.30192E+00_JPRB, 0.30231E+00_JPRB, 0.30797E+00_JPRB, 0.31998E+00_JPRB, 0.34651E+00_JPRB ,&
&  0.39015E+00_JPRB, 0.46624E+00_JPRB, 0.48342E+00_JPRB, 0.50349E+00_JPRB, 0.52473E+00_JPRB, 0.54519E+00_JPRB, 0.55959E+00_JPRB ,&
&  0.57135E+00_JPRB, 0.58307E+00_JPRB, 0.60496E+00_JPRB, 0.62758E+00_JPRB, 0.66489E+00_JPRB, 0.70108E+00_JPRB, 0.73248E+00_JPRB ,&
&  0.76144E+00_JPRB, 0.73488E+00_JPRB, 0.74112E+00_JPRB, 0.74903E+00_JPRB, 0.77909E+00_JPRB, 0.80585E+00_JPRB, 0.80086E+00_JPRB ,&
&  0.80127E+00_JPRB, 0.83435E+00_JPRB, 0.83954E+00_JPRB, 0.86252E+00_JPRB, 0.91424E+00_JPRB, 0.97115E+00_JPRB, 0.99344E+00_JPRB ,&
&  0.97463E+00_JPRB, 0.97503E+00_JPRB, 0.10222E+01_JPRB, 0.97712E+00_JPRB, 0.94967E+00_JPRB, 0.90582E+00_JPRB, 0.85290E+00_JPRB ,&
&  0.82438E+00_JPRB, 0.80949E+00_JPRB, 0.80392E+00_JPRB, 0.79696E+00_JPRB, 0.78207E+00_JPRB, 0.75821E+00_JPRB, 0.71538E+00_JPRB ,&
&  0.66048E+00_JPRB, 0.62158E+00_JPRB, 0.59420E+00_JPRB, 0.57646E+00_JPRB, 0.56371E+00_JPRB, 0.54996E+00_JPRB, 0.53927E+00_JPRB ,&
&  0.53109E+00_JPRB, 0.53111E+00_JPRB, 0.53806E+00_JPRB, 0.55275E+00_JPRB, 0.56761E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.60113E+00_JPRB, 0.62015E+00_JPRB, 0.63122E+00_JPRB, 0.62969E+00_JPRB, 0.61810E+00_JPRB, 0.60066E+00_JPRB, 0.58762E+00_JPRB ,&
&  0.58322E+00_JPRB, 0.58421E+00_JPRB, 0.58699E+00_JPRB, 0.58456E+00_JPRB, 0.58336E+00_JPRB, 0.58293E+00_JPRB, 0.58412E+00_JPRB ,&
&  0.58478E+00_JPRB, 0.58511E+00_JPRB, 0.58429E+00_JPRB, 0.57590E+00_JPRB, 0.56345E+00_JPRB, 0.54689E+00_JPRB, 0.52658E+00_JPRB ,&
&  0.50367E+00_JPRB, 0.48475E+00_JPRB, 0.46896E+00_JPRB, 0.45643E+00_JPRB, 0.44349E+00_JPRB, 0.43659E+00_JPRB, 0.43295E+00_JPRB ,&
&  0.43061E+00_JPRB, 0.43038E+00_JPRB, 0.42442E+00_JPRB, 0.41526E+00_JPRB, 0.39652E+00_JPRB, 0.37392E+00_JPRB, 0.35163E+00_JPRB ,&
&  0.33365E+00_JPRB, 0.32278E+00_JPRB, 0.31683E+00_JPRB, 0.31596E+00_JPRB, 0.32487E+00_JPRB, 0.34419E+00_JPRB, 0.38452E+00_JPRB ,&
&  0.45892E+00_JPRB, 0.48825E+00_JPRB, 0.50331E+00_JPRB, 0.52087E+00_JPRB, 0.53675E+00_JPRB, 0.54892E+00_JPRB, 0.55976E+00_JPRB ,&
&  0.56851E+00_JPRB, 0.57711E+00_JPRB, 0.59128E+00_JPRB, 0.60449E+00_JPRB, 0.63004E+00_JPRB, 0.65257E+00_JPRB, 0.72774E+00_JPRB ,&
&  0.87242E+00_JPRB, 0.88053E+00_JPRB, 0.81375E+00_JPRB, 0.82436E+00_JPRB, 0.86086E+00_JPRB, 0.89679E+00_JPRB, 0.93376E+00_JPRB ,&
&  0.96144E+00_JPRB, 0.97155E+00_JPRB, 0.97694E+00_JPRB, 0.98046E+00_JPRB, 0.10013E+01_JPRB, 0.10244E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10480E+01_JPRB, 0.11049E+01_JPRB, 0.94003E+00_JPRB, 0.86654E+00_JPRB, 0.84757E+00_JPRB, 0.81777E+00_JPRB ,&
&  0.79925E+00_JPRB, 0.79687E+00_JPRB, 0.81005E+00_JPRB, 0.81576E+00_JPRB, 0.77229E+00_JPRB, 0.71192E+00_JPRB, 0.64719E+00_JPRB ,&
&  0.61043E+00_JPRB, 0.59186E+00_JPRB, 0.57309E+00_JPRB, 0.56127E+00_JPRB, 0.55091E+00_JPRB, 0.53793E+00_JPRB, 0.52125E+00_JPRB ,&
&  0.51971E+00_JPRB, 0.52901E+00_JPRB, 0.54337E+00_JPRB, 0.56922E+00_JPRB, 0.58739E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.59721E+00_JPRB, 0.62447E+00_JPRB, 0.62906E+00_JPRB, 0.61167E+00_JPRB, 0.59217E+00_JPRB, 0.55819E+00_JPRB, 0.52827E+00_JPRB ,&
&  0.51556E+00_JPRB, 0.50954E+00_JPRB, 0.50919E+00_JPRB, 0.50999E+00_JPRB, 0.51362E+00_JPRB, 0.52047E+00_JPRB, 0.52773E+00_JPRB ,&
&  0.53175E+00_JPRB, 0.53507E+00_JPRB, 0.53906E+00_JPRB, 0.54366E+00_JPRB, 0.54166E+00_JPRB, 0.53008E+00_JPRB, 0.51560E+00_JPRB ,&
&  0.49549E+00_JPRB, 0.48071E+00_JPRB, 0.46966E+00_JPRB, 0.46072E+00_JPRB, 0.45460E+00_JPRB, 0.45265E+00_JPRB, 0.45370E+00_JPRB ,&
&  0.45223E+00_JPRB, 0.45122E+00_JPRB, 0.45050E+00_JPRB, 0.44393E+00_JPRB, 0.42671E+00_JPRB, 0.40076E+00_JPRB, 0.37155E+00_JPRB ,&
&  0.35205E+00_JPRB, 0.34490E+00_JPRB, 0.34770E+00_JPRB, 0.35920E+00_JPRB, 0.39198E+00_JPRB, 0.45120E+00_JPRB, 0.47777E+00_JPRB ,&
&  0.50025E+00_JPRB, 0.51388E+00_JPRB, 0.52608E+00_JPRB, 0.54142E+00_JPRB, 0.55409E+00_JPRB, 0.56324E+00_JPRB, 0.57185E+00_JPRB ,&
&  0.58530E+00_JPRB, 0.59968E+00_JPRB, 0.61939E+00_JPRB, 0.58284E+00_JPRB, 0.59847E+00_JPRB, 0.69470E+00_JPRB, 0.76710E+00_JPRB ,&
&  0.85807E+00_JPRB, 0.88489E+00_JPRB, 0.89537E+00_JPRB, 0.96881E+00_JPRB, 0.10049E+01_JPRB, 0.10549E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10479E+01_JPRB, 0.10288E+01_JPRB, 0.10138E+01_JPRB, 0.10289E+01_JPRB, 0.10640E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.10232E+01_JPRB, 0.94465E+00_JPRB, 0.83538E+00_JPRB, 0.79626E+00_JPRB, 0.79048E+00_JPRB, 0.79950E+00_JPRB ,&
&  0.79600E+00_JPRB, 0.80149E+00_JPRB, 0.80985E+00_JPRB, 0.80878E+00_JPRB, 0.73386E+00_JPRB, 0.65947E+00_JPRB, 0.59462E+00_JPRB ,&
&  0.56080E+00_JPRB, 0.55465E+00_JPRB, 0.54507E+00_JPRB, 0.54695E+00_JPRB, 0.53999E+00_JPRB, 0.53041E+00_JPRB, 0.51891E+00_JPRB ,&
&  0.51737E+00_JPRB, 0.52342E+00_JPRB, 0.54407E+00_JPRB, 0.56825E+00_JPRB, 0.58286E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.63218E+00_JPRB, 0.63748E+00_JPRB, 0.61671E+00_JPRB, 0.60136E+00_JPRB, 0.58381E+00_JPRB, 0.56379E+00_JPRB, 0.54124E+00_JPRB ,&
&  0.53615E+00_JPRB, 0.51236E+00_JPRB, 0.49389E+00_JPRB, 0.47835E+00_JPRB, 0.47006E+00_JPRB, 0.46690E+00_JPRB, 0.47404E+00_JPRB ,&
&  0.48920E+00_JPRB, 0.50869E+00_JPRB, 0.52103E+00_JPRB, 0.52216E+00_JPRB, 0.51752E+00_JPRB, 0.50483E+00_JPRB, 0.49536E+00_JPRB ,&
&  0.48031E+00_JPRB, 0.47223E+00_JPRB, 0.47219E+00_JPRB, 0.47531E+00_JPRB, 0.47881E+00_JPRB, 0.47769E+00_JPRB, 0.47176E+00_JPRB ,&
&  0.46383E+00_JPRB, 0.45982E+00_JPRB, 0.46264E+00_JPRB, 0.46460E+00_JPRB, 0.45476E+00_JPRB, 0.42318E+00_JPRB, 0.39245E+00_JPRB ,&
&  0.37261E+00_JPRB, 0.37137E+00_JPRB, 0.39847E+00_JPRB, 0.45953E+00_JPRB, 0.48647E+00_JPRB, 0.50442E+00_JPRB, 0.51502E+00_JPRB ,&
&  0.50998E+00_JPRB, 0.51446E+00_JPRB, 0.53825E+00_JPRB, 0.58266E+00_JPRB, 0.59058E+00_JPRB, 0.59848E+00_JPRB, 0.60634E+00_JPRB ,&
&  0.62566E+00_JPRB, 0.62527E+00_JPRB, 0.63989E+00_JPRB, 0.63363E+00_JPRB, 0.74201E+00_JPRB, 0.96980E+00_JPRB, 0.93444E+00_JPRB ,&
&  0.99533E+00_JPRB, 0.10238E+01_JPRB, 0.11058E+01_JPRB, 0.11426E+01_JPRB, 0.11476E+01_JPRB, 0.11581E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.10838E+01_JPRB, 0.10510E+01_JPRB, 0.10526E+01_JPRB, 0.10535E+01_JPRB, 0.10671E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10396E+01_JPRB, 0.97140E+00_JPRB, 0.88374E+00_JPRB, 0.83824E+00_JPRB, 0.79930E+00_JPRB, 0.77458E+00_JPRB ,&
&  0.76033E+00_JPRB, 0.77512E+00_JPRB, 0.79968E+00_JPRB, 0.80877E+00_JPRB, 0.74739E+00_JPRB, 0.66018E+00_JPRB, 0.58184E+00_JPRB ,&
&  0.53828E+00_JPRB, 0.52677E+00_JPRB, 0.50799E+00_JPRB, 0.50198E+00_JPRB, 0.50833E+00_JPRB, 0.51563E+00_JPRB, 0.51926E+00_JPRB ,&
&  0.52428E+00_JPRB, 0.53608E+00_JPRB, 0.56802E+00_JPRB, 0.60166E+00_JPRB, 0.62062E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.62926E+00_JPRB, 0.63641E+00_JPRB, 0.62827E+00_JPRB, 0.62521E+00_JPRB, 0.61872E+00_JPRB, 0.59616E+00_JPRB, 0.56079E+00_JPRB ,&
&  0.55825E+00_JPRB, 0.50889E+00_JPRB, 0.49063E+00_JPRB, 0.48100E+00_JPRB, 0.47936E+00_JPRB, 0.47334E+00_JPRB, 0.46402E+00_JPRB ,&
&  0.46761E+00_JPRB, 0.48329E+00_JPRB, 0.50750E+00_JPRB, 0.51899E+00_JPRB, 0.51822E+00_JPRB, 0.50645E+00_JPRB, 0.48851E+00_JPRB ,&
&  0.47103E+00_JPRB, 0.46662E+00_JPRB, 0.47483E+00_JPRB, 0.48714E+00_JPRB, 0.49523E+00_JPRB, 0.49368E+00_JPRB, 0.48849E+00_JPRB ,&
&  0.48162E+00_JPRB, 0.47452E+00_JPRB, 0.47289E+00_JPRB, 0.47416E+00_JPRB, 0.47298E+00_JPRB, 0.46216E+00_JPRB, 0.43157E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.41931E+00_JPRB, 0.47926E+00_JPRB, 0.51480E+00_JPRB, 0.53564E+00_JPRB, 0.54467E+00_JPRB, 0.55395E+00_JPRB ,&
&  0.56717E+00_JPRB, 0.58350E+00_JPRB, 0.60405E+00_JPRB, 0.62966E+00_JPRB, 0.64577E+00_JPRB, 0.65938E+00_JPRB, 0.67234E+00_JPRB ,&
&  0.71436E+00_JPRB, 0.62891E+00_JPRB, 0.60792E+00_JPRB, 0.65275E+00_JPRB, 0.91555E+00_JPRB, 0.10191E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.11323E+01_JPRB, 0.11751E+01_JPRB, 0.12404E+01_JPRB, 0.12789E+01_JPRB, 0.12886E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.12558E+01_JPRB, 0.12222E+01_JPRB, 0.11628E+01_JPRB, 0.11256E+01_JPRB, 0.10972E+01_JPRB, 0.11111E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10567E+01_JPRB, 0.99268E+00_JPRB, 0.92117E+00_JPRB, 0.86601E+00_JPRB, 0.83057E+00_JPRB, 0.80255E+00_JPRB ,&
&  0.78338E+00_JPRB, 0.78772E+00_JPRB, 0.80358E+00_JPRB, 0.82474E+00_JPRB, 0.79625E+00_JPRB, 0.72519E+00_JPRB, 0.65403E+00_JPRB ,&
&  0.58631E+00_JPRB, 0.55330E+00_JPRB, 0.51900E+00_JPRB, 0.51134E+00_JPRB, 0.52316E+00_JPRB, 0.53870E+00_JPRB, 0.55472E+00_JPRB ,&
&  0.57455E+00_JPRB, 0.58855E+00_JPRB, 0.60218E+00_JPRB, 0.62113E+00_JPRB, 0.62486E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.62885E+00_JPRB, 0.64410E+00_JPRB, 0.64579E+00_JPRB, 0.65194E+00_JPRB, 0.65033E+00_JPRB, 0.61748E+00_JPRB, 0.59494E+00_JPRB ,&
&  0.62496E+00_JPRB, 0.57613E+00_JPRB, 0.59708E+00_JPRB, 0.53012E+00_JPRB, 0.50354E+00_JPRB, 0.49574E+00_JPRB, 0.48117E+00_JPRB ,&
&  0.47902E+00_JPRB, 0.48967E+00_JPRB, 0.50275E+00_JPRB, 0.51400E+00_JPRB, 0.51961E+00_JPRB, 0.50653E+00_JPRB, 0.49077E+00_JPRB ,&
&  0.47143E+00_JPRB, 0.46636E+00_JPRB, 0.47562E+00_JPRB, 0.48743E+00_JPRB, 0.49827E+00_JPRB, 0.50133E+00_JPRB, 0.49757E+00_JPRB ,&
&  0.48886E+00_JPRB, 0.48421E+00_JPRB, 0.48553E+00_JPRB, 0.48469E+00_JPRB, 0.48576E+00_JPRB, 0.48062E+00_JPRB, 0.47425E+00_JPRB ,&
&  0.47168E+00_JPRB, 0.48045E+00_JPRB, 0.52298E+00_JPRB, 0.55289E+00_JPRB, 0.57035E+00_JPRB, 0.57750E+00_JPRB, 0.59067E+00_JPRB ,&
&  0.60327E+00_JPRB, 0.61982E+00_JPRB, 0.64796E+00_JPRB, 0.68334E+00_JPRB, 0.71294E+00_JPRB, 0.73467E+00_JPRB, 0.76024E+00_JPRB ,&
&  0.80980E+00_JPRB, 0.68225E+00_JPRB, 0.77117E+00_JPRB, 0.79364E+00_JPRB, 0.95813E+00_JPRB, 0.11555E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.13369E+01_JPRB, 0.13018E+01_JPRB, 0.13462E+01_JPRB, 0.13706E+01_JPRB, 0.14049E+01_JPRB, 0.14279E+01_JPRB, 0.14345E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.13362E+01_JPRB, 0.13313E+01_JPRB, 0.12599E+01_JPRB, 0.12076E+01_JPRB, 0.12013E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.10953E+01_JPRB, 0.10554E+01_JPRB, 0.94096E+00_JPRB, 0.90728E+00_JPRB, 0.87877E+00_JPRB, 0.81441E+00_JPRB ,&
&  0.79053E+00_JPRB, 0.76444E+00_JPRB, 0.74171E+00_JPRB, 0.80094E+00_JPRB, 0.79458E+00_JPRB, 0.73212E+00_JPRB, 0.65692E+00_JPRB ,&
&  0.60277E+00_JPRB, 0.56628E+00_JPRB, 0.58044E+00_JPRB, 0.58377E+00_JPRB, 0.57451E+00_JPRB, 0.57653E+00_JPRB, 0.61447E+00_JPRB ,&
&  0.61078E+00_JPRB, 0.62051E+00_JPRB, 0.64504E+00_JPRB, 0.64502E+00_JPRB, 0.63286E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.64177E+00_JPRB, 0.66348E+00_JPRB, 0.66131E+00_JPRB, 0.67301E+00_JPRB, 0.68055E+00_JPRB, 0.65091E+00_JPRB, 0.61670E+00_JPRB ,&
&  0.62184E+00_JPRB, 0.62009E+00_JPRB, 0.62024E+00_JPRB, 0.61960E+00_JPRB, 0.62170E+00_JPRB, 0.57398E+00_JPRB, 0.49446E+00_JPRB ,&
&  0.46893E+00_JPRB, 0.47645E+00_JPRB, 0.49565E+00_JPRB, 0.51007E+00_JPRB, 0.51238E+00_JPRB, 0.50540E+00_JPRB, 0.49779E+00_JPRB ,&
&  0.48822E+00_JPRB, 0.48147E+00_JPRB, 0.48582E+00_JPRB, 0.49071E+00_JPRB, 0.49485E+00_JPRB, 0.50229E+00_JPRB, 0.50557E+00_JPRB ,&
&  0.49851E+00_JPRB, 0.49789E+00_JPRB, 0.49637E+00_JPRB, 0.48834E+00_JPRB, 0.50094E+00_JPRB, 0.49705E+00_JPRB, 0.49324E+00_JPRB ,&
&  0.50626E+00_JPRB, 0.52970E+00_JPRB, 0.55345E+00_JPRB, 0.58205E+00_JPRB, 0.60324E+00_JPRB, 0.61529E+00_JPRB, 0.63039E+00_JPRB ,&
&  0.65336E+00_JPRB, 0.68532E+00_JPRB, 0.72228E+00_JPRB, 0.76578E+00_JPRB, 0.81398E+00_JPRB, 0.80543E+00_JPRB, 0.87926E+00_JPRB ,&
&  0.88747E+00_JPRB, 0.92718E+00_JPRB, 0.94861E+00_JPRB, 0.90006E+00_JPRB, 0.10671E+01_JPRB, 0.12366E+01_JPRB, 0.12511E+01_JPRB ,&
&  0.13364E+01_JPRB, 0.12842E+01_JPRB, 0.14082E+01_JPRB, 0.15494E+01_JPRB, 0.16632E+01_JPRB, 0.16529E+01_JPRB, 0.16277E+01_JPRB ,&
&  0.16935E+01_JPRB, 0.16807E+01_JPRB, 0.14943E+01_JPRB, 0.14206E+01_JPRB, 0.13446E+01_JPRB, 0.13227E+01_JPRB, 0.12433E+01_JPRB ,&
&  0.12024E+01_JPRB, 0.11870E+01_JPRB, 0.11444E+01_JPRB, 0.10886E+01_JPRB, 0.10209E+01_JPRB, 0.95924E+00_JPRB, 0.90085E+00_JPRB ,&
&  0.83408E+00_JPRB, 0.74631E+00_JPRB, 0.71128E+00_JPRB, 0.73907E+00_JPRB, 0.77754E+00_JPRB, 0.70236E+00_JPRB, 0.62284E+00_JPRB ,&
&  0.59540E+00_JPRB, 0.60055E+00_JPRB, 0.64792E+00_JPRB, 0.64958E+00_JPRB, 0.63762E+00_JPRB, 0.62762E+00_JPRB, 0.60786E+00_JPRB ,&
&  0.59224E+00_JPRB, 0.63724E+00_JPRB, 0.65127E+00_JPRB, 0.64574E+00_JPRB, 0.63649E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.66854E+00_JPRB, 0.69997E+00_JPRB, 0.69426E+00_JPRB, 0.70453E+00_JPRB, 0.71850E+00_JPRB, 0.68017E+00_JPRB, 0.66313E+00_JPRB ,&
&  0.66001E+00_JPRB, 0.65412E+00_JPRB, 0.64380E+00_JPRB, 0.63840E+00_JPRB, 0.63636E+00_JPRB, 0.62528E+00_JPRB, 0.56760E+00_JPRB ,&
&  0.50365E+00_JPRB, 0.49902E+00_JPRB, 0.49489E+00_JPRB, 0.50685E+00_JPRB, 0.51978E+00_JPRB, 0.51433E+00_JPRB, 0.50956E+00_JPRB ,&
&  0.50980E+00_JPRB, 0.50343E+00_JPRB, 0.49915E+00_JPRB, 0.50004E+00_JPRB, 0.50561E+00_JPRB, 0.51659E+00_JPRB, 0.52158E+00_JPRB ,&
&  0.51085E+00_JPRB, 0.50450E+00_JPRB, 0.50085E+00_JPRB, 0.50037E+00_JPRB, 0.51323E+00_JPRB, 0.51820E+00_JPRB, 0.52145E+00_JPRB ,&
&  0.53473E+00_JPRB, 0.55668E+00_JPRB, 0.57965E+00_JPRB, 0.61302E+00_JPRB, 0.64349E+00_JPRB, 0.66865E+00_JPRB, 0.70331E+00_JPRB ,&
&  0.76818E+00_JPRB, 0.82998E+00_JPRB, 0.86109E+00_JPRB, 0.90477E+00_JPRB, 0.93987E+00_JPRB, 0.98801E+00_JPRB, 0.10326E+01_JPRB ,&
&  0.10535E+01_JPRB, 0.10604E+01_JPRB, 0.10278E+01_JPRB, 0.10731E+01_JPRB, 0.12491E+01_JPRB, 0.13674E+01_JPRB, 0.14249E+01_JPRB ,&
&  0.15307E+01_JPRB, 0.14752E+01_JPRB, 0.18091E+01_JPRB, 0.19209E+01_JPRB, 0.18762E+01_JPRB, 0.18961E+01_JPRB, 0.17966E+01_JPRB ,&
&  0.17764E+01_JPRB, 0.16761E+01_JPRB, 0.15969E+01_JPRB, 0.15269E+01_JPRB, 0.14617E+01_JPRB, 0.14700E+01_JPRB, 0.13933E+01_JPRB ,&
&  0.13803E+01_JPRB, 0.13440E+01_JPRB, 0.11741E+01_JPRB, 0.10484E+01_JPRB, 0.92013E+00_JPRB, 0.96467E+00_JPRB, 0.92783E+00_JPRB ,&
&  0.84994E+00_JPRB, 0.72956E+00_JPRB, 0.75902E+00_JPRB, 0.79998E+00_JPRB, 0.85707E+00_JPRB, 0.84347E+00_JPRB, 0.71494E+00_JPRB ,&
&  0.65681E+00_JPRB, 0.67669E+00_JPRB, 0.68276E+00_JPRB, 0.69079E+00_JPRB, 0.69162E+00_JPRB, 0.68574E+00_JPRB, 0.63365E+00_JPRB ,&
&  0.60890E+00_JPRB, 0.64456E+00_JPRB, 0.65821E+00_JPRB, 0.65528E+00_JPRB, 0.65661E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.70489E+00_JPRB, 0.74476E+00_JPRB, 0.75201E+00_JPRB, 0.74849E+00_JPRB, 0.75694E+00_JPRB, 0.71711E+00_JPRB, 0.69957E+00_JPRB ,&
&  0.70102E+00_JPRB, 0.69301E+00_JPRB, 0.67018E+00_JPRB, 0.66199E+00_JPRB, 0.65653E+00_JPRB, 0.64442E+00_JPRB, 0.62460E+00_JPRB ,&
&  0.53829E+00_JPRB, 0.51592E+00_JPRB, 0.49189E+00_JPRB, 0.49644E+00_JPRB, 0.54006E+00_JPRB, 0.53375E+00_JPRB, 0.53177E+00_JPRB ,&
&  0.52966E+00_JPRB, 0.52811E+00_JPRB, 0.51828E+00_JPRB, 0.51588E+00_JPRB, 0.52117E+00_JPRB, 0.53704E+00_JPRB, 0.55290E+00_JPRB ,&
&  0.54353E+00_JPRB, 0.52830E+00_JPRB, 0.52032E+00_JPRB, 0.52014E+00_JPRB, 0.53113E+00_JPRB, 0.54432E+00_JPRB, 0.56021E+00_JPRB ,&
&  0.57735E+00_JPRB, 0.59454E+00_JPRB, 0.62589E+00_JPRB, 0.67171E+00_JPRB, 0.71110E+00_JPRB, 0.74881E+00_JPRB, 0.82643E+00_JPRB ,&
&  0.94265E+00_JPRB, 0.10310E+01_JPRB, 0.10746E+01_JPRB, 0.11092E+01_JPRB, 0.11105E+01_JPRB, 0.11566E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.12419E+01_JPRB, 0.11892E+01_JPRB, 0.11620E+01_JPRB, 0.13877E+01_JPRB, 0.16149E+01_JPRB, 0.17004E+01_JPRB, 0.16945E+01_JPRB ,&
&  0.17390E+01_JPRB, 0.16968E+01_JPRB, 0.18270E+01_JPRB, 0.19656E+01_JPRB, 0.19237E+01_JPRB, 0.20031E+01_JPRB, 0.18712E+01_JPRB ,&
&  0.16783E+01_JPRB, 0.15728E+01_JPRB, 0.15358E+01_JPRB, 0.15322E+01_JPRB, 0.14834E+01_JPRB, 0.15555E+01_JPRB, 0.15336E+01_JPRB ,&
&  0.15502E+01_JPRB, 0.13842E+01_JPRB, 0.10623E+01_JPRB, 0.98422E+00_JPRB, 0.80193E+00_JPRB, 0.81996E+00_JPRB, 0.98984E+00_JPRB ,&
&  0.96408E+00_JPRB, 0.88449E+00_JPRB, 0.88900E+00_JPRB, 0.94228E+00_JPRB, 0.10244E+01_JPRB, 0.11538E+01_JPRB, 0.98757E+00_JPRB ,&
&  0.81574E+00_JPRB, 0.78769E+00_JPRB, 0.78846E+00_JPRB, 0.78438E+00_JPRB, 0.77592E+00_JPRB, 0.77933E+00_JPRB, 0.75824E+00_JPRB ,&
&  0.73076E+00_JPRB, 0.71006E+00_JPRB, 0.70288E+00_JPRB, 0.69495E+00_JPRB, 0.69864E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.74664E+00_JPRB, 0.78137E+00_JPRB, 0.79895E+00_JPRB, 0.79024E+00_JPRB, 0.79375E+00_JPRB, 0.76551E+00_JPRB, 0.73882E+00_JPRB ,&
&  0.73549E+00_JPRB, 0.72401E+00_JPRB, 0.69755E+00_JPRB, 0.68856E+00_JPRB, 0.67870E+00_JPRB, 0.65975E+00_JPRB, 0.64034E+00_JPRB ,&
&  0.63240E+00_JPRB, 0.57479E+00_JPRB, 0.52929E+00_JPRB, 0.50285E+00_JPRB, 0.53360E+00_JPRB, 0.53389E+00_JPRB, 0.54372E+00_JPRB ,&
&  0.54134E+00_JPRB, 0.55422E+00_JPRB, 0.55033E+00_JPRB, 0.54660E+00_JPRB, 0.55704E+00_JPRB, 0.57705E+00_JPRB, 0.59406E+00_JPRB ,&
&  0.59315E+00_JPRB, 0.58760E+00_JPRB, 0.57835E+00_JPRB, 0.58064E+00_JPRB, 0.60325E+00_JPRB, 0.60004E+00_JPRB, 0.62468E+00_JPRB ,&
&  0.64118E+00_JPRB, 0.65732E+00_JPRB, 0.68656E+00_JPRB, 0.73809E+00_JPRB, 0.79030E+00_JPRB, 0.84996E+00_JPRB, 0.96363E+00_JPRB ,&
&  0.10825E+01_JPRB, 0.11351E+01_JPRB, 0.11366E+01_JPRB, 0.11630E+01_JPRB, 0.12438E+01_JPRB, 0.13415E+01_JPRB, 0.13146E+01_JPRB ,&
&  0.12990E+01_JPRB, 0.12019E+01_JPRB, 0.11161E+01_JPRB, 0.14568E+01_JPRB, 0.17678E+01_JPRB, 0.18134E+01_JPRB, 0.18115E+01_JPRB ,&
&  0.18775E+01_JPRB, 0.19148E+01_JPRB, 0.20549E+01_JPRB, 0.21163E+01_JPRB, 0.21248E+01_JPRB, 0.20841E+01_JPRB, 0.18943E+01_JPRB ,&
&  0.16644E+01_JPRB, 0.15598E+01_JPRB, 0.15969E+01_JPRB, 0.16085E+01_JPRB, 0.16167E+01_JPRB, 0.15604E+01_JPRB, 0.16712E+01_JPRB ,&
&  0.17783E+01_JPRB, 0.10430E+01_JPRB, 0.84914E+00_JPRB, 0.97728E+00_JPRB, 0.85175E+00_JPRB, 0.86509E+00_JPRB, 0.95755E+00_JPRB ,&
&  0.91771E+00_JPRB, 0.89511E+00_JPRB, 0.10068E+01_JPRB, 0.10739E+01_JPRB, 0.12047E+01_JPRB, 0.13957E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.98730E+00_JPRB, 0.98724E+00_JPRB, 0.96043E+00_JPRB, 0.96711E+00_JPRB, 0.94722E+00_JPRB, 0.91858E+00_JPRB ,&
&  0.87225E+00_JPRB, 0.83797E+00_JPRB, 0.79991E+00_JPRB, 0.77287E+00_JPRB, 0.75595E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.82681E+00_JPRB, 0.84710E+00_JPRB, 0.85012E+00_JPRB, 0.84099E+00_JPRB, 0.83628E+00_JPRB, 0.82603E+00_JPRB, 0.79151E+00_JPRB ,&
&  0.76920E+00_JPRB, 0.75015E+00_JPRB, 0.72390E+00_JPRB, 0.71385E+00_JPRB, 0.69506E+00_JPRB, 0.67353E+00_JPRB, 0.65500E+00_JPRB ,&
&  0.64863E+00_JPRB, 0.62713E+00_JPRB, 0.55520E+00_JPRB, 0.52573E+00_JPRB, 0.50341E+00_JPRB, 0.51979E+00_JPRB, 0.53499E+00_JPRB ,&
&  0.55681E+00_JPRB, 0.56925E+00_JPRB, 0.57720E+00_JPRB, 0.59386E+00_JPRB, 0.62088E+00_JPRB, 0.63699E+00_JPRB, 0.66523E+00_JPRB ,&
&  0.67094E+00_JPRB, 0.67803E+00_JPRB, 0.65223E+00_JPRB, 0.66595E+00_JPRB, 0.68979E+00_JPRB, 0.68495E+00_JPRB, 0.72811E+00_JPRB ,&
&  0.75149E+00_JPRB, 0.78043E+00_JPRB, 0.81064E+00_JPRB, 0.83633E+00_JPRB, 0.87408E+00_JPRB, 0.91214E+00_JPRB, 0.10127E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.11143E+01_JPRB, 0.10925E+01_JPRB, 0.10977E+01_JPRB, 0.11536E+01_JPRB, 0.12239E+01_JPRB, 0.12333E+01_JPRB ,&
&  0.12659E+01_JPRB, 0.10711E+01_JPRB, 0.82045E+00_JPRB, 0.12426E+01_JPRB, 0.17841E+01_JPRB, 0.17763E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.17505E+01_JPRB, 0.18877E+01_JPRB, 0.21078E+01_JPRB, 0.21954E+01_JPRB, 0.21923E+01_JPRB, 0.21062E+01_JPRB, 0.19751E+01_JPRB ,&
&  0.18259E+01_JPRB, 0.18364E+01_JPRB, 0.19652E+01_JPRB, 0.21714E+01_JPRB, 0.21982E+01_JPRB, 0.21840E+01_JPRB, 0.21369E+01_JPRB ,&
&  0.18631E+01_JPRB, 0.15844E+01_JPRB, 0.11090E+01_JPRB, 0.11418E+01_JPRB, 0.90575E+00_JPRB, 0.90095E+00_JPRB, 0.98725E+00_JPRB ,&
&  0.94784E+00_JPRB, 0.99891E+00_JPRB, 0.11280E+01_JPRB, 0.12272E+01_JPRB, 0.15448E+01_JPRB, 0.16350E+01_JPRB, 0.15476E+01_JPRB ,&
&  0.13494E+01_JPRB, 0.11717E+01_JPRB, 0.11969E+01_JPRB, 0.11841E+01_JPRB, 0.11938E+01_JPRB, 0.11528E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10343E+01_JPRB, 0.97713E+00_JPRB, 0.93638E+00_JPRB, 0.88260E+00_JPRB, 0.83700E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.92380E+00_JPRB, 0.91406E+00_JPRB, 0.88818E+00_JPRB, 0.85801E+00_JPRB, 0.84143E+00_JPRB, 0.83284E+00_JPRB, 0.81517E+00_JPRB ,&
&  0.79966E+00_JPRB, 0.77247E+00_JPRB, 0.75012E+00_JPRB, 0.73014E+00_JPRB, 0.71235E+00_JPRB, 0.69003E+00_JPRB, 0.67480E+00_JPRB ,&
&  0.66791E+00_JPRB, 0.61813E+00_JPRB, 0.53491E+00_JPRB, 0.49926E+00_JPRB, 0.48090E+00_JPRB, 0.49317E+00_JPRB, 0.51934E+00_JPRB ,&
&  0.55051E+00_JPRB, 0.57333E+00_JPRB, 0.61022E+00_JPRB, 0.66060E+00_JPRB, 0.71463E+00_JPRB, 0.75480E+00_JPRB, 0.78109E+00_JPRB ,&
&  0.86758E+00_JPRB, 0.79566E+00_JPRB, 0.80795E+00_JPRB, 0.79127E+00_JPRB, 0.82418E+00_JPRB, 0.83696E+00_JPRB, 0.83687E+00_JPRB ,&
&  0.83503E+00_JPRB, 0.83399E+00_JPRB, 0.83693E+00_JPRB, 0.83689E+00_JPRB, 0.85481E+00_JPRB, 0.91000E+00_JPRB, 0.10308E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.11085E+01_JPRB, 0.10584E+01_JPRB, 0.10997E+01_JPRB, 0.11805E+01_JPRB, 0.12327E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.12012E+01_JPRB, 0.11117E+01_JPRB, 0.13032E+01_JPRB, 0.15202E+01_JPRB, 0.18012E+01_JPRB, 0.18824E+01_JPRB, 0.17963E+01_JPRB ,&
&  0.19078E+01_JPRB, 0.19688E+01_JPRB, 0.20880E+01_JPRB, 0.17330E+01_JPRB, 0.19773E+01_JPRB, 0.19830E+01_JPRB, 0.19597E+01_JPRB ,&
&  0.19607E+01_JPRB, 0.21136E+01_JPRB, 0.22797E+01_JPRB, 0.25843E+01_JPRB, 0.28008E+01_JPRB, 0.29564E+01_JPRB, 0.22755E+01_JPRB ,&
&  0.17969E+01_JPRB, 0.19102E+01_JPRB, 0.17337E+01_JPRB, 0.14249E+01_JPRB, 0.11821E+01_JPRB, 0.11863E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.11514E+01_JPRB, 0.12132E+01_JPRB, 0.13234E+01_JPRB, 0.16362E+01_JPRB, 0.19875E+01_JPRB, 0.21026E+01_JPRB, 0.19227E+01_JPRB ,&
&  0.16030E+01_JPRB, 0.16057E+01_JPRB, 0.15309E+01_JPRB, 0.14656E+01_JPRB, 0.14470E+01_JPRB, 0.13832E+01_JPRB, 0.12826E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.11017E+01_JPRB, 0.10425E+01_JPRB, 0.98307E+00_JPRB, 0.93875E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.98990E+00_JPRB, 0.96861E+00_JPRB, 0.93278E+00_JPRB, 0.89817E+00_JPRB, 0.87095E+00_JPRB, 0.85483E+00_JPRB, 0.83851E+00_JPRB ,&
&  0.83573E+00_JPRB, 0.78468E+00_JPRB, 0.76412E+00_JPRB, 0.74485E+00_JPRB, 0.72878E+00_JPRB, 0.71228E+00_JPRB, 0.70346E+00_JPRB ,&
&  0.69574E+00_JPRB, 0.60444E+00_JPRB, 0.51839E+00_JPRB, 0.51097E+00_JPRB, 0.50772E+00_JPRB, 0.49759E+00_JPRB, 0.51742E+00_JPRB ,&
&  0.54915E+00_JPRB, 0.58944E+00_JPRB, 0.65063E+00_JPRB, 0.75669E+00_JPRB, 0.85223E+00_JPRB, 0.92530E+00_JPRB, 0.98497E+00_JPRB ,&
&  0.97234E+00_JPRB, 0.98224E+00_JPRB, 0.10035E+01_JPRB, 0.95920E+00_JPRB, 0.91918E+00_JPRB, 0.89739E+00_JPRB, 0.89034E+00_JPRB ,&
&  0.88373E+00_JPRB, 0.86733E+00_JPRB, 0.86686E+00_JPRB, 0.87739E+00_JPRB, 0.90565E+00_JPRB, 0.98002E+00_JPRB, 0.10897E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.11061E+01_JPRB, 0.10858E+01_JPRB, 0.11843E+01_JPRB, 0.11218E+01_JPRB, 0.10858E+01_JPRB, 0.10542E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.14844E+01_JPRB, 0.14871E+01_JPRB, 0.16445E+01_JPRB, 0.15294E+01_JPRB, 0.16521E+01_JPRB, 0.19163E+01_JPRB ,&
&  0.20680E+01_JPRB, 0.20896E+01_JPRB, 0.21526E+01_JPRB, 0.15138E+01_JPRB, 0.14157E+01_JPRB, 0.19439E+01_JPRB, 0.20798E+01_JPRB ,&
&  0.21310E+01_JPRB, 0.24344E+01_JPRB, 0.28157E+01_JPRB, 0.32888E+01_JPRB, 0.25012E+01_JPRB, 0.21878E+01_JPRB, 0.16868E+01_JPRB ,&
&  0.16401E+01_JPRB, 0.15029E+01_JPRB, 0.17069E+01_JPRB, 0.15774E+01_JPRB, 0.15443E+01_JPRB, 0.14919E+01_JPRB, 0.13974E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.14318E+01_JPRB, 0.16623E+01_JPRB, 0.21942E+01_JPRB, 0.26568E+01_JPRB, 0.24845E+01_JPRB, 0.24333E+01_JPRB ,&
&  0.23128E+01_JPRB, 0.21552E+01_JPRB, 0.19045E+01_JPRB, 0.18338E+01_JPRB, 0.17154E+01_JPRB, 0.15627E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.12535E+01_JPRB, 0.11744E+01_JPRB, 0.11029E+01_JPRB, 0.10452E+01_JPRB, 0.10099E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.10529E+01_JPRB, 0.10211E+01_JPRB, 0.98546E+00_JPRB, 0.94839E+00_JPRB, 0.91826E+00_JPRB, 0.89361E+00_JPRB, 0.87584E+00_JPRB ,&
&  0.87598E+00_JPRB, 0.81888E+00_JPRB, 0.77219E+00_JPRB, 0.75159E+00_JPRB, 0.74328E+00_JPRB, 0.73830E+00_JPRB, 0.73626E+00_JPRB ,&
&  0.73174E+00_JPRB, 0.66869E+00_JPRB, 0.55126E+00_JPRB, 0.51940E+00_JPRB, 0.53627E+00_JPRB, 0.52362E+00_JPRB, 0.52118E+00_JPRB ,&
&  0.56997E+00_JPRB, 0.64614E+00_JPRB, 0.76015E+00_JPRB, 0.92388E+00_JPRB, 0.10486E+01_JPRB, 0.11489E+01_JPRB, 0.12010E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11230E+01_JPRB, 0.10239E+01_JPRB, 0.93604E+00_JPRB, 0.88985E+00_JPRB, 0.86677E+00_JPRB, 0.86525E+00_JPRB ,&
&  0.87389E+00_JPRB, 0.87833E+00_JPRB, 0.88864E+00_JPRB, 0.89560E+00_JPRB, 0.94397E+00_JPRB, 0.10002E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.11118E+01_JPRB, 0.11477E+01_JPRB, 0.12424E+01_JPRB, 0.12567E+01_JPRB, 0.11816E+01_JPRB, 0.11524E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.14137E+01_JPRB, 0.15331E+01_JPRB, 0.16396E+01_JPRB, 0.17716E+01_JPRB, 0.18572E+01_JPRB, 0.21022E+01_JPRB ,&
&  0.17971E+01_JPRB, 0.14640E+01_JPRB, 0.12422E+01_JPRB, 0.10877E+01_JPRB, 0.15335E+01_JPRB, 0.22451E+01_JPRB, 0.25751E+01_JPRB ,&
&  0.28446E+01_JPRB, 0.31622E+01_JPRB, 0.35113E+01_JPRB, 0.31548E+01_JPRB, 0.24168E+01_JPRB, 0.19085E+01_JPRB, 0.22859E+01_JPRB ,&
&  0.23772E+01_JPRB, 0.23728E+01_JPRB, 0.22121E+01_JPRB, 0.19383E+01_JPRB, 0.15545E+01_JPRB, 0.17677E+01_JPRB, 0.17067E+01_JPRB ,&
&  0.16502E+01_JPRB, 0.16675E+01_JPRB, 0.23720E+01_JPRB, 0.30754E+01_JPRB, 0.31947E+01_JPRB, 0.31308E+01_JPRB, 0.31310E+01_JPRB ,&
&  0.28356E+01_JPRB, 0.26552E+01_JPRB, 0.23508E+01_JPRB, 0.20552E+01_JPRB, 0.18037E+01_JPRB, 0.15919E+01_JPRB, 0.14207E+01_JPRB ,&
&  0.13083E+01_JPRB, 0.12269E+01_JPRB, 0.11454E+01_JPRB, 0.11050E+01_JPRB, 0.10765E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.11408E+01_JPRB, 0.11010E+01_JPRB, 0.10634E+01_JPRB, 0.10264E+01_JPRB, 0.99202E+00_JPRB, 0.94496E+00_JPRB, 0.92604E+00_JPRB ,&
&  0.91308E+00_JPRB, 0.85438E+00_JPRB, 0.80005E+00_JPRB, 0.77781E+00_JPRB, 0.78533E+00_JPRB, 0.78960E+00_JPRB, 0.78388E+00_JPRB ,&
&  0.77520E+00_JPRB, 0.77745E+00_JPRB, 0.63102E+00_JPRB, 0.57077E+00_JPRB, 0.56767E+00_JPRB, 0.56108E+00_JPRB, 0.54221E+00_JPRB ,&
&  0.63169E+00_JPRB, 0.77830E+00_JPRB, 0.93817E+00_JPRB, 0.11197E+01_JPRB, 0.13083E+01_JPRB, 0.12711E+01_JPRB, 0.12607E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.11938E+01_JPRB, 0.10913E+01_JPRB, 0.10083E+01_JPRB, 0.95446E+00_JPRB, 0.93046E+00_JPRB, 0.93190E+00_JPRB ,&
&  0.94558E+00_JPRB, 0.95469E+00_JPRB, 0.95847E+00_JPRB, 0.97910E+00_JPRB, 0.99068E+00_JPRB, 0.10378E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.11480E+01_JPRB, 0.11230E+01_JPRB, 0.11695E+01_JPRB, 0.13061E+01_JPRB, 0.14322E+01_JPRB, 0.13324E+01_JPRB, 0.15551E+01_JPRB ,&
&  0.14530E+01_JPRB, 0.16239E+01_JPRB, 0.17278E+01_JPRB, 0.19728E+01_JPRB, 0.20351E+01_JPRB, 0.19406E+01_JPRB, 0.20401E+01_JPRB ,&
&  0.16868E+01_JPRB, 0.15258E+01_JPRB, 0.17105E+01_JPRB, 0.19538E+01_JPRB, 0.21093E+01_JPRB, 0.22967E+01_JPRB, 0.35535E+01_JPRB ,&
&  0.30619E+01_JPRB, 0.34679E+01_JPRB, 0.37698E+01_JPRB, 0.37076E+01_JPRB, 0.18044E+01_JPRB, 0.13802E+01_JPRB, 0.20326E+01_JPRB ,&
&  0.22330E+01_JPRB, 0.15455E+01_JPRB, 0.11212E+01_JPRB, 0.13587E+01_JPRB, 0.10369E+01_JPRB, 0.12510E+01_JPRB, 0.16821E+01_JPRB ,&
&  0.18381E+01_JPRB, 0.20442E+01_JPRB, 0.30454E+01_JPRB, 0.35532E+01_JPRB, 0.36435E+01_JPRB, 0.41877E+01_JPRB, 0.35622E+01_JPRB ,&
&  0.32496E+01_JPRB, 0.27133E+01_JPRB, 0.25627E+01_JPRB, 0.22174E+01_JPRB, 0.19247E+01_JPRB, 0.17209E+01_JPRB, 0.15594E+01_JPRB ,&
&  0.14348E+01_JPRB, 0.13283E+01_JPRB, 0.12514E+01_JPRB, 0.12087E+01_JPRB, 0.11774E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.12683E+01_JPRB, 0.12105E+01_JPRB, 0.11572E+01_JPRB, 0.11096E+01_JPRB, 0.10554E+01_JPRB, 0.10092E+01_JPRB, 0.98795E+00_JPRB ,&
&  0.95197E+00_JPRB, 0.89022E+00_JPRB, 0.84683E+00_JPRB, 0.81355E+00_JPRB, 0.81477E+00_JPRB, 0.82034E+00_JPRB, 0.80589E+00_JPRB ,&
&  0.78398E+00_JPRB, 0.77393E+00_JPRB, 0.71971E+00_JPRB, 0.67169E+00_JPRB, 0.64500E+00_JPRB, 0.63552E+00_JPRB, 0.66244E+00_JPRB ,&
&  0.81873E+00_JPRB, 0.10860E+01_JPRB, 0.12424E+01_JPRB, 0.13628E+01_JPRB, 0.14177E+01_JPRB, 0.13511E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.14381E+01_JPRB, 0.13694E+01_JPRB, 0.12565E+01_JPRB, 0.11461E+01_JPRB, 0.10770E+01_JPRB, 0.10374E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.10166E+01_JPRB, 0.10280E+01_JPRB, 0.10305E+01_JPRB, 0.10158E+01_JPRB, 0.10125E+01_JPRB, 0.10622E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.11642E+01_JPRB, 0.11405E+01_JPRB, 0.12109E+01_JPRB, 0.13238E+01_JPRB, 0.16070E+01_JPRB, 0.16878E+01_JPRB, 0.14047E+01_JPRB ,&
&  0.12266E+01_JPRB, 0.13476E+01_JPRB, 0.18481E+01_JPRB, 0.19706E+01_JPRB, 0.19518E+01_JPRB, 0.19287E+01_JPRB, 0.18368E+01_JPRB ,&
&  0.18490E+01_JPRB, 0.22043E+01_JPRB, 0.24087E+01_JPRB, 0.27650E+01_JPRB, 0.29089E+01_JPRB, 0.22930E+01_JPRB, 0.31633E+01_JPRB ,&
&  0.34789E+01_JPRB, 0.34856E+01_JPRB, 0.28294E+01_JPRB, 0.19892E+01_JPRB, 0.20553E+01_JPRB, 0.15551E+01_JPRB, 0.10569E+01_JPRB ,&
&  0.98932E+00_JPRB, 0.94347E+00_JPRB, 0.85598E+00_JPRB, 0.86501E+00_JPRB, 0.84401E+00_JPRB, 0.11428E+01_JPRB, 0.16467E+01_JPRB ,&
&  0.22659E+01_JPRB, 0.28972E+01_JPRB, 0.36028E+01_JPRB, 0.40033E+01_JPRB, 0.38557E+01_JPRB, 0.42476E+01_JPRB, 0.38263E+01_JPRB ,&
&  0.30205E+01_JPRB, 0.26403E+01_JPRB, 0.24497E+01_JPRB, 0.21717E+01_JPRB, 0.19557E+01_JPRB, 0.18095E+01_JPRB, 0.16912E+01_JPRB ,&
&  0.15716E+01_JPRB, 0.14664E+01_JPRB, 0.13944E+01_JPRB, 0.13531E+01_JPRB, 0.13177E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.13631E+01_JPRB, 0.13011E+01_JPRB, 0.12451E+01_JPRB, 0.11971E+01_JPRB, 0.11432E+01_JPRB, 0.10948E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.99648E+00_JPRB, 0.92392E+00_JPRB, 0.87662E+00_JPRB, 0.84715E+00_JPRB, 0.83947E+00_JPRB, 0.84090E+00_JPRB, 0.83138E+00_JPRB ,&
&  0.81180E+00_JPRB, 0.79696E+00_JPRB, 0.78758E+00_JPRB, 0.79255E+00_JPRB, 0.81985E+00_JPRB, 0.87193E+00_JPRB, 0.98572E+00_JPRB ,&
&  0.12582E+01_JPRB, 0.15823E+01_JPRB, 0.16667E+01_JPRB, 0.17205E+01_JPRB, 0.17517E+01_JPRB, 0.17233E+01_JPRB, 0.16924E+01_JPRB ,&
&  0.16347E+01_JPRB, 0.15424E+01_JPRB, 0.14066E+01_JPRB, 0.12657E+01_JPRB, 0.11652E+01_JPRB, 0.11111E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.11005E+01_JPRB, 0.10882E+01_JPRB, 0.10552E+01_JPRB, 0.10372E+01_JPRB, 0.11028E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.11941E+01_JPRB, 0.12289E+01_JPRB, 0.13249E+01_JPRB, 0.13800E+01_JPRB, 0.13447E+01_JPRB, 0.11351E+01_JPRB, 0.12410E+01_JPRB ,&
&  0.13829E+01_JPRB, 0.16953E+01_JPRB, 0.17567E+01_JPRB, 0.21341E+01_JPRB, 0.21157E+01_JPRB, 0.19429E+01_JPRB, 0.19550E+01_JPRB ,&
&  0.21660E+01_JPRB, 0.29500E+01_JPRB, 0.26038E+01_JPRB, 0.26434E+01_JPRB, 0.32969E+01_JPRB, 0.27005E+01_JPRB, 0.29239E+01_JPRB ,&
&  0.34648E+01_JPRB, 0.32497E+01_JPRB, 0.27157E+01_JPRB, 0.20339E+01_JPRB, 0.25986E+01_JPRB, 0.26072E+01_JPRB, 0.12788E+01_JPRB ,&
&  0.10437E+01_JPRB, 0.96907E+00_JPRB, 0.91289E+00_JPRB, 0.87174E+00_JPRB, 0.86537E+00_JPRB, 0.12708E+01_JPRB, 0.22404E+01_JPRB ,&
&  0.26677E+01_JPRB, 0.37573E+01_JPRB, 0.43065E+01_JPRB, 0.44459E+01_JPRB, 0.39865E+01_JPRB, 0.35637E+01_JPRB, 0.30325E+01_JPRB ,&
&  0.27908E+01_JPRB, 0.27098E+01_JPRB, 0.24764E+01_JPRB, 0.22930E+01_JPRB, 0.21171E+01_JPRB, 0.19646E+01_JPRB, 0.18261E+01_JPRB ,&
&  0.16891E+01_JPRB, 0.15900E+01_JPRB, 0.15145E+01_JPRB, 0.14815E+01_JPRB, 0.14262E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.14390E+01_JPRB, 0.13877E+01_JPRB, 0.13139E+01_JPRB, 0.12582E+01_JPRB, 0.12010E+01_JPRB, 0.11393E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.97118E+00_JPRB, 0.91549E+00_JPRB, 0.89154E+00_JPRB, 0.87743E+00_JPRB, 0.86073E+00_JPRB, 0.84905E+00_JPRB ,&
&  0.83275E+00_JPRB, 0.82630E+00_JPRB, 0.85215E+00_JPRB, 0.10113E+01_JPRB, 0.11278E+01_JPRB, 0.12309E+01_JPRB, 0.13780E+01_JPRB ,&
&  0.18643E+01_JPRB, 0.20246E+01_JPRB, 0.19464E+01_JPRB, 0.19187E+01_JPRB, 0.18277E+01_JPRB, 0.17526E+01_JPRB, 0.16558E+01_JPRB ,&
&  0.15666E+01_JPRB, 0.14919E+01_JPRB, 0.13935E+01_JPRB, 0.12908E+01_JPRB, 0.12070E+01_JPRB, 0.11698E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.11569E+01_JPRB, 0.11215E+01_JPRB, 0.10630E+01_JPRB, 0.10550E+01_JPRB, 0.11342E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.12458E+01_JPRB, 0.13425E+01_JPRB, 0.14095E+01_JPRB, 0.13866E+01_JPRB, 0.11556E+01_JPRB, 0.12875E+01_JPRB, 0.13384E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.14738E+01_JPRB, 0.14587E+01_JPRB, 0.17662E+01_JPRB, 0.20154E+01_JPRB, 0.19087E+01_JPRB, 0.18774E+01_JPRB ,&
&  0.24953E+01_JPRB, 0.25481E+01_JPRB, 0.23841E+01_JPRB, 0.25411E+01_JPRB, 0.28735E+01_JPRB, 0.37043E+01_JPRB, 0.24973E+01_JPRB ,&
&  0.22845E+01_JPRB, 0.27465E+01_JPRB, 0.23955E+01_JPRB, 0.18826E+01_JPRB, 0.23505E+01_JPRB, 0.31605E+01_JPRB, 0.25745E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.98142E+00_JPRB, 0.10584E+01_JPRB, 0.11802E+01_JPRB, 0.10875E+01_JPRB, 0.16274E+01_JPRB, 0.39905E+01_JPRB ,&
&  0.38343E+01_JPRB, 0.47222E+01_JPRB, 0.47120E+01_JPRB, 0.44029E+01_JPRB, 0.38720E+01_JPRB, 0.33251E+01_JPRB, 0.30531E+01_JPRB ,&
&  0.28033E+01_JPRB, 0.26800E+01_JPRB, 0.25716E+01_JPRB, 0.24522E+01_JPRB, 0.22930E+01_JPRB, 0.20887E+01_JPRB, 0.19071E+01_JPRB ,&
&  0.17683E+01_JPRB, 0.16710E+01_JPRB, 0.15881E+01_JPRB, 0.15316E+01_JPRB, 0.14784E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.15229E+01_JPRB, 0.14377E+01_JPRB, 0.13509E+01_JPRB, 0.12942E+01_JPRB, 0.12266E+01_JPRB, 0.11624E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.10337E+01_JPRB, 0.96157E+00_JPRB, 0.92995E+00_JPRB, 0.90756E+00_JPRB, 0.88393E+00_JPRB, 0.87215E+00_JPRB ,&
&  0.86310E+00_JPRB, 0.86880E+00_JPRB, 0.89076E+00_JPRB, 0.10344E+01_JPRB, 0.11948E+01_JPRB, 0.11854E+01_JPRB, 0.14373E+01_JPRB ,&
&  0.21255E+01_JPRB, 0.22227E+01_JPRB, 0.20746E+01_JPRB, 0.19858E+01_JPRB, 0.18675E+01_JPRB, 0.18030E+01_JPRB, 0.16723E+01_JPRB ,&
&  0.15012E+01_JPRB, 0.13999E+01_JPRB, 0.12980E+01_JPRB, 0.12630E+01_JPRB, 0.12407E+01_JPRB, 0.12166E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.11713E+01_JPRB, 0.11362E+01_JPRB, 0.10894E+01_JPRB, 0.10375E+01_JPRB, 0.10774E+01_JPRB, 0.11817E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.13527E+01_JPRB, 0.14480E+01_JPRB, 0.14537E+01_JPRB, 0.13309E+01_JPRB, 0.12904E+01_JPRB, 0.12888E+01_JPRB, 0.13358E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.13958E+01_JPRB, 0.15075E+01_JPRB, 0.16454E+01_JPRB, 0.18246E+01_JPRB, 0.18560E+01_JPRB, 0.19010E+01_JPRB ,&
&  0.22721E+01_JPRB, 0.22984E+01_JPRB, 0.21633E+01_JPRB, 0.24330E+01_JPRB, 0.27181E+01_JPRB, 0.34168E+01_JPRB, 0.34120E+01_JPRB ,&
&  0.27792E+01_JPRB, 0.21646E+01_JPRB, 0.19629E+01_JPRB, 0.19967E+01_JPRB, 0.22734E+01_JPRB, 0.29499E+01_JPRB, 0.43084E+01_JPRB ,&
&  0.43671E+01_JPRB, 0.29025E+01_JPRB, 0.24069E+01_JPRB, 0.32181E+01_JPRB, 0.27720E+01_JPRB, 0.26163E+01_JPRB, 0.34812E+01_JPRB ,&
&  0.43379E+01_JPRB, 0.49608E+01_JPRB, 0.48417E+01_JPRB, 0.43752E+01_JPRB, 0.39405E+01_JPRB, 0.33838E+01_JPRB, 0.31383E+01_JPRB ,&
&  0.29423E+01_JPRB, 0.27629E+01_JPRB, 0.27086E+01_JPRB, 0.25976E+01_JPRB, 0.23862E+01_JPRB, 0.21302E+01_JPRB, 0.19072E+01_JPRB ,&
&  0.17756E+01_JPRB, 0.16967E+01_JPRB, 0.16409E+01_JPRB, 0.16049E+01_JPRB, 0.15712E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.15434E+01_JPRB, 0.14335E+01_JPRB, 0.13545E+01_JPRB, 0.12911E+01_JPRB, 0.12171E+01_JPRB, 0.11761E+01_JPRB, 0.12130E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.10847E+01_JPRB, 0.10174E+01_JPRB, 0.96202E+00_JPRB, 0.92394E+00_JPRB, 0.90521E+00_JPRB, 0.89894E+00_JPRB ,&
&  0.88990E+00_JPRB, 0.89877E+00_JPRB, 0.92240E+00_JPRB, 0.10783E+01_JPRB, 0.13613E+01_JPRB, 0.18013E+01_JPRB, 0.14512E+01_JPRB ,&
&  0.21504E+01_JPRB, 0.25412E+01_JPRB, 0.22969E+01_JPRB, 0.20517E+01_JPRB, 0.18517E+01_JPRB, 0.18661E+01_JPRB, 0.16773E+01_JPRB ,&
&  0.14370E+01_JPRB, 0.13069E+01_JPRB, 0.12316E+01_JPRB, 0.12001E+01_JPRB, 0.11978E+01_JPRB, 0.11814E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10802E+01_JPRB, 0.10532E+01_JPRB, 0.10599E+01_JPRB, 0.11288E+01_JPRB, 0.12366E+01_JPRB, 0.13579E+01_JPRB ,&
&  0.14903E+01_JPRB, 0.15398E+01_JPRB, 0.15004E+01_JPRB, 0.13545E+01_JPRB, 0.13338E+01_JPRB, 0.13163E+01_JPRB, 0.13134E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.12256E+01_JPRB, 0.14190E+01_JPRB, 0.16326E+01_JPRB, 0.17491E+01_JPRB, 0.17154E+01_JPRB, 0.17466E+01_JPRB ,&
&  0.21612E+01_JPRB, 0.25173E+01_JPRB, 0.28048E+01_JPRB, 0.27376E+01_JPRB, 0.30522E+01_JPRB, 0.34438E+01_JPRB, 0.37551E+01_JPRB ,&
&  0.35917E+01_JPRB, 0.27100E+01_JPRB, 0.23272E+01_JPRB, 0.22244E+01_JPRB, 0.22437E+01_JPRB, 0.28426E+01_JPRB, 0.39301E+01_JPRB ,&
&  0.51477E+01_JPRB, 0.69138E+01_JPRB, 0.69141E+01_JPRB, 0.54502E+01_JPRB, 0.47233E+01_JPRB, 0.42129E+01_JPRB, 0.38890E+01_JPRB ,&
&  0.49133E+01_JPRB, 0.50758E+01_JPRB, 0.48361E+01_JPRB, 0.53315E+01_JPRB, 0.41647E+01_JPRB, 0.35098E+01_JPRB, 0.31789E+01_JPRB ,&
&  0.30455E+01_JPRB, 0.28760E+01_JPRB, 0.27137E+01_JPRB, 0.25317E+01_JPRB, 0.22536E+01_JPRB, 0.20046E+01_JPRB, 0.18455E+01_JPRB ,&
&  0.17547E+01_JPRB, 0.17038E+01_JPRB, 0.16648E+01_JPRB, 0.16454E+01_JPRB, 0.16192E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.14923E+01_JPRB, 0.13911E+01_JPRB, 0.12872E+01_JPRB, 0.12377E+01_JPRB, 0.11931E+01_JPRB, 0.11871E+01_JPRB, 0.12161E+01_JPRB ,&
&  0.12941E+01_JPRB, 0.11303E+01_JPRB, 0.10324E+01_JPRB, 0.95976E+00_JPRB, 0.92788E+00_JPRB, 0.92372E+00_JPRB, 0.92190E+00_JPRB ,&
&  0.92551E+00_JPRB, 0.94794E+00_JPRB, 0.10210E+01_JPRB, 0.11436E+01_JPRB, 0.14295E+01_JPRB, 0.17899E+01_JPRB, 0.19042E+01_JPRB ,&
&  0.23543E+01_JPRB, 0.32986E+01_JPRB, 0.25199E+01_JPRB, 0.20800E+01_JPRB, 0.19548E+01_JPRB, 0.21861E+01_JPRB, 0.18387E+01_JPRB ,&
&  0.14488E+01_JPRB, 0.13423E+01_JPRB, 0.13059E+01_JPRB, 0.12000E+01_JPRB, 0.11642E+01_JPRB, 0.11298E+01_JPRB, 0.11141E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.10800E+01_JPRB, 0.10656E+01_JPRB, 0.11085E+01_JPRB, 0.12009E+01_JPRB, 0.13207E+01_JPRB, 0.14957E+01_JPRB ,&
&  0.16512E+01_JPRB, 0.16021E+01_JPRB, 0.14727E+01_JPRB, 0.13992E+01_JPRB, 0.13677E+01_JPRB, 0.13691E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.13382E+01_JPRB, 0.12932E+01_JPRB, 0.14087E+01_JPRB, 0.14731E+01_JPRB, 0.15132E+01_JPRB, 0.16528E+01_JPRB, 0.17176E+01_JPRB ,&
&  0.21770E+01_JPRB, 0.23694E+01_JPRB, 0.30833E+01_JPRB, 0.28591E+01_JPRB, 0.32176E+01_JPRB, 0.35258E+01_JPRB, 0.33251E+01_JPRB ,&
&  0.27666E+01_JPRB, 0.24780E+01_JPRB, 0.21725E+01_JPRB, 0.21814E+01_JPRB, 0.25399E+01_JPRB, 0.33708E+01_JPRB, 0.42184E+01_JPRB ,&
&  0.51692E+01_JPRB, 0.61161E+01_JPRB, 0.68502E+01_JPRB, 0.55094E+01_JPRB, 0.49360E+01_JPRB, 0.46045E+01_JPRB, 0.46053E+01_JPRB ,&
&  0.52465E+01_JPRB, 0.53364E+01_JPRB, 0.53919E+01_JPRB, 0.51161E+01_JPRB, 0.38871E+01_JPRB, 0.32972E+01_JPRB, 0.30114E+01_JPRB ,&
&  0.28457E+01_JPRB, 0.26738E+01_JPRB, 0.23990E+01_JPRB, 0.21754E+01_JPRB, 0.20098E+01_JPRB, 0.18836E+01_JPRB, 0.17650E+01_JPRB ,&
&  0.16944E+01_JPRB, 0.16393E+01_JPRB, 0.16139E+01_JPRB, 0.16075E+01_JPRB, 0.15711E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.13812E+01_JPRB, 0.13056E+01_JPRB, 0.12541E+01_JPRB, 0.12354E+01_JPRB, 0.12080E+01_JPRB, 0.12312E+01_JPRB, 0.12819E+01_JPRB ,&
&  0.14139E+01_JPRB, 0.11024E+01_JPRB, 0.99779E+00_JPRB, 0.95845E+00_JPRB, 0.94509E+00_JPRB, 0.93742E+00_JPRB, 0.94572E+00_JPRB ,&
&  0.96437E+00_JPRB, 0.10280E+01_JPRB, 0.11441E+01_JPRB, 0.12983E+01_JPRB, 0.16093E+01_JPRB, 0.20713E+01_JPRB, 0.25316E+01_JPRB ,&
&  0.29730E+01_JPRB, 0.28930E+01_JPRB, 0.24729E+01_JPRB, 0.20508E+01_JPRB, 0.19166E+01_JPRB, 0.20785E+01_JPRB, 0.20131E+01_JPRB ,&
&  0.17140E+01_JPRB, 0.14642E+01_JPRB, 0.13509E+01_JPRB, 0.12266E+01_JPRB, 0.11505E+01_JPRB, 0.11368E+01_JPRB, 0.11176E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.10659E+01_JPRB, 0.10755E+01_JPRB, 0.11610E+01_JPRB, 0.12787E+01_JPRB, 0.14544E+01_JPRB, 0.17142E+01_JPRB ,&
&  0.17570E+01_JPRB, 0.16391E+01_JPRB, 0.15311E+01_JPRB, 0.14194E+01_JPRB, 0.14074E+01_JPRB, 0.14115E+01_JPRB, 0.14118E+01_JPRB ,&
&  0.13990E+01_JPRB, 0.14358E+01_JPRB, 0.14806E+01_JPRB, 0.14967E+01_JPRB, 0.14779E+01_JPRB, 0.17029E+01_JPRB, 0.18473E+01_JPRB ,&
&  0.21940E+01_JPRB, 0.25713E+01_JPRB, 0.33553E+01_JPRB, 0.27537E+01_JPRB, 0.33171E+01_JPRB, 0.33945E+01_JPRB, 0.30627E+01_JPRB ,&
&  0.25734E+01_JPRB, 0.23223E+01_JPRB, 0.22161E+01_JPRB, 0.23693E+01_JPRB, 0.30591E+01_JPRB, 0.39483E+01_JPRB, 0.55741E+01_JPRB ,&
&  0.55916E+01_JPRB, 0.62673E+01_JPRB, 0.58947E+01_JPRB, 0.57665E+01_JPRB, 0.50594E+01_JPRB, 0.47897E+01_JPRB, 0.46904E+01_JPRB ,&
&  0.50396E+01_JPRB, 0.49628E+01_JPRB, 0.45530E+01_JPRB, 0.40544E+01_JPRB, 0.32447E+01_JPRB, 0.27779E+01_JPRB, 0.25588E+01_JPRB ,&
&  0.23368E+01_JPRB, 0.21140E+01_JPRB, 0.19447E+01_JPRB, 0.18747E+01_JPRB, 0.17938E+01_JPRB, 0.17080E+01_JPRB, 0.16421E+01_JPRB ,&
&  0.16036E+01_JPRB, 0.15694E+01_JPRB, 0.15445E+01_JPRB, 0.15144E+01_JPRB, 0.14650E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.12728E+01_JPRB, 0.12264E+01_JPRB, 0.12023E+01_JPRB, 0.11653E+01_JPRB, 0.11301E+01_JPRB, 0.11224E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.10024E+01_JPRB, 0.98326E+00_JPRB, 0.97903E+00_JPRB, 0.96547E+00_JPRB, 0.95081E+00_JPRB, 0.96039E+00_JPRB ,&
&  0.10087E+01_JPRB, 0.11054E+01_JPRB, 0.12439E+01_JPRB, 0.14738E+01_JPRB, 0.17756E+01_JPRB, 0.22145E+01_JPRB, 0.26862E+01_JPRB ,&
&  0.27457E+01_JPRB, 0.19568E+01_JPRB, 0.21459E+01_JPRB, 0.20661E+01_JPRB, 0.18724E+01_JPRB, 0.19443E+01_JPRB, 0.19167E+01_JPRB ,&
&  0.15690E+01_JPRB, 0.14511E+01_JPRB, 0.14203E+01_JPRB, 0.13428E+01_JPRB, 0.11601E+01_JPRB, 0.11379E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.10865E+01_JPRB, 0.11325E+01_JPRB, 0.12758E+01_JPRB, 0.14625E+01_JPRB, 0.17422E+01_JPRB, 0.18910E+01_JPRB ,&
&  0.18067E+01_JPRB, 0.17382E+01_JPRB, 0.16948E+01_JPRB, 0.15708E+01_JPRB, 0.15421E+01_JPRB, 0.15530E+01_JPRB, 0.15549E+01_JPRB ,&
&  0.15494E+01_JPRB, 0.15471E+01_JPRB, 0.16155E+01_JPRB, 0.17459E+01_JPRB, 0.18717E+01_JPRB, 0.18250E+01_JPRB, 0.20911E+01_JPRB ,&
&  0.23779E+01_JPRB, 0.29059E+01_JPRB, 0.32673E+01_JPRB, 0.37929E+01_JPRB, 0.27602E+01_JPRB, 0.30094E+01_JPRB, 0.25011E+01_JPRB ,&
&  0.25147E+01_JPRB, 0.24351E+01_JPRB, 0.26702E+01_JPRB, 0.30975E+01_JPRB, 0.40929E+01_JPRB, 0.50121E+01_JPRB, 0.64664E+01_JPRB ,&
&  0.56931E+01_JPRB, 0.54230E+01_JPRB, 0.51591E+01_JPRB, 0.55070E+01_JPRB, 0.51276E+01_JPRB, 0.51777E+01_JPRB, 0.46583E+01_JPRB ,&
&  0.45524E+01_JPRB, 0.40487E+01_JPRB, 0.35499E+01_JPRB, 0.32298E+01_JPRB, 0.26151E+01_JPRB, 0.21273E+01_JPRB, 0.18658E+01_JPRB ,&
&  0.17168E+01_JPRB, 0.16747E+01_JPRB, 0.16518E+01_JPRB, 0.16338E+01_JPRB, 0.15938E+01_JPRB, 0.15537E+01_JPRB, 0.15243E+01_JPRB ,&
&  0.14980E+01_JPRB, 0.14633E+01_JPRB, 0.14329E+01_JPRB, 0.13974E+01_JPRB, 0.13441E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.11919E+01_JPRB, 0.11535E+01_JPRB, 0.11325E+01_JPRB, 0.11013E+01_JPRB, 0.10776E+01_JPRB, 0.10650E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.10116E+01_JPRB, 0.10060E+01_JPRB, 0.99095E+00_JPRB, 0.96331E+00_JPRB, 0.95836E+00_JPRB, 0.99827E+00_JPRB ,&
&  0.10941E+01_JPRB, 0.12442E+01_JPRB, 0.14230E+01_JPRB, 0.16168E+01_JPRB, 0.19061E+01_JPRB, 0.21466E+01_JPRB, 0.22540E+01_JPRB ,&
&  0.24150E+01_JPRB, 0.24604E+01_JPRB, 0.21046E+01_JPRB, 0.15361E+01_JPRB, 0.14244E+01_JPRB, 0.17031E+01_JPRB, 0.15456E+01_JPRB ,&
&  0.13966E+01_JPRB, 0.13312E+01_JPRB, 0.12939E+01_JPRB, 0.12514E+01_JPRB, 0.11855E+01_JPRB, 0.11575E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.11960E+01_JPRB, 0.13323E+01_JPRB, 0.15903E+01_JPRB, 0.18454E+01_JPRB, 0.20265E+01_JPRB, 0.20223E+01_JPRB ,&
&  0.19999E+01_JPRB, 0.20329E+01_JPRB, 0.20240E+01_JPRB, 0.18989E+01_JPRB, 0.18067E+01_JPRB, 0.18236E+01_JPRB, 0.18444E+01_JPRB ,&
&  0.18593E+01_JPRB, 0.18706E+01_JPRB, 0.18912E+01_JPRB, 0.20125E+01_JPRB, 0.21128E+01_JPRB, 0.20175E+01_JPRB, 0.21682E+01_JPRB ,&
&  0.26468E+01_JPRB, 0.33468E+01_JPRB, 0.33667E+01_JPRB, 0.40340E+01_JPRB, 0.23779E+01_JPRB, 0.26615E+01_JPRB, 0.27638E+01_JPRB ,&
&  0.25742E+01_JPRB, 0.29134E+01_JPRB, 0.33155E+01_JPRB, 0.39110E+01_JPRB, 0.48581E+01_JPRB, 0.54171E+01_JPRB, 0.59642E+01_JPRB ,&
&  0.50401E+01_JPRB, 0.45063E+01_JPRB, 0.44504E+01_JPRB, 0.46947E+01_JPRB, 0.47583E+01_JPRB, 0.46686E+01_JPRB, 0.38714E+01_JPRB ,&
&  0.32316E+01_JPRB, 0.32490E+01_JPRB, 0.28445E+01_JPRB, 0.27061E+01_JPRB, 0.20093E+01_JPRB, 0.15924E+01_JPRB, 0.15411E+01_JPRB ,&
&  0.14994E+01_JPRB, 0.14633E+01_JPRB, 0.14570E+01_JPRB, 0.14431E+01_JPRB, 0.14263E+01_JPRB, 0.14104E+01_JPRB, 0.13858E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.13489E+01_JPRB, 0.13265E+01_JPRB, 0.12932E+01_JPRB, 0.12466E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.11283E+01_JPRB, 0.11011E+01_JPRB, 0.10914E+01_JPRB, 0.10533E+01_JPRB, 0.10182E+01_JPRB, 0.10092E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.99701E+00_JPRB, 0.98027E+00_JPRB, 0.96674E+00_JPRB, 0.97588E+00_JPRB, 0.10217E+01_JPRB, 0.11044E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.13701E+01_JPRB, 0.15578E+01_JPRB, 0.17437E+01_JPRB, 0.19829E+01_JPRB, 0.20875E+01_JPRB, 0.20762E+01_JPRB ,&
&  0.21460E+01_JPRB, 0.22377E+01_JPRB, 0.22072E+01_JPRB, 0.21774E+01_JPRB, 0.18125E+01_JPRB, 0.19547E+01_JPRB, 0.20115E+01_JPRB ,&
&  0.19187E+01_JPRB, 0.15700E+01_JPRB, 0.13382E+01_JPRB, 0.12943E+01_JPRB, 0.12516E+01_JPRB, 0.12512E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.13752E+01_JPRB, 0.15241E+01_JPRB, 0.17592E+01_JPRB, 0.20159E+01_JPRB, 0.21880E+01_JPRB, 0.22676E+01_JPRB, 0.22961E+01_JPRB ,&
&  0.23410E+01_JPRB, 0.24048E+01_JPRB, 0.24693E+01_JPRB, 0.23920E+01_JPRB, 0.23634E+01_JPRB, 0.23580E+01_JPRB, 0.24337E+01_JPRB ,&
&  0.25106E+01_JPRB, 0.24132E+01_JPRB, 0.24240E+01_JPRB, 0.25905E+01_JPRB, 0.26271E+01_JPRB, 0.26068E+01_JPRB, 0.27205E+01_JPRB ,&
&  0.31951E+01_JPRB, 0.39803E+01_JPRB, 0.31679E+01_JPRB, 0.33095E+01_JPRB, 0.38109E+01_JPRB, 0.31110E+01_JPRB, 0.27124E+01_JPRB ,&
&  0.28697E+01_JPRB, 0.32224E+01_JPRB, 0.36083E+01_JPRB, 0.41535E+01_JPRB, 0.49930E+01_JPRB, 0.60429E+01_JPRB, 0.50189E+01_JPRB ,&
&  0.42414E+01_JPRB, 0.37393E+01_JPRB, 0.37589E+01_JPRB, 0.38288E+01_JPRB, 0.37972E+01_JPRB, 0.36491E+01_JPRB, 0.30726E+01_JPRB ,&
&  0.26819E+01_JPRB, 0.26952E+01_JPRB, 0.24769E+01_JPRB, 0.22412E+01_JPRB, 0.20624E+01_JPRB, 0.14850E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.13259E+01_JPRB, 0.13238E+01_JPRB, 0.13323E+01_JPRB, 0.13301E+01_JPRB, 0.13180E+01_JPRB, 0.12944E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.12630E+01_JPRB, 0.12459E+01_JPRB, 0.12292E+01_JPRB, 0.12078E+01_JPRB, 0.11668E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.10804E+01_JPRB, 0.10636E+01_JPRB, 0.10496E+01_JPRB, 0.10184E+01_JPRB, 0.98318E+00_JPRB, 0.97539E+00_JPRB, 0.96959E+00_JPRB ,&
&  0.94700E+00_JPRB, 0.92242E+00_JPRB, 0.89419E+00_JPRB, 0.90718E+00_JPRB, 0.94328E+00_JPRB, 0.10146E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.13389E+01_JPRB, 0.15366E+01_JPRB, 0.16993E+01_JPRB, 0.19283E+01_JPRB, 0.20540E+01_JPRB, 0.21320E+01_JPRB ,&
&  0.22286E+01_JPRB, 0.23400E+01_JPRB, 0.24318E+01_JPRB, 0.24938E+01_JPRB, 0.24985E+01_JPRB, 0.25562E+01_JPRB, 0.27033E+01_JPRB ,&
&  0.22765E+01_JPRB, 0.16854E+01_JPRB, 0.15019E+01_JPRB, 0.14132E+01_JPRB, 0.14155E+01_JPRB, 0.14652E+01_JPRB, 0.15713E+01_JPRB ,&
&  0.17364E+01_JPRB, 0.19266E+01_JPRB, 0.21656E+01_JPRB, 0.23532E+01_JPRB, 0.25042E+01_JPRB, 0.26172E+01_JPRB, 0.26987E+01_JPRB ,&
&  0.28312E+01_JPRB, 0.29735E+01_JPRB, 0.31737E+01_JPRB, 0.30612E+01_JPRB, 0.33633E+01_JPRB, 0.33534E+01_JPRB, 0.33802E+01_JPRB ,&
&  0.32906E+01_JPRB, 0.30718E+01_JPRB, 0.31742E+01_JPRB, 0.33432E+01_JPRB, 0.35132E+01_JPRB, 0.35466E+01_JPRB, 0.35170E+01_JPRB ,&
&  0.39760E+01_JPRB, 0.41184E+01_JPRB, 0.30725E+01_JPRB, 0.26825E+01_JPRB, 0.27543E+01_JPRB, 0.23318E+01_JPRB, 0.27413E+01_JPRB ,&
&  0.28567E+01_JPRB, 0.31411E+01_JPRB, 0.34363E+01_JPRB, 0.39144E+01_JPRB, 0.44692E+01_JPRB, 0.48586E+01_JPRB, 0.42189E+01_JPRB ,&
&  0.33145E+01_JPRB, 0.29563E+01_JPRB, 0.29193E+01_JPRB, 0.28625E+01_JPRB, 0.28274E+01_JPRB, 0.27748E+01_JPRB, 0.26168E+01_JPRB ,&
&  0.24343E+01_JPRB, 0.22224E+01_JPRB, 0.20641E+01_JPRB, 0.16780E+01_JPRB, 0.14500E+01_JPRB, 0.13097E+01_JPRB, 0.12194E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.12369E+01_JPRB, 0.12504E+01_JPRB, 0.12503E+01_JPRB, 0.12438E+01_JPRB, 0.12185E+01_JPRB, 0.12032E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.11785E+01_JPRB, 0.11603E+01_JPRB, 0.11443E+01_JPRB, 0.11100E+01_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.10056E+01_JPRB, 0.97977E+00_JPRB, 0.96211E+00_JPRB, 0.94301E+00_JPRB, 0.91034E+00_JPRB, 0.88703E+00_JPRB, 0.87668E+00_JPRB ,&
&  0.83593E+00_JPRB, 0.83364E+00_JPRB, 0.81575E+00_JPRB, 0.82722E+00_JPRB, 0.86594E+00_JPRB, 0.90457E+00_JPRB, 0.96153E+00_JPRB ,&
&  0.10097E+01_JPRB, 0.10786E+01_JPRB, 0.11942E+01_JPRB, 0.14373E+01_JPRB, 0.17156E+01_JPRB, 0.19352E+01_JPRB, 0.21955E+01_JPRB ,&
&  0.23206E+01_JPRB, 0.24343E+01_JPRB, 0.26374E+01_JPRB, 0.29140E+01_JPRB, 0.31230E+01_JPRB, 0.33152E+01_JPRB, 0.32965E+01_JPRB ,&
&  0.24866E+01_JPRB, 0.16143E+01_JPRB, 0.15908E+01_JPRB, 0.14864E+01_JPRB, 0.15635E+01_JPRB, 0.16728E+01_JPRB, 0.17952E+01_JPRB ,&
&  0.19836E+01_JPRB, 0.21810E+01_JPRB, 0.23862E+01_JPRB, 0.25563E+01_JPRB, 0.27450E+01_JPRB, 0.29446E+01_JPRB, 0.30659E+01_JPRB ,&
&  0.31888E+01_JPRB, 0.33115E+01_JPRB, 0.34077E+01_JPRB, 0.33439E+01_JPRB, 0.35423E+01_JPRB, 0.37934E+01_JPRB, 0.40046E+01_JPRB ,&
&  0.42082E+01_JPRB, 0.42398E+01_JPRB, 0.35788E+01_JPRB, 0.37259E+01_JPRB, 0.42558E+01_JPRB, 0.39838E+01_JPRB, 0.40996E+01_JPRB ,&
&  0.47819E+01_JPRB, 0.49263E+01_JPRB, 0.28124E+01_JPRB, 0.24125E+01_JPRB, 0.24602E+01_JPRB, 0.24348E+01_JPRB, 0.25307E+01_JPRB ,&
&  0.26341E+01_JPRB, 0.28878E+01_JPRB, 0.31780E+01_JPRB, 0.34179E+01_JPRB, 0.35310E+01_JPRB, 0.33263E+01_JPRB, 0.30853E+01_JPRB ,&
&  0.25438E+01_JPRB, 0.22884E+01_JPRB, 0.22845E+01_JPRB, 0.23120E+01_JPRB, 0.23930E+01_JPRB, 0.24823E+01_JPRB, 0.22328E+01_JPRB ,&
&  0.21542E+01_JPRB, 0.21332E+01_JPRB, 0.23354E+01_JPRB, 0.19030E+01_JPRB, 0.13505E+01_JPRB, 0.11808E+01_JPRB, 0.11002E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.11872E+01_JPRB, 0.12054E+01_JPRB, 0.12174E+01_JPRB, 0.12144E+01_JPRB, 0.12004E+01_JPRB, 0.11823E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.11315E+01_JPRB, 0.11013E+01_JPRB, 0.10692E+01_JPRB, 0.10389E+01_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.87726E+00_JPRB, 0.85265E+00_JPRB, 0.84436E+00_JPRB, 0.79904E+00_JPRB, 0.81236E+00_JPRB, 0.77938E+00_JPRB, 0.81512E+00_JPRB ,&
&  0.78936E+00_JPRB, 0.78764E+00_JPRB, 0.78948E+00_JPRB, 0.80721E+00_JPRB, 0.82474E+00_JPRB, 0.84671E+00_JPRB, 0.86583E+00_JPRB ,&
&  0.89092E+00_JPRB, 0.94427E+00_JPRB, 0.10230E+01_JPRB, 0.11184E+01_JPRB, 0.12437E+01_JPRB, 0.13735E+01_JPRB, 0.14930E+01_JPRB ,&
&  0.16524E+01_JPRB, 0.17963E+01_JPRB, 0.18954E+01_JPRB, 0.21193E+01_JPRB, 0.26842E+01_JPRB, 0.33792E+01_JPRB, 0.34951E+01_JPRB ,&
&  0.22689E+01_JPRB, 0.16847E+01_JPRB, 0.14981E+01_JPRB, 0.14064E+01_JPRB, 0.15169E+01_JPRB, 0.17481E+01_JPRB, 0.18654E+01_JPRB ,&
&  0.19942E+01_JPRB, 0.21502E+01_JPRB, 0.23239E+01_JPRB, 0.25142E+01_JPRB, 0.27412E+01_JPRB, 0.29576E+01_JPRB, 0.30832E+01_JPRB ,&
&  0.31185E+01_JPRB, 0.31945E+01_JPRB, 0.32227E+01_JPRB, 0.34529E+01_JPRB, 0.38986E+01_JPRB, 0.44229E+01_JPRB, 0.48145E+01_JPRB ,&
&  0.50232E+01_JPRB, 0.49461E+01_JPRB, 0.42179E+01_JPRB, 0.43055E+01_JPRB, 0.45525E+01_JPRB, 0.44710E+01_JPRB, 0.43167E+01_JPRB ,&
&  0.47698E+01_JPRB, 0.45053E+01_JPRB, 0.28982E+01_JPRB, 0.26143E+01_JPRB, 0.24809E+01_JPRB, 0.23789E+01_JPRB, 0.22395E+01_JPRB ,&
&  0.24155E+01_JPRB, 0.26339E+01_JPRB, 0.27795E+01_JPRB, 0.27902E+01_JPRB, 0.26747E+01_JPRB, 0.24611E+01_JPRB, 0.24002E+01_JPRB ,&
&  0.21706E+01_JPRB, 0.20785E+01_JPRB, 0.20919E+01_JPRB, 0.21461E+01_JPRB, 0.24784E+01_JPRB, 0.22421E+01_JPRB, 0.21794E+01_JPRB ,&
&  0.18970E+01_JPRB, 0.18784E+01_JPRB, 0.15792E+01_JPRB, 0.18274E+01_JPRB, 0.12394E+01_JPRB, 0.99729E+00_JPRB, 0.10077E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10743E+01_JPRB, 0.10753E+01_JPRB, 0.10919E+01_JPRB, 0.10981E+01_JPRB, 0.10842E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10628E+01_JPRB, 0.10378E+01_JPRB, 0.99523E+00_JPRB, 0.95995E+00_JPRB, 0.92243E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.83863E+00_JPRB, 0.83710E+00_JPRB, 0.81325E+00_JPRB, 0.77259E+00_JPRB, 0.74663E+00_JPRB, 0.72718E+00_JPRB, 0.73638E+00_JPRB ,&
&  0.74724E+00_JPRB, 0.77256E+00_JPRB, 0.79513E+00_JPRB, 0.81777E+00_JPRB, 0.82852E+00_JPRB, 0.83624E+00_JPRB, 0.85701E+00_JPRB ,&
&  0.88471E+00_JPRB, 0.91039E+00_JPRB, 0.94518E+00_JPRB, 0.99202E+00_JPRB, 0.10402E+01_JPRB, 0.11022E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.11893E+01_JPRB, 0.12350E+01_JPRB, 0.13252E+01_JPRB, 0.16431E+01_JPRB, 0.19926E+01_JPRB, 0.24511E+01_JPRB ,&
&  0.19072E+01_JPRB, 0.18414E+01_JPRB, 0.16239E+01_JPRB, 0.15174E+01_JPRB, 0.15779E+01_JPRB, 0.15774E+01_JPRB, 0.16387E+01_JPRB ,&
&  0.17304E+01_JPRB, 0.17955E+01_JPRB, 0.18548E+01_JPRB, 0.19223E+01_JPRB, 0.19918E+01_JPRB, 0.21247E+01_JPRB, 0.20900E+01_JPRB ,&
&  0.20858E+01_JPRB, 0.21869E+01_JPRB, 0.23162E+01_JPRB, 0.26837E+01_JPRB, 0.32260E+01_JPRB, 0.37509E+01_JPRB, 0.38110E+01_JPRB ,&
&  0.38721E+01_JPRB, 0.41665E+01_JPRB, 0.40937E+01_JPRB, 0.41551E+01_JPRB, 0.41070E+01_JPRB, 0.39651E+01_JPRB, 0.39787E+01_JPRB ,&
&  0.38007E+01_JPRB, 0.34254E+01_JPRB, 0.30278E+01_JPRB, 0.26129E+01_JPRB, 0.23407E+01_JPRB, 0.20984E+01_JPRB, 0.20517E+01_JPRB ,&
&  0.21202E+01_JPRB, 0.22494E+01_JPRB, 0.23084E+01_JPRB, 0.22038E+01_JPRB, 0.20968E+01_JPRB, 0.19568E+01_JPRB, 0.19681E+01_JPRB ,&
&  0.19364E+01_JPRB, 0.17963E+01_JPRB, 0.18025E+01_JPRB, 0.19699E+01_JPRB, 0.21939E+01_JPRB, 0.36030E+01_JPRB, 0.31456E+01_JPRB ,&
&  0.18018E+01_JPRB, 0.14675E+01_JPRB, 0.98686E+00_JPRB, 0.14113E+01_JPRB, 0.95407E+00_JPRB, 0.87080E+00_JPRB, 0.89639E+00_JPRB ,&
&  0.91350E+00_JPRB, 0.93074E+00_JPRB, 0.94199E+00_JPRB, 0.94555E+00_JPRB, 0.94714E+00_JPRB, 0.94931E+00_JPRB, 0.93518E+00_JPRB ,&
&  0.92345E+00_JPRB, 0.90525E+00_JPRB, 0.90063E+00_JPRB, 0.86488E+00_JPRB, 0.84811E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.79815E+00_JPRB, 0.79717E+00_JPRB, 0.76431E+00_JPRB, 0.71602E+00_JPRB, 0.69482E+00_JPRB, 0.69947E+00_JPRB, 0.71731E+00_JPRB ,&
&  0.73581E+00_JPRB, 0.75975E+00_JPRB, 0.78783E+00_JPRB, 0.81566E+00_JPRB, 0.84115E+00_JPRB, 0.86901E+00_JPRB, 0.90243E+00_JPRB ,&
&  0.93433E+00_JPRB, 0.97307E+00_JPRB, 0.10135E+01_JPRB, 0.10563E+01_JPRB, 0.10962E+01_JPRB, 0.11633E+01_JPRB, 0.12161E+01_JPRB ,&
&  0.12921E+01_JPRB, 0.13678E+01_JPRB, 0.14639E+01_JPRB, 0.15671E+01_JPRB, 0.17018E+01_JPRB, 0.18258E+01_JPRB, 0.17123E+01_JPRB ,&
&  0.20452E+01_JPRB, 0.18676E+01_JPRB, 0.16872E+01_JPRB, 0.15750E+01_JPRB, 0.14961E+01_JPRB, 0.14392E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.13528E+01_JPRB, 0.13230E+01_JPRB, 0.12824E+01_JPRB, 0.12492E+01_JPRB, 0.12368E+01_JPRB, 0.13349E+01_JPRB, 0.13638E+01_JPRB ,&
&  0.14971E+01_JPRB, 0.17266E+01_JPRB, 0.19370E+01_JPRB, 0.22856E+01_JPRB, 0.25776E+01_JPRB, 0.27953E+01_JPRB, 0.26828E+01_JPRB ,&
&  0.27297E+01_JPRB, 0.30228E+01_JPRB, 0.30109E+01_JPRB, 0.31234E+01_JPRB, 0.31073E+01_JPRB, 0.29587E+01_JPRB, 0.28672E+01_JPRB ,&
&  0.25778E+01_JPRB, 0.24825E+01_JPRB, 0.25002E+01_JPRB, 0.26775E+01_JPRB, 0.22658E+01_JPRB, 0.19852E+01_JPRB, 0.19751E+01_JPRB ,&
&  0.19128E+01_JPRB, 0.19509E+01_JPRB, 0.19542E+01_JPRB, 0.18322E+01_JPRB, 0.17312E+01_JPRB, 0.16488E+01_JPRB, 0.16771E+01_JPRB ,&
&  0.16890E+01_JPRB, 0.15528E+01_JPRB, 0.16167E+01_JPRB, 0.18829E+01_JPRB, 0.22860E+01_JPRB, 0.24962E+01_JPRB, 0.30131E+01_JPRB ,&
&  0.17418E+01_JPRB, 0.10099E+01_JPRB, 0.10011E+01_JPRB, 0.10306E+01_JPRB, 0.83129E+00_JPRB, 0.73707E+00_JPRB, 0.78584E+00_JPRB ,&
&  0.82257E+00_JPRB, 0.84675E+00_JPRB, 0.83261E+00_JPRB, 0.85828E+00_JPRB, 0.91000E+00_JPRB, 0.90636E+00_JPRB, 0.85601E+00_JPRB ,&
&  0.86624E+00_JPRB, 0.83027E+00_JPRB, 0.81373E+00_JPRB, 0.80968E+00_JPRB, 0.80637E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.74912E+00_JPRB, 0.73693E+00_JPRB, 0.70021E+00_JPRB, 0.65669E+00_JPRB, 0.64035E+00_JPRB, 0.65669E+00_JPRB, 0.68398E+00_JPRB ,&
&  0.70882E+00_JPRB, 0.73793E+00_JPRB, 0.76729E+00_JPRB, 0.79998E+00_JPRB, 0.83455E+00_JPRB, 0.86810E+00_JPRB, 0.89448E+00_JPRB ,&
&  0.91539E+00_JPRB, 0.94322E+00_JPRB, 0.96997E+00_JPRB, 0.99609E+00_JPRB, 0.10159E+01_JPRB, 0.10283E+01_JPRB, 0.10457E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.11724E+01_JPRB, 0.12599E+01_JPRB, 0.13565E+01_JPRB, 0.14456E+01_JPRB, 0.15504E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.18331E+01_JPRB, 0.18048E+01_JPRB, 0.16869E+01_JPRB, 0.15684E+01_JPRB, 0.14523E+01_JPRB, 0.13633E+01_JPRB, 0.12814E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.12715E+01_JPRB, 0.12288E+01_JPRB, 0.12417E+01_JPRB, 0.11218E+01_JPRB, 0.12056E+01_JPRB, 0.13267E+01_JPRB ,&
&  0.15471E+01_JPRB, 0.17980E+01_JPRB, 0.20190E+01_JPRB, 0.22230E+01_JPRB, 0.23185E+01_JPRB, 0.23957E+01_JPRB, 0.22787E+01_JPRB ,&
&  0.22940E+01_JPRB, 0.23090E+01_JPRB, 0.21841E+01_JPRB, 0.22997E+01_JPRB, 0.24064E+01_JPRB, 0.20767E+01_JPRB, 0.18443E+01_JPRB ,&
&  0.16864E+01_JPRB, 0.18983E+01_JPRB, 0.24378E+01_JPRB, 0.26702E+01_JPRB, 0.22478E+01_JPRB, 0.20652E+01_JPRB, 0.19295E+01_JPRB ,&
&  0.17720E+01_JPRB, 0.17463E+01_JPRB, 0.17008E+01_JPRB, 0.16031E+01_JPRB, 0.14574E+01_JPRB, 0.13758E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.14319E+01_JPRB, 0.13255E+01_JPRB, 0.13123E+01_JPRB, 0.15564E+01_JPRB, 0.17541E+01_JPRB, 0.16939E+01_JPRB, 0.28685E+01_JPRB ,&
&  0.17239E+01_JPRB, 0.12285E+01_JPRB, 0.16820E+01_JPRB, 0.67654E+00_JPRB, 0.71148E+00_JPRB, 0.64431E+00_JPRB, 0.65626E+00_JPRB ,&
&  0.55881E+00_JPRB, 0.65878E+00_JPRB, 0.10296E+01_JPRB, 0.13226E+01_JPRB, 0.15130E+01_JPRB, 0.15126E+01_JPRB, 0.91485E+00_JPRB ,&
&  0.82154E+00_JPRB, 0.80257E+00_JPRB, 0.78984E+00_JPRB, 0.77138E+00_JPRB, 0.75045E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.68928E+00_JPRB, 0.65184E+00_JPRB, 0.63215E+00_JPRB, 0.61642E+00_JPRB, 0.60645E+00_JPRB, 0.62748E+00_JPRB, 0.64639E+00_JPRB ,&
&  0.67494E+00_JPRB, 0.70794E+00_JPRB, 0.74687E+00_JPRB, 0.78210E+00_JPRB, 0.82833E+00_JPRB, 0.87519E+00_JPRB, 0.91600E+00_JPRB ,&
&  0.95284E+00_JPRB, 0.98682E+00_JPRB, 0.10215E+01_JPRB, 0.10446E+01_JPRB, 0.10666E+01_JPRB, 0.10864E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.11820E+01_JPRB, 0.12788E+01_JPRB, 0.13894E+01_JPRB, 0.14854E+01_JPRB, 0.16349E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.15926E+01_JPRB, 0.16510E+01_JPRB, 0.16361E+01_JPRB, 0.15427E+01_JPRB, 0.14435E+01_JPRB, 0.13678E+01_JPRB, 0.13219E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.13708E+01_JPRB, 0.13533E+01_JPRB, 0.14677E+01_JPRB, 0.14822E+01_JPRB, 0.14736E+01_JPRB, 0.16713E+01_JPRB ,&
&  0.18335E+01_JPRB, 0.20984E+01_JPRB, 0.22714E+01_JPRB, 0.24025E+01_JPRB, 0.25057E+01_JPRB, 0.25927E+01_JPRB, 0.26337E+01_JPRB ,&
&  0.26606E+01_JPRB, 0.26050E+01_JPRB, 0.25293E+01_JPRB, 0.27011E+01_JPRB, 0.27873E+01_JPRB, 0.23969E+01_JPRB, 0.21887E+01_JPRB ,&
&  0.16059E+01_JPRB, 0.14506E+01_JPRB, 0.21336E+01_JPRB, 0.25682E+01_JPRB, 0.22987E+01_JPRB, 0.21387E+01_JPRB, 0.18977E+01_JPRB ,&
&  0.16593E+01_JPRB, 0.15380E+01_JPRB, 0.14606E+01_JPRB, 0.13689E+01_JPRB, 0.12563E+01_JPRB, 0.11815E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.10344E+01_JPRB, 0.10027E+01_JPRB, 0.11234E+01_JPRB, 0.12312E+01_JPRB, 0.14939E+01_JPRB, 0.15900E+01_JPRB ,&
&  0.18320E+01_JPRB, 0.16332E+01_JPRB, 0.12163E+01_JPRB, 0.10738E+01_JPRB, 0.72407E+00_JPRB, 0.61499E+00_JPRB, 0.57996E+00_JPRB ,&
&  0.59289E+00_JPRB, 0.53636E+00_JPRB, 0.10358E+01_JPRB, 0.38431E+01_JPRB, 0.37717E+01_JPRB, 0.47588E+01_JPRB, 0.25241E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.80705E+00_JPRB, 0.72073E+00_JPRB, 0.71699E+00_JPRB, 0.70191E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.60561E+00_JPRB, 0.58187E+00_JPRB, 0.55750E+00_JPRB, 0.56493E+00_JPRB, 0.57322E+00_JPRB, 0.58329E+00_JPRB, 0.61499E+00_JPRB ,&
&  0.67905E+00_JPRB, 0.69670E+00_JPRB, 0.72728E+00_JPRB, 0.77855E+00_JPRB, 0.85433E+00_JPRB, 0.92250E+00_JPRB, 0.98830E+00_JPRB ,&
&  0.10570E+01_JPRB, 0.11124E+01_JPRB, 0.11452E+01_JPRB, 0.11654E+01_JPRB, 0.11988E+01_JPRB, 0.12377E+01_JPRB, 0.12871E+01_JPRB ,&
&  0.13196E+01_JPRB, 0.13828E+01_JPRB, 0.14899E+01_JPRB, 0.16053E+01_JPRB, 0.17141E+01_JPRB, 0.17529E+01_JPRB, 0.11368E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.14484E+01_JPRB, 0.15084E+01_JPRB, 0.14961E+01_JPRB, 0.14149E+01_JPRB, 0.13386E+01_JPRB, 0.13682E+01_JPRB ,&
&  0.14796E+01_JPRB, 0.16077E+01_JPRB, 0.16624E+01_JPRB, 0.16899E+01_JPRB, 0.19362E+01_JPRB, 0.19200E+01_JPRB, 0.22950E+01_JPRB ,&
&  0.24979E+01_JPRB, 0.26525E+01_JPRB, 0.28680E+01_JPRB, 0.29973E+01_JPRB, 0.31602E+01_JPRB, 0.33531E+01_JPRB, 0.35365E+01_JPRB ,&
&  0.36316E+01_JPRB, 0.36415E+01_JPRB, 0.37792E+01_JPRB, 0.42190E+01_JPRB, 0.39146E+01_JPRB, 0.43863E+01_JPRB, 0.35722E+01_JPRB ,&
&  0.23692E+01_JPRB, 0.19601E+01_JPRB, 0.23134E+01_JPRB, 0.24850E+01_JPRB, 0.23054E+01_JPRB, 0.21817E+01_JPRB, 0.19469E+01_JPRB ,&
&  0.16151E+01_JPRB, 0.14088E+01_JPRB, 0.13221E+01_JPRB, 0.12440E+01_JPRB, 0.11533E+01_JPRB, 0.10693E+01_JPRB, 0.10073E+01_JPRB ,&
&  0.93024E+00_JPRB, 0.86071E+00_JPRB, 0.88664E+00_JPRB, 0.91365E+00_JPRB, 0.99550E+00_JPRB, 0.13178E+01_JPRB, 0.19975E+01_JPRB ,&
&  0.31503E+01_JPRB, 0.17400E+01_JPRB, 0.12861E+01_JPRB, 0.10420E+01_JPRB, 0.80239E+00_JPRB, 0.68314E+00_JPRB, 0.61498E+00_JPRB ,&
&  0.59116E+00_JPRB, 0.70674E+00_JPRB, 0.83894E+00_JPRB, 0.15224E+01_JPRB, 0.26476E+01_JPRB, 0.26920E+01_JPRB, 0.18392E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.81590E+00_JPRB, 0.66012E+00_JPRB, 0.63002E+00_JPRB, 0.60859E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.55973E+00_JPRB, 0.54513E+00_JPRB, 0.54273E+00_JPRB, 0.54047E+00_JPRB, 0.57956E+00_JPRB, 0.60604E+00_JPRB, 0.63127E+00_JPRB ,&
&  0.67141E+00_JPRB, 0.69860E+00_JPRB, 0.73641E+00_JPRB, 0.80845E+00_JPRB, 0.89129E+00_JPRB, 0.96725E+00_JPRB, 0.10176E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.10790E+01_JPRB, 0.11024E+01_JPRB, 0.11246E+01_JPRB, 0.11808E+01_JPRB, 0.12324E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.15296E+01_JPRB, 0.16608E+01_JPRB, 0.17820E+01_JPRB, 0.19177E+01_JPRB, 0.20420E+01_JPRB, 0.21389E+01_JPRB ,&
&  0.96657E+00_JPRB, 0.13075E+01_JPRB, 0.13800E+01_JPRB, 0.13826E+01_JPRB, 0.13237E+01_JPRB, 0.13370E+01_JPRB, 0.15130E+01_JPRB ,&
&  0.17365E+01_JPRB, 0.18188E+01_JPRB, 0.20422E+01_JPRB, 0.19284E+01_JPRB, 0.23296E+01_JPRB, 0.24616E+01_JPRB, 0.27897E+01_JPRB ,&
&  0.30259E+01_JPRB, 0.32070E+01_JPRB, 0.34393E+01_JPRB, 0.36919E+01_JPRB, 0.39705E+01_JPRB, 0.42784E+01_JPRB, 0.46539E+01_JPRB ,&
&  0.50183E+01_JPRB, 0.54782E+01_JPRB, 0.60708E+01_JPRB, 0.61018E+01_JPRB, 0.56840E+01_JPRB, 0.61100E+01_JPRB, 0.57233E+01_JPRB ,&
&  0.47703E+01_JPRB, 0.33560E+01_JPRB, 0.31652E+01_JPRB, 0.27749E+01_JPRB, 0.24275E+01_JPRB, 0.22577E+01_JPRB, 0.19738E+01_JPRB ,&
&  0.16262E+01_JPRB, 0.14227E+01_JPRB, 0.12849E+01_JPRB, 0.12006E+01_JPRB, 0.10994E+01_JPRB, 0.10189E+01_JPRB, 0.95854E+00_JPRB ,&
&  0.90949E+00_JPRB, 0.91747E+00_JPRB, 0.93527E+00_JPRB, 0.88050E+00_JPRB, 0.85900E+00_JPRB, 0.96692E+00_JPRB, 0.11779E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.12386E+01_JPRB, 0.10611E+01_JPRB, 0.10454E+01_JPRB, 0.95859E+00_JPRB, 0.86233E+00_JPRB, 0.81333E+00_JPRB ,&
&  0.79335E+00_JPRB, 0.76105E+00_JPRB, 0.85660E+00_JPRB, 0.12844E+01_JPRB, 0.16108E+01_JPRB, 0.18271E+01_JPRB, 0.15183E+01_JPRB ,&
&  0.11019E+01_JPRB, 0.83520E+00_JPRB, 0.67615E+00_JPRB, 0.63386E+00_JPRB, 0.61066E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.60608E+00_JPRB, 0.59180E+00_JPRB, 0.59162E+00_JPRB, 0.58749E+00_JPRB, 0.62342E+00_JPRB, 0.66520E+00_JPRB, 0.67774E+00_JPRB ,&
&  0.67728E+00_JPRB, 0.70095E+00_JPRB, 0.72496E+00_JPRB, 0.76617E+00_JPRB, 0.84657E+00_JPRB, 0.89766E+00_JPRB, 0.95656E+00_JPRB ,&
&  0.10147E+01_JPRB, 0.10311E+01_JPRB, 0.10272E+01_JPRB, 0.10501E+01_JPRB, 0.11201E+01_JPRB, 0.11979E+01_JPRB, 0.13000E+01_JPRB ,&
&  0.14305E+01_JPRB, 0.15922E+01_JPRB, 0.17935E+01_JPRB, 0.20190E+01_JPRB, 0.22726E+01_JPRB, 0.25181E+01_JPRB, 0.26439E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.10859E+01_JPRB, 0.12912E+01_JPRB, 0.12900E+01_JPRB, 0.12530E+01_JPRB, 0.13439E+01_JPRB, 0.15087E+01_JPRB ,&
&  0.16181E+01_JPRB, 0.15504E+01_JPRB, 0.17250E+01_JPRB, 0.23029E+01_JPRB, 0.25757E+01_JPRB, 0.28250E+01_JPRB, 0.31120E+01_JPRB ,&
&  0.33352E+01_JPRB, 0.35780E+01_JPRB, 0.37947E+01_JPRB, 0.39878E+01_JPRB, 0.42292E+01_JPRB, 0.45426E+01_JPRB, 0.49587E+01_JPRB ,&
&  0.54920E+01_JPRB, 0.62667E+01_JPRB, 0.70194E+01_JPRB, 0.65082E+01_JPRB, 0.66902E+01_JPRB, 0.70752E+01_JPRB, 0.79898E+01_JPRB ,&
&  0.70618E+01_JPRB, 0.47382E+01_JPRB, 0.35683E+01_JPRB, 0.29479E+01_JPRB, 0.22760E+01_JPRB, 0.20188E+01_JPRB, 0.17503E+01_JPRB ,&
&  0.15498E+01_JPRB, 0.14101E+01_JPRB, 0.12922E+01_JPRB, 0.11965E+01_JPRB, 0.10798E+01_JPRB, 0.10149E+01_JPRB, 0.97539E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.10360E+01_JPRB, 0.10005E+01_JPRB, 0.94901E+00_JPRB, 0.92502E+00_JPRB, 0.97209E+00_JPRB, 0.10645E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.11090E+01_JPRB, 0.11156E+01_JPRB, 0.11278E+01_JPRB, 0.11298E+01_JPRB, 0.11248E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.96784E+00_JPRB, 0.94914E+00_JPRB, 0.92761E+00_JPRB, 0.10691E+01_JPRB, 0.13577E+01_JPRB, 0.13885E+01_JPRB ,&
&  0.11890E+01_JPRB, 0.92728E+00_JPRB, 0.76330E+00_JPRB, 0.71245E+00_JPRB, 0.68413E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.72959E+00_JPRB, 0.68292E+00_JPRB, 0.67159E+00_JPRB, 0.65480E+00_JPRB, 0.67085E+00_JPRB, 0.70134E+00_JPRB, 0.72249E+00_JPRB ,&
&  0.73107E+00_JPRB, 0.73154E+00_JPRB, 0.73631E+00_JPRB, 0.76314E+00_JPRB, 0.79880E+00_JPRB, 0.83918E+00_JPRB, 0.88875E+00_JPRB ,&
&  0.92412E+00_JPRB, 0.94344E+00_JPRB, 0.92924E+00_JPRB, 0.91279E+00_JPRB, 0.95396E+00_JPRB, 0.10201E+01_JPRB, 0.11098E+01_JPRB ,&
&  0.12238E+01_JPRB, 0.13622E+01_JPRB, 0.15670E+01_JPRB, 0.18868E+01_JPRB, 0.23683E+01_JPRB, 0.29531E+01_JPRB, 0.35655E+01_JPRB ,&
&  0.28863E+01_JPRB, 0.13240E+01_JPRB, 0.14185E+01_JPRB, 0.14968E+01_JPRB, 0.13594E+01_JPRB, 0.14565E+01_JPRB, 0.15187E+01_JPRB ,&
&  0.15209E+01_JPRB, 0.15467E+01_JPRB, 0.15349E+01_JPRB, 0.18062E+01_JPRB, 0.23780E+01_JPRB, 0.27400E+01_JPRB, 0.30414E+01_JPRB ,&
&  0.32827E+01_JPRB, 0.34579E+01_JPRB, 0.35698E+01_JPRB, 0.36856E+01_JPRB, 0.38900E+01_JPRB, 0.42054E+01_JPRB, 0.46245E+01_JPRB ,&
&  0.51924E+01_JPRB, 0.59295E+01_JPRB, 0.68030E+01_JPRB, 0.62856E+01_JPRB, 0.71234E+01_JPRB, 0.71118E+01_JPRB, 0.71732E+01_JPRB ,&
&  0.64785E+01_JPRB, 0.42545E+01_JPRB, 0.31318E+01_JPRB, 0.27824E+01_JPRB, 0.21037E+01_JPRB, 0.15314E+01_JPRB, 0.15424E+01_JPRB ,&
&  0.14488E+01_JPRB, 0.13701E+01_JPRB, 0.13255E+01_JPRB, 0.12058E+01_JPRB, 0.10974E+01_JPRB, 0.10514E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.12119E+01_JPRB, 0.11484E+01_JPRB, 0.11272E+01_JPRB, 0.10898E+01_JPRB, 0.11220E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.12385E+01_JPRB, 0.12620E+01_JPRB, 0.12952E+01_JPRB, 0.13310E+01_JPRB, 0.12707E+01_JPRB, 0.11977E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.11656E+01_JPRB, 0.10947E+01_JPRB, 0.10497E+01_JPRB, 0.99310E+00_JPRB, 0.10088E+01_JPRB, 0.11021E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.95056E+00_JPRB, 0.82852E+00_JPRB, 0.79872E+00_JPRB, 0.80241E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.85455E+00_JPRB, 0.75089E+00_JPRB, 0.69654E+00_JPRB, 0.69193E+00_JPRB, 0.67848E+00_JPRB, 0.68614E+00_JPRB, 0.69725E+00_JPRB ,&
&  0.69477E+00_JPRB, 0.71614E+00_JPRB, 0.71297E+00_JPRB, 0.73251E+00_JPRB, 0.75520E+00_JPRB, 0.76962E+00_JPRB, 0.80557E+00_JPRB ,&
&  0.83798E+00_JPRB, 0.86938E+00_JPRB, 0.89848E+00_JPRB, 0.87208E+00_JPRB, 0.88303E+00_JPRB, 0.90993E+00_JPRB, 0.96373E+00_JPRB ,&
&  0.10581E+01_JPRB, 0.11831E+01_JPRB, 0.13290E+01_JPRB, 0.15066E+01_JPRB, 0.17673E+01_JPRB, 0.22371E+01_JPRB, 0.31027E+01_JPRB ,&
&  0.52317E+01_JPRB, 0.44356E+01_JPRB, 0.13484E+01_JPRB, 0.17194E+01_JPRB, 0.16754E+01_JPRB, 0.16944E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.15938E+01_JPRB, 0.17082E+01_JPRB, 0.15208E+01_JPRB, 0.14541E+01_JPRB, 0.17469E+01_JPRB, 0.22379E+01_JPRB, 0.26117E+01_JPRB ,&
&  0.28578E+01_JPRB, 0.30438E+01_JPRB, 0.31250E+01_JPRB, 0.32024E+01_JPRB, 0.33163E+01_JPRB, 0.35366E+01_JPRB, 0.38860E+01_JPRB ,&
&  0.43979E+01_JPRB, 0.51268E+01_JPRB, 0.61048E+01_JPRB, 0.60893E+01_JPRB, 0.75432E+01_JPRB, 0.72060E+01_JPRB, 0.66578E+01_JPRB ,&
&  0.53823E+01_JPRB, 0.42216E+01_JPRB, 0.34337E+01_JPRB, 0.26397E+01_JPRB, 0.18459E+01_JPRB, 0.14461E+01_JPRB, 0.13911E+01_JPRB ,&
&  0.13473E+01_JPRB, 0.11973E+01_JPRB, 0.11643E+01_JPRB, 0.11476E+01_JPRB, 0.10902E+01_JPRB, 0.11634E+01_JPRB, 0.12770E+01_JPRB ,&
&  0.13498E+01_JPRB, 0.13449E+01_JPRB, 0.13048E+01_JPRB, 0.12852E+01_JPRB, 0.12662E+01_JPRB, 0.12688E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.13149E+01_JPRB, 0.13372E+01_JPRB, 0.13427E+01_JPRB, 0.13708E+01_JPRB, 0.14000E+01_JPRB, 0.13321E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.12525E+01_JPRB, 0.12188E+01_JPRB, 0.11907E+01_JPRB, 0.11099E+01_JPRB, 0.10421E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10085E+01_JPRB, 0.94375E+00_JPRB, 0.87412E+00_JPRB, 0.89345E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.91964E+00_JPRB, 0.85393E+00_JPRB, 0.75178E+00_JPRB, 0.71922E+00_JPRB, 0.71131E+00_JPRB, 0.69453E+00_JPRB, 0.67315E+00_JPRB ,&
&  0.64893E+00_JPRB, 0.63980E+00_JPRB, 0.66844E+00_JPRB, 0.70338E+00_JPRB, 0.73136E+00_JPRB, 0.74219E+00_JPRB, 0.76516E+00_JPRB ,&
&  0.77856E+00_JPRB, 0.80482E+00_JPRB, 0.85304E+00_JPRB, 0.87060E+00_JPRB, 0.86874E+00_JPRB, 0.87331E+00_JPRB, 0.88943E+00_JPRB ,&
&  0.92080E+00_JPRB, 0.97467E+00_JPRB, 0.10710E+01_JPRB, 0.11818E+01_JPRB, 0.13317E+01_JPRB, 0.15971E+01_JPRB, 0.21569E+01_JPRB ,&
&  0.33008E+01_JPRB, 0.60285E+01_JPRB, 0.14780E+01_JPRB, 0.20437E+01_JPRB, 0.20199E+01_JPRB, 0.18862E+01_JPRB, 0.17598E+01_JPRB ,&
&  0.17607E+01_JPRB, 0.17489E+01_JPRB, 0.16441E+01_JPRB, 0.14394E+01_JPRB, 0.12566E+01_JPRB, 0.15093E+01_JPRB, 0.19148E+01_JPRB ,&
&  0.22940E+01_JPRB, 0.24915E+01_JPRB, 0.25728E+01_JPRB, 0.25652E+01_JPRB, 0.26055E+01_JPRB, 0.27003E+01_JPRB, 0.29034E+01_JPRB ,&
&  0.32625E+01_JPRB, 0.38369E+01_JPRB, 0.47113E+01_JPRB, 0.51310E+01_JPRB, 0.69160E+01_JPRB, 0.73418E+01_JPRB, 0.69790E+01_JPRB ,&
&  0.55495E+01_JPRB, 0.48849E+01_JPRB, 0.41953E+01_JPRB, 0.29153E+01_JPRB, 0.20121E+01_JPRB, 0.15630E+01_JPRB, 0.14289E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.12087E+01_JPRB, 0.11932E+01_JPRB, 0.11631E+01_JPRB, 0.11835E+01_JPRB, 0.12661E+01_JPRB, 0.13697E+01_JPRB ,&
&  0.14003E+01_JPRB, 0.13976E+01_JPRB, 0.13604E+01_JPRB, 0.13482E+01_JPRB, 0.13470E+01_JPRB, 0.13496E+01_JPRB, 0.13841E+01_JPRB ,&
&  0.14007E+01_JPRB, 0.14124E+01_JPRB, 0.14053E+01_JPRB, 0.13683E+01_JPRB, 0.13779E+01_JPRB, 0.13347E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.12837E+01_JPRB, 0.13113E+01_JPRB, 0.12966E+01_JPRB, 0.12436E+01_JPRB, 0.12549E+01_JPRB, 0.11964E+01_JPRB, 0.11355E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.11478E+01_JPRB, 0.11066E+01_JPRB, 0.97106E+00_JPRB, 0.97233E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.97784E+00_JPRB, 0.92197E+00_JPRB, 0.84540E+00_JPRB, 0.76872E+00_JPRB, 0.76398E+00_JPRB, 0.76760E+00_JPRB, 0.72857E+00_JPRB ,&
&  0.67087E+00_JPRB, 0.66570E+00_JPRB, 0.67242E+00_JPRB, 0.69674E+00_JPRB, 0.71605E+00_JPRB, 0.73123E+00_JPRB, 0.77396E+00_JPRB ,&
&  0.77173E+00_JPRB, 0.76729E+00_JPRB, 0.79243E+00_JPRB, 0.83155E+00_JPRB, 0.84241E+00_JPRB, 0.84940E+00_JPRB, 0.87155E+00_JPRB ,&
&  0.88217E+00_JPRB, 0.87609E+00_JPRB, 0.89465E+00_JPRB, 0.94440E+00_JPRB, 0.10430E+01_JPRB, 0.12215E+01_JPRB, 0.15706E+01_JPRB ,&
&  0.26300E+01_JPRB, 0.63551E+01_JPRB, 0.38688E+01_JPRB, 0.36069E+01_JPRB, 0.26526E+01_JPRB, 0.21619E+01_JPRB, 0.19759E+01_JPRB ,&
&  0.21564E+01_JPRB, 0.20276E+01_JPRB, 0.18845E+01_JPRB, 0.14718E+01_JPRB, 0.12209E+01_JPRB, 0.11472E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.15865E+01_JPRB, 0.18091E+01_JPRB, 0.19664E+01_JPRB, 0.19814E+01_JPRB, 0.19986E+01_JPRB, 0.20667E+01_JPRB, 0.21613E+01_JPRB ,&
&  0.23575E+01_JPRB, 0.27386E+01_JPRB, 0.34124E+01_JPRB, 0.43085E+01_JPRB, 0.53796E+01_JPRB, 0.64439E+01_JPRB, 0.74756E+01_JPRB ,&
&  0.70881E+01_JPRB, 0.61020E+01_JPRB, 0.48905E+01_JPRB, 0.32563E+01_JPRB, 0.21343E+01_JPRB, 0.19124E+01_JPRB, 0.14895E+01_JPRB ,&
&  0.13853E+01_JPRB, 0.12791E+01_JPRB, 0.12728E+01_JPRB, 0.12952E+01_JPRB, 0.13353E+01_JPRB, 0.14024E+01_JPRB, 0.14571E+01_JPRB ,&
&  0.14641E+01_JPRB, 0.14356E+01_JPRB, 0.14137E+01_JPRB, 0.13979E+01_JPRB, 0.13876E+01_JPRB, 0.13841E+01_JPRB, 0.13927E+01_JPRB ,&
&  0.14047E+01_JPRB, 0.14297E+01_JPRB, 0.13721E+01_JPRB, 0.13564E+01_JPRB, 0.13549E+01_JPRB, 0.13279E+01_JPRB, 0.12629E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.13407E+01_JPRB, 0.13594E+01_JPRB, 0.13385E+01_JPRB, 0.13327E+01_JPRB, 0.13343E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.12585E+01_JPRB, 0.12226E+01_JPRB, 0.11269E+01_JPRB, 0.10674E+01_JPRB, 0.10463E+01_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.10406E+01_JPRB, 0.99525E+00_JPRB, 0.93488E+00_JPRB, 0.86037E+00_JPRB, 0.82189E+00_JPRB, 0.82151E+00_JPRB, 0.82691E+00_JPRB ,&
&  0.79098E+00_JPRB, 0.76620E+00_JPRB, 0.75552E+00_JPRB, 0.77028E+00_JPRB, 0.79898E+00_JPRB, 0.81754E+00_JPRB, 0.83020E+00_JPRB ,&
&  0.82368E+00_JPRB, 0.81820E+00_JPRB, 0.80214E+00_JPRB, 0.80410E+00_JPRB, 0.85400E+00_JPRB, 0.86189E+00_JPRB, 0.85483E+00_JPRB ,&
&  0.87436E+00_JPRB, 0.86618E+00_JPRB, 0.84910E+00_JPRB, 0.85786E+00_JPRB, 0.90047E+00_JPRB, 0.10195E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.24164E+01_JPRB, 0.58373E+01_JPRB, 0.26326E+01_JPRB, 0.41872E+01_JPRB, 0.29351E+01_JPRB, 0.23978E+01_JPRB, 0.19279E+01_JPRB ,&
&  0.19535E+01_JPRB, 0.22592E+01_JPRB, 0.18176E+01_JPRB, 0.15245E+01_JPRB, 0.12826E+01_JPRB, 0.11567E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.13557E+01_JPRB, 0.14244E+01_JPRB, 0.14970E+01_JPRB, 0.15417E+01_JPRB, 0.15187E+01_JPRB, 0.15310E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.16858E+01_JPRB, 0.18904E+01_JPRB, 0.23248E+01_JPRB, 0.30134E+01_JPRB, 0.45412E+01_JPRB, 0.61839E+01_JPRB, 0.79578E+01_JPRB ,&
&  0.79689E+01_JPRB, 0.77259E+01_JPRB, 0.57056E+01_JPRB, 0.36637E+01_JPRB, 0.24367E+01_JPRB, 0.20204E+01_JPRB, 0.16543E+01_JPRB ,&
&  0.14839E+01_JPRB, 0.14119E+01_JPRB, 0.13543E+01_JPRB, 0.13629E+01_JPRB, 0.14661E+01_JPRB, 0.15460E+01_JPRB, 0.15351E+01_JPRB ,&
&  0.15145E+01_JPRB, 0.14696E+01_JPRB, 0.14403E+01_JPRB, 0.14129E+01_JPRB, 0.14137E+01_JPRB, 0.14106E+01_JPRB, 0.13965E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.14084E+01_JPRB, 0.13636E+01_JPRB, 0.13668E+01_JPRB, 0.13649E+01_JPRB, 0.13451E+01_JPRB, 0.12822E+01_JPRB ,&
&  0.13060E+01_JPRB, 0.13409E+01_JPRB, 0.13463E+01_JPRB, 0.13253E+01_JPRB, 0.13280E+01_JPRB, 0.13904E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.12738E+01_JPRB, 0.11906E+01_JPRB, 0.11392E+01_JPRB, 0.11087E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.10982E+01_JPRB, 0.10420E+01_JPRB, 0.99693E+00_JPRB, 0.93627E+00_JPRB, 0.89250E+00_JPRB, 0.86959E+00_JPRB, 0.86757E+00_JPRB ,&
&  0.85291E+00_JPRB, 0.82281E+00_JPRB, 0.80523E+00_JPRB, 0.80657E+00_JPRB, 0.81073E+00_JPRB, 0.82665E+00_JPRB, 0.83952E+00_JPRB ,&
&  0.83804E+00_JPRB, 0.82737E+00_JPRB, 0.82192E+00_JPRB, 0.81988E+00_JPRB, 0.84334E+00_JPRB, 0.88475E+00_JPRB, 0.89898E+00_JPRB ,&
&  0.88633E+00_JPRB, 0.87473E+00_JPRB, 0.85255E+00_JPRB, 0.83714E+00_JPRB, 0.84824E+00_JPRB, 0.92591E+00_JPRB, 0.11603E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.67488E+01_JPRB, 0.27298E+01_JPRB, 0.45676E+01_JPRB, 0.31528E+01_JPRB, 0.25180E+01_JPRB, 0.19877E+01_JPRB ,&
&  0.19604E+01_JPRB, 0.21329E+01_JPRB, 0.18536E+01_JPRB, 0.16146E+01_JPRB, 0.14164E+01_JPRB, 0.12825E+01_JPRB, 0.12300E+01_JPRB ,&
&  0.12981E+01_JPRB, 0.13224E+01_JPRB, 0.13130E+01_JPRB, 0.13233E+01_JPRB, 0.12968E+01_JPRB, 0.12819E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.13042E+01_JPRB, 0.13448E+01_JPRB, 0.15531E+01_JPRB, 0.23694E+01_JPRB, 0.35896E+01_JPRB, 0.58692E+01_JPRB, 0.90962E+01_JPRB ,&
&  0.86709E+01_JPRB, 0.90357E+01_JPRB, 0.63968E+01_JPRB, 0.41371E+01_JPRB, 0.26941E+01_JPRB, 0.21256E+01_JPRB, 0.18815E+01_JPRB ,&
&  0.16766E+01_JPRB, 0.15336E+01_JPRB, 0.14429E+01_JPRB, 0.14293E+01_JPRB, 0.15009E+01_JPRB, 0.15413E+01_JPRB, 0.15686E+01_JPRB ,&
&  0.15539E+01_JPRB, 0.14765E+01_JPRB, 0.14341E+01_JPRB, 0.14162E+01_JPRB, 0.14237E+01_JPRB, 0.14184E+01_JPRB, 0.14011E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.13846E+01_JPRB, 0.13548E+01_JPRB, 0.13714E+01_JPRB, 0.13848E+01_JPRB, 0.13690E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.13290E+01_JPRB, 0.13204E+01_JPRB, 0.13270E+01_JPRB, 0.13293E+01_JPRB, 0.14040E+01_JPRB, 0.13479E+01_JPRB ,&
&  0.13258E+01_JPRB, 0.12883E+01_JPRB, 0.12220E+01_JPRB, 0.11789E+01_JPRB, 0.11521E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.11406E+01_JPRB, 0.10940E+01_JPRB, 0.10598E+01_JPRB, 0.10153E+01_JPRB, 0.95598E+00_JPRB, 0.92634E+00_JPRB, 0.90642E+00_JPRB ,&
&  0.89424E+00_JPRB, 0.86432E+00_JPRB, 0.84268E+00_JPRB, 0.84100E+00_JPRB, 0.84739E+00_JPRB, 0.84536E+00_JPRB, 0.84860E+00_JPRB ,&
&  0.85869E+00_JPRB, 0.86328E+00_JPRB, 0.85305E+00_JPRB, 0.83980E+00_JPRB, 0.83343E+00_JPRB, 0.86611E+00_JPRB, 0.88574E+00_JPRB ,&
&  0.89335E+00_JPRB, 0.87810E+00_JPRB, 0.85158E+00_JPRB, 0.82235E+00_JPRB, 0.81905E+00_JPRB, 0.85954E+00_JPRB, 0.10422E+01_JPRB ,&
&  0.19824E+01_JPRB, 0.26782E+01_JPRB, 0.47513E+01_JPRB, 0.48399E+01_JPRB, 0.33477E+01_JPRB, 0.26546E+01_JPRB, 0.22921E+01_JPRB ,&
&  0.22158E+01_JPRB, 0.20918E+01_JPRB, 0.18567E+01_JPRB, 0.16767E+01_JPRB, 0.15208E+01_JPRB, 0.13908E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.12511E+01_JPRB, 0.12480E+01_JPRB, 0.12489E+01_JPRB, 0.12469E+01_JPRB, 0.12409E+01_JPRB, 0.12385E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.12369E+01_JPRB, 0.12295E+01_JPRB, 0.12664E+01_JPRB, 0.16122E+01_JPRB, 0.30339E+01_JPRB, 0.51682E+01_JPRB, 0.80667E+01_JPRB ,&
&  0.91192E+01_JPRB, 0.85675E+01_JPRB, 0.59915E+01_JPRB, 0.42276E+01_JPRB, 0.28687E+01_JPRB, 0.21549E+01_JPRB, 0.19283E+01_JPRB ,&
&  0.18209E+01_JPRB, 0.17136E+01_JPRB, 0.15712E+01_JPRB, 0.15205E+01_JPRB, 0.15108E+01_JPRB, 0.15020E+01_JPRB, 0.15081E+01_JPRB ,&
&  0.14914E+01_JPRB, 0.14786E+01_JPRB, 0.14520E+01_JPRB, 0.14283E+01_JPRB, 0.14065E+01_JPRB, 0.13787E+01_JPRB, 0.13578E+01_JPRB ,&
&  0.13449E+01_JPRB, 0.13295E+01_JPRB, 0.13129E+01_JPRB, 0.13454E+01_JPRB, 0.14117E+01_JPRB, 0.14187E+01_JPRB, 0.13849E+01_JPRB ,&
&  0.13292E+01_JPRB, 0.13042E+01_JPRB, 0.12961E+01_JPRB, 0.13194E+01_JPRB, 0.12964E+01_JPRB, 0.14017E+01_JPRB, 0.13630E+01_JPRB ,&
&  0.13372E+01_JPRB, 0.13056E+01_JPRB, 0.12534E+01_JPRB, 0.12008E+01_JPRB, 0.11603E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.11522E+01_JPRB, 0.11004E+01_JPRB, 0.10637E+01_JPRB, 0.10391E+01_JPRB, 0.10015E+01_JPRB, 0.96331E+00_JPRB, 0.95268E+00_JPRB ,&
&  0.94926E+00_JPRB, 0.92834E+00_JPRB, 0.90228E+00_JPRB, 0.86965E+00_JPRB, 0.86042E+00_JPRB, 0.86608E+00_JPRB, 0.87365E+00_JPRB ,&
&  0.87570E+00_JPRB, 0.87835E+00_JPRB, 0.87689E+00_JPRB, 0.87243E+00_JPRB, 0.86642E+00_JPRB, 0.88432E+00_JPRB, 0.89221E+00_JPRB ,&
&  0.88399E+00_JPRB, 0.88274E+00_JPRB, 0.85886E+00_JPRB, 0.83444E+00_JPRB, 0.82470E+00_JPRB, 0.82076E+00_JPRB, 0.86316E+00_JPRB ,&
&  0.15038E+01_JPRB, 0.32692E+01_JPRB, 0.46990E+01_JPRB, 0.42772E+01_JPRB, 0.32732E+01_JPRB, 0.25556E+01_JPRB, 0.23528E+01_JPRB ,&
&  0.21917E+01_JPRB, 0.20429E+01_JPRB, 0.18470E+01_JPRB, 0.16938E+01_JPRB, 0.15685E+01_JPRB, 0.14609E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.12902E+01_JPRB, 0.12680E+01_JPRB, 0.12471E+01_JPRB, 0.12371E+01_JPRB, 0.12327E+01_JPRB, 0.12414E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.12492E+01_JPRB, 0.12457E+01_JPRB, 0.13564E+01_JPRB, 0.21656E+01_JPRB, 0.27686E+01_JPRB, 0.52155E+01_JPRB ,&
&  0.72991E+01_JPRB, 0.59207E+01_JPRB, 0.44926E+01_JPRB, 0.36328E+01_JPRB, 0.28641E+01_JPRB, 0.22014E+01_JPRB, 0.18812E+01_JPRB ,&
&  0.17599E+01_JPRB, 0.16823E+01_JPRB, 0.15851E+01_JPRB, 0.15163E+01_JPRB, 0.14637E+01_JPRB, 0.14393E+01_JPRB, 0.14265E+01_JPRB ,&
&  0.13996E+01_JPRB, 0.13963E+01_JPRB, 0.13876E+01_JPRB, 0.13790E+01_JPRB, 0.13550E+01_JPRB, 0.13284E+01_JPRB, 0.13050E+01_JPRB ,&
&  0.12882E+01_JPRB, 0.12675E+01_JPRB, 0.12541E+01_JPRB, 0.12831E+01_JPRB, 0.13782E+01_JPRB, 0.14500E+01_JPRB, 0.14470E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.13175E+01_JPRB, 0.13171E+01_JPRB, 0.13106E+01_JPRB, 0.13164E+01_JPRB, 0.14375E+01_JPRB, 0.13849E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.13219E+01_JPRB, 0.12832E+01_JPRB, 0.12344E+01_JPRB, 0.11940E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.11949E+01_JPRB, 0.11383E+01_JPRB, 0.10885E+01_JPRB, 0.10461E+01_JPRB, 0.10308E+01_JPRB, 0.10052E+01_JPRB, 0.97595E+00_JPRB ,&
&  0.96108E+00_JPRB, 0.96004E+00_JPRB, 0.95197E+00_JPRB, 0.93243E+00_JPRB, 0.91467E+00_JPRB, 0.90749E+00_JPRB, 0.90716E+00_JPRB ,&
&  0.90620E+00_JPRB, 0.90547E+00_JPRB, 0.90251E+00_JPRB, 0.89291E+00_JPRB, 0.89353E+00_JPRB, 0.90051E+00_JPRB, 0.90735E+00_JPRB ,&
&  0.88705E+00_JPRB, 0.87000E+00_JPRB, 0.85692E+00_JPRB, 0.84330E+00_JPRB, 0.83926E+00_JPRB, 0.82326E+00_JPRB, 0.81244E+00_JPRB ,&
&  0.10940E+01_JPRB, 0.21347E+01_JPRB, 0.30372E+01_JPRB, 0.37506E+01_JPRB, 0.32239E+01_JPRB, 0.26414E+01_JPRB, 0.23797E+01_JPRB ,&
&  0.21746E+01_JPRB, 0.19539E+01_JPRB, 0.17708E+01_JPRB, 0.16539E+01_JPRB, 0.15805E+01_JPRB, 0.14906E+01_JPRB, 0.13762E+01_JPRB ,&
&  0.12937E+01_JPRB, 0.12462E+01_JPRB, 0.12376E+01_JPRB, 0.12404E+01_JPRB, 0.12394E+01_JPRB, 0.12544E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.12667E+01_JPRB, 0.12519E+01_JPRB, 0.12265E+01_JPRB, 0.12241E+01_JPRB, 0.14793E+01_JPRB, 0.17358E+01_JPRB, 0.25630E+01_JPRB ,&
&  0.29858E+01_JPRB, 0.32002E+01_JPRB, 0.30610E+01_JPRB, 0.29210E+01_JPRB, 0.26593E+01_JPRB, 0.22085E+01_JPRB, 0.18478E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.15720E+01_JPRB, 0.15040E+01_JPRB, 0.14679E+01_JPRB, 0.14298E+01_JPRB, 0.13964E+01_JPRB, 0.13795E+01_JPRB ,&
&  0.13639E+01_JPRB, 0.13464E+01_JPRB, 0.13294E+01_JPRB, 0.13281E+01_JPRB, 0.13053E+01_JPRB, 0.12636E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.12123E+01_JPRB, 0.11967E+01_JPRB, 0.12091E+01_JPRB, 0.12505E+01_JPRB, 0.12979E+01_JPRB, 0.13353E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.13965E+01_JPRB, 0.13776E+01_JPRB, 0.13454E+01_JPRB, 0.13445E+01_JPRB, 0.13479E+01_JPRB, 0.14434E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.13305E+01_JPRB, 0.12967E+01_JPRB, 0.12369E+01_JPRB, 0.12455E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.11921E+01_JPRB, 0.11605E+01_JPRB, 0.11236E+01_JPRB, 0.10752E+01_JPRB, 0.10544E+01_JPRB, 0.10361E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.10062E+01_JPRB, 0.99681E+00_JPRB, 0.98962E+00_JPRB, 0.97337E+00_JPRB, 0.95619E+00_JPRB, 0.94478E+00_JPRB, 0.93168E+00_JPRB ,&
&  0.93105E+00_JPRB, 0.93030E+00_JPRB, 0.92134E+00_JPRB, 0.91150E+00_JPRB, 0.90596E+00_JPRB, 0.90139E+00_JPRB, 0.90384E+00_JPRB ,&
&  0.89440E+00_JPRB, 0.88179E+00_JPRB, 0.87981E+00_JPRB, 0.87058E+00_JPRB, 0.85575E+00_JPRB, 0.84406E+00_JPRB, 0.81482E+00_JPRB ,&
&  0.88175E+00_JPRB, 0.12333E+01_JPRB, 0.19003E+01_JPRB, 0.26513E+01_JPRB, 0.25990E+01_JPRB, 0.24646E+01_JPRB, 0.22263E+01_JPRB ,&
&  0.20058E+01_JPRB, 0.18330E+01_JPRB, 0.16413E+01_JPRB, 0.15115E+01_JPRB, 0.14645E+01_JPRB, 0.14158E+01_JPRB, 0.13609E+01_JPRB ,&
&  0.13019E+01_JPRB, 0.12518E+01_JPRB, 0.12271E+01_JPRB, 0.12333E+01_JPRB, 0.12270E+01_JPRB, 0.12249E+01_JPRB, 0.12260E+01_JPRB ,&
&  0.12151E+01_JPRB, 0.11984E+01_JPRB, 0.11657E+01_JPRB, 0.11348E+01_JPRB, 0.11133E+01_JPRB, 0.11289E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.16436E+01_JPRB, 0.18063E+01_JPRB, 0.18940E+01_JPRB, 0.20015E+01_JPRB, 0.20577E+01_JPRB, 0.19945E+01_JPRB, 0.18034E+01_JPRB ,&
&  0.16449E+01_JPRB, 0.15444E+01_JPRB, 0.14539E+01_JPRB, 0.14443E+01_JPRB, 0.14167E+01_JPRB, 0.13787E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.12968E+01_JPRB, 0.12802E+01_JPRB, 0.12516E+01_JPRB, 0.12293E+01_JPRB, 0.12052E+01_JPRB, 0.11738E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.11132E+01_JPRB, 0.11240E+01_JPRB, 0.11601E+01_JPRB, 0.12070E+01_JPRB, 0.12432E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.13419E+01_JPRB, 0.13099E+01_JPRB, 0.13004E+01_JPRB, 0.14160E+01_JPRB, 0.14089E+01_JPRB, 0.13604E+01_JPRB ,&
&  0.13276E+01_JPRB, 0.12964E+01_JPRB, 0.12742E+01_JPRB, 0.12048E+01_JPRB, 0.11798E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.10640E+01_JPRB, 0.10827E+01_JPRB, 0.10682E+01_JPRB, 0.10339E+01_JPRB, 0.10146E+01_JPRB, 0.10073E+01_JPRB, 0.10066E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.99343E+00_JPRB, 0.99805E+00_JPRB, 0.10029E+01_JPRB, 0.99734E+00_JPRB, 0.98160E+00_JPRB, 0.96927E+00_JPRB ,&
&  0.96405E+00_JPRB, 0.96573E+00_JPRB, 0.96896E+00_JPRB, 0.95722E+00_JPRB, 0.93383E+00_JPRB, 0.91755E+00_JPRB, 0.90379E+00_JPRB ,&
&  0.90197E+00_JPRB, 0.90146E+00_JPRB, 0.89868E+00_JPRB, 0.89897E+00_JPRB, 0.87585E+00_JPRB, 0.84970E+00_JPRB, 0.82898E+00_JPRB ,&
&  0.82675E+00_JPRB, 0.84158E+00_JPRB, 0.11709E+01_JPRB, 0.16339E+01_JPRB, 0.18508E+01_JPRB, 0.18213E+01_JPRB, 0.16487E+01_JPRB ,&
&  0.15376E+01_JPRB, 0.14654E+01_JPRB, 0.14041E+01_JPRB, 0.13313E+01_JPRB, 0.12839E+01_JPRB, 0.12534E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.11754E+01_JPRB, 0.11548E+01_JPRB, 0.11287E+01_JPRB, 0.11067E+01_JPRB, 0.10982E+01_JPRB, 0.10930E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.10816E+01_JPRB, 0.10734E+01_JPRB, 0.10615E+01_JPRB, 0.10278E+01_JPRB, 0.10142E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.12030E+01_JPRB, 0.12578E+01_JPRB, 0.13746E+01_JPRB, 0.14857E+01_JPRB, 0.15793E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.15223E+01_JPRB, 0.14739E+01_JPRB, 0.14208E+01_JPRB, 0.13967E+01_JPRB, 0.13692E+01_JPRB, 0.13164E+01_JPRB, 0.12600E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.11975E+01_JPRB, 0.11632E+01_JPRB, 0.11338E+01_JPRB, 0.11010E+01_JPRB, 0.10621E+01_JPRB, 0.10281E+01_JPRB ,&
&  0.10039E+01_JPRB, 0.99628E+00_JPRB, 0.10022E+01_JPRB, 0.10274E+01_JPRB, 0.10644E+01_JPRB, 0.11087E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.11943E+01_JPRB, 0.12252E+01_JPRB, 0.12604E+01_JPRB, 0.13132E+01_JPRB, 0.13154E+01_JPRB, 0.13060E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.12365E+01_JPRB, 0.11888E+01_JPRB, 0.11126E+01_JPRB, 0.10159E+01_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.10045E+01_JPRB, 0.10179E+01_JPRB, 0.10135E+01_JPRB, 0.99998E+00_JPRB, 0.98915E+00_JPRB, 0.98295E+00_JPRB, 0.98668E+00_JPRB ,&
&  0.98616E+00_JPRB, 0.98120E+00_JPRB, 0.98375E+00_JPRB, 0.98430E+00_JPRB, 0.98361E+00_JPRB, 0.97753E+00_JPRB, 0.97889E+00_JPRB ,&
&  0.97680E+00_JPRB, 0.97627E+00_JPRB, 0.97780E+00_JPRB, 0.96996E+00_JPRB, 0.94843E+00_JPRB, 0.93284E+00_JPRB, 0.91794E+00_JPRB ,&
&  0.91664E+00_JPRB, 0.91601E+00_JPRB, 0.91626E+00_JPRB, 0.90916E+00_JPRB, 0.88722E+00_JPRB, 0.86754E+00_JPRB, 0.85497E+00_JPRB ,&
&  0.85594E+00_JPRB, 0.74025E+00_JPRB, 0.88395E+00_JPRB, 0.12259E+01_JPRB, 0.13934E+01_JPRB, 0.13105E+01_JPRB, 0.12671E+01_JPRB ,&
&  0.12561E+01_JPRB, 0.12484E+01_JPRB, 0.12285E+01_JPRB, 0.11890E+01_JPRB, 0.11254E+01_JPRB, 0.10976E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10311E+01_JPRB, 0.10199E+01_JPRB, 0.10143E+01_JPRB, 0.99732E+00_JPRB, 0.97968E+00_JPRB, 0.97516E+00_JPRB ,&
&  0.97953E+00_JPRB, 0.97785E+00_JPRB, 0.98138E+00_JPRB, 0.98257E+00_JPRB, 0.97114E+00_JPRB, 0.97892E+00_JPRB, 0.98362E+00_JPRB ,&
&  0.10018E+01_JPRB, 0.10210E+01_JPRB, 0.10385E+01_JPRB, 0.10773E+01_JPRB, 0.11434E+01_JPRB, 0.12546E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.13255E+01_JPRB, 0.13075E+01_JPRB, 0.13014E+01_JPRB, 0.12931E+01_JPRB, 0.12726E+01_JPRB, 0.12315E+01_JPRB, 0.11867E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.11173E+01_JPRB, 0.10834E+01_JPRB, 0.10478E+01_JPRB, 0.10125E+01_JPRB, 0.97907E+00_JPRB, 0.95082E+00_JPRB ,&
&  0.93185E+00_JPRB, 0.92372E+00_JPRB, 0.93296E+00_JPRB, 0.95785E+00_JPRB, 0.99183E+00_JPRB, 0.10310E+01_JPRB, 0.10720E+01_JPRB ,&
&  0.11167E+01_JPRB, 0.11612E+01_JPRB, 0.12045E+01_JPRB, 0.11647E+01_JPRB, 0.12309E+01_JPRB, 0.12405E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.11729E+01_JPRB, 0.11003E+01_JPRB, 0.88689E+00_JPRB, 0.96837E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.97601E+00_JPRB, 0.98050E+00_JPRB, 0.98281E+00_JPRB, 0.96917E+00_JPRB, 0.95179E+00_JPRB, 0.94491E+00_JPRB, 0.95130E+00_JPRB ,&
&  0.94678E+00_JPRB, 0.93339E+00_JPRB, 0.93783E+00_JPRB, 0.93987E+00_JPRB, 0.94482E+00_JPRB, 0.94569E+00_JPRB, 0.94672E+00_JPRB ,&
&  0.94799E+00_JPRB, 0.94984E+00_JPRB, 0.94694E+00_JPRB, 0.95454E+00_JPRB, 0.94654E+00_JPRB, 0.94042E+00_JPRB, 0.93718E+00_JPRB ,&
&  0.92875E+00_JPRB, 0.92025E+00_JPRB, 0.91285E+00_JPRB, 0.90204E+00_JPRB, 0.89281E+00_JPRB, 0.88137E+00_JPRB, 0.86437E+00_JPRB ,&
&  0.87521E+00_JPRB, 0.73575E+00_JPRB, 0.85883E+00_JPRB, 0.10776E+01_JPRB, 0.10762E+01_JPRB, 0.10444E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10889E+01_JPRB, 0.10814E+01_JPRB, 0.10514E+01_JPRB, 0.98873E+00_JPRB, 0.95978E+00_JPRB, 0.94479E+00_JPRB ,&
&  0.92803E+00_JPRB, 0.91481E+00_JPRB, 0.90584E+00_JPRB, 0.90660E+00_JPRB, 0.90706E+00_JPRB, 0.90383E+00_JPRB, 0.89837E+00_JPRB ,&
&  0.89915E+00_JPRB, 0.90626E+00_JPRB, 0.90891E+00_JPRB, 0.91204E+00_JPRB, 0.91467E+00_JPRB, 0.90639E+00_JPRB, 0.90040E+00_JPRB ,&
&  0.89609E+00_JPRB, 0.89819E+00_JPRB, 0.90069E+00_JPRB, 0.91677E+00_JPRB, 0.94944E+00_JPRB, 0.10203E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.11156E+01_JPRB, 0.11234E+01_JPRB, 0.11254E+01_JPRB, 0.11249E+01_JPRB, 0.11267E+01_JPRB, 0.11181E+01_JPRB, 0.10938E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10292E+01_JPRB, 0.10006E+01_JPRB, 0.97797E+00_JPRB, 0.95481E+00_JPRB, 0.93708E+00_JPRB, 0.92877E+00_JPRB ,&
&  0.91792E+00_JPRB, 0.90440E+00_JPRB, 0.90760E+00_JPRB, 0.92492E+00_JPRB, 0.94399E+00_JPRB, 0.96147E+00_JPRB, 0.98969E+00_JPRB ,&
&  0.10334E+01_JPRB, 0.10773E+01_JPRB, 0.11127E+01_JPRB, 0.11472E+01_JPRB, 0.11609E+01_JPRB, 0.11634E+01_JPRB, 0.11535E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.11049E+01_JPRB, 0.83535E+00_JPRB, 0.94080E+00_JPRB, 0.96505E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.96768E+00_JPRB, 0.95796E+00_JPRB, 0.94638E+00_JPRB, 0.92949E+00_JPRB, 0.91184E+00_JPRB, 0.90063E+00_JPRB, 0.90521E+00_JPRB ,&
&  0.90297E+00_JPRB, 0.89036E+00_JPRB, 0.90037E+00_JPRB, 0.91285E+00_JPRB, 0.92233E+00_JPRB, 0.93046E+00_JPRB, 0.93174E+00_JPRB ,&
&  0.93339E+00_JPRB, 0.92897E+00_JPRB, 0.93151E+00_JPRB, 0.94366E+00_JPRB, 0.96266E+00_JPRB, 0.96453E+00_JPRB, 0.96153E+00_JPRB ,&
&  0.93940E+00_JPRB, 0.92830E+00_JPRB, 0.91805E+00_JPRB, 0.90545E+00_JPRB, 0.89343E+00_JPRB, 0.88792E+00_JPRB, 0.88828E+00_JPRB ,&
&  0.77345E+00_JPRB, 0.68638E+00_JPRB, 0.83065E+00_JPRB, 0.94120E+00_JPRB, 0.92947E+00_JPRB, 0.91429E+00_JPRB, 0.93147E+00_JPRB ,&
&  0.93773E+00_JPRB, 0.93312E+00_JPRB, 0.93391E+00_JPRB, 0.93034E+00_JPRB, 0.90088E+00_JPRB, 0.86465E+00_JPRB, 0.84769E+00_JPRB ,&
&  0.83818E+00_JPRB, 0.84361E+00_JPRB, 0.84614E+00_JPRB, 0.85407E+00_JPRB, 0.85434E+00_JPRB, 0.84697E+00_JPRB, 0.84410E+00_JPRB ,&
&  0.84248E+00_JPRB, 0.84968E+00_JPRB, 0.85577E+00_JPRB, 0.85946E+00_JPRB, 0.85931E+00_JPRB, 0.85754E+00_JPRB, 0.85161E+00_JPRB ,&
&  0.84659E+00_JPRB, 0.84338E+00_JPRB, 0.84067E+00_JPRB, 0.84668E+00_JPRB, 0.86405E+00_JPRB, 0.89856E+00_JPRB, 0.94824E+00_JPRB ,&
&  0.98137E+00_JPRB, 0.99571E+00_JPRB, 0.10031E+01_JPRB, 0.10013E+01_JPRB, 0.10088E+01_JPRB, 0.10096E+01_JPRB, 0.99183E+00_JPRB ,&
&  0.96912E+00_JPRB, 0.95051E+00_JPRB, 0.94205E+00_JPRB, 0.93306E+00_JPRB, 0.92658E+00_JPRB, 0.91905E+00_JPRB, 0.90865E+00_JPRB ,&
&  0.89825E+00_JPRB, 0.88459E+00_JPRB, 0.87471E+00_JPRB, 0.87630E+00_JPRB, 0.88987E+00_JPRB, 0.90257E+00_JPRB, 0.92676E+00_JPRB ,&
&  0.96614E+00_JPRB, 0.10115E+01_JPRB, 0.10485E+01_JPRB, 0.10854E+01_JPRB, 0.11054E+01_JPRB, 0.11124E+01_JPRB, 0.11021E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.10381E+01_JPRB, 0.10200E+01_JPRB, 0.99328E+00_JPRB, 0.97913E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.91647E+00_JPRB, 0.90919E+00_JPRB, 0.89495E+00_JPRB, 0.86931E+00_JPRB, 0.83573E+00_JPRB, 0.84321E+00_JPRB, 0.85320E+00_JPRB ,&
&  0.84555E+00_JPRB, 0.83541E+00_JPRB, 0.84597E+00_JPRB, 0.87130E+00_JPRB, 0.89540E+00_JPRB, 0.90864E+00_JPRB, 0.90968E+00_JPRB ,&
&  0.90334E+00_JPRB, 0.90419E+00_JPRB, 0.91204E+00_JPRB, 0.92938E+00_JPRB, 0.92899E+00_JPRB, 0.91135E+00_JPRB, 0.90056E+00_JPRB ,&
&  0.89792E+00_JPRB, 0.89622E+00_JPRB, 0.89383E+00_JPRB, 0.87706E+00_JPRB, 0.86609E+00_JPRB, 0.86286E+00_JPRB, 0.86263E+00_JPRB ,&
&  0.86051E+00_JPRB, 0.72454E+00_JPRB, 0.85181E+00_JPRB, 0.85630E+00_JPRB, 0.85748E+00_JPRB, 0.83945E+00_JPRB, 0.83592E+00_JPRB ,&
&  0.83838E+00_JPRB, 0.85305E+00_JPRB, 0.84873E+00_JPRB, 0.83001E+00_JPRB, 0.80958E+00_JPRB, 0.78418E+00_JPRB, 0.76665E+00_JPRB ,&
&  0.75772E+00_JPRB, 0.76146E+00_JPRB, 0.77225E+00_JPRB, 0.78426E+00_JPRB, 0.79650E+00_JPRB, 0.79856E+00_JPRB, 0.80207E+00_JPRB ,&
&  0.80335E+00_JPRB, 0.80634E+00_JPRB, 0.80627E+00_JPRB, 0.80889E+00_JPRB, 0.80842E+00_JPRB, 0.80354E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.78161E+00_JPRB, 0.77410E+00_JPRB, 0.77382E+00_JPRB, 0.77760E+00_JPRB, 0.79137E+00_JPRB, 0.81773E+00_JPRB, 0.85070E+00_JPRB ,&
&  0.88400E+00_JPRB, 0.90584E+00_JPRB, 0.91815E+00_JPRB, 0.92093E+00_JPRB, 0.92735E+00_JPRB, 0.93309E+00_JPRB, 0.92649E+00_JPRB ,&
&  0.91082E+00_JPRB, 0.89362E+00_JPRB, 0.88204E+00_JPRB, 0.87004E+00_JPRB, 0.86300E+00_JPRB, 0.85731E+00_JPRB, 0.84906E+00_JPRB ,&
&  0.83973E+00_JPRB, 0.83867E+00_JPRB, 0.84101E+00_JPRB, 0.83161E+00_JPRB, 0.83519E+00_JPRB, 0.84791E+00_JPRB, 0.87026E+00_JPRB ,&
&  0.89820E+00_JPRB, 0.93426E+00_JPRB, 0.96898E+00_JPRB, 0.99615E+00_JPRB, 0.10117E+01_JPRB, 0.10154E+01_JPRB, 0.10081E+01_JPRB ,&
&  0.98908E+00_JPRB, 0.96348E+00_JPRB, 0.94396E+00_JPRB, 0.93295E+00_JPRB, 0.93244E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.86486E+00_JPRB, 0.86525E+00_JPRB, 0.85498E+00_JPRB, 0.81694E+00_JPRB, 0.78774E+00_JPRB, 0.79633E+00_JPRB, 0.80396E+00_JPRB ,&
&  0.80330E+00_JPRB, 0.81873E+00_JPRB, 0.84390E+00_JPRB, 0.86918E+00_JPRB, 0.88596E+00_JPRB, 0.88525E+00_JPRB, 0.88841E+00_JPRB ,&
&  0.88494E+00_JPRB, 0.88226E+00_JPRB, 0.89191E+00_JPRB, 0.89687E+00_JPRB, 0.88618E+00_JPRB, 0.86544E+00_JPRB, 0.86368E+00_JPRB ,&
&  0.87021E+00_JPRB, 0.86587E+00_JPRB, 0.84766E+00_JPRB, 0.83539E+00_JPRB, 0.82548E+00_JPRB, 0.83123E+00_JPRB, 0.83939E+00_JPRB ,&
&  0.83557E+00_JPRB, 0.79993E+00_JPRB, 0.83984E+00_JPRB, 0.84615E+00_JPRB, 0.84466E+00_JPRB, 0.82288E+00_JPRB, 0.82419E+00_JPRB ,&
&  0.83304E+00_JPRB, 0.83821E+00_JPRB, 0.81963E+00_JPRB, 0.79871E+00_JPRB, 0.78102E+00_JPRB, 0.76540E+00_JPRB, 0.73912E+00_JPRB ,&
&  0.72978E+00_JPRB, 0.72724E+00_JPRB, 0.74137E+00_JPRB, 0.75665E+00_JPRB, 0.77434E+00_JPRB, 0.78197E+00_JPRB, 0.79260E+00_JPRB ,&
&  0.79401E+00_JPRB, 0.78706E+00_JPRB, 0.78006E+00_JPRB, 0.77369E+00_JPRB, 0.77095E+00_JPRB, 0.76392E+00_JPRB, 0.75350E+00_JPRB ,&
&  0.74420E+00_JPRB, 0.74243E+00_JPRB, 0.74769E+00_JPRB, 0.75771E+00_JPRB, 0.77285E+00_JPRB, 0.79970E+00_JPRB, 0.82428E+00_JPRB ,&
&  0.85254E+00_JPRB, 0.86512E+00_JPRB, 0.87472E+00_JPRB, 0.88890E+00_JPRB, 0.89565E+00_JPRB, 0.89275E+00_JPRB, 0.88775E+00_JPRB ,&
&  0.87935E+00_JPRB, 0.86736E+00_JPRB, 0.86655E+00_JPRB, 0.86039E+00_JPRB, 0.85619E+00_JPRB, 0.84367E+00_JPRB, 0.82537E+00_JPRB ,&
&  0.81802E+00_JPRB, 0.81333E+00_JPRB, 0.80559E+00_JPRB, 0.80329E+00_JPRB, 0.80104E+00_JPRB, 0.82013E+00_JPRB, 0.85378E+00_JPRB ,&
&  0.87342E+00_JPRB, 0.89939E+00_JPRB, 0.93529E+00_JPRB, 0.95785E+00_JPRB, 0.95758E+00_JPRB, 0.95559E+00_JPRB, 0.94281E+00_JPRB ,&
&  0.92314E+00_JPRB, 0.90896E+00_JPRB, 0.89565E+00_JPRB, 0.89043E+00_JPRB, 0.88585E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.81427E+00_JPRB, 0.83638E+00_JPRB, 0.80424E+00_JPRB, 0.78662E+00_JPRB, 0.76539E+00_JPRB, 0.75829E+00_JPRB, 0.74857E+00_JPRB ,&
&  0.76012E+00_JPRB, 0.77813E+00_JPRB, 0.80244E+00_JPRB, 0.81045E+00_JPRB, 0.81704E+00_JPRB, 0.81950E+00_JPRB, 0.83554E+00_JPRB ,&
&  0.82996E+00_JPRB, 0.82119E+00_JPRB, 0.82395E+00_JPRB, 0.81340E+00_JPRB, 0.80463E+00_JPRB, 0.81015E+00_JPRB, 0.82018E+00_JPRB ,&
&  0.83100E+00_JPRB, 0.82225E+00_JPRB, 0.80457E+00_JPRB, 0.77798E+00_JPRB, 0.76543E+00_JPRB, 0.76269E+00_JPRB, 0.77068E+00_JPRB ,&
&  0.77747E+00_JPRB, 0.81069E+00_JPRB, 0.74328E+00_JPRB, 0.80063E+00_JPRB, 0.80326E+00_JPRB, 0.79507E+00_JPRB, 0.79086E+00_JPRB ,&
&  0.79381E+00_JPRB, 0.78268E+00_JPRB, 0.77463E+00_JPRB, 0.76748E+00_JPRB, 0.74480E+00_JPRB, 0.72686E+00_JPRB, 0.70849E+00_JPRB ,&
&  0.69237E+00_JPRB, 0.68240E+00_JPRB, 0.68838E+00_JPRB, 0.70721E+00_JPRB, 0.72623E+00_JPRB, 0.73669E+00_JPRB, 0.74287E+00_JPRB ,&
&  0.74601E+00_JPRB, 0.73847E+00_JPRB, 0.72777E+00_JPRB, 0.72680E+00_JPRB, 0.73189E+00_JPRB, 0.73556E+00_JPRB, 0.73699E+00_JPRB ,&
&  0.73561E+00_JPRB, 0.71341E+00_JPRB, 0.70118E+00_JPRB, 0.71855E+00_JPRB, 0.73924E+00_JPRB, 0.76400E+00_JPRB, 0.79604E+00_JPRB ,&
&  0.82133E+00_JPRB, 0.83187E+00_JPRB, 0.83328E+00_JPRB, 0.84011E+00_JPRB, 0.84532E+00_JPRB, 0.85305E+00_JPRB, 0.85853E+00_JPRB ,&
&  0.84399E+00_JPRB, 0.83858E+00_JPRB, 0.83772E+00_JPRB, 0.83592E+00_JPRB, 0.82417E+00_JPRB, 0.79828E+00_JPRB, 0.76373E+00_JPRB ,&
&  0.74748E+00_JPRB, 0.73214E+00_JPRB, 0.72132E+00_JPRB, 0.71718E+00_JPRB, 0.71737E+00_JPRB, 0.74440E+00_JPRB, 0.78418E+00_JPRB ,&
&  0.82406E+00_JPRB, 0.85612E+00_JPRB, 0.88377E+00_JPRB, 0.90884E+00_JPRB, 0.90807E+00_JPRB, 0.88613E+00_JPRB, 0.87610E+00_JPRB ,&
&  0.86043E+00_JPRB, 0.84310E+00_JPRB, 0.83085E+00_JPRB, 0.82159E+00_JPRB, 0.81231E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.81182E+00_JPRB, 0.81386E+00_JPRB, 0.79977E+00_JPRB, 0.78995E+00_JPRB, 0.77188E+00_JPRB, 0.74801E+00_JPRB, 0.72782E+00_JPRB ,&
&  0.73025E+00_JPRB, 0.74682E+00_JPRB, 0.76249E+00_JPRB, 0.77361E+00_JPRB, 0.78896E+00_JPRB, 0.79181E+00_JPRB, 0.79748E+00_JPRB ,&
&  0.79208E+00_JPRB, 0.75439E+00_JPRB, 0.74823E+00_JPRB, 0.75568E+00_JPRB, 0.75752E+00_JPRB, 0.76417E+00_JPRB, 0.77588E+00_JPRB ,&
&  0.78651E+00_JPRB, 0.78210E+00_JPRB, 0.77373E+00_JPRB, 0.74376E+00_JPRB, 0.73686E+00_JPRB, 0.74170E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.75717E+00_JPRB, 0.77649E+00_JPRB, 0.77770E+00_JPRB, 0.78698E+00_JPRB, 0.79304E+00_JPRB, 0.79448E+00_JPRB, 0.78972E+00_JPRB ,&
&  0.76234E+00_JPRB, 0.73536E+00_JPRB, 0.72441E+00_JPRB, 0.70739E+00_JPRB, 0.68413E+00_JPRB, 0.66803E+00_JPRB, 0.66110E+00_JPRB ,&
&  0.65815E+00_JPRB, 0.66403E+00_JPRB, 0.67054E+00_JPRB, 0.68471E+00_JPRB, 0.69009E+00_JPRB, 0.69437E+00_JPRB, 0.70010E+00_JPRB ,&
&  0.70046E+00_JPRB, 0.69726E+00_JPRB, 0.69783E+00_JPRB, 0.70771E+00_JPRB, 0.71278E+00_JPRB, 0.70931E+00_JPRB, 0.70356E+00_JPRB ,&
&  0.68970E+00_JPRB, 0.67700E+00_JPRB, 0.68507E+00_JPRB, 0.70171E+00_JPRB, 0.71104E+00_JPRB, 0.73995E+00_JPRB, 0.77684E+00_JPRB ,&
&  0.80842E+00_JPRB, 0.82830E+00_JPRB, 0.82860E+00_JPRB, 0.82309E+00_JPRB, 0.81938E+00_JPRB, 0.82576E+00_JPRB, 0.83686E+00_JPRB ,&
&  0.81964E+00_JPRB, 0.80990E+00_JPRB, 0.80728E+00_JPRB, 0.78692E+00_JPRB, 0.74883E+00_JPRB, 0.71495E+00_JPRB, 0.69028E+00_JPRB ,&
&  0.67100E+00_JPRB, 0.66026E+00_JPRB, 0.66413E+00_JPRB, 0.66678E+00_JPRB, 0.67832E+00_JPRB, 0.71418E+00_JPRB, 0.75257E+00_JPRB ,&
&  0.79461E+00_JPRB, 0.81663E+00_JPRB, 0.82446E+00_JPRB, 0.83354E+00_JPRB, 0.84028E+00_JPRB, 0.82940E+00_JPRB, 0.81991E+00_JPRB ,&
&  0.81691E+00_JPRB, 0.80801E+00_JPRB, 0.79856E+00_JPRB, 0.78933E+00_JPRB, 0.79763E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.80340E+00_JPRB, 0.79678E+00_JPRB, 0.78401E+00_JPRB, 0.75668E+00_JPRB, 0.73795E+00_JPRB, 0.71054E+00_JPRB, 0.69540E+00_JPRB ,&
&  0.69822E+00_JPRB, 0.70034E+00_JPRB, 0.70581E+00_JPRB, 0.71482E+00_JPRB, 0.71226E+00_JPRB, 0.70710E+00_JPRB, 0.70177E+00_JPRB ,&
&  0.70406E+00_JPRB, 0.70433E+00_JPRB, 0.71280E+00_JPRB, 0.72802E+00_JPRB, 0.74405E+00_JPRB, 0.75357E+00_JPRB, 0.76424E+00_JPRB ,&
&  0.77246E+00_JPRB, 0.76662E+00_JPRB, 0.76189E+00_JPRB, 0.73047E+00_JPRB, 0.71214E+00_JPRB, 0.71698E+00_JPRB, 0.72456E+00_JPRB ,&
&  0.73478E+00_JPRB, 0.74592E+00_JPRB, 0.75414E+00_JPRB, 0.76122E+00_JPRB, 0.74717E+00_JPRB, 0.74222E+00_JPRB, 0.73051E+00_JPRB ,&
&  0.68568E+00_JPRB, 0.64212E+00_JPRB, 0.61847E+00_JPRB, 0.60964E+00_JPRB, 0.60266E+00_JPRB, 0.60817E+00_JPRB, 0.61682E+00_JPRB ,&
&  0.63158E+00_JPRB, 0.64476E+00_JPRB, 0.64790E+00_JPRB, 0.65049E+00_JPRB, 0.65183E+00_JPRB, 0.65133E+00_JPRB, 0.65279E+00_JPRB ,&
&  0.65152E+00_JPRB, 0.65151E+00_JPRB, 0.65933E+00_JPRB, 0.65979E+00_JPRB, 0.65022E+00_JPRB, 0.64653E+00_JPRB, 0.64343E+00_JPRB ,&
&  0.64103E+00_JPRB, 0.63906E+00_JPRB, 0.63612E+00_JPRB, 0.64708E+00_JPRB, 0.67397E+00_JPRB, 0.70455E+00_JPRB, 0.73074E+00_JPRB ,&
&  0.74655E+00_JPRB, 0.75976E+00_JPRB, 0.76312E+00_JPRB, 0.76448E+00_JPRB, 0.76374E+00_JPRB, 0.75442E+00_JPRB, 0.75049E+00_JPRB ,&
&  0.74266E+00_JPRB, 0.72316E+00_JPRB, 0.69680E+00_JPRB, 0.65623E+00_JPRB, 0.61653E+00_JPRB, 0.59335E+00_JPRB, 0.59225E+00_JPRB ,&
&  0.60407E+00_JPRB, 0.61659E+00_JPRB, 0.62281E+00_JPRB, 0.63037E+00_JPRB, 0.64421E+00_JPRB, 0.66108E+00_JPRB, 0.67951E+00_JPRB ,&
&  0.69117E+00_JPRB, 0.69165E+00_JPRB, 0.69011E+00_JPRB, 0.69915E+00_JPRB, 0.71103E+00_JPRB, 0.72992E+00_JPRB, 0.75051E+00_JPRB ,&
&  0.76832E+00_JPRB, 0.76851E+00_JPRB, 0.76934E+00_JPRB, 0.77121E+00_JPRB, 0.79188E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.79514E+00_JPRB, 0.79425E+00_JPRB, 0.76546E+00_JPRB, 0.73324E+00_JPRB, 0.71475E+00_JPRB, 0.69466E+00_JPRB, 0.67862E+00_JPRB ,&
&  0.67703E+00_JPRB, 0.67398E+00_JPRB, 0.67112E+00_JPRB, 0.66750E+00_JPRB, 0.66510E+00_JPRB, 0.66189E+00_JPRB, 0.64986E+00_JPRB ,&
&  0.64523E+00_JPRB, 0.65140E+00_JPRB, 0.67342E+00_JPRB, 0.69800E+00_JPRB, 0.71766E+00_JPRB, 0.74376E+00_JPRB, 0.75546E+00_JPRB ,&
&  0.76818E+00_JPRB, 0.78126E+00_JPRB, 0.78314E+00_JPRB, 0.74892E+00_JPRB, 0.72084E+00_JPRB, 0.72173E+00_JPRB, 0.73319E+00_JPRB ,&
&  0.73387E+00_JPRB, 0.73720E+00_JPRB, 0.73931E+00_JPRB, 0.73667E+00_JPRB, 0.71611E+00_JPRB, 0.67784E+00_JPRB, 0.64672E+00_JPRB ,&
&  0.62000E+00_JPRB, 0.59901E+00_JPRB, 0.59130E+00_JPRB, 0.57781E+00_JPRB, 0.57437E+00_JPRB, 0.58432E+00_JPRB, 0.59930E+00_JPRB ,&
&  0.61515E+00_JPRB, 0.62288E+00_JPRB, 0.63004E+00_JPRB, 0.62587E+00_JPRB, 0.62170E+00_JPRB, 0.62513E+00_JPRB, 0.62561E+00_JPRB ,&
&  0.62589E+00_JPRB, 0.61707E+00_JPRB, 0.61393E+00_JPRB, 0.60663E+00_JPRB, 0.60244E+00_JPRB, 0.60208E+00_JPRB, 0.60451E+00_JPRB ,&
&  0.60574E+00_JPRB, 0.61169E+00_JPRB, 0.61375E+00_JPRB, 0.61639E+00_JPRB, 0.62619E+00_JPRB, 0.64311E+00_JPRB, 0.65988E+00_JPRB ,&
&  0.67249E+00_JPRB, 0.68247E+00_JPRB, 0.68994E+00_JPRB, 0.69597E+00_JPRB, 0.69602E+00_JPRB, 0.69124E+00_JPRB, 0.67715E+00_JPRB ,&
&  0.64676E+00_JPRB, 0.61420E+00_JPRB, 0.58924E+00_JPRB, 0.56934E+00_JPRB, 0.56620E+00_JPRB, 0.57766E+00_JPRB, 0.59560E+00_JPRB ,&
&  0.60908E+00_JPRB, 0.61230E+00_JPRB, 0.60521E+00_JPRB, 0.60233E+00_JPRB, 0.60571E+00_JPRB, 0.61419E+00_JPRB, 0.62087E+00_JPRB ,&
&  0.61962E+00_JPRB, 0.61669E+00_JPRB, 0.62146E+00_JPRB, 0.63013E+00_JPRB, 0.64605E+00_JPRB, 0.66555E+00_JPRB, 0.68632E+00_JPRB ,&
&  0.70662E+00_JPRB, 0.72214E+00_JPRB, 0.74171E+00_JPRB, 0.76147E+00_JPRB, 0.78684E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.77526E+00_JPRB, 0.74723E+00_JPRB, 0.71758E+00_JPRB, 0.69250E+00_JPRB, 0.68112E+00_JPRB, 0.66410E+00_JPRB, 0.64959E+00_JPRB ,&
&  0.64865E+00_JPRB, 0.65616E+00_JPRB, 0.65076E+00_JPRB, 0.64398E+00_JPRB, 0.63442E+00_JPRB, 0.62690E+00_JPRB, 0.61815E+00_JPRB ,&
&  0.61400E+00_JPRB, 0.62476E+00_JPRB, 0.64986E+00_JPRB, 0.66194E+00_JPRB, 0.66762E+00_JPRB, 0.69779E+00_JPRB, 0.72943E+00_JPRB ,&
&  0.73941E+00_JPRB, 0.75403E+00_JPRB, 0.75785E+00_JPRB, 0.73131E+00_JPRB, 0.71393E+00_JPRB, 0.70720E+00_JPRB, 0.71094E+00_JPRB ,&
&  0.70652E+00_JPRB, 0.71076E+00_JPRB, 0.70180E+00_JPRB, 0.63367E+00_JPRB, 0.63237E+00_JPRB, 0.60199E+00_JPRB, 0.59086E+00_JPRB ,&
&  0.58437E+00_JPRB, 0.58817E+00_JPRB, 0.58266E+00_JPRB, 0.57173E+00_JPRB, 0.56733E+00_JPRB, 0.57082E+00_JPRB, 0.57350E+00_JPRB ,&
&  0.57067E+00_JPRB, 0.56100E+00_JPRB, 0.56584E+00_JPRB, 0.57220E+00_JPRB, 0.57575E+00_JPRB, 0.58306E+00_JPRB, 0.59388E+00_JPRB ,&
&  0.58381E+00_JPRB, 0.57391E+00_JPRB, 0.56528E+00_JPRB, 0.56611E+00_JPRB, 0.57320E+00_JPRB, 0.58717E+00_JPRB, 0.59123E+00_JPRB ,&
&  0.58890E+00_JPRB, 0.58793E+00_JPRB, 0.58958E+00_JPRB, 0.59644E+00_JPRB, 0.61091E+00_JPRB, 0.62513E+00_JPRB, 0.62621E+00_JPRB ,&
&  0.62761E+00_JPRB, 0.63273E+00_JPRB, 0.63879E+00_JPRB, 0.63983E+00_JPRB, 0.63515E+00_JPRB, 0.61967E+00_JPRB, 0.60193E+00_JPRB ,&
&  0.58525E+00_JPRB, 0.56694E+00_JPRB, 0.55202E+00_JPRB, 0.54821E+00_JPRB, 0.56285E+00_JPRB, 0.59344E+00_JPRB, 0.61075E+00_JPRB ,&
&  0.61529E+00_JPRB, 0.61740E+00_JPRB, 0.62000E+00_JPRB, 0.61987E+00_JPRB, 0.62072E+00_JPRB, 0.61778E+00_JPRB, 0.60996E+00_JPRB ,&
&  0.59791E+00_JPRB, 0.58733E+00_JPRB, 0.58630E+00_JPRB, 0.59300E+00_JPRB, 0.60041E+00_JPRB, 0.61502E+00_JPRB, 0.63368E+00_JPRB ,&
&  0.65708E+00_JPRB, 0.69591E+00_JPRB, 0.72816E+00_JPRB, 0.75288E+00_JPRB, 0.77026E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.75192E+00_JPRB, 0.72479E+00_JPRB, 0.70657E+00_JPRB, 0.69257E+00_JPRB, 0.67427E+00_JPRB, 0.66025E+00_JPRB, 0.65454E+00_JPRB ,&
&  0.66256E+00_JPRB, 0.66080E+00_JPRB, 0.65074E+00_JPRB, 0.64244E+00_JPRB, 0.62664E+00_JPRB, 0.61261E+00_JPRB, 0.60691E+00_JPRB ,&
&  0.61511E+00_JPRB, 0.63083E+00_JPRB, 0.65032E+00_JPRB, 0.65748E+00_JPRB, 0.65613E+00_JPRB, 0.66448E+00_JPRB, 0.68388E+00_JPRB ,&
&  0.71280E+00_JPRB, 0.73309E+00_JPRB, 0.73953E+00_JPRB, 0.72521E+00_JPRB, 0.70679E+00_JPRB, 0.69825E+00_JPRB, 0.68414E+00_JPRB ,&
&  0.66750E+00_JPRB, 0.66996E+00_JPRB, 0.60358E+00_JPRB, 0.59580E+00_JPRB, 0.57315E+00_JPRB, 0.55823E+00_JPRB, 0.55389E+00_JPRB ,&
&  0.56635E+00_JPRB, 0.56924E+00_JPRB, 0.55968E+00_JPRB, 0.54643E+00_JPRB, 0.53800E+00_JPRB, 0.52719E+00_JPRB, 0.51768E+00_JPRB ,&
&  0.51175E+00_JPRB, 0.51491E+00_JPRB, 0.52401E+00_JPRB, 0.53341E+00_JPRB, 0.54414E+00_JPRB, 0.55463E+00_JPRB, 0.55135E+00_JPRB ,&
&  0.53969E+00_JPRB, 0.53791E+00_JPRB, 0.54578E+00_JPRB, 0.56164E+00_JPRB, 0.57888E+00_JPRB, 0.58658E+00_JPRB, 0.58905E+00_JPRB ,&
&  0.57944E+00_JPRB, 0.56172E+00_JPRB, 0.54676E+00_JPRB, 0.53461E+00_JPRB, 0.50638E+00_JPRB, 0.44438E+00_JPRB, 0.42225E+00_JPRB ,&
&  0.43418E+00_JPRB, 0.49189E+00_JPRB, 0.55969E+00_JPRB, 0.55294E+00_JPRB, 0.55097E+00_JPRB, 0.53631E+00_JPRB, 0.49811E+00_JPRB ,&
&  0.45172E+00_JPRB, 0.39114E+00_JPRB, 0.37192E+00_JPRB, 0.36934E+00_JPRB, 0.36958E+00_JPRB, 0.37749E+00_JPRB, 0.39074E+00_JPRB ,&
&  0.40800E+00_JPRB, 0.43294E+00_JPRB, 0.44930E+00_JPRB, 0.45500E+00_JPRB, 0.45244E+00_JPRB, 0.43612E+00_JPRB, 0.42170E+00_JPRB ,&
&  0.41292E+00_JPRB, 0.43921E+00_JPRB, 0.50002E+00_JPRB, 0.58215E+00_JPRB, 0.58454E+00_JPRB, 0.61239E+00_JPRB, 0.63906E+00_JPRB ,&
&  0.66426E+00_JPRB, 0.70531E+00_JPRB, 0.73139E+00_JPRB, 0.73665E+00_JPRB, 0.75461E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.71072E+00_JPRB, 0.69775E+00_JPRB, 0.68870E+00_JPRB, 0.68227E+00_JPRB, 0.67493E+00_JPRB, 0.67323E+00_JPRB, 0.67223E+00_JPRB ,&
&  0.66622E+00_JPRB, 0.65774E+00_JPRB, 0.65888E+00_JPRB, 0.64211E+00_JPRB, 0.61798E+00_JPRB, 0.60183E+00_JPRB, 0.59960E+00_JPRB ,&
&  0.60327E+00_JPRB, 0.61029E+00_JPRB, 0.61907E+00_JPRB, 0.62684E+00_JPRB, 0.63139E+00_JPRB, 0.64298E+00_JPRB, 0.66368E+00_JPRB ,&
&  0.68764E+00_JPRB, 0.70108E+00_JPRB, 0.70479E+00_JPRB, 0.69614E+00_JPRB, 0.68353E+00_JPRB, 0.67070E+00_JPRB, 0.65929E+00_JPRB ,&
&  0.64774E+00_JPRB, 0.54617E+00_JPRB, 0.48059E+00_JPRB, 0.48860E+00_JPRB, 0.55349E+00_JPRB, 0.53690E+00_JPRB, 0.53633E+00_JPRB ,&
&  0.54875E+00_JPRB, 0.54757E+00_JPRB, 0.53350E+00_JPRB, 0.51598E+00_JPRB, 0.49983E+00_JPRB, 0.48837E+00_JPRB, 0.48070E+00_JPRB ,&
&  0.47769E+00_JPRB, 0.48499E+00_JPRB, 0.49261E+00_JPRB, 0.50432E+00_JPRB, 0.50660E+00_JPRB, 0.49306E+00_JPRB, 0.47673E+00_JPRB ,&
&  0.46130E+00_JPRB, 0.45564E+00_JPRB, 0.45841E+00_JPRB, 0.46612E+00_JPRB, 0.47623E+00_JPRB, 0.48409E+00_JPRB, 0.48143E+00_JPRB ,&
&  0.45367E+00_JPRB, 0.43014E+00_JPRB, 0.40183E+00_JPRB, 0.37503E+00_JPRB, 0.35192E+00_JPRB, 0.33658E+00_JPRB, 0.32838E+00_JPRB ,&
&  0.32685E+00_JPRB, 0.33582E+00_JPRB, 0.35819E+00_JPRB, 0.40750E+00_JPRB, 0.46899E+00_JPRB, 0.38931E+00_JPRB, 0.35100E+00_JPRB ,&
&  0.33042E+00_JPRB, 0.31699E+00_JPRB, 0.31182E+00_JPRB, 0.31304E+00_JPRB, 0.31807E+00_JPRB, 0.32510E+00_JPRB, 0.33064E+00_JPRB ,&
&  0.33517E+00_JPRB, 0.33854E+00_JPRB, 0.33953E+00_JPRB, 0.33980E+00_JPRB, 0.34089E+00_JPRB, 0.34196E+00_JPRB, 0.34073E+00_JPRB ,&
&  0.33974E+00_JPRB, 0.34740E+00_JPRB, 0.36689E+00_JPRB, 0.39702E+00_JPRB, 0.43116E+00_JPRB, 0.46876E+00_JPRB, 0.52195E+00_JPRB ,&
&  0.58740E+00_JPRB, 0.65923E+00_JPRB, 0.67929E+00_JPRB, 0.69418E+00_JPRB, 0.70507E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.65023E+00_JPRB, 0.65344E+00_JPRB, 0.65677E+00_JPRB, 0.65845E+00_JPRB, 0.65266E+00_JPRB, 0.64498E+00_JPRB, 0.64032E+00_JPRB ,&
&  0.64559E+00_JPRB, 0.64598E+00_JPRB, 0.62306E+00_JPRB, 0.60547E+00_JPRB, 0.59556E+00_JPRB, 0.59196E+00_JPRB, 0.58926E+00_JPRB ,&
&  0.58294E+00_JPRB, 0.58061E+00_JPRB, 0.58135E+00_JPRB, 0.59194E+00_JPRB, 0.60080E+00_JPRB, 0.60846E+00_JPRB, 0.61559E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.62132E+00_JPRB, 0.64241E+00_JPRB, 0.64088E+00_JPRB, 0.63756E+00_JPRB, 0.62984E+00_JPRB, 0.61486E+00_JPRB ,&
&  0.56514E+00_JPRB, 0.50081E+00_JPRB, 0.44251E+00_JPRB, 0.45563E+00_JPRB, 0.52919E+00_JPRB, 0.53544E+00_JPRB, 0.52237E+00_JPRB ,&
&  0.53327E+00_JPRB, 0.52794E+00_JPRB, 0.50972E+00_JPRB, 0.49589E+00_JPRB, 0.48014E+00_JPRB, 0.46994E+00_JPRB, 0.46238E+00_JPRB ,&
&  0.45920E+00_JPRB, 0.45885E+00_JPRB, 0.44235E+00_JPRB, 0.39184E+00_JPRB, 0.36414E+00_JPRB, 0.34653E+00_JPRB, 0.32815E+00_JPRB ,&
&  0.31332E+00_JPRB, 0.30476E+00_JPRB, 0.30395E+00_JPRB, 0.30435E+00_JPRB, 0.31016E+00_JPRB, 0.31356E+00_JPRB, 0.31664E+00_JPRB ,&
&  0.31726E+00_JPRB, 0.31639E+00_JPRB, 0.31249E+00_JPRB, 0.30807E+00_JPRB, 0.30489E+00_JPRB, 0.30473E+00_JPRB, 0.30429E+00_JPRB ,&
&  0.30717E+00_JPRB, 0.31098E+00_JPRB, 0.32831E+00_JPRB, 0.34635E+00_JPRB, 0.34226E+00_JPRB, 0.32715E+00_JPRB, 0.31343E+00_JPRB ,&
&  0.30263E+00_JPRB, 0.29725E+00_JPRB, 0.29369E+00_JPRB, 0.29315E+00_JPRB, 0.29666E+00_JPRB, 0.30176E+00_JPRB, 0.30297E+00_JPRB ,&
&  0.30260E+00_JPRB, 0.30316E+00_JPRB, 0.30308E+00_JPRB, 0.30341E+00_JPRB, 0.30390E+00_JPRB, 0.30672E+00_JPRB, 0.31217E+00_JPRB ,&
&  0.31851E+00_JPRB, 0.32998E+00_JPRB, 0.34274E+00_JPRB, 0.35980E+00_JPRB, 0.37233E+00_JPRB, 0.38208E+00_JPRB, 0.39610E+00_JPRB ,&
&  0.41498E+00_JPRB, 0.45553E+00_JPRB, 0.52468E+00_JPRB, 0.64798E+00_JPRB, 0.64111E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.61672E+00_JPRB, 0.62777E+00_JPRB, 0.63937E+00_JPRB, 0.63078E+00_JPRB, 0.61743E+00_JPRB, 0.60990E+00_JPRB, 0.60987E+00_JPRB ,&
&  0.61002E+00_JPRB, 0.60724E+00_JPRB, 0.59547E+00_JPRB, 0.58137E+00_JPRB, 0.56301E+00_JPRB, 0.51560E+00_JPRB, 0.49884E+00_JPRB ,&
&  0.48154E+00_JPRB, 0.48429E+00_JPRB, 0.48806E+00_JPRB, 0.49382E+00_JPRB, 0.50677E+00_JPRB, 0.51697E+00_JPRB, 0.51798E+00_JPRB ,&
&  0.50707E+00_JPRB, 0.48835E+00_JPRB, 0.47001E+00_JPRB, 0.45613E+00_JPRB, 0.45563E+00_JPRB, 0.45511E+00_JPRB, 0.44759E+00_JPRB ,&
&  0.43417E+00_JPRB, 0.42866E+00_JPRB, 0.42870E+00_JPRB, 0.44832E+00_JPRB, 0.49827E+00_JPRB, 0.51610E+00_JPRB, 0.49258E+00_JPRB ,&
&  0.49715E+00_JPRB, 0.49168E+00_JPRB, 0.48737E+00_JPRB, 0.48811E+00_JPRB, 0.47721E+00_JPRB, 0.45477E+00_JPRB, 0.42070E+00_JPRB ,&
&  0.37899E+00_JPRB, 0.35740E+00_JPRB, 0.33617E+00_JPRB, 0.31514E+00_JPRB, 0.29790E+00_JPRB, 0.28808E+00_JPRB, 0.27976E+00_JPRB ,&
&  0.27442E+00_JPRB, 0.26978E+00_JPRB, 0.26685E+00_JPRB, 0.26714E+00_JPRB, 0.26937E+00_JPRB, 0.27060E+00_JPRB, 0.27203E+00_JPRB ,&
&  0.27402E+00_JPRB, 0.27535E+00_JPRB, 0.27620E+00_JPRB, 0.27805E+00_JPRB, 0.27990E+00_JPRB, 0.28312E+00_JPRB, 0.28720E+00_JPRB ,&
&  0.29230E+00_JPRB, 0.29607E+00_JPRB, 0.29961E+00_JPRB, 0.30656E+00_JPRB, 0.30723E+00_JPRB, 0.30575E+00_JPRB, 0.29742E+00_JPRB ,&
&  0.29121E+00_JPRB, 0.28856E+00_JPRB, 0.28576E+00_JPRB, 0.28505E+00_JPRB, 0.28644E+00_JPRB, 0.28877E+00_JPRB, 0.29114E+00_JPRB ,&
&  0.29284E+00_JPRB, 0.29287E+00_JPRB, 0.29182E+00_JPRB, 0.29152E+00_JPRB, 0.29197E+00_JPRB, 0.29204E+00_JPRB, 0.29611E+00_JPRB ,&
&  0.30515E+00_JPRB, 0.31823E+00_JPRB, 0.33273E+00_JPRB, 0.35209E+00_JPRB, 0.37059E+00_JPRB, 0.38757E+00_JPRB, 0.41587E+00_JPRB ,&
&  0.45408E+00_JPRB, 0.55928E+00_JPRB, 0.62700E+00_JPRB, 0.57097E+00_JPRB, 0.59642E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.58969E+00_JPRB, 0.59720E+00_JPRB, 0.60755E+00_JPRB, 0.61193E+00_JPRB, 0.61961E+00_JPRB, 0.60790E+00_JPRB, 0.60042E+00_JPRB ,&
&  0.57010E+00_JPRB, 0.54929E+00_JPRB, 0.51873E+00_JPRB, 0.48161E+00_JPRB, 0.45650E+00_JPRB, 0.43739E+00_JPRB, 0.42978E+00_JPRB ,&
&  0.41895E+00_JPRB, 0.41259E+00_JPRB, 0.40540E+00_JPRB, 0.40228E+00_JPRB, 0.39939E+00_JPRB, 0.39960E+00_JPRB, 0.39846E+00_JPRB ,&
&  0.39424E+00_JPRB, 0.38849E+00_JPRB, 0.38153E+00_JPRB, 0.38372E+00_JPRB, 0.39646E+00_JPRB, 0.41098E+00_JPRB, 0.42847E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.45907E+00_JPRB, 0.46944E+00_JPRB, 0.47335E+00_JPRB, 0.45784E+00_JPRB, 0.46054E+00_JPRB, 0.46457E+00_JPRB ,&
&  0.47246E+00_JPRB, 0.47770E+00_JPRB, 0.47392E+00_JPRB, 0.46793E+00_JPRB, 0.41590E+00_JPRB, 0.38380E+00_JPRB, 0.35501E+00_JPRB ,&
&  0.33368E+00_JPRB, 0.32020E+00_JPRB, 0.30773E+00_JPRB, 0.29695E+00_JPRB, 0.28508E+00_JPRB, 0.27768E+00_JPRB, 0.27218E+00_JPRB ,&
&  0.26787E+00_JPRB, 0.26623E+00_JPRB, 0.26477E+00_JPRB, 0.26455E+00_JPRB, 0.26542E+00_JPRB, 0.26494E+00_JPRB, 0.26519E+00_JPRB ,&
&  0.26538E+00_JPRB, 0.26506E+00_JPRB, 0.26566E+00_JPRB, 0.26748E+00_JPRB, 0.26987E+00_JPRB, 0.27361E+00_JPRB, 0.27449E+00_JPRB ,&
&  0.27568E+00_JPRB, 0.27670E+00_JPRB, 0.27685E+00_JPRB, 0.27914E+00_JPRB, 0.28045E+00_JPRB, 0.28059E+00_JPRB, 0.27899E+00_JPRB ,&
&  0.27723E+00_JPRB, 0.27585E+00_JPRB, 0.27603E+00_JPRB, 0.27680E+00_JPRB, 0.27871E+00_JPRB, 0.28149E+00_JPRB, 0.28198E+00_JPRB ,&
&  0.28293E+00_JPRB, 0.28229E+00_JPRB, 0.28159E+00_JPRB, 0.28407E+00_JPRB, 0.28690E+00_JPRB, 0.28989E+00_JPRB, 0.29377E+00_JPRB ,&
&  0.29535E+00_JPRB, 0.30320E+00_JPRB, 0.31797E+00_JPRB, 0.34090E+00_JPRB, 0.37899E+00_JPRB, 0.39616E+00_JPRB, 0.43620E+00_JPRB ,&
&  0.49120E+00_JPRB, 0.62676E+00_JPRB, 0.69098E+00_JPRB, 0.55626E+00_JPRB, 0.57385E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.53050E+00_JPRB, 0.54290E+00_JPRB, 0.55923E+00_JPRB, 0.56941E+00_JPRB, 0.58258E+00_JPRB, 0.58473E+00_JPRB, 0.57751E+00_JPRB ,&
&  0.56002E+00_JPRB, 0.53246E+00_JPRB, 0.51622E+00_JPRB, 0.49474E+00_JPRB, 0.47357E+00_JPRB, 0.45164E+00_JPRB, 0.44171E+00_JPRB ,&
&  0.42970E+00_JPRB, 0.41552E+00_JPRB, 0.40127E+00_JPRB, 0.38870E+00_JPRB, 0.37654E+00_JPRB, 0.36549E+00_JPRB, 0.35931E+00_JPRB ,&
&  0.35298E+00_JPRB, 0.35282E+00_JPRB, 0.35228E+00_JPRB, 0.35624E+00_JPRB, 0.36787E+00_JPRB, 0.38632E+00_JPRB, 0.41226E+00_JPRB ,&
&  0.45059E+00_JPRB, 0.45393E+00_JPRB, 0.43953E+00_JPRB, 0.43342E+00_JPRB, 0.42590E+00_JPRB, 0.41985E+00_JPRB, 0.40949E+00_JPRB ,&
&  0.41708E+00_JPRB, 0.42587E+00_JPRB, 0.44076E+00_JPRB, 0.42664E+00_JPRB, 0.38009E+00_JPRB, 0.36090E+00_JPRB, 0.33712E+00_JPRB ,&
&  0.32636E+00_JPRB, 0.31652E+00_JPRB, 0.30842E+00_JPRB, 0.30002E+00_JPRB, 0.28977E+00_JPRB, 0.28389E+00_JPRB, 0.27744E+00_JPRB ,&
&  0.27335E+00_JPRB, 0.27217E+00_JPRB, 0.26988E+00_JPRB, 0.27048E+00_JPRB, 0.27075E+00_JPRB, 0.27022E+00_JPRB, 0.26990E+00_JPRB ,&
&  0.26901E+00_JPRB, 0.26803E+00_JPRB, 0.26777E+00_JPRB, 0.26860E+00_JPRB, 0.26941E+00_JPRB, 0.27017E+00_JPRB, 0.26758E+00_JPRB ,&
&  0.26632E+00_JPRB, 0.26511E+00_JPRB, 0.26510E+00_JPRB, 0.26702E+00_JPRB, 0.26769E+00_JPRB, 0.26899E+00_JPRB, 0.26938E+00_JPRB ,&
&  0.26969E+00_JPRB, 0.27049E+00_JPRB, 0.27010E+00_JPRB, 0.27101E+00_JPRB, 0.27128E+00_JPRB, 0.27254E+00_JPRB, 0.27322E+00_JPRB ,&
&  0.27398E+00_JPRB, 0.27631E+00_JPRB, 0.27873E+00_JPRB, 0.28244E+00_JPRB, 0.28742E+00_JPRB, 0.29024E+00_JPRB, 0.29500E+00_JPRB ,&
&  0.30030E+00_JPRB, 0.31000E+00_JPRB, 0.32962E+00_JPRB, 0.35152E+00_JPRB, 0.37927E+00_JPRB, 0.39009E+00_JPRB, 0.39583E+00_JPRB ,&
&  0.40545E+00_JPRB, 0.43491E+00_JPRB, 0.48562E+00_JPRB, 0.51300E+00_JPRB, 0.52157E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.51971E+00_JPRB, 0.52270E+00_JPRB, 0.52496E+00_JPRB, 0.52269E+00_JPRB, 0.52292E+00_JPRB, 0.52326E+00_JPRB, 0.52065E+00_JPRB ,&
&  0.51250E+00_JPRB, 0.51060E+00_JPRB, 0.50536E+00_JPRB, 0.49729E+00_JPRB, 0.48910E+00_JPRB, 0.47548E+00_JPRB, 0.45879E+00_JPRB ,&
&  0.43654E+00_JPRB, 0.41660E+00_JPRB, 0.40201E+00_JPRB, 0.38785E+00_JPRB, 0.37407E+00_JPRB, 0.36062E+00_JPRB, 0.35618E+00_JPRB ,&
&  0.35188E+00_JPRB, 0.35029E+00_JPRB, 0.35010E+00_JPRB, 0.35320E+00_JPRB, 0.36057E+00_JPRB, 0.36984E+00_JPRB, 0.38013E+00_JPRB ,&
&  0.39116E+00_JPRB, 0.40162E+00_JPRB, 0.40253E+00_JPRB, 0.39452E+00_JPRB, 0.38755E+00_JPRB, 0.38348E+00_JPRB, 0.38013E+00_JPRB ,&
&  0.37622E+00_JPRB, 0.36821E+00_JPRB, 0.35852E+00_JPRB, 0.34811E+00_JPRB, 0.33768E+00_JPRB, 0.33005E+00_JPRB, 0.32112E+00_JPRB ,&
&  0.31451E+00_JPRB, 0.30827E+00_JPRB, 0.30131E+00_JPRB, 0.29527E+00_JPRB, 0.29114E+00_JPRB, 0.28781E+00_JPRB, 0.28433E+00_JPRB ,&
&  0.28288E+00_JPRB, 0.28143E+00_JPRB, 0.27985E+00_JPRB, 0.27845E+00_JPRB, 0.27653E+00_JPRB, 0.27519E+00_JPRB, 0.27399E+00_JPRB ,&
&  0.27313E+00_JPRB, 0.27232E+00_JPRB, 0.27187E+00_JPRB, 0.27133E+00_JPRB, 0.27084E+00_JPRB, 0.26965E+00_JPRB, 0.26828E+00_JPRB ,&
&  0.26756E+00_JPRB, 0.26763E+00_JPRB, 0.26686E+00_JPRB, 0.26655E+00_JPRB, 0.26626E+00_JPRB, 0.26538E+00_JPRB, 0.26602E+00_JPRB ,&
&  0.26752E+00_JPRB, 0.26915E+00_JPRB, 0.26924E+00_JPRB, 0.27031E+00_JPRB, 0.27158E+00_JPRB, 0.27322E+00_JPRB, 0.27605E+00_JPRB ,&
&  0.27814E+00_JPRB, 0.28033E+00_JPRB, 0.28316E+00_JPRB, 0.28546E+00_JPRB, 0.28924E+00_JPRB, 0.29393E+00_JPRB, 0.29977E+00_JPRB ,&
&  0.30779E+00_JPRB, 0.31751E+00_JPRB, 0.33162E+00_JPRB, 0.34904E+00_JPRB, 0.37484E+00_JPRB, 0.38250E+00_JPRB, 0.38913E+00_JPRB ,&
&  0.39347E+00_JPRB, 0.40130E+00_JPRB, 0.48424E+00_JPRB, 0.49831E+00_JPRB, 0.51329E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.39728E+00_JPRB, 0.40685E+00_JPRB, 0.41436E+00_JPRB, 0.42117E+00_JPRB, 0.41609E+00_JPRB, 0.48075E+00_JPRB, 0.45936E+00_JPRB ,&
&  0.44914E+00_JPRB, 0.42299E+00_JPRB, 0.42558E+00_JPRB, 0.42230E+00_JPRB, 0.41655E+00_JPRB, 0.41074E+00_JPRB, 0.40567E+00_JPRB ,&
&  0.40039E+00_JPRB, 0.39306E+00_JPRB, 0.38905E+00_JPRB, 0.38356E+00_JPRB, 0.37818E+00_JPRB, 0.37261E+00_JPRB, 0.36922E+00_JPRB ,&
&  0.36669E+00_JPRB, 0.36522E+00_JPRB, 0.36409E+00_JPRB, 0.36152E+00_JPRB, 0.36308E+00_JPRB, 0.36356E+00_JPRB, 0.36102E+00_JPRB ,&
&  0.35633E+00_JPRB, 0.35214E+00_JPRB, 0.34725E+00_JPRB, 0.34230E+00_JPRB, 0.34020E+00_JPRB, 0.33811E+00_JPRB, 0.33584E+00_JPRB ,&
&  0.33252E+00_JPRB, 0.32874E+00_JPRB, 0.32459E+00_JPRB, 0.32113E+00_JPRB, 0.31766E+00_JPRB, 0.31395E+00_JPRB, 0.31033E+00_JPRB ,&
&  0.30643E+00_JPRB, 0.30252E+00_JPRB, 0.29859E+00_JPRB, 0.29613E+00_JPRB, 0.29408E+00_JPRB, 0.29221E+00_JPRB, 0.29050E+00_JPRB ,&
&  0.28991E+00_JPRB, 0.28916E+00_JPRB, 0.28820E+00_JPRB, 0.28697E+00_JPRB, 0.28591E+00_JPRB, 0.28476E+00_JPRB, 0.28388E+00_JPRB ,&
&  0.28386E+00_JPRB, 0.28327E+00_JPRB, 0.28266E+00_JPRB, 0.28184E+00_JPRB, 0.28149E+00_JPRB, 0.28056E+00_JPRB, 0.27939E+00_JPRB ,&
&  0.27836E+00_JPRB, 0.27742E+00_JPRB, 0.27728E+00_JPRB, 0.27709E+00_JPRB, 0.27690E+00_JPRB, 0.27670E+00_JPRB, 0.27719E+00_JPRB ,&
&  0.27780E+00_JPRB, 0.27858E+00_JPRB, 0.27982E+00_JPRB, 0.28121E+00_JPRB, 0.28245E+00_JPRB, 0.28369E+00_JPRB, 0.28544E+00_JPRB ,&
&  0.28760E+00_JPRB, 0.28978E+00_JPRB, 0.29225E+00_JPRB, 0.29428E+00_JPRB, 0.29806E+00_JPRB, 0.30219E+00_JPRB, 0.30680E+00_JPRB ,&
&  0.31230E+00_JPRB, 0.31998E+00_JPRB, 0.32911E+00_JPRB, 0.33750E+00_JPRB, 0.34719E+00_JPRB, 0.35621E+00_JPRB, 0.36359E+00_JPRB ,&
&  0.36993E+00_JPRB, 0.37646E+00_JPRB, 0.38090E+00_JPRB, 0.38612E+00_JPRB, 0.39170E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.35591E+00_JPRB, 0.35915E+00_JPRB, 0.36162E+00_JPRB, 0.36314E+00_JPRB, 0.36255E+00_JPRB, 0.36366E+00_JPRB, 0.36344E+00_JPRB ,&
&  0.36296E+00_JPRB, 0.36233E+00_JPRB, 0.36301E+00_JPRB, 0.36378E+00_JPRB, 0.36424E+00_JPRB, 0.36456E+00_JPRB, 0.36462E+00_JPRB ,&
&  0.36400E+00_JPRB, 0.36288E+00_JPRB, 0.35779E+00_JPRB, 0.35492E+00_JPRB, 0.35310E+00_JPRB, 0.35206E+00_JPRB, 0.35140E+00_JPRB ,&
&  0.35019E+00_JPRB, 0.34885E+00_JPRB, 0.34689E+00_JPRB, 0.34673E+00_JPRB, 0.34592E+00_JPRB, 0.34483E+00_JPRB, 0.34323E+00_JPRB ,&
&  0.34164E+00_JPRB, 0.34040E+00_JPRB, 0.33892E+00_JPRB, 0.33740E+00_JPRB, 0.33566E+00_JPRB, 0.33349E+00_JPRB, 0.33060E+00_JPRB ,&
&  0.32764E+00_JPRB, 0.32468E+00_JPRB, 0.32221E+00_JPRB, 0.31985E+00_JPRB, 0.31781E+00_JPRB, 0.31612E+00_JPRB, 0.31481E+00_JPRB ,&
&  0.31336E+00_JPRB, 0.31167E+00_JPRB, 0.30996E+00_JPRB, 0.30852E+00_JPRB, 0.30710E+00_JPRB, 0.30587E+00_JPRB, 0.30518E+00_JPRB ,&
&  0.30479E+00_JPRB, 0.30432E+00_JPRB, 0.30375E+00_JPRB, 0.30319E+00_JPRB, 0.30252E+00_JPRB, 0.30189E+00_JPRB, 0.30127E+00_JPRB ,&
&  0.30072E+00_JPRB, 0.30045E+00_JPRB, 0.30004E+00_JPRB, 0.29961E+00_JPRB, 0.29934E+00_JPRB, 0.29889E+00_JPRB, 0.29840E+00_JPRB ,&
&  0.29785E+00_JPRB, 0.29762E+00_JPRB, 0.29733E+00_JPRB, 0.29695E+00_JPRB, 0.29659E+00_JPRB, 0.29634E+00_JPRB, 0.29638E+00_JPRB ,&
&  0.29636E+00_JPRB, 0.29626E+00_JPRB, 0.29634E+00_JPRB, 0.29720E+00_JPRB, 0.29813E+00_JPRB, 0.29925E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.30209E+00_JPRB, 0.30376E+00_JPRB, 0.30567E+00_JPRB, 0.30749E+00_JPRB, 0.30998E+00_JPRB, 0.31245E+00_JPRB, 0.31494E+00_JPRB ,&
&  0.31734E+00_JPRB, 0.31985E+00_JPRB, 0.32242E+00_JPRB, 0.32512E+00_JPRB, 0.32879E+00_JPRB, 0.33162E+00_JPRB, 0.33482E+00_JPRB ,&
&  0.33789E+00_JPRB, 0.34143E+00_JPRB, 0.34488E+00_JPRB, 0.34798E+00_JPRB, 0.35098E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB, 0.32262E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB ,&
&  0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB, 0.52525E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.53321E+00_JPRB, 0.53318E+00_JPRB, 0.53320E+00_JPRB, 0.53336E+00_JPRB, 0.53316E+00_JPRB, 0.53271E+00_JPRB, 0.53201E+00_JPRB ,&
&  0.53121E+00_JPRB, 0.53034E+00_JPRB, 0.52918E+00_JPRB, 0.52796E+00_JPRB, 0.52677E+00_JPRB, 0.52557E+00_JPRB, 0.52397E+00_JPRB ,&
&  0.52222E+00_JPRB, 0.52021E+00_JPRB, 0.51812E+00_JPRB, 0.51610E+00_JPRB, 0.51470E+00_JPRB, 0.51360E+00_JPRB, 0.51287E+00_JPRB ,&
&  0.51211E+00_JPRB, 0.51148E+00_JPRB, 0.51089E+00_JPRB, 0.51050E+00_JPRB, 0.51037E+00_JPRB, 0.51010E+00_JPRB, 0.50990E+00_JPRB ,&
&  0.50972E+00_JPRB, 0.50974E+00_JPRB, 0.50955E+00_JPRB, 0.50936E+00_JPRB, 0.50913E+00_JPRB, 0.50933E+00_JPRB, 0.50939E+00_JPRB ,&
&  0.50966E+00_JPRB, 0.50983E+00_JPRB, 0.50996E+00_JPRB, 0.50998E+00_JPRB, 0.51011E+00_JPRB, 0.51020E+00_JPRB, 0.51024E+00_JPRB ,&
&  0.51036E+00_JPRB, 0.51076E+00_JPRB, 0.51172E+00_JPRB, 0.51303E+00_JPRB, 0.51432E+00_JPRB, 0.51574E+00_JPRB, 0.51718E+00_JPRB ,&
&  0.51866E+00_JPRB, 0.51993E+00_JPRB, 0.52149E+00_JPRB, 0.52299E+00_JPRB, 0.52473E+00_JPRB, 0.52633E+00_JPRB, 0.52805E+00_JPRB ,&
&  0.52966E+00_JPRB, 0.53146E+00_JPRB, 0.53353E+00_JPRB, 0.53569E+00_JPRB, 0.53775E+00_JPRB, 0.53956E+00_JPRB, 0.54150E+00_JPRB ,&
&  0.54353E+00_JPRB, 0.54547E+00_JPRB, 0.54692E+00_JPRB, 0.54851E+00_JPRB, 0.55015E+00_JPRB, 0.55179E+00_JPRB, 0.55297E+00_JPRB ,&
&  0.55419E+00_JPRB, 0.55555E+00_JPRB, 0.55690E+00_JPRB, 0.55730E+00_JPRB, 0.55759E+00_JPRB, 0.55771E+00_JPRB, 0.55749E+00_JPRB ,&
&  0.55657E+00_JPRB, 0.55555E+00_JPRB, 0.55450E+00_JPRB, 0.55331E+00_JPRB, 0.55175E+00_JPRB, 0.55001E+00_JPRB, 0.54800E+00_JPRB ,&
&  0.54592E+00_JPRB, 0.54342E+00_JPRB, 0.54104E+00_JPRB, 0.53931E+00_JPRB, 0.53819E+00_JPRB, 0.53692E+00_JPRB, 0.53562E+00_JPRB ,&
&  0.53444E+00_JPRB, 0.53401E+00_JPRB, 0.53378E+00_JPRB, 0.53354E+00_JPRB, 0.53342E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.53644E+00_JPRB, 0.53679E+00_JPRB, 0.53735E+00_JPRB, 0.53753E+00_JPRB, 0.53643E+00_JPRB, 0.53539E+00_JPRB, 0.53424E+00_JPRB ,&
&  0.53342E+00_JPRB, 0.53259E+00_JPRB, 0.53131E+00_JPRB, 0.52970E+00_JPRB, 0.52811E+00_JPRB, 0.52653E+00_JPRB, 0.52490E+00_JPRB ,&
&  0.52338E+00_JPRB, 0.52174E+00_JPRB, 0.51979E+00_JPRB, 0.51624E+00_JPRB, 0.51292E+00_JPRB, 0.51028E+00_JPRB, 0.50922E+00_JPRB ,&
&  0.50796E+00_JPRB, 0.50663E+00_JPRB, 0.50464E+00_JPRB, 0.50230E+00_JPRB, 0.49991E+00_JPRB, 0.49773E+00_JPRB, 0.49548E+00_JPRB ,&
&  0.49359E+00_JPRB, 0.49195E+00_JPRB, 0.49068E+00_JPRB, 0.48920E+00_JPRB, 0.48974E+00_JPRB, 0.49036E+00_JPRB, 0.49088E+00_JPRB ,&
&  0.49140E+00_JPRB, 0.49147E+00_JPRB, 0.49264E+00_JPRB, 0.49385E+00_JPRB, 0.49502E+00_JPRB, 0.49686E+00_JPRB, 0.49793E+00_JPRB ,&
&  0.49810E+00_JPRB, 0.49855E+00_JPRB, 0.50119E+00_JPRB, 0.50322E+00_JPRB, 0.50485E+00_JPRB, 0.50631E+00_JPRB, 0.50883E+00_JPRB ,&
&  0.51132E+00_JPRB, 0.51382E+00_JPRB, 0.51658E+00_JPRB, 0.52015E+00_JPRB, 0.52353E+00_JPRB, 0.52727E+00_JPRB, 0.53117E+00_JPRB ,&
&  0.53488E+00_JPRB, 0.53878E+00_JPRB, 0.54275E+00_JPRB, 0.54661E+00_JPRB, 0.55014E+00_JPRB, 0.55318E+00_JPRB, 0.55667E+00_JPRB ,&
&  0.56037E+00_JPRB, 0.56456E+00_JPRB, 0.56719E+00_JPRB, 0.57001E+00_JPRB, 0.57313E+00_JPRB, 0.57742E+00_JPRB, 0.57919E+00_JPRB ,&
&  0.58085E+00_JPRB, 0.58241E+00_JPRB, 0.58413E+00_JPRB, 0.58491E+00_JPRB, 0.58570E+00_JPRB, 0.58649E+00_JPRB, 0.58705E+00_JPRB ,&
&  0.58691E+00_JPRB, 0.58597E+00_JPRB, 0.58416E+00_JPRB, 0.58142E+00_JPRB, 0.57814E+00_JPRB, 0.57481E+00_JPRB, 0.57074E+00_JPRB ,&
&  0.56705E+00_JPRB, 0.56245E+00_JPRB, 0.55805E+00_JPRB, 0.55492E+00_JPRB, 0.55284E+00_JPRB, 0.54935E+00_JPRB, 0.54564E+00_JPRB ,&
&  0.54237E+00_JPRB, 0.53984E+00_JPRB, 0.53864E+00_JPRB, 0.53756E+00_JPRB, 0.53700E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.54072E+00_JPRB, 0.54111E+00_JPRB, 0.54132E+00_JPRB, 0.54105E+00_JPRB, 0.53941E+00_JPRB, 0.53796E+00_JPRB, 0.53639E+00_JPRB ,&
&  0.53513E+00_JPRB, 0.53348E+00_JPRB, 0.53213E+00_JPRB, 0.53095E+00_JPRB, 0.52992E+00_JPRB, 0.52923E+00_JPRB, 0.52854E+00_JPRB ,&
&  0.52815E+00_JPRB, 0.52737E+00_JPRB, 0.52676E+00_JPRB, 0.52515E+00_JPRB, 0.52201E+00_JPRB, 0.51710E+00_JPRB, 0.51188E+00_JPRB ,&
&  0.50679E+00_JPRB, 0.50207E+00_JPRB, 0.49697E+00_JPRB, 0.49191E+00_JPRB, 0.48753E+00_JPRB, 0.46032E+00_JPRB, 0.45235E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.45472E+00_JPRB, 0.45839E+00_JPRB, 0.46319E+00_JPRB, 0.46573E+00_JPRB, 0.45893E+00_JPRB, 0.45579E+00_JPRB ,&
&  0.43697E+00_JPRB, 0.43918E+00_JPRB, 0.44139E+00_JPRB, 0.44405E+00_JPRB, 0.44962E+00_JPRB, 0.45448E+00_JPRB, 0.46294E+00_JPRB ,&
&  0.48177E+00_JPRB, 0.48526E+00_JPRB, 0.48914E+00_JPRB, 0.49366E+00_JPRB, 0.49745E+00_JPRB, 0.50084E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.50790E+00_JPRB, 0.51131E+00_JPRB, 0.51484E+00_JPRB, 0.51894E+00_JPRB, 0.52376E+00_JPRB, 0.52896E+00_JPRB, 0.53437E+00_JPRB ,&
&  0.53950E+00_JPRB, 0.54555E+00_JPRB, 0.55183E+00_JPRB, 0.55812E+00_JPRB, 0.56424E+00_JPRB, 0.57025E+00_JPRB, 0.57672E+00_JPRB ,&
&  0.58277E+00_JPRB, 0.58918E+00_JPRB, 0.59375E+00_JPRB, 0.59867E+00_JPRB, 0.60204E+00_JPRB, 0.60593E+00_JPRB, 0.60696E+00_JPRB ,&
&  0.60838E+00_JPRB, 0.60674E+00_JPRB, 0.60661E+00_JPRB, 0.60550E+00_JPRB, 0.60458E+00_JPRB, 0.60417E+00_JPRB, 0.60334E+00_JPRB ,&
&  0.60310E+00_JPRB, 0.60196E+00_JPRB, 0.60081E+00_JPRB, 0.59830E+00_JPRB, 0.59436E+00_JPRB, 0.58938E+00_JPRB, 0.58401E+00_JPRB ,&
&  0.58008E+00_JPRB, 0.57723E+00_JPRB, 0.57476E+00_JPRB, 0.57280E+00_JPRB, 0.56933E+00_JPRB, 0.56462E+00_JPRB, 0.55922E+00_JPRB ,&
&  0.55465E+00_JPRB, 0.55010E+00_JPRB, 0.54647E+00_JPRB, 0.54341E+00_JPRB, 0.54166E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.54757E+00_JPRB, 0.54866E+00_JPRB, 0.54928E+00_JPRB, 0.54958E+00_JPRB, 0.54825E+00_JPRB, 0.54698E+00_JPRB, 0.54564E+00_JPRB ,&
&  0.54418E+00_JPRB, 0.54279E+00_JPRB, 0.54130E+00_JPRB, 0.54013E+00_JPRB, 0.53866E+00_JPRB, 0.53755E+00_JPRB, 0.53570E+00_JPRB ,&
&  0.53490E+00_JPRB, 0.53321E+00_JPRB, 0.53136E+00_JPRB, 0.52742E+00_JPRB, 0.52343E+00_JPRB, 0.51862E+00_JPRB, 0.51025E+00_JPRB ,&
&  0.50106E+00_JPRB, 0.49264E+00_JPRB, 0.47055E+00_JPRB, 0.46478E+00_JPRB, 0.45359E+00_JPRB, 0.44729E+00_JPRB, 0.44411E+00_JPRB ,&
&  0.43934E+00_JPRB, 0.43604E+00_JPRB, 0.42698E+00_JPRB, 0.41901E+00_JPRB, 0.41312E+00_JPRB, 0.40317E+00_JPRB, 0.39809E+00_JPRB ,&
&  0.39183E+00_JPRB, 0.38890E+00_JPRB, 0.39016E+00_JPRB, 0.39387E+00_JPRB, 0.40346E+00_JPRB, 0.41464E+00_JPRB, 0.42970E+00_JPRB ,&
&  0.45076E+00_JPRB, 0.46975E+00_JPRB, 0.48787E+00_JPRB, 0.48972E+00_JPRB, 0.49591E+00_JPRB, 0.50139E+00_JPRB, 0.50527E+00_JPRB ,&
&  0.50230E+00_JPRB, 0.50219E+00_JPRB, 0.51211E+00_JPRB, 0.51623E+00_JPRB, 0.52112E+00_JPRB, 0.52675E+00_JPRB, 0.53255E+00_JPRB ,&
&  0.53880E+00_JPRB, 0.54584E+00_JPRB, 0.55320E+00_JPRB, 0.56198E+00_JPRB, 0.57260E+00_JPRB, 0.58316E+00_JPRB, 0.59220E+00_JPRB ,&
&  0.60025E+00_JPRB, 0.61010E+00_JPRB, 0.61730E+00_JPRB, 0.62424E+00_JPRB, 0.63096E+00_JPRB, 0.64048E+00_JPRB, 0.64802E+00_JPRB ,&
&  0.65252E+00_JPRB, 0.65027E+00_JPRB, 0.64413E+00_JPRB, 0.63194E+00_JPRB, 0.62584E+00_JPRB, 0.63072E+00_JPRB, 0.62828E+00_JPRB ,&
&  0.62728E+00_JPRB, 0.62411E+00_JPRB, 0.62164E+00_JPRB, 0.61766E+00_JPRB, 0.61333E+00_JPRB, 0.60699E+00_JPRB, 0.59927E+00_JPRB ,&
&  0.59298E+00_JPRB, 0.58998E+00_JPRB, 0.58732E+00_JPRB, 0.58473E+00_JPRB, 0.57918E+00_JPRB, 0.57348E+00_JPRB, 0.56779E+00_JPRB ,&
&  0.56269E+00_JPRB, 0.55784E+00_JPRB, 0.55312E+00_JPRB, 0.54978E+00_JPRB, 0.54798E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.56193E+00_JPRB, 0.56313E+00_JPRB, 0.56262E+00_JPRB, 0.56221E+00_JPRB, 0.56045E+00_JPRB, 0.55894E+00_JPRB, 0.55670E+00_JPRB ,&
&  0.55455E+00_JPRB, 0.55239E+00_JPRB, 0.55031E+00_JPRB, 0.54838E+00_JPRB, 0.54642E+00_JPRB, 0.54444E+00_JPRB, 0.54313E+00_JPRB ,&
&  0.54176E+00_JPRB, 0.53934E+00_JPRB, 0.53548E+00_JPRB, 0.53061E+00_JPRB, 0.52634E+00_JPRB, 0.52150E+00_JPRB, 0.51561E+00_JPRB ,&
&  0.50923E+00_JPRB, 0.50035E+00_JPRB, 0.48585E+00_JPRB, 0.47953E+00_JPRB, 0.47056E+00_JPRB, 0.46706E+00_JPRB, 0.46138E+00_JPRB ,&
&  0.45864E+00_JPRB, 0.42753E+00_JPRB, 0.42710E+00_JPRB, 0.41533E+00_JPRB, 0.40676E+00_JPRB, 0.40024E+00_JPRB, 0.38642E+00_JPRB ,&
&  0.38038E+00_JPRB, 0.37252E+00_JPRB, 0.36936E+00_JPRB, 0.37276E+00_JPRB, 0.37925E+00_JPRB, 0.39266E+00_JPRB, 0.41019E+00_JPRB ,&
&  0.43169E+00_JPRB, 0.47604E+00_JPRB, 0.48735E+00_JPRB, 0.49526E+00_JPRB, 0.50306E+00_JPRB, 0.50890E+00_JPRB, 0.50168E+00_JPRB ,&
&  0.50021E+00_JPRB, 0.49851E+00_JPRB, 0.50166E+00_JPRB, 0.50861E+00_JPRB, 0.51774E+00_JPRB, 0.52128E+00_JPRB, 0.52653E+00_JPRB ,&
&  0.53155E+00_JPRB, 0.53979E+00_JPRB, 0.54856E+00_JPRB, 0.56118E+00_JPRB, 0.57899E+00_JPRB, 0.59752E+00_JPRB, 0.61377E+00_JPRB ,&
&  0.62670E+00_JPRB, 0.63409E+00_JPRB, 0.63738E+00_JPRB, 0.64529E+00_JPRB, 0.65693E+00_JPRB, 0.67394E+00_JPRB, 0.68491E+00_JPRB ,&
&  0.69182E+00_JPRB, 0.69802E+00_JPRB, 0.69662E+00_JPRB, 0.69102E+00_JPRB, 0.67812E+00_JPRB, 0.66552E+00_JPRB, 0.65916E+00_JPRB ,&
&  0.66142E+00_JPRB, 0.65672E+00_JPRB, 0.64936E+00_JPRB, 0.63849E+00_JPRB, 0.63138E+00_JPRB, 0.62222E+00_JPRB, 0.61287E+00_JPRB ,&
&  0.60502E+00_JPRB, 0.60065E+00_JPRB, 0.59519E+00_JPRB, 0.59094E+00_JPRB, 0.58382E+00_JPRB, 0.57841E+00_JPRB, 0.57419E+00_JPRB ,&
&  0.56973E+00_JPRB, 0.56636E+00_JPRB, 0.56237E+00_JPRB, 0.56068E+00_JPRB, 0.56137E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.57689E+00_JPRB, 0.57702E+00_JPRB, 0.57630E+00_JPRB, 0.57551E+00_JPRB, 0.57423E+00_JPRB, 0.57379E+00_JPRB, 0.57394E+00_JPRB ,&
&  0.57309E+00_JPRB, 0.57237E+00_JPRB, 0.57133E+00_JPRB, 0.57010E+00_JPRB, 0.56858E+00_JPRB, 0.56726E+00_JPRB, 0.56575E+00_JPRB ,&
&  0.56269E+00_JPRB, 0.55823E+00_JPRB, 0.55202E+00_JPRB, 0.54586E+00_JPRB, 0.54012E+00_JPRB, 0.53650E+00_JPRB, 0.53105E+00_JPRB ,&
&  0.52551E+00_JPRB, 0.51777E+00_JPRB, 0.50591E+00_JPRB, 0.49754E+00_JPRB, 0.49516E+00_JPRB, 0.48834E+00_JPRB, 0.48805E+00_JPRB ,&
&  0.48512E+00_JPRB, 0.47999E+00_JPRB, 0.46572E+00_JPRB, 0.46010E+00_JPRB, 0.45072E+00_JPRB, 0.40751E+00_JPRB, 0.39800E+00_JPRB ,&
&  0.38489E+00_JPRB, 0.36992E+00_JPRB, 0.36421E+00_JPRB, 0.36246E+00_JPRB, 0.36568E+00_JPRB, 0.38022E+00_JPRB, 0.39959E+00_JPRB ,&
&  0.42008E+00_JPRB, 0.47579E+00_JPRB, 0.49075E+00_JPRB, 0.50327E+00_JPRB, 0.51224E+00_JPRB, 0.51709E+00_JPRB, 0.51407E+00_JPRB ,&
&  0.51203E+00_JPRB, 0.51160E+00_JPRB, 0.51669E+00_JPRB, 0.52553E+00_JPRB, 0.53159E+00_JPRB, 0.52965E+00_JPRB, 0.52951E+00_JPRB ,&
&  0.52536E+00_JPRB, 0.53058E+00_JPRB, 0.54176E+00_JPRB, 0.55747E+00_JPRB, 0.57852E+00_JPRB, 0.59479E+00_JPRB, 0.61736E+00_JPRB ,&
&  0.63628E+00_JPRB, 0.64823E+00_JPRB, 0.66900E+00_JPRB, 0.69083E+00_JPRB, 0.70615E+00_JPRB, 0.71581E+00_JPRB, 0.71871E+00_JPRB ,&
&  0.71727E+00_JPRB, 0.73487E+00_JPRB, 0.74464E+00_JPRB, 0.74160E+00_JPRB, 0.73261E+00_JPRB, 0.71565E+00_JPRB, 0.69820E+00_JPRB ,&
&  0.68978E+00_JPRB, 0.69246E+00_JPRB, 0.68461E+00_JPRB, 0.66429E+00_JPRB, 0.64827E+00_JPRB, 0.63513E+00_JPRB, 0.62462E+00_JPRB ,&
&  0.61670E+00_JPRB, 0.61191E+00_JPRB, 0.60504E+00_JPRB, 0.59929E+00_JPRB, 0.59291E+00_JPRB, 0.58762E+00_JPRB, 0.58459E+00_JPRB ,&
&  0.58209E+00_JPRB, 0.58156E+00_JPRB, 0.58011E+00_JPRB, 0.57854E+00_JPRB, 0.57847E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.59262E+00_JPRB, 0.59418E+00_JPRB, 0.59492E+00_JPRB, 0.59488E+00_JPRB, 0.59542E+00_JPRB, 0.59746E+00_JPRB, 0.59670E+00_JPRB ,&
&  0.59392E+00_JPRB, 0.59160E+00_JPRB, 0.58776E+00_JPRB, 0.58666E+00_JPRB, 0.58520E+00_JPRB, 0.58347E+00_JPRB, 0.58192E+00_JPRB ,&
&  0.57874E+00_JPRB, 0.56754E+00_JPRB, 0.56023E+00_JPRB, 0.55714E+00_JPRB, 0.55361E+00_JPRB, 0.55163E+00_JPRB, 0.54965E+00_JPRB ,&
&  0.54369E+00_JPRB, 0.53536E+00_JPRB, 0.52709E+00_JPRB, 0.51357E+00_JPRB, 0.50396E+00_JPRB, 0.50489E+00_JPRB, 0.50951E+00_JPRB ,&
&  0.50382E+00_JPRB, 0.48891E+00_JPRB, 0.48928E+00_JPRB, 0.48585E+00_JPRB, 0.46565E+00_JPRB, 0.45788E+00_JPRB, 0.42390E+00_JPRB ,&
&  0.39976E+00_JPRB, 0.37824E+00_JPRB, 0.37071E+00_JPRB, 0.36581E+00_JPRB, 0.37060E+00_JPRB, 0.38294E+00_JPRB, 0.41202E+00_JPRB ,&
&  0.44306E+00_JPRB, 0.47645E+00_JPRB, 0.48887E+00_JPRB, 0.50283E+00_JPRB, 0.51192E+00_JPRB, 0.52519E+00_JPRB, 0.53620E+00_JPRB ,&
&  0.54238E+00_JPRB, 0.54704E+00_JPRB, 0.55448E+00_JPRB, 0.56021E+00_JPRB, 0.57308E+00_JPRB, 0.58368E+00_JPRB, 0.58803E+00_JPRB ,&
&  0.59651E+00_JPRB, 0.59773E+00_JPRB, 0.60644E+00_JPRB, 0.61699E+00_JPRB, 0.63076E+00_JPRB, 0.63805E+00_JPRB, 0.64649E+00_JPRB ,&
&  0.65672E+00_JPRB, 0.68200E+00_JPRB, 0.71989E+00_JPRB, 0.76778E+00_JPRB, 0.78168E+00_JPRB, 0.77819E+00_JPRB, 0.75952E+00_JPRB ,&
&  0.73702E+00_JPRB, 0.75992E+00_JPRB, 0.86148E+00_JPRB, 0.84775E+00_JPRB, 0.83547E+00_JPRB, 0.81827E+00_JPRB, 0.77909E+00_JPRB ,&
&  0.73357E+00_JPRB, 0.71491E+00_JPRB, 0.69807E+00_JPRB, 0.66939E+00_JPRB, 0.64651E+00_JPRB, 0.63098E+00_JPRB, 0.62233E+00_JPRB ,&
&  0.61683E+00_JPRB, 0.61589E+00_JPRB, 0.61105E+00_JPRB, 0.60782E+00_JPRB, 0.60194E+00_JPRB, 0.59594E+00_JPRB, 0.59371E+00_JPRB ,&
&  0.59181E+00_JPRB, 0.59239E+00_JPRB, 0.59370E+00_JPRB, 0.59438E+00_JPRB, 0.59409E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.60847E+00_JPRB, 0.61508E+00_JPRB, 0.62027E+00_JPRB, 0.62950E+00_JPRB, 0.63796E+00_JPRB, 0.63999E+00_JPRB, 0.63535E+00_JPRB ,&
&  0.62342E+00_JPRB, 0.60860E+00_JPRB, 0.59800E+00_JPRB, 0.59002E+00_JPRB, 0.58544E+00_JPRB, 0.58288E+00_JPRB, 0.58038E+00_JPRB ,&
&  0.57846E+00_JPRB, 0.57623E+00_JPRB, 0.57032E+00_JPRB, 0.56176E+00_JPRB, 0.55673E+00_JPRB, 0.55607E+00_JPRB, 0.55563E+00_JPRB ,&
&  0.55278E+00_JPRB, 0.54650E+00_JPRB, 0.53851E+00_JPRB, 0.52544E+00_JPRB, 0.51472E+00_JPRB, 0.51701E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.50684E+00_JPRB, 0.50382E+00_JPRB, 0.50019E+00_JPRB, 0.49506E+00_JPRB, 0.48930E+00_JPRB, 0.47379E+00_JPRB, 0.44540E+00_JPRB ,&
&  0.41584E+00_JPRB, 0.39245E+00_JPRB, 0.37854E+00_JPRB, 0.37449E+00_JPRB, 0.38284E+00_JPRB, 0.40466E+00_JPRB, 0.43568E+00_JPRB ,&
&  0.47210E+00_JPRB, 0.48896E+00_JPRB, 0.50513E+00_JPRB, 0.52270E+00_JPRB, 0.53705E+00_JPRB, 0.55447E+00_JPRB, 0.56994E+00_JPRB ,&
&  0.58595E+00_JPRB, 0.60849E+00_JPRB, 0.63850E+00_JPRB, 0.67750E+00_JPRB, 0.72526E+00_JPRB, 0.71341E+00_JPRB, 0.75028E+00_JPRB ,&
&  0.86075E+00_JPRB, 0.86140E+00_JPRB, 0.82376E+00_JPRB, 0.78114E+00_JPRB, 0.75379E+00_JPRB, 0.74428E+00_JPRB, 0.74731E+00_JPRB ,&
&  0.75454E+00_JPRB, 0.77061E+00_JPRB, 0.79914E+00_JPRB, 0.83449E+00_JPRB, 0.84532E+00_JPRB, 0.84012E+00_JPRB, 0.81319E+00_JPRB ,&
&  0.78903E+00_JPRB, 0.81463E+00_JPRB, 0.10239E+01_JPRB, 0.93699E+00_JPRB, 0.85799E+00_JPRB, 0.81970E+00_JPRB, 0.77372E+00_JPRB ,&
&  0.73979E+00_JPRB, 0.71330E+00_JPRB, 0.68586E+00_JPRB, 0.66449E+00_JPRB, 0.63845E+00_JPRB, 0.61461E+00_JPRB, 0.59396E+00_JPRB ,&
&  0.58754E+00_JPRB, 0.58678E+00_JPRB, 0.58757E+00_JPRB, 0.58886E+00_JPRB, 0.58648E+00_JPRB, 0.58679E+00_JPRB, 0.58917E+00_JPRB ,&
&  0.58688E+00_JPRB, 0.58722E+00_JPRB, 0.59252E+00_JPRB, 0.59802E+00_JPRB, 0.60302E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.59350E+00_JPRB, 0.61345E+00_JPRB, 0.64393E+00_JPRB, 0.66430E+00_JPRB, 0.67306E+00_JPRB, 0.65912E+00_JPRB, 0.64089E+00_JPRB ,&
&  0.62059E+00_JPRB, 0.59440E+00_JPRB, 0.57945E+00_JPRB, 0.57040E+00_JPRB, 0.56514E+00_JPRB, 0.56133E+00_JPRB, 0.55826E+00_JPRB ,&
&  0.55594E+00_JPRB, 0.55399E+00_JPRB, 0.55413E+00_JPRB, 0.55670E+00_JPRB, 0.55831E+00_JPRB, 0.55615E+00_JPRB, 0.55269E+00_JPRB ,&
&  0.54972E+00_JPRB, 0.54497E+00_JPRB, 0.53472E+00_JPRB, 0.52128E+00_JPRB, 0.51700E+00_JPRB, 0.52463E+00_JPRB, 0.52279E+00_JPRB ,&
&  0.51680E+00_JPRB, 0.50984E+00_JPRB, 0.50872E+00_JPRB, 0.50881E+00_JPRB, 0.50376E+00_JPRB, 0.49113E+00_JPRB, 0.46566E+00_JPRB ,&
&  0.43339E+00_JPRB, 0.40772E+00_JPRB, 0.40008E+00_JPRB, 0.41328E+00_JPRB, 0.43966E+00_JPRB, 0.47718E+00_JPRB, 0.49332E+00_JPRB ,&
&  0.50629E+00_JPRB, 0.51874E+00_JPRB, 0.53124E+00_JPRB, 0.55087E+00_JPRB, 0.57322E+00_JPRB, 0.59276E+00_JPRB, 0.61139E+00_JPRB ,&
&  0.64038E+00_JPRB, 0.67278E+00_JPRB, 0.71371E+00_JPRB, 0.68511E+00_JPRB, 0.70690E+00_JPRB, 0.76633E+00_JPRB, 0.79906E+00_JPRB ,&
&  0.81478E+00_JPRB, 0.84945E+00_JPRB, 0.86979E+00_JPRB, 0.89733E+00_JPRB, 0.88407E+00_JPRB, 0.86579E+00_JPRB, 0.84651E+00_JPRB ,&
&  0.84374E+00_JPRB, 0.84135E+00_JPRB, 0.84905E+00_JPRB, 0.88480E+00_JPRB, 0.91865E+00_JPRB, 0.92135E+00_JPRB, 0.89717E+00_JPRB ,&
&  0.85742E+00_JPRB, 0.79610E+00_JPRB, 0.73958E+00_JPRB, 0.70260E+00_JPRB, 0.70157E+00_JPRB, 0.71701E+00_JPRB, 0.72414E+00_JPRB ,&
&  0.72074E+00_JPRB, 0.71470E+00_JPRB, 0.69854E+00_JPRB, 0.68511E+00_JPRB, 0.63753E+00_JPRB, 0.60380E+00_JPRB, 0.57749E+00_JPRB ,&
&  0.56778E+00_JPRB, 0.56293E+00_JPRB, 0.56207E+00_JPRB, 0.57006E+00_JPRB, 0.56846E+00_JPRB, 0.57084E+00_JPRB, 0.57326E+00_JPRB ,&
&  0.57200E+00_JPRB, 0.56985E+00_JPRB, 0.57289E+00_JPRB, 0.57720E+00_JPRB, 0.58342E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.65960E+00_JPRB, 0.67626E+00_JPRB, 0.69106E+00_JPRB, 0.69862E+00_JPRB, 0.68957E+00_JPRB, 0.68177E+00_JPRB, 0.67326E+00_JPRB ,&
&  0.66889E+00_JPRB, 0.61928E+00_JPRB, 0.58702E+00_JPRB, 0.56326E+00_JPRB, 0.54603E+00_JPRB, 0.53067E+00_JPRB, 0.52622E+00_JPRB ,&
&  0.52776E+00_JPRB, 0.53148E+00_JPRB, 0.53453E+00_JPRB, 0.53740E+00_JPRB, 0.54015E+00_JPRB, 0.54109E+00_JPRB, 0.54333E+00_JPRB ,&
&  0.54683E+00_JPRB, 0.54722E+00_JPRB, 0.53856E+00_JPRB, 0.53113E+00_JPRB, 0.52987E+00_JPRB, 0.53215E+00_JPRB, 0.52818E+00_JPRB ,&
&  0.52318E+00_JPRB, 0.51834E+00_JPRB, 0.51640E+00_JPRB, 0.51727E+00_JPRB, 0.51426E+00_JPRB, 0.50265E+00_JPRB, 0.47448E+00_JPRB ,&
&  0.44094E+00_JPRB, 0.42911E+00_JPRB, 0.44669E+00_JPRB, 0.49225E+00_JPRB, 0.50953E+00_JPRB, 0.52168E+00_JPRB, 0.53539E+00_JPRB ,&
&  0.53104E+00_JPRB, 0.53186E+00_JPRB, 0.55006E+00_JPRB, 0.59343E+00_JPRB, 0.61281E+00_JPRB, 0.62984E+00_JPRB, 0.64993E+00_JPRB ,&
&  0.68617E+00_JPRB, 0.71581E+00_JPRB, 0.75061E+00_JPRB, 0.71933E+00_JPRB, 0.78198E+00_JPRB, 0.93374E+00_JPRB, 0.90660E+00_JPRB ,&
&  0.90676E+00_JPRB, 0.91481E+00_JPRB, 0.95964E+00_JPRB, 0.98798E+00_JPRB, 0.98571E+00_JPRB, 0.96805E+00_JPRB, 0.95928E+00_JPRB ,&
&  0.94547E+00_JPRB, 0.94177E+00_JPRB, 0.95454E+00_JPRB, 0.10052E+01_JPRB, 0.10058E+01_JPRB, 0.98955E+00_JPRB, 0.96413E+00_JPRB ,&
&  0.91837E+00_JPRB, 0.85370E+00_JPRB, 0.79417E+00_JPRB, 0.75048E+00_JPRB, 0.74027E+00_JPRB, 0.74432E+00_JPRB, 0.74099E+00_JPRB ,&
&  0.72671E+00_JPRB, 0.72467E+00_JPRB, 0.72664E+00_JPRB, 0.72446E+00_JPRB, 0.67582E+00_JPRB, 0.62109E+00_JPRB, 0.58051E+00_JPRB ,&
&  0.55136E+00_JPRB, 0.53476E+00_JPRB, 0.52278E+00_JPRB, 0.52826E+00_JPRB, 0.54147E+00_JPRB, 0.55781E+00_JPRB, 0.55924E+00_JPRB ,&
&  0.56324E+00_JPRB, 0.56652E+00_JPRB, 0.57949E+00_JPRB, 0.60722E+00_JPRB, 0.63544E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.69669E+00_JPRB, 0.71570E+00_JPRB, 0.73103E+00_JPRB, 0.72526E+00_JPRB, 0.71099E+00_JPRB, 0.69993E+00_JPRB, 0.68749E+00_JPRB ,&
&  0.68174E+00_JPRB, 0.60585E+00_JPRB, 0.57359E+00_JPRB, 0.55604E+00_JPRB, 0.54457E+00_JPRB, 0.52330E+00_JPRB, 0.50613E+00_JPRB ,&
&  0.50390E+00_JPRB, 0.51275E+00_JPRB, 0.52569E+00_JPRB, 0.53434E+00_JPRB, 0.54208E+00_JPRB, 0.54745E+00_JPRB, 0.55016E+00_JPRB ,&
&  0.55332E+00_JPRB, 0.55642E+00_JPRB, 0.55256E+00_JPRB, 0.54699E+00_JPRB, 0.54335E+00_JPRB, 0.54253E+00_JPRB, 0.54188E+00_JPRB ,&
&  0.53912E+00_JPRB, 0.53443E+00_JPRB, 0.52628E+00_JPRB, 0.52254E+00_JPRB, 0.51955E+00_JPRB, 0.52063E+00_JPRB, 0.51464E+00_JPRB ,&
&  0.46001E+00_JPRB, 0.46540E+00_JPRB, 0.51007E+00_JPRB, 0.51860E+00_JPRB, 0.53211E+00_JPRB, 0.53914E+00_JPRB, 0.54491E+00_JPRB ,&
&  0.55523E+00_JPRB, 0.57512E+00_JPRB, 0.60352E+00_JPRB, 0.63606E+00_JPRB, 0.66194E+00_JPRB, 0.68029E+00_JPRB, 0.69670E+00_JPRB ,&
&  0.74011E+00_JPRB, 0.68937E+00_JPRB, 0.67371E+00_JPRB, 0.73189E+00_JPRB, 0.86569E+00_JPRB, 0.93997E+00_JPRB, 0.10007E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10609E+01_JPRB, 0.10632E+01_JPRB, 0.10840E+01_JPRB, 0.10893E+01_JPRB, 0.10851E+01_JPRB, 0.11067E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.11232E+01_JPRB, 0.11501E+01_JPRB, 0.11710E+01_JPRB, 0.11279E+01_JPRB, 0.10944E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.10069E+01_JPRB, 0.95010E+00_JPRB, 0.88570E+00_JPRB, 0.83476E+00_JPRB, 0.79977E+00_JPRB, 0.79836E+00_JPRB, 0.79412E+00_JPRB ,&
&  0.77862E+00_JPRB, 0.76972E+00_JPRB, 0.75705E+00_JPRB, 0.74437E+00_JPRB, 0.71089E+00_JPRB, 0.67883E+00_JPRB, 0.66213E+00_JPRB ,&
&  0.61223E+00_JPRB, 0.57925E+00_JPRB, 0.54005E+00_JPRB, 0.53952E+00_JPRB, 0.56538E+00_JPRB, 0.58461E+00_JPRB, 0.59414E+00_JPRB ,&
&  0.61635E+00_JPRB, 0.63046E+00_JPRB, 0.63703E+00_JPRB, 0.65197E+00_JPRB, 0.67352E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.71763E+00_JPRB, 0.72670E+00_JPRB, 0.73051E+00_JPRB, 0.72658E+00_JPRB, 0.72896E+00_JPRB, 0.72474E+00_JPRB, 0.72674E+00_JPRB ,&
&  0.75208E+00_JPRB, 0.70324E+00_JPRB, 0.73752E+00_JPRB, 0.62621E+00_JPRB, 0.57060E+00_JPRB, 0.54058E+00_JPRB, 0.51881E+00_JPRB ,&
&  0.51444E+00_JPRB, 0.52129E+00_JPRB, 0.52825E+00_JPRB, 0.53747E+00_JPRB, 0.55193E+00_JPRB, 0.55394E+00_JPRB, 0.55511E+00_JPRB ,&
&  0.56066E+00_JPRB, 0.56628E+00_JPRB, 0.57260E+00_JPRB, 0.56238E+00_JPRB, 0.55720E+00_JPRB, 0.55506E+00_JPRB, 0.55923E+00_JPRB ,&
&  0.54247E+00_JPRB, 0.53841E+00_JPRB, 0.54497E+00_JPRB, 0.54137E+00_JPRB, 0.53940E+00_JPRB, 0.53727E+00_JPRB, 0.53949E+00_JPRB ,&
&  0.54567E+00_JPRB, 0.52333E+00_JPRB, 0.54772E+00_JPRB, 0.55718E+00_JPRB, 0.56389E+00_JPRB, 0.57109E+00_JPRB, 0.57632E+00_JPRB ,&
&  0.58689E+00_JPRB, 0.60551E+00_JPRB, 0.63281E+00_JPRB, 0.66114E+00_JPRB, 0.69406E+00_JPRB, 0.72264E+00_JPRB, 0.74518E+00_JPRB ,&
&  0.79849E+00_JPRB, 0.71478E+00_JPRB, 0.81945E+00_JPRB, 0.87661E+00_JPRB, 0.99590E+00_JPRB, 0.11487E+01_JPRB, 0.12123E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.13494E+01_JPRB, 0.13395E+01_JPRB, 0.13173E+01_JPRB, 0.12843E+01_JPRB, 0.12690E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.13168E+01_JPRB, 0.13393E+01_JPRB, 0.14353E+01_JPRB, 0.13974E+01_JPRB, 0.12941E+01_JPRB, 0.12290E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.10678E+01_JPRB, 0.10304E+01_JPRB, 0.93848E+00_JPRB, 0.91633E+00_JPRB, 0.90301E+00_JPRB, 0.84962E+00_JPRB ,&
&  0.82398E+00_JPRB, 0.78342E+00_JPRB, 0.73583E+00_JPRB, 0.74279E+00_JPRB, 0.70711E+00_JPRB, 0.68030E+00_JPRB, 0.67479E+00_JPRB ,&
&  0.65038E+00_JPRB, 0.60526E+00_JPRB, 0.60639E+00_JPRB, 0.62187E+00_JPRB, 0.61933E+00_JPRB, 0.61810E+00_JPRB, 0.64470E+00_JPRB ,&
&  0.65559E+00_JPRB, 0.66900E+00_JPRB, 0.70120E+00_JPRB, 0.70631E+00_JPRB, 0.71557E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.73009E+00_JPRB, 0.72146E+00_JPRB, 0.71803E+00_JPRB, 0.72219E+00_JPRB, 0.74023E+00_JPRB, 0.76998E+00_JPRB, 0.75921E+00_JPRB ,&
&  0.77896E+00_JPRB, 0.78282E+00_JPRB, 0.77020E+00_JPRB, 0.75002E+00_JPRB, 0.71719E+00_JPRB, 0.63332E+00_JPRB, 0.53952E+00_JPRB ,&
&  0.51226E+00_JPRB, 0.51900E+00_JPRB, 0.53291E+00_JPRB, 0.54010E+00_JPRB, 0.55045E+00_JPRB, 0.55214E+00_JPRB, 0.55540E+00_JPRB ,&
&  0.56324E+00_JPRB, 0.57119E+00_JPRB, 0.57692E+00_JPRB, 0.57690E+00_JPRB, 0.56771E+00_JPRB, 0.57199E+00_JPRB, 0.58448E+00_JPRB ,&
&  0.55838E+00_JPRB, 0.54446E+00_JPRB, 0.53960E+00_JPRB, 0.53141E+00_JPRB, 0.54929E+00_JPRB, 0.55747E+00_JPRB, 0.56539E+00_JPRB ,&
&  0.57492E+00_JPRB, 0.58366E+00_JPRB, 0.58597E+00_JPRB, 0.59182E+00_JPRB, 0.60082E+00_JPRB, 0.61751E+00_JPRB, 0.63517E+00_JPRB ,&
&  0.65698E+00_JPRB, 0.67902E+00_JPRB, 0.70595E+00_JPRB, 0.74170E+00_JPRB, 0.78688E+00_JPRB, 0.79018E+00_JPRB, 0.84510E+00_JPRB ,&
&  0.89772E+00_JPRB, 0.99988E+00_JPRB, 0.11191E+01_JPRB, 0.10972E+01_JPRB, 0.12307E+01_JPRB, 0.13550E+01_JPRB, 0.13780E+01_JPRB ,&
&  0.14692E+01_JPRB, 0.14383E+01_JPRB, 0.15027E+01_JPRB, 0.15644E+01_JPRB, 0.16126E+01_JPRB, 0.16073E+01_JPRB, 0.15501E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.17025E+01_JPRB, 0.17258E+01_JPRB, 0.16385E+01_JPRB, 0.14888E+01_JPRB, 0.13940E+01_JPRB, 0.12875E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.12608E+01_JPRB, 0.12202E+01_JPRB, 0.11748E+01_JPRB, 0.10936E+01_JPRB, 0.10064E+01_JPRB, 0.93886E+00_JPRB ,&
&  0.86271E+00_JPRB, 0.76220E+00_JPRB, 0.71417E+00_JPRB, 0.70408E+00_JPRB, 0.69861E+00_JPRB, 0.66455E+00_JPRB, 0.66001E+00_JPRB ,&
&  0.66278E+00_JPRB, 0.65594E+00_JPRB, 0.68427E+00_JPRB, 0.69245E+00_JPRB, 0.68563E+00_JPRB, 0.67299E+00_JPRB, 0.65216E+00_JPRB ,&
&  0.64046E+00_JPRB, 0.67838E+00_JPRB, 0.71164E+00_JPRB, 0.72906E+00_JPRB, 0.73327E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.73799E+00_JPRB, 0.72601E+00_JPRB, 0.70826E+00_JPRB, 0.71440E+00_JPRB, 0.74851E+00_JPRB, 0.78138E+00_JPRB, 0.79978E+00_JPRB ,&
&  0.79477E+00_JPRB, 0.79316E+00_JPRB, 0.78226E+00_JPRB, 0.76771E+00_JPRB, 0.74333E+00_JPRB, 0.71477E+00_JPRB, 0.64428E+00_JPRB ,&
&  0.55925E+00_JPRB, 0.54846E+00_JPRB, 0.52898E+00_JPRB, 0.53398E+00_JPRB, 0.55366E+00_JPRB, 0.55611E+00_JPRB, 0.56008E+00_JPRB ,&
&  0.56945E+00_JPRB, 0.57687E+00_JPRB, 0.58215E+00_JPRB, 0.57957E+00_JPRB, 0.57605E+00_JPRB, 0.60175E+00_JPRB, 0.59573E+00_JPRB ,&
&  0.56019E+00_JPRB, 0.53959E+00_JPRB, 0.53610E+00_JPRB, 0.54140E+00_JPRB, 0.55706E+00_JPRB, 0.57597E+00_JPRB, 0.58563E+00_JPRB ,&
&  0.59382E+00_JPRB, 0.60212E+00_JPRB, 0.60616E+00_JPRB, 0.61894E+00_JPRB, 0.63917E+00_JPRB, 0.66632E+00_JPRB, 0.69177E+00_JPRB ,&
&  0.71250E+00_JPRB, 0.73912E+00_JPRB, 0.77597E+00_JPRB, 0.82594E+00_JPRB, 0.86297E+00_JPRB, 0.90946E+00_JPRB, 0.98659E+00_JPRB ,&
&  0.10557E+01_JPRB, 0.11569E+01_JPRB, 0.12281E+01_JPRB, 0.13388E+01_JPRB, 0.15073E+01_JPRB, 0.15693E+01_JPRB, 0.15923E+01_JPRB ,&
&  0.16435E+01_JPRB, 0.16142E+01_JPRB, 0.18798E+01_JPRB, 0.19369E+01_JPRB, 0.19148E+01_JPRB, 0.19286E+01_JPRB, 0.18223E+01_JPRB ,&
&  0.17445E+01_JPRB, 0.17080E+01_JPRB, 0.16797E+01_JPRB, 0.16331E+01_JPRB, 0.15755E+01_JPRB, 0.15802E+01_JPRB, 0.14848E+01_JPRB ,&
&  0.14882E+01_JPRB, 0.14576E+01_JPRB, 0.12512E+01_JPRB, 0.10957E+01_JPRB, 0.95221E+00_JPRB, 0.96454E+00_JPRB, 0.94083E+00_JPRB ,&
&  0.86037E+00_JPRB, 0.74297E+00_JPRB, 0.73516E+00_JPRB, 0.72172E+00_JPRB, 0.72776E+00_JPRB, 0.74051E+00_JPRB, 0.72669E+00_JPRB ,&
&  0.73579E+00_JPRB, 0.76527E+00_JPRB, 0.77186E+00_JPRB, 0.80021E+00_JPRB, 0.78473E+00_JPRB, 0.75034E+00_JPRB, 0.66871E+00_JPRB ,&
&  0.64862E+00_JPRB, 0.67778E+00_JPRB, 0.70365E+00_JPRB, 0.72646E+00_JPRB, 0.73885E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.76148E+00_JPRB, 0.75339E+00_JPRB, 0.73725E+00_JPRB, 0.74020E+00_JPRB, 0.77065E+00_JPRB, 0.80327E+00_JPRB, 0.82081E+00_JPRB ,&
&  0.81544E+00_JPRB, 0.81535E+00_JPRB, 0.79842E+00_JPRB, 0.78278E+00_JPRB, 0.76530E+00_JPRB, 0.73677E+00_JPRB, 0.72794E+00_JPRB ,&
&  0.60858E+00_JPRB, 0.57005E+00_JPRB, 0.52036E+00_JPRB, 0.51630E+00_JPRB, 0.56236E+00_JPRB, 0.56593E+00_JPRB, 0.57430E+00_JPRB ,&
&  0.57747E+00_JPRB, 0.58920E+00_JPRB, 0.58810E+00_JPRB, 0.58512E+00_JPRB, 0.59867E+00_JPRB, 0.62369E+00_JPRB, 0.60704E+00_JPRB ,&
&  0.57924E+00_JPRB, 0.56012E+00_JPRB, 0.55965E+00_JPRB, 0.56254E+00_JPRB, 0.57309E+00_JPRB, 0.59118E+00_JPRB, 0.60399E+00_JPRB ,&
&  0.61443E+00_JPRB, 0.63787E+00_JPRB, 0.65762E+00_JPRB, 0.68437E+00_JPRB, 0.71620E+00_JPRB, 0.76114E+00_JPRB, 0.79836E+00_JPRB ,&
&  0.83861E+00_JPRB, 0.88690E+00_JPRB, 0.93741E+00_JPRB, 0.99739E+00_JPRB, 0.10268E+01_JPRB, 0.11032E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.13635E+01_JPRB, 0.14208E+01_JPRB, 0.14588E+01_JPRB, 0.17885E+01_JPRB, 0.19520E+01_JPRB, 0.19584E+01_JPRB, 0.18514E+01_JPRB ,&
&  0.18197E+01_JPRB, 0.17922E+01_JPRB, 0.18504E+01_JPRB, 0.19394E+01_JPRB, 0.19479E+01_JPRB, 0.20164E+01_JPRB, 0.19318E+01_JPRB ,&
&  0.17480E+01_JPRB, 0.16668E+01_JPRB, 0.16276E+01_JPRB, 0.16391E+01_JPRB, 0.15757E+01_JPRB, 0.16048E+01_JPRB, 0.15522E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.13780E+01_JPRB, 0.10514E+01_JPRB, 0.96649E+00_JPRB, 0.80156E+00_JPRB, 0.82021E+00_JPRB, 0.98946E+00_JPRB ,&
&  0.95910E+00_JPRB, 0.86805E+00_JPRB, 0.80435E+00_JPRB, 0.77128E+00_JPRB, 0.77467E+00_JPRB, 0.90438E+00_JPRB, 0.91915E+00_JPRB ,&
&  0.84328E+00_JPRB, 0.88132E+00_JPRB, 0.93435E+00_JPRB, 0.97042E+00_JPRB, 0.94769E+00_JPRB, 0.90328E+00_JPRB, 0.81645E+00_JPRB ,&
&  0.78984E+00_JPRB, 0.77425E+00_JPRB, 0.77238E+00_JPRB, 0.77158E+00_JPRB, 0.76763E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.82474E+00_JPRB, 0.81152E+00_JPRB, 0.79553E+00_JPRB, 0.79666E+00_JPRB, 0.81702E+00_JPRB, 0.84480E+00_JPRB, 0.86058E+00_JPRB ,&
&  0.85108E+00_JPRB, 0.84473E+00_JPRB, 0.82368E+00_JPRB, 0.80451E+00_JPRB, 0.78579E+00_JPRB, 0.76000E+00_JPRB, 0.74320E+00_JPRB ,&
&  0.73699E+00_JPRB, 0.63480E+00_JPRB, 0.55650E+00_JPRB, 0.51513E+00_JPRB, 0.54282E+00_JPRB, 0.55635E+00_JPRB, 0.57655E+00_JPRB ,&
&  0.57897E+00_JPRB, 0.60305E+00_JPRB, 0.60384E+00_JPRB, 0.60491E+00_JPRB, 0.64061E+00_JPRB, 0.64818E+00_JPRB, 0.63514E+00_JPRB ,&
&  0.62910E+00_JPRB, 0.61032E+00_JPRB, 0.60552E+00_JPRB, 0.61517E+00_JPRB, 0.63229E+00_JPRB, 0.61876E+00_JPRB, 0.64525E+00_JPRB ,&
&  0.67669E+00_JPRB, 0.70998E+00_JPRB, 0.73883E+00_JPRB, 0.78410E+00_JPRB, 0.84351E+00_JPRB, 0.89616E+00_JPRB, 0.95428E+00_JPRB ,&
&  0.10115E+01_JPRB, 0.10498E+01_JPRB, 0.10667E+01_JPRB, 0.11127E+01_JPRB, 0.11914E+01_JPRB, 0.13343E+01_JPRB, 0.13566E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.14834E+01_JPRB, 0.14457E+01_JPRB, 0.18404E+01_JPRB, 0.21226E+01_JPRB, 0.21360E+01_JPRB, 0.20798E+01_JPRB ,&
&  0.20553E+01_JPRB, 0.20255E+01_JPRB, 0.21232E+01_JPRB, 0.21557E+01_JPRB, 0.21716E+01_JPRB, 0.21474E+01_JPRB, 0.19979E+01_JPRB ,&
&  0.18114E+01_JPRB, 0.16864E+01_JPRB, 0.16257E+01_JPRB, 0.15655E+01_JPRB, 0.15123E+01_JPRB, 0.14649E+01_JPRB, 0.15220E+01_JPRB ,&
&  0.16386E+01_JPRB, 0.98729E+00_JPRB, 0.81248E+00_JPRB, 0.91817E+00_JPRB, 0.79751E+00_JPRB, 0.81903E+00_JPRB, 0.90654E+00_JPRB ,&
&  0.88997E+00_JPRB, 0.87485E+00_JPRB, 0.89666E+00_JPRB, 0.88827E+00_JPRB, 0.93627E+00_JPRB, 0.11596E+01_JPRB, 0.11982E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.10683E+01_JPRB, 0.11359E+01_JPRB, 0.11390E+01_JPRB, 0.11247E+01_JPRB, 0.10921E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.96872E+00_JPRB, 0.93135E+00_JPRB, 0.89364E+00_JPRB, 0.86174E+00_JPRB, 0.83981E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.93547E+00_JPRB, 0.91966E+00_JPRB, 0.90655E+00_JPRB, 0.89714E+00_JPRB, 0.90198E+00_JPRB, 0.92195E+00_JPRB, 0.92902E+00_JPRB ,&
&  0.91266E+00_JPRB, 0.89619E+00_JPRB, 0.86566E+00_JPRB, 0.83898E+00_JPRB, 0.81511E+00_JPRB, 0.79425E+00_JPRB, 0.77615E+00_JPRB ,&
&  0.75602E+00_JPRB, 0.70323E+00_JPRB, 0.59412E+00_JPRB, 0.54322E+00_JPRB, 0.51528E+00_JPRB, 0.53774E+00_JPRB, 0.56263E+00_JPRB ,&
&  0.58063E+00_JPRB, 0.60190E+00_JPRB, 0.61525E+00_JPRB, 0.66143E+00_JPRB, 0.72235E+00_JPRB, 0.70129E+00_JPRB, 0.71727E+00_JPRB ,&
&  0.71757E+00_JPRB, 0.69827E+00_JPRB, 0.66536E+00_JPRB, 0.68617E+00_JPRB, 0.67968E+00_JPRB, 0.67420E+00_JPRB, 0.74855E+00_JPRB ,&
&  0.79931E+00_JPRB, 0.83810E+00_JPRB, 0.87914E+00_JPRB, 0.92369E+00_JPRB, 0.97288E+00_JPRB, 0.99574E+00_JPRB, 0.10414E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.11364E+01_JPRB, 0.11765E+01_JPRB, 0.11979E+01_JPRB, 0.12575E+01_JPRB, 0.13440E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.12870E+01_JPRB, 0.10469E+01_JPRB, 0.14081E+01_JPRB, 0.20947E+01_JPRB, 0.22413E+01_JPRB, 0.21143E+01_JPRB ,&
&  0.22623E+01_JPRB, 0.23031E+01_JPRB, 0.24600E+01_JPRB, 0.25927E+01_JPRB, 0.25287E+01_JPRB, 0.23910E+01_JPRB, 0.22120E+01_JPRB ,&
&  0.19908E+01_JPRB, 0.18366E+01_JPRB, 0.17674E+01_JPRB, 0.18616E+01_JPRB, 0.18123E+01_JPRB, 0.18269E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.16205E+01_JPRB, 0.13689E+01_JPRB, 0.99996E+00_JPRB, 0.10273E+01_JPRB, 0.83736E+00_JPRB, 0.84988E+00_JPRB, 0.93656E+00_JPRB ,&
&  0.91044E+00_JPRB, 0.96685E+00_JPRB, 0.10130E+01_JPRB, 0.10286E+01_JPRB, 0.12447E+01_JPRB, 0.14523E+01_JPRB, 0.16423E+01_JPRB ,&
&  0.16088E+01_JPRB, 0.13781E+01_JPRB, 0.14333E+01_JPRB, 0.13916E+01_JPRB, 0.13890E+01_JPRB, 0.13510E+01_JPRB, 0.12909E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.11294E+01_JPRB, 0.10561E+01_JPRB, 0.99677E+00_JPRB, 0.95885E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.10220E+01_JPRB, 0.99187E+00_JPRB, 0.97245E+00_JPRB, 0.96351E+00_JPRB, 0.96849E+00_JPRB, 0.98535E+00_JPRB, 0.98274E+00_JPRB ,&
&  0.96259E+00_JPRB, 0.93380E+00_JPRB, 0.89780E+00_JPRB, 0.86541E+00_JPRB, 0.83903E+00_JPRB, 0.82010E+00_JPRB, 0.80410E+00_JPRB ,&
&  0.77610E+00_JPRB, 0.68776E+00_JPRB, 0.57248E+00_JPRB, 0.52000E+00_JPRB, 0.49801E+00_JPRB, 0.50670E+00_JPRB, 0.54532E+00_JPRB ,&
&  0.58542E+00_JPRB, 0.61760E+00_JPRB, 0.66217E+00_JPRB, 0.73504E+00_JPRB, 0.79889E+00_JPRB, 0.81074E+00_JPRB, 0.83180E+00_JPRB ,&
&  0.91255E+00_JPRB, 0.77853E+00_JPRB, 0.77386E+00_JPRB, 0.77035E+00_JPRB, 0.82716E+00_JPRB, 0.87334E+00_JPRB, 0.86783E+00_JPRB ,&
&  0.85902E+00_JPRB, 0.88826E+00_JPRB, 0.92299E+00_JPRB, 0.97271E+00_JPRB, 0.10197E+01_JPRB, 0.10881E+01_JPRB, 0.12119E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.13315E+01_JPRB, 0.12735E+01_JPRB, 0.12370E+01_JPRB, 0.13189E+01_JPRB, 0.13904E+01_JPRB, 0.13740E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.11906E+01_JPRB, 0.13635E+01_JPRB, 0.16871E+01_JPRB, 0.20770E+01_JPRB, 0.23757E+01_JPRB, 0.24428E+01_JPRB ,&
&  0.25423E+01_JPRB, 0.25190E+01_JPRB, 0.25917E+01_JPRB, 0.23827E+01_JPRB, 0.25953E+01_JPRB, 0.25388E+01_JPRB, 0.24430E+01_JPRB ,&
&  0.23900E+01_JPRB, 0.23666E+01_JPRB, 0.22884E+01_JPRB, 0.23535E+01_JPRB, 0.25126E+01_JPRB, 0.25896E+01_JPRB, 0.20695E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.16742E+01_JPRB, 0.14392E+01_JPRB, 0.12515E+01_JPRB, 0.11009E+01_JPRB, 0.11123E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.11463E+01_JPRB, 0.11441E+01_JPRB, 0.12664E+01_JPRB, 0.15028E+01_JPRB, 0.17482E+01_JPRB, 0.19353E+01_JPRB ,&
&  0.17861E+01_JPRB, 0.18813E+01_JPRB, 0.18194E+01_JPRB, 0.17405E+01_JPRB, 0.17390E+01_JPRB, 0.16913E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.13357E+01_JPRB, 0.12309E+01_JPRB, 0.11417E+01_JPRB, 0.10712E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.12063E+01_JPRB, 0.11570E+01_JPRB, 0.11217E+01_JPRB, 0.11027E+01_JPRB, 0.11026E+01_JPRB, 0.11107E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10351E+01_JPRB, 0.98700E+00_JPRB, 0.94615E+00_JPRB, 0.91080E+00_JPRB, 0.87583E+00_JPRB, 0.84918E+00_JPRB, 0.83053E+00_JPRB ,&
&  0.81286E+00_JPRB, 0.67919E+00_JPRB, 0.55343E+00_JPRB, 0.52800E+00_JPRB, 0.51403E+00_JPRB, 0.49729E+00_JPRB, 0.52653E+00_JPRB ,&
&  0.58366E+00_JPRB, 0.65385E+00_JPRB, 0.72327E+00_JPRB, 0.84377E+00_JPRB, 0.91177E+00_JPRB, 0.95640E+00_JPRB, 0.98064E+00_JPRB ,&
&  0.94396E+00_JPRB, 0.92866E+00_JPRB, 0.95571E+00_JPRB, 0.94598E+00_JPRB, 0.92568E+00_JPRB, 0.93071E+00_JPRB, 0.91615E+00_JPRB ,&
&  0.91754E+00_JPRB, 0.94121E+00_JPRB, 0.99253E+00_JPRB, 0.10714E+01_JPRB, 0.11649E+01_JPRB, 0.12859E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.13882E+01_JPRB, 0.12568E+01_JPRB, 0.12549E+01_JPRB, 0.11986E+01_JPRB, 0.12125E+01_JPRB, 0.11939E+01_JPRB ,&
&  0.14554E+01_JPRB, 0.15324E+01_JPRB, 0.14990E+01_JPRB, 0.18298E+01_JPRB, 0.18317E+01_JPRB, 0.20524E+01_JPRB, 0.25810E+01_JPRB ,&
&  0.26957E+01_JPRB, 0.27001E+01_JPRB, 0.27700E+01_JPRB, 0.22245E+01_JPRB, 0.19850E+01_JPRB, 0.27876E+01_JPRB, 0.29812E+01_JPRB ,&
&  0.31155E+01_JPRB, 0.32925E+01_JPRB, 0.33082E+01_JPRB, 0.34827E+01_JPRB, 0.29043E+01_JPRB, 0.23814E+01_JPRB, 0.17816E+01_JPRB ,&
&  0.16706E+01_JPRB, 0.15507E+01_JPRB, 0.16744E+01_JPRB, 0.15022E+01_JPRB, 0.14847E+01_JPRB, 0.13686E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.13398E+01_JPRB, 0.14240E+01_JPRB, 0.16241E+01_JPRB, 0.19045E+01_JPRB, 0.19110E+01_JPRB, 0.21780E+01_JPRB ,&
&  0.22724E+01_JPRB, 0.22194E+01_JPRB, 0.19635E+01_JPRB, 0.19732E+01_JPRB, 0.18988E+01_JPRB, 0.18260E+01_JPRB, 0.17346E+01_JPRB ,&
&  0.16207E+01_JPRB, 0.15117E+01_JPRB, 0.14227E+01_JPRB, 0.13402E+01_JPRB, 0.12689E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.13844E+01_JPRB, 0.13362E+01_JPRB, 0.12966E+01_JPRB, 0.12720E+01_JPRB, 0.12559E+01_JPRB, 0.12264E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.10831E+01_JPRB, 0.10247E+01_JPRB, 0.97147E+00_JPRB, 0.92974E+00_JPRB, 0.88539E+00_JPRB, 0.86493E+00_JPRB, 0.86075E+00_JPRB ,&
&  0.84660E+00_JPRB, 0.75347E+00_JPRB, 0.59636E+00_JPRB, 0.54835E+00_JPRB, 0.54486E+00_JPRB, 0.52302E+00_JPRB, 0.52847E+00_JPRB ,&
&  0.61844E+00_JPRB, 0.73098E+00_JPRB, 0.83875E+00_JPRB, 0.99575E+00_JPRB, 0.10639E+01_JPRB, 0.11294E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.10566E+01_JPRB, 0.98847E+00_JPRB, 0.93419E+00_JPRB, 0.89776E+00_JPRB, 0.88095E+00_JPRB, 0.90155E+00_JPRB ,&
&  0.93141E+00_JPRB, 0.10042E+01_JPRB, 0.10968E+01_JPRB, 0.11525E+01_JPRB, 0.12522E+01_JPRB, 0.13508E+01_JPRB, 0.14314E+01_JPRB ,&
&  0.14236E+01_JPRB, 0.13721E+01_JPRB, 0.13358E+01_JPRB, 0.14172E+01_JPRB, 0.14295E+01_JPRB, 0.13335E+01_JPRB, 0.13465E+01_JPRB ,&
&  0.14977E+01_JPRB, 0.15143E+01_JPRB, 0.15822E+01_JPRB, 0.17911E+01_JPRB, 0.20997E+01_JPRB, 0.22027E+01_JPRB, 0.25457E+01_JPRB ,&
&  0.21170E+01_JPRB, 0.18780E+01_JPRB, 0.16451E+01_JPRB, 0.14723E+01_JPRB, 0.19908E+01_JPRB, 0.34111E+01_JPRB, 0.37145E+01_JPRB ,&
&  0.37817E+01_JPRB, 0.40499E+01_JPRB, 0.41503E+01_JPRB, 0.36097E+01_JPRB, 0.28910E+01_JPRB, 0.22931E+01_JPRB, 0.27580E+01_JPRB ,&
&  0.27730E+01_JPRB, 0.30346E+01_JPRB, 0.27050E+01_JPRB, 0.20888E+01_JPRB, 0.16887E+01_JPRB, 0.18858E+01_JPRB, 0.18306E+01_JPRB ,&
&  0.17885E+01_JPRB, 0.18013E+01_JPRB, 0.22760E+01_JPRB, 0.23205E+01_JPRB, 0.23016E+01_JPRB, 0.24414E+01_JPRB, 0.26680E+01_JPRB ,&
&  0.24742E+01_JPRB, 0.23903E+01_JPRB, 0.22748E+01_JPRB, 0.21282E+01_JPRB, 0.20147E+01_JPRB, 0.19039E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.17126E+01_JPRB, 0.16343E+01_JPRB, 0.15516E+01_JPRB, 0.14907E+01_JPRB, 0.14344E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.15087E+01_JPRB, 0.14621E+01_JPRB, 0.14246E+01_JPRB, 0.13836E+01_JPRB, 0.13366E+01_JPRB, 0.12715E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.10467E+01_JPRB, 0.99436E+00_JPRB, 0.95296E+00_JPRB, 0.90850E+00_JPRB, 0.88323E+00_JPRB, 0.87522E+00_JPRB ,&
&  0.86716E+00_JPRB, 0.86505E+00_JPRB, 0.68261E+00_JPRB, 0.59998E+00_JPRB, 0.58977E+00_JPRB, 0.58913E+00_JPRB, 0.57635E+00_JPRB ,&
&  0.70259E+00_JPRB, 0.84773E+00_JPRB, 0.94343E+00_JPRB, 0.11151E+01_JPRB, 0.12881E+01_JPRB, 0.12087E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.10881E+01_JPRB, 0.10091E+01_JPRB, 0.96965E+00_JPRB, 0.94885E+00_JPRB, 0.96011E+00_JPRB, 0.99577E+00_JPRB ,&
&  0.10696E+01_JPRB, 0.11632E+01_JPRB, 0.11936E+01_JPRB, 0.12455E+01_JPRB, 0.13148E+01_JPRB, 0.14257E+01_JPRB, 0.14994E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.14079E+01_JPRB, 0.14424E+01_JPRB, 0.15931E+01_JPRB, 0.16241E+01_JPRB, 0.15411E+01_JPRB, 0.16926E+01_JPRB ,&
&  0.16881E+01_JPRB, 0.18382E+01_JPRB, 0.18014E+01_JPRB, 0.19646E+01_JPRB, 0.21130E+01_JPRB, 0.21217E+01_JPRB, 0.22840E+01_JPRB ,&
&  0.18075E+01_JPRB, 0.16242E+01_JPRB, 0.17263E+01_JPRB, 0.19911E+01_JPRB, 0.23337E+01_JPRB, 0.30405E+01_JPRB, 0.47111E+01_JPRB ,&
&  0.39471E+01_JPRB, 0.42191E+01_JPRB, 0.42168E+01_JPRB, 0.39928E+01_JPRB, 0.21799E+01_JPRB, 0.17845E+01_JPRB, 0.29732E+01_JPRB ,&
&  0.33455E+01_JPRB, 0.21032E+01_JPRB, 0.14456E+01_JPRB, 0.16852E+01_JPRB, 0.12940E+01_JPRB, 0.15748E+01_JPRB, 0.21435E+01_JPRB ,&
&  0.23280E+01_JPRB, 0.24578E+01_JPRB, 0.31421E+01_JPRB, 0.28279E+01_JPRB, 0.27002E+01_JPRB, 0.35156E+01_JPRB, 0.30402E+01_JPRB ,&
&  0.28025E+01_JPRB, 0.24219E+01_JPRB, 0.23897E+01_JPRB, 0.21410E+01_JPRB, 0.19855E+01_JPRB, 0.18903E+01_JPRB, 0.18334E+01_JPRB ,&
&  0.17980E+01_JPRB, 0.17552E+01_JPRB, 0.17001E+01_JPRB, 0.16334E+01_JPRB, 0.15628E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.15572E+01_JPRB, 0.15214E+01_JPRB, 0.14903E+01_JPRB, 0.14450E+01_JPRB, 0.13815E+01_JPRB, 0.13010E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.11517E+01_JPRB, 0.10824E+01_JPRB, 0.10271E+01_JPRB, 0.98284E+00_JPRB, 0.93909E+00_JPRB, 0.90638E+00_JPRB, 0.88789E+00_JPRB ,&
&  0.86811E+00_JPRB, 0.85483E+00_JPRB, 0.79482E+00_JPRB, 0.72405E+00_JPRB, 0.71380E+00_JPRB, 0.74428E+00_JPRB, 0.76106E+00_JPRB ,&
&  0.89785E+00_JPRB, 0.10358E+01_JPRB, 0.11431E+01_JPRB, 0.12645E+01_JPRB, 0.13041E+01_JPRB, 0.12169E+01_JPRB, 0.12312E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.11450E+01_JPRB, 0.10852E+01_JPRB, 0.10738E+01_JPRB, 0.10845E+01_JPRB, 0.11118E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.12414E+01_JPRB, 0.12423E+01_JPRB, 0.12568E+01_JPRB, 0.13599E+01_JPRB, 0.14902E+01_JPRB, 0.15507E+01_JPRB ,&
&  0.15064E+01_JPRB, 0.15094E+01_JPRB, 0.15639E+01_JPRB, 0.17136E+01_JPRB, 0.19386E+01_JPRB, 0.19394E+01_JPRB, 0.16553E+01_JPRB ,&
&  0.14686E+01_JPRB, 0.15862E+01_JPRB, 0.19888E+01_JPRB, 0.19286E+01_JPRB, 0.19640E+01_JPRB, 0.20442E+01_JPRB, 0.21228E+01_JPRB ,&
&  0.20686E+01_JPRB, 0.21172E+01_JPRB, 0.22340E+01_JPRB, 0.26470E+01_JPRB, 0.31362E+01_JPRB, 0.32361E+01_JPRB, 0.46430E+01_JPRB ,&
&  0.46951E+01_JPRB, 0.42610E+01_JPRB, 0.32199E+01_JPRB, 0.24154E+01_JPRB, 0.24508E+01_JPRB, 0.20159E+01_JPRB, 0.13799E+01_JPRB ,&
&  0.13257E+01_JPRB, 0.12570E+01_JPRB, 0.11075E+01_JPRB, 0.10875E+01_JPRB, 0.10225E+01_JPRB, 0.13946E+01_JPRB, 0.21506E+01_JPRB ,&
&  0.30246E+01_JPRB, 0.35065E+01_JPRB, 0.37039E+01_JPRB, 0.34055E+01_JPRB, 0.30904E+01_JPRB, 0.38857E+01_JPRB, 0.33884E+01_JPRB ,&
&  0.25874E+01_JPRB, 0.23285E+01_JPRB, 0.22598E+01_JPRB, 0.20934E+01_JPRB, 0.20080E+01_JPRB, 0.19340E+01_JPRB, 0.18743E+01_JPRB ,&
&  0.18170E+01_JPRB, 0.17697E+01_JPRB, 0.17190E+01_JPRB, 0.16585E+01_JPRB, 0.16049E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.15794E+01_JPRB, 0.15514E+01_JPRB, 0.15145E+01_JPRB, 0.14626E+01_JPRB, 0.13922E+01_JPRB, 0.13188E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.11514E+01_JPRB, 0.10883E+01_JPRB, 0.10330E+01_JPRB, 0.98068E+00_JPRB, 0.93755E+00_JPRB, 0.91308E+00_JPRB ,&
&  0.89729E+00_JPRB, 0.87856E+00_JPRB, 0.89097E+00_JPRB, 0.87858E+00_JPRB, 0.96257E+00_JPRB, 0.10875E+01_JPRB, 0.11735E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.14003E+01_JPRB, 0.15383E+01_JPRB, 0.15615E+01_JPRB, 0.15572E+01_JPRB, 0.15143E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.13370E+01_JPRB, 0.13085E+01_JPRB, 0.12890E+01_JPRB, 0.12663E+01_JPRB, 0.12467E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.12602E+01_JPRB, 0.12947E+01_JPRB, 0.12685E+01_JPRB, 0.13069E+01_JPRB, 0.14172E+01_JPRB, 0.15292E+01_JPRB, 0.16047E+01_JPRB ,&
&  0.16249E+01_JPRB, 0.16365E+01_JPRB, 0.17101E+01_JPRB, 0.18377E+01_JPRB, 0.17409E+01_JPRB, 0.14686E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.19408E+01_JPRB, 0.18979E+01_JPRB, 0.21680E+01_JPRB, 0.20992E+01_JPRB, 0.20038E+01_JPRB, 0.21969E+01_JPRB ,&
&  0.24268E+01_JPRB, 0.31219E+01_JPRB, 0.27562E+01_JPRB, 0.28070E+01_JPRB, 0.41554E+01_JPRB, 0.40026E+01_JPRB, 0.43475E+01_JPRB ,&
&  0.47095E+01_JPRB, 0.38796E+01_JPRB, 0.31514E+01_JPRB, 0.28021E+01_JPRB, 0.39180E+01_JPRB, 0.39543E+01_JPRB, 0.18685E+01_JPRB ,&
&  0.14828E+01_JPRB, 0.12941E+01_JPRB, 0.11701E+01_JPRB, 0.10570E+01_JPRB, 0.10088E+01_JPRB, 0.14765E+01_JPRB, 0.29518E+01_JPRB ,&
&  0.31897E+01_JPRB, 0.45883E+01_JPRB, 0.46183E+01_JPRB, 0.41581E+01_JPRB, 0.35294E+01_JPRB, 0.36822E+01_JPRB, 0.29667E+01_JPRB ,&
&  0.25658E+01_JPRB, 0.24009E+01_JPRB, 0.22153E+01_JPRB, 0.20596E+01_JPRB, 0.19070E+01_JPRB, 0.18183E+01_JPRB, 0.17801E+01_JPRB ,&
&  0.17458E+01_JPRB, 0.17122E+01_JPRB, 0.16821E+01_JPRB, 0.16453E+01_JPRB, 0.16120E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.15354E+01_JPRB, 0.14978E+01_JPRB, 0.14610E+01_JPRB, 0.14095E+01_JPRB, 0.13596E+01_JPRB, 0.13174E+01_JPRB, 0.12767E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.12002E+01_JPRB, 0.11373E+01_JPRB, 0.10702E+01_JPRB, 0.10092E+01_JPRB, 0.96126E+00_JPRB, 0.93631E+00_JPRB ,&
&  0.91973E+00_JPRB, 0.91675E+00_JPRB, 0.93221E+00_JPRB, 0.10517E+01_JPRB, 0.12384E+01_JPRB, 0.13756E+01_JPRB, 0.16154E+01_JPRB ,&
&  0.18838E+01_JPRB, 0.20158E+01_JPRB, 0.21882E+01_JPRB, 0.21609E+01_JPRB, 0.20589E+01_JPRB, 0.19308E+01_JPRB, 0.17896E+01_JPRB ,&
&  0.16885E+01_JPRB, 0.16441E+01_JPRB, 0.15647E+01_JPRB, 0.14821E+01_JPRB, 0.14058E+01_JPRB, 0.13545E+01_JPRB, 0.13310E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.13350E+01_JPRB, 0.13088E+01_JPRB, 0.13516E+01_JPRB, 0.14492E+01_JPRB, 0.15828E+01_JPRB, 0.16780E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.16965E+01_JPRB, 0.17518E+01_JPRB, 0.18410E+01_JPRB, 0.15254E+01_JPRB, 0.17173E+01_JPRB, 0.17216E+01_JPRB ,&
&  0.17360E+01_JPRB, 0.18071E+01_JPRB, 0.17723E+01_JPRB, 0.20193E+01_JPRB, 0.21166E+01_JPRB, 0.20236E+01_JPRB, 0.21999E+01_JPRB ,&
&  0.28890E+01_JPRB, 0.28092E+01_JPRB, 0.27328E+01_JPRB, 0.32627E+01_JPRB, 0.41697E+01_JPRB, 0.59955E+01_JPRB, 0.39013E+01_JPRB ,&
&  0.32917E+01_JPRB, 0.36945E+01_JPRB, 0.32854E+01_JPRB, 0.30381E+01_JPRB, 0.38747E+01_JPRB, 0.46992E+01_JPRB, 0.39706E+01_JPRB ,&
&  0.17267E+01_JPRB, 0.12549E+01_JPRB, 0.13152E+01_JPRB, 0.13662E+01_JPRB, 0.13115E+01_JPRB, 0.18498E+01_JPRB, 0.46716E+01_JPRB ,&
&  0.41122E+01_JPRB, 0.52513E+01_JPRB, 0.48793E+01_JPRB, 0.41957E+01_JPRB, 0.36321E+01_JPRB, 0.34462E+01_JPRB, 0.30120E+01_JPRB ,&
&  0.25171E+01_JPRB, 0.22416E+01_JPRB, 0.20191E+01_JPRB, 0.18673E+01_JPRB, 0.17821E+01_JPRB, 0.17173E+01_JPRB, 0.16776E+01_JPRB ,&
&  0.16443E+01_JPRB, 0.16191E+01_JPRB, 0.16033E+01_JPRB, 0.15902E+01_JPRB, 0.15692E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.14630E+01_JPRB, 0.14208E+01_JPRB, 0.13773E+01_JPRB, 0.13493E+01_JPRB, 0.13274E+01_JPRB, 0.12923E+01_JPRB, 0.12623E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.11850E+01_JPRB, 0.11323E+01_JPRB, 0.10700E+01_JPRB, 0.10129E+01_JPRB, 0.97259E+00_JPRB, 0.95083E+00_JPRB ,&
&  0.94850E+00_JPRB, 0.94813E+00_JPRB, 0.96399E+00_JPRB, 0.11155E+01_JPRB, 0.14071E+01_JPRB, 0.15027E+01_JPRB, 0.18006E+01_JPRB ,&
&  0.27704E+01_JPRB, 0.30591E+01_JPRB, 0.28629E+01_JPRB, 0.27229E+01_JPRB, 0.25974E+01_JPRB, 0.24783E+01_JPRB, 0.22918E+01_JPRB ,&
&  0.20681E+01_JPRB, 0.18818E+01_JPRB, 0.17017E+01_JPRB, 0.15686E+01_JPRB, 0.14728E+01_JPRB, 0.14166E+01_JPRB, 0.14061E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.13609E+01_JPRB, 0.13351E+01_JPRB, 0.13656E+01_JPRB, 0.14826E+01_JPRB, 0.16108E+01_JPRB, 0.17129E+01_JPRB ,&
&  0.17340E+01_JPRB, 0.17255E+01_JPRB, 0.17343E+01_JPRB, 0.16906E+01_JPRB, 0.16991E+01_JPRB, 0.17228E+01_JPRB, 0.17690E+01_JPRB ,&
&  0.17574E+01_JPRB, 0.18518E+01_JPRB, 0.19576E+01_JPRB, 0.19937E+01_JPRB, 0.20444E+01_JPRB, 0.21150E+01_JPRB, 0.23517E+01_JPRB ,&
&  0.28246E+01_JPRB, 0.29061E+01_JPRB, 0.31029E+01_JPRB, 0.39840E+01_JPRB, 0.45549E+01_JPRB, 0.52124E+01_JPRB, 0.48403E+01_JPRB ,&
&  0.39483E+01_JPRB, 0.30054E+01_JPRB, 0.28232E+01_JPRB, 0.32079E+01_JPRB, 0.36738E+01_JPRB, 0.40500E+01_JPRB, 0.58925E+01_JPRB ,&
&  0.58374E+01_JPRB, 0.38771E+01_JPRB, 0.32168E+01_JPRB, 0.41629E+01_JPRB, 0.35210E+01_JPRB, 0.31570E+01_JPRB, 0.39262E+01_JPRB ,&
&  0.45856E+01_JPRB, 0.54188E+01_JPRB, 0.48985E+01_JPRB, 0.39841E+01_JPRB, 0.36835E+01_JPRB, 0.33091E+01_JPRB, 0.28062E+01_JPRB ,&
&  0.23419E+01_JPRB, 0.21063E+01_JPRB, 0.19450E+01_JPRB, 0.17901E+01_JPRB, 0.16686E+01_JPRB, 0.16001E+01_JPRB, 0.15660E+01_JPRB ,&
&  0.15375E+01_JPRB, 0.15255E+01_JPRB, 0.15230E+01_JPRB, 0.15202E+01_JPRB, 0.14950E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.13618E+01_JPRB, 0.13314E+01_JPRB, 0.13155E+01_JPRB, 0.12784E+01_JPRB, 0.12364E+01_JPRB, 0.12015E+01_JPRB, 0.11885E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.11329E+01_JPRB, 0.10878E+01_JPRB, 0.10401E+01_JPRB, 0.99074E+00_JPRB, 0.96297E+00_JPRB, 0.95285E+00_JPRB ,&
&  0.95784E+00_JPRB, 0.97307E+00_JPRB, 0.10436E+01_JPRB, 0.12461E+01_JPRB, 0.15623E+01_JPRB, 0.20808E+01_JPRB, 0.19366E+01_JPRB ,&
&  0.29103E+01_JPRB, 0.37899E+01_JPRB, 0.35563E+01_JPRB, 0.33314E+01_JPRB, 0.31055E+01_JPRB, 0.29620E+01_JPRB, 0.25829E+01_JPRB ,&
&  0.22231E+01_JPRB, 0.19836E+01_JPRB, 0.17537E+01_JPRB, 0.15971E+01_JPRB, 0.15090E+01_JPRB, 0.14652E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.14001E+01_JPRB, 0.13599E+01_JPRB, 0.13349E+01_JPRB, 0.13693E+01_JPRB, 0.14592E+01_JPRB, 0.16044E+01_JPRB, 0.17003E+01_JPRB ,&
&  0.17427E+01_JPRB, 0.17377E+01_JPRB, 0.17190E+01_JPRB, 0.16204E+01_JPRB, 0.16613E+01_JPRB, 0.17069E+01_JPRB, 0.17700E+01_JPRB ,&
&  0.17305E+01_JPRB, 0.16538E+01_JPRB, 0.18369E+01_JPRB, 0.20232E+01_JPRB, 0.21083E+01_JPRB, 0.21194E+01_JPRB, 0.23280E+01_JPRB ,&
&  0.29222E+01_JPRB, 0.34717E+01_JPRB, 0.43807E+01_JPRB, 0.46879E+01_JPRB, 0.48681E+01_JPRB, 0.49586E+01_JPRB, 0.48935E+01_JPRB ,&
&  0.48311E+01_JPRB, 0.38886E+01_JPRB, 0.34850E+01_JPRB, 0.34319E+01_JPRB, 0.34320E+01_JPRB, 0.37821E+01_JPRB, 0.51936E+01_JPRB ,&
&  0.65445E+01_JPRB, 0.73644E+01_JPRB, 0.75374E+01_JPRB, 0.63479E+01_JPRB, 0.53748E+01_JPRB, 0.45488E+01_JPRB, 0.42401E+01_JPRB ,&
&  0.50837E+01_JPRB, 0.51880E+01_JPRB, 0.45818E+01_JPRB, 0.47433E+01_JPRB, 0.36160E+01_JPRB, 0.32322E+01_JPRB, 0.27171E+01_JPRB ,&
&  0.23381E+01_JPRB, 0.20642E+01_JPRB, 0.18526E+01_JPRB, 0.16986E+01_JPRB, 0.15962E+01_JPRB, 0.15047E+01_JPRB, 0.14464E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.14234E+01_JPRB, 0.14326E+01_JPRB, 0.14280E+01_JPRB, 0.13957E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.13049E+01_JPRB, 0.12880E+01_JPRB, 0.12629E+01_JPRB, 0.12149E+01_JPRB, 0.11784E+01_JPRB, 0.11555E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.11026E+01_JPRB, 0.10787E+01_JPRB, 0.10405E+01_JPRB, 0.99894E+00_JPRB, 0.96711E+00_JPRB, 0.94576E+00_JPRB, 0.94930E+00_JPRB ,&
&  0.99828E+00_JPRB, 0.10984E+01_JPRB, 0.12780E+01_JPRB, 0.14855E+01_JPRB, 0.18270E+01_JPRB, 0.23691E+01_JPRB, 0.27272E+01_JPRB ,&
&  0.31027E+01_JPRB, 0.46422E+01_JPRB, 0.38760E+01_JPRB, 0.35285E+01_JPRB, 0.32814E+01_JPRB, 0.30527E+01_JPRB, 0.25830E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.18734E+01_JPRB, 0.17410E+01_JPRB, 0.16301E+01_JPRB, 0.15576E+01_JPRB, 0.14829E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.14236E+01_JPRB, 0.13976E+01_JPRB, 0.13722E+01_JPRB, 0.14135E+01_JPRB, 0.15170E+01_JPRB, 0.16321E+01_JPRB, 0.17335E+01_JPRB ,&
&  0.17739E+01_JPRB, 0.17551E+01_JPRB, 0.16791E+01_JPRB, 0.16395E+01_JPRB, 0.16303E+01_JPRB, 0.16974E+01_JPRB, 0.17850E+01_JPRB ,&
&  0.18037E+01_JPRB, 0.17398E+01_JPRB, 0.18457E+01_JPRB, 0.19120E+01_JPRB, 0.19796E+01_JPRB, 0.22172E+01_JPRB, 0.24149E+01_JPRB ,&
&  0.30416E+01_JPRB, 0.31899E+01_JPRB, 0.46507E+01_JPRB, 0.43614E+01_JPRB, 0.43276E+01_JPRB, 0.45142E+01_JPRB, 0.42165E+01_JPRB ,&
&  0.36045E+01_JPRB, 0.33986E+01_JPRB, 0.31532E+01_JPRB, 0.32026E+01_JPRB, 0.35114E+01_JPRB, 0.41480E+01_JPRB, 0.53321E+01_JPRB ,&
&  0.57539E+01_JPRB, 0.55662E+01_JPRB, 0.56008E+01_JPRB, 0.46819E+01_JPRB, 0.44784E+01_JPRB, 0.41719E+01_JPRB, 0.40756E+01_JPRB ,&
&  0.49098E+01_JPRB, 0.48713E+01_JPRB, 0.45775E+01_JPRB, 0.42187E+01_JPRB, 0.31923E+01_JPRB, 0.27637E+01_JPRB, 0.23239E+01_JPRB ,&
&  0.20411E+01_JPRB, 0.18120E+01_JPRB, 0.16851E+01_JPRB, 0.15808E+01_JPRB, 0.14984E+01_JPRB, 0.14120E+01_JPRB, 0.13708E+01_JPRB ,&
&  0.13523E+01_JPRB, 0.13522E+01_JPRB, 0.13573E+01_JPRB, 0.13445E+01_JPRB, 0.13225E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.13060E+01_JPRB, 0.13071E+01_JPRB, 0.13035E+01_JPRB, 0.13227E+01_JPRB, 0.13636E+01_JPRB, 0.14284E+01_JPRB, 0.14607E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.10276E+01_JPRB, 0.10033E+01_JPRB, 0.97673E+00_JPRB, 0.95005E+00_JPRB, 0.95379E+00_JPRB, 0.10163E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.12608E+01_JPRB, 0.14226E+01_JPRB, 0.16934E+01_JPRB, 0.21018E+01_JPRB, 0.28020E+01_JPRB, 0.35511E+01_JPRB ,&
&  0.37388E+01_JPRB, 0.40181E+01_JPRB, 0.39243E+01_JPRB, 0.33499E+01_JPRB, 0.30343E+01_JPRB, 0.28655E+01_JPRB, 0.26659E+01_JPRB ,&
&  0.21897E+01_JPRB, 0.19359E+01_JPRB, 0.18801E+01_JPRB, 0.18716E+01_JPRB, 0.15414E+01_JPRB, 0.15054E+01_JPRB, 0.14921E+01_JPRB ,&
&  0.14870E+01_JPRB, 0.14973E+01_JPRB, 0.15091E+01_JPRB, 0.15646E+01_JPRB, 0.16553E+01_JPRB, 0.17545E+01_JPRB, 0.18309E+01_JPRB ,&
&  0.18605E+01_JPRB, 0.18434E+01_JPRB, 0.17912E+01_JPRB, 0.16748E+01_JPRB, 0.16647E+01_JPRB, 0.17254E+01_JPRB, 0.18066E+01_JPRB ,&
&  0.18566E+01_JPRB, 0.19175E+01_JPRB, 0.19624E+01_JPRB, 0.20090E+01_JPRB, 0.20223E+01_JPRB, 0.23964E+01_JPRB, 0.26597E+01_JPRB ,&
&  0.30678E+01_JPRB, 0.35716E+01_JPRB, 0.50108E+01_JPRB, 0.36913E+01_JPRB, 0.38729E+01_JPRB, 0.39817E+01_JPRB, 0.37309E+01_JPRB ,&
&  0.32247E+01_JPRB, 0.30604E+01_JPRB, 0.28646E+01_JPRB, 0.29522E+01_JPRB, 0.35528E+01_JPRB, 0.43195E+01_JPRB, 0.57306E+01_JPRB ,&
&  0.48079E+01_JPRB, 0.48197E+01_JPRB, 0.41301E+01_JPRB, 0.41383E+01_JPRB, 0.40437E+01_JPRB, 0.38103E+01_JPRB, 0.35461E+01_JPRB ,&
&  0.41910E+01_JPRB, 0.38899E+01_JPRB, 0.34279E+01_JPRB, 0.29614E+01_JPRB, 0.22553E+01_JPRB, 0.19392E+01_JPRB, 0.16601E+01_JPRB ,&
&  0.15404E+01_JPRB, 0.14818E+01_JPRB, 0.14720E+01_JPRB, 0.14216E+01_JPRB, 0.13872E+01_JPRB, 0.13529E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.13206E+01_JPRB, 0.13140E+01_JPRB, 0.13082E+01_JPRB, 0.13028E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.13457E+01_JPRB, 0.13283E+01_JPRB, 0.13113E+01_JPRB, 0.12997E+01_JPRB, 0.12886E+01_JPRB, 0.12565E+01_JPRB, 0.11890E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.10250E+01_JPRB, 0.10055E+01_JPRB, 0.99195E+00_JPRB, 0.99506E+00_JPRB, 0.10361E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.12049E+01_JPRB, 0.13637E+01_JPRB, 0.15827E+01_JPRB, 0.18833E+01_JPRB, 0.22915E+01_JPRB, 0.29478E+01_JPRB, 0.37047E+01_JPRB ,&
&  0.38379E+01_JPRB, 0.28550E+01_JPRB, 0.32000E+01_JPRB, 0.30872E+01_JPRB, 0.28991E+01_JPRB, 0.26908E+01_JPRB, 0.23410E+01_JPRB ,&
&  0.19775E+01_JPRB, 0.18728E+01_JPRB, 0.18361E+01_JPRB, 0.17449E+01_JPRB, 0.15554E+01_JPRB, 0.15887E+01_JPRB, 0.16328E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.17174E+01_JPRB, 0.17249E+01_JPRB, 0.17842E+01_JPRB, 0.18442E+01_JPRB, 0.19350E+01_JPRB, 0.20342E+01_JPRB ,&
&  0.20749E+01_JPRB, 0.20838E+01_JPRB, 0.20637E+01_JPRB, 0.19239E+01_JPRB, 0.18835E+01_JPRB, 0.19638E+01_JPRB, 0.20408E+01_JPRB ,&
&  0.20874E+01_JPRB, 0.20693E+01_JPRB, 0.21720E+01_JPRB, 0.24350E+01_JPRB, 0.27600E+01_JPRB, 0.26502E+01_JPRB, 0.31043E+01_JPRB ,&
&  0.34268E+01_JPRB, 0.40432E+01_JPRB, 0.39500E+01_JPRB, 0.45387E+01_JPRB, 0.30035E+01_JPRB, 0.32576E+01_JPRB, 0.29183E+01_JPRB ,&
&  0.29988E+01_JPRB, 0.27504E+01_JPRB, 0.27709E+01_JPRB, 0.31608E+01_JPRB, 0.39740E+01_JPRB, 0.49929E+01_JPRB, 0.51329E+01_JPRB ,&
&  0.36851E+01_JPRB, 0.33579E+01_JPRB, 0.35129E+01_JPRB, 0.39373E+01_JPRB, 0.39338E+01_JPRB, 0.37544E+01_JPRB, 0.31583E+01_JPRB ,&
&  0.31427E+01_JPRB, 0.27655E+01_JPRB, 0.24720E+01_JPRB, 0.21105E+01_JPRB, 0.15378E+01_JPRB, 0.13542E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.13700E+01_JPRB, 0.13803E+01_JPRB, 0.13863E+01_JPRB, 0.13667E+01_JPRB, 0.13440E+01_JPRB, 0.13282E+01_JPRB ,&
&  0.13212E+01_JPRB, 0.13250E+01_JPRB, 0.13294E+01_JPRB, 0.13303E+01_JPRB, 0.13392E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.12954E+01_JPRB, 0.12642E+01_JPRB, 0.12349E+01_JPRB, 0.12090E+01_JPRB, 0.11905E+01_JPRB, 0.11454E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.10337E+01_JPRB, 0.10324E+01_JPRB, 0.10435E+01_JPRB, 0.10768E+01_JPRB, 0.11151E+01_JPRB, 0.11854E+01_JPRB ,&
&  0.13059E+01_JPRB, 0.14809E+01_JPRB, 0.17188E+01_JPRB, 0.20148E+01_JPRB, 0.23985E+01_JPRB, 0.28547E+01_JPRB, 0.32078E+01_JPRB ,&
&  0.35596E+01_JPRB, 0.36598E+01_JPRB, 0.30760E+01_JPRB, 0.22322E+01_JPRB, 0.21267E+01_JPRB, 0.21714E+01_JPRB, 0.19295E+01_JPRB ,&
&  0.18772E+01_JPRB, 0.18054E+01_JPRB, 0.17381E+01_JPRB, 0.17315E+01_JPRB, 0.17402E+01_JPRB, 0.18067E+01_JPRB, 0.18669E+01_JPRB ,&
&  0.19103E+01_JPRB, 0.19453E+01_JPRB, 0.19939E+01_JPRB, 0.20900E+01_JPRB, 0.21798E+01_JPRB, 0.22842E+01_JPRB, 0.23644E+01_JPRB ,&
&  0.24155E+01_JPRB, 0.24942E+01_JPRB, 0.24935E+01_JPRB, 0.24136E+01_JPRB, 0.23263E+01_JPRB, 0.24064E+01_JPRB, 0.25091E+01_JPRB ,&
&  0.25961E+01_JPRB, 0.26311E+01_JPRB, 0.27134E+01_JPRB, 0.30546E+01_JPRB, 0.33565E+01_JPRB, 0.31221E+01_JPRB, 0.32675E+01_JPRB ,&
&  0.38257E+01_JPRB, 0.42323E+01_JPRB, 0.32270E+01_JPRB, 0.38304E+01_JPRB, 0.22723E+01_JPRB, 0.28099E+01_JPRB, 0.29907E+01_JPRB ,&
&  0.27735E+01_JPRB, 0.28203E+01_JPRB, 0.29761E+01_JPRB, 0.35117E+01_JPRB, 0.43009E+01_JPRB, 0.48082E+01_JPRB, 0.41468E+01_JPRB ,&
&  0.29657E+01_JPRB, 0.28698E+01_JPRB, 0.30772E+01_JPRB, 0.33090E+01_JPRB, 0.33469E+01_JPRB, 0.30894E+01_JPRB, 0.22790E+01_JPRB ,&
&  0.17404E+01_JPRB, 0.20258E+01_JPRB, 0.20161E+01_JPRB, 0.19180E+01_JPRB, 0.13692E+01_JPRB, 0.11892E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.13815E+01_JPRB, 0.14066E+01_JPRB, 0.14000E+01_JPRB, 0.13743E+01_JPRB, 0.13510E+01_JPRB, 0.13444E+01_JPRB ,&
&  0.13433E+01_JPRB, 0.13347E+01_JPRB, 0.13330E+01_JPRB, 0.13343E+01_JPRB, 0.13232E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.11842E+01_JPRB, 0.11445E+01_JPRB, 0.11199E+01_JPRB, 0.11021E+01_JPRB, 0.10962E+01_JPRB, 0.10696E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.10063E+01_JPRB, 0.99965E+00_JPRB, 0.10160E+01_JPRB, 0.10567E+01_JPRB, 0.11121E+01_JPRB, 0.11665E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.14025E+01_JPRB, 0.15941E+01_JPRB, 0.18423E+01_JPRB, 0.22108E+01_JPRB, 0.25266E+01_JPRB, 0.28348E+01_JPRB, 0.29349E+01_JPRB ,&
&  0.30794E+01_JPRB, 0.31155E+01_JPRB, 0.29426E+01_JPRB, 0.27630E+01_JPRB, 0.22220E+01_JPRB, 0.22510E+01_JPRB, 0.24234E+01_JPRB ,&
&  0.25401E+01_JPRB, 0.21852E+01_JPRB, 0.19158E+01_JPRB, 0.19428E+01_JPRB, 0.19663E+01_JPRB, 0.20471E+01_JPRB, 0.21238E+01_JPRB ,&
&  0.21869E+01_JPRB, 0.22542E+01_JPRB, 0.23253E+01_JPRB, 0.24289E+01_JPRB, 0.25419E+01_JPRB, 0.26106E+01_JPRB, 0.26761E+01_JPRB ,&
&  0.27198E+01_JPRB, 0.27734E+01_JPRB, 0.28227E+01_JPRB, 0.28355E+01_JPRB, 0.29154E+01_JPRB, 0.30397E+01_JPRB, 0.32574E+01_JPRB ,&
&  0.34821E+01_JPRB, 0.34803E+01_JPRB, 0.37360E+01_JPRB, 0.41585E+01_JPRB, 0.42253E+01_JPRB, 0.40748E+01_JPRB, 0.40569E+01_JPRB ,&
&  0.44716E+01_JPRB, 0.48454E+01_JPRB, 0.26791E+01_JPRB, 0.29539E+01_JPRB, 0.34304E+01_JPRB, 0.29680E+01_JPRB, 0.27122E+01_JPRB ,&
&  0.27736E+01_JPRB, 0.29318E+01_JPRB, 0.32389E+01_JPRB, 0.36278E+01_JPRB, 0.42693E+01_JPRB, 0.53814E+01_JPRB, 0.36682E+01_JPRB ,&
&  0.24521E+01_JPRB, 0.23224E+01_JPRB, 0.24518E+01_JPRB, 0.25717E+01_JPRB, 0.25747E+01_JPRB, 0.22166E+01_JPRB, 0.15553E+01_JPRB ,&
&  0.13670E+01_JPRB, 0.18450E+01_JPRB, 0.19447E+01_JPRB, 0.19726E+01_JPRB, 0.16871E+01_JPRB, 0.11464E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.13651E+01_JPRB, 0.13517E+01_JPRB, 0.13480E+01_JPRB, 0.13442E+01_JPRB, 0.13343E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.13292E+01_JPRB, 0.13263E+01_JPRB, 0.13239E+01_JPRB, 0.12892E+01_JPRB, 0.12300E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.10642E+01_JPRB, 0.10339E+01_JPRB, 0.10172E+01_JPRB, 0.99348E+00_JPRB, 0.97840E+00_JPRB, 0.95891E+00_JPRB, 0.94985E+00_JPRB ,&
&  0.94972E+00_JPRB, 0.95790E+00_JPRB, 0.98448E+00_JPRB, 0.10254E+01_JPRB, 0.10835E+01_JPRB, 0.11628E+01_JPRB, 0.13028E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.16707E+01_JPRB, 0.19492E+01_JPRB, 0.22822E+01_JPRB, 0.25542E+01_JPRB, 0.26782E+01_JPRB, 0.28401E+01_JPRB ,&
&  0.29558E+01_JPRB, 0.30034E+01_JPRB, 0.29751E+01_JPRB, 0.29252E+01_JPRB, 0.28460E+01_JPRB, 0.29065E+01_JPRB, 0.30859E+01_JPRB ,&
&  0.28738E+01_JPRB, 0.22540E+01_JPRB, 0.21068E+01_JPRB, 0.21135E+01_JPRB, 0.21869E+01_JPRB, 0.22482E+01_JPRB, 0.23487E+01_JPRB ,&
&  0.24335E+01_JPRB, 0.25012E+01_JPRB, 0.25799E+01_JPRB, 0.26698E+01_JPRB, 0.28047E+01_JPRB, 0.28998E+01_JPRB, 0.30136E+01_JPRB ,&
&  0.31209E+01_JPRB, 0.32323E+01_JPRB, 0.34132E+01_JPRB, 0.32746E+01_JPRB, 0.36272E+01_JPRB, 0.38957E+01_JPRB, 0.41527E+01_JPRB ,&
&  0.42613E+01_JPRB, 0.42577E+01_JPRB, 0.46232E+01_JPRB, 0.49089E+01_JPRB, 0.51126E+01_JPRB, 0.51730E+01_JPRB, 0.50318E+01_JPRB ,&
&  0.52392E+01_JPRB, 0.48740E+01_JPRB, 0.26674E+01_JPRB, 0.23268E+01_JPRB, 0.25156E+01_JPRB, 0.22683E+01_JPRB, 0.26736E+01_JPRB ,&
&  0.27518E+01_JPRB, 0.29542E+01_JPRB, 0.32096E+01_JPRB, 0.35701E+01_JPRB, 0.39964E+01_JPRB, 0.48859E+01_JPRB, 0.32790E+01_JPRB ,&
&  0.18698E+01_JPRB, 0.17404E+01_JPRB, 0.17719E+01_JPRB, 0.18259E+01_JPRB, 0.18876E+01_JPRB, 0.17164E+01_JPRB, 0.14888E+01_JPRB ,&
&  0.15035E+01_JPRB, 0.15974E+01_JPRB, 0.16628E+01_JPRB, 0.14915E+01_JPRB, 0.11970E+01_JPRB, 0.10816E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.12373E+01_JPRB, 0.12374E+01_JPRB, 0.12488E+01_JPRB, 0.12445E+01_JPRB, 0.12393E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.12158E+01_JPRB, 0.11916E+01_JPRB, 0.11392E+01_JPRB, 0.10945E+01_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.90575E+00_JPRB, 0.90563E+00_JPRB, 0.88994E+00_JPRB, 0.87130E+00_JPRB, 0.86200E+00_JPRB, 0.84189E+00_JPRB, 0.86699E+00_JPRB ,&
&  0.86779E+00_JPRB, 0.89646E+00_JPRB, 0.92611E+00_JPRB, 0.95925E+00_JPRB, 0.10179E+01_JPRB, 0.10810E+01_JPRB, 0.11728E+01_JPRB ,&
&  0.13296E+01_JPRB, 0.15789E+01_JPRB, 0.19023E+01_JPRB, 0.23084E+01_JPRB, 0.24866E+01_JPRB, 0.26170E+01_JPRB, 0.27926E+01_JPRB ,&
&  0.29990E+01_JPRB, 0.31753E+01_JPRB, 0.32916E+01_JPRB, 0.34333E+01_JPRB, 0.35986E+01_JPRB, 0.36012E+01_JPRB, 0.34980E+01_JPRB ,&
&  0.29210E+01_JPRB, 0.21561E+01_JPRB, 0.22990E+01_JPRB, 0.21596E+01_JPRB, 0.23033E+01_JPRB, 0.24349E+01_JPRB, 0.25393E+01_JPRB ,&
&  0.26483E+01_JPRB, 0.27296E+01_JPRB, 0.28409E+01_JPRB, 0.29719E+01_JPRB, 0.31434E+01_JPRB, 0.33125E+01_JPRB, 0.34902E+01_JPRB ,&
&  0.36362E+01_JPRB, 0.37772E+01_JPRB, 0.38782E+01_JPRB, 0.38339E+01_JPRB, 0.41164E+01_JPRB, 0.44919E+01_JPRB, 0.48424E+01_JPRB ,&
&  0.51247E+01_JPRB, 0.51245E+01_JPRB, 0.46195E+01_JPRB, 0.48567E+01_JPRB, 0.52199E+01_JPRB, 0.50052E+01_JPRB, 0.52080E+01_JPRB ,&
&  0.57680E+01_JPRB, 0.50882E+01_JPRB, 0.24817E+01_JPRB, 0.21404E+01_JPRB, 0.23013E+01_JPRB, 0.24099E+01_JPRB, 0.25126E+01_JPRB ,&
&  0.26018E+01_JPRB, 0.27090E+01_JPRB, 0.28702E+01_JPRB, 0.30573E+01_JPRB, 0.31432E+01_JPRB, 0.32390E+01_JPRB, 0.26193E+01_JPRB ,&
&  0.15340E+01_JPRB, 0.14507E+01_JPRB, 0.15376E+01_JPRB, 0.16480E+01_JPRB, 0.19121E+01_JPRB, 0.18642E+01_JPRB, 0.14812E+01_JPRB ,&
&  0.15898E+01_JPRB, 0.18774E+01_JPRB, 0.22644E+01_JPRB, 0.18800E+01_JPRB, 0.11575E+01_JPRB, 0.99293E+00_JPRB, 0.98946E+00_JPRB ,&
&  0.10342E+01_JPRB, 0.10393E+01_JPRB, 0.10623E+01_JPRB, 0.10788E+01_JPRB, 0.11000E+01_JPRB, 0.10860E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10224E+01_JPRB, 0.99399E+00_JPRB, 0.95288E+00_JPRB, 0.92282E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.76739E+00_JPRB, 0.73091E+00_JPRB, 0.69037E+00_JPRB, 0.68519E+00_JPRB, 0.68814E+00_JPRB, 0.67274E+00_JPRB, 0.71169E+00_JPRB ,&
&  0.68731E+00_JPRB, 0.72077E+00_JPRB, 0.74025E+00_JPRB, 0.74780E+00_JPRB, 0.80361E+00_JPRB, 0.86066E+00_JPRB, 0.93349E+00_JPRB ,&
&  0.10328E+01_JPRB, 0.11865E+01_JPRB, 0.13837E+01_JPRB, 0.16340E+01_JPRB, 0.19420E+01_JPRB, 0.21077E+01_JPRB, 0.22716E+01_JPRB ,&
&  0.24950E+01_JPRB, 0.27872E+01_JPRB, 0.31965E+01_JPRB, 0.36687E+01_JPRB, 0.42270E+01_JPRB, 0.41089E+01_JPRB, 0.35427E+01_JPRB ,&
&  0.26458E+01_JPRB, 0.21665E+01_JPRB, 0.20423E+01_JPRB, 0.19244E+01_JPRB, 0.20989E+01_JPRB, 0.24297E+01_JPRB, 0.26081E+01_JPRB ,&
&  0.27359E+01_JPRB, 0.28810E+01_JPRB, 0.30400E+01_JPRB, 0.32152E+01_JPRB, 0.34061E+01_JPRB, 0.36229E+01_JPRB, 0.37486E+01_JPRB ,&
&  0.38361E+01_JPRB, 0.39261E+01_JPRB, 0.40064E+01_JPRB, 0.42469E+01_JPRB, 0.45858E+01_JPRB, 0.50314E+01_JPRB, 0.51684E+01_JPRB ,&
&  0.52095E+01_JPRB, 0.51108E+01_JPRB, 0.44540E+01_JPRB, 0.44291E+01_JPRB, 0.45641E+01_JPRB, 0.46170E+01_JPRB, 0.47174E+01_JPRB ,&
&  0.51862E+01_JPRB, 0.41131E+01_JPRB, 0.24833E+01_JPRB, 0.23979E+01_JPRB, 0.24387E+01_JPRB, 0.25296E+01_JPRB, 0.22819E+01_JPRB ,&
&  0.23011E+01_JPRB, 0.23324E+01_JPRB, 0.23982E+01_JPRB, 0.23865E+01_JPRB, 0.23893E+01_JPRB, 0.23157E+01_JPRB, 0.20326E+01_JPRB ,&
&  0.15805E+01_JPRB, 0.14587E+01_JPRB, 0.14271E+01_JPRB, 0.14926E+01_JPRB, 0.18994E+01_JPRB, 0.18855E+01_JPRB, 0.17018E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.17284E+01_JPRB, 0.15207E+01_JPRB, 0.18109E+01_JPRB, 0.10102E+01_JPRB, 0.85597E+00_JPRB, 0.83388E+00_JPRB ,&
&  0.86215E+00_JPRB, 0.87086E+00_JPRB, 0.88854E+00_JPRB, 0.87721E+00_JPRB, 0.89161E+00_JPRB, 0.92668E+00_JPRB, 0.93385E+00_JPRB ,&
&  0.88570E+00_JPRB, 0.86990E+00_JPRB, 0.83343E+00_JPRB, 0.79196E+00_JPRB, 0.78164E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.64255E+00_JPRB, 0.63219E+00_JPRB, 0.62719E+00_JPRB, 0.62153E+00_JPRB, 0.63170E+00_JPRB, 0.63656E+00_JPRB, 0.64299E+00_JPRB ,&
&  0.64812E+00_JPRB, 0.66839E+00_JPRB, 0.68602E+00_JPRB, 0.70978E+00_JPRB, 0.73511E+00_JPRB, 0.78176E+00_JPRB, 0.83375E+00_JPRB ,&
&  0.89452E+00_JPRB, 0.96228E+00_JPRB, 0.10385E+01_JPRB, 0.11501E+01_JPRB, 0.13084E+01_JPRB, 0.14291E+01_JPRB, 0.15615E+01_JPRB ,&
&  0.17384E+01_JPRB, 0.20037E+01_JPRB, 0.23108E+01_JPRB, 0.27274E+01_JPRB, 0.32147E+01_JPRB, 0.33893E+01_JPRB, 0.32022E+01_JPRB ,&
&  0.24177E+01_JPRB, 0.23240E+01_JPRB, 0.21708E+01_JPRB, 0.19972E+01_JPRB, 0.20661E+01_JPRB, 0.20416E+01_JPRB, 0.21320E+01_JPRB ,&
&  0.22578E+01_JPRB, 0.23117E+01_JPRB, 0.23442E+01_JPRB, 0.24137E+01_JPRB, 0.24733E+01_JPRB, 0.25267E+01_JPRB, 0.25184E+01_JPRB ,&
&  0.25329E+01_JPRB, 0.26647E+01_JPRB, 0.31311E+01_JPRB, 0.37119E+01_JPRB, 0.41384E+01_JPRB, 0.43616E+01_JPRB, 0.41630E+01_JPRB ,&
&  0.38570E+01_JPRB, 0.36677E+01_JPRB, 0.34680E+01_JPRB, 0.34326E+01_JPRB, 0.35439E+01_JPRB, 0.36053E+01_JPRB, 0.38687E+01_JPRB ,&
&  0.36761E+01_JPRB, 0.30155E+01_JPRB, 0.26994E+01_JPRB, 0.25085E+01_JPRB, 0.24536E+01_JPRB, 0.23561E+01_JPRB, 0.21750E+01_JPRB ,&
&  0.22090E+01_JPRB, 0.21304E+01_JPRB, 0.20868E+01_JPRB, 0.19484E+01_JPRB, 0.17837E+01_JPRB, 0.16415E+01_JPRB, 0.15602E+01_JPRB ,&
&  0.14134E+01_JPRB, 0.13075E+01_JPRB, 0.13231E+01_JPRB, 0.15185E+01_JPRB, 0.18816E+01_JPRB, 0.30616E+01_JPRB, 0.26630E+01_JPRB ,&
&  0.15174E+01_JPRB, 0.12873E+01_JPRB, 0.87299E+00_JPRB, 0.11362E+01_JPRB, 0.81122E+00_JPRB, 0.73872E+00_JPRB, 0.73220E+00_JPRB ,&
&  0.73733E+00_JPRB, 0.73206E+00_JPRB, 0.71214E+00_JPRB, 0.70827E+00_JPRB, 0.71992E+00_JPRB, 0.73116E+00_JPRB, 0.72288E+00_JPRB ,&
&  0.68499E+00_JPRB, 0.66841E+00_JPRB, 0.69944E+00_JPRB, 0.65360E+00_JPRB, 0.64351E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.59014E+00_JPRB, 0.59153E+00_JPRB, 0.60866E+00_JPRB, 0.62482E+00_JPRB, 0.63565E+00_JPRB, 0.64696E+00_JPRB, 0.66345E+00_JPRB ,&
&  0.67666E+00_JPRB, 0.69272E+00_JPRB, 0.70979E+00_JPRB, 0.73610E+00_JPRB, 0.77157E+00_JPRB, 0.82336E+00_JPRB, 0.88273E+00_JPRB ,&
&  0.95627E+00_JPRB, 0.10361E+01_JPRB, 0.11274E+01_JPRB, 0.12281E+01_JPRB, 0.13529E+01_JPRB, 0.15270E+01_JPRB, 0.17655E+01_JPRB ,&
&  0.19088E+01_JPRB, 0.20937E+01_JPRB, 0.23282E+01_JPRB, 0.25474E+01_JPRB, 0.26140E+01_JPRB, 0.25826E+01_JPRB, 0.21730E+01_JPRB ,&
&  0.25872E+01_JPRB, 0.23313E+01_JPRB, 0.21230E+01_JPRB, 0.19382E+01_JPRB, 0.18109E+01_JPRB, 0.17086E+01_JPRB, 0.16255E+01_JPRB ,&
&  0.16037E+01_JPRB, 0.15404E+01_JPRB, 0.14629E+01_JPRB, 0.14141E+01_JPRB, 0.13895E+01_JPRB, 0.14102E+01_JPRB, 0.14055E+01_JPRB ,&
&  0.15599E+01_JPRB, 0.19217E+01_JPRB, 0.24014E+01_JPRB, 0.29919E+01_JPRB, 0.33343E+01_JPRB, 0.33125E+01_JPRB, 0.27618E+01_JPRB ,&
&  0.24406E+01_JPRB, 0.23264E+01_JPRB, 0.23148E+01_JPRB, 0.24630E+01_JPRB, 0.24846E+01_JPRB, 0.26162E+01_JPRB, 0.27053E+01_JPRB ,&
&  0.26874E+01_JPRB, 0.23541E+01_JPRB, 0.22951E+01_JPRB, 0.26856E+01_JPRB, 0.25087E+01_JPRB, 0.22710E+01_JPRB, 0.21977E+01_JPRB ,&
&  0.20775E+01_JPRB, 0.19978E+01_JPRB, 0.19423E+01_JPRB, 0.18072E+01_JPRB, 0.16304E+01_JPRB, 0.14593E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.12932E+01_JPRB, 0.11820E+01_JPRB, 0.11485E+01_JPRB, 0.12807E+01_JPRB, 0.16577E+01_JPRB, 0.20982E+01_JPRB, 0.29689E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.92638E+00_JPRB, 0.90601E+00_JPRB, 0.86171E+00_JPRB, 0.75070E+00_JPRB, 0.67233E+00_JPRB, 0.71811E+00_JPRB ,&
&  0.78188E+00_JPRB, 0.84889E+00_JPRB, 0.84026E+00_JPRB, 0.73037E+00_JPRB, 0.68832E+00_JPRB, 0.67007E+00_JPRB, 0.64631E+00_JPRB ,&
&  0.62668E+00_JPRB, 0.58990E+00_JPRB, 0.60631E+00_JPRB, 0.59032E+00_JPRB, 0.59054E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.55396E+00_JPRB, 0.55381E+00_JPRB, 0.56439E+00_JPRB, 0.57462E+00_JPRB, 0.57868E+00_JPRB, 0.58363E+00_JPRB, 0.58901E+00_JPRB ,&
&  0.59515E+00_JPRB, 0.59779E+00_JPRB, 0.60223E+00_JPRB, 0.61151E+00_JPRB, 0.63037E+00_JPRB, 0.67159E+00_JPRB, 0.72148E+00_JPRB ,&
&  0.76768E+00_JPRB, 0.81380E+00_JPRB, 0.86255E+00_JPRB, 0.90655E+00_JPRB, 0.95374E+00_JPRB, 0.99105E+00_JPRB, 0.10776E+01_JPRB ,&
&  0.11889E+01_JPRB, 0.13232E+01_JPRB, 0.14662E+01_JPRB, 0.15865E+01_JPRB, 0.17118E+01_JPRB, 0.17422E+01_JPRB, 0.15096E+01_JPRB ,&
&  0.22198E+01_JPRB, 0.21316E+01_JPRB, 0.19550E+01_JPRB, 0.17691E+01_JPRB, 0.16330E+01_JPRB, 0.15096E+01_JPRB, 0.13864E+01_JPRB ,&
&  0.12757E+01_JPRB, 0.13562E+01_JPRB, 0.12999E+01_JPRB, 0.13757E+01_JPRB, 0.11958E+01_JPRB, 0.12584E+01_JPRB, 0.15165E+01_JPRB ,&
&  0.17392E+01_JPRB, 0.19942E+01_JPRB, 0.23121E+01_JPRB, 0.26263E+01_JPRB, 0.28154E+01_JPRB, 0.27701E+01_JPRB, 0.24793E+01_JPRB ,&
&  0.23146E+01_JPRB, 0.22713E+01_JPRB, 0.22044E+01_JPRB, 0.24214E+01_JPRB, 0.23271E+01_JPRB, 0.21774E+01_JPRB, 0.22192E+01_JPRB ,&
&  0.19637E+01_JPRB, 0.19678E+01_JPRB, 0.22141E+01_JPRB, 0.26509E+01_JPRB, 0.24613E+01_JPRB, 0.23186E+01_JPRB, 0.22149E+01_JPRB ,&
&  0.20709E+01_JPRB, 0.19523E+01_JPRB, 0.18595E+01_JPRB, 0.17184E+01_JPRB, 0.15556E+01_JPRB, 0.13905E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.11894E+01_JPRB, 0.10923E+01_JPRB, 0.10330E+01_JPRB, 0.10489E+01_JPRB, 0.11900E+01_JPRB, 0.14197E+01_JPRB, 0.24741E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.12136E+01_JPRB, 0.13095E+01_JPRB, 0.54636E+00_JPRB, 0.62129E+00_JPRB, 0.63062E+00_JPRB, 0.73600E+00_JPRB ,&
&  0.88979E+00_JPRB, 0.13945E+01_JPRB, 0.20917E+01_JPRB, 0.18691E+01_JPRB, 0.14356E+01_JPRB, 0.94016E+00_JPRB, 0.58596E+00_JPRB ,&
&  0.56690E+00_JPRB, 0.56281E+00_JPRB, 0.56899E+00_JPRB, 0.56319E+00_JPRB, 0.56107E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.50131E+00_JPRB, 0.49303E+00_JPRB, 0.50359E+00_JPRB, 0.51507E+00_JPRB, 0.51213E+00_JPRB, 0.51283E+00_JPRB, 0.50796E+00_JPRB ,&
&  0.49880E+00_JPRB, 0.49204E+00_JPRB, 0.51589E+00_JPRB, 0.56095E+00_JPRB, 0.62504E+00_JPRB, 0.70939E+00_JPRB, 0.79641E+00_JPRB ,&
&  0.86201E+00_JPRB, 0.94223E+00_JPRB, 0.10178E+01_JPRB, 0.10819E+01_JPRB, 0.11039E+01_JPRB, 0.11217E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11707E+01_JPRB, 0.12691E+01_JPRB, 0.13790E+01_JPRB, 0.15586E+01_JPRB, 0.18245E+01_JPRB, 0.12026E+01_JPRB ,&
&  0.18073E+01_JPRB, 0.18077E+01_JPRB, 0.17452E+01_JPRB, 0.16050E+01_JPRB, 0.15294E+01_JPRB, 0.14059E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.13513E+01_JPRB, 0.14593E+01_JPRB, 0.14764E+01_JPRB, 0.16178E+01_JPRB, 0.17575E+01_JPRB, 0.18636E+01_JPRB, 0.21828E+01_JPRB ,&
&  0.24746E+01_JPRB, 0.28489E+01_JPRB, 0.32573E+01_JPRB, 0.35334E+01_JPRB, 0.37580E+01_JPRB, 0.38633E+01_JPRB, 0.38670E+01_JPRB ,&
&  0.36247E+01_JPRB, 0.34623E+01_JPRB, 0.36082E+01_JPRB, 0.39171E+01_JPRB, 0.34061E+01_JPRB, 0.31779E+01_JPRB, 0.26526E+01_JPRB ,&
&  0.19508E+01_JPRB, 0.17547E+01_JPRB, 0.19954E+01_JPRB, 0.23927E+01_JPRB, 0.23910E+01_JPRB, 0.22561E+01_JPRB, 0.20433E+01_JPRB ,&
&  0.18881E+01_JPRB, 0.17300E+01_JPRB, 0.16154E+01_JPRB, 0.14950E+01_JPRB, 0.13714E+01_JPRB, 0.12410E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.98437E+00_JPRB, 0.91732E+00_JPRB, 0.93977E+00_JPRB, 0.10505E+01_JPRB, 0.13654E+01_JPRB, 0.15168E+01_JPRB ,&
&  0.16551E+01_JPRB, 0.11452E+01_JPRB, 0.94259E+00_JPRB, 0.86718E+00_JPRB, 0.62233E+00_JPRB, 0.62614E+00_JPRB, 0.78579E+00_JPRB ,&
&  0.12033E+01_JPRB, 0.17295E+01_JPRB, 0.28608E+01_JPRB, 0.50836E+01_JPRB, 0.28235E+01_JPRB, 0.21882E+01_JPRB, 0.56647E+00_JPRB ,&
&  0.54572E+00_JPRB, 0.54669E+00_JPRB, 0.54488E+00_JPRB, 0.52963E+00_JPRB, 0.52485E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.48286E+00_JPRB, 0.47556E+00_JPRB, 0.48662E+00_JPRB, 0.48498E+00_JPRB, 0.48858E+00_JPRB, 0.49606E+00_JPRB, 0.50022E+00_JPRB ,&
&  0.50491E+00_JPRB, 0.52241E+00_JPRB, 0.61896E+00_JPRB, 0.75443E+00_JPRB, 0.88662E+00_JPRB, 0.99090E+00_JPRB, 0.10468E+01_JPRB ,&
&  0.11112E+01_JPRB, 0.11619E+01_JPRB, 0.11895E+01_JPRB, 0.12109E+01_JPRB, 0.12674E+01_JPRB, 0.13522E+01_JPRB, 0.14251E+01_JPRB ,&
&  0.14987E+01_JPRB, 0.15757E+01_JPRB, 0.16658E+01_JPRB, 0.18027E+01_JPRB, 0.19373E+01_JPRB, 0.19244E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.15735E+01_JPRB, 0.16636E+01_JPRB, 0.16991E+01_JPRB, 0.15986E+01_JPRB, 0.14905E+01_JPRB, 0.14429E+01_JPRB, 0.15732E+01_JPRB ,&
&  0.17404E+01_JPRB, 0.18405E+01_JPRB, 0.18538E+01_JPRB, 0.19327E+01_JPRB, 0.26551E+01_JPRB, 0.28813E+01_JPRB, 0.34826E+01_JPRB ,&
&  0.39108E+01_JPRB, 0.43672E+01_JPRB, 0.46974E+01_JPRB, 0.50542E+01_JPRB, 0.53519E+01_JPRB, 0.55697E+01_JPRB, 0.58747E+01_JPRB ,&
&  0.59898E+01_JPRB, 0.61329E+01_JPRB, 0.61665E+01_JPRB, 0.60200E+01_JPRB, 0.48687E+01_JPRB, 0.51181E+01_JPRB, 0.34183E+01_JPRB ,&
&  0.23642E+01_JPRB, 0.19995E+01_JPRB, 0.21892E+01_JPRB, 0.22199E+01_JPRB, 0.20581E+01_JPRB, 0.18257E+01_JPRB, 0.15453E+01_JPRB ,&
&  0.14442E+01_JPRB, 0.14019E+01_JPRB, 0.13643E+01_JPRB, 0.12829E+01_JPRB, 0.12241E+01_JPRB, 0.11140E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.99304E+00_JPRB, 0.93490E+00_JPRB, 0.86972E+00_JPRB, 0.90377E+00_JPRB, 0.10588E+01_JPRB, 0.14776E+01_JPRB, 0.21742E+01_JPRB ,&
&  0.23304E+01_JPRB, 0.12313E+01_JPRB, 0.10041E+01_JPRB, 0.83994E+00_JPRB, 0.75326E+00_JPRB, 0.75130E+00_JPRB, 0.81904E+00_JPRB ,&
&  0.10399E+01_JPRB, 0.18283E+01_JPRB, 0.26922E+01_JPRB, 0.20013E+01_JPRB, 0.22541E+01_JPRB, 0.15615E+01_JPRB, 0.66462E+00_JPRB ,&
&  0.53220E+00_JPRB, 0.51483E+00_JPRB, 0.51986E+00_JPRB, 0.51156E+00_JPRB, 0.49889E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.53228E+00_JPRB, 0.52124E+00_JPRB, 0.51981E+00_JPRB, 0.53001E+00_JPRB, 0.53647E+00_JPRB, 0.56231E+00_JPRB, 0.59405E+00_JPRB ,&
&  0.61232E+00_JPRB, 0.68820E+00_JPRB, 0.85902E+00_JPRB, 0.10081E+01_JPRB, 0.10799E+01_JPRB, 0.11341E+01_JPRB, 0.11129E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11272E+01_JPRB, 0.11135E+01_JPRB, 0.11478E+01_JPRB, 0.12132E+01_JPRB, 0.13067E+01_JPRB, 0.14288E+01_JPRB ,&
&  0.15659E+01_JPRB, 0.17020E+01_JPRB, 0.18270E+01_JPRB, 0.19446E+01_JPRB, 0.21051E+01_JPRB, 0.21342E+01_JPRB, 0.20977E+01_JPRB ,&
&  0.10139E+01_JPRB, 0.16283E+01_JPRB, 0.16707E+01_JPRB, 0.16015E+01_JPRB, 0.15197E+01_JPRB, 0.15654E+01_JPRB, 0.18521E+01_JPRB ,&
&  0.21292E+01_JPRB, 0.22216E+01_JPRB, 0.25952E+01_JPRB, 0.26064E+01_JPRB, 0.33561E+01_JPRB, 0.38234E+01_JPRB, 0.42877E+01_JPRB ,&
&  0.47329E+01_JPRB, 0.50652E+01_JPRB, 0.54198E+01_JPRB, 0.57652E+01_JPRB, 0.61048E+01_JPRB, 0.64065E+01_JPRB, 0.67450E+01_JPRB ,&
&  0.71460E+01_JPRB, 0.76886E+01_JPRB, 0.82673E+01_JPRB, 0.79725E+01_JPRB, 0.67163E+01_JPRB, 0.62304E+01_JPRB, 0.47948E+01_JPRB ,&
&  0.31390E+01_JPRB, 0.19167E+01_JPRB, 0.18733E+01_JPRB, 0.19309E+01_JPRB, 0.17235E+01_JPRB, 0.15243E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.12473E+01_JPRB, 0.12020E+01_JPRB, 0.11413E+01_JPRB, 0.11487E+01_JPRB, 0.11605E+01_JPRB, 0.11036E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10209E+01_JPRB, 0.97746E+00_JPRB, 0.92643E+00_JPRB, 0.91291E+00_JPRB, 0.97929E+00_JPRB, 0.11263E+01_JPRB, 0.12542E+01_JPRB ,&
&  0.12819E+01_JPRB, 0.11110E+01_JPRB, 0.97917E+00_JPRB, 0.94536E+00_JPRB, 0.89982E+00_JPRB, 0.86851E+00_JPRB, 0.84412E+00_JPRB ,&
&  0.88430E+00_JPRB, 0.10502E+01_JPRB, 0.14523E+01_JPRB, 0.18443E+01_JPRB, 0.14536E+01_JPRB, 0.11518E+01_JPRB, 0.67719E+00_JPRB ,&
&  0.55192E+00_JPRB, 0.53334E+00_JPRB, 0.55075E+00_JPRB, 0.55499E+00_JPRB, 0.55553E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.64446E+00_JPRB, 0.63082E+00_JPRB, 0.61069E+00_JPRB, 0.62596E+00_JPRB, 0.65919E+00_JPRB, 0.70350E+00_JPRB, 0.72398E+00_JPRB ,&
&  0.75182E+00_JPRB, 0.86173E+00_JPRB, 0.10071E+01_JPRB, 0.11376E+01_JPRB, 0.11688E+01_JPRB, 0.11470E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.10513E+01_JPRB, 0.10562E+01_JPRB, 0.10683E+01_JPRB, 0.11139E+01_JPRB, 0.12120E+01_JPRB, 0.13576E+01_JPRB ,&
&  0.15460E+01_JPRB, 0.17543E+01_JPRB, 0.19879E+01_JPRB, 0.22440E+01_JPRB, 0.24877E+01_JPRB, 0.26249E+01_JPRB, 0.25145E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.12042E+01_JPRB, 0.17090E+01_JPRB, 0.17244E+01_JPRB, 0.16826E+01_JPRB, 0.17335E+01_JPRB, 0.19492E+01_JPRB ,&
&  0.21700E+01_JPRB, 0.22621E+01_JPRB, 0.26022E+01_JPRB, 0.32024E+01_JPRB, 0.35361E+01_JPRB, 0.40691E+01_JPRB, 0.43899E+01_JPRB ,&
&  0.47194E+01_JPRB, 0.50492E+01_JPRB, 0.54066E+01_JPRB, 0.57737E+01_JPRB, 0.61381E+01_JPRB, 0.64806E+01_JPRB, 0.69068E+01_JPRB ,&
&  0.74736E+01_JPRB, 0.81066E+01_JPRB, 0.86333E+01_JPRB, 0.77728E+01_JPRB, 0.81892E+01_JPRB, 0.71384E+01_JPRB, 0.61278E+01_JPRB ,&
&  0.35087E+01_JPRB, 0.19613E+01_JPRB, 0.19123E+01_JPRB, 0.19623E+01_JPRB, 0.16713E+01_JPRB, 0.14269E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.10029E+01_JPRB, 0.10459E+01_JPRB, 0.11062E+01_JPRB, 0.10882E+01_JPRB, 0.11227E+01_JPRB, 0.10578E+01_JPRB ,&
&  0.10540E+01_JPRB, 0.10574E+01_JPRB, 0.10324E+01_JPRB, 0.10332E+01_JPRB, 0.10528E+01_JPRB, 0.10856E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10904E+01_JPRB, 0.10651E+01_JPRB, 0.10456E+01_JPRB, 0.10331E+01_JPRB, 0.10084E+01_JPRB, 0.97791E+00_JPRB, 0.92248E+00_JPRB ,&
&  0.86286E+00_JPRB, 0.86526E+00_JPRB, 0.96100E+00_JPRB, 0.10892E+01_JPRB, 0.10196E+01_JPRB, 0.88344E+00_JPRB, 0.70988E+00_JPRB ,&
&  0.69624E+00_JPRB, 0.71695E+00_JPRB, 0.71631E+00_JPRB, 0.69699E+00_JPRB, 0.67445E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.76961E+00_JPRB, 0.74106E+00_JPRB, 0.70117E+00_JPRB, 0.68915E+00_JPRB, 0.73952E+00_JPRB, 0.78919E+00_JPRB, 0.79252E+00_JPRB ,&
&  0.80513E+00_JPRB, 0.87634E+00_JPRB, 0.10038E+01_JPRB, 0.11049E+01_JPRB, 0.11409E+01_JPRB, 0.11373E+01_JPRB, 0.10577E+01_JPRB ,&
&  0.10341E+01_JPRB, 0.10236E+01_JPRB, 0.10067E+01_JPRB, 0.10085E+01_JPRB, 0.10261E+01_JPRB, 0.10680E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.12688E+01_JPRB, 0.14674E+01_JPRB, 0.17556E+01_JPRB, 0.21685E+01_JPRB, 0.27571E+01_JPRB, 0.34523E+01_JPRB, 0.40610E+01_JPRB ,&
&  0.30765E+01_JPRB, 0.12222E+01_JPRB, 0.14656E+01_JPRB, 0.19402E+01_JPRB, 0.18402E+01_JPRB, 0.19229E+01_JPRB, 0.19345E+01_JPRB ,&
&  0.20606E+01_JPRB, 0.22818E+01_JPRB, 0.25245E+01_JPRB, 0.30663E+01_JPRB, 0.34814E+01_JPRB, 0.39328E+01_JPRB, 0.41533E+01_JPRB ,&
&  0.43595E+01_JPRB, 0.46096E+01_JPRB, 0.48688E+01_JPRB, 0.51385E+01_JPRB, 0.54382E+01_JPRB, 0.57683E+01_JPRB, 0.62535E+01_JPRB ,&
&  0.68698E+01_JPRB, 0.74823E+01_JPRB, 0.79008E+01_JPRB, 0.67944E+01_JPRB, 0.71482E+01_JPRB, 0.56260E+01_JPRB, 0.41520E+01_JPRB ,&
&  0.31167E+01_JPRB, 0.23915E+01_JPRB, 0.21229E+01_JPRB, 0.20725E+01_JPRB, 0.17126E+01_JPRB, 0.12741E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.98727E+00_JPRB, 0.10284E+01_JPRB, 0.99730E+00_JPRB, 0.10621E+01_JPRB, 0.10886E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.11151E+01_JPRB, 0.11385E+01_JPRB, 0.11800E+01_JPRB, 0.11706E+01_JPRB, 0.11718E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11528E+01_JPRB, 0.11552E+01_JPRB, 0.11608E+01_JPRB, 0.11541E+01_JPRB, 0.10626E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.96493E+00_JPRB, 0.94416E+00_JPRB, 0.89992E+00_JPRB, 0.93165E+00_JPRB, 0.92298E+00_JPRB, 0.92396E+00_JPRB, 0.95507E+00_JPRB ,&
&  0.96435E+00_JPRB, 0.96487E+00_JPRB, 0.90789E+00_JPRB, 0.84778E+00_JPRB, 0.81120E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.85294E+00_JPRB, 0.79398E+00_JPRB, 0.75886E+00_JPRB, 0.76694E+00_JPRB, 0.81703E+00_JPRB, 0.86801E+00_JPRB, 0.86385E+00_JPRB ,&
&  0.84838E+00_JPRB, 0.87237E+00_JPRB, 0.94889E+00_JPRB, 0.10172E+01_JPRB, 0.10293E+01_JPRB, 0.10371E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10029E+01_JPRB, 0.99317E+00_JPRB, 0.97544E+00_JPRB, 0.97950E+00_JPRB, 0.98217E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.11736E+01_JPRB, 0.13701E+01_JPRB, 0.16658E+01_JPRB, 0.21077E+01_JPRB, 0.29362E+01_JPRB, 0.42220E+01_JPRB ,&
&  0.62129E+01_JPRB, 0.40811E+01_JPRB, 0.13927E+01_JPRB, 0.18741E+01_JPRB, 0.20402E+01_JPRB, 0.21329E+01_JPRB, 0.18757E+01_JPRB ,&
&  0.20244E+01_JPRB, 0.21805E+01_JPRB, 0.22376E+01_JPRB, 0.27627E+01_JPRB, 0.31879E+01_JPRB, 0.35064E+01_JPRB, 0.37168E+01_JPRB ,&
&  0.38000E+01_JPRB, 0.39403E+01_JPRB, 0.40705E+01_JPRB, 0.41875E+01_JPRB, 0.43313E+01_JPRB, 0.45547E+01_JPRB, 0.49206E+01_JPRB ,&
&  0.56011E+01_JPRB, 0.64520E+01_JPRB, 0.71645E+01_JPRB, 0.62527E+01_JPRB, 0.62506E+01_JPRB, 0.47530E+01_JPRB, 0.38605E+01_JPRB ,&
&  0.32640E+01_JPRB, 0.31082E+01_JPRB, 0.28502E+01_JPRB, 0.22867E+01_JPRB, 0.17860E+01_JPRB, 0.14770E+01_JPRB, 0.13957E+01_JPRB ,&
&  0.11562E+01_JPRB, 0.10414E+01_JPRB, 0.10395E+01_JPRB, 0.10874E+01_JPRB, 0.10977E+01_JPRB, 0.11114E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.11675E+01_JPRB, 0.12203E+01_JPRB, 0.12536E+01_JPRB, 0.12779E+01_JPRB, 0.13183E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.12733E+01_JPRB, 0.12718E+01_JPRB, 0.12746E+01_JPRB, 0.12095E+01_JPRB, 0.11176E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.10796E+01_JPRB, 0.10307E+01_JPRB, 0.10303E+01_JPRB, 0.10300E+01_JPRB, 0.10830E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.11755E+01_JPRB, 0.11384E+01_JPRB, 0.10577E+01_JPRB, 0.92182E+00_JPRB, 0.90526E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.92751E+00_JPRB, 0.90368E+00_JPRB, 0.86877E+00_JPRB, 0.84435E+00_JPRB, 0.86828E+00_JPRB, 0.90521E+00_JPRB, 0.90450E+00_JPRB ,&
&  0.87730E+00_JPRB, 0.85965E+00_JPRB, 0.89857E+00_JPRB, 0.94136E+00_JPRB, 0.94052E+00_JPRB, 0.96347E+00_JPRB, 0.97685E+00_JPRB ,&
&  0.10163E+01_JPRB, 0.10147E+01_JPRB, 0.99691E+00_JPRB, 0.97297E+00_JPRB, 0.97139E+00_JPRB, 0.95840E+00_JPRB, 0.96146E+00_JPRB ,&
&  0.98445E+00_JPRB, 0.10342E+01_JPRB, 0.11391E+01_JPRB, 0.13030E+01_JPRB, 0.15077E+01_JPRB, 0.18944E+01_JPRB, 0.26776E+01_JPRB ,&
&  0.35723E+01_JPRB, 0.45834E+01_JPRB, 0.15204E+01_JPRB, 0.20518E+01_JPRB, 0.25435E+01_JPRB, 0.24726E+01_JPRB, 0.22356E+01_JPRB ,&
&  0.22074E+01_JPRB, 0.21529E+01_JPRB, 0.21159E+01_JPRB, 0.22658E+01_JPRB, 0.26023E+01_JPRB, 0.28688E+01_JPRB, 0.28643E+01_JPRB ,&
&  0.29464E+01_JPRB, 0.31158E+01_JPRB, 0.33902E+01_JPRB, 0.34628E+01_JPRB, 0.34651E+01_JPRB, 0.34424E+01_JPRB, 0.35961E+01_JPRB ,&
&  0.40699E+01_JPRB, 0.49054E+01_JPRB, 0.61398E+01_JPRB, 0.57534E+01_JPRB, 0.57430E+01_JPRB, 0.49649E+01_JPRB, 0.43289E+01_JPRB ,&
&  0.38155E+01_JPRB, 0.37115E+01_JPRB, 0.34060E+01_JPRB, 0.26774E+01_JPRB, 0.20219E+01_JPRB, 0.17344E+01_JPRB, 0.16289E+01_JPRB ,&
&  0.13859E+01_JPRB, 0.12227E+01_JPRB, 0.12089E+01_JPRB, 0.12063E+01_JPRB, 0.12369E+01_JPRB, 0.12177E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.11993E+01_JPRB, 0.12313E+01_JPRB, 0.12800E+01_JPRB, 0.13281E+01_JPRB, 0.13891E+01_JPRB, 0.14139E+01_JPRB ,&
&  0.14019E+01_JPRB, 0.13932E+01_JPRB, 0.13703E+01_JPRB, 0.13046E+01_JPRB, 0.12190E+01_JPRB, 0.11768E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.11838E+01_JPRB, 0.11403E+01_JPRB, 0.10855E+01_JPRB, 0.11181E+01_JPRB, 0.11226E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.13600E+01_JPRB, 0.12955E+01_JPRB, 0.10133E+01_JPRB, 0.99168E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.10531E+01_JPRB, 0.10044E+01_JPRB, 0.97094E+00_JPRB, 0.93799E+00_JPRB, 0.91149E+00_JPRB, 0.94488E+00_JPRB, 0.93960E+00_JPRB ,&
&  0.91707E+00_JPRB, 0.88038E+00_JPRB, 0.89615E+00_JPRB, 0.94869E+00_JPRB, 0.95489E+00_JPRB, 0.93923E+00_JPRB, 0.92856E+00_JPRB ,&
&  0.92916E+00_JPRB, 0.94553E+00_JPRB, 0.94909E+00_JPRB, 0.96491E+00_JPRB, 0.96947E+00_JPRB, 0.96157E+00_JPRB, 0.95841E+00_JPRB ,&
&  0.97757E+00_JPRB, 0.10018E+01_JPRB, 0.10450E+01_JPRB, 0.10995E+01_JPRB, 0.11767E+01_JPRB, 0.12917E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.22079E+01_JPRB, 0.36578E+01_JPRB, 0.29394E+01_JPRB, 0.35165E+01_JPRB, 0.33398E+01_JPRB, 0.30699E+01_JPRB, 0.27332E+01_JPRB ,&
&  0.28914E+01_JPRB, 0.26446E+01_JPRB, 0.25285E+01_JPRB, 0.20625E+01_JPRB, 0.19879E+01_JPRB, 0.20951E+01_JPRB, 0.20527E+01_JPRB ,&
&  0.19743E+01_JPRB, 0.19608E+01_JPRB, 0.23151E+01_JPRB, 0.24973E+01_JPRB, 0.25781E+01_JPRB, 0.26056E+01_JPRB, 0.26514E+01_JPRB ,&
&  0.29016E+01_JPRB, 0.33599E+01_JPRB, 0.45364E+01_JPRB, 0.54448E+01_JPRB, 0.53026E+01_JPRB, 0.51602E+01_JPRB, 0.50219E+01_JPRB ,&
&  0.50415E+01_JPRB, 0.44611E+01_JPRB, 0.38903E+01_JPRB, 0.30759E+01_JPRB, 0.23356E+01_JPRB, 0.21933E+01_JPRB, 0.18346E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.14025E+01_JPRB, 0.13690E+01_JPRB, 0.13505E+01_JPRB, 0.13575E+01_JPRB, 0.13130E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.12541E+01_JPRB, 0.12395E+01_JPRB, 0.12766E+01_JPRB, 0.13103E+01_JPRB, 0.13532E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.14223E+01_JPRB, 0.14406E+01_JPRB, 0.13819E+01_JPRB, 0.13234E+01_JPRB, 0.12562E+01_JPRB, 0.12166E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.12310E+01_JPRB, 0.12225E+01_JPRB, 0.11717E+01_JPRB, 0.11403E+01_JPRB, 0.11233E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.11879E+01_JPRB, 0.11613E+01_JPRB, 0.11045E+01_JPRB, 0.10857E+01_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.11382E+01_JPRB, 0.11100E+01_JPRB, 0.10513E+01_JPRB, 0.10302E+01_JPRB, 0.98426E+00_JPRB, 0.10034E+01_JPRB, 0.10086E+01_JPRB ,&
&  0.97716E+00_JPRB, 0.94426E+00_JPRB, 0.91738E+00_JPRB, 0.93856E+00_JPRB, 0.96326E+00_JPRB, 0.95435E+00_JPRB, 0.91897E+00_JPRB ,&
&  0.90667E+00_JPRB, 0.90222E+00_JPRB, 0.89946E+00_JPRB, 0.90462E+00_JPRB, 0.92360E+00_JPRB, 0.95149E+00_JPRB, 0.95294E+00_JPRB ,&
&  0.95098E+00_JPRB, 0.99080E+00_JPRB, 0.10374E+01_JPRB, 0.10528E+01_JPRB, 0.10716E+01_JPRB, 0.11120E+01_JPRB, 0.12963E+01_JPRB ,&
&  0.15883E+01_JPRB, 0.36569E+01_JPRB, 0.23730E+01_JPRB, 0.45233E+01_JPRB, 0.36132E+01_JPRB, 0.32353E+01_JPRB, 0.26466E+01_JPRB ,&
&  0.24489E+01_JPRB, 0.25732E+01_JPRB, 0.21353E+01_JPRB, 0.18345E+01_JPRB, 0.16424E+01_JPRB, 0.15829E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.13350E+01_JPRB, 0.13297E+01_JPRB, 0.14644E+01_JPRB, 0.16451E+01_JPRB, 0.17712E+01_JPRB, 0.18461E+01_JPRB, 0.19541E+01_JPRB ,&
&  0.21290E+01_JPRB, 0.23539E+01_JPRB, 0.31309E+01_JPRB, 0.41665E+01_JPRB, 0.51149E+01_JPRB, 0.54104E+01_JPRB, 0.58437E+01_JPRB ,&
&  0.59891E+01_JPRB, 0.54182E+01_JPRB, 0.44266E+01_JPRB, 0.34558E+01_JPRB, 0.27806E+01_JPRB, 0.25365E+01_JPRB, 0.22237E+01_JPRB ,&
&  0.19356E+01_JPRB, 0.16732E+01_JPRB, 0.16002E+01_JPRB, 0.15287E+01_JPRB, 0.15136E+01_JPRB, 0.14560E+01_JPRB, 0.13536E+01_JPRB ,&
&  0.12795E+01_JPRB, 0.12708E+01_JPRB, 0.12935E+01_JPRB, 0.13156E+01_JPRB, 0.13479E+01_JPRB, 0.13546E+01_JPRB, 0.13587E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.14272E+01_JPRB, 0.13913E+01_JPRB, 0.13760E+01_JPRB, 0.13210E+01_JPRB, 0.12782E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.12924E+01_JPRB, 0.12444E+01_JPRB, 0.11843E+01_JPRB, 0.11495E+01_JPRB, 0.11531E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.11345E+01_JPRB, 0.11748E+01_JPRB, 0.11400E+01_JPRB, 0.11378E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.11751E+01_JPRB, 0.11678E+01_JPRB, 0.10936E+01_JPRB, 0.10654E+01_JPRB, 0.10242E+01_JPRB, 0.10059E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10029E+01_JPRB, 0.97831E+00_JPRB, 0.94723E+00_JPRB, 0.93291E+00_JPRB, 0.94693E+00_JPRB, 0.96216E+00_JPRB, 0.93080E+00_JPRB ,&
&  0.89455E+00_JPRB, 0.87964E+00_JPRB, 0.86929E+00_JPRB, 0.87117E+00_JPRB, 0.88486E+00_JPRB, 0.90238E+00_JPRB, 0.91480E+00_JPRB ,&
&  0.91268E+00_JPRB, 0.95804E+00_JPRB, 0.10183E+01_JPRB, 0.10522E+01_JPRB, 0.10560E+01_JPRB, 0.10934E+01_JPRB, 0.12322E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.50209E+01_JPRB, 0.26842E+01_JPRB, 0.47210E+01_JPRB, 0.37290E+01_JPRB, 0.32108E+01_JPRB, 0.25579E+01_JPRB ,&
&  0.23329E+01_JPRB, 0.22081E+01_JPRB, 0.19377E+01_JPRB, 0.17240E+01_JPRB, 0.15709E+01_JPRB, 0.14534E+01_JPRB, 0.13541E+01_JPRB ,&
&  0.12587E+01_JPRB, 0.12063E+01_JPRB, 0.12217E+01_JPRB, 0.13201E+01_JPRB, 0.13898E+01_JPRB, 0.14357E+01_JPRB, 0.15019E+01_JPRB ,&
&  0.16157E+01_JPRB, 0.17713E+01_JPRB, 0.22341E+01_JPRB, 0.35250E+01_JPRB, 0.45670E+01_JPRB, 0.54807E+01_JPRB, 0.79154E+01_JPRB ,&
&  0.75299E+01_JPRB, 0.68608E+01_JPRB, 0.49915E+01_JPRB, 0.38857E+01_JPRB, 0.31645E+01_JPRB, 0.27579E+01_JPRB, 0.25578E+01_JPRB ,&
&  0.23508E+01_JPRB, 0.20566E+01_JPRB, 0.19189E+01_JPRB, 0.18213E+01_JPRB, 0.17125E+01_JPRB, 0.15998E+01_JPRB, 0.14929E+01_JPRB ,&
&  0.14164E+01_JPRB, 0.13675E+01_JPRB, 0.13803E+01_JPRB, 0.13920E+01_JPRB, 0.14007E+01_JPRB, 0.13847E+01_JPRB, 0.13721E+01_JPRB ,&
&  0.13810E+01_JPRB, 0.14070E+01_JPRB, 0.13835E+01_JPRB, 0.13883E+01_JPRB, 0.13681E+01_JPRB, 0.13324E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.13620E+01_JPRB, 0.13610E+01_JPRB, 0.12864E+01_JPRB, 0.12359E+01_JPRB, 0.11773E+01_JPRB, 0.11960E+01_JPRB, 0.11337E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.10905E+01_JPRB, 0.11490E+01_JPRB, 0.11557E+01_JPRB, 0.11756E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.11908E+01_JPRB, 0.11742E+01_JPRB, 0.11345E+01_JPRB, 0.10734E+01_JPRB, 0.10465E+01_JPRB, 0.10185E+01_JPRB, 0.10022E+01_JPRB ,&
&  0.99577E+00_JPRB, 0.98109E+00_JPRB, 0.96659E+00_JPRB, 0.94472E+00_JPRB, 0.93560E+00_JPRB, 0.94806E+00_JPRB, 0.94269E+00_JPRB ,&
&  0.92455E+00_JPRB, 0.90041E+00_JPRB, 0.89072E+00_JPRB, 0.88046E+00_JPRB, 0.86916E+00_JPRB, 0.86883E+00_JPRB, 0.88075E+00_JPRB ,&
&  0.88957E+00_JPRB, 0.91097E+00_JPRB, 0.94868E+00_JPRB, 0.98477E+00_JPRB, 0.99418E+00_JPRB, 0.10211E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.18222E+01_JPRB, 0.39247E+01_JPRB, 0.45997E+01_JPRB, 0.38067E+01_JPRB, 0.33027E+01_JPRB, 0.27782E+01_JPRB ,&
&  0.24520E+01_JPRB, 0.20658E+01_JPRB, 0.17549E+01_JPRB, 0.15577E+01_JPRB, 0.14534E+01_JPRB, 0.13330E+01_JPRB, 0.12718E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.12003E+01_JPRB, 0.11917E+01_JPRB, 0.12122E+01_JPRB, 0.12208E+01_JPRB, 0.12316E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.14514E+01_JPRB, 0.16889E+01_JPRB, 0.25216E+01_JPRB, 0.41490E+01_JPRB, 0.57175E+01_JPRB, 0.78620E+01_JPRB ,&
&  0.88765E+01_JPRB, 0.80072E+01_JPRB, 0.55612E+01_JPRB, 0.42175E+01_JPRB, 0.34954E+01_JPRB, 0.30181E+01_JPRB, 0.27811E+01_JPRB ,&
&  0.26176E+01_JPRB, 0.23914E+01_JPRB, 0.22090E+01_JPRB, 0.21572E+01_JPRB, 0.19864E+01_JPRB, 0.17956E+01_JPRB, 0.16396E+01_JPRB ,&
&  0.15618E+01_JPRB, 0.15084E+01_JPRB, 0.14513E+01_JPRB, 0.14095E+01_JPRB, 0.14078E+01_JPRB, 0.13932E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.13678E+01_JPRB, 0.13766E+01_JPRB, 0.13601E+01_JPRB, 0.13542E+01_JPRB, 0.13759E+01_JPRB, 0.13787E+01_JPRB, 0.13795E+01_JPRB ,&
&  0.13728E+01_JPRB, 0.13802E+01_JPRB, 0.13325E+01_JPRB, 0.12926E+01_JPRB, 0.12050E+01_JPRB, 0.12380E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.10885E+01_JPRB, 0.10924E+01_JPRB, 0.11412E+01_JPRB, 0.11842E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.11653E+01_JPRB, 0.11543E+01_JPRB, 0.11299E+01_JPRB, 0.10931E+01_JPRB, 0.10620E+01_JPRB, 0.10442E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.10216E+01_JPRB, 0.10108E+01_JPRB, 0.98300E+00_JPRB, 0.95897E+00_JPRB, 0.95234E+00_JPRB, 0.95522E+00_JPRB ,&
&  0.94764E+00_JPRB, 0.93590E+00_JPRB, 0.93800E+00_JPRB, 0.91997E+00_JPRB, 0.89796E+00_JPRB, 0.88376E+00_JPRB, 0.88013E+00_JPRB ,&
&  0.88649E+00_JPRB, 0.89802E+00_JPRB, 0.92660E+00_JPRB, 0.95556E+00_JPRB, 0.96160E+00_JPRB, 0.96185E+00_JPRB, 0.99888E+00_JPRB ,&
&  0.15029E+01_JPRB, 0.29318E+01_JPRB, 0.44985E+01_JPRB, 0.42117E+01_JPRB, 0.36210E+01_JPRB, 0.30451E+01_JPRB, 0.27404E+01_JPRB ,&
&  0.22915E+01_JPRB, 0.20136E+01_JPRB, 0.17372E+01_JPRB, 0.15195E+01_JPRB, 0.13712E+01_JPRB, 0.12818E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.11601E+01_JPRB, 0.11528E+01_JPRB, 0.11485E+01_JPRB, 0.11429E+01_JPRB, 0.11303E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.12872E+01_JPRB, 0.14173E+01_JPRB, 0.18477E+01_JPRB, 0.34103E+01_JPRB, 0.43163E+01_JPRB, 0.67227E+01_JPRB ,&
&  0.91834E+01_JPRB, 0.75729E+01_JPRB, 0.57129E+01_JPRB, 0.44800E+01_JPRB, 0.37330E+01_JPRB, 0.32650E+01_JPRB, 0.29842E+01_JPRB ,&
&  0.27011E+01_JPRB, 0.25226E+01_JPRB, 0.23523E+01_JPRB, 0.21824E+01_JPRB, 0.20603E+01_JPRB, 0.18981E+01_JPRB, 0.17492E+01_JPRB ,&
&  0.16872E+01_JPRB, 0.16127E+01_JPRB, 0.15319E+01_JPRB, 0.14425E+01_JPRB, 0.14057E+01_JPRB, 0.13827E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.13503E+01_JPRB, 0.13342E+01_JPRB, 0.13299E+01_JPRB, 0.13650E+01_JPRB, 0.13975E+01_JPRB, 0.13916E+01_JPRB ,&
&  0.13446E+01_JPRB, 0.13359E+01_JPRB, 0.13564E+01_JPRB, 0.13058E+01_JPRB, 0.12525E+01_JPRB, 0.13135E+01_JPRB, 0.12277E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.11090E+01_JPRB, 0.10805E+01_JPRB, 0.10880E+01_JPRB, 0.11548E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.11530E+01_JPRB, 0.11472E+01_JPRB, 0.11265E+01_JPRB, 0.11058E+01_JPRB, 0.10942E+01_JPRB, 0.10748E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10522E+01_JPRB, 0.10388E+01_JPRB, 0.10289E+01_JPRB, 0.10136E+01_JPRB, 0.99251E+00_JPRB, 0.97297E+00_JPRB, 0.96413E+00_JPRB ,&
&  0.96160E+00_JPRB, 0.95097E+00_JPRB, 0.95392E+00_JPRB, 0.95444E+00_JPRB, 0.95078E+00_JPRB, 0.93931E+00_JPRB, 0.92802E+00_JPRB ,&
&  0.92022E+00_JPRB, 0.90514E+00_JPRB, 0.90216E+00_JPRB, 0.92590E+00_JPRB, 0.94725E+00_JPRB, 0.95410E+00_JPRB, 0.96199E+00_JPRB ,&
&  0.12040E+01_JPRB, 0.22978E+01_JPRB, 0.32914E+01_JPRB, 0.35186E+01_JPRB, 0.32908E+01_JPRB, 0.28774E+01_JPRB, 0.24753E+01_JPRB ,&
&  0.21230E+01_JPRB, 0.18484E+01_JPRB, 0.16719E+01_JPRB, 0.15331E+01_JPRB, 0.14087E+01_JPRB, 0.13066E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.11396E+01_JPRB, 0.11181E+01_JPRB, 0.11062E+01_JPRB, 0.10971E+01_JPRB, 0.10895E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.11340E+01_JPRB, 0.11879E+01_JPRB, 0.12616E+01_JPRB, 0.13985E+01_JPRB, 0.20831E+01_JPRB, 0.32768E+01_JPRB, 0.48499E+01_JPRB ,&
&  0.53286E+01_JPRB, 0.54164E+01_JPRB, 0.48910E+01_JPRB, 0.41345E+01_JPRB, 0.35169E+01_JPRB, 0.30767E+01_JPRB, 0.27945E+01_JPRB ,&
&  0.25271E+01_JPRB, 0.23341E+01_JPRB, 0.21925E+01_JPRB, 0.20129E+01_JPRB, 0.18382E+01_JPRB, 0.17292E+01_JPRB, 0.17212E+01_JPRB ,&
&  0.17381E+01_JPRB, 0.16873E+01_JPRB, 0.15925E+01_JPRB, 0.14772E+01_JPRB, 0.13679E+01_JPRB, 0.13178E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.12838E+01_JPRB, 0.13037E+01_JPRB, 0.13278E+01_JPRB, 0.13427E+01_JPRB, 0.13474E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.13456E+01_JPRB, 0.13313E+01_JPRB, 0.13217E+01_JPRB, 0.12811E+01_JPRB, 0.13343E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.11324E+01_JPRB, 0.11026E+01_JPRB, 0.10861E+01_JPRB, 0.11322E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11540E+01_JPRB, 0.11460E+01_JPRB, 0.11277E+01_JPRB, 0.11121E+01_JPRB, 0.10907E+01_JPRB, 0.10705E+01_JPRB, 0.10529E+01_JPRB ,&
&  0.10437E+01_JPRB, 0.10258E+01_JPRB, 0.10160E+01_JPRB, 0.10064E+01_JPRB, 0.10008E+01_JPRB, 0.98863E+00_JPRB, 0.97078E+00_JPRB ,&
&  0.95259E+00_JPRB, 0.94198E+00_JPRB, 0.93790E+00_JPRB, 0.94095E+00_JPRB, 0.95039E+00_JPRB, 0.95693E+00_JPRB, 0.95218E+00_JPRB ,&
&  0.94219E+00_JPRB, 0.92580E+00_JPRB, 0.90690E+00_JPRB, 0.92367E+00_JPRB, 0.94293E+00_JPRB, 0.95037E+00_JPRB, 0.94591E+00_JPRB ,&
&  0.10884E+01_JPRB, 0.15709E+01_JPRB, 0.22117E+01_JPRB, 0.23932E+01_JPRB, 0.23921E+01_JPRB, 0.23966E+01_JPRB, 0.21899E+01_JPRB ,&
&  0.19718E+01_JPRB, 0.17389E+01_JPRB, 0.15373E+01_JPRB, 0.14258E+01_JPRB, 0.13538E+01_JPRB, 0.12777E+01_JPRB, 0.12161E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.11076E+01_JPRB, 0.10819E+01_JPRB, 0.10715E+01_JPRB, 0.10667E+01_JPRB, 0.10669E+01_JPRB, 0.10736E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.11039E+01_JPRB, 0.11439E+01_JPRB, 0.12063E+01_JPRB, 0.13557E+01_JPRB, 0.18404E+01_JPRB, 0.26094E+01_JPRB ,&
&  0.31382E+01_JPRB, 0.35422E+01_JPRB, 0.38092E+01_JPRB, 0.37729E+01_JPRB, 0.34180E+01_JPRB, 0.30004E+01_JPRB, 0.26874E+01_JPRB ,&
&  0.24277E+01_JPRB, 0.21760E+01_JPRB, 0.19647E+01_JPRB, 0.17856E+01_JPRB, 0.16291E+01_JPRB, 0.15616E+01_JPRB, 0.16156E+01_JPRB ,&
&  0.16481E+01_JPRB, 0.16198E+01_JPRB, 0.15371E+01_JPRB, 0.14394E+01_JPRB, 0.13294E+01_JPRB, 0.12648E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.12451E+01_JPRB, 0.12587E+01_JPRB, 0.12792E+01_JPRB, 0.12986E+01_JPRB, 0.13101E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.13216E+01_JPRB, 0.12600E+01_JPRB, 0.12402E+01_JPRB, 0.13235E+01_JPRB, 0.13009E+01_JPRB, 0.12717E+01_JPRB ,&
&  0.12186E+01_JPRB, 0.11478E+01_JPRB, 0.10992E+01_JPRB, 0.10647E+01_JPRB, 0.11116E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.10089E+01_JPRB, 0.10337E+01_JPRB, 0.10424E+01_JPRB, 0.10431E+01_JPRB, 0.10314E+01_JPRB, 0.10122E+01_JPRB, 0.99610E+00_JPRB ,&
&  0.98918E+00_JPRB, 0.97425E+00_JPRB, 0.96730E+00_JPRB, 0.96367E+00_JPRB, 0.96568E+00_JPRB, 0.97124E+00_JPRB, 0.96884E+00_JPRB ,&
&  0.95683E+00_JPRB, 0.94019E+00_JPRB, 0.92885E+00_JPRB, 0.92723E+00_JPRB, 0.93637E+00_JPRB, 0.94693E+00_JPRB, 0.95362E+00_JPRB ,&
&  0.95569E+00_JPRB, 0.94767E+00_JPRB, 0.94247E+00_JPRB, 0.95324E+00_JPRB, 0.96401E+00_JPRB, 0.95799E+00_JPRB, 0.94202E+00_JPRB ,&
&  0.99950E+00_JPRB, 0.11107E+01_JPRB, 0.13532E+01_JPRB, 0.15693E+01_JPRB, 0.17574E+01_JPRB, 0.18258E+01_JPRB, 0.17780E+01_JPRB ,&
&  0.16631E+01_JPRB, 0.15549E+01_JPRB, 0.14290E+01_JPRB, 0.13171E+01_JPRB, 0.12351E+01_JPRB, 0.11878E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.11164E+01_JPRB, 0.10749E+01_JPRB, 0.10400E+01_JPRB, 0.10199E+01_JPRB, 0.10121E+01_JPRB, 0.10051E+01_JPRB, 0.99808E+00_JPRB ,&
&  0.98944E+00_JPRB, 0.98892E+00_JPRB, 0.10104E+01_JPRB, 0.10539E+01_JPRB, 0.11220E+01_JPRB, 0.13003E+01_JPRB, 0.16400E+01_JPRB ,&
&  0.19657E+01_JPRB, 0.23090E+01_JPRB, 0.26476E+01_JPRB, 0.29113E+01_JPRB, 0.29131E+01_JPRB, 0.27518E+01_JPRB, 0.25279E+01_JPRB ,&
&  0.22655E+01_JPRB, 0.19525E+01_JPRB, 0.16690E+01_JPRB, 0.15043E+01_JPRB, 0.14482E+01_JPRB, 0.14602E+01_JPRB, 0.15168E+01_JPRB ,&
&  0.15385E+01_JPRB, 0.15071E+01_JPRB, 0.14177E+01_JPRB, 0.13274E+01_JPRB, 0.12586E+01_JPRB, 0.12039E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11750E+01_JPRB, 0.11771E+01_JPRB, 0.11903E+01_JPRB, 0.12057E+01_JPRB, 0.12280E+01_JPRB, 0.12427E+01_JPRB, 0.12426E+01_JPRB ,&
&  0.12370E+01_JPRB, 0.12278E+01_JPRB, 0.12185E+01_JPRB, 0.12316E+01_JPRB, 0.12138E+01_JPRB, 0.12014E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.11565E+01_JPRB, 0.11083E+01_JPRB, 0.10578E+01_JPRB, 0.10252E+01_JPRB, 0.95448E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.93953E+00_JPRB, 0.96222E+00_JPRB, 0.97186E+00_JPRB, 0.97311E+00_JPRB, 0.97087E+00_JPRB, 0.95810E+00_JPRB, 0.94219E+00_JPRB ,&
&  0.92418E+00_JPRB, 0.91732E+00_JPRB, 0.91532E+00_JPRB, 0.91989E+00_JPRB, 0.92517E+00_JPRB, 0.93299E+00_JPRB, 0.94411E+00_JPRB ,&
&  0.95458E+00_JPRB, 0.95851E+00_JPRB, 0.96125E+00_JPRB, 0.95819E+00_JPRB, 0.95549E+00_JPRB, 0.95698E+00_JPRB, 0.96032E+00_JPRB ,&
&  0.96823E+00_JPRB, 0.96979E+00_JPRB, 0.96444E+00_JPRB, 0.97302E+00_JPRB, 0.98897E+00_JPRB, 0.98022E+00_JPRB, 0.96352E+00_JPRB ,&
&  0.97805E+00_JPRB, 0.91004E+00_JPRB, 0.96674E+00_JPRB, 0.11571E+01_JPRB, 0.13387E+01_JPRB, 0.14851E+01_JPRB, 0.15113E+01_JPRB ,&
&  0.14615E+01_JPRB, 0.13779E+01_JPRB, 0.12977E+01_JPRB, 0.12190E+01_JPRB, 0.11623E+01_JPRB, 0.11101E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.10256E+01_JPRB, 0.99050E+00_JPRB, 0.95971E+00_JPRB, 0.93381E+00_JPRB, 0.90908E+00_JPRB, 0.89397E+00_JPRB ,&
&  0.88975E+00_JPRB, 0.89912E+00_JPRB, 0.91723E+00_JPRB, 0.94928E+00_JPRB, 0.10060E+01_JPRB, 0.10920E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.14211E+01_JPRB, 0.16208E+01_JPRB, 0.18133E+01_JPRB, 0.19841E+01_JPRB, 0.21284E+01_JPRB, 0.21509E+01_JPRB, 0.20397E+01_JPRB ,&
&  0.18377E+01_JPRB, 0.16609E+01_JPRB, 0.15212E+01_JPRB, 0.14263E+01_JPRB, 0.14038E+01_JPRB, 0.14228E+01_JPRB, 0.14361E+01_JPRB ,&
&  0.14123E+01_JPRB, 0.13494E+01_JPRB, 0.12668E+01_JPRB, 0.11904E+01_JPRB, 0.11423E+01_JPRB, 0.11092E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.10482E+01_JPRB, 0.10514E+01_JPRB, 0.10697E+01_JPRB, 0.10969E+01_JPRB, 0.11256E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.11451E+01_JPRB, 0.11549E+01_JPRB, 0.10946E+01_JPRB, 0.11289E+01_JPRB, 0.11322E+01_JPRB, 0.11269E+01_JPRB ,&
&  0.11043E+01_JPRB, 0.10722E+01_JPRB, 0.10262E+01_JPRB, 0.86464E+00_JPRB, 0.91570E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.90790E+00_JPRB, 0.91313E+00_JPRB, 0.91745E+00_JPRB, 0.91354E+00_JPRB, 0.90966E+00_JPRB, 0.90235E+00_JPRB, 0.88688E+00_JPRB ,&
&  0.87219E+00_JPRB, 0.86706E+00_JPRB, 0.86802E+00_JPRB, 0.87007E+00_JPRB, 0.87351E+00_JPRB, 0.87967E+00_JPRB, 0.89069E+00_JPRB ,&
&  0.90398E+00_JPRB, 0.91752E+00_JPRB, 0.93329E+00_JPRB, 0.94728E+00_JPRB, 0.94901E+00_JPRB, 0.94404E+00_JPRB, 0.94231E+00_JPRB ,&
&  0.94684E+00_JPRB, 0.94463E+00_JPRB, 0.93606E+00_JPRB, 0.93701E+00_JPRB, 0.94772E+00_JPRB, 0.94118E+00_JPRB, 0.92452E+00_JPRB ,&
&  0.92416E+00_JPRB, 0.80610E+00_JPRB, 0.86158E+00_JPRB, 0.95722E+00_JPRB, 0.10892E+01_JPRB, 0.12015E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.12380E+01_JPRB, 0.11795E+01_JPRB, 0.11263E+01_JPRB, 0.10657E+01_JPRB, 0.10311E+01_JPRB, 0.10087E+01_JPRB, 0.98601E+00_JPRB ,&
&  0.95327E+00_JPRB, 0.91865E+00_JPRB, 0.88000E+00_JPRB, 0.84432E+00_JPRB, 0.81616E+00_JPRB, 0.80055E+00_JPRB, 0.79349E+00_JPRB ,&
&  0.79125E+00_JPRB, 0.80135E+00_JPRB, 0.80904E+00_JPRB, 0.82529E+00_JPRB, 0.85561E+00_JPRB, 0.90629E+00_JPRB, 0.98105E+00_JPRB ,&
&  0.10882E+01_JPRB, 0.12180E+01_JPRB, 0.13451E+01_JPRB, 0.14355E+01_JPRB, 0.14934E+01_JPRB, 0.15365E+01_JPRB, 0.15346E+01_JPRB ,&
&  0.14728E+01_JPRB, 0.14069E+01_JPRB, 0.13893E+01_JPRB, 0.13781E+01_JPRB, 0.13520E+01_JPRB, 0.13219E+01_JPRB, 0.12756E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.11427E+01_JPRB, 0.10784E+01_JPRB, 0.10210E+01_JPRB, 0.98353E+00_JPRB, 0.96128E+00_JPRB, 0.93236E+00_JPRB ,&
&  0.91365E+00_JPRB, 0.90928E+00_JPRB, 0.91802E+00_JPRB, 0.94098E+00_JPRB, 0.97267E+00_JPRB, 0.10026E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.10226E+01_JPRB, 0.10260E+01_JPRB, 0.10279E+01_JPRB, 0.10483E+01_JPRB, 0.10505E+01_JPRB, 0.10494E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10282E+01_JPRB, 0.81441E+00_JPRB, 0.87980E+00_JPRB, 0.89153E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.91834E+00_JPRB, 0.89830E+00_JPRB, 0.88742E+00_JPRB, 0.88097E+00_JPRB, 0.87627E+00_JPRB, 0.86950E+00_JPRB, 0.85602E+00_JPRB ,&
&  0.84189E+00_JPRB, 0.83451E+00_JPRB, 0.82820E+00_JPRB, 0.82036E+00_JPRB, 0.82278E+00_JPRB, 0.82974E+00_JPRB, 0.84158E+00_JPRB ,&
&  0.85412E+00_JPRB, 0.86965E+00_JPRB, 0.88412E+00_JPRB, 0.90184E+00_JPRB, 0.91243E+00_JPRB, 0.91895E+00_JPRB, 0.92148E+00_JPRB ,&
&  0.92279E+00_JPRB, 0.92209E+00_JPRB, 0.91294E+00_JPRB, 0.90111E+00_JPRB, 0.89873E+00_JPRB, 0.90326E+00_JPRB, 0.89920E+00_JPRB ,&
&  0.80193E+00_JPRB, 0.72619E+00_JPRB, 0.79412E+00_JPRB, 0.87366E+00_JPRB, 0.95892E+00_JPRB, 0.10333E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10116E+01_JPRB, 0.97354E+00_JPRB, 0.92969E+00_JPRB, 0.90831E+00_JPRB, 0.89801E+00_JPRB, 0.88589E+00_JPRB ,&
&  0.87459E+00_JPRB, 0.86051E+00_JPRB, 0.84049E+00_JPRB, 0.80920E+00_JPRB, 0.78559E+00_JPRB, 0.76836E+00_JPRB, 0.76283E+00_JPRB ,&
&  0.76297E+00_JPRB, 0.76580E+00_JPRB, 0.76934E+00_JPRB, 0.77346E+00_JPRB, 0.78396E+00_JPRB, 0.80406E+00_JPRB, 0.83303E+00_JPRB ,&
&  0.87998E+00_JPRB, 0.94027E+00_JPRB, 0.10143E+01_JPRB, 0.10673E+01_JPRB, 0.10984E+01_JPRB, 0.11264E+01_JPRB, 0.11636E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.11841E+01_JPRB, 0.11502E+01_JPRB, 0.11322E+01_JPRB, 0.11243E+01_JPRB, 0.11102E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.97616E+00_JPRB, 0.91673E+00_JPRB, 0.87857E+00_JPRB, 0.86604E+00_JPRB, 0.86093E+00_JPRB, 0.85194E+00_JPRB ,&
&  0.84355E+00_JPRB, 0.84143E+00_JPRB, 0.84731E+00_JPRB, 0.86005E+00_JPRB, 0.87733E+00_JPRB, 0.89779E+00_JPRB, 0.91357E+00_JPRB ,&
&  0.92723E+00_JPRB, 0.93936E+00_JPRB, 0.94804E+00_JPRB, 0.96265E+00_JPRB, 0.97800E+00_JPRB, 0.98410E+00_JPRB, 0.97820E+00_JPRB ,&
&  0.97519E+00_JPRB, 0.96301E+00_JPRB, 0.95135E+00_JPRB, 0.94495E+00_JPRB, 0.93512E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.87862E+00_JPRB, 0.85913E+00_JPRB, 0.84768E+00_JPRB, 0.84282E+00_JPRB, 0.83361E+00_JPRB, 0.81417E+00_JPRB, 0.79805E+00_JPRB ,&
&  0.78280E+00_JPRB, 0.77388E+00_JPRB, 0.76971E+00_JPRB, 0.76880E+00_JPRB, 0.77512E+00_JPRB, 0.78152E+00_JPRB, 0.78950E+00_JPRB ,&
&  0.80189E+00_JPRB, 0.82120E+00_JPRB, 0.83723E+00_JPRB, 0.85109E+00_JPRB, 0.86282E+00_JPRB, 0.87406E+00_JPRB, 0.87580E+00_JPRB ,&
&  0.87696E+00_JPRB, 0.88332E+00_JPRB, 0.88343E+00_JPRB, 0.86881E+00_JPRB, 0.85424E+00_JPRB, 0.85878E+00_JPRB, 0.85738E+00_JPRB ,&
&  0.84894E+00_JPRB, 0.73543E+00_JPRB, 0.79782E+00_JPRB, 0.83195E+00_JPRB, 0.87168E+00_JPRB, 0.91293E+00_JPRB, 0.93518E+00_JPRB ,&
&  0.92921E+00_JPRB, 0.90425E+00_JPRB, 0.87716E+00_JPRB, 0.84558E+00_JPRB, 0.83594E+00_JPRB, 0.82743E+00_JPRB, 0.81567E+00_JPRB ,&
&  0.80764E+00_JPRB, 0.80005E+00_JPRB, 0.78912E+00_JPRB, 0.77475E+00_JPRB, 0.75704E+00_JPRB, 0.74363E+00_JPRB, 0.73154E+00_JPRB ,&
&  0.72687E+00_JPRB, 0.71782E+00_JPRB, 0.71018E+00_JPRB, 0.71099E+00_JPRB, 0.72391E+00_JPRB, 0.73284E+00_JPRB, 0.73953E+00_JPRB ,&
&  0.75082E+00_JPRB, 0.76892E+00_JPRB, 0.79541E+00_JPRB, 0.82693E+00_JPRB, 0.85279E+00_JPRB, 0.87255E+00_JPRB, 0.89873E+00_JPRB ,&
&  0.93749E+00_JPRB, 0.96401E+00_JPRB, 0.96497E+00_JPRB, 0.94476E+00_JPRB, 0.92738E+00_JPRB, 0.91823E+00_JPRB, 0.89599E+00_JPRB ,&
&  0.86086E+00_JPRB, 0.83167E+00_JPRB, 0.81563E+00_JPRB, 0.80634E+00_JPRB, 0.79768E+00_JPRB, 0.78986E+00_JPRB, 0.78866E+00_JPRB ,&
&  0.78966E+00_JPRB, 0.78883E+00_JPRB, 0.78748E+00_JPRB, 0.79036E+00_JPRB, 0.79683E+00_JPRB, 0.80515E+00_JPRB, 0.81629E+00_JPRB ,&
&  0.83246E+00_JPRB, 0.84685E+00_JPRB, 0.85392E+00_JPRB, 0.86303E+00_JPRB, 0.87479E+00_JPRB, 0.87956E+00_JPRB, 0.89287E+00_JPRB ,&
&  0.89212E+00_JPRB, 0.88741E+00_JPRB, 0.88542E+00_JPRB, 0.89019E+00_JPRB, 0.89141E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.83694E+00_JPRB, 0.81619E+00_JPRB, 0.80360E+00_JPRB, 0.79168E+00_JPRB, 0.78343E+00_JPRB, 0.76536E+00_JPRB, 0.75493E+00_JPRB ,&
&  0.74782E+00_JPRB, 0.74303E+00_JPRB, 0.74132E+00_JPRB, 0.74121E+00_JPRB, 0.75178E+00_JPRB, 0.75895E+00_JPRB, 0.76216E+00_JPRB ,&
&  0.77048E+00_JPRB, 0.78899E+00_JPRB, 0.80630E+00_JPRB, 0.82046E+00_JPRB, 0.83104E+00_JPRB, 0.83805E+00_JPRB, 0.83683E+00_JPRB ,&
&  0.83944E+00_JPRB, 0.84648E+00_JPRB, 0.84951E+00_JPRB, 0.83777E+00_JPRB, 0.82322E+00_JPRB, 0.81559E+00_JPRB, 0.81704E+00_JPRB ,&
&  0.80625E+00_JPRB, 0.77169E+00_JPRB, 0.78995E+00_JPRB, 0.80346E+00_JPRB, 0.81991E+00_JPRB, 0.83387E+00_JPRB, 0.84823E+00_JPRB ,&
&  0.85313E+00_JPRB, 0.83431E+00_JPRB, 0.80794E+00_JPRB, 0.78908E+00_JPRB, 0.79277E+00_JPRB, 0.78935E+00_JPRB, 0.77730E+00_JPRB ,&
&  0.76984E+00_JPRB, 0.76314E+00_JPRB, 0.75367E+00_JPRB, 0.73935E+00_JPRB, 0.72662E+00_JPRB, 0.71723E+00_JPRB, 0.71025E+00_JPRB ,&
&  0.69708E+00_JPRB, 0.68635E+00_JPRB, 0.68063E+00_JPRB, 0.67450E+00_JPRB, 0.67000E+00_JPRB, 0.67561E+00_JPRB, 0.68208E+00_JPRB ,&
&  0.68678E+00_JPRB, 0.69453E+00_JPRB, 0.70905E+00_JPRB, 0.73029E+00_JPRB, 0.74986E+00_JPRB, 0.77031E+00_JPRB, 0.79260E+00_JPRB ,&
&  0.81628E+00_JPRB, 0.83814E+00_JPRB, 0.84742E+00_JPRB, 0.85231E+00_JPRB, 0.84467E+00_JPRB, 0.83116E+00_JPRB, 0.80574E+00_JPRB ,&
&  0.77710E+00_JPRB, 0.76620E+00_JPRB, 0.76225E+00_JPRB, 0.76403E+00_JPRB, 0.75945E+00_JPRB, 0.75561E+00_JPRB, 0.75452E+00_JPRB ,&
&  0.75121E+00_JPRB, 0.74766E+00_JPRB, 0.74220E+00_JPRB, 0.73725E+00_JPRB, 0.74045E+00_JPRB, 0.74616E+00_JPRB, 0.75943E+00_JPRB ,&
&  0.77772E+00_JPRB, 0.79731E+00_JPRB, 0.81670E+00_JPRB, 0.82864E+00_JPRB, 0.83164E+00_JPRB, 0.83319E+00_JPRB, 0.83568E+00_JPRB ,&
&  0.83612E+00_JPRB, 0.83357E+00_JPRB, 0.83897E+00_JPRB, 0.84308E+00_JPRB, 0.84756E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.77641E+00_JPRB, 0.75103E+00_JPRB, 0.73266E+00_JPRB, 0.72596E+00_JPRB, 0.72235E+00_JPRB, 0.70989E+00_JPRB, 0.70407E+00_JPRB ,&
&  0.70955E+00_JPRB, 0.71096E+00_JPRB, 0.71765E+00_JPRB, 0.71693E+00_JPRB, 0.71616E+00_JPRB, 0.71890E+00_JPRB, 0.71134E+00_JPRB ,&
&  0.71155E+00_JPRB, 0.72104E+00_JPRB, 0.73239E+00_JPRB, 0.74768E+00_JPRB, 0.76055E+00_JPRB, 0.77018E+00_JPRB, 0.77803E+00_JPRB ,&
&  0.78094E+00_JPRB, 0.78718E+00_JPRB, 0.79044E+00_JPRB, 0.78552E+00_JPRB, 0.77324E+00_JPRB, 0.76335E+00_JPRB, 0.76115E+00_JPRB ,&
&  0.75915E+00_JPRB, 0.75838E+00_JPRB, 0.71432E+00_JPRB, 0.73989E+00_JPRB, 0.74781E+00_JPRB, 0.75417E+00_JPRB, 0.76003E+00_JPRB ,&
&  0.75800E+00_JPRB, 0.74513E+00_JPRB, 0.73765E+00_JPRB, 0.72272E+00_JPRB, 0.72060E+00_JPRB, 0.72647E+00_JPRB, 0.72281E+00_JPRB ,&
&  0.71543E+00_JPRB, 0.70207E+00_JPRB, 0.68608E+00_JPRB, 0.67434E+00_JPRB, 0.67103E+00_JPRB, 0.66767E+00_JPRB, 0.65796E+00_JPRB ,&
&  0.65184E+00_JPRB, 0.65112E+00_JPRB, 0.64951E+00_JPRB, 0.64317E+00_JPRB, 0.62774E+00_JPRB, 0.61428E+00_JPRB, 0.61328E+00_JPRB ,&
&  0.61995E+00_JPRB, 0.63112E+00_JPRB, 0.65018E+00_JPRB, 0.67333E+00_JPRB, 0.69647E+00_JPRB, 0.71749E+00_JPRB, 0.73575E+00_JPRB ,&
&  0.74299E+00_JPRB, 0.74576E+00_JPRB, 0.74954E+00_JPRB, 0.75650E+00_JPRB, 0.75894E+00_JPRB, 0.75053E+00_JPRB, 0.72721E+00_JPRB ,&
&  0.70218E+00_JPRB, 0.69382E+00_JPRB, 0.69614E+00_JPRB, 0.70391E+00_JPRB, 0.70736E+00_JPRB, 0.70671E+00_JPRB, 0.70083E+00_JPRB ,&
&  0.69778E+00_JPRB, 0.68987E+00_JPRB, 0.68053E+00_JPRB, 0.67562E+00_JPRB, 0.67479E+00_JPRB, 0.68050E+00_JPRB, 0.69813E+00_JPRB ,&
&  0.71995E+00_JPRB, 0.74430E+00_JPRB, 0.76505E+00_JPRB, 0.77183E+00_JPRB, 0.77513E+00_JPRB, 0.77289E+00_JPRB, 0.76268E+00_JPRB ,&
&  0.76639E+00_JPRB, 0.78524E+00_JPRB, 0.79460E+00_JPRB, 0.79563E+00_JPRB, 0.79250E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.72003E+00_JPRB, 0.69131E+00_JPRB, 0.68003E+00_JPRB, 0.68222E+00_JPRB, 0.68558E+00_JPRB, 0.68093E+00_JPRB, 0.68783E+00_JPRB ,&
&  0.69868E+00_JPRB, 0.70701E+00_JPRB, 0.71158E+00_JPRB, 0.70756E+00_JPRB, 0.70487E+00_JPRB, 0.70165E+00_JPRB, 0.68639E+00_JPRB ,&
&  0.68111E+00_JPRB, 0.67931E+00_JPRB, 0.68386E+00_JPRB, 0.69178E+00_JPRB, 0.70067E+00_JPRB, 0.71393E+00_JPRB, 0.72376E+00_JPRB ,&
&  0.73003E+00_JPRB, 0.73380E+00_JPRB, 0.73793E+00_JPRB, 0.73793E+00_JPRB, 0.73389E+00_JPRB, 0.72386E+00_JPRB, 0.71606E+00_JPRB ,&
&  0.71124E+00_JPRB, 0.70074E+00_JPRB, 0.68979E+00_JPRB, 0.68840E+00_JPRB, 0.69150E+00_JPRB, 0.69470E+00_JPRB, 0.69554E+00_JPRB ,&
&  0.68510E+00_JPRB, 0.67024E+00_JPRB, 0.66996E+00_JPRB, 0.66899E+00_JPRB, 0.67318E+00_JPRB, 0.68497E+00_JPRB, 0.68782E+00_JPRB ,&
&  0.68306E+00_JPRB, 0.66713E+00_JPRB, 0.64644E+00_JPRB, 0.64194E+00_JPRB, 0.64333E+00_JPRB, 0.63435E+00_JPRB, 0.62610E+00_JPRB ,&
&  0.62482E+00_JPRB, 0.62218E+00_JPRB, 0.61635E+00_JPRB, 0.60812E+00_JPRB, 0.59482E+00_JPRB, 0.57782E+00_JPRB, 0.56985E+00_JPRB ,&
&  0.57141E+00_JPRB, 0.59111E+00_JPRB, 0.62403E+00_JPRB, 0.65936E+00_JPRB, 0.69097E+00_JPRB, 0.70526E+00_JPRB, 0.70345E+00_JPRB ,&
&  0.68811E+00_JPRB, 0.68196E+00_JPRB, 0.68582E+00_JPRB, 0.69090E+00_JPRB, 0.69339E+00_JPRB, 0.68313E+00_JPRB, 0.66419E+00_JPRB ,&
&  0.64981E+00_JPRB, 0.63997E+00_JPRB, 0.63896E+00_JPRB, 0.64366E+00_JPRB, 0.64823E+00_JPRB, 0.64938E+00_JPRB, 0.64412E+00_JPRB ,&
&  0.63934E+00_JPRB, 0.63358E+00_JPRB, 0.62930E+00_JPRB, 0.63079E+00_JPRB, 0.63434E+00_JPRB, 0.63860E+00_JPRB, 0.65279E+00_JPRB ,&
&  0.67365E+00_JPRB, 0.69391E+00_JPRB, 0.69826E+00_JPRB, 0.69351E+00_JPRB, 0.69225E+00_JPRB, 0.68871E+00_JPRB, 0.68534E+00_JPRB ,&
&  0.69497E+00_JPRB, 0.71729E+00_JPRB, 0.73316E+00_JPRB, 0.74143E+00_JPRB, 0.74098E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.63987E+00_JPRB, 0.62317E+00_JPRB, 0.62423E+00_JPRB, 0.63223E+00_JPRB, 0.63670E+00_JPRB, 0.64451E+00_JPRB, 0.66387E+00_JPRB ,&
&  0.67463E+00_JPRB, 0.68129E+00_JPRB, 0.68378E+00_JPRB, 0.67869E+00_JPRB, 0.67791E+00_JPRB, 0.67068E+00_JPRB, 0.65769E+00_JPRB ,&
&  0.65215E+00_JPRB, 0.65101E+00_JPRB, 0.65273E+00_JPRB, 0.65690E+00_JPRB, 0.65966E+00_JPRB, 0.66331E+00_JPRB, 0.66736E+00_JPRB ,&
&  0.66965E+00_JPRB, 0.66978E+00_JPRB, 0.66881E+00_JPRB, 0.66886E+00_JPRB, 0.66760E+00_JPRB, 0.65940E+00_JPRB, 0.64998E+00_JPRB ,&
&  0.64005E+00_JPRB, 0.63093E+00_JPRB, 0.62784E+00_JPRB, 0.63159E+00_JPRB, 0.63606E+00_JPRB, 0.63850E+00_JPRB, 0.63596E+00_JPRB ,&
&  0.63201E+00_JPRB, 0.63150E+00_JPRB, 0.63589E+00_JPRB, 0.64288E+00_JPRB, 0.65095E+00_JPRB, 0.66024E+00_JPRB, 0.66510E+00_JPRB ,&
&  0.65889E+00_JPRB, 0.64021E+00_JPRB, 0.62864E+00_JPRB, 0.62750E+00_JPRB, 0.62203E+00_JPRB, 0.60487E+00_JPRB, 0.59342E+00_JPRB ,&
&  0.58849E+00_JPRB, 0.57937E+00_JPRB, 0.56975E+00_JPRB, 0.55708E+00_JPRB, 0.54570E+00_JPRB, 0.54127E+00_JPRB, 0.54050E+00_JPRB ,&
&  0.54519E+00_JPRB, 0.56775E+00_JPRB, 0.60645E+00_JPRB, 0.64204E+00_JPRB, 0.65633E+00_JPRB, 0.64844E+00_JPRB, 0.62903E+00_JPRB ,&
&  0.61840E+00_JPRB, 0.61279E+00_JPRB, 0.61305E+00_JPRB, 0.61333E+00_JPRB, 0.61089E+00_JPRB, 0.60234E+00_JPRB, 0.59439E+00_JPRB ,&
&  0.58337E+00_JPRB, 0.57166E+00_JPRB, 0.56721E+00_JPRB, 0.56465E+00_JPRB, 0.56190E+00_JPRB, 0.56244E+00_JPRB, 0.56246E+00_JPRB ,&
&  0.56410E+00_JPRB, 0.56845E+00_JPRB, 0.57962E+00_JPRB, 0.58446E+00_JPRB, 0.58340E+00_JPRB, 0.57285E+00_JPRB, 0.57503E+00_JPRB ,&
&  0.57923E+00_JPRB, 0.57762E+00_JPRB, 0.57295E+00_JPRB, 0.57272E+00_JPRB, 0.57269E+00_JPRB, 0.57475E+00_JPRB, 0.58715E+00_JPRB ,&
&  0.61450E+00_JPRB, 0.63783E+00_JPRB, 0.65684E+00_JPRB, 0.66177E+00_JPRB, 0.65739E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.58090E+00_JPRB, 0.58642E+00_JPRB, 0.60009E+00_JPRB, 0.61127E+00_JPRB, 0.61487E+00_JPRB, 0.62478E+00_JPRB, 0.63587E+00_JPRB ,&
&  0.64414E+00_JPRB, 0.64515E+00_JPRB, 0.64114E+00_JPRB, 0.63129E+00_JPRB, 0.61974E+00_JPRB, 0.61012E+00_JPRB, 0.60651E+00_JPRB ,&
&  0.60879E+00_JPRB, 0.61174E+00_JPRB, 0.61723E+00_JPRB, 0.62043E+00_JPRB, 0.62451E+00_JPRB, 0.62382E+00_JPRB, 0.62488E+00_JPRB ,&
&  0.62730E+00_JPRB, 0.62873E+00_JPRB, 0.62930E+00_JPRB, 0.62535E+00_JPRB, 0.62203E+00_JPRB, 0.61896E+00_JPRB, 0.60947E+00_JPRB ,&
&  0.59919E+00_JPRB, 0.59266E+00_JPRB, 0.59644E+00_JPRB, 0.60734E+00_JPRB, 0.61405E+00_JPRB, 0.61045E+00_JPRB, 0.60840E+00_JPRB ,&
&  0.60861E+00_JPRB, 0.61449E+00_JPRB, 0.62398E+00_JPRB, 0.63697E+00_JPRB, 0.64504E+00_JPRB, 0.65547E+00_JPRB, 0.65590E+00_JPRB ,&
&  0.64934E+00_JPRB, 0.63983E+00_JPRB, 0.63657E+00_JPRB, 0.63454E+00_JPRB, 0.60303E+00_JPRB, 0.56861E+00_JPRB, 0.55190E+00_JPRB ,&
&  0.54286E+00_JPRB, 0.53186E+00_JPRB, 0.51593E+00_JPRB, 0.50506E+00_JPRB, 0.50436E+00_JPRB, 0.50980E+00_JPRB, 0.52176E+00_JPRB ,&
&  0.53165E+00_JPRB, 0.54617E+00_JPRB, 0.56325E+00_JPRB, 0.57451E+00_JPRB, 0.57457E+00_JPRB, 0.56132E+00_JPRB, 0.54829E+00_JPRB ,&
&  0.54000E+00_JPRB, 0.53697E+00_JPRB, 0.53485E+00_JPRB, 0.53281E+00_JPRB, 0.53125E+00_JPRB, 0.53073E+00_JPRB, 0.52991E+00_JPRB ,&
&  0.52583E+00_JPRB, 0.51890E+00_JPRB, 0.51053E+00_JPRB, 0.50618E+00_JPRB, 0.50628E+00_JPRB, 0.50732E+00_JPRB, 0.50975E+00_JPRB ,&
&  0.51875E+00_JPRB, 0.53021E+00_JPRB, 0.53884E+00_JPRB, 0.54144E+00_JPRB, 0.53699E+00_JPRB, 0.52804E+00_JPRB, 0.52295E+00_JPRB ,&
&  0.51958E+00_JPRB, 0.51731E+00_JPRB, 0.51405E+00_JPRB, 0.51284E+00_JPRB, 0.51321E+00_JPRB, 0.52169E+00_JPRB, 0.53885E+00_JPRB ,&
&  0.55924E+00_JPRB, 0.58508E+00_JPRB, 0.59634E+00_JPRB, 0.59442E+00_JPRB, 0.58714E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.54878E+00_JPRB, 0.56214E+00_JPRB, 0.57876E+00_JPRB, 0.59013E+00_JPRB, 0.59478E+00_JPRB, 0.59982E+00_JPRB, 0.60195E+00_JPRB ,&
&  0.59578E+00_JPRB, 0.59291E+00_JPRB, 0.58734E+00_JPRB, 0.57975E+00_JPRB, 0.57322E+00_JPRB, 0.57013E+00_JPRB, 0.57080E+00_JPRB ,&
&  0.57300E+00_JPRB, 0.57487E+00_JPRB, 0.57159E+00_JPRB, 0.57014E+00_JPRB, 0.57463E+00_JPRB, 0.58066E+00_JPRB, 0.58475E+00_JPRB ,&
&  0.58682E+00_JPRB, 0.58887E+00_JPRB, 0.59404E+00_JPRB, 0.59721E+00_JPRB, 0.59717E+00_JPRB, 0.59014E+00_JPRB, 0.58460E+00_JPRB ,&
&  0.57902E+00_JPRB, 0.57684E+00_JPRB, 0.58221E+00_JPRB, 0.57480E+00_JPRB, 0.58888E+00_JPRB, 0.58999E+00_JPRB, 0.59468E+00_JPRB ,&
&  0.60225E+00_JPRB, 0.61605E+00_JPRB, 0.62754E+00_JPRB, 0.63651E+00_JPRB, 0.63893E+00_JPRB, 0.64164E+00_JPRB, 0.63125E+00_JPRB ,&
&  0.61724E+00_JPRB, 0.59852E+00_JPRB, 0.57947E+00_JPRB, 0.55788E+00_JPRB, 0.53594E+00_JPRB, 0.51914E+00_JPRB, 0.51147E+00_JPRB ,&
&  0.50206E+00_JPRB, 0.48787E+00_JPRB, 0.47445E+00_JPRB, 0.46948E+00_JPRB, 0.47433E+00_JPRB, 0.48455E+00_JPRB, 0.50021E+00_JPRB ,&
&  0.51060E+00_JPRB, 0.51372E+00_JPRB, 0.51088E+00_JPRB, 0.50720E+00_JPRB, 0.50313E+00_JPRB, 0.49488E+00_JPRB, 0.48982E+00_JPRB ,&
&  0.48585E+00_JPRB, 0.48026E+00_JPRB, 0.47750E+00_JPRB, 0.47874E+00_JPRB, 0.48145E+00_JPRB, 0.48357E+00_JPRB, 0.48283E+00_JPRB ,&
&  0.48079E+00_JPRB, 0.47599E+00_JPRB, 0.46800E+00_JPRB, 0.46468E+00_JPRB, 0.46621E+00_JPRB, 0.47119E+00_JPRB, 0.47860E+00_JPRB ,&
&  0.48097E+00_JPRB, 0.48750E+00_JPRB, 0.49350E+00_JPRB, 0.49520E+00_JPRB, 0.49733E+00_JPRB, 0.49863E+00_JPRB, 0.49823E+00_JPRB ,&
&  0.49602E+00_JPRB, 0.49077E+00_JPRB, 0.48414E+00_JPRB, 0.48226E+00_JPRB, 0.48451E+00_JPRB, 0.49250E+00_JPRB, 0.50243E+00_JPRB ,&
&  0.51191E+00_JPRB, 0.52678E+00_JPRB, 0.53775E+00_JPRB, 0.54004E+00_JPRB, 0.54225E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.52485E+00_JPRB, 0.53710E+00_JPRB, 0.55125E+00_JPRB, 0.56062E+00_JPRB, 0.55918E+00_JPRB, 0.55864E+00_JPRB, 0.56419E+00_JPRB ,&
&  0.55675E+00_JPRB, 0.54874E+00_JPRB, 0.54265E+00_JPRB, 0.53691E+00_JPRB, 0.53543E+00_JPRB, 0.53475E+00_JPRB, 0.53828E+00_JPRB ,&
&  0.54091E+00_JPRB, 0.54023E+00_JPRB, 0.53910E+00_JPRB, 0.54053E+00_JPRB, 0.54792E+00_JPRB, 0.55695E+00_JPRB, 0.56450E+00_JPRB ,&
&  0.56645E+00_JPRB, 0.57065E+00_JPRB, 0.57707E+00_JPRB, 0.58251E+00_JPRB, 0.58508E+00_JPRB, 0.58635E+00_JPRB, 0.58469E+00_JPRB ,&
&  0.58147E+00_JPRB, 0.57722E+00_JPRB, 0.55990E+00_JPRB, 0.57154E+00_JPRB, 0.57612E+00_JPRB, 0.57649E+00_JPRB, 0.58044E+00_JPRB ,&
&  0.58767E+00_JPRB, 0.59703E+00_JPRB, 0.60191E+00_JPRB, 0.60365E+00_JPRB, 0.59865E+00_JPRB, 0.58871E+00_JPRB, 0.56360E+00_JPRB ,&
&  0.54257E+00_JPRB, 0.52101E+00_JPRB, 0.50744E+00_JPRB, 0.49880E+00_JPRB, 0.49162E+00_JPRB, 0.48570E+00_JPRB, 0.48084E+00_JPRB ,&
&  0.47406E+00_JPRB, 0.46809E+00_JPRB, 0.46436E+00_JPRB, 0.46400E+00_JPRB, 0.46769E+00_JPRB, 0.47828E+00_JPRB, 0.49135E+00_JPRB ,&
&  0.49692E+00_JPRB, 0.48843E+00_JPRB, 0.48003E+00_JPRB, 0.47030E+00_JPRB, 0.45338E+00_JPRB, 0.42774E+00_JPRB, 0.41105E+00_JPRB ,&
&  0.40769E+00_JPRB, 0.42032E+00_JPRB, 0.44050E+00_JPRB, 0.44722E+00_JPRB, 0.45343E+00_JPRB, 0.45071E+00_JPRB, 0.44490E+00_JPRB ,&
&  0.43405E+00_JPRB, 0.41197E+00_JPRB, 0.40145E+00_JPRB, 0.39893E+00_JPRB, 0.39744E+00_JPRB, 0.39907E+00_JPRB, 0.40218E+00_JPRB ,&
&  0.40731E+00_JPRB, 0.41380E+00_JPRB, 0.41833E+00_JPRB, 0.42002E+00_JPRB, 0.42268E+00_JPRB, 0.42303E+00_JPRB, 0.42192E+00_JPRB ,&
&  0.41883E+00_JPRB, 0.42679E+00_JPRB, 0.44657E+00_JPRB, 0.47339E+00_JPRB, 0.48020E+00_JPRB, 0.48880E+00_JPRB, 0.49759E+00_JPRB ,&
&  0.50325E+00_JPRB, 0.50938E+00_JPRB, 0.51051E+00_JPRB, 0.51104E+00_JPRB, 0.51592E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.49689E+00_JPRB, 0.50409E+00_JPRB, 0.51317E+00_JPRB, 0.52479E+00_JPRB, 0.52712E+00_JPRB, 0.52857E+00_JPRB, 0.53730E+00_JPRB ,&
&  0.53687E+00_JPRB, 0.53339E+00_JPRB, 0.53032E+00_JPRB, 0.53116E+00_JPRB, 0.53414E+00_JPRB, 0.53736E+00_JPRB, 0.53792E+00_JPRB ,&
&  0.52943E+00_JPRB, 0.52477E+00_JPRB, 0.52823E+00_JPRB, 0.53202E+00_JPRB, 0.53713E+00_JPRB, 0.54368E+00_JPRB, 0.54960E+00_JPRB ,&
&  0.55313E+00_JPRB, 0.55732E+00_JPRB, 0.56061E+00_JPRB, 0.56741E+00_JPRB, 0.57728E+00_JPRB, 0.58423E+00_JPRB, 0.58962E+00_JPRB ,&
&  0.57187E+00_JPRB, 0.52336E+00_JPRB, 0.49369E+00_JPRB, 0.49920E+00_JPRB, 0.54420E+00_JPRB, 0.55532E+00_JPRB, 0.55608E+00_JPRB ,&
&  0.55874E+00_JPRB, 0.56605E+00_JPRB, 0.56579E+00_JPRB, 0.56031E+00_JPRB, 0.54806E+00_JPRB, 0.52832E+00_JPRB, 0.50454E+00_JPRB ,&
&  0.48895E+00_JPRB, 0.47911E+00_JPRB, 0.47219E+00_JPRB, 0.46483E+00_JPRB, 0.46111E+00_JPRB, 0.45611E+00_JPRB, 0.44813E+00_JPRB ,&
&  0.44479E+00_JPRB, 0.44403E+00_JPRB, 0.44549E+00_JPRB, 0.44635E+00_JPRB, 0.44755E+00_JPRB, 0.44772E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.42966E+00_JPRB, 0.41299E+00_JPRB, 0.39740E+00_JPRB, 0.38038E+00_JPRB, 0.36630E+00_JPRB, 0.35822E+00_JPRB, 0.35136E+00_JPRB ,&
&  0.34914E+00_JPRB, 0.35683E+00_JPRB, 0.37712E+00_JPRB, 0.39912E+00_JPRB, 0.42627E+00_JPRB, 0.39946E+00_JPRB, 0.37611E+00_JPRB ,&
&  0.36584E+00_JPRB, 0.36064E+00_JPRB, 0.35772E+00_JPRB, 0.35812E+00_JPRB, 0.36011E+00_JPRB, 0.36064E+00_JPRB, 0.36088E+00_JPRB ,&
&  0.36282E+00_JPRB, 0.36600E+00_JPRB, 0.36804E+00_JPRB, 0.36991E+00_JPRB, 0.37272E+00_JPRB, 0.37586E+00_JPRB, 0.37747E+00_JPRB ,&
&  0.37855E+00_JPRB, 0.38439E+00_JPRB, 0.39519E+00_JPRB, 0.40783E+00_JPRB, 0.41985E+00_JPRB, 0.43229E+00_JPRB, 0.44930E+00_JPRB ,&
&  0.46950E+00_JPRB, 0.49187E+00_JPRB, 0.49284E+00_JPRB, 0.49479E+00_JPRB, 0.49471E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.48006E+00_JPRB, 0.49188E+00_JPRB, 0.49761E+00_JPRB, 0.50343E+00_JPRB, 0.50610E+00_JPRB, 0.50771E+00_JPRB, 0.51586E+00_JPRB ,&
&  0.52082E+00_JPRB, 0.52494E+00_JPRB, 0.52775E+00_JPRB, 0.53000E+00_JPRB, 0.52707E+00_JPRB, 0.52118E+00_JPRB, 0.52023E+00_JPRB ,&
&  0.51933E+00_JPRB, 0.51991E+00_JPRB, 0.51922E+00_JPRB, 0.51932E+00_JPRB, 0.51989E+00_JPRB, 0.52461E+00_JPRB, 0.53187E+00_JPRB ,&
&  0.53356E+00_JPRB, 0.53761E+00_JPRB, 0.55131E+00_JPRB, 0.55904E+00_JPRB, 0.56392E+00_JPRB, 0.56267E+00_JPRB, 0.55595E+00_JPRB ,&
&  0.52546E+00_JPRB, 0.49653E+00_JPRB, 0.47092E+00_JPRB, 0.47196E+00_JPRB, 0.52191E+00_JPRB, 0.52703E+00_JPRB, 0.52730E+00_JPRB ,&
&  0.52588E+00_JPRB, 0.52972E+00_JPRB, 0.52997E+00_JPRB, 0.51505E+00_JPRB, 0.50425E+00_JPRB, 0.49009E+00_JPRB, 0.47728E+00_JPRB ,&
&  0.46526E+00_JPRB, 0.45798E+00_JPRB, 0.44388E+00_JPRB, 0.41518E+00_JPRB, 0.39897E+00_JPRB, 0.39062E+00_JPRB, 0.37664E+00_JPRB ,&
&  0.36577E+00_JPRB, 0.36333E+00_JPRB, 0.36273E+00_JPRB, 0.36084E+00_JPRB, 0.36465E+00_JPRB, 0.36685E+00_JPRB, 0.36444E+00_JPRB ,&
&  0.35950E+00_JPRB, 0.35550E+00_JPRB, 0.35106E+00_JPRB, 0.34540E+00_JPRB, 0.34261E+00_JPRB, 0.34217E+00_JPRB, 0.34332E+00_JPRB ,&
&  0.34862E+00_JPRB, 0.35582E+00_JPRB, 0.37111E+00_JPRB, 0.38150E+00_JPRB, 0.37476E+00_JPRB, 0.36186E+00_JPRB, 0.35178E+00_JPRB ,&
&  0.34437E+00_JPRB, 0.34147E+00_JPRB, 0.33919E+00_JPRB, 0.33668E+00_JPRB, 0.33740E+00_JPRB, 0.33987E+00_JPRB, 0.34128E+00_JPRB ,&
&  0.34137E+00_JPRB, 0.34374E+00_JPRB, 0.34529E+00_JPRB, 0.34694E+00_JPRB, 0.34834E+00_JPRB, 0.35057E+00_JPRB, 0.35357E+00_JPRB ,&
&  0.35609E+00_JPRB, 0.36188E+00_JPRB, 0.36801E+00_JPRB, 0.37510E+00_JPRB, 0.37967E+00_JPRB, 0.38316E+00_JPRB, 0.39093E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.41273E+00_JPRB, 0.44542E+00_JPRB, 0.47897E+00_JPRB, 0.46897E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.46734E+00_JPRB, 0.47646E+00_JPRB, 0.48431E+00_JPRB, 0.49054E+00_JPRB, 0.49486E+00_JPRB, 0.49606E+00_JPRB, 0.49923E+00_JPRB ,&
&  0.50728E+00_JPRB, 0.51219E+00_JPRB, 0.51644E+00_JPRB, 0.51527E+00_JPRB, 0.51476E+00_JPRB, 0.49726E+00_JPRB, 0.49409E+00_JPRB ,&
&  0.48758E+00_JPRB, 0.48300E+00_JPRB, 0.47622E+00_JPRB, 0.47585E+00_JPRB, 0.47544E+00_JPRB, 0.47783E+00_JPRB, 0.47880E+00_JPRB ,&
&  0.48003E+00_JPRB, 0.47953E+00_JPRB, 0.47690E+00_JPRB, 0.47535E+00_JPRB, 0.47635E+00_JPRB, 0.47717E+00_JPRB, 0.47375E+00_JPRB ,&
&  0.46898E+00_JPRB, 0.46754E+00_JPRB, 0.47038E+00_JPRB, 0.47627E+00_JPRB, 0.51470E+00_JPRB, 0.50823E+00_JPRB, 0.50680E+00_JPRB ,&
&  0.50342E+00_JPRB, 0.49897E+00_JPRB, 0.49801E+00_JPRB, 0.49080E+00_JPRB, 0.48072E+00_JPRB, 0.45640E+00_JPRB, 0.44310E+00_JPRB ,&
&  0.40944E+00_JPRB, 0.39159E+00_JPRB, 0.37607E+00_JPRB, 0.36607E+00_JPRB, 0.35461E+00_JPRB, 0.34790E+00_JPRB, 0.34219E+00_JPRB ,&
&  0.33780E+00_JPRB, 0.33417E+00_JPRB, 0.33093E+00_JPRB, 0.33183E+00_JPRB, 0.33391E+00_JPRB, 0.33476E+00_JPRB, 0.33572E+00_JPRB ,&
&  0.33628E+00_JPRB, 0.33685E+00_JPRB, 0.33750E+00_JPRB, 0.33918E+00_JPRB, 0.34141E+00_JPRB, 0.34438E+00_JPRB, 0.34818E+00_JPRB ,&
&  0.35422E+00_JPRB, 0.35874E+00_JPRB, 0.36008E+00_JPRB, 0.36328E+00_JPRB, 0.36289E+00_JPRB, 0.35581E+00_JPRB, 0.34831E+00_JPRB ,&
&  0.34024E+00_JPRB, 0.33605E+00_JPRB, 0.33183E+00_JPRB, 0.32876E+00_JPRB, 0.32798E+00_JPRB, 0.32780E+00_JPRB, 0.32998E+00_JPRB ,&
&  0.33242E+00_JPRB, 0.33472E+00_JPRB, 0.33638E+00_JPRB, 0.33812E+00_JPRB, 0.34033E+00_JPRB, 0.34188E+00_JPRB, 0.34389E+00_JPRB ,&
&  0.34652E+00_JPRB, 0.35003E+00_JPRB, 0.35528E+00_JPRB, 0.36145E+00_JPRB, 0.37001E+00_JPRB, 0.38188E+00_JPRB, 0.40169E+00_JPRB ,&
&  0.42610E+00_JPRB, 0.47890E+00_JPRB, 0.48109E+00_JPRB, 0.45399E+00_JPRB, 0.46077E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.44338E+00_JPRB, 0.45104E+00_JPRB, 0.45813E+00_JPRB, 0.46589E+00_JPRB, 0.47560E+00_JPRB, 0.48003E+00_JPRB, 0.48202E+00_JPRB ,&
&  0.47757E+00_JPRB, 0.47376E+00_JPRB, 0.46348E+00_JPRB, 0.45518E+00_JPRB, 0.44611E+00_JPRB, 0.43908E+00_JPRB, 0.43290E+00_JPRB ,&
&  0.42853E+00_JPRB, 0.42524E+00_JPRB, 0.41957E+00_JPRB, 0.41589E+00_JPRB, 0.41227E+00_JPRB, 0.41271E+00_JPRB, 0.41364E+00_JPRB ,&
&  0.41378E+00_JPRB, 0.41472E+00_JPRB, 0.41306E+00_JPRB, 0.41824E+00_JPRB, 0.42772E+00_JPRB, 0.43885E+00_JPRB, 0.45340E+00_JPRB ,&
&  0.47365E+00_JPRB, 0.48422E+00_JPRB, 0.49971E+00_JPRB, 0.49935E+00_JPRB, 0.49765E+00_JPRB, 0.49405E+00_JPRB, 0.49091E+00_JPRB ,&
&  0.48604E+00_JPRB, 0.47894E+00_JPRB, 0.47340E+00_JPRB, 0.46610E+00_JPRB, 0.43497E+00_JPRB, 0.41158E+00_JPRB, 0.39337E+00_JPRB ,&
&  0.37858E+00_JPRB, 0.36458E+00_JPRB, 0.35176E+00_JPRB, 0.34548E+00_JPRB, 0.33971E+00_JPRB, 0.33629E+00_JPRB, 0.33265E+00_JPRB ,&
&  0.32874E+00_JPRB, 0.32780E+00_JPRB, 0.32592E+00_JPRB, 0.32516E+00_JPRB, 0.32649E+00_JPRB, 0.32623E+00_JPRB, 0.32702E+00_JPRB ,&
&  0.32787E+00_JPRB, 0.32950E+00_JPRB, 0.33308E+00_JPRB, 0.33571E+00_JPRB, 0.33832E+00_JPRB, 0.34069E+00_JPRB, 0.34171E+00_JPRB ,&
&  0.34393E+00_JPRB, 0.34592E+00_JPRB, 0.34582E+00_JPRB, 0.34491E+00_JPRB, 0.34275E+00_JPRB, 0.33760E+00_JPRB, 0.33261E+00_JPRB ,&
&  0.32720E+00_JPRB, 0.32330E+00_JPRB, 0.32207E+00_JPRB, 0.32085E+00_JPRB, 0.32186E+00_JPRB, 0.32256E+00_JPRB, 0.32324E+00_JPRB ,&
&  0.32525E+00_JPRB, 0.32723E+00_JPRB, 0.32970E+00_JPRB, 0.33341E+00_JPRB, 0.33604E+00_JPRB, 0.33817E+00_JPRB, 0.34076E+00_JPRB ,&
&  0.34024E+00_JPRB, 0.34494E+00_JPRB, 0.35095E+00_JPRB, 0.35973E+00_JPRB, 0.37639E+00_JPRB, 0.38723E+00_JPRB, 0.41110E+00_JPRB ,&
&  0.44273E+00_JPRB, 0.50175E+00_JPRB, 0.49315E+00_JPRB, 0.42990E+00_JPRB, 0.43618E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.42945E+00_JPRB, 0.43352E+00_JPRB, 0.43691E+00_JPRB, 0.44125E+00_JPRB, 0.44647E+00_JPRB, 0.45143E+00_JPRB, 0.45501E+00_JPRB ,&
&  0.45762E+00_JPRB, 0.45615E+00_JPRB, 0.45775E+00_JPRB, 0.45234E+00_JPRB, 0.44701E+00_JPRB, 0.43993E+00_JPRB, 0.43344E+00_JPRB ,&
&  0.42718E+00_JPRB, 0.41795E+00_JPRB, 0.40817E+00_JPRB, 0.39967E+00_JPRB, 0.39235E+00_JPRB, 0.38701E+00_JPRB, 0.38308E+00_JPRB ,&
&  0.38031E+00_JPRB, 0.38142E+00_JPRB, 0.38349E+00_JPRB, 0.38963E+00_JPRB, 0.39993E+00_JPRB, 0.41212E+00_JPRB, 0.43110E+00_JPRB ,&
&  0.45727E+00_JPRB, 0.45938E+00_JPRB, 0.46629E+00_JPRB, 0.46557E+00_JPRB, 0.46151E+00_JPRB, 0.46020E+00_JPRB, 0.45769E+00_JPRB ,&
&  0.45846E+00_JPRB, 0.44819E+00_JPRB, 0.45765E+00_JPRB, 0.43837E+00_JPRB, 0.40873E+00_JPRB, 0.39328E+00_JPRB, 0.37450E+00_JPRB ,&
&  0.36190E+00_JPRB, 0.35110E+00_JPRB, 0.34315E+00_JPRB, 0.34053E+00_JPRB, 0.33608E+00_JPRB, 0.33412E+00_JPRB, 0.32962E+00_JPRB ,&
&  0.32518E+00_JPRB, 0.32347E+00_JPRB, 0.32011E+00_JPRB, 0.32029E+00_JPRB, 0.32066E+00_JPRB, 0.32001E+00_JPRB, 0.32017E+00_JPRB ,&
&  0.32070E+00_JPRB, 0.32223E+00_JPRB, 0.32499E+00_JPRB, 0.32744E+00_JPRB, 0.32892E+00_JPRB, 0.32976E+00_JPRB, 0.32956E+00_JPRB ,&
&  0.33020E+00_JPRB, 0.32986E+00_JPRB, 0.32886E+00_JPRB, 0.32733E+00_JPRB, 0.32483E+00_JPRB, 0.32219E+00_JPRB, 0.31959E+00_JPRB ,&
&  0.31738E+00_JPRB, 0.31638E+00_JPRB, 0.31540E+00_JPRB, 0.31537E+00_JPRB, 0.31575E+00_JPRB, 0.31764E+00_JPRB, 0.31939E+00_JPRB ,&
&  0.32145E+00_JPRB, 0.32454E+00_JPRB, 0.32717E+00_JPRB, 0.33045E+00_JPRB, 0.33328E+00_JPRB, 0.33454E+00_JPRB, 0.33903E+00_JPRB ,&
&  0.34494E+00_JPRB, 0.35338E+00_JPRB, 0.36330E+00_JPRB, 0.37238E+00_JPRB, 0.38371E+00_JPRB, 0.39889E+00_JPRB, 0.41882E+00_JPRB ,&
&  0.43812E+00_JPRB, 0.45604E+00_JPRB, 0.44770E+00_JPRB, 0.42370E+00_JPRB, 0.42487E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.42452E+00_JPRB, 0.42394E+00_JPRB, 0.42544E+00_JPRB, 0.42753E+00_JPRB, 0.43291E+00_JPRB, 0.43954E+00_JPRB, 0.44664E+00_JPRB ,&
&  0.44666E+00_JPRB, 0.45041E+00_JPRB, 0.44904E+00_JPRB, 0.44535E+00_JPRB, 0.43953E+00_JPRB, 0.43306E+00_JPRB, 0.42834E+00_JPRB ,&
&  0.42174E+00_JPRB, 0.41319E+00_JPRB, 0.40492E+00_JPRB, 0.39625E+00_JPRB, 0.38819E+00_JPRB, 0.37983E+00_JPRB, 0.37580E+00_JPRB ,&
&  0.37227E+00_JPRB, 0.37146E+00_JPRB, 0.37351E+00_JPRB, 0.37580E+00_JPRB, 0.38106E+00_JPRB, 0.38832E+00_JPRB, 0.39806E+00_JPRB ,&
&  0.40718E+00_JPRB, 0.41838E+00_JPRB, 0.42438E+00_JPRB, 0.42420E+00_JPRB, 0.42420E+00_JPRB, 0.42229E+00_JPRB, 0.41895E+00_JPRB ,&
&  0.41190E+00_JPRB, 0.40426E+00_JPRB, 0.39508E+00_JPRB, 0.38537E+00_JPRB, 0.37543E+00_JPRB, 0.36672E+00_JPRB, 0.35861E+00_JPRB ,&
&  0.35206E+00_JPRB, 0.34650E+00_JPRB, 0.34123E+00_JPRB, 0.33677E+00_JPRB, 0.33334E+00_JPRB, 0.33088E+00_JPRB, 0.32748E+00_JPRB ,&
&  0.32479E+00_JPRB, 0.32237E+00_JPRB, 0.31943E+00_JPRB, 0.31750E+00_JPRB, 0.31574E+00_JPRB, 0.31461E+00_JPRB, 0.31367E+00_JPRB ,&
&  0.31382E+00_JPRB, 0.31401E+00_JPRB, 0.31515E+00_JPRB, 0.31629E+00_JPRB, 0.31754E+00_JPRB, 0.31775E+00_JPRB, 0.31769E+00_JPRB ,&
&  0.31734E+00_JPRB, 0.31694E+00_JPRB, 0.31511E+00_JPRB, 0.31365E+00_JPRB, 0.31259E+00_JPRB, 0.31142E+00_JPRB, 0.31119E+00_JPRB ,&
&  0.31195E+00_JPRB, 0.31274E+00_JPRB, 0.31326E+00_JPRB, 0.31501E+00_JPRB, 0.31672E+00_JPRB, 0.31932E+00_JPRB, 0.32253E+00_JPRB ,&
&  0.32481E+00_JPRB, 0.32752E+00_JPRB, 0.33050E+00_JPRB, 0.33398E+00_JPRB, 0.33822E+00_JPRB, 0.34252E+00_JPRB, 0.34771E+00_JPRB ,&
&  0.35360E+00_JPRB, 0.35947E+00_JPRB, 0.36606E+00_JPRB, 0.37337E+00_JPRB, 0.38218E+00_JPRB, 0.38705E+00_JPRB, 0.39544E+00_JPRB ,&
&  0.40112E+00_JPRB, 0.39443E+00_JPRB, 0.43472E+00_JPRB, 0.42820E+00_JPRB, 0.42454E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.36929E+00_JPRB, 0.37565E+00_JPRB, 0.37945E+00_JPRB, 0.38287E+00_JPRB, 0.37183E+00_JPRB, 0.40921E+00_JPRB, 0.40714E+00_JPRB ,&
&  0.40861E+00_JPRB, 0.39897E+00_JPRB, 0.40196E+00_JPRB, 0.40208E+00_JPRB, 0.40017E+00_JPRB, 0.39703E+00_JPRB, 0.39401E+00_JPRB ,&
&  0.39128E+00_JPRB, 0.38699E+00_JPRB, 0.38171E+00_JPRB, 0.37706E+00_JPRB, 0.37348E+00_JPRB, 0.37044E+00_JPRB, 0.36778E+00_JPRB ,&
&  0.36713E+00_JPRB, 0.36850E+00_JPRB, 0.37104E+00_JPRB, 0.36854E+00_JPRB, 0.36869E+00_JPRB, 0.36992E+00_JPRB, 0.37175E+00_JPRB ,&
&  0.37333E+00_JPRB, 0.37494E+00_JPRB, 0.37488E+00_JPRB, 0.37294E+00_JPRB, 0.37257E+00_JPRB, 0.37068E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.36626E+00_JPRB, 0.36351E+00_JPRB, 0.36016E+00_JPRB, 0.35750E+00_JPRB, 0.35449E+00_JPRB, 0.35081E+00_JPRB, 0.34731E+00_JPRB ,&
&  0.34429E+00_JPRB, 0.34155E+00_JPRB, 0.33829E+00_JPRB, 0.33528E+00_JPRB, 0.33237E+00_JPRB, 0.32942E+00_JPRB, 0.32667E+00_JPRB ,&
&  0.32456E+00_JPRB, 0.32278E+00_JPRB, 0.32104E+00_JPRB, 0.31920E+00_JPRB, 0.31746E+00_JPRB, 0.31577E+00_JPRB, 0.31448E+00_JPRB ,&
&  0.31452E+00_JPRB, 0.31385E+00_JPRB, 0.31321E+00_JPRB, 0.31301E+00_JPRB, 0.31333E+00_JPRB, 0.31287E+00_JPRB, 0.31274E+00_JPRB ,&
&  0.31276E+00_JPRB, 0.31285E+00_JPRB, 0.31311E+00_JPRB, 0.31343E+00_JPRB, 0.31388E+00_JPRB, 0.31449E+00_JPRB, 0.31548E+00_JPRB ,&
&  0.31665E+00_JPRB, 0.31808E+00_JPRB, 0.31964E+00_JPRB, 0.32122E+00_JPRB, 0.32283E+00_JPRB, 0.32454E+00_JPRB, 0.32648E+00_JPRB ,&
&  0.32840E+00_JPRB, 0.33026E+00_JPRB, 0.33204E+00_JPRB, 0.33379E+00_JPRB, 0.33688E+00_JPRB, 0.33980E+00_JPRB, 0.34292E+00_JPRB ,&
&  0.34630E+00_JPRB, 0.35026E+00_JPRB, 0.35445E+00_JPRB, 0.35906E+00_JPRB, 0.36438E+00_JPRB, 0.36499E+00_JPRB, 0.36536E+00_JPRB ,&
&  0.36521E+00_JPRB, 0.36832E+00_JPRB, 0.36904E+00_JPRB, 0.36754E+00_JPRB, 0.36636E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.34374E+00_JPRB, 0.34362E+00_JPRB, 0.34341E+00_JPRB, 0.34311E+00_JPRB, 0.34087E+00_JPRB, 0.34099E+00_JPRB, 0.34126E+00_JPRB ,&
&  0.34201E+00_JPRB, 0.34401E+00_JPRB, 0.34622E+00_JPRB, 0.34786E+00_JPRB, 0.34896E+00_JPRB, 0.34969E+00_JPRB, 0.35029E+00_JPRB ,&
&  0.35043E+00_JPRB, 0.35084E+00_JPRB, 0.34654E+00_JPRB, 0.34497E+00_JPRB, 0.34435E+00_JPRB, 0.34423E+00_JPRB, 0.34440E+00_JPRB ,&
&  0.34445E+00_JPRB, 0.34433E+00_JPRB, 0.34357E+00_JPRB, 0.34393E+00_JPRB, 0.34433E+00_JPRB, 0.34457E+00_JPRB, 0.34483E+00_JPRB ,&
&  0.34509E+00_JPRB, 0.34535E+00_JPRB, 0.34529E+00_JPRB, 0.34502E+00_JPRB, 0.34496E+00_JPRB, 0.34455E+00_JPRB, 0.34372E+00_JPRB ,&
&  0.34268E+00_JPRB, 0.34133E+00_JPRB, 0.34000E+00_JPRB, 0.33878E+00_JPRB, 0.33773E+00_JPRB, 0.33684E+00_JPRB, 0.33597E+00_JPRB ,&
&  0.33487E+00_JPRB, 0.33338E+00_JPRB, 0.33179E+00_JPRB, 0.33023E+00_JPRB, 0.32873E+00_JPRB, 0.32737E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32479E+00_JPRB, 0.32387E+00_JPRB, 0.32297E+00_JPRB, 0.32221E+00_JPRB, 0.32143E+00_JPRB, 0.32069E+00_JPRB, 0.31998E+00_JPRB ,&
&  0.31898E+00_JPRB, 0.31849E+00_JPRB, 0.31793E+00_JPRB, 0.31773E+00_JPRB, 0.31791E+00_JPRB, 0.31806E+00_JPRB, 0.31831E+00_JPRB ,&
&  0.31864E+00_JPRB, 0.31902E+00_JPRB, 0.31920E+00_JPRB, 0.31947E+00_JPRB, 0.31986E+00_JPRB, 0.32036E+00_JPRB, 0.32094E+00_JPRB ,&
&  0.32157E+00_JPRB, 0.32225E+00_JPRB, 0.32299E+00_JPRB, 0.32382E+00_JPRB, 0.32469E+00_JPRB, 0.32564E+00_JPRB, 0.32653E+00_JPRB ,&
&  0.32745E+00_JPRB, 0.32829E+00_JPRB, 0.32915E+00_JPRB, 0.32981E+00_JPRB, 0.33082E+00_JPRB, 0.33177E+00_JPRB, 0.33292E+00_JPRB ,&
&  0.33407E+00_JPRB, 0.33531E+00_JPRB, 0.33640E+00_JPRB, 0.33760E+00_JPRB, 0.33895E+00_JPRB, 0.33856E+00_JPRB, 0.33833E+00_JPRB ,&
&  0.33814E+00_JPRB, 0.33835E+00_JPRB, 0.33809E+00_JPRB, 0.33835E+00_JPRB, 0.33928E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB, 0.32590E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB, 0.63999E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.64801E+00_JPRB, 0.64833E+00_JPRB, 0.64852E+00_JPRB, 0.64856E+00_JPRB, 0.64854E+00_JPRB, 0.64874E+00_JPRB, 0.64885E+00_JPRB ,&
&  0.64896E+00_JPRB, 0.64907E+00_JPRB, 0.64954E+00_JPRB, 0.64962E+00_JPRB, 0.64968E+00_JPRB, 0.64953E+00_JPRB, 0.64939E+00_JPRB ,&
&  0.64916E+00_JPRB, 0.64893E+00_JPRB, 0.64860E+00_JPRB, 0.64839E+00_JPRB, 0.64827E+00_JPRB, 0.64830E+00_JPRB, 0.64827E+00_JPRB ,&
&  0.64796E+00_JPRB, 0.64731E+00_JPRB, 0.64681E+00_JPRB, 0.64606E+00_JPRB, 0.64512E+00_JPRB, 0.64373E+00_JPRB, 0.64205E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.63831E+00_JPRB, 0.63681E+00_JPRB, 0.63538E+00_JPRB, 0.63428E+00_JPRB, 0.63342E+00_JPRB, 0.63281E+00_JPRB ,&
&  0.63203E+00_JPRB, 0.63158E+00_JPRB, 0.63116E+00_JPRB, 0.63084E+00_JPRB, 0.63011E+00_JPRB, 0.62954E+00_JPRB, 0.62885E+00_JPRB ,&
&  0.62816E+00_JPRB, 0.62721E+00_JPRB, 0.62656E+00_JPRB, 0.62582E+00_JPRB, 0.62501E+00_JPRB, 0.62355E+00_JPRB, 0.62231E+00_JPRB ,&
&  0.62112E+00_JPRB, 0.61992E+00_JPRB, 0.61856E+00_JPRB, 0.61737E+00_JPRB, 0.61655E+00_JPRB, 0.61576E+00_JPRB, 0.61480E+00_JPRB ,&
&  0.61460E+00_JPRB, 0.61512E+00_JPRB, 0.61561E+00_JPRB, 0.61586E+00_JPRB, 0.61655E+00_JPRB, 0.61738E+00_JPRB, 0.61829E+00_JPRB ,&
&  0.61912E+00_JPRB, 0.62025E+00_JPRB, 0.62187E+00_JPRB, 0.62335E+00_JPRB, 0.62494E+00_JPRB, 0.62657E+00_JPRB, 0.62842E+00_JPRB ,&
&  0.63012E+00_JPRB, 0.63179E+00_JPRB, 0.63329E+00_JPRB, 0.63494E+00_JPRB, 0.63605E+00_JPRB, 0.63710E+00_JPRB, 0.63775E+00_JPRB ,&
&  0.63844E+00_JPRB, 0.63892E+00_JPRB, 0.63947E+00_JPRB, 0.63992E+00_JPRB, 0.64060E+00_JPRB, 0.64114E+00_JPRB, 0.64173E+00_JPRB ,&
&  0.64229E+00_JPRB, 0.64302E+00_JPRB, 0.64359E+00_JPRB, 0.64411E+00_JPRB, 0.64455E+00_JPRB, 0.64524E+00_JPRB, 0.64576E+00_JPRB ,&
&  0.64622E+00_JPRB, 0.64671E+00_JPRB, 0.64721E+00_JPRB, 0.64763E+00_JPRB, 0.64787E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.64278E+00_JPRB, 0.64494E+00_JPRB, 0.64615E+00_JPRB, 0.64730E+00_JPRB, 0.64834E+00_JPRB, 0.65002E+00_JPRB, 0.65125E+00_JPRB ,&
&  0.65247E+00_JPRB, 0.65416E+00_JPRB, 0.65659E+00_JPRB, 0.65800E+00_JPRB, 0.65905E+00_JPRB, 0.65957E+00_JPRB, 0.65996E+00_JPRB ,&
&  0.65978E+00_JPRB, 0.65916E+00_JPRB, 0.65832E+00_JPRB, 0.65757E+00_JPRB, 0.65682E+00_JPRB, 0.65615E+00_JPRB, 0.65515E+00_JPRB ,&
&  0.65332E+00_JPRB, 0.65097E+00_JPRB, 0.64841E+00_JPRB, 0.64451E+00_JPRB, 0.64018E+00_JPRB, 0.63561E+00_JPRB, 0.63078E+00_JPRB ,&
&  0.62563E+00_JPRB, 0.62160E+00_JPRB, 0.61776E+00_JPRB, 0.61414E+00_JPRB, 0.61278E+00_JPRB, 0.61143E+00_JPRB, 0.61001E+00_JPRB ,&
&  0.60857E+00_JPRB, 0.60626E+00_JPRB, 0.60556E+00_JPRB, 0.60519E+00_JPRB, 0.60525E+00_JPRB, 0.60636E+00_JPRB, 0.60725E+00_JPRB ,&
&  0.60780E+00_JPRB, 0.60766E+00_JPRB, 0.60735E+00_JPRB, 0.60753E+00_JPRB, 0.60800E+00_JPRB, 0.60787E+00_JPRB, 0.60675E+00_JPRB ,&
&  0.60593E+00_JPRB, 0.60458E+00_JPRB, 0.60300E+00_JPRB, 0.60113E+00_JPRB, 0.60023E+00_JPRB, 0.59902E+00_JPRB, 0.59765E+00_JPRB ,&
&  0.59686E+00_JPRB, 0.59683E+00_JPRB, 0.59702E+00_JPRB, 0.59731E+00_JPRB, 0.59831E+00_JPRB, 0.59962E+00_JPRB, 0.60091E+00_JPRB ,&
&  0.60228E+00_JPRB, 0.60463E+00_JPRB, 0.60733E+00_JPRB, 0.60974E+00_JPRB, 0.61223E+00_JPRB, 0.61468E+00_JPRB, 0.61752E+00_JPRB ,&
&  0.61972E+00_JPRB, 0.62141E+00_JPRB, 0.62213E+00_JPRB, 0.62366E+00_JPRB, 0.62491E+00_JPRB, 0.62663E+00_JPRB, 0.62775E+00_JPRB ,&
&  0.62954E+00_JPRB, 0.63057E+00_JPRB, 0.63116E+00_JPRB, 0.63177E+00_JPRB, 0.63251E+00_JPRB, 0.63302E+00_JPRB, 0.63320E+00_JPRB ,&
&  0.63325E+00_JPRB, 0.63345E+00_JPRB, 0.63353E+00_JPRB, 0.63352E+00_JPRB, 0.63381E+00_JPRB, 0.63424E+00_JPRB, 0.63480E+00_JPRB ,&
&  0.63553E+00_JPRB, 0.63695E+00_JPRB, 0.63873E+00_JPRB, 0.63999E+00_JPRB, 0.64121E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.63396E+00_JPRB, 0.63568E+00_JPRB, 0.63696E+00_JPRB, 0.63905E+00_JPRB, 0.64290E+00_JPRB, 0.64703E+00_JPRB, 0.64953E+00_JPRB ,&
&  0.65295E+00_JPRB, 0.65697E+00_JPRB, 0.66150E+00_JPRB, 0.66433E+00_JPRB, 0.66651E+00_JPRB, 0.66769E+00_JPRB, 0.66775E+00_JPRB ,&
&  0.66662E+00_JPRB, 0.66484E+00_JPRB, 0.66354E+00_JPRB, 0.66190E+00_JPRB, 0.65999E+00_JPRB, 0.65788E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.64836E+00_JPRB, 0.64530E+00_JPRB, 0.63911E+00_JPRB, 0.63204E+00_JPRB, 0.62632E+00_JPRB, 0.58645E+00_JPRB, 0.57304E+00_JPRB ,&
&  0.57178E+00_JPRB, 0.56831E+00_JPRB, 0.56921E+00_JPRB, 0.57074E+00_JPRB, 0.57563E+00_JPRB, 0.56919E+00_JPRB, 0.56497E+00_JPRB ,&
&  0.54080E+00_JPRB, 0.54138E+00_JPRB, 0.53755E+00_JPRB, 0.53557E+00_JPRB, 0.53813E+00_JPRB, 0.54008E+00_JPRB, 0.54830E+00_JPRB ,&
&  0.57028E+00_JPRB, 0.57656E+00_JPRB, 0.57971E+00_JPRB, 0.58226E+00_JPRB, 0.58520E+00_JPRB, 0.58700E+00_JPRB, 0.58856E+00_JPRB ,&
&  0.58921E+00_JPRB, 0.58910E+00_JPRB, 0.58933E+00_JPRB, 0.58885E+00_JPRB, 0.58833E+00_JPRB, 0.58715E+00_JPRB, 0.58635E+00_JPRB ,&
&  0.58584E+00_JPRB, 0.58660E+00_JPRB, 0.58810E+00_JPRB, 0.58958E+00_JPRB, 0.59174E+00_JPRB, 0.59322E+00_JPRB, 0.59459E+00_JPRB ,&
&  0.59591E+00_JPRB, 0.59720E+00_JPRB, 0.59887E+00_JPRB, 0.60065E+00_JPRB, 0.60288E+00_JPRB, 0.60413E+00_JPRB, 0.60615E+00_JPRB ,&
&  0.60769E+00_JPRB, 0.60978E+00_JPRB, 0.61142E+00_JPRB, 0.61492E+00_JPRB, 0.61739E+00_JPRB, 0.62117E+00_JPRB, 0.62436E+00_JPRB ,&
&  0.62834E+00_JPRB, 0.63085E+00_JPRB, 0.63312E+00_JPRB, 0.63463E+00_JPRB, 0.63633E+00_JPRB, 0.63756E+00_JPRB, 0.63793E+00_JPRB ,&
&  0.63791E+00_JPRB, 0.63690E+00_JPRB, 0.63571E+00_JPRB, 0.63402E+00_JPRB, 0.63269E+00_JPRB, 0.63092E+00_JPRB, 0.62952E+00_JPRB ,&
&  0.62774E+00_JPRB, 0.62763E+00_JPRB, 0.62870E+00_JPRB, 0.63060E+00_JPRB, 0.63229E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.64344E+00_JPRB, 0.64338E+00_JPRB, 0.64291E+00_JPRB, 0.64509E+00_JPRB, 0.64789E+00_JPRB, 0.65325E+00_JPRB, 0.65456E+00_JPRB ,&
&  0.65949E+00_JPRB, 0.66450E+00_JPRB, 0.67011E+00_JPRB, 0.67274E+00_JPRB, 0.67508E+00_JPRB, 0.67514E+00_JPRB, 0.67363E+00_JPRB ,&
&  0.67154E+00_JPRB, 0.66767E+00_JPRB, 0.66486E+00_JPRB, 0.66070E+00_JPRB, 0.65565E+00_JPRB, 0.65219E+00_JPRB, 0.64621E+00_JPRB ,&
&  0.64268E+00_JPRB, 0.63758E+00_JPRB, 0.59995E+00_JPRB, 0.58883E+00_JPRB, 0.56999E+00_JPRB, 0.55872E+00_JPRB, 0.55267E+00_JPRB ,&
&  0.54475E+00_JPRB, 0.53706E+00_JPRB, 0.52665E+00_JPRB, 0.51419E+00_JPRB, 0.50094E+00_JPRB, 0.48196E+00_JPRB, 0.47390E+00_JPRB ,&
&  0.46618E+00_JPRB, 0.46481E+00_JPRB, 0.46670E+00_JPRB, 0.47153E+00_JPRB, 0.48203E+00_JPRB, 0.49269E+00_JPRB, 0.50645E+00_JPRB ,&
&  0.52586E+00_JPRB, 0.54530E+00_JPRB, 0.55524E+00_JPRB, 0.55358E+00_JPRB, 0.56903E+00_JPRB, 0.57666E+00_JPRB, 0.58095E+00_JPRB ,&
&  0.58280E+00_JPRB, 0.58100E+00_JPRB, 0.58607E+00_JPRB, 0.58630E+00_JPRB, 0.58656E+00_JPRB, 0.58719E+00_JPRB, 0.58844E+00_JPRB ,&
&  0.59118E+00_JPRB, 0.59400E+00_JPRB, 0.59611E+00_JPRB, 0.59798E+00_JPRB, 0.59906E+00_JPRB, 0.59932E+00_JPRB, 0.59942E+00_JPRB ,&
&  0.59973E+00_JPRB, 0.60015E+00_JPRB, 0.60086E+00_JPRB, 0.60175E+00_JPRB, 0.60353E+00_JPRB, 0.60458E+00_JPRB, 0.60522E+00_JPRB ,&
&  0.60688E+00_JPRB, 0.60941E+00_JPRB, 0.61223E+00_JPRB, 0.61160E+00_JPRB, 0.61491E+00_JPRB, 0.62369E+00_JPRB, 0.62840E+00_JPRB ,&
&  0.63600E+00_JPRB, 0.64175E+00_JPRB, 0.64837E+00_JPRB, 0.65350E+00_JPRB, 0.65798E+00_JPRB, 0.65987E+00_JPRB, 0.65952E+00_JPRB ,&
&  0.65863E+00_JPRB, 0.65665E+00_JPRB, 0.65522E+00_JPRB, 0.65220E+00_JPRB, 0.64959E+00_JPRB, 0.64587E+00_JPRB, 0.64338E+00_JPRB ,&
&  0.64046E+00_JPRB, 0.63964E+00_JPRB, 0.63734E+00_JPRB, 0.63929E+00_JPRB, 0.64109E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.65818E+00_JPRB, 0.65745E+00_JPRB, 0.65848E+00_JPRB, 0.66353E+00_JPRB, 0.67267E+00_JPRB, 0.68311E+00_JPRB, 0.68245E+00_JPRB ,&
&  0.68294E+00_JPRB, 0.68114E+00_JPRB, 0.68371E+00_JPRB, 0.68352E+00_JPRB, 0.68281E+00_JPRB, 0.68030E+00_JPRB, 0.67822E+00_JPRB ,&
&  0.67598E+00_JPRB, 0.66917E+00_JPRB, 0.66051E+00_JPRB, 0.65643E+00_JPRB, 0.65518E+00_JPRB, 0.65389E+00_JPRB, 0.64804E+00_JPRB ,&
&  0.64090E+00_JPRB, 0.62595E+00_JPRB, 0.59731E+00_JPRB, 0.57862E+00_JPRB, 0.55845E+00_JPRB, 0.55073E+00_JPRB, 0.53754E+00_JPRB ,&
&  0.53861E+00_JPRB, 0.49740E+00_JPRB, 0.50000E+00_JPRB, 0.48297E+00_JPRB, 0.47131E+00_JPRB, 0.45815E+00_JPRB, 0.44272E+00_JPRB ,&
&  0.43834E+00_JPRB, 0.43267E+00_JPRB, 0.43184E+00_JPRB, 0.43990E+00_JPRB, 0.44977E+00_JPRB, 0.46620E+00_JPRB, 0.48369E+00_JPRB ,&
&  0.49537E+00_JPRB, 0.53977E+00_JPRB, 0.55352E+00_JPRB, 0.55889E+00_JPRB, 0.57308E+00_JPRB, 0.58267E+00_JPRB, 0.58399E+00_JPRB ,&
&  0.58485E+00_JPRB, 0.58426E+00_JPRB, 0.59091E+00_JPRB, 0.59513E+00_JPRB, 0.60106E+00_JPRB, 0.60087E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.60411E+00_JPRB, 0.60768E+00_JPRB, 0.61183E+00_JPRB, 0.61542E+00_JPRB, 0.61866E+00_JPRB, 0.62073E+00_JPRB, 0.62158E+00_JPRB ,&
&  0.62176E+00_JPRB, 0.62095E+00_JPRB, 0.62063E+00_JPRB, 0.62165E+00_JPRB, 0.62276E+00_JPRB, 0.62562E+00_JPRB, 0.62721E+00_JPRB ,&
&  0.62896E+00_JPRB, 0.62946E+00_JPRB, 0.62874E+00_JPRB, 0.62973E+00_JPRB, 0.63376E+00_JPRB, 0.63803E+00_JPRB, 0.64539E+00_JPRB ,&
&  0.65581E+00_JPRB, 0.66252E+00_JPRB, 0.67012E+00_JPRB, 0.67669E+00_JPRB, 0.68226E+00_JPRB, 0.68315E+00_JPRB, 0.68145E+00_JPRB ,&
&  0.67990E+00_JPRB, 0.67786E+00_JPRB, 0.67551E+00_JPRB, 0.67136E+00_JPRB, 0.66701E+00_JPRB, 0.66129E+00_JPRB, 0.65665E+00_JPRB ,&
&  0.65248E+00_JPRB, 0.65200E+00_JPRB, 0.64918E+00_JPRB, 0.65137E+00_JPRB, 0.65476E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.67463E+00_JPRB, 0.67614E+00_JPRB, 0.68133E+00_JPRB, 0.68841E+00_JPRB, 0.69610E+00_JPRB, 0.70492E+00_JPRB, 0.70869E+00_JPRB ,&
&  0.71145E+00_JPRB, 0.70970E+00_JPRB, 0.70622E+00_JPRB, 0.69442E+00_JPRB, 0.68668E+00_JPRB, 0.68296E+00_JPRB, 0.67845E+00_JPRB ,&
&  0.67255E+00_JPRB, 0.66819E+00_JPRB, 0.66475E+00_JPRB, 0.66277E+00_JPRB, 0.65840E+00_JPRB, 0.65545E+00_JPRB, 0.64530E+00_JPRB ,&
&  0.62699E+00_JPRB, 0.61005E+00_JPRB, 0.58522E+00_JPRB, 0.56467E+00_JPRB, 0.55065E+00_JPRB, 0.53703E+00_JPRB, 0.53582E+00_JPRB ,&
&  0.53257E+00_JPRB, 0.52998E+00_JPRB, 0.52633E+00_JPRB, 0.52310E+00_JPRB, 0.52404E+00_JPRB, 0.45453E+00_JPRB, 0.43861E+00_JPRB ,&
&  0.42830E+00_JPRB, 0.41973E+00_JPRB, 0.41781E+00_JPRB, 0.41812E+00_JPRB, 0.42476E+00_JPRB, 0.44605E+00_JPRB, 0.47400E+00_JPRB ,&
&  0.49171E+00_JPRB, 0.55177E+00_JPRB, 0.56354E+00_JPRB, 0.57428E+00_JPRB, 0.59296E+00_JPRB, 0.60497E+00_JPRB, 0.60871E+00_JPRB ,&
&  0.61650E+00_JPRB, 0.63031E+00_JPRB, 0.64732E+00_JPRB, 0.66350E+00_JPRB, 0.66466E+00_JPRB, 0.66933E+00_JPRB, 0.66870E+00_JPRB ,&
&  0.66372E+00_JPRB, 0.65537E+00_JPRB, 0.65260E+00_JPRB, 0.65151E+00_JPRB, 0.64994E+00_JPRB, 0.65018E+00_JPRB, 0.64767E+00_JPRB ,&
&  0.64981E+00_JPRB, 0.64857E+00_JPRB, 0.65425E+00_JPRB, 0.65678E+00_JPRB, 0.66210E+00_JPRB, 0.66785E+00_JPRB, 0.67059E+00_JPRB ,&
&  0.67063E+00_JPRB, 0.67121E+00_JPRB, 0.66687E+00_JPRB, 0.65918E+00_JPRB, 0.65684E+00_JPRB, 0.65401E+00_JPRB, 0.65689E+00_JPRB ,&
&  0.66234E+00_JPRB, 0.67294E+00_JPRB, 0.67894E+00_JPRB, 0.68549E+00_JPRB, 0.69114E+00_JPRB, 0.69173E+00_JPRB, 0.69335E+00_JPRB ,&
&  0.69387E+00_JPRB, 0.69495E+00_JPRB, 0.69325E+00_JPRB, 0.68939E+00_JPRB, 0.68289E+00_JPRB, 0.67462E+00_JPRB, 0.66626E+00_JPRB ,&
&  0.66255E+00_JPRB, 0.66336E+00_JPRB, 0.66213E+00_JPRB, 0.66448E+00_JPRB, 0.66942E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.69036E+00_JPRB, 0.69282E+00_JPRB, 0.70097E+00_JPRB, 0.70839E+00_JPRB, 0.71808E+00_JPRB, 0.72700E+00_JPRB, 0.73060E+00_JPRB ,&
&  0.74011E+00_JPRB, 0.73696E+00_JPRB, 0.72342E+00_JPRB, 0.70641E+00_JPRB, 0.68822E+00_JPRB, 0.67591E+00_JPRB, 0.67037E+00_JPRB ,&
&  0.66987E+00_JPRB, 0.66249E+00_JPRB, 0.65753E+00_JPRB, 0.65437E+00_JPRB, 0.64875E+00_JPRB, 0.64389E+00_JPRB, 0.63377E+00_JPRB ,&
&  0.61301E+00_JPRB, 0.59934E+00_JPRB, 0.58473E+00_JPRB, 0.56387E+00_JPRB, 0.54643E+00_JPRB, 0.53710E+00_JPRB, 0.53879E+00_JPRB ,&
&  0.53725E+00_JPRB, 0.52819E+00_JPRB, 0.52805E+00_JPRB, 0.52374E+00_JPRB, 0.51295E+00_JPRB, 0.50853E+00_JPRB, 0.45701E+00_JPRB ,&
&  0.44027E+00_JPRB, 0.42150E+00_JPRB, 0.41489E+00_JPRB, 0.41473E+00_JPRB, 0.42871E+00_JPRB, 0.45115E+00_JPRB, 0.49185E+00_JPRB ,&
&  0.52902E+00_JPRB, 0.56272E+00_JPRB, 0.58781E+00_JPRB, 0.60626E+00_JPRB, 0.62523E+00_JPRB, 0.64278E+00_JPRB, 0.65494E+00_JPRB ,&
&  0.67580E+00_JPRB, 0.70372E+00_JPRB, 0.73068E+00_JPRB, 0.75635E+00_JPRB, 0.77349E+00_JPRB, 0.77909E+00_JPRB, 0.78345E+00_JPRB ,&
&  0.77599E+00_JPRB, 0.71159E+00_JPRB, 0.68609E+00_JPRB, 0.67113E+00_JPRB, 0.66115E+00_JPRB, 0.66070E+00_JPRB, 0.65581E+00_JPRB ,&
&  0.65841E+00_JPRB, 0.66347E+00_JPRB, 0.66571E+00_JPRB, 0.66900E+00_JPRB, 0.67378E+00_JPRB, 0.67601E+00_JPRB, 0.68261E+00_JPRB ,&
&  0.69430E+00_JPRB, 0.71030E+00_JPRB, 0.73845E+00_JPRB, 0.71849E+00_JPRB, 0.69445E+00_JPRB, 0.67133E+00_JPRB, 0.65928E+00_JPRB ,&
&  0.65732E+00_JPRB, 0.66230E+00_JPRB, 0.67166E+00_JPRB, 0.67629E+00_JPRB, 0.68129E+00_JPRB, 0.68598E+00_JPRB, 0.69813E+00_JPRB ,&
&  0.70248E+00_JPRB, 0.70652E+00_JPRB, 0.70756E+00_JPRB, 0.70573E+00_JPRB, 0.69851E+00_JPRB, 0.68961E+00_JPRB, 0.68151E+00_JPRB ,&
&  0.67888E+00_JPRB, 0.68030E+00_JPRB, 0.68120E+00_JPRB, 0.68356E+00_JPRB, 0.68705E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.69658E+00_JPRB, 0.70621E+00_JPRB, 0.72302E+00_JPRB, 0.73729E+00_JPRB, 0.74880E+00_JPRB, 0.73972E+00_JPRB, 0.72847E+00_JPRB ,&
&  0.72641E+00_JPRB, 0.72416E+00_JPRB, 0.71292E+00_JPRB, 0.69978E+00_JPRB, 0.68061E+00_JPRB, 0.66557E+00_JPRB, 0.66087E+00_JPRB ,&
&  0.66006E+00_JPRB, 0.65435E+00_JPRB, 0.64571E+00_JPRB, 0.63355E+00_JPRB, 0.62304E+00_JPRB, 0.62015E+00_JPRB, 0.61380E+00_JPRB ,&
&  0.60104E+00_JPRB, 0.59346E+00_JPRB, 0.58494E+00_JPRB, 0.56991E+00_JPRB, 0.55567E+00_JPRB, 0.54762E+00_JPRB, 0.54445E+00_JPRB ,&
&  0.54237E+00_JPRB, 0.54113E+00_JPRB, 0.54074E+00_JPRB, 0.53556E+00_JPRB, 0.53377E+00_JPRB, 0.53116E+00_JPRB, 0.49294E+00_JPRB ,&
&  0.45557E+00_JPRB, 0.43062E+00_JPRB, 0.41655E+00_JPRB, 0.41628E+00_JPRB, 0.43611E+00_JPRB, 0.47618E+00_JPRB, 0.52088E+00_JPRB ,&
&  0.56026E+00_JPRB, 0.60131E+00_JPRB, 0.63389E+00_JPRB, 0.65296E+00_JPRB, 0.66799E+00_JPRB, 0.68689E+00_JPRB, 0.70910E+00_JPRB ,&
&  0.74029E+00_JPRB, 0.78011E+00_JPRB, 0.81654E+00_JPRB, 0.85448E+00_JPRB, 0.88227E+00_JPRB, 0.85588E+00_JPRB, 0.91416E+00_JPRB ,&
&  0.11081E+01_JPRB, 0.91349E+00_JPRB, 0.76354E+00_JPRB, 0.70832E+00_JPRB, 0.69731E+00_JPRB, 0.70114E+00_JPRB, 0.70658E+00_JPRB ,&
&  0.71511E+00_JPRB, 0.71595E+00_JPRB, 0.71079E+00_JPRB, 0.70704E+00_JPRB, 0.71126E+00_JPRB, 0.72067E+00_JPRB, 0.74241E+00_JPRB ,&
&  0.77779E+00_JPRB, 0.93143E+00_JPRB, 0.11723E+01_JPRB, 0.89800E+00_JPRB, 0.79658E+00_JPRB, 0.72371E+00_JPRB, 0.66471E+00_JPRB ,&
&  0.64107E+00_JPRB, 0.63600E+00_JPRB, 0.64256E+00_JPRB, 0.64850E+00_JPRB, 0.65024E+00_JPRB, 0.65557E+00_JPRB, 0.66452E+00_JPRB ,&
&  0.67890E+00_JPRB, 0.69369E+00_JPRB, 0.70316E+00_JPRB, 0.70865E+00_JPRB, 0.70648E+00_JPRB, 0.69785E+00_JPRB, 0.68520E+00_JPRB ,&
&  0.68547E+00_JPRB, 0.69235E+00_JPRB, 0.69714E+00_JPRB, 0.69919E+00_JPRB, 0.69692E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.69830E+00_JPRB, 0.72929E+00_JPRB, 0.76768E+00_JPRB, 0.77384E+00_JPRB, 0.76933E+00_JPRB, 0.73073E+00_JPRB, 0.69152E+00_JPRB ,&
&  0.66842E+00_JPRB, 0.65903E+00_JPRB, 0.65543E+00_JPRB, 0.64571E+00_JPRB, 0.63623E+00_JPRB, 0.63141E+00_JPRB, 0.62779E+00_JPRB ,&
&  0.62469E+00_JPRB, 0.62135E+00_JPRB, 0.61661E+00_JPRB, 0.60619E+00_JPRB, 0.59995E+00_JPRB, 0.60186E+00_JPRB, 0.60033E+00_JPRB ,&
&  0.59451E+00_JPRB, 0.59219E+00_JPRB, 0.58042E+00_JPRB, 0.56458E+00_JPRB, 0.55683E+00_JPRB, 0.55626E+00_JPRB, 0.55940E+00_JPRB ,&
&  0.55211E+00_JPRB, 0.54603E+00_JPRB, 0.55245E+00_JPRB, 0.56112E+00_JPRB, 0.55437E+00_JPRB, 0.54792E+00_JPRB, 0.50467E+00_JPRB ,&
&  0.46293E+00_JPRB, 0.43767E+00_JPRB, 0.43135E+00_JPRB, 0.45852E+00_JPRB, 0.50810E+00_JPRB, 0.56969E+00_JPRB, 0.61015E+00_JPRB ,&
&  0.63341E+00_JPRB, 0.66295E+00_JPRB, 0.68976E+00_JPRB, 0.71758E+00_JPRB, 0.75276E+00_JPRB, 0.79029E+00_JPRB, 0.83433E+00_JPRB ,&
&  0.86881E+00_JPRB, 0.88949E+00_JPRB, 0.88813E+00_JPRB, 0.81177E+00_JPRB, 0.80005E+00_JPRB, 0.80917E+00_JPRB, 0.88649E+00_JPRB ,&
&  0.96365E+00_JPRB, 0.88709E+00_JPRB, 0.83213E+00_JPRB, 0.82888E+00_JPRB, 0.82008E+00_JPRB, 0.82989E+00_JPRB, 0.83211E+00_JPRB ,&
&  0.81785E+00_JPRB, 0.79296E+00_JPRB, 0.76672E+00_JPRB, 0.74656E+00_JPRB, 0.74962E+00_JPRB, 0.77204E+00_JPRB, 0.79636E+00_JPRB ,&
&  0.84377E+00_JPRB, 0.94604E+00_JPRB, 0.92753E+00_JPRB, 0.72821E+00_JPRB, 0.69823E+00_JPRB, 0.68356E+00_JPRB, 0.66736E+00_JPRB ,&
&  0.64226E+00_JPRB, 0.62736E+00_JPRB, 0.62660E+00_JPRB, 0.62795E+00_JPRB, 0.62322E+00_JPRB, 0.61889E+00_JPRB, 0.61979E+00_JPRB ,&
&  0.62606E+00_JPRB, 0.63848E+00_JPRB, 0.65456E+00_JPRB, 0.68096E+00_JPRB, 0.69414E+00_JPRB, 0.69407E+00_JPRB, 0.68204E+00_JPRB ,&
&  0.67629E+00_JPRB, 0.67989E+00_JPRB, 0.68147E+00_JPRB, 0.67954E+00_JPRB, 0.68378E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.80022E+00_JPRB, 0.80778E+00_JPRB, 0.80377E+00_JPRB, 0.80115E+00_JPRB, 0.76668E+00_JPRB, 0.74648E+00_JPRB, 0.71140E+00_JPRB ,&
&  0.70189E+00_JPRB, 0.67663E+00_JPRB, 0.64418E+00_JPRB, 0.60830E+00_JPRB, 0.58901E+00_JPRB, 0.58143E+00_JPRB, 0.58311E+00_JPRB ,&
&  0.58949E+00_JPRB, 0.59699E+00_JPRB, 0.59918E+00_JPRB, 0.58940E+00_JPRB, 0.59100E+00_JPRB, 0.59398E+00_JPRB, 0.59308E+00_JPRB ,&
&  0.59111E+00_JPRB, 0.59202E+00_JPRB, 0.58544E+00_JPRB, 0.57448E+00_JPRB, 0.56804E+00_JPRB, 0.56554E+00_JPRB, 0.56041E+00_JPRB ,&
&  0.55067E+00_JPRB, 0.54741E+00_JPRB, 0.55855E+00_JPRB, 0.58178E+00_JPRB, 0.56402E+00_JPRB, 0.55596E+00_JPRB, 0.50484E+00_JPRB ,&
&  0.46718E+00_JPRB, 0.46111E+00_JPRB, 0.49155E+00_JPRB, 0.55730E+00_JPRB, 0.58976E+00_JPRB, 0.62453E+00_JPRB, 0.64531E+00_JPRB ,&
&  0.64543E+00_JPRB, 0.66331E+00_JPRB, 0.70902E+00_JPRB, 0.79332E+00_JPRB, 0.83067E+00_JPRB, 0.87944E+00_JPRB, 0.92737E+00_JPRB ,&
&  0.96383E+00_JPRB, 0.98981E+00_JPRB, 0.10007E+01_JPRB, 0.95333E+00_JPRB, 0.97698E+00_JPRB, 0.10680E+01_JPRB, 0.96849E+00_JPRB ,&
&  0.96342E+00_JPRB, 0.92644E+00_JPRB, 0.93056E+00_JPRB, 0.97920E+00_JPRB, 0.93700E+00_JPRB, 0.95224E+00_JPRB, 0.93338E+00_JPRB ,&
&  0.87095E+00_JPRB, 0.83572E+00_JPRB, 0.81028E+00_JPRB, 0.80348E+00_JPRB, 0.80629E+00_JPRB, 0.82607E+00_JPRB, 0.85904E+00_JPRB ,&
&  0.90667E+00_JPRB, 0.92555E+00_JPRB, 0.86449E+00_JPRB, 0.74269E+00_JPRB, 0.69663E+00_JPRB, 0.67611E+00_JPRB, 0.66059E+00_JPRB ,&
&  0.64336E+00_JPRB, 0.63203E+00_JPRB, 0.63239E+00_JPRB, 0.63157E+00_JPRB, 0.62556E+00_JPRB, 0.61261E+00_JPRB, 0.60154E+00_JPRB ,&
&  0.59706E+00_JPRB, 0.60005E+00_JPRB, 0.60219E+00_JPRB, 0.62829E+00_JPRB, 0.65711E+00_JPRB, 0.67903E+00_JPRB, 0.67834E+00_JPRB ,&
&  0.67794E+00_JPRB, 0.69018E+00_JPRB, 0.70739E+00_JPRB, 0.74238E+00_JPRB, 0.76855E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.82831E+00_JPRB, 0.83960E+00_JPRB, 0.84174E+00_JPRB, 0.82869E+00_JPRB, 0.80280E+00_JPRB, 0.76408E+00_JPRB, 0.68541E+00_JPRB ,&
&  0.68178E+00_JPRB, 0.63593E+00_JPRB, 0.60694E+00_JPRB, 0.58604E+00_JPRB, 0.57974E+00_JPRB, 0.56952E+00_JPRB, 0.56592E+00_JPRB ,&
&  0.56647E+00_JPRB, 0.57975E+00_JPRB, 0.58777E+00_JPRB, 0.58187E+00_JPRB, 0.58561E+00_JPRB, 0.59004E+00_JPRB, 0.58922E+00_JPRB ,&
&  0.59351E+00_JPRB, 0.59806E+00_JPRB, 0.59873E+00_JPRB, 0.59269E+00_JPRB, 0.57934E+00_JPRB, 0.57020E+00_JPRB, 0.56029E+00_JPRB ,&
&  0.55385E+00_JPRB, 0.55285E+00_JPRB, 0.56180E+00_JPRB, 0.58222E+00_JPRB, 0.57150E+00_JPRB, 0.55891E+00_JPRB, 0.53442E+00_JPRB ,&
&  0.48983E+00_JPRB, 0.50825E+00_JPRB, 0.57014E+00_JPRB, 0.60007E+00_JPRB, 0.63078E+00_JPRB, 0.66430E+00_JPRB, 0.71202E+00_JPRB ,&
&  0.75244E+00_JPRB, 0.79521E+00_JPRB, 0.84287E+00_JPRB, 0.89906E+00_JPRB, 0.96387E+00_JPRB, 0.10225E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.11035E+01_JPRB, 0.93178E+00_JPRB, 0.92428E+00_JPRB, 0.10711E+01_JPRB, 0.13023E+01_JPRB, 0.11326E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10044E+01_JPRB, 0.98431E+00_JPRB, 0.10412E+01_JPRB, 0.10516E+01_JPRB, 0.10945E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.10351E+01_JPRB, 0.98067E+00_JPRB, 0.93178E+00_JPRB, 0.92569E+00_JPRB, 0.93816E+00_JPRB, 0.95716E+00_JPRB, 0.96531E+00_JPRB ,&
&  0.95942E+00_JPRB, 0.93792E+00_JPRB, 0.88632E+00_JPRB, 0.81421E+00_JPRB, 0.76197E+00_JPRB, 0.73076E+00_JPRB, 0.70638E+00_JPRB ,&
&  0.68482E+00_JPRB, 0.66261E+00_JPRB, 0.64508E+00_JPRB, 0.63555E+00_JPRB, 0.63259E+00_JPRB, 0.63794E+00_JPRB, 0.65908E+00_JPRB ,&
&  0.64347E+00_JPRB, 0.63502E+00_JPRB, 0.62171E+00_JPRB, 0.63850E+00_JPRB, 0.67283E+00_JPRB, 0.70364E+00_JPRB, 0.71599E+00_JPRB ,&
&  0.73842E+00_JPRB, 0.74887E+00_JPRB, 0.74379E+00_JPRB, 0.76660E+00_JPRB, 0.78904E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.85206E+00_JPRB, 0.86868E+00_JPRB, 0.86940E+00_JPRB, 0.86877E+00_JPRB, 0.83836E+00_JPRB, 0.77442E+00_JPRB, 0.71603E+00_JPRB ,&
&  0.77480E+00_JPRB, 0.70260E+00_JPRB, 0.70809E+00_JPRB, 0.61729E+00_JPRB, 0.59006E+00_JPRB, 0.57886E+00_JPRB, 0.55973E+00_JPRB ,&
&  0.53832E+00_JPRB, 0.54713E+00_JPRB, 0.56104E+00_JPRB, 0.56691E+00_JPRB, 0.57292E+00_JPRB, 0.58129E+00_JPRB, 0.58049E+00_JPRB ,&
&  0.58547E+00_JPRB, 0.59585E+00_JPRB, 0.60862E+00_JPRB, 0.59993E+00_JPRB, 0.58587E+00_JPRB, 0.57991E+00_JPRB, 0.57141E+00_JPRB ,&
&  0.55853E+00_JPRB, 0.55304E+00_JPRB, 0.56141E+00_JPRB, 0.57849E+00_JPRB, 0.58493E+00_JPRB, 0.56607E+00_JPRB, 0.55372E+00_JPRB ,&
&  0.55115E+00_JPRB, 0.55863E+00_JPRB, 0.60619E+00_JPRB, 0.63747E+00_JPRB, 0.67756E+00_JPRB, 0.72678E+00_JPRB, 0.77525E+00_JPRB ,&
&  0.83039E+00_JPRB, 0.88935E+00_JPRB, 0.95403E+00_JPRB, 0.10313E+01_JPRB, 0.11040E+01_JPRB, 0.11771E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.10227E+01_JPRB, 0.12368E+01_JPRB, 0.13426E+01_JPRB, 0.15413E+01_JPRB, 0.15256E+01_JPRB, 0.14400E+01_JPRB ,&
&  0.14958E+01_JPRB, 0.13313E+01_JPRB, 0.12273E+01_JPRB, 0.11470E+01_JPRB, 0.11419E+01_JPRB, 0.12148E+01_JPRB, 0.12801E+01_JPRB ,&
&  0.13653E+01_JPRB, 0.13257E+01_JPRB, 0.12466E+01_JPRB, 0.12269E+01_JPRB, 0.12232E+01_JPRB, 0.12163E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11288E+01_JPRB, 0.10628E+01_JPRB, 0.99535E+00_JPRB, 0.90873E+00_JPRB, 0.86393E+00_JPRB, 0.82413E+00_JPRB, 0.77993E+00_JPRB ,&
&  0.75321E+00_JPRB, 0.70686E+00_JPRB, 0.66121E+00_JPRB, 0.63325E+00_JPRB, 0.61317E+00_JPRB, 0.60889E+00_JPRB, 0.62576E+00_JPRB ,&
&  0.63563E+00_JPRB, 0.63692E+00_JPRB, 0.68195E+00_JPRB, 0.71973E+00_JPRB, 0.74190E+00_JPRB, 0.76346E+00_JPRB, 0.81072E+00_JPRB ,&
&  0.77114E+00_JPRB, 0.78326E+00_JPRB, 0.82815E+00_JPRB, 0.83630E+00_JPRB, 0.84217E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.89601E+00_JPRB, 0.91951E+00_JPRB, 0.91061E+00_JPRB, 0.90820E+00_JPRB, 0.85532E+00_JPRB, 0.83032E+00_JPRB, 0.75837E+00_JPRB ,&
&  0.77710E+00_JPRB, 0.74954E+00_JPRB, 0.70148E+00_JPRB, 0.68550E+00_JPRB, 0.69276E+00_JPRB, 0.62175E+00_JPRB, 0.56445E+00_JPRB ,&
&  0.53164E+00_JPRB, 0.52384E+00_JPRB, 0.54163E+00_JPRB, 0.55446E+00_JPRB, 0.55895E+00_JPRB, 0.56549E+00_JPRB, 0.56016E+00_JPRB ,&
&  0.57943E+00_JPRB, 0.58991E+00_JPRB, 0.60249E+00_JPRB, 0.60418E+00_JPRB, 0.59803E+00_JPRB, 0.59754E+00_JPRB, 0.59961E+00_JPRB ,&
&  0.58525E+00_JPRB, 0.57994E+00_JPRB, 0.56758E+00_JPRB, 0.56359E+00_JPRB, 0.60561E+00_JPRB, 0.59677E+00_JPRB, 0.58504E+00_JPRB ,&
&  0.58990E+00_JPRB, 0.61006E+00_JPRB, 0.64082E+00_JPRB, 0.68364E+00_JPRB, 0.73333E+00_JPRB, 0.78403E+00_JPRB, 0.83864E+00_JPRB ,&
&  0.88913E+00_JPRB, 0.95482E+00_JPRB, 0.10051E+01_JPRB, 0.11011E+01_JPRB, 0.11716E+01_JPRB, 0.11213E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.15551E+01_JPRB, 0.17034E+01_JPRB, 0.16039E+01_JPRB, 0.18154E+01_JPRB, 0.17685E+01_JPRB, 0.16218E+01_JPRB ,&
&  0.16563E+01_JPRB, 0.15428E+01_JPRB, 0.15161E+01_JPRB, 0.15108E+01_JPRB, 0.15921E+01_JPRB, 0.15471E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.16079E+01_JPRB, 0.18460E+01_JPRB, 0.17700E+01_JPRB, 0.18069E+01_JPRB, 0.17621E+01_JPRB, 0.17011E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.14586E+01_JPRB, 0.13308E+01_JPRB, 0.12110E+01_JPRB, 0.11145E+01_JPRB, 0.10037E+01_JPRB, 0.92023E+00_JPRB, 0.85450E+00_JPRB ,&
&  0.78521E+00_JPRB, 0.70036E+00_JPRB, 0.66466E+00_JPRB, 0.63996E+00_JPRB, 0.62579E+00_JPRB, 0.59883E+00_JPRB, 0.60090E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.68190E+00_JPRB, 0.76409E+00_JPRB, 0.80845E+00_JPRB, 0.86837E+00_JPRB, 0.92906E+00_JPRB, 0.86392E+00_JPRB ,&
&  0.80738E+00_JPRB, 0.87805E+00_JPRB, 0.89181E+00_JPRB, 0.89549E+00_JPRB, 0.89231E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.10024E+01_JPRB, 0.10134E+01_JPRB, 0.99335E+00_JPRB, 0.96139E+00_JPRB, 0.88737E+00_JPRB, 0.84956E+00_JPRB, 0.79677E+00_JPRB ,&
&  0.79576E+00_JPRB, 0.76530E+00_JPRB, 0.71914E+00_JPRB, 0.70178E+00_JPRB, 0.70184E+00_JPRB, 0.68524E+00_JPRB, 0.61590E+00_JPRB ,&
&  0.56388E+00_JPRB, 0.54707E+00_JPRB, 0.53333E+00_JPRB, 0.54293E+00_JPRB, 0.55827E+00_JPRB, 0.55924E+00_JPRB, 0.57032E+00_JPRB ,&
&  0.59315E+00_JPRB, 0.60095E+00_JPRB, 0.60149E+00_JPRB, 0.60095E+00_JPRB, 0.60207E+00_JPRB, 0.61158E+00_JPRB, 0.61539E+00_JPRB ,&
&  0.59811E+00_JPRB, 0.58315E+00_JPRB, 0.56867E+00_JPRB, 0.56910E+00_JPRB, 0.61293E+00_JPRB, 0.63010E+00_JPRB, 0.62755E+00_JPRB ,&
&  0.63298E+00_JPRB, 0.64550E+00_JPRB, 0.66581E+00_JPRB, 0.70749E+00_JPRB, 0.76268E+00_JPRB, 0.82491E+00_JPRB, 0.89111E+00_JPRB ,&
&  0.95764E+00_JPRB, 0.10319E+01_JPRB, 0.10929E+01_JPRB, 0.11660E+01_JPRB, 0.11477E+01_JPRB, 0.12145E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.16564E+01_JPRB, 0.17601E+01_JPRB, 0.19803E+01_JPRB, 0.21589E+01_JPRB, 0.20592E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.19042E+01_JPRB, 0.18488E+01_JPRB, 0.23185E+01_JPRB, 0.21339E+01_JPRB, 0.19644E+01_JPRB, 0.19287E+01_JPRB, 0.18689E+01_JPRB ,&
&  0.19353E+01_JPRB, 0.20778E+01_JPRB, 0.21653E+01_JPRB, 0.22358E+01_JPRB, 0.22480E+01_JPRB, 0.22491E+01_JPRB, 0.21159E+01_JPRB ,&
&  0.20201E+01_JPRB, 0.18384E+01_JPRB, 0.14612E+01_JPRB, 0.11808E+01_JPRB, 0.97086E+00_JPRB, 0.95553E+00_JPRB, 0.88586E+00_JPRB ,&
&  0.79606E+00_JPRB, 0.70420E+00_JPRB, 0.68628E+00_JPRB, 0.66330E+00_JPRB, 0.65753E+00_JPRB, 0.64229E+00_JPRB, 0.62217E+00_JPRB ,&
&  0.64398E+00_JPRB, 0.72473E+00_JPRB, 0.78039E+00_JPRB, 0.87677E+00_JPRB, 0.99141E+00_JPRB, 0.10974E+01_JPRB, 0.94187E+00_JPRB ,&
&  0.95046E+00_JPRB, 0.10364E+01_JPRB, 0.10543E+01_JPRB, 0.10502E+01_JPRB, 0.10232E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.12044E+01_JPRB, 0.11678E+01_JPRB, 0.11150E+01_JPRB, 0.10328E+01_JPRB, 0.93180E+00_JPRB, 0.86546E+00_JPRB, 0.80721E+00_JPRB ,&
&  0.79634E+00_JPRB, 0.76736E+00_JPRB, 0.72829E+00_JPRB, 0.71459E+00_JPRB, 0.71009E+00_JPRB, 0.70323E+00_JPRB, 0.69586E+00_JPRB ,&
&  0.61629E+00_JPRB, 0.59653E+00_JPRB, 0.56318E+00_JPRB, 0.56739E+00_JPRB, 0.61547E+00_JPRB, 0.58456E+00_JPRB, 0.59206E+00_JPRB ,&
&  0.59983E+00_JPRB, 0.61335E+00_JPRB, 0.60637E+00_JPRB, 0.60673E+00_JPRB, 0.61994E+00_JPRB, 0.63557E+00_JPRB, 0.63885E+00_JPRB ,&
&  0.62051E+00_JPRB, 0.59912E+00_JPRB, 0.59282E+00_JPRB, 0.59785E+00_JPRB, 0.64079E+00_JPRB, 0.67439E+00_JPRB, 0.69295E+00_JPRB ,&
&  0.70561E+00_JPRB, 0.71468E+00_JPRB, 0.72398E+00_JPRB, 0.75283E+00_JPRB, 0.79981E+00_JPRB, 0.85744E+00_JPRB, 0.94198E+00_JPRB ,&
&  0.10312E+01_JPRB, 0.11142E+01_JPRB, 0.12012E+01_JPRB, 0.12848E+01_JPRB, 0.12881E+01_JPRB, 0.14215E+01_JPRB, 0.15719E+01_JPRB ,&
&  0.17142E+01_JPRB, 0.16724E+01_JPRB, 0.16959E+01_JPRB, 0.21704E+01_JPRB, 0.25310E+01_JPRB, 0.25378E+01_JPRB, 0.23870E+01_JPRB ,&
&  0.22319E+01_JPRB, 0.21164E+01_JPRB, 0.22377E+01_JPRB, 0.22215E+01_JPRB, 0.20694E+01_JPRB, 0.21450E+01_JPRB, 0.21516E+01_JPRB ,&
&  0.22078E+01_JPRB, 0.23494E+01_JPRB, 0.25264E+01_JPRB, 0.26503E+01_JPRB, 0.25841E+01_JPRB, 0.26244E+01_JPRB, 0.25400E+01_JPRB ,&
&  0.24301E+01_JPRB, 0.20586E+01_JPRB, 0.14815E+01_JPRB, 0.11805E+01_JPRB, 0.88541E+00_JPRB, 0.84440E+00_JPRB, 0.95928E+00_JPRB ,&
&  0.92657E+00_JPRB, 0.87153E+00_JPRB, 0.80599E+00_JPRB, 0.75990E+00_JPRB, 0.73156E+00_JPRB, 0.75104E+00_JPRB, 0.74960E+00_JPRB ,&
&  0.74657E+00_JPRB, 0.84073E+00_JPRB, 0.10050E+01_JPRB, 0.11941E+01_JPRB, 0.13091E+01_JPRB, 0.13549E+01_JPRB, 0.12425E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.12996E+01_JPRB, 0.12968E+01_JPRB, 0.13008E+01_JPRB, 0.12667E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.14113E+01_JPRB, 0.13445E+01_JPRB, 0.12386E+01_JPRB, 0.11031E+01_JPRB, 0.95891E+00_JPRB, 0.86508E+00_JPRB, 0.82245E+00_JPRB ,&
&  0.80968E+00_JPRB, 0.78358E+00_JPRB, 0.74438E+00_JPRB, 0.72617E+00_JPRB, 0.71875E+00_JPRB, 0.70981E+00_JPRB, 0.71267E+00_JPRB ,&
&  0.73504E+00_JPRB, 0.69496E+00_JPRB, 0.62159E+00_JPRB, 0.57465E+00_JPRB, 0.59751E+00_JPRB, 0.59441E+00_JPRB, 0.60461E+00_JPRB ,&
&  0.60890E+00_JPRB, 0.63326E+00_JPRB, 0.61549E+00_JPRB, 0.62563E+00_JPRB, 0.66259E+00_JPRB, 0.68925E+00_JPRB, 0.70393E+00_JPRB ,&
&  0.70046E+00_JPRB, 0.68697E+00_JPRB, 0.68721E+00_JPRB, 0.71342E+00_JPRB, 0.76296E+00_JPRB, 0.74890E+00_JPRB, 0.78123E+00_JPRB ,&
&  0.78952E+00_JPRB, 0.80241E+00_JPRB, 0.80096E+00_JPRB, 0.83184E+00_JPRB, 0.89226E+00_JPRB, 0.99095E+00_JPRB, 0.10664E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.11918E+01_JPRB, 0.12261E+01_JPRB, 0.13178E+01_JPRB, 0.14494E+01_JPRB, 0.16152E+01_JPRB, 0.16023E+01_JPRB ,&
&  0.16481E+01_JPRB, 0.16039E+01_JPRB, 0.15590E+01_JPRB, 0.19792E+01_JPRB, 0.24374E+01_JPRB, 0.25811E+01_JPRB, 0.26065E+01_JPRB ,&
&  0.25745E+01_JPRB, 0.25147E+01_JPRB, 0.26177E+01_JPRB, 0.25332E+01_JPRB, 0.24667E+01_JPRB, 0.25146E+01_JPRB, 0.26028E+01_JPRB ,&
&  0.26031E+01_JPRB, 0.27354E+01_JPRB, 0.29280E+01_JPRB, 0.29393E+01_JPRB, 0.29023E+01_JPRB, 0.27918E+01_JPRB, 0.28004E+01_JPRB ,&
&  0.27238E+01_JPRB, 0.16707E+01_JPRB, 0.13057E+01_JPRB, 0.13455E+01_JPRB, 0.10104E+01_JPRB, 0.87350E+00_JPRB, 0.91716E+00_JPRB ,&
&  0.87828E+00_JPRB, 0.86467E+00_JPRB, 0.89438E+00_JPRB, 0.89302E+00_JPRB, 0.89076E+00_JPRB, 0.97091E+00_JPRB, 0.10359E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.11775E+01_JPRB, 0.14595E+01_JPRB, 0.16417E+01_JPRB, 0.16846E+01_JPRB, 0.16734E+01_JPRB, 0.16359E+01_JPRB ,&
&  0.15876E+01_JPRB, 0.15397E+01_JPRB, 0.15084E+01_JPRB, 0.14948E+01_JPRB, 0.14703E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.16185E+01_JPRB, 0.15351E+01_JPRB, 0.13923E+01_JPRB, 0.12155E+01_JPRB, 0.10276E+01_JPRB, 0.90985E+00_JPRB, 0.88408E+00_JPRB ,&
&  0.86793E+00_JPRB, 0.84660E+00_JPRB, 0.80757E+00_JPRB, 0.77741E+00_JPRB, 0.76319E+00_JPRB, 0.74938E+00_JPRB, 0.74849E+00_JPRB ,&
&  0.74731E+00_JPRB, 0.74112E+00_JPRB, 0.64298E+00_JPRB, 0.57278E+00_JPRB, 0.53491E+00_JPRB, 0.55574E+00_JPRB, 0.57780E+00_JPRB ,&
&  0.60926E+00_JPRB, 0.64266E+00_JPRB, 0.65303E+00_JPRB, 0.71661E+00_JPRB, 0.78004E+00_JPRB, 0.81230E+00_JPRB, 0.85829E+00_JPRB ,&
&  0.86393E+00_JPRB, 0.84889E+00_JPRB, 0.83181E+00_JPRB, 0.87951E+00_JPRB, 0.88680E+00_JPRB, 0.84290E+00_JPRB, 0.89652E+00_JPRB ,&
&  0.91473E+00_JPRB, 0.93155E+00_JPRB, 0.94539E+00_JPRB, 0.99248E+00_JPRB, 0.10539E+01_JPRB, 0.10842E+01_JPRB, 0.10733E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.11576E+01_JPRB, 0.12424E+01_JPRB, 0.13243E+01_JPRB, 0.14268E+01_JPRB, 0.15315E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.15658E+01_JPRB, 0.14110E+01_JPRB, 0.12695E+01_JPRB, 0.16137E+01_JPRB, 0.23105E+01_JPRB, 0.26434E+01_JPRB, 0.25675E+01_JPRB ,&
&  0.27891E+01_JPRB, 0.28788E+01_JPRB, 0.30034E+01_JPRB, 0.29569E+01_JPRB, 0.28667E+01_JPRB, 0.28838E+01_JPRB, 0.30492E+01_JPRB ,&
&  0.31203E+01_JPRB, 0.32951E+01_JPRB, 0.33926E+01_JPRB, 0.33759E+01_JPRB, 0.33098E+01_JPRB, 0.33361E+01_JPRB, 0.33744E+01_JPRB ,&
&  0.30872E+01_JPRB, 0.25854E+01_JPRB, 0.18014E+01_JPRB, 0.17072E+01_JPRB, 0.11966E+01_JPRB, 0.97742E+00_JPRB, 0.97729E+00_JPRB ,&
&  0.89304E+00_JPRB, 0.91435E+00_JPRB, 0.98152E+00_JPRB, 0.10174E+01_JPRB, 0.11962E+01_JPRB, 0.13728E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.17501E+01_JPRB, 0.17132E+01_JPRB, 0.20015E+01_JPRB, 0.20501E+01_JPRB, 0.20488E+01_JPRB, 0.20547E+01_JPRB, 0.20432E+01_JPRB ,&
&  0.19750E+01_JPRB, 0.18865E+01_JPRB, 0.18127E+01_JPRB, 0.17406E+01_JPRB, 0.16805E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.17241E+01_JPRB, 0.16152E+01_JPRB, 0.14412E+01_JPRB, 0.12573E+01_JPRB, 0.10724E+01_JPRB, 0.94226E+00_JPRB, 0.92426E+00_JPRB ,&
&  0.95720E+00_JPRB, 0.95495E+00_JPRB, 0.93231E+00_JPRB, 0.88650E+00_JPRB, 0.85549E+00_JPRB, 0.83359E+00_JPRB, 0.81058E+00_JPRB ,&
&  0.78862E+00_JPRB, 0.71841E+00_JPRB, 0.60319E+00_JPRB, 0.53911E+00_JPRB, 0.51930E+00_JPRB, 0.51590E+00_JPRB, 0.55176E+00_JPRB ,&
&  0.60263E+00_JPRB, 0.69139E+00_JPRB, 0.77801E+00_JPRB, 0.88046E+00_JPRB, 0.10090E+01_JPRB, 0.11288E+01_JPRB, 0.11599E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.10207E+01_JPRB, 0.10434E+01_JPRB, 0.10261E+01_JPRB, 0.10534E+01_JPRB, 0.10433E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.10303E+01_JPRB, 0.10461E+01_JPRB, 0.10654E+01_JPRB, 0.10765E+01_JPRB, 0.10835E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.12326E+01_JPRB, 0.12972E+01_JPRB, 0.13528E+01_JPRB, 0.14406E+01_JPRB, 0.15549E+01_JPRB, 0.15981E+01_JPRB ,&
&  0.15498E+01_JPRB, 0.15078E+01_JPRB, 0.17133E+01_JPRB, 0.19469E+01_JPRB, 0.22286E+01_JPRB, 0.26255E+01_JPRB, 0.28481E+01_JPRB ,&
&  0.29891E+01_JPRB, 0.29170E+01_JPRB, 0.29661E+01_JPRB, 0.28095E+01_JPRB, 0.31949E+01_JPRB, 0.32623E+01_JPRB, 0.34289E+01_JPRB ,&
&  0.35965E+01_JPRB, 0.38624E+01_JPRB, 0.37685E+01_JPRB, 0.36749E+01_JPRB, 0.37774E+01_JPRB, 0.40383E+01_JPRB, 0.36370E+01_JPRB ,&
&  0.30649E+01_JPRB, 0.33048E+01_JPRB, 0.28957E+01_JPRB, 0.23720E+01_JPRB, 0.17957E+01_JPRB, 0.15283E+01_JPRB, 0.13063E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.10860E+01_JPRB, 0.11297E+01_JPRB, 0.13607E+01_JPRB, 0.17428E+01_JPRB, 0.20661E+01_JPRB, 0.22633E+01_JPRB ,&
&  0.23142E+01_JPRB, 0.27721E+01_JPRB, 0.27275E+01_JPRB, 0.25995E+01_JPRB, 0.25438E+01_JPRB, 0.25226E+01_JPRB, 0.24467E+01_JPRB ,&
&  0.22731E+01_JPRB, 0.21324E+01_JPRB, 0.20162E+01_JPRB, 0.19123E+01_JPRB, 0.18099E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.17731E+01_JPRB, 0.16735E+01_JPRB, 0.15003E+01_JPRB, 0.13022E+01_JPRB, 0.11260E+01_JPRB, 0.10036E+01_JPRB, 0.97856E+00_JPRB ,&
&  0.10019E+01_JPRB, 0.10237E+01_JPRB, 0.10359E+01_JPRB, 0.10333E+01_JPRB, 0.10141E+01_JPRB, 0.97219E+00_JPRB, 0.92191E+00_JPRB ,&
&  0.88133E+00_JPRB, 0.72818E+00_JPRB, 0.59964E+00_JPRB, 0.55679E+00_JPRB, 0.52481E+00_JPRB, 0.51087E+00_JPRB, 0.53411E+00_JPRB ,&
&  0.61989E+00_JPRB, 0.81444E+00_JPRB, 0.10257E+01_JPRB, 0.12467E+01_JPRB, 0.14285E+01_JPRB, 0.15338E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.13207E+01_JPRB, 0.12974E+01_JPRB, 0.12393E+01_JPRB, 0.12131E+01_JPRB, 0.11697E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.10972E+01_JPRB, 0.11280E+01_JPRB, 0.11519E+01_JPRB, 0.11721E+01_JPRB, 0.11922E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.12483E+01_JPRB, 0.12964E+01_JPRB, 0.13381E+01_JPRB, 0.14394E+01_JPRB, 0.12974E+01_JPRB, 0.13753E+01_JPRB, 0.15371E+01_JPRB ,&
&  0.18657E+01_JPRB, 0.19132E+01_JPRB, 0.18588E+01_JPRB, 0.20377E+01_JPRB, 0.19461E+01_JPRB, 0.21565E+01_JPRB, 0.26722E+01_JPRB ,&
&  0.28522E+01_JPRB, 0.28775E+01_JPRB, 0.30514E+01_JPRB, 0.26718E+01_JPRB, 0.26664E+01_JPRB, 0.38470E+01_JPRB, 0.41894E+01_JPRB ,&
&  0.42927E+01_JPRB, 0.43414E+01_JPRB, 0.41836E+01_JPRB, 0.42554E+01_JPRB, 0.37502E+01_JPRB, 0.33053E+01_JPRB, 0.25415E+01_JPRB ,&
&  0.27221E+01_JPRB, 0.26374E+01_JPRB, 0.29110E+01_JPRB, 0.26402E+01_JPRB, 0.24455E+01_JPRB, 0.21498E+01_JPRB, 0.17499E+01_JPRB ,&
&  0.15086E+01_JPRB, 0.13989E+01_JPRB, 0.14710E+01_JPRB, 0.19968E+01_JPRB, 0.26602E+01_JPRB, 0.27186E+01_JPRB, 0.32563E+01_JPRB ,&
&  0.35280E+01_JPRB, 0.35802E+01_JPRB, 0.31203E+01_JPRB, 0.30290E+01_JPRB, 0.28484E+01_JPRB, 0.26763E+01_JPRB, 0.24765E+01_JPRB ,&
&  0.23136E+01_JPRB, 0.21895E+01_JPRB, 0.20730E+01_JPRB, 0.19573E+01_JPRB, 0.18504E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.18119E+01_JPRB, 0.17248E+01_JPRB, 0.15555E+01_JPRB, 0.13554E+01_JPRB, 0.11848E+01_JPRB, 0.10779E+01_JPRB, 0.10579E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.11049E+01_JPRB, 0.11146E+01_JPRB, 0.11062E+01_JPRB, 0.10873E+01_JPRB, 0.10675E+01_JPRB, 0.10374E+01_JPRB ,&
&  0.98271E+00_JPRB, 0.82429E+00_JPRB, 0.64449E+00_JPRB, 0.57447E+00_JPRB, 0.56957E+00_JPRB, 0.54654E+00_JPRB, 0.55808E+00_JPRB ,&
&  0.73251E+00_JPRB, 0.10563E+01_JPRB, 0.14644E+01_JPRB, 0.17740E+01_JPRB, 0.19474E+01_JPRB, 0.18956E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.16812E+01_JPRB, 0.15809E+01_JPRB, 0.14207E+01_JPRB, 0.12899E+01_JPRB, 0.12162E+01_JPRB, 0.11762E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.10921E+01_JPRB, 0.11086E+01_JPRB, 0.11208E+01_JPRB, 0.11481E+01_JPRB, 0.11580E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.12779E+01_JPRB, 0.13672E+01_JPRB, 0.14939E+01_JPRB, 0.14280E+01_JPRB, 0.15120E+01_JPRB, 0.17512E+01_JPRB ,&
&  0.19615E+01_JPRB, 0.20100E+01_JPRB, 0.19892E+01_JPRB, 0.20084E+01_JPRB, 0.21316E+01_JPRB, 0.21800E+01_JPRB, 0.25778E+01_JPRB ,&
&  0.24938E+01_JPRB, 0.23725E+01_JPRB, 0.23708E+01_JPRB, 0.22985E+01_JPRB, 0.29610E+01_JPRB, 0.42503E+01_JPRB, 0.49353E+01_JPRB ,&
&  0.49195E+01_JPRB, 0.48457E+01_JPRB, 0.47633E+01_JPRB, 0.42785E+01_JPRB, 0.34360E+01_JPRB, 0.26440E+01_JPRB, 0.31554E+01_JPRB ,&
&  0.34257E+01_JPRB, 0.35803E+01_JPRB, 0.33998E+01_JPRB, 0.28141E+01_JPRB, 0.24315E+01_JPRB, 0.27005E+01_JPRB, 0.24229E+01_JPRB ,&
&  0.20014E+01_JPRB, 0.18658E+01_JPRB, 0.23948E+01_JPRB, 0.31151E+01_JPRB, 0.34853E+01_JPRB, 0.36049E+01_JPRB, 0.40922E+01_JPRB ,&
&  0.39454E+01_JPRB, 0.37919E+01_JPRB, 0.34846E+01_JPRB, 0.31264E+01_JPRB, 0.28256E+01_JPRB, 0.25836E+01_JPRB, 0.23938E+01_JPRB ,&
&  0.22874E+01_JPRB, 0.22125E+01_JPRB, 0.21108E+01_JPRB, 0.20075E+01_JPRB, 0.18954E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.18527E+01_JPRB, 0.17621E+01_JPRB, 0.15845E+01_JPRB, 0.13800E+01_JPRB, 0.12427E+01_JPRB, 0.11595E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.11357E+01_JPRB, 0.11358E+01_JPRB, 0.11254E+01_JPRB, 0.11118E+01_JPRB, 0.10960E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.99367E+00_JPRB, 0.75885E+00_JPRB, 0.65853E+00_JPRB, 0.63662E+00_JPRB, 0.64559E+00_JPRB, 0.66319E+00_JPRB ,&
&  0.96361E+00_JPRB, 0.15359E+01_JPRB, 0.20243E+01_JPRB, 0.23145E+01_JPRB, 0.23631E+01_JPRB, 0.19956E+01_JPRB, 0.18634E+01_JPRB ,&
&  0.18520E+01_JPRB, 0.17455E+01_JPRB, 0.15679E+01_JPRB, 0.14051E+01_JPRB, 0.12711E+01_JPRB, 0.12019E+01_JPRB, 0.11652E+01_JPRB ,&
&  0.11327E+01_JPRB, 0.11192E+01_JPRB, 0.11245E+01_JPRB, 0.11522E+01_JPRB, 0.11676E+01_JPRB, 0.11783E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.12403E+01_JPRB, 0.12999E+01_JPRB, 0.13776E+01_JPRB, 0.15563E+01_JPRB, 0.16162E+01_JPRB, 0.18075E+01_JPRB, 0.22087E+01_JPRB ,&
&  0.22502E+01_JPRB, 0.23295E+01_JPRB, 0.22181E+01_JPRB, 0.21623E+01_JPRB, 0.21762E+01_JPRB, 0.22237E+01_JPRB, 0.25834E+01_JPRB ,&
&  0.25525E+01_JPRB, 0.25719E+01_JPRB, 0.28139E+01_JPRB, 0.30090E+01_JPRB, 0.32809E+01_JPRB, 0.39839E+01_JPRB, 0.56995E+01_JPRB ,&
&  0.50061E+01_JPRB, 0.50110E+01_JPRB, 0.49869E+01_JPRB, 0.46665E+01_JPRB, 0.25378E+01_JPRB, 0.19490E+01_JPRB, 0.29940E+01_JPRB ,&
&  0.31678E+01_JPRB, 0.19742E+01_JPRB, 0.14475E+01_JPRB, 0.19623E+01_JPRB, 0.15460E+01_JPRB, 0.18457E+01_JPRB, 0.25559E+01_JPRB ,&
&  0.26460E+01_JPRB, 0.24387E+01_JPRB, 0.30821E+01_JPRB, 0.34526E+01_JPRB, 0.35719E+01_JPRB, 0.44177E+01_JPRB, 0.44732E+01_JPRB ,&
&  0.42763E+01_JPRB, 0.35638E+01_JPRB, 0.34591E+01_JPRB, 0.30671E+01_JPRB, 0.27711E+01_JPRB, 0.25947E+01_JPRB, 0.24572E+01_JPRB ,&
&  0.22973E+01_JPRB, 0.21450E+01_JPRB, 0.20431E+01_JPRB, 0.19904E+01_JPRB, 0.19229E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.18542E+01_JPRB, 0.17611E+01_JPRB, 0.15766E+01_JPRB, 0.13774E+01_JPRB, 0.12678E+01_JPRB, 0.11942E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11485E+01_JPRB, 0.11432E+01_JPRB, 0.11340E+01_JPRB, 0.11251E+01_JPRB, 0.11216E+01_JPRB, 0.11189E+01_JPRB ,&
&  0.10916E+01_JPRB, 0.10176E+01_JPRB, 0.88768E+00_JPRB, 0.79606E+00_JPRB, 0.77626E+00_JPRB, 0.78248E+00_JPRB, 0.88001E+00_JPRB ,&
&  0.12289E+01_JPRB, 0.19775E+01_JPRB, 0.25123E+01_JPRB, 0.26411E+01_JPRB, 0.23227E+01_JPRB, 0.20135E+01_JPRB, 0.20423E+01_JPRB ,&
&  0.20126E+01_JPRB, 0.18647E+01_JPRB, 0.16530E+01_JPRB, 0.15030E+01_JPRB, 0.13787E+01_JPRB, 0.12977E+01_JPRB, 0.12550E+01_JPRB ,&
&  0.12062E+01_JPRB, 0.11817E+01_JPRB, 0.11946E+01_JPRB, 0.12029E+01_JPRB, 0.12081E+01_JPRB, 0.12191E+01_JPRB, 0.12383E+01_JPRB ,&
&  0.12844E+01_JPRB, 0.13575E+01_JPRB, 0.14691E+01_JPRB, 0.16601E+01_JPRB, 0.20732E+01_JPRB, 0.23632E+01_JPRB, 0.22987E+01_JPRB ,&
&  0.21595E+01_JPRB, 0.22081E+01_JPRB, 0.24442E+01_JPRB, 0.22444E+01_JPRB, 0.21602E+01_JPRB, 0.22162E+01_JPRB, 0.25090E+01_JPRB ,&
&  0.29763E+01_JPRB, 0.32372E+01_JPRB, 0.30697E+01_JPRB, 0.32612E+01_JPRB, 0.36042E+01_JPRB, 0.34756E+01_JPRB, 0.48409E+01_JPRB ,&
&  0.53186E+01_JPRB, 0.46327E+01_JPRB, 0.36245E+01_JPRB, 0.28059E+01_JPRB, 0.29070E+01_JPRB, 0.22949E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.11341E+01_JPRB, 0.10332E+01_JPRB, 0.11357E+01_JPRB, 0.11385E+01_JPRB, 0.15645E+01_JPRB, 0.24235E+01_JPRB ,&
&  0.31557E+01_JPRB, 0.32531E+01_JPRB, 0.35489E+01_JPRB, 0.36137E+01_JPRB, 0.35597E+01_JPRB, 0.44392E+01_JPRB, 0.45707E+01_JPRB ,&
&  0.37062E+01_JPRB, 0.32463E+01_JPRB, 0.31668E+01_JPRB, 0.28370E+01_JPRB, 0.26786E+01_JPRB, 0.25514E+01_JPRB, 0.23770E+01_JPRB ,&
&  0.21638E+01_JPRB, 0.19737E+01_JPRB, 0.18945E+01_JPRB, 0.18956E+01_JPRB, 0.19074E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.18366E+01_JPRB, 0.17397E+01_JPRB, 0.15776E+01_JPRB, 0.14074E+01_JPRB, 0.13214E+01_JPRB, 0.12626E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.11806E+01_JPRB, 0.11735E+01_JPRB, 0.11725E+01_JPRB, 0.11711E+01_JPRB, 0.11706E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.10581E+01_JPRB, 0.98463E+00_JPRB, 0.92185E+00_JPRB, 0.93602E+00_JPRB, 0.94840E+00_JPRB, 0.11128E+01_JPRB ,&
&  0.15969E+01_JPRB, 0.24271E+01_JPRB, 0.28998E+01_JPRB, 0.28325E+01_JPRB, 0.25358E+01_JPRB, 0.23071E+01_JPRB, 0.21942E+01_JPRB ,&
&  0.20373E+01_JPRB, 0.19015E+01_JPRB, 0.17334E+01_JPRB, 0.15949E+01_JPRB, 0.14840E+01_JPRB, 0.14008E+01_JPRB, 0.13378E+01_JPRB ,&
&  0.12844E+01_JPRB, 0.12817E+01_JPRB, 0.12664E+01_JPRB, 0.12642E+01_JPRB, 0.12603E+01_JPRB, 0.12621E+01_JPRB, 0.12856E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.14677E+01_JPRB, 0.16781E+01_JPRB, 0.20297E+01_JPRB, 0.21902E+01_JPRB, 0.20052E+01_JPRB, 0.21545E+01_JPRB ,&
&  0.22365E+01_JPRB, 0.23919E+01_JPRB, 0.23211E+01_JPRB, 0.23810E+01_JPRB, 0.22305E+01_JPRB, 0.21686E+01_JPRB, 0.24874E+01_JPRB ,&
&  0.30462E+01_JPRB, 0.37086E+01_JPRB, 0.32072E+01_JPRB, 0.30911E+01_JPRB, 0.40331E+01_JPRB, 0.39347E+01_JPRB, 0.44541E+01_JPRB ,&
&  0.50513E+01_JPRB, 0.43177E+01_JPRB, 0.36317E+01_JPRB, 0.31507E+01_JPRB, 0.42237E+01_JPRB, 0.43431E+01_JPRB, 0.21529E+01_JPRB ,&
&  0.15109E+01_JPRB, 0.12365E+01_JPRB, 0.10962E+01_JPRB, 0.10430E+01_JPRB, 0.10665E+01_JPRB, 0.15572E+01_JPRB, 0.28374E+01_JPRB ,&
&  0.32067E+01_JPRB, 0.38172E+01_JPRB, 0.40409E+01_JPRB, 0.39893E+01_JPRB, 0.38612E+01_JPRB, 0.39460E+01_JPRB, 0.35977E+01_JPRB ,&
&  0.35901E+01_JPRB, 0.32937E+01_JPRB, 0.29310E+01_JPRB, 0.26354E+01_JPRB, 0.24117E+01_JPRB, 0.21603E+01_JPRB, 0.19470E+01_JPRB ,&
&  0.18028E+01_JPRB, 0.17570E+01_JPRB, 0.17539E+01_JPRB, 0.18035E+01_JPRB, 0.18792E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.18155E+01_JPRB, 0.17632E+01_JPRB, 0.16220E+01_JPRB, 0.14687E+01_JPRB, 0.13858E+01_JPRB, 0.13292E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.12526E+01_JPRB, 0.12207E+01_JPRB, 0.11976E+01_JPRB, 0.11868E+01_JPRB, 0.11914E+01_JPRB, 0.11860E+01_JPRB, 0.11746E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.10847E+01_JPRB, 0.10146E+01_JPRB, 0.10557E+01_JPRB, 0.10923E+01_JPRB, 0.11780E+01_JPRB, 0.14290E+01_JPRB ,&
&  0.21167E+01_JPRB, 0.30306E+01_JPRB, 0.32142E+01_JPRB, 0.30576E+01_JPRB, 0.27607E+01_JPRB, 0.24900E+01_JPRB, 0.20885E+01_JPRB ,&
&  0.18413E+01_JPRB, 0.17928E+01_JPRB, 0.16706E+01_JPRB, 0.16225E+01_JPRB, 0.15674E+01_JPRB, 0.14899E+01_JPRB, 0.14188E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.13301E+01_JPRB, 0.13137E+01_JPRB, 0.13069E+01_JPRB, 0.13006E+01_JPRB, 0.13037E+01_JPRB, 0.13678E+01_JPRB ,&
&  0.14892E+01_JPRB, 0.16505E+01_JPRB, 0.19394E+01_JPRB, 0.22138E+01_JPRB, 0.18854E+01_JPRB, 0.20652E+01_JPRB, 0.21229E+01_JPRB ,&
&  0.21392E+01_JPRB, 0.21693E+01_JPRB, 0.20502E+01_JPRB, 0.20856E+01_JPRB, 0.21843E+01_JPRB, 0.21825E+01_JPRB, 0.24206E+01_JPRB ,&
&  0.33221E+01_JPRB, 0.31094E+01_JPRB, 0.28656E+01_JPRB, 0.29922E+01_JPRB, 0.37605E+01_JPRB, 0.58061E+01_JPRB, 0.41228E+01_JPRB ,&
&  0.37836E+01_JPRB, 0.43133E+01_JPRB, 0.38428E+01_JPRB, 0.36752E+01_JPRB, 0.46373E+01_JPRB, 0.58672E+01_JPRB, 0.54719E+01_JPRB ,&
&  0.21960E+01_JPRB, 0.14288E+01_JPRB, 0.13950E+01_JPRB, 0.13712E+01_JPRB, 0.12870E+01_JPRB, 0.16408E+01_JPRB, 0.43524E+01_JPRB ,&
&  0.37651E+01_JPRB, 0.43343E+01_JPRB, 0.43339E+01_JPRB, 0.40132E+01_JPRB, 0.39509E+01_JPRB, 0.38943E+01_JPRB, 0.35761E+01_JPRB ,&
&  0.31709E+01_JPRB, 0.28041E+01_JPRB, 0.24621E+01_JPRB, 0.21229E+01_JPRB, 0.18555E+01_JPRB, 0.16796E+01_JPRB, 0.16044E+01_JPRB ,&
&  0.15877E+01_JPRB, 0.16107E+01_JPRB, 0.16555E+01_JPRB, 0.17249E+01_JPRB, 0.18043E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.17969E+01_JPRB, 0.17781E+01_JPRB, 0.16457E+01_JPRB, 0.15118E+01_JPRB, 0.14296E+01_JPRB, 0.13815E+01_JPRB, 0.13197E+01_JPRB ,&
&  0.13110E+01_JPRB, 0.12573E+01_JPRB, 0.11989E+01_JPRB, 0.11653E+01_JPRB, 0.11684E+01_JPRB, 0.11619E+01_JPRB, 0.11563E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.10957E+01_JPRB, 0.10457E+01_JPRB, 0.10552E+01_JPRB, 0.11740E+01_JPRB, 0.11397E+01_JPRB, 0.14035E+01_JPRB ,&
&  0.25176E+01_JPRB, 0.37877E+01_JPRB, 0.34005E+01_JPRB, 0.31163E+01_JPRB, 0.27634E+01_JPRB, 0.25035E+01_JPRB, 0.19067E+01_JPRB ,&
&  0.17186E+01_JPRB, 0.17006E+01_JPRB, 0.16379E+01_JPRB, 0.16287E+01_JPRB, 0.15905E+01_JPRB, 0.15323E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.13823E+01_JPRB, 0.13434E+01_JPRB, 0.13546E+01_JPRB, 0.13588E+01_JPRB, 0.13529E+01_JPRB, 0.14011E+01_JPRB, 0.15040E+01_JPRB ,&
&  0.16236E+01_JPRB, 0.18226E+01_JPRB, 0.21275E+01_JPRB, 0.21253E+01_JPRB, 0.20317E+01_JPRB, 0.20239E+01_JPRB, 0.20733E+01_JPRB ,&
&  0.20397E+01_JPRB, 0.20538E+01_JPRB, 0.20387E+01_JPRB, 0.19829E+01_JPRB, 0.20711E+01_JPRB, 0.22326E+01_JPRB, 0.24485E+01_JPRB ,&
&  0.29034E+01_JPRB, 0.27303E+01_JPRB, 0.25715E+01_JPRB, 0.29129E+01_JPRB, 0.36958E+01_JPRB, 0.52550E+01_JPRB, 0.54280E+01_JPRB ,&
&  0.48369E+01_JPRB, 0.36393E+01_JPRB, 0.35179E+01_JPRB, 0.44337E+01_JPRB, 0.51169E+01_JPRB, 0.57610E+01_JPRB, 0.79353E+01_JPRB ,&
&  0.78247E+01_JPRB, 0.42339E+01_JPRB, 0.33174E+01_JPRB, 0.41753E+01_JPRB, 0.28069E+01_JPRB, 0.20000E+01_JPRB, 0.30042E+01_JPRB ,&
&  0.36799E+01_JPRB, 0.42025E+01_JPRB, 0.43070E+01_JPRB, 0.41033E+01_JPRB, 0.40849E+01_JPRB, 0.37517E+01_JPRB, 0.30918E+01_JPRB ,&
&  0.25879E+01_JPRB, 0.21485E+01_JPRB, 0.18448E+01_JPRB, 0.16786E+01_JPRB, 0.16120E+01_JPRB, 0.15757E+01_JPRB, 0.15536E+01_JPRB ,&
&  0.15499E+01_JPRB, 0.15708E+01_JPRB, 0.16245E+01_JPRB, 0.17143E+01_JPRB, 0.17864E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.17729E+01_JPRB, 0.17353E+01_JPRB, 0.16219E+01_JPRB, 0.15152E+01_JPRB, 0.14445E+01_JPRB, 0.13877E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.14424E+01_JPRB, 0.12883E+01_JPRB, 0.11836E+01_JPRB, 0.11423E+01_JPRB, 0.11274E+01_JPRB, 0.11280E+01_JPRB, 0.11265E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.10972E+01_JPRB, 0.10816E+01_JPRB, 0.11085E+01_JPRB, 0.12263E+01_JPRB, 0.16187E+01_JPRB, 0.15964E+01_JPRB ,&
&  0.26673E+01_JPRB, 0.44170E+01_JPRB, 0.35344E+01_JPRB, 0.31548E+01_JPRB, 0.28457E+01_JPRB, 0.26710E+01_JPRB, 0.19322E+01_JPRB ,&
&  0.17098E+01_JPRB, 0.16617E+01_JPRB, 0.16198E+01_JPRB, 0.15943E+01_JPRB, 0.15827E+01_JPRB, 0.15405E+01_JPRB, 0.14296E+01_JPRB ,&
&  0.13627E+01_JPRB, 0.13546E+01_JPRB, 0.13871E+01_JPRB, 0.14093E+01_JPRB, 0.14442E+01_JPRB, 0.15371E+01_JPRB, 0.16140E+01_JPRB ,&
&  0.17535E+01_JPRB, 0.20789E+01_JPRB, 0.22818E+01_JPRB, 0.20688E+01_JPRB, 0.20212E+01_JPRB, 0.20438E+01_JPRB, 0.20757E+01_JPRB ,&
&  0.19451E+01_JPRB, 0.18044E+01_JPRB, 0.19157E+01_JPRB, 0.20157E+01_JPRB, 0.20999E+01_JPRB, 0.21726E+01_JPRB, 0.22813E+01_JPRB ,&
&  0.26709E+01_JPRB, 0.28647E+01_JPRB, 0.32450E+01_JPRB, 0.33190E+01_JPRB, 0.38192E+01_JPRB, 0.49423E+01_JPRB, 0.54326E+01_JPRB ,&
&  0.57683E+01_JPRB, 0.49136E+01_JPRB, 0.50215E+01_JPRB, 0.54686E+01_JPRB, 0.55559E+01_JPRB, 0.55446E+01_JPRB, 0.62984E+01_JPRB ,&
&  0.71409E+01_JPRB, 0.84249E+01_JPRB, 0.74100E+01_JPRB, 0.56893E+01_JPRB, 0.41082E+01_JPRB, 0.27951E+01_JPRB, 0.30893E+01_JPRB ,&
&  0.38468E+01_JPRB, 0.39602E+01_JPRB, 0.39637E+01_JPRB, 0.51261E+01_JPRB, 0.39260E+01_JPRB, 0.29941E+01_JPRB, 0.22556E+01_JPRB ,&
&  0.18385E+01_JPRB, 0.16874E+01_JPRB, 0.16500E+01_JPRB, 0.16035E+01_JPRB, 0.15825E+01_JPRB, 0.15624E+01_JPRB, 0.15621E+01_JPRB ,&
&  0.15857E+01_JPRB, 0.16267E+01_JPRB, 0.16846E+01_JPRB, 0.17499E+01_JPRB, 0.17897E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.16954E+01_JPRB, 0.16297E+01_JPRB, 0.15483E+01_JPRB, 0.14713E+01_JPRB, 0.13998E+01_JPRB, 0.13403E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.15372E+01_JPRB, 0.12488E+01_JPRB, 0.11417E+01_JPRB, 0.11120E+01_JPRB, 0.11043E+01_JPRB, 0.10993E+01_JPRB, 0.11083E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.11248E+01_JPRB, 0.11609E+01_JPRB, 0.12534E+01_JPRB, 0.14903E+01_JPRB, 0.20727E+01_JPRB, 0.25775E+01_JPRB ,&
&  0.30076E+01_JPRB, 0.52145E+01_JPRB, 0.36501E+01_JPRB, 0.31731E+01_JPRB, 0.28939E+01_JPRB, 0.27470E+01_JPRB, 0.21577E+01_JPRB ,&
&  0.17698E+01_JPRB, 0.16568E+01_JPRB, 0.16067E+01_JPRB, 0.15563E+01_JPRB, 0.15203E+01_JPRB, 0.14515E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.13938E+01_JPRB, 0.14196E+01_JPRB, 0.14540E+01_JPRB, 0.14932E+01_JPRB, 0.15435E+01_JPRB, 0.16540E+01_JPRB, 0.18003E+01_JPRB ,&
&  0.20718E+01_JPRB, 0.23714E+01_JPRB, 0.22410E+01_JPRB, 0.20912E+01_JPRB, 0.20052E+01_JPRB, 0.20508E+01_JPRB, 0.20759E+01_JPRB ,&
&  0.19710E+01_JPRB, 0.18502E+01_JPRB, 0.19387E+01_JPRB, 0.19730E+01_JPRB, 0.19856E+01_JPRB, 0.21508E+01_JPRB, 0.22264E+01_JPRB ,&
&  0.27081E+01_JPRB, 0.30211E+01_JPRB, 0.42241E+01_JPRB, 0.41696E+01_JPRB, 0.37971E+01_JPRB, 0.47023E+01_JPRB, 0.47771E+01_JPRB ,&
&  0.42504E+01_JPRB, 0.44991E+01_JPRB, 0.50628E+01_JPRB, 0.58141E+01_JPRB, 0.60016E+01_JPRB, 0.60566E+01_JPRB, 0.58322E+01_JPRB ,&
&  0.52283E+01_JPRB, 0.54194E+01_JPRB, 0.44677E+01_JPRB, 0.36636E+01_JPRB, 0.35972E+01_JPRB, 0.30496E+01_JPRB, 0.34754E+01_JPRB ,&
&  0.41679E+01_JPRB, 0.42014E+01_JPRB, 0.44722E+01_JPRB, 0.45011E+01_JPRB, 0.29134E+01_JPRB, 0.21009E+01_JPRB, 0.16924E+01_JPRB ,&
&  0.15934E+01_JPRB, 0.16091E+01_JPRB, 0.16245E+01_JPRB, 0.16048E+01_JPRB, 0.15956E+01_JPRB, 0.15969E+01_JPRB, 0.16200E+01_JPRB ,&
&  0.16654E+01_JPRB, 0.17151E+01_JPRB, 0.17408E+01_JPRB, 0.17473E+01_JPRB, 0.17435E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.16258E+01_JPRB, 0.15768E+01_JPRB, 0.15196E+01_JPRB, 0.14700E+01_JPRB, 0.14479E+01_JPRB, 0.14011E+01_JPRB, 0.14506E+01_JPRB ,&
&  0.16289E+01_JPRB, 0.12010E+01_JPRB, 0.11001E+01_JPRB, 0.10806E+01_JPRB, 0.10849E+01_JPRB, 0.11034E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.11908E+01_JPRB, 0.12411E+01_JPRB, 0.13437E+01_JPRB, 0.15801E+01_JPRB, 0.20674E+01_JPRB, 0.29233E+01_JPRB, 0.36673E+01_JPRB ,&
&  0.44551E+01_JPRB, 0.43876E+01_JPRB, 0.37214E+01_JPRB, 0.29430E+01_JPRB, 0.27487E+01_JPRB, 0.26646E+01_JPRB, 0.25471E+01_JPRB ,&
&  0.20227E+01_JPRB, 0.17924E+01_JPRB, 0.16928E+01_JPRB, 0.15909E+01_JPRB, 0.14755E+01_JPRB, 0.14582E+01_JPRB, 0.14769E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.15536E+01_JPRB, 0.16063E+01_JPRB, 0.16706E+01_JPRB, 0.17898E+01_JPRB, 0.19840E+01_JPRB, 0.22086E+01_JPRB ,&
&  0.24481E+01_JPRB, 0.25331E+01_JPRB, 0.23512E+01_JPRB, 0.21614E+01_JPRB, 0.20587E+01_JPRB, 0.20952E+01_JPRB, 0.20987E+01_JPRB ,&
&  0.20492E+01_JPRB, 0.20417E+01_JPRB, 0.20661E+01_JPRB, 0.20675E+01_JPRB, 0.19952E+01_JPRB, 0.22508E+01_JPRB, 0.24232E+01_JPRB ,&
&  0.29332E+01_JPRB, 0.35558E+01_JPRB, 0.48999E+01_JPRB, 0.40570E+01_JPRB, 0.42654E+01_JPRB, 0.44928E+01_JPRB, 0.45128E+01_JPRB ,&
&  0.41024E+01_JPRB, 0.46398E+01_JPRB, 0.53924E+01_JPRB, 0.58266E+01_JPRB, 0.63709E+01_JPRB, 0.60304E+01_JPRB, 0.61203E+01_JPRB ,&
&  0.45061E+01_JPRB, 0.42009E+01_JPRB, 0.30884E+01_JPRB, 0.32309E+01_JPRB, 0.33063E+01_JPRB, 0.32024E+01_JPRB, 0.33882E+01_JPRB ,&
&  0.39330E+01_JPRB, 0.38876E+01_JPRB, 0.35222E+01_JPRB, 0.28987E+01_JPRB, 0.18732E+01_JPRB, 0.15626E+01_JPRB, 0.14707E+01_JPRB ,&
&  0.14850E+01_JPRB, 0.15910E+01_JPRB, 0.16201E+01_JPRB, 0.16372E+01_JPRB, 0.16308E+01_JPRB, 0.16398E+01_JPRB, 0.16659E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.17241E+01_JPRB, 0.17254E+01_JPRB, 0.16995E+01_JPRB, 0.16692E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.15483E+01_JPRB, 0.15051E+01_JPRB, 0.14622E+01_JPRB, 0.13983E+01_JPRB, 0.12989E+01_JPRB, 0.12347E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.11514E+01_JPRB, 0.11103E+01_JPRB, 0.11185E+01_JPRB, 0.11298E+01_JPRB, 0.11532E+01_JPRB, 0.11983E+01_JPRB, 0.12668E+01_JPRB ,&
&  0.13456E+01_JPRB, 0.14781E+01_JPRB, 0.17387E+01_JPRB, 0.21491E+01_JPRB, 0.26494E+01_JPRB, 0.33534E+01_JPRB, 0.43087E+01_JPRB ,&
&  0.44749E+01_JPRB, 0.29129E+01_JPRB, 0.31266E+01_JPRB, 0.28704E+01_JPRB, 0.27304E+01_JPRB, 0.24756E+01_JPRB, 0.22832E+01_JPRB ,&
&  0.21054E+01_JPRB, 0.19887E+01_JPRB, 0.19225E+01_JPRB, 0.18070E+01_JPRB, 0.16449E+01_JPRB, 0.16982E+01_JPRB, 0.17561E+01_JPRB ,&
&  0.18101E+01_JPRB, 0.18875E+01_JPRB, 0.19838E+01_JPRB, 0.21268E+01_JPRB, 0.23240E+01_JPRB, 0.25130E+01_JPRB, 0.26606E+01_JPRB ,&
&  0.27220E+01_JPRB, 0.27566E+01_JPRB, 0.27010E+01_JPRB, 0.25558E+01_JPRB, 0.23755E+01_JPRB, 0.23380E+01_JPRB, 0.22611E+01_JPRB ,&
&  0.22060E+01_JPRB, 0.21581E+01_JPRB, 0.22358E+01_JPRB, 0.24048E+01_JPRB, 0.25292E+01_JPRB, 0.24619E+01_JPRB, 0.28651E+01_JPRB ,&
&  0.33406E+01_JPRB, 0.40370E+01_JPRB, 0.44387E+01_JPRB, 0.55710E+01_JPRB, 0.38500E+01_JPRB, 0.40703E+01_JPRB, 0.37587E+01_JPRB ,&
&  0.44703E+01_JPRB, 0.47752E+01_JPRB, 0.52621E+01_JPRB, 0.56707E+01_JPRB, 0.64158E+01_JPRB, 0.58560E+01_JPRB, 0.52875E+01_JPRB ,&
&  0.41087E+01_JPRB, 0.31462E+01_JPRB, 0.25904E+01_JPRB, 0.28399E+01_JPRB, 0.31630E+01_JPRB, 0.35837E+01_JPRB, 0.33229E+01_JPRB ,&
&  0.33214E+01_JPRB, 0.29179E+01_JPRB, 0.24261E+01_JPRB, 0.22087E+01_JPRB, 0.16240E+01_JPRB, 0.14711E+01_JPRB, 0.14816E+01_JPRB ,&
&  0.15294E+01_JPRB, 0.16169E+01_JPRB, 0.16364E+01_JPRB, 0.16288E+01_JPRB, 0.16234E+01_JPRB, 0.16238E+01_JPRB, 0.16355E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.16514E+01_JPRB, 0.16475E+01_JPRB, 0.16184E+01_JPRB, 0.15814E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.14628E+01_JPRB, 0.14097E+01_JPRB, 0.13160E+01_JPRB, 0.12120E+01_JPRB, 0.11417E+01_JPRB, 0.11019E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.11256E+01_JPRB, 0.11612E+01_JPRB, 0.12089E+01_JPRB, 0.12559E+01_JPRB, 0.13087E+01_JPRB, 0.14144E+01_JPRB ,&
&  0.15411E+01_JPRB, 0.17588E+01_JPRB, 0.20838E+01_JPRB, 0.24430E+01_JPRB, 0.27718E+01_JPRB, 0.31201E+01_JPRB, 0.34655E+01_JPRB ,&
&  0.38517E+01_JPRB, 0.38705E+01_JPRB, 0.33629E+01_JPRB, 0.23497E+01_JPRB, 0.21381E+01_JPRB, 0.23109E+01_JPRB, 0.22490E+01_JPRB ,&
&  0.21965E+01_JPRB, 0.21062E+01_JPRB, 0.20396E+01_JPRB, 0.20309E+01_JPRB, 0.20577E+01_JPRB, 0.21409E+01_JPRB, 0.22173E+01_JPRB ,&
&  0.23252E+01_JPRB, 0.24355E+01_JPRB, 0.25596E+01_JPRB, 0.26924E+01_JPRB, 0.27905E+01_JPRB, 0.28818E+01_JPRB, 0.29812E+01_JPRB ,&
&  0.30657E+01_JPRB, 0.32060E+01_JPRB, 0.33292E+01_JPRB, 0.33357E+01_JPRB, 0.30890E+01_JPRB, 0.29604E+01_JPRB, 0.27963E+01_JPRB ,&
&  0.27221E+01_JPRB, 0.26697E+01_JPRB, 0.27110E+01_JPRB, 0.29310E+01_JPRB, 0.31010E+01_JPRB, 0.28975E+01_JPRB, 0.30945E+01_JPRB ,&
&  0.38226E+01_JPRB, 0.45746E+01_JPRB, 0.42043E+01_JPRB, 0.53179E+01_JPRB, 0.31807E+01_JPRB, 0.37189E+01_JPRB, 0.41623E+01_JPRB ,&
&  0.40508E+01_JPRB, 0.43827E+01_JPRB, 0.48920E+01_JPRB, 0.52889E+01_JPRB, 0.53811E+01_JPRB, 0.49409E+01_JPRB, 0.44851E+01_JPRB ,&
&  0.36341E+01_JPRB, 0.27263E+01_JPRB, 0.23277E+01_JPRB, 0.24070E+01_JPRB, 0.27900E+01_JPRB, 0.32254E+01_JPRB, 0.25404E+01_JPRB ,&
&  0.19541E+01_JPRB, 0.21554E+01_JPRB, 0.20915E+01_JPRB, 0.21795E+01_JPRB, 0.17173E+01_JPRB, 0.14422E+01_JPRB, 0.15104E+01_JPRB ,&
&  0.15945E+01_JPRB, 0.16046E+01_JPRB, 0.16186E+01_JPRB, 0.15982E+01_JPRB, 0.15940E+01_JPRB, 0.15859E+01_JPRB, 0.15784E+01_JPRB ,&
&  0.15803E+01_JPRB, 0.15883E+01_JPRB, 0.15835E+01_JPRB, 0.15433E+01_JPRB, 0.14986E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.13241E+01_JPRB, 0.12191E+01_JPRB, 0.11122E+01_JPRB, 0.10404E+01_JPRB, 0.10091E+01_JPRB, 0.10049E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.11017E+01_JPRB, 0.11588E+01_JPRB, 0.12319E+01_JPRB, 0.13017E+01_JPRB, 0.13803E+01_JPRB, 0.14946E+01_JPRB ,&
&  0.16322E+01_JPRB, 0.18245E+01_JPRB, 0.20805E+01_JPRB, 0.23644E+01_JPRB, 0.26048E+01_JPRB, 0.28444E+01_JPRB, 0.30809E+01_JPRB ,&
&  0.33164E+01_JPRB, 0.34114E+01_JPRB, 0.33261E+01_JPRB, 0.31704E+01_JPRB, 0.25930E+01_JPRB, 0.27550E+01_JPRB, 0.29150E+01_JPRB ,&
&  0.29937E+01_JPRB, 0.25631E+01_JPRB, 0.23986E+01_JPRB, 0.23929E+01_JPRB, 0.24494E+01_JPRB, 0.25579E+01_JPRB, 0.26520E+01_JPRB ,&
&  0.27479E+01_JPRB, 0.28313E+01_JPRB, 0.29102E+01_JPRB, 0.29752E+01_JPRB, 0.30398E+01_JPRB, 0.31551E+01_JPRB, 0.33496E+01_JPRB ,&
&  0.34566E+01_JPRB, 0.35686E+01_JPRB, 0.36557E+01_JPRB, 0.37324E+01_JPRB, 0.39645E+01_JPRB, 0.41135E+01_JPRB, 0.40024E+01_JPRB ,&
&  0.38752E+01_JPRB, 0.36196E+01_JPRB, 0.37273E+01_JPRB, 0.40542E+01_JPRB, 0.41209E+01_JPRB, 0.38974E+01_JPRB, 0.39602E+01_JPRB ,&
&  0.45886E+01_JPRB, 0.52745E+01_JPRB, 0.37299E+01_JPRB, 0.41824E+01_JPRB, 0.48239E+01_JPRB, 0.39382E+01_JPRB, 0.34014E+01_JPRB ,&
&  0.30415E+01_JPRB, 0.32751E+01_JPRB, 0.39747E+01_JPRB, 0.42854E+01_JPRB, 0.42368E+01_JPRB, 0.48658E+01_JPRB, 0.39106E+01_JPRB ,&
&  0.31396E+01_JPRB, 0.23364E+01_JPRB, 0.20337E+01_JPRB, 0.19998E+01_JPRB, 0.21968E+01_JPRB, 0.22493E+01_JPRB, 0.15884E+01_JPRB ,&
&  0.14253E+01_JPRB, 0.18243E+01_JPRB, 0.18899E+01_JPRB, 0.20894E+01_JPRB, 0.19925E+01_JPRB, 0.13787E+01_JPRB, 0.14444E+01_JPRB ,&
&  0.15031E+01_JPRB, 0.15414E+01_JPRB, 0.15441E+01_JPRB, 0.15348E+01_JPRB, 0.15253E+01_JPRB, 0.15169E+01_JPRB, 0.15152E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.15132E+01_JPRB, 0.14997E+01_JPRB, 0.14586E+01_JPRB, 0.13978E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.10884E+01_JPRB, 0.99634E+00_JPRB, 0.93325E+00_JPRB, 0.91624E+00_JPRB, 0.92817E+00_JPRB, 0.94511E+00_JPRB, 0.95508E+00_JPRB ,&
&  0.98224E+00_JPRB, 0.10231E+01_JPRB, 0.10827E+01_JPRB, 0.11369E+01_JPRB, 0.12028E+01_JPRB, 0.12693E+01_JPRB, 0.13719E+01_JPRB ,&
&  0.14832E+01_JPRB, 0.16050E+01_JPRB, 0.17943E+01_JPRB, 0.20674E+01_JPRB, 0.22856E+01_JPRB, 0.24895E+01_JPRB, 0.27724E+01_JPRB ,&
&  0.30608E+01_JPRB, 0.32579E+01_JPRB, 0.34637E+01_JPRB, 0.36059E+01_JPRB, 0.35494E+01_JPRB, 0.34368E+01_JPRB, 0.35454E+01_JPRB ,&
&  0.30296E+01_JPRB, 0.24926E+01_JPRB, 0.25175E+01_JPRB, 0.25901E+01_JPRB, 0.27233E+01_JPRB, 0.27975E+01_JPRB, 0.28720E+01_JPRB ,&
&  0.29467E+01_JPRB, 0.29996E+01_JPRB, 0.30498E+01_JPRB, 0.31118E+01_JPRB, 0.32173E+01_JPRB, 0.33885E+01_JPRB, 0.36156E+01_JPRB ,&
&  0.37367E+01_JPRB, 0.38824E+01_JPRB, 0.40615E+01_JPRB, 0.37956E+01_JPRB, 0.42240E+01_JPRB, 0.48093E+01_JPRB, 0.51521E+01_JPRB ,&
&  0.51381E+01_JPRB, 0.48610E+01_JPRB, 0.49939E+01_JPRB, 0.51738E+01_JPRB, 0.52457E+01_JPRB, 0.51595E+01_JPRB, 0.51136E+01_JPRB ,&
&  0.54492E+01_JPRB, 0.52386E+01_JPRB, 0.37400E+01_JPRB, 0.33460E+01_JPRB, 0.34117E+01_JPRB, 0.27308E+01_JPRB, 0.26546E+01_JPRB ,&
&  0.22535E+01_JPRB, 0.23923E+01_JPRB, 0.29988E+01_JPRB, 0.34900E+01_JPRB, 0.37594E+01_JPRB, 0.43148E+01_JPRB, 0.36066E+01_JPRB ,&
&  0.24601E+01_JPRB, 0.18294E+01_JPRB, 0.15355E+01_JPRB, 0.14272E+01_JPRB, 0.15252E+01_JPRB, 0.15559E+01_JPRB, 0.14080E+01_JPRB ,&
&  0.14503E+01_JPRB, 0.14858E+01_JPRB, 0.15334E+01_JPRB, 0.14592E+01_JPRB, 0.13001E+01_JPRB, 0.12506E+01_JPRB, 0.13091E+01_JPRB ,&
&  0.13363E+01_JPRB, 0.13673E+01_JPRB, 0.13826E+01_JPRB, 0.13787E+01_JPRB, 0.13817E+01_JPRB, 0.13803E+01_JPRB, 0.13823E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.13743E+01_JPRB, 0.13407E+01_JPRB, 0.12890E+01_JPRB, 0.11999E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.88058E+00_JPRB, 0.82480E+00_JPRB, 0.81138E+00_JPRB, 0.82284E+00_JPRB, 0.83535E+00_JPRB, 0.84883E+00_JPRB, 0.86409E+00_JPRB ,&
&  0.87644E+00_JPRB, 0.88684E+00_JPRB, 0.90427E+00_JPRB, 0.91799E+00_JPRB, 0.97085E+00_JPRB, 0.10334E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.12299E+01_JPRB, 0.13367E+01_JPRB, 0.14331E+01_JPRB, 0.15883E+01_JPRB, 0.17034E+01_JPRB, 0.18243E+01_JPRB, 0.20103E+01_JPRB ,&
&  0.22415E+01_JPRB, 0.25226E+01_JPRB, 0.27901E+01_JPRB, 0.30868E+01_JPRB, 0.34993E+01_JPRB, 0.39184E+01_JPRB, 0.40217E+01_JPRB ,&
&  0.31481E+01_JPRB, 0.23480E+01_JPRB, 0.25947E+01_JPRB, 0.24845E+01_JPRB, 0.26467E+01_JPRB, 0.27818E+01_JPRB, 0.28514E+01_JPRB ,&
&  0.29362E+01_JPRB, 0.30003E+01_JPRB, 0.30605E+01_JPRB, 0.31281E+01_JPRB, 0.32617E+01_JPRB, 0.34247E+01_JPRB, 0.36322E+01_JPRB ,&
&  0.37989E+01_JPRB, 0.39325E+01_JPRB, 0.39880E+01_JPRB, 0.40251E+01_JPRB, 0.44508E+01_JPRB, 0.50362E+01_JPRB, 0.54655E+01_JPRB ,&
&  0.57337E+01_JPRB, 0.60136E+01_JPRB, 0.55544E+01_JPRB, 0.56276E+01_JPRB, 0.55470E+01_JPRB, 0.49999E+01_JPRB, 0.50877E+01_JPRB ,&
&  0.54470E+01_JPRB, 0.50617E+01_JPRB, 0.32094E+01_JPRB, 0.29309E+01_JPRB, 0.27810E+01_JPRB, 0.24532E+01_JPRB, 0.22323E+01_JPRB ,&
&  0.17871E+01_JPRB, 0.18193E+01_JPRB, 0.20956E+01_JPRB, 0.27325E+01_JPRB, 0.31007E+01_JPRB, 0.32903E+01_JPRB, 0.28759E+01_JPRB ,&
&  0.19642E+01_JPRB, 0.15109E+01_JPRB, 0.12814E+01_JPRB, 0.13012E+01_JPRB, 0.15268E+01_JPRB, 0.18021E+01_JPRB, 0.14638E+01_JPRB ,&
&  0.15105E+01_JPRB, 0.17837E+01_JPRB, 0.22446E+01_JPRB, 0.17232E+01_JPRB, 0.11781E+01_JPRB, 0.11043E+01_JPRB, 0.11155E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.11513E+01_JPRB, 0.11602E+01_JPRB, 0.11633E+01_JPRB, 0.11698E+01_JPRB, 0.11879E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.11474E+01_JPRB, 0.11189E+01_JPRB, 0.10363E+01_JPRB, 0.95154E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.73988E+00_JPRB, 0.73447E+00_JPRB, 0.73881E+00_JPRB, 0.75589E+00_JPRB, 0.75682E+00_JPRB, 0.75830E+00_JPRB, 0.79229E+00_JPRB ,&
&  0.78198E+00_JPRB, 0.77401E+00_JPRB, 0.78159E+00_JPRB, 0.76317E+00_JPRB, 0.79780E+00_JPRB, 0.83425E+00_JPRB, 0.89367E+00_JPRB ,&
&  0.95445E+00_JPRB, 0.99596E+00_JPRB, 0.10307E+01_JPRB, 0.10792E+01_JPRB, 0.11700E+01_JPRB, 0.12421E+01_JPRB, 0.13062E+01_JPRB ,&
&  0.14184E+01_JPRB, 0.15857E+01_JPRB, 0.17667E+01_JPRB, 0.21644E+01_JPRB, 0.30752E+01_JPRB, 0.38273E+01_JPRB, 0.38543E+01_JPRB ,&
&  0.29662E+01_JPRB, 0.22917E+01_JPRB, 0.21825E+01_JPRB, 0.20170E+01_JPRB, 0.21609E+01_JPRB, 0.24320E+01_JPRB, 0.25508E+01_JPRB ,&
&  0.25710E+01_JPRB, 0.26311E+01_JPRB, 0.26653E+01_JPRB, 0.27088E+01_JPRB, 0.27627E+01_JPRB, 0.28006E+01_JPRB, 0.27690E+01_JPRB ,&
&  0.26725E+01_JPRB, 0.25501E+01_JPRB, 0.26639E+01_JPRB, 0.32458E+01_JPRB, 0.39121E+01_JPRB, 0.47994E+01_JPRB, 0.47195E+01_JPRB ,&
&  0.45347E+01_JPRB, 0.45532E+01_JPRB, 0.46108E+01_JPRB, 0.45510E+01_JPRB, 0.43760E+01_JPRB, 0.41751E+01_JPRB, 0.42713E+01_JPRB ,&
&  0.45606E+01_JPRB, 0.42555E+01_JPRB, 0.28062E+01_JPRB, 0.26988E+01_JPRB, 0.24636E+01_JPRB, 0.23489E+01_JPRB, 0.19245E+01_JPRB ,&
&  0.15220E+01_JPRB, 0.14531E+01_JPRB, 0.15555E+01_JPRB, 0.18119E+01_JPRB, 0.22755E+01_JPRB, 0.24360E+01_JPRB, 0.21313E+01_JPRB ,&
&  0.16396E+01_JPRB, 0.13224E+01_JPRB, 0.11608E+01_JPRB, 0.11917E+01_JPRB, 0.15611E+01_JPRB, 0.18557E+01_JPRB, 0.20207E+01_JPRB ,&
&  0.14024E+01_JPRB, 0.15874E+01_JPRB, 0.14114E+01_JPRB, 0.17441E+01_JPRB, 0.99129E+00_JPRB, 0.87401E+00_JPRB, 0.86392E+00_JPRB ,&
&  0.87532E+00_JPRB, 0.86672E+00_JPRB, 0.85684E+00_JPRB, 0.83761E+00_JPRB, 0.81985E+00_JPRB, 0.87938E+00_JPRB, 0.93679E+00_JPRB ,&
&  0.88784E+00_JPRB, 0.91466E+00_JPRB, 0.87552E+00_JPRB, 0.80959E+00_JPRB, 0.77864E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.70548E+00_JPRB, 0.71807E+00_JPRB, 0.72297E+00_JPRB, 0.73458E+00_JPRB, 0.75700E+00_JPRB, 0.77576E+00_JPRB, 0.80023E+00_JPRB ,&
&  0.82822E+00_JPRB, 0.84644E+00_JPRB, 0.86603E+00_JPRB, 0.89016E+00_JPRB, 0.91078E+00_JPRB, 0.94722E+00_JPRB, 0.98223E+00_JPRB ,&
&  0.10254E+01_JPRB, 0.10603E+01_JPRB, 0.10959E+01_JPRB, 0.11497E+01_JPRB, 0.12165E+01_JPRB, 0.13170E+01_JPRB, 0.14268E+01_JPRB ,&
&  0.15735E+01_JPRB, 0.17464E+01_JPRB, 0.19533E+01_JPRB, 0.21285E+01_JPRB, 0.22289E+01_JPRB, 0.23094E+01_JPRB, 0.26448E+01_JPRB ,&
&  0.23342E+01_JPRB, 0.23730E+01_JPRB, 0.22659E+01_JPRB, 0.20413E+01_JPRB, 0.20343E+01_JPRB, 0.18812E+01_JPRB, 0.18523E+01_JPRB ,&
&  0.18848E+01_JPRB, 0.18350E+01_JPRB, 0.18073E+01_JPRB, 0.17817E+01_JPRB, 0.17423E+01_JPRB, 0.16471E+01_JPRB, 0.15317E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.16515E+01_JPRB, 0.19277E+01_JPRB, 0.24840E+01_JPRB, 0.30511E+01_JPRB, 0.32946E+01_JPRB, 0.30566E+01_JPRB ,&
&  0.29506E+01_JPRB, 0.29843E+01_JPRB, 0.29197E+01_JPRB, 0.27950E+01_JPRB, 0.29259E+01_JPRB, 0.30349E+01_JPRB, 0.31610E+01_JPRB ,&
&  0.30743E+01_JPRB, 0.27317E+01_JPRB, 0.25074E+01_JPRB, 0.22277E+01_JPRB, 0.21179E+01_JPRB, 0.20057E+01_JPRB, 0.17613E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.12982E+01_JPRB, 0.12420E+01_JPRB, 0.13181E+01_JPRB, 0.15889E+01_JPRB, 0.16293E+01_JPRB, 0.14716E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.10849E+01_JPRB, 0.10516E+01_JPRB, 0.11156E+01_JPRB, 0.14050E+01_JPRB, 0.30789E+01_JPRB, 0.40679E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.12184E+01_JPRB, 0.75931E+00_JPRB, 0.10669E+01_JPRB, 0.75792E+00_JPRB, 0.71294E+00_JPRB, 0.71116E+00_JPRB ,&
&  0.70367E+00_JPRB, 0.70244E+00_JPRB, 0.70242E+00_JPRB, 0.68947E+00_JPRB, 0.68303E+00_JPRB, 0.69327E+00_JPRB, 0.73621E+00_JPRB ,&
&  0.73609E+00_JPRB, 0.74211E+00_JPRB, 0.72391E+00_JPRB, 0.72379E+00_JPRB, 0.71192E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.71350E+00_JPRB, 0.73344E+00_JPRB, 0.75881E+00_JPRB, 0.79019E+00_JPRB, 0.81483E+00_JPRB, 0.84542E+00_JPRB, 0.87284E+00_JPRB ,&
&  0.90700E+00_JPRB, 0.93746E+00_JPRB, 0.96576E+00_JPRB, 0.99608E+00_JPRB, 0.10307E+01_JPRB, 0.10688E+01_JPRB, 0.11158E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.12169E+01_JPRB, 0.12729E+01_JPRB, 0.13369E+01_JPRB, 0.14119E+01_JPRB, 0.15039E+01_JPRB, 0.15940E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.17741E+01_JPRB, 0.18759E+01_JPRB, 0.19571E+01_JPRB, 0.19963E+01_JPRB, 0.20846E+01_JPRB, 0.19383E+01_JPRB ,&
&  0.23570E+01_JPRB, 0.22274E+01_JPRB, 0.20921E+01_JPRB, 0.19095E+01_JPRB, 0.17329E+01_JPRB, 0.15707E+01_JPRB, 0.14458E+01_JPRB ,&
&  0.14829E+01_JPRB, 0.14315E+01_JPRB, 0.14148E+01_JPRB, 0.14208E+01_JPRB, 0.13089E+01_JPRB, 0.12988E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.15747E+01_JPRB, 0.18174E+01_JPRB, 0.19872E+01_JPRB, 0.21863E+01_JPRB, 0.24238E+01_JPRB, 0.23493E+01_JPRB, 0.22029E+01_JPRB ,&
&  0.20601E+01_JPRB, 0.19831E+01_JPRB, 0.16139E+01_JPRB, 0.18442E+01_JPRB, 0.19180E+01_JPRB, 0.20075E+01_JPRB, 0.21629E+01_JPRB ,&
&  0.20307E+01_JPRB, 0.16421E+01_JPRB, 0.16819E+01_JPRB, 0.19333E+01_JPRB, 0.18453E+01_JPRB, 0.17074E+01_JPRB, 0.15865E+01_JPRB ,&
&  0.14304E+01_JPRB, 0.13050E+01_JPRB, 0.11970E+01_JPRB, 0.11576E+01_JPRB, 0.12352E+01_JPRB, 0.12239E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.98548E+00_JPRB, 0.96371E+00_JPRB, 0.98492E+00_JPRB, 0.10546E+01_JPRB, 0.13517E+01_JPRB, 0.17190E+01_JPRB, 0.27854E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.84284E+00_JPRB, 0.88876E+00_JPRB, 0.85724E+00_JPRB, 0.74245E+00_JPRB, 0.67645E+00_JPRB, 0.69737E+00_JPRB ,&
&  0.72492E+00_JPRB, 0.73617E+00_JPRB, 0.69368E+00_JPRB, 0.69364E+00_JPRB, 0.69244E+00_JPRB, 0.69248E+00_JPRB, 0.69728E+00_JPRB ,&
&  0.69648E+00_JPRB, 0.69871E+00_JPRB, 0.70833E+00_JPRB, 0.70495E+00_JPRB, 0.70498E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.62896E+00_JPRB, 0.64224E+00_JPRB, 0.66130E+00_JPRB, 0.67862E+00_JPRB, 0.69067E+00_JPRB, 0.70877E+00_JPRB, 0.72373E+00_JPRB ,&
&  0.74098E+00_JPRB, 0.75167E+00_JPRB, 0.76672E+00_JPRB, 0.78115E+00_JPRB, 0.79681E+00_JPRB, 0.81784E+00_JPRB, 0.83993E+00_JPRB ,&
&  0.85964E+00_JPRB, 0.89064E+00_JPRB, 0.92360E+00_JPRB, 0.94996E+00_JPRB, 0.10071E+01_JPRB, 0.11118E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.13424E+01_JPRB, 0.14887E+01_JPRB, 0.16298E+01_JPRB, 0.17179E+01_JPRB, 0.17538E+01_JPRB, 0.18758E+01_JPRB, 0.16571E+01_JPRB ,&
&  0.21316E+01_JPRB, 0.20918E+01_JPRB, 0.19134E+01_JPRB, 0.18089E+01_JPRB, 0.17011E+01_JPRB, 0.15990E+01_JPRB, 0.15526E+01_JPRB ,&
&  0.16130E+01_JPRB, 0.19556E+01_JPRB, 0.19574E+01_JPRB, 0.20240E+01_JPRB, 0.18028E+01_JPRB, 0.17742E+01_JPRB, 0.19929E+01_JPRB ,&
&  0.22492E+01_JPRB, 0.23640E+01_JPRB, 0.24235E+01_JPRB, 0.25655E+01_JPRB, 0.27210E+01_JPRB, 0.28037E+01_JPRB, 0.28386E+01_JPRB ,&
&  0.27343E+01_JPRB, 0.26392E+01_JPRB, 0.24840E+01_JPRB, 0.29521E+01_JPRB, 0.25919E+01_JPRB, 0.22384E+01_JPRB, 0.20302E+01_JPRB ,&
&  0.15470E+01_JPRB, 0.14075E+01_JPRB, 0.14761E+01_JPRB, 0.15217E+01_JPRB, 0.14866E+01_JPRB, 0.14801E+01_JPRB, 0.14409E+01_JPRB ,&
&  0.13654E+01_JPRB, 0.13082E+01_JPRB, 0.12247E+01_JPRB, 0.11971E+01_JPRB, 0.12033E+01_JPRB, 0.11397E+01_JPRB, 0.10083E+01_JPRB ,&
&  0.92847E+00_JPRB, 0.94146E+00_JPRB, 0.94178E+00_JPRB, 0.97638E+00_JPRB, 0.11324E+01_JPRB, 0.11985E+01_JPRB, 0.18235E+01_JPRB ,&
&  0.12921E+01_JPRB, 0.11111E+01_JPRB, 0.14621E+01_JPRB, 0.56165E+00_JPRB, 0.64760E+00_JPRB, 0.74472E+00_JPRB, 0.93737E+00_JPRB ,&
&  0.97502E+00_JPRB, 0.12431E+01_JPRB, 0.15715E+01_JPRB, 0.12504E+01_JPRB, 0.97004E+00_JPRB, 0.68462E+00_JPRB, 0.65446E+00_JPRB ,&
&  0.64749E+00_JPRB, 0.63861E+00_JPRB, 0.64088E+00_JPRB, 0.63306E+00_JPRB, 0.62967E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.53929E+00_JPRB, 0.54559E+00_JPRB, 0.55864E+00_JPRB, 0.56817E+00_JPRB, 0.57738E+00_JPRB, 0.58572E+00_JPRB, 0.59261E+00_JPRB ,&
&  0.61035E+00_JPRB, 0.62234E+00_JPRB, 0.65192E+00_JPRB, 0.67971E+00_JPRB, 0.70575E+00_JPRB, 0.75435E+00_JPRB, 0.79936E+00_JPRB ,&
&  0.86665E+00_JPRB, 0.95351E+00_JPRB, 0.10013E+01_JPRB, 0.10588E+01_JPRB, 0.10979E+01_JPRB, 0.11463E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.13244E+01_JPRB, 0.15828E+01_JPRB, 0.17743E+01_JPRB, 0.19774E+01_JPRB, 0.20133E+01_JPRB, 0.21906E+01_JPRB, 0.17174E+01_JPRB ,&
&  0.21677E+01_JPRB, 0.22262E+01_JPRB, 0.21208E+01_JPRB, 0.20864E+01_JPRB, 0.20825E+01_JPRB, 0.20973E+01_JPRB, 0.21114E+01_JPRB ,&
&  0.22441E+01_JPRB, 0.24250E+01_JPRB, 0.23990E+01_JPRB, 0.25089E+01_JPRB, 0.26008E+01_JPRB, 0.26253E+01_JPRB, 0.29566E+01_JPRB ,&
&  0.32654E+01_JPRB, 0.35163E+01_JPRB, 0.38244E+01_JPRB, 0.41956E+01_JPRB, 0.44556E+01_JPRB, 0.46173E+01_JPRB, 0.46869E+01_JPRB ,&
&  0.47213E+01_JPRB, 0.48801E+01_JPRB, 0.53038E+01_JPRB, 0.52717E+01_JPRB, 0.41275E+01_JPRB, 0.33179E+01_JPRB, 0.24844E+01_JPRB ,&
&  0.16073E+01_JPRB, 0.12617E+01_JPRB, 0.12205E+01_JPRB, 0.12340E+01_JPRB, 0.12086E+01_JPRB, 0.12058E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.12180E+01_JPRB, 0.12155E+01_JPRB, 0.12084E+01_JPRB, 0.11654E+01_JPRB, 0.10448E+01_JPRB, 0.91091E+00_JPRB ,&
&  0.87772E+00_JPRB, 0.87120E+00_JPRB, 0.88912E+00_JPRB, 0.94862E+00_JPRB, 0.10756E+01_JPRB, 0.13219E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.13962E+01_JPRB, 0.90850E+00_JPRB, 0.84076E+00_JPRB, 0.86483E+00_JPRB, 0.69838E+00_JPRB, 0.94612E+00_JPRB, 0.13594E+01_JPRB ,&
&  0.19822E+01_JPRB, 0.22053E+01_JPRB, 0.28637E+01_JPRB, 0.41688E+01_JPRB, 0.26356E+01_JPRB, 0.17127E+01_JPRB, 0.60869E+00_JPRB ,&
&  0.59985E+00_JPRB, 0.57672E+00_JPRB, 0.56966E+00_JPRB, 0.55337E+00_JPRB, 0.54574E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.44119E+00_JPRB, 0.43536E+00_JPRB, 0.44860E+00_JPRB, 0.46843E+00_JPRB, 0.48403E+00_JPRB, 0.51863E+00_JPRB, 0.56086E+00_JPRB ,&
&  0.60783E+00_JPRB, 0.66884E+00_JPRB, 0.72724E+00_JPRB, 0.77798E+00_JPRB, 0.85746E+00_JPRB, 0.96190E+00_JPRB, 0.10264E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.12161E+01_JPRB, 0.12779E+01_JPRB, 0.13277E+01_JPRB, 0.13954E+01_JPRB, 0.14873E+01_JPRB, 0.16107E+01_JPRB ,&
&  0.17386E+01_JPRB, 0.18812E+01_JPRB, 0.20647E+01_JPRB, 0.21927E+01_JPRB, 0.22694E+01_JPRB, 0.23162E+01_JPRB, 0.18228E+01_JPRB ,&
&  0.22825E+01_JPRB, 0.24821E+01_JPRB, 0.25692E+01_JPRB, 0.25529E+01_JPRB, 0.25173E+01_JPRB, 0.25162E+01_JPRB, 0.26027E+01_JPRB ,&
&  0.27427E+01_JPRB, 0.27870E+01_JPRB, 0.27689E+01_JPRB, 0.27879E+01_JPRB, 0.35595E+01_JPRB, 0.37091E+01_JPRB, 0.40825E+01_JPRB ,&
&  0.44398E+01_JPRB, 0.47537E+01_JPRB, 0.51613E+01_JPRB, 0.55912E+01_JPRB, 0.60367E+01_JPRB, 0.64147E+01_JPRB, 0.67565E+01_JPRB ,&
&  0.70782E+01_JPRB, 0.74222E+01_JPRB, 0.77380E+01_JPRB, 0.66280E+01_JPRB, 0.52055E+01_JPRB, 0.41998E+01_JPRB, 0.25036E+01_JPRB ,&
&  0.14997E+01_JPRB, 0.12516E+01_JPRB, 0.12288E+01_JPRB, 0.11513E+01_JPRB, 0.10346E+01_JPRB, 0.99703E+00_JPRB, 0.97201E+00_JPRB ,&
&  0.99889E+00_JPRB, 0.10613E+01_JPRB, 0.11001E+01_JPRB, 0.11020E+01_JPRB, 0.10422E+01_JPRB, 0.92910E+00_JPRB, 0.82319E+00_JPRB ,&
&  0.83720E+00_JPRB, 0.84412E+00_JPRB, 0.86130E+00_JPRB, 0.96369E+00_JPRB, 0.11208E+01_JPRB, 0.14597E+01_JPRB, 0.18479E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.91864E+00_JPRB, 0.79995E+00_JPRB, 0.75686E+00_JPRB, 0.72973E+00_JPRB, 0.79540E+00_JPRB, 0.95511E+00_JPRB ,&
&  0.13315E+01_JPRB, 0.21425E+01_JPRB, 0.23610E+01_JPRB, 0.15554E+01_JPRB, 0.20536E+01_JPRB, 0.13626E+01_JPRB, 0.58825E+00_JPRB ,&
&  0.56275E+00_JPRB, 0.54303E+00_JPRB, 0.51493E+00_JPRB, 0.48245E+00_JPRB, 0.45813E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.44204E+00_JPRB, 0.42322E+00_JPRB, 0.43612E+00_JPRB, 0.46269E+00_JPRB, 0.49361E+00_JPRB, 0.54252E+00_JPRB, 0.60533E+00_JPRB ,&
&  0.66556E+00_JPRB, 0.72962E+00_JPRB, 0.83677E+00_JPRB, 0.87789E+00_JPRB, 0.92430E+00_JPRB, 0.10019E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.11606E+01_JPRB, 0.12495E+01_JPRB, 0.13391E+01_JPRB, 0.14257E+01_JPRB, 0.15320E+01_JPRB, 0.16472E+01_JPRB, 0.17481E+01_JPRB ,&
&  0.18790E+01_JPRB, 0.20152E+01_JPRB, 0.21963E+01_JPRB, 0.23895E+01_JPRB, 0.25718E+01_JPRB, 0.26268E+01_JPRB, 0.27539E+01_JPRB ,&
&  0.18370E+01_JPRB, 0.25856E+01_JPRB, 0.26758E+01_JPRB, 0.27059E+01_JPRB, 0.26634E+01_JPRB, 0.26264E+01_JPRB, 0.27178E+01_JPRB ,&
&  0.28547E+01_JPRB, 0.28087E+01_JPRB, 0.29785E+01_JPRB, 0.32326E+01_JPRB, 0.41056E+01_JPRB, 0.43390E+01_JPRB, 0.46366E+01_JPRB ,&
&  0.49503E+01_JPRB, 0.52525E+01_JPRB, 0.56054E+01_JPRB, 0.60320E+01_JPRB, 0.64611E+01_JPRB, 0.68640E+01_JPRB, 0.72733E+01_JPRB ,&
&  0.76527E+01_JPRB, 0.80032E+01_JPRB, 0.83565E+01_JPRB, 0.68822E+01_JPRB, 0.56541E+01_JPRB, 0.45527E+01_JPRB, 0.28715E+01_JPRB ,&
&  0.17053E+01_JPRB, 0.12331E+01_JPRB, 0.12156E+01_JPRB, 0.12034E+01_JPRB, 0.10326E+01_JPRB, 0.94907E+00_JPRB, 0.84699E+00_JPRB ,&
&  0.84355E+00_JPRB, 0.87831E+00_JPRB, 0.90588E+00_JPRB, 0.91446E+00_JPRB, 0.91487E+00_JPRB, 0.82644E+00_JPRB, 0.76521E+00_JPRB ,&
&  0.78738E+00_JPRB, 0.79052E+00_JPRB, 0.81552E+00_JPRB, 0.87023E+00_JPRB, 0.95730E+00_JPRB, 0.10212E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.88823E+00_JPRB, 0.83493E+00_JPRB, 0.84110E+00_JPRB, 0.80228E+00_JPRB, 0.76341E+00_JPRB, 0.77366E+00_JPRB ,&
&  0.85415E+00_JPRB, 0.96314E+00_JPRB, 0.10511E+01_JPRB, 0.10260E+01_JPRB, 0.10435E+01_JPRB, 0.83681E+00_JPRB, 0.64333E+00_JPRB ,&
&  0.60465E+00_JPRB, 0.59550E+00_JPRB, 0.54842E+00_JPRB, 0.51398E+00_JPRB, 0.47536E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.54945E+00_JPRB, 0.48307E+00_JPRB, 0.47909E+00_JPRB, 0.52502E+00_JPRB, 0.57113E+00_JPRB, 0.62021E+00_JPRB, 0.66911E+00_JPRB ,&
&  0.72537E+00_JPRB, 0.76113E+00_JPRB, 0.82652E+00_JPRB, 0.86891E+00_JPRB, 0.90802E+00_JPRB, 0.94361E+00_JPRB, 0.99081E+00_JPRB ,&
&  0.10675E+01_JPRB, 0.11606E+01_JPRB, 0.12506E+01_JPRB, 0.13664E+01_JPRB, 0.14889E+01_JPRB, 0.16014E+01_JPRB, 0.17263E+01_JPRB ,&
&  0.18511E+01_JPRB, 0.19908E+01_JPRB, 0.21376E+01_JPRB, 0.23097E+01_JPRB, 0.24680E+01_JPRB, 0.25378E+01_JPRB, 0.24975E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.20481E+01_JPRB, 0.26491E+01_JPRB, 0.26317E+01_JPRB, 0.26195E+01_JPRB, 0.25584E+01_JPRB, 0.26307E+01_JPRB ,&
&  0.25722E+01_JPRB, 0.24407E+01_JPRB, 0.26713E+01_JPRB, 0.36893E+01_JPRB, 0.42397E+01_JPRB, 0.45261E+01_JPRB, 0.47043E+01_JPRB ,&
&  0.49258E+01_JPRB, 0.50982E+01_JPRB, 0.53086E+01_JPRB, 0.55383E+01_JPRB, 0.57923E+01_JPRB, 0.60371E+01_JPRB, 0.62667E+01_JPRB ,&
&  0.64741E+01_JPRB, 0.66033E+01_JPRB, 0.65774E+01_JPRB, 0.51807E+01_JPRB, 0.50857E+01_JPRB, 0.42685E+01_JPRB, 0.35806E+01_JPRB ,&
&  0.20749E+01_JPRB, 0.13303E+01_JPRB, 0.12659E+01_JPRB, 0.12467E+01_JPRB, 0.10663E+01_JPRB, 0.96241E+00_JPRB, 0.83130E+00_JPRB ,&
&  0.81298E+00_JPRB, 0.80978E+00_JPRB, 0.82916E+00_JPRB, 0.83256E+00_JPRB, 0.83463E+00_JPRB, 0.75861E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.78017E+00_JPRB, 0.78756E+00_JPRB, 0.81248E+00_JPRB, 0.85677E+00_JPRB, 0.91446E+00_JPRB, 0.96588E+00_JPRB, 0.98311E+00_JPRB ,&
&  0.97945E+00_JPRB, 0.95317E+00_JPRB, 0.94068E+00_JPRB, 0.92997E+00_JPRB, 0.91557E+00_JPRB, 0.89783E+00_JPRB, 0.87369E+00_JPRB ,&
&  0.84174E+00_JPRB, 0.84204E+00_JPRB, 0.86331E+00_JPRB, 0.78933E+00_JPRB, 0.82754E+00_JPRB, 0.75362E+00_JPRB, 0.73873E+00_JPRB ,&
&  0.72069E+00_JPRB, 0.70523E+00_JPRB, 0.65890E+00_JPRB, 0.63437E+00_JPRB, 0.59695E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.67618E+00_JPRB, 0.59147E+00_JPRB, 0.52682E+00_JPRB, 0.54212E+00_JPRB, 0.61156E+00_JPRB, 0.67033E+00_JPRB, 0.70476E+00_JPRB ,&
&  0.74746E+00_JPRB, 0.76490E+00_JPRB, 0.77887E+00_JPRB, 0.79690E+00_JPRB, 0.83653E+00_JPRB, 0.86754E+00_JPRB, 0.92584E+00_JPRB ,&
&  0.97860E+00_JPRB, 0.10320E+01_JPRB, 0.10889E+01_JPRB, 0.11823E+01_JPRB, 0.12977E+01_JPRB, 0.14132E+01_JPRB, 0.15218E+01_JPRB ,&
&  0.16456E+01_JPRB, 0.17968E+01_JPRB, 0.20025E+01_JPRB, 0.22499E+01_JPRB, 0.25931E+01_JPRB, 0.29736E+01_JPRB, 0.32659E+01_JPRB ,&
&  0.22771E+01_JPRB, 0.10316E+01_JPRB, 0.21365E+01_JPRB, 0.25880E+01_JPRB, 0.24799E+01_JPRB, 0.24354E+01_JPRB, 0.23559E+01_JPRB ,&
&  0.22894E+01_JPRB, 0.22677E+01_JPRB, 0.23486E+01_JPRB, 0.31112E+01_JPRB, 0.40016E+01_JPRB, 0.43219E+01_JPRB, 0.44378E+01_JPRB ,&
&  0.45539E+01_JPRB, 0.46001E+01_JPRB, 0.46751E+01_JPRB, 0.47885E+01_JPRB, 0.49310E+01_JPRB, 0.50187E+01_JPRB, 0.50880E+01_JPRB ,&
&  0.51426E+01_JPRB, 0.51328E+01_JPRB, 0.49385E+01_JPRB, 0.35397E+01_JPRB, 0.31481E+01_JPRB, 0.26879E+01_JPRB, 0.24212E+01_JPRB ,&
&  0.20791E+01_JPRB, 0.15356E+01_JPRB, 0.13672E+01_JPRB, 0.12957E+01_JPRB, 0.11364E+01_JPRB, 0.96530E+00_JPRB, 0.87821E+00_JPRB ,&
&  0.85079E+00_JPRB, 0.82150E+00_JPRB, 0.80171E+00_JPRB, 0.77946E+00_JPRB, 0.81309E+00_JPRB, 0.80852E+00_JPRB, 0.79391E+00_JPRB ,&
&  0.78853E+00_JPRB, 0.83554E+00_JPRB, 0.88520E+00_JPRB, 0.92876E+00_JPRB, 0.97336E+00_JPRB, 0.10236E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.10416E+01_JPRB, 0.10300E+01_JPRB, 0.10022E+01_JPRB, 0.97981E+00_JPRB, 0.94295E+00_JPRB, 0.93766E+00_JPRB ,&
&  0.93424E+00_JPRB, 0.93552E+00_JPRB, 0.91510E+00_JPRB, 0.88938E+00_JPRB, 0.90285E+00_JPRB, 0.88407E+00_JPRB, 0.88379E+00_JPRB ,&
&  0.87523E+00_JPRB, 0.85294E+00_JPRB, 0.79708E+00_JPRB, 0.74906E+00_JPRB, 0.72563E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.75906E+00_JPRB, 0.69864E+00_JPRB, 0.62342E+00_JPRB, 0.56993E+00_JPRB, 0.58705E+00_JPRB, 0.65742E+00_JPRB, 0.71623E+00_JPRB ,&
&  0.74043E+00_JPRB, 0.74501E+00_JPRB, 0.73990E+00_JPRB, 0.74429E+00_JPRB, 0.76755E+00_JPRB, 0.77657E+00_JPRB, 0.80896E+00_JPRB ,&
&  0.85825E+00_JPRB, 0.90137E+00_JPRB, 0.93749E+00_JPRB, 0.99787E+00_JPRB, 0.10629E+01_JPRB, 0.11676E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.14282E+01_JPRB, 0.15579E+01_JPRB, 0.17390E+01_JPRB, 0.20016E+01_JPRB, 0.24692E+01_JPRB, 0.32827E+01_JPRB ,&
&  0.47621E+01_JPRB, 0.31903E+01_JPRB, 0.13698E+01_JPRB, 0.23962E+01_JPRB, 0.25670E+01_JPRB, 0.24805E+01_JPRB, 0.21753E+01_JPRB ,&
&  0.21834E+01_JPRB, 0.20518E+01_JPRB, 0.20173E+01_JPRB, 0.23561E+01_JPRB, 0.28680E+01_JPRB, 0.34830E+01_JPRB, 0.38503E+01_JPRB ,&
&  0.39476E+01_JPRB, 0.39944E+01_JPRB, 0.40320E+01_JPRB, 0.41205E+01_JPRB, 0.42540E+01_JPRB, 0.43713E+01_JPRB, 0.43965E+01_JPRB ,&
&  0.44375E+01_JPRB, 0.45294E+01_JPRB, 0.44232E+01_JPRB, 0.32420E+01_JPRB, 0.29585E+01_JPRB, 0.28290E+01_JPRB, 0.26660E+01_JPRB ,&
&  0.22968E+01_JPRB, 0.19950E+01_JPRB, 0.17455E+01_JPRB, 0.14385E+01_JPRB, 0.12203E+01_JPRB, 0.10968E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.98980E+00_JPRB, 0.90790E+00_JPRB, 0.87817E+00_JPRB, 0.84669E+00_JPRB, 0.86173E+00_JPRB, 0.89925E+00_JPRB, 0.90764E+00_JPRB ,&
&  0.93434E+00_JPRB, 0.97643E+00_JPRB, 0.10070E+01_JPRB, 0.10359E+01_JPRB, 0.10720E+01_JPRB, 0.11012E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11326E+01_JPRB, 0.11142E+01_JPRB, 0.10889E+01_JPRB, 0.10396E+01_JPRB, 0.99683E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.99764E+00_JPRB, 0.99876E+00_JPRB, 0.97815E+00_JPRB, 0.98097E+00_JPRB, 0.96946E+00_JPRB, 0.98174E+00_JPRB, 0.10053E+01_JPRB ,&
&  0.10247E+01_JPRB, 0.99654E+00_JPRB, 0.90697E+00_JPRB, 0.81254E+00_JPRB, 0.78488E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.80647E+00_JPRB, 0.78443E+00_JPRB, 0.73685E+00_JPRB, 0.64544E+00_JPRB, 0.60388E+00_JPRB, 0.61494E+00_JPRB, 0.68219E+00_JPRB ,&
&  0.71400E+00_JPRB, 0.72724E+00_JPRB, 0.71601E+00_JPRB, 0.70541E+00_JPRB, 0.70496E+00_JPRB, 0.71712E+00_JPRB, 0.74804E+00_JPRB ,&
&  0.78523E+00_JPRB, 0.78978E+00_JPRB, 0.78471E+00_JPRB, 0.79559E+00_JPRB, 0.84215E+00_JPRB, 0.90743E+00_JPRB, 0.10126E+01_JPRB ,&
&  0.10975E+01_JPRB, 0.11467E+01_JPRB, 0.11966E+01_JPRB, 0.13125E+01_JPRB, 0.14550E+01_JPRB, 0.15284E+01_JPRB, 0.17566E+01_JPRB ,&
&  0.27413E+01_JPRB, 0.41835E+01_JPRB, 0.16243E+01_JPRB, 0.24696E+01_JPRB, 0.28890E+01_JPRB, 0.26149E+01_JPRB, 0.23905E+01_JPRB ,&
&  0.23023E+01_JPRB, 0.19401E+01_JPRB, 0.16445E+01_JPRB, 0.17798E+01_JPRB, 0.20560E+01_JPRB, 0.22864E+01_JPRB, 0.26553E+01_JPRB ,&
&  0.29876E+01_JPRB, 0.31984E+01_JPRB, 0.33650E+01_JPRB, 0.35028E+01_JPRB, 0.36906E+01_JPRB, 0.39424E+01_JPRB, 0.41683E+01_JPRB ,&
&  0.43391E+01_JPRB, 0.45842E+01_JPRB, 0.47612E+01_JPRB, 0.37609E+01_JPRB, 0.35748E+01_JPRB, 0.38279E+01_JPRB, 0.35917E+01_JPRB ,&
&  0.30510E+01_JPRB, 0.26188E+01_JPRB, 0.21494E+01_JPRB, 0.16878E+01_JPRB, 0.13877E+01_JPRB, 0.12664E+01_JPRB, 0.12000E+01_JPRB ,&
&  0.11115E+01_JPRB, 0.10189E+01_JPRB, 0.10005E+01_JPRB, 0.99178E+00_JPRB, 0.99320E+00_JPRB, 0.10076E+01_JPRB, 0.10391E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.11373E+01_JPRB, 0.11704E+01_JPRB, 0.11866E+01_JPRB, 0.11974E+01_JPRB, 0.12062E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.11935E+01_JPRB, 0.11803E+01_JPRB, 0.11594E+01_JPRB, 0.11321E+01_JPRB, 0.11016E+01_JPRB, 0.10853E+01_JPRB, 0.10780E+01_JPRB ,&
&  0.10649E+01_JPRB, 0.10442E+01_JPRB, 0.10055E+01_JPRB, 0.98103E+00_JPRB, 0.10053E+01_JPRB, 0.97712E+00_JPRB, 0.10218E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.11156E+01_JPRB, 0.10496E+01_JPRB, 0.84640E+00_JPRB, 0.81739E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.84848E+00_JPRB, 0.83550E+00_JPRB, 0.81671E+00_JPRB, 0.75142E+00_JPRB, 0.68406E+00_JPRB, 0.64839E+00_JPRB, 0.66076E+00_JPRB ,&
&  0.69936E+00_JPRB, 0.72716E+00_JPRB, 0.73197E+00_JPRB, 0.72632E+00_JPRB, 0.73003E+00_JPRB, 0.74600E+00_JPRB, 0.76691E+00_JPRB ,&
&  0.77994E+00_JPRB, 0.77598E+00_JPRB, 0.76040E+00_JPRB, 0.76222E+00_JPRB, 0.76810E+00_JPRB, 0.78600E+00_JPRB, 0.84117E+00_JPRB ,&
&  0.91670E+00_JPRB, 0.96024E+00_JPRB, 0.97232E+00_JPRB, 0.10104E+01_JPRB, 0.11202E+01_JPRB, 0.11614E+01_JPRB, 0.12205E+01_JPRB ,&
&  0.15973E+01_JPRB, 0.32297E+01_JPRB, 0.34432E+01_JPRB, 0.39612E+01_JPRB, 0.35520E+01_JPRB, 0.29623E+01_JPRB, 0.28148E+01_JPRB ,&
&  0.27984E+01_JPRB, 0.23085E+01_JPRB, 0.18897E+01_JPRB, 0.16005E+01_JPRB, 0.16694E+01_JPRB, 0.18049E+01_JPRB, 0.18552E+01_JPRB ,&
&  0.20181E+01_JPRB, 0.23627E+01_JPRB, 0.25770E+01_JPRB, 0.27854E+01_JPRB, 0.29824E+01_JPRB, 0.32159E+01_JPRB, 0.35494E+01_JPRB ,&
&  0.39504E+01_JPRB, 0.44654E+01_JPRB, 0.51038E+01_JPRB, 0.48052E+01_JPRB, 0.42008E+01_JPRB, 0.48620E+01_JPRB, 0.50484E+01_JPRB ,&
&  0.47595E+01_JPRB, 0.35763E+01_JPRB, 0.26856E+01_JPRB, 0.20441E+01_JPRB, 0.16289E+01_JPRB, 0.16251E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.11521E+01_JPRB, 0.11208E+01_JPRB, 0.11268E+01_JPRB, 0.11360E+01_JPRB, 0.11252E+01_JPRB, 0.11474E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.12287E+01_JPRB, 0.12512E+01_JPRB, 0.12664E+01_JPRB, 0.12829E+01_JPRB, 0.12933E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.12103E+01_JPRB, 0.11356E+01_JPRB, 0.11241E+01_JPRB, 0.11217E+01_JPRB, 0.11282E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.10849E+01_JPRB, 0.10848E+01_JPRB, 0.10497E+01_JPRB, 0.10221E+01_JPRB, 0.10273E+01_JPRB, 0.10094E+01_JPRB, 0.97540E+00_JPRB ,&
&  0.97890E+00_JPRB, 0.95674E+00_JPRB, 0.92095E+00_JPRB, 0.88753E+00_JPRB, 0.86413E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.90499E+00_JPRB, 0.89210E+00_JPRB, 0.88397E+00_JPRB, 0.85021E+00_JPRB, 0.80625E+00_JPRB, 0.76480E+00_JPRB, 0.74019E+00_JPRB ,&
&  0.73987E+00_JPRB, 0.75302E+00_JPRB, 0.76692E+00_JPRB, 0.76007E+00_JPRB, 0.76083E+00_JPRB, 0.78015E+00_JPRB, 0.79366E+00_JPRB ,&
&  0.80370E+00_JPRB, 0.80380E+00_JPRB, 0.79787E+00_JPRB, 0.79638E+00_JPRB, 0.79113E+00_JPRB, 0.77611E+00_JPRB, 0.77646E+00_JPRB ,&
&  0.80707E+00_JPRB, 0.83814E+00_JPRB, 0.85286E+00_JPRB, 0.86090E+00_JPRB, 0.92340E+00_JPRB, 0.97316E+00_JPRB, 0.10377E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.27790E+01_JPRB, 0.24203E+01_JPRB, 0.45778E+01_JPRB, 0.38908E+01_JPRB, 0.32617E+01_JPRB, 0.27716E+01_JPRB ,&
&  0.26551E+01_JPRB, 0.27418E+01_JPRB, 0.21721E+01_JPRB, 0.17809E+01_JPRB, 0.15933E+01_JPRB, 0.15879E+01_JPRB, 0.15598E+01_JPRB ,&
&  0.15277E+01_JPRB, 0.15898E+01_JPRB, 0.17217E+01_JPRB, 0.19257E+01_JPRB, 0.21723E+01_JPRB, 0.23898E+01_JPRB, 0.26437E+01_JPRB ,&
&  0.29514E+01_JPRB, 0.34707E+01_JPRB, 0.43075E+01_JPRB, 0.45240E+01_JPRB, 0.46653E+01_JPRB, 0.54988E+01_JPRB, 0.63950E+01_JPRB ,&
&  0.63027E+01_JPRB, 0.49582E+01_JPRB, 0.34254E+01_JPRB, 0.24498E+01_JPRB, 0.19896E+01_JPRB, 0.18523E+01_JPRB, 0.15897E+01_JPRB ,&
&  0.13840E+01_JPRB, 0.12821E+01_JPRB, 0.12421E+01_JPRB, 0.12278E+01_JPRB, 0.12383E+01_JPRB, 0.12418E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.13064E+01_JPRB, 0.13074E+01_JPRB, 0.12983E+01_JPRB, 0.12989E+01_JPRB, 0.13033E+01_JPRB, 0.12949E+01_JPRB ,&
&  0.12885E+01_JPRB, 0.12661E+01_JPRB, 0.11708E+01_JPRB, 0.11245E+01_JPRB, 0.11237E+01_JPRB, 0.11230E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.11235E+01_JPRB, 0.10835E+01_JPRB, 0.10299E+01_JPRB, 0.10315E+01_JPRB, 0.10426E+01_JPRB, 0.98388E+00_JPRB ,&
&  0.96475E+00_JPRB, 0.95020E+00_JPRB, 0.94197E+00_JPRB, 0.93377E+00_JPRB, 0.91923E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.91843E+00_JPRB, 0.90592E+00_JPRB, 0.90219E+00_JPRB, 0.88634E+00_JPRB, 0.85621E+00_JPRB, 0.82377E+00_JPRB, 0.80347E+00_JPRB ,&
&  0.78866E+00_JPRB, 0.78784E+00_JPRB, 0.79791E+00_JPRB, 0.79116E+00_JPRB, 0.79662E+00_JPRB, 0.81124E+00_JPRB, 0.82004E+00_JPRB ,&
&  0.81766E+00_JPRB, 0.81518E+00_JPRB, 0.80634E+00_JPRB, 0.80086E+00_JPRB, 0.79730E+00_JPRB, 0.78317E+00_JPRB, 0.77436E+00_JPRB ,&
&  0.77614E+00_JPRB, 0.78890E+00_JPRB, 0.80101E+00_JPRB, 0.81150E+00_JPRB, 0.84491E+00_JPRB, 0.87996E+00_JPRB, 0.93049E+00_JPRB ,&
&  0.11223E+01_JPRB, 0.30694E+01_JPRB, 0.22636E+01_JPRB, 0.42507E+01_JPRB, 0.37196E+01_JPRB, 0.30588E+01_JPRB, 0.25303E+01_JPRB ,&
&  0.24631E+01_JPRB, 0.24421E+01_JPRB, 0.21743E+01_JPRB, 0.19613E+01_JPRB, 0.17743E+01_JPRB, 0.16065E+01_JPRB, 0.14937E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.13390E+01_JPRB, 0.13377E+01_JPRB, 0.13884E+01_JPRB, 0.15285E+01_JPRB, 0.17093E+01_JPRB, 0.18846E+01_JPRB ,&
&  0.20743E+01_JPRB, 0.23803E+01_JPRB, 0.29632E+01_JPRB, 0.39261E+01_JPRB, 0.42531E+01_JPRB, 0.49754E+01_JPRB, 0.75611E+01_JPRB ,&
&  0.77967E+01_JPRB, 0.66074E+01_JPRB, 0.44305E+01_JPRB, 0.29050E+01_JPRB, 0.22984E+01_JPRB, 0.20537E+01_JPRB, 0.18197E+01_JPRB ,&
&  0.15970E+01_JPRB, 0.14260E+01_JPRB, 0.13628E+01_JPRB, 0.12992E+01_JPRB, 0.12723E+01_JPRB, 0.13042E+01_JPRB, 0.13553E+01_JPRB ,&
&  0.13822E+01_JPRB, 0.13703E+01_JPRB, 0.13644E+01_JPRB, 0.13203E+01_JPRB, 0.12872E+01_JPRB, 0.12621E+01_JPRB, 0.12655E+01_JPRB ,&
&  0.12801E+01_JPRB, 0.12837E+01_JPRB, 0.12194E+01_JPRB, 0.11569E+01_JPRB, 0.11415E+01_JPRB, 0.11438E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.11586E+01_JPRB, 0.11434E+01_JPRB, 0.10992E+01_JPRB, 0.10624E+01_JPRB, 0.10290E+01_JPRB, 0.10421E+01_JPRB, 0.97382E+00_JPRB ,&
&  0.94396E+00_JPRB, 0.93202E+00_JPRB, 0.93133E+00_JPRB, 0.93812E+00_JPRB, 0.93629E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.92801E+00_JPRB, 0.91198E+00_JPRB, 0.90845E+00_JPRB, 0.90315E+00_JPRB, 0.88860E+00_JPRB, 0.87054E+00_JPRB, 0.86426E+00_JPRB ,&
&  0.85648E+00_JPRB, 0.84968E+00_JPRB, 0.83793E+00_JPRB, 0.82041E+00_JPRB, 0.81991E+00_JPRB, 0.82108E+00_JPRB, 0.82027E+00_JPRB ,&
&  0.81691E+00_JPRB, 0.81459E+00_JPRB, 0.80151E+00_JPRB, 0.79458E+00_JPRB, 0.79576E+00_JPRB, 0.79046E+00_JPRB, 0.77806E+00_JPRB ,&
&  0.76610E+00_JPRB, 0.76375E+00_JPRB, 0.77518E+00_JPRB, 0.78871E+00_JPRB, 0.81641E+00_JPRB, 0.83268E+00_JPRB, 0.87480E+00_JPRB ,&
&  0.11136E+01_JPRB, 0.11192E+01_JPRB, 0.27371E+01_JPRB, 0.36655E+01_JPRB, 0.35441E+01_JPRB, 0.30117E+01_JPRB, 0.26419E+01_JPRB ,&
&  0.24682E+01_JPRB, 0.22785E+01_JPRB, 0.21328E+01_JPRB, 0.19403E+01_JPRB, 0.17351E+01_JPRB, 0.15173E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.12877E+01_JPRB, 0.12645E+01_JPRB, 0.12699E+01_JPRB, 0.12986E+01_JPRB, 0.13522E+01_JPRB, 0.14448E+01_JPRB ,&
&  0.15463E+01_JPRB, 0.16694E+01_JPRB, 0.19286E+01_JPRB, 0.26578E+01_JPRB, 0.38207E+01_JPRB, 0.47501E+01_JPRB, 0.66954E+01_JPRB ,&
&  0.84020E+01_JPRB, 0.77021E+01_JPRB, 0.53211E+01_JPRB, 0.34497E+01_JPRB, 0.26164E+01_JPRB, 0.22814E+01_JPRB, 0.20263E+01_JPRB ,&
&  0.18224E+01_JPRB, 0.16108E+01_JPRB, 0.14660E+01_JPRB, 0.13641E+01_JPRB, 0.13004E+01_JPRB, 0.12840E+01_JPRB, 0.13569E+01_JPRB ,&
&  0.14213E+01_JPRB, 0.14368E+01_JPRB, 0.13971E+01_JPRB, 0.13447E+01_JPRB, 0.13024E+01_JPRB, 0.12708E+01_JPRB, 0.12668E+01_JPRB ,&
&  0.12819E+01_JPRB, 0.12913E+01_JPRB, 0.12479E+01_JPRB, 0.11977E+01_JPRB, 0.11813E+01_JPRB, 0.11914E+01_JPRB, 0.11928E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.11424E+01_JPRB, 0.10925E+01_JPRB, 0.10660E+01_JPRB, 0.10068E+01_JPRB, 0.10254E+01_JPRB, 0.96336E+00_JPRB ,&
&  0.92962E+00_JPRB, 0.92113E+00_JPRB, 0.92796E+00_JPRB, 0.93473E+00_JPRB, 0.94984E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.93609E+00_JPRB, 0.91060E+00_JPRB, 0.90334E+00_JPRB, 0.90864E+00_JPRB, 0.90669E+00_JPRB, 0.91136E+00_JPRB, 0.91301E+00_JPRB ,&
&  0.90271E+00_JPRB, 0.87853E+00_JPRB, 0.85852E+00_JPRB, 0.83827E+00_JPRB, 0.83143E+00_JPRB, 0.82422E+00_JPRB, 0.82311E+00_JPRB ,&
&  0.81462E+00_JPRB, 0.80820E+00_JPRB, 0.79738E+00_JPRB, 0.79412E+00_JPRB, 0.79361E+00_JPRB, 0.78675E+00_JPRB, 0.76308E+00_JPRB ,&
&  0.75414E+00_JPRB, 0.75451E+00_JPRB, 0.76860E+00_JPRB, 0.78578E+00_JPRB, 0.80726E+00_JPRB, 0.81788E+00_JPRB, 0.84353E+00_JPRB ,&
&  0.10670E+01_JPRB, 0.17878E+01_JPRB, 0.28213E+01_JPRB, 0.29802E+01_JPRB, 0.29564E+01_JPRB, 0.26715E+01_JPRB, 0.24267E+01_JPRB ,&
&  0.21999E+01_JPRB, 0.21020E+01_JPRB, 0.20855E+01_JPRB, 0.18880E+01_JPRB, 0.15931E+01_JPRB, 0.13616E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.12274E+01_JPRB, 0.11937E+01_JPRB, 0.11781E+01_JPRB, 0.11823E+01_JPRB, 0.11874E+01_JPRB, 0.11970E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.12641E+01_JPRB, 0.13175E+01_JPRB, 0.14157E+01_JPRB, 0.17774E+01_JPRB, 0.31074E+01_JPRB, 0.36113E+01_JPRB, 0.57057E+01_JPRB ,&
&  0.83987E+01_JPRB, 0.74321E+01_JPRB, 0.56156E+01_JPRB, 0.39891E+01_JPRB, 0.30766E+01_JPRB, 0.26286E+01_JPRB, 0.22911E+01_JPRB ,&
&  0.20259E+01_JPRB, 0.18037E+01_JPRB, 0.16136E+01_JPRB, 0.14910E+01_JPRB, 0.13882E+01_JPRB, 0.13191E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.14089E+01_JPRB, 0.13864E+01_JPRB, 0.13452E+01_JPRB, 0.13102E+01_JPRB, 0.13011E+01_JPRB, 0.12958E+01_JPRB ,&
&  0.13013E+01_JPRB, 0.12992E+01_JPRB, 0.12643E+01_JPRB, 0.12065E+01_JPRB, 0.11909E+01_JPRB, 0.12085E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.10990E+01_JPRB, 0.10919E+01_JPRB, 0.10503E+01_JPRB, 0.10136E+01_JPRB, 0.10281E+01_JPRB, 0.95940E+00_JPRB ,&
&  0.91888E+00_JPRB, 0.90888E+00_JPRB, 0.92680E+00_JPRB, 0.94829E+00_JPRB, 0.97968E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.95917E+00_JPRB, 0.91685E+00_JPRB, 0.89484E+00_JPRB, 0.89745E+00_JPRB, 0.90734E+00_JPRB, 0.90762E+00_JPRB, 0.91290E+00_JPRB ,&
&  0.90302E+00_JPRB, 0.88224E+00_JPRB, 0.85910E+00_JPRB, 0.83937E+00_JPRB, 0.82882E+00_JPRB, 0.82155E+00_JPRB, 0.81590E+00_JPRB ,&
&  0.81033E+00_JPRB, 0.80403E+00_JPRB, 0.79483E+00_JPRB, 0.79701E+00_JPRB, 0.79890E+00_JPRB, 0.79613E+00_JPRB, 0.77072E+00_JPRB ,&
&  0.75586E+00_JPRB, 0.75771E+00_JPRB, 0.76852E+00_JPRB, 0.77785E+00_JPRB, 0.79194E+00_JPRB, 0.80655E+00_JPRB, 0.83051E+00_JPRB ,&
&  0.95449E+00_JPRB, 0.16007E+01_JPRB, 0.21845E+01_JPRB, 0.25572E+01_JPRB, 0.26641E+01_JPRB, 0.25356E+01_JPRB, 0.22903E+01_JPRB ,&
&  0.21016E+01_JPRB, 0.19945E+01_JPRB, 0.18952E+01_JPRB, 0.16531E+01_JPRB, 0.13974E+01_JPRB, 0.12563E+01_JPRB, 0.11839E+01_JPRB ,&
&  0.11512E+01_JPRB, 0.11417E+01_JPRB, 0.11308E+01_JPRB, 0.11234E+01_JPRB, 0.11159E+01_JPRB, 0.11191E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11609E+01_JPRB, 0.11957E+01_JPRB, 0.12641E+01_JPRB, 0.18297E+01_JPRB, 0.27273E+01_JPRB, 0.41255E+01_JPRB ,&
&  0.48226E+01_JPRB, 0.49175E+01_JPRB, 0.44296E+01_JPRB, 0.37983E+01_JPRB, 0.32590E+01_JPRB, 0.28384E+01_JPRB, 0.24747E+01_JPRB ,&
&  0.21448E+01_JPRB, 0.18559E+01_JPRB, 0.16223E+01_JPRB, 0.14724E+01_JPRB, 0.13783E+01_JPRB, 0.13056E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12885E+01_JPRB, 0.13132E+01_JPRB, 0.13216E+01_JPRB, 0.13197E+01_JPRB, 0.12997E+01_JPRB, 0.12769E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12559E+01_JPRB, 0.12517E+01_JPRB, 0.12188E+01_JPRB, 0.11837E+01_JPRB, 0.11594E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.10910E+01_JPRB, 0.10645E+01_JPRB, 0.10423E+01_JPRB, 0.10082E+01_JPRB, 0.10205E+01_JPRB, 0.95357E+00_JPRB ,&
&  0.91696E+00_JPRB, 0.90721E+00_JPRB, 0.91025E+00_JPRB, 0.92622E+00_JPRB, 0.96539E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.97349E+00_JPRB, 0.94308E+00_JPRB, 0.91315E+00_JPRB, 0.89717E+00_JPRB, 0.89267E+00_JPRB, 0.89245E+00_JPRB, 0.89271E+00_JPRB ,&
&  0.88911E+00_JPRB, 0.88012E+00_JPRB, 0.85691E+00_JPRB, 0.83808E+00_JPRB, 0.82417E+00_JPRB, 0.81839E+00_JPRB, 0.81331E+00_JPRB ,&
&  0.80998E+00_JPRB, 0.80578E+00_JPRB, 0.79835E+00_JPRB, 0.79626E+00_JPRB, 0.79746E+00_JPRB, 0.79126E+00_JPRB, 0.77102E+00_JPRB ,&
&  0.74928E+00_JPRB, 0.74917E+00_JPRB, 0.75842E+00_JPRB, 0.76764E+00_JPRB, 0.78136E+00_JPRB, 0.79422E+00_JPRB, 0.81884E+00_JPRB ,&
&  0.92863E+00_JPRB, 0.12777E+01_JPRB, 0.18412E+01_JPRB, 0.21504E+01_JPRB, 0.22820E+01_JPRB, 0.22935E+01_JPRB, 0.21423E+01_JPRB ,&
&  0.19509E+01_JPRB, 0.17519E+01_JPRB, 0.15904E+01_JPRB, 0.14197E+01_JPRB, 0.12981E+01_JPRB, 0.12059E+01_JPRB, 0.11430E+01_JPRB ,&
&  0.11098E+01_JPRB, 0.10985E+01_JPRB, 0.10878E+01_JPRB, 0.10746E+01_JPRB, 0.10652E+01_JPRB, 0.10605E+01_JPRB, 0.10601E+01_JPRB ,&
&  0.10656E+01_JPRB, 0.10792E+01_JPRB, 0.10992E+01_JPRB, 0.11229E+01_JPRB, 0.12045E+01_JPRB, 0.15320E+01_JPRB, 0.22764E+01_JPRB ,&
&  0.29455E+01_JPRB, 0.33300E+01_JPRB, 0.32775E+01_JPRB, 0.31416E+01_JPRB, 0.30146E+01_JPRB, 0.27455E+01_JPRB, 0.24587E+01_JPRB ,&
&  0.21749E+01_JPRB, 0.19130E+01_JPRB, 0.16493E+01_JPRB, 0.14394E+01_JPRB, 0.13057E+01_JPRB, 0.12289E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.11895E+01_JPRB, 0.12176E+01_JPRB, 0.12360E+01_JPRB, 0.12544E+01_JPRB, 0.12502E+01_JPRB, 0.12281E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.11688E+01_JPRB, 0.11382E+01_JPRB, 0.11227E+01_JPRB, 0.11021E+01_JPRB, 0.10795E+01_JPRB, 0.10678E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.10532E+01_JPRB, 0.10561E+01_JPRB, 0.10129E+01_JPRB, 0.99673E+00_JPRB, 0.10553E+01_JPRB, 0.10102E+01_JPRB, 0.96308E+00_JPRB ,&
&  0.92730E+00_JPRB, 0.91074E+00_JPRB, 0.90587E+00_JPRB, 0.91907E+00_JPRB, 0.96074E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.87448E+00_JPRB, 0.87369E+00_JPRB, 0.87218E+00_JPRB, 0.87555E+00_JPRB, 0.87741E+00_JPRB, 0.86982E+00_JPRB, 0.86359E+00_JPRB ,&
&  0.86204E+00_JPRB, 0.85707E+00_JPRB, 0.84940E+00_JPRB, 0.83714E+00_JPRB, 0.82133E+00_JPRB, 0.80884E+00_JPRB, 0.80406E+00_JPRB ,&
&  0.79718E+00_JPRB, 0.79369E+00_JPRB, 0.79193E+00_JPRB, 0.78638E+00_JPRB, 0.77980E+00_JPRB, 0.76987E+00_JPRB, 0.75258E+00_JPRB ,&
&  0.72985E+00_JPRB, 0.72146E+00_JPRB, 0.73150E+00_JPRB, 0.74250E+00_JPRB, 0.75176E+00_JPRB, 0.75740E+00_JPRB, 0.77796E+00_JPRB ,&
&  0.89257E+00_JPRB, 0.11074E+01_JPRB, 0.13526E+01_JPRB, 0.16472E+01_JPRB, 0.18209E+01_JPRB, 0.17743E+01_JPRB, 0.17296E+01_JPRB ,&
&  0.16380E+01_JPRB, 0.15399E+01_JPRB, 0.14423E+01_JPRB, 0.13344E+01_JPRB, 0.12406E+01_JPRB, 0.11672E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.10646E+01_JPRB, 0.10444E+01_JPRB, 0.10219E+01_JPRB, 0.10049E+01_JPRB, 0.99333E+00_JPRB, 0.99302E+00_JPRB ,&
&  0.99762E+00_JPRB, 0.10061E+01_JPRB, 0.10179E+01_JPRB, 0.10318E+01_JPRB, 0.10568E+01_JPRB, 0.11474E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.19009E+01_JPRB, 0.23071E+01_JPRB, 0.24993E+01_JPRB, 0.24623E+01_JPRB, 0.24026E+01_JPRB, 0.23148E+01_JPRB, 0.21070E+01_JPRB ,&
&  0.18536E+01_JPRB, 0.16850E+01_JPRB, 0.15561E+01_JPRB, 0.14444E+01_JPRB, 0.13604E+01_JPRB, 0.12891E+01_JPRB, 0.12340E+01_JPRB ,&
&  0.12017E+01_JPRB, 0.11941E+01_JPRB, 0.11922E+01_JPRB, 0.11899E+01_JPRB, 0.11778E+01_JPRB, 0.11450E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10057E+01_JPRB, 0.99792E+00_JPRB, 0.98150E+00_JPRB, 0.96649E+00_JPRB, 0.96585E+00_JPRB, 0.97552E+00_JPRB ,&
&  0.97783E+00_JPRB, 0.98535E+00_JPRB, 0.99025E+00_JPRB, 0.10151E+01_JPRB, 0.10077E+01_JPRB, 0.98127E+00_JPRB, 0.96420E+00_JPRB ,&
&  0.93685E+00_JPRB, 0.90880E+00_JPRB, 0.88784E+00_JPRB, 0.88347E+00_JPRB, 0.83638E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.85901E+00_JPRB, 0.85566E+00_JPRB, 0.84963E+00_JPRB, 0.85024E+00_JPRB, 0.85330E+00_JPRB, 0.84108E+00_JPRB, 0.82879E+00_JPRB ,&
&  0.82208E+00_JPRB, 0.82242E+00_JPRB, 0.82481E+00_JPRB, 0.82423E+00_JPRB, 0.81553E+00_JPRB, 0.80070E+00_JPRB, 0.79085E+00_JPRB ,&
&  0.77864E+00_JPRB, 0.76862E+00_JPRB, 0.76540E+00_JPRB, 0.76348E+00_JPRB, 0.75639E+00_JPRB, 0.74811E+00_JPRB, 0.74075E+00_JPRB ,&
&  0.72461E+00_JPRB, 0.71254E+00_JPRB, 0.70996E+00_JPRB, 0.71457E+00_JPRB, 0.72055E+00_JPRB, 0.72208E+00_JPRB, 0.75160E+00_JPRB ,&
&  0.88154E+00_JPRB, 0.92401E+00_JPRB, 0.94378E+00_JPRB, 0.11611E+01_JPRB, 0.13427E+01_JPRB, 0.13704E+01_JPRB, 0.13962E+01_JPRB ,&
&  0.14357E+01_JPRB, 0.14265E+01_JPRB, 0.13786E+01_JPRB, 0.12799E+01_JPRB, 0.11961E+01_JPRB, 0.11289E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.96813E+00_JPRB, 0.93858E+00_JPRB, 0.91478E+00_JPRB, 0.90235E+00_JPRB, 0.88597E+00_JPRB, 0.87907E+00_JPRB ,&
&  0.88090E+00_JPRB, 0.89207E+00_JPRB, 0.90595E+00_JPRB, 0.91877E+00_JPRB, 0.94071E+00_JPRB, 0.98202E+00_JPRB, 0.11030E+01_JPRB ,&
&  0.13323E+01_JPRB, 0.16517E+01_JPRB, 0.18859E+01_JPRB, 0.19630E+01_JPRB, 0.19331E+01_JPRB, 0.18404E+01_JPRB, 0.17045E+01_JPRB ,&
&  0.15578E+01_JPRB, 0.14401E+01_JPRB, 0.13560E+01_JPRB, 0.13097E+01_JPRB, 0.12739E+01_JPRB, 0.12438E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.11648E+01_JPRB, 0.11492E+01_JPRB, 0.11301E+01_JPRB, 0.11046E+01_JPRB, 0.10569E+01_JPRB, 0.99086E+00_JPRB ,&
&  0.93865E+00_JPRB, 0.90273E+00_JPRB, 0.88775E+00_JPRB, 0.87614E+00_JPRB, 0.87001E+00_JPRB, 0.88146E+00_JPRB, 0.89886E+00_JPRB ,&
&  0.90835E+00_JPRB, 0.91931E+00_JPRB, 0.94043E+00_JPRB, 0.91584E+00_JPRB, 0.95347E+00_JPRB, 0.95173E+00_JPRB, 0.95178E+00_JPRB ,&
&  0.93927E+00_JPRB, 0.91918E+00_JPRB, 0.89796E+00_JPRB, 0.77994E+00_JPRB, 0.84134E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.84505E+00_JPRB, 0.84044E+00_JPRB, 0.82674E+00_JPRB, 0.81760E+00_JPRB, 0.81133E+00_JPRB, 0.80330E+00_JPRB, 0.79016E+00_JPRB ,&
&  0.78244E+00_JPRB, 0.78076E+00_JPRB, 0.77989E+00_JPRB, 0.78008E+00_JPRB, 0.77904E+00_JPRB, 0.77125E+00_JPRB, 0.75976E+00_JPRB ,&
&  0.74870E+00_JPRB, 0.73943E+00_JPRB, 0.74050E+00_JPRB, 0.73884E+00_JPRB, 0.73226E+00_JPRB, 0.72678E+00_JPRB, 0.72015E+00_JPRB ,&
&  0.70466E+00_JPRB, 0.68784E+00_JPRB, 0.67501E+00_JPRB, 0.67847E+00_JPRB, 0.68577E+00_JPRB, 0.68211E+00_JPRB, 0.70823E+00_JPRB ,&
&  0.83095E+00_JPRB, 0.79868E+00_JPRB, 0.83670E+00_JPRB, 0.96068E+00_JPRB, 0.10385E+01_JPRB, 0.11102E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.12059E+01_JPRB, 0.12353E+01_JPRB, 0.11669E+01_JPRB, 0.10815E+01_JPRB, 0.99780E+00_JPRB, 0.92617E+00_JPRB, 0.87407E+00_JPRB ,&
&  0.83977E+00_JPRB, 0.82195E+00_JPRB, 0.80971E+00_JPRB, 0.79481E+00_JPRB, 0.78310E+00_JPRB, 0.77226E+00_JPRB, 0.76643E+00_JPRB ,&
&  0.76424E+00_JPRB, 0.76802E+00_JPRB, 0.77732E+00_JPRB, 0.79693E+00_JPRB, 0.81885E+00_JPRB, 0.84379E+00_JPRB, 0.88670E+00_JPRB ,&
&  0.97657E+00_JPRB, 0.11467E+01_JPRB, 0.13374E+01_JPRB, 0.14437E+01_JPRB, 0.14522E+01_JPRB, 0.13973E+01_JPRB, 0.13084E+01_JPRB ,&
&  0.12219E+01_JPRB, 0.11593E+01_JPRB, 0.11213E+01_JPRB, 0.10959E+01_JPRB, 0.10716E+01_JPRB, 0.10509E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10034E+01_JPRB, 0.99168E+00_JPRB, 0.97406E+00_JPRB, 0.94566E+00_JPRB, 0.90652E+00_JPRB, 0.86749E+00_JPRB ,&
&  0.83026E+00_JPRB, 0.80087E+00_JPRB, 0.78361E+00_JPRB, 0.78114E+00_JPRB, 0.78342E+00_JPRB, 0.79646E+00_JPRB, 0.81740E+00_JPRB ,&
&  0.83397E+00_JPRB, 0.84634E+00_JPRB, 0.85837E+00_JPRB, 0.88286E+00_JPRB, 0.89888E+00_JPRB, 0.91578E+00_JPRB, 0.92561E+00_JPRB ,&
&  0.92716E+00_JPRB, 0.92588E+00_JPRB, 0.75350E+00_JPRB, 0.82105E+00_JPRB, 0.83723E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.84007E+00_JPRB, 0.81987E+00_JPRB, 0.79601E+00_JPRB, 0.77908E+00_JPRB, 0.76982E+00_JPRB, 0.76551E+00_JPRB, 0.75994E+00_JPRB ,&
&  0.75255E+00_JPRB, 0.75154E+00_JPRB, 0.75222E+00_JPRB, 0.75267E+00_JPRB, 0.75211E+00_JPRB, 0.74694E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.72514E+00_JPRB, 0.71199E+00_JPRB, 0.70958E+00_JPRB, 0.70834E+00_JPRB, 0.70135E+00_JPRB, 0.69616E+00_JPRB, 0.69490E+00_JPRB ,&
&  0.68785E+00_JPRB, 0.67655E+00_JPRB, 0.65669E+00_JPRB, 0.65089E+00_JPRB, 0.65434E+00_JPRB, 0.65497E+00_JPRB, 0.66623E+00_JPRB ,&
&  0.68126E+00_JPRB, 0.69069E+00_JPRB, 0.76313E+00_JPRB, 0.81559E+00_JPRB, 0.84919E+00_JPRB, 0.88283E+00_JPRB, 0.91843E+00_JPRB ,&
&  0.91293E+00_JPRB, 0.91327E+00_JPRB, 0.87883E+00_JPRB, 0.82549E+00_JPRB, 0.78704E+00_JPRB, 0.76053E+00_JPRB, 0.74312E+00_JPRB ,&
&  0.73724E+00_JPRB, 0.73274E+00_JPRB, 0.72687E+00_JPRB, 0.72102E+00_JPRB, 0.71798E+00_JPRB, 0.71166E+00_JPRB, 0.70805E+00_JPRB ,&
&  0.70928E+00_JPRB, 0.71367E+00_JPRB, 0.71666E+00_JPRB, 0.72288E+00_JPRB, 0.73552E+00_JPRB, 0.75491E+00_JPRB, 0.77979E+00_JPRB ,&
&  0.81148E+00_JPRB, 0.86741E+00_JPRB, 0.96176E+00_JPRB, 0.10431E+01_JPRB, 0.10712E+01_JPRB, 0.10472E+01_JPRB, 0.99964E+00_JPRB ,&
&  0.95971E+00_JPRB, 0.93563E+00_JPRB, 0.91947E+00_JPRB, 0.90425E+00_JPRB, 0.88402E+00_JPRB, 0.86520E+00_JPRB, 0.84811E+00_JPRB ,&
&  0.84109E+00_JPRB, 0.82984E+00_JPRB, 0.81606E+00_JPRB, 0.79829E+00_JPRB, 0.78073E+00_JPRB, 0.76853E+00_JPRB, 0.75927E+00_JPRB ,&
&  0.74962E+00_JPRB, 0.73691E+00_JPRB, 0.72872E+00_JPRB, 0.72932E+00_JPRB, 0.73160E+00_JPRB, 0.73865E+00_JPRB, 0.75101E+00_JPRB ,&
&  0.76688E+00_JPRB, 0.78149E+00_JPRB, 0.79665E+00_JPRB, 0.82409E+00_JPRB, 0.85185E+00_JPRB, 0.87672E+00_JPRB, 0.89280E+00_JPRB ,&
&  0.90318E+00_JPRB, 0.89716E+00_JPRB, 0.89136E+00_JPRB, 0.87567E+00_JPRB, 0.85647E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.77407E+00_JPRB, 0.75042E+00_JPRB, 0.73154E+00_JPRB, 0.71875E+00_JPRB, 0.71109E+00_JPRB, 0.70889E+00_JPRB, 0.70510E+00_JPRB ,&
&  0.70193E+00_JPRB, 0.69732E+00_JPRB, 0.69452E+00_JPRB, 0.69706E+00_JPRB, 0.69829E+00_JPRB, 0.69841E+00_JPRB, 0.69844E+00_JPRB ,&
&  0.69130E+00_JPRB, 0.67676E+00_JPRB, 0.66533E+00_JPRB, 0.66170E+00_JPRB, 0.66185E+00_JPRB, 0.65704E+00_JPRB, 0.65509E+00_JPRB ,&
&  0.65808E+00_JPRB, 0.64983E+00_JPRB, 0.63578E+00_JPRB, 0.62667E+00_JPRB, 0.62752E+00_JPRB, 0.62531E+00_JPRB, 0.62539E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.62517E+00_JPRB, 0.68597E+00_JPRB, 0.69868E+00_JPRB, 0.71076E+00_JPRB, 0.71989E+00_JPRB, 0.72396E+00_JPRB ,&
&  0.72049E+00_JPRB, 0.71104E+00_JPRB, 0.70043E+00_JPRB, 0.68824E+00_JPRB, 0.67752E+00_JPRB, 0.66634E+00_JPRB, 0.65750E+00_JPRB ,&
&  0.65437E+00_JPRB, 0.65127E+00_JPRB, 0.65217E+00_JPRB, 0.65568E+00_JPRB, 0.66235E+00_JPRB, 0.66434E+00_JPRB, 0.66615E+00_JPRB ,&
&  0.66865E+00_JPRB, 0.67021E+00_JPRB, 0.66997E+00_JPRB, 0.66883E+00_JPRB, 0.66751E+00_JPRB, 0.67226E+00_JPRB, 0.68062E+00_JPRB ,&
&  0.69197E+00_JPRB, 0.71639E+00_JPRB, 0.75665E+00_JPRB, 0.79506E+00_JPRB, 0.81179E+00_JPRB, 0.80599E+00_JPRB, 0.78739E+00_JPRB ,&
&  0.77767E+00_JPRB, 0.77917E+00_JPRB, 0.78215E+00_JPRB, 0.77724E+00_JPRB, 0.76802E+00_JPRB, 0.75916E+00_JPRB, 0.74423E+00_JPRB ,&
&  0.72487E+00_JPRB, 0.70763E+00_JPRB, 0.69507E+00_JPRB, 0.68686E+00_JPRB, 0.68257E+00_JPRB, 0.67753E+00_JPRB, 0.67228E+00_JPRB ,&
&  0.67002E+00_JPRB, 0.66719E+00_JPRB, 0.66713E+00_JPRB, 0.67211E+00_JPRB, 0.67692E+00_JPRB, 0.68335E+00_JPRB, 0.69138E+00_JPRB ,&
&  0.70260E+00_JPRB, 0.71491E+00_JPRB, 0.72709E+00_JPRB, 0.74659E+00_JPRB, 0.77445E+00_JPRB, 0.79454E+00_JPRB, 0.80794E+00_JPRB ,&
&  0.82038E+00_JPRB, 0.82396E+00_JPRB, 0.81779E+00_JPRB, 0.80727E+00_JPRB, 0.79228E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.71865E+00_JPRB, 0.70452E+00_JPRB, 0.69335E+00_JPRB, 0.68447E+00_JPRB, 0.67831E+00_JPRB, 0.67642E+00_JPRB, 0.67367E+00_JPRB ,&
&  0.67085E+00_JPRB, 0.66800E+00_JPRB, 0.66032E+00_JPRB, 0.65325E+00_JPRB, 0.65302E+00_JPRB, 0.65922E+00_JPRB, 0.66373E+00_JPRB ,&
&  0.65999E+00_JPRB, 0.64816E+00_JPRB, 0.63715E+00_JPRB, 0.63432E+00_JPRB, 0.63659E+00_JPRB, 0.63849E+00_JPRB, 0.63775E+00_JPRB ,&
&  0.63931E+00_JPRB, 0.64056E+00_JPRB, 0.63222E+00_JPRB, 0.61450E+00_JPRB, 0.60664E+00_JPRB, 0.60230E+00_JPRB, 0.60908E+00_JPRB ,&
&  0.62664E+00_JPRB, 0.63903E+00_JPRB, 0.64850E+00_JPRB, 0.64413E+00_JPRB, 0.64341E+00_JPRB, 0.64068E+00_JPRB, 0.63445E+00_JPRB ,&
&  0.62872E+00_JPRB, 0.62193E+00_JPRB, 0.61669E+00_JPRB, 0.61307E+00_JPRB, 0.61120E+00_JPRB, 0.61041E+00_JPRB, 0.61213E+00_JPRB ,&
&  0.61035E+00_JPRB, 0.60638E+00_JPRB, 0.60780E+00_JPRB, 0.61435E+00_JPRB, 0.62311E+00_JPRB, 0.62841E+00_JPRB, 0.63330E+00_JPRB ,&
&  0.63733E+00_JPRB, 0.63948E+00_JPRB, 0.64220E+00_JPRB, 0.64256E+00_JPRB, 0.63835E+00_JPRB, 0.63730E+00_JPRB, 0.63707E+00_JPRB ,&
&  0.64007E+00_JPRB, 0.64829E+00_JPRB, 0.66707E+00_JPRB, 0.69038E+00_JPRB, 0.70014E+00_JPRB, 0.69334E+00_JPRB, 0.68323E+00_JPRB ,&
&  0.68224E+00_JPRB, 0.69025E+00_JPRB, 0.69946E+00_JPRB, 0.70540E+00_JPRB, 0.70574E+00_JPRB, 0.69729E+00_JPRB, 0.68553E+00_JPRB ,&
&  0.67239E+00_JPRB, 0.65894E+00_JPRB, 0.64656E+00_JPRB, 0.63823E+00_JPRB, 0.63128E+00_JPRB, 0.62570E+00_JPRB, 0.62204E+00_JPRB ,&
&  0.62038E+00_JPRB, 0.62018E+00_JPRB, 0.62359E+00_JPRB, 0.63096E+00_JPRB, 0.63632E+00_JPRB, 0.64612E+00_JPRB, 0.65757E+00_JPRB ,&
&  0.66899E+00_JPRB, 0.67941E+00_JPRB, 0.69162E+00_JPRB, 0.70899E+00_JPRB, 0.72877E+00_JPRB, 0.74327E+00_JPRB, 0.75467E+00_JPRB ,&
&  0.76190E+00_JPRB, 0.76068E+00_JPRB, 0.75217E+00_JPRB, 0.74242E+00_JPRB, 0.73188E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.66773E+00_JPRB, 0.65971E+00_JPRB, 0.65302E+00_JPRB, 0.64702E+00_JPRB, 0.64183E+00_JPRB, 0.63968E+00_JPRB, 0.63758E+00_JPRB ,&
&  0.63560E+00_JPRB, 0.63262E+00_JPRB, 0.63041E+00_JPRB, 0.62607E+00_JPRB, 0.62280E+00_JPRB, 0.62592E+00_JPRB, 0.63186E+00_JPRB ,&
&  0.63030E+00_JPRB, 0.62275E+00_JPRB, 0.61680E+00_JPRB, 0.61643E+00_JPRB, 0.61957E+00_JPRB, 0.62182E+00_JPRB, 0.62424E+00_JPRB ,&
&  0.62366E+00_JPRB, 0.62265E+00_JPRB, 0.62032E+00_JPRB, 0.60368E+00_JPRB, 0.58848E+00_JPRB, 0.58111E+00_JPRB, 0.57741E+00_JPRB ,&
&  0.58011E+00_JPRB, 0.59571E+00_JPRB, 0.57647E+00_JPRB, 0.58611E+00_JPRB, 0.58175E+00_JPRB, 0.57489E+00_JPRB, 0.57062E+00_JPRB ,&
&  0.56831E+00_JPRB, 0.56601E+00_JPRB, 0.56402E+00_JPRB, 0.56531E+00_JPRB, 0.56713E+00_JPRB, 0.56711E+00_JPRB, 0.56794E+00_JPRB ,&
&  0.56698E+00_JPRB, 0.56902E+00_JPRB, 0.57115E+00_JPRB, 0.57634E+00_JPRB, 0.58015E+00_JPRB, 0.58260E+00_JPRB, 0.58593E+00_JPRB ,&
&  0.59070E+00_JPRB, 0.59513E+00_JPRB, 0.59930E+00_JPRB, 0.60283E+00_JPRB, 0.60422E+00_JPRB, 0.60385E+00_JPRB, 0.59948E+00_JPRB ,&
&  0.59599E+00_JPRB, 0.59603E+00_JPRB, 0.60198E+00_JPRB, 0.61588E+00_JPRB, 0.62762E+00_JPRB, 0.62779E+00_JPRB, 0.62365E+00_JPRB ,&
&  0.62318E+00_JPRB, 0.62828E+00_JPRB, 0.63116E+00_JPRB, 0.63187E+00_JPRB, 0.63117E+00_JPRB, 0.62912E+00_JPRB, 0.62449E+00_JPRB ,&
&  0.61744E+00_JPRB, 0.61085E+00_JPRB, 0.60386E+00_JPRB, 0.59121E+00_JPRB, 0.58190E+00_JPRB, 0.58084E+00_JPRB, 0.58252E+00_JPRB ,&
&  0.58201E+00_JPRB, 0.58049E+00_JPRB, 0.58188E+00_JPRB, 0.58792E+00_JPRB, 0.58935E+00_JPRB, 0.59827E+00_JPRB, 0.61397E+00_JPRB ,&
&  0.62940E+00_JPRB, 0.64249E+00_JPRB, 0.65268E+00_JPRB, 0.66310E+00_JPRB, 0.67331E+00_JPRB, 0.68095E+00_JPRB, 0.68723E+00_JPRB ,&
&  0.68978E+00_JPRB, 0.68747E+00_JPRB, 0.68153E+00_JPRB, 0.67811E+00_JPRB, 0.67520E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.63756E+00_JPRB, 0.63578E+00_JPRB, 0.63369E+00_JPRB, 0.63090E+00_JPRB, 0.62698E+00_JPRB, 0.62349E+00_JPRB, 0.61838E+00_JPRB ,&
&  0.61341E+00_JPRB, 0.60960E+00_JPRB, 0.60796E+00_JPRB, 0.60723E+00_JPRB, 0.60842E+00_JPRB, 0.61280E+00_JPRB, 0.62001E+00_JPRB ,&
&  0.61998E+00_JPRB, 0.61256E+00_JPRB, 0.60880E+00_JPRB, 0.60861E+00_JPRB, 0.61084E+00_JPRB, 0.61239E+00_JPRB, 0.61472E+00_JPRB ,&
&  0.61603E+00_JPRB, 0.61617E+00_JPRB, 0.61456E+00_JPRB, 0.60623E+00_JPRB, 0.58850E+00_JPRB, 0.57808E+00_JPRB, 0.56935E+00_JPRB ,&
&  0.56484E+00_JPRB, 0.56288E+00_JPRB, 0.55885E+00_JPRB, 0.55768E+00_JPRB, 0.55631E+00_JPRB, 0.55439E+00_JPRB, 0.55434E+00_JPRB ,&
&  0.55262E+00_JPRB, 0.55034E+00_JPRB, 0.54935E+00_JPRB, 0.55013E+00_JPRB, 0.55205E+00_JPRB, 0.55360E+00_JPRB, 0.55322E+00_JPRB ,&
&  0.55383E+00_JPRB, 0.55716E+00_JPRB, 0.55765E+00_JPRB, 0.55726E+00_JPRB, 0.55658E+00_JPRB, 0.55568E+00_JPRB, 0.55547E+00_JPRB ,&
&  0.55782E+00_JPRB, 0.56325E+00_JPRB, 0.56816E+00_JPRB, 0.57331E+00_JPRB, 0.57582E+00_JPRB, 0.57702E+00_JPRB, 0.57694E+00_JPRB ,&
&  0.57342E+00_JPRB, 0.56894E+00_JPRB, 0.56727E+00_JPRB, 0.57068E+00_JPRB, 0.57778E+00_JPRB, 0.58391E+00_JPRB, 0.58594E+00_JPRB ,&
&  0.58742E+00_JPRB, 0.58848E+00_JPRB, 0.58688E+00_JPRB, 0.58660E+00_JPRB, 0.58485E+00_JPRB, 0.58104E+00_JPRB, 0.57896E+00_JPRB ,&
&  0.57675E+00_JPRB, 0.57427E+00_JPRB, 0.57153E+00_JPRB, 0.56207E+00_JPRB, 0.55462E+00_JPRB, 0.55074E+00_JPRB, 0.54847E+00_JPRB ,&
&  0.54805E+00_JPRB, 0.54749E+00_JPRB, 0.54748E+00_JPRB, 0.54866E+00_JPRB, 0.55109E+00_JPRB, 0.55932E+00_JPRB, 0.57482E+00_JPRB ,&
&  0.59659E+00_JPRB, 0.61635E+00_JPRB, 0.62675E+00_JPRB, 0.63374E+00_JPRB, 0.63787E+00_JPRB, 0.63968E+00_JPRB, 0.64155E+00_JPRB ,&
&  0.64475E+00_JPRB, 0.64575E+00_JPRB, 0.64358E+00_JPRB, 0.64100E+00_JPRB, 0.63963E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.60647E+00_JPRB, 0.60651E+00_JPRB, 0.60471E+00_JPRB, 0.60285E+00_JPRB, 0.60087E+00_JPRB, 0.59912E+00_JPRB, 0.59687E+00_JPRB ,&
&  0.59196E+00_JPRB, 0.58558E+00_JPRB, 0.58188E+00_JPRB, 0.58053E+00_JPRB, 0.58663E+00_JPRB, 0.59529E+00_JPRB, 0.60291E+00_JPRB ,&
&  0.60164E+00_JPRB, 0.59175E+00_JPRB, 0.58852E+00_JPRB, 0.59029E+00_JPRB, 0.59274E+00_JPRB, 0.59538E+00_JPRB, 0.59890E+00_JPRB ,&
&  0.60283E+00_JPRB, 0.60673E+00_JPRB, 0.60840E+00_JPRB, 0.60508E+00_JPRB, 0.59755E+00_JPRB, 0.58308E+00_JPRB, 0.57088E+00_JPRB ,&
&  0.56301E+00_JPRB, 0.55705E+00_JPRB, 0.55261E+00_JPRB, 0.55128E+00_JPRB, 0.54883E+00_JPRB, 0.54504E+00_JPRB, 0.53995E+00_JPRB ,&
&  0.53581E+00_JPRB, 0.53419E+00_JPRB, 0.53713E+00_JPRB, 0.53951E+00_JPRB, 0.54065E+00_JPRB, 0.54005E+00_JPRB, 0.53649E+00_JPRB ,&
&  0.53409E+00_JPRB, 0.53277E+00_JPRB, 0.53157E+00_JPRB, 0.53215E+00_JPRB, 0.52877E+00_JPRB, 0.52602E+00_JPRB, 0.52752E+00_JPRB ,&
&  0.53185E+00_JPRB, 0.53600E+00_JPRB, 0.54007E+00_JPRB, 0.54368E+00_JPRB, 0.54565E+00_JPRB, 0.54829E+00_JPRB, 0.54964E+00_JPRB ,&
&  0.54824E+00_JPRB, 0.54392E+00_JPRB, 0.53987E+00_JPRB, 0.53989E+00_JPRB, 0.54185E+00_JPRB, 0.54421E+00_JPRB, 0.54737E+00_JPRB ,&
&  0.54761E+00_JPRB, 0.54233E+00_JPRB, 0.53722E+00_JPRB, 0.53285E+00_JPRB, 0.52899E+00_JPRB, 0.52628E+00_JPRB, 0.52520E+00_JPRB ,&
&  0.52523E+00_JPRB, 0.52209E+00_JPRB, 0.51597E+00_JPRB, 0.51105E+00_JPRB, 0.51102E+00_JPRB, 0.51150E+00_JPRB, 0.50868E+00_JPRB ,&
&  0.51014E+00_JPRB, 0.51350E+00_JPRB, 0.51272E+00_JPRB, 0.51081E+00_JPRB, 0.51278E+00_JPRB, 0.51881E+00_JPRB, 0.53156E+00_JPRB ,&
&  0.54846E+00_JPRB, 0.56749E+00_JPRB, 0.58482E+00_JPRB, 0.59911E+00_JPRB, 0.60719E+00_JPRB, 0.60888E+00_JPRB, 0.60694E+00_JPRB ,&
&  0.60636E+00_JPRB, 0.60743E+00_JPRB, 0.60683E+00_JPRB, 0.60543E+00_JPRB, 0.60564E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.57515E+00_JPRB, 0.57485E+00_JPRB, 0.57479E+00_JPRB, 0.57574E+00_JPRB, 0.57636E+00_JPRB, 0.57516E+00_JPRB, 0.57470E+00_JPRB ,&
&  0.57138E+00_JPRB, 0.56667E+00_JPRB, 0.56410E+00_JPRB, 0.56632E+00_JPRB, 0.57637E+00_JPRB, 0.58853E+00_JPRB, 0.59545E+00_JPRB ,&
&  0.59014E+00_JPRB, 0.57674E+00_JPRB, 0.57339E+00_JPRB, 0.57565E+00_JPRB, 0.57794E+00_JPRB, 0.58204E+00_JPRB, 0.58800E+00_JPRB ,&
&  0.59348E+00_JPRB, 0.59719E+00_JPRB, 0.60130E+00_JPRB, 0.60291E+00_JPRB, 0.60267E+00_JPRB, 0.59499E+00_JPRB, 0.58346E+00_JPRB ,&
&  0.57243E+00_JPRB, 0.56427E+00_JPRB, 0.55831E+00_JPRB, 0.55438E+00_JPRB, 0.54913E+00_JPRB, 0.54419E+00_JPRB, 0.53788E+00_JPRB ,&
&  0.53522E+00_JPRB, 0.53552E+00_JPRB, 0.53700E+00_JPRB, 0.53529E+00_JPRB, 0.53251E+00_JPRB, 0.52654E+00_JPRB, 0.52117E+00_JPRB ,&
&  0.51843E+00_JPRB, 0.51863E+00_JPRB, 0.51874E+00_JPRB, 0.51660E+00_JPRB, 0.51193E+00_JPRB, 0.50834E+00_JPRB, 0.50891E+00_JPRB ,&
&  0.51439E+00_JPRB, 0.51398E+00_JPRB, 0.51294E+00_JPRB, 0.51431E+00_JPRB, 0.51713E+00_JPRB, 0.51944E+00_JPRB, 0.51873E+00_JPRB ,&
&  0.51768E+00_JPRB, 0.51697E+00_JPRB, 0.51653E+00_JPRB, 0.51779E+00_JPRB, 0.51746E+00_JPRB, 0.51489E+00_JPRB, 0.50888E+00_JPRB ,&
&  0.50001E+00_JPRB, 0.48912E+00_JPRB, 0.48475E+00_JPRB, 0.48160E+00_JPRB, 0.47864E+00_JPRB, 0.47837E+00_JPRB, 0.48027E+00_JPRB ,&
&  0.47918E+00_JPRB, 0.47307E+00_JPRB, 0.47051E+00_JPRB, 0.47174E+00_JPRB, 0.47209E+00_JPRB, 0.47576E+00_JPRB, 0.48113E+00_JPRB ,&
&  0.49317E+00_JPRB, 0.50025E+00_JPRB, 0.50240E+00_JPRB, 0.50013E+00_JPRB, 0.50023E+00_JPRB, 0.50201E+00_JPRB, 0.50681E+00_JPRB ,&
&  0.51399E+00_JPRB, 0.52190E+00_JPRB, 0.53027E+00_JPRB, 0.53955E+00_JPRB, 0.55059E+00_JPRB, 0.56099E+00_JPRB, 0.56519E+00_JPRB ,&
&  0.56583E+00_JPRB, 0.56675E+00_JPRB, 0.56859E+00_JPRB, 0.57062E+00_JPRB, 0.57383E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.53729E+00_JPRB, 0.53956E+00_JPRB, 0.54196E+00_JPRB, 0.54369E+00_JPRB, 0.54659E+00_JPRB, 0.55175E+00_JPRB, 0.55383E+00_JPRB ,&
&  0.55168E+00_JPRB, 0.54923E+00_JPRB, 0.54914E+00_JPRB, 0.55308E+00_JPRB, 0.56077E+00_JPRB, 0.56848E+00_JPRB, 0.57634E+00_JPRB ,&
&  0.57144E+00_JPRB, 0.55772E+00_JPRB, 0.55285E+00_JPRB, 0.55500E+00_JPRB, 0.56005E+00_JPRB, 0.56556E+00_JPRB, 0.57218E+00_JPRB ,&
&  0.57843E+00_JPRB, 0.58238E+00_JPRB, 0.58661E+00_JPRB, 0.59055E+00_JPRB, 0.59303E+00_JPRB, 0.59091E+00_JPRB, 0.58622E+00_JPRB ,&
&  0.58057E+00_JPRB, 0.57435E+00_JPRB, 0.56830E+00_JPRB, 0.55276E+00_JPRB, 0.55028E+00_JPRB, 0.54148E+00_JPRB, 0.53439E+00_JPRB ,&
&  0.52973E+00_JPRB, 0.52579E+00_JPRB, 0.52314E+00_JPRB, 0.51970E+00_JPRB, 0.51461E+00_JPRB, 0.50612E+00_JPRB, 0.50279E+00_JPRB ,&
&  0.50058E+00_JPRB, 0.50043E+00_JPRB, 0.49447E+00_JPRB, 0.49081E+00_JPRB, 0.48673E+00_JPRB, 0.48335E+00_JPRB, 0.48658E+00_JPRB ,&
&  0.49108E+00_JPRB, 0.48796E+00_JPRB, 0.48362E+00_JPRB, 0.48348E+00_JPRB, 0.48808E+00_JPRB, 0.48982E+00_JPRB, 0.49062E+00_JPRB ,&
&  0.49202E+00_JPRB, 0.49183E+00_JPRB, 0.48626E+00_JPRB, 0.48044E+00_JPRB, 0.47191E+00_JPRB, 0.46011E+00_JPRB, 0.45625E+00_JPRB ,&
&  0.45284E+00_JPRB, 0.44745E+00_JPRB, 0.44501E+00_JPRB, 0.44426E+00_JPRB, 0.44413E+00_JPRB, 0.44549E+00_JPRB, 0.44619E+00_JPRB ,&
&  0.44688E+00_JPRB, 0.44354E+00_JPRB, 0.43870E+00_JPRB, 0.43494E+00_JPRB, 0.42972E+00_JPRB, 0.43392E+00_JPRB, 0.44563E+00_JPRB ,&
&  0.45940E+00_JPRB, 0.46804E+00_JPRB, 0.47362E+00_JPRB, 0.47709E+00_JPRB, 0.48015E+00_JPRB, 0.48135E+00_JPRB, 0.48366E+00_JPRB ,&
&  0.48671E+00_JPRB, 0.48814E+00_JPRB, 0.48715E+00_JPRB, 0.48866E+00_JPRB, 0.49325E+00_JPRB, 0.49966E+00_JPRB, 0.50695E+00_JPRB ,&
&  0.51389E+00_JPRB, 0.51965E+00_JPRB, 0.52441E+00_JPRB, 0.53020E+00_JPRB, 0.53397E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.50570E+00_JPRB, 0.51033E+00_JPRB, 0.51490E+00_JPRB, 0.51984E+00_JPRB, 0.52673E+00_JPRB, 0.53059E+00_JPRB, 0.53199E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53359E+00_JPRB, 0.53475E+00_JPRB, 0.53902E+00_JPRB, 0.54577E+00_JPRB, 0.55107E+00_JPRB, 0.55750E+00_JPRB ,&
&  0.55749E+00_JPRB, 0.54779E+00_JPRB, 0.53943E+00_JPRB, 0.53923E+00_JPRB, 0.54507E+00_JPRB, 0.55106E+00_JPRB, 0.55836E+00_JPRB ,&
&  0.56546E+00_JPRB, 0.57041E+00_JPRB, 0.57306E+00_JPRB, 0.57436E+00_JPRB, 0.57607E+00_JPRB, 0.57524E+00_JPRB, 0.57191E+00_JPRB ,&
&  0.56706E+00_JPRB, 0.55671E+00_JPRB, 0.53707E+00_JPRB, 0.53466E+00_JPRB, 0.52589E+00_JPRB, 0.51841E+00_JPRB, 0.51330E+00_JPRB ,&
&  0.50926E+00_JPRB, 0.50565E+00_JPRB, 0.50131E+00_JPRB, 0.49707E+00_JPRB, 0.49207E+00_JPRB, 0.48370E+00_JPRB, 0.47883E+00_JPRB ,&
&  0.47428E+00_JPRB, 0.47065E+00_JPRB, 0.46682E+00_JPRB, 0.46593E+00_JPRB, 0.46519E+00_JPRB, 0.46695E+00_JPRB, 0.46749E+00_JPRB ,&
&  0.46349E+00_JPRB, 0.45895E+00_JPRB, 0.45559E+00_JPRB, 0.45710E+00_JPRB, 0.45964E+00_JPRB, 0.46138E+00_JPRB, 0.46196E+00_JPRB ,&
&  0.45878E+00_JPRB, 0.45299E+00_JPRB, 0.44688E+00_JPRB, 0.44017E+00_JPRB, 0.43202E+00_JPRB, 0.40634E+00_JPRB, 0.39577E+00_JPRB ,&
&  0.39409E+00_JPRB, 0.40395E+00_JPRB, 0.41512E+00_JPRB, 0.41506E+00_JPRB, 0.41935E+00_JPRB, 0.42017E+00_JPRB, 0.41760E+00_JPRB ,&
&  0.40724E+00_JPRB, 0.39425E+00_JPRB, 0.38680E+00_JPRB, 0.38383E+00_JPRB, 0.38182E+00_JPRB, 0.38510E+00_JPRB, 0.39285E+00_JPRB ,&
&  0.40464E+00_JPRB, 0.41679E+00_JPRB, 0.42574E+00_JPRB, 0.42969E+00_JPRB, 0.42815E+00_JPRB, 0.42506E+00_JPRB, 0.42097E+00_JPRB ,&
&  0.41730E+00_JPRB, 0.42462E+00_JPRB, 0.44073E+00_JPRB, 0.46034E+00_JPRB, 0.46314E+00_JPRB, 0.46933E+00_JPRB, 0.47364E+00_JPRB ,&
&  0.47967E+00_JPRB, 0.48723E+00_JPRB, 0.49336E+00_JPRB, 0.49936E+00_JPRB, 0.50284E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.48077E+00_JPRB, 0.48602E+00_JPRB, 0.49311E+00_JPRB, 0.50100E+00_JPRB, 0.50680E+00_JPRB, 0.50990E+00_JPRB, 0.51323E+00_JPRB ,&
&  0.51711E+00_JPRB, 0.51882E+00_JPRB, 0.51988E+00_JPRB, 0.52254E+00_JPRB, 0.52827E+00_JPRB, 0.53189E+00_JPRB, 0.53744E+00_JPRB ,&
&  0.53988E+00_JPRB, 0.53745E+00_JPRB, 0.53022E+00_JPRB, 0.52837E+00_JPRB, 0.53234E+00_JPRB, 0.53740E+00_JPRB, 0.54257E+00_JPRB ,&
&  0.54767E+00_JPRB, 0.55226E+00_JPRB, 0.55427E+00_JPRB, 0.55446E+00_JPRB, 0.55424E+00_JPRB, 0.55372E+00_JPRB, 0.55076E+00_JPRB ,&
&  0.54365E+00_JPRB, 0.50218E+00_JPRB, 0.47598E+00_JPRB, 0.48273E+00_JPRB, 0.49330E+00_JPRB, 0.48594E+00_JPRB, 0.48435E+00_JPRB ,&
&  0.48284E+00_JPRB, 0.47872E+00_JPRB, 0.47269E+00_JPRB, 0.46746E+00_JPRB, 0.46130E+00_JPRB, 0.45318E+00_JPRB, 0.44690E+00_JPRB ,&
&  0.44484E+00_JPRB, 0.45044E+00_JPRB, 0.45154E+00_JPRB, 0.45211E+00_JPRB, 0.44881E+00_JPRB, 0.44898E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.43897E+00_JPRB, 0.43104E+00_JPRB, 0.42781E+00_JPRB, 0.42691E+00_JPRB, 0.42811E+00_JPRB, 0.43093E+00_JPRB, 0.42959E+00_JPRB ,&
&  0.41704E+00_JPRB, 0.40513E+00_JPRB, 0.38592E+00_JPRB, 0.36723E+00_JPRB, 0.35391E+00_JPRB, 0.34808E+00_JPRB, 0.34393E+00_JPRB ,&
&  0.34375E+00_JPRB, 0.35137E+00_JPRB, 0.37139E+00_JPRB, 0.39468E+00_JPRB, 0.40457E+00_JPRB, 0.38571E+00_JPRB, 0.37119E+00_JPRB ,&
&  0.36273E+00_JPRB, 0.35804E+00_JPRB, 0.35385E+00_JPRB, 0.35285E+00_JPRB, 0.35398E+00_JPRB, 0.35705E+00_JPRB, 0.36117E+00_JPRB ,&
&  0.36689E+00_JPRB, 0.37204E+00_JPRB, 0.37486E+00_JPRB, 0.37619E+00_JPRB, 0.37705E+00_JPRB, 0.37804E+00_JPRB, 0.37752E+00_JPRB ,&
&  0.37786E+00_JPRB, 0.38258E+00_JPRB, 0.39092E+00_JPRB, 0.40089E+00_JPRB, 0.40962E+00_JPRB, 0.41975E+00_JPRB, 0.43381E+00_JPRB ,&
&  0.44876E+00_JPRB, 0.45919E+00_JPRB, 0.46323E+00_JPRB, 0.47015E+00_JPRB, 0.47586E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.46835E+00_JPRB, 0.47611E+00_JPRB, 0.48087E+00_JPRB, 0.49050E+00_JPRB, 0.49376E+00_JPRB, 0.49752E+00_JPRB, 0.50103E+00_JPRB ,&
&  0.50830E+00_JPRB, 0.51222E+00_JPRB, 0.51030E+00_JPRB, 0.51002E+00_JPRB, 0.50922E+00_JPRB, 0.50741E+00_JPRB, 0.50888E+00_JPRB ,&
&  0.50950E+00_JPRB, 0.51202E+00_JPRB, 0.50938E+00_JPRB, 0.50677E+00_JPRB, 0.50892E+00_JPRB, 0.51404E+00_JPRB, 0.51739E+00_JPRB ,&
&  0.51161E+00_JPRB, 0.50934E+00_JPRB, 0.51754E+00_JPRB, 0.51886E+00_JPRB, 0.51931E+00_JPRB, 0.52072E+00_JPRB, 0.52001E+00_JPRB ,&
&  0.50501E+00_JPRB, 0.47776E+00_JPRB, 0.45018E+00_JPRB, 0.45446E+00_JPRB, 0.46946E+00_JPRB, 0.46088E+00_JPRB, 0.45888E+00_JPRB ,&
&  0.45601E+00_JPRB, 0.45112E+00_JPRB, 0.44663E+00_JPRB, 0.44403E+00_JPRB, 0.44124E+00_JPRB, 0.43586E+00_JPRB, 0.43441E+00_JPRB ,&
&  0.43271E+00_JPRB, 0.44295E+00_JPRB, 0.43581E+00_JPRB, 0.41014E+00_JPRB, 0.39274E+00_JPRB, 0.38293E+00_JPRB, 0.37006E+00_JPRB ,&
&  0.35470E+00_JPRB, 0.34681E+00_JPRB, 0.34616E+00_JPRB, 0.34343E+00_JPRB, 0.34422E+00_JPRB, 0.34542E+00_JPRB, 0.34652E+00_JPRB ,&
&  0.34684E+00_JPRB, 0.34610E+00_JPRB, 0.34097E+00_JPRB, 0.33443E+00_JPRB, 0.33093E+00_JPRB, 0.33109E+00_JPRB, 0.33464E+00_JPRB ,&
&  0.34193E+00_JPRB, 0.34893E+00_JPRB, 0.36496E+00_JPRB, 0.37292E+00_JPRB, 0.36276E+00_JPRB, 0.35905E+00_JPRB, 0.35486E+00_JPRB ,&
&  0.34257E+00_JPRB, 0.33820E+00_JPRB, 0.33456E+00_JPRB, 0.33262E+00_JPRB, 0.33375E+00_JPRB, 0.33784E+00_JPRB, 0.33954E+00_JPRB ,&
&  0.34253E+00_JPRB, 0.34607E+00_JPRB, 0.34756E+00_JPRB, 0.34866E+00_JPRB, 0.34962E+00_JPRB, 0.35206E+00_JPRB, 0.35455E+00_JPRB ,&
&  0.35641E+00_JPRB, 0.35990E+00_JPRB, 0.36405E+00_JPRB, 0.36943E+00_JPRB, 0.37415E+00_JPRB, 0.37678E+00_JPRB, 0.38581E+00_JPRB ,&
&  0.39573E+00_JPRB, 0.41574E+00_JPRB, 0.44761E+00_JPRB, 0.47089E+00_JPRB, 0.46265E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.46850E+00_JPRB, 0.47261E+00_JPRB, 0.47261E+00_JPRB, 0.47586E+00_JPRB, 0.48174E+00_JPRB, 0.48839E+00_JPRB, 0.49424E+00_JPRB ,&
&  0.49994E+00_JPRB, 0.49963E+00_JPRB, 0.49465E+00_JPRB, 0.48632E+00_JPRB, 0.47953E+00_JPRB, 0.46284E+00_JPRB, 0.45463E+00_JPRB ,&
&  0.44768E+00_JPRB, 0.44812E+00_JPRB, 0.44846E+00_JPRB, 0.45037E+00_JPRB, 0.45121E+00_JPRB, 0.45639E+00_JPRB, 0.45756E+00_JPRB ,&
&  0.45429E+00_JPRB, 0.44855E+00_JPRB, 0.44258E+00_JPRB, 0.44128E+00_JPRB, 0.44392E+00_JPRB, 0.44555E+00_JPRB, 0.44198E+00_JPRB ,&
&  0.43817E+00_JPRB, 0.43808E+00_JPRB, 0.43850E+00_JPRB, 0.43992E+00_JPRB, 0.44924E+00_JPRB, 0.44225E+00_JPRB, 0.43737E+00_JPRB ,&
&  0.43030E+00_JPRB, 0.42443E+00_JPRB, 0.42066E+00_JPRB, 0.42118E+00_JPRB, 0.42523E+00_JPRB, 0.42291E+00_JPRB, 0.42606E+00_JPRB ,&
&  0.41596E+00_JPRB, 0.40745E+00_JPRB, 0.39159E+00_JPRB, 0.37850E+00_JPRB, 0.36335E+00_JPRB, 0.35304E+00_JPRB, 0.34378E+00_JPRB ,&
&  0.33598E+00_JPRB, 0.32913E+00_JPRB, 0.32281E+00_JPRB, 0.32099E+00_JPRB, 0.32206E+00_JPRB, 0.32176E+00_JPRB, 0.32074E+00_JPRB ,&
&  0.32100E+00_JPRB, 0.32187E+00_JPRB, 0.32254E+00_JPRB, 0.32368E+00_JPRB, 0.32571E+00_JPRB, 0.32891E+00_JPRB, 0.33335E+00_JPRB ,&
&  0.34011E+00_JPRB, 0.34530E+00_JPRB, 0.34800E+00_JPRB, 0.34986E+00_JPRB, 0.34661E+00_JPRB, 0.34063E+00_JPRB, 0.33503E+00_JPRB ,&
&  0.32885E+00_JPRB, 0.32604E+00_JPRB, 0.32331E+00_JPRB, 0.32200E+00_JPRB, 0.32164E+00_JPRB, 0.32257E+00_JPRB, 0.32474E+00_JPRB ,&
&  0.32694E+00_JPRB, 0.32995E+00_JPRB, 0.33192E+00_JPRB, 0.33422E+00_JPRB, 0.33689E+00_JPRB, 0.33917E+00_JPRB, 0.34149E+00_JPRB ,&
&  0.34411E+00_JPRB, 0.34618E+00_JPRB, 0.34982E+00_JPRB, 0.35438E+00_JPRB, 0.36331E+00_JPRB, 0.37297E+00_JPRB, 0.38994E+00_JPRB ,&
&  0.40892E+00_JPRB, 0.43933E+00_JPRB, 0.44616E+00_JPRB, 0.45834E+00_JPRB, 0.46756E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.46274E+00_JPRB, 0.46530E+00_JPRB, 0.46886E+00_JPRB, 0.47470E+00_JPRB, 0.48216E+00_JPRB, 0.48934E+00_JPRB, 0.49132E+00_JPRB ,&
&  0.48118E+00_JPRB, 0.46838E+00_JPRB, 0.44968E+00_JPRB, 0.43530E+00_JPRB, 0.42082E+00_JPRB, 0.41028E+00_JPRB, 0.40197E+00_JPRB ,&
&  0.39605E+00_JPRB, 0.39595E+00_JPRB, 0.39591E+00_JPRB, 0.39710E+00_JPRB, 0.39546E+00_JPRB, 0.39617E+00_JPRB, 0.39595E+00_JPRB ,&
&  0.39412E+00_JPRB, 0.39385E+00_JPRB, 0.39377E+00_JPRB, 0.39707E+00_JPRB, 0.40398E+00_JPRB, 0.41012E+00_JPRB, 0.41617E+00_JPRB ,&
&  0.42340E+00_JPRB, 0.42468E+00_JPRB, 0.42856E+00_JPRB, 0.42666E+00_JPRB, 0.42318E+00_JPRB, 0.41876E+00_JPRB, 0.41246E+00_JPRB ,&
&  0.40877E+00_JPRB, 0.40528E+00_JPRB, 0.40465E+00_JPRB, 0.40503E+00_JPRB, 0.40504E+00_JPRB, 0.40979E+00_JPRB, 0.40525E+00_JPRB ,&
&  0.39840E+00_JPRB, 0.38715E+00_JPRB, 0.37300E+00_JPRB, 0.36218E+00_JPRB, 0.35111E+00_JPRB, 0.34322E+00_JPRB, 0.33621E+00_JPRB ,&
&  0.32981E+00_JPRB, 0.32556E+00_JPRB, 0.32106E+00_JPRB, 0.31769E+00_JPRB, 0.31658E+00_JPRB, 0.31417E+00_JPRB, 0.31273E+00_JPRB ,&
&  0.31208E+00_JPRB, 0.31248E+00_JPRB, 0.31466E+00_JPRB, 0.31656E+00_JPRB, 0.31923E+00_JPRB, 0.32209E+00_JPRB, 0.32443E+00_JPRB ,&
&  0.32763E+00_JPRB, 0.33025E+00_JPRB, 0.33059E+00_JPRB, 0.33003E+00_JPRB, 0.32763E+00_JPRB, 0.32436E+00_JPRB, 0.32169E+00_JPRB ,&
&  0.31874E+00_JPRB, 0.31672E+00_JPRB, 0.31608E+00_JPRB, 0.31501E+00_JPRB, 0.31557E+00_JPRB, 0.31691E+00_JPRB, 0.31780E+00_JPRB ,&
&  0.31985E+00_JPRB, 0.32153E+00_JPRB, 0.32360E+00_JPRB, 0.32731E+00_JPRB, 0.33056E+00_JPRB, 0.33340E+00_JPRB, 0.33615E+00_JPRB ,&
&  0.33596E+00_JPRB, 0.33782E+00_JPRB, 0.34057E+00_JPRB, 0.34599E+00_JPRB, 0.36229E+00_JPRB, 0.37239E+00_JPRB, 0.39575E+00_JPRB ,&
&  0.41298E+00_JPRB, 0.44489E+00_JPRB, 0.44137E+00_JPRB, 0.45054E+00_JPRB, 0.45859E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.45665E+00_JPRB, 0.46388E+00_JPRB, 0.47043E+00_JPRB, 0.47588E+00_JPRB, 0.47936E+00_JPRB, 0.48279E+00_JPRB, 0.47980E+00_JPRB ,&
&  0.47376E+00_JPRB, 0.45974E+00_JPRB, 0.44867E+00_JPRB, 0.43707E+00_JPRB, 0.42760E+00_JPRB, 0.41826E+00_JPRB, 0.40969E+00_JPRB ,&
&  0.40118E+00_JPRB, 0.39398E+00_JPRB, 0.38775E+00_JPRB, 0.38217E+00_JPRB, 0.37468E+00_JPRB, 0.36800E+00_JPRB, 0.36263E+00_JPRB ,&
&  0.35817E+00_JPRB, 0.35809E+00_JPRB, 0.36076E+00_JPRB, 0.36550E+00_JPRB, 0.37446E+00_JPRB, 0.38287E+00_JPRB, 0.39219E+00_JPRB ,&
&  0.40617E+00_JPRB, 0.40779E+00_JPRB, 0.40657E+00_JPRB, 0.40601E+00_JPRB, 0.40095E+00_JPRB, 0.39448E+00_JPRB, 0.38568E+00_JPRB ,&
&  0.38896E+00_JPRB, 0.38698E+00_JPRB, 0.39584E+00_JPRB, 0.39570E+00_JPRB, 0.39437E+00_JPRB, 0.39621E+00_JPRB, 0.38922E+00_JPRB ,&
&  0.38513E+00_JPRB, 0.37524E+00_JPRB, 0.36751E+00_JPRB, 0.35853E+00_JPRB, 0.34889E+00_JPRB, 0.34326E+00_JPRB, 0.33577E+00_JPRB ,&
&  0.33046E+00_JPRB, 0.32676E+00_JPRB, 0.32177E+00_JPRB, 0.31883E+00_JPRB, 0.31611E+00_JPRB, 0.31320E+00_JPRB, 0.31165E+00_JPRB ,&
&  0.31049E+00_JPRB, 0.31007E+00_JPRB, 0.31034E+00_JPRB, 0.31061E+00_JPRB, 0.31124E+00_JPRB, 0.31233E+00_JPRB, 0.31311E+00_JPRB ,&
&  0.31402E+00_JPRB, 0.31442E+00_JPRB, 0.31407E+00_JPRB, 0.31372E+00_JPRB, 0.31238E+00_JPRB, 0.31155E+00_JPRB, 0.31035E+00_JPRB ,&
&  0.30955E+00_JPRB, 0.30990E+00_JPRB, 0.31017E+00_JPRB, 0.31092E+00_JPRB, 0.31164E+00_JPRB, 0.31341E+00_JPRB, 0.31503E+00_JPRB ,&
&  0.31678E+00_JPRB, 0.31910E+00_JPRB, 0.32091E+00_JPRB, 0.32335E+00_JPRB, 0.32618E+00_JPRB, 0.32786E+00_JPRB, 0.33117E+00_JPRB ,&
&  0.33430E+00_JPRB, 0.33835E+00_JPRB, 0.34575E+00_JPRB, 0.35366E+00_JPRB, 0.36432E+00_JPRB, 0.37613E+00_JPRB, 0.39054E+00_JPRB ,&
&  0.40864E+00_JPRB, 0.43247E+00_JPRB, 0.43294E+00_JPRB, 0.44394E+00_JPRB, 0.45043E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.44262E+00_JPRB, 0.44854E+00_JPRB, 0.45351E+00_JPRB, 0.45797E+00_JPRB, 0.46214E+00_JPRB, 0.46363E+00_JPRB, 0.46275E+00_JPRB ,&
&  0.45328E+00_JPRB, 0.44661E+00_JPRB, 0.43820E+00_JPRB, 0.43031E+00_JPRB, 0.42299E+00_JPRB, 0.41321E+00_JPRB, 0.40556E+00_JPRB ,&
&  0.39770E+00_JPRB, 0.38976E+00_JPRB, 0.38248E+00_JPRB, 0.37380E+00_JPRB, 0.36484E+00_JPRB, 0.35619E+00_JPRB, 0.35125E+00_JPRB ,&
&  0.34849E+00_JPRB, 0.34859E+00_JPRB, 0.35216E+00_JPRB, 0.35622E+00_JPRB, 0.36140E+00_JPRB, 0.36798E+00_JPRB, 0.37513E+00_JPRB ,&
&  0.37954E+00_JPRB, 0.38180E+00_JPRB, 0.38102E+00_JPRB, 0.37877E+00_JPRB, 0.37632E+00_JPRB, 0.37456E+00_JPRB, 0.37178E+00_JPRB ,&
&  0.36818E+00_JPRB, 0.36635E+00_JPRB, 0.36688E+00_JPRB, 0.36961E+00_JPRB, 0.37064E+00_JPRB, 0.36956E+00_JPRB, 0.36561E+00_JPRB ,&
&  0.36073E+00_JPRB, 0.35565E+00_JPRB, 0.35014E+00_JPRB, 0.34490E+00_JPRB, 0.34060E+00_JPRB, 0.33593E+00_JPRB, 0.33078E+00_JPRB ,&
&  0.32761E+00_JPRB, 0.32432E+00_JPRB, 0.32070E+00_JPRB, 0.31737E+00_JPRB, 0.31455E+00_JPRB, 0.31239E+00_JPRB, 0.31055E+00_JPRB ,&
&  0.30925E+00_JPRB, 0.30826E+00_JPRB, 0.30752E+00_JPRB, 0.30697E+00_JPRB, 0.30645E+00_JPRB, 0.30550E+00_JPRB, 0.30458E+00_JPRB ,&
&  0.30366E+00_JPRB, 0.30337E+00_JPRB, 0.30278E+00_JPRB, 0.30283E+00_JPRB, 0.30315E+00_JPRB, 0.30332E+00_JPRB, 0.30390E+00_JPRB ,&
&  0.30516E+00_JPRB, 0.30648E+00_JPRB, 0.30699E+00_JPRB, 0.30820E+00_JPRB, 0.30937E+00_JPRB, 0.31093E+00_JPRB, 0.31309E+00_JPRB ,&
&  0.31434E+00_JPRB, 0.31573E+00_JPRB, 0.31738E+00_JPRB, 0.31917E+00_JPRB, 0.32239E+00_JPRB, 0.32588E+00_JPRB, 0.32974E+00_JPRB ,&
&  0.33472E+00_JPRB, 0.33944E+00_JPRB, 0.34488E+00_JPRB, 0.35138E+00_JPRB, 0.36370E+00_JPRB, 0.37058E+00_JPRB, 0.37882E+00_JPRB ,&
&  0.38630E+00_JPRB, 0.38739E+00_JPRB, 0.42423E+00_JPRB, 0.42751E+00_JPRB, 0.43385E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.36628E+00_JPRB, 0.37321E+00_JPRB, 0.37723E+00_JPRB, 0.38107E+00_JPRB, 0.37413E+00_JPRB, 0.41419E+00_JPRB, 0.41336E+00_JPRB ,&
&  0.41298E+00_JPRB, 0.40067E+00_JPRB, 0.39984E+00_JPRB, 0.39568E+00_JPRB, 0.39032E+00_JPRB, 0.38415E+00_JPRB, 0.37880E+00_JPRB ,&
&  0.37381E+00_JPRB, 0.36808E+00_JPRB, 0.36355E+00_JPRB, 0.35933E+00_JPRB, 0.35589E+00_JPRB, 0.35287E+00_JPRB, 0.35076E+00_JPRB ,&
&  0.35033E+00_JPRB, 0.35212E+00_JPRB, 0.35581E+00_JPRB, 0.35736E+00_JPRB, 0.35765E+00_JPRB, 0.35788E+00_JPRB, 0.35820E+00_JPRB ,&
&  0.35788E+00_JPRB, 0.35737E+00_JPRB, 0.35564E+00_JPRB, 0.35260E+00_JPRB, 0.35052E+00_JPRB, 0.34931E+00_JPRB, 0.34848E+00_JPRB ,&
&  0.34776E+00_JPRB, 0.34725E+00_JPRB, 0.34668E+00_JPRB, 0.34652E+00_JPRB, 0.34586E+00_JPRB, 0.34461E+00_JPRB, 0.34241E+00_JPRB ,&
&  0.34006E+00_JPRB, 0.33752E+00_JPRB, 0.33431E+00_JPRB, 0.33173E+00_JPRB, 0.32920E+00_JPRB, 0.32665E+00_JPRB, 0.32406E+00_JPRB ,&
&  0.32260E+00_JPRB, 0.32119E+00_JPRB, 0.31939E+00_JPRB, 0.31712E+00_JPRB, 0.31544E+00_JPRB, 0.31368E+00_JPRB, 0.31216E+00_JPRB ,&
&  0.31121E+00_JPRB, 0.31017E+00_JPRB, 0.30911E+00_JPRB, 0.30802E+00_JPRB, 0.30727E+00_JPRB, 0.30634E+00_JPRB, 0.30545E+00_JPRB ,&
&  0.30464E+00_JPRB, 0.30369E+00_JPRB, 0.30345E+00_JPRB, 0.30333E+00_JPRB, 0.30340E+00_JPRB, 0.30340E+00_JPRB, 0.30374E+00_JPRB ,&
&  0.30428E+00_JPRB, 0.30510E+00_JPRB, 0.30612E+00_JPRB, 0.30706E+00_JPRB, 0.30796E+00_JPRB, 0.30888E+00_JPRB, 0.31029E+00_JPRB ,&
&  0.31177E+00_JPRB, 0.31331E+00_JPRB, 0.31489E+00_JPRB, 0.31609E+00_JPRB, 0.31900E+00_JPRB, 0.32171E+00_JPRB, 0.32457E+00_JPRB ,&
&  0.32786E+00_JPRB, 0.33181E+00_JPRB, 0.33614E+00_JPRB, 0.34086E+00_JPRB, 0.34724E+00_JPRB, 0.35076E+00_JPRB, 0.35317E+00_JPRB ,&
&  0.35585E+00_JPRB, 0.36067E+00_JPRB, 0.36160E+00_JPRB, 0.36255E+00_JPRB, 0.36414E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.33893E+00_JPRB, 0.33917E+00_JPRB, 0.33909E+00_JPRB, 0.33872E+00_JPRB, 0.33534E+00_JPRB, 0.33502E+00_JPRB, 0.33462E+00_JPRB ,&
&  0.33453E+00_JPRB, 0.33464E+00_JPRB, 0.33543E+00_JPRB, 0.33638E+00_JPRB, 0.33736E+00_JPRB, 0.33831E+00_JPRB, 0.33921E+00_JPRB ,&
&  0.33979E+00_JPRB, 0.34051E+00_JPRB, 0.33887E+00_JPRB, 0.33780E+00_JPRB, 0.33760E+00_JPRB, 0.33762E+00_JPRB, 0.33754E+00_JPRB ,&
&  0.33735E+00_JPRB, 0.33713E+00_JPRB, 0.33647E+00_JPRB, 0.33604E+00_JPRB, 0.33585E+00_JPRB, 0.33559E+00_JPRB, 0.33519E+00_JPRB ,&
&  0.33475E+00_JPRB, 0.33441E+00_JPRB, 0.33400E+00_JPRB, 0.33350E+00_JPRB, 0.33347E+00_JPRB, 0.33347E+00_JPRB, 0.33295E+00_JPRB ,&
&  0.33232E+00_JPRB, 0.33129E+00_JPRB, 0.33036E+00_JPRB, 0.32949E+00_JPRB, 0.32879E+00_JPRB, 0.32851E+00_JPRB, 0.32817E+00_JPRB ,&
&  0.32767E+00_JPRB, 0.32697E+00_JPRB, 0.32631E+00_JPRB, 0.32542E+00_JPRB, 0.32462E+00_JPRB, 0.32388E+00_JPRB, 0.32301E+00_JPRB ,&
&  0.32205E+00_JPRB, 0.32107E+00_JPRB, 0.32005E+00_JPRB, 0.31888E+00_JPRB, 0.31773E+00_JPRB, 0.31654E+00_JPRB, 0.31524E+00_JPRB ,&
&  0.31395E+00_JPRB, 0.31344E+00_JPRB, 0.31284E+00_JPRB, 0.31233E+00_JPRB, 0.31194E+00_JPRB, 0.31165E+00_JPRB, 0.31136E+00_JPRB ,&
&  0.31112E+00_JPRB, 0.31087E+00_JPRB, 0.31076E+00_JPRB, 0.31065E+00_JPRB, 0.31066E+00_JPRB, 0.31073E+00_JPRB, 0.31095E+00_JPRB ,&
&  0.31119E+00_JPRB, 0.31150E+00_JPRB, 0.31192E+00_JPRB, 0.31265E+00_JPRB, 0.31337E+00_JPRB, 0.31421E+00_JPRB, 0.31503E+00_JPRB ,&
&  0.31595E+00_JPRB, 0.31684E+00_JPRB, 0.31776E+00_JPRB, 0.31864E+00_JPRB, 0.31985E+00_JPRB, 0.32101E+00_JPRB, 0.32229E+00_JPRB ,&
&  0.32356E+00_JPRB, 0.32494E+00_JPRB, 0.32629E+00_JPRB, 0.32775E+00_JPRB, 0.32943E+00_JPRB, 0.32988E+00_JPRB, 0.33036E+00_JPRB ,&
&  0.33095E+00_JPRB, 0.33182E+00_JPRB, 0.33259E+00_JPRB, 0.33368E+00_JPRB, 0.33554E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB ,&
&  0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB, 0.32232E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB, 0.63673E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.63541E+00_JPRB, 0.63622E+00_JPRB, 0.63696E+00_JPRB, 0.63741E+00_JPRB, 0.63767E+00_JPRB, 0.63783E+00_JPRB, 0.63790E+00_JPRB ,&
&  0.63776E+00_JPRB, 0.63796E+00_JPRB, 0.63832E+00_JPRB, 0.63870E+00_JPRB, 0.63904E+00_JPRB, 0.63929E+00_JPRB, 0.63927E+00_JPRB ,&
&  0.63971E+00_JPRB, 0.64177E+00_JPRB, 0.64303E+00_JPRB, 0.64363E+00_JPRB, 0.64413E+00_JPRB, 0.64453E+00_JPRB, 0.64478E+00_JPRB ,&
&  0.64509E+00_JPRB, 0.64543E+00_JPRB, 0.64576E+00_JPRB, 0.64608E+00_JPRB, 0.64647E+00_JPRB, 0.64686E+00_JPRB, 0.64728E+00_JPRB ,&
&  0.64793E+00_JPRB, 0.64915E+00_JPRB, 0.65075E+00_JPRB, 0.65257E+00_JPRB, 0.65482E+00_JPRB, 0.65761E+00_JPRB, 0.65993E+00_JPRB ,&
&  0.66168E+00_JPRB, 0.66260E+00_JPRB, 0.66182E+00_JPRB, 0.65977E+00_JPRB, 0.65685E+00_JPRB, 0.65364E+00_JPRB, 0.65036E+00_JPRB ,&
&  0.64662E+00_JPRB, 0.64324E+00_JPRB, 0.64024E+00_JPRB, 0.63756E+00_JPRB, 0.63506E+00_JPRB, 0.63288E+00_JPRB, 0.63077E+00_JPRB ,&
&  0.62911E+00_JPRB, 0.62744E+00_JPRB, 0.62597E+00_JPRB, 0.62454E+00_JPRB, 0.62302E+00_JPRB, 0.62119E+00_JPRB, 0.61931E+00_JPRB ,&
&  0.61788E+00_JPRB, 0.61689E+00_JPRB, 0.61619E+00_JPRB, 0.61551E+00_JPRB, 0.61516E+00_JPRB, 0.61553E+00_JPRB, 0.61618E+00_JPRB ,&
&  0.61675E+00_JPRB, 0.61755E+00_JPRB, 0.61857E+00_JPRB, 0.61926E+00_JPRB, 0.61975E+00_JPRB, 0.61996E+00_JPRB, 0.62000E+00_JPRB ,&
&  0.61994E+00_JPRB, 0.62028E+00_JPRB, 0.62072E+00_JPRB, 0.62156E+00_JPRB, 0.62237E+00_JPRB, 0.62334E+00_JPRB, 0.62441E+00_JPRB ,&
&  0.62522E+00_JPRB, 0.62615E+00_JPRB, 0.62695E+00_JPRB, 0.62801E+00_JPRB, 0.62894E+00_JPRB, 0.62963E+00_JPRB, 0.63009E+00_JPRB ,&
&  0.63048E+00_JPRB, 0.63070E+00_JPRB, 0.63095E+00_JPRB, 0.63104E+00_JPRB, 0.63131E+00_JPRB, 0.63163E+00_JPRB, 0.63197E+00_JPRB ,&
&  0.63230E+00_JPRB, 0.63280E+00_JPRB, 0.63321E+00_JPRB, 0.63398E+00_JPRB, 0.63468E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.63990E+00_JPRB, 0.64112E+00_JPRB, 0.64196E+00_JPRB, 0.64232E+00_JPRB, 0.64227E+00_JPRB, 0.64154E+00_JPRB, 0.64024E+00_JPRB ,&
&  0.63827E+00_JPRB, 0.63676E+00_JPRB, 0.63582E+00_JPRB, 0.63470E+00_JPRB, 0.63296E+00_JPRB, 0.63349E+00_JPRB, 0.63479E+00_JPRB ,&
&  0.63375E+00_JPRB, 0.63224E+00_JPRB, 0.63043E+00_JPRB, 0.62892E+00_JPRB, 0.62726E+00_JPRB, 0.62537E+00_JPRB, 0.62423E+00_JPRB ,&
&  0.62375E+00_JPRB, 0.62282E+00_JPRB, 0.62130E+00_JPRB, 0.61896E+00_JPRB, 0.61838E+00_JPRB, 0.61784E+00_JPRB, 0.61718E+00_JPRB ,&
&  0.61609E+00_JPRB, 0.61319E+00_JPRB, 0.61061E+00_JPRB, 0.60905E+00_JPRB, 0.61271E+00_JPRB, 0.61692E+00_JPRB, 0.62184E+00_JPRB ,&
&  0.62739E+00_JPRB, 0.63109E+00_JPRB, 0.63578E+00_JPRB, 0.63940E+00_JPRB, 0.64071E+00_JPRB, 0.64414E+00_JPRB, 0.64633E+00_JPRB ,&
&  0.64624E+00_JPRB, 0.64388E+00_JPRB, 0.64206E+00_JPRB, 0.64163E+00_JPRB, 0.64150E+00_JPRB, 0.64095E+00_JPRB, 0.64006E+00_JPRB ,&
&  0.63947E+00_JPRB, 0.63854E+00_JPRB, 0.63708E+00_JPRB, 0.63496E+00_JPRB, 0.63208E+00_JPRB, 0.62892E+00_JPRB, 0.62541E+00_JPRB ,&
&  0.62258E+00_JPRB, 0.61931E+00_JPRB, 0.61612E+00_JPRB, 0.61291E+00_JPRB, 0.61030E+00_JPRB, 0.60718E+00_JPRB, 0.60412E+00_JPRB ,&
&  0.60252E+00_JPRB, 0.60326E+00_JPRB, 0.60478E+00_JPRB, 0.60484E+00_JPRB, 0.60505E+00_JPRB, 0.60506E+00_JPRB, 0.60622E+00_JPRB ,&
&  0.60756E+00_JPRB, 0.61034E+00_JPRB, 0.61340E+00_JPRB, 0.61771E+00_JPRB, 0.62169E+00_JPRB, 0.62558E+00_JPRB, 0.62859E+00_JPRB ,&
&  0.63128E+00_JPRB, 0.63307E+00_JPRB, 0.63419E+00_JPRB, 0.63481E+00_JPRB, 0.63559E+00_JPRB, 0.63602E+00_JPRB, 0.63589E+00_JPRB ,&
&  0.63551E+00_JPRB, 0.63530E+00_JPRB, 0.63488E+00_JPRB, 0.63416E+00_JPRB, 0.63385E+00_JPRB, 0.63321E+00_JPRB, 0.63309E+00_JPRB ,&
&  0.63308E+00_JPRB, 0.63383E+00_JPRB, 0.63473E+00_JPRB, 0.63632E+00_JPRB, 0.63814E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.63335E+00_JPRB, 0.63631E+00_JPRB, 0.63858E+00_JPRB, 0.63938E+00_JPRB, 0.63963E+00_JPRB, 0.63974E+00_JPRB, 0.63933E+00_JPRB ,&
&  0.63876E+00_JPRB, 0.63788E+00_JPRB, 0.63770E+00_JPRB, 0.63717E+00_JPRB, 0.63646E+00_JPRB, 0.63945E+00_JPRB, 0.63965E+00_JPRB ,&
&  0.63827E+00_JPRB, 0.63594E+00_JPRB, 0.63381E+00_JPRB, 0.63072E+00_JPRB, 0.62667E+00_JPRB, 0.62138E+00_JPRB, 0.61183E+00_JPRB ,&
&  0.60600E+00_JPRB, 0.59989E+00_JPRB, 0.59301E+00_JPRB, 0.59146E+00_JPRB, 0.58985E+00_JPRB, 0.56830E+00_JPRB, 0.56851E+00_JPRB ,&
&  0.59016E+00_JPRB, 0.59884E+00_JPRB, 0.60145E+00_JPRB, 0.60510E+00_JPRB, 0.57893E+00_JPRB, 0.55020E+00_JPRB, 0.53185E+00_JPRB ,&
&  0.50334E+00_JPRB, 0.52979E+00_JPRB, 0.54575E+00_JPRB, 0.55088E+00_JPRB, 0.54992E+00_JPRB, 0.54668E+00_JPRB, 0.56720E+00_JPRB ,&
&  0.61483E+00_JPRB, 0.62382E+00_JPRB, 0.63532E+00_JPRB, 0.64797E+00_JPRB, 0.66036E+00_JPRB, 0.66860E+00_JPRB, 0.67798E+00_JPRB ,&
&  0.68263E+00_JPRB, 0.68554E+00_JPRB, 0.68336E+00_JPRB, 0.67901E+00_JPRB, 0.66927E+00_JPRB, 0.66135E+00_JPRB, 0.65377E+00_JPRB ,&
&  0.64947E+00_JPRB, 0.64484E+00_JPRB, 0.64071E+00_JPRB, 0.63482E+00_JPRB, 0.62849E+00_JPRB, 0.62133E+00_JPRB, 0.61406E+00_JPRB ,&
&  0.60663E+00_JPRB, 0.59872E+00_JPRB, 0.59363E+00_JPRB, 0.58969E+00_JPRB, 0.59033E+00_JPRB, 0.59042E+00_JPRB, 0.59327E+00_JPRB ,&
&  0.59589E+00_JPRB, 0.60015E+00_JPRB, 0.60496E+00_JPRB, 0.61157E+00_JPRB, 0.61702E+00_JPRB, 0.62125E+00_JPRB, 0.62407E+00_JPRB ,&
&  0.62628E+00_JPRB, 0.62743E+00_JPRB, 0.62796E+00_JPRB, 0.62774E+00_JPRB, 0.62799E+00_JPRB, 0.62844E+00_JPRB, 0.62843E+00_JPRB ,&
&  0.62886E+00_JPRB, 0.62929E+00_JPRB, 0.62993E+00_JPRB, 0.63005E+00_JPRB, 0.62979E+00_JPRB, 0.62789E+00_JPRB, 0.62584E+00_JPRB ,&
&  0.62376E+00_JPRB, 0.62368E+00_JPRB, 0.62440E+00_JPRB, 0.62676E+00_JPRB, 0.62948E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.64355E+00_JPRB, 0.64451E+00_JPRB, 0.64613E+00_JPRB, 0.64705E+00_JPRB, 0.64738E+00_JPRB, 0.64713E+00_JPRB, 0.64655E+00_JPRB ,&
&  0.64620E+00_JPRB, 0.64627E+00_JPRB, 0.64575E+00_JPRB, 0.64433E+00_JPRB, 0.64408E+00_JPRB, 0.64423E+00_JPRB, 0.64364E+00_JPRB ,&
&  0.64073E+00_JPRB, 0.63711E+00_JPRB, 0.63404E+00_JPRB, 0.62982E+00_JPRB, 0.62574E+00_JPRB, 0.62216E+00_JPRB, 0.61416E+00_JPRB ,&
&  0.60882E+00_JPRB, 0.60014E+00_JPRB, 0.56877E+00_JPRB, 0.57179E+00_JPRB, 0.56619E+00_JPRB, 0.56548E+00_JPRB, 0.56917E+00_JPRB ,&
&  0.55989E+00_JPRB, 0.55849E+00_JPRB, 0.54622E+00_JPRB, 0.52801E+00_JPRB, 0.51907E+00_JPRB, 0.50339E+00_JPRB, 0.50422E+00_JPRB ,&
&  0.50130E+00_JPRB, 0.50662E+00_JPRB, 0.51874E+00_JPRB, 0.52299E+00_JPRB, 0.52096E+00_JPRB, 0.51952E+00_JPRB, 0.52930E+00_JPRB ,&
&  0.55664E+00_JPRB, 0.59749E+00_JPRB, 0.65118E+00_JPRB, 0.67845E+00_JPRB, 0.70401E+00_JPRB, 0.72616E+00_JPRB, 0.74647E+00_JPRB ,&
&  0.76106E+00_JPRB, 0.77261E+00_JPRB, 0.77264E+00_JPRB, 0.77033E+00_JPRB, 0.75814E+00_JPRB, 0.74012E+00_JPRB, 0.71788E+00_JPRB ,&
&  0.69646E+00_JPRB, 0.68087E+00_JPRB, 0.66929E+00_JPRB, 0.66282E+00_JPRB, 0.65981E+00_JPRB, 0.65729E+00_JPRB, 0.65593E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.63536E+00_JPRB, 0.62295E+00_JPRB, 0.61070E+00_JPRB, 0.60714E+00_JPRB, 0.60554E+00_JPRB, 0.60900E+00_JPRB ,&
&  0.61318E+00_JPRB, 0.62200E+00_JPRB, 0.63262E+00_JPRB, 0.63646E+00_JPRB, 0.64236E+00_JPRB, 0.64944E+00_JPRB, 0.64898E+00_JPRB ,&
&  0.64847E+00_JPRB, 0.64680E+00_JPRB, 0.64348E+00_JPRB, 0.63933E+00_JPRB, 0.63460E+00_JPRB, 0.63023E+00_JPRB, 0.62679E+00_JPRB ,&
&  0.62568E+00_JPRB, 0.62666E+00_JPRB, 0.63128E+00_JPRB, 0.63654E+00_JPRB, 0.64281E+00_JPRB, 0.64620E+00_JPRB, 0.64580E+00_JPRB ,&
&  0.64168E+00_JPRB, 0.63959E+00_JPRB, 0.63805E+00_JPRB, 0.64009E+00_JPRB, 0.64198E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.65954E+00_JPRB, 0.66236E+00_JPRB, 0.66823E+00_JPRB, 0.67426E+00_JPRB, 0.67914E+00_JPRB, 0.68008E+00_JPRB, 0.67305E+00_JPRB ,&
&  0.66654E+00_JPRB, 0.66020E+00_JPRB, 0.65511E+00_JPRB, 0.65143E+00_JPRB, 0.64966E+00_JPRB, 0.64630E+00_JPRB, 0.64244E+00_JPRB ,&
&  0.63903E+00_JPRB, 0.63715E+00_JPRB, 0.63655E+00_JPRB, 0.63599E+00_JPRB, 0.63722E+00_JPRB, 0.63644E+00_JPRB, 0.63435E+00_JPRB ,&
&  0.63244E+00_JPRB, 0.62022E+00_JPRB, 0.59519E+00_JPRB, 0.58656E+00_JPRB, 0.57736E+00_JPRB, 0.57790E+00_JPRB, 0.57809E+00_JPRB ,&
&  0.57281E+00_JPRB, 0.51665E+00_JPRB, 0.52523E+00_JPRB, 0.51199E+00_JPRB, 0.50446E+00_JPRB, 0.50509E+00_JPRB, 0.49234E+00_JPRB ,&
&  0.48410E+00_JPRB, 0.47388E+00_JPRB, 0.47438E+00_JPRB, 0.48157E+00_JPRB, 0.48101E+00_JPRB, 0.49072E+00_JPRB, 0.50445E+00_JPRB ,&
&  0.53107E+00_JPRB, 0.64026E+00_JPRB, 0.68952E+00_JPRB, 0.71863E+00_JPRB, 0.74271E+00_JPRB, 0.77090E+00_JPRB, 0.79843E+00_JPRB ,&
&  0.81926E+00_JPRB, 0.84508E+00_JPRB, 0.85547E+00_JPRB, 0.84988E+00_JPRB, 0.85558E+00_JPRB, 0.84610E+00_JPRB, 0.82302E+00_JPRB ,&
&  0.78943E+00_JPRB, 0.74582E+00_JPRB, 0.71639E+00_JPRB, 0.69841E+00_JPRB, 0.68502E+00_JPRB, 0.67444E+00_JPRB, 0.66764E+00_JPRB ,&
&  0.67120E+00_JPRB, 0.66534E+00_JPRB, 0.65754E+00_JPRB, 0.64304E+00_JPRB, 0.63537E+00_JPRB, 0.63143E+00_JPRB, 0.63298E+00_JPRB ,&
&  0.63709E+00_JPRB, 0.64655E+00_JPRB, 0.65867E+00_JPRB, 0.66842E+00_JPRB, 0.67355E+00_JPRB, 0.67305E+00_JPRB, 0.67703E+00_JPRB ,&
&  0.67641E+00_JPRB, 0.67298E+00_JPRB, 0.66893E+00_JPRB, 0.66445E+00_JPRB, 0.65995E+00_JPRB, 0.65705E+00_JPRB, 0.65886E+00_JPRB ,&
&  0.66168E+00_JPRB, 0.66342E+00_JPRB, 0.66402E+00_JPRB, 0.66621E+00_JPRB, 0.67045E+00_JPRB, 0.67415E+00_JPRB, 0.67546E+00_JPRB ,&
&  0.66946E+00_JPRB, 0.66319E+00_JPRB, 0.65795E+00_JPRB, 0.65782E+00_JPRB, 0.65813E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.65215E+00_JPRB, 0.66300E+00_JPRB, 0.68192E+00_JPRB, 0.69756E+00_JPRB, 0.70844E+00_JPRB, 0.70743E+00_JPRB, 0.69630E+00_JPRB ,&
&  0.68514E+00_JPRB, 0.67690E+00_JPRB, 0.66897E+00_JPRB, 0.66175E+00_JPRB, 0.65338E+00_JPRB, 0.64688E+00_JPRB, 0.64200E+00_JPRB ,&
&  0.63925E+00_JPRB, 0.63888E+00_JPRB, 0.63879E+00_JPRB, 0.63865E+00_JPRB, 0.63963E+00_JPRB, 0.64161E+00_JPRB, 0.64167E+00_JPRB ,&
&  0.64156E+00_JPRB, 0.63223E+00_JPRB, 0.61070E+00_JPRB, 0.59357E+00_JPRB, 0.58715E+00_JPRB, 0.57678E+00_JPRB, 0.58030E+00_JPRB ,&
&  0.57980E+00_JPRB, 0.57831E+00_JPRB, 0.57084E+00_JPRB, 0.56737E+00_JPRB, 0.54772E+00_JPRB, 0.47726E+00_JPRB, 0.48160E+00_JPRB ,&
&  0.46963E+00_JPRB, 0.45392E+00_JPRB, 0.45135E+00_JPRB, 0.45149E+00_JPRB, 0.45351E+00_JPRB, 0.47644E+00_JPRB, 0.50186E+00_JPRB ,&
&  0.52902E+00_JPRB, 0.65137E+00_JPRB, 0.68484E+00_JPRB, 0.72063E+00_JPRB, 0.75261E+00_JPRB, 0.79236E+00_JPRB, 0.83420E+00_JPRB ,&
&  0.86005E+00_JPRB, 0.88794E+00_JPRB, 0.90879E+00_JPRB, 0.92330E+00_JPRB, 0.92701E+00_JPRB, 0.92914E+00_JPRB, 0.92973E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.84098E+00_JPRB, 0.79160E+00_JPRB, 0.76282E+00_JPRB, 0.73628E+00_JPRB, 0.71822E+00_JPRB, 0.70378E+00_JPRB ,&
&  0.69896E+00_JPRB, 0.68213E+00_JPRB, 0.67922E+00_JPRB, 0.67012E+00_JPRB, 0.66521E+00_JPRB, 0.66177E+00_JPRB, 0.65750E+00_JPRB ,&
&  0.65052E+00_JPRB, 0.65359E+00_JPRB, 0.66863E+00_JPRB, 0.67627E+00_JPRB, 0.68184E+00_JPRB, 0.67792E+00_JPRB, 0.67035E+00_JPRB ,&
&  0.65735E+00_JPRB, 0.65560E+00_JPRB, 0.64966E+00_JPRB, 0.64790E+00_JPRB, 0.64857E+00_JPRB, 0.65752E+00_JPRB, 0.66954E+00_JPRB ,&
&  0.67752E+00_JPRB, 0.67993E+00_JPRB, 0.67930E+00_JPRB, 0.67959E+00_JPRB, 0.67817E+00_JPRB, 0.67523E+00_JPRB, 0.66948E+00_JPRB ,&
&  0.66384E+00_JPRB, 0.66062E+00_JPRB, 0.65440E+00_JPRB, 0.64966E+00_JPRB, 0.64840E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.64501E+00_JPRB, 0.66897E+00_JPRB, 0.69749E+00_JPRB, 0.71702E+00_JPRB, 0.72575E+00_JPRB, 0.71338E+00_JPRB, 0.69614E+00_JPRB ,&
&  0.68233E+00_JPRB, 0.67245E+00_JPRB, 0.66490E+00_JPRB, 0.65972E+00_JPRB, 0.65354E+00_JPRB, 0.64746E+00_JPRB, 0.64224E+00_JPRB ,&
&  0.63841E+00_JPRB, 0.62790E+00_JPRB, 0.62441E+00_JPRB, 0.62715E+00_JPRB, 0.62889E+00_JPRB, 0.63344E+00_JPRB, 0.63722E+00_JPRB ,&
&  0.63443E+00_JPRB, 0.62556E+00_JPRB, 0.61767E+00_JPRB, 0.60046E+00_JPRB, 0.58367E+00_JPRB, 0.57855E+00_JPRB, 0.58398E+00_JPRB ,&
&  0.57026E+00_JPRB, 0.54848E+00_JPRB, 0.56840E+00_JPRB, 0.58447E+00_JPRB, 0.55566E+00_JPRB, 0.52900E+00_JPRB, 0.48828E+00_JPRB ,&
&  0.47494E+00_JPRB, 0.45009E+00_JPRB, 0.44430E+00_JPRB, 0.44428E+00_JPRB, 0.45360E+00_JPRB, 0.47333E+00_JPRB, 0.51439E+00_JPRB ,&
&  0.56656E+00_JPRB, 0.65368E+00_JPRB, 0.68154E+00_JPRB, 0.72571E+00_JPRB, 0.76364E+00_JPRB, 0.79739E+00_JPRB, 0.82136E+00_JPRB ,&
&  0.85946E+00_JPRB, 0.91301E+00_JPRB, 0.94648E+00_JPRB, 0.95407E+00_JPRB, 0.96753E+00_JPRB, 0.99193E+00_JPRB, 0.10380E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.96501E+00_JPRB, 0.88196E+00_JPRB, 0.82864E+00_JPRB, 0.79546E+00_JPRB, 0.78312E+00_JPRB, 0.73974E+00_JPRB ,&
&  0.72548E+00_JPRB, 0.72169E+00_JPRB, 0.71167E+00_JPRB, 0.70713E+00_JPRB, 0.70395E+00_JPRB, 0.70083E+00_JPRB, 0.69361E+00_JPRB ,&
&  0.68353E+00_JPRB, 0.68945E+00_JPRB, 0.75357E+00_JPRB, 0.76869E+00_JPRB, 0.75511E+00_JPRB, 0.71279E+00_JPRB, 0.66821E+00_JPRB ,&
&  0.63793E+00_JPRB, 0.62633E+00_JPRB, 0.62197E+00_JPRB, 0.61973E+00_JPRB, 0.62567E+00_JPRB, 0.63532E+00_JPRB, 0.65118E+00_JPRB ,&
&  0.65194E+00_JPRB, 0.66125E+00_JPRB, 0.67039E+00_JPRB, 0.67492E+00_JPRB, 0.67383E+00_JPRB, 0.66839E+00_JPRB, 0.66436E+00_JPRB ,&
&  0.66069E+00_JPRB, 0.65672E+00_JPRB, 0.65007E+00_JPRB, 0.64231E+00_JPRB, 0.63866E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.62827E+00_JPRB, 0.66544E+00_JPRB, 0.71188E+00_JPRB, 0.74226E+00_JPRB, 0.74267E+00_JPRB, 0.71586E+00_JPRB, 0.68778E+00_JPRB ,&
&  0.66973E+00_JPRB, 0.66129E+00_JPRB, 0.65683E+00_JPRB, 0.65182E+00_JPRB, 0.64704E+00_JPRB, 0.64198E+00_JPRB, 0.63687E+00_JPRB ,&
&  0.63283E+00_JPRB, 0.62907E+00_JPRB, 0.62288E+00_JPRB, 0.61476E+00_JPRB, 0.61024E+00_JPRB, 0.61106E+00_JPRB, 0.61475E+00_JPRB ,&
&  0.61861E+00_JPRB, 0.61624E+00_JPRB, 0.61449E+00_JPRB, 0.60457E+00_JPRB, 0.59203E+00_JPRB, 0.57763E+00_JPRB, 0.56754E+00_JPRB ,&
&  0.56382E+00_JPRB, 0.56695E+00_JPRB, 0.56066E+00_JPRB, 0.58116E+00_JPRB, 0.57683E+00_JPRB, 0.54729E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.48743E+00_JPRB, 0.45872E+00_JPRB, 0.44808E+00_JPRB, 0.45311E+00_JPRB, 0.48201E+00_JPRB, 0.51795E+00_JPRB, 0.57157E+00_JPRB ,&
&  0.66461E+00_JPRB, 0.72486E+00_JPRB, 0.76695E+00_JPRB, 0.80387E+00_JPRB, 0.83290E+00_JPRB, 0.86465E+00_JPRB, 0.90161E+00_JPRB ,&
&  0.94937E+00_JPRB, 0.99209E+00_JPRB, 0.10143E+01_JPRB, 0.99118E+00_JPRB, 0.98207E+00_JPRB, 0.92973E+00_JPRB, 0.10461E+01_JPRB ,&
&  0.13813E+01_JPRB, 0.11640E+01_JPRB, 0.10171E+01_JPRB, 0.94196E+00_JPRB, 0.89552E+00_JPRB, 0.85886E+00_JPRB, 0.82560E+00_JPRB ,&
&  0.79096E+00_JPRB, 0.77107E+00_JPRB, 0.76135E+00_JPRB, 0.72951E+00_JPRB, 0.69882E+00_JPRB, 0.69919E+00_JPRB, 0.74047E+00_JPRB ,&
&  0.84251E+00_JPRB, 0.99432E+00_JPRB, 0.11883E+01_JPRB, 0.85185E+00_JPRB, 0.74666E+00_JPRB, 0.67252E+00_JPRB, 0.63870E+00_JPRB ,&
&  0.62931E+00_JPRB, 0.62853E+00_JPRB, 0.63011E+00_JPRB, 0.62600E+00_JPRB, 0.62228E+00_JPRB, 0.62405E+00_JPRB, 0.62521E+00_JPRB ,&
&  0.62364E+00_JPRB, 0.63275E+00_JPRB, 0.64609E+00_JPRB, 0.65660E+00_JPRB, 0.66543E+00_JPRB, 0.66718E+00_JPRB, 0.66749E+00_JPRB ,&
&  0.66531E+00_JPRB, 0.65743E+00_JPRB, 0.64747E+00_JPRB, 0.63005E+00_JPRB, 0.62003E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.62365E+00_JPRB, 0.66636E+00_JPRB, 0.72696E+00_JPRB, 0.76241E+00_JPRB, 0.76027E+00_JPRB, 0.71883E+00_JPRB, 0.66910E+00_JPRB ,&
&  0.63659E+00_JPRB, 0.61685E+00_JPRB, 0.61133E+00_JPRB, 0.60851E+00_JPRB, 0.60790E+00_JPRB, 0.60779E+00_JPRB, 0.60653E+00_JPRB ,&
&  0.60399E+00_JPRB, 0.60172E+00_JPRB, 0.59884E+00_JPRB, 0.59622E+00_JPRB, 0.59219E+00_JPRB, 0.58914E+00_JPRB, 0.58921E+00_JPRB ,&
&  0.59159E+00_JPRB, 0.59141E+00_JPRB, 0.59044E+00_JPRB, 0.58872E+00_JPRB, 0.59026E+00_JPRB, 0.59460E+00_JPRB, 0.59260E+00_JPRB ,&
&  0.57671E+00_JPRB, 0.56533E+00_JPRB, 0.57267E+00_JPRB, 0.58916E+00_JPRB, 0.58798E+00_JPRB, 0.56564E+00_JPRB, 0.52588E+00_JPRB ,&
&  0.49502E+00_JPRB, 0.47429E+00_JPRB, 0.49026E+00_JPRB, 0.53352E+00_JPRB, 0.59092E+00_JPRB, 0.70920E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.86623E+00_JPRB, 0.90645E+00_JPRB, 0.93127E+00_JPRB, 0.95679E+00_JPRB, 0.10071E+01_JPRB, 0.10588E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.11216E+01_JPRB, 0.11344E+01_JPRB, 0.11371E+01_JPRB, 0.90308E+00_JPRB, 0.88025E+00_JPRB, 0.96596E+00_JPRB, 0.10721E+01_JPRB ,&
&  0.11379E+01_JPRB, 0.10752E+01_JPRB, 0.10407E+01_JPRB, 0.10567E+01_JPRB, 0.99417E+00_JPRB, 0.95235E+00_JPRB, 0.92129E+00_JPRB ,&
&  0.88931E+00_JPRB, 0.88032E+00_JPRB, 0.82624E+00_JPRB, 0.76717E+00_JPRB, 0.76399E+00_JPRB, 0.76600E+00_JPRB, 0.77628E+00_JPRB ,&
&  0.81084E+00_JPRB, 0.83578E+00_JPRB, 0.77429E+00_JPRB, 0.63318E+00_JPRB, 0.59789E+00_JPRB, 0.58520E+00_JPRB, 0.58788E+00_JPRB ,&
&  0.59449E+00_JPRB, 0.60113E+00_JPRB, 0.60982E+00_JPRB, 0.61132E+00_JPRB, 0.61003E+00_JPRB, 0.60642E+00_JPRB, 0.60079E+00_JPRB ,&
&  0.59358E+00_JPRB, 0.59480E+00_JPRB, 0.60412E+00_JPRB, 0.60767E+00_JPRB, 0.62358E+00_JPRB, 0.63812E+00_JPRB, 0.65188E+00_JPRB ,&
&  0.64902E+00_JPRB, 0.64275E+00_JPRB, 0.63513E+00_JPRB, 0.61853E+00_JPRB, 0.60700E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.66558E+00_JPRB, 0.70796E+00_JPRB, 0.74558E+00_JPRB, 0.76831E+00_JPRB, 0.76115E+00_JPRB, 0.73171E+00_JPRB, 0.66795E+00_JPRB ,&
&  0.64623E+00_JPRB, 0.61115E+00_JPRB, 0.59903E+00_JPRB, 0.58681E+00_JPRB, 0.57650E+00_JPRB, 0.56848E+00_JPRB, 0.56611E+00_JPRB ,&
&  0.56911E+00_JPRB, 0.57259E+00_JPRB, 0.57456E+00_JPRB, 0.57642E+00_JPRB, 0.57491E+00_JPRB, 0.57403E+00_JPRB, 0.57407E+00_JPRB ,&
&  0.57405E+00_JPRB, 0.57446E+00_JPRB, 0.57674E+00_JPRB, 0.58013E+00_JPRB, 0.58522E+00_JPRB, 0.58939E+00_JPRB, 0.58713E+00_JPRB ,&
&  0.57716E+00_JPRB, 0.56859E+00_JPRB, 0.57976E+00_JPRB, 0.59697E+00_JPRB, 0.59592E+00_JPRB, 0.58312E+00_JPRB, 0.54034E+00_JPRB ,&
&  0.50820E+00_JPRB, 0.52021E+00_JPRB, 0.60411E+00_JPRB, 0.74978E+00_JPRB, 0.81455E+00_JPRB, 0.87790E+00_JPRB, 0.91813E+00_JPRB ,&
&  0.87744E+00_JPRB, 0.88836E+00_JPRB, 0.94994E+00_JPRB, 0.11259E+01_JPRB, 0.11598E+01_JPRB, 0.12017E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.13060E+01_JPRB, 0.12999E+01_JPRB, 0.12709E+01_JPRB, 0.10873E+01_JPRB, 0.12224E+01_JPRB, 0.14781E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.13013E+01_JPRB, 0.12686E+01_JPRB, 0.12989E+01_JPRB, 0.12203E+01_JPRB, 0.11045E+01_JPRB, 0.10417E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.99803E+00_JPRB, 0.97347E+00_JPRB, 0.87452E+00_JPRB, 0.84341E+00_JPRB, 0.85101E+00_JPRB, 0.82129E+00_JPRB, 0.79600E+00_JPRB ,&
&  0.77132E+00_JPRB, 0.72164E+00_JPRB, 0.68104E+00_JPRB, 0.63628E+00_JPRB, 0.60681E+00_JPRB, 0.60039E+00_JPRB, 0.60291E+00_JPRB ,&
&  0.60572E+00_JPRB, 0.60667E+00_JPRB, 0.61199E+00_JPRB, 0.61766E+00_JPRB, 0.61608E+00_JPRB, 0.60570E+00_JPRB, 0.60409E+00_JPRB ,&
&  0.59950E+00_JPRB, 0.58933E+00_JPRB, 0.58188E+00_JPRB, 0.58912E+00_JPRB, 0.59831E+00_JPRB, 0.61358E+00_JPRB, 0.63055E+00_JPRB ,&
&  0.64168E+00_JPRB, 0.64864E+00_JPRB, 0.64466E+00_JPRB, 0.64707E+00_JPRB, 0.64468E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.70809E+00_JPRB, 0.72746E+00_JPRB, 0.75393E+00_JPRB, 0.77424E+00_JPRB, 0.78590E+00_JPRB, 0.73030E+00_JPRB, 0.67807E+00_JPRB ,&
&  0.69646E+00_JPRB, 0.62915E+00_JPRB, 0.59752E+00_JPRB, 0.58486E+00_JPRB, 0.58025E+00_JPRB, 0.56449E+00_JPRB, 0.55292E+00_JPRB ,&
&  0.54882E+00_JPRB, 0.55366E+00_JPRB, 0.56101E+00_JPRB, 0.56489E+00_JPRB, 0.56609E+00_JPRB, 0.56683E+00_JPRB, 0.56679E+00_JPRB ,&
&  0.56761E+00_JPRB, 0.56967E+00_JPRB, 0.57263E+00_JPRB, 0.57572E+00_JPRB, 0.57724E+00_JPRB, 0.57993E+00_JPRB, 0.57971E+00_JPRB ,&
&  0.57322E+00_JPRB, 0.57093E+00_JPRB, 0.58376E+00_JPRB, 0.60502E+00_JPRB, 0.61421E+00_JPRB, 0.61801E+00_JPRB, 0.60236E+00_JPRB ,&
&  0.54633E+00_JPRB, 0.60101E+00_JPRB, 0.79909E+00_JPRB, 0.86898E+00_JPRB, 0.91737E+00_JPRB, 0.98032E+00_JPRB, 0.10351E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.11132E+01_JPRB, 0.11437E+01_JPRB, 0.11860E+01_JPRB, 0.12410E+01_JPRB, 0.12897E+01_JPRB, 0.13559E+01_JPRB ,&
&  0.15127E+01_JPRB, 0.12048E+01_JPRB, 0.11218E+01_JPRB, 0.11870E+01_JPRB, 0.16347E+01_JPRB, 0.16941E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.14899E+01_JPRB, 0.14400E+01_JPRB, 0.13882E+01_JPRB, 0.12850E+01_JPRB, 0.12061E+01_JPRB, 0.11809E+01_JPRB, 0.11687E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.10684E+01_JPRB, 0.95591E+00_JPRB, 0.93465E+00_JPRB, 0.93292E+00_JPRB, 0.88263E+00_JPRB, 0.81242E+00_JPRB ,&
&  0.75793E+00_JPRB, 0.71541E+00_JPRB, 0.68536E+00_JPRB, 0.66389E+00_JPRB, 0.64697E+00_JPRB, 0.63702E+00_JPRB, 0.63004E+00_JPRB ,&
&  0.62684E+00_JPRB, 0.62516E+00_JPRB, 0.61527E+00_JPRB, 0.61802E+00_JPRB, 0.62167E+00_JPRB, 0.61968E+00_JPRB, 0.62915E+00_JPRB ,&
&  0.63035E+00_JPRB, 0.62398E+00_JPRB, 0.60949E+00_JPRB, 0.61667E+00_JPRB, 0.63430E+00_JPRB, 0.66451E+00_JPRB, 0.69294E+00_JPRB ,&
&  0.72785E+00_JPRB, 0.72400E+00_JPRB, 0.69292E+00_JPRB, 0.68565E+00_JPRB, 0.68717E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.73881E+00_JPRB, 0.75484E+00_JPRB, 0.76671E+00_JPRB, 0.78475E+00_JPRB, 0.79963E+00_JPRB, 0.75237E+00_JPRB, 0.71808E+00_JPRB ,&
&  0.77788E+00_JPRB, 0.69416E+00_JPRB, 0.73964E+00_JPRB, 0.65149E+00_JPRB, 0.60839E+00_JPRB, 0.58597E+00_JPRB, 0.56910E+00_JPRB ,&
&  0.55404E+00_JPRB, 0.55863E+00_JPRB, 0.56583E+00_JPRB, 0.57018E+00_JPRB, 0.57755E+00_JPRB, 0.57360E+00_JPRB, 0.57008E+00_JPRB ,&
&  0.56785E+00_JPRB, 0.56882E+00_JPRB, 0.57564E+00_JPRB, 0.57626E+00_JPRB, 0.57508E+00_JPRB, 0.57389E+00_JPRB, 0.56703E+00_JPRB ,&
&  0.55358E+00_JPRB, 0.55682E+00_JPRB, 0.58058E+00_JPRB, 0.60325E+00_JPRB, 0.62622E+00_JPRB, 0.64049E+00_JPRB, 0.66118E+00_JPRB ,&
&  0.68286E+00_JPRB, 0.70840E+00_JPRB, 0.84549E+00_JPRB, 0.88963E+00_JPRB, 0.93864E+00_JPRB, 0.97449E+00_JPRB, 0.10174E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.10973E+01_JPRB, 0.11196E+01_JPRB, 0.11636E+01_JPRB, 0.12498E+01_JPRB, 0.13127E+01_JPRB, 0.14218E+01_JPRB ,&
&  0.16229E+01_JPRB, 0.12365E+01_JPRB, 0.14445E+01_JPRB, 0.14417E+01_JPRB, 0.16802E+01_JPRB, 0.18588E+01_JPRB, 0.18466E+01_JPRB ,&
&  0.20511E+01_JPRB, 0.18143E+01_JPRB, 0.16413E+01_JPRB, 0.14548E+01_JPRB, 0.13100E+01_JPRB, 0.12793E+01_JPRB, 0.12645E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.12684E+01_JPRB, 0.10974E+01_JPRB, 0.10191E+01_JPRB, 0.99158E+00_JPRB, 0.96935E+00_JPRB, 0.89185E+00_JPRB ,&
&  0.85057E+00_JPRB, 0.82505E+00_JPRB, 0.78905E+00_JPRB, 0.75062E+00_JPRB, 0.72124E+00_JPRB, 0.70482E+00_JPRB, 0.68156E+00_JPRB ,&
&  0.67681E+00_JPRB, 0.66313E+00_JPRB, 0.62276E+00_JPRB, 0.60970E+00_JPRB, 0.60707E+00_JPRB, 0.59454E+00_JPRB, 0.60593E+00_JPRB ,&
&  0.62539E+00_JPRB, 0.63102E+00_JPRB, 0.67123E+00_JPRB, 0.72579E+00_JPRB, 0.74387E+00_JPRB, 0.75332E+00_JPRB, 0.82685E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.77904E+00_JPRB, 0.77712E+00_JPRB, 0.74519E+00_JPRB, 0.73145E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.77981E+00_JPRB, 0.77953E+00_JPRB, 0.78060E+00_JPRB, 0.79611E+00_JPRB, 0.81073E+00_JPRB, 0.81485E+00_JPRB, 0.78111E+00_JPRB ,&
&  0.82742E+00_JPRB, 0.80022E+00_JPRB, 0.78211E+00_JPRB, 0.77524E+00_JPRB, 0.75948E+00_JPRB, 0.67311E+00_JPRB, 0.58441E+00_JPRB ,&
&  0.56021E+00_JPRB, 0.55553E+00_JPRB, 0.57210E+00_JPRB, 0.57382E+00_JPRB, 0.57964E+00_JPRB, 0.58024E+00_JPRB, 0.57422E+00_JPRB ,&
&  0.57313E+00_JPRB, 0.57432E+00_JPRB, 0.58405E+00_JPRB, 0.58597E+00_JPRB, 0.57970E+00_JPRB, 0.57781E+00_JPRB, 0.57379E+00_JPRB ,&
&  0.55248E+00_JPRB, 0.54567E+00_JPRB, 0.55778E+00_JPRB, 0.56610E+00_JPRB, 0.62433E+00_JPRB, 0.65308E+00_JPRB, 0.68315E+00_JPRB ,&
&  0.71473E+00_JPRB, 0.75528E+00_JPRB, 0.82302E+00_JPRB, 0.87799E+00_JPRB, 0.91538E+00_JPRB, 0.94385E+00_JPRB, 0.98557E+00_JPRB ,&
&  0.10182E+01_JPRB, 0.10627E+01_JPRB, 0.11147E+01_JPRB, 0.12005E+01_JPRB, 0.13283E+01_JPRB, 0.12361E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.15696E+01_JPRB, 0.17410E+01_JPRB, 0.18965E+01_JPRB, 0.17883E+01_JPRB, 0.19829E+01_JPRB, 0.21068E+01_JPRB, 0.20485E+01_JPRB ,&
&  0.20060E+01_JPRB, 0.17965E+01_JPRB, 0.17644E+01_JPRB, 0.17368E+01_JPRB, 0.17369E+01_JPRB, 0.16197E+01_JPRB, 0.15059E+01_JPRB ,&
&  0.15912E+01_JPRB, 0.18153E+01_JPRB, 0.14437E+01_JPRB, 0.13193E+01_JPRB, 0.12109E+01_JPRB, 0.11739E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.10806E+01_JPRB, 0.10434E+01_JPRB, 0.98955E+00_JPRB, 0.88336E+00_JPRB, 0.83474E+00_JPRB, 0.77691E+00_JPRB ,&
&  0.74646E+00_JPRB, 0.70346E+00_JPRB, 0.64216E+00_JPRB, 0.61344E+00_JPRB, 0.59271E+00_JPRB, 0.58263E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.65932E+00_JPRB, 0.72339E+00_JPRB, 0.85828E+00_JPRB, 0.90473E+00_JPRB, 0.93230E+00_JPRB, 0.97148E+00_JPRB, 0.85283E+00_JPRB ,&
&  0.78218E+00_JPRB, 0.85515E+00_JPRB, 0.82968E+00_JPRB, 0.80057E+00_JPRB, 0.78362E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.85230E+00_JPRB, 0.83972E+00_JPRB, 0.82646E+00_JPRB, 0.82840E+00_JPRB, 0.83861E+00_JPRB, 0.83469E+00_JPRB, 0.83777E+00_JPRB ,&
&  0.85110E+00_JPRB, 0.84357E+00_JPRB, 0.81949E+00_JPRB, 0.80498E+00_JPRB, 0.78630E+00_JPRB, 0.78153E+00_JPRB, 0.69259E+00_JPRB ,&
&  0.60860E+00_JPRB, 0.58340E+00_JPRB, 0.56533E+00_JPRB, 0.56850E+00_JPRB, 0.58891E+00_JPRB, 0.58773E+00_JPRB, 0.58568E+00_JPRB ,&
&  0.58586E+00_JPRB, 0.58803E+00_JPRB, 0.59304E+00_JPRB, 0.59366E+00_JPRB, 0.59668E+00_JPRB, 0.61127E+00_JPRB, 0.61530E+00_JPRB ,&
&  0.58056E+00_JPRB, 0.54409E+00_JPRB, 0.53462E+00_JPRB, 0.55037E+00_JPRB, 0.60378E+00_JPRB, 0.65881E+00_JPRB, 0.69736E+00_JPRB ,&
&  0.73059E+00_JPRB, 0.76665E+00_JPRB, 0.81166E+00_JPRB, 0.86813E+00_JPRB, 0.90286E+00_JPRB, 0.93666E+00_JPRB, 0.97487E+00_JPRB ,&
&  0.10179E+01_JPRB, 0.10799E+01_JPRB, 0.11616E+01_JPRB, 0.12840E+01_JPRB, 0.12856E+01_JPRB, 0.13312E+01_JPRB, 0.14963E+01_JPRB ,&
&  0.16182E+01_JPRB, 0.17577E+01_JPRB, 0.18895E+01_JPRB, 0.22100E+01_JPRB, 0.24161E+01_JPRB, 0.23551E+01_JPRB, 0.22165E+01_JPRB ,&
&  0.21216E+01_JPRB, 0.19060E+01_JPRB, 0.21801E+01_JPRB, 0.20185E+01_JPRB, 0.18924E+01_JPRB, 0.18966E+01_JPRB, 0.18101E+01_JPRB ,&
&  0.17942E+01_JPRB, 0.18707E+01_JPRB, 0.18139E+01_JPRB, 0.17902E+01_JPRB, 0.16595E+01_JPRB, 0.16319E+01_JPRB, 0.15178E+01_JPRB ,&
&  0.15246E+01_JPRB, 0.15137E+01_JPRB, 0.13064E+01_JPRB, 0.11232E+01_JPRB, 0.95301E+00_JPRB, 0.91616E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.78709E+00_JPRB, 0.72497E+00_JPRB, 0.68591E+00_JPRB, 0.64128E+00_JPRB, 0.63176E+00_JPRB, 0.65227E+00_JPRB, 0.67125E+00_JPRB ,&
&  0.74409E+00_JPRB, 0.89653E+00_JPRB, 0.98009E+00_JPRB, 0.10326E+01_JPRB, 0.10713E+01_JPRB, 0.11098E+01_JPRB, 0.87700E+00_JPRB ,&
&  0.84358E+00_JPRB, 0.90355E+00_JPRB, 0.89169E+00_JPRB, 0.87126E+00_JPRB, 0.85730E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.96319E+00_JPRB, 0.93507E+00_JPRB, 0.90796E+00_JPRB, 0.89042E+00_JPRB, 0.87748E+00_JPRB, 0.86864E+00_JPRB, 0.87807E+00_JPRB ,&
&  0.88863E+00_JPRB, 0.87574E+00_JPRB, 0.85578E+00_JPRB, 0.83936E+00_JPRB, 0.82137E+00_JPRB, 0.81858E+00_JPRB, 0.82867E+00_JPRB ,&
&  0.69978E+00_JPRB, 0.64131E+00_JPRB, 0.57790E+00_JPRB, 0.56789E+00_JPRB, 0.62969E+00_JPRB, 0.60744E+00_JPRB, 0.59727E+00_JPRB ,&
&  0.59131E+00_JPRB, 0.60126E+00_JPRB, 0.60624E+00_JPRB, 0.63409E+00_JPRB, 0.66576E+00_JPRB, 0.70115E+00_JPRB, 0.70322E+00_JPRB ,&
&  0.65044E+00_JPRB, 0.59005E+00_JPRB, 0.56696E+00_JPRB, 0.56804E+00_JPRB, 0.60913E+00_JPRB, 0.65053E+00_JPRB, 0.72190E+00_JPRB ,&
&  0.76803E+00_JPRB, 0.80750E+00_JPRB, 0.84614E+00_JPRB, 0.89767E+00_JPRB, 0.93634E+00_JPRB, 0.98090E+00_JPRB, 0.10375E+01_JPRB ,&
&  0.11018E+01_JPRB, 0.11803E+01_JPRB, 0.12633E+01_JPRB, 0.13694E+01_JPRB, 0.13962E+01_JPRB, 0.14625E+01_JPRB, 0.15533E+01_JPRB ,&
&  0.16488E+01_JPRB, 0.17154E+01_JPRB, 0.18708E+01_JPRB, 0.25711E+01_JPRB, 0.30446E+01_JPRB, 0.30890E+01_JPRB, 0.27803E+01_JPRB ,&
&  0.24445E+01_JPRB, 0.21159E+01_JPRB, 0.20981E+01_JPRB, 0.19931E+01_JPRB, 0.18773E+01_JPRB, 0.19830E+01_JPRB, 0.19368E+01_JPRB ,&
&  0.18659E+01_JPRB, 0.18991E+01_JPRB, 0.19157E+01_JPRB, 0.19876E+01_JPRB, 0.19104E+01_JPRB, 0.19340E+01_JPRB, 0.18837E+01_JPRB ,&
&  0.19105E+01_JPRB, 0.17687E+01_JPRB, 0.13583E+01_JPRB, 0.11591E+01_JPRB, 0.96828E+00_JPRB, 0.90732E+00_JPRB, 0.10354E+01_JPRB ,&
&  0.92804E+00_JPRB, 0.87229E+00_JPRB, 0.83349E+00_JPRB, 0.80128E+00_JPRB, 0.83338E+00_JPRB, 0.95348E+00_JPRB, 0.88081E+00_JPRB ,&
&  0.85094E+00_JPRB, 0.98209E+00_JPRB, 0.11864E+01_JPRB, 0.13327E+01_JPRB, 0.13933E+01_JPRB, 0.13595E+01_JPRB, 0.11206E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.10491E+01_JPRB, 0.10287E+01_JPRB, 0.99557E+00_JPRB, 0.97595E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.11318E+01_JPRB, 0.10934E+01_JPRB, 0.10475E+01_JPRB, 0.98915E+00_JPRB, 0.95421E+00_JPRB, 0.94328E+00_JPRB, 0.94827E+00_JPRB ,&
&  0.94186E+00_JPRB, 0.91348E+00_JPRB, 0.89293E+00_JPRB, 0.87463E+00_JPRB, 0.86717E+00_JPRB, 0.87156E+00_JPRB, 0.87739E+00_JPRB ,&
&  0.89067E+00_JPRB, 0.77623E+00_JPRB, 0.66196E+00_JPRB, 0.59168E+00_JPRB, 0.61243E+00_JPRB, 0.60148E+00_JPRB, 0.60290E+00_JPRB ,&
&  0.60373E+00_JPRB, 0.64295E+00_JPRB, 0.67167E+00_JPRB, 0.73547E+00_JPRB, 0.80890E+00_JPRB, 0.86886E+00_JPRB, 0.86483E+00_JPRB ,&
&  0.79685E+00_JPRB, 0.70763E+00_JPRB, 0.66051E+00_JPRB, 0.65528E+00_JPRB, 0.69824E+00_JPRB, 0.68987E+00_JPRB, 0.75615E+00_JPRB ,&
&  0.80188E+00_JPRB, 0.84924E+00_JPRB, 0.90016E+00_JPRB, 0.94488E+00_JPRB, 0.10042E+01_JPRB, 0.10758E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.12319E+01_JPRB, 0.12954E+01_JPRB, 0.13225E+01_JPRB, 0.13712E+01_JPRB, 0.14862E+01_JPRB, 0.16116E+01_JPRB, 0.15680E+01_JPRB ,&
&  0.16240E+01_JPRB, 0.16868E+01_JPRB, 0.18322E+01_JPRB, 0.25189E+01_JPRB, 0.31390E+01_JPRB, 0.33127E+01_JPRB, 0.31875E+01_JPRB ,&
&  0.29147E+01_JPRB, 0.26337E+01_JPRB, 0.25931E+01_JPRB, 0.23586E+01_JPRB, 0.22315E+01_JPRB, 0.22463E+01_JPRB, 0.22271E+01_JPRB ,&
&  0.21499E+01_JPRB, 0.21528E+01_JPRB, 0.21971E+01_JPRB, 0.21515E+01_JPRB, 0.21629E+01_JPRB, 0.21630E+01_JPRB, 0.22406E+01_JPRB ,&
&  0.24725E+01_JPRB, 0.17248E+01_JPRB, 0.13810E+01_JPRB, 0.14276E+01_JPRB, 0.11821E+01_JPRB, 0.10404E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.93477E+00_JPRB, 0.95793E+00_JPRB, 0.10705E+01_JPRB, 0.10570E+01_JPRB, 0.12181E+01_JPRB, 0.14452E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.12829E+01_JPRB, 0.15982E+01_JPRB, 0.18218E+01_JPRB, 0.18591E+01_JPRB, 0.17197E+01_JPRB, 0.15257E+01_JPRB ,&
&  0.13584E+01_JPRB, 0.12755E+01_JPRB, 0.12378E+01_JPRB, 0.12013E+01_JPRB, 0.11724E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.13275E+01_JPRB, 0.12711E+01_JPRB, 0.12218E+01_JPRB, 0.11675E+01_JPRB, 0.11277E+01_JPRB, 0.10953E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.96233E+00_JPRB, 0.93731E+00_JPRB, 0.92734E+00_JPRB, 0.92902E+00_JPRB, 0.94012E+00_JPRB, 0.94576E+00_JPRB ,&
&  0.95919E+00_JPRB, 0.90655E+00_JPRB, 0.75644E+00_JPRB, 0.65306E+00_JPRB, 0.58456E+00_JPRB, 0.59027E+00_JPRB, 0.59773E+00_JPRB ,&
&  0.62906E+00_JPRB, 0.69848E+00_JPRB, 0.82933E+00_JPRB, 0.98387E+00_JPRB, 0.10708E+01_JPRB, 0.10818E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.97312E+00_JPRB, 0.88538E+00_JPRB, 0.79561E+00_JPRB, 0.80089E+00_JPRB, 0.79439E+00_JPRB, 0.77674E+00_JPRB, 0.83671E+00_JPRB ,&
&  0.88129E+00_JPRB, 0.93620E+00_JPRB, 0.98323E+00_JPRB, 0.10497E+01_JPRB, 0.11062E+01_JPRB, 0.11500E+01_JPRB, 0.11906E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.12712E+01_JPRB, 0.13087E+01_JPRB, 0.13393E+01_JPRB, 0.14151E+01_JPRB, 0.15205E+01_JPRB, 0.15426E+01_JPRB ,&
&  0.16892E+01_JPRB, 0.16821E+01_JPRB, 0.17368E+01_JPRB, 0.25319E+01_JPRB, 0.37161E+01_JPRB, 0.37659E+01_JPRB, 0.34306E+01_JPRB ,&
&  0.33503E+01_JPRB, 0.32941E+01_JPRB, 0.33943E+01_JPRB, 0.33417E+01_JPRB, 0.30883E+01_JPRB, 0.30115E+01_JPRB, 0.28928E+01_JPRB ,&
&  0.27906E+01_JPRB, 0.27280E+01_JPRB, 0.26546E+01_JPRB, 0.27196E+01_JPRB, 0.27687E+01_JPRB, 0.28819E+01_JPRB, 0.30042E+01_JPRB ,&
&  0.30063E+01_JPRB, 0.26739E+01_JPRB, 0.18862E+01_JPRB, 0.17839E+01_JPRB, 0.14229E+01_JPRB, 0.12703E+01_JPRB, 0.12635E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11929E+01_JPRB, 0.13182E+01_JPRB, 0.13183E+01_JPRB, 0.17586E+01_JPRB, 0.20057E+01_JPRB, 0.20147E+01_JPRB ,&
&  0.19788E+01_JPRB, 0.19121E+01_JPRB, 0.22673E+01_JPRB, 0.23896E+01_JPRB, 0.23540E+01_JPRB, 0.21889E+01_JPRB, 0.19666E+01_JPRB ,&
&  0.17285E+01_JPRB, 0.15837E+01_JPRB, 0.15128E+01_JPRB, 0.14593E+01_JPRB, 0.13903E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.15210E+01_JPRB, 0.14388E+01_JPRB, 0.13620E+01_JPRB, 0.13052E+01_JPRB, 0.12582E+01_JPRB, 0.12028E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10291E+01_JPRB, 0.10134E+01_JPRB, 0.99745E+00_JPRB, 0.99071E+00_JPRB, 0.99496E+00_JPRB, 0.10021E+01_JPRB ,&
&  0.10030E+01_JPRB, 0.91918E+00_JPRB, 0.75605E+00_JPRB, 0.64591E+00_JPRB, 0.57508E+00_JPRB, 0.55497E+00_JPRB, 0.58166E+00_JPRB ,&
&  0.64873E+00_JPRB, 0.85273E+00_JPRB, 0.11406E+01_JPRB, 0.13005E+01_JPRB, 0.13892E+01_JPRB, 0.13994E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.13222E+01_JPRB, 0.11052E+01_JPRB, 0.10616E+01_JPRB, 0.97040E+00_JPRB, 0.95726E+00_JPRB, 0.97733E+00_JPRB, 0.97357E+00_JPRB ,&
&  0.95316E+00_JPRB, 0.98040E+00_JPRB, 0.10093E+01_JPRB, 0.10603E+01_JPRB, 0.10934E+01_JPRB, 0.11404E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.13127E+01_JPRB, 0.13629E+01_JPRB, 0.13351E+01_JPRB, 0.13119E+01_JPRB, 0.15166E+01_JPRB, 0.17835E+01_JPRB, 0.19495E+01_JPRB ,&
&  0.20018E+01_JPRB, 0.21570E+01_JPRB, 0.31140E+01_JPRB, 0.37021E+01_JPRB, 0.39820E+01_JPRB, 0.43565E+01_JPRB, 0.42963E+01_JPRB ,&
&  0.39197E+01_JPRB, 0.35829E+01_JPRB, 0.34220E+01_JPRB, 0.30701E+01_JPRB, 0.35197E+01_JPRB, 0.36648E+01_JPRB, 0.36836E+01_JPRB ,&
&  0.36877E+01_JPRB, 0.35460E+01_JPRB, 0.34465E+01_JPRB, 0.34979E+01_JPRB, 0.36305E+01_JPRB, 0.37890E+01_JPRB, 0.36675E+01_JPRB ,&
&  0.32651E+01_JPRB, 0.33962E+01_JPRB, 0.29004E+01_JPRB, 0.23516E+01_JPRB, 0.19178E+01_JPRB, 0.17710E+01_JPRB, 0.16351E+01_JPRB ,&
&  0.14868E+01_JPRB, 0.14601E+01_JPRB, 0.16657E+01_JPRB, 0.19535E+01_JPRB, 0.24096E+01_JPRB, 0.28083E+01_JPRB, 0.28275E+01_JPRB ,&
&  0.26911E+01_JPRB, 0.29547E+01_JPRB, 0.28532E+01_JPRB, 0.26582E+01_JPRB, 0.25280E+01_JPRB, 0.23316E+01_JPRB, 0.21605E+01_JPRB ,&
&  0.19815E+01_JPRB, 0.18663E+01_JPRB, 0.17867E+01_JPRB, 0.17083E+01_JPRB, 0.16156E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.16321E+01_JPRB, 0.15443E+01_JPRB, 0.14603E+01_JPRB, 0.13878E+01_JPRB, 0.13132E+01_JPRB, 0.12463E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.10944E+01_JPRB, 0.10659E+01_JPRB, 0.10455E+01_JPRB, 0.10370E+01_JPRB, 0.10256E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10128E+01_JPRB, 0.88786E+00_JPRB, 0.74701E+00_JPRB, 0.68794E+00_JPRB, 0.62859E+00_JPRB, 0.57936E+00_JPRB, 0.59529E+00_JPRB ,&
&  0.73479E+00_JPRB, 0.11268E+01_JPRB, 0.14795E+01_JPRB, 0.16627E+01_JPRB, 0.17069E+01_JPRB, 0.16560E+01_JPRB, 0.15527E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.14130E+01_JPRB, 0.13849E+01_JPRB, 0.13056E+01_JPRB, 0.12094E+01_JPRB, 0.11547E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.10838E+01_JPRB, 0.10957E+01_JPRB, 0.11251E+01_JPRB, 0.11613E+01_JPRB, 0.12248E+01_JPRB, 0.13028E+01_JPRB, 0.13872E+01_JPRB ,&
&  0.14152E+01_JPRB, 0.14275E+01_JPRB, 0.13794E+01_JPRB, 0.14028E+01_JPRB, 0.14853E+01_JPRB, 0.20388E+01_JPRB, 0.23655E+01_JPRB ,&
&  0.29592E+01_JPRB, 0.34629E+01_JPRB, 0.35979E+01_JPRB, 0.42125E+01_JPRB, 0.36367E+01_JPRB, 0.40909E+01_JPRB, 0.45694E+01_JPRB ,&
&  0.40517E+01_JPRB, 0.36240E+01_JPRB, 0.34901E+01_JPRB, 0.29986E+01_JPRB, 0.29651E+01_JPRB, 0.43570E+01_JPRB, 0.46908E+01_JPRB ,&
&  0.48311E+01_JPRB, 0.51031E+01_JPRB, 0.48249E+01_JPRB, 0.45729E+01_JPRB, 0.43255E+01_JPRB, 0.39899E+01_JPRB, 0.30520E+01_JPRB ,&
&  0.30591E+01_JPRB, 0.27885E+01_JPRB, 0.28706E+01_JPRB, 0.25401E+01_JPRB, 0.23282E+01_JPRB, 0.20979E+01_JPRB, 0.18713E+01_JPRB ,&
&  0.17378E+01_JPRB, 0.17303E+01_JPRB, 0.23775E+01_JPRB, 0.32625E+01_JPRB, 0.38169E+01_JPRB, 0.34808E+01_JPRB, 0.40008E+01_JPRB ,&
&  0.38636E+01_JPRB, 0.33016E+01_JPRB, 0.26777E+01_JPRB, 0.24559E+01_JPRB, 0.22784E+01_JPRB, 0.21336E+01_JPRB, 0.20284E+01_JPRB ,&
&  0.19319E+01_JPRB, 0.18447E+01_JPRB, 0.17819E+01_JPRB, 0.17484E+01_JPRB, 0.16994E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.16639E+01_JPRB, 0.15811E+01_JPRB, 0.14922E+01_JPRB, 0.14185E+01_JPRB, 0.13475E+01_JPRB, 0.12817E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.11932E+01_JPRB, 0.11532E+01_JPRB, 0.10974E+01_JPRB, 0.10605E+01_JPRB, 0.10420E+01_JPRB, 0.10456E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.10139E+01_JPRB, 0.92658E+00_JPRB, 0.78789E+00_JPRB, 0.72791E+00_JPRB, 0.71847E+00_JPRB, 0.67113E+00_JPRB, 0.67096E+00_JPRB ,&
&  0.97393E+00_JPRB, 0.16180E+01_JPRB, 0.19287E+01_JPRB, 0.19490E+01_JPRB, 0.19340E+01_JPRB, 0.18581E+01_JPRB, 0.17069E+01_JPRB ,&
&  0.16800E+01_JPRB, 0.16022E+01_JPRB, 0.15251E+01_JPRB, 0.14357E+01_JPRB, 0.13515E+01_JPRB, 0.12662E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.11714E+01_JPRB, 0.11980E+01_JPRB, 0.12042E+01_JPRB, 0.12807E+01_JPRB, 0.13392E+01_JPRB, 0.13740E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.14273E+01_JPRB, 0.14258E+01_JPRB, 0.14728E+01_JPRB, 0.17821E+01_JPRB, 0.24878E+01_JPRB, 0.30928E+01_JPRB ,&
&  0.34760E+01_JPRB, 0.38041E+01_JPRB, 0.41653E+01_JPRB, 0.43425E+01_JPRB, 0.49279E+01_JPRB, 0.48041E+01_JPRB, 0.48518E+01_JPRB ,&
&  0.37438E+01_JPRB, 0.30565E+01_JPRB, 0.25428E+01_JPRB, 0.21374E+01_JPRB, 0.34608E+01_JPRB, 0.57732E+01_JPRB, 0.65996E+01_JPRB ,&
&  0.66491E+01_JPRB, 0.69237E+01_JPRB, 0.66483E+01_JPRB, 0.55791E+01_JPRB, 0.46906E+01_JPRB, 0.37831E+01_JPRB, 0.42561E+01_JPRB ,&
&  0.40965E+01_JPRB, 0.39545E+01_JPRB, 0.34753E+01_JPRB, 0.27841E+01_JPRB, 0.22810E+01_JPRB, 0.23893E+01_JPRB, 0.21703E+01_JPRB ,&
&  0.20105E+01_JPRB, 0.22119E+01_JPRB, 0.38831E+01_JPRB, 0.47926E+01_JPRB, 0.47337E+01_JPRB, 0.44319E+01_JPRB, 0.49369E+01_JPRB ,&
&  0.39388E+01_JPRB, 0.31435E+01_JPRB, 0.26006E+01_JPRB, 0.22424E+01_JPRB, 0.20595E+01_JPRB, 0.19542E+01_JPRB, 0.19068E+01_JPRB ,&
&  0.18769E+01_JPRB, 0.18563E+01_JPRB, 0.18100E+01_JPRB, 0.17577E+01_JPRB, 0.17115E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.17047E+01_JPRB, 0.16221E+01_JPRB, 0.15474E+01_JPRB, 0.14686E+01_JPRB, 0.13907E+01_JPRB, 0.13333E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.12413E+01_JPRB, 0.11924E+01_JPRB, 0.11448E+01_JPRB, 0.11093E+01_JPRB, 0.10921E+01_JPRB, 0.10836E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.10125E+01_JPRB, 0.86573E+00_JPRB, 0.80305E+00_JPRB, 0.79352E+00_JPRB, 0.78683E+00_JPRB, 0.85852E+00_JPRB ,&
&  0.14856E+01_JPRB, 0.22429E+01_JPRB, 0.22893E+01_JPRB, 0.21018E+01_JPRB, 0.20379E+01_JPRB, 0.17045E+01_JPRB, 0.16015E+01_JPRB ,&
&  0.16924E+01_JPRB, 0.16398E+01_JPRB, 0.15741E+01_JPRB, 0.14997E+01_JPRB, 0.14307E+01_JPRB, 0.13725E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.12859E+01_JPRB, 0.12684E+01_JPRB, 0.12943E+01_JPRB, 0.13714E+01_JPRB, 0.14115E+01_JPRB, 0.14255E+01_JPRB, 0.14330E+01_JPRB ,&
&  0.14270E+01_JPRB, 0.14235E+01_JPRB, 0.14467E+01_JPRB, 0.16406E+01_JPRB, 0.20805E+01_JPRB, 0.29495E+01_JPRB, 0.43316E+01_JPRB ,&
&  0.43817E+01_JPRB, 0.48355E+01_JPRB, 0.47020E+01_JPRB, 0.54367E+01_JPRB, 0.58414E+01_JPRB, 0.54277E+01_JPRB, 0.50039E+01_JPRB ,&
&  0.36714E+01_JPRB, 0.29932E+01_JPRB, 0.27831E+01_JPRB, 0.31887E+01_JPRB, 0.44029E+01_JPRB, 0.58339E+01_JPRB, 0.74374E+01_JPRB ,&
&  0.66196E+01_JPRB, 0.65533E+01_JPRB, 0.65001E+01_JPRB, 0.59586E+01_JPRB, 0.36445E+01_JPRB, 0.28244E+01_JPRB, 0.40786E+01_JPRB ,&
&  0.40951E+01_JPRB, 0.24469E+01_JPRB, 0.17179E+01_JPRB, 0.20209E+01_JPRB, 0.15149E+01_JPRB, 0.17256E+01_JPRB, 0.23192E+01_JPRB ,&
&  0.25128E+01_JPRB, 0.30984E+01_JPRB, 0.48232E+01_JPRB, 0.46986E+01_JPRB, 0.47814E+01_JPRB, 0.55908E+01_JPRB, 0.45908E+01_JPRB ,&
&  0.35889E+01_JPRB, 0.27838E+01_JPRB, 0.25841E+01_JPRB, 0.22612E+01_JPRB, 0.20970E+01_JPRB, 0.19986E+01_JPRB, 0.19793E+01_JPRB ,&
&  0.20176E+01_JPRB, 0.20137E+01_JPRB, 0.19562E+01_JPRB, 0.18658E+01_JPRB, 0.17796E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.17689E+01_JPRB, 0.16922E+01_JPRB, 0.16147E+01_JPRB, 0.15420E+01_JPRB, 0.14759E+01_JPRB, 0.14179E+01_JPRB, 0.13421E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.11965E+01_JPRB, 0.11740E+01_JPRB, 0.11484E+01_JPRB, 0.11338E+01_JPRB, 0.11336E+01_JPRB, 0.11142E+01_JPRB ,&
&  0.10802E+01_JPRB, 0.10411E+01_JPRB, 0.97847E+00_JPRB, 0.91626E+00_JPRB, 0.89005E+00_JPRB, 0.87249E+00_JPRB, 0.11743E+01_JPRB ,&
&  0.22029E+01_JPRB, 0.29012E+01_JPRB, 0.26215E+01_JPRB, 0.21881E+01_JPRB, 0.18401E+01_JPRB, 0.15817E+01_JPRB, 0.16785E+01_JPRB ,&
&  0.17108E+01_JPRB, 0.16914E+01_JPRB, 0.16110E+01_JPRB, 0.15434E+01_JPRB, 0.14834E+01_JPRB, 0.14258E+01_JPRB, 0.13936E+01_JPRB ,&
&  0.13625E+01_JPRB, 0.13490E+01_JPRB, 0.14015E+01_JPRB, 0.14197E+01_JPRB, 0.14343E+01_JPRB, 0.14266E+01_JPRB, 0.14646E+01_JPRB ,&
&  0.14596E+01_JPRB, 0.14786E+01_JPRB, 0.15623E+01_JPRB, 0.18431E+01_JPRB, 0.26401E+01_JPRB, 0.38585E+01_JPRB, 0.42728E+01_JPRB ,&
&  0.43645E+01_JPRB, 0.46831E+01_JPRB, 0.55700E+01_JPRB, 0.56956E+01_JPRB, 0.58380E+01_JPRB, 0.56344E+01_JPRB, 0.47675E+01_JPRB ,&
&  0.41653E+01_JPRB, 0.37555E+01_JPRB, 0.32998E+01_JPRB, 0.37771E+01_JPRB, 0.54583E+01_JPRB, 0.58751E+01_JPRB, 0.69747E+01_JPRB ,&
&  0.69229E+01_JPRB, 0.62085E+01_JPRB, 0.48573E+01_JPRB, 0.37768E+01_JPRB, 0.39977E+01_JPRB, 0.29960E+01_JPRB, 0.19160E+01_JPRB ,&
&  0.16901E+01_JPRB, 0.14953E+01_JPRB, 0.13096E+01_JPRB, 0.13064E+01_JPRB, 0.12405E+01_JPRB, 0.15730E+01_JPRB, 0.23666E+01_JPRB ,&
&  0.32422E+01_JPRB, 0.39518E+01_JPRB, 0.45572E+01_JPRB, 0.43059E+01_JPRB, 0.41565E+01_JPRB, 0.50284E+01_JPRB, 0.45789E+01_JPRB ,&
&  0.29462E+01_JPRB, 0.24445E+01_JPRB, 0.24051E+01_JPRB, 0.22287E+01_JPRB, 0.21031E+01_JPRB, 0.20730E+01_JPRB, 0.21356E+01_JPRB ,&
&  0.22051E+01_JPRB, 0.21871E+01_JPRB, 0.20990E+01_JPRB, 0.19662E+01_JPRB, 0.18512E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.17720E+01_JPRB, 0.16729E+01_JPRB, 0.16077E+01_JPRB, 0.15692E+01_JPRB, 0.15267E+01_JPRB, 0.14457E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.12251E+01_JPRB, 0.11861E+01_JPRB, 0.11775E+01_JPRB, 0.11631E+01_JPRB, 0.11485E+01_JPRB, 0.11401E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.10491E+01_JPRB, 0.10454E+01_JPRB, 0.10295E+01_JPRB, 0.10515E+01_JPRB, 0.10723E+01_JPRB, 0.16206E+01_JPRB ,&
&  0.30132E+01_JPRB, 0.34633E+01_JPRB, 0.28043E+01_JPRB, 0.21335E+01_JPRB, 0.18479E+01_JPRB, 0.17583E+01_JPRB, 0.17945E+01_JPRB ,&
&  0.17286E+01_JPRB, 0.16429E+01_JPRB, 0.15062E+01_JPRB, 0.14468E+01_JPRB, 0.14332E+01_JPRB, 0.14128E+01_JPRB, 0.14041E+01_JPRB ,&
&  0.13812E+01_JPRB, 0.13957E+01_JPRB, 0.13960E+01_JPRB, 0.14024E+01_JPRB, 0.14269E+01_JPRB, 0.14643E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.16392E+01_JPRB, 0.18223E+01_JPRB, 0.23739E+01_JPRB, 0.32498E+01_JPRB, 0.36947E+01_JPRB, 0.43812E+01_JPRB ,&
&  0.50879E+01_JPRB, 0.56205E+01_JPRB, 0.53216E+01_JPRB, 0.61428E+01_JPRB, 0.58997E+01_JPRB, 0.52203E+01_JPRB, 0.46422E+01_JPRB ,&
&  0.42677E+01_JPRB, 0.47661E+01_JPRB, 0.39341E+01_JPRB, 0.36434E+01_JPRB, 0.59503E+01_JPRB, 0.66019E+01_JPRB, 0.70877E+01_JPRB ,&
&  0.75431E+01_JPRB, 0.64352E+01_JPRB, 0.51642E+01_JPRB, 0.45805E+01_JPRB, 0.61921E+01_JPRB, 0.58858E+01_JPRB, 0.23814E+01_JPRB ,&
&  0.17280E+01_JPRB, 0.15175E+01_JPRB, 0.12875E+01_JPRB, 0.11799E+01_JPRB, 0.11872E+01_JPRB, 0.15569E+01_JPRB, 0.28319E+01_JPRB ,&
&  0.31789E+01_JPRB, 0.42708E+01_JPRB, 0.43729E+01_JPRB, 0.42267E+01_JPRB, 0.38107E+01_JPRB, 0.37022E+01_JPRB, 0.33569E+01_JPRB ,&
&  0.29563E+01_JPRB, 0.25760E+01_JPRB, 0.22126E+01_JPRB, 0.20070E+01_JPRB, 0.19310E+01_JPRB, 0.19819E+01_JPRB, 0.21250E+01_JPRB ,&
&  0.21986E+01_JPRB, 0.21980E+01_JPRB, 0.21209E+01_JPRB, 0.19967E+01_JPRB, 0.18701E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.17076E+01_JPRB, 0.16628E+01_JPRB, 0.16338E+01_JPRB, 0.15756E+01_JPRB, 0.15069E+01_JPRB, 0.14148E+01_JPRB, 0.13004E+01_JPRB ,&
&  0.12378E+01_JPRB, 0.12087E+01_JPRB, 0.12019E+01_JPRB, 0.11760E+01_JPRB, 0.11429E+01_JPRB, 0.11204E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10436E+01_JPRB, 0.10392E+01_JPRB, 0.11349E+01_JPRB, 0.12676E+01_JPRB, 0.15486E+01_JPRB, 0.22448E+01_JPRB ,&
&  0.36873E+01_JPRB, 0.38519E+01_JPRB, 0.30091E+01_JPRB, 0.24168E+01_JPRB, 0.21224E+01_JPRB, 0.20375E+01_JPRB, 0.18585E+01_JPRB ,&
&  0.16837E+01_JPRB, 0.15892E+01_JPRB, 0.14501E+01_JPRB, 0.14470E+01_JPRB, 0.14491E+01_JPRB, 0.14592E+01_JPRB, 0.14629E+01_JPRB ,&
&  0.14844E+01_JPRB, 0.14333E+01_JPRB, 0.14364E+01_JPRB, 0.14517E+01_JPRB, 0.14908E+01_JPRB, 0.15771E+01_JPRB, 0.16660E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.18975E+01_JPRB, 0.22490E+01_JPRB, 0.28693E+01_JPRB, 0.31272E+01_JPRB, 0.39899E+01_JPRB, 0.44704E+01_JPRB ,&
&  0.48966E+01_JPRB, 0.50187E+01_JPRB, 0.48241E+01_JPRB, 0.54124E+01_JPRB, 0.56982E+01_JPRB, 0.51431E+01_JPRB, 0.46780E+01_JPRB ,&
&  0.49952E+01_JPRB, 0.44103E+01_JPRB, 0.39837E+01_JPRB, 0.36492E+01_JPRB, 0.49085E+01_JPRB, 0.83696E+01_JPRB, 0.65875E+01_JPRB ,&
&  0.63767E+01_JPRB, 0.72235E+01_JPRB, 0.61540E+01_JPRB, 0.57262E+01_JPRB, 0.64893E+01_JPRB, 0.76107E+01_JPRB, 0.58369E+01_JPRB ,&
&  0.21840E+01_JPRB, 0.15028E+01_JPRB, 0.15075E+01_JPRB, 0.14689E+01_JPRB, 0.12898E+01_JPRB, 0.16501E+01_JPRB, 0.45188E+01_JPRB ,&
&  0.37755E+01_JPRB, 0.48343E+01_JPRB, 0.46733E+01_JPRB, 0.42395E+01_JPRB, 0.38882E+01_JPRB, 0.32998E+01_JPRB, 0.29814E+01_JPRB ,&
&  0.24497E+01_JPRB, 0.21152E+01_JPRB, 0.19491E+01_JPRB, 0.18615E+01_JPRB, 0.18557E+01_JPRB, 0.18925E+01_JPRB, 0.19972E+01_JPRB ,&
&  0.20667E+01_JPRB, 0.20798E+01_JPRB, 0.20090E+01_JPRB, 0.18983E+01_JPRB, 0.17906E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.16589E+01_JPRB, 0.16373E+01_JPRB, 0.15975E+01_JPRB, 0.15243E+01_JPRB, 0.14541E+01_JPRB, 0.13562E+01_JPRB, 0.12783E+01_JPRB ,&
&  0.12592E+01_JPRB, 0.12345E+01_JPRB, 0.12104E+01_JPRB, 0.11782E+01_JPRB, 0.11398E+01_JPRB, 0.11081E+01_JPRB, 0.10937E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.10469E+01_JPRB, 0.10390E+01_JPRB, 0.11534E+01_JPRB, 0.15324E+01_JPRB, 0.20130E+01_JPRB, 0.25061E+01_JPRB ,&
&  0.42413E+01_JPRB, 0.42459E+01_JPRB, 0.28914E+01_JPRB, 0.25000E+01_JPRB, 0.23634E+01_JPRB, 0.22479E+01_JPRB, 0.19503E+01_JPRB ,&
&  0.17411E+01_JPRB, 0.16346E+01_JPRB, 0.14960E+01_JPRB, 0.14585E+01_JPRB, 0.14737E+01_JPRB, 0.15273E+01_JPRB, 0.15657E+01_JPRB ,&
&  0.15106E+01_JPRB, 0.15153E+01_JPRB, 0.15703E+01_JPRB, 0.15944E+01_JPRB, 0.16430E+01_JPRB, 0.17590E+01_JPRB, 0.18704E+01_JPRB ,&
&  0.20214E+01_JPRB, 0.22654E+01_JPRB, 0.26994E+01_JPRB, 0.30641E+01_JPRB, 0.35634E+01_JPRB, 0.39378E+01_JPRB, 0.44305E+01_JPRB ,&
&  0.44638E+01_JPRB, 0.45814E+01_JPRB, 0.47372E+01_JPRB, 0.49225E+01_JPRB, 0.50406E+01_JPRB, 0.49460E+01_JPRB, 0.48712E+01_JPRB ,&
&  0.50953E+01_JPRB, 0.43995E+01_JPRB, 0.38514E+01_JPRB, 0.37158E+01_JPRB, 0.46242E+01_JPRB, 0.74751E+01_JPRB, 0.84028E+01_JPRB ,&
&  0.85231E+01_JPRB, 0.61616E+01_JPRB, 0.56321E+01_JPRB, 0.65915E+01_JPRB, 0.63714E+01_JPRB, 0.68202E+01_JPRB, 0.89626E+01_JPRB ,&
&  0.78718E+01_JPRB, 0.39771E+01_JPRB, 0.31496E+01_JPRB, 0.40073E+01_JPRB, 0.23719E+01_JPRB, 0.18101E+01_JPRB, 0.29557E+01_JPRB ,&
&  0.36816E+01_JPRB, 0.43815E+01_JPRB, 0.43425E+01_JPRB, 0.41297E+01_JPRB, 0.37936E+01_JPRB, 0.29645E+01_JPRB, 0.24490E+01_JPRB ,&
&  0.19920E+01_JPRB, 0.18369E+01_JPRB, 0.17628E+01_JPRB, 0.16762E+01_JPRB, 0.16489E+01_JPRB, 0.17121E+01_JPRB, 0.17988E+01_JPRB ,&
&  0.18684E+01_JPRB, 0.18885E+01_JPRB, 0.18656E+01_JPRB, 0.18015E+01_JPRB, 0.17149E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.15946E+01_JPRB, 0.15832E+01_JPRB, 0.15634E+01_JPRB, 0.14845E+01_JPRB, 0.14042E+01_JPRB, 0.13257E+01_JPRB, 0.13021E+01_JPRB ,&
&  0.12687E+01_JPRB, 0.12389E+01_JPRB, 0.12070E+01_JPRB, 0.11628E+01_JPRB, 0.11416E+01_JPRB, 0.11142E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.11093E+01_JPRB, 0.11611E+01_JPRB, 0.14747E+01_JPRB, 0.20997E+01_JPRB, 0.34038E+01_JPRB, 0.29147E+01_JPRB ,&
&  0.42563E+01_JPRB, 0.46067E+01_JPRB, 0.29673E+01_JPRB, 0.27273E+01_JPRB, 0.26313E+01_JPRB, 0.26030E+01_JPRB, 0.21879E+01_JPRB ,&
&  0.18925E+01_JPRB, 0.17592E+01_JPRB, 0.16590E+01_JPRB, 0.15644E+01_JPRB, 0.15673E+01_JPRB, 0.16040E+01_JPRB, 0.15668E+01_JPRB ,&
&  0.15941E+01_JPRB, 0.16810E+01_JPRB, 0.17251E+01_JPRB, 0.17549E+01_JPRB, 0.18480E+01_JPRB, 0.19737E+01_JPRB, 0.21256E+01_JPRB ,&
&  0.23668E+01_JPRB, 0.26740E+01_JPRB, 0.31204E+01_JPRB, 0.32954E+01_JPRB, 0.35934E+01_JPRB, 0.39794E+01_JPRB, 0.41819E+01_JPRB ,&
&  0.40293E+01_JPRB, 0.38450E+01_JPRB, 0.42475E+01_JPRB, 0.45974E+01_JPRB, 0.46615E+01_JPRB, 0.46747E+01_JPRB, 0.49026E+01_JPRB ,&
&  0.53720E+01_JPRB, 0.52096E+01_JPRB, 0.53312E+01_JPRB, 0.46979E+01_JPRB, 0.56393E+01_JPRB, 0.75865E+01_JPRB, 0.77392E+01_JPRB ,&
&  0.93931E+01_JPRB, 0.81368E+01_JPRB, 0.76948E+01_JPRB, 0.70363E+01_JPRB, 0.62988E+01_JPRB, 0.63400E+01_JPRB, 0.69262E+01_JPRB ,&
&  0.72542E+01_JPRB, 0.79342E+01_JPRB, 0.74247E+01_JPRB, 0.44701E+01_JPRB, 0.28151E+01_JPRB, 0.21627E+01_JPRB, 0.24840E+01_JPRB ,&
&  0.33239E+01_JPRB, 0.35159E+01_JPRB, 0.34909E+01_JPRB, 0.50027E+01_JPRB, 0.32517E+01_JPRB, 0.23902E+01_JPRB, 0.19056E+01_JPRB ,&
&  0.16598E+01_JPRB, 0.15976E+01_JPRB, 0.15447E+01_JPRB, 0.14587E+01_JPRB, 0.14484E+01_JPRB, 0.14979E+01_JPRB, 0.15736E+01_JPRB ,&
&  0.16551E+01_JPRB, 0.17119E+01_JPRB, 0.17432E+01_JPRB, 0.17047E+01_JPRB, 0.16389E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.15003E+01_JPRB, 0.15011E+01_JPRB, 0.14589E+01_JPRB, 0.13998E+01_JPRB, 0.13531E+01_JPRB, 0.13151E+01_JPRB, 0.13136E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.12027E+01_JPRB, 0.11760E+01_JPRB, 0.11525E+01_JPRB, 0.11679E+01_JPRB, 0.11909E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.13002E+01_JPRB, 0.13980E+01_JPRB, 0.16685E+01_JPRB, 0.23028E+01_JPRB, 0.31880E+01_JPRB, 0.44174E+01_JPRB, 0.45061E+01_JPRB ,&
&  0.41893E+01_JPRB, 0.50531E+01_JPRB, 0.29914E+01_JPRB, 0.30174E+01_JPRB, 0.30574E+01_JPRB, 0.34180E+01_JPRB, 0.29037E+01_JPRB ,&
&  0.22603E+01_JPRB, 0.20503E+01_JPRB, 0.19223E+01_JPRB, 0.17307E+01_JPRB, 0.16780E+01_JPRB, 0.16596E+01_JPRB, 0.16815E+01_JPRB ,&
&  0.17802E+01_JPRB, 0.18865E+01_JPRB, 0.19541E+01_JPRB, 0.20315E+01_JPRB, 0.21407E+01_JPRB, 0.22659E+01_JPRB, 0.24853E+01_JPRB ,&
&  0.27974E+01_JPRB, 0.31406E+01_JPRB, 0.33297E+01_JPRB, 0.34593E+01_JPRB, 0.35684E+01_JPRB, 0.38442E+01_JPRB, 0.39876E+01_JPRB ,&
&  0.39131E+01_JPRB, 0.37500E+01_JPRB, 0.40286E+01_JPRB, 0.41331E+01_JPRB, 0.42115E+01_JPRB, 0.47097E+01_JPRB, 0.48988E+01_JPRB ,&
&  0.55682E+01_JPRB, 0.53187E+01_JPRB, 0.62048E+01_JPRB, 0.58933E+01_JPRB, 0.62144E+01_JPRB, 0.69842E+01_JPRB, 0.61891E+01_JPRB ,&
&  0.61861E+01_JPRB, 0.68022E+01_JPRB, 0.69007E+01_JPRB, 0.67829E+01_JPRB, 0.64864E+01_JPRB, 0.62586E+01_JPRB, 0.58932E+01_JPRB ,&
&  0.51470E+01_JPRB, 0.49982E+01_JPRB, 0.44006E+01_JPRB, 0.28880E+01_JPRB, 0.22569E+01_JPRB, 0.20335E+01_JPRB, 0.22679E+01_JPRB ,&
&  0.28702E+01_JPRB, 0.30736E+01_JPRB, 0.35256E+01_JPRB, 0.35036E+01_JPRB, 0.22677E+01_JPRB, 0.17172E+01_JPRB, 0.15420E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.14134E+01_JPRB, 0.13306E+01_JPRB, 0.12620E+01_JPRB, 0.12732E+01_JPRB, 0.13376E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.15440E+01_JPRB, 0.15440E+01_JPRB, 0.15138E+01_JPRB, 0.15047E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.14932E+01_JPRB, 0.14974E+01_JPRB, 0.14957E+01_JPRB, 0.15181E+01_JPRB, 0.15543E+01_JPRB, 0.16076E+01_JPRB, 0.15915E+01_JPRB ,&
&  0.14253E+01_JPRB, 0.11685E+01_JPRB, 0.11871E+01_JPRB, 0.12365E+01_JPRB, 0.13414E+01_JPRB, 0.14445E+01_JPRB, 0.15834E+01_JPRB ,&
&  0.17972E+01_JPRB, 0.20920E+01_JPRB, 0.26034E+01_JPRB, 0.33598E+01_JPRB, 0.42932E+01_JPRB, 0.54277E+01_JPRB, 0.57224E+01_JPRB ,&
&  0.54216E+01_JPRB, 0.37953E+01_JPRB, 0.32761E+01_JPRB, 0.32292E+01_JPRB, 0.30634E+01_JPRB, 0.34256E+01_JPRB, 0.32878E+01_JPRB ,&
&  0.26779E+01_JPRB, 0.23689E+01_JPRB, 0.22292E+01_JPRB, 0.20488E+01_JPRB, 0.19160E+01_JPRB, 0.19757E+01_JPRB, 0.20723E+01_JPRB ,&
&  0.22085E+01_JPRB, 0.22961E+01_JPRB, 0.23287E+01_JPRB, 0.23699E+01_JPRB, 0.24693E+01_JPRB, 0.26173E+01_JPRB, 0.28989E+01_JPRB ,&
&  0.32078E+01_JPRB, 0.34533E+01_JPRB, 0.35632E+01_JPRB, 0.34997E+01_JPRB, 0.36013E+01_JPRB, 0.37519E+01_JPRB, 0.38133E+01_JPRB ,&
&  0.38167E+01_JPRB, 0.39249E+01_JPRB, 0.40546E+01_JPRB, 0.42161E+01_JPRB, 0.41467E+01_JPRB, 0.47535E+01_JPRB, 0.48911E+01_JPRB ,&
&  0.52646E+01_JPRB, 0.51831E+01_JPRB, 0.60596E+01_JPRB, 0.55072E+01_JPRB, 0.61553E+01_JPRB, 0.60925E+01_JPRB, 0.55879E+01_JPRB ,&
&  0.55825E+01_JPRB, 0.63404E+01_JPRB, 0.65920E+01_JPRB, 0.64868E+01_JPRB, 0.63556E+01_JPRB, 0.55194E+01_JPRB, 0.56994E+01_JPRB ,&
&  0.44115E+01_JPRB, 0.43825E+01_JPRB, 0.29221E+01_JPRB, 0.21651E+01_JPRB, 0.17719E+01_JPRB, 0.20413E+01_JPRB, 0.23747E+01_JPRB ,&
&  0.28160E+01_JPRB, 0.27497E+01_JPRB, 0.24186E+01_JPRB, 0.23068E+01_JPRB, 0.18164E+01_JPRB, 0.14891E+01_JPRB, 0.14102E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13050E+01_JPRB, 0.12310E+01_JPRB, 0.12169E+01_JPRB, 0.12377E+01_JPRB, 0.13211E+01_JPRB, 0.13732E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.14532E+01_JPRB, 0.14520E+01_JPRB, 0.14596E+01_JPRB, 0.14708E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.14861E+01_JPRB, 0.14613E+01_JPRB, 0.14547E+01_JPRB, 0.14444E+01_JPRB, 0.14261E+01_JPRB, 0.13579E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12149E+01_JPRB, 0.13244E+01_JPRB, 0.14763E+01_JPRB, 0.16666E+01_JPRB, 0.18750E+01_JPRB, 0.21185E+01_JPRB ,&
&  0.24343E+01_JPRB, 0.28388E+01_JPRB, 0.33490E+01_JPRB, 0.38096E+01_JPRB, 0.42908E+01_JPRB, 0.49266E+01_JPRB, 0.53725E+01_JPRB ,&
&  0.47418E+01_JPRB, 0.29659E+01_JPRB, 0.32439E+01_JPRB, 0.33840E+01_JPRB, 0.32786E+01_JPRB, 0.33038E+01_JPRB, 0.31035E+01_JPRB ,&
&  0.27745E+01_JPRB, 0.26300E+01_JPRB, 0.25650E+01_JPRB, 0.24869E+01_JPRB, 0.23606E+01_JPRB, 0.24459E+01_JPRB, 0.25245E+01_JPRB ,&
&  0.25609E+01_JPRB, 0.25415E+01_JPRB, 0.25348E+01_JPRB, 0.26487E+01_JPRB, 0.28714E+01_JPRB, 0.30302E+01_JPRB, 0.32482E+01_JPRB ,&
&  0.34361E+01_JPRB, 0.36653E+01_JPRB, 0.37832E+01_JPRB, 0.38516E+01_JPRB, 0.40156E+01_JPRB, 0.41258E+01_JPRB, 0.41194E+01_JPRB ,&
&  0.40571E+01_JPRB, 0.40379E+01_JPRB, 0.42091E+01_JPRB, 0.47243E+01_JPRB, 0.50402E+01_JPRB, 0.46369E+01_JPRB, 0.49308E+01_JPRB ,&
&  0.51626E+01_JPRB, 0.54332E+01_JPRB, 0.53384E+01_JPRB, 0.69861E+01_JPRB, 0.53401E+01_JPRB, 0.53506E+01_JPRB, 0.46968E+01_JPRB ,&
&  0.53416E+01_JPRB, 0.56823E+01_JPRB, 0.60899E+01_JPRB, 0.61564E+01_JPRB, 0.58680E+01_JPRB, 0.52042E+01_JPRB, 0.54440E+01_JPRB ,&
&  0.46783E+01_JPRB, 0.39333E+01_JPRB, 0.27289E+01_JPRB, 0.19718E+01_JPRB, 0.19516E+01_JPRB, 0.32194E+01_JPRB, 0.31607E+01_JPRB ,&
&  0.27580E+01_JPRB, 0.22061E+01_JPRB, 0.21323E+01_JPRB, 0.24192E+01_JPRB, 0.18213E+01_JPRB, 0.14007E+01_JPRB, 0.12923E+01_JPRB ,&
&  0.12981E+01_JPRB, 0.12767E+01_JPRB, 0.12276E+01_JPRB, 0.12310E+01_JPRB, 0.12856E+01_JPRB, 0.13692E+01_JPRB, 0.14249E+01_JPRB ,&
&  0.14485E+01_JPRB, 0.14343E+01_JPRB, 0.14399E+01_JPRB, 0.14539E+01_JPRB, 0.14920E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.13980E+01_JPRB, 0.13489E+01_JPRB, 0.13079E+01_JPRB, 0.12661E+01_JPRB, 0.12231E+01_JPRB, 0.11995E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.12451E+01_JPRB, 0.13303E+01_JPRB, 0.14589E+01_JPRB, 0.16185E+01_JPRB, 0.18256E+01_JPRB, 0.20762E+01_JPRB, 0.23511E+01_JPRB ,&
&  0.27058E+01_JPRB, 0.30459E+01_JPRB, 0.32361E+01_JPRB, 0.33726E+01_JPRB, 0.35446E+01_JPRB, 0.37990E+01_JPRB, 0.40378E+01_JPRB ,&
&  0.42399E+01_JPRB, 0.40866E+01_JPRB, 0.36407E+01_JPRB, 0.27277E+01_JPRB, 0.25162E+01_JPRB, 0.29457E+01_JPRB, 0.29071E+01_JPRB ,&
&  0.28545E+01_JPRB, 0.27558E+01_JPRB, 0.26814E+01_JPRB, 0.26216E+01_JPRB, 0.26254E+01_JPRB, 0.26844E+01_JPRB, 0.27234E+01_JPRB ,&
&  0.27201E+01_JPRB, 0.27395E+01_JPRB, 0.28086E+01_JPRB, 0.29645E+01_JPRB, 0.31987E+01_JPRB, 0.33609E+01_JPRB, 0.35256E+01_JPRB ,&
&  0.36675E+01_JPRB, 0.39109E+01_JPRB, 0.40582E+01_JPRB, 0.42508E+01_JPRB, 0.43524E+01_JPRB, 0.45241E+01_JPRB, 0.47607E+01_JPRB ,&
&  0.48223E+01_JPRB, 0.49341E+01_JPRB, 0.51781E+01_JPRB, 0.55932E+01_JPRB, 0.57986E+01_JPRB, 0.51550E+01_JPRB, 0.50547E+01_JPRB ,&
&  0.55009E+01_JPRB, 0.60166E+01_JPRB, 0.56543E+01_JPRB, 0.72858E+01_JPRB, 0.46508E+01_JPRB, 0.49920E+01_JPRB, 0.48806E+01_JPRB ,&
&  0.47188E+01_JPRB, 0.48646E+01_JPRB, 0.51842E+01_JPRB, 0.53919E+01_JPRB, 0.50562E+01_JPRB, 0.44994E+01_JPRB, 0.52285E+01_JPRB ,&
&  0.47765E+01_JPRB, 0.34700E+01_JPRB, 0.25078E+01_JPRB, 0.17366E+01_JPRB, 0.21681E+01_JPRB, 0.37260E+01_JPRB, 0.28103E+01_JPRB ,&
&  0.18781E+01_JPRB, 0.19651E+01_JPRB, 0.21692E+01_JPRB, 0.24016E+01_JPRB, 0.18681E+01_JPRB, 0.12745E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.12768E+01_JPRB, 0.12857E+01_JPRB, 0.13111E+01_JPRB, 0.13339E+01_JPRB, 0.13729E+01_JPRB, 0.14277E+01_JPRB, 0.14746E+01_JPRB ,&
&  0.14777E+01_JPRB, 0.14674E+01_JPRB, 0.14884E+01_JPRB, 0.14810E+01_JPRB, 0.14474E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.11927E+01_JPRB, 0.11320E+01_JPRB, 0.10848E+01_JPRB, 0.10690E+01_JPRB, 0.10767E+01_JPRB, 0.11016E+01_JPRB, 0.11480E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.13364E+01_JPRB, 0.14441E+01_JPRB, 0.15546E+01_JPRB, 0.16967E+01_JPRB, 0.18521E+01_JPRB, 0.20400E+01_JPRB ,&
&  0.22459E+01_JPRB, 0.24597E+01_JPRB, 0.25530E+01_JPRB, 0.25580E+01_JPRB, 0.26009E+01_JPRB, 0.26621E+01_JPRB, 0.28982E+01_JPRB ,&
&  0.33455E+01_JPRB, 0.37282E+01_JPRB, 0.39347E+01_JPRB, 0.37421E+01_JPRB, 0.30108E+01_JPRB, 0.30752E+01_JPRB, 0.33425E+01_JPRB ,&
&  0.34149E+01_JPRB, 0.30860E+01_JPRB, 0.28241E+01_JPRB, 0.27908E+01_JPRB, 0.27953E+01_JPRB, 0.28470E+01_JPRB, 0.28818E+01_JPRB ,&
&  0.28890E+01_JPRB, 0.29286E+01_JPRB, 0.30322E+01_JPRB, 0.31591E+01_JPRB, 0.33413E+01_JPRB, 0.35359E+01_JPRB, 0.37432E+01_JPRB ,&
&  0.38688E+01_JPRB, 0.40151E+01_JPRB, 0.40299E+01_JPRB, 0.39626E+01_JPRB, 0.41090E+01_JPRB, 0.45829E+01_JPRB, 0.52376E+01_JPRB ,&
&  0.56485E+01_JPRB, 0.56694E+01_JPRB, 0.61156E+01_JPRB, 0.66276E+01_JPRB, 0.65818E+01_JPRB, 0.60741E+01_JPRB, 0.60077E+01_JPRB ,&
&  0.62861E+01_JPRB, 0.70395E+01_JPRB, 0.52180E+01_JPRB, 0.61142E+01_JPRB, 0.62551E+01_JPRB, 0.53540E+01_JPRB, 0.44673E+01_JPRB ,&
&  0.39878E+01_JPRB, 0.35914E+01_JPRB, 0.30700E+01_JPRB, 0.32352E+01_JPRB, 0.34251E+01_JPRB, 0.36992E+01_JPRB, 0.43435E+01_JPRB ,&
&  0.41170E+01_JPRB, 0.29987E+01_JPRB, 0.20767E+01_JPRB, 0.16216E+01_JPRB, 0.19224E+01_JPRB, 0.26633E+01_JPRB, 0.19903E+01_JPRB ,&
&  0.16543E+01_JPRB, 0.18379E+01_JPRB, 0.19312E+01_JPRB, 0.22350E+01_JPRB, 0.20973E+01_JPRB, 0.11958E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.12938E+01_JPRB, 0.13120E+01_JPRB, 0.13287E+01_JPRB, 0.13404E+01_JPRB, 0.13737E+01_JPRB, 0.14020E+01_JPRB ,&
&  0.14068E+01_JPRB, 0.13993E+01_JPRB, 0.13730E+01_JPRB, 0.13236E+01_JPRB, 0.12532E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.95776E+00_JPRB, 0.94949E+00_JPRB, 0.93901E+00_JPRB, 0.96207E+00_JPRB, 0.98881E+00_JPRB, 0.10001E+01_JPRB, 0.10311E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.11119E+01_JPRB, 0.11504E+01_JPRB, 0.11854E+01_JPRB, 0.12925E+01_JPRB, 0.14097E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.16209E+01_JPRB, 0.16805E+01_JPRB, 0.16713E+01_JPRB, 0.16140E+01_JPRB, 0.15611E+01_JPRB, 0.14408E+01_JPRB, 0.13266E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.18148E+01_JPRB, 0.24874E+01_JPRB, 0.31682E+01_JPRB, 0.38167E+01_JPRB, 0.34493E+01_JPRB, 0.38371E+01_JPRB ,&
&  0.33733E+01_JPRB, 0.26992E+01_JPRB, 0.26952E+01_JPRB, 0.28057E+01_JPRB, 0.29326E+01_JPRB, 0.29324E+01_JPRB, 0.29849E+01_JPRB ,&
&  0.30246E+01_JPRB, 0.30765E+01_JPRB, 0.31602E+01_JPRB, 0.33185E+01_JPRB, 0.34575E+01_JPRB, 0.35682E+01_JPRB, 0.36997E+01_JPRB ,&
&  0.38177E+01_JPRB, 0.38849E+01_JPRB, 0.39820E+01_JPRB, 0.37485E+01_JPRB, 0.41245E+01_JPRB, 0.48579E+01_JPRB, 0.55000E+01_JPRB ,&
&  0.58260E+01_JPRB, 0.60310E+01_JPRB, 0.66467E+01_JPRB, 0.69472E+01_JPRB, 0.71541E+01_JPRB, 0.70929E+01_JPRB, 0.70926E+01_JPRB ,&
&  0.71514E+01_JPRB, 0.65734E+01_JPRB, 0.48379E+01_JPRB, 0.51873E+01_JPRB, 0.46021E+01_JPRB, 0.36932E+01_JPRB, 0.36677E+01_JPRB ,&
&  0.32308E+01_JPRB, 0.27712E+01_JPRB, 0.21694E+01_JPRB, 0.21142E+01_JPRB, 0.23858E+01_JPRB, 0.27119E+01_JPRB, 0.34674E+01_JPRB ,&
&  0.29030E+01_JPRB, 0.22355E+01_JPRB, 0.16532E+01_JPRB, 0.15532E+01_JPRB, 0.16521E+01_JPRB, 0.18304E+01_JPRB, 0.17905E+01_JPRB ,&
&  0.17791E+01_JPRB, 0.16982E+01_JPRB, 0.18095E+01_JPRB, 0.20601E+01_JPRB, 0.13446E+01_JPRB, 0.10559E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.11221E+01_JPRB, 0.11267E+01_JPRB, 0.11207E+01_JPRB, 0.11340E+01_JPRB, 0.11605E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.11415E+01_JPRB, 0.10840E+01_JPRB, 0.10290E+01_JPRB, 0.99262E+00_JPRB, 0.96608E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.74337E+00_JPRB, 0.74671E+00_JPRB, 0.75369E+00_JPRB, 0.76748E+00_JPRB, 0.77372E+00_JPRB, 0.77704E+00_JPRB, 0.76886E+00_JPRB ,&
&  0.76437E+00_JPRB, 0.75447E+00_JPRB, 0.75344E+00_JPRB, 0.75868E+00_JPRB, 0.79293E+00_JPRB, 0.85414E+00_JPRB, 0.93402E+00_JPRB ,&
&  0.95784E+00_JPRB, 0.95253E+00_JPRB, 0.90449E+00_JPRB, 0.86369E+00_JPRB, 0.87425E+00_JPRB, 0.86132E+00_JPRB, 0.82445E+00_JPRB ,&
&  0.80209E+00_JPRB, 0.89081E+00_JPRB, 0.11305E+01_JPRB, 0.15131E+01_JPRB, 0.21981E+01_JPRB, 0.26644E+01_JPRB, 0.35401E+01_JPRB ,&
&  0.35849E+01_JPRB, 0.24178E+01_JPRB, 0.25221E+01_JPRB, 0.24231E+01_JPRB, 0.25901E+01_JPRB, 0.26862E+01_JPRB, 0.27243E+01_JPRB ,&
&  0.28125E+01_JPRB, 0.29088E+01_JPRB, 0.30014E+01_JPRB, 0.30999E+01_JPRB, 0.31778E+01_JPRB, 0.31967E+01_JPRB, 0.32170E+01_JPRB ,&
&  0.32477E+01_JPRB, 0.32500E+01_JPRB, 0.33424E+01_JPRB, 0.36088E+01_JPRB, 0.42955E+01_JPRB, 0.50589E+01_JPRB, 0.52107E+01_JPRB ,&
&  0.53225E+01_JPRB, 0.57899E+01_JPRB, 0.57595E+01_JPRB, 0.61893E+01_JPRB, 0.63812E+01_JPRB, 0.62133E+01_JPRB, 0.66425E+01_JPRB ,&
&  0.69134E+01_JPRB, 0.57951E+01_JPRB, 0.38989E+01_JPRB, 0.40313E+01_JPRB, 0.36608E+01_JPRB, 0.32544E+01_JPRB, 0.32875E+01_JPRB ,&
&  0.29959E+01_JPRB, 0.26751E+01_JPRB, 0.23360E+01_JPRB, 0.21174E+01_JPRB, 0.20863E+01_JPRB, 0.19886E+01_JPRB, 0.20673E+01_JPRB ,&
&  0.19039E+01_JPRB, 0.18509E+01_JPRB, 0.17415E+01_JPRB, 0.17103E+01_JPRB, 0.19263E+01_JPRB, 0.22798E+01_JPRB, 0.19297E+01_JPRB ,&
&  0.18581E+01_JPRB, 0.18573E+01_JPRB, 0.21593E+01_JPRB, 0.23079E+01_JPRB, 0.12163E+01_JPRB, 0.88270E+00_JPRB, 0.82055E+00_JPRB ,&
&  0.82986E+00_JPRB, 0.82374E+00_JPRB, 0.81583E+00_JPRB, 0.80123E+00_JPRB, 0.83504E+00_JPRB, 0.89303E+00_JPRB, 0.87213E+00_JPRB ,&
&  0.86491E+00_JPRB, 0.80681E+00_JPRB, 0.78142E+00_JPRB, 0.77302E+00_JPRB, 0.76628E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.68918E+00_JPRB, 0.68116E+00_JPRB, 0.67359E+00_JPRB, 0.66575E+00_JPRB, 0.67770E+00_JPRB, 0.68595E+00_JPRB, 0.70470E+00_JPRB ,&
&  0.72547E+00_JPRB, 0.73476E+00_JPRB, 0.74960E+00_JPRB, 0.75797E+00_JPRB, 0.78534E+00_JPRB, 0.80888E+00_JPRB, 0.85012E+00_JPRB ,&
&  0.89220E+00_JPRB, 0.91684E+00_JPRB, 0.94402E+00_JPRB, 0.97199E+00_JPRB, 0.98754E+00_JPRB, 0.10060E+01_JPRB, 0.99470E+00_JPRB ,&
&  0.97557E+00_JPRB, 0.97563E+00_JPRB, 0.10142E+01_JPRB, 0.11831E+01_JPRB, 0.15065E+01_JPRB, 0.19235E+01_JPRB, 0.27810E+01_JPRB ,&
&  0.29814E+01_JPRB, 0.21640E+01_JPRB, 0.18739E+01_JPRB, 0.16656E+01_JPRB, 0.17566E+01_JPRB, 0.18876E+01_JPRB, 0.19020E+01_JPRB ,&
&  0.19346E+01_JPRB, 0.19548E+01_JPRB, 0.19523E+01_JPRB, 0.19225E+01_JPRB, 0.18280E+01_JPRB, 0.17069E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.16733E+01_JPRB, 0.19418E+01_JPRB, 0.26119E+01_JPRB, 0.32157E+01_JPRB, 0.41702E+01_JPRB, 0.39763E+01_JPRB ,&
&  0.40562E+01_JPRB, 0.43043E+01_JPRB, 0.42699E+01_JPRB, 0.45805E+01_JPRB, 0.47115E+01_JPRB, 0.49044E+01_JPRB, 0.49224E+01_JPRB ,&
&  0.52565E+01_JPRB, 0.45859E+01_JPRB, 0.32302E+01_JPRB, 0.34413E+01_JPRB, 0.32034E+01_JPRB, 0.31648E+01_JPRB, 0.29688E+01_JPRB ,&
&  0.27638E+01_JPRB, 0.25249E+01_JPRB, 0.23185E+01_JPRB, 0.21544E+01_JPRB, 0.21188E+01_JPRB, 0.20411E+01_JPRB, 0.18630E+01_JPRB ,&
&  0.18391E+01_JPRB, 0.18565E+01_JPRB, 0.18211E+01_JPRB, 0.17637E+01_JPRB, 0.20914E+01_JPRB, 0.22467E+01_JPRB, 0.28194E+01_JPRB ,&
&  0.18901E+01_JPRB, 0.17123E+01_JPRB, 0.13003E+01_JPRB, 0.16531E+01_JPRB, 0.96404E+00_JPRB, 0.70460E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.67437E+00_JPRB, 0.65139E+00_JPRB, 0.63259E+00_JPRB, 0.62885E+00_JPRB, 0.64284E+00_JPRB, 0.67924E+00_JPRB, 0.67298E+00_JPRB ,&
&  0.67651E+00_JPRB, 0.68039E+00_JPRB, 0.69394E+00_JPRB, 0.71369E+00_JPRB, 0.70694E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.73464E+00_JPRB, 0.74022E+00_JPRB, 0.75563E+00_JPRB, 0.77961E+00_JPRB, 0.79561E+00_JPRB, 0.81033E+00_JPRB, 0.82428E+00_JPRB ,&
&  0.85370E+00_JPRB, 0.88608E+00_JPRB, 0.91577E+00_JPRB, 0.94500E+00_JPRB, 0.97535E+00_JPRB, 0.10001E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10838E+01_JPRB, 0.11256E+01_JPRB, 0.11537E+01_JPRB, 0.11870E+01_JPRB, 0.12164E+01_JPRB, 0.12073E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.12855E+01_JPRB, 0.13265E+01_JPRB, 0.14065E+01_JPRB, 0.15105E+01_JPRB, 0.15607E+01_JPRB, 0.18498E+01_JPRB ,&
&  0.19326E+01_JPRB, 0.20630E+01_JPRB, 0.17622E+01_JPRB, 0.15118E+01_JPRB, 0.14481E+01_JPRB, 0.12963E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.12595E+01_JPRB, 0.12520E+01_JPRB, 0.12230E+01_JPRB, 0.11706E+01_JPRB, 0.12557E+01_JPRB, 0.14618E+01_JPRB ,&
&  0.17809E+01_JPRB, 0.19714E+01_JPRB, 0.22154E+01_JPRB, 0.24217E+01_JPRB, 0.27070E+01_JPRB, 0.29027E+01_JPRB, 0.28821E+01_JPRB ,&
&  0.29028E+01_JPRB, 0.28646E+01_JPRB, 0.27270E+01_JPRB, 0.31560E+01_JPRB, 0.34914E+01_JPRB, 0.36427E+01_JPRB, 0.36801E+01_JPRB ,&
&  0.36845E+01_JPRB, 0.32654E+01_JPRB, 0.31298E+01_JPRB, 0.29524E+01_JPRB, 0.27610E+01_JPRB, 0.27615E+01_JPRB, 0.26031E+01_JPRB ,&
&  0.23987E+01_JPRB, 0.22605E+01_JPRB, 0.22165E+01_JPRB, 0.22157E+01_JPRB, 0.22591E+01_JPRB, 0.22584E+01_JPRB, 0.20872E+01_JPRB ,&
&  0.19601E+01_JPRB, 0.18816E+01_JPRB, 0.18195E+01_JPRB, 0.17865E+01_JPRB, 0.18815E+01_JPRB, 0.38235E+01_JPRB, 0.48251E+01_JPRB ,&
&  0.18170E+01_JPRB, 0.12611E+01_JPRB, 0.72564E+00_JPRB, 0.88732E+00_JPRB, 0.73058E+00_JPRB, 0.68318E+00_JPRB, 0.75202E+00_JPRB ,&
&  0.76587E+00_JPRB, 0.75103E+00_JPRB, 0.68410E+00_JPRB, 0.62021E+00_JPRB, 0.64275E+00_JPRB, 0.65676E+00_JPRB, 0.65799E+00_JPRB ,&
&  0.67639E+00_JPRB, 0.68549E+00_JPRB, 0.69999E+00_JPRB, 0.72591E+00_JPRB, 0.72450E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.78701E+00_JPRB, 0.80870E+00_JPRB, 0.83581E+00_JPRB, 0.87093E+00_JPRB, 0.90570E+00_JPRB, 0.93477E+00_JPRB, 0.95651E+00_JPRB ,&
&  0.98839E+00_JPRB, 0.10293E+01_JPRB, 0.10564E+01_JPRB, 0.10713E+01_JPRB, 0.11030E+01_JPRB, 0.11302E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.12750E+01_JPRB, 0.13319E+01_JPRB, 0.13849E+01_JPRB, 0.14302E+01_JPRB, 0.14789E+01_JPRB, 0.15248E+01_JPRB ,&
&  0.15620E+01_JPRB, 0.16364E+01_JPRB, 0.16919E+01_JPRB, 0.17247E+01_JPRB, 0.17998E+01_JPRB, 0.19403E+01_JPRB, 0.17013E+01_JPRB ,&
&  0.21323E+01_JPRB, 0.19363E+01_JPRB, 0.17090E+01_JPRB, 0.15452E+01_JPRB, 0.14274E+01_JPRB, 0.13640E+01_JPRB, 0.13430E+01_JPRB ,&
&  0.13984E+01_JPRB, 0.15601E+01_JPRB, 0.16955E+01_JPRB, 0.17284E+01_JPRB, 0.16857E+01_JPRB, 0.18917E+01_JPRB, 0.23234E+01_JPRB ,&
&  0.25556E+01_JPRB, 0.26931E+01_JPRB, 0.27948E+01_JPRB, 0.28330E+01_JPRB, 0.29693E+01_JPRB, 0.31655E+01_JPRB, 0.32785E+01_JPRB ,&
&  0.32758E+01_JPRB, 0.31751E+01_JPRB, 0.28159E+01_JPRB, 0.34312E+01_JPRB, 0.33867E+01_JPRB, 0.32023E+01_JPRB, 0.28725E+01_JPRB ,&
&  0.25559E+01_JPRB, 0.22491E+01_JPRB, 0.23393E+01_JPRB, 0.24684E+01_JPRB, 0.23640E+01_JPRB, 0.22850E+01_JPRB, 0.21183E+01_JPRB ,&
&  0.19480E+01_JPRB, 0.19128E+01_JPRB, 0.19581E+01_JPRB, 0.20595E+01_JPRB, 0.21464E+01_JPRB, 0.21390E+01_JPRB, 0.19878E+01_JPRB ,&
&  0.18353E+01_JPRB, 0.16825E+01_JPRB, 0.15937E+01_JPRB, 0.15522E+01_JPRB, 0.16374E+01_JPRB, 0.19631E+01_JPRB, 0.28826E+01_JPRB ,&
&  0.15143E+01_JPRB, 0.85108E+00_JPRB, 0.84138E+00_JPRB, 0.74704E+00_JPRB, 0.75285E+00_JPRB, 0.70201E+00_JPRB, 0.85372E+00_JPRB ,&
&  0.10628E+01_JPRB, 0.12106E+01_JPRB, 0.11849E+01_JPRB, 0.92133E+00_JPRB, 0.76174E+00_JPRB, 0.71691E+00_JPRB, 0.71364E+00_JPRB ,&
&  0.72852E+00_JPRB, 0.73249E+00_JPRB, 0.74140E+00_JPRB, 0.75392E+00_JPRB, 0.77057E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.76758E+00_JPRB, 0.78382E+00_JPRB, 0.80387E+00_JPRB, 0.82539E+00_JPRB, 0.85151E+00_JPRB, 0.87881E+00_JPRB, 0.89891E+00_JPRB ,&
&  0.92200E+00_JPRB, 0.94823E+00_JPRB, 0.97271E+00_JPRB, 0.99060E+00_JPRB, 0.10138E+01_JPRB, 0.10543E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.11340E+01_JPRB, 0.12101E+01_JPRB, 0.12621E+01_JPRB, 0.13426E+01_JPRB, 0.14225E+01_JPRB, 0.15133E+01_JPRB, 0.15859E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.17816E+01_JPRB, 0.18424E+01_JPRB, 0.18463E+01_JPRB, 0.19039E+01_JPRB, 0.21213E+01_JPRB, 0.18124E+01_JPRB ,&
&  0.21364E+01_JPRB, 0.20458E+01_JPRB, 0.18669E+01_JPRB, 0.18038E+01_JPRB, 0.17623E+01_JPRB, 0.18082E+01_JPRB, 0.18943E+01_JPRB ,&
&  0.20292E+01_JPRB, 0.23050E+01_JPRB, 0.23094E+01_JPRB, 0.23837E+01_JPRB, 0.25146E+01_JPRB, 0.28844E+01_JPRB, 0.32376E+01_JPRB ,&
&  0.33919E+01_JPRB, 0.36608E+01_JPRB, 0.38057E+01_JPRB, 0.39380E+01_JPRB, 0.41420E+01_JPRB, 0.43613E+01_JPRB, 0.45200E+01_JPRB ,&
&  0.44660E+01_JPRB, 0.43726E+01_JPRB, 0.40277E+01_JPRB, 0.43211E+01_JPRB, 0.40526E+01_JPRB, 0.32210E+01_JPRB, 0.24624E+01_JPRB ,&
&  0.17851E+01_JPRB, 0.18339E+01_JPRB, 0.19379E+01_JPRB, 0.20260E+01_JPRB, 0.18883E+01_JPRB, 0.17700E+01_JPRB, 0.15113E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.13962E+01_JPRB, 0.14926E+01_JPRB, 0.16354E+01_JPRB, 0.17151E+01_JPRB, 0.16963E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.14627E+01_JPRB, 0.13353E+01_JPRB, 0.12957E+01_JPRB, 0.13092E+01_JPRB, 0.13040E+01_JPRB, 0.12807E+01_JPRB, 0.17318E+01_JPRB ,&
&  0.12022E+01_JPRB, 0.10404E+01_JPRB, 0.11973E+01_JPRB, 0.55388E+00_JPRB, 0.71640E+00_JPRB, 0.89795E+00_JPRB, 0.11687E+01_JPRB ,&
&  0.13653E+01_JPRB, 0.18856E+01_JPRB, 0.23100E+01_JPRB, 0.17601E+01_JPRB, 0.12484E+01_JPRB, 0.77194E+00_JPRB, 0.68298E+00_JPRB ,&
&  0.69971E+00_JPRB, 0.70886E+00_JPRB, 0.72224E+00_JPRB, 0.72761E+00_JPRB, 0.74706E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.69763E+00_JPRB, 0.69485E+00_JPRB, 0.69683E+00_JPRB, 0.68808E+00_JPRB, 0.68516E+00_JPRB, 0.68732E+00_JPRB, 0.67809E+00_JPRB ,&
&  0.68513E+00_JPRB, 0.70404E+00_JPRB, 0.71131E+00_JPRB, 0.73763E+00_JPRB, 0.75846E+00_JPRB, 0.81862E+00_JPRB, 0.90901E+00_JPRB ,&
&  0.98593E+00_JPRB, 0.11345E+01_JPRB, 0.12312E+01_JPRB, 0.13415E+01_JPRB, 0.14406E+01_JPRB, 0.15486E+01_JPRB, 0.16426E+01_JPRB ,&
&  0.17644E+01_JPRB, 0.19197E+01_JPRB, 0.20753E+01_JPRB, 0.21929E+01_JPRB, 0.22451E+01_JPRB, 0.25639E+01_JPRB, 0.20436E+01_JPRB ,&
&  0.22814E+01_JPRB, 0.23046E+01_JPRB, 0.22007E+01_JPRB, 0.21673E+01_JPRB, 0.21445E+01_JPRB, 0.21562E+01_JPRB, 0.21823E+01_JPRB ,&
&  0.22831E+01_JPRB, 0.23724E+01_JPRB, 0.24474E+01_JPRB, 0.27634E+01_JPRB, 0.31137E+01_JPRB, 0.34269E+01_JPRB, 0.37343E+01_JPRB ,&
&  0.40221E+01_JPRB, 0.43182E+01_JPRB, 0.46752E+01_JPRB, 0.50425E+01_JPRB, 0.54130E+01_JPRB, 0.56409E+01_JPRB, 0.59167E+01_JPRB ,&
&  0.60597E+01_JPRB, 0.61547E+01_JPRB, 0.57329E+01_JPRB, 0.49647E+01_JPRB, 0.41527E+01_JPRB, 0.32075E+01_JPRB, 0.22889E+01_JPRB ,&
&  0.16811E+01_JPRB, 0.15703E+01_JPRB, 0.15524E+01_JPRB, 0.15153E+01_JPRB, 0.13346E+01_JPRB, 0.12186E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.98864E+00_JPRB, 0.10475E+01_JPRB, 0.11418E+01_JPRB, 0.11755E+01_JPRB, 0.11656E+01_JPRB, 0.11395E+01_JPRB ,&
&  0.10932E+01_JPRB, 0.10324E+01_JPRB, 0.10522E+01_JPRB, 0.10858E+01_JPRB, 0.11618E+01_JPRB, 0.13062E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.84139E+00_JPRB, 0.79691E+00_JPRB, 0.78547E+00_JPRB, 0.75741E+00_JPRB, 0.96073E+00_JPRB, 0.13081E+01_JPRB ,&
&  0.19308E+01_JPRB, 0.21443E+01_JPRB, 0.23557E+01_JPRB, 0.28949E+01_JPRB, 0.25181E+01_JPRB, 0.18871E+01_JPRB, 0.62135E+00_JPRB ,&
&  0.62529E+00_JPRB, 0.63346E+00_JPRB, 0.66547E+00_JPRB, 0.67740E+00_JPRB, 0.68455E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.58632E+00_JPRB, 0.56809E+00_JPRB, 0.55986E+00_JPRB, 0.55661E+00_JPRB, 0.54090E+00_JPRB, 0.54350E+00_JPRB, 0.54325E+00_JPRB ,&
&  0.56453E+00_JPRB, 0.59452E+00_JPRB, 0.64561E+00_JPRB, 0.71106E+00_JPRB, 0.80187E+00_JPRB, 0.88246E+00_JPRB, 0.98537E+00_JPRB ,&
&  0.10633E+01_JPRB, 0.11593E+01_JPRB, 0.12541E+01_JPRB, 0.13536E+01_JPRB, 0.14640E+01_JPRB, 0.15656E+01_JPRB, 0.16963E+01_JPRB ,&
&  0.18197E+01_JPRB, 0.19628E+01_JPRB, 0.21377E+01_JPRB, 0.22947E+01_JPRB, 0.24193E+01_JPRB, 0.25210E+01_JPRB, 0.19775E+01_JPRB ,&
&  0.24095E+01_JPRB, 0.25175E+01_JPRB, 0.25486E+01_JPRB, 0.24835E+01_JPRB, 0.23809E+01_JPRB, 0.22806E+01_JPRB, 0.22911E+01_JPRB ,&
&  0.23318E+01_JPRB, 0.23543E+01_JPRB, 0.23648E+01_JPRB, 0.24743E+01_JPRB, 0.31537E+01_JPRB, 0.34551E+01_JPRB, 0.37769E+01_JPRB ,&
&  0.41168E+01_JPRB, 0.44972E+01_JPRB, 0.48941E+01_JPRB, 0.52952E+01_JPRB, 0.55647E+01_JPRB, 0.58535E+01_JPRB, 0.61535E+01_JPRB ,&
&  0.63937E+01_JPRB, 0.65977E+01_JPRB, 0.62987E+01_JPRB, 0.48709E+01_JPRB, 0.39492E+01_JPRB, 0.31431E+01_JPRB, 0.19586E+01_JPRB ,&
&  0.14558E+01_JPRB, 0.13253E+01_JPRB, 0.13055E+01_JPRB, 0.12357E+01_JPRB, 0.10778E+01_JPRB, 0.97084E+00_JPRB, 0.89455E+00_JPRB ,&
&  0.78690E+00_JPRB, 0.75881E+00_JPRB, 0.76115E+00_JPRB, 0.83345E+00_JPRB, 0.87792E+00_JPRB, 0.83974E+00_JPRB, 0.81836E+00_JPRB ,&
&  0.77503E+00_JPRB, 0.74414E+00_JPRB, 0.81212E+00_JPRB, 0.91905E+00_JPRB, 0.11454E+01_JPRB, 0.13943E+01_JPRB, 0.15713E+01_JPRB ,&
&  0.15454E+01_JPRB, 0.85659E+00_JPRB, 0.76699E+00_JPRB, 0.73612E+00_JPRB, 0.74326E+00_JPRB, 0.79529E+00_JPRB, 0.88927E+00_JPRB ,&
&  0.11077E+01_JPRB, 0.14952E+01_JPRB, 0.13022E+01_JPRB, 0.10009E+01_JPRB, 0.10395E+01_JPRB, 0.96318E+00_JPRB, 0.56975E+00_JPRB ,&
&  0.57459E+00_JPRB, 0.57137E+00_JPRB, 0.58947E+00_JPRB, 0.59588E+00_JPRB, 0.59989E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.51795E+00_JPRB, 0.49559E+00_JPRB, 0.47385E+00_JPRB, 0.46517E+00_JPRB, 0.45927E+00_JPRB, 0.45310E+00_JPRB, 0.46968E+00_JPRB ,&
&  0.50908E+00_JPRB, 0.54689E+00_JPRB, 0.60943E+00_JPRB, 0.70533E+00_JPRB, 0.80936E+00_JPRB, 0.88478E+00_JPRB, 0.96439E+00_JPRB ,&
&  0.10379E+01_JPRB, 0.11134E+01_JPRB, 0.11782E+01_JPRB, 0.12445E+01_JPRB, 0.13297E+01_JPRB, 0.14364E+01_JPRB, 0.15819E+01_JPRB ,&
&  0.17535E+01_JPRB, 0.19253E+01_JPRB, 0.21336E+01_JPRB, 0.23796E+01_JPRB, 0.26192E+01_JPRB, 0.27396E+01_JPRB, 0.29501E+01_JPRB ,&
&  0.20649E+01_JPRB, 0.26285E+01_JPRB, 0.26557E+01_JPRB, 0.25983E+01_JPRB, 0.25094E+01_JPRB, 0.24207E+01_JPRB, 0.23853E+01_JPRB ,&
&  0.23705E+01_JPRB, 0.22272E+01_JPRB, 0.22380E+01_JPRB, 0.23522E+01_JPRB, 0.27710E+01_JPRB, 0.30964E+01_JPRB, 0.34174E+01_JPRB ,&
&  0.37589E+01_JPRB, 0.41417E+01_JPRB, 0.45418E+01_JPRB, 0.49241E+01_JPRB, 0.52251E+01_JPRB, 0.55487E+01_JPRB, 0.57973E+01_JPRB ,&
&  0.61021E+01_JPRB, 0.63607E+01_JPRB, 0.64363E+01_JPRB, 0.52033E+01_JPRB, 0.40532E+01_JPRB, 0.34177E+01_JPRB, 0.23652E+01_JPRB ,&
&  0.14931E+01_JPRB, 0.11501E+01_JPRB, 0.11574E+01_JPRB, 0.11765E+01_JPRB, 0.10618E+01_JPRB, 0.97140E+00_JPRB, 0.86664E+00_JPRB ,&
&  0.78298E+00_JPRB, 0.69289E+00_JPRB, 0.66184E+00_JPRB, 0.68954E+00_JPRB, 0.75227E+00_JPRB, 0.70953E+00_JPRB, 0.64974E+00_JPRB ,&
&  0.61075E+00_JPRB, 0.60135E+00_JPRB, 0.67347E+00_JPRB, 0.75868E+00_JPRB, 0.88088E+00_JPRB, 0.94918E+00_JPRB, 0.99044E+00_JPRB ,&
&  0.10224E+01_JPRB, 0.82334E+00_JPRB, 0.78736E+00_JPRB, 0.78869E+00_JPRB, 0.77754E+00_JPRB, 0.77924E+00_JPRB, 0.79560E+00_JPRB ,&
&  0.83106E+00_JPRB, 0.89836E+00_JPRB, 0.91144E+00_JPRB, 0.84844E+00_JPRB, 0.75983E+00_JPRB, 0.70295E+00_JPRB, 0.65572E+00_JPRB ,&
&  0.65896E+00_JPRB, 0.64487E+00_JPRB, 0.62822E+00_JPRB, 0.59665E+00_JPRB, 0.55902E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.53376E+00_JPRB, 0.49665E+00_JPRB, 0.44895E+00_JPRB, 0.41087E+00_JPRB, 0.43999E+00_JPRB, 0.44625E+00_JPRB, 0.46982E+00_JPRB ,&
&  0.48304E+00_JPRB, 0.51957E+00_JPRB, 0.56053E+00_JPRB, 0.62057E+00_JPRB, 0.70529E+00_JPRB, 0.79355E+00_JPRB, 0.87191E+00_JPRB ,&
&  0.94392E+00_JPRB, 0.10131E+01_JPRB, 0.10662E+01_JPRB, 0.10931E+01_JPRB, 0.11343E+01_JPRB, 0.12009E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.14050E+01_JPRB, 0.15555E+01_JPRB, 0.17571E+01_JPRB, 0.20125E+01_JPRB, 0.23071E+01_JPRB, 0.24918E+01_JPRB, 0.26428E+01_JPRB ,&
&  0.14337E+01_JPRB, 0.22807E+01_JPRB, 0.27761E+01_JPRB, 0.26369E+01_JPRB, 0.25628E+01_JPRB, 0.24850E+01_JPRB, 0.24285E+01_JPRB ,&
&  0.22435E+01_JPRB, 0.20393E+01_JPRB, 0.20218E+01_JPRB, 0.23400E+01_JPRB, 0.23259E+01_JPRB, 0.26142E+01_JPRB, 0.28505E+01_JPRB ,&
&  0.31074E+01_JPRB, 0.33830E+01_JPRB, 0.37039E+01_JPRB, 0.41033E+01_JPRB, 0.45110E+01_JPRB, 0.48471E+01_JPRB, 0.51356E+01_JPRB ,&
&  0.55274E+01_JPRB, 0.58977E+01_JPRB, 0.60331E+01_JPRB, 0.46077E+01_JPRB, 0.41254E+01_JPRB, 0.38227E+01_JPRB, 0.33377E+01_JPRB ,&
&  0.18447E+01_JPRB, 0.11733E+01_JPRB, 0.11743E+01_JPRB, 0.12162E+01_JPRB, 0.11300E+01_JPRB, 0.10476E+01_JPRB, 0.96163E+00_JPRB ,&
&  0.84822E+00_JPRB, 0.76573E+00_JPRB, 0.70433E+00_JPRB, 0.68065E+00_JPRB, 0.71138E+00_JPRB, 0.71391E+00_JPRB, 0.67355E+00_JPRB ,&
&  0.63008E+00_JPRB, 0.63361E+00_JPRB, 0.68272E+00_JPRB, 0.72684E+00_JPRB, 0.79634E+00_JPRB, 0.83860E+00_JPRB, 0.89318E+00_JPRB ,&
&  0.89574E+00_JPRB, 0.81064E+00_JPRB, 0.80186E+00_JPRB, 0.81529E+00_JPRB, 0.82248E+00_JPRB, 0.82919E+00_JPRB, 0.83162E+00_JPRB ,&
&  0.83006E+00_JPRB, 0.85040E+00_JPRB, 0.85545E+00_JPRB, 0.85901E+00_JPRB, 0.82941E+00_JPRB, 0.79750E+00_JPRB, 0.77536E+00_JPRB ,&
&  0.74705E+00_JPRB, 0.71357E+00_JPRB, 0.68446E+00_JPRB, 0.63308E+00_JPRB, 0.57987E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.56337E+00_JPRB, 0.51929E+00_JPRB, 0.47306E+00_JPRB, 0.45718E+00_JPRB, 0.48948E+00_JPRB, 0.51955E+00_JPRB, 0.51917E+00_JPRB ,&
&  0.51564E+00_JPRB, 0.50661E+00_JPRB, 0.50595E+00_JPRB, 0.52449E+00_JPRB, 0.58192E+00_JPRB, 0.67002E+00_JPRB, 0.74747E+00_JPRB ,&
&  0.81577E+00_JPRB, 0.86786E+00_JPRB, 0.90788E+00_JPRB, 0.92092E+00_JPRB, 0.91224E+00_JPRB, 0.93622E+00_JPRB, 0.98132E+00_JPRB ,&
&  0.10598E+01_JPRB, 0.11640E+01_JPRB, 0.13017E+01_JPRB, 0.15036E+01_JPRB, 0.18028E+01_JPRB, 0.21717E+01_JPRB, 0.25515E+01_JPRB ,&
&  0.18195E+01_JPRB, 0.79529E+00_JPRB, 0.23973E+01_JPRB, 0.29456E+01_JPRB, 0.27563E+01_JPRB, 0.26865E+01_JPRB, 0.24789E+01_JPRB ,&
&  0.22344E+01_JPRB, 0.21891E+01_JPRB, 0.20414E+01_JPRB, 0.21866E+01_JPRB, 0.21065E+01_JPRB, 0.21838E+01_JPRB, 0.22605E+01_JPRB ,&
&  0.23796E+01_JPRB, 0.25247E+01_JPRB, 0.27547E+01_JPRB, 0.30372E+01_JPRB, 0.33523E+01_JPRB, 0.38319E+01_JPRB, 0.41827E+01_JPRB ,&
&  0.44884E+01_JPRB, 0.48088E+01_JPRB, 0.49041E+01_JPRB, 0.35046E+01_JPRB, 0.27560E+01_JPRB, 0.23679E+01_JPRB, 0.20951E+01_JPRB ,&
&  0.17236E+01_JPRB, 0.12966E+01_JPRB, 0.12457E+01_JPRB, 0.12850E+01_JPRB, 0.12314E+01_JPRB, 0.11086E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.94964E+00_JPRB, 0.86114E+00_JPRB, 0.78927E+00_JPRB, 0.73218E+00_JPRB, 0.73349E+00_JPRB, 0.74722E+00_JPRB, 0.75115E+00_JPRB ,&
&  0.72437E+00_JPRB, 0.71912E+00_JPRB, 0.74078E+00_JPRB, 0.74271E+00_JPRB, 0.74797E+00_JPRB, 0.75575E+00_JPRB, 0.76879E+00_JPRB ,&
&  0.81018E+00_JPRB, 0.80010E+00_JPRB, 0.80630E+00_JPRB, 0.82797E+00_JPRB, 0.84131E+00_JPRB, 0.83102E+00_JPRB, 0.84319E+00_JPRB ,&
&  0.86271E+00_JPRB, 0.88088E+00_JPRB, 0.87162E+00_JPRB, 0.88253E+00_JPRB, 0.86890E+00_JPRB, 0.85760E+00_JPRB, 0.85502E+00_JPRB ,&
&  0.84987E+00_JPRB, 0.82954E+00_JPRB, 0.78169E+00_JPRB, 0.68478E+00_JPRB, 0.62124E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.61304E+00_JPRB, 0.54169E+00_JPRB, 0.50052E+00_JPRB, 0.50647E+00_JPRB, 0.53740E+00_JPRB, 0.55054E+00_JPRB, 0.55515E+00_JPRB ,&
&  0.54650E+00_JPRB, 0.53322E+00_JPRB, 0.51187E+00_JPRB, 0.50262E+00_JPRB, 0.51251E+00_JPRB, 0.53719E+00_JPRB, 0.59731E+00_JPRB ,&
&  0.65721E+00_JPRB, 0.71518E+00_JPRB, 0.76021E+00_JPRB, 0.75976E+00_JPRB, 0.72349E+00_JPRB, 0.71635E+00_JPRB, 0.73620E+00_JPRB ,&
&  0.76972E+00_JPRB, 0.83143E+00_JPRB, 0.91660E+00_JPRB, 0.10346E+01_JPRB, 0.12228E+01_JPRB, 0.15476E+01_JPRB, 0.20565E+01_JPRB ,&
&  0.29598E+01_JPRB, 0.20039E+01_JPRB, 0.12504E+01_JPRB, 0.25895E+01_JPRB, 0.31500E+01_JPRB, 0.32483E+01_JPRB, 0.26706E+01_JPRB ,&
&  0.25626E+01_JPRB, 0.23947E+01_JPRB, 0.22055E+01_JPRB, 0.22224E+01_JPRB, 0.20427E+01_JPRB, 0.19899E+01_JPRB, 0.19430E+01_JPRB ,&
&  0.18977E+01_JPRB, 0.19962E+01_JPRB, 0.21419E+01_JPRB, 0.22841E+01_JPRB, 0.24332E+01_JPRB, 0.27560E+01_JPRB, 0.31748E+01_JPRB ,&
&  0.35842E+01_JPRB, 0.38501E+01_JPRB, 0.39482E+01_JPRB, 0.29121E+01_JPRB, 0.23611E+01_JPRB, 0.20965E+01_JPRB, 0.19607E+01_JPRB ,&
&  0.16962E+01_JPRB, 0.15237E+01_JPRB, 0.14715E+01_JPRB, 0.13953E+01_JPRB, 0.13118E+01_JPRB, 0.12207E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.10917E+01_JPRB, 0.98300E+00_JPRB, 0.91113E+00_JPRB, 0.86407E+00_JPRB, 0.84318E+00_JPRB, 0.83924E+00_JPRB, 0.82993E+00_JPRB ,&
&  0.83166E+00_JPRB, 0.83263E+00_JPRB, 0.83466E+00_JPRB, 0.80858E+00_JPRB, 0.78012E+00_JPRB, 0.76120E+00_JPRB, 0.73975E+00_JPRB ,&
&  0.76045E+00_JPRB, 0.78777E+00_JPRB, 0.80496E+00_JPRB, 0.83297E+00_JPRB, 0.83515E+00_JPRB, 0.83074E+00_JPRB, 0.85827E+00_JPRB ,&
&  0.87074E+00_JPRB, 0.89147E+00_JPRB, 0.88402E+00_JPRB, 0.88973E+00_JPRB, 0.87387E+00_JPRB, 0.86146E+00_JPRB, 0.86433E+00_JPRB ,&
&  0.89829E+00_JPRB, 0.93298E+00_JPRB, 0.90014E+00_JPRB, 0.72682E+00_JPRB, 0.65616E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.59900E+00_JPRB, 0.56658E+00_JPRB, 0.52563E+00_JPRB, 0.53542E+00_JPRB, 0.54464E+00_JPRB, 0.55782E+00_JPRB, 0.56474E+00_JPRB ,&
&  0.56963E+00_JPRB, 0.57177E+00_JPRB, 0.57458E+00_JPRB, 0.56181E+00_JPRB, 0.52952E+00_JPRB, 0.51616E+00_JPRB, 0.51789E+00_JPRB ,&
&  0.53489E+00_JPRB, 0.57586E+00_JPRB, 0.62324E+00_JPRB, 0.63253E+00_JPRB, 0.62449E+00_JPRB, 0.61391E+00_JPRB, 0.61009E+00_JPRB ,&
&  0.61890E+00_JPRB, 0.63830E+00_JPRB, 0.66667E+00_JPRB, 0.72603E+00_JPRB, 0.83284E+00_JPRB, 0.10043E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.20747E+01_JPRB, 0.22312E+01_JPRB, 0.97483E+00_JPRB, 0.20375E+01_JPRB, 0.31813E+01_JPRB, 0.33980E+01_JPRB, 0.32241E+01_JPRB ,&
&  0.30306E+01_JPRB, 0.26864E+01_JPRB, 0.24301E+01_JPRB, 0.21932E+01_JPRB, 0.19272E+01_JPRB, 0.17853E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.15768E+01_JPRB, 0.16269E+01_JPRB, 0.17743E+01_JPRB, 0.18844E+01_JPRB, 0.19237E+01_JPRB, 0.20222E+01_JPRB, 0.23264E+01_JPRB ,&
&  0.27839E+01_JPRB, 0.32490E+01_JPRB, 0.34694E+01_JPRB, 0.27226E+01_JPRB, 0.23599E+01_JPRB, 0.23518E+01_JPRB, 0.22476E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.17855E+01_JPRB, 0.16135E+01_JPRB, 0.15098E+01_JPRB, 0.13983E+01_JPRB, 0.13470E+01_JPRB, 0.13533E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.11233E+01_JPRB, 0.10388E+01_JPRB, 0.98293E+00_JPRB, 0.94469E+00_JPRB, 0.92315E+00_JPRB, 0.90701E+00_JPRB ,&
&  0.89606E+00_JPRB, 0.90482E+00_JPRB, 0.92022E+00_JPRB, 0.90451E+00_JPRB, 0.86772E+00_JPRB, 0.82588E+00_JPRB, 0.78478E+00_JPRB ,&
&  0.77272E+00_JPRB, 0.77796E+00_JPRB, 0.80583E+00_JPRB, 0.82392E+00_JPRB, 0.83600E+00_JPRB, 0.84892E+00_JPRB, 0.86776E+00_JPRB ,&
&  0.88980E+00_JPRB, 0.89939E+00_JPRB, 0.88022E+00_JPRB, 0.85754E+00_JPRB, 0.86945E+00_JPRB, 0.85019E+00_JPRB, 0.83328E+00_JPRB ,&
&  0.86648E+00_JPRB, 0.93054E+00_JPRB, 0.10165E+01_JPRB, 0.73217E+00_JPRB, 0.65471E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.59850E+00_JPRB, 0.55850E+00_JPRB, 0.55334E+00_JPRB, 0.55632E+00_JPRB, 0.56571E+00_JPRB, 0.58092E+00_JPRB, 0.59775E+00_JPRB ,&
&  0.60412E+00_JPRB, 0.60560E+00_JPRB, 0.60531E+00_JPRB, 0.59938E+00_JPRB, 0.58175E+00_JPRB, 0.56260E+00_JPRB, 0.54317E+00_JPRB ,&
&  0.53505E+00_JPRB, 0.53834E+00_JPRB, 0.55701E+00_JPRB, 0.56663E+00_JPRB, 0.56717E+00_JPRB, 0.56359E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.57125E+00_JPRB, 0.57404E+00_JPRB, 0.57558E+00_JPRB, 0.59780E+00_JPRB, 0.64737E+00_JPRB, 0.74572E+00_JPRB, 0.93160E+00_JPRB ,&
&  0.14650E+01_JPRB, 0.19084E+01_JPRB, 0.18440E+01_JPRB, 0.30369E+01_JPRB, 0.37911E+01_JPRB, 0.37404E+01_JPRB, 0.36595E+01_JPRB ,&
&  0.34239E+01_JPRB, 0.30342E+01_JPRB, 0.28288E+01_JPRB, 0.22940E+01_JPRB, 0.19222E+01_JPRB, 0.16504E+01_JPRB, 0.14497E+01_JPRB ,&
&  0.12970E+01_JPRB, 0.12560E+01_JPRB, 0.13464E+01_JPRB, 0.15123E+01_JPRB, 0.16239E+01_JPRB, 0.16666E+01_JPRB, 0.17856E+01_JPRB ,&
&  0.21015E+01_JPRB, 0.26810E+01_JPRB, 0.31611E+01_JPRB, 0.29306E+01_JPRB, 0.23651E+01_JPRB, 0.26130E+01_JPRB, 0.28878E+01_JPRB ,&
&  0.32678E+01_JPRB, 0.23247E+01_JPRB, 0.18177E+01_JPRB, 0.16263E+01_JPRB, 0.14799E+01_JPRB, 0.15824E+01_JPRB, 0.14969E+01_JPRB ,&
&  0.13828E+01_JPRB, 0.12651E+01_JPRB, 0.11617E+01_JPRB, 0.10931E+01_JPRB, 0.10409E+01_JPRB, 0.10014E+01_JPRB, 0.97606E+00_JPRB ,&
&  0.96107E+00_JPRB, 0.95386E+00_JPRB, 0.96377E+00_JPRB, 0.96486E+00_JPRB, 0.94774E+00_JPRB, 0.91080E+00_JPRB, 0.86054E+00_JPRB ,&
&  0.81807E+00_JPRB, 0.80109E+00_JPRB, 0.78332E+00_JPRB, 0.80009E+00_JPRB, 0.82640E+00_JPRB, 0.84263E+00_JPRB, 0.84862E+00_JPRB ,&
&  0.87384E+00_JPRB, 0.89202E+00_JPRB, 0.87993E+00_JPRB, 0.85588E+00_JPRB, 0.84692E+00_JPRB, 0.84220E+00_JPRB, 0.80993E+00_JPRB ,&
&  0.79129E+00_JPRB, 0.77717E+00_JPRB, 0.73531E+00_JPRB, 0.69548E+00_JPRB, 0.64187E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.61014E+00_JPRB, 0.58520E+00_JPRB, 0.57976E+00_JPRB, 0.58687E+00_JPRB, 0.59666E+00_JPRB, 0.60803E+00_JPRB, 0.62465E+00_JPRB ,&
&  0.63309E+00_JPRB, 0.63407E+00_JPRB, 0.63201E+00_JPRB, 0.62787E+00_JPRB, 0.61505E+00_JPRB, 0.59968E+00_JPRB, 0.58254E+00_JPRB ,&
&  0.56653E+00_JPRB, 0.53919E+00_JPRB, 0.52504E+00_JPRB, 0.53112E+00_JPRB, 0.53387E+00_JPRB, 0.53499E+00_JPRB, 0.53773E+00_JPRB ,&
&  0.53887E+00_JPRB, 0.54511E+00_JPRB, 0.54549E+00_JPRB, 0.55527E+00_JPRB, 0.57871E+00_JPRB, 0.64116E+00_JPRB, 0.75571E+00_JPRB ,&
&  0.11103E+01_JPRB, 0.17301E+01_JPRB, 0.15468E+01_JPRB, 0.34742E+01_JPRB, 0.39431E+01_JPRB, 0.38861E+01_JPRB, 0.34982E+01_JPRB ,&
&  0.32274E+01_JPRB, 0.33070E+01_JPRB, 0.25684E+01_JPRB, 0.20979E+01_JPRB, 0.18423E+01_JPRB, 0.16022E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.10491E+01_JPRB, 0.10254E+01_JPRB, 0.10996E+01_JPRB, 0.12448E+01_JPRB, 0.13762E+01_JPRB, 0.14771E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.20845E+01_JPRB, 0.27048E+01_JPRB, 0.26862E+01_JPRB, 0.25242E+01_JPRB, 0.29069E+01_JPRB, 0.36570E+01_JPRB ,&
&  0.44349E+01_JPRB, 0.34481E+01_JPRB, 0.22499E+01_JPRB, 0.18090E+01_JPRB, 0.16878E+01_JPRB, 0.17337E+01_JPRB, 0.16904E+01_JPRB ,&
&  0.15561E+01_JPRB, 0.14395E+01_JPRB, 0.13043E+01_JPRB, 0.12134E+01_JPRB, 0.11490E+01_JPRB, 0.10794E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.98804E+00_JPRB, 0.97898E+00_JPRB, 0.96671E+00_JPRB, 0.96188E+00_JPRB, 0.95392E+00_JPRB, 0.93148E+00_JPRB ,&
&  0.88991E+00_JPRB, 0.84198E+00_JPRB, 0.78983E+00_JPRB, 0.77823E+00_JPRB, 0.80077E+00_JPRB, 0.82268E+00_JPRB, 0.83610E+00_JPRB ,&
&  0.87801E+00_JPRB, 0.88897E+00_JPRB, 0.86519E+00_JPRB, 0.82840E+00_JPRB, 0.82167E+00_JPRB, 0.84490E+00_JPRB, 0.81103E+00_JPRB ,&
&  0.79786E+00_JPRB, 0.77879E+00_JPRB, 0.74959E+00_JPRB, 0.71467E+00_JPRB, 0.66260E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.65280E+00_JPRB, 0.63297E+00_JPRB, 0.62213E+00_JPRB, 0.62158E+00_JPRB, 0.61990E+00_JPRB, 0.62478E+00_JPRB, 0.63797E+00_JPRB ,&
&  0.65555E+00_JPRB, 0.66005E+00_JPRB, 0.65672E+00_JPRB, 0.65221E+00_JPRB, 0.64321E+00_JPRB, 0.62872E+00_JPRB, 0.61216E+00_JPRB ,&
&  0.59746E+00_JPRB, 0.57641E+00_JPRB, 0.54778E+00_JPRB, 0.52564E+00_JPRB, 0.52652E+00_JPRB, 0.53150E+00_JPRB, 0.53408E+00_JPRB ,&
&  0.53462E+00_JPRB, 0.54211E+00_JPRB, 0.54536E+00_JPRB, 0.55620E+00_JPRB, 0.56584E+00_JPRB, 0.60097E+00_JPRB, 0.70595E+00_JPRB ,&
&  0.98219E+00_JPRB, 0.20473E+01_JPRB, 0.16611E+01_JPRB, 0.32169E+01_JPRB, 0.36713E+01_JPRB, 0.35695E+01_JPRB, 0.31623E+01_JPRB ,&
&  0.29217E+01_JPRB, 0.27337E+01_JPRB, 0.22581E+01_JPRB, 0.18662E+01_JPRB, 0.16221E+01_JPRB, 0.14329E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.10537E+01_JPRB, 0.10047E+01_JPRB, 0.97869E+00_JPRB, 0.10205E+01_JPRB, 0.11144E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.13551E+01_JPRB, 0.15860E+01_JPRB, 0.21123E+01_JPRB, 0.26478E+01_JPRB, 0.25039E+01_JPRB, 0.29275E+01_JPRB, 0.43996E+01_JPRB ,&
&  0.51639E+01_JPRB, 0.49976E+01_JPRB, 0.31861E+01_JPRB, 0.22082E+01_JPRB, 0.19016E+01_JPRB, 0.19217E+01_JPRB, 0.19006E+01_JPRB ,&
&  0.17728E+01_JPRB, 0.16713E+01_JPRB, 0.15168E+01_JPRB, 0.13524E+01_JPRB, 0.12323E+01_JPRB, 0.11483E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.99114E+00_JPRB, 0.96952E+00_JPRB, 0.94649E+00_JPRB, 0.93448E+00_JPRB, 0.93608E+00_JPRB, 0.93947E+00_JPRB ,&
&  0.93248E+00_JPRB, 0.90191E+00_JPRB, 0.83591E+00_JPRB, 0.79278E+00_JPRB, 0.79244E+00_JPRB, 0.80926E+00_JPRB, 0.82137E+00_JPRB ,&
&  0.87210E+00_JPRB, 0.88101E+00_JPRB, 0.84618E+00_JPRB, 0.82333E+00_JPRB, 0.80730E+00_JPRB, 0.84852E+00_JPRB, 0.81341E+00_JPRB ,&
&  0.79659E+00_JPRB, 0.77923E+00_JPRB, 0.76921E+00_JPRB, 0.74562E+00_JPRB, 0.69870E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.68911E+00_JPRB, 0.66552E+00_JPRB, 0.65440E+00_JPRB, 0.64163E+00_JPRB, 0.63297E+00_JPRB, 0.63236E+00_JPRB, 0.64310E+00_JPRB ,&
&  0.66332E+00_JPRB, 0.67703E+00_JPRB, 0.66904E+00_JPRB, 0.66372E+00_JPRB, 0.65969E+00_JPRB, 0.64799E+00_JPRB, 0.63283E+00_JPRB ,&
&  0.61748E+00_JPRB, 0.60293E+00_JPRB, 0.58354E+00_JPRB, 0.56017E+00_JPRB, 0.54391E+00_JPRB, 0.54242E+00_JPRB, 0.53977E+00_JPRB ,&
&  0.53828E+00_JPRB, 0.54333E+00_JPRB, 0.54752E+00_JPRB, 0.55509E+00_JPRB, 0.56379E+00_JPRB, 0.58506E+00_JPRB, 0.68046E+00_JPRB ,&
&  0.10036E+01_JPRB, 0.78200E+00_JPRB, 0.20630E+01_JPRB, 0.28282E+01_JPRB, 0.29639E+01_JPRB, 0.27700E+01_JPRB, 0.26642E+01_JPRB ,&
&  0.25211E+01_JPRB, 0.22555E+01_JPRB, 0.20064E+01_JPRB, 0.17885E+01_JPRB, 0.15179E+01_JPRB, 0.13385E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.10751E+01_JPRB, 0.10179E+01_JPRB, 0.96183E+00_JPRB, 0.92709E+00_JPRB, 0.94150E+00_JPRB, 0.10061E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.12202E+01_JPRB, 0.15865E+01_JPRB, 0.21133E+01_JPRB, 0.24568E+01_JPRB, 0.28398E+01_JPRB, 0.35952E+01_JPRB ,&
&  0.47505E+01_JPRB, 0.52387E+01_JPRB, 0.39402E+01_JPRB, 0.28673E+01_JPRB, 0.22793E+01_JPRB, 0.21401E+01_JPRB, 0.21355E+01_JPRB ,&
&  0.20157E+01_JPRB, 0.18654E+01_JPRB, 0.16888E+01_JPRB, 0.14924E+01_JPRB, 0.13258E+01_JPRB, 0.12127E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.10037E+01_JPRB, 0.96770E+00_JPRB, 0.94160E+00_JPRB, 0.92413E+00_JPRB, 0.91814E+00_JPRB, 0.92303E+00_JPRB ,&
&  0.93530E+00_JPRB, 0.93284E+00_JPRB, 0.89356E+00_JPRB, 0.85359E+00_JPRB, 0.84517E+00_JPRB, 0.85522E+00_JPRB, 0.87066E+00_JPRB ,&
&  0.89790E+00_JPRB, 0.89441E+00_JPRB, 0.85316E+00_JPRB, 0.82952E+00_JPRB, 0.79058E+00_JPRB, 0.81735E+00_JPRB, 0.80121E+00_JPRB ,&
&  0.78630E+00_JPRB, 0.77737E+00_JPRB, 0.77842E+00_JPRB, 0.76367E+00_JPRB, 0.72793E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.71590E+00_JPRB, 0.69193E+00_JPRB, 0.67526E+00_JPRB, 0.65310E+00_JPRB, 0.64246E+00_JPRB, 0.64210E+00_JPRB, 0.65278E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.66808E+00_JPRB, 0.66085E+00_JPRB, 0.65330E+00_JPRB, 0.64409E+00_JPRB, 0.63667E+00_JPRB, 0.62677E+00_JPRB ,&
&  0.62072E+00_JPRB, 0.61531E+00_JPRB, 0.60540E+00_JPRB, 0.59102E+00_JPRB, 0.57293E+00_JPRB, 0.56314E+00_JPRB, 0.55352E+00_JPRB ,&
&  0.54523E+00_JPRB, 0.54152E+00_JPRB, 0.54189E+00_JPRB, 0.55064E+00_JPRB, 0.56332E+00_JPRB, 0.58245E+00_JPRB, 0.64536E+00_JPRB ,&
&  0.10221E+01_JPRB, 0.14779E+01_JPRB, 0.23846E+01_JPRB, 0.24849E+01_JPRB, 0.22395E+01_JPRB, 0.19490E+01_JPRB, 0.19133E+01_JPRB ,&
&  0.18458E+01_JPRB, 0.17473E+01_JPRB, 0.16223E+01_JPRB, 0.15344E+01_JPRB, 0.14394E+01_JPRB, 0.13741E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.10883E+01_JPRB, 0.98706E+00_JPRB, 0.90636E+00_JPRB, 0.85343E+00_JPRB, 0.83424E+00_JPRB, 0.85355E+00_JPRB ,&
&  0.89212E+00_JPRB, 0.94842E+00_JPRB, 0.11572E+01_JPRB, 0.15998E+01_JPRB, 0.22937E+01_JPRB, 0.24560E+01_JPRB, 0.33605E+01_JPRB ,&
&  0.45605E+01_JPRB, 0.45237E+01_JPRB, 0.40001E+01_JPRB, 0.34209E+01_JPRB, 0.28374E+01_JPRB, 0.26095E+01_JPRB, 0.25536E+01_JPRB ,&
&  0.23637E+01_JPRB, 0.20594E+01_JPRB, 0.17584E+01_JPRB, 0.15339E+01_JPRB, 0.13802E+01_JPRB, 0.12847E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11049E+01_JPRB, 0.10448E+01_JPRB, 0.99706E+00_JPRB, 0.96676E+00_JPRB, 0.95270E+00_JPRB, 0.94788E+00_JPRB ,&
&  0.95070E+00_JPRB, 0.94885E+00_JPRB, 0.93571E+00_JPRB, 0.91623E+00_JPRB, 0.93371E+00_JPRB, 0.96991E+00_JPRB, 0.98446E+00_JPRB ,&
&  0.96591E+00_JPRB, 0.93401E+00_JPRB, 0.88306E+00_JPRB, 0.83680E+00_JPRB, 0.79825E+00_JPRB, 0.81239E+00_JPRB, 0.79135E+00_JPRB ,&
&  0.76590E+00_JPRB, 0.76113E+00_JPRB, 0.77569E+00_JPRB, 0.77252E+00_JPRB, 0.75681E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.73803E+00_JPRB, 0.71188E+00_JPRB, 0.68871E+00_JPRB, 0.66809E+00_JPRB, 0.65222E+00_JPRB, 0.64807E+00_JPRB, 0.65224E+00_JPRB ,&
&  0.65615E+00_JPRB, 0.65266E+00_JPRB, 0.64454E+00_JPRB, 0.63399E+00_JPRB, 0.62611E+00_JPRB, 0.61870E+00_JPRB, 0.61225E+00_JPRB ,&
&  0.60389E+00_JPRB, 0.60461E+00_JPRB, 0.60043E+00_JPRB, 0.59158E+00_JPRB, 0.58087E+00_JPRB, 0.56786E+00_JPRB, 0.55538E+00_JPRB ,&
&  0.54656E+00_JPRB, 0.54231E+00_JPRB, 0.54132E+00_JPRB, 0.54556E+00_JPRB, 0.55935E+00_JPRB, 0.57622E+00_JPRB, 0.60971E+00_JPRB ,&
&  0.88832E+00_JPRB, 0.14386E+01_JPRB, 0.18183E+01_JPRB, 0.20644E+01_JPRB, 0.18344E+01_JPRB, 0.15648E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.13410E+01_JPRB, 0.12826E+01_JPRB, 0.12077E+01_JPRB, 0.11176E+01_JPRB, 0.10719E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.96568E+00_JPRB, 0.91324E+00_JPRB, 0.86644E+00_JPRB, 0.83936E+00_JPRB, 0.81546E+00_JPRB, 0.79023E+00_JPRB, 0.77698E+00_JPRB ,&
&  0.78279E+00_JPRB, 0.80034E+00_JPRB, 0.86996E+00_JPRB, 0.11347E+01_JPRB, 0.17094E+01_JPRB, 0.21677E+01_JPRB, 0.27462E+01_JPRB ,&
&  0.29140E+01_JPRB, 0.30749E+01_JPRB, 0.31776E+01_JPRB, 0.30921E+01_JPRB, 0.28558E+01_JPRB, 0.27127E+01_JPRB, 0.26896E+01_JPRB ,&
&  0.25236E+01_JPRB, 0.21193E+01_JPRB, 0.17392E+01_JPRB, 0.15076E+01_JPRB, 0.13926E+01_JPRB, 0.13354E+01_JPRB, 0.12874E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.12120E+01_JPRB, 0.11589E+01_JPRB, 0.10996E+01_JPRB, 0.10468E+01_JPRB, 0.10023E+01_JPRB, 0.97525E+00_JPRB ,&
&  0.95406E+00_JPRB, 0.94270E+00_JPRB, 0.95077E+00_JPRB, 0.97502E+00_JPRB, 0.10015E+01_JPRB, 0.10234E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.10341E+01_JPRB, 0.99639E+00_JPRB, 0.91043E+00_JPRB, 0.85048E+00_JPRB, 0.80433E+00_JPRB, 0.79784E+00_JPRB, 0.76529E+00_JPRB ,&
&  0.74416E+00_JPRB, 0.73649E+00_JPRB, 0.74813E+00_JPRB, 0.75629E+00_JPRB, 0.75921E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.80947E+00_JPRB, 0.75820E+00_JPRB, 0.71426E+00_JPRB, 0.68092E+00_JPRB, 0.66300E+00_JPRB, 0.65422E+00_JPRB, 0.64732E+00_JPRB ,&
&  0.64217E+00_JPRB, 0.63370E+00_JPRB, 0.61991E+00_JPRB, 0.60927E+00_JPRB, 0.60152E+00_JPRB, 0.59680E+00_JPRB, 0.59204E+00_JPRB ,&
&  0.58033E+00_JPRB, 0.57796E+00_JPRB, 0.57778E+00_JPRB, 0.57256E+00_JPRB, 0.56534E+00_JPRB, 0.55688E+00_JPRB, 0.54580E+00_JPRB ,&
&  0.53752E+00_JPRB, 0.53432E+00_JPRB, 0.53545E+00_JPRB, 0.53956E+00_JPRB, 0.54463E+00_JPRB, 0.55824E+00_JPRB, 0.59534E+00_JPRB ,&
&  0.83118E+00_JPRB, 0.11372E+01_JPRB, 0.13847E+01_JPRB, 0.14976E+01_JPRB, 0.12880E+01_JPRB, 0.11720E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11186E+01_JPRB, 0.10600E+01_JPRB, 0.96297E+00_JPRB, 0.87423E+00_JPRB, 0.82569E+00_JPRB, 0.79339E+00_JPRB ,&
&  0.76463E+00_JPRB, 0.74149E+00_JPRB, 0.72056E+00_JPRB, 0.71703E+00_JPRB, 0.71451E+00_JPRB, 0.70726E+00_JPRB, 0.70121E+00_JPRB ,&
&  0.70029E+00_JPRB, 0.70679E+00_JPRB, 0.72590E+00_JPRB, 0.82369E+00_JPRB, 0.11060E+01_JPRB, 0.14238E+01_JPRB, 0.17428E+01_JPRB ,&
&  0.18942E+01_JPRB, 0.20429E+01_JPRB, 0.22165E+01_JPRB, 0.23349E+01_JPRB, 0.23444E+01_JPRB, 0.23396E+01_JPRB, 0.23356E+01_JPRB ,&
&  0.22703E+01_JPRB, 0.20305E+01_JPRB, 0.17139E+01_JPRB, 0.14849E+01_JPRB, 0.13617E+01_JPRB, 0.12992E+01_JPRB, 0.12572E+01_JPRB ,&
&  0.12206E+01_JPRB, 0.11868E+01_JPRB, 0.11422E+01_JPRB, 0.10836E+01_JPRB, 0.10209E+01_JPRB, 0.97591E+00_JPRB, 0.94298E+00_JPRB ,&
&  0.92168E+00_JPRB, 0.92411E+00_JPRB, 0.94903E+00_JPRB, 0.99278E+00_JPRB, 0.10224E+01_JPRB, 0.10342E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10110E+01_JPRB, 0.99800E+00_JPRB, 0.90902E+00_JPRB, 0.84846E+00_JPRB, 0.85822E+00_JPRB, 0.79432E+00_JPRB, 0.74033E+00_JPRB ,&
&  0.72187E+00_JPRB, 0.70765E+00_JPRB, 0.70533E+00_JPRB, 0.72154E+00_JPRB, 0.83250E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.74464E+00_JPRB, 0.72973E+00_JPRB, 0.70266E+00_JPRB, 0.67739E+00_JPRB, 0.66225E+00_JPRB, 0.65373E+00_JPRB, 0.64809E+00_JPRB ,&
&  0.63942E+00_JPRB, 0.62665E+00_JPRB, 0.61176E+00_JPRB, 0.59417E+00_JPRB, 0.58264E+00_JPRB, 0.57626E+00_JPRB, 0.57139E+00_JPRB ,&
&  0.56429E+00_JPRB, 0.55308E+00_JPRB, 0.54922E+00_JPRB, 0.54577E+00_JPRB, 0.54400E+00_JPRB, 0.54144E+00_JPRB, 0.53542E+00_JPRB ,&
&  0.52814E+00_JPRB, 0.52451E+00_JPRB, 0.52717E+00_JPRB, 0.52852E+00_JPRB, 0.53086E+00_JPRB, 0.53826E+00_JPRB, 0.56344E+00_JPRB ,&
&  0.71618E+00_JPRB, 0.87535E+00_JPRB, 0.10500E+01_JPRB, 0.11891E+01_JPRB, 0.11060E+01_JPRB, 0.10264E+01_JPRB, 0.98073E+00_JPRB ,&
&  0.92479E+00_JPRB, 0.88868E+00_JPRB, 0.83031E+00_JPRB, 0.76278E+00_JPRB, 0.70648E+00_JPRB, 0.68488E+00_JPRB, 0.67199E+00_JPRB ,&
&  0.66144E+00_JPRB, 0.65291E+00_JPRB, 0.64602E+00_JPRB, 0.64222E+00_JPRB, 0.64190E+00_JPRB, 0.63956E+00_JPRB, 0.63724E+00_JPRB ,&
&  0.63818E+00_JPRB, 0.64418E+00_JPRB, 0.65380E+00_JPRB, 0.67882E+00_JPRB, 0.78490E+00_JPRB, 0.98356E+00_JPRB, 0.11971E+01_JPRB ,&
&  0.13258E+01_JPRB, 0.14311E+01_JPRB, 0.15884E+01_JPRB, 0.17190E+01_JPRB, 0.18022E+01_JPRB, 0.18343E+01_JPRB, 0.18339E+01_JPRB ,&
&  0.18200E+01_JPRB, 0.17396E+01_JPRB, 0.15585E+01_JPRB, 0.13671E+01_JPRB, 0.12450E+01_JPRB, 0.11668E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10144E+01_JPRB, 0.96663E+00_JPRB, 0.92588E+00_JPRB, 0.89795E+00_JPRB, 0.87222E+00_JPRB, 0.85323E+00_JPRB ,&
&  0.86068E+00_JPRB, 0.89385E+00_JPRB, 0.94190E+00_JPRB, 0.97262E+00_JPRB, 0.97955E+00_JPRB, 0.97685E+00_JPRB, 0.96188E+00_JPRB ,&
&  0.94619E+00_JPRB, 0.93581E+00_JPRB, 0.90912E+00_JPRB, 0.87958E+00_JPRB, 0.83410E+00_JPRB, 0.78894E+00_JPRB, 0.72837E+00_JPRB ,&
&  0.69825E+00_JPRB, 0.68273E+00_JPRB, 0.66901E+00_JPRB, 0.67846E+00_JPRB, 0.72738E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.67547E+00_JPRB, 0.67677E+00_JPRB, 0.67379E+00_JPRB, 0.66670E+00_JPRB, 0.66327E+00_JPRB, 0.65817E+00_JPRB, 0.64787E+00_JPRB ,&
&  0.63491E+00_JPRB, 0.62289E+00_JPRB, 0.61030E+00_JPRB, 0.59398E+00_JPRB, 0.57668E+00_JPRB, 0.56869E+00_JPRB, 0.56530E+00_JPRB ,&
&  0.55949E+00_JPRB, 0.54982E+00_JPRB, 0.54088E+00_JPRB, 0.53826E+00_JPRB, 0.53691E+00_JPRB, 0.53575E+00_JPRB, 0.53178E+00_JPRB ,&
&  0.52562E+00_JPRB, 0.52432E+00_JPRB, 0.52708E+00_JPRB, 0.52715E+00_JPRB, 0.52273E+00_JPRB, 0.52483E+00_JPRB, 0.54640E+00_JPRB ,&
&  0.65645E+00_JPRB, 0.66121E+00_JPRB, 0.77232E+00_JPRB, 0.94547E+00_JPRB, 0.88496E+00_JPRB, 0.85358E+00_JPRB, 0.79241E+00_JPRB ,&
&  0.73066E+00_JPRB, 0.69585E+00_JPRB, 0.67150E+00_JPRB, 0.64469E+00_JPRB, 0.61917E+00_JPRB, 0.60553E+00_JPRB, 0.60535E+00_JPRB ,&
&  0.61249E+00_JPRB, 0.61544E+00_JPRB, 0.61487E+00_JPRB, 0.60697E+00_JPRB, 0.60140E+00_JPRB, 0.59665E+00_JPRB, 0.59352E+00_JPRB ,&
&  0.59155E+00_JPRB, 0.59481E+00_JPRB, 0.60157E+00_JPRB, 0.61026E+00_JPRB, 0.64060E+00_JPRB, 0.73544E+00_JPRB, 0.89054E+00_JPRB ,&
&  0.10263E+01_JPRB, 0.11343E+01_JPRB, 0.12404E+01_JPRB, 0.13512E+01_JPRB, 0.14355E+01_JPRB, 0.15115E+01_JPRB, 0.15525E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.14621E+01_JPRB, 0.13626E+01_JPRB, 0.12450E+01_JPRB, 0.11334E+01_JPRB, 0.10428E+01_JPRB, 0.97312E+00_JPRB ,&
&  0.92039E+00_JPRB, 0.88600E+00_JPRB, 0.85871E+00_JPRB, 0.83041E+00_JPRB, 0.81447E+00_JPRB, 0.81203E+00_JPRB, 0.82347E+00_JPRB ,&
&  0.84853E+00_JPRB, 0.87696E+00_JPRB, 0.89544E+00_JPRB, 0.89888E+00_JPRB, 0.88800E+00_JPRB, 0.87577E+00_JPRB, 0.86319E+00_JPRB ,&
&  0.86063E+00_JPRB, 0.86585E+00_JPRB, 0.86812E+00_JPRB, 0.83166E+00_JPRB, 0.83108E+00_JPRB, 0.80710E+00_JPRB, 0.75931E+00_JPRB ,&
&  0.71363E+00_JPRB, 0.68304E+00_JPRB, 0.66385E+00_JPRB, 0.59628E+00_JPRB, 0.67005E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.64241E+00_JPRB, 0.64561E+00_JPRB, 0.64769E+00_JPRB, 0.64872E+00_JPRB, 0.64776E+00_JPRB, 0.64175E+00_JPRB, 0.63009E+00_JPRB ,&
&  0.61755E+00_JPRB, 0.60517E+00_JPRB, 0.59504E+00_JPRB, 0.58423E+00_JPRB, 0.56834E+00_JPRB, 0.55702E+00_JPRB, 0.55340E+00_JPRB ,&
&  0.55018E+00_JPRB, 0.54352E+00_JPRB, 0.53611E+00_JPRB, 0.53439E+00_JPRB, 0.53167E+00_JPRB, 0.52782E+00_JPRB, 0.52261E+00_JPRB ,&
&  0.51746E+00_JPRB, 0.51559E+00_JPRB, 0.51551E+00_JPRB, 0.51595E+00_JPRB, 0.51235E+00_JPRB, 0.51444E+00_JPRB, 0.53690E+00_JPRB ,&
&  0.62420E+00_JPRB, 0.57216E+00_JPRB, 0.66343E+00_JPRB, 0.75917E+00_JPRB, 0.70220E+00_JPRB, 0.65812E+00_JPRB, 0.62212E+00_JPRB ,&
&  0.59443E+00_JPRB, 0.57888E+00_JPRB, 0.57036E+00_JPRB, 0.56214E+00_JPRB, 0.55555E+00_JPRB, 0.54910E+00_JPRB, 0.54879E+00_JPRB ,&
&  0.55202E+00_JPRB, 0.56140E+00_JPRB, 0.56704E+00_JPRB, 0.56700E+00_JPRB, 0.56179E+00_JPRB, 0.55580E+00_JPRB, 0.55035E+00_JPRB ,&
&  0.54698E+00_JPRB, 0.54717E+00_JPRB, 0.55007E+00_JPRB, 0.55660E+00_JPRB, 0.57209E+00_JPRB, 0.61256E+00_JPRB, 0.69805E+00_JPRB ,&
&  0.80575E+00_JPRB, 0.89175E+00_JPRB, 0.97551E+00_JPRB, 0.10818E+01_JPRB, 0.11875E+01_JPRB, 0.12648E+01_JPRB, 0.12938E+01_JPRB ,&
&  0.12740E+01_JPRB, 0.12121E+01_JPRB, 0.11532E+01_JPRB, 0.10887E+01_JPRB, 0.10239E+01_JPRB, 0.96698E+00_JPRB, 0.92609E+00_JPRB ,&
&  0.89848E+00_JPRB, 0.88019E+00_JPRB, 0.87035E+00_JPRB, 0.86485E+00_JPRB, 0.86615E+00_JPRB, 0.86993E+00_JPRB, 0.87609E+00_JPRB ,&
&  0.87249E+00_JPRB, 0.86370E+00_JPRB, 0.83858E+00_JPRB, 0.80183E+00_JPRB, 0.77474E+00_JPRB, 0.75978E+00_JPRB, 0.75255E+00_JPRB ,&
&  0.75318E+00_JPRB, 0.76196E+00_JPRB, 0.77447E+00_JPRB, 0.79462E+00_JPRB, 0.79477E+00_JPRB, 0.79281E+00_JPRB, 0.77272E+00_JPRB ,&
&  0.73424E+00_JPRB, 0.69432E+00_JPRB, 0.57979E+00_JPRB, 0.63044E+00_JPRB, 0.64382E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.65265E+00_JPRB, 0.64635E+00_JPRB, 0.64080E+00_JPRB, 0.63500E+00_JPRB, 0.62707E+00_JPRB, 0.61701E+00_JPRB, 0.60510E+00_JPRB ,&
&  0.59357E+00_JPRB, 0.58232E+00_JPRB, 0.57103E+00_JPRB, 0.56091E+00_JPRB, 0.55112E+00_JPRB, 0.54576E+00_JPRB, 0.54361E+00_JPRB ,&
&  0.54328E+00_JPRB, 0.54166E+00_JPRB, 0.53877E+00_JPRB, 0.53779E+00_JPRB, 0.53676E+00_JPRB, 0.53339E+00_JPRB, 0.52877E+00_JPRB ,&
&  0.52174E+00_JPRB, 0.51568E+00_JPRB, 0.51304E+00_JPRB, 0.51486E+00_JPRB, 0.51337E+00_JPRB, 0.51655E+00_JPRB, 0.53009E+00_JPRB ,&
&  0.52868E+00_JPRB, 0.51905E+00_JPRB, 0.57553E+00_JPRB, 0.64564E+00_JPRB, 0.60454E+00_JPRB, 0.56604E+00_JPRB, 0.55042E+00_JPRB ,&
&  0.53715E+00_JPRB, 0.52860E+00_JPRB, 0.52285E+00_JPRB, 0.52002E+00_JPRB, 0.52060E+00_JPRB, 0.52047E+00_JPRB, 0.52017E+00_JPRB ,&
&  0.52097E+00_JPRB, 0.52555E+00_JPRB, 0.53067E+00_JPRB, 0.53396E+00_JPRB, 0.53342E+00_JPRB, 0.53046E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.51988E+00_JPRB, 0.51660E+00_JPRB, 0.51619E+00_JPRB, 0.52044E+00_JPRB, 0.52925E+00_JPRB, 0.54783E+00_JPRB, 0.58807E+00_JPRB ,&
&  0.65249E+00_JPRB, 0.71972E+00_JPRB, 0.78260E+00_JPRB, 0.85831E+00_JPRB, 0.95501E+00_JPRB, 0.10467E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.10608E+01_JPRB, 0.99795E+00_JPRB, 0.94176E+00_JPRB, 0.89826E+00_JPRB, 0.87026E+00_JPRB, 0.85382E+00_JPRB ,&
&  0.84784E+00_JPRB, 0.84522E+00_JPRB, 0.84498E+00_JPRB, 0.84559E+00_JPRB, 0.84496E+00_JPRB, 0.83881E+00_JPRB, 0.82674E+00_JPRB ,&
&  0.80524E+00_JPRB, 0.77098E+00_JPRB, 0.73136E+00_JPRB, 0.69878E+00_JPRB, 0.67611E+00_JPRB, 0.66460E+00_JPRB, 0.66199E+00_JPRB ,&
&  0.66689E+00_JPRB, 0.68059E+00_JPRB, 0.69964E+00_JPRB, 0.73201E+00_JPRB, 0.75527E+00_JPRB, 0.76969E+00_JPRB, 0.77335E+00_JPRB ,&
&  0.75495E+00_JPRB, 0.72233E+00_JPRB, 0.69001E+00_JPRB, 0.67198E+00_JPRB, 0.66327E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.65364E+00_JPRB, 0.63575E+00_JPRB, 0.61845E+00_JPRB, 0.60627E+00_JPRB, 0.59657E+00_JPRB, 0.58714E+00_JPRB, 0.57668E+00_JPRB ,&
&  0.56435E+00_JPRB, 0.55196E+00_JPRB, 0.54067E+00_JPRB, 0.53036E+00_JPRB, 0.52495E+00_JPRB, 0.52216E+00_JPRB, 0.52206E+00_JPRB ,&
&  0.52453E+00_JPRB, 0.52904E+00_JPRB, 0.53003E+00_JPRB, 0.52874E+00_JPRB, 0.52902E+00_JPRB, 0.52841E+00_JPRB, 0.52584E+00_JPRB ,&
&  0.52049E+00_JPRB, 0.51422E+00_JPRB, 0.51041E+00_JPRB, 0.51047E+00_JPRB, 0.51040E+00_JPRB, 0.51209E+00_JPRB, 0.51507E+00_JPRB ,&
&  0.52682E+00_JPRB, 0.49528E+00_JPRB, 0.53460E+00_JPRB, 0.57900E+00_JPRB, 0.56470E+00_JPRB, 0.54602E+00_JPRB, 0.53027E+00_JPRB ,&
&  0.51708E+00_JPRB, 0.50967E+00_JPRB, 0.50572E+00_JPRB, 0.50662E+00_JPRB, 0.51009E+00_JPRB, 0.51224E+00_JPRB, 0.51323E+00_JPRB ,&
&  0.51096E+00_JPRB, 0.51123E+00_JPRB, 0.51002E+00_JPRB, 0.50932E+00_JPRB, 0.50973E+00_JPRB, 0.51076E+00_JPRB, 0.51058E+00_JPRB ,&
&  0.50820E+00_JPRB, 0.50405E+00_JPRB, 0.50002E+00_JPRB, 0.49913E+00_JPRB, 0.50066E+00_JPRB, 0.50596E+00_JPRB, 0.51972E+00_JPRB ,&
&  0.54641E+00_JPRB, 0.58085E+00_JPRB, 0.62358E+00_JPRB, 0.68034E+00_JPRB, 0.75840E+00_JPRB, 0.84018E+00_JPRB, 0.90427E+00_JPRB ,&
&  0.92871E+00_JPRB, 0.91781E+00_JPRB, 0.88658E+00_JPRB, 0.84714E+00_JPRB, 0.80967E+00_JPRB, 0.77448E+00_JPRB, 0.74441E+00_JPRB ,&
&  0.72272E+00_JPRB, 0.70639E+00_JPRB, 0.69445E+00_JPRB, 0.68646E+00_JPRB, 0.67848E+00_JPRB, 0.66723E+00_JPRB, 0.65568E+00_JPRB ,&
&  0.64211E+00_JPRB, 0.62864E+00_JPRB, 0.61713E+00_JPRB, 0.60671E+00_JPRB, 0.59710E+00_JPRB, 0.58950E+00_JPRB, 0.58715E+00_JPRB ,&
&  0.59175E+00_JPRB, 0.60217E+00_JPRB, 0.62073E+00_JPRB, 0.64646E+00_JPRB, 0.67628E+00_JPRB, 0.70050E+00_JPRB, 0.71565E+00_JPRB ,&
&  0.71829E+00_JPRB, 0.71360E+00_JPRB, 0.69949E+00_JPRB, 0.68359E+00_JPRB, 0.66991E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.62690E+00_JPRB, 0.61198E+00_JPRB, 0.59703E+00_JPRB, 0.58478E+00_JPRB, 0.57466E+00_JPRB, 0.56369E+00_JPRB, 0.55145E+00_JPRB ,&
&  0.54109E+00_JPRB, 0.53058E+00_JPRB, 0.52288E+00_JPRB, 0.51651E+00_JPRB, 0.51204E+00_JPRB, 0.51048E+00_JPRB, 0.51103E+00_JPRB ,&
&  0.51370E+00_JPRB, 0.51950E+00_JPRB, 0.52181E+00_JPRB, 0.51946E+00_JPRB, 0.52080E+00_JPRB, 0.52287E+00_JPRB, 0.52287E+00_JPRB ,&
&  0.51951E+00_JPRB, 0.51531E+00_JPRB, 0.51084E+00_JPRB, 0.50802E+00_JPRB, 0.50767E+00_JPRB, 0.50930E+00_JPRB, 0.50745E+00_JPRB ,&
&  0.50538E+00_JPRB, 0.50852E+00_JPRB, 0.52678E+00_JPRB, 0.54297E+00_JPRB, 0.54242E+00_JPRB, 0.53199E+00_JPRB, 0.51963E+00_JPRB ,&
&  0.50654E+00_JPRB, 0.49957E+00_JPRB, 0.49917E+00_JPRB, 0.50156E+00_JPRB, 0.50474E+00_JPRB, 0.50679E+00_JPRB, 0.50922E+00_JPRB ,&
&  0.50645E+00_JPRB, 0.50469E+00_JPRB, 0.50161E+00_JPRB, 0.49887E+00_JPRB, 0.49894E+00_JPRB, 0.50001E+00_JPRB, 0.50099E+00_JPRB ,&
&  0.50030E+00_JPRB, 0.50093E+00_JPRB, 0.50127E+00_JPRB, 0.49982E+00_JPRB, 0.50078E+00_JPRB, 0.50290E+00_JPRB, 0.50689E+00_JPRB ,&
&  0.51542E+00_JPRB, 0.52981E+00_JPRB, 0.54864E+00_JPRB, 0.57792E+00_JPRB, 0.62644E+00_JPRB, 0.68322E+00_JPRB, 0.72945E+00_JPRB ,&
&  0.75426E+00_JPRB, 0.75823E+00_JPRB, 0.74870E+00_JPRB, 0.73074E+00_JPRB, 0.71254E+00_JPRB, 0.69425E+00_JPRB, 0.67386E+00_JPRB ,&
&  0.65211E+00_JPRB, 0.63502E+00_JPRB, 0.62044E+00_JPRB, 0.60914E+00_JPRB, 0.59944E+00_JPRB, 0.59076E+00_JPRB, 0.58437E+00_JPRB ,&
&  0.58024E+00_JPRB, 0.57634E+00_JPRB, 0.57079E+00_JPRB, 0.56481E+00_JPRB, 0.56061E+00_JPRB, 0.55542E+00_JPRB, 0.55251E+00_JPRB ,&
&  0.55395E+00_JPRB, 0.55993E+00_JPRB, 0.57087E+00_JPRB, 0.58841E+00_JPRB, 0.60779E+00_JPRB, 0.62565E+00_JPRB, 0.64070E+00_JPRB ,&
&  0.65223E+00_JPRB, 0.65800E+00_JPRB, 0.65702E+00_JPRB, 0.65134E+00_JPRB, 0.64124E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.57349E+00_JPRB, 0.56617E+00_JPRB, 0.55697E+00_JPRB, 0.54743E+00_JPRB, 0.53820E+00_JPRB, 0.52899E+00_JPRB, 0.52061E+00_JPRB ,&
&  0.51364E+00_JPRB, 0.50931E+00_JPRB, 0.50667E+00_JPRB, 0.50470E+00_JPRB, 0.50102E+00_JPRB, 0.49937E+00_JPRB, 0.50008E+00_JPRB ,&
&  0.50270E+00_JPRB, 0.50537E+00_JPRB, 0.50673E+00_JPRB, 0.50547E+00_JPRB, 0.50735E+00_JPRB, 0.50906E+00_JPRB, 0.50875E+00_JPRB ,&
&  0.50716E+00_JPRB, 0.50538E+00_JPRB, 0.50225E+00_JPRB, 0.49885E+00_JPRB, 0.49844E+00_JPRB, 0.49903E+00_JPRB, 0.49582E+00_JPRB ,&
&  0.48895E+00_JPRB, 0.49764E+00_JPRB, 0.49037E+00_JPRB, 0.51173E+00_JPRB, 0.51788E+00_JPRB, 0.51110E+00_JPRB, 0.50154E+00_JPRB ,&
&  0.49578E+00_JPRB, 0.49437E+00_JPRB, 0.49711E+00_JPRB, 0.49988E+00_JPRB, 0.50153E+00_JPRB, 0.50251E+00_JPRB, 0.50241E+00_JPRB ,&
&  0.49383E+00_JPRB, 0.48808E+00_JPRB, 0.48643E+00_JPRB, 0.48643E+00_JPRB, 0.48602E+00_JPRB, 0.48638E+00_JPRB, 0.48817E+00_JPRB ,&
&  0.48977E+00_JPRB, 0.49084E+00_JPRB, 0.49189E+00_JPRB, 0.49416E+00_JPRB, 0.49555E+00_JPRB, 0.49605E+00_JPRB, 0.49785E+00_JPRB ,&
&  0.50121E+00_JPRB, 0.50693E+00_JPRB, 0.51442E+00_JPRB, 0.52499E+00_JPRB, 0.54692E+00_JPRB, 0.57952E+00_JPRB, 0.60953E+00_JPRB ,&
&  0.62719E+00_JPRB, 0.63396E+00_JPRB, 0.63403E+00_JPRB, 0.62682E+00_JPRB, 0.61499E+00_JPRB, 0.60382E+00_JPRB, 0.59323E+00_JPRB ,&
&  0.58000E+00_JPRB, 0.57009E+00_JPRB, 0.56099E+00_JPRB, 0.55033E+00_JPRB, 0.54069E+00_JPRB, 0.53349E+00_JPRB, 0.52855E+00_JPRB ,&
&  0.52633E+00_JPRB, 0.52640E+00_JPRB, 0.52524E+00_JPRB, 0.52477E+00_JPRB, 0.52652E+00_JPRB, 0.52901E+00_JPRB, 0.52831E+00_JPRB ,&
&  0.52658E+00_JPRB, 0.52631E+00_JPRB, 0.52899E+00_JPRB, 0.53460E+00_JPRB, 0.54137E+00_JPRB, 0.55020E+00_JPRB, 0.55991E+00_JPRB ,&
&  0.56924E+00_JPRB, 0.57636E+00_JPRB, 0.58032E+00_JPRB, 0.58174E+00_JPRB, 0.57904E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.51806E+00_JPRB, 0.51595E+00_JPRB, 0.51292E+00_JPRB, 0.50823E+00_JPRB, 0.50308E+00_JPRB, 0.49852E+00_JPRB, 0.49488E+00_JPRB ,&
&  0.49255E+00_JPRB, 0.49181E+00_JPRB, 0.49224E+00_JPRB, 0.49242E+00_JPRB, 0.49232E+00_JPRB, 0.49329E+00_JPRB, 0.49380E+00_JPRB ,&
&  0.49553E+00_JPRB, 0.49582E+00_JPRB, 0.49288E+00_JPRB, 0.49180E+00_JPRB, 0.49403E+00_JPRB, 0.49859E+00_JPRB, 0.49984E+00_JPRB ,&
&  0.49693E+00_JPRB, 0.49596E+00_JPRB, 0.49495E+00_JPRB, 0.49204E+00_JPRB, 0.48990E+00_JPRB, 0.48891E+00_JPRB, 0.48531E+00_JPRB ,&
&  0.47974E+00_JPRB, 0.48270E+00_JPRB, 0.48443E+00_JPRB, 0.49205E+00_JPRB, 0.49541E+00_JPRB, 0.49631E+00_JPRB, 0.49728E+00_JPRB ,&
&  0.49525E+00_JPRB, 0.49424E+00_JPRB, 0.49346E+00_JPRB, 0.49352E+00_JPRB, 0.49126E+00_JPRB, 0.48760E+00_JPRB, 0.48070E+00_JPRB ,&
&  0.46881E+00_JPRB, 0.46392E+00_JPRB, 0.46537E+00_JPRB, 0.46971E+00_JPRB, 0.46886E+00_JPRB, 0.46910E+00_JPRB, 0.47214E+00_JPRB ,&
&  0.47617E+00_JPRB, 0.47918E+00_JPRB, 0.48101E+00_JPRB, 0.48208E+00_JPRB, 0.48358E+00_JPRB, 0.48499E+00_JPRB, 0.48569E+00_JPRB ,&
&  0.48669E+00_JPRB, 0.48873E+00_JPRB, 0.49217E+00_JPRB, 0.49733E+00_JPRB, 0.50577E+00_JPRB, 0.52048E+00_JPRB, 0.53615E+00_JPRB ,&
&  0.54678E+00_JPRB, 0.55067E+00_JPRB, 0.55114E+00_JPRB, 0.54765E+00_JPRB, 0.54027E+00_JPRB, 0.53241E+00_JPRB, 0.52449E+00_JPRB ,&
&  0.51407E+00_JPRB, 0.50502E+00_JPRB, 0.49778E+00_JPRB, 0.49330E+00_JPRB, 0.49053E+00_JPRB, 0.48925E+00_JPRB, 0.48934E+00_JPRB ,&
&  0.48986E+00_JPRB, 0.49244E+00_JPRB, 0.49853E+00_JPRB, 0.50177E+00_JPRB, 0.50300E+00_JPRB, 0.50604E+00_JPRB, 0.50767E+00_JPRB ,&
&  0.50591E+00_JPRB, 0.50437E+00_JPRB, 0.50368E+00_JPRB, 0.50629E+00_JPRB, 0.50816E+00_JPRB, 0.50806E+00_JPRB, 0.50869E+00_JPRB ,&
&  0.51214E+00_JPRB, 0.51608E+00_JPRB, 0.51755E+00_JPRB, 0.51838E+00_JPRB, 0.51921E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.47450E+00_JPRB, 0.47460E+00_JPRB, 0.47390E+00_JPRB, 0.47263E+00_JPRB, 0.47136E+00_JPRB, 0.47038E+00_JPRB, 0.47051E+00_JPRB ,&
&  0.47083E+00_JPRB, 0.47172E+00_JPRB, 0.47295E+00_JPRB, 0.47397E+00_JPRB, 0.47591E+00_JPRB, 0.47817E+00_JPRB, 0.48016E+00_JPRB ,&
&  0.48216E+00_JPRB, 0.48152E+00_JPRB, 0.47798E+00_JPRB, 0.47452E+00_JPRB, 0.47518E+00_JPRB, 0.48074E+00_JPRB, 0.48673E+00_JPRB ,&
&  0.48779E+00_JPRB, 0.48833E+00_JPRB, 0.48818E+00_JPRB, 0.48631E+00_JPRB, 0.48402E+00_JPRB, 0.48345E+00_JPRB, 0.47998E+00_JPRB ,&
&  0.47497E+00_JPRB, 0.47432E+00_JPRB, 0.47353E+00_JPRB, 0.47288E+00_JPRB, 0.47517E+00_JPRB, 0.48074E+00_JPRB, 0.48256E+00_JPRB ,&
&  0.48317E+00_JPRB, 0.48148E+00_JPRB, 0.47754E+00_JPRB, 0.47329E+00_JPRB, 0.46796E+00_JPRB, 0.45893E+00_JPRB, 0.44869E+00_JPRB ,&
&  0.44064E+00_JPRB, 0.44009E+00_JPRB, 0.44569E+00_JPRB, 0.45133E+00_JPRB, 0.44949E+00_JPRB, 0.44756E+00_JPRB, 0.44879E+00_JPRB ,&
&  0.45246E+00_JPRB, 0.45742E+00_JPRB, 0.46064E+00_JPRB, 0.45999E+00_JPRB, 0.45886E+00_JPRB, 0.46078E+00_JPRB, 0.46353E+00_JPRB ,&
&  0.46472E+00_JPRB, 0.46600E+00_JPRB, 0.46699E+00_JPRB, 0.46902E+00_JPRB, 0.47127E+00_JPRB, 0.47433E+00_JPRB, 0.48011E+00_JPRB ,&
&  0.48423E+00_JPRB, 0.48576E+00_JPRB, 0.48353E+00_JPRB, 0.47812E+00_JPRB, 0.47233E+00_JPRB, 0.46699E+00_JPRB, 0.46413E+00_JPRB ,&
&  0.46064E+00_JPRB, 0.45984E+00_JPRB, 0.45861E+00_JPRB, 0.45542E+00_JPRB, 0.45190E+00_JPRB, 0.45157E+00_JPRB, 0.45512E+00_JPRB ,&
&  0.46023E+00_JPRB, 0.46460E+00_JPRB, 0.47095E+00_JPRB, 0.47357E+00_JPRB, 0.47520E+00_JPRB, 0.47775E+00_JPRB, 0.47869E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.47386E+00_JPRB, 0.47333E+00_JPRB, 0.47633E+00_JPRB, 0.47734E+00_JPRB, 0.47617E+00_JPRB, 0.47262E+00_JPRB ,&
&  0.47112E+00_JPRB, 0.47259E+00_JPRB, 0.47300E+00_JPRB, 0.47245E+00_JPRB, 0.47306E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.44593E+00_JPRB, 0.44820E+00_JPRB, 0.45067E+00_JPRB, 0.45292E+00_JPRB, 0.45433E+00_JPRB, 0.45530E+00_JPRB, 0.45447E+00_JPRB ,&
&  0.45443E+00_JPRB, 0.45626E+00_JPRB, 0.45862E+00_JPRB, 0.46062E+00_JPRB, 0.46193E+00_JPRB, 0.46429E+00_JPRB, 0.46684E+00_JPRB ,&
&  0.46860E+00_JPRB, 0.46763E+00_JPRB, 0.46336E+00_JPRB, 0.46085E+00_JPRB, 0.45954E+00_JPRB, 0.46497E+00_JPRB, 0.47396E+00_JPRB ,&
&  0.47786E+00_JPRB, 0.48005E+00_JPRB, 0.48265E+00_JPRB, 0.48280E+00_JPRB, 0.48256E+00_JPRB, 0.48224E+00_JPRB, 0.47771E+00_JPRB ,&
&  0.47238E+00_JPRB, 0.46784E+00_JPRB, 0.46425E+00_JPRB, 0.46116E+00_JPRB, 0.46230E+00_JPRB, 0.46179E+00_JPRB, 0.46178E+00_JPRB ,&
&  0.46164E+00_JPRB, 0.45915E+00_JPRB, 0.45472E+00_JPRB, 0.44761E+00_JPRB, 0.43904E+00_JPRB, 0.42794E+00_JPRB, 0.41999E+00_JPRB ,&
&  0.41669E+00_JPRB, 0.42066E+00_JPRB, 0.42972E+00_JPRB, 0.43322E+00_JPRB, 0.42885E+00_JPRB, 0.42702E+00_JPRB, 0.42987E+00_JPRB ,&
&  0.43489E+00_JPRB, 0.44081E+00_JPRB, 0.44334E+00_JPRB, 0.44267E+00_JPRB, 0.44053E+00_JPRB, 0.44215E+00_JPRB, 0.44670E+00_JPRB ,&
&  0.45054E+00_JPRB, 0.45236E+00_JPRB, 0.45127E+00_JPRB, 0.45021E+00_JPRB, 0.44801E+00_JPRB, 0.44368E+00_JPRB, 0.43811E+00_JPRB ,&
&  0.43295E+00_JPRB, 0.42975E+00_JPRB, 0.42722E+00_JPRB, 0.42428E+00_JPRB, 0.42290E+00_JPRB, 0.42284E+00_JPRB, 0.42474E+00_JPRB ,&
&  0.42523E+00_JPRB, 0.42677E+00_JPRB, 0.42669E+00_JPRB, 0.42243E+00_JPRB, 0.41916E+00_JPRB, 0.42101E+00_JPRB, 0.42313E+00_JPRB ,&
&  0.42633E+00_JPRB, 0.42952E+00_JPRB, 0.43251E+00_JPRB, 0.43344E+00_JPRB, 0.43486E+00_JPRB, 0.43781E+00_JPRB, 0.43971E+00_JPRB ,&
&  0.44071E+00_JPRB, 0.44136E+00_JPRB, 0.44330E+00_JPRB, 0.44677E+00_JPRB, 0.45021E+00_JPRB, 0.45307E+00_JPRB, 0.45260E+00_JPRB ,&
&  0.45096E+00_JPRB, 0.44870E+00_JPRB, 0.44842E+00_JPRB, 0.44806E+00_JPRB, 0.44609E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.43903E+00_JPRB, 0.43854E+00_JPRB, 0.44173E+00_JPRB, 0.44516E+00_JPRB, 0.44568E+00_JPRB, 0.44491E+00_JPRB, 0.44194E+00_JPRB ,&
&  0.43996E+00_JPRB, 0.44108E+00_JPRB, 0.44297E+00_JPRB, 0.44448E+00_JPRB, 0.44712E+00_JPRB, 0.45079E+00_JPRB, 0.45373E+00_JPRB ,&
&  0.45398E+00_JPRB, 0.45185E+00_JPRB, 0.44826E+00_JPRB, 0.44813E+00_JPRB, 0.44784E+00_JPRB, 0.45027E+00_JPRB, 0.45836E+00_JPRB ,&
&  0.46418E+00_JPRB, 0.46805E+00_JPRB, 0.47124E+00_JPRB, 0.47148E+00_JPRB, 0.47062E+00_JPRB, 0.47212E+00_JPRB, 0.46843E+00_JPRB ,&
&  0.46182E+00_JPRB, 0.45599E+00_JPRB, 0.44971E+00_JPRB, 0.44263E+00_JPRB, 0.44564E+00_JPRB, 0.44358E+00_JPRB, 0.44086E+00_JPRB ,&
&  0.43645E+00_JPRB, 0.43022E+00_JPRB, 0.42266E+00_JPRB, 0.41544E+00_JPRB, 0.40995E+00_JPRB, 0.40483E+00_JPRB, 0.39872E+00_JPRB ,&
&  0.39676E+00_JPRB, 0.39955E+00_JPRB, 0.40541E+00_JPRB, 0.40692E+00_JPRB, 0.40303E+00_JPRB, 0.40308E+00_JPRB, 0.40975E+00_JPRB ,&
&  0.41727E+00_JPRB, 0.42116E+00_JPRB, 0.42127E+00_JPRB, 0.42191E+00_JPRB, 0.42252E+00_JPRB, 0.42401E+00_JPRB, 0.42474E+00_JPRB ,&
&  0.42357E+00_JPRB, 0.42167E+00_JPRB, 0.41888E+00_JPRB, 0.41293E+00_JPRB, 0.40617E+00_JPRB, 0.40008E+00_JPRB, 0.39813E+00_JPRB ,&
&  0.40134E+00_JPRB, 0.40220E+00_JPRB, 0.40381E+00_JPRB, 0.40637E+00_JPRB, 0.40722E+00_JPRB, 0.40662E+00_JPRB, 0.40578E+00_JPRB ,&
&  0.40739E+00_JPRB, 0.40569E+00_JPRB, 0.40621E+00_JPRB, 0.40329E+00_JPRB, 0.39885E+00_JPRB, 0.39434E+00_JPRB, 0.38992E+00_JPRB ,&
&  0.38770E+00_JPRB, 0.38752E+00_JPRB, 0.38980E+00_JPRB, 0.39065E+00_JPRB, 0.39262E+00_JPRB, 0.39506E+00_JPRB, 0.39715E+00_JPRB ,&
&  0.39850E+00_JPRB, 0.40050E+00_JPRB, 0.40414E+00_JPRB, 0.41085E+00_JPRB, 0.41784E+00_JPRB, 0.42399E+00_JPRB, 0.42989E+00_JPRB ,&
&  0.43509E+00_JPRB, 0.43851E+00_JPRB, 0.44421E+00_JPRB, 0.44478E+00_JPRB, 0.44307E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.43694E+00_JPRB, 0.43360E+00_JPRB, 0.43305E+00_JPRB, 0.43333E+00_JPRB, 0.43192E+00_JPRB, 0.43016E+00_JPRB, 0.42679E+00_JPRB ,&
&  0.42396E+00_JPRB, 0.42422E+00_JPRB, 0.42569E+00_JPRB, 0.42855E+00_JPRB, 0.43358E+00_JPRB, 0.43892E+00_JPRB, 0.43876E+00_JPRB ,&
&  0.43585E+00_JPRB, 0.43481E+00_JPRB, 0.43726E+00_JPRB, 0.43897E+00_JPRB, 0.43891E+00_JPRB, 0.43960E+00_JPRB, 0.44360E+00_JPRB ,&
&  0.44846E+00_JPRB, 0.45074E+00_JPRB, 0.45381E+00_JPRB, 0.45332E+00_JPRB, 0.44922E+00_JPRB, 0.44946E+00_JPRB, 0.44752E+00_JPRB ,&
&  0.44243E+00_JPRB, 0.43575E+00_JPRB, 0.42601E+00_JPRB, 0.42649E+00_JPRB, 0.42270E+00_JPRB, 0.41946E+00_JPRB, 0.41443E+00_JPRB ,&
&  0.40875E+00_JPRB, 0.40358E+00_JPRB, 0.39793E+00_JPRB, 0.39346E+00_JPRB, 0.39141E+00_JPRB, 0.39130E+00_JPRB, 0.38912E+00_JPRB ,&
&  0.38806E+00_JPRB, 0.39080E+00_JPRB, 0.39198E+00_JPRB, 0.38712E+00_JPRB, 0.38535E+00_JPRB, 0.39108E+00_JPRB, 0.39653E+00_JPRB ,&
&  0.39922E+00_JPRB, 0.39953E+00_JPRB, 0.39666E+00_JPRB, 0.39589E+00_JPRB, 0.39485E+00_JPRB, 0.39285E+00_JPRB, 0.38919E+00_JPRB ,&
&  0.38344E+00_JPRB, 0.38056E+00_JPRB, 0.37862E+00_JPRB, 0.37479E+00_JPRB, 0.36721E+00_JPRB, 0.35155E+00_JPRB, 0.34299E+00_JPRB ,&
&  0.34479E+00_JPRB, 0.35436E+00_JPRB, 0.36825E+00_JPRB, 0.37670E+00_JPRB, 0.38508E+00_JPRB, 0.39082E+00_JPRB, 0.39096E+00_JPRB ,&
&  0.38968E+00_JPRB, 0.37022E+00_JPRB, 0.35931E+00_JPRB, 0.34944E+00_JPRB, 0.34076E+00_JPRB, 0.33589E+00_JPRB, 0.33405E+00_JPRB ,&
&  0.33718E+00_JPRB, 0.34296E+00_JPRB, 0.34907E+00_JPRB, 0.35245E+00_JPRB, 0.35261E+00_JPRB, 0.35029E+00_JPRB, 0.34720E+00_JPRB ,&
&  0.34811E+00_JPRB, 0.35666E+00_JPRB, 0.37189E+00_JPRB, 0.38454E+00_JPRB, 0.39468E+00_JPRB, 0.40573E+00_JPRB, 0.41473E+00_JPRB ,&
&  0.42110E+00_JPRB, 0.42714E+00_JPRB, 0.43414E+00_JPRB, 0.43656E+00_JPRB, 0.43730E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.42096E+00_JPRB, 0.41625E+00_JPRB, 0.41577E+00_JPRB, 0.41580E+00_JPRB, 0.41412E+00_JPRB, 0.41342E+00_JPRB, 0.41215E+00_JPRB ,&
&  0.41123E+00_JPRB, 0.40907E+00_JPRB, 0.40734E+00_JPRB, 0.40815E+00_JPRB, 0.41070E+00_JPRB, 0.41354E+00_JPRB, 0.41451E+00_JPRB ,&
&  0.41473E+00_JPRB, 0.41665E+00_JPRB, 0.42054E+00_JPRB, 0.42411E+00_JPRB, 0.42451E+00_JPRB, 0.42467E+00_JPRB, 0.42655E+00_JPRB ,&
&  0.42889E+00_JPRB, 0.42973E+00_JPRB, 0.43267E+00_JPRB, 0.43024E+00_JPRB, 0.42540E+00_JPRB, 0.42372E+00_JPRB, 0.42223E+00_JPRB ,&
&  0.41450E+00_JPRB, 0.38966E+00_JPRB, 0.37435E+00_JPRB, 0.38047E+00_JPRB, 0.39514E+00_JPRB, 0.39341E+00_JPRB, 0.39084E+00_JPRB ,&
&  0.38813E+00_JPRB, 0.38439E+00_JPRB, 0.38130E+00_JPRB, 0.38016E+00_JPRB, 0.38067E+00_JPRB, 0.38110E+00_JPRB, 0.38017E+00_JPRB ,&
&  0.38121E+00_JPRB, 0.38751E+00_JPRB, 0.39368E+00_JPRB, 0.38852E+00_JPRB, 0.38194E+00_JPRB, 0.37350E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.36334E+00_JPRB, 0.35877E+00_JPRB, 0.35553E+00_JPRB, 0.35465E+00_JPRB, 0.35443E+00_JPRB, 0.35205E+00_JPRB, 0.34795E+00_JPRB ,&
&  0.34183E+00_JPRB, 0.33197E+00_JPRB, 0.32241E+00_JPRB, 0.31296E+00_JPRB, 0.30678E+00_JPRB, 0.30217E+00_JPRB, 0.29825E+00_JPRB ,&
&  0.30107E+00_JPRB, 0.31541E+00_JPRB, 0.33673E+00_JPRB, 0.35785E+00_JPRB, 0.37243E+00_JPRB, 0.35183E+00_JPRB, 0.34222E+00_JPRB ,&
&  0.33000E+00_JPRB, 0.31927E+00_JPRB, 0.31168E+00_JPRB, 0.30616E+00_JPRB, 0.30205E+00_JPRB, 0.29959E+00_JPRB, 0.29929E+00_JPRB ,&
&  0.30277E+00_JPRB, 0.30745E+00_JPRB, 0.31184E+00_JPRB, 0.31362E+00_JPRB, 0.31353E+00_JPRB, 0.31187E+00_JPRB, 0.31097E+00_JPRB ,&
&  0.31285E+00_JPRB, 0.31981E+00_JPRB, 0.33057E+00_JPRB, 0.34637E+00_JPRB, 0.36001E+00_JPRB, 0.37386E+00_JPRB, 0.38899E+00_JPRB ,&
&  0.40437E+00_JPRB, 0.41672E+00_JPRB, 0.42024E+00_JPRB, 0.42387E+00_JPRB, 0.42437E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.40974E+00_JPRB, 0.40724E+00_JPRB, 0.40512E+00_JPRB, 0.40535E+00_JPRB, 0.40541E+00_JPRB, 0.40755E+00_JPRB, 0.40589E+00_JPRB ,&
&  0.40098E+00_JPRB, 0.39629E+00_JPRB, 0.39513E+00_JPRB, 0.39548E+00_JPRB, 0.39451E+00_JPRB, 0.39300E+00_JPRB, 0.39409E+00_JPRB ,&
&  0.39265E+00_JPRB, 0.39526E+00_JPRB, 0.39883E+00_JPRB, 0.40074E+00_JPRB, 0.40135E+00_JPRB, 0.40136E+00_JPRB, 0.40171E+00_JPRB ,&
&  0.39525E+00_JPRB, 0.39296E+00_JPRB, 0.39885E+00_JPRB, 0.39562E+00_JPRB, 0.39124E+00_JPRB, 0.38743E+00_JPRB, 0.38600E+00_JPRB ,&
&  0.37628E+00_JPRB, 0.36309E+00_JPRB, 0.35083E+00_JPRB, 0.35898E+00_JPRB, 0.37581E+00_JPRB, 0.37126E+00_JPRB, 0.36886E+00_JPRB ,&
&  0.36870E+00_JPRB, 0.36795E+00_JPRB, 0.36898E+00_JPRB, 0.36972E+00_JPRB, 0.37138E+00_JPRB, 0.37208E+00_JPRB, 0.37416E+00_JPRB ,&
&  0.37623E+00_JPRB, 0.38400E+00_JPRB, 0.38680E+00_JPRB, 0.36418E+00_JPRB, 0.34598E+00_JPRB, 0.33211E+00_JPRB, 0.31860E+00_JPRB ,&
&  0.30658E+00_JPRB, 0.30560E+00_JPRB, 0.30150E+00_JPRB, 0.29300E+00_JPRB, 0.29333E+00_JPRB, 0.29431E+00_JPRB, 0.29328E+00_JPRB ,&
&  0.29054E+00_JPRB, 0.28895E+00_JPRB, 0.28751E+00_JPRB, 0.28499E+00_JPRB, 0.28421E+00_JPRB, 0.28593E+00_JPRB, 0.29058E+00_JPRB ,&
&  0.29754E+00_JPRB, 0.30465E+00_JPRB, 0.32085E+00_JPRB, 0.33413E+00_JPRB, 0.32859E+00_JPRB, 0.32273E+00_JPRB, 0.31594E+00_JPRB ,&
&  0.30289E+00_JPRB, 0.29679E+00_JPRB, 0.29059E+00_JPRB, 0.28552E+00_JPRB, 0.28315E+00_JPRB, 0.28323E+00_JPRB, 0.28387E+00_JPRB ,&
&  0.28504E+00_JPRB, 0.28745E+00_JPRB, 0.28842E+00_JPRB, 0.28817E+00_JPRB, 0.28746E+00_JPRB, 0.28843E+00_JPRB, 0.29070E+00_JPRB ,&
&  0.29455E+00_JPRB, 0.30097E+00_JPRB, 0.30851E+00_JPRB, 0.31797E+00_JPRB, 0.32646E+00_JPRB, 0.33442E+00_JPRB, 0.34715E+00_JPRB ,&
&  0.35576E+00_JPRB, 0.37683E+00_JPRB, 0.40372E+00_JPRB, 0.42393E+00_JPRB, 0.41450E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.38777E+00_JPRB, 0.38565E+00_JPRB, 0.38294E+00_JPRB, 0.38253E+00_JPRB, 0.38345E+00_JPRB, 0.38569E+00_JPRB, 0.38282E+00_JPRB ,&
&  0.37770E+00_JPRB, 0.37555E+00_JPRB, 0.37672E+00_JPRB, 0.37580E+00_JPRB, 0.37584E+00_JPRB, 0.36637E+00_JPRB, 0.36102E+00_JPRB ,&
&  0.35392E+00_JPRB, 0.35219E+00_JPRB, 0.35065E+00_JPRB, 0.35182E+00_JPRB, 0.35373E+00_JPRB, 0.35482E+00_JPRB, 0.35018E+00_JPRB ,&
&  0.34553E+00_JPRB, 0.34140E+00_JPRB, 0.33676E+00_JPRB, 0.33476E+00_JPRB, 0.33606E+00_JPRB, 0.33713E+00_JPRB, 0.33692E+00_JPRB ,&
&  0.33769E+00_JPRB, 0.34084E+00_JPRB, 0.34402E+00_JPRB, 0.34809E+00_JPRB, 0.36092E+00_JPRB, 0.35784E+00_JPRB, 0.35655E+00_JPRB ,&
&  0.35724E+00_JPRB, 0.35811E+00_JPRB, 0.35978E+00_JPRB, 0.36118E+00_JPRB, 0.36662E+00_JPRB, 0.36915E+00_JPRB, 0.37131E+00_JPRB ,&
&  0.35816E+00_JPRB, 0.35204E+00_JPRB, 0.34491E+00_JPRB, 0.33516E+00_JPRB, 0.32154E+00_JPRB, 0.31101E+00_JPRB, 0.30144E+00_JPRB ,&
&  0.29346E+00_JPRB, 0.28600E+00_JPRB, 0.27885E+00_JPRB, 0.27613E+00_JPRB, 0.27506E+00_JPRB, 0.27335E+00_JPRB, 0.27306E+00_JPRB ,&
&  0.27366E+00_JPRB, 0.27455E+00_JPRB, 0.27506E+00_JPRB, 0.27684E+00_JPRB, 0.27978E+00_JPRB, 0.28447E+00_JPRB, 0.28945E+00_JPRB ,&
&  0.29674E+00_JPRB, 0.30164E+00_JPRB, 0.30759E+00_JPRB, 0.31324E+00_JPRB, 0.31175E+00_JPRB, 0.30593E+00_JPRB, 0.29994E+00_JPRB ,&
&  0.29195E+00_JPRB, 0.28743E+00_JPRB, 0.28272E+00_JPRB, 0.27930E+00_JPRB, 0.27731E+00_JPRB, 0.27603E+00_JPRB, 0.27610E+00_JPRB ,&
&  0.27594E+00_JPRB, 0.27631E+00_JPRB, 0.27562E+00_JPRB, 0.27591E+00_JPRB, 0.27693E+00_JPRB, 0.27919E+00_JPRB, 0.28219E+00_JPRB ,&
&  0.28685E+00_JPRB, 0.29223E+00_JPRB, 0.29851E+00_JPRB, 0.30530E+00_JPRB, 0.31570E+00_JPRB, 0.32862E+00_JPRB, 0.34625E+00_JPRB ,&
&  0.36206E+00_JPRB, 0.38766E+00_JPRB, 0.39730E+00_JPRB, 0.39368E+00_JPRB, 0.39179E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.37038E+00_JPRB, 0.37202E+00_JPRB, 0.37288E+00_JPRB, 0.37507E+00_JPRB, 0.37764E+00_JPRB, 0.37828E+00_JPRB, 0.37828E+00_JPRB ,&
&  0.36862E+00_JPRB, 0.36146E+00_JPRB, 0.35457E+00_JPRB, 0.34444E+00_JPRB, 0.33638E+00_JPRB, 0.32932E+00_JPRB, 0.32428E+00_JPRB ,&
&  0.31850E+00_JPRB, 0.31600E+00_JPRB, 0.31254E+00_JPRB, 0.31159E+00_JPRB, 0.31071E+00_JPRB, 0.31095E+00_JPRB, 0.30901E+00_JPRB ,&
&  0.30711E+00_JPRB, 0.30865E+00_JPRB, 0.30953E+00_JPRB, 0.31403E+00_JPRB, 0.32089E+00_JPRB, 0.32852E+00_JPRB, 0.33603E+00_JPRB ,&
&  0.34567E+00_JPRB, 0.34946E+00_JPRB, 0.35272E+00_JPRB, 0.35352E+00_JPRB, 0.35268E+00_JPRB, 0.35267E+00_JPRB, 0.35105E+00_JPRB ,&
&  0.35150E+00_JPRB, 0.35163E+00_JPRB, 0.35385E+00_JPRB, 0.35535E+00_JPRB, 0.35359E+00_JPRB, 0.35220E+00_JPRB, 0.34752E+00_JPRB ,&
&  0.33976E+00_JPRB, 0.33366E+00_JPRB, 0.32682E+00_JPRB, 0.32133E+00_JPRB, 0.31266E+00_JPRB, 0.30521E+00_JPRB, 0.29799E+00_JPRB ,&
&  0.29192E+00_JPRB, 0.28754E+00_JPRB, 0.28221E+00_JPRB, 0.27791E+00_JPRB, 0.27584E+00_JPRB, 0.27324E+00_JPRB, 0.27213E+00_JPRB ,&
&  0.27213E+00_JPRB, 0.27267E+00_JPRB, 0.27538E+00_JPRB, 0.27750E+00_JPRB, 0.28079E+00_JPRB, 0.28466E+00_JPRB, 0.28682E+00_JPRB ,&
&  0.29086E+00_JPRB, 0.29436E+00_JPRB, 0.29628E+00_JPRB, 0.29782E+00_JPRB, 0.29682E+00_JPRB, 0.29315E+00_JPRB, 0.28961E+00_JPRB ,&
&  0.28476E+00_JPRB, 0.28146E+00_JPRB, 0.27935E+00_JPRB, 0.27701E+00_JPRB, 0.27576E+00_JPRB, 0.27462E+00_JPRB, 0.27328E+00_JPRB ,&
&  0.27225E+00_JPRB, 0.27174E+00_JPRB, 0.27129E+00_JPRB, 0.27264E+00_JPRB, 0.27403E+00_JPRB, 0.27659E+00_JPRB, 0.27966E+00_JPRB ,&
&  0.28241E+00_JPRB, 0.28671E+00_JPRB, 0.29138E+00_JPRB, 0.29781E+00_JPRB, 0.31001E+00_JPRB, 0.31906E+00_JPRB, 0.33902E+00_JPRB ,&
&  0.34945E+00_JPRB, 0.36862E+00_JPRB, 0.37724E+00_JPRB, 0.36964E+00_JPRB, 0.36985E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.35866E+00_JPRB, 0.36106E+00_JPRB, 0.36155E+00_JPRB, 0.36341E+00_JPRB, 0.36488E+00_JPRB, 0.36521E+00_JPRB, 0.36441E+00_JPRB ,&
&  0.36238E+00_JPRB, 0.35731E+00_JPRB, 0.35294E+00_JPRB, 0.34615E+00_JPRB, 0.34010E+00_JPRB, 0.33319E+00_JPRB, 0.32727E+00_JPRB ,&
&  0.32062E+00_JPRB, 0.31450E+00_JPRB, 0.30810E+00_JPRB, 0.30262E+00_JPRB, 0.29844E+00_JPRB, 0.29496E+00_JPRB, 0.29161E+00_JPRB ,&
&  0.28874E+00_JPRB, 0.29043E+00_JPRB, 0.29359E+00_JPRB, 0.29933E+00_JPRB, 0.30810E+00_JPRB, 0.31575E+00_JPRB, 0.32418E+00_JPRB ,&
&  0.33795E+00_JPRB, 0.34020E+00_JPRB, 0.34046E+00_JPRB, 0.34309E+00_JPRB, 0.34286E+00_JPRB, 0.34355E+00_JPRB, 0.33417E+00_JPRB ,&
&  0.33863E+00_JPRB, 0.34104E+00_JPRB, 0.35013E+00_JPRB, 0.35286E+00_JPRB, 0.34671E+00_JPRB, 0.34285E+00_JPRB, 0.33506E+00_JPRB ,&
&  0.32959E+00_JPRB, 0.32491E+00_JPRB, 0.32179E+00_JPRB, 0.31999E+00_JPRB, 0.31404E+00_JPRB, 0.30963E+00_JPRB, 0.30223E+00_JPRB ,&
&  0.29636E+00_JPRB, 0.29271E+00_JPRB, 0.28808E+00_JPRB, 0.28536E+00_JPRB, 0.28309E+00_JPRB, 0.28077E+00_JPRB, 0.27899E+00_JPRB ,&
&  0.27801E+00_JPRB, 0.27719E+00_JPRB, 0.27803E+00_JPRB, 0.27896E+00_JPRB, 0.28076E+00_JPRB, 0.28221E+00_JPRB, 0.28241E+00_JPRB ,&
&  0.28267E+00_JPRB, 0.28324E+00_JPRB, 0.28351E+00_JPRB, 0.28443E+00_JPRB, 0.28384E+00_JPRB, 0.28263E+00_JPRB, 0.28118E+00_JPRB ,&
&  0.27949E+00_JPRB, 0.27874E+00_JPRB, 0.27745E+00_JPRB, 0.27660E+00_JPRB, 0.27419E+00_JPRB, 0.27259E+00_JPRB, 0.27176E+00_JPRB ,&
&  0.27102E+00_JPRB, 0.27163E+00_JPRB, 0.27184E+00_JPRB, 0.27294E+00_JPRB, 0.27458E+00_JPRB, 0.27595E+00_JPRB, 0.27845E+00_JPRB ,&
&  0.28153E+00_JPRB, 0.28507E+00_JPRB, 0.29136E+00_JPRB, 0.29764E+00_JPRB, 0.30456E+00_JPRB, 0.31212E+00_JPRB, 0.31750E+00_JPRB ,&
&  0.32568E+00_JPRB, 0.33868E+00_JPRB, 0.34059E+00_JPRB, 0.34818E+00_JPRB, 0.35459E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.34047E+00_JPRB, 0.34553E+00_JPRB, 0.34872E+00_JPRB, 0.35081E+00_JPRB, 0.35296E+00_JPRB, 0.35215E+00_JPRB, 0.35031E+00_JPRB ,&
&  0.34566E+00_JPRB, 0.34413E+00_JPRB, 0.34117E+00_JPRB, 0.33775E+00_JPRB, 0.33386E+00_JPRB, 0.32869E+00_JPRB, 0.32459E+00_JPRB ,&
&  0.31987E+00_JPRB, 0.31463E+00_JPRB, 0.30860E+00_JPRB, 0.30166E+00_JPRB, 0.29533E+00_JPRB, 0.28932E+00_JPRB, 0.28642E+00_JPRB ,&
&  0.28519E+00_JPRB, 0.28645E+00_JPRB, 0.29003E+00_JPRB, 0.29403E+00_JPRB, 0.29882E+00_JPRB, 0.30420E+00_JPRB, 0.31088E+00_JPRB ,&
&  0.31639E+00_JPRB, 0.32285E+00_JPRB, 0.32671E+00_JPRB, 0.32821E+00_JPRB, 0.32679E+00_JPRB, 0.32693E+00_JPRB, 0.32614E+00_JPRB ,&
&  0.32510E+00_JPRB, 0.32370E+00_JPRB, 0.32367E+00_JPRB, 0.32456E+00_JPRB, 0.32444E+00_JPRB, 0.32313E+00_JPRB, 0.32076E+00_JPRB ,&
&  0.31830E+00_JPRB, 0.31618E+00_JPRB, 0.31310E+00_JPRB, 0.31046E+00_JPRB, 0.30852E+00_JPRB, 0.30590E+00_JPRB, 0.30223E+00_JPRB ,&
&  0.29902E+00_JPRB, 0.29588E+00_JPRB, 0.29222E+00_JPRB, 0.28927E+00_JPRB, 0.28636E+00_JPRB, 0.28429E+00_JPRB, 0.28242E+00_JPRB ,&
&  0.28172E+00_JPRB, 0.28091E+00_JPRB, 0.28074E+00_JPRB, 0.28067E+00_JPRB, 0.28124E+00_JPRB, 0.28087E+00_JPRB, 0.28030E+00_JPRB ,&
&  0.27963E+00_JPRB, 0.27943E+00_JPRB, 0.27874E+00_JPRB, 0.27847E+00_JPRB, 0.27818E+00_JPRB, 0.27720E+00_JPRB, 0.27663E+00_JPRB ,&
&  0.27648E+00_JPRB, 0.27615E+00_JPRB, 0.27524E+00_JPRB, 0.27498E+00_JPRB, 0.27437E+00_JPRB, 0.27356E+00_JPRB, 0.27385E+00_JPRB ,&
&  0.27362E+00_JPRB, 0.27387E+00_JPRB, 0.27427E+00_JPRB, 0.27486E+00_JPRB, 0.27618E+00_JPRB, 0.27781E+00_JPRB, 0.27997E+00_JPRB ,&
&  0.28335E+00_JPRB, 0.28634E+00_JPRB, 0.28929E+00_JPRB, 0.29264E+00_JPRB, 0.29946E+00_JPRB, 0.30152E+00_JPRB, 0.30562E+00_JPRB ,&
&  0.30888E+00_JPRB, 0.30346E+00_JPRB, 0.32756E+00_JPRB, 0.32900E+00_JPRB, 0.33253E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.29503E+00_JPRB, 0.30026E+00_JPRB, 0.30356E+00_JPRB, 0.30578E+00_JPRB, 0.29912E+00_JPRB, 0.32012E+00_JPRB, 0.31847E+00_JPRB ,&
&  0.31938E+00_JPRB, 0.31545E+00_JPRB, 0.31794E+00_JPRB, 0.31726E+00_JPRB, 0.31512E+00_JPRB, 0.31215E+00_JPRB, 0.30908E+00_JPRB ,&
&  0.30582E+00_JPRB, 0.30182E+00_JPRB, 0.29827E+00_JPRB, 0.29476E+00_JPRB, 0.29186E+00_JPRB, 0.28925E+00_JPRB, 0.28792E+00_JPRB ,&
&  0.28811E+00_JPRB, 0.28991E+00_JPRB, 0.29295E+00_JPRB, 0.29428E+00_JPRB, 0.29577E+00_JPRB, 0.29736E+00_JPRB, 0.29912E+00_JPRB ,&
&  0.30041E+00_JPRB, 0.30235E+00_JPRB, 0.30331E+00_JPRB, 0.30275E+00_JPRB, 0.30206E+00_JPRB, 0.30222E+00_JPRB, 0.30268E+00_JPRB ,&
&  0.30385E+00_JPRB, 0.30467E+00_JPRB, 0.30524E+00_JPRB, 0.30591E+00_JPRB, 0.30656E+00_JPRB, 0.30630E+00_JPRB, 0.30510E+00_JPRB ,&
&  0.30407E+00_JPRB, 0.30322E+00_JPRB, 0.30123E+00_JPRB, 0.29935E+00_JPRB, 0.29758E+00_JPRB, 0.29572E+00_JPRB, 0.29375E+00_JPRB ,&
&  0.29237E+00_JPRB, 0.29098E+00_JPRB, 0.28922E+00_JPRB, 0.28720E+00_JPRB, 0.28537E+00_JPRB, 0.28377E+00_JPRB, 0.28255E+00_JPRB ,&
&  0.28240E+00_JPRB, 0.28175E+00_JPRB, 0.28118E+00_JPRB, 0.28062E+00_JPRB, 0.28051E+00_JPRB, 0.27995E+00_JPRB, 0.27951E+00_JPRB ,&
&  0.27916E+00_JPRB, 0.27871E+00_JPRB, 0.27818E+00_JPRB, 0.27764E+00_JPRB, 0.27726E+00_JPRB, 0.27679E+00_JPRB, 0.27629E+00_JPRB ,&
&  0.27595E+00_JPRB, 0.27568E+00_JPRB, 0.27539E+00_JPRB, 0.27484E+00_JPRB, 0.27428E+00_JPRB, 0.27363E+00_JPRB, 0.27357E+00_JPRB ,&
&  0.27370E+00_JPRB, 0.27398E+00_JPRB, 0.27451E+00_JPRB, 0.27464E+00_JPRB, 0.27568E+00_JPRB, 0.27667E+00_JPRB, 0.27786E+00_JPRB ,&
&  0.27908E+00_JPRB, 0.28101E+00_JPRB, 0.28314E+00_JPRB, 0.28550E+00_JPRB, 0.28938E+00_JPRB, 0.29031E+00_JPRB, 0.29100E+00_JPRB ,&
&  0.29191E+00_JPRB, 0.29617E+00_JPRB, 0.29659E+00_JPRB, 0.29572E+00_JPRB, 0.29580E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.28543E+00_JPRB, 0.28568E+00_JPRB, 0.28555E+00_JPRB, 0.28519E+00_JPRB, 0.28244E+00_JPRB, 0.28120E+00_JPRB, 0.28013E+00_JPRB ,&
&  0.27933E+00_JPRB, 0.27893E+00_JPRB, 0.27943E+00_JPRB, 0.28010E+00_JPRB, 0.28080E+00_JPRB, 0.28154E+00_JPRB, 0.28222E+00_JPRB ,&
&  0.28258E+00_JPRB, 0.28300E+00_JPRB, 0.28166E+00_JPRB, 0.28101E+00_JPRB, 0.28079E+00_JPRB, 0.28083E+00_JPRB, 0.28080E+00_JPRB ,&
&  0.28089E+00_JPRB, 0.28095E+00_JPRB, 0.28089E+00_JPRB, 0.28133E+00_JPRB, 0.28207E+00_JPRB, 0.28266E+00_JPRB, 0.28328E+00_JPRB ,&
&  0.28390E+00_JPRB, 0.28482E+00_JPRB, 0.28566E+00_JPRB, 0.28648E+00_JPRB, 0.28762E+00_JPRB, 0.28878E+00_JPRB, 0.28951E+00_JPRB ,&
&  0.29012E+00_JPRB, 0.29041E+00_JPRB, 0.29074E+00_JPRB, 0.29101E+00_JPRB, 0.29141E+00_JPRB, 0.29188E+00_JPRB, 0.29202E+00_JPRB ,&
&  0.29200E+00_JPRB, 0.29188E+00_JPRB, 0.29165E+00_JPRB, 0.29101E+00_JPRB, 0.29032E+00_JPRB, 0.28965E+00_JPRB, 0.28898E+00_JPRB ,&
&  0.28841E+00_JPRB, 0.28774E+00_JPRB, 0.28699E+00_JPRB, 0.28621E+00_JPRB, 0.28538E+00_JPRB, 0.28458E+00_JPRB, 0.28377E+00_JPRB ,&
&  0.28294E+00_JPRB, 0.28242E+00_JPRB, 0.28183E+00_JPRB, 0.28132E+00_JPRB, 0.28091E+00_JPRB, 0.28048E+00_JPRB, 0.28006E+00_JPRB ,&
&  0.27966E+00_JPRB, 0.27917E+00_JPRB, 0.27872E+00_JPRB, 0.27822E+00_JPRB, 0.27779E+00_JPRB, 0.27729E+00_JPRB, 0.27689E+00_JPRB ,&
&  0.27650E+00_JPRB, 0.27615E+00_JPRB, 0.27587E+00_JPRB, 0.27573E+00_JPRB, 0.27564E+00_JPRB, 0.27567E+00_JPRB, 0.27570E+00_JPRB ,&
&  0.27576E+00_JPRB, 0.27578E+00_JPRB, 0.27580E+00_JPRB, 0.27561E+00_JPRB, 0.27587E+00_JPRB, 0.27614E+00_JPRB, 0.27652E+00_JPRB ,&
&  0.27688E+00_JPRB, 0.27750E+00_JPRB, 0.27813E+00_JPRB, 0.27885E+00_JPRB, 0.28001E+00_JPRB, 0.28043E+00_JPRB, 0.28077E+00_JPRB ,&
&  0.28112E+00_JPRB, 0.28187E+00_JPRB, 0.28247E+00_JPRB, 0.28278E+00_JPRB, 0.28349E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB, 0.28031E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB, 0.79852E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.77500E+00_JPRB, 0.77151E+00_JPRB, 0.76788E+00_JPRB, 0.76407E+00_JPRB, 0.75995E+00_JPRB, 0.75524E+00_JPRB, 0.75003E+00_JPRB ,&
&  0.74532E+00_JPRB, 0.74098E+00_JPRB, 0.73708E+00_JPRB, 0.73357E+00_JPRB, 0.73057E+00_JPRB, 0.72824E+00_JPRB, 0.72738E+00_JPRB ,&
&  0.72734E+00_JPRB, 0.72744E+00_JPRB, 0.72781E+00_JPRB, 0.72813E+00_JPRB, 0.72806E+00_JPRB, 0.72778E+00_JPRB, 0.72735E+00_JPRB ,&
&  0.72714E+00_JPRB, 0.72704E+00_JPRB, 0.72701E+00_JPRB, 0.72741E+00_JPRB, 0.72794E+00_JPRB, 0.72850E+00_JPRB, 0.72897E+00_JPRB ,&
&  0.72996E+00_JPRB, 0.73119E+00_JPRB, 0.73290E+00_JPRB, 0.73505E+00_JPRB, 0.73944E+00_JPRB, 0.74414E+00_JPRB, 0.74901E+00_JPRB ,&
&  0.75359E+00_JPRB, 0.75817E+00_JPRB, 0.76213E+00_JPRB, 0.76561E+00_JPRB, 0.76884E+00_JPRB, 0.77288E+00_JPRB, 0.77746E+00_JPRB ,&
&  0.78243E+00_JPRB, 0.78787E+00_JPRB, 0.79412E+00_JPRB, 0.80031E+00_JPRB, 0.80595E+00_JPRB, 0.81169E+00_JPRB, 0.81780E+00_JPRB ,&
&  0.82373E+00_JPRB, 0.82964E+00_JPRB, 0.83552E+00_JPRB, 0.84161E+00_JPRB, 0.84752E+00_JPRB, 0.85444E+00_JPRB, 0.86258E+00_JPRB ,&
&  0.87174E+00_JPRB, 0.87894E+00_JPRB, 0.88720E+00_JPRB, 0.89607E+00_JPRB, 0.90915E+00_JPRB, 0.91557E+00_JPRB, 0.92170E+00_JPRB ,&
&  0.92712E+00_JPRB, 0.93589E+00_JPRB, 0.94135E+00_JPRB, 0.94684E+00_JPRB, 0.95203E+00_JPRB, 0.95659E+00_JPRB, 0.95968E+00_JPRB ,&
&  0.96238E+00_JPRB, 0.96481E+00_JPRB, 0.96528E+00_JPRB, 0.96476E+00_JPRB, 0.96297E+00_JPRB, 0.96072E+00_JPRB, 0.95514E+00_JPRB ,&
&  0.95000E+00_JPRB, 0.94506E+00_JPRB, 0.94051E+00_JPRB, 0.93400E+00_JPRB, 0.92733E+00_JPRB, 0.92046E+00_JPRB, 0.91339E+00_JPRB ,&
&  0.90431E+00_JPRB, 0.89501E+00_JPRB, 0.88506E+00_JPRB, 0.87442E+00_JPRB, 0.86407E+00_JPRB, 0.85325E+00_JPRB, 0.84146E+00_JPRB ,&
&  0.82836E+00_JPRB, 0.81604E+00_JPRB, 0.80227E+00_JPRB, 0.78860E+00_JPRB, 0.77881E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.75664E+00_JPRB, 0.74472E+00_JPRB, 0.73972E+00_JPRB, 0.73652E+00_JPRB, 0.73386E+00_JPRB, 0.73039E+00_JPRB, 0.72631E+00_JPRB ,&
&  0.72317E+00_JPRB, 0.71951E+00_JPRB, 0.71670E+00_JPRB, 0.71621E+00_JPRB, 0.71480E+00_JPRB, 0.71404E+00_JPRB, 0.71326E+00_JPRB ,&
&  0.71289E+00_JPRB, 0.71243E+00_JPRB, 0.71266E+00_JPRB, 0.71154E+00_JPRB, 0.70977E+00_JPRB, 0.70723E+00_JPRB, 0.70524E+00_JPRB ,&
&  0.70331E+00_JPRB, 0.70197E+00_JPRB, 0.69995E+00_JPRB, 0.69832E+00_JPRB, 0.69904E+00_JPRB, 0.69916E+00_JPRB, 0.69858E+00_JPRB ,&
&  0.70121E+00_JPRB, 0.70270E+00_JPRB, 0.70196E+00_JPRB, 0.69929E+00_JPRB, 0.69623E+00_JPRB, 0.69478E+00_JPRB, 0.69702E+00_JPRB ,&
&  0.70108E+00_JPRB, 0.71253E+00_JPRB, 0.72175E+00_JPRB, 0.73049E+00_JPRB, 0.73886E+00_JPRB, 0.74907E+00_JPRB, 0.76121E+00_JPRB ,&
&  0.77381E+00_JPRB, 0.78355E+00_JPRB, 0.79446E+00_JPRB, 0.80785E+00_JPRB, 0.82247E+00_JPRB, 0.83804E+00_JPRB, 0.85528E+00_JPRB ,&
&  0.86976E+00_JPRB, 0.88459E+00_JPRB, 0.90189E+00_JPRB, 0.92082E+00_JPRB, 0.93688E+00_JPRB, 0.95822E+00_JPRB, 0.98395E+00_JPRB ,&
&  0.10168E+01_JPRB, 0.10332E+01_JPRB, 0.10516E+01_JPRB, 0.10698E+01_JPRB, 0.10933E+01_JPRB, 0.11068E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.11348E+01_JPRB, 0.11320E+01_JPRB, 0.11270E+01_JPRB, 0.11236E+01_JPRB, 0.11190E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11092E+01_JPRB, 0.11023E+01_JPRB, 0.10952E+01_JPRB, 0.10856E+01_JPRB, 0.10759E+01_JPRB, 0.10573E+01_JPRB ,&
&  0.10404E+01_JPRB, 0.10274E+01_JPRB, 0.10189E+01_JPRB, 0.10073E+01_JPRB, 0.99599E+00_JPRB, 0.98560E+00_JPRB, 0.97565E+00_JPRB ,&
&  0.96568E+00_JPRB, 0.95401E+00_JPRB, 0.94152E+00_JPRB, 0.92804E+00_JPRB, 0.91693E+00_JPRB, 0.90116E+00_JPRB, 0.88322E+00_JPRB ,&
&  0.86220E+00_JPRB, 0.84391E+00_JPRB, 0.82231E+00_JPRB, 0.79686E+00_JPRB, 0.77191E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.77427E+00_JPRB, 0.74611E+00_JPRB, 0.73145E+00_JPRB, 0.72746E+00_JPRB, 0.72537E+00_JPRB, 0.72324E+00_JPRB, 0.72109E+00_JPRB ,&
&  0.71764E+00_JPRB, 0.71534E+00_JPRB, 0.71637E+00_JPRB, 0.71787E+00_JPRB, 0.71801E+00_JPRB, 0.71788E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.71436E+00_JPRB, 0.71058E+00_JPRB, 0.70681E+00_JPRB, 0.70019E+00_JPRB, 0.69318E+00_JPRB, 0.68438E+00_JPRB, 0.67227E+00_JPRB ,&
&  0.66251E+00_JPRB, 0.65383E+00_JPRB, 0.64246E+00_JPRB, 0.63917E+00_JPRB, 0.63818E+00_JPRB, 0.61267E+00_JPRB, 0.60777E+00_JPRB ,&
&  0.61745E+00_JPRB, 0.61946E+00_JPRB, 0.61995E+00_JPRB, 0.62024E+00_JPRB, 0.61526E+00_JPRB, 0.60319E+00_JPRB, 0.59924E+00_JPRB ,&
&  0.57991E+00_JPRB, 0.59802E+00_JPRB, 0.61044E+00_JPRB, 0.62102E+00_JPRB, 0.63142E+00_JPRB, 0.64594E+00_JPRB, 0.67562E+00_JPRB ,&
&  0.73104E+00_JPRB, 0.75486E+00_JPRB, 0.78216E+00_JPRB, 0.80839E+00_JPRB, 0.84261E+00_JPRB, 0.87843E+00_JPRB, 0.91164E+00_JPRB ,&
&  0.94064E+00_JPRB, 0.97186E+00_JPRB, 0.10081E+01_JPRB, 0.10549E+01_JPRB, 0.10928E+01_JPRB, 0.11275E+01_JPRB, 0.11524E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.12020E+01_JPRB, 0.12168E+01_JPRB, 0.12256E+01_JPRB, 0.12357E+01_JPRB, 0.12401E+01_JPRB, 0.12460E+01_JPRB ,&
&  0.12488E+01_JPRB, 0.12488E+01_JPRB, 0.12389E+01_JPRB, 0.12205E+01_JPRB, 0.12084E+01_JPRB, 0.12040E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.11869E+01_JPRB, 0.11761E+01_JPRB, 0.11625E+01_JPRB, 0.11456E+01_JPRB, 0.11240E+01_JPRB, 0.11052E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.10472E+01_JPRB, 0.10346E+01_JPRB, 0.10193E+01_JPRB, 0.10035E+01_JPRB, 0.98797E+00_JPRB, 0.97064E+00_JPRB ,&
&  0.95510E+00_JPRB, 0.94135E+00_JPRB, 0.93029E+00_JPRB, 0.91836E+00_JPRB, 0.91014E+00_JPRB, 0.89868E+00_JPRB, 0.88838E+00_JPRB ,&
&  0.87661E+00_JPRB, 0.86625E+00_JPRB, 0.84988E+00_JPRB, 0.82770E+00_JPRB, 0.79876E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.80902E+00_JPRB, 0.77806E+00_JPRB, 0.74567E+00_JPRB, 0.72760E+00_JPRB, 0.72350E+00_JPRB, 0.72009E+00_JPRB, 0.71742E+00_JPRB ,&
&  0.71606E+00_JPRB, 0.71882E+00_JPRB, 0.72175E+00_JPRB, 0.72347E+00_JPRB, 0.72269E+00_JPRB, 0.71993E+00_JPRB, 0.71551E+00_JPRB ,&
&  0.70914E+00_JPRB, 0.70110E+00_JPRB, 0.69431E+00_JPRB, 0.68740E+00_JPRB, 0.67977E+00_JPRB, 0.67166E+00_JPRB, 0.65918E+00_JPRB ,&
&  0.64693E+00_JPRB, 0.63556E+00_JPRB, 0.59701E+00_JPRB, 0.59587E+00_JPRB, 0.58909E+00_JPRB, 0.58769E+00_JPRB, 0.59187E+00_JPRB ,&
&  0.59025E+00_JPRB, 0.57915E+00_JPRB, 0.56630E+00_JPRB, 0.54851E+00_JPRB, 0.53559E+00_JPRB, 0.52163E+00_JPRB, 0.51543E+00_JPRB ,&
&  0.51397E+00_JPRB, 0.51836E+00_JPRB, 0.52925E+00_JPRB, 0.53917E+00_JPRB, 0.55027E+00_JPRB, 0.56075E+00_JPRB, 0.57995E+00_JPRB ,&
&  0.62862E+00_JPRB, 0.69558E+00_JPRB, 0.77701E+00_JPRB, 0.82058E+00_JPRB, 0.87290E+00_JPRB, 0.92486E+00_JPRB, 0.96448E+00_JPRB ,&
&  0.10017E+01_JPRB, 0.10409E+01_JPRB, 0.10827E+01_JPRB, 0.11286E+01_JPRB, 0.11727E+01_JPRB, 0.12084E+01_JPRB, 0.12537E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.13213E+01_JPRB, 0.13336E+01_JPRB, 0.13397E+01_JPRB, 0.13389E+01_JPRB, 0.13267E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.13045E+01_JPRB, 0.12930E+01_JPRB, 0.12772E+01_JPRB, 0.12419E+01_JPRB, 0.12241E+01_JPRB, 0.12188E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.11494E+01_JPRB, 0.11232E+01_JPRB, 0.10871E+01_JPRB, 0.10576E+01_JPRB, 0.10473E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10251E+01_JPRB, 0.10170E+01_JPRB, 0.10022E+01_JPRB, 0.97812E+00_JPRB, 0.95382E+00_JPRB, 0.93335E+00_JPRB ,&
&  0.91509E+00_JPRB, 0.89670E+00_JPRB, 0.88021E+00_JPRB, 0.86367E+00_JPRB, 0.85351E+00_JPRB, 0.84567E+00_JPRB, 0.84378E+00_JPRB ,&
&  0.84518E+00_JPRB, 0.84809E+00_JPRB, 0.84828E+00_JPRB, 0.84460E+00_JPRB, 0.83063E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.89046E+00_JPRB, 0.87770E+00_JPRB, 0.85117E+00_JPRB, 0.81512E+00_JPRB, 0.78706E+00_JPRB, 0.76737E+00_JPRB, 0.75764E+00_JPRB ,&
&  0.75155E+00_JPRB, 0.74876E+00_JPRB, 0.74554E+00_JPRB, 0.74201E+00_JPRB, 0.73602E+00_JPRB, 0.72861E+00_JPRB, 0.72082E+00_JPRB ,&
&  0.70791E+00_JPRB, 0.69569E+00_JPRB, 0.68326E+00_JPRB, 0.67543E+00_JPRB, 0.66893E+00_JPRB, 0.66381E+00_JPRB, 0.65909E+00_JPRB ,&
&  0.65387E+00_JPRB, 0.64711E+00_JPRB, 0.62589E+00_JPRB, 0.61813E+00_JPRB, 0.60964E+00_JPRB, 0.60946E+00_JPRB, 0.60564E+00_JPRB ,&
&  0.61753E+00_JPRB, 0.55925E+00_JPRB, 0.55207E+00_JPRB, 0.53626E+00_JPRB, 0.52503E+00_JPRB, 0.51189E+00_JPRB, 0.49810E+00_JPRB ,&
&  0.49064E+00_JPRB, 0.48233E+00_JPRB, 0.48116E+00_JPRB, 0.49160E+00_JPRB, 0.50035E+00_JPRB, 0.51673E+00_JPRB, 0.53426E+00_JPRB ,&
&  0.56764E+00_JPRB, 0.70467E+00_JPRB, 0.77269E+00_JPRB, 0.82261E+00_JPRB, 0.87329E+00_JPRB, 0.92109E+00_JPRB, 0.94809E+00_JPRB ,&
&  0.97736E+00_JPRB, 0.10148E+01_JPRB, 0.10458E+01_JPRB, 0.10586E+01_JPRB, 0.11400E+01_JPRB, 0.11991E+01_JPRB, 0.12612E+01_JPRB ,&
&  0.13177E+01_JPRB, 0.13706E+01_JPRB, 0.14154E+01_JPRB, 0.14490E+01_JPRB, 0.14766E+01_JPRB, 0.14750E+01_JPRB, 0.14550E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.14280E+01_JPRB, 0.14206E+01_JPRB, 0.13742E+01_JPRB, 0.13443E+01_JPRB, 0.13261E+01_JPRB, 0.12783E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.11799E+01_JPRB, 0.11332E+01_JPRB, 0.10845E+01_JPRB, 0.10405E+01_JPRB, 0.10158E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.99942E+00_JPRB, 0.98480E+00_JPRB, 0.96010E+00_JPRB, 0.93263E+00_JPRB, 0.91442E+00_JPRB, 0.89259E+00_JPRB ,&
&  0.87057E+00_JPRB, 0.84488E+00_JPRB, 0.82853E+00_JPRB, 0.81718E+00_JPRB, 0.81357E+00_JPRB, 0.81600E+00_JPRB, 0.82329E+00_JPRB ,&
&  0.83279E+00_JPRB, 0.84546E+00_JPRB, 0.86131E+00_JPRB, 0.87666E+00_JPRB, 0.88756E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.88217E+00_JPRB, 0.89069E+00_JPRB, 0.89240E+00_JPRB, 0.88531E+00_JPRB, 0.87095E+00_JPRB, 0.85168E+00_JPRB, 0.83215E+00_JPRB ,&
&  0.81401E+00_JPRB, 0.79974E+00_JPRB, 0.78602E+00_JPRB, 0.77528E+00_JPRB, 0.76243E+00_JPRB, 0.75348E+00_JPRB, 0.74315E+00_JPRB ,&
&  0.73679E+00_JPRB, 0.73380E+00_JPRB, 0.72522E+00_JPRB, 0.71530E+00_JPRB, 0.69806E+00_JPRB, 0.68279E+00_JPRB, 0.67083E+00_JPRB ,&
&  0.66003E+00_JPRB, 0.65075E+00_JPRB, 0.63029E+00_JPRB, 0.61564E+00_JPRB, 0.60778E+00_JPRB, 0.60039E+00_JPRB, 0.60605E+00_JPRB ,&
&  0.60393E+00_JPRB, 0.60857E+00_JPRB, 0.61272E+00_JPRB, 0.60588E+00_JPRB, 0.59782E+00_JPRB, 0.49675E+00_JPRB, 0.46981E+00_JPRB ,&
&  0.46263E+00_JPRB, 0.45586E+00_JPRB, 0.45546E+00_JPRB, 0.46273E+00_JPRB, 0.47424E+00_JPRB, 0.50476E+00_JPRB, 0.52658E+00_JPRB ,&
&  0.54307E+00_JPRB, 0.68698E+00_JPRB, 0.74634E+00_JPRB, 0.79897E+00_JPRB, 0.83820E+00_JPRB, 0.86988E+00_JPRB, 0.89042E+00_JPRB ,&
&  0.91061E+00_JPRB, 0.93914E+00_JPRB, 0.95558E+00_JPRB, 0.99055E+00_JPRB, 0.10581E+01_JPRB, 0.11725E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.13892E+01_JPRB, 0.14444E+01_JPRB, 0.14923E+01_JPRB, 0.15121E+01_JPRB, 0.15517E+01_JPRB, 0.15768E+01_JPRB, 0.15817E+01_JPRB ,&
&  0.15939E+01_JPRB, 0.15725E+01_JPRB, 0.15823E+01_JPRB, 0.15581E+01_JPRB, 0.15365E+01_JPRB, 0.15124E+01_JPRB, 0.14238E+01_JPRB ,&
&  0.13204E+01_JPRB, 0.12517E+01_JPRB, 0.11889E+01_JPRB, 0.11087E+01_JPRB, 0.10495E+01_JPRB, 0.10067E+01_JPRB, 0.99078E+00_JPRB ,&
&  0.98091E+00_JPRB, 0.98084E+00_JPRB, 0.95752E+00_JPRB, 0.92794E+00_JPRB, 0.90667E+00_JPRB, 0.89245E+00_JPRB, 0.86779E+00_JPRB ,&
&  0.84616E+00_JPRB, 0.82533E+00_JPRB, 0.81360E+00_JPRB, 0.80239E+00_JPRB, 0.79499E+00_JPRB, 0.79299E+00_JPRB, 0.79573E+00_JPRB ,&
&  0.80383E+00_JPRB, 0.81583E+00_JPRB, 0.83324E+00_JPRB, 0.85286E+00_JPRB, 0.87373E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.86615E+00_JPRB, 0.88136E+00_JPRB, 0.88879E+00_JPRB, 0.88599E+00_JPRB, 0.87107E+00_JPRB, 0.85138E+00_JPRB, 0.83391E+00_JPRB ,&
&  0.81716E+00_JPRB, 0.80144E+00_JPRB, 0.78657E+00_JPRB, 0.77519E+00_JPRB, 0.76508E+00_JPRB, 0.75778E+00_JPRB, 0.75221E+00_JPRB ,&
&  0.74856E+00_JPRB, 0.73015E+00_JPRB, 0.72271E+00_JPRB, 0.72170E+00_JPRB, 0.72279E+00_JPRB, 0.71976E+00_JPRB, 0.71319E+00_JPRB ,&
&  0.69131E+00_JPRB, 0.66137E+00_JPRB, 0.63327E+00_JPRB, 0.60900E+00_JPRB, 0.58940E+00_JPRB, 0.58107E+00_JPRB, 0.58508E+00_JPRB ,&
&  0.58926E+00_JPRB, 0.58333E+00_JPRB, 0.58502E+00_JPRB, 0.60407E+00_JPRB, 0.61081E+00_JPRB, 0.54911E+00_JPRB, 0.46950E+00_JPRB ,&
&  0.45952E+00_JPRB, 0.44262E+00_JPRB, 0.44251E+00_JPRB, 0.44991E+00_JPRB, 0.46390E+00_JPRB, 0.48493E+00_JPRB, 0.52042E+00_JPRB ,&
&  0.56596E+00_JPRB, 0.66921E+00_JPRB, 0.72575E+00_JPRB, 0.76807E+00_JPRB, 0.80586E+00_JPRB, 0.83843E+00_JPRB, 0.85451E+00_JPRB ,&
&  0.86423E+00_JPRB, 0.88491E+00_JPRB, 0.92273E+00_JPRB, 0.96743E+00_JPRB, 0.10456E+01_JPRB, 0.12066E+01_JPRB, 0.13437E+01_JPRB ,&
&  0.15311E+01_JPRB, 0.16279E+01_JPRB, 0.16122E+01_JPRB, 0.15362E+01_JPRB, 0.15660E+01_JPRB, 0.16417E+01_JPRB, 0.16591E+01_JPRB ,&
&  0.17253E+01_JPRB, 0.18098E+01_JPRB, 0.17976E+01_JPRB, 0.17203E+01_JPRB, 0.16719E+01_JPRB, 0.16461E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.13861E+01_JPRB, 0.13376E+01_JPRB, 0.13486E+01_JPRB, 0.12133E+01_JPRB, 0.11208E+01_JPRB, 0.10335E+01_JPRB, 0.98408E+00_JPRB ,&
&  0.95242E+00_JPRB, 0.92667E+00_JPRB, 0.89232E+00_JPRB, 0.86508E+00_JPRB, 0.84999E+00_JPRB, 0.84019E+00_JPRB, 0.82894E+00_JPRB ,&
&  0.81684E+00_JPRB, 0.80615E+00_JPRB, 0.79557E+00_JPRB, 0.78800E+00_JPRB, 0.78001E+00_JPRB, 0.77619E+00_JPRB, 0.77607E+00_JPRB ,&
&  0.78120E+00_JPRB, 0.79221E+00_JPRB, 0.80856E+00_JPRB, 0.82678E+00_JPRB, 0.84891E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.87256E+00_JPRB, 0.88369E+00_JPRB, 0.87508E+00_JPRB, 0.84958E+00_JPRB, 0.81372E+00_JPRB, 0.76974E+00_JPRB, 0.73878E+00_JPRB ,&
&  0.72156E+00_JPRB, 0.71411E+00_JPRB, 0.70816E+00_JPRB, 0.70070E+00_JPRB, 0.69780E+00_JPRB, 0.69712E+00_JPRB, 0.69850E+00_JPRB ,&
&  0.70207E+00_JPRB, 0.70600E+00_JPRB, 0.70680E+00_JPRB, 0.69965E+00_JPRB, 0.69949E+00_JPRB, 0.70088E+00_JPRB, 0.70375E+00_JPRB ,&
&  0.69706E+00_JPRB, 0.67270E+00_JPRB, 0.64246E+00_JPRB, 0.62021E+00_JPRB, 0.59800E+00_JPRB, 0.59125E+00_JPRB, 0.57889E+00_JPRB ,&
&  0.55610E+00_JPRB, 0.55108E+00_JPRB, 0.57229E+00_JPRB, 0.60293E+00_JPRB, 0.59866E+00_JPRB, 0.61795E+00_JPRB, 0.50377E+00_JPRB ,&
&  0.46782E+00_JPRB, 0.44870E+00_JPRB, 0.44470E+00_JPRB, 0.45641E+00_JPRB, 0.48732E+00_JPRB, 0.52518E+00_JPRB, 0.55919E+00_JPRB ,&
&  0.60320E+00_JPRB, 0.66415E+00_JPRB, 0.69856E+00_JPRB, 0.72986E+00_JPRB, 0.76769E+00_JPRB, 0.80662E+00_JPRB, 0.83538E+00_JPRB ,&
&  0.85442E+00_JPRB, 0.88794E+00_JPRB, 0.94577E+00_JPRB, 0.10370E+01_JPRB, 0.12364E+01_JPRB, 0.12224E+01_JPRB, 0.12751E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.18678E+01_JPRB, 0.17330E+01_JPRB, 0.16137E+01_JPRB, 0.15746E+01_JPRB, 0.16200E+01_JPRB, 0.17030E+01_JPRB ,&
&  0.18707E+01_JPRB, 0.20237E+01_JPRB, 0.20298E+01_JPRB, 0.18487E+01_JPRB, 0.17576E+01_JPRB, 0.17092E+01_JPRB, 0.15434E+01_JPRB ,&
&  0.13623E+01_JPRB, 0.13976E+01_JPRB, 0.14684E+01_JPRB, 0.12445E+01_JPRB, 0.10906E+01_JPRB, 0.99253E+00_JPRB, 0.95101E+00_JPRB ,&
&  0.92637E+00_JPRB, 0.90469E+00_JPRB, 0.87850E+00_JPRB, 0.85324E+00_JPRB, 0.82163E+00_JPRB, 0.79435E+00_JPRB, 0.77097E+00_JPRB ,&
&  0.75517E+00_JPRB, 0.74419E+00_JPRB, 0.74265E+00_JPRB, 0.74607E+00_JPRB, 0.74771E+00_JPRB, 0.74953E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.77303E+00_JPRB, 0.77993E+00_JPRB, 0.79462E+00_JPRB, 0.81504E+00_JPRB, 0.84386E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.82310E+00_JPRB, 0.82479E+00_JPRB, 0.80973E+00_JPRB, 0.78022E+00_JPRB, 0.74541E+00_JPRB, 0.69929E+00_JPRB, 0.66353E+00_JPRB ,&
&  0.64295E+00_JPRB, 0.63175E+00_JPRB, 0.62597E+00_JPRB, 0.62357E+00_JPRB, 0.62496E+00_JPRB, 0.63025E+00_JPRB, 0.63067E+00_JPRB ,&
&  0.63281E+00_JPRB, 0.63466E+00_JPRB, 0.63888E+00_JPRB, 0.64586E+00_JPRB, 0.65121E+00_JPRB, 0.65590E+00_JPRB, 0.66102E+00_JPRB ,&
&  0.66579E+00_JPRB, 0.65754E+00_JPRB, 0.63402E+00_JPRB, 0.61598E+00_JPRB, 0.61844E+00_JPRB, 0.63507E+00_JPRB, 0.61824E+00_JPRB ,&
&  0.57518E+00_JPRB, 0.55288E+00_JPRB, 0.55254E+00_JPRB, 0.60998E+00_JPRB, 0.64874E+00_JPRB, 0.66405E+00_JPRB, 0.54936E+00_JPRB ,&
&  0.48859E+00_JPRB, 0.46556E+00_JPRB, 0.46934E+00_JPRB, 0.49984E+00_JPRB, 0.55149E+00_JPRB, 0.60964E+00_JPRB, 0.67578E+00_JPRB ,&
&  0.72286E+00_JPRB, 0.76251E+00_JPRB, 0.77379E+00_JPRB, 0.79705E+00_JPRB, 0.83612E+00_JPRB, 0.87200E+00_JPRB, 0.91130E+00_JPRB ,&
&  0.93705E+00_JPRB, 0.97801E+00_JPRB, 0.10951E+01_JPRB, 0.97565E+00_JPRB, 0.10316E+01_JPRB, 0.11006E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.14865E+01_JPRB, 0.15775E+01_JPRB, 0.15961E+01_JPRB, 0.17145E+01_JPRB, 0.16276E+01_JPRB, 0.15715E+01_JPRB, 0.16177E+01_JPRB ,&
&  0.18831E+01_JPRB, 0.21314E+01_JPRB, 0.21002E+01_JPRB, 0.19248E+01_JPRB, 0.18651E+01_JPRB, 0.18326E+01_JPRB, 0.16323E+01_JPRB ,&
&  0.13799E+01_JPRB, 0.12399E+01_JPRB, 0.11198E+01_JPRB, 0.97569E+00_JPRB, 0.91855E+00_JPRB, 0.88537E+00_JPRB, 0.86559E+00_JPRB ,&
&  0.85250E+00_JPRB, 0.83422E+00_JPRB, 0.82032E+00_JPRB, 0.76947E+00_JPRB, 0.72270E+00_JPRB, 0.67581E+00_JPRB, 0.64558E+00_JPRB ,&
&  0.62753E+00_JPRB, 0.62697E+00_JPRB, 0.63807E+00_JPRB, 0.66069E+00_JPRB, 0.70282E+00_JPRB, 0.74121E+00_JPRB, 0.76729E+00_JPRB ,&
&  0.77517E+00_JPRB, 0.76033E+00_JPRB, 0.75735E+00_JPRB, 0.76649E+00_JPRB, 0.80137E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.78885E+00_JPRB, 0.75262E+00_JPRB, 0.73020E+00_JPRB, 0.73428E+00_JPRB, 0.69139E+00_JPRB, 0.64823E+00_JPRB, 0.62335E+00_JPRB ,&
&  0.62350E+00_JPRB, 0.61944E+00_JPRB, 0.61965E+00_JPRB, 0.59592E+00_JPRB, 0.57472E+00_JPRB, 0.56337E+00_JPRB, 0.55890E+00_JPRB ,&
&  0.56926E+00_JPRB, 0.58265E+00_JPRB, 0.59377E+00_JPRB, 0.59334E+00_JPRB, 0.59168E+00_JPRB, 0.59495E+00_JPRB, 0.60496E+00_JPRB ,&
&  0.61822E+00_JPRB, 0.62698E+00_JPRB, 0.62264E+00_JPRB, 0.61749E+00_JPRB, 0.63869E+00_JPRB, 0.66418E+00_JPRB, 0.63751E+00_JPRB ,&
&  0.60577E+00_JPRB, 0.57994E+00_JPRB, 0.57442E+00_JPRB, 0.61634E+00_JPRB, 0.68479E+00_JPRB, 0.71243E+00_JPRB, 0.59719E+00_JPRB ,&
&  0.51781E+00_JPRB, 0.50657E+00_JPRB, 0.55064E+00_JPRB, 0.66783E+00_JPRB, 0.71433E+00_JPRB, 0.76690E+00_JPRB, 0.83575E+00_JPRB ,&
&  0.78818E+00_JPRB, 0.79229E+00_JPRB, 0.81910E+00_JPRB, 0.95879E+00_JPRB, 0.99632E+00_JPRB, 0.10125E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10718E+01_JPRB, 0.12133E+01_JPRB, 0.10509E+01_JPRB, 0.11127E+01_JPRB, 0.13386E+01_JPRB, 0.13549E+01_JPRB ,&
&  0.14710E+01_JPRB, 0.15239E+01_JPRB, 0.17137E+01_JPRB, 0.16915E+01_JPRB, 0.15785E+01_JPRB, 0.15529E+01_JPRB, 0.16902E+01_JPRB ,&
&  0.20267E+01_JPRB, 0.23160E+01_JPRB, 0.22681E+01_JPRB, 0.21865E+01_JPRB, 0.21366E+01_JPRB, 0.20427E+01_JPRB, 0.18168E+01_JPRB ,&
&  0.14919E+01_JPRB, 0.12483E+01_JPRB, 0.10688E+01_JPRB, 0.91431E+00_JPRB, 0.84139E+00_JPRB, 0.81386E+00_JPRB, 0.78461E+00_JPRB ,&
&  0.77561E+00_JPRB, 0.76483E+00_JPRB, 0.75350E+00_JPRB, 0.70010E+00_JPRB, 0.65494E+00_JPRB, 0.62416E+00_JPRB, 0.60155E+00_JPRB ,&
&  0.59343E+00_JPRB, 0.58057E+00_JPRB, 0.57198E+00_JPRB, 0.59040E+00_JPRB, 0.64577E+00_JPRB, 0.71256E+00_JPRB, 0.74410E+00_JPRB ,&
&  0.73257E+00_JPRB, 0.71508E+00_JPRB, 0.73951E+00_JPRB, 0.78269E+00_JPRB, 0.80334E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.80664E+00_JPRB, 0.79617E+00_JPRB, 0.78896E+00_JPRB, 0.77724E+00_JPRB, 0.75623E+00_JPRB, 0.70220E+00_JPRB, 0.66623E+00_JPRB ,&
&  0.67451E+00_JPRB, 0.64841E+00_JPRB, 0.63860E+00_JPRB, 0.60641E+00_JPRB, 0.57895E+00_JPRB, 0.55054E+00_JPRB, 0.53622E+00_JPRB ,&
&  0.52410E+00_JPRB, 0.53467E+00_JPRB, 0.55945E+00_JPRB, 0.56729E+00_JPRB, 0.56847E+00_JPRB, 0.56841E+00_JPRB, 0.57916E+00_JPRB ,&
&  0.59254E+00_JPRB, 0.61062E+00_JPRB, 0.62204E+00_JPRB, 0.63070E+00_JPRB, 0.64946E+00_JPRB, 0.67495E+00_JPRB, 0.67341E+00_JPRB ,&
&  0.66844E+00_JPRB, 0.64496E+00_JPRB, 0.62465E+00_JPRB, 0.65237E+00_JPRB, 0.71640E+00_JPRB, 0.77756E+00_JPRB, 0.74775E+00_JPRB ,&
&  0.58957E+00_JPRB, 0.59629E+00_JPRB, 0.76379E+00_JPRB, 0.82292E+00_JPRB, 0.88113E+00_JPRB, 0.91864E+00_JPRB, 0.96686E+00_JPRB ,&
&  0.10099E+01_JPRB, 0.10535E+01_JPRB, 0.10849E+01_JPRB, 0.11103E+01_JPRB, 0.11276E+01_JPRB, 0.11482E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.10061E+01_JPRB, 0.96548E+00_JPRB, 0.10315E+01_JPRB, 0.13341E+01_JPRB, 0.14144E+01_JPRB, 0.14007E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.15077E+01_JPRB, 0.16029E+01_JPRB, 0.17467E+01_JPRB, 0.17967E+01_JPRB, 0.19034E+01_JPRB, 0.20612E+01_JPRB ,&
&  0.23189E+01_JPRB, 0.26105E+01_JPRB, 0.26449E+01_JPRB, 0.26057E+01_JPRB, 0.24842E+01_JPRB, 0.22772E+01_JPRB, 0.19445E+01_JPRB ,&
&  0.15364E+01_JPRB, 0.12559E+01_JPRB, 0.10315E+01_JPRB, 0.87620E+00_JPRB, 0.78797E+00_JPRB, 0.74806E+00_JPRB, 0.71796E+00_JPRB ,&
&  0.71419E+00_JPRB, 0.70739E+00_JPRB, 0.69276E+00_JPRB, 0.65234E+00_JPRB, 0.62720E+00_JPRB, 0.63777E+00_JPRB, 0.64848E+00_JPRB ,&
&  0.65437E+00_JPRB, 0.64464E+00_JPRB, 0.60950E+00_JPRB, 0.62488E+00_JPRB, 0.67877E+00_JPRB, 0.73450E+00_JPRB, 0.75768E+00_JPRB ,&
&  0.75493E+00_JPRB, 0.74096E+00_JPRB, 0.75420E+00_JPRB, 0.78437E+00_JPRB, 0.79602E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.83577E+00_JPRB, 0.83978E+00_JPRB, 0.82353E+00_JPRB, 0.80970E+00_JPRB, 0.79035E+00_JPRB, 0.72357E+00_JPRB, 0.70504E+00_JPRB ,&
&  0.77279E+00_JPRB, 0.71532E+00_JPRB, 0.76708E+00_JPRB, 0.66164E+00_JPRB, 0.63556E+00_JPRB, 0.60460E+00_JPRB, 0.57948E+00_JPRB ,&
&  0.54743E+00_JPRB, 0.53506E+00_JPRB, 0.54647E+00_JPRB, 0.55977E+00_JPRB, 0.56061E+00_JPRB, 0.55698E+00_JPRB, 0.56081E+00_JPRB ,&
&  0.56881E+00_JPRB, 0.59364E+00_JPRB, 0.62693E+00_JPRB, 0.64384E+00_JPRB, 0.65960E+00_JPRB, 0.68722E+00_JPRB, 0.70141E+00_JPRB ,&
&  0.68169E+00_JPRB, 0.68162E+00_JPRB, 0.68473E+00_JPRB, 0.70443E+00_JPRB, 0.74560E+00_JPRB, 0.80773E+00_JPRB, 0.84888E+00_JPRB ,&
&  0.86418E+00_JPRB, 0.75358E+00_JPRB, 0.86426E+00_JPRB, 0.89337E+00_JPRB, 0.95495E+00_JPRB, 0.99038E+00_JPRB, 0.10155E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10888E+01_JPRB, 0.11320E+01_JPRB, 0.11710E+01_JPRB, 0.12105E+01_JPRB, 0.12467E+01_JPRB, 0.12476E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.99012E+00_JPRB, 0.10519E+01_JPRB, 0.11025E+01_JPRB, 0.14569E+01_JPRB, 0.16302E+01_JPRB, 0.16037E+01_JPRB ,&
&  0.18979E+01_JPRB, 0.18284E+01_JPRB, 0.19898E+01_JPRB, 0.21405E+01_JPRB, 0.23183E+01_JPRB, 0.25429E+01_JPRB, 0.26780E+01_JPRB ,&
&  0.27415E+01_JPRB, 0.28766E+01_JPRB, 0.30456E+01_JPRB, 0.28565E+01_JPRB, 0.25475E+01_JPRB, 0.21919E+01_JPRB, 0.17750E+01_JPRB ,&
&  0.14902E+01_JPRB, 0.12466E+01_JPRB, 0.10428E+01_JPRB, 0.90409E+00_JPRB, 0.79683E+00_JPRB, 0.72660E+00_JPRB, 0.68795E+00_JPRB ,&
&  0.66667E+00_JPRB, 0.64435E+00_JPRB, 0.63374E+00_JPRB, 0.61401E+00_JPRB, 0.60906E+00_JPRB, 0.62409E+00_JPRB, 0.66371E+00_JPRB ,&
&  0.67752E+00_JPRB, 0.68489E+00_JPRB, 0.73360E+00_JPRB, 0.81296E+00_JPRB, 0.82711E+00_JPRB, 0.84758E+00_JPRB, 0.93093E+00_JPRB ,&
&  0.82819E+00_JPRB, 0.83644E+00_JPRB, 0.88374E+00_JPRB, 0.86989E+00_JPRB, 0.83996E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.90099E+00_JPRB, 0.89569E+00_JPRB, 0.86454E+00_JPRB, 0.84063E+00_JPRB, 0.83000E+00_JPRB, 0.79414E+00_JPRB, 0.76120E+00_JPRB ,&
&  0.81635E+00_JPRB, 0.80588E+00_JPRB, 0.80644E+00_JPRB, 0.79971E+00_JPRB, 0.78776E+00_JPRB, 0.71946E+00_JPRB, 0.66451E+00_JPRB ,&
&  0.61639E+00_JPRB, 0.58877E+00_JPRB, 0.59260E+00_JPRB, 0.59142E+00_JPRB, 0.58733E+00_JPRB, 0.57517E+00_JPRB, 0.56908E+00_JPRB ,&
&  0.56505E+00_JPRB, 0.58044E+00_JPRB, 0.61588E+00_JPRB, 0.64579E+00_JPRB, 0.65565E+00_JPRB, 0.66861E+00_JPRB, 0.69403E+00_JPRB ,&
&  0.68066E+00_JPRB, 0.68885E+00_JPRB, 0.70323E+00_JPRB, 0.67328E+00_JPRB, 0.76716E+00_JPRB, 0.81243E+00_JPRB, 0.83800E+00_JPRB ,&
&  0.88652E+00_JPRB, 0.92832E+00_JPRB, 0.94782E+00_JPRB, 0.95385E+00_JPRB, 0.97393E+00_JPRB, 0.98580E+00_JPRB, 0.10098E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10692E+01_JPRB, 0.11187E+01_JPRB, 0.11784E+01_JPRB, 0.12945E+01_JPRB, 0.12030E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.13651E+01_JPRB, 0.13940E+01_JPRB, 0.14688E+01_JPRB, 0.13408E+01_JPRB, 0.16884E+01_JPRB, 0.19098E+01_JPRB, 0.18104E+01_JPRB ,&
&  0.18990E+01_JPRB, 0.18934E+01_JPRB, 0.23244E+01_JPRB, 0.26603E+01_JPRB, 0.29359E+01_JPRB, 0.31818E+01_JPRB, 0.32853E+01_JPRB ,&
&  0.32824E+01_JPRB, 0.35081E+01_JPRB, 0.34784E+01_JPRB, 0.31540E+01_JPRB, 0.27003E+01_JPRB, 0.23299E+01_JPRB, 0.18765E+01_JPRB ,&
&  0.15051E+01_JPRB, 0.12091E+01_JPRB, 0.10293E+01_JPRB, 0.94081E+00_JPRB, 0.79271E+00_JPRB, 0.74176E+00_JPRB, 0.68772E+00_JPRB ,&
&  0.63656E+00_JPRB, 0.60242E+00_JPRB, 0.60273E+00_JPRB, 0.60917E+00_JPRB, 0.60667E+00_JPRB, 0.62510E+00_JPRB, 0.64629E+00_JPRB ,&
&  0.71309E+00_JPRB, 0.84356E+00_JPRB, 0.10932E+01_JPRB, 0.11144E+01_JPRB, 0.10950E+01_JPRB, 0.10758E+01_JPRB, 0.95901E+00_JPRB ,&
&  0.89269E+00_JPRB, 0.98639E+00_JPRB, 0.95278E+00_JPRB, 0.91797E+00_JPRB, 0.89263E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.95371E+00_JPRB, 0.93707E+00_JPRB, 0.91502E+00_JPRB, 0.89062E+00_JPRB, 0.86843E+00_JPRB, 0.84401E+00_JPRB, 0.82966E+00_JPRB ,&
&  0.83625E+00_JPRB, 0.83138E+00_JPRB, 0.83062E+00_JPRB, 0.82454E+00_JPRB, 0.80847E+00_JPRB, 0.82059E+00_JPRB, 0.74378E+00_JPRB ,&
&  0.66686E+00_JPRB, 0.63301E+00_JPRB, 0.60975E+00_JPRB, 0.61226E+00_JPRB, 0.62752E+00_JPRB, 0.62046E+00_JPRB, 0.60722E+00_JPRB ,&
&  0.59256E+00_JPRB, 0.58748E+00_JPRB, 0.60020E+00_JPRB, 0.62381E+00_JPRB, 0.63977E+00_JPRB, 0.65447E+00_JPRB, 0.67219E+00_JPRB ,&
&  0.65550E+00_JPRB, 0.63499E+00_JPRB, 0.62975E+00_JPRB, 0.64085E+00_JPRB, 0.71076E+00_JPRB, 0.80638E+00_JPRB, 0.86050E+00_JPRB ,&
&  0.88226E+00_JPRB, 0.89909E+00_JPRB, 0.93612E+00_JPRB, 0.95092E+00_JPRB, 0.95863E+00_JPRB, 0.97938E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10696E+01_JPRB, 0.11212E+01_JPRB, 0.12058E+01_JPRB, 0.12246E+01_JPRB, 0.12203E+01_JPRB, 0.13389E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.15627E+01_JPRB, 0.16006E+01_JPRB, 0.17230E+01_JPRB, 0.19638E+01_JPRB, 0.19831E+01_JPRB, 0.19944E+01_JPRB ,&
&  0.21746E+01_JPRB, 0.22716E+01_JPRB, 0.29084E+01_JPRB, 0.31776E+01_JPRB, 0.32968E+01_JPRB, 0.35412E+01_JPRB, 0.35723E+01_JPRB ,&
&  0.36524E+01_JPRB, 0.36876E+01_JPRB, 0.36469E+01_JPRB, 0.33262E+01_JPRB, 0.28146E+01_JPRB, 0.24075E+01_JPRB, 0.18659E+01_JPRB ,&
&  0.15191E+01_JPRB, 0.12248E+01_JPRB, 0.94317E+00_JPRB, 0.82786E+00_JPRB, 0.72420E+00_JPRB, 0.70115E+00_JPRB, 0.66431E+00_JPRB ,&
&  0.61920E+00_JPRB, 0.61328E+00_JPRB, 0.62299E+00_JPRB, 0.67801E+00_JPRB, 0.69421E+00_JPRB, 0.66053E+00_JPRB, 0.71601E+00_JPRB ,&
&  0.82598E+00_JPRB, 0.11300E+01_JPRB, 0.12058E+01_JPRB, 0.13131E+01_JPRB, 0.12715E+01_JPRB, 0.12479E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.11093E+01_JPRB, 0.10313E+01_JPRB, 0.98612E+00_JPRB, 0.96761E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.99036E+00_JPRB, 0.96519E+00_JPRB, 0.95132E+00_JPRB, 0.94417E+00_JPRB, 0.93054E+00_JPRB, 0.89737E+00_JPRB, 0.89538E+00_JPRB ,&
&  0.88835E+00_JPRB, 0.87204E+00_JPRB, 0.86663E+00_JPRB, 0.86253E+00_JPRB, 0.84875E+00_JPRB, 0.85292E+00_JPRB, 0.87451E+00_JPRB ,&
&  0.77107E+00_JPRB, 0.72474E+00_JPRB, 0.65556E+00_JPRB, 0.63427E+00_JPRB, 0.68859E+00_JPRB, 0.66783E+00_JPRB, 0.65688E+00_JPRB ,&
&  0.63054E+00_JPRB, 0.61304E+00_JPRB, 0.59469E+00_JPRB, 0.60244E+00_JPRB, 0.62213E+00_JPRB, 0.64186E+00_JPRB, 0.65330E+00_JPRB ,&
&  0.62768E+00_JPRB, 0.60124E+00_JPRB, 0.59912E+00_JPRB, 0.61515E+00_JPRB, 0.66727E+00_JPRB, 0.74763E+00_JPRB, 0.84090E+00_JPRB ,&
&  0.88154E+00_JPRB, 0.89495E+00_JPRB, 0.91822E+00_JPRB, 0.94482E+00_JPRB, 0.97856E+00_JPRB, 0.10302E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.11094E+01_JPRB, 0.11575E+01_JPRB, 0.12299E+01_JPRB, 0.11929E+01_JPRB, 0.12071E+01_JPRB, 0.13710E+01_JPRB ,&
&  0.15530E+01_JPRB, 0.15338E+01_JPRB, 0.16125E+01_JPRB, 0.22423E+01_JPRB, 0.26370E+01_JPRB, 0.27343E+01_JPRB, 0.26765E+01_JPRB ,&
&  0.27793E+01_JPRB, 0.29378E+01_JPRB, 0.31468E+01_JPRB, 0.34086E+01_JPRB, 0.35780E+01_JPRB, 0.37661E+01_JPRB, 0.37782E+01_JPRB ,&
&  0.37878E+01_JPRB, 0.37986E+01_JPRB, 0.36759E+01_JPRB, 0.33785E+01_JPRB, 0.27610E+01_JPRB, 0.22874E+01_JPRB, 0.17687E+01_JPRB ,&
&  0.14795E+01_JPRB, 0.11363E+01_JPRB, 0.80280E+00_JPRB, 0.67773E+00_JPRB, 0.63251E+00_JPRB, 0.68237E+00_JPRB, 0.87794E+00_JPRB ,&
&  0.67893E+00_JPRB, 0.71133E+00_JPRB, 0.75994E+00_JPRB, 0.86511E+00_JPRB, 0.95056E+00_JPRB, 0.96464E+00_JPRB, 0.84508E+00_JPRB ,&
&  0.86008E+00_JPRB, 0.10873E+01_JPRB, 0.14141E+01_JPRB, 0.15400E+01_JPRB, 0.14487E+01_JPRB, 0.14314E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.13858E+01_JPRB, 0.12363E+01_JPRB, 0.11183E+01_JPRB, 0.10439E+01_JPRB, 0.10259E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.99938E+00_JPRB, 0.99450E+00_JPRB, 0.99872E+00_JPRB, 0.10066E+01_JPRB, 0.10097E+01_JPRB, 0.96497E+00_JPRB, 0.95497E+00_JPRB ,&
&  0.94157E+00_JPRB, 0.92375E+00_JPRB, 0.90728E+00_JPRB, 0.89945E+00_JPRB, 0.88837E+00_JPRB, 0.88004E+00_JPRB, 0.87901E+00_JPRB ,&
&  0.90051E+00_JPRB, 0.83355E+00_JPRB, 0.74822E+00_JPRB, 0.65585E+00_JPRB, 0.67219E+00_JPRB, 0.66411E+00_JPRB, 0.66144E+00_JPRB ,&
&  0.64474E+00_JPRB, 0.64048E+00_JPRB, 0.61906E+00_JPRB, 0.60607E+00_JPRB, 0.62132E+00_JPRB, 0.64435E+00_JPRB, 0.65497E+00_JPRB ,&
&  0.64042E+00_JPRB, 0.61452E+00_JPRB, 0.60968E+00_JPRB, 0.63840E+00_JPRB, 0.72430E+00_JPRB, 0.73098E+00_JPRB, 0.83083E+00_JPRB ,&
&  0.88502E+00_JPRB, 0.91966E+00_JPRB, 0.95115E+00_JPRB, 0.10009E+01_JPRB, 0.10571E+01_JPRB, 0.11296E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.12001E+01_JPRB, 0.11674E+01_JPRB, 0.12351E+01_JPRB, 0.13021E+01_JPRB, 0.13517E+01_JPRB, 0.14206E+01_JPRB ,&
&  0.15115E+01_JPRB, 0.15672E+01_JPRB, 0.18144E+01_JPRB, 0.25497E+01_JPRB, 0.29880E+01_JPRB, 0.30285E+01_JPRB, 0.32211E+01_JPRB ,&
&  0.34663E+01_JPRB, 0.37046E+01_JPRB, 0.39491E+01_JPRB, 0.40044E+01_JPRB, 0.40630E+01_JPRB, 0.39426E+01_JPRB, 0.38994E+01_JPRB ,&
&  0.38158E+01_JPRB, 0.38431E+01_JPRB, 0.37979E+01_JPRB, 0.33122E+01_JPRB, 0.26769E+01_JPRB, 0.21479E+01_JPRB, 0.18565E+01_JPRB ,&
&  0.17633E+01_JPRB, 0.94242E+00_JPRB, 0.67284E+00_JPRB, 0.55913E+00_JPRB, 0.54366E+00_JPRB, 0.57649E+00_JPRB, 0.71014E+00_JPRB ,&
&  0.65346E+00_JPRB, 0.77820E+00_JPRB, 0.10209E+01_JPRB, 0.11698E+01_JPRB, 0.14291E+01_JPRB, 0.17198E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.12530E+01_JPRB, 0.17171E+01_JPRB, 0.18163E+01_JPRB, 0.16593E+01_JPRB, 0.16202E+01_JPRB, 0.16340E+01_JPRB ,&
&  0.15182E+01_JPRB, 0.14233E+01_JPRB, 0.12371E+01_JPRB, 0.10831E+01_JPRB, 0.10264E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.10839E+01_JPRB, 0.10714E+01_JPRB, 0.10863E+01_JPRB, 0.10985E+01_JPRB, 0.10885E+01_JPRB, 0.10506E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.97080E+00_JPRB, 0.94291E+00_JPRB, 0.92704E+00_JPRB, 0.91543E+00_JPRB, 0.90610E+00_JPRB, 0.89023E+00_JPRB ,&
&  0.87079E+00_JPRB, 0.87143E+00_JPRB, 0.77054E+00_JPRB, 0.68817E+00_JPRB, 0.61438E+00_JPRB, 0.62658E+00_JPRB, 0.64652E+00_JPRB ,&
&  0.67314E+00_JPRB, 0.68860E+00_JPRB, 0.70325E+00_JPRB, 0.71496E+00_JPRB, 0.72751E+00_JPRB, 0.70487E+00_JPRB, 0.72146E+00_JPRB ,&
&  0.70720E+00_JPRB, 0.68214E+00_JPRB, 0.65268E+00_JPRB, 0.71310E+00_JPRB, 0.77757E+00_JPRB, 0.77920E+00_JPRB, 0.91290E+00_JPRB ,&
&  0.98246E+00_JPRB, 0.10328E+01_JPRB, 0.10734E+01_JPRB, 0.11229E+01_JPRB, 0.11744E+01_JPRB, 0.11782E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.11021E+01_JPRB, 0.11581E+01_JPRB, 0.11965E+01_JPRB, 0.13528E+01_JPRB, 0.14407E+01_JPRB, 0.15484E+01_JPRB ,&
&  0.17017E+01_JPRB, 0.17315E+01_JPRB, 0.18717E+01_JPRB, 0.26682E+01_JPRB, 0.36397E+01_JPRB, 0.37077E+01_JPRB, 0.38439E+01_JPRB ,&
&  0.41301E+01_JPRB, 0.42256E+01_JPRB, 0.41911E+01_JPRB, 0.40166E+01_JPRB, 0.40129E+01_JPRB, 0.40474E+01_JPRB, 0.41020E+01_JPRB ,&
&  0.39383E+01_JPRB, 0.38553E+01_JPRB, 0.37195E+01_JPRB, 0.33471E+01_JPRB, 0.27756E+01_JPRB, 0.24479E+01_JPRB, 0.21384E+01_JPRB ,&
&  0.18463E+01_JPRB, 0.11988E+01_JPRB, 0.78016E+00_JPRB, 0.64821E+00_JPRB, 0.61239E+00_JPRB, 0.62169E+00_JPRB, 0.73798E+00_JPRB ,&
&  0.76515E+00_JPRB, 0.97139E+00_JPRB, 0.13076E+01_JPRB, 0.14796E+01_JPRB, 0.22405E+01_JPRB, 0.27899E+01_JPRB, 0.25771E+01_JPRB ,&
&  0.20626E+01_JPRB, 0.17734E+01_JPRB, 0.21443E+01_JPRB, 0.21447E+01_JPRB, 0.19736E+01_JPRB, 0.18004E+01_JPRB, 0.17562E+01_JPRB ,&
&  0.16455E+01_JPRB, 0.15572E+01_JPRB, 0.14236E+01_JPRB, 0.12353E+01_JPRB, 0.11267E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.12427E+01_JPRB, 0.11859E+01_JPRB, 0.11567E+01_JPRB, 0.11411E+01_JPRB, 0.10888E+01_JPRB, 0.10534E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.10146E+01_JPRB, 0.98278E+00_JPRB, 0.97236E+00_JPRB, 0.96717E+00_JPRB, 0.96009E+00_JPRB, 0.91775E+00_JPRB ,&
&  0.86417E+00_JPRB, 0.82911E+00_JPRB, 0.71959E+00_JPRB, 0.64089E+00_JPRB, 0.60560E+00_JPRB, 0.63290E+00_JPRB, 0.69149E+00_JPRB ,&
&  0.77789E+00_JPRB, 0.86493E+00_JPRB, 0.93927E+00_JPRB, 0.94381E+00_JPRB, 0.94731E+00_JPRB, 0.92102E+00_JPRB, 0.90291E+00_JPRB ,&
&  0.93171E+00_JPRB, 0.80222E+00_JPRB, 0.81095E+00_JPRB, 0.82962E+00_JPRB, 0.90424E+00_JPRB, 0.95576E+00_JPRB, 0.10067E+01_JPRB ,&
&  0.10179E+01_JPRB, 0.10525E+01_JPRB, 0.10820E+01_JPRB, 0.11111E+01_JPRB, 0.11348E+01_JPRB, 0.11398E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11461E+01_JPRB, 0.11492E+01_JPRB, 0.13856E+01_JPRB, 0.19102E+01_JPRB, 0.21901E+01_JPRB, 0.22494E+01_JPRB ,&
&  0.21705E+01_JPRB, 0.22876E+01_JPRB, 0.33219E+01_JPRB, 0.38002E+01_JPRB, 0.40543E+01_JPRB, 0.46363E+01_JPRB, 0.48546E+01_JPRB ,&
&  0.46386E+01_JPRB, 0.42132E+01_JPRB, 0.38435E+01_JPRB, 0.34980E+01_JPRB, 0.42423E+01_JPRB, 0.45130E+01_JPRB, 0.44708E+01_JPRB ,&
&  0.39973E+01_JPRB, 0.38638E+01_JPRB, 0.34867E+01_JPRB, 0.31370E+01_JPRB, 0.28888E+01_JPRB, 0.29054E+01_JPRB, 0.23168E+01_JPRB ,&
&  0.17806E+01_JPRB, 0.16404E+01_JPRB, 0.12309E+01_JPRB, 0.10294E+01_JPRB, 0.95308E+00_JPRB, 0.95016E+00_JPRB, 0.10079E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.13515E+01_JPRB, 0.17057E+01_JPRB, 0.20840E+01_JPRB, 0.29820E+01_JPRB, 0.37070E+01_JPRB, 0.37327E+01_JPRB ,&
&  0.30023E+01_JPRB, 0.29239E+01_JPRB, 0.26405E+01_JPRB, 0.24881E+01_JPRB, 0.22972E+01_JPRB, 0.20412E+01_JPRB, 0.19005E+01_JPRB ,&
&  0.17350E+01_JPRB, 0.16264E+01_JPRB, 0.15449E+01_JPRB, 0.14187E+01_JPRB, 0.13338E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.14643E+01_JPRB, 0.13798E+01_JPRB, 0.12624E+01_JPRB, 0.11786E+01_JPRB, 0.11061E+01_JPRB, 0.10636E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10170E+01_JPRB, 0.10234E+01_JPRB, 0.10367E+01_JPRB, 0.10351E+01_JPRB, 0.10085E+01_JPRB, 0.93420E+00_JPRB ,&
&  0.86420E+00_JPRB, 0.76981E+00_JPRB, 0.68437E+00_JPRB, 0.67525E+00_JPRB, 0.72408E+00_JPRB, 0.76601E+00_JPRB, 0.82939E+00_JPRB ,&
&  0.97400E+00_JPRB, 0.11972E+01_JPRB, 0.12796E+01_JPRB, 0.13216E+01_JPRB, 0.13146E+01_JPRB, 0.12595E+01_JPRB, 0.11816E+01_JPRB ,&
&  0.10732E+01_JPRB, 0.10641E+01_JPRB, 0.11018E+01_JPRB, 0.10846E+01_JPRB, 0.10563E+01_JPRB, 0.10538E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10587E+01_JPRB, 0.10995E+01_JPRB, 0.11255E+01_JPRB, 0.11849E+01_JPRB, 0.12263E+01_JPRB, 0.12634E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.12376E+01_JPRB, 0.14152E+01_JPRB, 0.20904E+01_JPRB, 0.27473E+01_JPRB, 0.32469E+01_JPRB, 0.31758E+01_JPRB ,&
&  0.35155E+01_JPRB, 0.37923E+01_JPRB, 0.39690E+01_JPRB, 0.44379E+01_JPRB, 0.36873E+01_JPRB, 0.41498E+01_JPRB, 0.47776E+01_JPRB ,&
&  0.45759E+01_JPRB, 0.41473E+01_JPRB, 0.39422E+01_JPRB, 0.38424E+01_JPRB, 0.43517E+01_JPRB, 0.58381E+01_JPRB, 0.53728E+01_JPRB ,&
&  0.44991E+01_JPRB, 0.40204E+01_JPRB, 0.34827E+01_JPRB, 0.33684E+01_JPRB, 0.29114E+01_JPRB, 0.25480E+01_JPRB, 0.18052E+01_JPRB ,&
&  0.17771E+01_JPRB, 0.15780E+01_JPRB, 0.15732E+01_JPRB, 0.14456E+01_JPRB, 0.14640E+01_JPRB, 0.14054E+01_JPRB, 0.14163E+01_JPRB ,&
&  0.16246E+01_JPRB, 0.18729E+01_JPRB, 0.26136E+01_JPRB, 0.32467E+01_JPRB, 0.47953E+01_JPRB, 0.46694E+01_JPRB, 0.46532E+01_JPRB ,&
&  0.40912E+01_JPRB, 0.37876E+01_JPRB, 0.30917E+01_JPRB, 0.29766E+01_JPRB, 0.26438E+01_JPRB, 0.23522E+01_JPRB, 0.20952E+01_JPRB ,&
&  0.18428E+01_JPRB, 0.16743E+01_JPRB, 0.15682E+01_JPRB, 0.15018E+01_JPRB, 0.14979E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.14215E+01_JPRB, 0.13484E+01_JPRB, 0.12661E+01_JPRB, 0.11815E+01_JPRB, 0.10940E+01_JPRB, 0.10465E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.10105E+01_JPRB, 0.10258E+01_JPRB, 0.10373E+01_JPRB, 0.10402E+01_JPRB, 0.10403E+01_JPRB, 0.10372E+01_JPRB, 0.95322E+00_JPRB ,&
&  0.86983E+00_JPRB, 0.78618E+00_JPRB, 0.68940E+00_JPRB, 0.68249E+00_JPRB, 0.83068E+00_JPRB, 0.94038E+00_JPRB, 0.93864E+00_JPRB ,&
&  0.13381E+01_JPRB, 0.16669E+01_JPRB, 0.16944E+01_JPRB, 0.16988E+01_JPRB, 0.17134E+01_JPRB, 0.16797E+01_JPRB, 0.15227E+01_JPRB ,&
&  0.14438E+01_JPRB, 0.13793E+01_JPRB, 0.12558E+01_JPRB, 0.11594E+01_JPRB, 0.10942E+01_JPRB, 0.10586E+01_JPRB, 0.10236E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.10166E+01_JPRB, 0.10367E+01_JPRB, 0.10724E+01_JPRB, 0.11484E+01_JPRB, 0.11462E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.14160E+01_JPRB, 0.18228E+01_JPRB, 0.26897E+01_JPRB, 0.38129E+01_JPRB, 0.45392E+01_JPRB, 0.46521E+01_JPRB ,&
&  0.43719E+01_JPRB, 0.43580E+01_JPRB, 0.45069E+01_JPRB, 0.44281E+01_JPRB, 0.43551E+01_JPRB, 0.44521E+01_JPRB, 0.48132E+01_JPRB ,&
&  0.37861E+01_JPRB, 0.31483E+01_JPRB, 0.28113E+01_JPRB, 0.33935E+01_JPRB, 0.54865E+01_JPRB, 0.71175E+01_JPRB, 0.70160E+01_JPRB ,&
&  0.59055E+01_JPRB, 0.49309E+01_JPRB, 0.41578E+01_JPRB, 0.35673E+01_JPRB, 0.34740E+01_JPRB, 0.23925E+01_JPRB, 0.25241E+01_JPRB ,&
&  0.25300E+01_JPRB, 0.25357E+01_JPRB, 0.23070E+01_JPRB, 0.19270E+01_JPRB, 0.17676E+01_JPRB, 0.19798E+01_JPRB, 0.21678E+01_JPRB ,&
&  0.25170E+01_JPRB, 0.28876E+01_JPRB, 0.46639E+01_JPRB, 0.55372E+01_JPRB, 0.64059E+01_JPRB, 0.56194E+01_JPRB, 0.49802E+01_JPRB ,&
&  0.40337E+01_JPRB, 0.37862E+01_JPRB, 0.36568E+01_JPRB, 0.33397E+01_JPRB, 0.29611E+01_JPRB, 0.26619E+01_JPRB, 0.23033E+01_JPRB ,&
&  0.20152E+01_JPRB, 0.17805E+01_JPRB, 0.16047E+01_JPRB, 0.14950E+01_JPRB, 0.14608E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.14156E+01_JPRB, 0.13545E+01_JPRB, 0.12640E+01_JPRB, 0.11332E+01_JPRB, 0.10605E+01_JPRB, 0.10377E+01_JPRB, 0.10483E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10708E+01_JPRB, 0.10574E+01_JPRB, 0.10390E+01_JPRB, 0.10433E+01_JPRB, 0.10263E+01_JPRB, 0.96375E+00_JPRB ,&
&  0.90641E+00_JPRB, 0.85049E+00_JPRB, 0.73479E+00_JPRB, 0.76492E+00_JPRB, 0.98986E+00_JPRB, 0.11447E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.19321E+01_JPRB, 0.20160E+01_JPRB, 0.17552E+01_JPRB, 0.17296E+01_JPRB, 0.19805E+01_JPRB, 0.18013E+01_JPRB, 0.16214E+01_JPRB ,&
&  0.15719E+01_JPRB, 0.14511E+01_JPRB, 0.12914E+01_JPRB, 0.11694E+01_JPRB, 0.10915E+01_JPRB, 0.10451E+01_JPRB, 0.10064E+01_JPRB ,&
&  0.99591E+00_JPRB, 0.10062E+01_JPRB, 0.10609E+01_JPRB, 0.11299E+01_JPRB, 0.11378E+01_JPRB, 0.11538E+01_JPRB, 0.12796E+01_JPRB ,&
&  0.14071E+01_JPRB, 0.15605E+01_JPRB, 0.20156E+01_JPRB, 0.28785E+01_JPRB, 0.41775E+01_JPRB, 0.53110E+01_JPRB, 0.65412E+01_JPRB ,&
&  0.57667E+01_JPRB, 0.58454E+01_JPRB, 0.51859E+01_JPRB, 0.55339E+01_JPRB, 0.49900E+01_JPRB, 0.48050E+01_JPRB, 0.47500E+01_JPRB ,&
&  0.34762E+01_JPRB, 0.29882E+01_JPRB, 0.34762E+01_JPRB, 0.52874E+01_JPRB, 0.59149E+01_JPRB, 0.66979E+01_JPRB, 0.81615E+01_JPRB ,&
&  0.64532E+01_JPRB, 0.53472E+01_JPRB, 0.47619E+01_JPRB, 0.46529E+01_JPRB, 0.29719E+01_JPRB, 0.23055E+01_JPRB, 0.34281E+01_JPRB ,&
&  0.31956E+01_JPRB, 0.18892E+01_JPRB, 0.14417E+01_JPRB, 0.16659E+01_JPRB, 0.13363E+01_JPRB, 0.16458E+01_JPRB, 0.25865E+01_JPRB ,&
&  0.31353E+01_JPRB, 0.37704E+01_JPRB, 0.60675E+01_JPRB, 0.57817E+01_JPRB, 0.60989E+01_JPRB, 0.63025E+01_JPRB, 0.49658E+01_JPRB ,&
&  0.43764E+01_JPRB, 0.34820E+01_JPRB, 0.34909E+01_JPRB, 0.31363E+01_JPRB, 0.29052E+01_JPRB, 0.27421E+01_JPRB, 0.24413E+01_JPRB ,&
&  0.21285E+01_JPRB, 0.18581E+01_JPRB, 0.16477E+01_JPRB, 0.15160E+01_JPRB, 0.14602E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.13537E+01_JPRB, 0.12810E+01_JPRB, 0.12383E+01_JPRB, 0.11611E+01_JPRB, 0.11016E+01_JPRB, 0.10856E+01_JPRB, 0.11158E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.10592E+01_JPRB, 0.10109E+01_JPRB, 0.99817E+00_JPRB, 0.10131E+01_JPRB, 0.10086E+01_JPRB, 0.95720E+00_JPRB ,&
&  0.91031E+00_JPRB, 0.86602E+00_JPRB, 0.85323E+00_JPRB, 0.95542E+00_JPRB, 0.13544E+01_JPRB, 0.13983E+01_JPRB, 0.18665E+01_JPRB ,&
&  0.24943E+01_JPRB, 0.21437E+01_JPRB, 0.18065E+01_JPRB, 0.17895E+01_JPRB, 0.17002E+01_JPRB, 0.15464E+01_JPRB, 0.15557E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.13782E+01_JPRB, 0.12682E+01_JPRB, 0.11849E+01_JPRB, 0.10938E+01_JPRB, 0.10248E+01_JPRB, 0.10017E+01_JPRB ,&
&  0.10151E+01_JPRB, 0.10679E+01_JPRB, 0.11691E+01_JPRB, 0.11428E+01_JPRB, 0.11626E+01_JPRB, 0.12107E+01_JPRB, 0.13508E+01_JPRB ,&
&  0.15070E+01_JPRB, 0.17062E+01_JPRB, 0.21283E+01_JPRB, 0.30851E+01_JPRB, 0.51617E+01_JPRB, 0.68657E+01_JPRB, 0.67738E+01_JPRB ,&
&  0.60482E+01_JPRB, 0.58055E+01_JPRB, 0.59889E+01_JPRB, 0.58162E+01_JPRB, 0.50004E+01_JPRB, 0.48911E+01_JPRB, 0.45926E+01_JPRB ,&
&  0.40168E+01_JPRB, 0.37856E+01_JPRB, 0.39896E+01_JPRB, 0.56580E+01_JPRB, 0.69301E+01_JPRB, 0.70951E+01_JPRB, 0.80650E+01_JPRB ,&
&  0.71926E+01_JPRB, 0.58187E+01_JPRB, 0.43773E+01_JPRB, 0.34955E+01_JPRB, 0.34838E+01_JPRB, 0.32020E+01_JPRB, 0.23576E+01_JPRB ,&
&  0.19696E+01_JPRB, 0.16228E+01_JPRB, 0.13715E+01_JPRB, 0.13616E+01_JPRB, 0.12004E+01_JPRB, 0.15002E+01_JPRB, 0.23568E+01_JPRB ,&
&  0.36263E+01_JPRB, 0.44052E+01_JPRB, 0.55901E+01_JPRB, 0.55402E+01_JPRB, 0.56639E+01_JPRB, 0.66722E+01_JPRB, 0.61783E+01_JPRB ,&
&  0.38693E+01_JPRB, 0.31943E+01_JPRB, 0.32046E+01_JPRB, 0.30116E+01_JPRB, 0.28276E+01_JPRB, 0.26506E+01_JPRB, 0.23646E+01_JPRB ,&
&  0.20377E+01_JPRB, 0.17426E+01_JPRB, 0.15873E+01_JPRB, 0.15295E+01_JPRB, 0.14615E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.11686E+01_JPRB, 0.11509E+01_JPRB, 0.11623E+01_JPRB, 0.11447E+01_JPRB, 0.11167E+01_JPRB, 0.11262E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.98448E+00_JPRB, 0.96247E+00_JPRB, 0.97875E+00_JPRB, 0.99087E+00_JPRB, 0.99384E+00_JPRB, 0.95935E+00_JPRB ,&
&  0.91397E+00_JPRB, 0.89680E+00_JPRB, 0.94620E+00_JPRB, 0.12168E+01_JPRB, 0.19212E+01_JPRB, 0.18654E+01_JPRB, 0.26731E+01_JPRB ,&
&  0.29097E+01_JPRB, 0.21669E+01_JPRB, 0.18476E+01_JPRB, 0.17104E+01_JPRB, 0.16293E+01_JPRB, 0.14743E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.12762E+01_JPRB, 0.12754E+01_JPRB, 0.11759E+01_JPRB, 0.11088E+01_JPRB, 0.10439E+01_JPRB, 0.10192E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.12032E+01_JPRB, 0.11917E+01_JPRB, 0.11952E+01_JPRB, 0.12610E+01_JPRB, 0.13300E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.18753E+01_JPRB, 0.23635E+01_JPRB, 0.39275E+01_JPRB, 0.55117E+01_JPRB, 0.59000E+01_JPRB, 0.65286E+01_JPRB ,&
&  0.66772E+01_JPRB, 0.67633E+01_JPRB, 0.58290E+01_JPRB, 0.61828E+01_JPRB, 0.50920E+01_JPRB, 0.46857E+01_JPRB, 0.44881E+01_JPRB ,&
&  0.41071E+01_JPRB, 0.45834E+01_JPRB, 0.45943E+01_JPRB, 0.50022E+01_JPRB, 0.69531E+01_JPRB, 0.81222E+01_JPRB, 0.80259E+01_JPRB ,&
&  0.76450E+01_JPRB, 0.63159E+01_JPRB, 0.49677E+01_JPRB, 0.43878E+01_JPRB, 0.56275E+01_JPRB, 0.64046E+01_JPRB, 0.29840E+01_JPRB ,&
&  0.20219E+01_JPRB, 0.18486E+01_JPRB, 0.14209E+01_JPRB, 0.12235E+01_JPRB, 0.12057E+01_JPRB, 0.15354E+01_JPRB, 0.27824E+01_JPRB ,&
&  0.31951E+01_JPRB, 0.51606E+01_JPRB, 0.56146E+01_JPRB, 0.59364E+01_JPRB, 0.56872E+01_JPRB, 0.56404E+01_JPRB, 0.46771E+01_JPRB ,&
&  0.46752E+01_JPRB, 0.40798E+01_JPRB, 0.34107E+01_JPRB, 0.30397E+01_JPRB, 0.26726E+01_JPRB, 0.23353E+01_JPRB, 0.20934E+01_JPRB ,&
&  0.19074E+01_JPRB, 0.16863E+01_JPRB, 0.15491E+01_JPRB, 0.14484E+01_JPRB, 0.12869E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.99864E+00_JPRB, 0.10321E+01_JPRB, 0.10854E+01_JPRB, 0.10936E+01_JPRB, 0.10762E+01_JPRB, 0.10889E+01_JPRB, 0.10387E+01_JPRB ,&
&  0.97634E+00_JPRB, 0.95251E+00_JPRB, 0.95736E+00_JPRB, 0.98549E+00_JPRB, 0.10291E+01_JPRB, 0.10604E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.97294E+00_JPRB, 0.96268E+00_JPRB, 0.10673E+01_JPRB, 0.17367E+01_JPRB, 0.24099E+01_JPRB, 0.30260E+01_JPRB, 0.34833E+01_JPRB ,&
&  0.32052E+01_JPRB, 0.23329E+01_JPRB, 0.19068E+01_JPRB, 0.18235E+01_JPRB, 0.17555E+01_JPRB, 0.16683E+01_JPRB, 0.14373E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.12315E+01_JPRB, 0.11066E+01_JPRB, 0.10702E+01_JPRB, 0.10518E+01_JPRB, 0.10729E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.12427E+01_JPRB, 0.12836E+01_JPRB, 0.13547E+01_JPRB, 0.14495E+01_JPRB, 0.15143E+01_JPRB, 0.16252E+01_JPRB ,&
&  0.18317E+01_JPRB, 0.21789E+01_JPRB, 0.29235E+01_JPRB, 0.46640E+01_JPRB, 0.49659E+01_JPRB, 0.63593E+01_JPRB, 0.65247E+01_JPRB ,&
&  0.64471E+01_JPRB, 0.62744E+01_JPRB, 0.55707E+01_JPRB, 0.56305E+01_JPRB, 0.50713E+01_JPRB, 0.46660E+01_JPRB, 0.43565E+01_JPRB ,&
&  0.47419E+01_JPRB, 0.46819E+01_JPRB, 0.58255E+01_JPRB, 0.56615E+01_JPRB, 0.64258E+01_JPRB, 0.10450E+02_JPRB, 0.81695E+01_JPRB ,&
&  0.68189E+01_JPRB, 0.70093E+01_JPRB, 0.57281E+01_JPRB, 0.53486E+01_JPRB, 0.65822E+01_JPRB, 0.81512E+01_JPRB, 0.68310E+01_JPRB ,&
&  0.26050E+01_JPRB, 0.17554E+01_JPRB, 0.15656E+01_JPRB, 0.13960E+01_JPRB, 0.12092E+01_JPRB, 0.15303E+01_JPRB, 0.51562E+01_JPRB ,&
&  0.37672E+01_JPRB, 0.52484E+01_JPRB, 0.53816E+01_JPRB, 0.54114E+01_JPRB, 0.56337E+01_JPRB, 0.50549E+01_JPRB, 0.45621E+01_JPRB ,&
&  0.37644E+01_JPRB, 0.30628E+01_JPRB, 0.25451E+01_JPRB, 0.21707E+01_JPRB, 0.19357E+01_JPRB, 0.18139E+01_JPRB, 0.17790E+01_JPRB ,&
&  0.17152E+01_JPRB, 0.15507E+01_JPRB, 0.13937E+01_JPRB, 0.11928E+01_JPRB, 0.10250E+01_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.10121E+01_JPRB, 0.10292E+01_JPRB, 0.10567E+01_JPRB, 0.11008E+01_JPRB, 0.11311E+01_JPRB, 0.11118E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10609E+01_JPRB, 0.10836E+01_JPRB, 0.11454E+01_JPRB, 0.11876E+01_JPRB, 0.12373E+01_JPRB, 0.12250E+01_JPRB, 0.11672E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.11242E+01_JPRB, 0.14828E+01_JPRB, 0.22824E+01_JPRB, 0.27833E+01_JPRB, 0.32318E+01_JPRB, 0.39009E+01_JPRB ,&
&  0.38618E+01_JPRB, 0.25592E+01_JPRB, 0.19377E+01_JPRB, 0.19572E+01_JPRB, 0.19451E+01_JPRB, 0.19625E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.13371E+01_JPRB, 0.12441E+01_JPRB, 0.11302E+01_JPRB, 0.11231E+01_JPRB, 0.11405E+01_JPRB, 0.11810E+01_JPRB, 0.12539E+01_JPRB ,&
&  0.13172E+01_JPRB, 0.14211E+01_JPRB, 0.15456E+01_JPRB, 0.16497E+01_JPRB, 0.17237E+01_JPRB, 0.17795E+01_JPRB, 0.18934E+01_JPRB ,&
&  0.21604E+01_JPRB, 0.26238E+01_JPRB, 0.34508E+01_JPRB, 0.45851E+01_JPRB, 0.55596E+01_JPRB, 0.58437E+01_JPRB, 0.61818E+01_JPRB ,&
&  0.61140E+01_JPRB, 0.60360E+01_JPRB, 0.56886E+01_JPRB, 0.52215E+01_JPRB, 0.47367E+01_JPRB, 0.46609E+01_JPRB, 0.44727E+01_JPRB ,&
&  0.50982E+01_JPRB, 0.57787E+01_JPRB, 0.67905E+01_JPRB, 0.65982E+01_JPRB, 0.69259E+01_JPRB, 0.98516E+01_JPRB, 0.10475E+02_JPRB ,&
&  0.84936E+01_JPRB, 0.62296E+01_JPRB, 0.54408E+01_JPRB, 0.59103E+01_JPRB, 0.66171E+01_JPRB, 0.73794E+01_JPRB, 0.10531E+02_JPRB ,&
&  0.85240E+01_JPRB, 0.37152E+01_JPRB, 0.21019E+01_JPRB, 0.21426E+01_JPRB, 0.12454E+01_JPRB, 0.11287E+01_JPRB, 0.22297E+01_JPRB ,&
&  0.31006E+01_JPRB, 0.39392E+01_JPRB, 0.40449E+01_JPRB, 0.44251E+01_JPRB, 0.45278E+01_JPRB, 0.37472E+01_JPRB, 0.28264E+01_JPRB ,&
&  0.20995E+01_JPRB, 0.17055E+01_JPRB, 0.14972E+01_JPRB, 0.13968E+01_JPRB, 0.14272E+01_JPRB, 0.14992E+01_JPRB, 0.15569E+01_JPRB ,&
&  0.15283E+01_JPRB, 0.13830E+01_JPRB, 0.11653E+01_JPRB, 0.10029E+01_JPRB, 0.97238E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.11642E+01_JPRB, 0.11580E+01_JPRB, 0.11733E+01_JPRB, 0.12225E+01_JPRB, 0.12455E+01_JPRB, 0.12618E+01_JPRB, 0.13666E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.14977E+01_JPRB, 0.15360E+01_JPRB, 0.15577E+01_JPRB, 0.15347E+01_JPRB, 0.14405E+01_JPRB, 0.13653E+01_JPRB ,&
&  0.13800E+01_JPRB, 0.16752E+01_JPRB, 0.23055E+01_JPRB, 0.31416E+01_JPRB, 0.35149E+01_JPRB, 0.45491E+01_JPRB, 0.37649E+01_JPRB ,&
&  0.37370E+01_JPRB, 0.32280E+01_JPRB, 0.21933E+01_JPRB, 0.22654E+01_JPRB, 0.22613E+01_JPRB, 0.23496E+01_JPRB, 0.17156E+01_JPRB ,&
&  0.14503E+01_JPRB, 0.13517E+01_JPRB, 0.12419E+01_JPRB, 0.12337E+01_JPRB, 0.12715E+01_JPRB, 0.13541E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.15894E+01_JPRB, 0.17829E+01_JPRB, 0.19506E+01_JPRB, 0.20331E+01_JPRB, 0.20784E+01_JPRB, 0.21393E+01_JPRB, 0.23102E+01_JPRB ,&
&  0.26076E+01_JPRB, 0.31132E+01_JPRB, 0.39406E+01_JPRB, 0.48228E+01_JPRB, 0.54044E+01_JPRB, 0.53792E+01_JPRB, 0.56276E+01_JPRB ,&
&  0.53771E+01_JPRB, 0.50162E+01_JPRB, 0.50796E+01_JPRB, 0.49114E+01_JPRB, 0.45953E+01_JPRB, 0.44412E+01_JPRB, 0.43739E+01_JPRB ,&
&  0.56120E+01_JPRB, 0.74839E+01_JPRB, 0.96234E+01_JPRB, 0.83871E+01_JPRB, 0.82850E+01_JPRB, 0.10166E+02_JPRB, 0.10569E+02_JPRB ,&
&  0.92215E+01_JPRB, 0.68901E+01_JPRB, 0.62297E+01_JPRB, 0.57175E+01_JPRB, 0.57755E+01_JPRB, 0.62178E+01_JPRB, 0.77456E+01_JPRB ,&
&  0.83080E+01_JPRB, 0.84062E+01_JPRB, 0.43237E+01_JPRB, 0.18268E+01_JPRB, 0.12583E+01_JPRB, 0.13787E+01_JPRB, 0.17500E+01_JPRB ,&
&  0.23849E+01_JPRB, 0.27273E+01_JPRB, 0.28360E+01_JPRB, 0.37087E+01_JPRB, 0.33083E+01_JPRB, 0.25768E+01_JPRB, 0.19788E+01_JPRB ,&
&  0.15924E+01_JPRB, 0.13115E+01_JPRB, 0.11742E+01_JPRB, 0.11685E+01_JPRB, 0.12720E+01_JPRB, 0.13779E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.11492E+01_JPRB, 0.10705E+01_JPRB, 0.10691E+01_JPRB, 0.11154E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.13644E+01_JPRB, 0.14030E+01_JPRB, 0.14715E+01_JPRB, 0.15706E+01_JPRB, 0.16671E+01_JPRB, 0.17862E+01_JPRB, 0.19166E+01_JPRB ,&
&  0.19960E+01_JPRB, 0.19979E+01_JPRB, 0.19787E+01_JPRB, 0.19164E+01_JPRB, 0.18810E+01_JPRB, 0.18732E+01_JPRB, 0.20035E+01_JPRB ,&
&  0.23928E+01_JPRB, 0.29959E+01_JPRB, 0.36025E+01_JPRB, 0.40011E+01_JPRB, 0.41497E+01_JPRB, 0.47157E+01_JPRB, 0.47917E+01_JPRB ,&
&  0.35291E+01_JPRB, 0.42472E+01_JPRB, 0.25173E+01_JPRB, 0.26437E+01_JPRB, 0.27663E+01_JPRB, 0.30550E+01_JPRB, 0.22349E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.16016E+01_JPRB, 0.15371E+01_JPRB, 0.15362E+01_JPRB, 0.15678E+01_JPRB, 0.16938E+01_JPRB, 0.18733E+01_JPRB ,&
&  0.20565E+01_JPRB, 0.22060E+01_JPRB, 0.23169E+01_JPRB, 0.23828E+01_JPRB, 0.24585E+01_JPRB, 0.25597E+01_JPRB, 0.28114E+01_JPRB ,&
&  0.31649E+01_JPRB, 0.37169E+01_JPRB, 0.43872E+01_JPRB, 0.50815E+01_JPRB, 0.50768E+01_JPRB, 0.49900E+01_JPRB, 0.51235E+01_JPRB ,&
&  0.49490E+01_JPRB, 0.46166E+01_JPRB, 0.46498E+01_JPRB, 0.43884E+01_JPRB, 0.41359E+01_JPRB, 0.43924E+01_JPRB, 0.46363E+01_JPRB ,&
&  0.65914E+01_JPRB, 0.80426E+01_JPRB, 0.10996E+02_JPRB, 0.10254E+02_JPRB, 0.89758E+01_JPRB, 0.94518E+01_JPRB, 0.82741E+01_JPRB ,&
&  0.65471E+01_JPRB, 0.59148E+01_JPRB, 0.55130E+01_JPRB, 0.50864E+01_JPRB, 0.50085E+01_JPRB, 0.52894E+01_JPRB, 0.60418E+01_JPRB ,&
&  0.63793E+01_JPRB, 0.60435E+01_JPRB, 0.41281E+01_JPRB, 0.19122E+01_JPRB, 0.14150E+01_JPRB, 0.13730E+01_JPRB, 0.16456E+01_JPRB ,&
&  0.22426E+01_JPRB, 0.24411E+01_JPRB, 0.24336E+01_JPRB, 0.23591E+01_JPRB, 0.19062E+01_JPRB, 0.15214E+01_JPRB, 0.12537E+01_JPRB ,&
&  0.11664E+01_JPRB, 0.11271E+01_JPRB, 0.11207E+01_JPRB, 0.12076E+01_JPRB, 0.12928E+01_JPRB, 0.13224E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.11186E+01_JPRB, 0.11126E+01_JPRB, 0.11853E+01_JPRB, 0.12594E+01_JPRB, 0.13221E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.15924E+01_JPRB, 0.16901E+01_JPRB, 0.18233E+01_JPRB, 0.20181E+01_JPRB, 0.22762E+01_JPRB, 0.25639E+01_JPRB, 0.25830E+01_JPRB ,&
&  0.25465E+01_JPRB, 0.24661E+01_JPRB, 0.25636E+01_JPRB, 0.26669E+01_JPRB, 0.27892E+01_JPRB, 0.29859E+01_JPRB, 0.32801E+01_JPRB ,&
&  0.36460E+01_JPRB, 0.40091E+01_JPRB, 0.43499E+01_JPRB, 0.46908E+01_JPRB, 0.50461E+01_JPRB, 0.57863E+01_JPRB, 0.60991E+01_JPRB ,&
&  0.55861E+01_JPRB, 0.34161E+01_JPRB, 0.30921E+01_JPRB, 0.27465E+01_JPRB, 0.28139E+01_JPRB, 0.31160E+01_JPRB, 0.29094E+01_JPRB ,&
&  0.23255E+01_JPRB, 0.20922E+01_JPRB, 0.20191E+01_JPRB, 0.20066E+01_JPRB, 0.19828E+01_JPRB, 0.21388E+01_JPRB, 0.22913E+01_JPRB ,&
&  0.24148E+01_JPRB, 0.25016E+01_JPRB, 0.25972E+01_JPRB, 0.26783E+01_JPRB, 0.27767E+01_JPRB, 0.29454E+01_JPRB, 0.32605E+01_JPRB ,&
&  0.36197E+01_JPRB, 0.40732E+01_JPRB, 0.45435E+01_JPRB, 0.47997E+01_JPRB, 0.47361E+01_JPRB, 0.46612E+01_JPRB, 0.46792E+01_JPRB ,&
&  0.46194E+01_JPRB, 0.45618E+01_JPRB, 0.44540E+01_JPRB, 0.42954E+01_JPRB, 0.41184E+01_JPRB, 0.48047E+01_JPRB, 0.57051E+01_JPRB ,&
&  0.74342E+01_JPRB, 0.86487E+01_JPRB, 0.11123E+02_JPRB, 0.98690E+01_JPRB, 0.85868E+01_JPRB, 0.77829E+01_JPRB, 0.66850E+01_JPRB ,&
&  0.53283E+01_JPRB, 0.50308E+01_JPRB, 0.46281E+01_JPRB, 0.39038E+01_JPRB, 0.35666E+01_JPRB, 0.34930E+01_JPRB, 0.44699E+01_JPRB ,&
&  0.49279E+01_JPRB, 0.48652E+01_JPRB, 0.29161E+01_JPRB, 0.18633E+01_JPRB, 0.14919E+01_JPRB, 0.14294E+01_JPRB, 0.17383E+01_JPRB ,&
&  0.22774E+01_JPRB, 0.21964E+01_JPRB, 0.18894E+01_JPRB, 0.18018E+01_JPRB, 0.14982E+01_JPRB, 0.12101E+01_JPRB, 0.10113E+01_JPRB ,&
&  0.10357E+01_JPRB, 0.10348E+01_JPRB, 0.10655E+01_JPRB, 0.11401E+01_JPRB, 0.11665E+01_JPRB, 0.12145E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.13307E+01_JPRB, 0.13930E+01_JPRB, 0.14548E+01_JPRB, 0.15122E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.16808E+01_JPRB, 0.17956E+01_JPRB, 0.19379E+01_JPRB, 0.21071E+01_JPRB, 0.22904E+01_JPRB, 0.24413E+01_JPRB, 0.25488E+01_JPRB ,&
&  0.27112E+01_JPRB, 0.28965E+01_JPRB, 0.30909E+01_JPRB, 0.32515E+01_JPRB, 0.33714E+01_JPRB, 0.35304E+01_JPRB, 0.37091E+01_JPRB ,&
&  0.38794E+01_JPRB, 0.40629E+01_JPRB, 0.42918E+01_JPRB, 0.45751E+01_JPRB, 0.49602E+01_JPRB, 0.54722E+01_JPRB, 0.57629E+01_JPRB ,&
&  0.46714E+01_JPRB, 0.28678E+01_JPRB, 0.32218E+01_JPRB, 0.29911E+01_JPRB, 0.29221E+01_JPRB, 0.29778E+01_JPRB, 0.28015E+01_JPRB ,&
&  0.25359E+01_JPRB, 0.24620E+01_JPRB, 0.25003E+01_JPRB, 0.24719E+01_JPRB, 0.23731E+01_JPRB, 0.24701E+01_JPRB, 0.25654E+01_JPRB ,&
&  0.26493E+01_JPRB, 0.27292E+01_JPRB, 0.28095E+01_JPRB, 0.28728E+01_JPRB, 0.29843E+01_JPRB, 0.31732E+01_JPRB, 0.34687E+01_JPRB ,&
&  0.38213E+01_JPRB, 0.41327E+01_JPRB, 0.44122E+01_JPRB, 0.45979E+01_JPRB, 0.45609E+01_JPRB, 0.45288E+01_JPRB, 0.45963E+01_JPRB ,&
&  0.46773E+01_JPRB, 0.46071E+01_JPRB, 0.47200E+01_JPRB, 0.50686E+01_JPRB, 0.55391E+01_JPRB, 0.57293E+01_JPRB, 0.71044E+01_JPRB ,&
&  0.83277E+01_JPRB, 0.89294E+01_JPRB, 0.96642E+01_JPRB, 0.11416E+02_JPRB, 0.73277E+01_JPRB, 0.64783E+01_JPRB, 0.53013E+01_JPRB ,&
&  0.52120E+01_JPRB, 0.47456E+01_JPRB, 0.40038E+01_JPRB, 0.30100E+01_JPRB, 0.25553E+01_JPRB, 0.23017E+01_JPRB, 0.29197E+01_JPRB ,&
&  0.37395E+01_JPRB, 0.36562E+01_JPRB, 0.25294E+01_JPRB, 0.19981E+01_JPRB, 0.17644E+01_JPRB, 0.19726E+01_JPRB, 0.22734E+01_JPRB ,&
&  0.23616E+01_JPRB, 0.19464E+01_JPRB, 0.17672E+01_JPRB, 0.19552E+01_JPRB, 0.14823E+01_JPRB, 0.10081E+01_JPRB, 0.81723E+00_JPRB ,&
&  0.81854E+00_JPRB, 0.86502E+00_JPRB, 0.94139E+00_JPRB, 0.10407E+01_JPRB, 0.10918E+01_JPRB, 0.11542E+01_JPRB, 0.11994E+01_JPRB ,&
&  0.13187E+01_JPRB, 0.14063E+01_JPRB, 0.14779E+01_JPRB, 0.15214E+01_JPRB, 0.15971E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.15853E+01_JPRB, 0.16929E+01_JPRB, 0.17858E+01_JPRB, 0.18913E+01_JPRB, 0.19896E+01_JPRB, 0.20857E+01_JPRB, 0.22187E+01_JPRB ,&
&  0.23777E+01_JPRB, 0.25592E+01_JPRB, 0.27325E+01_JPRB, 0.28673E+01_JPRB, 0.30088E+01_JPRB, 0.31076E+01_JPRB, 0.31961E+01_JPRB ,&
&  0.32428E+01_JPRB, 0.33152E+01_JPRB, 0.33785E+01_JPRB, 0.34806E+01_JPRB, 0.36373E+01_JPRB, 0.38225E+01_JPRB, 0.40011E+01_JPRB ,&
&  0.40985E+01_JPRB, 0.38704E+01_JPRB, 0.32467E+01_JPRB, 0.23485E+01_JPRB, 0.21467E+01_JPRB, 0.24910E+01_JPRB, 0.25107E+01_JPRB ,&
&  0.25110E+01_JPRB, 0.24695E+01_JPRB, 0.24341E+01_JPRB, 0.25141E+01_JPRB, 0.25622E+01_JPRB, 0.26133E+01_JPRB, 0.26461E+01_JPRB ,&
&  0.26909E+01_JPRB, 0.27694E+01_JPRB, 0.28199E+01_JPRB, 0.28732E+01_JPRB, 0.29683E+01_JPRB, 0.31534E+01_JPRB, 0.34233E+01_JPRB ,&
&  0.37475E+01_JPRB, 0.40555E+01_JPRB, 0.42513E+01_JPRB, 0.43980E+01_JPRB, 0.43821E+01_JPRB, 0.44909E+01_JPRB, 0.47522E+01_JPRB ,&
&  0.50999E+01_JPRB, 0.52976E+01_JPRB, 0.56550E+01_JPRB, 0.62965E+01_JPRB, 0.67697E+01_JPRB, 0.66201E+01_JPRB, 0.74871E+01_JPRB ,&
&  0.84461E+01_JPRB, 0.87791E+01_JPRB, 0.86769E+01_JPRB, 0.11037E+02_JPRB, 0.69239E+01_JPRB, 0.67122E+01_JPRB, 0.60950E+01_JPRB ,&
&  0.53810E+01_JPRB, 0.51360E+01_JPRB, 0.36949E+01_JPRB, 0.24912E+01_JPRB, 0.20537E+01_JPRB, 0.18444E+01_JPRB, 0.23685E+01_JPRB ,&
&  0.30233E+01_JPRB, 0.26184E+01_JPRB, 0.20494E+01_JPRB, 0.18664E+01_JPRB, 0.18815E+01_JPRB, 0.24049E+01_JPRB, 0.26534E+01_JPRB ,&
&  0.20881E+01_JPRB, 0.17715E+01_JPRB, 0.17159E+01_JPRB, 0.22103E+01_JPRB, 0.17992E+01_JPRB, 0.92277E+00_JPRB, 0.72674E+00_JPRB ,&
&  0.73364E+00_JPRB, 0.78977E+00_JPRB, 0.85550E+00_JPRB, 0.91400E+00_JPRB, 0.96479E+00_JPRB, 0.10277E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.12850E+01_JPRB, 0.13661E+01_JPRB, 0.14400E+01_JPRB, 0.15051E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.12109E+01_JPRB, 0.13063E+01_JPRB, 0.13966E+01_JPRB, 0.14592E+01_JPRB, 0.14898E+01_JPRB, 0.15154E+01_JPRB, 0.15236E+01_JPRB ,&
&  0.15404E+01_JPRB, 0.15407E+01_JPRB, 0.15945E+01_JPRB, 0.17186E+01_JPRB, 0.19397E+01_JPRB, 0.20748E+01_JPRB, 0.22252E+01_JPRB ,&
&  0.22320E+01_JPRB, 0.22423E+01_JPRB, 0.22518E+01_JPRB, 0.22992E+01_JPRB, 0.23865E+01_JPRB, 0.25099E+01_JPRB, 0.28037E+01_JPRB ,&
&  0.33835E+01_JPRB, 0.37479E+01_JPRB, 0.36732E+01_JPRB, 0.32730E+01_JPRB, 0.26465E+01_JPRB, 0.25410E+01_JPRB, 0.29126E+01_JPRB ,&
&  0.29578E+01_JPRB, 0.26887E+01_JPRB, 0.25119E+01_JPRB, 0.24956E+01_JPRB, 0.25133E+01_JPRB, 0.25353E+01_JPRB, 0.25524E+01_JPRB ,&
&  0.25928E+01_JPRB, 0.26496E+01_JPRB, 0.26949E+01_JPRB, 0.27487E+01_JPRB, 0.28416E+01_JPRB, 0.29825E+01_JPRB, 0.31643E+01_JPRB ,&
&  0.34061E+01_JPRB, 0.36206E+01_JPRB, 0.37538E+01_JPRB, 0.38284E+01_JPRB, 0.39935E+01_JPRB, 0.43920E+01_JPRB, 0.49868E+01_JPRB ,&
&  0.55355E+01_JPRB, 0.58170E+01_JPRB, 0.64196E+01_JPRB, 0.70579E+01_JPRB, 0.71562E+01_JPRB, 0.71219E+01_JPRB, 0.75383E+01_JPRB ,&
&  0.78324E+01_JPRB, 0.79176E+01_JPRB, 0.68756E+01_JPRB, 0.94921E+01_JPRB, 0.96882E+01_JPRB, 0.80834E+01_JPRB, 0.63455E+01_JPRB ,&
&  0.55744E+01_JPRB, 0.50266E+01_JPRB, 0.39632E+01_JPRB, 0.28279E+01_JPRB, 0.22972E+01_JPRB, 0.23711E+01_JPRB, 0.25952E+01_JPRB ,&
&  0.26969E+01_JPRB, 0.21135E+01_JPRB, 0.19004E+01_JPRB, 0.19127E+01_JPRB, 0.18330E+01_JPRB, 0.24136E+01_JPRB, 0.22142E+01_JPRB ,&
&  0.16189E+01_JPRB, 0.14917E+01_JPRB, 0.15838E+01_JPRB, 0.21216E+01_JPRB, 0.27863E+01_JPRB, 0.86692E+00_JPRB, 0.64516E+00_JPRB ,&
&  0.65820E+00_JPRB, 0.69331E+00_JPRB, 0.73746E+00_JPRB, 0.75867E+00_JPRB, 0.76333E+00_JPRB, 0.80472E+00_JPRB, 0.86869E+00_JPRB ,&
&  0.90485E+00_JPRB, 0.93557E+00_JPRB, 0.97403E+00_JPRB, 0.10650E+01_JPRB, 0.11305E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.77616E+00_JPRB, 0.82544E+00_JPRB, 0.86121E+00_JPRB, 0.89647E+00_JPRB, 0.91207E+00_JPRB, 0.90327E+00_JPRB, 0.85991E+00_JPRB ,&
&  0.82558E+00_JPRB, 0.79570E+00_JPRB, 0.79395E+00_JPRB, 0.78417E+00_JPRB, 0.82817E+00_JPRB, 0.90999E+00_JPRB, 0.96000E+00_JPRB ,&
&  0.11005E+01_JPRB, 0.10200E+01_JPRB, 0.10087E+01_JPRB, 0.10363E+01_JPRB, 0.10365E+01_JPRB, 0.10834E+01_JPRB, 0.12634E+01_JPRB ,&
&  0.15420E+01_JPRB, 0.18550E+01_JPRB, 0.23143E+01_JPRB, 0.30828E+01_JPRB, 0.33830E+01_JPRB, 0.28519E+01_JPRB, 0.31978E+01_JPRB ,&
&  0.28787E+01_JPRB, 0.23693E+01_JPRB, 0.23021E+01_JPRB, 0.22270E+01_JPRB, 0.22702E+01_JPRB, 0.22911E+01_JPRB, 0.23524E+01_JPRB ,&
&  0.24224E+01_JPRB, 0.25006E+01_JPRB, 0.25728E+01_JPRB, 0.26390E+01_JPRB, 0.27141E+01_JPRB, 0.28023E+01_JPRB, 0.29212E+01_JPRB ,&
&  0.30152E+01_JPRB, 0.30973E+01_JPRB, 0.32443E+01_JPRB, 0.33310E+01_JPRB, 0.37747E+01_JPRB, 0.44194E+01_JPRB, 0.50713E+01_JPRB ,&
&  0.55259E+01_JPRB, 0.58460E+01_JPRB, 0.63991E+01_JPRB, 0.65854E+01_JPRB, 0.66849E+01_JPRB, 0.67891E+01_JPRB, 0.67329E+01_JPRB ,&
&  0.64360E+01_JPRB, 0.52997E+01_JPRB, 0.48503E+01_JPRB, 0.74841E+01_JPRB, 0.72072E+01_JPRB, 0.56643E+01_JPRB, 0.56120E+01_JPRB ,&
&  0.51428E+01_JPRB, 0.48035E+01_JPRB, 0.40624E+01_JPRB, 0.32013E+01_JPRB, 0.25845E+01_JPRB, 0.22708E+01_JPRB, 0.24759E+01_JPRB ,&
&  0.21339E+01_JPRB, 0.17430E+01_JPRB, 0.18435E+01_JPRB, 0.19863E+01_JPRB, 0.19342E+01_JPRB, 0.21176E+01_JPRB, 0.18847E+01_JPRB ,&
&  0.15844E+01_JPRB, 0.13498E+01_JPRB, 0.15975E+01_JPRB, 0.21520E+01_JPRB, 0.14770E+01_JPRB, 0.71919E+00_JPRB, 0.56949E+00_JPRB ,&
&  0.57797E+00_JPRB, 0.59118E+00_JPRB, 0.60331E+00_JPRB, 0.61099E+00_JPRB, 0.61753E+00_JPRB, 0.63721E+00_JPRB, 0.65112E+00_JPRB ,&
&  0.66466E+00_JPRB, 0.69516E+00_JPRB, 0.70868E+00_JPRB, 0.72765E+00_JPRB, 0.75684E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.66294E+00_JPRB, 0.66568E+00_JPRB, 0.66116E+00_JPRB, 0.66499E+00_JPRB, 0.67316E+00_JPRB, 0.69302E+00_JPRB, 0.70631E+00_JPRB ,&
&  0.70894E+00_JPRB, 0.71846E+00_JPRB, 0.70202E+00_JPRB, 0.70859E+00_JPRB, 0.73770E+00_JPRB, 0.76204E+00_JPRB, 0.78815E+00_JPRB ,&
&  0.85249E+00_JPRB, 0.86853E+00_JPRB, 0.86638E+00_JPRB, 0.87459E+00_JPRB, 0.86551E+00_JPRB, 0.92935E+00_JPRB, 0.93277E+00_JPRB ,&
&  0.90287E+00_JPRB, 0.92817E+00_JPRB, 0.10977E+01_JPRB, 0.15472E+01_JPRB, 0.20017E+01_JPRB, 0.21031E+01_JPRB, 0.25481E+01_JPRB ,&
&  0.26017E+01_JPRB, 0.18787E+01_JPRB, 0.19035E+01_JPRB, 0.16508E+01_JPRB, 0.16747E+01_JPRB, 0.17588E+01_JPRB, 0.17947E+01_JPRB ,&
&  0.18446E+01_JPRB, 0.18484E+01_JPRB, 0.18799E+01_JPRB, 0.19330E+01_JPRB, 0.19672E+01_JPRB, 0.20117E+01_JPRB, 0.20643E+01_JPRB ,&
&  0.20065E+01_JPRB, 0.21035E+01_JPRB, 0.23902E+01_JPRB, 0.28736E+01_JPRB, 0.34329E+01_JPRB, 0.42182E+01_JPRB, 0.45890E+01_JPRB ,&
&  0.48589E+01_JPRB, 0.51638E+01_JPRB, 0.48963E+01_JPRB, 0.50636E+01_JPRB, 0.55039E+01_JPRB, 0.51735E+01_JPRB, 0.49607E+01_JPRB ,&
&  0.47306E+01_JPRB, 0.37247E+01_JPRB, 0.35771E+01_JPRB, 0.51830E+01_JPRB, 0.53646E+01_JPRB, 0.49485E+01_JPRB, 0.49394E+01_JPRB ,&
&  0.46792E+01_JPRB, 0.42465E+01_JPRB, 0.35444E+01_JPRB, 0.29895E+01_JPRB, 0.24976E+01_JPRB, 0.20461E+01_JPRB, 0.19997E+01_JPRB ,&
&  0.16385E+01_JPRB, 0.16527E+01_JPRB, 0.19770E+01_JPRB, 0.21703E+01_JPRB, 0.22852E+01_JPRB, 0.26199E+01_JPRB, 0.19748E+01_JPRB ,&
&  0.14526E+01_JPRB, 0.14071E+01_JPRB, 0.20584E+01_JPRB, 0.27398E+01_JPRB, 0.12544E+01_JPRB, 0.73305E+00_JPRB, 0.50668E+00_JPRB ,&
&  0.52287E+00_JPRB, 0.54158E+00_JPRB, 0.54704E+00_JPRB, 0.55343E+00_JPRB, 0.56878E+00_JPRB, 0.58144E+00_JPRB, 0.60417E+00_JPRB ,&
&  0.61738E+00_JPRB, 0.62782E+00_JPRB, 0.63893E+00_JPRB, 0.65696E+00_JPRB, 0.65564E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.72426E+00_JPRB, 0.73767E+00_JPRB, 0.74829E+00_JPRB, 0.77188E+00_JPRB, 0.80590E+00_JPRB, 0.82170E+00_JPRB, 0.84731E+00_JPRB ,&
&  0.86530E+00_JPRB, 0.89253E+00_JPRB, 0.91983E+00_JPRB, 0.95213E+00_JPRB, 0.97970E+00_JPRB, 0.99639E+00_JPRB, 0.10326E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10679E+01_JPRB, 0.10877E+01_JPRB, 0.11263E+01_JPRB, 0.11346E+01_JPRB, 0.11817E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.11830E+01_JPRB, 0.12051E+01_JPRB, 0.13106E+01_JPRB, 0.14365E+01_JPRB, 0.16734E+01_JPRB, 0.21606E+01_JPRB ,&
&  0.20374E+01_JPRB, 0.15808E+01_JPRB, 0.13856E+01_JPRB, 0.11791E+01_JPRB, 0.11695E+01_JPRB, 0.11944E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.11561E+01_JPRB, 0.11604E+01_JPRB, 0.11904E+01_JPRB, 0.12941E+01_JPRB, 0.14961E+01_JPRB, 0.16595E+01_JPRB ,&
&  0.17249E+01_JPRB, 0.20164E+01_JPRB, 0.23692E+01_JPRB, 0.28320E+01_JPRB, 0.31608E+01_JPRB, 0.40238E+01_JPRB, 0.41161E+01_JPRB ,&
&  0.40310E+01_JPRB, 0.39350E+01_JPRB, 0.33634E+01_JPRB, 0.35417E+01_JPRB, 0.39415E+01_JPRB, 0.38066E+01_JPRB, 0.33834E+01_JPRB ,&
&  0.33602E+01_JPRB, 0.31548E+01_JPRB, 0.32436E+01_JPRB, 0.42359E+01_JPRB, 0.45148E+01_JPRB, 0.45399E+01_JPRB, 0.42846E+01_JPRB ,&
&  0.39489E+01_JPRB, 0.34446E+01_JPRB, 0.29877E+01_JPRB, 0.27428E+01_JPRB, 0.25786E+01_JPRB, 0.21876E+01_JPRB, 0.18379E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.18181E+01_JPRB, 0.20990E+01_JPRB, 0.22535E+01_JPRB, 0.27048E+01_JPRB, 0.24008E+01_JPRB, 0.26167E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13620E+01_JPRB, 0.10594E+01_JPRB, 0.14296E+01_JPRB, 0.87086E+00_JPRB, 0.58264E+00_JPRB, 0.54624E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.57046E+00_JPRB, 0.58568E+00_JPRB, 0.59719E+00_JPRB, 0.61145E+00_JPRB, 0.63664E+00_JPRB, 0.66357E+00_JPRB ,&
&  0.69008E+00_JPRB, 0.68391E+00_JPRB, 0.68802E+00_JPRB, 0.69769E+00_JPRB, 0.71658E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.78386E+00_JPRB, 0.81205E+00_JPRB, 0.84054E+00_JPRB, 0.86939E+00_JPRB, 0.89554E+00_JPRB, 0.92091E+00_JPRB, 0.94691E+00_JPRB ,&
&  0.97248E+00_JPRB, 0.10014E+01_JPRB, 0.10323E+01_JPRB, 0.10665E+01_JPRB, 0.10981E+01_JPRB, 0.11190E+01_JPRB, 0.11468E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.11928E+01_JPRB, 0.11955E+01_JPRB, 0.12407E+01_JPRB, 0.12851E+01_JPRB, 0.13118E+01_JPRB, 0.13518E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.13575E+01_JPRB, 0.13443E+01_JPRB, 0.13821E+01_JPRB, 0.14171E+01_JPRB, 0.13736E+01_JPRB, 0.15808E+01_JPRB ,&
&  0.13179E+01_JPRB, 0.13966E+01_JPRB, 0.12596E+01_JPRB, 0.10990E+01_JPRB, 0.10573E+01_JPRB, 0.93636E+00_JPRB, 0.87907E+00_JPRB ,&
&  0.95916E+00_JPRB, 0.10042E+01_JPRB, 0.10634E+01_JPRB, 0.11687E+01_JPRB, 0.13716E+01_JPRB, 0.16982E+01_JPRB, 0.19680E+01_JPRB ,&
&  0.22941E+01_JPRB, 0.26185E+01_JPRB, 0.29887E+01_JPRB, 0.33998E+01_JPRB, 0.38600E+01_JPRB, 0.41293E+01_JPRB, 0.40989E+01_JPRB ,&
&  0.39155E+01_JPRB, 0.36718E+01_JPRB, 0.27661E+01_JPRB, 0.31328E+01_JPRB, 0.37158E+01_JPRB, 0.35999E+01_JPRB, 0.30289E+01_JPRB ,&
&  0.27170E+01_JPRB, 0.26677E+01_JPRB, 0.29796E+01_JPRB, 0.33315E+01_JPRB, 0.35603E+01_JPRB, 0.36369E+01_JPRB, 0.34375E+01_JPRB ,&
&  0.31906E+01_JPRB, 0.27327E+01_JPRB, 0.25222E+01_JPRB, 0.25539E+01_JPRB, 0.24855E+01_JPRB, 0.22697E+01_JPRB, 0.18851E+01_JPRB ,&
&  0.17860E+01_JPRB, 0.19220E+01_JPRB, 0.21000E+01_JPRB, 0.22260E+01_JPRB, 0.24100E+01_JPRB, 0.35411E+01_JPRB, 0.34825E+01_JPRB ,&
&  0.14169E+01_JPRB, 0.11154E+01_JPRB, 0.63709E+00_JPRB, 0.72899E+00_JPRB, 0.74521E+00_JPRB, 0.66573E+00_JPRB, 0.70840E+00_JPRB ,&
&  0.74377E+00_JPRB, 0.71247E+00_JPRB, 0.70472E+00_JPRB, 0.68292E+00_JPRB, 0.68591E+00_JPRB, 0.68463E+00_JPRB, 0.70199E+00_JPRB ,&
&  0.71374E+00_JPRB, 0.71888E+00_JPRB, 0.73227E+00_JPRB, 0.74930E+00_JPRB, 0.76321E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.81823E+00_JPRB, 0.85040E+00_JPRB, 0.88258E+00_JPRB, 0.91209E+00_JPRB, 0.94093E+00_JPRB, 0.96874E+00_JPRB, 0.99772E+00_JPRB ,&
&  0.10269E+01_JPRB, 0.10581E+01_JPRB, 0.10888E+01_JPRB, 0.11267E+01_JPRB, 0.11635E+01_JPRB, 0.11868E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.13156E+01_JPRB, 0.13554E+01_JPRB, 0.14217E+01_JPRB, 0.14785E+01_JPRB, 0.15202E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.16106E+01_JPRB, 0.16188E+01_JPRB, 0.16287E+01_JPRB, 0.16780E+01_JPRB, 0.17149E+01_JPRB, 0.18890E+01_JPRB, 0.16244E+01_JPRB ,&
&  0.16221E+01_JPRB, 0.14472E+01_JPRB, 0.13305E+01_JPRB, 0.12442E+01_JPRB, 0.11723E+01_JPRB, 0.11104E+01_JPRB, 0.10835E+01_JPRB ,&
&  0.11285E+01_JPRB, 0.11656E+01_JPRB, 0.12489E+01_JPRB, 0.13791E+01_JPRB, 0.16060E+01_JPRB, 0.19659E+01_JPRB, 0.22646E+01_JPRB ,&
&  0.25973E+01_JPRB, 0.29855E+01_JPRB, 0.33541E+01_JPRB, 0.37445E+01_JPRB, 0.41821E+01_JPRB, 0.46334E+01_JPRB, 0.50078E+01_JPRB ,&
&  0.50639E+01_JPRB, 0.49678E+01_JPRB, 0.42011E+01_JPRB, 0.46616E+01_JPRB, 0.49667E+01_JPRB, 0.44048E+01_JPRB, 0.32852E+01_JPRB ,&
&  0.25503E+01_JPRB, 0.21016E+01_JPRB, 0.21858E+01_JPRB, 0.25249E+01_JPRB, 0.26650E+01_JPRB, 0.26926E+01_JPRB, 0.25563E+01_JPRB ,&
&  0.23696E+01_JPRB, 0.21013E+01_JPRB, 0.20000E+01_JPRB, 0.20473E+01_JPRB, 0.21528E+01_JPRB, 0.21873E+01_JPRB, 0.20191E+01_JPRB ,&
&  0.19546E+01_JPRB, 0.19123E+01_JPRB, 0.18675E+01_JPRB, 0.17949E+01_JPRB, 0.18747E+01_JPRB, 0.17841E+01_JPRB, 0.19403E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.84257E+00_JPRB, 0.82103E+00_JPRB, 0.71750E+00_JPRB, 0.78940E+00_JPRB, 0.79775E+00_JPRB, 0.96384E+00_JPRB ,&
&  0.10976E+01_JPRB, 0.11498E+01_JPRB, 0.10807E+01_JPRB, 0.90248E+00_JPRB, 0.76612E+00_JPRB, 0.71425E+00_JPRB, 0.71850E+00_JPRB ,&
&  0.72539E+00_JPRB, 0.72934E+00_JPRB, 0.74533E+00_JPRB, 0.76909E+00_JPRB, 0.79091E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.78659E+00_JPRB, 0.81733E+00_JPRB, 0.85094E+00_JPRB, 0.87881E+00_JPRB, 0.91304E+00_JPRB, 0.96097E+00_JPRB, 0.10086E+01_JPRB ,&
&  0.10506E+01_JPRB, 0.10835E+01_JPRB, 0.11111E+01_JPRB, 0.11484E+01_JPRB, 0.11961E+01_JPRB, 0.12636E+01_JPRB, 0.13487E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.14796E+01_JPRB, 0.15404E+01_JPRB, 0.16380E+01_JPRB, 0.17029E+01_JPRB, 0.18116E+01_JPRB, 0.18990E+01_JPRB ,&
&  0.19459E+01_JPRB, 0.19703E+01_JPRB, 0.20017E+01_JPRB, 0.20609E+01_JPRB, 0.20632E+01_JPRB, 0.21721E+01_JPRB, 0.17622E+01_JPRB ,&
&  0.18041E+01_JPRB, 0.17172E+01_JPRB, 0.15816E+01_JPRB, 0.15221E+01_JPRB, 0.14719E+01_JPRB, 0.14429E+01_JPRB, 0.14415E+01_JPRB ,&
&  0.14567E+01_JPRB, 0.14555E+01_JPRB, 0.13998E+01_JPRB, 0.14687E+01_JPRB, 0.16648E+01_JPRB, 0.20234E+01_JPRB, 0.23418E+01_JPRB ,&
&  0.26967E+01_JPRB, 0.30631E+01_JPRB, 0.34492E+01_JPRB, 0.39442E+01_JPRB, 0.45853E+01_JPRB, 0.52663E+01_JPRB, 0.59438E+01_JPRB ,&
&  0.62130E+01_JPRB, 0.63369E+01_JPRB, 0.57858E+01_JPRB, 0.58521E+01_JPRB, 0.59504E+01_JPRB, 0.47917E+01_JPRB, 0.32583E+01_JPRB ,&
&  0.19756E+01_JPRB, 0.17899E+01_JPRB, 0.19503E+01_JPRB, 0.20556E+01_JPRB, 0.19072E+01_JPRB, 0.17933E+01_JPRB, 0.16385E+01_JPRB ,&
&  0.14719E+01_JPRB, 0.13403E+01_JPRB, 0.13580E+01_JPRB, 0.14915E+01_JPRB, 0.16536E+01_JPRB, 0.18460E+01_JPRB, 0.18965E+01_JPRB ,&
&  0.18538E+01_JPRB, 0.17075E+01_JPRB, 0.15742E+01_JPRB, 0.14895E+01_JPRB, 0.14067E+01_JPRB, 0.12646E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.94737E+00_JPRB, 0.93144E+00_JPRB, 0.63342E+00_JPRB, 0.79898E+00_JPRB, 0.99206E+00_JPRB, 0.12544E+01_JPRB ,&
&  0.13750E+01_JPRB, 0.18219E+01_JPRB, 0.22109E+01_JPRB, 0.15846E+01_JPRB, 0.11906E+01_JPRB, 0.77498E+00_JPRB, 0.70872E+00_JPRB ,&
&  0.71592E+00_JPRB, 0.71162E+00_JPRB, 0.72060E+00_JPRB, 0.74000E+00_JPRB, 0.76041E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.72288E+00_JPRB, 0.74593E+00_JPRB, 0.77615E+00_JPRB, 0.78697E+00_JPRB, 0.80081E+00_JPRB, 0.82398E+00_JPRB, 0.84859E+00_JPRB ,&
&  0.87162E+00_JPRB, 0.89448E+00_JPRB, 0.91421E+00_JPRB, 0.94184E+00_JPRB, 0.97776E+00_JPRB, 0.10619E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.12972E+01_JPRB, 0.14108E+01_JPRB, 0.15508E+01_JPRB, 0.16355E+01_JPRB, 0.17575E+01_JPRB, 0.18933E+01_JPRB, 0.19949E+01_JPRB ,&
&  0.21083E+01_JPRB, 0.21682E+01_JPRB, 0.22253E+01_JPRB, 0.23582E+01_JPRB, 0.23192E+01_JPRB, 0.23829E+01_JPRB, 0.18315E+01_JPRB ,&
&  0.19026E+01_JPRB, 0.18822E+01_JPRB, 0.18014E+01_JPRB, 0.17637E+01_JPRB, 0.17066E+01_JPRB, 0.16210E+01_JPRB, 0.15611E+01_JPRB ,&
&  0.15262E+01_JPRB, 0.14244E+01_JPRB, 0.13136E+01_JPRB, 0.13802E+01_JPRB, 0.15207E+01_JPRB, 0.17969E+01_JPRB, 0.21751E+01_JPRB ,&
&  0.25641E+01_JPRB, 0.29261E+01_JPRB, 0.33099E+01_JPRB, 0.38609E+01_JPRB, 0.45858E+01_JPRB, 0.53554E+01_JPRB, 0.61668E+01_JPRB ,&
&  0.67294E+01_JPRB, 0.71570E+01_JPRB, 0.72618E+01_JPRB, 0.61331E+01_JPRB, 0.57704E+01_JPRB, 0.43108E+01_JPRB, 0.27277E+01_JPRB ,&
&  0.19456E+01_JPRB, 0.17170E+01_JPRB, 0.17222E+01_JPRB, 0.16741E+01_JPRB, 0.14260E+01_JPRB, 0.12780E+01_JPRB, 0.11148E+01_JPRB ,&
&  0.98791E+00_JPRB, 0.95884E+00_JPRB, 0.10121E+01_JPRB, 0.11602E+01_JPRB, 0.13143E+01_JPRB, 0.15053E+01_JPRB, 0.15686E+01_JPRB ,&
&  0.15063E+01_JPRB, 0.13672E+01_JPRB, 0.12776E+01_JPRB, 0.12213E+01_JPRB, 0.12241E+01_JPRB, 0.12819E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.98395E+00_JPRB, 0.79530E+00_JPRB, 0.75921E+00_JPRB, 0.81231E+00_JPRB, 0.84484E+00_JPRB, 0.10923E+01_JPRB, 0.15211E+01_JPRB ,&
&  0.21453E+01_JPRB, 0.22480E+01_JPRB, 0.26892E+01_JPRB, 0.35457E+01_JPRB, 0.26058E+01_JPRB, 0.18660E+01_JPRB, 0.63595E+00_JPRB ,&
&  0.65663E+00_JPRB, 0.65905E+00_JPRB, 0.66620E+00_JPRB, 0.67390E+00_JPRB, 0.69230E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.60040E+00_JPRB, 0.63183E+00_JPRB, 0.62557E+00_JPRB, 0.63159E+00_JPRB, 0.64042E+00_JPRB, 0.65548E+00_JPRB, 0.68605E+00_JPRB ,&
&  0.69811E+00_JPRB, 0.74761E+00_JPRB, 0.80403E+00_JPRB, 0.85383E+00_JPRB, 0.91635E+00_JPRB, 0.10033E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.13317E+01_JPRB, 0.14188E+01_JPRB, 0.14917E+01_JPRB, 0.16054E+01_JPRB, 0.17463E+01_JPRB, 0.18492E+01_JPRB ,&
&  0.20014E+01_JPRB, 0.21152E+01_JPRB, 0.22426E+01_JPRB, 0.24189E+01_JPRB, 0.23262E+01_JPRB, 0.22962E+01_JPRB, 0.16255E+01_JPRB ,&
&  0.18715E+01_JPRB, 0.19123E+01_JPRB, 0.18910E+01_JPRB, 0.18548E+01_JPRB, 0.17683E+01_JPRB, 0.16270E+01_JPRB, 0.15386E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.13708E+01_JPRB, 0.12650E+01_JPRB, 0.12134E+01_JPRB, 0.13623E+01_JPRB, 0.15110E+01_JPRB, 0.18523E+01_JPRB ,&
&  0.22541E+01_JPRB, 0.27013E+01_JPRB, 0.31363E+01_JPRB, 0.36668E+01_JPRB, 0.43625E+01_JPRB, 0.51619E+01_JPRB, 0.59754E+01_JPRB ,&
&  0.65886E+01_JPRB, 0.71002E+01_JPRB, 0.70287E+01_JPRB, 0.58143E+01_JPRB, 0.51048E+01_JPRB, 0.37507E+01_JPRB, 0.23390E+01_JPRB ,&
&  0.18147E+01_JPRB, 0.16627E+01_JPRB, 0.16543E+01_JPRB, 0.15178E+01_JPRB, 0.12392E+01_JPRB, 0.10594E+01_JPRB, 0.90182E+00_JPRB ,&
&  0.77902E+00_JPRB, 0.77378E+00_JPRB, 0.86969E+00_JPRB, 0.10055E+01_JPRB, 0.11215E+01_JPRB, 0.12431E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.11673E+01_JPRB, 0.10577E+01_JPRB, 0.99988E+00_JPRB, 0.10397E+01_JPRB, 0.12122E+01_JPRB, 0.14009E+01_JPRB, 0.15110E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.79627E+00_JPRB, 0.76169E+00_JPRB, 0.76933E+00_JPRB, 0.80386E+00_JPRB, 0.87903E+00_JPRB, 0.10029E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.15798E+01_JPRB, 0.15089E+01_JPRB, 0.12248E+01_JPRB, 0.17079E+01_JPRB, 0.14366E+01_JPRB, 0.57893E+00_JPRB ,&
&  0.56159E+00_JPRB, 0.55084E+00_JPRB, 0.56301E+00_JPRB, 0.57198E+00_JPRB, 0.59163E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.56182E+00_JPRB, 0.55108E+00_JPRB, 0.53315E+00_JPRB, 0.54525E+00_JPRB, 0.54854E+00_JPRB, 0.57192E+00_JPRB, 0.59925E+00_JPRB ,&
&  0.63823E+00_JPRB, 0.71185E+00_JPRB, 0.79118E+00_JPRB, 0.86194E+00_JPRB, 0.91584E+00_JPRB, 0.97753E+00_JPRB, 0.10352E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.11311E+01_JPRB, 0.11952E+01_JPRB, 0.12786E+01_JPRB, 0.13863E+01_JPRB, 0.15122E+01_JPRB, 0.16637E+01_JPRB ,&
&  0.18326E+01_JPRB, 0.20104E+01_JPRB, 0.22311E+01_JPRB, 0.23419E+01_JPRB, 0.24190E+01_JPRB, 0.24927E+01_JPRB, 0.24918E+01_JPRB ,&
&  0.16546E+01_JPRB, 0.20242E+01_JPRB, 0.19535E+01_JPRB, 0.18634E+01_JPRB, 0.17848E+01_JPRB, 0.16554E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.13376E+01_JPRB, 0.12771E+01_JPRB, 0.11498E+01_JPRB, 0.11887E+01_JPRB, 0.12946E+01_JPRB, 0.15040E+01_JPRB ,&
&  0.18383E+01_JPRB, 0.23177E+01_JPRB, 0.28964E+01_JPRB, 0.34485E+01_JPRB, 0.39984E+01_JPRB, 0.46112E+01_JPRB, 0.54718E+01_JPRB ,&
&  0.64124E+01_JPRB, 0.70684E+01_JPRB, 0.74673E+01_JPRB, 0.64449E+01_JPRB, 0.50513E+01_JPRB, 0.42973E+01_JPRB, 0.30798E+01_JPRB ,&
&  0.19062E+01_JPRB, 0.15078E+01_JPRB, 0.15298E+01_JPRB, 0.14856E+01_JPRB, 0.12326E+01_JPRB, 0.10618E+01_JPRB, 0.89706E+00_JPRB ,&
&  0.75795E+00_JPRB, 0.68954E+00_JPRB, 0.71970E+00_JPRB, 0.85948E+00_JPRB, 0.93740E+00_JPRB, 0.97488E+00_JPRB, 0.94693E+00_JPRB ,&
&  0.91368E+00_JPRB, 0.84141E+00_JPRB, 0.76645E+00_JPRB, 0.76560E+00_JPRB, 0.81671E+00_JPRB, 0.86025E+00_JPRB, 0.85171E+00_JPRB ,&
&  0.82050E+00_JPRB, 0.80579E+00_JPRB, 0.80200E+00_JPRB, 0.81286E+00_JPRB, 0.81611E+00_JPRB, 0.82835E+00_JPRB, 0.84715E+00_JPRB ,&
&  0.86954E+00_JPRB, 0.90427E+00_JPRB, 0.87644E+00_JPRB, 0.84366E+00_JPRB, 0.89694E+00_JPRB, 0.94464E+00_JPRB, 0.63514E+00_JPRB ,&
&  0.55352E+00_JPRB, 0.52244E+00_JPRB, 0.53068E+00_JPRB, 0.52682E+00_JPRB, 0.57243E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.57700E+00_JPRB, 0.55579E+00_JPRB, 0.54462E+00_JPRB, 0.53669E+00_JPRB, 0.52118E+00_JPRB, 0.55139E+00_JPRB, 0.56269E+00_JPRB ,&
&  0.59691E+00_JPRB, 0.65197E+00_JPRB, 0.72215E+00_JPRB, 0.77383E+00_JPRB, 0.80111E+00_JPRB, 0.84801E+00_JPRB, 0.88885E+00_JPRB ,&
&  0.91917E+00_JPRB, 0.94295E+00_JPRB, 0.97541E+00_JPRB, 0.10170E+01_JPRB, 0.10870E+01_JPRB, 0.11930E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.15358E+01_JPRB, 0.17358E+01_JPRB, 0.19540E+01_JPRB, 0.21624E+01_JPRB, 0.23185E+01_JPRB, 0.24415E+01_JPRB, 0.23970E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.18739E+01_JPRB, 0.21609E+01_JPRB, 0.19852E+01_JPRB, 0.18357E+01_JPRB, 0.16893E+01_JPRB, 0.15410E+01_JPRB ,&
&  0.14072E+01_JPRB, 0.12789E+01_JPRB, 0.12235E+01_JPRB, 0.12105E+01_JPRB, 0.10933E+01_JPRB, 0.11354E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.14497E+01_JPRB, 0.18030E+01_JPRB, 0.24437E+01_JPRB, 0.31350E+01_JPRB, 0.35216E+01_JPRB, 0.39075E+01_JPRB, 0.46170E+01_JPRB ,&
&  0.57108E+01_JPRB, 0.65669E+01_JPRB, 0.71892E+01_JPRB, 0.53895E+01_JPRB, 0.45320E+01_JPRB, 0.43681E+01_JPRB, 0.39004E+01_JPRB ,&
&  0.22609E+01_JPRB, 0.15320E+01_JPRB, 0.15477E+01_JPRB, 0.15399E+01_JPRB, 0.13020E+01_JPRB, 0.11625E+01_JPRB, 0.10078E+01_JPRB ,&
&  0.83253E+00_JPRB, 0.72358E+00_JPRB, 0.64990E+00_JPRB, 0.68998E+00_JPRB, 0.73361E+00_JPRB, 0.75849E+00_JPRB, 0.71507E+00_JPRB ,&
&  0.74925E+00_JPRB, 0.72396E+00_JPRB, 0.70476E+00_JPRB, 0.70836E+00_JPRB, 0.77766E+00_JPRB, 0.81094E+00_JPRB, 0.82582E+00_JPRB ,&
&  0.84707E+00_JPRB, 0.86408E+00_JPRB, 0.86475E+00_JPRB, 0.87069E+00_JPRB, 0.87484E+00_JPRB, 0.86866E+00_JPRB, 0.86006E+00_JPRB ,&
&  0.84023E+00_JPRB, 0.82734E+00_JPRB, 0.78972E+00_JPRB, 0.74815E+00_JPRB, 0.69231E+00_JPRB, 0.69412E+00_JPRB, 0.63055E+00_JPRB ,&
&  0.58861E+00_JPRB, 0.56197E+00_JPRB, 0.54551E+00_JPRB, 0.54310E+00_JPRB, 0.55462E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.60202E+00_JPRB, 0.59696E+00_JPRB, 0.59011E+00_JPRB, 0.57803E+00_JPRB, 0.56213E+00_JPRB, 0.56122E+00_JPRB, 0.54735E+00_JPRB ,&
&  0.54843E+00_JPRB, 0.57834E+00_JPRB, 0.62597E+00_JPRB, 0.66413E+00_JPRB, 0.70002E+00_JPRB, 0.72787E+00_JPRB, 0.76205E+00_JPRB ,&
&  0.77691E+00_JPRB, 0.78056E+00_JPRB, 0.77864E+00_JPRB, 0.77997E+00_JPRB, 0.79772E+00_JPRB, 0.85595E+00_JPRB, 0.95583E+00_JPRB ,&
&  0.10993E+01_JPRB, 0.12889E+01_JPRB, 0.15282E+01_JPRB, 0.17960E+01_JPRB, 0.20756E+01_JPRB, 0.23554E+01_JPRB, 0.25068E+01_JPRB ,&
&  0.16130E+01_JPRB, 0.80059E+00_JPRB, 0.20091E+01_JPRB, 0.22229E+01_JPRB, 0.20183E+01_JPRB, 0.18884E+01_JPRB, 0.16616E+01_JPRB ,&
&  0.14505E+01_JPRB, 0.13648E+01_JPRB, 0.12516E+01_JPRB, 0.11727E+01_JPRB, 0.10649E+01_JPRB, 0.10605E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.11840E+01_JPRB, 0.14083E+01_JPRB, 0.18603E+01_JPRB, 0.26181E+01_JPRB, 0.31290E+01_JPRB, 0.33326E+01_JPRB, 0.37873E+01_JPRB ,&
&  0.47114E+01_JPRB, 0.55252E+01_JPRB, 0.58671E+01_JPRB, 0.39125E+01_JPRB, 0.26529E+01_JPRB, 0.23320E+01_JPRB, 0.21462E+01_JPRB ,&
&  0.20164E+01_JPRB, 0.16215E+01_JPRB, 0.15863E+01_JPRB, 0.16109E+01_JPRB, 0.14363E+01_JPRB, 0.12379E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.10191E+01_JPRB, 0.88249E+00_JPRB, 0.76897E+00_JPRB, 0.71296E+00_JPRB, 0.68096E+00_JPRB, 0.68756E+00_JPRB, 0.68028E+00_JPRB ,&
&  0.67774E+00_JPRB, 0.69584E+00_JPRB, 0.70193E+00_JPRB, 0.71887E+00_JPRB, 0.76774E+00_JPRB, 0.81576E+00_JPRB, 0.84608E+00_JPRB ,&
&  0.87123E+00_JPRB, 0.89770E+00_JPRB, 0.91228E+00_JPRB, 0.91612E+00_JPRB, 0.91498E+00_JPRB, 0.89510E+00_JPRB, 0.89811E+00_JPRB ,&
&  0.90046E+00_JPRB, 0.89774E+00_JPRB, 0.85992E+00_JPRB, 0.82870E+00_JPRB, 0.76155E+00_JPRB, 0.71262E+00_JPRB, 0.64338E+00_JPRB ,&
&  0.61567E+00_JPRB, 0.57552E+00_JPRB, 0.54926E+00_JPRB, 0.54920E+00_JPRB, 0.56628E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.61811E+00_JPRB, 0.61889E+00_JPRB, 0.62181E+00_JPRB, 0.62539E+00_JPRB, 0.62195E+00_JPRB, 0.62239E+00_JPRB, 0.59872E+00_JPRB ,&
&  0.56539E+00_JPRB, 0.55363E+00_JPRB, 0.57828E+00_JPRB, 0.58352E+00_JPRB, 0.58599E+00_JPRB, 0.61327E+00_JPRB, 0.64336E+00_JPRB ,&
&  0.66168E+00_JPRB, 0.67133E+00_JPRB, 0.67160E+00_JPRB, 0.67091E+00_JPRB, 0.65935E+00_JPRB, 0.66291E+00_JPRB, 0.69786E+00_JPRB ,&
&  0.76128E+00_JPRB, 0.87678E+00_JPRB, 0.10513E+01_JPRB, 0.12726E+01_JPRB, 0.15360E+01_JPRB, 0.19144E+01_JPRB, 0.22817E+01_JPRB ,&
&  0.29586E+01_JPRB, 0.19819E+01_JPRB, 0.11305E+01_JPRB, 0.20242E+01_JPRB, 0.22713E+01_JPRB, 0.22420E+01_JPRB, 0.18565E+01_JPRB ,&
&  0.17271E+01_JPRB, 0.16301E+01_JPRB, 0.15162E+01_JPRB, 0.13617E+01_JPRB, 0.11520E+01_JPRB, 0.10720E+01_JPRB, 0.10084E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.10982E+01_JPRB, 0.13489E+01_JPRB, 0.18702E+01_JPRB, 0.25395E+01_JPRB, 0.28897E+01_JPRB, 0.30912E+01_JPRB ,&
&  0.37176E+01_JPRB, 0.45430E+01_JPRB, 0.47476E+01_JPRB, 0.31117E+01_JPRB, 0.22262E+01_JPRB, 0.19978E+01_JPRB, 0.19441E+01_JPRB ,&
&  0.18461E+01_JPRB, 0.18379E+01_JPRB, 0.18636E+01_JPRB, 0.17018E+01_JPRB, 0.15057E+01_JPRB, 0.13599E+01_JPRB, 0.13515E+01_JPRB ,&
&  0.11906E+01_JPRB, 0.10534E+01_JPRB, 0.93581E+00_JPRB, 0.81916E+00_JPRB, 0.72451E+00_JPRB, 0.70143E+00_JPRB, 0.68992E+00_JPRB ,&
&  0.69621E+00_JPRB, 0.72049E+00_JPRB, 0.72501E+00_JPRB, 0.75046E+00_JPRB, 0.77729E+00_JPRB, 0.80453E+00_JPRB, 0.83077E+00_JPRB ,&
&  0.85516E+00_JPRB, 0.88297E+00_JPRB, 0.90927E+00_JPRB, 0.91489E+00_JPRB, 0.90586E+00_JPRB, 0.89176E+00_JPRB, 0.90642E+00_JPRB ,&
&  0.91491E+00_JPRB, 0.92909E+00_JPRB, 0.91614E+00_JPRB, 0.90709E+00_JPRB, 0.85233E+00_JPRB, 0.79939E+00_JPRB, 0.73902E+00_JPRB ,&
&  0.71995E+00_JPRB, 0.67334E+00_JPRB, 0.63882E+00_JPRB, 0.59965E+00_JPRB, 0.60624E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.64329E+00_JPRB, 0.63572E+00_JPRB, 0.63729E+00_JPRB, 0.64200E+00_JPRB, 0.64360E+00_JPRB, 0.64714E+00_JPRB, 0.64705E+00_JPRB ,&
&  0.61255E+00_JPRB, 0.57994E+00_JPRB, 0.56170E+00_JPRB, 0.54930E+00_JPRB, 0.54344E+00_JPRB, 0.55908E+00_JPRB, 0.56832E+00_JPRB ,&
&  0.56825E+00_JPRB, 0.57047E+00_JPRB, 0.59781E+00_JPRB, 0.60651E+00_JPRB, 0.60439E+00_JPRB, 0.59658E+00_JPRB, 0.59586E+00_JPRB ,&
&  0.61361E+00_JPRB, 0.65814E+00_JPRB, 0.74835E+00_JPRB, 0.87357E+00_JPRB, 0.10144E+01_JPRB, 0.12640E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.21380E+01_JPRB, 0.22032E+01_JPRB, 0.99317E+00_JPRB, 0.17479E+01_JPRB, 0.23825E+01_JPRB, 0.24466E+01_JPRB, 0.22540E+01_JPRB ,&
&  0.20785E+01_JPRB, 0.19228E+01_JPRB, 0.19589E+01_JPRB, 0.17739E+01_JPRB, 0.13903E+01_JPRB, 0.11596E+01_JPRB, 0.10049E+01_JPRB ,&
&  0.92266E+00_JPRB, 0.91703E+00_JPRB, 0.10093E+01_JPRB, 0.12650E+01_JPRB, 0.17864E+01_JPRB, 0.22721E+01_JPRB, 0.25139E+01_JPRB ,&
&  0.28100E+01_JPRB, 0.35772E+01_JPRB, 0.41174E+01_JPRB, 0.28130E+01_JPRB, 0.22250E+01_JPRB, 0.22628E+01_JPRB, 0.22309E+01_JPRB ,&
&  0.21096E+01_JPRB, 0.20401E+01_JPRB, 0.19766E+01_JPRB, 0.18119E+01_JPRB, 0.15992E+01_JPRB, 0.15019E+01_JPRB, 0.14962E+01_JPRB ,&
&  0.13476E+01_JPRB, 0.12127E+01_JPRB, 0.10888E+01_JPRB, 0.97022E+00_JPRB, 0.84504E+00_JPRB, 0.75472E+00_JPRB, 0.74122E+00_JPRB ,&
&  0.74112E+00_JPRB, 0.76482E+00_JPRB, 0.76749E+00_JPRB, 0.78331E+00_JPRB, 0.80052E+00_JPRB, 0.81712E+00_JPRB, 0.82912E+00_JPRB ,&
&  0.84311E+00_JPRB, 0.86340E+00_JPRB, 0.88682E+00_JPRB, 0.89076E+00_JPRB, 0.88943E+00_JPRB, 0.88494E+00_JPRB, 0.88859E+00_JPRB ,&
&  0.89923E+00_JPRB, 0.91197E+00_JPRB, 0.91160E+00_JPRB, 0.90959E+00_JPRB, 0.91815E+00_JPRB, 0.89283E+00_JPRB, 0.85657E+00_JPRB ,&
&  0.88378E+00_JPRB, 0.91049E+00_JPRB, 0.84178E+00_JPRB, 0.69401E+00_JPRB, 0.66115E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.69130E+00_JPRB, 0.67675E+00_JPRB, 0.66709E+00_JPRB, 0.66616E+00_JPRB, 0.66425E+00_JPRB, 0.66527E+00_JPRB, 0.67491E+00_JPRB ,&
&  0.66224E+00_JPRB, 0.63840E+00_JPRB, 0.61018E+00_JPRB, 0.57972E+00_JPRB, 0.55695E+00_JPRB, 0.55807E+00_JPRB, 0.56501E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.56234E+00_JPRB, 0.56331E+00_JPRB, 0.57224E+00_JPRB, 0.57145E+00_JPRB, 0.56559E+00_JPRB, 0.55923E+00_JPRB ,&
&  0.55816E+00_JPRB, 0.56840E+00_JPRB, 0.60271E+00_JPRB, 0.66362E+00_JPRB, 0.73987E+00_JPRB, 0.86678E+00_JPRB, 0.10554E+01_JPRB ,&
&  0.14853E+01_JPRB, 0.17689E+01_JPRB, 0.18901E+01_JPRB, 0.28388E+01_JPRB, 0.29838E+01_JPRB, 0.28037E+01_JPRB, 0.27479E+01_JPRB ,&
&  0.26382E+01_JPRB, 0.24599E+01_JPRB, 0.25997E+01_JPRB, 0.20697E+01_JPRB, 0.16410E+01_JPRB, 0.12903E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.93911E+00_JPRB, 0.86432E+00_JPRB, 0.85675E+00_JPRB, 0.92325E+00_JPRB, 0.11814E+01_JPRB, 0.16360E+01_JPRB, 0.19974E+01_JPRB ,&
&  0.22596E+01_JPRB, 0.27671E+01_JPRB, 0.35275E+01_JPRB, 0.29610E+01_JPRB, 0.23779E+01_JPRB, 0.27390E+01_JPRB, 0.31007E+01_JPRB ,&
&  0.35053E+01_JPRB, 0.25530E+01_JPRB, 0.21854E+01_JPRB, 0.19645E+01_JPRB, 0.16960E+01_JPRB, 0.18293E+01_JPRB, 0.16206E+01_JPRB ,&
&  0.14700E+01_JPRB, 0.13563E+01_JPRB, 0.12367E+01_JPRB, 0.11208E+01_JPRB, 0.10060E+01_JPRB, 0.89955E+00_JPRB, 0.84190E+00_JPRB ,&
&  0.83035E+00_JPRB, 0.83465E+00_JPRB, 0.83011E+00_JPRB, 0.83897E+00_JPRB, 0.84111E+00_JPRB, 0.85165E+00_JPRB, 0.85970E+00_JPRB ,&
&  0.86722E+00_JPRB, 0.87040E+00_JPRB, 0.85603E+00_JPRB, 0.86290E+00_JPRB, 0.86732E+00_JPRB, 0.86380E+00_JPRB, 0.85575E+00_JPRB ,&
&  0.86238E+00_JPRB, 0.87776E+00_JPRB, 0.88673E+00_JPRB, 0.89272E+00_JPRB, 0.90364E+00_JPRB, 0.89161E+00_JPRB, 0.86019E+00_JPRB ,&
&  0.84161E+00_JPRB, 0.83896E+00_JPRB, 0.82152E+00_JPRB, 0.75196E+00_JPRB, 0.71697E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.72076E+00_JPRB, 0.70670E+00_JPRB, 0.69707E+00_JPRB, 0.68678E+00_JPRB, 0.67475E+00_JPRB, 0.67314E+00_JPRB, 0.68217E+00_JPRB ,&
&  0.68097E+00_JPRB, 0.66630E+00_JPRB, 0.64479E+00_JPRB, 0.62286E+00_JPRB, 0.59592E+00_JPRB, 0.58105E+00_JPRB, 0.57157E+00_JPRB ,&
&  0.56508E+00_JPRB, 0.56255E+00_JPRB, 0.55693E+00_JPRB, 0.55606E+00_JPRB, 0.55494E+00_JPRB, 0.54298E+00_JPRB, 0.53396E+00_JPRB ,&
&  0.52934E+00_JPRB, 0.53407E+00_JPRB, 0.54825E+00_JPRB, 0.57985E+00_JPRB, 0.62820E+00_JPRB, 0.71017E+00_JPRB, 0.81686E+00_JPRB ,&
&  0.10879E+01_JPRB, 0.15072E+01_JPRB, 0.13930E+01_JPRB, 0.29862E+01_JPRB, 0.29588E+01_JPRB, 0.28994E+01_JPRB, 0.27328E+01_JPRB ,&
&  0.27141E+01_JPRB, 0.30019E+01_JPRB, 0.26299E+01_JPRB, 0.21820E+01_JPRB, 0.17171E+01_JPRB, 0.13817E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.97715E+00_JPRB, 0.88612E+00_JPRB, 0.84335E+00_JPRB, 0.82605E+00_JPRB, 0.85774E+00_JPRB, 0.11118E+01_JPRB, 0.15164E+01_JPRB ,&
&  0.18211E+01_JPRB, 0.21867E+01_JPRB, 0.28838E+01_JPRB, 0.27632E+01_JPRB, 0.25780E+01_JPRB, 0.30830E+01_JPRB, 0.40594E+01_JPRB ,&
&  0.48993E+01_JPRB, 0.36718E+01_JPRB, 0.26536E+01_JPRB, 0.22344E+01_JPRB, 0.20072E+01_JPRB, 0.19483E+01_JPRB, 0.17611E+01_JPRB ,&
&  0.15598E+01_JPRB, 0.14495E+01_JPRB, 0.13556E+01_JPRB, 0.12614E+01_JPRB, 0.11585E+01_JPRB, 0.10630E+01_JPRB, 0.97611E+00_JPRB ,&
&  0.92932E+00_JPRB, 0.91183E+00_JPRB, 0.90327E+00_JPRB, 0.91081E+00_JPRB, 0.91620E+00_JPRB, 0.92899E+00_JPRB, 0.92957E+00_JPRB ,&
&  0.93086E+00_JPRB, 0.91338E+00_JPRB, 0.86145E+00_JPRB, 0.84800E+00_JPRB, 0.84907E+00_JPRB, 0.84300E+00_JPRB, 0.83070E+00_JPRB ,&
&  0.84120E+00_JPRB, 0.85298E+00_JPRB, 0.85290E+00_JPRB, 0.85249E+00_JPRB, 0.87288E+00_JPRB, 0.88090E+00_JPRB, 0.86536E+00_JPRB ,&
&  0.83486E+00_JPRB, 0.80793E+00_JPRB, 0.78346E+00_JPRB, 0.75291E+00_JPRB, 0.73867E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.73214E+00_JPRB, 0.72145E+00_JPRB, 0.71114E+00_JPRB, 0.69758E+00_JPRB, 0.67914E+00_JPRB, 0.67106E+00_JPRB, 0.67649E+00_JPRB ,&
&  0.68315E+00_JPRB, 0.67448E+00_JPRB, 0.65860E+00_JPRB, 0.63740E+00_JPRB, 0.61111E+00_JPRB, 0.59614E+00_JPRB, 0.58551E+00_JPRB ,&
&  0.57023E+00_JPRB, 0.55775E+00_JPRB, 0.54221E+00_JPRB, 0.53647E+00_JPRB, 0.53652E+00_JPRB, 0.52809E+00_JPRB, 0.51138E+00_JPRB ,&
&  0.51216E+00_JPRB, 0.52176E+00_JPRB, 0.53389E+00_JPRB, 0.55002E+00_JPRB, 0.58459E+00_JPRB, 0.64655E+00_JPRB, 0.74228E+00_JPRB ,&
&  0.93998E+00_JPRB, 0.17573E+01_JPRB, 0.12911E+01_JPRB, 0.23189E+01_JPRB, 0.23881E+01_JPRB, 0.25554E+01_JPRB, 0.26061E+01_JPRB ,&
&  0.26743E+01_JPRB, 0.27184E+01_JPRB, 0.25121E+01_JPRB, 0.20613E+01_JPRB, 0.16205E+01_JPRB, 0.13356E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.10356E+01_JPRB, 0.94434E+00_JPRB, 0.87947E+00_JPRB, 0.83653E+00_JPRB, 0.80910E+00_JPRB, 0.83970E+00_JPRB, 0.10491E+01_JPRB ,&
&  0.14024E+01_JPRB, 0.17233E+01_JPRB, 0.22461E+01_JPRB, 0.28059E+01_JPRB, 0.25883E+01_JPRB, 0.29429E+01_JPRB, 0.43977E+01_JPRB ,&
&  0.53115E+01_JPRB, 0.52928E+01_JPRB, 0.34969E+01_JPRB, 0.26976E+01_JPRB, 0.23602E+01_JPRB, 0.21661E+01_JPRB, 0.18785E+01_JPRB ,&
&  0.16147E+01_JPRB, 0.14822E+01_JPRB, 0.14116E+01_JPRB, 0.13506E+01_JPRB, 0.12730E+01_JPRB, 0.11655E+01_JPRB, 0.10612E+01_JPRB ,&
&  0.97544E+00_JPRB, 0.94825E+00_JPRB, 0.94371E+00_JPRB, 0.95101E+00_JPRB, 0.96920E+00_JPRB, 0.98928E+00_JPRB, 0.10019E+01_JPRB ,&
&  0.99677E+00_JPRB, 0.97828E+00_JPRB, 0.90247E+00_JPRB, 0.86497E+00_JPRB, 0.85406E+00_JPRB, 0.84319E+00_JPRB, 0.82866E+00_JPRB ,&
&  0.83452E+00_JPRB, 0.83818E+00_JPRB, 0.83331E+00_JPRB, 0.83800E+00_JPRB, 0.85317E+00_JPRB, 0.87491E+00_JPRB, 0.83845E+00_JPRB ,&
&  0.83385E+00_JPRB, 0.81317E+00_JPRB, 0.78843E+00_JPRB, 0.76062E+00_JPRB, 0.74684E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.73028E+00_JPRB, 0.72107E+00_JPRB, 0.71067E+00_JPRB, 0.69849E+00_JPRB, 0.68455E+00_JPRB, 0.67503E+00_JPRB, 0.67297E+00_JPRB ,&
&  0.67277E+00_JPRB, 0.67273E+00_JPRB, 0.66300E+00_JPRB, 0.64049E+00_JPRB, 0.61703E+00_JPRB, 0.59605E+00_JPRB, 0.58361E+00_JPRB ,&
&  0.57301E+00_JPRB, 0.55983E+00_JPRB, 0.54425E+00_JPRB, 0.52955E+00_JPRB, 0.52510E+00_JPRB, 0.52460E+00_JPRB, 0.51489E+00_JPRB ,&
&  0.52016E+00_JPRB, 0.52888E+00_JPRB, 0.53644E+00_JPRB, 0.54446E+00_JPRB, 0.57156E+00_JPRB, 0.62438E+00_JPRB, 0.71279E+00_JPRB ,&
&  0.88481E+00_JPRB, 0.76938E+00_JPRB, 0.14475E+01_JPRB, 0.17573E+01_JPRB, 0.19205E+01_JPRB, 0.20653E+01_JPRB, 0.23743E+01_JPRB ,&
&  0.26128E+01_JPRB, 0.25253E+01_JPRB, 0.22576E+01_JPRB, 0.18907E+01_JPRB, 0.15589E+01_JPRB, 0.13160E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.97625E+00_JPRB, 0.90748E+00_JPRB, 0.84869E+00_JPRB, 0.80965E+00_JPRB, 0.79028E+00_JPRB, 0.81800E+00_JPRB ,&
&  0.95964E+00_JPRB, 0.12212E+01_JPRB, 0.15803E+01_JPRB, 0.22143E+01_JPRB, 0.25576E+01_JPRB, 0.27937E+01_JPRB, 0.33480E+01_JPRB ,&
&  0.45115E+01_JPRB, 0.52263E+01_JPRB, 0.40911E+01_JPRB, 0.31831E+01_JPRB, 0.26860E+01_JPRB, 0.22816E+01_JPRB, 0.19362E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.15000E+01_JPRB, 0.14027E+01_JPRB, 0.13456E+01_JPRB, 0.13076E+01_JPRB, 0.12512E+01_JPRB, 0.11729E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.10491E+01_JPRB, 0.10286E+01_JPRB, 0.10253E+01_JPRB, 0.10291E+01_JPRB, 0.10450E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.99488E+00_JPRB, 0.93951E+00_JPRB, 0.89437E+00_JPRB, 0.87717E+00_JPRB, 0.85853E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.82534E+00_JPRB, 0.82522E+00_JPRB, 0.82646E+00_JPRB, 0.83700E+00_JPRB, 0.83703E+00_JPRB, 0.85952E+00_JPRB, 0.82168E+00_JPRB ,&
&  0.81122E+00_JPRB, 0.79102E+00_JPRB, 0.77063E+00_JPRB, 0.75100E+00_JPRB, 0.74343E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.72949E+00_JPRB, 0.71957E+00_JPRB, 0.70700E+00_JPRB, 0.69331E+00_JPRB, 0.67855E+00_JPRB, 0.66768E+00_JPRB, 0.66297E+00_JPRB ,&
&  0.66000E+00_JPRB, 0.65880E+00_JPRB, 0.65787E+00_JPRB, 0.64200E+00_JPRB, 0.62189E+00_JPRB, 0.60323E+00_JPRB, 0.59176E+00_JPRB ,&
&  0.58391E+00_JPRB, 0.57478E+00_JPRB, 0.56293E+00_JPRB, 0.54800E+00_JPRB, 0.54004E+00_JPRB, 0.53427E+00_JPRB, 0.52973E+00_JPRB ,&
&  0.53457E+00_JPRB, 0.53839E+00_JPRB, 0.54355E+00_JPRB, 0.55207E+00_JPRB, 0.57451E+00_JPRB, 0.62193E+00_JPRB, 0.70272E+00_JPRB ,&
&  0.88217E+00_JPRB, 0.12980E+01_JPRB, 0.16664E+01_JPRB, 0.15614E+01_JPRB, 0.15673E+01_JPRB, 0.16170E+01_JPRB, 0.18978E+01_JPRB ,&
&  0.20944E+01_JPRB, 0.21542E+01_JPRB, 0.19620E+01_JPRB, 0.17020E+01_JPRB, 0.14720E+01_JPRB, 0.12925E+01_JPRB, 0.11624E+01_JPRB ,&
&  0.10427E+01_JPRB, 0.93717E+00_JPRB, 0.86048E+00_JPRB, 0.80669E+00_JPRB, 0.77205E+00_JPRB, 0.75058E+00_JPRB, 0.74332E+00_JPRB ,&
&  0.76517E+00_JPRB, 0.85402E+00_JPRB, 0.10613E+01_JPRB, 0.14890E+01_JPRB, 0.24242E+01_JPRB, 0.24495E+01_JPRB, 0.30841E+01_JPRB ,&
&  0.42518E+01_JPRB, 0.41533E+01_JPRB, 0.37072E+01_JPRB, 0.31427E+01_JPRB, 0.26728E+01_JPRB, 0.22922E+01_JPRB, 0.19856E+01_JPRB ,&
&  0.17580E+01_JPRB, 0.15932E+01_JPRB, 0.14555E+01_JPRB, 0.13537E+01_JPRB, 0.12873E+01_JPRB, 0.12576E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.11530E+01_JPRB, 0.11221E+01_JPRB, 0.11062E+01_JPRB, 0.10908E+01_JPRB, 0.10786E+01_JPRB, 0.10445E+01_JPRB ,&
&  0.10056E+01_JPRB, 0.96459E+00_JPRB, 0.92750E+00_JPRB, 0.88937E+00_JPRB, 0.87772E+00_JPRB, 0.86211E+00_JPRB, 0.83445E+00_JPRB ,&
&  0.80933E+00_JPRB, 0.80360E+00_JPRB, 0.81765E+00_JPRB, 0.82291E+00_JPRB, 0.83389E+00_JPRB, 0.85571E+00_JPRB, 0.81731E+00_JPRB ,&
&  0.79863E+00_JPRB, 0.77006E+00_JPRB, 0.74631E+00_JPRB, 0.73649E+00_JPRB, 0.74068E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.71632E+00_JPRB, 0.70302E+00_JPRB, 0.69018E+00_JPRB, 0.67670E+00_JPRB, 0.66143E+00_JPRB, 0.65099E+00_JPRB, 0.64252E+00_JPRB ,&
&  0.63665E+00_JPRB, 0.63343E+00_JPRB, 0.63421E+00_JPRB, 0.63448E+00_JPRB, 0.63032E+00_JPRB, 0.61962E+00_JPRB, 0.60230E+00_JPRB ,&
&  0.59103E+00_JPRB, 0.58049E+00_JPRB, 0.56713E+00_JPRB, 0.55083E+00_JPRB, 0.53511E+00_JPRB, 0.52699E+00_JPRB, 0.52493E+00_JPRB ,&
&  0.52873E+00_JPRB, 0.53318E+00_JPRB, 0.53631E+00_JPRB, 0.54795E+00_JPRB, 0.56978E+00_JPRB, 0.61547E+00_JPRB, 0.68220E+00_JPRB ,&
&  0.81106E+00_JPRB, 0.12303E+01_JPRB, 0.13426E+01_JPRB, 0.13434E+01_JPRB, 0.13730E+01_JPRB, 0.13866E+01_JPRB, 0.15231E+01_JPRB ,&
&  0.16546E+01_JPRB, 0.17383E+01_JPRB, 0.16865E+01_JPRB, 0.14983E+01_JPRB, 0.13357E+01_JPRB, 0.12225E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.91604E+00_JPRB, 0.84674E+00_JPRB, 0.79955E+00_JPRB, 0.76354E+00_JPRB, 0.73639E+00_JPRB, 0.72064E+00_JPRB ,&
&  0.71689E+00_JPRB, 0.72900E+00_JPRB, 0.78703E+00_JPRB, 0.93165E+00_JPRB, 0.15198E+01_JPRB, 0.20554E+01_JPRB, 0.25250E+01_JPRB ,&
&  0.27196E+01_JPRB, 0.28038E+01_JPRB, 0.28310E+01_JPRB, 0.26295E+01_JPRB, 0.23755E+01_JPRB, 0.20887E+01_JPRB, 0.18838E+01_JPRB ,&
&  0.17441E+01_JPRB, 0.15905E+01_JPRB, 0.14528E+01_JPRB, 0.13285E+01_JPRB, 0.12393E+01_JPRB, 0.11874E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.10961E+01_JPRB, 0.10728E+01_JPRB, 0.10530E+01_JPRB, 0.10311E+01_JPRB, 0.10045E+01_JPRB, 0.97060E+00_JPRB ,&
&  0.93116E+00_JPRB, 0.89221E+00_JPRB, 0.86457E+00_JPRB, 0.84630E+00_JPRB, 0.83285E+00_JPRB, 0.81452E+00_JPRB, 0.79311E+00_JPRB ,&
&  0.78635E+00_JPRB, 0.78922E+00_JPRB, 0.79645E+00_JPRB, 0.80164E+00_JPRB, 0.80455E+00_JPRB, 0.82424E+00_JPRB, 0.79529E+00_JPRB ,&
&  0.76807E+00_JPRB, 0.74091E+00_JPRB, 0.72146E+00_JPRB, 0.71117E+00_JPRB, 0.72453E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.70341E+00_JPRB, 0.68538E+00_JPRB, 0.66653E+00_JPRB, 0.64547E+00_JPRB, 0.62855E+00_JPRB, 0.61895E+00_JPRB, 0.61248E+00_JPRB ,&
&  0.60930E+00_JPRB, 0.60844E+00_JPRB, 0.60744E+00_JPRB, 0.60538E+00_JPRB, 0.60193E+00_JPRB, 0.59727E+00_JPRB, 0.58819E+00_JPRB ,&
&  0.57698E+00_JPRB, 0.56316E+00_JPRB, 0.55139E+00_JPRB, 0.54134E+00_JPRB, 0.52415E+00_JPRB, 0.51280E+00_JPRB, 0.51290E+00_JPRB ,&
&  0.51719E+00_JPRB, 0.52189E+00_JPRB, 0.52589E+00_JPRB, 0.53635E+00_JPRB, 0.56161E+00_JPRB, 0.60766E+00_JPRB, 0.66131E+00_JPRB ,&
&  0.77221E+00_JPRB, 0.90846E+00_JPRB, 0.10379E+01_JPRB, 0.10662E+01_JPRB, 0.10849E+01_JPRB, 0.11406E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.12715E+01_JPRB, 0.13759E+01_JPRB, 0.13592E+01_JPRB, 0.12363E+01_JPRB, 0.11053E+01_JPRB, 0.10107E+01_JPRB, 0.95042E+00_JPRB ,&
&  0.90516E+00_JPRB, 0.86388E+00_JPRB, 0.82880E+00_JPRB, 0.80322E+00_JPRB, 0.77897E+00_JPRB, 0.75439E+00_JPRB, 0.73055E+00_JPRB ,&
&  0.71445E+00_JPRB, 0.70435E+00_JPRB, 0.70574E+00_JPRB, 0.73704E+00_JPRB, 0.87096E+00_JPRB, 0.11386E+01_JPRB, 0.15068E+01_JPRB ,&
&  0.18068E+01_JPRB, 0.19834E+01_JPRB, 0.21239E+01_JPRB, 0.21228E+01_JPRB, 0.20308E+01_JPRB, 0.19028E+01_JPRB, 0.17714E+01_JPRB ,&
&  0.16093E+01_JPRB, 0.14201E+01_JPRB, 0.12766E+01_JPRB, 0.11723E+01_JPRB, 0.10973E+01_JPRB, 0.10469E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.99037E+00_JPRB, 0.98283E+00_JPRB, 0.10035E+01_JPRB, 0.10240E+01_JPRB, 0.10211E+01_JPRB, 0.98257E+00_JPRB, 0.92737E+00_JPRB ,&
&  0.87748E+00_JPRB, 0.83905E+00_JPRB, 0.80738E+00_JPRB, 0.78278E+00_JPRB, 0.76316E+00_JPRB, 0.74801E+00_JPRB, 0.73232E+00_JPRB ,&
&  0.72270E+00_JPRB, 0.73162E+00_JPRB, 0.72946E+00_JPRB, 0.74363E+00_JPRB, 0.77866E+00_JPRB, 0.76145E+00_JPRB, 0.73365E+00_JPRB ,&
&  0.71186E+00_JPRB, 0.69823E+00_JPRB, 0.68571E+00_JPRB, 0.67503E+00_JPRB, 0.72057E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.61837E+00_JPRB, 0.61392E+00_JPRB, 0.60261E+00_JPRB, 0.59147E+00_JPRB, 0.58337E+00_JPRB, 0.58007E+00_JPRB, 0.57724E+00_JPRB ,&
&  0.57571E+00_JPRB, 0.57514E+00_JPRB, 0.57479E+00_JPRB, 0.57202E+00_JPRB, 0.56644E+00_JPRB, 0.55819E+00_JPRB, 0.54803E+00_JPRB ,&
&  0.53862E+00_JPRB, 0.52914E+00_JPRB, 0.52063E+00_JPRB, 0.51185E+00_JPRB, 0.50085E+00_JPRB, 0.49421E+00_JPRB, 0.49613E+00_JPRB ,&
&  0.50216E+00_JPRB, 0.50906E+00_JPRB, 0.51523E+00_JPRB, 0.52482E+00_JPRB, 0.55473E+00_JPRB, 0.59701E+00_JPRB, 0.63651E+00_JPRB ,&
&  0.73238E+00_JPRB, 0.76024E+00_JPRB, 0.81920E+00_JPRB, 0.88270E+00_JPRB, 0.94939E+00_JPRB, 0.97345E+00_JPRB, 0.99712E+00_JPRB ,&
&  0.10260E+01_JPRB, 0.10640E+01_JPRB, 0.10642E+01_JPRB, 0.10156E+01_JPRB, 0.91999E+00_JPRB, 0.82844E+00_JPRB, 0.77315E+00_JPRB ,&
&  0.74583E+00_JPRB, 0.73297E+00_JPRB, 0.72117E+00_JPRB, 0.70999E+00_JPRB, 0.70063E+00_JPRB, 0.69139E+00_JPRB, 0.68184E+00_JPRB ,&
&  0.67277E+00_JPRB, 0.66558E+00_JPRB, 0.66143E+00_JPRB, 0.66204E+00_JPRB, 0.68416E+00_JPRB, 0.76432E+00_JPRB, 0.94895E+00_JPRB ,&
&  0.12076E+01_JPRB, 0.14497E+01_JPRB, 0.15855E+01_JPRB, 0.16658E+01_JPRB, 0.17032E+01_JPRB, 0.16294E+01_JPRB, 0.14547E+01_JPRB ,&
&  0.12910E+01_JPRB, 0.11482E+01_JPRB, 0.10397E+01_JPRB, 0.95839E+00_JPRB, 0.89942E+00_JPRB, 0.85975E+00_JPRB, 0.83715E+00_JPRB ,&
&  0.82878E+00_JPRB, 0.82947E+00_JPRB, 0.83908E+00_JPRB, 0.84843E+00_JPRB, 0.85229E+00_JPRB, 0.84330E+00_JPRB, 0.82292E+00_JPRB ,&
&  0.79389E+00_JPRB, 0.77239E+00_JPRB, 0.75066E+00_JPRB, 0.72615E+00_JPRB, 0.70415E+00_JPRB, 0.68366E+00_JPRB, 0.66749E+00_JPRB ,&
&  0.65743E+00_JPRB, 0.65922E+00_JPRB, 0.66949E+00_JPRB, 0.69297E+00_JPRB, 0.68149E+00_JPRB, 0.66391E+00_JPRB, 0.64813E+00_JPRB ,&
&  0.64099E+00_JPRB, 0.63869E+00_JPRB, 0.62881E+00_JPRB, 0.61932E+00_JPRB, 0.61643E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.57194E+00_JPRB, 0.56760E+00_JPRB, 0.55847E+00_JPRB, 0.55056E+00_JPRB, 0.54793E+00_JPRB, 0.54833E+00_JPRB, 0.54984E+00_JPRB ,&
&  0.54908E+00_JPRB, 0.54583E+00_JPRB, 0.54102E+00_JPRB, 0.53733E+00_JPRB, 0.53212E+00_JPRB, 0.52498E+00_JPRB, 0.51618E+00_JPRB ,&
&  0.50759E+00_JPRB, 0.49977E+00_JPRB, 0.49360E+00_JPRB, 0.48717E+00_JPRB, 0.48230E+00_JPRB, 0.48171E+00_JPRB, 0.48330E+00_JPRB ,&
&  0.48982E+00_JPRB, 0.49828E+00_JPRB, 0.50524E+00_JPRB, 0.52175E+00_JPRB, 0.54327E+00_JPRB, 0.57741E+00_JPRB, 0.61545E+00_JPRB ,&
&  0.69514E+00_JPRB, 0.64193E+00_JPRB, 0.68782E+00_JPRB, 0.78073E+00_JPRB, 0.84235E+00_JPRB, 0.86562E+00_JPRB, 0.84410E+00_JPRB ,&
&  0.83144E+00_JPRB, 0.83542E+00_JPRB, 0.83681E+00_JPRB, 0.83032E+00_JPRB, 0.79902E+00_JPRB, 0.73461E+00_JPRB, 0.67057E+00_JPRB ,&
&  0.62715E+00_JPRB, 0.61154E+00_JPRB, 0.60433E+00_JPRB, 0.59420E+00_JPRB, 0.58389E+00_JPRB, 0.57815E+00_JPRB, 0.57714E+00_JPRB ,&
&  0.57623E+00_JPRB, 0.57469E+00_JPRB, 0.57197E+00_JPRB, 0.56887E+00_JPRB, 0.56965E+00_JPRB, 0.58824E+00_JPRB, 0.65536E+00_JPRB ,&
&  0.79196E+00_JPRB, 0.95505E+00_JPRB, 0.11095E+01_JPRB, 0.12277E+01_JPRB, 0.12548E+01_JPRB, 0.12217E+01_JPRB, 0.11558E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.98884E+00_JPRB, 0.91891E+00_JPRB, 0.86137E+00_JPRB, 0.81460E+00_JPRB, 0.77769E+00_JPRB, 0.75050E+00_JPRB ,&
&  0.73601E+00_JPRB, 0.72671E+00_JPRB, 0.71992E+00_JPRB, 0.71071E+00_JPRB, 0.70109E+00_JPRB, 0.69467E+00_JPRB, 0.69155E+00_JPRB ,&
&  0.68489E+00_JPRB, 0.67680E+00_JPRB, 0.66946E+00_JPRB, 0.66078E+00_JPRB, 0.65119E+00_JPRB, 0.64170E+00_JPRB, 0.63309E+00_JPRB ,&
&  0.62744E+00_JPRB, 0.62371E+00_JPRB, 0.62243E+00_JPRB, 0.61960E+00_JPRB, 0.62723E+00_JPRB, 0.61258E+00_JPRB, 0.60105E+00_JPRB ,&
&  0.59578E+00_JPRB, 0.58926E+00_JPRB, 0.58496E+00_JPRB, 0.54385E+00_JPRB, 0.56971E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.53044E+00_JPRB, 0.53185E+00_JPRB, 0.53009E+00_JPRB, 0.52425E+00_JPRB, 0.52037E+00_JPRB, 0.51824E+00_JPRB, 0.51780E+00_JPRB ,&
&  0.51796E+00_JPRB, 0.51684E+00_JPRB, 0.51347E+00_JPRB, 0.50736E+00_JPRB, 0.50192E+00_JPRB, 0.49519E+00_JPRB, 0.48816E+00_JPRB ,&
&  0.48211E+00_JPRB, 0.47684E+00_JPRB, 0.47367E+00_JPRB, 0.46990E+00_JPRB, 0.46862E+00_JPRB, 0.47019E+00_JPRB, 0.47229E+00_JPRB ,&
&  0.47560E+00_JPRB, 0.47859E+00_JPRB, 0.48810E+00_JPRB, 0.50278E+00_JPRB, 0.52960E+00_JPRB, 0.54872E+00_JPRB, 0.58248E+00_JPRB ,&
&  0.65671E+00_JPRB, 0.57263E+00_JPRB, 0.62928E+00_JPRB, 0.70028E+00_JPRB, 0.74737E+00_JPRB, 0.77314E+00_JPRB, 0.73192E+00_JPRB ,&
&  0.69937E+00_JPRB, 0.68021E+00_JPRB, 0.67226E+00_JPRB, 0.66879E+00_JPRB, 0.66589E+00_JPRB, 0.64805E+00_JPRB, 0.62453E+00_JPRB ,&
&  0.59939E+00_JPRB, 0.57907E+00_JPRB, 0.56186E+00_JPRB, 0.54402E+00_JPRB, 0.52814E+00_JPRB, 0.51784E+00_JPRB, 0.51280E+00_JPRB ,&
&  0.51146E+00_JPRB, 0.51137E+00_JPRB, 0.51094E+00_JPRB, 0.51127E+00_JPRB, 0.51311E+00_JPRB, 0.51769E+00_JPRB, 0.52908E+00_JPRB ,&
&  0.56798E+00_JPRB, 0.64088E+00_JPRB, 0.72731E+00_JPRB, 0.81159E+00_JPRB, 0.87129E+00_JPRB, 0.90037E+00_JPRB, 0.91387E+00_JPRB ,&
&  0.90287E+00_JPRB, 0.87244E+00_JPRB, 0.83257E+00_JPRB, 0.79160E+00_JPRB, 0.75446E+00_JPRB, 0.72457E+00_JPRB, 0.69840E+00_JPRB ,&
&  0.67480E+00_JPRB, 0.65140E+00_JPRB, 0.63127E+00_JPRB, 0.61548E+00_JPRB, 0.60111E+00_JPRB, 0.58853E+00_JPRB, 0.58140E+00_JPRB ,&
&  0.57846E+00_JPRB, 0.57804E+00_JPRB, 0.57725E+00_JPRB, 0.57814E+00_JPRB, 0.57953E+00_JPRB, 0.58054E+00_JPRB, 0.57940E+00_JPRB ,&
&  0.57498E+00_JPRB, 0.57211E+00_JPRB, 0.56849E+00_JPRB, 0.57368E+00_JPRB, 0.56965E+00_JPRB, 0.56513E+00_JPRB, 0.55891E+00_JPRB ,&
&  0.55446E+00_JPRB, 0.54594E+00_JPRB, 0.49165E+00_JPRB, 0.51598E+00_JPRB, 0.52485E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.50398E+00_JPRB, 0.50414E+00_JPRB, 0.50559E+00_JPRB, 0.50554E+00_JPRB, 0.50461E+00_JPRB, 0.50275E+00_JPRB, 0.49937E+00_JPRB ,&
&  0.49434E+00_JPRB, 0.49079E+00_JPRB, 0.48779E+00_JPRB, 0.48378E+00_JPRB, 0.47928E+00_JPRB, 0.47551E+00_JPRB, 0.47214E+00_JPRB ,&
&  0.46863E+00_JPRB, 0.46482E+00_JPRB, 0.46207E+00_JPRB, 0.46014E+00_JPRB, 0.45812E+00_JPRB, 0.45737E+00_JPRB, 0.45840E+00_JPRB ,&
&  0.45824E+00_JPRB, 0.46127E+00_JPRB, 0.47330E+00_JPRB, 0.48908E+00_JPRB, 0.51115E+00_JPRB, 0.53225E+00_JPRB, 0.55354E+00_JPRB ,&
&  0.54892E+00_JPRB, 0.49896E+00_JPRB, 0.54097E+00_JPRB, 0.59573E+00_JPRB, 0.65049E+00_JPRB, 0.68738E+00_JPRB, 0.65897E+00_JPRB ,&
&  0.62020E+00_JPRB, 0.58790E+00_JPRB, 0.56869E+00_JPRB, 0.55882E+00_JPRB, 0.55927E+00_JPRB, 0.55924E+00_JPRB, 0.55833E+00_JPRB ,&
&  0.55265E+00_JPRB, 0.54029E+00_JPRB, 0.52591E+00_JPRB, 0.51217E+00_JPRB, 0.50200E+00_JPRB, 0.49403E+00_JPRB, 0.48743E+00_JPRB ,&
&  0.48391E+00_JPRB, 0.48269E+00_JPRB, 0.48217E+00_JPRB, 0.48214E+00_JPRB, 0.48288E+00_JPRB, 0.48578E+00_JPRB, 0.48929E+00_JPRB ,&
&  0.49431E+00_JPRB, 0.51036E+00_JPRB, 0.54596E+00_JPRB, 0.59194E+00_JPRB, 0.63934E+00_JPRB, 0.67642E+00_JPRB, 0.70372E+00_JPRB ,&
&  0.72437E+00_JPRB, 0.73537E+00_JPRB, 0.73195E+00_JPRB, 0.71353E+00_JPRB, 0.68819E+00_JPRB, 0.66327E+00_JPRB, 0.63902E+00_JPRB ,&
&  0.61452E+00_JPRB, 0.58645E+00_JPRB, 0.55861E+00_JPRB, 0.54021E+00_JPRB, 0.52931E+00_JPRB, 0.51937E+00_JPRB, 0.51167E+00_JPRB ,&
&  0.50856E+00_JPRB, 0.50900E+00_JPRB, 0.51135E+00_JPRB, 0.51345E+00_JPRB, 0.51549E+00_JPRB, 0.51851E+00_JPRB, 0.52071E+00_JPRB ,&
&  0.52174E+00_JPRB, 0.52392E+00_JPRB, 0.52787E+00_JPRB, 0.53711E+00_JPRB, 0.54275E+00_JPRB, 0.54414E+00_JPRB, 0.54082E+00_JPRB ,&
&  0.53174E+00_JPRB, 0.52030E+00_JPRB, 0.50899E+00_JPRB, 0.50262E+00_JPRB, 0.50218E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.48572E+00_JPRB, 0.48509E+00_JPRB, 0.48491E+00_JPRB, 0.48341E+00_JPRB, 0.48182E+00_JPRB, 0.48000E+00_JPRB, 0.47562E+00_JPRB ,&
&  0.47350E+00_JPRB, 0.46906E+00_JPRB, 0.46494E+00_JPRB, 0.46271E+00_JPRB, 0.46009E+00_JPRB, 0.45708E+00_JPRB, 0.45494E+00_JPRB ,&
&  0.45265E+00_JPRB, 0.44983E+00_JPRB, 0.44761E+00_JPRB, 0.44476E+00_JPRB, 0.44064E+00_JPRB, 0.43772E+00_JPRB, 0.43808E+00_JPRB ,&
&  0.43860E+00_JPRB, 0.44134E+00_JPRB, 0.45173E+00_JPRB, 0.46232E+00_JPRB, 0.47596E+00_JPRB, 0.49056E+00_JPRB, 0.50297E+00_JPRB ,&
&  0.52466E+00_JPRB, 0.46809E+00_JPRB, 0.50686E+00_JPRB, 0.53897E+00_JPRB, 0.58547E+00_JPRB, 0.62010E+00_JPRB, 0.60639E+00_JPRB ,&
&  0.57183E+00_JPRB, 0.53556E+00_JPRB, 0.51291E+00_JPRB, 0.50223E+00_JPRB, 0.50586E+00_JPRB, 0.51095E+00_JPRB, 0.50995E+00_JPRB ,&
&  0.50426E+00_JPRB, 0.49195E+00_JPRB, 0.47801E+00_JPRB, 0.46925E+00_JPRB, 0.46448E+00_JPRB, 0.46066E+00_JPRB, 0.45689E+00_JPRB ,&
&  0.45418E+00_JPRB, 0.45300E+00_JPRB, 0.45234E+00_JPRB, 0.45198E+00_JPRB, 0.45251E+00_JPRB, 0.45417E+00_JPRB, 0.45643E+00_JPRB ,&
&  0.45793E+00_JPRB, 0.45833E+00_JPRB, 0.46311E+00_JPRB, 0.47842E+00_JPRB, 0.50014E+00_JPRB, 0.52237E+00_JPRB, 0.54129E+00_JPRB ,&
&  0.55770E+00_JPRB, 0.57298E+00_JPRB, 0.58365E+00_JPRB, 0.58510E+00_JPRB, 0.57726E+00_JPRB, 0.56391E+00_JPRB, 0.54877E+00_JPRB ,&
&  0.53203E+00_JPRB, 0.51403E+00_JPRB, 0.49400E+00_JPRB, 0.48094E+00_JPRB, 0.47404E+00_JPRB, 0.46982E+00_JPRB, 0.46686E+00_JPRB ,&
&  0.46510E+00_JPRB, 0.46447E+00_JPRB, 0.46590E+00_JPRB, 0.46800E+00_JPRB, 0.46919E+00_JPRB, 0.47022E+00_JPRB, 0.47219E+00_JPRB ,&
&  0.47588E+00_JPRB, 0.48181E+00_JPRB, 0.48944E+00_JPRB, 0.49996E+00_JPRB, 0.50901E+00_JPRB, 0.51454E+00_JPRB, 0.51693E+00_JPRB ,&
&  0.51373E+00_JPRB, 0.50660E+00_JPRB, 0.49700E+00_JPRB, 0.48872E+00_JPRB, 0.48668E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.47191E+00_JPRB, 0.46746E+00_JPRB, 0.46427E+00_JPRB, 0.45944E+00_JPRB, 0.45542E+00_JPRB, 0.45222E+00_JPRB, 0.44903E+00_JPRB ,&
&  0.44682E+00_JPRB, 0.44671E+00_JPRB, 0.44506E+00_JPRB, 0.44398E+00_JPRB, 0.44244E+00_JPRB, 0.43911E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.43322E+00_JPRB, 0.43142E+00_JPRB, 0.42979E+00_JPRB, 0.42784E+00_JPRB, 0.42563E+00_JPRB, 0.42260E+00_JPRB, 0.42222E+00_JPRB ,&
&  0.42263E+00_JPRB, 0.42529E+00_JPRB, 0.42981E+00_JPRB, 0.43507E+00_JPRB, 0.44406E+00_JPRB, 0.45676E+00_JPRB, 0.46911E+00_JPRB ,&
&  0.48394E+00_JPRB, 0.47109E+00_JPRB, 0.49552E+00_JPRB, 0.51058E+00_JPRB, 0.53715E+00_JPRB, 0.55977E+00_JPRB, 0.56041E+00_JPRB ,&
&  0.54656E+00_JPRB, 0.52208E+00_JPRB, 0.50257E+00_JPRB, 0.49215E+00_JPRB, 0.49145E+00_JPRB, 0.48961E+00_JPRB, 0.48061E+00_JPRB ,&
&  0.46730E+00_JPRB, 0.45197E+00_JPRB, 0.44055E+00_JPRB, 0.43442E+00_JPRB, 0.43192E+00_JPRB, 0.43078E+00_JPRB, 0.42845E+00_JPRB ,&
&  0.42588E+00_JPRB, 0.42340E+00_JPRB, 0.42205E+00_JPRB, 0.42135E+00_JPRB, 0.42069E+00_JPRB, 0.42069E+00_JPRB, 0.42134E+00_JPRB ,&
&  0.42201E+00_JPRB, 0.42220E+00_JPRB, 0.42203E+00_JPRB, 0.42396E+00_JPRB, 0.43047E+00_JPRB, 0.44051E+00_JPRB, 0.45130E+00_JPRB ,&
&  0.45943E+00_JPRB, 0.46736E+00_JPRB, 0.47492E+00_JPRB, 0.48045E+00_JPRB, 0.48071E+00_JPRB, 0.47490E+00_JPRB, 0.46589E+00_JPRB ,&
&  0.45761E+00_JPRB, 0.44971E+00_JPRB, 0.44158E+00_JPRB, 0.43494E+00_JPRB, 0.43098E+00_JPRB, 0.42976E+00_JPRB, 0.42958E+00_JPRB ,&
&  0.42972E+00_JPRB, 0.43018E+00_JPRB, 0.43151E+00_JPRB, 0.43392E+00_JPRB, 0.43568E+00_JPRB, 0.43726E+00_JPRB, 0.43870E+00_JPRB ,&
&  0.44269E+00_JPRB, 0.44769E+00_JPRB, 0.45485E+00_JPRB, 0.46543E+00_JPRB, 0.47524E+00_JPRB, 0.48239E+00_JPRB, 0.48833E+00_JPRB ,&
&  0.49088E+00_JPRB, 0.49000E+00_JPRB, 0.48671E+00_JPRB, 0.48235E+00_JPRB, 0.47750E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.44412E+00_JPRB, 0.43750E+00_JPRB, 0.43264E+00_JPRB, 0.42898E+00_JPRB, 0.42801E+00_JPRB, 0.42589E+00_JPRB, 0.42183E+00_JPRB ,&
&  0.41859E+00_JPRB, 0.41962E+00_JPRB, 0.42247E+00_JPRB, 0.42369E+00_JPRB, 0.42277E+00_JPRB, 0.42088E+00_JPRB, 0.41746E+00_JPRB ,&
&  0.41300E+00_JPRB, 0.41078E+00_JPRB, 0.41065E+00_JPRB, 0.41178E+00_JPRB, 0.41140E+00_JPRB, 0.40898E+00_JPRB, 0.40682E+00_JPRB ,&
&  0.40696E+00_JPRB, 0.40847E+00_JPRB, 0.40947E+00_JPRB, 0.41182E+00_JPRB, 0.41823E+00_JPRB, 0.43041E+00_JPRB, 0.44140E+00_JPRB ,&
&  0.44854E+00_JPRB, 0.45196E+00_JPRB, 0.44643E+00_JPRB, 0.46481E+00_JPRB, 0.47966E+00_JPRB, 0.49908E+00_JPRB, 0.51269E+00_JPRB ,&
&  0.51614E+00_JPRB, 0.50915E+00_JPRB, 0.49407E+00_JPRB, 0.48166E+00_JPRB, 0.47283E+00_JPRB, 0.46138E+00_JPRB, 0.44440E+00_JPRB ,&
&  0.42714E+00_JPRB, 0.41338E+00_JPRB, 0.40516E+00_JPRB, 0.40127E+00_JPRB, 0.40072E+00_JPRB, 0.40089E+00_JPRB, 0.39863E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39327E+00_JPRB, 0.39211E+00_JPRB, 0.39109E+00_JPRB, 0.38978E+00_JPRB, 0.38871E+00_JPRB, 0.38828E+00_JPRB ,&
&  0.38828E+00_JPRB, 0.38824E+00_JPRB, 0.38845E+00_JPRB, 0.38898E+00_JPRB, 0.38950E+00_JPRB, 0.39155E+00_JPRB, 0.39604E+00_JPRB ,&
&  0.40072E+00_JPRB, 0.40480E+00_JPRB, 0.40835E+00_JPRB, 0.41116E+00_JPRB, 0.41248E+00_JPRB, 0.41176E+00_JPRB, 0.40830E+00_JPRB ,&
&  0.40406E+00_JPRB, 0.40165E+00_JPRB, 0.39979E+00_JPRB, 0.39702E+00_JPRB, 0.39432E+00_JPRB, 0.39359E+00_JPRB, 0.39448E+00_JPRB ,&
&  0.39574E+00_JPRB, 0.39776E+00_JPRB, 0.39966E+00_JPRB, 0.40172E+00_JPRB, 0.40305E+00_JPRB, 0.40483E+00_JPRB, 0.40665E+00_JPRB ,&
&  0.40909E+00_JPRB, 0.41260E+00_JPRB, 0.41746E+00_JPRB, 0.42383E+00_JPRB, 0.43065E+00_JPRB, 0.43624E+00_JPRB, 0.44239E+00_JPRB ,&
&  0.44818E+00_JPRB, 0.45163E+00_JPRB, 0.45261E+00_JPRB, 0.45273E+00_JPRB, 0.45015E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.41016E+00_JPRB, 0.40588E+00_JPRB, 0.40260E+00_JPRB, 0.40146E+00_JPRB, 0.40254E+00_JPRB, 0.40177E+00_JPRB, 0.39860E+00_JPRB ,&
&  0.39580E+00_JPRB, 0.39683E+00_JPRB, 0.40049E+00_JPRB, 0.40292E+00_JPRB, 0.40413E+00_JPRB, 0.40506E+00_JPRB, 0.40345E+00_JPRB ,&
&  0.39907E+00_JPRB, 0.39504E+00_JPRB, 0.39558E+00_JPRB, 0.39774E+00_JPRB, 0.39862E+00_JPRB, 0.39644E+00_JPRB, 0.39395E+00_JPRB ,&
&  0.39381E+00_JPRB, 0.39390E+00_JPRB, 0.39589E+00_JPRB, 0.39781E+00_JPRB, 0.40316E+00_JPRB, 0.41317E+00_JPRB, 0.42167E+00_JPRB ,&
&  0.42679E+00_JPRB, 0.42565E+00_JPRB, 0.42460E+00_JPRB, 0.42945E+00_JPRB, 0.43570E+00_JPRB, 0.44945E+00_JPRB, 0.46468E+00_JPRB ,&
&  0.47262E+00_JPRB, 0.47097E+00_JPRB, 0.45791E+00_JPRB, 0.44393E+00_JPRB, 0.43204E+00_JPRB, 0.41903E+00_JPRB, 0.40614E+00_JPRB ,&
&  0.39417E+00_JPRB, 0.38582E+00_JPRB, 0.38107E+00_JPRB, 0.37896E+00_JPRB, 0.37815E+00_JPRB, 0.37703E+00_JPRB, 0.37403E+00_JPRB ,&
&  0.37033E+00_JPRB, 0.36742E+00_JPRB, 0.36529E+00_JPRB, 0.36376E+00_JPRB, 0.36238E+00_JPRB, 0.36168E+00_JPRB, 0.36083E+00_JPRB ,&
&  0.36040E+00_JPRB, 0.36073E+00_JPRB, 0.36145E+00_JPRB, 0.36125E+00_JPRB, 0.36078E+00_JPRB, 0.35993E+00_JPRB, 0.36044E+00_JPRB ,&
&  0.36292E+00_JPRB, 0.36414E+00_JPRB, 0.36516E+00_JPRB, 0.36717E+00_JPRB, 0.36863E+00_JPRB, 0.36968E+00_JPRB, 0.36937E+00_JPRB ,&
&  0.36870E+00_JPRB, 0.36860E+00_JPRB, 0.36857E+00_JPRB, 0.36775E+00_JPRB, 0.36672E+00_JPRB, 0.36618E+00_JPRB, 0.36629E+00_JPRB ,&
&  0.36790E+00_JPRB, 0.37004E+00_JPRB, 0.37176E+00_JPRB, 0.37350E+00_JPRB, 0.37552E+00_JPRB, 0.37838E+00_JPRB, 0.37972E+00_JPRB ,&
&  0.38055E+00_JPRB, 0.38224E+00_JPRB, 0.38568E+00_JPRB, 0.39008E+00_JPRB, 0.39437E+00_JPRB, 0.39754E+00_JPRB, 0.40113E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.40791E+00_JPRB, 0.41006E+00_JPRB, 0.41085E+00_JPRB, 0.41122E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.37847E+00_JPRB, 0.37700E+00_JPRB, 0.37639E+00_JPRB, 0.37662E+00_JPRB, 0.37813E+00_JPRB, 0.37751E+00_JPRB, 0.37648E+00_JPRB ,&
&  0.37571E+00_JPRB, 0.37779E+00_JPRB, 0.38195E+00_JPRB, 0.38471E+00_JPRB, 0.38562E+00_JPRB, 0.38593E+00_JPRB, 0.38538E+00_JPRB ,&
&  0.38453E+00_JPRB, 0.38383E+00_JPRB, 0.38281E+00_JPRB, 0.38369E+00_JPRB, 0.38329E+00_JPRB, 0.38155E+00_JPRB, 0.38188E+00_JPRB ,&
&  0.38158E+00_JPRB, 0.38203E+00_JPRB, 0.38459E+00_JPRB, 0.38693E+00_JPRB, 0.39222E+00_JPRB, 0.40168E+00_JPRB, 0.40891E+00_JPRB ,&
&  0.41164E+00_JPRB, 0.40668E+00_JPRB, 0.40155E+00_JPRB, 0.40191E+00_JPRB, 0.40485E+00_JPRB, 0.41139E+00_JPRB, 0.41941E+00_JPRB ,&
&  0.42362E+00_JPRB, 0.41828E+00_JPRB, 0.40685E+00_JPRB, 0.39631E+00_JPRB, 0.38783E+00_JPRB, 0.38071E+00_JPRB, 0.37440E+00_JPRB ,&
&  0.36869E+00_JPRB, 0.36492E+00_JPRB, 0.36337E+00_JPRB, 0.36311E+00_JPRB, 0.36237E+00_JPRB, 0.35880E+00_JPRB, 0.35446E+00_JPRB ,&
&  0.34926E+00_JPRB, 0.34555E+00_JPRB, 0.34257E+00_JPRB, 0.34146E+00_JPRB, 0.34143E+00_JPRB, 0.34094E+00_JPRB, 0.33957E+00_JPRB ,&
&  0.33794E+00_JPRB, 0.33730E+00_JPRB, 0.33674E+00_JPRB, 0.33528E+00_JPRB, 0.33346E+00_JPRB, 0.33263E+00_JPRB, 0.33177E+00_JPRB ,&
&  0.33106E+00_JPRB, 0.33195E+00_JPRB, 0.33388E+00_JPRB, 0.33637E+00_JPRB, 0.33707E+00_JPRB, 0.33867E+00_JPRB, 0.34081E+00_JPRB ,&
&  0.34039E+00_JPRB, 0.33776E+00_JPRB, 0.33579E+00_JPRB, 0.33555E+00_JPRB, 0.33696E+00_JPRB, 0.33937E+00_JPRB, 0.34046E+00_JPRB ,&
&  0.34143E+00_JPRB, 0.34150E+00_JPRB, 0.34263E+00_JPRB, 0.34438E+00_JPRB, 0.34700E+00_JPRB, 0.34962E+00_JPRB, 0.35132E+00_JPRB ,&
&  0.35231E+00_JPRB, 0.35309E+00_JPRB, 0.35560E+00_JPRB, 0.35932E+00_JPRB, 0.36255E+00_JPRB, 0.36613E+00_JPRB, 0.36892E+00_JPRB ,&
&  0.37094E+00_JPRB, 0.37311E+00_JPRB, 0.37464E+00_JPRB, 0.37576E+00_JPRB, 0.37721E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.35772E+00_JPRB, 0.35847E+00_JPRB, 0.35988E+00_JPRB, 0.36042E+00_JPRB, 0.36142E+00_JPRB, 0.36162E+00_JPRB, 0.36253E+00_JPRB ,&
&  0.36297E+00_JPRB, 0.36479E+00_JPRB, 0.36738E+00_JPRB, 0.36757E+00_JPRB, 0.36738E+00_JPRB, 0.36735E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.37072E+00_JPRB, 0.37168E+00_JPRB, 0.37128E+00_JPRB, 0.37160E+00_JPRB, 0.37015E+00_JPRB, 0.36862E+00_JPRB, 0.37018E+00_JPRB ,&
&  0.37043E+00_JPRB, 0.37112E+00_JPRB, 0.37345E+00_JPRB, 0.37581E+00_JPRB, 0.38096E+00_JPRB, 0.38779E+00_JPRB, 0.39464E+00_JPRB ,&
&  0.39807E+00_JPRB, 0.39487E+00_JPRB, 0.38738E+00_JPRB, 0.38184E+00_JPRB, 0.38052E+00_JPRB, 0.38157E+00_JPRB, 0.38434E+00_JPRB ,&
&  0.38378E+00_JPRB, 0.37757E+00_JPRB, 0.36918E+00_JPRB, 0.36419E+00_JPRB, 0.36166E+00_JPRB, 0.35782E+00_JPRB, 0.35250E+00_JPRB ,&
&  0.34845E+00_JPRB, 0.34711E+00_JPRB, 0.34738E+00_JPRB, 0.34660E+00_JPRB, 0.34438E+00_JPRB, 0.34042E+00_JPRB, 0.33547E+00_JPRB ,&
&  0.33097E+00_JPRB, 0.32661E+00_JPRB, 0.32378E+00_JPRB, 0.32201E+00_JPRB, 0.32180E+00_JPRB, 0.32168E+00_JPRB, 0.32037E+00_JPRB ,&
&  0.31786E+00_JPRB, 0.31613E+00_JPRB, 0.31403E+00_JPRB, 0.31050E+00_JPRB, 0.30802E+00_JPRB, 0.30729E+00_JPRB, 0.30639E+00_JPRB ,&
&  0.30528E+00_JPRB, 0.30555E+00_JPRB, 0.30616E+00_JPRB, 0.30640E+00_JPRB, 0.30809E+00_JPRB, 0.31056E+00_JPRB, 0.30837E+00_JPRB ,&
&  0.30614E+00_JPRB, 0.30462E+00_JPRB, 0.30468E+00_JPRB, 0.30655E+00_JPRB, 0.30967E+00_JPRB, 0.31298E+00_JPRB, 0.31564E+00_JPRB ,&
&  0.31674E+00_JPRB, 0.31696E+00_JPRB, 0.31730E+00_JPRB, 0.31776E+00_JPRB, 0.31974E+00_JPRB, 0.32332E+00_JPRB, 0.32725E+00_JPRB ,&
&  0.32893E+00_JPRB, 0.33166E+00_JPRB, 0.33484E+00_JPRB, 0.33890E+00_JPRB, 0.34142E+00_JPRB, 0.34387E+00_JPRB, 0.34606E+00_JPRB ,&
&  0.34760E+00_JPRB, 0.34954E+00_JPRB, 0.35205E+00_JPRB, 0.35402E+00_JPRB, 0.35601E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.34243E+00_JPRB, 0.34405E+00_JPRB, 0.34502E+00_JPRB, 0.34678E+00_JPRB, 0.34982E+00_JPRB, 0.35129E+00_JPRB, 0.35097E+00_JPRB ,&
&  0.35104E+00_JPRB, 0.35272E+00_JPRB, 0.35416E+00_JPRB, 0.35487E+00_JPRB, 0.35264E+00_JPRB, 0.35234E+00_JPRB, 0.35434E+00_JPRB ,&
&  0.35698E+00_JPRB, 0.35904E+00_JPRB, 0.35933E+00_JPRB, 0.35955E+00_JPRB, 0.35838E+00_JPRB, 0.35660E+00_JPRB, 0.35787E+00_JPRB ,&
&  0.35901E+00_JPRB, 0.35948E+00_JPRB, 0.36102E+00_JPRB, 0.36295E+00_JPRB, 0.36695E+00_JPRB, 0.37174E+00_JPRB, 0.37599E+00_JPRB ,&
&  0.37926E+00_JPRB, 0.37842E+00_JPRB, 0.37033E+00_JPRB, 0.36199E+00_JPRB, 0.36154E+00_JPRB, 0.36208E+00_JPRB, 0.36165E+00_JPRB ,&
&  0.35930E+00_JPRB, 0.35611E+00_JPRB, 0.35402E+00_JPRB, 0.35231E+00_JPRB, 0.34990E+00_JPRB, 0.34502E+00_JPRB, 0.33962E+00_JPRB ,&
&  0.33516E+00_JPRB, 0.33365E+00_JPRB, 0.33236E+00_JPRB, 0.32964E+00_JPRB, 0.32541E+00_JPRB, 0.32175E+00_JPRB, 0.31788E+00_JPRB ,&
&  0.31153E+00_JPRB, 0.30552E+00_JPRB, 0.30217E+00_JPRB, 0.30108E+00_JPRB, 0.30112E+00_JPRB, 0.30031E+00_JPRB, 0.29797E+00_JPRB ,&
&  0.29529E+00_JPRB, 0.29225E+00_JPRB, 0.29043E+00_JPRB, 0.29001E+00_JPRB, 0.29140E+00_JPRB, 0.28961E+00_JPRB, 0.28459E+00_JPRB ,&
&  0.28207E+00_JPRB, 0.28114E+00_JPRB, 0.28226E+00_JPRB, 0.28165E+00_JPRB, 0.28118E+00_JPRB, 0.28023E+00_JPRB, 0.28078E+00_JPRB ,&
&  0.28212E+00_JPRB, 0.28214E+00_JPRB, 0.28506E+00_JPRB, 0.29036E+00_JPRB, 0.29498E+00_JPRB, 0.29768E+00_JPRB, 0.29939E+00_JPRB ,&
&  0.30010E+00_JPRB, 0.29989E+00_JPRB, 0.29963E+00_JPRB, 0.30063E+00_JPRB, 0.30351E+00_JPRB, 0.30745E+00_JPRB, 0.31086E+00_JPRB ,&
&  0.31137E+00_JPRB, 0.31266E+00_JPRB, 0.31614E+00_JPRB, 0.32138E+00_JPRB, 0.32441E+00_JPRB, 0.32671E+00_JPRB, 0.32845E+00_JPRB ,&
&  0.32908E+00_JPRB, 0.33063E+00_JPRB, 0.33441E+00_JPRB, 0.33685E+00_JPRB, 0.33938E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.32684E+00_JPRB, 0.32771E+00_JPRB, 0.32975E+00_JPRB, 0.33233E+00_JPRB, 0.33366E+00_JPRB, 0.33317E+00_JPRB, 0.33204E+00_JPRB ,&
&  0.33246E+00_JPRB, 0.33510E+00_JPRB, 0.33792E+00_JPRB, 0.33900E+00_JPRB, 0.33964E+00_JPRB, 0.34028E+00_JPRB, 0.34234E+00_JPRB ,&
&  0.34519E+00_JPRB, 0.34851E+00_JPRB, 0.34865E+00_JPRB, 0.34865E+00_JPRB, 0.34789E+00_JPRB, 0.34652E+00_JPRB, 0.34765E+00_JPRB ,&
&  0.35075E+00_JPRB, 0.35078E+00_JPRB, 0.35041E+00_JPRB, 0.34972E+00_JPRB, 0.35187E+00_JPRB, 0.35592E+00_JPRB, 0.35889E+00_JPRB ,&
&  0.35901E+00_JPRB, 0.35236E+00_JPRB, 0.34366E+00_JPRB, 0.34316E+00_JPRB, 0.34482E+00_JPRB, 0.34636E+00_JPRB, 0.34628E+00_JPRB ,&
&  0.34572E+00_JPRB, 0.34469E+00_JPRB, 0.34345E+00_JPRB, 0.34207E+00_JPRB, 0.34002E+00_JPRB, 0.33481E+00_JPRB, 0.32879E+00_JPRB ,&
&  0.32399E+00_JPRB, 0.32019E+00_JPRB, 0.31659E+00_JPRB, 0.31127E+00_JPRB, 0.30592E+00_JPRB, 0.30200E+00_JPRB, 0.29724E+00_JPRB ,&
&  0.29149E+00_JPRB, 0.28765E+00_JPRB, 0.28638E+00_JPRB, 0.28603E+00_JPRB, 0.28581E+00_JPRB, 0.28468E+00_JPRB, 0.28319E+00_JPRB ,&
&  0.28073E+00_JPRB, 0.27887E+00_JPRB, 0.27712E+00_JPRB, 0.27591E+00_JPRB, 0.26880E+00_JPRB, 0.26116E+00_JPRB, 0.25537E+00_JPRB ,&
&  0.25520E+00_JPRB, 0.25784E+00_JPRB, 0.26070E+00_JPRB, 0.26220E+00_JPRB, 0.26536E+00_JPRB, 0.26653E+00_JPRB, 0.26591E+00_JPRB ,&
&  0.26204E+00_JPRB, 0.25990E+00_JPRB, 0.26169E+00_JPRB, 0.26334E+00_JPRB, 0.26411E+00_JPRB, 0.26585E+00_JPRB, 0.26732E+00_JPRB ,&
&  0.27050E+00_JPRB, 0.27259E+00_JPRB, 0.27517E+00_JPRB, 0.27814E+00_JPRB, 0.28053E+00_JPRB, 0.28139E+00_JPRB, 0.27917E+00_JPRB ,&
&  0.27951E+00_JPRB, 0.28608E+00_JPRB, 0.29695E+00_JPRB, 0.30638E+00_JPRB, 0.30960E+00_JPRB, 0.31243E+00_JPRB, 0.31477E+00_JPRB ,&
&  0.31607E+00_JPRB, 0.32008E+00_JPRB, 0.32336E+00_JPRB, 0.32499E+00_JPRB, 0.32585E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.31560E+00_JPRB, 0.31479E+00_JPRB, 0.31328E+00_JPRB, 0.31207E+00_JPRB, 0.31039E+00_JPRB, 0.30889E+00_JPRB, 0.30968E+00_JPRB ,&
&  0.31275E+00_JPRB, 0.31678E+00_JPRB, 0.32067E+00_JPRB, 0.32364E+00_JPRB, 0.32590E+00_JPRB, 0.32706E+00_JPRB, 0.32879E+00_JPRB ,&
&  0.33074E+00_JPRB, 0.33434E+00_JPRB, 0.33615E+00_JPRB, 0.33767E+00_JPRB, 0.33686E+00_JPRB, 0.33594E+00_JPRB, 0.33698E+00_JPRB ,&
&  0.34026E+00_JPRB, 0.34154E+00_JPRB, 0.34043E+00_JPRB, 0.33802E+00_JPRB, 0.33864E+00_JPRB, 0.34122E+00_JPRB, 0.34138E+00_JPRB ,&
&  0.33418E+00_JPRB, 0.31947E+00_JPRB, 0.31013E+00_JPRB, 0.31450E+00_JPRB, 0.32312E+00_JPRB, 0.32705E+00_JPRB, 0.33029E+00_JPRB ,&
&  0.33233E+00_JPRB, 0.33242E+00_JPRB, 0.33127E+00_JPRB, 0.32915E+00_JPRB, 0.32576E+00_JPRB, 0.31998E+00_JPRB, 0.31333E+00_JPRB ,&
&  0.30758E+00_JPRB, 0.30501E+00_JPRB, 0.30141E+00_JPRB, 0.29482E+00_JPRB, 0.28864E+00_JPRB, 0.28492E+00_JPRB, 0.27984E+00_JPRB ,&
&  0.27449E+00_JPRB, 0.27089E+00_JPRB, 0.27014E+00_JPRB, 0.26945E+00_JPRB, 0.26896E+00_JPRB, 0.26774E+00_JPRB, 0.26440E+00_JPRB ,&
&  0.26145E+00_JPRB, 0.25663E+00_JPRB, 0.25083E+00_JPRB, 0.24540E+00_JPRB, 0.24185E+00_JPRB, 0.23792E+00_JPRB, 0.23282E+00_JPRB ,&
&  0.23034E+00_JPRB, 0.23351E+00_JPRB, 0.24407E+00_JPRB, 0.25489E+00_JPRB, 0.25620E+00_JPRB, 0.24459E+00_JPRB, 0.24171E+00_JPRB ,&
&  0.24206E+00_JPRB, 0.24224E+00_JPRB, 0.24281E+00_JPRB, 0.24458E+00_JPRB, 0.24538E+00_JPRB, 0.24483E+00_JPRB, 0.24549E+00_JPRB ,&
&  0.24816E+00_JPRB, 0.25086E+00_JPRB, 0.25220E+00_JPRB, 0.25393E+00_JPRB, 0.25503E+00_JPRB, 0.25492E+00_JPRB, 0.25446E+00_JPRB ,&
&  0.25536E+00_JPRB, 0.25924E+00_JPRB, 0.26529E+00_JPRB, 0.27366E+00_JPRB, 0.27918E+00_JPRB, 0.28418E+00_JPRB, 0.29021E+00_JPRB ,&
&  0.29799E+00_JPRB, 0.30458E+00_JPRB, 0.30964E+00_JPRB, 0.31397E+00_JPRB, 0.31537E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.30483E+00_JPRB, 0.30574E+00_JPRB, 0.30119E+00_JPRB, 0.29737E+00_JPRB, 0.29540E+00_JPRB, 0.29496E+00_JPRB, 0.29669E+00_JPRB ,&
&  0.30058E+00_JPRB, 0.30380E+00_JPRB, 0.30686E+00_JPRB, 0.30957E+00_JPRB, 0.31073E+00_JPRB, 0.31041E+00_JPRB, 0.31157E+00_JPRB ,&
&  0.31278E+00_JPRB, 0.31606E+00_JPRB, 0.31858E+00_JPRB, 0.32223E+00_JPRB, 0.32257E+00_JPRB, 0.32335E+00_JPRB, 0.32469E+00_JPRB ,&
&  0.32413E+00_JPRB, 0.32416E+00_JPRB, 0.32541E+00_JPRB, 0.32262E+00_JPRB, 0.32091E+00_JPRB, 0.31844E+00_JPRB, 0.31676E+00_JPRB ,&
&  0.30942E+00_JPRB, 0.30174E+00_JPRB, 0.29545E+00_JPRB, 0.30038E+00_JPRB, 0.30803E+00_JPRB, 0.30894E+00_JPRB, 0.31368E+00_JPRB ,&
&  0.31542E+00_JPRB, 0.31353E+00_JPRB, 0.31178E+00_JPRB, 0.30811E+00_JPRB, 0.30470E+00_JPRB, 0.30071E+00_JPRB, 0.29746E+00_JPRB ,&
&  0.29366E+00_JPRB, 0.29450E+00_JPRB, 0.28926E+00_JPRB, 0.27400E+00_JPRB, 0.26700E+00_JPRB, 0.26252E+00_JPRB, 0.25445E+00_JPRB ,&
&  0.24595E+00_JPRB, 0.24419E+00_JPRB, 0.24395E+00_JPRB, 0.24072E+00_JPRB, 0.24027E+00_JPRB, 0.24060E+00_JPRB, 0.24040E+00_JPRB ,&
&  0.23865E+00_JPRB, 0.23832E+00_JPRB, 0.23684E+00_JPRB, 0.23455E+00_JPRB, 0.23221E+00_JPRB, 0.23119E+00_JPRB, 0.23160E+00_JPRB ,&
&  0.23327E+00_JPRB, 0.23476E+00_JPRB, 0.24094E+00_JPRB, 0.24441E+00_JPRB, 0.23786E+00_JPRB, 0.23606E+00_JPRB, 0.23791E+00_JPRB ,&
&  0.23311E+00_JPRB, 0.23233E+00_JPRB, 0.23236E+00_JPRB, 0.23238E+00_JPRB, 0.23288E+00_JPRB, 0.23390E+00_JPRB, 0.23358E+00_JPRB ,&
&  0.23435E+00_JPRB, 0.23616E+00_JPRB, 0.23671E+00_JPRB, 0.23678E+00_JPRB, 0.23638E+00_JPRB, 0.23690E+00_JPRB, 0.23840E+00_JPRB ,&
&  0.24005E+00_JPRB, 0.24335E+00_JPRB, 0.24675E+00_JPRB, 0.25181E+00_JPRB, 0.25587E+00_JPRB, 0.25916E+00_JPRB, 0.26534E+00_JPRB ,&
&  0.27015E+00_JPRB, 0.28106E+00_JPRB, 0.29878E+00_JPRB, 0.30661E+00_JPRB, 0.29976E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.29350E+00_JPRB, 0.29202E+00_JPRB, 0.28996E+00_JPRB, 0.28822E+00_JPRB, 0.28927E+00_JPRB, 0.29062E+00_JPRB, 0.29264E+00_JPRB ,&
&  0.29486E+00_JPRB, 0.29559E+00_JPRB, 0.29568E+00_JPRB, 0.29294E+00_JPRB, 0.29164E+00_JPRB, 0.28621E+00_JPRB, 0.28444E+00_JPRB ,&
&  0.28319E+00_JPRB, 0.28589E+00_JPRB, 0.28927E+00_JPRB, 0.29425E+00_JPRB, 0.29608E+00_JPRB, 0.29802E+00_JPRB, 0.29674E+00_JPRB ,&
&  0.29426E+00_JPRB, 0.29139E+00_JPRB, 0.28845E+00_JPRB, 0.28615E+00_JPRB, 0.28541E+00_JPRB, 0.28319E+00_JPRB, 0.28099E+00_JPRB ,&
&  0.28107E+00_JPRB, 0.28276E+00_JPRB, 0.28342E+00_JPRB, 0.28411E+00_JPRB, 0.28983E+00_JPRB, 0.28979E+00_JPRB, 0.29239E+00_JPRB ,&
&  0.29340E+00_JPRB, 0.29460E+00_JPRB, 0.29445E+00_JPRB, 0.29324E+00_JPRB, 0.29212E+00_JPRB, 0.28519E+00_JPRB, 0.28258E+00_JPRB ,&
&  0.27574E+00_JPRB, 0.27191E+00_JPRB, 0.26463E+00_JPRB, 0.25771E+00_JPRB, 0.24948E+00_JPRB, 0.24330E+00_JPRB, 0.23801E+00_JPRB ,&
&  0.23380E+00_JPRB, 0.23059E+00_JPRB, 0.22807E+00_JPRB, 0.22777E+00_JPRB, 0.22778E+00_JPRB, 0.22718E+00_JPRB, 0.22736E+00_JPRB ,&
&  0.22850E+00_JPRB, 0.22973E+00_JPRB, 0.22959E+00_JPRB, 0.23024E+00_JPRB, 0.23077E+00_JPRB, 0.23155E+00_JPRB, 0.23319E+00_JPRB ,&
&  0.23572E+00_JPRB, 0.23766E+00_JPRB, 0.23805E+00_JPRB, 0.24050E+00_JPRB, 0.23981E+00_JPRB, 0.23686E+00_JPRB, 0.23406E+00_JPRB ,&
&  0.23060E+00_JPRB, 0.22911E+00_JPRB, 0.22731E+00_JPRB, 0.22627E+00_JPRB, 0.22608E+00_JPRB, 0.22620E+00_JPRB, 0.22708E+00_JPRB ,&
&  0.22806E+00_JPRB, 0.22910E+00_JPRB, 0.22944E+00_JPRB, 0.22989E+00_JPRB, 0.23022E+00_JPRB, 0.23120E+00_JPRB, 0.23318E+00_JPRB ,&
&  0.23596E+00_JPRB, 0.23916E+00_JPRB, 0.24230E+00_JPRB, 0.24606E+00_JPRB, 0.25221E+00_JPRB, 0.26011E+00_JPRB, 0.27197E+00_JPRB ,&
&  0.28037E+00_JPRB, 0.29086E+00_JPRB, 0.28840E+00_JPRB, 0.29294E+00_JPRB, 0.29437E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.28484E+00_JPRB, 0.28470E+00_JPRB, 0.28354E+00_JPRB, 0.28292E+00_JPRB, 0.28343E+00_JPRB, 0.28433E+00_JPRB, 0.28576E+00_JPRB ,&
&  0.28237E+00_JPRB, 0.28032E+00_JPRB, 0.27599E+00_JPRB, 0.27124E+00_JPRB, 0.26692E+00_JPRB, 0.26397E+00_JPRB, 0.26236E+00_JPRB ,&
&  0.26229E+00_JPRB, 0.26498E+00_JPRB, 0.26674E+00_JPRB, 0.26924E+00_JPRB, 0.26927E+00_JPRB, 0.26944E+00_JPRB, 0.26761E+00_JPRB ,&
&  0.26566E+00_JPRB, 0.26494E+00_JPRB, 0.26515E+00_JPRB, 0.26650E+00_JPRB, 0.26909E+00_JPRB, 0.27049E+00_JPRB, 0.27237E+00_JPRB ,&
&  0.27499E+00_JPRB, 0.27371E+00_JPRB, 0.27431E+00_JPRB, 0.27355E+00_JPRB, 0.27432E+00_JPRB, 0.27586E+00_JPRB, 0.27679E+00_JPRB ,&
&  0.27857E+00_JPRB, 0.28021E+00_JPRB, 0.28059E+00_JPRB, 0.27981E+00_JPRB, 0.27515E+00_JPRB, 0.27323E+00_JPRB, 0.26863E+00_JPRB ,&
&  0.26285E+00_JPRB, 0.25747E+00_JPRB, 0.25072E+00_JPRB, 0.24587E+00_JPRB, 0.23982E+00_JPRB, 0.23630E+00_JPRB, 0.23356E+00_JPRB ,&
&  0.23100E+00_JPRB, 0.23011E+00_JPRB, 0.22877E+00_JPRB, 0.22723E+00_JPRB, 0.22682E+00_JPRB, 0.22521E+00_JPRB, 0.22512E+00_JPRB ,&
&  0.22559E+00_JPRB, 0.22601E+00_JPRB, 0.22730E+00_JPRB, 0.22778E+00_JPRB, 0.22864E+00_JPRB, 0.22936E+00_JPRB, 0.22975E+00_JPRB ,&
&  0.23092E+00_JPRB, 0.23267E+00_JPRB, 0.23257E+00_JPRB, 0.23311E+00_JPRB, 0.23222E+00_JPRB, 0.23003E+00_JPRB, 0.22772E+00_JPRB ,&
&  0.22514E+00_JPRB, 0.22316E+00_JPRB, 0.22258E+00_JPRB, 0.22168E+00_JPRB, 0.22180E+00_JPRB, 0.22221E+00_JPRB, 0.22229E+00_JPRB ,&
&  0.22299E+00_JPRB, 0.22361E+00_JPRB, 0.22461E+00_JPRB, 0.22697E+00_JPRB, 0.22851E+00_JPRB, 0.23057E+00_JPRB, 0.23278E+00_JPRB ,&
&  0.23451E+00_JPRB, 0.23777E+00_JPRB, 0.24062E+00_JPRB, 0.24453E+00_JPRB, 0.25373E+00_JPRB, 0.26108E+00_JPRB, 0.27428E+00_JPRB ,&
&  0.27797E+00_JPRB, 0.28539E+00_JPRB, 0.28135E+00_JPRB, 0.28396E+00_JPRB, 0.28535E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.27923E+00_JPRB, 0.28017E+00_JPRB, 0.27985E+00_JPRB, 0.28087E+00_JPRB, 0.28155E+00_JPRB, 0.28268E+00_JPRB, 0.28240E+00_JPRB ,&
&  0.28214E+00_JPRB, 0.27855E+00_JPRB, 0.27663E+00_JPRB, 0.27310E+00_JPRB, 0.26995E+00_JPRB, 0.26644E+00_JPRB, 0.26364E+00_JPRB ,&
&  0.26189E+00_JPRB, 0.26050E+00_JPRB, 0.25925E+00_JPRB, 0.25748E+00_JPRB, 0.25516E+00_JPRB, 0.25280E+00_JPRB, 0.25066E+00_JPRB ,&
&  0.24868E+00_JPRB, 0.24891E+00_JPRB, 0.25048E+00_JPRB, 0.25292E+00_JPRB, 0.25747E+00_JPRB, 0.26044E+00_JPRB, 0.26432E+00_JPRB ,&
&  0.26979E+00_JPRB, 0.26498E+00_JPRB, 0.26527E+00_JPRB, 0.26597E+00_JPRB, 0.26601E+00_JPRB, 0.26616E+00_JPRB, 0.26437E+00_JPRB ,&
&  0.27010E+00_JPRB, 0.27048E+00_JPRB, 0.27475E+00_JPRB, 0.27322E+00_JPRB, 0.26778E+00_JPRB, 0.26432E+00_JPRB, 0.25901E+00_JPRB ,&
&  0.25568E+00_JPRB, 0.25065E+00_JPRB, 0.24725E+00_JPRB, 0.24393E+00_JPRB, 0.23957E+00_JPRB, 0.23781E+00_JPRB, 0.23477E+00_JPRB ,&
&  0.23301E+00_JPRB, 0.23268E+00_JPRB, 0.23150E+00_JPRB, 0.23153E+00_JPRB, 0.23045E+00_JPRB, 0.22895E+00_JPRB, 0.22802E+00_JPRB ,&
&  0.22724E+00_JPRB, 0.22639E+00_JPRB, 0.22614E+00_JPRB, 0.22580E+00_JPRB, 0.22594E+00_JPRB, 0.22587E+00_JPRB, 0.22581E+00_JPRB ,&
&  0.22595E+00_JPRB, 0.22609E+00_JPRB, 0.22553E+00_JPRB, 0.22544E+00_JPRB, 0.22435E+00_JPRB, 0.22336E+00_JPRB, 0.22180E+00_JPRB ,&
&  0.22079E+00_JPRB, 0.22012E+00_JPRB, 0.21901E+00_JPRB, 0.21830E+00_JPRB, 0.21799E+00_JPRB, 0.21849E+00_JPRB, 0.21917E+00_JPRB ,&
&  0.21999E+00_JPRB, 0.22163E+00_JPRB, 0.22296E+00_JPRB, 0.22477E+00_JPRB, 0.22674E+00_JPRB, 0.22824E+00_JPRB, 0.23046E+00_JPRB ,&
&  0.23263E+00_JPRB, 0.23489E+00_JPRB, 0.23886E+00_JPRB, 0.24313E+00_JPRB, 0.24874E+00_JPRB, 0.25408E+00_JPRB, 0.25593E+00_JPRB ,&
&  0.25776E+00_JPRB, 0.26317E+00_JPRB, 0.26596E+00_JPRB, 0.27653E+00_JPRB, 0.27824E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.26739E+00_JPRB, 0.26931E+00_JPRB, 0.27118E+00_JPRB, 0.27297E+00_JPRB, 0.27524E+00_JPRB, 0.27621E+00_JPRB, 0.27633E+00_JPRB ,&
&  0.27385E+00_JPRB, 0.27353E+00_JPRB, 0.27204E+00_JPRB, 0.26995E+00_JPRB, 0.26826E+00_JPRB, 0.26427E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.25787E+00_JPRB, 0.25486E+00_JPRB, 0.25233E+00_JPRB, 0.24879E+00_JPRB, 0.24546E+00_JPRB, 0.24206E+00_JPRB, 0.24149E+00_JPRB ,&
&  0.24142E+00_JPRB, 0.24234E+00_JPRB, 0.24463E+00_JPRB, 0.24621E+00_JPRB, 0.24855E+00_JPRB, 0.25099E+00_JPRB, 0.25461E+00_JPRB ,&
&  0.25779E+00_JPRB, 0.26005E+00_JPRB, 0.26021E+00_JPRB, 0.25876E+00_JPRB, 0.25793E+00_JPRB, 0.25795E+00_JPRB, 0.25787E+00_JPRB ,&
&  0.25745E+00_JPRB, 0.25759E+00_JPRB, 0.25736E+00_JPRB, 0.25725E+00_JPRB, 0.25596E+00_JPRB, 0.25327E+00_JPRB, 0.25042E+00_JPRB ,&
&  0.24771E+00_JPRB, 0.24528E+00_JPRB, 0.24284E+00_JPRB, 0.24067E+00_JPRB, 0.23941E+00_JPRB, 0.23808E+00_JPRB, 0.23630E+00_JPRB ,&
&  0.23529E+00_JPRB, 0.23422E+00_JPRB, 0.23305E+00_JPRB, 0.23220E+00_JPRB, 0.23131E+00_JPRB, 0.23069E+00_JPRB, 0.22998E+00_JPRB ,&
&  0.22950E+00_JPRB, 0.22865E+00_JPRB, 0.22794E+00_JPRB, 0.22714E+00_JPRB, 0.22682E+00_JPRB, 0.22585E+00_JPRB, 0.22491E+00_JPRB ,&
&  0.22371E+00_JPRB, 0.22297E+00_JPRB, 0.22157E+00_JPRB, 0.22073E+00_JPRB, 0.21973E+00_JPRB, 0.21879E+00_JPRB, 0.21807E+00_JPRB ,&
&  0.21796E+00_JPRB, 0.21773E+00_JPRB, 0.21688E+00_JPRB, 0.21653E+00_JPRB, 0.21636E+00_JPRB, 0.21679E+00_JPRB, 0.21841E+00_JPRB ,&
&  0.21944E+00_JPRB, 0.22056E+00_JPRB, 0.22161E+00_JPRB, 0.22238E+00_JPRB, 0.22379E+00_JPRB, 0.22512E+00_JPRB, 0.22648E+00_JPRB ,&
&  0.22813E+00_JPRB, 0.22950E+00_JPRB, 0.23109E+00_JPRB, 0.23293E+00_JPRB, 0.23783E+00_JPRB, 0.24071E+00_JPRB, 0.24348E+00_JPRB ,&
&  0.24497E+00_JPRB, 0.24181E+00_JPRB, 0.25746E+00_JPRB, 0.26032E+00_JPRB, 0.26260E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.24168E+00_JPRB, 0.24473E+00_JPRB, 0.24626E+00_JPRB, 0.24710E+00_JPRB, 0.23734E+00_JPRB, 0.24919E+00_JPRB, 0.24568E+00_JPRB ,&
&  0.24531E+00_JPRB, 0.24548E+00_JPRB, 0.24820E+00_JPRB, 0.24888E+00_JPRB, 0.24866E+00_JPRB, 0.24809E+00_JPRB, 0.24716E+00_JPRB ,&
&  0.24621E+00_JPRB, 0.24486E+00_JPRB, 0.24414E+00_JPRB, 0.24331E+00_JPRB, 0.24267E+00_JPRB, 0.24197E+00_JPRB, 0.24151E+00_JPRB ,&
&  0.24168E+00_JPRB, 0.24279E+00_JPRB, 0.24469E+00_JPRB, 0.24383E+00_JPRB, 0.24367E+00_JPRB, 0.24399E+00_JPRB, 0.24460E+00_JPRB ,&
&  0.24477E+00_JPRB, 0.24475E+00_JPRB, 0.24431E+00_JPRB, 0.24293E+00_JPRB, 0.24324E+00_JPRB, 0.24363E+00_JPRB, 0.24410E+00_JPRB ,&
&  0.24432E+00_JPRB, 0.24425E+00_JPRB, 0.24342E+00_JPRB, 0.24263E+00_JPRB, 0.24181E+00_JPRB, 0.24076E+00_JPRB, 0.23941E+00_JPRB ,&
&  0.23831E+00_JPRB, 0.23747E+00_JPRB, 0.23639E+00_JPRB, 0.23529E+00_JPRB, 0.23434E+00_JPRB, 0.23340E+00_JPRB, 0.23248E+00_JPRB ,&
&  0.23189E+00_JPRB, 0.23146E+00_JPRB, 0.23098E+00_JPRB, 0.23028E+00_JPRB, 0.22952E+00_JPRB, 0.22880E+00_JPRB, 0.22816E+00_JPRB ,&
&  0.22775E+00_JPRB, 0.22703E+00_JPRB, 0.22631E+00_JPRB, 0.22552E+00_JPRB, 0.22493E+00_JPRB, 0.22392E+00_JPRB, 0.22304E+00_JPRB ,&
&  0.22218E+00_JPRB, 0.22139E+00_JPRB, 0.22076E+00_JPRB, 0.22028E+00_JPRB, 0.21983E+00_JPRB, 0.21948E+00_JPRB, 0.21918E+00_JPRB ,&
&  0.21893E+00_JPRB, 0.21869E+00_JPRB, 0.21855E+00_JPRB, 0.21833E+00_JPRB, 0.21808E+00_JPRB, 0.21794E+00_JPRB, 0.21822E+00_JPRB ,&
&  0.21844E+00_JPRB, 0.21857E+00_JPRB, 0.21861E+00_JPRB, 0.21812E+00_JPRB, 0.21857E+00_JPRB, 0.21896E+00_JPRB, 0.21945E+00_JPRB ,&
&  0.21986E+00_JPRB, 0.22080E+00_JPRB, 0.22196E+00_JPRB, 0.22339E+00_JPRB, 0.22622E+00_JPRB, 0.22864E+00_JPRB, 0.23124E+00_JPRB ,&
&  0.23414E+00_JPRB, 0.23842E+00_JPRB, 0.23995E+00_JPRB, 0.24085E+00_JPRB, 0.24137E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.22998E+00_JPRB, 0.22978E+00_JPRB, 0.22939E+00_JPRB, 0.22821E+00_JPRB, 0.22498E+00_JPRB, 0.22434E+00_JPRB, 0.22426E+00_JPRB ,&
&  0.22445E+00_JPRB, 0.22457E+00_JPRB, 0.22575E+00_JPRB, 0.22701E+00_JPRB, 0.22827E+00_JPRB, 0.22925E+00_JPRB, 0.23012E+00_JPRB ,&
&  0.23059E+00_JPRB, 0.23107E+00_JPRB, 0.22955E+00_JPRB, 0.22881E+00_JPRB, 0.22857E+00_JPRB, 0.22854E+00_JPRB, 0.22853E+00_JPRB ,&
&  0.22845E+00_JPRB, 0.22843E+00_JPRB, 0.22829E+00_JPRB, 0.22859E+00_JPRB, 0.22903E+00_JPRB, 0.22938E+00_JPRB, 0.22979E+00_JPRB ,&
&  0.23011E+00_JPRB, 0.23040E+00_JPRB, 0.23060E+00_JPRB, 0.23076E+00_JPRB, 0.23124E+00_JPRB, 0.23154E+00_JPRB, 0.23152E+00_JPRB ,&
&  0.23139E+00_JPRB, 0.23103E+00_JPRB, 0.23063E+00_JPRB, 0.23019E+00_JPRB, 0.22990E+00_JPRB, 0.22959E+00_JPRB, 0.22928E+00_JPRB ,&
&  0.22896E+00_JPRB, 0.22864E+00_JPRB, 0.22827E+00_JPRB, 0.22786E+00_JPRB, 0.22745E+00_JPRB, 0.22717E+00_JPRB, 0.22684E+00_JPRB ,&
&  0.22657E+00_JPRB, 0.22630E+00_JPRB, 0.22601E+00_JPRB, 0.22562E+00_JPRB, 0.22521E+00_JPRB, 0.22479E+00_JPRB, 0.22433E+00_JPRB ,&
&  0.22372E+00_JPRB, 0.22330E+00_JPRB, 0.22287E+00_JPRB, 0.22250E+00_JPRB, 0.22217E+00_JPRB, 0.22182E+00_JPRB, 0.22151E+00_JPRB ,&
&  0.22120E+00_JPRB, 0.22090E+00_JPRB, 0.22053E+00_JPRB, 0.22019E+00_JPRB, 0.21988E+00_JPRB, 0.21959E+00_JPRB, 0.21935E+00_JPRB ,&
&  0.21911E+00_JPRB, 0.21887E+00_JPRB, 0.21864E+00_JPRB, 0.21861E+00_JPRB, 0.21853E+00_JPRB, 0.21852E+00_JPRB, 0.21851E+00_JPRB ,&
&  0.21851E+00_JPRB, 0.21849E+00_JPRB, 0.21846E+00_JPRB, 0.21828E+00_JPRB, 0.21844E+00_JPRB, 0.21867E+00_JPRB, 0.21906E+00_JPRB ,&
&  0.21951E+00_JPRB, 0.22012E+00_JPRB, 0.22076E+00_JPRB, 0.22146E+00_JPRB, 0.22228E+00_JPRB, 0.22296E+00_JPRB, 0.22372E+00_JPRB ,&
&  0.22457E+00_JPRB, 0.22543E+00_JPRB, 0.22605E+00_JPRB, 0.22678E+00_JPRB, 0.22790E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB, 0.22251E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB, 0.80017E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.81187E+00_JPRB, 0.81059E+00_JPRB, 0.80949E+00_JPRB, 0.80888E+00_JPRB, 0.80856E+00_JPRB, 0.80826E+00_JPRB, 0.80691E+00_JPRB ,&
&  0.80527E+00_JPRB, 0.80325E+00_JPRB, 0.80114E+00_JPRB, 0.79857E+00_JPRB, 0.79540E+00_JPRB, 0.79164E+00_JPRB, 0.78824E+00_JPRB ,&
&  0.78505E+00_JPRB, 0.78242E+00_JPRB, 0.77966E+00_JPRB, 0.77773E+00_JPRB, 0.77553E+00_JPRB, 0.77329E+00_JPRB, 0.77095E+00_JPRB ,&
&  0.76908E+00_JPRB, 0.76734E+00_JPRB, 0.76569E+00_JPRB, 0.76426E+00_JPRB, 0.76367E+00_JPRB, 0.76287E+00_JPRB, 0.76202E+00_JPRB ,&
&  0.76128E+00_JPRB, 0.76086E+00_JPRB, 0.76053E+00_JPRB, 0.76017E+00_JPRB, 0.75992E+00_JPRB, 0.76012E+00_JPRB, 0.76032E+00_JPRB ,&
&  0.76074E+00_JPRB, 0.76184E+00_JPRB, 0.76348E+00_JPRB, 0.76497E+00_JPRB, 0.76665E+00_JPRB, 0.76899E+00_JPRB, 0.77186E+00_JPRB ,&
&  0.77442E+00_JPRB, 0.77686E+00_JPRB, 0.77968E+00_JPRB, 0.78295E+00_JPRB, 0.78612E+00_JPRB, 0.78957E+00_JPRB, 0.79288E+00_JPRB ,&
&  0.79617E+00_JPRB, 0.79882E+00_JPRB, 0.80141E+00_JPRB, 0.80268E+00_JPRB, 0.80449E+00_JPRB, 0.80601E+00_JPRB, 0.80775E+00_JPRB ,&
&  0.80822E+00_JPRB, 0.80850E+00_JPRB, 0.80891E+00_JPRB, 0.80962E+00_JPRB, 0.81005E+00_JPRB, 0.81062E+00_JPRB, 0.81132E+00_JPRB ,&
&  0.81233E+00_JPRB, 0.81286E+00_JPRB, 0.81369E+00_JPRB, 0.81427E+00_JPRB, 0.81505E+00_JPRB, 0.81521E+00_JPRB, 0.81574E+00_JPRB ,&
&  0.81632E+00_JPRB, 0.81723E+00_JPRB, 0.81737E+00_JPRB, 0.81762E+00_JPRB, 0.81778E+00_JPRB, 0.81809E+00_JPRB, 0.81838E+00_JPRB ,&
&  0.81879E+00_JPRB, 0.81907E+00_JPRB, 0.81932E+00_JPRB, 0.81901E+00_JPRB, 0.81830E+00_JPRB, 0.81766E+00_JPRB, 0.81716E+00_JPRB ,&
&  0.81689E+00_JPRB, 0.81656E+00_JPRB, 0.81663E+00_JPRB, 0.81695E+00_JPRB, 0.81765E+00_JPRB, 0.81761E+00_JPRB, 0.81755E+00_JPRB ,&
&  0.81732E+00_JPRB, 0.81698E+00_JPRB, 0.81592E+00_JPRB, 0.81466E+00_JPRB, 0.81326E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.81459E+00_JPRB, 0.81226E+00_JPRB, 0.81067E+00_JPRB, 0.81002E+00_JPRB, 0.80981E+00_JPRB, 0.80939E+00_JPRB, 0.80877E+00_JPRB ,&
&  0.80669E+00_JPRB, 0.80217E+00_JPRB, 0.79697E+00_JPRB, 0.79122E+00_JPRB, 0.78419E+00_JPRB, 0.77779E+00_JPRB, 0.77354E+00_JPRB ,&
&  0.76914E+00_JPRB, 0.76475E+00_JPRB, 0.76048E+00_JPRB, 0.75698E+00_JPRB, 0.75301E+00_JPRB, 0.74879E+00_JPRB, 0.74476E+00_JPRB ,&
&  0.74152E+00_JPRB, 0.73783E+00_JPRB, 0.73367E+00_JPRB, 0.72834E+00_JPRB, 0.72488E+00_JPRB, 0.72119E+00_JPRB, 0.71791E+00_JPRB ,&
&  0.71359E+00_JPRB, 0.71164E+00_JPRB, 0.71072E+00_JPRB, 0.71098E+00_JPRB, 0.71494E+00_JPRB, 0.71779E+00_JPRB, 0.71828E+00_JPRB ,&
&  0.71734E+00_JPRB, 0.71297E+00_JPRB, 0.71340E+00_JPRB, 0.71455E+00_JPRB, 0.71704E+00_JPRB, 0.72239E+00_JPRB, 0.72919E+00_JPRB ,&
&  0.73894E+00_JPRB, 0.75072E+00_JPRB, 0.76298E+00_JPRB, 0.77235E+00_JPRB, 0.78151E+00_JPRB, 0.78995E+00_JPRB, 0.79623E+00_JPRB ,&
&  0.80470E+00_JPRB, 0.81199E+00_JPRB, 0.81934E+00_JPRB, 0.82580E+00_JPRB, 0.83318E+00_JPRB, 0.83903E+00_JPRB, 0.84331E+00_JPRB ,&
&  0.84440E+00_JPRB, 0.84405E+00_JPRB, 0.84363E+00_JPRB, 0.84346E+00_JPRB, 0.84183E+00_JPRB, 0.84103E+00_JPRB, 0.84125E+00_JPRB ,&
&  0.84221E+00_JPRB, 0.84253E+00_JPRB, 0.84418E+00_JPRB, 0.84599E+00_JPRB, 0.84812E+00_JPRB, 0.84790E+00_JPRB, 0.84828E+00_JPRB ,&
&  0.84832E+00_JPRB, 0.84867E+00_JPRB, 0.84746E+00_JPRB, 0.84485E+00_JPRB, 0.84278E+00_JPRB, 0.84206E+00_JPRB, 0.84049E+00_JPRB ,&
&  0.83886E+00_JPRB, 0.83764E+00_JPRB, 0.83722E+00_JPRB, 0.83591E+00_JPRB, 0.83380E+00_JPRB, 0.83191E+00_JPRB, 0.83110E+00_JPRB ,&
&  0.83005E+00_JPRB, 0.82853E+00_JPRB, 0.82820E+00_JPRB, 0.82813E+00_JPRB, 0.83006E+00_JPRB, 0.83003E+00_JPRB, 0.82965E+00_JPRB ,&
&  0.82824E+00_JPRB, 0.82719E+00_JPRB, 0.82481E+00_JPRB, 0.82148E+00_JPRB, 0.81754E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.82887E+00_JPRB, 0.82787E+00_JPRB, 0.82887E+00_JPRB, 0.82744E+00_JPRB, 0.82483E+00_JPRB, 0.82044E+00_JPRB, 0.81761E+00_JPRB ,&
&  0.81256E+00_JPRB, 0.80464E+00_JPRB, 0.79292E+00_JPRB, 0.78296E+00_JPRB, 0.77314E+00_JPRB, 0.76675E+00_JPRB, 0.76077E+00_JPRB ,&
&  0.75479E+00_JPRB, 0.74983E+00_JPRB, 0.74458E+00_JPRB, 0.73797E+00_JPRB, 0.72969E+00_JPRB, 0.72000E+00_JPRB, 0.70712E+00_JPRB ,&
&  0.69784E+00_JPRB, 0.68895E+00_JPRB, 0.68156E+00_JPRB, 0.67678E+00_JPRB, 0.67619E+00_JPRB, 0.65435E+00_JPRB, 0.65125E+00_JPRB ,&
&  0.66353E+00_JPRB, 0.65087E+00_JPRB, 0.63835E+00_JPRB, 0.61799E+00_JPRB, 0.61079E+00_JPRB, 0.62164E+00_JPRB, 0.63012E+00_JPRB ,&
&  0.62321E+00_JPRB, 0.65676E+00_JPRB, 0.65970E+00_JPRB, 0.66081E+00_JPRB, 0.65804E+00_JPRB, 0.65156E+00_JPRB, 0.66178E+00_JPRB ,&
&  0.68709E+00_JPRB, 0.69793E+00_JPRB, 0.72125E+00_JPRB, 0.74693E+00_JPRB, 0.76460E+00_JPRB, 0.78280E+00_JPRB, 0.80124E+00_JPRB ,&
&  0.81948E+00_JPRB, 0.83350E+00_JPRB, 0.84534E+00_JPRB, 0.85602E+00_JPRB, 0.87143E+00_JPRB, 0.88658E+00_JPRB, 0.89984E+00_JPRB ,&
&  0.90891E+00_JPRB, 0.91453E+00_JPRB, 0.92118E+00_JPRB, 0.92783E+00_JPRB, 0.93716E+00_JPRB, 0.94538E+00_JPRB, 0.95373E+00_JPRB ,&
&  0.96037E+00_JPRB, 0.96589E+00_JPRB, 0.96950E+00_JPRB, 0.97117E+00_JPRB, 0.97169E+00_JPRB, 0.96876E+00_JPRB, 0.96832E+00_JPRB ,&
&  0.96264E+00_JPRB, 0.95597E+00_JPRB, 0.94712E+00_JPRB, 0.93544E+00_JPRB, 0.92680E+00_JPRB, 0.92005E+00_JPRB, 0.91244E+00_JPRB ,&
&  0.90453E+00_JPRB, 0.89657E+00_JPRB, 0.88948E+00_JPRB, 0.88068E+00_JPRB, 0.87242E+00_JPRB, 0.86403E+00_JPRB, 0.85837E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.85090E+00_JPRB, 0.84960E+00_JPRB, 0.84788E+00_JPRB, 0.84814E+00_JPRB, 0.84730E+00_JPRB, 0.84817E+00_JPRB ,&
&  0.84751E+00_JPRB, 0.84345E+00_JPRB, 0.83747E+00_JPRB, 0.83298E+00_JPRB, 0.82943E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.83401E+00_JPRB, 0.83596E+00_JPRB, 0.84171E+00_JPRB, 0.83986E+00_JPRB, 0.83606E+00_JPRB, 0.82651E+00_JPRB, 0.82144E+00_JPRB ,&
&  0.81216E+00_JPRB, 0.80315E+00_JPRB, 0.78494E+00_JPRB, 0.77320E+00_JPRB, 0.76241E+00_JPRB, 0.75573E+00_JPRB, 0.75039E+00_JPRB ,&
&  0.74617E+00_JPRB, 0.74057E+00_JPRB, 0.73373E+00_JPRB, 0.72342E+00_JPRB, 0.71044E+00_JPRB, 0.69561E+00_JPRB, 0.67455E+00_JPRB ,&
&  0.65690E+00_JPRB, 0.64452E+00_JPRB, 0.62840E+00_JPRB, 0.63528E+00_JPRB, 0.62742E+00_JPRB, 0.62132E+00_JPRB, 0.61717E+00_JPRB ,&
&  0.60558E+00_JPRB, 0.59169E+00_JPRB, 0.57190E+00_JPRB, 0.56197E+00_JPRB, 0.55533E+00_JPRB, 0.54706E+00_JPRB, 0.56171E+00_JPRB ,&
&  0.57267E+00_JPRB, 0.60183E+00_JPRB, 0.61380E+00_JPRB, 0.61651E+00_JPRB, 0.62060E+00_JPRB, 0.61919E+00_JPRB, 0.62740E+00_JPRB ,&
&  0.63805E+00_JPRB, 0.65147E+00_JPRB, 0.69651E+00_JPRB, 0.72483E+00_JPRB, 0.74413E+00_JPRB, 0.76270E+00_JPRB, 0.77655E+00_JPRB ,&
&  0.79432E+00_JPRB, 0.80645E+00_JPRB, 0.81885E+00_JPRB, 0.83424E+00_JPRB, 0.85687E+00_JPRB, 0.88321E+00_JPRB, 0.90675E+00_JPRB ,&
&  0.92761E+00_JPRB, 0.94097E+00_JPRB, 0.95801E+00_JPRB, 0.97768E+00_JPRB, 0.10043E+01_JPRB, 0.10340E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11307E+01_JPRB, 0.11576E+01_JPRB, 0.11817E+01_JPRB, 0.11949E+01_JPRB, 0.11982E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11612E+01_JPRB, 0.11347E+01_JPRB, 0.10934E+01_JPRB, 0.10620E+01_JPRB, 0.10452E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.98731E+00_JPRB, 0.96307E+00_JPRB, 0.94117E+00_JPRB, 0.91767E+00_JPRB, 0.89662E+00_JPRB, 0.87697E+00_JPRB, 0.86365E+00_JPRB ,&
&  0.85310E+00_JPRB, 0.84670E+00_JPRB, 0.84292E+00_JPRB, 0.84090E+00_JPRB, 0.84169E+00_JPRB, 0.84171E+00_JPRB, 0.84536E+00_JPRB ,&
&  0.84597E+00_JPRB, 0.84341E+00_JPRB, 0.83717E+00_JPRB, 0.83397E+00_JPRB, 0.83188E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.84064E+00_JPRB, 0.84117E+00_JPRB, 0.83919E+00_JPRB, 0.82910E+00_JPRB, 0.82247E+00_JPRB, 0.81476E+00_JPRB, 0.81206E+00_JPRB ,&
&  0.80272E+00_JPRB, 0.79432E+00_JPRB, 0.77637E+00_JPRB, 0.76533E+00_JPRB, 0.75283E+00_JPRB, 0.74559E+00_JPRB, 0.73860E+00_JPRB ,&
&  0.73355E+00_JPRB, 0.72716E+00_JPRB, 0.72260E+00_JPRB, 0.71235E+00_JPRB, 0.70244E+00_JPRB, 0.68700E+00_JPRB, 0.67004E+00_JPRB ,&
&  0.65210E+00_JPRB, 0.63392E+00_JPRB, 0.61963E+00_JPRB, 0.62237E+00_JPRB, 0.61446E+00_JPRB, 0.60853E+00_JPRB, 0.60178E+00_JPRB ,&
&  0.59708E+00_JPRB, 0.56041E+00_JPRB, 0.57157E+00_JPRB, 0.55793E+00_JPRB, 0.54297E+00_JPRB, 0.52374E+00_JPRB, 0.50792E+00_JPRB ,&
&  0.51186E+00_JPRB, 0.51486E+00_JPRB, 0.52737E+00_JPRB, 0.54648E+00_JPRB, 0.55502E+00_JPRB, 0.57091E+00_JPRB, 0.58169E+00_JPRB ,&
&  0.57717E+00_JPRB, 0.65353E+00_JPRB, 0.69271E+00_JPRB, 0.71502E+00_JPRB, 0.73730E+00_JPRB, 0.75571E+00_JPRB, 0.76835E+00_JPRB ,&
&  0.79068E+00_JPRB, 0.81595E+00_JPRB, 0.84345E+00_JPRB, 0.87110E+00_JPRB, 0.91102E+00_JPRB, 0.93912E+00_JPRB, 0.96530E+00_JPRB ,&
&  0.98699E+00_JPRB, 0.99753E+00_JPRB, 0.10155E+01_JPRB, 0.10430E+01_JPRB, 0.10784E+01_JPRB, 0.11183E+01_JPRB, 0.11628E+01_JPRB ,&
&  0.12135E+01_JPRB, 0.12623E+01_JPRB, 0.13141E+01_JPRB, 0.13592E+01_JPRB, 0.13947E+01_JPRB, 0.14175E+01_JPRB, 0.14418E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.14048E+01_JPRB, 0.13673E+01_JPRB, 0.13139E+01_JPRB, 0.12476E+01_JPRB, 0.11735E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10179E+01_JPRB, 0.97950E+00_JPRB, 0.94331E+00_JPRB, 0.91110E+00_JPRB, 0.88455E+00_JPRB, 0.86899E+00_JPRB ,&
&  0.85562E+00_JPRB, 0.84678E+00_JPRB, 0.84124E+00_JPRB, 0.83667E+00_JPRB, 0.83452E+00_JPRB, 0.82992E+00_JPRB, 0.82925E+00_JPRB ,&
&  0.82989E+00_JPRB, 0.83329E+00_JPRB, 0.83322E+00_JPRB, 0.83489E+00_JPRB, 0.83720E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.85042E+00_JPRB, 0.85322E+00_JPRB, 0.84868E+00_JPRB, 0.84197E+00_JPRB, 0.83697E+00_JPRB, 0.82478E+00_JPRB, 0.81741E+00_JPRB ,&
&  0.81163E+00_JPRB, 0.80582E+00_JPRB, 0.79404E+00_JPRB, 0.78188E+00_JPRB, 0.76592E+00_JPRB, 0.75394E+00_JPRB, 0.74074E+00_JPRB ,&
&  0.72976E+00_JPRB, 0.71688E+00_JPRB, 0.70248E+00_JPRB, 0.68901E+00_JPRB, 0.67868E+00_JPRB, 0.66579E+00_JPRB, 0.65645E+00_JPRB ,&
&  0.64592E+00_JPRB, 0.63231E+00_JPRB, 0.62280E+00_JPRB, 0.62294E+00_JPRB, 0.61396E+00_JPRB, 0.60534E+00_JPRB, 0.60644E+00_JPRB ,&
&  0.61173E+00_JPRB, 0.60609E+00_JPRB, 0.59831E+00_JPRB, 0.58261E+00_JPRB, 0.55353E+00_JPRB, 0.49493E+00_JPRB, 0.50731E+00_JPRB ,&
&  0.50222E+00_JPRB, 0.49256E+00_JPRB, 0.49654E+00_JPRB, 0.50635E+00_JPRB, 0.51686E+00_JPRB, 0.54542E+00_JPRB, 0.54820E+00_JPRB ,&
&  0.53801E+00_JPRB, 0.64599E+00_JPRB, 0.66983E+00_JPRB, 0.69788E+00_JPRB, 0.72486E+00_JPRB, 0.74687E+00_JPRB, 0.76906E+00_JPRB ,&
&  0.78740E+00_JPRB, 0.81533E+00_JPRB, 0.84179E+00_JPRB, 0.88174E+00_JPRB, 0.92076E+00_JPRB, 0.96786E+00_JPRB, 0.10095E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.10492E+01_JPRB, 0.10709E+01_JPRB, 0.10880E+01_JPRB, 0.11095E+01_JPRB, 0.11484E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.13214E+01_JPRB, 0.14205E+01_JPRB, 0.14954E+01_JPRB, 0.15347E+01_JPRB, 0.15896E+01_JPRB, 0.16522E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.16345E+01_JPRB, 0.16073E+01_JPRB, 0.14922E+01_JPRB, 0.13686E+01_JPRB, 0.12841E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.11459E+01_JPRB, 0.10854E+01_JPRB, 0.10220E+01_JPRB, 0.97405E+00_JPRB, 0.93687E+00_JPRB, 0.91010E+00_JPRB, 0.89583E+00_JPRB ,&
&  0.87969E+00_JPRB, 0.86638E+00_JPRB, 0.85370E+00_JPRB, 0.84858E+00_JPRB, 0.83677E+00_JPRB, 0.82689E+00_JPRB, 0.81645E+00_JPRB ,&
&  0.81468E+00_JPRB, 0.81585E+00_JPRB, 0.82416E+00_JPRB, 0.83301E+00_JPRB, 0.83882E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.85916E+00_JPRB, 0.87674E+00_JPRB, 0.88391E+00_JPRB, 0.88634E+00_JPRB, 0.88837E+00_JPRB, 0.87043E+00_JPRB, 0.85360E+00_JPRB ,&
&  0.84076E+00_JPRB, 0.82854E+00_JPRB, 0.81622E+00_JPRB, 0.80363E+00_JPRB, 0.78995E+00_JPRB, 0.77900E+00_JPRB, 0.76871E+00_JPRB ,&
&  0.75942E+00_JPRB, 0.71775E+00_JPRB, 0.69700E+00_JPRB, 0.68118E+00_JPRB, 0.66607E+00_JPRB, 0.65813E+00_JPRB, 0.65440E+00_JPRB ,&
&  0.64604E+00_JPRB, 0.64110E+00_JPRB, 0.63832E+00_JPRB, 0.63167E+00_JPRB, 0.62183E+00_JPRB, 0.61003E+00_JPRB, 0.60646E+00_JPRB ,&
&  0.61199E+00_JPRB, 0.59941E+00_JPRB, 0.58908E+00_JPRB, 0.57325E+00_JPRB, 0.54244E+00_JPRB, 0.52146E+00_JPRB, 0.51756E+00_JPRB ,&
&  0.51242E+00_JPRB, 0.48884E+00_JPRB, 0.49073E+00_JPRB, 0.49435E+00_JPRB, 0.49638E+00_JPRB, 0.50912E+00_JPRB, 0.53473E+00_JPRB ,&
&  0.56065E+00_JPRB, 0.64536E+00_JPRB, 0.67936E+00_JPRB, 0.71963E+00_JPRB, 0.75261E+00_JPRB, 0.78005E+00_JPRB, 0.80440E+00_JPRB ,&
&  0.82606E+00_JPRB, 0.85774E+00_JPRB, 0.89362E+00_JPRB, 0.94009E+00_JPRB, 0.98659E+00_JPRB, 0.10227E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.11032E+01_JPRB, 0.11194E+01_JPRB, 0.11191E+01_JPRB, 0.11358E+01_JPRB, 0.11869E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.13577E+01_JPRB, 0.15139E+01_JPRB, 0.16719E+01_JPRB, 0.17465E+01_JPRB, 0.18306E+01_JPRB, 0.18703E+01_JPRB ,&
&  0.17879E+01_JPRB, 0.17417E+01_JPRB, 0.17928E+01_JPRB, 0.16549E+01_JPRB, 0.15059E+01_JPRB, 0.13798E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.11149E+01_JPRB, 0.10586E+01_JPRB, 0.10158E+01_JPRB, 0.97317E+00_JPRB, 0.93647E+00_JPRB, 0.93056E+00_JPRB ,&
&  0.92515E+00_JPRB, 0.91791E+00_JPRB, 0.90617E+00_JPRB, 0.89261E+00_JPRB, 0.86636E+00_JPRB, 0.83867E+00_JPRB, 0.81484E+00_JPRB ,&
&  0.80842E+00_JPRB, 0.80741E+00_JPRB, 0.81719E+00_JPRB, 0.82642E+00_JPRB, 0.84021E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.86056E+00_JPRB, 0.87515E+00_JPRB, 0.88213E+00_JPRB, 0.89047E+00_JPRB, 0.90499E+00_JPRB, 0.85986E+00_JPRB, 0.82735E+00_JPRB ,&
&  0.80875E+00_JPRB, 0.80940E+00_JPRB, 0.80847E+00_JPRB, 0.80188E+00_JPRB, 0.79046E+00_JPRB, 0.77833E+00_JPRB, 0.76857E+00_JPRB ,&
&  0.76007E+00_JPRB, 0.73982E+00_JPRB, 0.70540E+00_JPRB, 0.65771E+00_JPRB, 0.64466E+00_JPRB, 0.64073E+00_JPRB, 0.64286E+00_JPRB ,&
&  0.64268E+00_JPRB, 0.63819E+00_JPRB, 0.63956E+00_JPRB, 0.62890E+00_JPRB, 0.62109E+00_JPRB, 0.60005E+00_JPRB, 0.58363E+00_JPRB ,&
&  0.58503E+00_JPRB, 0.58162E+00_JPRB, 0.59102E+00_JPRB, 0.58044E+00_JPRB, 0.57030E+00_JPRB, 0.56351E+00_JPRB, 0.50710E+00_JPRB ,&
&  0.49743E+00_JPRB, 0.48421E+00_JPRB, 0.48012E+00_JPRB, 0.47999E+00_JPRB, 0.49863E+00_JPRB, 0.51545E+00_JPRB, 0.53707E+00_JPRB ,&
&  0.55672E+00_JPRB, 0.65132E+00_JPRB, 0.68179E+00_JPRB, 0.71137E+00_JPRB, 0.74026E+00_JPRB, 0.77490E+00_JPRB, 0.82047E+00_JPRB ,&
&  0.86426E+00_JPRB, 0.90767E+00_JPRB, 0.94298E+00_JPRB, 0.98945E+00_JPRB, 0.10366E+01_JPRB, 0.10183E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.14173E+01_JPRB, 0.13360E+01_JPRB, 0.12604E+01_JPRB, 0.12221E+01_JPRB, 0.12238E+01_JPRB, 0.12402E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.13753E+01_JPRB, 0.15527E+01_JPRB, 0.18240E+01_JPRB, 0.20226E+01_JPRB, 0.20366E+01_JPRB, 0.20752E+01_JPRB, 0.20586E+01_JPRB ,&
&  0.19227E+01_JPRB, 0.18387E+01_JPRB, 0.19347E+01_JPRB, 0.16065E+01_JPRB, 0.13775E+01_JPRB, 0.12595E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.10881E+01_JPRB, 0.10514E+01_JPRB, 0.10084E+01_JPRB, 0.95458E+00_JPRB, 0.92341E+00_JPRB, 0.91881E+00_JPRB ,&
&  0.91462E+00_JPRB, 0.90633E+00_JPRB, 0.90078E+00_JPRB, 0.89103E+00_JPRB, 0.87175E+00_JPRB, 0.84983E+00_JPRB, 0.84192E+00_JPRB ,&
&  0.84003E+00_JPRB, 0.83618E+00_JPRB, 0.83330E+00_JPRB, 0.83404E+00_JPRB, 0.84273E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.85540E+00_JPRB, 0.86890E+00_JPRB, 0.87689E+00_JPRB, 0.86874E+00_JPRB, 0.85740E+00_JPRB, 0.79259E+00_JPRB, 0.72321E+00_JPRB ,&
&  0.67425E+00_JPRB, 0.67789E+00_JPRB, 0.69889E+00_JPRB, 0.71556E+00_JPRB, 0.71484E+00_JPRB, 0.70013E+00_JPRB, 0.69396E+00_JPRB ,&
&  0.69943E+00_JPRB, 0.69733E+00_JPRB, 0.67311E+00_JPRB, 0.65739E+00_JPRB, 0.63123E+00_JPRB, 0.62628E+00_JPRB, 0.62821E+00_JPRB ,&
&  0.63109E+00_JPRB, 0.62939E+00_JPRB, 0.62907E+00_JPRB, 0.62626E+00_JPRB, 0.62525E+00_JPRB, 0.61949E+00_JPRB, 0.60767E+00_JPRB ,&
&  0.59048E+00_JPRB, 0.58649E+00_JPRB, 0.59154E+00_JPRB, 0.60236E+00_JPRB, 0.59812E+00_JPRB, 0.59136E+00_JPRB, 0.52272E+00_JPRB ,&
&  0.49603E+00_JPRB, 0.47842E+00_JPRB, 0.48026E+00_JPRB, 0.49460E+00_JPRB, 0.52689E+00_JPRB, 0.53703E+00_JPRB, 0.60941E+00_JPRB ,&
&  0.66584E+00_JPRB, 0.69763E+00_JPRB, 0.71245E+00_JPRB, 0.72572E+00_JPRB, 0.75457E+00_JPRB, 0.81451E+00_JPRB, 0.88831E+00_JPRB ,&
&  0.93602E+00_JPRB, 0.96827E+00_JPRB, 0.96571E+00_JPRB, 0.83611E+00_JPRB, 0.83952E+00_JPRB, 0.91519E+00_JPRB, 0.10274E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.12024E+01_JPRB, 0.12078E+01_JPRB, 0.12640E+01_JPRB, 0.12646E+01_JPRB, 0.12659E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.14105E+01_JPRB, 0.16408E+01_JPRB, 0.20034E+01_JPRB, 0.21989E+01_JPRB, 0.21538E+01_JPRB, 0.21903E+01_JPRB, 0.21185E+01_JPRB ,&
&  0.20405E+01_JPRB, 0.18342E+01_JPRB, 0.16545E+01_JPRB, 0.14302E+01_JPRB, 0.12540E+01_JPRB, 0.11556E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.99658E+00_JPRB, 0.98922E+00_JPRB, 0.96242E+00_JPRB, 0.91008E+00_JPRB, 0.86295E+00_JPRB, 0.82715E+00_JPRB ,&
&  0.80223E+00_JPRB, 0.78973E+00_JPRB, 0.78907E+00_JPRB, 0.77349E+00_JPRB, 0.77530E+00_JPRB, 0.78830E+00_JPRB, 0.79451E+00_JPRB ,&
&  0.79369E+00_JPRB, 0.79241E+00_JPRB, 0.80172E+00_JPRB, 0.81543E+00_JPRB, 0.83039E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.90712E+00_JPRB, 0.88224E+00_JPRB, 0.84742E+00_JPRB, 0.84168E+00_JPRB, 0.81638E+00_JPRB, 0.79807E+00_JPRB, 0.72894E+00_JPRB ,&
&  0.64237E+00_JPRB, 0.61686E+00_JPRB, 0.63969E+00_JPRB, 0.64681E+00_JPRB, 0.63031E+00_JPRB, 0.62460E+00_JPRB, 0.62990E+00_JPRB ,&
&  0.63670E+00_JPRB, 0.66044E+00_JPRB, 0.65823E+00_JPRB, 0.64206E+00_JPRB, 0.63187E+00_JPRB, 0.62707E+00_JPRB, 0.63023E+00_JPRB ,&
&  0.63139E+00_JPRB, 0.63135E+00_JPRB, 0.62474E+00_JPRB, 0.61319E+00_JPRB, 0.61097E+00_JPRB, 0.60972E+00_JPRB, 0.60744E+00_JPRB ,&
&  0.60764E+00_JPRB, 0.60455E+00_JPRB, 0.61377E+00_JPRB, 0.62060E+00_JPRB, 0.61366E+00_JPRB, 0.59238E+00_JPRB, 0.53215E+00_JPRB ,&
&  0.49859E+00_JPRB, 0.48663E+00_JPRB, 0.51577E+00_JPRB, 0.58543E+00_JPRB, 0.63202E+00_JPRB, 0.67736E+00_JPRB, 0.71151E+00_JPRB ,&
&  0.68645E+00_JPRB, 0.69360E+00_JPRB, 0.73164E+00_JPRB, 0.84685E+00_JPRB, 0.89547E+00_JPRB, 0.95864E+00_JPRB, 0.10077E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.10795E+01_JPRB, 0.10843E+01_JPRB, 0.93274E+00_JPRB, 0.94963E+00_JPRB, 0.11568E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.12987E+01_JPRB, 0.13741E+01_JPRB, 0.13576E+01_JPRB, 0.12777E+01_JPRB, 0.12878E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.15394E+01_JPRB, 0.17968E+01_JPRB, 0.22864E+01_JPRB, 0.25655E+01_JPRB, 0.25305E+01_JPRB, 0.25139E+01_JPRB, 0.24662E+01_JPRB ,&
&  0.24206E+01_JPRB, 0.21948E+01_JPRB, 0.19003E+01_JPRB, 0.16248E+01_JPRB, 0.13886E+01_JPRB, 0.12042E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10187E+01_JPRB, 0.10141E+01_JPRB, 0.98660E+00_JPRB, 0.92392E+00_JPRB, 0.85431E+00_JPRB, 0.79762E+00_JPRB ,&
&  0.76160E+00_JPRB, 0.72760E+00_JPRB, 0.69338E+00_JPRB, 0.68242E+00_JPRB, 0.71956E+00_JPRB, 0.77134E+00_JPRB, 0.79002E+00_JPRB ,&
&  0.77651E+00_JPRB, 0.77511E+00_JPRB, 0.81361E+00_JPRB, 0.85505E+00_JPRB, 0.87221E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.93616E+00_JPRB, 0.89940E+00_JPRB, 0.86751E+00_JPRB, 0.83774E+00_JPRB, 0.81456E+00_JPRB, 0.77589E+00_JPRB, 0.70154E+00_JPRB ,&
&  0.62755E+00_JPRB, 0.59264E+00_JPRB, 0.60238E+00_JPRB, 0.58912E+00_JPRB, 0.56964E+00_JPRB, 0.55815E+00_JPRB, 0.56000E+00_JPRB ,&
&  0.57033E+00_JPRB, 0.59908E+00_JPRB, 0.61964E+00_JPRB, 0.62517E+00_JPRB, 0.62192E+00_JPRB, 0.61791E+00_JPRB, 0.62482E+00_JPRB ,&
&  0.63493E+00_JPRB, 0.64599E+00_JPRB, 0.65133E+00_JPRB, 0.64671E+00_JPRB, 0.63881E+00_JPRB, 0.63557E+00_JPRB, 0.63575E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.64213E+00_JPRB, 0.63035E+00_JPRB, 0.63215E+00_JPRB, 0.61182E+00_JPRB, 0.61017E+00_JPRB, 0.58168E+00_JPRB ,&
&  0.52977E+00_JPRB, 0.53971E+00_JPRB, 0.65952E+00_JPRB, 0.70349E+00_JPRB, 0.71934E+00_JPRB, 0.75519E+00_JPRB, 0.80456E+00_JPRB ,&
&  0.85021E+00_JPRB, 0.89341E+00_JPRB, 0.93296E+00_JPRB, 0.97665E+00_JPRB, 0.10175E+01_JPRB, 0.10730E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.12114E+01_JPRB, 0.10326E+01_JPRB, 0.98435E+00_JPRB, 0.10435E+01_JPRB, 0.13413E+01_JPRB, 0.14091E+01_JPRB, 0.14219E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.14839E+01_JPRB, 0.14707E+01_JPRB, 0.14912E+01_JPRB, 0.15606E+01_JPRB, 0.16778E+01_JPRB, 0.18679E+01_JPRB ,&
&  0.20459E+01_JPRB, 0.23897E+01_JPRB, 0.29045E+01_JPRB, 0.30398E+01_JPRB, 0.30081E+01_JPRB, 0.29958E+01_JPRB, 0.29032E+01_JPRB ,&
&  0.27795E+01_JPRB, 0.25359E+01_JPRB, 0.21329E+01_JPRB, 0.17946E+01_JPRB, 0.15148E+01_JPRB, 0.13028E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.10506E+01_JPRB, 0.10256E+01_JPRB, 0.10129E+01_JPRB, 0.98859E+00_JPRB, 0.94798E+00_JPRB, 0.89492E+00_JPRB, 0.83501E+00_JPRB ,&
&  0.76362E+00_JPRB, 0.72407E+00_JPRB, 0.68509E+00_JPRB, 0.70512E+00_JPRB, 0.76232E+00_JPRB, 0.82160E+00_JPRB, 0.84066E+00_JPRB ,&
&  0.85316E+00_JPRB, 0.84839E+00_JPRB, 0.85179E+00_JPRB, 0.87894E+00_JPRB, 0.91535E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.94439E+00_JPRB, 0.90045E+00_JPRB, 0.85306E+00_JPRB, 0.80393E+00_JPRB, 0.76519E+00_JPRB, 0.73327E+00_JPRB, 0.69602E+00_JPRB ,&
&  0.73499E+00_JPRB, 0.65319E+00_JPRB, 0.72707E+00_JPRB, 0.62217E+00_JPRB, 0.59183E+00_JPRB, 0.58385E+00_JPRB, 0.56551E+00_JPRB ,&
&  0.55398E+00_JPRB, 0.55649E+00_JPRB, 0.56563E+00_JPRB, 0.57760E+00_JPRB, 0.57805E+00_JPRB, 0.57754E+00_JPRB, 0.58632E+00_JPRB ,&
&  0.60335E+00_JPRB, 0.62838E+00_JPRB, 0.65634E+00_JPRB, 0.67130E+00_JPRB, 0.67622E+00_JPRB, 0.67309E+00_JPRB, 0.67719E+00_JPRB ,&
&  0.64512E+00_JPRB, 0.63594E+00_JPRB, 0.64048E+00_JPRB, 0.63926E+00_JPRB, 0.63457E+00_JPRB, 0.62159E+00_JPRB, 0.63666E+00_JPRB ,&
&  0.62785E+00_JPRB, 0.60765E+00_JPRB, 0.71583E+00_JPRB, 0.73421E+00_JPRB, 0.76463E+00_JPRB, 0.79863E+00_JPRB, 0.83638E+00_JPRB ,&
&  0.87607E+00_JPRB, 0.92344E+00_JPRB, 0.97705E+00_JPRB, 0.10169E+01_JPRB, 0.10791E+01_JPRB, 0.11533E+01_JPRB, 0.12210E+01_JPRB ,&
&  0.13052E+01_JPRB, 0.10586E+01_JPRB, 0.11574E+01_JPRB, 0.12221E+01_JPRB, 0.14686E+01_JPRB, 0.17489E+01_JPRB, 0.17850E+01_JPRB ,&
&  0.19973E+01_JPRB, 0.19824E+01_JPRB, 0.19567E+01_JPRB, 0.19313E+01_JPRB, 0.20596E+01_JPRB, 0.22372E+01_JPRB, 0.24390E+01_JPRB ,&
&  0.27139E+01_JPRB, 0.30942E+01_JPRB, 0.36439E+01_JPRB, 0.36776E+01_JPRB, 0.36178E+01_JPRB, 0.35207E+01_JPRB, 0.33332E+01_JPRB ,&
&  0.30729E+01_JPRB, 0.27535E+01_JPRB, 0.23527E+01_JPRB, 0.19187E+01_JPRB, 0.16242E+01_JPRB, 0.13774E+01_JPRB, 0.11758E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.95866E+00_JPRB, 0.95016E+00_JPRB, 0.92737E+00_JPRB, 0.91072E+00_JPRB, 0.87384E+00_JPRB, 0.81318E+00_JPRB ,&
&  0.75023E+00_JPRB, 0.76862E+00_JPRB, 0.82072E+00_JPRB, 0.91343E+00_JPRB, 0.95076E+00_JPRB, 0.95991E+00_JPRB, 0.10107E+01_JPRB ,&
&  0.93074E+00_JPRB, 0.96159E+00_JPRB, 0.10199E+01_JPRB, 0.99751E+00_JPRB, 0.97859E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.97280E+00_JPRB, 0.90520E+00_JPRB, 0.81278E+00_JPRB, 0.76248E+00_JPRB, 0.75925E+00_JPRB, 0.75630E+00_JPRB, 0.73075E+00_JPRB ,&
&  0.76047E+00_JPRB, 0.74472E+00_JPRB, 0.76983E+00_JPRB, 0.80607E+00_JPRB, 0.83971E+00_JPRB, 0.75089E+00_JPRB, 0.62310E+00_JPRB ,&
&  0.58211E+00_JPRB, 0.58626E+00_JPRB, 0.59117E+00_JPRB, 0.58883E+00_JPRB, 0.58921E+00_JPRB, 0.58550E+00_JPRB, 0.58591E+00_JPRB ,&
&  0.59208E+00_JPRB, 0.60712E+00_JPRB, 0.64344E+00_JPRB, 0.65983E+00_JPRB, 0.66774E+00_JPRB, 0.67096E+00_JPRB, 0.68197E+00_JPRB ,&
&  0.65207E+00_JPRB, 0.64099E+00_JPRB, 0.63513E+00_JPRB, 0.61127E+00_JPRB, 0.64507E+00_JPRB, 0.65831E+00_JPRB, 0.66865E+00_JPRB ,&
&  0.68158E+00_JPRB, 0.70619E+00_JPRB, 0.73855E+00_JPRB, 0.76882E+00_JPRB, 0.79790E+00_JPRB, 0.82859E+00_JPRB, 0.86140E+00_JPRB ,&
&  0.89545E+00_JPRB, 0.93786E+00_JPRB, 0.98757E+00_JPRB, 0.10474E+01_JPRB, 0.11777E+01_JPRB, 0.11647E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.13249E+01_JPRB, 0.14630E+01_JPRB, 0.16101E+01_JPRB, 0.15569E+01_JPRB, 0.16943E+01_JPRB, 0.19019E+01_JPRB, 0.18955E+01_JPRB ,&
&  0.19478E+01_JPRB, 0.19378E+01_JPRB, 0.20752E+01_JPRB, 0.22672E+01_JPRB, 0.25299E+01_JPRB, 0.27548E+01_JPRB, 0.29737E+01_JPRB ,&
&  0.33412E+01_JPRB, 0.40194E+01_JPRB, 0.43039E+01_JPRB, 0.41924E+01_JPRB, 0.40382E+01_JPRB, 0.38533E+01_JPRB, 0.35611E+01_JPRB ,&
&  0.32332E+01_JPRB, 0.28553E+01_JPRB, 0.24334E+01_JPRB, 0.20075E+01_JPRB, 0.16415E+01_JPRB, 0.14410E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.88855E+00_JPRB, 0.89220E+00_JPRB, 0.90016E+00_JPRB, 0.88711E+00_JPRB, 0.83404E+00_JPRB, 0.78752E+00_JPRB ,&
&  0.82173E+00_JPRB, 0.92825E+00_JPRB, 0.12066E+01_JPRB, 0.12713E+01_JPRB, 0.12793E+01_JPRB, 0.12233E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.91839E+00_JPRB, 0.10551E+01_JPRB, 0.10329E+01_JPRB, 0.10341E+01_JPRB, 0.10168E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.10145E+01_JPRB, 0.91485E+00_JPRB, 0.78934E+00_JPRB, 0.74958E+00_JPRB, 0.76542E+00_JPRB, 0.76653E+00_JPRB, 0.76450E+00_JPRB ,&
&  0.77805E+00_JPRB, 0.78461E+00_JPRB, 0.79511E+00_JPRB, 0.81112E+00_JPRB, 0.82132E+00_JPRB, 0.85332E+00_JPRB, 0.77758E+00_JPRB ,&
&  0.68026E+00_JPRB, 0.64928E+00_JPRB, 0.62385E+00_JPRB, 0.61812E+00_JPRB, 0.64158E+00_JPRB, 0.64494E+00_JPRB, 0.64449E+00_JPRB ,&
&  0.65250E+00_JPRB, 0.65372E+00_JPRB, 0.66737E+00_JPRB, 0.66565E+00_JPRB, 0.66187E+00_JPRB, 0.66775E+00_JPRB, 0.67651E+00_JPRB ,&
&  0.64153E+00_JPRB, 0.61391E+00_JPRB, 0.59662E+00_JPRB, 0.58782E+00_JPRB, 0.61787E+00_JPRB, 0.66275E+00_JPRB, 0.69036E+00_JPRB ,&
&  0.70869E+00_JPRB, 0.72906E+00_JPRB, 0.75317E+00_JPRB, 0.78350E+00_JPRB, 0.81690E+00_JPRB, 0.85076E+00_JPRB, 0.87780E+00_JPRB ,&
&  0.90696E+00_JPRB, 0.94068E+00_JPRB, 0.98792E+00_JPRB, 0.10688E+01_JPRB, 0.11179E+01_JPRB, 0.11893E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.14479E+01_JPRB, 0.15955E+01_JPRB, 0.17471E+01_JPRB, 0.20076E+01_JPRB, 0.22030E+01_JPRB, 0.21786E+01_JPRB, 0.21397E+01_JPRB ,&
&  0.21837E+01_JPRB, 0.22091E+01_JPRB, 0.25826E+01_JPRB, 0.28448E+01_JPRB, 0.30336E+01_JPRB, 0.32940E+01_JPRB, 0.35242E+01_JPRB ,&
&  0.39235E+01_JPRB, 0.43459E+01_JPRB, 0.44540E+01_JPRB, 0.43289E+01_JPRB, 0.41460E+01_JPRB, 0.39719E+01_JPRB, 0.36459E+01_JPRB ,&
&  0.32793E+01_JPRB, 0.28410E+01_JPRB, 0.23324E+01_JPRB, 0.19101E+01_JPRB, 0.15341E+01_JPRB, 0.14235E+01_JPRB, 0.12793E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.91750E+00_JPRB, 0.92531E+00_JPRB, 0.98911E+00_JPRB, 0.98024E+00_JPRB, 0.90406E+00_JPRB, 0.85712E+00_JPRB ,&
&  0.91623E+00_JPRB, 0.11340E+01_JPRB, 0.13014E+01_JPRB, 0.14199E+01_JPRB, 0.14289E+01_JPRB, 0.13353E+01_JPRB, 0.98673E+00_JPRB ,&
&  0.96807E+00_JPRB, 0.10886E+01_JPRB, 0.10850E+01_JPRB, 0.10767E+01_JPRB, 0.10600E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.10736E+01_JPRB, 0.95048E+00_JPRB, 0.83441E+00_JPRB, 0.77339E+00_JPRB, 0.78310E+00_JPRB, 0.79424E+00_JPRB, 0.79969E+00_JPRB ,&
&  0.81626E+00_JPRB, 0.83000E+00_JPRB, 0.84224E+00_JPRB, 0.85307E+00_JPRB, 0.85763E+00_JPRB, 0.85412E+00_JPRB, 0.89696E+00_JPRB ,&
&  0.75419E+00_JPRB, 0.69931E+00_JPRB, 0.65858E+00_JPRB, 0.65776E+00_JPRB, 0.74071E+00_JPRB, 0.76378E+00_JPRB, 0.79130E+00_JPRB ,&
&  0.78805E+00_JPRB, 0.79479E+00_JPRB, 0.78332E+00_JPRB, 0.75946E+00_JPRB, 0.73511E+00_JPRB, 0.72387E+00_JPRB, 0.70393E+00_JPRB ,&
&  0.65499E+00_JPRB, 0.61933E+00_JPRB, 0.60972E+00_JPRB, 0.61162E+00_JPRB, 0.64495E+00_JPRB, 0.67405E+00_JPRB, 0.72481E+00_JPRB ,&
&  0.75306E+00_JPRB, 0.77808E+00_JPRB, 0.80467E+00_JPRB, 0.83353E+00_JPRB, 0.85864E+00_JPRB, 0.89042E+00_JPRB, 0.92693E+00_JPRB ,&
&  0.96504E+00_JPRB, 0.10050E+01_JPRB, 0.10758E+01_JPRB, 0.11621E+01_JPRB, 0.11669E+01_JPRB, 0.12474E+01_JPRB, 0.14443E+01_JPRB ,&
&  0.16385E+01_JPRB, 0.17469E+01_JPRB, 0.20546E+01_JPRB, 0.29527E+01_JPRB, 0.32160E+01_JPRB, 0.31248E+01_JPRB, 0.29836E+01_JPRB ,&
&  0.29266E+01_JPRB, 0.28855E+01_JPRB, 0.30306E+01_JPRB, 0.32339E+01_JPRB, 0.34331E+01_JPRB, 0.37716E+01_JPRB, 0.40859E+01_JPRB ,&
&  0.42898E+01_JPRB, 0.43707E+01_JPRB, 0.43259E+01_JPRB, 0.41898E+01_JPRB, 0.39235E+01_JPRB, 0.37237E+01_JPRB, 0.34910E+01_JPRB ,&
&  0.31857E+01_JPRB, 0.27995E+01_JPRB, 0.22028E+01_JPRB, 0.18214E+01_JPRB, 0.13879E+01_JPRB, 0.12664E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.10029E+01_JPRB, 0.10495E+01_JPRB, 0.11620E+01_JPRB, 0.11879E+01_JPRB, 0.11605E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.91081E+00_JPRB, 0.10844E+01_JPRB, 0.15057E+01_JPRB, 0.16146E+01_JPRB, 0.15861E+01_JPRB, 0.14565E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.12115E+01_JPRB, 0.11567E+01_JPRB, 0.11502E+01_JPRB, 0.11436E+01_JPRB, 0.11292E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.11201E+01_JPRB, 0.99413E+00_JPRB, 0.89069E+00_JPRB, 0.82238E+00_JPRB, 0.82071E+00_JPRB, 0.84045E+00_JPRB, 0.85470E+00_JPRB ,&
&  0.86900E+00_JPRB, 0.88053E+00_JPRB, 0.88979E+00_JPRB, 0.89841E+00_JPRB, 0.90022E+00_JPRB, 0.87573E+00_JPRB, 0.85507E+00_JPRB ,&
&  0.84324E+00_JPRB, 0.74621E+00_JPRB, 0.71231E+00_JPRB, 0.69858E+00_JPRB, 0.77613E+00_JPRB, 0.84759E+00_JPRB, 0.93092E+00_JPRB ,&
&  0.10034E+01_JPRB, 0.10550E+01_JPRB, 0.10337E+01_JPRB, 0.97261E+00_JPRB, 0.90852E+00_JPRB, 0.85534E+00_JPRB, 0.78299E+00_JPRB ,&
&  0.70937E+00_JPRB, 0.67273E+00_JPRB, 0.66890E+00_JPRB, 0.70136E+00_JPRB, 0.76097E+00_JPRB, 0.72277E+00_JPRB, 0.78660E+00_JPRB ,&
&  0.82280E+00_JPRB, 0.85905E+00_JPRB, 0.87535E+00_JPRB, 0.89881E+00_JPRB, 0.95882E+00_JPRB, 0.10339E+01_JPRB, 0.11023E+01_JPRB ,&
&  0.11645E+01_JPRB, 0.11834E+01_JPRB, 0.11751E+01_JPRB, 0.12481E+01_JPRB, 0.13236E+01_JPRB, 0.13911E+01_JPRB, 0.15340E+01_JPRB ,&
&  0.17906E+01_JPRB, 0.19632E+01_JPRB, 0.23664E+01_JPRB, 0.34676E+01_JPRB, 0.38410E+01_JPRB, 0.37541E+01_JPRB, 0.38007E+01_JPRB ,&
&  0.38159E+01_JPRB, 0.37410E+01_JPRB, 0.37109E+01_JPRB, 0.36625E+01_JPRB, 0.39104E+01_JPRB, 0.43110E+01_JPRB, 0.46292E+01_JPRB ,&
&  0.45012E+01_JPRB, 0.41951E+01_JPRB, 0.40672E+01_JPRB, 0.38009E+01_JPRB, 0.35357E+01_JPRB, 0.32369E+01_JPRB, 0.32679E+01_JPRB ,&
&  0.33007E+01_JPRB, 0.23928E+01_JPRB, 0.19603E+01_JPRB, 0.18406E+01_JPRB, 0.13846E+01_JPRB, 0.11930E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.97329E+00_JPRB, 0.11174E+01_JPRB, 0.12723E+01_JPRB, 0.14542E+01_JPRB, 0.15406E+01_JPRB, 0.14650E+01_JPRB ,&
&  0.13512E+01_JPRB, 0.14293E+01_JPRB, 0.19315E+01_JPRB, 0.19782E+01_JPRB, 0.18189E+01_JPRB, 0.16139E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.13016E+01_JPRB, 0.12763E+01_JPRB, 0.12517E+01_JPRB, 0.12210E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.12293E+01_JPRB, 0.11001E+01_JPRB, 0.98013E+00_JPRB, 0.91385E+00_JPRB, 0.91323E+00_JPRB, 0.92656E+00_JPRB, 0.94232E+00_JPRB ,&
&  0.92704E+00_JPRB, 0.91564E+00_JPRB, 0.92892E+00_JPRB, 0.93165E+00_JPRB, 0.92717E+00_JPRB, 0.89937E+00_JPRB, 0.85152E+00_JPRB ,&
&  0.82466E+00_JPRB, 0.80076E+00_JPRB, 0.75066E+00_JPRB, 0.76205E+00_JPRB, 0.80446E+00_JPRB, 0.94312E+00_JPRB, 0.10719E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.13160E+01_JPRB, 0.13363E+01_JPRB, 0.13110E+01_JPRB, 0.12322E+01_JPRB, 0.11143E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.90732E+00_JPRB, 0.86011E+00_JPRB, 0.82402E+00_JPRB, 0.89883E+00_JPRB, 0.92571E+00_JPRB, 0.88119E+00_JPRB, 0.97368E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.10434E+01_JPRB, 0.10605E+01_JPRB, 0.11239E+01_JPRB, 0.11927E+01_JPRB, 0.12177E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.12127E+01_JPRB, 0.12560E+01_JPRB, 0.12470E+01_JPRB, 0.13160E+01_JPRB, 0.15185E+01_JPRB, 0.18668E+01_JPRB ,&
&  0.22631E+01_JPRB, 0.22768E+01_JPRB, 0.23660E+01_JPRB, 0.34515E+01_JPRB, 0.44667E+01_JPRB, 0.44939E+01_JPRB, 0.44173E+01_JPRB ,&
&  0.44555E+01_JPRB, 0.42221E+01_JPRB, 0.40002E+01_JPRB, 0.39516E+01_JPRB, 0.45553E+01_JPRB, 0.49873E+01_JPRB, 0.51252E+01_JPRB ,&
&  0.47998E+01_JPRB, 0.43185E+01_JPRB, 0.40787E+01_JPRB, 0.37978E+01_JPRB, 0.33891E+01_JPRB, 0.32480E+01_JPRB, 0.33328E+01_JPRB ,&
&  0.32823E+01_JPRB, 0.29199E+01_JPRB, 0.22481E+01_JPRB, 0.20197E+01_JPRB, 0.16178E+01_JPRB, 0.14223E+01_JPRB, 0.14451E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.12138E+01_JPRB, 0.12784E+01_JPRB, 0.13627E+01_JPRB, 0.20804E+01_JPRB, 0.22799E+01_JPRB, 0.24591E+01_JPRB ,&
&  0.24388E+01_JPRB, 0.23096E+01_JPRB, 0.28175E+01_JPRB, 0.26791E+01_JPRB, 0.22818E+01_JPRB, 0.18797E+01_JPRB, 0.15810E+01_JPRB ,&
&  0.14679E+01_JPRB, 0.14655E+01_JPRB, 0.14234E+01_JPRB, 0.13799E+01_JPRB, 0.13425E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.13101E+01_JPRB, 0.11740E+01_JPRB, 0.10545E+01_JPRB, 0.98228E+00_JPRB, 0.96156E+00_JPRB, 0.96921E+00_JPRB, 0.96920E+00_JPRB ,&
&  0.95308E+00_JPRB, 0.95008E+00_JPRB, 0.95159E+00_JPRB, 0.93444E+00_JPRB, 0.91173E+00_JPRB, 0.88370E+00_JPRB, 0.84312E+00_JPRB ,&
&  0.81235E+00_JPRB, 0.79844E+00_JPRB, 0.76350E+00_JPRB, 0.78903E+00_JPRB, 0.87209E+00_JPRB, 0.10714E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.15632E+01_JPRB, 0.16869E+01_JPRB, 0.17225E+01_JPRB, 0.16987E+01_JPRB, 0.16730E+01_JPRB, 0.15802E+01_JPRB, 0.14346E+01_JPRB ,&
&  0.13760E+01_JPRB, 0.11726E+01_JPRB, 0.12290E+01_JPRB, 0.12074E+01_JPRB, 0.12604E+01_JPRB, 0.12541E+01_JPRB, 0.12212E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.11953E+01_JPRB, 0.11929E+01_JPRB, 0.12184E+01_JPRB, 0.12545E+01_JPRB, 0.12757E+01_JPRB, 0.13013E+01_JPRB ,&
&  0.13292E+01_JPRB, 0.12920E+01_JPRB, 0.12240E+01_JPRB, 0.12319E+01_JPRB, 0.16339E+01_JPRB, 0.22974E+01_JPRB, 0.29250E+01_JPRB ,&
&  0.29268E+01_JPRB, 0.26857E+01_JPRB, 0.38404E+01_JPRB, 0.45078E+01_JPRB, 0.45347E+01_JPRB, 0.50514E+01_JPRB, 0.50163E+01_JPRB ,&
&  0.45497E+01_JPRB, 0.40105E+01_JPRB, 0.36182E+01_JPRB, 0.38679E+01_JPRB, 0.55025E+01_JPRB, 0.59609E+01_JPRB, 0.59734E+01_JPRB ,&
&  0.53751E+01_JPRB, 0.47242E+01_JPRB, 0.41432E+01_JPRB, 0.37536E+01_JPRB, 0.35447E+01_JPRB, 0.37122E+01_JPRB, 0.35153E+01_JPRB ,&
&  0.32881E+01_JPRB, 0.34851E+01_JPRB, 0.29200E+01_JPRB, 0.23106E+01_JPRB, 0.19598E+01_JPRB, 0.18237E+01_JPRB, 0.17482E+01_JPRB ,&
&  0.16023E+01_JPRB, 0.15625E+01_JPRB, 0.16070E+01_JPRB, 0.19999E+01_JPRB, 0.30696E+01_JPRB, 0.32026E+01_JPRB, 0.32410E+01_JPRB ,&
&  0.33873E+01_JPRB, 0.41115E+01_JPRB, 0.36618E+01_JPRB, 0.31384E+01_JPRB, 0.26115E+01_JPRB, 0.21578E+01_JPRB, 0.18244E+01_JPRB ,&
&  0.16369E+01_JPRB, 0.16106E+01_JPRB, 0.15750E+01_JPRB, 0.15050E+01_JPRB, 0.14416E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.14318E+01_JPRB, 0.12685E+01_JPRB, 0.11841E+01_JPRB, 0.11047E+01_JPRB, 0.10667E+01_JPRB, 0.10370E+01_JPRB, 0.97801E+00_JPRB ,&
&  0.93690E+00_JPRB, 0.94584E+00_JPRB, 0.92997E+00_JPRB, 0.91613E+00_JPRB, 0.90052E+00_JPRB, 0.86517E+00_JPRB, 0.82871E+00_JPRB ,&
&  0.81560E+00_JPRB, 0.79617E+00_JPRB, 0.80876E+00_JPRB, 0.90928E+00_JPRB, 0.10312E+01_JPRB, 0.12046E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.18333E+01_JPRB, 0.20344E+01_JPRB, 0.20752E+01_JPRB, 0.21521E+01_JPRB, 0.21766E+01_JPRB, 0.21475E+01_JPRB, 0.19904E+01_JPRB ,&
&  0.17524E+01_JPRB, 0.17091E+01_JPRB, 0.17616E+01_JPRB, 0.16827E+01_JPRB, 0.16257E+01_JPRB, 0.15546E+01_JPRB, 0.14781E+01_JPRB ,&
&  0.13971E+01_JPRB, 0.13318E+01_JPRB, 0.13142E+01_JPRB, 0.13301E+01_JPRB, 0.13609E+01_JPRB, 0.14246E+01_JPRB, 0.14183E+01_JPRB ,&
&  0.13434E+01_JPRB, 0.12995E+01_JPRB, 0.12645E+01_JPRB, 0.15787E+01_JPRB, 0.23749E+01_JPRB, 0.37040E+01_JPRB, 0.41907E+01_JPRB ,&
&  0.43917E+01_JPRB, 0.39035E+01_JPRB, 0.40060E+01_JPRB, 0.46787E+01_JPRB, 0.40009E+01_JPRB, 0.44251E+01_JPRB, 0.45952E+01_JPRB ,&
&  0.41087E+01_JPRB, 0.39236E+01_JPRB, 0.44718E+01_JPRB, 0.54749E+01_JPRB, 0.67894E+01_JPRB, 0.82492E+01_JPRB, 0.80995E+01_JPRB ,&
&  0.66939E+01_JPRB, 0.54759E+01_JPRB, 0.43501E+01_JPRB, 0.40104E+01_JPRB, 0.35888E+01_JPRB, 0.35300E+01_JPRB, 0.28672E+01_JPRB ,&
&  0.29325E+01_JPRB, 0.25370E+01_JPRB, 0.27002E+01_JPRB, 0.23946E+01_JPRB, 0.22586E+01_JPRB, 0.20729E+01_JPRB, 0.19331E+01_JPRB ,&
&  0.19051E+01_JPRB, 0.19511E+01_JPRB, 0.22332E+01_JPRB, 0.31774E+01_JPRB, 0.44384E+01_JPRB, 0.38519E+01_JPRB, 0.46597E+01_JPRB ,&
&  0.60044E+01_JPRB, 0.57007E+01_JPRB, 0.40032E+01_JPRB, 0.33629E+01_JPRB, 0.27566E+01_JPRB, 0.23328E+01_JPRB, 0.20982E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.17919E+01_JPRB, 0.17295E+01_JPRB, 0.16585E+01_JPRB, 0.15779E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.15135E+01_JPRB, 0.13497E+01_JPRB, 0.12782E+01_JPRB, 0.12456E+01_JPRB, 0.11846E+01_JPRB, 0.10606E+01_JPRB, 0.99001E+00_JPRB ,&
&  0.99979E+00_JPRB, 0.97117E+00_JPRB, 0.91235E+00_JPRB, 0.87591E+00_JPRB, 0.86883E+00_JPRB, 0.86835E+00_JPRB, 0.84041E+00_JPRB ,&
&  0.85310E+00_JPRB, 0.91144E+00_JPRB, 0.98851E+00_JPRB, 0.10927E+01_JPRB, 0.12497E+01_JPRB, 0.13931E+01_JPRB, 0.15557E+01_JPRB ,&
&  0.22489E+01_JPRB, 0.24457E+01_JPRB, 0.23381E+01_JPRB, 0.23842E+01_JPRB, 0.24913E+01_JPRB, 0.25453E+01_JPRB, 0.24236E+01_JPRB ,&
&  0.23999E+01_JPRB, 0.23687E+01_JPRB, 0.20305E+01_JPRB, 0.17927E+01_JPRB, 0.16462E+01_JPRB, 0.15664E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.13644E+01_JPRB, 0.12838E+01_JPRB, 0.12747E+01_JPRB, 0.12761E+01_JPRB, 0.13164E+01_JPRB, 0.12572E+01_JPRB, 0.12434E+01_JPRB ,&
&  0.12573E+01_JPRB, 0.13185E+01_JPRB, 0.14564E+01_JPRB, 0.20505E+01_JPRB, 0.33579E+01_JPRB, 0.50801E+01_JPRB, 0.55446E+01_JPRB ,&
&  0.48421E+01_JPRB, 0.40627E+01_JPRB, 0.42109E+01_JPRB, 0.46720E+01_JPRB, 0.45080E+01_JPRB, 0.42700E+01_JPRB, 0.40082E+01_JPRB ,&
&  0.33992E+01_JPRB, 0.35717E+01_JPRB, 0.45461E+01_JPRB, 0.62049E+01_JPRB, 0.85712E+01_JPRB, 0.10864E+02_JPRB, 0.10556E+02_JPRB ,&
&  0.83580E+01_JPRB, 0.65524E+01_JPRB, 0.52371E+01_JPRB, 0.41753E+01_JPRB, 0.40725E+01_JPRB, 0.31398E+01_JPRB, 0.35208E+01_JPRB ,&
&  0.35621E+01_JPRB, 0.34267E+01_JPRB, 0.28454E+01_JPRB, 0.23457E+01_JPRB, 0.19634E+01_JPRB, 0.21717E+01_JPRB, 0.21434E+01_JPRB ,&
&  0.23126E+01_JPRB, 0.25300E+01_JPRB, 0.42933E+01_JPRB, 0.51030E+01_JPRB, 0.55417E+01_JPRB, 0.55195E+01_JPRB, 0.71845E+01_JPRB ,&
&  0.66964E+01_JPRB, 0.57237E+01_JPRB, 0.44620E+01_JPRB, 0.36139E+01_JPRB, 0.29733E+01_JPRB, 0.26055E+01_JPRB, 0.23783E+01_JPRB ,&
&  0.21370E+01_JPRB, 0.20079E+01_JPRB, 0.19106E+01_JPRB, 0.18087E+01_JPRB, 0.16863E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.15014E+01_JPRB, 0.14154E+01_JPRB, 0.13891E+01_JPRB, 0.12884E+01_JPRB, 0.11280E+01_JPRB, 0.10243E+01_JPRB, 0.10100E+01_JPRB ,&
&  0.99681E+00_JPRB, 0.95616E+00_JPRB, 0.90396E+00_JPRB, 0.86775E+00_JPRB, 0.87234E+00_JPRB, 0.88345E+00_JPRB, 0.89750E+00_JPRB ,&
&  0.98067E+00_JPRB, 0.11594E+01_JPRB, 0.12857E+01_JPRB, 0.14178E+01_JPRB, 0.15188E+01_JPRB, 0.15931E+01_JPRB, 0.18149E+01_JPRB ,&
&  0.26793E+01_JPRB, 0.25548E+01_JPRB, 0.21504E+01_JPRB, 0.21963E+01_JPRB, 0.25382E+01_JPRB, 0.24649E+01_JPRB, 0.24166E+01_JPRB ,&
&  0.24282E+01_JPRB, 0.22662E+01_JPRB, 0.19582E+01_JPRB, 0.16971E+01_JPRB, 0.15330E+01_JPRB, 0.14368E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.12670E+01_JPRB, 0.13047E+01_JPRB, 0.13400E+01_JPRB, 0.13164E+01_JPRB, 0.12916E+01_JPRB, 0.13028E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.14212E+01_JPRB, 0.16761E+01_JPRB, 0.25060E+01_JPRB, 0.39734E+01_JPRB, 0.59208E+01_JPRB, 0.67630E+01_JPRB ,&
&  0.53420E+01_JPRB, 0.48532E+01_JPRB, 0.47836E+01_JPRB, 0.58319E+01_JPRB, 0.44283E+01_JPRB, 0.41144E+01_JPRB, 0.40215E+01_JPRB ,&
&  0.38504E+01_JPRB, 0.46208E+01_JPRB, 0.63765E+01_JPRB, 0.84294E+01_JPRB, 0.90514E+01_JPRB, 0.10607E+02_JPRB, 0.12186E+02_JPRB ,&
&  0.89330E+01_JPRB, 0.68456E+01_JPRB, 0.57490E+01_JPRB, 0.53268E+01_JPRB, 0.35124E+01_JPRB, 0.24703E+01_JPRB, 0.35779E+01_JPRB ,&
&  0.35252E+01_JPRB, 0.20302E+01_JPRB, 0.15040E+01_JPRB, 0.17014E+01_JPRB, 0.12763E+01_JPRB, 0.14849E+01_JPRB, 0.22023E+01_JPRB ,&
&  0.27751E+01_JPRB, 0.33850E+01_JPRB, 0.67387E+01_JPRB, 0.62981E+01_JPRB, 0.67360E+01_JPRB, 0.81589E+01_JPRB, 0.85467E+01_JPRB ,&
&  0.74512E+01_JPRB, 0.46553E+01_JPRB, 0.41699E+01_JPRB, 0.36301E+01_JPRB, 0.29708E+01_JPRB, 0.24915E+01_JPRB, 0.21530E+01_JPRB ,&
&  0.18874E+01_JPRB, 0.17285E+01_JPRB, 0.16361E+01_JPRB, 0.15920E+01_JPRB, 0.15863E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.12982E+01_JPRB, 0.12759E+01_JPRB, 0.12292E+01_JPRB, 0.11049E+01_JPRB, 0.10158E+01_JPRB, 0.99235E+00_JPRB, 0.98288E+00_JPRB ,&
&  0.96585E+00_JPRB, 0.90757E+00_JPRB, 0.87611E+00_JPRB, 0.88484E+00_JPRB, 0.90142E+00_JPRB, 0.92306E+00_JPRB, 0.97207E+00_JPRB ,&
&  0.10597E+01_JPRB, 0.12379E+01_JPRB, 0.16118E+01_JPRB, 0.18768E+01_JPRB, 0.20124E+01_JPRB, 0.17173E+01_JPRB, 0.21958E+01_JPRB ,&
&  0.28003E+01_JPRB, 0.23165E+01_JPRB, 0.19466E+01_JPRB, 0.21276E+01_JPRB, 0.22579E+01_JPRB, 0.20962E+01_JPRB, 0.20748E+01_JPRB ,&
&  0.20889E+01_JPRB, 0.19685E+01_JPRB, 0.17945E+01_JPRB, 0.16888E+01_JPRB, 0.15303E+01_JPRB, 0.13720E+01_JPRB, 0.13464E+01_JPRB ,&
&  0.13615E+01_JPRB, 0.14369E+01_JPRB, 0.15122E+01_JPRB, 0.14777E+01_JPRB, 0.14723E+01_JPRB, 0.14539E+01_JPRB, 0.14658E+01_JPRB ,&
&  0.15150E+01_JPRB, 0.16695E+01_JPRB, 0.20941E+01_JPRB, 0.32767E+01_JPRB, 0.53808E+01_JPRB, 0.79604E+01_JPRB, 0.65717E+01_JPRB ,&
&  0.48554E+01_JPRB, 0.43203E+01_JPRB, 0.53438E+01_JPRB, 0.55380E+01_JPRB, 0.42744E+01_JPRB, 0.42380E+01_JPRB, 0.42084E+01_JPRB ,&
&  0.49064E+01_JPRB, 0.59450E+01_JPRB, 0.66252E+01_JPRB, 0.83850E+01_JPRB, 0.10560E+02_JPRB, 0.10917E+02_JPRB, 0.11104E+02_JPRB ,&
&  0.89889E+01_JPRB, 0.67956E+01_JPRB, 0.48803E+01_JPRB, 0.38811E+01_JPRB, 0.44266E+01_JPRB, 0.33161E+01_JPRB, 0.21200E+01_JPRB ,&
&  0.16474E+01_JPRB, 0.12969E+01_JPRB, 0.10440E+01_JPRB, 0.10195E+01_JPRB, 0.91076E+00_JPRB, 0.12575E+01_JPRB, 0.20889E+01_JPRB ,&
&  0.40011E+01_JPRB, 0.56443E+01_JPRB, 0.74227E+01_JPRB, 0.75678E+01_JPRB, 0.70659E+01_JPRB, 0.85777E+01_JPRB, 0.93562E+01_JPRB ,&
&  0.57788E+01_JPRB, 0.40904E+01_JPRB, 0.38274E+01_JPRB, 0.30902E+01_JPRB, 0.23232E+01_JPRB, 0.18795E+01_JPRB, 0.16045E+01_JPRB ,&
&  0.14522E+01_JPRB, 0.13620E+01_JPRB, 0.13096E+01_JPRB, 0.13080E+01_JPRB, 0.13274E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.11255E+01_JPRB, 0.11053E+01_JPRB, 0.10824E+01_JPRB, 0.10185E+01_JPRB, 0.96705E+00_JPRB, 0.97205E+00_JPRB, 0.97673E+00_JPRB ,&
&  0.94172E+00_JPRB, 0.89808E+00_JPRB, 0.89773E+00_JPRB, 0.92518E+00_JPRB, 0.96074E+00_JPRB, 0.10220E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.11883E+01_JPRB, 0.14425E+01_JPRB, 0.19594E+01_JPRB, 0.24500E+01_JPRB, 0.27327E+01_JPRB, 0.21442E+01_JPRB, 0.25962E+01_JPRB ,&
&  0.27483E+01_JPRB, 0.22199E+01_JPRB, 0.20188E+01_JPRB, 0.21543E+01_JPRB, 0.21730E+01_JPRB, 0.18855E+01_JPRB, 0.19047E+01_JPRB ,&
&  0.18891E+01_JPRB, 0.18209E+01_JPRB, 0.17298E+01_JPRB, 0.16826E+01_JPRB, 0.15616E+01_JPRB, 0.15899E+01_JPRB, 0.16921E+01_JPRB ,&
&  0.17480E+01_JPRB, 0.18272E+01_JPRB, 0.17927E+01_JPRB, 0.17501E+01_JPRB, 0.17305E+01_JPRB, 0.17238E+01_JPRB, 0.17619E+01_JPRB ,&
&  0.18792E+01_JPRB, 0.21917E+01_JPRB, 0.30188E+01_JPRB, 0.48769E+01_JPRB, 0.65951E+01_JPRB, 0.66653E+01_JPRB, 0.59371E+01_JPRB ,&
&  0.53015E+01_JPRB, 0.52691E+01_JPRB, 0.52598E+01_JPRB, 0.57919E+01_JPRB, 0.44689E+01_JPRB, 0.42928E+01_JPRB, 0.43934E+01_JPRB ,&
&  0.51867E+01_JPRB, 0.71044E+01_JPRB, 0.78500E+01_JPRB, 0.89737E+01_JPRB, 0.12954E+02_JPRB, 0.13437E+02_JPRB, 0.10411E+02_JPRB ,&
&  0.88363E+01_JPRB, 0.66197E+01_JPRB, 0.52645E+01_JPRB, 0.45664E+01_JPRB, 0.66398E+01_JPRB, 0.69673E+01_JPRB, 0.24136E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.11104E+01_JPRB, 0.81793E+00_JPRB, 0.79084E+00_JPRB, 0.85390E+00_JPRB, 0.12726E+01_JPRB, 0.31689E+01_JPRB ,&
&  0.37545E+01_JPRB, 0.60981E+01_JPRB, 0.68552E+01_JPRB, 0.74350E+01_JPRB, 0.70163E+01_JPRB, 0.71027E+01_JPRB, 0.59273E+01_JPRB ,&
&  0.49238E+01_JPRB, 0.38110E+01_JPRB, 0.28575E+01_JPRB, 0.22038E+01_JPRB, 0.17342E+01_JPRB, 0.14861E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.12572E+01_JPRB, 0.12157E+01_JPRB, 0.11759E+01_JPRB, 0.11307E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.97915E+00_JPRB, 0.99880E+00_JPRB, 0.10258E+01_JPRB, 0.10214E+01_JPRB, 0.10166E+01_JPRB, 0.10415E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.98672E+00_JPRB, 0.95854E+00_JPRB, 0.97534E+00_JPRB, 0.10145E+01_JPRB, 0.10772E+01_JPRB, 0.11649E+01_JPRB, 0.12613E+01_JPRB ,&
&  0.14429E+01_JPRB, 0.18550E+01_JPRB, 0.24463E+01_JPRB, 0.31953E+01_JPRB, 0.34083E+01_JPRB, 0.30391E+01_JPRB, 0.30736E+01_JPRB ,&
&  0.28262E+01_JPRB, 0.23510E+01_JPRB, 0.21050E+01_JPRB, 0.22190E+01_JPRB, 0.22021E+01_JPRB, 0.20344E+01_JPRB, 0.19295E+01_JPRB ,&
&  0.19401E+01_JPRB, 0.19685E+01_JPRB, 0.19500E+01_JPRB, 0.19999E+01_JPRB, 0.20985E+01_JPRB, 0.22149E+01_JPRB, 0.23115E+01_JPRB ,&
&  0.23562E+01_JPRB, 0.22943E+01_JPRB, 0.21813E+01_JPRB, 0.21110E+01_JPRB, 0.20968E+01_JPRB, 0.21403E+01_JPRB, 0.22589E+01_JPRB ,&
&  0.25102E+01_JPRB, 0.31276E+01_JPRB, 0.44532E+01_JPRB, 0.61425E+01_JPRB, 0.60289E+01_JPRB, 0.64305E+01_JPRB, 0.58543E+01_JPRB ,&
&  0.52691E+01_JPRB, 0.51814E+01_JPRB, 0.48810E+01_JPRB, 0.52037E+01_JPRB, 0.47968E+01_JPRB, 0.47342E+01_JPRB, 0.49037E+01_JPRB ,&
&  0.68972E+01_JPRB, 0.83560E+01_JPRB, 0.10153E+02_JPRB, 0.10894E+02_JPRB, 0.13393E+02_JPRB, 0.17182E+02_JPRB, 0.10719E+02_JPRB ,&
&  0.76734E+01_JPRB, 0.70124E+01_JPRB, 0.56962E+01_JPRB, 0.50637E+01_JPRB, 0.62084E+01_JPRB, 0.84519E+01_JPRB, 0.62781E+01_JPRB ,&
&  0.19547E+01_JPRB, 0.12016E+01_JPRB, 0.10403E+01_JPRB, 0.10439E+01_JPRB, 0.86515E+00_JPRB, 0.13242E+01_JPRB, 0.45420E+01_JPRB ,&
&  0.40230E+01_JPRB, 0.53440E+01_JPRB, 0.59491E+01_JPRB, 0.64970E+01_JPRB, 0.65042E+01_JPRB, 0.48906E+01_JPRB, 0.37201E+01_JPRB ,&
&  0.24829E+01_JPRB, 0.18188E+01_JPRB, 0.15586E+01_JPRB, 0.13903E+01_JPRB, 0.11985E+01_JPRB, 0.11020E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.10181E+01_JPRB, 0.99697E+00_JPRB, 0.96962E+00_JPRB, 0.96992E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.94776E+00_JPRB, 0.10078E+01_JPRB, 0.10635E+01_JPRB, 0.11203E+01_JPRB, 0.11579E+01_JPRB, 0.11567E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.11044E+01_JPRB, 0.11293E+01_JPRB, 0.11778E+01_JPRB, 0.12805E+01_JPRB, 0.14277E+01_JPRB, 0.16534E+01_JPRB ,&
&  0.20129E+01_JPRB, 0.25227E+01_JPRB, 0.31320E+01_JPRB, 0.37125E+01_JPRB, 0.37623E+01_JPRB, 0.32303E+01_JPRB, 0.31676E+01_JPRB ,&
&  0.30017E+01_JPRB, 0.26170E+01_JPRB, 0.22789E+01_JPRB, 0.24620E+01_JPRB, 0.25465E+01_JPRB, 0.24828E+01_JPRB, 0.23683E+01_JPRB ,&
&  0.23868E+01_JPRB, 0.24342E+01_JPRB, 0.24941E+01_JPRB, 0.26178E+01_JPRB, 0.27377E+01_JPRB, 0.28228E+01_JPRB, 0.28544E+01_JPRB ,&
&  0.27682E+01_JPRB, 0.26362E+01_JPRB, 0.25670E+01_JPRB, 0.25749E+01_JPRB, 0.26385E+01_JPRB, 0.27364E+01_JPRB, 0.29630E+01_JPRB ,&
&  0.34621E+01_JPRB, 0.43608E+01_JPRB, 0.55527E+01_JPRB, 0.60635E+01_JPRB, 0.59150E+01_JPRB, 0.57176E+01_JPRB, 0.53445E+01_JPRB ,&
&  0.50363E+01_JPRB, 0.50794E+01_JPRB, 0.50168E+01_JPRB, 0.50401E+01_JPRB, 0.50288E+01_JPRB, 0.53830E+01_JPRB, 0.60417E+01_JPRB ,&
&  0.85351E+01_JPRB, 0.10519E+02_JPRB, 0.10974E+02_JPRB, 0.10998E+02_JPRB, 0.11598E+02_JPRB, 0.13380E+02_JPRB, 0.12264E+02_JPRB ,&
&  0.90105E+01_JPRB, 0.64647E+01_JPRB, 0.54115E+01_JPRB, 0.55104E+01_JPRB, 0.56801E+01_JPRB, 0.64952E+01_JPRB, 0.79034E+01_JPRB ,&
&  0.69710E+01_JPRB, 0.26675E+01_JPRB, 0.16835E+01_JPRB, 0.23971E+01_JPRB, 0.14205E+01_JPRB, 0.14071E+01_JPRB, 0.24184E+01_JPRB ,&
&  0.31346E+01_JPRB, 0.38482E+01_JPRB, 0.43406E+01_JPRB, 0.51448E+01_JPRB, 0.44265E+01_JPRB, 0.31924E+01_JPRB, 0.21791E+01_JPRB ,&
&  0.15214E+01_JPRB, 0.12743E+01_JPRB, 0.12174E+01_JPRB, 0.10818E+01_JPRB, 0.98085E+00_JPRB, 0.95364E+00_JPRB, 0.91992E+00_JPRB ,&
&  0.88653E+00_JPRB, 0.89565E+00_JPRB, 0.91236E+00_JPRB, 0.91117E+00_JPRB, 0.90717E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.10270E+01_JPRB, 0.11020E+01_JPRB, 0.12090E+01_JPRB, 0.12706E+01_JPRB, 0.12866E+01_JPRB, 0.12949E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.13910E+01_JPRB, 0.14658E+01_JPRB, 0.15886E+01_JPRB, 0.17976E+01_JPRB, 0.20566E+01_JPRB, 0.23886E+01_JPRB ,&
&  0.27940E+01_JPRB, 0.32387E+01_JPRB, 0.36997E+01_JPRB, 0.41388E+01_JPRB, 0.44177E+01_JPRB, 0.46587E+01_JPRB, 0.30854E+01_JPRB ,&
&  0.26334E+01_JPRB, 0.30209E+01_JPRB, 0.25094E+01_JPRB, 0.27797E+01_JPRB, 0.29263E+01_JPRB, 0.29681E+01_JPRB, 0.27675E+01_JPRB ,&
&  0.27721E+01_JPRB, 0.28532E+01_JPRB, 0.29505E+01_JPRB, 0.30074E+01_JPRB, 0.30381E+01_JPRB, 0.30622E+01_JPRB, 0.30395E+01_JPRB ,&
&  0.29926E+01_JPRB, 0.29761E+01_JPRB, 0.30219E+01_JPRB, 0.31069E+01_JPRB, 0.32384E+01_JPRB, 0.34583E+01_JPRB, 0.38339E+01_JPRB ,&
&  0.43850E+01_JPRB, 0.50480E+01_JPRB, 0.56375E+01_JPRB, 0.54636E+01_JPRB, 0.52939E+01_JPRB, 0.51157E+01_JPRB, 0.50649E+01_JPRB ,&
&  0.48650E+01_JPRB, 0.45881E+01_JPRB, 0.49341E+01_JPRB, 0.53214E+01_JPRB, 0.54911E+01_JPRB, 0.59413E+01_JPRB, 0.70534E+01_JPRB ,&
&  0.96431E+01_JPRB, 0.11152E+02_JPRB, 0.11726E+02_JPRB, 0.10032E+02_JPRB, 0.94054E+01_JPRB, 0.10356E+02_JPRB, 0.10630E+02_JPRB ,&
&  0.90050E+01_JPRB, 0.68713E+01_JPRB, 0.57683E+01_JPRB, 0.51619E+01_JPRB, 0.47944E+01_JPRB, 0.50686E+01_JPRB, 0.55778E+01_JPRB ,&
&  0.59155E+01_JPRB, 0.65571E+01_JPRB, 0.51911E+01_JPRB, 0.29747E+01_JPRB, 0.20373E+01_JPRB, 0.19188E+01_JPRB, 0.23323E+01_JPRB ,&
&  0.27030E+01_JPRB, 0.28554E+01_JPRB, 0.28347E+01_JPRB, 0.42602E+01_JPRB, 0.24604E+01_JPRB, 0.17180E+01_JPRB, 0.12472E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10654E+01_JPRB, 0.10615E+01_JPRB, 0.98641E+00_JPRB, 0.92256E+00_JPRB, 0.93766E+00_JPRB, 0.90348E+00_JPRB ,&
&  0.92275E+00_JPRB, 0.94982E+00_JPRB, 0.95471E+00_JPRB, 0.96328E+00_JPRB, 0.99555E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.12092E+01_JPRB, 0.12834E+01_JPRB, 0.13724E+01_JPRB, 0.14279E+01_JPRB, 0.14942E+01_JPRB, 0.15581E+01_JPRB, 0.16579E+01_JPRB ,&
&  0.17725E+01_JPRB, 0.19063E+01_JPRB, 0.20617E+01_JPRB, 0.22608E+01_JPRB, 0.25228E+01_JPRB, 0.28031E+01_JPRB, 0.31134E+01_JPRB ,&
&  0.34039E+01_JPRB, 0.36569E+01_JPRB, 0.39006E+01_JPRB, 0.41547E+01_JPRB, 0.43558E+01_JPRB, 0.46293E+01_JPRB, 0.40154E+01_JPRB ,&
&  0.24715E+01_JPRB, 0.36090E+01_JPRB, 0.26970E+01_JPRB, 0.30465E+01_JPRB, 0.31400E+01_JPRB, 0.33057E+01_JPRB, 0.30468E+01_JPRB ,&
&  0.29465E+01_JPRB, 0.29988E+01_JPRB, 0.30336E+01_JPRB, 0.30800E+01_JPRB, 0.31092E+01_JPRB, 0.31450E+01_JPRB, 0.31779E+01_JPRB ,&
&  0.32304E+01_JPRB, 0.33259E+01_JPRB, 0.34530E+01_JPRB, 0.35961E+01_JPRB, 0.38218E+01_JPRB, 0.41352E+01_JPRB, 0.44467E+01_JPRB ,&
&  0.47659E+01_JPRB, 0.50813E+01_JPRB, 0.51620E+01_JPRB, 0.49834E+01_JPRB, 0.48838E+01_JPRB, 0.49909E+01_JPRB, 0.52348E+01_JPRB ,&
&  0.51303E+01_JPRB, 0.49314E+01_JPRB, 0.52487E+01_JPRB, 0.54621E+01_JPRB, 0.56539E+01_JPRB, 0.66325E+01_JPRB, 0.76213E+01_JPRB ,&
&  0.94927E+01_JPRB, 0.98749E+01_JPRB, 0.11400E+02_JPRB, 0.94929E+01_JPRB, 0.78599E+01_JPRB, 0.81840E+01_JPRB, 0.74701E+01_JPRB ,&
&  0.61361E+01_JPRB, 0.55994E+01_JPRB, 0.48166E+01_JPRB, 0.39923E+01_JPRB, 0.39228E+01_JPRB, 0.40188E+01_JPRB, 0.44433E+01_JPRB ,&
&  0.48294E+01_JPRB, 0.48936E+01_JPRB, 0.35939E+01_JPRB, 0.22010E+01_JPRB, 0.19581E+01_JPRB, 0.20108E+01_JPRB, 0.23364E+01_JPRB ,&
&  0.23686E+01_JPRB, 0.23663E+01_JPRB, 0.25233E+01_JPRB, 0.23765E+01_JPRB, 0.14139E+01_JPRB, 0.10721E+01_JPRB, 0.94915E+00_JPRB ,&
&  0.96626E+00_JPRB, 0.10077E+01_JPRB, 0.10086E+01_JPRB, 0.98609E+00_JPRB, 0.93036E+00_JPRB, 0.97272E+00_JPRB, 0.98437E+00_JPRB ,&
&  0.10412E+01_JPRB, 0.10594E+01_JPRB, 0.10973E+01_JPRB, 0.11334E+01_JPRB, 0.11646E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.14508E+01_JPRB, 0.15595E+01_JPRB, 0.17184E+01_JPRB, 0.18621E+01_JPRB, 0.19776E+01_JPRB, 0.20765E+01_JPRB, 0.21552E+01_JPRB ,&
&  0.22180E+01_JPRB, 0.23015E+01_JPRB, 0.24961E+01_JPRB, 0.26954E+01_JPRB, 0.29433E+01_JPRB, 0.32136E+01_JPRB, 0.34706E+01_JPRB ,&
&  0.36854E+01_JPRB, 0.38846E+01_JPRB, 0.40898E+01_JPRB, 0.43736E+01_JPRB, 0.47185E+01_JPRB, 0.53137E+01_JPRB, 0.51166E+01_JPRB ,&
&  0.36867E+01_JPRB, 0.28487E+01_JPRB, 0.28812E+01_JPRB, 0.29696E+01_JPRB, 0.29214E+01_JPRB, 0.32002E+01_JPRB, 0.33292E+01_JPRB ,&
&  0.31094E+01_JPRB, 0.30647E+01_JPRB, 0.30833E+01_JPRB, 0.31210E+01_JPRB, 0.31053E+01_JPRB, 0.31930E+01_JPRB, 0.32971E+01_JPRB ,&
&  0.34438E+01_JPRB, 0.36116E+01_JPRB, 0.37863E+01_JPRB, 0.39946E+01_JPRB, 0.42225E+01_JPRB, 0.44712E+01_JPRB, 0.46875E+01_JPRB ,&
&  0.48619E+01_JPRB, 0.50045E+01_JPRB, 0.49819E+01_JPRB, 0.48142E+01_JPRB, 0.49200E+01_JPRB, 0.51632E+01_JPRB, 0.52893E+01_JPRB ,&
&  0.53096E+01_JPRB, 0.54537E+01_JPRB, 0.58090E+01_JPRB, 0.60431E+01_JPRB, 0.60758E+01_JPRB, 0.70387E+01_JPRB, 0.77532E+01_JPRB ,&
&  0.86027E+01_JPRB, 0.87936E+01_JPRB, 0.10325E+02_JPRB, 0.87331E+01_JPRB, 0.73827E+01_JPRB, 0.68569E+01_JPRB, 0.59451E+01_JPRB ,&
&  0.50459E+01_JPRB, 0.47234E+01_JPRB, 0.42148E+01_JPRB, 0.30992E+01_JPRB, 0.31891E+01_JPRB, 0.29908E+01_JPRB, 0.38841E+01_JPRB ,&
&  0.44700E+01_JPRB, 0.48905E+01_JPRB, 0.29354E+01_JPRB, 0.21932E+01_JPRB, 0.19196E+01_JPRB, 0.21079E+01_JPRB, 0.21582E+01_JPRB ,&
&  0.20191E+01_JPRB, 0.18125E+01_JPRB, 0.16839E+01_JPRB, 0.15986E+01_JPRB, 0.10411E+01_JPRB, 0.89738E+00_JPRB, 0.86315E+00_JPRB ,&
&  0.92146E+00_JPRB, 0.97794E+00_JPRB, 0.98901E+00_JPRB, 0.98361E+00_JPRB, 0.10011E+01_JPRB, 0.10719E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11811E+01_JPRB, 0.12259E+01_JPRB, 0.12794E+01_JPRB, 0.13136E+01_JPRB, 0.13620E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.15043E+01_JPRB, 0.16112E+01_JPRB, 0.17752E+01_JPRB, 0.19278E+01_JPRB, 0.20511E+01_JPRB, 0.21246E+01_JPRB, 0.21614E+01_JPRB ,&
&  0.22331E+01_JPRB, 0.23456E+01_JPRB, 0.25140E+01_JPRB, 0.26694E+01_JPRB, 0.28516E+01_JPRB, 0.30368E+01_JPRB, 0.32200E+01_JPRB ,&
&  0.33893E+01_JPRB, 0.35450E+01_JPRB, 0.37474E+01_JPRB, 0.40048E+01_JPRB, 0.43113E+01_JPRB, 0.47854E+01_JPRB, 0.48385E+01_JPRB ,&
&  0.38056E+01_JPRB, 0.23351E+01_JPRB, 0.25984E+01_JPRB, 0.26582E+01_JPRB, 0.27301E+01_JPRB, 0.30679E+01_JPRB, 0.31996E+01_JPRB ,&
&  0.28882E+01_JPRB, 0.28971E+01_JPRB, 0.30211E+01_JPRB, 0.30653E+01_JPRB, 0.30660E+01_JPRB, 0.32547E+01_JPRB, 0.34210E+01_JPRB ,&
&  0.36053E+01_JPRB, 0.37668E+01_JPRB, 0.39215E+01_JPRB, 0.40893E+01_JPRB, 0.42444E+01_JPRB, 0.44174E+01_JPRB, 0.46163E+01_JPRB ,&
&  0.48259E+01_JPRB, 0.49879E+01_JPRB, 0.49567E+01_JPRB, 0.49315E+01_JPRB, 0.50486E+01_JPRB, 0.53372E+01_JPRB, 0.56084E+01_JPRB ,&
&  0.57806E+01_JPRB, 0.58098E+01_JPRB, 0.62896E+01_JPRB, 0.69274E+01_JPRB, 0.72315E+01_JPRB, 0.70854E+01_JPRB, 0.76170E+01_JPRB ,&
&  0.79874E+01_JPRB, 0.79240E+01_JPRB, 0.84582E+01_JPRB, 0.93942E+01_JPRB, 0.64651E+01_JPRB, 0.61504E+01_JPRB, 0.48883E+01_JPRB ,&
&  0.49519E+01_JPRB, 0.45151E+01_JPRB, 0.39278E+01_JPRB, 0.32103E+01_JPRB, 0.26533E+01_JPRB, 0.24633E+01_JPRB, 0.31374E+01_JPRB ,&
&  0.39639E+01_JPRB, 0.34849E+01_JPRB, 0.23972E+01_JPRB, 0.20061E+01_JPRB, 0.19815E+01_JPRB, 0.24598E+01_JPRB, 0.24121E+01_JPRB ,&
&  0.20167E+01_JPRB, 0.17310E+01_JPRB, 0.16458E+01_JPRB, 0.20697E+01_JPRB, 0.12758E+01_JPRB, 0.87951E+00_JPRB, 0.80486E+00_JPRB ,&
&  0.87048E+00_JPRB, 0.93782E+00_JPRB, 0.99881E+00_JPRB, 0.10451E+01_JPRB, 0.10886E+01_JPRB, 0.11482E+01_JPRB, 0.11796E+01_JPRB ,&
&  0.12553E+01_JPRB, 0.12734E+01_JPRB, 0.13458E+01_JPRB, 0.13885E+01_JPRB, 0.14424E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.14914E+01_JPRB, 0.15695E+01_JPRB, 0.16519E+01_JPRB, 0.17501E+01_JPRB, 0.18528E+01_JPRB, 0.19293E+01_JPRB, 0.19768E+01_JPRB ,&
&  0.20415E+01_JPRB, 0.21344E+01_JPRB, 0.22436E+01_JPRB, 0.23372E+01_JPRB, 0.24747E+01_JPRB, 0.26025E+01_JPRB, 0.27314E+01_JPRB ,&
&  0.28606E+01_JPRB, 0.30116E+01_JPRB, 0.32100E+01_JPRB, 0.33898E+01_JPRB, 0.35061E+01_JPRB, 0.36229E+01_JPRB, 0.37450E+01_JPRB ,&
&  0.38585E+01_JPRB, 0.33054E+01_JPRB, 0.29155E+01_JPRB, 0.22641E+01_JPRB, 0.18577E+01_JPRB, 0.23682E+01_JPRB, 0.23867E+01_JPRB ,&
&  0.24325E+01_JPRB, 0.24557E+01_JPRB, 0.25193E+01_JPRB, 0.26421E+01_JPRB, 0.28481E+01_JPRB, 0.30530E+01_JPRB, 0.32242E+01_JPRB ,&
&  0.33662E+01_JPRB, 0.34806E+01_JPRB, 0.35756E+01_JPRB, 0.36836E+01_JPRB, 0.38129E+01_JPRB, 0.39369E+01_JPRB, 0.40698E+01_JPRB ,&
&  0.42464E+01_JPRB, 0.44794E+01_JPRB, 0.46400E+01_JPRB, 0.48314E+01_JPRB, 0.49971E+01_JPRB, 0.53969E+01_JPRB, 0.58285E+01_JPRB ,&
&  0.61080E+01_JPRB, 0.62644E+01_JPRB, 0.67313E+01_JPRB, 0.72787E+01_JPRB, 0.73644E+01_JPRB, 0.65761E+01_JPRB, 0.68940E+01_JPRB ,&
&  0.73230E+01_JPRB, 0.71251E+01_JPRB, 0.69607E+01_JPRB, 0.89527E+01_JPRB, 0.55129E+01_JPRB, 0.57801E+01_JPRB, 0.55550E+01_JPRB ,&
&  0.47119E+01_JPRB, 0.43802E+01_JPRB, 0.37260E+01_JPRB, 0.27761E+01_JPRB, 0.22735E+01_JPRB, 0.21037E+01_JPRB, 0.26604E+01_JPRB ,&
&  0.30697E+01_JPRB, 0.24923E+01_JPRB, 0.18153E+01_JPRB, 0.15812E+01_JPRB, 0.17676E+01_JPRB, 0.24794E+01_JPRB, 0.25485E+01_JPRB ,&
&  0.19190E+01_JPRB, 0.17508E+01_JPRB, 0.15972E+01_JPRB, 0.25573E+01_JPRB, 0.19011E+01_JPRB, 0.87967E+00_JPRB, 0.70936E+00_JPRB ,&
&  0.74113E+00_JPRB, 0.81635E+00_JPRB, 0.91952E+00_JPRB, 0.99305E+00_JPRB, 0.10291E+01_JPRB, 0.10738E+01_JPRB, 0.11176E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.12348E+01_JPRB, 0.13207E+01_JPRB, 0.13786E+01_JPRB, 0.14346E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.11903E+01_JPRB, 0.12502E+01_JPRB, 0.12987E+01_JPRB, 0.13255E+01_JPRB, 0.13388E+01_JPRB, 0.13362E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.13097E+01_JPRB, 0.14111E+01_JPRB, 0.16336E+01_JPRB, 0.18228E+01_JPRB, 0.19464E+01_JPRB, 0.20695E+01_JPRB ,&
&  0.21900E+01_JPRB, 0.22384E+01_JPRB, 0.21873E+01_JPRB, 0.21779E+01_JPRB, 0.21124E+01_JPRB, 0.20308E+01_JPRB, 0.22302E+01_JPRB ,&
&  0.25867E+01_JPRB, 0.29852E+01_JPRB, 0.32402E+01_JPRB, 0.34701E+01_JPRB, 0.28099E+01_JPRB, 0.23117E+01_JPRB, 0.25755E+01_JPRB ,&
&  0.29020E+01_JPRB, 0.26046E+01_JPRB, 0.21773E+01_JPRB, 0.21944E+01_JPRB, 0.22580E+01_JPRB, 0.23928E+01_JPRB, 0.25874E+01_JPRB ,&
&  0.27236E+01_JPRB, 0.28199E+01_JPRB, 0.29071E+01_JPRB, 0.29842E+01_JPRB, 0.30604E+01_JPRB, 0.31367E+01_JPRB, 0.32320E+01_JPRB ,&
&  0.33133E+01_JPRB, 0.33975E+01_JPRB, 0.36431E+01_JPRB, 0.39898E+01_JPRB, 0.43997E+01_JPRB, 0.49540E+01_JPRB, 0.54372E+01_JPRB ,&
&  0.57383E+01_JPRB, 0.59771E+01_JPRB, 0.66995E+01_JPRB, 0.71711E+01_JPRB, 0.67981E+01_JPRB, 0.61738E+01_JPRB, 0.64497E+01_JPRB ,&
&  0.65095E+01_JPRB, 0.60984E+01_JPRB, 0.48424E+01_JPRB, 0.70022E+01_JPRB, 0.70170E+01_JPRB, 0.59507E+01_JPRB, 0.48050E+01_JPRB ,&
&  0.42099E+01_JPRB, 0.39500E+01_JPRB, 0.34561E+01_JPRB, 0.27874E+01_JPRB, 0.22010E+01_JPRB, 0.24825E+01_JPRB, 0.26238E+01_JPRB ,&
&  0.26386E+01_JPRB, 0.20444E+01_JPRB, 0.14789E+01_JPRB, 0.13047E+01_JPRB, 0.13779E+01_JPRB, 0.21296E+01_JPRB, 0.20880E+01_JPRB ,&
&  0.15987E+01_JPRB, 0.14655E+01_JPRB, 0.13712E+01_JPRB, 0.20635E+01_JPRB, 0.32093E+01_JPRB, 0.93674E+00_JPRB, 0.62917E+00_JPRB ,&
&  0.63693E+00_JPRB, 0.67704E+00_JPRB, 0.71939E+00_JPRB, 0.73564E+00_JPRB, 0.74115E+00_JPRB, 0.77880E+00_JPRB, 0.84032E+00_JPRB ,&
&  0.91415E+00_JPRB, 0.97909E+00_JPRB, 0.10225E+01_JPRB, 0.10860E+01_JPRB, 0.11381E+01_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.83968E+00_JPRB, 0.86972E+00_JPRB, 0.87525E+00_JPRB, 0.88744E+00_JPRB, 0.88314E+00_JPRB, 0.88291E+00_JPRB, 0.85486E+00_JPRB ,&
&  0.83736E+00_JPRB, 0.86192E+00_JPRB, 0.93359E+00_JPRB, 0.10660E+01_JPRB, 0.11139E+01_JPRB, 0.11533E+01_JPRB, 0.11804E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.11662E+01_JPRB, 0.11262E+01_JPRB, 0.10530E+01_JPRB, 0.10245E+01_JPRB, 0.10492E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.15438E+01_JPRB, 0.18042E+01_JPRB, 0.21158E+01_JPRB, 0.28380E+01_JPRB, 0.25870E+01_JPRB, 0.25690E+01_JPRB ,&
&  0.30383E+01_JPRB, 0.23518E+01_JPRB, 0.20011E+01_JPRB, 0.17934E+01_JPRB, 0.16866E+01_JPRB, 0.16421E+01_JPRB, 0.17224E+01_JPRB ,&
&  0.17704E+01_JPRB, 0.18498E+01_JPRB, 0.19444E+01_JPRB, 0.19862E+01_JPRB, 0.20308E+01_JPRB, 0.20101E+01_JPRB, 0.19745E+01_JPRB ,&
&  0.19337E+01_JPRB, 0.20703E+01_JPRB, 0.23651E+01_JPRB, 0.25777E+01_JPRB, 0.31209E+01_JPRB, 0.36167E+01_JPRB, 0.40450E+01_JPRB ,&
&  0.42941E+01_JPRB, 0.46185E+01_JPRB, 0.52641E+01_JPRB, 0.54486E+01_JPRB, 0.55612E+01_JPRB, 0.53095E+01_JPRB, 0.51674E+01_JPRB ,&
&  0.48779E+01_JPRB, 0.39256E+01_JPRB, 0.32470E+01_JPRB, 0.48905E+01_JPRB, 0.50021E+01_JPRB, 0.39169E+01_JPRB, 0.40249E+01_JPRB ,&
&  0.36834E+01_JPRB, 0.34461E+01_JPRB, 0.32559E+01_JPRB, 0.26745E+01_JPRB, 0.20512E+01_JPRB, 0.21325E+01_JPRB, 0.24975E+01_JPRB ,&
&  0.21623E+01_JPRB, 0.15461E+01_JPRB, 0.11769E+01_JPRB, 0.11182E+01_JPRB, 0.12271E+01_JPRB, 0.17336E+01_JPRB, 0.18506E+01_JPRB ,&
&  0.16972E+01_JPRB, 0.12777E+01_JPRB, 0.14731E+01_JPRB, 0.23384E+01_JPRB, 0.16917E+01_JPRB, 0.83564E+00_JPRB, 0.55240E+00_JPRB ,&
&  0.53575E+00_JPRB, 0.55357E+00_JPRB, 0.57545E+00_JPRB, 0.58599E+00_JPRB, 0.59122E+00_JPRB, 0.60308E+00_JPRB, 0.64501E+00_JPRB ,&
&  0.68209E+00_JPRB, 0.71994E+00_JPRB, 0.75136E+00_JPRB, 0.79408E+00_JPRB, 0.82086E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.79911E+00_JPRB, 0.81904E+00_JPRB, 0.84421E+00_JPRB, 0.86097E+00_JPRB, 0.88225E+00_JPRB, 0.90024E+00_JPRB, 0.94477E+00_JPRB ,&
&  0.98674E+00_JPRB, 0.10234E+01_JPRB, 0.10726E+01_JPRB, 0.11253E+01_JPRB, 0.11433E+01_JPRB, 0.11609E+01_JPRB, 0.11738E+01_JPRB ,&
&  0.11964E+01_JPRB, 0.12122E+01_JPRB, 0.11968E+01_JPRB, 0.11921E+01_JPRB, 0.11858E+01_JPRB, 0.12673E+01_JPRB, 0.12569E+01_JPRB ,&
&  0.12402E+01_JPRB, 0.12602E+01_JPRB, 0.12719E+01_JPRB, 0.13424E+01_JPRB, 0.15621E+01_JPRB, 0.17872E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.23442E+01_JPRB, 0.19178E+01_JPRB, 0.17446E+01_JPRB, 0.13421E+01_JPRB, 0.12595E+01_JPRB, 0.12788E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.13049E+01_JPRB, 0.14016E+01_JPRB, 0.14194E+01_JPRB, 0.15068E+01_JPRB, 0.15796E+01_JPRB, 0.16324E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.18412E+01_JPRB, 0.19640E+01_JPRB, 0.21506E+01_JPRB, 0.23269E+01_JPRB, 0.27926E+01_JPRB, 0.30040E+01_JPRB ,&
&  0.30950E+01_JPRB, 0.32649E+01_JPRB, 0.30521E+01_JPRB, 0.31077E+01_JPRB, 0.35452E+01_JPRB, 0.35461E+01_JPRB, 0.34569E+01_JPRB ,&
&  0.35569E+01_JPRB, 0.31533E+01_JPRB, 0.23307E+01_JPRB, 0.34638E+01_JPRB, 0.38258E+01_JPRB, 0.34608E+01_JPRB, 0.33761E+01_JPRB ,&
&  0.31272E+01_JPRB, 0.27611E+01_JPRB, 0.24827E+01_JPRB, 0.23019E+01_JPRB, 0.19161E+01_JPRB, 0.17171E+01_JPRB, 0.18127E+01_JPRB ,&
&  0.15449E+01_JPRB, 0.12535E+01_JPRB, 0.11106E+01_JPRB, 0.11898E+01_JPRB, 0.15850E+01_JPRB, 0.26088E+01_JPRB, 0.23379E+01_JPRB ,&
&  0.16019E+01_JPRB, 0.12805E+01_JPRB, 0.18941E+01_JPRB, 0.28455E+01_JPRB, 0.14962E+01_JPRB, 0.88818E+00_JPRB, 0.55113E+00_JPRB ,&
&  0.50871E+00_JPRB, 0.53285E+00_JPRB, 0.56095E+00_JPRB, 0.58418E+00_JPRB, 0.61275E+00_JPRB, 0.63898E+00_JPRB, 0.66065E+00_JPRB ,&
&  0.68159E+00_JPRB, 0.70837E+00_JPRB, 0.73262E+00_JPRB, 0.76204E+00_JPRB, 0.78818E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.94929E+00_JPRB, 0.99766E+00_JPRB, 0.10339E+01_JPRB, 0.10644E+01_JPRB, 0.10907E+01_JPRB, 0.11172E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.12135E+01_JPRB, 0.12429E+01_JPRB, 0.12802E+01_JPRB, 0.13270E+01_JPRB, 0.13693E+01_JPRB, 0.13884E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.14416E+01_JPRB, 0.14655E+01_JPRB, 0.15252E+01_JPRB, 0.15467E+01_JPRB, 0.15887E+01_JPRB, 0.15941E+01_JPRB ,&
&  0.15843E+01_JPRB, 0.15811E+01_JPRB, 0.15855E+01_JPRB, 0.15999E+01_JPRB, 0.16082E+01_JPRB, 0.17342E+01_JPRB, 0.19256E+01_JPRB ,&
&  0.19067E+01_JPRB, 0.19088E+01_JPRB, 0.14371E+01_JPRB, 0.11875E+01_JPRB, 0.11895E+01_JPRB, 0.12258E+01_JPRB, 0.12395E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.13206E+01_JPRB, 0.13927E+01_JPRB, 0.14325E+01_JPRB, 0.15586E+01_JPRB, 0.17746E+01_JPRB, 0.18752E+01_JPRB ,&
&  0.20253E+01_JPRB, 0.21524E+01_JPRB, 0.22265E+01_JPRB, 0.24336E+01_JPRB, 0.24645E+01_JPRB, 0.30083E+01_JPRB, 0.32796E+01_JPRB ,&
&  0.33505E+01_JPRB, 0.31840E+01_JPRB, 0.24477E+01_JPRB, 0.24478E+01_JPRB, 0.26949E+01_JPRB, 0.26171E+01_JPRB, 0.25274E+01_JPRB ,&
&  0.29448E+01_JPRB, 0.28370E+01_JPRB, 0.24608E+01_JPRB, 0.30872E+01_JPRB, 0.32527E+01_JPRB, 0.31457E+01_JPRB, 0.27492E+01_JPRB ,&
&  0.23832E+01_JPRB, 0.18301E+01_JPRB, 0.17802E+01_JPRB, 0.17777E+01_JPRB, 0.16115E+01_JPRB, 0.14358E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.11152E+01_JPRB, 0.11351E+01_JPRB, 0.13172E+01_JPRB, 0.21060E+01_JPRB, 0.25245E+01_JPRB, 0.30071E+01_JPRB ,&
&  0.13384E+01_JPRB, 0.12958E+01_JPRB, 0.10252E+01_JPRB, 0.13293E+01_JPRB, 0.10512E+01_JPRB, 0.66380E+00_JPRB, 0.54806E+00_JPRB ,&
&  0.57701E+00_JPRB, 0.62524E+00_JPRB, 0.66590E+00_JPRB, 0.69750E+00_JPRB, 0.72707E+00_JPRB, 0.75905E+00_JPRB, 0.77743E+00_JPRB ,&
&  0.80052E+00_JPRB, 0.82673E+00_JPRB, 0.85222E+00_JPRB, 0.88890E+00_JPRB, 0.91430E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.10335E+01_JPRB, 0.10740E+01_JPRB, 0.11100E+01_JPRB, 0.11444E+01_JPRB, 0.11823E+01_JPRB, 0.12227E+01_JPRB, 0.12596E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.13257E+01_JPRB, 0.13550E+01_JPRB, 0.13835E+01_JPRB, 0.14212E+01_JPRB, 0.14581E+01_JPRB, 0.14825E+01_JPRB ,&
&  0.15098E+01_JPRB, 0.15544E+01_JPRB, 0.16012E+01_JPRB, 0.16458E+01_JPRB, 0.16702E+01_JPRB, 0.16938E+01_JPRB, 0.16942E+01_JPRB ,&
&  0.16811E+01_JPRB, 0.16763E+01_JPRB, 0.16934E+01_JPRB, 0.17226E+01_JPRB, 0.17684E+01_JPRB, 0.17699E+01_JPRB, 0.19137E+01_JPRB ,&
&  0.17494E+01_JPRB, 0.19959E+01_JPRB, 0.16061E+01_JPRB, 0.14206E+01_JPRB, 0.14458E+01_JPRB, 0.13809E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.14714E+01_JPRB, 0.14986E+01_JPRB, 0.15571E+01_JPRB, 0.16080E+01_JPRB, 0.17293E+01_JPRB, 0.19089E+01_JPRB, 0.20460E+01_JPRB ,&
&  0.22522E+01_JPRB, 0.24826E+01_JPRB, 0.26666E+01_JPRB, 0.28752E+01_JPRB, 0.33285E+01_JPRB, 0.37225E+01_JPRB, 0.39193E+01_JPRB ,&
&  0.38765E+01_JPRB, 0.36831E+01_JPRB, 0.27687E+01_JPRB, 0.28564E+01_JPRB, 0.30237E+01_JPRB, 0.27843E+01_JPRB, 0.25258E+01_JPRB ,&
&  0.26295E+01_JPRB, 0.21742E+01_JPRB, 0.24832E+01_JPRB, 0.25189E+01_JPRB, 0.23959E+01_JPRB, 0.21031E+01_JPRB, 0.17172E+01_JPRB ,&
&  0.13749E+01_JPRB, 0.11152E+01_JPRB, 0.11350E+01_JPRB, 0.13056E+01_JPRB, 0.12424E+01_JPRB, 0.10217E+01_JPRB, 0.97178E+00_JPRB ,&
&  0.92978E+00_JPRB, 0.89257E+00_JPRB, 0.94944E+00_JPRB, 0.12217E+01_JPRB, 0.17905E+01_JPRB, 0.31123E+01_JPRB, 0.28655E+01_JPRB ,&
&  0.12571E+01_JPRB, 0.11221E+01_JPRB, 0.66124E+00_JPRB, 0.69985E+00_JPRB, 0.71867E+00_JPRB, 0.59858E+00_JPRB, 0.60362E+00_JPRB ,&
&  0.70948E+00_JPRB, 0.81430E+00_JPRB, 0.83603E+00_JPRB, 0.78902E+00_JPRB, 0.78092E+00_JPRB, 0.80065E+00_JPRB, 0.83088E+00_JPRB ,&
&  0.86209E+00_JPRB, 0.89210E+00_JPRB, 0.92335E+00_JPRB, 0.95768E+00_JPRB, 0.99359E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.10743E+01_JPRB, 0.11160E+01_JPRB, 0.11531E+01_JPRB, 0.11867E+01_JPRB, 0.12274E+01_JPRB, 0.12715E+01_JPRB, 0.13133E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.13865E+01_JPRB, 0.14246E+01_JPRB, 0.14613E+01_JPRB, 0.14980E+01_JPRB, 0.15377E+01_JPRB, 0.15851E+01_JPRB ,&
&  0.16301E+01_JPRB, 0.16643E+01_JPRB, 0.17205E+01_JPRB, 0.17869E+01_JPRB, 0.18338E+01_JPRB, 0.18644E+01_JPRB, 0.18806E+01_JPRB ,&
&  0.18987E+01_JPRB, 0.19474E+01_JPRB, 0.19802E+01_JPRB, 0.20080E+01_JPRB, 0.20567E+01_JPRB, 0.22318E+01_JPRB, 0.20119E+01_JPRB ,&
&  0.22368E+01_JPRB, 0.19928E+01_JPRB, 0.17105E+01_JPRB, 0.16229E+01_JPRB, 0.16082E+01_JPRB, 0.16089E+01_JPRB, 0.16245E+01_JPRB ,&
&  0.16750E+01_JPRB, 0.16531E+01_JPRB, 0.16746E+01_JPRB, 0.17151E+01_JPRB, 0.18242E+01_JPRB, 0.19856E+01_JPRB, 0.21572E+01_JPRB ,&
&  0.24270E+01_JPRB, 0.27668E+01_JPRB, 0.31610E+01_JPRB, 0.35889E+01_JPRB, 0.40966E+01_JPRB, 0.45303E+01_JPRB, 0.48006E+01_JPRB ,&
&  0.48548E+01_JPRB, 0.47943E+01_JPRB, 0.40458E+01_JPRB, 0.42733E+01_JPRB, 0.41983E+01_JPRB, 0.34372E+01_JPRB, 0.27881E+01_JPRB ,&
&  0.22813E+01_JPRB, 0.17348E+01_JPRB, 0.18129E+01_JPRB, 0.18062E+01_JPRB, 0.16247E+01_JPRB, 0.13528E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.82304E+00_JPRB, 0.72588E+00_JPRB, 0.72763E+00_JPRB, 0.87732E+00_JPRB, 0.87867E+00_JPRB, 0.76320E+00_JPRB, 0.76346E+00_JPRB ,&
&  0.76889E+00_JPRB, 0.76506E+00_JPRB, 0.84609E+00_JPRB, 0.10900E+01_JPRB, 0.14681E+01_JPRB, 0.14824E+01_JPRB, 0.15305E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.88400E+00_JPRB, 0.82180E+00_JPRB, 0.69746E+00_JPRB, 0.72903E+00_JPRB, 0.72888E+00_JPRB, 0.82466E+00_JPRB ,&
&  0.98507E+00_JPRB, 0.12008E+01_JPRB, 0.11734E+01_JPRB, 0.97328E+00_JPRB, 0.82909E+00_JPRB, 0.80989E+00_JPRB, 0.84008E+00_JPRB ,&
&  0.87444E+00_JPRB, 0.90603E+00_JPRB, 0.94428E+00_JPRB, 0.98591E+00_JPRB, 0.10292E+01_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.10820E+01_JPRB, 0.11215E+01_JPRB, 0.11551E+01_JPRB, 0.11848E+01_JPRB, 0.12247E+01_JPRB, 0.12666E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.13512E+01_JPRB, 0.13921E+01_JPRB, 0.14365E+01_JPRB, 0.14794E+01_JPRB, 0.15275E+01_JPRB, 0.15780E+01_JPRB, 0.16331E+01_JPRB ,&
&  0.16855E+01_JPRB, 0.17368E+01_JPRB, 0.17824E+01_JPRB, 0.18693E+01_JPRB, 0.19685E+01_JPRB, 0.20411E+01_JPRB, 0.20926E+01_JPRB ,&
&  0.21180E+01_JPRB, 0.21763E+01_JPRB, 0.22455E+01_JPRB, 0.23057E+01_JPRB, 0.23293E+01_JPRB, 0.24749E+01_JPRB, 0.22423E+01_JPRB ,&
&  0.22393E+01_JPRB, 0.20363E+01_JPRB, 0.18272E+01_JPRB, 0.17949E+01_JPRB, 0.17728E+01_JPRB, 0.17769E+01_JPRB, 0.17770E+01_JPRB ,&
&  0.17725E+01_JPRB, 0.17854E+01_JPRB, 0.17698E+01_JPRB, 0.17907E+01_JPRB, 0.18647E+01_JPRB, 0.20222E+01_JPRB, 0.22233E+01_JPRB ,&
&  0.24970E+01_JPRB, 0.28658E+01_JPRB, 0.33124E+01_JPRB, 0.38020E+01_JPRB, 0.43601E+01_JPRB, 0.48659E+01_JPRB, 0.53392E+01_JPRB ,&
&  0.56941E+01_JPRB, 0.58972E+01_JPRB, 0.53954E+01_JPRB, 0.53486E+01_JPRB, 0.50194E+01_JPRB, 0.34076E+01_JPRB, 0.24217E+01_JPRB ,&
&  0.16866E+01_JPRB, 0.15786E+01_JPRB, 0.16055E+01_JPRB, 0.14511E+01_JPRB, 0.11344E+01_JPRB, 0.95622E+00_JPRB, 0.76422E+00_JPRB ,&
&  0.61292E+00_JPRB, 0.56220E+00_JPRB, 0.57699E+00_JPRB, 0.65559E+00_JPRB, 0.65083E+00_JPRB, 0.60614E+00_JPRB, 0.64301E+00_JPRB ,&
&  0.66488E+00_JPRB, 0.72441E+00_JPRB, 0.82704E+00_JPRB, 0.99401E+00_JPRB, 0.11619E+01_JPRB, 0.11659E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.10144E+01_JPRB, 0.98801E+00_JPRB, 0.87437E+00_JPRB, 0.65088E+00_JPRB, 0.81378E+00_JPRB, 0.92411E+00_JPRB, 0.10587E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.15915E+01_JPRB, 0.18131E+01_JPRB, 0.14288E+01_JPRB, 0.11323E+01_JPRB, 0.86210E+00_JPRB, 0.84299E+00_JPRB ,&
&  0.88018E+00_JPRB, 0.90040E+00_JPRB, 0.93869E+00_JPRB, 0.98519E+00_JPRB, 0.10351E+01_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.10348E+01_JPRB, 0.10779E+01_JPRB, 0.11063E+01_JPRB, 0.11187E+01_JPRB, 0.11523E+01_JPRB, 0.12042E+01_JPRB, 0.12421E+01_JPRB ,&
&  0.12831E+01_JPRB, 0.13172E+01_JPRB, 0.13637E+01_JPRB, 0.14077E+01_JPRB, 0.14543E+01_JPRB, 0.15026E+01_JPRB, 0.15580E+01_JPRB ,&
&  0.16091E+01_JPRB, 0.16928E+01_JPRB, 0.17505E+01_JPRB, 0.18481E+01_JPRB, 0.19538E+01_JPRB, 0.20843E+01_JPRB, 0.21654E+01_JPRB ,&
&  0.21947E+01_JPRB, 0.22669E+01_JPRB, 0.23519E+01_JPRB, 0.24493E+01_JPRB, 0.24569E+01_JPRB, 0.24869E+01_JPRB, 0.19449E+01_JPRB ,&
&  0.21011E+01_JPRB, 0.20284E+01_JPRB, 0.19365E+01_JPRB, 0.19049E+01_JPRB, 0.18632E+01_JPRB, 0.18204E+01_JPRB, 0.17752E+01_JPRB ,&
&  0.17750E+01_JPRB, 0.17531E+01_JPRB, 0.16840E+01_JPRB, 0.17827E+01_JPRB, 0.18596E+01_JPRB, 0.19844E+01_JPRB, 0.21854E+01_JPRB ,&
&  0.24105E+01_JPRB, 0.27355E+01_JPRB, 0.31585E+01_JPRB, 0.36646E+01_JPRB, 0.42214E+01_JPRB, 0.47806E+01_JPRB, 0.53329E+01_JPRB ,&
&  0.58480E+01_JPRB, 0.62563E+01_JPRB, 0.63205E+01_JPRB, 0.51660E+01_JPRB, 0.42780E+01_JPRB, 0.29486E+01_JPRB, 0.21746E+01_JPRB ,&
&  0.17552E+01_JPRB, 0.15685E+01_JPRB, 0.14759E+01_JPRB, 0.12914E+01_JPRB, 0.96529E+00_JPRB, 0.79278E+00_JPRB, 0.63571E+00_JPRB ,&
&  0.57482E+00_JPRB, 0.53661E+00_JPRB, 0.54064E+00_JPRB, 0.54721E+00_JPRB, 0.55339E+00_JPRB, 0.54782E+00_JPRB, 0.58015E+00_JPRB ,&
&  0.61382E+00_JPRB, 0.68486E+00_JPRB, 0.81697E+00_JPRB, 0.10048E+01_JPRB, 0.11812E+01_JPRB, 0.12661E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.88354E+00_JPRB, 0.83444E+00_JPRB, 0.86015E+00_JPRB, 0.89789E+00_JPRB, 0.10776E+01_JPRB, 0.12879E+01_JPRB ,&
&  0.15470E+01_JPRB, 0.15692E+01_JPRB, 0.18480E+01_JPRB, 0.24016E+01_JPRB, 0.21642E+01_JPRB, 0.18764E+01_JPRB, 0.77168E+00_JPRB ,&
&  0.82628E+00_JPRB, 0.84923E+00_JPRB, 0.89663E+00_JPRB, 0.94435E+00_JPRB, 0.10032E+01_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.83036E+00_JPRB, 0.87052E+00_JPRB, 0.90637E+00_JPRB, 0.92286E+00_JPRB, 0.97203E+00_JPRB, 0.10250E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.11526E+01_JPRB, 0.11959E+01_JPRB, 0.12511E+01_JPRB, 0.12981E+01_JPRB, 0.13443E+01_JPRB, 0.13927E+01_JPRB ,&
&  0.14400E+01_JPRB, 0.15108E+01_JPRB, 0.15837E+01_JPRB, 0.16861E+01_JPRB, 0.17931E+01_JPRB, 0.19254E+01_JPRB, 0.20481E+01_JPRB ,&
&  0.21430E+01_JPRB, 0.22400E+01_JPRB, 0.23536E+01_JPRB, 0.24316E+01_JPRB, 0.24108E+01_JPRB, 0.23645E+01_JPRB, 0.17431E+01_JPRB ,&
&  0.19813E+01_JPRB, 0.20156E+01_JPRB, 0.20079E+01_JPRB, 0.19670E+01_JPRB, 0.18987E+01_JPRB, 0.18160E+01_JPRB, 0.17714E+01_JPRB ,&
&  0.17376E+01_JPRB, 0.16775E+01_JPRB, 0.16119E+01_JPRB, 0.15853E+01_JPRB, 0.18054E+01_JPRB, 0.19204E+01_JPRB, 0.20951E+01_JPRB ,&
&  0.23265E+01_JPRB, 0.26118E+01_JPRB, 0.29609E+01_JPRB, 0.34610E+01_JPRB, 0.40131E+01_JPRB, 0.45630E+01_JPRB, 0.51312E+01_JPRB ,&
&  0.55782E+01_JPRB, 0.58091E+01_JPRB, 0.58633E+01_JPRB, 0.46495E+01_JPRB, 0.37945E+01_JPRB, 0.29161E+01_JPRB, 0.20207E+01_JPRB ,&
&  0.16494E+01_JPRB, 0.15069E+01_JPRB, 0.14272E+01_JPRB, 0.12218E+01_JPRB, 0.93892E+00_JPRB, 0.74208E+00_JPRB, 0.60573E+00_JPRB ,&
&  0.55369E+00_JPRB, 0.51468E+00_JPRB, 0.51623E+00_JPRB, 0.52530E+00_JPRB, 0.54286E+00_JPRB, 0.55625E+00_JPRB, 0.58337E+00_JPRB ,&
&  0.62237E+00_JPRB, 0.68219E+00_JPRB, 0.77951E+00_JPRB, 0.93856E+00_JPRB, 0.11641E+01_JPRB, 0.14123E+01_JPRB, 0.13594E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.88872E+00_JPRB, 0.81640E+00_JPRB, 0.78188E+00_JPRB, 0.79327E+00_JPRB, 0.84872E+00_JPRB, 0.92015E+00_JPRB ,&
&  0.10519E+01_JPRB, 0.11252E+01_JPRB, 0.10082E+01_JPRB, 0.84206E+00_JPRB, 0.10977E+01_JPRB, 0.10925E+01_JPRB, 0.62698E+00_JPRB ,&
&  0.68670E+00_JPRB, 0.67596E+00_JPRB, 0.71073E+00_JPRB, 0.75260E+00_JPRB, 0.80344E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.63278E+00_JPRB, 0.69443E+00_JPRB, 0.72526E+00_JPRB, 0.76422E+00_JPRB, 0.79812E+00_JPRB, 0.84663E+00_JPRB, 0.89393E+00_JPRB ,&
&  0.94186E+00_JPRB, 0.99618E+00_JPRB, 0.10389E+01_JPRB, 0.10925E+01_JPRB, 0.11394E+01_JPRB, 0.11819E+01_JPRB, 0.12240E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.13254E+01_JPRB, 0.13875E+01_JPRB, 0.14599E+01_JPRB, 0.15374E+01_JPRB, 0.16392E+01_JPRB, 0.17807E+01_JPRB ,&
&  0.19525E+01_JPRB, 0.21202E+01_JPRB, 0.22706E+01_JPRB, 0.23964E+01_JPRB, 0.24660E+01_JPRB, 0.25661E+01_JPRB, 0.25849E+01_JPRB ,&
&  0.17539E+01_JPRB, 0.21328E+01_JPRB, 0.20559E+01_JPRB, 0.19576E+01_JPRB, 0.18815E+01_JPRB, 0.18087E+01_JPRB, 0.17828E+01_JPRB ,&
&  0.17457E+01_JPRB, 0.16110E+01_JPRB, 0.15776E+01_JPRB, 0.15464E+01_JPRB, 0.16626E+01_JPRB, 0.17869E+01_JPRB, 0.19601E+01_JPRB ,&
&  0.21953E+01_JPRB, 0.24950E+01_JPRB, 0.28201E+01_JPRB, 0.31703E+01_JPRB, 0.36421E+01_JPRB, 0.42224E+01_JPRB, 0.47555E+01_JPRB ,&
&  0.53160E+01_JPRB, 0.58845E+01_JPRB, 0.62304E+01_JPRB, 0.52253E+01_JPRB, 0.42417E+01_JPRB, 0.37358E+01_JPRB, 0.26595E+01_JPRB ,&
&  0.17074E+01_JPRB, 0.13507E+01_JPRB, 0.13199E+01_JPRB, 0.12242E+01_JPRB, 0.94731E+00_JPRB, 0.76992E+00_JPRB, 0.65051E+00_JPRB ,&
&  0.59463E+00_JPRB, 0.54649E+00_JPRB, 0.53423E+00_JPRB, 0.54306E+00_JPRB, 0.55994E+00_JPRB, 0.58082E+00_JPRB, 0.62300E+00_JPRB ,&
&  0.65529E+00_JPRB, 0.68581E+00_JPRB, 0.72376E+00_JPRB, 0.76113E+00_JPRB, 0.83274E+00_JPRB, 0.94990E+00_JPRB, 0.96361E+00_JPRB ,&
&  0.91447E+00_JPRB, 0.86839E+00_JPRB, 0.82140E+00_JPRB, 0.80381E+00_JPRB, 0.77169E+00_JPRB, 0.75523E+00_JPRB, 0.76432E+00_JPRB ,&
&  0.78265E+00_JPRB, 0.79236E+00_JPRB, 0.74578E+00_JPRB, 0.70893E+00_JPRB, 0.69345E+00_JPRB, 0.66102E+00_JPRB, 0.60775E+00_JPRB ,&
&  0.58846E+00_JPRB, 0.57171E+00_JPRB, 0.55306E+00_JPRB, 0.56377E+00_JPRB, 0.59407E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.53944E+00_JPRB, 0.56230E+00_JPRB, 0.59094E+00_JPRB, 0.62136E+00_JPRB, 0.65044E+00_JPRB, 0.67969E+00_JPRB, 0.72361E+00_JPRB ,&
&  0.77478E+00_JPRB, 0.81543E+00_JPRB, 0.84021E+00_JPRB, 0.87907E+00_JPRB, 0.92530E+00_JPRB, 0.98116E+00_JPRB, 0.10342E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.11560E+01_JPRB, 0.12128E+01_JPRB, 0.12716E+01_JPRB, 0.13327E+01_JPRB, 0.13993E+01_JPRB, 0.14991E+01_JPRB ,&
&  0.16286E+01_JPRB, 0.17986E+01_JPRB, 0.20095E+01_JPRB, 0.22154E+01_JPRB, 0.24041E+01_JPRB, 0.25912E+01_JPRB, 0.26545E+01_JPRB ,&
&  0.15510E+01_JPRB, 0.20283E+01_JPRB, 0.23170E+01_JPRB, 0.20667E+01_JPRB, 0.18774E+01_JPRB, 0.17650E+01_JPRB, 0.17088E+01_JPRB ,&
&  0.16137E+01_JPRB, 0.14859E+01_JPRB, 0.14491E+01_JPRB, 0.15548E+01_JPRB, 0.15254E+01_JPRB, 0.16127E+01_JPRB, 0.17300E+01_JPRB ,&
&  0.19144E+01_JPRB, 0.21506E+01_JPRB, 0.23719E+01_JPRB, 0.26333E+01_JPRB, 0.30113E+01_JPRB, 0.34860E+01_JPRB, 0.40660E+01_JPRB ,&
&  0.47997E+01_JPRB, 0.56561E+01_JPRB, 0.64530E+01_JPRB, 0.51451E+01_JPRB, 0.46395E+01_JPRB, 0.43525E+01_JPRB, 0.36789E+01_JPRB ,&
&  0.20887E+01_JPRB, 0.13628E+01_JPRB, 0.13251E+01_JPRB, 0.12481E+01_JPRB, 0.10287E+01_JPRB, 0.87944E+00_JPRB, 0.77890E+00_JPRB ,&
&  0.70575E+00_JPRB, 0.64969E+00_JPRB, 0.62092E+00_JPRB, 0.61407E+00_JPRB, 0.62015E+00_JPRB, 0.64832E+00_JPRB, 0.68299E+00_JPRB ,&
&  0.71280E+00_JPRB, 0.71898E+00_JPRB, 0.75452E+00_JPRB, 0.77086E+00_JPRB, 0.81774E+00_JPRB, 0.85707E+00_JPRB, 0.88355E+00_JPRB ,&
&  0.89219E+00_JPRB, 0.88055E+00_JPRB, 0.86970E+00_JPRB, 0.85990E+00_JPRB, 0.85248E+00_JPRB, 0.84221E+00_JPRB, 0.82935E+00_JPRB ,&
&  0.81167E+00_JPRB, 0.81379E+00_JPRB, 0.79162E+00_JPRB, 0.77253E+00_JPRB, 0.76781E+00_JPRB, 0.72836E+00_JPRB, 0.70707E+00_JPRB ,&
&  0.68577E+00_JPRB, 0.63308E+00_JPRB, 0.59718E+00_JPRB, 0.56912E+00_JPRB, 0.54621E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.57709E+00_JPRB, 0.54190E+00_JPRB, 0.54532E+00_JPRB, 0.55272E+00_JPRB, 0.57498E+00_JPRB, 0.59834E+00_JPRB, 0.59506E+00_JPRB ,&
&  0.60775E+00_JPRB, 0.62723E+00_JPRB, 0.65452E+00_JPRB, 0.68257E+00_JPRB, 0.71600E+00_JPRB, 0.75381E+00_JPRB, 0.79519E+00_JPRB ,&
&  0.84825E+00_JPRB, 0.91214E+00_JPRB, 0.98103E+00_JPRB, 0.10448E+01_JPRB, 0.11096E+01_JPRB, 0.11627E+01_JPRB, 0.12162E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.14002E+01_JPRB, 0.15696E+01_JPRB, 0.17831E+01_JPRB, 0.20600E+01_JPRB, 0.23910E+01_JPRB, 0.26203E+01_JPRB ,&
&  0.18399E+01_JPRB, 0.92736E+00_JPRB, 0.21426E+01_JPRB, 0.23689E+01_JPRB, 0.20315E+01_JPRB, 0.18787E+01_JPRB, 0.16980E+01_JPRB ,&
&  0.15779E+01_JPRB, 0.15509E+01_JPRB, 0.14456E+01_JPRB, 0.14538E+01_JPRB, 0.14145E+01_JPRB, 0.14496E+01_JPRB, 0.15008E+01_JPRB ,&
&  0.15703E+01_JPRB, 0.17141E+01_JPRB, 0.19378E+01_JPRB, 0.20765E+01_JPRB, 0.22825E+01_JPRB, 0.26388E+01_JPRB, 0.31687E+01_JPRB ,&
&  0.39123E+01_JPRB, 0.48261E+01_JPRB, 0.57465E+01_JPRB, 0.44281E+01_JPRB, 0.33963E+01_JPRB, 0.26635E+01_JPRB, 0.21528E+01_JPRB ,&
&  0.18346E+01_JPRB, 0.14095E+01_JPRB, 0.13176E+01_JPRB, 0.12582E+01_JPRB, 0.11165E+01_JPRB, 0.94676E+00_JPRB, 0.91216E+00_JPRB ,&
&  0.83905E+00_JPRB, 0.78486E+00_JPRB, 0.75261E+00_JPRB, 0.73065E+00_JPRB, 0.72608E+00_JPRB, 0.74403E+00_JPRB, 0.76324E+00_JPRB ,&
&  0.78719E+00_JPRB, 0.80361E+00_JPRB, 0.83143E+00_JPRB, 0.86422E+00_JPRB, 0.87232E+00_JPRB, 0.87346E+00_JPRB, 0.88760E+00_JPRB ,&
&  0.88619E+00_JPRB, 0.89517E+00_JPRB, 0.90629E+00_JPRB, 0.91023E+00_JPRB, 0.91164E+00_JPRB, 0.89859E+00_JPRB, 0.90366E+00_JPRB ,&
&  0.89854E+00_JPRB, 0.89271E+00_JPRB, 0.87240E+00_JPRB, 0.87173E+00_JPRB, 0.87138E+00_JPRB, 0.86108E+00_JPRB, 0.87021E+00_JPRB ,&
&  0.85606E+00_JPRB, 0.81858E+00_JPRB, 0.74836E+00_JPRB, 0.67225E+00_JPRB, 0.62858E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.71114E+00_JPRB, 0.65380E+00_JPRB, 0.61061E+00_JPRB, 0.58761E+00_JPRB, 0.57321E+00_JPRB, 0.56825E+00_JPRB, 0.56711E+00_JPRB ,&
&  0.57228E+00_JPRB, 0.57825E+00_JPRB, 0.56883E+00_JPRB, 0.57752E+00_JPRB, 0.59676E+00_JPRB, 0.62152E+00_JPRB, 0.65339E+00_JPRB ,&
&  0.67838E+00_JPRB, 0.70538E+00_JPRB, 0.73514E+00_JPRB, 0.77669E+00_JPRB, 0.82877E+00_JPRB, 0.88718E+00_JPRB, 0.94475E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10709E+01_JPRB, 0.11686E+01_JPRB, 0.13095E+01_JPRB, 0.15088E+01_JPRB, 0.18079E+01_JPRB, 0.21511E+01_JPRB ,&
&  0.26339E+01_JPRB, 0.18941E+01_JPRB, 0.12760E+01_JPRB, 0.21534E+01_JPRB, 0.22619E+01_JPRB, 0.21310E+01_JPRB, 0.17827E+01_JPRB ,&
&  0.17636E+01_JPRB, 0.17628E+01_JPRB, 0.15824E+01_JPRB, 0.14699E+01_JPRB, 0.13311E+01_JPRB, 0.13215E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13190E+01_JPRB, 0.13411E+01_JPRB, 0.14857E+01_JPRB, 0.15944E+01_JPRB, 0.16848E+01_JPRB, 0.19116E+01_JPRB, 0.23798E+01_JPRB ,&
&  0.30240E+01_JPRB, 0.38672E+01_JPRB, 0.48201E+01_JPRB, 0.40181E+01_JPRB, 0.29620E+01_JPRB, 0.21869E+01_JPRB, 0.18877E+01_JPRB ,&
&  0.17100E+01_JPRB, 0.15792E+01_JPRB, 0.14189E+01_JPRB, 0.12814E+01_JPRB, 0.11440E+01_JPRB, 0.10148E+01_JPRB, 0.99319E+00_JPRB ,&
&  0.95275E+00_JPRB, 0.89018E+00_JPRB, 0.84597E+00_JPRB, 0.80842E+00_JPRB, 0.79938E+00_JPRB, 0.80969E+00_JPRB, 0.82220E+00_JPRB ,&
&  0.83729E+00_JPRB, 0.85652E+00_JPRB, 0.88762E+00_JPRB, 0.92466E+00_JPRB, 0.93384E+00_JPRB, 0.92319E+00_JPRB, 0.91656E+00_JPRB ,&
&  0.90795E+00_JPRB, 0.90433E+00_JPRB, 0.91906E+00_JPRB, 0.92593E+00_JPRB, 0.92141E+00_JPRB, 0.90528E+00_JPRB, 0.91937E+00_JPRB ,&
&  0.91948E+00_JPRB, 0.92820E+00_JPRB, 0.91905E+00_JPRB, 0.92721E+00_JPRB, 0.92834E+00_JPRB, 0.94908E+00_JPRB, 0.97100E+00_JPRB ,&
&  0.98692E+00_JPRB, 0.97371E+00_JPRB, 0.92699E+00_JPRB, 0.80742E+00_JPRB, 0.75618E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.77444E+00_JPRB, 0.73722E+00_JPRB, 0.70865E+00_JPRB, 0.68309E+00_JPRB, 0.65723E+00_JPRB, 0.63255E+00_JPRB, 0.61290E+00_JPRB ,&
&  0.59044E+00_JPRB, 0.58752E+00_JPRB, 0.58461E+00_JPRB, 0.57891E+00_JPRB, 0.57162E+00_JPRB, 0.57523E+00_JPRB, 0.58708E+00_JPRB ,&
&  0.59600E+00_JPRB, 0.60393E+00_JPRB, 0.61595E+00_JPRB, 0.62306E+00_JPRB, 0.63346E+00_JPRB, 0.65502E+00_JPRB, 0.68945E+00_JPRB ,&
&  0.73812E+00_JPRB, 0.79310E+00_JPRB, 0.86491E+00_JPRB, 0.95407E+00_JPRB, 0.10466E+01_JPRB, 0.11931E+01_JPRB, 0.13758E+01_JPRB ,&
&  0.18162E+01_JPRB, 0.18719E+01_JPRB, 0.10661E+01_JPRB, 0.18867E+01_JPRB, 0.23936E+01_JPRB, 0.22672E+01_JPRB, 0.21037E+01_JPRB ,&
&  0.21004E+01_JPRB, 0.19698E+01_JPRB, 0.17755E+01_JPRB, 0.15194E+01_JPRB, 0.13009E+01_JPRB, 0.12431E+01_JPRB, 0.12277E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.11433E+01_JPRB, 0.11725E+01_JPRB, 0.12086E+01_JPRB, 0.12521E+01_JPRB, 0.13504E+01_JPRB, 0.16700E+01_JPRB ,&
&  0.21937E+01_JPRB, 0.29894E+01_JPRB, 0.38908E+01_JPRB, 0.36715E+01_JPRB, 0.27316E+01_JPRB, 0.22878E+01_JPRB, 0.20686E+01_JPRB ,&
&  0.19849E+01_JPRB, 0.18185E+01_JPRB, 0.14897E+01_JPRB, 0.12938E+01_JPRB, 0.11509E+01_JPRB, 0.10407E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10105E+01_JPRB, 0.95535E+00_JPRB, 0.92227E+00_JPRB, 0.87710E+00_JPRB, 0.85271E+00_JPRB, 0.84403E+00_JPRB, 0.84954E+00_JPRB ,&
&  0.86051E+00_JPRB, 0.87230E+00_JPRB, 0.89105E+00_JPRB, 0.91658E+00_JPRB, 0.93447E+00_JPRB, 0.94101E+00_JPRB, 0.93521E+00_JPRB ,&
&  0.92899E+00_JPRB, 0.92764E+00_JPRB, 0.93104E+00_JPRB, 0.93113E+00_JPRB, 0.92594E+00_JPRB, 0.91867E+00_JPRB, 0.90935E+00_JPRB ,&
&  0.90507E+00_JPRB, 0.90519E+00_JPRB, 0.89627E+00_JPRB, 0.88677E+00_JPRB, 0.92003E+00_JPRB, 0.93035E+00_JPRB, 0.93930E+00_JPRB ,&
&  0.97787E+00_JPRB, 0.10059E+01_JPRB, 0.98968E+00_JPRB, 0.86963E+00_JPRB, 0.81568E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.80369E+00_JPRB, 0.77293E+00_JPRB, 0.74905E+00_JPRB, 0.73078E+00_JPRB, 0.70875E+00_JPRB, 0.68717E+00_JPRB, 0.66861E+00_JPRB ,&
&  0.64448E+00_JPRB, 0.62994E+00_JPRB, 0.62056E+00_JPRB, 0.61050E+00_JPRB, 0.60033E+00_JPRB, 0.59096E+00_JPRB, 0.58327E+00_JPRB ,&
&  0.57561E+00_JPRB, 0.56924E+00_JPRB, 0.56591E+00_JPRB, 0.55933E+00_JPRB, 0.55374E+00_JPRB, 0.55375E+00_JPRB, 0.56174E+00_JPRB ,&
&  0.58779E+00_JPRB, 0.62776E+00_JPRB, 0.67778E+00_JPRB, 0.73318E+00_JPRB, 0.78194E+00_JPRB, 0.84785E+00_JPRB, 0.92888E+00_JPRB ,&
&  0.12682E+01_JPRB, 0.14760E+01_JPRB, 0.17236E+01_JPRB, 0.27909E+01_JPRB, 0.28554E+01_JPRB, 0.25550E+01_JPRB, 0.25346E+01_JPRB ,&
&  0.25816E+01_JPRB, 0.24015E+01_JPRB, 0.21899E+01_JPRB, 0.17277E+01_JPRB, 0.13974E+01_JPRB, 0.12294E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.10675E+01_JPRB, 0.10110E+01_JPRB, 0.10060E+01_JPRB, 0.99712E+00_JPRB, 0.10137E+01_JPRB, 0.11346E+01_JPRB, 0.12979E+01_JPRB ,&
&  0.16332E+01_JPRB, 0.22195E+01_JPRB, 0.30350E+01_JPRB, 0.35661E+01_JPRB, 0.27552E+01_JPRB, 0.25313E+01_JPRB, 0.25532E+01_JPRB ,&
&  0.30689E+01_JPRB, 0.23376E+01_JPRB, 0.16939E+01_JPRB, 0.13447E+01_JPRB, 0.11532E+01_JPRB, 0.10774E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.99826E+00_JPRB, 0.96759E+00_JPRB, 0.94422E+00_JPRB, 0.91762E+00_JPRB, 0.89863E+00_JPRB, 0.88033E+00_JPRB, 0.87079E+00_JPRB ,&
&  0.86869E+00_JPRB, 0.87173E+00_JPRB, 0.87667E+00_JPRB, 0.88814E+00_JPRB, 0.90912E+00_JPRB, 0.92574E+00_JPRB, 0.93122E+00_JPRB ,&
&  0.93338E+00_JPRB, 0.93725E+00_JPRB, 0.91248E+00_JPRB, 0.90706E+00_JPRB, 0.91285E+00_JPRB, 0.90603E+00_JPRB, 0.87592E+00_JPRB ,&
&  0.86390E+00_JPRB, 0.87769E+00_JPRB, 0.87865E+00_JPRB, 0.86958E+00_JPRB, 0.88045E+00_JPRB, 0.89054E+00_JPRB, 0.87096E+00_JPRB ,&
&  0.87625E+00_JPRB, 0.88675E+00_JPRB, 0.88277E+00_JPRB, 0.86080E+00_JPRB, 0.83551E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.81853E+00_JPRB, 0.79336E+00_JPRB, 0.76840E+00_JPRB, 0.74494E+00_JPRB, 0.71934E+00_JPRB, 0.69868E+00_JPRB, 0.68652E+00_JPRB ,&
&  0.66864E+00_JPRB, 0.65341E+00_JPRB, 0.64259E+00_JPRB, 0.63716E+00_JPRB, 0.63274E+00_JPRB, 0.62121E+00_JPRB, 0.59975E+00_JPRB ,&
&  0.57798E+00_JPRB, 0.56284E+00_JPRB, 0.55372E+00_JPRB, 0.54454E+00_JPRB, 0.53535E+00_JPRB, 0.53057E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.54312E+00_JPRB, 0.56005E+00_JPRB, 0.59343E+00_JPRB, 0.63201E+00_JPRB, 0.66637E+00_JPRB, 0.69988E+00_JPRB, 0.72570E+00_JPRB ,&
&  0.91716E+00_JPRB, 0.12625E+01_JPRB, 0.12759E+01_JPRB, 0.23846E+01_JPRB, 0.25517E+01_JPRB, 0.25025E+01_JPRB, 0.25218E+01_JPRB ,&
&  0.25408E+01_JPRB, 0.27817E+01_JPRB, 0.23768E+01_JPRB, 0.19454E+01_JPRB, 0.15749E+01_JPRB, 0.13318E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.97631E+00_JPRB, 0.93411E+00_JPRB, 0.91787E+00_JPRB, 0.91614E+00_JPRB, 0.97595E+00_JPRB, 0.11346E+01_JPRB ,&
&  0.13680E+01_JPRB, 0.17751E+01_JPRB, 0.23673E+01_JPRB, 0.27864E+01_JPRB, 0.26669E+01_JPRB, 0.26940E+01_JPRB, 0.30023E+01_JPRB ,&
&  0.37900E+01_JPRB, 0.32036E+01_JPRB, 0.20645E+01_JPRB, 0.14643E+01_JPRB, 0.12018E+01_JPRB, 0.10940E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.98702E+00_JPRB, 0.95696E+00_JPRB, 0.93410E+00_JPRB, 0.91900E+00_JPRB, 0.90831E+00_JPRB, 0.90436E+00_JPRB, 0.89679E+00_JPRB ,&
&  0.88658E+00_JPRB, 0.87963E+00_JPRB, 0.87512E+00_JPRB, 0.87287E+00_JPRB, 0.87824E+00_JPRB, 0.89241E+00_JPRB, 0.90765E+00_JPRB ,&
&  0.91954E+00_JPRB, 0.92831E+00_JPRB, 0.90640E+00_JPRB, 0.88693E+00_JPRB, 0.88641E+00_JPRB, 0.87784E+00_JPRB, 0.85317E+00_JPRB ,&
&  0.85692E+00_JPRB, 0.87019E+00_JPRB, 0.86735E+00_JPRB, 0.84750E+00_JPRB, 0.85311E+00_JPRB, 0.87320E+00_JPRB, 0.84649E+00_JPRB ,&
&  0.84217E+00_JPRB, 0.84820E+00_JPRB, 0.85545E+00_JPRB, 0.84637E+00_JPRB, 0.83411E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.80144E+00_JPRB, 0.78604E+00_JPRB, 0.76538E+00_JPRB, 0.74258E+00_JPRB, 0.71952E+00_JPRB, 0.69968E+00_JPRB, 0.68841E+00_JPRB ,&
&  0.67494E+00_JPRB, 0.66127E+00_JPRB, 0.64930E+00_JPRB, 0.64219E+00_JPRB, 0.63893E+00_JPRB, 0.62759E+00_JPRB, 0.60698E+00_JPRB ,&
&  0.58518E+00_JPRB, 0.56589E+00_JPRB, 0.54972E+00_JPRB, 0.53954E+00_JPRB, 0.52891E+00_JPRB, 0.52411E+00_JPRB, 0.52401E+00_JPRB ,&
&  0.52632E+00_JPRB, 0.53526E+00_JPRB, 0.55519E+00_JPRB, 0.58594E+00_JPRB, 0.61602E+00_JPRB, 0.64062E+00_JPRB, 0.65710E+00_JPRB ,&
&  0.76756E+00_JPRB, 0.14691E+01_JPRB, 0.11908E+01_JPRB, 0.18471E+01_JPRB, 0.20078E+01_JPRB, 0.21259E+01_JPRB, 0.22889E+01_JPRB ,&
&  0.23567E+01_JPRB, 0.24573E+01_JPRB, 0.23054E+01_JPRB, 0.20236E+01_JPRB, 0.17015E+01_JPRB, 0.14732E+01_JPRB, 0.12938E+01_JPRB ,&
&  0.11641E+01_JPRB, 0.10732E+01_JPRB, 0.10046E+01_JPRB, 0.95151E+00_JPRB, 0.91743E+00_JPRB, 0.92620E+00_JPRB, 0.10490E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.16479E+01_JPRB, 0.20278E+01_JPRB, 0.24499E+01_JPRB, 0.23617E+01_JPRB, 0.26688E+01_JPRB, 0.35838E+01_JPRB ,&
&  0.41528E+01_JPRB, 0.40846E+01_JPRB, 0.26725E+01_JPRB, 0.17311E+01_JPRB, 0.13223E+01_JPRB, 0.11273E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.95566E+00_JPRB, 0.92224E+00_JPRB, 0.90378E+00_JPRB, 0.89815E+00_JPRB, 0.90136E+00_JPRB, 0.91181E+00_JPRB, 0.91234E+00_JPRB ,&
&  0.91077E+00_JPRB, 0.90682E+00_JPRB, 0.90231E+00_JPRB, 0.89167E+00_JPRB, 0.87682E+00_JPRB, 0.87249E+00_JPRB, 0.87763E+00_JPRB ,&
&  0.88989E+00_JPRB, 0.90338E+00_JPRB, 0.89202E+00_JPRB, 0.87792E+00_JPRB, 0.87009E+00_JPRB, 0.86081E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.85499E+00_JPRB, 0.85982E+00_JPRB, 0.85599E+00_JPRB, 0.85404E+00_JPRB, 0.84554E+00_JPRB, 0.86761E+00_JPRB, 0.83485E+00_JPRB ,&
&  0.82694E+00_JPRB, 0.83099E+00_JPRB, 0.83348E+00_JPRB, 0.82071E+00_JPRB, 0.81069E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.76291E+00_JPRB, 0.75023E+00_JPRB, 0.73180E+00_JPRB, 0.71286E+00_JPRB, 0.69990E+00_JPRB, 0.68546E+00_JPRB, 0.67406E+00_JPRB ,&
&  0.66392E+00_JPRB, 0.65465E+00_JPRB, 0.64721E+00_JPRB, 0.63900E+00_JPRB, 0.62834E+00_JPRB, 0.61357E+00_JPRB, 0.59872E+00_JPRB ,&
&  0.58349E+00_JPRB, 0.56896E+00_JPRB, 0.55392E+00_JPRB, 0.53836E+00_JPRB, 0.52568E+00_JPRB, 0.51733E+00_JPRB, 0.51175E+00_JPRB ,&
&  0.51096E+00_JPRB, 0.51450E+00_JPRB, 0.52633E+00_JPRB, 0.54981E+00_JPRB, 0.57975E+00_JPRB, 0.60742E+00_JPRB, 0.63455E+00_JPRB ,&
&  0.75077E+00_JPRB, 0.69594E+00_JPRB, 0.13470E+01_JPRB, 0.15902E+01_JPRB, 0.15615E+01_JPRB, 0.16012E+01_JPRB, 0.19078E+01_JPRB ,&
&  0.20985E+01_JPRB, 0.21728E+01_JPRB, 0.21718E+01_JPRB, 0.20423E+01_JPRB, 0.17966E+01_JPRB, 0.16135E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.12815E+01_JPRB, 0.11809E+01_JPRB, 0.10815E+01_JPRB, 0.10081E+01_JPRB, 0.99241E+00_JPRB, 0.10574E+01_JPRB ,&
&  0.13425E+01_JPRB, 0.16037E+01_JPRB, 0.18150E+01_JPRB, 0.20778E+01_JPRB, 0.21603E+01_JPRB, 0.25598E+01_JPRB, 0.30488E+01_JPRB ,&
&  0.40552E+01_JPRB, 0.41600E+01_JPRB, 0.30321E+01_JPRB, 0.21239E+01_JPRB, 0.15842E+01_JPRB, 0.12974E+01_JPRB, 0.11034E+01_JPRB ,&
&  0.99106E+00_JPRB, 0.92456E+00_JPRB, 0.88289E+00_JPRB, 0.87295E+00_JPRB, 0.88819E+00_JPRB, 0.91545E+00_JPRB, 0.93902E+00_JPRB ,&
&  0.96509E+00_JPRB, 0.98444E+00_JPRB, 0.98770E+00_JPRB, 0.97722E+00_JPRB, 0.93887E+00_JPRB, 0.89335E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.85824E+00_JPRB, 0.86572E+00_JPRB, 0.86428E+00_JPRB, 0.85567E+00_JPRB, 0.85160E+00_JPRB, 0.84664E+00_JPRB, 0.84188E+00_JPRB ,&
&  0.84592E+00_JPRB, 0.85260E+00_JPRB, 0.85178E+00_JPRB, 0.85795E+00_JPRB, 0.83545E+00_JPRB, 0.86140E+00_JPRB, 0.83541E+00_JPRB ,&
&  0.82035E+00_JPRB, 0.81208E+00_JPRB, 0.80382E+00_JPRB, 0.78629E+00_JPRB, 0.77016E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.71972E+00_JPRB, 0.70727E+00_JPRB, 0.68959E+00_JPRB, 0.67269E+00_JPRB, 0.66305E+00_JPRB, 0.65623E+00_JPRB, 0.65132E+00_JPRB ,&
&  0.64806E+00_JPRB, 0.63915E+00_JPRB, 0.62989E+00_JPRB, 0.61687E+00_JPRB, 0.59993E+00_JPRB, 0.58531E+00_JPRB, 0.57597E+00_JPRB ,&
&  0.56509E+00_JPRB, 0.55943E+00_JPRB, 0.55198E+00_JPRB, 0.53800E+00_JPRB, 0.52254E+00_JPRB, 0.50966E+00_JPRB, 0.50138E+00_JPRB ,&
&  0.49727E+00_JPRB, 0.49962E+00_JPRB, 0.50714E+00_JPRB, 0.52448E+00_JPRB, 0.54316E+00_JPRB, 0.56512E+00_JPRB, 0.60359E+00_JPRB ,&
&  0.75137E+00_JPRB, 0.12735E+01_JPRB, 0.16267E+01_JPRB, 0.14498E+01_JPRB, 0.13317E+01_JPRB, 0.12446E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.16052E+01_JPRB, 0.17765E+01_JPRB, 0.18760E+01_JPRB, 0.19056E+01_JPRB, 0.17809E+01_JPRB, 0.16889E+01_JPRB, 0.15970E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.13957E+01_JPRB, 0.12528E+01_JPRB, 0.11137E+01_JPRB, 0.10376E+01_JPRB, 0.10351E+01_JPRB, 0.11181E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.15268E+01_JPRB, 0.16390E+01_JPRB, 0.18569E+01_JPRB, 0.22090E+01_JPRB, 0.22484E+01_JPRB, 0.29545E+01_JPRB ,&
&  0.37709E+01_JPRB, 0.33840E+01_JPRB, 0.27955E+01_JPRB, 0.22144E+01_JPRB, 0.17749E+01_JPRB, 0.14204E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.97174E+00_JPRB, 0.90280E+00_JPRB, 0.86713E+00_JPRB, 0.88030E+00_JPRB, 0.92996E+00_JPRB, 0.97459E+00_JPRB ,&
&  0.10360E+01_JPRB, 0.11036E+01_JPRB, 0.11473E+01_JPRB, 0.11494E+01_JPRB, 0.11033E+01_JPRB, 0.10231E+01_JPRB, 0.94424E+00_JPRB ,&
&  0.89131E+00_JPRB, 0.85795E+00_JPRB, 0.84715E+00_JPRB, 0.83737E+00_JPRB, 0.84216E+00_JPRB, 0.85409E+00_JPRB, 0.85051E+00_JPRB ,&
&  0.84715E+00_JPRB, 0.85225E+00_JPRB, 0.86854E+00_JPRB, 0.85738E+00_JPRB, 0.83371E+00_JPRB, 0.86609E+00_JPRB, 0.83957E+00_JPRB ,&
&  0.81476E+00_JPRB, 0.79134E+00_JPRB, 0.77219E+00_JPRB, 0.74319E+00_JPRB, 0.72660E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.69473E+00_JPRB, 0.67719E+00_JPRB, 0.65258E+00_JPRB, 0.63207E+00_JPRB, 0.62078E+00_JPRB, 0.61480E+00_JPRB, 0.61359E+00_JPRB ,&
&  0.61369E+00_JPRB, 0.60616E+00_JPRB, 0.59602E+00_JPRB, 0.58045E+00_JPRB, 0.56658E+00_JPRB, 0.55377E+00_JPRB, 0.54298E+00_JPRB ,&
&  0.53421E+00_JPRB, 0.52949E+00_JPRB, 0.52460E+00_JPRB, 0.51895E+00_JPRB, 0.51055E+00_JPRB, 0.49846E+00_JPRB, 0.49034E+00_JPRB ,&
&  0.48516E+00_JPRB, 0.48445E+00_JPRB, 0.48843E+00_JPRB, 0.49974E+00_JPRB, 0.51428E+00_JPRB, 0.53311E+00_JPRB, 0.56209E+00_JPRB ,&
&  0.65844E+00_JPRB, 0.10725E+01_JPRB, 0.11495E+01_JPRB, 0.11215E+01_JPRB, 0.11354E+01_JPRB, 0.11111E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.13019E+01_JPRB, 0.14187E+01_JPRB, 0.15221E+01_JPRB, 0.15580E+01_JPRB, 0.14974E+01_JPRB, 0.14375E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.11802E+01_JPRB, 0.10873E+01_JPRB, 0.10025E+01_JPRB, 0.94408E+00_JPRB, 0.95407E+00_JPRB, 0.10171E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.12436E+01_JPRB, 0.13306E+01_JPRB, 0.15085E+01_JPRB, 0.18764E+01_JPRB, 0.21049E+01_JPRB, 0.24088E+01_JPRB ,&
&  0.24812E+01_JPRB, 0.23967E+01_JPRB, 0.22410E+01_JPRB, 0.20274E+01_JPRB, 0.17470E+01_JPRB, 0.14836E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.99588E+00_JPRB, 0.92445E+00_JPRB, 0.89186E+00_JPRB, 0.90144E+00_JPRB, 0.94781E+00_JPRB, 0.10122E+01_JPRB ,&
&  0.10955E+01_JPRB, 0.11825E+01_JPRB, 0.12431E+01_JPRB, 0.12713E+01_JPRB, 0.12564E+01_JPRB, 0.12077E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.10502E+01_JPRB, 0.97928E+00_JPRB, 0.92501E+00_JPRB, 0.90268E+00_JPRB, 0.89973E+00_JPRB, 0.89948E+00_JPRB, 0.89700E+00_JPRB ,&
&  0.89976E+00_JPRB, 0.89992E+00_JPRB, 0.89476E+00_JPRB, 0.86898E+00_JPRB, 0.83576E+00_JPRB, 0.85328E+00_JPRB, 0.82539E+00_JPRB ,&
&  0.79721E+00_JPRB, 0.76563E+00_JPRB, 0.73816E+00_JPRB, 0.70596E+00_JPRB, 0.70242E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.71716E+00_JPRB, 0.68377E+00_JPRB, 0.64350E+00_JPRB, 0.61385E+00_JPRB, 0.59436E+00_JPRB, 0.58143E+00_JPRB, 0.57572E+00_JPRB ,&
&  0.57488E+00_JPRB, 0.56489E+00_JPRB, 0.54743E+00_JPRB, 0.53214E+00_JPRB, 0.52396E+00_JPRB, 0.51757E+00_JPRB, 0.50947E+00_JPRB ,&
&  0.50325E+00_JPRB, 0.49766E+00_JPRB, 0.49327E+00_JPRB, 0.49170E+00_JPRB, 0.49095E+00_JPRB, 0.48859E+00_JPRB, 0.48214E+00_JPRB ,&
&  0.47295E+00_JPRB, 0.46907E+00_JPRB, 0.46940E+00_JPRB, 0.47709E+00_JPRB, 0.48963E+00_JPRB, 0.50587E+00_JPRB, 0.53173E+00_JPRB ,&
&  0.63033E+00_JPRB, 0.71412E+00_JPRB, 0.79150E+00_JPRB, 0.80908E+00_JPRB, 0.84103E+00_JPRB, 0.91424E+00_JPRB, 0.97871E+00_JPRB ,&
&  0.10398E+01_JPRB, 0.11240E+01_JPRB, 0.12305E+01_JPRB, 0.12782E+01_JPRB, 0.12354E+01_JPRB, 0.11589E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.98404E+00_JPRB, 0.94110E+00_JPRB, 0.90329E+00_JPRB, 0.88989E+00_JPRB, 0.90720E+00_JPRB, 0.94262E+00_JPRB ,&
&  0.96982E+00_JPRB, 0.97030E+00_JPRB, 0.10635E+01_JPRB, 0.12094E+01_JPRB, 0.14183E+01_JPRB, 0.15691E+01_JPRB, 0.17189E+01_JPRB ,&
&  0.18407E+01_JPRB, 0.18929E+01_JPRB, 0.19066E+01_JPRB, 0.18570E+01_JPRB, 0.17255E+01_JPRB, 0.15682E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.12379E+01_JPRB, 0.11207E+01_JPRB, 0.10537E+01_JPRB, 0.10183E+01_JPRB, 0.10100E+01_JPRB, 0.10199E+01_JPRB, 0.10496E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.11427E+01_JPRB, 0.11822E+01_JPRB, 0.12018E+01_JPRB, 0.11987E+01_JPRB, 0.11925E+01_JPRB, 0.11812E+01_JPRB ,&
&  0.11638E+01_JPRB, 0.11355E+01_JPRB, 0.10930E+01_JPRB, 0.10450E+01_JPRB, 0.10017E+01_JPRB, 0.97040E+00_JPRB, 0.94493E+00_JPRB ,&
&  0.91878E+00_JPRB, 0.89867E+00_JPRB, 0.86041E+00_JPRB, 0.83476E+00_JPRB, 0.85039E+00_JPRB, 0.82517E+00_JPRB, 0.79424E+00_JPRB ,&
&  0.75784E+00_JPRB, 0.72549E+00_JPRB, 0.69557E+00_JPRB, 0.66437E+00_JPRB, 0.70752E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.61692E+00_JPRB, 0.61132E+00_JPRB, 0.59550E+00_JPRB, 0.57506E+00_JPRB, 0.56068E+00_JPRB, 0.54800E+00_JPRB, 0.53789E+00_JPRB ,&
&  0.53147E+00_JPRB, 0.52240E+00_JPRB, 0.50947E+00_JPRB, 0.49929E+00_JPRB, 0.49751E+00_JPRB, 0.49451E+00_JPRB, 0.48917E+00_JPRB ,&
&  0.48004E+00_JPRB, 0.47115E+00_JPRB, 0.46509E+00_JPRB, 0.46334E+00_JPRB, 0.46460E+00_JPRB, 0.46705E+00_JPRB, 0.46753E+00_JPRB ,&
&  0.46362E+00_JPRB, 0.45724E+00_JPRB, 0.45535E+00_JPRB, 0.45705E+00_JPRB, 0.46336E+00_JPRB, 0.47456E+00_JPRB, 0.49840E+00_JPRB ,&
&  0.57044E+00_JPRB, 0.57470E+00_JPRB, 0.61032E+00_JPRB, 0.63276E+00_JPRB, 0.66931E+00_JPRB, 0.73638E+00_JPRB, 0.81236E+00_JPRB ,&
&  0.85597E+00_JPRB, 0.89822E+00_JPRB, 0.97046E+00_JPRB, 0.10304E+01_JPRB, 0.10306E+01_JPRB, 0.99074E+00_JPRB, 0.93342E+00_JPRB ,&
&  0.87584E+00_JPRB, 0.83288E+00_JPRB, 0.79760E+00_JPRB, 0.77164E+00_JPRB, 0.76459E+00_JPRB, 0.77229E+00_JPRB, 0.77762E+00_JPRB ,&
&  0.76108E+00_JPRB, 0.76831E+00_JPRB, 0.83446E+00_JPRB, 0.93804E+00_JPRB, 0.10720E+01_JPRB, 0.12085E+01_JPRB, 0.13474E+01_JPRB ,&
&  0.14620E+01_JPRB, 0.15302E+01_JPRB, 0.15832E+01_JPRB, 0.16226E+01_JPRB, 0.16054E+01_JPRB, 0.15739E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.13607E+01_JPRB, 0.12590E+01_JPRB, 0.11878E+01_JPRB, 0.11306E+01_JPRB, 0.10881E+01_JPRB, 0.10605E+01_JPRB, 0.10457E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.10509E+01_JPRB, 0.10518E+01_JPRB, 0.10460E+01_JPRB, 0.10386E+01_JPRB, 0.10344E+01_JPRB, 0.10315E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.10289E+01_JPRB, 0.10175E+01_JPRB, 0.99120E+00_JPRB, 0.95250E+00_JPRB, 0.91319E+00_JPRB, 0.87450E+00_JPRB ,&
&  0.84179E+00_JPRB, 0.82367E+00_JPRB, 0.80266E+00_JPRB, 0.79211E+00_JPRB, 0.76784E+00_JPRB, 0.74449E+00_JPRB, 0.72087E+00_JPRB ,&
&  0.69576E+00_JPRB, 0.67052E+00_JPRB, 0.63960E+00_JPRB, 0.60967E+00_JPRB, 0.59951E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.55273E+00_JPRB, 0.55690E+00_JPRB, 0.55523E+00_JPRB, 0.54223E+00_JPRB, 0.52811E+00_JPRB, 0.52000E+00_JPRB, 0.51086E+00_JPRB ,&
&  0.50058E+00_JPRB, 0.49246E+00_JPRB, 0.48115E+00_JPRB, 0.47531E+00_JPRB, 0.47294E+00_JPRB, 0.47228E+00_JPRB, 0.46965E+00_JPRB ,&
&  0.46305E+00_JPRB, 0.45318E+00_JPRB, 0.44581E+00_JPRB, 0.44291E+00_JPRB, 0.44410E+00_JPRB, 0.44670E+00_JPRB, 0.44874E+00_JPRB ,&
&  0.44668E+00_JPRB, 0.44451E+00_JPRB, 0.44200E+00_JPRB, 0.44030E+00_JPRB, 0.44092E+00_JPRB, 0.44961E+00_JPRB, 0.46746E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.49038E+00_JPRB, 0.50844E+00_JPRB, 0.54332E+00_JPRB, 0.58092E+00_JPRB, 0.63774E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.73907E+00_JPRB, 0.74863E+00_JPRB, 0.77504E+00_JPRB, 0.82168E+00_JPRB, 0.82951E+00_JPRB, 0.82324E+00_JPRB, 0.80007E+00_JPRB ,&
&  0.76192E+00_JPRB, 0.71551E+00_JPRB, 0.67111E+00_JPRB, 0.64198E+00_JPRB, 0.63627E+00_JPRB, 0.64165E+00_JPRB, 0.63988E+00_JPRB ,&
&  0.61655E+00_JPRB, 0.60947E+00_JPRB, 0.63983E+00_JPRB, 0.68581E+00_JPRB, 0.74245E+00_JPRB, 0.83070E+00_JPRB, 0.94232E+00_JPRB ,&
&  0.10412E+01_JPRB, 0.11412E+01_JPRB, 0.12450E+01_JPRB, 0.13240E+01_JPRB, 0.13860E+01_JPRB, 0.14044E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.13142E+01_JPRB, 0.12423E+01_JPRB, 0.11726E+01_JPRB, 0.11045E+01_JPRB, 0.10460E+01_JPRB, 0.10076E+01_JPRB, 0.98211E+00_JPRB ,&
&  0.96437E+00_JPRB, 0.94981E+00_JPRB, 0.92881E+00_JPRB, 0.90005E+00_JPRB, 0.87007E+00_JPRB, 0.84459E+00_JPRB, 0.82244E+00_JPRB ,&
&  0.80363E+00_JPRB, 0.78391E+00_JPRB, 0.76819E+00_JPRB, 0.75620E+00_JPRB, 0.74332E+00_JPRB, 0.73129E+00_JPRB, 0.72010E+00_JPRB ,&
&  0.70787E+00_JPRB, 0.70171E+00_JPRB, 0.68816E+00_JPRB, 0.67327E+00_JPRB, 0.67766E+00_JPRB, 0.66483E+00_JPRB, 0.65296E+00_JPRB ,&
&  0.63460E+00_JPRB, 0.61386E+00_JPRB, 0.59050E+00_JPRB, 0.53673E+00_JPRB, 0.55100E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.50704E+00_JPRB, 0.51110E+00_JPRB, 0.51029E+00_JPRB, 0.50420E+00_JPRB, 0.49502E+00_JPRB, 0.48860E+00_JPRB, 0.47922E+00_JPRB ,&
&  0.46889E+00_JPRB, 0.46382E+00_JPRB, 0.45770E+00_JPRB, 0.45486E+00_JPRB, 0.45528E+00_JPRB, 0.45137E+00_JPRB, 0.44755E+00_JPRB ,&
&  0.44388E+00_JPRB, 0.43729E+00_JPRB, 0.43291E+00_JPRB, 0.42945E+00_JPRB, 0.42923E+00_JPRB, 0.42974E+00_JPRB, 0.42955E+00_JPRB ,&
&  0.42689E+00_JPRB, 0.42262E+00_JPRB, 0.41891E+00_JPRB, 0.41620E+00_JPRB, 0.41716E+00_JPRB, 0.42387E+00_JPRB, 0.43768E+00_JPRB ,&
&  0.47567E+00_JPRB, 0.43425E+00_JPRB, 0.45544E+00_JPRB, 0.47936E+00_JPRB, 0.51591E+00_JPRB, 0.55094E+00_JPRB, 0.57272E+00_JPRB ,&
&  0.59823E+00_JPRB, 0.62229E+00_JPRB, 0.64535E+00_JPRB, 0.66947E+00_JPRB, 0.67224E+00_JPRB, 0.66393E+00_JPRB, 0.65385E+00_JPRB ,&
&  0.63709E+00_JPRB, 0.61517E+00_JPRB, 0.59287E+00_JPRB, 0.57107E+00_JPRB, 0.56111E+00_JPRB, 0.56115E+00_JPRB, 0.55419E+00_JPRB ,&
&  0.53610E+00_JPRB, 0.52156E+00_JPRB, 0.51519E+00_JPRB, 0.52079E+00_JPRB, 0.53891E+00_JPRB, 0.57201E+00_JPRB, 0.63660E+00_JPRB ,&
&  0.71282E+00_JPRB, 0.80388E+00_JPRB, 0.90599E+00_JPRB, 0.99114E+00_JPRB, 0.10558E+01_JPRB, 0.10897E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.10946E+01_JPRB, 0.10740E+01_JPRB, 0.10480E+01_JPRB, 0.10118E+01_JPRB, 0.96269E+00_JPRB, 0.91370E+00_JPRB, 0.87318E+00_JPRB ,&
&  0.84137E+00_JPRB, 0.80920E+00_JPRB, 0.77527E+00_JPRB, 0.73853E+00_JPRB, 0.69923E+00_JPRB, 0.66230E+00_JPRB, 0.63127E+00_JPRB ,&
&  0.60505E+00_JPRB, 0.58286E+00_JPRB, 0.56911E+00_JPRB, 0.56307E+00_JPRB, 0.56638E+00_JPRB, 0.57267E+00_JPRB, 0.57653E+00_JPRB ,&
&  0.57955E+00_JPRB, 0.58476E+00_JPRB, 0.58656E+00_JPRB, 0.59260E+00_JPRB, 0.58931E+00_JPRB, 0.58423E+00_JPRB, 0.57988E+00_JPRB ,&
&  0.57107E+00_JPRB, 0.55423E+00_JPRB, 0.48874E+00_JPRB, 0.50644E+00_JPRB, 0.50576E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.47261E+00_JPRB, 0.46759E+00_JPRB, 0.46355E+00_JPRB, 0.46519E+00_JPRB, 0.46426E+00_JPRB, 0.46053E+00_JPRB, 0.45409E+00_JPRB ,&
&  0.44365E+00_JPRB, 0.43696E+00_JPRB, 0.43254E+00_JPRB, 0.43189E+00_JPRB, 0.43531E+00_JPRB, 0.43358E+00_JPRB, 0.42850E+00_JPRB ,&
&  0.42455E+00_JPRB, 0.42004E+00_JPRB, 0.41658E+00_JPRB, 0.41266E+00_JPRB, 0.40927E+00_JPRB, 0.40750E+00_JPRB, 0.40636E+00_JPRB ,&
&  0.40417E+00_JPRB, 0.40015E+00_JPRB, 0.39628E+00_JPRB, 0.39374E+00_JPRB, 0.39526E+00_JPRB, 0.40100E+00_JPRB, 0.41006E+00_JPRB ,&
&  0.41007E+00_JPRB, 0.39419E+00_JPRB, 0.41583E+00_JPRB, 0.44295E+00_JPRB, 0.46242E+00_JPRB, 0.48395E+00_JPRB, 0.50255E+00_JPRB ,&
&  0.52009E+00_JPRB, 0.52662E+00_JPRB, 0.53476E+00_JPRB, 0.54356E+00_JPRB, 0.54952E+00_JPRB, 0.54794E+00_JPRB, 0.54426E+00_JPRB ,&
&  0.54005E+00_JPRB, 0.53504E+00_JPRB, 0.52899E+00_JPRB, 0.51945E+00_JPRB, 0.51013E+00_JPRB, 0.50191E+00_JPRB, 0.49362E+00_JPRB ,&
&  0.48392E+00_JPRB, 0.47468E+00_JPRB, 0.46877E+00_JPRB, 0.46750E+00_JPRB, 0.47181E+00_JPRB, 0.48003E+00_JPRB, 0.49503E+00_JPRB ,&
&  0.51406E+00_JPRB, 0.54956E+00_JPRB, 0.61581E+00_JPRB, 0.68890E+00_JPRB, 0.74945E+00_JPRB, 0.79259E+00_JPRB, 0.82710E+00_JPRB ,&
&  0.84110E+00_JPRB, 0.84194E+00_JPRB, 0.83718E+00_JPRB, 0.82429E+00_JPRB, 0.80314E+00_JPRB, 0.77303E+00_JPRB, 0.73616E+00_JPRB ,&
&  0.70248E+00_JPRB, 0.67261E+00_JPRB, 0.64320E+00_JPRB, 0.61395E+00_JPRB, 0.58469E+00_JPRB, 0.55746E+00_JPRB, 0.53405E+00_JPRB ,&
&  0.51389E+00_JPRB, 0.49614E+00_JPRB, 0.48277E+00_JPRB, 0.47514E+00_JPRB, 0.47518E+00_JPRB, 0.48087E+00_JPRB, 0.48935E+00_JPRB ,&
&  0.49637E+00_JPRB, 0.50570E+00_JPRB, 0.51351E+00_JPRB, 0.52038E+00_JPRB, 0.52487E+00_JPRB, 0.52280E+00_JPRB, 0.52019E+00_JPRB ,&
&  0.51537E+00_JPRB, 0.50488E+00_JPRB, 0.49502E+00_JPRB, 0.48425E+00_JPRB, 0.47618E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.43614E+00_JPRB, 0.42942E+00_JPRB, 0.42864E+00_JPRB, 0.43236E+00_JPRB, 0.43681E+00_JPRB, 0.43693E+00_JPRB, 0.43295E+00_JPRB ,&
&  0.42303E+00_JPRB, 0.41436E+00_JPRB, 0.40785E+00_JPRB, 0.40555E+00_JPRB, 0.40845E+00_JPRB, 0.40945E+00_JPRB, 0.40667E+00_JPRB ,&
&  0.40326E+00_JPRB, 0.39679E+00_JPRB, 0.39108E+00_JPRB, 0.38545E+00_JPRB, 0.38133E+00_JPRB, 0.37934E+00_JPRB, 0.37836E+00_JPRB ,&
&  0.37663E+00_JPRB, 0.37440E+00_JPRB, 0.37335E+00_JPRB, 0.37260E+00_JPRB, 0.37370E+00_JPRB, 0.37750E+00_JPRB, 0.38314E+00_JPRB ,&
&  0.39449E+00_JPRB, 0.38144E+00_JPRB, 0.40309E+00_JPRB, 0.40881E+00_JPRB, 0.41699E+00_JPRB, 0.42895E+00_JPRB, 0.44476E+00_JPRB ,&
&  0.45911E+00_JPRB, 0.46511E+00_JPRB, 0.46802E+00_JPRB, 0.46983E+00_JPRB, 0.47010E+00_JPRB, 0.47218E+00_JPRB, 0.47069E+00_JPRB ,&
&  0.46645E+00_JPRB, 0.46433E+00_JPRB, 0.46319E+00_JPRB, 0.46078E+00_JPRB, 0.45856E+00_JPRB, 0.45485E+00_JPRB, 0.45049E+00_JPRB ,&
&  0.44640E+00_JPRB, 0.44188E+00_JPRB, 0.43782E+00_JPRB, 0.43339E+00_JPRB, 0.43020E+00_JPRB, 0.43028E+00_JPRB, 0.43417E+00_JPRB ,&
&  0.43896E+00_JPRB, 0.44747E+00_JPRB, 0.46675E+00_JPRB, 0.49686E+00_JPRB, 0.53397E+00_JPRB, 0.57110E+00_JPRB, 0.60096E+00_JPRB ,&
&  0.61961E+00_JPRB, 0.62949E+00_JPRB, 0.63116E+00_JPRB, 0.62797E+00_JPRB, 0.61897E+00_JPRB, 0.60651E+00_JPRB, 0.58937E+00_JPRB ,&
&  0.57199E+00_JPRB, 0.55512E+00_JPRB, 0.53776E+00_JPRB, 0.52026E+00_JPRB, 0.50145E+00_JPRB, 0.48203E+00_JPRB, 0.46430E+00_JPRB ,&
&  0.44989E+00_JPRB, 0.43930E+00_JPRB, 0.43155E+00_JPRB, 0.42670E+00_JPRB, 0.42399E+00_JPRB, 0.42488E+00_JPRB, 0.42806E+00_JPRB ,&
&  0.43322E+00_JPRB, 0.43986E+00_JPRB, 0.44723E+00_JPRB, 0.45329E+00_JPRB, 0.45903E+00_JPRB, 0.46081E+00_JPRB, 0.46175E+00_JPRB ,&
&  0.46011E+00_JPRB, 0.45487E+00_JPRB, 0.45064E+00_JPRB, 0.44612E+00_JPRB, 0.44203E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.40184E+00_JPRB, 0.39884E+00_JPRB, 0.40132E+00_JPRB, 0.40586E+00_JPRB, 0.41300E+00_JPRB, 0.41299E+00_JPRB, 0.41109E+00_JPRB ,&
&  0.40540E+00_JPRB, 0.39760E+00_JPRB, 0.39014E+00_JPRB, 0.38335E+00_JPRB, 0.38104E+00_JPRB, 0.38220E+00_JPRB, 0.38236E+00_JPRB ,&
&  0.38091E+00_JPRB, 0.37605E+00_JPRB, 0.36959E+00_JPRB, 0.36345E+00_JPRB, 0.35902E+00_JPRB, 0.35651E+00_JPRB, 0.35449E+00_JPRB ,&
&  0.35341E+00_JPRB, 0.35158E+00_JPRB, 0.35235E+00_JPRB, 0.35231E+00_JPRB, 0.35285E+00_JPRB, 0.35623E+00_JPRB, 0.35990E+00_JPRB ,&
&  0.36298E+00_JPRB, 0.37865E+00_JPRB, 0.37474E+00_JPRB, 0.37350E+00_JPRB, 0.37780E+00_JPRB, 0.38717E+00_JPRB, 0.40114E+00_JPRB ,&
&  0.41465E+00_JPRB, 0.42158E+00_JPRB, 0.42378E+00_JPRB, 0.42127E+00_JPRB, 0.41808E+00_JPRB, 0.41675E+00_JPRB, 0.41744E+00_JPRB ,&
&  0.41708E+00_JPRB, 0.41359E+00_JPRB, 0.40885E+00_JPRB, 0.40470E+00_JPRB, 0.40322E+00_JPRB, 0.40237E+00_JPRB, 0.40107E+00_JPRB ,&
&  0.39943E+00_JPRB, 0.39731E+00_JPRB, 0.39331E+00_JPRB, 0.38799E+00_JPRB, 0.38290E+00_JPRB, 0.38078E+00_JPRB, 0.38234E+00_JPRB ,&
&  0.38662E+00_JPRB, 0.39300E+00_JPRB, 0.40264E+00_JPRB, 0.41505E+00_JPRB, 0.42865E+00_JPRB, 0.44570E+00_JPRB, 0.46224E+00_JPRB ,&
&  0.47454E+00_JPRB, 0.48085E+00_JPRB, 0.48201E+00_JPRB, 0.48033E+00_JPRB, 0.47632E+00_JPRB, 0.47085E+00_JPRB, 0.46509E+00_JPRB ,&
&  0.45945E+00_JPRB, 0.45310E+00_JPRB, 0.44587E+00_JPRB, 0.43749E+00_JPRB, 0.42678E+00_JPRB, 0.41556E+00_JPRB, 0.40626E+00_JPRB ,&
&  0.39974E+00_JPRB, 0.39511E+00_JPRB, 0.39177E+00_JPRB, 0.38897E+00_JPRB, 0.38732E+00_JPRB, 0.38716E+00_JPRB, 0.38878E+00_JPRB ,&
&  0.39200E+00_JPRB, 0.39636E+00_JPRB, 0.40174E+00_JPRB, 0.40693E+00_JPRB, 0.41034E+00_JPRB, 0.41294E+00_JPRB, 0.41543E+00_JPRB ,&
&  0.41586E+00_JPRB, 0.41323E+00_JPRB, 0.41123E+00_JPRB, 0.40977E+00_JPRB, 0.40653E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.37271E+00_JPRB, 0.37241E+00_JPRB, 0.37722E+00_JPRB, 0.38228E+00_JPRB, 0.38801E+00_JPRB, 0.39037E+00_JPRB, 0.38829E+00_JPRB ,&
&  0.38542E+00_JPRB, 0.38143E+00_JPRB, 0.37532E+00_JPRB, 0.36758E+00_JPRB, 0.36059E+00_JPRB, 0.35892E+00_JPRB, 0.35937E+00_JPRB ,&
&  0.35968E+00_JPRB, 0.35716E+00_JPRB, 0.35269E+00_JPRB, 0.34786E+00_JPRB, 0.34252E+00_JPRB, 0.33782E+00_JPRB, 0.33427E+00_JPRB ,&
&  0.33323E+00_JPRB, 0.33268E+00_JPRB, 0.33347E+00_JPRB, 0.33435E+00_JPRB, 0.33540E+00_JPRB, 0.33788E+00_JPRB, 0.34399E+00_JPRB ,&
&  0.34301E+00_JPRB, 0.34943E+00_JPRB, 0.34000E+00_JPRB, 0.34317E+00_JPRB, 0.34664E+00_JPRB, 0.35215E+00_JPRB, 0.35971E+00_JPRB ,&
&  0.36857E+00_JPRB, 0.37492E+00_JPRB, 0.37870E+00_JPRB, 0.37849E+00_JPRB, 0.37638E+00_JPRB, 0.37642E+00_JPRB, 0.37711E+00_JPRB ,&
&  0.37722E+00_JPRB, 0.37678E+00_JPRB, 0.37464E+00_JPRB, 0.36991E+00_JPRB, 0.36345E+00_JPRB, 0.35806E+00_JPRB, 0.35443E+00_JPRB ,&
&  0.35403E+00_JPRB, 0.35400E+00_JPRB, 0.35259E+00_JPRB, 0.35038E+00_JPRB, 0.34788E+00_JPRB, 0.34671E+00_JPRB, 0.34809E+00_JPRB ,&
&  0.35117E+00_JPRB, 0.35454E+00_JPRB, 0.35778E+00_JPRB, 0.36360E+00_JPRB, 0.37151E+00_JPRB, 0.37892E+00_JPRB, 0.38590E+00_JPRB ,&
&  0.39186E+00_JPRB, 0.39391E+00_JPRB, 0.39273E+00_JPRB, 0.38990E+00_JPRB, 0.38766E+00_JPRB, 0.38600E+00_JPRB, 0.38541E+00_JPRB ,&
&  0.38441E+00_JPRB, 0.38262E+00_JPRB, 0.37953E+00_JPRB, 0.37524E+00_JPRB, 0.37085E+00_JPRB, 0.36653E+00_JPRB, 0.36324E+00_JPRB ,&
&  0.36121E+00_JPRB, 0.35977E+00_JPRB, 0.35824E+00_JPRB, 0.35678E+00_JPRB, 0.35660E+00_JPRB, 0.35687E+00_JPRB, 0.35781E+00_JPRB ,&
&  0.35945E+00_JPRB, 0.36190E+00_JPRB, 0.36535E+00_JPRB, 0.36899E+00_JPRB, 0.36913E+00_JPRB, 0.36924E+00_JPRB, 0.37132E+00_JPRB ,&
&  0.37351E+00_JPRB, 0.37427E+00_JPRB, 0.37406E+00_JPRB, 0.37390E+00_JPRB, 0.37410E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.34843E+00_JPRB, 0.35031E+00_JPRB, 0.35444E+00_JPRB, 0.36071E+00_JPRB, 0.36423E+00_JPRB, 0.36846E+00_JPRB, 0.36855E+00_JPRB ,&
&  0.36592E+00_JPRB, 0.36231E+00_JPRB, 0.35792E+00_JPRB, 0.35221E+00_JPRB, 0.34599E+00_JPRB, 0.34133E+00_JPRB, 0.33978E+00_JPRB ,&
&  0.33968E+00_JPRB, 0.33822E+00_JPRB, 0.33488E+00_JPRB, 0.33059E+00_JPRB, 0.32591E+00_JPRB, 0.32207E+00_JPRB, 0.31918E+00_JPRB ,&
&  0.31848E+00_JPRB, 0.31913E+00_JPRB, 0.32055E+00_JPRB, 0.32148E+00_JPRB, 0.32168E+00_JPRB, 0.32392E+00_JPRB, 0.32849E+00_JPRB ,&
&  0.32711E+00_JPRB, 0.32555E+00_JPRB, 0.31831E+00_JPRB, 0.31772E+00_JPRB, 0.31958E+00_JPRB, 0.32225E+00_JPRB, 0.32559E+00_JPRB ,&
&  0.32943E+00_JPRB, 0.33317E+00_JPRB, 0.33727E+00_JPRB, 0.34059E+00_JPRB, 0.34271E+00_JPRB, 0.34442E+00_JPRB, 0.34550E+00_JPRB ,&
&  0.34606E+00_JPRB, 0.34522E+00_JPRB, 0.34304E+00_JPRB, 0.33905E+00_JPRB, 0.33395E+00_JPRB, 0.33042E+00_JPRB, 0.32906E+00_JPRB ,&
&  0.32793E+00_JPRB, 0.32810E+00_JPRB, 0.32816E+00_JPRB, 0.32730E+00_JPRB, 0.32592E+00_JPRB, 0.32561E+00_JPRB, 0.32659E+00_JPRB ,&
&  0.32781E+00_JPRB, 0.32825E+00_JPRB, 0.32804E+00_JPRB, 0.32926E+00_JPRB, 0.33297E+00_JPRB, 0.33736E+00_JPRB, 0.34066E+00_JPRB ,&
&  0.34270E+00_JPRB, 0.34231E+00_JPRB, 0.34028E+00_JPRB, 0.33710E+00_JPRB, 0.33460E+00_JPRB, 0.33419E+00_JPRB, 0.33571E+00_JPRB ,&
&  0.33703E+00_JPRB, 0.33723E+00_JPRB, 0.33731E+00_JPRB, 0.33544E+00_JPRB, 0.33328E+00_JPRB, 0.33226E+00_JPRB, 0.33154E+00_JPRB ,&
&  0.33036E+00_JPRB, 0.32901E+00_JPRB, 0.32789E+00_JPRB, 0.32742E+00_JPRB, 0.32772E+00_JPRB, 0.32748E+00_JPRB, 0.32794E+00_JPRB ,&
&  0.33039E+00_JPRB, 0.33356E+00_JPRB, 0.33643E+00_JPRB, 0.33830E+00_JPRB, 0.33890E+00_JPRB, 0.33837E+00_JPRB, 0.34005E+00_JPRB ,&
&  0.34319E+00_JPRB, 0.34561E+00_JPRB, 0.34572E+00_JPRB, 0.34524E+00_JPRB, 0.34670E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.32756E+00_JPRB, 0.33013E+00_JPRB, 0.33453E+00_JPRB, 0.33961E+00_JPRB, 0.34283E+00_JPRB, 0.34762E+00_JPRB, 0.34854E+00_JPRB ,&
&  0.34795E+00_JPRB, 0.34617E+00_JPRB, 0.34183E+00_JPRB, 0.33646E+00_JPRB, 0.33115E+00_JPRB, 0.32660E+00_JPRB, 0.32346E+00_JPRB ,&
&  0.32175E+00_JPRB, 0.32022E+00_JPRB, 0.31835E+00_JPRB, 0.31553E+00_JPRB, 0.31224E+00_JPRB, 0.30933E+00_JPRB, 0.30715E+00_JPRB ,&
&  0.30658E+00_JPRB, 0.30676E+00_JPRB, 0.30733E+00_JPRB, 0.30742E+00_JPRB, 0.30704E+00_JPRB, 0.30810E+00_JPRB, 0.31000E+00_JPRB ,&
&  0.30913E+00_JPRB, 0.30481E+00_JPRB, 0.30149E+00_JPRB, 0.30115E+00_JPRB, 0.30109E+00_JPRB, 0.30146E+00_JPRB, 0.30229E+00_JPRB ,&
&  0.30290E+00_JPRB, 0.30354E+00_JPRB, 0.30554E+00_JPRB, 0.30803E+00_JPRB, 0.31033E+00_JPRB, 0.31355E+00_JPRB, 0.31521E+00_JPRB ,&
&  0.31549E+00_JPRB, 0.31470E+00_JPRB, 0.31317E+00_JPRB, 0.31060E+00_JPRB, 0.30898E+00_JPRB, 0.30876E+00_JPRB, 0.30826E+00_JPRB ,&
&  0.30916E+00_JPRB, 0.31008E+00_JPRB, 0.31089E+00_JPRB, 0.30974E+00_JPRB, 0.30761E+00_JPRB, 0.30731E+00_JPRB, 0.30790E+00_JPRB ,&
&  0.30821E+00_JPRB, 0.30743E+00_JPRB, 0.30574E+00_JPRB, 0.30502E+00_JPRB, 0.30516E+00_JPRB, 0.30628E+00_JPRB, 0.30670E+00_JPRB ,&
&  0.30611E+00_JPRB, 0.30423E+00_JPRB, 0.30275E+00_JPRB, 0.30193E+00_JPRB, 0.30147E+00_JPRB, 0.30173E+00_JPRB, 0.30163E+00_JPRB ,&
&  0.30281E+00_JPRB, 0.30393E+00_JPRB, 0.30315E+00_JPRB, 0.30233E+00_JPRB, 0.30154E+00_JPRB, 0.30132E+00_JPRB, 0.30085E+00_JPRB ,&
&  0.29992E+00_JPRB, 0.29891E+00_JPRB, 0.29834E+00_JPRB, 0.29836E+00_JPRB, 0.29728E+00_JPRB, 0.29590E+00_JPRB, 0.29652E+00_JPRB ,&
&  0.29992E+00_JPRB, 0.30370E+00_JPRB, 0.30696E+00_JPRB, 0.30906E+00_JPRB, 0.30970E+00_JPRB, 0.31110E+00_JPRB, 0.31390E+00_JPRB ,&
&  0.31623E+00_JPRB, 0.31719E+00_JPRB, 0.31962E+00_JPRB, 0.32181E+00_JPRB, 0.32410E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.30948E+00_JPRB, 0.31323E+00_JPRB, 0.31581E+00_JPRB, 0.31924E+00_JPRB, 0.32219E+00_JPRB, 0.32744E+00_JPRB, 0.32953E+00_JPRB ,&
&  0.32947E+00_JPRB, 0.32937E+00_JPRB, 0.32801E+00_JPRB, 0.32535E+00_JPRB, 0.32111E+00_JPRB, 0.31662E+00_JPRB, 0.31236E+00_JPRB ,&
&  0.30904E+00_JPRB, 0.30638E+00_JPRB, 0.30435E+00_JPRB, 0.30256E+00_JPRB, 0.30006E+00_JPRB, 0.29773E+00_JPRB, 0.29602E+00_JPRB ,&
&  0.29535E+00_JPRB, 0.29485E+00_JPRB, 0.29470E+00_JPRB, 0.29481E+00_JPRB, 0.29441E+00_JPRB, 0.29371E+00_JPRB, 0.29358E+00_JPRB ,&
&  0.29222E+00_JPRB, 0.28825E+00_JPRB, 0.28584E+00_JPRB, 0.28498E+00_JPRB, 0.28410E+00_JPRB, 0.28449E+00_JPRB, 0.28398E+00_JPRB ,&
&  0.28366E+00_JPRB, 0.28382E+00_JPRB, 0.28548E+00_JPRB, 0.28780E+00_JPRB, 0.28852E+00_JPRB, 0.28976E+00_JPRB, 0.29165E+00_JPRB ,&
&  0.29244E+00_JPRB, 0.29164E+00_JPRB, 0.28972E+00_JPRB, 0.28941E+00_JPRB, 0.29062E+00_JPRB, 0.29109E+00_JPRB, 0.29172E+00_JPRB ,&
&  0.29249E+00_JPRB, 0.29376E+00_JPRB, 0.29446E+00_JPRB, 0.29247E+00_JPRB, 0.29043E+00_JPRB, 0.28971E+00_JPRB, 0.28972E+00_JPRB ,&
&  0.28952E+00_JPRB, 0.28858E+00_JPRB, 0.28642E+00_JPRB, 0.28436E+00_JPRB, 0.28148E+00_JPRB, 0.27921E+00_JPRB, 0.27806E+00_JPRB ,&
&  0.27845E+00_JPRB, 0.27887E+00_JPRB, 0.28030E+00_JPRB, 0.28157E+00_JPRB, 0.28214E+00_JPRB, 0.28210E+00_JPRB, 0.28052E+00_JPRB ,&
&  0.27885E+00_JPRB, 0.27831E+00_JPRB, 0.27708E+00_JPRB, 0.27576E+00_JPRB, 0.27593E+00_JPRB, 0.27595E+00_JPRB, 0.27540E+00_JPRB ,&
&  0.27484E+00_JPRB, 0.27530E+00_JPRB, 0.27564E+00_JPRB, 0.27459E+00_JPRB, 0.27328E+00_JPRB, 0.27189E+00_JPRB, 0.27233E+00_JPRB ,&
&  0.27400E+00_JPRB, 0.27653E+00_JPRB, 0.27942E+00_JPRB, 0.28255E+00_JPRB, 0.28568E+00_JPRB, 0.28843E+00_JPRB, 0.29125E+00_JPRB ,&
&  0.29466E+00_JPRB, 0.29562E+00_JPRB, 0.29787E+00_JPRB, 0.30140E+00_JPRB, 0.30477E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.28850E+00_JPRB, 0.29395E+00_JPRB, 0.29674E+00_JPRB, 0.29885E+00_JPRB, 0.30134E+00_JPRB, 0.30618E+00_JPRB, 0.30964E+00_JPRB ,&
&  0.31045E+00_JPRB, 0.31069E+00_JPRB, 0.31058E+00_JPRB, 0.30970E+00_JPRB, 0.30803E+00_JPRB, 0.30635E+00_JPRB, 0.30342E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29628E+00_JPRB, 0.29321E+00_JPRB, 0.29034E+00_JPRB, 0.28765E+00_JPRB, 0.28520E+00_JPRB, 0.28342E+00_JPRB ,&
&  0.28315E+00_JPRB, 0.28336E+00_JPRB, 0.28276E+00_JPRB, 0.28276E+00_JPRB, 0.28263E+00_JPRB, 0.28139E+00_JPRB, 0.27950E+00_JPRB ,&
&  0.27711E+00_JPRB, 0.27330E+00_JPRB, 0.26945E+00_JPRB, 0.26859E+00_JPRB, 0.26827E+00_JPRB, 0.26723E+00_JPRB, 0.26573E+00_JPRB ,&
&  0.26687E+00_JPRB, 0.26895E+00_JPRB, 0.27146E+00_JPRB, 0.27290E+00_JPRB, 0.27394E+00_JPRB, 0.27571E+00_JPRB, 0.27785E+00_JPRB ,&
&  0.27828E+00_JPRB, 0.27553E+00_JPRB, 0.27302E+00_JPRB, 0.27280E+00_JPRB, 0.27306E+00_JPRB, 0.27296E+00_JPRB, 0.27364E+00_JPRB ,&
&  0.27447E+00_JPRB, 0.27627E+00_JPRB, 0.27743E+00_JPRB, 0.27615E+00_JPRB, 0.27377E+00_JPRB, 0.27205E+00_JPRB, 0.27098E+00_JPRB ,&
&  0.27052E+00_JPRB, 0.27005E+00_JPRB, 0.26912E+00_JPRB, 0.26751E+00_JPRB, 0.26563E+00_JPRB, 0.26373E+00_JPRB, 0.26174E+00_JPRB ,&
&  0.26000E+00_JPRB, 0.26030E+00_JPRB, 0.26326E+00_JPRB, 0.26480E+00_JPRB, 0.26403E+00_JPRB, 0.26247E+00_JPRB, 0.26099E+00_JPRB ,&
&  0.26023E+00_JPRB, 0.25744E+00_JPRB, 0.25486E+00_JPRB, 0.25296E+00_JPRB, 0.25184E+00_JPRB, 0.25022E+00_JPRB, 0.24974E+00_JPRB ,&
&  0.25039E+00_JPRB, 0.25241E+00_JPRB, 0.25271E+00_JPRB, 0.25226E+00_JPRB, 0.25149E+00_JPRB, 0.25090E+00_JPRB, 0.25191E+00_JPRB ,&
&  0.25336E+00_JPRB, 0.25589E+00_JPRB, 0.25877E+00_JPRB, 0.26258E+00_JPRB, 0.26586E+00_JPRB, 0.26971E+00_JPRB, 0.27278E+00_JPRB ,&
&  0.27475E+00_JPRB, 0.27533E+00_JPRB, 0.27663E+00_JPRB, 0.27881E+00_JPRB, 0.28253E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.26638E+00_JPRB, 0.27248E+00_JPRB, 0.27754E+00_JPRB, 0.28001E+00_JPRB, 0.28138E+00_JPRB, 0.28373E+00_JPRB, 0.28635E+00_JPRB ,&
&  0.28842E+00_JPRB, 0.28983E+00_JPRB, 0.29068E+00_JPRB, 0.29086E+00_JPRB, 0.29019E+00_JPRB, 0.28884E+00_JPRB, 0.28715E+00_JPRB ,&
&  0.28519E+00_JPRB, 0.28228E+00_JPRB, 0.27955E+00_JPRB, 0.27753E+00_JPRB, 0.27593E+00_JPRB, 0.27411E+00_JPRB, 0.27270E+00_JPRB ,&
&  0.27137E+00_JPRB, 0.27125E+00_JPRB, 0.27078E+00_JPRB, 0.26993E+00_JPRB, 0.26899E+00_JPRB, 0.26761E+00_JPRB, 0.26651E+00_JPRB ,&
&  0.26336E+00_JPRB, 0.25832E+00_JPRB, 0.25667E+00_JPRB, 0.25521E+00_JPRB, 0.25537E+00_JPRB, 0.25450E+00_JPRB, 0.25330E+00_JPRB ,&
&  0.25481E+00_JPRB, 0.25786E+00_JPRB, 0.26026E+00_JPRB, 0.26153E+00_JPRB, 0.26246E+00_JPRB, 0.26399E+00_JPRB, 0.26562E+00_JPRB ,&
&  0.26331E+00_JPRB, 0.26029E+00_JPRB, 0.25814E+00_JPRB, 0.25728E+00_JPRB, 0.25747E+00_JPRB, 0.25870E+00_JPRB, 0.26083E+00_JPRB ,&
&  0.26273E+00_JPRB, 0.26389E+00_JPRB, 0.26363E+00_JPRB, 0.26202E+00_JPRB, 0.25993E+00_JPRB, 0.25896E+00_JPRB, 0.25850E+00_JPRB ,&
&  0.25831E+00_JPRB, 0.25785E+00_JPRB, 0.25729E+00_JPRB, 0.25581E+00_JPRB, 0.24851E+00_JPRB, 0.24546E+00_JPRB, 0.24280E+00_JPRB ,&
&  0.24378E+00_JPRB, 0.24263E+00_JPRB, 0.23883E+00_JPRB, 0.24628E+00_JPRB, 0.25386E+00_JPRB, 0.24783E+00_JPRB, 0.24115E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23529E+00_JPRB, 0.23244E+00_JPRB, 0.22669E+00_JPRB, 0.22554E+00_JPRB, 0.22604E+00_JPRB, 0.22541E+00_JPRB ,&
&  0.22616E+00_JPRB, 0.22557E+00_JPRB, 0.22662E+00_JPRB, 0.22705E+00_JPRB, 0.22732E+00_JPRB, 0.22857E+00_JPRB, 0.22978E+00_JPRB ,&
&  0.23287E+00_JPRB, 0.23775E+00_JPRB, 0.24270E+00_JPRB, 0.24302E+00_JPRB, 0.24659E+00_JPRB, 0.24945E+00_JPRB, 0.25115E+00_JPRB ,&
&  0.25255E+00_JPRB, 0.25477E+00_JPRB, 0.25683E+00_JPRB, 0.25861E+00_JPRB, 0.26147E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.25119E+00_JPRB, 0.25388E+00_JPRB, 0.25710E+00_JPRB, 0.26024E+00_JPRB, 0.26242E+00_JPRB, 0.26461E+00_JPRB, 0.26657E+00_JPRB ,&
&  0.26832E+00_JPRB, 0.26958E+00_JPRB, 0.27047E+00_JPRB, 0.27106E+00_JPRB, 0.27152E+00_JPRB, 0.27140E+00_JPRB, 0.27086E+00_JPRB ,&
&  0.27029E+00_JPRB, 0.26914E+00_JPRB, 0.26750E+00_JPRB, 0.26621E+00_JPRB, 0.26544E+00_JPRB, 0.26430E+00_JPRB, 0.26259E+00_JPRB ,&
&  0.26076E+00_JPRB, 0.25901E+00_JPRB, 0.25745E+00_JPRB, 0.25574E+00_JPRB, 0.25544E+00_JPRB, 0.25604E+00_JPRB, 0.25658E+00_JPRB ,&
&  0.25042E+00_JPRB, 0.24485E+00_JPRB, 0.24722E+00_JPRB, 0.24828E+00_JPRB, 0.24396E+00_JPRB, 0.24528E+00_JPRB, 0.24594E+00_JPRB ,&
&  0.24683E+00_JPRB, 0.24822E+00_JPRB, 0.24973E+00_JPRB, 0.25038E+00_JPRB, 0.25083E+00_JPRB, 0.25166E+00_JPRB, 0.25114E+00_JPRB ,&
&  0.24903E+00_JPRB, 0.24730E+00_JPRB, 0.24579E+00_JPRB, 0.24421E+00_JPRB, 0.24395E+00_JPRB, 0.24618E+00_JPRB, 0.24765E+00_JPRB ,&
&  0.24670E+00_JPRB, 0.24504E+00_JPRB, 0.24346E+00_JPRB, 0.24404E+00_JPRB, 0.24507E+00_JPRB, 0.24565E+00_JPRB, 0.24301E+00_JPRB ,&
&  0.24229E+00_JPRB, 0.23913E+00_JPRB, 0.23639E+00_JPRB, 0.23364E+00_JPRB, 0.23258E+00_JPRB, 0.22844E+00_JPRB, 0.22387E+00_JPRB ,&
&  0.22309E+00_JPRB, 0.22813E+00_JPRB, 0.23523E+00_JPRB, 0.23634E+00_JPRB, 0.21979E+00_JPRB, 0.22371E+00_JPRB, 0.22544E+00_JPRB ,&
&  0.22057E+00_JPRB, 0.21429E+00_JPRB, 0.20960E+00_JPRB, 0.20760E+00_JPRB, 0.20683E+00_JPRB, 0.20604E+00_JPRB, 0.20651E+00_JPRB ,&
&  0.20746E+00_JPRB, 0.20781E+00_JPRB, 0.20839E+00_JPRB, 0.20987E+00_JPRB, 0.21135E+00_JPRB, 0.21289E+00_JPRB, 0.21432E+00_JPRB ,&
&  0.21759E+00_JPRB, 0.22147E+00_JPRB, 0.22421E+00_JPRB, 0.22760E+00_JPRB, 0.23120E+00_JPRB, 0.23466E+00_JPRB, 0.23768E+00_JPRB ,&
&  0.24096E+00_JPRB, 0.24215E+00_JPRB, 0.24408E+00_JPRB, 0.24772E+00_JPRB, 0.24951E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.24386E+00_JPRB, 0.24711E+00_JPRB, 0.24920E+00_JPRB, 0.25124E+00_JPRB, 0.25284E+00_JPRB, 0.25448E+00_JPRB, 0.25523E+00_JPRB ,&
&  0.25593E+00_JPRB, 0.25594E+00_JPRB, 0.25647E+00_JPRB, 0.25651E+00_JPRB, 0.25679E+00_JPRB, 0.25553E+00_JPRB, 0.25510E+00_JPRB ,&
&  0.25468E+00_JPRB, 0.25531E+00_JPRB, 0.25552E+00_JPRB, 0.25556E+00_JPRB, 0.25484E+00_JPRB, 0.25380E+00_JPRB, 0.25203E+00_JPRB ,&
&  0.24829E+00_JPRB, 0.24529E+00_JPRB, 0.24522E+00_JPRB, 0.24435E+00_JPRB, 0.24472E+00_JPRB, 0.24456E+00_JPRB, 0.24507E+00_JPRB ,&
&  0.23972E+00_JPRB, 0.23829E+00_JPRB, 0.23956E+00_JPRB, 0.23810E+00_JPRB, 0.23913E+00_JPRB, 0.23967E+00_JPRB, 0.24165E+00_JPRB ,&
&  0.24191E+00_JPRB, 0.24200E+00_JPRB, 0.24285E+00_JPRB, 0.24210E+00_JPRB, 0.24234E+00_JPRB, 0.24297E+00_JPRB, 0.24175E+00_JPRB ,&
&  0.23808E+00_JPRB, 0.23551E+00_JPRB, 0.23167E+00_JPRB, 0.22268E+00_JPRB, 0.22148E+00_JPRB, 0.22269E+00_JPRB, 0.22167E+00_JPRB ,&
&  0.21874E+00_JPRB, 0.21989E+00_JPRB, 0.21943E+00_JPRB, 0.21744E+00_JPRB, 0.22001E+00_JPRB, 0.22368E+00_JPRB, 0.22383E+00_JPRB ,&
&  0.22216E+00_JPRB, 0.22173E+00_JPRB, 0.22093E+00_JPRB, 0.21936E+00_JPRB, 0.21747E+00_JPRB, 0.21675E+00_JPRB, 0.21606E+00_JPRB ,&
&  0.21559E+00_JPRB, 0.21360E+00_JPRB, 0.21739E+00_JPRB, 0.21770E+00_JPRB, 0.21140E+00_JPRB, 0.21552E+00_JPRB, 0.21323E+00_JPRB ,&
&  0.20440E+00_JPRB, 0.20059E+00_JPRB, 0.19830E+00_JPRB, 0.19630E+00_JPRB, 0.19583E+00_JPRB, 0.19575E+00_JPRB, 0.19533E+00_JPRB ,&
&  0.19624E+00_JPRB, 0.19754E+00_JPRB, 0.19811E+00_JPRB, 0.19898E+00_JPRB, 0.20022E+00_JPRB, 0.20209E+00_JPRB, 0.20460E+00_JPRB ,&
&  0.20683E+00_JPRB, 0.20984E+00_JPRB, 0.21126E+00_JPRB, 0.21360E+00_JPRB, 0.21531E+00_JPRB, 0.21728E+00_JPRB, 0.22217E+00_JPRB ,&
&  0.22585E+00_JPRB, 0.23309E+00_JPRB, 0.24264E+00_JPRB, 0.24461E+00_JPRB, 0.24060E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.23867E+00_JPRB, 0.24112E+00_JPRB, 0.24326E+00_JPRB, 0.24507E+00_JPRB, 0.24698E+00_JPRB, 0.24855E+00_JPRB, 0.24936E+00_JPRB ,&
&  0.24948E+00_JPRB, 0.24865E+00_JPRB, 0.24861E+00_JPRB, 0.24671E+00_JPRB, 0.24489E+00_JPRB, 0.24045E+00_JPRB, 0.24034E+00_JPRB ,&
&  0.23944E+00_JPRB, 0.24045E+00_JPRB, 0.23996E+00_JPRB, 0.24034E+00_JPRB, 0.23937E+00_JPRB, 0.23879E+00_JPRB, 0.23645E+00_JPRB ,&
&  0.23310E+00_JPRB, 0.23075E+00_JPRB, 0.22858E+00_JPRB, 0.22813E+00_JPRB, 0.22913E+00_JPRB, 0.22975E+00_JPRB, 0.22925E+00_JPRB ,&
&  0.22931E+00_JPRB, 0.22951E+00_JPRB, 0.22987E+00_JPRB, 0.22958E+00_JPRB, 0.23534E+00_JPRB, 0.23788E+00_JPRB, 0.23913E+00_JPRB ,&
&  0.23866E+00_JPRB, 0.23864E+00_JPRB, 0.23854E+00_JPRB, 0.23719E+00_JPRB, 0.23590E+00_JPRB, 0.23040E+00_JPRB, 0.22710E+00_JPRB ,&
&  0.21890E+00_JPRB, 0.21291E+00_JPRB, 0.20843E+00_JPRB, 0.20567E+00_JPRB, 0.20462E+00_JPRB, 0.20488E+00_JPRB, 0.20461E+00_JPRB ,&
&  0.20286E+00_JPRB, 0.20119E+00_JPRB, 0.20014E+00_JPRB, 0.20114E+00_JPRB, 0.20274E+00_JPRB, 0.20444E+00_JPRB, 0.20637E+00_JPRB ,&
&  0.20801E+00_JPRB, 0.20847E+00_JPRB, 0.20842E+00_JPRB, 0.20903E+00_JPRB, 0.20951E+00_JPRB, 0.21023E+00_JPRB, 0.21069E+00_JPRB ,&
&  0.21196E+00_JPRB, 0.21070E+00_JPRB, 0.21149E+00_JPRB, 0.21311E+00_JPRB, 0.21008E+00_JPRB, 0.20535E+00_JPRB, 0.20076E+00_JPRB ,&
&  0.19623E+00_JPRB, 0.19370E+00_JPRB, 0.19157E+00_JPRB, 0.19015E+00_JPRB, 0.18976E+00_JPRB, 0.18972E+00_JPRB, 0.19024E+00_JPRB ,&
&  0.19096E+00_JPRB, 0.19210E+00_JPRB, 0.19280E+00_JPRB, 0.19386E+00_JPRB, 0.19517E+00_JPRB, 0.19669E+00_JPRB, 0.19850E+00_JPRB ,&
&  0.20076E+00_JPRB, 0.20285E+00_JPRB, 0.20436E+00_JPRB, 0.20641E+00_JPRB, 0.21025E+00_JPRB, 0.21518E+00_JPRB, 0.22251E+00_JPRB ,&
&  0.23025E+00_JPRB, 0.23526E+00_JPRB, 0.23037E+00_JPRB, 0.23435E+00_JPRB, 0.23733E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.23863E+00_JPRB, 0.24074E+00_JPRB, 0.24238E+00_JPRB, 0.24410E+00_JPRB, 0.24545E+00_JPRB, 0.24666E+00_JPRB, 0.24744E+00_JPRB ,&
&  0.24427E+00_JPRB, 0.24267E+00_JPRB, 0.23963E+00_JPRB, 0.23724E+00_JPRB, 0.23419E+00_JPRB, 0.23067E+00_JPRB, 0.22967E+00_JPRB ,&
&  0.22772E+00_JPRB, 0.22738E+00_JPRB, 0.22562E+00_JPRB, 0.22486E+00_JPRB, 0.22332E+00_JPRB, 0.22227E+00_JPRB, 0.22050E+00_JPRB ,&
&  0.21743E+00_JPRB, 0.21597E+00_JPRB, 0.21461E+00_JPRB, 0.21488E+00_JPRB, 0.21657E+00_JPRB, 0.21830E+00_JPRB, 0.22018E+00_JPRB ,&
&  0.22296E+00_JPRB, 0.22392E+00_JPRB, 0.22658E+00_JPRB, 0.22865E+00_JPRB, 0.23256E+00_JPRB, 0.23587E+00_JPRB, 0.23668E+00_JPRB ,&
&  0.23657E+00_JPRB, 0.23364E+00_JPRB, 0.23241E+00_JPRB, 0.22928E+00_JPRB, 0.22224E+00_JPRB, 0.21869E+00_JPRB, 0.21331E+00_JPRB ,&
&  0.20734E+00_JPRB, 0.20291E+00_JPRB, 0.19980E+00_JPRB, 0.19895E+00_JPRB, 0.19789E+00_JPRB, 0.19769E+00_JPRB, 0.19768E+00_JPRB ,&
&  0.19684E+00_JPRB, 0.19585E+00_JPRB, 0.19463E+00_JPRB, 0.19403E+00_JPRB, 0.19468E+00_JPRB, 0.19580E+00_JPRB, 0.19813E+00_JPRB ,&
&  0.20039E+00_JPRB, 0.20173E+00_JPRB, 0.20314E+00_JPRB, 0.20364E+00_JPRB, 0.20379E+00_JPRB, 0.20409E+00_JPRB, 0.20415E+00_JPRB ,&
&  0.20506E+00_JPRB, 0.20527E+00_JPRB, 0.20462E+00_JPRB, 0.20356E+00_JPRB, 0.20071E+00_JPRB, 0.19693E+00_JPRB, 0.19405E+00_JPRB ,&
&  0.19153E+00_JPRB, 0.18989E+00_JPRB, 0.18932E+00_JPRB, 0.18868E+00_JPRB, 0.18896E+00_JPRB, 0.18939E+00_JPRB, 0.18974E+00_JPRB ,&
&  0.19038E+00_JPRB, 0.19097E+00_JPRB, 0.19154E+00_JPRB, 0.19303E+00_JPRB, 0.19427E+00_JPRB, 0.19615E+00_JPRB, 0.19812E+00_JPRB ,&
&  0.19948E+00_JPRB, 0.20152E+00_JPRB, 0.20328E+00_JPRB, 0.20520E+00_JPRB, 0.21109E+00_JPRB, 0.21557E+00_JPRB, 0.22473E+00_JPRB ,&
&  0.23004E+00_JPRB, 0.23021E+00_JPRB, 0.22847E+00_JPRB, 0.23586E+00_JPRB, 0.23733E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.23838E+00_JPRB, 0.23940E+00_JPRB, 0.23948E+00_JPRB, 0.24063E+00_JPRB, 0.24130E+00_JPRB, 0.24202E+00_JPRB, 0.24063E+00_JPRB ,&
&  0.23951E+00_JPRB, 0.23823E+00_JPRB, 0.23718E+00_JPRB, 0.23372E+00_JPRB, 0.23105E+00_JPRB, 0.22766E+00_JPRB, 0.22474E+00_JPRB ,&
&  0.22210E+00_JPRB, 0.21957E+00_JPRB, 0.21717E+00_JPRB, 0.21510E+00_JPRB, 0.21311E+00_JPRB, 0.21128E+00_JPRB, 0.20945E+00_JPRB ,&
&  0.20674E+00_JPRB, 0.20598E+00_JPRB, 0.20593E+00_JPRB, 0.20740E+00_JPRB, 0.21055E+00_JPRB, 0.21238E+00_JPRB, 0.21462E+00_JPRB ,&
&  0.21671E+00_JPRB, 0.21541E+00_JPRB, 0.21919E+00_JPRB, 0.22276E+00_JPRB, 0.22575E+00_JPRB, 0.22835E+00_JPRB, 0.23075E+00_JPRB ,&
&  0.23133E+00_JPRB, 0.22430E+00_JPRB, 0.22513E+00_JPRB, 0.21924E+00_JPRB, 0.21376E+00_JPRB, 0.21182E+00_JPRB, 0.20749E+00_JPRB ,&
&  0.20610E+00_JPRB, 0.20268E+00_JPRB, 0.20048E+00_JPRB, 0.19848E+00_JPRB, 0.19591E+00_JPRB, 0.19540E+00_JPRB, 0.19449E+00_JPRB ,&
&  0.19406E+00_JPRB, 0.19437E+00_JPRB, 0.19352E+00_JPRB, 0.19337E+00_JPRB, 0.19295E+00_JPRB, 0.19262E+00_JPRB, 0.19336E+00_JPRB ,&
&  0.19440E+00_JPRB, 0.19553E+00_JPRB, 0.19656E+00_JPRB, 0.19673E+00_JPRB, 0.19662E+00_JPRB, 0.19659E+00_JPRB, 0.19649E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19694E+00_JPRB, 0.19647E+00_JPRB, 0.19592E+00_JPRB, 0.19458E+00_JPRB, 0.19319E+00_JPRB, 0.19183E+00_JPRB ,&
&  0.19059E+00_JPRB, 0.18992E+00_JPRB, 0.18933E+00_JPRB, 0.18913E+00_JPRB, 0.18910E+00_JPRB, 0.18946E+00_JPRB, 0.18983E+00_JPRB ,&
&  0.19018E+00_JPRB, 0.19109E+00_JPRB, 0.19175E+00_JPRB, 0.19287E+00_JPRB, 0.19421E+00_JPRB, 0.19541E+00_JPRB, 0.19724E+00_JPRB ,&
&  0.19923E+00_JPRB, 0.20139E+00_JPRB, 0.20454E+00_JPRB, 0.20720E+00_JPRB, 0.21060E+00_JPRB, 0.21487E+00_JPRB, 0.21786E+00_JPRB ,&
&  0.22101E+00_JPRB, 0.22350E+00_JPRB, 0.22296E+00_JPRB, 0.23166E+00_JPRB, 0.23649E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.22836E+00_JPRB, 0.22941E+00_JPRB, 0.22994E+00_JPRB, 0.23035E+00_JPRB, 0.23161E+00_JPRB, 0.23156E+00_JPRB, 0.23127E+00_JPRB ,&
&  0.22872E+00_JPRB, 0.22787E+00_JPRB, 0.22639E+00_JPRB, 0.22423E+00_JPRB, 0.22196E+00_JPRB, 0.21800E+00_JPRB, 0.21573E+00_JPRB ,&
&  0.21357E+00_JPRB, 0.21126E+00_JPRB, 0.20985E+00_JPRB, 0.20786E+00_JPRB, 0.20615E+00_JPRB, 0.20412E+00_JPRB, 0.20349E+00_JPRB ,&
&  0.20241E+00_JPRB, 0.20230E+00_JPRB, 0.20326E+00_JPRB, 0.20392E+00_JPRB, 0.20527E+00_JPRB, 0.20668E+00_JPRB, 0.20853E+00_JPRB ,&
&  0.20872E+00_JPRB, 0.20943E+00_JPRB, 0.21022E+00_JPRB, 0.21080E+00_JPRB, 0.21307E+00_JPRB, 0.21466E+00_JPRB, 0.21527E+00_JPRB ,&
&  0.21477E+00_JPRB, 0.21365E+00_JPRB, 0.21153E+00_JPRB, 0.20907E+00_JPRB, 0.20637E+00_JPRB, 0.20351E+00_JPRB, 0.20077E+00_JPRB ,&
&  0.19872E+00_JPRB, 0.19679E+00_JPRB, 0.19532E+00_JPRB, 0.19415E+00_JPRB, 0.19347E+00_JPRB, 0.19294E+00_JPRB, 0.19240E+00_JPRB ,&
&  0.19215E+00_JPRB, 0.19197E+00_JPRB, 0.19163E+00_JPRB, 0.19176E+00_JPRB, 0.19174E+00_JPRB, 0.19192E+00_JPRB, 0.19213E+00_JPRB ,&
&  0.19250E+00_JPRB, 0.19264E+00_JPRB, 0.19265E+00_JPRB, 0.19238E+00_JPRB, 0.19209E+00_JPRB, 0.19174E+00_JPRB, 0.19138E+00_JPRB ,&
&  0.19101E+00_JPRB, 0.19104E+00_JPRB, 0.19071E+00_JPRB, 0.19048E+00_JPRB, 0.19018E+00_JPRB, 0.18958E+00_JPRB, 0.18939E+00_JPRB ,&
&  0.18946E+00_JPRB, 0.18959E+00_JPRB, 0.18934E+00_JPRB, 0.18941E+00_JPRB, 0.18960E+00_JPRB, 0.18992E+00_JPRB, 0.19092E+00_JPRB ,&
&  0.19151E+00_JPRB, 0.19212E+00_JPRB, 0.19267E+00_JPRB, 0.19348E+00_JPRB, 0.19493E+00_JPRB, 0.19639E+00_JPRB, 0.19802E+00_JPRB ,&
&  0.19999E+00_JPRB, 0.20181E+00_JPRB, 0.20363E+00_JPRB, 0.20542E+00_JPRB, 0.20845E+00_JPRB, 0.21048E+00_JPRB, 0.21315E+00_JPRB ,&
&  0.21539E+00_JPRB, 0.21328E+00_JPRB, 0.22262E+00_JPRB, 0.22314E+00_JPRB, 0.22437E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.21055E+00_JPRB, 0.21031E+00_JPRB, 0.20896E+00_JPRB, 0.20744E+00_JPRB, 0.19860E+00_JPRB, 0.20552E+00_JPRB, 0.20490E+00_JPRB ,&
&  0.20575E+00_JPRB, 0.20665E+00_JPRB, 0.20865E+00_JPRB, 0.20917E+00_JPRB, 0.20907E+00_JPRB, 0.20917E+00_JPRB, 0.20904E+00_JPRB ,&
&  0.20881E+00_JPRB, 0.20799E+00_JPRB, 0.20765E+00_JPRB, 0.20704E+00_JPRB, 0.20639E+00_JPRB, 0.20521E+00_JPRB, 0.20393E+00_JPRB ,&
&  0.20334E+00_JPRB, 0.20340E+00_JPRB, 0.20383E+00_JPRB, 0.20159E+00_JPRB, 0.20051E+00_JPRB, 0.19978E+00_JPRB, 0.19938E+00_JPRB ,&
&  0.19876E+00_JPRB, 0.19854E+00_JPRB, 0.19853E+00_JPRB, 0.19846E+00_JPRB, 0.20019E+00_JPRB, 0.20075E+00_JPRB, 0.20118E+00_JPRB ,&
&  0.20130E+00_JPRB, 0.20119E+00_JPRB, 0.20043E+00_JPRB, 0.19971E+00_JPRB, 0.19898E+00_JPRB, 0.19766E+00_JPRB, 0.19628E+00_JPRB ,&
&  0.19512E+00_JPRB, 0.19425E+00_JPRB, 0.19322E+00_JPRB, 0.19245E+00_JPRB, 0.19183E+00_JPRB, 0.19124E+00_JPRB, 0.19087E+00_JPRB ,&
&  0.19080E+00_JPRB, 0.19085E+00_JPRB, 0.19088E+00_JPRB, 0.19092E+00_JPRB, 0.19092E+00_JPRB, 0.19096E+00_JPRB, 0.19104E+00_JPRB ,&
&  0.19156E+00_JPRB, 0.19161E+00_JPRB, 0.19146E+00_JPRB, 0.19115E+00_JPRB, 0.19108E+00_JPRB, 0.19074E+00_JPRB, 0.19052E+00_JPRB ,&
&  0.19033E+00_JPRB, 0.19010E+00_JPRB, 0.19000E+00_JPRB, 0.18999E+00_JPRB, 0.19002E+00_JPRB, 0.19002E+00_JPRB, 0.19014E+00_JPRB ,&
&  0.19033E+00_JPRB, 0.19057E+00_JPRB, 0.19101E+00_JPRB, 0.19135E+00_JPRB, 0.19166E+00_JPRB, 0.19192E+00_JPRB, 0.19251E+00_JPRB ,&
&  0.19313E+00_JPRB, 0.19373E+00_JPRB, 0.19433E+00_JPRB, 0.19470E+00_JPRB, 0.19560E+00_JPRB, 0.19650E+00_JPRB, 0.19749E+00_JPRB ,&
&  0.19857E+00_JPRB, 0.19992E+00_JPRB, 0.20135E+00_JPRB, 0.20278E+00_JPRB, 0.20471E+00_JPRB, 0.20623E+00_JPRB, 0.20773E+00_JPRB ,&
&  0.20940E+00_JPRB, 0.21252E+00_JPRB, 0.21320E+00_JPRB, 0.21313E+00_JPRB, 0.21218E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.20558E+00_JPRB, 0.20550E+00_JPRB, 0.20528E+00_JPRB, 0.20473E+00_JPRB, 0.20375E+00_JPRB, 0.20351E+00_JPRB, 0.20319E+00_JPRB ,&
&  0.20277E+00_JPRB, 0.20190E+00_JPRB, 0.20189E+00_JPRB, 0.20178E+00_JPRB, 0.20165E+00_JPRB, 0.20149E+00_JPRB, 0.20116E+00_JPRB ,&
&  0.20050E+00_JPRB, 0.19949E+00_JPRB, 0.19655E+00_JPRB, 0.19513E+00_JPRB, 0.19421E+00_JPRB, 0.19368E+00_JPRB, 0.19325E+00_JPRB ,&
&  0.19298E+00_JPRB, 0.19283E+00_JPRB, 0.19268E+00_JPRB, 0.19320E+00_JPRB, 0.19343E+00_JPRB, 0.19351E+00_JPRB, 0.19357E+00_JPRB ,&
&  0.19356E+00_JPRB, 0.19367E+00_JPRB, 0.19368E+00_JPRB, 0.19367E+00_JPRB, 0.19382E+00_JPRB, 0.19383E+00_JPRB, 0.19361E+00_JPRB ,&
&  0.19336E+00_JPRB, 0.19306E+00_JPRB, 0.19278E+00_JPRB, 0.19241E+00_JPRB, 0.19215E+00_JPRB, 0.19194E+00_JPRB, 0.19187E+00_JPRB ,&
&  0.19169E+00_JPRB, 0.19155E+00_JPRB, 0.19141E+00_JPRB, 0.19136E+00_JPRB, 0.19125E+00_JPRB, 0.19121E+00_JPRB, 0.19106E+00_JPRB ,&
&  0.19118E+00_JPRB, 0.19123E+00_JPRB, 0.19134E+00_JPRB, 0.19140E+00_JPRB, 0.19146E+00_JPRB, 0.19146E+00_JPRB, 0.19141E+00_JPRB ,&
&  0.19123E+00_JPRB, 0.19114E+00_JPRB, 0.19101E+00_JPRB, 0.19092E+00_JPRB, 0.19091E+00_JPRB, 0.19090E+00_JPRB, 0.19092E+00_JPRB ,&
&  0.19094E+00_JPRB, 0.19108E+00_JPRB, 0.19114E+00_JPRB, 0.19121E+00_JPRB, 0.19130E+00_JPRB, 0.19139E+00_JPRB, 0.19146E+00_JPRB ,&
&  0.19157E+00_JPRB, 0.19168E+00_JPRB, 0.19181E+00_JPRB, 0.19198E+00_JPRB, 0.19219E+00_JPRB, 0.19249E+00_JPRB, 0.19284E+00_JPRB ,&
&  0.19320E+00_JPRB, 0.19356E+00_JPRB, 0.19392E+00_JPRB, 0.19427E+00_JPRB, 0.19484E+00_JPRB, 0.19542E+00_JPRB, 0.19615E+00_JPRB ,&
&  0.19684E+00_JPRB, 0.19752E+00_JPRB, 0.19814E+00_JPRB, 0.19882E+00_JPRB, 0.19979E+00_JPRB, 0.20026E+00_JPRB, 0.20065E+00_JPRB ,&
&  0.20129E+00_JPRB, 0.20197E+00_JPRB, 0.20261E+00_JPRB, 0.20317E+00_JPRB, 0.20406E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB ,&
&  0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB, 0.19340E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB, 0.74260E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.74709E+00_JPRB, 0.74495E+00_JPRB, 0.74308E+00_JPRB, 0.74103E+00_JPRB, 0.73882E+00_JPRB, 0.73705E+00_JPRB, 0.73540E+00_JPRB ,&
&  0.73396E+00_JPRB, 0.73319E+00_JPRB, 0.73278E+00_JPRB, 0.73254E+00_JPRB, 0.73255E+00_JPRB, 0.73248E+00_JPRB, 0.73250E+00_JPRB ,&
&  0.73242E+00_JPRB, 0.73247E+00_JPRB, 0.73246E+00_JPRB, 0.73208E+00_JPRB, 0.73142E+00_JPRB, 0.73057E+00_JPRB, 0.72947E+00_JPRB ,&
&  0.72789E+00_JPRB, 0.72621E+00_JPRB, 0.72462E+00_JPRB, 0.72290E+00_JPRB, 0.72134E+00_JPRB, 0.71960E+00_JPRB, 0.71774E+00_JPRB ,&
&  0.71608E+00_JPRB, 0.71512E+00_JPRB, 0.71435E+00_JPRB, 0.71357E+00_JPRB, 0.71320E+00_JPRB, 0.71403E+00_JPRB, 0.71475E+00_JPRB ,&
&  0.71555E+00_JPRB, 0.71653E+00_JPRB, 0.71856E+00_JPRB, 0.72057E+00_JPRB, 0.72246E+00_JPRB, 0.72436E+00_JPRB, 0.72622E+00_JPRB ,&
&  0.72707E+00_JPRB, 0.72742E+00_JPRB, 0.72746E+00_JPRB, 0.72753E+00_JPRB, 0.72709E+00_JPRB, 0.72687E+00_JPRB, 0.72670E+00_JPRB ,&
&  0.72641E+00_JPRB, 0.72615E+00_JPRB, 0.72574E+00_JPRB, 0.72466E+00_JPRB, 0.72201E+00_JPRB, 0.72012E+00_JPRB, 0.71933E+00_JPRB ,&
&  0.71939E+00_JPRB, 0.71963E+00_JPRB, 0.72029E+00_JPRB, 0.72133E+00_JPRB, 0.72307E+00_JPRB, 0.72497E+00_JPRB, 0.72720E+00_JPRB ,&
&  0.72964E+00_JPRB, 0.73264E+00_JPRB, 0.73652E+00_JPRB, 0.74061E+00_JPRB, 0.74515E+00_JPRB, 0.75046E+00_JPRB, 0.75531E+00_JPRB ,&
&  0.76031E+00_JPRB, 0.76558E+00_JPRB, 0.77009E+00_JPRB, 0.77442E+00_JPRB, 0.77870E+00_JPRB, 0.78311E+00_JPRB, 0.78647E+00_JPRB ,&
&  0.78815E+00_JPRB, 0.78941E+00_JPRB, 0.79034E+00_JPRB, 0.78983E+00_JPRB, 0.78829E+00_JPRB, 0.78707E+00_JPRB, 0.78577E+00_JPRB ,&
&  0.78410E+00_JPRB, 0.78190E+00_JPRB, 0.77982E+00_JPRB, 0.77748E+00_JPRB, 0.77419E+00_JPRB, 0.77054E+00_JPRB, 0.76709E+00_JPRB ,&
&  0.76366E+00_JPRB, 0.75989E+00_JPRB, 0.75642E+00_JPRB, 0.75312E+00_JPRB, 0.75004E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.77231E+00_JPRB, 0.76407E+00_JPRB, 0.75662E+00_JPRB, 0.74982E+00_JPRB, 0.74428E+00_JPRB, 0.74003E+00_JPRB, 0.73647E+00_JPRB ,&
&  0.73344E+00_JPRB, 0.73198E+00_JPRB, 0.73048E+00_JPRB, 0.72905E+00_JPRB, 0.72765E+00_JPRB, 0.72570E+00_JPRB, 0.72336E+00_JPRB ,&
&  0.72120E+00_JPRB, 0.71978E+00_JPRB, 0.71833E+00_JPRB, 0.71651E+00_JPRB, 0.71339E+00_JPRB, 0.70878E+00_JPRB, 0.70341E+00_JPRB ,&
&  0.69728E+00_JPRB, 0.68990E+00_JPRB, 0.68146E+00_JPRB, 0.67271E+00_JPRB, 0.66818E+00_JPRB, 0.66293E+00_JPRB, 0.65790E+00_JPRB ,&
&  0.65353E+00_JPRB, 0.65221E+00_JPRB, 0.65121E+00_JPRB, 0.65046E+00_JPRB, 0.65041E+00_JPRB, 0.65208E+00_JPRB, 0.65306E+00_JPRB ,&
&  0.65390E+00_JPRB, 0.65619E+00_JPRB, 0.66242E+00_JPRB, 0.66911E+00_JPRB, 0.67688E+00_JPRB, 0.68630E+00_JPRB, 0.69707E+00_JPRB ,&
&  0.70514E+00_JPRB, 0.70978E+00_JPRB, 0.71405E+00_JPRB, 0.71804E+00_JPRB, 0.72042E+00_JPRB, 0.72187E+00_JPRB, 0.72258E+00_JPRB ,&
&  0.72475E+00_JPRB, 0.72613E+00_JPRB, 0.72746E+00_JPRB, 0.72844E+00_JPRB, 0.72743E+00_JPRB, 0.72643E+00_JPRB, 0.72526E+00_JPRB ,&
&  0.72315E+00_JPRB, 0.72263E+00_JPRB, 0.72372E+00_JPRB, 0.72500E+00_JPRB, 0.72794E+00_JPRB, 0.73258E+00_JPRB, 0.73853E+00_JPRB ,&
&  0.74698E+00_JPRB, 0.75868E+00_JPRB, 0.77310E+00_JPRB, 0.78917E+00_JPRB, 0.80772E+00_JPRB, 0.83087E+00_JPRB, 0.84950E+00_JPRB ,&
&  0.86641E+00_JPRB, 0.88089E+00_JPRB, 0.89391E+00_JPRB, 0.90337E+00_JPRB, 0.91033E+00_JPRB, 0.91704E+00_JPRB, 0.91744E+00_JPRB ,&
&  0.91631E+00_JPRB, 0.91442E+00_JPRB, 0.91303E+00_JPRB, 0.90857E+00_JPRB, 0.90296E+00_JPRB, 0.89712E+00_JPRB, 0.89159E+00_JPRB ,&
&  0.88314E+00_JPRB, 0.87444E+00_JPRB, 0.86585E+00_JPRB, 0.85748E+00_JPRB, 0.84791E+00_JPRB, 0.83859E+00_JPRB, 0.82936E+00_JPRB ,&
&  0.82008E+00_JPRB, 0.81009E+00_JPRB, 0.80052E+00_JPRB, 0.79121E+00_JPRB, 0.78188E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.80230E+00_JPRB, 0.78815E+00_JPRB, 0.77528E+00_JPRB, 0.76392E+00_JPRB, 0.75408E+00_JPRB, 0.74637E+00_JPRB, 0.74023E+00_JPRB ,&
&  0.73537E+00_JPRB, 0.73265E+00_JPRB, 0.72991E+00_JPRB, 0.72678E+00_JPRB, 0.72350E+00_JPRB, 0.72282E+00_JPRB, 0.72032E+00_JPRB ,&
&  0.71664E+00_JPRB, 0.71182E+00_JPRB, 0.70710E+00_JPRB, 0.70042E+00_JPRB, 0.69107E+00_JPRB, 0.68015E+00_JPRB, 0.66680E+00_JPRB ,&
&  0.65820E+00_JPRB, 0.64698E+00_JPRB, 0.63498E+00_JPRB, 0.62245E+00_JPRB, 0.61366E+00_JPRB, 0.59387E+00_JPRB, 0.58674E+00_JPRB ,&
&  0.58199E+00_JPRB, 0.56892E+00_JPRB, 0.56231E+00_JPRB, 0.55967E+00_JPRB, 0.57387E+00_JPRB, 0.59035E+00_JPRB, 0.59920E+00_JPRB ,&
&  0.58803E+00_JPRB, 0.59437E+00_JPRB, 0.59152E+00_JPRB, 0.58905E+00_JPRB, 0.59052E+00_JPRB, 0.59562E+00_JPRB, 0.61925E+00_JPRB ,&
&  0.66177E+00_JPRB, 0.67834E+00_JPRB, 0.68776E+00_JPRB, 0.69659E+00_JPRB, 0.70188E+00_JPRB, 0.70701E+00_JPRB, 0.71029E+00_JPRB ,&
&  0.71412E+00_JPRB, 0.71728E+00_JPRB, 0.72240E+00_JPRB, 0.72502E+00_JPRB, 0.72918E+00_JPRB, 0.73341E+00_JPRB, 0.73739E+00_JPRB ,&
&  0.73484E+00_JPRB, 0.73382E+00_JPRB, 0.73449E+00_JPRB, 0.73779E+00_JPRB, 0.74369E+00_JPRB, 0.75599E+00_JPRB, 0.77111E+00_JPRB ,&
&  0.78780E+00_JPRB, 0.80132E+00_JPRB, 0.82282E+00_JPRB, 0.85143E+00_JPRB, 0.88579E+00_JPRB, 0.92786E+00_JPRB, 0.97016E+00_JPRB ,&
&  0.10120E+01_JPRB, 0.10467E+01_JPRB, 0.10804E+01_JPRB, 0.10979E+01_JPRB, 0.11077E+01_JPRB, 0.11146E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.11080E+01_JPRB, 0.11018E+01_JPRB, 0.10929E+01_JPRB, 0.10806E+01_JPRB, 0.10642E+01_JPRB, 0.10453E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.10010E+01_JPRB, 0.97937E+00_JPRB, 0.95813E+00_JPRB, 0.93692E+00_JPRB, 0.91736E+00_JPRB, 0.89981E+00_JPRB ,&
&  0.88311E+00_JPRB, 0.86597E+00_JPRB, 0.84989E+00_JPRB, 0.83420E+00_JPRB, 0.81841E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.83131E+00_JPRB, 0.81575E+00_JPRB, 0.79921E+00_JPRB, 0.78413E+00_JPRB, 0.77140E+00_JPRB, 0.76162E+00_JPRB, 0.75467E+00_JPRB ,&
&  0.74883E+00_JPRB, 0.74189E+00_JPRB, 0.73465E+00_JPRB, 0.72697E+00_JPRB, 0.72014E+00_JPRB, 0.71720E+00_JPRB, 0.71474E+00_JPRB ,&
&  0.71276E+00_JPRB, 0.70682E+00_JPRB, 0.69881E+00_JPRB, 0.68665E+00_JPRB, 0.67367E+00_JPRB, 0.66196E+00_JPRB, 0.64839E+00_JPRB ,&
&  0.63698E+00_JPRB, 0.62503E+00_JPRB, 0.60649E+00_JPRB, 0.59514E+00_JPRB, 0.57151E+00_JPRB, 0.55598E+00_JPRB, 0.54638E+00_JPRB ,&
&  0.53971E+00_JPRB, 0.54118E+00_JPRB, 0.54582E+00_JPRB, 0.55324E+00_JPRB, 0.55499E+00_JPRB, 0.55132E+00_JPRB, 0.55040E+00_JPRB ,&
&  0.54375E+00_JPRB, 0.53917E+00_JPRB, 0.53598E+00_JPRB, 0.53525E+00_JPRB, 0.53788E+00_JPRB, 0.54525E+00_JPRB, 0.56080E+00_JPRB ,&
&  0.59508E+00_JPRB, 0.63919E+00_JPRB, 0.67967E+00_JPRB, 0.69947E+00_JPRB, 0.69761E+00_JPRB, 0.70286E+00_JPRB, 0.70210E+00_JPRB ,&
&  0.70384E+00_JPRB, 0.70351E+00_JPRB, 0.70744E+00_JPRB, 0.71498E+00_JPRB, 0.72397E+00_JPRB, 0.73242E+00_JPRB, 0.74174E+00_JPRB ,&
&  0.74579E+00_JPRB, 0.75004E+00_JPRB, 0.74781E+00_JPRB, 0.75306E+00_JPRB, 0.76639E+00_JPRB, 0.78325E+00_JPRB, 0.79225E+00_JPRB ,&
&  0.80662E+00_JPRB, 0.82581E+00_JPRB, 0.86089E+00_JPRB, 0.91335E+00_JPRB, 0.96977E+00_JPRB, 0.10219E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.11767E+01_JPRB, 0.12331E+01_JPRB, 0.12572E+01_JPRB, 0.12580E+01_JPRB, 0.12645E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12400E+01_JPRB, 0.12351E+01_JPRB, 0.12302E+01_JPRB, 0.12132E+01_JPRB, 0.11860E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.11254E+01_JPRB, 0.10909E+01_JPRB, 0.10581E+01_JPRB, 0.10248E+01_JPRB, 0.99344E+00_JPRB, 0.96287E+00_JPRB, 0.93709E+00_JPRB ,&
&  0.91213E+00_JPRB, 0.89262E+00_JPRB, 0.87454E+00_JPRB, 0.85977E+00_JPRB, 0.84575E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.85818E+00_JPRB, 0.84800E+00_JPRB, 0.83759E+00_JPRB, 0.82386E+00_JPRB, 0.80981E+00_JPRB, 0.79683E+00_JPRB, 0.78649E+00_JPRB ,&
&  0.77778E+00_JPRB, 0.76851E+00_JPRB, 0.75946E+00_JPRB, 0.74859E+00_JPRB, 0.73832E+00_JPRB, 0.72904E+00_JPRB, 0.72144E+00_JPRB ,&
&  0.71530E+00_JPRB, 0.70530E+00_JPRB, 0.69551E+00_JPRB, 0.68319E+00_JPRB, 0.67157E+00_JPRB, 0.65829E+00_JPRB, 0.64335E+00_JPRB ,&
&  0.63185E+00_JPRB, 0.61428E+00_JPRB, 0.59787E+00_JPRB, 0.58803E+00_JPRB, 0.56565E+00_JPRB, 0.55467E+00_JPRB, 0.54952E+00_JPRB ,&
&  0.56497E+00_JPRB, 0.56115E+00_JPRB, 0.57232E+00_JPRB, 0.55675E+00_JPRB, 0.54470E+00_JPRB, 0.53352E+00_JPRB, 0.52274E+00_JPRB ,&
&  0.51552E+00_JPRB, 0.50829E+00_JPRB, 0.50866E+00_JPRB, 0.51284E+00_JPRB, 0.51569E+00_JPRB, 0.52618E+00_JPRB, 0.54102E+00_JPRB ,&
&  0.57682E+00_JPRB, 0.67177E+00_JPRB, 0.68890E+00_JPRB, 0.70807E+00_JPRB, 0.70929E+00_JPRB, 0.71281E+00_JPRB, 0.71519E+00_JPRB ,&
&  0.71744E+00_JPRB, 0.71768E+00_JPRB, 0.71689E+00_JPRB, 0.71785E+00_JPRB, 0.73208E+00_JPRB, 0.74266E+00_JPRB, 0.75600E+00_JPRB ,&
&  0.76181E+00_JPRB, 0.76934E+00_JPRB, 0.77203E+00_JPRB, 0.77929E+00_JPRB, 0.79917E+00_JPRB, 0.81146E+00_JPRB, 0.81463E+00_JPRB ,&
&  0.83613E+00_JPRB, 0.86650E+00_JPRB, 0.91361E+00_JPRB, 0.97090E+00_JPRB, 0.10140E+01_JPRB, 0.10505E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.11441E+01_JPRB, 0.12080E+01_JPRB, 0.12768E+01_JPRB, 0.13086E+01_JPRB, 0.13148E+01_JPRB, 0.13077E+01_JPRB, 0.12970E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.12765E+01_JPRB, 0.12662E+01_JPRB, 0.12610E+01_JPRB, 0.12395E+01_JPRB, 0.12124E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11158E+01_JPRB, 0.10894E+01_JPRB, 0.10592E+01_JPRB, 0.10275E+01_JPRB, 0.99658E+00_JPRB, 0.96644E+00_JPRB ,&
&  0.93758E+00_JPRB, 0.91417E+00_JPRB, 0.89241E+00_JPRB, 0.87843E+00_JPRB, 0.86593E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.87965E+00_JPRB, 0.86996E+00_JPRB, 0.86090E+00_JPRB, 0.85198E+00_JPRB, 0.84178E+00_JPRB, 0.83124E+00_JPRB, 0.82036E+00_JPRB ,&
&  0.81075E+00_JPRB, 0.80175E+00_JPRB, 0.79212E+00_JPRB, 0.77983E+00_JPRB, 0.76703E+00_JPRB, 0.75276E+00_JPRB, 0.74133E+00_JPRB ,&
&  0.72936E+00_JPRB, 0.71638E+00_JPRB, 0.69791E+00_JPRB, 0.68084E+00_JPRB, 0.66894E+00_JPRB, 0.65798E+00_JPRB, 0.64435E+00_JPRB ,&
&  0.63246E+00_JPRB, 0.61637E+00_JPRB, 0.59570E+00_JPRB, 0.57787E+00_JPRB, 0.55746E+00_JPRB, 0.55395E+00_JPRB, 0.58495E+00_JPRB ,&
&  0.60042E+00_JPRB, 0.60142E+00_JPRB, 0.59893E+00_JPRB, 0.59083E+00_JPRB, 0.58216E+00_JPRB, 0.53523E+00_JPRB, 0.53842E+00_JPRB ,&
&  0.52094E+00_JPRB, 0.50675E+00_JPRB, 0.50481E+00_JPRB, 0.50556E+00_JPRB, 0.51167E+00_JPRB, 0.52668E+00_JPRB, 0.54470E+00_JPRB ,&
&  0.57941E+00_JPRB, 0.68698E+00_JPRB, 0.70612E+00_JPRB, 0.72425E+00_JPRB, 0.72853E+00_JPRB, 0.73214E+00_JPRB, 0.73706E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.74419E+00_JPRB, 0.74940E+00_JPRB, 0.75743E+00_JPRB, 0.76719E+00_JPRB, 0.77110E+00_JPRB, 0.78578E+00_JPRB ,&
&  0.79678E+00_JPRB, 0.80735E+00_JPRB, 0.81353E+00_JPRB, 0.81271E+00_JPRB, 0.82328E+00_JPRB, 0.82096E+00_JPRB, 0.82189E+00_JPRB ,&
&  0.86328E+00_JPRB, 0.92408E+00_JPRB, 0.97802E+00_JPRB, 0.10278E+01_JPRB, 0.10718E+01_JPRB, 0.11207E+01_JPRB, 0.11804E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.12822E+01_JPRB, 0.13547E+01_JPRB, 0.13797E+01_JPRB, 0.13796E+01_JPRB, 0.13765E+01_JPRB, 0.13812E+01_JPRB ,&
&  0.13864E+01_JPRB, 0.13946E+01_JPRB, 0.13568E+01_JPRB, 0.13343E+01_JPRB, 0.12858E+01_JPRB, 0.12231E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.10534E+01_JPRB, 0.10202E+01_JPRB, 0.99631E+00_JPRB, 0.97849E+00_JPRB, 0.96618E+00_JPRB, 0.95492E+00_JPRB ,&
&  0.94451E+00_JPRB, 0.93098E+00_JPRB, 0.91645E+00_JPRB, 0.90199E+00_JPRB, 0.88963E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.88596E+00_JPRB, 0.87934E+00_JPRB, 0.86983E+00_JPRB, 0.86149E+00_JPRB, 0.85039E+00_JPRB, 0.84208E+00_JPRB, 0.83387E+00_JPRB ,&
&  0.82850E+00_JPRB, 0.82519E+00_JPRB, 0.82122E+00_JPRB, 0.81703E+00_JPRB, 0.80998E+00_JPRB, 0.80138E+00_JPRB, 0.78968E+00_JPRB ,&
&  0.77241E+00_JPRB, 0.73168E+00_JPRB, 0.70206E+00_JPRB, 0.68840E+00_JPRB, 0.67507E+00_JPRB, 0.66413E+00_JPRB, 0.65375E+00_JPRB ,&
&  0.63797E+00_JPRB, 0.62353E+00_JPRB, 0.60591E+00_JPRB, 0.58161E+00_JPRB, 0.56991E+00_JPRB, 0.57837E+00_JPRB, 0.58473E+00_JPRB ,&
&  0.59414E+00_JPRB, 0.60915E+00_JPRB, 0.60603E+00_JPRB, 0.59842E+00_JPRB, 0.60389E+00_JPRB, 0.60298E+00_JPRB, 0.55987E+00_JPRB ,&
&  0.53096E+00_JPRB, 0.50948E+00_JPRB, 0.50504E+00_JPRB, 0.50481E+00_JPRB, 0.51177E+00_JPRB, 0.52883E+00_JPRB, 0.57245E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.74147E+00_JPRB, 0.73757E+00_JPRB, 0.73818E+00_JPRB, 0.74528E+00_JPRB, 0.75141E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.76569E+00_JPRB, 0.77820E+00_JPRB, 0.79460E+00_JPRB, 0.81067E+00_JPRB, 0.83077E+00_JPRB, 0.84606E+00_JPRB, 0.85706E+00_JPRB ,&
&  0.87782E+00_JPRB, 0.86433E+00_JPRB, 0.86211E+00_JPRB, 0.85348E+00_JPRB, 0.84855E+00_JPRB, 0.83460E+00_JPRB, 0.85496E+00_JPRB ,&
&  0.90716E+00_JPRB, 0.96168E+00_JPRB, 0.10206E+01_JPRB, 0.10691E+01_JPRB, 0.11206E+01_JPRB, 0.11894E+01_JPRB, 0.12655E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.14243E+01_JPRB, 0.15537E+01_JPRB, 0.15373E+01_JPRB, 0.15128E+01_JPRB, 0.14995E+01_JPRB, 0.15052E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.15241E+01_JPRB, 0.14573E+01_JPRB, 0.13578E+01_JPRB, 0.12685E+01_JPRB, 0.11644E+01_JPRB, 0.11024E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.99243E+00_JPRB, 0.95976E+00_JPRB, 0.94173E+00_JPRB, 0.93029E+00_JPRB, 0.91691E+00_JPRB, 0.90517E+00_JPRB ,&
&  0.90021E+00_JPRB, 0.89778E+00_JPRB, 0.89785E+00_JPRB, 0.89445E+00_JPRB, 0.89077E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.87059E+00_JPRB, 0.87396E+00_JPRB, 0.87326E+00_JPRB, 0.86623E+00_JPRB, 0.84440E+00_JPRB, 0.81042E+00_JPRB, 0.78911E+00_JPRB ,&
&  0.77582E+00_JPRB, 0.77327E+00_JPRB, 0.77006E+00_JPRB, 0.76610E+00_JPRB, 0.76730E+00_JPRB, 0.77413E+00_JPRB, 0.78081E+00_JPRB ,&
&  0.78182E+00_JPRB, 0.77150E+00_JPRB, 0.75181E+00_JPRB, 0.72045E+00_JPRB, 0.69505E+00_JPRB, 0.67591E+00_JPRB, 0.66494E+00_JPRB ,&
&  0.65746E+00_JPRB, 0.64624E+00_JPRB, 0.63152E+00_JPRB, 0.60834E+00_JPRB, 0.59196E+00_JPRB, 0.57889E+00_JPRB, 0.58457E+00_JPRB ,&
&  0.60166E+00_JPRB, 0.60732E+00_JPRB, 0.60627E+00_JPRB, 0.61742E+00_JPRB, 0.62252E+00_JPRB, 0.63556E+00_JPRB, 0.58428E+00_JPRB ,&
&  0.54757E+00_JPRB, 0.53145E+00_JPRB, 0.51942E+00_JPRB, 0.51626E+00_JPRB, 0.52676E+00_JPRB, 0.55156E+00_JPRB, 0.60332E+00_JPRB ,&
&  0.71809E+00_JPRB, 0.74324E+00_JPRB, 0.75254E+00_JPRB, 0.76678E+00_JPRB, 0.77790E+00_JPRB, 0.78249E+00_JPRB, 0.79191E+00_JPRB ,&
&  0.81096E+00_JPRB, 0.84348E+00_JPRB, 0.87692E+00_JPRB, 0.88172E+00_JPRB, 0.88400E+00_JPRB, 0.88506E+00_JPRB, 0.98920E+00_JPRB ,&
&  0.12708E+01_JPRB, 0.11568E+01_JPRB, 0.10135E+01_JPRB, 0.92715E+00_JPRB, 0.88742E+00_JPRB, 0.87220E+00_JPRB, 0.90158E+00_JPRB ,&
&  0.94965E+00_JPRB, 0.10044E+01_JPRB, 0.10807E+01_JPRB, 0.11407E+01_JPRB, 0.12025E+01_JPRB, 0.12874E+01_JPRB, 0.14052E+01_JPRB ,&
&  0.15250E+01_JPRB, 0.16859E+01_JPRB, 0.20354E+01_JPRB, 0.18041E+01_JPRB, 0.16707E+01_JPRB, 0.16254E+01_JPRB, 0.16105E+01_JPRB ,&
&  0.15617E+01_JPRB, 0.15650E+01_JPRB, 0.14612E+01_JPRB, 0.13181E+01_JPRB, 0.12423E+01_JPRB, 0.11586E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.98261E+00_JPRB, 0.95227E+00_JPRB, 0.92844E+00_JPRB, 0.92086E+00_JPRB, 0.91286E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.88925E+00_JPRB, 0.87814E+00_JPRB, 0.87056E+00_JPRB, 0.86542E+00_JPRB, 0.86467E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.83808E+00_JPRB, 0.86083E+00_JPRB, 0.88022E+00_JPRB, 0.86084E+00_JPRB, 0.82974E+00_JPRB, 0.76951E+00_JPRB, 0.72370E+00_JPRB ,&
&  0.69173E+00_JPRB, 0.68313E+00_JPRB, 0.67962E+00_JPRB, 0.68138E+00_JPRB, 0.68875E+00_JPRB, 0.70163E+00_JPRB, 0.71028E+00_JPRB ,&
&  0.71931E+00_JPRB, 0.71871E+00_JPRB, 0.71802E+00_JPRB, 0.70844E+00_JPRB, 0.69103E+00_JPRB, 0.67038E+00_JPRB, 0.65602E+00_JPRB ,&
&  0.64776E+00_JPRB, 0.64270E+00_JPRB, 0.63494E+00_JPRB, 0.62268E+00_JPRB, 0.61398E+00_JPRB, 0.60197E+00_JPRB, 0.60484E+00_JPRB ,&
&  0.59724E+00_JPRB, 0.60052E+00_JPRB, 0.61606E+00_JPRB, 0.62587E+00_JPRB, 0.63921E+00_JPRB, 0.65381E+00_JPRB, 0.61573E+00_JPRB ,&
&  0.57352E+00_JPRB, 0.55369E+00_JPRB, 0.54776E+00_JPRB, 0.55159E+00_JPRB, 0.59445E+00_JPRB, 0.69954E+00_JPRB, 0.74375E+00_JPRB ,&
&  0.76716E+00_JPRB, 0.78296E+00_JPRB, 0.80049E+00_JPRB, 0.82276E+00_JPRB, 0.82935E+00_JPRB, 0.83520E+00_JPRB, 0.85439E+00_JPRB ,&
&  0.88732E+00_JPRB, 0.94094E+00_JPRB, 0.98620E+00_JPRB, 0.88126E+00_JPRB, 0.84011E+00_JPRB, 0.87627E+00_JPRB, 0.93732E+00_JPRB ,&
&  0.10205E+01_JPRB, 0.10086E+01_JPRB, 0.98466E+00_JPRB, 0.10096E+01_JPRB, 0.10147E+01_JPRB, 0.10206E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.10373E+01_JPRB, 0.10926E+01_JPRB, 0.12126E+01_JPRB, 0.12771E+01_JPRB, 0.13761E+01_JPRB, 0.15201E+01_JPRB, 0.16470E+01_JPRB ,&
&  0.17915E+01_JPRB, 0.18991E+01_JPRB, 0.19513E+01_JPRB, 0.18827E+01_JPRB, 0.18302E+01_JPRB, 0.18295E+01_JPRB, 0.17975E+01_JPRB ,&
&  0.16678E+01_JPRB, 0.15742E+01_JPRB, 0.14952E+01_JPRB, 0.14134E+01_JPRB, 0.12958E+01_JPRB, 0.11987E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.10029E+01_JPRB, 0.93255E+00_JPRB, 0.89610E+00_JPRB, 0.87195E+00_JPRB, 0.86410E+00_JPRB, 0.86053E+00_JPRB, 0.84823E+00_JPRB ,&
&  0.82980E+00_JPRB, 0.81022E+00_JPRB, 0.80129E+00_JPRB, 0.80734E+00_JPRB, 0.82902E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.88794E+00_JPRB, 0.88201E+00_JPRB, 0.84649E+00_JPRB, 0.85157E+00_JPRB, 0.82082E+00_JPRB, 0.79186E+00_JPRB, 0.74907E+00_JPRB ,&
&  0.72151E+00_JPRB, 0.68584E+00_JPRB, 0.66128E+00_JPRB, 0.62920E+00_JPRB, 0.61028E+00_JPRB, 0.60146E+00_JPRB, 0.61814E+00_JPRB ,&
&  0.64139E+00_JPRB, 0.65824E+00_JPRB, 0.66825E+00_JPRB, 0.67009E+00_JPRB, 0.66714E+00_JPRB, 0.65774E+00_JPRB, 0.64764E+00_JPRB ,&
&  0.63900E+00_JPRB, 0.63839E+00_JPRB, 0.63352E+00_JPRB, 0.62780E+00_JPRB, 0.62106E+00_JPRB, 0.61511E+00_JPRB, 0.61328E+00_JPRB ,&
&  0.60398E+00_JPRB, 0.59904E+00_JPRB, 0.60560E+00_JPRB, 0.62545E+00_JPRB, 0.64601E+00_JPRB, 0.66149E+00_JPRB, 0.63071E+00_JPRB ,&
&  0.59524E+00_JPRB, 0.58783E+00_JPRB, 0.60166E+00_JPRB, 0.69729E+00_JPRB, 0.72663E+00_JPRB, 0.74053E+00_JPRB, 0.76801E+00_JPRB ,&
&  0.75029E+00_JPRB, 0.74892E+00_JPRB, 0.76853E+00_JPRB, 0.83947E+00_JPRB, 0.86319E+00_JPRB, 0.87728E+00_JPRB, 0.90392E+00_JPRB ,&
&  0.94332E+00_JPRB, 0.10039E+01_JPRB, 0.10663E+01_JPRB, 0.97552E+00_JPRB, 0.10271E+01_JPRB, 0.11965E+01_JPRB, 0.11061E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.10682E+01_JPRB, 0.11039E+01_JPRB, 0.11363E+01_JPRB, 0.11317E+01_JPRB, 0.11660E+01_JPRB, 0.11983E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.12530E+01_JPRB, 0.13978E+01_JPRB, 0.15423E+01_JPRB, 0.16754E+01_JPRB, 0.18493E+01_JPRB, 0.20298E+01_JPRB ,&
&  0.22496E+01_JPRB, 0.24460E+01_JPRB, 0.25881E+01_JPRB, 0.25697E+01_JPRB, 0.25080E+01_JPRB, 0.23888E+01_JPRB, 0.22149E+01_JPRB ,&
&  0.19717E+01_JPRB, 0.17950E+01_JPRB, 0.16749E+01_JPRB, 0.15785E+01_JPRB, 0.14579E+01_JPRB, 0.13009E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.95217E+00_JPRB, 0.85605E+00_JPRB, 0.81702E+00_JPRB, 0.80591E+00_JPRB, 0.81000E+00_JPRB, 0.81398E+00_JPRB, 0.80676E+00_JPRB ,&
&  0.79550E+00_JPRB, 0.78784E+00_JPRB, 0.78450E+00_JPRB, 0.79909E+00_JPRB, 0.85681E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.91802E+00_JPRB, 0.89688E+00_JPRB, 0.87387E+00_JPRB, 0.86656E+00_JPRB, 0.86000E+00_JPRB, 0.81655E+00_JPRB, 0.74398E+00_JPRB ,&
&  0.75252E+00_JPRB, 0.70599E+00_JPRB, 0.69388E+00_JPRB, 0.67937E+00_JPRB, 0.65986E+00_JPRB, 0.62225E+00_JPRB, 0.59299E+00_JPRB ,&
&  0.59338E+00_JPRB, 0.61125E+00_JPRB, 0.63047E+00_JPRB, 0.64700E+00_JPRB, 0.65320E+00_JPRB, 0.64936E+00_JPRB, 0.64263E+00_JPRB ,&
&  0.63694E+00_JPRB, 0.63621E+00_JPRB, 0.63861E+00_JPRB, 0.64169E+00_JPRB, 0.63999E+00_JPRB, 0.63717E+00_JPRB, 0.63840E+00_JPRB ,&
&  0.63733E+00_JPRB, 0.63410E+00_JPRB, 0.62777E+00_JPRB, 0.63730E+00_JPRB, 0.64623E+00_JPRB, 0.66645E+00_JPRB, 0.68606E+00_JPRB ,&
&  0.62551E+00_JPRB, 0.63622E+00_JPRB, 0.72423E+00_JPRB, 0.73757E+00_JPRB, 0.75455E+00_JPRB, 0.77041E+00_JPRB, 0.78334E+00_JPRB ,&
&  0.80007E+00_JPRB, 0.82564E+00_JPRB, 0.85718E+00_JPRB, 0.88589E+00_JPRB, 0.90964E+00_JPRB, 0.93970E+00_JPRB, 0.96621E+00_JPRB ,&
&  0.10368E+01_JPRB, 0.96086E+00_JPRB, 0.92793E+00_JPRB, 0.99367E+00_JPRB, 0.13009E+01_JPRB, 0.13237E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.13492E+01_JPRB, 0.13081E+01_JPRB, 0.13063E+01_JPRB, 0.13512E+01_JPRB, 0.13768E+01_JPRB, 0.14486E+01_JPRB, 0.15549E+01_JPRB ,&
&  0.15662E+01_JPRB, 0.16795E+01_JPRB, 0.18256E+01_JPRB, 0.19563E+01_JPRB, 0.21013E+01_JPRB, 0.23160E+01_JPRB, 0.25559E+01_JPRB ,&
&  0.28084E+01_JPRB, 0.30259E+01_JPRB, 0.31263E+01_JPRB, 0.30845E+01_JPRB, 0.29457E+01_JPRB, 0.27626E+01_JPRB, 0.25333E+01_JPRB ,&
&  0.22363E+01_JPRB, 0.20238E+01_JPRB, 0.18792E+01_JPRB, 0.17373E+01_JPRB, 0.16073E+01_JPRB, 0.14360E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.93952E+00_JPRB, 0.83235E+00_JPRB, 0.81076E+00_JPRB, 0.80737E+00_JPRB, 0.81344E+00_JPRB, 0.82667E+00_JPRB, 0.83821E+00_JPRB ,&
&  0.84345E+00_JPRB, 0.82865E+00_JPRB, 0.82651E+00_JPRB, 0.85735E+00_JPRB, 0.89476E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.88448E+00_JPRB, 0.87415E+00_JPRB, 0.87450E+00_JPRB, 0.89034E+00_JPRB, 0.87902E+00_JPRB, 0.83466E+00_JPRB, 0.78610E+00_JPRB ,&
&  0.89668E+00_JPRB, 0.87138E+00_JPRB, 0.92519E+00_JPRB, 0.77494E+00_JPRB, 0.71988E+00_JPRB, 0.69578E+00_JPRB, 0.66167E+00_JPRB ,&
&  0.63464E+00_JPRB, 0.61932E+00_JPRB, 0.61409E+00_JPRB, 0.62684E+00_JPRB, 0.63350E+00_JPRB, 0.63072E+00_JPRB, 0.62717E+00_JPRB ,&
&  0.62485E+00_JPRB, 0.62807E+00_JPRB, 0.63768E+00_JPRB, 0.64256E+00_JPRB, 0.64606E+00_JPRB, 0.64969E+00_JPRB, 0.65645E+00_JPRB ,&
&  0.64072E+00_JPRB, 0.63702E+00_JPRB, 0.64190E+00_JPRB, 0.64428E+00_JPRB, 0.65629E+00_JPRB, 0.66466E+00_JPRB, 0.68514E+00_JPRB ,&
&  0.72932E+00_JPRB, 0.70327E+00_JPRB, 0.78322E+00_JPRB, 0.79139E+00_JPRB, 0.80320E+00_JPRB, 0.81467E+00_JPRB, 0.82432E+00_JPRB ,&
&  0.83316E+00_JPRB, 0.85098E+00_JPRB, 0.87842E+00_JPRB, 0.90261E+00_JPRB, 0.93790E+00_JPRB, 0.98156E+00_JPRB, 0.10164E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.96464E+00_JPRB, 0.10450E+01_JPRB, 0.10966E+01_JPRB, 0.13204E+01_JPRB, 0.15794E+01_JPRB, 0.15542E+01_JPRB ,&
&  0.18257E+01_JPRB, 0.17255E+01_JPRB, 0.16520E+01_JPRB, 0.15879E+01_JPRB, 0.16396E+01_JPRB, 0.17356E+01_JPRB, 0.18547E+01_JPRB ,&
&  0.19679E+01_JPRB, 0.20877E+01_JPRB, 0.22833E+01_JPRB, 0.24562E+01_JPRB, 0.26365E+01_JPRB, 0.28436E+01_JPRB, 0.30389E+01_JPRB ,&
&  0.32123E+01_JPRB, 0.33217E+01_JPRB, 0.33860E+01_JPRB, 0.32035E+01_JPRB, 0.30398E+01_JPRB, 0.28642E+01_JPRB, 0.25186E+01_JPRB ,&
&  0.22531E+01_JPRB, 0.20499E+01_JPRB, 0.18805E+01_JPRB, 0.17551E+01_JPRB, 0.15334E+01_JPRB, 0.13492E+01_JPRB, 0.11749E+01_JPRB ,&
&  0.94992E+00_JPRB, 0.87000E+00_JPRB, 0.87652E+00_JPRB, 0.88704E+00_JPRB, 0.88359E+00_JPRB, 0.88988E+00_JPRB, 0.89575E+00_JPRB ,&
&  0.87989E+00_JPRB, 0.87617E+00_JPRB, 0.90506E+00_JPRB, 0.90426E+00_JPRB, 0.89516E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.92880E+00_JPRB, 0.94009E+00_JPRB, 0.94567E+00_JPRB, 0.93921E+00_JPRB, 0.91660E+00_JPRB, 0.93076E+00_JPRB, 0.88412E+00_JPRB ,&
&  0.95717E+00_JPRB, 0.97728E+00_JPRB, 0.96665E+00_JPRB, 0.97278E+00_JPRB, 0.99630E+00_JPRB, 0.92326E+00_JPRB, 0.77233E+00_JPRB ,&
&  0.70065E+00_JPRB, 0.68930E+00_JPRB, 0.68444E+00_JPRB, 0.67900E+00_JPRB, 0.67729E+00_JPRB, 0.67212E+00_JPRB, 0.66895E+00_JPRB ,&
&  0.66649E+00_JPRB, 0.66302E+00_JPRB, 0.66728E+00_JPRB, 0.67122E+00_JPRB, 0.67065E+00_JPRB, 0.67126E+00_JPRB, 0.67877E+00_JPRB ,&
&  0.65750E+00_JPRB, 0.64438E+00_JPRB, 0.63620E+00_JPRB, 0.62548E+00_JPRB, 0.64742E+00_JPRB, 0.66602E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.74037E+00_JPRB, 0.78439E+00_JPRB, 0.80905E+00_JPRB, 0.82974E+00_JPRB, 0.84643E+00_JPRB, 0.86240E+00_JPRB, 0.87805E+00_JPRB ,&
&  0.89238E+00_JPRB, 0.91013E+00_JPRB, 0.93631E+00_JPRB, 0.95627E+00_JPRB, 0.10146E+01_JPRB, 0.10104E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.13365E+01_JPRB, 0.14319E+01_JPRB, 0.13143E+01_JPRB, 0.14953E+01_JPRB, 0.17210E+01_JPRB, 0.17478E+01_JPRB ,&
&  0.18336E+01_JPRB, 0.18448E+01_JPRB, 0.19175E+01_JPRB, 0.20139E+01_JPRB, 0.22545E+01_JPRB, 0.24341E+01_JPRB, 0.25245E+01_JPRB ,&
&  0.26138E+01_JPRB, 0.29154E+01_JPRB, 0.31253E+01_JPRB, 0.32598E+01_JPRB, 0.33208E+01_JPRB, 0.33733E+01_JPRB, 0.34555E+01_JPRB ,&
&  0.35157E+01_JPRB, 0.35720E+01_JPRB, 0.35588E+01_JPRB, 0.34445E+01_JPRB, 0.32674E+01_JPRB, 0.30227E+01_JPRB, 0.26910E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.20261E+01_JPRB, 0.17247E+01_JPRB, 0.15389E+01_JPRB, 0.13650E+01_JPRB, 0.11939E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.96066E+00_JPRB, 0.95039E+00_JPRB, 0.10219E+01_JPRB, 0.10233E+01_JPRB, 0.10210E+01_JPRB, 0.10032E+01_JPRB, 0.93111E+00_JPRB ,&
&  0.91533E+00_JPRB, 0.94210E+00_JPRB, 0.95391E+00_JPRB, 0.94442E+00_JPRB, 0.93183E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.10538E+01_JPRB, 0.10482E+01_JPRB, 0.10391E+01_JPRB, 0.10210E+01_JPRB, 0.10134E+01_JPRB, 0.10147E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.10385E+01_JPRB, 0.10352E+01_JPRB, 0.10160E+01_JPRB, 0.99790E+00_JPRB, 0.99513E+00_JPRB, 0.90523E+00_JPRB ,&
&  0.78911E+00_JPRB, 0.74961E+00_JPRB, 0.73757E+00_JPRB, 0.74173E+00_JPRB, 0.75539E+00_JPRB, 0.76142E+00_JPRB, 0.76372E+00_JPRB ,&
&  0.76765E+00_JPRB, 0.75794E+00_JPRB, 0.76093E+00_JPRB, 0.75965E+00_JPRB, 0.75113E+00_JPRB, 0.74492E+00_JPRB, 0.73900E+00_JPRB ,&
&  0.69895E+00_JPRB, 0.67112E+00_JPRB, 0.65629E+00_JPRB, 0.64768E+00_JPRB, 0.66368E+00_JPRB, 0.69351E+00_JPRB, 0.73640E+00_JPRB ,&
&  0.78757E+00_JPRB, 0.82687E+00_JPRB, 0.86191E+00_JPRB, 0.89494E+00_JPRB, 0.91885E+00_JPRB, 0.94295E+00_JPRB, 0.96486E+00_JPRB ,&
&  0.98256E+00_JPRB, 0.99929E+00_JPRB, 0.10100E+01_JPRB, 0.10285E+01_JPRB, 0.10322E+01_JPRB, 0.11392E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.14376E+01_JPRB, 0.15548E+01_JPRB, 0.16507E+01_JPRB, 0.18810E+01_JPRB, 0.20214E+01_JPRB, 0.20005E+01_JPRB, 0.20613E+01_JPRB ,&
&  0.22642E+01_JPRB, 0.24959E+01_JPRB, 0.29740E+01_JPRB, 0.30166E+01_JPRB, 0.31097E+01_JPRB, 0.32338E+01_JPRB, 0.32260E+01_JPRB ,&
&  0.33360E+01_JPRB, 0.34714E+01_JPRB, 0.36640E+01_JPRB, 0.36814E+01_JPRB, 0.36284E+01_JPRB, 0.36607E+01_JPRB, 0.36554E+01_JPRB ,&
&  0.36973E+01_JPRB, 0.37427E+01_JPRB, 0.35840E+01_JPRB, 0.34358E+01_JPRB, 0.31373E+01_JPRB, 0.31311E+01_JPRB, 0.29041E+01_JPRB ,&
&  0.24409E+01_JPRB, 0.18241E+01_JPRB, 0.16508E+01_JPRB, 0.14587E+01_JPRB, 0.13280E+01_JPRB, 0.11823E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.10958E+01_JPRB, 0.11101E+01_JPRB, 0.11224E+01_JPRB, 0.11348E+01_JPRB, 0.11195E+01_JPRB, 0.10137E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.10522E+01_JPRB, 0.10628E+01_JPRB, 0.10609E+01_JPRB, 0.10565E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.12312E+01_JPRB, 0.12012E+01_JPRB, 0.11756E+01_JPRB, 0.11587E+01_JPRB, 0.11381E+01_JPRB, 0.11107E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.11045E+01_JPRB, 0.11061E+01_JPRB, 0.10724E+01_JPRB, 0.10363E+01_JPRB, 0.10129E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.89597E+00_JPRB, 0.86810E+00_JPRB, 0.86544E+00_JPRB, 0.89270E+00_JPRB, 0.97221E+00_JPRB, 0.99393E+00_JPRB, 0.10264E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.99254E+00_JPRB, 0.98042E+00_JPRB, 0.95931E+00_JPRB, 0.92509E+00_JPRB, 0.89884E+00_JPRB, 0.86476E+00_JPRB ,&
&  0.79299E+00_JPRB, 0.73884E+00_JPRB, 0.71903E+00_JPRB, 0.70970E+00_JPRB, 0.73440E+00_JPRB, 0.76459E+00_JPRB, 0.82072E+00_JPRB ,&
&  0.87278E+00_JPRB, 0.92182E+00_JPRB, 0.96728E+00_JPRB, 0.10178E+01_JPRB, 0.10560E+01_JPRB, 0.10926E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11285E+01_JPRB, 0.11321E+01_JPRB, 0.11463E+01_JPRB, 0.11326E+01_JPRB, 0.11468E+01_JPRB, 0.14167E+01_JPRB, 0.17665E+01_JPRB ,&
&  0.19006E+01_JPRB, 0.20504E+01_JPRB, 0.26318E+01_JPRB, 0.34717E+01_JPRB, 0.32956E+01_JPRB, 0.29925E+01_JPRB, 0.29141E+01_JPRB ,&
&  0.31349E+01_JPRB, 0.34429E+01_JPRB, 0.37758E+01_JPRB, 0.39055E+01_JPRB, 0.39021E+01_JPRB, 0.40239E+01_JPRB, 0.40629E+01_JPRB ,&
&  0.40508E+01_JPRB, 0.41009E+01_JPRB, 0.40472E+01_JPRB, 0.39333E+01_JPRB, 0.38261E+01_JPRB, 0.39024E+01_JPRB, 0.38976E+01_JPRB ,&
&  0.38569E+01_JPRB, 0.37182E+01_JPRB, 0.34504E+01_JPRB, 0.33014E+01_JPRB, 0.27788E+01_JPRB, 0.26920E+01_JPRB, 0.30065E+01_JPRB ,&
&  0.24868E+01_JPRB, 0.19631E+01_JPRB, 0.16794E+01_JPRB, 0.15063E+01_JPRB, 0.14334E+01_JPRB, 0.14262E+01_JPRB, 0.13567E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.12376E+01_JPRB, 0.13455E+01_JPRB, 0.14032E+01_JPRB, 0.14193E+01_JPRB, 0.14126E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.12721E+01_JPRB, 0.12703E+01_JPRB, 0.12691E+01_JPRB, 0.12556E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.14151E+01_JPRB, 0.13518E+01_JPRB, 0.13143E+01_JPRB, 0.12622E+01_JPRB, 0.12044E+01_JPRB, 0.11548E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11559E+01_JPRB, 0.11433E+01_JPRB, 0.11024E+01_JPRB, 0.10711E+01_JPRB, 0.10497E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.99584E+00_JPRB, 0.98122E+00_JPRB, 0.98540E+00_JPRB, 0.11461E+01_JPRB, 0.13016E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.14774E+01_JPRB, 0.14958E+01_JPRB, 0.14690E+01_JPRB, 0.13753E+01_JPRB, 0.12767E+01_JPRB, 0.11869E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.99308E+00_JPRB, 0.90298E+00_JPRB, 0.85793E+00_JPRB, 0.85507E+00_JPRB, 0.90143E+00_JPRB, 0.87627E+00_JPRB, 0.93732E+00_JPRB ,&
&  0.99771E+00_JPRB, 0.10636E+01_JPRB, 0.11153E+01_JPRB, 0.11795E+01_JPRB, 0.12216E+01_JPRB, 0.12540E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.12585E+01_JPRB, 0.11549E+01_JPRB, 0.12026E+01_JPRB, 0.13548E+01_JPRB, 0.19020E+01_JPRB, 0.24960E+01_JPRB ,&
&  0.28997E+01_JPRB, 0.35383E+01_JPRB, 0.43482E+01_JPRB, 0.49162E+01_JPRB, 0.44522E+01_JPRB, 0.40772E+01_JPRB, 0.41159E+01_JPRB ,&
&  0.43850E+01_JPRB, 0.46833E+01_JPRB, 0.48097E+01_JPRB, 0.46202E+01_JPRB, 0.46887E+01_JPRB, 0.48134E+01_JPRB, 0.49125E+01_JPRB ,&
&  0.48881E+01_JPRB, 0.47127E+01_JPRB, 0.44699E+01_JPRB, 0.42405E+01_JPRB, 0.40918E+01_JPRB, 0.38040E+01_JPRB, 0.37321E+01_JPRB ,&
&  0.38179E+01_JPRB, 0.32595E+01_JPRB, 0.29471E+01_JPRB, 0.30701E+01_JPRB, 0.25384E+01_JPRB, 0.24806E+01_JPRB, 0.26648E+01_JPRB ,&
&  0.22754E+01_JPRB, 0.19070E+01_JPRB, 0.17205E+01_JPRB, 0.15575E+01_JPRB, 0.15376E+01_JPRB, 0.17397E+01_JPRB, 0.18747E+01_JPRB ,&
&  0.17750E+01_JPRB, 0.16899E+01_JPRB, 0.19320E+01_JPRB, 0.20879E+01_JPRB, 0.21754E+01_JPRB, 0.20880E+01_JPRB, 0.19383E+01_JPRB ,&
&  0.18126E+01_JPRB, 0.17262E+01_JPRB, 0.16579E+01_JPRB, 0.15831E+01_JPRB, 0.15043E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.15512E+01_JPRB, 0.15178E+01_JPRB, 0.14522E+01_JPRB, 0.13593E+01_JPRB, 0.12821E+01_JPRB, 0.12294E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.11280E+01_JPRB, 0.11119E+01_JPRB, 0.10824E+01_JPRB, 0.10721E+01_JPRB, 0.10746E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.10771E+01_JPRB, 0.10764E+01_JPRB, 0.10339E+01_JPRB, 0.10499E+01_JPRB, 0.12213E+01_JPRB, 0.17121E+01_JPRB, 0.20699E+01_JPRB ,&
&  0.21997E+01_JPRB, 0.22074E+01_JPRB, 0.21198E+01_JPRB, 0.20282E+01_JPRB, 0.18762E+01_JPRB, 0.16614E+01_JPRB, 0.15197E+01_JPRB ,&
&  0.13837E+01_JPRB, 0.12476E+01_JPRB, 0.11132E+01_JPRB, 0.11129E+01_JPRB, 0.11226E+01_JPRB, 0.10727E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.12679E+01_JPRB, 0.13029E+01_JPRB, 0.13357E+01_JPRB, 0.13496E+01_JPRB, 0.12826E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.11804E+01_JPRB, 0.12174E+01_JPRB, 0.12652E+01_JPRB, 0.14656E+01_JPRB, 0.20020E+01_JPRB, 0.29432E+01_JPRB, 0.37797E+01_JPRB ,&
&  0.43166E+01_JPRB, 0.45886E+01_JPRB, 0.43367E+01_JPRB, 0.47932E+01_JPRB, 0.55290E+01_JPRB, 0.53733E+01_JPRB, 0.51158E+01_JPRB ,&
&  0.51300E+01_JPRB, 0.51266E+01_JPRB, 0.50521E+01_JPRB, 0.51541E+01_JPRB, 0.55904E+01_JPRB, 0.58257E+01_JPRB, 0.58368E+01_JPRB ,&
&  0.51086E+01_JPRB, 0.48325E+01_JPRB, 0.46921E+01_JPRB, 0.44963E+01_JPRB, 0.40072E+01_JPRB, 0.38210E+01_JPRB, 0.39063E+01_JPRB ,&
&  0.38134E+01_JPRB, 0.37532E+01_JPRB, 0.29911E+01_JPRB, 0.28145E+01_JPRB, 0.21872E+01_JPRB, 0.21117E+01_JPRB, 0.23695E+01_JPRB ,&
&  0.21716E+01_JPRB, 0.19899E+01_JPRB, 0.17578E+01_JPRB, 0.16898E+01_JPRB, 0.20573E+01_JPRB, 0.24815E+01_JPRB, 0.30449E+01_JPRB ,&
&  0.30209E+01_JPRB, 0.25844E+01_JPRB, 0.28029E+01_JPRB, 0.29165E+01_JPRB, 0.30502E+01_JPRB, 0.28769E+01_JPRB, 0.25562E+01_JPRB ,&
&  0.22940E+01_JPRB, 0.20929E+01_JPRB, 0.18983E+01_JPRB, 0.17217E+01_JPRB, 0.16109E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.16062E+01_JPRB, 0.15600E+01_JPRB, 0.14746E+01_JPRB, 0.13712E+01_JPRB, 0.12961E+01_JPRB, 0.12387E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.11050E+01_JPRB, 0.10826E+01_JPRB, 0.10677E+01_JPRB, 0.10618E+01_JPRB, 0.10611E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10958E+01_JPRB, 0.10695E+01_JPRB, 0.11058E+01_JPRB, 0.13830E+01_JPRB, 0.19943E+01_JPRB, 0.25866E+01_JPRB ,&
&  0.28498E+01_JPRB, 0.27866E+01_JPRB, 0.26666E+01_JPRB, 0.25416E+01_JPRB, 0.24422E+01_JPRB, 0.22923E+01_JPRB, 0.20966E+01_JPRB ,&
&  0.18948E+01_JPRB, 0.15400E+01_JPRB, 0.14473E+01_JPRB, 0.13667E+01_JPRB, 0.13990E+01_JPRB, 0.14171E+01_JPRB, 0.13932E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.13377E+01_JPRB, 0.13266E+01_JPRB, 0.12816E+01_JPRB, 0.12534E+01_JPRB, 0.11966E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.13175E+01_JPRB, 0.14147E+01_JPRB, 0.17608E+01_JPRB, 0.23677E+01_JPRB, 0.34225E+01_JPRB, 0.48835E+01_JPRB, 0.51089E+01_JPRB ,&
&  0.42413E+01_JPRB, 0.41017E+01_JPRB, 0.56860E+01_JPRB, 0.54861E+01_JPRB, 0.57173E+01_JPRB, 0.64187E+01_JPRB, 0.60409E+01_JPRB ,&
&  0.54491E+01_JPRB, 0.50710E+01_JPRB, 0.49740E+01_JPRB, 0.53457E+01_JPRB, 0.71350E+01_JPRB, 0.74463E+01_JPRB, 0.69564E+01_JPRB ,&
&  0.58031E+01_JPRB, 0.54949E+01_JPRB, 0.50318E+01_JPRB, 0.45232E+01_JPRB, 0.42279E+01_JPRB, 0.44380E+01_JPRB, 0.42022E+01_JPRB ,&
&  0.39045E+01_JPRB, 0.41991E+01_JPRB, 0.33415E+01_JPRB, 0.25662E+01_JPRB, 0.21429E+01_JPRB, 0.21917E+01_JPRB, 0.23334E+01_JPRB ,&
&  0.23041E+01_JPRB, 0.21959E+01_JPRB, 0.21283E+01_JPRB, 0.25506E+01_JPRB, 0.35471E+01_JPRB, 0.43417E+01_JPRB, 0.42934E+01_JPRB ,&
&  0.41561E+01_JPRB, 0.44769E+01_JPRB, 0.42002E+01_JPRB, 0.41644E+01_JPRB, 0.41112E+01_JPRB, 0.36008E+01_JPRB, 0.29363E+01_JPRB ,&
&  0.23769E+01_JPRB, 0.20235E+01_JPRB, 0.18293E+01_JPRB, 0.17151E+01_JPRB, 0.16462E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.16889E+01_JPRB, 0.15938E+01_JPRB, 0.14954E+01_JPRB, 0.13995E+01_JPRB, 0.13107E+01_JPRB, 0.12385E+01_JPRB, 0.11761E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.10729E+01_JPRB, 0.10656E+01_JPRB, 0.10759E+01_JPRB, 0.10846E+01_JPRB, 0.10656E+01_JPRB, 0.10691E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.11262E+01_JPRB, 0.11138E+01_JPRB, 0.12298E+01_JPRB, 0.15123E+01_JPRB, 0.19494E+01_JPRB, 0.25924E+01_JPRB ,&
&  0.30459E+01_JPRB, 0.29141E+01_JPRB, 0.26711E+01_JPRB, 0.27205E+01_JPRB, 0.28290E+01_JPRB, 0.28245E+01_JPRB, 0.26563E+01_JPRB ,&
&  0.22761E+01_JPRB, 0.20183E+01_JPRB, 0.19192E+01_JPRB, 0.17450E+01_JPRB, 0.16565E+01_JPRB, 0.15523E+01_JPRB, 0.14201E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.12642E+01_JPRB, 0.12397E+01_JPRB, 0.12298E+01_JPRB, 0.12710E+01_JPRB, 0.13308E+01_JPRB, 0.14200E+01_JPRB ,&
&  0.15920E+01_JPRB, 0.19116E+01_JPRB, 0.24723E+01_JPRB, 0.33594E+01_JPRB, 0.41023E+01_JPRB, 0.60868E+01_JPRB, 0.57219E+01_JPRB ,&
&  0.52978E+01_JPRB, 0.52712E+01_JPRB, 0.52611E+01_JPRB, 0.57928E+01_JPRB, 0.51322E+01_JPRB, 0.56487E+01_JPRB, 0.58883E+01_JPRB ,&
&  0.56265E+01_JPRB, 0.55050E+01_JPRB, 0.58590E+01_JPRB, 0.61220E+01_JPRB, 0.75214E+01_JPRB, 0.95765E+01_JPRB, 0.87451E+01_JPRB ,&
&  0.71359E+01_JPRB, 0.62820E+01_JPRB, 0.53853E+01_JPRB, 0.48793E+01_JPRB, 0.41284E+01_JPRB, 0.41020E+01_JPRB, 0.35893E+01_JPRB ,&
&  0.32952E+01_JPRB, 0.26763E+01_JPRB, 0.28839E+01_JPRB, 0.24547E+01_JPRB, 0.23166E+01_JPRB, 0.22779E+01_JPRB, 0.23035E+01_JPRB ,&
&  0.24449E+01_JPRB, 0.26045E+01_JPRB, 0.31678E+01_JPRB, 0.42799E+01_JPRB, 0.57836E+01_JPRB, 0.52383E+01_JPRB, 0.55554E+01_JPRB ,&
&  0.56990E+01_JPRB, 0.56128E+01_JPRB, 0.47365E+01_JPRB, 0.44481E+01_JPRB, 0.39710E+01_JPRB, 0.35509E+01_JPRB, 0.30960E+01_JPRB ,&
&  0.26261E+01_JPRB, 0.22344E+01_JPRB, 0.20161E+01_JPRB, 0.18839E+01_JPRB, 0.17817E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.16379E+01_JPRB, 0.15256E+01_JPRB, 0.14374E+01_JPRB, 0.13551E+01_JPRB, 0.12778E+01_JPRB, 0.11931E+01_JPRB, 0.11061E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.10170E+01_JPRB, 0.10268E+01_JPRB, 0.10473E+01_JPRB, 0.10638E+01_JPRB, 0.10912E+01_JPRB, 0.11156E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.12119E+01_JPRB, 0.12065E+01_JPRB, 0.12848E+01_JPRB, 0.15506E+01_JPRB, 0.18851E+01_JPRB, 0.22731E+01_JPRB ,&
&  0.30349E+01_JPRB, 0.25958E+01_JPRB, 0.22773E+01_JPRB, 0.24757E+01_JPRB, 0.30050E+01_JPRB, 0.32997E+01_JPRB, 0.31086E+01_JPRB ,&
&  0.29905E+01_JPRB, 0.26690E+01_JPRB, 0.21927E+01_JPRB, 0.18323E+01_JPRB, 0.15518E+01_JPRB, 0.13538E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11079E+01_JPRB, 0.11224E+01_JPRB, 0.11477E+01_JPRB, 0.12258E+01_JPRB, 0.12806E+01_JPRB, 0.15401E+01_JPRB ,&
&  0.18261E+01_JPRB, 0.21838E+01_JPRB, 0.26755E+01_JPRB, 0.32184E+01_JPRB, 0.41066E+01_JPRB, 0.66684E+01_JPRB, 0.76299E+01_JPRB ,&
&  0.58072E+01_JPRB, 0.53594E+01_JPRB, 0.57220E+01_JPRB, 0.59634E+01_JPRB, 0.60574E+01_JPRB, 0.55891E+01_JPRB, 0.54635E+01_JPRB ,&
&  0.43816E+01_JPRB, 0.39910E+01_JPRB, 0.42933E+01_JPRB, 0.55631E+01_JPRB, 0.91033E+01_JPRB, 0.12097E+02_JPRB, 0.11112E+02_JPRB ,&
&  0.91656E+01_JPRB, 0.76660E+01_JPRB, 0.64136E+01_JPRB, 0.48507E+01_JPRB, 0.42542E+01_JPRB, 0.36667E+01_JPRB, 0.42399E+01_JPRB ,&
&  0.39891E+01_JPRB, 0.36097E+01_JPRB, 0.29777E+01_JPRB, 0.23006E+01_JPRB, 0.18798E+01_JPRB, 0.22283E+01_JPRB, 0.24308E+01_JPRB ,&
&  0.27891E+01_JPRB, 0.32365E+01_JPRB, 0.53484E+01_JPRB, 0.61872E+01_JPRB, 0.65689E+01_JPRB, 0.58398E+01_JPRB, 0.68921E+01_JPRB ,&
&  0.62972E+01_JPRB, 0.60576E+01_JPRB, 0.51211E+01_JPRB, 0.39443E+01_JPRB, 0.31614E+01_JPRB, 0.27078E+01_JPRB, 0.24709E+01_JPRB ,&
&  0.23737E+01_JPRB, 0.22772E+01_JPRB, 0.21189E+01_JPRB, 0.19482E+01_JPRB, 0.17993E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.14081E+01_JPRB, 0.13038E+01_JPRB, 0.12364E+01_JPRB, 0.11811E+01_JPRB, 0.11188E+01_JPRB, 0.10563E+01_JPRB, 0.99899E+00_JPRB ,&
&  0.96843E+00_JPRB, 0.96798E+00_JPRB, 0.99058E+00_JPRB, 0.10364E+01_JPRB, 0.10820E+01_JPRB, 0.11324E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.13346E+01_JPRB, 0.13664E+01_JPRB, 0.14602E+01_JPRB, 0.16995E+01_JPRB, 0.18637E+01_JPRB, 0.22538E+01_JPRB ,&
&  0.29061E+01_JPRB, 0.23149E+01_JPRB, 0.20583E+01_JPRB, 0.23194E+01_JPRB, 0.31406E+01_JPRB, 0.32465E+01_JPRB, 0.29932E+01_JPRB ,&
&  0.30516E+01_JPRB, 0.25854E+01_JPRB, 0.19268E+01_JPRB, 0.15673E+01_JPRB, 0.13402E+01_JPRB, 0.11657E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.11144E+01_JPRB, 0.11645E+01_JPRB, 0.12135E+01_JPRB, 0.12690E+01_JPRB, 0.14281E+01_JPRB, 0.17338E+01_JPRB ,&
&  0.20225E+01_JPRB, 0.23270E+01_JPRB, 0.25918E+01_JPRB, 0.29625E+01_JPRB, 0.40078E+01_JPRB, 0.67014E+01_JPRB, 0.96324E+01_JPRB ,&
&  0.78527E+01_JPRB, 0.72324E+01_JPRB, 0.68805E+01_JPRB, 0.68782E+01_JPRB, 0.59370E+01_JPRB, 0.54084E+01_JPRB, 0.51516E+01_JPRB ,&
&  0.41862E+01_JPRB, 0.42335E+01_JPRB, 0.55832E+01_JPRB, 0.79023E+01_JPRB, 0.98045E+01_JPRB, 0.11870E+02_JPRB, 0.13802E+02_JPRB ,&
&  0.10241E+02_JPRB, 0.82916E+01_JPRB, 0.70750E+01_JPRB, 0.60551E+01_JPRB, 0.35218E+01_JPRB, 0.25368E+01_JPRB, 0.35156E+01_JPRB ,&
&  0.32201E+01_JPRB, 0.18609E+01_JPRB, 0.14686E+01_JPRB, 0.18324E+01_JPRB, 0.13513E+01_JPRB, 0.16093E+01_JPRB, 0.25825E+01_JPRB ,&
&  0.32749E+01_JPRB, 0.41055E+01_JPRB, 0.74190E+01_JPRB, 0.68031E+01_JPRB, 0.70745E+01_JPRB, 0.86487E+01_JPRB, 0.83805E+01_JPRB ,&
&  0.76210E+01_JPRB, 0.47812E+01_JPRB, 0.33951E+01_JPRB, 0.25804E+01_JPRB, 0.21487E+01_JPRB, 0.20679E+01_JPRB, 0.20139E+01_JPRB ,&
&  0.19674E+01_JPRB, 0.18697E+01_JPRB, 0.17738E+01_JPRB, 0.16359E+01_JPRB, 0.15297E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.11235E+01_JPRB, 0.10533E+01_JPRB, 0.10340E+01_JPRB, 0.10140E+01_JPRB, 0.96568E+00_JPRB, 0.93562E+00_JPRB, 0.91304E+00_JPRB ,&
&  0.92146E+00_JPRB, 0.95391E+00_JPRB, 0.10017E+01_JPRB, 0.10696E+01_JPRB, 0.11517E+01_JPRB, 0.12280E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.14313E+01_JPRB, 0.16166E+01_JPRB, 0.18443E+01_JPRB, 0.21521E+01_JPRB, 0.19913E+01_JPRB, 0.27381E+01_JPRB ,&
&  0.27937E+01_JPRB, 0.21468E+01_JPRB, 0.20608E+01_JPRB, 0.23971E+01_JPRB, 0.27468E+01_JPRB, 0.26094E+01_JPRB, 0.24356E+01_JPRB ,&
&  0.22947E+01_JPRB, 0.19084E+01_JPRB, 0.15646E+01_JPRB, 0.13458E+01_JPRB, 0.11992E+01_JPRB, 0.11255E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11848E+01_JPRB, 0.12788E+01_JPRB, 0.13640E+01_JPRB, 0.13393E+01_JPRB, 0.14624E+01_JPRB, 0.16356E+01_JPRB, 0.18918E+01_JPRB ,&
&  0.21251E+01_JPRB, 0.23514E+01_JPRB, 0.26058E+01_JPRB, 0.32138E+01_JPRB, 0.51635E+01_JPRB, 0.80091E+01_JPRB, 0.87741E+01_JPRB ,&
&  0.78552E+01_JPRB, 0.72302E+01_JPRB, 0.78886E+01_JPRB, 0.70619E+01_JPRB, 0.60690E+01_JPRB, 0.59803E+01_JPRB, 0.55570E+01_JPRB ,&
&  0.56706E+01_JPRB, 0.63828E+01_JPRB, 0.73916E+01_JPRB, 0.93241E+01_JPRB, 0.12061E+02_JPRB, 0.12606E+02_JPRB, 0.12835E+02_JPRB ,&
&  0.11222E+02_JPRB, 0.85429E+01_JPRB, 0.58010E+01_JPRB, 0.40598E+01_JPRB, 0.38467E+01_JPRB, 0.32254E+01_JPRB, 0.21383E+01_JPRB ,&
&  0.17387E+01_JPRB, 0.14804E+01_JPRB, 0.11030E+01_JPRB, 0.12021E+01_JPRB, 0.10983E+01_JPRB, 0.13912E+01_JPRB, 0.24678E+01_JPRB ,&
&  0.43945E+01_JPRB, 0.57856E+01_JPRB, 0.74418E+01_JPRB, 0.68633E+01_JPRB, 0.69973E+01_JPRB, 0.90566E+01_JPRB, 0.91490E+01_JPRB ,&
&  0.42975E+01_JPRB, 0.26847E+01_JPRB, 0.21265E+01_JPRB, 0.19098E+01_JPRB, 0.18196E+01_JPRB, 0.17565E+01_JPRB, 0.16929E+01_JPRB ,&
&  0.15913E+01_JPRB, 0.15001E+01_JPRB, 0.14550E+01_JPRB, 0.13802E+01_JPRB, 0.12480E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.98715E+00_JPRB, 0.97376E+00_JPRB, 0.97715E+00_JPRB, 0.95735E+00_JPRB, 0.90526E+00_JPRB, 0.91015E+00_JPRB, 0.92741E+00_JPRB ,&
&  0.96363E+00_JPRB, 0.10161E+01_JPRB, 0.10903E+01_JPRB, 0.11939E+01_JPRB, 0.13355E+01_JPRB, 0.14092E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.14712E+01_JPRB, 0.16593E+01_JPRB, 0.19770E+01_JPRB, 0.24226E+01_JPRB, 0.27807E+01_JPRB, 0.23411E+01_JPRB, 0.32556E+01_JPRB ,&
&  0.26848E+01_JPRB, 0.21482E+01_JPRB, 0.20983E+01_JPRB, 0.22739E+01_JPRB, 0.23118E+01_JPRB, 0.21557E+01_JPRB, 0.19736E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.16021E+01_JPRB, 0.13889E+01_JPRB, 0.12686E+01_JPRB, 0.11772E+01_JPRB, 0.12019E+01_JPRB, 0.12745E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.14726E+01_JPRB, 0.14937E+01_JPRB, 0.16140E+01_JPRB, 0.18572E+01_JPRB, 0.20594E+01_JPRB, 0.22385E+01_JPRB ,&
&  0.23670E+01_JPRB, 0.26266E+01_JPRB, 0.31720E+01_JPRB, 0.46899E+01_JPRB, 0.63010E+01_JPRB, 0.68511E+01_JPRB, 0.78611E+01_JPRB ,&
&  0.84203E+01_JPRB, 0.84897E+01_JPRB, 0.74918E+01_JPRB, 0.78339E+01_JPRB, 0.68489E+01_JPRB, 0.62111E+01_JPRB, 0.62769E+01_JPRB ,&
&  0.69032E+01_JPRB, 0.88674E+01_JPRB, 0.97809E+01_JPRB, 0.10664E+02_JPRB, 0.15136E+02_JPRB, 0.16507E+02_JPRB, 0.13193E+02_JPRB ,&
&  0.11615E+02_JPRB, 0.83113E+01_JPRB, 0.58063E+01_JPRB, 0.42596E+01_JPRB, 0.52887E+01_JPRB, 0.59374E+01_JPRB, 0.29865E+01_JPRB ,&
&  0.19796E+01_JPRB, 0.15248E+01_JPRB, 0.11237E+01_JPRB, 0.10935E+01_JPRB, 0.11062E+01_JPRB, 0.14339E+01_JPRB, 0.33223E+01_JPRB ,&
&  0.40908E+01_JPRB, 0.66661E+01_JPRB, 0.76884E+01_JPRB, 0.74368E+01_JPRB, 0.72165E+01_JPRB, 0.66180E+01_JPRB, 0.42497E+01_JPRB ,&
&  0.25812E+01_JPRB, 0.17513E+01_JPRB, 0.15998E+01_JPRB, 0.17075E+01_JPRB, 0.17347E+01_JPRB, 0.16336E+01_JPRB, 0.14938E+01_JPRB ,&
&  0.14015E+01_JPRB, 0.13698E+01_JPRB, 0.12997E+01_JPRB, 0.12280E+01_JPRB, 0.11046E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.99116E+00_JPRB, 0.10163E+01_JPRB, 0.10424E+01_JPRB, 0.98392E+00_JPRB, 0.96898E+00_JPRB, 0.98665E+00_JPRB, 0.98595E+00_JPRB ,&
&  0.10496E+01_JPRB, 0.11423E+01_JPRB, 0.12453E+01_JPRB, 0.14231E+01_JPRB, 0.16000E+01_JPRB, 0.16358E+01_JPRB, 0.16463E+01_JPRB ,&
&  0.17358E+01_JPRB, 0.20368E+01_JPRB, 0.25199E+01_JPRB, 0.31354E+01_JPRB, 0.32253E+01_JPRB, 0.30075E+01_JPRB, 0.35100E+01_JPRB ,&
&  0.27626E+01_JPRB, 0.22940E+01_JPRB, 0.20912E+01_JPRB, 0.22171E+01_JPRB, 0.22270E+01_JPRB, 0.21372E+01_JPRB, 0.19012E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.16205E+01_JPRB, 0.13789E+01_JPRB, 0.14218E+01_JPRB, 0.13763E+01_JPRB, 0.14552E+01_JPRB, 0.15189E+01_JPRB ,&
&  0.16244E+01_JPRB, 0.16971E+01_JPRB, 0.17884E+01_JPRB, 0.20031E+01_JPRB, 0.22882E+01_JPRB, 0.25389E+01_JPRB, 0.27674E+01_JPRB ,&
&  0.29619E+01_JPRB, 0.34718E+01_JPRB, 0.44679E+01_JPRB, 0.60153E+01_JPRB, 0.59008E+01_JPRB, 0.76752E+01_JPRB, 0.79211E+01_JPRB ,&
&  0.82106E+01_JPRB, 0.80605E+01_JPRB, 0.71430E+01_JPRB, 0.72313E+01_JPRB, 0.72562E+01_JPRB, 0.69881E+01_JPRB, 0.72611E+01_JPRB ,&
&  0.94066E+01_JPRB, 0.10720E+02_JPRB, 0.12119E+02_JPRB, 0.13045E+02_JPRB, 0.17076E+02_JPRB, 0.23194E+02_JPRB, 0.13809E+02_JPRB ,&
&  0.95425E+01_JPRB, 0.84117E+01_JPRB, 0.59737E+01_JPRB, 0.47547E+01_JPRB, 0.60054E+01_JPRB, 0.72959E+01_JPRB, 0.69030E+01_JPRB ,&
&  0.28418E+01_JPRB, 0.16194E+01_JPRB, 0.14468E+01_JPRB, 0.14504E+01_JPRB, 0.12100E+01_JPRB, 0.15874E+01_JPRB, 0.54666E+01_JPRB ,&
&  0.43648E+01_JPRB, 0.68175E+01_JPRB, 0.72794E+01_JPRB, 0.70005E+01_JPRB, 0.71896E+01_JPRB, 0.50596E+01_JPRB, 0.29630E+01_JPRB ,&
&  0.14550E+01_JPRB, 0.13894E+01_JPRB, 0.15402E+01_JPRB, 0.14922E+01_JPRB, 0.14668E+01_JPRB, 0.13776E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.11969E+01_JPRB, 0.11176E+01_JPRB, 0.10491E+01_JPRB, 0.10063E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.10461E+01_JPRB, 0.10817E+01_JPRB, 0.10844E+01_JPRB, 0.10834E+01_JPRB, 0.10985E+01_JPRB, 0.11157E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.13430E+01_JPRB, 0.15436E+01_JPRB, 0.18005E+01_JPRB, 0.19615E+01_JPRB, 0.19637E+01_JPRB, 0.20038E+01_JPRB ,&
&  0.22167E+01_JPRB, 0.26301E+01_JPRB, 0.31437E+01_JPRB, 0.35101E+01_JPRB, 0.33908E+01_JPRB, 0.30872E+01_JPRB, 0.33595E+01_JPRB ,&
&  0.30479E+01_JPRB, 0.25273E+01_JPRB, 0.21047E+01_JPRB, 0.22394E+01_JPRB, 0.24086E+01_JPRB, 0.24532E+01_JPRB, 0.21700E+01_JPRB ,&
&  0.19879E+01_JPRB, 0.19484E+01_JPRB, 0.18123E+01_JPRB, 0.18248E+01_JPRB, 0.18618E+01_JPRB, 0.19620E+01_JPRB, 0.20395E+01_JPRB ,&
&  0.20198E+01_JPRB, 0.20988E+01_JPRB, 0.22865E+01_JPRB, 0.25632E+01_JPRB, 0.28872E+01_JPRB, 0.31949E+01_JPRB, 0.35456E+01_JPRB ,&
&  0.40090E+01_JPRB, 0.47801E+01_JPRB, 0.57726E+01_JPRB, 0.61652E+01_JPRB, 0.66649E+01_JPRB, 0.72808E+01_JPRB, 0.77476E+01_JPRB ,&
&  0.79700E+01_JPRB, 0.80335E+01_JPRB, 0.76898E+01_JPRB, 0.73463E+01_JPRB, 0.75418E+01_JPRB, 0.80311E+01_JPRB, 0.92487E+01_JPRB ,&
&  0.11724E+02_JPRB, 0.13284E+02_JPRB, 0.13621E+02_JPRB, 0.14369E+02_JPRB, 0.15880E+02_JPRB, 0.17023E+02_JPRB, 0.14103E+02_JPRB ,&
&  0.10288E+02_JPRB, 0.72019E+01_JPRB, 0.56671E+01_JPRB, 0.59592E+01_JPRB, 0.73168E+01_JPRB, 0.77265E+01_JPRB, 0.10170E+02_JPRB ,&
&  0.99718E+01_JPRB, 0.38896E+01_JPRB, 0.25918E+01_JPRB, 0.34468E+01_JPRB, 0.17264E+01_JPRB, 0.15256E+01_JPRB, 0.28427E+01_JPRB ,&
&  0.45947E+01_JPRB, 0.62050E+01_JPRB, 0.61872E+01_JPRB, 0.59944E+01_JPRB, 0.62250E+01_JPRB, 0.35910E+01_JPRB, 0.17580E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.13406E+01_JPRB, 0.13432E+01_JPRB, 0.12144E+01_JPRB, 0.11693E+01_JPRB, 0.12082E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.10931E+01_JPRB, 0.10989E+01_JPRB, 0.10728E+01_JPRB, 0.10363E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.11710E+01_JPRB, 0.12045E+01_JPRB, 0.12647E+01_JPRB, 0.13084E+01_JPRB, 0.13194E+01_JPRB, 0.13552E+01_JPRB, 0.14117E+01_JPRB ,&
&  0.15586E+01_JPRB, 0.17761E+01_JPRB, 0.20224E+01_JPRB, 0.22621E+01_JPRB, 0.23442E+01_JPRB, 0.23703E+01_JPRB, 0.25226E+01_JPRB ,&
&  0.28110E+01_JPRB, 0.32136E+01_JPRB, 0.36011E+01_JPRB, 0.38950E+01_JPRB, 0.39227E+01_JPRB, 0.40550E+01_JPRB, 0.33738E+01_JPRB ,&
&  0.29183E+01_JPRB, 0.28928E+01_JPRB, 0.22894E+01_JPRB, 0.24791E+01_JPRB, 0.26738E+01_JPRB, 0.27963E+01_JPRB, 0.25913E+01_JPRB ,&
&  0.24643E+01_JPRB, 0.24211E+01_JPRB, 0.23404E+01_JPRB, 0.23388E+01_JPRB, 0.24380E+01_JPRB, 0.25418E+01_JPRB, 0.26298E+01_JPRB ,&
&  0.27247E+01_JPRB, 0.28143E+01_JPRB, 0.30249E+01_JPRB, 0.33068E+01_JPRB, 0.36159E+01_JPRB, 0.39778E+01_JPRB, 0.44760E+01_JPRB ,&
&  0.50340E+01_JPRB, 0.55459E+01_JPRB, 0.60540E+01_JPRB, 0.60764E+01_JPRB, 0.65046E+01_JPRB, 0.70183E+01_JPRB, 0.74062E+01_JPRB ,&
&  0.72171E+01_JPRB, 0.69452E+01_JPRB, 0.74998E+01_JPRB, 0.83058E+01_JPRB, 0.89515E+01_JPRB, 0.94792E+01_JPRB, 0.10716E+02_JPRB ,&
&  0.12964E+02_JPRB, 0.14425E+02_JPRB, 0.14885E+02_JPRB, 0.12994E+02_JPRB, 0.11944E+02_JPRB, 0.11232E+02_JPRB, 0.11020E+02_JPRB ,&
&  0.93632E+01_JPRB, 0.73920E+01_JPRB, 0.65927E+01_JPRB, 0.70028E+01_JPRB, 0.76808E+01_JPRB, 0.79358E+01_JPRB, 0.91682E+01_JPRB ,&
&  0.10292E+02_JPRB, 0.91542E+01_JPRB, 0.71220E+01_JPRB, 0.43741E+01_JPRB, 0.26402E+01_JPRB, 0.21907E+01_JPRB, 0.28222E+01_JPRB ,&
&  0.43940E+01_JPRB, 0.47728E+01_JPRB, 0.46719E+01_JPRB, 0.60315E+01_JPRB, 0.44470E+01_JPRB, 0.24137E+01_JPRB, 0.13595E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.11568E+01_JPRB, 0.11608E+01_JPRB, 0.11045E+01_JPRB, 0.11258E+01_JPRB, 0.11562E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.11380E+01_JPRB, 0.11786E+01_JPRB, 0.11481E+01_JPRB, 0.11210E+01_JPRB, 0.11432E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.14138E+01_JPRB, 0.14872E+01_JPRB, 0.15516E+01_JPRB, 0.15930E+01_JPRB, 0.16613E+01_JPRB, 0.16984E+01_JPRB, 0.18083E+01_JPRB ,&
&  0.20137E+01_JPRB, 0.22367E+01_JPRB, 0.24443E+01_JPRB, 0.25708E+01_JPRB, 0.26291E+01_JPRB, 0.27332E+01_JPRB, 0.29469E+01_JPRB ,&
&  0.31845E+01_JPRB, 0.34716E+01_JPRB, 0.37246E+01_JPRB, 0.40029E+01_JPRB, 0.42083E+01_JPRB, 0.47318E+01_JPRB, 0.42593E+01_JPRB ,&
&  0.28130E+01_JPRB, 0.36948E+01_JPRB, 0.25746E+01_JPRB, 0.28367E+01_JPRB, 0.29706E+01_JPRB, 0.31646E+01_JPRB, 0.29112E+01_JPRB ,&
&  0.27572E+01_JPRB, 0.27884E+01_JPRB, 0.28001E+01_JPRB, 0.27692E+01_JPRB, 0.27899E+01_JPRB, 0.29775E+01_JPRB, 0.31595E+01_JPRB ,&
&  0.33186E+01_JPRB, 0.34547E+01_JPRB, 0.36521E+01_JPRB, 0.39037E+01_JPRB, 0.41928E+01_JPRB, 0.45453E+01_JPRB, 0.49575E+01_JPRB ,&
&  0.53107E+01_JPRB, 0.56479E+01_JPRB, 0.57745E+01_JPRB, 0.58932E+01_JPRB, 0.60877E+01_JPRB, 0.64193E+01_JPRB, 0.67966E+01_JPRB ,&
&  0.69275E+01_JPRB, 0.70186E+01_JPRB, 0.77437E+01_JPRB, 0.82550E+01_JPRB, 0.86464E+01_JPRB, 0.96116E+01_JPRB, 0.10279E+02_JPRB ,&
&  0.12094E+02_JPRB, 0.12136E+02_JPRB, 0.12345E+02_JPRB, 0.10292E+02_JPRB, 0.82080E+01_JPRB, 0.72834E+01_JPRB, 0.70129E+01_JPRB ,&
&  0.62438E+01_JPRB, 0.62072E+01_JPRB, 0.61634E+01_JPRB, 0.65500E+01_JPRB, 0.72607E+01_JPRB, 0.79468E+01_JPRB, 0.77805E+01_JPRB ,&
&  0.75416E+01_JPRB, 0.53740E+01_JPRB, 0.39830E+01_JPRB, 0.33974E+01_JPRB, 0.25180E+01_JPRB, 0.22582E+01_JPRB, 0.30530E+01_JPRB ,&
&  0.40814E+01_JPRB, 0.42544E+01_JPRB, 0.45697E+01_JPRB, 0.44996E+01_JPRB, 0.26188E+01_JPRB, 0.15553E+01_JPRB, 0.11079E+01_JPRB ,&
&  0.10171E+01_JPRB, 0.10636E+01_JPRB, 0.10733E+01_JPRB, 0.10761E+01_JPRB, 0.11259E+01_JPRB, 0.10915E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.12205E+01_JPRB, 0.12578E+01_JPRB, 0.12717E+01_JPRB, 0.13275E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.17039E+01_JPRB, 0.18714E+01_JPRB, 0.20413E+01_JPRB, 0.21685E+01_JPRB, 0.21671E+01_JPRB, 0.21751E+01_JPRB, 0.22320E+01_JPRB ,&
&  0.23122E+01_JPRB, 0.24287E+01_JPRB, 0.25760E+01_JPRB, 0.27045E+01_JPRB, 0.28156E+01_JPRB, 0.29951E+01_JPRB, 0.32548E+01_JPRB ,&
&  0.35465E+01_JPRB, 0.38511E+01_JPRB, 0.41565E+01_JPRB, 0.45293E+01_JPRB, 0.50039E+01_JPRB, 0.57525E+01_JPRB, 0.56701E+01_JPRB ,&
&  0.40327E+01_JPRB, 0.30955E+01_JPRB, 0.30725E+01_JPRB, 0.29461E+01_JPRB, 0.28852E+01_JPRB, 0.31527E+01_JPRB, 0.32065E+01_JPRB ,&
&  0.30456E+01_JPRB, 0.30092E+01_JPRB, 0.30534E+01_JPRB, 0.30426E+01_JPRB, 0.30188E+01_JPRB, 0.32329E+01_JPRB, 0.34326E+01_JPRB ,&
&  0.36345E+01_JPRB, 0.37769E+01_JPRB, 0.39349E+01_JPRB, 0.41300E+01_JPRB, 0.43530E+01_JPRB, 0.46027E+01_JPRB, 0.48814E+01_JPRB ,&
&  0.50726E+01_JPRB, 0.52884E+01_JPRB, 0.54171E+01_JPRB, 0.55255E+01_JPRB, 0.56778E+01_JPRB, 0.57961E+01_JPRB, 0.59629E+01_JPRB ,&
&  0.63133E+01_JPRB, 0.68384E+01_JPRB, 0.73869E+01_JPRB, 0.77393E+01_JPRB, 0.78287E+01_JPRB, 0.87065E+01_JPRB, 0.91934E+01_JPRB ,&
&  0.96111E+01_JPRB, 0.90387E+01_JPRB, 0.90881E+01_JPRB, 0.77241E+01_JPRB, 0.65689E+01_JPRB, 0.55151E+01_JPRB, 0.51625E+01_JPRB ,&
&  0.48552E+01_JPRB, 0.47346E+01_JPRB, 0.45227E+01_JPRB, 0.47952E+01_JPRB, 0.57038E+01_JPRB, 0.62515E+01_JPRB, 0.64329E+01_JPRB ,&
&  0.53655E+01_JPRB, 0.40266E+01_JPRB, 0.28366E+01_JPRB, 0.29053E+01_JPRB, 0.23909E+01_JPRB, 0.26967E+01_JPRB, 0.31607E+01_JPRB ,&
&  0.34499E+01_JPRB, 0.29851E+01_JPRB, 0.30921E+01_JPRB, 0.29356E+01_JPRB, 0.17425E+01_JPRB, 0.11541E+01_JPRB, 0.91631E+00_JPRB ,&
&  0.84980E+00_JPRB, 0.90041E+00_JPRB, 0.92989E+00_JPRB, 0.93776E+00_JPRB, 0.10051E+01_JPRB, 0.10652E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.12587E+01_JPRB, 0.13369E+01_JPRB, 0.14174E+01_JPRB, 0.15438E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.17190E+01_JPRB, 0.18464E+01_JPRB, 0.19814E+01_JPRB, 0.20104E+01_JPRB, 0.20467E+01_JPRB, 0.20823E+01_JPRB, 0.21370E+01_JPRB ,&
&  0.22715E+01_JPRB, 0.24025E+01_JPRB, 0.25510E+01_JPRB, 0.26811E+01_JPRB, 0.27974E+01_JPRB, 0.29831E+01_JPRB, 0.32083E+01_JPRB ,&
&  0.33997E+01_JPRB, 0.35844E+01_JPRB, 0.37745E+01_JPRB, 0.40291E+01_JPRB, 0.43075E+01_JPRB, 0.46952E+01_JPRB, 0.48882E+01_JPRB ,&
&  0.38485E+01_JPRB, 0.23480E+01_JPRB, 0.25512E+01_JPRB, 0.26433E+01_JPRB, 0.27613E+01_JPRB, 0.30831E+01_JPRB, 0.32068E+01_JPRB ,&
&  0.29436E+01_JPRB, 0.29321E+01_JPRB, 0.30475E+01_JPRB, 0.30239E+01_JPRB, 0.28968E+01_JPRB, 0.31251E+01_JPRB, 0.33523E+01_JPRB ,&
&  0.35747E+01_JPRB, 0.37654E+01_JPRB, 0.39169E+01_JPRB, 0.40843E+01_JPRB, 0.42225E+01_JPRB, 0.43534E+01_JPRB, 0.44747E+01_JPRB ,&
&  0.45996E+01_JPRB, 0.46791E+01_JPRB, 0.48044E+01_JPRB, 0.50642E+01_JPRB, 0.51490E+01_JPRB, 0.51920E+01_JPRB, 0.52261E+01_JPRB ,&
&  0.54614E+01_JPRB, 0.58152E+01_JPRB, 0.64043E+01_JPRB, 0.69340E+01_JPRB, 0.73406E+01_JPRB, 0.72796E+01_JPRB, 0.74111E+01_JPRB ,&
&  0.70486E+01_JPRB, 0.63345E+01_JPRB, 0.64795E+01_JPRB, 0.67936E+01_JPRB, 0.49036E+01_JPRB, 0.43375E+01_JPRB, 0.37873E+01_JPRB ,&
&  0.38262E+01_JPRB, 0.33875E+01_JPRB, 0.30835E+01_JPRB, 0.32379E+01_JPRB, 0.42030E+01_JPRB, 0.44607E+01_JPRB, 0.46213E+01_JPRB ,&
&  0.38949E+01_JPRB, 0.25356E+01_JPRB, 0.22722E+01_JPRB, 0.27103E+01_JPRB, 0.27714E+01_JPRB, 0.39448E+01_JPRB, 0.33072E+01_JPRB ,&
&  0.23221E+01_JPRB, 0.19824E+01_JPRB, 0.22701E+01_JPRB, 0.23235E+01_JPRB, 0.14522E+01_JPRB, 0.10061E+01_JPRB, 0.84745E+00_JPRB ,&
&  0.78940E+00_JPRB, 0.78358E+00_JPRB, 0.82582E+00_JPRB, 0.82533E+00_JPRB, 0.88534E+00_JPRB, 0.94261E+00_JPRB, 0.10622E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.12552E+01_JPRB, 0.13359E+01_JPRB, 0.14424E+01_JPRB, 0.15657E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.12364E+01_JPRB, 0.13137E+01_JPRB, 0.13441E+01_JPRB, 0.14959E+01_JPRB, 0.16681E+01_JPRB, 0.17986E+01_JPRB, 0.19156E+01_JPRB ,&
&  0.20744E+01_JPRB, 0.22777E+01_JPRB, 0.24426E+01_JPRB, 0.25635E+01_JPRB, 0.26917E+01_JPRB, 0.28595E+01_JPRB, 0.29142E+01_JPRB ,&
&  0.29144E+01_JPRB, 0.29145E+01_JPRB, 0.29374E+01_JPRB, 0.29510E+01_JPRB, 0.29918E+01_JPRB, 0.30266E+01_JPRB, 0.32837E+01_JPRB ,&
&  0.32755E+01_JPRB, 0.30264E+01_JPRB, 0.27524E+01_JPRB, 0.20766E+01_JPRB, 0.16902E+01_JPRB, 0.21630E+01_JPRB, 0.21850E+01_JPRB ,&
&  0.24266E+01_JPRB, 0.24983E+01_JPRB, 0.24726E+01_JPRB, 0.26145E+01_JPRB, 0.27297E+01_JPRB, 0.28874E+01_JPRB, 0.30937E+01_JPRB ,&
&  0.32550E+01_JPRB, 0.34286E+01_JPRB, 0.35871E+01_JPRB, 0.37308E+01_JPRB, 0.38340E+01_JPRB, 0.38956E+01_JPRB, 0.39141E+01_JPRB ,&
&  0.39439E+01_JPRB, 0.40330E+01_JPRB, 0.41756E+01_JPRB, 0.45041E+01_JPRB, 0.46240E+01_JPRB, 0.47524E+01_JPRB, 0.47049E+01_JPRB ,&
&  0.47944E+01_JPRB, 0.49621E+01_JPRB, 0.53670E+01_JPRB, 0.56992E+01_JPRB, 0.57746E+01_JPRB, 0.54174E+01_JPRB, 0.52428E+01_JPRB ,&
&  0.49721E+01_JPRB, 0.45998E+01_JPRB, 0.46694E+01_JPRB, 0.51625E+01_JPRB, 0.35220E+01_JPRB, 0.35749E+01_JPRB, 0.35507E+01_JPRB ,&
&  0.29788E+01_JPRB, 0.26053E+01_JPRB, 0.23579E+01_JPRB, 0.21639E+01_JPRB, 0.26780E+01_JPRB, 0.31775E+01_JPRB, 0.37498E+01_JPRB ,&
&  0.30019E+01_JPRB, 0.19722E+01_JPRB, 0.19025E+01_JPRB, 0.24700E+01_JPRB, 0.29276E+01_JPRB, 0.39264E+01_JPRB, 0.27666E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.18337E+01_JPRB, 0.21514E+01_JPRB, 0.25304E+01_JPRB, 0.17162E+01_JPRB, 0.95890E+00_JPRB, 0.77705E+00_JPRB ,&
&  0.73959E+00_JPRB, 0.74222E+00_JPRB, 0.75483E+00_JPRB, 0.75803E+00_JPRB, 0.79514E+00_JPRB, 0.83708E+00_JPRB, 0.86010E+00_JPRB ,&
&  0.88606E+00_JPRB, 0.90574E+00_JPRB, 0.95426E+00_JPRB, 0.10321E+01_JPRB, 0.11463E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.82220E+00_JPRB, 0.83266E+00_JPRB, 0.82365E+00_JPRB, 0.85857E+00_JPRB, 0.92381E+00_JPRB, 0.10579E+01_JPRB, 0.11934E+01_JPRB ,&
&  0.13478E+01_JPRB, 0.15373E+01_JPRB, 0.18094E+01_JPRB, 0.20703E+01_JPRB, 0.21831E+01_JPRB, 0.21658E+01_JPRB, 0.19688E+01_JPRB ,&
&  0.17018E+01_JPRB, 0.14150E+01_JPRB, 0.12376E+01_JPRB, 0.11641E+01_JPRB, 0.12178E+01_JPRB, 0.12410E+01_JPRB, 0.13216E+01_JPRB ,&
&  0.14769E+01_JPRB, 0.17345E+01_JPRB, 0.19612E+01_JPRB, 0.24883E+01_JPRB, 0.26750E+01_JPRB, 0.20752E+01_JPRB, 0.20820E+01_JPRB ,&
&  0.26736E+01_JPRB, 0.27278E+01_JPRB, 0.19855E+01_JPRB, 0.19403E+01_JPRB, 0.21420E+01_JPRB, 0.23103E+01_JPRB, 0.25410E+01_JPRB ,&
&  0.26584E+01_JPRB, 0.27816E+01_JPRB, 0.28898E+01_JPRB, 0.29505E+01_JPRB, 0.29722E+01_JPRB, 0.29172E+01_JPRB, 0.28645E+01_JPRB ,&
&  0.28369E+01_JPRB, 0.28245E+01_JPRB, 0.30358E+01_JPRB, 0.33188E+01_JPRB, 0.36212E+01_JPRB, 0.38666E+01_JPRB, 0.39535E+01_JPRB ,&
&  0.40215E+01_JPRB, 0.41219E+01_JPRB, 0.45544E+01_JPRB, 0.46705E+01_JPRB, 0.44009E+01_JPRB, 0.41191E+01_JPRB, 0.37590E+01_JPRB ,&
&  0.33774E+01_JPRB, 0.28552E+01_JPRB, 0.22351E+01_JPRB, 0.33929E+01_JPRB, 0.38665E+01_JPRB, 0.32794E+01_JPRB, 0.26379E+01_JPRB ,&
&  0.23230E+01_JPRB, 0.20978E+01_JPRB, 0.18724E+01_JPRB, 0.17829E+01_JPRB, 0.19996E+01_JPRB, 0.29779E+01_JPRB, 0.30646E+01_JPRB ,&
&  0.25253E+01_JPRB, 0.15834E+01_JPRB, 0.13768E+01_JPRB, 0.16563E+01_JPRB, 0.20176E+01_JPRB, 0.25947E+01_JPRB, 0.17795E+01_JPRB ,&
&  0.13607E+01_JPRB, 0.17273E+01_JPRB, 0.19944E+01_JPRB, 0.24793E+01_JPRB, 0.25590E+01_JPRB, 0.78558E+00_JPRB, 0.71830E+00_JPRB ,&
&  0.69997E+00_JPRB, 0.69583E+00_JPRB, 0.71553E+00_JPRB, 0.70940E+00_JPRB, 0.71543E+00_JPRB, 0.70745E+00_JPRB, 0.69488E+00_JPRB ,&
&  0.69713E+00_JPRB, 0.71787E+00_JPRB, 0.74388E+00_JPRB, 0.76107E+00_JPRB, 0.79583E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.76121E+00_JPRB, 0.76296E+00_JPRB, 0.76868E+00_JPRB, 0.77839E+00_JPRB, 0.78454E+00_JPRB, 0.79056E+00_JPRB, 0.81967E+00_JPRB ,&
&  0.86998E+00_JPRB, 0.94852E+00_JPRB, 0.10611E+01_JPRB, 0.11415E+01_JPRB, 0.11236E+01_JPRB, 0.10352E+01_JPRB, 0.98197E+00_JPRB ,&
&  0.95666E+00_JPRB, 0.93886E+00_JPRB, 0.97136E+00_JPRB, 0.10272E+01_JPRB, 0.10970E+01_JPRB, 0.11138E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11773E+01_JPRB, 0.12259E+01_JPRB, 0.13407E+01_JPRB, 0.14400E+01_JPRB, 0.21491E+01_JPRB, 0.20795E+01_JPRB, 0.18085E+01_JPRB ,&
&  0.21616E+01_JPRB, 0.22329E+01_JPRB, 0.19778E+01_JPRB, 0.16123E+01_JPRB, 0.15192E+01_JPRB, 0.14976E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.15431E+01_JPRB, 0.16715E+01_JPRB, 0.17821E+01_JPRB, 0.19041E+01_JPRB, 0.19923E+01_JPRB, 0.20236E+01_JPRB, 0.21024E+01_JPRB ,&
&  0.21806E+01_JPRB, 0.22662E+01_JPRB, 0.23781E+01_JPRB, 0.21955E+01_JPRB, 0.24335E+01_JPRB, 0.27169E+01_JPRB, 0.29287E+01_JPRB ,&
&  0.30369E+01_JPRB, 0.31032E+01_JPRB, 0.33895E+01_JPRB, 0.33148E+01_JPRB, 0.31742E+01_JPRB, 0.28930E+01_JPRB, 0.26393E+01_JPRB ,&
&  0.24321E+01_JPRB, 0.17888E+01_JPRB, 0.11731E+01_JPRB, 0.17280E+01_JPRB, 0.20807E+01_JPRB, 0.17656E+01_JPRB, 0.18432E+01_JPRB ,&
&  0.17197E+01_JPRB, 0.16013E+01_JPRB, 0.14871E+01_JPRB, 0.14777E+01_JPRB, 0.14697E+01_JPRB, 0.19281E+01_JPRB, 0.22733E+01_JPRB ,&
&  0.18501E+01_JPRB, 0.12626E+01_JPRB, 0.10680E+01_JPRB, 0.11358E+01_JPRB, 0.13372E+01_JPRB, 0.16274E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.16503E+01_JPRB, 0.14058E+01_JPRB, 0.17631E+01_JPRB, 0.22249E+01_JPRB, 0.12250E+01_JPRB, 0.64368E+00_JPRB, 0.59845E+00_JPRB ,&
&  0.59864E+00_JPRB, 0.61067E+00_JPRB, 0.63410E+00_JPRB, 0.65011E+00_JPRB, 0.66170E+00_JPRB, 0.67089E+00_JPRB, 0.68893E+00_JPRB ,&
&  0.70371E+00_JPRB, 0.72090E+00_JPRB, 0.73053E+00_JPRB, 0.74190E+00_JPRB, 0.75407E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.97359E+00_JPRB, 0.99430E+00_JPRB, 0.10242E+01_JPRB, 0.10545E+01_JPRB, 0.10758E+01_JPRB, 0.11062E+01_JPRB, 0.11398E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.12440E+01_JPRB, 0.12870E+01_JPRB, 0.13035E+01_JPRB, 0.13175E+01_JPRB, 0.13352E+01_JPRB, 0.13356E+01_JPRB ,&
&  0.13676E+01_JPRB, 0.13854E+01_JPRB, 0.14626E+01_JPRB, 0.14963E+01_JPRB, 0.15388E+01_JPRB, 0.15916E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.15790E+01_JPRB, 0.15650E+01_JPRB, 0.15758E+01_JPRB, 0.15643E+01_JPRB, 0.16468E+01_JPRB, 0.16690E+01_JPRB, 0.16181E+01_JPRB ,&
&  0.16770E+01_JPRB, 0.16987E+01_JPRB, 0.18400E+01_JPRB, 0.14357E+01_JPRB, 0.13913E+01_JPRB, 0.14153E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.13852E+01_JPRB, 0.14811E+01_JPRB, 0.15808E+01_JPRB, 0.17186E+01_JPRB, 0.17952E+01_JPRB, 0.18597E+01_JPRB ,&
&  0.19355E+01_JPRB, 0.20159E+01_JPRB, 0.20642E+01_JPRB, 0.20413E+01_JPRB, 0.20680E+01_JPRB, 0.24477E+01_JPRB, 0.26198E+01_JPRB ,&
&  0.26975E+01_JPRB, 0.27857E+01_JPRB, 0.23544E+01_JPRB, 0.22019E+01_JPRB, 0.23234E+01_JPRB, 0.20691E+01_JPRB, 0.19667E+01_JPRB ,&
&  0.21446E+01_JPRB, 0.18352E+01_JPRB, 0.96226E+00_JPRB, 0.12441E+01_JPRB, 0.14318E+01_JPRB, 0.14684E+01_JPRB, 0.14722E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.12000E+01_JPRB, 0.11245E+01_JPRB, 0.11990E+01_JPRB, 0.12559E+01_JPRB, 0.12314E+01_JPRB, 0.13205E+01_JPRB ,&
&  0.11957E+01_JPRB, 0.10488E+01_JPRB, 0.97787E+00_JPRB, 0.11595E+01_JPRB, 0.17282E+01_JPRB, 0.27925E+01_JPRB, 0.24796E+01_JPRB ,&
&  0.17257E+01_JPRB, 0.15471E+01_JPRB, 0.25136E+01_JPRB, 0.34060E+01_JPRB, 0.12835E+01_JPRB, 0.79331E+00_JPRB, 0.59375E+00_JPRB ,&
&  0.62743E+00_JPRB, 0.64388E+00_JPRB, 0.65607E+00_JPRB, 0.68338E+00_JPRB, 0.72542E+00_JPRB, 0.74770E+00_JPRB, 0.75798E+00_JPRB ,&
&  0.78732E+00_JPRB, 0.82974E+00_JPRB, 0.86214E+00_JPRB, 0.90166E+00_JPRB, 0.93742E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.11694E+01_JPRB, 0.12017E+01_JPRB, 0.12267E+01_JPRB, 0.12538E+01_JPRB, 0.12868E+01_JPRB, 0.13198E+01_JPRB, 0.13430E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.13998E+01_JPRB, 0.14123E+01_JPRB, 0.14356E+01_JPRB, 0.14437E+01_JPRB, 0.15121E+01_JPRB, 0.15401E+01_JPRB ,&
&  0.16067E+01_JPRB, 0.16477E+01_JPRB, 0.16651E+01_JPRB, 0.16727E+01_JPRB, 0.16926E+01_JPRB, 0.16819E+01_JPRB, 0.16692E+01_JPRB ,&
&  0.16979E+01_JPRB, 0.17563E+01_JPRB, 0.18127E+01_JPRB, 0.18643E+01_JPRB, 0.18814E+01_JPRB, 0.19674E+01_JPRB, 0.20874E+01_JPRB ,&
&  0.17034E+01_JPRB, 0.19090E+01_JPRB, 0.18316E+01_JPRB, 0.15674E+01_JPRB, 0.15864E+01_JPRB, 0.16265E+01_JPRB, 0.15715E+01_JPRB ,&
&  0.15367E+01_JPRB, 0.15912E+01_JPRB, 0.16305E+01_JPRB, 0.16765E+01_JPRB, 0.17627E+01_JPRB, 0.18577E+01_JPRB, 0.19293E+01_JPRB ,&
&  0.20393E+01_JPRB, 0.21254E+01_JPRB, 0.21920E+01_JPRB, 0.23001E+01_JPRB, 0.22651E+01_JPRB, 0.26875E+01_JPRB, 0.28328E+01_JPRB ,&
&  0.28638E+01_JPRB, 0.27830E+01_JPRB, 0.21496E+01_JPRB, 0.20583E+01_JPRB, 0.21457E+01_JPRB, 0.20467E+01_JPRB, 0.19285E+01_JPRB ,&
&  0.21057E+01_JPRB, 0.19705E+01_JPRB, 0.13353E+01_JPRB, 0.13953E+01_JPRB, 0.13881E+01_JPRB, 0.13653E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.96253E+00_JPRB, 0.91224E+00_JPRB, 0.94259E+00_JPRB, 0.95580E+00_JPRB, 0.93116E+00_JPRB, 0.95987E+00_JPRB ,&
&  0.97591E+00_JPRB, 0.97339E+00_JPRB, 0.10142E+01_JPRB, 0.11139E+01_JPRB, 0.18581E+01_JPRB, 0.23936E+01_JPRB, 0.33010E+01_JPRB ,&
&  0.14622E+01_JPRB, 0.14627E+01_JPRB, 0.12932E+01_JPRB, 0.17910E+01_JPRB, 0.10600E+01_JPRB, 0.66397E+00_JPRB, 0.63643E+00_JPRB ,&
&  0.70680E+00_JPRB, 0.74107E+00_JPRB, 0.75326E+00_JPRB, 0.77661E+00_JPRB, 0.83024E+00_JPRB, 0.86536E+00_JPRB, 0.89967E+00_JPRB ,&
&  0.91867E+00_JPRB, 0.97263E+00_JPRB, 0.10423E+01_JPRB, 0.10958E+01_JPRB, 0.11253E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.12650E+01_JPRB, 0.13070E+01_JPRB, 0.13444E+01_JPRB, 0.13811E+01_JPRB, 0.14166E+01_JPRB, 0.14531E+01_JPRB, 0.14852E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.15324E+01_JPRB, 0.15438E+01_JPRB, 0.15604E+01_JPRB, 0.15721E+01_JPRB, 0.16283E+01_JPRB, 0.16479E+01_JPRB ,&
&  0.17094E+01_JPRB, 0.17376E+01_JPRB, 0.17348E+01_JPRB, 0.17734E+01_JPRB, 0.17563E+01_JPRB, 0.17251E+01_JPRB, 0.17199E+01_JPRB ,&
&  0.17348E+01_JPRB, 0.17397E+01_JPRB, 0.17453E+01_JPRB, 0.17612E+01_JPRB, 0.17744E+01_JPRB, 0.17480E+01_JPRB, 0.18952E+01_JPRB ,&
&  0.15328E+01_JPRB, 0.21261E+01_JPRB, 0.21361E+01_JPRB, 0.19039E+01_JPRB, 0.18222E+01_JPRB, 0.16957E+01_JPRB, 0.16826E+01_JPRB ,&
&  0.16987E+01_JPRB, 0.16890E+01_JPRB, 0.17092E+01_JPRB, 0.17581E+01_JPRB, 0.18424E+01_JPRB, 0.19490E+01_JPRB, 0.20458E+01_JPRB ,&
&  0.21724E+01_JPRB, 0.22907E+01_JPRB, 0.23857E+01_JPRB, 0.24749E+01_JPRB, 0.26444E+01_JPRB, 0.27754E+01_JPRB, 0.28270E+01_JPRB ,&
&  0.28298E+01_JPRB, 0.28555E+01_JPRB, 0.22463E+01_JPRB, 0.22162E+01_JPRB, 0.24041E+01_JPRB, 0.23057E+01_JPRB, 0.20528E+01_JPRB ,&
&  0.20760E+01_JPRB, 0.18814E+01_JPRB, 0.16661E+01_JPRB, 0.14711E+01_JPRB, 0.13427E+01_JPRB, 0.12310E+01_JPRB, 0.11052E+01_JPRB ,&
&  0.89992E+00_JPRB, 0.76001E+00_JPRB, 0.72869E+00_JPRB, 0.77256E+00_JPRB, 0.78941E+00_JPRB, 0.79163E+00_JPRB, 0.77865E+00_JPRB ,&
&  0.80577E+00_JPRB, 0.86070E+00_JPRB, 0.96733E+00_JPRB, 0.12021E+01_JPRB, 0.15688E+01_JPRB, 0.30361E+01_JPRB, 0.31546E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.13031E+01_JPRB, 0.79477E+00_JPRB, 0.91215E+00_JPRB, 0.91718E+00_JPRB, 0.76015E+00_JPRB, 0.76978E+00_JPRB ,&
&  0.88252E+00_JPRB, 0.94842E+00_JPRB, 0.97745E+00_JPRB, 0.92445E+00_JPRB, 0.95253E+00_JPRB, 0.94919E+00_JPRB, 0.96514E+00_JPRB ,&
&  0.10000E+01_JPRB, 0.10508E+01_JPRB, 0.11113E+01_JPRB, 0.11684E+01_JPRB, 0.12182E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.13063E+01_JPRB, 0.13510E+01_JPRB, 0.13976E+01_JPRB, 0.14488E+01_JPRB, 0.15010E+01_JPRB, 0.15426E+01_JPRB, 0.15837E+01_JPRB ,&
&  0.16235E+01_JPRB, 0.16529E+01_JPRB, 0.16643E+01_JPRB, 0.16731E+01_JPRB, 0.16829E+01_JPRB, 0.17346E+01_JPRB, 0.17763E+01_JPRB ,&
&  0.17756E+01_JPRB, 0.18042E+01_JPRB, 0.18230E+01_JPRB, 0.18477E+01_JPRB, 0.18393E+01_JPRB, 0.17990E+01_JPRB, 0.18174E+01_JPRB ,&
&  0.18167E+01_JPRB, 0.18288E+01_JPRB, 0.18425E+01_JPRB, 0.18924E+01_JPRB, 0.19613E+01_JPRB, 0.21393E+01_JPRB, 0.18953E+01_JPRB ,&
&  0.21140E+01_JPRB, 0.22845E+01_JPRB, 0.22455E+01_JPRB, 0.20705E+01_JPRB, 0.18881E+01_JPRB, 0.18430E+01_JPRB, 0.18289E+01_JPRB ,&
&  0.18420E+01_JPRB, 0.17996E+01_JPRB, 0.18012E+01_JPRB, 0.18175E+01_JPRB, 0.18671E+01_JPRB, 0.19717E+01_JPRB, 0.20561E+01_JPRB ,&
&  0.21801E+01_JPRB, 0.23716E+01_JPRB, 0.25589E+01_JPRB, 0.27265E+01_JPRB, 0.29412E+01_JPRB, 0.31635E+01_JPRB, 0.32968E+01_JPRB ,&
&  0.33569E+01_JPRB, 0.34277E+01_JPRB, 0.31244E+01_JPRB, 0.31749E+01_JPRB, 0.33377E+01_JPRB, 0.30101E+01_JPRB, 0.25111E+01_JPRB ,&
&  0.22767E+01_JPRB, 0.17607E+01_JPRB, 0.14717E+01_JPRB, 0.15468E+01_JPRB, 0.13536E+01_JPRB, 0.11661E+01_JPRB, 0.97432E+00_JPRB ,&
&  0.76992E+00_JPRB, 0.67088E+00_JPRB, 0.66884E+00_JPRB, 0.71639E+00_JPRB, 0.71700E+00_JPRB, 0.73508E+00_JPRB, 0.78036E+00_JPRB ,&
&  0.85338E+00_JPRB, 0.93847E+00_JPRB, 0.10653E+01_JPRB, 0.12861E+01_JPRB, 0.15808E+01_JPRB, 0.16197E+01_JPRB, 0.17742E+01_JPRB ,&
&  0.12627E+01_JPRB, 0.10375E+01_JPRB, 0.10446E+01_JPRB, 0.99059E+00_JPRB, 0.10110E+01_JPRB, 0.96360E+00_JPRB, 0.10603E+01_JPRB ,&
&  0.12070E+01_JPRB, 0.13516E+01_JPRB, 0.13664E+01_JPRB, 0.11600E+01_JPRB, 0.10646E+01_JPRB, 0.10289E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.10961E+01_JPRB, 0.11489E+01_JPRB, 0.12086E+01_JPRB, 0.12609E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.13037E+01_JPRB, 0.13511E+01_JPRB, 0.14009E+01_JPRB, 0.14502E+01_JPRB, 0.14914E+01_JPRB, 0.15264E+01_JPRB, 0.15555E+01_JPRB ,&
&  0.15943E+01_JPRB, 0.16326E+01_JPRB, 0.16622E+01_JPRB, 0.16802E+01_JPRB, 0.16996E+01_JPRB, 0.17364E+01_JPRB, 0.17802E+01_JPRB ,&
&  0.18325E+01_JPRB, 0.18736E+01_JPRB, 0.19416E+01_JPRB, 0.19521E+01_JPRB, 0.19629E+01_JPRB, 0.19910E+01_JPRB, 0.20575E+01_JPRB ,&
&  0.20948E+01_JPRB, 0.21014E+01_JPRB, 0.21187E+01_JPRB, 0.21890E+01_JPRB, 0.22609E+01_JPRB, 0.23944E+01_JPRB, 0.21686E+01_JPRB ,&
&  0.23393E+01_JPRB, 0.24679E+01_JPRB, 0.23787E+01_JPRB, 0.22623E+01_JPRB, 0.20741E+01_JPRB, 0.20106E+01_JPRB, 0.19848E+01_JPRB ,&
&  0.19590E+01_JPRB, 0.19358E+01_JPRB, 0.19026E+01_JPRB, 0.18979E+01_JPRB, 0.19203E+01_JPRB, 0.19784E+01_JPRB, 0.20771E+01_JPRB ,&
&  0.21802E+01_JPRB, 0.23296E+01_JPRB, 0.25613E+01_JPRB, 0.27928E+01_JPRB, 0.31028E+01_JPRB, 0.34082E+01_JPRB, 0.36423E+01_JPRB ,&
&  0.38678E+01_JPRB, 0.40924E+01_JPRB, 0.38644E+01_JPRB, 0.40042E+01_JPRB, 0.42465E+01_JPRB, 0.37356E+01_JPRB, 0.28522E+01_JPRB ,&
&  0.19526E+01_JPRB, 0.16264E+01_JPRB, 0.15952E+01_JPRB, 0.15986E+01_JPRB, 0.13489E+01_JPRB, 0.11424E+01_JPRB, 0.98800E+00_JPRB ,&
&  0.81482E+00_JPRB, 0.69684E+00_JPRB, 0.70613E+00_JPRB, 0.70307E+00_JPRB, 0.70063E+00_JPRB, 0.74592E+00_JPRB, 0.81050E+00_JPRB ,&
&  0.89630E+00_JPRB, 0.10319E+01_JPRB, 0.11839E+01_JPRB, 0.13571E+01_JPRB, 0.14808E+01_JPRB, 0.14398E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.12596E+01_JPRB, 0.12291E+01_JPRB, 0.93455E+00_JPRB, 0.11837E+01_JPRB, 0.12577E+01_JPRB, 0.13802E+01_JPRB ,&
&  0.13410E+01_JPRB, 0.17636E+01_JPRB, 0.21208E+01_JPRB, 0.16747E+01_JPRB, 0.13694E+01_JPRB, 0.10768E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.10937E+01_JPRB, 0.11408E+01_JPRB, 0.11981E+01_JPRB, 0.12522E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.12598E+01_JPRB, 0.13122E+01_JPRB, 0.13678E+01_JPRB, 0.14134E+01_JPRB, 0.14473E+01_JPRB, 0.14503E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.14678E+01_JPRB, 0.14947E+01_JPRB, 0.15298E+01_JPRB, 0.15674E+01_JPRB, 0.15978E+01_JPRB, 0.16295E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.17252E+01_JPRB, 0.17880E+01_JPRB, 0.18723E+01_JPRB, 0.19522E+01_JPRB, 0.20467E+01_JPRB, 0.21372E+01_JPRB, 0.22392E+01_JPRB ,&
&  0.23323E+01_JPRB, 0.24345E+01_JPRB, 0.24815E+01_JPRB, 0.25550E+01_JPRB, 0.26013E+01_JPRB, 0.27078E+01_JPRB, 0.21926E+01_JPRB ,&
&  0.24373E+01_JPRB, 0.25206E+01_JPRB, 0.24909E+01_JPRB, 0.23552E+01_JPRB, 0.21627E+01_JPRB, 0.20675E+01_JPRB, 0.19972E+01_JPRB ,&
&  0.19648E+01_JPRB, 0.19013E+01_JPRB, 0.17883E+01_JPRB, 0.18553E+01_JPRB, 0.19263E+01_JPRB, 0.19265E+01_JPRB, 0.20363E+01_JPRB ,&
&  0.21485E+01_JPRB, 0.23097E+01_JPRB, 0.25062E+01_JPRB, 0.27937E+01_JPRB, 0.31528E+01_JPRB, 0.34952E+01_JPRB, 0.38438E+01_JPRB ,&
&  0.42049E+01_JPRB, 0.45854E+01_JPRB, 0.48758E+01_JPRB, 0.44559E+01_JPRB, 0.44522E+01_JPRB, 0.39082E+01_JPRB, 0.28518E+01_JPRB ,&
&  0.18573E+01_JPRB, 0.15827E+01_JPRB, 0.15977E+01_JPRB, 0.16152E+01_JPRB, 0.13812E+01_JPRB, 0.11512E+01_JPRB, 0.98468E+00_JPRB ,&
&  0.82910E+00_JPRB, 0.75276E+00_JPRB, 0.73797E+00_JPRB, 0.71397E+00_JPRB, 0.71455E+00_JPRB, 0.76310E+00_JPRB, 0.83019E+00_JPRB ,&
&  0.91501E+00_JPRB, 0.10357E+01_JPRB, 0.11873E+01_JPRB, 0.13547E+01_JPRB, 0.15193E+01_JPRB, 0.15777E+01_JPRB, 0.14115E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.12374E+01_JPRB, 0.12602E+01_JPRB, 0.13488E+01_JPRB, 0.13579E+01_JPRB, 0.15877E+01_JPRB, 0.18251E+01_JPRB ,&
&  0.20487E+01_JPRB, 0.18864E+01_JPRB, 0.21875E+01_JPRB, 0.30030E+01_JPRB, 0.27600E+01_JPRB, 0.22884E+01_JPRB, 0.97629E+00_JPRB ,&
&  0.98990E+00_JPRB, 0.10018E+01_JPRB, 0.10554E+01_JPRB, 0.11245E+01_JPRB, 0.12027E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.10137E+01_JPRB, 0.10843E+01_JPRB, 0.11575E+01_JPRB, 0.12547E+01_JPRB, 0.13423E+01_JPRB, 0.13644E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.13505E+01_JPRB, 0.13587E+01_JPRB, 0.13850E+01_JPRB, 0.14222E+01_JPRB, 0.14568E+01_JPRB, 0.14865E+01_JPRB, 0.15230E+01_JPRB ,&
&  0.15716E+01_JPRB, 0.16286E+01_JPRB, 0.16964E+01_JPRB, 0.17713E+01_JPRB, 0.18491E+01_JPRB, 0.19600E+01_JPRB, 0.20927E+01_JPRB ,&
&  0.22377E+01_JPRB, 0.23829E+01_JPRB, 0.24993E+01_JPRB, 0.26152E+01_JPRB, 0.27156E+01_JPRB, 0.27115E+01_JPRB, 0.19666E+01_JPRB ,&
&  0.24224E+01_JPRB, 0.24786E+01_JPRB, 0.25305E+01_JPRB, 0.24026E+01_JPRB, 0.21768E+01_JPRB, 0.20443E+01_JPRB, 0.19896E+01_JPRB ,&
&  0.19304E+01_JPRB, 0.18430E+01_JPRB, 0.17407E+01_JPRB, 0.16781E+01_JPRB, 0.18320E+01_JPRB, 0.18287E+01_JPRB, 0.19567E+01_JPRB ,&
&  0.20863E+01_JPRB, 0.22416E+01_JPRB, 0.24264E+01_JPRB, 0.27139E+01_JPRB, 0.30890E+01_JPRB, 0.34760E+01_JPRB, 0.38985E+01_JPRB ,&
&  0.43263E+01_JPRB, 0.47092E+01_JPRB, 0.49279E+01_JPRB, 0.43217E+01_JPRB, 0.40987E+01_JPRB, 0.37654E+01_JPRB, 0.24471E+01_JPRB ,&
&  0.18697E+01_JPRB, 0.17471E+01_JPRB, 0.18042E+01_JPRB, 0.17022E+01_JPRB, 0.14699E+01_JPRB, 0.12761E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.86380E+00_JPRB, 0.76341E+00_JPRB, 0.74721E+00_JPRB, 0.73948E+00_JPRB, 0.74819E+00_JPRB, 0.80227E+00_JPRB, 0.86982E+00_JPRB ,&
&  0.94207E+00_JPRB, 0.10313E+01_JPRB, 0.11124E+01_JPRB, 0.11904E+01_JPRB, 0.12771E+01_JPRB, 0.14241E+01_JPRB, 0.15168E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.12078E+01_JPRB, 0.11885E+01_JPRB, 0.11942E+01_JPRB, 0.12158E+01_JPRB, 0.12810E+01_JPRB, 0.13517E+01_JPRB ,&
&  0.14798E+01_JPRB, 0.15515E+01_JPRB, 0.13096E+01_JPRB, 0.10928E+01_JPRB, 0.13849E+01_JPRB, 0.14883E+01_JPRB, 0.83589E+00_JPRB ,&
&  0.81152E+00_JPRB, 0.83584E+00_JPRB, 0.86207E+00_JPRB, 0.91119E+00_JPRB, 0.96095E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.80581E+00_JPRB, 0.83147E+00_JPRB, 0.92643E+00_JPRB, 0.96250E+00_JPRB, 0.10621E+01_JPRB, 0.11144E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12158E+01_JPRB, 0.12303E+01_JPRB, 0.12506E+01_JPRB, 0.12758E+01_JPRB, 0.13095E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.13797E+01_JPRB, 0.14269E+01_JPRB, 0.14816E+01_JPRB, 0.15400E+01_JPRB, 0.16165E+01_JPRB, 0.17076E+01_JPRB, 0.18334E+01_JPRB ,&
&  0.19936E+01_JPRB, 0.21818E+01_JPRB, 0.23785E+01_JPRB, 0.25791E+01_JPRB, 0.27897E+01_JPRB, 0.29521E+01_JPRB, 0.30259E+01_JPRB ,&
&  0.20410E+01_JPRB, 0.25150E+01_JPRB, 0.24831E+01_JPRB, 0.23537E+01_JPRB, 0.21836E+01_JPRB, 0.20732E+01_JPRB, 0.20220E+01_JPRB ,&
&  0.19671E+01_JPRB, 0.18180E+01_JPRB, 0.17618E+01_JPRB, 0.16852E+01_JPRB, 0.16902E+01_JPRB, 0.17031E+01_JPRB, 0.18244E+01_JPRB ,&
&  0.19436E+01_JPRB, 0.21099E+01_JPRB, 0.23283E+01_JPRB, 0.25525E+01_JPRB, 0.29066E+01_JPRB, 0.33779E+01_JPRB, 0.38678E+01_JPRB ,&
&  0.43842E+01_JPRB, 0.49285E+01_JPRB, 0.52508E+01_JPRB, 0.46602E+01_JPRB, 0.40624E+01_JPRB, 0.38130E+01_JPRB, 0.27233E+01_JPRB ,&
&  0.21486E+01_JPRB, 0.18351E+01_JPRB, 0.19494E+01_JPRB, 0.19179E+01_JPRB, 0.16392E+01_JPRB, 0.14644E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.86138E+00_JPRB, 0.79918E+00_JPRB, 0.79824E+00_JPRB, 0.81700E+00_JPRB, 0.85943E+00_JPRB, 0.89635E+00_JPRB ,&
&  0.93993E+00_JPRB, 0.98771E+00_JPRB, 0.10294E+01_JPRB, 0.10742E+01_JPRB, 0.10959E+01_JPRB, 0.11160E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.11181E+01_JPRB, 0.11532E+01_JPRB, 0.11455E+01_JPRB, 0.11495E+01_JPRB, 0.11259E+01_JPRB, 0.11242E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.11561E+01_JPRB, 0.11646E+01_JPRB, 0.11433E+01_JPRB, 0.10996E+01_JPRB, 0.10179E+01_JPRB, 0.93456E+00_JPRB, 0.77675E+00_JPRB ,&
&  0.75522E+00_JPRB, 0.73819E+00_JPRB, 0.75489E+00_JPRB, 0.76854E+00_JPRB, 0.79221E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.69790E+00_JPRB, 0.72182E+00_JPRB, 0.74625E+00_JPRB, 0.77500E+00_JPRB, 0.82074E+00_JPRB, 0.87675E+00_JPRB, 0.95059E+00_JPRB ,&
&  0.10267E+01_JPRB, 0.10567E+01_JPRB, 0.10667E+01_JPRB, 0.10740E+01_JPRB, 0.10890E+01_JPRB, 0.11201E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.11957E+01_JPRB, 0.12303E+01_JPRB, 0.12633E+01_JPRB, 0.13019E+01_JPRB, 0.13500E+01_JPRB, 0.14214E+01_JPRB, 0.15099E+01_JPRB ,&
&  0.16226E+01_JPRB, 0.18042E+01_JPRB, 0.20440E+01_JPRB, 0.23223E+01_JPRB, 0.26250E+01_JPRB, 0.28732E+01_JPRB, 0.30012E+01_JPRB ,&
&  0.16694E+01_JPRB, 0.21552E+01_JPRB, 0.25889E+01_JPRB, 0.24260E+01_JPRB, 0.22481E+01_JPRB, 0.21503E+01_JPRB, 0.21001E+01_JPRB ,&
&  0.19696E+01_JPRB, 0.17984E+01_JPRB, 0.17245E+01_JPRB, 0.17546E+01_JPRB, 0.16026E+01_JPRB, 0.15936E+01_JPRB, 0.16579E+01_JPRB ,&
&  0.17503E+01_JPRB, 0.18500E+01_JPRB, 0.20168E+01_JPRB, 0.22449E+01_JPRB, 0.25681E+01_JPRB, 0.30569E+01_JPRB, 0.36164E+01_JPRB ,&
&  0.42247E+01_JPRB, 0.49336E+01_JPRB, 0.55716E+01_JPRB, 0.50733E+01_JPRB, 0.50130E+01_JPRB, 0.51634E+01_JPRB, 0.46592E+01_JPRB ,&
&  0.30005E+01_JPRB, 0.22045E+01_JPRB, 0.22679E+01_JPRB, 0.22390E+01_JPRB, 0.18754E+01_JPRB, 0.16947E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.11398E+01_JPRB, 0.93389E+00_JPRB, 0.88141E+00_JPRB, 0.89250E+00_JPRB, 0.92578E+00_JPRB, 0.95708E+00_JPRB ,&
&  0.97375E+00_JPRB, 0.98128E+00_JPRB, 0.99935E+00_JPRB, 0.10233E+01_JPRB, 0.10638E+01_JPRB, 0.10812E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.11147E+01_JPRB, 0.11304E+01_JPRB, 0.11549E+01_JPRB, 0.11777E+01_JPRB, 0.11851E+01_JPRB, 0.11732E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.11440E+01_JPRB, 0.11562E+01_JPRB, 0.11395E+01_JPRB, 0.10869E+01_JPRB, 0.97935E+00_JPRB, 0.89157E+00_JPRB ,&
&  0.80237E+00_JPRB, 0.78735E+00_JPRB, 0.75658E+00_JPRB, 0.72933E+00_JPRB, 0.70530E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.69035E+00_JPRB, 0.68610E+00_JPRB, 0.69410E+00_JPRB, 0.70115E+00_JPRB, 0.71709E+00_JPRB, 0.75011E+00_JPRB, 0.79162E+00_JPRB ,&
&  0.82736E+00_JPRB, 0.88807E+00_JPRB, 0.93158E+00_JPRB, 0.93407E+00_JPRB, 0.92914E+00_JPRB, 0.94429E+00_JPRB, 0.97613E+00_JPRB ,&
&  0.10176E+01_JPRB, 0.10548E+01_JPRB, 0.10816E+01_JPRB, 0.11057E+01_JPRB, 0.11363E+01_JPRB, 0.11682E+01_JPRB, 0.12097E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.13805E+01_JPRB, 0.15520E+01_JPRB, 0.18221E+01_JPRB, 0.21941E+01_JPRB, 0.26274E+01_JPRB, 0.29251E+01_JPRB ,&
&  0.20951E+01_JPRB, 0.10279E+01_JPRB, 0.22527E+01_JPRB, 0.26688E+01_JPRB, 0.24096E+01_JPRB, 0.23376E+01_JPRB, 0.21969E+01_JPRB ,&
&  0.20438E+01_JPRB, 0.19743E+01_JPRB, 0.18453E+01_JPRB, 0.17619E+01_JPRB, 0.15639E+01_JPRB, 0.15079E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.15310E+01_JPRB, 0.15874E+01_JPRB, 0.16996E+01_JPRB, 0.18698E+01_JPRB, 0.21140E+01_JPRB, 0.25113E+01_JPRB, 0.31232E+01_JPRB ,&
&  0.37877E+01_JPRB, 0.45917E+01_JPRB, 0.56416E+01_JPRB, 0.55452E+01_JPRB, 0.58044E+01_JPRB, 0.52977E+01_JPRB, 0.43995E+01_JPRB ,&
&  0.33774E+01_JPRB, 0.26584E+01_JPRB, 0.26519E+01_JPRB, 0.26545E+01_JPRB, 0.22175E+01_JPRB, 0.18537E+01_JPRB, 0.18247E+01_JPRB ,&
&  0.15577E+01_JPRB, 0.14358E+01_JPRB, 0.12066E+01_JPRB, 0.10454E+01_JPRB, 0.99509E+00_JPRB, 0.10146E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.10333E+01_JPRB, 0.10372E+01_JPRB, 0.10520E+01_JPRB, 0.10774E+01_JPRB, 0.10941E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.11430E+01_JPRB, 0.11659E+01_JPRB, 0.11885E+01_JPRB, 0.12047E+01_JPRB, 0.11845E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.11872E+01_JPRB, 0.11819E+01_JPRB, 0.11678E+01_JPRB, 0.11689E+01_JPRB, 0.11410E+01_JPRB, 0.10739E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.96813E+00_JPRB, 0.89347E+00_JPRB, 0.80244E+00_JPRB, 0.74474E+00_JPRB, 0.72162E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.81032E+00_JPRB, 0.76850E+00_JPRB, 0.72203E+00_JPRB, 0.70916E+00_JPRB, 0.70770E+00_JPRB, 0.70418E+00_JPRB, 0.72407E+00_JPRB ,&
&  0.74192E+00_JPRB, 0.76978E+00_JPRB, 0.78043E+00_JPRB, 0.80772E+00_JPRB, 0.81993E+00_JPRB, 0.82548E+00_JPRB, 0.84448E+00_JPRB ,&
&  0.86217E+00_JPRB, 0.88340E+00_JPRB, 0.90617E+00_JPRB, 0.93259E+00_JPRB, 0.95945E+00_JPRB, 0.97172E+00_JPRB, 0.98704E+00_JPRB ,&
&  0.10212E+01_JPRB, 0.10673E+01_JPRB, 0.11403E+01_JPRB, 0.12957E+01_JPRB, 0.15708E+01_JPRB, 0.19627E+01_JPRB, 0.24257E+01_JPRB ,&
&  0.28681E+01_JPRB, 0.20373E+01_JPRB, 0.14062E+01_JPRB, 0.24753E+01_JPRB, 0.27272E+01_JPRB, 0.26874E+01_JPRB, 0.22980E+01_JPRB ,&
&  0.23143E+01_JPRB, 0.22027E+01_JPRB, 0.20630E+01_JPRB, 0.18864E+01_JPRB, 0.16223E+01_JPRB, 0.14926E+01_JPRB, 0.14028E+01_JPRB ,&
&  0.13597E+01_JPRB, 0.13489E+01_JPRB, 0.14085E+01_JPRB, 0.15635E+01_JPRB, 0.17462E+01_JPRB, 0.20081E+01_JPRB, 0.25092E+01_JPRB ,&
&  0.31542E+01_JPRB, 0.39579E+01_JPRB, 0.52170E+01_JPRB, 0.58909E+01_JPRB, 0.67887E+01_JPRB, 0.57831E+01_JPRB, 0.46900E+01_JPRB ,&
&  0.36184E+01_JPRB, 0.34096E+01_JPRB, 0.34996E+01_JPRB, 0.31039E+01_JPRB, 0.24940E+01_JPRB, 0.22095E+01_JPRB, 0.21829E+01_JPRB ,&
&  0.18568E+01_JPRB, 0.16886E+01_JPRB, 0.15427E+01_JPRB, 0.13353E+01_JPRB, 0.11999E+01_JPRB, 0.11518E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.11011E+01_JPRB, 0.10872E+01_JPRB, 0.10920E+01_JPRB, 0.11024E+01_JPRB, 0.11092E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.11379E+01_JPRB, 0.11564E+01_JPRB, 0.11780E+01_JPRB, 0.11894E+01_JPRB, 0.11757E+01_JPRB, 0.11537E+01_JPRB, 0.11779E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.12085E+01_JPRB, 0.11923E+01_JPRB, 0.11915E+01_JPRB, 0.11727E+01_JPRB, 0.11605E+01_JPRB, 0.11465E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.10966E+01_JPRB, 0.10028E+01_JPRB, 0.88674E+00_JPRB, 0.85228E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.92457E+00_JPRB, 0.88975E+00_JPRB, 0.84676E+00_JPRB, 0.80308E+00_JPRB, 0.77936E+00_JPRB, 0.75543E+00_JPRB, 0.72626E+00_JPRB ,&
&  0.70977E+00_JPRB, 0.70164E+00_JPRB, 0.68838E+00_JPRB, 0.69915E+00_JPRB, 0.72759E+00_JPRB, 0.74343E+00_JPRB, 0.75726E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.78860E+00_JPRB, 0.78717E+00_JPRB, 0.79148E+00_JPRB, 0.80842E+00_JPRB, 0.82169E+00_JPRB, 0.82394E+00_JPRB ,&
&  0.83311E+00_JPRB, 0.85611E+00_JPRB, 0.88939E+00_JPRB, 0.95459E+00_JPRB, 0.10900E+01_JPRB, 0.13115E+01_JPRB, 0.16288E+01_JPRB ,&
&  0.21202E+01_JPRB, 0.21581E+01_JPRB, 0.11578E+01_JPRB, 0.20955E+01_JPRB, 0.28919E+01_JPRB, 0.29333E+01_JPRB, 0.27059E+01_JPRB ,&
&  0.26825E+01_JPRB, 0.25891E+01_JPRB, 0.24475E+01_JPRB, 0.20775E+01_JPRB, 0.18135E+01_JPRB, 0.16128E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.13116E+01_JPRB, 0.12218E+01_JPRB, 0.11950E+01_JPRB, 0.12710E+01_JPRB, 0.14203E+01_JPRB, 0.15692E+01_JPRB, 0.18568E+01_JPRB ,&
&  0.23663E+01_JPRB, 0.30297E+01_JPRB, 0.41223E+01_JPRB, 0.52723E+01_JPRB, 0.69972E+01_JPRB, 0.70203E+01_JPRB, 0.56575E+01_JPRB ,&
&  0.43926E+01_JPRB, 0.42852E+01_JPRB, 0.42241E+01_JPRB, 0.36825E+01_JPRB, 0.28195E+01_JPRB, 0.25539E+01_JPRB, 0.24978E+01_JPRB ,&
&  0.21733E+01_JPRB, 0.19190E+01_JPRB, 0.18016E+01_JPRB, 0.16685E+01_JPRB, 0.14801E+01_JPRB, 0.13191E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.11869E+01_JPRB, 0.11684E+01_JPRB, 0.11504E+01_JPRB, 0.11398E+01_JPRB, 0.11353E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.11665E+01_JPRB, 0.11811E+01_JPRB, 0.11755E+01_JPRB, 0.11682E+01_JPRB, 0.11639E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.11605E+01_JPRB, 0.11653E+01_JPRB, 0.11609E+01_JPRB, 0.11278E+01_JPRB, 0.11683E+01_JPRB, 0.11637E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.12001E+01_JPRB, 0.11285E+01_JPRB, 0.99213E+00_JPRB, 0.96024E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.96967E+00_JPRB, 0.94232E+00_JPRB, 0.91511E+00_JPRB, 0.89158E+00_JPRB, 0.87097E+00_JPRB, 0.84958E+00_JPRB, 0.81672E+00_JPRB ,&
&  0.77618E+00_JPRB, 0.74918E+00_JPRB, 0.71073E+00_JPRB, 0.69585E+00_JPRB, 0.69836E+00_JPRB, 0.70225E+00_JPRB, 0.71446E+00_JPRB ,&
&  0.72454E+00_JPRB, 0.73578E+00_JPRB, 0.74293E+00_JPRB, 0.74579E+00_JPRB, 0.74924E+00_JPRB, 0.75179E+00_JPRB, 0.75040E+00_JPRB ,&
&  0.74390E+00_JPRB, 0.74773E+00_JPRB, 0.76218E+00_JPRB, 0.78698E+00_JPRB, 0.84385E+00_JPRB, 0.95195E+00_JPRB, 0.11155E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.17584E+01_JPRB, 0.19112E+01_JPRB, 0.28924E+01_JPRB, 0.34664E+01_JPRB, 0.33381E+01_JPRB, 0.33232E+01_JPRB ,&
&  0.33958E+01_JPRB, 0.33435E+01_JPRB, 0.30470E+01_JPRB, 0.23321E+01_JPRB, 0.18824E+01_JPRB, 0.16532E+01_JPRB, 0.14796E+01_JPRB ,&
&  0.13245E+01_JPRB, 0.12069E+01_JPRB, 0.11184E+01_JPRB, 0.10915E+01_JPRB, 0.11437E+01_JPRB, 0.12628E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.16942E+01_JPRB, 0.21902E+01_JPRB, 0.29304E+01_JPRB, 0.39072E+01_JPRB, 0.56338E+01_JPRB, 0.69078E+01_JPRB, 0.65705E+01_JPRB ,&
&  0.61078E+01_JPRB, 0.55171E+01_JPRB, 0.48396E+01_JPRB, 0.42921E+01_JPRB, 0.31921E+01_JPRB, 0.30778E+01_JPRB, 0.27094E+01_JPRB ,&
&  0.23458E+01_JPRB, 0.20835E+01_JPRB, 0.19346E+01_JPRB, 0.18449E+01_JPRB, 0.17137E+01_JPRB, 0.15291E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.12571E+01_JPRB, 0.12422E+01_JPRB, 0.12229E+01_JPRB, 0.12053E+01_JPRB, 0.11933E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.11724E+01_JPRB, 0.11507E+01_JPRB, 0.11544E+01_JPRB, 0.11444E+01_JPRB, 0.11295E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.11438E+01_JPRB, 0.11476E+01_JPRB, 0.11383E+01_JPRB, 0.11316E+01_JPRB, 0.11263E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.11058E+01_JPRB, 0.10755E+01_JPRB, 0.10219E+01_JPRB, 0.10028E+01_JPRB, 0.98831E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.97885E+00_JPRB, 0.95577E+00_JPRB, 0.93928E+00_JPRB, 0.91967E+00_JPRB, 0.89943E+00_JPRB, 0.88570E+00_JPRB, 0.87286E+00_JPRB ,&
&  0.85176E+00_JPRB, 0.80791E+00_JPRB, 0.76258E+00_JPRB, 0.73724E+00_JPRB, 0.72549E+00_JPRB, 0.70709E+00_JPRB, 0.70537E+00_JPRB ,&
&  0.70811E+00_JPRB, 0.71689E+00_JPRB, 0.71687E+00_JPRB, 0.71764E+00_JPRB, 0.71867E+00_JPRB, 0.71993E+00_JPRB, 0.71664E+00_JPRB ,&
&  0.70589E+00_JPRB, 0.70018E+00_JPRB, 0.70434E+00_JPRB, 0.71398E+00_JPRB, 0.73681E+00_JPRB, 0.78825E+00_JPRB, 0.85900E+00_JPRB ,&
&  0.10991E+01_JPRB, 0.14211E+01_JPRB, 0.15044E+01_JPRB, 0.28475E+01_JPRB, 0.33722E+01_JPRB, 0.36089E+01_JPRB, 0.34431E+01_JPRB ,&
&  0.35185E+01_JPRB, 0.39232E+01_JPRB, 0.32881E+01_JPRB, 0.24009E+01_JPRB, 0.18108E+01_JPRB, 0.15388E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.11409E+01_JPRB, 0.10648E+01_JPRB, 0.10209E+01_JPRB, 0.10240E+01_JPRB, 0.10570E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.15915E+01_JPRB, 0.20945E+01_JPRB, 0.26016E+01_JPRB, 0.38720E+01_JPRB, 0.53207E+01_JPRB, 0.61558E+01_JPRB ,&
&  0.65175E+01_JPRB, 0.64058E+01_JPRB, 0.52424E+01_JPRB, 0.45263E+01_JPRB, 0.36749E+01_JPRB, 0.32175E+01_JPRB, 0.27571E+01_JPRB ,&
&  0.24368E+01_JPRB, 0.21863E+01_JPRB, 0.20048E+01_JPRB, 0.18987E+01_JPRB, 0.17936E+01_JPRB, 0.16652E+01_JPRB, 0.15054E+01_JPRB ,&
&  0.13693E+01_JPRB, 0.13000E+01_JPRB, 0.12686E+01_JPRB, 0.12516E+01_JPRB, 0.12466E+01_JPRB, 0.12369E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.12046E+01_JPRB, 0.11875E+01_JPRB, 0.11503E+01_JPRB, 0.11442E+01_JPRB, 0.11180E+01_JPRB, 0.10761E+01_JPRB, 0.10654E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.11361E+01_JPRB, 0.11352E+01_JPRB, 0.11068E+01_JPRB, 0.11155E+01_JPRB, 0.11304E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10420E+01_JPRB, 0.10162E+01_JPRB, 0.10023E+01_JPRB, 0.99361E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.97007E+00_JPRB, 0.93976E+00_JPRB, 0.91778E+00_JPRB, 0.89896E+00_JPRB, 0.88057E+00_JPRB, 0.86357E+00_JPRB, 0.85725E+00_JPRB ,&
&  0.84938E+00_JPRB, 0.83532E+00_JPRB, 0.80460E+00_JPRB, 0.76907E+00_JPRB, 0.74941E+00_JPRB, 0.73863E+00_JPRB, 0.71960E+00_JPRB ,&
&  0.71034E+00_JPRB, 0.71009E+00_JPRB, 0.70985E+00_JPRB, 0.70392E+00_JPRB, 0.69757E+00_JPRB, 0.69203E+00_JPRB, 0.68647E+00_JPRB ,&
&  0.67104E+00_JPRB, 0.66357E+00_JPRB, 0.66314E+00_JPRB, 0.66349E+00_JPRB, 0.67003E+00_JPRB, 0.70920E+00_JPRB, 0.78318E+00_JPRB ,&
&  0.92686E+00_JPRB, 0.15792E+01_JPRB, 0.14930E+01_JPRB, 0.23222E+01_JPRB, 0.26526E+01_JPRB, 0.30432E+01_JPRB, 0.32739E+01_JPRB ,&
&  0.35389E+01_JPRB, 0.38308E+01_JPRB, 0.31157E+01_JPRB, 0.23105E+01_JPRB, 0.16574E+01_JPRB, 0.13133E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.10256E+01_JPRB, 0.97695E+00_JPRB, 0.95863E+00_JPRB, 0.96069E+00_JPRB, 0.97149E+00_JPRB, 0.99054E+00_JPRB ,&
&  0.10619E+01_JPRB, 0.12466E+01_JPRB, 0.16738E+01_JPRB, 0.22831E+01_JPRB, 0.26814E+01_JPRB, 0.36604E+01_JPRB, 0.51177E+01_JPRB ,&
&  0.60366E+01_JPRB, 0.66289E+01_JPRB, 0.55029E+01_JPRB, 0.45739E+01_JPRB, 0.38644E+01_JPRB, 0.33169E+01_JPRB, 0.28517E+01_JPRB ,&
&  0.24942E+01_JPRB, 0.22307E+01_JPRB, 0.20396E+01_JPRB, 0.19047E+01_JPRB, 0.18092E+01_JPRB, 0.17344E+01_JPRB, 0.16272E+01_JPRB ,&
&  0.15034E+01_JPRB, 0.14069E+01_JPRB, 0.13436E+01_JPRB, 0.13083E+01_JPRB, 0.12971E+01_JPRB, 0.12772E+01_JPRB, 0.12391E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.11908E+01_JPRB, 0.11560E+01_JPRB, 0.11452E+01_JPRB, 0.11241E+01_JPRB, 0.10763E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.11165E+01_JPRB, 0.11201E+01_JPRB, 0.11172E+01_JPRB, 0.11172E+01_JPRB, 0.11598E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.10256E+01_JPRB, 0.10066E+01_JPRB, 0.99922E+00_JPRB, 0.98826E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.94710E+00_JPRB, 0.91589E+00_JPRB, 0.88996E+00_JPRB, 0.86402E+00_JPRB, 0.84387E+00_JPRB, 0.82251E+00_JPRB, 0.80811E+00_JPRB ,&
&  0.80070E+00_JPRB, 0.79942E+00_JPRB, 0.79516E+00_JPRB, 0.78043E+00_JPRB, 0.75665E+00_JPRB, 0.73718E+00_JPRB, 0.72278E+00_JPRB ,&
&  0.70885E+00_JPRB, 0.70397E+00_JPRB, 0.70138E+00_JPRB, 0.69104E+00_JPRB, 0.67951E+00_JPRB, 0.66817E+00_JPRB, 0.65596E+00_JPRB ,&
&  0.64445E+00_JPRB, 0.63835E+00_JPRB, 0.63415E+00_JPRB, 0.63028E+00_JPRB, 0.62806E+00_JPRB, 0.64946E+00_JPRB, 0.75611E+00_JPRB ,&
&  0.98717E+00_JPRB, 0.86868E+00_JPRB, 0.18571E+01_JPRB, 0.21229E+01_JPRB, 0.22150E+01_JPRB, 0.23787E+01_JPRB, 0.28030E+01_JPRB ,&
&  0.32419E+01_JPRB, 0.32131E+01_JPRB, 0.27762E+01_JPRB, 0.21696E+01_JPRB, 0.16353E+01_JPRB, 0.12895E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.99039E+00_JPRB, 0.92217E+00_JPRB, 0.89019E+00_JPRB, 0.87256E+00_JPRB, 0.87309E+00_JPRB, 0.88833E+00_JPRB, 0.90575E+00_JPRB ,&
&  0.94633E+00_JPRB, 0.10590E+01_JPRB, 0.13581E+01_JPRB, 0.19436E+01_JPRB, 0.25836E+01_JPRB, 0.30645E+01_JPRB, 0.39868E+01_JPRB ,&
&  0.52197E+01_JPRB, 0.56643E+01_JPRB, 0.47153E+01_JPRB, 0.39964E+01_JPRB, 0.35337E+01_JPRB, 0.31975E+01_JPRB, 0.28597E+01_JPRB ,&
&  0.25321E+01_JPRB, 0.22166E+01_JPRB, 0.19870E+01_JPRB, 0.18252E+01_JPRB, 0.17336E+01_JPRB, 0.16997E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.16238E+01_JPRB, 0.15559E+01_JPRB, 0.14863E+01_JPRB, 0.14225E+01_JPRB, 0.13737E+01_JPRB, 0.13286E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.11798E+01_JPRB, 0.11439E+01_JPRB, 0.11157E+01_JPRB, 0.11038E+01_JPRB, 0.10876E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10909E+01_JPRB, 0.10905E+01_JPRB, 0.10967E+01_JPRB, 0.10778E+01_JPRB, 0.11217E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.10691E+01_JPRB, 0.10364E+01_JPRB, 0.10027E+01_JPRB, 0.98399E+00_JPRB, 0.96995E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.91956E+00_JPRB, 0.89421E+00_JPRB, 0.85993E+00_JPRB, 0.82968E+00_JPRB, 0.80557E+00_JPRB, 0.78009E+00_JPRB, 0.76186E+00_JPRB ,&
&  0.74847E+00_JPRB, 0.74948E+00_JPRB, 0.75497E+00_JPRB, 0.75715E+00_JPRB, 0.74869E+00_JPRB, 0.72841E+00_JPRB, 0.70790E+00_JPRB ,&
&  0.69265E+00_JPRB, 0.68636E+00_JPRB, 0.68370E+00_JPRB, 0.67709E+00_JPRB, 0.66728E+00_JPRB, 0.65453E+00_JPRB, 0.64257E+00_JPRB ,&
&  0.63096E+00_JPRB, 0.62272E+00_JPRB, 0.61487E+00_JPRB, 0.60864E+00_JPRB, 0.60177E+00_JPRB, 0.61231E+00_JPRB, 0.71467E+00_JPRB ,&
&  0.10330E+01_JPRB, 0.14257E+01_JPRB, 0.18125E+01_JPRB, 0.17034E+01_JPRB, 0.16963E+01_JPRB, 0.17854E+01_JPRB, 0.20996E+01_JPRB ,&
&  0.22804E+01_JPRB, 0.23555E+01_JPRB, 0.22251E+01_JPRB, 0.19168E+01_JPRB, 0.15509E+01_JPRB, 0.12942E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.95804E+00_JPRB, 0.86543E+00_JPRB, 0.81464E+00_JPRB, 0.79945E+00_JPRB, 0.80485E+00_JPRB, 0.82031E+00_JPRB, 0.83311E+00_JPRB ,&
&  0.85544E+00_JPRB, 0.91722E+00_JPRB, 0.11377E+01_JPRB, 0.16608E+01_JPRB, 0.25640E+01_JPRB, 0.26862E+01_JPRB, 0.36734E+01_JPRB ,&
&  0.46624E+01_JPRB, 0.42394E+01_JPRB, 0.38411E+01_JPRB, 0.34969E+01_JPRB, 0.32274E+01_JPRB, 0.29879E+01_JPRB, 0.26104E+01_JPRB ,&
&  0.23069E+01_JPRB, 0.20226E+01_JPRB, 0.18073E+01_JPRB, 0.16809E+01_JPRB, 0.16025E+01_JPRB, 0.15716E+01_JPRB, 0.15667E+01_JPRB ,&
&  0.15582E+01_JPRB, 0.15370E+01_JPRB, 0.15154E+01_JPRB, 0.14692E+01_JPRB, 0.14058E+01_JPRB, 0.13426E+01_JPRB, 0.12777E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.11651E+01_JPRB, 0.11131E+01_JPRB, 0.10687E+01_JPRB, 0.10597E+01_JPRB, 0.10613E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10283E+01_JPRB, 0.10420E+01_JPRB, 0.10388E+01_JPRB, 0.10345E+01_JPRB, 0.10842E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.10174E+01_JPRB, 0.98438E+00_JPRB, 0.95683E+00_JPRB, 0.93937E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.89618E+00_JPRB, 0.86461E+00_JPRB, 0.82999E+00_JPRB, 0.79812E+00_JPRB, 0.76919E+00_JPRB, 0.74678E+00_JPRB, 0.73236E+00_JPRB ,&
&  0.71979E+00_JPRB, 0.71277E+00_JPRB, 0.71611E+00_JPRB, 0.72284E+00_JPRB, 0.72209E+00_JPRB, 0.71038E+00_JPRB, 0.69286E+00_JPRB ,&
&  0.67468E+00_JPRB, 0.66046E+00_JPRB, 0.65015E+00_JPRB, 0.64552E+00_JPRB, 0.63933E+00_JPRB, 0.63215E+00_JPRB, 0.62040E+00_JPRB ,&
&  0.61052E+00_JPRB, 0.60261E+00_JPRB, 0.59168E+00_JPRB, 0.58032E+00_JPRB, 0.57176E+00_JPRB, 0.57559E+00_JPRB, 0.65319E+00_JPRB ,&
&  0.89614E+00_JPRB, 0.12349E+01_JPRB, 0.12668E+01_JPRB, 0.13046E+01_JPRB, 0.14193E+01_JPRB, 0.14544E+01_JPRB, 0.15756E+01_JPRB ,&
&  0.16379E+01_JPRB, 0.16754E+01_JPRB, 0.16895E+01_JPRB, 0.15891E+01_JPRB, 0.13900E+01_JPRB, 0.12330E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.96439E+00_JPRB, 0.86625E+00_JPRB, 0.80454E+00_JPRB, 0.77839E+00_JPRB, 0.77755E+00_JPRB, 0.78762E+00_JPRB, 0.79285E+00_JPRB ,&
&  0.79920E+00_JPRB, 0.81033E+00_JPRB, 0.88906E+00_JPRB, 0.10905E+01_JPRB, 0.16053E+01_JPRB, 0.19883E+01_JPRB, 0.25582E+01_JPRB ,&
&  0.26766E+01_JPRB, 0.26462E+01_JPRB, 0.25815E+01_JPRB, 0.25136E+01_JPRB, 0.24185E+01_JPRB, 0.22607E+01_JPRB, 0.21188E+01_JPRB ,&
&  0.19400E+01_JPRB, 0.17511E+01_JPRB, 0.16007E+01_JPRB, 0.14979E+01_JPRB, 0.14314E+01_JPRB, 0.13904E+01_JPRB, 0.13740E+01_JPRB ,&
&  0.13511E+01_JPRB, 0.13439E+01_JPRB, 0.13219E+01_JPRB, 0.13026E+01_JPRB, 0.12878E+01_JPRB, 0.12495E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.10712E+01_JPRB, 0.10275E+01_JPRB, 0.10009E+01_JPRB, 0.98560E+00_JPRB, 0.97375E+00_JPRB, 0.96258E+00_JPRB ,&
&  0.96470E+00_JPRB, 0.96608E+00_JPRB, 0.96929E+00_JPRB, 0.96379E+00_JPRB, 0.95310E+00_JPRB, 0.99039E+00_JPRB, 0.98163E+00_JPRB ,&
&  0.97288E+00_JPRB, 0.96220E+00_JPRB, 0.94313E+00_JPRB, 0.91839E+00_JPRB, 0.91578E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.88392E+00_JPRB, 0.85624E+00_JPRB, 0.81989E+00_JPRB, 0.78442E+00_JPRB, 0.75165E+00_JPRB, 0.72577E+00_JPRB, 0.71023E+00_JPRB ,&
&  0.69914E+00_JPRB, 0.69203E+00_JPRB, 0.68879E+00_JPRB, 0.68704E+00_JPRB, 0.68405E+00_JPRB, 0.68012E+00_JPRB, 0.67290E+00_JPRB ,&
&  0.66289E+00_JPRB, 0.65039E+00_JPRB, 0.63895E+00_JPRB, 0.63067E+00_JPRB, 0.62437E+00_JPRB, 0.61988E+00_JPRB, 0.61317E+00_JPRB ,&
&  0.60164E+00_JPRB, 0.58850E+00_JPRB, 0.57261E+00_JPRB, 0.55677E+00_JPRB, 0.54614E+00_JPRB, 0.54639E+00_JPRB, 0.59538E+00_JPRB ,&
&  0.76875E+00_JPRB, 0.86878E+00_JPRB, 0.95858E+00_JPRB, 0.10256E+01_JPRB, 0.11197E+01_JPRB, 0.11791E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.12358E+01_JPRB, 0.12633E+01_JPRB, 0.13156E+01_JPRB, 0.13067E+01_JPRB, 0.12072E+01_JPRB, 0.11089E+01_JPRB, 0.10173E+01_JPRB ,&
&  0.93529E+00_JPRB, 0.86426E+00_JPRB, 0.80627E+00_JPRB, 0.77940E+00_JPRB, 0.77110E+00_JPRB, 0.77320E+00_JPRB, 0.77228E+00_JPRB ,&
&  0.77128E+00_JPRB, 0.76223E+00_JPRB, 0.76380E+00_JPRB, 0.79214E+00_JPRB, 0.87352E+00_JPRB, 0.10410E+01_JPRB, 0.13152E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.14908E+01_JPRB, 0.15376E+01_JPRB, 0.15836E+01_JPRB, 0.15622E+01_JPRB, 0.15218E+01_JPRB, 0.14818E+01_JPRB ,&
&  0.14366E+01_JPRB, 0.13852E+01_JPRB, 0.13373E+01_JPRB, 0.12765E+01_JPRB, 0.12235E+01_JPRB, 0.11850E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.11359E+01_JPRB, 0.11273E+01_JPRB, 0.11124E+01_JPRB, 0.10816E+01_JPRB, 0.10590E+01_JPRB, 0.10350E+01_JPRB, 0.99817E+00_JPRB ,&
&  0.94563E+00_JPRB, 0.91145E+00_JPRB, 0.88947E+00_JPRB, 0.88332E+00_JPRB, 0.88355E+00_JPRB, 0.88561E+00_JPRB, 0.88130E+00_JPRB ,&
&  0.88124E+00_JPRB, 0.89283E+00_JPRB, 0.88198E+00_JPRB, 0.87280E+00_JPRB, 0.90822E+00_JPRB, 0.90605E+00_JPRB, 0.90453E+00_JPRB ,&
&  0.90071E+00_JPRB, 0.89273E+00_JPRB, 0.87821E+00_JPRB, 0.85339E+00_JPRB, 0.88548E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.76446E+00_JPRB, 0.76321E+00_JPRB, 0.74433E+00_JPRB, 0.72267E+00_JPRB, 0.70310E+00_JPRB, 0.68849E+00_JPRB, 0.67675E+00_JPRB ,&
&  0.66822E+00_JPRB, 0.66153E+00_JPRB, 0.65740E+00_JPRB, 0.65483E+00_JPRB, 0.65094E+00_JPRB, 0.64497E+00_JPRB, 0.63826E+00_JPRB ,&
&  0.63213E+00_JPRB, 0.62448E+00_JPRB, 0.61300E+00_JPRB, 0.60466E+00_JPRB, 0.59965E+00_JPRB, 0.59295E+00_JPRB, 0.58554E+00_JPRB ,&
&  0.57642E+00_JPRB, 0.55993E+00_JPRB, 0.54339E+00_JPRB, 0.53177E+00_JPRB, 0.52239E+00_JPRB, 0.51977E+00_JPRB, 0.55193E+00_JPRB ,&
&  0.66004E+00_JPRB, 0.66637E+00_JPRB, 0.74284E+00_JPRB, 0.81974E+00_JPRB, 0.91504E+00_JPRB, 0.96864E+00_JPRB, 0.98951E+00_JPRB ,&
&  0.99952E+00_JPRB, 0.10319E+01_JPRB, 0.10585E+01_JPRB, 0.10948E+01_JPRB, 0.10551E+01_JPRB, 0.98605E+00_JPRB, 0.92692E+00_JPRB ,&
&  0.87168E+00_JPRB, 0.82538E+00_JPRB, 0.78716E+00_JPRB, 0.76105E+00_JPRB, 0.75059E+00_JPRB, 0.74620E+00_JPRB, 0.73700E+00_JPRB ,&
&  0.73128E+00_JPRB, 0.72017E+00_JPRB, 0.71193E+00_JPRB, 0.71360E+00_JPRB, 0.70710E+00_JPRB, 0.74508E+00_JPRB, 0.81066E+00_JPRB ,&
&  0.87381E+00_JPRB, 0.96096E+00_JPRB, 0.10096E+01_JPRB, 0.10430E+01_JPRB, 0.10471E+01_JPRB, 0.10489E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10440E+01_JPRB, 0.10338E+01_JPRB, 0.10124E+01_JPRB, 0.98500E+00_JPRB, 0.95926E+00_JPRB, 0.93678E+00_JPRB ,&
&  0.91811E+00_JPRB, 0.90326E+00_JPRB, 0.89318E+00_JPRB, 0.87083E+00_JPRB, 0.83557E+00_JPRB, 0.80478E+00_JPRB, 0.78054E+00_JPRB ,&
&  0.75907E+00_JPRB, 0.74911E+00_JPRB, 0.74689E+00_JPRB, 0.75465E+00_JPRB, 0.76256E+00_JPRB, 0.77024E+00_JPRB, 0.77377E+00_JPRB ,&
&  0.77613E+00_JPRB, 0.78596E+00_JPRB, 0.80341E+00_JPRB, 0.82439E+00_JPRB, 0.83355E+00_JPRB, 0.83842E+00_JPRB, 0.83819E+00_JPRB ,&
&  0.83398E+00_JPRB, 0.82738E+00_JPRB, 0.81085E+00_JPRB, 0.78951E+00_JPRB, 0.75444E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.69095E+00_JPRB, 0.69046E+00_JPRB, 0.68048E+00_JPRB, 0.66579E+00_JPRB, 0.65359E+00_JPRB, 0.64320E+00_JPRB, 0.63521E+00_JPRB ,&
&  0.62327E+00_JPRB, 0.61273E+00_JPRB, 0.60902E+00_JPRB, 0.60902E+00_JPRB, 0.60818E+00_JPRB, 0.60365E+00_JPRB, 0.59746E+00_JPRB ,&
&  0.59013E+00_JPRB, 0.58184E+00_JPRB, 0.57205E+00_JPRB, 0.56219E+00_JPRB, 0.55412E+00_JPRB, 0.54457E+00_JPRB, 0.53633E+00_JPRB ,&
&  0.52827E+00_JPRB, 0.52064E+00_JPRB, 0.50936E+00_JPRB, 0.49620E+00_JPRB, 0.48975E+00_JPRB, 0.48429E+00_JPRB, 0.51211E+00_JPRB ,&
&  0.60170E+00_JPRB, 0.53208E+00_JPRB, 0.58358E+00_JPRB, 0.66653E+00_JPRB, 0.75600E+00_JPRB, 0.84167E+00_JPRB, 0.86990E+00_JPRB ,&
&  0.88154E+00_JPRB, 0.89449E+00_JPRB, 0.90329E+00_JPRB, 0.91988E+00_JPRB, 0.92426E+00_JPRB, 0.88730E+00_JPRB, 0.83620E+00_JPRB ,&
&  0.79559E+00_JPRB, 0.76019E+00_JPRB, 0.73752E+00_JPRB, 0.71597E+00_JPRB, 0.70455E+00_JPRB, 0.69856E+00_JPRB, 0.68528E+00_JPRB ,&
&  0.67282E+00_JPRB, 0.67140E+00_JPRB, 0.67392E+00_JPRB, 0.68096E+00_JPRB, 0.68612E+00_JPRB, 0.69044E+00_JPRB, 0.68053E+00_JPRB ,&
&  0.67133E+00_JPRB, 0.69613E+00_JPRB, 0.74106E+00_JPRB, 0.76803E+00_JPRB, 0.78576E+00_JPRB, 0.78785E+00_JPRB, 0.78238E+00_JPRB ,&
&  0.78088E+00_JPRB, 0.78122E+00_JPRB, 0.78576E+00_JPRB, 0.78295E+00_JPRB, 0.77528E+00_JPRB, 0.75953E+00_JPRB, 0.73956E+00_JPRB ,&
&  0.71990E+00_JPRB, 0.70252E+00_JPRB, 0.68860E+00_JPRB, 0.67585E+00_JPRB, 0.66078E+00_JPRB, 0.64154E+00_JPRB, 0.63028E+00_JPRB ,&
&  0.62426E+00_JPRB, 0.62368E+00_JPRB, 0.62748E+00_JPRB, 0.63510E+00_JPRB, 0.64643E+00_JPRB, 0.65691E+00_JPRB, 0.66984E+00_JPRB ,&
&  0.68071E+00_JPRB, 0.69152E+00_JPRB, 0.70270E+00_JPRB, 0.70742E+00_JPRB, 0.74180E+00_JPRB, 0.75169E+00_JPRB, 0.76070E+00_JPRB ,&
&  0.76194E+00_JPRB, 0.76019E+00_JPRB, 0.75319E+00_JPRB, 0.66687E+00_JPRB, 0.68918E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.62814E+00_JPRB, 0.62560E+00_JPRB, 0.61814E+00_JPRB, 0.60967E+00_JPRB, 0.60283E+00_JPRB, 0.59591E+00_JPRB, 0.59036E+00_JPRB ,&
&  0.58218E+00_JPRB, 0.57183E+00_JPRB, 0.56436E+00_JPRB, 0.56151E+00_JPRB, 0.56039E+00_JPRB, 0.55726E+00_JPRB, 0.55136E+00_JPRB ,&
&  0.54367E+00_JPRB, 0.53512E+00_JPRB, 0.52675E+00_JPRB, 0.51657E+00_JPRB, 0.50717E+00_JPRB, 0.49829E+00_JPRB, 0.49049E+00_JPRB ,&
&  0.48264E+00_JPRB, 0.47424E+00_JPRB, 0.46598E+00_JPRB, 0.45986E+00_JPRB, 0.45526E+00_JPRB, 0.45374E+00_JPRB, 0.47657E+00_JPRB ,&
&  0.54705E+00_JPRB, 0.47197E+00_JPRB, 0.50801E+00_JPRB, 0.55834E+00_JPRB, 0.63756E+00_JPRB, 0.73432E+00_JPRB, 0.77841E+00_JPRB ,&
&  0.79544E+00_JPRB, 0.80353E+00_JPRB, 0.80213E+00_JPRB, 0.80024E+00_JPRB, 0.80731E+00_JPRB, 0.80070E+00_JPRB, 0.77296E+00_JPRB ,&
&  0.74100E+00_JPRB, 0.71123E+00_JPRB, 0.69035E+00_JPRB, 0.67484E+00_JPRB, 0.65988E+00_JPRB, 0.64664E+00_JPRB, 0.62962E+00_JPRB ,&
&  0.61099E+00_JPRB, 0.60516E+00_JPRB, 0.60553E+00_JPRB, 0.60468E+00_JPRB, 0.60447E+00_JPRB, 0.59934E+00_JPRB, 0.58006E+00_JPRB ,&
&  0.56165E+00_JPRB, 0.55002E+00_JPRB, 0.54839E+00_JPRB, 0.55785E+00_JPRB, 0.56974E+00_JPRB, 0.57827E+00_JPRB, 0.57889E+00_JPRB ,&
&  0.57528E+00_JPRB, 0.56780E+00_JPRB, 0.56500E+00_JPRB, 0.56471E+00_JPRB, 0.56413E+00_JPRB, 0.56342E+00_JPRB, 0.55907E+00_JPRB ,&
&  0.55267E+00_JPRB, 0.54933E+00_JPRB, 0.54980E+00_JPRB, 0.55093E+00_JPRB, 0.55016E+00_JPRB, 0.54709E+00_JPRB, 0.54343E+00_JPRB ,&
&  0.53833E+00_JPRB, 0.53479E+00_JPRB, 0.53575E+00_JPRB, 0.54198E+00_JPRB, 0.55120E+00_JPRB, 0.56133E+00_JPRB, 0.57248E+00_JPRB ,&
&  0.58461E+00_JPRB, 0.59692E+00_JPRB, 0.60925E+00_JPRB, 0.63091E+00_JPRB, 0.64195E+00_JPRB, 0.65577E+00_JPRB, 0.66954E+00_JPRB ,&
&  0.67220E+00_JPRB, 0.66851E+00_JPRB, 0.57665E+00_JPRB, 0.61446E+00_JPRB, 0.62293E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.58568E+00_JPRB, 0.57506E+00_JPRB, 0.56277E+00_JPRB, 0.55476E+00_JPRB, 0.55171E+00_JPRB, 0.54834E+00_JPRB, 0.54504E+00_JPRB ,&
&  0.53952E+00_JPRB, 0.53327E+00_JPRB, 0.52698E+00_JPRB, 0.52141E+00_JPRB, 0.51686E+00_JPRB, 0.51374E+00_JPRB, 0.50945E+00_JPRB ,&
&  0.50283E+00_JPRB, 0.49494E+00_JPRB, 0.48649E+00_JPRB, 0.47681E+00_JPRB, 0.46435E+00_JPRB, 0.45420E+00_JPRB, 0.44614E+00_JPRB ,&
&  0.43984E+00_JPRB, 0.43485E+00_JPRB, 0.43253E+00_JPRB, 0.42991E+00_JPRB, 0.42715E+00_JPRB, 0.42710E+00_JPRB, 0.44376E+00_JPRB ,&
&  0.45841E+00_JPRB, 0.43142E+00_JPRB, 0.45833E+00_JPRB, 0.49584E+00_JPRB, 0.55629E+00_JPRB, 0.65076E+00_JPRB, 0.71075E+00_JPRB ,&
&  0.74102E+00_JPRB, 0.75340E+00_JPRB, 0.75473E+00_JPRB, 0.74460E+00_JPRB, 0.73522E+00_JPRB, 0.73637E+00_JPRB, 0.73056E+00_JPRB ,&
&  0.70986E+00_JPRB, 0.68230E+00_JPRB, 0.65698E+00_JPRB, 0.63802E+00_JPRB, 0.61603E+00_JPRB, 0.59673E+00_JPRB, 0.57675E+00_JPRB ,&
&  0.55942E+00_JPRB, 0.54969E+00_JPRB, 0.54588E+00_JPRB, 0.54291E+00_JPRB, 0.53908E+00_JPRB, 0.52888E+00_JPRB, 0.51380E+00_JPRB ,&
&  0.50150E+00_JPRB, 0.49039E+00_JPRB, 0.47981E+00_JPRB, 0.47112E+00_JPRB, 0.46542E+00_JPRB, 0.46193E+00_JPRB, 0.45945E+00_JPRB ,&
&  0.45976E+00_JPRB, 0.46189E+00_JPRB, 0.46604E+00_JPRB, 0.47023E+00_JPRB, 0.47263E+00_JPRB, 0.47231E+00_JPRB, 0.47155E+00_JPRB ,&
&  0.47061E+00_JPRB, 0.46694E+00_JPRB, 0.46666E+00_JPRB, 0.47215E+00_JPRB, 0.47957E+00_JPRB, 0.48425E+00_JPRB, 0.48576E+00_JPRB ,&
&  0.48378E+00_JPRB, 0.48161E+00_JPRB, 0.48138E+00_JPRB, 0.48387E+00_JPRB, 0.48789E+00_JPRB, 0.49410E+00_JPRB, 0.50164E+00_JPRB ,&
&  0.50934E+00_JPRB, 0.51920E+00_JPRB, 0.53239E+00_JPRB, 0.55096E+00_JPRB, 0.56493E+00_JPRB, 0.57610E+00_JPRB, 0.58601E+00_JPRB ,&
&  0.58860E+00_JPRB, 0.58947E+00_JPRB, 0.59389E+00_JPRB, 0.59295E+00_JPRB, 0.59106E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.52076E+00_JPRB, 0.51462E+00_JPRB, 0.50710E+00_JPRB, 0.50065E+00_JPRB, 0.49728E+00_JPRB, 0.49475E+00_JPRB, 0.49057E+00_JPRB ,&
&  0.48709E+00_JPRB, 0.48408E+00_JPRB, 0.48088E+00_JPRB, 0.47817E+00_JPRB, 0.47579E+00_JPRB, 0.47389E+00_JPRB, 0.47036E+00_JPRB ,&
&  0.46414E+00_JPRB, 0.45611E+00_JPRB, 0.44694E+00_JPRB, 0.43897E+00_JPRB, 0.42928E+00_JPRB, 0.41924E+00_JPRB, 0.41162E+00_JPRB ,&
&  0.40535E+00_JPRB, 0.40102E+00_JPRB, 0.39918E+00_JPRB, 0.39686E+00_JPRB, 0.39735E+00_JPRB, 0.40022E+00_JPRB, 0.41100E+00_JPRB ,&
&  0.43588E+00_JPRB, 0.41916E+00_JPRB, 0.44555E+00_JPRB, 0.47029E+00_JPRB, 0.52001E+00_JPRB, 0.59535E+00_JPRB, 0.64414E+00_JPRB ,&
&  0.67383E+00_JPRB, 0.69741E+00_JPRB, 0.71345E+00_JPRB, 0.71396E+00_JPRB, 0.70118E+00_JPRB, 0.69501E+00_JPRB, 0.69180E+00_JPRB ,&
&  0.67967E+00_JPRB, 0.65809E+00_JPRB, 0.62837E+00_JPRB, 0.59674E+00_JPRB, 0.56861E+00_JPRB, 0.54744E+00_JPRB, 0.53145E+00_JPRB ,&
&  0.51927E+00_JPRB, 0.50950E+00_JPRB, 0.50213E+00_JPRB, 0.49065E+00_JPRB, 0.47729E+00_JPRB, 0.46372E+00_JPRB, 0.45028E+00_JPRB ,&
&  0.43687E+00_JPRB, 0.42399E+00_JPRB, 0.41299E+00_JPRB, 0.40409E+00_JPRB, 0.39881E+00_JPRB, 0.39582E+00_JPRB, 0.39420E+00_JPRB ,&
&  0.39397E+00_JPRB, 0.39580E+00_JPRB, 0.39911E+00_JPRB, 0.40337E+00_JPRB, 0.40719E+00_JPRB, 0.40853E+00_JPRB, 0.40652E+00_JPRB ,&
&  0.40541E+00_JPRB, 0.40476E+00_JPRB, 0.40563E+00_JPRB, 0.40902E+00_JPRB, 0.41166E+00_JPRB, 0.41472E+00_JPRB, 0.41918E+00_JPRB ,&
&  0.42274E+00_JPRB, 0.42429E+00_JPRB, 0.42514E+00_JPRB, 0.42629E+00_JPRB, 0.42795E+00_JPRB, 0.43108E+00_JPRB, 0.43611E+00_JPRB ,&
&  0.44218E+00_JPRB, 0.44911E+00_JPRB, 0.45935E+00_JPRB, 0.47281E+00_JPRB, 0.48542E+00_JPRB, 0.49604E+00_JPRB, 0.50498E+00_JPRB ,&
&  0.50899E+00_JPRB, 0.51227E+00_JPRB, 0.51810E+00_JPRB, 0.52121E+00_JPRB, 0.52335E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.46788E+00_JPRB, 0.46498E+00_JPRB, 0.46138E+00_JPRB, 0.45810E+00_JPRB, 0.45549E+00_JPRB, 0.45110E+00_JPRB, 0.44613E+00_JPRB ,&
&  0.44324E+00_JPRB, 0.44086E+00_JPRB, 0.43941E+00_JPRB, 0.43668E+00_JPRB, 0.43548E+00_JPRB, 0.43493E+00_JPRB, 0.43091E+00_JPRB ,&
&  0.42577E+00_JPRB, 0.41805E+00_JPRB, 0.41030E+00_JPRB, 0.40309E+00_JPRB, 0.39609E+00_JPRB, 0.38943E+00_JPRB, 0.38208E+00_JPRB ,&
&  0.37521E+00_JPRB, 0.37002E+00_JPRB, 0.36694E+00_JPRB, 0.36441E+00_JPRB, 0.36558E+00_JPRB, 0.37160E+00_JPRB, 0.38156E+00_JPRB ,&
&  0.39304E+00_JPRB, 0.40909E+00_JPRB, 0.42708E+00_JPRB, 0.44612E+00_JPRB, 0.48394E+00_JPRB, 0.53895E+00_JPRB, 0.57917E+00_JPRB ,&
&  0.59928E+00_JPRB, 0.62222E+00_JPRB, 0.65150E+00_JPRB, 0.67088E+00_JPRB, 0.67254E+00_JPRB, 0.66263E+00_JPRB, 0.65490E+00_JPRB ,&
&  0.64442E+00_JPRB, 0.62424E+00_JPRB, 0.60188E+00_JPRB, 0.57604E+00_JPRB, 0.55010E+00_JPRB, 0.52886E+00_JPRB, 0.51241E+00_JPRB ,&
&  0.49832E+00_JPRB, 0.48561E+00_JPRB, 0.47349E+00_JPRB, 0.46044E+00_JPRB, 0.44610E+00_JPRB, 0.43014E+00_JPRB, 0.41307E+00_JPRB ,&
&  0.39682E+00_JPRB, 0.38341E+00_JPRB, 0.37337E+00_JPRB, 0.36772E+00_JPRB, 0.36422E+00_JPRB, 0.36107E+00_JPRB, 0.35872E+00_JPRB ,&
&  0.35824E+00_JPRB, 0.35918E+00_JPRB, 0.36130E+00_JPRB, 0.36431E+00_JPRB, 0.36742E+00_JPRB, 0.36896E+00_JPRB, 0.36685E+00_JPRB ,&
&  0.36419E+00_JPRB, 0.36423E+00_JPRB, 0.36526E+00_JPRB, 0.36333E+00_JPRB, 0.36078E+00_JPRB, 0.36187E+00_JPRB, 0.36538E+00_JPRB ,&
&  0.37107E+00_JPRB, 0.37609E+00_JPRB, 0.37983E+00_JPRB, 0.38269E+00_JPRB, 0.38467E+00_JPRB, 0.38810E+00_JPRB, 0.39241E+00_JPRB ,&
&  0.39798E+00_JPRB, 0.40503E+00_JPRB, 0.41486E+00_JPRB, 0.42708E+00_JPRB, 0.43812E+00_JPRB, 0.44728E+00_JPRB, 0.45269E+00_JPRB ,&
&  0.45604E+00_JPRB, 0.45834E+00_JPRB, 0.46063E+00_JPRB, 0.46402E+00_JPRB, 0.46721E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.41691E+00_JPRB, 0.41692E+00_JPRB, 0.41748E+00_JPRB, 0.41721E+00_JPRB, 0.41362E+00_JPRB, 0.41002E+00_JPRB, 0.40607E+00_JPRB ,&
&  0.40422E+00_JPRB, 0.40306E+00_JPRB, 0.40292E+00_JPRB, 0.40020E+00_JPRB, 0.39832E+00_JPRB, 0.39749E+00_JPRB, 0.39394E+00_JPRB ,&
&  0.38997E+00_JPRB, 0.38122E+00_JPRB, 0.37193E+00_JPRB, 0.36517E+00_JPRB, 0.36090E+00_JPRB, 0.35771E+00_JPRB, 0.35150E+00_JPRB ,&
&  0.34668E+00_JPRB, 0.34280E+00_JPRB, 0.34073E+00_JPRB, 0.34053E+00_JPRB, 0.34276E+00_JPRB, 0.34900E+00_JPRB, 0.35912E+00_JPRB ,&
&  0.36722E+00_JPRB, 0.38381E+00_JPRB, 0.38723E+00_JPRB, 0.40743E+00_JPRB, 0.43128E+00_JPRB, 0.46720E+00_JPRB, 0.50228E+00_JPRB ,&
&  0.52210E+00_JPRB, 0.53478E+00_JPRB, 0.55527E+00_JPRB, 0.58000E+00_JPRB, 0.59212E+00_JPRB, 0.59479E+00_JPRB, 0.59045E+00_JPRB ,&
&  0.58322E+00_JPRB, 0.57475E+00_JPRB, 0.56497E+00_JPRB, 0.55191E+00_JPRB, 0.53402E+00_JPRB, 0.51437E+00_JPRB, 0.49561E+00_JPRB ,&
&  0.47871E+00_JPRB, 0.46233E+00_JPRB, 0.44600E+00_JPRB, 0.42812E+00_JPRB, 0.40973E+00_JPRB, 0.39211E+00_JPRB, 0.37498E+00_JPRB ,&
&  0.36117E+00_JPRB, 0.35079E+00_JPRB, 0.34322E+00_JPRB, 0.33758E+00_JPRB, 0.33315E+00_JPRB, 0.32894E+00_JPRB, 0.32660E+00_JPRB ,&
&  0.32602E+00_JPRB, 0.32732E+00_JPRB, 0.33003E+00_JPRB, 0.33250E+00_JPRB, 0.33551E+00_JPRB, 0.33607E+00_JPRB, 0.33246E+00_JPRB ,&
&  0.32899E+00_JPRB, 0.32838E+00_JPRB, 0.32450E+00_JPRB, 0.32223E+00_JPRB, 0.32188E+00_JPRB, 0.32268E+00_JPRB, 0.32519E+00_JPRB ,&
&  0.32989E+00_JPRB, 0.33606E+00_JPRB, 0.34181E+00_JPRB, 0.34693E+00_JPRB, 0.35157E+00_JPRB, 0.35607E+00_JPRB, 0.36107E+00_JPRB ,&
&  0.36628E+00_JPRB, 0.37127E+00_JPRB, 0.37883E+00_JPRB, 0.38730E+00_JPRB, 0.39457E+00_JPRB, 0.40246E+00_JPRB, 0.40723E+00_JPRB ,&
&  0.40911E+00_JPRB, 0.40967E+00_JPRB, 0.41161E+00_JPRB, 0.41472E+00_JPRB, 0.41671E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.37425E+00_JPRB, 0.37591E+00_JPRB, 0.37795E+00_JPRB, 0.37804E+00_JPRB, 0.37704E+00_JPRB, 0.37661E+00_JPRB, 0.37523E+00_JPRB ,&
&  0.37261E+00_JPRB, 0.37107E+00_JPRB, 0.37121E+00_JPRB, 0.36973E+00_JPRB, 0.36685E+00_JPRB, 0.36337E+00_JPRB, 0.35958E+00_JPRB ,&
&  0.35636E+00_JPRB, 0.35287E+00_JPRB, 0.34682E+00_JPRB, 0.33908E+00_JPRB, 0.33244E+00_JPRB, 0.32806E+00_JPRB, 0.32553E+00_JPRB ,&
&  0.32419E+00_JPRB, 0.32375E+00_JPRB, 0.32308E+00_JPRB, 0.32355E+00_JPRB, 0.32571E+00_JPRB, 0.33087E+00_JPRB, 0.33851E+00_JPRB ,&
&  0.34604E+00_JPRB, 0.35442E+00_JPRB, 0.36307E+00_JPRB, 0.37292E+00_JPRB, 0.38536E+00_JPRB, 0.40287E+00_JPRB, 0.42269E+00_JPRB ,&
&  0.44354E+00_JPRB, 0.45116E+00_JPRB, 0.45356E+00_JPRB, 0.46183E+00_JPRB, 0.47687E+00_JPRB, 0.48896E+00_JPRB, 0.49716E+00_JPRB ,&
&  0.50175E+00_JPRB, 0.50162E+00_JPRB, 0.49901E+00_JPRB, 0.49168E+00_JPRB, 0.48032E+00_JPRB, 0.46567E+00_JPRB, 0.44978E+00_JPRB ,&
&  0.43422E+00_JPRB, 0.41814E+00_JPRB, 0.40107E+00_JPRB, 0.38437E+00_JPRB, 0.36923E+00_JPRB, 0.35540E+00_JPRB, 0.34248E+00_JPRB ,&
&  0.33135E+00_JPRB, 0.32277E+00_JPRB, 0.31605E+00_JPRB, 0.31118E+00_JPRB, 0.30708E+00_JPRB, 0.30380E+00_JPRB, 0.30239E+00_JPRB ,&
&  0.30227E+00_JPRB, 0.30329E+00_JPRB, 0.30382E+00_JPRB, 0.30546E+00_JPRB, 0.30833E+00_JPRB, 0.30922E+00_JPRB, 0.30736E+00_JPRB ,&
&  0.30273E+00_JPRB, 0.29846E+00_JPRB, 0.29437E+00_JPRB, 0.29400E+00_JPRB, 0.29432E+00_JPRB, 0.29621E+00_JPRB, 0.29882E+00_JPRB ,&
&  0.30341E+00_JPRB, 0.30927E+00_JPRB, 0.31473E+00_JPRB, 0.31902E+00_JPRB, 0.32392E+00_JPRB, 0.32832E+00_JPRB, 0.33251E+00_JPRB ,&
&  0.33691E+00_JPRB, 0.34139E+00_JPRB, 0.34713E+00_JPRB, 0.35293E+00_JPRB, 0.35824E+00_JPRB, 0.36244E+00_JPRB, 0.36541E+00_JPRB ,&
&  0.36733E+00_JPRB, 0.36785E+00_JPRB, 0.36820E+00_JPRB, 0.36992E+00_JPRB, 0.37239E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.33765E+00_JPRB, 0.33991E+00_JPRB, 0.34274E+00_JPRB, 0.34478E+00_JPRB, 0.34529E+00_JPRB, 0.34455E+00_JPRB, 0.34362E+00_JPRB ,&
&  0.34310E+00_JPRB, 0.34242E+00_JPRB, 0.34054E+00_JPRB, 0.33773E+00_JPRB, 0.33500E+00_JPRB, 0.33218E+00_JPRB, 0.33032E+00_JPRB ,&
&  0.32862E+00_JPRB, 0.32651E+00_JPRB, 0.32235E+00_JPRB, 0.31604E+00_JPRB, 0.30949E+00_JPRB, 0.30577E+00_JPRB, 0.30416E+00_JPRB ,&
&  0.30426E+00_JPRB, 0.30497E+00_JPRB, 0.30750E+00_JPRB, 0.30916E+00_JPRB, 0.31148E+00_JPRB, 0.31553E+00_JPRB, 0.32134E+00_JPRB ,&
&  0.32672E+00_JPRB, 0.33198E+00_JPRB, 0.33775E+00_JPRB, 0.34307E+00_JPRB, 0.34842E+00_JPRB, 0.35586E+00_JPRB, 0.36523E+00_JPRB ,&
&  0.37837E+00_JPRB, 0.38875E+00_JPRB, 0.38934E+00_JPRB, 0.38976E+00_JPRB, 0.39481E+00_JPRB, 0.40206E+00_JPRB, 0.40826E+00_JPRB ,&
&  0.41326E+00_JPRB, 0.41378E+00_JPRB, 0.41230E+00_JPRB, 0.40827E+00_JPRB, 0.40296E+00_JPRB, 0.39632E+00_JPRB, 0.38726E+00_JPRB ,&
&  0.37632E+00_JPRB, 0.36429E+00_JPRB, 0.35252E+00_JPRB, 0.34102E+00_JPRB, 0.32979E+00_JPRB, 0.31960E+00_JPRB, 0.31073E+00_JPRB ,&
&  0.30290E+00_JPRB, 0.29680E+00_JPRB, 0.29104E+00_JPRB, 0.28719E+00_JPRB, 0.28446E+00_JPRB, 0.28155E+00_JPRB, 0.28059E+00_JPRB ,&
&  0.27915E+00_JPRB, 0.27818E+00_JPRB, 0.27788E+00_JPRB, 0.27928E+00_JPRB, 0.28234E+00_JPRB, 0.28419E+00_JPRB, 0.28270E+00_JPRB ,&
&  0.27920E+00_JPRB, 0.27712E+00_JPRB, 0.27591E+00_JPRB, 0.27515E+00_JPRB, 0.27292E+00_JPRB, 0.27302E+00_JPRB, 0.27565E+00_JPRB ,&
&  0.28106E+00_JPRB, 0.28661E+00_JPRB, 0.29100E+00_JPRB, 0.29380E+00_JPRB, 0.29696E+00_JPRB, 0.29991E+00_JPRB, 0.30250E+00_JPRB ,&
&  0.30505E+00_JPRB, 0.30685E+00_JPRB, 0.30811E+00_JPRB, 0.30971E+00_JPRB, 0.31224E+00_JPRB, 0.31466E+00_JPRB, 0.31702E+00_JPRB ,&
&  0.31972E+00_JPRB, 0.32409E+00_JPRB, 0.32855E+00_JPRB, 0.33183E+00_JPRB, 0.33547E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.31226E+00_JPRB, 0.31499E+00_JPRB, 0.31720E+00_JPRB, 0.31679E+00_JPRB, 0.31431E+00_JPRB, 0.31290E+00_JPRB, 0.31288E+00_JPRB ,&
&  0.31308E+00_JPRB, 0.31254E+00_JPRB, 0.31067E+00_JPRB, 0.31027E+00_JPRB, 0.31019E+00_JPRB, 0.30934E+00_JPRB, 0.30900E+00_JPRB ,&
&  0.30798E+00_JPRB, 0.30491E+00_JPRB, 0.30020E+00_JPRB, 0.29440E+00_JPRB, 0.29025E+00_JPRB, 0.28838E+00_JPRB, 0.28856E+00_JPRB ,&
&  0.28991E+00_JPRB, 0.29085E+00_JPRB, 0.29350E+00_JPRB, 0.29544E+00_JPRB, 0.29768E+00_JPRB, 0.30089E+00_JPRB, 0.30445E+00_JPRB ,&
&  0.30688E+00_JPRB, 0.30957E+00_JPRB, 0.31258E+00_JPRB, 0.31562E+00_JPRB, 0.31967E+00_JPRB, 0.32603E+00_JPRB, 0.33188E+00_JPRB ,&
&  0.34068E+00_JPRB, 0.34761E+00_JPRB, 0.34475E+00_JPRB, 0.34036E+00_JPRB, 0.34010E+00_JPRB, 0.34310E+00_JPRB, 0.34708E+00_JPRB ,&
&  0.35148E+00_JPRB, 0.35570E+00_JPRB, 0.35904E+00_JPRB, 0.35979E+00_JPRB, 0.35745E+00_JPRB, 0.35227E+00_JPRB, 0.34465E+00_JPRB ,&
&  0.33586E+00_JPRB, 0.32665E+00_JPRB, 0.31742E+00_JPRB, 0.30798E+00_JPRB, 0.29980E+00_JPRB, 0.29227E+00_JPRB, 0.28563E+00_JPRB ,&
&  0.27954E+00_JPRB, 0.27382E+00_JPRB, 0.26942E+00_JPRB, 0.26637E+00_JPRB, 0.26462E+00_JPRB, 0.26259E+00_JPRB, 0.26208E+00_JPRB ,&
&  0.26096E+00_JPRB, 0.26035E+00_JPRB, 0.26007E+00_JPRB, 0.26058E+00_JPRB, 0.26188E+00_JPRB, 0.26279E+00_JPRB, 0.26214E+00_JPRB ,&
&  0.26181E+00_JPRB, 0.26340E+00_JPRB, 0.26324E+00_JPRB, 0.26059E+00_JPRB, 0.25725E+00_JPRB, 0.25663E+00_JPRB, 0.25846E+00_JPRB ,&
&  0.26187E+00_JPRB, 0.26562E+00_JPRB, 0.26891E+00_JPRB, 0.27166E+00_JPRB, 0.27399E+00_JPRB, 0.27684E+00_JPRB, 0.27941E+00_JPRB ,&
&  0.28165E+00_JPRB, 0.28305E+00_JPRB, 0.28507E+00_JPRB, 0.28721E+00_JPRB, 0.28934E+00_JPRB, 0.29079E+00_JPRB, 0.29059E+00_JPRB ,&
&  0.28998E+00_JPRB, 0.29175E+00_JPRB, 0.29703E+00_JPRB, 0.30454E+00_JPRB, 0.30923E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.28997E+00_JPRB, 0.29216E+00_JPRB, 0.29312E+00_JPRB, 0.29232E+00_JPRB, 0.29106E+00_JPRB, 0.29136E+00_JPRB, 0.29174E+00_JPRB ,&
&  0.29245E+00_JPRB, 0.29229E+00_JPRB, 0.29045E+00_JPRB, 0.28873E+00_JPRB, 0.29015E+00_JPRB, 0.28985E+00_JPRB, 0.28955E+00_JPRB ,&
&  0.28812E+00_JPRB, 0.28624E+00_JPRB, 0.28338E+00_JPRB, 0.28014E+00_JPRB, 0.27636E+00_JPRB, 0.27456E+00_JPRB, 0.27565E+00_JPRB ,&
&  0.27784E+00_JPRB, 0.27870E+00_JPRB, 0.27942E+00_JPRB, 0.28007E+00_JPRB, 0.28167E+00_JPRB, 0.28383E+00_JPRB, 0.28538E+00_JPRB ,&
&  0.28644E+00_JPRB, 0.28927E+00_JPRB, 0.29103E+00_JPRB, 0.29078E+00_JPRB, 0.29501E+00_JPRB, 0.29991E+00_JPRB, 0.30501E+00_JPRB ,&
&  0.31057E+00_JPRB, 0.31420E+00_JPRB, 0.31450E+00_JPRB, 0.31307E+00_JPRB, 0.31191E+00_JPRB, 0.31306E+00_JPRB, 0.31405E+00_JPRB ,&
&  0.31561E+00_JPRB, 0.31989E+00_JPRB, 0.32484E+00_JPRB, 0.32747E+00_JPRB, 0.32703E+00_JPRB, 0.32286E+00_JPRB, 0.31524E+00_JPRB ,&
&  0.30472E+00_JPRB, 0.29325E+00_JPRB, 0.28359E+00_JPRB, 0.27580E+00_JPRB, 0.26991E+00_JPRB, 0.26544E+00_JPRB, 0.26182E+00_JPRB ,&
&  0.25780E+00_JPRB, 0.25478E+00_JPRB, 0.25435E+00_JPRB, 0.25460E+00_JPRB, 0.25395E+00_JPRB, 0.25109E+00_JPRB, 0.24911E+00_JPRB ,&
&  0.24742E+00_JPRB, 0.24782E+00_JPRB, 0.24904E+00_JPRB, 0.24952E+00_JPRB, 0.24977E+00_JPRB, 0.24980E+00_JPRB, 0.24979E+00_JPRB ,&
&  0.25092E+00_JPRB, 0.25060E+00_JPRB, 0.24913E+00_JPRB, 0.24687E+00_JPRB, 0.24489E+00_JPRB, 0.24466E+00_JPRB, 0.24546E+00_JPRB ,&
&  0.24672E+00_JPRB, 0.24892E+00_JPRB, 0.25156E+00_JPRB, 0.25419E+00_JPRB, 0.25625E+00_JPRB, 0.25728E+00_JPRB, 0.25786E+00_JPRB ,&
&  0.25833E+00_JPRB, 0.25867E+00_JPRB, 0.26060E+00_JPRB, 0.26510E+00_JPRB, 0.26920E+00_JPRB, 0.27216E+00_JPRB, 0.27351E+00_JPRB ,&
&  0.27270E+00_JPRB, 0.27310E+00_JPRB, 0.27620E+00_JPRB, 0.28209E+00_JPRB, 0.28644E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.27257E+00_JPRB, 0.27467E+00_JPRB, 0.27498E+00_JPRB, 0.27412E+00_JPRB, 0.27317E+00_JPRB, 0.27382E+00_JPRB, 0.27499E+00_JPRB ,&
&  0.27560E+00_JPRB, 0.27565E+00_JPRB, 0.27436E+00_JPRB, 0.27188E+00_JPRB, 0.26983E+00_JPRB, 0.26973E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.26723E+00_JPRB, 0.26834E+00_JPRB, 0.26800E+00_JPRB, 0.26731E+00_JPRB, 0.26577E+00_JPRB, 0.26525E+00_JPRB, 0.26656E+00_JPRB ,&
&  0.26900E+00_JPRB, 0.26947E+00_JPRB, 0.26990E+00_JPRB, 0.27044E+00_JPRB, 0.27161E+00_JPRB, 0.27322E+00_JPRB, 0.27393E+00_JPRB ,&
&  0.27406E+00_JPRB, 0.27218E+00_JPRB, 0.27089E+00_JPRB, 0.27313E+00_JPRB, 0.27716E+00_JPRB, 0.28271E+00_JPRB, 0.28671E+00_JPRB ,&
&  0.29018E+00_JPRB, 0.29252E+00_JPRB, 0.29347E+00_JPRB, 0.29320E+00_JPRB, 0.29331E+00_JPRB, 0.29551E+00_JPRB, 0.29746E+00_JPRB ,&
&  0.29847E+00_JPRB, 0.29898E+00_JPRB, 0.29819E+00_JPRB, 0.29759E+00_JPRB, 0.29463E+00_JPRB, 0.28952E+00_JPRB, 0.28086E+00_JPRB ,&
&  0.27142E+00_JPRB, 0.26352E+00_JPRB, 0.25753E+00_JPRB, 0.25288E+00_JPRB, 0.24983E+00_JPRB, 0.24975E+00_JPRB, 0.24966E+00_JPRB ,&
&  0.24898E+00_JPRB, 0.24828E+00_JPRB, 0.24947E+00_JPRB, 0.24955E+00_JPRB, 0.24346E+00_JPRB, 0.23893E+00_JPRB, 0.23375E+00_JPRB ,&
&  0.23233E+00_JPRB, 0.23222E+00_JPRB, 0.23191E+00_JPRB, 0.23518E+00_JPRB, 0.23990E+00_JPRB, 0.24095E+00_JPRB, 0.23940E+00_JPRB ,&
&  0.23257E+00_JPRB, 0.22811E+00_JPRB, 0.22508E+00_JPRB, 0.22221E+00_JPRB, 0.22070E+00_JPRB, 0.22136E+00_JPRB, 0.22242E+00_JPRB ,&
&  0.22513E+00_JPRB, 0.22666E+00_JPRB, 0.22957E+00_JPRB, 0.23132E+00_JPRB, 0.23136E+00_JPRB, 0.23166E+00_JPRB, 0.23106E+00_JPRB ,&
&  0.23333E+00_JPRB, 0.23897E+00_JPRB, 0.24686E+00_JPRB, 0.24877E+00_JPRB, 0.25410E+00_JPRB, 0.25783E+00_JPRB, 0.25929E+00_JPRB ,&
&  0.26001E+00_JPRB, 0.26000E+00_JPRB, 0.25950E+00_JPRB, 0.26325E+00_JPRB, 0.26897E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.25831E+00_JPRB, 0.25968E+00_JPRB, 0.26000E+00_JPRB, 0.25965E+00_JPRB, 0.25913E+00_JPRB, 0.25928E+00_JPRB, 0.25959E+00_JPRB ,&
&  0.25936E+00_JPRB, 0.25842E+00_JPRB, 0.25703E+00_JPRB, 0.25457E+00_JPRB, 0.25149E+00_JPRB, 0.24962E+00_JPRB, 0.25066E+00_JPRB ,&
&  0.25255E+00_JPRB, 0.25414E+00_JPRB, 0.25590E+00_JPRB, 0.25695E+00_JPRB, 0.25748E+00_JPRB, 0.25845E+00_JPRB, 0.26005E+00_JPRB ,&
&  0.26128E+00_JPRB, 0.26172E+00_JPRB, 0.26287E+00_JPRB, 0.26368E+00_JPRB, 0.26564E+00_JPRB, 0.26737E+00_JPRB, 0.26744E+00_JPRB ,&
&  0.26313E+00_JPRB, 0.25510E+00_JPRB, 0.25040E+00_JPRB, 0.25323E+00_JPRB, 0.26018E+00_JPRB, 0.26645E+00_JPRB, 0.27072E+00_JPRB ,&
&  0.27341E+00_JPRB, 0.27444E+00_JPRB, 0.27453E+00_JPRB, 0.27585E+00_JPRB, 0.27814E+00_JPRB, 0.28057E+00_JPRB, 0.28037E+00_JPRB ,&
&  0.27849E+00_JPRB, 0.27701E+00_JPRB, 0.27482E+00_JPRB, 0.27057E+00_JPRB, 0.26442E+00_JPRB, 0.25820E+00_JPRB, 0.25255E+00_JPRB ,&
&  0.24726E+00_JPRB, 0.24222E+00_JPRB, 0.23828E+00_JPRB, 0.23580E+00_JPRB, 0.23524E+00_JPRB, 0.23702E+00_JPRB, 0.23640E+00_JPRB ,&
&  0.23726E+00_JPRB, 0.23521E+00_JPRB, 0.23289E+00_JPRB, 0.22986E+00_JPRB, 0.22724E+00_JPRB, 0.22225E+00_JPRB, 0.21576E+00_JPRB ,&
&  0.21262E+00_JPRB, 0.21594E+00_JPRB, 0.22417E+00_JPRB, 0.22986E+00_JPRB, 0.22494E+00_JPRB, 0.21938E+00_JPRB, 0.21665E+00_JPRB ,&
&  0.21267E+00_JPRB, 0.20792E+00_JPRB, 0.20472E+00_JPRB, 0.20395E+00_JPRB, 0.20415E+00_JPRB, 0.20440E+00_JPRB, 0.20567E+00_JPRB ,&
&  0.20771E+00_JPRB, 0.20945E+00_JPRB, 0.21038E+00_JPRB, 0.21163E+00_JPRB, 0.21219E+00_JPRB, 0.21231E+00_JPRB, 0.21244E+00_JPRB ,&
&  0.21466E+00_JPRB, 0.21912E+00_JPRB, 0.22473E+00_JPRB, 0.23081E+00_JPRB, 0.23527E+00_JPRB, 0.23934E+00_JPRB, 0.24235E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.24489E+00_JPRB, 0.24798E+00_JPRB, 0.25256E+00_JPRB, 0.25589E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.24234E+00_JPRB, 0.24631E+00_JPRB, 0.24755E+00_JPRB, 0.24735E+00_JPRB, 0.24602E+00_JPRB, 0.24476E+00_JPRB, 0.24425E+00_JPRB ,&
&  0.24382E+00_JPRB, 0.24253E+00_JPRB, 0.24158E+00_JPRB, 0.24098E+00_JPRB, 0.24033E+00_JPRB, 0.23939E+00_JPRB, 0.23973E+00_JPRB ,&
&  0.24140E+00_JPRB, 0.24408E+00_JPRB, 0.24643E+00_JPRB, 0.24811E+00_JPRB, 0.24825E+00_JPRB, 0.24901E+00_JPRB, 0.25010E+00_JPRB ,&
&  0.25017E+00_JPRB, 0.24989E+00_JPRB, 0.25299E+00_JPRB, 0.25273E+00_JPRB, 0.25345E+00_JPRB, 0.25306E+00_JPRB, 0.25295E+00_JPRB ,&
&  0.24704E+00_JPRB, 0.24123E+00_JPRB, 0.23948E+00_JPRB, 0.24219E+00_JPRB, 0.24993E+00_JPRB, 0.25118E+00_JPRB, 0.25548E+00_JPRB ,&
&  0.25778E+00_JPRB, 0.25856E+00_JPRB, 0.26039E+00_JPRB, 0.26182E+00_JPRB, 0.26349E+00_JPRB, 0.26301E+00_JPRB, 0.26085E+00_JPRB ,&
&  0.25760E+00_JPRB, 0.25665E+00_JPRB, 0.25330E+00_JPRB, 0.23786E+00_JPRB, 0.22889E+00_JPRB, 0.22372E+00_JPRB, 0.21814E+00_JPRB ,&
&  0.21296E+00_JPRB, 0.21059E+00_JPRB, 0.21034E+00_JPRB, 0.20861E+00_JPRB, 0.20964E+00_JPRB, 0.21249E+00_JPRB, 0.21367E+00_JPRB ,&
&  0.21336E+00_JPRB, 0.21393E+00_JPRB, 0.21361E+00_JPRB, 0.21240E+00_JPRB, 0.20996E+00_JPRB, 0.20847E+00_JPRB, 0.20765E+00_JPRB ,&
&  0.20835E+00_JPRB, 0.20783E+00_JPRB, 0.21234E+00_JPRB, 0.21318E+00_JPRB, 0.20627E+00_JPRB, 0.20601E+00_JPRB, 0.20498E+00_JPRB ,&
&  0.19868E+00_JPRB, 0.19522E+00_JPRB, 0.19320E+00_JPRB, 0.19240E+00_JPRB, 0.19365E+00_JPRB, 0.19540E+00_JPRB, 0.19614E+00_JPRB ,&
&  0.19632E+00_JPRB, 0.19769E+00_JPRB, 0.19832E+00_JPRB, 0.19899E+00_JPRB, 0.19942E+00_JPRB, 0.20048E+00_JPRB, 0.20205E+00_JPRB ,&
&  0.20398E+00_JPRB, 0.20758E+00_JPRB, 0.21101E+00_JPRB, 0.21481E+00_JPRB, 0.21694E+00_JPRB, 0.21851E+00_JPRB, 0.22248E+00_JPRB ,&
&  0.22416E+00_JPRB, 0.23215E+00_JPRB, 0.24344E+00_JPRB, 0.24403E+00_JPRB, 0.23706E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.23381E+00_JPRB, 0.23548E+00_JPRB, 0.23657E+00_JPRB, 0.23546E+00_JPRB, 0.23562E+00_JPRB, 0.23535E+00_JPRB, 0.23528E+00_JPRB ,&
&  0.23505E+00_JPRB, 0.23427E+00_JPRB, 0.23459E+00_JPRB, 0.23217E+00_JPRB, 0.23084E+00_JPRB, 0.22788E+00_JPRB, 0.22760E+00_JPRB ,&
&  0.22625E+00_JPRB, 0.22662E+00_JPRB, 0.22735E+00_JPRB, 0.22924E+00_JPRB, 0.22977E+00_JPRB, 0.23149E+00_JPRB, 0.23046E+00_JPRB ,&
&  0.23096E+00_JPRB, 0.23143E+00_JPRB, 0.22996E+00_JPRB, 0.22950E+00_JPRB, 0.22968E+00_JPRB, 0.22966E+00_JPRB, 0.22984E+00_JPRB ,&
&  0.23234E+00_JPRB, 0.23476E+00_JPRB, 0.23709E+00_JPRB, 0.23812E+00_JPRB, 0.24267E+00_JPRB, 0.24394E+00_JPRB, 0.24655E+00_JPRB ,&
&  0.24759E+00_JPRB, 0.24875E+00_JPRB, 0.24991E+00_JPRB, 0.24909E+00_JPRB, 0.24772E+00_JPRB, 0.24311E+00_JPRB, 0.24158E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.22451E+00_JPRB, 0.21755E+00_JPRB, 0.21106E+00_JPRB, 0.20597E+00_JPRB, 0.20249E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19750E+00_JPRB, 0.19526E+00_JPRB, 0.19271E+00_JPRB, 0.19300E+00_JPRB, 0.19399E+00_JPRB, 0.19554E+00_JPRB, 0.19716E+00_JPRB ,&
&  0.19905E+00_JPRB, 0.20013E+00_JPRB, 0.20056E+00_JPRB, 0.20145E+00_JPRB, 0.20155E+00_JPRB, 0.20189E+00_JPRB, 0.20212E+00_JPRB ,&
&  0.20374E+00_JPRB, 0.20418E+00_JPRB, 0.20416E+00_JPRB, 0.20620E+00_JPRB, 0.20394E+00_JPRB, 0.20064E+00_JPRB, 0.19731E+00_JPRB ,&
&  0.19367E+00_JPRB, 0.19142E+00_JPRB, 0.18945E+00_JPRB, 0.18869E+00_JPRB, 0.18921E+00_JPRB, 0.19015E+00_JPRB, 0.19130E+00_JPRB ,&
&  0.19145E+00_JPRB, 0.19220E+00_JPRB, 0.19274E+00_JPRB, 0.19379E+00_JPRB, 0.19489E+00_JPRB, 0.19619E+00_JPRB, 0.19766E+00_JPRB ,&
&  0.19991E+00_JPRB, 0.20250E+00_JPRB, 0.20505E+00_JPRB, 0.20762E+00_JPRB, 0.21179E+00_JPRB, 0.21649E+00_JPRB, 0.22382E+00_JPRB ,&
&  0.22927E+00_JPRB, 0.23419E+00_JPRB, 0.23056E+00_JPRB, 0.23522E+00_JPRB, 0.23574E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.22797E+00_JPRB, 0.22864E+00_JPRB, 0.22621E+00_JPRB, 0.22630E+00_JPRB, 0.22691E+00_JPRB, 0.22718E+00_JPRB, 0.22765E+00_JPRB ,&
&  0.22492E+00_JPRB, 0.22436E+00_JPRB, 0.22210E+00_JPRB, 0.22046E+00_JPRB, 0.21816E+00_JPRB, 0.21592E+00_JPRB, 0.21431E+00_JPRB ,&
&  0.21236E+00_JPRB, 0.21194E+00_JPRB, 0.21107E+00_JPRB, 0.21241E+00_JPRB, 0.21254E+00_JPRB, 0.21462E+00_JPRB, 0.21626E+00_JPRB ,&
&  0.21703E+00_JPRB, 0.21825E+00_JPRB, 0.21643E+00_JPRB, 0.21561E+00_JPRB, 0.21724E+00_JPRB, 0.21848E+00_JPRB, 0.22110E+00_JPRB ,&
&  0.22370E+00_JPRB, 0.22712E+00_JPRB, 0.23293E+00_JPRB, 0.23485E+00_JPRB, 0.23751E+00_JPRB, 0.23905E+00_JPRB, 0.23884E+00_JPRB ,&
&  0.23850E+00_JPRB, 0.23785E+00_JPRB, 0.23854E+00_JPRB, 0.23814E+00_JPRB, 0.23069E+00_JPRB, 0.22524E+00_JPRB, 0.21935E+00_JPRB ,&
&  0.21414E+00_JPRB, 0.20901E+00_JPRB, 0.20429E+00_JPRB, 0.20113E+00_JPRB, 0.19746E+00_JPRB, 0.19492E+00_JPRB, 0.19292E+00_JPRB ,&
&  0.19096E+00_JPRB, 0.18996E+00_JPRB, 0.18850E+00_JPRB, 0.18810E+00_JPRB, 0.18856E+00_JPRB, 0.18964E+00_JPRB, 0.19117E+00_JPRB ,&
&  0.19296E+00_JPRB, 0.19417E+00_JPRB, 0.19568E+00_JPRB, 0.19707E+00_JPRB, 0.19806E+00_JPRB, 0.19862E+00_JPRB, 0.19820E+00_JPRB ,&
&  0.19844E+00_JPRB, 0.19881E+00_JPRB, 0.19832E+00_JPRB, 0.19839E+00_JPRB, 0.19706E+00_JPRB, 0.19546E+00_JPRB, 0.19385E+00_JPRB ,&
&  0.19153E+00_JPRB, 0.18976E+00_JPRB, 0.18915E+00_JPRB, 0.18882E+00_JPRB, 0.18973E+00_JPRB, 0.19108E+00_JPRB, 0.19191E+00_JPRB ,&
&  0.19226E+00_JPRB, 0.19208E+00_JPRB, 0.19202E+00_JPRB, 0.19344E+00_JPRB, 0.19491E+00_JPRB, 0.19679E+00_JPRB, 0.19878E+00_JPRB ,&
&  0.19933E+00_JPRB, 0.20091E+00_JPRB, 0.20225E+00_JPRB, 0.20402E+00_JPRB, 0.21024E+00_JPRB, 0.21475E+00_JPRB, 0.22337E+00_JPRB ,&
&  0.22620E+00_JPRB, 0.22359E+00_JPRB, 0.22027E+00_JPRB, 0.22633E+00_JPRB, 0.22762E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.22635E+00_JPRB, 0.22897E+00_JPRB, 0.22720E+00_JPRB, 0.22677E+00_JPRB, 0.22576E+00_JPRB, 0.22531E+00_JPRB, 0.22370E+00_JPRB ,&
&  0.22346E+00_JPRB, 0.22185E+00_JPRB, 0.22171E+00_JPRB, 0.22005E+00_JPRB, 0.21840E+00_JPRB, 0.21595E+00_JPRB, 0.21351E+00_JPRB ,&
&  0.21118E+00_JPRB, 0.20918E+00_JPRB, 0.20789E+00_JPRB, 0.20754E+00_JPRB, 0.20652E+00_JPRB, 0.20694E+00_JPRB, 0.20991E+00_JPRB ,&
&  0.21057E+00_JPRB, 0.21193E+00_JPRB, 0.21190E+00_JPRB, 0.21313E+00_JPRB, 0.21478E+00_JPRB, 0.21492E+00_JPRB, 0.21708E+00_JPRB ,&
&  0.21727E+00_JPRB, 0.21887E+00_JPRB, 0.22624E+00_JPRB, 0.22997E+00_JPRB, 0.23177E+00_JPRB, 0.23491E+00_JPRB, 0.23580E+00_JPRB ,&
&  0.23392E+00_JPRB, 0.22776E+00_JPRB, 0.23067E+00_JPRB, 0.22701E+00_JPRB, 0.21988E+00_JPRB, 0.21724E+00_JPRB, 0.21159E+00_JPRB ,&
&  0.20892E+00_JPRB, 0.20513E+00_JPRB, 0.20234E+00_JPRB, 0.20016E+00_JPRB, 0.19653E+00_JPRB, 0.19427E+00_JPRB, 0.19169E+00_JPRB ,&
&  0.18936E+00_JPRB, 0.18852E+00_JPRB, 0.18698E+00_JPRB, 0.18700E+00_JPRB, 0.18734E+00_JPRB, 0.18801E+00_JPRB, 0.18906E+00_JPRB ,&
&  0.19037E+00_JPRB, 0.19156E+00_JPRB, 0.19266E+00_JPRB, 0.19359E+00_JPRB, 0.19436E+00_JPRB, 0.19478E+00_JPRB, 0.19458E+00_JPRB ,&
&  0.19470E+00_JPRB, 0.19460E+00_JPRB, 0.19404E+00_JPRB, 0.19386E+00_JPRB, 0.19276E+00_JPRB, 0.19233E+00_JPRB, 0.19186E+00_JPRB ,&
&  0.19167E+00_JPRB, 0.19170E+00_JPRB, 0.19158E+00_JPRB, 0.19180E+00_JPRB, 0.19210E+00_JPRB, 0.19291E+00_JPRB, 0.19352E+00_JPRB ,&
&  0.19400E+00_JPRB, 0.19506E+00_JPRB, 0.19570E+00_JPRB, 0.19660E+00_JPRB, 0.19787E+00_JPRB, 0.19835E+00_JPRB, 0.19977E+00_JPRB ,&
&  0.20095E+00_JPRB, 0.20238E+00_JPRB, 0.20513E+00_JPRB, 0.20783E+00_JPRB, 0.21181E+00_JPRB, 0.21649E+00_JPRB, 0.21769E+00_JPRB ,&
&  0.21838E+00_JPRB, 0.21777E+00_JPRB, 0.21482E+00_JPRB, 0.22079E+00_JPRB, 0.22362E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.21871E+00_JPRB, 0.22075E+00_JPRB, 0.22208E+00_JPRB, 0.22299E+00_JPRB, 0.22459E+00_JPRB, 0.22534E+00_JPRB, 0.22597E+00_JPRB ,&
&  0.22392E+00_JPRB, 0.22342E+00_JPRB, 0.22242E+00_JPRB, 0.22113E+00_JPRB, 0.22031E+00_JPRB, 0.21758E+00_JPRB, 0.21501E+00_JPRB ,&
&  0.21225E+00_JPRB, 0.20985E+00_JPRB, 0.20863E+00_JPRB, 0.20743E+00_JPRB, 0.20613E+00_JPRB, 0.20505E+00_JPRB, 0.20787E+00_JPRB ,&
&  0.20958E+00_JPRB, 0.21060E+00_JPRB, 0.21222E+00_JPRB, 0.21165E+00_JPRB, 0.21305E+00_JPRB, 0.21526E+00_JPRB, 0.21828E+00_JPRB ,&
&  0.21976E+00_JPRB, 0.22281E+00_JPRB, 0.22455E+00_JPRB, 0.22543E+00_JPRB, 0.22675E+00_JPRB, 0.22776E+00_JPRB, 0.22751E+00_JPRB ,&
&  0.22603E+00_JPRB, 0.22408E+00_JPRB, 0.22167E+00_JPRB, 0.21851E+00_JPRB, 0.21524E+00_JPRB, 0.21213E+00_JPRB, 0.20901E+00_JPRB ,&
&  0.20684E+00_JPRB, 0.20483E+00_JPRB, 0.20263E+00_JPRB, 0.20060E+00_JPRB, 0.19864E+00_JPRB, 0.19687E+00_JPRB, 0.19508E+00_JPRB ,&
&  0.19370E+00_JPRB, 0.19260E+00_JPRB, 0.19121E+00_JPRB, 0.19057E+00_JPRB, 0.18993E+00_JPRB, 0.18970E+00_JPRB, 0.18965E+00_JPRB ,&
&  0.19027E+00_JPRB, 0.19100E+00_JPRB, 0.19173E+00_JPRB, 0.19215E+00_JPRB, 0.19256E+00_JPRB, 0.19276E+00_JPRB, 0.19312E+00_JPRB ,&
&  0.19348E+00_JPRB, 0.19388E+00_JPRB, 0.19339E+00_JPRB, 0.19294E+00_JPRB, 0.19233E+00_JPRB, 0.19163E+00_JPRB, 0.19141E+00_JPRB ,&
&  0.19176E+00_JPRB, 0.19203E+00_JPRB, 0.19176E+00_JPRB, 0.19208E+00_JPRB, 0.19268E+00_JPRB, 0.19329E+00_JPRB, 0.19490E+00_JPRB ,&
&  0.19577E+00_JPRB, 0.19658E+00_JPRB, 0.19735E+00_JPRB, 0.19793E+00_JPRB, 0.19887E+00_JPRB, 0.19960E+00_JPRB, 0.20054E+00_JPRB ,&
&  0.20167E+00_JPRB, 0.20274E+00_JPRB, 0.20422E+00_JPRB, 0.20614E+00_JPRB, 0.21086E+00_JPRB, 0.21371E+00_JPRB, 0.21591E+00_JPRB ,&
&  0.21646E+00_JPRB, 0.21096E+00_JPRB, 0.21814E+00_JPRB, 0.21646E+00_JPRB, 0.21581E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.21235E+00_JPRB, 0.21410E+00_JPRB, 0.21533E+00_JPRB, 0.21710E+00_JPRB, 0.21082E+00_JPRB, 0.21761E+00_JPRB, 0.21395E+00_JPRB ,&
&  0.21021E+00_JPRB, 0.20522E+00_JPRB, 0.20579E+00_JPRB, 0.20524E+00_JPRB, 0.20428E+00_JPRB, 0.20406E+00_JPRB, 0.20434E+00_JPRB ,&
&  0.20478E+00_JPRB, 0.20489E+00_JPRB, 0.20579E+00_JPRB, 0.20647E+00_JPRB, 0.20701E+00_JPRB, 0.20745E+00_JPRB, 0.20802E+00_JPRB ,&
&  0.20882E+00_JPRB, 0.21006E+00_JPRB, 0.21133E+00_JPRB, 0.20888E+00_JPRB, 0.20852E+00_JPRB, 0.20867E+00_JPRB, 0.20924E+00_JPRB ,&
&  0.20944E+00_JPRB, 0.21063E+00_JPRB, 0.21160E+00_JPRB, 0.21235E+00_JPRB, 0.21463E+00_JPRB, 0.21485E+00_JPRB, 0.21476E+00_JPRB ,&
&  0.21435E+00_JPRB, 0.21340E+00_JPRB, 0.21208E+00_JPRB, 0.21083E+00_JPRB, 0.20955E+00_JPRB, 0.20783E+00_JPRB, 0.20618E+00_JPRB ,&
&  0.20484E+00_JPRB, 0.20376E+00_JPRB, 0.20231E+00_JPRB, 0.20091E+00_JPRB, 0.19965E+00_JPRB, 0.19845E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19669E+00_JPRB, 0.19606E+00_JPRB, 0.19540E+00_JPRB, 0.19490E+00_JPRB, 0.19437E+00_JPRB, 0.19387E+00_JPRB, 0.19341E+00_JPRB ,&
&  0.19365E+00_JPRB, 0.19384E+00_JPRB, 0.19403E+00_JPRB, 0.19424E+00_JPRB, 0.19482E+00_JPRB, 0.19497E+00_JPRB, 0.19514E+00_JPRB ,&
&  0.19527E+00_JPRB, 0.19520E+00_JPRB, 0.19506E+00_JPRB, 0.19480E+00_JPRB, 0.19450E+00_JPRB, 0.19412E+00_JPRB, 0.19393E+00_JPRB ,&
&  0.19391E+00_JPRB, 0.19403E+00_JPRB, 0.19444E+00_JPRB, 0.19472E+00_JPRB, 0.19501E+00_JPRB, 0.19526E+00_JPRB, 0.19584E+00_JPRB ,&
&  0.19642E+00_JPRB, 0.19699E+00_JPRB, 0.19754E+00_JPRB, 0.19737E+00_JPRB, 0.19765E+00_JPRB, 0.19791E+00_JPRB, 0.19816E+00_JPRB ,&
&  0.19831E+00_JPRB, 0.19886E+00_JPRB, 0.19976E+00_JPRB, 0.20102E+00_JPRB, 0.20338E+00_JPRB, 0.20530E+00_JPRB, 0.20708E+00_JPRB ,&
&  0.20898E+00_JPRB, 0.21288E+00_JPRB, 0.21415E+00_JPRB, 0.21432E+00_JPRB, 0.21394E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.20218E+00_JPRB, 0.20318E+00_JPRB, 0.20384E+00_JPRB, 0.20434E+00_JPRB, 0.20404E+00_JPRB, 0.20405E+00_JPRB, 0.20403E+00_JPRB ,&
&  0.20389E+00_JPRB, 0.20377E+00_JPRB, 0.20408E+00_JPRB, 0.20433E+00_JPRB, 0.20465E+00_JPRB, 0.20500E+00_JPRB, 0.20546E+00_JPRB ,&
&  0.20563E+00_JPRB, 0.20603E+00_JPRB, 0.20446E+00_JPRB, 0.20314E+00_JPRB, 0.20223E+00_JPRB, 0.20165E+00_JPRB, 0.20132E+00_JPRB ,&
&  0.20105E+00_JPRB, 0.20088E+00_JPRB, 0.20070E+00_JPRB, 0.20182E+00_JPRB, 0.20253E+00_JPRB, 0.20315E+00_JPRB, 0.20363E+00_JPRB ,&
&  0.20379E+00_JPRB, 0.20407E+00_JPRB, 0.20431E+00_JPRB, 0.20457E+00_JPRB, 0.20487E+00_JPRB, 0.20486E+00_JPRB, 0.20475E+00_JPRB ,&
&  0.20455E+00_JPRB, 0.20419E+00_JPRB, 0.20377E+00_JPRB, 0.20337E+00_JPRB, 0.20304E+00_JPRB, 0.20265E+00_JPRB, 0.20221E+00_JPRB ,&
&  0.20180E+00_JPRB, 0.20133E+00_JPRB, 0.20088E+00_JPRB, 0.20045E+00_JPRB, 0.20003E+00_JPRB, 0.19968E+00_JPRB, 0.19930E+00_JPRB ,&
&  0.19904E+00_JPRB, 0.19881E+00_JPRB, 0.19856E+00_JPRB, 0.19839E+00_JPRB, 0.19810E+00_JPRB, 0.19801E+00_JPRB, 0.19777E+00_JPRB ,&
&  0.19770E+00_JPRB, 0.19762E+00_JPRB, 0.19758E+00_JPRB, 0.19751E+00_JPRB, 0.19753E+00_JPRB, 0.19753E+00_JPRB, 0.19754E+00_JPRB ,&
&  0.19754E+00_JPRB, 0.19774E+00_JPRB, 0.19761E+00_JPRB, 0.19744E+00_JPRB, 0.19725E+00_JPRB, 0.19707E+00_JPRB, 0.19689E+00_JPRB ,&
&  0.19673E+00_JPRB, 0.19656E+00_JPRB, 0.19646E+00_JPRB, 0.19650E+00_JPRB, 0.19656E+00_JPRB, 0.19666E+00_JPRB, 0.19677E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19702E+00_JPRB, 0.19715E+00_JPRB, 0.19710E+00_JPRB, 0.19712E+00_JPRB, 0.19715E+00_JPRB, 0.19717E+00_JPRB ,&
&  0.19719E+00_JPRB, 0.19724E+00_JPRB, 0.19734E+00_JPRB, 0.19750E+00_JPRB, 0.19785E+00_JPRB, 0.19797E+00_JPRB, 0.19817E+00_JPRB ,&
&  0.19844E+00_JPRB, 0.19893E+00_JPRB, 0.19935E+00_JPRB, 0.19981E+00_JPRB, 0.20032E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB, 0.19885E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB, 0.72384E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.72070E+00_JPRB, 0.71976E+00_JPRB, 0.71893E+00_JPRB, 0.71812E+00_JPRB, 0.71720E+00_JPRB, 0.71658E+00_JPRB, 0.71608E+00_JPRB ,&
&  0.71557E+00_JPRB, 0.71509E+00_JPRB, 0.71457E+00_JPRB, 0.71418E+00_JPRB, 0.71393E+00_JPRB, 0.71357E+00_JPRB, 0.71321E+00_JPRB ,&
&  0.71273E+00_JPRB, 0.71228E+00_JPRB, 0.71200E+00_JPRB, 0.71135E+00_JPRB, 0.71074E+00_JPRB, 0.71018E+00_JPRB, 0.70923E+00_JPRB ,&
&  0.70794E+00_JPRB, 0.70670E+00_JPRB, 0.70536E+00_JPRB, 0.70424E+00_JPRB, 0.70338E+00_JPRB, 0.70212E+00_JPRB, 0.70063E+00_JPRB ,&
&  0.69934E+00_JPRB, 0.69848E+00_JPRB, 0.69794E+00_JPRB, 0.69762E+00_JPRB, 0.69748E+00_JPRB, 0.69743E+00_JPRB, 0.69735E+00_JPRB ,&
&  0.69736E+00_JPRB, 0.69772E+00_JPRB, 0.69844E+00_JPRB, 0.69947E+00_JPRB, 0.70075E+00_JPRB, 0.70233E+00_JPRB, 0.70457E+00_JPRB ,&
&  0.70705E+00_JPRB, 0.70959E+00_JPRB, 0.71223E+00_JPRB, 0.71462E+00_JPRB, 0.71676E+00_JPRB, 0.71864E+00_JPRB, 0.72004E+00_JPRB ,&
&  0.72157E+00_JPRB, 0.72317E+00_JPRB, 0.72483E+00_JPRB, 0.72606E+00_JPRB, 0.72714E+00_JPRB, 0.72814E+00_JPRB, 0.72923E+00_JPRB ,&
&  0.72980E+00_JPRB, 0.73022E+00_JPRB, 0.73073E+00_JPRB, 0.73143E+00_JPRB, 0.73190E+00_JPRB, 0.73221E+00_JPRB, 0.73255E+00_JPRB ,&
&  0.73309E+00_JPRB, 0.73351E+00_JPRB, 0.73371E+00_JPRB, 0.73394E+00_JPRB, 0.73438E+00_JPRB, 0.73502E+00_JPRB, 0.73568E+00_JPRB ,&
&  0.73625E+00_JPRB, 0.73679E+00_JPRB, 0.73716E+00_JPRB, 0.73735E+00_JPRB, 0.73730E+00_JPRB, 0.73691E+00_JPRB, 0.73593E+00_JPRB ,&
&  0.73472E+00_JPRB, 0.73339E+00_JPRB, 0.73226E+00_JPRB, 0.73154E+00_JPRB, 0.73076E+00_JPRB, 0.73011E+00_JPRB, 0.72962E+00_JPRB ,&
&  0.72945E+00_JPRB, 0.72936E+00_JPRB, 0.72926E+00_JPRB, 0.72926E+00_JPRB, 0.72914E+00_JPRB, 0.72877E+00_JPRB, 0.72823E+00_JPRB ,&
&  0.72746E+00_JPRB, 0.72638E+00_JPRB, 0.72506E+00_JPRB, 0.72358E+00_JPRB, 0.72208E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.71116E+00_JPRB, 0.70893E+00_JPRB, 0.70733E+00_JPRB, 0.70630E+00_JPRB, 0.70534E+00_JPRB, 0.70449E+00_JPRB, 0.70360E+00_JPRB ,&
&  0.70273E+00_JPRB, 0.70277E+00_JPRB, 0.70238E+00_JPRB, 0.70211E+00_JPRB, 0.70177E+00_JPRB, 0.70202E+00_JPRB, 0.70194E+00_JPRB ,&
&  0.70148E+00_JPRB, 0.70006E+00_JPRB, 0.69899E+00_JPRB, 0.69730E+00_JPRB, 0.69714E+00_JPRB, 0.69699E+00_JPRB, 0.69826E+00_JPRB ,&
&  0.69765E+00_JPRB, 0.69677E+00_JPRB, 0.69498E+00_JPRB, 0.68978E+00_JPRB, 0.68530E+00_JPRB, 0.68093E+00_JPRB, 0.67718E+00_JPRB ,&
&  0.67369E+00_JPRB, 0.67110E+00_JPRB, 0.67010E+00_JPRB, 0.67003E+00_JPRB, 0.67330E+00_JPRB, 0.67420E+00_JPRB, 0.67449E+00_JPRB ,&
&  0.67360E+00_JPRB, 0.66961E+00_JPRB, 0.67059E+00_JPRB, 0.67188E+00_JPRB, 0.67385E+00_JPRB, 0.67842E+00_JPRB, 0.68243E+00_JPRB ,&
&  0.68719E+00_JPRB, 0.69257E+00_JPRB, 0.70121E+00_JPRB, 0.70738E+00_JPRB, 0.71355E+00_JPRB, 0.71881E+00_JPRB, 0.72165E+00_JPRB ,&
&  0.72437E+00_JPRB, 0.72681E+00_JPRB, 0.72927E+00_JPRB, 0.73120E+00_JPRB, 0.73270E+00_JPRB, 0.73432E+00_JPRB, 0.73623E+00_JPRB ,&
&  0.73747E+00_JPRB, 0.73855E+00_JPRB, 0.73973E+00_JPRB, 0.74089E+00_JPRB, 0.74164E+00_JPRB, 0.74236E+00_JPRB, 0.74327E+00_JPRB ,&
&  0.74421E+00_JPRB, 0.74405E+00_JPRB, 0.74390E+00_JPRB, 0.74456E+00_JPRB, 0.74529E+00_JPRB, 0.74602E+00_JPRB, 0.74590E+00_JPRB ,&
&  0.74601E+00_JPRB, 0.74577E+00_JPRB, 0.74583E+00_JPRB, 0.74594E+00_JPRB, 0.74573E+00_JPRB, 0.74555E+00_JPRB, 0.74473E+00_JPRB ,&
&  0.74349E+00_JPRB, 0.74165E+00_JPRB, 0.74006E+00_JPRB, 0.73930E+00_JPRB, 0.73833E+00_JPRB, 0.73812E+00_JPRB, 0.73823E+00_JPRB ,&
&  0.73877E+00_JPRB, 0.73763E+00_JPRB, 0.73616E+00_JPRB, 0.73417E+00_JPRB, 0.73218E+00_JPRB, 0.73000E+00_JPRB, 0.72793E+00_JPRB ,&
&  0.72583E+00_JPRB, 0.72334E+00_JPRB, 0.72039E+00_JPRB, 0.71722E+00_JPRB, 0.71410E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.71255E+00_JPRB, 0.70846E+00_JPRB, 0.70568E+00_JPRB, 0.70398E+00_JPRB, 0.70231E+00_JPRB, 0.70064E+00_JPRB, 0.69956E+00_JPRB ,&
&  0.69851E+00_JPRB, 0.69651E+00_JPRB, 0.69541E+00_JPRB, 0.69459E+00_JPRB, 0.69333E+00_JPRB, 0.69375E+00_JPRB, 0.69366E+00_JPRB ,&
&  0.69411E+00_JPRB, 0.69337E+00_JPRB, 0.69506E+00_JPRB, 0.69350E+00_JPRB, 0.69229E+00_JPRB, 0.69082E+00_JPRB, 0.68720E+00_JPRB ,&
&  0.68627E+00_JPRB, 0.68507E+00_JPRB, 0.68063E+00_JPRB, 0.67511E+00_JPRB, 0.67181E+00_JPRB, 0.65477E+00_JPRB, 0.65603E+00_JPRB ,&
&  0.66378E+00_JPRB, 0.65239E+00_JPRB, 0.64671E+00_JPRB, 0.64081E+00_JPRB, 0.63248E+00_JPRB, 0.63253E+00_JPRB, 0.63695E+00_JPRB ,&
&  0.63152E+00_JPRB, 0.64777E+00_JPRB, 0.65007E+00_JPRB, 0.65153E+00_JPRB, 0.65337E+00_JPRB, 0.65081E+00_JPRB, 0.65570E+00_JPRB ,&
&  0.67528E+00_JPRB, 0.67883E+00_JPRB, 0.68742E+00_JPRB, 0.69518E+00_JPRB, 0.70493E+00_JPRB, 0.71396E+00_JPRB, 0.72390E+00_JPRB ,&
&  0.73132E+00_JPRB, 0.73650E+00_JPRB, 0.74011E+00_JPRB, 0.74087E+00_JPRB, 0.74166E+00_JPRB, 0.74172E+00_JPRB, 0.74291E+00_JPRB ,&
&  0.74276E+00_JPRB, 0.74401E+00_JPRB, 0.74624E+00_JPRB, 0.74894E+00_JPRB, 0.75254E+00_JPRB, 0.75479E+00_JPRB, 0.75688E+00_JPRB ,&
&  0.75821E+00_JPRB, 0.75854E+00_JPRB, 0.75813E+00_JPRB, 0.75797E+00_JPRB, 0.75674E+00_JPRB, 0.75629E+00_JPRB, 0.75568E+00_JPRB ,&
&  0.75626E+00_JPRB, 0.75601E+00_JPRB, 0.75660E+00_JPRB, 0.75682E+00_JPRB, 0.75785E+00_JPRB, 0.75990E+00_JPRB, 0.76157E+00_JPRB ,&
&  0.76399E+00_JPRB, 0.76512E+00_JPRB, 0.76552E+00_JPRB, 0.76534E+00_JPRB, 0.76473E+00_JPRB, 0.76501E+00_JPRB, 0.76508E+00_JPRB ,&
&  0.76570E+00_JPRB, 0.76226E+00_JPRB, 0.75816E+00_JPRB, 0.75377E+00_JPRB, 0.74994E+00_JPRB, 0.74644E+00_JPRB, 0.74269E+00_JPRB ,&
&  0.73923E+00_JPRB, 0.73451E+00_JPRB, 0.72924E+00_JPRB, 0.72327E+00_JPRB, 0.71751E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.72760E+00_JPRB, 0.72140E+00_JPRB, 0.71589E+00_JPRB, 0.71199E+00_JPRB, 0.70832E+00_JPRB, 0.70530E+00_JPRB, 0.70278E+00_JPRB ,&
&  0.70053E+00_JPRB, 0.69862E+00_JPRB, 0.69752E+00_JPRB, 0.69638E+00_JPRB, 0.69553E+00_JPRB, 0.69432E+00_JPRB, 0.69353E+00_JPRB ,&
&  0.69513E+00_JPRB, 0.69473E+00_JPRB, 0.69653E+00_JPRB, 0.69428E+00_JPRB, 0.69234E+00_JPRB, 0.68817E+00_JPRB, 0.67864E+00_JPRB ,&
&  0.67412E+00_JPRB, 0.67009E+00_JPRB, 0.65560E+00_JPRB, 0.66678E+00_JPRB, 0.66116E+00_JPRB, 0.65767E+00_JPRB, 0.65223E+00_JPRB ,&
&  0.63942E+00_JPRB, 0.63310E+00_JPRB, 0.62505E+00_JPRB, 0.62465E+00_JPRB, 0.62960E+00_JPRB, 0.62727E+00_JPRB, 0.63118E+00_JPRB ,&
&  0.62704E+00_JPRB, 0.62559E+00_JPRB, 0.62646E+00_JPRB, 0.62654E+00_JPRB, 0.63131E+00_JPRB, 0.63204E+00_JPRB, 0.63607E+00_JPRB ,&
&  0.64537E+00_JPRB, 0.65799E+00_JPRB, 0.67654E+00_JPRB, 0.68395E+00_JPRB, 0.69906E+00_JPRB, 0.71134E+00_JPRB, 0.72294E+00_JPRB ,&
&  0.73049E+00_JPRB, 0.73522E+00_JPRB, 0.73857E+00_JPRB, 0.74008E+00_JPRB, 0.73995E+00_JPRB, 0.73666E+00_JPRB, 0.73646E+00_JPRB ,&
&  0.73687E+00_JPRB, 0.73860E+00_JPRB, 0.74100E+00_JPRB, 0.74331E+00_JPRB, 0.74678E+00_JPRB, 0.75005E+00_JPRB, 0.75437E+00_JPRB ,&
&  0.75832E+00_JPRB, 0.76243E+00_JPRB, 0.76334E+00_JPRB, 0.76530E+00_JPRB, 0.76638E+00_JPRB, 0.76756E+00_JPRB, 0.76864E+00_JPRB ,&
&  0.76996E+00_JPRB, 0.77174E+00_JPRB, 0.77503E+00_JPRB, 0.77217E+00_JPRB, 0.77412E+00_JPRB, 0.78509E+00_JPRB, 0.78797E+00_JPRB ,&
&  0.79174E+00_JPRB, 0.79230E+00_JPRB, 0.79045E+00_JPRB, 0.78841E+00_JPRB, 0.78637E+00_JPRB, 0.78570E+00_JPRB, 0.78531E+00_JPRB ,&
&  0.78474E+00_JPRB, 0.77966E+00_JPRB, 0.77447E+00_JPRB, 0.76982E+00_JPRB, 0.76476E+00_JPRB, 0.76088E+00_JPRB, 0.75693E+00_JPRB ,&
&  0.75412E+00_JPRB, 0.75167E+00_JPRB, 0.74858E+00_JPRB, 0.74301E+00_JPRB, 0.73580E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.74636E+00_JPRB, 0.73930E+00_JPRB, 0.73366E+00_JPRB, 0.72889E+00_JPRB, 0.72535E+00_JPRB, 0.72236E+00_JPRB, 0.71939E+00_JPRB ,&
&  0.71639E+00_JPRB, 0.71289E+00_JPRB, 0.70923E+00_JPRB, 0.70661E+00_JPRB, 0.70456E+00_JPRB, 0.70262E+00_JPRB, 0.70038E+00_JPRB ,&
&  0.69927E+00_JPRB, 0.69773E+00_JPRB, 0.69866E+00_JPRB, 0.69607E+00_JPRB, 0.69681E+00_JPRB, 0.69216E+00_JPRB, 0.68122E+00_JPRB ,&
&  0.67073E+00_JPRB, 0.65984E+00_JPRB, 0.64855E+00_JPRB, 0.65841E+00_JPRB, 0.65737E+00_JPRB, 0.65772E+00_JPRB, 0.64392E+00_JPRB ,&
&  0.63140E+00_JPRB, 0.61867E+00_JPRB, 0.63871E+00_JPRB, 0.63135E+00_JPRB, 0.62716E+00_JPRB, 0.61941E+00_JPRB, 0.60332E+00_JPRB ,&
&  0.60041E+00_JPRB, 0.59095E+00_JPRB, 0.59031E+00_JPRB, 0.59462E+00_JPRB, 0.60033E+00_JPRB, 0.60861E+00_JPRB, 0.61691E+00_JPRB ,&
&  0.61687E+00_JPRB, 0.64826E+00_JPRB, 0.66918E+00_JPRB, 0.68478E+00_JPRB, 0.69983E+00_JPRB, 0.71222E+00_JPRB, 0.72122E+00_JPRB ,&
&  0.72935E+00_JPRB, 0.73559E+00_JPRB, 0.74161E+00_JPRB, 0.74366E+00_JPRB, 0.75283E+00_JPRB, 0.75493E+00_JPRB, 0.75801E+00_JPRB ,&
&  0.76162E+00_JPRB, 0.76149E+00_JPRB, 0.76030E+00_JPRB, 0.75719E+00_JPRB, 0.75591E+00_JPRB, 0.75363E+00_JPRB, 0.75502E+00_JPRB ,&
&  0.75552E+00_JPRB, 0.75741E+00_JPRB, 0.76223E+00_JPRB, 0.76681E+00_JPRB, 0.77099E+00_JPRB, 0.77500E+00_JPRB, 0.77924E+00_JPRB ,&
&  0.78320E+00_JPRB, 0.79056E+00_JPRB, 0.79771E+00_JPRB, 0.81061E+00_JPRB, 0.82171E+00_JPRB, 0.82713E+00_JPRB, 0.83109E+00_JPRB ,&
&  0.83589E+00_JPRB, 0.83047E+00_JPRB, 0.82517E+00_JPRB, 0.82160E+00_JPRB, 0.81948E+00_JPRB, 0.81843E+00_JPRB, 0.81869E+00_JPRB ,&
&  0.81765E+00_JPRB, 0.81288E+00_JPRB, 0.80707E+00_JPRB, 0.80016E+00_JPRB, 0.79334E+00_JPRB, 0.78802E+00_JPRB, 0.78342E+00_JPRB ,&
&  0.77940E+00_JPRB, 0.77534E+00_JPRB, 0.76898E+00_JPRB, 0.76132E+00_JPRB, 0.75360E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.77291E+00_JPRB, 0.76569E+00_JPRB, 0.76005E+00_JPRB, 0.75497E+00_JPRB, 0.75151E+00_JPRB, 0.74856E+00_JPRB, 0.74589E+00_JPRB ,&
&  0.74377E+00_JPRB, 0.73914E+00_JPRB, 0.73373E+00_JPRB, 0.72711E+00_JPRB, 0.72247E+00_JPRB, 0.71826E+00_JPRB, 0.71361E+00_JPRB ,&
&  0.70695E+00_JPRB, 0.69982E+00_JPRB, 0.69215E+00_JPRB, 0.68603E+00_JPRB, 0.68720E+00_JPRB, 0.68381E+00_JPRB, 0.67872E+00_JPRB ,&
&  0.66876E+00_JPRB, 0.65702E+00_JPRB, 0.64761E+00_JPRB, 0.65053E+00_JPRB, 0.65295E+00_JPRB, 0.64377E+00_JPRB, 0.64210E+00_JPRB ,&
&  0.65421E+00_JPRB, 0.65595E+00_JPRB, 0.64981E+00_JPRB, 0.64678E+00_JPRB, 0.61999E+00_JPRB, 0.58223E+00_JPRB, 0.60357E+00_JPRB ,&
&  0.60328E+00_JPRB, 0.58683E+00_JPRB, 0.58347E+00_JPRB, 0.58131E+00_JPRB, 0.58801E+00_JPRB, 0.60299E+00_JPRB, 0.61709E+00_JPRB ,&
&  0.62077E+00_JPRB, 0.67435E+00_JPRB, 0.69712E+00_JPRB, 0.71305E+00_JPRB, 0.72337E+00_JPRB, 0.73225E+00_JPRB, 0.73954E+00_JPRB ,&
&  0.74623E+00_JPRB, 0.75661E+00_JPRB, 0.77024E+00_JPRB, 0.78387E+00_JPRB, 0.79698E+00_JPRB, 0.80676E+00_JPRB, 0.81727E+00_JPRB ,&
&  0.83171E+00_JPRB, 0.83416E+00_JPRB, 0.82243E+00_JPRB, 0.80943E+00_JPRB, 0.79792E+00_JPRB, 0.78716E+00_JPRB, 0.77798E+00_JPRB ,&
&  0.77422E+00_JPRB, 0.77195E+00_JPRB, 0.77795E+00_JPRB, 0.77980E+00_JPRB, 0.78376E+00_JPRB, 0.78884E+00_JPRB, 0.80107E+00_JPRB ,&
&  0.80914E+00_JPRB, 0.82121E+00_JPRB, 0.83519E+00_JPRB, 0.85374E+00_JPRB, 0.86815E+00_JPRB, 0.87435E+00_JPRB, 0.87789E+00_JPRB ,&
&  0.87837E+00_JPRB, 0.88468E+00_JPRB, 0.88592E+00_JPRB, 0.89041E+00_JPRB, 0.89869E+00_JPRB, 0.90259E+00_JPRB, 0.90716E+00_JPRB ,&
&  0.90913E+00_JPRB, 0.90840E+00_JPRB, 0.90294E+00_JPRB, 0.89366E+00_JPRB, 0.88087E+00_JPRB, 0.86647E+00_JPRB, 0.84934E+00_JPRB ,&
&  0.83178E+00_JPRB, 0.81317E+00_JPRB, 0.79758E+00_JPRB, 0.78769E+00_JPRB, 0.77946E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.81981E+00_JPRB, 0.80535E+00_JPRB, 0.79487E+00_JPRB, 0.78872E+00_JPRB, 0.78370E+00_JPRB, 0.77928E+00_JPRB, 0.77511E+00_JPRB ,&
&  0.77095E+00_JPRB, 0.76637E+00_JPRB, 0.76017E+00_JPRB, 0.75336E+00_JPRB, 0.74760E+00_JPRB, 0.74208E+00_JPRB, 0.73760E+00_JPRB ,&
&  0.72945E+00_JPRB, 0.71024E+00_JPRB, 0.70589E+00_JPRB, 0.69616E+00_JPRB, 0.68080E+00_JPRB, 0.67201E+00_JPRB, 0.66527E+00_JPRB ,&
&  0.65719E+00_JPRB, 0.65386E+00_JPRB, 0.65457E+00_JPRB, 0.64925E+00_JPRB, 0.64696E+00_JPRB, 0.65727E+00_JPRB, 0.66022E+00_JPRB ,&
&  0.63655E+00_JPRB, 0.63642E+00_JPRB, 0.66778E+00_JPRB, 0.66862E+00_JPRB, 0.63290E+00_JPRB, 0.62183E+00_JPRB, 0.62474E+00_JPRB ,&
&  0.62236E+00_JPRB, 0.59727E+00_JPRB, 0.59251E+00_JPRB, 0.59070E+00_JPRB, 0.60164E+00_JPRB, 0.61334E+00_JPRB, 0.64192E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.71435E+00_JPRB, 0.73634E+00_JPRB, 0.75109E+00_JPRB, 0.76280E+00_JPRB, 0.77545E+00_JPRB, 0.78901E+00_JPRB ,&
&  0.80782E+00_JPRB, 0.83043E+00_JPRB, 0.86029E+00_JPRB, 0.88745E+00_JPRB, 0.91392E+00_JPRB, 0.92649E+00_JPRB, 0.92854E+00_JPRB ,&
&  0.95148E+00_JPRB, 0.95472E+00_JPRB, 0.92474E+00_JPRB, 0.90066E+00_JPRB, 0.87532E+00_JPRB, 0.85114E+00_JPRB, 0.82543E+00_JPRB ,&
&  0.81920E+00_JPRB, 0.82744E+00_JPRB, 0.82100E+00_JPRB, 0.81617E+00_JPRB, 0.81549E+00_JPRB, 0.81725E+00_JPRB, 0.82378E+00_JPRB ,&
&  0.82801E+00_JPRB, 0.86053E+00_JPRB, 0.96421E+00_JPRB, 0.99086E+00_JPRB, 0.10002E+01_JPRB, 0.10017E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.10185E+01_JPRB, 0.10327E+01_JPRB, 0.10394E+01_JPRB, 0.10466E+01_JPRB, 0.10392E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10383E+01_JPRB, 0.10293E+01_JPRB, 0.10174E+01_JPRB, 0.10082E+01_JPRB, 0.99707E+00_JPRB, 0.98621E+00_JPRB, 0.96979E+00_JPRB ,&
&  0.94819E+00_JPRB, 0.92267E+00_JPRB, 0.89643E+00_JPRB, 0.86861E+00_JPRB, 0.84183E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.88811E+00_JPRB, 0.85401E+00_JPRB, 0.83095E+00_JPRB, 0.82090E+00_JPRB, 0.81452E+00_JPRB, 0.79739E+00_JPRB, 0.78434E+00_JPRB ,&
&  0.77519E+00_JPRB, 0.77156E+00_JPRB, 0.76551E+00_JPRB, 0.75731E+00_JPRB, 0.75228E+00_JPRB, 0.74779E+00_JPRB, 0.74545E+00_JPRB ,&
&  0.74202E+00_JPRB, 0.73347E+00_JPRB, 0.71543E+00_JPRB, 0.69448E+00_JPRB, 0.67953E+00_JPRB, 0.66776E+00_JPRB, 0.65869E+00_JPRB ,&
&  0.65116E+00_JPRB, 0.64976E+00_JPRB, 0.65828E+00_JPRB, 0.65806E+00_JPRB, 0.65730E+00_JPRB, 0.65290E+00_JPRB, 0.64884E+00_JPRB ,&
&  0.65196E+00_JPRB, 0.66481E+00_JPRB, 0.64792E+00_JPRB, 0.65172E+00_JPRB, 0.66574E+00_JPRB, 0.65278E+00_JPRB, 0.63850E+00_JPRB ,&
&  0.62663E+00_JPRB, 0.61607E+00_JPRB, 0.61096E+00_JPRB, 0.61099E+00_JPRB, 0.62905E+00_JPRB, 0.65180E+00_JPRB, 0.68899E+00_JPRB ,&
&  0.73246E+00_JPRB, 0.76041E+00_JPRB, 0.77622E+00_JPRB, 0.79047E+00_JPRB, 0.80533E+00_JPRB, 0.82199E+00_JPRB, 0.84893E+00_JPRB ,&
&  0.89297E+00_JPRB, 0.94046E+00_JPRB, 0.98764E+00_JPRB, 0.10191E+01_JPRB, 0.10523E+01_JPRB, 0.10281E+01_JPRB, 0.10312E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11909E+01_JPRB, 0.11273E+01_JPRB, 0.10735E+01_JPRB, 0.10189E+01_JPRB, 0.97177E+00_JPRB, 0.93380E+00_JPRB ,&
&  0.90919E+00_JPRB, 0.89638E+00_JPRB, 0.88676E+00_JPRB, 0.87170E+00_JPRB, 0.86223E+00_JPRB, 0.86951E+00_JPRB, 0.89703E+00_JPRB ,&
&  0.95245E+00_JPRB, 0.11085E+01_JPRB, 0.14224E+01_JPRB, 0.12403E+01_JPRB, 0.11726E+01_JPRB, 0.11612E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.11890E+01_JPRB, 0.12085E+01_JPRB, 0.12175E+01_JPRB, 0.11960E+01_JPRB, 0.11576E+01_JPRB, 0.11126E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10324E+01_JPRB, 0.10156E+01_JPRB, 0.10099E+01_JPRB, 0.99827E+00_JPRB, 0.98601E+00_JPRB, 0.97492E+00_JPRB ,&
&  0.96274E+00_JPRB, 0.95525E+00_JPRB, 0.95211E+00_JPRB, 0.94191E+00_JPRB, 0.92443E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.90502E+00_JPRB, 0.91056E+00_JPRB, 0.90138E+00_JPRB, 0.88515E+00_JPRB, 0.86717E+00_JPRB, 0.82360E+00_JPRB, 0.79004E+00_JPRB ,&
&  0.77286E+00_JPRB, 0.76641E+00_JPRB, 0.76590E+00_JPRB, 0.76333E+00_JPRB, 0.76122E+00_JPRB, 0.75749E+00_JPRB, 0.74971E+00_JPRB ,&
&  0.73810E+00_JPRB, 0.72912E+00_JPRB, 0.72105E+00_JPRB, 0.70744E+00_JPRB, 0.68964E+00_JPRB, 0.67095E+00_JPRB, 0.65619E+00_JPRB ,&
&  0.64601E+00_JPRB, 0.64226E+00_JPRB, 0.64977E+00_JPRB, 0.65778E+00_JPRB, 0.66629E+00_JPRB, 0.66953E+00_JPRB, 0.66688E+00_JPRB ,&
&  0.65390E+00_JPRB, 0.65035E+00_JPRB, 0.66065E+00_JPRB, 0.66164E+00_JPRB, 0.66728E+00_JPRB, 0.67077E+00_JPRB, 0.65888E+00_JPRB ,&
&  0.64757E+00_JPRB, 0.63882E+00_JPRB, 0.64225E+00_JPRB, 0.65670E+00_JPRB, 0.69898E+00_JPRB, 0.74980E+00_JPRB, 0.77128E+00_JPRB ,&
&  0.79108E+00_JPRB, 0.80319E+00_JPRB, 0.81516E+00_JPRB, 0.83522E+00_JPRB, 0.86203E+00_JPRB, 0.89447E+00_JPRB, 0.95462E+00_JPRB ,&
&  0.10390E+01_JPRB, 0.11021E+01_JPRB, 0.11604E+01_JPRB, 0.10874E+01_JPRB, 0.10602E+01_JPRB, 0.10881E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.12259E+01_JPRB, 0.12197E+01_JPRB, 0.12272E+01_JPRB, 0.11794E+01_JPRB, 0.11193E+01_JPRB, 0.10379E+01_JPRB ,&
&  0.97764E+00_JPRB, 0.94124E+00_JPRB, 0.92994E+00_JPRB, 0.92072E+00_JPRB, 0.93697E+00_JPRB, 0.97095E+00_JPRB, 0.10289E+01_JPRB ,&
&  0.11209E+01_JPRB, 0.12319E+01_JPRB, 0.12930E+01_JPRB, 0.12560E+01_JPRB, 0.12822E+01_JPRB, 0.13393E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.13742E+01_JPRB, 0.13785E+01_JPRB, 0.13644E+01_JPRB, 0.12763E+01_JPRB, 0.11970E+01_JPRB, 0.11079E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.94581E+00_JPRB, 0.90608E+00_JPRB, 0.89454E+00_JPRB, 0.88583E+00_JPRB, 0.87537E+00_JPRB, 0.85675E+00_JPRB ,&
&  0.83828E+00_JPRB, 0.83562E+00_JPRB, 0.84846E+00_JPRB, 0.86607E+00_JPRB, 0.88997E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.94774E+00_JPRB, 0.96599E+00_JPRB, 0.97310E+00_JPRB, 0.97365E+00_JPRB, 0.93262E+00_JPRB, 0.89204E+00_JPRB, 0.85235E+00_JPRB ,&
&  0.83076E+00_JPRB, 0.80552E+00_JPRB, 0.78879E+00_JPRB, 0.76855E+00_JPRB, 0.75061E+00_JPRB, 0.74099E+00_JPRB, 0.73961E+00_JPRB ,&
&  0.73982E+00_JPRB, 0.73207E+00_JPRB, 0.72092E+00_JPRB, 0.70479E+00_JPRB, 0.69075E+00_JPRB, 0.67783E+00_JPRB, 0.66616E+00_JPRB ,&
&  0.65700E+00_JPRB, 0.65540E+00_JPRB, 0.65915E+00_JPRB, 0.66832E+00_JPRB, 0.67534E+00_JPRB, 0.67936E+00_JPRB, 0.68011E+00_JPRB ,&
&  0.67449E+00_JPRB, 0.66567E+00_JPRB, 0.66941E+00_JPRB, 0.67906E+00_JPRB, 0.68561E+00_JPRB, 0.69427E+00_JPRB, 0.67889E+00_JPRB ,&
&  0.66985E+00_JPRB, 0.67474E+00_JPRB, 0.69760E+00_JPRB, 0.75765E+00_JPRB, 0.78736E+00_JPRB, 0.80400E+00_JPRB, 0.81618E+00_JPRB ,&
&  0.80618E+00_JPRB, 0.80602E+00_JPRB, 0.82630E+00_JPRB, 0.87377E+00_JPRB, 0.92525E+00_JPRB, 0.10128E+01_JPRB, 0.11245E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.12444E+01_JPRB, 0.12666E+01_JPRB, 0.11895E+01_JPRB, 0.11920E+01_JPRB, 0.13416E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.14224E+01_JPRB, 0.14401E+01_JPRB, 0.14815E+01_JPRB, 0.14407E+01_JPRB, 0.13133E+01_JPRB, 0.12194E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.10474E+01_JPRB, 0.10483E+01_JPRB, 0.10831E+01_JPRB, 0.11545E+01_JPRB, 0.12911E+01_JPRB, 0.14614E+01_JPRB ,&
&  0.16232E+01_JPRB, 0.17191E+01_JPRB, 0.17782E+01_JPRB, 0.17668E+01_JPRB, 0.17435E+01_JPRB, 0.17085E+01_JPRB, 0.16823E+01_JPRB ,&
&  0.16493E+01_JPRB, 0.16386E+01_JPRB, 0.16230E+01_JPRB, 0.16054E+01_JPRB, 0.14957E+01_JPRB, 0.13114E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.99750E+00_JPRB, 0.92252E+00_JPRB, 0.87138E+00_JPRB, 0.85939E+00_JPRB, 0.86190E+00_JPRB, 0.86373E+00_JPRB, 0.84097E+00_JPRB ,&
&  0.83081E+00_JPRB, 0.84252E+00_JPRB, 0.86230E+00_JPRB, 0.88261E+00_JPRB, 0.91376E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.10262E+01_JPRB, 0.10429E+01_JPRB, 0.10502E+01_JPRB, 0.10403E+01_JPRB, 0.10101E+01_JPRB, 0.95589E+00_JPRB, 0.88998E+00_JPRB ,&
&  0.86632E+00_JPRB, 0.82996E+00_JPRB, 0.81305E+00_JPRB, 0.79391E+00_JPRB, 0.77690E+00_JPRB, 0.75368E+00_JPRB, 0.73063E+00_JPRB ,&
&  0.72447E+00_JPRB, 0.72834E+00_JPRB, 0.72798E+00_JPRB, 0.72002E+00_JPRB, 0.70857E+00_JPRB, 0.69551E+00_JPRB, 0.68411E+00_JPRB ,&
&  0.67670E+00_JPRB, 0.67667E+00_JPRB, 0.68539E+00_JPRB, 0.69669E+00_JPRB, 0.70187E+00_JPRB, 0.70497E+00_JPRB, 0.71069E+00_JPRB ,&
&  0.71048E+00_JPRB, 0.70289E+00_JPRB, 0.70071E+00_JPRB, 0.70522E+00_JPRB, 0.70872E+00_JPRB, 0.72791E+00_JPRB, 0.72952E+00_JPRB ,&
&  0.70353E+00_JPRB, 0.72529E+00_JPRB, 0.77823E+00_JPRB, 0.80504E+00_JPRB, 0.82300E+00_JPRB, 0.83627E+00_JPRB, 0.84771E+00_JPRB ,&
&  0.85814E+00_JPRB, 0.87474E+00_JPRB, 0.89411E+00_JPRB, 0.91578E+00_JPRB, 0.10017E+01_JPRB, 0.11988E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.12408E+01_JPRB, 0.11770E+01_JPRB, 0.11949E+01_JPRB, 0.13706E+01_JPRB, 0.14727E+01_JPRB, 0.15383E+01_JPRB ,&
&  0.16351E+01_JPRB, 0.16460E+01_JPRB, 0.16271E+01_JPRB, 0.15662E+01_JPRB, 0.14599E+01_JPRB, 0.13812E+01_JPRB, 0.13626E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.13362E+01_JPRB, 0.14180E+01_JPRB, 0.15880E+01_JPRB, 0.17835E+01_JPRB, 0.19466E+01_JPRB, 0.20638E+01_JPRB ,&
&  0.21744E+01_JPRB, 0.22262E+01_JPRB, 0.22018E+01_JPRB, 0.21534E+01_JPRB, 0.20658E+01_JPRB, 0.20147E+01_JPRB, 0.19944E+01_JPRB ,&
&  0.19676E+01_JPRB, 0.19446E+01_JPRB, 0.18966E+01_JPRB, 0.18272E+01_JPRB, 0.17110E+01_JPRB, 0.15952E+01_JPRB, 0.14326E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.10331E+01_JPRB, 0.92548E+00_JPRB, 0.89557E+00_JPRB, 0.89825E+00_JPRB, 0.91023E+00_JPRB, 0.90598E+00_JPRB ,&
&  0.92250E+00_JPRB, 0.93405E+00_JPRB, 0.93854E+00_JPRB, 0.96410E+00_JPRB, 0.99725E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.11061E+01_JPRB, 0.11125E+01_JPRB, 0.11065E+01_JPRB, 0.10975E+01_JPRB, 0.10761E+01_JPRB, 0.10263E+01_JPRB, 0.97971E+00_JPRB ,&
&  0.10196E+01_JPRB, 0.97445E+00_JPRB, 0.97927E+00_JPRB, 0.86810E+00_JPRB, 0.82585E+00_JPRB, 0.80977E+00_JPRB, 0.77914E+00_JPRB ,&
&  0.76141E+00_JPRB, 0.75516E+00_JPRB, 0.74579E+00_JPRB, 0.74187E+00_JPRB, 0.73501E+00_JPRB, 0.72063E+00_JPRB, 0.70953E+00_JPRB ,&
&  0.70528E+00_JPRB, 0.71026E+00_JPRB, 0.72142E+00_JPRB, 0.73072E+00_JPRB, 0.73916E+00_JPRB, 0.74642E+00_JPRB, 0.75152E+00_JPRB ,&
&  0.73969E+00_JPRB, 0.73493E+00_JPRB, 0.73445E+00_JPRB, 0.72986E+00_JPRB, 0.73487E+00_JPRB, 0.74860E+00_JPRB, 0.77276E+00_JPRB ,&
&  0.79032E+00_JPRB, 0.78217E+00_JPRB, 0.82565E+00_JPRB, 0.84731E+00_JPRB, 0.86273E+00_JPRB, 0.87431E+00_JPRB, 0.88233E+00_JPRB ,&
&  0.89075E+00_JPRB, 0.90673E+00_JPRB, 0.92742E+00_JPRB, 0.95274E+00_JPRB, 0.11089E+01_JPRB, 0.14218E+01_JPRB, 0.14614E+01_JPRB ,&
&  0.14593E+01_JPRB, 0.12650E+01_JPRB, 0.13970E+01_JPRB, 0.14644E+01_JPRB, 0.16036E+01_JPRB, 0.17355E+01_JPRB, 0.17480E+01_JPRB ,&
&  0.19571E+01_JPRB, 0.20513E+01_JPRB, 0.19887E+01_JPRB, 0.18509E+01_JPRB, 0.17241E+01_JPRB, 0.16674E+01_JPRB, 0.16531E+01_JPRB ,&
&  0.16900E+01_JPRB, 0.16760E+01_JPRB, 0.18917E+01_JPRB, 0.20776E+01_JPRB, 0.21762E+01_JPRB, 0.22678E+01_JPRB, 0.23750E+01_JPRB ,&
&  0.24481E+01_JPRB, 0.24590E+01_JPRB, 0.24545E+01_JPRB, 0.23295E+01_JPRB, 0.22658E+01_JPRB, 0.22161E+01_JPRB, 0.21236E+01_JPRB ,&
&  0.20642E+01_JPRB, 0.20166E+01_JPRB, 0.19486E+01_JPRB, 0.19614E+01_JPRB, 0.17945E+01_JPRB, 0.16735E+01_JPRB, 0.15962E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.11720E+01_JPRB, 0.10884E+01_JPRB, 0.10543E+01_JPRB, 0.10284E+01_JPRB, 0.10244E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.10141E+01_JPRB, 0.10625E+01_JPRB, 0.10795E+01_JPRB, 0.10894E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.11827E+01_JPRB, 0.11907E+01_JPRB, 0.11860E+01_JPRB, 0.11767E+01_JPRB, 0.11638E+01_JPRB, 0.11393E+01_JPRB, 0.10858E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10600E+01_JPRB, 0.10202E+01_JPRB, 0.99455E+00_JPRB, 0.99043E+00_JPRB, 0.92751E+00_JPRB, 0.82535E+00_JPRB ,&
&  0.78635E+00_JPRB, 0.78449E+00_JPRB, 0.78999E+00_JPRB, 0.78729E+00_JPRB, 0.78180E+00_JPRB, 0.77292E+00_JPRB, 0.76665E+00_JPRB ,&
&  0.76561E+00_JPRB, 0.76924E+00_JPRB, 0.78110E+00_JPRB, 0.79611E+00_JPRB, 0.80005E+00_JPRB, 0.80452E+00_JPRB, 0.81101E+00_JPRB ,&
&  0.79044E+00_JPRB, 0.77134E+00_JPRB, 0.75875E+00_JPRB, 0.74352E+00_JPRB, 0.76960E+00_JPRB, 0.78979E+00_JPRB, 0.81698E+00_JPRB ,&
&  0.84363E+00_JPRB, 0.86785E+00_JPRB, 0.89259E+00_JPRB, 0.91046E+00_JPRB, 0.91927E+00_JPRB, 0.92677E+00_JPRB, 0.93328E+00_JPRB ,&
&  0.94251E+00_JPRB, 0.95986E+00_JPRB, 0.98429E+00_JPRB, 0.10120E+01_JPRB, 0.13154E+01_JPRB, 0.16491E+01_JPRB, 0.17644E+01_JPRB ,&
&  0.17277E+01_JPRB, 0.18780E+01_JPRB, 0.19881E+01_JPRB, 0.19172E+01_JPRB, 0.20372E+01_JPRB, 0.21180E+01_JPRB, 0.20762E+01_JPRB ,&
&  0.21924E+01_JPRB, 0.22129E+01_JPRB, 0.21931E+01_JPRB, 0.21655E+01_JPRB, 0.21545E+01_JPRB, 0.21110E+01_JPRB, 0.20830E+01_JPRB ,&
&  0.21414E+01_JPRB, 0.23376E+01_JPRB, 0.23852E+01_JPRB, 0.24543E+01_JPRB, 0.24705E+01_JPRB, 0.25123E+01_JPRB, 0.25563E+01_JPRB ,&
&  0.25914E+01_JPRB, 0.26398E+01_JPRB, 0.26536E+01_JPRB, 0.25889E+01_JPRB, 0.24945E+01_JPRB, 0.23928E+01_JPRB, 0.22666E+01_JPRB ,&
&  0.21680E+01_JPRB, 0.19609E+01_JPRB, 0.17968E+01_JPRB, 0.17446E+01_JPRB, 0.16949E+01_JPRB, 0.15601E+01_JPRB, 0.15152E+01_JPRB ,&
&  0.14629E+01_JPRB, 0.13665E+01_JPRB, 0.14082E+01_JPRB, 0.13229E+01_JPRB, 0.12512E+01_JPRB, 0.11688E+01_JPRB, 0.10610E+01_JPRB ,&
&  0.10291E+01_JPRB, 0.10916E+01_JPRB, 0.11448E+01_JPRB, 0.11738E+01_JPRB, 0.11786E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.12549E+01_JPRB, 0.12668E+01_JPRB, 0.12719E+01_JPRB, 0.12670E+01_JPRB, 0.12446E+01_JPRB, 0.12114E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.11037E+01_JPRB, 0.10693E+01_JPRB, 0.10426E+01_JPRB, 0.10308E+01_JPRB, 0.10173E+01_JPRB, 0.95053E+00_JPRB ,&
&  0.86445E+00_JPRB, 0.83836E+00_JPRB, 0.81685E+00_JPRB, 0.81318E+00_JPRB, 0.82387E+00_JPRB, 0.82952E+00_JPRB, 0.83609E+00_JPRB ,&
&  0.85545E+00_JPRB, 0.86864E+00_JPRB, 0.87523E+00_JPRB, 0.88138E+00_JPRB, 0.88485E+00_JPRB, 0.89429E+00_JPRB, 0.90311E+00_JPRB ,&
&  0.87025E+00_JPRB, 0.82904E+00_JPRB, 0.80707E+00_JPRB, 0.79927E+00_JPRB, 0.82625E+00_JPRB, 0.86163E+00_JPRB, 0.89955E+00_JPRB ,&
&  0.93143E+00_JPRB, 0.95629E+00_JPRB, 0.97901E+00_JPRB, 0.99471E+00_JPRB, 0.99898E+00_JPRB, 0.10034E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10466E+01_JPRB, 0.10595E+01_JPRB, 0.11022E+01_JPRB, 0.15874E+01_JPRB, 0.21692E+01_JPRB, 0.24585E+01_JPRB ,&
&  0.24551E+01_JPRB, 0.24994E+01_JPRB, 0.25712E+01_JPRB, 0.27542E+01_JPRB, 0.27614E+01_JPRB, 0.26033E+01_JPRB, 0.24647E+01_JPRB ,&
&  0.25037E+01_JPRB, 0.24134E+01_JPRB, 0.26161E+01_JPRB, 0.26164E+01_JPRB, 0.26557E+01_JPRB, 0.27225E+01_JPRB, 0.27074E+01_JPRB ,&
&  0.27766E+01_JPRB, 0.28132E+01_JPRB, 0.27918E+01_JPRB, 0.27069E+01_JPRB, 0.26677E+01_JPRB, 0.27047E+01_JPRB, 0.27385E+01_JPRB ,&
&  0.28367E+01_JPRB, 0.29160E+01_JPRB, 0.27994E+01_JPRB, 0.27033E+01_JPRB, 0.25767E+01_JPRB, 0.26957E+01_JPRB, 0.24877E+01_JPRB ,&
&  0.21463E+01_JPRB, 0.17361E+01_JPRB, 0.17218E+01_JPRB, 0.17129E+01_JPRB, 0.17300E+01_JPRB, 0.16793E+01_JPRB, 0.15694E+01_JPRB ,&
&  0.15634E+01_JPRB, 0.16846E+01_JPRB, 0.16974E+01_JPRB, 0.15276E+01_JPRB, 0.14288E+01_JPRB, 0.13104E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.12136E+01_JPRB, 0.12384E+01_JPRB, 0.12359E+01_JPRB, 0.12351E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.13770E+01_JPRB, 0.13451E+01_JPRB, 0.13314E+01_JPRB, 0.13213E+01_JPRB, 0.12965E+01_JPRB, 0.12664E+01_JPRB, 0.12311E+01_JPRB ,&
&  0.12020E+01_JPRB, 0.11752E+01_JPRB, 0.11486E+01_JPRB, 0.11257E+01_JPRB, 0.11111E+01_JPRB, 0.10878E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.95717E+00_JPRB, 0.90986E+00_JPRB, 0.88100E+00_JPRB, 0.87708E+00_JPRB, 0.92242E+00_JPRB, 0.92038E+00_JPRB, 0.92965E+00_JPRB ,&
&  0.94340E+00_JPRB, 0.95582E+00_JPRB, 0.94922E+00_JPRB, 0.94987E+00_JPRB, 0.96208E+00_JPRB, 0.99813E+00_JPRB, 0.10402E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.94902E+00_JPRB, 0.92472E+00_JPRB, 0.91694E+00_JPRB, 0.95394E+00_JPRB, 0.98460E+00_JPRB, 0.10451E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.10846E+01_JPRB, 0.10898E+01_JPRB, 0.10917E+01_JPRB, 0.10983E+01_JPRB, 0.11166E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.11614E+01_JPRB, 0.11661E+01_JPRB, 0.12754E+01_JPRB, 0.20206E+01_JPRB, 0.28339E+01_JPRB, 0.33172E+01_JPRB ,&
&  0.35514E+01_JPRB, 0.34633E+01_JPRB, 0.35057E+01_JPRB, 0.40348E+01_JPRB, 0.39346E+01_JPRB, 0.35327E+01_JPRB, 0.31745E+01_JPRB ,&
&  0.30279E+01_JPRB, 0.29021E+01_JPRB, 0.30417E+01_JPRB, 0.31621E+01_JPRB, 0.30949E+01_JPRB, 0.31079E+01_JPRB, 0.31074E+01_JPRB ,&
&  0.31224E+01_JPRB, 0.31431E+01_JPRB, 0.30240E+01_JPRB, 0.29023E+01_JPRB, 0.28059E+01_JPRB, 0.28967E+01_JPRB, 0.29609E+01_JPRB ,&
&  0.31413E+01_JPRB, 0.31107E+01_JPRB, 0.27888E+01_JPRB, 0.27767E+01_JPRB, 0.22680E+01_JPRB, 0.21526E+01_JPRB, 0.24298E+01_JPRB ,&
&  0.20797E+01_JPRB, 0.18060E+01_JPRB, 0.17386E+01_JPRB, 0.17917E+01_JPRB, 0.18443E+01_JPRB, 0.19515E+01_JPRB, 0.18876E+01_JPRB ,&
&  0.16693E+01_JPRB, 0.17465E+01_JPRB, 0.19774E+01_JPRB, 0.17835E+01_JPRB, 0.16413E+01_JPRB, 0.15375E+01_JPRB, 0.13930E+01_JPRB ,&
&  0.14125E+01_JPRB, 0.13939E+01_JPRB, 0.13791E+01_JPRB, 0.13657E+01_JPRB, 0.13720E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.14616E+01_JPRB, 0.13597E+01_JPRB, 0.13564E+01_JPRB, 0.13216E+01_JPRB, 0.12971E+01_JPRB, 0.12723E+01_JPRB, 0.12581E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.12396E+01_JPRB, 0.12213E+01_JPRB, 0.11994E+01_JPRB, 0.11820E+01_JPRB, 0.11568E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.10333E+01_JPRB, 0.10110E+01_JPRB, 0.97178E+00_JPRB, 0.10106E+01_JPRB, 0.10118E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10506E+01_JPRB, 0.10788E+01_JPRB, 0.10774E+01_JPRB, 0.10811E+01_JPRB, 0.11051E+01_JPRB, 0.11709E+01_JPRB, 0.12469E+01_JPRB ,&
&  0.12541E+01_JPRB, 0.11924E+01_JPRB, 0.11594E+01_JPRB, 0.11765E+01_JPRB, 0.12515E+01_JPRB, 0.11847E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.12073E+01_JPRB, 0.11943E+01_JPRB, 0.11844E+01_JPRB, 0.11901E+01_JPRB, 0.12265E+01_JPRB, 0.12792E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.13091E+01_JPRB, 0.12466E+01_JPRB, 0.12333E+01_JPRB, 0.16166E+01_JPRB, 0.25704E+01_JPRB, 0.36742E+01_JPRB, 0.41740E+01_JPRB ,&
&  0.43179E+01_JPRB, 0.42677E+01_JPRB, 0.42469E+01_JPRB, 0.47147E+01_JPRB, 0.45354E+01_JPRB, 0.40889E+01_JPRB, 0.38447E+01_JPRB ,&
&  0.36885E+01_JPRB, 0.36059E+01_JPRB, 0.36773E+01_JPRB, 0.34875E+01_JPRB, 0.32760E+01_JPRB, 0.31536E+01_JPRB, 0.30606E+01_JPRB ,&
&  0.29364E+01_JPRB, 0.29144E+01_JPRB, 0.28911E+01_JPRB, 0.27336E+01_JPRB, 0.26654E+01_JPRB, 0.26271E+01_JPRB, 0.28951E+01_JPRB ,&
&  0.32616E+01_JPRB, 0.25574E+01_JPRB, 0.21586E+01_JPRB, 0.23601E+01_JPRB, 0.18384E+01_JPRB, 0.17720E+01_JPRB, 0.19564E+01_JPRB ,&
&  0.18286E+01_JPRB, 0.18029E+01_JPRB, 0.19320E+01_JPRB, 0.19015E+01_JPRB, 0.20041E+01_JPRB, 0.22206E+01_JPRB, 0.21972E+01_JPRB ,&
&  0.20544E+01_JPRB, 0.19608E+01_JPRB, 0.23489E+01_JPRB, 0.21283E+01_JPRB, 0.19325E+01_JPRB, 0.18313E+01_JPRB, 0.17379E+01_JPRB ,&
&  0.16796E+01_JPRB, 0.16237E+01_JPRB, 0.15605E+01_JPRB, 0.15206E+01_JPRB, 0.15120E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.14727E+01_JPRB, 0.13666E+01_JPRB, 0.14024E+01_JPRB, 0.13848E+01_JPRB, 0.13703E+01_JPRB, 0.13576E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.13296E+01_JPRB, 0.13169E+01_JPRB, 0.12939E+01_JPRB, 0.12504E+01_JPRB, 0.12186E+01_JPRB, 0.11917E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.11524E+01_JPRB, 0.10988E+01_JPRB, 0.10791E+01_JPRB, 0.10497E+01_JPRB, 0.10824E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.11782E+01_JPRB, 0.12477E+01_JPRB, 0.13212E+01_JPRB, 0.13992E+01_JPRB, 0.14642E+01_JPRB, 0.15088E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.16046E+01_JPRB, 0.15208E+01_JPRB, 0.14415E+01_JPRB, 0.14663E+01_JPRB, 0.14794E+01_JPRB, 0.13880E+01_JPRB, 0.13869E+01_JPRB ,&
&  0.13549E+01_JPRB, 0.13337E+01_JPRB, 0.13264E+01_JPRB, 0.13580E+01_JPRB, 0.13625E+01_JPRB, 0.12993E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.12051E+01_JPRB, 0.12444E+01_JPRB, 0.15378E+01_JPRB, 0.22305E+01_JPRB, 0.33818E+01_JPRB, 0.47301E+01_JPRB, 0.51047E+01_JPRB ,&
&  0.51296E+01_JPRB, 0.46255E+01_JPRB, 0.36500E+01_JPRB, 0.40552E+01_JPRB, 0.49907E+01_JPRB, 0.47075E+01_JPRB, 0.42525E+01_JPRB ,&
&  0.42233E+01_JPRB, 0.39516E+01_JPRB, 0.38462E+01_JPRB, 0.35116E+01_JPRB, 0.32585E+01_JPRB, 0.30171E+01_JPRB, 0.28062E+01_JPRB ,&
&  0.26369E+01_JPRB, 0.25794E+01_JPRB, 0.25729E+01_JPRB, 0.26101E+01_JPRB, 0.26119E+01_JPRB, 0.28204E+01_JPRB, 0.32013E+01_JPRB ,&
&  0.32218E+01_JPRB, 0.29874E+01_JPRB, 0.23320E+01_JPRB, 0.22507E+01_JPRB, 0.17318E+01_JPRB, 0.16658E+01_JPRB, 0.18233E+01_JPRB ,&
&  0.17930E+01_JPRB, 0.20329E+01_JPRB, 0.23916E+01_JPRB, 0.24517E+01_JPRB, 0.28323E+01_JPRB, 0.29427E+01_JPRB, 0.28690E+01_JPRB ,&
&  0.27178E+01_JPRB, 0.24523E+01_JPRB, 0.28587E+01_JPRB, 0.25146E+01_JPRB, 0.22113E+01_JPRB, 0.20719E+01_JPRB, 0.19651E+01_JPRB ,&
&  0.18907E+01_JPRB, 0.17914E+01_JPRB, 0.16864E+01_JPRB, 0.16216E+01_JPRB, 0.15688E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.14747E+01_JPRB, 0.14064E+01_JPRB, 0.14436E+01_JPRB, 0.14081E+01_JPRB, 0.14169E+01_JPRB, 0.13862E+01_JPRB, 0.13831E+01_JPRB ,&
&  0.13701E+01_JPRB, 0.13366E+01_JPRB, 0.12908E+01_JPRB, 0.12484E+01_JPRB, 0.12181E+01_JPRB, 0.11919E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.11685E+01_JPRB, 0.11464E+01_JPRB, 0.10637E+01_JPRB, 0.10209E+01_JPRB, 0.10038E+01_JPRB, 0.10561E+01_JPRB, 0.11599E+01_JPRB ,&
&  0.13019E+01_JPRB, 0.14485E+01_JPRB, 0.16426E+01_JPRB, 0.17620E+01_JPRB, 0.18457E+01_JPRB, 0.19063E+01_JPRB, 0.19794E+01_JPRB ,&
&  0.20341E+01_JPRB, 0.17816E+01_JPRB, 0.17201E+01_JPRB, 0.16045E+01_JPRB, 0.15623E+01_JPRB, 0.15181E+01_JPRB, 0.14586E+01_JPRB ,&
&  0.14110E+01_JPRB, 0.13937E+01_JPRB, 0.13765E+01_JPRB, 0.13243E+01_JPRB, 0.12634E+01_JPRB, 0.12228E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.12772E+01_JPRB, 0.14968E+01_JPRB, 0.21824E+01_JPRB, 0.31900E+01_JPRB, 0.47426E+01_JPRB, 0.61453E+01_JPRB, 0.61337E+01_JPRB ,&
&  0.47599E+01_JPRB, 0.41498E+01_JPRB, 0.53828E+01_JPRB, 0.47547E+01_JPRB, 0.48559E+01_JPRB, 0.51915E+01_JPRB, 0.48236E+01_JPRB ,&
&  0.42244E+01_JPRB, 0.35716E+01_JPRB, 0.34928E+01_JPRB, 0.33207E+01_JPRB, 0.41869E+01_JPRB, 0.35809E+01_JPRB, 0.31420E+01_JPRB ,&
&  0.28756E+01_JPRB, 0.28323E+01_JPRB, 0.27778E+01_JPRB, 0.28626E+01_JPRB, 0.31719E+01_JPRB, 0.37585E+01_JPRB, 0.39144E+01_JPRB ,&
&  0.35297E+01_JPRB, 0.35229E+01_JPRB, 0.29389E+01_JPRB, 0.23988E+01_JPRB, 0.20394E+01_JPRB, 0.19999E+01_JPRB, 0.20279E+01_JPRB ,&
&  0.21510E+01_JPRB, 0.23478E+01_JPRB, 0.29487E+01_JPRB, 0.36020E+01_JPRB, 0.41501E+01_JPRB, 0.41021E+01_JPRB, 0.35118E+01_JPRB ,&
&  0.33481E+01_JPRB, 0.37743E+01_JPRB, 0.37150E+01_JPRB, 0.29216E+01_JPRB, 0.24575E+01_JPRB, 0.22862E+01_JPRB, 0.21525E+01_JPRB ,&
&  0.19826E+01_JPRB, 0.18168E+01_JPRB, 0.16963E+01_JPRB, 0.16273E+01_JPRB, 0.15842E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.15488E+01_JPRB, 0.14862E+01_JPRB, 0.14156E+01_JPRB, 0.13852E+01_JPRB, 0.13989E+01_JPRB, 0.13555E+01_JPRB, 0.13236E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12400E+01_JPRB, 0.12341E+01_JPRB, 0.12354E+01_JPRB, 0.12229E+01_JPRB, 0.11994E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.11005E+01_JPRB, 0.10260E+01_JPRB, 0.10323E+01_JPRB, 0.10319E+01_JPRB, 0.10446E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.14355E+01_JPRB, 0.17425E+01_JPRB, 0.19816E+01_JPRB, 0.20831E+01_JPRB, 0.20936E+01_JPRB, 0.21117E+01_JPRB, 0.21383E+01_JPRB ,&
&  0.19971E+01_JPRB, 0.19684E+01_JPRB, 0.19237E+01_JPRB, 0.17762E+01_JPRB, 0.16586E+01_JPRB, 0.15480E+01_JPRB, 0.14858E+01_JPRB ,&
&  0.14477E+01_JPRB, 0.14408E+01_JPRB, 0.13858E+01_JPRB, 0.13154E+01_JPRB, 0.13226E+01_JPRB, 0.13333E+01_JPRB, 0.13400E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.18828E+01_JPRB, 0.27609E+01_JPRB, 0.39177E+01_JPRB, 0.43363E+01_JPRB, 0.56637E+01_JPRB, 0.56328E+01_JPRB ,&
&  0.62779E+01_JPRB, 0.58540E+01_JPRB, 0.54813E+01_JPRB, 0.59351E+01_JPRB, 0.49327E+01_JPRB, 0.49749E+01_JPRB, 0.48291E+01_JPRB ,&
&  0.42689E+01_JPRB, 0.40394E+01_JPRB, 0.44377E+01_JPRB, 0.40910E+01_JPRB, 0.44316E+01_JPRB, 0.49363E+01_JPRB, 0.43897E+01_JPRB ,&
&  0.41428E+01_JPRB, 0.39263E+01_JPRB, 0.36763E+01_JPRB, 0.38689E+01_JPRB, 0.38966E+01_JPRB, 0.40228E+01_JPRB, 0.34460E+01_JPRB ,&
&  0.32657E+01_JPRB, 0.27533E+01_JPRB, 0.29359E+01_JPRB, 0.26280E+01_JPRB, 0.25388E+01_JPRB, 0.24344E+01_JPRB, 0.23950E+01_JPRB ,&
&  0.26652E+01_JPRB, 0.29007E+01_JPRB, 0.38386E+01_JPRB, 0.48158E+01_JPRB, 0.57230E+01_JPRB, 0.50020E+01_JPRB, 0.44258E+01_JPRB ,&
&  0.46961E+01_JPRB, 0.48399E+01_JPRB, 0.40010E+01_JPRB, 0.31434E+01_JPRB, 0.26614E+01_JPRB, 0.24675E+01_JPRB, 0.23256E+01_JPRB ,&
&  0.21576E+01_JPRB, 0.19910E+01_JPRB, 0.18609E+01_JPRB, 0.17640E+01_JPRB, 0.16824E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.14476E+01_JPRB, 0.14070E+01_JPRB, 0.13243E+01_JPRB, 0.13184E+01_JPRB, 0.13497E+01_JPRB, 0.13140E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.12059E+01_JPRB, 0.12172E+01_JPRB, 0.12297E+01_JPRB, 0.12183E+01_JPRB, 0.12072E+01_JPRB, 0.11848E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.11135E+01_JPRB, 0.10477E+01_JPRB, 0.10454E+01_JPRB, 0.10914E+01_JPRB, 0.11301E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.17445E+01_JPRB, 0.21387E+01_JPRB, 0.22584E+01_JPRB, 0.22326E+01_JPRB, 0.21499E+01_JPRB, 0.20961E+01_JPRB, 0.20186E+01_JPRB ,&
&  0.19552E+01_JPRB, 0.18417E+01_JPRB, 0.17555E+01_JPRB, 0.16835E+01_JPRB, 0.16122E+01_JPRB, 0.15408E+01_JPRB, 0.14797E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.13252E+01_JPRB, 0.12334E+01_JPRB, 0.11755E+01_JPRB, 0.12049E+01_JPRB, 0.11953E+01_JPRB, 0.12717E+01_JPRB ,&
&  0.15727E+01_JPRB, 0.21142E+01_JPRB, 0.28065E+01_JPRB, 0.36915E+01_JPRB, 0.44005E+01_JPRB, 0.57645E+01_JPRB, 0.64547E+01_JPRB ,&
&  0.61287E+01_JPRB, 0.54465E+01_JPRB, 0.59525E+01_JPRB, 0.65341E+01_JPRB, 0.62891E+01_JPRB, 0.54130E+01_JPRB, 0.50784E+01_JPRB ,&
&  0.38543E+01_JPRB, 0.31657E+01_JPRB, 0.30430E+01_JPRB, 0.35566E+01_JPRB, 0.54208E+01_JPRB, 0.70409E+01_JPRB, 0.66979E+01_JPRB ,&
&  0.65301E+01_JPRB, 0.61070E+01_JPRB, 0.56049E+01_JPRB, 0.48612E+01_JPRB, 0.44304E+01_JPRB, 0.40738E+01_JPRB, 0.50317E+01_JPRB ,&
&  0.48721E+01_JPRB, 0.47811E+01_JPRB, 0.40096E+01_JPRB, 0.32094E+01_JPRB, 0.26527E+01_JPRB, 0.28784E+01_JPRB, 0.29674E+01_JPRB ,&
&  0.32804E+01_JPRB, 0.37309E+01_JPRB, 0.63121E+01_JPRB, 0.73275E+01_JPRB, 0.71534E+01_JPRB, 0.64948E+01_JPRB, 0.63376E+01_JPRB ,&
&  0.57895E+01_JPRB, 0.55692E+01_JPRB, 0.44880E+01_JPRB, 0.33201E+01_JPRB, 0.26436E+01_JPRB, 0.22659E+01_JPRB, 0.20576E+01_JPRB ,&
&  0.19880E+01_JPRB, 0.19181E+01_JPRB, 0.18080E+01_JPRB, 0.16728E+01_JPRB, 0.15686E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.13513E+01_JPRB, 0.13125E+01_JPRB, 0.12257E+01_JPRB, 0.12146E+01_JPRB, 0.12490E+01_JPRB, 0.12742E+01_JPRB, 0.12546E+01_JPRB ,&
&  0.12286E+01_JPRB, 0.12391E+01_JPRB, 0.12556E+01_JPRB, 0.12428E+01_JPRB, 0.12195E+01_JPRB, 0.12143E+01_JPRB, 0.11975E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.12256E+01_JPRB, 0.11537E+01_JPRB, 0.11883E+01_JPRB, 0.12743E+01_JPRB, 0.13856E+01_JPRB, 0.15634E+01_JPRB ,&
&  0.22136E+01_JPRB, 0.23840E+01_JPRB, 0.22041E+01_JPRB, 0.21363E+01_JPRB, 0.21248E+01_JPRB, 0.18839E+01_JPRB, 0.18160E+01_JPRB ,&
&  0.18168E+01_JPRB, 0.17127E+01_JPRB, 0.16393E+01_JPRB, 0.15792E+01_JPRB, 0.15511E+01_JPRB, 0.15286E+01_JPRB, 0.14596E+01_JPRB ,&
&  0.13525E+01_JPRB, 0.12372E+01_JPRB, 0.11852E+01_JPRB, 0.11799E+01_JPRB, 0.11241E+01_JPRB, 0.11381E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.16311E+01_JPRB, 0.21690E+01_JPRB, 0.25829E+01_JPRB, 0.31715E+01_JPRB, 0.39902E+01_JPRB, 0.54977E+01_JPRB, 0.75828E+01_JPRB ,&
&  0.68898E+01_JPRB, 0.66801E+01_JPRB, 0.64721E+01_JPRB, 0.70748E+01_JPRB, 0.61766E+01_JPRB, 0.53554E+01_JPRB, 0.48947E+01_JPRB ,&
&  0.36623E+01_JPRB, 0.31621E+01_JPRB, 0.36716E+01_JPRB, 0.48529E+01_JPRB, 0.55362E+01_JPRB, 0.64880E+01_JPRB, 0.81263E+01_JPRB ,&
&  0.73085E+01_JPRB, 0.70269E+01_JPRB, 0.67119E+01_JPRB, 0.63146E+01_JPRB, 0.38020E+01_JPRB, 0.31260E+01_JPRB, 0.45295E+01_JPRB ,&
&  0.46993E+01_JPRB, 0.30554E+01_JPRB, 0.20141E+01_JPRB, 0.24959E+01_JPRB, 0.18359E+01_JPRB, 0.20674E+01_JPRB, 0.29490E+01_JPRB ,&
&  0.39064E+01_JPRB, 0.53002E+01_JPRB, 0.91436E+01_JPRB, 0.84605E+01_JPRB, 0.80031E+01_JPRB, 0.92242E+01_JPRB, 0.76538E+01_JPRB ,&
&  0.73681E+01_JPRB, 0.52409E+01_JPRB, 0.41268E+01_JPRB, 0.28885E+01_JPRB, 0.20727E+01_JPRB, 0.17130E+01_JPRB, 0.15958E+01_JPRB ,&
&  0.15585E+01_JPRB, 0.15695E+01_JPRB, 0.15655E+01_JPRB, 0.15026E+01_JPRB, 0.14215E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.13909E+01_JPRB, 0.12825E+01_JPRB, 0.11452E+01_JPRB, 0.11301E+01_JPRB, 0.11578E+01_JPRB, 0.12008E+01_JPRB, 0.11960E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.12582E+01_JPRB, 0.13254E+01_JPRB, 0.13173E+01_JPRB, 0.12682E+01_JPRB, 0.12564E+01_JPRB, 0.12282E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.13091E+01_JPRB, 0.14232E+01_JPRB, 0.15705E+01_JPRB, 0.17440E+01_JPRB, 0.18417E+01_JPRB, 0.22612E+01_JPRB ,&
&  0.25978E+01_JPRB, 0.23331E+01_JPRB, 0.21029E+01_JPRB, 0.20473E+01_JPRB, 0.19107E+01_JPRB, 0.16517E+01_JPRB, 0.16761E+01_JPRB ,&
&  0.16762E+01_JPRB, 0.16234E+01_JPRB, 0.15614E+01_JPRB, 0.15054E+01_JPRB, 0.14959E+01_JPRB, 0.14898E+01_JPRB, 0.14338E+01_JPRB ,&
&  0.13177E+01_JPRB, 0.12654E+01_JPRB, 0.12225E+01_JPRB, 0.11282E+01_JPRB, 0.10854E+01_JPRB, 0.11226E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.16329E+01_JPRB, 0.20648E+01_JPRB, 0.23448E+01_JPRB, 0.27796E+01_JPRB, 0.44281E+01_JPRB, 0.57383E+01_JPRB, 0.58570E+01_JPRB ,&
&  0.56277E+01_JPRB, 0.56549E+01_JPRB, 0.70872E+01_JPRB, 0.68917E+01_JPRB, 0.63538E+01_JPRB, 0.60915E+01_JPRB, 0.51931E+01_JPRB ,&
&  0.45678E+01_JPRB, 0.42276E+01_JPRB, 0.47137E+01_JPRB, 0.56605E+01_JPRB, 0.67910E+01_JPRB, 0.69094E+01_JPRB, 0.84811E+01_JPRB ,&
&  0.85880E+01_JPRB, 0.80672E+01_JPRB, 0.63024E+01_JPRB, 0.47642E+01_JPRB, 0.43054E+01_JPRB, 0.32616E+01_JPRB, 0.22720E+01_JPRB ,&
&  0.19971E+01_JPRB, 0.19118E+01_JPRB, 0.17131E+01_JPRB, 0.17313E+01_JPRB, 0.15592E+01_JPRB, 0.18275E+01_JPRB, 0.26772E+01_JPRB ,&
&  0.50683E+01_JPRB, 0.77225E+01_JPRB, 0.91597E+01_JPRB, 0.78813E+01_JPRB, 0.73455E+01_JPRB, 0.91650E+01_JPRB, 0.93830E+01_JPRB ,&
&  0.55877E+01_JPRB, 0.40738E+01_JPRB, 0.27669E+01_JPRB, 0.19484E+01_JPRB, 0.15937E+01_JPRB, 0.14144E+01_JPRB, 0.13662E+01_JPRB ,&
&  0.13629E+01_JPRB, 0.13545E+01_JPRB, 0.13737E+01_JPRB, 0.14019E+01_JPRB, 0.13841E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.13661E+01_JPRB, 0.12125E+01_JPRB, 0.10727E+01_JPRB, 0.10391E+01_JPRB, 0.10890E+01_JPRB, 0.11169E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.13139E+01_JPRB, 0.13856E+01_JPRB, 0.13699E+01_JPRB, 0.13296E+01_JPRB, 0.12982E+01_JPRB, 0.12849E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.15290E+01_JPRB, 0.18637E+01_JPRB, 0.22120E+01_JPRB, 0.25551E+01_JPRB, 0.24980E+01_JPRB, 0.28564E+01_JPRB ,&
&  0.25793E+01_JPRB, 0.21813E+01_JPRB, 0.21166E+01_JPRB, 0.20094E+01_JPRB, 0.18497E+01_JPRB, 0.16555E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.15790E+01_JPRB, 0.15858E+01_JPRB, 0.15194E+01_JPRB, 0.15213E+01_JPRB, 0.15134E+01_JPRB, 0.14752E+01_JPRB, 0.13877E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.13029E+01_JPRB, 0.11732E+01_JPRB, 0.11592E+01_JPRB, 0.11663E+01_JPRB, 0.12491E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.17302E+01_JPRB, 0.20819E+01_JPRB, 0.24233E+01_JPRB, 0.33314E+01_JPRB, 0.42318E+01_JPRB, 0.41248E+01_JPRB, 0.51509E+01_JPRB ,&
&  0.59898E+01_JPRB, 0.65655E+01_JPRB, 0.65200E+01_JPRB, 0.72689E+01_JPRB, 0.69079E+01_JPRB, 0.62479E+01_JPRB, 0.56768E+01_JPRB ,&
&  0.49724E+01_JPRB, 0.54703E+01_JPRB, 0.70888E+01_JPRB, 0.78442E+01_JPRB, 0.97122E+01_JPRB, 0.93374E+01_JPRB, 0.98829E+01_JPRB ,&
&  0.10192E+02_JPRB, 0.92732E+01_JPRB, 0.74239E+01_JPRB, 0.58949E+01_JPRB, 0.63700E+01_JPRB, 0.59351E+01_JPRB, 0.32541E+01_JPRB ,&
&  0.26961E+01_JPRB, 0.23475E+01_JPRB, 0.19225E+01_JPRB, 0.16651E+01_JPRB, 0.16012E+01_JPRB, 0.19512E+01_JPRB, 0.34278E+01_JPRB ,&
&  0.45459E+01_JPRB, 0.81234E+01_JPRB, 0.95061E+01_JPRB, 0.82162E+01_JPRB, 0.70703E+01_JPRB, 0.83959E+01_JPRB, 0.61461E+01_JPRB ,&
&  0.49277E+01_JPRB, 0.34025E+01_JPRB, 0.19772E+01_JPRB, 0.15290E+01_JPRB, 0.14338E+01_JPRB, 0.13675E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.12222E+01_JPRB, 0.12194E+01_JPRB, 0.12600E+01_JPRB, 0.12970E+01_JPRB, 0.13030E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.11972E+01_JPRB, 0.10782E+01_JPRB, 0.10032E+01_JPRB, 0.95649E+00_JPRB, 0.10231E+01_JPRB, 0.11195E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.12634E+01_JPRB, 0.13949E+01_JPRB, 0.14643E+01_JPRB, 0.14573E+01_JPRB, 0.13892E+01_JPRB, 0.13790E+01_JPRB, 0.14571E+01_JPRB ,&
&  0.16557E+01_JPRB, 0.20007E+01_JPRB, 0.24907E+01_JPRB, 0.30151E+01_JPRB, 0.32369E+01_JPRB, 0.33259E+01_JPRB, 0.31073E+01_JPRB ,&
&  0.24412E+01_JPRB, 0.21250E+01_JPRB, 0.21296E+01_JPRB, 0.20805E+01_JPRB, 0.18607E+01_JPRB, 0.17558E+01_JPRB, 0.16182E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.15934E+01_JPRB, 0.15771E+01_JPRB, 0.15910E+01_JPRB, 0.15520E+01_JPRB, 0.14905E+01_JPRB, 0.14346E+01_JPRB ,&
&  0.13772E+01_JPRB, 0.12530E+01_JPRB, 0.11913E+01_JPRB, 0.12045E+01_JPRB, 0.13411E+01_JPRB, 0.15619E+01_JPRB, 0.18080E+01_JPRB ,&
&  0.21118E+01_JPRB, 0.24748E+01_JPRB, 0.30026E+01_JPRB, 0.39555E+01_JPRB, 0.36189E+01_JPRB, 0.45733E+01_JPRB, 0.52451E+01_JPRB ,&
&  0.55540E+01_JPRB, 0.57976E+01_JPRB, 0.56475E+01_JPRB, 0.63294E+01_JPRB, 0.67221E+01_JPRB, 0.62472E+01_JPRB, 0.56220E+01_JPRB ,&
&  0.59520E+01_JPRB, 0.65537E+01_JPRB, 0.95236E+01_JPRB, 0.11012E+02_JPRB, 0.12186E+02_JPRB, 0.15677E+02_JPRB, 0.11608E+02_JPRB ,&
&  0.97935E+01_JPRB, 0.10124E+02_JPRB, 0.85329E+01_JPRB, 0.69257E+01_JPRB, 0.69568E+01_JPRB, 0.81183E+01_JPRB, 0.87610E+01_JPRB ,&
&  0.39665E+01_JPRB, 0.24444E+01_JPRB, 0.23798E+01_JPRB, 0.21425E+01_JPRB, 0.18072E+01_JPRB, 0.21050E+01_JPRB, 0.53988E+01_JPRB ,&
&  0.45678E+01_JPRB, 0.75555E+01_JPRB, 0.86161E+01_JPRB, 0.74229E+01_JPRB, 0.67839E+01_JPRB, 0.65128E+01_JPRB, 0.53672E+01_JPRB ,&
&  0.36875E+01_JPRB, 0.24872E+01_JPRB, 0.17047E+01_JPRB, 0.14154E+01_JPRB, 0.13577E+01_JPRB, 0.13228E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.11651E+01_JPRB, 0.11909E+01_JPRB, 0.11661E+01_JPRB, 0.12180E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.10924E+01_JPRB, 0.10377E+01_JPRB, 0.10517E+01_JPRB, 0.10652E+01_JPRB, 0.11160E+01_JPRB, 0.11683E+01_JPRB, 0.12498E+01_JPRB ,&
&  0.13899E+01_JPRB, 0.15401E+01_JPRB, 0.16504E+01_JPRB, 0.16475E+01_JPRB, 0.15983E+01_JPRB, 0.16966E+01_JPRB, 0.19295E+01_JPRB ,&
&  0.22513E+01_JPRB, 0.26532E+01_JPRB, 0.31112E+01_JPRB, 0.34192E+01_JPRB, 0.35804E+01_JPRB, 0.32757E+01_JPRB, 0.28851E+01_JPRB ,&
&  0.23541E+01_JPRB, 0.20787E+01_JPRB, 0.19769E+01_JPRB, 0.20037E+01_JPRB, 0.19645E+01_JPRB, 0.18032E+01_JPRB, 0.16213E+01_JPRB ,&
&  0.15851E+01_JPRB, 0.15890E+01_JPRB, 0.15772E+01_JPRB, 0.15990E+01_JPRB, 0.16170E+01_JPRB, 0.16157E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.14300E+01_JPRB, 0.13634E+01_JPRB, 0.13402E+01_JPRB, 0.14621E+01_JPRB, 0.17705E+01_JPRB, 0.21211E+01_JPRB, 0.24581E+01_JPRB ,&
&  0.28117E+01_JPRB, 0.31772E+01_JPRB, 0.36504E+01_JPRB, 0.39640E+01_JPRB, 0.44370E+01_JPRB, 0.48271E+01_JPRB, 0.51025E+01_JPRB ,&
&  0.51115E+01_JPRB, 0.54219E+01_JPRB, 0.57169E+01_JPRB, 0.59694E+01_JPRB, 0.62244E+01_JPRB, 0.60863E+01_JPRB, 0.59196E+01_JPRB ,&
&  0.69596E+01_JPRB, 0.89439E+01_JPRB, 0.11392E+02_JPRB, 0.12454E+02_JPRB, 0.13057E+02_JPRB, 0.15848E+02_JPRB, 0.15630E+02_JPRB ,&
&  0.12215E+02_JPRB, 0.89815E+01_JPRB, 0.77841E+01_JPRB, 0.78770E+01_JPRB, 0.82604E+01_JPRB, 0.95803E+01_JPRB, 0.14041E+02_JPRB ,&
&  0.13905E+02_JPRB, 0.62490E+01_JPRB, 0.39749E+01_JPRB, 0.43798E+01_JPRB, 0.24321E+01_JPRB, 0.20379E+01_JPRB, 0.33473E+01_JPRB ,&
&  0.45746E+01_JPRB, 0.67644E+01_JPRB, 0.79286E+01_JPRB, 0.63313E+01_JPRB, 0.58206E+01_JPRB, 0.45824E+01_JPRB, 0.35057E+01_JPRB ,&
&  0.25933E+01_JPRB, 0.18618E+01_JPRB, 0.14337E+01_JPRB, 0.13112E+01_JPRB, 0.12370E+01_JPRB, 0.12455E+01_JPRB, 0.12717E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.11167E+01_JPRB, 0.11226E+01_JPRB, 0.11260E+01_JPRB, 0.11008E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.11240E+01_JPRB, 0.11283E+01_JPRB, 0.11850E+01_JPRB, 0.12498E+01_JPRB, 0.13114E+01_JPRB, 0.13810E+01_JPRB, 0.14580E+01_JPRB ,&
&  0.16116E+01_JPRB, 0.17982E+01_JPRB, 0.19172E+01_JPRB, 0.19943E+01_JPRB, 0.20925E+01_JPRB, 0.22903E+01_JPRB, 0.25513E+01_JPRB ,&
&  0.28556E+01_JPRB, 0.31706E+01_JPRB, 0.34762E+01_JPRB, 0.38054E+01_JPRB, 0.42545E+01_JPRB, 0.45000E+01_JPRB, 0.26912E+01_JPRB ,&
&  0.22459E+01_JPRB, 0.23000E+01_JPRB, 0.19858E+01_JPRB, 0.21110E+01_JPRB, 0.21714E+01_JPRB, 0.21446E+01_JPRB, 0.17738E+01_JPRB ,&
&  0.15882E+01_JPRB, 0.15710E+01_JPRB, 0.15753E+01_JPRB, 0.16335E+01_JPRB, 0.17144E+01_JPRB, 0.18478E+01_JPRB, 0.18663E+01_JPRB ,&
&  0.18577E+01_JPRB, 0.19073E+01_JPRB, 0.19969E+01_JPRB, 0.22985E+01_JPRB, 0.27259E+01_JPRB, 0.30723E+01_JPRB, 0.33177E+01_JPRB ,&
&  0.35232E+01_JPRB, 0.37682E+01_JPRB, 0.41204E+01_JPRB, 0.43648E+01_JPRB, 0.48314E+01_JPRB, 0.50515E+01_JPRB, 0.51960E+01_JPRB ,&
&  0.50089E+01_JPRB, 0.47716E+01_JPRB, 0.52470E+01_JPRB, 0.56378E+01_JPRB, 0.58101E+01_JPRB, 0.58983E+01_JPRB, 0.64783E+01_JPRB ,&
&  0.87737E+01_JPRB, 0.11500E+02_JPRB, 0.14456E+02_JPRB, 0.12988E+02_JPRB, 0.12447E+02_JPRB, 0.13828E+02_JPRB, 0.15020E+02_JPRB ,&
&  0.12515E+02_JPRB, 0.98834E+01_JPRB, 0.92906E+01_JPRB, 0.90064E+01_JPRB, 0.94116E+01_JPRB, 0.99515E+01_JPRB, 0.12385E+02_JPRB ,&
&  0.14078E+02_JPRB, 0.10777E+02_JPRB, 0.61647E+01_JPRB, 0.33109E+01_JPRB, 0.22990E+01_JPRB, 0.23414E+01_JPRB, 0.29919E+01_JPRB ,&
&  0.47966E+01_JPRB, 0.59985E+01_JPRB, 0.70982E+01_JPRB, 0.72572E+01_JPRB, 0.40683E+01_JPRB, 0.27572E+01_JPRB, 0.21534E+01_JPRB ,&
&  0.15284E+01_JPRB, 0.12382E+01_JPRB, 0.12136E+01_JPRB, 0.11807E+01_JPRB, 0.11271E+01_JPRB, 0.11801E+01_JPRB, 0.12845E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.10870E+01_JPRB, 0.10932E+01_JPRB, 0.10907E+01_JPRB, 0.10944E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.11812E+01_JPRB, 0.12217E+01_JPRB, 0.12981E+01_JPRB, 0.14188E+01_JPRB, 0.15652E+01_JPRB, 0.17131E+01_JPRB, 0.17627E+01_JPRB ,&
&  0.18380E+01_JPRB, 0.20002E+01_JPRB, 0.21593E+01_JPRB, 0.23176E+01_JPRB, 0.24819E+01_JPRB, 0.26757E+01_JPRB, 0.28843E+01_JPRB ,&
&  0.31102E+01_JPRB, 0.33291E+01_JPRB, 0.35537E+01_JPRB, 0.38553E+01_JPRB, 0.42463E+01_JPRB, 0.44699E+01_JPRB, 0.33226E+01_JPRB ,&
&  0.22633E+01_JPRB, 0.31574E+01_JPRB, 0.21840E+01_JPRB, 0.23824E+01_JPRB, 0.25374E+01_JPRB, 0.27734E+01_JPRB, 0.22083E+01_JPRB ,&
&  0.16982E+01_JPRB, 0.16557E+01_JPRB, 0.16832E+01_JPRB, 0.16973E+01_JPRB, 0.17201E+01_JPRB, 0.19839E+01_JPRB, 0.22501E+01_JPRB ,&
&  0.23729E+01_JPRB, 0.25431E+01_JPRB, 0.27975E+01_JPRB, 0.31624E+01_JPRB, 0.34519E+01_JPRB, 0.36905E+01_JPRB, 0.38803E+01_JPRB ,&
&  0.40628E+01_JPRB, 0.43230E+01_JPRB, 0.45404E+01_JPRB, 0.48444E+01_JPRB, 0.49856E+01_JPRB, 0.52307E+01_JPRB, 0.53644E+01_JPRB ,&
&  0.51883E+01_JPRB, 0.49650E+01_JPRB, 0.51995E+01_JPRB, 0.52642E+01_JPRB, 0.56807E+01_JPRB, 0.69684E+01_JPRB, 0.80275E+01_JPRB ,&
&  0.10550E+02_JPRB, 0.11972E+02_JPRB, 0.14534E+02_JPRB, 0.11722E+02_JPRB, 0.10663E+02_JPRB, 0.11226E+02_JPRB, 0.10557E+02_JPRB ,&
&  0.91161E+01_JPRB, 0.91575E+01_JPRB, 0.91001E+01_JPRB, 0.93434E+01_JPRB, 0.97090E+01_JPRB, 0.10094E+02_JPRB, 0.10887E+02_JPRB ,&
&  0.91636E+01_JPRB, 0.48128E+01_JPRB, 0.30923E+01_JPRB, 0.23548E+01_JPRB, 0.19775E+01_JPRB, 0.21948E+01_JPRB, 0.34248E+01_JPRB ,&
&  0.53827E+01_JPRB, 0.60443E+01_JPRB, 0.67595E+01_JPRB, 0.48852E+01_JPRB, 0.23491E+01_JPRB, 0.16229E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.96818E+00_JPRB, 0.92044E+00_JPRB, 0.10069E+01_JPRB, 0.10377E+01_JPRB, 0.10103E+01_JPRB, 0.10562E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10730E+01_JPRB, 0.11325E+01_JPRB, 0.11236E+01_JPRB, 0.11430E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.12186E+01_JPRB, 0.12798E+01_JPRB, 0.14125E+01_JPRB, 0.15955E+01_JPRB, 0.18583E+01_JPRB, 0.21493E+01_JPRB, 0.23006E+01_JPRB ,&
&  0.21662E+01_JPRB, 0.20294E+01_JPRB, 0.22667E+01_JPRB, 0.24876E+01_JPRB, 0.26570E+01_JPRB, 0.28606E+01_JPRB, 0.30694E+01_JPRB ,&
&  0.33202E+01_JPRB, 0.35836E+01_JPRB, 0.38790E+01_JPRB, 0.42466E+01_JPRB, 0.46732E+01_JPRB, 0.50816E+01_JPRB, 0.49775E+01_JPRB ,&
&  0.35799E+01_JPRB, 0.26479E+01_JPRB, 0.26725E+01_JPRB, 0.24592E+01_JPRB, 0.24005E+01_JPRB, 0.26456E+01_JPRB, 0.26504E+01_JPRB ,&
&  0.21868E+01_JPRB, 0.18825E+01_JPRB, 0.19162E+01_JPRB, 0.19215E+01_JPRB, 0.15457E+01_JPRB, 0.18183E+01_JPRB, 0.22509E+01_JPRB ,&
&  0.25154E+01_JPRB, 0.27541E+01_JPRB, 0.30690E+01_JPRB, 0.34180E+01_JPRB, 0.36774E+01_JPRB, 0.39907E+01_JPRB, 0.42749E+01_JPRB ,&
&  0.44980E+01_JPRB, 0.47558E+01_JPRB, 0.48933E+01_JPRB, 0.49334E+01_JPRB, 0.51365E+01_JPRB, 0.53777E+01_JPRB, 0.54547E+01_JPRB ,&
&  0.54677E+01_JPRB, 0.57094E+01_JPRB, 0.60188E+01_JPRB, 0.64083E+01_JPRB, 0.67531E+01_JPRB, 0.82093E+01_JPRB, 0.93666E+01_JPRB ,&
&  0.10623E+02_JPRB, 0.10987E+02_JPRB, 0.12129E+02_JPRB, 0.95972E+01_JPRB, 0.88207E+01_JPRB, 0.84842E+01_JPRB, 0.81414E+01_JPRB ,&
&  0.77541E+01_JPRB, 0.81583E+01_JPRB, 0.82221E+01_JPRB, 0.87992E+01_JPRB, 0.94747E+01_JPRB, 0.92361E+01_JPRB, 0.93944E+01_JPRB ,&
&  0.56591E+01_JPRB, 0.33178E+01_JPRB, 0.20712E+01_JPRB, 0.20477E+01_JPRB, 0.20997E+01_JPRB, 0.24688E+01_JPRB, 0.32161E+01_JPRB ,&
&  0.43646E+01_JPRB, 0.37591E+01_JPRB, 0.35516E+01_JPRB, 0.31181E+01_JPRB, 0.16237E+01_JPRB, 0.10992E+01_JPRB, 0.89828E+00_JPRB ,&
&  0.81537E+00_JPRB, 0.83496E+00_JPRB, 0.89528E+00_JPRB, 0.90659E+00_JPRB, 0.88245E+00_JPRB, 0.92437E+00_JPRB, 0.99723E+00_JPRB ,&
&  0.10269E+01_JPRB, 0.10567E+01_JPRB, 0.11235E+01_JPRB, 0.11335E+01_JPRB, 0.11877E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.11497E+01_JPRB, 0.12561E+01_JPRB, 0.13476E+01_JPRB, 0.14532E+01_JPRB, 0.15595E+01_JPRB, 0.17134E+01_JPRB, 0.17851E+01_JPRB ,&
&  0.17748E+01_JPRB, 0.17813E+01_JPRB, 0.19454E+01_JPRB, 0.21413E+01_JPRB, 0.23123E+01_JPRB, 0.24175E+01_JPRB, 0.25372E+01_JPRB ,&
&  0.26659E+01_JPRB, 0.27898E+01_JPRB, 0.29256E+01_JPRB, 0.30561E+01_JPRB, 0.32616E+01_JPRB, 0.35427E+01_JPRB, 0.37942E+01_JPRB ,&
&  0.33628E+01_JPRB, 0.21756E+01_JPRB, 0.24449E+01_JPRB, 0.23977E+01_JPRB, 0.23652E+01_JPRB, 0.24870E+01_JPRB, 0.26437E+01_JPRB ,&
&  0.21821E+01_JPRB, 0.19832E+01_JPRB, 0.20210E+01_JPRB, 0.20805E+01_JPRB, 0.14649E+01_JPRB, 0.16204E+01_JPRB, 0.19916E+01_JPRB ,&
&  0.23336E+01_JPRB, 0.26724E+01_JPRB, 0.30114E+01_JPRB, 0.33068E+01_JPRB, 0.36103E+01_JPRB, 0.39886E+01_JPRB, 0.43365E+01_JPRB ,&
&  0.46203E+01_JPRB, 0.48394E+01_JPRB, 0.50183E+01_JPRB, 0.51987E+01_JPRB, 0.53858E+01_JPRB, 0.56652E+01_JPRB, 0.57629E+01_JPRB ,&
&  0.59656E+01_JPRB, 0.61187E+01_JPRB, 0.68248E+01_JPRB, 0.76899E+01_JPRB, 0.83833E+01_JPRB, 0.83722E+01_JPRB, 0.90293E+01_JPRB ,&
&  0.92287E+01_JPRB, 0.89849E+01_JPRB, 0.91694E+01_JPRB, 0.91956E+01_JPRB, 0.64107E+01_JPRB, 0.61281E+01_JPRB, 0.62265E+01_JPRB ,&
&  0.69998E+01_JPRB, 0.69708E+01_JPRB, 0.71556E+01_JPRB, 0.74994E+01_JPRB, 0.76061E+01_JPRB, 0.74794E+01_JPRB, 0.67723E+01_JPRB ,&
&  0.45044E+01_JPRB, 0.25782E+01_JPRB, 0.18809E+01_JPRB, 0.20234E+01_JPRB, 0.24422E+01_JPRB, 0.34134E+01_JPRB, 0.28317E+01_JPRB ,&
&  0.31802E+01_JPRB, 0.23297E+01_JPRB, 0.22673E+01_JPRB, 0.26896E+01_JPRB, 0.16022E+01_JPRB, 0.10702E+01_JPRB, 0.85755E+00_JPRB ,&
&  0.78799E+00_JPRB, 0.80732E+00_JPRB, 0.81058E+00_JPRB, 0.80033E+00_JPRB, 0.79380E+00_JPRB, 0.83953E+00_JPRB, 0.89981E+00_JPRB ,&
&  0.93159E+00_JPRB, 0.96899E+00_JPRB, 0.10165E+01_JPRB, 0.10669E+01_JPRB, 0.11317E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.92609E+00_JPRB, 0.95227E+00_JPRB, 0.10068E+01_JPRB, 0.10979E+01_JPRB, 0.11821E+01_JPRB, 0.12652E+01_JPRB, 0.13242E+01_JPRB ,&
&  0.14266E+01_JPRB, 0.15173E+01_JPRB, 0.16201E+01_JPRB, 0.16983E+01_JPRB, 0.17599E+01_JPRB, 0.18947E+01_JPRB, 0.19844E+01_JPRB ,&
&  0.20065E+01_JPRB, 0.19790E+01_JPRB, 0.20036E+01_JPRB, 0.20734E+01_JPRB, 0.22581E+01_JPRB, 0.25785E+01_JPRB, 0.29457E+01_JPRB ,&
&  0.33178E+01_JPRB, 0.31536E+01_JPRB, 0.28242E+01_JPRB, 0.18250E+01_JPRB, 0.14936E+01_JPRB, 0.17658E+01_JPRB, 0.18135E+01_JPRB ,&
&  0.19129E+01_JPRB, 0.19584E+01_JPRB, 0.17376E+01_JPRB, 0.16098E+01_JPRB, 0.15001E+01_JPRB, 0.15138E+01_JPRB, 0.17095E+01_JPRB ,&
&  0.19668E+01_JPRB, 0.22706E+01_JPRB, 0.26190E+01_JPRB, 0.29004E+01_JPRB, 0.32019E+01_JPRB, 0.35885E+01_JPRB, 0.39976E+01_JPRB ,&
&  0.43407E+01_JPRB, 0.46455E+01_JPRB, 0.49161E+01_JPRB, 0.53341E+01_JPRB, 0.56246E+01_JPRB, 0.60164E+01_JPRB, 0.63032E+01_JPRB ,&
&  0.66330E+01_JPRB, 0.68029E+01_JPRB, 0.74055E+01_JPRB, 0.80757E+01_JPRB, 0.82220E+01_JPRB, 0.75229E+01_JPRB, 0.73747E+01_JPRB ,&
&  0.74068E+01_JPRB, 0.70026E+01_JPRB, 0.69113E+01_JPRB, 0.73122E+01_JPRB, 0.49761E+01_JPRB, 0.51489E+01_JPRB, 0.57821E+01_JPRB ,&
&  0.57203E+01_JPRB, 0.58239E+01_JPRB, 0.55724E+01_JPRB, 0.48187E+01_JPRB, 0.45561E+01_JPRB, 0.46889E+01_JPRB, 0.45177E+01_JPRB ,&
&  0.35502E+01_JPRB, 0.23298E+01_JPRB, 0.17489E+01_JPRB, 0.15616E+01_JPRB, 0.20814E+01_JPRB, 0.34924E+01_JPRB, 0.25717E+01_JPRB ,&
&  0.20331E+01_JPRB, 0.19666E+01_JPRB, 0.18564E+01_JPRB, 0.28541E+01_JPRB, 0.21118E+01_JPRB, 0.12923E+01_JPRB, 0.93544E+00_JPRB ,&
&  0.78421E+00_JPRB, 0.76147E+00_JPRB, 0.76663E+00_JPRB, 0.75209E+00_JPRB, 0.76620E+00_JPRB, 0.80692E+00_JPRB, 0.81626E+00_JPRB ,&
&  0.87353E+00_JPRB, 0.88931E+00_JPRB, 0.92070E+00_JPRB, 0.94574E+00_JPRB, 0.92022E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.66095E+00_JPRB, 0.65635E+00_JPRB, 0.68075E+00_JPRB, 0.76194E+00_JPRB, 0.82055E+00_JPRB, 0.86795E+00_JPRB, 0.94246E+00_JPRB ,&
&  0.10367E+01_JPRB, 0.11747E+01_JPRB, 0.12849E+01_JPRB, 0.13226E+01_JPRB, 0.12802E+01_JPRB, 0.12388E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.11186E+01_JPRB, 0.10313E+01_JPRB, 0.10133E+01_JPRB, 0.10689E+01_JPRB, 0.12051E+01_JPRB, 0.14001E+01_JPRB ,&
&  0.17104E+01_JPRB, 0.19725E+01_JPRB, 0.22012E+01_JPRB, 0.28141E+01_JPRB, 0.28125E+01_JPRB, 0.16876E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.26436E+01_JPRB, 0.27085E+01_JPRB, 0.18718E+01_JPRB, 0.16318E+01_JPRB, 0.15023E+01_JPRB, 0.14245E+01_JPRB, 0.14824E+01_JPRB ,&
&  0.16543E+01_JPRB, 0.18427E+01_JPRB, 0.20972E+01_JPRB, 0.24208E+01_JPRB, 0.27627E+01_JPRB, 0.30711E+01_JPRB, 0.34434E+01_JPRB ,&
&  0.37686E+01_JPRB, 0.39767E+01_JPRB, 0.42496E+01_JPRB, 0.46528E+01_JPRB, 0.51851E+01_JPRB, 0.58304E+01_JPRB, 0.64190E+01_JPRB ,&
&  0.68997E+01_JPRB, 0.71591E+01_JPRB, 0.77226E+01_JPRB, 0.80711E+01_JPRB, 0.74421E+01_JPRB, 0.67618E+01_JPRB, 0.61806E+01_JPRB ,&
&  0.56574E+01_JPRB, 0.48928E+01_JPRB, 0.43486E+01_JPRB, 0.53792E+01_JPRB, 0.55531E+01_JPRB, 0.48629E+01_JPRB, 0.47163E+01_JPRB ,&
&  0.47287E+01_JPRB, 0.46597E+01_JPRB, 0.41384E+01_JPRB, 0.32124E+01_JPRB, 0.25475E+01_JPRB, 0.28763E+01_JPRB, 0.30693E+01_JPRB ,&
&  0.26215E+01_JPRB, 0.18591E+01_JPRB, 0.14819E+01_JPRB, 0.13740E+01_JPRB, 0.14676E+01_JPRB, 0.24192E+01_JPRB, 0.18492E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.15554E+01_JPRB, 0.15226E+01_JPRB, 0.22690E+01_JPRB, 0.29995E+01_JPRB, 0.16073E+01_JPRB, 0.99223E+00_JPRB ,&
&  0.75473E+00_JPRB, 0.73622E+00_JPRB, 0.74137E+00_JPRB, 0.74145E+00_JPRB, 0.74243E+00_JPRB, 0.75994E+00_JPRB, 0.76188E+00_JPRB ,&
&  0.77494E+00_JPRB, 0.76168E+00_JPRB, 0.76331E+00_JPRB, 0.72585E+00_JPRB, 0.66948E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.69674E+00_JPRB, 0.69469E+00_JPRB, 0.68217E+00_JPRB, 0.67764E+00_JPRB, 0.68758E+00_JPRB, 0.70443E+00_JPRB, 0.72954E+00_JPRB ,&
&  0.76897E+00_JPRB, 0.81564E+00_JPRB, 0.85924E+00_JPRB, 0.90947E+00_JPRB, 0.91658E+00_JPRB, 0.85647E+00_JPRB, 0.84856E+00_JPRB ,&
&  0.84084E+00_JPRB, 0.85057E+00_JPRB, 0.84240E+00_JPRB, 0.93299E+00_JPRB, 0.95332E+00_JPRB, 0.96070E+00_JPRB, 0.10773E+01_JPRB ,&
&  0.10808E+01_JPRB, 0.12154E+01_JPRB, 0.13092E+01_JPRB, 0.14442E+01_JPRB, 0.23033E+01_JPRB, 0.21825E+01_JPRB, 0.17080E+01_JPRB ,&
&  0.24043E+01_JPRB, 0.24539E+01_JPRB, 0.18713E+01_JPRB, 0.15041E+01_JPRB, 0.13912E+01_JPRB, 0.12836E+01_JPRB, 0.12492E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.14737E+01_JPRB, 0.17296E+01_JPRB, 0.19935E+01_JPRB, 0.22136E+01_JPRB, 0.24188E+01_JPRB, 0.26456E+01_JPRB ,&
&  0.28186E+01_JPRB, 0.30035E+01_JPRB, 0.31110E+01_JPRB, 0.33180E+01_JPRB, 0.39548E+01_JPRB, 0.45778E+01_JPRB, 0.50761E+01_JPRB ,&
&  0.54775E+01_JPRB, 0.58624E+01_JPRB, 0.64704E+01_JPRB, 0.62709E+01_JPRB, 0.57141E+01_JPRB, 0.53691E+01_JPRB, 0.45204E+01_JPRB ,&
&  0.36731E+01_JPRB, 0.28586E+01_JPRB, 0.25665E+01_JPRB, 0.32362E+01_JPRB, 0.33701E+01_JPRB, 0.29502E+01_JPRB, 0.34177E+01_JPRB ,&
&  0.35465E+01_JPRB, 0.36103E+01_JPRB, 0.29827E+01_JPRB, 0.25653E+01_JPRB, 0.20890E+01_JPRB, 0.20611E+01_JPRB, 0.21998E+01_JPRB ,&
&  0.18118E+01_JPRB, 0.13637E+01_JPRB, 0.11777E+01_JPRB, 0.11178E+01_JPRB, 0.11496E+01_JPRB, 0.15917E+01_JPRB, 0.17966E+01_JPRB ,&
&  0.16938E+01_JPRB, 0.14767E+01_JPRB, 0.14938E+01_JPRB, 0.22541E+01_JPRB, 0.18675E+01_JPRB, 0.11245E+01_JPRB, 0.75297E+00_JPRB ,&
&  0.67030E+00_JPRB, 0.67556E+00_JPRB, 0.67251E+00_JPRB, 0.66983E+00_JPRB, 0.68109E+00_JPRB, 0.69900E+00_JPRB, 0.68576E+00_JPRB ,&
&  0.68382E+00_JPRB, 0.67072E+00_JPRB, 0.66110E+00_JPRB, 0.65696E+00_JPRB, 0.66662E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.85099E+00_JPRB, 0.85957E+00_JPRB, 0.86939E+00_JPRB, 0.88204E+00_JPRB, 0.90721E+00_JPRB, 0.93572E+00_JPRB, 0.96018E+00_JPRB ,&
&  0.10065E+01_JPRB, 0.10419E+01_JPRB, 0.10762E+01_JPRB, 0.10962E+01_JPRB, 0.11149E+01_JPRB, 0.11291E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.11610E+01_JPRB, 0.11917E+01_JPRB, 0.11902E+01_JPRB, 0.11979E+01_JPRB, 0.12243E+01_JPRB, 0.12411E+01_JPRB, 0.12208E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11713E+01_JPRB, 0.11975E+01_JPRB, 0.12421E+01_JPRB, 0.14603E+01_JPRB, 0.16221E+01_JPRB, 0.16237E+01_JPRB ,&
&  0.18544E+01_JPRB, 0.20324E+01_JPRB, 0.20112E+01_JPRB, 0.15319E+01_JPRB, 0.14022E+01_JPRB, 0.13435E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.12996E+01_JPRB, 0.13591E+01_JPRB, 0.14425E+01_JPRB, 0.15327E+01_JPRB, 0.16408E+01_JPRB, 0.16835E+01_JPRB, 0.17593E+01_JPRB ,&
&  0.17896E+01_JPRB, 0.18593E+01_JPRB, 0.19707E+01_JPRB, 0.22337E+01_JPRB, 0.27099E+01_JPRB, 0.32912E+01_JPRB, 0.33881E+01_JPRB ,&
&  0.34535E+01_JPRB, 0.38016E+01_JPRB, 0.35763E+01_JPRB, 0.34577E+01_JPRB, 0.33367E+01_JPRB, 0.28644E+01_JPRB, 0.23840E+01_JPRB ,&
&  0.22878E+01_JPRB, 0.20707E+01_JPRB, 0.15967E+01_JPRB, 0.19721E+01_JPRB, 0.22267E+01_JPRB, 0.22817E+01_JPRB, 0.25958E+01_JPRB ,&
&  0.27814E+01_JPRB, 0.24241E+01_JPRB, 0.19251E+01_JPRB, 0.18739E+01_JPRB, 0.18227E+01_JPRB, 0.15058E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.12839E+01_JPRB, 0.11444E+01_JPRB, 0.11000E+01_JPRB, 0.11239E+01_JPRB, 0.13925E+01_JPRB, 0.23156E+01_JPRB, 0.24775E+01_JPRB ,&
&  0.19059E+01_JPRB, 0.16076E+01_JPRB, 0.19008E+01_JPRB, 0.29982E+01_JPRB, 0.16177E+01_JPRB, 0.10563E+01_JPRB, 0.71442E+00_JPRB ,&
&  0.69709E+00_JPRB, 0.70227E+00_JPRB, 0.70770E+00_JPRB, 0.71315E+00_JPRB, 0.72137E+00_JPRB, 0.74107E+00_JPRB, 0.73396E+00_JPRB ,&
&  0.72700E+00_JPRB, 0.72301E+00_JPRB, 0.73897E+00_JPRB, 0.76408E+00_JPRB, 0.81268E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.97738E+00_JPRB, 0.97988E+00_JPRB, 0.99875E+00_JPRB, 0.10229E+01_JPRB, 0.10464E+01_JPRB, 0.10742E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11653E+01_JPRB, 0.12044E+01_JPRB, 0.12418E+01_JPRB, 0.12699E+01_JPRB, 0.12796E+01_JPRB, 0.12954E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.13312E+01_JPRB, 0.13238E+01_JPRB, 0.13462E+01_JPRB, 0.13838E+01_JPRB, 0.14305E+01_JPRB, 0.14507E+01_JPRB, 0.14723E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.15465E+01_JPRB, 0.16041E+01_JPRB, 0.16741E+01_JPRB, 0.17860E+01_JPRB, 0.19751E+01_JPRB, 0.21490E+01_JPRB ,&
&  0.18278E+01_JPRB, 0.21943E+01_JPRB, 0.19534E+01_JPRB, 0.16021E+01_JPRB, 0.15166E+01_JPRB, 0.15676E+01_JPRB, 0.15428E+01_JPRB ,&
&  0.15199E+01_JPRB, 0.15615E+01_JPRB, 0.16070E+01_JPRB, 0.16498E+01_JPRB, 0.16904E+01_JPRB, 0.17453E+01_JPRB, 0.17522E+01_JPRB ,&
&  0.18015E+01_JPRB, 0.18378E+01_JPRB, 0.18836E+01_JPRB, 0.19940E+01_JPRB, 0.20287E+01_JPRB, 0.25712E+01_JPRB, 0.24939E+01_JPRB ,&
&  0.23285E+01_JPRB, 0.23823E+01_JPRB, 0.18712E+01_JPRB, 0.19277E+01_JPRB, 0.19394E+01_JPRB, 0.16703E+01_JPRB, 0.14976E+01_JPRB ,&
&  0.19268E+01_JPRB, 0.20732E+01_JPRB, 0.15730E+01_JPRB, 0.17961E+01_JPRB, 0.19380E+01_JPRB, 0.20097E+01_JPRB, 0.20560E+01_JPRB ,&
&  0.20166E+01_JPRB, 0.17251E+01_JPRB, 0.14744E+01_JPRB, 0.13535E+01_JPRB, 0.13009E+01_JPRB, 0.12177E+01_JPRB, 0.11425E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10423E+01_JPRB, 0.10333E+01_JPRB, 0.10655E+01_JPRB, 0.16175E+01_JPRB, 0.22005E+01_JPRB, 0.32611E+01_JPRB ,&
&  0.17671E+01_JPRB, 0.15395E+01_JPRB, 0.12241E+01_JPRB, 0.18221E+01_JPRB, 0.12646E+01_JPRB, 0.79207E+00_JPRB, 0.77473E+00_JPRB ,&
&  0.83049E+00_JPRB, 0.87034E+00_JPRB, 0.86783E+00_JPRB, 0.84984E+00_JPRB, 0.84743E+00_JPRB, 0.85180E+00_JPRB, 0.82826E+00_JPRB ,&
&  0.81671E+00_JPRB, 0.83593E+00_JPRB, 0.87401E+00_JPRB, 0.89300E+00_JPRB, 0.93641E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.10477E+01_JPRB, 0.10772E+01_JPRB, 0.11092E+01_JPRB, 0.11440E+01_JPRB, 0.11777E+01_JPRB, 0.12098E+01_JPRB, 0.12450E+01_JPRB ,&
&  0.12883E+01_JPRB, 0.13197E+01_JPRB, 0.13508E+01_JPRB, 0.13692E+01_JPRB, 0.13787E+01_JPRB, 0.13986E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.14108E+01_JPRB, 0.14157E+01_JPRB, 0.14409E+01_JPRB, 0.14584E+01_JPRB, 0.14713E+01_JPRB, 0.14767E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.15131E+01_JPRB, 0.15667E+01_JPRB, 0.16085E+01_JPRB, 0.16617E+01_JPRB, 0.17080E+01_JPRB, 0.17357E+01_JPRB, 0.18794E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.24832E+01_JPRB, 0.23482E+01_JPRB, 0.18817E+01_JPRB, 0.18169E+01_JPRB, 0.17214E+01_JPRB, 0.17224E+01_JPRB ,&
&  0.17495E+01_JPRB, 0.17841E+01_JPRB, 0.18448E+01_JPRB, 0.19344E+01_JPRB, 0.20152E+01_JPRB, 0.20897E+01_JPRB, 0.21266E+01_JPRB ,&
&  0.21880E+01_JPRB, 0.22515E+01_JPRB, 0.23210E+01_JPRB, 0.23623E+01_JPRB, 0.24398E+01_JPRB, 0.25092E+01_JPRB, 0.24509E+01_JPRB ,&
&  0.22658E+01_JPRB, 0.20865E+01_JPRB, 0.15200E+01_JPRB, 0.14945E+01_JPRB, 0.16049E+01_JPRB, 0.15259E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.17893E+01_JPRB, 0.19284E+01_JPRB, 0.17623E+01_JPRB, 0.17260E+01_JPRB, 0.16993E+01_JPRB, 0.16076E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.13741E+01_JPRB, 0.11830E+01_JPRB, 0.10588E+01_JPRB, 0.98336E+00_JPRB, 0.98715E+00_JPRB, 0.97308E+00_JPRB, 0.94186E+00_JPRB ,&
&  0.89851E+00_JPRB, 0.92657E+00_JPRB, 0.10119E+01_JPRB, 0.11447E+01_JPRB, 0.14654E+01_JPRB, 0.29671E+01_JPRB, 0.31865E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.12742E+01_JPRB, 0.84048E+00_JPRB, 0.97977E+00_JPRB, 0.98810E+00_JPRB, 0.91897E+00_JPRB, 0.10208E+01_JPRB ,&
&  0.11228E+01_JPRB, 0.11438E+01_JPRB, 0.10682E+01_JPRB, 0.97357E+00_JPRB, 0.93601E+00_JPRB, 0.92357E+00_JPRB, 0.89900E+00_JPRB ,&
&  0.89923E+00_JPRB, 0.93112E+00_JPRB, 0.96192E+00_JPRB, 0.98848E+00_JPRB, 0.10177E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.11136E+01_JPRB, 0.11419E+01_JPRB, 0.11706E+01_JPRB, 0.12125E+01_JPRB, 0.12570E+01_JPRB, 0.12990E+01_JPRB, 0.13337E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.13975E+01_JPRB, 0.14285E+01_JPRB, 0.14529E+01_JPRB, 0.14717E+01_JPRB, 0.14871E+01_JPRB, 0.14944E+01_JPRB ,&
&  0.15000E+01_JPRB, 0.15120E+01_JPRB, 0.15370E+01_JPRB, 0.15621E+01_JPRB, 0.15806E+01_JPRB, 0.16027E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.16546E+01_JPRB, 0.16761E+01_JPRB, 0.17238E+01_JPRB, 0.17947E+01_JPRB, 0.18883E+01_JPRB, 0.21177E+01_JPRB, 0.20240E+01_JPRB ,&
&  0.23239E+01_JPRB, 0.26322E+01_JPRB, 0.25979E+01_JPRB, 0.21113E+01_JPRB, 0.19404E+01_JPRB, 0.18883E+01_JPRB, 0.19050E+01_JPRB ,&
&  0.19370E+01_JPRB, 0.19521E+01_JPRB, 0.19963E+01_JPRB, 0.20820E+01_JPRB, 0.22097E+01_JPRB, 0.23415E+01_JPRB, 0.24148E+01_JPRB ,&
&  0.25126E+01_JPRB, 0.26289E+01_JPRB, 0.27536E+01_JPRB, 0.28712E+01_JPRB, 0.30465E+01_JPRB, 0.31568E+01_JPRB, 0.31192E+01_JPRB ,&
&  0.29631E+01_JPRB, 0.28397E+01_JPRB, 0.24114E+01_JPRB, 0.23160E+01_JPRB, 0.24063E+01_JPRB, 0.21161E+01_JPRB, 0.18752E+01_JPRB ,&
&  0.20555E+01_JPRB, 0.17229E+01_JPRB, 0.14815E+01_JPRB, 0.15910E+01_JPRB, 0.14984E+01_JPRB, 0.13375E+01_JPRB, 0.12535E+01_JPRB ,&
&  0.10588E+01_JPRB, 0.89855E+00_JPRB, 0.78584E+00_JPRB, 0.76167E+00_JPRB, 0.77989E+00_JPRB, 0.82142E+00_JPRB, 0.83546E+00_JPRB ,&
&  0.87943E+00_JPRB, 0.96273E+00_JPRB, 0.10615E+01_JPRB, 0.12244E+01_JPRB, 0.15910E+01_JPRB, 0.16257E+01_JPRB, 0.18762E+01_JPRB ,&
&  0.13160E+01_JPRB, 0.10828E+01_JPRB, 0.11181E+01_JPRB, 0.10739E+01_JPRB, 0.11232E+01_JPRB, 0.11417E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.14703E+01_JPRB, 0.16213E+01_JPRB, 0.14849E+01_JPRB, 0.12852E+01_JPRB, 0.10849E+01_JPRB, 0.10013E+01_JPRB, 0.96706E+00_JPRB ,&
&  0.96983E+00_JPRB, 0.10077E+01_JPRB, 0.10382E+01_JPRB, 0.10584E+01_JPRB, 0.10848E+01_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.11251E+01_JPRB, 0.11547E+01_JPRB, 0.11798E+01_JPRB, 0.12121E+01_JPRB, 0.12508E+01_JPRB, 0.12936E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.13643E+01_JPRB, 0.13950E+01_JPRB, 0.14263E+01_JPRB, 0.14610E+01_JPRB, 0.14910E+01_JPRB, 0.15287E+01_JPRB, 0.15523E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.16005E+01_JPRB, 0.16272E+01_JPRB, 0.16625E+01_JPRB, 0.17023E+01_JPRB, 0.17471E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.18328E+01_JPRB, 0.18449E+01_JPRB, 0.18647E+01_JPRB, 0.18934E+01_JPRB, 0.19074E+01_JPRB, 0.20697E+01_JPRB, 0.20238E+01_JPRB ,&
&  0.26444E+01_JPRB, 0.29398E+01_JPRB, 0.28044E+01_JPRB, 0.23322E+01_JPRB, 0.21591E+01_JPRB, 0.21003E+01_JPRB, 0.21055E+01_JPRB ,&
&  0.21098E+01_JPRB, 0.21184E+01_JPRB, 0.21154E+01_JPRB, 0.21670E+01_JPRB, 0.22928E+01_JPRB, 0.24917E+01_JPRB, 0.26634E+01_JPRB ,&
&  0.27985E+01_JPRB, 0.29742E+01_JPRB, 0.31894E+01_JPRB, 0.34353E+01_JPRB, 0.37304E+01_JPRB, 0.39074E+01_JPRB, 0.39803E+01_JPRB ,&
&  0.39699E+01_JPRB, 0.39153E+01_JPRB, 0.35565E+01_JPRB, 0.37296E+01_JPRB, 0.38341E+01_JPRB, 0.30679E+01_JPRB, 0.24476E+01_JPRB ,&
&  0.19757E+01_JPRB, 0.16302E+01_JPRB, 0.15799E+01_JPRB, 0.15226E+01_JPRB, 0.13465E+01_JPRB, 0.12362E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.90437E+00_JPRB, 0.79470E+00_JPRB, 0.73191E+00_JPRB, 0.68685E+00_JPRB, 0.69512E+00_JPRB, 0.73252E+00_JPRB, 0.82357E+00_JPRB ,&
&  0.90641E+00_JPRB, 0.10004E+01_JPRB, 0.11265E+01_JPRB, 0.12583E+01_JPRB, 0.14525E+01_JPRB, 0.14711E+01_JPRB, 0.16455E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.13209E+01_JPRB, 0.13005E+01_JPRB, 0.97606E+00_JPRB, 0.11902E+01_JPRB, 0.12836E+01_JPRB, 0.14374E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.19520E+01_JPRB, 0.22988E+01_JPRB, 0.19373E+01_JPRB, 0.14406E+01_JPRB, 0.10592E+01_JPRB, 0.97435E+00_JPRB ,&
&  0.98446E+00_JPRB, 0.10195E+01_JPRB, 0.10535E+01_JPRB, 0.10747E+01_JPRB, 0.10991E+01_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.10738E+01_JPRB, 0.11013E+01_JPRB, 0.11274E+01_JPRB, 0.11483E+01_JPRB, 0.11619E+01_JPRB, 0.11869E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.12970E+01_JPRB, 0.13341E+01_JPRB, 0.13715E+01_JPRB, 0.14102E+01_JPRB, 0.14500E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.15795E+01_JPRB, 0.16523E+01_JPRB, 0.17104E+01_JPRB, 0.17970E+01_JPRB, 0.18759E+01_JPRB, 0.19469E+01_JPRB, 0.20668E+01_JPRB ,&
&  0.21780E+01_JPRB, 0.22972E+01_JPRB, 0.23914E+01_JPRB, 0.24488E+01_JPRB, 0.24442E+01_JPRB, 0.26153E+01_JPRB, 0.24102E+01_JPRB ,&
&  0.29125E+01_JPRB, 0.32035E+01_JPRB, 0.29957E+01_JPRB, 0.25182E+01_JPRB, 0.23617E+01_JPRB, 0.22686E+01_JPRB, 0.21982E+01_JPRB ,&
&  0.21877E+01_JPRB, 0.21438E+01_JPRB, 0.20365E+01_JPRB, 0.21460E+01_JPRB, 0.22810E+01_JPRB, 0.25001E+01_JPRB, 0.27625E+01_JPRB ,&
&  0.29815E+01_JPRB, 0.32249E+01_JPRB, 0.34851E+01_JPRB, 0.37700E+01_JPRB, 0.41249E+01_JPRB, 0.44181E+01_JPRB, 0.46559E+01_JPRB ,&
&  0.48422E+01_JPRB, 0.49644E+01_JPRB, 0.50569E+01_JPRB, 0.47854E+01_JPRB, 0.47541E+01_JPRB, 0.41472E+01_JPRB, 0.30358E+01_JPRB ,&
&  0.18877E+01_JPRB, 0.15527E+01_JPRB, 0.15426E+01_JPRB, 0.14690E+01_JPRB, 0.13206E+01_JPRB, 0.11257E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.85298E+00_JPRB, 0.74906E+00_JPRB, 0.71227E+00_JPRB, 0.68716E+00_JPRB, 0.68515E+00_JPRB, 0.74244E+00_JPRB, 0.80171E+00_JPRB ,&
&  0.90116E+00_JPRB, 0.10322E+01_JPRB, 0.11622E+01_JPRB, 0.13039E+01_JPRB, 0.15412E+01_JPRB, 0.17539E+01_JPRB, 0.15589E+01_JPRB ,&
&  0.14930E+01_JPRB, 0.13121E+01_JPRB, 0.12914E+01_JPRB, 0.13997E+01_JPRB, 0.13064E+01_JPRB, 0.15130E+01_JPRB, 0.17367E+01_JPRB ,&
&  0.20653E+01_JPRB, 0.19662E+01_JPRB, 0.22304E+01_JPRB, 0.32467E+01_JPRB, 0.27556E+01_JPRB, 0.23582E+01_JPRB, 0.96344E+00_JPRB ,&
&  0.97072E+00_JPRB, 0.10002E+01_JPRB, 0.10408E+01_JPRB, 0.10553E+01_JPRB, 0.10695E+01_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.93547E+00_JPRB, 0.98289E+00_JPRB, 0.10006E+01_JPRB, 0.10459E+01_JPRB, 0.10566E+01_JPRB, 0.10598E+01_JPRB, 0.10920E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.11531E+01_JPRB, 0.11963E+01_JPRB, 0.12400E+01_JPRB, 0.12777E+01_JPRB, 0.13037E+01_JPRB, 0.13452E+01_JPRB ,&
&  0.14078E+01_JPRB, 0.14863E+01_JPRB, 0.15866E+01_JPRB, 0.17195E+01_JPRB, 0.18256E+01_JPRB, 0.19354E+01_JPRB, 0.20763E+01_JPRB ,&
&  0.22033E+01_JPRB, 0.23942E+01_JPRB, 0.25843E+01_JPRB, 0.27648E+01_JPRB, 0.28862E+01_JPRB, 0.29310E+01_JPRB, 0.22883E+01_JPRB ,&
&  0.31286E+01_JPRB, 0.31444E+01_JPRB, 0.30216E+01_JPRB, 0.26481E+01_JPRB, 0.24761E+01_JPRB, 0.23468E+01_JPRB, 0.22570E+01_JPRB ,&
&  0.21972E+01_JPRB, 0.21087E+01_JPRB, 0.20003E+01_JPRB, 0.19398E+01_JPRB, 0.22326E+01_JPRB, 0.23800E+01_JPRB, 0.26330E+01_JPRB ,&
&  0.28966E+01_JPRB, 0.31474E+01_JPRB, 0.34055E+01_JPRB, 0.36879E+01_JPRB, 0.40590E+01_JPRB, 0.44789E+01_JPRB, 0.48835E+01_JPRB ,&
&  0.52522E+01_JPRB, 0.54659E+01_JPRB, 0.57574E+01_JPRB, 0.51659E+01_JPRB, 0.50129E+01_JPRB, 0.46896E+01_JPRB, 0.28984E+01_JPRB ,&
&  0.17240E+01_JPRB, 0.15726E+01_JPRB, 0.16227E+01_JPRB, 0.15271E+01_JPRB, 0.13265E+01_JPRB, 0.11173E+01_JPRB, 0.96383E+00_JPRB ,&
&  0.82118E+00_JPRB, 0.77298E+00_JPRB, 0.73621E+00_JPRB, 0.71673E+00_JPRB, 0.71636E+00_JPRB, 0.76199E+00_JPRB, 0.81080E+00_JPRB ,&
&  0.91180E+00_JPRB, 0.99863E+00_JPRB, 0.10987E+01_JPRB, 0.12020E+01_JPRB, 0.13491E+01_JPRB, 0.15259E+01_JPRB, 0.16940E+01_JPRB ,&
&  0.14220E+01_JPRB, 0.12474E+01_JPRB, 0.12142E+01_JPRB, 0.12524E+01_JPRB, 0.12710E+01_JPRB, 0.13517E+01_JPRB, 0.14541E+01_JPRB ,&
&  0.15946E+01_JPRB, 0.16898E+01_JPRB, 0.15260E+01_JPRB, 0.12649E+01_JPRB, 0.15647E+01_JPRB, 0.15963E+01_JPRB, 0.80454E+00_JPRB ,&
&  0.80554E+00_JPRB, 0.84809E+00_JPRB, 0.87896E+00_JPRB, 0.91209E+00_JPRB, 0.94246E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.76528E+00_JPRB, 0.77923E+00_JPRB, 0.78499E+00_JPRB, 0.83738E+00_JPRB, 0.85803E+00_JPRB, 0.89174E+00_JPRB, 0.91881E+00_JPRB ,&
&  0.95232E+00_JPRB, 0.99683E+00_JPRB, 0.10473E+01_JPRB, 0.10962E+01_JPRB, 0.11403E+01_JPRB, 0.11760E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.12459E+01_JPRB, 0.12879E+01_JPRB, 0.13432E+01_JPRB, 0.14209E+01_JPRB, 0.15431E+01_JPRB, 0.16845E+01_JPRB, 0.18394E+01_JPRB ,&
&  0.19998E+01_JPRB, 0.22040E+01_JPRB, 0.24348E+01_JPRB, 0.26888E+01_JPRB, 0.29511E+01_JPRB, 0.31413E+01_JPRB, 0.32779E+01_JPRB ,&
&  0.24800E+01_JPRB, 0.29969E+01_JPRB, 0.29411E+01_JPRB, 0.27047E+01_JPRB, 0.25334E+01_JPRB, 0.24371E+01_JPRB, 0.23394E+01_JPRB ,&
&  0.22546E+01_JPRB, 0.20892E+01_JPRB, 0.20286E+01_JPRB, 0.19981E+01_JPRB, 0.21235E+01_JPRB, 0.22224E+01_JPRB, 0.24038E+01_JPRB ,&
&  0.26537E+01_JPRB, 0.28546E+01_JPRB, 0.30990E+01_JPRB, 0.33529E+01_JPRB, 0.36853E+01_JPRB, 0.41965E+01_JPRB, 0.47897E+01_JPRB ,&
&  0.54345E+01_JPRB, 0.59515E+01_JPRB, 0.63297E+01_JPRB, 0.55956E+01_JPRB, 0.51283E+01_JPRB, 0.46400E+01_JPRB, 0.28481E+01_JPRB ,&
&  0.18514E+01_JPRB, 0.15313E+01_JPRB, 0.16090E+01_JPRB, 0.16359E+01_JPRB, 0.13922E+01_JPRB, 0.11995E+01_JPRB, 0.10123E+01_JPRB ,&
&  0.87415E+00_JPRB, 0.80076E+00_JPRB, 0.77144E+00_JPRB, 0.74547E+00_JPRB, 0.74940E+00_JPRB, 0.78272E+00_JPRB, 0.81811E+00_JPRB ,&
&  0.88404E+00_JPRB, 0.91890E+00_JPRB, 0.99568E+00_JPRB, 0.10613E+01_JPRB, 0.11281E+01_JPRB, 0.11745E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12298E+01_JPRB, 0.12271E+01_JPRB, 0.12471E+01_JPRB, 0.12452E+01_JPRB, 0.12539E+01_JPRB, 0.12768E+01_JPRB ,&
&  0.13015E+01_JPRB, 0.13109E+01_JPRB, 0.12627E+01_JPRB, 0.12061E+01_JPRB, 0.11116E+01_JPRB, 0.10299E+01_JPRB, 0.80131E+00_JPRB ,&
&  0.75796E+00_JPRB, 0.71551E+00_JPRB, 0.74092E+00_JPRB, 0.77209E+00_JPRB, 0.76736E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.67809E+00_JPRB, 0.66798E+00_JPRB, 0.65681E+00_JPRB, 0.66888E+00_JPRB, 0.68268E+00_JPRB, 0.71748E+00_JPRB, 0.72357E+00_JPRB ,&
&  0.74773E+00_JPRB, 0.81322E+00_JPRB, 0.89727E+00_JPRB, 0.95476E+00_JPRB, 0.10012E+01_JPRB, 0.10470E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11373E+01_JPRB, 0.11555E+01_JPRB, 0.11806E+01_JPRB, 0.12309E+01_JPRB, 0.13130E+01_JPRB, 0.14436E+01_JPRB ,&
&  0.15845E+01_JPRB, 0.17456E+01_JPRB, 0.19473E+01_JPRB, 0.21955E+01_JPRB, 0.24636E+01_JPRB, 0.26390E+01_JPRB, 0.27852E+01_JPRB ,&
&  0.16230E+01_JPRB, 0.25003E+01_JPRB, 0.30200E+01_JPRB, 0.28404E+01_JPRB, 0.26227E+01_JPRB, 0.25034E+01_JPRB, 0.23690E+01_JPRB ,&
&  0.21639E+01_JPRB, 0.19561E+01_JPRB, 0.18881E+01_JPRB, 0.20637E+01_JPRB, 0.20006E+01_JPRB, 0.20505E+01_JPRB, 0.21141E+01_JPRB ,&
&  0.22982E+01_JPRB, 0.24358E+01_JPRB, 0.25171E+01_JPRB, 0.26326E+01_JPRB, 0.28229E+01_JPRB, 0.32499E+01_JPRB, 0.39832E+01_JPRB ,&
&  0.49812E+01_JPRB, 0.60482E+01_JPRB, 0.66528E+01_JPRB, 0.57179E+01_JPRB, 0.58809E+01_JPRB, 0.53705E+01_JPRB, 0.43442E+01_JPRB ,&
&  0.24223E+01_JPRB, 0.16652E+01_JPRB, 0.16966E+01_JPRB, 0.17175E+01_JPRB, 0.15019E+01_JPRB, 0.13472E+01_JPRB, 0.11785E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.91690E+00_JPRB, 0.85903E+00_JPRB, 0.82259E+00_JPRB, 0.80752E+00_JPRB, 0.81551E+00_JPRB, 0.84121E+00_JPRB ,&
&  0.85980E+00_JPRB, 0.88334E+00_JPRB, 0.92576E+00_JPRB, 0.97544E+00_JPRB, 0.10340E+01_JPRB, 0.10851E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.12008E+01_JPRB, 0.12264E+01_JPRB, 0.12445E+01_JPRB, 0.12520E+01_JPRB, 0.12590E+01_JPRB, 0.12466E+01_JPRB ,&
&  0.12318E+01_JPRB, 0.12459E+01_JPRB, 0.12431E+01_JPRB, 0.12191E+01_JPRB, 0.11526E+01_JPRB, 0.10881E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.92856E+00_JPRB, 0.80804E+00_JPRB, 0.71305E+00_JPRB, 0.69231E+00_JPRB, 0.68156E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.64448E+00_JPRB, 0.61731E+00_JPRB, 0.61237E+00_JPRB, 0.62079E+00_JPRB, 0.64085E+00_JPRB, 0.65370E+00_JPRB, 0.65473E+00_JPRB ,&
&  0.66119E+00_JPRB, 0.67907E+00_JPRB, 0.70073E+00_JPRB, 0.76604E+00_JPRB, 0.84266E+00_JPRB, 0.90543E+00_JPRB, 0.94835E+00_JPRB ,&
&  0.98184E+00_JPRB, 0.99162E+00_JPRB, 0.98594E+00_JPRB, 0.98702E+00_JPRB, 0.99205E+00_JPRB, 0.10165E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.13048E+01_JPRB, 0.14663E+01_JPRB, 0.16914E+01_JPRB, 0.19963E+01_JPRB, 0.23179E+01_JPRB, 0.25645E+01_JPRB ,&
&  0.18916E+01_JPRB, 0.93133E+00_JPRB, 0.26706E+01_JPRB, 0.32064E+01_JPRB, 0.28380E+01_JPRB, 0.26941E+01_JPRB, 0.24242E+01_JPRB ,&
&  0.20758E+01_JPRB, 0.19286E+01_JPRB, 0.18116E+01_JPRB, 0.18835E+01_JPRB, 0.18598E+01_JPRB, 0.19076E+01_JPRB, 0.19599E+01_JPRB ,&
&  0.20574E+01_JPRB, 0.22029E+01_JPRB, 0.22291E+01_JPRB, 0.21983E+01_JPRB, 0.22060E+01_JPRB, 0.23585E+01_JPRB, 0.28384E+01_JPRB ,&
&  0.38205E+01_JPRB, 0.52925E+01_JPRB, 0.66731E+01_JPRB, 0.60035E+01_JPRB, 0.62155E+01_JPRB, 0.48541E+01_JPRB, 0.34285E+01_JPRB ,&
&  0.24585E+01_JPRB, 0.18982E+01_JPRB, 0.18115E+01_JPRB, 0.18169E+01_JPRB, 0.16462E+01_JPRB, 0.14432E+01_JPRB, 0.13889E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.10553E+01_JPRB, 0.96303E+00_JPRB, 0.90985E+00_JPRB, 0.88259E+00_JPRB, 0.87678E+00_JPRB, 0.88283E+00_JPRB ,&
&  0.88293E+00_JPRB, 0.89072E+00_JPRB, 0.90580E+00_JPRB, 0.94292E+00_JPRB, 0.99432E+00_JPRB, 0.10600E+01_JPRB, 0.11114E+01_JPRB ,&
&  0.11396E+01_JPRB, 0.11584E+01_JPRB, 0.11793E+01_JPRB, 0.11895E+01_JPRB, 0.11969E+01_JPRB, 0.11729E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11837E+01_JPRB, 0.12066E+01_JPRB, 0.12271E+01_JPRB, 0.12577E+01_JPRB, 0.12131E+01_JPRB, 0.11690E+01_JPRB, 0.11708E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.10224E+01_JPRB, 0.86439E+00_JPRB, 0.72951E+00_JPRB, 0.67653E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.72506E+00_JPRB, 0.68988E+00_JPRB, 0.65825E+00_JPRB, 0.65330E+00_JPRB, 0.66935E+00_JPRB, 0.66499E+00_JPRB, 0.66338E+00_JPRB ,&
&  0.65851E+00_JPRB, 0.65264E+00_JPRB, 0.63667E+00_JPRB, 0.64719E+00_JPRB, 0.68781E+00_JPRB, 0.75396E+00_JPRB, 0.80430E+00_JPRB ,&
&  0.84676E+00_JPRB, 0.85966E+00_JPRB, 0.85714E+00_JPRB, 0.85334E+00_JPRB, 0.84891E+00_JPRB, 0.84967E+00_JPRB, 0.86151E+00_JPRB ,&
&  0.89873E+00_JPRB, 0.96102E+00_JPRB, 0.10465E+01_JPRB, 0.11582E+01_JPRB, 0.13266E+01_JPRB, 0.16344E+01_JPRB, 0.21169E+01_JPRB ,&
&  0.28941E+01_JPRB, 0.20859E+01_JPRB, 0.12553E+01_JPRB, 0.27877E+01_JPRB, 0.31263E+01_JPRB, 0.30901E+01_JPRB, 0.25663E+01_JPRB ,&
&  0.23338E+01_JPRB, 0.20441E+01_JPRB, 0.18270E+01_JPRB, 0.18557E+01_JPRB, 0.17219E+01_JPRB, 0.17405E+01_JPRB, 0.18226E+01_JPRB ,&
&  0.18935E+01_JPRB, 0.19702E+01_JPRB, 0.20364E+01_JPRB, 0.20114E+01_JPRB, 0.19278E+01_JPRB, 0.18924E+01_JPRB, 0.20547E+01_JPRB ,&
&  0.26568E+01_JPRB, 0.39093E+01_JPRB, 0.59389E+01_JPRB, 0.61394E+01_JPRB, 0.63517E+01_JPRB, 0.44185E+01_JPRB, 0.31522E+01_JPRB ,&
&  0.25285E+01_JPRB, 0.23852E+01_JPRB, 0.22251E+01_JPRB, 0.19782E+01_JPRB, 0.17208E+01_JPRB, 0.15668E+01_JPRB, 0.15679E+01_JPRB ,&
&  0.13797E+01_JPRB, 0.12416E+01_JPRB, 0.11222E+01_JPRB, 0.10475E+01_JPRB, 0.99483E+00_JPRB, 0.96102E+00_JPRB, 0.94089E+00_JPRB ,&
&  0.94215E+00_JPRB, 0.93946E+00_JPRB, 0.94743E+00_JPRB, 0.96193E+00_JPRB, 0.98530E+00_JPRB, 0.10247E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11226E+01_JPRB, 0.11271E+01_JPRB, 0.11240E+01_JPRB, 0.10983E+01_JPRB, 0.10816E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.11314E+01_JPRB, 0.11408E+01_JPRB, 0.11799E+01_JPRB, 0.12013E+01_JPRB, 0.12079E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.12464E+01_JPRB, 0.11965E+01_JPRB, 0.10546E+01_JPRB, 0.83083E+00_JPRB, 0.76098E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.82855E+00_JPRB, 0.80123E+00_JPRB, 0.78541E+00_JPRB, 0.78665E+00_JPRB, 0.78814E+00_JPRB, 0.77007E+00_JPRB, 0.74519E+00_JPRB ,&
&  0.71162E+00_JPRB, 0.68584E+00_JPRB, 0.64971E+00_JPRB, 0.61370E+00_JPRB, 0.60578E+00_JPRB, 0.62526E+00_JPRB, 0.66534E+00_JPRB ,&
&  0.70236E+00_JPRB, 0.71767E+00_JPRB, 0.72819E+00_JPRB, 0.73714E+00_JPRB, 0.74538E+00_JPRB, 0.74897E+00_JPRB, 0.74926E+00_JPRB ,&
&  0.75922E+00_JPRB, 0.78093E+00_JPRB, 0.81394E+00_JPRB, 0.85683E+00_JPRB, 0.92274E+00_JPRB, 0.10476E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.18289E+01_JPRB, 0.21349E+01_JPRB, 0.10154E+01_JPRB, 0.21543E+01_JPRB, 0.31200E+01_JPRB, 0.31347E+01_JPRB, 0.29470E+01_JPRB ,&
&  0.26700E+01_JPRB, 0.22389E+01_JPRB, 0.20695E+01_JPRB, 0.18716E+01_JPRB, 0.16658E+01_JPRB, 0.16080E+01_JPRB, 0.16264E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.16142E+01_JPRB, 0.16279E+01_JPRB, 0.16884E+01_JPRB, 0.16684E+01_JPRB, 0.15996E+01_JPRB, 0.16224E+01_JPRB ,&
&  0.19121E+01_JPRB, 0.27695E+01_JPRB, 0.46656E+01_JPRB, 0.58558E+01_JPRB, 0.62577E+01_JPRB, 0.48320E+01_JPRB, 0.35506E+01_JPRB ,&
&  0.30398E+01_JPRB, 0.30716E+01_JPRB, 0.27963E+01_JPRB, 0.22926E+01_JPRB, 0.18827E+01_JPRB, 0.17519E+01_JPRB, 0.17082E+01_JPRB ,&
&  0.15490E+01_JPRB, 0.14017E+01_JPRB, 0.13046E+01_JPRB, 0.12238E+01_JPRB, 0.11596E+01_JPRB, 0.10929E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.10220E+01_JPRB, 0.10100E+01_JPRB, 0.10066E+01_JPRB, 0.10036E+01_JPRB, 0.10001E+01_JPRB, 0.10069E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10802E+01_JPRB, 0.10850E+01_JPRB, 0.10631E+01_JPRB, 0.10475E+01_JPRB, 0.10493E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10888E+01_JPRB, 0.10906E+01_JPRB, 0.10978E+01_JPRB, 0.11574E+01_JPRB, 0.11783E+01_JPRB, 0.12130E+01_JPRB ,&
&  0.12553E+01_JPRB, 0.12997E+01_JPRB, 0.12540E+01_JPRB, 0.94898E+00_JPRB, 0.88062E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.91656E+00_JPRB, 0.88353E+00_JPRB, 0.87977E+00_JPRB, 0.88239E+00_JPRB, 0.88169E+00_JPRB, 0.87186E+00_JPRB, 0.84852E+00_JPRB ,&
&  0.81795E+00_JPRB, 0.79240E+00_JPRB, 0.76218E+00_JPRB, 0.72254E+00_JPRB, 0.66601E+00_JPRB, 0.61671E+00_JPRB, 0.61008E+00_JPRB ,&
&  0.63582E+00_JPRB, 0.64276E+00_JPRB, 0.64865E+00_JPRB, 0.65799E+00_JPRB, 0.66999E+00_JPRB, 0.68373E+00_JPRB, 0.69368E+00_JPRB ,&
&  0.69917E+00_JPRB, 0.70885E+00_JPRB, 0.72320E+00_JPRB, 0.74037E+00_JPRB, 0.76871E+00_JPRB, 0.82814E+00_JPRB, 0.94698E+00_JPRB ,&
&  0.12692E+01_JPRB, 0.17575E+01_JPRB, 0.18745E+01_JPRB, 0.31369E+01_JPRB, 0.36194E+01_JPRB, 0.34461E+01_JPRB, 0.34628E+01_JPRB ,&
&  0.33790E+01_JPRB, 0.29725E+01_JPRB, 0.25325E+01_JPRB, 0.19956E+01_JPRB, 0.17076E+01_JPRB, 0.15756E+01_JPRB, 0.15088E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.14102E+01_JPRB, 0.13432E+01_JPRB, 0.13374E+01_JPRB, 0.13424E+01_JPRB, 0.13232E+01_JPRB, 0.13351E+01_JPRB ,&
&  0.14701E+01_JPRB, 0.19812E+01_JPRB, 0.33286E+01_JPRB, 0.54938E+01_JPRB, 0.62049E+01_JPRB, 0.55451E+01_JPRB, 0.45178E+01_JPRB ,&
&  0.44840E+01_JPRB, 0.41796E+01_JPRB, 0.36985E+01_JPRB, 0.28802E+01_JPRB, 0.22476E+01_JPRB, 0.21798E+01_JPRB, 0.18592E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.15227E+01_JPRB, 0.14381E+01_JPRB, 0.13758E+01_JPRB, 0.13215E+01_JPRB, 0.12490E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.10709E+01_JPRB, 0.10535E+01_JPRB, 0.10446E+01_JPRB, 0.10309E+01_JPRB, 0.10182E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.10316E+01_JPRB, 0.10435E+01_JPRB, 0.10104E+01_JPRB, 0.10041E+01_JPRB, 0.10140E+01_JPRB, 0.10180E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.10472E+01_JPRB, 0.10736E+01_JPRB, 0.10806E+01_JPRB, 0.11085E+01_JPRB, 0.11417E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.11470E+01_JPRB, 0.10983E+01_JPRB, 0.10092E+01_JPRB, 0.96310E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.96288E+00_JPRB, 0.93318E+00_JPRB, 0.91819E+00_JPRB, 0.90886E+00_JPRB, 0.90478E+00_JPRB, 0.90114E+00_JPRB, 0.88303E+00_JPRB ,&
&  0.86180E+00_JPRB, 0.84767E+00_JPRB, 0.82381E+00_JPRB, 0.79833E+00_JPRB, 0.74898E+00_JPRB, 0.68500E+00_JPRB, 0.65405E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.63282E+00_JPRB, 0.64080E+00_JPRB, 0.64593E+00_JPRB, 0.64893E+00_JPRB, 0.65630E+00_JPRB, 0.66372E+00_JPRB ,&
&  0.67272E+00_JPRB, 0.67765E+00_JPRB, 0.68655E+00_JPRB, 0.69566E+00_JPRB, 0.70914E+00_JPRB, 0.75035E+00_JPRB, 0.82307E+00_JPRB ,&
&  0.10262E+01_JPRB, 0.16105E+01_JPRB, 0.13980E+01_JPRB, 0.30480E+01_JPRB, 0.34555E+01_JPRB, 0.34895E+01_JPRB, 0.32956E+01_JPRB ,&
&  0.33228E+01_JPRB, 0.33026E+01_JPRB, 0.25408E+01_JPRB, 0.20527E+01_JPRB, 0.17289E+01_JPRB, 0.15645E+01_JPRB, 0.14672E+01_JPRB ,&
&  0.13980E+01_JPRB, 0.13281E+01_JPRB, 0.12418E+01_JPRB, 0.11760E+01_JPRB, 0.11506E+01_JPRB, 0.11344E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.14657E+01_JPRB, 0.23326E+01_JPRB, 0.42331E+01_JPRB, 0.59408E+01_JPRB, 0.61509E+01_JPRB, 0.54868E+01_JPRB ,&
&  0.55156E+01_JPRB, 0.55636E+01_JPRB, 0.49867E+01_JPRB, 0.38017E+01_JPRB, 0.29670E+01_JPRB, 0.25541E+01_JPRB, 0.21314E+01_JPRB ,&
&  0.18610E+01_JPRB, 0.16940E+01_JPRB, 0.15906E+01_JPRB, 0.15033E+01_JPRB, 0.14314E+01_JPRB, 0.13540E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.11052E+01_JPRB, 0.10743E+01_JPRB, 0.10602E+01_JPRB, 0.10550E+01_JPRB, 0.10461E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10399E+01_JPRB, 0.98697E+00_JPRB, 0.97052E+00_JPRB, 0.98032E+00_JPRB, 0.98643E+00_JPRB, 0.99673E+00_JPRB ,&
&  0.10227E+01_JPRB, 0.10354E+01_JPRB, 0.10455E+01_JPRB, 0.10398E+01_JPRB, 0.10626E+01_JPRB, 0.11196E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.10948E+01_JPRB, 0.10652E+01_JPRB, 0.10252E+01_JPRB, 0.99275E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.97763E+00_JPRB, 0.95619E+00_JPRB, 0.93090E+00_JPRB, 0.90845E+00_JPRB, 0.90565E+00_JPRB, 0.90577E+00_JPRB, 0.89166E+00_JPRB ,&
&  0.87664E+00_JPRB, 0.85638E+00_JPRB, 0.82494E+00_JPRB, 0.78994E+00_JPRB, 0.75127E+00_JPRB, 0.70475E+00_JPRB, 0.66255E+00_JPRB ,&
&  0.64358E+00_JPRB, 0.62822E+00_JPRB, 0.62917E+00_JPRB, 0.63572E+00_JPRB, 0.64465E+00_JPRB, 0.65004E+00_JPRB, 0.65711E+00_JPRB ,&
&  0.66437E+00_JPRB, 0.66906E+00_JPRB, 0.67432E+00_JPRB, 0.67905E+00_JPRB, 0.68424E+00_JPRB, 0.70852E+00_JPRB, 0.78607E+00_JPRB ,&
&  0.95425E+00_JPRB, 0.18863E+01_JPRB, 0.14213E+01_JPRB, 0.25131E+01_JPRB, 0.28836E+01_JPRB, 0.30845E+01_JPRB, 0.29318E+01_JPRB ,&
&  0.29739E+01_JPRB, 0.28039E+01_JPRB, 0.23373E+01_JPRB, 0.19816E+01_JPRB, 0.17241E+01_JPRB, 0.15588E+01_JPRB, 0.14446E+01_JPRB ,&
&  0.13645E+01_JPRB, 0.12745E+01_JPRB, 0.11788E+01_JPRB, 0.10944E+01_JPRB, 0.10519E+01_JPRB, 0.10309E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.11632E+01_JPRB, 0.16344E+01_JPRB, 0.31501E+01_JPRB, 0.50059E+01_JPRB, 0.60981E+01_JPRB, 0.63067E+01_JPRB ,&
&  0.64133E+01_JPRB, 0.68502E+01_JPRB, 0.62319E+01_JPRB, 0.49079E+01_JPRB, 0.38635E+01_JPRB, 0.31089E+01_JPRB, 0.25311E+01_JPRB ,&
&  0.21779E+01_JPRB, 0.19670E+01_JPRB, 0.18198E+01_JPRB, 0.16968E+01_JPRB, 0.15781E+01_JPRB, 0.14453E+01_JPRB, 0.13056E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.11259E+01_JPRB, 0.10825E+01_JPRB, 0.10707E+01_JPRB, 0.10711E+01_JPRB, 0.10700E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10629E+01_JPRB, 0.10087E+01_JPRB, 0.98442E+00_JPRB, 0.98138E+00_JPRB, 0.98892E+00_JPRB, 0.99333E+00_JPRB ,&
&  0.10283E+01_JPRB, 0.10359E+01_JPRB, 0.10244E+01_JPRB, 0.10217E+01_JPRB, 0.10404E+01_JPRB, 0.11118E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.10702E+01_JPRB, 0.10423E+01_JPRB, 0.10246E+01_JPRB, 0.10026E+01_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.98354E+00_JPRB, 0.96080E+00_JPRB, 0.94655E+00_JPRB, 0.93606E+00_JPRB, 0.93739E+00_JPRB, 0.93251E+00_JPRB, 0.91157E+00_JPRB ,&
&  0.87830E+00_JPRB, 0.83852E+00_JPRB, 0.79771E+00_JPRB, 0.76421E+00_JPRB, 0.73835E+00_JPRB, 0.71074E+00_JPRB, 0.68746E+00_JPRB ,&
&  0.66802E+00_JPRB, 0.64699E+00_JPRB, 0.63363E+00_JPRB, 0.62717E+00_JPRB, 0.62869E+00_JPRB, 0.63360E+00_JPRB, 0.64149E+00_JPRB ,&
&  0.65148E+00_JPRB, 0.65813E+00_JPRB, 0.65935E+00_JPRB, 0.66168E+00_JPRB, 0.66099E+00_JPRB, 0.68236E+00_JPRB, 0.76306E+00_JPRB ,&
&  0.96241E+00_JPRB, 0.92290E+00_JPRB, 0.16274E+01_JPRB, 0.19910E+01_JPRB, 0.22532E+01_JPRB, 0.25169E+01_JPRB, 0.26971E+01_JPRB ,&
&  0.27840E+01_JPRB, 0.24857E+01_JPRB, 0.21041E+01_JPRB, 0.18426E+01_JPRB, 0.16345E+01_JPRB, 0.14853E+01_JPRB, 0.13715E+01_JPRB ,&
&  0.12615E+01_JPRB, 0.11790E+01_JPRB, 0.11123E+01_JPRB, 0.10315E+01_JPRB, 0.98239E+00_JPRB, 0.96355E+00_JPRB, 0.95759E+00_JPRB ,&
&  0.94998E+00_JPRB, 0.99089E+00_JPRB, 0.12176E+01_JPRB, 0.20111E+01_JPRB, 0.40161E+01_JPRB, 0.55988E+01_JPRB, 0.65966E+01_JPRB ,&
&  0.77516E+01_JPRB, 0.83008E+01_JPRB, 0.74173E+01_JPRB, 0.59721E+01_JPRB, 0.45606E+01_JPRB, 0.36087E+01_JPRB, 0.28257E+01_JPRB ,&
&  0.23719E+01_JPRB, 0.21190E+01_JPRB, 0.19717E+01_JPRB, 0.18540E+01_JPRB, 0.17308E+01_JPRB, 0.15604E+01_JPRB, 0.13682E+01_JPRB ,&
&  0.12161E+01_JPRB, 0.11290E+01_JPRB, 0.10754E+01_JPRB, 0.10454E+01_JPRB, 0.10327E+01_JPRB, 0.10361E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10641E+01_JPRB, 0.10774E+01_JPRB, 0.10290E+01_JPRB, 0.99497E+00_JPRB, 0.98843E+00_JPRB, 0.10022E+01_JPRB, 0.10153E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.10018E+01_JPRB, 0.98512E+00_JPRB, 0.98818E+00_JPRB, 0.10043E+01_JPRB, 0.10982E+01_JPRB, 0.11098E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10679E+01_JPRB, 0.10306E+01_JPRB, 0.10187E+01_JPRB, 0.10099E+01_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.98535E+00_JPRB, 0.96654E+00_JPRB, 0.96645E+00_JPRB, 0.98723E+00_JPRB, 0.10065E+01_JPRB, 0.99198E+00_JPRB, 0.93417E+00_JPRB ,&
&  0.87833E+00_JPRB, 0.83003E+00_JPRB, 0.78602E+00_JPRB, 0.75174E+00_JPRB, 0.72886E+00_JPRB, 0.71189E+00_JPRB, 0.69534E+00_JPRB ,&
&  0.68244E+00_JPRB, 0.67058E+00_JPRB, 0.64953E+00_JPRB, 0.63021E+00_JPRB, 0.62104E+00_JPRB, 0.61913E+00_JPRB, 0.62330E+00_JPRB ,&
&  0.62863E+00_JPRB, 0.63461E+00_JPRB, 0.63847E+00_JPRB, 0.64106E+00_JPRB, 0.64159E+00_JPRB, 0.66093E+00_JPRB, 0.73102E+00_JPRB ,&
&  0.95883E+00_JPRB, 0.14094E+01_JPRB, 0.16737E+01_JPRB, 0.16932E+01_JPRB, 0.18382E+01_JPRB, 0.21058E+01_JPRB, 0.24256E+01_JPRB ,&
&  0.24171E+01_JPRB, 0.21940E+01_JPRB, 0.18660E+01_JPRB, 0.16371E+01_JPRB, 0.14637E+01_JPRB, 0.13405E+01_JPRB, 0.12232E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.10636E+01_JPRB, 0.99953E+00_JPRB, 0.94344E+00_JPRB, 0.92130E+00_JPRB, 0.92010E+00_JPRB, 0.92148E+00_JPRB ,&
&  0.91750E+00_JPRB, 0.91622E+00_JPRB, 0.10082E+01_JPRB, 0.15153E+01_JPRB, 0.31304E+01_JPRB, 0.42267E+01_JPRB, 0.60821E+01_JPRB ,&
&  0.79910E+01_JPRB, 0.79794E+01_JPRB, 0.72504E+01_JPRB, 0.60657E+01_JPRB, 0.47417E+01_JPRB, 0.36221E+01_JPRB, 0.29076E+01_JPRB ,&
&  0.24602E+01_JPRB, 0.21113E+01_JPRB, 0.19006E+01_JPRB, 0.17664E+01_JPRB, 0.16557E+01_JPRB, 0.15497E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.11746E+01_JPRB, 0.10952E+01_JPRB, 0.10416E+01_JPRB, 0.10140E+01_JPRB, 0.10082E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.10163E+01_JPRB, 0.10185E+01_JPRB, 0.99928E+00_JPRB, 0.99828E+00_JPRB, 0.10103E+01_JPRB, 0.10174E+01_JPRB ,&
&  0.98950E+00_JPRB, 0.96591E+00_JPRB, 0.96224E+00_JPRB, 0.96035E+00_JPRB, 0.99550E+00_JPRB, 0.10953E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.10589E+01_JPRB, 0.10356E+01_JPRB, 0.10119E+01_JPRB, 0.10114E+01_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.97369E+00_JPRB, 0.95332E+00_JPRB, 0.94782E+00_JPRB, 0.97713E+00_JPRB, 0.10108E+01_JPRB, 0.10052E+01_JPRB, 0.95166E+00_JPRB ,&
&  0.87712E+00_JPRB, 0.81731E+00_JPRB, 0.77581E+00_JPRB, 0.74650E+00_JPRB, 0.72371E+00_JPRB, 0.70381E+00_JPRB, 0.68249E+00_JPRB ,&
&  0.66560E+00_JPRB, 0.65540E+00_JPRB, 0.64804E+00_JPRB, 0.63889E+00_JPRB, 0.62396E+00_JPRB, 0.61243E+00_JPRB, 0.60833E+00_JPRB ,&
&  0.60615E+00_JPRB, 0.60572E+00_JPRB, 0.61106E+00_JPRB, 0.61761E+00_JPRB, 0.62063E+00_JPRB, 0.63248E+00_JPRB, 0.67846E+00_JPRB ,&
&  0.84086E+00_JPRB, 0.12362E+01_JPRB, 0.12455E+01_JPRB, 0.13670E+01_JPRB, 0.16068E+01_JPRB, 0.18388E+01_JPRB, 0.20489E+01_JPRB ,&
&  0.20496E+01_JPRB, 0.19324E+01_JPRB, 0.17059E+01_JPRB, 0.14798E+01_JPRB, 0.12967E+01_JPRB, 0.11850E+01_JPRB, 0.11061E+01_JPRB ,&
&  0.10375E+01_JPRB, 0.97101E+00_JPRB, 0.91263E+00_JPRB, 0.86892E+00_JPRB, 0.83959E+00_JPRB, 0.82851E+00_JPRB, 0.83179E+00_JPRB ,&
&  0.83511E+00_JPRB, 0.83479E+00_JPRB, 0.85179E+00_JPRB, 0.10300E+01_JPRB, 0.18573E+01_JPRB, 0.30478E+01_JPRB, 0.43574E+01_JPRB ,&
&  0.48271E+01_JPRB, 0.50700E+01_JPRB, 0.50347E+01_JPRB, 0.47282E+01_JPRB, 0.41534E+01_JPRB, 0.35758E+01_JPRB, 0.29512E+01_JPRB ,&
&  0.24882E+01_JPRB, 0.21769E+01_JPRB, 0.19508E+01_JPRB, 0.17581E+01_JPRB, 0.15842E+01_JPRB, 0.14441E+01_JPRB, 0.13460E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12113E+01_JPRB, 0.11514E+01_JPRB, 0.10968E+01_JPRB, 0.10558E+01_JPRB, 0.10327E+01_JPRB, 0.10282E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.99611E+00_JPRB, 0.98683E+00_JPRB, 0.98475E+00_JPRB, 0.98815E+00_JPRB, 0.98536E+00_JPRB, 0.97829E+00_JPRB ,&
&  0.97833E+00_JPRB, 0.97672E+00_JPRB, 0.97169E+00_JPRB, 0.96697E+00_JPRB, 0.99321E+00_JPRB, 0.10648E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10764E+01_JPRB, 0.10531E+01_JPRB, 0.10331E+01_JPRB, 0.10184E+01_JPRB, 0.10069E+01_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.10504E+01_JPRB, 0.96756E+00_JPRB, 0.92089E+00_JPRB, 0.91161E+00_JPRB, 0.92220E+00_JPRB, 0.91139E+00_JPRB, 0.88157E+00_JPRB ,&
&  0.84729E+00_JPRB, 0.81538E+00_JPRB, 0.78624E+00_JPRB, 0.75411E+00_JPRB, 0.72260E+00_JPRB, 0.69515E+00_JPRB, 0.66943E+00_JPRB ,&
&  0.64546E+00_JPRB, 0.63110E+00_JPRB, 0.62148E+00_JPRB, 0.61482E+00_JPRB, 0.60996E+00_JPRB, 0.60151E+00_JPRB, 0.59283E+00_JPRB ,&
&  0.58617E+00_JPRB, 0.57648E+00_JPRB, 0.57293E+00_JPRB, 0.57986E+00_JPRB, 0.58791E+00_JPRB, 0.59844E+00_JPRB, 0.63336E+00_JPRB ,&
&  0.75979E+00_JPRB, 0.92798E+00_JPRB, 0.10541E+01_JPRB, 0.11686E+01_JPRB, 0.12853E+01_JPRB, 0.14222E+01_JPRB, 0.14673E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.14087E+01_JPRB, 0.13719E+01_JPRB, 0.12555E+01_JPRB, 0.11257E+01_JPRB, 0.10384E+01_JPRB, 0.97689E+00_JPRB ,&
&  0.92510E+00_JPRB, 0.87176E+00_JPRB, 0.81856E+00_JPRB, 0.77638E+00_JPRB, 0.75106E+00_JPRB, 0.74136E+00_JPRB, 0.74173E+00_JPRB ,&
&  0.74632E+00_JPRB, 0.75352E+00_JPRB, 0.75954E+00_JPRB, 0.78568E+00_JPRB, 0.97446E+00_JPRB, 0.14939E+01_JPRB, 0.23003E+01_JPRB ,&
&  0.27227E+01_JPRB, 0.28931E+01_JPRB, 0.30255E+01_JPRB, 0.30576E+01_JPRB, 0.28679E+01_JPRB, 0.26493E+01_JPRB, 0.24011E+01_JPRB ,&
&  0.21523E+01_JPRB, 0.18911E+01_JPRB, 0.16674E+01_JPRB, 0.14915E+01_JPRB, 0.13546E+01_JPRB, 0.12599E+01_JPRB, 0.12063E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.11683E+01_JPRB, 0.11560E+01_JPRB, 0.11302E+01_JPRB, 0.10972E+01_JPRB, 0.10721E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.10145E+01_JPRB, 0.99606E+00_JPRB, 0.98172E+00_JPRB, 0.97296E+00_JPRB, 0.96409E+00_JPRB, 0.95894E+00_JPRB ,&
&  0.95783E+00_JPRB, 0.96399E+00_JPRB, 0.94789E+00_JPRB, 0.94497E+00_JPRB, 0.10037E+01_JPRB, 0.10247E+01_JPRB, 0.10467E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10504E+01_JPRB, 0.10201E+01_JPRB, 0.10099E+01_JPRB, 0.11082E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.94683E+00_JPRB, 0.90978E+00_JPRB, 0.86119E+00_JPRB, 0.83453E+00_JPRB, 0.81756E+00_JPRB, 0.79273E+00_JPRB, 0.77275E+00_JPRB ,&
&  0.75604E+00_JPRB, 0.73853E+00_JPRB, 0.71618E+00_JPRB, 0.69006E+00_JPRB, 0.66486E+00_JPRB, 0.64638E+00_JPRB, 0.63132E+00_JPRB ,&
&  0.61741E+00_JPRB, 0.60415E+00_JPRB, 0.59210E+00_JPRB, 0.58297E+00_JPRB, 0.57350E+00_JPRB, 0.56626E+00_JPRB, 0.56031E+00_JPRB ,&
&  0.55349E+00_JPRB, 0.54534E+00_JPRB, 0.53485E+00_JPRB, 0.53819E+00_JPRB, 0.54935E+00_JPRB, 0.56411E+00_JPRB, 0.58364E+00_JPRB ,&
&  0.66004E+00_JPRB, 0.73694E+00_JPRB, 0.84956E+00_JPRB, 0.97195E+00_JPRB, 0.10608E+01_JPRB, 0.11262E+01_JPRB, 0.11229E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.10757E+01_JPRB, 0.10509E+01_JPRB, 0.99994E+00_JPRB, 0.96228E+00_JPRB, 0.88667E+00_JPRB, 0.81852E+00_JPRB ,&
&  0.77070E+00_JPRB, 0.73969E+00_JPRB, 0.71381E+00_JPRB, 0.68813E+00_JPRB, 0.67292E+00_JPRB, 0.66982E+00_JPRB, 0.67577E+00_JPRB ,&
&  0.68218E+00_JPRB, 0.69077E+00_JPRB, 0.69878E+00_JPRB, 0.70954E+00_JPRB, 0.73892E+00_JPRB, 0.89000E+00_JPRB, 0.12843E+01_JPRB ,&
&  0.15979E+01_JPRB, 0.18038E+01_JPRB, 0.19417E+01_JPRB, 0.20003E+01_JPRB, 0.19506E+01_JPRB, 0.18417E+01_JPRB, 0.17003E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.13827E+01_JPRB, 0.12337E+01_JPRB, 0.11177E+01_JPRB, 0.10292E+01_JPRB, 0.97128E+00_JPRB, 0.94221E+00_JPRB ,&
&  0.94105E+00_JPRB, 0.95966E+00_JPRB, 0.98654E+00_JPRB, 0.10115E+01_JPRB, 0.10242E+01_JPRB, 0.10223E+01_JPRB, 0.10132E+01_JPRB ,&
&  0.99748E+00_JPRB, 0.97643E+00_JPRB, 0.95256E+00_JPRB, 0.93166E+00_JPRB, 0.91460E+00_JPRB, 0.90172E+00_JPRB, 0.89319E+00_JPRB ,&
&  0.88787E+00_JPRB, 0.88691E+00_JPRB, 0.89185E+00_JPRB, 0.90939E+00_JPRB, 0.91817E+00_JPRB, 0.93515E+00_JPRB, 0.94518E+00_JPRB ,&
&  0.95570E+00_JPRB, 0.95572E+00_JPRB, 0.93997E+00_JPRB, 0.90610E+00_JPRB, 0.90900E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.82201E+00_JPRB, 0.81085E+00_JPRB, 0.78483E+00_JPRB, 0.75702E+00_JPRB, 0.73404E+00_JPRB, 0.71239E+00_JPRB, 0.69273E+00_JPRB ,&
&  0.67510E+00_JPRB, 0.65809E+00_JPRB, 0.63938E+00_JPRB, 0.62171E+00_JPRB, 0.60472E+00_JPRB, 0.59102E+00_JPRB, 0.57870E+00_JPRB ,&
&  0.57000E+00_JPRB, 0.56340E+00_JPRB, 0.55662E+00_JPRB, 0.54889E+00_JPRB, 0.54120E+00_JPRB, 0.53382E+00_JPRB, 0.52618E+00_JPRB ,&
&  0.52017E+00_JPRB, 0.51673E+00_JPRB, 0.50766E+00_JPRB, 0.50472E+00_JPRB, 0.51297E+00_JPRB, 0.52631E+00_JPRB, 0.54011E+00_JPRB ,&
&  0.59109E+00_JPRB, 0.59123E+00_JPRB, 0.61511E+00_JPRB, 0.72960E+00_JPRB, 0.82179E+00_JPRB, 0.92274E+00_JPRB, 0.96493E+00_JPRB ,&
&  0.97050E+00_JPRB, 0.97775E+00_JPRB, 0.94591E+00_JPRB, 0.90325E+00_JPRB, 0.83825E+00_JPRB, 0.78399E+00_JPRB, 0.74380E+00_JPRB ,&
&  0.70857E+00_JPRB, 0.67608E+00_JPRB, 0.65083E+00_JPRB, 0.63126E+00_JPRB, 0.61673E+00_JPRB, 0.61110E+00_JPRB, 0.61347E+00_JPRB ,&
&  0.61778E+00_JPRB, 0.62229E+00_JPRB, 0.62836E+00_JPRB, 0.63958E+00_JPRB, 0.65518E+00_JPRB, 0.68983E+00_JPRB, 0.82058E+00_JPRB ,&
&  0.10306E+01_JPRB, 0.11987E+01_JPRB, 0.13492E+01_JPRB, 0.14376E+01_JPRB, 0.14379E+01_JPRB, 0.13708E+01_JPRB, 0.12417E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.99225E+00_JPRB, 0.92407E+00_JPRB, 0.88236E+00_JPRB, 0.85994E+00_JPRB, 0.84473E+00_JPRB, 0.83209E+00_JPRB ,&
&  0.82090E+00_JPRB, 0.81560E+00_JPRB, 0.82030E+00_JPRB, 0.83413E+00_JPRB, 0.85413E+00_JPRB, 0.86983E+00_JPRB, 0.87533E+00_JPRB ,&
&  0.87264E+00_JPRB, 0.86446E+00_JPRB, 0.85523E+00_JPRB, 0.84345E+00_JPRB, 0.82972E+00_JPRB, 0.81493E+00_JPRB, 0.80511E+00_JPRB ,&
&  0.80188E+00_JPRB, 0.80432E+00_JPRB, 0.81043E+00_JPRB, 0.80630E+00_JPRB, 0.83386E+00_JPRB, 0.84147E+00_JPRB, 0.84476E+00_JPRB ,&
&  0.84580E+00_JPRB, 0.84030E+00_JPRB, 0.82962E+00_JPRB, 0.74122E+00_JPRB, 0.80500E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.70199E+00_JPRB, 0.69941E+00_JPRB, 0.68237E+00_JPRB, 0.66302E+00_JPRB, 0.64334E+00_JPRB, 0.62439E+00_JPRB, 0.60954E+00_JPRB ,&
&  0.59761E+00_JPRB, 0.58315E+00_JPRB, 0.57063E+00_JPRB, 0.56179E+00_JPRB, 0.55273E+00_JPRB, 0.54449E+00_JPRB, 0.53530E+00_JPRB ,&
&  0.52783E+00_JPRB, 0.52279E+00_JPRB, 0.51890E+00_JPRB, 0.51443E+00_JPRB, 0.50954E+00_JPRB, 0.50394E+00_JPRB, 0.49857E+00_JPRB ,&
&  0.49380E+00_JPRB, 0.49064E+00_JPRB, 0.48496E+00_JPRB, 0.48104E+00_JPRB, 0.48439E+00_JPRB, 0.49393E+00_JPRB, 0.50299E+00_JPRB ,&
&  0.54604E+00_JPRB, 0.49847E+00_JPRB, 0.51302E+00_JPRB, 0.56618E+00_JPRB, 0.62976E+00_JPRB, 0.71198E+00_JPRB, 0.77933E+00_JPRB ,&
&  0.80820E+00_JPRB, 0.83429E+00_JPRB, 0.81860E+00_JPRB, 0.79025E+00_JPRB, 0.74342E+00_JPRB, 0.69787E+00_JPRB, 0.66353E+00_JPRB ,&
&  0.64327E+00_JPRB, 0.62778E+00_JPRB, 0.60992E+00_JPRB, 0.59036E+00_JPRB, 0.57266E+00_JPRB, 0.56059E+00_JPRB, 0.55563E+00_JPRB ,&
&  0.55684E+00_JPRB, 0.55921E+00_JPRB, 0.55989E+00_JPRB, 0.56387E+00_JPRB, 0.57419E+00_JPRB, 0.59098E+00_JPRB, 0.62745E+00_JPRB ,&
&  0.70642E+00_JPRB, 0.81233E+00_JPRB, 0.90736E+00_JPRB, 0.99742E+00_JPRB, 0.10555E+01_JPRB, 0.10722E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.94374E+00_JPRB, 0.87320E+00_JPRB, 0.82648E+00_JPRB, 0.80132E+00_JPRB, 0.78634E+00_JPRB, 0.77555E+00_JPRB, 0.76001E+00_JPRB ,&
&  0.74041E+00_JPRB, 0.71778E+00_JPRB, 0.69813E+00_JPRB, 0.67853E+00_JPRB, 0.66509E+00_JPRB, 0.66155E+00_JPRB, 0.66568E+00_JPRB ,&
&  0.67171E+00_JPRB, 0.67559E+00_JPRB, 0.67848E+00_JPRB, 0.68142E+00_JPRB, 0.68282E+00_JPRB, 0.68392E+00_JPRB, 0.68470E+00_JPRB ,&
&  0.68879E+00_JPRB, 0.69527E+00_JPRB, 0.70202E+00_JPRB, 0.71527E+00_JPRB, 0.72429E+00_JPRB, 0.72940E+00_JPRB, 0.73192E+00_JPRB ,&
&  0.73094E+00_JPRB, 0.73351E+00_JPRB, 0.63838E+00_JPRB, 0.67707E+00_JPRB, 0.69291E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.61110E+00_JPRB, 0.60977E+00_JPRB, 0.60402E+00_JPRB, 0.58997E+00_JPRB, 0.57362E+00_JPRB, 0.55922E+00_JPRB, 0.54957E+00_JPRB ,&
&  0.54306E+00_JPRB, 0.53468E+00_JPRB, 0.52756E+00_JPRB, 0.52274E+00_JPRB, 0.51697E+00_JPRB, 0.51005E+00_JPRB, 0.50032E+00_JPRB ,&
&  0.49182E+00_JPRB, 0.48510E+00_JPRB, 0.48029E+00_JPRB, 0.47747E+00_JPRB, 0.47577E+00_JPRB, 0.47396E+00_JPRB, 0.47116E+00_JPRB ,&
&  0.46849E+00_JPRB, 0.46723E+00_JPRB, 0.46577E+00_JPRB, 0.46424E+00_JPRB, 0.46464E+00_JPRB, 0.47050E+00_JPRB, 0.47832E+00_JPRB ,&
&  0.47450E+00_JPRB, 0.45179E+00_JPRB, 0.46476E+00_JPRB, 0.48947E+00_JPRB, 0.52324E+00_JPRB, 0.57996E+00_JPRB, 0.63922E+00_JPRB ,&
&  0.67471E+00_JPRB, 0.69412E+00_JPRB, 0.67877E+00_JPRB, 0.65790E+00_JPRB, 0.63646E+00_JPRB, 0.61181E+00_JPRB, 0.59172E+00_JPRB ,&
&  0.57530E+00_JPRB, 0.56400E+00_JPRB, 0.55452E+00_JPRB, 0.54152E+00_JPRB, 0.52624E+00_JPRB, 0.51388E+00_JPRB, 0.50648E+00_JPRB ,&
&  0.50399E+00_JPRB, 0.50569E+00_JPRB, 0.50819E+00_JPRB, 0.51067E+00_JPRB, 0.51540E+00_JPRB, 0.52492E+00_JPRB, 0.53965E+00_JPRB ,&
&  0.55892E+00_JPRB, 0.59580E+00_JPRB, 0.64646E+00_JPRB, 0.69666E+00_JPRB, 0.74509E+00_JPRB, 0.78068E+00_JPRB, 0.80168E+00_JPRB ,&
&  0.80301E+00_JPRB, 0.78411E+00_JPRB, 0.75684E+00_JPRB, 0.72811E+00_JPRB, 0.69438E+00_JPRB, 0.65412E+00_JPRB, 0.61140E+00_JPRB ,&
&  0.57565E+00_JPRB, 0.54860E+00_JPRB, 0.53097E+00_JPRB, 0.52255E+00_JPRB, 0.51913E+00_JPRB, 0.52114E+00_JPRB, 0.52687E+00_JPRB ,&
&  0.53503E+00_JPRB, 0.54412E+00_JPRB, 0.55116E+00_JPRB, 0.55837E+00_JPRB, 0.56457E+00_JPRB, 0.57114E+00_JPRB, 0.57810E+00_JPRB ,&
&  0.58583E+00_JPRB, 0.59313E+00_JPRB, 0.60027E+00_JPRB, 0.61104E+00_JPRB, 0.61922E+00_JPRB, 0.62675E+00_JPRB, 0.62994E+00_JPRB ,&
&  0.62580E+00_JPRB, 0.62248E+00_JPRB, 0.61816E+00_JPRB, 0.61247E+00_JPRB, 0.61313E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.54734E+00_JPRB, 0.54543E+00_JPRB, 0.54316E+00_JPRB, 0.53932E+00_JPRB, 0.53252E+00_JPRB, 0.52305E+00_JPRB, 0.51529E+00_JPRB ,&
&  0.51013E+00_JPRB, 0.50477E+00_JPRB, 0.50013E+00_JPRB, 0.49464E+00_JPRB, 0.48808E+00_JPRB, 0.47993E+00_JPRB, 0.46939E+00_JPRB ,&
&  0.45957E+00_JPRB, 0.45062E+00_JPRB, 0.44468E+00_JPRB, 0.44182E+00_JPRB, 0.44161E+00_JPRB, 0.44149E+00_JPRB, 0.43940E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.43786E+00_JPRB, 0.43803E+00_JPRB, 0.43754E+00_JPRB, 0.43755E+00_JPRB, 0.44192E+00_JPRB, 0.44670E+00_JPRB ,&
&  0.46033E+00_JPRB, 0.43974E+00_JPRB, 0.45423E+00_JPRB, 0.45991E+00_JPRB, 0.47352E+00_JPRB, 0.50458E+00_JPRB, 0.54633E+00_JPRB ,&
&  0.58002E+00_JPRB, 0.58205E+00_JPRB, 0.56680E+00_JPRB, 0.54965E+00_JPRB, 0.54108E+00_JPRB, 0.53116E+00_JPRB, 0.51980E+00_JPRB ,&
&  0.50898E+00_JPRB, 0.50123E+00_JPRB, 0.49466E+00_JPRB, 0.48833E+00_JPRB, 0.48004E+00_JPRB, 0.46978E+00_JPRB, 0.46023E+00_JPRB ,&
&  0.45381E+00_JPRB, 0.45179E+00_JPRB, 0.45338E+00_JPRB, 0.45670E+00_JPRB, 0.46050E+00_JPRB, 0.46703E+00_JPRB, 0.47575E+00_JPRB ,&
&  0.48567E+00_JPRB, 0.49228E+00_JPRB, 0.50033E+00_JPRB, 0.51738E+00_JPRB, 0.53797E+00_JPRB, 0.55455E+00_JPRB, 0.56551E+00_JPRB ,&
&  0.57081E+00_JPRB, 0.56722E+00_JPRB, 0.55726E+00_JPRB, 0.54260E+00_JPRB, 0.52542E+00_JPRB, 0.50761E+00_JPRB, 0.49172E+00_JPRB ,&
&  0.47881E+00_JPRB, 0.46813E+00_JPRB, 0.45927E+00_JPRB, 0.45319E+00_JPRB, 0.45020E+00_JPRB, 0.44953E+00_JPRB, 0.45103E+00_JPRB ,&
&  0.45586E+00_JPRB, 0.46250E+00_JPRB, 0.46869E+00_JPRB, 0.47466E+00_JPRB, 0.47992E+00_JPRB, 0.48433E+00_JPRB, 0.48979E+00_JPRB ,&
&  0.49687E+00_JPRB, 0.50219E+00_JPRB, 0.50641E+00_JPRB, 0.51202E+00_JPRB, 0.51877E+00_JPRB, 0.52626E+00_JPRB, 0.53319E+00_JPRB ,&
&  0.53783E+00_JPRB, 0.54008E+00_JPRB, 0.54318E+00_JPRB, 0.54452E+00_JPRB, 0.54779E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.50300E+00_JPRB, 0.50215E+00_JPRB, 0.50018E+00_JPRB, 0.49873E+00_JPRB, 0.49598E+00_JPRB, 0.49148E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.48168E+00_JPRB, 0.47760E+00_JPRB, 0.47320E+00_JPRB, 0.46735E+00_JPRB, 0.45945E+00_JPRB, 0.45156E+00_JPRB, 0.44258E+00_JPRB ,&
&  0.43404E+00_JPRB, 0.42659E+00_JPRB, 0.42128E+00_JPRB, 0.41802E+00_JPRB, 0.41542E+00_JPRB, 0.41307E+00_JPRB, 0.41036E+00_JPRB ,&
&  0.40865E+00_JPRB, 0.40791E+00_JPRB, 0.40809E+00_JPRB, 0.40882E+00_JPRB, 0.40953E+00_JPRB, 0.41271E+00_JPRB, 0.41497E+00_JPRB ,&
&  0.41995E+00_JPRB, 0.43382E+00_JPRB, 0.44387E+00_JPRB, 0.44772E+00_JPRB, 0.45412E+00_JPRB, 0.46489E+00_JPRB, 0.47955E+00_JPRB ,&
&  0.49315E+00_JPRB, 0.49904E+00_JPRB, 0.49135E+00_JPRB, 0.47900E+00_JPRB, 0.47162E+00_JPRB, 0.46830E+00_JPRB, 0.46099E+00_JPRB ,&
&  0.45346E+00_JPRB, 0.44763E+00_JPRB, 0.44302E+00_JPRB, 0.43955E+00_JPRB, 0.43616E+00_JPRB, 0.43134E+00_JPRB, 0.42595E+00_JPRB ,&
&  0.41934E+00_JPRB, 0.41422E+00_JPRB, 0.41048E+00_JPRB, 0.40917E+00_JPRB, 0.40955E+00_JPRB, 0.41141E+00_JPRB, 0.41467E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.42585E+00_JPRB, 0.42866E+00_JPRB, 0.43009E+00_JPRB, 0.43254E+00_JPRB, 0.43696E+00_JPRB, 0.44079E+00_JPRB ,&
&  0.44214E+00_JPRB, 0.44124E+00_JPRB, 0.43804E+00_JPRB, 0.43292E+00_JPRB, 0.42653E+00_JPRB, 0.42079E+00_JPRB, 0.41599E+00_JPRB ,&
&  0.41164E+00_JPRB, 0.40736E+00_JPRB, 0.40328E+00_JPRB, 0.40099E+00_JPRB, 0.39979E+00_JPRB, 0.39953E+00_JPRB, 0.40093E+00_JPRB ,&
&  0.40403E+00_JPRB, 0.40794E+00_JPRB, 0.41256E+00_JPRB, 0.41716E+00_JPRB, 0.42155E+00_JPRB, 0.42510E+00_JPRB, 0.42912E+00_JPRB ,&
&  0.43522E+00_JPRB, 0.44196E+00_JPRB, 0.44856E+00_JPRB, 0.45448E+00_JPRB, 0.46097E+00_JPRB, 0.46872E+00_JPRB, 0.47711E+00_JPRB ,&
&  0.48283E+00_JPRB, 0.48663E+00_JPRB, 0.49136E+00_JPRB, 0.49664E+00_JPRB, 0.50134E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.45362E+00_JPRB, 0.45587E+00_JPRB, 0.45703E+00_JPRB, 0.45848E+00_JPRB, 0.45883E+00_JPRB, 0.45741E+00_JPRB, 0.45373E+00_JPRB ,&
&  0.44936E+00_JPRB, 0.44436E+00_JPRB, 0.43928E+00_JPRB, 0.43287E+00_JPRB, 0.42606E+00_JPRB, 0.42028E+00_JPRB, 0.41394E+00_JPRB ,&
&  0.40744E+00_JPRB, 0.40140E+00_JPRB, 0.39691E+00_JPRB, 0.39331E+00_JPRB, 0.38979E+00_JPRB, 0.38623E+00_JPRB, 0.38188E+00_JPRB ,&
&  0.37807E+00_JPRB, 0.37469E+00_JPRB, 0.37491E+00_JPRB, 0.37778E+00_JPRB, 0.38022E+00_JPRB, 0.38193E+00_JPRB, 0.38355E+00_JPRB ,&
&  0.38440E+00_JPRB, 0.40175E+00_JPRB, 0.39688E+00_JPRB, 0.41049E+00_JPRB, 0.41511E+00_JPRB, 0.41583E+00_JPRB, 0.41702E+00_JPRB ,&
&  0.42114E+00_JPRB, 0.42632E+00_JPRB, 0.43000E+00_JPRB, 0.42498E+00_JPRB, 0.41743E+00_JPRB, 0.41163E+00_JPRB, 0.40580E+00_JPRB ,&
&  0.39814E+00_JPRB, 0.39472E+00_JPRB, 0.39423E+00_JPRB, 0.39427E+00_JPRB, 0.39429E+00_JPRB, 0.39375E+00_JPRB, 0.39228E+00_JPRB ,&
&  0.38986E+00_JPRB, 0.38636E+00_JPRB, 0.38232E+00_JPRB, 0.37843E+00_JPRB, 0.37593E+00_JPRB, 0.37348E+00_JPRB, 0.37164E+00_JPRB ,&
&  0.37149E+00_JPRB, 0.37327E+00_JPRB, 0.37518E+00_JPRB, 0.37591E+00_JPRB, 0.37529E+00_JPRB, 0.37471E+00_JPRB, 0.37467E+00_JPRB ,&
&  0.37434E+00_JPRB, 0.37354E+00_JPRB, 0.37217E+00_JPRB, 0.37006E+00_JPRB, 0.36764E+00_JPRB, 0.36591E+00_JPRB, 0.36487E+00_JPRB ,&
&  0.36358E+00_JPRB, 0.36120E+00_JPRB, 0.35816E+00_JPRB, 0.35751E+00_JPRB, 0.35762E+00_JPRB, 0.35810E+00_JPRB, 0.35883E+00_JPRB ,&
&  0.36041E+00_JPRB, 0.36331E+00_JPRB, 0.36675E+00_JPRB, 0.37033E+00_JPRB, 0.37401E+00_JPRB, 0.37882E+00_JPRB, 0.38412E+00_JPRB ,&
&  0.38979E+00_JPRB, 0.39577E+00_JPRB, 0.40169E+00_JPRB, 0.40737E+00_JPRB, 0.41226E+00_JPRB, 0.41771E+00_JPRB, 0.42456E+00_JPRB ,&
&  0.43034E+00_JPRB, 0.43488E+00_JPRB, 0.43881E+00_JPRB, 0.44388E+00_JPRB, 0.44943E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.41313E+00_JPRB, 0.41864E+00_JPRB, 0.42201E+00_JPRB, 0.42399E+00_JPRB, 0.42490E+00_JPRB, 0.42362E+00_JPRB, 0.42069E+00_JPRB ,&
&  0.41586E+00_JPRB, 0.41078E+00_JPRB, 0.40539E+00_JPRB, 0.39961E+00_JPRB, 0.39437E+00_JPRB, 0.38976E+00_JPRB, 0.38554E+00_JPRB ,&
&  0.38119E+00_JPRB, 0.37711E+00_JPRB, 0.37374E+00_JPRB, 0.37039E+00_JPRB, 0.36649E+00_JPRB, 0.36159E+00_JPRB, 0.35558E+00_JPRB ,&
&  0.35173E+00_JPRB, 0.34869E+00_JPRB, 0.34933E+00_JPRB, 0.35252E+00_JPRB, 0.35514E+00_JPRB, 0.35661E+00_JPRB, 0.35804E+00_JPRB ,&
&  0.36009E+00_JPRB, 0.36486E+00_JPRB, 0.36879E+00_JPRB, 0.37387E+00_JPRB, 0.37752E+00_JPRB, 0.37737E+00_JPRB, 0.37499E+00_JPRB ,&
&  0.37356E+00_JPRB, 0.37469E+00_JPRB, 0.37683E+00_JPRB, 0.37805E+00_JPRB, 0.37662E+00_JPRB, 0.37301E+00_JPRB, 0.36734E+00_JPRB ,&
&  0.35845E+00_JPRB, 0.35201E+00_JPRB, 0.35063E+00_JPRB, 0.35208E+00_JPRB, 0.35439E+00_JPRB, 0.35613E+00_JPRB, 0.35713E+00_JPRB ,&
&  0.35738E+00_JPRB, 0.35652E+00_JPRB, 0.35499E+00_JPRB, 0.35307E+00_JPRB, 0.35067E+00_JPRB, 0.34819E+00_JPRB, 0.34669E+00_JPRB ,&
&  0.34526E+00_JPRB, 0.34426E+00_JPRB, 0.34398E+00_JPRB, 0.34389E+00_JPRB, 0.34277E+00_JPRB, 0.34075E+00_JPRB, 0.33936E+00_JPRB ,&
&  0.33764E+00_JPRB, 0.33655E+00_JPRB, 0.33506E+00_JPRB, 0.33347E+00_JPRB, 0.33154E+00_JPRB, 0.32971E+00_JPRB, 0.32881E+00_JPRB ,&
&  0.32863E+00_JPRB, 0.32726E+00_JPRB, 0.32572E+00_JPRB, 0.32576E+00_JPRB, 0.32666E+00_JPRB, 0.32786E+00_JPRB, 0.32872E+00_JPRB ,&
&  0.32955E+00_JPRB, 0.33067E+00_JPRB, 0.33323E+00_JPRB, 0.33632E+00_JPRB, 0.33996E+00_JPRB, 0.34430E+00_JPRB, 0.34912E+00_JPRB ,&
&  0.35455E+00_JPRB, 0.36059E+00_JPRB, 0.36716E+00_JPRB, 0.37336E+00_JPRB, 0.37875E+00_JPRB, 0.38258E+00_JPRB, 0.38735E+00_JPRB ,&
&  0.39290E+00_JPRB, 0.39667E+00_JPRB, 0.39978E+00_JPRB, 0.40346E+00_JPRB, 0.40764E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.37418E+00_JPRB, 0.37987E+00_JPRB, 0.38422E+00_JPRB, 0.38687E+00_JPRB, 0.38734E+00_JPRB, 0.38621E+00_JPRB, 0.38328E+00_JPRB ,&
&  0.37874E+00_JPRB, 0.37398E+00_JPRB, 0.36936E+00_JPRB, 0.36563E+00_JPRB, 0.36314E+00_JPRB, 0.36097E+00_JPRB, 0.35893E+00_JPRB ,&
&  0.35623E+00_JPRB, 0.35317E+00_JPRB, 0.34967E+00_JPRB, 0.34661E+00_JPRB, 0.34283E+00_JPRB, 0.33776E+00_JPRB, 0.33381E+00_JPRB ,&
&  0.33095E+00_JPRB, 0.32839E+00_JPRB, 0.32750E+00_JPRB, 0.32883E+00_JPRB, 0.33140E+00_JPRB, 0.33389E+00_JPRB, 0.33672E+00_JPRB ,&
&  0.33884E+00_JPRB, 0.34090E+00_JPRB, 0.34283E+00_JPRB, 0.34309E+00_JPRB, 0.34301E+00_JPRB, 0.34270E+00_JPRB, 0.34205E+00_JPRB ,&
&  0.34136E+00_JPRB, 0.34070E+00_JPRB, 0.33955E+00_JPRB, 0.33936E+00_JPRB, 0.33905E+00_JPRB, 0.33750E+00_JPRB, 0.33480E+00_JPRB ,&
&  0.33049E+00_JPRB, 0.32473E+00_JPRB, 0.32037E+00_JPRB, 0.31843E+00_JPRB, 0.31801E+00_JPRB, 0.31850E+00_JPRB, 0.31952E+00_JPRB ,&
&  0.32074E+00_JPRB, 0.32249E+00_JPRB, 0.32416E+00_JPRB, 0.32500E+00_JPRB, 0.32483E+00_JPRB, 0.32459E+00_JPRB, 0.32317E+00_JPRB ,&
&  0.32181E+00_JPRB, 0.32089E+00_JPRB, 0.32032E+00_JPRB, 0.31971E+00_JPRB, 0.31738E+00_JPRB, 0.31438E+00_JPRB, 0.31341E+00_JPRB ,&
&  0.31147E+00_JPRB, 0.31019E+00_JPRB, 0.30842E+00_JPRB, 0.30724E+00_JPRB, 0.30592E+00_JPRB, 0.30421E+00_JPRB, 0.30249E+00_JPRB ,&
&  0.30125E+00_JPRB, 0.30059E+00_JPRB, 0.30030E+00_JPRB, 0.29964E+00_JPRB, 0.30027E+00_JPRB, 0.30136E+00_JPRB, 0.30304E+00_JPRB ,&
&  0.30431E+00_JPRB, 0.30544E+00_JPRB, 0.30722E+00_JPRB, 0.30936E+00_JPRB, 0.31254E+00_JPRB, 0.31625E+00_JPRB, 0.32047E+00_JPRB ,&
&  0.32415E+00_JPRB, 0.32888E+00_JPRB, 0.33412E+00_JPRB, 0.33908E+00_JPRB, 0.34308E+00_JPRB, 0.34604E+00_JPRB, 0.35011E+00_JPRB ,&
&  0.35631E+00_JPRB, 0.36113E+00_JPRB, 0.36348E+00_JPRB, 0.36634E+00_JPRB, 0.36935E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.34386E+00_JPRB, 0.34604E+00_JPRB, 0.34820E+00_JPRB, 0.35046E+00_JPRB, 0.35209E+00_JPRB, 0.35277E+00_JPRB, 0.35141E+00_JPRB ,&
&  0.34856E+00_JPRB, 0.34512E+00_JPRB, 0.34160E+00_JPRB, 0.33860E+00_JPRB, 0.33675E+00_JPRB, 0.33547E+00_JPRB, 0.33445E+00_JPRB ,&
&  0.33348E+00_JPRB, 0.33247E+00_JPRB, 0.33003E+00_JPRB, 0.32665E+00_JPRB, 0.32220E+00_JPRB, 0.31828E+00_JPRB, 0.31675E+00_JPRB ,&
&  0.31471E+00_JPRB, 0.31326E+00_JPRB, 0.31211E+00_JPRB, 0.31094E+00_JPRB, 0.31037E+00_JPRB, 0.31124E+00_JPRB, 0.31323E+00_JPRB ,&
&  0.31529E+00_JPRB, 0.31719E+00_JPRB, 0.31883E+00_JPRB, 0.31956E+00_JPRB, 0.31889E+00_JPRB, 0.31794E+00_JPRB, 0.31672E+00_JPRB ,&
&  0.31514E+00_JPRB, 0.31346E+00_JPRB, 0.31190E+00_JPRB, 0.30987E+00_JPRB, 0.30700E+00_JPRB, 0.30349E+00_JPRB, 0.30097E+00_JPRB ,&
&  0.29910E+00_JPRB, 0.29734E+00_JPRB, 0.29555E+00_JPRB, 0.29463E+00_JPRB, 0.29391E+00_JPRB, 0.29360E+00_JPRB, 0.29367E+00_JPRB ,&
&  0.29419E+00_JPRB, 0.29559E+00_JPRB, 0.29760E+00_JPRB, 0.29986E+00_JPRB, 0.30072E+00_JPRB, 0.30101E+00_JPRB, 0.30120E+00_JPRB ,&
&  0.30030E+00_JPRB, 0.29869E+00_JPRB, 0.29656E+00_JPRB, 0.29413E+00_JPRB, 0.29237E+00_JPRB, 0.29104E+00_JPRB, 0.28958E+00_JPRB ,&
&  0.28725E+00_JPRB, 0.28579E+00_JPRB, 0.28509E+00_JPRB, 0.28494E+00_JPRB, 0.28509E+00_JPRB, 0.28503E+00_JPRB, 0.28411E+00_JPRB ,&
&  0.28302E+00_JPRB, 0.28216E+00_JPRB, 0.28124E+00_JPRB, 0.28030E+00_JPRB, 0.27857E+00_JPRB, 0.27935E+00_JPRB, 0.28071E+00_JPRB ,&
&  0.28115E+00_JPRB, 0.28110E+00_JPRB, 0.28134E+00_JPRB, 0.28298E+00_JPRB, 0.28491E+00_JPRB, 0.28733E+00_JPRB, 0.29035E+00_JPRB ,&
&  0.29302E+00_JPRB, 0.29660E+00_JPRB, 0.30157E+00_JPRB, 0.30626E+00_JPRB, 0.30970E+00_JPRB, 0.31150E+00_JPRB, 0.31513E+00_JPRB ,&
&  0.32236E+00_JPRB, 0.32955E+00_JPRB, 0.33379E+00_JPRB, 0.33790E+00_JPRB, 0.34184E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.31655E+00_JPRB, 0.31770E+00_JPRB, 0.31866E+00_JPRB, 0.31985E+00_JPRB, 0.32148E+00_JPRB, 0.32257E+00_JPRB, 0.32344E+00_JPRB ,&
&  0.32281E+00_JPRB, 0.32085E+00_JPRB, 0.31848E+00_JPRB, 0.31599E+00_JPRB, 0.31479E+00_JPRB, 0.31444E+00_JPRB, 0.31436E+00_JPRB ,&
&  0.31367E+00_JPRB, 0.31327E+00_JPRB, 0.31114E+00_JPRB, 0.30763E+00_JPRB, 0.30339E+00_JPRB, 0.30033E+00_JPRB, 0.29931E+00_JPRB ,&
&  0.29856E+00_JPRB, 0.29740E+00_JPRB, 0.29662E+00_JPRB, 0.29548E+00_JPRB, 0.29242E+00_JPRB, 0.29091E+00_JPRB, 0.29127E+00_JPRB ,&
&  0.29209E+00_JPRB, 0.29368E+00_JPRB, 0.29373E+00_JPRB, 0.29164E+00_JPRB, 0.29384E+00_JPRB, 0.29371E+00_JPRB, 0.29177E+00_JPRB ,&
&  0.28924E+00_JPRB, 0.28662E+00_JPRB, 0.28423E+00_JPRB, 0.28173E+00_JPRB, 0.27910E+00_JPRB, 0.27707E+00_JPRB, 0.27575E+00_JPRB ,&
&  0.27543E+00_JPRB, 0.27582E+00_JPRB, 0.27653E+00_JPRB, 0.27761E+00_JPRB, 0.27829E+00_JPRB, 0.27867E+00_JPRB, 0.27878E+00_JPRB ,&
&  0.27898E+00_JPRB, 0.27929E+00_JPRB, 0.27978E+00_JPRB, 0.28008E+00_JPRB, 0.28039E+00_JPRB, 0.28055E+00_JPRB, 0.28016E+00_JPRB ,&
&  0.27867E+00_JPRB, 0.27636E+00_JPRB, 0.27341E+00_JPRB, 0.27209E+00_JPRB, 0.27181E+00_JPRB, 0.26970E+00_JPRB, 0.26679E+00_JPRB ,&
&  0.26428E+00_JPRB, 0.26406E+00_JPRB, 0.26576E+00_JPRB, 0.26729E+00_JPRB, 0.26851E+00_JPRB, 0.26896E+00_JPRB, 0.26868E+00_JPRB ,&
&  0.26760E+00_JPRB, 0.26485E+00_JPRB, 0.26191E+00_JPRB, 0.25951E+00_JPRB, 0.25797E+00_JPRB, 0.25791E+00_JPRB, 0.25904E+00_JPRB ,&
&  0.25932E+00_JPRB, 0.25962E+00_JPRB, 0.26099E+00_JPRB, 0.26233E+00_JPRB, 0.26345E+00_JPRB, 0.26463E+00_JPRB, 0.26635E+00_JPRB ,&
&  0.26864E+00_JPRB, 0.27256E+00_JPRB, 0.27743E+00_JPRB, 0.28306E+00_JPRB, 0.28699E+00_JPRB, 0.28646E+00_JPRB, 0.28802E+00_JPRB ,&
&  0.29407E+00_JPRB, 0.30080E+00_JPRB, 0.30597E+00_JPRB, 0.30990E+00_JPRB, 0.31361E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.29455E+00_JPRB, 0.29597E+00_JPRB, 0.29699E+00_JPRB, 0.29772E+00_JPRB, 0.29911E+00_JPRB, 0.30028E+00_JPRB, 0.30185E+00_JPRB ,&
&  0.30295E+00_JPRB, 0.30158E+00_JPRB, 0.29979E+00_JPRB, 0.29821E+00_JPRB, 0.29745E+00_JPRB, 0.29644E+00_JPRB, 0.29550E+00_JPRB ,&
&  0.29586E+00_JPRB, 0.29494E+00_JPRB, 0.29191E+00_JPRB, 0.28935E+00_JPRB, 0.28539E+00_JPRB, 0.28284E+00_JPRB, 0.28243E+00_JPRB ,&
&  0.28411E+00_JPRB, 0.28349E+00_JPRB, 0.28213E+00_JPRB, 0.28041E+00_JPRB, 0.27840E+00_JPRB, 0.27723E+00_JPRB, 0.27595E+00_JPRB ,&
&  0.27511E+00_JPRB, 0.27354E+00_JPRB, 0.26897E+00_JPRB, 0.26932E+00_JPRB, 0.26907E+00_JPRB, 0.26934E+00_JPRB, 0.26893E+00_JPRB ,&
&  0.26836E+00_JPRB, 0.26733E+00_JPRB, 0.26544E+00_JPRB, 0.26328E+00_JPRB, 0.26147E+00_JPRB, 0.26051E+00_JPRB, 0.26054E+00_JPRB ,&
&  0.26124E+00_JPRB, 0.26278E+00_JPRB, 0.26498E+00_JPRB, 0.26733E+00_JPRB, 0.26834E+00_JPRB, 0.26914E+00_JPRB, 0.27041E+00_JPRB ,&
&  0.27100E+00_JPRB, 0.27109E+00_JPRB, 0.27095E+00_JPRB, 0.27053E+00_JPRB, 0.26969E+00_JPRB, 0.26825E+00_JPRB, 0.26708E+00_JPRB ,&
&  0.26467E+00_JPRB, 0.26159E+00_JPRB, 0.25937E+00_JPRB, 0.25674E+00_JPRB, 0.24971E+00_JPRB, 0.23944E+00_JPRB, 0.23377E+00_JPRB ,&
&  0.23455E+00_JPRB, 0.23958E+00_JPRB, 0.24443E+00_JPRB, 0.24838E+00_JPRB, 0.25518E+00_JPRB, 0.25527E+00_JPRB, 0.25053E+00_JPRB ,&
&  0.23989E+00_JPRB, 0.23632E+00_JPRB, 0.23328E+00_JPRB, 0.22824E+00_JPRB, 0.22530E+00_JPRB, 0.22513E+00_JPRB, 0.22538E+00_JPRB ,&
&  0.22768E+00_JPRB, 0.23042E+00_JPRB, 0.23370E+00_JPRB, 0.23571E+00_JPRB, 0.23609E+00_JPRB, 0.23518E+00_JPRB, 0.23377E+00_JPRB ,&
&  0.23424E+00_JPRB, 0.23927E+00_JPRB, 0.24860E+00_JPRB, 0.25687E+00_JPRB, 0.26222E+00_JPRB, 0.26461E+00_JPRB, 0.26709E+00_JPRB ,&
&  0.27205E+00_JPRB, 0.27832E+00_JPRB, 0.28430E+00_JPRB, 0.28846E+00_JPRB, 0.29185E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.27240E+00_JPRB, 0.27379E+00_JPRB, 0.27504E+00_JPRB, 0.27615E+00_JPRB, 0.27728E+00_JPRB, 0.27858E+00_JPRB, 0.27976E+00_JPRB ,&
&  0.28121E+00_JPRB, 0.28213E+00_JPRB, 0.28227E+00_JPRB, 0.28104E+00_JPRB, 0.28011E+00_JPRB, 0.27948E+00_JPRB, 0.27943E+00_JPRB ,&
&  0.27694E+00_JPRB, 0.27369E+00_JPRB, 0.27215E+00_JPRB, 0.27164E+00_JPRB, 0.27073E+00_JPRB, 0.27038E+00_JPRB, 0.27129E+00_JPRB ,&
&  0.27221E+00_JPRB, 0.27133E+00_JPRB, 0.27056E+00_JPRB, 0.26871E+00_JPRB, 0.26726E+00_JPRB, 0.26615E+00_JPRB, 0.26475E+00_JPRB ,&
&  0.26003E+00_JPRB, 0.24961E+00_JPRB, 0.24368E+00_JPRB, 0.24849E+00_JPRB, 0.25441E+00_JPRB, 0.25345E+00_JPRB, 0.25375E+00_JPRB ,&
&  0.25355E+00_JPRB, 0.25227E+00_JPRB, 0.25080E+00_JPRB, 0.24947E+00_JPRB, 0.24911E+00_JPRB, 0.24977E+00_JPRB, 0.25120E+00_JPRB ,&
&  0.25300E+00_JPRB, 0.25630E+00_JPRB, 0.25950E+00_JPRB, 0.26068E+00_JPRB, 0.26199E+00_JPRB, 0.26273E+00_JPRB, 0.26289E+00_JPRB ,&
&  0.26194E+00_JPRB, 0.26023E+00_JPRB, 0.25863E+00_JPRB, 0.25703E+00_JPRB, 0.25544E+00_JPRB, 0.25340E+00_JPRB, 0.24808E+00_JPRB ,&
&  0.24283E+00_JPRB, 0.23696E+00_JPRB, 0.22995E+00_JPRB, 0.22290E+00_JPRB, 0.21699E+00_JPRB, 0.21270E+00_JPRB, 0.20884E+00_JPRB ,&
&  0.20869E+00_JPRB, 0.21381E+00_JPRB, 0.22322E+00_JPRB, 0.23355E+00_JPRB, 0.23577E+00_JPRB, 0.22739E+00_JPRB, 0.22170E+00_JPRB ,&
&  0.21545E+00_JPRB, 0.21017E+00_JPRB, 0.20700E+00_JPRB, 0.20529E+00_JPRB, 0.20400E+00_JPRB, 0.20386E+00_JPRB, 0.20454E+00_JPRB ,&
&  0.20627E+00_JPRB, 0.20807E+00_JPRB, 0.20970E+00_JPRB, 0.21087E+00_JPRB, 0.21139E+00_JPRB, 0.21129E+00_JPRB, 0.21088E+00_JPRB ,&
&  0.21180E+00_JPRB, 0.21492E+00_JPRB, 0.21979E+00_JPRB, 0.22532E+00_JPRB, 0.23025E+00_JPRB, 0.23513E+00_JPRB, 0.24150E+00_JPRB ,&
&  0.24940E+00_JPRB, 0.25481E+00_JPRB, 0.26042E+00_JPRB, 0.26652E+00_JPRB, 0.27020E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.25431E+00_JPRB, 0.25710E+00_JPRB, 0.25791E+00_JPRB, 0.25935E+00_JPRB, 0.25958E+00_JPRB, 0.26067E+00_JPRB, 0.26129E+00_JPRB ,&
&  0.26347E+00_JPRB, 0.26553E+00_JPRB, 0.26594E+00_JPRB, 0.26390E+00_JPRB, 0.26220E+00_JPRB, 0.25819E+00_JPRB, 0.25638E+00_JPRB ,&
&  0.25551E+00_JPRB, 0.25631E+00_JPRB, 0.25638E+00_JPRB, 0.25634E+00_JPRB, 0.25617E+00_JPRB, 0.25604E+00_JPRB, 0.25668E+00_JPRB ,&
&  0.25431E+00_JPRB, 0.25258E+00_JPRB, 0.25453E+00_JPRB, 0.25316E+00_JPRB, 0.25204E+00_JPRB, 0.25022E+00_JPRB, 0.24919E+00_JPRB ,&
&  0.24439E+00_JPRB, 0.23797E+00_JPRB, 0.23197E+00_JPRB, 0.23625E+00_JPRB, 0.24345E+00_JPRB, 0.24008E+00_JPRB, 0.23963E+00_JPRB ,&
&  0.23951E+00_JPRB, 0.23869E+00_JPRB, 0.23872E+00_JPRB, 0.23864E+00_JPRB, 0.23999E+00_JPRB, 0.24133E+00_JPRB, 0.24404E+00_JPRB ,&
&  0.24763E+00_JPRB, 0.25243E+00_JPRB, 0.25245E+00_JPRB, 0.24507E+00_JPRB, 0.24227E+00_JPRB, 0.24148E+00_JPRB, 0.23768E+00_JPRB ,&
&  0.23296E+00_JPRB, 0.23183E+00_JPRB, 0.23030E+00_JPRB, 0.22625E+00_JPRB, 0.22460E+00_JPRB, 0.22392E+00_JPRB, 0.22049E+00_JPRB ,&
&  0.21628E+00_JPRB, 0.21316E+00_JPRB, 0.20914E+00_JPRB, 0.20513E+00_JPRB, 0.20244E+00_JPRB, 0.20125E+00_JPRB, 0.20155E+00_JPRB ,&
&  0.20399E+00_JPRB, 0.20612E+00_JPRB, 0.21272E+00_JPRB, 0.21653E+00_JPRB, 0.21240E+00_JPRB, 0.20998E+00_JPRB, 0.20657E+00_JPRB ,&
&  0.19994E+00_JPRB, 0.19709E+00_JPRB, 0.19473E+00_JPRB, 0.19295E+00_JPRB, 0.19279E+00_JPRB, 0.19357E+00_JPRB, 0.19399E+00_JPRB ,&
&  0.19480E+00_JPRB, 0.19619E+00_JPRB, 0.19686E+00_JPRB, 0.19725E+00_JPRB, 0.19738E+00_JPRB, 0.19782E+00_JPRB, 0.19872E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.20217E+00_JPRB, 0.20518E+00_JPRB, 0.20827E+00_JPRB, 0.21069E+00_JPRB, 0.21273E+00_JPRB, 0.21711E+00_JPRB ,&
&  0.22173E+00_JPRB, 0.23251E+00_JPRB, 0.24734E+00_JPRB, 0.25641E+00_JPRB, 0.25190E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.25079E+00_JPRB, 0.25257E+00_JPRB, 0.25219E+00_JPRB, 0.25233E+00_JPRB, 0.25171E+00_JPRB, 0.25230E+00_JPRB, 0.25090E+00_JPRB ,&
&  0.25021E+00_JPRB, 0.24825E+00_JPRB, 0.24654E+00_JPRB, 0.24269E+00_JPRB, 0.24058E+00_JPRB, 0.23416E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23005E+00_JPRB, 0.23061E+00_JPRB, 0.23063E+00_JPRB, 0.23106E+00_JPRB, 0.23164E+00_JPRB, 0.23313E+00_JPRB, 0.23263E+00_JPRB ,&
&  0.23117E+00_JPRB, 0.22908E+00_JPRB, 0.22657E+00_JPRB, 0.22430E+00_JPRB, 0.22468E+00_JPRB, 0.22413E+00_JPRB, 0.22287E+00_JPRB ,&
&  0.22188E+00_JPRB, 0.22259E+00_JPRB, 0.22303E+00_JPRB, 0.22452E+00_JPRB, 0.22950E+00_JPRB, 0.22729E+00_JPRB, 0.22644E+00_JPRB ,&
&  0.22687E+00_JPRB, 0.22747E+00_JPRB, 0.22881E+00_JPRB, 0.23003E+00_JPRB, 0.23338E+00_JPRB, 0.23302E+00_JPRB, 0.23670E+00_JPRB ,&
&  0.23542E+00_JPRB, 0.23470E+00_JPRB, 0.23308E+00_JPRB, 0.22987E+00_JPRB, 0.22488E+00_JPRB, 0.22172E+00_JPRB, 0.21876E+00_JPRB ,&
&  0.21539E+00_JPRB, 0.21257E+00_JPRB, 0.20888E+00_JPRB, 0.20760E+00_JPRB, 0.20556E+00_JPRB, 0.20375E+00_JPRB, 0.20213E+00_JPRB ,&
&  0.20103E+00_JPRB, 0.19945E+00_JPRB, 0.19816E+00_JPRB, 0.19729E+00_JPRB, 0.19711E+00_JPRB, 0.19730E+00_JPRB, 0.19863E+00_JPRB ,&
&  0.20103E+00_JPRB, 0.20285E+00_JPRB, 0.20432E+00_JPRB, 0.20596E+00_JPRB, 0.20430E+00_JPRB, 0.20129E+00_JPRB, 0.19841E+00_JPRB ,&
&  0.19398E+00_JPRB, 0.19167E+00_JPRB, 0.18938E+00_JPRB, 0.18799E+00_JPRB, 0.18757E+00_JPRB, 0.18737E+00_JPRB, 0.18753E+00_JPRB ,&
&  0.18771E+00_JPRB, 0.18824E+00_JPRB, 0.18863E+00_JPRB, 0.18922E+00_JPRB, 0.19003E+00_JPRB, 0.19082E+00_JPRB, 0.19203E+00_JPRB ,&
&  0.19384E+00_JPRB, 0.19611E+00_JPRB, 0.19892E+00_JPRB, 0.20199E+00_JPRB, 0.20605E+00_JPRB, 0.21122E+00_JPRB, 0.21860E+00_JPRB ,&
&  0.22552E+00_JPRB, 0.23548E+00_JPRB, 0.23616E+00_JPRB, 0.24505E+00_JPRB, 0.24963E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.24800E+00_JPRB, 0.25033E+00_JPRB, 0.25090E+00_JPRB, 0.25152E+00_JPRB, 0.25036E+00_JPRB, 0.24720E+00_JPRB, 0.24384E+00_JPRB ,&
&  0.23716E+00_JPRB, 0.23196E+00_JPRB, 0.22571E+00_JPRB, 0.22002E+00_JPRB, 0.21454E+00_JPRB, 0.21119E+00_JPRB, 0.20905E+00_JPRB ,&
&  0.20858E+00_JPRB, 0.20977E+00_JPRB, 0.21056E+00_JPRB, 0.21106E+00_JPRB, 0.21111E+00_JPRB, 0.21192E+00_JPRB, 0.21157E+00_JPRB ,&
&  0.21083E+00_JPRB, 0.20976E+00_JPRB, 0.20899E+00_JPRB, 0.20968E+00_JPRB, 0.21183E+00_JPRB, 0.21400E+00_JPRB, 0.21583E+00_JPRB ,&
&  0.21724E+00_JPRB, 0.21613E+00_JPRB, 0.21607E+00_JPRB, 0.21522E+00_JPRB, 0.21512E+00_JPRB, 0.21501E+00_JPRB, 0.21553E+00_JPRB ,&
&  0.21729E+00_JPRB, 0.21866E+00_JPRB, 0.22154E+00_JPRB, 0.22294E+00_JPRB, 0.22205E+00_JPRB, 0.22560E+00_JPRB, 0.22703E+00_JPRB ,&
&  0.22695E+00_JPRB, 0.22348E+00_JPRB, 0.21953E+00_JPRB, 0.21625E+00_JPRB, 0.21265E+00_JPRB, 0.20961E+00_JPRB, 0.20713E+00_JPRB ,&
&  0.20489E+00_JPRB, 0.20376E+00_JPRB, 0.20133E+00_JPRB, 0.19928E+00_JPRB, 0.19795E+00_JPRB, 0.19656E+00_JPRB, 0.19571E+00_JPRB ,&
&  0.19524E+00_JPRB, 0.19430E+00_JPRB, 0.19419E+00_JPRB, 0.19365E+00_JPRB, 0.19374E+00_JPRB, 0.19384E+00_JPRB, 0.19418E+00_JPRB ,&
&  0.19509E+00_JPRB, 0.19632E+00_JPRB, 0.19623E+00_JPRB, 0.19615E+00_JPRB, 0.19431E+00_JPRB, 0.19231E+00_JPRB, 0.19065E+00_JPRB ,&
&  0.18833E+00_JPRB, 0.18669E+00_JPRB, 0.18568E+00_JPRB, 0.18464E+00_JPRB, 0.18442E+00_JPRB, 0.18439E+00_JPRB, 0.18425E+00_JPRB ,&
&  0.18444E+00_JPRB, 0.18480E+00_JPRB, 0.18541E+00_JPRB, 0.18673E+00_JPRB, 0.18789E+00_JPRB, 0.18955E+00_JPRB, 0.19137E+00_JPRB ,&
&  0.19276E+00_JPRB, 0.19533E+00_JPRB, 0.19795E+00_JPRB, 0.20087E+00_JPRB, 0.20727E+00_JPRB, 0.21260E+00_JPRB, 0.22195E+00_JPRB ,&
&  0.22830E+00_JPRB, 0.23670E+00_JPRB, 0.23386E+00_JPRB, 0.24033E+00_JPRB, 0.24532E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.24628E+00_JPRB, 0.24757E+00_JPRB, 0.24700E+00_JPRB, 0.24698E+00_JPRB, 0.24512E+00_JPRB, 0.24257E+00_JPRB, 0.23872E+00_JPRB ,&
&  0.23526E+00_JPRB, 0.22899E+00_JPRB, 0.22434E+00_JPRB, 0.21910E+00_JPRB, 0.21472E+00_JPRB, 0.21078E+00_JPRB, 0.20795E+00_JPRB ,&
&  0.20653E+00_JPRB, 0.20585E+00_JPRB, 0.20541E+00_JPRB, 0.20431E+00_JPRB, 0.20235E+00_JPRB, 0.20107E+00_JPRB, 0.19947E+00_JPRB ,&
&  0.19831E+00_JPRB, 0.19832E+00_JPRB, 0.19937E+00_JPRB, 0.20140E+00_JPRB, 0.20507E+00_JPRB, 0.20740E+00_JPRB, 0.20961E+00_JPRB ,&
&  0.21132E+00_JPRB, 0.20684E+00_JPRB, 0.20691E+00_JPRB, 0.20759E+00_JPRB, 0.20786E+00_JPRB, 0.20760E+00_JPRB, 0.20734E+00_JPRB ,&
&  0.21177E+00_JPRB, 0.21301E+00_JPRB, 0.21936E+00_JPRB, 0.21894E+00_JPRB, 0.21810E+00_JPRB, 0.22036E+00_JPRB, 0.21950E+00_JPRB ,&
&  0.21996E+00_JPRB, 0.21652E+00_JPRB, 0.21396E+00_JPRB, 0.20984E+00_JPRB, 0.20557E+00_JPRB, 0.20340E+00_JPRB, 0.20091E+00_JPRB ,&
&  0.20012E+00_JPRB, 0.19996E+00_JPRB, 0.19822E+00_JPRB, 0.19713E+00_JPRB, 0.19503E+00_JPRB, 0.19266E+00_JPRB, 0.19124E+00_JPRB ,&
&  0.18979E+00_JPRB, 0.18866E+00_JPRB, 0.18808E+00_JPRB, 0.18746E+00_JPRB, 0.18718E+00_JPRB, 0.18701E+00_JPRB, 0.18683E+00_JPRB ,&
&  0.18684E+00_JPRB, 0.18678E+00_JPRB, 0.18614E+00_JPRB, 0.18585E+00_JPRB, 0.18462E+00_JPRB, 0.18379E+00_JPRB, 0.18311E+00_JPRB ,&
&  0.18246E+00_JPRB, 0.18222E+00_JPRB, 0.18209E+00_JPRB, 0.18215E+00_JPRB, 0.18232E+00_JPRB, 0.18291E+00_JPRB, 0.18359E+00_JPRB ,&
&  0.18425E+00_JPRB, 0.18540E+00_JPRB, 0.18629E+00_JPRB, 0.18759E+00_JPRB, 0.18914E+00_JPRB, 0.19046E+00_JPRB, 0.19256E+00_JPRB ,&
&  0.19485E+00_JPRB, 0.19750E+00_JPRB, 0.20134E+00_JPRB, 0.20455E+00_JPRB, 0.20861E+00_JPRB, 0.21393E+00_JPRB, 0.21950E+00_JPRB ,&
&  0.22548E+00_JPRB, 0.23156E+00_JPRB, 0.23179E+00_JPRB, 0.23927E+00_JPRB, 0.24385E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.23540E+00_JPRB, 0.23615E+00_JPRB, 0.23691E+00_JPRB, 0.23714E+00_JPRB, 0.23718E+00_JPRB, 0.23610E+00_JPRB, 0.23469E+00_JPRB ,&
&  0.23016E+00_JPRB, 0.22697E+00_JPRB, 0.22328E+00_JPRB, 0.21957E+00_JPRB, 0.21630E+00_JPRB, 0.21264E+00_JPRB, 0.20953E+00_JPRB ,&
&  0.20669E+00_JPRB, 0.20451E+00_JPRB, 0.20263E+00_JPRB, 0.20024E+00_JPRB, 0.19725E+00_JPRB, 0.19452E+00_JPRB, 0.19297E+00_JPRB ,&
&  0.19264E+00_JPRB, 0.19312E+00_JPRB, 0.19483E+00_JPRB, 0.19649E+00_JPRB, 0.19823E+00_JPRB, 0.19954E+00_JPRB, 0.20088E+00_JPRB ,&
&  0.20033E+00_JPRB, 0.20004E+00_JPRB, 0.19972E+00_JPRB, 0.19907E+00_JPRB, 0.20000E+00_JPRB, 0.20124E+00_JPRB, 0.20211E+00_JPRB ,&
&  0.20314E+00_JPRB, 0.20544E+00_JPRB, 0.20722E+00_JPRB, 0.20882E+00_JPRB, 0.20905E+00_JPRB, 0.20858E+00_JPRB, 0.20715E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20300E+00_JPRB, 0.20097E+00_JPRB, 0.19920E+00_JPRB, 0.19783E+00_JPRB, 0.19620E+00_JPRB, 0.19477E+00_JPRB ,&
&  0.19407E+00_JPRB, 0.19344E+00_JPRB, 0.19255E+00_JPRB, 0.19168E+00_JPRB, 0.19033E+00_JPRB, 0.18915E+00_JPRB, 0.18793E+00_JPRB ,&
&  0.18684E+00_JPRB, 0.18581E+00_JPRB, 0.18486E+00_JPRB, 0.18388E+00_JPRB, 0.18321E+00_JPRB, 0.18260E+00_JPRB, 0.18201E+00_JPRB ,&
&  0.18140E+00_JPRB, 0.18120E+00_JPRB, 0.18076E+00_JPRB, 0.18061E+00_JPRB, 0.18031E+00_JPRB, 0.18012E+00_JPRB, 0.18032E+00_JPRB ,&
&  0.18094E+00_JPRB, 0.18156E+00_JPRB, 0.18185E+00_JPRB, 0.18227E+00_JPRB, 0.18271E+00_JPRB, 0.18327E+00_JPRB, 0.18420E+00_JPRB ,&
&  0.18492E+00_JPRB, 0.18573E+00_JPRB, 0.18635E+00_JPRB, 0.18713E+00_JPRB, 0.18867E+00_JPRB, 0.19024E+00_JPRB, 0.19203E+00_JPRB ,&
&  0.19428E+00_JPRB, 0.19661E+00_JPRB, 0.19932E+00_JPRB, 0.20189E+00_JPRB, 0.20542E+00_JPRB, 0.20715E+00_JPRB, 0.21073E+00_JPRB ,&
&  0.21467E+00_JPRB, 0.21524E+00_JPRB, 0.22893E+00_JPRB, 0.23024E+00_JPRB, 0.23213E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.20593E+00_JPRB, 0.20792E+00_JPRB, 0.20906E+00_JPRB, 0.21032E+00_JPRB, 0.20738E+00_JPRB, 0.21983E+00_JPRB, 0.21749E+00_JPRB ,&
&  0.21619E+00_JPRB, 0.21193E+00_JPRB, 0.21191E+00_JPRB, 0.21024E+00_JPRB, 0.20805E+00_JPRB, 0.20556E+00_JPRB, 0.20325E+00_JPRB ,&
&  0.20109E+00_JPRB, 0.19868E+00_JPRB, 0.19702E+00_JPRB, 0.19574E+00_JPRB, 0.19456E+00_JPRB, 0.19349E+00_JPRB, 0.19270E+00_JPRB ,&
&  0.19277E+00_JPRB, 0.19346E+00_JPRB, 0.19475E+00_JPRB, 0.19460E+00_JPRB, 0.19419E+00_JPRB, 0.19368E+00_JPRB, 0.19339E+00_JPRB ,&
&  0.19271E+00_JPRB, 0.19223E+00_JPRB, 0.19178E+00_JPRB, 0.19114E+00_JPRB, 0.19214E+00_JPRB, 0.19322E+00_JPRB, 0.19427E+00_JPRB ,&
&  0.19526E+00_JPRB, 0.19624E+00_JPRB, 0.19642E+00_JPRB, 0.19639E+00_JPRB, 0.19604E+00_JPRB, 0.19558E+00_JPRB, 0.19474E+00_JPRB ,&
&  0.19394E+00_JPRB, 0.19307E+00_JPRB, 0.19195E+00_JPRB, 0.19123E+00_JPRB, 0.19058E+00_JPRB, 0.18999E+00_JPRB, 0.18945E+00_JPRB ,&
&  0.18917E+00_JPRB, 0.18899E+00_JPRB, 0.18875E+00_JPRB, 0.18838E+00_JPRB, 0.18794E+00_JPRB, 0.18749E+00_JPRB, 0.18710E+00_JPRB ,&
&  0.18680E+00_JPRB, 0.18620E+00_JPRB, 0.18567E+00_JPRB, 0.18511E+00_JPRB, 0.18469E+00_JPRB, 0.18410E+00_JPRB, 0.18362E+00_JPRB ,&
&  0.18314E+00_JPRB, 0.18278E+00_JPRB, 0.18264E+00_JPRB, 0.18254E+00_JPRB, 0.18245E+00_JPRB, 0.18247E+00_JPRB, 0.18259E+00_JPRB ,&
&  0.18278E+00_JPRB, 0.18299E+00_JPRB, 0.18333E+00_JPRB, 0.18357E+00_JPRB, 0.18381E+00_JPRB, 0.18401E+00_JPRB, 0.18448E+00_JPRB ,&
&  0.18506E+00_JPRB, 0.18565E+00_JPRB, 0.18617E+00_JPRB, 0.18630E+00_JPRB, 0.18718E+00_JPRB, 0.18815E+00_JPRB, 0.18921E+00_JPRB ,&
&  0.19034E+00_JPRB, 0.19181E+00_JPRB, 0.19342E+00_JPRB, 0.19519E+00_JPRB, 0.19732E+00_JPRB, 0.19857E+00_JPRB, 0.20007E+00_JPRB ,&
&  0.20212E+00_JPRB, 0.20557E+00_JPRB, 0.20652E+00_JPRB, 0.20672E+00_JPRB, 0.20658E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.19385E+00_JPRB, 0.19355E+00_JPRB, 0.19320E+00_JPRB, 0.19278E+00_JPRB, 0.19157E+00_JPRB, 0.19093E+00_JPRB, 0.19040E+00_JPRB ,&
&  0.19007E+00_JPRB, 0.18985E+00_JPRB, 0.18992E+00_JPRB, 0.19008E+00_JPRB, 0.19025E+00_JPRB, 0.19042E+00_JPRB, 0.19056E+00_JPRB ,&
&  0.19054E+00_JPRB, 0.19055E+00_JPRB, 0.18969E+00_JPRB, 0.18899E+00_JPRB, 0.18854E+00_JPRB, 0.18819E+00_JPRB, 0.18776E+00_JPRB ,&
&  0.18739E+00_JPRB, 0.18698E+00_JPRB, 0.18649E+00_JPRB, 0.18620E+00_JPRB, 0.18625E+00_JPRB, 0.18610E+00_JPRB, 0.18614E+00_JPRB ,&
&  0.18618E+00_JPRB, 0.18646E+00_JPRB, 0.18662E+00_JPRB, 0.18680E+00_JPRB, 0.18734E+00_JPRB, 0.18785E+00_JPRB, 0.18803E+00_JPRB ,&
&  0.18816E+00_JPRB, 0.18802E+00_JPRB, 0.18787E+00_JPRB, 0.18776E+00_JPRB, 0.18772E+00_JPRB, 0.18775E+00_JPRB, 0.18774E+00_JPRB ,&
&  0.18764E+00_JPRB, 0.18747E+00_JPRB, 0.18723E+00_JPRB, 0.18695E+00_JPRB, 0.18669E+00_JPRB, 0.18662E+00_JPRB, 0.18666E+00_JPRB ,&
&  0.18669E+00_JPRB, 0.18658E+00_JPRB, 0.18638E+00_JPRB, 0.18602E+00_JPRB, 0.18566E+00_JPRB, 0.18529E+00_JPRB, 0.18496E+00_JPRB ,&
&  0.18462E+00_JPRB, 0.18447E+00_JPRB, 0.18431E+00_JPRB, 0.18419E+00_JPRB, 0.18411E+00_JPRB, 0.18398E+00_JPRB, 0.18386E+00_JPRB ,&
&  0.18375E+00_JPRB, 0.18365E+00_JPRB, 0.18353E+00_JPRB, 0.18344E+00_JPRB, 0.18337E+00_JPRB, 0.18331E+00_JPRB, 0.18329E+00_JPRB ,&
&  0.18330E+00_JPRB, 0.18330E+00_JPRB, 0.18332E+00_JPRB, 0.18348E+00_JPRB, 0.18366E+00_JPRB, 0.18389E+00_JPRB, 0.18415E+00_JPRB ,&
&  0.18445E+00_JPRB, 0.18478E+00_JPRB, 0.18511E+00_JPRB, 0.18535E+00_JPRB, 0.18574E+00_JPRB, 0.18614E+00_JPRB, 0.18656E+00_JPRB ,&
&  0.18698E+00_JPRB, 0.18749E+00_JPRB, 0.18806E+00_JPRB, 0.18873E+00_JPRB, 0.18953E+00_JPRB, 0.18993E+00_JPRB, 0.19048E+00_JPRB ,&
&  0.19112E+00_JPRB, 0.19201E+00_JPRB, 0.19247E+00_JPRB, 0.19279E+00_JPRB, 0.19312E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB ,&
&  0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB, 0.18583E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB, 0.71934E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.72541E+00_JPRB, 0.72504E+00_JPRB, 0.72466E+00_JPRB, 0.72435E+00_JPRB, 0.72405E+00_JPRB, 0.72387E+00_JPRB, 0.72363E+00_JPRB ,&
&  0.72362E+00_JPRB, 0.72359E+00_JPRB, 0.72354E+00_JPRB, 0.72337E+00_JPRB, 0.72338E+00_JPRB, 0.72330E+00_JPRB, 0.72318E+00_JPRB ,&
&  0.72296E+00_JPRB, 0.72285E+00_JPRB, 0.72243E+00_JPRB, 0.72200E+00_JPRB, 0.72142E+00_JPRB, 0.72100E+00_JPRB, 0.72034E+00_JPRB ,&
&  0.71989E+00_JPRB, 0.71913E+00_JPRB, 0.71841E+00_JPRB, 0.71714E+00_JPRB, 0.71620E+00_JPRB, 0.71518E+00_JPRB, 0.71431E+00_JPRB ,&
&  0.71354E+00_JPRB, 0.71306E+00_JPRB, 0.71259E+00_JPRB, 0.71193E+00_JPRB, 0.71140E+00_JPRB, 0.71151E+00_JPRB, 0.71177E+00_JPRB ,&
&  0.71203E+00_JPRB, 0.71242E+00_JPRB, 0.71294E+00_JPRB, 0.71314E+00_JPRB, 0.71303E+00_JPRB, 0.71310E+00_JPRB, 0.71305E+00_JPRB ,&
&  0.71285E+00_JPRB, 0.71248E+00_JPRB, 0.71212E+00_JPRB, 0.71204E+00_JPRB, 0.71196E+00_JPRB, 0.71167E+00_JPRB, 0.71147E+00_JPRB ,&
&  0.71130E+00_JPRB, 0.71109E+00_JPRB, 0.71079E+00_JPRB, 0.71032E+00_JPRB, 0.70997E+00_JPRB, 0.70962E+00_JPRB, 0.70922E+00_JPRB ,&
&  0.70877E+00_JPRB, 0.70822E+00_JPRB, 0.70766E+00_JPRB, 0.70705E+00_JPRB, 0.70640E+00_JPRB, 0.70592E+00_JPRB, 0.70558E+00_JPRB ,&
&  0.70536E+00_JPRB, 0.70534E+00_JPRB, 0.70546E+00_JPRB, 0.70561E+00_JPRB, 0.70588E+00_JPRB, 0.70632E+00_JPRB, 0.70694E+00_JPRB ,&
&  0.70766E+00_JPRB, 0.70847E+00_JPRB, 0.70954E+00_JPRB, 0.71084E+00_JPRB, 0.71209E+00_JPRB, 0.71337E+00_JPRB, 0.71480E+00_JPRB ,&
&  0.71607E+00_JPRB, 0.71705E+00_JPRB, 0.71778E+00_JPRB, 0.71854E+00_JPRB, 0.71970E+00_JPRB, 0.72063E+00_JPRB, 0.72140E+00_JPRB ,&
&  0.72209E+00_JPRB, 0.72285E+00_JPRB, 0.72356E+00_JPRB, 0.72432E+00_JPRB, 0.72465E+00_JPRB, 0.72499E+00_JPRB, 0.72528E+00_JPRB ,&
&  0.72554E+00_JPRB, 0.72565E+00_JPRB, 0.72570E+00_JPRB, 0.72570E+00_JPRB, 0.72560E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.72373E+00_JPRB, 0.72282E+00_JPRB, 0.72223E+00_JPRB, 0.72195E+00_JPRB, 0.72180E+00_JPRB, 0.72185E+00_JPRB, 0.72203E+00_JPRB ,&
&  0.72238E+00_JPRB, 0.72240E+00_JPRB, 0.72203E+00_JPRB, 0.72213E+00_JPRB, 0.72280E+00_JPRB, 0.72363E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.72334E+00_JPRB, 0.72311E+00_JPRB, 0.72308E+00_JPRB, 0.72256E+00_JPRB, 0.72164E+00_JPRB, 0.72035E+00_JPRB, 0.71903E+00_JPRB ,&
&  0.71799E+00_JPRB, 0.71658E+00_JPRB, 0.71500E+00_JPRB, 0.71175E+00_JPRB, 0.70897E+00_JPRB, 0.70624E+00_JPRB, 0.70364E+00_JPRB ,&
&  0.70004E+00_JPRB, 0.69694E+00_JPRB, 0.69453E+00_JPRB, 0.69255E+00_JPRB, 0.69565E+00_JPRB, 0.69769E+00_JPRB, 0.69884E+00_JPRB ,&
&  0.69895E+00_JPRB, 0.69567E+00_JPRB, 0.69469E+00_JPRB, 0.69418E+00_JPRB, 0.69394E+00_JPRB, 0.69672E+00_JPRB, 0.69862E+00_JPRB ,&
&  0.70021E+00_JPRB, 0.70149E+00_JPRB, 0.70446E+00_JPRB, 0.70591E+00_JPRB, 0.70695E+00_JPRB, 0.70753E+00_JPRB, 0.70741E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70848E+00_JPRB, 0.70893E+00_JPRB, 0.70875E+00_JPRB, 0.70845E+00_JPRB, 0.70812E+00_JPRB, 0.70739E+00_JPRB ,&
&  0.70613E+00_JPRB, 0.70481E+00_JPRB, 0.70346E+00_JPRB, 0.70202E+00_JPRB, 0.70038E+00_JPRB, 0.69921E+00_JPRB, 0.69820E+00_JPRB ,&
&  0.69723E+00_JPRB, 0.69648E+00_JPRB, 0.69600E+00_JPRB, 0.69566E+00_JPRB, 0.69553E+00_JPRB, 0.69595E+00_JPRB, 0.69711E+00_JPRB ,&
&  0.69890E+00_JPRB, 0.70122E+00_JPRB, 0.70395E+00_JPRB, 0.70686E+00_JPRB, 0.70938E+00_JPRB, 0.71161E+00_JPRB, 0.71432E+00_JPRB ,&
&  0.71722E+00_JPRB, 0.71949E+00_JPRB, 0.72136E+00_JPRB, 0.72302E+00_JPRB, 0.72510E+00_JPRB, 0.72632E+00_JPRB, 0.72696E+00_JPRB ,&
&  0.72740E+00_JPRB, 0.72843E+00_JPRB, 0.72954E+00_JPRB, 0.73082E+00_JPRB, 0.73143E+00_JPRB, 0.73135E+00_JPRB, 0.73078E+00_JPRB ,&
&  0.72999E+00_JPRB, 0.72920E+00_JPRB, 0.72803E+00_JPRB, 0.72654E+00_JPRB, 0.72507E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.71940E+00_JPRB, 0.72050E+00_JPRB, 0.72221E+00_JPRB, 0.72436E+00_JPRB, 0.72691E+00_JPRB, 0.72872E+00_JPRB, 0.73072E+00_JPRB ,&
&  0.73209E+00_JPRB, 0.73269E+00_JPRB, 0.73241E+00_JPRB, 0.73309E+00_JPRB, 0.73331E+00_JPRB, 0.73382E+00_JPRB, 0.73304E+00_JPRB ,&
&  0.73286E+00_JPRB, 0.73176E+00_JPRB, 0.73258E+00_JPRB, 0.73295E+00_JPRB, 0.73295E+00_JPRB, 0.73311E+00_JPRB, 0.72736E+00_JPRB ,&
&  0.72202E+00_JPRB, 0.71813E+00_JPRB, 0.71238E+00_JPRB, 0.70683E+00_JPRB, 0.70306E+00_JPRB, 0.67926E+00_JPRB, 0.67211E+00_JPRB ,&
&  0.68304E+00_JPRB, 0.68994E+00_JPRB, 0.69566E+00_JPRB, 0.69957E+00_JPRB, 0.68589E+00_JPRB, 0.67040E+00_JPRB, 0.66115E+00_JPRB ,&
&  0.63861E+00_JPRB, 0.64530E+00_JPRB, 0.65005E+00_JPRB, 0.65385E+00_JPRB, 0.66011E+00_JPRB, 0.66213E+00_JPRB, 0.66820E+00_JPRB ,&
&  0.68289E+00_JPRB, 0.68693E+00_JPRB, 0.69259E+00_JPRB, 0.69735E+00_JPRB, 0.70029E+00_JPRB, 0.70383E+00_JPRB, 0.71100E+00_JPRB ,&
&  0.71422E+00_JPRB, 0.71723E+00_JPRB, 0.71920E+00_JPRB, 0.71905E+00_JPRB, 0.71936E+00_JPRB, 0.71859E+00_JPRB, 0.71752E+00_JPRB ,&
&  0.71450E+00_JPRB, 0.71288E+00_JPRB, 0.71156E+00_JPRB, 0.70998E+00_JPRB, 0.70881E+00_JPRB, 0.70541E+00_JPRB, 0.70292E+00_JPRB ,&
&  0.70134E+00_JPRB, 0.70055E+00_JPRB, 0.70028E+00_JPRB, 0.70087E+00_JPRB, 0.70263E+00_JPRB, 0.70511E+00_JPRB, 0.70763E+00_JPRB ,&
&  0.71102E+00_JPRB, 0.71409E+00_JPRB, 0.71788E+00_JPRB, 0.72169E+00_JPRB, 0.72473E+00_JPRB, 0.72722E+00_JPRB, 0.72958E+00_JPRB ,&
&  0.73256E+00_JPRB, 0.73580E+00_JPRB, 0.73760E+00_JPRB, 0.73754E+00_JPRB, 0.73943E+00_JPRB, 0.74091E+00_JPRB, 0.74141E+00_JPRB ,&
&  0.74165E+00_JPRB, 0.74229E+00_JPRB, 0.74144E+00_JPRB, 0.73962E+00_JPRB, 0.73721E+00_JPRB, 0.73488E+00_JPRB, 0.73232E+00_JPRB ,&
&  0.72956E+00_JPRB, 0.72690E+00_JPRB, 0.72424E+00_JPRB, 0.72159E+00_JPRB, 0.71982E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.72412E+00_JPRB, 0.72818E+00_JPRB, 0.73335E+00_JPRB, 0.73644E+00_JPRB, 0.74035E+00_JPRB, 0.74191E+00_JPRB, 0.74427E+00_JPRB ,&
&  0.74493E+00_JPRB, 0.74605E+00_JPRB, 0.74679E+00_JPRB, 0.74694E+00_JPRB, 0.74676E+00_JPRB, 0.74578E+00_JPRB, 0.74496E+00_JPRB ,&
&  0.74461E+00_JPRB, 0.74381E+00_JPRB, 0.74440E+00_JPRB, 0.74458E+00_JPRB, 0.74423E+00_JPRB, 0.74386E+00_JPRB, 0.73729E+00_JPRB ,&
&  0.72990E+00_JPRB, 0.72290E+00_JPRB, 0.69291E+00_JPRB, 0.69185E+00_JPRB, 0.68793E+00_JPRB, 0.68599E+00_JPRB, 0.68795E+00_JPRB ,&
&  0.68404E+00_JPRB, 0.67941E+00_JPRB, 0.66234E+00_JPRB, 0.64582E+00_JPRB, 0.63411E+00_JPRB, 0.62157E+00_JPRB, 0.62157E+00_JPRB ,&
&  0.62073E+00_JPRB, 0.62547E+00_JPRB, 0.62990E+00_JPRB, 0.63254E+00_JPRB, 0.64098E+00_JPRB, 0.64552E+00_JPRB, 0.65503E+00_JPRB ,&
&  0.66704E+00_JPRB, 0.67490E+00_JPRB, 0.67359E+00_JPRB, 0.67156E+00_JPRB, 0.68934E+00_JPRB, 0.70152E+00_JPRB, 0.71254E+00_JPRB ,&
&  0.71958E+00_JPRB, 0.72110E+00_JPRB, 0.72457E+00_JPRB, 0.72466E+00_JPRB, 0.72581E+00_JPRB, 0.72902E+00_JPRB, 0.72991E+00_JPRB ,&
&  0.73164E+00_JPRB, 0.73055E+00_JPRB, 0.72774E+00_JPRB, 0.72390E+00_JPRB, 0.71763E+00_JPRB, 0.71353E+00_JPRB, 0.71108E+00_JPRB ,&
&  0.71105E+00_JPRB, 0.71332E+00_JPRB, 0.71655E+00_JPRB, 0.72120E+00_JPRB, 0.72603E+00_JPRB, 0.72977E+00_JPRB, 0.73216E+00_JPRB ,&
&  0.73389E+00_JPRB, 0.73479E+00_JPRB, 0.73964E+00_JPRB, 0.74042E+00_JPRB, 0.74305E+00_JPRB, 0.75013E+00_JPRB, 0.74981E+00_JPRB ,&
&  0.75159E+00_JPRB, 0.75604E+00_JPRB, 0.75893E+00_JPRB, 0.76135E+00_JPRB, 0.76298E+00_JPRB, 0.76365E+00_JPRB, 0.76457E+00_JPRB ,&
&  0.76470E+00_JPRB, 0.76339E+00_JPRB, 0.76020E+00_JPRB, 0.75575E+00_JPRB, 0.75094E+00_JPRB, 0.74600E+00_JPRB, 0.73962E+00_JPRB ,&
&  0.73338E+00_JPRB, 0.72730E+00_JPRB, 0.72305E+00_JPRB, 0.72023E+00_JPRB, 0.72135E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.73853E+00_JPRB, 0.74264E+00_JPRB, 0.74902E+00_JPRB, 0.75125E+00_JPRB, 0.75489E+00_JPRB, 0.75619E+00_JPRB, 0.75747E+00_JPRB ,&
&  0.75805E+00_JPRB, 0.75867E+00_JPRB, 0.75846E+00_JPRB, 0.75844E+00_JPRB, 0.75786E+00_JPRB, 0.75750E+00_JPRB, 0.75689E+00_JPRB ,&
&  0.75594E+00_JPRB, 0.75493E+00_JPRB, 0.75405E+00_JPRB, 0.75322E+00_JPRB, 0.75325E+00_JPRB, 0.75320E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.74418E+00_JPRB, 0.73205E+00_JPRB, 0.70907E+00_JPRB, 0.70638E+00_JPRB, 0.70345E+00_JPRB, 0.70492E+00_JPRB, 0.70359E+00_JPRB ,&
&  0.69379E+00_JPRB, 0.64897E+00_JPRB, 0.65308E+00_JPRB, 0.64357E+00_JPRB, 0.64035E+00_JPRB, 0.63669E+00_JPRB, 0.62474E+00_JPRB ,&
&  0.62070E+00_JPRB, 0.61587E+00_JPRB, 0.61497E+00_JPRB, 0.62145E+00_JPRB, 0.62743E+00_JPRB, 0.63654E+00_JPRB, 0.64739E+00_JPRB ,&
&  0.64561E+00_JPRB, 0.66143E+00_JPRB, 0.67186E+00_JPRB, 0.67853E+00_JPRB, 0.69618E+00_JPRB, 0.71100E+00_JPRB, 0.71484E+00_JPRB ,&
&  0.71951E+00_JPRB, 0.71794E+00_JPRB, 0.72110E+00_JPRB, 0.72835E+00_JPRB, 0.73619E+00_JPRB, 0.73606E+00_JPRB, 0.73650E+00_JPRB ,&
&  0.73555E+00_JPRB, 0.73441E+00_JPRB, 0.73294E+00_JPRB, 0.73066E+00_JPRB, 0.72963E+00_JPRB, 0.73165E+00_JPRB, 0.73481E+00_JPRB ,&
&  0.74105E+00_JPRB, 0.74735E+00_JPRB, 0.75439E+00_JPRB, 0.76082E+00_JPRB, 0.76590E+00_JPRB, 0.77063E+00_JPRB, 0.77516E+00_JPRB ,&
&  0.77980E+00_JPRB, 0.78307E+00_JPRB, 0.78377E+00_JPRB, 0.78426E+00_JPRB, 0.78319E+00_JPRB, 0.78028E+00_JPRB, 0.78058E+00_JPRB ,&
&  0.78393E+00_JPRB, 0.78600E+00_JPRB, 0.78693E+00_JPRB, 0.78793E+00_JPRB, 0.78986E+00_JPRB, 0.79067E+00_JPRB, 0.79086E+00_JPRB ,&
&  0.79034E+00_JPRB, 0.78811E+00_JPRB, 0.78445E+00_JPRB, 0.77657E+00_JPRB, 0.76813E+00_JPRB, 0.76241E+00_JPRB, 0.75318E+00_JPRB ,&
&  0.74426E+00_JPRB, 0.73474E+00_JPRB, 0.73068E+00_JPRB, 0.72844E+00_JPRB, 0.73276E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.75781E+00_JPRB, 0.76395E+00_JPRB, 0.77099E+00_JPRB, 0.77462E+00_JPRB, 0.77833E+00_JPRB, 0.77917E+00_JPRB, 0.78086E+00_JPRB ,&
&  0.78178E+00_JPRB, 0.78311E+00_JPRB, 0.78358E+00_JPRB, 0.78364E+00_JPRB, 0.78276E+00_JPRB, 0.78159E+00_JPRB, 0.77966E+00_JPRB ,&
&  0.77647E+00_JPRB, 0.77339E+00_JPRB, 0.76869E+00_JPRB, 0.76479E+00_JPRB, 0.76283E+00_JPRB, 0.76343E+00_JPRB, 0.76136E+00_JPRB ,&
&  0.75609E+00_JPRB, 0.74529E+00_JPRB, 0.72772E+00_JPRB, 0.72159E+00_JPRB, 0.72185E+00_JPRB, 0.71168E+00_JPRB, 0.70894E+00_JPRB ,&
&  0.70537E+00_JPRB, 0.70367E+00_JPRB, 0.69446E+00_JPRB, 0.68885E+00_JPRB, 0.67455E+00_JPRB, 0.61833E+00_JPRB, 0.61490E+00_JPRB ,&
&  0.61207E+00_JPRB, 0.60443E+00_JPRB, 0.60383E+00_JPRB, 0.60595E+00_JPRB, 0.61253E+00_JPRB, 0.63448E+00_JPRB, 0.65300E+00_JPRB ,&
&  0.65039E+00_JPRB, 0.66901E+00_JPRB, 0.67716E+00_JPRB, 0.68529E+00_JPRB, 0.70024E+00_JPRB, 0.71376E+00_JPRB, 0.71558E+00_JPRB ,&
&  0.71952E+00_JPRB, 0.72291E+00_JPRB, 0.73021E+00_JPRB, 0.73783E+00_JPRB, 0.74361E+00_JPRB, 0.74669E+00_JPRB, 0.75082E+00_JPRB ,&
&  0.75569E+00_JPRB, 0.75954E+00_JPRB, 0.76438E+00_JPRB, 0.76599E+00_JPRB, 0.76631E+00_JPRB, 0.76721E+00_JPRB, 0.77062E+00_JPRB ,&
&  0.77836E+00_JPRB, 0.78583E+00_JPRB, 0.79716E+00_JPRB, 0.79721E+00_JPRB, 0.80115E+00_JPRB, 0.80439E+00_JPRB, 0.81204E+00_JPRB ,&
&  0.81708E+00_JPRB, 0.83048E+00_JPRB, 0.84046E+00_JPRB, 0.84592E+00_JPRB, 0.83809E+00_JPRB, 0.83012E+00_JPRB, 0.82449E+00_JPRB ,&
&  0.82203E+00_JPRB, 0.82242E+00_JPRB, 0.81554E+00_JPRB, 0.81361E+00_JPRB, 0.81348E+00_JPRB, 0.81422E+00_JPRB, 0.81497E+00_JPRB ,&
&  0.81433E+00_JPRB, 0.80991E+00_JPRB, 0.80290E+00_JPRB, 0.79265E+00_JPRB, 0.78375E+00_JPRB, 0.77568E+00_JPRB, 0.76087E+00_JPRB ,&
&  0.74916E+00_JPRB, 0.73877E+00_JPRB, 0.73734E+00_JPRB, 0.73928E+00_JPRB, 0.74768E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.77751E+00_JPRB, 0.78781E+00_JPRB, 0.79771E+00_JPRB, 0.80591E+00_JPRB, 0.80924E+00_JPRB, 0.80955E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80459E+00_JPRB, 0.80370E+00_JPRB, 0.80240E+00_JPRB, 0.80144E+00_JPRB, 0.80024E+00_JPRB, 0.79764E+00_JPRB, 0.79434E+00_JPRB ,&
&  0.79155E+00_JPRB, 0.77728E+00_JPRB, 0.76954E+00_JPRB, 0.76844E+00_JPRB, 0.76618E+00_JPRB, 0.76562E+00_JPRB, 0.76531E+00_JPRB ,&
&  0.75912E+00_JPRB, 0.75001E+00_JPRB, 0.74297E+00_JPRB, 0.73313E+00_JPRB, 0.72332E+00_JPRB, 0.72785E+00_JPRB, 0.72656E+00_JPRB ,&
&  0.70681E+00_JPRB, 0.69176E+00_JPRB, 0.69794E+00_JPRB, 0.69964E+00_JPRB, 0.67179E+00_JPRB, 0.64262E+00_JPRB, 0.62394E+00_JPRB ,&
&  0.62709E+00_JPRB, 0.61350E+00_JPRB, 0.61135E+00_JPRB, 0.61342E+00_JPRB, 0.62299E+00_JPRB, 0.63291E+00_JPRB, 0.65180E+00_JPRB ,&
&  0.65694E+00_JPRB, 0.66090E+00_JPRB, 0.67745E+00_JPRB, 0.69444E+00_JPRB, 0.70481E+00_JPRB, 0.71543E+00_JPRB, 0.72075E+00_JPRB ,&
&  0.73049E+00_JPRB, 0.74023E+00_JPRB, 0.74886E+00_JPRB, 0.75675E+00_JPRB, 0.76597E+00_JPRB, 0.76943E+00_JPRB, 0.77522E+00_JPRB ,&
&  0.81180E+00_JPRB, 0.84063E+00_JPRB, 0.85429E+00_JPRB, 0.86403E+00_JPRB, 0.86558E+00_JPRB, 0.86247E+00_JPRB, 0.85049E+00_JPRB ,&
&  0.84614E+00_JPRB, 0.85024E+00_JPRB, 0.85326E+00_JPRB, 0.85887E+00_JPRB, 0.86244E+00_JPRB, 0.86196E+00_JPRB, 0.85586E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.85012E+00_JPRB, 0.93193E+00_JPRB, 0.95067E+00_JPRB, 0.93851E+00_JPRB, 0.91263E+00_JPRB, 0.88415E+00_JPRB ,&
&  0.86329E+00_JPRB, 0.85038E+00_JPRB, 0.84279E+00_JPRB, 0.83700E+00_JPRB, 0.83278E+00_JPRB, 0.82634E+00_JPRB, 0.82675E+00_JPRB ,&
&  0.82352E+00_JPRB, 0.81572E+00_JPRB, 0.80731E+00_JPRB, 0.79775E+00_JPRB, 0.78871E+00_JPRB, 0.78104E+00_JPRB, 0.76968E+00_JPRB ,&
&  0.75951E+00_JPRB, 0.75060E+00_JPRB, 0.75113E+00_JPRB, 0.75663E+00_JPRB, 0.76718E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.80015E+00_JPRB, 0.81325E+00_JPRB, 0.82671E+00_JPRB, 0.83644E+00_JPRB, 0.83645E+00_JPRB, 0.82135E+00_JPRB, 0.80787E+00_JPRB ,&
&  0.79901E+00_JPRB, 0.79542E+00_JPRB, 0.79294E+00_JPRB, 0.78946E+00_JPRB, 0.78732E+00_JPRB, 0.78524E+00_JPRB, 0.78383E+00_JPRB ,&
&  0.78309E+00_JPRB, 0.77976E+00_JPRB, 0.77203E+00_JPRB, 0.76276E+00_JPRB, 0.75791E+00_JPRB, 0.75758E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.75627E+00_JPRB, 0.75277E+00_JPRB, 0.75300E+00_JPRB, 0.74127E+00_JPRB, 0.73089E+00_JPRB, 0.71511E+00_JPRB, 0.70618E+00_JPRB ,&
&  0.70787E+00_JPRB, 0.70956E+00_JPRB, 0.69341E+00_JPRB, 0.69301E+00_JPRB, 0.69596E+00_JPRB, 0.66805E+00_JPRB, 0.64523E+00_JPRB ,&
&  0.63280E+00_JPRB, 0.62395E+00_JPRB, 0.62006E+00_JPRB, 0.62201E+00_JPRB, 0.64518E+00_JPRB, 0.67812E+00_JPRB, 0.68632E+00_JPRB ,&
&  0.65302E+00_JPRB, 0.67060E+00_JPRB, 0.69373E+00_JPRB, 0.70618E+00_JPRB, 0.71639E+00_JPRB, 0.73066E+00_JPRB, 0.74491E+00_JPRB ,&
&  0.75737E+00_JPRB, 0.76769E+00_JPRB, 0.77362E+00_JPRB, 0.78757E+00_JPRB, 0.79288E+00_JPRB, 0.78770E+00_JPRB, 0.80213E+00_JPRB ,&
&  0.89794E+00_JPRB, 0.97364E+00_JPRB, 0.10035E+01_JPRB, 0.10089E+01_JPRB, 0.10021E+01_JPRB, 0.98366E+00_JPRB, 0.97025E+00_JPRB ,&
&  0.95815E+00_JPRB, 0.94715E+00_JPRB, 0.93848E+00_JPRB, 0.93172E+00_JPRB, 0.92496E+00_JPRB, 0.91548E+00_JPRB, 0.90935E+00_JPRB ,&
&  0.90253E+00_JPRB, 0.92039E+00_JPRB, 0.11451E+01_JPRB, 0.11253E+01_JPRB, 0.10662E+01_JPRB, 0.10215E+01_JPRB, 0.97549E+00_JPRB ,&
&  0.94079E+00_JPRB, 0.91751E+00_JPRB, 0.90346E+00_JPRB, 0.88776E+00_JPRB, 0.85479E+00_JPRB, 0.83020E+00_JPRB, 0.81882E+00_JPRB ,&
&  0.81264E+00_JPRB, 0.80194E+00_JPRB, 0.79324E+00_JPRB, 0.78566E+00_JPRB, 0.77963E+00_JPRB, 0.77695E+00_JPRB, 0.77175E+00_JPRB ,&
&  0.76676E+00_JPRB, 0.76810E+00_JPRB, 0.77329E+00_JPRB, 0.78310E+00_JPRB, 0.79160E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.81873E+00_JPRB, 0.84661E+00_JPRB, 0.86607E+00_JPRB, 0.87245E+00_JPRB, 0.86606E+00_JPRB, 0.82643E+00_JPRB, 0.79104E+00_JPRB ,&
&  0.77040E+00_JPRB, 0.76325E+00_JPRB, 0.76319E+00_JPRB, 0.76316E+00_JPRB, 0.76437E+00_JPRB, 0.76626E+00_JPRB, 0.76633E+00_JPRB ,&
&  0.76425E+00_JPRB, 0.76356E+00_JPRB, 0.76313E+00_JPRB, 0.75953E+00_JPRB, 0.75392E+00_JPRB, 0.75020E+00_JPRB, 0.74999E+00_JPRB ,&
&  0.74948E+00_JPRB, 0.74696E+00_JPRB, 0.74518E+00_JPRB, 0.73960E+00_JPRB, 0.73695E+00_JPRB, 0.73076E+00_JPRB, 0.71917E+00_JPRB ,&
&  0.70354E+00_JPRB, 0.70442E+00_JPRB, 0.71345E+00_JPRB, 0.71223E+00_JPRB, 0.71495E+00_JPRB, 0.69688E+00_JPRB, 0.66950E+00_JPRB ,&
&  0.65130E+00_JPRB, 0.64450E+00_JPRB, 0.65748E+00_JPRB, 0.69480E+00_JPRB, 0.71772E+00_JPRB, 0.68378E+00_JPRB, 0.69827E+00_JPRB ,&
&  0.70146E+00_JPRB, 0.70370E+00_JPRB, 0.70935E+00_JPRB, 0.72570E+00_JPRB, 0.74423E+00_JPRB, 0.76099E+00_JPRB, 0.77406E+00_JPRB ,&
&  0.78660E+00_JPRB, 0.79654E+00_JPRB, 0.80435E+00_JPRB, 0.79423E+00_JPRB, 0.79680E+00_JPRB, 0.83496E+00_JPRB, 0.88365E+00_JPRB ,&
&  0.94702E+00_JPRB, 0.99370E+00_JPRB, 0.10124E+01_JPRB, 0.10527E+01_JPRB, 0.10776E+01_JPRB, 0.10817E+01_JPRB, 0.10743E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10388E+01_JPRB, 0.10152E+01_JPRB, 0.10023E+01_JPRB, 0.10017E+01_JPRB, 0.10092E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.10016E+01_JPRB, 0.97483E+00_JPRB, 0.94980E+00_JPRB, 0.90454E+00_JPRB, 0.90478E+00_JPRB, 0.92694E+00_JPRB, 0.94612E+00_JPRB ,&
&  0.94190E+00_JPRB, 0.93163E+00_JPRB, 0.92290E+00_JPRB, 0.90692E+00_JPRB, 0.85265E+00_JPRB, 0.81571E+00_JPRB, 0.79368E+00_JPRB ,&
&  0.78069E+00_JPRB, 0.76413E+00_JPRB, 0.76442E+00_JPRB, 0.76112E+00_JPRB, 0.76569E+00_JPRB, 0.77145E+00_JPRB, 0.76783E+00_JPRB ,&
&  0.76565E+00_JPRB, 0.76656E+00_JPRB, 0.77418E+00_JPRB, 0.78378E+00_JPRB, 0.79941E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.89859E+00_JPRB, 0.91102E+00_JPRB, 0.90860E+00_JPRB, 0.90876E+00_JPRB, 0.88936E+00_JPRB, 0.86410E+00_JPRB, 0.83575E+00_JPRB ,&
&  0.81807E+00_JPRB, 0.79321E+00_JPRB, 0.76919E+00_JPRB, 0.74750E+00_JPRB, 0.73601E+00_JPRB, 0.73210E+00_JPRB, 0.73534E+00_JPRB ,&
&  0.74177E+00_JPRB, 0.74898E+00_JPRB, 0.75358E+00_JPRB, 0.75122E+00_JPRB, 0.74635E+00_JPRB, 0.74453E+00_JPRB, 0.74575E+00_JPRB ,&
&  0.74487E+00_JPRB, 0.74328E+00_JPRB, 0.74088E+00_JPRB, 0.73743E+00_JPRB, 0.73313E+00_JPRB, 0.72790E+00_JPRB, 0.72620E+00_JPRB ,&
&  0.72589E+00_JPRB, 0.72612E+00_JPRB, 0.73143E+00_JPRB, 0.73161E+00_JPRB, 0.73713E+00_JPRB, 0.72181E+00_JPRB, 0.69847E+00_JPRB ,&
&  0.67908E+00_JPRB, 0.67993E+00_JPRB, 0.72629E+00_JPRB, 0.70473E+00_JPRB, 0.72465E+00_JPRB, 0.73892E+00_JPRB, 0.73641E+00_JPRB ,&
&  0.73524E+00_JPRB, 0.73917E+00_JPRB, 0.75490E+00_JPRB, 0.77028E+00_JPRB, 0.78451E+00_JPRB, 0.80092E+00_JPRB, 0.81482E+00_JPRB ,&
&  0.83210E+00_JPRB, 0.84868E+00_JPRB, 0.85478E+00_JPRB, 0.84208E+00_JPRB, 0.89815E+00_JPRB, 0.10779E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11829E+01_JPRB, 0.12264E+01_JPRB, 0.12615E+01_JPRB, 0.12411E+01_JPRB, 0.12242E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.11841E+01_JPRB, 0.11666E+01_JPRB, 0.11544E+01_JPRB, 0.11712E+01_JPRB, 0.11739E+01_JPRB, 0.11773E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.11628E+01_JPRB, 0.11335E+01_JPRB, 0.10851E+01_JPRB, 0.10599E+01_JPRB, 0.10432E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.10084E+01_JPRB, 0.99497E+00_JPRB, 0.96843E+00_JPRB, 0.90264E+00_JPRB, 0.83910E+00_JPRB, 0.80082E+00_JPRB ,&
&  0.77401E+00_JPRB, 0.75236E+00_JPRB, 0.75139E+00_JPRB, 0.75729E+00_JPRB, 0.76873E+00_JPRB, 0.78055E+00_JPRB, 0.77741E+00_JPRB ,&
&  0.78288E+00_JPRB, 0.79229E+00_JPRB, 0.81360E+00_JPRB, 0.84268E+00_JPRB, 0.87205E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.97902E+00_JPRB, 0.98271E+00_JPRB, 0.96942E+00_JPRB, 0.95775E+00_JPRB, 0.93899E+00_JPRB, 0.90248E+00_JPRB, 0.85196E+00_JPRB ,&
&  0.83089E+00_JPRB, 0.79209E+00_JPRB, 0.76908E+00_JPRB, 0.75975E+00_JPRB, 0.75252E+00_JPRB, 0.74050E+00_JPRB, 0.72732E+00_JPRB ,&
&  0.72713E+00_JPRB, 0.73834E+00_JPRB, 0.74926E+00_JPRB, 0.75314E+00_JPRB, 0.75486E+00_JPRB, 0.75223E+00_JPRB, 0.75184E+00_JPRB ,&
&  0.75126E+00_JPRB, 0.75113E+00_JPRB, 0.75265E+00_JPRB, 0.74594E+00_JPRB, 0.74130E+00_JPRB, 0.73929E+00_JPRB, 0.73736E+00_JPRB ,&
&  0.74085E+00_JPRB, 0.74095E+00_JPRB, 0.74738E+00_JPRB, 0.74601E+00_JPRB, 0.74385E+00_JPRB, 0.74625E+00_JPRB, 0.70884E+00_JPRB ,&
&  0.69940E+00_JPRB, 0.72441E+00_JPRB, 0.71632E+00_JPRB, 0.72666E+00_JPRB, 0.74443E+00_JPRB, 0.75572E+00_JPRB, 0.76462E+00_JPRB ,&
&  0.77031E+00_JPRB, 0.77801E+00_JPRB, 0.78861E+00_JPRB, 0.80293E+00_JPRB, 0.82136E+00_JPRB, 0.83944E+00_JPRB, 0.85518E+00_JPRB ,&
&  0.88462E+00_JPRB, 0.86129E+00_JPRB, 0.86640E+00_JPRB, 0.93246E+00_JPRB, 0.11412E+01_JPRB, 0.12234E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.14162E+01_JPRB, 0.14308E+01_JPRB, 0.14518E+01_JPRB, 0.14395E+01_JPRB, 0.14256E+01_JPRB, 0.14381E+01_JPRB ,&
&  0.14258E+01_JPRB, 0.14402E+01_JPRB, 0.14443E+01_JPRB, 0.14696E+01_JPRB, 0.14715E+01_JPRB, 0.14894E+01_JPRB, 0.15030E+01_JPRB ,&
&  0.14923E+01_JPRB, 0.14547E+01_JPRB, 0.13806E+01_JPRB, 0.13049E+01_JPRB, 0.12348E+01_JPRB, 0.11947E+01_JPRB, 0.11613E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.10981E+01_JPRB, 0.10658E+01_JPRB, 0.10196E+01_JPRB, 0.95830E+00_JPRB, 0.90813E+00_JPRB, 0.86798E+00_JPRB ,&
&  0.81302E+00_JPRB, 0.79145E+00_JPRB, 0.79244E+00_JPRB, 0.79742E+00_JPRB, 0.80930E+00_JPRB, 0.83227E+00_JPRB, 0.83046E+00_JPRB ,&
&  0.85417E+00_JPRB, 0.87278E+00_JPRB, 0.88391E+00_JPRB, 0.90732E+00_JPRB, 0.94058E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.10557E+01_JPRB, 0.10532E+01_JPRB, 0.10363E+01_JPRB, 0.10187E+01_JPRB, 0.99371E+00_JPRB, 0.94025E+00_JPRB, 0.89893E+00_JPRB ,&
&  0.91374E+00_JPRB, 0.87648E+00_JPRB, 0.85900E+00_JPRB, 0.80815E+00_JPRB, 0.78201E+00_JPRB, 0.76314E+00_JPRB, 0.74517E+00_JPRB ,&
&  0.74019E+00_JPRB, 0.74674E+00_JPRB, 0.75434E+00_JPRB, 0.76226E+00_JPRB, 0.76399E+00_JPRB, 0.76204E+00_JPRB, 0.75885E+00_JPRB ,&
&  0.75808E+00_JPRB, 0.76281E+00_JPRB, 0.76558E+00_JPRB, 0.76752E+00_JPRB, 0.76849E+00_JPRB, 0.76791E+00_JPRB, 0.76066E+00_JPRB ,&
&  0.75611E+00_JPRB, 0.75635E+00_JPRB, 0.76133E+00_JPRB, 0.75931E+00_JPRB, 0.76019E+00_JPRB, 0.75857E+00_JPRB, 0.75488E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.73605E+00_JPRB, 0.74374E+00_JPRB, 0.75599E+00_JPRB, 0.76993E+00_JPRB, 0.78138E+00_JPRB, 0.79170E+00_JPRB ,&
&  0.80435E+00_JPRB, 0.81920E+00_JPRB, 0.83632E+00_JPRB, 0.85361E+00_JPRB, 0.87306E+00_JPRB, 0.89583E+00_JPRB, 0.91412E+00_JPRB ,&
&  0.97276E+00_JPRB, 0.93324E+00_JPRB, 0.10622E+01_JPRB, 0.11797E+01_JPRB, 0.13754E+01_JPRB, 0.15378E+01_JPRB, 0.15827E+01_JPRB ,&
&  0.17316E+01_JPRB, 0.17690E+01_JPRB, 0.17933E+01_JPRB, 0.17736E+01_JPRB, 0.17467E+01_JPRB, 0.17401E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.17948E+01_JPRB, 0.17774E+01_JPRB, 0.18925E+01_JPRB, 0.18914E+01_JPRB, 0.18812E+01_JPRB, 0.18935E+01_JPRB, 0.18777E+01_JPRB ,&
&  0.18320E+01_JPRB, 0.17502E+01_JPRB, 0.16403E+01_JPRB, 0.14646E+01_JPRB, 0.13885E+01_JPRB, 0.13230E+01_JPRB, 0.12236E+01_JPRB ,&
&  0.11682E+01_JPRB, 0.11090E+01_JPRB, 0.10554E+01_JPRB, 0.10443E+01_JPRB, 0.96963E+00_JPRB, 0.91310E+00_JPRB, 0.87807E+00_JPRB ,&
&  0.83580E+00_JPRB, 0.82707E+00_JPRB, 0.84853E+00_JPRB, 0.87514E+00_JPRB, 0.89278E+00_JPRB, 0.89815E+00_JPRB, 0.92817E+00_JPRB ,&
&  0.91449E+00_JPRB, 0.94282E+00_JPRB, 0.10007E+01_JPRB, 0.10237E+01_JPRB, 0.10415E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.11663E+01_JPRB, 0.11540E+01_JPRB, 0.11312E+01_JPRB, 0.11035E+01_JPRB, 0.10531E+01_JPRB, 0.99023E+00_JPRB, 0.94014E+00_JPRB ,&
&  0.94279E+00_JPRB, 0.93005E+00_JPRB, 0.91188E+00_JPRB, 0.88445E+00_JPRB, 0.85273E+00_JPRB, 0.81046E+00_JPRB, 0.76368E+00_JPRB ,&
&  0.75185E+00_JPRB, 0.76515E+00_JPRB, 0.77737E+00_JPRB, 0.78705E+00_JPRB, 0.78562E+00_JPRB, 0.78323E+00_JPRB, 0.78318E+00_JPRB ,&
&  0.78543E+00_JPRB, 0.79020E+00_JPRB, 0.79651E+00_JPRB, 0.80125E+00_JPRB, 0.80175E+00_JPRB, 0.80226E+00_JPRB, 0.79953E+00_JPRB ,&
&  0.78034E+00_JPRB, 0.76849E+00_JPRB, 0.76359E+00_JPRB, 0.76394E+00_JPRB, 0.78655E+00_JPRB, 0.80052E+00_JPRB, 0.81092E+00_JPRB ,&
&  0.81243E+00_JPRB, 0.80969E+00_JPRB, 0.81117E+00_JPRB, 0.82359E+00_JPRB, 0.83576E+00_JPRB, 0.84172E+00_JPRB, 0.85169E+00_JPRB ,&
&  0.86715E+00_JPRB, 0.88588E+00_JPRB, 0.90690E+00_JPRB, 0.93451E+00_JPRB, 0.96785E+00_JPRB, 0.97097E+00_JPRB, 0.10215E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.12664E+01_JPRB, 0.14351E+01_JPRB, 0.14839E+01_JPRB, 0.16791E+01_JPRB, 0.18350E+01_JPRB, 0.18809E+01_JPRB ,&
&  0.20005E+01_JPRB, 0.20381E+01_JPRB, 0.21540E+01_JPRB, 0.21886E+01_JPRB, 0.22263E+01_JPRB, 0.22540E+01_JPRB, 0.22633E+01_JPRB ,&
&  0.22838E+01_JPRB, 0.23458E+01_JPRB, 0.24136E+01_JPRB, 0.24299E+01_JPRB, 0.23812E+01_JPRB, 0.23016E+01_JPRB, 0.21880E+01_JPRB ,&
&  0.20756E+01_JPRB, 0.19467E+01_JPRB, 0.18183E+01_JPRB, 0.17166E+01_JPRB, 0.15948E+01_JPRB, 0.14599E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.10589E+01_JPRB, 0.99739E+00_JPRB, 0.99394E+00_JPRB, 0.97429E+00_JPRB, 0.90631E+00_JPRB, 0.87098E+00_JPRB ,&
&  0.86373E+00_JPRB, 0.89294E+00_JPRB, 0.93658E+00_JPRB, 0.96400E+00_JPRB, 0.10132E+01_JPRB, 0.10502E+01_JPRB, 0.99721E+00_JPRB ,&
&  0.96476E+00_JPRB, 0.10505E+01_JPRB, 0.11059E+01_JPRB, 0.11395E+01_JPRB, 0.11610E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.12677E+01_JPRB, 0.12455E+01_JPRB, 0.12219E+01_JPRB, 0.11952E+01_JPRB, 0.11421E+01_JPRB, 0.10695E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.99611E+00_JPRB, 0.96951E+00_JPRB, 0.94259E+00_JPRB, 0.91711E+00_JPRB, 0.89324E+00_JPRB, 0.86418E+00_JPRB, 0.83300E+00_JPRB ,&
&  0.81239E+00_JPRB, 0.81732E+00_JPRB, 0.81112E+00_JPRB, 0.81010E+00_JPRB, 0.81740E+00_JPRB, 0.81298E+00_JPRB, 0.81327E+00_JPRB ,&
&  0.82302E+00_JPRB, 0.82810E+00_JPRB, 0.82901E+00_JPRB, 0.82720E+00_JPRB, 0.82713E+00_JPRB, 0.83083E+00_JPRB, 0.82859E+00_JPRB ,&
&  0.80287E+00_JPRB, 0.78254E+00_JPRB, 0.76864E+00_JPRB, 0.76864E+00_JPRB, 0.79574E+00_JPRB, 0.83637E+00_JPRB, 0.86804E+00_JPRB ,&
&  0.89133E+00_JPRB, 0.90848E+00_JPRB, 0.91502E+00_JPRB, 0.92033E+00_JPRB, 0.92042E+00_JPRB, 0.91857E+00_JPRB, 0.91979E+00_JPRB ,&
&  0.92611E+00_JPRB, 0.94035E+00_JPRB, 0.96140E+00_JPRB, 0.99952E+00_JPRB, 0.10279E+01_JPRB, 0.10654E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.13500E+01_JPRB, 0.15539E+01_JPRB, 0.16888E+01_JPRB, 0.19251E+01_JPRB, 0.21226E+01_JPRB, 0.22032E+01_JPRB, 0.22389E+01_JPRB ,&
&  0.23365E+01_JPRB, 0.23295E+01_JPRB, 0.25204E+01_JPRB, 0.25185E+01_JPRB, 0.25459E+01_JPRB, 0.26611E+01_JPRB, 0.26659E+01_JPRB ,&
&  0.26435E+01_JPRB, 0.26071E+01_JPRB, 0.27232E+01_JPRB, 0.27336E+01_JPRB, 0.26935E+01_JPRB, 0.26169E+01_JPRB, 0.24685E+01_JPRB ,&
&  0.23188E+01_JPRB, 0.21567E+01_JPRB, 0.18576E+01_JPRB, 0.16358E+01_JPRB, 0.14030E+01_JPRB, 0.13807E+01_JPRB, 0.12660E+01_JPRB ,&
&  0.11470E+01_JPRB, 0.10014E+01_JPRB, 0.99155E+00_JPRB, 0.99404E+00_JPRB, 0.10146E+01_JPRB, 0.98576E+00_JPRB, 0.93354E+00_JPRB ,&
&  0.93430E+00_JPRB, 0.98442E+00_JPRB, 0.10074E+01_JPRB, 0.10480E+01_JPRB, 0.11252E+01_JPRB, 0.12081E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10868E+01_JPRB, 0.11998E+01_JPRB, 0.12503E+01_JPRB, 0.12751E+01_JPRB, 0.12794E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.13053E+01_JPRB, 0.12708E+01_JPRB, 0.12389E+01_JPRB, 0.12121E+01_JPRB, 0.11752E+01_JPRB, 0.11334E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.10330E+01_JPRB, 0.99170E+00_JPRB, 0.96477E+00_JPRB, 0.94977E+00_JPRB, 0.93573E+00_JPRB, 0.90699E+00_JPRB, 0.90337E+00_JPRB ,&
&  0.87011E+00_JPRB, 0.85982E+00_JPRB, 0.82340E+00_JPRB, 0.82695E+00_JPRB, 0.86619E+00_JPRB, 0.84459E+00_JPRB, 0.84458E+00_JPRB ,&
&  0.84842E+00_JPRB, 0.85716E+00_JPRB, 0.85020E+00_JPRB, 0.84168E+00_JPRB, 0.83825E+00_JPRB, 0.83960E+00_JPRB, 0.84364E+00_JPRB ,&
&  0.82434E+00_JPRB, 0.78704E+00_JPRB, 0.76672E+00_JPRB, 0.76940E+00_JPRB, 0.80650E+00_JPRB, 0.85429E+00_JPRB, 0.90684E+00_JPRB ,&
&  0.93676E+00_JPRB, 0.96197E+00_JPRB, 0.97922E+00_JPRB, 0.98899E+00_JPRB, 0.98984E+00_JPRB, 0.99055E+00_JPRB, 0.99459E+00_JPRB ,&
&  0.99933E+00_JPRB, 0.10150E+01_JPRB, 0.10469E+01_JPRB, 0.10879E+01_JPRB, 0.11002E+01_JPRB, 0.11898E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.17975E+01_JPRB, 0.19215E+01_JPRB, 0.20535E+01_JPRB, 0.25109E+01_JPRB, 0.27655E+01_JPRB, 0.28455E+01_JPRB, 0.28684E+01_JPRB ,&
&  0.28994E+01_JPRB, 0.28816E+01_JPRB, 0.28970E+01_JPRB, 0.30147E+01_JPRB, 0.29998E+01_JPRB, 0.30102E+01_JPRB, 0.29351E+01_JPRB ,&
&  0.28187E+01_JPRB, 0.28266E+01_JPRB, 0.28685E+01_JPRB, 0.28094E+01_JPRB, 0.26991E+01_JPRB, 0.26616E+01_JPRB, 0.25918E+01_JPRB ,&
&  0.24557E+01_JPRB, 0.21710E+01_JPRB, 0.16371E+01_JPRB, 0.14562E+01_JPRB, 0.11480E+01_JPRB, 0.11470E+01_JPRB, 0.13111E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.11705E+01_JPRB, 0.11145E+01_JPRB, 0.10983E+01_JPRB, 0.11077E+01_JPRB, 0.11497E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10705E+01_JPRB, 0.11173E+01_JPRB, 0.12070E+01_JPRB, 0.13326E+01_JPRB, 0.14005E+01_JPRB, 0.12862E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.13831E+01_JPRB, 0.13902E+01_JPRB, 0.13831E+01_JPRB, 0.13502E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.12666E+01_JPRB, 0.12313E+01_JPRB, 0.11946E+01_JPRB, 0.11807E+01_JPRB, 0.11790E+01_JPRB, 0.11671E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.10157E+01_JPRB, 0.98714E+00_JPRB, 0.98207E+00_JPRB, 0.97909E+00_JPRB, 0.95210E+00_JPRB, 0.94104E+00_JPRB ,&
&  0.96124E+00_JPRB, 0.93568E+00_JPRB, 0.87285E+00_JPRB, 0.83007E+00_JPRB, 0.85443E+00_JPRB, 0.84914E+00_JPRB, 0.85560E+00_JPRB ,&
&  0.84886E+00_JPRB, 0.87161E+00_JPRB, 0.86610E+00_JPRB, 0.86358E+00_JPRB, 0.86539E+00_JPRB, 0.87694E+00_JPRB, 0.90944E+00_JPRB ,&
&  0.90731E+00_JPRB, 0.85277E+00_JPRB, 0.82100E+00_JPRB, 0.83378E+00_JPRB, 0.89702E+00_JPRB, 0.89248E+00_JPRB, 0.93799E+00_JPRB ,&
&  0.97240E+00_JPRB, 0.10060E+01_JPRB, 0.10334E+01_JPRB, 0.10578E+01_JPRB, 0.10751E+01_JPRB, 0.10870E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.11427E+01_JPRB, 0.11440E+01_JPRB, 0.11777E+01_JPRB, 0.12826E+01_JPRB, 0.15676E+01_JPRB, 0.19317E+01_JPRB ,&
&  0.21776E+01_JPRB, 0.22181E+01_JPRB, 0.22571E+01_JPRB, 0.26843E+01_JPRB, 0.30591E+01_JPRB, 0.32219E+01_JPRB, 0.33981E+01_JPRB ,&
&  0.34522E+01_JPRB, 0.34600E+01_JPRB, 0.34406E+01_JPRB, 0.32159E+01_JPRB, 0.28767E+01_JPRB, 0.26579E+01_JPRB, 0.25766E+01_JPRB ,&
&  0.25068E+01_JPRB, 0.25384E+01_JPRB, 0.26533E+01_JPRB, 0.26109E+01_JPRB, 0.25949E+01_JPRB, 0.24902E+01_JPRB, 0.26143E+01_JPRB ,&
&  0.25504E+01_JPRB, 0.15976E+01_JPRB, 0.12567E+01_JPRB, 0.13229E+01_JPRB, 0.11148E+01_JPRB, 0.11238E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.11983E+01_JPRB, 0.12456E+01_JPRB, 0.12722E+01_JPRB, 0.12839E+01_JPRB, 0.13767E+01_JPRB, 0.14037E+01_JPRB ,&
&  0.13545E+01_JPRB, 0.12835E+01_JPRB, 0.13593E+01_JPRB, 0.14913E+01_JPRB, 0.16209E+01_JPRB, 0.16318E+01_JPRB, 0.15815E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.15230E+01_JPRB, 0.14833E+01_JPRB, 0.14080E+01_JPRB, 0.13218E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.12854E+01_JPRB, 0.12528E+01_JPRB, 0.12105E+01_JPRB, 0.12090E+01_JPRB, 0.12245E+01_JPRB, 0.12133E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10242E+01_JPRB, 0.10019E+01_JPRB, 0.99814E+00_JPRB, 0.10052E+01_JPRB, 0.10013E+01_JPRB, 0.98810E+00_JPRB ,&
&  0.96431E+00_JPRB, 0.98199E+00_JPRB, 0.91289E+00_JPRB, 0.87186E+00_JPRB, 0.82189E+00_JPRB, 0.83777E+00_JPRB, 0.85716E+00_JPRB ,&
&  0.87479E+00_JPRB, 0.89015E+00_JPRB, 0.92578E+00_JPRB, 0.97370E+00_JPRB, 0.10175E+01_JPRB, 0.10556E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11159E+01_JPRB, 0.10308E+01_JPRB, 0.95302E+00_JPRB, 0.97058E+00_JPRB, 0.10051E+01_JPRB, 0.96659E+00_JPRB, 0.10304E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.11126E+01_JPRB, 0.11484E+01_JPRB, 0.11679E+01_JPRB, 0.11723E+01_JPRB, 0.11469E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11759E+01_JPRB, 0.12173E+01_JPRB, 0.12695E+01_JPRB, 0.15732E+01_JPRB, 0.20105E+01_JPRB, 0.23498E+01_JPRB ,&
&  0.25787E+01_JPRB, 0.22212E+01_JPRB, 0.19810E+01_JPRB, 0.25731E+01_JPRB, 0.36539E+01_JPRB, 0.40071E+01_JPRB, 0.37937E+01_JPRB ,&
&  0.40401E+01_JPRB, 0.37258E+01_JPRB, 0.32720E+01_JPRB, 0.29579E+01_JPRB, 0.27269E+01_JPRB, 0.25594E+01_JPRB, 0.24914E+01_JPRB ,&
&  0.23564E+01_JPRB, 0.23652E+01_JPRB, 0.24991E+01_JPRB, 0.25815E+01_JPRB, 0.25687E+01_JPRB, 0.25549E+01_JPRB, 0.26355E+01_JPRB ,&
&  0.24756E+01_JPRB, 0.20246E+01_JPRB, 0.14502E+01_JPRB, 0.13426E+01_JPRB, 0.11061E+01_JPRB, 0.10894E+01_JPRB, 0.12133E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.12798E+01_JPRB, 0.13274E+01_JPRB, 0.13336E+01_JPRB, 0.14875E+01_JPRB, 0.15925E+01_JPRB, 0.17554E+01_JPRB ,&
&  0.18584E+01_JPRB, 0.16334E+01_JPRB, 0.17650E+01_JPRB, 0.19107E+01_JPRB, 0.19518E+01_JPRB, 0.18495E+01_JPRB, 0.17274E+01_JPRB ,&
&  0.16518E+01_JPRB, 0.15939E+01_JPRB, 0.15158E+01_JPRB, 0.14049E+01_JPRB, 0.13271E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.13141E+01_JPRB, 0.12988E+01_JPRB, 0.12535E+01_JPRB, 0.12504E+01_JPRB, 0.12437E+01_JPRB, 0.12091E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.10914E+01_JPRB, 0.10687E+01_JPRB, 0.10540E+01_JPRB, 0.10510E+01_JPRB, 0.10409E+01_JPRB, 0.10172E+01_JPRB ,&
&  0.98366E+00_JPRB, 0.97032E+00_JPRB, 0.88997E+00_JPRB, 0.84106E+00_JPRB, 0.81306E+00_JPRB, 0.82343E+00_JPRB, 0.85731E+00_JPRB ,&
&  0.91357E+00_JPRB, 0.99383E+00_JPRB, 0.11258E+01_JPRB, 0.12476E+01_JPRB, 0.14104E+01_JPRB, 0.15341E+01_JPRB, 0.16090E+01_JPRB ,&
&  0.15982E+01_JPRB, 0.12900E+01_JPRB, 0.12462E+01_JPRB, 0.11767E+01_JPRB, 0.11776E+01_JPRB, 0.11500E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.11580E+01_JPRB, 0.11828E+01_JPRB, 0.11982E+01_JPRB, 0.11825E+01_JPRB, 0.11637E+01_JPRB, 0.11572E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.12552E+01_JPRB, 0.12727E+01_JPRB, 0.13298E+01_JPRB, 0.15794E+01_JPRB, 0.21035E+01_JPRB, 0.25927E+01_JPRB, 0.29091E+01_JPRB ,&
&  0.27525E+01_JPRB, 0.25904E+01_JPRB, 0.34635E+01_JPRB, 0.38295E+01_JPRB, 0.36369E+01_JPRB, 0.41866E+01_JPRB, 0.42968E+01_JPRB ,&
&  0.39061E+01_JPRB, 0.31895E+01_JPRB, 0.28579E+01_JPRB, 0.26244E+01_JPRB, 0.30705E+01_JPRB, 0.28941E+01_JPRB, 0.25762E+01_JPRB ,&
&  0.23092E+01_JPRB, 0.24546E+01_JPRB, 0.25217E+01_JPRB, 0.26434E+01_JPRB, 0.28204E+01_JPRB, 0.29232E+01_JPRB, 0.26684E+01_JPRB ,&
&  0.23469E+01_JPRB, 0.24817E+01_JPRB, 0.19895E+01_JPRB, 0.15190E+01_JPRB, 0.12966E+01_JPRB, 0.12887E+01_JPRB, 0.13357E+01_JPRB ,&
&  0.13869E+01_JPRB, 0.14258E+01_JPRB, 0.14461E+01_JPRB, 0.15601E+01_JPRB, 0.18196E+01_JPRB, 0.19832E+01_JPRB, 0.20786E+01_JPRB ,&
&  0.20706E+01_JPRB, 0.22799E+01_JPRB, 0.23177E+01_JPRB, 0.22809E+01_JPRB, 0.20949E+01_JPRB, 0.18570E+01_JPRB, 0.17162E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.16036E+01_JPRB, 0.14967E+01_JPRB, 0.13853E+01_JPRB, 0.13180E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.13342E+01_JPRB, 0.13342E+01_JPRB, 0.13197E+01_JPRB, 0.12954E+01_JPRB, 0.12636E+01_JPRB, 0.12281E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.11336E+01_JPRB, 0.11270E+01_JPRB, 0.11305E+01_JPRB, 0.11248E+01_JPRB, 0.10993E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10385E+01_JPRB, 0.96581E+00_JPRB, 0.87235E+00_JPRB, 0.86988E+00_JPRB, 0.86564E+00_JPRB, 0.84727E+00_JPRB, 0.89353E+00_JPRB ,&
&  0.99194E+00_JPRB, 0.12027E+01_JPRB, 0.14434E+01_JPRB, 0.16950E+01_JPRB, 0.18402E+01_JPRB, 0.18790E+01_JPRB, 0.18744E+01_JPRB ,&
&  0.17252E+01_JPRB, 0.16298E+01_JPRB, 0.15547E+01_JPRB, 0.14571E+01_JPRB, 0.14016E+01_JPRB, 0.13629E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.12618E+01_JPRB, 0.12386E+01_JPRB, 0.12304E+01_JPRB, 0.11979E+01_JPRB, 0.12007E+01_JPRB, 0.12378E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.13569E+01_JPRB, 0.15217E+01_JPRB, 0.19454E+01_JPRB, 0.21725E+01_JPRB, 0.25659E+01_JPRB, 0.29360E+01_JPRB ,&
&  0.38881E+01_JPRB, 0.41756E+01_JPRB, 0.38080E+01_JPRB, 0.45145E+01_JPRB, 0.30136E+01_JPRB, 0.34798E+01_JPRB, 0.40103E+01_JPRB ,&
&  0.37041E+01_JPRB, 0.35372E+01_JPRB, 0.37487E+01_JPRB, 0.33592E+01_JPRB, 0.28861E+01_JPRB, 0.38976E+01_JPRB, 0.32074E+01_JPRB ,&
&  0.27307E+01_JPRB, 0.27364E+01_JPRB, 0.29033E+01_JPRB, 0.32743E+01_JPRB, 0.28847E+01_JPRB, 0.25871E+01_JPRB, 0.21019E+01_JPRB ,&
&  0.20876E+01_JPRB, 0.18932E+01_JPRB, 0.20267E+01_JPRB, 0.17746E+01_JPRB, 0.17583E+01_JPRB, 0.16271E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.15886E+01_JPRB, 0.15985E+01_JPRB, 0.16299E+01_JPRB, 0.19248E+01_JPRB, 0.24308E+01_JPRB, 0.22753E+01_JPRB, 0.24739E+01_JPRB ,&
&  0.26375E+01_JPRB, 0.26907E+01_JPRB, 0.24679E+01_JPRB, 0.22830E+01_JPRB, 0.19691E+01_JPRB, 0.17809E+01_JPRB, 0.17012E+01_JPRB ,&
&  0.16415E+01_JPRB, 0.15335E+01_JPRB, 0.14111E+01_JPRB, 0.13599E+01_JPRB, 0.13116E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.13368E+01_JPRB, 0.13181E+01_JPRB, 0.13092E+01_JPRB, 0.12838E+01_JPRB, 0.12739E+01_JPRB, 0.12485E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.11701E+01_JPRB, 0.11684E+01_JPRB, 0.11657E+01_JPRB, 0.11550E+01_JPRB, 0.11422E+01_JPRB, 0.11054E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10383E+01_JPRB, 0.91358E+00_JPRB, 0.87842E+00_JPRB, 0.89705E+00_JPRB, 0.89772E+00_JPRB, 0.94639E+00_JPRB ,&
&  0.12240E+01_JPRB, 0.16246E+01_JPRB, 0.18727E+01_JPRB, 0.20316E+01_JPRB, 0.20765E+01_JPRB, 0.19728E+01_JPRB, 0.18709E+01_JPRB ,&
&  0.18338E+01_JPRB, 0.16411E+01_JPRB, 0.15229E+01_JPRB, 0.14867E+01_JPRB, 0.14294E+01_JPRB, 0.13675E+01_JPRB, 0.13163E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12092E+01_JPRB, 0.11772E+01_JPRB, 0.11497E+01_JPRB, 0.11761E+01_JPRB, 0.12344E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.14821E+01_JPRB, 0.18046E+01_JPRB, 0.22352E+01_JPRB, 0.25953E+01_JPRB, 0.29802E+01_JPRB, 0.38291E+01_JPRB ,&
&  0.42625E+01_JPRB, 0.41334E+01_JPRB, 0.42005E+01_JPRB, 0.43772E+01_JPRB, 0.41028E+01_JPRB, 0.35950E+01_JPRB, 0.38812E+01_JPRB ,&
&  0.30616E+01_JPRB, 0.27781E+01_JPRB, 0.27345E+01_JPRB, 0.28101E+01_JPRB, 0.35728E+01_JPRB, 0.53732E+01_JPRB, 0.47901E+01_JPRB ,&
&  0.42791E+01_JPRB, 0.37160E+01_JPRB, 0.37946E+01_JPRB, 0.35435E+01_JPRB, 0.31117E+01_JPRB, 0.23395E+01_JPRB, 0.28488E+01_JPRB ,&
&  0.29545E+01_JPRB, 0.30527E+01_JPRB, 0.27614E+01_JPRB, 0.22412E+01_JPRB, 0.18747E+01_JPRB, 0.20054E+01_JPRB, 0.19731E+01_JPRB ,&
&  0.19624E+01_JPRB, 0.19246E+01_JPRB, 0.23997E+01_JPRB, 0.27442E+01_JPRB, 0.29441E+01_JPRB, 0.29352E+01_JPRB, 0.32075E+01_JPRB ,&
&  0.30057E+01_JPRB, 0.28931E+01_JPRB, 0.25654E+01_JPRB, 0.21440E+01_JPRB, 0.18755E+01_JPRB, 0.17510E+01_JPRB, 0.16586E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.14208E+01_JPRB, 0.13499E+01_JPRB, 0.13072E+01_JPRB, 0.12940E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.13163E+01_JPRB, 0.12973E+01_JPRB, 0.12666E+01_JPRB, 0.12504E+01_JPRB, 0.12329E+01_JPRB, 0.12028E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.12156E+01_JPRB, 0.11904E+01_JPRB, 0.11570E+01_JPRB, 0.11339E+01_JPRB, 0.11216E+01_JPRB, 0.11479E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.11746E+01_JPRB, 0.11636E+01_JPRB, 0.10004E+01_JPRB, 0.95984E+00_JPRB, 0.97332E+00_JPRB, 0.10203E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.15724E+01_JPRB, 0.20226E+01_JPRB, 0.20996E+01_JPRB, 0.21192E+01_JPRB, 0.20824E+01_JPRB, 0.19475E+01_JPRB, 0.18857E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.17138E+01_JPRB, 0.15673E+01_JPRB, 0.14781E+01_JPRB, 0.13919E+01_JPRB, 0.13340E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.11876E+01_JPRB, 0.11595E+01_JPRB, 0.11636E+01_JPRB, 0.11739E+01_JPRB, 0.12828E+01_JPRB, 0.14087E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.17826E+01_JPRB, 0.21026E+01_JPRB, 0.25369E+01_JPRB, 0.30367E+01_JPRB, 0.35388E+01_JPRB, 0.49216E+01_JPRB ,&
&  0.47879E+01_JPRB, 0.47911E+01_JPRB, 0.46443E+01_JPRB, 0.52517E+01_JPRB, 0.40278E+01_JPRB, 0.34758E+01_JPRB, 0.36491E+01_JPRB ,&
&  0.28679E+01_JPRB, 0.26876E+01_JPRB, 0.31936E+01_JPRB, 0.41667E+01_JPRB, 0.43398E+01_JPRB, 0.48120E+01_JPRB, 0.63791E+01_JPRB ,&
&  0.56410E+01_JPRB, 0.49069E+01_JPRB, 0.46212E+01_JPRB, 0.45584E+01_JPRB, 0.23792E+01_JPRB, 0.19262E+01_JPRB, 0.29487E+01_JPRB ,&
&  0.30920E+01_JPRB, 0.21021E+01_JPRB, 0.16129E+01_JPRB, 0.19285E+01_JPRB, 0.15078E+01_JPRB, 0.17223E+01_JPRB, 0.22686E+01_JPRB ,&
&  0.23566E+01_JPRB, 0.25254E+01_JPRB, 0.36415E+01_JPRB, 0.35652E+01_JPRB, 0.34973E+01_JPRB, 0.45159E+01_JPRB, 0.38825E+01_JPRB ,&
&  0.34092E+01_JPRB, 0.26499E+01_JPRB, 0.23556E+01_JPRB, 0.19884E+01_JPRB, 0.17825E+01_JPRB, 0.16320E+01_JPRB, 0.15183E+01_JPRB ,&
&  0.13989E+01_JPRB, 0.13308E+01_JPRB, 0.13271E+01_JPRB, 0.13241E+01_JPRB, 0.13022E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.13023E+01_JPRB, 0.12591E+01_JPRB, 0.12096E+01_JPRB, 0.12028E+01_JPRB, 0.11777E+01_JPRB, 0.11487E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.11798E+01_JPRB, 0.11109E+01_JPRB, 0.10931E+01_JPRB, 0.11080E+01_JPRB, 0.12069E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.12713E+01_JPRB, 0.12101E+01_JPRB, 0.11973E+01_JPRB, 0.12359E+01_JPRB, 0.13088E+01_JPRB, 0.15190E+01_JPRB ,&
&  0.21034E+01_JPRB, 0.22640E+01_JPRB, 0.21017E+01_JPRB, 0.19990E+01_JPRB, 0.19605E+01_JPRB, 0.18779E+01_JPRB, 0.19207E+01_JPRB ,&
&  0.18409E+01_JPRB, 0.16772E+01_JPRB, 0.15527E+01_JPRB, 0.14612E+01_JPRB, 0.14176E+01_JPRB, 0.13804E+01_JPRB, 0.13354E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.12014E+01_JPRB, 0.11953E+01_JPRB, 0.11840E+01_JPRB, 0.12450E+01_JPRB, 0.13861E+01_JPRB, 0.15524E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.20686E+01_JPRB, 0.23937E+01_JPRB, 0.29198E+01_JPRB, 0.38070E+01_JPRB, 0.42870E+01_JPRB, 0.40029E+01_JPRB ,&
&  0.36871E+01_JPRB, 0.36500E+01_JPRB, 0.47282E+01_JPRB, 0.47729E+01_JPRB, 0.41454E+01_JPRB, 0.38291E+01_JPRB, 0.36208E+01_JPRB ,&
&  0.32614E+01_JPRB, 0.31974E+01_JPRB, 0.37000E+01_JPRB, 0.48390E+01_JPRB, 0.54927E+01_JPRB, 0.52098E+01_JPRB, 0.66489E+01_JPRB ,&
&  0.67297E+01_JPRB, 0.58606E+01_JPRB, 0.40661E+01_JPRB, 0.30309E+01_JPRB, 0.27285E+01_JPRB, 0.23800E+01_JPRB, 0.17118E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.15381E+01_JPRB, 0.14445E+01_JPRB, 0.14630E+01_JPRB, 0.14174E+01_JPRB, 0.17825E+01_JPRB, 0.22884E+01_JPRB ,&
&  0.30386E+01_JPRB, 0.36455E+01_JPRB, 0.41417E+01_JPRB, 0.41119E+01_JPRB, 0.39380E+01_JPRB, 0.53014E+01_JPRB, 0.41513E+01_JPRB ,&
&  0.28051E+01_JPRB, 0.22915E+01_JPRB, 0.20530E+01_JPRB, 0.18207E+01_JPRB, 0.16471E+01_JPRB, 0.14896E+01_JPRB, 0.13682E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13043E+01_JPRB, 0.13149E+01_JPRB, 0.13154E+01_JPRB, 0.12813E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.12874E+01_JPRB, 0.12442E+01_JPRB, 0.11970E+01_JPRB, 0.11591E+01_JPRB, 0.11107E+01_JPRB, 0.11298E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.11672E+01_JPRB, 0.10867E+01_JPRB, 0.10919E+01_JPRB, 0.11873E+01_JPRB, 0.12976E+01_JPRB, 0.13354E+01_JPRB ,&
&  0.13940E+01_JPRB, 0.13931E+01_JPRB, 0.14674E+01_JPRB, 0.15742E+01_JPRB, 0.17802E+01_JPRB, 0.18560E+01_JPRB, 0.22686E+01_JPRB ,&
&  0.26124E+01_JPRB, 0.23078E+01_JPRB, 0.20114E+01_JPRB, 0.19209E+01_JPRB, 0.19184E+01_JPRB, 0.19039E+01_JPRB, 0.18961E+01_JPRB ,&
&  0.17486E+01_JPRB, 0.16156E+01_JPRB, 0.15020E+01_JPRB, 0.14470E+01_JPRB, 0.14263E+01_JPRB, 0.13573E+01_JPRB, 0.13063E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12300E+01_JPRB, 0.12164E+01_JPRB, 0.12550E+01_JPRB, 0.13934E+01_JPRB, 0.15861E+01_JPRB, 0.18072E+01_JPRB ,&
&  0.20590E+01_JPRB, 0.24172E+01_JPRB, 0.28124E+01_JPRB, 0.34424E+01_JPRB, 0.34773E+01_JPRB, 0.30284E+01_JPRB, 0.34617E+01_JPRB ,&
&  0.38042E+01_JPRB, 0.41256E+01_JPRB, 0.40190E+01_JPRB, 0.47834E+01_JPRB, 0.43916E+01_JPRB, 0.37230E+01_JPRB, 0.34945E+01_JPRB ,&
&  0.32058E+01_JPRB, 0.39507E+01_JPRB, 0.47084E+01_JPRB, 0.54321E+01_JPRB, 0.72327E+01_JPRB, 0.69375E+01_JPRB, 0.75098E+01_JPRB ,&
&  0.77885E+01_JPRB, 0.64256E+01_JPRB, 0.46674E+01_JPRB, 0.35227E+01_JPRB, 0.41866E+01_JPRB, 0.47324E+01_JPRB, 0.21085E+01_JPRB ,&
&  0.17232E+01_JPRB, 0.15997E+01_JPRB, 0.15617E+01_JPRB, 0.14609E+01_JPRB, 0.14517E+01_JPRB, 0.18500E+01_JPRB, 0.31752E+01_JPRB ,&
&  0.31469E+01_JPRB, 0.47418E+01_JPRB, 0.46586E+01_JPRB, 0.45422E+01_JPRB, 0.41689E+01_JPRB, 0.50668E+01_JPRB, 0.35249E+01_JPRB ,&
&  0.27972E+01_JPRB, 0.23528E+01_JPRB, 0.20022E+01_JPRB, 0.17966E+01_JPRB, 0.15887E+01_JPRB, 0.14078E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.12907E+01_JPRB, 0.12586E+01_JPRB, 0.12523E+01_JPRB, 0.12553E+01_JPRB, 0.12624E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.12767E+01_JPRB, 0.12316E+01_JPRB, 0.11956E+01_JPRB, 0.11654E+01_JPRB, 0.11312E+01_JPRB, 0.12070E+01_JPRB, 0.12851E+01_JPRB ,&
&  0.12950E+01_JPRB, 0.12089E+01_JPRB, 0.11549E+01_JPRB, 0.11902E+01_JPRB, 0.12941E+01_JPRB, 0.14166E+01_JPRB, 0.14929E+01_JPRB ,&
&  0.15772E+01_JPRB, 0.16646E+01_JPRB, 0.18258E+01_JPRB, 0.22349E+01_JPRB, 0.25393E+01_JPRB, 0.27137E+01_JPRB, 0.30751E+01_JPRB ,&
&  0.27524E+01_JPRB, 0.21411E+01_JPRB, 0.19536E+01_JPRB, 0.19534E+01_JPRB, 0.18741E+01_JPRB, 0.18094E+01_JPRB, 0.17565E+01_JPRB ,&
&  0.16575E+01_JPRB, 0.15755E+01_JPRB, 0.15058E+01_JPRB, 0.15093E+01_JPRB, 0.14970E+01_JPRB, 0.14208E+01_JPRB, 0.13824E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.13248E+01_JPRB, 0.13553E+01_JPRB, 0.14055E+01_JPRB, 0.16018E+01_JPRB, 0.18017E+01_JPRB, 0.20612E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.27253E+01_JPRB, 0.30973E+01_JPRB, 0.35378E+01_JPRB, 0.30254E+01_JPRB, 0.36234E+01_JPRB, 0.38298E+01_JPRB ,&
&  0.40093E+01_JPRB, 0.38552E+01_JPRB, 0.34701E+01_JPRB, 0.39617E+01_JPRB, 0.40852E+01_JPRB, 0.35300E+01_JPRB, 0.32668E+01_JPRB ,&
&  0.38446E+01_JPRB, 0.45222E+01_JPRB, 0.57815E+01_JPRB, 0.67955E+01_JPRB, 0.78338E+01_JPRB, 0.10849E+02_JPRB, 0.76937E+01_JPRB ,&
&  0.64517E+01_JPRB, 0.63546E+01_JPRB, 0.53220E+01_JPRB, 0.44996E+01_JPRB, 0.50809E+01_JPRB, 0.74653E+01_JPRB, 0.53980E+01_JPRB ,&
&  0.20318E+01_JPRB, 0.15511E+01_JPRB, 0.17446E+01_JPRB, 0.17695E+01_JPRB, 0.15564E+01_JPRB, 0.19141E+01_JPRB, 0.51262E+01_JPRB ,&
&  0.39766E+01_JPRB, 0.54680E+01_JPRB, 0.47814E+01_JPRB, 0.44325E+01_JPRB, 0.43367E+01_JPRB, 0.46073E+01_JPRB, 0.37452E+01_JPRB ,&
&  0.27651E+01_JPRB, 0.23025E+01_JPRB, 0.19628E+01_JPRB, 0.17217E+01_JPRB, 0.15024E+01_JPRB, 0.13656E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.12561E+01_JPRB, 0.12436E+01_JPRB, 0.12408E+01_JPRB, 0.12416E+01_JPRB, 0.12598E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.12810E+01_JPRB, 0.12072E+01_JPRB, 0.11734E+01_JPRB, 0.11472E+01_JPRB, 0.12075E+01_JPRB, 0.12943E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.13904E+01_JPRB, 0.13926E+01_JPRB, 0.13898E+01_JPRB, 0.14751E+01_JPRB, 0.16452E+01_JPRB, 0.17864E+01_JPRB ,&
&  0.19631E+01_JPRB, 0.21610E+01_JPRB, 0.24301E+01_JPRB, 0.28624E+01_JPRB, 0.33022E+01_JPRB, 0.32810E+01_JPRB, 0.31673E+01_JPRB ,&
&  0.26527E+01_JPRB, 0.21109E+01_JPRB, 0.18714E+01_JPRB, 0.19519E+01_JPRB, 0.19074E+01_JPRB, 0.18508E+01_JPRB, 0.18505E+01_JPRB ,&
&  0.18384E+01_JPRB, 0.17789E+01_JPRB, 0.17144E+01_JPRB, 0.17190E+01_JPRB, 0.16334E+01_JPRB, 0.16119E+01_JPRB, 0.16992E+01_JPRB ,&
&  0.16623E+01_JPRB, 0.16548E+01_JPRB, 0.16751E+01_JPRB, 0.17398E+01_JPRB, 0.19485E+01_JPRB, 0.21527E+01_JPRB, 0.24249E+01_JPRB ,&
&  0.27296E+01_JPRB, 0.29737E+01_JPRB, 0.32931E+01_JPRB, 0.33605E+01_JPRB, 0.34017E+01_JPRB, 0.35599E+01_JPRB, 0.38834E+01_JPRB ,&
&  0.39592E+01_JPRB, 0.37816E+01_JPRB, 0.35495E+01_JPRB, 0.35495E+01_JPRB, 0.35432E+01_JPRB, 0.33730E+01_JPRB, 0.34952E+01_JPRB ,&
&  0.45898E+01_JPRB, 0.60725E+01_JPRB, 0.71734E+01_JPRB, 0.81304E+01_JPRB, 0.87086E+01_JPRB, 0.10945E+02_JPRB, 0.10008E+02_JPRB ,&
&  0.76473E+01_JPRB, 0.59139E+01_JPRB, 0.54692E+01_JPRB, 0.60649E+01_JPRB, 0.65586E+01_JPRB, 0.73599E+01_JPRB, 0.84739E+01_JPRB ,&
&  0.83373E+01_JPRB, 0.44280E+01_JPRB, 0.32605E+01_JPRB, 0.45786E+01_JPRB, 0.30233E+01_JPRB, 0.21528E+01_JPRB, 0.30470E+01_JPRB ,&
&  0.42684E+01_JPRB, 0.51244E+01_JPRB, 0.48769E+01_JPRB, 0.43220E+01_JPRB, 0.43899E+01_JPRB, 0.40869E+01_JPRB, 0.32378E+01_JPRB ,&
&  0.24767E+01_JPRB, 0.20159E+01_JPRB, 0.17361E+01_JPRB, 0.15453E+01_JPRB, 0.13731E+01_JPRB, 0.13029E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.12615E+01_JPRB, 0.12773E+01_JPRB, 0.12792E+01_JPRB, 0.12678E+01_JPRB, 0.12843E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.12419E+01_JPRB, 0.11825E+01_JPRB, 0.11505E+01_JPRB, 0.11640E+01_JPRB, 0.12744E+01_JPRB, 0.13918E+01_JPRB, 0.14465E+01_JPRB ,&
&  0.15036E+01_JPRB, 0.15449E+01_JPRB, 0.16057E+01_JPRB, 0.16144E+01_JPRB, 0.17789E+01_JPRB, 0.20797E+01_JPRB, 0.23330E+01_JPRB ,&
&  0.25414E+01_JPRB, 0.27625E+01_JPRB, 0.30307E+01_JPRB, 0.34734E+01_JPRB, 0.41066E+01_JPRB, 0.49116E+01_JPRB, 0.29032E+01_JPRB ,&
&  0.23893E+01_JPRB, 0.22011E+01_JPRB, 0.19517E+01_JPRB, 0.21390E+01_JPRB, 0.20937E+01_JPRB, 0.20904E+01_JPRB, 0.20169E+01_JPRB ,&
&  0.21006E+01_JPRB, 0.20805E+01_JPRB, 0.20631E+01_JPRB, 0.21231E+01_JPRB, 0.20691E+01_JPRB, 0.22379E+01_JPRB, 0.23872E+01_JPRB ,&
&  0.23371E+01_JPRB, 0.23121E+01_JPRB, 0.23946E+01_JPRB, 0.24858E+01_JPRB, 0.26137E+01_JPRB, 0.27409E+01_JPRB, 0.29085E+01_JPRB ,&
&  0.30874E+01_JPRB, 0.32944E+01_JPRB, 0.35860E+01_JPRB, 0.34034E+01_JPRB, 0.33666E+01_JPRB, 0.34421E+01_JPRB, 0.36813E+01_JPRB ,&
&  0.35112E+01_JPRB, 0.31249E+01_JPRB, 0.31935E+01_JPRB, 0.33710E+01_JPRB, 0.33794E+01_JPRB, 0.34163E+01_JPRB, 0.42728E+01_JPRB ,&
&  0.60749E+01_JPRB, 0.78788E+01_JPRB, 0.98665E+01_JPRB, 0.89737E+01_JPRB, 0.90609E+01_JPRB, 0.97872E+01_JPRB, 0.93756E+01_JPRB ,&
&  0.85703E+01_JPRB, 0.71812E+01_JPRB, 0.67160E+01_JPRB, 0.68704E+01_JPRB, 0.66200E+01_JPRB, 0.63245E+01_JPRB, 0.64034E+01_JPRB ,&
&  0.77835E+01_JPRB, 0.97012E+01_JPRB, 0.81370E+01_JPRB, 0.51107E+01_JPRB, 0.35939E+01_JPRB, 0.25490E+01_JPRB, 0.29222E+01_JPRB ,&
&  0.45422E+01_JPRB, 0.44767E+01_JPRB, 0.42425E+01_JPRB, 0.47401E+01_JPRB, 0.38044E+01_JPRB, 0.31925E+01_JPRB, 0.24609E+01_JPRB ,&
&  0.19098E+01_JPRB, 0.15216E+01_JPRB, 0.13830E+01_JPRB, 0.12612E+01_JPRB, 0.12378E+01_JPRB, 0.12508E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.13289E+01_JPRB, 0.13106E+01_JPRB, 0.12872E+01_JPRB, 0.12844E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.12406E+01_JPRB, 0.11977E+01_JPRB, 0.11703E+01_JPRB, 0.12171E+01_JPRB, 0.13424E+01_JPRB, 0.14477E+01_JPRB, 0.14941E+01_JPRB ,&
&  0.15347E+01_JPRB, 0.15843E+01_JPRB, 0.17185E+01_JPRB, 0.18532E+01_JPRB, 0.21482E+01_JPRB, 0.24680E+01_JPRB, 0.27602E+01_JPRB ,&
&  0.29865E+01_JPRB, 0.31699E+01_JPRB, 0.34251E+01_JPRB, 0.38085E+01_JPRB, 0.42328E+01_JPRB, 0.46001E+01_JPRB, 0.34950E+01_JPRB ,&
&  0.21459E+01_JPRB, 0.27786E+01_JPRB, 0.22018E+01_JPRB, 0.24632E+01_JPRB, 0.25760E+01_JPRB, 0.28195E+01_JPRB, 0.26374E+01_JPRB ,&
&  0.26420E+01_JPRB, 0.25611E+01_JPRB, 0.26093E+01_JPRB, 0.27010E+01_JPRB, 0.26481E+01_JPRB, 0.28438E+01_JPRB, 0.30084E+01_JPRB ,&
&  0.30944E+01_JPRB, 0.31929E+01_JPRB, 0.32258E+01_JPRB, 0.32116E+01_JPRB, 0.32248E+01_JPRB, 0.32638E+01_JPRB, 0.33410E+01_JPRB ,&
&  0.35209E+01_JPRB, 0.37559E+01_JPRB, 0.38158E+01_JPRB, 0.37515E+01_JPRB, 0.35123E+01_JPRB, 0.35630E+01_JPRB, 0.36631E+01_JPRB ,&
&  0.33731E+01_JPRB, 0.29970E+01_JPRB, 0.31399E+01_JPRB, 0.33079E+01_JPRB, 0.35969E+01_JPRB, 0.46858E+01_JPRB, 0.59004E+01_JPRB ,&
&  0.77461E+01_JPRB, 0.86367E+01_JPRB, 0.11024E+02_JPRB, 0.82232E+01_JPRB, 0.78538E+01_JPRB, 0.82858E+01_JPRB, 0.73309E+01_JPRB ,&
&  0.64368E+01_JPRB, 0.64842E+01_JPRB, 0.63447E+01_JPRB, 0.67447E+01_JPRB, 0.66330E+01_JPRB, 0.59705E+01_JPRB, 0.52455E+01_JPRB ,&
&  0.54756E+01_JPRB, 0.58417E+01_JPRB, 0.49959E+01_JPRB, 0.33487E+01_JPRB, 0.28779E+01_JPRB, 0.27828E+01_JPRB, 0.37587E+01_JPRB ,&
&  0.44089E+01_JPRB, 0.43128E+01_JPRB, 0.42340E+01_JPRB, 0.39664E+01_JPRB, 0.27178E+01_JPRB, 0.21110E+01_JPRB, 0.15509E+01_JPRB ,&
&  0.11908E+01_JPRB, 0.10531E+01_JPRB, 0.11706E+01_JPRB, 0.11468E+01_JPRB, 0.11785E+01_JPRB, 0.12287E+01_JPRB, 0.12467E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12835E+01_JPRB, 0.12904E+01_JPRB, 0.12778E+01_JPRB, 0.12707E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.11473E+01_JPRB, 0.11330E+01_JPRB, 0.11292E+01_JPRB, 0.12325E+01_JPRB, 0.14064E+01_JPRB, 0.15558E+01_JPRB, 0.15841E+01_JPRB ,&
&  0.16179E+01_JPRB, 0.15258E+01_JPRB, 0.17303E+01_JPRB, 0.19258E+01_JPRB, 0.22126E+01_JPRB, 0.25404E+01_JPRB, 0.29062E+01_JPRB ,&
&  0.32639E+01_JPRB, 0.35956E+01_JPRB, 0.38856E+01_JPRB, 0.42196E+01_JPRB, 0.46581E+01_JPRB, 0.51140E+01_JPRB, 0.45822E+01_JPRB ,&
&  0.33084E+01_JPRB, 0.24381E+01_JPRB, 0.24652E+01_JPRB, 0.25756E+01_JPRB, 0.26826E+01_JPRB, 0.29576E+01_JPRB, 0.30950E+01_JPRB ,&
&  0.31739E+01_JPRB, 0.29079E+01_JPRB, 0.28951E+01_JPRB, 0.30247E+01_JPRB, 0.29006E+01_JPRB, 0.31028E+01_JPRB, 0.33361E+01_JPRB ,&
&  0.34862E+01_JPRB, 0.36051E+01_JPRB, 0.36345E+01_JPRB, 0.36190E+01_JPRB, 0.36336E+01_JPRB, 0.37115E+01_JPRB, 0.38396E+01_JPRB ,&
&  0.40802E+01_JPRB, 0.43748E+01_JPRB, 0.45360E+01_JPRB, 0.42948E+01_JPRB, 0.41697E+01_JPRB, 0.42421E+01_JPRB, 0.41698E+01_JPRB ,&
&  0.39241E+01_JPRB, 0.38855E+01_JPRB, 0.41922E+01_JPRB, 0.47932E+01_JPRB, 0.51837E+01_JPRB, 0.65064E+01_JPRB, 0.73357E+01_JPRB ,&
&  0.82727E+01_JPRB, 0.88049E+01_JPRB, 0.98091E+01_JPRB, 0.69947E+01_JPRB, 0.69595E+01_JPRB, 0.68964E+01_JPRB, 0.63290E+01_JPRB ,&
&  0.59711E+01_JPRB, 0.61456E+01_JPRB, 0.62270E+01_JPRB, 0.62248E+01_JPRB, 0.61326E+01_JPRB, 0.51386E+01_JPRB, 0.48372E+01_JPRB ,&
&  0.44797E+01_JPRB, 0.48011E+01_JPRB, 0.37903E+01_JPRB, 0.32786E+01_JPRB, 0.28546E+01_JPRB, 0.30330E+01_JPRB, 0.33666E+01_JPRB ,&
&  0.38283E+01_JPRB, 0.36956E+01_JPRB, 0.32642E+01_JPRB, 0.24997E+01_JPRB, 0.15249E+01_JPRB, 0.11380E+01_JPRB, 0.96791E+00_JPRB ,&
&  0.95621E+00_JPRB, 0.10006E+01_JPRB, 0.10565E+01_JPRB, 0.10820E+01_JPRB, 0.11071E+01_JPRB, 0.11431E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11300E+01_JPRB, 0.11183E+01_JPRB, 0.11605E+01_JPRB, 0.11759E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.98516E+00_JPRB, 0.95412E+00_JPRB, 0.95353E+00_JPRB, 0.10620E+01_JPRB, 0.11910E+01_JPRB, 0.12702E+01_JPRB, 0.12682E+01_JPRB ,&
&  0.12683E+01_JPRB, 0.12845E+01_JPRB, 0.14375E+01_JPRB, 0.16323E+01_JPRB, 0.18632E+01_JPRB, 0.21667E+01_JPRB, 0.24897E+01_JPRB ,&
&  0.28289E+01_JPRB, 0.30888E+01_JPRB, 0.32793E+01_JPRB, 0.35009E+01_JPRB, 0.37983E+01_JPRB, 0.41282E+01_JPRB, 0.42702E+01_JPRB ,&
&  0.34050E+01_JPRB, 0.20817E+01_JPRB, 0.23207E+01_JPRB, 0.25743E+01_JPRB, 0.28622E+01_JPRB, 0.30999E+01_JPRB, 0.31608E+01_JPRB ,&
&  0.28211E+01_JPRB, 0.27465E+01_JPRB, 0.28717E+01_JPRB, 0.29924E+01_JPRB, 0.28538E+01_JPRB, 0.30727E+01_JPRB, 0.33115E+01_JPRB ,&
&  0.35563E+01_JPRB, 0.37379E+01_JPRB, 0.38449E+01_JPRB, 0.39420E+01_JPRB, 0.40471E+01_JPRB, 0.42068E+01_JPRB, 0.44697E+01_JPRB ,&
&  0.48202E+01_JPRB, 0.51383E+01_JPRB, 0.53962E+01_JPRB, 0.54027E+01_JPRB, 0.53688E+01_JPRB, 0.55186E+01_JPRB, 0.54941E+01_JPRB ,&
&  0.55069E+01_JPRB, 0.55220E+01_JPRB, 0.61677E+01_JPRB, 0.70771E+01_JPRB, 0.76207E+01_JPRB, 0.70009E+01_JPRB, 0.76971E+01_JPRB ,&
&  0.79177E+01_JPRB, 0.77855E+01_JPRB, 0.72800E+01_JPRB, 0.75260E+01_JPRB, 0.52991E+01_JPRB, 0.55205E+01_JPRB, 0.54122E+01_JPRB ,&
&  0.59291E+01_JPRB, 0.58883E+01_JPRB, 0.58357E+01_JPRB, 0.55517E+01_JPRB, 0.56110E+01_JPRB, 0.46541E+01_JPRB, 0.41713E+01_JPRB ,&
&  0.40363E+01_JPRB, 0.40208E+01_JPRB, 0.36068E+01_JPRB, 0.34403E+01_JPRB, 0.31213E+01_JPRB, 0.32623E+01_JPRB, 0.30297E+01_JPRB ,&
&  0.31320E+01_JPRB, 0.25643E+01_JPRB, 0.21383E+01_JPRB, 0.16152E+01_JPRB, 0.95067E+00_JPRB, 0.81974E+00_JPRB, 0.84783E+00_JPRB ,&
&  0.88778E+00_JPRB, 0.93128E+00_JPRB, 0.93831E+00_JPRB, 0.97812E+00_JPRB, 0.10059E+01_JPRB, 0.99357E+00_JPRB, 0.96285E+00_JPRB ,&
&  0.96275E+00_JPRB, 0.99677E+00_JPRB, 0.99946E+00_JPRB, 0.10036E+01_JPRB, 0.99998E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.79271E+00_JPRB, 0.79399E+00_JPRB, 0.81275E+00_JPRB, 0.86485E+00_JPRB, 0.96251E+00_JPRB, 0.10297E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10780E+01_JPRB, 0.11194E+01_JPRB, 0.12079E+01_JPRB, 0.12711E+01_JPRB, 0.14024E+01_JPRB, 0.15970E+01_JPRB, 0.17909E+01_JPRB ,&
&  0.20435E+01_JPRB, 0.22672E+01_JPRB, 0.24417E+01_JPRB, 0.26849E+01_JPRB, 0.29172E+01_JPRB, 0.32379E+01_JPRB, 0.34448E+01_JPRB ,&
&  0.34099E+01_JPRB, 0.28174E+01_JPRB, 0.25992E+01_JPRB, 0.21436E+01_JPRB, 0.20661E+01_JPRB, 0.25177E+01_JPRB, 0.23954E+01_JPRB ,&
&  0.23940E+01_JPRB, 0.24563E+01_JPRB, 0.24640E+01_JPRB, 0.25778E+01_JPRB, 0.27491E+01_JPRB, 0.29560E+01_JPRB, 0.32302E+01_JPRB ,&
&  0.35331E+01_JPRB, 0.38071E+01_JPRB, 0.40542E+01_JPRB, 0.42867E+01_JPRB, 0.44700E+01_JPRB, 0.47057E+01_JPRB, 0.50341E+01_JPRB ,&
&  0.54161E+01_JPRB, 0.57370E+01_JPRB, 0.61128E+01_JPRB, 0.66055E+01_JPRB, 0.66091E+01_JPRB, 0.68647E+01_JPRB, 0.69976E+01_JPRB ,&
&  0.70895E+01_JPRB, 0.70362E+01_JPRB, 0.71292E+01_JPRB, 0.74427E+01_JPRB, 0.74030E+01_JPRB, 0.65858E+01_JPRB, 0.65617E+01_JPRB ,&
&  0.67942E+01_JPRB, 0.63313E+01_JPRB, 0.54714E+01_JPRB, 0.61323E+01_JPRB, 0.41752E+01_JPRB, 0.47881E+01_JPRB, 0.53341E+01_JPRB ,&
&  0.52422E+01_JPRB, 0.50882E+01_JPRB, 0.47931E+01_JPRB, 0.46793E+01_JPRB, 0.46148E+01_JPRB, 0.42018E+01_JPRB, 0.39797E+01_JPRB ,&
&  0.37517E+01_JPRB, 0.30358E+01_JPRB, 0.27892E+01_JPRB, 0.29963E+01_JPRB, 0.32204E+01_JPRB, 0.29112E+01_JPRB, 0.22941E+01_JPRB ,&
&  0.19006E+01_JPRB, 0.20499E+01_JPRB, 0.19108E+01_JPRB, 0.20464E+01_JPRB, 0.13305E+01_JPRB, 0.79770E+00_JPRB, 0.79118E+00_JPRB ,&
&  0.81053E+00_JPRB, 0.83173E+00_JPRB, 0.82484E+00_JPRB, 0.86737E+00_JPRB, 0.87817E+00_JPRB, 0.85927E+00_JPRB, 0.86114E+00_JPRB ,&
&  0.88083E+00_JPRB, 0.87809E+00_JPRB, 0.87416E+00_JPRB, 0.85218E+00_JPRB, 0.83296E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.76465E+00_JPRB, 0.75233E+00_JPRB, 0.75762E+00_JPRB, 0.75677E+00_JPRB, 0.77145E+00_JPRB, 0.77661E+00_JPRB, 0.79794E+00_JPRB ,&
&  0.83367E+00_JPRB, 0.87429E+00_JPRB, 0.87693E+00_JPRB, 0.86295E+00_JPRB, 0.87149E+00_JPRB, 0.89962E+00_JPRB, 0.97937E+00_JPRB ,&
&  0.11450E+01_JPRB, 0.13284E+01_JPRB, 0.14488E+01_JPRB, 0.15810E+01_JPRB, 0.18436E+01_JPRB, 0.23286E+01_JPRB, 0.27966E+01_JPRB ,&
&  0.31518E+01_JPRB, 0.33789E+01_JPRB, 0.33193E+01_JPRB, 0.34130E+01_JPRB, 0.27231E+01_JPRB, 0.23250E+01_JPRB, 0.26028E+01_JPRB ,&
&  0.28685E+01_JPRB, 0.28035E+01_JPRB, 0.23910E+01_JPRB, 0.23812E+01_JPRB, 0.24471E+01_JPRB, 0.26258E+01_JPRB, 0.30007E+01_JPRB ,&
&  0.33693E+01_JPRB, 0.37623E+01_JPRB, 0.40924E+01_JPRB, 0.43826E+01_JPRB, 0.46320E+01_JPRB, 0.49182E+01_JPRB, 0.51998E+01_JPRB ,&
&  0.55022E+01_JPRB, 0.57448E+01_JPRB, 0.60012E+01_JPRB, 0.65102E+01_JPRB, 0.70074E+01_JPRB, 0.74378E+01_JPRB, 0.76723E+01_JPRB ,&
&  0.77245E+01_JPRB, 0.72930E+01_JPRB, 0.73339E+01_JPRB, 0.73969E+01_JPRB, 0.69379E+01_JPRB, 0.62927E+01_JPRB, 0.60365E+01_JPRB ,&
&  0.57606E+01_JPRB, 0.53025E+01_JPRB, 0.40268E+01_JPRB, 0.48286E+01_JPRB, 0.54410E+01_JPRB, 0.48803E+01_JPRB, 0.46484E+01_JPRB ,&
&  0.43384E+01_JPRB, 0.41337E+01_JPRB, 0.38612E+01_JPRB, 0.38860E+01_JPRB, 0.41247E+01_JPRB, 0.52190E+01_JPRB, 0.40837E+01_JPRB ,&
&  0.34296E+01_JPRB, 0.22113E+01_JPRB, 0.18953E+01_JPRB, 0.21445E+01_JPRB, 0.24084E+01_JPRB, 0.22115E+01_JPRB, 0.15716E+01_JPRB ,&
&  0.14271E+01_JPRB, 0.17105E+01_JPRB, 0.18384E+01_JPRB, 0.23749E+01_JPRB, 0.24995E+01_JPRB, 0.85071E+00_JPRB, 0.73095E+00_JPRB ,&
&  0.72895E+00_JPRB, 0.72975E+00_JPRB, 0.72432E+00_JPRB, 0.74454E+00_JPRB, 0.74343E+00_JPRB, 0.74953E+00_JPRB, 0.77346E+00_JPRB ,&
&  0.79052E+00_JPRB, 0.79276E+00_JPRB, 0.78948E+00_JPRB, 0.76552E+00_JPRB, 0.77022E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.83559E+00_JPRB, 0.83515E+00_JPRB, 0.84186E+00_JPRB, 0.83435E+00_JPRB, 0.82655E+00_JPRB, 0.82609E+00_JPRB, 0.83129E+00_JPRB ,&
&  0.83420E+00_JPRB, 0.82277E+00_JPRB, 0.80831E+00_JPRB, 0.79921E+00_JPRB, 0.79119E+00_JPRB, 0.75429E+00_JPRB, 0.70342E+00_JPRB ,&
&  0.71930E+00_JPRB, 0.74356E+00_JPRB, 0.76283E+00_JPRB, 0.80539E+00_JPRB, 0.91359E+00_JPRB, 0.10827E+01_JPRB, 0.13248E+01_JPRB ,&
&  0.17862E+01_JPRB, 0.22930E+01_JPRB, 0.26530E+01_JPRB, 0.27423E+01_JPRB, 0.30628E+01_JPRB, 0.24806E+01_JPRB, 0.25998E+01_JPRB ,&
&  0.30560E+01_JPRB, 0.25610E+01_JPRB, 0.22915E+01_JPRB, 0.21597E+01_JPRB, 0.22053E+01_JPRB, 0.23664E+01_JPRB, 0.26262E+01_JPRB ,&
&  0.30048E+01_JPRB, 0.34039E+01_JPRB, 0.37529E+01_JPRB, 0.41049E+01_JPRB, 0.44450E+01_JPRB, 0.47919E+01_JPRB, 0.50898E+01_JPRB ,&
&  0.52942E+01_JPRB, 0.55355E+01_JPRB, 0.58531E+01_JPRB, 0.60798E+01_JPRB, 0.67223E+01_JPRB, 0.71287E+01_JPRB, 0.73249E+01_JPRB ,&
&  0.71605E+01_JPRB, 0.68166E+01_JPRB, 0.68216E+01_JPRB, 0.64520E+01_JPRB, 0.60382E+01_JPRB, 0.56531E+01_JPRB, 0.52215E+01_JPRB ,&
&  0.48067E+01_JPRB, 0.42222E+01_JPRB, 0.33502E+01_JPRB, 0.37335E+01_JPRB, 0.39287E+01_JPRB, 0.35513E+01_JPRB, 0.40796E+01_JPRB ,&
&  0.37637E+01_JPRB, 0.35909E+01_JPRB, 0.34443E+01_JPRB, 0.32879E+01_JPRB, 0.31620E+01_JPRB, 0.36689E+01_JPRB, 0.36313E+01_JPRB ,&
&  0.24699E+01_JPRB, 0.16202E+01_JPRB, 0.13221E+01_JPRB, 0.13613E+01_JPRB, 0.15570E+01_JPRB, 0.16586E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.14749E+01_JPRB, 0.16615E+01_JPRB, 0.21502E+01_JPRB, 0.14443E+01_JPRB, 0.83794E+00_JPRB, 0.67886E+00_JPRB ,&
&  0.64481E+00_JPRB, 0.64223E+00_JPRB, 0.65037E+00_JPRB, 0.68018E+00_JPRB, 0.70676E+00_JPRB, 0.73343E+00_JPRB, 0.75647E+00_JPRB ,&
&  0.76915E+00_JPRB, 0.79040E+00_JPRB, 0.80808E+00_JPRB, 0.82681E+00_JPRB, 0.83823E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.94266E+00_JPRB, 0.96185E+00_JPRB, 0.95725E+00_JPRB, 0.96739E+00_JPRB, 0.97762E+00_JPRB, 0.99766E+00_JPRB, 0.99874E+00_JPRB ,&
&  0.99308E+00_JPRB, 0.99524E+00_JPRB, 0.97264E+00_JPRB, 0.97834E+00_JPRB, 0.97893E+00_JPRB, 0.98111E+00_JPRB, 0.96730E+00_JPRB ,&
&  0.97513E+00_JPRB, 0.99678E+00_JPRB, 0.10037E+01_JPRB, 0.10022E+01_JPRB, 0.98372E+00_JPRB, 0.95182E+00_JPRB, 0.92878E+00_JPRB ,&
&  0.98885E+00_JPRB, 0.11255E+01_JPRB, 0.12975E+01_JPRB, 0.14248E+01_JPRB, 0.17234E+01_JPRB, 0.17962E+01_JPRB, 0.20269E+01_JPRB ,&
&  0.25378E+01_JPRB, 0.22414E+01_JPRB, 0.22531E+01_JPRB, 0.19369E+01_JPRB, 0.19312E+01_JPRB, 0.19114E+01_JPRB, 0.19349E+01_JPRB ,&
&  0.20338E+01_JPRB, 0.22156E+01_JPRB, 0.24201E+01_JPRB, 0.26472E+01_JPRB, 0.28394E+01_JPRB, 0.29224E+01_JPRB, 0.29707E+01_JPRB ,&
&  0.30251E+01_JPRB, 0.33360E+01_JPRB, 0.41589E+01_JPRB, 0.48604E+01_JPRB, 0.56597E+01_JPRB, 0.60286E+01_JPRB, 0.60805E+01_JPRB ,&
&  0.59996E+01_JPRB, 0.60057E+01_JPRB, 0.51695E+01_JPRB, 0.45068E+01_JPRB, 0.43274E+01_JPRB, 0.38512E+01_JPRB, 0.36821E+01_JPRB ,&
&  0.37938E+01_JPRB, 0.35422E+01_JPRB, 0.27514E+01_JPRB, 0.29407E+01_JPRB, 0.31368E+01_JPRB, 0.30143E+01_JPRB, 0.34411E+01_JPRB ,&
&  0.29679E+01_JPRB, 0.26525E+01_JPRB, 0.25156E+01_JPRB, 0.24302E+01_JPRB, 0.23160E+01_JPRB, 0.21907E+01_JPRB, 0.22045E+01_JPRB ,&
&  0.17052E+01_JPRB, 0.13907E+01_JPRB, 0.12685E+01_JPRB, 0.12869E+01_JPRB, 0.16033E+01_JPRB, 0.21999E+01_JPRB, 0.17240E+01_JPRB ,&
&  0.16421E+01_JPRB, 0.15916E+01_JPRB, 0.21693E+01_JPRB, 0.31181E+01_JPRB, 0.14135E+01_JPRB, 0.89149E+00_JPRB, 0.65181E+00_JPRB ,&
&  0.64247E+00_JPRB, 0.67541E+00_JPRB, 0.70612E+00_JPRB, 0.74574E+00_JPRB, 0.76961E+00_JPRB, 0.78030E+00_JPRB, 0.79814E+00_JPRB ,&
&  0.83679E+00_JPRB, 0.87281E+00_JPRB, 0.92227E+00_JPRB, 0.93120E+00_JPRB, 0.92231E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.10118E+01_JPRB, 0.10389E+01_JPRB, 0.10456E+01_JPRB, 0.10688E+01_JPRB, 0.10988E+01_JPRB, 0.11128E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.11119E+01_JPRB, 0.11099E+01_JPRB, 0.11292E+01_JPRB, 0.11348E+01_JPRB, 0.11357E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.11624E+01_JPRB, 0.11804E+01_JPRB, 0.12226E+01_JPRB, 0.12253E+01_JPRB, 0.12190E+01_JPRB, 0.12247E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.12841E+01_JPRB, 0.13117E+01_JPRB, 0.13534E+01_JPRB, 0.13918E+01_JPRB, 0.15317E+01_JPRB, 0.17158E+01_JPRB, 0.20405E+01_JPRB ,&
&  0.21459E+01_JPRB, 0.23073E+01_JPRB, 0.20397E+01_JPRB, 0.17423E+01_JPRB, 0.17272E+01_JPRB, 0.17466E+01_JPRB, 0.16645E+01_JPRB ,&
&  0.15838E+01_JPRB, 0.15567E+01_JPRB, 0.15335E+01_JPRB, 0.15344E+01_JPRB, 0.15529E+01_JPRB, 0.15135E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.14368E+01_JPRB, 0.14814E+01_JPRB, 0.19344E+01_JPRB, 0.25502E+01_JPRB, 0.31849E+01_JPRB, 0.37135E+01_JPRB, 0.33419E+01_JPRB ,&
&  0.31924E+01_JPRB, 0.34278E+01_JPRB, 0.31702E+01_JPRB, 0.29230E+01_JPRB, 0.26315E+01_JPRB, 0.23547E+01_JPRB, 0.22155E+01_JPRB ,&
&  0.24392E+01_JPRB, 0.26211E+01_JPRB, 0.23842E+01_JPRB, 0.26698E+01_JPRB, 0.27364E+01_JPRB, 0.26984E+01_JPRB, 0.23942E+01_JPRB ,&
&  0.20446E+01_JPRB, 0.18833E+01_JPRB, 0.18434E+01_JPRB, 0.18378E+01_JPRB, 0.18533E+01_JPRB, 0.17337E+01_JPRB, 0.15446E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.12628E+01_JPRB, 0.12263E+01_JPRB, 0.11874E+01_JPRB, 0.15943E+01_JPRB, 0.19763E+01_JPRB, 0.25400E+01_JPRB ,&
&  0.17817E+01_JPRB, 0.16413E+01_JPRB, 0.14270E+01_JPRB, 0.22372E+01_JPRB, 0.11914E+01_JPRB, 0.69368E+00_JPRB, 0.65704E+00_JPRB ,&
&  0.73100E+00_JPRB, 0.77700E+00_JPRB, 0.80114E+00_JPRB, 0.83663E+00_JPRB, 0.85405E+00_JPRB, 0.85726E+00_JPRB, 0.87381E+00_JPRB ,&
&  0.92028E+00_JPRB, 0.93758E+00_JPRB, 0.95456E+00_JPRB, 0.96117E+00_JPRB, 0.98490E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.10483E+01_JPRB, 0.10872E+01_JPRB, 0.11119E+01_JPRB, 0.11367E+01_JPRB, 0.11686E+01_JPRB, 0.11896E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.12130E+01_JPRB, 0.12256E+01_JPRB, 0.12434E+01_JPRB, 0.12522E+01_JPRB, 0.12478E+01_JPRB, 0.12555E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.13151E+01_JPRB, 0.13572E+01_JPRB, 0.13534E+01_JPRB, 0.13532E+01_JPRB, 0.13810E+01_JPRB, 0.14191E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.14139E+01_JPRB, 0.14261E+01_JPRB, 0.14477E+01_JPRB, 0.14875E+01_JPRB, 0.15108E+01_JPRB, 0.16421E+01_JPRB ,&
&  0.17499E+01_JPRB, 0.24559E+01_JPRB, 0.22510E+01_JPRB, 0.19427E+01_JPRB, 0.19216E+01_JPRB, 0.17744E+01_JPRB, 0.17541E+01_JPRB ,&
&  0.17517E+01_JPRB, 0.16899E+01_JPRB, 0.16557E+01_JPRB, 0.16189E+01_JPRB, 0.16274E+01_JPRB, 0.16505E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.16338E+01_JPRB, 0.16751E+01_JPRB, 0.18275E+01_JPRB, 0.20285E+01_JPRB, 0.22286E+01_JPRB, 0.23036E+01_JPRB, 0.21302E+01_JPRB ,&
&  0.20092E+01_JPRB, 0.18113E+01_JPRB, 0.15120E+01_JPRB, 0.15479E+01_JPRB, 0.15131E+01_JPRB, 0.12486E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.18186E+01_JPRB, 0.22784E+01_JPRB, 0.22943E+01_JPRB, 0.22172E+01_JPRB, 0.22415E+01_JPRB, 0.18810E+01_JPRB ,&
&  0.16454E+01_JPRB, 0.15099E+01_JPRB, 0.15040E+01_JPRB, 0.14624E+01_JPRB, 0.14852E+01_JPRB, 0.14536E+01_JPRB, 0.13158E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.11754E+01_JPRB, 0.11978E+01_JPRB, 0.12821E+01_JPRB, 0.14557E+01_JPRB, 0.27297E+01_JPRB, 0.41557E+01_JPRB ,&
&  0.20919E+01_JPRB, 0.14822E+01_JPRB, 0.87386E+00_JPRB, 0.10943E+01_JPRB, 0.83324E+00_JPRB, 0.68296E+00_JPRB, 0.71972E+00_JPRB ,&
&  0.84229E+00_JPRB, 0.89956E+00_JPRB, 0.91406E+00_JPRB, 0.91291E+00_JPRB, 0.89647E+00_JPRB, 0.88296E+00_JPRB, 0.87999E+00_JPRB ,&
&  0.90518E+00_JPRB, 0.93092E+00_JPRB, 0.96004E+00_JPRB, 0.98105E+00_JPRB, 0.10095E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.10632E+01_JPRB, 0.11017E+01_JPRB, 0.11334E+01_JPRB, 0.11694E+01_JPRB, 0.12082E+01_JPRB, 0.12441E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.13042E+01_JPRB, 0.13268E+01_JPRB, 0.13488E+01_JPRB, 0.13716E+01_JPRB, 0.13975E+01_JPRB, 0.14180E+01_JPRB, 0.14447E+01_JPRB ,&
&  0.14848E+01_JPRB, 0.15208E+01_JPRB, 0.15646E+01_JPRB, 0.15716E+01_JPRB, 0.15798E+01_JPRB, 0.16239E+01_JPRB, 0.16640E+01_JPRB ,&
&  0.16759E+01_JPRB, 0.16985E+01_JPRB, 0.17345E+01_JPRB, 0.17836E+01_JPRB, 0.18535E+01_JPRB, 0.20108E+01_JPRB, 0.19922E+01_JPRB ,&
&  0.22924E+01_JPRB, 0.25572E+01_JPRB, 0.24295E+01_JPRB, 0.21859E+01_JPRB, 0.20860E+01_JPRB, 0.20586E+01_JPRB, 0.20938E+01_JPRB ,&
&  0.21637E+01_JPRB, 0.21448E+01_JPRB, 0.21446E+01_JPRB, 0.21300E+01_JPRB, 0.21392E+01_JPRB, 0.21901E+01_JPRB, 0.22396E+01_JPRB ,&
&  0.23072E+01_JPRB, 0.23962E+01_JPRB, 0.25225E+01_JPRB, 0.26904E+01_JPRB, 0.28753E+01_JPRB, 0.28831E+01_JPRB, 0.28032E+01_JPRB ,&
&  0.25981E+01_JPRB, 0.23724E+01_JPRB, 0.18270E+01_JPRB, 0.17064E+01_JPRB, 0.15894E+01_JPRB, 0.13704E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.15523E+01_JPRB, 0.16291E+01_JPRB, 0.17999E+01_JPRB, 0.20576E+01_JPRB, 0.19325E+01_JPRB, 0.17925E+01_JPRB, 0.15769E+01_JPRB ,&
&  0.14040E+01_JPRB, 0.12920E+01_JPRB, 0.12549E+01_JPRB, 0.12066E+01_JPRB, 0.12012E+01_JPRB, 0.12060E+01_JPRB, 0.11096E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10939E+01_JPRB, 0.11354E+01_JPRB, 0.12321E+01_JPRB, 0.14719E+01_JPRB, 0.16802E+01_JPRB, 0.25978E+01_JPRB ,&
&  0.18627E+01_JPRB, 0.10901E+01_JPRB, 0.10483E+01_JPRB, 0.92537E+00_JPRB, 0.81913E+00_JPRB, 0.73346E+00_JPRB, 0.79734E+00_JPRB ,&
&  0.95212E+00_JPRB, 0.10800E+01_JPRB, 0.11131E+01_JPRB, 0.10635E+01_JPRB, 0.98042E+00_JPRB, 0.93646E+00_JPRB, 0.91595E+00_JPRB ,&
&  0.93700E+00_JPRB, 0.95720E+00_JPRB, 0.98081E+00_JPRB, 0.99655E+00_JPRB, 0.10229E+01_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.10498E+01_JPRB, 0.10926E+01_JPRB, 0.11299E+01_JPRB, 0.11675E+01_JPRB, 0.12053E+01_JPRB, 0.12435E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.13152E+01_JPRB, 0.13549E+01_JPRB, 0.13996E+01_JPRB, 0.14439E+01_JPRB, 0.14868E+01_JPRB, 0.15332E+01_JPRB, 0.15753E+01_JPRB ,&
&  0.16270E+01_JPRB, 0.16676E+01_JPRB, 0.17165E+01_JPRB, 0.17366E+01_JPRB, 0.17622E+01_JPRB, 0.18056E+01_JPRB, 0.18487E+01_JPRB ,&
&  0.18659E+01_JPRB, 0.18918E+01_JPRB, 0.19249E+01_JPRB, 0.19750E+01_JPRB, 0.20060E+01_JPRB, 0.21480E+01_JPRB, 0.20202E+01_JPRB ,&
&  0.25425E+01_JPRB, 0.28191E+01_JPRB, 0.26894E+01_JPRB, 0.24696E+01_JPRB, 0.23619E+01_JPRB, 0.24048E+01_JPRB, 0.24699E+01_JPRB ,&
&  0.25607E+01_JPRB, 0.26990E+01_JPRB, 0.26325E+01_JPRB, 0.26095E+01_JPRB, 0.26357E+01_JPRB, 0.27297E+01_JPRB, 0.28735E+01_JPRB ,&
&  0.30229E+01_JPRB, 0.31862E+01_JPRB, 0.33611E+01_JPRB, 0.35338E+01_JPRB, 0.37003E+01_JPRB, 0.38117E+01_JPRB, 0.38393E+01_JPRB ,&
&  0.37331E+01_JPRB, 0.35626E+01_JPRB, 0.31682E+01_JPRB, 0.31045E+01_JPRB, 0.29096E+01_JPRB, 0.23745E+01_JPRB, 0.18667E+01_JPRB ,&
&  0.17672E+01_JPRB, 0.17440E+01_JPRB, 0.16753E+01_JPRB, 0.18302E+01_JPRB, 0.17305E+01_JPRB, 0.15869E+01_JPRB, 0.14087E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.11306E+01_JPRB, 0.11064E+01_JPRB, 0.10456E+01_JPRB, 0.99388E+00_JPRB, 0.96349E+00_JPRB, 0.94290E+00_JPRB ,&
&  0.96576E+00_JPRB, 0.10094E+01_JPRB, 0.10740E+01_JPRB, 0.11580E+01_JPRB, 0.12512E+01_JPRB, 0.12917E+01_JPRB, 0.20732E+01_JPRB ,&
&  0.15844E+01_JPRB, 0.12467E+01_JPRB, 0.14429E+01_JPRB, 0.75032E+00_JPRB, 0.79512E+00_JPRB, 0.85421E+00_JPRB, 0.98710E+00_JPRB ,&
&  0.10720E+01_JPRB, 0.14561E+01_JPRB, 0.19264E+01_JPRB, 0.15477E+01_JPRB, 0.12777E+01_JPRB, 0.10141E+01_JPRB, 0.93193E+00_JPRB ,&
&  0.94030E+00_JPRB, 0.95239E+00_JPRB, 0.97131E+00_JPRB, 0.98531E+00_JPRB, 0.10111E+01_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.10209E+01_JPRB, 0.10557E+01_JPRB, 0.10959E+01_JPRB, 0.11336E+01_JPRB, 0.11641E+01_JPRB, 0.11930E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.12509E+01_JPRB, 0.12686E+01_JPRB, 0.12993E+01_JPRB, 0.13415E+01_JPRB, 0.13898E+01_JPRB, 0.14461E+01_JPRB, 0.15058E+01_JPRB ,&
&  0.15850E+01_JPRB, 0.16611E+01_JPRB, 0.17585E+01_JPRB, 0.18475E+01_JPRB, 0.19227E+01_JPRB, 0.20027E+01_JPRB, 0.20882E+01_JPRB ,&
&  0.21916E+01_JPRB, 0.22642E+01_JPRB, 0.23237E+01_JPRB, 0.24278E+01_JPRB, 0.25257E+01_JPRB, 0.26684E+01_JPRB, 0.22581E+01_JPRB ,&
&  0.28218E+01_JPRB, 0.32357E+01_JPRB, 0.30226E+01_JPRB, 0.27268E+01_JPRB, 0.25936E+01_JPRB, 0.26138E+01_JPRB, 0.26400E+01_JPRB ,&
&  0.26979E+01_JPRB, 0.27324E+01_JPRB, 0.26525E+01_JPRB, 0.28572E+01_JPRB, 0.30196E+01_JPRB, 0.31156E+01_JPRB, 0.33403E+01_JPRB ,&
&  0.35715E+01_JPRB, 0.38070E+01_JPRB, 0.40192E+01_JPRB, 0.42163E+01_JPRB, 0.43948E+01_JPRB, 0.45832E+01_JPRB, 0.46656E+01_JPRB ,&
&  0.46826E+01_JPRB, 0.46844E+01_JPRB, 0.46128E+01_JPRB, 0.41804E+01_JPRB, 0.39693E+01_JPRB, 0.36960E+01_JPRB, 0.29398E+01_JPRB ,&
&  0.22887E+01_JPRB, 0.17642E+01_JPRB, 0.15823E+01_JPRB, 0.15816E+01_JPRB, 0.14553E+01_JPRB, 0.13036E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.97382E+00_JPRB, 0.97474E+00_JPRB, 0.92448E+00_JPRB, 0.80869E+00_JPRB, 0.83082E+00_JPRB, 0.86449E+00_JPRB ,&
&  0.87384E+00_JPRB, 0.91419E+00_JPRB, 0.97009E+00_JPRB, 0.10185E+01_JPRB, 0.11430E+01_JPRB, 0.14138E+01_JPRB, 0.15966E+01_JPRB ,&
&  0.16436E+01_JPRB, 0.11693E+01_JPRB, 0.10797E+01_JPRB, 0.10535E+01_JPRB, 0.90359E+00_JPRB, 0.11278E+01_JPRB, 0.14918E+01_JPRB ,&
&  0.21207E+01_JPRB, 0.22193E+01_JPRB, 0.27933E+01_JPRB, 0.42847E+01_JPRB, 0.32678E+01_JPRB, 0.22218E+01_JPRB, 0.94114E+00_JPRB ,&
&  0.95291E+00_JPRB, 0.95524E+00_JPRB, 0.98194E+00_JPRB, 0.99061E+00_JPRB, 0.10086E+01_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.86251E+00_JPRB, 0.86990E+00_JPRB, 0.90849E+00_JPRB, 0.95241E+00_JPRB, 0.99469E+00_JPRB, 0.10357E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.11182E+01_JPRB, 0.11612E+01_JPRB, 0.11846E+01_JPRB, 0.11984E+01_JPRB, 0.12238E+01_JPRB, 0.12536E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.13719E+01_JPRB, 0.14506E+01_JPRB, 0.15600E+01_JPRB, 0.16486E+01_JPRB, 0.17474E+01_JPRB, 0.18780E+01_JPRB, 0.20090E+01_JPRB ,&
&  0.21515E+01_JPRB, 0.23270E+01_JPRB, 0.24715E+01_JPRB, 0.26122E+01_JPRB, 0.27651E+01_JPRB, 0.27743E+01_JPRB, 0.20906E+01_JPRB ,&
&  0.30185E+01_JPRB, 0.33874E+01_JPRB, 0.32751E+01_JPRB, 0.29474E+01_JPRB, 0.27559E+01_JPRB, 0.26994E+01_JPRB, 0.27230E+01_JPRB ,&
&  0.27312E+01_JPRB, 0.26645E+01_JPRB, 0.25852E+01_JPRB, 0.26152E+01_JPRB, 0.30947E+01_JPRB, 0.31545E+01_JPRB, 0.34017E+01_JPRB ,&
&  0.36517E+01_JPRB, 0.38942E+01_JPRB, 0.40985E+01_JPRB, 0.42748E+01_JPRB, 0.44123E+01_JPRB, 0.45791E+01_JPRB, 0.47610E+01_JPRB ,&
&  0.49280E+01_JPRB, 0.50243E+01_JPRB, 0.51795E+01_JPRB, 0.47072E+01_JPRB, 0.44741E+01_JPRB, 0.48025E+01_JPRB, 0.42321E+01_JPRB ,&
&  0.26707E+01_JPRB, 0.17931E+01_JPRB, 0.17363E+01_JPRB, 0.15721E+01_JPRB, 0.13540E+01_JPRB, 0.11830E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.96970E+00_JPRB, 0.90598E+00_JPRB, 0.87448E+00_JPRB, 0.82428E+00_JPRB, 0.80988E+00_JPRB, 0.83024E+00_JPRB, 0.83169E+00_JPRB ,&
&  0.87043E+00_JPRB, 0.84597E+00_JPRB, 0.84991E+00_JPRB, 0.88941E+00_JPRB, 0.10228E+01_JPRB, 0.13132E+01_JPRB, 0.18272E+01_JPRB ,&
&  0.19577E+01_JPRB, 0.11476E+01_JPRB, 0.10543E+01_JPRB, 0.10089E+01_JPRB, 0.10314E+01_JPRB, 0.11516E+01_JPRB, 0.13514E+01_JPRB ,&
&  0.17192E+01_JPRB, 0.22292E+01_JPRB, 0.21012E+01_JPRB, 0.16486E+01_JPRB, 0.20087E+01_JPRB, 0.17138E+01_JPRB, 0.83789E+00_JPRB ,&
&  0.84638E+00_JPRB, 0.82165E+00_JPRB, 0.83429E+00_JPRB, 0.84888E+00_JPRB, 0.85839E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.70196E+00_JPRB, 0.69315E+00_JPRB, 0.70667E+00_JPRB, 0.73445E+00_JPRB, 0.76401E+00_JPRB, 0.81597E+00_JPRB, 0.87895E+00_JPRB ,&
&  0.95525E+00_JPRB, 0.10130E+01_JPRB, 0.10403E+01_JPRB, 0.10569E+01_JPRB, 0.10795E+01_JPRB, 0.11069E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11973E+01_JPRB, 0.12520E+01_JPRB, 0.13273E+01_JPRB, 0.13912E+01_JPRB, 0.14647E+01_JPRB, 0.15938E+01_JPRB, 0.17474E+01_JPRB ,&
&  0.19319E+01_JPRB, 0.21300E+01_JPRB, 0.23559E+01_JPRB, 0.25926E+01_JPRB, 0.28407E+01_JPRB, 0.29606E+01_JPRB, 0.30333E+01_JPRB ,&
&  0.22468E+01_JPRB, 0.31845E+01_JPRB, 0.32933E+01_JPRB, 0.30767E+01_JPRB, 0.28780E+01_JPRB, 0.28546E+01_JPRB, 0.28121E+01_JPRB ,&
&  0.27472E+01_JPRB, 0.25300E+01_JPRB, 0.24932E+01_JPRB, 0.25462E+01_JPRB, 0.28297E+01_JPRB, 0.29830E+01_JPRB, 0.32025E+01_JPRB ,&
&  0.34133E+01_JPRB, 0.35933E+01_JPRB, 0.37515E+01_JPRB, 0.39120E+01_JPRB, 0.41150E+01_JPRB, 0.43878E+01_JPRB, 0.47362E+01_JPRB ,&
&  0.50552E+01_JPRB, 0.52985E+01_JPRB, 0.54606E+01_JPRB, 0.51064E+01_JPRB, 0.47422E+01_JPRB, 0.52701E+01_JPRB, 0.49999E+01_JPRB ,&
&  0.30533E+01_JPRB, 0.19252E+01_JPRB, 0.19365E+01_JPRB, 0.18705E+01_JPRB, 0.15329E+01_JPRB, 0.13374E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.97196E+00_JPRB, 0.85357E+00_JPRB, 0.86167E+00_JPRB, 0.85163E+00_JPRB, 0.85992E+00_JPRB, 0.86163E+00_JPRB, 0.87750E+00_JPRB ,&
&  0.91679E+00_JPRB, 0.88094E+00_JPRB, 0.86939E+00_JPRB, 0.87577E+00_JPRB, 0.92341E+00_JPRB, 0.98559E+00_JPRB, 0.10430E+01_JPRB ,&
&  0.11158E+01_JPRB, 0.10341E+01_JPRB, 0.10112E+01_JPRB, 0.10300E+01_JPRB, 0.10414E+01_JPRB, 0.10726E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.12068E+01_JPRB, 0.12292E+01_JPRB, 0.12085E+01_JPRB, 0.10894E+01_JPRB, 0.10067E+01_JPRB, 0.76331E+00_JPRB ,&
&  0.74658E+00_JPRB, 0.72352E+00_JPRB, 0.72158E+00_JPRB, 0.71883E+00_JPRB, 0.71957E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.61304E+00_JPRB, 0.60916E+00_JPRB, 0.62106E+00_JPRB, 0.63691E+00_JPRB, 0.66589E+00_JPRB, 0.71427E+00_JPRB, 0.78157E+00_JPRB ,&
&  0.85558E+00_JPRB, 0.90206E+00_JPRB, 0.94227E+00_JPRB, 0.94363E+00_JPRB, 0.95327E+00_JPRB, 0.98353E+00_JPRB, 0.10106E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.11209E+01_JPRB, 0.11578E+01_JPRB, 0.11767E+01_JPRB, 0.12048E+01_JPRB, 0.12716E+01_JPRB, 0.13660E+01_JPRB ,&
&  0.14903E+01_JPRB, 0.16703E+01_JPRB, 0.19176E+01_JPRB, 0.22498E+01_JPRB, 0.26156E+01_JPRB, 0.27947E+01_JPRB, 0.28468E+01_JPRB ,&
&  0.13779E+01_JPRB, 0.23536E+01_JPRB, 0.32101E+01_JPRB, 0.32138E+01_JPRB, 0.31274E+01_JPRB, 0.30586E+01_JPRB, 0.30157E+01_JPRB ,&
&  0.26955E+01_JPRB, 0.23212E+01_JPRB, 0.21922E+01_JPRB, 0.25140E+01_JPRB, 0.25090E+01_JPRB, 0.27071E+01_JPRB, 0.28875E+01_JPRB ,&
&  0.30866E+01_JPRB, 0.32574E+01_JPRB, 0.33679E+01_JPRB, 0.34686E+01_JPRB, 0.36396E+01_JPRB, 0.39001E+01_JPRB, 0.42261E+01_JPRB ,&
&  0.46487E+01_JPRB, 0.51680E+01_JPRB, 0.55324E+01_JPRB, 0.53136E+01_JPRB, 0.51808E+01_JPRB, 0.55517E+01_JPRB, 0.69251E+01_JPRB ,&
&  0.42232E+01_JPRB, 0.23388E+01_JPRB, 0.22980E+01_JPRB, 0.22768E+01_JPRB, 0.18226E+01_JPRB, 0.16028E+01_JPRB, 0.14163E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.10146E+01_JPRB, 0.95331E+00_JPRB, 0.91617E+00_JPRB, 0.92764E+00_JPRB, 0.96330E+00_JPRB, 0.10308E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10524E+01_JPRB, 0.10267E+01_JPRB, 0.10271E+01_JPRB, 0.10194E+01_JPRB, 0.10281E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.10150E+01_JPRB, 0.10038E+01_JPRB, 0.10300E+01_JPRB, 0.10712E+01_JPRB, 0.10809E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10454E+01_JPRB, 0.10614E+01_JPRB, 0.10593E+01_JPRB, 0.10252E+01_JPRB, 0.96823E+00_JPRB, 0.90148E+00_JPRB, 0.79473E+00_JPRB ,&
&  0.74091E+00_JPRB, 0.67735E+00_JPRB, 0.63486E+00_JPRB, 0.61512E+00_JPRB, 0.62339E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.59324E+00_JPRB, 0.58913E+00_JPRB, 0.59348E+00_JPRB, 0.62511E+00_JPRB, 0.62530E+00_JPRB, 0.65034E+00_JPRB, 0.68640E+00_JPRB ,&
&  0.75904E+00_JPRB, 0.82408E+00_JPRB, 0.88656E+00_JPRB, 0.89005E+00_JPRB, 0.88758E+00_JPRB, 0.87014E+00_JPRB, 0.87794E+00_JPRB ,&
&  0.90301E+00_JPRB, 0.94176E+00_JPRB, 0.98519E+00_JPRB, 0.10109E+01_JPRB, 0.10330E+01_JPRB, 0.10579E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.11359E+01_JPRB, 0.12097E+01_JPRB, 0.13571E+01_JPRB, 0.16436E+01_JPRB, 0.21204E+01_JPRB, 0.26556E+01_JPRB, 0.30806E+01_JPRB ,&
&  0.21210E+01_JPRB, 0.84859E+00_JPRB, 0.25082E+01_JPRB, 0.33155E+01_JPRB, 0.32990E+01_JPRB, 0.32008E+01_JPRB, 0.30632E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.23589E+01_JPRB, 0.20239E+01_JPRB, 0.21898E+01_JPRB, 0.20692E+01_JPRB, 0.21862E+01_JPRB, 0.23900E+01_JPRB ,&
&  0.26183E+01_JPRB, 0.28327E+01_JPRB, 0.30339E+01_JPRB, 0.31147E+01_JPRB, 0.31528E+01_JPRB, 0.32599E+01_JPRB, 0.34811E+01_JPRB ,&
&  0.38741E+01_JPRB, 0.45346E+01_JPRB, 0.54032E+01_JPRB, 0.57905E+01_JPRB, 0.67572E+01_JPRB, 0.63752E+01_JPRB, 0.69304E+01_JPRB ,&
&  0.44536E+01_JPRB, 0.29166E+01_JPRB, 0.27020E+01_JPRB, 0.26923E+01_JPRB, 0.21798E+01_JPRB, 0.17844E+01_JPRB, 0.17465E+01_JPRB ,&
&  0.13696E+01_JPRB, 0.11945E+01_JPRB, 0.11242E+01_JPRB, 0.10679E+01_JPRB, 0.10304E+01_JPRB, 0.10728E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.12162E+01_JPRB, 0.12016E+01_JPRB, 0.11914E+01_JPRB, 0.11756E+01_JPRB, 0.11397E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.11111E+01_JPRB, 0.11176E+01_JPRB, 0.11247E+01_JPRB, 0.11313E+01_JPRB, 0.11035E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.11034E+01_JPRB, 0.10768E+01_JPRB, 0.10772E+01_JPRB, 0.10338E+01_JPRB, 0.97456E+00_JPRB, 0.92706E+00_JPRB ,&
&  0.85073E+00_JPRB, 0.74323E+00_JPRB, 0.66074E+00_JPRB, 0.62725E+00_JPRB, 0.60083E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.65046E+00_JPRB, 0.63077E+00_JPRB, 0.63518E+00_JPRB, 0.65312E+00_JPRB, 0.67006E+00_JPRB, 0.68450E+00_JPRB, 0.66979E+00_JPRB ,&
&  0.69331E+00_JPRB, 0.74600E+00_JPRB, 0.78017E+00_JPRB, 0.81735E+00_JPRB, 0.81245E+00_JPRB, 0.79516E+00_JPRB, 0.78236E+00_JPRB ,&
&  0.77903E+00_JPRB, 0.78547E+00_JPRB, 0.81208E+00_JPRB, 0.82743E+00_JPRB, 0.84396E+00_JPRB, 0.86649E+00_JPRB, 0.88046E+00_JPRB ,&
&  0.89258E+00_JPRB, 0.91544E+00_JPRB, 0.96711E+00_JPRB, 0.10838E+01_JPRB, 0.13164E+01_JPRB, 0.17798E+01_JPRB, 0.24913E+01_JPRB ,&
&  0.38195E+01_JPRB, 0.27632E+01_JPRB, 0.11799E+01_JPRB, 0.27136E+01_JPRB, 0.35144E+01_JPRB, 0.35872E+01_JPRB, 0.30499E+01_JPRB ,&
&  0.28601E+01_JPRB, 0.24241E+01_JPRB, 0.20739E+01_JPRB, 0.21139E+01_JPRB, 0.17968E+01_JPRB, 0.16891E+01_JPRB, 0.17371E+01_JPRB ,&
&  0.18981E+01_JPRB, 0.21603E+01_JPRB, 0.24509E+01_JPRB, 0.26545E+01_JPRB, 0.27615E+01_JPRB, 0.28083E+01_JPRB, 0.28877E+01_JPRB ,&
&  0.31412E+01_JPRB, 0.35590E+01_JPRB, 0.43193E+01_JPRB, 0.56488E+01_JPRB, 0.83110E+01_JPRB, 0.77672E+01_JPRB, 0.77321E+01_JPRB ,&
&  0.47611E+01_JPRB, 0.39238E+01_JPRB, 0.35768E+01_JPRB, 0.32152E+01_JPRB, 0.25193E+01_JPRB, 0.21854E+01_JPRB, 0.21816E+01_JPRB ,&
&  0.17039E+01_JPRB, 0.13983E+01_JPRB, 0.12839E+01_JPRB, 0.11983E+01_JPRB, 0.11554E+01_JPRB, 0.11698E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.12883E+01_JPRB, 0.12874E+01_JPRB, 0.12646E+01_JPRB, 0.12470E+01_JPRB, 0.12344E+01_JPRB, 0.12171E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.11735E+01_JPRB, 0.11786E+01_JPRB, 0.11993E+01_JPRB, 0.11846E+01_JPRB, 0.11416E+01_JPRB, 0.11061E+01_JPRB, 0.11288E+01_JPRB ,&
&  0.11178E+01_JPRB, 0.11415E+01_JPRB, 0.11185E+01_JPRB, 0.11327E+01_JPRB, 0.10997E+01_JPRB, 0.10600E+01_JPRB, 0.10193E+01_JPRB ,&
&  0.96199E+00_JPRB, 0.85980E+00_JPRB, 0.75349E+00_JPRB, 0.65981E+00_JPRB, 0.64960E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.74071E+00_JPRB, 0.72199E+00_JPRB, 0.72888E+00_JPRB, 0.72360E+00_JPRB, 0.71436E+00_JPRB, 0.71705E+00_JPRB, 0.71283E+00_JPRB ,&
&  0.70088E+00_JPRB, 0.69422E+00_JPRB, 0.68442E+00_JPRB, 0.72156E+00_JPRB, 0.74317E+00_JPRB, 0.72923E+00_JPRB, 0.71504E+00_JPRB ,&
&  0.70220E+00_JPRB, 0.70287E+00_JPRB, 0.71409E+00_JPRB, 0.72283E+00_JPRB, 0.73012E+00_JPRB, 0.74488E+00_JPRB, 0.75690E+00_JPRB ,&
&  0.75409E+00_JPRB, 0.75603E+00_JPRB, 0.77217E+00_JPRB, 0.81335E+00_JPRB, 0.90098E+00_JPRB, 0.10749E+01_JPRB, 0.14217E+01_JPRB ,&
&  0.23220E+01_JPRB, 0.24308E+01_JPRB, 0.10557E+01_JPRB, 0.21166E+01_JPRB, 0.35618E+01_JPRB, 0.37970E+01_JPRB, 0.36168E+01_JPRB ,&
&  0.33239E+01_JPRB, 0.26145E+01_JPRB, 0.22642E+01_JPRB, 0.21821E+01_JPRB, 0.18395E+01_JPRB, 0.15875E+01_JPRB, 0.14340E+01_JPRB ,&
&  0.13854E+01_JPRB, 0.14639E+01_JPRB, 0.16731E+01_JPRB, 0.19305E+01_JPRB, 0.21589E+01_JPRB, 0.22938E+01_JPRB, 0.23631E+01_JPRB ,&
&  0.24648E+01_JPRB, 0.26912E+01_JPRB, 0.31823E+01_JPRB, 0.38606E+01_JPRB, 0.67215E+01_JPRB, 0.82728E+01_JPRB, 0.87403E+01_JPRB ,&
&  0.63763E+01_JPRB, 0.54506E+01_JPRB, 0.46804E+01_JPRB, 0.39256E+01_JPRB, 0.30185E+01_JPRB, 0.27569E+01_JPRB, 0.27052E+01_JPRB ,&
&  0.21797E+01_JPRB, 0.17307E+01_JPRB, 0.15203E+01_JPRB, 0.13819E+01_JPRB, 0.12895E+01_JPRB, 0.12536E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12713E+01_JPRB, 0.12860E+01_JPRB, 0.12616E+01_JPRB, 0.12446E+01_JPRB, 0.12460E+01_JPRB, 0.12456E+01_JPRB, 0.12396E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.12211E+01_JPRB, 0.12344E+01_JPRB, 0.12204E+01_JPRB, 0.12182E+01_JPRB, 0.12235E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.11362E+01_JPRB, 0.11372E+01_JPRB, 0.11252E+01_JPRB, 0.10947E+01_JPRB, 0.11442E+01_JPRB, 0.11322E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.11968E+01_JPRB, 0.11255E+01_JPRB, 0.80633E+00_JPRB, 0.75263E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.82027E+00_JPRB, 0.79246E+00_JPRB, 0.78869E+00_JPRB, 0.78086E+00_JPRB, 0.76505E+00_JPRB, 0.75382E+00_JPRB, 0.75331E+00_JPRB ,&
&  0.73338E+00_JPRB, 0.70224E+00_JPRB, 0.67116E+00_JPRB, 0.66755E+00_JPRB, 0.68085E+00_JPRB, 0.69419E+00_JPRB, 0.68183E+00_JPRB ,&
&  0.66287E+00_JPRB, 0.65974E+00_JPRB, 0.67746E+00_JPRB, 0.68705E+00_JPRB, 0.68817E+00_JPRB, 0.69382E+00_JPRB, 0.69504E+00_JPRB ,&
&  0.69298E+00_JPRB, 0.68707E+00_JPRB, 0.68773E+00_JPRB, 0.70393E+00_JPRB, 0.74092E+00_JPRB, 0.82073E+00_JPRB, 0.95617E+00_JPRB ,&
&  0.14456E+01_JPRB, 0.18739E+01_JPRB, 0.23953E+01_JPRB, 0.36329E+01_JPRB, 0.46145E+01_JPRB, 0.44792E+01_JPRB, 0.43179E+01_JPRB ,&
&  0.41238E+01_JPRB, 0.34564E+01_JPRB, 0.28295E+01_JPRB, 0.23671E+01_JPRB, 0.19455E+01_JPRB, 0.16401E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.12612E+01_JPRB, 0.12734E+01_JPRB, 0.13629E+01_JPRB, 0.15062E+01_JPRB, 0.16596E+01_JPRB, 0.17678E+01_JPRB ,&
&  0.18593E+01_JPRB, 0.20190E+01_JPRB, 0.23426E+01_JPRB, 0.27731E+01_JPRB, 0.43850E+01_JPRB, 0.70375E+01_JPRB, 0.84335E+01_JPRB ,&
&  0.83741E+01_JPRB, 0.74015E+01_JPRB, 0.60520E+01_JPRB, 0.48429E+01_JPRB, 0.35244E+01_JPRB, 0.35684E+01_JPRB, 0.31565E+01_JPRB ,&
&  0.26365E+01_JPRB, 0.22056E+01_JPRB, 0.18768E+01_JPRB, 0.16501E+01_JPRB, 0.14853E+01_JPRB, 0.13633E+01_JPRB, 0.12993E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12710E+01_JPRB, 0.12511E+01_JPRB, 0.12273E+01_JPRB, 0.12338E+01_JPRB, 0.12491E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.12410E+01_JPRB, 0.12373E+01_JPRB, 0.12005E+01_JPRB, 0.12184E+01_JPRB, 0.12444E+01_JPRB, 0.12244E+01_JPRB, 0.11794E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11312E+01_JPRB, 0.11324E+01_JPRB, 0.11207E+01_JPRB, 0.11300E+01_JPRB, 0.11437E+01_JPRB, 0.11272E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.10821E+01_JPRB, 0.99740E+00_JPRB, 0.89504E+00_JPRB, 0.85154E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.90085E+00_JPRB, 0.87148E+00_JPRB, 0.85188E+00_JPRB, 0.83215E+00_JPRB, 0.82283E+00_JPRB, 0.82297E+00_JPRB, 0.81822E+00_JPRB ,&
&  0.80143E+00_JPRB, 0.76840E+00_JPRB, 0.72488E+00_JPRB, 0.68539E+00_JPRB, 0.67976E+00_JPRB, 0.67399E+00_JPRB, 0.67242E+00_JPRB ,&
&  0.65648E+00_JPRB, 0.65123E+00_JPRB, 0.66017E+00_JPRB, 0.66338E+00_JPRB, 0.66337E+00_JPRB, 0.66142E+00_JPRB, 0.65972E+00_JPRB ,&
&  0.66383E+00_JPRB, 0.66669E+00_JPRB, 0.66544E+00_JPRB, 0.66728E+00_JPRB, 0.68153E+00_JPRB, 0.72836E+00_JPRB, 0.82620E+00_JPRB ,&
&  0.11073E+01_JPRB, 0.17260E+01_JPRB, 0.19381E+01_JPRB, 0.42419E+01_JPRB, 0.49707E+01_JPRB, 0.49232E+01_JPRB, 0.44327E+01_JPRB ,&
&  0.39207E+01_JPRB, 0.38245E+01_JPRB, 0.30710E+01_JPRB, 0.24132E+01_JPRB, 0.19016E+01_JPRB, 0.16196E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.12704E+01_JPRB, 0.11961E+01_JPRB, 0.11710E+01_JPRB, 0.11700E+01_JPRB, 0.11843E+01_JPRB, 0.12419E+01_JPRB, 0.13050E+01_JPRB ,&
&  0.13783E+01_JPRB, 0.15532E+01_JPRB, 0.18784E+01_JPRB, 0.21922E+01_JPRB, 0.31709E+01_JPRB, 0.52399E+01_JPRB, 0.70390E+01_JPRB ,&
&  0.78627E+01_JPRB, 0.74212E+01_JPRB, 0.65221E+01_JPRB, 0.53087E+01_JPRB, 0.42420E+01_JPRB, 0.38371E+01_JPRB, 0.33958E+01_JPRB ,&
&  0.29498E+01_JPRB, 0.26447E+01_JPRB, 0.22355E+01_JPRB, 0.19343E+01_JPRB, 0.17679E+01_JPRB, 0.15881E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.13341E+01_JPRB, 0.12891E+01_JPRB, 0.12605E+01_JPRB, 0.12289E+01_JPRB, 0.12203E+01_JPRB, 0.12400E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.12316E+01_JPRB, 0.11705E+01_JPRB, 0.11626E+01_JPRB, 0.11943E+01_JPRB, 0.11856E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.11150E+01_JPRB, 0.11121E+01_JPRB, 0.11009E+01_JPRB, 0.10771E+01_JPRB, 0.11527E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.10508E+01_JPRB, 0.10016E+01_JPRB, 0.95491E+00_JPRB, 0.92937E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.94933E+00_JPRB, 0.92850E+00_JPRB, 0.91044E+00_JPRB, 0.88649E+00_JPRB, 0.87283E+00_JPRB, 0.86758E+00_JPRB, 0.85931E+00_JPRB ,&
&  0.83805E+00_JPRB, 0.80727E+00_JPRB, 0.77005E+00_JPRB, 0.72090E+00_JPRB, 0.68950E+00_JPRB, 0.67989E+00_JPRB, 0.68452E+00_JPRB ,&
&  0.67136E+00_JPRB, 0.65218E+00_JPRB, 0.65296E+00_JPRB, 0.65806E+00_JPRB, 0.65790E+00_JPRB, 0.64975E+00_JPRB, 0.64048E+00_JPRB ,&
&  0.64496E+00_JPRB, 0.65243E+00_JPRB, 0.65741E+00_JPRB, 0.65917E+00_JPRB, 0.66367E+00_JPRB, 0.68304E+00_JPRB, 0.75939E+00_JPRB ,&
&  0.10101E+01_JPRB, 0.23041E+01_JPRB, 0.20837E+01_JPRB, 0.39764E+01_JPRB, 0.43025E+01_JPRB, 0.42630E+01_JPRB, 0.39342E+01_JPRB ,&
&  0.35224E+01_JPRB, 0.32174E+01_JPRB, 0.26399E+01_JPRB, 0.22073E+01_JPRB, 0.18135E+01_JPRB, 0.15216E+01_JPRB, 0.13336E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.11246E+01_JPRB, 0.10730E+01_JPRB, 0.10567E+01_JPRB, 0.10528E+01_JPRB, 0.10608E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.12429E+01_JPRB, 0.15533E+01_JPRB, 0.22449E+01_JPRB, 0.28546E+01_JPRB, 0.43084E+01_JPRB, 0.64375E+01_JPRB ,&
&  0.74216E+01_JPRB, 0.73228E+01_JPRB, 0.61444E+01_JPRB, 0.50773E+01_JPRB, 0.42629E+01_JPRB, 0.37076E+01_JPRB, 0.32441E+01_JPRB ,&
&  0.29296E+01_JPRB, 0.26993E+01_JPRB, 0.24576E+01_JPRB, 0.21621E+01_JPRB, 0.19433E+01_JPRB, 0.17833E+01_JPRB, 0.16291E+01_JPRB ,&
&  0.14751E+01_JPRB, 0.13503E+01_JPRB, 0.12788E+01_JPRB, 0.12243E+01_JPRB, 0.11931E+01_JPRB, 0.12028E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.11965E+01_JPRB, 0.11358E+01_JPRB, 0.11192E+01_JPRB, 0.11475E+01_JPRB, 0.11455E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.11035E+01_JPRB, 0.11052E+01_JPRB, 0.10961E+01_JPRB, 0.11048E+01_JPRB, 0.10639E+01_JPRB, 0.11501E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.10677E+01_JPRB, 0.10203E+01_JPRB, 0.10036E+01_JPRB, 0.98053E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.97339E+00_JPRB, 0.95164E+00_JPRB, 0.93605E+00_JPRB, 0.92338E+00_JPRB, 0.91347E+00_JPRB, 0.89717E+00_JPRB, 0.87802E+00_JPRB ,&
&  0.86022E+00_JPRB, 0.83920E+00_JPRB, 0.80335E+00_JPRB, 0.75200E+00_JPRB, 0.70860E+00_JPRB, 0.68549E+00_JPRB, 0.68517E+00_JPRB ,&
&  0.68449E+00_JPRB, 0.67208E+00_JPRB, 0.66192E+00_JPRB, 0.66019E+00_JPRB, 0.65835E+00_JPRB, 0.64872E+00_JPRB, 0.63684E+00_JPRB ,&
&  0.62925E+00_JPRB, 0.63147E+00_JPRB, 0.63696E+00_JPRB, 0.64362E+00_JPRB, 0.64914E+00_JPRB, 0.65804E+00_JPRB, 0.70960E+00_JPRB ,&
&  0.96614E+00_JPRB, 0.10399E+01_JPRB, 0.25105E+01_JPRB, 0.34050E+01_JPRB, 0.34466E+01_JPRB, 0.33266E+01_JPRB, 0.33261E+01_JPRB ,&
&  0.30618E+01_JPRB, 0.26702E+01_JPRB, 0.22062E+01_JPRB, 0.18298E+01_JPRB, 0.15491E+01_JPRB, 0.13640E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.11232E+01_JPRB, 0.10448E+01_JPRB, 0.98725E+00_JPRB, 0.95554E+00_JPRB, 0.95909E+00_JPRB, 0.97297E+00_JPRB, 0.99207E+00_JPRB ,&
&  0.10180E+01_JPRB, 0.10712E+01_JPRB, 0.12377E+01_JPRB, 0.17791E+01_JPRB, 0.30136E+01_JPRB, 0.40768E+01_JPRB, 0.56984E+01_JPRB ,&
&  0.69196E+01_JPRB, 0.64520E+01_JPRB, 0.52579E+01_JPRB, 0.44026E+01_JPRB, 0.37594E+01_JPRB, 0.32964E+01_JPRB, 0.29448E+01_JPRB ,&
&  0.26776E+01_JPRB, 0.24874E+01_JPRB, 0.23262E+01_JPRB, 0.21497E+01_JPRB, 0.19960E+01_JPRB, 0.18749E+01_JPRB, 0.17603E+01_JPRB ,&
&  0.16273E+01_JPRB, 0.14954E+01_JPRB, 0.13747E+01_JPRB, 0.12686E+01_JPRB, 0.11839E+01_JPRB, 0.11622E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.11360E+01_JPRB, 0.11040E+01_JPRB, 0.11013E+01_JPRB, 0.11357E+01_JPRB, 0.11338E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10923E+01_JPRB, 0.10773E+01_JPRB, 0.10546E+01_JPRB, 0.99632E+00_JPRB, 0.10947E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.10708E+01_JPRB, 0.10294E+01_JPRB, 0.10186E+01_JPRB, 0.10024E+01_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.97236E+00_JPRB, 0.94908E+00_JPRB, 0.93399E+00_JPRB, 0.92364E+00_JPRB, 0.91477E+00_JPRB, 0.90582E+00_JPRB, 0.88435E+00_JPRB ,&
&  0.86128E+00_JPRB, 0.84770E+00_JPRB, 0.82777E+00_JPRB, 0.80089E+00_JPRB, 0.76001E+00_JPRB, 0.72602E+00_JPRB, 0.71411E+00_JPRB ,&
&  0.70765E+00_JPRB, 0.69761E+00_JPRB, 0.67947E+00_JPRB, 0.66714E+00_JPRB, 0.65607E+00_JPRB, 0.64323E+00_JPRB, 0.63480E+00_JPRB ,&
&  0.62194E+00_JPRB, 0.61709E+00_JPRB, 0.62062E+00_JPRB, 0.62879E+00_JPRB, 0.63632E+00_JPRB, 0.64520E+00_JPRB, 0.66327E+00_JPRB ,&
&  0.87699E+00_JPRB, 0.18157E+01_JPRB, 0.24454E+01_JPRB, 0.27978E+01_JPRB, 0.28105E+01_JPRB, 0.26035E+01_JPRB, 0.25960E+01_JPRB ,&
&  0.23206E+01_JPRB, 0.20399E+01_JPRB, 0.17566E+01_JPRB, 0.14954E+01_JPRB, 0.12872E+01_JPRB, 0.11742E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.10441E+01_JPRB, 0.98323E+00_JPRB, 0.93202E+00_JPRB, 0.88837E+00_JPRB, 0.87288E+00_JPRB, 0.89085E+00_JPRB, 0.92186E+00_JPRB ,&
&  0.93943E+00_JPRB, 0.96082E+00_JPRB, 0.10406E+01_JPRB, 0.13633E+01_JPRB, 0.23622E+01_JPRB, 0.29005E+01_JPRB, 0.43682E+01_JPRB ,&
&  0.52683E+01_JPRB, 0.47655E+01_JPRB, 0.40098E+01_JPRB, 0.34419E+01_JPRB, 0.29581E+01_JPRB, 0.25809E+01_JPRB, 0.23060E+01_JPRB ,&
&  0.21477E+01_JPRB, 0.20386E+01_JPRB, 0.19415E+01_JPRB, 0.18468E+01_JPRB, 0.17592E+01_JPRB, 0.16900E+01_JPRB, 0.16354E+01_JPRB ,&
&  0.15793E+01_JPRB, 0.15057E+01_JPRB, 0.14203E+01_JPRB, 0.13230E+01_JPRB, 0.12263E+01_JPRB, 0.11649E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.10870E+01_JPRB, 0.10727E+01_JPRB, 0.10805E+01_JPRB, 0.11115E+01_JPRB, 0.11225E+01_JPRB, 0.11139E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10498E+01_JPRB, 0.10273E+01_JPRB, 0.96853E+00_JPRB, 0.94732E+00_JPRB, 0.10284E+01_JPRB, 0.10171E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.99607E+00_JPRB, 0.98431E+00_JPRB, 0.98247E+00_JPRB, 0.99853E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.97896E+00_JPRB, 0.94836E+00_JPRB, 0.92573E+00_JPRB, 0.91038E+00_JPRB, 0.90113E+00_JPRB, 0.89052E+00_JPRB, 0.87267E+00_JPRB ,&
&  0.85280E+00_JPRB, 0.83357E+00_JPRB, 0.81659E+00_JPRB, 0.80398E+00_JPRB, 0.77731E+00_JPRB, 0.73124E+00_JPRB, 0.69473E+00_JPRB ,&
&  0.68186E+00_JPRB, 0.67189E+00_JPRB, 0.66278E+00_JPRB, 0.64833E+00_JPRB, 0.63693E+00_JPRB, 0.63156E+00_JPRB, 0.62614E+00_JPRB ,&
&  0.62069E+00_JPRB, 0.61553E+00_JPRB, 0.61730E+00_JPRB, 0.62268E+00_JPRB, 0.62720E+00_JPRB, 0.63115E+00_JPRB, 0.63979E+00_JPRB ,&
&  0.74769E+00_JPRB, 0.13208E+01_JPRB, 0.16177E+01_JPRB, 0.20602E+01_JPRB, 0.22032E+01_JPRB, 0.20459E+01_JPRB, 0.19435E+01_JPRB ,&
&  0.17924E+01_JPRB, 0.16180E+01_JPRB, 0.14590E+01_JPRB, 0.13096E+01_JPRB, 0.11448E+01_JPRB, 0.10452E+01_JPRB, 0.99696E+00_JPRB ,&
&  0.96398E+00_JPRB, 0.92199E+00_JPRB, 0.88053E+00_JPRB, 0.83907E+00_JPRB, 0.81199E+00_JPRB, 0.81806E+00_JPRB, 0.83642E+00_JPRB ,&
&  0.83898E+00_JPRB, 0.84144E+00_JPRB, 0.86215E+00_JPRB, 0.98768E+00_JPRB, 0.14779E+01_JPRB, 0.19818E+01_JPRB, 0.26608E+01_JPRB ,&
&  0.27300E+01_JPRB, 0.28874E+01_JPRB, 0.26655E+01_JPRB, 0.23946E+01_JPRB, 0.21461E+01_JPRB, 0.19522E+01_JPRB, 0.18053E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.16192E+01_JPRB, 0.15545E+01_JPRB, 0.15038E+01_JPRB, 0.14643E+01_JPRB, 0.14173E+01_JPRB, 0.13899E+01_JPRB ,&
&  0.13765E+01_JPRB, 0.13475E+01_JPRB, 0.13084E+01_JPRB, 0.12611E+01_JPRB, 0.12048E+01_JPRB, 0.11571E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.10517E+01_JPRB, 0.10310E+01_JPRB, 0.10356E+01_JPRB, 0.10455E+01_JPRB, 0.10520E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10241E+01_JPRB, 0.10106E+01_JPRB, 0.98727E+00_JPRB, 0.93624E+00_JPRB, 0.91544E+00_JPRB, 0.96618E+00_JPRB, 0.95691E+00_JPRB ,&
&  0.93951E+00_JPRB, 0.92600E+00_JPRB, 0.92967E+00_JPRB, 0.94405E+00_JPRB, 0.10036E+01_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.99268E+00_JPRB, 0.95835E+00_JPRB, 0.92003E+00_JPRB, 0.89391E+00_JPRB, 0.87481E+00_JPRB, 0.85757E+00_JPRB, 0.83612E+00_JPRB ,&
&  0.81548E+00_JPRB, 0.79618E+00_JPRB, 0.77965E+00_JPRB, 0.76694E+00_JPRB, 0.75355E+00_JPRB, 0.72224E+00_JPRB, 0.67917E+00_JPRB ,&
&  0.65039E+00_JPRB, 0.64014E+00_JPRB, 0.63359E+00_JPRB, 0.62237E+00_JPRB, 0.61197E+00_JPRB, 0.60755E+00_JPRB, 0.60402E+00_JPRB ,&
&  0.60430E+00_JPRB, 0.60596E+00_JPRB, 0.61039E+00_JPRB, 0.61236E+00_JPRB, 0.60923E+00_JPRB, 0.61512E+00_JPRB, 0.61610E+00_JPRB ,&
&  0.69472E+00_JPRB, 0.90528E+00_JPRB, 0.11294E+01_JPRB, 0.13842E+01_JPRB, 0.15724E+01_JPRB, 0.16231E+01_JPRB, 0.15739E+01_JPRB ,&
&  0.15017E+01_JPRB, 0.14265E+01_JPRB, 0.13203E+01_JPRB, 0.12196E+01_JPRB, 0.11067E+01_JPRB, 0.98821E+00_JPRB, 0.91944E+00_JPRB ,&
&  0.89404E+00_JPRB, 0.86606E+00_JPRB, 0.83994E+00_JPRB, 0.81215E+00_JPRB, 0.78808E+00_JPRB, 0.77871E+00_JPRB, 0.77897E+00_JPRB ,&
&  0.77314E+00_JPRB, 0.76483E+00_JPRB, 0.75687E+00_JPRB, 0.76541E+00_JPRB, 0.84651E+00_JPRB, 0.10856E+01_JPRB, 0.13988E+01_JPRB ,&
&  0.15334E+01_JPRB, 0.16355E+01_JPRB, 0.17037E+01_JPRB, 0.17037E+01_JPRB, 0.16293E+01_JPRB, 0.15181E+01_JPRB, 0.14008E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.12531E+01_JPRB, 0.12206E+01_JPRB, 0.12100E+01_JPRB, 0.12062E+01_JPRB, 0.11932E+01_JPRB, 0.11679E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11359E+01_JPRB, 0.11355E+01_JPRB, 0.11277E+01_JPRB, 0.11150E+01_JPRB, 0.10992E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10882E+01_JPRB, 0.10775E+01_JPRB, 0.10425E+01_JPRB, 0.10070E+01_JPRB, 0.99995E+00_JPRB, 0.99550E+00_JPRB, 0.98420E+00_JPRB ,&
&  0.97331E+00_JPRB, 0.97188E+00_JPRB, 0.95335E+00_JPRB, 0.94824E+00_JPRB, 0.96214E+00_JPRB, 0.93345E+00_JPRB, 0.91671E+00_JPRB ,&
&  0.89020E+00_JPRB, 0.87534E+00_JPRB, 0.88228E+00_JPRB, 0.90477E+00_JPRB, 0.97600E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.84701E+00_JPRB, 0.83908E+00_JPRB, 0.82880E+00_JPRB, 0.81908E+00_JPRB, 0.80422E+00_JPRB, 0.78852E+00_JPRB, 0.76915E+00_JPRB ,&
&  0.75054E+00_JPRB, 0.73285E+00_JPRB, 0.71848E+00_JPRB, 0.71364E+00_JPRB, 0.70508E+00_JPRB, 0.69208E+00_JPRB, 0.66183E+00_JPRB ,&
&  0.63671E+00_JPRB, 0.61794E+00_JPRB, 0.60629E+00_JPRB, 0.60021E+00_JPRB, 0.59550E+00_JPRB, 0.59050E+00_JPRB, 0.58819E+00_JPRB ,&
&  0.58884E+00_JPRB, 0.58846E+00_JPRB, 0.59079E+00_JPRB, 0.58960E+00_JPRB, 0.59109E+00_JPRB, 0.58622E+00_JPRB, 0.58217E+00_JPRB ,&
&  0.62441E+00_JPRB, 0.70964E+00_JPRB, 0.84354E+00_JPRB, 0.98944E+00_JPRB, 0.11505E+01_JPRB, 0.12619E+01_JPRB, 0.13341E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.13055E+01_JPRB, 0.12168E+01_JPRB, 0.11158E+01_JPRB, 0.10369E+01_JPRB, 0.95611E+00_JPRB, 0.87615E+00_JPRB ,&
&  0.83572E+00_JPRB, 0.81258E+00_JPRB, 0.79377E+00_JPRB, 0.77886E+00_JPRB, 0.76604E+00_JPRB, 0.75780E+00_JPRB, 0.75146E+00_JPRB ,&
&  0.73862E+00_JPRB, 0.72346E+00_JPRB, 0.70597E+00_JPRB, 0.69869E+00_JPRB, 0.70373E+00_JPRB, 0.75961E+00_JPRB, 0.91051E+00_JPRB ,&
&  0.99474E+00_JPRB, 0.10413E+01_JPRB, 0.10795E+01_JPRB, 0.11401E+01_JPRB, 0.11826E+01_JPRB, 0.11816E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.10101E+01_JPRB, 0.98352E+00_JPRB, 0.97374E+00_JPRB, 0.97631E+00_JPRB, 0.98049E+00_JPRB, 0.98256E+00_JPRB ,&
&  0.97078E+00_JPRB, 0.95829E+00_JPRB, 0.95441E+00_JPRB, 0.95630E+00_JPRB, 0.96437E+00_JPRB, 0.97168E+00_JPRB, 0.98343E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.10363E+01_JPRB, 0.10588E+01_JPRB, 0.10371E+01_JPRB, 0.99610E+00_JPRB, 0.96514E+00_JPRB, 0.94342E+00_JPRB ,&
&  0.92925E+00_JPRB, 0.92744E+00_JPRB, 0.93083E+00_JPRB, 0.94239E+00_JPRB, 0.93708E+00_JPRB, 0.91225E+00_JPRB, 0.88173E+00_JPRB ,&
&  0.85474E+00_JPRB, 0.83729E+00_JPRB, 0.83575E+00_JPRB, 0.84076E+00_JPRB, 0.82183E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.75061E+00_JPRB, 0.74421E+00_JPRB, 0.73680E+00_JPRB, 0.73174E+00_JPRB, 0.72242E+00_JPRB, 0.71186E+00_JPRB, 0.70031E+00_JPRB ,&
&  0.68584E+00_JPRB, 0.67156E+00_JPRB, 0.65990E+00_JPRB, 0.65656E+00_JPRB, 0.65565E+00_JPRB, 0.65540E+00_JPRB, 0.64740E+00_JPRB ,&
&  0.63316E+00_JPRB, 0.61943E+00_JPRB, 0.60722E+00_JPRB, 0.59884E+00_JPRB, 0.59474E+00_JPRB, 0.59022E+00_JPRB, 0.58486E+00_JPRB ,&
&  0.58054E+00_JPRB, 0.57934E+00_JPRB, 0.57690E+00_JPRB, 0.57419E+00_JPRB, 0.57035E+00_JPRB, 0.56313E+00_JPRB, 0.56122E+00_JPRB ,&
&  0.57716E+00_JPRB, 0.57414E+00_JPRB, 0.64634E+00_JPRB, 0.70951E+00_JPRB, 0.78215E+00_JPRB, 0.89615E+00_JPRB, 0.10271E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11366E+01_JPRB, 0.10964E+01_JPRB, 0.10240E+01_JPRB, 0.97119E+00_JPRB, 0.93287E+00_JPRB, 0.87942E+00_JPRB ,&
&  0.82677E+00_JPRB, 0.79080E+00_JPRB, 0.76324E+00_JPRB, 0.74148E+00_JPRB, 0.72548E+00_JPRB, 0.71169E+00_JPRB, 0.69876E+00_JPRB ,&
&  0.68225E+00_JPRB, 0.66591E+00_JPRB, 0.65252E+00_JPRB, 0.64634E+00_JPRB, 0.64856E+00_JPRB, 0.65803E+00_JPRB, 0.70020E+00_JPRB ,&
&  0.74679E+00_JPRB, 0.77604E+00_JPRB, 0.78290E+00_JPRB, 0.80945E+00_JPRB, 0.85195E+00_JPRB, 0.88744E+00_JPRB, 0.89723E+00_JPRB ,&
&  0.87478E+00_JPRB, 0.83846E+00_JPRB, 0.80661E+00_JPRB, 0.78289E+00_JPRB, 0.77096E+00_JPRB, 0.77367E+00_JPRB, 0.78743E+00_JPRB ,&
&  0.79437E+00_JPRB, 0.79356E+00_JPRB, 0.79463E+00_JPRB, 0.80081E+00_JPRB, 0.80996E+00_JPRB, 0.82550E+00_JPRB, 0.84715E+00_JPRB ,&
&  0.87463E+00_JPRB, 0.90708E+00_JPRB, 0.94318E+00_JPRB, 0.97037E+00_JPRB, 0.97530E+00_JPRB, 0.96707E+00_JPRB, 0.95082E+00_JPRB ,&
&  0.93263E+00_JPRB, 0.91699E+00_JPRB, 0.90103E+00_JPRB, 0.86992E+00_JPRB, 0.88919E+00_JPRB, 0.87674E+00_JPRB, 0.84773E+00_JPRB ,&
&  0.82138E+00_JPRB, 0.79681E+00_JPRB, 0.77853E+00_JPRB, 0.71612E+00_JPRB, 0.74937E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.67089E+00_JPRB, 0.66460E+00_JPRB, 0.65631E+00_JPRB, 0.64967E+00_JPRB, 0.64536E+00_JPRB, 0.63941E+00_JPRB, 0.63468E+00_JPRB ,&
&  0.62815E+00_JPRB, 0.62277E+00_JPRB, 0.61387E+00_JPRB, 0.60814E+00_JPRB, 0.60607E+00_JPRB, 0.60710E+00_JPRB, 0.60815E+00_JPRB ,&
&  0.60446E+00_JPRB, 0.59699E+00_JPRB, 0.58893E+00_JPRB, 0.58229E+00_JPRB, 0.57650E+00_JPRB, 0.56980E+00_JPRB, 0.56294E+00_JPRB ,&
&  0.55655E+00_JPRB, 0.55106E+00_JPRB, 0.54620E+00_JPRB, 0.54439E+00_JPRB, 0.54259E+00_JPRB, 0.53976E+00_JPRB, 0.53562E+00_JPRB ,&
&  0.54055E+00_JPRB, 0.48974E+00_JPRB, 0.51211E+00_JPRB, 0.53757E+00_JPRB, 0.57013E+00_JPRB, 0.64823E+00_JPRB, 0.75913E+00_JPRB ,&
&  0.86380E+00_JPRB, 0.93196E+00_JPRB, 0.94774E+00_JPRB, 0.91533E+00_JPRB, 0.87857E+00_JPRB, 0.85602E+00_JPRB, 0.83728E+00_JPRB ,&
&  0.80155E+00_JPRB, 0.76935E+00_JPRB, 0.73729E+00_JPRB, 0.70500E+00_JPRB, 0.67511E+00_JPRB, 0.64864E+00_JPRB, 0.62726E+00_JPRB ,&
&  0.61317E+00_JPRB, 0.60596E+00_JPRB, 0.60209E+00_JPRB, 0.60029E+00_JPRB, 0.60140E+00_JPRB, 0.60355E+00_JPRB, 0.60610E+00_JPRB ,&
&  0.61564E+00_JPRB, 0.63529E+00_JPRB, 0.64416E+00_JPRB, 0.65220E+00_JPRB, 0.67056E+00_JPRB, 0.69619E+00_JPRB, 0.71007E+00_JPRB ,&
&  0.71103E+00_JPRB, 0.70220E+00_JPRB, 0.68875E+00_JPRB, 0.67380E+00_JPRB, 0.66068E+00_JPRB, 0.64991E+00_JPRB, 0.64767E+00_JPRB ,&
&  0.64807E+00_JPRB, 0.65156E+00_JPRB, 0.65671E+00_JPRB, 0.66691E+00_JPRB, 0.67970E+00_JPRB, 0.69266E+00_JPRB, 0.70521E+00_JPRB ,&
&  0.71772E+00_JPRB, 0.73224E+00_JPRB, 0.75357E+00_JPRB, 0.77838E+00_JPRB, 0.79922E+00_JPRB, 0.80986E+00_JPRB, 0.81241E+00_JPRB ,&
&  0.80870E+00_JPRB, 0.80235E+00_JPRB, 0.79065E+00_JPRB, 0.78635E+00_JPRB, 0.77878E+00_JPRB, 0.77419E+00_JPRB, 0.76149E+00_JPRB ,&
&  0.74612E+00_JPRB, 0.72970E+00_JPRB, 0.64203E+00_JPRB, 0.67249E+00_JPRB, 0.67254E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.61413E+00_JPRB, 0.60465E+00_JPRB, 0.59600E+00_JPRB, 0.58784E+00_JPRB, 0.58203E+00_JPRB, 0.57832E+00_JPRB, 0.57716E+00_JPRB ,&
&  0.57882E+00_JPRB, 0.58108E+00_JPRB, 0.57986E+00_JPRB, 0.57355E+00_JPRB, 0.56727E+00_JPRB, 0.56381E+00_JPRB, 0.56309E+00_JPRB ,&
&  0.56178E+00_JPRB, 0.55817E+00_JPRB, 0.55311E+00_JPRB, 0.54807E+00_JPRB, 0.54195E+00_JPRB, 0.53638E+00_JPRB, 0.53121E+00_JPRB ,&
&  0.52659E+00_JPRB, 0.52366E+00_JPRB, 0.52104E+00_JPRB, 0.51884E+00_JPRB, 0.51776E+00_JPRB, 0.51556E+00_JPRB, 0.51722E+00_JPRB ,&
&  0.48740E+00_JPRB, 0.45484E+00_JPRB, 0.47003E+00_JPRB, 0.48423E+00_JPRB, 0.49865E+00_JPRB, 0.53538E+00_JPRB, 0.59583E+00_JPRB ,&
&  0.67520E+00_JPRB, 0.74506E+00_JPRB, 0.78812E+00_JPRB, 0.78599E+00_JPRB, 0.76601E+00_JPRB, 0.75433E+00_JPRB, 0.74868E+00_JPRB ,&
&  0.73040E+00_JPRB, 0.70829E+00_JPRB, 0.68810E+00_JPRB, 0.66429E+00_JPRB, 0.63885E+00_JPRB, 0.61641E+00_JPRB, 0.59882E+00_JPRB ,&
&  0.58569E+00_JPRB, 0.57788E+00_JPRB, 0.57464E+00_JPRB, 0.57179E+00_JPRB, 0.56882E+00_JPRB, 0.56730E+00_JPRB, 0.56446E+00_JPRB ,&
&  0.56338E+00_JPRB, 0.56445E+00_JPRB, 0.57032E+00_JPRB, 0.57507E+00_JPRB, 0.58197E+00_JPRB, 0.58855E+00_JPRB, 0.59145E+00_JPRB ,&
&  0.58920E+00_JPRB, 0.58681E+00_JPRB, 0.58695E+00_JPRB, 0.58381E+00_JPRB, 0.57484E+00_JPRB, 0.56495E+00_JPRB, 0.56062E+00_JPRB ,&
&  0.56063E+00_JPRB, 0.56183E+00_JPRB, 0.56401E+00_JPRB, 0.56895E+00_JPRB, 0.57612E+00_JPRB, 0.58431E+00_JPRB, 0.59248E+00_JPRB ,&
&  0.60046E+00_JPRB, 0.60771E+00_JPRB, 0.61534E+00_JPRB, 0.62570E+00_JPRB, 0.64034E+00_JPRB, 0.65421E+00_JPRB, 0.66567E+00_JPRB ,&
&  0.67415E+00_JPRB, 0.67972E+00_JPRB, 0.68352E+00_JPRB, 0.68830E+00_JPRB, 0.69181E+00_JPRB, 0.69256E+00_JPRB, 0.68695E+00_JPRB ,&
&  0.67544E+00_JPRB, 0.65983E+00_JPRB, 0.64914E+00_JPRB, 0.63478E+00_JPRB, 0.62276E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.54825E+00_JPRB, 0.54224E+00_JPRB, 0.53554E+00_JPRB, 0.52896E+00_JPRB, 0.52290E+00_JPRB, 0.52030E+00_JPRB, 0.52020E+00_JPRB ,&
&  0.52359E+00_JPRB, 0.52617E+00_JPRB, 0.52982E+00_JPRB, 0.52896E+00_JPRB, 0.52625E+00_JPRB, 0.52371E+00_JPRB, 0.52123E+00_JPRB ,&
&  0.51848E+00_JPRB, 0.51465E+00_JPRB, 0.51073E+00_JPRB, 0.50773E+00_JPRB, 0.50436E+00_JPRB, 0.49968E+00_JPRB, 0.49514E+00_JPRB ,&
&  0.49129E+00_JPRB, 0.48891E+00_JPRB, 0.48701E+00_JPRB, 0.48519E+00_JPRB, 0.48322E+00_JPRB, 0.48251E+00_JPRB, 0.48090E+00_JPRB ,&
&  0.48231E+00_JPRB, 0.45167E+00_JPRB, 0.46424E+00_JPRB, 0.46865E+00_JPRB, 0.47669E+00_JPRB, 0.49433E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.55258E+00_JPRB, 0.59628E+00_JPRB, 0.63379E+00_JPRB, 0.64871E+00_JPRB, 0.64414E+00_JPRB, 0.64152E+00_JPRB, 0.64588E+00_JPRB ,&
&  0.64419E+00_JPRB, 0.63708E+00_JPRB, 0.63125E+00_JPRB, 0.62264E+00_JPRB, 0.60916E+00_JPRB, 0.59126E+00_JPRB, 0.57512E+00_JPRB ,&
&  0.56518E+00_JPRB, 0.55919E+00_JPRB, 0.55360E+00_JPRB, 0.54738E+00_JPRB, 0.53869E+00_JPRB, 0.52987E+00_JPRB, 0.52192E+00_JPRB ,&
&  0.51467E+00_JPRB, 0.50650E+00_JPRB, 0.50178E+00_JPRB, 0.50049E+00_JPRB, 0.49976E+00_JPRB, 0.50114E+00_JPRB, 0.50193E+00_JPRB ,&
&  0.49987E+00_JPRB, 0.49595E+00_JPRB, 0.49400E+00_JPRB, 0.49449E+00_JPRB, 0.49427E+00_JPRB, 0.49372E+00_JPRB, 0.49288E+00_JPRB ,&
&  0.49413E+00_JPRB, 0.49559E+00_JPRB, 0.49578E+00_JPRB, 0.49640E+00_JPRB, 0.49865E+00_JPRB, 0.50311E+00_JPRB, 0.50821E+00_JPRB ,&
&  0.51379E+00_JPRB, 0.51983E+00_JPRB, 0.52547E+00_JPRB, 0.53110E+00_JPRB, 0.53917E+00_JPRB, 0.54781E+00_JPRB, 0.55823E+00_JPRB ,&
&  0.56951E+00_JPRB, 0.57939E+00_JPRB, 0.58786E+00_JPRB, 0.59483E+00_JPRB, 0.59869E+00_JPRB, 0.59840E+00_JPRB, 0.59558E+00_JPRB ,&
&  0.59088E+00_JPRB, 0.58064E+00_JPRB, 0.57150E+00_JPRB, 0.56179E+00_JPRB, 0.55329E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.49839E+00_JPRB, 0.49266E+00_JPRB, 0.48656E+00_JPRB, 0.48181E+00_JPRB, 0.47787E+00_JPRB, 0.47427E+00_JPRB, 0.47321E+00_JPRB ,&
&  0.47468E+00_JPRB, 0.47801E+00_JPRB, 0.48256E+00_JPRB, 0.48564E+00_JPRB, 0.48730E+00_JPRB, 0.48870E+00_JPRB, 0.48860E+00_JPRB ,&
&  0.48760E+00_JPRB, 0.48562E+00_JPRB, 0.48277E+00_JPRB, 0.47895E+00_JPRB, 0.47464E+00_JPRB, 0.47053E+00_JPRB, 0.46530E+00_JPRB ,&
&  0.46128E+00_JPRB, 0.45844E+00_JPRB, 0.45641E+00_JPRB, 0.45509E+00_JPRB, 0.45505E+00_JPRB, 0.45517E+00_JPRB, 0.45495E+00_JPRB ,&
&  0.45220E+00_JPRB, 0.44998E+00_JPRB, 0.45672E+00_JPRB, 0.45785E+00_JPRB, 0.46418E+00_JPRB, 0.47706E+00_JPRB, 0.48903E+00_JPRB ,&
&  0.49863E+00_JPRB, 0.51597E+00_JPRB, 0.53876E+00_JPRB, 0.55554E+00_JPRB, 0.55851E+00_JPRB, 0.55848E+00_JPRB, 0.56503E+00_JPRB ,&
&  0.56924E+00_JPRB, 0.56838E+00_JPRB, 0.56909E+00_JPRB, 0.56853E+00_JPRB, 0.56521E+00_JPRB, 0.56033E+00_JPRB, 0.55208E+00_JPRB ,&
&  0.54307E+00_JPRB, 0.53527E+00_JPRB, 0.52687E+00_JPRB, 0.51822E+00_JPRB, 0.50708E+00_JPRB, 0.49486E+00_JPRB, 0.48185E+00_JPRB ,&
&  0.46882E+00_JPRB, 0.45936E+00_JPRB, 0.45317E+00_JPRB, 0.44832E+00_JPRB, 0.44445E+00_JPRB, 0.44295E+00_JPRB, 0.44130E+00_JPRB ,&
&  0.44040E+00_JPRB, 0.44028E+00_JPRB, 0.43867E+00_JPRB, 0.43792E+00_JPRB, 0.43920E+00_JPRB, 0.44059E+00_JPRB, 0.44363E+00_JPRB ,&
&  0.44534E+00_JPRB, 0.44679E+00_JPRB, 0.44792E+00_JPRB, 0.44799E+00_JPRB, 0.44711E+00_JPRB, 0.44846E+00_JPRB, 0.45144E+00_JPRB ,&
&  0.45472E+00_JPRB, 0.45970E+00_JPRB, 0.46517E+00_JPRB, 0.46999E+00_JPRB, 0.47461E+00_JPRB, 0.48177E+00_JPRB, 0.49096E+00_JPRB ,&
&  0.50126E+00_JPRB, 0.51228E+00_JPRB, 0.52199E+00_JPRB, 0.52914E+00_JPRB, 0.53105E+00_JPRB, 0.53061E+00_JPRB, 0.52942E+00_JPRB ,&
&  0.52609E+00_JPRB, 0.51847E+00_JPRB, 0.51333E+00_JPRB, 0.50889E+00_JPRB, 0.50369E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.44131E+00_JPRB, 0.43895E+00_JPRB, 0.43697E+00_JPRB, 0.43596E+00_JPRB, 0.43551E+00_JPRB, 0.43566E+00_JPRB, 0.43679E+00_JPRB ,&
&  0.43920E+00_JPRB, 0.44172E+00_JPRB, 0.44712E+00_JPRB, 0.45068E+00_JPRB, 0.45278E+00_JPRB, 0.45508E+00_JPRB, 0.45651E+00_JPRB ,&
&  0.45569E+00_JPRB, 0.45477E+00_JPRB, 0.45326E+00_JPRB, 0.45029E+00_JPRB, 0.44561E+00_JPRB, 0.44245E+00_JPRB, 0.43927E+00_JPRB ,&
&  0.43488E+00_JPRB, 0.42982E+00_JPRB, 0.42517E+00_JPRB, 0.42204E+00_JPRB, 0.42079E+00_JPRB, 0.41911E+00_JPRB, 0.41849E+00_JPRB ,&
&  0.41694E+00_JPRB, 0.42272E+00_JPRB, 0.41133E+00_JPRB, 0.42101E+00_JPRB, 0.42946E+00_JPRB, 0.43795E+00_JPRB, 0.44709E+00_JPRB ,&
&  0.45358E+00_JPRB, 0.46121E+00_JPRB, 0.47421E+00_JPRB, 0.48517E+00_JPRB, 0.48953E+00_JPRB, 0.48816E+00_JPRB, 0.48992E+00_JPRB ,&
&  0.49129E+00_JPRB, 0.48826E+00_JPRB, 0.48790E+00_JPRB, 0.49097E+00_JPRB, 0.49416E+00_JPRB, 0.49476E+00_JPRB, 0.49401E+00_JPRB ,&
&  0.49324E+00_JPRB, 0.48852E+00_JPRB, 0.47992E+00_JPRB, 0.47036E+00_JPRB, 0.46116E+00_JPRB, 0.45111E+00_JPRB, 0.43913E+00_JPRB ,&
&  0.42621E+00_JPRB, 0.41582E+00_JPRB, 0.40724E+00_JPRB, 0.40147E+00_JPRB, 0.39831E+00_JPRB, 0.39596E+00_JPRB, 0.39410E+00_JPRB ,&
&  0.39375E+00_JPRB, 0.39387E+00_JPRB, 0.39338E+00_JPRB, 0.39267E+00_JPRB, 0.39341E+00_JPRB, 0.39473E+00_JPRB, 0.39879E+00_JPRB ,&
&  0.40362E+00_JPRB, 0.40697E+00_JPRB, 0.40932E+00_JPRB, 0.41112E+00_JPRB, 0.41028E+00_JPRB, 0.41133E+00_JPRB, 0.41363E+00_JPRB ,&
&  0.41603E+00_JPRB, 0.41891E+00_JPRB, 0.42090E+00_JPRB, 0.42306E+00_JPRB, 0.42606E+00_JPRB, 0.43045E+00_JPRB, 0.43717E+00_JPRB ,&
&  0.44399E+00_JPRB, 0.45128E+00_JPRB, 0.45892E+00_JPRB, 0.46488E+00_JPRB, 0.46828E+00_JPRB, 0.46901E+00_JPRB, 0.46858E+00_JPRB ,&
&  0.46577E+00_JPRB, 0.45987E+00_JPRB, 0.45374E+00_JPRB, 0.44890E+00_JPRB, 0.44472E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.39956E+00_JPRB, 0.39982E+00_JPRB, 0.40106E+00_JPRB, 0.40134E+00_JPRB, 0.40126E+00_JPRB, 0.40236E+00_JPRB, 0.40590E+00_JPRB ,&
&  0.40933E+00_JPRB, 0.41210E+00_JPRB, 0.41511E+00_JPRB, 0.41751E+00_JPRB, 0.41942E+00_JPRB, 0.42247E+00_JPRB, 0.42487E+00_JPRB ,&
&  0.42579E+00_JPRB, 0.42501E+00_JPRB, 0.42365E+00_JPRB, 0.42156E+00_JPRB, 0.41853E+00_JPRB, 0.41505E+00_JPRB, 0.41188E+00_JPRB ,&
&  0.40785E+00_JPRB, 0.40326E+00_JPRB, 0.39863E+00_JPRB, 0.39575E+00_JPRB, 0.39411E+00_JPRB, 0.39291E+00_JPRB, 0.39276E+00_JPRB ,&
&  0.39258E+00_JPRB, 0.39483E+00_JPRB, 0.39763E+00_JPRB, 0.40124E+00_JPRB, 0.40470E+00_JPRB, 0.40926E+00_JPRB, 0.41423E+00_JPRB ,&
&  0.41896E+00_JPRB, 0.42170E+00_JPRB, 0.42525E+00_JPRB, 0.42639E+00_JPRB, 0.43154E+00_JPRB, 0.43559E+00_JPRB, 0.43787E+00_JPRB ,&
&  0.43735E+00_JPRB, 0.43082E+00_JPRB, 0.42627E+00_JPRB, 0.42686E+00_JPRB, 0.42956E+00_JPRB, 0.43232E+00_JPRB, 0.43482E+00_JPRB ,&
&  0.43563E+00_JPRB, 0.43420E+00_JPRB, 0.42957E+00_JPRB, 0.42287E+00_JPRB, 0.41561E+00_JPRB, 0.40757E+00_JPRB, 0.39721E+00_JPRB ,&
&  0.38773E+00_JPRB, 0.38003E+00_JPRB, 0.37371E+00_JPRB, 0.36809E+00_JPRB, 0.36411E+00_JPRB, 0.36014E+00_JPRB, 0.35903E+00_JPRB ,&
&  0.35976E+00_JPRB, 0.36212E+00_JPRB, 0.36326E+00_JPRB, 0.36234E+00_JPRB, 0.36098E+00_JPRB, 0.36112E+00_JPRB, 0.36419E+00_JPRB ,&
&  0.36969E+00_JPRB, 0.37423E+00_JPRB, 0.37640E+00_JPRB, 0.37718E+00_JPRB, 0.37720E+00_JPRB, 0.37917E+00_JPRB, 0.38235E+00_JPRB ,&
&  0.38424E+00_JPRB, 0.38522E+00_JPRB, 0.38635E+00_JPRB, 0.38609E+00_JPRB, 0.38797E+00_JPRB, 0.39133E+00_JPRB, 0.39525E+00_JPRB ,&
&  0.39917E+00_JPRB, 0.40410E+00_JPRB, 0.40894E+00_JPRB, 0.41257E+00_JPRB, 0.41550E+00_JPRB, 0.41534E+00_JPRB, 0.41498E+00_JPRB ,&
&  0.41416E+00_JPRB, 0.41164E+00_JPRB, 0.40894E+00_JPRB, 0.40560E+00_JPRB, 0.40186E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.36527E+00_JPRB, 0.36867E+00_JPRB, 0.37280E+00_JPRB, 0.37350E+00_JPRB, 0.37359E+00_JPRB, 0.37499E+00_JPRB, 0.37839E+00_JPRB ,&
&  0.38116E+00_JPRB, 0.38262E+00_JPRB, 0.38367E+00_JPRB, 0.38465E+00_JPRB, 0.38635E+00_JPRB, 0.38930E+00_JPRB, 0.39106E+00_JPRB ,&
&  0.39102E+00_JPRB, 0.39053E+00_JPRB, 0.38959E+00_JPRB, 0.38852E+00_JPRB, 0.38635E+00_JPRB, 0.38318E+00_JPRB, 0.38023E+00_JPRB ,&
&  0.37631E+00_JPRB, 0.37254E+00_JPRB, 0.36938E+00_JPRB, 0.36665E+00_JPRB, 0.36462E+00_JPRB, 0.36365E+00_JPRB, 0.36446E+00_JPRB ,&
&  0.36575E+00_JPRB, 0.36773E+00_JPRB, 0.37050E+00_JPRB, 0.37393E+00_JPRB, 0.37730E+00_JPRB, 0.38112E+00_JPRB, 0.38453E+00_JPRB ,&
&  0.38727E+00_JPRB, 0.38986E+00_JPRB, 0.39210E+00_JPRB, 0.39060E+00_JPRB, 0.38926E+00_JPRB, 0.39221E+00_JPRB, 0.39581E+00_JPRB ,&
&  0.39459E+00_JPRB, 0.38994E+00_JPRB, 0.38411E+00_JPRB, 0.38123E+00_JPRB, 0.38073E+00_JPRB, 0.37990E+00_JPRB, 0.37862E+00_JPRB ,&
&  0.37772E+00_JPRB, 0.37664E+00_JPRB, 0.37448E+00_JPRB, 0.37103E+00_JPRB, 0.36616E+00_JPRB, 0.36017E+00_JPRB, 0.35451E+00_JPRB ,&
&  0.35098E+00_JPRB, 0.34845E+00_JPRB, 0.34785E+00_JPRB, 0.34498E+00_JPRB, 0.33968E+00_JPRB, 0.33443E+00_JPRB, 0.33331E+00_JPRB ,&
&  0.33391E+00_JPRB, 0.33560E+00_JPRB, 0.33640E+00_JPRB, 0.33555E+00_JPRB, 0.33556E+00_JPRB, 0.33570E+00_JPRB, 0.33718E+00_JPRB ,&
&  0.34259E+00_JPRB, 0.34681E+00_JPRB, 0.34731E+00_JPRB, 0.34656E+00_JPRB, 0.34473E+00_JPRB, 0.34458E+00_JPRB, 0.34610E+00_JPRB ,&
&  0.34786E+00_JPRB, 0.34901E+00_JPRB, 0.34970E+00_JPRB, 0.34925E+00_JPRB, 0.34927E+00_JPRB, 0.35070E+00_JPRB, 0.35264E+00_JPRB ,&
&  0.35510E+00_JPRB, 0.35815E+00_JPRB, 0.36079E+00_JPRB, 0.36336E+00_JPRB, 0.36570E+00_JPRB, 0.36674E+00_JPRB, 0.36770E+00_JPRB ,&
&  0.36870E+00_JPRB, 0.36856E+00_JPRB, 0.36779E+00_JPRB, 0.36700E+00_JPRB, 0.36607E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.33877E+00_JPRB, 0.34367E+00_JPRB, 0.34959E+00_JPRB, 0.35060E+00_JPRB, 0.35224E+00_JPRB, 0.35431E+00_JPRB, 0.35776E+00_JPRB ,&
&  0.36055E+00_JPRB, 0.36161E+00_JPRB, 0.36081E+00_JPRB, 0.35985E+00_JPRB, 0.35969E+00_JPRB, 0.36021E+00_JPRB, 0.36038E+00_JPRB ,&
&  0.36063E+00_JPRB, 0.36096E+00_JPRB, 0.36083E+00_JPRB, 0.36031E+00_JPRB, 0.35849E+00_JPRB, 0.35587E+00_JPRB, 0.35282E+00_JPRB ,&
&  0.34832E+00_JPRB, 0.34504E+00_JPRB, 0.34338E+00_JPRB, 0.34222E+00_JPRB, 0.34062E+00_JPRB, 0.34056E+00_JPRB, 0.34202E+00_JPRB ,&
&  0.34566E+00_JPRB, 0.34846E+00_JPRB, 0.35210E+00_JPRB, 0.35572E+00_JPRB, 0.35775E+00_JPRB, 0.35929E+00_JPRB, 0.36108E+00_JPRB ,&
&  0.36222E+00_JPRB, 0.36251E+00_JPRB, 0.36277E+00_JPRB, 0.36312E+00_JPRB, 0.36142E+00_JPRB, 0.36310E+00_JPRB, 0.36619E+00_JPRB ,&
&  0.36741E+00_JPRB, 0.36471E+00_JPRB, 0.36018E+00_JPRB, 0.35667E+00_JPRB, 0.35318E+00_JPRB, 0.35045E+00_JPRB, 0.34806E+00_JPRB ,&
&  0.34614E+00_JPRB, 0.34486E+00_JPRB, 0.34412E+00_JPRB, 0.34171E+00_JPRB, 0.33551E+00_JPRB, 0.32959E+00_JPRB, 0.32634E+00_JPRB ,&
&  0.32483E+00_JPRB, 0.32431E+00_JPRB, 0.32557E+00_JPRB, 0.32387E+00_JPRB, 0.31877E+00_JPRB, 0.31443E+00_JPRB, 0.31285E+00_JPRB ,&
&  0.31156E+00_JPRB, 0.31142E+00_JPRB, 0.31282E+00_JPRB, 0.31359E+00_JPRB, 0.31392E+00_JPRB, 0.31358E+00_JPRB, 0.31503E+00_JPRB ,&
&  0.31733E+00_JPRB, 0.31984E+00_JPRB, 0.32003E+00_JPRB, 0.31719E+00_JPRB, 0.31308E+00_JPRB, 0.31043E+00_JPRB, 0.31073E+00_JPRB ,&
&  0.31343E+00_JPRB, 0.31598E+00_JPRB, 0.31723E+00_JPRB, 0.31800E+00_JPRB, 0.31939E+00_JPRB, 0.32206E+00_JPRB, 0.32540E+00_JPRB ,&
&  0.32693E+00_JPRB, 0.32840E+00_JPRB, 0.33044E+00_JPRB, 0.33252E+00_JPRB, 0.33386E+00_JPRB, 0.33452E+00_JPRB, 0.33462E+00_JPRB ,&
&  0.33392E+00_JPRB, 0.33445E+00_JPRB, 0.33584E+00_JPRB, 0.33753E+00_JPRB, 0.33840E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.31963E+00_JPRB, 0.32289E+00_JPRB, 0.32592E+00_JPRB, 0.32732E+00_JPRB, 0.32883E+00_JPRB, 0.33075E+00_JPRB, 0.33436E+00_JPRB ,&
&  0.33883E+00_JPRB, 0.34071E+00_JPRB, 0.33920E+00_JPRB, 0.33636E+00_JPRB, 0.33475E+00_JPRB, 0.33436E+00_JPRB, 0.33462E+00_JPRB ,&
&  0.33481E+00_JPRB, 0.33487E+00_JPRB, 0.33401E+00_JPRB, 0.33185E+00_JPRB, 0.32869E+00_JPRB, 0.32534E+00_JPRB, 0.32252E+00_JPRB ,&
&  0.32137E+00_JPRB, 0.32214E+00_JPRB, 0.32378E+00_JPRB, 0.32336E+00_JPRB, 0.32321E+00_JPRB, 0.32454E+00_JPRB, 0.32693E+00_JPRB ,&
&  0.33109E+00_JPRB, 0.33278E+00_JPRB, 0.33398E+00_JPRB, 0.33035E+00_JPRB, 0.33394E+00_JPRB, 0.33532E+00_JPRB, 0.33550E+00_JPRB ,&
&  0.33648E+00_JPRB, 0.33737E+00_JPRB, 0.33863E+00_JPRB, 0.33824E+00_JPRB, 0.33452E+00_JPRB, 0.33440E+00_JPRB, 0.33783E+00_JPRB ,&
&  0.34058E+00_JPRB, 0.33919E+00_JPRB, 0.33581E+00_JPRB, 0.33266E+00_JPRB, 0.32993E+00_JPRB, 0.32774E+00_JPRB, 0.32591E+00_JPRB ,&
&  0.32507E+00_JPRB, 0.32397E+00_JPRB, 0.32135E+00_JPRB, 0.31533E+00_JPRB, 0.30874E+00_JPRB, 0.30424E+00_JPRB, 0.30085E+00_JPRB ,&
&  0.30038E+00_JPRB, 0.30218E+00_JPRB, 0.30410E+00_JPRB, 0.30412E+00_JPRB, 0.30209E+00_JPRB, 0.29848E+00_JPRB, 0.29399E+00_JPRB ,&
&  0.28884E+00_JPRB, 0.28739E+00_JPRB, 0.28769E+00_JPRB, 0.28729E+00_JPRB, 0.28785E+00_JPRB, 0.28985E+00_JPRB, 0.29222E+00_JPRB ,&
&  0.29524E+00_JPRB, 0.29494E+00_JPRB, 0.29314E+00_JPRB, 0.29031E+00_JPRB, 0.28750E+00_JPRB, 0.28738E+00_JPRB, 0.28866E+00_JPRB ,&
&  0.29011E+00_JPRB, 0.29157E+00_JPRB, 0.29341E+00_JPRB, 0.29558E+00_JPRB, 0.29802E+00_JPRB, 0.30092E+00_JPRB, 0.30309E+00_JPRB ,&
&  0.30294E+00_JPRB, 0.30169E+00_JPRB, 0.30298E+00_JPRB, 0.30543E+00_JPRB, 0.30676E+00_JPRB, 0.30746E+00_JPRB, 0.30786E+00_JPRB ,&
&  0.30719E+00_JPRB, 0.30671E+00_JPRB, 0.30836E+00_JPRB, 0.31280E+00_JPRB, 0.31686E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.30227E+00_JPRB, 0.30368E+00_JPRB, 0.30381E+00_JPRB, 0.30431E+00_JPRB, 0.30497E+00_JPRB, 0.30689E+00_JPRB, 0.31122E+00_JPRB ,&
&  0.31562E+00_JPRB, 0.31762E+00_JPRB, 0.31577E+00_JPRB, 0.31320E+00_JPRB, 0.31189E+00_JPRB, 0.31145E+00_JPRB, 0.31155E+00_JPRB ,&
&  0.31162E+00_JPRB, 0.31074E+00_JPRB, 0.30777E+00_JPRB, 0.30355E+00_JPRB, 0.30005E+00_JPRB, 0.29857E+00_JPRB, 0.29831E+00_JPRB ,&
&  0.30159E+00_JPRB, 0.30375E+00_JPRB, 0.30573E+00_JPRB, 0.30795E+00_JPRB, 0.30894E+00_JPRB, 0.30912E+00_JPRB, 0.31106E+00_JPRB ,&
&  0.31185E+00_JPRB, 0.30832E+00_JPRB, 0.30262E+00_JPRB, 0.30579E+00_JPRB, 0.30829E+00_JPRB, 0.30976E+00_JPRB, 0.30898E+00_JPRB ,&
&  0.30952E+00_JPRB, 0.31174E+00_JPRB, 0.31426E+00_JPRB, 0.31616E+00_JPRB, 0.31569E+00_JPRB, 0.31370E+00_JPRB, 0.31426E+00_JPRB ,&
&  0.31576E+00_JPRB, 0.31482E+00_JPRB, 0.31153E+00_JPRB, 0.30845E+00_JPRB, 0.30497E+00_JPRB, 0.30328E+00_JPRB, 0.30309E+00_JPRB ,&
&  0.30237E+00_JPRB, 0.30013E+00_JPRB, 0.29576E+00_JPRB, 0.29126E+00_JPRB, 0.28785E+00_JPRB, 0.28410E+00_JPRB, 0.28181E+00_JPRB ,&
&  0.28171E+00_JPRB, 0.28258E+00_JPRB, 0.28241E+00_JPRB, 0.28013E+00_JPRB, 0.27287E+00_JPRB, 0.26444E+00_JPRB, 0.25695E+00_JPRB ,&
&  0.25555E+00_JPRB, 0.25950E+00_JPRB, 0.26638E+00_JPRB, 0.26992E+00_JPRB, 0.27328E+00_JPRB, 0.27518E+00_JPRB, 0.27623E+00_JPRB ,&
&  0.27282E+00_JPRB, 0.26112E+00_JPRB, 0.25593E+00_JPRB, 0.25229E+00_JPRB, 0.24919E+00_JPRB, 0.24965E+00_JPRB, 0.25200E+00_JPRB ,&
&  0.25650E+00_JPRB, 0.26096E+00_JPRB, 0.26464E+00_JPRB, 0.26710E+00_JPRB, 0.26810E+00_JPRB, 0.26857E+00_JPRB, 0.26796E+00_JPRB ,&
&  0.26880E+00_JPRB, 0.27350E+00_JPRB, 0.28199E+00_JPRB, 0.28789E+00_JPRB, 0.28909E+00_JPRB, 0.29059E+00_JPRB, 0.29027E+00_JPRB ,&
&  0.29011E+00_JPRB, 0.29208E+00_JPRB, 0.29310E+00_JPRB, 0.29574E+00_JPRB, 0.29930E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.28834E+00_JPRB, 0.28958E+00_JPRB, 0.28965E+00_JPRB, 0.28847E+00_JPRB, 0.28698E+00_JPRB, 0.28683E+00_JPRB, 0.28805E+00_JPRB ,&
&  0.29093E+00_JPRB, 0.29289E+00_JPRB, 0.29315E+00_JPRB, 0.29301E+00_JPRB, 0.29208E+00_JPRB, 0.29021E+00_JPRB, 0.28883E+00_JPRB ,&
&  0.28786E+00_JPRB, 0.28748E+00_JPRB, 0.28713E+00_JPRB, 0.28709E+00_JPRB, 0.28698E+00_JPRB, 0.28746E+00_JPRB, 0.28774E+00_JPRB ,&
&  0.29088E+00_JPRB, 0.29036E+00_JPRB, 0.28957E+00_JPRB, 0.28937E+00_JPRB, 0.28932E+00_JPRB, 0.28958E+00_JPRB, 0.28940E+00_JPRB ,&
&  0.28540E+00_JPRB, 0.27793E+00_JPRB, 0.27231E+00_JPRB, 0.27947E+00_JPRB, 0.28908E+00_JPRB, 0.28925E+00_JPRB, 0.28983E+00_JPRB ,&
&  0.29137E+00_JPRB, 0.29421E+00_JPRB, 0.29661E+00_JPRB, 0.29760E+00_JPRB, 0.29657E+00_JPRB, 0.29538E+00_JPRB, 0.29536E+00_JPRB ,&
&  0.29605E+00_JPRB, 0.29433E+00_JPRB, 0.28838E+00_JPRB, 0.28122E+00_JPRB, 0.27398E+00_JPRB, 0.27139E+00_JPRB, 0.27006E+00_JPRB ,&
&  0.26730E+00_JPRB, 0.26557E+00_JPRB, 0.26510E+00_JPRB, 0.26473E+00_JPRB, 0.26526E+00_JPRB, 0.26245E+00_JPRB, 0.25770E+00_JPRB ,&
&  0.25540E+00_JPRB, 0.25082E+00_JPRB, 0.24597E+00_JPRB, 0.24084E+00_JPRB, 0.23663E+00_JPRB, 0.23138E+00_JPRB, 0.22772E+00_JPRB ,&
&  0.22740E+00_JPRB, 0.23267E+00_JPRB, 0.24384E+00_JPRB, 0.25340E+00_JPRB, 0.25575E+00_JPRB, 0.24872E+00_JPRB, 0.24331E+00_JPRB ,&
&  0.23564E+00_JPRB, 0.22834E+00_JPRB, 0.22417E+00_JPRB, 0.22346E+00_JPRB, 0.22376E+00_JPRB, 0.22476E+00_JPRB, 0.22705E+00_JPRB ,&
&  0.23008E+00_JPRB, 0.23370E+00_JPRB, 0.23637E+00_JPRB, 0.23859E+00_JPRB, 0.24020E+00_JPRB, 0.24063E+00_JPRB, 0.24029E+00_JPRB ,&
&  0.24003E+00_JPRB, 0.24159E+00_JPRB, 0.24407E+00_JPRB, 0.24887E+00_JPRB, 0.25145E+00_JPRB, 0.25517E+00_JPRB, 0.25963E+00_JPRB ,&
&  0.26665E+00_JPRB, 0.27432E+00_JPRB, 0.27898E+00_JPRB, 0.28229E+00_JPRB, 0.28569E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.27383E+00_JPRB, 0.27889E+00_JPRB, 0.28097E+00_JPRB, 0.28006E+00_JPRB, 0.27934E+00_JPRB, 0.27948E+00_JPRB, 0.27976E+00_JPRB ,&
&  0.28049E+00_JPRB, 0.27975E+00_JPRB, 0.27803E+00_JPRB, 0.27661E+00_JPRB, 0.27511E+00_JPRB, 0.27328E+00_JPRB, 0.27236E+00_JPRB ,&
&  0.27175E+00_JPRB, 0.27242E+00_JPRB, 0.27297E+00_JPRB, 0.27462E+00_JPRB, 0.27543E+00_JPRB, 0.27702E+00_JPRB, 0.27473E+00_JPRB ,&
&  0.27033E+00_JPRB, 0.26751E+00_JPRB, 0.26997E+00_JPRB, 0.27019E+00_JPRB, 0.27070E+00_JPRB, 0.27126E+00_JPRB, 0.27284E+00_JPRB ,&
&  0.26972E+00_JPRB, 0.26725E+00_JPRB, 0.26597E+00_JPRB, 0.27368E+00_JPRB, 0.27987E+00_JPRB, 0.27731E+00_JPRB, 0.27744E+00_JPRB ,&
&  0.27949E+00_JPRB, 0.28212E+00_JPRB, 0.28343E+00_JPRB, 0.28338E+00_JPRB, 0.28199E+00_JPRB, 0.28166E+00_JPRB, 0.28037E+00_JPRB ,&
&  0.27688E+00_JPRB, 0.27404E+00_JPRB, 0.26324E+00_JPRB, 0.24495E+00_JPRB, 0.23950E+00_JPRB, 0.23746E+00_JPRB, 0.23362E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22654E+00_JPRB, 0.22682E+00_JPRB, 0.22421E+00_JPRB, 0.22424E+00_JPRB, 0.22402E+00_JPRB, 0.22240E+00_JPRB ,&
&  0.22050E+00_JPRB, 0.21935E+00_JPRB, 0.21776E+00_JPRB, 0.21500E+00_JPRB, 0.21287E+00_JPRB, 0.21261E+00_JPRB, 0.21378E+00_JPRB ,&
&  0.21611E+00_JPRB, 0.21917E+00_JPRB, 0.22868E+00_JPRB, 0.23409E+00_JPRB, 0.23223E+00_JPRB, 0.22938E+00_JPRB, 0.22226E+00_JPRB ,&
&  0.21482E+00_JPRB, 0.21265E+00_JPRB, 0.21113E+00_JPRB, 0.20978E+00_JPRB, 0.21023E+00_JPRB, 0.21146E+00_JPRB, 0.21268E+00_JPRB ,&
&  0.21434E+00_JPRB, 0.21647E+00_JPRB, 0.21780E+00_JPRB, 0.21859E+00_JPRB, 0.21930E+00_JPRB, 0.21962E+00_JPRB, 0.22017E+00_JPRB ,&
&  0.22007E+00_JPRB, 0.22159E+00_JPRB, 0.22265E+00_JPRB, 0.22529E+00_JPRB, 0.22698E+00_JPRB, 0.22945E+00_JPRB, 0.23522E+00_JPRB ,&
&  0.23924E+00_JPRB, 0.25196E+00_JPRB, 0.26787E+00_JPRB, 0.27231E+00_JPRB, 0.26818E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.26397E+00_JPRB, 0.26626E+00_JPRB, 0.26759E+00_JPRB, 0.26565E+00_JPRB, 0.26550E+00_JPRB, 0.26559E+00_JPRB, 0.26663E+00_JPRB ,&
&  0.26699E+00_JPRB, 0.26622E+00_JPRB, 0.26582E+00_JPRB, 0.26363E+00_JPRB, 0.26198E+00_JPRB, 0.25578E+00_JPRB, 0.25363E+00_JPRB ,&
&  0.25177E+00_JPRB, 0.25294E+00_JPRB, 0.25360E+00_JPRB, 0.25575E+00_JPRB, 0.25533E+00_JPRB, 0.25355E+00_JPRB, 0.24956E+00_JPRB ,&
&  0.24704E+00_JPRB, 0.24610E+00_JPRB, 0.24476E+00_JPRB, 0.24558E+00_JPRB, 0.24822E+00_JPRB, 0.25070E+00_JPRB, 0.25251E+00_JPRB ,&
&  0.25416E+00_JPRB, 0.25711E+00_JPRB, 0.25990E+00_JPRB, 0.26324E+00_JPRB, 0.26905E+00_JPRB, 0.26847E+00_JPRB, 0.26989E+00_JPRB ,&
&  0.27091E+00_JPRB, 0.27175E+00_JPRB, 0.27206E+00_JPRB, 0.27110E+00_JPRB, 0.26890E+00_JPRB, 0.26216E+00_JPRB, 0.26026E+00_JPRB ,&
&  0.24845E+00_JPRB, 0.24140E+00_JPRB, 0.23537E+00_JPRB, 0.23114E+00_JPRB, 0.22431E+00_JPRB, 0.22075E+00_JPRB, 0.21669E+00_JPRB ,&
&  0.21324E+00_JPRB, 0.20946E+00_JPRB, 0.20673E+00_JPRB, 0.20573E+00_JPRB, 0.20540E+00_JPRB, 0.20509E+00_JPRB, 0.20413E+00_JPRB ,&
&  0.20432E+00_JPRB, 0.20382E+00_JPRB, 0.20347E+00_JPRB, 0.20311E+00_JPRB, 0.20286E+00_JPRB, 0.20302E+00_JPRB, 0.20401E+00_JPRB ,&
&  0.20641E+00_JPRB, 0.20841E+00_JPRB, 0.21069E+00_JPRB, 0.21323E+00_JPRB, 0.21292E+00_JPRB, 0.21085E+00_JPRB, 0.20771E+00_JPRB ,&
&  0.20485E+00_JPRB, 0.20297E+00_JPRB, 0.20098E+00_JPRB, 0.20041E+00_JPRB, 0.20115E+00_JPRB, 0.20204E+00_JPRB, 0.20370E+00_JPRB ,&
&  0.20516E+00_JPRB, 0.20639E+00_JPRB, 0.20721E+00_JPRB, 0.20769E+00_JPRB, 0.20847E+00_JPRB, 0.20915E+00_JPRB, 0.21036E+00_JPRB ,&
&  0.21170E+00_JPRB, 0.21333E+00_JPRB, 0.21473E+00_JPRB, 0.21674E+00_JPRB, 0.21997E+00_JPRB, 0.22448E+00_JPRB, 0.23249E+00_JPRB ,&
&  0.23931E+00_JPRB, 0.25692E+00_JPRB, 0.25930E+00_JPRB, 0.26249E+00_JPRB, 0.26466E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.25302E+00_JPRB, 0.25488E+00_JPRB, 0.25546E+00_JPRB, 0.25624E+00_JPRB, 0.25599E+00_JPRB, 0.25593E+00_JPRB, 0.25652E+00_JPRB ,&
&  0.25223E+00_JPRB, 0.24952E+00_JPRB, 0.24475E+00_JPRB, 0.24069E+00_JPRB, 0.23674E+00_JPRB, 0.23369E+00_JPRB, 0.23187E+00_JPRB ,&
&  0.23080E+00_JPRB, 0.23103E+00_JPRB, 0.23068E+00_JPRB, 0.23094E+00_JPRB, 0.22928E+00_JPRB, 0.22791E+00_JPRB, 0.22678E+00_JPRB ,&
&  0.22577E+00_JPRB, 0.22699E+00_JPRB, 0.22810E+00_JPRB, 0.23198E+00_JPRB, 0.23741E+00_JPRB, 0.24327E+00_JPRB, 0.24779E+00_JPRB ,&
&  0.25074E+00_JPRB, 0.25314E+00_JPRB, 0.25653E+00_JPRB, 0.25802E+00_JPRB, 0.25990E+00_JPRB, 0.26150E+00_JPRB, 0.26181E+00_JPRB ,&
&  0.26308E+00_JPRB, 0.26256E+00_JPRB, 0.26208E+00_JPRB, 0.26162E+00_JPRB, 0.25208E+00_JPRB, 0.24552E+00_JPRB, 0.23860E+00_JPRB ,&
&  0.23226E+00_JPRB, 0.22571E+00_JPRB, 0.21976E+00_JPRB, 0.21669E+00_JPRB, 0.21269E+00_JPRB, 0.21042E+00_JPRB, 0.20846E+00_JPRB ,&
&  0.20694E+00_JPRB, 0.20536E+00_JPRB, 0.20307E+00_JPRB, 0.20083E+00_JPRB, 0.19951E+00_JPRB, 0.19839E+00_JPRB, 0.19694E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19615E+00_JPRB, 0.19626E+00_JPRB, 0.19594E+00_JPRB, 0.19575E+00_JPRB, 0.19557E+00_JPRB, 0.19559E+00_JPRB ,&
&  0.19609E+00_JPRB, 0.19718E+00_JPRB, 0.19772E+00_JPRB, 0.19844E+00_JPRB, 0.19849E+00_JPRB, 0.19775E+00_JPRB, 0.19717E+00_JPRB ,&
&  0.19640E+00_JPRB, 0.19582E+00_JPRB, 0.19587E+00_JPRB, 0.19571E+00_JPRB, 0.19628E+00_JPRB, 0.19704E+00_JPRB, 0.19776E+00_JPRB ,&
&  0.19882E+00_JPRB, 0.19953E+00_JPRB, 0.20030E+00_JPRB, 0.20138E+00_JPRB, 0.20222E+00_JPRB, 0.20333E+00_JPRB, 0.20459E+00_JPRB ,&
&  0.20543E+00_JPRB, 0.20740E+00_JPRB, 0.20905E+00_JPRB, 0.21116E+00_JPRB, 0.21558E+00_JPRB, 0.21945E+00_JPRB, 0.22925E+00_JPRB ,&
&  0.23702E+00_JPRB, 0.25231E+00_JPRB, 0.25310E+00_JPRB, 0.25028E+00_JPRB, 0.25158E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.24670E+00_JPRB, 0.24806E+00_JPRB, 0.24823E+00_JPRB, 0.24887E+00_JPRB, 0.24882E+00_JPRB, 0.25010E+00_JPRB, 0.24974E+00_JPRB ,&
&  0.24981E+00_JPRB, 0.24625E+00_JPRB, 0.24410E+00_JPRB, 0.24131E+00_JPRB, 0.23887E+00_JPRB, 0.23577E+00_JPRB, 0.23279E+00_JPRB ,&
&  0.23049E+00_JPRB, 0.22839E+00_JPRB, 0.22606E+00_JPRB, 0.22298E+00_JPRB, 0.21985E+00_JPRB, 0.21775E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21538E+00_JPRB, 0.21691E+00_JPRB, 0.21927E+00_JPRB, 0.22345E+00_JPRB, 0.22871E+00_JPRB, 0.23271E+00_JPRB, 0.23821E+00_JPRB ,&
&  0.24403E+00_JPRB, 0.24588E+00_JPRB, 0.24945E+00_JPRB, 0.25087E+00_JPRB, 0.25059E+00_JPRB, 0.25149E+00_JPRB, 0.24983E+00_JPRB ,&
&  0.24984E+00_JPRB, 0.24549E+00_JPRB, 0.25003E+00_JPRB, 0.24545E+00_JPRB, 0.23757E+00_JPRB, 0.23461E+00_JPRB, 0.22798E+00_JPRB ,&
&  0.22392E+00_JPRB, 0.21912E+00_JPRB, 0.21614E+00_JPRB, 0.21458E+00_JPRB, 0.21064E+00_JPRB, 0.20861E+00_JPRB, 0.20530E+00_JPRB ,&
&  0.20223E+00_JPRB, 0.20140E+00_JPRB, 0.19962E+00_JPRB, 0.19904E+00_JPRB, 0.19815E+00_JPRB, 0.19679E+00_JPRB, 0.19531E+00_JPRB ,&
&  0.19436E+00_JPRB, 0.19359E+00_JPRB, 0.19327E+00_JPRB, 0.19274E+00_JPRB, 0.19193E+00_JPRB, 0.19110E+00_JPRB, 0.19009E+00_JPRB ,&
&  0.18949E+00_JPRB, 0.18929E+00_JPRB, 0.18921E+00_JPRB, 0.18939E+00_JPRB, 0.18954E+00_JPRB, 0.18942E+00_JPRB, 0.18970E+00_JPRB ,&
&  0.19007E+00_JPRB, 0.19083E+00_JPRB, 0.19136E+00_JPRB, 0.19209E+00_JPRB, 0.19268E+00_JPRB, 0.19346E+00_JPRB, 0.19393E+00_JPRB ,&
&  0.19454E+00_JPRB, 0.19557E+00_JPRB, 0.19647E+00_JPRB, 0.19756E+00_JPRB, 0.19870E+00_JPRB, 0.19963E+00_JPRB, 0.20125E+00_JPRB ,&
&  0.20288E+00_JPRB, 0.20450E+00_JPRB, 0.20691E+00_JPRB, 0.20923E+00_JPRB, 0.21223E+00_JPRB, 0.21700E+00_JPRB, 0.22227E+00_JPRB ,&
&  0.22890E+00_JPRB, 0.23836E+00_JPRB, 0.23861E+00_JPRB, 0.24300E+00_JPRB, 0.24508E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.24576E+00_JPRB, 0.24713E+00_JPRB, 0.24823E+00_JPRB, 0.24877E+00_JPRB, 0.24958E+00_JPRB, 0.24956E+00_JPRB, 0.24934E+00_JPRB ,&
&  0.24602E+00_JPRB, 0.24472E+00_JPRB, 0.24269E+00_JPRB, 0.24038E+00_JPRB, 0.23835E+00_JPRB, 0.23501E+00_JPRB, 0.23219E+00_JPRB ,&
&  0.22963E+00_JPRB, 0.22672E+00_JPRB, 0.22407E+00_JPRB, 0.22089E+00_JPRB, 0.21802E+00_JPRB, 0.21535E+00_JPRB, 0.21433E+00_JPRB ,&
&  0.21401E+00_JPRB, 0.21489E+00_JPRB, 0.21719E+00_JPRB, 0.21882E+00_JPRB, 0.22110E+00_JPRB, 0.22393E+00_JPRB, 0.22816E+00_JPRB ,&
&  0.23108E+00_JPRB, 0.23489E+00_JPRB, 0.23727E+00_JPRB, 0.23785E+00_JPRB, 0.23718E+00_JPRB, 0.23687E+00_JPRB, 0.23589E+00_JPRB ,&
&  0.23361E+00_JPRB, 0.23131E+00_JPRB, 0.22938E+00_JPRB, 0.22727E+00_JPRB, 0.22506E+00_JPRB, 0.22259E+00_JPRB, 0.21939E+00_JPRB ,&
&  0.21658E+00_JPRB, 0.21410E+00_JPRB, 0.21166E+00_JPRB, 0.20971E+00_JPRB, 0.20773E+00_JPRB, 0.20591E+00_JPRB, 0.20378E+00_JPRB ,&
&  0.20213E+00_JPRB, 0.20073E+00_JPRB, 0.19894E+00_JPRB, 0.19762E+00_JPRB, 0.19607E+00_JPRB, 0.19479E+00_JPRB, 0.19338E+00_JPRB ,&
&  0.19237E+00_JPRB, 0.19154E+00_JPRB, 0.19088E+00_JPRB, 0.19027E+00_JPRB, 0.18997E+00_JPRB, 0.18970E+00_JPRB, 0.18936E+00_JPRB ,&
&  0.18910E+00_JPRB, 0.18931E+00_JPRB, 0.18952E+00_JPRB, 0.18972E+00_JPRB, 0.18996E+00_JPRB, 0.18962E+00_JPRB, 0.18971E+00_JPRB ,&
&  0.19005E+00_JPRB, 0.19037E+00_JPRB, 0.19025E+00_JPRB, 0.19062E+00_JPRB, 0.19130E+00_JPRB, 0.19213E+00_JPRB, 0.19346E+00_JPRB ,&
&  0.19450E+00_JPRB, 0.19558E+00_JPRB, 0.19662E+00_JPRB, 0.19768E+00_JPRB, 0.19912E+00_JPRB, 0.20068E+00_JPRB, 0.20228E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20597E+00_JPRB, 0.20780E+00_JPRB, 0.20967E+00_JPRB, 0.21274E+00_JPRB, 0.21543E+00_JPRB, 0.21963E+00_JPRB ,&
&  0.22378E+00_JPRB, 0.22487E+00_JPRB, 0.23959E+00_JPRB, 0.24062E+00_JPRB, 0.24230E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.22403E+00_JPRB, 0.22615E+00_JPRB, 0.22727E+00_JPRB, 0.22832E+00_JPRB, 0.22387E+00_JPRB, 0.23588E+00_JPRB, 0.23345E+00_JPRB ,&
&  0.23296E+00_JPRB, 0.22854E+00_JPRB, 0.22958E+00_JPRB, 0.22923E+00_JPRB, 0.22793E+00_JPRB, 0.22641E+00_JPRB, 0.22459E+00_JPRB ,&
&  0.22276E+00_JPRB, 0.22036E+00_JPRB, 0.21854E+00_JPRB, 0.21702E+00_JPRB, 0.21562E+00_JPRB, 0.21444E+00_JPRB, 0.21326E+00_JPRB ,&
&  0.21332E+00_JPRB, 0.21424E+00_JPRB, 0.21559E+00_JPRB, 0.21457E+00_JPRB, 0.21490E+00_JPRB, 0.21579E+00_JPRB, 0.21708E+00_JPRB ,&
&  0.21808E+00_JPRB, 0.21966E+00_JPRB, 0.22071E+00_JPRB, 0.22107E+00_JPRB, 0.22203E+00_JPRB, 0.22132E+00_JPRB, 0.22081E+00_JPRB ,&
&  0.21995E+00_JPRB, 0.21875E+00_JPRB, 0.21724E+00_JPRB, 0.21593E+00_JPRB, 0.21450E+00_JPRB, 0.21272E+00_JPRB, 0.21088E+00_JPRB ,&
&  0.20924E+00_JPRB, 0.20780E+00_JPRB, 0.20617E+00_JPRB, 0.20484E+00_JPRB, 0.20357E+00_JPRB, 0.20231E+00_JPRB, 0.20111E+00_JPRB ,&
&  0.20015E+00_JPRB, 0.19930E+00_JPRB, 0.19838E+00_JPRB, 0.19747E+00_JPRB, 0.19649E+00_JPRB, 0.19555E+00_JPRB, 0.19465E+00_JPRB ,&
&  0.19432E+00_JPRB, 0.19405E+00_JPRB, 0.19376E+00_JPRB, 0.19343E+00_JPRB, 0.19344E+00_JPRB, 0.19341E+00_JPRB, 0.19331E+00_JPRB ,&
&  0.19318E+00_JPRB, 0.19293E+00_JPRB, 0.19286E+00_JPRB, 0.19279E+00_JPRB, 0.19271E+00_JPRB, 0.19264E+00_JPRB, 0.19276E+00_JPRB ,&
&  0.19296E+00_JPRB, 0.19324E+00_JPRB, 0.19383E+00_JPRB, 0.19445E+00_JPRB, 0.19511E+00_JPRB, 0.19571E+00_JPRB, 0.19658E+00_JPRB ,&
&  0.19756E+00_JPRB, 0.19848E+00_JPRB, 0.19926E+00_JPRB, 0.19991E+00_JPRB, 0.20106E+00_JPRB, 0.20216E+00_JPRB, 0.20324E+00_JPRB ,&
&  0.20442E+00_JPRB, 0.20593E+00_JPRB, 0.20750E+00_JPRB, 0.20913E+00_JPRB, 0.21106E+00_JPRB, 0.21257E+00_JPRB, 0.21413E+00_JPRB ,&
&  0.21598E+00_JPRB, 0.21951E+00_JPRB, 0.22166E+00_JPRB, 0.22296E+00_JPRB, 0.22423E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.20799E+00_JPRB, 0.20811E+00_JPRB, 0.20811E+00_JPRB, 0.20800E+00_JPRB, 0.20764E+00_JPRB, 0.20752E+00_JPRB, 0.20741E+00_JPRB ,&
&  0.20729E+00_JPRB, 0.20700E+00_JPRB, 0.20722E+00_JPRB, 0.20730E+00_JPRB, 0.20731E+00_JPRB, 0.20728E+00_JPRB, 0.20708E+00_JPRB ,&
&  0.20666E+00_JPRB, 0.20614E+00_JPRB, 0.20484E+00_JPRB, 0.20460E+00_JPRB, 0.20457E+00_JPRB, 0.20479E+00_JPRB, 0.20494E+00_JPRB ,&
&  0.20524E+00_JPRB, 0.20554E+00_JPRB, 0.20580E+00_JPRB, 0.20661E+00_JPRB, 0.20695E+00_JPRB, 0.20733E+00_JPRB, 0.20771E+00_JPRB ,&
&  0.20807E+00_JPRB, 0.20850E+00_JPRB, 0.20884E+00_JPRB, 0.20919E+00_JPRB, 0.20954E+00_JPRB, 0.20950E+00_JPRB, 0.20925E+00_JPRB ,&
&  0.20886E+00_JPRB, 0.20824E+00_JPRB, 0.20758E+00_JPRB, 0.20697E+00_JPRB, 0.20642E+00_JPRB, 0.20585E+00_JPRB, 0.20531E+00_JPRB ,&
&  0.20474E+00_JPRB, 0.20410E+00_JPRB, 0.20343E+00_JPRB, 0.20282E+00_JPRB, 0.20220E+00_JPRB, 0.20166E+00_JPRB, 0.20104E+00_JPRB ,&
&  0.20058E+00_JPRB, 0.20010E+00_JPRB, 0.19965E+00_JPRB, 0.19917E+00_JPRB, 0.19876E+00_JPRB, 0.19835E+00_JPRB, 0.19793E+00_JPRB ,&
&  0.19750E+00_JPRB, 0.19730E+00_JPRB, 0.19701E+00_JPRB, 0.19677E+00_JPRB, 0.19653E+00_JPRB, 0.19637E+00_JPRB, 0.19613E+00_JPRB ,&
&  0.19590E+00_JPRB, 0.19569E+00_JPRB, 0.19562E+00_JPRB, 0.19547E+00_JPRB, 0.19545E+00_JPRB, 0.19542E+00_JPRB, 0.19551E+00_JPRB ,&
&  0.19558E+00_JPRB, 0.19569E+00_JPRB, 0.19588E+00_JPRB, 0.19623E+00_JPRB, 0.19652E+00_JPRB, 0.19684E+00_JPRB, 0.19716E+00_JPRB ,&
&  0.19753E+00_JPRB, 0.19791E+00_JPRB, 0.19831E+00_JPRB, 0.19868E+00_JPRB, 0.19917E+00_JPRB, 0.19967E+00_JPRB, 0.20023E+00_JPRB ,&
&  0.20075E+00_JPRB, 0.20132E+00_JPRB, 0.20182E+00_JPRB, 0.20242E+00_JPRB, 0.20294E+00_JPRB, 0.20325E+00_JPRB, 0.20355E+00_JPRB ,&
&  0.20406E+00_JPRB, 0.20455E+00_JPRB, 0.20513E+00_JPRB, 0.20565E+00_JPRB, 0.20656E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB, 0.19964E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB, 0.71460E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.70472E+00_JPRB, 0.70498E+00_JPRB, 0.70519E+00_JPRB, 0.70539E+00_JPRB, 0.70557E+00_JPRB, 0.70575E+00_JPRB, 0.70596E+00_JPRB ,&
&  0.70608E+00_JPRB, 0.70640E+00_JPRB, 0.70685E+00_JPRB, 0.70736E+00_JPRB, 0.70783E+00_JPRB, 0.70831E+00_JPRB, 0.70877E+00_JPRB ,&
&  0.70923E+00_JPRB, 0.70950E+00_JPRB, 0.70981E+00_JPRB, 0.71036E+00_JPRB, 0.71107E+00_JPRB, 0.71191E+00_JPRB, 0.71312E+00_JPRB ,&
&  0.71360E+00_JPRB, 0.71344E+00_JPRB, 0.71336E+00_JPRB, 0.71404E+00_JPRB, 0.71390E+00_JPRB, 0.71384E+00_JPRB, 0.71388E+00_JPRB ,&
&  0.71384E+00_JPRB, 0.71373E+00_JPRB, 0.71347E+00_JPRB, 0.71322E+00_JPRB, 0.71314E+00_JPRB, 0.71332E+00_JPRB, 0.71359E+00_JPRB ,&
&  0.71399E+00_JPRB, 0.71433E+00_JPRB, 0.71515E+00_JPRB, 0.71602E+00_JPRB, 0.71702E+00_JPRB, 0.71772E+00_JPRB, 0.71852E+00_JPRB ,&
&  0.71906E+00_JPRB, 0.71967E+00_JPRB, 0.72056E+00_JPRB, 0.72176E+00_JPRB, 0.72304E+00_JPRB, 0.72423E+00_JPRB, 0.72515E+00_JPRB ,&
&  0.72651E+00_JPRB, 0.72831E+00_JPRB, 0.73026E+00_JPRB, 0.73238E+00_JPRB, 0.73400E+00_JPRB, 0.73571E+00_JPRB, 0.73757E+00_JPRB ,&
&  0.73914E+00_JPRB, 0.74000E+00_JPRB, 0.74102E+00_JPRB, 0.74209E+00_JPRB, 0.74282E+00_JPRB, 0.74209E+00_JPRB, 0.74153E+00_JPRB ,&
&  0.74103E+00_JPRB, 0.74019E+00_JPRB, 0.73802E+00_JPRB, 0.73618E+00_JPRB, 0.73450E+00_JPRB, 0.73278E+00_JPRB, 0.73082E+00_JPRB ,&
&  0.72907E+00_JPRB, 0.72746E+00_JPRB, 0.72572E+00_JPRB, 0.72385E+00_JPRB, 0.72219E+00_JPRB, 0.72061E+00_JPRB, 0.71908E+00_JPRB ,&
&  0.71766E+00_JPRB, 0.71645E+00_JPRB, 0.71529E+00_JPRB, 0.71423E+00_JPRB, 0.71303E+00_JPRB, 0.71189E+00_JPRB, 0.71074E+00_JPRB ,&
&  0.70971E+00_JPRB, 0.70863E+00_JPRB, 0.70765E+00_JPRB, 0.70669E+00_JPRB, 0.70603E+00_JPRB, 0.70559E+00_JPRB, 0.70521E+00_JPRB ,&
&  0.70490E+00_JPRB, 0.70473E+00_JPRB, 0.70471E+00_JPRB, 0.70466E+00_JPRB, 0.70461E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.70040E+00_JPRB, 0.70115E+00_JPRB, 0.70181E+00_JPRB, 0.70237E+00_JPRB, 0.70309E+00_JPRB, 0.70390E+00_JPRB, 0.70461E+00_JPRB ,&
&  0.70517E+00_JPRB, 0.70605E+00_JPRB, 0.70706E+00_JPRB, 0.70808E+00_JPRB, 0.70903E+00_JPRB, 0.71006E+00_JPRB, 0.71100E+00_JPRB ,&
&  0.71192E+00_JPRB, 0.71280E+00_JPRB, 0.71371E+00_JPRB, 0.71454E+00_JPRB, 0.71535E+00_JPRB, 0.71587E+00_JPRB, 0.71804E+00_JPRB ,&
&  0.71985E+00_JPRB, 0.72060E+00_JPRB, 0.71978E+00_JPRB, 0.71369E+00_JPRB, 0.70927E+00_JPRB, 0.70527E+00_JPRB, 0.70194E+00_JPRB ,&
&  0.69850E+00_JPRB, 0.69848E+00_JPRB, 0.69900E+00_JPRB, 0.69961E+00_JPRB, 0.70250E+00_JPRB, 0.70202E+00_JPRB, 0.70127E+00_JPRB ,&
&  0.69932E+00_JPRB, 0.69439E+00_JPRB, 0.69522E+00_JPRB, 0.69622E+00_JPRB, 0.69727E+00_JPRB, 0.70006E+00_JPRB, 0.70221E+00_JPRB ,&
&  0.70479E+00_JPRB, 0.70760E+00_JPRB, 0.71268E+00_JPRB, 0.71828E+00_JPRB, 0.72335E+00_JPRB, 0.72761E+00_JPRB, 0.73163E+00_JPRB ,&
&  0.73519E+00_JPRB, 0.73891E+00_JPRB, 0.74298E+00_JPRB, 0.74777E+00_JPRB, 0.75153E+00_JPRB, 0.75558E+00_JPRB, 0.75883E+00_JPRB ,&
&  0.76065E+00_JPRB, 0.76130E+00_JPRB, 0.76341E+00_JPRB, 0.76655E+00_JPRB, 0.76979E+00_JPRB, 0.76993E+00_JPRB, 0.76881E+00_JPRB ,&
&  0.76657E+00_JPRB, 0.76432E+00_JPRB, 0.76053E+00_JPRB, 0.75718E+00_JPRB, 0.75358E+00_JPRB, 0.74991E+00_JPRB, 0.74559E+00_JPRB ,&
&  0.74132E+00_JPRB, 0.73718E+00_JPRB, 0.73394E+00_JPRB, 0.73080E+00_JPRB, 0.72784E+00_JPRB, 0.72507E+00_JPRB, 0.72327E+00_JPRB ,&
&  0.72137E+00_JPRB, 0.71974E+00_JPRB, 0.71799E+00_JPRB, 0.71623E+00_JPRB, 0.71338E+00_JPRB, 0.71059E+00_JPRB, 0.70762E+00_JPRB ,&
&  0.70519E+00_JPRB, 0.70277E+00_JPRB, 0.70081E+00_JPRB, 0.69914E+00_JPRB, 0.69842E+00_JPRB, 0.69805E+00_JPRB, 0.69792E+00_JPRB ,&
&  0.69788E+00_JPRB, 0.69827E+00_JPRB, 0.69882E+00_JPRB, 0.69929E+00_JPRB, 0.69979E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.70525E+00_JPRB, 0.70642E+00_JPRB, 0.70756E+00_JPRB, 0.70821E+00_JPRB, 0.70952E+00_JPRB, 0.71071E+00_JPRB, 0.71189E+00_JPRB ,&
&  0.71305E+00_JPRB, 0.71389E+00_JPRB, 0.71461E+00_JPRB, 0.71544E+00_JPRB, 0.71613E+00_JPRB, 0.71758E+00_JPRB, 0.71890E+00_JPRB ,&
&  0.72005E+00_JPRB, 0.72073E+00_JPRB, 0.72260E+00_JPRB, 0.72430E+00_JPRB, 0.72515E+00_JPRB, 0.72581E+00_JPRB, 0.72092E+00_JPRB ,&
&  0.71783E+00_JPRB, 0.71574E+00_JPRB, 0.71308E+00_JPRB, 0.70697E+00_JPRB, 0.70391E+00_JPRB, 0.68780E+00_JPRB, 0.69342E+00_JPRB ,&
&  0.71517E+00_JPRB, 0.71397E+00_JPRB, 0.70918E+00_JPRB, 0.70098E+00_JPRB, 0.67175E+00_JPRB, 0.65939E+00_JPRB, 0.65729E+00_JPRB ,&
&  0.64110E+00_JPRB, 0.65722E+00_JPRB, 0.66199E+00_JPRB, 0.66411E+00_JPRB, 0.66796E+00_JPRB, 0.66528E+00_JPRB, 0.66570E+00_JPRB ,&
&  0.67655E+00_JPRB, 0.67700E+00_JPRB, 0.68322E+00_JPRB, 0.69097E+00_JPRB, 0.69939E+00_JPRB, 0.70734E+00_JPRB, 0.71638E+00_JPRB ,&
&  0.72359E+00_JPRB, 0.72892E+00_JPRB, 0.73267E+00_JPRB, 0.73409E+00_JPRB, 0.73588E+00_JPRB, 0.73807E+00_JPRB, 0.74254E+00_JPRB ,&
&  0.74948E+00_JPRB, 0.75680E+00_JPRB, 0.76055E+00_JPRB, 0.75933E+00_JPRB, 0.76061E+00_JPRB, 0.76143E+00_JPRB, 0.76223E+00_JPRB ,&
&  0.76185E+00_JPRB, 0.76064E+00_JPRB, 0.75703E+00_JPRB, 0.75611E+00_JPRB, 0.75539E+00_JPRB, 0.75341E+00_JPRB, 0.75024E+00_JPRB ,&
&  0.74631E+00_JPRB, 0.74081E+00_JPRB, 0.73637E+00_JPRB, 0.73307E+00_JPRB, 0.73025E+00_JPRB, 0.72789E+00_JPRB, 0.72714E+00_JPRB ,&
&  0.72529E+00_JPRB, 0.72391E+00_JPRB, 0.72113E+00_JPRB, 0.71721E+00_JPRB, 0.71284E+00_JPRB, 0.70922E+00_JPRB, 0.70532E+00_JPRB ,&
&  0.70309E+00_JPRB, 0.70166E+00_JPRB, 0.70089E+00_JPRB, 0.69957E+00_JPRB, 0.69945E+00_JPRB, 0.69944E+00_JPRB, 0.70002E+00_JPRB ,&
&  0.70063E+00_JPRB, 0.70166E+00_JPRB, 0.70239E+00_JPRB, 0.70337E+00_JPRB, 0.70390E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.70766E+00_JPRB, 0.70772E+00_JPRB, 0.70816E+00_JPRB, 0.70843E+00_JPRB, 0.70952E+00_JPRB, 0.71037E+00_JPRB, 0.71281E+00_JPRB ,&
&  0.71511E+00_JPRB, 0.71740E+00_JPRB, 0.71957E+00_JPRB, 0.72138E+00_JPRB, 0.72317E+00_JPRB, 0.72397E+00_JPRB, 0.72437E+00_JPRB ,&
&  0.72468E+00_JPRB, 0.72469E+00_JPRB, 0.72584E+00_JPRB, 0.72721E+00_JPRB, 0.72755E+00_JPRB, 0.72861E+00_JPRB, 0.72346E+00_JPRB ,&
&  0.71952E+00_JPRB, 0.71684E+00_JPRB, 0.69969E+00_JPRB, 0.70856E+00_JPRB, 0.70715E+00_JPRB, 0.70863E+00_JPRB, 0.70560E+00_JPRB ,&
&  0.68589E+00_JPRB, 0.67720E+00_JPRB, 0.65567E+00_JPRB, 0.63823E+00_JPRB, 0.64065E+00_JPRB, 0.62936E+00_JPRB, 0.63152E+00_JPRB ,&
&  0.62964E+00_JPRB, 0.62512E+00_JPRB, 0.62316E+00_JPRB, 0.62159E+00_JPRB, 0.62438E+00_JPRB, 0.62628E+00_JPRB, 0.62853E+00_JPRB ,&
&  0.63532E+00_JPRB, 0.64117E+00_JPRB, 0.64995E+00_JPRB, 0.65105E+00_JPRB, 0.66259E+00_JPRB, 0.67561E+00_JPRB, 0.68911E+00_JPRB ,&
&  0.69775E+00_JPRB, 0.69637E+00_JPRB, 0.70043E+00_JPRB, 0.70668E+00_JPRB, 0.71184E+00_JPRB, 0.71741E+00_JPRB, 0.72616E+00_JPRB ,&
&  0.73735E+00_JPRB, 0.74772E+00_JPRB, 0.75119E+00_JPRB, 0.74937E+00_JPRB, 0.74614E+00_JPRB, 0.74343E+00_JPRB, 0.74348E+00_JPRB ,&
&  0.74352E+00_JPRB, 0.74652E+00_JPRB, 0.74762E+00_JPRB, 0.75062E+00_JPRB, 0.75232E+00_JPRB, 0.75630E+00_JPRB, 0.75885E+00_JPRB ,&
&  0.76033E+00_JPRB, 0.76017E+00_JPRB, 0.76260E+00_JPRB, 0.75689E+00_JPRB, 0.75576E+00_JPRB, 0.75364E+00_JPRB, 0.74737E+00_JPRB ,&
&  0.74116E+00_JPRB, 0.73632E+00_JPRB, 0.73123E+00_JPRB, 0.72645E+00_JPRB, 0.72202E+00_JPRB, 0.71882E+00_JPRB, 0.71651E+00_JPRB ,&
&  0.71522E+00_JPRB, 0.71344E+00_JPRB, 0.71205E+00_JPRB, 0.70847E+00_JPRB, 0.70782E+00_JPRB, 0.70552E+00_JPRB, 0.70433E+00_JPRB ,&
&  0.70348E+00_JPRB, 0.70434E+00_JPRB, 0.70445E+00_JPRB, 0.70580E+00_JPRB, 0.70719E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.70130E+00_JPRB, 0.70414E+00_JPRB, 0.70949E+00_JPRB, 0.71407E+00_JPRB, 0.71853E+00_JPRB, 0.72209E+00_JPRB, 0.72402E+00_JPRB ,&
&  0.72550E+00_JPRB, 0.72600E+00_JPRB, 0.72656E+00_JPRB, 0.72739E+00_JPRB, 0.72882E+00_JPRB, 0.72996E+00_JPRB, 0.73148E+00_JPRB ,&
&  0.73200E+00_JPRB, 0.73228E+00_JPRB, 0.73229E+00_JPRB, 0.73233E+00_JPRB, 0.73276E+00_JPRB, 0.73450E+00_JPRB, 0.73352E+00_JPRB ,&
&  0.73382E+00_JPRB, 0.72898E+00_JPRB, 0.71630E+00_JPRB, 0.72084E+00_JPRB, 0.72237E+00_JPRB, 0.72382E+00_JPRB, 0.71572E+00_JPRB ,&
&  0.69274E+00_JPRB, 0.65701E+00_JPRB, 0.67756E+00_JPRB, 0.66553E+00_JPRB, 0.65269E+00_JPRB, 0.63926E+00_JPRB, 0.61853E+00_JPRB ,&
&  0.60594E+00_JPRB, 0.58941E+00_JPRB, 0.57706E+00_JPRB, 0.57268E+00_JPRB, 0.57199E+00_JPRB, 0.57779E+00_JPRB, 0.58597E+00_JPRB ,&
&  0.58942E+00_JPRB, 0.60016E+00_JPRB, 0.60698E+00_JPRB, 0.61627E+00_JPRB, 0.62788E+00_JPRB, 0.64294E+00_JPRB, 0.64889E+00_JPRB ,&
&  0.65934E+00_JPRB, 0.66324E+00_JPRB, 0.67355E+00_JPRB, 0.68753E+00_JPRB, 0.70632E+00_JPRB, 0.71803E+00_JPRB, 0.72565E+00_JPRB ,&
&  0.73461E+00_JPRB, 0.73928E+00_JPRB, 0.74196E+00_JPRB, 0.74316E+00_JPRB, 0.74625E+00_JPRB, 0.74972E+00_JPRB, 0.75351E+00_JPRB ,&
&  0.75707E+00_JPRB, 0.76296E+00_JPRB, 0.76931E+00_JPRB, 0.77668E+00_JPRB, 0.78111E+00_JPRB, 0.78902E+00_JPRB, 0.79835E+00_JPRB ,&
&  0.80712E+00_JPRB, 0.81191E+00_JPRB, 0.82193E+00_JPRB, 0.81389E+00_JPRB, 0.81073E+00_JPRB, 0.79544E+00_JPRB, 0.78467E+00_JPRB ,&
&  0.77635E+00_JPRB, 0.76610E+00_JPRB, 0.75736E+00_JPRB, 0.74978E+00_JPRB, 0.74386E+00_JPRB, 0.73869E+00_JPRB, 0.73349E+00_JPRB ,&
&  0.72828E+00_JPRB, 0.72181E+00_JPRB, 0.71791E+00_JPRB, 0.71297E+00_JPRB, 0.71031E+00_JPRB, 0.70644E+00_JPRB, 0.70408E+00_JPRB ,&
&  0.70210E+00_JPRB, 0.70135E+00_JPRB, 0.70048E+00_JPRB, 0.70125E+00_JPRB, 0.70153E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.70438E+00_JPRB, 0.70977E+00_JPRB, 0.71628E+00_JPRB, 0.72170E+00_JPRB, 0.72434E+00_JPRB, 0.72647E+00_JPRB, 0.72476E+00_JPRB ,&
&  0.72374E+00_JPRB, 0.72202E+00_JPRB, 0.72079E+00_JPRB, 0.72097E+00_JPRB, 0.72223E+00_JPRB, 0.72463E+00_JPRB, 0.72754E+00_JPRB ,&
&  0.73018E+00_JPRB, 0.73346E+00_JPRB, 0.73481E+00_JPRB, 0.73649E+00_JPRB, 0.73866E+00_JPRB, 0.74321E+00_JPRB, 0.74521E+00_JPRB ,&
&  0.74756E+00_JPRB, 0.74206E+00_JPRB, 0.72980E+00_JPRB, 0.72642E+00_JPRB, 0.73254E+00_JPRB, 0.71959E+00_JPRB, 0.71982E+00_JPRB ,&
&  0.72833E+00_JPRB, 0.72950E+00_JPRB, 0.71597E+00_JPRB, 0.70618E+00_JPRB, 0.68165E+00_JPRB, 0.61195E+00_JPRB, 0.61133E+00_JPRB ,&
&  0.59862E+00_JPRB, 0.57521E+00_JPRB, 0.56290E+00_JPRB, 0.55204E+00_JPRB, 0.54690E+00_JPRB, 0.55564E+00_JPRB, 0.56963E+00_JPRB ,&
&  0.57266E+00_JPRB, 0.58820E+00_JPRB, 0.59442E+00_JPRB, 0.60482E+00_JPRB, 0.61607E+00_JPRB, 0.63232E+00_JPRB, 0.64109E+00_JPRB ,&
&  0.65070E+00_JPRB, 0.66058E+00_JPRB, 0.67629E+00_JPRB, 0.69683E+00_JPRB, 0.71386E+00_JPRB, 0.72274E+00_JPRB, 0.72163E+00_JPRB ,&
&  0.73149E+00_JPRB, 0.72609E+00_JPRB, 0.73083E+00_JPRB, 0.73687E+00_JPRB, 0.74127E+00_JPRB, 0.74863E+00_JPRB, 0.75165E+00_JPRB ,&
&  0.76209E+00_JPRB, 0.77967E+00_JPRB, 0.79833E+00_JPRB, 0.80845E+00_JPRB, 0.81697E+00_JPRB, 0.82709E+00_JPRB, 0.84298E+00_JPRB ,&
&  0.85810E+00_JPRB, 0.87648E+00_JPRB, 0.89233E+00_JPRB, 0.87614E+00_JPRB, 0.86856E+00_JPRB, 0.84808E+00_JPRB, 0.83358E+00_JPRB ,&
&  0.81570E+00_JPRB, 0.80356E+00_JPRB, 0.78365E+00_JPRB, 0.77246E+00_JPRB, 0.76116E+00_JPRB, 0.75211E+00_JPRB, 0.74296E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.72787E+00_JPRB, 0.72349E+00_JPRB, 0.72090E+00_JPRB, 0.71797E+00_JPRB, 0.71330E+00_JPRB, 0.70944E+00_JPRB ,&
&  0.70345E+00_JPRB, 0.69960E+00_JPRB, 0.69687E+00_JPRB, 0.69702E+00_JPRB, 0.69897E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.69863E+00_JPRB, 0.70601E+00_JPRB, 0.71188E+00_JPRB, 0.71972E+00_JPRB, 0.71954E+00_JPRB, 0.72169E+00_JPRB, 0.71547E+00_JPRB ,&
&  0.71124E+00_JPRB, 0.70888E+00_JPRB, 0.70779E+00_JPRB, 0.70931E+00_JPRB, 0.71203E+00_JPRB, 0.71561E+00_JPRB, 0.72064E+00_JPRB ,&
&  0.72574E+00_JPRB, 0.72531E+00_JPRB, 0.72929E+00_JPRB, 0.73685E+00_JPRB, 0.74019E+00_JPRB, 0.74539E+00_JPRB, 0.74861E+00_JPRB ,&
&  0.74949E+00_JPRB, 0.74734E+00_JPRB, 0.74496E+00_JPRB, 0.73639E+00_JPRB, 0.73349E+00_JPRB, 0.73818E+00_JPRB, 0.74221E+00_JPRB ,&
&  0.72344E+00_JPRB, 0.71508E+00_JPRB, 0.73276E+00_JPRB, 0.73713E+00_JPRB, 0.69411E+00_JPRB, 0.65561E+00_JPRB, 0.63720E+00_JPRB ,&
&  0.61273E+00_JPRB, 0.57663E+00_JPRB, 0.56358E+00_JPRB, 0.55465E+00_JPRB, 0.55717E+00_JPRB, 0.56095E+00_JPRB, 0.57132E+00_JPRB ,&
&  0.57795E+00_JPRB, 0.58677E+00_JPRB, 0.59958E+00_JPRB, 0.61626E+00_JPRB, 0.62714E+00_JPRB, 0.63975E+00_JPRB, 0.64968E+00_JPRB ,&
&  0.66057E+00_JPRB, 0.67525E+00_JPRB, 0.69573E+00_JPRB, 0.72148E+00_JPRB, 0.74056E+00_JPRB, 0.75434E+00_JPRB, 0.75997E+00_JPRB ,&
&  0.78730E+00_JPRB, 0.77201E+00_JPRB, 0.77692E+00_JPRB, 0.78001E+00_JPRB, 0.78180E+00_JPRB, 0.78757E+00_JPRB, 0.79193E+00_JPRB ,&
&  0.80919E+00_JPRB, 0.83545E+00_JPRB, 0.84392E+00_JPRB, 0.84911E+00_JPRB, 0.85368E+00_JPRB, 0.86132E+00_JPRB, 0.88141E+00_JPRB ,&
&  0.90090E+00_JPRB, 0.93591E+00_JPRB, 0.10169E+01_JPRB, 0.98078E+00_JPRB, 0.95293E+00_JPRB, 0.92225E+00_JPRB, 0.88987E+00_JPRB ,&
&  0.85996E+00_JPRB, 0.83299E+00_JPRB, 0.80637E+00_JPRB, 0.78247E+00_JPRB, 0.76554E+00_JPRB, 0.74985E+00_JPRB, 0.74518E+00_JPRB ,&
&  0.74107E+00_JPRB, 0.73569E+00_JPRB, 0.73207E+00_JPRB, 0.72882E+00_JPRB, 0.72217E+00_JPRB, 0.71578E+00_JPRB, 0.71105E+00_JPRB ,&
&  0.70231E+00_JPRB, 0.69748E+00_JPRB, 0.69316E+00_JPRB, 0.69304E+00_JPRB, 0.69405E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.69332E+00_JPRB, 0.70859E+00_JPRB, 0.71899E+00_JPRB, 0.72507E+00_JPRB, 0.72067E+00_JPRB, 0.72168E+00_JPRB, 0.71424E+00_JPRB ,&
&  0.71007E+00_JPRB, 0.71042E+00_JPRB, 0.71056E+00_JPRB, 0.71112E+00_JPRB, 0.71204E+00_JPRB, 0.71277E+00_JPRB, 0.71474E+00_JPRB ,&
&  0.71823E+00_JPRB, 0.72280E+00_JPRB, 0.72642E+00_JPRB, 0.73153E+00_JPRB, 0.73820E+00_JPRB, 0.74597E+00_JPRB, 0.75257E+00_JPRB ,&
&  0.75463E+00_JPRB, 0.75253E+00_JPRB, 0.75615E+00_JPRB, 0.75209E+00_JPRB, 0.74786E+00_JPRB, 0.74226E+00_JPRB, 0.73981E+00_JPRB ,&
&  0.74790E+00_JPRB, 0.75784E+00_JPRB, 0.74647E+00_JPRB, 0.73951E+00_JPRB, 0.73496E+00_JPRB, 0.69938E+00_JPRB, 0.66338E+00_JPRB ,&
&  0.62640E+00_JPRB, 0.59465E+00_JPRB, 0.57985E+00_JPRB, 0.57232E+00_JPRB, 0.57832E+00_JPRB, 0.59209E+00_JPRB, 0.60430E+00_JPRB ,&
&  0.59966E+00_JPRB, 0.61799E+00_JPRB, 0.63668E+00_JPRB, 0.64842E+00_JPRB, 0.65775E+00_JPRB, 0.66727E+00_JPRB, 0.67653E+00_JPRB ,&
&  0.69126E+00_JPRB, 0.71439E+00_JPRB, 0.75603E+00_JPRB, 0.80279E+00_JPRB, 0.83848E+00_JPRB, 0.85230E+00_JPRB, 0.90367E+00_JPRB ,&
&  0.10229E+01_JPRB, 0.95067E+00_JPRB, 0.91269E+00_JPRB, 0.89897E+00_JPRB, 0.88378E+00_JPRB, 0.87936E+00_JPRB, 0.88733E+00_JPRB ,&
&  0.90430E+00_JPRB, 0.92072E+00_JPRB, 0.92401E+00_JPRB, 0.92134E+00_JPRB, 0.92377E+00_JPRB, 0.93251E+00_JPRB, 0.95228E+00_JPRB ,&
&  0.97367E+00_JPRB, 0.10343E+01_JPRB, 0.11769E+01_JPRB, 0.10468E+01_JPRB, 0.97702E+00_JPRB, 0.94200E+00_JPRB, 0.90792E+00_JPRB ,&
&  0.87799E+00_JPRB, 0.85175E+00_JPRB, 0.82461E+00_JPRB, 0.79224E+00_JPRB, 0.76655E+00_JPRB, 0.74351E+00_JPRB, 0.73558E+00_JPRB ,&
&  0.73039E+00_JPRB, 0.72481E+00_JPRB, 0.72415E+00_JPRB, 0.71999E+00_JPRB, 0.71427E+00_JPRB, 0.71321E+00_JPRB, 0.71002E+00_JPRB ,&
&  0.70084E+00_JPRB, 0.69230E+00_JPRB, 0.68565E+00_JPRB, 0.68357E+00_JPRB, 0.68456E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.71175E+00_JPRB, 0.72035E+00_JPRB, 0.72986E+00_JPRB, 0.75061E+00_JPRB, 0.75748E+00_JPRB, 0.75247E+00_JPRB, 0.73461E+00_JPRB ,&
&  0.72326E+00_JPRB, 0.71901E+00_JPRB, 0.72248E+00_JPRB, 0.72690E+00_JPRB, 0.72909E+00_JPRB, 0.72910E+00_JPRB, 0.72683E+00_JPRB ,&
&  0.72266E+00_JPRB, 0.72192E+00_JPRB, 0.72437E+00_JPRB, 0.72884E+00_JPRB, 0.73491E+00_JPRB, 0.74186E+00_JPRB, 0.75038E+00_JPRB ,&
&  0.75717E+00_JPRB, 0.75862E+00_JPRB, 0.75972E+00_JPRB, 0.75808E+00_JPRB, 0.75968E+00_JPRB, 0.76430E+00_JPRB, 0.76247E+00_JPRB ,&
&  0.75268E+00_JPRB, 0.76037E+00_JPRB, 0.76922E+00_JPRB, 0.75515E+00_JPRB, 0.74816E+00_JPRB, 0.72186E+00_JPRB, 0.68664E+00_JPRB ,&
&  0.65662E+00_JPRB, 0.62710E+00_JPRB, 0.61932E+00_JPRB, 0.62115E+00_JPRB, 0.62874E+00_JPRB, 0.62692E+00_JPRB, 0.64548E+00_JPRB ,&
&  0.66073E+00_JPRB, 0.67096E+00_JPRB, 0.67903E+00_JPRB, 0.68518E+00_JPRB, 0.69190E+00_JPRB, 0.70312E+00_JPRB, 0.72603E+00_JPRB ,&
&  0.76808E+00_JPRB, 0.82353E+00_JPRB, 0.88333E+00_JPRB, 0.86693E+00_JPRB, 0.87968E+00_JPRB, 0.91359E+00_JPRB, 0.96843E+00_JPRB ,&
&  0.10099E+01_JPRB, 0.10080E+01_JPRB, 0.10048E+01_JPRB, 0.10459E+01_JPRB, 0.10698E+01_JPRB, 0.10959E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.10771E+01_JPRB, 0.10553E+01_JPRB, 0.10397E+01_JPRB, 0.10404E+01_JPRB, 0.10593E+01_JPRB, 0.10757E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.10458E+01_JPRB, 0.99905E+00_JPRB, 0.94591E+00_JPRB, 0.89529E+00_JPRB, 0.88400E+00_JPRB, 0.88830E+00_JPRB, 0.88943E+00_JPRB ,&
&  0.87287E+00_JPRB, 0.85028E+00_JPRB, 0.82652E+00_JPRB, 0.79590E+00_JPRB, 0.76492E+00_JPRB, 0.73315E+00_JPRB, 0.71835E+00_JPRB ,&
&  0.70987E+00_JPRB, 0.70536E+00_JPRB, 0.70937E+00_JPRB, 0.69831E+00_JPRB, 0.69855E+00_JPRB, 0.70120E+00_JPRB, 0.69132E+00_JPRB ,&
&  0.68324E+00_JPRB, 0.68117E+00_JPRB, 0.67486E+00_JPRB, 0.67909E+00_JPRB, 0.68908E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.73968E+00_JPRB, 0.76818E+00_JPRB, 0.79404E+00_JPRB, 0.80678E+00_JPRB, 0.81264E+00_JPRB, 0.81143E+00_JPRB, 0.79358E+00_JPRB ,&
&  0.78796E+00_JPRB, 0.76200E+00_JPRB, 0.74349E+00_JPRB, 0.72469E+00_JPRB, 0.71074E+00_JPRB, 0.70356E+00_JPRB, 0.70662E+00_JPRB ,&
&  0.71259E+00_JPRB, 0.71821E+00_JPRB, 0.72202E+00_JPRB, 0.72435E+00_JPRB, 0.72879E+00_JPRB, 0.73583E+00_JPRB, 0.74568E+00_JPRB ,&
&  0.75617E+00_JPRB, 0.76346E+00_JPRB, 0.76916E+00_JPRB, 0.77309E+00_JPRB, 0.77569E+00_JPRB, 0.77603E+00_JPRB, 0.77054E+00_JPRB ,&
&  0.77123E+00_JPRB, 0.77060E+00_JPRB, 0.77255E+00_JPRB, 0.76427E+00_JPRB, 0.75831E+00_JPRB, 0.73793E+00_JPRB, 0.70015E+00_JPRB ,&
&  0.68158E+00_JPRB, 0.67145E+00_JPRB, 0.67632E+00_JPRB, 0.67558E+00_JPRB, 0.68547E+00_JPRB, 0.69863E+00_JPRB, 0.70223E+00_JPRB ,&
&  0.70057E+00_JPRB, 0.70149E+00_JPRB, 0.71250E+00_JPRB, 0.71995E+00_JPRB, 0.72987E+00_JPRB, 0.74593E+00_JPRB, 0.76453E+00_JPRB ,&
&  0.79945E+00_JPRB, 0.85330E+00_JPRB, 0.93783E+00_JPRB, 0.96785E+00_JPRB, 0.10438E+01_JPRB, 0.12153E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.12120E+01_JPRB, 0.12651E+01_JPRB, 0.13508E+01_JPRB, 0.13340E+01_JPRB, 0.13199E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.12381E+01_JPRB, 0.12191E+01_JPRB, 0.12453E+01_JPRB, 0.12362E+01_JPRB, 0.12238E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.11517E+01_JPRB, 0.10813E+01_JPRB, 0.10126E+01_JPRB, 0.94701E+00_JPRB, 0.91338E+00_JPRB, 0.89366E+00_JPRB, 0.88082E+00_JPRB ,&
&  0.86392E+00_JPRB, 0.85022E+00_JPRB, 0.83744E+00_JPRB, 0.81812E+00_JPRB, 0.78413E+00_JPRB, 0.74459E+00_JPRB, 0.72967E+00_JPRB ,&
&  0.71754E+00_JPRB, 0.70760E+00_JPRB, 0.70190E+00_JPRB, 0.69611E+00_JPRB, 0.69841E+00_JPRB, 0.69537E+00_JPRB, 0.68828E+00_JPRB ,&
&  0.69037E+00_JPRB, 0.68415E+00_JPRB, 0.67871E+00_JPRB, 0.68408E+00_JPRB, 0.71135E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.79244E+00_JPRB, 0.82854E+00_JPRB, 0.84295E+00_JPRB, 0.85228E+00_JPRB, 0.86068E+00_JPRB, 0.85969E+00_JPRB, 0.82576E+00_JPRB ,&
&  0.81499E+00_JPRB, 0.76029E+00_JPRB, 0.73579E+00_JPRB, 0.71866E+00_JPRB, 0.71037E+00_JPRB, 0.70084E+00_JPRB, 0.69109E+00_JPRB ,&
&  0.69471E+00_JPRB, 0.70771E+00_JPRB, 0.72080E+00_JPRB, 0.72654E+00_JPRB, 0.73195E+00_JPRB, 0.73646E+00_JPRB, 0.74235E+00_JPRB ,&
&  0.75175E+00_JPRB, 0.76289E+00_JPRB, 0.77518E+00_JPRB, 0.78051E+00_JPRB, 0.78227E+00_JPRB, 0.78258E+00_JPRB, 0.78112E+00_JPRB ,&
&  0.77993E+00_JPRB, 0.77092E+00_JPRB, 0.77253E+00_JPRB, 0.76935E+00_JPRB, 0.75979E+00_JPRB, 0.75596E+00_JPRB, 0.73056E+00_JPRB ,&
&  0.70801E+00_JPRB, 0.72428E+00_JPRB, 0.72657E+00_JPRB, 0.73927E+00_JPRB, 0.74773E+00_JPRB, 0.75235E+00_JPRB, 0.75357E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.74827E+00_JPRB, 0.74981E+00_JPRB, 0.75415E+00_JPRB, 0.76069E+00_JPRB, 0.77035E+00_JPRB, 0.77989E+00_JPRB ,&
&  0.78955E+00_JPRB, 0.77494E+00_JPRB, 0.83260E+00_JPRB, 0.98977E+00_JPRB, 0.13095E+01_JPRB, 0.13256E+01_JPRB, 0.13770E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.14861E+01_JPRB, 0.15273E+01_JPRB, 0.15634E+01_JPRB, 0.15655E+01_JPRB, 0.15493E+01_JPRB, 0.15529E+01_JPRB ,&
&  0.15039E+01_JPRB, 0.15222E+01_JPRB, 0.14954E+01_JPRB, 0.14831E+01_JPRB, 0.14406E+01_JPRB, 0.14158E+01_JPRB, 0.13754E+01_JPRB ,&
&  0.13148E+01_JPRB, 0.12600E+01_JPRB, 0.11746E+01_JPRB, 0.10764E+01_JPRB, 0.99565E+00_JPRB, 0.95244E+00_JPRB, 0.91478E+00_JPRB ,&
&  0.88818E+00_JPRB, 0.87297E+00_JPRB, 0.85534E+00_JPRB, 0.84526E+00_JPRB, 0.82335E+00_JPRB, 0.79826E+00_JPRB, 0.79159E+00_JPRB ,&
&  0.76457E+00_JPRB, 0.74701E+00_JPRB, 0.73164E+00_JPRB, 0.72389E+00_JPRB, 0.73128E+00_JPRB, 0.73735E+00_JPRB, 0.73904E+00_JPRB ,&
&  0.74794E+00_JPRB, 0.74064E+00_JPRB, 0.74074E+00_JPRB, 0.74996E+00_JPRB, 0.76627E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.84073E+00_JPRB, 0.86732E+00_JPRB, 0.87899E+00_JPRB, 0.89176E+00_JPRB, 0.90800E+00_JPRB, 0.90621E+00_JPRB, 0.89782E+00_JPRB ,&
&  0.91290E+00_JPRB, 0.87589E+00_JPRB, 0.90847E+00_JPRB, 0.78623E+00_JPRB, 0.73831E+00_JPRB, 0.72663E+00_JPRB, 0.71213E+00_JPRB ,&
&  0.71035E+00_JPRB, 0.71675E+00_JPRB, 0.72399E+00_JPRB, 0.73200E+00_JPRB, 0.74016E+00_JPRB, 0.74168E+00_JPRB, 0.74376E+00_JPRB ,&
&  0.74950E+00_JPRB, 0.76105E+00_JPRB, 0.77760E+00_JPRB, 0.78559E+00_JPRB, 0.79214E+00_JPRB, 0.79866E+00_JPRB, 0.79393E+00_JPRB ,&
&  0.78525E+00_JPRB, 0.78759E+00_JPRB, 0.78589E+00_JPRB, 0.77499E+00_JPRB, 0.76309E+00_JPRB, 0.76083E+00_JPRB, 0.77383E+00_JPRB ,&
&  0.77204E+00_JPRB, 0.76464E+00_JPRB, 0.78057E+00_JPRB, 0.78703E+00_JPRB, 0.78868E+00_JPRB, 0.78759E+00_JPRB, 0.78587E+00_JPRB ,&
&  0.78518E+00_JPRB, 0.78301E+00_JPRB, 0.78561E+00_JPRB, 0.78678E+00_JPRB, 0.78927E+00_JPRB, 0.79377E+00_JPRB, 0.79574E+00_JPRB ,&
&  0.80764E+00_JPRB, 0.80092E+00_JPRB, 0.92502E+00_JPRB, 0.10747E+01_JPRB, 0.13576E+01_JPRB, 0.15277E+01_JPRB, 0.15998E+01_JPRB ,&
&  0.18433E+01_JPRB, 0.17884E+01_JPRB, 0.18861E+01_JPRB, 0.18776E+01_JPRB, 0.18526E+01_JPRB, 0.18074E+01_JPRB, 0.17457E+01_JPRB ,&
&  0.17319E+01_JPRB, 0.17099E+01_JPRB, 0.17787E+01_JPRB, 0.17387E+01_JPRB, 0.16490E+01_JPRB, 0.16077E+01_JPRB, 0.15755E+01_JPRB ,&
&  0.15544E+01_JPRB, 0.15266E+01_JPRB, 0.14546E+01_JPRB, 0.12597E+01_JPRB, 0.11629E+01_JPRB, 0.10831E+01_JPRB, 0.98268E+00_JPRB ,&
&  0.93360E+00_JPRB, 0.88494E+00_JPRB, 0.84877E+00_JPRB, 0.84299E+00_JPRB, 0.82674E+00_JPRB, 0.81113E+00_JPRB, 0.80870E+00_JPRB ,&
&  0.78840E+00_JPRB, 0.77848E+00_JPRB, 0.77301E+00_JPRB, 0.77832E+00_JPRB, 0.78588E+00_JPRB, 0.79279E+00_JPRB, 0.79808E+00_JPRB ,&
&  0.79084E+00_JPRB, 0.78128E+00_JPRB, 0.79216E+00_JPRB, 0.79725E+00_JPRB, 0.81492E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.86977E+00_JPRB, 0.89613E+00_JPRB, 0.91603E+00_JPRB, 0.93433E+00_JPRB, 0.95900E+00_JPRB, 0.98287E+00_JPRB, 0.97783E+00_JPRB ,&
&  0.98439E+00_JPRB, 0.98472E+00_JPRB, 0.98268E+00_JPRB, 0.96546E+00_JPRB, 0.93199E+00_JPRB, 0.84536E+00_JPRB, 0.74175E+00_JPRB ,&
&  0.71154E+00_JPRB, 0.71707E+00_JPRB, 0.72839E+00_JPRB, 0.73373E+00_JPRB, 0.73755E+00_JPRB, 0.74214E+00_JPRB, 0.74392E+00_JPRB ,&
&  0.74991E+00_JPRB, 0.75914E+00_JPRB, 0.77429E+00_JPRB, 0.79314E+00_JPRB, 0.81041E+00_JPRB, 0.81931E+00_JPRB, 0.81653E+00_JPRB ,&
&  0.80443E+00_JPRB, 0.80032E+00_JPRB, 0.78792E+00_JPRB, 0.77531E+00_JPRB, 0.77680E+00_JPRB, 0.78406E+00_JPRB, 0.80257E+00_JPRB ,&
&  0.81920E+00_JPRB, 0.82900E+00_JPRB, 0.83221E+00_JPRB, 0.83586E+00_JPRB, 0.83690E+00_JPRB, 0.83206E+00_JPRB, 0.82396E+00_JPRB ,&
&  0.81691E+00_JPRB, 0.81192E+00_JPRB, 0.80904E+00_JPRB, 0.81121E+00_JPRB, 0.80926E+00_JPRB, 0.82376E+00_JPRB, 0.84403E+00_JPRB ,&
&  0.86701E+00_JPRB, 0.96039E+00_JPRB, 0.11115E+01_JPRB, 0.11710E+01_JPRB, 0.14428E+01_JPRB, 0.16164E+01_JPRB, 0.17804E+01_JPRB ,&
&  0.19089E+01_JPRB, 0.18723E+01_JPRB, 0.20808E+01_JPRB, 0.21472E+01_JPRB, 0.21733E+01_JPRB, 0.21235E+01_JPRB, 0.19972E+01_JPRB ,&
&  0.19332E+01_JPRB, 0.20150E+01_JPRB, 0.20659E+01_JPRB, 0.20436E+01_JPRB, 0.19604E+01_JPRB, 0.19303E+01_JPRB, 0.19070E+01_JPRB ,&
&  0.19282E+01_JPRB, 0.19434E+01_JPRB, 0.18573E+01_JPRB, 0.16805E+01_JPRB, 0.14665E+01_JPRB, 0.12767E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.10026E+01_JPRB, 0.88758E+00_JPRB, 0.84552E+00_JPRB, 0.83515E+00_JPRB, 0.83572E+00_JPRB, 0.81601E+00_JPRB, 0.81289E+00_JPRB ,&
&  0.81771E+00_JPRB, 0.81853E+00_JPRB, 0.82328E+00_JPRB, 0.82818E+00_JPRB, 0.85114E+00_JPRB, 0.86271E+00_JPRB, 0.82912E+00_JPRB ,&
&  0.80614E+00_JPRB, 0.80577E+00_JPRB, 0.81098E+00_JPRB, 0.81921E+00_JPRB, 0.84204E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.91048E+00_JPRB, 0.94107E+00_JPRB, 0.96283E+00_JPRB, 0.98009E+00_JPRB, 0.10069E+01_JPRB, 0.10467E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.10445E+01_JPRB, 0.10396E+01_JPRB, 0.10284E+01_JPRB, 0.10028E+01_JPRB, 0.95917E+00_JPRB, 0.86008E+00_JPRB ,&
&  0.75565E+00_JPRB, 0.74468E+00_JPRB, 0.72946E+00_JPRB, 0.73417E+00_JPRB, 0.74086E+00_JPRB, 0.74064E+00_JPRB, 0.73837E+00_JPRB ,&
&  0.74531E+00_JPRB, 0.75464E+00_JPRB, 0.77083E+00_JPRB, 0.79077E+00_JPRB, 0.79940E+00_JPRB, 0.80862E+00_JPRB, 0.81905E+00_JPRB ,&
&  0.80851E+00_JPRB, 0.79354E+00_JPRB, 0.78215E+00_JPRB, 0.77575E+00_JPRB, 0.79153E+00_JPRB, 0.81640E+00_JPRB, 0.83908E+00_JPRB ,&
&  0.85297E+00_JPRB, 0.85946E+00_JPRB, 0.86397E+00_JPRB, 0.86762E+00_JPRB, 0.86903E+00_JPRB, 0.86411E+00_JPRB, 0.85276E+00_JPRB ,&
&  0.83932E+00_JPRB, 0.83148E+00_JPRB, 0.82753E+00_JPRB, 0.83245E+00_JPRB, 0.84058E+00_JPRB, 0.88093E+00_JPRB, 0.93087E+00_JPRB ,&
&  0.99698E+00_JPRB, 0.11060E+01_JPRB, 0.12004E+01_JPRB, 0.13806E+01_JPRB, 0.16294E+01_JPRB, 0.18380E+01_JPRB, 0.19853E+01_JPRB ,&
&  0.21381E+01_JPRB, 0.21369E+01_JPRB, 0.24461E+01_JPRB, 0.24714E+01_JPRB, 0.23823E+01_JPRB, 0.23574E+01_JPRB, 0.22116E+01_JPRB ,&
&  0.20922E+01_JPRB, 0.19937E+01_JPRB, 0.21113E+01_JPRB, 0.21781E+01_JPRB, 0.22213E+01_JPRB, 0.22802E+01_JPRB, 0.22896E+01_JPRB ,&
&  0.23079E+01_JPRB, 0.22879E+01_JPRB, 0.19958E+01_JPRB, 0.16984E+01_JPRB, 0.13880E+01_JPRB, 0.13394E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.88497E+00_JPRB, 0.86686E+00_JPRB, 0.85582E+00_JPRB, 0.87143E+00_JPRB, 0.86429E+00_JPRB, 0.86351E+00_JPRB ,&
&  0.87938E+00_JPRB, 0.89986E+00_JPRB, 0.89917E+00_JPRB, 0.91240E+00_JPRB, 0.94053E+00_JPRB, 0.94259E+00_JPRB, 0.89672E+00_JPRB ,&
&  0.86987E+00_JPRB, 0.85991E+00_JPRB, 0.84060E+00_JPRB, 0.84555E+00_JPRB, 0.87196E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.96619E+00_JPRB, 0.99094E+00_JPRB, 0.10091E+01_JPRB, 0.10317E+01_JPRB, 0.10661E+01_JPRB, 0.11041E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.10917E+01_JPRB, 0.10825E+01_JPRB, 0.10660E+01_JPRB, 0.10441E+01_JPRB, 0.10006E+01_JPRB, 0.94875E+00_JPRB ,&
&  0.81527E+00_JPRB, 0.77412E+00_JPRB, 0.72425E+00_JPRB, 0.72330E+00_JPRB, 0.76987E+00_JPRB, 0.76072E+00_JPRB, 0.75990E+00_JPRB ,&
&  0.75545E+00_JPRB, 0.76514E+00_JPRB, 0.77357E+00_JPRB, 0.78594E+00_JPRB, 0.78426E+00_JPRB, 0.79691E+00_JPRB, 0.81293E+00_JPRB ,&
&  0.80641E+00_JPRB, 0.79751E+00_JPRB, 0.79728E+00_JPRB, 0.80375E+00_JPRB, 0.82515E+00_JPRB, 0.85523E+00_JPRB, 0.88883E+00_JPRB ,&
&  0.90178E+00_JPRB, 0.91217E+00_JPRB, 0.91557E+00_JPRB, 0.91714E+00_JPRB, 0.91544E+00_JPRB, 0.90782E+00_JPRB, 0.89496E+00_JPRB ,&
&  0.87911E+00_JPRB, 0.86717E+00_JPRB, 0.86943E+00_JPRB, 0.88343E+00_JPRB, 0.88745E+00_JPRB, 0.94764E+00_JPRB, 0.10723E+01_JPRB ,&
&  0.12265E+01_JPRB, 0.12940E+01_JPRB, 0.14072E+01_JPRB, 0.17929E+01_JPRB, 0.20952E+01_JPRB, 0.22814E+01_JPRB, 0.23461E+01_JPRB ,&
&  0.23967E+01_JPRB, 0.24109E+01_JPRB, 0.24276E+01_JPRB, 0.24857E+01_JPRB, 0.24204E+01_JPRB, 0.24563E+01_JPRB, 0.23500E+01_JPRB ,&
&  0.21647E+01_JPRB, 0.21398E+01_JPRB, 0.22568E+01_JPRB, 0.23338E+01_JPRB, 0.23701E+01_JPRB, 0.24548E+01_JPRB, 0.25304E+01_JPRB ,&
&  0.24990E+01_JPRB, 0.22258E+01_JPRB, 0.17041E+01_JPRB, 0.15345E+01_JPRB, 0.11742E+01_JPRB, 0.11427E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.10717E+01_JPRB, 0.97144E+00_JPRB, 0.92739E+00_JPRB, 0.93044E+00_JPRB, 0.10281E+01_JPRB, 0.10383E+01_JPRB ,&
&  0.97872E+00_JPRB, 0.99762E+00_JPRB, 0.10263E+01_JPRB, 0.10640E+01_JPRB, 0.10990E+01_JPRB, 0.10983E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.10232E+01_JPRB, 0.97663E+00_JPRB, 0.93568E+00_JPRB, 0.92449E+00_JPRB, 0.93840E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.10638E+01_JPRB, 0.10690E+01_JPRB, 0.10776E+01_JPRB, 0.10945E+01_JPRB, 0.11323E+01_JPRB, 0.11596E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.11306E+01_JPRB, 0.11163E+01_JPRB, 0.10965E+01_JPRB, 0.10610E+01_JPRB, 0.10043E+01_JPRB, 0.95428E+00_JPRB ,&
&  0.93779E+00_JPRB, 0.85015E+00_JPRB, 0.78048E+00_JPRB, 0.73399E+00_JPRB, 0.76250E+00_JPRB, 0.76465E+00_JPRB, 0.77958E+00_JPRB ,&
&  0.77819E+00_JPRB, 0.79788E+00_JPRB, 0.79997E+00_JPRB, 0.79696E+00_JPRB, 0.79293E+00_JPRB, 0.81589E+00_JPRB, 0.84461E+00_JPRB ,&
&  0.86293E+00_JPRB, 0.86987E+00_JPRB, 0.87429E+00_JPRB, 0.89602E+00_JPRB, 0.93558E+00_JPRB, 0.91107E+00_JPRB, 0.94522E+00_JPRB ,&
&  0.95821E+00_JPRB, 0.96510E+00_JPRB, 0.96861E+00_JPRB, 0.97704E+00_JPRB, 0.97757E+00_JPRB, 0.97997E+00_JPRB, 0.98398E+00_JPRB ,&
&  0.99060E+00_JPRB, 0.98969E+00_JPRB, 0.96319E+00_JPRB, 0.95788E+00_JPRB, 0.97261E+00_JPRB, 0.10800E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.12628E+01_JPRB, 0.13386E+01_JPRB, 0.17816E+01_JPRB, 0.21572E+01_JPRB, 0.23055E+01_JPRB, 0.24401E+01_JPRB ,&
&  0.25182E+01_JPRB, 0.25568E+01_JPRB, 0.26304E+01_JPRB, 0.26555E+01_JPRB, 0.26760E+01_JPRB, 0.26671E+01_JPRB, 0.26168E+01_JPRB ,&
&  0.24700E+01_JPRB, 0.24283E+01_JPRB, 0.25195E+01_JPRB, 0.24784E+01_JPRB, 0.24545E+01_JPRB, 0.23591E+01_JPRB, 0.25402E+01_JPRB ,&
&  0.25489E+01_JPRB, 0.15983E+01_JPRB, 0.13060E+01_JPRB, 0.14750E+01_JPRB, 0.12345E+01_JPRB, 0.11854E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.11811E+01_JPRB, 0.11120E+01_JPRB, 0.10808E+01_JPRB, 0.10351E+01_JPRB, 0.10549E+01_JPRB, 0.12825E+01_JPRB, 0.12983E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.12183E+01_JPRB, 0.12782E+01_JPRB, 0.12922E+01_JPRB, 0.12874E+01_JPRB, 0.12975E+01_JPRB, 0.12585E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.11201E+01_JPRB, 0.10727E+01_JPRB, 0.10554E+01_JPRB, 0.10581E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.12043E+01_JPRB, 0.11828E+01_JPRB, 0.11678E+01_JPRB, 0.11753E+01_JPRB, 0.12060E+01_JPRB, 0.12217E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.11718E+01_JPRB, 0.11514E+01_JPRB, 0.11192E+01_JPRB, 0.10680E+01_JPRB, 0.10097E+01_JPRB, 0.95818E+00_JPRB ,&
&  0.92613E+00_JPRB, 0.88959E+00_JPRB, 0.80009E+00_JPRB, 0.76132E+00_JPRB, 0.73338E+00_JPRB, 0.75188E+00_JPRB, 0.77495E+00_JPRB ,&
&  0.79920E+00_JPRB, 0.81185E+00_JPRB, 0.81797E+00_JPRB, 0.82585E+00_JPRB, 0.83660E+00_JPRB, 0.87602E+00_JPRB, 0.93450E+00_JPRB ,&
&  0.96962E+00_JPRB, 0.98679E+00_JPRB, 0.96860E+00_JPRB, 0.10185E+01_JPRB, 0.10196E+01_JPRB, 0.98511E+00_JPRB, 0.10318E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10623E+01_JPRB, 0.10596E+01_JPRB, 0.10722E+01_JPRB, 0.10793E+01_JPRB, 0.10585E+01_JPRB, 0.10329E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.98260E+00_JPRB, 0.97947E+00_JPRB, 0.97316E+00_JPRB, 0.98531E+00_JPRB, 0.10508E+01_JPRB, 0.10659E+01_JPRB ,&
&  0.10726E+01_JPRB, 0.96060E+00_JPRB, 0.94108E+00_JPRB, 0.13140E+01_JPRB, 0.19101E+01_JPRB, 0.22508E+01_JPRB, 0.22667E+01_JPRB ,&
&  0.24850E+01_JPRB, 0.26357E+01_JPRB, 0.28173E+01_JPRB, 0.29249E+01_JPRB, 0.29548E+01_JPRB, 0.28818E+01_JPRB, 0.27944E+01_JPRB ,&
&  0.27123E+01_JPRB, 0.26933E+01_JPRB, 0.26965E+01_JPRB, 0.26953E+01_JPRB, 0.26563E+01_JPRB, 0.27319E+01_JPRB, 0.29179E+01_JPRB ,&
&  0.27496E+01_JPRB, 0.24035E+01_JPRB, 0.16582E+01_JPRB, 0.16376E+01_JPRB, 0.12783E+01_JPRB, 0.12152E+01_JPRB, 0.12888E+01_JPRB ,&
&  0.12205E+01_JPRB, 0.12269E+01_JPRB, 0.11992E+01_JPRB, 0.11563E+01_JPRB, 0.12878E+01_JPRB, 0.15482E+01_JPRB, 0.16114E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.15580E+01_JPRB, 0.16747E+01_JPRB, 0.16879E+01_JPRB, 0.16650E+01_JPRB, 0.16321E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.13560E+01_JPRB, 0.12770E+01_JPRB, 0.12380E+01_JPRB, 0.12167E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.12996E+01_JPRB, 0.12524E+01_JPRB, 0.12241E+01_JPRB, 0.12240E+01_JPRB, 0.12451E+01_JPRB, 0.12523E+01_JPRB, 0.12300E+01_JPRB ,&
&  0.12115E+01_JPRB, 0.11882E+01_JPRB, 0.11675E+01_JPRB, 0.11168E+01_JPRB, 0.10684E+01_JPRB, 0.10072E+01_JPRB, 0.95179E+00_JPRB ,&
&  0.91847E+00_JPRB, 0.86113E+00_JPRB, 0.78230E+00_JPRB, 0.74833E+00_JPRB, 0.72166E+00_JPRB, 0.72579E+00_JPRB, 0.75562E+00_JPRB ,&
&  0.79360E+00_JPRB, 0.82443E+00_JPRB, 0.87042E+00_JPRB, 0.89165E+00_JPRB, 0.92782E+00_JPRB, 0.99330E+00_JPRB, 0.10697E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.10707E+01_JPRB, 0.11001E+01_JPRB, 0.10929E+01_JPRB, 0.11084E+01_JPRB, 0.11026E+01_JPRB, 0.10917E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.10981E+01_JPRB, 0.11143E+01_JPRB, 0.11113E+01_JPRB, 0.10941E+01_JPRB, 0.10720E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10329E+01_JPRB, 0.10043E+01_JPRB, 0.97271E+00_JPRB, 0.10377E+01_JPRB, 0.10887E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.91818E+00_JPRB, 0.95957E+00_JPRB, 0.12444E+01_JPRB, 0.16169E+01_JPRB, 0.20114E+01_JPRB, 0.23952E+01_JPRB, 0.26334E+01_JPRB ,&
&  0.28343E+01_JPRB, 0.29442E+01_JPRB, 0.30398E+01_JPRB, 0.27415E+01_JPRB, 0.29913E+01_JPRB, 0.29562E+01_JPRB, 0.28564E+01_JPRB ,&
&  0.27567E+01_JPRB, 0.27851E+01_JPRB, 0.27398E+01_JPRB, 0.28614E+01_JPRB, 0.30799E+01_JPRB, 0.34037E+01_JPRB, 0.30405E+01_JPRB ,&
&  0.25977E+01_JPRB, 0.27704E+01_JPRB, 0.23939E+01_JPRB, 0.19138E+01_JPRB, 0.15185E+01_JPRB, 0.14638E+01_JPRB, 0.14276E+01_JPRB ,&
&  0.13757E+01_JPRB, 0.13384E+01_JPRB, 0.12909E+01_JPRB, 0.13785E+01_JPRB, 0.16556E+01_JPRB, 0.19038E+01_JPRB, 0.19047E+01_JPRB ,&
&  0.18706E+01_JPRB, 0.20333E+01_JPRB, 0.20694E+01_JPRB, 0.20557E+01_JPRB, 0.20485E+01_JPRB, 0.19952E+01_JPRB, 0.18876E+01_JPRB ,&
&  0.17447E+01_JPRB, 0.16153E+01_JPRB, 0.15156E+01_JPRB, 0.14295E+01_JPRB, 0.13572E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.13575E+01_JPRB, 0.13025E+01_JPRB, 0.12763E+01_JPRB, 0.12780E+01_JPRB, 0.12974E+01_JPRB, 0.12900E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.11848E+01_JPRB, 0.11360E+01_JPRB, 0.10908E+01_JPRB, 0.10555E+01_JPRB, 0.10019E+01_JPRB, 0.95605E+00_JPRB ,&
&  0.91746E+00_JPRB, 0.85152E+00_JPRB, 0.79379E+00_JPRB, 0.78850E+00_JPRB, 0.77110E+00_JPRB, 0.73762E+00_JPRB, 0.75480E+00_JPRB ,&
&  0.79986E+00_JPRB, 0.86517E+00_JPRB, 0.93540E+00_JPRB, 0.10159E+01_JPRB, 0.10883E+01_JPRB, 0.11803E+01_JPRB, 0.12471E+01_JPRB ,&
&  0.12133E+01_JPRB, 0.12014E+01_JPRB, 0.12132E+01_JPRB, 0.12025E+01_JPRB, 0.11540E+01_JPRB, 0.11330E+01_JPRB, 0.11357E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.11591E+01_JPRB, 0.11655E+01_JPRB, 0.11547E+01_JPRB, 0.11498E+01_JPRB, 0.11416E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10438E+01_JPRB, 0.99600E+00_JPRB, 0.10122E+01_JPRB, 0.94942E+00_JPRB, 0.92553E+00_JPRB, 0.92324E+00_JPRB ,&
&  0.10303E+01_JPRB, 0.12555E+01_JPRB, 0.14346E+01_JPRB, 0.20084E+01_JPRB, 0.20207E+01_JPRB, 0.21943E+01_JPRB, 0.26679E+01_JPRB ,&
&  0.29169E+01_JPRB, 0.29967E+01_JPRB, 0.30036E+01_JPRB, 0.24676E+01_JPRB, 0.22303E+01_JPRB, 0.29371E+01_JPRB, 0.30127E+01_JPRB ,&
&  0.29295E+01_JPRB, 0.29327E+01_JPRB, 0.30472E+01_JPRB, 0.35691E+01_JPRB, 0.29707E+01_JPRB, 0.26592E+01_JPRB, 0.21628E+01_JPRB ,&
&  0.22618E+01_JPRB, 0.19876E+01_JPRB, 0.22414E+01_JPRB, 0.19741E+01_JPRB, 0.18833E+01_JPRB, 0.17356E+01_JPRB, 0.15441E+01_JPRB ,&
&  0.14579E+01_JPRB, 0.14193E+01_JPRB, 0.14492E+01_JPRB, 0.17596E+01_JPRB, 0.21700E+01_JPRB, 0.22271E+01_JPRB, 0.23486E+01_JPRB ,&
&  0.24477E+01_JPRB, 0.24943E+01_JPRB, 0.23633E+01_JPRB, 0.23673E+01_JPRB, 0.22957E+01_JPRB, 0.21983E+01_JPRB, 0.20780E+01_JPRB ,&
&  0.19483E+01_JPRB, 0.18226E+01_JPRB, 0.16867E+01_JPRB, 0.15538E+01_JPRB, 0.14409E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.14254E+01_JPRB, 0.13780E+01_JPRB, 0.13616E+01_JPRB, 0.13614E+01_JPRB, 0.13517E+01_JPRB, 0.13195E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.11564E+01_JPRB, 0.10957E+01_JPRB, 0.10509E+01_JPRB, 0.10269E+01_JPRB, 0.99891E+00_JPRB, 0.96483E+00_JPRB ,&
&  0.93726E+00_JPRB, 0.91175E+00_JPRB, 0.85058E+00_JPRB, 0.82200E+00_JPRB, 0.85901E+00_JPRB, 0.81180E+00_JPRB, 0.76941E+00_JPRB ,&
&  0.83717E+00_JPRB, 0.92730E+00_JPRB, 0.10303E+01_JPRB, 0.11552E+01_JPRB, 0.12615E+01_JPRB, 0.13633E+01_JPRB, 0.14167E+01_JPRB ,&
&  0.13749E+01_JPRB, 0.12983E+01_JPRB, 0.12344E+01_JPRB, 0.11713E+01_JPRB, 0.11108E+01_JPRB, 0.10988E+01_JPRB, 0.11074E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.11682E+01_JPRB, 0.11586E+01_JPRB, 0.11126E+01_JPRB, 0.10967E+01_JPRB, 0.10462E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.10123E+01_JPRB, 0.10059E+01_JPRB, 0.99390E+00_JPRB, 0.10490E+01_JPRB, 0.97152E+00_JPRB, 0.92636E+00_JPRB, 0.95886E+00_JPRB ,&
&  0.10861E+01_JPRB, 0.13270E+01_JPRB, 0.15640E+01_JPRB, 0.19931E+01_JPRB, 0.25564E+01_JPRB, 0.26548E+01_JPRB, 0.29422E+01_JPRB ,&
&  0.25477E+01_JPRB, 0.21657E+01_JPRB, 0.19237E+01_JPRB, 0.17343E+01_JPRB, 0.22179E+01_JPRB, 0.32564E+01_JPRB, 0.36506E+01_JPRB ,&
&  0.36136E+01_JPRB, 0.36459E+01_JPRB, 0.40146E+01_JPRB, 0.37944E+01_JPRB, 0.28198E+01_JPRB, 0.20803E+01_JPRB, 0.26896E+01_JPRB ,&
&  0.28846E+01_JPRB, 0.29188E+01_JPRB, 0.26488E+01_JPRB, 0.21579E+01_JPRB, 0.17451E+01_JPRB, 0.19418E+01_JPRB, 0.17921E+01_JPRB ,&
&  0.16143E+01_JPRB, 0.15505E+01_JPRB, 0.19165E+01_JPRB, 0.24743E+01_JPRB, 0.26969E+01_JPRB, 0.29963E+01_JPRB, 0.30889E+01_JPRB ,&
&  0.29313E+01_JPRB, 0.28170E+01_JPRB, 0.26630E+01_JPRB, 0.24837E+01_JPRB, 0.23197E+01_JPRB, 0.21527E+01_JPRB, 0.20217E+01_JPRB ,&
&  0.19096E+01_JPRB, 0.18072E+01_JPRB, 0.16951E+01_JPRB, 0.15864E+01_JPRB, 0.14941E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.14789E+01_JPRB, 0.14536E+01_JPRB, 0.14305E+01_JPRB, 0.14061E+01_JPRB, 0.13703E+01_JPRB, 0.13247E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.11258E+01_JPRB, 0.10652E+01_JPRB, 0.10341E+01_JPRB, 0.10064E+01_JPRB, 0.97202E+00_JPRB, 0.95754E+00_JPRB ,&
&  0.97141E+00_JPRB, 0.10025E+01_JPRB, 0.92659E+00_JPRB, 0.90284E+00_JPRB, 0.92959E+00_JPRB, 0.93327E+00_JPRB, 0.81976E+00_JPRB ,&
&  0.91724E+00_JPRB, 0.10182E+01_JPRB, 0.11061E+01_JPRB, 0.12446E+01_JPRB, 0.14665E+01_JPRB, 0.14301E+01_JPRB, 0.14033E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.13200E+01_JPRB, 0.12605E+01_JPRB, 0.11797E+01_JPRB, 0.11219E+01_JPRB, 0.11269E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.11907E+01_JPRB, 0.11612E+01_JPRB, 0.11033E+01_JPRB, 0.10553E+01_JPRB, 0.10216E+01_JPRB, 0.99677E+00_JPRB ,&
&  0.99342E+00_JPRB, 0.98765E+00_JPRB, 0.98795E+00_JPRB, 0.10657E+01_JPRB, 0.10206E+01_JPRB, 0.10101E+01_JPRB, 0.11864E+01_JPRB ,&
&  0.13746E+01_JPRB, 0.17752E+01_JPRB, 0.19507E+01_JPRB, 0.22391E+01_JPRB, 0.25617E+01_JPRB, 0.26962E+01_JPRB, 0.28470E+01_JPRB ,&
&  0.22903E+01_JPRB, 0.20646E+01_JPRB, 0.23829E+01_JPRB, 0.28039E+01_JPRB, 0.28468E+01_JPRB, 0.31656E+01_JPRB, 0.52465E+01_JPRB ,&
&  0.38013E+01_JPRB, 0.40908E+01_JPRB, 0.46387E+01_JPRB, 0.46683E+01_JPRB, 0.22107E+01_JPRB, 0.15786E+01_JPRB, 0.24762E+01_JPRB ,&
&  0.25440E+01_JPRB, 0.15641E+01_JPRB, 0.12002E+01_JPRB, 0.14905E+01_JPRB, 0.11901E+01_JPRB, 0.13632E+01_JPRB, 0.16989E+01_JPRB ,&
&  0.18033E+01_JPRB, 0.18586E+01_JPRB, 0.24214E+01_JPRB, 0.27048E+01_JPRB, 0.29109E+01_JPRB, 0.42827E+01_JPRB, 0.37803E+01_JPRB ,&
&  0.34633E+01_JPRB, 0.29065E+01_JPRB, 0.27359E+01_JPRB, 0.24314E+01_JPRB, 0.22138E+01_JPRB, 0.20195E+01_JPRB, 0.18638E+01_JPRB ,&
&  0.17490E+01_JPRB, 0.16569E+01_JPRB, 0.15908E+01_JPRB, 0.15376E+01_JPRB, 0.15088E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.15252E+01_JPRB, 0.14956E+01_JPRB, 0.14644E+01_JPRB, 0.14251E+01_JPRB, 0.13775E+01_JPRB, 0.13142E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.10889E+01_JPRB, 0.10404E+01_JPRB, 0.10122E+01_JPRB, 0.97940E+00_JPRB, 0.95131E+00_JPRB, 0.95833E+00_JPRB ,&
&  0.98593E+00_JPRB, 0.10252E+01_JPRB, 0.10270E+01_JPRB, 0.10222E+01_JPRB, 0.10539E+01_JPRB, 0.10703E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.11757E+01_JPRB, 0.12210E+01_JPRB, 0.13092E+01_JPRB, 0.13897E+01_JPRB, 0.13552E+01_JPRB, 0.13864E+01_JPRB ,&
&  0.13612E+01_JPRB, 0.13347E+01_JPRB, 0.12700E+01_JPRB, 0.12054E+01_JPRB, 0.11803E+01_JPRB, 0.11764E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.12164E+01_JPRB, 0.11738E+01_JPRB, 0.11011E+01_JPRB, 0.10599E+01_JPRB, 0.10223E+01_JPRB, 0.99901E+00_JPRB ,&
&  0.99045E+00_JPRB, 0.10104E+01_JPRB, 0.10529E+01_JPRB, 0.11988E+01_JPRB, 0.14849E+01_JPRB, 0.16549E+01_JPRB, 0.15282E+01_JPRB ,&
&  0.14514E+01_JPRB, 0.16903E+01_JPRB, 0.21741E+01_JPRB, 0.22864E+01_JPRB, 0.24700E+01_JPRB, 0.26472E+01_JPRB, 0.26431E+01_JPRB ,&
&  0.26397E+01_JPRB, 0.30280E+01_JPRB, 0.31558E+01_JPRB, 0.36858E+01_JPRB, 0.41132E+01_JPRB, 0.33827E+01_JPRB, 0.43226E+01_JPRB ,&
&  0.44299E+01_JPRB, 0.43206E+01_JPRB, 0.35333E+01_JPRB, 0.26409E+01_JPRB, 0.29284E+01_JPRB, 0.21683E+01_JPRB, 0.13358E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.10510E+01_JPRB, 0.98966E+00_JPRB, 0.10203E+01_JPRB, 0.10236E+01_JPRB, 0.12843E+01_JPRB, 0.16418E+01_JPRB ,&
&  0.21753E+01_JPRB, 0.24557E+01_JPRB, 0.26268E+01_JPRB, 0.27452E+01_JPRB, 0.28043E+01_JPRB, 0.40598E+01_JPRB, 0.36786E+01_JPRB ,&
&  0.30147E+01_JPRB, 0.27243E+01_JPRB, 0.25030E+01_JPRB, 0.22702E+01_JPRB, 0.20995E+01_JPRB, 0.19604E+01_JPRB, 0.18466E+01_JPRB ,&
&  0.17623E+01_JPRB, 0.16950E+01_JPRB, 0.16457E+01_JPRB, 0.16034E+01_JPRB, 0.15567E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.15163E+01_JPRB, 0.14811E+01_JPRB, 0.14455E+01_JPRB, 0.13930E+01_JPRB, 0.13336E+01_JPRB, 0.12583E+01_JPRB, 0.11656E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.10253E+01_JPRB, 0.99665E+00_JPRB, 0.96744E+00_JPRB, 0.94882E+00_JPRB, 0.94196E+00_JPRB, 0.96435E+00_JPRB ,&
&  0.10015E+01_JPRB, 0.10498E+01_JPRB, 0.11177E+01_JPRB, 0.11809E+01_JPRB, 0.13591E+01_JPRB, 0.13925E+01_JPRB, 0.14196E+01_JPRB ,&
&  0.14809E+01_JPRB, 0.14506E+01_JPRB, 0.14428E+01_JPRB, 0.14394E+01_JPRB, 0.14560E+01_JPRB, 0.14179E+01_JPRB, 0.13953E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.13717E+01_JPRB, 0.13179E+01_JPRB, 0.12658E+01_JPRB, 0.12293E+01_JPRB, 0.12309E+01_JPRB, 0.12208E+01_JPRB ,&
&  0.12305E+01_JPRB, 0.12341E+01_JPRB, 0.12000E+01_JPRB, 0.11281E+01_JPRB, 0.10863E+01_JPRB, 0.10629E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10779E+01_JPRB, 0.11485E+01_JPRB, 0.12466E+01_JPRB, 0.14629E+01_JPRB, 0.14600E+01_JPRB, 0.12486E+01_JPRB, 0.14009E+01_JPRB ,&
&  0.15326E+01_JPRB, 0.19183E+01_JPRB, 0.20813E+01_JPRB, 0.23685E+01_JPRB, 0.24451E+01_JPRB, 0.23885E+01_JPRB, 0.26227E+01_JPRB ,&
&  0.30243E+01_JPRB, 0.43662E+01_JPRB, 0.34616E+01_JPRB, 0.36018E+01_JPRB, 0.58005E+01_JPRB, 0.47582E+01_JPRB, 0.44317E+01_JPRB ,&
&  0.50533E+01_JPRB, 0.44677E+01_JPRB, 0.44302E+01_JPRB, 0.38575E+01_JPRB, 0.63195E+01_JPRB, 0.67886E+01_JPRB, 0.19816E+01_JPRB ,&
&  0.13472E+01_JPRB, 0.11616E+01_JPRB, 0.10532E+01_JPRB, 0.96219E+00_JPRB, 0.10251E+01_JPRB, 0.13617E+01_JPRB, 0.20194E+01_JPRB ,&
&  0.23448E+01_JPRB, 0.27838E+01_JPRB, 0.28379E+01_JPRB, 0.29930E+01_JPRB, 0.28397E+01_JPRB, 0.32650E+01_JPRB, 0.27412E+01_JPRB ,&
&  0.26023E+01_JPRB, 0.24793E+01_JPRB, 0.22460E+01_JPRB, 0.20550E+01_JPRB, 0.19420E+01_JPRB, 0.18590E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.17129E+01_JPRB, 0.16699E+01_JPRB, 0.16311E+01_JPRB, 0.15918E+01_JPRB, 0.15534E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.14523E+01_JPRB, 0.14207E+01_JPRB, 0.13782E+01_JPRB, 0.13129E+01_JPRB, 0.12585E+01_JPRB, 0.11899E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10139E+01_JPRB, 0.97976E+00_JPRB, 0.94692E+00_JPRB, 0.94122E+00_JPRB, 0.95068E+00_JPRB, 0.98630E+00_JPRB ,&
&  0.10380E+01_JPRB, 0.11088E+01_JPRB, 0.12047E+01_JPRB, 0.14558E+01_JPRB, 0.17297E+01_JPRB, 0.17976E+01_JPRB, 0.21571E+01_JPRB ,&
&  0.20566E+01_JPRB, 0.18799E+01_JPRB, 0.17647E+01_JPRB, 0.16565E+01_JPRB, 0.15788E+01_JPRB, 0.14870E+01_JPRB, 0.14419E+01_JPRB ,&
&  0.13973E+01_JPRB, 0.14300E+01_JPRB, 0.13722E+01_JPRB, 0.13278E+01_JPRB, 0.13091E+01_JPRB, 0.12861E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.12692E+01_JPRB, 0.12712E+01_JPRB, 0.12476E+01_JPRB, 0.11917E+01_JPRB, 0.11506E+01_JPRB, 0.11370E+01_JPRB, 0.11589E+01_JPRB ,&
&  0.12273E+01_JPRB, 0.13463E+01_JPRB, 0.14733E+01_JPRB, 0.16175E+01_JPRB, 0.13434E+01_JPRB, 0.15271E+01_JPRB, 0.16113E+01_JPRB ,&
&  0.18192E+01_JPRB, 0.20046E+01_JPRB, 0.20156E+01_JPRB, 0.21583E+01_JPRB, 0.24038E+01_JPRB, 0.24461E+01_JPRB, 0.27695E+01_JPRB ,&
&  0.37182E+01_JPRB, 0.37599E+01_JPRB, 0.33992E+01_JPRB, 0.38350E+01_JPRB, 0.52630E+01_JPRB, 0.78360E+01_JPRB, 0.41039E+01_JPRB ,&
&  0.33363E+01_JPRB, 0.46962E+01_JPRB, 0.50341E+01_JPRB, 0.49654E+01_JPRB, 0.73086E+01_JPRB, 0.97644E+01_JPRB, 0.59885E+01_JPRB ,&
&  0.16748E+01_JPRB, 0.11745E+01_JPRB, 0.13257E+01_JPRB, 0.11811E+01_JPRB, 0.11041E+01_JPRB, 0.14566E+01_JPRB, 0.33320E+01_JPRB ,&
&  0.27498E+01_JPRB, 0.30854E+01_JPRB, 0.31234E+01_JPRB, 0.30663E+01_JPRB, 0.28610E+01_JPRB, 0.28345E+01_JPRB, 0.25212E+01_JPRB ,&
&  0.21543E+01_JPRB, 0.20031E+01_JPRB, 0.18189E+01_JPRB, 0.17212E+01_JPRB, 0.16862E+01_JPRB, 0.16615E+01_JPRB, 0.16077E+01_JPRB ,&
&  0.15700E+01_JPRB, 0.15469E+01_JPRB, 0.15242E+01_JPRB, 0.15060E+01_JPRB, 0.14846E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.13648E+01_JPRB, 0.13344E+01_JPRB, 0.12782E+01_JPRB, 0.12319E+01_JPRB, 0.12157E+01_JPRB, 0.11575E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.10604E+01_JPRB, 0.10359E+01_JPRB, 0.10060E+01_JPRB, 0.10015E+01_JPRB, 0.98399E+00_JPRB, 0.10034E+01_JPRB, 0.10478E+01_JPRB ,&
&  0.11144E+01_JPRB, 0.11952E+01_JPRB, 0.13069E+01_JPRB, 0.16038E+01_JPRB, 0.20151E+01_JPRB, 0.21022E+01_JPRB, 0.25653E+01_JPRB ,&
&  0.29197E+01_JPRB, 0.25671E+01_JPRB, 0.21524E+01_JPRB, 0.19112E+01_JPRB, 0.17435E+01_JPRB, 0.16415E+01_JPRB, 0.15512E+01_JPRB ,&
&  0.14999E+01_JPRB, 0.15393E+01_JPRB, 0.14584E+01_JPRB, 0.13936E+01_JPRB, 0.13337E+01_JPRB, 0.13172E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.13347E+01_JPRB, 0.13518E+01_JPRB, 0.13392E+01_JPRB, 0.13110E+01_JPRB, 0.12806E+01_JPRB, 0.12804E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.16509E+01_JPRB, 0.17418E+01_JPRB, 0.17199E+01_JPRB, 0.17337E+01_JPRB, 0.17588E+01_JPRB, 0.19493E+01_JPRB ,&
&  0.21176E+01_JPRB, 0.22137E+01_JPRB, 0.22544E+01_JPRB, 0.22747E+01_JPRB, 0.24944E+01_JPRB, 0.27942E+01_JPRB, 0.32374E+01_JPRB ,&
&  0.39612E+01_JPRB, 0.42150E+01_JPRB, 0.38178E+01_JPRB, 0.41174E+01_JPRB, 0.50164E+01_JPRB, 0.62288E+01_JPRB, 0.55663E+01_JPRB ,&
&  0.45416E+01_JPRB, 0.40247E+01_JPRB, 0.46895E+01_JPRB, 0.60898E+01_JPRB, 0.69253E+01_JPRB, 0.69772E+01_JPRB, 0.84070E+01_JPRB ,&
&  0.69871E+01_JPRB, 0.32003E+01_JPRB, 0.24570E+01_JPRB, 0.28663E+01_JPRB, 0.19946E+01_JPRB, 0.19980E+01_JPRB, 0.26839E+01_JPRB ,&
&  0.28687E+01_JPRB, 0.32175E+01_JPRB, 0.32918E+01_JPRB, 0.28049E+01_JPRB, 0.26892E+01_JPRB, 0.25507E+01_JPRB, 0.21589E+01_JPRB ,&
&  0.18514E+01_JPRB, 0.16720E+01_JPRB, 0.15615E+01_JPRB, 0.14925E+01_JPRB, 0.14532E+01_JPRB, 0.14144E+01_JPRB, 0.13918E+01_JPRB ,&
&  0.13674E+01_JPRB, 0.13584E+01_JPRB, 0.13607E+01_JPRB, 0.13657E+01_JPRB, 0.13733E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.12909E+01_JPRB, 0.12500E+01_JPRB, 0.12410E+01_JPRB, 0.12519E+01_JPRB, 0.12382E+01_JPRB, 0.12126E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11225E+01_JPRB, 0.11522E+01_JPRB, 0.11456E+01_JPRB, 0.11092E+01_JPRB, 0.11167E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.12310E+01_JPRB, 0.13680E+01_JPRB, 0.15828E+01_JPRB, 0.20402E+01_JPRB, 0.25579E+01_JPRB, 0.34159E+01_JPRB, 0.26409E+01_JPRB ,&
&  0.29346E+01_JPRB, 0.29043E+01_JPRB, 0.23300E+01_JPRB, 0.21005E+01_JPRB, 0.19595E+01_JPRB, 0.18966E+01_JPRB, 0.17276E+01_JPRB ,&
&  0.16958E+01_JPRB, 0.16614E+01_JPRB, 0.15583E+01_JPRB, 0.14361E+01_JPRB, 0.13666E+01_JPRB, 0.14344E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.14602E+01_JPRB, 0.15021E+01_JPRB, 0.14862E+01_JPRB, 0.14950E+01_JPRB, 0.14849E+01_JPRB, 0.15017E+01_JPRB, 0.16166E+01_JPRB ,&
&  0.17773E+01_JPRB, 0.19115E+01_JPRB, 0.19967E+01_JPRB, 0.19196E+01_JPRB, 0.20088E+01_JPRB, 0.21367E+01_JPRB, 0.23891E+01_JPRB ,&
&  0.24425E+01_JPRB, 0.22265E+01_JPRB, 0.22970E+01_JPRB, 0.25536E+01_JPRB, 0.28795E+01_JPRB, 0.30372E+01_JPRB, 0.34541E+01_JPRB ,&
&  0.44204E+01_JPRB, 0.52156E+01_JPRB, 0.55321E+01_JPRB, 0.47060E+01_JPRB, 0.47735E+01_JPRB, 0.56881E+01_JPRB, 0.62963E+01_JPRB ,&
&  0.61929E+01_JPRB, 0.57472E+01_JPRB, 0.60481E+01_JPRB, 0.59040E+01_JPRB, 0.56338E+01_JPRB, 0.52553E+01_JPRB, 0.55214E+01_JPRB ,&
&  0.62369E+01_JPRB, 0.67123E+01_JPRB, 0.47645E+01_JPRB, 0.36285E+01_JPRB, 0.33613E+01_JPRB, 0.29138E+01_JPRB, 0.27501E+01_JPRB ,&
&  0.31005E+01_JPRB, 0.31567E+01_JPRB, 0.31007E+01_JPRB, 0.33220E+01_JPRB, 0.26661E+01_JPRB, 0.21825E+01_JPRB, 0.16807E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.12789E+01_JPRB, 0.12371E+01_JPRB, 0.12230E+01_JPRB, 0.12576E+01_JPRB, 0.12631E+01_JPRB, 0.12661E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.12691E+01_JPRB, 0.12978E+01_JPRB, 0.13212E+01_JPRB, 0.13197E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.12442E+01_JPRB, 0.12427E+01_JPRB, 0.12682E+01_JPRB, 0.12927E+01_JPRB, 0.12943E+01_JPRB, 0.13159E+01_JPRB, 0.13069E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.12102E+01_JPRB, 0.12451E+01_JPRB, 0.12264E+01_JPRB, 0.12080E+01_JPRB, 0.12260E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.14026E+01_JPRB, 0.16344E+01_JPRB, 0.20189E+01_JPRB, 0.25199E+01_JPRB, 0.28800E+01_JPRB, 0.33074E+01_JPRB, 0.35133E+01_JPRB ,&
&  0.26026E+01_JPRB, 0.30494E+01_JPRB, 0.22973E+01_JPRB, 0.21842E+01_JPRB, 0.21750E+01_JPRB, 0.23961E+01_JPRB, 0.22222E+01_JPRB ,&
&  0.19010E+01_JPRB, 0.17030E+01_JPRB, 0.16813E+01_JPRB, 0.16064E+01_JPRB, 0.15422E+01_JPRB, 0.15611E+01_JPRB, 0.15882E+01_JPRB ,&
&  0.15766E+01_JPRB, 0.15917E+01_JPRB, 0.16500E+01_JPRB, 0.17011E+01_JPRB, 0.17028E+01_JPRB, 0.17306E+01_JPRB, 0.18019E+01_JPRB ,&
&  0.19897E+01_JPRB, 0.21516E+01_JPRB, 0.22044E+01_JPRB, 0.22712E+01_JPRB, 0.23247E+01_JPRB, 0.26177E+01_JPRB, 0.29032E+01_JPRB ,&
&  0.28489E+01_JPRB, 0.25476E+01_JPRB, 0.25899E+01_JPRB, 0.27862E+01_JPRB, 0.29668E+01_JPRB, 0.34474E+01_JPRB, 0.38133E+01_JPRB ,&
&  0.48797E+01_JPRB, 0.49828E+01_JPRB, 0.60146E+01_JPRB, 0.47606E+01_JPRB, 0.45991E+01_JPRB, 0.53979E+01_JPRB, 0.52046E+01_JPRB ,&
&  0.46649E+01_JPRB, 0.53075E+01_JPRB, 0.54293E+01_JPRB, 0.54986E+01_JPRB, 0.52541E+01_JPRB, 0.46677E+01_JPRB, 0.43408E+01_JPRB ,&
&  0.44373E+01_JPRB, 0.46700E+01_JPRB, 0.43475E+01_JPRB, 0.35483E+01_JPRB, 0.32597E+01_JPRB, 0.29251E+01_JPRB, 0.30602E+01_JPRB ,&
&  0.34753E+01_JPRB, 0.36859E+01_JPRB, 0.36338E+01_JPRB, 0.32879E+01_JPRB, 0.20008E+01_JPRB, 0.15046E+01_JPRB, 0.12282E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.10921E+01_JPRB, 0.11163E+01_JPRB, 0.11725E+01_JPRB, 0.12207E+01_JPRB, 0.12423E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.12377E+01_JPRB, 0.12562E+01_JPRB, 0.12670E+01_JPRB, 0.12660E+01_JPRB, 0.12579E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.12652E+01_JPRB, 0.12910E+01_JPRB, 0.13238E+01_JPRB, 0.13752E+01_JPRB, 0.14139E+01_JPRB, 0.15035E+01_JPRB, 0.15888E+01_JPRB ,&
&  0.14751E+01_JPRB, 0.11493E+01_JPRB, 0.11868E+01_JPRB, 0.12256E+01_JPRB, 0.12707E+01_JPRB, 0.13342E+01_JPRB, 0.14399E+01_JPRB ,&
&  0.16478E+01_JPRB, 0.19213E+01_JPRB, 0.22422E+01_JPRB, 0.25615E+01_JPRB, 0.29350E+01_JPRB, 0.33300E+01_JPRB, 0.33905E+01_JPRB ,&
&  0.27616E+01_JPRB, 0.23619E+01_JPRB, 0.24368E+01_JPRB, 0.23964E+01_JPRB, 0.24162E+01_JPRB, 0.25029E+01_JPRB, 0.25534E+01_JPRB ,&
&  0.21437E+01_JPRB, 0.19202E+01_JPRB, 0.19263E+01_JPRB, 0.19679E+01_JPRB, 0.17238E+01_JPRB, 0.17895E+01_JPRB, 0.18408E+01_JPRB ,&
&  0.18513E+01_JPRB, 0.18630E+01_JPRB, 0.19487E+01_JPRB, 0.20289E+01_JPRB, 0.20154E+01_JPRB, 0.20397E+01_JPRB, 0.21455E+01_JPRB ,&
&  0.23895E+01_JPRB, 0.25725E+01_JPRB, 0.26779E+01_JPRB, 0.26927E+01_JPRB, 0.28696E+01_JPRB, 0.31869E+01_JPRB, 0.32993E+01_JPRB ,&
&  0.31365E+01_JPRB, 0.30795E+01_JPRB, 0.31400E+01_JPRB, 0.32957E+01_JPRB, 0.32520E+01_JPRB, 0.39269E+01_JPRB, 0.42509E+01_JPRB ,&
&  0.47168E+01_JPRB, 0.48949E+01_JPRB, 0.57998E+01_JPRB, 0.43216E+01_JPRB, 0.46722E+01_JPRB, 0.49596E+01_JPRB, 0.46165E+01_JPRB ,&
&  0.41968E+01_JPRB, 0.44625E+01_JPRB, 0.44215E+01_JPRB, 0.44933E+01_JPRB, 0.44575E+01_JPRB, 0.38508E+01_JPRB, 0.40033E+01_JPRB ,&
&  0.37869E+01_JPRB, 0.41723E+01_JPRB, 0.38963E+01_JPRB, 0.37188E+01_JPRB, 0.31534E+01_JPRB, 0.29912E+01_JPRB, 0.31025E+01_JPRB ,&
&  0.35520E+01_JPRB, 0.36656E+01_JPRB, 0.31195E+01_JPRB, 0.23909E+01_JPRB, 0.13497E+01_JPRB, 0.10794E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10889E+01_JPRB, 0.11462E+01_JPRB, 0.11978E+01_JPRB, 0.12265E+01_JPRB, 0.12332E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.12475E+01_JPRB, 0.12544E+01_JPRB, 0.12506E+01_JPRB, 0.12521E+01_JPRB, 0.12562E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.12724E+01_JPRB, 0.12746E+01_JPRB, 0.12803E+01_JPRB, 0.12577E+01_JPRB, 0.12266E+01_JPRB, 0.11857E+01_JPRB, 0.11118E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10425E+01_JPRB, 0.11078E+01_JPRB, 0.11916E+01_JPRB, 0.12738E+01_JPRB, 0.13735E+01_JPRB, 0.15287E+01_JPRB ,&
&  0.17176E+01_JPRB, 0.18656E+01_JPRB, 0.20353E+01_JPRB, 0.23098E+01_JPRB, 0.26401E+01_JPRB, 0.27674E+01_JPRB, 0.28597E+01_JPRB ,&
&  0.25840E+01_JPRB, 0.19351E+01_JPRB, 0.23705E+01_JPRB, 0.26441E+01_JPRB, 0.27285E+01_JPRB, 0.27993E+01_JPRB, 0.26666E+01_JPRB ,&
&  0.22035E+01_JPRB, 0.22090E+01_JPRB, 0.23505E+01_JPRB, 0.24040E+01_JPRB, 0.20702E+01_JPRB, 0.21313E+01_JPRB, 0.22026E+01_JPRB ,&
&  0.22734E+01_JPRB, 0.23272E+01_JPRB, 0.24584E+01_JPRB, 0.25413E+01_JPRB, 0.25688E+01_JPRB, 0.26606E+01_JPRB, 0.28217E+01_JPRB ,&
&  0.29922E+01_JPRB, 0.31234E+01_JPRB, 0.33056E+01_JPRB, 0.34251E+01_JPRB, 0.35564E+01_JPRB, 0.37389E+01_JPRB, 0.37000E+01_JPRB ,&
&  0.35460E+01_JPRB, 0.34042E+01_JPRB, 0.36070E+01_JPRB, 0.40875E+01_JPRB, 0.44754E+01_JPRB, 0.40683E+01_JPRB, 0.45242E+01_JPRB ,&
&  0.45906E+01_JPRB, 0.46942E+01_JPRB, 0.46440E+01_JPRB, 0.53736E+01_JPRB, 0.37361E+01_JPRB, 0.41416E+01_JPRB, 0.36413E+01_JPRB ,&
&  0.38111E+01_JPRB, 0.36511E+01_JPRB, 0.35202E+01_JPRB, 0.35352E+01_JPRB, 0.35880E+01_JPRB, 0.32130E+01_JPRB, 0.33617E+01_JPRB ,&
&  0.35408E+01_JPRB, 0.36807E+01_JPRB, 0.37226E+01_JPRB, 0.37923E+01_JPRB, 0.35418E+01_JPRB, 0.34942E+01_JPRB, 0.34422E+01_JPRB ,&
&  0.36777E+01_JPRB, 0.29712E+01_JPRB, 0.22734E+01_JPRB, 0.16284E+01_JPRB, 0.11045E+01_JPRB, 0.94424E+00_JPRB, 0.99336E+00_JPRB ,&
&  0.10462E+01_JPRB, 0.11089E+01_JPRB, 0.11515E+01_JPRB, 0.11840E+01_JPRB, 0.12044E+01_JPRB, 0.12185E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.12557E+01_JPRB, 0.12603E+01_JPRB, 0.12587E+01_JPRB, 0.12593E+01_JPRB, 0.12647E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.12425E+01_JPRB, 0.12185E+01_JPRB, 0.11703E+01_JPRB, 0.10939E+01_JPRB, 0.10124E+01_JPRB, 0.93234E+00_JPRB, 0.88539E+00_JPRB ,&
&  0.85096E+00_JPRB, 0.88239E+00_JPRB, 0.96511E+00_JPRB, 0.10771E+01_JPRB, 0.12061E+01_JPRB, 0.13365E+01_JPRB, 0.14591E+01_JPRB ,&
&  0.15341E+01_JPRB, 0.15844E+01_JPRB, 0.16907E+01_JPRB, 0.19402E+01_JPRB, 0.22247E+01_JPRB, 0.24404E+01_JPRB, 0.25808E+01_JPRB ,&
&  0.26916E+01_JPRB, 0.27089E+01_JPRB, 0.25622E+01_JPRB, 0.21217E+01_JPRB, 0.20202E+01_JPRB, 0.25535E+01_JPRB, 0.23197E+01_JPRB ,&
&  0.22698E+01_JPRB, 0.22442E+01_JPRB, 0.23017E+01_JPRB, 0.23921E+01_JPRB, 0.24086E+01_JPRB, 0.24543E+01_JPRB, 0.25813E+01_JPRB ,&
&  0.26910E+01_JPRB, 0.28226E+01_JPRB, 0.29520E+01_JPRB, 0.30495E+01_JPRB, 0.31666E+01_JPRB, 0.33129E+01_JPRB, 0.34564E+01_JPRB ,&
&  0.35943E+01_JPRB, 0.37967E+01_JPRB, 0.40330E+01_JPRB, 0.42164E+01_JPRB, 0.41802E+01_JPRB, 0.42762E+01_JPRB, 0.42051E+01_JPRB ,&
&  0.40865E+01_JPRB, 0.39984E+01_JPRB, 0.40318E+01_JPRB, 0.43370E+01_JPRB, 0.44740E+01_JPRB, 0.39399E+01_JPRB, 0.39910E+01_JPRB ,&
&  0.43157E+01_JPRB, 0.44099E+01_JPRB, 0.38741E+01_JPRB, 0.50277E+01_JPRB, 0.29641E+01_JPRB, 0.35538E+01_JPRB, 0.36899E+01_JPRB ,&
&  0.33724E+01_JPRB, 0.30588E+01_JPRB, 0.29869E+01_JPRB, 0.30244E+01_JPRB, 0.29730E+01_JPRB, 0.28690E+01_JPRB, 0.31199E+01_JPRB ,&
&  0.32479E+01_JPRB, 0.33526E+01_JPRB, 0.35937E+01_JPRB, 0.36331E+01_JPRB, 0.36059E+01_JPRB, 0.33365E+01_JPRB, 0.29008E+01_JPRB ,&
&  0.24476E+01_JPRB, 0.24112E+01_JPRB, 0.20545E+01_JPRB, 0.17937E+01_JPRB, 0.12083E+01_JPRB, 0.92672E+00_JPRB, 0.98842E+00_JPRB ,&
&  0.10379E+01_JPRB, 0.10740E+01_JPRB, 0.11236E+01_JPRB, 0.11611E+01_JPRB, 0.11739E+01_JPRB, 0.11902E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12227E+01_JPRB, 0.12228E+01_JPRB, 0.12280E+01_JPRB, 0.12315E+01_JPRB, 0.12414E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.10328E+01_JPRB, 0.10006E+01_JPRB, 0.94901E+00_JPRB, 0.90034E+00_JPRB, 0.85239E+00_JPRB, 0.79765E+00_JPRB, 0.78804E+00_JPRB ,&
&  0.76617E+00_JPRB, 0.77592E+00_JPRB, 0.81358E+00_JPRB, 0.86173E+00_JPRB, 0.93703E+00_JPRB, 0.10446E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11963E+01_JPRB, 0.12108E+01_JPRB, 0.12799E+01_JPRB, 0.13868E+01_JPRB, 0.15908E+01_JPRB, 0.19574E+01_JPRB, 0.25701E+01_JPRB ,&
&  0.29157E+01_JPRB, 0.31047E+01_JPRB, 0.30499E+01_JPRB, 0.28619E+01_JPRB, 0.25256E+01_JPRB, 0.26940E+01_JPRB, 0.29575E+01_JPRB ,&
&  0.31372E+01_JPRB, 0.28776E+01_JPRB, 0.25095E+01_JPRB, 0.25391E+01_JPRB, 0.26242E+01_JPRB, 0.27291E+01_JPRB, 0.28711E+01_JPRB ,&
&  0.30174E+01_JPRB, 0.31326E+01_JPRB, 0.32523E+01_JPRB, 0.33951E+01_JPRB, 0.35223E+01_JPRB, 0.36471E+01_JPRB, 0.38033E+01_JPRB ,&
&  0.39792E+01_JPRB, 0.41478E+01_JPRB, 0.42809E+01_JPRB, 0.44505E+01_JPRB, 0.46975E+01_JPRB, 0.49297E+01_JPRB, 0.49680E+01_JPRB ,&
&  0.48983E+01_JPRB, 0.46012E+01_JPRB, 0.46980E+01_JPRB, 0.49445E+01_JPRB, 0.47646E+01_JPRB, 0.43218E+01_JPRB, 0.41948E+01_JPRB ,&
&  0.43860E+01_JPRB, 0.45090E+01_JPRB, 0.30941E+01_JPRB, 0.39660E+01_JPRB, 0.44420E+01_JPRB, 0.38049E+01_JPRB, 0.32193E+01_JPRB ,&
&  0.28397E+01_JPRB, 0.26078E+01_JPRB, 0.26821E+01_JPRB, 0.27390E+01_JPRB, 0.29647E+01_JPRB, 0.29163E+01_JPRB, 0.25084E+01_JPRB ,&
&  0.25009E+01_JPRB, 0.26427E+01_JPRB, 0.29993E+01_JPRB, 0.31642E+01_JPRB, 0.30229E+01_JPRB, 0.26684E+01_JPRB, 0.21256E+01_JPRB ,&
&  0.17647E+01_JPRB, 0.20130E+01_JPRB, 0.18004E+01_JPRB, 0.17788E+01_JPRB, 0.14811E+01_JPRB, 0.89961E+00_JPRB, 0.95658E+00_JPRB ,&
&  0.98642E+00_JPRB, 0.10179E+01_JPRB, 0.10548E+01_JPRB, 0.10764E+01_JPRB, 0.10991E+01_JPRB, 0.11147E+01_JPRB, 0.11354E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11146E+01_JPRB, 0.11005E+01_JPRB, 0.10907E+01_JPRB, 0.10691E+01_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.90828E+00_JPRB, 0.89442E+00_JPRB, 0.87964E+00_JPRB, 0.87355E+00_JPRB, 0.87737E+00_JPRB, 0.86873E+00_JPRB, 0.84791E+00_JPRB ,&
&  0.81778E+00_JPRB, 0.80445E+00_JPRB, 0.77750E+00_JPRB, 0.75237E+00_JPRB, 0.76152E+00_JPRB, 0.82775E+00_JPRB, 0.89691E+00_JPRB ,&
&  0.93795E+00_JPRB, 0.95365E+00_JPRB, 0.94649E+00_JPRB, 0.93879E+00_JPRB, 0.97338E+00_JPRB, 0.10167E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.18258E+01_JPRB, 0.25410E+01_JPRB, 0.30704E+01_JPRB, 0.33509E+01_JPRB, 0.32877E+01_JPRB, 0.29724E+01_JPRB, 0.36473E+01_JPRB ,&
&  0.33486E+01_JPRB, 0.26020E+01_JPRB, 0.23932E+01_JPRB, 0.23588E+01_JPRB, 0.24233E+01_JPRB, 0.25622E+01_JPRB, 0.27308E+01_JPRB ,&
&  0.28887E+01_JPRB, 0.30461E+01_JPRB, 0.32094E+01_JPRB, 0.33970E+01_JPRB, 0.35965E+01_JPRB, 0.38090E+01_JPRB, 0.40240E+01_JPRB ,&
&  0.42883E+01_JPRB, 0.44649E+01_JPRB, 0.46632E+01_JPRB, 0.45458E+01_JPRB, 0.50292E+01_JPRB, 0.54299E+01_JPRB, 0.56382E+01_JPRB ,&
&  0.55980E+01_JPRB, 0.54294E+01_JPRB, 0.55922E+01_JPRB, 0.56039E+01_JPRB, 0.55668E+01_JPRB, 0.53065E+01_JPRB, 0.49287E+01_JPRB ,&
&  0.50372E+01_JPRB, 0.46761E+01_JPRB, 0.31565E+01_JPRB, 0.32243E+01_JPRB, 0.35145E+01_JPRB, 0.27595E+01_JPRB, 0.28999E+01_JPRB ,&
&  0.24787E+01_JPRB, 0.22025E+01_JPRB, 0.19980E+01_JPRB, 0.20146E+01_JPRB, 0.23229E+01_JPRB, 0.26190E+01_JPRB, 0.21996E+01_JPRB ,&
&  0.17970E+01_JPRB, 0.18917E+01_JPRB, 0.21365E+01_JPRB, 0.22902E+01_JPRB, 0.22132E+01_JPRB, 0.19923E+01_JPRB, 0.17507E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.16482E+01_JPRB, 0.15464E+01_JPRB, 0.14212E+01_JPRB, 0.10237E+01_JPRB, 0.83293E+00_JPRB, 0.86301E+00_JPRB ,&
&  0.88275E+00_JPRB, 0.88249E+00_JPRB, 0.89745E+00_JPRB, 0.91112E+00_JPRB, 0.93556E+00_JPRB, 0.93890E+00_JPRB, 0.95863E+00_JPRB ,&
&  0.96931E+00_JPRB, 0.96570E+00_JPRB, 0.96113E+00_JPRB, 0.96272E+00_JPRB, 0.93728E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.99073E+00_JPRB, 0.99815E+00_JPRB, 0.10144E+01_JPRB, 0.10314E+01_JPRB, 0.10540E+01_JPRB, 0.10377E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.97153E+00_JPRB, 0.96181E+00_JPRB, 0.95163E+00_JPRB, 0.94621E+00_JPRB, 0.95941E+00_JPRB, 0.99338E+00_JPRB, 0.10557E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10785E+01_JPRB, 0.10628E+01_JPRB, 0.10422E+01_JPRB, 0.98985E+00_JPRB, 0.94172E+00_JPRB, 0.91039E+00_JPRB ,&
&  0.95153E+00_JPRB, 0.12224E+01_JPRB, 0.15756E+01_JPRB, 0.19255E+01_JPRB, 0.23781E+01_JPRB, 0.23509E+01_JPRB, 0.28676E+01_JPRB ,&
&  0.34189E+01_JPRB, 0.24007E+01_JPRB, 0.23799E+01_JPRB, 0.21054E+01_JPRB, 0.21277E+01_JPRB, 0.21611E+01_JPRB, 0.23182E+01_JPRB ,&
&  0.25047E+01_JPRB, 0.26479E+01_JPRB, 0.27770E+01_JPRB, 0.29151E+01_JPRB, 0.30750E+01_JPRB, 0.32167E+01_JPRB, 0.33044E+01_JPRB ,&
&  0.33817E+01_JPRB, 0.32933E+01_JPRB, 0.34013E+01_JPRB, 0.41171E+01_JPRB, 0.47872E+01_JPRB, 0.51505E+01_JPRB, 0.54226E+01_JPRB ,&
&  0.57838E+01_JPRB, 0.60378E+01_JPRB, 0.55838E+01_JPRB, 0.55400E+01_JPRB, 0.58448E+01_JPRB, 0.52798E+01_JPRB, 0.52952E+01_JPRB ,&
&  0.54820E+01_JPRB, 0.47508E+01_JPRB, 0.28915E+01_JPRB, 0.31392E+01_JPRB, 0.32374E+01_JPRB, 0.28922E+01_JPRB, 0.27961E+01_JPRB ,&
&  0.23448E+01_JPRB, 0.20383E+01_JPRB, 0.16959E+01_JPRB, 0.15512E+01_JPRB, 0.15883E+01_JPRB, 0.18030E+01_JPRB, 0.18160E+01_JPRB ,&
&  0.14942E+01_JPRB, 0.14545E+01_JPRB, 0.16924E+01_JPRB, 0.19914E+01_JPRB, 0.21035E+01_JPRB, 0.19194E+01_JPRB, 0.15974E+01_JPRB ,&
&  0.16993E+01_JPRB, 0.17747E+01_JPRB, 0.22015E+01_JPRB, 0.20499E+01_JPRB, 0.10943E+01_JPRB, 0.86551E+00_JPRB, 0.82682E+00_JPRB ,&
&  0.84469E+00_JPRB, 0.84428E+00_JPRB, 0.83889E+00_JPRB, 0.85352E+00_JPRB, 0.86207E+00_JPRB, 0.86439E+00_JPRB, 0.90112E+00_JPRB ,&
&  0.94572E+00_JPRB, 0.95091E+00_JPRB, 0.96079E+00_JPRB, 0.96654E+00_JPRB, 0.97719E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.11094E+01_JPRB, 0.11317E+01_JPRB, 0.11609E+01_JPRB, 0.11697E+01_JPRB, 0.11781E+01_JPRB, 0.11698E+01_JPRB, 0.11608E+01_JPRB ,&
&  0.11417E+01_JPRB, 0.11365E+01_JPRB, 0.11410E+01_JPRB, 0.11472E+01_JPRB, 0.11650E+01_JPRB, 0.11852E+01_JPRB, 0.12257E+01_JPRB ,&
&  0.12300E+01_JPRB, 0.12387E+01_JPRB, 0.12498E+01_JPRB, 0.12725E+01_JPRB, 0.12896E+01_JPRB, 0.12781E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.12101E+01_JPRB, 0.12028E+01_JPRB, 0.12186E+01_JPRB, 0.12584E+01_JPRB, 0.14724E+01_JPRB, 0.18480E+01_JPRB, 0.24854E+01_JPRB ,&
&  0.27690E+01_JPRB, 0.24289E+01_JPRB, 0.21088E+01_JPRB, 0.18320E+01_JPRB, 0.18005E+01_JPRB, 0.18257E+01_JPRB, 0.17387E+01_JPRB ,&
&  0.17277E+01_JPRB, 0.17066E+01_JPRB, 0.17507E+01_JPRB, 0.18400E+01_JPRB, 0.19130E+01_JPRB, 0.18928E+01_JPRB, 0.18568E+01_JPRB ,&
&  0.17034E+01_JPRB, 0.16557E+01_JPRB, 0.18098E+01_JPRB, 0.22171E+01_JPRB, 0.30913E+01_JPRB, 0.36845E+01_JPRB, 0.35129E+01_JPRB ,&
&  0.39340E+01_JPRB, 0.47153E+01_JPRB, 0.49999E+01_JPRB, 0.50472E+01_JPRB, 0.48487E+01_JPRB, 0.45555E+01_JPRB, 0.43735E+01_JPRB ,&
&  0.42897E+01_JPRB, 0.38393E+01_JPRB, 0.29283E+01_JPRB, 0.30871E+01_JPRB, 0.29594E+01_JPRB, 0.29178E+01_JPRB, 0.25621E+01_JPRB ,&
&  0.21718E+01_JPRB, 0.19130E+01_JPRB, 0.17233E+01_JPRB, 0.15469E+01_JPRB, 0.14311E+01_JPRB, 0.14742E+01_JPRB, 0.14054E+01_JPRB ,&
&  0.13077E+01_JPRB, 0.13579E+01_JPRB, 0.14465E+01_JPRB, 0.16140E+01_JPRB, 0.20769E+01_JPRB, 0.19257E+01_JPRB, 0.19926E+01_JPRB ,&
&  0.17903E+01_JPRB, 0.18625E+01_JPRB, 0.15087E+01_JPRB, 0.21931E+01_JPRB, 0.10873E+01_JPRB, 0.79181E+00_JPRB, 0.80024E+00_JPRB ,&
&  0.84533E+00_JPRB, 0.86811E+00_JPRB, 0.90198E+00_JPRB, 0.93534E+00_JPRB, 0.94545E+00_JPRB, 0.95639E+00_JPRB, 0.98812E+00_JPRB ,&
&  0.10238E+01_JPRB, 0.10474E+01_JPRB, 0.10693E+01_JPRB, 0.10762E+01_JPRB, 0.10926E+01_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.11830E+01_JPRB, 0.12064E+01_JPRB, 0.12278E+01_JPRB, 0.12455E+01_JPRB, 0.12545E+01_JPRB, 0.12605E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.12649E+01_JPRB, 0.12662E+01_JPRB, 0.12848E+01_JPRB, 0.13149E+01_JPRB, 0.13313E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.13651E+01_JPRB, 0.13885E+01_JPRB, 0.14222E+01_JPRB, 0.14527E+01_JPRB, 0.14956E+01_JPRB, 0.14984E+01_JPRB, 0.14787E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.14568E+01_JPRB, 0.14649E+01_JPRB, 0.14767E+01_JPRB, 0.15000E+01_JPRB, 0.15430E+01_JPRB, 0.18024E+01_JPRB ,&
&  0.21403E+01_JPRB, 0.25671E+01_JPRB, 0.23459E+01_JPRB, 0.21216E+01_JPRB, 0.21222E+01_JPRB, 0.19198E+01_JPRB, 0.18030E+01_JPRB ,&
&  0.16328E+01_JPRB, 0.14759E+01_JPRB, 0.14346E+01_JPRB, 0.14332E+01_JPRB, 0.15127E+01_JPRB, 0.16009E+01_JPRB, 0.16767E+01_JPRB ,&
&  0.18106E+01_JPRB, 0.19289E+01_JPRB, 0.20837E+01_JPRB, 0.21957E+01_JPRB, 0.22941E+01_JPRB, 0.26136E+01_JPRB, 0.24886E+01_JPRB ,&
&  0.25981E+01_JPRB, 0.29463E+01_JPRB, 0.31800E+01_JPRB, 0.33524E+01_JPRB, 0.31491E+01_JPRB, 0.29176E+01_JPRB, 0.26064E+01_JPRB ,&
&  0.22989E+01_JPRB, 0.22382E+01_JPRB, 0.25043E+01_JPRB, 0.24907E+01_JPRB, 0.24668E+01_JPRB, 0.24877E+01_JPRB, 0.22310E+01_JPRB ,&
&  0.19875E+01_JPRB, 0.18114E+01_JPRB, 0.16420E+01_JPRB, 0.14913E+01_JPRB, 0.14012E+01_JPRB, 0.13799E+01_JPRB, 0.13280E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.12880E+01_JPRB, 0.13490E+01_JPRB, 0.14690E+01_JPRB, 0.16515E+01_JPRB, 0.27996E+01_JPRB, 0.33746E+01_JPRB ,&
&  0.21303E+01_JPRB, 0.17433E+01_JPRB, 0.10480E+01_JPRB, 0.15269E+01_JPRB, 0.93144E+00_JPRB, 0.76830E+00_JPRB, 0.82060E+00_JPRB ,&
&  0.92449E+00_JPRB, 0.10064E+01_JPRB, 0.10501E+01_JPRB, 0.11022E+01_JPRB, 0.10254E+01_JPRB, 0.10212E+01_JPRB, 0.10305E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10836E+01_JPRB, 0.11102E+01_JPRB, 0.11326E+01_JPRB, 0.11587E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.12141E+01_JPRB, 0.12381E+01_JPRB, 0.12616E+01_JPRB, 0.12885E+01_JPRB, 0.13120E+01_JPRB, 0.13396E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.13715E+01_JPRB, 0.13825E+01_JPRB, 0.13878E+01_JPRB, 0.14046E+01_JPRB, 0.14241E+01_JPRB, 0.14589E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.15503E+01_JPRB, 0.16061E+01_JPRB, 0.16701E+01_JPRB, 0.17167E+01_JPRB, 0.17582E+01_JPRB, 0.17967E+01_JPRB, 0.18318E+01_JPRB ,&
&  0.18538E+01_JPRB, 0.18513E+01_JPRB, 0.18821E+01_JPRB, 0.19459E+01_JPRB, 0.20162E+01_JPRB, 0.21545E+01_JPRB, 0.19878E+01_JPRB ,&
&  0.24830E+01_JPRB, 0.26924E+01_JPRB, 0.26854E+01_JPRB, 0.25260E+01_JPRB, 0.25101E+01_JPRB, 0.25045E+01_JPRB, 0.25472E+01_JPRB ,&
&  0.26364E+01_JPRB, 0.25391E+01_JPRB, 0.24169E+01_JPRB, 0.22165E+01_JPRB, 0.21073E+01_JPRB, 0.21552E+01_JPRB, 0.22948E+01_JPRB ,&
&  0.23785E+01_JPRB, 0.24796E+01_JPRB, 0.25137E+01_JPRB, 0.25246E+01_JPRB, 0.25472E+01_JPRB, 0.25554E+01_JPRB, 0.24125E+01_JPRB ,&
&  0.23008E+01_JPRB, 0.22668E+01_JPRB, 0.19729E+01_JPRB, 0.20049E+01_JPRB, 0.20074E+01_JPRB, 0.18384E+01_JPRB, 0.15709E+01_JPRB ,&
&  0.15041E+01_JPRB, 0.14173E+01_JPRB, 0.15086E+01_JPRB, 0.19228E+01_JPRB, 0.20766E+01_JPRB, 0.20716E+01_JPRB, 0.19500E+01_JPRB ,&
&  0.17519E+01_JPRB, 0.15545E+01_JPRB, 0.13450E+01_JPRB, 0.12278E+01_JPRB, 0.12071E+01_JPRB, 0.12570E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.12702E+01_JPRB, 0.12585E+01_JPRB, 0.12746E+01_JPRB, 0.13239E+01_JPRB, 0.15863E+01_JPRB, 0.20675E+01_JPRB, 0.31364E+01_JPRB ,&
&  0.22369E+01_JPRB, 0.13054E+01_JPRB, 0.12326E+01_JPRB, 0.12557E+01_JPRB, 0.89559E+00_JPRB, 0.72914E+00_JPRB, 0.81142E+00_JPRB ,&
&  0.10308E+01_JPRB, 0.11653E+01_JPRB, 0.11690E+01_JPRB, 0.11593E+01_JPRB, 0.11174E+01_JPRB, 0.11292E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.11334E+01_JPRB, 0.11495E+01_JPRB, 0.11665E+01_JPRB, 0.11894E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.12257E+01_JPRB, 0.12560E+01_JPRB, 0.12845E+01_JPRB, 0.13125E+01_JPRB, 0.13356E+01_JPRB, 0.13662E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.13997E+01_JPRB, 0.14136E+01_JPRB, 0.14386E+01_JPRB, 0.14670E+01_JPRB, 0.15021E+01_JPRB, 0.15442E+01_JPRB, 0.15951E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.17177E+01_JPRB, 0.17837E+01_JPRB, 0.18375E+01_JPRB, 0.18670E+01_JPRB, 0.19095E+01_JPRB, 0.19531E+01_JPRB ,&
&  0.19691E+01_JPRB, 0.19678E+01_JPRB, 0.20108E+01_JPRB, 0.20783E+01_JPRB, 0.21130E+01_JPRB, 0.22289E+01_JPRB, 0.18680E+01_JPRB ,&
&  0.26360E+01_JPRB, 0.29601E+01_JPRB, 0.31007E+01_JPRB, 0.29564E+01_JPRB, 0.28993E+01_JPRB, 0.30017E+01_JPRB, 0.31284E+01_JPRB ,&
&  0.32898E+01_JPRB, 0.34348E+01_JPRB, 0.33561E+01_JPRB, 0.31707E+01_JPRB, 0.29711E+01_JPRB, 0.30047E+01_JPRB, 0.30784E+01_JPRB ,&
&  0.31410E+01_JPRB, 0.31894E+01_JPRB, 0.31822E+01_JPRB, 0.31669E+01_JPRB, 0.31615E+01_JPRB, 0.31242E+01_JPRB, 0.30268E+01_JPRB ,&
&  0.29258E+01_JPRB, 0.27994E+01_JPRB, 0.24533E+01_JPRB, 0.26785E+01_JPRB, 0.25020E+01_JPRB, 0.20826E+01_JPRB, 0.16690E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.13769E+01_JPRB, 0.15929E+01_JPRB, 0.16617E+01_JPRB, 0.16435E+01_JPRB, 0.16227E+01_JPRB, 0.15430E+01_JPRB ,&
&  0.13566E+01_JPRB, 0.11875E+01_JPRB, 0.10877E+01_JPRB, 0.10689E+01_JPRB, 0.10985E+01_JPRB, 0.11480E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.11810E+01_JPRB, 0.11865E+01_JPRB, 0.11820E+01_JPRB, 0.11765E+01_JPRB, 0.12933E+01_JPRB, 0.15376E+01_JPRB, 0.25521E+01_JPRB ,&
&  0.19791E+01_JPRB, 0.14278E+01_JPRB, 0.18976E+01_JPRB, 0.74557E+00_JPRB, 0.77199E+00_JPRB, 0.67986E+00_JPRB, 0.75660E+00_JPRB ,&
&  0.89905E+00_JPRB, 0.12462E+01_JPRB, 0.16561E+01_JPRB, 0.14579E+01_JPRB, 0.13483E+01_JPRB, 0.12544E+01_JPRB, 0.11739E+01_JPRB ,&
&  0.11507E+01_JPRB, 0.11528E+01_JPRB, 0.11634E+01_JPRB, 0.11725E+01_JPRB, 0.11955E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.12447E+01_JPRB, 0.12795E+01_JPRB, 0.12958E+01_JPRB, 0.13263E+01_JPRB, 0.13172E+01_JPRB, 0.13441E+01_JPRB, 0.13547E+01_JPRB ,&
&  0.13843E+01_JPRB, 0.14100E+01_JPRB, 0.14451E+01_JPRB, 0.14819E+01_JPRB, 0.15298E+01_JPRB, 0.15769E+01_JPRB, 0.16286E+01_JPRB ,&
&  0.16888E+01_JPRB, 0.17573E+01_JPRB, 0.18154E+01_JPRB, 0.18928E+01_JPRB, 0.19311E+01_JPRB, 0.19980E+01_JPRB, 0.20608E+01_JPRB ,&
&  0.21602E+01_JPRB, 0.21817E+01_JPRB, 0.22154E+01_JPRB, 0.22621E+01_JPRB, 0.22788E+01_JPRB, 0.24028E+01_JPRB, 0.18203E+01_JPRB ,&
&  0.27917E+01_JPRB, 0.31592E+01_JPRB, 0.34211E+01_JPRB, 0.33395E+01_JPRB, 0.32214E+01_JPRB, 0.32277E+01_JPRB, 0.32916E+01_JPRB ,&
&  0.34380E+01_JPRB, 0.35594E+01_JPRB, 0.35397E+01_JPRB, 0.37640E+01_JPRB, 0.38693E+01_JPRB, 0.38028E+01_JPRB, 0.39352E+01_JPRB ,&
&  0.40415E+01_JPRB, 0.41361E+01_JPRB, 0.42030E+01_JPRB, 0.42789E+01_JPRB, 0.43530E+01_JPRB, 0.44786E+01_JPRB, 0.45645E+01_JPRB ,&
&  0.45444E+01_JPRB, 0.45434E+01_JPRB, 0.45040E+01_JPRB, 0.42686E+01_JPRB, 0.40148E+01_JPRB, 0.33512E+01_JPRB, 0.27033E+01_JPRB ,&
&  0.18272E+01_JPRB, 0.14827E+01_JPRB, 0.14993E+01_JPRB, 0.15037E+01_JPRB, 0.14098E+01_JPRB, 0.13426E+01_JPRB, 0.12125E+01_JPRB ,&
&  0.10585E+01_JPRB, 0.10131E+01_JPRB, 0.99678E+00_JPRB, 0.10207E+01_JPRB, 0.10646E+01_JPRB, 0.10924E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.11273E+01_JPRB, 0.10855E+01_JPRB, 0.10543E+01_JPRB, 0.11424E+01_JPRB, 0.14171E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.18352E+01_JPRB, 0.13766E+01_JPRB, 0.11849E+01_JPRB, 0.10605E+01_JPRB, 0.78293E+00_JPRB, 0.72296E+00_JPRB, 0.87213E+00_JPRB ,&
&  0.13924E+01_JPRB, 0.17465E+01_JPRB, 0.24363E+01_JPRB, 0.40833E+01_JPRB, 0.29068E+01_JPRB, 0.22023E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.11622E+01_JPRB, 0.11651E+01_JPRB, 0.11687E+01_JPRB, 0.12040E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.11210E+01_JPRB, 0.11630E+01_JPRB, 0.12072E+01_JPRB, 0.12512E+01_JPRB, 0.12812E+01_JPRB, 0.12980E+01_JPRB, 0.13078E+01_JPRB ,&
&  0.13411E+01_JPRB, 0.13377E+01_JPRB, 0.13728E+01_JPRB, 0.13690E+01_JPRB, 0.14035E+01_JPRB, 0.14201E+01_JPRB, 0.14630E+01_JPRB ,&
&  0.15223E+01_JPRB, 0.15941E+01_JPRB, 0.16769E+01_JPRB, 0.17467E+01_JPRB, 0.18146E+01_JPRB, 0.19283E+01_JPRB, 0.20703E+01_JPRB ,&
&  0.22338E+01_JPRB, 0.23425E+01_JPRB, 0.24374E+01_JPRB, 0.25090E+01_JPRB, 0.24927E+01_JPRB, 0.24699E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.27109E+01_JPRB, 0.32582E+01_JPRB, 0.37014E+01_JPRB, 0.37531E+01_JPRB, 0.35667E+01_JPRB, 0.33887E+01_JPRB, 0.34326E+01_JPRB ,&
&  0.35048E+01_JPRB, 0.34973E+01_JPRB, 0.34677E+01_JPRB, 0.35726E+01_JPRB, 0.40653E+01_JPRB, 0.40001E+01_JPRB, 0.41391E+01_JPRB ,&
&  0.42870E+01_JPRB, 0.44592E+01_JPRB, 0.46783E+01_JPRB, 0.49346E+01_JPRB, 0.51922E+01_JPRB, 0.54381E+01_JPRB, 0.56975E+01_JPRB ,&
&  0.59074E+01_JPRB, 0.59932E+01_JPRB, 0.60487E+01_JPRB, 0.53713E+01_JPRB, 0.49339E+01_JPRB, 0.49644E+01_JPRB, 0.36324E+01_JPRB ,&
&  0.20436E+01_JPRB, 0.15949E+01_JPRB, 0.15623E+01_JPRB, 0.14982E+01_JPRB, 0.13982E+01_JPRB, 0.12414E+01_JPRB, 0.11109E+01_JPRB ,&
&  0.10142E+01_JPRB, 0.98684E+00_JPRB, 0.98490E+00_JPRB, 0.10288E+01_JPRB, 0.10640E+01_JPRB, 0.10948E+01_JPRB, 0.11025E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.11030E+01_JPRB, 0.10574E+01_JPRB, 0.10450E+01_JPRB, 0.11497E+01_JPRB, 0.14410E+01_JPRB, 0.19003E+01_JPRB ,&
&  0.20534E+01_JPRB, 0.12056E+01_JPRB, 0.11121E+01_JPRB, 0.94815E+00_JPRB, 0.86138E+00_JPRB, 0.87361E+00_JPRB, 0.10408E+01_JPRB ,&
&  0.15166E+01_JPRB, 0.22933E+01_JPRB, 0.25066E+01_JPRB, 0.21378E+01_JPRB, 0.25876E+01_JPRB, 0.18997E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.10933E+01_JPRB, 0.10642E+01_JPRB, 0.10610E+01_JPRB, 0.10791E+01_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.93019E+00_JPRB, 0.96010E+00_JPRB, 0.99289E+00_JPRB, 0.10395E+01_JPRB, 0.10925E+01_JPRB, 0.11321E+01_JPRB, 0.11733E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.12254E+01_JPRB, 0.12597E+01_JPRB, 0.12494E+01_JPRB, 0.12813E+01_JPRB, 0.12955E+01_JPRB, 0.13189E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.14013E+01_JPRB, 0.14644E+01_JPRB, 0.15426E+01_JPRB, 0.16521E+01_JPRB, 0.17812E+01_JPRB, 0.19344E+01_JPRB ,&
&  0.21204E+01_JPRB, 0.22941E+01_JPRB, 0.24699E+01_JPRB, 0.25986E+01_JPRB, 0.27031E+01_JPRB, 0.27685E+01_JPRB, 0.28019E+01_JPRB ,&
&  0.17159E+01_JPRB, 0.29798E+01_JPRB, 0.34016E+01_JPRB, 0.36789E+01_JPRB, 0.38796E+01_JPRB, 0.37104E+01_JPRB, 0.37449E+01_JPRB ,&
&  0.35855E+01_JPRB, 0.33284E+01_JPRB, 0.33471E+01_JPRB, 0.34545E+01_JPRB, 0.37026E+01_JPRB, 0.36759E+01_JPRB, 0.37348E+01_JPRB ,&
&  0.38254E+01_JPRB, 0.40021E+01_JPRB, 0.42454E+01_JPRB, 0.45286E+01_JPRB, 0.48677E+01_JPRB, 0.52680E+01_JPRB, 0.57424E+01_JPRB ,&
&  0.62465E+01_JPRB, 0.66989E+01_JPRB, 0.71045E+01_JPRB, 0.62895E+01_JPRB, 0.55568E+01_JPRB, 0.57160E+01_JPRB, 0.47055E+01_JPRB ,&
&  0.25678E+01_JPRB, 0.16434E+01_JPRB, 0.16392E+01_JPRB, 0.17095E+01_JPRB, 0.15488E+01_JPRB, 0.14075E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.10567E+01_JPRB, 0.10976E+01_JPRB, 0.10861E+01_JPRB, 0.11361E+01_JPRB, 0.11471E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.11918E+01_JPRB, 0.11719E+01_JPRB, 0.11637E+01_JPRB, 0.11494E+01_JPRB, 0.11396E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11508E+01_JPRB, 0.10873E+01_JPRB, 0.10326E+01_JPRB, 0.10242E+01_JPRB, 0.10032E+01_JPRB, 0.99121E+00_JPRB, 0.10735E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.13968E+01_JPRB, 0.16829E+01_JPRB, 0.17518E+01_JPRB, 0.16519E+01_JPRB, 0.12744E+01_JPRB, 0.97188E+00_JPRB ,&
&  0.97514E+00_JPRB, 0.93773E+00_JPRB, 0.90822E+00_JPRB, 0.89338E+00_JPRB, 0.90418E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.83900E+00_JPRB, 0.86405E+00_JPRB, 0.86110E+00_JPRB, 0.89429E+00_JPRB, 0.91812E+00_JPRB, 0.95271E+00_JPRB, 0.99421E+00_JPRB ,&
&  0.10289E+01_JPRB, 0.10600E+01_JPRB, 0.11081E+01_JPRB, 0.11138E+01_JPRB, 0.11524E+01_JPRB, 0.11762E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.12099E+01_JPRB, 0.12450E+01_JPRB, 0.12797E+01_JPRB, 0.13209E+01_JPRB, 0.13853E+01_JPRB, 0.14919E+01_JPRB, 0.16116E+01_JPRB ,&
&  0.17595E+01_JPRB, 0.19413E+01_JPRB, 0.21805E+01_JPRB, 0.24210E+01_JPRB, 0.26444E+01_JPRB, 0.27610E+01_JPRB, 0.27553E+01_JPRB ,&
&  0.13300E+01_JPRB, 0.17268E+01_JPRB, 0.28587E+01_JPRB, 0.33072E+01_JPRB, 0.35892E+01_JPRB, 0.37211E+01_JPRB, 0.37182E+01_JPRB ,&
&  0.34494E+01_JPRB, 0.30281E+01_JPRB, 0.29863E+01_JPRB, 0.33890E+01_JPRB, 0.33142E+01_JPRB, 0.33176E+01_JPRB, 0.33262E+01_JPRB ,&
&  0.33650E+01_JPRB, 0.34604E+01_JPRB, 0.36205E+01_JPRB, 0.38376E+01_JPRB, 0.41773E+01_JPRB, 0.46226E+01_JPRB, 0.51984E+01_JPRB ,&
&  0.59003E+01_JPRB, 0.66304E+01_JPRB, 0.72828E+01_JPRB, 0.64324E+01_JPRB, 0.62924E+01_JPRB, 0.62695E+01_JPRB, 0.65074E+01_JPRB ,&
&  0.33571E+01_JPRB, 0.18618E+01_JPRB, 0.18903E+01_JPRB, 0.19847E+01_JPRB, 0.17456E+01_JPRB, 0.16411E+01_JPRB, 0.15346E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.12842E+01_JPRB, 0.12616E+01_JPRB, 0.12307E+01_JPRB, 0.12276E+01_JPRB, 0.12545E+01_JPRB, 0.13014E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.13476E+01_JPRB, 0.13504E+01_JPRB, 0.13447E+01_JPRB, 0.12991E+01_JPRB, 0.12494E+01_JPRB, 0.12226E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.11809E+01_JPRB, 0.11727E+01_JPRB, 0.12013E+01_JPRB, 0.12202E+01_JPRB, 0.12174E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11807E+01_JPRB, 0.11757E+01_JPRB, 0.12755E+01_JPRB, 0.12982E+01_JPRB, 0.12562E+01_JPRB, 0.10652E+01_JPRB, 0.89731E+00_JPRB ,&
&  0.88137E+00_JPRB, 0.85611E+00_JPRB, 0.83370E+00_JPRB, 0.82999E+00_JPRB, 0.82820E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.77343E+00_JPRB, 0.75982E+00_JPRB, 0.74347E+00_JPRB, 0.75546E+00_JPRB, 0.76865E+00_JPRB, 0.78766E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.89870E+00_JPRB, 0.91740E+00_JPRB, 0.94637E+00_JPRB, 0.96136E+00_JPRB, 0.99122E+00_JPRB, 0.10065E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.10836E+01_JPRB, 0.11232E+01_JPRB, 0.11597E+01_JPRB, 0.11890E+01_JPRB, 0.12321E+01_JPRB, 0.12963E+01_JPRB ,&
&  0.13664E+01_JPRB, 0.14593E+01_JPRB, 0.16127E+01_JPRB, 0.18750E+01_JPRB, 0.22843E+01_JPRB, 0.27724E+01_JPRB, 0.31557E+01_JPRB ,&
&  0.24522E+01_JPRB, 0.10022E+01_JPRB, 0.20982E+01_JPRB, 0.29943E+01_JPRB, 0.31511E+01_JPRB, 0.33341E+01_JPRB, 0.32370E+01_JPRB ,&
&  0.29727E+01_JPRB, 0.27289E+01_JPRB, 0.25968E+01_JPRB, 0.29809E+01_JPRB, 0.30204E+01_JPRB, 0.30298E+01_JPRB, 0.30247E+01_JPRB ,&
&  0.30064E+01_JPRB, 0.30315E+01_JPRB, 0.30895E+01_JPRB, 0.32125E+01_JPRB, 0.34632E+01_JPRB, 0.38685E+01_JPRB, 0.44100E+01_JPRB ,&
&  0.51451E+01_JPRB, 0.61518E+01_JPRB, 0.71744E+01_JPRB, 0.66641E+01_JPRB, 0.68886E+01_JPRB, 0.61089E+01_JPRB, 0.52282E+01_JPRB ,&
&  0.30710E+01_JPRB, 0.20914E+01_JPRB, 0.20861E+01_JPRB, 0.22539E+01_JPRB, 0.20058E+01_JPRB, 0.17460E+01_JPRB, 0.18150E+01_JPRB ,&
&  0.16506E+01_JPRB, 0.15497E+01_JPRB, 0.14801E+01_JPRB, 0.14276E+01_JPRB, 0.14094E+01_JPRB, 0.14034E+01_JPRB, 0.14016E+01_JPRB ,&
&  0.14007E+01_JPRB, 0.14240E+01_JPRB, 0.14529E+01_JPRB, 0.14496E+01_JPRB, 0.14155E+01_JPRB, 0.13862E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.13850E+01_JPRB, 0.13714E+01_JPRB, 0.13809E+01_JPRB, 0.13945E+01_JPRB, 0.13909E+01_JPRB, 0.13432E+01_JPRB, 0.13317E+01_JPRB ,&
&  0.13015E+01_JPRB, 0.12807E+01_JPRB, 0.12337E+01_JPRB, 0.11642E+01_JPRB, 0.11119E+01_JPRB, 0.10178E+01_JPRB, 0.91412E+00_JPRB ,&
&  0.87908E+00_JPRB, 0.84155E+00_JPRB, 0.81809E+00_JPRB, 0.80399E+00_JPRB, 0.79017E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.84335E+00_JPRB, 0.82466E+00_JPRB, 0.78900E+00_JPRB, 0.73675E+00_JPRB, 0.72963E+00_JPRB, 0.72627E+00_JPRB, 0.73697E+00_JPRB ,&
&  0.78722E+00_JPRB, 0.84718E+00_JPRB, 0.86490E+00_JPRB, 0.86701E+00_JPRB, 0.84462E+00_JPRB, 0.85446E+00_JPRB, 0.87363E+00_JPRB ,&
&  0.90393E+00_JPRB, 0.93381E+00_JPRB, 0.97407E+00_JPRB, 0.10239E+01_JPRB, 0.10653E+01_JPRB, 0.10988E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.11623E+01_JPRB, 0.11962E+01_JPRB, 0.12640E+01_JPRB, 0.14200E+01_JPRB, 0.17673E+01_JPRB, 0.23740E+01_JPRB ,&
&  0.37494E+01_JPRB, 0.31426E+01_JPRB, 0.14029E+01_JPRB, 0.25800E+01_JPRB, 0.29847E+01_JPRB, 0.32667E+01_JPRB, 0.27910E+01_JPRB ,&
&  0.26822E+01_JPRB, 0.23722E+01_JPRB, 0.21599E+01_JPRB, 0.24018E+01_JPRB, 0.24705E+01_JPRB, 0.25305E+01_JPRB, 0.25413E+01_JPRB ,&
&  0.24721E+01_JPRB, 0.24948E+01_JPRB, 0.25429E+01_JPRB, 0.26088E+01_JPRB, 0.27295E+01_JPRB, 0.30203E+01_JPRB, 0.35553E+01_JPRB ,&
&  0.42480E+01_JPRB, 0.51973E+01_JPRB, 0.66424E+01_JPRB, 0.73206E+01_JPRB, 0.80324E+01_JPRB, 0.59669E+01_JPRB, 0.45595E+01_JPRB ,&
&  0.28924E+01_JPRB, 0.25475E+01_JPRB, 0.26185E+01_JPRB, 0.25802E+01_JPRB, 0.23229E+01_JPRB, 0.21055E+01_JPRB, 0.21673E+01_JPRB ,&
&  0.19511E+01_JPRB, 0.18081E+01_JPRB, 0.17292E+01_JPRB, 0.16560E+01_JPRB, 0.16039E+01_JPRB, 0.15741E+01_JPRB, 0.15486E+01_JPRB ,&
&  0.15231E+01_JPRB, 0.15293E+01_JPRB, 0.15456E+01_JPRB, 0.15595E+01_JPRB, 0.15563E+01_JPRB, 0.15315E+01_JPRB, 0.15251E+01_JPRB ,&
&  0.15188E+01_JPRB, 0.15197E+01_JPRB, 0.15206E+01_JPRB, 0.15332E+01_JPRB, 0.14794E+01_JPRB, 0.14060E+01_JPRB, 0.14070E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.13855E+01_JPRB, 0.13386E+01_JPRB, 0.13129E+01_JPRB, 0.12050E+01_JPRB, 0.10871E+01_JPRB, 0.98062E+00_JPRB ,&
&  0.95304E+00_JPRB, 0.93146E+00_JPRB, 0.88408E+00_JPRB, 0.84072E+00_JPRB, 0.83511E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.93877E+00_JPRB, 0.94991E+00_JPRB, 0.94344E+00_JPRB, 0.88889E+00_JPRB, 0.83876E+00_JPRB, 0.79020E+00_JPRB, 0.75807E+00_JPRB ,&
&  0.74947E+00_JPRB, 0.75091E+00_JPRB, 0.78061E+00_JPRB, 0.80949E+00_JPRB, 0.80321E+00_JPRB, 0.77709E+00_JPRB, 0.76977E+00_JPRB ,&
&  0.78247E+00_JPRB, 0.81533E+00_JPRB, 0.84782E+00_JPRB, 0.89786E+00_JPRB, 0.94249E+00_JPRB, 0.97412E+00_JPRB, 0.10066E+01_JPRB ,&
&  0.10038E+01_JPRB, 0.10051E+01_JPRB, 0.10152E+01_JPRB, 0.10323E+01_JPRB, 0.10695E+01_JPRB, 0.11693E+01_JPRB, 0.13274E+01_JPRB ,&
&  0.18414E+01_JPRB, 0.32496E+01_JPRB, 0.18092E+01_JPRB, 0.25922E+01_JPRB, 0.31323E+01_JPRB, 0.32173E+01_JPRB, 0.28443E+01_JPRB ,&
&  0.25761E+01_JPRB, 0.22066E+01_JPRB, 0.20370E+01_JPRB, 0.20445E+01_JPRB, 0.18899E+01_JPRB, 0.19062E+01_JPRB, 0.19257E+01_JPRB ,&
&  0.19501E+01_JPRB, 0.20037E+01_JPRB, 0.20525E+01_JPRB, 0.20947E+01_JPRB, 0.21157E+01_JPRB, 0.22047E+01_JPRB, 0.25113E+01_JPRB ,&
&  0.31185E+01_JPRB, 0.40092E+01_JPRB, 0.55415E+01_JPRB, 0.69697E+01_JPRB, 0.85783E+01_JPRB, 0.66257E+01_JPRB, 0.48312E+01_JPRB ,&
&  0.34612E+01_JPRB, 0.32261E+01_JPRB, 0.32015E+01_JPRB, 0.31307E+01_JPRB, 0.27879E+01_JPRB, 0.27554E+01_JPRB, 0.27889E+01_JPRB ,&
&  0.24492E+01_JPRB, 0.21746E+01_JPRB, 0.20315E+01_JPRB, 0.19339E+01_JPRB, 0.18337E+01_JPRB, 0.17583E+01_JPRB, 0.17079E+01_JPRB ,&
&  0.16689E+01_JPRB, 0.16505E+01_JPRB, 0.16640E+01_JPRB, 0.16796E+01_JPRB, 0.16942E+01_JPRB, 0.16833E+01_JPRB, 0.16617E+01_JPRB ,&
&  0.16179E+01_JPRB, 0.15913E+01_JPRB, 0.15765E+01_JPRB, 0.15636E+01_JPRB, 0.15192E+01_JPRB, 0.14620E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.14439E+01_JPRB, 0.14630E+01_JPRB, 0.14399E+01_JPRB, 0.13780E+01_JPRB, 0.13664E+01_JPRB, 0.12569E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.12545E+01_JPRB, 0.12853E+01_JPRB, 0.93893E+00_JPRB, 0.92961E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.10178E+01_JPRB, 0.10161E+01_JPRB, 0.10007E+01_JPRB, 0.99492E+00_JPRB, 0.95706E+00_JPRB, 0.90651E+00_JPRB, 0.85679E+00_JPRB ,&
&  0.82035E+00_JPRB, 0.79553E+00_JPRB, 0.78708E+00_JPRB, 0.78026E+00_JPRB, 0.77986E+00_JPRB, 0.77154E+00_JPRB, 0.76776E+00_JPRB ,&
&  0.75770E+00_JPRB, 0.76726E+00_JPRB, 0.78852E+00_JPRB, 0.82661E+00_JPRB, 0.87112E+00_JPRB, 0.88499E+00_JPRB, 0.89721E+00_JPRB ,&
&  0.90548E+00_JPRB, 0.90510E+00_JPRB, 0.90644E+00_JPRB, 0.91726E+00_JPRB, 0.94085E+00_JPRB, 0.97874E+00_JPRB, 0.10518E+01_JPRB ,&
&  0.12634E+01_JPRB, 0.26940E+01_JPRB, 0.40657E+01_JPRB, 0.46031E+01_JPRB, 0.40825E+01_JPRB, 0.34890E+01_JPRB, 0.31252E+01_JPRB ,&
&  0.30201E+01_JPRB, 0.27373E+01_JPRB, 0.24680E+01_JPRB, 0.20483E+01_JPRB, 0.17137E+01_JPRB, 0.15997E+01_JPRB, 0.15997E+01_JPRB ,&
&  0.16895E+01_JPRB, 0.17371E+01_JPRB, 0.17559E+01_JPRB, 0.17425E+01_JPRB, 0.16980E+01_JPRB, 0.16851E+01_JPRB, 0.17630E+01_JPRB ,&
&  0.20367E+01_JPRB, 0.26453E+01_JPRB, 0.37832E+01_JPRB, 0.57670E+01_JPRB, 0.79068E+01_JPRB, 0.70577E+01_JPRB, 0.56399E+01_JPRB ,&
&  0.51395E+01_JPRB, 0.46164E+01_JPRB, 0.41621E+01_JPRB, 0.39146E+01_JPRB, 0.31206E+01_JPRB, 0.37617E+01_JPRB, 0.35225E+01_JPRB ,&
&  0.32217E+01_JPRB, 0.30150E+01_JPRB, 0.27252E+01_JPRB, 0.24135E+01_JPRB, 0.21847E+01_JPRB, 0.19782E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.17707E+01_JPRB, 0.17761E+01_JPRB, 0.17997E+01_JPRB, 0.18428E+01_JPRB, 0.18897E+01_JPRB, 0.18990E+01_JPRB, 0.18509E+01_JPRB ,&
&  0.17449E+01_JPRB, 0.16510E+01_JPRB, 0.15257E+01_JPRB, 0.15036E+01_JPRB, 0.14835E+01_JPRB, 0.14299E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14642E+01_JPRB, 0.14728E+01_JPRB, 0.14405E+01_JPRB, 0.13991E+01_JPRB, 0.13630E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.11857E+01_JPRB, 0.12154E+01_JPRB, 0.11415E+01_JPRB, 0.10656E+01_JPRB, 0.10412E+01_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.10890E+01_JPRB, 0.10496E+01_JPRB, 0.10252E+01_JPRB, 0.10198E+01_JPRB, 0.99748E+00_JPRB, 0.97968E+00_JPRB, 0.96681E+00_JPRB ,&
&  0.91114E+00_JPRB, 0.86608E+00_JPRB, 0.82401E+00_JPRB, 0.80264E+00_JPRB, 0.79704E+00_JPRB, 0.79047E+00_JPRB, 0.79824E+00_JPRB ,&
&  0.79372E+00_JPRB, 0.77944E+00_JPRB, 0.77901E+00_JPRB, 0.80075E+00_JPRB, 0.83235E+00_JPRB, 0.84164E+00_JPRB, 0.85854E+00_JPRB ,&
&  0.87724E+00_JPRB, 0.87663E+00_JPRB, 0.86652E+00_JPRB, 0.86500E+00_JPRB, 0.87083E+00_JPRB, 0.87488E+00_JPRB, 0.91480E+00_JPRB ,&
&  0.11124E+01_JPRB, 0.27369E+01_JPRB, 0.27815E+01_JPRB, 0.52439E+01_JPRB, 0.45883E+01_JPRB, 0.39578E+01_JPRB, 0.31300E+01_JPRB ,&
&  0.31605E+01_JPRB, 0.35233E+01_JPRB, 0.27825E+01_JPRB, 0.22776E+01_JPRB, 0.18720E+01_JPRB, 0.15858E+01_JPRB, 0.15032E+01_JPRB ,&
&  0.15483E+01_JPRB, 0.16099E+01_JPRB, 0.16332E+01_JPRB, 0.15693E+01_JPRB, 0.14921E+01_JPRB, 0.14582E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.15358E+01_JPRB, 0.17310E+01_JPRB, 0.23021E+01_JPRB, 0.34952E+01_JPRB, 0.61285E+01_JPRB, 0.73341E+01_JPRB, 0.66437E+01_JPRB ,&
&  0.63208E+01_JPRB, 0.64540E+01_JPRB, 0.58357E+01_JPRB, 0.51429E+01_JPRB, 0.42135E+01_JPRB, 0.42062E+01_JPRB, 0.39624E+01_JPRB ,&
&  0.36945E+01_JPRB, 0.36133E+01_JPRB, 0.34798E+01_JPRB, 0.31945E+01_JPRB, 0.28421E+01_JPRB, 0.24808E+01_JPRB, 0.21524E+01_JPRB ,&
&  0.19234E+01_JPRB, 0.18213E+01_JPRB, 0.17845E+01_JPRB, 0.18433E+01_JPRB, 0.19528E+01_JPRB, 0.20273E+01_JPRB, 0.20352E+01_JPRB ,&
&  0.19471E+01_JPRB, 0.18066E+01_JPRB, 0.15748E+01_JPRB, 0.14607E+01_JPRB, 0.14353E+01_JPRB, 0.13812E+01_JPRB, 0.13549E+01_JPRB ,&
&  0.14065E+01_JPRB, 0.14517E+01_JPRB, 0.14506E+01_JPRB, 0.14212E+01_JPRB, 0.14030E+01_JPRB, 0.14241E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12726E+01_JPRB, 0.12617E+01_JPRB, 0.11930E+01_JPRB, 0.11454E+01_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.11672E+01_JPRB, 0.11056E+01_JPRB, 0.10575E+01_JPRB, 0.10348E+01_JPRB, 0.10249E+01_JPRB, 0.10147E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.96838E+00_JPRB, 0.92542E+00_JPRB, 0.88169E+00_JPRB, 0.84501E+00_JPRB, 0.83532E+00_JPRB, 0.82685E+00_JPRB, 0.82553E+00_JPRB ,&
&  0.81880E+00_JPRB, 0.80406E+00_JPRB, 0.79276E+00_JPRB, 0.79885E+00_JPRB, 0.80883E+00_JPRB, 0.82008E+00_JPRB, 0.83931E+00_JPRB ,&
&  0.84626E+00_JPRB, 0.85556E+00_JPRB, 0.85786E+00_JPRB, 0.85319E+00_JPRB, 0.84406E+00_JPRB, 0.83375E+00_JPRB, 0.83726E+00_JPRB ,&
&  0.10855E+01_JPRB, 0.33983E+01_JPRB, 0.25188E+01_JPRB, 0.47948E+01_JPRB, 0.41995E+01_JPRB, 0.38151E+01_JPRB, 0.30551E+01_JPRB ,&
&  0.28959E+01_JPRB, 0.31955E+01_JPRB, 0.27976E+01_JPRB, 0.24123E+01_JPRB, 0.21153E+01_JPRB, 0.18430E+01_JPRB, 0.16544E+01_JPRB ,&
&  0.15878E+01_JPRB, 0.15699E+01_JPRB, 0.15407E+01_JPRB, 0.14809E+01_JPRB, 0.14314E+01_JPRB, 0.13949E+01_JPRB, 0.13910E+01_JPRB ,&
&  0.14011E+01_JPRB, 0.14505E+01_JPRB, 0.16617E+01_JPRB, 0.23949E+01_JPRB, 0.40657E+01_JPRB, 0.62156E+01_JPRB, 0.76365E+01_JPRB ,&
&  0.77693E+01_JPRB, 0.84318E+01_JPRB, 0.75429E+01_JPRB, 0.64309E+01_JPRB, 0.55322E+01_JPRB, 0.50077E+01_JPRB, 0.45339E+01_JPRB ,&
&  0.40484E+01_JPRB, 0.37551E+01_JPRB, 0.36622E+01_JPRB, 0.35961E+01_JPRB, 0.34498E+01_JPRB, 0.31892E+01_JPRB, 0.28498E+01_JPRB ,&
&  0.24917E+01_JPRB, 0.22330E+01_JPRB, 0.20606E+01_JPRB, 0.19880E+01_JPRB, 0.19966E+01_JPRB, 0.20499E+01_JPRB, 0.20723E+01_JPRB ,&
&  0.19903E+01_JPRB, 0.18785E+01_JPRB, 0.16696E+01_JPRB, 0.15115E+01_JPRB, 0.14593E+01_JPRB, 0.13984E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.14119E+01_JPRB, 0.14503E+01_JPRB, 0.14346E+01_JPRB, 0.14098E+01_JPRB, 0.13724E+01_JPRB, 0.14407E+01_JPRB, 0.13583E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.12924E+01_JPRB, 0.12775E+01_JPRB, 0.12485E+01_JPRB, 0.12211E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.12159E+01_JPRB, 0.11708E+01_JPRB, 0.11229E+01_JPRB, 0.10850E+01_JPRB, 0.10551E+01_JPRB, 0.10393E+01_JPRB, 0.10263E+01_JPRB ,&
&  0.99653E+00_JPRB, 0.96250E+00_JPRB, 0.93104E+00_JPRB, 0.90547E+00_JPRB, 0.88588E+00_JPRB, 0.87411E+00_JPRB, 0.85968E+00_JPRB ,&
&  0.84176E+00_JPRB, 0.81243E+00_JPRB, 0.81171E+00_JPRB, 0.82177E+00_JPRB, 0.82270E+00_JPRB, 0.82542E+00_JPRB, 0.84914E+00_JPRB ,&
&  0.84485E+00_JPRB, 0.84381E+00_JPRB, 0.84669E+00_JPRB, 0.84841E+00_JPRB, 0.83704E+00_JPRB, 0.83508E+00_JPRB, 0.82483E+00_JPRB ,&
&  0.10397E+01_JPRB, 0.13671E+01_JPRB, 0.29800E+01_JPRB, 0.41488E+01_JPRB, 0.37901E+01_JPRB, 0.34043E+01_JPRB, 0.30730E+01_JPRB ,&
&  0.29833E+01_JPRB, 0.28778E+01_JPRB, 0.25568E+01_JPRB, 0.23264E+01_JPRB, 0.21092E+01_JPRB, 0.19364E+01_JPRB, 0.18008E+01_JPRB ,&
&  0.16984E+01_JPRB, 0.15988E+01_JPRB, 0.15220E+01_JPRB, 0.14448E+01_JPRB, 0.13882E+01_JPRB, 0.13548E+01_JPRB, 0.13453E+01_JPRB ,&
&  0.13392E+01_JPRB, 0.13389E+01_JPRB, 0.13992E+01_JPRB, 0.17775E+01_JPRB, 0.31560E+01_JPRB, 0.49726E+01_JPRB, 0.69862E+01_JPRB ,&
&  0.84297E+01_JPRB, 0.90692E+01_JPRB, 0.82979E+01_JPRB, 0.73206E+01_JPRB, 0.63380E+01_JPRB, 0.55772E+01_JPRB, 0.49873E+01_JPRB ,&
&  0.44540E+01_JPRB, 0.39905E+01_JPRB, 0.36292E+01_JPRB, 0.34354E+01_JPRB, 0.33220E+01_JPRB, 0.32350E+01_JPRB, 0.31266E+01_JPRB ,&
&  0.29639E+01_JPRB, 0.27262E+01_JPRB, 0.24751E+01_JPRB, 0.22746E+01_JPRB, 0.21568E+01_JPRB, 0.21119E+01_JPRB, 0.20758E+01_JPRB ,&
&  0.19704E+01_JPRB, 0.18569E+01_JPRB, 0.16910E+01_JPRB, 0.15796E+01_JPRB, 0.15528E+01_JPRB, 0.15070E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.14614E+01_JPRB, 0.14391E+01_JPRB, 0.14227E+01_JPRB, 0.13664E+01_JPRB, 0.14407E+01_JPRB, 0.13838E+01_JPRB ,&
&  0.13301E+01_JPRB, 0.13072E+01_JPRB, 0.12758E+01_JPRB, 0.12529E+01_JPRB, 0.12411E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.12487E+01_JPRB, 0.12230E+01_JPRB, 0.11713E+01_JPRB, 0.11211E+01_JPRB, 0.10845E+01_JPRB, 0.10588E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10115E+01_JPRB, 0.98058E+00_JPRB, 0.95232E+00_JPRB, 0.93550E+00_JPRB, 0.92917E+00_JPRB, 0.92159E+00_JPRB, 0.90176E+00_JPRB ,&
&  0.87533E+00_JPRB, 0.85158E+00_JPRB, 0.84786E+00_JPRB, 0.85224E+00_JPRB, 0.85441E+00_JPRB, 0.83917E+00_JPRB, 0.84844E+00_JPRB ,&
&  0.85433E+00_JPRB, 0.85184E+00_JPRB, 0.84875E+00_JPRB, 0.84430E+00_JPRB, 0.83773E+00_JPRB, 0.83219E+00_JPRB, 0.82187E+00_JPRB ,&
&  0.96751E+00_JPRB, 0.18885E+01_JPRB, 0.29598E+01_JPRB, 0.36106E+01_JPRB, 0.35423E+01_JPRB, 0.30864E+01_JPRB, 0.28127E+01_JPRB ,&
&  0.25802E+01_JPRB, 0.24695E+01_JPRB, 0.22441E+01_JPRB, 0.20835E+01_JPRB, 0.19333E+01_JPRB, 0.18043E+01_JPRB, 0.16909E+01_JPRB ,&
&  0.16067E+01_JPRB, 0.15368E+01_JPRB, 0.14672E+01_JPRB, 0.14051E+01_JPRB, 0.13574E+01_JPRB, 0.13190E+01_JPRB, 0.12985E+01_JPRB ,&
&  0.12882E+01_JPRB, 0.12765E+01_JPRB, 0.12869E+01_JPRB, 0.14415E+01_JPRB, 0.23933E+01_JPRB, 0.31013E+01_JPRB, 0.53916E+01_JPRB ,&
&  0.72293E+01_JPRB, 0.69113E+01_JPRB, 0.66234E+01_JPRB, 0.60956E+01_JPRB, 0.54150E+01_JPRB, 0.48372E+01_JPRB, 0.42288E+01_JPRB ,&
&  0.37036E+01_JPRB, 0.32910E+01_JPRB, 0.29545E+01_JPRB, 0.27245E+01_JPRB, 0.25844E+01_JPRB, 0.25150E+01_JPRB, 0.24909E+01_JPRB ,&
&  0.24811E+01_JPRB, 0.24637E+01_JPRB, 0.24196E+01_JPRB, 0.23382E+01_JPRB, 0.22223E+01_JPRB, 0.20996E+01_JPRB, 0.19770E+01_JPRB ,&
&  0.18501E+01_JPRB, 0.17635E+01_JPRB, 0.16511E+01_JPRB, 0.15782E+01_JPRB, 0.15934E+01_JPRB, 0.16272E+01_JPRB, 0.15915E+01_JPRB ,&
&  0.15139E+01_JPRB, 0.14768E+01_JPRB, 0.14614E+01_JPRB, 0.14047E+01_JPRB, 0.13430E+01_JPRB, 0.14392E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.13575E+01_JPRB, 0.13233E+01_JPRB, 0.12931E+01_JPRB, 0.12609E+01_JPRB, 0.12610E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.12724E+01_JPRB, 0.12260E+01_JPRB, 0.11799E+01_JPRB, 0.11331E+01_JPRB, 0.10987E+01_JPRB, 0.10627E+01_JPRB, 0.10280E+01_JPRB ,&
&  0.99996E+00_JPRB, 0.98344E+00_JPRB, 0.96859E+00_JPRB, 0.94830E+00_JPRB, 0.94450E+00_JPRB, 0.93654E+00_JPRB, 0.91786E+00_JPRB ,&
&  0.88884E+00_JPRB, 0.86722E+00_JPRB, 0.85735E+00_JPRB, 0.85406E+00_JPRB, 0.85896E+00_JPRB, 0.85708E+00_JPRB, 0.86145E+00_JPRB ,&
&  0.86779E+00_JPRB, 0.85217E+00_JPRB, 0.83958E+00_JPRB, 0.83625E+00_JPRB, 0.83273E+00_JPRB, 0.83214E+00_JPRB, 0.82914E+00_JPRB ,&
&  0.92926E+00_JPRB, 0.14536E+01_JPRB, 0.20766E+01_JPRB, 0.28806E+01_JPRB, 0.29915E+01_JPRB, 0.27352E+01_JPRB, 0.24913E+01_JPRB ,&
&  0.22919E+01_JPRB, 0.21435E+01_JPRB, 0.19614E+01_JPRB, 0.17513E+01_JPRB, 0.15864E+01_JPRB, 0.14763E+01_JPRB, 0.14005E+01_JPRB ,&
&  0.13579E+01_JPRB, 0.13238E+01_JPRB, 0.12894E+01_JPRB, 0.12574E+01_JPRB, 0.12273E+01_JPRB, 0.12012E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.11693E+01_JPRB, 0.11705E+01_JPRB, 0.12095E+01_JPRB, 0.16030E+01_JPRB, 0.20272E+01_JPRB, 0.30486E+01_JPRB ,&
&  0.35094E+01_JPRB, 0.37410E+01_JPRB, 0.39482E+01_JPRB, 0.39105E+01_JPRB, 0.36693E+01_JPRB, 0.34161E+01_JPRB, 0.31459E+01_JPRB ,&
&  0.28545E+01_JPRB, 0.25668E+01_JPRB, 0.23683E+01_JPRB, 0.22239E+01_JPRB, 0.20902E+01_JPRB, 0.19916E+01_JPRB, 0.19347E+01_JPRB ,&
&  0.19253E+01_JPRB, 0.19628E+01_JPRB, 0.20099E+01_JPRB, 0.20426E+01_JPRB, 0.20237E+01_JPRB, 0.19698E+01_JPRB, 0.18768E+01_JPRB ,&
&  0.17516E+01_JPRB, 0.16495E+01_JPRB, 0.15926E+01_JPRB, 0.15724E+01_JPRB, 0.15801E+01_JPRB, 0.15874E+01_JPRB, 0.15922E+01_JPRB ,&
&  0.15831E+01_JPRB, 0.15427E+01_JPRB, 0.15002E+01_JPRB, 0.14420E+01_JPRB, 0.13858E+01_JPRB, 0.14531E+01_JPRB, 0.14281E+01_JPRB ,&
&  0.13890E+01_JPRB, 0.13477E+01_JPRB, 0.13174E+01_JPRB, 0.12845E+01_JPRB, 0.13064E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.13305E+01_JPRB, 0.12538E+01_JPRB, 0.11937E+01_JPRB, 0.11487E+01_JPRB, 0.11183E+01_JPRB, 0.10811E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.99227E+00_JPRB, 0.97826E+00_JPRB, 0.95326E+00_JPRB, 0.93316E+00_JPRB, 0.91477E+00_JPRB, 0.89853E+00_JPRB ,&
&  0.87456E+00_JPRB, 0.86244E+00_JPRB, 0.85264E+00_JPRB, 0.84913E+00_JPRB, 0.84313E+00_JPRB, 0.84167E+00_JPRB, 0.84892E+00_JPRB ,&
&  0.85246E+00_JPRB, 0.83592E+00_JPRB, 0.83277E+00_JPRB, 0.82856E+00_JPRB, 0.82274E+00_JPRB, 0.81837E+00_JPRB, 0.81467E+00_JPRB ,&
&  0.92057E+00_JPRB, 0.11303E+01_JPRB, 0.15029E+01_JPRB, 0.19077E+01_JPRB, 0.20964E+01_JPRB, 0.21671E+01_JPRB, 0.20877E+01_JPRB ,&
&  0.19623E+01_JPRB, 0.18167E+01_JPRB, 0.16594E+01_JPRB, 0.15106E+01_JPRB, 0.13703E+01_JPRB, 0.12693E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.11427E+01_JPRB, 0.11241E+01_JPRB, 0.11090E+01_JPRB, 0.10940E+01_JPRB, 0.10752E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10652E+01_JPRB, 0.10539E+01_JPRB, 0.10521E+01_JPRB, 0.10728E+01_JPRB, 0.11424E+01_JPRB, 0.13140E+01_JPRB, 0.17733E+01_JPRB ,&
&  0.20754E+01_JPRB, 0.23133E+01_JPRB, 0.25609E+01_JPRB, 0.26591E+01_JPRB, 0.25898E+01_JPRB, 0.24552E+01_JPRB, 0.22691E+01_JPRB ,&
&  0.21198E+01_JPRB, 0.20139E+01_JPRB, 0.19171E+01_JPRB, 0.18212E+01_JPRB, 0.17403E+01_JPRB, 0.16789E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.16080E+01_JPRB, 0.16119E+01_JPRB, 0.16384E+01_JPRB, 0.16699E+01_JPRB, 0.16933E+01_JPRB, 0.16954E+01_JPRB, 0.16744E+01_JPRB ,&
&  0.16117E+01_JPRB, 0.15272E+01_JPRB, 0.14750E+01_JPRB, 0.14753E+01_JPRB, 0.15079E+01_JPRB, 0.15278E+01_JPRB, 0.15363E+01_JPRB ,&
&  0.15429E+01_JPRB, 0.15425E+01_JPRB, 0.14927E+01_JPRB, 0.14460E+01_JPRB, 0.15021E+01_JPRB, 0.14687E+01_JPRB, 0.14268E+01_JPRB ,&
&  0.13816E+01_JPRB, 0.13494E+01_JPRB, 0.13161E+01_JPRB, 0.12791E+01_JPRB, 0.13282E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.11587E+01_JPRB, 0.11457E+01_JPRB, 0.11146E+01_JPRB, 0.10889E+01_JPRB, 0.10749E+01_JPRB, 0.10611E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.98766E+00_JPRB, 0.97053E+00_JPRB, 0.94606E+00_JPRB, 0.92027E+00_JPRB, 0.90220E+00_JPRB, 0.88534E+00_JPRB ,&
&  0.86467E+00_JPRB, 0.85368E+00_JPRB, 0.84826E+00_JPRB, 0.84195E+00_JPRB, 0.83320E+00_JPRB, 0.82603E+00_JPRB, 0.82831E+00_JPRB ,&
&  0.82738E+00_JPRB, 0.82754E+00_JPRB, 0.82914E+00_JPRB, 0.82189E+00_JPRB, 0.81126E+00_JPRB, 0.80542E+00_JPRB, 0.79164E+00_JPRB ,&
&  0.83025E+00_JPRB, 0.88709E+00_JPRB, 0.10667E+01_JPRB, 0.12922E+01_JPRB, 0.14889E+01_JPRB, 0.16310E+01_JPRB, 0.17034E+01_JPRB ,&
&  0.16516E+01_JPRB, 0.15294E+01_JPRB, 0.14076E+01_JPRB, 0.13282E+01_JPRB, 0.12533E+01_JPRB, 0.11910E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.10769E+01_JPRB, 0.10570E+01_JPRB, 0.10381E+01_JPRB, 0.10183E+01_JPRB, 0.10008E+01_JPRB, 0.98767E+00_JPRB ,&
&  0.97546E+00_JPRB, 0.97119E+00_JPRB, 0.97202E+00_JPRB, 0.97021E+00_JPRB, 0.98706E+00_JPRB, 0.10758E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.14158E+01_JPRB, 0.15767E+01_JPRB, 0.17861E+01_JPRB, 0.19089E+01_JPRB, 0.18940E+01_JPRB, 0.18298E+01_JPRB, 0.17394E+01_JPRB ,&
&  0.16617E+01_JPRB, 0.15789E+01_JPRB, 0.15241E+01_JPRB, 0.14699E+01_JPRB, 0.14119E+01_JPRB, 0.13635E+01_JPRB, 0.13357E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.13227E+01_JPRB, 0.13356E+01_JPRB, 0.13567E+01_JPRB, 0.13741E+01_JPRB, 0.13902E+01_JPRB, 0.14062E+01_JPRB ,&
&  0.14149E+01_JPRB, 0.13924E+01_JPRB, 0.13545E+01_JPRB, 0.13538E+01_JPRB, 0.13872E+01_JPRB, 0.14161E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.14519E+01_JPRB, 0.14692E+01_JPRB, 0.14742E+01_JPRB, 0.14785E+01_JPRB, 0.14556E+01_JPRB, 0.14052E+01_JPRB, 0.13557E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.12738E+01_JPRB, 0.12471E+01_JPRB, 0.12102E+01_JPRB, 0.11140E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.10829E+01_JPRB, 0.10633E+01_JPRB, 0.10363E+01_JPRB, 0.10126E+01_JPRB, 0.10009E+01_JPRB, 0.99665E+00_JPRB, 0.99489E+00_JPRB ,&
&  0.98710E+00_JPRB, 0.97514E+00_JPRB, 0.96096E+00_JPRB, 0.94312E+00_JPRB, 0.92262E+00_JPRB, 0.90360E+00_JPRB, 0.88672E+00_JPRB ,&
&  0.86330E+00_JPRB, 0.84782E+00_JPRB, 0.84391E+00_JPRB, 0.83994E+00_JPRB, 0.83617E+00_JPRB, 0.82924E+00_JPRB, 0.82561E+00_JPRB ,&
&  0.82619E+00_JPRB, 0.82754E+00_JPRB, 0.82819E+00_JPRB, 0.81944E+00_JPRB, 0.81040E+00_JPRB, 0.80642E+00_JPRB, 0.79802E+00_JPRB ,&
&  0.80323E+00_JPRB, 0.73473E+00_JPRB, 0.86994E+00_JPRB, 0.10176E+01_JPRB, 0.11139E+01_JPRB, 0.12368E+01_JPRB, 0.13591E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.13386E+01_JPRB, 0.12627E+01_JPRB, 0.11991E+01_JPRB, 0.11416E+01_JPRB, 0.10982E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.10223E+01_JPRB, 0.10032E+01_JPRB, 0.97963E+00_JPRB, 0.95474E+00_JPRB, 0.93724E+00_JPRB, 0.92410E+00_JPRB ,&
&  0.91517E+00_JPRB, 0.91189E+00_JPRB, 0.90900E+00_JPRB, 0.91324E+00_JPRB, 0.91800E+00_JPRB, 0.94788E+00_JPRB, 0.10404E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11984E+01_JPRB, 0.13250E+01_JPRB, 0.13881E+01_JPRB, 0.13915E+01_JPRB, 0.13747E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12376E+01_JPRB, 0.11974E+01_JPRB, 0.11587E+01_JPRB, 0.11307E+01_JPRB, 0.11154E+01_JPRB, 0.11172E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.11197E+01_JPRB, 0.11218E+01_JPRB, 0.11288E+01_JPRB, 0.11341E+01_JPRB, 0.11400E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.12385E+01_JPRB, 0.12479E+01_JPRB, 0.12375E+01_JPRB, 0.12377E+01_JPRB, 0.12586E+01_JPRB, 0.12816E+01_JPRB ,&
&  0.13003E+01_JPRB, 0.13161E+01_JPRB, 0.13338E+01_JPRB, 0.12891E+01_JPRB, 0.13330E+01_JPRB, 0.13050E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.12281E+01_JPRB, 0.11961E+01_JPRB, 0.11691E+01_JPRB, 0.10186E+01_JPRB, 0.10664E+01_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.98569E+00_JPRB, 0.96941E+00_JPRB, 0.94734E+00_JPRB, 0.92768E+00_JPRB, 0.91431E+00_JPRB, 0.90846E+00_JPRB, 0.90600E+00_JPRB ,&
&  0.90925E+00_JPRB, 0.91234E+00_JPRB, 0.91214E+00_JPRB, 0.90815E+00_JPRB, 0.90109E+00_JPRB, 0.88916E+00_JPRB, 0.87799E+00_JPRB ,&
&  0.86190E+00_JPRB, 0.84527E+00_JPRB, 0.83812E+00_JPRB, 0.83765E+00_JPRB, 0.83550E+00_JPRB, 0.82693E+00_JPRB, 0.81963E+00_JPRB ,&
&  0.81504E+00_JPRB, 0.81515E+00_JPRB, 0.80691E+00_JPRB, 0.80005E+00_JPRB, 0.79188E+00_JPRB, 0.79129E+00_JPRB, 0.78920E+00_JPRB ,&
&  0.78688E+00_JPRB, 0.67298E+00_JPRB, 0.79863E+00_JPRB, 0.89378E+00_JPRB, 0.93532E+00_JPRB, 0.10160E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.11579E+01_JPRB, 0.11426E+01_JPRB, 0.11117E+01_JPRB, 0.10686E+01_JPRB, 0.10329E+01_JPRB, 0.10049E+01_JPRB ,&
&  0.97795E+00_JPRB, 0.95270E+00_JPRB, 0.92586E+00_JPRB, 0.89885E+00_JPRB, 0.87831E+00_JPRB, 0.85784E+00_JPRB, 0.84042E+00_JPRB ,&
&  0.83197E+00_JPRB, 0.83185E+00_JPRB, 0.82808E+00_JPRB, 0.82881E+00_JPRB, 0.83383E+00_JPRB, 0.84016E+00_JPRB, 0.87656E+00_JPRB ,&
&  0.91523E+00_JPRB, 0.95319E+00_JPRB, 0.99588E+00_JPRB, 0.10271E+01_JPRB, 0.10380E+01_JPRB, 0.10492E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10233E+01_JPRB, 0.10014E+01_JPRB, 0.97827E+00_JPRB, 0.95844E+00_JPRB, 0.94426E+00_JPRB, 0.94060E+00_JPRB ,&
&  0.94225E+00_JPRB, 0.93209E+00_JPRB, 0.92128E+00_JPRB, 0.91801E+00_JPRB, 0.92008E+00_JPRB, 0.92987E+00_JPRB, 0.95236E+00_JPRB ,&
&  0.98780E+00_JPRB, 0.10319E+01_JPRB, 0.10666E+01_JPRB, 0.10812E+01_JPRB, 0.10837E+01_JPRB, 0.10922E+01_JPRB, 0.11083E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.11402E+01_JPRB, 0.11532E+01_JPRB, 0.11822E+01_JPRB, 0.11816E+01_JPRB, 0.11643E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.10812E+01_JPRB, 0.89918E+00_JPRB, 0.96695E+00_JPRB, 0.97830E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.90181E+00_JPRB, 0.88684E+00_JPRB, 0.86642E+00_JPRB, 0.84977E+00_JPRB, 0.83638E+00_JPRB, 0.83031E+00_JPRB, 0.82659E+00_JPRB ,&
&  0.83022E+00_JPRB, 0.83839E+00_JPRB, 0.84721E+00_JPRB, 0.85319E+00_JPRB, 0.85793E+00_JPRB, 0.85820E+00_JPRB, 0.85348E+00_JPRB ,&
&  0.84920E+00_JPRB, 0.83721E+00_JPRB, 0.82700E+00_JPRB, 0.82579E+00_JPRB, 0.82233E+00_JPRB, 0.81765E+00_JPRB, 0.81039E+00_JPRB ,&
&  0.80261E+00_JPRB, 0.79482E+00_JPRB, 0.78115E+00_JPRB, 0.77325E+00_JPRB, 0.77013E+00_JPRB, 0.77181E+00_JPRB, 0.77961E+00_JPRB ,&
&  0.70126E+00_JPRB, 0.62549E+00_JPRB, 0.71513E+00_JPRB, 0.81594E+00_JPRB, 0.86102E+00_JPRB, 0.90648E+00_JPRB, 0.94614E+00_JPRB ,&
&  0.99997E+00_JPRB, 0.10360E+01_JPRB, 0.10589E+01_JPRB, 0.10582E+01_JPRB, 0.10387E+01_JPRB, 0.10099E+01_JPRB, 0.97742E+00_JPRB ,&
&  0.93625E+00_JPRB, 0.88876E+00_JPRB, 0.85518E+00_JPRB, 0.83538E+00_JPRB, 0.81945E+00_JPRB, 0.80350E+00_JPRB, 0.79082E+00_JPRB ,&
&  0.78252E+00_JPRB, 0.77659E+00_JPRB, 0.77174E+00_JPRB, 0.76862E+00_JPRB, 0.76997E+00_JPRB, 0.77337E+00_JPRB, 0.78252E+00_JPRB ,&
&  0.79990E+00_JPRB, 0.81787E+00_JPRB, 0.83422E+00_JPRB, 0.84457E+00_JPRB, 0.85375E+00_JPRB, 0.86922E+00_JPRB, 0.88096E+00_JPRB ,&
&  0.89109E+00_JPRB, 0.89136E+00_JPRB, 0.88533E+00_JPRB, 0.87475E+00_JPRB, 0.85978E+00_JPRB, 0.84464E+00_JPRB, 0.83249E+00_JPRB ,&
&  0.82425E+00_JPRB, 0.81375E+00_JPRB, 0.80892E+00_JPRB, 0.81033E+00_JPRB, 0.81539E+00_JPRB, 0.82257E+00_JPRB, 0.83200E+00_JPRB ,&
&  0.84949E+00_JPRB, 0.87818E+00_JPRB, 0.90361E+00_JPRB, 0.92040E+00_JPRB, 0.93103E+00_JPRB, 0.93872E+00_JPRB, 0.95021E+00_JPRB ,&
&  0.96301E+00_JPRB, 0.97702E+00_JPRB, 0.99483E+00_JPRB, 0.10143E+01_JPRB, 0.10201E+01_JPRB, 0.10150E+01_JPRB, 0.10014E+01_JPRB ,&
&  0.98049E+00_JPRB, 0.95345E+00_JPRB, 0.93948E+00_JPRB, 0.92308E+00_JPRB, 0.91116E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.79042E+00_JPRB, 0.78149E+00_JPRB, 0.76820E+00_JPRB, 0.75965E+00_JPRB, 0.75529E+00_JPRB, 0.75688E+00_JPRB, 0.76011E+00_JPRB ,&
&  0.76389E+00_JPRB, 0.76508E+00_JPRB, 0.77301E+00_JPRB, 0.78215E+00_JPRB, 0.78874E+00_JPRB, 0.79371E+00_JPRB, 0.79359E+00_JPRB ,&
&  0.79350E+00_JPRB, 0.79347E+00_JPRB, 0.78898E+00_JPRB, 0.78435E+00_JPRB, 0.77810E+00_JPRB, 0.77823E+00_JPRB, 0.77698E+00_JPRB ,&
&  0.77319E+00_JPRB, 0.76647E+00_JPRB, 0.75616E+00_JPRB, 0.74409E+00_JPRB, 0.73769E+00_JPRB, 0.73512E+00_JPRB, 0.73147E+00_JPRB ,&
&  0.73717E+00_JPRB, 0.65679E+00_JPRB, 0.70250E+00_JPRB, 0.73939E+00_JPRB, 0.78664E+00_JPRB, 0.82587E+00_JPRB, 0.86171E+00_JPRB ,&
&  0.89785E+00_JPRB, 0.92855E+00_JPRB, 0.94333E+00_JPRB, 0.94692E+00_JPRB, 0.92931E+00_JPRB, 0.89489E+00_JPRB, 0.85594E+00_JPRB ,&
&  0.82349E+00_JPRB, 0.79937E+00_JPRB, 0.77512E+00_JPRB, 0.75996E+00_JPRB, 0.74757E+00_JPRB, 0.73200E+00_JPRB, 0.71741E+00_JPRB ,&
&  0.70806E+00_JPRB, 0.70138E+00_JPRB, 0.69852E+00_JPRB, 0.69880E+00_JPRB, 0.70094E+00_JPRB, 0.70290E+00_JPRB, 0.70317E+00_JPRB ,&
&  0.70619E+00_JPRB, 0.71295E+00_JPRB, 0.71999E+00_JPRB, 0.72789E+00_JPRB, 0.73756E+00_JPRB, 0.74928E+00_JPRB, 0.75438E+00_JPRB ,&
&  0.75151E+00_JPRB, 0.75339E+00_JPRB, 0.75473E+00_JPRB, 0.75213E+00_JPRB, 0.74822E+00_JPRB, 0.74339E+00_JPRB, 0.73661E+00_JPRB ,&
&  0.73096E+00_JPRB, 0.73017E+00_JPRB, 0.73079E+00_JPRB, 0.73569E+00_JPRB, 0.73969E+00_JPRB, 0.74954E+00_JPRB, 0.75937E+00_JPRB ,&
&  0.77102E+00_JPRB, 0.78962E+00_JPRB, 0.81094E+00_JPRB, 0.82860E+00_JPRB, 0.84003E+00_JPRB, 0.84745E+00_JPRB, 0.85416E+00_JPRB ,&
&  0.86010E+00_JPRB, 0.86638E+00_JPRB, 0.87409E+00_JPRB, 0.87921E+00_JPRB, 0.87799E+00_JPRB, 0.87293E+00_JPRB, 0.86612E+00_JPRB ,&
&  0.85081E+00_JPRB, 0.83005E+00_JPRB, 0.81704E+00_JPRB, 0.80534E+00_JPRB, 0.79586E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.71581E+00_JPRB, 0.70964E+00_JPRB, 0.70353E+00_JPRB, 0.70209E+00_JPRB, 0.70056E+00_JPRB, 0.70174E+00_JPRB, 0.70362E+00_JPRB ,&
&  0.70982E+00_JPRB, 0.71399E+00_JPRB, 0.72365E+00_JPRB, 0.73205E+00_JPRB, 0.73672E+00_JPRB, 0.74209E+00_JPRB, 0.74554E+00_JPRB ,&
&  0.74722E+00_JPRB, 0.74927E+00_JPRB, 0.75055E+00_JPRB, 0.75522E+00_JPRB, 0.75603E+00_JPRB, 0.75546E+00_JPRB, 0.75412E+00_JPRB ,&
&  0.75091E+00_JPRB, 0.74647E+00_JPRB, 0.73835E+00_JPRB, 0.72812E+00_JPRB, 0.71600E+00_JPRB, 0.70942E+00_JPRB, 0.70142E+00_JPRB ,&
&  0.70102E+00_JPRB, 0.67871E+00_JPRB, 0.69229E+00_JPRB, 0.69081E+00_JPRB, 0.71525E+00_JPRB, 0.75315E+00_JPRB, 0.79484E+00_JPRB ,&
&  0.82020E+00_JPRB, 0.83039E+00_JPRB, 0.82706E+00_JPRB, 0.81833E+00_JPRB, 0.79877E+00_JPRB, 0.77978E+00_JPRB, 0.75865E+00_JPRB ,&
&  0.73511E+00_JPRB, 0.72126E+00_JPRB, 0.70545E+00_JPRB, 0.69373E+00_JPRB, 0.68842E+00_JPRB, 0.68427E+00_JPRB, 0.67703E+00_JPRB ,&
&  0.66610E+00_JPRB, 0.65638E+00_JPRB, 0.65496E+00_JPRB, 0.65749E+00_JPRB, 0.66074E+00_JPRB, 0.66278E+00_JPRB, 0.66133E+00_JPRB ,&
&  0.65528E+00_JPRB, 0.65190E+00_JPRB, 0.65423E+00_JPRB, 0.65833E+00_JPRB, 0.65918E+00_JPRB, 0.65608E+00_JPRB, 0.65224E+00_JPRB ,&
&  0.65255E+00_JPRB, 0.65564E+00_JPRB, 0.65875E+00_JPRB, 0.66293E+00_JPRB, 0.66535E+00_JPRB, 0.66810E+00_JPRB, 0.66887E+00_JPRB ,&
&  0.67274E+00_JPRB, 0.67663E+00_JPRB, 0.68510E+00_JPRB, 0.69337E+00_JPRB, 0.69494E+00_JPRB, 0.70372E+00_JPRB, 0.71359E+00_JPRB ,&
&  0.72493E+00_JPRB, 0.73900E+00_JPRB, 0.75222E+00_JPRB, 0.76390E+00_JPRB, 0.77530E+00_JPRB, 0.78219E+00_JPRB, 0.78966E+00_JPRB ,&
&  0.79847E+00_JPRB, 0.80099E+00_JPRB, 0.80734E+00_JPRB, 0.80806E+00_JPRB, 0.80366E+00_JPRB, 0.79569E+00_JPRB, 0.78315E+00_JPRB ,&
&  0.76738E+00_JPRB, 0.74688E+00_JPRB, 0.73373E+00_JPRB, 0.72485E+00_JPRB, 0.71925E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.63517E+00_JPRB, 0.63219E+00_JPRB, 0.63178E+00_JPRB, 0.63448E+00_JPRB, 0.63780E+00_JPRB, 0.64021E+00_JPRB, 0.64552E+00_JPRB ,&
&  0.65521E+00_JPRB, 0.66304E+00_JPRB, 0.67293E+00_JPRB, 0.68336E+00_JPRB, 0.69157E+00_JPRB, 0.69716E+00_JPRB, 0.70024E+00_JPRB ,&
&  0.69896E+00_JPRB, 0.69692E+00_JPRB, 0.70074E+00_JPRB, 0.70603E+00_JPRB, 0.70769E+00_JPRB, 0.70735E+00_JPRB, 0.70733E+00_JPRB ,&
&  0.70632E+00_JPRB, 0.70296E+00_JPRB, 0.69819E+00_JPRB, 0.69137E+00_JPRB, 0.68362E+00_JPRB, 0.67863E+00_JPRB, 0.67426E+00_JPRB ,&
&  0.66552E+00_JPRB, 0.66801E+00_JPRB, 0.61688E+00_JPRB, 0.63073E+00_JPRB, 0.64234E+00_JPRB, 0.66555E+00_JPRB, 0.68970E+00_JPRB ,&
&  0.70575E+00_JPRB, 0.71353E+00_JPRB, 0.71627E+00_JPRB, 0.71226E+00_JPRB, 0.69784E+00_JPRB, 0.69002E+00_JPRB, 0.68203E+00_JPRB ,&
&  0.67117E+00_JPRB, 0.66336E+00_JPRB, 0.64917E+00_JPRB, 0.63364E+00_JPRB, 0.62835E+00_JPRB, 0.62883E+00_JPRB, 0.62830E+00_JPRB ,&
&  0.62272E+00_JPRB, 0.61705E+00_JPRB, 0.61691E+00_JPRB, 0.61707E+00_JPRB, 0.61648E+00_JPRB, 0.61105E+00_JPRB, 0.60439E+00_JPRB ,&
&  0.59411E+00_JPRB, 0.58435E+00_JPRB, 0.57791E+00_JPRB, 0.57393E+00_JPRB, 0.57114E+00_JPRB, 0.56799E+00_JPRB, 0.56459E+00_JPRB ,&
&  0.57037E+00_JPRB, 0.58413E+00_JPRB, 0.59678E+00_JPRB, 0.60666E+00_JPRB, 0.61297E+00_JPRB, 0.61679E+00_JPRB, 0.62187E+00_JPRB ,&
&  0.62787E+00_JPRB, 0.63848E+00_JPRB, 0.64645E+00_JPRB, 0.65360E+00_JPRB, 0.65865E+00_JPRB, 0.66342E+00_JPRB, 0.66910E+00_JPRB ,&
&  0.67470E+00_JPRB, 0.68413E+00_JPRB, 0.69256E+00_JPRB, 0.70238E+00_JPRB, 0.70929E+00_JPRB, 0.71683E+00_JPRB, 0.72388E+00_JPRB ,&
&  0.73098E+00_JPRB, 0.73683E+00_JPRB, 0.73906E+00_JPRB, 0.73688E+00_JPRB, 0.72919E+00_JPRB, 0.71940E+00_JPRB, 0.70856E+00_JPRB ,&
&  0.69441E+00_JPRB, 0.67740E+00_JPRB, 0.66018E+00_JPRB, 0.64871E+00_JPRB, 0.63903E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.57414E+00_JPRB, 0.57403E+00_JPRB, 0.57807E+00_JPRB, 0.58200E+00_JPRB, 0.58504E+00_JPRB, 0.58970E+00_JPRB, 0.59935E+00_JPRB ,&
&  0.60917E+00_JPRB, 0.61872E+00_JPRB, 0.62523E+00_JPRB, 0.63315E+00_JPRB, 0.64452E+00_JPRB, 0.65244E+00_JPRB, 0.65874E+00_JPRB ,&
&  0.66161E+00_JPRB, 0.66257E+00_JPRB, 0.66436E+00_JPRB, 0.66806E+00_JPRB, 0.66796E+00_JPRB, 0.66448E+00_JPRB, 0.66324E+00_JPRB ,&
&  0.66298E+00_JPRB, 0.66083E+00_JPRB, 0.65981E+00_JPRB, 0.65962E+00_JPRB, 0.65557E+00_JPRB, 0.64983E+00_JPRB, 0.64741E+00_JPRB ,&
&  0.64534E+00_JPRB, 0.63067E+00_JPRB, 0.62191E+00_JPRB, 0.61383E+00_JPRB, 0.61635E+00_JPRB, 0.62673E+00_JPRB, 0.63862E+00_JPRB ,&
&  0.64563E+00_JPRB, 0.65032E+00_JPRB, 0.65041E+00_JPRB, 0.64958E+00_JPRB, 0.64459E+00_JPRB, 0.63338E+00_JPRB, 0.62462E+00_JPRB ,&
&  0.61905E+00_JPRB, 0.61012E+00_JPRB, 0.58682E+00_JPRB, 0.57249E+00_JPRB, 0.56831E+00_JPRB, 0.57128E+00_JPRB, 0.57626E+00_JPRB ,&
&  0.57792E+00_JPRB, 0.57449E+00_JPRB, 0.56731E+00_JPRB, 0.55951E+00_JPRB, 0.54959E+00_JPRB, 0.53846E+00_JPRB, 0.52876E+00_JPRB ,&
&  0.52357E+00_JPRB, 0.51992E+00_JPRB, 0.51711E+00_JPRB, 0.52033E+00_JPRB, 0.51993E+00_JPRB, 0.51641E+00_JPRB, 0.51953E+00_JPRB ,&
&  0.53310E+00_JPRB, 0.55043E+00_JPRB, 0.56650E+00_JPRB, 0.57526E+00_JPRB, 0.57890E+00_JPRB, 0.58306E+00_JPRB, 0.58749E+00_JPRB ,&
&  0.59322E+00_JPRB, 0.59843E+00_JPRB, 0.60391E+00_JPRB, 0.61178E+00_JPRB, 0.62137E+00_JPRB, 0.62991E+00_JPRB, 0.63365E+00_JPRB ,&
&  0.63691E+00_JPRB, 0.63973E+00_JPRB, 0.64293E+00_JPRB, 0.65052E+00_JPRB, 0.66022E+00_JPRB, 0.66717E+00_JPRB, 0.67220E+00_JPRB ,&
&  0.67686E+00_JPRB, 0.67880E+00_JPRB, 0.67695E+00_JPRB, 0.67207E+00_JPRB, 0.66590E+00_JPRB, 0.65447E+00_JPRB, 0.64002E+00_JPRB ,&
&  0.62334E+00_JPRB, 0.60757E+00_JPRB, 0.59536E+00_JPRB, 0.58476E+00_JPRB, 0.57710E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.52753E+00_JPRB, 0.52621E+00_JPRB, 0.52835E+00_JPRB, 0.53413E+00_JPRB, 0.53811E+00_JPRB, 0.54175E+00_JPRB, 0.54963E+00_JPRB ,&
&  0.55707E+00_JPRB, 0.56391E+00_JPRB, 0.57170E+00_JPRB, 0.57979E+00_JPRB, 0.58592E+00_JPRB, 0.59504E+00_JPRB, 0.60399E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.61262E+00_JPRB, 0.61452E+00_JPRB, 0.61618E+00_JPRB, 0.61649E+00_JPRB, 0.61434E+00_JPRB, 0.61406E+00_JPRB ,&
&  0.61218E+00_JPRB, 0.60992E+00_JPRB, 0.61173E+00_JPRB, 0.61160E+00_JPRB, 0.61237E+00_JPRB, 0.61387E+00_JPRB, 0.61238E+00_JPRB ,&
&  0.60833E+00_JPRB, 0.59358E+00_JPRB, 0.58751E+00_JPRB, 0.58355E+00_JPRB, 0.58699E+00_JPRB, 0.58727E+00_JPRB, 0.58510E+00_JPRB ,&
&  0.57782E+00_JPRB, 0.57310E+00_JPRB, 0.57467E+00_JPRB, 0.57679E+00_JPRB, 0.57069E+00_JPRB, 0.55749E+00_JPRB, 0.54790E+00_JPRB ,&
&  0.54093E+00_JPRB, 0.52927E+00_JPRB, 0.51100E+00_JPRB, 0.49935E+00_JPRB, 0.49418E+00_JPRB, 0.48938E+00_JPRB, 0.48556E+00_JPRB ,&
&  0.48374E+00_JPRB, 0.48298E+00_JPRB, 0.48408E+00_JPRB, 0.48323E+00_JPRB, 0.47971E+00_JPRB, 0.47571E+00_JPRB, 0.47450E+00_JPRB ,&
&  0.47777E+00_JPRB, 0.48087E+00_JPRB, 0.48489E+00_JPRB, 0.48789E+00_JPRB, 0.48775E+00_JPRB, 0.48831E+00_JPRB, 0.49260E+00_JPRB ,&
&  0.50028E+00_JPRB, 0.50814E+00_JPRB, 0.51482E+00_JPRB, 0.51732E+00_JPRB, 0.52135E+00_JPRB, 0.52931E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.52880E+00_JPRB, 0.53005E+00_JPRB, 0.53404E+00_JPRB, 0.54274E+00_JPRB, 0.55611E+00_JPRB, 0.56522E+00_JPRB, 0.56710E+00_JPRB ,&
&  0.56769E+00_JPRB, 0.57048E+00_JPRB, 0.57545E+00_JPRB, 0.58678E+00_JPRB, 0.59865E+00_JPRB, 0.60700E+00_JPRB, 0.61021E+00_JPRB ,&
&  0.60671E+00_JPRB, 0.59924E+00_JPRB, 0.58640E+00_JPRB, 0.57167E+00_JPRB, 0.55878E+00_JPRB, 0.54881E+00_JPRB, 0.54050E+00_JPRB ,&
&  0.53229E+00_JPRB, 0.52828E+00_JPRB, 0.52735E+00_JPRB, 0.52857E+00_JPRB, 0.52863E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.49039E+00_JPRB, 0.49165E+00_JPRB, 0.49362E+00_JPRB, 0.49638E+00_JPRB, 0.49924E+00_JPRB, 0.50081E+00_JPRB, 0.50427E+00_JPRB ,&
&  0.50656E+00_JPRB, 0.51360E+00_JPRB, 0.52490E+00_JPRB, 0.53593E+00_JPRB, 0.54374E+00_JPRB, 0.55022E+00_JPRB, 0.55686E+00_JPRB ,&
&  0.56249E+00_JPRB, 0.56983E+00_JPRB, 0.57109E+00_JPRB, 0.56906E+00_JPRB, 0.56676E+00_JPRB, 0.56507E+00_JPRB, 0.56347E+00_JPRB ,&
&  0.56094E+00_JPRB, 0.56068E+00_JPRB, 0.56420E+00_JPRB, 0.56807E+00_JPRB, 0.57186E+00_JPRB, 0.57226E+00_JPRB, 0.56988E+00_JPRB ,&
&  0.57211E+00_JPRB, 0.56105E+00_JPRB, 0.55344E+00_JPRB, 0.54501E+00_JPRB, 0.53767E+00_JPRB, 0.52558E+00_JPRB, 0.51649E+00_JPRB ,&
&  0.50560E+00_JPRB, 0.49583E+00_JPRB, 0.49149E+00_JPRB, 0.48704E+00_JPRB, 0.47702E+00_JPRB, 0.46588E+00_JPRB, 0.45814E+00_JPRB ,&
&  0.45320E+00_JPRB, 0.44811E+00_JPRB, 0.44158E+00_JPRB, 0.43587E+00_JPRB, 0.43114E+00_JPRB, 0.42942E+00_JPRB, 0.43095E+00_JPRB ,&
&  0.43552E+00_JPRB, 0.44023E+00_JPRB, 0.44343E+00_JPRB, 0.44455E+00_JPRB, 0.44311E+00_JPRB, 0.44065E+00_JPRB, 0.43948E+00_JPRB ,&
&  0.44072E+00_JPRB, 0.44296E+00_JPRB, 0.44495E+00_JPRB, 0.45010E+00_JPRB, 0.45125E+00_JPRB, 0.44966E+00_JPRB, 0.44428E+00_JPRB ,&
&  0.44271E+00_JPRB, 0.44243E+00_JPRB, 0.44486E+00_JPRB, 0.45048E+00_JPRB, 0.45952E+00_JPRB, 0.46836E+00_JPRB, 0.47454E+00_JPRB ,&
&  0.47719E+00_JPRB, 0.47839E+00_JPRB, 0.48118E+00_JPRB, 0.48507E+00_JPRB, 0.48718E+00_JPRB, 0.48727E+00_JPRB, 0.48638E+00_JPRB ,&
&  0.48972E+00_JPRB, 0.49796E+00_JPRB, 0.50682E+00_JPRB, 0.51331E+00_JPRB, 0.51706E+00_JPRB, 0.51685E+00_JPRB, 0.51028E+00_JPRB ,&
&  0.49758E+00_JPRB, 0.48272E+00_JPRB, 0.46839E+00_JPRB, 0.45652E+00_JPRB, 0.44837E+00_JPRB, 0.44442E+00_JPRB, 0.44716E+00_JPRB ,&
&  0.45474E+00_JPRB, 0.46452E+00_JPRB, 0.47319E+00_JPRB, 0.47934E+00_JPRB, 0.48605E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.45574E+00_JPRB, 0.45565E+00_JPRB, 0.45732E+00_JPRB, 0.45642E+00_JPRB, 0.45364E+00_JPRB, 0.45103E+00_JPRB, 0.45378E+00_JPRB ,&
&  0.46026E+00_JPRB, 0.47223E+00_JPRB, 0.48841E+00_JPRB, 0.50267E+00_JPRB, 0.51114E+00_JPRB, 0.51607E+00_JPRB, 0.52365E+00_JPRB ,&
&  0.53091E+00_JPRB, 0.53902E+00_JPRB, 0.53942E+00_JPRB, 0.53085E+00_JPRB, 0.52364E+00_JPRB, 0.51991E+00_JPRB, 0.51957E+00_JPRB ,&
&  0.51885E+00_JPRB, 0.52039E+00_JPRB, 0.52322E+00_JPRB, 0.52608E+00_JPRB, 0.52484E+00_JPRB, 0.52230E+00_JPRB, 0.52501E+00_JPRB ,&
&  0.52190E+00_JPRB, 0.51077E+00_JPRB, 0.49908E+00_JPRB, 0.47346E+00_JPRB, 0.47166E+00_JPRB, 0.46038E+00_JPRB, 0.45204E+00_JPRB ,&
&  0.44084E+00_JPRB, 0.42682E+00_JPRB, 0.41654E+00_JPRB, 0.41003E+00_JPRB, 0.40694E+00_JPRB, 0.40450E+00_JPRB, 0.40265E+00_JPRB ,&
&  0.40154E+00_JPRB, 0.40175E+00_JPRB, 0.40377E+00_JPRB, 0.40150E+00_JPRB, 0.40028E+00_JPRB, 0.40114E+00_JPRB, 0.40316E+00_JPRB ,&
&  0.40391E+00_JPRB, 0.40488E+00_JPRB, 0.40583E+00_JPRB, 0.40575E+00_JPRB, 0.40440E+00_JPRB, 0.40201E+00_JPRB, 0.39985E+00_JPRB ,&
&  0.39927E+00_JPRB, 0.40105E+00_JPRB, 0.40207E+00_JPRB, 0.39847E+00_JPRB, 0.39093E+00_JPRB, 0.38075E+00_JPRB, 0.37420E+00_JPRB ,&
&  0.37197E+00_JPRB, 0.37499E+00_JPRB, 0.38386E+00_JPRB, 0.39575E+00_JPRB, 0.40639E+00_JPRB, 0.41293E+00_JPRB, 0.41700E+00_JPRB ,&
&  0.42122E+00_JPRB, 0.41827E+00_JPRB, 0.41382E+00_JPRB, 0.41028E+00_JPRB, 0.40725E+00_JPRB, 0.40301E+00_JPRB, 0.40298E+00_JPRB ,&
&  0.40629E+00_JPRB, 0.40987E+00_JPRB, 0.41139E+00_JPRB, 0.41080E+00_JPRB, 0.40894E+00_JPRB, 0.40480E+00_JPRB, 0.39899E+00_JPRB ,&
&  0.39154E+00_JPRB, 0.38281E+00_JPRB, 0.38146E+00_JPRB, 0.38280E+00_JPRB, 0.38680E+00_JPRB, 0.39459E+00_JPRB, 0.40529E+00_JPRB ,&
&  0.41647E+00_JPRB, 0.42534E+00_JPRB, 0.43504E+00_JPRB, 0.44404E+00_JPRB, 0.45219E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.43339E+00_JPRB, 0.43475E+00_JPRB, 0.43210E+00_JPRB, 0.42947E+00_JPRB, 0.42936E+00_JPRB, 0.43071E+00_JPRB, 0.43589E+00_JPRB ,&
&  0.44557E+00_JPRB, 0.45882E+00_JPRB, 0.47163E+00_JPRB, 0.48214E+00_JPRB, 0.49133E+00_JPRB, 0.49529E+00_JPRB, 0.50105E+00_JPRB ,&
&  0.50740E+00_JPRB, 0.51324E+00_JPRB, 0.51395E+00_JPRB, 0.50361E+00_JPRB, 0.49364E+00_JPRB, 0.48833E+00_JPRB, 0.48830E+00_JPRB ,&
&  0.48987E+00_JPRB, 0.49212E+00_JPRB, 0.49782E+00_JPRB, 0.49954E+00_JPRB, 0.49302E+00_JPRB, 0.49086E+00_JPRB, 0.48808E+00_JPRB ,&
&  0.48162E+00_JPRB, 0.46820E+00_JPRB, 0.43484E+00_JPRB, 0.42773E+00_JPRB, 0.41869E+00_JPRB, 0.40925E+00_JPRB, 0.40033E+00_JPRB ,&
&  0.38993E+00_JPRB, 0.37921E+00_JPRB, 0.37192E+00_JPRB, 0.36747E+00_JPRB, 0.36782E+00_JPRB, 0.36908E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.37108E+00_JPRB, 0.37339E+00_JPRB, 0.37571E+00_JPRB, 0.37635E+00_JPRB, 0.37636E+00_JPRB, 0.37610E+00_JPRB, 0.37256E+00_JPRB ,&
&  0.36941E+00_JPRB, 0.36815E+00_JPRB, 0.36726E+00_JPRB, 0.36655E+00_JPRB, 0.36690E+00_JPRB, 0.36775E+00_JPRB, 0.36811E+00_JPRB ,&
&  0.36566E+00_JPRB, 0.36220E+00_JPRB, 0.35827E+00_JPRB, 0.35088E+00_JPRB, 0.33673E+00_JPRB, 0.32299E+00_JPRB, 0.31362E+00_JPRB ,&
&  0.31275E+00_JPRB, 0.32328E+00_JPRB, 0.34354E+00_JPRB, 0.35571E+00_JPRB, 0.36347E+00_JPRB, 0.36700E+00_JPRB, 0.37053E+00_JPRB ,&
&  0.36273E+00_JPRB, 0.34096E+00_JPRB, 0.32827E+00_JPRB, 0.32045E+00_JPRB, 0.31542E+00_JPRB, 0.31224E+00_JPRB, 0.31180E+00_JPRB ,&
&  0.31538E+00_JPRB, 0.32103E+00_JPRB, 0.32759E+00_JPRB, 0.33075E+00_JPRB, 0.32979E+00_JPRB, 0.32692E+00_JPRB, 0.32348E+00_JPRB ,&
&  0.32165E+00_JPRB, 0.32722E+00_JPRB, 0.34221E+00_JPRB, 0.36102E+00_JPRB, 0.36967E+00_JPRB, 0.37660E+00_JPRB, 0.38296E+00_JPRB ,&
&  0.38990E+00_JPRB, 0.40030E+00_JPRB, 0.41239E+00_JPRB, 0.42166E+00_JPRB, 0.42971E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.40338E+00_JPRB, 0.40702E+00_JPRB, 0.40918E+00_JPRB, 0.41245E+00_JPRB, 0.42015E+00_JPRB, 0.43062E+00_JPRB, 0.44260E+00_JPRB ,&
&  0.45434E+00_JPRB, 0.46445E+00_JPRB, 0.46972E+00_JPRB, 0.47511E+00_JPRB, 0.48006E+00_JPRB, 0.48229E+00_JPRB, 0.48710E+00_JPRB ,&
&  0.48762E+00_JPRB, 0.48387E+00_JPRB, 0.47750E+00_JPRB, 0.46934E+00_JPRB, 0.46496E+00_JPRB, 0.46598E+00_JPRB, 0.46849E+00_JPRB ,&
&  0.46988E+00_JPRB, 0.47088E+00_JPRB, 0.47056E+00_JPRB, 0.46660E+00_JPRB, 0.46296E+00_JPRB, 0.46073E+00_JPRB, 0.45596E+00_JPRB ,&
&  0.44405E+00_JPRB, 0.39334E+00_JPRB, 0.35623E+00_JPRB, 0.35332E+00_JPRB, 0.36756E+00_JPRB, 0.36928E+00_JPRB, 0.36495E+00_JPRB ,&
&  0.35967E+00_JPRB, 0.35456E+00_JPRB, 0.35151E+00_JPRB, 0.34919E+00_JPRB, 0.34637E+00_JPRB, 0.34365E+00_JPRB, 0.34383E+00_JPRB ,&
&  0.34495E+00_JPRB, 0.34277E+00_JPRB, 0.33980E+00_JPRB, 0.33332E+00_JPRB, 0.32884E+00_JPRB, 0.32788E+00_JPRB, 0.32716E+00_JPRB ,&
&  0.32314E+00_JPRB, 0.31892E+00_JPRB, 0.31934E+00_JPRB, 0.32109E+00_JPRB, 0.32363E+00_JPRB, 0.32528E+00_JPRB, 0.32221E+00_JPRB ,&
&  0.31814E+00_JPRB, 0.31106E+00_JPRB, 0.30312E+00_JPRB, 0.29332E+00_JPRB, 0.28422E+00_JPRB, 0.27723E+00_JPRB, 0.27136E+00_JPRB ,&
&  0.27129E+00_JPRB, 0.27902E+00_JPRB, 0.29429E+00_JPRB, 0.30970E+00_JPRB, 0.32174E+00_JPRB, 0.31548E+00_JPRB, 0.30635E+00_JPRB ,&
&  0.29491E+00_JPRB, 0.28475E+00_JPRB, 0.27751E+00_JPRB, 0.27434E+00_JPRB, 0.27308E+00_JPRB, 0.27294E+00_JPRB, 0.27426E+00_JPRB ,&
&  0.27676E+00_JPRB, 0.27969E+00_JPRB, 0.28195E+00_JPRB, 0.28297E+00_JPRB, 0.28326E+00_JPRB, 0.28330E+00_JPRB, 0.28260E+00_JPRB ,&
&  0.28189E+00_JPRB, 0.28475E+00_JPRB, 0.29206E+00_JPRB, 0.30105E+00_JPRB, 0.30929E+00_JPRB, 0.31752E+00_JPRB, 0.33071E+00_JPRB ,&
&  0.34685E+00_JPRB, 0.36578E+00_JPRB, 0.37985E+00_JPRB, 0.39089E+00_JPRB, 0.39973E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.39152E+00_JPRB, 0.40069E+00_JPRB, 0.41215E+00_JPRB, 0.42243E+00_JPRB, 0.43197E+00_JPRB, 0.44077E+00_JPRB, 0.44896E+00_JPRB ,&
&  0.45394E+00_JPRB, 0.45682E+00_JPRB, 0.45806E+00_JPRB, 0.46187E+00_JPRB, 0.45857E+00_JPRB, 0.45077E+00_JPRB, 0.44490E+00_JPRB ,&
&  0.43988E+00_JPRB, 0.43603E+00_JPRB, 0.43056E+00_JPRB, 0.42766E+00_JPRB, 0.42723E+00_JPRB, 0.42707E+00_JPRB, 0.42117E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.40787E+00_JPRB, 0.41198E+00_JPRB, 0.40576E+00_JPRB, 0.39954E+00_JPRB, 0.39336E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.36971E+00_JPRB, 0.34869E+00_JPRB, 0.32263E+00_JPRB, 0.32093E+00_JPRB, 0.33659E+00_JPRB, 0.32875E+00_JPRB, 0.33303E+00_JPRB ,&
&  0.33436E+00_JPRB, 0.33425E+00_JPRB, 0.33279E+00_JPRB, 0.33092E+00_JPRB, 0.32745E+00_JPRB, 0.32660E+00_JPRB, 0.32456E+00_JPRB ,&
&  0.31752E+00_JPRB, 0.31216E+00_JPRB, 0.30331E+00_JPRB, 0.28444E+00_JPRB, 0.28076E+00_JPRB, 0.28074E+00_JPRB, 0.27679E+00_JPRB ,&
&  0.26936E+00_JPRB, 0.26521E+00_JPRB, 0.26438E+00_JPRB, 0.25922E+00_JPRB, 0.26154E+00_JPRB, 0.26412E+00_JPRB, 0.26393E+00_JPRB ,&
&  0.26374E+00_JPRB, 0.26317E+00_JPRB, 0.26085E+00_JPRB, 0.25681E+00_JPRB, 0.25345E+00_JPRB, 0.25265E+00_JPRB, 0.25414E+00_JPRB ,&
&  0.25696E+00_JPRB, 0.26141E+00_JPRB, 0.27334E+00_JPRB, 0.28233E+00_JPRB, 0.28068E+00_JPRB, 0.27710E+00_JPRB, 0.26758E+00_JPRB ,&
&  0.25783E+00_JPRB, 0.25437E+00_JPRB, 0.25126E+00_JPRB, 0.24945E+00_JPRB, 0.24960E+00_JPRB, 0.25109E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.25288E+00_JPRB, 0.25430E+00_JPRB, 0.25525E+00_JPRB, 0.25565E+00_JPRB, 0.25634E+00_JPRB, 0.25718E+00_JPRB, 0.25889E+00_JPRB ,&
&  0.26143E+00_JPRB, 0.26602E+00_JPRB, 0.27198E+00_JPRB, 0.27845E+00_JPRB, 0.28328E+00_JPRB, 0.28581E+00_JPRB, 0.29123E+00_JPRB ,&
&  0.29764E+00_JPRB, 0.31449E+00_JPRB, 0.34396E+00_JPRB, 0.38705E+00_JPRB, 0.38952E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.38800E+00_JPRB, 0.40041E+00_JPRB, 0.41555E+00_JPRB, 0.42529E+00_JPRB, 0.43137E+00_JPRB, 0.43461E+00_JPRB, 0.43410E+00_JPRB ,&
&  0.43081E+00_JPRB, 0.42998E+00_JPRB, 0.44078E+00_JPRB, 0.42166E+00_JPRB, 0.40450E+00_JPRB, 0.37225E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.34245E+00_JPRB, 0.33998E+00_JPRB, 0.33921E+00_JPRB, 0.34361E+00_JPRB, 0.34689E+00_JPRB, 0.34941E+00_JPRB, 0.34490E+00_JPRB ,&
&  0.33890E+00_JPRB, 0.32939E+00_JPRB, 0.32141E+00_JPRB, 0.31415E+00_JPRB, 0.31459E+00_JPRB, 0.31337E+00_JPRB, 0.31068E+00_JPRB ,&
&  0.30791E+00_JPRB, 0.30641E+00_JPRB, 0.30675E+00_JPRB, 0.30799E+00_JPRB, 0.31953E+00_JPRB, 0.31786E+00_JPRB, 0.31899E+00_JPRB ,&
&  0.31908E+00_JPRB, 0.31905E+00_JPRB, 0.31795E+00_JPRB, 0.31616E+00_JPRB, 0.31110E+00_JPRB, 0.29417E+00_JPRB, 0.29217E+00_JPRB ,&
&  0.28440E+00_JPRB, 0.27414E+00_JPRB, 0.26389E+00_JPRB, 0.25675E+00_JPRB, 0.25240E+00_JPRB, 0.25111E+00_JPRB, 0.24644E+00_JPRB ,&
&  0.24150E+00_JPRB, 0.23532E+00_JPRB, 0.23269E+00_JPRB, 0.23293E+00_JPRB, 0.23385E+00_JPRB, 0.23454E+00_JPRB, 0.23506E+00_JPRB ,&
&  0.23661E+00_JPRB, 0.23732E+00_JPRB, 0.23778E+00_JPRB, 0.23871E+00_JPRB, 0.23961E+00_JPRB, 0.24084E+00_JPRB, 0.24288E+00_JPRB ,&
&  0.24649E+00_JPRB, 0.25021E+00_JPRB, 0.25382E+00_JPRB, 0.25751E+00_JPRB, 0.25655E+00_JPRB, 0.25223E+00_JPRB, 0.24674E+00_JPRB ,&
&  0.24261E+00_JPRB, 0.24107E+00_JPRB, 0.23958E+00_JPRB, 0.23922E+00_JPRB, 0.23916E+00_JPRB, 0.23945E+00_JPRB, 0.24002E+00_JPRB ,&
&  0.24092E+00_JPRB, 0.24168E+00_JPRB, 0.24260E+00_JPRB, 0.24381E+00_JPRB, 0.24543E+00_JPRB, 0.24691E+00_JPRB, 0.24896E+00_JPRB ,&
&  0.25203E+00_JPRB, 0.25543E+00_JPRB, 0.26019E+00_JPRB, 0.26454E+00_JPRB, 0.27116E+00_JPRB, 0.27982E+00_JPRB, 0.29393E+00_JPRB ,&
&  0.30851E+00_JPRB, 0.34113E+00_JPRB, 0.35526E+00_JPRB, 0.36931E+00_JPRB, 0.37783E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.38734E+00_JPRB, 0.39753E+00_JPRB, 0.41155E+00_JPRB, 0.42100E+00_JPRB, 0.42806E+00_JPRB, 0.42397E+00_JPRB, 0.41996E+00_JPRB ,&
&  0.41560E+00_JPRB, 0.40210E+00_JPRB, 0.38471E+00_JPRB, 0.35561E+00_JPRB, 0.33114E+00_JPRB, 0.31254E+00_JPRB, 0.30153E+00_JPRB ,&
&  0.29430E+00_JPRB, 0.29156E+00_JPRB, 0.28849E+00_JPRB, 0.28729E+00_JPRB, 0.28515E+00_JPRB, 0.28607E+00_JPRB, 0.28680E+00_JPRB ,&
&  0.28592E+00_JPRB, 0.28511E+00_JPRB, 0.28495E+00_JPRB, 0.28825E+00_JPRB, 0.29480E+00_JPRB, 0.29980E+00_JPRB, 0.30477E+00_JPRB ,&
&  0.30797E+00_JPRB, 0.30492E+00_JPRB, 0.30686E+00_JPRB, 0.30532E+00_JPRB, 0.30473E+00_JPRB, 0.30634E+00_JPRB, 0.30842E+00_JPRB ,&
&  0.30881E+00_JPRB, 0.30842E+00_JPRB, 0.30711E+00_JPRB, 0.30312E+00_JPRB, 0.29008E+00_JPRB, 0.28364E+00_JPRB, 0.27498E+00_JPRB ,&
&  0.26680E+00_JPRB, 0.25726E+00_JPRB, 0.24914E+00_JPRB, 0.24557E+00_JPRB, 0.24046E+00_JPRB, 0.23878E+00_JPRB, 0.23539E+00_JPRB ,&
&  0.23159E+00_JPRB, 0.22808E+00_JPRB, 0.22610E+00_JPRB, 0.22517E+00_JPRB, 0.22488E+00_JPRB, 0.22331E+00_JPRB, 0.22284E+00_JPRB ,&
&  0.22334E+00_JPRB, 0.22381E+00_JPRB, 0.22505E+00_JPRB, 0.22608E+00_JPRB, 0.22711E+00_JPRB, 0.22791E+00_JPRB, 0.22883E+00_JPRB ,&
&  0.23056E+00_JPRB, 0.23286E+00_JPRB, 0.23387E+00_JPRB, 0.23489E+00_JPRB, 0.23451E+00_JPRB, 0.23370E+00_JPRB, 0.23258E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23128E+00_JPRB, 0.23126E+00_JPRB, 0.23110E+00_JPRB, 0.23153E+00_JPRB, 0.23241E+00_JPRB, 0.23320E+00_JPRB ,&
&  0.23481E+00_JPRB, 0.23622E+00_JPRB, 0.23788E+00_JPRB, 0.24004E+00_JPRB, 0.24236E+00_JPRB, 0.24520E+00_JPRB, 0.24835E+00_JPRB ,&
&  0.25091E+00_JPRB, 0.25441E+00_JPRB, 0.25737E+00_JPRB, 0.25976E+00_JPRB, 0.26659E+00_JPRB, 0.27439E+00_JPRB, 0.29042E+00_JPRB ,&
&  0.30728E+00_JPRB, 0.34471E+00_JPRB, 0.38498E+00_JPRB, 0.37287E+00_JPRB, 0.37891E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.35961E+00_JPRB, 0.37105E+00_JPRB, 0.37968E+00_JPRB, 0.38514E+00_JPRB, 0.39027E+00_JPRB, 0.39198E+00_JPRB, 0.39900E+00_JPRB ,&
&  0.39356E+00_JPRB, 0.37596E+00_JPRB, 0.36484E+00_JPRB, 0.34809E+00_JPRB, 0.32775E+00_JPRB, 0.31079E+00_JPRB, 0.30179E+00_JPRB ,&
&  0.29627E+00_JPRB, 0.29160E+00_JPRB, 0.28617E+00_JPRB, 0.28035E+00_JPRB, 0.27461E+00_JPRB, 0.27026E+00_JPRB, 0.26774E+00_JPRB ,&
&  0.26540E+00_JPRB, 0.26673E+00_JPRB, 0.27018E+00_JPRB, 0.27527E+00_JPRB, 0.28263E+00_JPRB, 0.28685E+00_JPRB, 0.29038E+00_JPRB ,&
&  0.29112E+00_JPRB, 0.28045E+00_JPRB, 0.28408E+00_JPRB, 0.28749E+00_JPRB, 0.28685E+00_JPRB, 0.29193E+00_JPRB, 0.30037E+00_JPRB ,&
&  0.29641E+00_JPRB, 0.28570E+00_JPRB, 0.28741E+00_JPRB, 0.27988E+00_JPRB, 0.27098E+00_JPRB, 0.27024E+00_JPRB, 0.26427E+00_JPRB ,&
&  0.26299E+00_JPRB, 0.25594E+00_JPRB, 0.25077E+00_JPRB, 0.24740E+00_JPRB, 0.24020E+00_JPRB, 0.23740E+00_JPRB, 0.23333E+00_JPRB ,&
&  0.22943E+00_JPRB, 0.22704E+00_JPRB, 0.22407E+00_JPRB, 0.22296E+00_JPRB, 0.22191E+00_JPRB, 0.22041E+00_JPRB, 0.21920E+00_JPRB ,&
&  0.21845E+00_JPRB, 0.21778E+00_JPRB, 0.21776E+00_JPRB, 0.21768E+00_JPRB, 0.21748E+00_JPRB, 0.21772E+00_JPRB, 0.21773E+00_JPRB ,&
&  0.21837E+00_JPRB, 0.21924E+00_JPRB, 0.21993E+00_JPRB, 0.22118E+00_JPRB, 0.22162E+00_JPRB, 0.22226E+00_JPRB, 0.22267E+00_JPRB ,&
&  0.22306E+00_JPRB, 0.22371E+00_JPRB, 0.22384E+00_JPRB, 0.22442E+00_JPRB, 0.22504E+00_JPRB, 0.22656E+00_JPRB, 0.22803E+00_JPRB ,&
&  0.22988E+00_JPRB, 0.23218E+00_JPRB, 0.23457E+00_JPRB, 0.23712E+00_JPRB, 0.24032E+00_JPRB, 0.24310E+00_JPRB, 0.24692E+00_JPRB ,&
&  0.25108E+00_JPRB, 0.25510E+00_JPRB, 0.26024E+00_JPRB, 0.26641E+00_JPRB, 0.27486E+00_JPRB, 0.28508E+00_JPRB, 0.29356E+00_JPRB ,&
&  0.30306E+00_JPRB, 0.32747E+00_JPRB, 0.33388E+00_JPRB, 0.34090E+00_JPRB, 0.35040E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.34858E+00_JPRB, 0.35399E+00_JPRB, 0.35815E+00_JPRB, 0.36087E+00_JPRB, 0.36365E+00_JPRB, 0.36538E+00_JPRB, 0.36606E+00_JPRB ,&
&  0.35941E+00_JPRB, 0.35453E+00_JPRB, 0.34731E+00_JPRB, 0.33846E+00_JPRB, 0.33029E+00_JPRB, 0.32155E+00_JPRB, 0.31267E+00_JPRB ,&
&  0.30532E+00_JPRB, 0.29841E+00_JPRB, 0.29164E+00_JPRB, 0.28440E+00_JPRB, 0.27694E+00_JPRB, 0.26942E+00_JPRB, 0.26679E+00_JPRB ,&
&  0.26665E+00_JPRB, 0.26875E+00_JPRB, 0.27192E+00_JPRB, 0.27347E+00_JPRB, 0.27437E+00_JPRB, 0.27471E+00_JPRB, 0.27344E+00_JPRB ,&
&  0.26937E+00_JPRB, 0.27053E+00_JPRB, 0.27225E+00_JPRB, 0.27334E+00_JPRB, 0.27540E+00_JPRB, 0.27720E+00_JPRB, 0.27717E+00_JPRB ,&
&  0.27500E+00_JPRB, 0.27225E+00_JPRB, 0.26849E+00_JPRB, 0.26493E+00_JPRB, 0.26395E+00_JPRB, 0.26195E+00_JPRB, 0.25968E+00_JPRB ,&
&  0.25849E+00_JPRB, 0.25464E+00_JPRB, 0.25020E+00_JPRB, 0.24707E+00_JPRB, 0.24250E+00_JPRB, 0.23941E+00_JPRB, 0.23531E+00_JPRB ,&
&  0.23214E+00_JPRB, 0.22939E+00_JPRB, 0.22636E+00_JPRB, 0.22411E+00_JPRB, 0.22206E+00_JPRB, 0.22054E+00_JPRB, 0.21899E+00_JPRB ,&
&  0.21811E+00_JPRB, 0.21720E+00_JPRB, 0.21659E+00_JPRB, 0.21586E+00_JPRB, 0.21535E+00_JPRB, 0.21487E+00_JPRB, 0.21449E+00_JPRB ,&
&  0.21427E+00_JPRB, 0.21464E+00_JPRB, 0.21486E+00_JPRB, 0.21529E+00_JPRB, 0.21578E+00_JPRB, 0.21599E+00_JPRB, 0.21652E+00_JPRB ,&
&  0.21765E+00_JPRB, 0.21887E+00_JPRB, 0.21953E+00_JPRB, 0.22047E+00_JPRB, 0.22156E+00_JPRB, 0.22285E+00_JPRB, 0.22474E+00_JPRB ,&
&  0.22669E+00_JPRB, 0.22890E+00_JPRB, 0.23142E+00_JPRB, 0.23385E+00_JPRB, 0.23714E+00_JPRB, 0.24052E+00_JPRB, 0.24413E+00_JPRB ,&
&  0.24858E+00_JPRB, 0.25325E+00_JPRB, 0.25818E+00_JPRB, 0.26361E+00_JPRB, 0.27154E+00_JPRB, 0.27742E+00_JPRB, 0.28461E+00_JPRB ,&
&  0.29115E+00_JPRB, 0.29422E+00_JPRB, 0.32774E+00_JPRB, 0.33252E+00_JPRB, 0.33955E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.27284E+00_JPRB, 0.27904E+00_JPRB, 0.28483E+00_JPRB, 0.29318E+00_JPRB, 0.29613E+00_JPRB, 0.33252E+00_JPRB, 0.32751E+00_JPRB ,&
&  0.32491E+00_JPRB, 0.31327E+00_JPRB, 0.31264E+00_JPRB, 0.30943E+00_JPRB, 0.30509E+00_JPRB, 0.30045E+00_JPRB, 0.29558E+00_JPRB ,&
&  0.29107E+00_JPRB, 0.28565E+00_JPRB, 0.28134E+00_JPRB, 0.27785E+00_JPRB, 0.27476E+00_JPRB, 0.27168E+00_JPRB, 0.26906E+00_JPRB ,&
&  0.26807E+00_JPRB, 0.26814E+00_JPRB, 0.26746E+00_JPRB, 0.26241E+00_JPRB, 0.25862E+00_JPRB, 0.25641E+00_JPRB, 0.25589E+00_JPRB ,&
&  0.25597E+00_JPRB, 0.25834E+00_JPRB, 0.26010E+00_JPRB, 0.26169E+00_JPRB, 0.26392E+00_JPRB, 0.26301E+00_JPRB, 0.26210E+00_JPRB ,&
&  0.26072E+00_JPRB, 0.25928E+00_JPRB, 0.25784E+00_JPRB, 0.25652E+00_JPRB, 0.25520E+00_JPRB, 0.25322E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.25024E+00_JPRB, 0.24819E+00_JPRB, 0.24502E+00_JPRB, 0.24295E+00_JPRB, 0.24043E+00_JPRB, 0.23830E+00_JPRB, 0.23606E+00_JPRB ,&
&  0.23437E+00_JPRB, 0.23287E+00_JPRB, 0.23119E+00_JPRB, 0.22916E+00_JPRB, 0.22720E+00_JPRB, 0.22547E+00_JPRB, 0.22385E+00_JPRB ,&
&  0.22303E+00_JPRB, 0.22214E+00_JPRB, 0.22125E+00_JPRB, 0.22031E+00_JPRB, 0.21985E+00_JPRB, 0.21934E+00_JPRB, 0.21886E+00_JPRB ,&
&  0.21848E+00_JPRB, 0.21818E+00_JPRB, 0.21813E+00_JPRB, 0.21817E+00_JPRB, 0.21824E+00_JPRB, 0.21861E+00_JPRB, 0.21908E+00_JPRB ,&
&  0.21973E+00_JPRB, 0.22053E+00_JPRB, 0.22170E+00_JPRB, 0.22291E+00_JPRB, 0.22398E+00_JPRB, 0.22481E+00_JPRB, 0.22583E+00_JPRB ,&
&  0.22747E+00_JPRB, 0.22933E+00_JPRB, 0.23140E+00_JPRB, 0.23285E+00_JPRB, 0.23517E+00_JPRB, 0.23728E+00_JPRB, 0.23941E+00_JPRB ,&
&  0.24153E+00_JPRB, 0.24423E+00_JPRB, 0.24740E+00_JPRB, 0.25092E+00_JPRB, 0.25513E+00_JPRB, 0.25821E+00_JPRB, 0.26112E+00_JPRB ,&
&  0.26408E+00_JPRB, 0.26869E+00_JPRB, 0.27049E+00_JPRB, 0.27117E+00_JPRB, 0.27192E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.25258E+00_JPRB, 0.25247E+00_JPRB, 0.25269E+00_JPRB, 0.25290E+00_JPRB, 0.25298E+00_JPRB, 0.25345E+00_JPRB, 0.25340E+00_JPRB ,&
&  0.25315E+00_JPRB, 0.25285E+00_JPRB, 0.25319E+00_JPRB, 0.25357E+00_JPRB, 0.25379E+00_JPRB, 0.25381E+00_JPRB, 0.25340E+00_JPRB ,&
&  0.25262E+00_JPRB, 0.25166E+00_JPRB, 0.24993E+00_JPRB, 0.24920E+00_JPRB, 0.24867E+00_JPRB, 0.24824E+00_JPRB, 0.24743E+00_JPRB ,&
&  0.24670E+00_JPRB, 0.24545E+00_JPRB, 0.24380E+00_JPRB, 0.24279E+00_JPRB, 0.24283E+00_JPRB, 0.24343E+00_JPRB, 0.24437E+00_JPRB ,&
&  0.24512E+00_JPRB, 0.24563E+00_JPRB, 0.24590E+00_JPRB, 0.24624E+00_JPRB, 0.24669E+00_JPRB, 0.24660E+00_JPRB, 0.24637E+00_JPRB ,&
&  0.24605E+00_JPRB, 0.24547E+00_JPRB, 0.24491E+00_JPRB, 0.24441E+00_JPRB, 0.24397E+00_JPRB, 0.24341E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24199E+00_JPRB, 0.24106E+00_JPRB, 0.23989E+00_JPRB, 0.23898E+00_JPRB, 0.23799E+00_JPRB, 0.23720E+00_JPRB, 0.23620E+00_JPRB ,&
&  0.23535E+00_JPRB, 0.23458E+00_JPRB, 0.23380E+00_JPRB, 0.23308E+00_JPRB, 0.23240E+00_JPRB, 0.23177E+00_JPRB, 0.23116E+00_JPRB ,&
&  0.23056E+00_JPRB, 0.23008E+00_JPRB, 0.22954E+00_JPRB, 0.22904E+00_JPRB, 0.22869E+00_JPRB, 0.22839E+00_JPRB, 0.22815E+00_JPRB ,&
&  0.22794E+00_JPRB, 0.22790E+00_JPRB, 0.22789E+00_JPRB, 0.22788E+00_JPRB, 0.22788E+00_JPRB, 0.22801E+00_JPRB, 0.22814E+00_JPRB ,&
&  0.22831E+00_JPRB, 0.22842E+00_JPRB, 0.22853E+00_JPRB, 0.22871E+00_JPRB, 0.22891E+00_JPRB, 0.22914E+00_JPRB, 0.22946E+00_JPRB ,&
&  0.22983E+00_JPRB, 0.23028E+00_JPRB, 0.23077E+00_JPRB, 0.23100E+00_JPRB, 0.23170E+00_JPRB, 0.23251E+00_JPRB, 0.23346E+00_JPRB ,&
&  0.23453E+00_JPRB, 0.23583E+00_JPRB, 0.23734E+00_JPRB, 0.23919E+00_JPRB, 0.24115E+00_JPRB, 0.24231E+00_JPRB, 0.24349E+00_JPRB ,&
&  0.24472E+00_JPRB, 0.24618E+00_JPRB, 0.24757E+00_JPRB, 0.24896E+00_JPRB, 0.25059E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB ,&
&  0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB, 0.23499E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB, 0.63310E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.64533E+00_JPRB, 0.64475E+00_JPRB, 0.64418E+00_JPRB, 0.64355E+00_JPRB, 0.64274E+00_JPRB, 0.64193E+00_JPRB, 0.64125E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.64009E+00_JPRB, 0.63925E+00_JPRB, 0.63839E+00_JPRB, 0.63751E+00_JPRB, 0.63673E+00_JPRB, 0.63587E+00_JPRB ,&
&  0.63499E+00_JPRB, 0.63410E+00_JPRB, 0.63342E+00_JPRB, 0.63258E+00_JPRB, 0.63169E+00_JPRB, 0.63087E+00_JPRB, 0.63013E+00_JPRB ,&
&  0.62943E+00_JPRB, 0.62864E+00_JPRB, 0.62783E+00_JPRB, 0.62685E+00_JPRB, 0.62622E+00_JPRB, 0.62532E+00_JPRB, 0.62459E+00_JPRB ,&
&  0.62376E+00_JPRB, 0.62318E+00_JPRB, 0.62253E+00_JPRB, 0.62192E+00_JPRB, 0.62120E+00_JPRB, 0.62111E+00_JPRB, 0.62069E+00_JPRB ,&
&  0.62048E+00_JPRB, 0.62027E+00_JPRB, 0.62030E+00_JPRB, 0.62025E+00_JPRB, 0.62028E+00_JPRB, 0.62039E+00_JPRB, 0.62091E+00_JPRB ,&
&  0.62135E+00_JPRB, 0.62187E+00_JPRB, 0.62265E+00_JPRB, 0.62338E+00_JPRB, 0.62404E+00_JPRB, 0.62473E+00_JPRB, 0.62550E+00_JPRB ,&
&  0.62653E+00_JPRB, 0.62753E+00_JPRB, 0.62847E+00_JPRB, 0.62943E+00_JPRB, 0.63038E+00_JPRB, 0.63116E+00_JPRB, 0.63181E+00_JPRB ,&
&  0.63222E+00_JPRB, 0.63271E+00_JPRB, 0.63317E+00_JPRB, 0.63367E+00_JPRB, 0.63380E+00_JPRB, 0.63414E+00_JPRB, 0.63434E+00_JPRB ,&
&  0.63452E+00_JPRB, 0.63423E+00_JPRB, 0.63431E+00_JPRB, 0.63421E+00_JPRB, 0.63431E+00_JPRB, 0.63429E+00_JPRB, 0.63467E+00_JPRB ,&
&  0.63496E+00_JPRB, 0.63557E+00_JPRB, 0.63622E+00_JPRB, 0.63710E+00_JPRB, 0.63776E+00_JPRB, 0.63851E+00_JPRB, 0.63917E+00_JPRB ,&
&  0.63993E+00_JPRB, 0.64072E+00_JPRB, 0.64157E+00_JPRB, 0.64216E+00_JPRB, 0.64268E+00_JPRB, 0.64329E+00_JPRB, 0.64393E+00_JPRB ,&
&  0.64476E+00_JPRB, 0.64524E+00_JPRB, 0.64578E+00_JPRB, 0.64625E+00_JPRB, 0.64646E+00_JPRB, 0.64661E+00_JPRB, 0.64661E+00_JPRB ,&
&  0.64650E+00_JPRB, 0.64633E+00_JPRB, 0.64610E+00_JPRB, 0.64591E+00_JPRB, 0.64571E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.65518E+00_JPRB, 0.65486E+00_JPRB, 0.65428E+00_JPRB, 0.65339E+00_JPRB, 0.65232E+00_JPRB, 0.65100E+00_JPRB, 0.64970E+00_JPRB ,&
&  0.64824E+00_JPRB, 0.64684E+00_JPRB, 0.64506E+00_JPRB, 0.64329E+00_JPRB, 0.64145E+00_JPRB, 0.63971E+00_JPRB, 0.63783E+00_JPRB ,&
&  0.63606E+00_JPRB, 0.63430E+00_JPRB, 0.63306E+00_JPRB, 0.63170E+00_JPRB, 0.63025E+00_JPRB, 0.62850E+00_JPRB, 0.62707E+00_JPRB ,&
&  0.62553E+00_JPRB, 0.62373E+00_JPRB, 0.62177E+00_JPRB, 0.61836E+00_JPRB, 0.61542E+00_JPRB, 0.61273E+00_JPRB, 0.61040E+00_JPRB ,&
&  0.60826E+00_JPRB, 0.60625E+00_JPRB, 0.60473E+00_JPRB, 0.60371E+00_JPRB, 0.60473E+00_JPRB, 0.60495E+00_JPRB, 0.60463E+00_JPRB ,&
&  0.60413E+00_JPRB, 0.60204E+00_JPRB, 0.60141E+00_JPRB, 0.60123E+00_JPRB, 0.60132E+00_JPRB, 0.60282E+00_JPRB, 0.60409E+00_JPRB ,&
&  0.60543E+00_JPRB, 0.60655E+00_JPRB, 0.60870E+00_JPRB, 0.61075E+00_JPRB, 0.61267E+00_JPRB, 0.61473E+00_JPRB, 0.61595E+00_JPRB ,&
&  0.61733E+00_JPRB, 0.61932E+00_JPRB, 0.62120E+00_JPRB, 0.62318E+00_JPRB, 0.62438E+00_JPRB, 0.62547E+00_JPRB, 0.62626E+00_JPRB ,&
&  0.62692E+00_JPRB, 0.62784E+00_JPRB, 0.62883E+00_JPRB, 0.62975E+00_JPRB, 0.63037E+00_JPRB, 0.63096E+00_JPRB, 0.63135E+00_JPRB ,&
&  0.63147E+00_JPRB, 0.63098E+00_JPRB, 0.63083E+00_JPRB, 0.63056E+00_JPRB, 0.63008E+00_JPRB, 0.62942E+00_JPRB, 0.62924E+00_JPRB ,&
&  0.62901E+00_JPRB, 0.62856E+00_JPRB, 0.62785E+00_JPRB, 0.62817E+00_JPRB, 0.62864E+00_JPRB, 0.62889E+00_JPRB, 0.62893E+00_JPRB ,&
&  0.63109E+00_JPRB, 0.63406E+00_JPRB, 0.63752E+00_JPRB, 0.64223E+00_JPRB, 0.64570E+00_JPRB, 0.64790E+00_JPRB, 0.64940E+00_JPRB ,&
&  0.65001E+00_JPRB, 0.65062E+00_JPRB, 0.65114E+00_JPRB, 0.65199E+00_JPRB, 0.65257E+00_JPRB, 0.65364E+00_JPRB, 0.65513E+00_JPRB ,&
&  0.65616E+00_JPRB, 0.65649E+00_JPRB, 0.65639E+00_JPRB, 0.65603E+00_JPRB, 0.65558E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.65862E+00_JPRB, 0.65755E+00_JPRB, 0.65643E+00_JPRB, 0.65460E+00_JPRB, 0.65360E+00_JPRB, 0.65203E+00_JPRB, 0.65064E+00_JPRB ,&
&  0.64892E+00_JPRB, 0.64688E+00_JPRB, 0.64528E+00_JPRB, 0.64390E+00_JPRB, 0.64244E+00_JPRB, 0.64099E+00_JPRB, 0.63938E+00_JPRB ,&
&  0.63772E+00_JPRB, 0.63585E+00_JPRB, 0.63504E+00_JPRB, 0.63374E+00_JPRB, 0.63229E+00_JPRB, 0.63068E+00_JPRB, 0.62675E+00_JPRB ,&
&  0.62294E+00_JPRB, 0.61830E+00_JPRB, 0.61341E+00_JPRB, 0.60921E+00_JPRB, 0.60659E+00_JPRB, 0.58952E+00_JPRB, 0.58671E+00_JPRB ,&
&  0.59402E+00_JPRB, 0.59818E+00_JPRB, 0.60271E+00_JPRB, 0.60734E+00_JPRB, 0.59688E+00_JPRB, 0.58486E+00_JPRB, 0.57775E+00_JPRB ,&
&  0.56144E+00_JPRB, 0.56812E+00_JPRB, 0.57167E+00_JPRB, 0.57500E+00_JPRB, 0.57978E+00_JPRB, 0.57990E+00_JPRB, 0.58278E+00_JPRB ,&
&  0.59328E+00_JPRB, 0.59340E+00_JPRB, 0.59486E+00_JPRB, 0.59650E+00_JPRB, 0.59811E+00_JPRB, 0.59975E+00_JPRB, 0.60284E+00_JPRB ,&
&  0.60539E+00_JPRB, 0.60832E+00_JPRB, 0.61156E+00_JPRB, 0.61309E+00_JPRB, 0.61459E+00_JPRB, 0.61660E+00_JPRB, 0.61846E+00_JPRB ,&
&  0.61932E+00_JPRB, 0.62051E+00_JPRB, 0.62186E+00_JPRB, 0.62316E+00_JPRB, 0.62486E+00_JPRB, 0.62606E+00_JPRB, 0.62706E+00_JPRB ,&
&  0.62785E+00_JPRB, 0.62771E+00_JPRB, 0.62772E+00_JPRB, 0.62798E+00_JPRB, 0.62796E+00_JPRB, 0.62773E+00_JPRB, 0.62795E+00_JPRB ,&
&  0.62860E+00_JPRB, 0.62904E+00_JPRB, 0.62994E+00_JPRB, 0.63126E+00_JPRB, 0.63471E+00_JPRB, 0.63944E+00_JPRB, 0.64883E+00_JPRB ,&
&  0.65645E+00_JPRB, 0.66301E+00_JPRB, 0.66538E+00_JPRB, 0.66800E+00_JPRB, 0.67004E+00_JPRB, 0.67052E+00_JPRB, 0.67033E+00_JPRB ,&
&  0.66905E+00_JPRB, 0.66679E+00_JPRB, 0.66455E+00_JPRB, 0.66248E+00_JPRB, 0.66132E+00_JPRB, 0.66053E+00_JPRB, 0.66059E+00_JPRB ,&
&  0.66134E+00_JPRB, 0.66255E+00_JPRB, 0.66287E+00_JPRB, 0.66152E+00_JPRB, 0.65982E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.66806E+00_JPRB, 0.66537E+00_JPRB, 0.66447E+00_JPRB, 0.66207E+00_JPRB, 0.65988E+00_JPRB, 0.65688E+00_JPRB, 0.65438E+00_JPRB ,&
&  0.65145E+00_JPRB, 0.64876E+00_JPRB, 0.64638E+00_JPRB, 0.64416E+00_JPRB, 0.64295E+00_JPRB, 0.64159E+00_JPRB, 0.64068E+00_JPRB ,&
&  0.63955E+00_JPRB, 0.63800E+00_JPRB, 0.63715E+00_JPRB, 0.63590E+00_JPRB, 0.63470E+00_JPRB, 0.63334E+00_JPRB, 0.63015E+00_JPRB ,&
&  0.62441E+00_JPRB, 0.61646E+00_JPRB, 0.60010E+00_JPRB, 0.60302E+00_JPRB, 0.60149E+00_JPRB, 0.59966E+00_JPRB, 0.60054E+00_JPRB ,&
&  0.59230E+00_JPRB, 0.58993E+00_JPRB, 0.58030E+00_JPRB, 0.57183E+00_JPRB, 0.56896E+00_JPRB, 0.56104E+00_JPRB, 0.56133E+00_JPRB ,&
&  0.55873E+00_JPRB, 0.55739E+00_JPRB, 0.55851E+00_JPRB, 0.55785E+00_JPRB, 0.56245E+00_JPRB, 0.56441E+00_JPRB, 0.56768E+00_JPRB ,&
&  0.57397E+00_JPRB, 0.57906E+00_JPRB, 0.58217E+00_JPRB, 0.57647E+00_JPRB, 0.57924E+00_JPRB, 0.58207E+00_JPRB, 0.58569E+00_JPRB ,&
&  0.58919E+00_JPRB, 0.59169E+00_JPRB, 0.59439E+00_JPRB, 0.59631E+00_JPRB, 0.59836E+00_JPRB, 0.60154E+00_JPRB, 0.60481E+00_JPRB ,&
&  0.60882E+00_JPRB, 0.61167E+00_JPRB, 0.61350E+00_JPRB, 0.61540E+00_JPRB, 0.61735E+00_JPRB, 0.61930E+00_JPRB, 0.62179E+00_JPRB ,&
&  0.62489E+00_JPRB, 0.62873E+00_JPRB, 0.63282E+00_JPRB, 0.63708E+00_JPRB, 0.64101E+00_JPRB, 0.64614E+00_JPRB, 0.65195E+00_JPRB ,&
&  0.66051E+00_JPRB, 0.66820E+00_JPRB, 0.68019E+00_JPRB, 0.68748E+00_JPRB, 0.70303E+00_JPRB, 0.71754E+00_JPRB, 0.72619E+00_JPRB ,&
&  0.72414E+00_JPRB, 0.71967E+00_JPRB, 0.71545E+00_JPRB, 0.71095E+00_JPRB, 0.70697E+00_JPRB, 0.70177E+00_JPRB, 0.69805E+00_JPRB ,&
&  0.69425E+00_JPRB, 0.69057E+00_JPRB, 0.68687E+00_JPRB, 0.68314E+00_JPRB, 0.68003E+00_JPRB, 0.67662E+00_JPRB, 0.67454E+00_JPRB ,&
&  0.67385E+00_JPRB, 0.67450E+00_JPRB, 0.67460E+00_JPRB, 0.67360E+00_JPRB, 0.67108E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.67952E+00_JPRB, 0.67927E+00_JPRB, 0.68075E+00_JPRB, 0.67965E+00_JPRB, 0.67690E+00_JPRB, 0.67399E+00_JPRB, 0.66805E+00_JPRB ,&
&  0.66207E+00_JPRB, 0.65607E+00_JPRB, 0.65068E+00_JPRB, 0.64660E+00_JPRB, 0.64320E+00_JPRB, 0.64045E+00_JPRB, 0.63927E+00_JPRB ,&
&  0.63760E+00_JPRB, 0.63729E+00_JPRB, 0.63708E+00_JPRB, 0.63713E+00_JPRB, 0.63799E+00_JPRB, 0.63824E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.63496E+00_JPRB, 0.62561E+00_JPRB, 0.61289E+00_JPRB, 0.61513E+00_JPRB, 0.61675E+00_JPRB, 0.61928E+00_JPRB, 0.62239E+00_JPRB ,&
&  0.61920E+00_JPRB, 0.59543E+00_JPRB, 0.60326E+00_JPRB, 0.59278E+00_JPRB, 0.58759E+00_JPRB, 0.58162E+00_JPRB, 0.56415E+00_JPRB ,&
&  0.55809E+00_JPRB, 0.54754E+00_JPRB, 0.54159E+00_JPRB, 0.54158E+00_JPRB, 0.54392E+00_JPRB, 0.55013E+00_JPRB, 0.55798E+00_JPRB ,&
&  0.55860E+00_JPRB, 0.56974E+00_JPRB, 0.57100E+00_JPRB, 0.57039E+00_JPRB, 0.57322E+00_JPRB, 0.57670E+00_JPRB, 0.57483E+00_JPRB ,&
&  0.57580E+00_JPRB, 0.57518E+00_JPRB, 0.57474E+00_JPRB, 0.57828E+00_JPRB, 0.58493E+00_JPRB, 0.58828E+00_JPRB, 0.59167E+00_JPRB ,&
&  0.59663E+00_JPRB, 0.59957E+00_JPRB, 0.60136E+00_JPRB, 0.60380E+00_JPRB, 0.60711E+00_JPRB, 0.61053E+00_JPRB, 0.61572E+00_JPRB ,&
&  0.62164E+00_JPRB, 0.63191E+00_JPRB, 0.64154E+00_JPRB, 0.65607E+00_JPRB, 0.66962E+00_JPRB, 0.68782E+00_JPRB, 0.70370E+00_JPRB ,&
&  0.72054E+00_JPRB, 0.73412E+00_JPRB, 0.75080E+00_JPRB, 0.76575E+00_JPRB, 0.77769E+00_JPRB, 0.77381E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.74922E+00_JPRB, 0.73425E+00_JPRB, 0.72571E+00_JPRB, 0.71891E+00_JPRB, 0.71404E+00_JPRB, 0.71174E+00_JPRB, 0.70922E+00_JPRB ,&
&  0.70784E+00_JPRB, 0.70534E+00_JPRB, 0.70212E+00_JPRB, 0.69779E+00_JPRB, 0.69417E+00_JPRB, 0.69029E+00_JPRB, 0.68777E+00_JPRB ,&
&  0.68440E+00_JPRB, 0.68332E+00_JPRB, 0.68178E+00_JPRB, 0.68086E+00_JPRB, 0.68017E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.69147E+00_JPRB, 0.69520E+00_JPRB, 0.69399E+00_JPRB, 0.69209E+00_JPRB, 0.68551E+00_JPRB, 0.67989E+00_JPRB, 0.67116E+00_JPRB ,&
&  0.66260E+00_JPRB, 0.65393E+00_JPRB, 0.64599E+00_JPRB, 0.64041E+00_JPRB, 0.63604E+00_JPRB, 0.63298E+00_JPRB, 0.63172E+00_JPRB ,&
&  0.63135E+00_JPRB, 0.63225E+00_JPRB, 0.63307E+00_JPRB, 0.63412E+00_JPRB, 0.63664E+00_JPRB, 0.63955E+00_JPRB, 0.64266E+00_JPRB ,&
&  0.64337E+00_JPRB, 0.63936E+00_JPRB, 0.62769E+00_JPRB, 0.62673E+00_JPRB, 0.63244E+00_JPRB, 0.62837E+00_JPRB, 0.64003E+00_JPRB ,&
&  0.64892E+00_JPRB, 0.65260E+00_JPRB, 0.64344E+00_JPRB, 0.63695E+00_JPRB, 0.62435E+00_JPRB, 0.57579E+00_JPRB, 0.57287E+00_JPRB ,&
&  0.56470E+00_JPRB, 0.55161E+00_JPRB, 0.54593E+00_JPRB, 0.54105E+00_JPRB, 0.54009E+00_JPRB, 0.54802E+00_JPRB, 0.55938E+00_JPRB ,&
&  0.56151E+00_JPRB, 0.57848E+00_JPRB, 0.57818E+00_JPRB, 0.57711E+00_JPRB, 0.57816E+00_JPRB, 0.58174E+00_JPRB, 0.57946E+00_JPRB ,&
&  0.58025E+00_JPRB, 0.58043E+00_JPRB, 0.58064E+00_JPRB, 0.58531E+00_JPRB, 0.59006E+00_JPRB, 0.59382E+00_JPRB, 0.59950E+00_JPRB ,&
&  0.61186E+00_JPRB, 0.61999E+00_JPRB, 0.62450E+00_JPRB, 0.62850E+00_JPRB, 0.63476E+00_JPRB, 0.64197E+00_JPRB, 0.65227E+00_JPRB ,&
&  0.65981E+00_JPRB, 0.66937E+00_JPRB, 0.68055E+00_JPRB, 0.69254E+00_JPRB, 0.70474E+00_JPRB, 0.71854E+00_JPRB, 0.73345E+00_JPRB ,&
&  0.74163E+00_JPRB, 0.75302E+00_JPRB, 0.76571E+00_JPRB, 0.78713E+00_JPRB, 0.78761E+00_JPRB, 0.77375E+00_JPRB, 0.76283E+00_JPRB ,&
&  0.75485E+00_JPRB, 0.75566E+00_JPRB, 0.74817E+00_JPRB, 0.74343E+00_JPRB, 0.73803E+00_JPRB, 0.73600E+00_JPRB, 0.73203E+00_JPRB ,&
&  0.72895E+00_JPRB, 0.72452E+00_JPRB, 0.71849E+00_JPRB, 0.71143E+00_JPRB, 0.70620E+00_JPRB, 0.70070E+00_JPRB, 0.69692E+00_JPRB ,&
&  0.69227E+00_JPRB, 0.69014E+00_JPRB, 0.68879E+00_JPRB, 0.68874E+00_JPRB, 0.68831E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.70212E+00_JPRB, 0.70389E+00_JPRB, 0.69693E+00_JPRB, 0.69362E+00_JPRB, 0.68331E+00_JPRB, 0.67293E+00_JPRB, 0.65994E+00_JPRB ,&
&  0.64952E+00_JPRB, 0.64159E+00_JPRB, 0.63403E+00_JPRB, 0.63036E+00_JPRB, 0.62886E+00_JPRB, 0.62841E+00_JPRB, 0.63014E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.63065E+00_JPRB, 0.63347E+00_JPRB, 0.63830E+00_JPRB, 0.64188E+00_JPRB, 0.64500E+00_JPRB, 0.64820E+00_JPRB ,&
&  0.64736E+00_JPRB, 0.64604E+00_JPRB, 0.64383E+00_JPRB, 0.63706E+00_JPRB, 0.63442E+00_JPRB, 0.64380E+00_JPRB, 0.65726E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.65186E+00_JPRB, 0.65835E+00_JPRB, 0.65862E+00_JPRB, 0.64018E+00_JPRB, 0.61350E+00_JPRB, 0.58977E+00_JPRB ,&
&  0.57287E+00_JPRB, 0.55021E+00_JPRB, 0.54053E+00_JPRB, 0.53547E+00_JPRB, 0.53985E+00_JPRB, 0.54844E+00_JPRB, 0.56472E+00_JPRB ,&
&  0.57438E+00_JPRB, 0.57821E+00_JPRB, 0.58399E+00_JPRB, 0.58933E+00_JPRB, 0.59023E+00_JPRB, 0.59355E+00_JPRB, 0.59419E+00_JPRB ,&
&  0.59899E+00_JPRB, 0.60221E+00_JPRB, 0.60564E+00_JPRB, 0.61312E+00_JPRB, 0.62451E+00_JPRB, 0.63643E+00_JPRB, 0.65465E+00_JPRB ,&
&  0.69947E+00_JPRB, 0.71385E+00_JPRB, 0.72351E+00_JPRB, 0.72842E+00_JPRB, 0.73527E+00_JPRB, 0.73745E+00_JPRB, 0.73001E+00_JPRB ,&
&  0.73051E+00_JPRB, 0.74199E+00_JPRB, 0.74456E+00_JPRB, 0.74829E+00_JPRB, 0.75361E+00_JPRB, 0.75900E+00_JPRB, 0.76578E+00_JPRB ,&
&  0.76096E+00_JPRB, 0.77540E+00_JPRB, 0.84333E+00_JPRB, 0.86392E+00_JPRB, 0.85029E+00_JPRB, 0.83174E+00_JPRB, 0.81323E+00_JPRB ,&
&  0.79873E+00_JPRB, 0.78583E+00_JPRB, 0.77461E+00_JPRB, 0.76535E+00_JPRB, 0.75854E+00_JPRB, 0.75472E+00_JPRB, 0.75364E+00_JPRB ,&
&  0.74669E+00_JPRB, 0.73855E+00_JPRB, 0.72956E+00_JPRB, 0.72287E+00_JPRB, 0.71733E+00_JPRB, 0.71118E+00_JPRB, 0.70761E+00_JPRB ,&
&  0.70410E+00_JPRB, 0.70413E+00_JPRB, 0.70428E+00_JPRB, 0.70481E+00_JPRB, 0.70407E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.70432E+00_JPRB, 0.70660E+00_JPRB, 0.70565E+00_JPRB, 0.69563E+00_JPRB, 0.67451E+00_JPRB, 0.65461E+00_JPRB, 0.64001E+00_JPRB ,&
&  0.63348E+00_JPRB, 0.63006E+00_JPRB, 0.62718E+00_JPRB, 0.62573E+00_JPRB, 0.62590E+00_JPRB, 0.62751E+00_JPRB, 0.63042E+00_JPRB ,&
&  0.63471E+00_JPRB, 0.63875E+00_JPRB, 0.64075E+00_JPRB, 0.64169E+00_JPRB, 0.64487E+00_JPRB, 0.65063E+00_JPRB, 0.65690E+00_JPRB ,&
&  0.65990E+00_JPRB, 0.65895E+00_JPRB, 0.66014E+00_JPRB, 0.65758E+00_JPRB, 0.65568E+00_JPRB, 0.65236E+00_JPRB, 0.65287E+00_JPRB ,&
&  0.66206E+00_JPRB, 0.66920E+00_JPRB, 0.65973E+00_JPRB, 0.66076E+00_JPRB, 0.66691E+00_JPRB, 0.64337E+00_JPRB, 0.61090E+00_JPRB ,&
&  0.57623E+00_JPRB, 0.55042E+00_JPRB, 0.53933E+00_JPRB, 0.53662E+00_JPRB, 0.54470E+00_JPRB, 0.56295E+00_JPRB, 0.58478E+00_JPRB ,&
&  0.58570E+00_JPRB, 0.59178E+00_JPRB, 0.60184E+00_JPRB, 0.60761E+00_JPRB, 0.60874E+00_JPRB, 0.60850E+00_JPRB, 0.60933E+00_JPRB ,&
&  0.61346E+00_JPRB, 0.62477E+00_JPRB, 0.64427E+00_JPRB, 0.67240E+00_JPRB, 0.71409E+00_JPRB, 0.73357E+00_JPRB, 0.77332E+00_JPRB ,&
&  0.83635E+00_JPRB, 0.83454E+00_JPRB, 0.84047E+00_JPRB, 0.83789E+00_JPRB, 0.83472E+00_JPRB, 0.82869E+00_JPRB, 0.82717E+00_JPRB ,&
&  0.82743E+00_JPRB, 0.82469E+00_JPRB, 0.81645E+00_JPRB, 0.80791E+00_JPRB, 0.80925E+00_JPRB, 0.81840E+00_JPRB, 0.83931E+00_JPRB ,&
&  0.87256E+00_JPRB, 0.92488E+00_JPRB, 0.10524E+01_JPRB, 0.97079E+00_JPRB, 0.92636E+00_JPRB, 0.90706E+00_JPRB, 0.88018E+00_JPRB ,&
&  0.85755E+00_JPRB, 0.84231E+00_JPRB, 0.83259E+00_JPRB, 0.82050E+00_JPRB, 0.79773E+00_JPRB, 0.78119E+00_JPRB, 0.76879E+00_JPRB ,&
&  0.75856E+00_JPRB, 0.74842E+00_JPRB, 0.74063E+00_JPRB, 0.73512E+00_JPRB, 0.72924E+00_JPRB, 0.72455E+00_JPRB, 0.72237E+00_JPRB ,&
&  0.71858E+00_JPRB, 0.71608E+00_JPRB, 0.71415E+00_JPRB, 0.70994E+00_JPRB, 0.70662E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.70921E+00_JPRB, 0.71142E+00_JPRB, 0.70276E+00_JPRB, 0.69101E+00_JPRB, 0.66640E+00_JPRB, 0.64778E+00_JPRB, 0.63559E+00_JPRB ,&
&  0.63102E+00_JPRB, 0.62634E+00_JPRB, 0.62370E+00_JPRB, 0.62309E+00_JPRB, 0.62458E+00_JPRB, 0.62832E+00_JPRB, 0.63114E+00_JPRB ,&
&  0.63354E+00_JPRB, 0.63795E+00_JPRB, 0.64340E+00_JPRB, 0.64745E+00_JPRB, 0.65081E+00_JPRB, 0.65510E+00_JPRB, 0.66082E+00_JPRB ,&
&  0.66703E+00_JPRB, 0.66998E+00_JPRB, 0.67145E+00_JPRB, 0.67249E+00_JPRB, 0.67673E+00_JPRB, 0.68270E+00_JPRB, 0.68355E+00_JPRB ,&
&  0.67454E+00_JPRB, 0.67156E+00_JPRB, 0.67824E+00_JPRB, 0.67822E+00_JPRB, 0.68245E+00_JPRB, 0.66576E+00_JPRB, 0.62829E+00_JPRB ,&
&  0.58922E+00_JPRB, 0.56292E+00_JPRB, 0.55933E+00_JPRB, 0.56991E+00_JPRB, 0.58947E+00_JPRB, 0.59622E+00_JPRB, 0.60751E+00_JPRB ,&
&  0.61640E+00_JPRB, 0.61931E+00_JPRB, 0.62170E+00_JPRB, 0.62782E+00_JPRB, 0.63122E+00_JPRB, 0.63116E+00_JPRB, 0.63370E+00_JPRB ,&
&  0.64262E+00_JPRB, 0.65832E+00_JPRB, 0.66584E+00_JPRB, 0.66690E+00_JPRB, 0.69914E+00_JPRB, 0.78097E+00_JPRB, 0.85360E+00_JPRB ,&
&  0.87006E+00_JPRB, 0.87993E+00_JPRB, 0.88961E+00_JPRB, 0.92022E+00_JPRB, 0.93369E+00_JPRB, 0.94682E+00_JPRB, 0.94814E+00_JPRB ,&
&  0.93709E+00_JPRB, 0.91674E+00_JPRB, 0.89958E+00_JPRB, 0.89965E+00_JPRB, 0.92491E+00_JPRB, 0.97304E+00_JPRB, 0.10213E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.10290E+01_JPRB, 0.94913E+00_JPRB, 0.87679E+00_JPRB, 0.85474E+00_JPRB, 0.86075E+00_JPRB, 0.87237E+00_JPRB ,&
&  0.86796E+00_JPRB, 0.86024E+00_JPRB, 0.85127E+00_JPRB, 0.84199E+00_JPRB, 0.80435E+00_JPRB, 0.78035E+00_JPRB, 0.76109E+00_JPRB ,&
&  0.74809E+00_JPRB, 0.73882E+00_JPRB, 0.73949E+00_JPRB, 0.73604E+00_JPRB, 0.73225E+00_JPRB, 0.72720E+00_JPRB, 0.71955E+00_JPRB ,&
&  0.71493E+00_JPRB, 0.71148E+00_JPRB, 0.70681E+00_JPRB, 0.70390E+00_JPRB, 0.70372E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.72317E+00_JPRB, 0.71743E+00_JPRB, 0.71086E+00_JPRB, 0.69566E+00_JPRB, 0.68350E+00_JPRB, 0.67684E+00_JPRB, 0.67164E+00_JPRB ,&
&  0.67632E+00_JPRB, 0.65751E+00_JPRB, 0.63887E+00_JPRB, 0.62641E+00_JPRB, 0.61988E+00_JPRB, 0.61902E+00_JPRB, 0.62475E+00_JPRB ,&
&  0.63269E+00_JPRB, 0.63985E+00_JPRB, 0.64638E+00_JPRB, 0.65370E+00_JPRB, 0.66064E+00_JPRB, 0.66538E+00_JPRB, 0.67023E+00_JPRB ,&
&  0.67640E+00_JPRB, 0.68263E+00_JPRB, 0.68835E+00_JPRB, 0.69633E+00_JPRB, 0.70181E+00_JPRB, 0.70497E+00_JPRB, 0.70269E+00_JPRB ,&
&  0.69656E+00_JPRB, 0.69141E+00_JPRB, 0.69226E+00_JPRB, 0.69419E+00_JPRB, 0.69509E+00_JPRB, 0.68024E+00_JPRB, 0.64256E+00_JPRB ,&
&  0.60851E+00_JPRB, 0.59424E+00_JPRB, 0.60779E+00_JPRB, 0.62379E+00_JPRB, 0.63369E+00_JPRB, 0.64179E+00_JPRB, 0.64245E+00_JPRB ,&
&  0.64014E+00_JPRB, 0.64001E+00_JPRB, 0.64768E+00_JPRB, 0.65576E+00_JPRB, 0.65808E+00_JPRB, 0.66014E+00_JPRB, 0.66491E+00_JPRB ,&
&  0.67804E+00_JPRB, 0.69322E+00_JPRB, 0.70064E+00_JPRB, 0.69761E+00_JPRB, 0.76875E+00_JPRB, 0.10002E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.10450E+01_JPRB, 0.10729E+01_JPRB, 0.11001E+01_JPRB, 0.10865E+01_JPRB, 0.10817E+01_JPRB, 0.10828E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.10503E+01_JPRB, 0.10566E+01_JPRB, 0.11036E+01_JPRB, 0.11403E+01_JPRB, 0.11640E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.11136E+01_JPRB, 0.10380E+01_JPRB, 0.96819E+00_JPRB, 0.93289E+00_JPRB, 0.91026E+00_JPRB, 0.89656E+00_JPRB ,&
&  0.88150E+00_JPRB, 0.87598E+00_JPRB, 0.87516E+00_JPRB, 0.87081E+00_JPRB, 0.83666E+00_JPRB, 0.79467E+00_JPRB, 0.77000E+00_JPRB ,&
&  0.75500E+00_JPRB, 0.74236E+00_JPRB, 0.73167E+00_JPRB, 0.72527E+00_JPRB, 0.72827E+00_JPRB, 0.73064E+00_JPRB, 0.72468E+00_JPRB ,&
&  0.72515E+00_JPRB, 0.71907E+00_JPRB, 0.71714E+00_JPRB, 0.71797E+00_JPRB, 0.72417E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.72914E+00_JPRB, 0.72620E+00_JPRB, 0.71989E+00_JPRB, 0.71132E+00_JPRB, 0.70830E+00_JPRB, 0.70456E+00_JPRB, 0.69265E+00_JPRB ,&
&  0.70199E+00_JPRB, 0.66518E+00_JPRB, 0.64582E+00_JPRB, 0.63767E+00_JPRB, 0.63655E+00_JPRB, 0.63163E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.62843E+00_JPRB, 0.63936E+00_JPRB, 0.65138E+00_JPRB, 0.66048E+00_JPRB, 0.66916E+00_JPRB, 0.67483E+00_JPRB, 0.67993E+00_JPRB ,&
&  0.68643E+00_JPRB, 0.69524E+00_JPRB, 0.70729E+00_JPRB, 0.71535E+00_JPRB, 0.71929E+00_JPRB, 0.72252E+00_JPRB, 0.72075E+00_JPRB ,&
&  0.71720E+00_JPRB, 0.70918E+00_JPRB, 0.70468E+00_JPRB, 0.70255E+00_JPRB, 0.69965E+00_JPRB, 0.69957E+00_JPRB, 0.68079E+00_JPRB ,&
&  0.64828E+00_JPRB, 0.64861E+00_JPRB, 0.66054E+00_JPRB, 0.67144E+00_JPRB, 0.67201E+00_JPRB, 0.66820E+00_JPRB, 0.66767E+00_JPRB ,&
&  0.66936E+00_JPRB, 0.67122E+00_JPRB, 0.67653E+00_JPRB, 0.67838E+00_JPRB, 0.68176E+00_JPRB, 0.68503E+00_JPRB, 0.69783E+00_JPRB ,&
&  0.72554E+00_JPRB, 0.70665E+00_JPRB, 0.71471E+00_JPRB, 0.76248E+00_JPRB, 0.95217E+00_JPRB, 0.10441E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.12678E+01_JPRB, 0.12683E+01_JPRB, 0.12744E+01_JPRB, 0.12850E+01_JPRB, 0.12932E+01_JPRB, 0.13077E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.12954E+01_JPRB, 0.13212E+01_JPRB, 0.13634E+01_JPRB, 0.13725E+01_JPRB, 0.13672E+01_JPRB, 0.13449E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.12468E+01_JPRB, 0.11650E+01_JPRB, 0.10853E+01_JPRB, 0.10182E+01_JPRB, 0.98461E+00_JPRB, 0.96135E+00_JPRB ,&
&  0.94114E+00_JPRB, 0.93001E+00_JPRB, 0.91838E+00_JPRB, 0.90342E+00_JPRB, 0.88455E+00_JPRB, 0.86493E+00_JPRB, 0.85832E+00_JPRB ,&
&  0.82034E+00_JPRB, 0.79486E+00_JPRB, 0.76829E+00_JPRB, 0.75471E+00_JPRB, 0.75429E+00_JPRB, 0.75725E+00_JPRB, 0.75806E+00_JPRB ,&
&  0.76212E+00_JPRB, 0.74950E+00_JPRB, 0.74599E+00_JPRB, 0.74237E+00_JPRB, 0.73890E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.73557E+00_JPRB, 0.73773E+00_JPRB, 0.73358E+00_JPRB, 0.73015E+00_JPRB, 0.73387E+00_JPRB, 0.73353E+00_JPRB, 0.73151E+00_JPRB ,&
&  0.77002E+00_JPRB, 0.74604E+00_JPRB, 0.75263E+00_JPRB, 0.69118E+00_JPRB, 0.66524E+00_JPRB, 0.65177E+00_JPRB, 0.63548E+00_JPRB ,&
&  0.63312E+00_JPRB, 0.63845E+00_JPRB, 0.64632E+00_JPRB, 0.65729E+00_JPRB, 0.66877E+00_JPRB, 0.67599E+00_JPRB, 0.68279E+00_JPRB ,&
&  0.69092E+00_JPRB, 0.70148E+00_JPRB, 0.71153E+00_JPRB, 0.71878E+00_JPRB, 0.72348E+00_JPRB, 0.72862E+00_JPRB, 0.72520E+00_JPRB ,&
&  0.71300E+00_JPRB, 0.70803E+00_JPRB, 0.70623E+00_JPRB, 0.70350E+00_JPRB, 0.70622E+00_JPRB, 0.70763E+00_JPRB, 0.71722E+00_JPRB ,&
&  0.71560E+00_JPRB, 0.70735E+00_JPRB, 0.71320E+00_JPRB, 0.70889E+00_JPRB, 0.70455E+00_JPRB, 0.69874E+00_JPRB, 0.69294E+00_JPRB ,&
&  0.69002E+00_JPRB, 0.69132E+00_JPRB, 0.69742E+00_JPRB, 0.70218E+00_JPRB, 0.70664E+00_JPRB, 0.71670E+00_JPRB, 0.74336E+00_JPRB ,&
&  0.78623E+00_JPRB, 0.74982E+00_JPRB, 0.82557E+00_JPRB, 0.86796E+00_JPRB, 0.10664E+01_JPRB, 0.12812E+01_JPRB, 0.13403E+01_JPRB ,&
&  0.15602E+01_JPRB, 0.15764E+01_JPRB, 0.15553E+01_JPRB, 0.15413E+01_JPRB, 0.15548E+01_JPRB, 0.15282E+01_JPRB, 0.14800E+01_JPRB ,&
&  0.14751E+01_JPRB, 0.14558E+01_JPRB, 0.15431E+01_JPRB, 0.15456E+01_JPRB, 0.15033E+01_JPRB, 0.14720E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.13632E+01_JPRB, 0.13131E+01_JPRB, 0.12641E+01_JPRB, 0.11627E+01_JPRB, 0.11258E+01_JPRB, 0.10893E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.98828E+00_JPRB, 0.94958E+00_JPRB, 0.91206E+00_JPRB, 0.92074E+00_JPRB, 0.89361E+00_JPRB, 0.87869E+00_JPRB, 0.87208E+00_JPRB ,&
&  0.84723E+00_JPRB, 0.81958E+00_JPRB, 0.82360E+00_JPRB, 0.81898E+00_JPRB, 0.80396E+00_JPRB, 0.79165E+00_JPRB, 0.79101E+00_JPRB ,&
&  0.77802E+00_JPRB, 0.77226E+00_JPRB, 0.77794E+00_JPRB, 0.76104E+00_JPRB, 0.74645E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.75856E+00_JPRB, 0.75790E+00_JPRB, 0.75091E+00_JPRB, 0.75226E+00_JPRB, 0.76164E+00_JPRB, 0.76966E+00_JPRB, 0.77780E+00_JPRB ,&
&  0.79963E+00_JPRB, 0.80707E+00_JPRB, 0.80449E+00_JPRB, 0.78954E+00_JPRB, 0.76496E+00_JPRB, 0.71034E+00_JPRB, 0.64730E+00_JPRB ,&
&  0.63299E+00_JPRB, 0.64293E+00_JPRB, 0.65480E+00_JPRB, 0.66141E+00_JPRB, 0.67068E+00_JPRB, 0.68068E+00_JPRB, 0.68980E+00_JPRB ,&
&  0.69955E+00_JPRB, 0.70727E+00_JPRB, 0.71564E+00_JPRB, 0.72264E+00_JPRB, 0.72666E+00_JPRB, 0.73292E+00_JPRB, 0.73521E+00_JPRB ,&
&  0.71652E+00_JPRB, 0.70675E+00_JPRB, 0.70123E+00_JPRB, 0.70578E+00_JPRB, 0.72223E+00_JPRB, 0.73260E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.75400E+00_JPRB, 0.75833E+00_JPRB, 0.75496E+00_JPRB, 0.75059E+00_JPRB, 0.74408E+00_JPRB, 0.73950E+00_JPRB, 0.73320E+00_JPRB ,&
&  0.72885E+00_JPRB, 0.72758E+00_JPRB, 0.73241E+00_JPRB, 0.73660E+00_JPRB, 0.74357E+00_JPRB, 0.76578E+00_JPRB, 0.82949E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.97308E+00_JPRB, 0.10153E+01_JPRB, 0.10269E+01_JPRB, 0.12071E+01_JPRB, 0.13794E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.15935E+01_JPRB, 0.15747E+01_JPRB, 0.16677E+01_JPRB, 0.17672E+01_JPRB, 0.18444E+01_JPRB, 0.18174E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.17011E+01_JPRB, 0.17791E+01_JPRB, 0.17820E+01_JPRB, 0.17758E+01_JPRB, 0.17062E+01_JPRB, 0.16539E+01_JPRB, 0.15835E+01_JPRB ,&
&  0.15646E+01_JPRB, 0.15498E+01_JPRB, 0.14876E+01_JPRB, 0.14122E+01_JPRB, 0.13254E+01_JPRB, 0.12334E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.94523E+00_JPRB, 0.90504E+00_JPRB, 0.90162E+00_JPRB, 0.90855E+00_JPRB, 0.88243E+00_JPRB, 0.87513E+00_JPRB ,&
&  0.87096E+00_JPRB, 0.87087E+00_JPRB, 0.89035E+00_JPRB, 0.87017E+00_JPRB, 0.83990E+00_JPRB, 0.80728E+00_JPRB, 0.78909E+00_JPRB ,&
&  0.77044E+00_JPRB, 0.77925E+00_JPRB, 0.78402E+00_JPRB, 0.77519E+00_JPRB, 0.76286E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.77843E+00_JPRB, 0.76796E+00_JPRB, 0.76283E+00_JPRB, 0.77079E+00_JPRB, 0.78445E+00_JPRB, 0.80103E+00_JPRB, 0.82400E+00_JPRB ,&
&  0.84618E+00_JPRB, 0.85533E+00_JPRB, 0.84123E+00_JPRB, 0.81748E+00_JPRB, 0.79711E+00_JPRB, 0.78292E+00_JPRB, 0.74240E+00_JPRB ,&
&  0.68642E+00_JPRB, 0.68085E+00_JPRB, 0.67121E+00_JPRB, 0.67441E+00_JPRB, 0.68779E+00_JPRB, 0.69785E+00_JPRB, 0.70840E+00_JPRB ,&
&  0.72224E+00_JPRB, 0.72877E+00_JPRB, 0.73274E+00_JPRB, 0.73573E+00_JPRB, 0.74109E+00_JPRB, 0.75650E+00_JPRB, 0.76676E+00_JPRB ,&
&  0.74106E+00_JPRB, 0.72445E+00_JPRB, 0.72375E+00_JPRB, 0.73318E+00_JPRB, 0.75187E+00_JPRB, 0.77243E+00_JPRB, 0.78659E+00_JPRB ,&
&  0.79707E+00_JPRB, 0.80249E+00_JPRB, 0.80195E+00_JPRB, 0.79821E+00_JPRB, 0.79191E+00_JPRB, 0.78425E+00_JPRB, 0.77941E+00_JPRB ,&
&  0.77550E+00_JPRB, 0.77568E+00_JPRB, 0.78145E+00_JPRB, 0.78696E+00_JPRB, 0.79394E+00_JPRB, 0.84374E+00_JPRB, 0.96167E+00_JPRB ,&
&  0.10969E+01_JPRB, 0.12374E+01_JPRB, 0.12811E+01_JPRB, 0.13929E+01_JPRB, 0.15089E+01_JPRB, 0.16333E+01_JPRB, 0.17205E+01_JPRB ,&
&  0.17955E+01_JPRB, 0.17351E+01_JPRB, 0.19359E+01_JPRB, 0.20354E+01_JPRB, 0.20377E+01_JPRB, 0.20797E+01_JPRB, 0.19799E+01_JPRB ,&
&  0.19113E+01_JPRB, 0.18442E+01_JPRB, 0.19106E+01_JPRB, 0.19161E+01_JPRB, 0.18773E+01_JPRB, 0.18946E+01_JPRB, 0.18356E+01_JPRB ,&
&  0.17992E+01_JPRB, 0.17433E+01_JPRB, 0.15386E+01_JPRB, 0.13751E+01_JPRB, 0.12061E+01_JPRB, 0.12473E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.95309E+00_JPRB, 0.95319E+00_JPRB, 0.94863E+00_JPRB, 0.96753E+00_JPRB, 0.97916E+00_JPRB, 0.94753E+00_JPRB ,&
&  0.95026E+00_JPRB, 0.96989E+00_JPRB, 0.94612E+00_JPRB, 0.92069E+00_JPRB, 0.88970E+00_JPRB, 0.84827E+00_JPRB, 0.82316E+00_JPRB ,&
&  0.82527E+00_JPRB, 0.83004E+00_JPRB, 0.81971E+00_JPRB, 0.80089E+00_JPRB, 0.78828E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.82026E+00_JPRB, 0.80959E+00_JPRB, 0.80521E+00_JPRB, 0.81136E+00_JPRB, 0.82392E+00_JPRB, 0.84518E+00_JPRB, 0.86936E+00_JPRB ,&
&  0.88695E+00_JPRB, 0.88646E+00_JPRB, 0.86220E+00_JPRB, 0.84142E+00_JPRB, 0.82816E+00_JPRB, 0.81573E+00_JPRB, 0.81223E+00_JPRB ,&
&  0.74315E+00_JPRB, 0.71429E+00_JPRB, 0.68303E+00_JPRB, 0.68279E+00_JPRB, 0.70795E+00_JPRB, 0.71733E+00_JPRB, 0.72812E+00_JPRB ,&
&  0.73848E+00_JPRB, 0.75167E+00_JPRB, 0.75627E+00_JPRB, 0.76104E+00_JPRB, 0.76966E+00_JPRB, 0.79207E+00_JPRB, 0.81555E+00_JPRB ,&
&  0.79511E+00_JPRB, 0.77685E+00_JPRB, 0.78158E+00_JPRB, 0.79663E+00_JPRB, 0.81013E+00_JPRB, 0.83232E+00_JPRB, 0.85698E+00_JPRB ,&
&  0.86376E+00_JPRB, 0.86045E+00_JPRB, 0.85093E+00_JPRB, 0.84402E+00_JPRB, 0.83923E+00_JPRB, 0.83433E+00_JPRB, 0.83623E+00_JPRB ,&
&  0.84047E+00_JPRB, 0.84421E+00_JPRB, 0.85007E+00_JPRB, 0.85282E+00_JPRB, 0.85292E+00_JPRB, 0.93100E+00_JPRB, 0.10864E+01_JPRB ,&
&  0.12863E+01_JPRB, 0.13322E+01_JPRB, 0.14188E+01_JPRB, 0.18159E+01_JPRB, 0.20341E+01_JPRB, 0.21272E+01_JPRB, 0.21079E+01_JPRB ,&
&  0.20688E+01_JPRB, 0.19674E+01_JPRB, 0.19692E+01_JPRB, 0.20234E+01_JPRB, 0.20208E+01_JPRB, 0.21314E+01_JPRB, 0.20851E+01_JPRB ,&
&  0.19656E+01_JPRB, 0.19319E+01_JPRB, 0.19345E+01_JPRB, 0.19176E+01_JPRB, 0.18688E+01_JPRB, 0.18875E+01_JPRB, 0.18560E+01_JPRB ,&
&  0.17815E+01_JPRB, 0.15774E+01_JPRB, 0.12654E+01_JPRB, 0.12028E+01_JPRB, 0.10272E+01_JPRB, 0.10712E+01_JPRB, 0.12397E+01_JPRB ,&
&  0.12498E+01_JPRB, 0.11799E+01_JPRB, 0.11119E+01_JPRB, 0.10822E+01_JPRB, 0.10798E+01_JPRB, 0.11558E+01_JPRB, 0.11188E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.10683E+01_JPRB, 0.10612E+01_JPRB, 0.10296E+01_JPRB, 0.98512E+00_JPRB, 0.94723E+00_JPRB, 0.92260E+00_JPRB ,&
&  0.91531E+00_JPRB, 0.91047E+00_JPRB, 0.89611E+00_JPRB, 0.85773E+00_JPRB, 0.83125E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.89162E+00_JPRB, 0.88024E+00_JPRB, 0.86892E+00_JPRB, 0.87190E+00_JPRB, 0.88342E+00_JPRB, 0.90161E+00_JPRB, 0.91965E+00_JPRB ,&
&  0.92500E+00_JPRB, 0.90992E+00_JPRB, 0.88020E+00_JPRB, 0.86710E+00_JPRB, 0.85620E+00_JPRB, 0.83968E+00_JPRB, 0.83438E+00_JPRB ,&
&  0.84537E+00_JPRB, 0.77967E+00_JPRB, 0.73248E+00_JPRB, 0.69756E+00_JPRB, 0.71161E+00_JPRB, 0.71988E+00_JPRB, 0.73636E+00_JPRB ,&
&  0.74913E+00_JPRB, 0.77380E+00_JPRB, 0.78452E+00_JPRB, 0.79235E+00_JPRB, 0.80845E+00_JPRB, 0.84311E+00_JPRB, 0.87615E+00_JPRB ,&
&  0.87898E+00_JPRB, 0.88218E+00_JPRB, 0.89680E+00_JPRB, 0.92353E+00_JPRB, 0.94334E+00_JPRB, 0.91541E+00_JPRB, 0.93718E+00_JPRB ,&
&  0.93357E+00_JPRB, 0.92278E+00_JPRB, 0.90547E+00_JPRB, 0.90327E+00_JPRB, 0.90686E+00_JPRB, 0.91607E+00_JPRB, 0.92854E+00_JPRB ,&
&  0.94398E+00_JPRB, 0.94491E+00_JPRB, 0.92175E+00_JPRB, 0.91462E+00_JPRB, 0.92920E+00_JPRB, 0.10525E+01_JPRB, 0.11475E+01_JPRB ,&
&  0.13072E+01_JPRB, 0.13197E+01_JPRB, 0.13383E+01_JPRB, 0.17124E+01_JPRB, 0.20832E+01_JPRB, 0.22441E+01_JPRB, 0.23006E+01_JPRB ,&
&  0.22721E+01_JPRB, 0.22067E+01_JPRB, 0.21670E+01_JPRB, 0.20868E+01_JPRB, 0.21028E+01_JPRB, 0.21662E+01_JPRB, 0.21299E+01_JPRB ,&
&  0.20090E+01_JPRB, 0.19111E+01_JPRB, 0.18834E+01_JPRB, 0.18169E+01_JPRB, 0.17624E+01_JPRB, 0.16580E+01_JPRB, 0.17019E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.11570E+01_JPRB, 0.10079E+01_JPRB, 0.11288E+01_JPRB, 0.10136E+01_JPRB, 0.10474E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.11780E+01_JPRB, 0.12201E+01_JPRB, 0.11974E+01_JPRB, 0.12333E+01_JPRB, 0.13055E+01_JPRB, 0.12849E+01_JPRB ,&
&  0.12513E+01_JPRB, 0.11942E+01_JPRB, 0.11974E+01_JPRB, 0.11619E+01_JPRB, 0.11091E+01_JPRB, 0.10729E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.99127E+00_JPRB, 0.97965E+00_JPRB, 0.94063E+00_JPRB, 0.90599E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.99258E+00_JPRB, 0.97663E+00_JPRB, 0.96488E+00_JPRB, 0.96282E+00_JPRB, 0.96554E+00_JPRB, 0.97550E+00_JPRB, 0.97774E+00_JPRB ,&
&  0.96617E+00_JPRB, 0.93065E+00_JPRB, 0.89827E+00_JPRB, 0.89378E+00_JPRB, 0.87850E+00_JPRB, 0.85998E+00_JPRB, 0.86143E+00_JPRB ,&
&  0.88050E+00_JPRB, 0.86736E+00_JPRB, 0.77656E+00_JPRB, 0.73205E+00_JPRB, 0.69745E+00_JPRB, 0.71052E+00_JPRB, 0.72610E+00_JPRB ,&
&  0.75543E+00_JPRB, 0.77714E+00_JPRB, 0.79155E+00_JPRB, 0.81212E+00_JPRB, 0.85145E+00_JPRB, 0.90189E+00_JPRB, 0.95734E+00_JPRB ,&
&  0.98319E+00_JPRB, 0.10102E+01_JPRB, 0.10004E+01_JPRB, 0.10488E+01_JPRB, 0.10561E+01_JPRB, 0.10139E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.10119E+01_JPRB, 0.99517E+00_JPRB, 0.99606E+00_JPRB, 0.99777E+00_JPRB, 0.98423E+00_JPRB, 0.96508E+00_JPRB ,&
&  0.95182E+00_JPRB, 0.94627E+00_JPRB, 0.93832E+00_JPRB, 0.92848E+00_JPRB, 0.95991E+00_JPRB, 0.10462E+01_JPRB, 0.10970E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.10940E+01_JPRB, 0.10117E+01_JPRB, 0.13563E+01_JPRB, 0.18071E+01_JPRB, 0.19739E+01_JPRB, 0.19622E+01_JPRB ,&
&  0.21252E+01_JPRB, 0.21703E+01_JPRB, 0.22170E+01_JPRB, 0.22838E+01_JPRB, 0.22953E+01_JPRB, 0.22867E+01_JPRB, 0.22178E+01_JPRB ,&
&  0.20476E+01_JPRB, 0.19300E+01_JPRB, 0.18713E+01_JPRB, 0.19012E+01_JPRB, 0.18328E+01_JPRB, 0.17927E+01_JPRB, 0.17707E+01_JPRB ,&
&  0.15986E+01_JPRB, 0.14496E+01_JPRB, 0.11379E+01_JPRB, 0.11565E+01_JPRB, 0.98423E+00_JPRB, 0.99401E+00_JPRB, 0.11105E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.11891E+01_JPRB, 0.12213E+01_JPRB, 0.12086E+01_JPRB, 0.13873E+01_JPRB, 0.14736E+01_JPRB, 0.14784E+01_JPRB ,&
&  0.14440E+01_JPRB, 0.13078E+01_JPRB, 0.13344E+01_JPRB, 0.13480E+01_JPRB, 0.13205E+01_JPRB, 0.12895E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.11466E+01_JPRB, 0.11111E+01_JPRB, 0.10605E+01_JPRB, 0.10181E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.11038E+01_JPRB, 0.10689E+01_JPRB, 0.10465E+01_JPRB, 0.10372E+01_JPRB, 0.10344E+01_JPRB, 0.10315E+01_JPRB, 0.10217E+01_JPRB ,&
&  0.99861E+00_JPRB, 0.95111E+00_JPRB, 0.92907E+00_JPRB, 0.92014E+00_JPRB, 0.89620E+00_JPRB, 0.87820E+00_JPRB, 0.88701E+00_JPRB ,&
&  0.91110E+00_JPRB, 0.87110E+00_JPRB, 0.76403E+00_JPRB, 0.70628E+00_JPRB, 0.67387E+00_JPRB, 0.67857E+00_JPRB, 0.70192E+00_JPRB ,&
&  0.73327E+00_JPRB, 0.75834E+00_JPRB, 0.78787E+00_JPRB, 0.82704E+00_JPRB, 0.88904E+00_JPRB, 0.95318E+00_JPRB, 0.10581E+01_JPRB ,&
&  0.11869E+01_JPRB, 0.11043E+01_JPRB, 0.11300E+01_JPRB, 0.11228E+01_JPRB, 0.11491E+01_JPRB, 0.11283E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10322E+01_JPRB, 0.10079E+01_JPRB, 0.98129E+00_JPRB, 0.95728E+00_JPRB, 0.94362E+00_JPRB, 0.94601E+00_JPRB ,&
&  0.97053E+00_JPRB, 0.97978E+00_JPRB, 0.96551E+00_JPRB, 0.97578E+00_JPRB, 0.10592E+01_JPRB, 0.10868E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.10666E+01_JPRB, 0.11302E+01_JPRB, 0.14330E+01_JPRB, 0.16673E+01_JPRB, 0.18371E+01_JPRB, 0.19917E+01_JPRB, 0.20448E+01_JPRB ,&
&  0.21526E+01_JPRB, 0.20995E+01_JPRB, 0.21440E+01_JPRB, 0.19673E+01_JPRB, 0.22119E+01_JPRB, 0.22225E+01_JPRB, 0.21855E+01_JPRB ,&
&  0.20921E+01_JPRB, 0.20653E+01_JPRB, 0.19941E+01_JPRB, 0.20469E+01_JPRB, 0.20388E+01_JPRB, 0.20595E+01_JPRB, 0.16756E+01_JPRB ,&
&  0.14183E+01_JPRB, 0.15148E+01_JPRB, 0.13851E+01_JPRB, 0.12403E+01_JPRB, 0.11062E+01_JPRB, 0.11203E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.11835E+01_JPRB, 0.11774E+01_JPRB, 0.12422E+01_JPRB, 0.14613E+01_JPRB, 0.16726E+01_JPRB, 0.17003E+01_JPRB ,&
&  0.15721E+01_JPRB, 0.16088E+01_JPRB, 0.16269E+01_JPRB, 0.16121E+01_JPRB, 0.16327E+01_JPRB, 0.15962E+01_JPRB, 0.15277E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.13708E+01_JPRB, 0.13016E+01_JPRB, 0.12286E+01_JPRB, 0.11548E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.12215E+01_JPRB, 0.11819E+01_JPRB, 0.11472E+01_JPRB, 0.11228E+01_JPRB, 0.11008E+01_JPRB, 0.10851E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.96903E+00_JPRB, 0.96103E+00_JPRB, 0.94633E+00_JPRB, 0.92469E+00_JPRB, 0.90975E+00_JPRB, 0.92133E+00_JPRB ,&
&  0.94594E+00_JPRB, 0.86927E+00_JPRB, 0.76174E+00_JPRB, 0.74799E+00_JPRB, 0.71835E+00_JPRB, 0.68157E+00_JPRB, 0.68847E+00_JPRB ,&
&  0.71473E+00_JPRB, 0.74525E+00_JPRB, 0.77817E+00_JPRB, 0.85019E+00_JPRB, 0.94497E+00_JPRB, 0.10532E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.12370E+01_JPRB, 0.12947E+01_JPRB, 0.12492E+01_JPRB, 0.11945E+01_JPRB, 0.11366E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.10203E+01_JPRB, 0.99201E+00_JPRB, 0.97196E+00_JPRB, 0.96868E+00_JPRB, 0.96748E+00_JPRB, 0.97141E+00_JPRB ,&
&  0.98084E+00_JPRB, 0.97312E+00_JPRB, 0.95250E+00_JPRB, 0.10330E+01_JPRB, 0.96682E+00_JPRB, 0.94892E+00_JPRB, 0.96663E+00_JPRB ,&
&  0.12130E+01_JPRB, 0.14327E+01_JPRB, 0.15777E+01_JPRB, 0.18999E+01_JPRB, 0.16756E+01_JPRB, 0.18146E+01_JPRB, 0.21803E+01_JPRB ,&
&  0.22587E+01_JPRB, 0.22559E+01_JPRB, 0.23954E+01_JPRB, 0.20015E+01_JPRB, 0.18291E+01_JPRB, 0.24207E+01_JPRB, 0.24506E+01_JPRB ,&
&  0.23599E+01_JPRB, 0.23225E+01_JPRB, 0.22942E+01_JPRB, 0.25234E+01_JPRB, 0.18037E+01_JPRB, 0.15584E+01_JPRB, 0.12798E+01_JPRB ,&
&  0.12814E+01_JPRB, 0.12407E+01_JPRB, 0.13944E+01_JPRB, 0.13311E+01_JPRB, 0.13289E+01_JPRB, 0.12857E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.11784E+01_JPRB, 0.11735E+01_JPRB, 0.11935E+01_JPRB, 0.13874E+01_JPRB, 0.17650E+01_JPRB, 0.17825E+01_JPRB, 0.19449E+01_JPRB ,&
&  0.19870E+01_JPRB, 0.21025E+01_JPRB, 0.19920E+01_JPRB, 0.20030E+01_JPRB, 0.19621E+01_JPRB, 0.18772E+01_JPRB, 0.17499E+01_JPRB ,&
&  0.16497E+01_JPRB, 0.15539E+01_JPRB, 0.14512E+01_JPRB, 0.13583E+01_JPRB, 0.12807E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.13091E+01_JPRB, 0.12544E+01_JPRB, 0.12057E+01_JPRB, 0.11637E+01_JPRB, 0.11394E+01_JPRB, 0.11173E+01_JPRB, 0.10846E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.97036E+00_JPRB, 0.96974E+00_JPRB, 0.93576E+00_JPRB, 0.91560E+00_JPRB, 0.91647E+00_JPRB, 0.94798E+00_JPRB ,&
&  0.97359E+00_JPRB, 0.95750E+00_JPRB, 0.83261E+00_JPRB, 0.77265E+00_JPRB, 0.77433E+00_JPRB, 0.72260E+00_JPRB, 0.69930E+00_JPRB ,&
&  0.72929E+00_JPRB, 0.76387E+00_JPRB, 0.82987E+00_JPRB, 0.95685E+00_JPRB, 0.10938E+01_JPRB, 0.12259E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.13843E+01_JPRB, 0.13699E+01_JPRB, 0.13140E+01_JPRB, 0.12379E+01_JPRB, 0.11503E+01_JPRB, 0.10875E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.10130E+01_JPRB, 0.99598E+00_JPRB, 0.99239E+00_JPRB, 0.98006E+00_JPRB, 0.98323E+00_JPRB, 0.95044E+00_JPRB, 0.94111E+00_JPRB ,&
&  0.94935E+00_JPRB, 0.95966E+00_JPRB, 0.97836E+00_JPRB, 0.11064E+01_JPRB, 0.10696E+01_JPRB, 0.10070E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.13529E+01_JPRB, 0.15621E+01_JPRB, 0.18075E+01_JPRB, 0.21798E+01_JPRB, 0.23070E+01_JPRB, 0.25126E+01_JPRB ,&
&  0.20729E+01_JPRB, 0.18656E+01_JPRB, 0.19260E+01_JPRB, 0.18660E+01_JPRB, 0.23730E+01_JPRB, 0.32855E+01_JPRB, 0.33543E+01_JPRB ,&
&  0.31988E+01_JPRB, 0.30371E+01_JPRB, 0.30125E+01_JPRB, 0.24546E+01_JPRB, 0.17932E+01_JPRB, 0.14383E+01_JPRB, 0.16670E+01_JPRB ,&
&  0.17505E+01_JPRB, 0.18550E+01_JPRB, 0.17077E+01_JPRB, 0.14703E+01_JPRB, 0.13041E+01_JPRB, 0.15076E+01_JPRB, 0.14695E+01_JPRB ,&
&  0.13489E+01_JPRB, 0.12918E+01_JPRB, 0.15931E+01_JPRB, 0.20174E+01_JPRB, 0.22315E+01_JPRB, 0.22976E+01_JPRB, 0.26554E+01_JPRB ,&
&  0.27739E+01_JPRB, 0.28641E+01_JPRB, 0.27451E+01_JPRB, 0.25703E+01_JPRB, 0.23628E+01_JPRB, 0.21349E+01_JPRB, 0.19531E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.16715E+01_JPRB, 0.15484E+01_JPRB, 0.14456E+01_JPRB, 0.13697E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.13764E+01_JPRB, 0.13046E+01_JPRB, 0.12422E+01_JPRB, 0.12029E+01_JPRB, 0.11695E+01_JPRB, 0.11314E+01_JPRB, 0.10780E+01_JPRB ,&
&  0.99907E+00_JPRB, 0.97895E+00_JPRB, 0.96863E+00_JPRB, 0.93095E+00_JPRB, 0.92001E+00_JPRB, 0.92938E+00_JPRB, 0.95784E+00_JPRB ,&
&  0.99259E+00_JPRB, 0.10605E+01_JPRB, 0.92236E+00_JPRB, 0.86494E+00_JPRB, 0.85100E+00_JPRB, 0.82591E+00_JPRB, 0.74899E+00_JPRB ,&
&  0.77704E+00_JPRB, 0.82697E+00_JPRB, 0.93021E+00_JPRB, 0.10994E+01_JPRB, 0.13031E+01_JPRB, 0.13594E+01_JPRB, 0.13847E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.13643E+01_JPRB, 0.12905E+01_JPRB, 0.12151E+01_JPRB, 0.11283E+01_JPRB, 0.10828E+01_JPRB, 0.10594E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10583E+01_JPRB, 0.10563E+01_JPRB, 0.10726E+01_JPRB, 0.10386E+01_JPRB, 0.99257E+00_JPRB, 0.97469E+00_JPRB ,&
&  0.96182E+00_JPRB, 0.97439E+00_JPRB, 0.10149E+01_JPRB, 0.11599E+01_JPRB, 0.12436E+01_JPRB, 0.11954E+01_JPRB, 0.13285E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.16313E+01_JPRB, 0.17382E+01_JPRB, 0.19647E+01_JPRB, 0.21924E+01_JPRB, 0.23158E+01_JPRB, 0.25216E+01_JPRB ,&
&  0.23857E+01_JPRB, 0.25575E+01_JPRB, 0.31898E+01_JPRB, 0.39340E+01_JPRB, 0.34563E+01_JPRB, 0.34879E+01_JPRB, 0.48160E+01_JPRB ,&
&  0.35200E+01_JPRB, 0.34657E+01_JPRB, 0.35974E+01_JPRB, 0.34922E+01_JPRB, 0.15357E+01_JPRB, 0.12721E+01_JPRB, 0.18068E+01_JPRB ,&
&  0.18946E+01_JPRB, 0.12986E+01_JPRB, 0.10375E+01_JPRB, 0.11917E+01_JPRB, 0.10056E+01_JPRB, 0.11867E+01_JPRB, 0.16357E+01_JPRB ,&
&  0.16989E+01_JPRB, 0.17285E+01_JPRB, 0.23369E+01_JPRB, 0.26376E+01_JPRB, 0.26087E+01_JPRB, 0.34720E+01_JPRB, 0.36130E+01_JPRB ,&
&  0.36575E+01_JPRB, 0.33282E+01_JPRB, 0.33082E+01_JPRB, 0.30174E+01_JPRB, 0.26802E+01_JPRB, 0.23711E+01_JPRB, 0.21214E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.17331E+01_JPRB, 0.16212E+01_JPRB, 0.15163E+01_JPRB, 0.14364E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.13467E+01_JPRB, 0.12778E+01_JPRB, 0.12106E+01_JPRB, 0.11680E+01_JPRB, 0.11420E+01_JPRB, 0.11064E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.98734E+00_JPRB, 0.99963E+00_JPRB, 0.96480E+00_JPRB, 0.93588E+00_JPRB, 0.91729E+00_JPRB, 0.92354E+00_JPRB, 0.95536E+00_JPRB ,&
&  0.98519E+00_JPRB, 0.10330E+01_JPRB, 0.10264E+01_JPRB, 0.97344E+00_JPRB, 0.95406E+00_JPRB, 0.91617E+00_JPRB, 0.86168E+00_JPRB ,&
&  0.89151E+00_JPRB, 0.97744E+00_JPRB, 0.11015E+01_JPRB, 0.12694E+01_JPRB, 0.13484E+01_JPRB, 0.13643E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.14014E+01_JPRB, 0.13259E+01_JPRB, 0.12631E+01_JPRB, 0.11978E+01_JPRB, 0.11415E+01_JPRB, 0.11118E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10801E+01_JPRB, 0.10868E+01_JPRB, 0.10513E+01_JPRB, 0.10118E+01_JPRB, 0.98259E+00_JPRB, 0.96086E+00_JPRB ,&
&  0.95822E+00_JPRB, 0.10109E+01_JPRB, 0.10964E+01_JPRB, 0.12287E+01_JPRB, 0.14782E+01_JPRB, 0.15756E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.13589E+01_JPRB, 0.14879E+01_JPRB, 0.19149E+01_JPRB, 0.19116E+01_JPRB, 0.18749E+01_JPRB, 0.20008E+01_JPRB, 0.20829E+01_JPRB ,&
&  0.24013E+01_JPRB, 0.30854E+01_JPRB, 0.33287E+01_JPRB, 0.42016E+01_JPRB, 0.45081E+01_JPRB, 0.33409E+01_JPRB, 0.38390E+01_JPRB ,&
&  0.37933E+01_JPRB, 0.33076E+01_JPRB, 0.25375E+01_JPRB, 0.18997E+01_JPRB, 0.20899E+01_JPRB, 0.18734E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.10246E+01_JPRB, 0.98176E+00_JPRB, 0.98036E+00_JPRB, 0.96869E+00_JPRB, 0.12076E+01_JPRB, 0.16039E+01_JPRB ,&
&  0.21514E+01_JPRB, 0.25523E+01_JPRB, 0.29435E+01_JPRB, 0.30470E+01_JPRB, 0.28223E+01_JPRB, 0.37534E+01_JPRB, 0.41574E+01_JPRB ,&
&  0.34437E+01_JPRB, 0.31130E+01_JPRB, 0.30262E+01_JPRB, 0.27544E+01_JPRB, 0.24826E+01_JPRB, 0.22414E+01_JPRB, 0.20188E+01_JPRB ,&
&  0.18326E+01_JPRB, 0.17012E+01_JPRB, 0.16025E+01_JPRB, 0.15091E+01_JPRB, 0.14236E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.12786E+01_JPRB, 0.12313E+01_JPRB, 0.11986E+01_JPRB, 0.11698E+01_JPRB, 0.11349E+01_JPRB, 0.11033E+01_JPRB, 0.10349E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.10010E+01_JPRB, 0.94871E+00_JPRB, 0.91184E+00_JPRB, 0.89416E+00_JPRB, 0.90563E+00_JPRB, 0.93723E+00_JPRB ,&
&  0.96463E+00_JPRB, 0.10178E+01_JPRB, 0.11296E+01_JPRB, 0.11402E+01_JPRB, 0.12342E+01_JPRB, 0.11351E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11410E+01_JPRB, 0.12308E+01_JPRB, 0.13556E+01_JPRB, 0.14262E+01_JPRB, 0.14142E+01_JPRB, 0.14630E+01_JPRB, 0.14278E+01_JPRB ,&
&  0.13585E+01_JPRB, 0.13256E+01_JPRB, 0.12465E+01_JPRB, 0.11863E+01_JPRB, 0.11553E+01_JPRB, 0.11158E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10933E+01_JPRB, 0.10574E+01_JPRB, 0.10454E+01_JPRB, 0.10407E+01_JPRB, 0.10221E+01_JPRB, 0.10080E+01_JPRB ,&
&  0.10373E+01_JPRB, 0.11193E+01_JPRB, 0.12259E+01_JPRB, 0.13521E+01_JPRB, 0.13774E+01_JPRB, 0.11980E+01_JPRB, 0.13508E+01_JPRB ,&
&  0.14832E+01_JPRB, 0.17503E+01_JPRB, 0.18187E+01_JPRB, 0.20414E+01_JPRB, 0.19154E+01_JPRB, 0.18697E+01_JPRB, 0.20975E+01_JPRB ,&
&  0.26076E+01_JPRB, 0.37203E+01_JPRB, 0.33861E+01_JPRB, 0.39987E+01_JPRB, 0.53668E+01_JPRB, 0.40027E+01_JPRB, 0.33601E+01_JPRB ,&
&  0.36233E+01_JPRB, 0.30402E+01_JPRB, 0.25484E+01_JPRB, 0.22952E+01_JPRB, 0.33502E+01_JPRB, 0.40435E+01_JPRB, 0.17135E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.12265E+01_JPRB, 0.11742E+01_JPRB, 0.10888E+01_JPRB, 0.10830E+01_JPRB, 0.13800E+01_JPRB, 0.20831E+01_JPRB ,&
&  0.24386E+01_JPRB, 0.31885E+01_JPRB, 0.34971E+01_JPRB, 0.34233E+01_JPRB, 0.28826E+01_JPRB, 0.33345E+01_JPRB, 0.31379E+01_JPRB ,&
&  0.33445E+01_JPRB, 0.31860E+01_JPRB, 0.29090E+01_JPRB, 0.26316E+01_JPRB, 0.23603E+01_JPRB, 0.21123E+01_JPRB, 0.18995E+01_JPRB ,&
&  0.17187E+01_JPRB, 0.15882E+01_JPRB, 0.14906E+01_JPRB, 0.14143E+01_JPRB, 0.13393E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.12744E+01_JPRB, 0.12328E+01_JPRB, 0.11944E+01_JPRB, 0.11521E+01_JPRB, 0.11267E+01_JPRB, 0.10879E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.98837E+00_JPRB, 0.93150E+00_JPRB, 0.89775E+00_JPRB, 0.88985E+00_JPRB, 0.89954E+00_JPRB, 0.92339E+00_JPRB ,&
&  0.96765E+00_JPRB, 0.10133E+01_JPRB, 0.11409E+01_JPRB, 0.14375E+01_JPRB, 0.15997E+01_JPRB, 0.14982E+01_JPRB, 0.16019E+01_JPRB ,&
&  0.15312E+01_JPRB, 0.15562E+01_JPRB, 0.16270E+01_JPRB, 0.15702E+01_JPRB, 0.15374E+01_JPRB, 0.15504E+01_JPRB, 0.14301E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.13305E+01_JPRB, 0.12558E+01_JPRB, 0.12085E+01_JPRB, 0.11664E+01_JPRB, 0.11506E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.11281E+01_JPRB, 0.11030E+01_JPRB, 0.10896E+01_JPRB, 0.10954E+01_JPRB, 0.11059E+01_JPRB, 0.11048E+01_JPRB, 0.11259E+01_JPRB ,&
&  0.12032E+01_JPRB, 0.12828E+01_JPRB, 0.13896E+01_JPRB, 0.14806E+01_JPRB, 0.12319E+01_JPRB, 0.14072E+01_JPRB, 0.14518E+01_JPRB ,&
&  0.15033E+01_JPRB, 0.15832E+01_JPRB, 0.15604E+01_JPRB, 0.17236E+01_JPRB, 0.18984E+01_JPRB, 0.18983E+01_JPRB, 0.21063E+01_JPRB ,&
&  0.30208E+01_JPRB, 0.31245E+01_JPRB, 0.33175E+01_JPRB, 0.40623E+01_JPRB, 0.46029E+01_JPRB, 0.61265E+01_JPRB, 0.31052E+01_JPRB ,&
&  0.23506E+01_JPRB, 0.28270E+01_JPRB, 0.27827E+01_JPRB, 0.28788E+01_JPRB, 0.37461E+01_JPRB, 0.56028E+01_JPRB, 0.46915E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.12804E+01_JPRB, 0.14805E+01_JPRB, 0.14873E+01_JPRB, 0.13380E+01_JPRB, 0.17236E+01_JPRB, 0.35709E+01_JPRB ,&
&  0.33667E+01_JPRB, 0.38308E+01_JPRB, 0.37491E+01_JPRB, 0.32103E+01_JPRB, 0.28154E+01_JPRB, 0.30584E+01_JPRB, 0.32879E+01_JPRB ,&
&  0.31506E+01_JPRB, 0.28572E+01_JPRB, 0.26007E+01_JPRB, 0.23828E+01_JPRB, 0.21236E+01_JPRB, 0.18705E+01_JPRB, 0.16593E+01_JPRB ,&
&  0.15132E+01_JPRB, 0.14317E+01_JPRB, 0.13879E+01_JPRB, 0.13527E+01_JPRB, 0.13143E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.12168E+01_JPRB, 0.11823E+01_JPRB, 0.11549E+01_JPRB, 0.11347E+01_JPRB, 0.11291E+01_JPRB, 0.10910E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.99106E+00_JPRB, 0.93760E+00_JPRB, 0.89789E+00_JPRB, 0.89814E+00_JPRB, 0.91681E+00_JPRB, 0.93658E+00_JPRB ,&
&  0.96024E+00_JPRB, 0.10190E+01_JPRB, 0.11823E+01_JPRB, 0.15632E+01_JPRB, 0.18275E+01_JPRB, 0.17497E+01_JPRB, 0.18568E+01_JPRB ,&
&  0.20095E+01_JPRB, 0.19078E+01_JPRB, 0.17192E+01_JPRB, 0.15939E+01_JPRB, 0.16293E+01_JPRB, 0.16052E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.13758E+01_JPRB, 0.13107E+01_JPRB, 0.12631E+01_JPRB, 0.12371E+01_JPRB, 0.12231E+01_JPRB, 0.12196E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.11796E+01_JPRB, 0.11818E+01_JPRB, 0.11890E+01_JPRB, 0.12248E+01_JPRB, 0.12296E+01_JPRB, 0.12975E+01_JPRB ,&
&  0.13989E+01_JPRB, 0.14707E+01_JPRB, 0.15407E+01_JPRB, 0.14681E+01_JPRB, 0.14755E+01_JPRB, 0.14791E+01_JPRB, 0.15366E+01_JPRB ,&
&  0.15359E+01_JPRB, 0.15808E+01_JPRB, 0.16642E+01_JPRB, 0.17581E+01_JPRB, 0.19605E+01_JPRB, 0.21425E+01_JPRB, 0.24677E+01_JPRB ,&
&  0.31101E+01_JPRB, 0.34504E+01_JPRB, 0.34507E+01_JPRB, 0.37670E+01_JPRB, 0.42903E+01_JPRB, 0.52271E+01_JPRB, 0.40014E+01_JPRB ,&
&  0.30039E+01_JPRB, 0.25324E+01_JPRB, 0.29203E+01_JPRB, 0.39721E+01_JPRB, 0.47146E+01_JPRB, 0.61198E+01_JPRB, 0.81984E+01_JPRB ,&
&  0.89130E+01_JPRB, 0.46739E+01_JPRB, 0.36633E+01_JPRB, 0.43700E+01_JPRB, 0.32321E+01_JPRB, 0.26846E+01_JPRB, 0.33835E+01_JPRB ,&
&  0.37831E+01_JPRB, 0.39415E+01_JPRB, 0.36873E+01_JPRB, 0.29601E+01_JPRB, 0.28363E+01_JPRB, 0.31192E+01_JPRB, 0.30550E+01_JPRB ,&
&  0.27066E+01_JPRB, 0.24475E+01_JPRB, 0.21807E+01_JPRB, 0.18985E+01_JPRB, 0.16665E+01_JPRB, 0.15021E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.13619E+01_JPRB, 0.13399E+01_JPRB, 0.13230E+01_JPRB, 0.12922E+01_JPRB, 0.12546E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.11600E+01_JPRB, 0.11509E+01_JPRB, 0.11584E+01_JPRB, 0.11609E+01_JPRB, 0.11451E+01_JPRB, 0.10994E+01_JPRB, 0.10970E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.10246E+01_JPRB, 0.95254E+00_JPRB, 0.92081E+00_JPRB, 0.93115E+00_JPRB, 0.96023E+00_JPRB, 0.97732E+00_JPRB ,&
&  0.10087E+01_JPRB, 0.11077E+01_JPRB, 0.13882E+01_JPRB, 0.17404E+01_JPRB, 0.21025E+01_JPRB, 0.27197E+01_JPRB, 0.19393E+01_JPRB ,&
&  0.20437E+01_JPRB, 0.19461E+01_JPRB, 0.16708E+01_JPRB, 0.16229E+01_JPRB, 0.16736E+01_JPRB, 0.16894E+01_JPRB, 0.15617E+01_JPRB ,&
&  0.14850E+01_JPRB, 0.14609E+01_JPRB, 0.14208E+01_JPRB, 0.13799E+01_JPRB, 0.13534E+01_JPRB, 0.13670E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.13563E+01_JPRB, 0.13566E+01_JPRB, 0.13299E+01_JPRB, 0.13625E+01_JPRB, 0.14019E+01_JPRB, 0.14239E+01_JPRB, 0.15127E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.16232E+01_JPRB, 0.16349E+01_JPRB, 0.15514E+01_JPRB, 0.15954E+01_JPRB, 0.16297E+01_JPRB, 0.16900E+01_JPRB ,&
&  0.16150E+01_JPRB, 0.14990E+01_JPRB, 0.17013E+01_JPRB, 0.19950E+01_JPRB, 0.22476E+01_JPRB, 0.23528E+01_JPRB, 0.27042E+01_JPRB ,&
&  0.34504E+01_JPRB, 0.40486E+01_JPRB, 0.43226E+01_JPRB, 0.36143E+01_JPRB, 0.39327E+01_JPRB, 0.46131E+01_JPRB, 0.47987E+01_JPRB ,&
&  0.46431E+01_JPRB, 0.40428E+01_JPRB, 0.43825E+01_JPRB, 0.47653E+01_JPRB, 0.51123E+01_JPRB, 0.62598E+01_JPRB, 0.64560E+01_JPRB ,&
&  0.77276E+01_JPRB, 0.96242E+01_JPRB, 0.79052E+01_JPRB, 0.59845E+01_JPRB, 0.48548E+01_JPRB, 0.40291E+01_JPRB, 0.38799E+01_JPRB ,&
&  0.41785E+01_JPRB, 0.38423E+01_JPRB, 0.33691E+01_JPRB, 0.34041E+01_JPRB, 0.28924E+01_JPRB, 0.28289E+01_JPRB, 0.25359E+01_JPRB ,&
&  0.21079E+01_JPRB, 0.17983E+01_JPRB, 0.15114E+01_JPRB, 0.13385E+01_JPRB, 0.12469E+01_JPRB, 0.12254E+01_JPRB, 0.12229E+01_JPRB ,&
&  0.12302E+01_JPRB, 0.12298E+01_JPRB, 0.12238E+01_JPRB, 0.12122E+01_JPRB, 0.11864E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.11586E+01_JPRB, 0.11641E+01_JPRB, 0.11861E+01_JPRB, 0.11931E+01_JPRB, 0.11744E+01_JPRB, 0.11604E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.10340E+01_JPRB, 0.10066E+01_JPRB, 0.10078E+01_JPRB, 0.10297E+01_JPRB, 0.10425E+01_JPRB, 0.10569E+01_JPRB ,&
&  0.11243E+01_JPRB, 0.13079E+01_JPRB, 0.16122E+01_JPRB, 0.19004E+01_JPRB, 0.21898E+01_JPRB, 0.25655E+01_JPRB, 0.25075E+01_JPRB ,&
&  0.19025E+01_JPRB, 0.20961E+01_JPRB, 0.17012E+01_JPRB, 0.17811E+01_JPRB, 0.18735E+01_JPRB, 0.20365E+01_JPRB, 0.18578E+01_JPRB ,&
&  0.16880E+01_JPRB, 0.16489E+01_JPRB, 0.17066E+01_JPRB, 0.16401E+01_JPRB, 0.15379E+01_JPRB, 0.14975E+01_JPRB, 0.15423E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.15618E+01_JPRB, 0.15596E+01_JPRB, 0.16080E+01_JPRB, 0.16423E+01_JPRB, 0.16525E+01_JPRB, 0.16811E+01_JPRB ,&
&  0.16988E+01_JPRB, 0.17086E+01_JPRB, 0.16796E+01_JPRB, 0.17064E+01_JPRB, 0.17358E+01_JPRB, 0.18301E+01_JPRB, 0.19088E+01_JPRB ,&
&  0.18678E+01_JPRB, 0.17769E+01_JPRB, 0.19340E+01_JPRB, 0.21439E+01_JPRB, 0.23435E+01_JPRB, 0.27048E+01_JPRB, 0.29840E+01_JPRB ,&
&  0.37062E+01_JPRB, 0.38631E+01_JPRB, 0.46329E+01_JPRB, 0.34008E+01_JPRB, 0.35782E+01_JPRB, 0.41556E+01_JPRB, 0.40962E+01_JPRB ,&
&  0.35058E+01_JPRB, 0.38780E+01_JPRB, 0.42514E+01_JPRB, 0.48430E+01_JPRB, 0.52769E+01_JPRB, 0.57125E+01_JPRB, 0.52717E+01_JPRB ,&
&  0.54070E+01_JPRB, 0.58805E+01_JPRB, 0.55531E+01_JPRB, 0.45019E+01_JPRB, 0.40986E+01_JPRB, 0.35802E+01_JPRB, 0.38858E+01_JPRB ,&
&  0.41054E+01_JPRB, 0.38350E+01_JPRB, 0.37177E+01_JPRB, 0.35088E+01_JPRB, 0.25228E+01_JPRB, 0.21898E+01_JPRB, 0.18070E+01_JPRB ,&
&  0.14702E+01_JPRB, 0.12658E+01_JPRB, 0.10980E+01_JPRB, 0.10725E+01_JPRB, 0.10861E+01_JPRB, 0.11047E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.11582E+01_JPRB, 0.11708E+01_JPRB, 0.11731E+01_JPRB, 0.11629E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.11813E+01_JPRB, 0.11867E+01_JPRB, 0.12126E+01_JPRB, 0.12441E+01_JPRB, 0.12696E+01_JPRB, 0.13068E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.10835E+01_JPRB, 0.11082E+01_JPRB, 0.11247E+01_JPRB, 0.11428E+01_JPRB, 0.11360E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.13144E+01_JPRB, 0.15246E+01_JPRB, 0.17999E+01_JPRB, 0.21166E+01_JPRB, 0.24328E+01_JPRB, 0.28072E+01_JPRB, 0.28823E+01_JPRB ,&
&  0.22406E+01_JPRB, 0.19761E+01_JPRB, 0.18519E+01_JPRB, 0.19616E+01_JPRB, 0.20201E+01_JPRB, 0.22264E+01_JPRB, 0.23147E+01_JPRB ,&
&  0.22126E+01_JPRB, 0.20790E+01_JPRB, 0.21565E+01_JPRB, 0.19400E+01_JPRB, 0.16262E+01_JPRB, 0.16724E+01_JPRB, 0.17410E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.18030E+01_JPRB, 0.18328E+01_JPRB, 0.18861E+01_JPRB, 0.19132E+01_JPRB, 0.18863E+01_JPRB, 0.18518E+01_JPRB ,&
&  0.18490E+01_JPRB, 0.18772E+01_JPRB, 0.18801E+01_JPRB, 0.18579E+01_JPRB, 0.19423E+01_JPRB, 0.20459E+01_JPRB, 0.21192E+01_JPRB ,&
&  0.21729E+01_JPRB, 0.22733E+01_JPRB, 0.23887E+01_JPRB, 0.25638E+01_JPRB, 0.26033E+01_JPRB, 0.30735E+01_JPRB, 0.33588E+01_JPRB ,&
&  0.37372E+01_JPRB, 0.38939E+01_JPRB, 0.43370E+01_JPRB, 0.31144E+01_JPRB, 0.35882E+01_JPRB, 0.38257E+01_JPRB, 0.38586E+01_JPRB ,&
&  0.36316E+01_JPRB, 0.39601E+01_JPRB, 0.41637E+01_JPRB, 0.44388E+01_JPRB, 0.47167E+01_JPRB, 0.44246E+01_JPRB, 0.44744E+01_JPRB ,&
&  0.41643E+01_JPRB, 0.46368E+01_JPRB, 0.43804E+01_JPRB, 0.40697E+01_JPRB, 0.35270E+01_JPRB, 0.32854E+01_JPRB, 0.34368E+01_JPRB ,&
&  0.36955E+01_JPRB, 0.34751E+01_JPRB, 0.31740E+01_JPRB, 0.26433E+01_JPRB, 0.18301E+01_JPRB, 0.15166E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.10342E+01_JPRB, 0.10017E+01_JPRB, 0.10447E+01_JPRB, 0.10694E+01_JPRB, 0.11316E+01_JPRB, 0.11418E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11522E+01_JPRB, 0.11777E+01_JPRB, 0.11900E+01_JPRB, 0.11882E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.11553E+01_JPRB, 0.11480E+01_JPRB, 0.11417E+01_JPRB, 0.11494E+01_JPRB, 0.11718E+01_JPRB, 0.11822E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.11644E+01_JPRB, 0.12161E+01_JPRB, 0.12419E+01_JPRB, 0.12522E+01_JPRB, 0.12694E+01_JPRB, 0.13489E+01_JPRB ,&
&  0.14899E+01_JPRB, 0.17062E+01_JPRB, 0.19592E+01_JPRB, 0.21469E+01_JPRB, 0.23107E+01_JPRB, 0.25061E+01_JPRB, 0.26990E+01_JPRB ,&
&  0.23199E+01_JPRB, 0.17539E+01_JPRB, 0.19838E+01_JPRB, 0.21252E+01_JPRB, 0.21890E+01_JPRB, 0.25433E+01_JPRB, 0.29468E+01_JPRB ,&
&  0.23482E+01_JPRB, 0.21741E+01_JPRB, 0.22099E+01_JPRB, 0.20698E+01_JPRB, 0.17792E+01_JPRB, 0.18518E+01_JPRB, 0.19423E+01_JPRB ,&
&  0.20085E+01_JPRB, 0.20498E+01_JPRB, 0.21231E+01_JPRB, 0.21918E+01_JPRB, 0.22114E+01_JPRB, 0.21754E+01_JPRB, 0.21260E+01_JPRB ,&
&  0.21544E+01_JPRB, 0.22086E+01_JPRB, 0.22706E+01_JPRB, 0.22731E+01_JPRB, 0.23305E+01_JPRB, 0.24422E+01_JPRB, 0.25475E+01_JPRB ,&
&  0.26461E+01_JPRB, 0.26804E+01_JPRB, 0.28563E+01_JPRB, 0.31981E+01_JPRB, 0.35076E+01_JPRB, 0.32017E+01_JPRB, 0.35473E+01_JPRB ,&
&  0.35469E+01_JPRB, 0.35037E+01_JPRB, 0.33783E+01_JPRB, 0.39984E+01_JPRB, 0.27849E+01_JPRB, 0.32550E+01_JPRB, 0.31742E+01_JPRB ,&
&  0.37966E+01_JPRB, 0.37836E+01_JPRB, 0.38922E+01_JPRB, 0.40609E+01_JPRB, 0.42361E+01_JPRB, 0.38110E+01_JPRB, 0.37302E+01_JPRB ,&
&  0.38060E+01_JPRB, 0.38502E+01_JPRB, 0.38046E+01_JPRB, 0.37764E+01_JPRB, 0.34624E+01_JPRB, 0.33613E+01_JPRB, 0.32231E+01_JPRB ,&
&  0.32425E+01_JPRB, 0.27802E+01_JPRB, 0.23167E+01_JPRB, 0.17894E+01_JPRB, 0.12791E+01_JPRB, 0.11283E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10944E+01_JPRB, 0.10791E+01_JPRB, 0.10798E+01_JPRB, 0.11048E+01_JPRB, 0.11535E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.11610E+01_JPRB, 0.11696E+01_JPRB, 0.11886E+01_JPRB, 0.11931E+01_JPRB, 0.11813E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.11442E+01_JPRB, 0.11309E+01_JPRB, 0.11082E+01_JPRB, 0.11189E+01_JPRB, 0.11254E+01_JPRB, 0.11264E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.11504E+01_JPRB, 0.12092E+01_JPRB, 0.12737E+01_JPRB, 0.13173E+01_JPRB, 0.13337E+01_JPRB, 0.13599E+01_JPRB, 0.14391E+01_JPRB ,&
&  0.15544E+01_JPRB, 0.17272E+01_JPRB, 0.19409E+01_JPRB, 0.20920E+01_JPRB, 0.22082E+01_JPRB, 0.22557E+01_JPRB, 0.23486E+01_JPRB ,&
&  0.23033E+01_JPRB, 0.22978E+01_JPRB, 0.21912E+01_JPRB, 0.18560E+01_JPRB, 0.18373E+01_JPRB, 0.23467E+01_JPRB, 0.22841E+01_JPRB ,&
&  0.20935E+01_JPRB, 0.20134E+01_JPRB, 0.19864E+01_JPRB, 0.19728E+01_JPRB, 0.19715E+01_JPRB, 0.20549E+01_JPRB, 0.21605E+01_JPRB ,&
&  0.22317E+01_JPRB, 0.23268E+01_JPRB, 0.24169E+01_JPRB, 0.24799E+01_JPRB, 0.25116E+01_JPRB, 0.25642E+01_JPRB, 0.25866E+01_JPRB ,&
&  0.26891E+01_JPRB, 0.28154E+01_JPRB, 0.29426E+01_JPRB, 0.29928E+01_JPRB, 0.29590E+01_JPRB, 0.30578E+01_JPRB, 0.31496E+01_JPRB ,&
&  0.32440E+01_JPRB, 0.32495E+01_JPRB, 0.32373E+01_JPRB, 0.34024E+01_JPRB, 0.34842E+01_JPRB, 0.30260E+01_JPRB, 0.30415E+01_JPRB ,&
&  0.32751E+01_JPRB, 0.32330E+01_JPRB, 0.28742E+01_JPRB, 0.36517E+01_JPRB, 0.22578E+01_JPRB, 0.29537E+01_JPRB, 0.35246E+01_JPRB ,&
&  0.34779E+01_JPRB, 0.34761E+01_JPRB, 0.35513E+01_JPRB, 0.36236E+01_JPRB, 0.35972E+01_JPRB, 0.32866E+01_JPRB, 0.34364E+01_JPRB ,&
&  0.35607E+01_JPRB, 0.34954E+01_JPRB, 0.34444E+01_JPRB, 0.33859E+01_JPRB, 0.32564E+01_JPRB, 0.29393E+01_JPRB, 0.24868E+01_JPRB ,&
&  0.20559E+01_JPRB, 0.22146E+01_JPRB, 0.18611E+01_JPRB, 0.15766E+01_JPRB, 0.11601E+01_JPRB, 0.10039E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10881E+01_JPRB, 0.10882E+01_JPRB, 0.11124E+01_JPRB, 0.11286E+01_JPRB, 0.11461E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.11675E+01_JPRB, 0.11781E+01_JPRB, 0.11731E+01_JPRB, 0.11606E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.10893E+01_JPRB, 0.10738E+01_JPRB, 0.10562E+01_JPRB, 0.10933E+01_JPRB, 0.10884E+01_JPRB, 0.10795E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.11449E+01_JPRB, 0.11920E+01_JPRB, 0.12444E+01_JPRB, 0.13080E+01_JPRB, 0.13939E+01_JPRB, 0.14714E+01_JPRB ,&
&  0.15602E+01_JPRB, 0.16474E+01_JPRB, 0.17948E+01_JPRB, 0.19808E+01_JPRB, 0.22221E+01_JPRB, 0.23493E+01_JPRB, 0.23594E+01_JPRB ,&
&  0.22379E+01_JPRB, 0.23042E+01_JPRB, 0.24053E+01_JPRB, 0.25066E+01_JPRB, 0.22350E+01_JPRB, 0.25121E+01_JPRB, 0.27036E+01_JPRB ,&
&  0.26486E+01_JPRB, 0.23340E+01_JPRB, 0.20891E+01_JPRB, 0.20878E+01_JPRB, 0.21178E+01_JPRB, 0.22298E+01_JPRB, 0.23589E+01_JPRB ,&
&  0.24672E+01_JPRB, 0.25837E+01_JPRB, 0.26660E+01_JPRB, 0.27507E+01_JPRB, 0.28690E+01_JPRB, 0.30476E+01_JPRB, 0.31415E+01_JPRB ,&
&  0.32383E+01_JPRB, 0.34217E+01_JPRB, 0.35198E+01_JPRB, 0.34872E+01_JPRB, 0.35944E+01_JPRB, 0.36714E+01_JPRB, 0.37351E+01_JPRB ,&
&  0.38436E+01_JPRB, 0.36495E+01_JPRB, 0.37066E+01_JPRB, 0.38168E+01_JPRB, 0.36472E+01_JPRB, 0.32513E+01_JPRB, 0.31605E+01_JPRB ,&
&  0.32616E+01_JPRB, 0.32593E+01_JPRB, 0.23387E+01_JPRB, 0.32651E+01_JPRB, 0.39875E+01_JPRB, 0.34512E+01_JPRB, 0.33490E+01_JPRB ,&
&  0.32248E+01_JPRB, 0.32711E+01_JPRB, 0.33814E+01_JPRB, 0.34948E+01_JPRB, 0.35321E+01_JPRB, 0.35090E+01_JPRB, 0.32242E+01_JPRB ,&
&  0.34807E+01_JPRB, 0.33731E+01_JPRB, 0.31818E+01_JPRB, 0.29736E+01_JPRB, 0.27022E+01_JPRB, 0.22750E+01_JPRB, 0.18479E+01_JPRB ,&
&  0.15976E+01_JPRB, 0.18934E+01_JPRB, 0.16891E+01_JPRB, 0.16157E+01_JPRB, 0.14200E+01_JPRB, 0.98084E+00_JPRB, 0.10374E+01_JPRB ,&
&  0.10634E+01_JPRB, 0.10795E+01_JPRB, 0.10873E+01_JPRB, 0.10768E+01_JPRB, 0.10869E+01_JPRB, 0.11005E+01_JPRB, 0.11141E+01_JPRB ,&
&  0.11182E+01_JPRB, 0.11144E+01_JPRB, 0.11180E+01_JPRB, 0.11082E+01_JPRB, 0.11023E+01_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.10635E+01_JPRB, 0.10608E+01_JPRB, 0.10493E+01_JPRB, 0.10635E+01_JPRB, 0.10528E+01_JPRB, 0.10738E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.10346E+01_JPRB, 0.10349E+01_JPRB, 0.10474E+01_JPRB, 0.10630E+01_JPRB, 0.11008E+01_JPRB, 0.11717E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.13436E+01_JPRB, 0.13793E+01_JPRB, 0.15012E+01_JPRB, 0.17893E+01_JPRB, 0.20928E+01_JPRB, 0.22040E+01_JPRB ,&
&  0.23478E+01_JPRB, 0.25553E+01_JPRB, 0.27370E+01_JPRB, 0.29607E+01_JPRB, 0.30553E+01_JPRB, 0.31806E+01_JPRB, 0.34778E+01_JPRB ,&
&  0.29552E+01_JPRB, 0.23347E+01_JPRB, 0.21653E+01_JPRB, 0.21387E+01_JPRB, 0.22097E+01_JPRB, 0.23321E+01_JPRB, 0.24516E+01_JPRB ,&
&  0.25660E+01_JPRB, 0.26938E+01_JPRB, 0.28122E+01_JPRB, 0.29701E+01_JPRB, 0.31892E+01_JPRB, 0.34382E+01_JPRB, 0.36022E+01_JPRB ,&
&  0.37109E+01_JPRB, 0.38539E+01_JPRB, 0.40199E+01_JPRB, 0.39023E+01_JPRB, 0.43975E+01_JPRB, 0.45642E+01_JPRB, 0.46409E+01_JPRB ,&
&  0.45704E+01_JPRB, 0.43267E+01_JPRB, 0.43503E+01_JPRB, 0.43430E+01_JPRB, 0.43027E+01_JPRB, 0.39041E+01_JPRB, 0.36181E+01_JPRB ,&
&  0.35676E+01_JPRB, 0.30255E+01_JPRB, 0.21830E+01_JPRB, 0.24248E+01_JPRB, 0.27516E+01_JPRB, 0.24658E+01_JPRB, 0.31463E+01_JPRB ,&
&  0.30600E+01_JPRB, 0.30107E+01_JPRB, 0.30626E+01_JPRB, 0.31587E+01_JPRB, 0.33278E+01_JPRB, 0.32440E+01_JPRB, 0.30962E+01_JPRB ,&
&  0.30809E+01_JPRB, 0.29192E+01_JPRB, 0.26639E+01_JPRB, 0.23436E+01_JPRB, 0.20206E+01_JPRB, 0.17436E+01_JPRB, 0.15890E+01_JPRB ,&
&  0.15372E+01_JPRB, 0.15595E+01_JPRB, 0.15147E+01_JPRB, 0.13500E+01_JPRB, 0.10786E+01_JPRB, 0.92336E+00_JPRB, 0.94688E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.10135E+01_JPRB, 0.10219E+01_JPRB, 0.10141E+01_JPRB, 0.10144E+01_JPRB, 0.10138E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10668E+01_JPRB, 0.10715E+01_JPRB, 0.10719E+01_JPRB, 0.10720E+01_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.10196E+01_JPRB, 0.10163E+01_JPRB, 0.10154E+01_JPRB, 0.10310E+01_JPRB, 0.10430E+01_JPRB, 0.10257E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.10081E+01_JPRB, 0.98763E+00_JPRB, 0.99855E+00_JPRB, 0.10046E+01_JPRB, 0.10253E+01_JPRB, 0.10490E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10502E+01_JPRB, 0.99816E+00_JPRB, 0.99879E+00_JPRB, 0.10997E+01_JPRB, 0.12868E+01_JPRB, 0.16492E+01_JPRB ,&
&  0.20601E+01_JPRB, 0.22825E+01_JPRB, 0.24506E+01_JPRB, 0.25323E+01_JPRB, 0.25206E+01_JPRB, 0.24730E+01_JPRB, 0.33343E+01_JPRB ,&
&  0.35308E+01_JPRB, 0.22936E+01_JPRB, 0.22608E+01_JPRB, 0.21064E+01_JPRB, 0.21858E+01_JPRB, 0.22448E+01_JPRB, 0.23431E+01_JPRB ,&
&  0.24370E+01_JPRB, 0.25769E+01_JPRB, 0.27681E+01_JPRB, 0.30198E+01_JPRB, 0.32713E+01_JPRB, 0.35337E+01_JPRB, 0.37414E+01_JPRB ,&
&  0.38869E+01_JPRB, 0.40164E+01_JPRB, 0.42400E+01_JPRB, 0.43284E+01_JPRB, 0.43997E+01_JPRB, 0.46433E+01_JPRB, 0.48977E+01_JPRB ,&
&  0.50843E+01_JPRB, 0.54349E+01_JPRB, 0.48218E+01_JPRB, 0.47712E+01_JPRB, 0.49185E+01_JPRB, 0.41880E+01_JPRB, 0.39659E+01_JPRB ,&
&  0.38314E+01_JPRB, 0.29674E+01_JPRB, 0.19727E+01_JPRB, 0.21219E+01_JPRB, 0.23289E+01_JPRB, 0.24593E+01_JPRB, 0.26836E+01_JPRB ,&
&  0.25812E+01_JPRB, 0.24592E+01_JPRB, 0.24915E+01_JPRB, 0.26571E+01_JPRB, 0.26846E+01_JPRB, 0.25933E+01_JPRB, 0.24457E+01_JPRB ,&
&  0.23343E+01_JPRB, 0.21647E+01_JPRB, 0.20593E+01_JPRB, 0.18785E+01_JPRB, 0.17466E+01_JPRB, 0.15972E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.15379E+01_JPRB, 0.17875E+01_JPRB, 0.22423E+01_JPRB, 0.18667E+01_JPRB, 0.11251E+01_JPRB, 0.94276E+00_JPRB, 0.90443E+00_JPRB ,&
&  0.92339E+00_JPRB, 0.93549E+00_JPRB, 0.96297E+00_JPRB, 0.96726E+00_JPRB, 0.95803E+00_JPRB, 0.94825E+00_JPRB, 0.94232E+00_JPRB ,&
&  0.96877E+00_JPRB, 0.10055E+01_JPRB, 0.10247E+01_JPRB, 0.10194E+01_JPRB, 0.10144E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10706E+01_JPRB, 0.10773E+01_JPRB, 0.11035E+01_JPRB, 0.11125E+01_JPRB, 0.11112E+01_JPRB, 0.11175E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.11127E+01_JPRB, 0.11212E+01_JPRB, 0.11329E+01_JPRB, 0.11506E+01_JPRB, 0.11683E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.11716E+01_JPRB, 0.12056E+01_JPRB, 0.12287E+01_JPRB, 0.12227E+01_JPRB, 0.12157E+01_JPRB, 0.12451E+01_JPRB ,&
&  0.13194E+01_JPRB, 0.13665E+01_JPRB, 0.13850E+01_JPRB, 0.13752E+01_JPRB, 0.14350E+01_JPRB, 0.16801E+01_JPRB, 0.25827E+01_JPRB ,&
&  0.29930E+01_JPRB, 0.23083E+01_JPRB, 0.20251E+01_JPRB, 0.18460E+01_JPRB, 0.19011E+01_JPRB, 0.20361E+01_JPRB, 0.20524E+01_JPRB ,&
&  0.20591E+01_JPRB, 0.21367E+01_JPRB, 0.22186E+01_JPRB, 0.22998E+01_JPRB, 0.24129E+01_JPRB, 0.24970E+01_JPRB, 0.25417E+01_JPRB ,&
&  0.25017E+01_JPRB, 0.24893E+01_JPRB, 0.25773E+01_JPRB, 0.27352E+01_JPRB, 0.31988E+01_JPRB, 0.38714E+01_JPRB, 0.37281E+01_JPRB ,&
&  0.37431E+01_JPRB, 0.41551E+01_JPRB, 0.44910E+01_JPRB, 0.46579E+01_JPRB, 0.44373E+01_JPRB, 0.39652E+01_JPRB, 0.35736E+01_JPRB ,&
&  0.31723E+01_JPRB, 0.25827E+01_JPRB, 0.21092E+01_JPRB, 0.23915E+01_JPRB, 0.24259E+01_JPRB, 0.24742E+01_JPRB, 0.22671E+01_JPRB ,&
&  0.21278E+01_JPRB, 0.18938E+01_JPRB, 0.18732E+01_JPRB, 0.19751E+01_JPRB, 0.20325E+01_JPRB, 0.19569E+01_JPRB, 0.18382E+01_JPRB ,&
&  0.17057E+01_JPRB, 0.16838E+01_JPRB, 0.17666E+01_JPRB, 0.17816E+01_JPRB, 0.18081E+01_JPRB, 0.14994E+01_JPRB, 0.13881E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.17688E+01_JPRB, 0.15101E+01_JPRB, 0.18867E+01_JPRB, 0.10210E+01_JPRB, 0.84514E+00_JPRB, 0.84173E+00_JPRB ,&
&  0.86672E+00_JPRB, 0.90190E+00_JPRB, 0.91420E+00_JPRB, 0.10112E+01_JPRB, 0.10029E+01_JPRB, 0.10029E+01_JPRB, 0.10236E+01_JPRB ,&
&  0.10078E+01_JPRB, 0.10146E+01_JPRB, 0.10235E+01_JPRB, 0.10450E+01_JPRB, 0.10593E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.11404E+01_JPRB, 0.11477E+01_JPRB, 0.11602E+01_JPRB, 0.11787E+01_JPRB, 0.11913E+01_JPRB, 0.12013E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11974E+01_JPRB, 0.12110E+01_JPRB, 0.12312E+01_JPRB, 0.12522E+01_JPRB, 0.12705E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.13112E+01_JPRB, 0.13449E+01_JPRB, 0.13798E+01_JPRB, 0.13970E+01_JPRB, 0.13981E+01_JPRB, 0.14052E+01_JPRB ,&
&  0.13979E+01_JPRB, 0.14117E+01_JPRB, 0.14547E+01_JPRB, 0.14865E+01_JPRB, 0.15302E+01_JPRB, 0.16435E+01_JPRB, 0.19877E+01_JPRB ,&
&  0.22815E+01_JPRB, 0.24737E+01_JPRB, 0.21496E+01_JPRB, 0.19420E+01_JPRB, 0.19419E+01_JPRB, 0.18555E+01_JPRB, 0.17862E+01_JPRB ,&
&  0.17132E+01_JPRB, 0.16897E+01_JPRB, 0.16312E+01_JPRB, 0.16403E+01_JPRB, 0.16239E+01_JPRB, 0.16601E+01_JPRB, 0.16678E+01_JPRB ,&
&  0.17475E+01_JPRB, 0.18458E+01_JPRB, 0.20477E+01_JPRB, 0.22268E+01_JPRB, 0.24602E+01_JPRB, 0.26304E+01_JPRB, 0.23738E+01_JPRB ,&
&  0.23340E+01_JPRB, 0.26386E+01_JPRB, 0.31174E+01_JPRB, 0.34593E+01_JPRB, 0.32567E+01_JPRB, 0.29087E+01_JPRB, 0.26140E+01_JPRB ,&
&  0.23189E+01_JPRB, 0.21412E+01_JPRB, 0.24659E+01_JPRB, 0.24273E+01_JPRB, 0.23869E+01_JPRB, 0.21938E+01_JPRB, 0.19336E+01_JPRB ,&
&  0.16703E+01_JPRB, 0.13713E+01_JPRB, 0.13375E+01_JPRB, 0.13302E+01_JPRB, 0.13431E+01_JPRB, 0.13004E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12540E+01_JPRB, 0.15020E+01_JPRB, 0.18131E+01_JPRB, 0.20470E+01_JPRB, 0.25551E+01_JPRB, 0.22411E+01_JPRB ,&
&  0.15287E+01_JPRB, 0.14834E+01_JPRB, 0.89907E+00_JPRB, 0.12305E+01_JPRB, 0.82721E+00_JPRB, 0.78347E+00_JPRB, 0.86011E+00_JPRB ,&
&  0.97490E+00_JPRB, 0.10771E+01_JPRB, 0.11533E+01_JPRB, 0.11272E+01_JPRB, 0.10647E+01_JPRB, 0.10625E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.10978E+01_JPRB, 0.11074E+01_JPRB, 0.11170E+01_JPRB, 0.11230E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.11681E+01_JPRB, 0.11858E+01_JPRB, 0.12084E+01_JPRB, 0.12345E+01_JPRB, 0.12579E+01_JPRB, 0.12798E+01_JPRB, 0.12976E+01_JPRB ,&
&  0.13105E+01_JPRB, 0.13315E+01_JPRB, 0.13645E+01_JPRB, 0.14009E+01_JPRB, 0.14209E+01_JPRB, 0.14431E+01_JPRB, 0.14545E+01_JPRB ,&
&  0.14905E+01_JPRB, 0.15342E+01_JPRB, 0.15616E+01_JPRB, 0.15946E+01_JPRB, 0.16167E+01_JPRB, 0.16405E+01_JPRB, 0.16722E+01_JPRB ,&
&  0.16949E+01_JPRB, 0.17509E+01_JPRB, 0.18403E+01_JPRB, 0.19107E+01_JPRB, 0.20191E+01_JPRB, 0.22739E+01_JPRB, 0.20924E+01_JPRB ,&
&  0.25088E+01_JPRB, 0.24684E+01_JPRB, 0.23091E+01_JPRB, 0.20731E+01_JPRB, 0.19970E+01_JPRB, 0.19235E+01_JPRB, 0.18634E+01_JPRB ,&
&  0.18214E+01_JPRB, 0.17451E+01_JPRB, 0.17692E+01_JPRB, 0.17644E+01_JPRB, 0.17589E+01_JPRB, 0.18842E+01_JPRB, 0.20033E+01_JPRB ,&
&  0.20641E+01_JPRB, 0.21303E+01_JPRB, 0.22266E+01_JPRB, 0.22849E+01_JPRB, 0.23810E+01_JPRB, 0.24327E+01_JPRB, 0.23133E+01_JPRB ,&
&  0.21477E+01_JPRB, 0.20811E+01_JPRB, 0.20306E+01_JPRB, 0.21075E+01_JPRB, 0.20684E+01_JPRB, 0.18133E+01_JPRB, 0.15361E+01_JPRB ,&
&  0.17411E+01_JPRB, 0.17120E+01_JPRB, 0.18106E+01_JPRB, 0.25032E+01_JPRB, 0.23205E+01_JPRB, 0.20506E+01_JPRB, 0.16777E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.10949E+01_JPRB, 0.99889E+00_JPRB, 0.10082E+01_JPRB, 0.97979E+00_JPRB, 0.97455E+00_JPRB, 0.95658E+00_JPRB ,&
&  0.92879E+00_JPRB, 0.99299E+00_JPRB, 0.12351E+01_JPRB, 0.16018E+01_JPRB, 0.20391E+01_JPRB, 0.22077E+01_JPRB, 0.28438E+01_JPRB ,&
&  0.18376E+01_JPRB, 0.10609E+01_JPRB, 0.11436E+01_JPRB, 0.10078E+01_JPRB, 0.87172E+00_JPRB, 0.75143E+00_JPRB, 0.86545E+00_JPRB ,&
&  0.10789E+01_JPRB, 0.11491E+01_JPRB, 0.11764E+01_JPRB, 0.11362E+01_JPRB, 0.11345E+01_JPRB, 0.11445E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.11478E+01_JPRB, 0.11543E+01_JPRB, 0.11535E+01_JPRB, 0.11563E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.12030E+01_JPRB, 0.12281E+01_JPRB, 0.12617E+01_JPRB, 0.13017E+01_JPRB, 0.13281E+01_JPRB, 0.13636E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.14070E+01_JPRB, 0.14498E+01_JPRB, 0.14918E+01_JPRB, 0.15365E+01_JPRB, 0.15825E+01_JPRB, 0.16251E+01_JPRB, 0.16684E+01_JPRB ,&
&  0.17052E+01_JPRB, 0.17463E+01_JPRB, 0.17729E+01_JPRB, 0.17950E+01_JPRB, 0.17979E+01_JPRB, 0.18045E+01_JPRB, 0.17986E+01_JPRB ,&
&  0.17889E+01_JPRB, 0.18020E+01_JPRB, 0.18340E+01_JPRB, 0.18629E+01_JPRB, 0.19243E+01_JPRB, 0.21635E+01_JPRB, 0.17439E+01_JPRB ,&
&  0.24157E+01_JPRB, 0.24501E+01_JPRB, 0.24321E+01_JPRB, 0.22712E+01_JPRB, 0.21973E+01_JPRB, 0.22296E+01_JPRB, 0.23111E+01_JPRB ,&
&  0.24346E+01_JPRB, 0.25967E+01_JPRB, 0.27460E+01_JPRB, 0.26416E+01_JPRB, 0.24856E+01_JPRB, 0.25508E+01_JPRB, 0.26589E+01_JPRB ,&
&  0.27178E+01_JPRB, 0.27411E+01_JPRB, 0.27742E+01_JPRB, 0.27516E+01_JPRB, 0.27345E+01_JPRB, 0.27056E+01_JPRB, 0.26258E+01_JPRB ,&
&  0.24945E+01_JPRB, 0.23131E+01_JPRB, 0.20794E+01_JPRB, 0.22641E+01_JPRB, 0.22818E+01_JPRB, 0.19029E+01_JPRB, 0.13467E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.14900E+01_JPRB, 0.18198E+01_JPRB, 0.24720E+01_JPRB, 0.21549E+01_JPRB, 0.18295E+01_JPRB, 0.16002E+01_JPRB ,&
&  0.12565E+01_JPRB, 0.10031E+01_JPRB, 0.87616E+00_JPRB, 0.87695E+00_JPRB, 0.87524E+00_JPRB, 0.88700E+00_JPRB, 0.87025E+00_JPRB ,&
&  0.85675E+00_JPRB, 0.89092E+00_JPRB, 0.10477E+01_JPRB, 0.12714E+01_JPRB, 0.15978E+01_JPRB, 0.17075E+01_JPRB, 0.26984E+01_JPRB ,&
&  0.16998E+01_JPRB, 0.13458E+01_JPRB, 0.18095E+01_JPRB, 0.76390E+00_JPRB, 0.85446E+00_JPRB, 0.92615E+00_JPRB, 0.10328E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.13629E+01_JPRB, 0.16458E+01_JPRB, 0.14661E+01_JPRB, 0.13402E+01_JPRB, 0.12387E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.11967E+01_JPRB, 0.11930E+01_JPRB, 0.11979E+01_JPRB, 0.11982E+01_JPRB, 0.11945E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.12701E+01_JPRB, 0.12837E+01_JPRB, 0.12881E+01_JPRB, 0.13319E+01_JPRB, 0.13423E+01_JPRB, 0.13704E+01_JPRB, 0.13821E+01_JPRB ,&
&  0.14158E+01_JPRB, 0.14482E+01_JPRB, 0.14889E+01_JPRB, 0.15466E+01_JPRB, 0.15875E+01_JPRB, 0.16260E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.17324E+01_JPRB, 0.17953E+01_JPRB, 0.18143E+01_JPRB, 0.18244E+01_JPRB, 0.18191E+01_JPRB, 0.18323E+01_JPRB, 0.18364E+01_JPRB ,&
&  0.18482E+01_JPRB, 0.18566E+01_JPRB, 0.18917E+01_JPRB, 0.19440E+01_JPRB, 0.20182E+01_JPRB, 0.21890E+01_JPRB, 0.15692E+01_JPRB ,&
&  0.23420E+01_JPRB, 0.25602E+01_JPRB, 0.25210E+01_JPRB, 0.25226E+01_JPRB, 0.24294E+01_JPRB, 0.25131E+01_JPRB, 0.26670E+01_JPRB ,&
&  0.28731E+01_JPRB, 0.30803E+01_JPRB, 0.31526E+01_JPRB, 0.31504E+01_JPRB, 0.31521E+01_JPRB, 0.32801E+01_JPRB, 0.34115E+01_JPRB ,&
&  0.35006E+01_JPRB, 0.35601E+01_JPRB, 0.36306E+01_JPRB, 0.36750E+01_JPRB, 0.37326E+01_JPRB, 0.38023E+01_JPRB, 0.38074E+01_JPRB ,&
&  0.37403E+01_JPRB, 0.36773E+01_JPRB, 0.37204E+01_JPRB, 0.39561E+01_JPRB, 0.39239E+01_JPRB, 0.33512E+01_JPRB, 0.25305E+01_JPRB ,&
&  0.15742E+01_JPRB, 0.12474E+01_JPRB, 0.17073E+01_JPRB, 0.23061E+01_JPRB, 0.19765E+01_JPRB, 0.16763E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.99225E+00_JPRB, 0.85101E+00_JPRB, 0.84802E+00_JPRB, 0.87577E+00_JPRB, 0.93908E+00_JPRB, 0.91375E+00_JPRB ,&
&  0.94326E+00_JPRB, 0.98476E+00_JPRB, 0.10371E+01_JPRB, 0.11716E+01_JPRB, 0.14524E+01_JPRB, 0.16736E+01_JPRB, 0.15926E+01_JPRB ,&
&  0.18214E+01_JPRB, 0.13248E+01_JPRB, 0.12147E+01_JPRB, 0.12900E+01_JPRB, 0.11462E+01_JPRB, 0.13254E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.20865E+01_JPRB, 0.19876E+01_JPRB, 0.25752E+01_JPRB, 0.38847E+01_JPRB, 0.30591E+01_JPRB, 0.22285E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.12690E+01_JPRB, 0.12533E+01_JPRB, 0.12741E+01_JPRB, 0.12699E+01_JPRB, 0.12676E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.11927E+01_JPRB, 0.11844E+01_JPRB, 0.11956E+01_JPRB, 0.12048E+01_JPRB, 0.12062E+01_JPRB, 0.12233E+01_JPRB, 0.12530E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.13234E+01_JPRB, 0.14016E+01_JPRB, 0.14651E+01_JPRB, 0.15492E+01_JPRB, 0.15906E+01_JPRB, 0.15930E+01_JPRB ,&
&  0.16168E+01_JPRB, 0.16860E+01_JPRB, 0.17511E+01_JPRB, 0.17952E+01_JPRB, 0.17875E+01_JPRB, 0.18105E+01_JPRB, 0.18903E+01_JPRB ,&
&  0.20018E+01_JPRB, 0.20878E+01_JPRB, 0.21672E+01_JPRB, 0.22727E+01_JPRB, 0.23276E+01_JPRB, 0.22652E+01_JPRB, 0.16235E+01_JPRB ,&
&  0.22391E+01_JPRB, 0.25623E+01_JPRB, 0.27085E+01_JPRB, 0.27176E+01_JPRB, 0.27585E+01_JPRB, 0.26922E+01_JPRB, 0.28391E+01_JPRB ,&
&  0.30526E+01_JPRB, 0.32537E+01_JPRB, 0.32245E+01_JPRB, 0.30929E+01_JPRB, 0.35475E+01_JPRB, 0.36909E+01_JPRB, 0.39075E+01_JPRB ,&
&  0.40916E+01_JPRB, 0.42573E+01_JPRB, 0.44170E+01_JPRB, 0.46039E+01_JPRB, 0.47714E+01_JPRB, 0.49199E+01_JPRB, 0.50394E+01_JPRB ,&
&  0.50875E+01_JPRB, 0.51053E+01_JPRB, 0.52365E+01_JPRB, 0.50377E+01_JPRB, 0.49151E+01_JPRB, 0.51005E+01_JPRB, 0.42984E+01_JPRB ,&
&  0.23807E+01_JPRB, 0.14873E+01_JPRB, 0.18532E+01_JPRB, 0.20799E+01_JPRB, 0.17499E+01_JPRB, 0.14960E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.96470E+00_JPRB, 0.91539E+00_JPRB, 0.96466E+00_JPRB, 0.10610E+01_JPRB, 0.11562E+01_JPRB, 0.11792E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11534E+01_JPRB, 0.11893E+01_JPRB, 0.12737E+01_JPRB, 0.14554E+01_JPRB, 0.18040E+01_JPRB, 0.22761E+01_JPRB ,&
&  0.21530E+01_JPRB, 0.14157E+01_JPRB, 0.13877E+01_JPRB, 0.13922E+01_JPRB, 0.14412E+01_JPRB, 0.15690E+01_JPRB, 0.18665E+01_JPRB ,&
&  0.23453E+01_JPRB, 0.28465E+01_JPRB, 0.27215E+01_JPRB, 0.22676E+01_JPRB, 0.28080E+01_JPRB, 0.19262E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11525E+01_JPRB, 0.11832E+01_JPRB, 0.11922E+01_JPRB, 0.11960E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.10267E+01_JPRB, 0.10122E+01_JPRB, 0.10045E+01_JPRB, 0.99839E+00_JPRB, 0.10128E+01_JPRB, 0.10551E+01_JPRB, 0.11061E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.12348E+01_JPRB, 0.13218E+01_JPRB, 0.13791E+01_JPRB, 0.14480E+01_JPRB, 0.14780E+01_JPRB, 0.14980E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.15594E+01_JPRB, 0.16266E+01_JPRB, 0.16939E+01_JPRB, 0.17517E+01_JPRB, 0.18422E+01_JPRB, 0.19569E+01_JPRB ,&
&  0.20993E+01_JPRB, 0.22298E+01_JPRB, 0.23761E+01_JPRB, 0.25832E+01_JPRB, 0.26995E+01_JPRB, 0.27788E+01_JPRB, 0.28862E+01_JPRB ,&
&  0.15862E+01_JPRB, 0.23657E+01_JPRB, 0.25963E+01_JPRB, 0.26538E+01_JPRB, 0.29366E+01_JPRB, 0.30479E+01_JPRB, 0.31423E+01_JPRB ,&
&  0.33229E+01_JPRB, 0.32802E+01_JPRB, 0.32352E+01_JPRB, 0.32663E+01_JPRB, 0.35770E+01_JPRB, 0.37486E+01_JPRB, 0.39428E+01_JPRB ,&
&  0.40519E+01_JPRB, 0.41558E+01_JPRB, 0.43570E+01_JPRB, 0.45845E+01_JPRB, 0.48283E+01_JPRB, 0.50724E+01_JPRB, 0.53728E+01_JPRB ,&
&  0.56083E+01_JPRB, 0.58056E+01_JPRB, 0.61417E+01_JPRB, 0.56199E+01_JPRB, 0.52521E+01_JPRB, 0.56203E+01_JPRB, 0.51235E+01_JPRB ,&
&  0.34387E+01_JPRB, 0.18712E+01_JPRB, 0.18373E+01_JPRB, 0.20461E+01_JPRB, 0.16361E+01_JPRB, 0.14054E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.99275E+00_JPRB, 0.10606E+01_JPRB, 0.11576E+01_JPRB, 0.12798E+01_JPRB, 0.13880E+01_JPRB, 0.14276E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.13159E+01_JPRB, 0.12576E+01_JPRB, 0.12546E+01_JPRB, 0.12843E+01_JPRB, 0.13291E+01_JPRB, 0.13486E+01_JPRB ,&
&  0.13543E+01_JPRB, 0.13187E+01_JPRB, 0.13321E+01_JPRB, 0.13628E+01_JPRB, 0.13721E+01_JPRB, 0.13479E+01_JPRB, 0.14697E+01_JPRB ,&
&  0.16914E+01_JPRB, 0.18971E+01_JPRB, 0.21348E+01_JPRB, 0.20539E+01_JPRB, 0.18318E+01_JPRB, 0.13480E+01_JPRB, 0.94795E+00_JPRB ,&
&  0.97681E+00_JPRB, 0.10048E+01_JPRB, 0.10160E+01_JPRB, 0.10385E+01_JPRB, 0.10341E+01_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.92334E+00_JPRB, 0.91682E+00_JPRB, 0.91123E+00_JPRB, 0.92134E+00_JPRB, 0.93491E+00_JPRB, 0.99268E+00_JPRB, 0.10879E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.12554E+01_JPRB, 0.13171E+01_JPRB, 0.13637E+01_JPRB, 0.13906E+01_JPRB, 0.13860E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.14346E+01_JPRB, 0.14627E+01_JPRB, 0.15209E+01_JPRB, 0.16016E+01_JPRB, 0.16842E+01_JPRB, 0.17769E+01_JPRB, 0.18880E+01_JPRB ,&
&  0.20255E+01_JPRB, 0.22116E+01_JPRB, 0.24178E+01_JPRB, 0.26995E+01_JPRB, 0.29605E+01_JPRB, 0.31790E+01_JPRB, 0.32671E+01_JPRB ,&
&  0.15282E+01_JPRB, 0.16879E+01_JPRB, 0.24586E+01_JPRB, 0.24580E+01_JPRB, 0.27286E+01_JPRB, 0.30662E+01_JPRB, 0.34588E+01_JPRB ,&
&  0.33479E+01_JPRB, 0.29407E+01_JPRB, 0.29434E+01_JPRB, 0.33572E+01_JPRB, 0.33802E+01_JPRB, 0.35367E+01_JPRB, 0.36820E+01_JPRB ,&
&  0.37394E+01_JPRB, 0.37520E+01_JPRB, 0.38273E+01_JPRB, 0.40149E+01_JPRB, 0.42795E+01_JPRB, 0.45643E+01_JPRB, 0.49024E+01_JPRB ,&
&  0.52578E+01_JPRB, 0.56550E+01_JPRB, 0.60436E+01_JPRB, 0.51489E+01_JPRB, 0.55389E+01_JPRB, 0.60376E+01_JPRB, 0.63493E+01_JPRB ,&
&  0.47583E+01_JPRB, 0.27576E+01_JPRB, 0.22120E+01_JPRB, 0.22561E+01_JPRB, 0.17501E+01_JPRB, 0.16224E+01_JPRB, 0.13925E+01_JPRB ,&
&  0.13171E+01_JPRB, 0.12147E+01_JPRB, 0.12795E+01_JPRB, 0.13811E+01_JPRB, 0.14969E+01_JPRB, 0.15925E+01_JPRB, 0.16189E+01_JPRB ,&
&  0.15326E+01_JPRB, 0.14737E+01_JPRB, 0.14405E+01_JPRB, 0.14363E+01_JPRB, 0.14468E+01_JPRB, 0.14548E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.14282E+01_JPRB, 0.14170E+01_JPRB, 0.14488E+01_JPRB, 0.14663E+01_JPRB, 0.14780E+01_JPRB, 0.14826E+01_JPRB, 0.14854E+01_JPRB ,&
&  0.14097E+01_JPRB, 0.13460E+01_JPRB, 0.14025E+01_JPRB, 0.12764E+01_JPRB, 0.11179E+01_JPRB, 0.96681E+00_JPRB, 0.83313E+00_JPRB ,&
&  0.86035E+00_JPRB, 0.87571E+00_JPRB, 0.87995E+00_JPRB, 0.89488E+00_JPRB, 0.91568E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.82505E+00_JPRB, 0.83329E+00_JPRB, 0.82770E+00_JPRB, 0.85234E+00_JPRB, 0.85808E+00_JPRB, 0.94164E+00_JPRB, 0.10201E+01_JPRB ,&
&  0.11483E+01_JPRB, 0.12480E+01_JPRB, 0.13188E+01_JPRB, 0.13545E+01_JPRB, 0.13489E+01_JPRB, 0.13212E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.13408E+01_JPRB, 0.13697E+01_JPRB, 0.14137E+01_JPRB, 0.14660E+01_JPRB, 0.15385E+01_JPRB, 0.16177E+01_JPRB ,&
&  0.17353E+01_JPRB, 0.19107E+01_JPRB, 0.21437E+01_JPRB, 0.24773E+01_JPRB, 0.29242E+01_JPRB, 0.35300E+01_JPRB, 0.41710E+01_JPRB ,&
&  0.33680E+01_JPRB, 0.16472E+01_JPRB, 0.24618E+01_JPRB, 0.23483E+01_JPRB, 0.22041E+01_JPRB, 0.26750E+01_JPRB, 0.32127E+01_JPRB ,&
&  0.30463E+01_JPRB, 0.27759E+01_JPRB, 0.26665E+01_JPRB, 0.30759E+01_JPRB, 0.31361E+01_JPRB, 0.32695E+01_JPRB, 0.33754E+01_JPRB ,&
&  0.34567E+01_JPRB, 0.34790E+01_JPRB, 0.34970E+01_JPRB, 0.36501E+01_JPRB, 0.39064E+01_JPRB, 0.42270E+01_JPRB, 0.45670E+01_JPRB ,&
&  0.49533E+01_JPRB, 0.53922E+01_JPRB, 0.59140E+01_JPRB, 0.51252E+01_JPRB, 0.57890E+01_JPRB, 0.58162E+01_JPRB, 0.50510E+01_JPRB ,&
&  0.46423E+01_JPRB, 0.31380E+01_JPRB, 0.24015E+01_JPRB, 0.24411E+01_JPRB, 0.20461E+01_JPRB, 0.15753E+01_JPRB, 0.15462E+01_JPRB ,&
&  0.14617E+01_JPRB, 0.14509E+01_JPRB, 0.14535E+01_JPRB, 0.15667E+01_JPRB, 0.16389E+01_JPRB, 0.16923E+01_JPRB, 0.16819E+01_JPRB ,&
&  0.16720E+01_JPRB, 0.16148E+01_JPRB, 0.15893E+01_JPRB, 0.15881E+01_JPRB, 0.15871E+01_JPRB, 0.16083E+01_JPRB, 0.16035E+01_JPRB ,&
&  0.15878E+01_JPRB, 0.15969E+01_JPRB, 0.16439E+01_JPRB, 0.16941E+01_JPRB, 0.17029E+01_JPRB, 0.15989E+01_JPRB, 0.15059E+01_JPRB ,&
&  0.14417E+01_JPRB, 0.13795E+01_JPRB, 0.12332E+01_JPRB, 0.10758E+01_JPRB, 0.99943E+00_JPRB, 0.90147E+00_JPRB, 0.82482E+00_JPRB ,&
&  0.83363E+00_JPRB, 0.83155E+00_JPRB, 0.82141E+00_JPRB, 0.82509E+00_JPRB, 0.82694E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.89853E+00_JPRB, 0.88564E+00_JPRB, 0.85048E+00_JPRB, 0.83475E+00_JPRB, 0.83497E+00_JPRB, 0.87271E+00_JPRB, 0.91712E+00_JPRB ,&
&  0.98589E+00_JPRB, 0.10864E+01_JPRB, 0.11767E+01_JPRB, 0.12327E+01_JPRB, 0.12354E+01_JPRB, 0.12517E+01_JPRB, 0.12351E+01_JPRB ,&
&  0.12206E+01_JPRB, 0.12341E+01_JPRB, 0.12551E+01_JPRB, 0.12665E+01_JPRB, 0.12904E+01_JPRB, 0.13243E+01_JPRB, 0.13610E+01_JPRB ,&
&  0.13825E+01_JPRB, 0.14146E+01_JPRB, 0.14682E+01_JPRB, 0.16512E+01_JPRB, 0.19753E+01_JPRB, 0.24832E+01_JPRB, 0.34076E+01_JPRB ,&
&  0.53072E+01_JPRB, 0.54654E+01_JPRB, 0.26723E+01_JPRB, 0.28749E+01_JPRB, 0.24977E+01_JPRB, 0.27238E+01_JPRB, 0.26606E+01_JPRB ,&
&  0.28265E+01_JPRB, 0.25680E+01_JPRB, 0.23966E+01_JPRB, 0.26926E+01_JPRB, 0.27735E+01_JPRB, 0.29036E+01_JPRB, 0.30107E+01_JPRB ,&
&  0.31568E+01_JPRB, 0.32336E+01_JPRB, 0.32069E+01_JPRB, 0.32593E+01_JPRB, 0.34787E+01_JPRB, 0.37802E+01_JPRB, 0.41320E+01_JPRB ,&
&  0.45537E+01_JPRB, 0.49600E+01_JPRB, 0.54934E+01_JPRB, 0.51190E+01_JPRB, 0.57367E+01_JPRB, 0.52785E+01_JPRB, 0.42070E+01_JPRB ,&
&  0.35009E+01_JPRB, 0.30896E+01_JPRB, 0.28009E+01_JPRB, 0.26809E+01_JPRB, 0.23603E+01_JPRB, 0.18002E+01_JPRB, 0.17331E+01_JPRB ,&
&  0.16488E+01_JPRB, 0.16006E+01_JPRB, 0.15859E+01_JPRB, 0.16225E+01_JPRB, 0.16921E+01_JPRB, 0.17229E+01_JPRB, 0.17367E+01_JPRB ,&
&  0.17268E+01_JPRB, 0.16872E+01_JPRB, 0.16684E+01_JPRB, 0.16639E+01_JPRB, 0.16655E+01_JPRB, 0.16890E+01_JPRB, 0.16972E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.17315E+01_JPRB, 0.17826E+01_JPRB, 0.18313E+01_JPRB, 0.17854E+01_JPRB, 0.16696E+01_JPRB, 0.15448E+01_JPRB ,&
&  0.14957E+01_JPRB, 0.14876E+01_JPRB, 0.13807E+01_JPRB, 0.12926E+01_JPRB, 0.11902E+01_JPRB, 0.11335E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10433E+01_JPRB, 0.10022E+01_JPRB, 0.89419E+00_JPRB, 0.89380E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.10264E+01_JPRB, 0.97729E+00_JPRB, 0.97585E+00_JPRB, 0.97522E+00_JPRB, 0.96767E+00_JPRB, 0.97583E+00_JPRB, 0.92133E+00_JPRB ,&
&  0.89853E+00_JPRB, 0.93928E+00_JPRB, 0.10106E+01_JPRB, 0.10604E+01_JPRB, 0.10935E+01_JPRB, 0.11030E+01_JPRB, 0.10968E+01_JPRB ,&
&  0.10986E+01_JPRB, 0.11324E+01_JPRB, 0.11584E+01_JPRB, 0.11750E+01_JPRB, 0.11998E+01_JPRB, 0.12206E+01_JPRB, 0.12466E+01_JPRB ,&
&  0.12688E+01_JPRB, 0.12403E+01_JPRB, 0.11969E+01_JPRB, 0.11939E+01_JPRB, 0.13131E+01_JPRB, 0.15216E+01_JPRB, 0.18477E+01_JPRB ,&
&  0.30908E+01_JPRB, 0.68028E+01_JPRB, 0.32144E+01_JPRB, 0.31495E+01_JPRB, 0.30501E+01_JPRB, 0.29097E+01_JPRB, 0.27411E+01_JPRB ,&
&  0.27967E+01_JPRB, 0.23832E+01_JPRB, 0.20982E+01_JPRB, 0.22290E+01_JPRB, 0.23114E+01_JPRB, 0.24036E+01_JPRB, 0.25219E+01_JPRB ,&
&  0.26417E+01_JPRB, 0.27346E+01_JPRB, 0.27910E+01_JPRB, 0.27706E+01_JPRB, 0.29248E+01_JPRB, 0.31482E+01_JPRB, 0.34606E+01_JPRB ,&
&  0.38274E+01_JPRB, 0.43041E+01_JPRB, 0.49382E+01_JPRB, 0.49220E+01_JPRB, 0.54139E+01_JPRB, 0.50667E+01_JPRB, 0.41167E+01_JPRB ,&
&  0.35617E+01_JPRB, 0.33964E+01_JPRB, 0.30268E+01_JPRB, 0.28061E+01_JPRB, 0.25320E+01_JPRB, 0.21386E+01_JPRB, 0.20748E+01_JPRB ,&
&  0.18413E+01_JPRB, 0.17385E+01_JPRB, 0.17190E+01_JPRB, 0.17079E+01_JPRB, 0.17319E+01_JPRB, 0.17433E+01_JPRB, 0.17502E+01_JPRB ,&
&  0.17424E+01_JPRB, 0.17072E+01_JPRB, 0.16912E+01_JPRB, 0.17072E+01_JPRB, 0.17171E+01_JPRB, 0.17122E+01_JPRB, 0.17256E+01_JPRB ,&
&  0.17380E+01_JPRB, 0.17763E+01_JPRB, 0.18461E+01_JPRB, 0.18460E+01_JPRB, 0.17845E+01_JPRB, 0.16613E+01_JPRB, 0.15400E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.14924E+01_JPRB, 0.14397E+01_JPRB, 0.13717E+01_JPRB, 0.13519E+01_JPRB, 0.12972E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.13730E+01_JPRB, 0.14543E+01_JPRB, 0.15130E+01_JPRB, 0.10536E+01_JPRB, 0.10508E+01_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.11098E+01_JPRB, 0.10590E+01_JPRB, 0.10632E+01_JPRB, 0.11022E+01_JPRB, 0.11101E+01_JPRB, 0.10877E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.94827E+00_JPRB, 0.92933E+00_JPRB, 0.94511E+00_JPRB, 0.97289E+00_JPRB, 0.99846E+00_JPRB, 0.10217E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.10196E+01_JPRB, 0.10412E+01_JPRB, 0.10621E+01_JPRB, 0.10940E+01_JPRB, 0.11232E+01_JPRB, 0.11506E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.11744E+01_JPRB, 0.11481E+01_JPRB, 0.11355E+01_JPRB, 0.11493E+01_JPRB, 0.12136E+01_JPRB, 0.13653E+01_JPRB ,&
&  0.20162E+01_JPRB, 0.57656E+01_JPRB, 0.52970E+01_JPRB, 0.39871E+01_JPRB, 0.31956E+01_JPRB, 0.28039E+01_JPRB, 0.26557E+01_JPRB ,&
&  0.29347E+01_JPRB, 0.29485E+01_JPRB, 0.25909E+01_JPRB, 0.21507E+01_JPRB, 0.20182E+01_JPRB, 0.20569E+01_JPRB, 0.21289E+01_JPRB ,&
&  0.21526E+01_JPRB, 0.21852E+01_JPRB, 0.22380E+01_JPRB, 0.22436E+01_JPRB, 0.22851E+01_JPRB, 0.24736E+01_JPRB, 0.27106E+01_JPRB ,&
&  0.29528E+01_JPRB, 0.33892E+01_JPRB, 0.41678E+01_JPRB, 0.49695E+01_JPRB, 0.50556E+01_JPRB, 0.51877E+01_JPRB, 0.49006E+01_JPRB ,&
&  0.50846E+01_JPRB, 0.44001E+01_JPRB, 0.34484E+01_JPRB, 0.27941E+01_JPRB, 0.24082E+01_JPRB, 0.24828E+01_JPRB, 0.23387E+01_JPRB ,&
&  0.19877E+01_JPRB, 0.18279E+01_JPRB, 0.17618E+01_JPRB, 0.17492E+01_JPRB, 0.17544E+01_JPRB, 0.17581E+01_JPRB, 0.17270E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.16906E+01_JPRB, 0.16938E+01_JPRB, 0.17167E+01_JPRB, 0.17225E+01_JPRB, 0.17219E+01_JPRB, 0.17320E+01_JPRB ,&
&  0.17236E+01_JPRB, 0.17402E+01_JPRB, 0.17521E+01_JPRB, 0.18003E+01_JPRB, 0.17465E+01_JPRB, 0.16173E+01_JPRB, 0.15011E+01_JPRB ,&
&  0.14485E+01_JPRB, 0.14637E+01_JPRB, 0.14612E+01_JPRB, 0.14335E+01_JPRB, 0.14093E+01_JPRB, 0.13814E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.13324E+01_JPRB, 0.13246E+01_JPRB, 0.12317E+01_JPRB, 0.11497E+01_JPRB, 0.11511E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.12062E+01_JPRB, 0.11729E+01_JPRB, 0.11506E+01_JPRB, 0.11617E+01_JPRB, 0.11723E+01_JPRB, 0.11552E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10723E+01_JPRB, 0.10232E+01_JPRB, 0.10220E+01_JPRB, 0.10127E+01_JPRB, 0.10056E+01_JPRB, 0.10074E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10450E+01_JPRB, 0.10394E+01_JPRB, 0.10421E+01_JPRB, 0.10632E+01_JPRB, 0.10912E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.11197E+01_JPRB, 0.11095E+01_JPRB, 0.11037E+01_JPRB, 0.10944E+01_JPRB, 0.11204E+01_JPRB, 0.12044E+01_JPRB ,&
&  0.16214E+01_JPRB, 0.45867E+01_JPRB, 0.30630E+01_JPRB, 0.40761E+01_JPRB, 0.31402E+01_JPRB, 0.27368E+01_JPRB, 0.22995E+01_JPRB ,&
&  0.25109E+01_JPRB, 0.30441E+01_JPRB, 0.24335E+01_JPRB, 0.20635E+01_JPRB, 0.18790E+01_JPRB, 0.18459E+01_JPRB, 0.18531E+01_JPRB ,&
&  0.18610E+01_JPRB, 0.18891E+01_JPRB, 0.18916E+01_JPRB, 0.19294E+01_JPRB, 0.19223E+01_JPRB, 0.20135E+01_JPRB, 0.21652E+01_JPRB ,&
&  0.22973E+01_JPRB, 0.25196E+01_JPRB, 0.30568E+01_JPRB, 0.37512E+01_JPRB, 0.50660E+01_JPRB, 0.57673E+01_JPRB, 0.61030E+01_JPRB ,&
&  0.63343E+01_JPRB, 0.53772E+01_JPRB, 0.42097E+01_JPRB, 0.32316E+01_JPRB, 0.26274E+01_JPRB, 0.25432E+01_JPRB, 0.25028E+01_JPRB ,&
&  0.22052E+01_JPRB, 0.19881E+01_JPRB, 0.18652E+01_JPRB, 0.18071E+01_JPRB, 0.18087E+01_JPRB, 0.18095E+01_JPRB, 0.17478E+01_JPRB ,&
&  0.16985E+01_JPRB, 0.16904E+01_JPRB, 0.17071E+01_JPRB, 0.17288E+01_JPRB, 0.17228E+01_JPRB, 0.16948E+01_JPRB, 0.16926E+01_JPRB ,&
&  0.16955E+01_JPRB, 0.16969E+01_JPRB, 0.16892E+01_JPRB, 0.17781E+01_JPRB, 0.17193E+01_JPRB, 0.16170E+01_JPRB, 0.15116E+01_JPRB ,&
&  0.14739E+01_JPRB, 0.14609E+01_JPRB, 0.14471E+01_JPRB, 0.14153E+01_JPRB, 0.14382E+01_JPRB, 0.14732E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.13537E+01_JPRB, 0.13304E+01_JPRB, 0.12930E+01_JPRB, 0.12365E+01_JPRB, 0.12324E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.12451E+01_JPRB, 0.12216E+01_JPRB, 0.12235E+01_JPRB, 0.12224E+01_JPRB, 0.12105E+01_JPRB, 0.11810E+01_JPRB, 0.11599E+01_JPRB ,&
&  0.11535E+01_JPRB, 0.11251E+01_JPRB, 0.11017E+01_JPRB, 0.10727E+01_JPRB, 0.10572E+01_JPRB, 0.10425E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10509E+01_JPRB, 0.10567E+01_JPRB, 0.10491E+01_JPRB, 0.10472E+01_JPRB, 0.10353E+01_JPRB, 0.10432E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10784E+01_JPRB, 0.10870E+01_JPRB, 0.10852E+01_JPRB, 0.10751E+01_JPRB, 0.10732E+01_JPRB, 0.11296E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.48062E+01_JPRB, 0.27174E+01_JPRB, 0.40868E+01_JPRB, 0.30757E+01_JPRB, 0.27053E+01_JPRB, 0.22157E+01_JPRB ,&
&  0.23620E+01_JPRB, 0.27878E+01_JPRB, 0.23397E+01_JPRB, 0.19567E+01_JPRB, 0.18265E+01_JPRB, 0.17538E+01_JPRB, 0.16877E+01_JPRB ,&
&  0.16756E+01_JPRB, 0.17108E+01_JPRB, 0.17292E+01_JPRB, 0.17618E+01_JPRB, 0.17864E+01_JPRB, 0.17805E+01_JPRB, 0.18315E+01_JPRB ,&
&  0.18766E+01_JPRB, 0.19569E+01_JPRB, 0.22843E+01_JPRB, 0.34082E+01_JPRB, 0.44746E+01_JPRB, 0.61192E+01_JPRB, 0.78263E+01_JPRB ,&
&  0.75167E+01_JPRB, 0.64385E+01_JPRB, 0.45707E+01_JPRB, 0.36896E+01_JPRB, 0.30402E+01_JPRB, 0.27641E+01_JPRB, 0.26090E+01_JPRB ,&
&  0.24437E+01_JPRB, 0.22780E+01_JPRB, 0.20705E+01_JPRB, 0.19685E+01_JPRB, 0.19366E+01_JPRB, 0.19109E+01_JPRB, 0.18637E+01_JPRB ,&
&  0.18081E+01_JPRB, 0.17967E+01_JPRB, 0.18021E+01_JPRB, 0.18019E+01_JPRB, 0.17882E+01_JPRB, 0.17408E+01_JPRB, 0.17114E+01_JPRB ,&
&  0.17181E+01_JPRB, 0.17055E+01_JPRB, 0.17032E+01_JPRB, 0.18280E+01_JPRB, 0.17677E+01_JPRB, 0.16403E+01_JPRB, 0.15498E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.14852E+01_JPRB, 0.14656E+01_JPRB, 0.14517E+01_JPRB, 0.14643E+01_JPRB, 0.15573E+01_JPRB, 0.14673E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.13476E+01_JPRB, 0.13104E+01_JPRB, 0.12793E+01_JPRB, 0.12626E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.13068E+01_JPRB, 0.12847E+01_JPRB, 0.12818E+01_JPRB, 0.12964E+01_JPRB, 0.12734E+01_JPRB, 0.12355E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.11842E+01_JPRB, 0.11576E+01_JPRB, 0.11161E+01_JPRB, 0.10988E+01_JPRB, 0.10892E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10872E+01_JPRB, 0.10658E+01_JPRB, 0.10529E+01_JPRB, 0.10453E+01_JPRB, 0.10355E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10383E+01_JPRB, 0.10447E+01_JPRB, 0.10651E+01_JPRB, 0.10675E+01_JPRB, 0.10695E+01_JPRB, 0.10542E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.13182E+01_JPRB, 0.17935E+01_JPRB, 0.31920E+01_JPRB, 0.37410E+01_JPRB, 0.30539E+01_JPRB, 0.27200E+01_JPRB, 0.24990E+01_JPRB ,&
&  0.24911E+01_JPRB, 0.25037E+01_JPRB, 0.22219E+01_JPRB, 0.19711E+01_JPRB, 0.18322E+01_JPRB, 0.17084E+01_JPRB, 0.16039E+01_JPRB ,&
&  0.15313E+01_JPRB, 0.15132E+01_JPRB, 0.15478E+01_JPRB, 0.16102E+01_JPRB, 0.16529E+01_JPRB, 0.16611E+01_JPRB, 0.16289E+01_JPRB ,&
&  0.16066E+01_JPRB, 0.16051E+01_JPRB, 0.17420E+01_JPRB, 0.25015E+01_JPRB, 0.42641E+01_JPRB, 0.63131E+01_JPRB, 0.76220E+01_JPRB ,&
&  0.77487E+01_JPRB, 0.64286E+01_JPRB, 0.46509E+01_JPRB, 0.39055E+01_JPRB, 0.33766E+01_JPRB, 0.29711E+01_JPRB, 0.26437E+01_JPRB ,&
&  0.24534E+01_JPRB, 0.23718E+01_JPRB, 0.22690E+01_JPRB, 0.21408E+01_JPRB, 0.20250E+01_JPRB, 0.19574E+01_JPRB, 0.19089E+01_JPRB ,&
&  0.18661E+01_JPRB, 0.18744E+01_JPRB, 0.19044E+01_JPRB, 0.18701E+01_JPRB, 0.18222E+01_JPRB, 0.17644E+01_JPRB, 0.17320E+01_JPRB ,&
&  0.17410E+01_JPRB, 0.17539E+01_JPRB, 0.17568E+01_JPRB, 0.18260E+01_JPRB, 0.17803E+01_JPRB, 0.17101E+01_JPRB, 0.16262E+01_JPRB ,&
&  0.15538E+01_JPRB, 0.14926E+01_JPRB, 0.14610E+01_JPRB, 0.14950E+01_JPRB, 0.14782E+01_JPRB, 0.15785E+01_JPRB, 0.15413E+01_JPRB ,&
&  0.14604E+01_JPRB, 0.14022E+01_JPRB, 0.13664E+01_JPRB, 0.13434E+01_JPRB, 0.13318E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.13670E+01_JPRB, 0.13521E+01_JPRB, 0.13580E+01_JPRB, 0.13693E+01_JPRB, 0.13559E+01_JPRB, 0.13168E+01_JPRB, 0.12756E+01_JPRB ,&
&  0.12432E+01_JPRB, 0.12080E+01_JPRB, 0.11679E+01_JPRB, 0.11236E+01_JPRB, 0.11083E+01_JPRB, 0.11168E+01_JPRB, 0.11294E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.11242E+01_JPRB, 0.10981E+01_JPRB, 0.10692E+01_JPRB, 0.10574E+01_JPRB, 0.10497E+01_JPRB, 0.10532E+01_JPRB ,&
&  0.10431E+01_JPRB, 0.10390E+01_JPRB, 0.10444E+01_JPRB, 0.10504E+01_JPRB, 0.10565E+01_JPRB, 0.10466E+01_JPRB, 0.10485E+01_JPRB ,&
&  0.11965E+01_JPRB, 0.23931E+01_JPRB, 0.29003E+01_JPRB, 0.30488E+01_JPRB, 0.29031E+01_JPRB, 0.26378E+01_JPRB, 0.25141E+01_JPRB ,&
&  0.23072E+01_JPRB, 0.22385E+01_JPRB, 0.21445E+01_JPRB, 0.20196E+01_JPRB, 0.18750E+01_JPRB, 0.17331E+01_JPRB, 0.16135E+01_JPRB ,&
&  0.15213E+01_JPRB, 0.14715E+01_JPRB, 0.14549E+01_JPRB, 0.14768E+01_JPRB, 0.15142E+01_JPRB, 0.15480E+01_JPRB, 0.15288E+01_JPRB ,&
&  0.14885E+01_JPRB, 0.14505E+01_JPRB, 0.14644E+01_JPRB, 0.18410E+01_JPRB, 0.32207E+01_JPRB, 0.41237E+01_JPRB, 0.60852E+01_JPRB ,&
&  0.67290E+01_JPRB, 0.50272E+01_JPRB, 0.39690E+01_JPRB, 0.34713E+01_JPRB, 0.31095E+01_JPRB, 0.27469E+01_JPRB, 0.24730E+01_JPRB ,&
&  0.23414E+01_JPRB, 0.22596E+01_JPRB, 0.22278E+01_JPRB, 0.21435E+01_JPRB, 0.19898E+01_JPRB, 0.18528E+01_JPRB, 0.17856E+01_JPRB ,&
&  0.17482E+01_JPRB, 0.17192E+01_JPRB, 0.17251E+01_JPRB, 0.17448E+01_JPRB, 0.17369E+01_JPRB, 0.16819E+01_JPRB, 0.16517E+01_JPRB ,&
&  0.16526E+01_JPRB, 0.16880E+01_JPRB, 0.17287E+01_JPRB, 0.17421E+01_JPRB, 0.17461E+01_JPRB, 0.17570E+01_JPRB, 0.16810E+01_JPRB ,&
&  0.15617E+01_JPRB, 0.14844E+01_JPRB, 0.14610E+01_JPRB, 0.14711E+01_JPRB, 0.15014E+01_JPRB, 0.16106E+01_JPRB, 0.15632E+01_JPRB ,&
&  0.14895E+01_JPRB, 0.14295E+01_JPRB, 0.14006E+01_JPRB, 0.13793E+01_JPRB, 0.13894E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.13908E+01_JPRB, 0.13765E+01_JPRB, 0.13817E+01_JPRB, 0.13857E+01_JPRB, 0.13738E+01_JPRB, 0.13353E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.12762E+01_JPRB, 0.12247E+01_JPRB, 0.11725E+01_JPRB, 0.11328E+01_JPRB, 0.11275E+01_JPRB, 0.11357E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.11298E+01_JPRB, 0.11085E+01_JPRB, 0.10845E+01_JPRB, 0.10696E+01_JPRB, 0.10547E+01_JPRB, 0.10329E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.10242E+01_JPRB, 0.10283E+01_JPRB, 0.10381E+01_JPRB, 0.10367E+01_JPRB, 0.10402E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.16244E+01_JPRB, 0.18856E+01_JPRB, 0.23530E+01_JPRB, 0.25424E+01_JPRB, 0.24449E+01_JPRB, 0.23212E+01_JPRB ,&
&  0.21591E+01_JPRB, 0.20519E+01_JPRB, 0.19968E+01_JPRB, 0.19248E+01_JPRB, 0.18109E+01_JPRB, 0.16910E+01_JPRB, 0.15834E+01_JPRB ,&
&  0.15073E+01_JPRB, 0.14605E+01_JPRB, 0.14464E+01_JPRB, 0.14321E+01_JPRB, 0.14241E+01_JPRB, 0.14400E+01_JPRB, 0.14552E+01_JPRB ,&
&  0.14089E+01_JPRB, 0.13676E+01_JPRB, 0.13406E+01_JPRB, 0.14195E+01_JPRB, 0.19931E+01_JPRB, 0.27907E+01_JPRB, 0.38487E+01_JPRB ,&
&  0.36158E+01_JPRB, 0.31691E+01_JPRB, 0.28850E+01_JPRB, 0.26665E+01_JPRB, 0.25018E+01_JPRB, 0.22890E+01_JPRB, 0.20985E+01_JPRB ,&
&  0.19334E+01_JPRB, 0.18524E+01_JPRB, 0.17846E+01_JPRB, 0.17439E+01_JPRB, 0.16944E+01_JPRB, 0.16595E+01_JPRB, 0.15885E+01_JPRB ,&
&  0.15419E+01_JPRB, 0.15115E+01_JPRB, 0.15047E+01_JPRB, 0.15356E+01_JPRB, 0.15839E+01_JPRB, 0.16002E+01_JPRB, 0.16054E+01_JPRB ,&
&  0.16221E+01_JPRB, 0.16360E+01_JPRB, 0.16794E+01_JPRB, 0.17280E+01_JPRB, 0.16963E+01_JPRB, 0.16487E+01_JPRB, 0.16118E+01_JPRB ,&
&  0.15765E+01_JPRB, 0.15148E+01_JPRB, 0.14624E+01_JPRB, 0.14724E+01_JPRB, 0.14814E+01_JPRB, 0.15768E+01_JPRB, 0.15578E+01_JPRB ,&
&  0.14970E+01_JPRB, 0.14211E+01_JPRB, 0.13902E+01_JPRB, 0.13737E+01_JPRB, 0.13919E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.13897E+01_JPRB, 0.13966E+01_JPRB, 0.13915E+01_JPRB, 0.13849E+01_JPRB, 0.13723E+01_JPRB, 0.13399E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.12714E+01_JPRB, 0.12344E+01_JPRB, 0.11824E+01_JPRB, 0.11496E+01_JPRB, 0.11364E+01_JPRB, 0.11315E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.11169E+01_JPRB, 0.11044E+01_JPRB, 0.10977E+01_JPRB, 0.10833E+01_JPRB, 0.10701E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.10067E+01_JPRB, 0.10030E+01_JPRB, 0.10112E+01_JPRB, 0.10195E+01_JPRB, 0.10254E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.11400E+01_JPRB, 0.13202E+01_JPRB, 0.16465E+01_JPRB, 0.18902E+01_JPRB, 0.20280E+01_JPRB, 0.19812E+01_JPRB ,&
&  0.18931E+01_JPRB, 0.18076E+01_JPRB, 0.17583E+01_JPRB, 0.16909E+01_JPRB, 0.16083E+01_JPRB, 0.15221E+01_JPRB, 0.14578E+01_JPRB ,&
&  0.14271E+01_JPRB, 0.13990E+01_JPRB, 0.13749E+01_JPRB, 0.13563E+01_JPRB, 0.13459E+01_JPRB, 0.13453E+01_JPRB, 0.13627E+01_JPRB ,&
&  0.13583E+01_JPRB, 0.13225E+01_JPRB, 0.12815E+01_JPRB, 0.12584E+01_JPRB, 0.13523E+01_JPRB, 0.16203E+01_JPRB, 0.20945E+01_JPRB ,&
&  0.23472E+01_JPRB, 0.22038E+01_JPRB, 0.20903E+01_JPRB, 0.19891E+01_JPRB, 0.19023E+01_JPRB, 0.18394E+01_JPRB, 0.17764E+01_JPRB ,&
&  0.17272E+01_JPRB, 0.16933E+01_JPRB, 0.16724E+01_JPRB, 0.16478E+01_JPRB, 0.16186E+01_JPRB, 0.15863E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.15106E+01_JPRB, 0.14902E+01_JPRB, 0.14738E+01_JPRB, 0.14869E+01_JPRB, 0.15088E+01_JPRB, 0.15287E+01_JPRB, 0.15474E+01_JPRB ,&
&  0.15704E+01_JPRB, 0.16140E+01_JPRB, 0.16442E+01_JPRB, 0.16687E+01_JPRB, 0.16459E+01_JPRB, 0.16026E+01_JPRB, 0.15720E+01_JPRB ,&
&  0.15363E+01_JPRB, 0.15037E+01_JPRB, 0.14318E+01_JPRB, 0.14270E+01_JPRB, 0.15340E+01_JPRB, 0.15214E+01_JPRB, 0.15022E+01_JPRB ,&
&  0.14871E+01_JPRB, 0.14453E+01_JPRB, 0.13854E+01_JPRB, 0.13281E+01_JPRB, 0.13463E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.12550E+01_JPRB, 0.12881E+01_JPRB, 0.12882E+01_JPRB, 0.12866E+01_JPRB, 0.12842E+01_JPRB, 0.12706E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.12251E+01_JPRB, 0.12094E+01_JPRB, 0.11867E+01_JPRB, 0.11572E+01_JPRB, 0.11355E+01_JPRB, 0.11194E+01_JPRB, 0.11045E+01_JPRB ,&
&  0.10938E+01_JPRB, 0.10831E+01_JPRB, 0.10739E+01_JPRB, 0.10639E+01_JPRB, 0.10487E+01_JPRB, 0.10357E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.99898E+00_JPRB, 0.99763E+00_JPRB, 0.10015E+01_JPRB, 0.10101E+01_JPRB, 0.10205E+01_JPRB, 0.10282E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10012E+01_JPRB, 0.10629E+01_JPRB, 0.12346E+01_JPRB, 0.14529E+01_JPRB, 0.16400E+01_JPRB, 0.16935E+01_JPRB ,&
&  0.16640E+01_JPRB, 0.16155E+01_JPRB, 0.15553E+01_JPRB, 0.14959E+01_JPRB, 0.14433E+01_JPRB, 0.14044E+01_JPRB, 0.13488E+01_JPRB ,&
&  0.13129E+01_JPRB, 0.12920E+01_JPRB, 0.12856E+01_JPRB, 0.12745E+01_JPRB, 0.12608E+01_JPRB, 0.12518E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.12602E+01_JPRB, 0.12420E+01_JPRB, 0.12167E+01_JPRB, 0.12019E+01_JPRB, 0.12223E+01_JPRB, 0.13353E+01_JPRB, 0.15315E+01_JPRB ,&
&  0.17171E+01_JPRB, 0.17007E+01_JPRB, 0.16434E+01_JPRB, 0.15799E+01_JPRB, 0.15448E+01_JPRB, 0.15267E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.14996E+01_JPRB, 0.14885E+01_JPRB, 0.15014E+01_JPRB, 0.15080E+01_JPRB, 0.15166E+01_JPRB, 0.15150E+01_JPRB, 0.15101E+01_JPRB ,&
&  0.14948E+01_JPRB, 0.14740E+01_JPRB, 0.14638E+01_JPRB, 0.14643E+01_JPRB, 0.14781E+01_JPRB, 0.14867E+01_JPRB, 0.14824E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.14897E+01_JPRB, 0.15368E+01_JPRB, 0.15742E+01_JPRB, 0.15582E+01_JPRB, 0.15300E+01_JPRB, 0.15014E+01_JPRB ,&
&  0.14855E+01_JPRB, 0.14614E+01_JPRB, 0.14441E+01_JPRB, 0.14695E+01_JPRB, 0.14658E+01_JPRB, 0.14483E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.14127E+01_JPRB, 0.13858E+01_JPRB, 0.13579E+01_JPRB, 0.13057E+01_JPRB, 0.11948E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.11390E+01_JPRB, 0.11720E+01_JPRB, 0.11905E+01_JPRB, 0.11956E+01_JPRB, 0.12046E+01_JPRB, 0.12113E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.12093E+01_JPRB, 0.12034E+01_JPRB, 0.11884E+01_JPRB, 0.11638E+01_JPRB, 0.11381E+01_JPRB, 0.11096E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10625E+01_JPRB, 0.10427E+01_JPRB, 0.10247E+01_JPRB, 0.10086E+01_JPRB, 0.99571E+00_JPRB, 0.98918E+00_JPRB ,&
&  0.98194E+00_JPRB, 0.97805E+00_JPRB, 0.98216E+00_JPRB, 0.98832E+00_JPRB, 0.99760E+00_JPRB, 0.10083E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.87143E+00_JPRB, 0.91391E+00_JPRB, 0.10356E+01_JPRB, 0.11602E+01_JPRB, 0.12990E+01_JPRB, 0.14175E+01_JPRB ,&
&  0.14485E+01_JPRB, 0.14610E+01_JPRB, 0.14290E+01_JPRB, 0.13903E+01_JPRB, 0.13666E+01_JPRB, 0.13285E+01_JPRB, 0.12678E+01_JPRB ,&
&  0.12208E+01_JPRB, 0.12082E+01_JPRB, 0.12137E+01_JPRB, 0.12137E+01_JPRB, 0.12020E+01_JPRB, 0.11884E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.11840E+01_JPRB, 0.11864E+01_JPRB, 0.11883E+01_JPRB, 0.11885E+01_JPRB, 0.12101E+01_JPRB, 0.12642E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.14010E+01_JPRB, 0.14120E+01_JPRB, 0.13942E+01_JPRB, 0.13603E+01_JPRB, 0.13403E+01_JPRB, 0.13446E+01_JPRB, 0.13462E+01_JPRB ,&
&  0.13423E+01_JPRB, 0.13391E+01_JPRB, 0.13487E+01_JPRB, 0.13598E+01_JPRB, 0.13780E+01_JPRB, 0.13966E+01_JPRB, 0.14253E+01_JPRB ,&
&  0.14479E+01_JPRB, 0.14522E+01_JPRB, 0.14515E+01_JPRB, 0.14371E+01_JPRB, 0.14109E+01_JPRB, 0.13918E+01_JPRB, 0.13835E+01_JPRB ,&
&  0.13789E+01_JPRB, 0.13878E+01_JPRB, 0.14138E+01_JPRB, 0.14406E+01_JPRB, 0.14406E+01_JPRB, 0.14349E+01_JPRB, 0.14288E+01_JPRB ,&
&  0.14266E+01_JPRB, 0.14216E+01_JPRB, 0.14198E+01_JPRB, 0.13476E+01_JPRB, 0.13824E+01_JPRB, 0.13680E+01_JPRB, 0.13363E+01_JPRB ,&
&  0.12998E+01_JPRB, 0.12614E+01_JPRB, 0.12311E+01_JPRB, 0.10449E+01_JPRB, 0.11147E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.10592E+01_JPRB, 0.10787E+01_JPRB, 0.10840E+01_JPRB, 0.10995E+01_JPRB, 0.11111E+01_JPRB, 0.11202E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.11362E+01_JPRB, 0.11284E+01_JPRB, 0.11120E+01_JPRB, 0.10851E+01_JPRB, 0.10562E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.10063E+01_JPRB, 0.99440E+00_JPRB, 0.98038E+00_JPRB, 0.97133E+00_JPRB, 0.96605E+00_JPRB, 0.96643E+00_JPRB ,&
&  0.97299E+00_JPRB, 0.97603E+00_JPRB, 0.96986E+00_JPRB, 0.96742E+00_JPRB, 0.97514E+00_JPRB, 0.98995E+00_JPRB, 0.99602E+00_JPRB ,&
&  0.99791E+00_JPRB, 0.80382E+00_JPRB, 0.85954E+00_JPRB, 0.92933E+00_JPRB, 0.10045E+01_JPRB, 0.11071E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.13051E+01_JPRB, 0.12973E+01_JPRB, 0.12755E+01_JPRB, 0.12486E+01_JPRB, 0.12240E+01_JPRB, 0.11921E+01_JPRB ,&
&  0.11605E+01_JPRB, 0.11440E+01_JPRB, 0.11415E+01_JPRB, 0.11424E+01_JPRB, 0.11417E+01_JPRB, 0.11441E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.11341E+01_JPRB, 0.11355E+01_JPRB, 0.11393E+01_JPRB, 0.11428E+01_JPRB, 0.11523E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.12023E+01_JPRB, 0.12040E+01_JPRB, 0.11918E+01_JPRB, 0.11865E+01_JPRB, 0.11920E+01_JPRB, 0.12052E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.12305E+01_JPRB, 0.12391E+01_JPRB, 0.12512E+01_JPRB, 0.12693E+01_JPRB, 0.12912E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.13344E+01_JPRB, 0.13389E+01_JPRB, 0.13420E+01_JPRB, 0.13338E+01_JPRB, 0.13293E+01_JPRB, 0.13057E+01_JPRB, 0.12793E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.12727E+01_JPRB, 0.12772E+01_JPRB, 0.12928E+01_JPRB, 0.12992E+01_JPRB, 0.12943E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.13009E+01_JPRB, 0.12997E+01_JPRB, 0.12844E+01_JPRB, 0.12702E+01_JPRB, 0.12449E+01_JPRB, 0.12236E+01_JPRB, 0.11929E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.11293E+01_JPRB, 0.91652E+00_JPRB, 0.10107E+01_JPRB, 0.10324E+01_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.10281E+01_JPRB, 0.10339E+01_JPRB, 0.10372E+01_JPRB, 0.10427E+01_JPRB, 0.10453E+01_JPRB, 0.10459E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10563E+01_JPRB, 0.10483E+01_JPRB, 0.10447E+01_JPRB, 0.10336E+01_JPRB, 0.10174E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.98596E+00_JPRB, 0.97765E+00_JPRB, 0.97212E+00_JPRB, 0.96645E+00_JPRB, 0.95794E+00_JPRB, 0.94808E+00_JPRB, 0.95079E+00_JPRB ,&
&  0.95728E+00_JPRB, 0.96444E+00_JPRB, 0.96575E+00_JPRB, 0.96867E+00_JPRB, 0.97359E+00_JPRB, 0.97918E+00_JPRB, 0.98922E+00_JPRB ,&
&  0.87475E+00_JPRB, 0.74842E+00_JPRB, 0.80665E+00_JPRB, 0.86816E+00_JPRB, 0.92938E+00_JPRB, 0.10245E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.11569E+01_JPRB, 0.11955E+01_JPRB, 0.12027E+01_JPRB, 0.11753E+01_JPRB, 0.11470E+01_JPRB, 0.11399E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.11280E+01_JPRB, 0.11264E+01_JPRB, 0.11271E+01_JPRB, 0.11302E+01_JPRB, 0.11359E+01_JPRB, 0.11383E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.11223E+01_JPRB, 0.11061E+01_JPRB, 0.10822E+01_JPRB, 0.10606E+01_JPRB, 0.10529E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10458E+01_JPRB, 0.10539E+01_JPRB, 0.10658E+01_JPRB, 0.10735E+01_JPRB, 0.10850E+01_JPRB, 0.11002E+01_JPRB, 0.11213E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.11704E+01_JPRB, 0.11888E+01_JPRB, 0.12074E+01_JPRB, 0.12159E+01_JPRB, 0.12254E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.12433E+01_JPRB, 0.12305E+01_JPRB, 0.12195E+01_JPRB, 0.12173E+01_JPRB, 0.12177E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.11918E+01_JPRB, 0.11883E+01_JPRB, 0.11933E+01_JPRB, 0.11900E+01_JPRB, 0.11799E+01_JPRB, 0.11710E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11435E+01_JPRB, 0.11239E+01_JPRB, 0.11095E+01_JPRB, 0.10915E+01_JPRB, 0.10772E+01_JPRB, 0.10636E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.10345E+01_JPRB, 0.10360E+01_JPRB, 0.10280E+01_JPRB, 0.10239E+01_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.98525E+00_JPRB, 0.98913E+00_JPRB, 0.99358E+00_JPRB, 0.99530E+00_JPRB, 0.99309E+00_JPRB, 0.99237E+00_JPRB, 0.99608E+00_JPRB ,&
&  0.99948E+00_JPRB, 0.99339E+00_JPRB, 0.98664E+00_JPRB, 0.98102E+00_JPRB, 0.97810E+00_JPRB, 0.96769E+00_JPRB, 0.95201E+00_JPRB ,&
&  0.94044E+00_JPRB, 0.93782E+00_JPRB, 0.94181E+00_JPRB, 0.94352E+00_JPRB, 0.93367E+00_JPRB, 0.92358E+00_JPRB, 0.91985E+00_JPRB ,&
&  0.92061E+00_JPRB, 0.92631E+00_JPRB, 0.93600E+00_JPRB, 0.94097E+00_JPRB, 0.94875E+00_JPRB, 0.96207E+00_JPRB, 0.96157E+00_JPRB ,&
&  0.95881E+00_JPRB, 0.82023E+00_JPRB, 0.86159E+00_JPRB, 0.87866E+00_JPRB, 0.91315E+00_JPRB, 0.96750E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.10939E+01_JPRB, 0.11062E+01_JPRB, 0.10989E+01_JPRB, 0.10804E+01_JPRB, 0.10634E+01_JPRB, 0.10609E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10741E+01_JPRB, 0.10883E+01_JPRB, 0.10973E+01_JPRB, 0.11000E+01_JPRB, 0.11010E+01_JPRB, 0.11039E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.10823E+01_JPRB, 0.10549E+01_JPRB, 0.10205E+01_JPRB, 0.99143E+00_JPRB, 0.97506E+00_JPRB, 0.96129E+00_JPRB ,&
&  0.94491E+00_JPRB, 0.94058E+00_JPRB, 0.94862E+00_JPRB, 0.95963E+00_JPRB, 0.97537E+00_JPRB, 0.99323E+00_JPRB, 0.10082E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.10455E+01_JPRB, 0.10690E+01_JPRB, 0.10906E+01_JPRB, 0.11084E+01_JPRB, 0.11213E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11233E+01_JPRB, 0.11089E+01_JPRB, 0.10993E+01_JPRB, 0.10969E+01_JPRB, 0.11038E+01_JPRB, 0.11069E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.10918E+01_JPRB, 0.10862E+01_JPRB, 0.10727E+01_JPRB, 0.10571E+01_JPRB, 0.10414E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.10125E+01_JPRB, 0.10054E+01_JPRB, 0.99877E+00_JPRB, 0.98950E+00_JPRB, 0.97626E+00_JPRB, 0.96754E+00_JPRB, 0.96002E+00_JPRB ,&
&  0.95371E+00_JPRB, 0.94752E+00_JPRB, 0.94815E+00_JPRB, 0.95874E+00_JPRB, 0.97443E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.93197E+00_JPRB, 0.93843E+00_JPRB, 0.94854E+00_JPRB, 0.96038E+00_JPRB, 0.96557E+00_JPRB, 0.96672E+00_JPRB, 0.96584E+00_JPRB ,&
&  0.96800E+00_JPRB, 0.96109E+00_JPRB, 0.95596E+00_JPRB, 0.95269E+00_JPRB, 0.95171E+00_JPRB, 0.94725E+00_JPRB, 0.93649E+00_JPRB ,&
&  0.92725E+00_JPRB, 0.92284E+00_JPRB, 0.92610E+00_JPRB, 0.92643E+00_JPRB, 0.90999E+00_JPRB, 0.90416E+00_JPRB, 0.89552E+00_JPRB ,&
&  0.89372E+00_JPRB, 0.89273E+00_JPRB, 0.89454E+00_JPRB, 0.89933E+00_JPRB, 0.91450E+00_JPRB, 0.92507E+00_JPRB, 0.93019E+00_JPRB ,&
&  0.92678E+00_JPRB, 0.85488E+00_JPRB, 0.88385E+00_JPRB, 0.89422E+00_JPRB, 0.91587E+00_JPRB, 0.93949E+00_JPRB, 0.96361E+00_JPRB ,&
&  0.99624E+00_JPRB, 0.10158E+01_JPRB, 0.10260E+01_JPRB, 0.10299E+01_JPRB, 0.10320E+01_JPRB, 0.10305E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.10277E+01_JPRB, 0.10480E+01_JPRB, 0.10594E+01_JPRB, 0.10659E+01_JPRB, 0.10696E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10439E+01_JPRB, 0.10159E+01_JPRB, 0.98127E+00_JPRB, 0.94195E+00_JPRB, 0.91304E+00_JPRB, 0.89637E+00_JPRB ,&
&  0.88786E+00_JPRB, 0.89066E+00_JPRB, 0.89760E+00_JPRB, 0.90499E+00_JPRB, 0.91371E+00_JPRB, 0.92745E+00_JPRB, 0.93639E+00_JPRB ,&
&  0.94723E+00_JPRB, 0.95665E+00_JPRB, 0.96962E+00_JPRB, 0.98960E+00_JPRB, 0.10087E+01_JPRB, 0.10185E+01_JPRB, 0.10183E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.10275E+01_JPRB, 0.10240E+01_JPRB, 0.10234E+01_JPRB, 0.10188E+01_JPRB, 0.10200E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.10017E+01_JPRB, 0.99237E+00_JPRB, 0.97515E+00_JPRB, 0.95911E+00_JPRB, 0.94654E+00_JPRB, 0.93582E+00_JPRB ,&
&  0.93266E+00_JPRB, 0.93513E+00_JPRB, 0.93952E+00_JPRB, 0.93330E+00_JPRB, 0.91742E+00_JPRB, 0.90640E+00_JPRB, 0.89404E+00_JPRB ,&
&  0.88575E+00_JPRB, 0.88719E+00_JPRB, 0.88675E+00_JPRB, 0.89421E+00_JPRB, 0.91407E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.87724E+00_JPRB, 0.89060E+00_JPRB, 0.90753E+00_JPRB, 0.92532E+00_JPRB, 0.93727E+00_JPRB, 0.94504E+00_JPRB, 0.94416E+00_JPRB ,&
&  0.93425E+00_JPRB, 0.92440E+00_JPRB, 0.92707E+00_JPRB, 0.92758E+00_JPRB, 0.92656E+00_JPRB, 0.92427E+00_JPRB, 0.91674E+00_JPRB ,&
&  0.90595E+00_JPRB, 0.89649E+00_JPRB, 0.89200E+00_JPRB, 0.88827E+00_JPRB, 0.87760E+00_JPRB, 0.86915E+00_JPRB, 0.85991E+00_JPRB ,&
&  0.85862E+00_JPRB, 0.86007E+00_JPRB, 0.86165E+00_JPRB, 0.86040E+00_JPRB, 0.86688E+00_JPRB, 0.87567E+00_JPRB, 0.87894E+00_JPRB ,&
&  0.87222E+00_JPRB, 0.88329E+00_JPRB, 0.79762E+00_JPRB, 0.86692E+00_JPRB, 0.89079E+00_JPRB, 0.90515E+00_JPRB, 0.92243E+00_JPRB ,&
&  0.93585E+00_JPRB, 0.94990E+00_JPRB, 0.96041E+00_JPRB, 0.96799E+00_JPRB, 0.98846E+00_JPRB, 0.10029E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.10063E+01_JPRB, 0.10050E+01_JPRB, 0.10069E+01_JPRB, 0.10090E+01_JPRB, 0.10158E+01_JPRB, 0.10183E+01_JPRB, 0.10141E+01_JPRB ,&
&  0.10068E+01_JPRB, 0.99445E+00_JPRB, 0.97509E+00_JPRB, 0.95177E+00_JPRB, 0.91562E+00_JPRB, 0.88407E+00_JPRB, 0.86366E+00_JPRB ,&
&  0.84809E+00_JPRB, 0.84411E+00_JPRB, 0.84029E+00_JPRB, 0.83695E+00_JPRB, 0.83741E+00_JPRB, 0.84145E+00_JPRB, 0.84257E+00_JPRB ,&
&  0.85059E+00_JPRB, 0.85983E+00_JPRB, 0.86408E+00_JPRB, 0.87207E+00_JPRB, 0.88302E+00_JPRB, 0.89861E+00_JPRB, 0.91360E+00_JPRB ,&
&  0.92779E+00_JPRB, 0.93820E+00_JPRB, 0.94096E+00_JPRB, 0.94172E+00_JPRB, 0.93899E+00_JPRB, 0.93709E+00_JPRB, 0.92954E+00_JPRB ,&
&  0.92363E+00_JPRB, 0.92046E+00_JPRB, 0.91357E+00_JPRB, 0.90051E+00_JPRB, 0.89153E+00_JPRB, 0.88372E+00_JPRB, 0.88101E+00_JPRB ,&
&  0.88089E+00_JPRB, 0.88172E+00_JPRB, 0.87911E+00_JPRB, 0.87957E+00_JPRB, 0.87244E+00_JPRB, 0.85059E+00_JPRB, 0.83325E+00_JPRB ,&
&  0.83147E+00_JPRB, 0.83626E+00_JPRB, 0.84066E+00_JPRB, 0.84927E+00_JPRB, 0.86453E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.88916E+00_JPRB, 0.89556E+00_JPRB, 0.90559E+00_JPRB, 0.92032E+00_JPRB, 0.93601E+00_JPRB, 0.94670E+00_JPRB, 0.94750E+00_JPRB ,&
&  0.93730E+00_JPRB, 0.92447E+00_JPRB, 0.92050E+00_JPRB, 0.92022E+00_JPRB, 0.91742E+00_JPRB, 0.91592E+00_JPRB, 0.91093E+00_JPRB ,&
&  0.89878E+00_JPRB, 0.88055E+00_JPRB, 0.87155E+00_JPRB, 0.85681E+00_JPRB, 0.84289E+00_JPRB, 0.83119E+00_JPRB, 0.83022E+00_JPRB ,&
&  0.83806E+00_JPRB, 0.84745E+00_JPRB, 0.85916E+00_JPRB, 0.87210E+00_JPRB, 0.87800E+00_JPRB, 0.87504E+00_JPRB, 0.87059E+00_JPRB ,&
&  0.86679E+00_JPRB, 0.86362E+00_JPRB, 0.87314E+00_JPRB, 0.88123E+00_JPRB, 0.88649E+00_JPRB, 0.89723E+00_JPRB, 0.90704E+00_JPRB ,&
&  0.91640E+00_JPRB, 0.92917E+00_JPRB, 0.94917E+00_JPRB, 0.96429E+00_JPRB, 0.97623E+00_JPRB, 0.98820E+00_JPRB, 0.99072E+00_JPRB ,&
&  0.99587E+00_JPRB, 0.10068E+01_JPRB, 0.10092E+01_JPRB, 0.10004E+01_JPRB, 0.98685E+00_JPRB, 0.97909E+00_JPRB, 0.97572E+00_JPRB ,&
&  0.96783E+00_JPRB, 0.94886E+00_JPRB, 0.93391E+00_JPRB, 0.92125E+00_JPRB, 0.90049E+00_JPRB, 0.87845E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.82469E+00_JPRB, 0.80360E+00_JPRB, 0.78730E+00_JPRB, 0.77253E+00_JPRB, 0.76856E+00_JPRB, 0.75955E+00_JPRB, 0.75340E+00_JPRB ,&
&  0.75876E+00_JPRB, 0.77870E+00_JPRB, 0.79421E+00_JPRB, 0.80091E+00_JPRB, 0.80511E+00_JPRB, 0.82252E+00_JPRB, 0.84341E+00_JPRB ,&
&  0.86004E+00_JPRB, 0.86793E+00_JPRB, 0.86533E+00_JPRB, 0.86873E+00_JPRB, 0.87455E+00_JPRB, 0.87893E+00_JPRB, 0.87226E+00_JPRB ,&
&  0.86299E+00_JPRB, 0.85711E+00_JPRB, 0.85559E+00_JPRB, 0.85145E+00_JPRB, 0.85254E+00_JPRB, 0.85213E+00_JPRB, 0.84912E+00_JPRB ,&
&  0.84336E+00_JPRB, 0.83439E+00_JPRB, 0.82241E+00_JPRB, 0.81425E+00_JPRB, 0.80231E+00_JPRB, 0.78444E+00_JPRB, 0.78370E+00_JPRB ,&
&  0.79445E+00_JPRB, 0.81251E+00_JPRB, 0.83033E+00_JPRB, 0.85194E+00_JPRB, 0.87734E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.91248E+00_JPRB, 0.92061E+00_JPRB, 0.92622E+00_JPRB, 0.93057E+00_JPRB, 0.94025E+00_JPRB, 0.95249E+00_JPRB, 0.95324E+00_JPRB ,&
&  0.93906E+00_JPRB, 0.92540E+00_JPRB, 0.91579E+00_JPRB, 0.91335E+00_JPRB, 0.91532E+00_JPRB, 0.91779E+00_JPRB, 0.91172E+00_JPRB ,&
&  0.88914E+00_JPRB, 0.86496E+00_JPRB, 0.84998E+00_JPRB, 0.83521E+00_JPRB, 0.82579E+00_JPRB, 0.81949E+00_JPRB, 0.82551E+00_JPRB ,&
&  0.83470E+00_JPRB, 0.84970E+00_JPRB, 0.86163E+00_JPRB, 0.87002E+00_JPRB, 0.87291E+00_JPRB, 0.86054E+00_JPRB, 0.86097E+00_JPRB ,&
&  0.86707E+00_JPRB, 0.86845E+00_JPRB, 0.88270E+00_JPRB, 0.88145E+00_JPRB, 0.86721E+00_JPRB, 0.85538E+00_JPRB, 0.84405E+00_JPRB ,&
&  0.84127E+00_JPRB, 0.85271E+00_JPRB, 0.87847E+00_JPRB, 0.91594E+00_JPRB, 0.93881E+00_JPRB, 0.94789E+00_JPRB, 0.94404E+00_JPRB ,&
&  0.94555E+00_JPRB, 0.95704E+00_JPRB, 0.96368E+00_JPRB, 0.95563E+00_JPRB, 0.93787E+00_JPRB, 0.92360E+00_JPRB, 0.91143E+00_JPRB ,&
&  0.90314E+00_JPRB, 0.89278E+00_JPRB, 0.88541E+00_JPRB, 0.87569E+00_JPRB, 0.85311E+00_JPRB, 0.82624E+00_JPRB, 0.80255E+00_JPRB ,&
&  0.78464E+00_JPRB, 0.76686E+00_JPRB, 0.74548E+00_JPRB, 0.73802E+00_JPRB, 0.74119E+00_JPRB, 0.71815E+00_JPRB, 0.69385E+00_JPRB ,&
&  0.69827E+00_JPRB, 0.73095E+00_JPRB, 0.75704E+00_JPRB, 0.76861E+00_JPRB, 0.77100E+00_JPRB, 0.77646E+00_JPRB, 0.78100E+00_JPRB ,&
&  0.78886E+00_JPRB, 0.79880E+00_JPRB, 0.80654E+00_JPRB, 0.81858E+00_JPRB, 0.83081E+00_JPRB, 0.82914E+00_JPRB, 0.81401E+00_JPRB ,&
&  0.79780E+00_JPRB, 0.78885E+00_JPRB, 0.78549E+00_JPRB, 0.78811E+00_JPRB, 0.79178E+00_JPRB, 0.79004E+00_JPRB, 0.78431E+00_JPRB ,&
&  0.77234E+00_JPRB, 0.75387E+00_JPRB, 0.73797E+00_JPRB, 0.72827E+00_JPRB, 0.72674E+00_JPRB, 0.73238E+00_JPRB, 0.74831E+00_JPRB ,&
&  0.77967E+00_JPRB, 0.82766E+00_JPRB, 0.86435E+00_JPRB, 0.88640E+00_JPRB, 0.89411E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.93725E+00_JPRB, 0.94574E+00_JPRB, 0.95569E+00_JPRB, 0.96932E+00_JPRB, 0.98216E+00_JPRB, 0.97954E+00_JPRB, 0.96705E+00_JPRB ,&
&  0.94346E+00_JPRB, 0.93417E+00_JPRB, 0.93854E+00_JPRB, 0.94084E+00_JPRB, 0.94253E+00_JPRB, 0.94023E+00_JPRB, 0.92358E+00_JPRB ,&
&  0.88431E+00_JPRB, 0.85296E+00_JPRB, 0.83776E+00_JPRB, 0.82959E+00_JPRB, 0.82664E+00_JPRB, 0.83275E+00_JPRB, 0.83976E+00_JPRB ,&
&  0.85474E+00_JPRB, 0.86315E+00_JPRB, 0.87434E+00_JPRB, 0.88350E+00_JPRB, 0.88620E+00_JPRB, 0.88991E+00_JPRB, 0.89035E+00_JPRB ,&
&  0.87519E+00_JPRB, 0.87476E+00_JPRB, 0.87506E+00_JPRB, 0.86443E+00_JPRB, 0.84136E+00_JPRB, 0.81821E+00_JPRB, 0.79034E+00_JPRB ,&
&  0.77159E+00_JPRB, 0.76214E+00_JPRB, 0.78778E+00_JPRB, 0.85051E+00_JPRB, 0.89525E+00_JPRB, 0.90352E+00_JPRB, 0.89452E+00_JPRB ,&
&  0.89527E+00_JPRB, 0.90228E+00_JPRB, 0.90604E+00_JPRB, 0.89489E+00_JPRB, 0.87545E+00_JPRB, 0.85842E+00_JPRB, 0.84252E+00_JPRB ,&
&  0.83808E+00_JPRB, 0.83856E+00_JPRB, 0.82245E+00_JPRB, 0.79086E+00_JPRB, 0.76287E+00_JPRB, 0.74288E+00_JPRB, 0.73211E+00_JPRB ,&
&  0.72510E+00_JPRB, 0.71454E+00_JPRB, 0.70736E+00_JPRB, 0.71119E+00_JPRB, 0.69069E+00_JPRB, 0.67284E+00_JPRB, 0.67696E+00_JPRB ,&
&  0.69388E+00_JPRB, 0.70054E+00_JPRB, 0.71095E+00_JPRB, 0.71544E+00_JPRB, 0.71386E+00_JPRB, 0.71149E+00_JPRB, 0.71380E+00_JPRB ,&
&  0.72373E+00_JPRB, 0.73504E+00_JPRB, 0.74342E+00_JPRB, 0.74882E+00_JPRB, 0.75397E+00_JPRB, 0.74134E+00_JPRB, 0.71964E+00_JPRB ,&
&  0.70235E+00_JPRB, 0.69954E+00_JPRB, 0.70433E+00_JPRB, 0.70394E+00_JPRB, 0.70255E+00_JPRB, 0.70303E+00_JPRB, 0.70226E+00_JPRB ,&
&  0.69999E+00_JPRB, 0.69319E+00_JPRB, 0.68477E+00_JPRB, 0.68439E+00_JPRB, 0.69506E+00_JPRB, 0.71823E+00_JPRB, 0.75428E+00_JPRB ,&
&  0.80677E+00_JPRB, 0.84620E+00_JPRB, 0.88548E+00_JPRB, 0.90300E+00_JPRB, 0.90851E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.96852E+00_JPRB, 0.99136E+00_JPRB, 0.10106E+01_JPRB, 0.10237E+01_JPRB, 0.10259E+01_JPRB, 0.10190E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.97229E+00_JPRB, 0.95486E+00_JPRB, 0.96366E+00_JPRB, 0.97826E+00_JPRB, 0.98687E+00_JPRB, 0.98659E+00_JPRB, 0.95436E+00_JPRB ,&
&  0.89822E+00_JPRB, 0.87663E+00_JPRB, 0.87296E+00_JPRB, 0.88187E+00_JPRB, 0.89900E+00_JPRB, 0.90444E+00_JPRB, 0.89066E+00_JPRB ,&
&  0.87699E+00_JPRB, 0.87756E+00_JPRB, 0.87969E+00_JPRB, 0.88649E+00_JPRB, 0.87628E+00_JPRB, 0.85513E+00_JPRB, 0.84515E+00_JPRB ,&
&  0.84138E+00_JPRB, 0.83212E+00_JPRB, 0.82225E+00_JPRB, 0.77340E+00_JPRB, 0.79341E+00_JPRB, 0.78109E+00_JPRB, 0.76149E+00_JPRB ,&
&  0.74284E+00_JPRB, 0.74461E+00_JPRB, 0.78716E+00_JPRB, 0.83484E+00_JPRB, 0.84684E+00_JPRB, 0.84908E+00_JPRB, 0.84283E+00_JPRB ,&
&  0.83857E+00_JPRB, 0.83881E+00_JPRB, 0.84032E+00_JPRB, 0.83223E+00_JPRB, 0.81203E+00_JPRB, 0.78587E+00_JPRB, 0.75742E+00_JPRB ,&
&  0.74067E+00_JPRB, 0.71662E+00_JPRB, 0.67364E+00_JPRB, 0.65371E+00_JPRB, 0.65238E+00_JPRB, 0.66080E+00_JPRB, 0.66473E+00_JPRB ,&
&  0.66166E+00_JPRB, 0.65375E+00_JPRB, 0.65607E+00_JPRB, 0.65189E+00_JPRB, 0.62967E+00_JPRB, 0.61553E+00_JPRB, 0.60920E+00_JPRB ,&
&  0.59852E+00_JPRB, 0.59911E+00_JPRB, 0.60562E+00_JPRB, 0.61857E+00_JPRB, 0.62992E+00_JPRB, 0.64571E+00_JPRB, 0.65296E+00_JPRB ,&
&  0.65586E+00_JPRB, 0.65161E+00_JPRB, 0.64476E+00_JPRB, 0.63193E+00_JPRB, 0.61259E+00_JPRB, 0.60512E+00_JPRB, 0.60691E+00_JPRB ,&
&  0.61654E+00_JPRB, 0.62363E+00_JPRB, 0.62402E+00_JPRB, 0.62347E+00_JPRB, 0.61216E+00_JPRB, 0.59961E+00_JPRB, 0.59046E+00_JPRB ,&
&  0.58678E+00_JPRB, 0.59089E+00_JPRB, 0.60808E+00_JPRB, 0.63461E+00_JPRB, 0.66799E+00_JPRB, 0.70850E+00_JPRB, 0.75406E+00_JPRB ,&
&  0.78572E+00_JPRB, 0.80370E+00_JPRB, 0.82819E+00_JPRB, 0.87723E+00_JPRB, 0.92561E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.96674E+00_JPRB, 0.99473E+00_JPRB, 0.10245E+01_JPRB, 0.10416E+01_JPRB, 0.10426E+01_JPRB, 0.10396E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.99378E+00_JPRB, 0.96880E+00_JPRB, 0.96486E+00_JPRB, 0.96894E+00_JPRB, 0.98044E+00_JPRB, 0.99859E+00_JPRB, 0.99878E+00_JPRB ,&
&  0.97019E+00_JPRB, 0.94945E+00_JPRB, 0.93875E+00_JPRB, 0.93039E+00_JPRB, 0.91196E+00_JPRB, 0.88605E+00_JPRB, 0.86898E+00_JPRB ,&
&  0.86366E+00_JPRB, 0.86283E+00_JPRB, 0.85357E+00_JPRB, 0.83979E+00_JPRB, 0.80818E+00_JPRB, 0.76898E+00_JPRB, 0.73714E+00_JPRB ,&
&  0.72478E+00_JPRB, 0.71262E+00_JPRB, 0.67289E+00_JPRB, 0.68538E+00_JPRB, 0.68960E+00_JPRB, 0.68987E+00_JPRB, 0.68617E+00_JPRB ,&
&  0.69649E+00_JPRB, 0.72965E+00_JPRB, 0.76330E+00_JPRB, 0.77300E+00_JPRB, 0.78192E+00_JPRB, 0.78330E+00_JPRB, 0.78422E+00_JPRB ,&
&  0.78838E+00_JPRB, 0.78043E+00_JPRB, 0.75083E+00_JPRB, 0.70674E+00_JPRB, 0.66335E+00_JPRB, 0.62965E+00_JPRB, 0.60113E+00_JPRB ,&
&  0.58189E+00_JPRB, 0.56996E+00_JPRB, 0.56044E+00_JPRB, 0.55732E+00_JPRB, 0.56561E+00_JPRB, 0.58118E+00_JPRB, 0.57606E+00_JPRB ,&
&  0.55700E+00_JPRB, 0.53867E+00_JPRB, 0.51528E+00_JPRB, 0.48367E+00_JPRB, 0.45053E+00_JPRB, 0.41375E+00_JPRB, 0.39405E+00_JPRB ,&
&  0.39941E+00_JPRB, 0.43964E+00_JPRB, 0.49181E+00_JPRB, 0.51390E+00_JPRB, 0.54246E+00_JPRB, 0.57070E+00_JPRB, 0.55987E+00_JPRB ,&
&  0.50887E+00_JPRB, 0.44416E+00_JPRB, 0.42145E+00_JPRB, 0.40718E+00_JPRB, 0.40011E+00_JPRB, 0.39571E+00_JPRB, 0.39402E+00_JPRB ,&
&  0.39883E+00_JPRB, 0.40988E+00_JPRB, 0.42045E+00_JPRB, 0.41826E+00_JPRB, 0.40849E+00_JPRB, 0.40391E+00_JPRB, 0.40580E+00_JPRB ,&
&  0.41839E+00_JPRB, 0.44874E+00_JPRB, 0.51126E+00_JPRB, 0.62280E+00_JPRB, 0.66307E+00_JPRB, 0.69861E+00_JPRB, 0.73861E+00_JPRB ,&
&  0.77728E+00_JPRB, 0.81231E+00_JPRB, 0.84158E+00_JPRB, 0.87728E+00_JPRB, 0.92535E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.90203E+00_JPRB, 0.92205E+00_JPRB, 0.94468E+00_JPRB, 0.95660E+00_JPRB, 0.95808E+00_JPRB, 0.95536E+00_JPRB, 0.94679E+00_JPRB ,&
&  0.93195E+00_JPRB, 0.91406E+00_JPRB, 0.90067E+00_JPRB, 0.89123E+00_JPRB, 0.88633E+00_JPRB, 0.88811E+00_JPRB, 0.89195E+00_JPRB ,&
&  0.88949E+00_JPRB, 0.88414E+00_JPRB, 0.87079E+00_JPRB, 0.85358E+00_JPRB, 0.83548E+00_JPRB, 0.82161E+00_JPRB, 0.81122E+00_JPRB ,&
&  0.80117E+00_JPRB, 0.78643E+00_JPRB, 0.76366E+00_JPRB, 0.72816E+00_JPRB, 0.68602E+00_JPRB, 0.64744E+00_JPRB, 0.61568E+00_JPRB ,&
&  0.58572E+00_JPRB, 0.52066E+00_JPRB, 0.48504E+00_JPRB, 0.49310E+00_JPRB, 0.57809E+00_JPRB, 0.61311E+00_JPRB, 0.63906E+00_JPRB ,&
&  0.67423E+00_JPRB, 0.71198E+00_JPRB, 0.73022E+00_JPRB, 0.73026E+00_JPRB, 0.72433E+00_JPRB, 0.71136E+00_JPRB, 0.69124E+00_JPRB ,&
&  0.66583E+00_JPRB, 0.63499E+00_JPRB, 0.60851E+00_JPRB, 0.57833E+00_JPRB, 0.54782E+00_JPRB, 0.52565E+00_JPRB, 0.51037E+00_JPRB ,&
&  0.49989E+00_JPRB, 0.48972E+00_JPRB, 0.48141E+00_JPRB, 0.47131E+00_JPRB, 0.46217E+00_JPRB, 0.44093E+00_JPRB, 0.41738E+00_JPRB ,&
&  0.39617E+00_JPRB, 0.37728E+00_JPRB, 0.35870E+00_JPRB, 0.34013E+00_JPRB, 0.32498E+00_JPRB, 0.31482E+00_JPRB, 0.30703E+00_JPRB ,&
&  0.30539E+00_JPRB, 0.31367E+00_JPRB, 0.33491E+00_JPRB, 0.36769E+00_JPRB, 0.39995E+00_JPRB, 0.37961E+00_JPRB, 0.36142E+00_JPRB ,&
&  0.33852E+00_JPRB, 0.32537E+00_JPRB, 0.32026E+00_JPRB, 0.31948E+00_JPRB, 0.32052E+00_JPRB, 0.32259E+00_JPRB, 0.32509E+00_JPRB ,&
&  0.32907E+00_JPRB, 0.33299E+00_JPRB, 0.33682E+00_JPRB, 0.33988E+00_JPRB, 0.34185E+00_JPRB, 0.34492E+00_JPRB, 0.35000E+00_JPRB ,&
&  0.35866E+00_JPRB, 0.37240E+00_JPRB, 0.39331E+00_JPRB, 0.42056E+00_JPRB, 0.44984E+00_JPRB, 0.47609E+00_JPRB, 0.52949E+00_JPRB ,&
&  0.61704E+00_JPRB, 0.72714E+00_JPRB, 0.78855E+00_JPRB, 0.84836E+00_JPRB, 0.87850E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.83196E+00_JPRB, 0.84664E+00_JPRB, 0.85842E+00_JPRB, 0.86187E+00_JPRB, 0.86144E+00_JPRB, 0.85646E+00_JPRB, 0.84418E+00_JPRB ,&
&  0.82437E+00_JPRB, 0.79795E+00_JPRB, 0.77205E+00_JPRB, 0.75146E+00_JPRB, 0.73831E+00_JPRB, 0.74016E+00_JPRB, 0.74774E+00_JPRB ,&
&  0.76381E+00_JPRB, 0.78816E+00_JPRB, 0.79235E+00_JPRB, 0.78453E+00_JPRB, 0.74813E+00_JPRB, 0.71013E+00_JPRB, 0.67578E+00_JPRB ,&
&  0.64471E+00_JPRB, 0.61447E+00_JPRB, 0.60329E+00_JPRB, 0.58376E+00_JPRB, 0.57004E+00_JPRB, 0.55285E+00_JPRB, 0.52743E+00_JPRB ,&
&  0.48608E+00_JPRB, 0.45637E+00_JPRB, 0.44279E+00_JPRB, 0.45072E+00_JPRB, 0.53296E+00_JPRB, 0.59438E+00_JPRB, 0.60787E+00_JPRB ,&
&  0.64129E+00_JPRB, 0.66430E+00_JPRB, 0.65858E+00_JPRB, 0.63781E+00_JPRB, 0.61555E+00_JPRB, 0.59185E+00_JPRB, 0.57135E+00_JPRB ,&
&  0.55100E+00_JPRB, 0.52449E+00_JPRB, 0.47373E+00_JPRB, 0.40173E+00_JPRB, 0.36687E+00_JPRB, 0.35374E+00_JPRB, 0.33937E+00_JPRB ,&
&  0.32867E+00_JPRB, 0.32127E+00_JPRB, 0.31805E+00_JPRB, 0.31348E+00_JPRB, 0.31305E+00_JPRB, 0.30937E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.29998E+00_JPRB, 0.29568E+00_JPRB, 0.29025E+00_JPRB, 0.28503E+00_JPRB, 0.28057E+00_JPRB, 0.27995E+00_JPRB, 0.28000E+00_JPRB ,&
&  0.28143E+00_JPRB, 0.28596E+00_JPRB, 0.29959E+00_JPRB, 0.30685E+00_JPRB, 0.30754E+00_JPRB, 0.30448E+00_JPRB, 0.29717E+00_JPRB ,&
&  0.28941E+00_JPRB, 0.28814E+00_JPRB, 0.28679E+00_JPRB, 0.28760E+00_JPRB, 0.29098E+00_JPRB, 0.29510E+00_JPRB, 0.29811E+00_JPRB ,&
&  0.30052E+00_JPRB, 0.30351E+00_JPRB, 0.30611E+00_JPRB, 0.30898E+00_JPRB, 0.31262E+00_JPRB, 0.31687E+00_JPRB, 0.32272E+00_JPRB ,&
&  0.32864E+00_JPRB, 0.33716E+00_JPRB, 0.34840E+00_JPRB, 0.36490E+00_JPRB, 0.38043E+00_JPRB, 0.39099E+00_JPRB, 0.40633E+00_JPRB ,&
&  0.42362E+00_JPRB, 0.47327E+00_JPRB, 0.60971E+00_JPRB, 0.82934E+00_JPRB, 0.86182E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.74998E+00_JPRB, 0.76369E+00_JPRB, 0.77878E+00_JPRB, 0.78313E+00_JPRB, 0.79362E+00_JPRB, 0.79227E+00_JPRB, 0.77439E+00_JPRB ,&
&  0.74628E+00_JPRB, 0.71902E+00_JPRB, 0.70431E+00_JPRB, 0.68336E+00_JPRB, 0.65441E+00_JPRB, 0.59536E+00_JPRB, 0.57301E+00_JPRB ,&
&  0.55383E+00_JPRB, 0.55294E+00_JPRB, 0.54287E+00_JPRB, 0.54240E+00_JPRB, 0.53067E+00_JPRB, 0.51888E+00_JPRB, 0.49920E+00_JPRB ,&
&  0.48140E+00_JPRB, 0.46502E+00_JPRB, 0.45332E+00_JPRB, 0.44859E+00_JPRB, 0.44603E+00_JPRB, 0.44018E+00_JPRB, 0.42602E+00_JPRB ,&
&  0.41637E+00_JPRB, 0.41432E+00_JPRB, 0.41626E+00_JPRB, 0.42192E+00_JPRB, 0.46168E+00_JPRB, 0.51929E+00_JPRB, 0.52126E+00_JPRB ,&
&  0.54239E+00_JPRB, 0.55449E+00_JPRB, 0.56655E+00_JPRB, 0.55448E+00_JPRB, 0.52059E+00_JPRB, 0.47859E+00_JPRB, 0.42611E+00_JPRB ,&
&  0.37635E+00_JPRB, 0.34660E+00_JPRB, 0.32231E+00_JPRB, 0.30455E+00_JPRB, 0.29173E+00_JPRB, 0.28688E+00_JPRB, 0.28053E+00_JPRB ,&
&  0.27658E+00_JPRB, 0.27349E+00_JPRB, 0.27231E+00_JPRB, 0.27200E+00_JPRB, 0.27118E+00_JPRB, 0.26885E+00_JPRB, 0.26824E+00_JPRB ,&
&  0.26901E+00_JPRB, 0.26905E+00_JPRB, 0.26859E+00_JPRB, 0.26960E+00_JPRB, 0.27065E+00_JPRB, 0.27237E+00_JPRB, 0.27414E+00_JPRB ,&
&  0.27688E+00_JPRB, 0.27839E+00_JPRB, 0.28062E+00_JPRB, 0.28318E+00_JPRB, 0.28206E+00_JPRB, 0.28107E+00_JPRB, 0.27836E+00_JPRB ,&
&  0.27504E+00_JPRB, 0.27523E+00_JPRB, 0.27493E+00_JPRB, 0.27582E+00_JPRB, 0.27798E+00_JPRB, 0.28066E+00_JPRB, 0.28405E+00_JPRB ,&
&  0.28721E+00_JPRB, 0.29049E+00_JPRB, 0.29298E+00_JPRB, 0.29552E+00_JPRB, 0.29984E+00_JPRB, 0.30426E+00_JPRB, 0.30993E+00_JPRB ,&
&  0.31577E+00_JPRB, 0.32251E+00_JPRB, 0.33157E+00_JPRB, 0.34337E+00_JPRB, 0.36113E+00_JPRB, 0.38021E+00_JPRB, 0.41012E+00_JPRB ,&
&  0.43402E+00_JPRB, 0.52917E+00_JPRB, 0.76222E+00_JPRB, 0.69712E+00_JPRB, 0.72040E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.71171E+00_JPRB, 0.72993E+00_JPRB, 0.75408E+00_JPRB, 0.74736E+00_JPRB, 0.76693E+00_JPRB, 0.75356E+00_JPRB, 0.72964E+00_JPRB ,&
&  0.68226E+00_JPRB, 0.62712E+00_JPRB, 0.57994E+00_JPRB, 0.52799E+00_JPRB, 0.48636E+00_JPRB, 0.45688E+00_JPRB, 0.44187E+00_JPRB ,&
&  0.43337E+00_JPRB, 0.43044E+00_JPRB, 0.42739E+00_JPRB, 0.42577E+00_JPRB, 0.41935E+00_JPRB, 0.41505E+00_JPRB, 0.40846E+00_JPRB ,&
&  0.40259E+00_JPRB, 0.39969E+00_JPRB, 0.39662E+00_JPRB, 0.40162E+00_JPRB, 0.40733E+00_JPRB, 0.41289E+00_JPRB, 0.41505E+00_JPRB ,&
&  0.41639E+00_JPRB, 0.41794E+00_JPRB, 0.42119E+00_JPRB, 0.41875E+00_JPRB, 0.43038E+00_JPRB, 0.45092E+00_JPRB, 0.46709E+00_JPRB ,&
&  0.48285E+00_JPRB, 0.49561E+00_JPRB, 0.49057E+00_JPRB, 0.46568E+00_JPRB, 0.39891E+00_JPRB, 0.36491E+00_JPRB, 0.33373E+00_JPRB ,&
&  0.31348E+00_JPRB, 0.29832E+00_JPRB, 0.28775E+00_JPRB, 0.28186E+00_JPRB, 0.27658E+00_JPRB, 0.27409E+00_JPRB, 0.27141E+00_JPRB ,&
&  0.26896E+00_JPRB, 0.26776E+00_JPRB, 0.26607E+00_JPRB, 0.26517E+00_JPRB, 0.26398E+00_JPRB, 0.26236E+00_JPRB, 0.26143E+00_JPRB ,&
&  0.26194E+00_JPRB, 0.26194E+00_JPRB, 0.26344E+00_JPRB, 0.26459E+00_JPRB, 0.26671E+00_JPRB, 0.26839E+00_JPRB, 0.27092E+00_JPRB ,&
&  0.27331E+00_JPRB, 0.27538E+00_JPRB, 0.27533E+00_JPRB, 0.27511E+00_JPRB, 0.27428E+00_JPRB, 0.27397E+00_JPRB, 0.27331E+00_JPRB ,&
&  0.27218E+00_JPRB, 0.27266E+00_JPRB, 0.27339E+00_JPRB, 0.27408E+00_JPRB, 0.27574E+00_JPRB, 0.27782E+00_JPRB, 0.28069E+00_JPRB ,&
&  0.28449E+00_JPRB, 0.28836E+00_JPRB, 0.29172E+00_JPRB, 0.29544E+00_JPRB, 0.30004E+00_JPRB, 0.30507E+00_JPRB, 0.31117E+00_JPRB ,&
&  0.31591E+00_JPRB, 0.32504E+00_JPRB, 0.33814E+00_JPRB, 0.35215E+00_JPRB, 0.37625E+00_JPRB, 0.39595E+00_JPRB, 0.42904E+00_JPRB ,&
&  0.47456E+00_JPRB, 0.63618E+00_JPRB, 0.91676E+00_JPRB, 0.69960E+00_JPRB, 0.69428E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.62845E+00_JPRB, 0.63849E+00_JPRB, 0.65255E+00_JPRB, 0.66409E+00_JPRB, 0.68736E+00_JPRB, 0.70046E+00_JPRB, 0.74405E+00_JPRB ,&
&  0.69868E+00_JPRB, 0.62100E+00_JPRB, 0.58876E+00_JPRB, 0.55293E+00_JPRB, 0.51288E+00_JPRB, 0.48370E+00_JPRB, 0.46325E+00_JPRB ,&
&  0.45052E+00_JPRB, 0.43954E+00_JPRB, 0.42868E+00_JPRB, 0.41553E+00_JPRB, 0.40225E+00_JPRB, 0.38808E+00_JPRB, 0.37803E+00_JPRB ,&
&  0.37233E+00_JPRB, 0.37157E+00_JPRB, 0.37101E+00_JPRB, 0.37460E+00_JPRB, 0.37932E+00_JPRB, 0.38533E+00_JPRB, 0.39286E+00_JPRB ,&
&  0.41178E+00_JPRB, 0.42077E+00_JPRB, 0.42413E+00_JPRB, 0.42343E+00_JPRB, 0.42389E+00_JPRB, 0.42700E+00_JPRB, 0.41769E+00_JPRB ,&
&  0.42137E+00_JPRB, 0.43821E+00_JPRB, 0.44324E+00_JPRB, 0.42157E+00_JPRB, 0.36688E+00_JPRB, 0.33520E+00_JPRB, 0.31216E+00_JPRB ,&
&  0.30456E+00_JPRB, 0.29372E+00_JPRB, 0.28695E+00_JPRB, 0.28017E+00_JPRB, 0.27512E+00_JPRB, 0.27317E+00_JPRB, 0.27107E+00_JPRB ,&
&  0.26984E+00_JPRB, 0.26939E+00_JPRB, 0.26737E+00_JPRB, 0.26663E+00_JPRB, 0.26522E+00_JPRB, 0.26438E+00_JPRB, 0.26402E+00_JPRB ,&
&  0.26383E+00_JPRB, 0.26354E+00_JPRB, 0.26386E+00_JPRB, 0.26446E+00_JPRB, 0.26545E+00_JPRB, 0.26623E+00_JPRB, 0.26697E+00_JPRB ,&
&  0.26814E+00_JPRB, 0.26926E+00_JPRB, 0.26967E+00_JPRB, 0.27050E+00_JPRB, 0.27118E+00_JPRB, 0.27277E+00_JPRB, 0.27359E+00_JPRB ,&
&  0.27440E+00_JPRB, 0.27518E+00_JPRB, 0.27535E+00_JPRB, 0.27633E+00_JPRB, 0.27756E+00_JPRB, 0.27987E+00_JPRB, 0.28238E+00_JPRB ,&
&  0.28553E+00_JPRB, 0.28935E+00_JPRB, 0.29342E+00_JPRB, 0.29756E+00_JPRB, 0.30276E+00_JPRB, 0.30796E+00_JPRB, 0.31508E+00_JPRB ,&
&  0.32268E+00_JPRB, 0.33327E+00_JPRB, 0.34833E+00_JPRB, 0.36595E+00_JPRB, 0.38796E+00_JPRB, 0.40664E+00_JPRB, 0.42527E+00_JPRB ,&
&  0.45756E+00_JPRB, 0.55180E+00_JPRB, 0.61188E+00_JPRB, 0.60653E+00_JPRB, 0.62039E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.60398E+00_JPRB, 0.60738E+00_JPRB, 0.61092E+00_JPRB, 0.61586E+00_JPRB, 0.62128E+00_JPRB, 0.62613E+00_JPRB, 0.62987E+00_JPRB ,&
&  0.61106E+00_JPRB, 0.58793E+00_JPRB, 0.56864E+00_JPRB, 0.54558E+00_JPRB, 0.52446E+00_JPRB, 0.50587E+00_JPRB, 0.48546E+00_JPRB ,&
&  0.46486E+00_JPRB, 0.44443E+00_JPRB, 0.42160E+00_JPRB, 0.40276E+00_JPRB, 0.38433E+00_JPRB, 0.36942E+00_JPRB, 0.36108E+00_JPRB ,&
&  0.35606E+00_JPRB, 0.35447E+00_JPRB, 0.35549E+00_JPRB, 0.35707E+00_JPRB, 0.35892E+00_JPRB, 0.36304E+00_JPRB, 0.36917E+00_JPRB ,&
&  0.37773E+00_JPRB, 0.39379E+00_JPRB, 0.40794E+00_JPRB, 0.41263E+00_JPRB, 0.40813E+00_JPRB, 0.39880E+00_JPRB, 0.39059E+00_JPRB ,&
&  0.38096E+00_JPRB, 0.36902E+00_JPRB, 0.35370E+00_JPRB, 0.33842E+00_JPRB, 0.32418E+00_JPRB, 0.31386E+00_JPRB, 0.30480E+00_JPRB ,&
&  0.29693E+00_JPRB, 0.28986E+00_JPRB, 0.28425E+00_JPRB, 0.28053E+00_JPRB, 0.27734E+00_JPRB, 0.27489E+00_JPRB, 0.27241E+00_JPRB ,&
&  0.27138E+00_JPRB, 0.27032E+00_JPRB, 0.26969E+00_JPRB, 0.26936E+00_JPRB, 0.26875E+00_JPRB, 0.26852E+00_JPRB, 0.26836E+00_JPRB ,&
&  0.26796E+00_JPRB, 0.26727E+00_JPRB, 0.26691E+00_JPRB, 0.26669E+00_JPRB, 0.26727E+00_JPRB, 0.26779E+00_JPRB, 0.26858E+00_JPRB ,&
&  0.26973E+00_JPRB, 0.27192E+00_JPRB, 0.27322E+00_JPRB, 0.27446E+00_JPRB, 0.27511E+00_JPRB, 0.27577E+00_JPRB, 0.27661E+00_JPRB ,&
&  0.27794E+00_JPRB, 0.27948E+00_JPRB, 0.28079E+00_JPRB, 0.28256E+00_JPRB, 0.28436E+00_JPRB, 0.28684E+00_JPRB, 0.28958E+00_JPRB ,&
&  0.29252E+00_JPRB, 0.29614E+00_JPRB, 0.30055E+00_JPRB, 0.30443E+00_JPRB, 0.31080E+00_JPRB, 0.31726E+00_JPRB, 0.32463E+00_JPRB ,&
&  0.33245E+00_JPRB, 0.34096E+00_JPRB, 0.35244E+00_JPRB, 0.36624E+00_JPRB, 0.38390E+00_JPRB, 0.39117E+00_JPRB, 0.40069E+00_JPRB ,&
&  0.41564E+00_JPRB, 0.44676E+00_JPRB, 0.58436E+00_JPRB, 0.59806E+00_JPRB, 0.60361E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.38245E+00_JPRB, 0.38823E+00_JPRB, 0.39194E+00_JPRB, 0.40069E+00_JPRB, 0.41011E+00_JPRB, 0.51024E+00_JPRB, 0.50532E+00_JPRB ,&
&  0.50487E+00_JPRB, 0.46143E+00_JPRB, 0.45398E+00_JPRB, 0.44203E+00_JPRB, 0.42676E+00_JPRB, 0.40794E+00_JPRB, 0.39637E+00_JPRB ,&
&  0.38678E+00_JPRB, 0.37538E+00_JPRB, 0.36492E+00_JPRB, 0.35863E+00_JPRB, 0.35364E+00_JPRB, 0.34903E+00_JPRB, 0.34606E+00_JPRB ,&
&  0.34439E+00_JPRB, 0.34413E+00_JPRB, 0.34421E+00_JPRB, 0.34393E+00_JPRB, 0.34681E+00_JPRB, 0.35017E+00_JPRB, 0.35365E+00_JPRB ,&
&  0.35481E+00_JPRB, 0.35956E+00_JPRB, 0.36085E+00_JPRB, 0.35790E+00_JPRB, 0.35330E+00_JPRB, 0.34580E+00_JPRB, 0.33825E+00_JPRB ,&
&  0.33048E+00_JPRB, 0.32346E+00_JPRB, 0.31757E+00_JPRB, 0.31229E+00_JPRB, 0.30615E+00_JPRB, 0.30005E+00_JPRB, 0.29597E+00_JPRB ,&
&  0.29215E+00_JPRB, 0.28819E+00_JPRB, 0.28495E+00_JPRB, 0.28337E+00_JPRB, 0.28161E+00_JPRB, 0.28040E+00_JPRB, 0.27943E+00_JPRB ,&
&  0.27917E+00_JPRB, 0.27900E+00_JPRB, 0.27904E+00_JPRB, 0.27930E+00_JPRB, 0.27925E+00_JPRB, 0.27915E+00_JPRB, 0.27909E+00_JPRB ,&
&  0.27973E+00_JPRB, 0.27980E+00_JPRB, 0.27978E+00_JPRB, 0.27963E+00_JPRB, 0.28052E+00_JPRB, 0.28094E+00_JPRB, 0.28152E+00_JPRB ,&
&  0.28229E+00_JPRB, 0.28326E+00_JPRB, 0.28397E+00_JPRB, 0.28473E+00_JPRB, 0.28547E+00_JPRB, 0.28641E+00_JPRB, 0.28751E+00_JPRB ,&
&  0.28866E+00_JPRB, 0.29033E+00_JPRB, 0.29206E+00_JPRB, 0.29355E+00_JPRB, 0.29491E+00_JPRB, 0.29674E+00_JPRB, 0.29881E+00_JPRB ,&
&  0.30179E+00_JPRB, 0.30464E+00_JPRB, 0.30755E+00_JPRB, 0.30904E+00_JPRB, 0.31174E+00_JPRB, 0.31405E+00_JPRB, 0.31664E+00_JPRB ,&
&  0.31980E+00_JPRB, 0.32512E+00_JPRB, 0.33319E+00_JPRB, 0.34225E+00_JPRB, 0.35272E+00_JPRB, 0.36012E+00_JPRB, 0.36479E+00_JPRB ,&
&  0.36857E+00_JPRB, 0.37281E+00_JPRB, 0.37737E+00_JPRB, 0.37975E+00_JPRB, 0.38151E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.34832E+00_JPRB, 0.34821E+00_JPRB, 0.34827E+00_JPRB, 0.34787E+00_JPRB, 0.34607E+00_JPRB, 0.34456E+00_JPRB, 0.34315E+00_JPRB ,&
&  0.34222E+00_JPRB, 0.34109E+00_JPRB, 0.34053E+00_JPRB, 0.34018E+00_JPRB, 0.33960E+00_JPRB, 0.33883E+00_JPRB, 0.33783E+00_JPRB ,&
&  0.33651E+00_JPRB, 0.33482E+00_JPRB, 0.33284E+00_JPRB, 0.33218E+00_JPRB, 0.33187E+00_JPRB, 0.33157E+00_JPRB, 0.33118E+00_JPRB ,&
&  0.33065E+00_JPRB, 0.33020E+00_JPRB, 0.32940E+00_JPRB, 0.32951E+00_JPRB, 0.32943E+00_JPRB, 0.32935E+00_JPRB, 0.32939E+00_JPRB ,&
&  0.32950E+00_JPRB, 0.32977E+00_JPRB, 0.32965E+00_JPRB, 0.32922E+00_JPRB, 0.32841E+00_JPRB, 0.32672E+00_JPRB, 0.32399E+00_JPRB ,&
&  0.32089E+00_JPRB, 0.31779E+00_JPRB, 0.31531E+00_JPRB, 0.31303E+00_JPRB, 0.31095E+00_JPRB, 0.30907E+00_JPRB, 0.30752E+00_JPRB ,&
&  0.30562E+00_JPRB, 0.30374E+00_JPRB, 0.30200E+00_JPRB, 0.30070E+00_JPRB, 0.29924E+00_JPRB, 0.29795E+00_JPRB, 0.29669E+00_JPRB ,&
&  0.29580E+00_JPRB, 0.29503E+00_JPRB, 0.29437E+00_JPRB, 0.29402E+00_JPRB, 0.29376E+00_JPRB, 0.29353E+00_JPRB, 0.29326E+00_JPRB ,&
&  0.29303E+00_JPRB, 0.29291E+00_JPRB, 0.29284E+00_JPRB, 0.29288E+00_JPRB, 0.29306E+00_JPRB, 0.29303E+00_JPRB, 0.29297E+00_JPRB ,&
&  0.29300E+00_JPRB, 0.29340E+00_JPRB, 0.29364E+00_JPRB, 0.29398E+00_JPRB, 0.29447E+00_JPRB, 0.29516E+00_JPRB, 0.29591E+00_JPRB ,&
&  0.29666E+00_JPRB, 0.29753E+00_JPRB, 0.29845E+00_JPRB, 0.29962E+00_JPRB, 0.30090E+00_JPRB, 0.30216E+00_JPRB, 0.30338E+00_JPRB ,&
&  0.30468E+00_JPRB, 0.30589E+00_JPRB, 0.30704E+00_JPRB, 0.30782E+00_JPRB, 0.30903E+00_JPRB, 0.31030E+00_JPRB, 0.31179E+00_JPRB ,&
&  0.31351E+00_JPRB, 0.31584E+00_JPRB, 0.31920E+00_JPRB, 0.32322E+00_JPRB, 0.32849E+00_JPRB, 0.33221E+00_JPRB, 0.33582E+00_JPRB ,&
&  0.33831E+00_JPRB, 0.34032E+00_JPRB, 0.34189E+00_JPRB, 0.34382E+00_JPRB, 0.34595E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB, 0.31199E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2050:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2050
