!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_A1B2060 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS2060* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB ,&
&  0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB, 0.40505E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.42020E+00_JPRB, 0.41948E+00_JPRB, 0.41881E+00_JPRB, 0.41821E+00_JPRB, 0.41766E+00_JPRB, 0.41726E+00_JPRB, 0.41692E+00_JPRB ,&
&  0.41645E+00_JPRB, 0.41604E+00_JPRB, 0.41554E+00_JPRB, 0.41504E+00_JPRB, 0.41456E+00_JPRB, 0.41403E+00_JPRB, 0.41339E+00_JPRB ,&
&  0.41261E+00_JPRB, 0.41190E+00_JPRB, 0.41111E+00_JPRB, 0.41012E+00_JPRB, 0.40911E+00_JPRB, 0.40804E+00_JPRB, 0.40676E+00_JPRB ,&
&  0.40528E+00_JPRB, 0.40384E+00_JPRB, 0.40227E+00_JPRB, 0.40066E+00_JPRB, 0.39893E+00_JPRB, 0.39704E+00_JPRB, 0.39534E+00_JPRB ,&
&  0.39410E+00_JPRB, 0.39278E+00_JPRB, 0.39154E+00_JPRB, 0.39031E+00_JPRB, 0.38936E+00_JPRB, 0.38870E+00_JPRB, 0.38800E+00_JPRB ,&
&  0.38740E+00_JPRB, 0.38695E+00_JPRB, 0.38708E+00_JPRB, 0.38740E+00_JPRB, 0.38787E+00_JPRB, 0.38885E+00_JPRB, 0.39036E+00_JPRB ,&
&  0.39164E+00_JPRB, 0.39277E+00_JPRB, 0.39411E+00_JPRB, 0.39560E+00_JPRB, 0.39716E+00_JPRB, 0.39874E+00_JPRB, 0.40018E+00_JPRB ,&
&  0.40127E+00_JPRB, 0.40218E+00_JPRB, 0.40315E+00_JPRB, 0.40396E+00_JPRB, 0.40440E+00_JPRB, 0.40483E+00_JPRB, 0.40530E+00_JPRB ,&
&  0.40564E+00_JPRB, 0.40584E+00_JPRB, 0.40615E+00_JPRB, 0.40668E+00_JPRB, 0.40715E+00_JPRB, 0.40766E+00_JPRB, 0.40841E+00_JPRB ,&
&  0.40942E+00_JPRB, 0.41018E+00_JPRB, 0.41109E+00_JPRB, 0.41228E+00_JPRB, 0.41383E+00_JPRB, 0.41568E+00_JPRB, 0.41715E+00_JPRB ,&
&  0.41870E+00_JPRB, 0.42033E+00_JPRB, 0.42205E+00_JPRB, 0.42326E+00_JPRB, 0.42457E+00_JPRB, 0.42578E+00_JPRB, 0.42694E+00_JPRB ,&
&  0.42726E+00_JPRB, 0.42778E+00_JPRB, 0.42845E+00_JPRB, 0.42906E+00_JPRB, 0.42859E+00_JPRB, 0.42831E+00_JPRB, 0.42810E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.42752E+00_JPRB, 0.42704E+00_JPRB, 0.42638E+00_JPRB, 0.42603E+00_JPRB, 0.42506E+00_JPRB, 0.42435E+00_JPRB ,&
&  0.42361E+00_JPRB, 0.42303E+00_JPRB, 0.42231E+00_JPRB, 0.42157E+00_JPRB, 0.42082E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.43518E+00_JPRB, 0.43391E+00_JPRB, 0.43255E+00_JPRB, 0.43114E+00_JPRB, 0.42967E+00_JPRB, 0.42854E+00_JPRB, 0.42740E+00_JPRB ,&
&  0.42621E+00_JPRB, 0.42525E+00_JPRB, 0.42414E+00_JPRB, 0.42321E+00_JPRB, 0.42240E+00_JPRB, 0.42206E+00_JPRB, 0.42135E+00_JPRB ,&
&  0.42052E+00_JPRB, 0.41930E+00_JPRB, 0.41825E+00_JPRB, 0.41643E+00_JPRB, 0.41440E+00_JPRB, 0.41167E+00_JPRB, 0.40911E+00_JPRB ,&
&  0.40608E+00_JPRB, 0.40255E+00_JPRB, 0.39945E+00_JPRB, 0.39647E+00_JPRB, 0.39345E+00_JPRB, 0.39030E+00_JPRB, 0.38805E+00_JPRB ,&
&  0.38662E+00_JPRB, 0.38467E+00_JPRB, 0.38300E+00_JPRB, 0.38156E+00_JPRB, 0.37988E+00_JPRB, 0.37879E+00_JPRB, 0.37781E+00_JPRB ,&
&  0.37594E+00_JPRB, 0.37409E+00_JPRB, 0.37433E+00_JPRB, 0.37489E+00_JPRB, 0.37612E+00_JPRB, 0.37846E+00_JPRB, 0.38277E+00_JPRB ,&
&  0.38838E+00_JPRB, 0.39374E+00_JPRB, 0.39947E+00_JPRB, 0.40463E+00_JPRB, 0.40881E+00_JPRB, 0.41213E+00_JPRB, 0.41486E+00_JPRB ,&
&  0.41662E+00_JPRB, 0.41793E+00_JPRB, 0.41866E+00_JPRB, 0.41870E+00_JPRB, 0.41803E+00_JPRB, 0.41779E+00_JPRB, 0.41787E+00_JPRB ,&
&  0.41813E+00_JPRB, 0.41677E+00_JPRB, 0.41639E+00_JPRB, 0.41679E+00_JPRB, 0.41782E+00_JPRB, 0.41881E+00_JPRB, 0.42089E+00_JPRB ,&
&  0.42383E+00_JPRB, 0.42748E+00_JPRB, 0.43079E+00_JPRB, 0.43521E+00_JPRB, 0.44038E+00_JPRB, 0.44636E+00_JPRB, 0.44993E+00_JPRB ,&
&  0.45433E+00_JPRB, 0.45899E+00_JPRB, 0.46393E+00_JPRB, 0.46512E+00_JPRB, 0.46648E+00_JPRB, 0.46753E+00_JPRB, 0.46885E+00_JPRB ,&
&  0.46697E+00_JPRB, 0.46634E+00_JPRB, 0.46600E+00_JPRB, 0.46683E+00_JPRB, 0.46453E+00_JPRB, 0.46345E+00_JPRB, 0.46232E+00_JPRB ,&
&  0.46289E+00_JPRB, 0.45919E+00_JPRB, 0.45705E+00_JPRB, 0.45434E+00_JPRB, 0.45271E+00_JPRB, 0.44895E+00_JPRB, 0.44598E+00_JPRB ,&
&  0.44343E+00_JPRB, 0.44182E+00_JPRB, 0.43982E+00_JPRB, 0.43809E+00_JPRB, 0.43667E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.45066E+00_JPRB, 0.44770E+00_JPRB, 0.44340E+00_JPRB, 0.43802E+00_JPRB, 0.43307E+00_JPRB, 0.42939E+00_JPRB, 0.42550E+00_JPRB ,&
&  0.42239E+00_JPRB, 0.41966E+00_JPRB, 0.41778E+00_JPRB, 0.41637E+00_JPRB, 0.41497E+00_JPRB, 0.41545E+00_JPRB, 0.41604E+00_JPRB ,&
&  0.41660E+00_JPRB, 0.41634E+00_JPRB, 0.41753E+00_JPRB, 0.41692E+00_JPRB, 0.41561E+00_JPRB, 0.41263E+00_JPRB, 0.40849E+00_JPRB ,&
&  0.40370E+00_JPRB, 0.39824E+00_JPRB, 0.39192E+00_JPRB, 0.38471E+00_JPRB, 0.37785E+00_JPRB, 0.35463E+00_JPRB, 0.34884E+00_JPRB ,&
&  0.34988E+00_JPRB, 0.34612E+00_JPRB, 0.34323E+00_JPRB, 0.34175E+00_JPRB, 0.33982E+00_JPRB, 0.33731E+00_JPRB, 0.33464E+00_JPRB ,&
&  0.31707E+00_JPRB, 0.31578E+00_JPRB, 0.31365E+00_JPRB, 0.31251E+00_JPRB, 0.31550E+00_JPRB, 0.32597E+00_JPRB, 0.34214E+00_JPRB ,&
&  0.37071E+00_JPRB, 0.38312E+00_JPRB, 0.39352E+00_JPRB, 0.40598E+00_JPRB, 0.41566E+00_JPRB, 0.42287E+00_JPRB, 0.42993E+00_JPRB ,&
&  0.43587E+00_JPRB, 0.44164E+00_JPRB, 0.44591E+00_JPRB, 0.44822E+00_JPRB, 0.44853E+00_JPRB, 0.44851E+00_JPRB, 0.44827E+00_JPRB ,&
&  0.44792E+00_JPRB, 0.44751E+00_JPRB, 0.44770E+00_JPRB, 0.44769E+00_JPRB, 0.44727E+00_JPRB, 0.44787E+00_JPRB, 0.45128E+00_JPRB ,&
&  0.45641E+00_JPRB, 0.46422E+00_JPRB, 0.47097E+00_JPRB, 0.48041E+00_JPRB, 0.48901E+00_JPRB, 0.49993E+00_JPRB, 0.50415E+00_JPRB ,&
&  0.50939E+00_JPRB, 0.51270E+00_JPRB, 0.51721E+00_JPRB, 0.51712E+00_JPRB, 0.51693E+00_JPRB, 0.51588E+00_JPRB, 0.51594E+00_JPRB ,&
&  0.51294E+00_JPRB, 0.51083E+00_JPRB, 0.50807E+00_JPRB, 0.50764E+00_JPRB, 0.50416E+00_JPRB, 0.50328E+00_JPRB, 0.50104E+00_JPRB ,&
&  0.50202E+00_JPRB, 0.49771E+00_JPRB, 0.49589E+00_JPRB, 0.49069E+00_JPRB, 0.48648E+00_JPRB, 0.47892E+00_JPRB, 0.47236E+00_JPRB ,&
&  0.46674E+00_JPRB, 0.46247E+00_JPRB, 0.45889E+00_JPRB, 0.45585E+00_JPRB, 0.45307E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.46122E+00_JPRB, 0.45324E+00_JPRB, 0.44448E+00_JPRB, 0.43716E+00_JPRB, 0.43044E+00_JPRB, 0.42554E+00_JPRB, 0.42089E+00_JPRB ,&
&  0.41740E+00_JPRB, 0.41405E+00_JPRB, 0.41161E+00_JPRB, 0.40938E+00_JPRB, 0.40775E+00_JPRB, 0.40684E+00_JPRB, 0.40604E+00_JPRB ,&
&  0.40597E+00_JPRB, 0.40565E+00_JPRB, 0.40607E+00_JPRB, 0.40586E+00_JPRB, 0.40608E+00_JPRB, 0.40404E+00_JPRB, 0.39988E+00_JPRB ,&
&  0.39487E+00_JPRB, 0.38929E+00_JPRB, 0.37026E+00_JPRB, 0.36323E+00_JPRB, 0.34745E+00_JPRB, 0.33583E+00_JPRB, 0.32898E+00_JPRB ,&
&  0.31715E+00_JPRB, 0.31072E+00_JPRB, 0.29886E+00_JPRB, 0.29199E+00_JPRB, 0.29021E+00_JPRB, 0.28234E+00_JPRB, 0.27910E+00_JPRB ,&
&  0.27365E+00_JPRB, 0.26395E+00_JPRB, 0.26026E+00_JPRB, 0.25881E+00_JPRB, 0.26213E+00_JPRB, 0.27094E+00_JPRB, 0.28694E+00_JPRB ,&
&  0.31726E+00_JPRB, 0.35115E+00_JPRB, 0.39774E+00_JPRB, 0.41124E+00_JPRB, 0.41330E+00_JPRB, 0.42398E+00_JPRB, 0.43258E+00_JPRB ,&
&  0.43812E+00_JPRB, 0.45021E+00_JPRB, 0.46323E+00_JPRB, 0.46864E+00_JPRB, 0.47244E+00_JPRB, 0.47393E+00_JPRB, 0.47509E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.48111E+00_JPRB, 0.48428E+00_JPRB, 0.48765E+00_JPRB, 0.49118E+00_JPRB, 0.49486E+00_JPRB, 0.50246E+00_JPRB ,&
&  0.50901E+00_JPRB, 0.52293E+00_JPRB, 0.53418E+00_JPRB, 0.55157E+00_JPRB, 0.55859E+00_JPRB, 0.56710E+00_JPRB, 0.56905E+00_JPRB ,&
&  0.57285E+00_JPRB, 0.57485E+00_JPRB, 0.57634E+00_JPRB, 0.57111E+00_JPRB, 0.56977E+00_JPRB, 0.57294E+00_JPRB, 0.57132E+00_JPRB ,&
&  0.56882E+00_JPRB, 0.56698E+00_JPRB, 0.56452E+00_JPRB, 0.56251E+00_JPRB, 0.56098E+00_JPRB, 0.56126E+00_JPRB, 0.56002E+00_JPRB ,&
&  0.56186E+00_JPRB, 0.55881E+00_JPRB, 0.55563E+00_JPRB, 0.54930E+00_JPRB, 0.54120E+00_JPRB, 0.52987E+00_JPRB, 0.51568E+00_JPRB ,&
&  0.50347E+00_JPRB, 0.49401E+00_JPRB, 0.48416E+00_JPRB, 0.47655E+00_JPRB, 0.46868E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.44160E+00_JPRB, 0.43160E+00_JPRB, 0.42335E+00_JPRB, 0.41754E+00_JPRB, 0.41255E+00_JPRB, 0.40940E+00_JPRB, 0.40630E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.40273E+00_JPRB, 0.40190E+00_JPRB, 0.40080E+00_JPRB, 0.40062E+00_JPRB, 0.40079E+00_JPRB, 0.40160E+00_JPRB ,&
&  0.40271E+00_JPRB, 0.40322E+00_JPRB, 0.40395E+00_JPRB, 0.40233E+00_JPRB, 0.40080E+00_JPRB, 0.39815E+00_JPRB, 0.39571E+00_JPRB ,&
&  0.38995E+00_JPRB, 0.38313E+00_JPRB, 0.36928E+00_JPRB, 0.36136E+00_JPRB, 0.34704E+00_JPRB, 0.33650E+00_JPRB, 0.32618E+00_JPRB ,&
&  0.31751E+00_JPRB, 0.29967E+00_JPRB, 0.30440E+00_JPRB, 0.29324E+00_JPRB, 0.28254E+00_JPRB, 0.27151E+00_JPRB, 0.25851E+00_JPRB ,&
&  0.25347E+00_JPRB, 0.24074E+00_JPRB, 0.23615E+00_JPRB, 0.23631E+00_JPRB, 0.23838E+00_JPRB, 0.24767E+00_JPRB, 0.26221E+00_JPRB ,&
&  0.29165E+00_JPRB, 0.36609E+00_JPRB, 0.39223E+00_JPRB, 0.40396E+00_JPRB, 0.40106E+00_JPRB, 0.40239E+00_JPRB, 0.39912E+00_JPRB ,&
&  0.41207E+00_JPRB, 0.42931E+00_JPRB, 0.44747E+00_JPRB, 0.45935E+00_JPRB, 0.47401E+00_JPRB, 0.48226E+00_JPRB, 0.48945E+00_JPRB ,&
&  0.49784E+00_JPRB, 0.51113E+00_JPRB, 0.52337E+00_JPRB, 0.53625E+00_JPRB, 0.55250E+00_JPRB, 0.56867E+00_JPRB, 0.58713E+00_JPRB ,&
&  0.59784E+00_JPRB, 0.61959E+00_JPRB, 0.62674E+00_JPRB, 0.63642E+00_JPRB, 0.63714E+00_JPRB, 0.63518E+00_JPRB, 0.63540E+00_JPRB ,&
&  0.63805E+00_JPRB, 0.64112E+00_JPRB, 0.64326E+00_JPRB, 0.64131E+00_JPRB, 0.64034E+00_JPRB, 0.63112E+00_JPRB, 0.62722E+00_JPRB ,&
&  0.62632E+00_JPRB, 0.62413E+00_JPRB, 0.62108E+00_JPRB, 0.61591E+00_JPRB, 0.60906E+00_JPRB, 0.60133E+00_JPRB, 0.59658E+00_JPRB ,&
&  0.59053E+00_JPRB, 0.58286E+00_JPRB, 0.57417E+00_JPRB, 0.56541E+00_JPRB, 0.55610E+00_JPRB, 0.54638E+00_JPRB, 0.53038E+00_JPRB ,&
&  0.51051E+00_JPRB, 0.49348E+00_JPRB, 0.47761E+00_JPRB, 0.46534E+00_JPRB, 0.45278E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.42618E+00_JPRB, 0.41690E+00_JPRB, 0.40928E+00_JPRB, 0.40517E+00_JPRB, 0.40101E+00_JPRB, 0.39931E+00_JPRB, 0.39666E+00_JPRB ,&
&  0.39531E+00_JPRB, 0.39391E+00_JPRB, 0.39339E+00_JPRB, 0.39285E+00_JPRB, 0.39301E+00_JPRB, 0.39333E+00_JPRB, 0.39446E+00_JPRB ,&
&  0.39574E+00_JPRB, 0.39786E+00_JPRB, 0.40004E+00_JPRB, 0.40001E+00_JPRB, 0.39971E+00_JPRB, 0.39816E+00_JPRB, 0.39603E+00_JPRB ,&
&  0.39107E+00_JPRB, 0.38469E+00_JPRB, 0.37125E+00_JPRB, 0.36136E+00_JPRB, 0.35263E+00_JPRB, 0.34044E+00_JPRB, 0.33730E+00_JPRB ,&
&  0.33334E+00_JPRB, 0.32396E+00_JPRB, 0.31065E+00_JPRB, 0.30491E+00_JPRB, 0.29663E+00_JPRB, 0.26877E+00_JPRB, 0.26927E+00_JPRB ,&
&  0.26141E+00_JPRB, 0.24279E+00_JPRB, 0.23735E+00_JPRB, 0.23352E+00_JPRB, 0.23498E+00_JPRB, 0.24313E+00_JPRB, 0.25598E+00_JPRB ,&
&  0.28407E+00_JPRB, 0.37034E+00_JPRB, 0.39166E+00_JPRB, 0.39949E+00_JPRB, 0.39134E+00_JPRB, 0.39486E+00_JPRB, 0.39541E+00_JPRB ,&
&  0.40489E+00_JPRB, 0.41380E+00_JPRB, 0.42513E+00_JPRB, 0.43451E+00_JPRB, 0.44442E+00_JPRB, 0.44915E+00_JPRB, 0.45961E+00_JPRB ,&
&  0.47920E+00_JPRB, 0.50345E+00_JPRB, 0.52571E+00_JPRB, 0.55458E+00_JPRB, 0.59135E+00_JPRB, 0.65570E+00_JPRB, 0.72170E+00_JPRB ,&
&  0.72699E+00_JPRB, 0.71863E+00_JPRB, 0.71468E+00_JPRB, 0.70751E+00_JPRB, 0.70573E+00_JPRB, 0.70460E+00_JPRB, 0.70832E+00_JPRB ,&
&  0.71748E+00_JPRB, 0.72587E+00_JPRB, 0.71189E+00_JPRB, 0.68545E+00_JPRB, 0.67401E+00_JPRB, 0.65547E+00_JPRB, 0.65197E+00_JPRB ,&
&  0.64721E+00_JPRB, 0.65622E+00_JPRB, 0.65050E+00_JPRB, 0.64016E+00_JPRB, 0.63003E+00_JPRB, 0.61857E+00_JPRB, 0.61071E+00_JPRB ,&
&  0.60157E+00_JPRB, 0.59001E+00_JPRB, 0.57777E+00_JPRB, 0.56765E+00_JPRB, 0.55872E+00_JPRB, 0.55146E+00_JPRB, 0.54051E+00_JPRB ,&
&  0.51913E+00_JPRB, 0.49248E+00_JPRB, 0.47210E+00_JPRB, 0.45308E+00_JPRB, 0.43776E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.41846E+00_JPRB, 0.41254E+00_JPRB, 0.40887E+00_JPRB, 0.40831E+00_JPRB, 0.40711E+00_JPRB, 0.40684E+00_JPRB, 0.40401E+00_JPRB ,&
&  0.40134E+00_JPRB, 0.39855E+00_JPRB, 0.39610E+00_JPRB, 0.39381E+00_JPRB, 0.39244E+00_JPRB, 0.39101E+00_JPRB, 0.39014E+00_JPRB ,&
&  0.38947E+00_JPRB, 0.38485E+00_JPRB, 0.38353E+00_JPRB, 0.38571E+00_JPRB, 0.38858E+00_JPRB, 0.39067E+00_JPRB, 0.39320E+00_JPRB ,&
&  0.39043E+00_JPRB, 0.38464E+00_JPRB, 0.37658E+00_JPRB, 0.36584E+00_JPRB, 0.35753E+00_JPRB, 0.35414E+00_JPRB, 0.34808E+00_JPRB ,&
&  0.33604E+00_JPRB, 0.32586E+00_JPRB, 0.32956E+00_JPRB, 0.32589E+00_JPRB, 0.30672E+00_JPRB, 0.29681E+00_JPRB, 0.28582E+00_JPRB ,&
&  0.26940E+00_JPRB, 0.24642E+00_JPRB, 0.24120E+00_JPRB, 0.23900E+00_JPRB, 0.24413E+00_JPRB, 0.25278E+00_JPRB, 0.27208E+00_JPRB ,&
&  0.30676E+00_JPRB, 0.37524E+00_JPRB, 0.39327E+00_JPRB, 0.40268E+00_JPRB, 0.40727E+00_JPRB, 0.40711E+00_JPRB, 0.40706E+00_JPRB ,&
&  0.41358E+00_JPRB, 0.41978E+00_JPRB, 0.42963E+00_JPRB, 0.44054E+00_JPRB, 0.45486E+00_JPRB, 0.46711E+00_JPRB, 0.47612E+00_JPRB ,&
&  0.50516E+00_JPRB, 0.52072E+00_JPRB, 0.53483E+00_JPRB, 0.58392E+00_JPRB, 0.67551E+00_JPRB, 0.77380E+00_JPRB, 0.81579E+00_JPRB ,&
&  0.80958E+00_JPRB, 0.80009E+00_JPRB, 0.77048E+00_JPRB, 0.75276E+00_JPRB, 0.74515E+00_JPRB, 0.74311E+00_JPRB, 0.74492E+00_JPRB ,&
&  0.75079E+00_JPRB, 0.76601E+00_JPRB, 0.81680E+00_JPRB, 0.77520E+00_JPRB, 0.75019E+00_JPRB, 0.72473E+00_JPRB, 0.69574E+00_JPRB ,&
&  0.67369E+00_JPRB, 0.66167E+00_JPRB, 0.65385E+00_JPRB, 0.64322E+00_JPRB, 0.63250E+00_JPRB, 0.62341E+00_JPRB, 0.62062E+00_JPRB ,&
&  0.61240E+00_JPRB, 0.59996E+00_JPRB, 0.58213E+00_JPRB, 0.57195E+00_JPRB, 0.56250E+00_JPRB, 0.55361E+00_JPRB, 0.54418E+00_JPRB ,&
&  0.52207E+00_JPRB, 0.49221E+00_JPRB, 0.46865E+00_JPRB, 0.44775E+00_JPRB, 0.43060E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.42235E+00_JPRB, 0.41956E+00_JPRB, 0.42097E+00_JPRB, 0.42605E+00_JPRB, 0.42974E+00_JPRB, 0.42397E+00_JPRB, 0.41679E+00_JPRB ,&
&  0.41174E+00_JPRB, 0.40964E+00_JPRB, 0.40849E+00_JPRB, 0.40524E+00_JPRB, 0.40277E+00_JPRB, 0.40025E+00_JPRB, 0.39774E+00_JPRB ,&
&  0.39510E+00_JPRB, 0.39208E+00_JPRB, 0.38844E+00_JPRB, 0.38310E+00_JPRB, 0.38039E+00_JPRB, 0.38175E+00_JPRB, 0.38577E+00_JPRB ,&
&  0.38943E+00_JPRB, 0.38882E+00_JPRB, 0.38550E+00_JPRB, 0.37656E+00_JPRB, 0.36664E+00_JPRB, 0.35859E+00_JPRB, 0.35783E+00_JPRB ,&
&  0.35210E+00_JPRB, 0.34151E+00_JPRB, 0.32964E+00_JPRB, 0.33323E+00_JPRB, 0.33377E+00_JPRB, 0.31387E+00_JPRB, 0.29652E+00_JPRB ,&
&  0.27888E+00_JPRB, 0.26274E+00_JPRB, 0.25550E+00_JPRB, 0.25523E+00_JPRB, 0.26427E+00_JPRB, 0.28118E+00_JPRB, 0.30475E+00_JPRB ,&
&  0.34085E+00_JPRB, 0.37483E+00_JPRB, 0.39079E+00_JPRB, 0.40359E+00_JPRB, 0.40585E+00_JPRB, 0.40421E+00_JPRB, 0.40243E+00_JPRB ,&
&  0.40426E+00_JPRB, 0.40832E+00_JPRB, 0.41542E+00_JPRB, 0.42102E+00_JPRB, 0.43347E+00_JPRB, 0.45767E+00_JPRB, 0.50947E+00_JPRB ,&
&  0.64283E+00_JPRB, 0.64938E+00_JPRB, 0.59064E+00_JPRB, 0.64597E+00_JPRB, 0.74231E+00_JPRB, 0.83037E+00_JPRB, 0.89405E+00_JPRB ,&
&  0.90127E+00_JPRB, 0.85621E+00_JPRB, 0.81769E+00_JPRB, 0.79573E+00_JPRB, 0.78094E+00_JPRB, 0.76779E+00_JPRB, 0.76225E+00_JPRB ,&
&  0.76637E+00_JPRB, 0.78345E+00_JPRB, 0.86996E+00_JPRB, 0.76389E+00_JPRB, 0.70679E+00_JPRB, 0.68615E+00_JPRB, 0.66303E+00_JPRB ,&
&  0.64283E+00_JPRB, 0.63304E+00_JPRB, 0.63489E+00_JPRB, 0.64065E+00_JPRB, 0.61537E+00_JPRB, 0.59284E+00_JPRB, 0.57688E+00_JPRB ,&
&  0.57203E+00_JPRB, 0.56283E+00_JPRB, 0.55611E+00_JPRB, 0.55774E+00_JPRB, 0.55400E+00_JPRB, 0.53970E+00_JPRB, 0.53076E+00_JPRB ,&
&  0.51541E+00_JPRB, 0.48907E+00_JPRB, 0.47097E+00_JPRB, 0.45310E+00_JPRB, 0.43422E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.42068E+00_JPRB, 0.43040E+00_JPRB, 0.44403E+00_JPRB, 0.45326E+00_JPRB, 0.45540E+00_JPRB, 0.43606E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.40521E+00_JPRB, 0.40082E+00_JPRB, 0.40156E+00_JPRB, 0.40188E+00_JPRB, 0.40160E+00_JPRB, 0.40168E+00_JPRB, 0.40108E+00_JPRB ,&
&  0.39732E+00_JPRB, 0.39212E+00_JPRB, 0.38723E+00_JPRB, 0.38250E+00_JPRB, 0.37896E+00_JPRB, 0.37835E+00_JPRB, 0.38003E+00_JPRB ,&
&  0.38332E+00_JPRB, 0.38511E+00_JPRB, 0.38468E+00_JPRB, 0.37964E+00_JPRB, 0.37581E+00_JPRB, 0.37242E+00_JPRB, 0.36533E+00_JPRB ,&
&  0.35298E+00_JPRB, 0.35074E+00_JPRB, 0.34677E+00_JPRB, 0.34172E+00_JPRB, 0.34256E+00_JPRB, 0.32935E+00_JPRB, 0.31088E+00_JPRB ,&
&  0.29279E+00_JPRB, 0.27927E+00_JPRB, 0.27727E+00_JPRB, 0.28564E+00_JPRB, 0.30547E+00_JPRB, 0.33278E+00_JPRB, 0.34804E+00_JPRB ,&
&  0.36207E+00_JPRB, 0.37470E+00_JPRB, 0.38368E+00_JPRB, 0.39191E+00_JPRB, 0.39786E+00_JPRB, 0.40043E+00_JPRB, 0.40067E+00_JPRB ,&
&  0.39938E+00_JPRB, 0.39929E+00_JPRB, 0.40121E+00_JPRB, 0.38558E+00_JPRB, 0.39637E+00_JPRB, 0.46645E+00_JPRB, 0.53638E+00_JPRB ,&
&  0.62348E+00_JPRB, 0.64847E+00_JPRB, 0.66836E+00_JPRB, 0.73559E+00_JPRB, 0.80079E+00_JPRB, 0.85804E+00_JPRB, 0.90486E+00_JPRB ,&
&  0.91869E+00_JPRB, 0.87007E+00_JPRB, 0.82289E+00_JPRB, 0.80568E+00_JPRB, 0.79827E+00_JPRB, 0.78892E+00_JPRB, 0.75893E+00_JPRB ,&
&  0.73677E+00_JPRB, 0.70033E+00_JPRB, 0.65296E+00_JPRB, 0.61150E+00_JPRB, 0.60478E+00_JPRB, 0.62159E+00_JPRB, 0.64757E+00_JPRB ,&
&  0.65063E+00_JPRB, 0.64902E+00_JPRB, 0.64875E+00_JPRB, 0.65544E+00_JPRB, 0.60043E+00_JPRB, 0.56505E+00_JPRB, 0.54138E+00_JPRB ,&
&  0.53155E+00_JPRB, 0.52508E+00_JPRB, 0.52046E+00_JPRB, 0.53002E+00_JPRB, 0.53034E+00_JPRB, 0.52367E+00_JPRB, 0.50299E+00_JPRB ,&
&  0.47577E+00_JPRB, 0.45375E+00_JPRB, 0.43770E+00_JPRB, 0.42636E+00_JPRB, 0.42156E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.46748E+00_JPRB, 0.47514E+00_JPRB, 0.47860E+00_JPRB, 0.48081E+00_JPRB, 0.47476E+00_JPRB, 0.46461E+00_JPRB, 0.45385E+00_JPRB ,&
&  0.45184E+00_JPRB, 0.44134E+00_JPRB, 0.43125E+00_JPRB, 0.41704E+00_JPRB, 0.40470E+00_JPRB, 0.39432E+00_JPRB, 0.39245E+00_JPRB ,&
&  0.39467E+00_JPRB, 0.39709E+00_JPRB, 0.39516E+00_JPRB, 0.38907E+00_JPRB, 0.38299E+00_JPRB, 0.37973E+00_JPRB, 0.38007E+00_JPRB ,&
&  0.38218E+00_JPRB, 0.38508E+00_JPRB, 0.38622E+00_JPRB, 0.38491E+00_JPRB, 0.38326E+00_JPRB, 0.38068E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.36822E+00_JPRB, 0.36232E+00_JPRB, 0.35870E+00_JPRB, 0.35648E+00_JPRB, 0.35763E+00_JPRB, 0.34488E+00_JPRB, 0.32656E+00_JPRB ,&
&  0.30921E+00_JPRB, 0.30337E+00_JPRB, 0.31363E+00_JPRB, 0.34235E+00_JPRB, 0.36177E+00_JPRB, 0.37779E+00_JPRB, 0.38972E+00_JPRB ,&
&  0.38883E+00_JPRB, 0.38691E+00_JPRB, 0.39294E+00_JPRB, 0.40841E+00_JPRB, 0.41139E+00_JPRB, 0.41226E+00_JPRB, 0.40997E+00_JPRB ,&
&  0.40617E+00_JPRB, 0.40551E+00_JPRB, 0.40564E+00_JPRB, 0.39602E+00_JPRB, 0.45221E+00_JPRB, 0.56606E+00_JPRB, 0.59895E+00_JPRB ,&
&  0.67615E+00_JPRB, 0.73065E+00_JPRB, 0.80221E+00_JPRB, 0.82197E+00_JPRB, 0.81036E+00_JPRB, 0.83868E+00_JPRB, 0.90773E+00_JPRB ,&
&  0.92176E+00_JPRB, 0.89737E+00_JPRB, 0.86521E+00_JPRB, 0.85293E+00_JPRB, 0.81752E+00_JPRB, 0.78012E+00_JPRB, 0.74617E+00_JPRB ,&
&  0.73884E+00_JPRB, 0.72045E+00_JPRB, 0.68404E+00_JPRB, 0.64157E+00_JPRB, 0.62594E+00_JPRB, 0.62415E+00_JPRB, 0.63611E+00_JPRB ,&
&  0.64141E+00_JPRB, 0.65248E+00_JPRB, 0.65956E+00_JPRB, 0.66671E+00_JPRB, 0.61840E+00_JPRB, 0.56171E+00_JPRB, 0.52610E+00_JPRB ,&
&  0.50492E+00_JPRB, 0.49036E+00_JPRB, 0.47663E+00_JPRB, 0.48362E+00_JPRB, 0.49507E+00_JPRB, 0.49428E+00_JPRB, 0.47031E+00_JPRB ,&
&  0.45099E+00_JPRB, 0.44060E+00_JPRB, 0.43629E+00_JPRB, 0.44714E+00_JPRB, 0.45940E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.49847E+00_JPRB, 0.50764E+00_JPRB, 0.50618E+00_JPRB, 0.50215E+00_JPRB, 0.50169E+00_JPRB, 0.49282E+00_JPRB, 0.47483E+00_JPRB ,&
&  0.46968E+00_JPRB, 0.44658E+00_JPRB, 0.43518E+00_JPRB, 0.42508E+00_JPRB, 0.41701E+00_JPRB, 0.40671E+00_JPRB, 0.39231E+00_JPRB ,&
&  0.38786E+00_JPRB, 0.39150E+00_JPRB, 0.39814E+00_JPRB, 0.39781E+00_JPRB, 0.39374E+00_JPRB, 0.38884E+00_JPRB, 0.38678E+00_JPRB ,&
&  0.38737E+00_JPRB, 0.39000E+00_JPRB, 0.39394E+00_JPRB, 0.39239E+00_JPRB, 0.38832E+00_JPRB, 0.38497E+00_JPRB, 0.38520E+00_JPRB ,&
&  0.38097E+00_JPRB, 0.37357E+00_JPRB, 0.37303E+00_JPRB, 0.37224E+00_JPRB, 0.37380E+00_JPRB, 0.37383E+00_JPRB, 0.35781E+00_JPRB ,&
&  0.33858E+00_JPRB, 0.34522E+00_JPRB, 0.37398E+00_JPRB, 0.39467E+00_JPRB, 0.41313E+00_JPRB, 0.42115E+00_JPRB, 0.42668E+00_JPRB ,&
&  0.43091E+00_JPRB, 0.43591E+00_JPRB, 0.43816E+00_JPRB, 0.44072E+00_JPRB, 0.44112E+00_JPRB, 0.43924E+00_JPRB, 0.43535E+00_JPRB ,&
&  0.43240E+00_JPRB, 0.40399E+00_JPRB, 0.39781E+00_JPRB, 0.42094E+00_JPRB, 0.56473E+00_JPRB, 0.59134E+00_JPRB, 0.62275E+00_JPRB ,&
&  0.70608E+00_JPRB, 0.76147E+00_JPRB, 0.83290E+00_JPRB, 0.87743E+00_JPRB, 0.88345E+00_JPRB, 0.89129E+00_JPRB, 0.94724E+00_JPRB ,&
&  0.98449E+00_JPRB, 0.97619E+00_JPRB, 0.95096E+00_JPRB, 0.91230E+00_JPRB, 0.85200E+00_JPRB, 0.82553E+00_JPRB, 0.80789E+00_JPRB ,&
&  0.81256E+00_JPRB, 0.80765E+00_JPRB, 0.76885E+00_JPRB, 0.72055E+00_JPRB, 0.68657E+00_JPRB, 0.68646E+00_JPRB, 0.69155E+00_JPRB ,&
&  0.69308E+00_JPRB, 0.70354E+00_JPRB, 0.69901E+00_JPRB, 0.68693E+00_JPRB, 0.65406E+00_JPRB, 0.62205E+00_JPRB, 0.60479E+00_JPRB ,&
&  0.56385E+00_JPRB, 0.52129E+00_JPRB, 0.48350E+00_JPRB, 0.47975E+00_JPRB, 0.48970E+00_JPRB, 0.48803E+00_JPRB, 0.47215E+00_JPRB ,&
&  0.46707E+00_JPRB, 0.46134E+00_JPRB, 0.45886E+00_JPRB, 0.47255E+00_JPRB, 0.48367E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.50702E+00_JPRB, 0.51445E+00_JPRB, 0.51287E+00_JPRB, 0.50853E+00_JPRB, 0.51487E+00_JPRB, 0.50770E+00_JPRB, 0.50671E+00_JPRB ,&
&  0.52238E+00_JPRB, 0.51191E+00_JPRB, 0.52799E+00_JPRB, 0.46868E+00_JPRB, 0.43292E+00_JPRB, 0.41946E+00_JPRB, 0.40445E+00_JPRB ,&
&  0.39973E+00_JPRB, 0.40420E+00_JPRB, 0.40608E+00_JPRB, 0.40538E+00_JPRB, 0.40430E+00_JPRB, 0.39763E+00_JPRB, 0.39482E+00_JPRB ,&
&  0.39610E+00_JPRB, 0.40018E+00_JPRB, 0.40495E+00_JPRB, 0.40303E+00_JPRB, 0.39937E+00_JPRB, 0.39658E+00_JPRB, 0.39356E+00_JPRB ,&
&  0.38860E+00_JPRB, 0.38811E+00_JPRB, 0.38868E+00_JPRB, 0.38749E+00_JPRB, 0.38811E+00_JPRB, 0.39038E+00_JPRB, 0.38938E+00_JPRB ,&
&  0.39456E+00_JPRB, 0.39587E+00_JPRB, 0.42523E+00_JPRB, 0.44496E+00_JPRB, 0.45603E+00_JPRB, 0.45886E+00_JPRB, 0.46163E+00_JPRB ,&
&  0.46836E+00_JPRB, 0.47755E+00_JPRB, 0.48763E+00_JPRB, 0.49548E+00_JPRB, 0.49570E+00_JPRB, 0.49288E+00_JPRB, 0.48699E+00_JPRB ,&
&  0.48230E+00_JPRB, 0.42964E+00_JPRB, 0.47892E+00_JPRB, 0.50835E+00_JPRB, 0.61534E+00_JPRB, 0.73784E+00_JPRB, 0.74181E+00_JPRB ,&
&  0.91584E+00_JPRB, 0.89454E+00_JPRB, 0.93501E+00_JPRB, 0.94208E+00_JPRB, 0.94750E+00_JPRB, 0.97752E+00_JPRB, 0.10324E+01_JPRB ,&
&  0.11190E+01_JPRB, 0.10709E+01_JPRB, 0.10792E+01_JPRB, 0.99737E+00_JPRB, 0.94178E+00_JPRB, 0.93747E+00_JPRB, 0.92667E+00_JPRB ,&
&  0.92923E+00_JPRB, 0.92835E+00_JPRB, 0.90962E+00_JPRB, 0.81391E+00_JPRB, 0.78777E+00_JPRB, 0.77662E+00_JPRB, 0.73148E+00_JPRB ,&
&  0.74385E+00_JPRB, 0.73788E+00_JPRB, 0.70317E+00_JPRB, 0.72980E+00_JPRB, 0.67563E+00_JPRB, 0.64342E+00_JPRB, 0.64011E+00_JPRB ,&
&  0.63149E+00_JPRB, 0.56372E+00_JPRB, 0.52939E+00_JPRB, 0.51812E+00_JPRB, 0.50423E+00_JPRB, 0.49165E+00_JPRB, 0.49109E+00_JPRB ,&
&  0.47690E+00_JPRB, 0.47860E+00_JPRB, 0.49566E+00_JPRB, 0.49714E+00_JPRB, 0.49805E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.51739E+00_JPRB, 0.51644E+00_JPRB, 0.51452E+00_JPRB, 0.51437E+00_JPRB, 0.53171E+00_JPRB, 0.53909E+00_JPRB, 0.53935E+00_JPRB ,&
&  0.55485E+00_JPRB, 0.55200E+00_JPRB, 0.54982E+00_JPRB, 0.54094E+00_JPRB, 0.52823E+00_JPRB, 0.48599E+00_JPRB, 0.42484E+00_JPRB ,&
&  0.40373E+00_JPRB, 0.40560E+00_JPRB, 0.41240E+00_JPRB, 0.41315E+00_JPRB, 0.40900E+00_JPRB, 0.40359E+00_JPRB, 0.40132E+00_JPRB ,&
&  0.40507E+00_JPRB, 0.41003E+00_JPRB, 0.41556E+00_JPRB, 0.41698E+00_JPRB, 0.41432E+00_JPRB, 0.41290E+00_JPRB, 0.41062E+00_JPRB ,&
&  0.40338E+00_JPRB, 0.40213E+00_JPRB, 0.40152E+00_JPRB, 0.39834E+00_JPRB, 0.41143E+00_JPRB, 0.41874E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.43651E+00_JPRB, 0.46027E+00_JPRB, 0.47754E+00_JPRB, 0.49379E+00_JPRB, 0.49755E+00_JPRB, 0.50007E+00_JPRB, 0.50986E+00_JPRB ,&
&  0.52685E+00_JPRB, 0.54287E+00_JPRB, 0.55789E+00_JPRB, 0.56924E+00_JPRB, 0.57351E+00_JPRB, 0.55881E+00_JPRB, 0.57350E+00_JPRB ,&
&  0.57383E+00_JPRB, 0.59753E+00_JPRB, 0.65682E+00_JPRB, 0.66104E+00_JPRB, 0.75877E+00_JPRB, 0.83935E+00_JPRB, 0.84255E+00_JPRB ,&
&  0.92516E+00_JPRB, 0.91249E+00_JPRB, 0.10259E+01_JPRB, 0.11081E+01_JPRB, 0.11852E+01_JPRB, 0.12197E+01_JPRB, 0.12658E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14465E+01_JPRB, 0.12712E+01_JPRB, 0.11843E+01_JPRB, 0.11132E+01_JPRB, 0.11125E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10991E+01_JPRB, 0.10795E+01_JPRB, 0.10473E+01_JPRB, 0.98078E+00_JPRB, 0.90943E+00_JPRB, 0.84119E+00_JPRB ,&
&  0.80123E+00_JPRB, 0.72558E+00_JPRB, 0.68708E+00_JPRB, 0.69036E+00_JPRB, 0.68713E+00_JPRB, 0.63122E+00_JPRB, 0.64852E+00_JPRB ,&
&  0.67504E+00_JPRB, 0.62806E+00_JPRB, 0.58365E+00_JPRB, 0.54787E+00_JPRB, 0.53498E+00_JPRB, 0.51871E+00_JPRB, 0.48954E+00_JPRB ,&
&  0.46942E+00_JPRB, 0.49377E+00_JPRB, 0.51038E+00_JPRB, 0.51301E+00_JPRB, 0.51506E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.53253E+00_JPRB, 0.52475E+00_JPRB, 0.51625E+00_JPRB, 0.52154E+00_JPRB, 0.54196E+00_JPRB, 0.55011E+00_JPRB, 0.56438E+00_JPRB ,&
&  0.57298E+00_JPRB, 0.57391E+00_JPRB, 0.57188E+00_JPRB, 0.55678E+00_JPRB, 0.54721E+00_JPRB, 0.53982E+00_JPRB, 0.50257E+00_JPRB ,&
&  0.45412E+00_JPRB, 0.43823E+00_JPRB, 0.42431E+00_JPRB, 0.42098E+00_JPRB, 0.41961E+00_JPRB, 0.41362E+00_JPRB, 0.41035E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.41841E+00_JPRB, 0.42181E+00_JPRB, 0.42428E+00_JPRB, 0.42509E+00_JPRB, 0.42745E+00_JPRB, 0.42646E+00_JPRB ,&
&  0.41794E+00_JPRB, 0.41669E+00_JPRB, 0.41942E+00_JPRB, 0.41775E+00_JPRB, 0.43626E+00_JPRB, 0.45535E+00_JPRB, 0.46953E+00_JPRB ,&
&  0.49394E+00_JPRB, 0.51561E+00_JPRB, 0.52866E+00_JPRB, 0.53887E+00_JPRB, 0.54152E+00_JPRB, 0.55455E+00_JPRB, 0.57759E+00_JPRB ,&
&  0.60662E+00_JPRB, 0.62704E+00_JPRB, 0.64299E+00_JPRB, 0.65954E+00_JPRB, 0.65440E+00_JPRB, 0.66192E+00_JPRB, 0.70319E+00_JPRB ,&
&  0.75512E+00_JPRB, 0.82454E+00_JPRB, 0.85032E+00_JPRB, 0.89947E+00_JPRB, 0.99712E+00_JPRB, 0.10357E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.10246E+01_JPRB, 0.13555E+01_JPRB, 0.14005E+01_JPRB, 0.13894E+01_JPRB, 0.14533E+01_JPRB, 0.14334E+01_JPRB ,&
&  0.14431E+01_JPRB, 0.13767E+01_JPRB, 0.13104E+01_JPRB, 0.12760E+01_JPRB, 0.12534E+01_JPRB, 0.13183E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.12743E+01_JPRB, 0.11076E+01_JPRB, 0.97103E+00_JPRB, 0.83749E+00_JPRB, 0.87213E+00_JPRB, 0.83397E+00_JPRB ,&
&  0.79015E+00_JPRB, 0.68383E+00_JPRB, 0.71217E+00_JPRB, 0.72304E+00_JPRB, 0.73306E+00_JPRB, 0.76033E+00_JPRB, 0.76305E+00_JPRB ,&
&  0.76387E+00_JPRB, 0.76416E+00_JPRB, 0.67688E+00_JPRB, 0.61365E+00_JPRB, 0.58478E+00_JPRB, 0.55494E+00_JPRB, 0.50830E+00_JPRB ,&
&  0.49560E+00_JPRB, 0.52157E+00_JPRB, 0.53961E+00_JPRB, 0.54375E+00_JPRB, 0.53989E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.55589E+00_JPRB, 0.54431E+00_JPRB, 0.53023E+00_JPRB, 0.53893E+00_JPRB, 0.55566E+00_JPRB, 0.56134E+00_JPRB, 0.57666E+00_JPRB ,&
&  0.59225E+00_JPRB, 0.59330E+00_JPRB, 0.57859E+00_JPRB, 0.56595E+00_JPRB, 0.55814E+00_JPRB, 0.55326E+00_JPRB, 0.55149E+00_JPRB ,&
&  0.49829E+00_JPRB, 0.46505E+00_JPRB, 0.43345E+00_JPRB, 0.42728E+00_JPRB, 0.44680E+00_JPRB, 0.43603E+00_JPRB, 0.43330E+00_JPRB ,&
&  0.43014E+00_JPRB, 0.43299E+00_JPRB, 0.43026E+00_JPRB, 0.42975E+00_JPRB, 0.43123E+00_JPRB, 0.43551E+00_JPRB, 0.43980E+00_JPRB ,&
&  0.43524E+00_JPRB, 0.44120E+00_JPRB, 0.45209E+00_JPRB, 0.45432E+00_JPRB, 0.47633E+00_JPRB, 0.49739E+00_JPRB, 0.52697E+00_JPRB ,&
&  0.55650E+00_JPRB, 0.56728E+00_JPRB, 0.57275E+00_JPRB, 0.58564E+00_JPRB, 0.60181E+00_JPRB, 0.63432E+00_JPRB, 0.67498E+00_JPRB ,&
&  0.71802E+00_JPRB, 0.75058E+00_JPRB, 0.77195E+00_JPRB, 0.78274E+00_JPRB, 0.76882E+00_JPRB, 0.79802E+00_JPRB, 0.86680E+00_JPRB ,&
&  0.98228E+00_JPRB, 0.10216E+01_JPRB, 0.10224E+01_JPRB, 0.12785E+01_JPRB, 0.14498E+01_JPRB, 0.14469E+01_JPRB, 0.13127E+01_JPRB ,&
&  0.12400E+01_JPRB, 0.11694E+01_JPRB, 0.12979E+01_JPRB, 0.13996E+01_JPRB, 0.13801E+01_JPRB, 0.15049E+01_JPRB, 0.14581E+01_JPRB ,&
&  0.13374E+01_JPRB, 0.12485E+01_JPRB, 0.12040E+01_JPRB, 0.12608E+01_JPRB, 0.12641E+01_JPRB, 0.13554E+01_JPRB, 0.13615E+01_JPRB ,&
&  0.13689E+01_JPRB, 0.11877E+01_JPRB, 0.89616E+00_JPRB, 0.82529E+00_JPRB, 0.66365E+00_JPRB, 0.67259E+00_JPRB, 0.81382E+00_JPRB ,&
&  0.81078E+00_JPRB, 0.81116E+00_JPRB, 0.80305E+00_JPRB, 0.81632E+00_JPRB, 0.82727E+00_JPRB, 0.96406E+00_JPRB, 0.10071E+01_JPRB ,&
&  0.88891E+00_JPRB, 0.88209E+00_JPRB, 0.83621E+00_JPRB, 0.77523E+00_JPRB, 0.71527E+00_JPRB, 0.64356E+00_JPRB, 0.58978E+00_JPRB ,&
&  0.57492E+00_JPRB, 0.57267E+00_JPRB, 0.58011E+00_JPRB, 0.57472E+00_JPRB, 0.56283E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.59270E+00_JPRB, 0.58177E+00_JPRB, 0.56179E+00_JPRB, 0.56501E+00_JPRB, 0.57246E+00_JPRB, 0.58027E+00_JPRB, 0.59735E+00_JPRB ,&
&  0.61326E+00_JPRB, 0.60422E+00_JPRB, 0.58446E+00_JPRB, 0.57693E+00_JPRB, 0.57183E+00_JPRB, 0.56363E+00_JPRB, 0.56192E+00_JPRB ,&
&  0.56593E+00_JPRB, 0.53049E+00_JPRB, 0.48267E+00_JPRB, 0.44574E+00_JPRB, 0.45471E+00_JPRB, 0.45289E+00_JPRB, 0.46033E+00_JPRB ,&
&  0.45569E+00_JPRB, 0.46599E+00_JPRB, 0.45701E+00_JPRB, 0.45271E+00_JPRB, 0.45113E+00_JPRB, 0.45665E+00_JPRB, 0.46871E+00_JPRB ,&
&  0.48351E+00_JPRB, 0.51022E+00_JPRB, 0.53819E+00_JPRB, 0.54588E+00_JPRB, 0.57078E+00_JPRB, 0.56177E+00_JPRB, 0.59437E+00_JPRB ,&
&  0.62045E+00_JPRB, 0.62473E+00_JPRB, 0.62943E+00_JPRB, 0.65281E+00_JPRB, 0.69277E+00_JPRB, 0.74461E+00_JPRB, 0.81055E+00_JPRB ,&
&  0.87503E+00_JPRB, 0.90311E+00_JPRB, 0.88515E+00_JPRB, 0.87425E+00_JPRB, 0.90419E+00_JPRB, 0.97449E+00_JPRB, 0.98289E+00_JPRB ,&
&  0.10417E+01_JPRB, 0.10557E+01_JPRB, 0.10060E+01_JPRB, 0.13545E+01_JPRB, 0.16682E+01_JPRB, 0.17131E+01_JPRB, 0.16161E+01_JPRB ,&
&  0.15229E+01_JPRB, 0.14337E+01_JPRB, 0.15147E+01_JPRB, 0.15482E+01_JPRB, 0.15474E+01_JPRB, 0.15410E+01_JPRB, 0.14492E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.12095E+01_JPRB, 0.12270E+01_JPRB, 0.12428E+01_JPRB, 0.12765E+01_JPRB, 0.12472E+01_JPRB, 0.12982E+01_JPRB ,&
&  0.13534E+01_JPRB, 0.80033E+00_JPRB, 0.65825E+00_JPRB, 0.75059E+00_JPRB, 0.64929E+00_JPRB, 0.66268E+00_JPRB, 0.74244E+00_JPRB ,&
&  0.73299E+00_JPRB, 0.73766E+00_JPRB, 0.82343E+00_JPRB, 0.89969E+00_JPRB, 0.10079E+01_JPRB, 0.11970E+01_JPRB, 0.11950E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.10308E+01_JPRB, 0.10425E+01_JPRB, 0.10044E+01_JPRB, 0.91222E+00_JPRB, 0.78963E+00_JPRB, 0.69785E+00_JPRB ,&
&  0.65357E+00_JPRB, 0.64085E+00_JPRB, 0.63516E+00_JPRB, 0.62040E+00_JPRB, 0.59974E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.64978E+00_JPRB, 0.63460E+00_JPRB, 0.59852E+00_JPRB, 0.59014E+00_JPRB, 0.60088E+00_JPRB, 0.62011E+00_JPRB, 0.63693E+00_JPRB ,&
&  0.63716E+00_JPRB, 0.61838E+00_JPRB, 0.60322E+00_JPRB, 0.59728E+00_JPRB, 0.58730E+00_JPRB, 0.57238E+00_JPRB, 0.56522E+00_JPRB ,&
&  0.56244E+00_JPRB, 0.55617E+00_JPRB, 0.50926E+00_JPRB, 0.48516E+00_JPRB, 0.45896E+00_JPRB, 0.46494E+00_JPRB, 0.47472E+00_JPRB ,&
&  0.48751E+00_JPRB, 0.48541E+00_JPRB, 0.47922E+00_JPRB, 0.48125E+00_JPRB, 0.48333E+00_JPRB, 0.49333E+00_JPRB, 0.53716E+00_JPRB ,&
&  0.58782E+00_JPRB, 0.64059E+00_JPRB, 0.64898E+00_JPRB, 0.67254E+00_JPRB, 0.66373E+00_JPRB, 0.64103E+00_JPRB, 0.68818E+00_JPRB ,&
&  0.70314E+00_JPRB, 0.70258E+00_JPRB, 0.70668E+00_JPRB, 0.73978E+00_JPRB, 0.79115E+00_JPRB, 0.83222E+00_JPRB, 0.89192E+00_JPRB ,&
&  0.93215E+00_JPRB, 0.94493E+00_JPRB, 0.93231E+00_JPRB, 0.90553E+00_JPRB, 0.95099E+00_JPRB, 0.97958E+00_JPRB, 0.92617E+00_JPRB ,&
&  0.94078E+00_JPRB, 0.84413E+00_JPRB, 0.70815E+00_JPRB, 0.10530E+01_JPRB, 0.16772E+01_JPRB, 0.17958E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.15930E+01_JPRB, 0.15820E+01_JPRB, 0.16977E+01_JPRB, 0.17998E+01_JPRB, 0.17563E+01_JPRB, 0.16230E+01_JPRB, 0.14833E+01_JPRB ,&
&  0.13362E+01_JPRB, 0.12886E+01_JPRB, 0.13332E+01_JPRB, 0.15450E+01_JPRB, 0.15846E+01_JPRB, 0.15801E+01_JPRB, 0.15320E+01_JPRB ,&
&  0.12961E+01_JPRB, 0.10975E+01_JPRB, 0.78944E+00_JPRB, 0.80562E+00_JPRB, 0.64909E+00_JPRB, 0.65242E+00_JPRB, 0.72825E+00_JPRB ,&
&  0.71374E+00_JPRB, 0.76252E+00_JPRB, 0.85714E+00_JPRB, 0.95628E+00_JPRB, 0.12687E+01_JPRB, 0.14698E+01_JPRB, 0.15264E+01_JPRB ,&
&  0.14597E+01_JPRB, 0.12802E+01_JPRB, 0.12988E+01_JPRB, 0.11983E+01_JPRB, 0.11144E+01_JPRB, 0.97576E+00_JPRB, 0.84920E+00_JPRB ,&
&  0.77170E+00_JPRB, 0.74059E+00_JPRB, 0.72047E+00_JPRB, 0.69365E+00_JPRB, 0.66232E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.68345E+00_JPRB, 0.65037E+00_JPRB, 0.59731E+00_JPRB, 0.59116E+00_JPRB, 0.60904E+00_JPRB, 0.64707E+00_JPRB, 0.66817E+00_JPRB ,&
&  0.65820E+00_JPRB, 0.63757E+00_JPRB, 0.62123E+00_JPRB, 0.61181E+00_JPRB, 0.59482E+00_JPRB, 0.57291E+00_JPRB, 0.56362E+00_JPRB ,&
&  0.55669E+00_JPRB, 0.54022E+00_JPRB, 0.49938E+00_JPRB, 0.47505E+00_JPRB, 0.45374E+00_JPRB, 0.45285E+00_JPRB, 0.46638E+00_JPRB ,&
&  0.48014E+00_JPRB, 0.49127E+00_JPRB, 0.50668E+00_JPRB, 0.51834E+00_JPRB, 0.52750E+00_JPRB, 0.56746E+00_JPRB, 0.64919E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.75561E+00_JPRB, 0.77398E+00_JPRB, 0.74645E+00_JPRB, 0.74175E+00_JPRB, 0.74543E+00_JPRB, 0.73864E+00_JPRB ,&
&  0.71142E+00_JPRB, 0.69699E+00_JPRB, 0.71027E+00_JPRB, 0.74567E+00_JPRB, 0.82445E+00_JPRB, 0.89777E+00_JPRB, 0.99470E+00_JPRB ,&
&  0.10353E+01_JPRB, 0.10123E+01_JPRB, 0.95367E+00_JPRB, 0.94108E+00_JPRB, 0.99827E+00_JPRB, 0.99292E+00_JPRB, 0.95414E+00_JPRB ,&
&  0.81550E+00_JPRB, 0.72383E+00_JPRB, 0.82157E+00_JPRB, 0.10731E+01_JPRB, 0.14958E+01_JPRB, 0.18110E+01_JPRB, 0.17276E+01_JPRB ,&
&  0.17379E+01_JPRB, 0.17122E+01_JPRB, 0.17392E+01_JPRB, 0.14414E+01_JPRB, 0.15922E+01_JPRB, 0.15156E+01_JPRB, 0.14266E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.13792E+01_JPRB, 0.15267E+01_JPRB, 0.17859E+01_JPRB, 0.19545E+01_JPRB, 0.20431E+01_JPRB, 0.15600E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.13040E+01_JPRB, 0.11778E+01_JPRB, 0.98289E+00_JPRB, 0.83393E+00_JPRB, 0.84866E+00_JPRB, 0.83998E+00_JPRB ,&
&  0.83884E+00_JPRB, 0.85703E+00_JPRB, 0.95776E+00_JPRB, 0.12036E+01_JPRB, 0.16245E+01_JPRB, 0.19283E+01_JPRB, 0.19168E+01_JPRB ,&
&  0.17047E+01_JPRB, 0.17186E+01_JPRB, 0.15645E+01_JPRB, 0.13645E+01_JPRB, 0.12448E+01_JPRB, 0.11305E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.91277E+00_JPRB, 0.86072E+00_JPRB, 0.82278E+00_JPRB, 0.77503E+00_JPRB, 0.71683E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.71474E+00_JPRB, 0.67872E+00_JPRB, 0.62719E+00_JPRB, 0.62273E+00_JPRB, 0.65384E+00_JPRB, 0.69217E+00_JPRB, 0.69830E+00_JPRB ,&
&  0.66498E+00_JPRB, 0.64239E+00_JPRB, 0.62483E+00_JPRB, 0.62258E+00_JPRB, 0.61149E+00_JPRB, 0.58549E+00_JPRB, 0.56712E+00_JPRB ,&
&  0.56414E+00_JPRB, 0.53166E+00_JPRB, 0.48991E+00_JPRB, 0.48546E+00_JPRB, 0.47684E+00_JPRB, 0.45029E+00_JPRB, 0.45593E+00_JPRB ,&
&  0.47328E+00_JPRB, 0.49553E+00_JPRB, 0.52208E+00_JPRB, 0.56276E+00_JPRB, 0.62029E+00_JPRB, 0.71129E+00_JPRB, 0.82409E+00_JPRB ,&
&  0.84353E+00_JPRB, 0.84755E+00_JPRB, 0.87465E+00_JPRB, 0.84613E+00_JPRB, 0.78336E+00_JPRB, 0.74833E+00_JPRB, 0.73228E+00_JPRB ,&
&  0.70915E+00_JPRB, 0.69954E+00_JPRB, 0.74000E+00_JPRB, 0.82501E+00_JPRB, 0.94595E+00_JPRB, 0.10276E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.10492E+01_JPRB, 0.10342E+01_JPRB, 0.11147E+01_JPRB, 0.87031E+00_JPRB, 0.82773E+00_JPRB, 0.79616E+00_JPRB ,&
&  0.89117E+00_JPRB, 0.85137E+00_JPRB, 0.86479E+00_JPRB, 0.11174E+01_JPRB, 0.11445E+01_JPRB, 0.14032E+01_JPRB, 0.16947E+01_JPRB ,&
&  0.18454E+01_JPRB, 0.18680E+01_JPRB, 0.18539E+01_JPRB, 0.11681E+01_JPRB, 0.10910E+01_JPRB, 0.14220E+01_JPRB, 0.15008E+01_JPRB ,&
&  0.15032E+01_JPRB, 0.16707E+01_JPRB, 0.18940E+01_JPRB, 0.22503E+01_JPRB, 0.17267E+01_JPRB, 0.14854E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11456E+01_JPRB, 0.10670E+01_JPRB, 0.12552E+01_JPRB, 0.11475E+01_JPRB, 0.11548E+01_JPRB, 0.11091E+01_JPRB, 0.10230E+01_JPRB ,&
&  0.10312E+01_JPRB, 0.10872E+01_JPRB, 0.12484E+01_JPRB, 0.16662E+01_JPRB, 0.21663E+01_JPRB, 0.21810E+01_JPRB, 0.23829E+01_JPRB ,&
&  0.23160E+01_JPRB, 0.20360E+01_JPRB, 0.16756E+01_JPRB, 0.15059E+01_JPRB, 0.13834E+01_JPRB, 0.12852E+01_JPRB, 0.11772E+01_JPRB ,&
&  0.10604E+01_JPRB, 0.97354E+00_JPRB, 0.90286E+00_JPRB, 0.82916E+00_JPRB, 0.75874E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.77215E+00_JPRB, 0.73419E+00_JPRB, 0.67344E+00_JPRB, 0.67594E+00_JPRB, 0.71455E+00_JPRB, 0.73146E+00_JPRB, 0.70360E+00_JPRB ,&
&  0.67589E+00_JPRB, 0.64679E+00_JPRB, 0.63559E+00_JPRB, 0.62707E+00_JPRB, 0.61376E+00_JPRB, 0.60765E+00_JPRB, 0.58759E+00_JPRB ,&
&  0.58965E+00_JPRB, 0.57439E+00_JPRB, 0.52000E+00_JPRB, 0.49221E+00_JPRB, 0.49546E+00_JPRB, 0.46687E+00_JPRB, 0.44953E+00_JPRB ,&
&  0.48194E+00_JPRB, 0.52104E+00_JPRB, 0.57615E+00_JPRB, 0.64978E+00_JPRB, 0.74464E+00_JPRB, 0.90861E+00_JPRB, 0.99483E+00_JPRB ,&
&  0.95373E+00_JPRB, 0.91637E+00_JPRB, 0.84227E+00_JPRB, 0.77193E+00_JPRB, 0.71595E+00_JPRB, 0.68889E+00_JPRB, 0.67810E+00_JPRB ,&
&  0.67906E+00_JPRB, 0.70222E+00_JPRB, 0.78069E+00_JPRB, 0.89707E+00_JPRB, 0.10185E+01_JPRB, 0.10673E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.11803E+01_JPRB, 0.11915E+01_JPRB, 0.11261E+01_JPRB, 0.10113E+01_JPRB, 0.85484E+00_JPRB, 0.84961E+00_JPRB ,&
&  0.89172E+00_JPRB, 0.83064E+00_JPRB, 0.87852E+00_JPRB, 0.11197E+01_JPRB, 0.13116E+01_JPRB, 0.14104E+01_JPRB, 0.17175E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.11527E+01_JPRB, 0.98046E+00_JPRB, 0.81911E+00_JPRB, 0.11899E+01_JPRB, 0.16900E+01_JPRB, 0.18730E+01_JPRB ,&
&  0.19580E+01_JPRB, 0.21433E+01_JPRB, 0.23989E+01_JPRB, 0.21809E+01_JPRB, 0.15963E+01_JPRB, 0.13082E+01_JPRB, 0.16112E+01_JPRB ,&
&  0.17055E+01_JPRB, 0.17906E+01_JPRB, 0.17408E+01_JPRB, 0.15011E+01_JPRB, 0.12378E+01_JPRB, 0.13998E+01_JPRB, 0.13808E+01_JPRB ,&
&  0.14152E+01_JPRB, 0.14636E+01_JPRB, 0.18965E+01_JPRB, 0.24371E+01_JPRB, 0.26872E+01_JPRB, 0.29596E+01_JPRB, 0.30748E+01_JPRB ,&
&  0.26531E+01_JPRB, 0.23294E+01_JPRB, 0.19513E+01_JPRB, 0.16657E+01_JPRB, 0.14903E+01_JPRB, 0.13789E+01_JPRB, 0.12893E+01_JPRB ,&
&  0.11767E+01_JPRB, 0.10938E+01_JPRB, 0.10075E+01_JPRB, 0.90023E+00_JPRB, 0.81797E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.84938E+00_JPRB, 0.79531E+00_JPRB, 0.72578E+00_JPRB, 0.72727E+00_JPRB, 0.76799E+00_JPRB, 0.75201E+00_JPRB, 0.70679E+00_JPRB ,&
&  0.69180E+00_JPRB, 0.66273E+00_JPRB, 0.64968E+00_JPRB, 0.63353E+00_JPRB, 0.61638E+00_JPRB, 0.60551E+00_JPRB, 0.60336E+00_JPRB ,&
&  0.62513E+00_JPRB, 0.65830E+00_JPRB, 0.59238E+00_JPRB, 0.54868E+00_JPRB, 0.52944E+00_JPRB, 0.50839E+00_JPRB, 0.46741E+00_JPRB ,&
&  0.51502E+00_JPRB, 0.58843E+00_JPRB, 0.66583E+00_JPRB, 0.77090E+00_JPRB, 0.91818E+00_JPRB, 0.96214E+00_JPRB, 0.96098E+00_JPRB ,&
&  0.93726E+00_JPRB, 0.88092E+00_JPRB, 0.80325E+00_JPRB, 0.73212E+00_JPRB, 0.69447E+00_JPRB, 0.68337E+00_JPRB, 0.68193E+00_JPRB ,&
&  0.70571E+00_JPRB, 0.77700E+00_JPRB, 0.86808E+00_JPRB, 0.97899E+00_JPRB, 0.10965E+01_JPRB, 0.11547E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.13156E+01_JPRB, 0.13267E+01_JPRB, 0.12759E+01_JPRB, 0.12303E+01_JPRB, 0.10535E+01_JPRB, 0.11955E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.12027E+01_JPRB, 0.11432E+01_JPRB, 0.14126E+01_JPRB, 0.13700E+01_JPRB, 0.13642E+01_JPRB, 0.15410E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.11078E+01_JPRB, 0.12763E+01_JPRB, 0.14721E+01_JPRB, 0.16040E+01_JPRB, 0.18009E+01_JPRB, 0.26339E+01_JPRB ,&
&  0.22174E+01_JPRB, 0.25389E+01_JPRB, 0.27796E+01_JPRB, 0.26933E+01_JPRB, 0.12227E+01_JPRB, 0.95170E+00_JPRB, 0.15360E+01_JPRB ,&
&  0.17197E+01_JPRB, 0.11779E+01_JPRB, 0.89349E+00_JPRB, 0.11249E+01_JPRB, 0.87878E+00_JPRB, 0.10494E+01_JPRB, 0.14878E+01_JPRB ,&
&  0.17232E+01_JPRB, 0.20152E+01_JPRB, 0.28627E+01_JPRB, 0.30856E+01_JPRB, 0.30068E+01_JPRB, 0.37457E+01_JPRB, 0.31865E+01_JPRB ,&
&  0.27910E+01_JPRB, 0.21988E+01_JPRB, 0.20310E+01_JPRB, 0.17803E+01_JPRB, 0.16009E+01_JPRB, 0.14941E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.11917E+01_JPRB, 0.10973E+01_JPRB, 0.99465E+00_JPRB, 0.91281E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.90414E+00_JPRB, 0.84688E+00_JPRB, 0.78830E+00_JPRB, 0.78104E+00_JPRB, 0.80433E+00_JPRB, 0.75254E+00_JPRB, 0.71828E+00_JPRB ,&
&  0.68785E+00_JPRB, 0.67108E+00_JPRB, 0.65179E+00_JPRB, 0.62877E+00_JPRB, 0.60962E+00_JPRB, 0.61039E+00_JPRB, 0.62053E+00_JPRB ,&
&  0.64290E+00_JPRB, 0.70450E+00_JPRB, 0.73745E+00_JPRB, 0.69822E+00_JPRB, 0.66599E+00_JPRB, 0.61857E+00_JPRB, 0.57772E+00_JPRB ,&
&  0.62262E+00_JPRB, 0.73054E+00_JPRB, 0.79934E+00_JPRB, 0.84748E+00_JPRB, 0.88431E+00_JPRB, 0.86410E+00_JPRB, 0.88869E+00_JPRB ,&
&  0.88335E+00_JPRB, 0.82707E+00_JPRB, 0.76899E+00_JPRB, 0.72210E+00_JPRB, 0.70322E+00_JPRB, 0.70226E+00_JPRB, 0.73491E+00_JPRB ,&
&  0.79001E+00_JPRB, 0.85321E+00_JPRB, 0.92630E+00_JPRB, 0.10635E+01_JPRB, 0.12030E+01_JPRB, 0.12641E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.13676E+01_JPRB, 0.13941E+01_JPRB, 0.14059E+01_JPRB, 0.14299E+01_JPRB, 0.15423E+01_JPRB, 0.13884E+01_JPRB, 0.11728E+01_JPRB ,&
&  0.10674E+01_JPRB, 0.12200E+01_JPRB, 0.16949E+01_JPRB, 0.16048E+01_JPRB, 0.13792E+01_JPRB, 0.14168E+01_JPRB, 0.14042E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.16108E+01_JPRB, 0.18308E+01_JPRB, 0.20715E+01_JPRB, 0.22433E+01_JPRB, 0.18202E+01_JPRB, 0.25604E+01_JPRB ,&
&  0.27585E+01_JPRB, 0.26971E+01_JPRB, 0.20173E+01_JPRB, 0.13915E+01_JPRB, 0.14191E+01_JPRB, 0.10864E+01_JPRB, 0.77181E+00_JPRB ,&
&  0.73028E+00_JPRB, 0.70181E+00_JPRB, 0.64768E+00_JPRB, 0.68709E+00_JPRB, 0.70186E+00_JPRB, 0.97300E+00_JPRB, 0.14950E+01_JPRB ,&
&  0.22350E+01_JPRB, 0.28876E+01_JPRB, 0.33188E+01_JPRB, 0.33600E+01_JPRB, 0.32126E+01_JPRB, 0.37301E+01_JPRB, 0.32826E+01_JPRB ,&
&  0.23375E+01_JPRB, 0.20053E+01_JPRB, 0.18600E+01_JPRB, 0.17789E+01_JPRB, 0.16993E+01_JPRB, 0.16186E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.13680E+01_JPRB, 0.12716E+01_JPRB, 0.11842E+01_JPRB, 0.10697E+01_JPRB, 0.97834E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.95158E+00_JPRB, 0.89935E+00_JPRB, 0.87324E+00_JPRB, 0.84837E+00_JPRB, 0.83033E+00_JPRB, 0.75623E+00_JPRB, 0.72789E+00_JPRB ,&
&  0.68238E+00_JPRB, 0.66631E+00_JPRB, 0.63530E+00_JPRB, 0.61760E+00_JPRB, 0.60637E+00_JPRB, 0.62055E+00_JPRB, 0.64869E+00_JPRB ,&
&  0.68541E+00_JPRB, 0.78467E+00_JPRB, 0.91853E+00_JPRB, 0.97600E+00_JPRB, 0.10223E+01_JPRB, 0.97866E+00_JPRB, 0.90407E+00_JPRB ,&
&  0.86967E+00_JPRB, 0.96212E+00_JPRB, 0.93634E+00_JPRB, 0.90896E+00_JPRB, 0.89791E+00_JPRB, 0.86960E+00_JPRB, 0.84681E+00_JPRB ,&
&  0.83716E+00_JPRB, 0.79403E+00_JPRB, 0.73874E+00_JPRB, 0.71655E+00_JPRB, 0.71964E+00_JPRB, 0.74465E+00_JPRB, 0.78837E+00_JPRB ,&
&  0.86273E+00_JPRB, 0.90406E+00_JPRB, 0.10011E+01_JPRB, 0.11614E+01_JPRB, 0.12886E+01_JPRB, 0.13373E+01_JPRB, 0.14081E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.14714E+01_JPRB, 0.14532E+01_JPRB, 0.14144E+01_JPRB, 0.12300E+01_JPRB, 0.10290E+01_JPRB, 0.12314E+01_JPRB ,&
&  0.14101E+01_JPRB, 0.17402E+01_JPRB, 0.19206E+01_JPRB, 0.19971E+01_JPRB, 0.15433E+01_JPRB, 0.14367E+01_JPRB, 0.14520E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.25625E+01_JPRB, 0.23277E+01_JPRB, 0.21382E+01_JPRB, 0.26718E+01_JPRB, 0.22349E+01_JPRB, 0.23657E+01_JPRB ,&
&  0.28503E+01_JPRB, 0.26754E+01_JPRB, 0.20662E+01_JPRB, 0.15959E+01_JPRB, 0.20246E+01_JPRB, 0.20289E+01_JPRB, 0.99188E+00_JPRB ,&
&  0.80283E+00_JPRB, 0.74643E+00_JPRB, 0.70331E+00_JPRB, 0.67283E+00_JPRB, 0.69084E+00_JPRB, 0.10208E+01_JPRB, 0.19313E+01_JPRB ,&
&  0.26446E+01_JPRB, 0.35568E+01_JPRB, 0.36836E+01_JPRB, 0.35702E+01_JPRB, 0.32795E+01_JPRB, 0.33444E+01_JPRB, 0.28135E+01_JPRB ,&
&  0.23550E+01_JPRB, 0.20882E+01_JPRB, 0.19093E+01_JPRB, 0.18207E+01_JPRB, 0.17398E+01_JPRB, 0.16588E+01_JPRB, 0.15545E+01_JPRB ,&
&  0.14327E+01_JPRB, 0.13303E+01_JPRB, 0.12372E+01_JPRB, 0.11182E+01_JPRB, 0.10207E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.98570E+00_JPRB, 0.95040E+00_JPRB, 0.94209E+00_JPRB, 0.92092E+00_JPRB, 0.86404E+00_JPRB, 0.79106E+00_JPRB, 0.72493E+00_JPRB ,&
&  0.68336E+00_JPRB, 0.65389E+00_JPRB, 0.63108E+00_JPRB, 0.62043E+00_JPRB, 0.62066E+00_JPRB, 0.64319E+00_JPRB, 0.67793E+00_JPRB ,&
&  0.75303E+00_JPRB, 0.88419E+00_JPRB, 0.10333E+01_JPRB, 0.13014E+01_JPRB, 0.13841E+01_JPRB, 0.12980E+01_JPRB, 0.13739E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.12749E+01_JPRB, 0.11500E+01_JPRB, 0.10304E+01_JPRB, 0.95342E+00_JPRB, 0.91163E+00_JPRB, 0.87189E+00_JPRB ,&
&  0.81770E+00_JPRB, 0.78974E+00_JPRB, 0.74841E+00_JPRB, 0.75978E+00_JPRB, 0.78099E+00_JPRB, 0.81042E+00_JPRB, 0.86476E+00_JPRB ,&
&  0.89360E+00_JPRB, 0.92937E+00_JPRB, 0.10632E+01_JPRB, 0.12168E+01_JPRB, 0.13247E+01_JPRB, 0.14084E+01_JPRB, 0.15068E+01_JPRB ,&
&  0.15340E+01_JPRB, 0.15254E+01_JPRB, 0.14795E+01_JPRB, 0.14128E+01_JPRB, 0.11606E+01_JPRB, 0.13164E+01_JPRB, 0.13702E+01_JPRB ,&
&  0.15132E+01_JPRB, 0.17002E+01_JPRB, 0.17649E+01_JPRB, 0.18736E+01_JPRB, 0.16481E+01_JPRB, 0.14866E+01_JPRB, 0.14690E+01_JPRB ,&
&  0.20031E+01_JPRB, 0.21515E+01_JPRB, 0.19411E+01_JPRB, 0.21303E+01_JPRB, 0.26535E+01_JPRB, 0.33879E+01_JPRB, 0.21706E+01_JPRB ,&
&  0.19190E+01_JPRB, 0.24831E+01_JPRB, 0.20816E+01_JPRB, 0.18340E+01_JPRB, 0.21945E+01_JPRB, 0.27096E+01_JPRB, 0.19892E+01_JPRB ,&
&  0.99020E+00_JPRB, 0.79882E+00_JPRB, 0.83013E+00_JPRB, 0.89676E+00_JPRB, 0.89994E+00_JPRB, 0.13553E+01_JPRB, 0.34570E+01_JPRB ,&
&  0.36690E+01_JPRB, 0.42509E+01_JPRB, 0.39545E+01_JPRB, 0.35317E+01_JPRB, 0.32796E+01_JPRB, 0.31387E+01_JPRB, 0.27526E+01_JPRB ,&
&  0.23581E+01_JPRB, 0.21247E+01_JPRB, 0.19427E+01_JPRB, 0.18120E+01_JPRB, 0.16898E+01_JPRB, 0.15717E+01_JPRB, 0.14609E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.12967E+01_JPRB, 0.12237E+01_JPRB, 0.11269E+01_JPRB, 0.10416E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.99110E+00_JPRB, 0.96521E+00_JPRB, 0.96034E+00_JPRB, 0.93735E+00_JPRB, 0.89921E+00_JPRB, 0.85900E+00_JPRB, 0.77575E+00_JPRB ,&
&  0.70292E+00_JPRB, 0.65636E+00_JPRB, 0.63663E+00_JPRB, 0.63488E+00_JPRB, 0.64932E+00_JPRB, 0.67928E+00_JPRB, 0.71345E+00_JPRB ,&
&  0.83213E+00_JPRB, 0.95130E+00_JPRB, 0.11462E+01_JPRB, 0.14175E+01_JPRB, 0.15659E+01_JPRB, 0.14348E+01_JPRB, 0.15423E+01_JPRB ,&
&  0.17937E+01_JPRB, 0.15667E+01_JPRB, 0.13410E+01_JPRB, 0.12266E+01_JPRB, 0.11319E+01_JPRB, 0.10585E+01_JPRB, 0.97705E+00_JPRB ,&
&  0.90627E+00_JPRB, 0.89973E+00_JPRB, 0.85925E+00_JPRB, 0.86739E+00_JPRB, 0.87035E+00_JPRB, 0.91235E+00_JPRB, 0.90078E+00_JPRB ,&
&  0.90332E+00_JPRB, 0.99518E+00_JPRB, 0.11245E+01_JPRB, 0.12467E+01_JPRB, 0.13580E+01_JPRB, 0.14589E+01_JPRB, 0.15768E+01_JPRB ,&
&  0.16334E+01_JPRB, 0.16030E+01_JPRB, 0.15249E+01_JPRB, 0.13856E+01_JPRB, 0.13537E+01_JPRB, 0.13725E+01_JPRB, 0.14722E+01_JPRB ,&
&  0.15260E+01_JPRB, 0.16439E+01_JPRB, 0.16955E+01_JPRB, 0.16847E+01_JPRB, 0.15826E+01_JPRB, 0.15040E+01_JPRB, 0.15237E+01_JPRB ,&
&  0.18952E+01_JPRB, 0.19470E+01_JPRB, 0.17808E+01_JPRB, 0.22508E+01_JPRB, 0.28410E+01_JPRB, 0.32865E+01_JPRB, 0.30478E+01_JPRB ,&
&  0.25783E+01_JPRB, 0.21178E+01_JPRB, 0.19459E+01_JPRB, 0.22019E+01_JPRB, 0.24314E+01_JPRB, 0.27200E+01_JPRB, 0.34451E+01_JPRB ,&
&  0.35681E+01_JPRB, 0.21576E+01_JPRB, 0.18258E+01_JPRB, 0.23863E+01_JPRB, 0.20468E+01_JPRB, 0.21207E+01_JPRB, 0.30332E+01_JPRB ,&
&  0.37708E+01_JPRB, 0.42598E+01_JPRB, 0.40508E+01_JPRB, 0.35321E+01_JPRB, 0.32869E+01_JPRB, 0.29630E+01_JPRB, 0.26442E+01_JPRB ,&
&  0.22655E+01_JPRB, 0.19702E+01_JPRB, 0.17658E+01_JPRB, 0.16143E+01_JPRB, 0.14732E+01_JPRB, 0.13620E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12101E+01_JPRB, 0.11640E+01_JPRB, 0.10933E+01_JPRB, 0.10282E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.97959E+00_JPRB, 0.96472E+00_JPRB, 0.95755E+00_JPRB, 0.94088E+00_JPRB, 0.95991E+00_JPRB, 0.95278E+00_JPRB, 0.87333E+00_JPRB ,&
&  0.79562E+00_JPRB, 0.67943E+00_JPRB, 0.66232E+00_JPRB, 0.67276E+00_JPRB, 0.69551E+00_JPRB, 0.73082E+00_JPRB, 0.80531E+00_JPRB ,&
&  0.92275E+00_JPRB, 0.10424E+01_JPRB, 0.12849E+01_JPRB, 0.15842E+01_JPRB, 0.18839E+01_JPRB, 0.22968E+01_JPRB, 0.16193E+01_JPRB ,&
&  0.16232E+01_JPRB, 0.15433E+01_JPRB, 0.13810E+01_JPRB, 0.13534E+01_JPRB, 0.12991E+01_JPRB, 0.12725E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.10209E+01_JPRB, 0.97652E+00_JPRB, 0.94131E+00_JPRB, 0.95888E+00_JPRB, 0.96435E+00_JPRB, 0.91747E+00_JPRB ,&
&  0.97343E+00_JPRB, 0.10652E+01_JPRB, 0.11765E+01_JPRB, 0.12820E+01_JPRB, 0.13951E+01_JPRB, 0.15200E+01_JPRB, 0.16071E+01_JPRB ,&
&  0.16982E+01_JPRB, 0.16920E+01_JPRB, 0.16097E+01_JPRB, 0.14461E+01_JPRB, 0.14285E+01_JPRB, 0.14336E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.13682E+01_JPRB, 0.13154E+01_JPRB, 0.14842E+01_JPRB, 0.15716E+01_JPRB, 0.15185E+01_JPRB, 0.14189E+01_JPRB, 0.14543E+01_JPRB ,&
&  0.18360E+01_JPRB, 0.21162E+01_JPRB, 0.24407E+01_JPRB, 0.26444E+01_JPRB, 0.30563E+01_JPRB, 0.33156E+01_JPRB, 0.34222E+01_JPRB ,&
&  0.34579E+01_JPRB, 0.29606E+01_JPRB, 0.26964E+01_JPRB, 0.27467E+01_JPRB, 0.27145E+01_JPRB, 0.28930E+01_JPRB, 0.33048E+01_JPRB ,&
&  0.38749E+01_JPRB, 0.47324E+01_JPRB, 0.48512E+01_JPRB, 0.39803E+01_JPRB, 0.35233E+01_JPRB, 0.30243E+01_JPRB, 0.32348E+01_JPRB ,&
&  0.37748E+01_JPRB, 0.37278E+01_JPRB, 0.35934E+01_JPRB, 0.37207E+01_JPRB, 0.29228E+01_JPRB, 0.25571E+01_JPRB, 0.22760E+01_JPRB ,&
&  0.20418E+01_JPRB, 0.17783E+01_JPRB, 0.15793E+01_JPRB, 0.14215E+01_JPRB, 0.12688E+01_JPRB, 0.11901E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.11248E+01_JPRB, 0.10952E+01_JPRB, 0.10489E+01_JPRB, 0.10100E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.96980E+00_JPRB, 0.94994E+00_JPRB, 0.94310E+00_JPRB, 0.94007E+00_JPRB, 0.10054E+01_JPRB, 0.10235E+01_JPRB, 0.10109E+01_JPRB ,&
&  0.98630E+00_JPRB, 0.73522E+00_JPRB, 0.70589E+00_JPRB, 0.71873E+00_JPRB, 0.75290E+00_JPRB, 0.80558E+00_JPRB, 0.90002E+00_JPRB ,&
&  0.99637E+00_JPRB, 0.11393E+01_JPRB, 0.14025E+01_JPRB, 0.17437E+01_JPRB, 0.20324E+01_JPRB, 0.22153E+01_JPRB, 0.21558E+01_JPRB ,&
&  0.14042E+01_JPRB, 0.16204E+01_JPRB, 0.14391E+01_JPRB, 0.15124E+01_JPRB, 0.15170E+01_JPRB, 0.14774E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.11189E+01_JPRB, 0.10974E+01_JPRB, 0.11126E+01_JPRB, 0.10940E+01_JPRB, 0.10273E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.11835E+01_JPRB, 0.12994E+01_JPRB, 0.13910E+01_JPRB, 0.15124E+01_JPRB, 0.16252E+01_JPRB, 0.16993E+01_JPRB ,&
&  0.17818E+01_JPRB, 0.17842E+01_JPRB, 0.16445E+01_JPRB, 0.15621E+01_JPRB, 0.15160E+01_JPRB, 0.15182E+01_JPRB, 0.14986E+01_JPRB ,&
&  0.13950E+01_JPRB, 0.13127E+01_JPRB, 0.14058E+01_JPRB, 0.14017E+01_JPRB, 0.13524E+01_JPRB, 0.14323E+01_JPRB, 0.15534E+01_JPRB ,&
&  0.20189E+01_JPRB, 0.22802E+01_JPRB, 0.31283E+01_JPRB, 0.29019E+01_JPRB, 0.29613E+01_JPRB, 0.32886E+01_JPRB, 0.31892E+01_JPRB ,&
&  0.28591E+01_JPRB, 0.29247E+01_JPRB, 0.28236E+01_JPRB, 0.30160E+01_JPRB, 0.31183E+01_JPRB, 0.33536E+01_JPRB, 0.34873E+01_JPRB ,&
&  0.37761E+01_JPRB, 0.41851E+01_JPRB, 0.46543E+01_JPRB, 0.38131E+01_JPRB, 0.34198E+01_JPRB, 0.31574E+01_JPRB, 0.33374E+01_JPRB ,&
&  0.35622E+01_JPRB, 0.34635E+01_JPRB, 0.35434E+01_JPRB, 0.32942E+01_JPRB, 0.24737E+01_JPRB, 0.21876E+01_JPRB, 0.19501E+01_JPRB ,&
&  0.17605E+01_JPRB, 0.15592E+01_JPRB, 0.13955E+01_JPRB, 0.12525E+01_JPRB, 0.11545E+01_JPRB, 0.11186E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.10799E+01_JPRB, 0.10633E+01_JPRB, 0.10275E+01_JPRB, 0.99552E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.97404E+00_JPRB, 0.96515E+00_JPRB, 0.96600E+00_JPRB, 0.95170E+00_JPRB, 0.10198E+01_JPRB, 0.11283E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.77133E+00_JPRB, 0.75336E+00_JPRB, 0.76990E+00_JPRB, 0.80543E+00_JPRB, 0.88160E+00_JPRB, 0.97431E+00_JPRB ,&
&  0.10929E+01_JPRB, 0.12662E+01_JPRB, 0.15477E+01_JPRB, 0.18877E+01_JPRB, 0.21430E+01_JPRB, 0.24306E+01_JPRB, 0.24628E+01_JPRB ,&
&  0.17524E+01_JPRB, 0.14382E+01_JPRB, 0.14682E+01_JPRB, 0.15656E+01_JPRB, 0.15286E+01_JPRB, 0.15812E+01_JPRB, 0.15722E+01_JPRB ,&
&  0.14502E+01_JPRB, 0.13209E+01_JPRB, 0.13332E+01_JPRB, 0.13356E+01_JPRB, 0.11639E+01_JPRB, 0.11494E+01_JPRB, 0.11707E+01_JPRB ,&
&  0.12427E+01_JPRB, 0.13435E+01_JPRB, 0.14482E+01_JPRB, 0.15334E+01_JPRB, 0.16590E+01_JPRB, 0.17502E+01_JPRB, 0.18303E+01_JPRB ,&
&  0.18571E+01_JPRB, 0.18708E+01_JPRB, 0.17579E+01_JPRB, 0.16314E+01_JPRB, 0.15900E+01_JPRB, 0.15602E+01_JPRB, 0.15011E+01_JPRB ,&
&  0.14445E+01_JPRB, 0.14657E+01_JPRB, 0.14770E+01_JPRB, 0.14370E+01_JPRB, 0.14183E+01_JPRB, 0.16640E+01_JPRB, 0.19213E+01_JPRB ,&
&  0.22634E+01_JPRB, 0.26990E+01_JPRB, 0.36184E+01_JPRB, 0.28294E+01_JPRB, 0.28893E+01_JPRB, 0.30879E+01_JPRB, 0.30600E+01_JPRB ,&
&  0.28691E+01_JPRB, 0.29633E+01_JPRB, 0.29972E+01_JPRB, 0.31326E+01_JPRB, 0.33260E+01_JPRB, 0.33339E+01_JPRB, 0.40095E+01_JPRB ,&
&  0.37982E+01_JPRB, 0.43597E+01_JPRB, 0.40006E+01_JPRB, 0.37606E+01_JPRB, 0.31765E+01_JPRB, 0.30848E+01_JPRB, 0.29949E+01_JPRB ,&
&  0.31339E+01_JPRB, 0.30517E+01_JPRB, 0.28345E+01_JPRB, 0.25765E+01_JPRB, 0.20449E+01_JPRB, 0.18444E+01_JPRB, 0.16423E+01_JPRB ,&
&  0.14545E+01_JPRB, 0.13174E+01_JPRB, 0.12467E+01_JPRB, 0.11868E+01_JPRB, 0.11568E+01_JPRB, 0.11334E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.10682E+01_JPRB, 0.10500E+01_JPRB, 0.10170E+01_JPRB, 0.99110E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.97057E+00_JPRB, 0.95371E+00_JPRB, 0.93724E+00_JPRB, 0.88811E+00_JPRB, 0.86203E+00_JPRB, 0.86357E+00_JPRB, 0.84574E+00_JPRB ,&
&  0.82075E+00_JPRB, 0.78942E+00_JPRB, 0.79756E+00_JPRB, 0.83207E+00_JPRB, 0.88201E+00_JPRB, 0.96612E+00_JPRB, 0.10771E+01_JPRB ,&
&  0.12225E+01_JPRB, 0.13688E+01_JPRB, 0.15987E+01_JPRB, 0.18351E+01_JPRB, 0.20249E+01_JPRB, 0.21683E+01_JPRB, 0.22189E+01_JPRB ,&
&  0.17873E+01_JPRB, 0.13186E+01_JPRB, 0.15825E+01_JPRB, 0.17245E+01_JPRB, 0.17057E+01_JPRB, 0.18073E+01_JPRB, 0.17861E+01_JPRB ,&
&  0.15654E+01_JPRB, 0.15123E+01_JPRB, 0.15507E+01_JPRB, 0.14589E+01_JPRB, 0.12647E+01_JPRB, 0.12929E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.14414E+01_JPRB, 0.15150E+01_JPRB, 0.16057E+01_JPRB, 0.16976E+01_JPRB, 0.17967E+01_JPRB, 0.19163E+01_JPRB, 0.19916E+01_JPRB ,&
&  0.19972E+01_JPRB, 0.19780E+01_JPRB, 0.19262E+01_JPRB, 0.17756E+01_JPRB, 0.16772E+01_JPRB, 0.16280E+01_JPRB, 0.15678E+01_JPRB ,&
&  0.15530E+01_JPRB, 0.15408E+01_JPRB, 0.16118E+01_JPRB, 0.17850E+01_JPRB, 0.20647E+01_JPRB, 0.20712E+01_JPRB, 0.23757E+01_JPRB ,&
&  0.26070E+01_JPRB, 0.31179E+01_JPRB, 0.32738E+01_JPRB, 0.37079E+01_JPRB, 0.23147E+01_JPRB, 0.26837E+01_JPRB, 0.25179E+01_JPRB ,&
&  0.30091E+01_JPRB, 0.29077E+01_JPRB, 0.30360E+01_JPRB, 0.32776E+01_JPRB, 0.37145E+01_JPRB, 0.37254E+01_JPRB, 0.39314E+01_JPRB ,&
&  0.38461E+01_JPRB, 0.37070E+01_JPRB, 0.33345E+01_JPRB, 0.32611E+01_JPRB, 0.30681E+01_JPRB, 0.32042E+01_JPRB, 0.27475E+01_JPRB ,&
&  0.27229E+01_JPRB, 0.25747E+01_JPRB, 0.24051E+01_JPRB, 0.22358E+01_JPRB, 0.17874E+01_JPRB, 0.14917E+01_JPRB, 0.13244E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.12084E+01_JPRB, 0.11684E+01_JPRB, 0.11420E+01_JPRB, 0.11332E+01_JPRB, 0.11243E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.10685E+01_JPRB, 0.10477E+01_JPRB, 0.10185E+01_JPRB, 0.99447E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.94718E+00_JPRB, 0.92324E+00_JPRB, 0.89742E+00_JPRB, 0.86373E+00_JPRB, 0.84423E+00_JPRB, 0.82449E+00_JPRB, 0.81892E+00_JPRB ,&
&  0.81634E+00_JPRB, 0.83081E+00_JPRB, 0.84334E+00_JPRB, 0.87360E+00_JPRB, 0.91489E+00_JPRB, 0.98821E+00_JPRB, 0.10593E+01_JPRB ,&
&  0.11444E+01_JPRB, 0.12631E+01_JPRB, 0.14862E+01_JPRB, 0.17613E+01_JPRB, 0.19752E+01_JPRB, 0.20843E+01_JPRB, 0.19647E+01_JPRB ,&
&  0.18439E+01_JPRB, 0.18033E+01_JPRB, 0.17076E+01_JPRB, 0.14285E+01_JPRB, 0.13982E+01_JPRB, 0.17466E+01_JPRB, 0.16499E+01_JPRB ,&
&  0.15208E+01_JPRB, 0.14941E+01_JPRB, 0.14640E+01_JPRB, 0.14305E+01_JPRB, 0.14016E+01_JPRB, 0.14382E+01_JPRB, 0.15015E+01_JPRB ,&
&  0.15769E+01_JPRB, 0.16507E+01_JPRB, 0.17577E+01_JPRB, 0.18643E+01_JPRB, 0.19764E+01_JPRB, 0.21330E+01_JPRB, 0.22101E+01_JPRB ,&
&  0.21935E+01_JPRB, 0.21733E+01_JPRB, 0.21118E+01_JPRB, 0.19677E+01_JPRB, 0.18074E+01_JPRB, 0.17806E+01_JPRB, 0.18001E+01_JPRB ,&
&  0.18416E+01_JPRB, 0.19126E+01_JPRB, 0.20080E+01_JPRB, 0.22770E+01_JPRB, 0.25242E+01_JPRB, 0.23965E+01_JPRB, 0.24377E+01_JPRB ,&
&  0.28213E+01_JPRB, 0.32719E+01_JPRB, 0.29071E+01_JPRB, 0.34086E+01_JPRB, 0.19589E+01_JPRB, 0.24200E+01_JPRB, 0.28416E+01_JPRB ,&
&  0.27437E+01_JPRB, 0.28306E+01_JPRB, 0.31023E+01_JPRB, 0.34966E+01_JPRB, 0.39351E+01_JPRB, 0.36941E+01_JPRB, 0.36479E+01_JPRB ,&
&  0.33547E+01_JPRB, 0.29149E+01_JPRB, 0.27827E+01_JPRB, 0.27150E+01_JPRB, 0.28171E+01_JPRB, 0.28369E+01_JPRB, 0.22523E+01_JPRB ,&
&  0.19517E+01_JPRB, 0.21849E+01_JPRB, 0.22253E+01_JPRB, 0.21082E+01_JPRB, 0.15864E+01_JPRB, 0.12313E+01_JPRB, 0.11740E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.11335E+01_JPRB, 0.11177E+01_JPRB, 0.11142E+01_JPRB, 0.11186E+01_JPRB, 0.11143E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10690E+01_JPRB, 0.10438E+01_JPRB, 0.10124E+01_JPRB, 0.97875E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.91087E+00_JPRB, 0.89036E+00_JPRB, 0.86463E+00_JPRB, 0.83736E+00_JPRB, 0.79069E+00_JPRB, 0.76454E+00_JPRB, 0.75957E+00_JPRB ,&
&  0.77504E+00_JPRB, 0.79992E+00_JPRB, 0.81824E+00_JPRB, 0.82591E+00_JPRB, 0.85434E+00_JPRB, 0.87210E+00_JPRB, 0.90294E+00_JPRB ,&
&  0.96573E+00_JPRB, 0.10697E+01_JPRB, 0.12667E+01_JPRB, 0.15811E+01_JPRB, 0.18809E+01_JPRB, 0.21566E+01_JPRB, 0.20317E+01_JPRB ,&
&  0.19001E+01_JPRB, 0.18219E+01_JPRB, 0.18047E+01_JPRB, 0.18124E+01_JPRB, 0.16223E+01_JPRB, 0.17865E+01_JPRB, 0.18499E+01_JPRB ,&
&  0.18694E+01_JPRB, 0.17585E+01_JPRB, 0.15902E+01_JPRB, 0.15561E+01_JPRB, 0.15407E+01_JPRB, 0.15692E+01_JPRB, 0.16285E+01_JPRB ,&
&  0.17025E+01_JPRB, 0.17808E+01_JPRB, 0.19067E+01_JPRB, 0.20624E+01_JPRB, 0.22485E+01_JPRB, 0.24186E+01_JPRB, 0.24837E+01_JPRB ,&
&  0.24732E+01_JPRB, 0.24574E+01_JPRB, 0.24109E+01_JPRB, 0.22552E+01_JPRB, 0.21811E+01_JPRB, 0.22155E+01_JPRB, 0.23058E+01_JPRB ,&
&  0.24262E+01_JPRB, 0.23995E+01_JPRB, 0.25313E+01_JPRB, 0.28323E+01_JPRB, 0.29156E+01_JPRB, 0.28367E+01_JPRB, 0.28237E+01_JPRB ,&
&  0.30606E+01_JPRB, 0.33871E+01_JPRB, 0.22920E+01_JPRB, 0.27491E+01_JPRB, 0.33055E+01_JPRB, 0.28070E+01_JPRB, 0.26539E+01_JPRB ,&
&  0.27108E+01_JPRB, 0.28655E+01_JPRB, 0.31673E+01_JPRB, 0.35317E+01_JPRB, 0.39418E+01_JPRB, 0.42400E+01_JPRB, 0.30665E+01_JPRB ,&
&  0.27607E+01_JPRB, 0.23609E+01_JPRB, 0.22833E+01_JPRB, 0.23121E+01_JPRB, 0.23899E+01_JPRB, 0.22941E+01_JPRB, 0.18242E+01_JPRB ,&
&  0.16933E+01_JPRB, 0.18644E+01_JPRB, 0.18401E+01_JPRB, 0.18747E+01_JPRB, 0.17373E+01_JPRB, 0.11067E+01_JPRB, 0.11258E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.10949E+01_JPRB, 0.10889E+01_JPRB, 0.10893E+01_JPRB, 0.10938E+01_JPRB, 0.10931E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.10399E+01_JPRB, 0.10070E+01_JPRB, 0.97274E+00_JPRB, 0.93942E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.88381E+00_JPRB, 0.86112E+00_JPRB, 0.83217E+00_JPRB, 0.78819E+00_JPRB, 0.74209E+00_JPRB, 0.72256E+00_JPRB, 0.71328E+00_JPRB ,&
&  0.71785E+00_JPRB, 0.74074E+00_JPRB, 0.75480E+00_JPRB, 0.75242E+00_JPRB, 0.74855E+00_JPRB, 0.76386E+00_JPRB, 0.81149E+00_JPRB ,&
&  0.87171E+00_JPRB, 0.94645E+00_JPRB, 0.10923E+01_JPRB, 0.14005E+01_JPRB, 0.17951E+01_JPRB, 0.21156E+01_JPRB, 0.20858E+01_JPRB ,&
&  0.19600E+01_JPRB, 0.19703E+01_JPRB, 0.19940E+01_JPRB, 0.20532E+01_JPRB, 0.20989E+01_JPRB, 0.22007E+01_JPRB, 0.23024E+01_JPRB ,&
&  0.20681E+01_JPRB, 0.17059E+01_JPRB, 0.16194E+01_JPRB, 0.15942E+01_JPRB, 0.16272E+01_JPRB, 0.16794E+01_JPRB, 0.17445E+01_JPRB ,&
&  0.18255E+01_JPRB, 0.19430E+01_JPRB, 0.21078E+01_JPRB, 0.23573E+01_JPRB, 0.26189E+01_JPRB, 0.27745E+01_JPRB, 0.28537E+01_JPRB ,&
&  0.29020E+01_JPRB, 0.29320E+01_JPRB, 0.30062E+01_JPRB, 0.27133E+01_JPRB, 0.28129E+01_JPRB, 0.28986E+01_JPRB, 0.29849E+01_JPRB ,&
&  0.29624E+01_JPRB, 0.28765E+01_JPRB, 0.31038E+01_JPRB, 0.32812E+01_JPRB, 0.34562E+01_JPRB, 0.34436E+01_JPRB, 0.33428E+01_JPRB ,&
&  0.34785E+01_JPRB, 0.31640E+01_JPRB, 0.20574E+01_JPRB, 0.20189E+01_JPRB, 0.22014E+01_JPRB, 0.20441E+01_JPRB, 0.25329E+01_JPRB ,&
&  0.25463E+01_JPRB, 0.26694E+01_JPRB, 0.28317E+01_JPRB, 0.30141E+01_JPRB, 0.32042E+01_JPRB, 0.32870E+01_JPRB, 0.25919E+01_JPRB ,&
&  0.21709E+01_JPRB, 0.19459E+01_JPRB, 0.18988E+01_JPRB, 0.18539E+01_JPRB, 0.18143E+01_JPRB, 0.16963E+01_JPRB, 0.16200E+01_JPRB ,&
&  0.16003E+01_JPRB, 0.15686E+01_JPRB, 0.14890E+01_JPRB, 0.13708E+01_JPRB, 0.12100E+01_JPRB, 0.10760E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.10615E+01_JPRB, 0.10587E+01_JPRB, 0.10540E+01_JPRB, 0.10541E+01_JPRB, 0.10486E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10117E+01_JPRB, 0.98588E+00_JPRB, 0.95972E+00_JPRB, 0.92970E+00_JPRB, 0.90222E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.86248E+00_JPRB, 0.83522E+00_JPRB, 0.79743E+00_JPRB, 0.75829E+00_JPRB, 0.75458E+00_JPRB, 0.75901E+00_JPRB, 0.74368E+00_JPRB ,&
&  0.73551E+00_JPRB, 0.73011E+00_JPRB, 0.74795E+00_JPRB, 0.75199E+00_JPRB, 0.75743E+00_JPRB, 0.77486E+00_JPRB, 0.79618E+00_JPRB ,&
&  0.81527E+00_JPRB, 0.83030E+00_JPRB, 0.90302E+00_JPRB, 0.10594E+01_JPRB, 0.13575E+01_JPRB, 0.17561E+01_JPRB, 0.20522E+01_JPRB ,&
&  0.22319E+01_JPRB, 0.23911E+01_JPRB, 0.25416E+01_JPRB, 0.26728E+01_JPRB, 0.25503E+01_JPRB, 0.26497E+01_JPRB, 0.29167E+01_JPRB ,&
&  0.21815E+01_JPRB, 0.15894E+01_JPRB, 0.17001E+01_JPRB, 0.15807E+01_JPRB, 0.16353E+01_JPRB, 0.17505E+01_JPRB, 0.18496E+01_JPRB ,&
&  0.19820E+01_JPRB, 0.21353E+01_JPRB, 0.23214E+01_JPRB, 0.25749E+01_JPRB, 0.28414E+01_JPRB, 0.30313E+01_JPRB, 0.31820E+01_JPRB ,&
&  0.33074E+01_JPRB, 0.34377E+01_JPRB, 0.36240E+01_JPRB, 0.36073E+01_JPRB, 0.35311E+01_JPRB, 0.36580E+01_JPRB, 0.37227E+01_JPRB ,&
&  0.38156E+01_JPRB, 0.38222E+01_JPRB, 0.33427E+01_JPRB, 0.34830E+01_JPRB, 0.39378E+01_JPRB, 0.37279E+01_JPRB, 0.36717E+01_JPRB ,&
&  0.39302E+01_JPRB, 0.32580E+01_JPRB, 0.18804E+01_JPRB, 0.17929E+01_JPRB, 0.19656E+01_JPRB, 0.21608E+01_JPRB, 0.23561E+01_JPRB ,&
&  0.24469E+01_JPRB, 0.25206E+01_JPRB, 0.25179E+01_JPRB, 0.25254E+01_JPRB, 0.24080E+01_JPRB, 0.22505E+01_JPRB, 0.20780E+01_JPRB ,&
&  0.17955E+01_JPRB, 0.17112E+01_JPRB, 0.16389E+01_JPRB, 0.15949E+01_JPRB, 0.15963E+01_JPRB, 0.15313E+01_JPRB, 0.14668E+01_JPRB ,&
&  0.15813E+01_JPRB, 0.16417E+01_JPRB, 0.18020E+01_JPRB, 0.14628E+01_JPRB, 0.11430E+01_JPRB, 0.10782E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10761E+01_JPRB, 0.10632E+01_JPRB, 0.10494E+01_JPRB, 0.10391E+01_JPRB, 0.10172E+01_JPRB, 0.98546E+00_JPRB ,&
&  0.95307E+00_JPRB, 0.92930E+00_JPRB, 0.91501E+00_JPRB, 0.89784E+00_JPRB, 0.87516E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.82430E+00_JPRB, 0.77862E+00_JPRB, 0.76059E+00_JPRB, 0.80126E+00_JPRB, 0.77720E+00_JPRB, 0.80256E+00_JPRB, 0.79611E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.79973E+00_JPRB, 0.81936E+00_JPRB, 0.84561E+00_JPRB, 0.87684E+00_JPRB, 0.90208E+00_JPRB, 0.91319E+00_JPRB ,&
&  0.92556E+00_JPRB, 0.93690E+00_JPRB, 0.95540E+00_JPRB, 0.99175E+00_JPRB, 0.10630E+01_JPRB, 0.11860E+01_JPRB, 0.14277E+01_JPRB ,&
&  0.16802E+01_JPRB, 0.18240E+01_JPRB, 0.18877E+01_JPRB, 0.19072E+01_JPRB, 0.21360E+01_JPRB, 0.26703E+01_JPRB, 0.30864E+01_JPRB ,&
&  0.18899E+01_JPRB, 0.16935E+01_JPRB, 0.15385E+01_JPRB, 0.14102E+01_JPRB, 0.14932E+01_JPRB, 0.17216E+01_JPRB, 0.18575E+01_JPRB ,&
&  0.19498E+01_JPRB, 0.20959E+01_JPRB, 0.22736E+01_JPRB, 0.24930E+01_JPRB, 0.26881E+01_JPRB, 0.28867E+01_JPRB, 0.30927E+01_JPRB ,&
&  0.33121E+01_JPRB, 0.35900E+01_JPRB, 0.38784E+01_JPRB, 0.41785E+01_JPRB, 0.41606E+01_JPRB, 0.46658E+01_JPRB, 0.47779E+01_JPRB ,&
&  0.48548E+01_JPRB, 0.46324E+01_JPRB, 0.40039E+01_JPRB, 0.40913E+01_JPRB, 0.43912E+01_JPRB, 0.43437E+01_JPRB, 0.40188E+01_JPRB ,&
&  0.40064E+01_JPRB, 0.30146E+01_JPRB, 0.20151E+01_JPRB, 0.19854E+01_JPRB, 0.20065E+01_JPRB, 0.20951E+01_JPRB, 0.20357E+01_JPRB ,&
&  0.21311E+01_JPRB, 0.21230E+01_JPRB, 0.20632E+01_JPRB, 0.19569E+01_JPRB, 0.18368E+01_JPRB, 0.17896E+01_JPRB, 0.17692E+01_JPRB ,&
&  0.16197E+01_JPRB, 0.15714E+01_JPRB, 0.15209E+01_JPRB, 0.15398E+01_JPRB, 0.16080E+01_JPRB, 0.13706E+01_JPRB, 0.14086E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.15790E+01_JPRB, 0.14017E+01_JPRB, 0.15829E+01_JPRB, 0.11095E+01_JPRB, 0.10174E+01_JPRB, 0.10331E+01_JPRB ,&
&  0.10673E+01_JPRB, 0.10351E+01_JPRB, 0.10254E+01_JPRB, 0.10131E+01_JPRB, 0.10096E+01_JPRB, 0.99026E+00_JPRB, 0.94043E+00_JPRB ,&
&  0.90178E+00_JPRB, 0.87934E+00_JPRB, 0.87312E+00_JPRB, 0.86350E+00_JPRB, 0.85439E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.77199E+00_JPRB, 0.75244E+00_JPRB, 0.81104E+00_JPRB, 0.77954E+00_JPRB, 0.78170E+00_JPRB, 0.80217E+00_JPRB, 0.82376E+00_JPRB ,&
&  0.83380E+00_JPRB, 0.85310E+00_JPRB, 0.88612E+00_JPRB, 0.93491E+00_JPRB, 0.98098E+00_JPRB, 0.10163E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.11038E+01_JPRB, 0.11142E+01_JPRB, 0.11244E+01_JPRB, 0.11564E+01_JPRB, 0.11941E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.13381E+01_JPRB, 0.13659E+01_JPRB, 0.13755E+01_JPRB, 0.14248E+01_JPRB, 0.14839E+01_JPRB, 0.16494E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.15985E+01_JPRB, 0.16697E+01_JPRB, 0.15854E+01_JPRB, 0.15129E+01_JPRB, 0.15831E+01_JPRB, 0.15712E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.18179E+01_JPRB, 0.18708E+01_JPRB, 0.19749E+01_JPRB, 0.20733E+01_JPRB, 0.21993E+01_JPRB, 0.23643E+01_JPRB, 0.25309E+01_JPRB ,&
&  0.27401E+01_JPRB, 0.29896E+01_JPRB, 0.31838E+01_JPRB, 0.34919E+01_JPRB, 0.37951E+01_JPRB, 0.41208E+01_JPRB, 0.42952E+01_JPRB ,&
&  0.44044E+01_JPRB, 0.44793E+01_JPRB, 0.41404E+01_JPRB, 0.41776E+01_JPRB, 0.41918E+01_JPRB, 0.39864E+01_JPRB, 0.34928E+01_JPRB ,&
&  0.29849E+01_JPRB, 0.25159E+01_JPRB, 0.21950E+01_JPRB, 0.20238E+01_JPRB, 0.19580E+01_JPRB, 0.18612E+01_JPRB, 0.18167E+01_JPRB ,&
&  0.18208E+01_JPRB, 0.17358E+01_JPRB, 0.16332E+01_JPRB, 0.15093E+01_JPRB, 0.14537E+01_JPRB, 0.14505E+01_JPRB, 0.14649E+01_JPRB ,&
&  0.14213E+01_JPRB, 0.13872E+01_JPRB, 0.14164E+01_JPRB, 0.15475E+01_JPRB, 0.17416E+01_JPRB, 0.22569E+01_JPRB, 0.17580E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.12153E+01_JPRB, 0.88794E+00_JPRB, 0.11483E+01_JPRB, 0.95905E+00_JPRB, 0.92455E+00_JPRB, 0.96317E+00_JPRB ,&
&  0.98125E+00_JPRB, 0.97709E+00_JPRB, 0.10154E+01_JPRB, 0.97999E+00_JPRB, 0.94795E+00_JPRB, 0.93068E+00_JPRB, 0.92809E+00_JPRB ,&
&  0.87815E+00_JPRB, 0.82513E+00_JPRB, 0.77106E+00_JPRB, 0.77987E+00_JPRB, 0.73818E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.67438E+00_JPRB, 0.71369E+00_JPRB, 0.81488E+00_JPRB, 0.75618E+00_JPRB, 0.76015E+00_JPRB, 0.78328E+00_JPRB, 0.81627E+00_JPRB ,&
&  0.85133E+00_JPRB, 0.89456E+00_JPRB, 0.94040E+00_JPRB, 0.10019E+01_JPRB, 0.10565E+01_JPRB, 0.11034E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11879E+01_JPRB, 0.12228E+01_JPRB, 0.12564E+01_JPRB, 0.12754E+01_JPRB, 0.12887E+01_JPRB, 0.13211E+01_JPRB, 0.13619E+01_JPRB ,&
&  0.14427E+01_JPRB, 0.14861E+01_JPRB, 0.15388E+01_JPRB, 0.16207E+01_JPRB, 0.16860E+01_JPRB, 0.17707E+01_JPRB, 0.15998E+01_JPRB ,&
&  0.18309E+01_JPRB, 0.16560E+01_JPRB, 0.15986E+01_JPRB, 0.15911E+01_JPRB, 0.15643E+01_JPRB, 0.15453E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.16450E+01_JPRB, 0.16203E+01_JPRB, 0.16549E+01_JPRB, 0.16590E+01_JPRB, 0.17377E+01_JPRB, 0.18802E+01_JPRB, 0.19820E+01_JPRB ,&
&  0.21556E+01_JPRB, 0.23651E+01_JPRB, 0.25949E+01_JPRB, 0.27745E+01_JPRB, 0.29328E+01_JPRB, 0.29455E+01_JPRB, 0.29510E+01_JPRB ,&
&  0.30386E+01_JPRB, 0.32264E+01_JPRB, 0.31824E+01_JPRB, 0.32776E+01_JPRB, 0.32715E+01_JPRB, 0.30022E+01_JPRB, 0.23633E+01_JPRB ,&
&  0.20522E+01_JPRB, 0.21106E+01_JPRB, 0.18555E+01_JPRB, 0.21135E+01_JPRB, 0.18801E+01_JPRB, 0.16688E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.15453E+01_JPRB, 0.14245E+01_JPRB, 0.13251E+01_JPRB, 0.11858E+01_JPRB, 0.11684E+01_JPRB, 0.11775E+01_JPRB, 0.12022E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.11361E+01_JPRB, 0.11511E+01_JPRB, 0.13117E+01_JPRB, 0.18332E+01_JPRB, 0.19124E+01_JPRB, 0.21062E+01_JPRB ,&
&  0.12171E+01_JPRB, 0.75709E+00_JPRB, 0.83508E+00_JPRB, 0.85352E+00_JPRB, 0.80840E+00_JPRB, 0.78490E+00_JPRB, 0.84610E+00_JPRB ,&
&  0.92150E+00_JPRB, 0.92254E+00_JPRB, 0.89828E+00_JPRB, 0.90161E+00_JPRB, 0.89608E+00_JPRB, 0.89787E+00_JPRB, 0.81918E+00_JPRB ,&
&  0.79590E+00_JPRB, 0.73920E+00_JPRB, 0.70618E+00_JPRB, 0.69118E+00_JPRB, 0.65660E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.63937E+00_JPRB, 0.71873E+00_JPRB, 0.79525E+00_JPRB, 0.73730E+00_JPRB, 0.75536E+00_JPRB, 0.78687E+00_JPRB, 0.82960E+00_JPRB ,&
&  0.87715E+00_JPRB, 0.93118E+00_JPRB, 0.98689E+00_JPRB, 0.10515E+01_JPRB, 0.11034E+01_JPRB, 0.11517E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.12542E+01_JPRB, 0.12797E+01_JPRB, 0.13240E+01_JPRB, 0.13425E+01_JPRB, 0.13743E+01_JPRB, 0.13915E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.15022E+01_JPRB, 0.15609E+01_JPRB, 0.16289E+01_JPRB, 0.16932E+01_JPRB, 0.17252E+01_JPRB, 0.18599E+01_JPRB, 0.15387E+01_JPRB ,&
&  0.18253E+01_JPRB, 0.16593E+01_JPRB, 0.15772E+01_JPRB, 0.16039E+01_JPRB, 0.15834E+01_JPRB, 0.15900E+01_JPRB, 0.15930E+01_JPRB ,&
&  0.15947E+01_JPRB, 0.16144E+01_JPRB, 0.16415E+01_JPRB, 0.16303E+01_JPRB, 0.15514E+01_JPRB, 0.16397E+01_JPRB, 0.17366E+01_JPRB ,&
&  0.18543E+01_JPRB, 0.20004E+01_JPRB, 0.21147E+01_JPRB, 0.21856E+01_JPRB, 0.22213E+01_JPRB, 0.21749E+01_JPRB, 0.21267E+01_JPRB ,&
&  0.20696E+01_JPRB, 0.20989E+01_JPRB, 0.19959E+01_JPRB, 0.21696E+01_JPRB, 0.24160E+01_JPRB, 0.20477E+01_JPRB, 0.15575E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.16532E+01_JPRB, 0.18955E+01_JPRB, 0.20812E+01_JPRB, 0.17511E+01_JPRB, 0.15749E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.13263E+01_JPRB, 0.11928E+01_JPRB, 0.10859E+01_JPRB, 0.96769E+00_JPRB, 0.92275E+00_JPRB, 0.91536E+00_JPRB, 0.93070E+00_JPRB ,&
&  0.91278E+00_JPRB, 0.90174E+00_JPRB, 0.92420E+00_JPRB, 0.99573E+00_JPRB, 0.12296E+01_JPRB, 0.14069E+01_JPRB, 0.24650E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.90148E+00_JPRB, 0.13667E+01_JPRB, 0.54664E+00_JPRB, 0.63385E+00_JPRB, 0.67196E+00_JPRB, 0.71202E+00_JPRB ,&
&  0.66851E+00_JPRB, 0.84809E+00_JPRB, 0.11938E+01_JPRB, 0.12144E+01_JPRB, 0.12702E+01_JPRB, 0.13446E+01_JPRB, 0.82650E+00_JPRB ,&
&  0.68258E+00_JPRB, 0.67632E+00_JPRB, 0.68212E+00_JPRB, 0.63227E+00_JPRB, 0.62502E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.60574E+00_JPRB, 0.66928E+00_JPRB, 0.74664E+00_JPRB, 0.80469E+00_JPRB, 0.79714E+00_JPRB, 0.80730E+00_JPRB, 0.83471E+00_JPRB ,&
&  0.86744E+00_JPRB, 0.90696E+00_JPRB, 0.94628E+00_JPRB, 0.99664E+00_JPRB, 0.10320E+01_JPRB, 0.10766E+01_JPRB, 0.11405E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.12579E+01_JPRB, 0.12971E+01_JPRB, 0.13420E+01_JPRB, 0.13848E+01_JPRB, 0.14277E+01_JPRB, 0.14727E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.15524E+01_JPRB, 0.15966E+01_JPRB, 0.16972E+01_JPRB, 0.17808E+01_JPRB, 0.20383E+01_JPRB, 0.15305E+01_JPRB ,&
&  0.17636E+01_JPRB, 0.16312E+01_JPRB, 0.15462E+01_JPRB, 0.15823E+01_JPRB, 0.15811E+01_JPRB, 0.15678E+01_JPRB, 0.15659E+01_JPRB ,&
&  0.16340E+01_JPRB, 0.16732E+01_JPRB, 0.16951E+01_JPRB, 0.18348E+01_JPRB, 0.18047E+01_JPRB, 0.18123E+01_JPRB, 0.19921E+01_JPRB ,&
&  0.21658E+01_JPRB, 0.23191E+01_JPRB, 0.23802E+01_JPRB, 0.24704E+01_JPRB, 0.24751E+01_JPRB, 0.24243E+01_JPRB, 0.23580E+01_JPRB ,&
&  0.22744E+01_JPRB, 0.22607E+01_JPRB, 0.23420E+01_JPRB, 0.25919E+01_JPRB, 0.26905E+01_JPRB, 0.23865E+01_JPRB, 0.19439E+01_JPRB ,&
&  0.14128E+01_JPRB, 0.12329E+01_JPRB, 0.16853E+01_JPRB, 0.19092E+01_JPRB, 0.16298E+01_JPRB, 0.14844E+01_JPRB, 0.12845E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.10554E+01_JPRB, 0.96484E+00_JPRB, 0.86265E+00_JPRB, 0.81530E+00_JPRB, 0.80453E+00_JPRB, 0.84481E+00_JPRB ,&
&  0.86509E+00_JPRB, 0.84259E+00_JPRB, 0.84503E+00_JPRB, 0.87452E+00_JPRB, 0.10982E+01_JPRB, 0.14732E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.15393E+01_JPRB, 0.10469E+01_JPRB, 0.88251E+00_JPRB, 0.88594E+00_JPRB, 0.62410E+00_JPRB, 0.64986E+00_JPRB, 0.69709E+00_JPRB ,&
&  0.88287E+00_JPRB, 0.96205E+00_JPRB, 0.12932E+01_JPRB, 0.35043E+01_JPRB, 0.40039E+01_JPRB, 0.49964E+01_JPRB, 0.20203E+01_JPRB ,&
&  0.96640E+00_JPRB, 0.66006E+00_JPRB, 0.59593E+00_JPRB, 0.59838E+00_JPRB, 0.59018E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.57409E+00_JPRB, 0.62458E+00_JPRB, 0.66844E+00_JPRB, 0.73155E+00_JPRB, 0.75904E+00_JPRB, 0.77236E+00_JPRB, 0.77873E+00_JPRB ,&
&  0.77272E+00_JPRB, 0.78154E+00_JPRB, 0.80765E+00_JPRB, 0.87096E+00_JPRB, 0.93875E+00_JPRB, 0.10077E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.12713E+01_JPRB, 0.12809E+01_JPRB, 0.13127E+01_JPRB, 0.13716E+01_JPRB, 0.14269E+01_JPRB, 0.14587E+01_JPRB ,&
&  0.14855E+01_JPRB, 0.15458E+01_JPRB, 0.16174E+01_JPRB, 0.17188E+01_JPRB, 0.18451E+01_JPRB, 0.19752E+01_JPRB, 0.14699E+01_JPRB ,&
&  0.17211E+01_JPRB, 0.16019E+01_JPRB, 0.15861E+01_JPRB, 0.16126E+01_JPRB, 0.15537E+01_JPRB, 0.14779E+01_JPRB, 0.15000E+01_JPRB ,&
&  0.16137E+01_JPRB, 0.17650E+01_JPRB, 0.18886E+01_JPRB, 0.19755E+01_JPRB, 0.21766E+01_JPRB, 0.22020E+01_JPRB, 0.25364E+01_JPRB ,&
&  0.28226E+01_JPRB, 0.30076E+01_JPRB, 0.30970E+01_JPRB, 0.31824E+01_JPRB, 0.32874E+01_JPRB, 0.34307E+01_JPRB, 0.35971E+01_JPRB ,&
&  0.36984E+01_JPRB, 0.37543E+01_JPRB, 0.38604E+01_JPRB, 0.39728E+01_JPRB, 0.35647E+01_JPRB, 0.38079E+01_JPRB, 0.30523E+01_JPRB ,&
&  0.19988E+01_JPRB, 0.15747E+01_JPRB, 0.18163E+01_JPRB, 0.17894E+01_JPRB, 0.15727E+01_JPRB, 0.14030E+01_JPRB, 0.12403E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.97675E+00_JPRB, 0.89592E+00_JPRB, 0.81901E+00_JPRB, 0.75942E+00_JPRB, 0.74127E+00_JPRB, 0.74647E+00_JPRB ,&
&  0.75588E+00_JPRB, 0.75543E+00_JPRB, 0.76016E+00_JPRB, 0.81621E+00_JPRB, 0.98286E+00_JPRB, 0.13831E+01_JPRB, 0.19831E+01_JPRB ,&
&  0.23057E+01_JPRB, 0.11073E+01_JPRB, 0.85978E+00_JPRB, 0.77925E+00_JPRB, 0.75227E+00_JPRB, 0.77783E+00_JPRB, 0.82049E+00_JPRB ,&
&  0.96699E+00_JPRB, 0.12449E+01_JPRB, 0.11470E+01_JPRB, 0.13258E+01_JPRB, 0.25885E+01_JPRB, 0.27988E+01_JPRB, 0.16297E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.75061E+00_JPRB, 0.61515E+00_JPRB, 0.57439E+00_JPRB, 0.56503E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.57906E+00_JPRB, 0.59531E+00_JPRB, 0.61924E+00_JPRB, 0.64862E+00_JPRB, 0.64977E+00_JPRB, 0.65513E+00_JPRB, 0.64563E+00_JPRB ,&
&  0.64605E+00_JPRB, 0.66822E+00_JPRB, 0.72257E+00_JPRB, 0.81511E+00_JPRB, 0.88527E+00_JPRB, 0.98037E+00_JPRB, 0.11134E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.11875E+01_JPRB, 0.12006E+01_JPRB, 0.12303E+01_JPRB, 0.12700E+01_JPRB, 0.13207E+01_JPRB, 0.13693E+01_JPRB ,&
&  0.14264E+01_JPRB, 0.15386E+01_JPRB, 0.16427E+01_JPRB, 0.17771E+01_JPRB, 0.19089E+01_JPRB, 0.20050E+01_JPRB, 0.21956E+01_JPRB ,&
&  0.12657E+01_JPRB, 0.16846E+01_JPRB, 0.16742E+01_JPRB, 0.16386E+01_JPRB, 0.14233E+01_JPRB, 0.13117E+01_JPRB, 0.14017E+01_JPRB ,&
&  0.16935E+01_JPRB, 0.19789E+01_JPRB, 0.23299E+01_JPRB, 0.22184E+01_JPRB, 0.24866E+01_JPRB, 0.26824E+01_JPRB, 0.30817E+01_JPRB ,&
&  0.32511E+01_JPRB, 0.34131E+01_JPRB, 0.36154E+01_JPRB, 0.38440E+01_JPRB, 0.40894E+01_JPRB, 0.43796E+01_JPRB, 0.46484E+01_JPRB ,&
&  0.48611E+01_JPRB, 0.49695E+01_JPRB, 0.51983E+01_JPRB, 0.46807E+01_JPRB, 0.40835E+01_JPRB, 0.43251E+01_JPRB, 0.41399E+01_JPRB ,&
&  0.33756E+01_JPRB, 0.22871E+01_JPRB, 0.21117E+01_JPRB, 0.20101E+01_JPRB, 0.16963E+01_JPRB, 0.14990E+01_JPRB, 0.13132E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.91438E+00_JPRB, 0.83603E+00_JPRB, 0.77110E+00_JPRB, 0.73150E+00_JPRB, 0.73185E+00_JPRB, 0.75931E+00_JPRB ,&
&  0.77347E+00_JPRB, 0.77603E+00_JPRB, 0.76726E+00_JPRB, 0.78817E+00_JPRB, 0.86249E+00_JPRB, 0.93436E+00_JPRB, 0.99366E+00_JPRB ,&
&  0.95558E+00_JPRB, 0.91150E+00_JPRB, 0.88450E+00_JPRB, 0.88485E+00_JPRB, 0.88785E+00_JPRB, 0.87872E+00_JPRB, 0.90157E+00_JPRB ,&
&  0.90703E+00_JPRB, 0.89597E+00_JPRB, 0.98168E+00_JPRB, 0.11920E+01_JPRB, 0.12509E+01_JPRB, 0.13170E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.88416E+00_JPRB, 0.75452E+00_JPRB, 0.66870E+00_JPRB, 0.61291E+00_JPRB, 0.56797E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.57264E+00_JPRB, 0.57143E+00_JPRB, 0.57064E+00_JPRB, 0.58597E+00_JPRB, 0.58842E+00_JPRB, 0.59264E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.61947E+00_JPRB, 0.67539E+00_JPRB, 0.76130E+00_JPRB, 0.86070E+00_JPRB, 0.93686E+00_JPRB, 0.10286E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.10998E+01_JPRB, 0.11115E+01_JPRB, 0.11512E+01_JPRB, 0.11882E+01_JPRB, 0.12432E+01_JPRB, 0.13047E+01_JPRB ,&
&  0.13728E+01_JPRB, 0.14673E+01_JPRB, 0.15939E+01_JPRB, 0.17469E+01_JPRB, 0.19186E+01_JPRB, 0.20645E+01_JPRB, 0.22231E+01_JPRB ,&
&  0.88835E+00_JPRB, 0.11927E+01_JPRB, 0.16403E+01_JPRB, 0.17181E+01_JPRB, 0.13331E+01_JPRB, 0.12006E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.14208E+01_JPRB, 0.18403E+01_JPRB, 0.21461E+01_JPRB, 0.25302E+01_JPRB, 0.26438E+01_JPRB, 0.29833E+01_JPRB, 0.32365E+01_JPRB ,&
&  0.33318E+01_JPRB, 0.34635E+01_JPRB, 0.36956E+01_JPRB, 0.39825E+01_JPRB, 0.42664E+01_JPRB, 0.45244E+01_JPRB, 0.48010E+01_JPRB ,&
&  0.50974E+01_JPRB, 0.53134E+01_JPRB, 0.55537E+01_JPRB, 0.47133E+01_JPRB, 0.49185E+01_JPRB, 0.52119E+01_JPRB, 0.58335E+01_JPRB ,&
&  0.47235E+01_JPRB, 0.28496E+01_JPRB, 0.21473E+01_JPRB, 0.21360E+01_JPRB, 0.18257E+01_JPRB, 0.16347E+01_JPRB, 0.13778E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.94998E+00_JPRB, 0.86758E+00_JPRB, 0.84265E+00_JPRB, 0.83081E+00_JPRB, 0.84014E+00_JPRB, 0.85278E+00_JPRB ,&
&  0.85621E+00_JPRB, 0.86437E+00_JPRB, 0.85499E+00_JPRB, 0.85307E+00_JPRB, 0.91012E+00_JPRB, 0.96442E+00_JPRB, 0.99425E+00_JPRB ,&
&  0.99402E+00_JPRB, 0.98703E+00_JPRB, 0.99176E+00_JPRB, 0.10076E+01_JPRB, 0.10319E+01_JPRB, 0.10349E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.92686E+00_JPRB, 0.87294E+00_JPRB, 0.85612E+00_JPRB, 0.83619E+00_JPRB, 0.82524E+00_JPRB, 0.85203E+00_JPRB, 0.80202E+00_JPRB ,&
&  0.77190E+00_JPRB, 0.76143E+00_JPRB, 0.73134E+00_JPRB, 0.66250E+00_JPRB, 0.58984E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.59731E+00_JPRB, 0.59231E+00_JPRB, 0.58284E+00_JPRB, 0.58698E+00_JPRB, 0.58016E+00_JPRB, 0.58949E+00_JPRB, 0.60580E+00_JPRB ,&
&  0.63942E+00_JPRB, 0.70171E+00_JPRB, 0.78165E+00_JPRB, 0.85389E+00_JPRB, 0.93667E+00_JPRB, 0.10045E+01_JPRB, 0.10230E+01_JPRB ,&
&  0.10194E+01_JPRB, 0.10301E+01_JPRB, 0.10382E+01_JPRB, 0.10792E+01_JPRB, 0.11200E+01_JPRB, 0.11771E+01_JPRB, 0.12506E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.14577E+01_JPRB, 0.16060E+01_JPRB, 0.17848E+01_JPRB, 0.20251E+01_JPRB, 0.23089E+01_JPRB, 0.26295E+01_JPRB ,&
&  0.18972E+01_JPRB, 0.10139E+01_JPRB, 0.17373E+01_JPRB, 0.18926E+01_JPRB, 0.14583E+01_JPRB, 0.13030E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.16321E+01_JPRB, 0.19975E+01_JPRB, 0.23752E+01_JPRB, 0.25755E+01_JPRB, 0.28861E+01_JPRB, 0.31143E+01_JPRB ,&
&  0.32369E+01_JPRB, 0.33732E+01_JPRB, 0.35364E+01_JPRB, 0.37235E+01_JPRB, 0.39547E+01_JPRB, 0.41917E+01_JPRB, 0.44817E+01_JPRB ,&
&  0.48284E+01_JPRB, 0.51109E+01_JPRB, 0.53207E+01_JPRB, 0.45997E+01_JPRB, 0.49624E+01_JPRB, 0.47359E+01_JPRB, 0.44839E+01_JPRB ,&
&  0.38793E+01_JPRB, 0.23774E+01_JPRB, 0.17547E+01_JPRB, 0.20518E+01_JPRB, 0.18583E+01_JPRB, 0.13963E+01_JPRB, 0.14330E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.10384E+01_JPRB, 0.99829E+00_JPRB, 0.10061E+01_JPRB, 0.96068E+00_JPRB, 0.93974E+00_JPRB, 0.92021E+00_JPRB ,&
&  0.95489E+00_JPRB, 0.95511E+00_JPRB, 0.94006E+00_JPRB, 0.95285E+00_JPRB, 0.98740E+00_JPRB, 0.10361E+01_JPRB, 0.10694E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.11116E+01_JPRB, 0.11186E+01_JPRB, 0.11412E+01_JPRB, 0.11562E+01_JPRB, 0.10849E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.97501E+00_JPRB, 0.95201E+00_JPRB, 0.92986E+00_JPRB, 0.90890E+00_JPRB, 0.86770E+00_JPRB, 0.86672E+00_JPRB, 0.89158E+00_JPRB ,&
&  0.92090E+00_JPRB, 0.92416E+00_JPRB, 0.84987E+00_JPRB, 0.71298E+00_JPRB, 0.62594E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.60912E+00_JPRB, 0.58769E+00_JPRB, 0.57924E+00_JPRB, 0.59734E+00_JPRB, 0.59452E+00_JPRB, 0.60139E+00_JPRB, 0.62239E+00_JPRB ,&
&  0.65640E+00_JPRB, 0.70412E+00_JPRB, 0.74471E+00_JPRB, 0.79919E+00_JPRB, 0.86934E+00_JPRB, 0.93705E+00_JPRB, 0.95656E+00_JPRB ,&
&  0.95278E+00_JPRB, 0.95689E+00_JPRB, 0.97354E+00_JPRB, 0.99817E+00_JPRB, 0.10456E+01_JPRB, 0.10998E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.12503E+01_JPRB, 0.13758E+01_JPRB, 0.15131E+01_JPRB, 0.16672E+01_JPRB, 0.18743E+01_JPRB, 0.22086E+01_JPRB, 0.27101E+01_JPRB ,&
&  0.40822E+01_JPRB, 0.42532E+01_JPRB, 0.15204E+01_JPRB, 0.19316E+01_JPRB, 0.17304E+01_JPRB, 0.14960E+01_JPRB, 0.11955E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.14226E+01_JPRB, 0.18037E+01_JPRB, 0.22212E+01_JPRB, 0.22758E+01_JPRB, 0.25251E+01_JPRB, 0.27383E+01_JPRB ,&
&  0.29845E+01_JPRB, 0.31530E+01_JPRB, 0.32427E+01_JPRB, 0.32896E+01_JPRB, 0.33985E+01_JPRB, 0.35665E+01_JPRB, 0.38629E+01_JPRB ,&
&  0.43173E+01_JPRB, 0.48273E+01_JPRB, 0.52288E+01_JPRB, 0.48742E+01_JPRB, 0.51228E+01_JPRB, 0.44546E+01_JPRB, 0.39006E+01_JPRB ,&
&  0.29394E+01_JPRB, 0.24158E+01_JPRB, 0.21748E+01_JPRB, 0.21252E+01_JPRB, 0.18078E+01_JPRB, 0.15810E+01_JPRB, 0.16492E+01_JPRB ,&
&  0.15017E+01_JPRB, 0.13064E+01_JPRB, 0.11787E+01_JPRB, 0.11030E+01_JPRB, 0.10680E+01_JPRB, 0.10305E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10586E+01_JPRB, 0.10357E+01_JPRB, 0.10631E+01_JPRB, 0.10907E+01_JPRB, 0.11273E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11820E+01_JPRB, 0.12093E+01_JPRB, 0.12204E+01_JPRB, 0.12175E+01_JPRB, 0.11413E+01_JPRB, 0.10663E+01_JPRB, 0.10675E+01_JPRB ,&
&  0.10170E+01_JPRB, 0.10249E+01_JPRB, 0.10404E+01_JPRB, 0.10510E+01_JPRB, 0.10227E+01_JPRB, 0.10301E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.10785E+01_JPRB, 0.97512E+00_JPRB, 0.75171E+00_JPRB, 0.65270E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.66029E+00_JPRB, 0.63728E+00_JPRB, 0.63294E+00_JPRB, 0.64211E+00_JPRB, 0.64531E+00_JPRB, 0.63032E+00_JPRB, 0.63125E+00_JPRB ,&
&  0.64541E+00_JPRB, 0.66721E+00_JPRB, 0.66921E+00_JPRB, 0.70529E+00_JPRB, 0.78716E+00_JPRB, 0.87021E+00_JPRB, 0.88576E+00_JPRB ,&
&  0.88182E+00_JPRB, 0.87417E+00_JPRB, 0.87258E+00_JPRB, 0.89432E+00_JPRB, 0.92296E+00_JPRB, 0.98775E+00_JPRB, 0.10585E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.12207E+01_JPRB, 0.12919E+01_JPRB, 0.13495E+01_JPRB, 0.14687E+01_JPRB, 0.16879E+01_JPRB, 0.22750E+01_JPRB ,&
&  0.33624E+01_JPRB, 0.64590E+01_JPRB, 0.18267E+01_JPRB, 0.18966E+01_JPRB, 0.18824E+01_JPRB, 0.16985E+01_JPRB, 0.14970E+01_JPRB ,&
&  0.14591E+01_JPRB, 0.13755E+01_JPRB, 0.16543E+01_JPRB, 0.20299E+01_JPRB, 0.19751E+01_JPRB, 0.20558E+01_JPRB, 0.21858E+01_JPRB ,&
&  0.24994E+01_JPRB, 0.27318E+01_JPRB, 0.28831E+01_JPRB, 0.28773E+01_JPRB, 0.28483E+01_JPRB, 0.28215E+01_JPRB, 0.28986E+01_JPRB ,&
&  0.32701E+01_JPRB, 0.39841E+01_JPRB, 0.47619E+01_JPRB, 0.49558E+01_JPRB, 0.53740E+01_JPRB, 0.46873E+01_JPRB, 0.42197E+01_JPRB ,&
&  0.34079E+01_JPRB, 0.30088E+01_JPRB, 0.28912E+01_JPRB, 0.25864E+01_JPRB, 0.20937E+01_JPRB, 0.18648E+01_JPRB, 0.18512E+01_JPRB ,&
&  0.17385E+01_JPRB, 0.15440E+01_JPRB, 0.13404E+01_JPRB, 0.11896E+01_JPRB, 0.11543E+01_JPRB, 0.11720E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.12106E+01_JPRB, 0.11844E+01_JPRB, 0.12308E+01_JPRB, 0.12482E+01_JPRB, 0.12500E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.12720E+01_JPRB, 0.12767E+01_JPRB, 0.12204E+01_JPRB, 0.11104E+01_JPRB, 0.10409E+01_JPRB, 0.10398E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10775E+01_JPRB, 0.10956E+01_JPRB, 0.10690E+01_JPRB, 0.11100E+01_JPRB, 0.10999E+01_JPRB, 0.10923E+01_JPRB ,&
&  0.11068E+01_JPRB, 0.11560E+01_JPRB, 0.11782E+01_JPRB, 0.86103E+00_JPRB, 0.72920E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.75499E+00_JPRB, 0.72207E+00_JPRB, 0.69598E+00_JPRB, 0.67437E+00_JPRB, 0.65969E+00_JPRB, 0.66174E+00_JPRB, 0.66627E+00_JPRB ,&
&  0.66499E+00_JPRB, 0.67138E+00_JPRB, 0.66860E+00_JPRB, 0.67346E+00_JPRB, 0.69473E+00_JPRB, 0.74131E+00_JPRB, 0.79611E+00_JPRB ,&
&  0.80438E+00_JPRB, 0.77845E+00_JPRB, 0.77411E+00_JPRB, 0.77136E+00_JPRB, 0.79542E+00_JPRB, 0.83196E+00_JPRB, 0.90015E+00_JPRB ,&
&  0.99583E+00_JPRB, 0.10806E+01_JPRB, 0.11171E+01_JPRB, 0.11215E+01_JPRB, 0.11539E+01_JPRB, 0.12307E+01_JPRB, 0.17364E+01_JPRB ,&
&  0.28284E+01_JPRB, 0.60187E+01_JPRB, 0.40466E+01_JPRB, 0.29197E+01_JPRB, 0.20721E+01_JPRB, 0.17710E+01_JPRB, 0.17196E+01_JPRB ,&
&  0.19219E+01_JPRB, 0.18543E+01_JPRB, 0.18916E+01_JPRB, 0.18306E+01_JPRB, 0.17243E+01_JPRB, 0.16339E+01_JPRB, 0.17163E+01_JPRB ,&
&  0.19650E+01_JPRB, 0.22082E+01_JPRB, 0.23856E+01_JPRB, 0.24334E+01_JPRB, 0.23822E+01_JPRB, 0.23278E+01_JPRB, 0.22847E+01_JPRB ,&
&  0.22764E+01_JPRB, 0.25686E+01_JPRB, 0.36234E+01_JPRB, 0.48182E+01_JPRB, 0.55147E+01_JPRB, 0.50804E+01_JPRB, 0.47768E+01_JPRB ,&
&  0.45953E+01_JPRB, 0.39990E+01_JPRB, 0.36090E+01_JPRB, 0.30305E+01_JPRB, 0.23672E+01_JPRB, 0.23893E+01_JPRB, 0.20099E+01_JPRB ,&
&  0.18400E+01_JPRB, 0.16888E+01_JPRB, 0.15085E+01_JPRB, 0.13165E+01_JPRB, 0.12922E+01_JPRB, 0.12914E+01_JPRB, 0.12478E+01_JPRB ,&
&  0.12773E+01_JPRB, 0.13195E+01_JPRB, 0.12957E+01_JPRB, 0.13037E+01_JPRB, 0.13049E+01_JPRB, 0.13114E+01_JPRB, 0.13077E+01_JPRB ,&
&  0.13013E+01_JPRB, 0.13222E+01_JPRB, 0.12437E+01_JPRB, 0.11600E+01_JPRB, 0.11056E+01_JPRB, 0.10428E+01_JPRB, 0.10218E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.11388E+01_JPRB, 0.11466E+01_JPRB, 0.11517E+01_JPRB, 0.11320E+01_JPRB, 0.11340E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.10687E+01_JPRB, 0.10369E+01_JPRB, 0.10310E+01_JPRB, 0.94199E+00_JPRB, 0.82730E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.81915E+00_JPRB, 0.78712E+00_JPRB, 0.72190E+00_JPRB, 0.66830E+00_JPRB, 0.65547E+00_JPRB, 0.66188E+00_JPRB, 0.67701E+00_JPRB ,&
&  0.68673E+00_JPRB, 0.69213E+00_JPRB, 0.69099E+00_JPRB, 0.68284E+00_JPRB, 0.68473E+00_JPRB, 0.66208E+00_JPRB, 0.68295E+00_JPRB ,&
&  0.71742E+00_JPRB, 0.72726E+00_JPRB, 0.74379E+00_JPRB, 0.71979E+00_JPRB, 0.71889E+00_JPRB, 0.71545E+00_JPRB, 0.75012E+00_JPRB ,&
&  0.82918E+00_JPRB, 0.93549E+00_JPRB, 0.10001E+01_JPRB, 0.10155E+01_JPRB, 0.10300E+01_JPRB, 0.11025E+01_JPRB, 0.14298E+01_JPRB ,&
&  0.23439E+01_JPRB, 0.51462E+01_JPRB, 0.23620E+01_JPRB, 0.36134E+01_JPRB, 0.24779E+01_JPRB, 0.19120E+01_JPRB, 0.15511E+01_JPRB ,&
&  0.16602E+01_JPRB, 0.20151E+01_JPRB, 0.17214E+01_JPRB, 0.15841E+01_JPRB, 0.14976E+01_JPRB, 0.14529E+01_JPRB, 0.14475E+01_JPRB ,&
&  0.15287E+01_JPRB, 0.17340E+01_JPRB, 0.19388E+01_JPRB, 0.20320E+01_JPRB, 0.19828E+01_JPRB, 0.18678E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.18128E+01_JPRB, 0.18412E+01_JPRB, 0.22889E+01_JPRB, 0.30338E+01_JPRB, 0.47777E+01_JPRB, 0.53947E+01_JPRB, 0.55554E+01_JPRB ,&
&  0.56748E+01_JPRB, 0.51394E+01_JPRB, 0.42828E+01_JPRB, 0.34989E+01_JPRB, 0.28269E+01_JPRB, 0.26885E+01_JPRB, 0.22551E+01_JPRB ,&
&  0.19803E+01_JPRB, 0.17943E+01_JPRB, 0.16247E+01_JPRB, 0.14726E+01_JPRB, 0.13921E+01_JPRB, 0.13647E+01_JPRB, 0.13282E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.14386E+01_JPRB, 0.14172E+01_JPRB, 0.13640E+01_JPRB, 0.13369E+01_JPRB, 0.13306E+01_JPRB, 0.13182E+01_JPRB ,&
&  0.13185E+01_JPRB, 0.13446E+01_JPRB, 0.12647E+01_JPRB, 0.11459E+01_JPRB, 0.11085E+01_JPRB, 0.10921E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.12106E+01_JPRB, 0.12061E+01_JPRB, 0.11607E+01_JPRB, 0.11546E+01_JPRB, 0.11941E+01_JPRB, 0.11206E+01_JPRB ,&
&  0.10635E+01_JPRB, 0.10200E+01_JPRB, 0.98296E+00_JPRB, 0.94406E+00_JPRB, 0.88792E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.83088E+00_JPRB, 0.77164E+00_JPRB, 0.70515E+00_JPRB, 0.67131E+00_JPRB, 0.67150E+00_JPRB, 0.65951E+00_JPRB, 0.67157E+00_JPRB ,&
&  0.69879E+00_JPRB, 0.70407E+00_JPRB, 0.70507E+00_JPRB, 0.69350E+00_JPRB, 0.68208E+00_JPRB, 0.65698E+00_JPRB, 0.63394E+00_JPRB ,&
&  0.65713E+00_JPRB, 0.72129E+00_JPRB, 0.75101E+00_JPRB, 0.73850E+00_JPRB, 0.71985E+00_JPRB, 0.69764E+00_JPRB, 0.69390E+00_JPRB ,&
&  0.72412E+00_JPRB, 0.79263E+00_JPRB, 0.86817E+00_JPRB, 0.90773E+00_JPRB, 0.92819E+00_JPRB, 0.96125E+00_JPRB, 0.11665E+01_JPRB ,&
&  0.18215E+01_JPRB, 0.51810E+01_JPRB, 0.21473E+01_JPRB, 0.38589E+01_JPRB, 0.28111E+01_JPRB, 0.21811E+01_JPRB, 0.16308E+01_JPRB ,&
&  0.15517E+01_JPRB, 0.16949E+01_JPRB, 0.15552E+01_JPRB, 0.14656E+01_JPRB, 0.13805E+01_JPRB, 0.13492E+01_JPRB, 0.13597E+01_JPRB ,&
&  0.13741E+01_JPRB, 0.13986E+01_JPRB, 0.14886E+01_JPRB, 0.15707E+01_JPRB, 0.15814E+01_JPRB, 0.15182E+01_JPRB, 0.14543E+01_JPRB ,&
&  0.14219E+01_JPRB, 0.14310E+01_JPRB, 0.16471E+01_JPRB, 0.21995E+01_JPRB, 0.34212E+01_JPRB, 0.50029E+01_JPRB, 0.66574E+01_JPRB ,&
&  0.71405E+01_JPRB, 0.65323E+01_JPRB, 0.52478E+01_JPRB, 0.42833E+01_JPRB, 0.35722E+01_JPRB, 0.32723E+01_JPRB, 0.27606E+01_JPRB ,&
&  0.23490E+01_JPRB, 0.20672E+01_JPRB, 0.18454E+01_JPRB, 0.17034E+01_JPRB, 0.15765E+01_JPRB, 0.15360E+01_JPRB, 0.14976E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.16073E+01_JPRB, 0.16061E+01_JPRB, 0.15239E+01_JPRB, 0.14156E+01_JPRB, 0.13639E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.12912E+01_JPRB, 0.13192E+01_JPRB, 0.12900E+01_JPRB, 0.12047E+01_JPRB, 0.11542E+01_JPRB, 0.11417E+01_JPRB, 0.11395E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.12704E+01_JPRB, 0.12606E+01_JPRB, 0.12314E+01_JPRB, 0.11935E+01_JPRB, 0.12382E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.10780E+01_JPRB, 0.10291E+01_JPRB, 0.98386E+00_JPRB, 0.92369E+00_JPRB, 0.89448E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.84963E+00_JPRB, 0.77497E+00_JPRB, 0.72239E+00_JPRB, 0.68699E+00_JPRB, 0.68402E+00_JPRB, 0.67667E+00_JPRB, 0.67528E+00_JPRB ,&
&  0.69613E+00_JPRB, 0.70896E+00_JPRB, 0.72207E+00_JPRB, 0.71110E+00_JPRB, 0.68931E+00_JPRB, 0.68406E+00_JPRB, 0.66734E+00_JPRB ,&
&  0.66220E+00_JPRB, 0.70820E+00_JPRB, 0.75318E+00_JPRB, 0.76283E+00_JPRB, 0.74308E+00_JPRB, 0.70433E+00_JPRB, 0.68578E+00_JPRB ,&
&  0.69188E+00_JPRB, 0.71356E+00_JPRB, 0.75350E+00_JPRB, 0.79613E+00_JPRB, 0.82792E+00_JPRB, 0.85400E+00_JPRB, 0.91593E+00_JPRB ,&
&  0.12071E+01_JPRB, 0.17882E+01_JPRB, 0.29932E+01_JPRB, 0.37365E+01_JPRB, 0.29861E+01_JPRB, 0.23033E+01_JPRB, 0.18243E+01_JPRB ,&
&  0.16448E+01_JPRB, 0.15520E+01_JPRB, 0.14603E+01_JPRB, 0.13661E+01_JPRB, 0.12786E+01_JPRB, 0.12205E+01_JPRB, 0.12203E+01_JPRB ,&
&  0.12044E+01_JPRB, 0.11879E+01_JPRB, 0.11726E+01_JPRB, 0.11790E+01_JPRB, 0.11875E+01_JPRB, 0.11790E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11618E+01_JPRB, 0.12503E+01_JPRB, 0.16433E+01_JPRB, 0.28649E+01_JPRB, 0.44263E+01_JPRB, 0.66945E+01_JPRB ,&
&  0.80179E+01_JPRB, 0.72692E+01_JPRB, 0.60409E+01_JPRB, 0.50125E+01_JPRB, 0.42684E+01_JPRB, 0.38531E+01_JPRB, 0.34181E+01_JPRB ,&
&  0.30052E+01_JPRB, 0.25959E+01_JPRB, 0.22583E+01_JPRB, 0.20489E+01_JPRB, 0.18276E+01_JPRB, 0.17274E+01_JPRB, 0.17318E+01_JPRB ,&
&  0.16884E+01_JPRB, 0.16893E+01_JPRB, 0.17059E+01_JPRB, 0.17670E+01_JPRB, 0.16564E+01_JPRB, 0.15253E+01_JPRB, 0.14040E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.13022E+01_JPRB, 0.12720E+01_JPRB, 0.12573E+01_JPRB, 0.12506E+01_JPRB, 0.12385E+01_JPRB, 0.12293E+01_JPRB ,&
&  0.12466E+01_JPRB, 0.12807E+01_JPRB, 0.12897E+01_JPRB, 0.12770E+01_JPRB, 0.11878E+01_JPRB, 0.12528E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.10451E+01_JPRB, 0.10057E+01_JPRB, 0.95769E+00_JPRB, 0.92651E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.93416E+00_JPRB, 0.86198E+00_JPRB, 0.78897E+00_JPRB, 0.74214E+00_JPRB, 0.70780E+00_JPRB, 0.69311E+00_JPRB, 0.68450E+00_JPRB ,&
&  0.69186E+00_JPRB, 0.70070E+00_JPRB, 0.71630E+00_JPRB, 0.71452E+00_JPRB, 0.69651E+00_JPRB, 0.70398E+00_JPRB, 0.72488E+00_JPRB ,&
&  0.72330E+00_JPRB, 0.73516E+00_JPRB, 0.75520E+00_JPRB, 0.74447E+00_JPRB, 0.72082E+00_JPRB, 0.69620E+00_JPRB, 0.67644E+00_JPRB ,&
&  0.67826E+00_JPRB, 0.68374E+00_JPRB, 0.69796E+00_JPRB, 0.72401E+00_JPRB, 0.75992E+00_JPRB, 0.79268E+00_JPRB, 0.81980E+00_JPRB ,&
&  0.91474E+00_JPRB, 0.18913E+01_JPRB, 0.27374E+01_JPRB, 0.32314E+01_JPRB, 0.29375E+01_JPRB, 0.22986E+01_JPRB, 0.19415E+01_JPRB ,&
&  0.16211E+01_JPRB, 0.14645E+01_JPRB, 0.13657E+01_JPRB, 0.12866E+01_JPRB, 0.11924E+01_JPRB, 0.11385E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.10432E+01_JPRB, 0.98873E+00_JPRB, 0.94598E+00_JPRB, 0.93816E+00_JPRB, 0.94447E+00_JPRB, 0.95374E+00_JPRB ,&
&  0.96406E+00_JPRB, 0.98151E+00_JPRB, 0.10366E+01_JPRB, 0.12000E+01_JPRB, 0.20948E+01_JPRB, 0.26539E+01_JPRB, 0.49425E+01_JPRB ,&
&  0.61320E+01_JPRB, 0.53114E+01_JPRB, 0.47765E+01_JPRB, 0.43874E+01_JPRB, 0.39600E+01_JPRB, 0.35771E+01_JPRB, 0.33289E+01_JPRB ,&
&  0.30762E+01_JPRB, 0.27924E+01_JPRB, 0.25542E+01_JPRB, 0.23794E+01_JPRB, 0.22123E+01_JPRB, 0.20870E+01_JPRB, 0.19361E+01_JPRB ,&
&  0.18789E+01_JPRB, 0.17991E+01_JPRB, 0.17703E+01_JPRB, 0.18211E+01_JPRB, 0.18527E+01_JPRB, 0.17447E+01_JPRB, 0.16050E+01_JPRB ,&
&  0.14589E+01_JPRB, 0.13732E+01_JPRB, 0.13136E+01_JPRB, 0.12907E+01_JPRB, 0.13057E+01_JPRB, 0.13179E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.12771E+01_JPRB, 0.12691E+01_JPRB, 0.12953E+01_JPRB, 0.12660E+01_JPRB, 0.11846E+01_JPRB, 0.12569E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.10384E+01_JPRB, 0.99498E+00_JPRB, 0.95410E+00_JPRB, 0.96861E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.97870E+00_JPRB, 0.93184E+00_JPRB, 0.87267E+00_JPRB, 0.82436E+00_JPRB, 0.78273E+00_JPRB, 0.75457E+00_JPRB, 0.74648E+00_JPRB ,&
&  0.73889E+00_JPRB, 0.72799E+00_JPRB, 0.73390E+00_JPRB, 0.73321E+00_JPRB, 0.73293E+00_JPRB, 0.73529E+00_JPRB, 0.75556E+00_JPRB ,&
&  0.77056E+00_JPRB, 0.77125E+00_JPRB, 0.76574E+00_JPRB, 0.74967E+00_JPRB, 0.73001E+00_JPRB, 0.70740E+00_JPRB, 0.69238E+00_JPRB ,&
&  0.68391E+00_JPRB, 0.68387E+00_JPRB, 0.69007E+00_JPRB, 0.70363E+00_JPRB, 0.73080E+00_JPRB, 0.76650E+00_JPRB, 0.80043E+00_JPRB ,&
&  0.81908E+00_JPRB, 0.12210E+01_JPRB, 0.16894E+01_JPRB, 0.25052E+01_JPRB, 0.27219E+01_JPRB, 0.23005E+01_JPRB, 0.19293E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.14474E+01_JPRB, 0.13186E+01_JPRB, 0.12290E+01_JPRB, 0.11389E+01_JPRB, 0.10886E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10085E+01_JPRB, 0.96179E+00_JPRB, 0.90707E+00_JPRB, 0.86823E+00_JPRB, 0.85874E+00_JPRB, 0.86649E+00_JPRB, 0.86826E+00_JPRB ,&
&  0.86580E+00_JPRB, 0.87694E+00_JPRB, 0.89435E+00_JPRB, 0.94372E+00_JPRB, 0.13092E+01_JPRB, 0.17206E+01_JPRB, 0.26723E+01_JPRB ,&
&  0.27475E+01_JPRB, 0.26164E+01_JPRB, 0.25995E+01_JPRB, 0.25746E+01_JPRB, 0.26131E+01_JPRB, 0.26336E+01_JPRB, 0.26803E+01_JPRB ,&
&  0.27083E+01_JPRB, 0.26751E+01_JPRB, 0.25890E+01_JPRB, 0.25200E+01_JPRB, 0.24526E+01_JPRB, 0.23741E+01_JPRB, 0.22275E+01_JPRB ,&
&  0.21105E+01_JPRB, 0.20135E+01_JPRB, 0.19476E+01_JPRB, 0.18847E+01_JPRB, 0.18772E+01_JPRB, 0.18271E+01_JPRB, 0.17241E+01_JPRB ,&
&  0.15877E+01_JPRB, 0.14561E+01_JPRB, 0.13954E+01_JPRB, 0.13671E+01_JPRB, 0.13455E+01_JPRB, 0.13175E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.13173E+01_JPRB, 0.13114E+01_JPRB, 0.12942E+01_JPRB, 0.12579E+01_JPRB, 0.11857E+01_JPRB, 0.12436E+01_JPRB, 0.11943E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.10505E+01_JPRB, 0.98619E+00_JPRB, 0.95114E+00_JPRB, 0.99369E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.10635E+01_JPRB, 0.10156E+01_JPRB, 0.96243E+00_JPRB, 0.89332E+00_JPRB, 0.83717E+00_JPRB, 0.81135E+00_JPRB, 0.79564E+00_JPRB ,&
&  0.77549E+00_JPRB, 0.75673E+00_JPRB, 0.75526E+00_JPRB, 0.76337E+00_JPRB, 0.77186E+00_JPRB, 0.77782E+00_JPRB, 0.77541E+00_JPRB ,&
&  0.77528E+00_JPRB, 0.77805E+00_JPRB, 0.76815E+00_JPRB, 0.75192E+00_JPRB, 0.73437E+00_JPRB, 0.71942E+00_JPRB, 0.70789E+00_JPRB ,&
&  0.70101E+00_JPRB, 0.70290E+00_JPRB, 0.70466E+00_JPRB, 0.71531E+00_JPRB, 0.73582E+00_JPRB, 0.76183E+00_JPRB, 0.79637E+00_JPRB ,&
&  0.81789E+00_JPRB, 0.82561E+00_JPRB, 0.11396E+01_JPRB, 0.15846E+01_JPRB, 0.19556E+01_JPRB, 0.19859E+01_JPRB, 0.18039E+01_JPRB ,&
&  0.16385E+01_JPRB, 0.14654E+01_JPRB, 0.13307E+01_JPRB, 0.12198E+01_JPRB, 0.11271E+01_JPRB, 0.10553E+01_JPRB, 0.10035E+01_JPRB ,&
&  0.95421E+00_JPRB, 0.91365E+00_JPRB, 0.87578E+00_JPRB, 0.84255E+00_JPRB, 0.83109E+00_JPRB, 0.82463E+00_JPRB, 0.81897E+00_JPRB ,&
&  0.81471E+00_JPRB, 0.81442E+00_JPRB, 0.81725E+00_JPRB, 0.82792E+00_JPRB, 0.88044E+00_JPRB, 0.99732E+00_JPRB, 0.12694E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.15070E+01_JPRB, 0.15347E+01_JPRB, 0.15365E+01_JPRB, 0.15734E+01_JPRB, 0.16624E+01_JPRB, 0.18012E+01_JPRB ,&
&  0.19698E+01_JPRB, 0.21235E+01_JPRB, 0.22430E+01_JPRB, 0.23161E+01_JPRB, 0.23306E+01_JPRB, 0.22976E+01_JPRB, 0.22317E+01_JPRB ,&
&  0.21582E+01_JPRB, 0.20456E+01_JPRB, 0.19335E+01_JPRB, 0.18401E+01_JPRB, 0.17887E+01_JPRB, 0.17538E+01_JPRB, 0.16915E+01_JPRB ,&
&  0.15998E+01_JPRB, 0.14929E+01_JPRB, 0.14126E+01_JPRB, 0.13675E+01_JPRB, 0.13416E+01_JPRB, 0.13136E+01_JPRB, 0.13041E+01_JPRB ,&
&  0.12973E+01_JPRB, 0.12940E+01_JPRB, 0.12403E+01_JPRB, 0.12205E+01_JPRB, 0.12531E+01_JPRB, 0.12059E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.10532E+01_JPRB, 0.10029E+01_JPRB, 0.96290E+00_JPRB, 0.10530E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.98742E+00_JPRB, 0.98109E+00_JPRB, 0.94490E+00_JPRB, 0.89328E+00_JPRB, 0.84331E+00_JPRB, 0.80286E+00_JPRB, 0.78157E+00_JPRB ,&
&  0.76635E+00_JPRB, 0.75694E+00_JPRB, 0.76360E+00_JPRB, 0.77974E+00_JPRB, 0.79878E+00_JPRB, 0.80590E+00_JPRB, 0.80211E+00_JPRB ,&
&  0.78904E+00_JPRB, 0.76748E+00_JPRB, 0.75176E+00_JPRB, 0.74144E+00_JPRB, 0.72893E+00_JPRB, 0.72203E+00_JPRB, 0.71809E+00_JPRB ,&
&  0.71975E+00_JPRB, 0.72405E+00_JPRB, 0.72644E+00_JPRB, 0.73554E+00_JPRB, 0.75163E+00_JPRB, 0.77688E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.81663E+00_JPRB, 0.72912E+00_JPRB, 0.85167E+00_JPRB, 0.11217E+01_JPRB, 0.14197E+01_JPRB, 0.15761E+01_JPRB, 0.15760E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.14083E+01_JPRB, 0.13119E+01_JPRB, 0.12041E+01_JPRB, 0.11065E+01_JPRB, 0.10263E+01_JPRB, 0.95623E+00_JPRB ,&
&  0.90163E+00_JPRB, 0.86325E+00_JPRB, 0.84047E+00_JPRB, 0.82430E+00_JPRB, 0.81007E+00_JPRB, 0.78712E+00_JPRB, 0.77373E+00_JPRB ,&
&  0.77219E+00_JPRB, 0.77311E+00_JPRB, 0.77599E+00_JPRB, 0.78183E+00_JPRB, 0.80678E+00_JPRB, 0.84630E+00_JPRB, 0.89827E+00_JPRB ,&
&  0.95426E+00_JPRB, 0.10063E+01_JPRB, 0.10357E+01_JPRB, 0.10508E+01_JPRB, 0.10602E+01_JPRB, 0.11250E+01_JPRB, 0.11917E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.14529E+01_JPRB, 0.16433E+01_JPRB, 0.17663E+01_JPRB, 0.17942E+01_JPRB, 0.17517E+01_JPRB, 0.16771E+01_JPRB ,&
&  0.16210E+01_JPRB, 0.15597E+01_JPRB, 0.14966E+01_JPRB, 0.14419E+01_JPRB, 0.14129E+01_JPRB, 0.14193E+01_JPRB, 0.14283E+01_JPRB ,&
&  0.14266E+01_JPRB, 0.14002E+01_JPRB, 0.13656E+01_JPRB, 0.13495E+01_JPRB, 0.13410E+01_JPRB, 0.13197E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.12301E+01_JPRB, 0.12025E+01_JPRB, 0.11983E+01_JPRB, 0.11586E+01_JPRB, 0.11278E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.10208E+01_JPRB, 0.97543E+00_JPRB, 0.93462E+00_JPRB, 0.92099E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.87760E+00_JPRB, 0.91365E+00_JPRB, 0.91891E+00_JPRB, 0.88976E+00_JPRB, 0.85327E+00_JPRB, 0.82657E+00_JPRB, 0.80826E+00_JPRB ,&
&  0.80191E+00_JPRB, 0.79604E+00_JPRB, 0.80028E+00_JPRB, 0.81403E+00_JPRB, 0.82872E+00_JPRB, 0.83186E+00_JPRB, 0.81361E+00_JPRB ,&
&  0.78827E+00_JPRB, 0.76627E+00_JPRB, 0.75286E+00_JPRB, 0.74567E+00_JPRB, 0.73833E+00_JPRB, 0.73425E+00_JPRB, 0.74131E+00_JPRB ,&
&  0.75033E+00_JPRB, 0.76173E+00_JPRB, 0.77487E+00_JPRB, 0.78656E+00_JPRB, 0.79637E+00_JPRB, 0.80642E+00_JPRB, 0.82913E+00_JPRB ,&
&  0.84524E+00_JPRB, 0.67830E+00_JPRB, 0.72238E+00_JPRB, 0.88708E+00_JPRB, 0.10689E+01_JPRB, 0.12779E+01_JPRB, 0.13692E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.12924E+01_JPRB, 0.12194E+01_JPRB, 0.11398E+01_JPRB, 0.10579E+01_JPRB, 0.98736E+00_JPRB, 0.92394E+00_JPRB ,&
&  0.87943E+00_JPRB, 0.84571E+00_JPRB, 0.82381E+00_JPRB, 0.80128E+00_JPRB, 0.78610E+00_JPRB, 0.77383E+00_JPRB, 0.76344E+00_JPRB ,&
&  0.75553E+00_JPRB, 0.74741E+00_JPRB, 0.74711E+00_JPRB, 0.75227E+00_JPRB, 0.75428E+00_JPRB, 0.76411E+00_JPRB, 0.76413E+00_JPRB ,&
&  0.78897E+00_JPRB, 0.81983E+00_JPRB, 0.83601E+00_JPRB, 0.84294E+00_JPRB, 0.85466E+00_JPRB, 0.87758E+00_JPRB, 0.91015E+00_JPRB ,&
&  0.95213E+00_JPRB, 0.10345E+01_JPRB, 0.11269E+01_JPRB, 0.12121E+01_JPRB, 0.12627E+01_JPRB, 0.12727E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.12619E+01_JPRB, 0.12544E+01_JPRB, 0.12368E+01_JPRB, 0.12103E+01_JPRB, 0.11758E+01_JPRB, 0.11603E+01_JPRB, 0.11733E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.12319E+01_JPRB, 0.12494E+01_JPRB, 0.12664E+01_JPRB, 0.12607E+01_JPRB, 0.12328E+01_JPRB, 0.12041E+01_JPRB ,&
&  0.11705E+01_JPRB, 0.11292E+01_JPRB, 0.10914E+01_JPRB, 0.10103E+01_JPRB, 0.10542E+01_JPRB, 0.10454E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.99917E+00_JPRB, 0.96071E+00_JPRB, 0.93183E+00_JPRB, 0.76520E+00_JPRB, 0.83902E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.82704E+00_JPRB, 0.85988E+00_JPRB, 0.87288E+00_JPRB, 0.86712E+00_JPRB, 0.84834E+00_JPRB, 0.83628E+00_JPRB, 0.82693E+00_JPRB ,&
&  0.82050E+00_JPRB, 0.81542E+00_JPRB, 0.82414E+00_JPRB, 0.83071E+00_JPRB, 0.83205E+00_JPRB, 0.82200E+00_JPRB, 0.81018E+00_JPRB ,&
&  0.79276E+00_JPRB, 0.77218E+00_JPRB, 0.75672E+00_JPRB, 0.74348E+00_JPRB, 0.74054E+00_JPRB, 0.74744E+00_JPRB, 0.76254E+00_JPRB ,&
&  0.78267E+00_JPRB, 0.79698E+00_JPRB, 0.80921E+00_JPRB, 0.82285E+00_JPRB, 0.82699E+00_JPRB, 0.82919E+00_JPRB, 0.83466E+00_JPRB ,&
&  0.84397E+00_JPRB, 0.66022E+00_JPRB, 0.70997E+00_JPRB, 0.78299E+00_JPRB, 0.86322E+00_JPRB, 0.98621E+00_JPRB, 0.10877E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.10802E+01_JPRB, 0.10410E+01_JPRB, 0.99648E+00_JPRB, 0.94993E+00_JPRB, 0.90900E+00_JPRB, 0.86626E+00_JPRB ,&
&  0.83360E+00_JPRB, 0.81222E+00_JPRB, 0.80190E+00_JPRB, 0.79202E+00_JPRB, 0.78077E+00_JPRB, 0.76667E+00_JPRB, 0.75059E+00_JPRB ,&
&  0.73426E+00_JPRB, 0.72451E+00_JPRB, 0.71769E+00_JPRB, 0.71269E+00_JPRB, 0.71396E+00_JPRB, 0.71815E+00_JPRB, 0.72032E+00_JPRB ,&
&  0.73038E+00_JPRB, 0.74081E+00_JPRB, 0.74714E+00_JPRB, 0.74850E+00_JPRB, 0.75022E+00_JPRB, 0.75606E+00_JPRB, 0.77744E+00_JPRB ,&
&  0.79884E+00_JPRB, 0.83358E+00_JPRB, 0.87913E+00_JPRB, 0.93953E+00_JPRB, 0.98817E+00_JPRB, 0.10121E+01_JPRB, 0.10238E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10190E+01_JPRB, 0.99760E+00_JPRB, 0.97669E+00_JPRB, 0.95958E+00_JPRB, 0.95371E+00_JPRB, 0.95372E+00_JPRB ,&
&  0.96550E+00_JPRB, 0.98580E+00_JPRB, 0.10044E+01_JPRB, 0.10268E+01_JPRB, 0.10477E+01_JPRB, 0.10450E+01_JPRB, 0.10124E+01_JPRB ,&
&  0.98362E+00_JPRB, 0.96597E+00_JPRB, 0.95249E+00_JPRB, 0.96078E+00_JPRB, 0.95668E+00_JPRB, 0.94472E+00_JPRB, 0.91920E+00_JPRB ,&
&  0.90047E+00_JPRB, 0.88400E+00_JPRB, 0.68444E+00_JPRB, 0.77241E+00_JPRB, 0.79364E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.81954E+00_JPRB, 0.81636E+00_JPRB, 0.81169E+00_JPRB, 0.81741E+00_JPRB, 0.82312E+00_JPRB, 0.82368E+00_JPRB, 0.82349E+00_JPRB ,&
&  0.82094E+00_JPRB, 0.81692E+00_JPRB, 0.82437E+00_JPRB, 0.83086E+00_JPRB, 0.83051E+00_JPRB, 0.82222E+00_JPRB, 0.81323E+00_JPRB ,&
&  0.80608E+00_JPRB, 0.79589E+00_JPRB, 0.78588E+00_JPRB, 0.78133E+00_JPRB, 0.78316E+00_JPRB, 0.79637E+00_JPRB, 0.81684E+00_JPRB ,&
&  0.83206E+00_JPRB, 0.83101E+00_JPRB, 0.83469E+00_JPRB, 0.84633E+00_JPRB, 0.84985E+00_JPRB, 0.85272E+00_JPRB, 0.86065E+00_JPRB ,&
&  0.74267E+00_JPRB, 0.61945E+00_JPRB, 0.68768E+00_JPRB, 0.75069E+00_JPRB, 0.80244E+00_JPRB, 0.86404E+00_JPRB, 0.93981E+00_JPRB ,&
&  0.96690E+00_JPRB, 0.95160E+00_JPRB, 0.93298E+00_JPRB, 0.90785E+00_JPRB, 0.87844E+00_JPRB, 0.85765E+00_JPRB, 0.84118E+00_JPRB ,&
&  0.83661E+00_JPRB, 0.82516E+00_JPRB, 0.80754E+00_JPRB, 0.79300E+00_JPRB, 0.77997E+00_JPRB, 0.76302E+00_JPRB, 0.74653E+00_JPRB ,&
&  0.72943E+00_JPRB, 0.71458E+00_JPRB, 0.70901E+00_JPRB, 0.70556E+00_JPRB, 0.70674E+00_JPRB, 0.70588E+00_JPRB, 0.70021E+00_JPRB ,&
&  0.69672E+00_JPRB, 0.69854E+00_JPRB, 0.70782E+00_JPRB, 0.71684E+00_JPRB, 0.72077E+00_JPRB, 0.72121E+00_JPRB, 0.73243E+00_JPRB ,&
&  0.74372E+00_JPRB, 0.76273E+00_JPRB, 0.78639E+00_JPRB, 0.82135E+00_JPRB, 0.84968E+00_JPRB, 0.86392E+00_JPRB, 0.87358E+00_JPRB ,&
&  0.88472E+00_JPRB, 0.88084E+00_JPRB, 0.86381E+00_JPRB, 0.85061E+00_JPRB, 0.84414E+00_JPRB, 0.83575E+00_JPRB, 0.83143E+00_JPRB ,&
&  0.83758E+00_JPRB, 0.84564E+00_JPRB, 0.84815E+00_JPRB, 0.85748E+00_JPRB, 0.87022E+00_JPRB, 0.88074E+00_JPRB, 0.88587E+00_JPRB ,&
&  0.86683E+00_JPRB, 0.86689E+00_JPRB, 0.86512E+00_JPRB, 0.87258E+00_JPRB, 0.88067E+00_JPRB, 0.86661E+00_JPRB, 0.85157E+00_JPRB ,&
&  0.83781E+00_JPRB, 0.81273E+00_JPRB, 0.80992E+00_JPRB, 0.80872E+00_JPRB, 0.81051E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.77322E+00_JPRB, 0.76460E+00_JPRB, 0.75936E+00_JPRB, 0.76904E+00_JPRB, 0.78580E+00_JPRB, 0.79919E+00_JPRB, 0.80113E+00_JPRB ,&
&  0.80227E+00_JPRB, 0.80749E+00_JPRB, 0.81587E+00_JPRB, 0.81759E+00_JPRB, 0.80966E+00_JPRB, 0.79177E+00_JPRB, 0.78193E+00_JPRB ,&
&  0.77853E+00_JPRB, 0.77842E+00_JPRB, 0.77752E+00_JPRB, 0.78123E+00_JPRB, 0.80013E+00_JPRB, 0.83016E+00_JPRB, 0.85863E+00_JPRB ,&
&  0.87256E+00_JPRB, 0.86695E+00_JPRB, 0.86071E+00_JPRB, 0.85582E+00_JPRB, 0.85490E+00_JPRB, 0.86018E+00_JPRB, 0.86153E+00_JPRB ,&
&  0.85430E+00_JPRB, 0.70884E+00_JPRB, 0.76733E+00_JPRB, 0.79054E+00_JPRB, 0.81779E+00_JPRB, 0.84989E+00_JPRB, 0.89449E+00_JPRB ,&
&  0.90627E+00_JPRB, 0.89445E+00_JPRB, 0.87082E+00_JPRB, 0.85359E+00_JPRB, 0.83625E+00_JPRB, 0.82926E+00_JPRB, 0.82451E+00_JPRB ,&
&  0.82196E+00_JPRB, 0.81858E+00_JPRB, 0.79643E+00_JPRB, 0.76086E+00_JPRB, 0.74062E+00_JPRB, 0.72726E+00_JPRB, 0.71565E+00_JPRB ,&
&  0.70468E+00_JPRB, 0.69062E+00_JPRB, 0.67713E+00_JPRB, 0.66599E+00_JPRB, 0.66643E+00_JPRB, 0.66378E+00_JPRB, 0.65049E+00_JPRB ,&
&  0.64473E+00_JPRB, 0.64242E+00_JPRB, 0.65012E+00_JPRB, 0.66127E+00_JPRB, 0.66657E+00_JPRB, 0.66341E+00_JPRB, 0.67432E+00_JPRB ,&
&  0.69860E+00_JPRB, 0.71553E+00_JPRB, 0.73001E+00_JPRB, 0.74765E+00_JPRB, 0.75790E+00_JPRB, 0.76032E+00_JPRB, 0.76472E+00_JPRB ,&
&  0.77481E+00_JPRB, 0.77552E+00_JPRB, 0.76775E+00_JPRB, 0.76193E+00_JPRB, 0.76373E+00_JPRB, 0.76524E+00_JPRB, 0.77475E+00_JPRB ,&
&  0.78371E+00_JPRB, 0.78724E+00_JPRB, 0.78391E+00_JPRB, 0.78680E+00_JPRB, 0.79340E+00_JPRB, 0.80441E+00_JPRB, 0.81118E+00_JPRB ,&
&  0.81817E+00_JPRB, 0.81282E+00_JPRB, 0.79882E+00_JPRB, 0.79832E+00_JPRB, 0.80281E+00_JPRB, 0.79967E+00_JPRB, 0.78561E+00_JPRB ,&
&  0.77284E+00_JPRB, 0.76294E+00_JPRB, 0.76448E+00_JPRB, 0.76533E+00_JPRB, 0.76993E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.75958E+00_JPRB, 0.75552E+00_JPRB, 0.75413E+00_JPRB, 0.76513E+00_JPRB, 0.78301E+00_JPRB, 0.78812E+00_JPRB, 0.79073E+00_JPRB ,&
&  0.80419E+00_JPRB, 0.81436E+00_JPRB, 0.81430E+00_JPRB, 0.81269E+00_JPRB, 0.80631E+00_JPRB, 0.79180E+00_JPRB, 0.78040E+00_JPRB ,&
&  0.78496E+00_JPRB, 0.78994E+00_JPRB, 0.79139E+00_JPRB, 0.81472E+00_JPRB, 0.84585E+00_JPRB, 0.88306E+00_JPRB, 0.90825E+00_JPRB ,&
&  0.91714E+00_JPRB, 0.90970E+00_JPRB, 0.90594E+00_JPRB, 0.90071E+00_JPRB, 0.89538E+00_JPRB, 0.89723E+00_JPRB, 0.89180E+00_JPRB ,&
&  0.88193E+00_JPRB, 0.79303E+00_JPRB, 0.82414E+00_JPRB, 0.83423E+00_JPRB, 0.85083E+00_JPRB, 0.87637E+00_JPRB, 0.90896E+00_JPRB ,&
&  0.91192E+00_JPRB, 0.90556E+00_JPRB, 0.89258E+00_JPRB, 0.86866E+00_JPRB, 0.85031E+00_JPRB, 0.84014E+00_JPRB, 0.82276E+00_JPRB ,&
&  0.79509E+00_JPRB, 0.75779E+00_JPRB, 0.72479E+00_JPRB, 0.70225E+00_JPRB, 0.69171E+00_JPRB, 0.68834E+00_JPRB, 0.68285E+00_JPRB ,&
&  0.67987E+00_JPRB, 0.67740E+00_JPRB, 0.66603E+00_JPRB, 0.65580E+00_JPRB, 0.64509E+00_JPRB, 0.63750E+00_JPRB, 0.62387E+00_JPRB ,&
&  0.61562E+00_JPRB, 0.60458E+00_JPRB, 0.60376E+00_JPRB, 0.62395E+00_JPRB, 0.64579E+00_JPRB, 0.65171E+00_JPRB, 0.66601E+00_JPRB ,&
&  0.68555E+00_JPRB, 0.69471E+00_JPRB, 0.69711E+00_JPRB, 0.69471E+00_JPRB, 0.68739E+00_JPRB, 0.68691E+00_JPRB, 0.69555E+00_JPRB ,&
&  0.70391E+00_JPRB, 0.71493E+00_JPRB, 0.71957E+00_JPRB, 0.72477E+00_JPRB, 0.73188E+00_JPRB, 0.73740E+00_JPRB, 0.74826E+00_JPRB ,&
&  0.75390E+00_JPRB, 0.75775E+00_JPRB, 0.76865E+00_JPRB, 0.78007E+00_JPRB, 0.78543E+00_JPRB, 0.78829E+00_JPRB, 0.79817E+00_JPRB ,&
&  0.80257E+00_JPRB, 0.80111E+00_JPRB, 0.79204E+00_JPRB, 0.77920E+00_JPRB, 0.77022E+00_JPRB, 0.75721E+00_JPRB, 0.73681E+00_JPRB ,&
&  0.71800E+00_JPRB, 0.71520E+00_JPRB, 0.72412E+00_JPRB, 0.73402E+00_JPRB, 0.75078E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.74419E+00_JPRB, 0.75550E+00_JPRB, 0.76846E+00_JPRB, 0.78107E+00_JPRB, 0.79426E+00_JPRB, 0.79396E+00_JPRB, 0.79598E+00_JPRB ,&
&  0.81638E+00_JPRB, 0.82047E+00_JPRB, 0.82054E+00_JPRB, 0.81227E+00_JPRB, 0.80608E+00_JPRB, 0.80580E+00_JPRB, 0.79652E+00_JPRB ,&
&  0.79163E+00_JPRB, 0.79745E+00_JPRB, 0.81515E+00_JPRB, 0.84256E+00_JPRB, 0.86937E+00_JPRB, 0.90496E+00_JPRB, 0.93137E+00_JPRB ,&
&  0.93539E+00_JPRB, 0.92435E+00_JPRB, 0.92449E+00_JPRB, 0.92775E+00_JPRB, 0.92780E+00_JPRB, 0.91082E+00_JPRB, 0.90666E+00_JPRB ,&
&  0.88647E+00_JPRB, 0.89589E+00_JPRB, 0.76554E+00_JPRB, 0.83610E+00_JPRB, 0.88081E+00_JPRB, 0.91680E+00_JPRB, 0.93088E+00_JPRB ,&
&  0.92387E+00_JPRB, 0.91064E+00_JPRB, 0.88507E+00_JPRB, 0.85635E+00_JPRB, 0.84615E+00_JPRB, 0.82602E+00_JPRB, 0.80264E+00_JPRB ,&
&  0.75372E+00_JPRB, 0.69113E+00_JPRB, 0.65923E+00_JPRB, 0.64752E+00_JPRB, 0.64154E+00_JPRB, 0.64227E+00_JPRB, 0.64694E+00_JPRB ,&
&  0.65003E+00_JPRB, 0.65010E+00_JPRB, 0.63861E+00_JPRB, 0.62158E+00_JPRB, 0.60078E+00_JPRB, 0.58958E+00_JPRB, 0.58723E+00_JPRB ,&
&  0.58408E+00_JPRB, 0.57064E+00_JPRB, 0.57391E+00_JPRB, 0.60425E+00_JPRB, 0.62970E+00_JPRB, 0.64867E+00_JPRB, 0.65803E+00_JPRB ,&
&  0.66502E+00_JPRB, 0.65979E+00_JPRB, 0.65669E+00_JPRB, 0.64745E+00_JPRB, 0.62347E+00_JPRB, 0.61669E+00_JPRB, 0.63772E+00_JPRB ,&
&  0.66179E+00_JPRB, 0.67570E+00_JPRB, 0.68414E+00_JPRB, 0.69686E+00_JPRB, 0.71054E+00_JPRB, 0.71830E+00_JPRB, 0.72441E+00_JPRB ,&
&  0.72861E+00_JPRB, 0.73276E+00_JPRB, 0.74625E+00_JPRB, 0.76167E+00_JPRB, 0.76892E+00_JPRB, 0.77044E+00_JPRB, 0.77079E+00_JPRB ,&
&  0.76836E+00_JPRB, 0.76103E+00_JPRB, 0.75239E+00_JPRB, 0.74435E+00_JPRB, 0.72448E+00_JPRB, 0.70068E+00_JPRB, 0.68177E+00_JPRB ,&
&  0.67136E+00_JPRB, 0.68079E+00_JPRB, 0.68825E+00_JPRB, 0.70075E+00_JPRB, 0.72741E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.78061E+00_JPRB, 0.79430E+00_JPRB, 0.80564E+00_JPRB, 0.82627E+00_JPRB, 0.84488E+00_JPRB, 0.84209E+00_JPRB, 0.83551E+00_JPRB ,&
&  0.83137E+00_JPRB, 0.82761E+00_JPRB, 0.82115E+00_JPRB, 0.81202E+00_JPRB, 0.80591E+00_JPRB, 0.80554E+00_JPRB, 0.80600E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.82698E+00_JPRB, 0.85955E+00_JPRB, 0.88503E+00_JPRB, 0.91297E+00_JPRB, 0.95591E+00_JPRB, 0.97607E+00_JPRB ,&
&  0.95897E+00_JPRB, 0.95764E+00_JPRB, 0.94675E+00_JPRB, 0.93223E+00_JPRB, 0.93417E+00_JPRB, 0.92513E+00_JPRB, 0.89724E+00_JPRB ,&
&  0.89101E+00_JPRB, 0.88524E+00_JPRB, 0.88107E+00_JPRB, 0.91033E+00_JPRB, 0.93417E+00_JPRB, 0.93585E+00_JPRB, 0.93435E+00_JPRB ,&
&  0.91187E+00_JPRB, 0.87483E+00_JPRB, 0.83821E+00_JPRB, 0.81970E+00_JPRB, 0.81732E+00_JPRB, 0.79732E+00_JPRB, 0.76925E+00_JPRB ,&
&  0.71159E+00_JPRB, 0.66418E+00_JPRB, 0.63786E+00_JPRB, 0.62203E+00_JPRB, 0.61667E+00_JPRB, 0.61362E+00_JPRB, 0.60882E+00_JPRB ,&
&  0.61390E+00_JPRB, 0.60838E+00_JPRB, 0.57988E+00_JPRB, 0.54966E+00_JPRB, 0.54387E+00_JPRB, 0.55284E+00_JPRB, 0.57369E+00_JPRB ,&
&  0.58891E+00_JPRB, 0.58390E+00_JPRB, 0.59076E+00_JPRB, 0.59904E+00_JPRB, 0.61717E+00_JPRB, 0.63591E+00_JPRB, 0.63784E+00_JPRB ,&
&  0.63862E+00_JPRB, 0.63392E+00_JPRB, 0.62104E+00_JPRB, 0.61211E+00_JPRB, 0.60045E+00_JPRB, 0.59534E+00_JPRB, 0.60625E+00_JPRB ,&
&  0.62356E+00_JPRB, 0.63339E+00_JPRB, 0.64402E+00_JPRB, 0.65777E+00_JPRB, 0.66947E+00_JPRB, 0.67817E+00_JPRB, 0.68462E+00_JPRB ,&
&  0.69468E+00_JPRB, 0.70950E+00_JPRB, 0.72476E+00_JPRB, 0.73829E+00_JPRB, 0.73855E+00_JPRB, 0.72423E+00_JPRB, 0.69863E+00_JPRB ,&
&  0.68163E+00_JPRB, 0.66849E+00_JPRB, 0.66577E+00_JPRB, 0.67214E+00_JPRB, 0.66754E+00_JPRB, 0.64793E+00_JPRB, 0.63774E+00_JPRB ,&
&  0.64811E+00_JPRB, 0.67193E+00_JPRB, 0.69453E+00_JPRB, 0.72527E+00_JPRB, 0.75746E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.81595E+00_JPRB, 0.84716E+00_JPRB, 0.87089E+00_JPRB, 0.87887E+00_JPRB, 0.87629E+00_JPRB, 0.86758E+00_JPRB, 0.84544E+00_JPRB ,&
&  0.82130E+00_JPRB, 0.80449E+00_JPRB, 0.78660E+00_JPRB, 0.76037E+00_JPRB, 0.73868E+00_JPRB, 0.73439E+00_JPRB, 0.74711E+00_JPRB ,&
&  0.79342E+00_JPRB, 0.83447E+00_JPRB, 0.85103E+00_JPRB, 0.86461E+00_JPRB, 0.91428E+00_JPRB, 0.95590E+00_JPRB, 0.96468E+00_JPRB ,&
&  0.95892E+00_JPRB, 0.96009E+00_JPRB, 0.92358E+00_JPRB, 0.89057E+00_JPRB, 0.89097E+00_JPRB, 0.89661E+00_JPRB, 0.87857E+00_JPRB ,&
&  0.90441E+00_JPRB, 0.92233E+00_JPRB, 0.91746E+00_JPRB, 0.89499E+00_JPRB, 0.88916E+00_JPRB, 0.87910E+00_JPRB, 0.86652E+00_JPRB ,&
&  0.84435E+00_JPRB, 0.79612E+00_JPRB, 0.76460E+00_JPRB, 0.76180E+00_JPRB, 0.75459E+00_JPRB, 0.74765E+00_JPRB, 0.73451E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.65825E+00_JPRB, 0.63086E+00_JPRB, 0.61203E+00_JPRB, 0.60007E+00_JPRB, 0.59057E+00_JPRB, 0.58045E+00_JPRB ,&
&  0.57160E+00_JPRB, 0.54330E+00_JPRB, 0.51404E+00_JPRB, 0.50694E+00_JPRB, 0.51213E+00_JPRB, 0.51985E+00_JPRB, 0.54831E+00_JPRB ,&
&  0.57348E+00_JPRB, 0.57195E+00_JPRB, 0.56859E+00_JPRB, 0.57164E+00_JPRB, 0.58125E+00_JPRB, 0.58259E+00_JPRB, 0.57301E+00_JPRB ,&
&  0.56836E+00_JPRB, 0.56621E+00_JPRB, 0.55664E+00_JPRB, 0.54943E+00_JPRB, 0.54880E+00_JPRB, 0.55255E+00_JPRB, 0.56652E+00_JPRB ,&
&  0.57504E+00_JPRB, 0.57609E+00_JPRB, 0.58942E+00_JPRB, 0.60638E+00_JPRB, 0.61688E+00_JPRB, 0.62534E+00_JPRB, 0.63161E+00_JPRB ,&
&  0.64554E+00_JPRB, 0.66534E+00_JPRB, 0.68052E+00_JPRB, 0.68282E+00_JPRB, 0.66005E+00_JPRB, 0.63589E+00_JPRB, 0.61699E+00_JPRB ,&
&  0.60865E+00_JPRB, 0.61305E+00_JPRB, 0.62091E+00_JPRB, 0.62859E+00_JPRB, 0.63550E+00_JPRB, 0.64025E+00_JPRB, 0.63975E+00_JPRB ,&
&  0.65947E+00_JPRB, 0.69044E+00_JPRB, 0.71534E+00_JPRB, 0.74496E+00_JPRB, 0.77600E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.88083E+00_JPRB, 0.89815E+00_JPRB, 0.90950E+00_JPRB, 0.90972E+00_JPRB, 0.89383E+00_JPRB, 0.86700E+00_JPRB, 0.84422E+00_JPRB ,&
&  0.81830E+00_JPRB, 0.79308E+00_JPRB, 0.76916E+00_JPRB, 0.73631E+00_JPRB, 0.71867E+00_JPRB, 0.72215E+00_JPRB, 0.77122E+00_JPRB ,&
&  0.83672E+00_JPRB, 0.84181E+00_JPRB, 0.83597E+00_JPRB, 0.86123E+00_JPRB, 0.91346E+00_JPRB, 0.94504E+00_JPRB, 0.96562E+00_JPRB ,&
&  0.96743E+00_JPRB, 0.93384E+00_JPRB, 0.90098E+00_JPRB, 0.88825E+00_JPRB, 0.91230E+00_JPRB, 0.93032E+00_JPRB, 0.93526E+00_JPRB ,&
&  0.93091E+00_JPRB, 0.91054E+00_JPRB, 0.89285E+00_JPRB, 0.87789E+00_JPRB, 0.88264E+00_JPRB, 0.85770E+00_JPRB, 0.80514E+00_JPRB ,&
&  0.73863E+00_JPRB, 0.71190E+00_JPRB, 0.69964E+00_JPRB, 0.68493E+00_JPRB, 0.67417E+00_JPRB, 0.67471E+00_JPRB, 0.66599E+00_JPRB ,&
&  0.65251E+00_JPRB, 0.63638E+00_JPRB, 0.62076E+00_JPRB, 0.60258E+00_JPRB, 0.58818E+00_JPRB, 0.56776E+00_JPRB, 0.54862E+00_JPRB ,&
&  0.53087E+00_JPRB, 0.50869E+00_JPRB, 0.48817E+00_JPRB, 0.47359E+00_JPRB, 0.46692E+00_JPRB, 0.47783E+00_JPRB, 0.51888E+00_JPRB ,&
&  0.55071E+00_JPRB, 0.54779E+00_JPRB, 0.54565E+00_JPRB, 0.55027E+00_JPRB, 0.54386E+00_JPRB, 0.53406E+00_JPRB, 0.52866E+00_JPRB ,&
&  0.52356E+00_JPRB, 0.51967E+00_JPRB, 0.51816E+00_JPRB, 0.52029E+00_JPRB, 0.52265E+00_JPRB, 0.52078E+00_JPRB, 0.52266E+00_JPRB ,&
&  0.52659E+00_JPRB, 0.53154E+00_JPRB, 0.54854E+00_JPRB, 0.57296E+00_JPRB, 0.59403E+00_JPRB, 0.60948E+00_JPRB, 0.62377E+00_JPRB ,&
&  0.64539E+00_JPRB, 0.65867E+00_JPRB, 0.64627E+00_JPRB, 0.63466E+00_JPRB, 0.62897E+00_JPRB, 0.63454E+00_JPRB, 0.64523E+00_JPRB ,&
&  0.66575E+00_JPRB, 0.67828E+00_JPRB, 0.68446E+00_JPRB, 0.68580E+00_JPRB, 0.68884E+00_JPRB, 0.68964E+00_JPRB, 0.70227E+00_JPRB ,&
&  0.73623E+00_JPRB, 0.76542E+00_JPRB, 0.77245E+00_JPRB, 0.80468E+00_JPRB, 0.84611E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.91867E+00_JPRB, 0.92251E+00_JPRB, 0.91232E+00_JPRB, 0.90772E+00_JPRB, 0.89701E+00_JPRB, 0.88327E+00_JPRB, 0.86105E+00_JPRB ,&
&  0.83503E+00_JPRB, 0.80701E+00_JPRB, 0.78024E+00_JPRB, 0.75824E+00_JPRB, 0.76185E+00_JPRB, 0.79424E+00_JPRB, 0.82170E+00_JPRB ,&
&  0.81095E+00_JPRB, 0.80723E+00_JPRB, 0.82429E+00_JPRB, 0.85483E+00_JPRB, 0.88788E+00_JPRB, 0.91827E+00_JPRB, 0.93970E+00_JPRB ,&
&  0.95691E+00_JPRB, 0.93868E+00_JPRB, 0.94312E+00_JPRB, 0.94587E+00_JPRB, 0.94014E+00_JPRB, 0.92210E+00_JPRB, 0.89317E+00_JPRB ,&
&  0.86471E+00_JPRB, 0.83745E+00_JPRB, 0.80388E+00_JPRB, 0.74612E+00_JPRB, 0.81928E+00_JPRB, 0.74463E+00_JPRB, 0.71669E+00_JPRB ,&
&  0.68921E+00_JPRB, 0.67071E+00_JPRB, 0.64981E+00_JPRB, 0.62741E+00_JPRB, 0.61982E+00_JPRB, 0.61536E+00_JPRB, 0.60739E+00_JPRB ,&
&  0.60070E+00_JPRB, 0.59310E+00_JPRB, 0.58160E+00_JPRB, 0.56736E+00_JPRB, 0.55413E+00_JPRB, 0.54014E+00_JPRB, 0.52722E+00_JPRB ,&
&  0.50828E+00_JPRB, 0.48041E+00_JPRB, 0.45109E+00_JPRB, 0.42938E+00_JPRB, 0.41793E+00_JPRB, 0.42750E+00_JPRB, 0.46134E+00_JPRB ,&
&  0.49595E+00_JPRB, 0.50191E+00_JPRB, 0.51188E+00_JPRB, 0.51518E+00_JPRB, 0.50671E+00_JPRB, 0.49608E+00_JPRB, 0.47812E+00_JPRB ,&
&  0.46491E+00_JPRB, 0.46317E+00_JPRB, 0.46458E+00_JPRB, 0.46908E+00_JPRB, 0.47410E+00_JPRB, 0.48100E+00_JPRB, 0.49060E+00_JPRB ,&
&  0.49931E+00_JPRB, 0.50248E+00_JPRB, 0.51490E+00_JPRB, 0.54172E+00_JPRB, 0.57298E+00_JPRB, 0.60485E+00_JPRB, 0.63240E+00_JPRB ,&
&  0.65149E+00_JPRB, 0.66510E+00_JPRB, 0.67365E+00_JPRB, 0.68887E+00_JPRB, 0.70461E+00_JPRB, 0.72024E+00_JPRB, 0.72820E+00_JPRB ,&
&  0.72559E+00_JPRB, 0.72953E+00_JPRB, 0.74735E+00_JPRB, 0.77598E+00_JPRB, 0.79586E+00_JPRB, 0.82967E+00_JPRB, 0.85079E+00_JPRB ,&
&  0.87303E+00_JPRB, 0.87857E+00_JPRB, 0.87295E+00_JPRB, 0.88526E+00_JPRB, 0.90168E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.90259E+00_JPRB, 0.91292E+00_JPRB, 0.91062E+00_JPRB, 0.90940E+00_JPRB, 0.91097E+00_JPRB, 0.90389E+00_JPRB, 0.89017E+00_JPRB ,&
&  0.87626E+00_JPRB, 0.84972E+00_JPRB, 0.81893E+00_JPRB, 0.80489E+00_JPRB, 0.81186E+00_JPRB, 0.81139E+00_JPRB, 0.79284E+00_JPRB ,&
&  0.79131E+00_JPRB, 0.80502E+00_JPRB, 0.82790E+00_JPRB, 0.85330E+00_JPRB, 0.88736E+00_JPRB, 0.92266E+00_JPRB, 0.94889E+00_JPRB ,&
&  0.95937E+00_JPRB, 0.95786E+00_JPRB, 0.95045E+00_JPRB, 0.92382E+00_JPRB, 0.88801E+00_JPRB, 0.86055E+00_JPRB, 0.83967E+00_JPRB ,&
&  0.81375E+00_JPRB, 0.76472E+00_JPRB, 0.69623E+00_JPRB, 0.69050E+00_JPRB, 0.62841E+00_JPRB, 0.60993E+00_JPRB, 0.63266E+00_JPRB ,&
&  0.64813E+00_JPRB, 0.63029E+00_JPRB, 0.60610E+00_JPRB, 0.58604E+00_JPRB, 0.58196E+00_JPRB, 0.58042E+00_JPRB, 0.57511E+00_JPRB ,&
&  0.57106E+00_JPRB, 0.56418E+00_JPRB, 0.54992E+00_JPRB, 0.53694E+00_JPRB, 0.52471E+00_JPRB, 0.51244E+00_JPRB, 0.50381E+00_JPRB ,&
&  0.47480E+00_JPRB, 0.43071E+00_JPRB, 0.39777E+00_JPRB, 0.38309E+00_JPRB, 0.37488E+00_JPRB, 0.37868E+00_JPRB, 0.39204E+00_JPRB ,&
&  0.40327E+00_JPRB, 0.40458E+00_JPRB, 0.39934E+00_JPRB, 0.38614E+00_JPRB, 0.36446E+00_JPRB, 0.32089E+00_JPRB, 0.29527E+00_JPRB ,&
&  0.31201E+00_JPRB, 0.34405E+00_JPRB, 0.37775E+00_JPRB, 0.38989E+00_JPRB, 0.39899E+00_JPRB, 0.40055E+00_JPRB, 0.38504E+00_JPRB ,&
&  0.35570E+00_JPRB, 0.32826E+00_JPRB, 0.32365E+00_JPRB, 0.32593E+00_JPRB, 0.33158E+00_JPRB, 0.34144E+00_JPRB, 0.35772E+00_JPRB ,&
&  0.38229E+00_JPRB, 0.41972E+00_JPRB, 0.44219E+00_JPRB, 0.44490E+00_JPRB, 0.43761E+00_JPRB, 0.42646E+00_JPRB, 0.42401E+00_JPRB ,&
&  0.43840E+00_JPRB, 0.49836E+00_JPRB, 0.61923E+00_JPRB, 0.76452E+00_JPRB, 0.76649E+00_JPRB, 0.81114E+00_JPRB, 0.84578E+00_JPRB ,&
&  0.87566E+00_JPRB, 0.88916E+00_JPRB, 0.87601E+00_JPRB, 0.87680E+00_JPRB, 0.88290E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.86095E+00_JPRB, 0.88087E+00_JPRB, 0.89170E+00_JPRB, 0.90563E+00_JPRB, 0.90590E+00_JPRB, 0.87587E+00_JPRB, 0.86158E+00_JPRB ,&
&  0.86625E+00_JPRB, 0.86114E+00_JPRB, 0.84976E+00_JPRB, 0.84244E+00_JPRB, 0.83328E+00_JPRB, 0.82343E+00_JPRB, 0.81537E+00_JPRB ,&
&  0.80717E+00_JPRB, 0.80200E+00_JPRB, 0.80877E+00_JPRB, 0.82246E+00_JPRB, 0.83363E+00_JPRB, 0.83965E+00_JPRB, 0.84467E+00_JPRB ,&
&  0.84436E+00_JPRB, 0.81150E+00_JPRB, 0.76623E+00_JPRB, 0.73325E+00_JPRB, 0.71706E+00_JPRB, 0.70331E+00_JPRB, 0.66971E+00_JPRB ,&
&  0.60019E+00_JPRB, 0.47920E+00_JPRB, 0.42456E+00_JPRB, 0.46654E+00_JPRB, 0.58158E+00_JPRB, 0.54842E+00_JPRB, 0.56986E+00_JPRB ,&
&  0.60092E+00_JPRB, 0.61709E+00_JPRB, 0.61296E+00_JPRB, 0.60247E+00_JPRB, 0.59899E+00_JPRB, 0.60267E+00_JPRB, 0.59963E+00_JPRB ,&
&  0.59652E+00_JPRB, 0.58074E+00_JPRB, 0.57122E+00_JPRB, 0.55966E+00_JPRB, 0.54146E+00_JPRB, 0.51544E+00_JPRB, 0.49931E+00_JPRB ,&
&  0.46002E+00_JPRB, 0.42141E+00_JPRB, 0.40967E+00_JPRB, 0.40945E+00_JPRB, 0.37824E+00_JPRB, 0.36543E+00_JPRB, 0.34978E+00_JPRB ,&
&  0.31103E+00_JPRB, 0.28985E+00_JPRB, 0.25855E+00_JPRB, 0.23905E+00_JPRB, 0.22499E+00_JPRB, 0.21020E+00_JPRB, 0.20793E+00_JPRB ,&
&  0.21330E+00_JPRB, 0.22657E+00_JPRB, 0.24736E+00_JPRB, 0.27927E+00_JPRB, 0.30767E+00_JPRB, 0.26898E+00_JPRB, 0.25232E+00_JPRB ,&
&  0.23771E+00_JPRB, 0.23448E+00_JPRB, 0.24092E+00_JPRB, 0.24829E+00_JPRB, 0.25562E+00_JPRB, 0.26416E+00_JPRB, 0.27263E+00_JPRB ,&
&  0.27873E+00_JPRB, 0.28172E+00_JPRB, 0.28222E+00_JPRB, 0.28137E+00_JPRB, 0.27985E+00_JPRB, 0.28032E+00_JPRB, 0.28369E+00_JPRB ,&
&  0.29040E+00_JPRB, 0.30305E+00_JPRB, 0.32707E+00_JPRB, 0.36254E+00_JPRB, 0.40847E+00_JPRB, 0.46893E+00_JPRB, 0.56064E+00_JPRB ,&
&  0.68509E+00_JPRB, 0.84865E+00_JPRB, 0.89491E+00_JPRB, 0.90408E+00_JPRB, 0.87265E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.86559E+00_JPRB, 0.86280E+00_JPRB, 0.87000E+00_JPRB, 0.85975E+00_JPRB, 0.84087E+00_JPRB, 0.83207E+00_JPRB, 0.84917E+00_JPRB ,&
&  0.86893E+00_JPRB, 0.83928E+00_JPRB, 0.81736E+00_JPRB, 0.79803E+00_JPRB, 0.78395E+00_JPRB, 0.76988E+00_JPRB, 0.74802E+00_JPRB ,&
&  0.72800E+00_JPRB, 0.72059E+00_JPRB, 0.71854E+00_JPRB, 0.71342E+00_JPRB, 0.70316E+00_JPRB, 0.69315E+00_JPRB, 0.67555E+00_JPRB ,&
&  0.64232E+00_JPRB, 0.60731E+00_JPRB, 0.58932E+00_JPRB, 0.56567E+00_JPRB, 0.54898E+00_JPRB, 0.52957E+00_JPRB, 0.50198E+00_JPRB ,&
&  0.45628E+00_JPRB, 0.39843E+00_JPRB, 0.36450E+00_JPRB, 0.40202E+00_JPRB, 0.49471E+00_JPRB, 0.48761E+00_JPRB, 0.50787E+00_JPRB ,&
&  0.54295E+00_JPRB, 0.56427E+00_JPRB, 0.57508E+00_JPRB, 0.57871E+00_JPRB, 0.58824E+00_JPRB, 0.59457E+00_JPRB, 0.59119E+00_JPRB ,&
&  0.57600E+00_JPRB, 0.52369E+00_JPRB, 0.48014E+00_JPRB, 0.37587E+00_JPRB, 0.30539E+00_JPRB, 0.26805E+00_JPRB, 0.23790E+00_JPRB ,&
&  0.21791E+00_JPRB, 0.20646E+00_JPRB, 0.20392E+00_JPRB, 0.20247E+00_JPRB, 0.20797E+00_JPRB, 0.21470E+00_JPRB, 0.21473E+00_JPRB ,&
&  0.21031E+00_JPRB, 0.20177E+00_JPRB, 0.19014E+00_JPRB, 0.18347E+00_JPRB, 0.17983E+00_JPRB, 0.17995E+00_JPRB, 0.18196E+00_JPRB ,&
&  0.18305E+00_JPRB, 0.18644E+00_JPRB, 0.19904E+00_JPRB, 0.20969E+00_JPRB, 0.22321E+00_JPRB, 0.20638E+00_JPRB, 0.19213E+00_JPRB ,&
&  0.18670E+00_JPRB, 0.19180E+00_JPRB, 0.19931E+00_JPRB, 0.20490E+00_JPRB, 0.21076E+00_JPRB, 0.21552E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.22506E+00_JPRB, 0.22851E+00_JPRB, 0.23004E+00_JPRB, 0.23175E+00_JPRB, 0.23328E+00_JPRB, 0.23563E+00_JPRB, 0.23846E+00_JPRB ,&
&  0.24336E+00_JPRB, 0.25303E+00_JPRB, 0.26555E+00_JPRB, 0.28632E+00_JPRB, 0.30580E+00_JPRB, 0.32508E+00_JPRB, 0.34777E+00_JPRB ,&
&  0.36820E+00_JPRB, 0.44926E+00_JPRB, 0.64235E+00_JPRB, 0.96141E+00_JPRB, 0.95507E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.81265E+00_JPRB, 0.82487E+00_JPRB, 0.84162E+00_JPRB, 0.81824E+00_JPRB, 0.80315E+00_JPRB, 0.80649E+00_JPRB, 0.80728E+00_JPRB ,&
&  0.78391E+00_JPRB, 0.74807E+00_JPRB, 0.69837E+00_JPRB, 0.65678E+00_JPRB, 0.59253E+00_JPRB, 0.50958E+00_JPRB, 0.47178E+00_JPRB ,&
&  0.43697E+00_JPRB, 0.43315E+00_JPRB, 0.43513E+00_JPRB, 0.45122E+00_JPRB, 0.45988E+00_JPRB, 0.45647E+00_JPRB, 0.43439E+00_JPRB ,&
&  0.40729E+00_JPRB, 0.38057E+00_JPRB, 0.36299E+00_JPRB, 0.34946E+00_JPRB, 0.34941E+00_JPRB, 0.34752E+00_JPRB, 0.34099E+00_JPRB ,&
&  0.33302E+00_JPRB, 0.33807E+00_JPRB, 0.35587E+00_JPRB, 0.38319E+00_JPRB, 0.43103E+00_JPRB, 0.41085E+00_JPRB, 0.41877E+00_JPRB ,&
&  0.44926E+00_JPRB, 0.47126E+00_JPRB, 0.48008E+00_JPRB, 0.48338E+00_JPRB, 0.47617E+00_JPRB, 0.49932E+00_JPRB, 0.44286E+00_JPRB ,&
&  0.33634E+00_JPRB, 0.27717E+00_JPRB, 0.23469E+00_JPRB, 0.20675E+00_JPRB, 0.18744E+00_JPRB, 0.17598E+00_JPRB, 0.16305E+00_JPRB ,&
&  0.15693E+00_JPRB, 0.15223E+00_JPRB, 0.15123E+00_JPRB, 0.15327E+00_JPRB, 0.15618E+00_JPRB, 0.15985E+00_JPRB, 0.16112E+00_JPRB ,&
&  0.16326E+00_JPRB, 0.16330E+00_JPRB, 0.16249E+00_JPRB, 0.16239E+00_JPRB, 0.16123E+00_JPRB, 0.16182E+00_JPRB, 0.16155E+00_JPRB ,&
&  0.16112E+00_JPRB, 0.16071E+00_JPRB, 0.16237E+00_JPRB, 0.16324E+00_JPRB, 0.16402E+00_JPRB, 0.16093E+00_JPRB, 0.15950E+00_JPRB ,&
&  0.15999E+00_JPRB, 0.16234E+00_JPRB, 0.16602E+00_JPRB, 0.17016E+00_JPRB, 0.17588E+00_JPRB, 0.18031E+00_JPRB, 0.18542E+00_JPRB ,&
&  0.18982E+00_JPRB, 0.19414E+00_JPRB, 0.19834E+00_JPRB, 0.20264E+00_JPRB, 0.20735E+00_JPRB, 0.21205E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.22205E+00_JPRB, 0.23067E+00_JPRB, 0.24124E+00_JPRB, 0.25957E+00_JPRB, 0.28409E+00_JPRB, 0.31006E+00_JPRB, 0.35618E+00_JPRB ,&
&  0.41369E+00_JPRB, 0.64171E+00_JPRB, 0.94472E+00_JPRB, 0.77755E+00_JPRB, 0.79497E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.80272E+00_JPRB, 0.82102E+00_JPRB, 0.83692E+00_JPRB, 0.83427E+00_JPRB, 0.81697E+00_JPRB, 0.75944E+00_JPRB, 0.69608E+00_JPRB ,&
&  0.61754E+00_JPRB, 0.55193E+00_JPRB, 0.48249E+00_JPRB, 0.42270E+00_JPRB, 0.37221E+00_JPRB, 0.33326E+00_JPRB, 0.30912E+00_JPRB ,&
&  0.29184E+00_JPRB, 0.28420E+00_JPRB, 0.28085E+00_JPRB, 0.28539E+00_JPRB, 0.28748E+00_JPRB, 0.29296E+00_JPRB, 0.28734E+00_JPRB ,&
&  0.28037E+00_JPRB, 0.27269E+00_JPRB, 0.27059E+00_JPRB, 0.27921E+00_JPRB, 0.29271E+00_JPRB, 0.30575E+00_JPRB, 0.32281E+00_JPRB ,&
&  0.34115E+00_JPRB, 0.34707E+00_JPRB, 0.36175E+00_JPRB, 0.36534E+00_JPRB, 0.35101E+00_JPRB, 0.35299E+00_JPRB, 0.35899E+00_JPRB ,&
&  0.37240E+00_JPRB, 0.39100E+00_JPRB, 0.40164E+00_JPRB, 0.42139E+00_JPRB, 0.36724E+00_JPRB, 0.30725E+00_JPRB, 0.25493E+00_JPRB ,&
&  0.21583E+00_JPRB, 0.19337E+00_JPRB, 0.17707E+00_JPRB, 0.16882E+00_JPRB, 0.16047E+00_JPRB, 0.15593E+00_JPRB, 0.15118E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14911E+00_JPRB, 0.14869E+00_JPRB, 0.14851E+00_JPRB, 0.14909E+00_JPRB, 0.14997E+00_JPRB, 0.14948E+00_JPRB ,&
&  0.14882E+00_JPRB, 0.14857E+00_JPRB, 0.14809E+00_JPRB, 0.14765E+00_JPRB, 0.14671E+00_JPRB, 0.14646E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14344E+00_JPRB, 0.14182E+00_JPRB, 0.14089E+00_JPRB, 0.14085E+00_JPRB, 0.14126E+00_JPRB, 0.14168E+00_JPRB, 0.14242E+00_JPRB ,&
&  0.14382E+00_JPRB, 0.14507E+00_JPRB, 0.14717E+00_JPRB, 0.14911E+00_JPRB, 0.15227E+00_JPRB, 0.15502E+00_JPRB, 0.15885E+00_JPRB ,&
&  0.16294E+00_JPRB, 0.16788E+00_JPRB, 0.17363E+00_JPRB, 0.18013E+00_JPRB, 0.18555E+00_JPRB, 0.19084E+00_JPRB, 0.19655E+00_JPRB ,&
&  0.20099E+00_JPRB, 0.20871E+00_JPRB, 0.21737E+00_JPRB, 0.23177E+00_JPRB, 0.26188E+00_JPRB, 0.28508E+00_JPRB, 0.33318E+00_JPRB ,&
&  0.41229E+00_JPRB, 0.66462E+00_JPRB, 0.89796E+00_JPRB, 0.75269E+00_JPRB, 0.78430E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.68080E+00_JPRB, 0.71193E+00_JPRB, 0.73578E+00_JPRB, 0.73763E+00_JPRB, 0.73607E+00_JPRB, 0.69861E+00_JPRB, 0.64026E+00_JPRB ,&
&  0.53420E+00_JPRB, 0.46477E+00_JPRB, 0.42044E+00_JPRB, 0.37859E+00_JPRB, 0.34080E+00_JPRB, 0.31501E+00_JPRB, 0.29758E+00_JPRB ,&
&  0.28442E+00_JPRB, 0.27715E+00_JPRB, 0.27167E+00_JPRB, 0.26707E+00_JPRB, 0.26205E+00_JPRB, 0.25721E+00_JPRB, 0.24861E+00_JPRB ,&
&  0.24329E+00_JPRB, 0.24005E+00_JPRB, 0.23847E+00_JPRB, 0.24089E+00_JPRB, 0.24782E+00_JPRB, 0.25442E+00_JPRB, 0.26491E+00_JPRB ,&
&  0.27702E+00_JPRB, 0.29135E+00_JPRB, 0.30044E+00_JPRB, 0.30419E+00_JPRB, 0.30736E+00_JPRB, 0.30596E+00_JPRB, 0.28399E+00_JPRB ,&
&  0.28878E+00_JPRB, 0.31043E+00_JPRB, 0.32865E+00_JPRB, 0.33736E+00_JPRB, 0.29181E+00_JPRB, 0.25051E+00_JPRB, 0.20976E+00_JPRB ,&
&  0.19075E+00_JPRB, 0.17497E+00_JPRB, 0.16656E+00_JPRB, 0.16007E+00_JPRB, 0.15498E+00_JPRB, 0.15232E+00_JPRB, 0.14996E+00_JPRB ,&
&  0.14799E+00_JPRB, 0.14734E+00_JPRB, 0.14603E+00_JPRB, 0.14560E+00_JPRB, 0.14473E+00_JPRB, 0.14467E+00_JPRB, 0.14344E+00_JPRB ,&
&  0.14215E+00_JPRB, 0.14109E+00_JPRB, 0.14038E+00_JPRB, 0.13998E+00_JPRB, 0.13903E+00_JPRB, 0.13805E+00_JPRB, 0.13586E+00_JPRB ,&
&  0.13432E+00_JPRB, 0.13283E+00_JPRB, 0.13172E+00_JPRB, 0.13184E+00_JPRB, 0.13248E+00_JPRB, 0.13460E+00_JPRB, 0.13573E+00_JPRB ,&
&  0.13590E+00_JPRB, 0.13593E+00_JPRB, 0.13585E+00_JPRB, 0.13660E+00_JPRB, 0.13774E+00_JPRB, 0.13976E+00_JPRB, 0.14260E+00_JPRB ,&
&  0.14604E+00_JPRB, 0.15137E+00_JPRB, 0.15663E+00_JPRB, 0.16244E+00_JPRB, 0.16780E+00_JPRB, 0.17293E+00_JPRB, 0.17983E+00_JPRB ,&
&  0.18683E+00_JPRB, 0.19522E+00_JPRB, 0.20781E+00_JPRB, 0.22103E+00_JPRB, 0.24498E+00_JPRB, 0.26446E+00_JPRB, 0.29481E+00_JPRB ,&
&  0.36397E+00_JPRB, 0.50950E+00_JPRB, 0.59569E+00_JPRB, 0.60906E+00_JPRB, 0.64269E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.49840E+00_JPRB, 0.51013E+00_JPRB, 0.52533E+00_JPRB, 0.53456E+00_JPRB, 0.54510E+00_JPRB, 0.51265E+00_JPRB, 0.48702E+00_JPRB ,&
&  0.43427E+00_JPRB, 0.40396E+00_JPRB, 0.37871E+00_JPRB, 0.35554E+00_JPRB, 0.33485E+00_JPRB, 0.31612E+00_JPRB, 0.29882E+00_JPRB ,&
&  0.28776E+00_JPRB, 0.28121E+00_JPRB, 0.27235E+00_JPRB, 0.26225E+00_JPRB, 0.25141E+00_JPRB, 0.24192E+00_JPRB, 0.23723E+00_JPRB ,&
&  0.23246E+00_JPRB, 0.22908E+00_JPRB, 0.22769E+00_JPRB, 0.22759E+00_JPRB, 0.22940E+00_JPRB, 0.22945E+00_JPRB, 0.22998E+00_JPRB ,&
&  0.22643E+00_JPRB, 0.23376E+00_JPRB, 0.24349E+00_JPRB, 0.24794E+00_JPRB, 0.25634E+00_JPRB, 0.25516E+00_JPRB, 0.25052E+00_JPRB ,&
&  0.24310E+00_JPRB, 0.23461E+00_JPRB, 0.22483E+00_JPRB, 0.21273E+00_JPRB, 0.20234E+00_JPRB, 0.19479E+00_JPRB, 0.18727E+00_JPRB ,&
&  0.18285E+00_JPRB, 0.17719E+00_JPRB, 0.17026E+00_JPRB, 0.16421E+00_JPRB, 0.15895E+00_JPRB, 0.15470E+00_JPRB, 0.15077E+00_JPRB ,&
&  0.14801E+00_JPRB, 0.14646E+00_JPRB, 0.14496E+00_JPRB, 0.14471E+00_JPRB, 0.14363E+00_JPRB, 0.14248E+00_JPRB, 0.14133E+00_JPRB ,&
&  0.14070E+00_JPRB, 0.13935E+00_JPRB, 0.13826E+00_JPRB, 0.13726E+00_JPRB, 0.13575E+00_JPRB, 0.13445E+00_JPRB, 0.13299E+00_JPRB ,&
&  0.13174E+00_JPRB, 0.13111E+00_JPRB, 0.13073E+00_JPRB, 0.13065E+00_JPRB, 0.13095E+00_JPRB, 0.13089E+00_JPRB, 0.13116E+00_JPRB ,&
&  0.13149E+00_JPRB, 0.13197E+00_JPRB, 0.13191E+00_JPRB, 0.13225E+00_JPRB, 0.13316E+00_JPRB, 0.13443E+00_JPRB, 0.13758E+00_JPRB ,&
&  0.14117E+00_JPRB, 0.14577E+00_JPRB, 0.15073E+00_JPRB, 0.15628E+00_JPRB, 0.16184E+00_JPRB, 0.16728E+00_JPRB, 0.17327E+00_JPRB ,&
&  0.18107E+00_JPRB, 0.18815E+00_JPRB, 0.19652E+00_JPRB, 0.20933E+00_JPRB, 0.23778E+00_JPRB, 0.25282E+00_JPRB, 0.26706E+00_JPRB ,&
&  0.29237E+00_JPRB, 0.32591E+00_JPRB, 0.42964E+00_JPRB, 0.46199E+00_JPRB, 0.47946E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.24290E+00_JPRB, 0.24276E+00_JPRB, 0.24320E+00_JPRB, 0.24921E+00_JPRB, 0.25884E+00_JPRB, 0.33643E+00_JPRB, 0.32706E+00_JPRB ,&
&  0.31947E+00_JPRB, 0.29161E+00_JPRB, 0.28840E+00_JPRB, 0.28155E+00_JPRB, 0.27218E+00_JPRB, 0.26485E+00_JPRB, 0.26064E+00_JPRB ,&
&  0.25596E+00_JPRB, 0.25071E+00_JPRB, 0.24798E+00_JPRB, 0.24492E+00_JPRB, 0.24187E+00_JPRB, 0.23828E+00_JPRB, 0.23551E+00_JPRB ,&
&  0.23282E+00_JPRB, 0.23095E+00_JPRB, 0.22890E+00_JPRB, 0.22469E+00_JPRB, 0.22289E+00_JPRB, 0.21944E+00_JPRB, 0.21434E+00_JPRB ,&
&  0.20763E+00_JPRB, 0.20416E+00_JPRB, 0.20032E+00_JPRB, 0.19691E+00_JPRB, 0.20024E+00_JPRB, 0.19946E+00_JPRB, 0.19738E+00_JPRB ,&
&  0.19377E+00_JPRB, 0.19003E+00_JPRB, 0.18511E+00_JPRB, 0.18039E+00_JPRB, 0.17650E+00_JPRB, 0.17333E+00_JPRB, 0.17062E+00_JPRB ,&
&  0.16848E+00_JPRB, 0.16670E+00_JPRB, 0.16411E+00_JPRB, 0.16175E+00_JPRB, 0.15925E+00_JPRB, 0.15706E+00_JPRB, 0.15476E+00_JPRB ,&
&  0.15293E+00_JPRB, 0.15122E+00_JPRB, 0.14944E+00_JPRB, 0.14763E+00_JPRB, 0.14619E+00_JPRB, 0.14431E+00_JPRB, 0.14290E+00_JPRB ,&
&  0.14168E+00_JPRB, 0.14011E+00_JPRB, 0.13894E+00_JPRB, 0.13751E+00_JPRB, 0.13646E+00_JPRB, 0.13557E+00_JPRB, 0.13472E+00_JPRB ,&
&  0.13360E+00_JPRB, 0.13284E+00_JPRB, 0.13256E+00_JPRB, 0.13234E+00_JPRB, 0.13200E+00_JPRB, 0.13206E+00_JPRB, 0.13241E+00_JPRB ,&
&  0.13290E+00_JPRB, 0.13349E+00_JPRB, 0.13445E+00_JPRB, 0.13556E+00_JPRB, 0.13723E+00_JPRB, 0.13946E+00_JPRB, 0.14298E+00_JPRB ,&
&  0.14680E+00_JPRB, 0.15102E+00_JPRB, 0.15529E+00_JPRB, 0.15990E+00_JPRB, 0.16463E+00_JPRB, 0.16938E+00_JPRB, 0.17483E+00_JPRB ,&
&  0.18075E+00_JPRB, 0.18540E+00_JPRB, 0.18980E+00_JPRB, 0.19447E+00_JPRB, 0.20456E+00_JPRB, 0.21201E+00_JPRB, 0.21755E+00_JPRB ,&
&  0.22383E+00_JPRB, 0.23247E+00_JPRB, 0.23707E+00_JPRB, 0.24010E+00_JPRB, 0.24169E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.21498E+00_JPRB, 0.21780E+00_JPRB, 0.21970E+00_JPRB, 0.22090E+00_JPRB, 0.22060E+00_JPRB, 0.22152E+00_JPRB, 0.22246E+00_JPRB ,&
&  0.22371E+00_JPRB, 0.22533E+00_JPRB, 0.22651E+00_JPRB, 0.22757E+00_JPRB, 0.22807E+00_JPRB, 0.22809E+00_JPRB, 0.22811E+00_JPRB ,&
&  0.22791E+00_JPRB, 0.22766E+00_JPRB, 0.22380E+00_JPRB, 0.22033E+00_JPRB, 0.21781E+00_JPRB, 0.21633E+00_JPRB, 0.21506E+00_JPRB ,&
&  0.21403E+00_JPRB, 0.21273E+00_JPRB, 0.21127E+00_JPRB, 0.20914E+00_JPRB, 0.20579E+00_JPRB, 0.20252E+00_JPRB, 0.19913E+00_JPRB ,&
&  0.19607E+00_JPRB, 0.19363E+00_JPRB, 0.19137E+00_JPRB, 0.18911E+00_JPRB, 0.18782E+00_JPRB, 0.18630E+00_JPRB, 0.18471E+00_JPRB ,&
&  0.18263E+00_JPRB, 0.18038E+00_JPRB, 0.17825E+00_JPRB, 0.17606E+00_JPRB, 0.17395E+00_JPRB, 0.17188E+00_JPRB, 0.17036E+00_JPRB ,&
&  0.16878E+00_JPRB, 0.16760E+00_JPRB, 0.16629E+00_JPRB, 0.16509E+00_JPRB, 0.16384E+00_JPRB, 0.16264E+00_JPRB, 0.16123E+00_JPRB ,&
&  0.16025E+00_JPRB, 0.15924E+00_JPRB, 0.15820E+00_JPRB, 0.15725E+00_JPRB, 0.15630E+00_JPRB, 0.15537E+00_JPRB, 0.15433E+00_JPRB ,&
&  0.15336E+00_JPRB, 0.15274E+00_JPRB, 0.15209E+00_JPRB, 0.15137E+00_JPRB, 0.15070E+00_JPRB, 0.15024E+00_JPRB, 0.14984E+00_JPRB ,&
&  0.14948E+00_JPRB, 0.14945E+00_JPRB, 0.14962E+00_JPRB, 0.14979E+00_JPRB, 0.14997E+00_JPRB, 0.15038E+00_JPRB, 0.15099E+00_JPRB ,&
&  0.15171E+00_JPRB, 0.15249E+00_JPRB, 0.15348E+00_JPRB, 0.15490E+00_JPRB, 0.15647E+00_JPRB, 0.15827E+00_JPRB, 0.16024E+00_JPRB ,&
&  0.16237E+00_JPRB, 0.16464E+00_JPRB, 0.16703E+00_JPRB, 0.16932E+00_JPRB, 0.17184E+00_JPRB, 0.17432E+00_JPRB, 0.17673E+00_JPRB ,&
&  0.17925E+00_JPRB, 0.18177E+00_JPRB, 0.18439E+00_JPRB, 0.18680E+00_JPRB, 0.19059E+00_JPRB, 0.19307E+00_JPRB, 0.19562E+00_JPRB ,&
&  0.19793E+00_JPRB, 0.20062E+00_JPRB, 0.20333E+00_JPRB, 0.20667E+00_JPRB, 0.21001E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB, 0.17834E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB ,&
&  0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB, 0.45250E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.48132E+00_JPRB, 0.47966E+00_JPRB, 0.47771E+00_JPRB, 0.47599E+00_JPRB, 0.47386E+00_JPRB, 0.47214E+00_JPRB, 0.47043E+00_JPRB ,&
&  0.46889E+00_JPRB, 0.46719E+00_JPRB, 0.46566E+00_JPRB, 0.46382E+00_JPRB, 0.46209E+00_JPRB, 0.46046E+00_JPRB, 0.45893E+00_JPRB ,&
&  0.45737E+00_JPRB, 0.45593E+00_JPRB, 0.45447E+00_JPRB, 0.45288E+00_JPRB, 0.45156E+00_JPRB, 0.44999E+00_JPRB, 0.44788E+00_JPRB ,&
&  0.44447E+00_JPRB, 0.44081E+00_JPRB, 0.43758E+00_JPRB, 0.43532E+00_JPRB, 0.43364E+00_JPRB, 0.43225E+00_JPRB, 0.43101E+00_JPRB ,&
&  0.43015E+00_JPRB, 0.42969E+00_JPRB, 0.42942E+00_JPRB, 0.42898E+00_JPRB, 0.42890E+00_JPRB, 0.42977E+00_JPRB, 0.43023E+00_JPRB ,&
&  0.43037E+00_JPRB, 0.43056E+00_JPRB, 0.43140E+00_JPRB, 0.43208E+00_JPRB, 0.43274E+00_JPRB, 0.43361E+00_JPRB, 0.43524E+00_JPRB ,&
&  0.43686E+00_JPRB, 0.43835E+00_JPRB, 0.44006E+00_JPRB, 0.44183E+00_JPRB, 0.44360E+00_JPRB, 0.44539E+00_JPRB, 0.44775E+00_JPRB ,&
&  0.45032E+00_JPRB, 0.45259E+00_JPRB, 0.45457E+00_JPRB, 0.45616E+00_JPRB, 0.45750E+00_JPRB, 0.45818E+00_JPRB, 0.45861E+00_JPRB ,&
&  0.45846E+00_JPRB, 0.45904E+00_JPRB, 0.45908E+00_JPRB, 0.45918E+00_JPRB, 0.45882E+00_JPRB, 0.45939E+00_JPRB, 0.45966E+00_JPRB ,&
&  0.46062E+00_JPRB, 0.46117E+00_JPRB, 0.46230E+00_JPRB, 0.46337E+00_JPRB, 0.46515E+00_JPRB, 0.46720E+00_JPRB, 0.46947E+00_JPRB ,&
&  0.47200E+00_JPRB, 0.47482E+00_JPRB, 0.47808E+00_JPRB, 0.47997E+00_JPRB, 0.48187E+00_JPRB, 0.48377E+00_JPRB, 0.48588E+00_JPRB ,&
&  0.48745E+00_JPRB, 0.48893E+00_JPRB, 0.49050E+00_JPRB, 0.49170E+00_JPRB, 0.49235E+00_JPRB, 0.49281E+00_JPRB, 0.49328E+00_JPRB ,&
&  0.49336E+00_JPRB, 0.49337E+00_JPRB, 0.49325E+00_JPRB, 0.49321E+00_JPRB, 0.49247E+00_JPRB, 0.49138E+00_JPRB, 0.49010E+00_JPRB ,&
&  0.48903E+00_JPRB, 0.48756E+00_JPRB, 0.48614E+00_JPRB, 0.48461E+00_JPRB, 0.48314E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.53791E+00_JPRB, 0.53490E+00_JPRB, 0.53173E+00_JPRB, 0.52882E+00_JPRB, 0.52473E+00_JPRB, 0.52058E+00_JPRB, 0.51630E+00_JPRB ,&
&  0.51195E+00_JPRB, 0.50683E+00_JPRB, 0.50139E+00_JPRB, 0.49567E+00_JPRB, 0.48989E+00_JPRB, 0.48456E+00_JPRB, 0.48015E+00_JPRB ,&
&  0.47616E+00_JPRB, 0.47191E+00_JPRB, 0.46822E+00_JPRB, 0.46344E+00_JPRB, 0.45859E+00_JPRB, 0.45431E+00_JPRB, 0.45138E+00_JPRB ,&
&  0.44661E+00_JPRB, 0.44099E+00_JPRB, 0.43505E+00_JPRB, 0.42853E+00_JPRB, 0.42240E+00_JPRB, 0.41629E+00_JPRB, 0.41016E+00_JPRB ,&
&  0.40431E+00_JPRB, 0.39987E+00_JPRB, 0.39595E+00_JPRB, 0.39202E+00_JPRB, 0.38815E+00_JPRB, 0.38715E+00_JPRB, 0.38599E+00_JPRB ,&
&  0.38500E+00_JPRB, 0.38495E+00_JPRB, 0.38788E+00_JPRB, 0.39056E+00_JPRB, 0.39335E+00_JPRB, 0.39623E+00_JPRB, 0.40106E+00_JPRB ,&
&  0.40555E+00_JPRB, 0.41089E+00_JPRB, 0.41760E+00_JPRB, 0.42414E+00_JPRB, 0.42939E+00_JPRB, 0.43319E+00_JPRB, 0.43822E+00_JPRB ,&
&  0.44171E+00_JPRB, 0.44631E+00_JPRB, 0.44943E+00_JPRB, 0.45079E+00_JPRB, 0.45043E+00_JPRB, 0.45019E+00_JPRB, 0.45071E+00_JPRB ,&
&  0.45055E+00_JPRB, 0.45279E+00_JPRB, 0.45495E+00_JPRB, 0.45719E+00_JPRB, 0.45797E+00_JPRB, 0.46120E+00_JPRB, 0.46382E+00_JPRB ,&
&  0.46858E+00_JPRB, 0.47516E+00_JPRB, 0.48200E+00_JPRB, 0.48952E+00_JPRB, 0.49875E+00_JPRB, 0.50948E+00_JPRB, 0.51813E+00_JPRB ,&
&  0.52641E+00_JPRB, 0.53338E+00_JPRB, 0.53907E+00_JPRB, 0.54213E+00_JPRB, 0.54502E+00_JPRB, 0.54783E+00_JPRB, 0.55117E+00_JPRB ,&
&  0.55269E+00_JPRB, 0.55447E+00_JPRB, 0.55631E+00_JPRB, 0.55774E+00_JPRB, 0.55724E+00_JPRB, 0.55656E+00_JPRB, 0.55637E+00_JPRB ,&
&  0.55595E+00_JPRB, 0.55518E+00_JPRB, 0.55453E+00_JPRB, 0.55434E+00_JPRB, 0.55357E+00_JPRB, 0.55239E+00_JPRB, 0.55116E+00_JPRB ,&
&  0.55000E+00_JPRB, 0.54801E+00_JPRB, 0.54558E+00_JPRB, 0.54315E+00_JPRB, 0.54096E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.55976E+00_JPRB, 0.55895E+00_JPRB, 0.55802E+00_JPRB, 0.55691E+00_JPRB, 0.55445E+00_JPRB, 0.55160E+00_JPRB, 0.54795E+00_JPRB ,&
&  0.54359E+00_JPRB, 0.53739E+00_JPRB, 0.53091E+00_JPRB, 0.52400E+00_JPRB, 0.51749E+00_JPRB, 0.51194E+00_JPRB, 0.50672E+00_JPRB ,&
&  0.50114E+00_JPRB, 0.49385E+00_JPRB, 0.48536E+00_JPRB, 0.47428E+00_JPRB, 0.46398E+00_JPRB, 0.45528E+00_JPRB, 0.44577E+00_JPRB ,&
&  0.43195E+00_JPRB, 0.41660E+00_JPRB, 0.40288E+00_JPRB, 0.38910E+00_JPRB, 0.37895E+00_JPRB, 0.35322E+00_JPRB, 0.34477E+00_JPRB ,&
&  0.33884E+00_JPRB, 0.33275E+00_JPRB, 0.32983E+00_JPRB, 0.32759E+00_JPRB, 0.32790E+00_JPRB, 0.32363E+00_JPRB, 0.32129E+00_JPRB ,&
&  0.30994E+00_JPRB, 0.31192E+00_JPRB, 0.31512E+00_JPRB, 0.31906E+00_JPRB, 0.32687E+00_JPRB, 0.33579E+00_JPRB, 0.34972E+00_JPRB ,&
&  0.37460E+00_JPRB, 0.38568E+00_JPRB, 0.40241E+00_JPRB, 0.41546E+00_JPRB, 0.42330E+00_JPRB, 0.42764E+00_JPRB, 0.43258E+00_JPRB ,&
&  0.43787E+00_JPRB, 0.44566E+00_JPRB, 0.45032E+00_JPRB, 0.45351E+00_JPRB, 0.45335E+00_JPRB, 0.45204E+00_JPRB, 0.45156E+00_JPRB ,&
&  0.45043E+00_JPRB, 0.45251E+00_JPRB, 0.45649E+00_JPRB, 0.46475E+00_JPRB, 0.47624E+00_JPRB, 0.49126E+00_JPRB, 0.50710E+00_JPRB ,&
&  0.52324E+00_JPRB, 0.54107E+00_JPRB, 0.55609E+00_JPRB, 0.57034E+00_JPRB, 0.58527E+00_JPRB, 0.59756E+00_JPRB, 0.60789E+00_JPRB ,&
&  0.61493E+00_JPRB, 0.61723E+00_JPRB, 0.61447E+00_JPRB, 0.61085E+00_JPRB, 0.60603E+00_JPRB, 0.60248E+00_JPRB, 0.60186E+00_JPRB ,&
&  0.60131E+00_JPRB, 0.60165E+00_JPRB, 0.60256E+00_JPRB, 0.60057E+00_JPRB, 0.59798E+00_JPRB, 0.59475E+00_JPRB, 0.59159E+00_JPRB ,&
&  0.58773E+00_JPRB, 0.58444E+00_JPRB, 0.58139E+00_JPRB, 0.57850E+00_JPRB, 0.57600E+00_JPRB, 0.57248E+00_JPRB, 0.56954E+00_JPRB ,&
&  0.56651E+00_JPRB, 0.56542E+00_JPRB, 0.56352E+00_JPRB, 0.56206E+00_JPRB, 0.56059E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.52563E+00_JPRB, 0.52566E+00_JPRB, 0.52654E+00_JPRB, 0.52692E+00_JPRB, 0.52832E+00_JPRB, 0.52920E+00_JPRB, 0.53055E+00_JPRB ,&
&  0.53117E+00_JPRB, 0.53182E+00_JPRB, 0.53124E+00_JPRB, 0.52994E+00_JPRB, 0.52747E+00_JPRB, 0.52506E+00_JPRB, 0.52225E+00_JPRB ,&
&  0.51809E+00_JPRB, 0.50945E+00_JPRB, 0.49717E+00_JPRB, 0.48327E+00_JPRB, 0.46729E+00_JPRB, 0.44984E+00_JPRB, 0.42992E+00_JPRB ,&
&  0.40910E+00_JPRB, 0.38953E+00_JPRB, 0.36562E+00_JPRB, 0.35543E+00_JPRB, 0.34035E+00_JPRB, 0.32852E+00_JPRB, 0.31984E+00_JPRB ,&
&  0.30949E+00_JPRB, 0.30227E+00_JPRB, 0.29502E+00_JPRB, 0.28790E+00_JPRB, 0.28372E+00_JPRB, 0.27626E+00_JPRB, 0.27283E+00_JPRB ,&
&  0.27046E+00_JPRB, 0.26941E+00_JPRB, 0.26976E+00_JPRB, 0.27189E+00_JPRB, 0.27662E+00_JPRB, 0.28265E+00_JPRB, 0.29426E+00_JPRB ,&
&  0.32035E+00_JPRB, 0.35757E+00_JPRB, 0.41444E+00_JPRB, 0.42587E+00_JPRB, 0.42537E+00_JPRB, 0.43365E+00_JPRB, 0.44274E+00_JPRB ,&
&  0.45154E+00_JPRB, 0.45941E+00_JPRB, 0.46570E+00_JPRB, 0.46882E+00_JPRB, 0.47273E+00_JPRB, 0.47609E+00_JPRB, 0.47987E+00_JPRB ,&
&  0.48380E+00_JPRB, 0.49309E+00_JPRB, 0.50641E+00_JPRB, 0.52236E+00_JPRB, 0.53943E+00_JPRB, 0.55379E+00_JPRB, 0.56662E+00_JPRB ,&
&  0.57702E+00_JPRB, 0.58858E+00_JPRB, 0.60464E+00_JPRB, 0.62197E+00_JPRB, 0.63971E+00_JPRB, 0.64973E+00_JPRB, 0.65818E+00_JPRB ,&
&  0.66327E+00_JPRB, 0.66254E+00_JPRB, 0.65591E+00_JPRB, 0.64780E+00_JPRB, 0.64162E+00_JPRB, 0.64466E+00_JPRB, 0.64313E+00_JPRB ,&
&  0.64190E+00_JPRB, 0.63560E+00_JPRB, 0.62571E+00_JPRB, 0.61346E+00_JPRB, 0.60439E+00_JPRB, 0.59620E+00_JPRB, 0.59099E+00_JPRB ,&
&  0.58627E+00_JPRB, 0.58148E+00_JPRB, 0.57516E+00_JPRB, 0.56691E+00_JPRB, 0.55836E+00_JPRB, 0.54933E+00_JPRB, 0.54222E+00_JPRB ,&
&  0.53589E+00_JPRB, 0.53208E+00_JPRB, 0.52886E+00_JPRB, 0.52722E+00_JPRB, 0.52567E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.50402E+00_JPRB, 0.50844E+00_JPRB, 0.51309E+00_JPRB, 0.51644E+00_JPRB, 0.51927E+00_JPRB, 0.52144E+00_JPRB, 0.52223E+00_JPRB ,&
&  0.52255E+00_JPRB, 0.52250E+00_JPRB, 0.52280E+00_JPRB, 0.52285E+00_JPRB, 0.52318E+00_JPRB, 0.52408E+00_JPRB, 0.52464E+00_JPRB ,&
&  0.52497E+00_JPRB, 0.51871E+00_JPRB, 0.50566E+00_JPRB, 0.49067E+00_JPRB, 0.47441E+00_JPRB, 0.45388E+00_JPRB, 0.42890E+00_JPRB ,&
&  0.40620E+00_JPRB, 0.38950E+00_JPRB, 0.37174E+00_JPRB, 0.35990E+00_JPRB, 0.34717E+00_JPRB, 0.33966E+00_JPRB, 0.32916E+00_JPRB ,&
&  0.32501E+00_JPRB, 0.30180E+00_JPRB, 0.29882E+00_JPRB, 0.28416E+00_JPRB, 0.27329E+00_JPRB, 0.26433E+00_JPRB, 0.25777E+00_JPRB ,&
&  0.25604E+00_JPRB, 0.25357E+00_JPRB, 0.25364E+00_JPRB, 0.25689E+00_JPRB, 0.26099E+00_JPRB, 0.26977E+00_JPRB, 0.27971E+00_JPRB ,&
&  0.30264E+00_JPRB, 0.37839E+00_JPRB, 0.40691E+00_JPRB, 0.41622E+00_JPRB, 0.42231E+00_JPRB, 0.43647E+00_JPRB, 0.44979E+00_JPRB ,&
&  0.46707E+00_JPRB, 0.47812E+00_JPRB, 0.48828E+00_JPRB, 0.49388E+00_JPRB, 0.51000E+00_JPRB, 0.52433E+00_JPRB, 0.53694E+00_JPRB ,&
&  0.55035E+00_JPRB, 0.56362E+00_JPRB, 0.57902E+00_JPRB, 0.58860E+00_JPRB, 0.58959E+00_JPRB, 0.58787E+00_JPRB, 0.58581E+00_JPRB ,&
&  0.58884E+00_JPRB, 0.59396E+00_JPRB, 0.61347E+00_JPRB, 0.63967E+00_JPRB, 0.66909E+00_JPRB, 0.68930E+00_JPRB, 0.69940E+00_JPRB ,&
&  0.70594E+00_JPRB, 0.70867E+00_JPRB, 0.71457E+00_JPRB, 0.72408E+00_JPRB, 0.71678E+00_JPRB, 0.70196E+00_JPRB, 0.69300E+00_JPRB ,&
&  0.68418E+00_JPRB, 0.66359E+00_JPRB, 0.64663E+00_JPRB, 0.63161E+00_JPRB, 0.61997E+00_JPRB, 0.60986E+00_JPRB, 0.60031E+00_JPRB ,&
&  0.58849E+00_JPRB, 0.57615E+00_JPRB, 0.56205E+00_JPRB, 0.54440E+00_JPRB, 0.52910E+00_JPRB, 0.51355E+00_JPRB, 0.50438E+00_JPRB ,&
&  0.49678E+00_JPRB, 0.49507E+00_JPRB, 0.49352E+00_JPRB, 0.49643E+00_JPRB, 0.49855E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.48883E+00_JPRB, 0.49949E+00_JPRB, 0.50831E+00_JPRB, 0.51408E+00_JPRB, 0.51783E+00_JPRB, 0.52047E+00_JPRB, 0.52029E+00_JPRB ,&
&  0.51990E+00_JPRB, 0.51843E+00_JPRB, 0.51817E+00_JPRB, 0.51746E+00_JPRB, 0.51811E+00_JPRB, 0.51771E+00_JPRB, 0.51746E+00_JPRB ,&
&  0.51745E+00_JPRB, 0.51601E+00_JPRB, 0.50947E+00_JPRB, 0.49661E+00_JPRB, 0.48028E+00_JPRB, 0.46212E+00_JPRB, 0.43624E+00_JPRB ,&
&  0.41054E+00_JPRB, 0.39395E+00_JPRB, 0.37894E+00_JPRB, 0.36672E+00_JPRB, 0.35942E+00_JPRB, 0.35020E+00_JPRB, 0.34808E+00_JPRB ,&
&  0.34548E+00_JPRB, 0.33665E+00_JPRB, 0.32555E+00_JPRB, 0.31394E+00_JPRB, 0.30198E+00_JPRB, 0.27101E+00_JPRB, 0.26707E+00_JPRB ,&
&  0.25928E+00_JPRB, 0.25220E+00_JPRB, 0.25047E+00_JPRB, 0.25160E+00_JPRB, 0.25472E+00_JPRB, 0.26454E+00_JPRB, 0.27694E+00_JPRB ,&
&  0.29496E+00_JPRB, 0.38203E+00_JPRB, 0.40199E+00_JPRB, 0.41393E+00_JPRB, 0.43313E+00_JPRB, 0.45298E+00_JPRB, 0.46949E+00_JPRB ,&
&  0.48472E+00_JPRB, 0.49757E+00_JPRB, 0.51452E+00_JPRB, 0.53036E+00_JPRB, 0.54798E+00_JPRB, 0.56419E+00_JPRB, 0.57583E+00_JPRB ,&
&  0.58628E+00_JPRB, 0.58834E+00_JPRB, 0.61031E+00_JPRB, 0.61843E+00_JPRB, 0.61338E+00_JPRB, 0.62880E+00_JPRB, 0.63688E+00_JPRB ,&
&  0.63437E+00_JPRB, 0.62727E+00_JPRB, 0.64147E+00_JPRB, 0.67723E+00_JPRB, 0.72799E+00_JPRB, 0.76760E+00_JPRB, 0.77896E+00_JPRB ,&
&  0.78306E+00_JPRB, 0.77798E+00_JPRB, 0.77108E+00_JPRB, 0.76980E+00_JPRB, 0.74889E+00_JPRB, 0.71854E+00_JPRB, 0.69728E+00_JPRB ,&
&  0.67950E+00_JPRB, 0.67689E+00_JPRB, 0.66689E+00_JPRB, 0.65556E+00_JPRB, 0.64193E+00_JPRB, 0.62838E+00_JPRB, 0.60871E+00_JPRB ,&
&  0.58069E+00_JPRB, 0.55001E+00_JPRB, 0.52573E+00_JPRB, 0.50453E+00_JPRB, 0.49236E+00_JPRB, 0.47896E+00_JPRB, 0.47255E+00_JPRB ,&
&  0.46640E+00_JPRB, 0.46616E+00_JPRB, 0.46631E+00_JPRB, 0.47156E+00_JPRB, 0.47800E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.48734E+00_JPRB, 0.50367E+00_JPRB, 0.51745E+00_JPRB, 0.52578E+00_JPRB, 0.52757E+00_JPRB, 0.52836E+00_JPRB, 0.52701E+00_JPRB ,&
&  0.52635E+00_JPRB, 0.52545E+00_JPRB, 0.52400E+00_JPRB, 0.52050E+00_JPRB, 0.51713E+00_JPRB, 0.51267E+00_JPRB, 0.51010E+00_JPRB ,&
&  0.50686E+00_JPRB, 0.49886E+00_JPRB, 0.49328E+00_JPRB, 0.48724E+00_JPRB, 0.47665E+00_JPRB, 0.46223E+00_JPRB, 0.44102E+00_JPRB ,&
&  0.41677E+00_JPRB, 0.39847E+00_JPRB, 0.38592E+00_JPRB, 0.37374E+00_JPRB, 0.36392E+00_JPRB, 0.35784E+00_JPRB, 0.35464E+00_JPRB ,&
&  0.35372E+00_JPRB, 0.34852E+00_JPRB, 0.34487E+00_JPRB, 0.32970E+00_JPRB, 0.31076E+00_JPRB, 0.30375E+00_JPRB, 0.28574E+00_JPRB ,&
&  0.26893E+00_JPRB, 0.25622E+00_JPRB, 0.25322E+00_JPRB, 0.25355E+00_JPRB, 0.25859E+00_JPRB, 0.26941E+00_JPRB, 0.29334E+00_JPRB ,&
&  0.33363E+00_JPRB, 0.40309E+00_JPRB, 0.41833E+00_JPRB, 0.43692E+00_JPRB, 0.45709E+00_JPRB, 0.47648E+00_JPRB, 0.49038E+00_JPRB ,&
&  0.50099E+00_JPRB, 0.51123E+00_JPRB, 0.53000E+00_JPRB, 0.54930E+00_JPRB, 0.58167E+00_JPRB, 0.61311E+00_JPRB, 0.63988E+00_JPRB ,&
&  0.66519E+00_JPRB, 0.63907E+00_JPRB, 0.64413E+00_JPRB, 0.65073E+00_JPRB, 0.67643E+00_JPRB, 0.69844E+00_JPRB, 0.69273E+00_JPRB ,&
&  0.69197E+00_JPRB, 0.72073E+00_JPRB, 0.72496E+00_JPRB, 0.74502E+00_JPRB, 0.78975E+00_JPRB, 0.83859E+00_JPRB, 0.85791E+00_JPRB ,&
&  0.84139E+00_JPRB, 0.84239E+00_JPRB, 0.88647E+00_JPRB, 0.84578E+00_JPRB, 0.82066E+00_JPRB, 0.78023E+00_JPRB, 0.73235E+00_JPRB ,&
&  0.70643E+00_JPRB, 0.69228E+00_JPRB, 0.68573E+00_JPRB, 0.67735E+00_JPRB, 0.66271E+00_JPRB, 0.64090E+00_JPRB, 0.60275E+00_JPRB ,&
&  0.55462E+00_JPRB, 0.52035E+00_JPRB, 0.49610E+00_JPRB, 0.48032E+00_JPRB, 0.46911E+00_JPRB, 0.45691E+00_JPRB, 0.44786E+00_JPRB ,&
&  0.44140E+00_JPRB, 0.44217E+00_JPRB, 0.44863E+00_JPRB, 0.46174E+00_JPRB, 0.47520E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.50637E+00_JPRB, 0.52318E+00_JPRB, 0.53307E+00_JPRB, 0.53265E+00_JPRB, 0.52380E+00_JPRB, 0.50927E+00_JPRB, 0.49833E+00_JPRB ,&
&  0.49492E+00_JPRB, 0.49583E+00_JPRB, 0.49825E+00_JPRB, 0.49585E+00_JPRB, 0.49433E+00_JPRB, 0.49326E+00_JPRB, 0.49353E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.49349E+00_JPRB, 0.49268E+00_JPRB, 0.48509E+00_JPRB, 0.47393E+00_JPRB, 0.45947E+00_JPRB, 0.44235E+00_JPRB ,&
&  0.42379E+00_JPRB, 0.40874E+00_JPRB, 0.39623E+00_JPRB, 0.38642E+00_JPRB, 0.37561E+00_JPRB, 0.36965E+00_JPRB, 0.36607E+00_JPRB ,&
&  0.36361E+00_JPRB, 0.36349E+00_JPRB, 0.35939E+00_JPRB, 0.35157E+00_JPRB, 0.33592E+00_JPRB, 0.31715E+00_JPRB, 0.29737E+00_JPRB ,&
&  0.28153E+00_JPRB, 0.27183E+00_JPRB, 0.26640E+00_JPRB, 0.26571E+00_JPRB, 0.27364E+00_JPRB, 0.29116E+00_JPRB, 0.32774E+00_JPRB ,&
&  0.39654E+00_JPRB, 0.42285E+00_JPRB, 0.43661E+00_JPRB, 0.45365E+00_JPRB, 0.46925E+00_JPRB, 0.48137E+00_JPRB, 0.49219E+00_JPRB ,&
&  0.50056E+00_JPRB, 0.50873E+00_JPRB, 0.52137E+00_JPRB, 0.53291E+00_JPRB, 0.55575E+00_JPRB, 0.57343E+00_JPRB, 0.63748E+00_JPRB ,&
&  0.76575E+00_JPRB, 0.77151E+00_JPRB, 0.70902E+00_JPRB, 0.71876E+00_JPRB, 0.74867E+00_JPRB, 0.77813E+00_JPRB, 0.80900E+00_JPRB ,&
&  0.83251E+00_JPRB, 0.84131E+00_JPRB, 0.84593E+00_JPRB, 0.84820E+00_JPRB, 0.86534E+00_JPRB, 0.88538E+00_JPRB, 0.88996E+00_JPRB ,&
&  0.88334E+00_JPRB, 0.90890E+00_JPRB, 0.96557E+00_JPRB, 0.81590E+00_JPRB, 0.74847E+00_JPRB, 0.73019E+00_JPRB, 0.70263E+00_JPRB ,&
&  0.68549E+00_JPRB, 0.68227E+00_JPRB, 0.69216E+00_JPRB, 0.69495E+00_JPRB, 0.65480E+00_JPRB, 0.60055E+00_JPRB, 0.54380E+00_JPRB ,&
&  0.51252E+00_JPRB, 0.49644E+00_JPRB, 0.47907E+00_JPRB, 0.46752E+00_JPRB, 0.45749E+00_JPRB, 0.44574E+00_JPRB, 0.43260E+00_JPRB ,&
&  0.43211E+00_JPRB, 0.44010E+00_JPRB, 0.45300E+00_JPRB, 0.47631E+00_JPRB, 0.49336E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.50326E+00_JPRB, 0.52690E+00_JPRB, 0.53247E+00_JPRB, 0.51889E+00_JPRB, 0.50327E+00_JPRB, 0.47400E+00_JPRB, 0.44805E+00_JPRB ,&
&  0.43750E+00_JPRB, 0.43215E+00_JPRB, 0.43185E+00_JPRB, 0.43240E+00_JPRB, 0.43533E+00_JPRB, 0.44095E+00_JPRB, 0.44686E+00_JPRB ,&
&  0.44967E+00_JPRB, 0.45189E+00_JPRB, 0.45495E+00_JPRB, 0.45860E+00_JPRB, 0.45629E+00_JPRB, 0.44595E+00_JPRB, 0.43342E+00_JPRB ,&
&  0.41662E+00_JPRB, 0.40466E+00_JPRB, 0.39591E+00_JPRB, 0.38893E+00_JPRB, 0.38421E+00_JPRB, 0.38285E+00_JPRB, 0.38382E+00_JPRB ,&
&  0.38244E+00_JPRB, 0.38116E+00_JPRB, 0.38073E+00_JPRB, 0.37599E+00_JPRB, 0.36158E+00_JPRB, 0.33993E+00_JPRB, 0.31462E+00_JPRB ,&
&  0.29752E+00_JPRB, 0.29102E+00_JPRB, 0.29321E+00_JPRB, 0.30403E+00_JPRB, 0.33449E+00_JPRB, 0.39091E+00_JPRB, 0.41474E+00_JPRB ,&
&  0.43507E+00_JPRB, 0.44774E+00_JPRB, 0.45941E+00_JPRB, 0.47447E+00_JPRB, 0.48697E+00_JPRB, 0.49618E+00_JPRB, 0.50483E+00_JPRB ,&
&  0.51766E+00_JPRB, 0.53095E+00_JPRB, 0.54930E+00_JPRB, 0.51421E+00_JPRB, 0.52783E+00_JPRB, 0.61533E+00_JPRB, 0.67432E+00_JPRB ,&
&  0.75389E+00_JPRB, 0.77546E+00_JPRB, 0.78350E+00_JPRB, 0.84723E+00_JPRB, 0.87673E+00_JPRB, 0.91847E+00_JPRB, 0.92978E+00_JPRB ,&
&  0.92622E+00_JPRB, 0.91111E+00_JPRB, 0.89406E+00_JPRB, 0.87934E+00_JPRB, 0.89102E+00_JPRB, 0.92074E+00_JPRB, 0.90914E+00_JPRB ,&
&  0.89569E+00_JPRB, 0.88592E+00_JPRB, 0.81713E+00_JPRB, 0.71922E+00_JPRB, 0.68416E+00_JPRB, 0.67894E+00_JPRB, 0.68673E+00_JPRB ,&
&  0.68317E+00_JPRB, 0.68714E+00_JPRB, 0.69262E+00_JPRB, 0.68849E+00_JPRB, 0.61926E+00_JPRB, 0.55371E+00_JPRB, 0.49894E+00_JPRB ,&
&  0.47030E+00_JPRB, 0.46488E+00_JPRB, 0.45533E+00_JPRB, 0.45577E+00_JPRB, 0.44877E+00_JPRB, 0.44142E+00_JPRB, 0.43165E+00_JPRB ,&
&  0.43017E+00_JPRB, 0.43579E+00_JPRB, 0.45411E+00_JPRB, 0.47599E+00_JPRB, 0.48992E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.53406E+00_JPRB, 0.53859E+00_JPRB, 0.52274E+00_JPRB, 0.51130E+00_JPRB, 0.49703E+00_JPRB, 0.48067E+00_JPRB, 0.46167E+00_JPRB ,&
&  0.45907E+00_JPRB, 0.43731E+00_JPRB, 0.42074E+00_JPRB, 0.40650E+00_JPRB, 0.39859E+00_JPRB, 0.39502E+00_JPRB, 0.40064E+00_JPRB ,&
&  0.41335E+00_JPRB, 0.42992E+00_JPRB, 0.44031E+00_JPRB, 0.44101E+00_JPRB, 0.43644E+00_JPRB, 0.42515E+00_JPRB, 0.41673E+00_JPRB ,&
&  0.40410E+00_JPRB, 0.39751E+00_JPRB, 0.39767E+00_JPRB, 0.40053E+00_JPRB, 0.40367E+00_JPRB, 0.40305E+00_JPRB, 0.39833E+00_JPRB ,&
&  0.39175E+00_JPRB, 0.38834E+00_JPRB, 0.39091E+00_JPRB, 0.39338E+00_JPRB, 0.38547E+00_JPRB, 0.35886E+00_JPRB, 0.33252E+00_JPRB ,&
&  0.31491E+00_JPRB, 0.31399E+00_JPRB, 0.33896E+00_JPRB, 0.39742E+00_JPRB, 0.42145E+00_JPRB, 0.43807E+00_JPRB, 0.44891E+00_JPRB ,&
&  0.44426E+00_JPRB, 0.44875E+00_JPRB, 0.47130E+00_JPRB, 0.51373E+00_JPRB, 0.52160E+00_JPRB, 0.52931E+00_JPRB, 0.53680E+00_JPRB ,&
&  0.55496E+00_JPRB, 0.55454E+00_JPRB, 0.56846E+00_JPRB, 0.56158E+00_JPRB, 0.66358E+00_JPRB, 0.87438E+00_JPRB, 0.83189E+00_JPRB ,&
&  0.88102E+00_JPRB, 0.90197E+00_JPRB, 0.97280E+00_JPRB, 0.10035E+01_JPRB, 0.10044E+01_JPRB, 0.10112E+01_JPRB, 0.10045E+01_JPRB ,&
&  0.96604E+00_JPRB, 0.94510E+00_JPRB, 0.91459E+00_JPRB, 0.91366E+00_JPRB, 0.91292E+00_JPRB, 0.92456E+00_JPRB, 0.92322E+00_JPRB ,&
&  0.91529E+00_JPRB, 0.89953E+00_JPRB, 0.83908E+00_JPRB, 0.76073E+00_JPRB, 0.72029E+00_JPRB, 0.68569E+00_JPRB, 0.66401E+00_JPRB ,&
&  0.65164E+00_JPRB, 0.66437E+00_JPRB, 0.68425E+00_JPRB, 0.68742E+00_JPRB, 0.62881E+00_JPRB, 0.55357E+00_JPRB, 0.48836E+00_JPRB ,&
&  0.45061E+00_JPRB, 0.44053E+00_JPRB, 0.42301E+00_JPRB, 0.41683E+00_JPRB, 0.42293E+00_JPRB, 0.43091E+00_JPRB, 0.43375E+00_JPRB ,&
&  0.43771E+00_JPRB, 0.44825E+00_JPRB, 0.47632E+00_JPRB, 0.50628E+00_JPRB, 0.52375E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.53221E+00_JPRB, 0.53796E+00_JPRB, 0.53384E+00_JPRB, 0.53328E+00_JPRB, 0.52968E+00_JPRB, 0.51131E+00_JPRB, 0.48113E+00_JPRB ,&
&  0.48198E+00_JPRB, 0.43478E+00_JPRB, 0.41822E+00_JPRB, 0.40911E+00_JPRB, 0.40756E+00_JPRB, 0.40199E+00_JPRB, 0.39310E+00_JPRB ,&
&  0.39560E+00_JPRB, 0.40872E+00_JPRB, 0.42940E+00_JPRB, 0.43921E+00_JPRB, 0.43786E+00_JPRB, 0.42715E+00_JPRB, 0.41151E+00_JPRB ,&
&  0.39668E+00_JPRB, 0.39297E+00_JPRB, 0.39980E+00_JPRB, 0.40997E+00_JPRB, 0.41657E+00_JPRB, 0.41535E+00_JPRB, 0.41122E+00_JPRB ,&
&  0.40589E+00_JPRB, 0.40041E+00_JPRB, 0.39940E+00_JPRB, 0.40101E+00_JPRB, 0.40062E+00_JPRB, 0.39159E+00_JPRB, 0.36705E+00_JPRB ,&
&  0.34810E+00_JPRB, 0.35645E+00_JPRB, 0.41456E+00_JPRB, 0.44617E+00_JPRB, 0.46659E+00_JPRB, 0.47609E+00_JPRB, 0.48576E+00_JPRB ,&
&  0.49898E+00_JPRB, 0.51490E+00_JPRB, 0.53448E+00_JPRB, 0.55880E+00_JPRB, 0.57373E+00_JPRB, 0.58606E+00_JPRB, 0.59748E+00_JPRB ,&
&  0.63691E+00_JPRB, 0.55725E+00_JPRB, 0.53701E+00_JPRB, 0.57727E+00_JPRB, 0.82934E+00_JPRB, 0.92178E+00_JPRB, 0.95825E+00_JPRB ,&
&  0.10307E+01_JPRB, 0.10058E+01_JPRB, 0.10392E+01_JPRB, 0.10923E+01_JPRB, 0.11223E+01_JPRB, 0.11295E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10704E+01_JPRB, 0.10152E+01_JPRB, 0.97856E+00_JPRB, 0.95249E+00_JPRB, 0.96474E+00_JPRB, 0.95232E+00_JPRB ,&
&  0.92693E+00_JPRB, 0.91469E+00_JPRB, 0.85780E+00_JPRB, 0.79423E+00_JPRB, 0.74522E+00_JPRB, 0.71329E+00_JPRB, 0.68865E+00_JPRB ,&
&  0.67197E+00_JPRB, 0.67576E+00_JPRB, 0.68845E+00_JPRB, 0.70033E+00_JPRB, 0.66896E+00_JPRB, 0.60839E+00_JPRB, 0.55054E+00_JPRB ,&
&  0.49158E+00_JPRB, 0.46306E+00_JPRB, 0.43234E+00_JPRB, 0.42588E+00_JPRB, 0.43755E+00_JPRB, 0.45197E+00_JPRB, 0.46564E+00_JPRB ,&
&  0.48279E+00_JPRB, 0.49488E+00_JPRB, 0.50701E+00_JPRB, 0.52428E+00_JPRB, 0.52828E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.53111E+00_JPRB, 0.54414E+00_JPRB, 0.54942E+00_JPRB, 0.55750E+00_JPRB, 0.55802E+00_JPRB, 0.53049E+00_JPRB, 0.51387E+00_JPRB ,&
&  0.54740E+00_JPRB, 0.49846E+00_JPRB, 0.51897E+00_JPRB, 0.45480E+00_JPRB, 0.42948E+00_JPRB, 0.42217E+00_JPRB, 0.40886E+00_JPRB ,&
&  0.40662E+00_JPRB, 0.41565E+00_JPRB, 0.42660E+00_JPRB, 0.43609E+00_JPRB, 0.44027E+00_JPRB, 0.42798E+00_JPRB, 0.41399E+00_JPRB ,&
&  0.39747E+00_JPRB, 0.39298E+00_JPRB, 0.40070E+00_JPRB, 0.41026E+00_JPRB, 0.41912E+00_JPRB, 0.42151E+00_JPRB, 0.41835E+00_JPRB ,&
&  0.41084E+00_JPRB, 0.40716E+00_JPRB, 0.40911E+00_JPRB, 0.40916E+00_JPRB, 0.41105E+00_JPRB, 0.40706E+00_JPRB, 0.40270E+00_JPRB ,&
&  0.40289E+00_JPRB, 0.41123E+00_JPRB, 0.45247E+00_JPRB, 0.48130E+00_JPRB, 0.49916E+00_JPRB, 0.50726E+00_JPRB, 0.52078E+00_JPRB ,&
&  0.53393E+00_JPRB, 0.55065E+00_JPRB, 0.57765E+00_JPRB, 0.61079E+00_JPRB, 0.63833E+00_JPRB, 0.65714E+00_JPRB, 0.67908E+00_JPRB ,&
&  0.72599E+00_JPRB, 0.60617E+00_JPRB, 0.69004E+00_JPRB, 0.70829E+00_JPRB, 0.86639E+00_JPRB, 0.10553E+01_JPRB, 0.10470E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.11636E+01_JPRB, 0.11967E+01_JPRB, 0.12108E+01_JPRB, 0.12366E+01_JPRB, 0.12545E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.13007E+01_JPRB, 0.11734E+01_JPRB, 0.11651E+01_JPRB, 0.10989E+01_JPRB, 0.10516E+01_JPRB, 0.10457E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.96818E+00_JPRB, 0.94945E+00_JPRB, 0.91320E+00_JPRB, 0.81190E+00_JPRB, 0.78134E+00_JPRB, 0.75558E+00_JPRB, 0.69884E+00_JPRB ,&
&  0.67759E+00_JPRB, 0.65494E+00_JPRB, 0.63382E+00_JPRB, 0.67846E+00_JPRB, 0.66511E+00_JPRB, 0.61169E+00_JPRB, 0.55210E+00_JPRB ,&
&  0.50576E+00_JPRB, 0.47335E+00_JPRB, 0.48483E+00_JPRB, 0.48810E+00_JPRB, 0.48137E+00_JPRB, 0.48334E+00_JPRB, 0.51618E+00_JPRB ,&
&  0.51342E+00_JPRB, 0.52211E+00_JPRB, 0.54387E+00_JPRB, 0.54435E+00_JPRB, 0.53460E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.54132E+00_JPRB, 0.55972E+00_JPRB, 0.56278E+00_JPRB, 0.57590E+00_JPRB, 0.58349E+00_JPRB, 0.56063E+00_JPRB, 0.53189E+00_JPRB ,&
&  0.53845E+00_JPRB, 0.53673E+00_JPRB, 0.53634E+00_JPRB, 0.53650E+00_JPRB, 0.53851E+00_JPRB, 0.49441E+00_JPRB, 0.42099E+00_JPRB ,&
&  0.39765E+00_JPRB, 0.40396E+00_JPRB, 0.42052E+00_JPRB, 0.43274E+00_JPRB, 0.43444E+00_JPRB, 0.42766E+00_JPRB, 0.42039E+00_JPRB ,&
&  0.41211E+00_JPRB, 0.40616E+00_JPRB, 0.40959E+00_JPRB, 0.41346E+00_JPRB, 0.41669E+00_JPRB, 0.42268E+00_JPRB, 0.42527E+00_JPRB ,&
&  0.41896E+00_JPRB, 0.41828E+00_JPRB, 0.41742E+00_JPRB, 0.41104E+00_JPRB, 0.42370E+00_JPRB, 0.42128E+00_JPRB, 0.41915E+00_JPRB ,&
&  0.43302E+00_JPRB, 0.45647E+00_JPRB, 0.47999E+00_JPRB, 0.50856E+00_JPRB, 0.53078E+00_JPRB, 0.54425E+00_JPRB, 0.56018E+00_JPRB ,&
&  0.58264E+00_JPRB, 0.61368E+00_JPRB, 0.64954E+00_JPRB, 0.69093E+00_JPRB, 0.73621E+00_JPRB, 0.72333E+00_JPRB, 0.78976E+00_JPRB ,&
&  0.79804E+00_JPRB, 0.83929E+00_JPRB, 0.85824E+00_JPRB, 0.80919E+00_JPRB, 0.97112E+00_JPRB, 0.11295E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11974E+01_JPRB, 0.11412E+01_JPRB, 0.12506E+01_JPRB, 0.13726E+01_JPRB, 0.14705E+01_JPRB, 0.14556E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.14953E+01_JPRB, 0.14901E+01_JPRB, 0.13146E+01_JPRB, 0.12464E+01_JPRB, 0.11764E+01_JPRB, 0.11559E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.10328E+01_JPRB, 0.99327E+00_JPRB, 0.94265E+00_JPRB, 0.88190E+00_JPRB, 0.82718E+00_JPRB, 0.77532E+00_JPRB ,&
&  0.71573E+00_JPRB, 0.63887E+00_JPRB, 0.60637E+00_JPRB, 0.62171E+00_JPRB, 0.64850E+00_JPRB, 0.58339E+00_JPRB, 0.52073E+00_JPRB ,&
&  0.49949E+00_JPRB, 0.50260E+00_JPRB, 0.54166E+00_JPRB, 0.54187E+00_JPRB, 0.53239E+00_JPRB, 0.52352E+00_JPRB, 0.50706E+00_JPRB ,&
&  0.49591E+00_JPRB, 0.53517E+00_JPRB, 0.54753E+00_JPRB, 0.54356E+00_JPRB, 0.53674E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.56358E+00_JPRB, 0.58915E+00_JPRB, 0.58928E+00_JPRB, 0.60233E+00_JPRB, 0.61538E+00_JPRB, 0.58573E+00_JPRB, 0.57281E+00_JPRB ,&
&  0.56995E+00_JPRB, 0.56478E+00_JPRB, 0.55599E+00_JPRB, 0.55154E+00_JPRB, 0.54985E+00_JPRB, 0.54017E+00_JPRB, 0.48785E+00_JPRB ,&
&  0.42925E+00_JPRB, 0.42413E+00_JPRB, 0.42014E+00_JPRB, 0.43003E+00_JPRB, 0.44058E+00_JPRB, 0.43553E+00_JPRB, 0.43086E+00_JPRB ,&
&  0.43088E+00_JPRB, 0.42530E+00_JPRB, 0.42149E+00_JPRB, 0.42191E+00_JPRB, 0.42626E+00_JPRB, 0.43517E+00_JPRB, 0.43906E+00_JPRB ,&
&  0.42962E+00_JPRB, 0.42393E+00_JPRB, 0.42098E+00_JPRB, 0.42118E+00_JPRB, 0.43395E+00_JPRB, 0.43949E+00_JPRB, 0.44364E+00_JPRB ,&
&  0.45797E+00_JPRB, 0.48034E+00_JPRB, 0.50380E+00_JPRB, 0.53790E+00_JPRB, 0.56974E+00_JPRB, 0.59645E+00_JPRB, 0.63087E+00_JPRB ,&
&  0.69217E+00_JPRB, 0.75047E+00_JPRB, 0.78108E+00_JPRB, 0.82344E+00_JPRB, 0.85553E+00_JPRB, 0.89472E+00_JPRB, 0.93278E+00_JPRB ,&
&  0.95288E+00_JPRB, 0.96333E+00_JPRB, 0.92895E+00_JPRB, 0.96849E+00_JPRB, 0.11365E+01_JPRB, 0.12427E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.13703E+01_JPRB, 0.13107E+01_JPRB, 0.16185E+01_JPRB, 0.17132E+01_JPRB, 0.16612E+01_JPRB, 0.16758E+01_JPRB, 0.15831E+01_JPRB ,&
&  0.15678E+01_JPRB, 0.14824E+01_JPRB, 0.14044E+01_JPRB, 0.13395E+01_JPRB, 0.12794E+01_JPRB, 0.12884E+01_JPRB, 0.12182E+01_JPRB ,&
&  0.12068E+01_JPRB, 0.11739E+01_JPRB, 0.10185E+01_JPRB, 0.90621E+00_JPRB, 0.79325E+00_JPRB, 0.83223E+00_JPRB, 0.79990E+00_JPRB ,&
&  0.72979E+00_JPRB, 0.62442E+00_JPRB, 0.64536E+00_JPRB, 0.67002E+00_JPRB, 0.71224E+00_JPRB, 0.69428E+00_JPRB, 0.59025E+00_JPRB ,&
&  0.54739E+00_JPRB, 0.56679E+00_JPRB, 0.56984E+00_JPRB, 0.57357E+00_JPRB, 0.57335E+00_JPRB, 0.56641E+00_JPRB, 0.52312E+00_JPRB ,&
&  0.50672E+00_JPRB, 0.53922E+00_JPRB, 0.55208E+00_JPRB, 0.55054E+00_JPRB, 0.55329E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.59357E+00_JPRB, 0.62620E+00_JPRB, 0.63535E+00_JPRB, 0.63843E+00_JPRB, 0.64770E+00_JPRB, 0.61668E+00_JPRB, 0.60377E+00_JPRB ,&
&  0.60510E+00_JPRB, 0.59755E+00_JPRB, 0.57784E+00_JPRB, 0.57121E+00_JPRB, 0.56688E+00_JPRB, 0.55643E+00_JPRB, 0.53968E+00_JPRB ,&
&  0.46095E+00_JPRB, 0.43900E+00_JPRB, 0.41740E+00_JPRB, 0.42066E+00_JPRB, 0.45767E+00_JPRB, 0.45195E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.44817E+00_JPRB, 0.44681E+00_JPRB, 0.43837E+00_JPRB, 0.43613E+00_JPRB, 0.44026E+00_JPRB, 0.45309E+00_JPRB, 0.46589E+00_JPRB ,&
&  0.45756E+00_JPRB, 0.44440E+00_JPRB, 0.43781E+00_JPRB, 0.43842E+00_JPRB, 0.44979E+00_JPRB, 0.46275E+00_JPRB, 0.47789E+00_JPRB ,&
&  0.49591E+00_JPRB, 0.51468E+00_JPRB, 0.54731E+00_JPRB, 0.59498E+00_JPRB, 0.63620E+00_JPRB, 0.67493E+00_JPRB, 0.74889E+00_JPRB ,&
&  0.85607E+00_JPRB, 0.93763E+00_JPRB, 0.98022E+00_JPRB, 0.10148E+01_JPRB, 0.10158E+01_JPRB, 0.10552E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.10689E+01_JPRB, 0.10363E+01_JPRB, 0.12431E+01_JPRB, 0.14619E+01_JPRB, 0.15406E+01_JPRB, 0.15246E+01_JPRB ,&
&  0.15556E+01_JPRB, 0.15079E+01_JPRB, 0.16247E+01_JPRB, 0.17446E+01_JPRB, 0.16999E+01_JPRB, 0.17679E+01_JPRB, 0.16452E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.13779E+01_JPRB, 0.13424E+01_JPRB, 0.13431E+01_JPRB, 0.12973E+01_JPRB, 0.13619E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.12062E+01_JPRB, 0.91924E+00_JPRB, 0.85040E+00_JPRB, 0.69102E+00_JPRB, 0.70729E+00_JPRB, 0.85654E+00_JPRB ,&
&  0.83144E+00_JPRB, 0.75978E+00_JPRB, 0.75489E+00_JPRB, 0.78824E+00_JPRB, 0.84613E+00_JPRB, 0.94100E+00_JPRB, 0.80473E+00_JPRB ,&
&  0.66739E+00_JPRB, 0.65010E+00_JPRB, 0.65221E+00_JPRB, 0.64515E+00_JPRB, 0.63661E+00_JPRB, 0.63676E+00_JPRB, 0.62006E+00_JPRB ,&
&  0.60019E+00_JPRB, 0.58779E+00_JPRB, 0.58506E+00_JPRB, 0.58113E+00_JPRB, 0.58706E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.62836E+00_JPRB, 0.65718E+00_JPRB, 0.67378E+00_JPRB, 0.67187E+00_JPRB, 0.67934E+00_JPRB, 0.65787E+00_JPRB, 0.63786E+00_JPRB ,&
&  0.63506E+00_JPRB, 0.62408E+00_JPRB, 0.60075E+00_JPRB, 0.59335E+00_JPRB, 0.58511E+00_JPRB, 0.56929E+00_JPRB, 0.55256E+00_JPRB ,&
&  0.54575E+00_JPRB, 0.49026E+00_JPRB, 0.44979E+00_JPRB, 0.42628E+00_JPRB, 0.45210E+00_JPRB, 0.45208E+00_JPRB, 0.46021E+00_JPRB ,&
&  0.45826E+00_JPRB, 0.46934E+00_JPRB, 0.46593E+00_JPRB, 0.46236E+00_JPRB, 0.47060E+00_JPRB, 0.48670E+00_JPRB, 0.50044E+00_JPRB ,&
&  0.49931E+00_JPRB, 0.49466E+00_JPRB, 0.48730E+00_JPRB, 0.49038E+00_JPRB, 0.51245E+00_JPRB, 0.51120E+00_JPRB, 0.53551E+00_JPRB ,&
&  0.55441E+00_JPRB, 0.57410E+00_JPRB, 0.60657E+00_JPRB, 0.66072E+00_JPRB, 0.71443E+00_JPRB, 0.77320E+00_JPRB, 0.87838E+00_JPRB ,&
&  0.98577E+00_JPRB, 0.10329E+01_JPRB, 0.10346E+01_JPRB, 0.10605E+01_JPRB, 0.11384E+01_JPRB, 0.12313E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.10754E+01_JPRB, 0.98826E+00_JPRB, 0.13059E+01_JPRB, 0.16021E+01_JPRB, 0.16430E+01_JPRB, 0.16315E+01_JPRB ,&
&  0.16831E+01_JPRB, 0.17100E+01_JPRB, 0.18375E+01_JPRB, 0.18866E+01_JPRB, 0.18857E+01_JPRB, 0.18401E+01_JPRB, 0.16640E+01_JPRB ,&
&  0.14579E+01_JPRB, 0.13642E+01_JPRB, 0.13998E+01_JPRB, 0.14155E+01_JPRB, 0.14245E+01_JPRB, 0.13734E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.90486E+00_JPRB, 0.73175E+00_JPRB, 0.84339E+00_JPRB, 0.73346E+00_JPRB, 0.74470E+00_JPRB, 0.82607E+00_JPRB ,&
&  0.79123E+00_JPRB, 0.76972E+00_JPRB, 0.85619E+00_JPRB, 0.89955E+00_JPRB, 0.99860E+00_JPRB, 0.11413E+01_JPRB, 0.10054E+01_JPRB ,&
&  0.87341E+00_JPRB, 0.80653E+00_JPRB, 0.80555E+00_JPRB, 0.78145E+00_JPRB, 0.78302E+00_JPRB, 0.76447E+00_JPRB, 0.74294E+00_JPRB ,&
&  0.70870E+00_JPRB, 0.68421E+00_JPRB, 0.65780E+00_JPRB, 0.64155E+00_JPRB, 0.63304E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.69499E+00_JPRB, 0.71365E+00_JPRB, 0.71898E+00_JPRB, 0.71701E+00_JPRB, 0.71846E+00_JPRB, 0.71029E+00_JPRB, 0.68335E+00_JPRB ,&
&  0.66402E+00_JPRB, 0.64696E+00_JPRB, 0.62335E+00_JPRB, 0.61476E+00_JPRB, 0.59956E+00_JPRB, 0.58149E+00_JPRB, 0.56547E+00_JPRB ,&
&  0.55948E+00_JPRB, 0.53796E+00_JPRB, 0.47375E+00_JPRB, 0.44718E+00_JPRB, 0.42705E+00_JPRB, 0.44059E+00_JPRB, 0.45311E+00_JPRB ,&
&  0.47163E+00_JPRB, 0.48227E+00_JPRB, 0.48864E+00_JPRB, 0.50176E+00_JPRB, 0.52338E+00_JPRB, 0.53642E+00_JPRB, 0.56015E+00_JPRB ,&
&  0.56510E+00_JPRB, 0.57153E+00_JPRB, 0.55106E+00_JPRB, 0.56401E+00_JPRB, 0.58839E+00_JPRB, 0.58772E+00_JPRB, 0.63233E+00_JPRB ,&
&  0.66160E+00_JPRB, 0.69498E+00_JPRB, 0.72887E+00_JPRB, 0.75722E+00_JPRB, 0.79577E+00_JPRB, 0.83069E+00_JPRB, 0.91978E+00_JPRB ,&
&  0.98668E+00_JPRB, 0.10098E+01_JPRB, 0.99190E+00_JPRB, 0.99770E+00_JPRB, 0.10499E+01_JPRB, 0.11130E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.95219E+00_JPRB, 0.72052E+00_JPRB, 0.11065E+01_JPRB, 0.16086E+01_JPRB, 0.16029E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.15621E+01_JPRB, 0.16841E+01_JPRB, 0.18894E+01_JPRB, 0.19652E+01_JPRB, 0.19535E+01_JPRB, 0.18674E+01_JPRB, 0.17467E+01_JPRB ,&
&  0.16144E+01_JPRB, 0.16276E+01_JPRB, 0.17479E+01_JPRB, 0.19457E+01_JPRB, 0.19726E+01_JPRB, 0.19579E+01_JPRB, 0.19068E+01_JPRB ,&
&  0.16471E+01_JPRB, 0.13865E+01_JPRB, 0.95989E+00_JPRB, 0.98770E+00_JPRB, 0.78107E+00_JPRB, 0.77595E+00_JPRB, 0.84947E+00_JPRB ,&
&  0.81556E+00_JPRB, 0.85860E+00_JPRB, 0.96013E+00_JPRB, 0.10312E+01_JPRB, 0.12873E+01_JPRB, 0.13377E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.95297E+00_JPRB, 0.97252E+00_JPRB, 0.95964E+00_JPRB, 0.96460E+00_JPRB, 0.93013E+00_JPRB, 0.88886E+00_JPRB ,&
&  0.83847E+00_JPRB, 0.79636E+00_JPRB, 0.76515E+00_JPRB, 0.72782E+00_JPRB, 0.69867E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.77127E+00_JPRB, 0.76541E+00_JPRB, 0.74783E+00_JPRB, 0.72905E+00_JPRB, 0.72046E+00_JPRB, 0.71376E+00_JPRB, 0.69877E+00_JPRB ,&
&  0.68738E+00_JPRB, 0.66567E+00_JPRB, 0.64594E+00_JPRB, 0.62854E+00_JPRB, 0.61479E+00_JPRB, 0.59645E+00_JPRB, 0.58287E+00_JPRB ,&
&  0.57607E+00_JPRB, 0.53045E+00_JPRB, 0.45650E+00_JPRB, 0.42469E+00_JPRB, 0.40829E+00_JPRB, 0.41832E+00_JPRB, 0.44038E+00_JPRB ,&
&  0.46672E+00_JPRB, 0.48591E+00_JPRB, 0.51610E+00_JPRB, 0.55659E+00_JPRB, 0.60113E+00_JPRB, 0.63574E+00_JPRB, 0.65903E+00_JPRB ,&
&  0.73281E+00_JPRB, 0.67222E+00_JPRB, 0.68642E+00_JPRB, 0.67478E+00_JPRB, 0.71218E+00_JPRB, 0.73261E+00_JPRB, 0.73882E+00_JPRB ,&
&  0.74155E+00_JPRB, 0.74510E+00_JPRB, 0.75218E+00_JPRB, 0.75448E+00_JPRB, 0.77227E+00_JPRB, 0.82285E+00_JPRB, 0.93181E+00_JPRB ,&
&  0.10180E+01_JPRB, 0.10042E+01_JPRB, 0.95899E+00_JPRB, 0.99665E+00_JPRB, 0.10698E+01_JPRB, 0.11148E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.98865E+00_JPRB, 0.11568E+01_JPRB, 0.13614E+01_JPRB, 0.16258E+01_JPRB, 0.16998E+01_JPRB, 0.16123E+01_JPRB ,&
&  0.17086E+01_JPRB, 0.17618E+01_JPRB, 0.18710E+01_JPRB, 0.15420E+01_JPRB, 0.17547E+01_JPRB, 0.17588E+01_JPRB, 0.17421E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.18837E+01_JPRB, 0.20415E+01_JPRB, 0.23290E+01_JPRB, 0.25354E+01_JPRB, 0.26799E+01_JPRB, 0.20487E+01_JPRB ,&
&  0.15955E+01_JPRB, 0.16868E+01_JPRB, 0.15224E+01_JPRB, 0.12410E+01_JPRB, 0.10208E+01_JPRB, 0.10206E+01_JPRB, 0.10003E+01_JPRB ,&
&  0.98694E+00_JPRB, 0.10350E+01_JPRB, 0.11181E+01_JPRB, 0.13759E+01_JPRB, 0.16574E+01_JPRB, 0.17285E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.12957E+01_JPRB, 0.12953E+01_JPRB, 0.12346E+01_JPRB, 0.11815E+01_JPRB, 0.11731E+01_JPRB, 0.11212E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.95485E+00_JPRB, 0.89839E+00_JPRB, 0.85506E+00_JPRB, 0.81225E+00_JPRB, 0.78107E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.82508E+00_JPRB, 0.81039E+00_JPRB, 0.78492E+00_JPRB, 0.76180E+00_JPRB, 0.74277E+00_JPRB, 0.72900E+00_JPRB, 0.71505E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.67234E+00_JPRB, 0.65605E+00_JPRB, 0.64122E+00_JPRB, 0.62938E+00_JPRB, 0.61601E+00_JPRB, 0.60798E+00_JPRB ,&
&  0.60058E+00_JPRB, 0.51759E+00_JPRB, 0.44182E+00_JPRB, 0.43495E+00_JPRB, 0.43163E+00_JPRB, 0.42259E+00_JPRB, 0.43914E+00_JPRB ,&
&  0.46585E+00_JPRB, 0.49869E+00_JPRB, 0.54731E+00_JPRB, 0.63474E+00_JPRB, 0.71672E+00_JPRB, 0.78100E+00_JPRB, 0.83348E+00_JPRB ,&
&  0.82081E+00_JPRB, 0.83339E+00_JPRB, 0.86548E+00_JPRB, 0.83306E+00_JPRB, 0.80431E+00_JPRB, 0.78994E+00_JPRB, 0.78838E+00_JPRB ,&
&  0.78687E+00_JPRB, 0.77549E+00_JPRB, 0.77903E+00_JPRB, 0.79136E+00_JPRB, 0.81876E+00_JPRB, 0.88667E+00_JPRB, 0.98517E+00_JPRB ,&
&  0.10313E+01_JPRB, 0.10002E+01_JPRB, 0.98194E+00_JPRB, 0.10715E+01_JPRB, 0.10058E+01_JPRB, 0.96835E+00_JPRB, 0.93911E+00_JPRB ,&
&  0.11759E+01_JPRB, 0.13309E+01_JPRB, 0.13297E+01_JPRB, 0.14762E+01_JPRB, 0.13783E+01_JPRB, 0.14906E+01_JPRB, 0.17266E+01_JPRB ,&
&  0.18632E+01_JPRB, 0.18807E+01_JPRB, 0.19361E+01_JPRB, 0.13480E+01_JPRB, 0.12588E+01_JPRB, 0.17315E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.19027E+01_JPRB, 0.21819E+01_JPRB, 0.25444E+01_JPRB, 0.29866E+01_JPRB, 0.22733E+01_JPRB, 0.19766E+01_JPRB, 0.15061E+01_JPRB ,&
&  0.14547E+01_JPRB, 0.13231E+01_JPRB, 0.14943E+01_JPRB, 0.13755E+01_JPRB, 0.13375E+01_JPRB, 0.12870E+01_JPRB, 0.11999E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.12115E+01_JPRB, 0.13954E+01_JPRB, 0.18281E+01_JPRB, 0.21954E+01_JPRB, 0.20448E+01_JPRB, 0.19845E+01_JPRB ,&
&  0.18760E+01_JPRB, 0.17396E+01_JPRB, 0.15347E+01_JPRB, 0.14746E+01_JPRB, 0.13806E+01_JPRB, 0.12623E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.96126E+00_JPRB, 0.90627E+00_JPRB, 0.86426E+00_JPRB, 0.83902E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.87626E+00_JPRB, 0.85316E+00_JPRB, 0.82769E+00_JPRB, 0.80138E+00_JPRB, 0.77903E+00_JPRB, 0.75880E+00_JPRB, 0.74453E+00_JPRB ,&
&  0.74629E+00_JPRB, 0.70033E+00_JPRB, 0.66295E+00_JPRB, 0.64733E+00_JPRB, 0.64163E+00_JPRB, 0.63837E+00_JPRB, 0.63687E+00_JPRB ,&
&  0.63152E+00_JPRB, 0.57244E+00_JPRB, 0.46997E+00_JPRB, 0.44238E+00_JPRB, 0.45618E+00_JPRB, 0.44510E+00_JPRB, 0.44271E+00_JPRB ,&
&  0.48292E+00_JPRB, 0.54260E+00_JPRB, 0.63277E+00_JPRB, 0.76946E+00_JPRB, 0.87908E+00_JPRB, 0.96599E+00_JPRB, 0.10147E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.95185E+00_JPRB, 0.87396E+00_JPRB, 0.80259E+00_JPRB, 0.76827E+00_JPRB, 0.75440E+00_JPRB, 0.76048E+00_JPRB ,&
&  0.77423E+00_JPRB, 0.78235E+00_JPRB, 0.79408E+00_JPRB, 0.80183E+00_JPRB, 0.84834E+00_JPRB, 0.89784E+00_JPRB, 0.97929E+00_JPRB ,&
&  0.10140E+01_JPRB, 0.10000E+01_JPRB, 0.10346E+01_JPRB, 0.11193E+01_JPRB, 0.11247E+01_JPRB, 0.10531E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.12666E+01_JPRB, 0.13740E+01_JPRB, 0.14796E+01_JPRB, 0.16062E+01_JPRB, 0.16891E+01_JPRB, 0.19128E+01_JPRB ,&
&  0.16283E+01_JPRB, 0.13179E+01_JPRB, 0.11114E+01_JPRB, 0.97033E+00_JPRB, 0.13830E+01_JPRB, 0.20249E+01_JPRB, 0.23228E+01_JPRB ,&
&  0.25791E+01_JPRB, 0.28726E+01_JPRB, 0.31957E+01_JPRB, 0.28687E+01_JPRB, 0.21954E+01_JPRB, 0.17074E+01_JPRB, 0.20319E+01_JPRB ,&
&  0.20969E+01_JPRB, 0.20673E+01_JPRB, 0.19123E+01_JPRB, 0.16725E+01_JPRB, 0.13323E+01_JPRB, 0.15115E+01_JPRB, 0.14552E+01_JPRB ,&
&  0.13992E+01_JPRB, 0.13982E+01_JPRB, 0.19515E+01_JPRB, 0.25227E+01_JPRB, 0.26252E+01_JPRB, 0.25878E+01_JPRB, 0.25585E+01_JPRB ,&
&  0.23123E+01_JPRB, 0.21604E+01_JPRB, 0.19039E+01_JPRB, 0.16634E+01_JPRB, 0.14627E+01_JPRB, 0.12901E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10011E+01_JPRB, 0.94060E+00_JPRB, 0.91235E+00_JPRB, 0.89255E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.95044E+00_JPRB, 0.92028E+00_JPRB, 0.89164E+00_JPRB, 0.86444E+00_JPRB, 0.83788E+00_JPRB, 0.80035E+00_JPRB, 0.78622E+00_JPRB ,&
&  0.77753E+00_JPRB, 0.73118E+00_JPRB, 0.68797E+00_JPRB, 0.67081E+00_JPRB, 0.67766E+00_JPRB, 0.68121E+00_JPRB, 0.67607E+00_JPRB ,&
&  0.66832E+00_JPRB, 0.66696E+00_JPRB, 0.53804E+00_JPRB, 0.48618E+00_JPRB, 0.48285E+00_JPRB, 0.47630E+00_JPRB, 0.45999E+00_JPRB ,&
&  0.53060E+00_JPRB, 0.64307E+00_JPRB, 0.76803E+00_JPRB, 0.92085E+00_JPRB, 0.10899E+01_JPRB, 0.10639E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10109E+01_JPRB, 0.92821E+00_JPRB, 0.86260E+00_JPRB, 0.82362E+00_JPRB, 0.80949E+00_JPRB, 0.81620E+00_JPRB ,&
&  0.83288E+00_JPRB, 0.84461E+00_JPRB, 0.85097E+00_JPRB, 0.87391E+00_JPRB, 0.88521E+00_JPRB, 0.92615E+00_JPRB, 0.10030E+01_JPRB ,&
&  0.10262E+01_JPRB, 0.10059E+01_JPRB, 0.10488E+01_JPRB, 0.11741E+01_JPRB, 0.12796E+01_JPRB, 0.11874E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.12982E+01_JPRB, 0.14584E+01_JPRB, 0.15545E+01_JPRB, 0.18088E+01_JPRB, 0.18716E+01_JPRB, 0.17766E+01_JPRB, 0.18618E+01_JPRB ,&
&  0.15335E+01_JPRB, 0.13824E+01_JPRB, 0.15507E+01_JPRB, 0.17726E+01_JPRB, 0.19304E+01_JPRB, 0.21002E+01_JPRB, 0.32468E+01_JPRB ,&
&  0.27959E+01_JPRB, 0.31616E+01_JPRB, 0.34297E+01_JPRB, 0.33577E+01_JPRB, 0.16176E+01_JPRB, 0.12056E+01_JPRB, 0.17527E+01_JPRB ,&
&  0.19097E+01_JPRB, 0.13195E+01_JPRB, 0.95494E+00_JPRB, 0.11597E+01_JPRB, 0.88282E+00_JPRB, 0.10610E+01_JPRB, 0.14155E+01_JPRB ,&
&  0.15303E+01_JPRB, 0.16864E+01_JPRB, 0.24678E+01_JPRB, 0.28849E+01_JPRB, 0.29807E+01_JPRB, 0.34278E+01_JPRB, 0.29195E+01_JPRB ,&
&  0.26649E+01_JPRB, 0.22434E+01_JPRB, 0.21104E+01_JPRB, 0.18161E+01_JPRB, 0.15723E+01_JPRB, 0.14043E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.10901E+01_JPRB, 0.10316E+01_JPRB, 0.10006E+01_JPRB, 0.97792E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.10546E+01_JPRB, 0.10093E+01_JPRB, 0.96829E+00_JPRB, 0.93307E+00_JPRB, 0.89096E+00_JPRB, 0.85331E+00_JPRB, 0.83831E+00_JPRB ,&
&  0.81068E+00_JPRB, 0.76327E+00_JPRB, 0.72984E+00_JPRB, 0.70289E+00_JPRB, 0.70381E+00_JPRB, 0.70786E+00_JPRB, 0.69355E+00_JPRB ,&
&  0.67351E+00_JPRB, 0.66263E+00_JPRB, 0.61425E+00_JPRB, 0.57017E+00_JPRB, 0.54622E+00_JPRB, 0.53412E+00_JPRB, 0.55186E+00_JPRB ,&
&  0.67024E+00_JPRB, 0.87265E+00_JPRB, 0.99965E+00_JPRB, 0.11061E+01_JPRB, 0.11655E+01_JPRB, 0.11174E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.12025E+01_JPRB, 0.11553E+01_JPRB, 0.10649E+01_JPRB, 0.97823E+00_JPRB, 0.92700E+00_JPRB, 0.89916E+00_JPRB, 0.88374E+00_JPRB ,&
&  0.88786E+00_JPRB, 0.90270E+00_JPRB, 0.91094E+00_JPRB, 0.90202E+00_JPRB, 0.90007E+00_JPRB, 0.94523E+00_JPRB, 0.10208E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.10193E+01_JPRB, 0.10836E+01_JPRB, 0.11847E+01_JPRB, 0.14357E+01_JPRB, 0.15069E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.12039E+01_JPRB, 0.16642E+01_JPRB, 0.18018E+01_JPRB, 0.17855E+01_JPRB, 0.17589E+01_JPRB, 0.16662E+01_JPRB ,&
&  0.16690E+01_JPRB, 0.19939E+01_JPRB, 0.21942E+01_JPRB, 0.25169E+01_JPRB, 0.26552E+01_JPRB, 0.21029E+01_JPRB, 0.28967E+01_JPRB ,&
&  0.31740E+01_JPRB, 0.31747E+01_JPRB, 0.25666E+01_JPRB, 0.17857E+01_JPRB, 0.18122E+01_JPRB, 0.13362E+01_JPRB, 0.90283E+00_JPRB ,&
&  0.84414E+00_JPRB, 0.80466E+00_JPRB, 0.72829E+00_JPRB, 0.73674E+00_JPRB, 0.71774E+00_JPRB, 0.95750E+00_JPRB, 0.13666E+01_JPRB ,&
&  0.18455E+01_JPRB, 0.23308E+01_JPRB, 0.28923E+01_JPRB, 0.32343E+01_JPRB, 0.31218E+01_JPRB, 0.34353E+01_JPRB, 0.31410E+01_JPRB ,&
&  0.24941E+01_JPRB, 0.21959E+01_JPRB, 0.20216E+01_JPRB, 0.17827E+01_JPRB, 0.16004E+01_JPRB, 0.14789E+01_JPRB, 0.13842E+01_JPRB ,&
&  0.12893E+01_JPRB, 0.12066E+01_JPRB, 0.11512E+01_JPRB, 0.11198E+01_JPRB, 0.10928E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.11348E+01_JPRB, 0.10862E+01_JPRB, 0.10431E+01_JPRB, 0.10069E+01_JPRB, 0.96355E+00_JPRB, 0.92469E+00_JPRB, 0.89713E+00_JPRB ,&
&  0.85054E+00_JPRB, 0.79562E+00_JPRB, 0.75797E+00_JPRB, 0.73320E+00_JPRB, 0.72592E+00_JPRB, 0.72549E+00_JPRB, 0.71468E+00_JPRB ,&
&  0.69584E+00_JPRB, 0.68062E+00_JPRB, 0.67040E+00_JPRB, 0.66674E+00_JPRB, 0.68159E+00_JPRB, 0.71182E+00_JPRB, 0.79243E+00_JPRB ,&
&  0.99557E+00_JPRB, 0.12475E+01_JPRB, 0.13279E+01_JPRB, 0.13857E+01_JPRB, 0.14282E+01_JPRB, 0.14169E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.13041E+01_JPRB, 0.11934E+01_JPRB, 0.10805E+01_JPRB, 0.10013E+01_JPRB, 0.96021E+00_JPRB, 0.94342E+00_JPRB ,&
&  0.93937E+00_JPRB, 0.96345E+00_JPRB, 0.95615E+00_JPRB, 0.92952E+00_JPRB, 0.91829E+00_JPRB, 0.98033E+00_JPRB, 0.10478E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.10985E+01_JPRB, 0.11855E+01_JPRB, 0.12333E+01_JPRB, 0.11969E+01_JPRB, 0.10078E+01_JPRB, 0.11044E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.15207E+01_JPRB, 0.15821E+01_JPRB, 0.19503E+01_JPRB, 0.19393E+01_JPRB, 0.17730E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.19623E+01_JPRB, 0.26869E+01_JPRB, 0.23781E+01_JPRB, 0.24069E+01_JPRB, 0.29941E+01_JPRB, 0.24479E+01_JPRB, 0.26528E+01_JPRB ,&
&  0.31421E+01_JPRB, 0.29386E+01_JPRB, 0.24421E+01_JPRB, 0.18152E+01_JPRB, 0.22609E+01_JPRB, 0.21713E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.88297E+00_JPRB, 0.81835E+00_JPRB, 0.76876E+00_JPRB, 0.73540E+00_JPRB, 0.72845E+00_JPRB, 0.10583E+01_JPRB, 0.18205E+01_JPRB ,&
&  0.21328E+01_JPRB, 0.29715E+01_JPRB, 0.34352E+01_JPRB, 0.35722E+01_JPRB, 0.31894E+01_JPRB, 0.28778E+01_JPRB, 0.25382E+01_JPRB ,&
&  0.23064E+01_JPRB, 0.22373E+01_JPRB, 0.20312E+01_JPRB, 0.18701E+01_JPRB, 0.17234E+01_JPRB, 0.16004E+01_JPRB, 0.14901E+01_JPRB ,&
&  0.13831E+01_JPRB, 0.13076E+01_JPRB, 0.12499E+01_JPRB, 0.12253E+01_JPRB, 0.11831E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.11962E+01_JPRB, 0.11571E+01_JPRB, 0.11013E+01_JPRB, 0.10593E+01_JPRB, 0.10143E+01_JPRB, 0.96598E+00_JPRB, 0.94176E+00_JPRB ,&
&  0.89876E+00_JPRB, 0.84046E+00_JPRB, 0.79398E+00_JPRB, 0.77343E+00_JPRB, 0.75962E+00_JPRB, 0.74277E+00_JPRB, 0.72941E+00_JPRB ,&
&  0.71255E+00_JPRB, 0.70438E+00_JPRB, 0.72067E+00_JPRB, 0.83490E+00_JPRB, 0.91618E+00_JPRB, 0.98877E+00_JPRB, 0.10897E+01_JPRB ,&
&  0.14553E+01_JPRB, 0.15926E+01_JPRB, 0.15548E+01_JPRB, 0.15504E+01_JPRB, 0.14969E+01_JPRB, 0.14494E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.13315E+01_JPRB, 0.12780E+01_JPRB, 0.11956E+01_JPRB, 0.11108E+01_JPRB, 0.10419E+01_JPRB, 0.10127E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.10095E+01_JPRB, 0.98115E+00_JPRB, 0.93593E+00_JPRB, 0.93545E+00_JPRB, 0.10082E+01_JPRB, 0.10790E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.12010E+01_JPRB, 0.12622E+01_JPRB, 0.12397E+01_JPRB, 0.10279E+01_JPRB, 0.11480E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.12537E+01_JPRB, 0.13254E+01_JPRB, 0.13168E+01_JPRB, 0.16142E+01_JPRB, 0.18485E+01_JPRB, 0.17454E+01_JPRB, 0.17080E+01_JPRB ,&
&  0.22590E+01_JPRB, 0.23164E+01_JPRB, 0.21643E+01_JPRB, 0.23024E+01_JPRB, 0.26016E+01_JPRB, 0.33418E+01_JPRB, 0.22449E+01_JPRB ,&
&  0.20509E+01_JPRB, 0.24659E+01_JPRB, 0.21426E+01_JPRB, 0.16683E+01_JPRB, 0.20256E+01_JPRB, 0.25511E+01_JPRB, 0.20733E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.82767E+00_JPRB, 0.88615E+00_JPRB, 0.98165E+00_JPRB, 0.90390E+00_JPRB, 0.13307E+01_JPRB, 0.31654E+01_JPRB ,&
&  0.30311E+01_JPRB, 0.37108E+01_JPRB, 0.37519E+01_JPRB, 0.35221E+01_JPRB, 0.30639E+01_JPRB, 0.26480E+01_JPRB, 0.25015E+01_JPRB ,&
&  0.22840E+01_JPRB, 0.21733E+01_JPRB, 0.20797E+01_JPRB, 0.19805E+01_JPRB, 0.18498E+01_JPRB, 0.16900E+01_JPRB, 0.15507E+01_JPRB ,&
&  0.14453E+01_JPRB, 0.13734E+01_JPRB, 0.13113E+01_JPRB, 0.12687E+01_JPRB, 0.12267E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.12641E+01_JPRB, 0.12002E+01_JPRB, 0.11357E+01_JPRB, 0.10940E+01_JPRB, 0.10415E+01_JPRB, 0.99293E+00_JPRB, 0.99503E+00_JPRB ,&
&  0.96393E+00_JPRB, 0.89854E+00_JPRB, 0.83391E+00_JPRB, 0.80583E+00_JPRB, 0.78474E+00_JPRB, 0.76209E+00_JPRB, 0.74925E+00_JPRB ,&
&  0.73835E+00_JPRB, 0.73987E+00_JPRB, 0.75291E+00_JPRB, 0.85801E+00_JPRB, 0.97310E+00_JPRB, 0.96147E+00_JPRB, 0.11468E+01_JPRB ,&
&  0.16678E+01_JPRB, 0.17629E+01_JPRB, 0.16714E+01_JPRB, 0.16254E+01_JPRB, 0.15530E+01_JPRB, 0.15054E+01_JPRB, 0.14208E+01_JPRB ,&
&  0.12906E+01_JPRB, 0.12182E+01_JPRB, 0.11303E+01_JPRB, 0.10994E+01_JPRB, 0.10783E+01_JPRB, 0.10574E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.99259E+00_JPRB, 0.95611E+00_JPRB, 0.91712E+00_JPRB, 0.95726E+00_JPRB, 0.10521E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.12956E+01_JPRB, 0.13022E+01_JPRB, 0.11899E+01_JPRB, 0.11532E+01_JPRB, 0.11515E+01_JPRB, 0.11971E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.12610E+01_JPRB, 0.13693E+01_JPRB, 0.15038E+01_JPRB, 0.16710E+01_JPRB, 0.16954E+01_JPRB, 0.17274E+01_JPRB ,&
&  0.20552E+01_JPRB, 0.20809E+01_JPRB, 0.19519E+01_JPRB, 0.21909E+01_JPRB, 0.24491E+01_JPRB, 0.30752E+01_JPRB, 0.30619E+01_JPRB ,&
&  0.24895E+01_JPRB, 0.19342E+01_JPRB, 0.17427E+01_JPRB, 0.17450E+01_JPRB, 0.19426E+01_JPRB, 0.23786E+01_JPRB, 0.33377E+01_JPRB ,&
&  0.33752E+01_JPRB, 0.22825E+01_JPRB, 0.19118E+01_JPRB, 0.25211E+01_JPRB, 0.21859E+01_JPRB, 0.20683E+01_JPRB, 0.27336E+01_JPRB ,&
&  0.34063E+01_JPRB, 0.38889E+01_JPRB, 0.38034E+01_JPRB, 0.34122E+01_JPRB, 0.30876E+01_JPRB, 0.26765E+01_JPRB, 0.25196E+01_JPRB ,&
&  0.23576E+01_JPRB, 0.22112E+01_JPRB, 0.21641E+01_JPRB, 0.20793E+01_JPRB, 0.19154E+01_JPRB, 0.17195E+01_JPRB, 0.15515E+01_JPRB ,&
&  0.14540E+01_JPRB, 0.13960E+01_JPRB, 0.13543E+01_JPRB, 0.13268E+01_JPRB, 0.13008E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.12876E+01_JPRB, 0.12066E+01_JPRB, 0.11493E+01_JPRB, 0.11022E+01_JPRB, 0.10448E+01_JPRB, 0.10172E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.94573E+00_JPRB, 0.88482E+00_JPRB, 0.83504E+00_JPRB, 0.80036E+00_JPRB, 0.78124E+00_JPRB, 0.77293E+00_JPRB ,&
&  0.76402E+00_JPRB, 0.76953E+00_JPRB, 0.78298E+00_JPRB, 0.89770E+00_JPRB, 0.11091E+01_JPRB, 0.14395E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.17171E+01_JPRB, 0.20237E+01_JPRB, 0.18673E+01_JPRB, 0.16960E+01_JPRB, 0.15474E+01_JPRB, 0.15595E+01_JPRB, 0.14239E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.11474E+01_JPRB, 0.10817E+01_JPRB, 0.10514E+01_JPRB, 0.10479E+01_JPRB, 0.10329E+01_JPRB, 0.99469E+00_JPRB ,&
&  0.96901E+00_JPRB, 0.94837E+00_JPRB, 0.92918E+00_JPRB, 0.94031E+00_JPRB, 0.10038E+01_JPRB, 0.11011E+01_JPRB, 0.12108E+01_JPRB ,&
&  0.13312E+01_JPRB, 0.13780E+01_JPRB, 0.13437E+01_JPRB, 0.12121E+01_JPRB, 0.11953E+01_JPRB, 0.11809E+01_JPRB, 0.11816E+01_JPRB ,&
&  0.11376E+01_JPRB, 0.11044E+01_JPRB, 0.12871E+01_JPRB, 0.14889E+01_JPRB, 0.15973E+01_JPRB, 0.15604E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.19502E+01_JPRB, 0.22700E+01_JPRB, 0.25225E+01_JPRB, 0.24559E+01_JPRB, 0.27386E+01_JPRB, 0.30887E+01_JPRB, 0.33627E+01_JPRB ,&
&  0.32102E+01_JPRB, 0.24103E+01_JPRB, 0.20429E+01_JPRB, 0.19043E+01_JPRB, 0.18930E+01_JPRB, 0.22862E+01_JPRB, 0.30611E+01_JPRB ,&
&  0.39431E+01_JPRB, 0.52164E+01_JPRB, 0.52319E+01_JPRB, 0.41472E+01_JPRB, 0.36156E+01_JPRB, 0.32351E+01_JPRB, 0.29990E+01_JPRB ,&
&  0.38025E+01_JPRB, 0.39269E+01_JPRB, 0.37466E+01_JPRB, 0.41222E+01_JPRB, 0.32548E+01_JPRB, 0.27649E+01_JPRB, 0.25250E+01_JPRB ,&
&  0.24204E+01_JPRB, 0.22896E+01_JPRB, 0.21652E+01_JPRB, 0.20280E+01_JPRB, 0.18164E+01_JPRB, 0.16279E+01_JPRB, 0.15101E+01_JPRB ,&
&  0.14436E+01_JPRB, 0.14067E+01_JPRB, 0.13770E+01_JPRB, 0.13623E+01_JPRB, 0.13433E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.12590E+01_JPRB, 0.11850E+01_JPRB, 0.11078E+01_JPRB, 0.10740E+01_JPRB, 0.10427E+01_JPRB, 0.10425E+01_JPRB, 0.10715E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.99418E+00_JPRB, 0.90223E+00_JPRB, 0.83566E+00_JPRB, 0.80595E+00_JPRB, 0.80037E+00_JPRB, 0.79612E+00_JPRB ,&
&  0.79868E+00_JPRB, 0.81530E+00_JPRB, 0.86781E+00_JPRB, 0.95511E+00_JPRB, 0.11733E+01_JPRB, 0.14657E+01_JPRB, 0.15511E+01_JPRB ,&
&  0.18925E+01_JPRB, 0.26159E+01_JPRB, 0.20535E+01_JPRB, 0.17308E+01_JPRB, 0.16255E+01_JPRB, 0.17973E+01_JPRB, 0.15384E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.11779E+01_JPRB, 0.11489E+01_JPRB, 0.10560E+01_JPRB, 0.10225E+01_JPRB, 0.98890E+00_JPRB, 0.97732E+00_JPRB ,&
&  0.96438E+00_JPRB, 0.95245E+00_JPRB, 0.94249E+00_JPRB, 0.98367E+00_JPRB, 0.10677E+01_JPRB, 0.11759E+01_JPRB, 0.13341E+01_JPRB ,&
&  0.14758E+01_JPRB, 0.14330E+01_JPRB, 0.13167E+01_JPRB, 0.12523E+01_JPRB, 0.12262E+01_JPRB, 0.12298E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.11655E+01_JPRB, 0.12747E+01_JPRB, 0.13349E+01_JPRB, 0.13706E+01_JPRB, 0.14958E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.19605E+01_JPRB, 0.21297E+01_JPRB, 0.27665E+01_JPRB, 0.25571E+01_JPRB, 0.28785E+01_JPRB, 0.31538E+01_JPRB, 0.29701E+01_JPRB ,&
&  0.24636E+01_JPRB, 0.21923E+01_JPRB, 0.18944E+01_JPRB, 0.18512E+01_JPRB, 0.20827E+01_JPRB, 0.26587E+01_JPRB, 0.32610E+01_JPRB ,&
&  0.39429E+01_JPRB, 0.46369E+01_JPRB, 0.51916E+01_JPRB, 0.41891E+01_JPRB, 0.37624E+01_JPRB, 0.35067E+01_JPRB, 0.35133E+01_JPRB ,&
&  0.40166E+01_JPRB, 0.41106E+01_JPRB, 0.41796E+01_JPRB, 0.40120E+01_JPRB, 0.30441E+01_JPRB, 0.26040E+01_JPRB, 0.23934E+01_JPRB ,&
&  0.22707E+01_JPRB, 0.21406E+01_JPRB, 0.19342E+01_JPRB, 0.17648E+01_JPRB, 0.16385E+01_JPRB, 0.15452E+01_JPRB, 0.14569E+01_JPRB ,&
&  0.14045E+01_JPRB, 0.13617E+01_JPRB, 0.13416E+01_JPRB, 0.13385E+01_JPRB, 0.13141E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.11850E+01_JPRB, 0.11307E+01_JPRB, 0.10968E+01_JPRB, 0.10879E+01_JPRB, 0.10728E+01_JPRB, 0.11037E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.97588E+00_JPRB, 0.87498E+00_JPRB, 0.83767E+00_JPRB, 0.82514E+00_JPRB, 0.81773E+00_JPRB, 0.82372E+00_JPRB ,&
&  0.83771E+00_JPRB, 0.88567E+00_JPRB, 0.97167E+00_JPRB, 0.10924E+01_JPRB, 0.13486E+01_JPRB, 0.17241E+01_JPRB, 0.20664E+01_JPRB ,&
&  0.23739E+01_JPRB, 0.23265E+01_JPRB, 0.20440E+01_JPRB, 0.17182E+01_JPRB, 0.16069E+01_JPRB, 0.17217E+01_JPRB, 0.16740E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.12681E+01_JPRB, 0.11767E+01_JPRB, 0.10720E+01_JPRB, 0.10122E+01_JPRB, 0.99864E+00_JPRB, 0.98367E+00_JPRB ,&
&  0.95784E+00_JPRB, 0.94174E+00_JPRB, 0.95272E+00_JPRB, 0.10314E+01_JPRB, 0.11376E+01_JPRB, 0.12957E+01_JPRB, 0.15307E+01_JPRB ,&
&  0.15715E+01_JPRB, 0.14653E+01_JPRB, 0.13685E+01_JPRB, 0.12684E+01_JPRB, 0.12603E+01_JPRB, 0.12662E+01_JPRB, 0.12683E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.12947E+01_JPRB, 0.13364E+01_JPRB, 0.13500E+01_JPRB, 0.13305E+01_JPRB, 0.15344E+01_JPRB, 0.16611E+01_JPRB ,&
&  0.19708E+01_JPRB, 0.23052E+01_JPRB, 0.30037E+01_JPRB, 0.24541E+01_JPRB, 0.29573E+01_JPRB, 0.30283E+01_JPRB, 0.27299E+01_JPRB ,&
&  0.22827E+01_JPRB, 0.20381E+01_JPRB, 0.18988E+01_JPRB, 0.19717E+01_JPRB, 0.24512E+01_JPRB, 0.30630E+01_JPRB, 0.42343E+01_JPRB ,&
&  0.42477E+01_JPRB, 0.47617E+01_JPRB, 0.44847E+01_JPRB, 0.43828E+01_JPRB, 0.38498E+01_JPRB, 0.36458E+01_JPRB, 0.35816E+01_JPRB ,&
&  0.38703E+01_JPRB, 0.38409E+01_JPRB, 0.35552E+01_JPRB, 0.32008E+01_JPRB, 0.25674E+01_JPRB, 0.22184E+01_JPRB, 0.20556E+01_JPRB ,&
&  0.18925E+01_JPRB, 0.17265E+01_JPRB, 0.16015E+01_JPRB, 0.15509E+01_JPRB, 0.14906E+01_JPRB, 0.14260E+01_JPRB, 0.13742E+01_JPRB ,&
&  0.13442E+01_JPRB, 0.13176E+01_JPRB, 0.12997E+01_JPRB, 0.12803E+01_JPRB, 0.12460E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.11058E+01_JPRB, 0.10721E+01_JPRB, 0.10572E+01_JPRB, 0.10300E+01_JPRB, 0.10057E+01_JPRB, 0.10018E+01_JPRB, 0.99078E+00_JPRB ,&
&  0.93357E+00_JPRB, 0.88258E+00_JPRB, 0.86443E+00_JPRB, 0.86069E+00_JPRB, 0.84946E+00_JPRB, 0.83642E+00_JPRB, 0.84291E+00_JPRB ,&
&  0.87931E+00_JPRB, 0.95479E+00_JPRB, 0.10670E+01_JPRB, 0.12622E+01_JPRB, 0.15200E+01_JPRB, 0.18701E+01_JPRB, 0.22273E+01_JPRB ,&
&  0.22584E+01_JPRB, 0.16234E+01_JPRB, 0.17978E+01_JPRB, 0.17549E+01_JPRB, 0.15830E+01_JPRB, 0.16226E+01_JPRB, 0.16028E+01_JPRB ,&
&  0.13436E+01_JPRB, 0.12546E+01_JPRB, 0.12255E+01_JPRB, 0.11617E+01_JPRB, 0.10256E+01_JPRB, 0.10042E+01_JPRB, 0.97707E+00_JPRB ,&
&  0.96127E+00_JPRB, 0.96255E+00_JPRB, 0.10055E+01_JPRB, 0.11355E+01_JPRB, 0.13036E+01_JPRB, 0.15548E+01_JPRB, 0.16900E+01_JPRB ,&
&  0.16168E+01_JPRB, 0.15540E+01_JPRB, 0.15154E+01_JPRB, 0.14039E+01_JPRB, 0.13800E+01_JPRB, 0.13919E+01_JPRB, 0.13951E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13900E+01_JPRB, 0.14526E+01_JPRB, 0.15705E+01_JPRB, 0.16834E+01_JPRB, 0.16390E+01_JPRB, 0.18777E+01_JPRB ,&
&  0.21325E+01_JPRB, 0.26003E+01_JPRB, 0.29130E+01_JPRB, 0.33731E+01_JPRB, 0.24500E+01_JPRB, 0.26781E+01_JPRB, 0.22198E+01_JPRB ,&
&  0.22078E+01_JPRB, 0.20774E+01_JPRB, 0.22144E+01_JPRB, 0.24993E+01_JPRB, 0.31951E+01_JPRB, 0.38216E+01_JPRB, 0.48690E+01_JPRB ,&
&  0.43299E+01_JPRB, 0.41440E+01_JPRB, 0.39522E+01_JPRB, 0.41977E+01_JPRB, 0.39077E+01_JPRB, 0.39343E+01_JPRB, 0.35647E+01_JPRB ,&
&  0.35128E+01_JPRB, 0.31580E+01_JPRB, 0.27961E+01_JPRB, 0.25674E+01_JPRB, 0.21139E+01_JPRB, 0.17373E+01_JPRB, 0.15462E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.14077E+01_JPRB, 0.13928E+01_JPRB, 0.13808E+01_JPRB, 0.13514E+01_JPRB, 0.13200E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.12757E+01_JPRB, 0.12495E+01_JPRB, 0.12279E+01_JPRB, 0.12024E+01_JPRB, 0.11610E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.10425E+01_JPRB, 0.10130E+01_JPRB, 0.99902E+00_JPRB, 0.97739E+00_JPRB, 0.96107E+00_JPRB, 0.94993E+00_JPRB, 0.93637E+00_JPRB ,&
&  0.91049E+00_JPRB, 0.89623E+00_JPRB, 0.89119E+00_JPRB, 0.88001E+00_JPRB, 0.85539E+00_JPRB, 0.84883E+00_JPRB, 0.87995E+00_JPRB ,&
&  0.95789E+00_JPRB, 0.10837E+01_JPRB, 0.12384E+01_JPRB, 0.14099E+01_JPRB, 0.16552E+01_JPRB, 0.18540E+01_JPRB, 0.19348E+01_JPRB ,&
&  0.20720E+01_JPRB, 0.21132E+01_JPRB, 0.18355E+01_JPRB, 0.13468E+01_JPRB, 0.12357E+01_JPRB, 0.14634E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.12275E+01_JPRB, 0.11755E+01_JPRB, 0.11418E+01_JPRB, 0.11047E+01_JPRB, 0.10518E+01_JPRB, 0.10272E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10640E+01_JPRB, 0.11874E+01_JPRB, 0.14199E+01_JPRB, 0.16490E+01_JPRB, 0.18117E+01_JPRB, 0.18094E+01_JPRB ,&
&  0.17910E+01_JPRB, 0.18199E+01_JPRB, 0.18116E+01_JPRB, 0.16999E+01_JPRB, 0.16177E+01_JPRB, 0.16343E+01_JPRB, 0.16543E+01_JPRB ,&
&  0.16687E+01_JPRB, 0.16796E+01_JPRB, 0.16983E+01_JPRB, 0.18071E+01_JPRB, 0.18965E+01_JPRB, 0.18087E+01_JPRB, 0.19429E+01_JPRB ,&
&  0.23671E+01_JPRB, 0.29833E+01_JPRB, 0.29879E+01_JPRB, 0.35765E+01_JPRB, 0.20979E+01_JPRB, 0.23505E+01_JPRB, 0.24072E+01_JPRB ,&
&  0.21754E+01_JPRB, 0.23859E+01_JPRB, 0.26575E+01_JPRB, 0.30785E+01_JPRB, 0.37504E+01_JPRB, 0.41110E+01_JPRB, 0.45135E+01_JPRB ,&
&  0.38676E+01_JPRB, 0.34799E+01_JPRB, 0.34374E+01_JPRB, 0.36087E+01_JPRB, 0.36441E+01_JPRB, 0.35656E+01_JPRB, 0.29939E+01_JPRB ,&
&  0.25281E+01_JPRB, 0.25655E+01_JPRB, 0.22764E+01_JPRB, 0.21875E+01_JPRB, 0.16944E+01_JPRB, 0.13509E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.12665E+01_JPRB, 0.12616E+01_JPRB, 0.12470E+01_JPRB, 0.12313E+01_JPRB, 0.12175E+01_JPRB, 0.11986E+01_JPRB ,&
&  0.11832E+01_JPRB, 0.11703E+01_JPRB, 0.11518E+01_JPRB, 0.11241E+01_JPRB, 0.10859E+01_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.99243E+00_JPRB, 0.97145E+00_JPRB, 0.96639E+00_JPRB, 0.93773E+00_JPRB, 0.90991E+00_JPRB, 0.90354E+00_JPRB, 0.90862E+00_JPRB ,&
&  0.89989E+00_JPRB, 0.89154E+00_JPRB, 0.87734E+00_JPRB, 0.86576E+00_JPRB, 0.87124E+00_JPRB, 0.90865E+00_JPRB, 0.97859E+00_JPRB ,&
&  0.10734E+01_JPRB, 0.12096E+01_JPRB, 0.13798E+01_JPRB, 0.15445E+01_JPRB, 0.17506E+01_JPRB, 0.18427E+01_JPRB, 0.18256E+01_JPRB ,&
&  0.18876E+01_JPRB, 0.19676E+01_JPRB, 0.19455E+01_JPRB, 0.19199E+01_JPRB, 0.15864E+01_JPRB, 0.16891E+01_JPRB, 0.17392E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.13730E+01_JPRB, 0.11898E+01_JPRB, 0.11523E+01_JPRB, 0.11139E+01_JPRB, 0.11142E+01_JPRB, 0.11510E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.13617E+01_JPRB, 0.15732E+01_JPRB, 0.18045E+01_JPRB, 0.19587E+01_JPRB, 0.20303E+01_JPRB, 0.20568E+01_JPRB ,&
&  0.20983E+01_JPRB, 0.21552E+01_JPRB, 0.22131E+01_JPRB, 0.21432E+01_JPRB, 0.21180E+01_JPRB, 0.21143E+01_JPRB, 0.21833E+01_JPRB ,&
&  0.22535E+01_JPRB, 0.21659E+01_JPRB, 0.21762E+01_JPRB, 0.23250E+01_JPRB, 0.23558E+01_JPRB, 0.23359E+01_JPRB, 0.24343E+01_JPRB ,&
&  0.28498E+01_JPRB, 0.35374E+01_JPRB, 0.28011E+01_JPRB, 0.29143E+01_JPRB, 0.33218E+01_JPRB, 0.26936E+01_JPRB, 0.23008E+01_JPRB ,&
&  0.23630E+01_JPRB, 0.25947E+01_JPRB, 0.28576E+01_JPRB, 0.32463E+01_JPRB, 0.38496E+01_JPRB, 0.45865E+01_JPRB, 0.38320E+01_JPRB ,&
&  0.32883E+01_JPRB, 0.29225E+01_JPRB, 0.29313E+01_JPRB, 0.29759E+01_JPRB, 0.29480E+01_JPRB, 0.28405E+01_JPRB, 0.24277E+01_JPRB ,&
&  0.21438E+01_JPRB, 0.21769E+01_JPRB, 0.20342E+01_JPRB, 0.18978E+01_JPRB, 0.18213E+01_JPRB, 0.12987E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11682E+01_JPRB, 0.11713E+01_JPRB, 0.11650E+01_JPRB, 0.11526E+01_JPRB, 0.11326E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.10903E+01_JPRB, 0.10752E+01_JPRB, 0.10582E+01_JPRB, 0.10231E+01_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.95640E+00_JPRB, 0.94448E+00_JPRB, 0.93598E+00_JPRB, 0.91350E+00_JPRB, 0.88629E+00_JPRB, 0.88175E+00_JPRB, 0.87576E+00_JPRB ,&
&  0.85522E+00_JPRB, 0.83363E+00_JPRB, 0.80876E+00_JPRB, 0.81961E+00_JPRB, 0.84941E+00_JPRB, 0.90952E+00_JPRB, 0.98187E+00_JPRB ,&
&  0.10714E+01_JPRB, 0.12020E+01_JPRB, 0.13818E+01_JPRB, 0.15274E+01_JPRB, 0.17294E+01_JPRB, 0.18322E+01_JPRB, 0.18915E+01_JPRB ,&
&  0.19734E+01_JPRB, 0.20682E+01_JPRB, 0.21444E+01_JPRB, 0.21884E+01_JPRB, 0.21713E+01_JPRB, 0.21962E+01_JPRB, 0.22915E+01_JPRB ,&
&  0.19212E+01_JPRB, 0.14464E+01_JPRB, 0.13143E+01_JPRB, 0.12571E+01_JPRB, 0.12644E+01_JPRB, 0.13079E+01_JPRB, 0.14044E+01_JPRB ,&
&  0.15537E+01_JPRB, 0.17250E+01_JPRB, 0.19404E+01_JPRB, 0.21087E+01_JPRB, 0.22447E+01_JPRB, 0.23468E+01_JPRB, 0.24206E+01_JPRB ,&
&  0.25399E+01_JPRB, 0.26685E+01_JPRB, 0.28518E+01_JPRB, 0.27478E+01_JPRB, 0.30173E+01_JPRB, 0.30094E+01_JPRB, 0.30338E+01_JPRB ,&
&  0.29540E+01_JPRB, 0.27567E+01_JPRB, 0.28487E+01_JPRB, 0.29985E+01_JPRB, 0.31488E+01_JPRB, 0.31760E+01_JPRB, 0.31410E+01_JPRB ,&
&  0.35397E+01_JPRB, 0.36555E+01_JPRB, 0.27104E+01_JPRB, 0.23530E+01_JPRB, 0.23940E+01_JPRB, 0.19936E+01_JPRB, 0.22841E+01_JPRB ,&
&  0.23352E+01_JPRB, 0.25261E+01_JPRB, 0.27299E+01_JPRB, 0.30748E+01_JPRB, 0.34696E+01_JPRB, 0.37375E+01_JPRB, 0.32588E+01_JPRB ,&
&  0.26120E+01_JPRB, 0.23475E+01_JPRB, 0.23123E+01_JPRB, 0.22680E+01_JPRB, 0.22488E+01_JPRB, 0.22145E+01_JPRB, 0.21023E+01_JPRB ,&
&  0.19727E+01_JPRB, 0.18119E+01_JPRB, 0.16985E+01_JPRB, 0.14194E+01_JPRB, 0.12514E+01_JPRB, 0.11633E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.11031E+01_JPRB, 0.11106E+01_JPRB, 0.11062E+01_JPRB, 0.10998E+01_JPRB, 0.10771E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10411E+01_JPRB, 0.10242E+01_JPRB, 0.10106E+01_JPRB, 0.98049E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.90031E+00_JPRB, 0.88021E+00_JPRB, 0.86842E+00_JPRB, 0.85661E+00_JPRB, 0.83166E+00_JPRB, 0.81338E+00_JPRB, 0.80360E+00_JPRB ,&
&  0.76621E+00_JPRB, 0.76319E+00_JPRB, 0.74685E+00_JPRB, 0.75597E+00_JPRB, 0.78921E+00_JPRB, 0.82226E+00_JPRB, 0.87291E+00_JPRB ,&
&  0.91836E+00_JPRB, 0.98201E+00_JPRB, 0.10870E+01_JPRB, 0.13071E+01_JPRB, 0.15570E+01_JPRB, 0.17489E+01_JPRB, 0.19756E+01_JPRB ,&
&  0.20759E+01_JPRB, 0.21653E+01_JPRB, 0.23338E+01_JPRB, 0.25598E+01_JPRB, 0.27109E+01_JPRB, 0.28361E+01_JPRB, 0.27928E+01_JPRB ,&
&  0.20920E+01_JPRB, 0.14026E+01_JPRB, 0.14070E+01_JPRB, 0.13268E+01_JPRB, 0.13988E+01_JPRB, 0.14965E+01_JPRB, 0.16073E+01_JPRB ,&
&  0.17774E+01_JPRB, 0.19552E+01_JPRB, 0.21396E+01_JPRB, 0.22918E+01_JPRB, 0.24627E+01_JPRB, 0.26441E+01_JPRB, 0.27541E+01_JPRB ,&
&  0.28636E+01_JPRB, 0.29752E+01_JPRB, 0.30644E+01_JPRB, 0.30089E+01_JPRB, 0.31861E+01_JPRB, 0.34127E+01_JPRB, 0.36037E+01_JPRB ,&
&  0.37867E+01_JPRB, 0.38102E+01_JPRB, 0.32103E+01_JPRB, 0.33400E+01_JPRB, 0.38139E+01_JPRB, 0.35641E+01_JPRB, 0.36587E+01_JPRB ,&
&  0.42577E+01_JPRB, 0.43729E+01_JPRB, 0.24718E+01_JPRB, 0.21029E+01_JPRB, 0.21195E+01_JPRB, 0.20576E+01_JPRB, 0.21061E+01_JPRB ,&
&  0.21602E+01_JPRB, 0.23396E+01_JPRB, 0.25533E+01_JPRB, 0.27201E+01_JPRB, 0.27931E+01_JPRB, 0.26270E+01_JPRB, 0.24416E+01_JPRB ,&
&  0.20391E+01_JPRB, 0.18460E+01_JPRB, 0.18438E+01_JPRB, 0.18658E+01_JPRB, 0.19186E+01_JPRB, 0.19812E+01_JPRB, 0.18010E+01_JPRB ,&
&  0.17418E+01_JPRB, 0.17153E+01_JPRB, 0.18730E+01_JPRB, 0.15648E+01_JPRB, 0.11612E+01_JPRB, 0.10567E+01_JPRB, 0.99862E+00_JPRB ,&
&  0.10432E+01_JPRB, 0.10716E+01_JPRB, 0.10847E+01_JPRB, 0.10922E+01_JPRB, 0.10891E+01_JPRB, 0.10754E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.10115E+01_JPRB, 0.98368E+00_JPRB, 0.95476E+00_JPRB, 0.92782E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.79472E+00_JPRB, 0.77522E+00_JPRB, 0.77202E+00_JPRB, 0.73516E+00_JPRB, 0.75047E+00_JPRB, 0.72375E+00_JPRB, 0.75659E+00_JPRB ,&
&  0.73124E+00_JPRB, 0.72779E+00_JPRB, 0.72838E+00_JPRB, 0.74324E+00_JPRB, 0.75826E+00_JPRB, 0.77834E+00_JPRB, 0.79544E+00_JPRB ,&
&  0.81975E+00_JPRB, 0.86926E+00_JPRB, 0.94096E+00_JPRB, 0.10262E+01_JPRB, 0.11376E+01_JPRB, 0.12510E+01_JPRB, 0.13534E+01_JPRB ,&
&  0.14905E+01_JPRB, 0.16121E+01_JPRB, 0.16899E+01_JPRB, 0.18771E+01_JPRB, 0.23739E+01_JPRB, 0.29839E+01_JPRB, 0.30686E+01_JPRB ,&
&  0.19853E+01_JPRB, 0.14745E+01_JPRB, 0.13304E+01_JPRB, 0.12536E+01_JPRB, 0.13531E+01_JPRB, 0.15636E+01_JPRB, 0.16720E+01_JPRB ,&
&  0.17877E+01_JPRB, 0.19277E+01_JPRB, 0.20840E+01_JPRB, 0.22549E+01_JPRB, 0.24593E+01_JPRB, 0.26569E+01_JPRB, 0.27719E+01_JPRB ,&
&  0.28035E+01_JPRB, 0.28739E+01_JPRB, 0.29012E+01_JPRB, 0.31151E+01_JPRB, 0.35147E+01_JPRB, 0.39853E+01_JPRB, 0.43402E+01_JPRB ,&
&  0.45294E+01_JPRB, 0.44557E+01_JPRB, 0.37917E+01_JPRB, 0.38648E+01_JPRB, 0.40838E+01_JPRB, 0.40038E+01_JPRB, 0.38564E+01_JPRB ,&
&  0.42513E+01_JPRB, 0.39903E+01_JPRB, 0.25328E+01_JPRB, 0.22626E+01_JPRB, 0.21213E+01_JPRB, 0.20084E+01_JPRB, 0.18790E+01_JPRB ,&
&  0.19970E+01_JPRB, 0.21565E+01_JPRB, 0.22629E+01_JPRB, 0.22632E+01_JPRB, 0.21722E+01_JPRB, 0.20124E+01_JPRB, 0.19629E+01_JPRB ,&
&  0.17780E+01_JPRB, 0.17112E+01_JPRB, 0.17162E+01_JPRB, 0.17322E+01_JPRB, 0.19699E+01_JPRB, 0.17762E+01_JPRB, 0.17438E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.15035E+01_JPRB, 0.12762E+01_JPRB, 0.14993E+01_JPRB, 0.10891E+01_JPRB, 0.91097E+00_JPRB, 0.92663E+00_JPRB ,&
&  0.94829E+00_JPRB, 0.98352E+00_JPRB, 0.98170E+00_JPRB, 0.99355E+00_JPRB, 0.99904E+00_JPRB, 0.98626E+00_JPRB, 0.98040E+00_JPRB ,&
&  0.96336E+00_JPRB, 0.94110E+00_JPRB, 0.90187E+00_JPRB, 0.86827E+00_JPRB, 0.83364E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.76597E+00_JPRB, 0.76685E+00_JPRB, 0.74702E+00_JPRB, 0.71250E+00_JPRB, 0.68965E+00_JPRB, 0.67403E+00_JPRB, 0.68275E+00_JPRB ,&
&  0.69300E+00_JPRB, 0.71629E+00_JPRB, 0.73634E+00_JPRB, 0.75713E+00_JPRB, 0.76726E+00_JPRB, 0.77475E+00_JPRB, 0.79350E+00_JPRB ,&
&  0.81852E+00_JPRB, 0.84219E+00_JPRB, 0.87345E+00_JPRB, 0.91572E+00_JPRB, 0.95754E+00_JPRB, 0.10108E+01_JPRB, 0.10374E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10785E+01_JPRB, 0.11134E+01_JPRB, 0.11863E+01_JPRB, 0.14575E+01_JPRB, 0.17495E+01_JPRB, 0.21089E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.16183E+01_JPRB, 0.14473E+01_JPRB, 0.13541E+01_JPRB, 0.14087E+01_JPRB, 0.14097E+01_JPRB, 0.14674E+01_JPRB ,&
&  0.15515E+01_JPRB, 0.16102E+01_JPRB, 0.16631E+01_JPRB, 0.17255E+01_JPRB, 0.17898E+01_JPRB, 0.19128E+01_JPRB, 0.18818E+01_JPRB ,&
&  0.18784E+01_JPRB, 0.19718E+01_JPRB, 0.20903E+01_JPRB, 0.24239E+01_JPRB, 0.29140E+01_JPRB, 0.33831E+01_JPRB, 0.34379E+01_JPRB ,&
&  0.34927E+01_JPRB, 0.37581E+01_JPRB, 0.36872E+01_JPRB, 0.37370E+01_JPRB, 0.36877E+01_JPRB, 0.35528E+01_JPRB, 0.35565E+01_JPRB ,&
&  0.33844E+01_JPRB, 0.30193E+01_JPRB, 0.26342E+01_JPRB, 0.22556E+01_JPRB, 0.20039E+01_JPRB, 0.17872E+01_JPRB, 0.17360E+01_JPRB ,&
&  0.17704E+01_JPRB, 0.18654E+01_JPRB, 0.19094E+01_JPRB, 0.18245E+01_JPRB, 0.17444E+01_JPRB, 0.16386E+01_JPRB, 0.16422E+01_JPRB ,&
&  0.16129E+01_JPRB, 0.14934E+01_JPRB, 0.14758E+01_JPRB, 0.15727E+01_JPRB, 0.17141E+01_JPRB, 0.27554E+01_JPRB, 0.24470E+01_JPRB ,&
&  0.14236E+01_JPRB, 0.11641E+01_JPRB, 0.82493E+00_JPRB, 0.11827E+01_JPRB, 0.86050E+00_JPRB, 0.80615E+00_JPRB, 0.83058E+00_JPRB ,&
&  0.84949E+00_JPRB, 0.86586E+00_JPRB, 0.87652E+00_JPRB, 0.87549E+00_JPRB, 0.87537E+00_JPRB, 0.87577E+00_JPRB, 0.86076E+00_JPRB ,&
&  0.84816E+00_JPRB, 0.83017E+00_JPRB, 0.82520E+00_JPRB, 0.79068E+00_JPRB, 0.77383E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.73284E+00_JPRB, 0.73214E+00_JPRB, 0.70357E+00_JPRB, 0.66170E+00_JPRB, 0.64309E+00_JPRB, 0.64923E+00_JPRB, 0.66637E+00_JPRB ,&
&  0.68354E+00_JPRB, 0.70605E+00_JPRB, 0.73229E+00_JPRB, 0.75818E+00_JPRB, 0.78167E+00_JPRB, 0.80721E+00_JPRB, 0.83775E+00_JPRB ,&
&  0.86681E+00_JPRB, 0.90227E+00_JPRB, 0.93890E+00_JPRB, 0.97701E+00_JPRB, 0.10116E+01_JPRB, 0.10704E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.12473E+01_JPRB, 0.13303E+01_JPRB, 0.14169E+01_JPRB, 0.15286E+01_JPRB, 0.16232E+01_JPRB, 0.14832E+01_JPRB ,&
&  0.17585E+01_JPRB, 0.16445E+01_JPRB, 0.15028E+01_JPRB, 0.14058E+01_JPRB, 0.13350E+01_JPRB, 0.12845E+01_JPRB, 0.12316E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.11880E+01_JPRB, 0.11521E+01_JPRB, 0.11243E+01_JPRB, 0.11176E+01_JPRB, 0.12089E+01_JPRB, 0.12325E+01_JPRB ,&
&  0.13544E+01_JPRB, 0.15625E+01_JPRB, 0.17521E+01_JPRB, 0.20647E+01_JPRB, 0.23290E+01_JPRB, 0.25239E+01_JPRB, 0.24242E+01_JPRB ,&
&  0.24661E+01_JPRB, 0.27282E+01_JPRB, 0.27136E+01_JPRB, 0.28107E+01_JPRB, 0.27911E+01_JPRB, 0.26525E+01_JPRB, 0.25634E+01_JPRB ,&
&  0.22934E+01_JPRB, 0.21851E+01_JPRB, 0.21764E+01_JPRB, 0.23161E+01_JPRB, 0.19525E+01_JPRB, 0.17048E+01_JPRB, 0.16858E+01_JPRB ,&
&  0.16148E+01_JPRB, 0.16451E+01_JPRB, 0.16476E+01_JPRB, 0.15474E+01_JPRB, 0.14656E+01_JPRB, 0.13981E+01_JPRB, 0.14128E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.12936E+01_JPRB, 0.13184E+01_JPRB, 0.14928E+01_JPRB, 0.17732E+01_JPRB, 0.19201E+01_JPRB, 0.23149E+01_JPRB ,&
&  0.13614E+01_JPRB, 0.80893E+00_JPRB, 0.82356E+00_JPRB, 0.87838E+00_JPRB, 0.74472E+00_JPRB, 0.68822E+00_JPRB, 0.73574E+00_JPRB ,&
&  0.77510E+00_JPRB, 0.79972E+00_JPRB, 0.78768E+00_JPRB, 0.81234E+00_JPRB, 0.85765E+00_JPRB, 0.85089E+00_JPRB, 0.79821E+00_JPRB ,&
&  0.80690E+00_JPRB, 0.77081E+00_JPRB, 0.75329E+00_JPRB, 0.74707E+00_JPRB, 0.74221E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.69257E+00_JPRB, 0.68161E+00_JPRB, 0.65065E+00_JPRB, 0.61170E+00_JPRB, 0.59728E+00_JPRB, 0.61426E+00_JPRB, 0.64086E+00_JPRB ,&
&  0.66372E+00_JPRB, 0.69069E+00_JPRB, 0.71802E+00_JPRB, 0.74827E+00_JPRB, 0.78074E+00_JPRB, 0.81181E+00_JPRB, 0.83601E+00_JPRB ,&
&  0.85432E+00_JPRB, 0.87898E+00_JPRB, 0.90211E+00_JPRB, 0.92360E+00_JPRB, 0.93721E+00_JPRB, 0.94546E+00_JPRB, 0.95614E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.10587E+01_JPRB, 0.11293E+01_JPRB, 0.12052E+01_JPRB, 0.12699E+01_JPRB, 0.13276E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.15898E+01_JPRB, 0.15901E+01_JPRB, 0.14930E+01_JPRB, 0.13911E+01_JPRB, 0.12874E+01_JPRB, 0.12085E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.10675E+01_JPRB, 0.11191E+01_JPRB, 0.10884E+01_JPRB, 0.11000E+01_JPRB, 0.10211E+01_JPRB, 0.11003E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.14054E+01_JPRB, 0.16285E+01_JPRB, 0.18262E+01_JPRB, 0.20073E+01_JPRB, 0.20934E+01_JPRB, 0.21632E+01_JPRB, 0.20590E+01_JPRB ,&
&  0.20732E+01_JPRB, 0.20868E+01_JPRB, 0.19708E+01_JPRB, 0.20707E+01_JPRB, 0.21629E+01_JPRB, 0.18638E+01_JPRB, 0.16508E+01_JPRB ,&
&  0.15022E+01_JPRB, 0.16742E+01_JPRB, 0.21333E+01_JPRB, 0.23151E+01_JPRB, 0.19490E+01_JPRB, 0.17876E+01_JPRB, 0.16611E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.14956E+01_JPRB, 0.14561E+01_JPRB, 0.13742E+01_JPRB, 0.12513E+01_JPRB, 0.11822E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.11180E+01_JPRB, 0.10890E+01_JPRB, 0.12518E+01_JPRB, 0.13818E+01_JPRB, 0.13219E+01_JPRB, 0.21916E+01_JPRB ,&
&  0.13529E+01_JPRB, 0.97307E+00_JPRB, 0.13217E+01_JPRB, 0.57919E+00_JPRB, 0.63724E+00_JPRB, 0.59208E+00_JPRB, 0.60829E+00_JPRB ,&
&  0.52088E+00_JPRB, 0.61208E+00_JPRB, 0.97826E+00_JPRB, 0.12855E+01_JPRB, 0.14739E+01_JPRB, 0.14693E+01_JPRB, 0.86953E+00_JPRB ,&
&  0.77468E+00_JPRB, 0.75276E+00_JPRB, 0.73742E+00_JPRB, 0.71724E+00_JPRB, 0.69489E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.64529E+00_JPRB, 0.61000E+00_JPRB, 0.59375E+00_JPRB, 0.58197E+00_JPRB, 0.57241E+00_JPRB, 0.59280E+00_JPRB, 0.61038E+00_JPRB ,&
&  0.63657E+00_JPRB, 0.66737E+00_JPRB, 0.70402E+00_JPRB, 0.73677E+00_JPRB, 0.78115E+00_JPRB, 0.82531E+00_JPRB, 0.86201E+00_JPRB ,&
&  0.89358E+00_JPRB, 0.92079E+00_JPRB, 0.94644E+00_JPRB, 0.96195E+00_JPRB, 0.97477E+00_JPRB, 0.98558E+00_JPRB, 0.99484E+00_JPRB ,&
&  0.10028E+01_JPRB, 0.10441E+01_JPRB, 0.11152E+01_JPRB, 0.11972E+01_JPRB, 0.12625E+01_JPRB, 0.13610E+01_JPRB, 0.90173E+00_JPRB ,&
&  0.13845E+01_JPRB, 0.14425E+01_JPRB, 0.14315E+01_JPRB, 0.13498E+01_JPRB, 0.12616E+01_JPRB, 0.11924E+01_JPRB, 0.11450E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.11719E+01_JPRB, 0.11639E+01_JPRB, 0.12883E+01_JPRB, 0.13379E+01_JPRB, 0.13431E+01_JPRB, 0.15190E+01_JPRB ,&
&  0.16644E+01_JPRB, 0.18968E+01_JPRB, 0.20497E+01_JPRB, 0.21667E+01_JPRB, 0.22591E+01_JPRB, 0.23385E+01_JPRB, 0.23759E+01_JPRB ,&
&  0.23999E+01_JPRB, 0.23500E+01_JPRB, 0.22830E+01_JPRB, 0.24346E+01_JPRB, 0.25069E+01_JPRB, 0.21533E+01_JPRB, 0.19639E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.12899E+01_JPRB, 0.18747E+01_JPRB, 0.22326E+01_JPRB, 0.19993E+01_JPRB, 0.18652E+01_JPRB, 0.16534E+01_JPRB ,&
&  0.14439E+01_JPRB, 0.13368E+01_JPRB, 0.12687E+01_JPRB, 0.11901E+01_JPRB, 0.10940E+01_JPRB, 0.10294E+01_JPRB, 0.10082E+01_JPRB ,&
&  0.98440E+00_JPRB, 0.88930E+00_JPRB, 0.85533E+00_JPRB, 0.93368E+00_JPRB, 0.10022E+01_JPRB, 0.11926E+01_JPRB, 0.12545E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.13051E+01_JPRB, 0.99354E+00_JPRB, 0.89293E+00_JPRB, 0.63483E+00_JPRB, 0.55767E+00_JPRB, 0.53550E+00_JPRB ,&
&  0.55204E+00_JPRB, 0.50118E+00_JPRB, 0.99912E+00_JPRB, 0.37977E+01_JPRB, 0.37391E+01_JPRB, 0.47280E+01_JPRB, 0.24910E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.77010E+00_JPRB, 0.68053E+00_JPRB, 0.67471E+00_JPRB, 0.65826E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.57311E+00_JPRB, 0.55121E+00_JPRB, 0.52958E+00_JPRB, 0.53864E+00_JPRB, 0.54726E+00_JPRB, 0.55607E+00_JPRB, 0.58596E+00_JPRB ,&
&  0.64686E+00_JPRB, 0.66306E+00_JPRB, 0.69201E+00_JPRB, 0.73991E+00_JPRB, 0.81040E+00_JPRB, 0.87047E+00_JPRB, 0.92727E+00_JPRB ,&
&  0.98591E+00_JPRB, 0.10287E+01_JPRB, 0.10451E+01_JPRB, 0.10505E+01_JPRB, 0.10688E+01_JPRB, 0.10927E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11887E+01_JPRB, 0.12641E+01_JPRB, 0.13475E+01_JPRB, 0.14247E+01_JPRB, 0.14420E+01_JPRB, 0.93589E+00_JPRB ,&
&  0.11536E+01_JPRB, 0.12426E+01_JPRB, 0.12957E+01_JPRB, 0.12828E+01_JPRB, 0.12106E+01_JPRB, 0.11379E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.13607E+01_JPRB, 0.14367E+01_JPRB, 0.14925E+01_JPRB, 0.17307E+01_JPRB, 0.17418E+01_JPRB, 0.20723E+01_JPRB ,&
&  0.22532E+01_JPRB, 0.23913E+01_JPRB, 0.25821E+01_JPRB, 0.26978E+01_JPRB, 0.28447E+01_JPRB, 0.30186E+01_JPRB, 0.31839E+01_JPRB ,&
&  0.32692E+01_JPRB, 0.32782E+01_JPRB, 0.34034E+01_JPRB, 0.37959E+01_JPRB, 0.35182E+01_JPRB, 0.39481E+01_JPRB, 0.32103E+01_JPRB ,&
&  0.21287E+01_JPRB, 0.17588E+01_JPRB, 0.20618E+01_JPRB, 0.21838E+01_JPRB, 0.20197E+01_JPRB, 0.19187E+01_JPRB, 0.17141E+01_JPRB ,&
&  0.14257E+01_JPRB, 0.12441E+01_JPRB, 0.11650E+01_JPRB, 0.10966E+01_JPRB, 0.10189E+01_JPRB, 0.94363E+00_JPRB, 0.88459E+00_JPRB ,&
&  0.81495E+00_JPRB, 0.75820E+00_JPRB, 0.77869E+00_JPRB, 0.79451E+00_JPRB, 0.85363E+00_JPRB, 0.10945E+01_JPRB, 0.16167E+01_JPRB ,&
&  0.25524E+01_JPRB, 0.13935E+01_JPRB, 0.10550E+01_JPRB, 0.87895E+00_JPRB, 0.70503E+00_JPRB, 0.61561E+00_JPRB, 0.56397E+00_JPRB ,&
&  0.55020E+00_JPRB, 0.64418E+00_JPRB, 0.77549E+00_JPRB, 0.14736E+01_JPRB, 0.26101E+01_JPRB, 0.26616E+01_JPRB, 0.18089E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.78544E+00_JPRB, 0.63267E+00_JPRB, 0.60011E+00_JPRB, 0.57716E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.53509E+00_JPRB, 0.52116E+00_JPRB, 0.51980E+00_JPRB, 0.51835E+00_JPRB, 0.55637E+00_JPRB, 0.58190E+00_JPRB, 0.60575E+00_JPRB ,&
&  0.64303E+00_JPRB, 0.66754E+00_JPRB, 0.70295E+00_JPRB, 0.76947E+00_JPRB, 0.84394E+00_JPRB, 0.90946E+00_JPRB, 0.94876E+00_JPRB ,&
&  0.97744E+00_JPRB, 0.98727E+00_JPRB, 0.99486E+00_JPRB, 0.10001E+01_JPRB, 0.10368E+01_JPRB, 0.10672E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.12850E+01_JPRB, 0.13826E+01_JPRB, 0.14730E+01_JPRB, 0.15722E+01_JPRB, 0.16576E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.79088E+00_JPRB, 0.10883E+01_JPRB, 0.11538E+01_JPRB, 0.11542E+01_JPRB, 0.11024E+01_JPRB, 0.11038E+01_JPRB, 0.12426E+01_JPRB ,&
&  0.14283E+01_JPRB, 0.15127E+01_JPRB, 0.17173E+01_JPRB, 0.17041E+01_JPRB, 0.20884E+01_JPRB, 0.22211E+01_JPRB, 0.25126E+01_JPRB ,&
&  0.27238E+01_JPRB, 0.28849E+01_JPRB, 0.30936E+01_JPRB, 0.33199E+01_JPRB, 0.35701E+01_JPRB, 0.38467E+01_JPRB, 0.41852E+01_JPRB ,&
&  0.45126E+01_JPRB, 0.49260E+01_JPRB, 0.54607E+01_JPRB, 0.54887E+01_JPRB, 0.51110E+01_JPRB, 0.55020E+01_JPRB, 0.51740E+01_JPRB ,&
&  0.43155E+01_JPRB, 0.30303E+01_JPRB, 0.28506E+01_JPRB, 0.24768E+01_JPRB, 0.21583E+01_JPRB, 0.20077E+01_JPRB, 0.17614E+01_JPRB ,&
&  0.14562E+01_JPRB, 0.12794E+01_JPRB, 0.11550E+01_JPRB, 0.10786E+01_JPRB, 0.98505E+00_JPRB, 0.91274E+00_JPRB, 0.85790E+00_JPRB ,&
&  0.81612E+00_JPRB, 0.82630E+00_JPRB, 0.84348E+00_JPRB, 0.80302E+00_JPRB, 0.78031E+00_JPRB, 0.85634E+00_JPRB, 0.10149E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.10596E+01_JPRB, 0.92513E+00_JPRB, 0.91454E+00_JPRB, 0.85493E+00_JPRB, 0.78809E+00_JPRB, 0.75341E+00_JPRB ,&
&  0.74249E+00_JPRB, 0.71532E+00_JPRB, 0.80830E+00_JPRB, 0.12206E+01_JPRB, 0.15661E+01_JPRB, 0.17926E+01_JPRB, 0.14865E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.80528E+00_JPRB, 0.64877E+00_JPRB, 0.60662E+00_JPRB, 0.58338E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.58224E+00_JPRB, 0.56907E+00_JPRB, 0.56888E+00_JPRB, 0.56500E+00_JPRB, 0.59977E+00_JPRB, 0.63969E+00_JPRB, 0.65136E+00_JPRB ,&
&  0.64910E+00_JPRB, 0.67027E+00_JPRB, 0.69122E+00_JPRB, 0.72805E+00_JPRB, 0.80201E+00_JPRB, 0.84485E+00_JPRB, 0.89456E+00_JPRB ,&
&  0.94347E+00_JPRB, 0.95090E+00_JPRB, 0.93540E+00_JPRB, 0.94166E+00_JPRB, 0.99004E+00_JPRB, 0.10415E+01_JPRB, 0.11142E+01_JPRB ,&
&  0.12101E+01_JPRB, 0.13278E+01_JPRB, 0.14743E+01_JPRB, 0.16415E+01_JPRB, 0.18312E+01_JPRB, 0.20077E+01_JPRB, 0.20789E+01_JPRB ,&
&  0.88460E+00_JPRB, 0.86591E+00_JPRB, 0.10472E+01_JPRB, 0.10522E+01_JPRB, 0.10166E+01_JPRB, 0.10871E+01_JPRB, 0.12236E+01_JPRB ,&
&  0.13294E+01_JPRB, 0.13084E+01_JPRB, 0.14861E+01_JPRB, 0.20312E+01_JPRB, 0.23101E+01_JPRB, 0.25421E+01_JPRB, 0.27975E+01_JPRB ,&
&  0.29984E+01_JPRB, 0.32159E+01_JPRB, 0.34106E+01_JPRB, 0.35841E+01_JPRB, 0.38012E+01_JPRB, 0.40826E+01_JPRB, 0.44570E+01_JPRB ,&
&  0.49369E+01_JPRB, 0.56356E+01_JPRB, 0.63160E+01_JPRB, 0.58584E+01_JPRB, 0.60283E+01_JPRB, 0.63874E+01_JPRB, 0.72387E+01_JPRB ,&
&  0.64098E+01_JPRB, 0.43124E+01_JPRB, 0.32394E+01_JPRB, 0.26625E+01_JPRB, 0.20481E+01_JPRB, 0.18188E+01_JPRB, 0.15827E+01_JPRB ,&
&  0.14053E+01_JPRB, 0.12847E+01_JPRB, 0.11821E+01_JPRB, 0.10918E+01_JPRB, 0.98007E+00_JPRB, 0.92293E+00_JPRB, 0.88912E+00_JPRB ,&
&  0.91631E+00_JPRB, 0.94439E+00_JPRB, 0.92225E+00_JPRB, 0.88926E+00_JPRB, 0.85896E+00_JPRB, 0.89233E+00_JPRB, 0.96665E+00_JPRB ,&
&  0.10135E+01_JPRB, 0.10208E+01_JPRB, 0.10304E+01_JPRB, 0.10422E+01_JPRB, 0.10496E+01_JPRB, 0.10495E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.96359E+00_JPRB, 0.91140E+00_JPRB, 0.89843E+00_JPRB, 0.88128E+00_JPRB, 0.10257E+01_JPRB, 0.13179E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.11549E+01_JPRB, 0.89584E+00_JPRB, 0.73298E+00_JPRB, 0.68220E+00_JPRB, 0.65496E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.70045E+00_JPRB, 0.65811E+00_JPRB, 0.64742E+00_JPRB, 0.63055E+00_JPRB, 0.64560E+00_JPRB, 0.67441E+00_JPRB, 0.69491E+00_JPRB ,&
&  0.70194E+00_JPRB, 0.70162E+00_JPRB, 0.70454E+00_JPRB, 0.72805E+00_JPRB, 0.76022E+00_JPRB, 0.79637E+00_JPRB, 0.84089E+00_JPRB ,&
&  0.87180E+00_JPRB, 0.88722E+00_JPRB, 0.86704E+00_JPRB, 0.84100E+00_JPRB, 0.86780E+00_JPRB, 0.91418E+00_JPRB, 0.97951E+00_JPRB ,&
&  0.10636E+01_JPRB, 0.11645E+01_JPRB, 0.13124E+01_JPRB, 0.15489E+01_JPRB, 0.19074E+01_JPRB, 0.23416E+01_JPRB, 0.27950E+01_JPRB ,&
&  0.22481E+01_JPRB, 0.10486E+01_JPRB, 0.11085E+01_JPRB, 0.11901E+01_JPRB, 0.10872E+01_JPRB, 0.11622E+01_JPRB, 0.12148E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.12907E+01_JPRB, 0.13243E+01_JPRB, 0.16075E+01_JPRB, 0.21352E+01_JPRB, 0.24599E+01_JPRB, 0.27318E+01_JPRB ,&
&  0.29493E+01_JPRB, 0.31065E+01_JPRB, 0.32077E+01_JPRB, 0.33119E+01_JPRB, 0.34960E+01_JPRB, 0.37791E+01_JPRB, 0.41555E+01_JPRB ,&
&  0.46671E+01_JPRB, 0.53300E+01_JPRB, 0.61176E+01_JPRB, 0.56542E+01_JPRB, 0.64125E+01_JPRB, 0.64073E+01_JPRB, 0.64725E+01_JPRB ,&
&  0.58611E+01_JPRB, 0.38558E+01_JPRB, 0.28390E+01_JPRB, 0.25285E+01_JPRB, 0.19099E+01_JPRB, 0.13907E+01_JPRB, 0.14086E+01_JPRB ,&
&  0.13280E+01_JPRB, 0.12612E+01_JPRB, 0.12246E+01_JPRB, 0.11108E+01_JPRB, 0.10078E+01_JPRB, 0.96655E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.11119E+01_JPRB, 0.10676E+01_JPRB, 0.10516E+01_JPRB, 0.10159E+01_JPRB, 0.10421E+01_JPRB, 0.10842E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.11531E+01_JPRB, 0.11748E+01_JPRB, 0.12066E+01_JPRB, 0.12431E+01_JPRB, 0.11854E+01_JPRB, 0.11136E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.10862E+01_JPRB, 0.10233E+01_JPRB, 0.98628E+00_JPRB, 0.94170E+00_JPRB, 0.96285E+00_JPRB, 0.10590E+01_JPRB ,&
&  0.10356E+01_JPRB, 0.91399E+00_JPRB, 0.79450E+00_JPRB, 0.76409E+00_JPRB, 0.76786E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.81293E+00_JPRB, 0.72002E+00_JPRB, 0.66945E+00_JPRB, 0.66476E+00_JPRB, 0.65151E+00_JPRB, 0.65853E+00_JPRB, 0.66893E+00_JPRB ,&
&  0.66663E+00_JPRB, 0.68721E+00_JPRB, 0.68293E+00_JPRB, 0.70018E+00_JPRB, 0.72194E+00_JPRB, 0.73400E+00_JPRB, 0.76597E+00_JPRB ,&
&  0.79561E+00_JPRB, 0.82500E+00_JPRB, 0.85055E+00_JPRB, 0.81953E+00_JPRB, 0.82182E+00_JPRB, 0.83755E+00_JPRB, 0.87511E+00_JPRB ,&
&  0.94289E+00_JPRB, 0.10337E+01_JPRB, 0.11395E+01_JPRB, 0.12698E+01_JPRB, 0.14624E+01_JPRB, 0.18105E+01_JPRB, 0.24570E+01_JPRB ,&
&  0.40547E+01_JPRB, 0.33963E+01_JPRB, 0.10700E+01_JPRB, 0.13567E+01_JPRB, 0.13285E+01_JPRB, 0.13399E+01_JPRB, 0.11779E+01_JPRB ,&
&  0.12597E+01_JPRB, 0.13692E+01_JPRB, 0.12789E+01_JPRB, 0.12842E+01_JPRB, 0.15696E+01_JPRB, 0.20080E+01_JPRB, 0.23443E+01_JPRB ,&
&  0.25672E+01_JPRB, 0.27338E+01_JPRB, 0.28070E+01_JPRB, 0.28770E+01_JPRB, 0.29798E+01_JPRB, 0.31775E+01_JPRB, 0.34912E+01_JPRB ,&
&  0.39514E+01_JPRB, 0.46076E+01_JPRB, 0.54864E+01_JPRB, 0.54726E+01_JPRB, 0.67817E+01_JPRB, 0.64816E+01_JPRB, 0.59950E+01_JPRB ,&
&  0.48544E+01_JPRB, 0.38100E+01_JPRB, 0.31068E+01_JPRB, 0.24017E+01_JPRB, 0.16820E+01_JPRB, 0.13216E+01_JPRB, 0.12836E+01_JPRB ,&
&  0.12496E+01_JPRB, 0.11112E+01_JPRB, 0.10823E+01_JPRB, 0.10656E+01_JPRB, 0.10095E+01_JPRB, 0.10760E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.12362E+01_JPRB, 0.12125E+01_JPRB, 0.11934E+01_JPRB, 0.11756E+01_JPRB, 0.11785E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.12431E+01_JPRB, 0.12468E+01_JPRB, 0.12742E+01_JPRB, 0.13024E+01_JPRB, 0.12363E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.11585E+01_JPRB, 0.11287E+01_JPRB, 0.11057E+01_JPRB, 0.10378E+01_JPRB, 0.98045E+00_JPRB, 0.96046E+00_JPRB ,&
&  0.10320E+01_JPRB, 0.96160E+00_JPRB, 0.89958E+00_JPRB, 0.83009E+00_JPRB, 0.84900E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.86876E+00_JPRB, 0.81008E+00_JPRB, 0.71698E+00_JPRB, 0.68598E+00_JPRB, 0.67915E+00_JPRB, 0.66445E+00_JPRB, 0.64513E+00_JPRB ,&
&  0.62177E+00_JPRB, 0.61339E+00_JPRB, 0.64105E+00_JPRB, 0.67465E+00_JPRB, 0.70252E+00_JPRB, 0.71097E+00_JPRB, 0.72933E+00_JPRB ,&
&  0.74018E+00_JPRB, 0.76482E+00_JPRB, 0.81050E+00_JPRB, 0.82441E+00_JPRB, 0.81757E+00_JPRB, 0.81618E+00_JPRB, 0.82627E+00_JPRB ,&
&  0.84753E+00_JPRB, 0.88379E+00_JPRB, 0.95193E+00_JPRB, 0.10305E+01_JPRB, 0.11415E+01_JPRB, 0.13369E+01_JPRB, 0.17536E+01_JPRB ,&
&  0.26250E+01_JPRB, 0.47140E+01_JPRB, 0.12050E+01_JPRB, 0.16320E+01_JPRB, 0.15985E+01_JPRB, 0.14835E+01_JPRB, 0.13824E+01_JPRB ,&
&  0.13756E+01_JPRB, 0.13786E+01_JPRB, 0.13497E+01_JPRB, 0.12458E+01_JPRB, 0.11205E+01_JPRB, 0.13569E+01_JPRB, 0.17216E+01_JPRB ,&
&  0.20621E+01_JPRB, 0.22377E+01_JPRB, 0.23097E+01_JPRB, 0.23031E+01_JPRB, 0.23401E+01_JPRB, 0.24247E+01_JPRB, 0.26072E+01_JPRB ,&
&  0.29301E+01_JPRB, 0.34482E+01_JPRB, 0.42335E+01_JPRB, 0.46100E+01_JPRB, 0.62127E+01_JPRB, 0.65988E+01_JPRB, 0.62767E+01_JPRB ,&
&  0.49938E+01_JPRB, 0.44005E+01_JPRB, 0.37866E+01_JPRB, 0.26492E+01_JPRB, 0.18355E+01_JPRB, 0.14312E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.11259E+01_JPRB, 0.11115E+01_JPRB, 0.10837E+01_JPRB, 0.11002E+01_JPRB, 0.11733E+01_JPRB, 0.12639E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.12891E+01_JPRB, 0.12615E+01_JPRB, 0.12479E+01_JPRB, 0.12469E+01_JPRB, 0.12500E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.13075E+01_JPRB, 0.13003E+01_JPRB, 0.12651E+01_JPRB, 0.12721E+01_JPRB, 0.12304E+01_JPRB, 0.11728E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.12039E+01_JPRB, 0.11925E+01_JPRB, 0.11447E+01_JPRB, 0.11605E+01_JPRB, 0.11112E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.10824E+01_JPRB, 0.10472E+01_JPRB, 0.91512E+00_JPRB, 0.91834E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.91694E+00_JPRB, 0.86581E+00_JPRB, 0.79812E+00_JPRB, 0.72754E+00_JPRB, 0.72538E+00_JPRB, 0.73096E+00_JPRB, 0.69660E+00_JPRB ,&
&  0.64248E+00_JPRB, 0.63773E+00_JPRB, 0.64548E+00_JPRB, 0.66931E+00_JPRB, 0.68884E+00_JPRB, 0.70030E+00_JPRB, 0.73653E+00_JPRB ,&
&  0.73229E+00_JPRB, 0.72771E+00_JPRB, 0.75168E+00_JPRB, 0.78882E+00_JPRB, 0.79612E+00_JPRB, 0.79860E+00_JPRB, 0.81686E+00_JPRB ,&
&  0.82519E+00_JPRB, 0.81684E+00_JPRB, 0.82526E+00_JPRB, 0.85807E+00_JPRB, 0.93113E+00_JPRB, 0.10634E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.21317E+01_JPRB, 0.50521E+01_JPRB, 0.32492E+01_JPRB, 0.28786E+01_JPRB, 0.20961E+01_JPRB, 0.16972E+01_JPRB, 0.15447E+01_JPRB ,&
&  0.16728E+01_JPRB, 0.15948E+01_JPRB, 0.15427E+01_JPRB, 0.12532E+01_JPRB, 0.10678E+01_JPRB, 0.10229E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.14238E+01_JPRB, 0.16207E+01_JPRB, 0.17614E+01_JPRB, 0.17759E+01_JPRB, 0.17927E+01_JPRB, 0.18531E+01_JPRB, 0.19379E+01_JPRB ,&
&  0.21152E+01_JPRB, 0.24596E+01_JPRB, 0.30638E+01_JPRB, 0.38668E+01_JPRB, 0.48288E+01_JPRB, 0.57871E+01_JPRB, 0.67215E+01_JPRB ,&
&  0.63765E+01_JPRB, 0.54986E+01_JPRB, 0.44106E+01_JPRB, 0.29481E+01_JPRB, 0.19425E+01_JPRB, 0.17511E+01_JPRB, 0.13757E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.11879E+01_JPRB, 0.11841E+01_JPRB, 0.12063E+01_JPRB, 0.12434E+01_JPRB, 0.13007E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.13278E+01_JPRB, 0.13105E+01_JPRB, 0.12930E+01_JPRB, 0.12827E+01_JPRB, 0.12794E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.12982E+01_JPRB, 0.13192E+01_JPRB, 0.12634E+01_JPRB, 0.12482E+01_JPRB, 0.12452E+01_JPRB, 0.12196E+01_JPRB, 0.11594E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.12289E+01_JPRB, 0.12468E+01_JPRB, 0.12285E+01_JPRB, 0.12248E+01_JPRB, 0.12290E+01_JPRB, 0.11751E+01_JPRB ,&
&  0.11656E+01_JPRB, 0.11345E+01_JPRB, 0.10504E+01_JPRB, 0.99683E+00_JPRB, 0.98041E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.97319E+00_JPRB, 0.93198E+00_JPRB, 0.87804E+00_JPRB, 0.81222E+00_JPRB, 0.77782E+00_JPRB, 0.77921E+00_JPRB, 0.78839E+00_JPRB ,&
&  0.75677E+00_JPRB, 0.73315E+00_JPRB, 0.72364E+00_JPRB, 0.73844E+00_JPRB, 0.76522E+00_JPRB, 0.77966E+00_JPRB, 0.78854E+00_JPRB ,&
&  0.77967E+00_JPRB, 0.77357E+00_JPRB, 0.75821E+00_JPRB, 0.76141E+00_JPRB, 0.81042E+00_JPRB, 0.81359E+00_JPRB, 0.80444E+00_JPRB ,&
&  0.82319E+00_JPRB, 0.81674E+00_JPRB, 0.79788E+00_JPRB, 0.80171E+00_JPRB, 0.83000E+00_JPRB, 0.91595E+00_JPRB, 0.11122E+01_JPRB ,&
&  0.19714E+01_JPRB, 0.46701E+01_JPRB, 0.22340E+01_JPRB, 0.33112E+01_JPRB, 0.23117E+01_JPRB, 0.18835E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.15426E+01_JPRB, 0.17989E+01_JPRB, 0.14910E+01_JPRB, 0.12864E+01_JPRB, 0.11033E+01_JPRB, 0.10123E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.12607E+01_JPRB, 0.13309E+01_JPRB, 0.13748E+01_JPRB, 0.13561E+01_JPRB, 0.13663E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.15084E+01_JPRB, 0.16941E+01_JPRB, 0.20845E+01_JPRB, 0.26997E+01_JPRB, 0.40698E+01_JPRB, 0.55475E+01_JPRB, 0.71509E+01_JPRB ,&
&  0.71795E+01_JPRB, 0.69729E+01_JPRB, 0.51471E+01_JPRB, 0.33108E+01_JPRB, 0.22124E+01_JPRB, 0.18421E+01_JPRB, 0.15208E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.13073E+01_JPRB, 0.12582E+01_JPRB, 0.12693E+01_JPRB, 0.13648E+01_JPRB, 0.14328E+01_JPRB, 0.14180E+01_JPRB ,&
&  0.14003E+01_JPRB, 0.13603E+01_JPRB, 0.13352E+01_JPRB, 0.13076E+01_JPRB, 0.13067E+01_JPRB, 0.13018E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.12821E+01_JPRB, 0.12959E+01_JPRB, 0.12531E+01_JPRB, 0.12551E+01_JPRB, 0.12527E+01_JPRB, 0.12342E+01_JPRB, 0.11764E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.12288E+01_JPRB, 0.12345E+01_JPRB, 0.12150E+01_JPRB, 0.12183E+01_JPRB, 0.12768E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.11788E+01_JPRB, 0.11028E+01_JPRB, 0.10591E+01_JPRB, 0.10352E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.10267E+01_JPRB, 0.97613E+00_JPRB, 0.93593E+00_JPRB, 0.88291E+00_JPRB, 0.84432E+00_JPRB, 0.82365E+00_JPRB, 0.82418E+00_JPRB ,&
&  0.81273E+00_JPRB, 0.78473E+00_JPRB, 0.76799E+00_JPRB, 0.76983E+00_JPRB, 0.77255E+00_JPRB, 0.78543E+00_JPRB, 0.79580E+00_JPRB ,&
&  0.79285E+00_JPRB, 0.78150E+00_JPRB, 0.77610E+00_JPRB, 0.77569E+00_JPRB, 0.79992E+00_JPRB, 0.83899E+00_JPRB, 0.85019E+00_JPRB ,&
&  0.83782E+00_JPRB, 0.82797E+00_JPRB, 0.80656E+00_JPRB, 0.78962E+00_JPRB, 0.79496E+00_JPRB, 0.84835E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.18699E+01_JPRB, 0.52381E+01_JPRB, 0.22165E+01_JPRB, 0.35541E+01_JPRB, 0.24674E+01_JPRB, 0.19780E+01_JPRB, 0.15740E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.17105E+01_JPRB, 0.15195E+01_JPRB, 0.13497E+01_JPRB, 0.11990E+01_JPRB, 0.11015E+01_JPRB, 0.10693E+01_JPRB ,&
&  0.11339E+01_JPRB, 0.11576E+01_JPRB, 0.11548E+01_JPRB, 0.11693E+01_JPRB, 0.11472E+01_JPRB, 0.11347E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.11960E+01_JPRB, 0.13854E+01_JPRB, 0.21209E+01_JPRB, 0.32097E+01_JPRB, 0.52574E+01_JPRB, 0.81905E+01_JPRB ,&
&  0.78714E+01_JPRB, 0.81743E+01_JPRB, 0.57763E+01_JPRB, 0.37349E+01_JPRB, 0.24393E+01_JPRB, 0.19327E+01_JPRB, 0.17192E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.14152E+01_JPRB, 0.13384E+01_JPRB, 0.13317E+01_JPRB, 0.13974E+01_JPRB, 0.14276E+01_JPRB, 0.14487E+01_JPRB ,&
&  0.14356E+01_JPRB, 0.13636E+01_JPRB, 0.13256E+01_JPRB, 0.13082E+01_JPRB, 0.13137E+01_JPRB, 0.13074E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.12775E+01_JPRB, 0.12721E+01_JPRB, 0.12447E+01_JPRB, 0.12600E+01_JPRB, 0.12724E+01_JPRB, 0.12580E+01_JPRB, 0.11957E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.12197E+01_JPRB, 0.12113E+01_JPRB, 0.12183E+01_JPRB, 0.12209E+01_JPRB, 0.12889E+01_JPRB, 0.12392E+01_JPRB ,&
&  0.12212E+01_JPRB, 0.11885E+01_JPRB, 0.11282E+01_JPRB, 0.10929E+01_JPRB, 0.10742E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.10665E+01_JPRB, 0.10269E+01_JPRB, 0.99738E+00_JPRB, 0.95787E+00_JPRB, 0.90429E+00_JPRB, 0.87716E+00_JPRB, 0.85927E+00_JPRB ,&
&  0.84909E+00_JPRB, 0.82089E+00_JPRB, 0.80060E+00_JPRB, 0.79969E+00_JPRB, 0.80596E+00_JPRB, 0.80239E+00_JPRB, 0.80403E+00_JPRB ,&
&  0.81284E+00_JPRB, 0.81662E+00_JPRB, 0.80657E+00_JPRB, 0.79428E+00_JPRB, 0.78977E+00_JPRB, 0.82221E+00_JPRB, 0.83946E+00_JPRB ,&
&  0.84585E+00_JPRB, 0.83239E+00_JPRB, 0.80731E+00_JPRB, 0.77820E+00_JPRB, 0.77317E+00_JPRB, 0.80063E+00_JPRB, 0.92720E+00_JPRB ,&
&  0.16190E+01_JPRB, 0.21174E+01_JPRB, 0.36915E+01_JPRB, 0.37264E+01_JPRB, 0.26059E+01_JPRB, 0.20851E+01_JPRB, 0.18111E+01_JPRB ,&
&  0.17723E+01_JPRB, 0.16869E+01_JPRB, 0.15219E+01_JPRB, 0.13915E+01_JPRB, 0.12767E+01_JPRB, 0.11810E+01_JPRB, 0.11115E+01_JPRB ,&
&  0.10873E+01_JPRB, 0.10889E+01_JPRB, 0.10910E+01_JPRB, 0.10925E+01_JPRB, 0.10885E+01_JPRB, 0.10872E+01_JPRB, 0.10882E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.10815E+01_JPRB, 0.11177E+01_JPRB, 0.14310E+01_JPRB, 0.27054E+01_JPRB, 0.46202E+01_JPRB, 0.72675E+01_JPRB ,&
&  0.82999E+01_JPRB, 0.77730E+01_JPRB, 0.54203E+01_JPRB, 0.38189E+01_JPRB, 0.25947E+01_JPRB, 0.19544E+01_JPRB, 0.17547E+01_JPRB ,&
&  0.16627E+01_JPRB, 0.15722E+01_JPRB, 0.14493E+01_JPRB, 0.14094E+01_JPRB, 0.14031E+01_JPRB, 0.13899E+01_JPRB, 0.13927E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.13654E+01_JPRB, 0.13413E+01_JPRB, 0.13185E+01_JPRB, 0.12960E+01_JPRB, 0.12695E+01_JPRB, 0.12508E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.12230E+01_JPRB, 0.12086E+01_JPRB, 0.12402E+01_JPRB, 0.13026E+01_JPRB, 0.13091E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.12243E+01_JPRB, 0.11997E+01_JPRB, 0.11910E+01_JPRB, 0.12128E+01_JPRB, 0.11923E+01_JPRB, 0.12908E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.12051E+01_JPRB, 0.11574E+01_JPRB, 0.11120E+01_JPRB, 0.10809E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.10776E+01_JPRB, 0.10323E+01_JPRB, 0.10013E+01_JPRB, 0.98061E+00_JPRB, 0.94685E+00_JPRB, 0.91210E+00_JPRB, 0.90281E+00_JPRB ,&
&  0.89984E+00_JPRB, 0.87990E+00_JPRB, 0.85528E+00_JPRB, 0.82427E+00_JPRB, 0.81602E+00_JPRB, 0.82147E+00_JPRB, 0.82835E+00_JPRB ,&
&  0.82955E+00_JPRB, 0.83145E+00_JPRB, 0.82994E+00_JPRB, 0.82608E+00_JPRB, 0.82124E+00_JPRB, 0.83997E+00_JPRB, 0.84766E+00_JPRB ,&
&  0.83892E+00_JPRB, 0.83807E+00_JPRB, 0.81494E+00_JPRB, 0.79111E+00_JPRB, 0.78101E+00_JPRB, 0.77377E+00_JPRB, 0.79826E+00_JPRB ,&
&  0.12640E+01_JPRB, 0.24972E+01_JPRB, 0.35920E+01_JPRB, 0.32893E+01_JPRB, 0.25539E+01_JPRB, 0.20219E+01_JPRB, 0.18741E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.16584E+01_JPRB, 0.15173E+01_JPRB, 0.14055E+01_JPRB, 0.13121E+01_JPRB, 0.12348E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11020E+01_JPRB, 0.10833E+01_JPRB, 0.10740E+01_JPRB, 0.10715E+01_JPRB, 0.10794E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.10894E+01_JPRB, 0.10901E+01_JPRB, 0.11943E+01_JPRB, 0.19270E+01_JPRB, 0.24712E+01_JPRB, 0.46947E+01_JPRB ,&
&  0.66224E+01_JPRB, 0.53659E+01_JPRB, 0.40623E+01_JPRB, 0.32817E+01_JPRB, 0.25903E+01_JPRB, 0.19950E+01_JPRB, 0.17103E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.15374E+01_JPRB, 0.14522E+01_JPRB, 0.13952E+01_JPRB, 0.13515E+01_JPRB, 0.13292E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.12880E+01_JPRB, 0.12806E+01_JPRB, 0.12730E+01_JPRB, 0.12503E+01_JPRB, 0.12267E+01_JPRB, 0.12063E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.11714E+01_JPRB, 0.11591E+01_JPRB, 0.11879E+01_JPRB, 0.12785E+01_JPRB, 0.13453E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.12734E+01_JPRB, 0.12144E+01_JPRB, 0.12136E+01_JPRB, 0.12059E+01_JPRB, 0.12093E+01_JPRB, 0.13260E+01_JPRB, 0.12782E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.12223E+01_JPRB, 0.11878E+01_JPRB, 0.11441E+01_JPRB, 0.11123E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.11185E+01_JPRB, 0.10688E+01_JPRB, 0.10257E+01_JPRB, 0.98772E+00_JPRB, 0.97468E+00_JPRB, 0.95172E+00_JPRB, 0.92454E+00_JPRB ,&
&  0.91050E+00_JPRB, 0.90977E+00_JPRB, 0.90242E+00_JPRB, 0.88420E+00_JPRB, 0.86778E+00_JPRB, 0.86129E+00_JPRB, 0.86108E+00_JPRB ,&
&  0.85997E+00_JPRB, 0.85895E+00_JPRB, 0.85592E+00_JPRB, 0.84673E+00_JPRB, 0.84807E+00_JPRB, 0.85560E+00_JPRB, 0.86287E+00_JPRB ,&
&  0.84308E+00_JPRB, 0.82672E+00_JPRB, 0.81385E+00_JPRB, 0.80061E+00_JPRB, 0.79669E+00_JPRB, 0.78003E+00_JPRB, 0.76486E+00_JPRB ,&
&  0.96455E+00_JPRB, 0.16809E+01_JPRB, 0.23679E+01_JPRB, 0.29115E+01_JPRB, 0.25244E+01_JPRB, 0.20986E+01_JPRB, 0.19079E+01_JPRB ,&
&  0.17612E+01_JPRB, 0.15990E+01_JPRB, 0.14670E+01_JPRB, 0.13810E+01_JPRB, 0.13294E+01_JPRB, 0.12649E+01_JPRB, 0.11770E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.10794E+01_JPRB, 0.10727E+01_JPRB, 0.10750E+01_JPRB, 0.10725E+01_JPRB, 0.10878E+01_JPRB, 0.11008E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.10941E+01_JPRB, 0.10750E+01_JPRB, 0.10786E+01_JPRB, 0.13223E+01_JPRB, 0.15592E+01_JPRB, 0.23133E+01_JPRB ,&
&  0.27031E+01_JPRB, 0.29001E+01_JPRB, 0.27736E+01_JPRB, 0.26443E+01_JPRB, 0.24077E+01_JPRB, 0.20023E+01_JPRB, 0.16806E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.14361E+01_JPRB, 0.13752E+01_JPRB, 0.13445E+01_JPRB, 0.13122E+01_JPRB, 0.12824E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.12533E+01_JPRB, 0.12379E+01_JPRB, 0.12234E+01_JPRB, 0.12244E+01_JPRB, 0.12053E+01_JPRB, 0.11684E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.11106E+01_JPRB, 0.11229E+01_JPRB, 0.11635E+01_JPRB, 0.12077E+01_JPRB, 0.12409E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12755E+01_JPRB, 0.12446E+01_JPRB, 0.12409E+01_JPRB, 0.12403E+01_JPRB, 0.13330E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.12567E+01_JPRB, 0.12332E+01_JPRB, 0.12036E+01_JPRB, 0.11485E+01_JPRB, 0.11621E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.11184E+01_JPRB, 0.10926E+01_JPRB, 0.10615E+01_JPRB, 0.10181E+01_JPRB, 0.99935E+00_JPRB, 0.98224E+00_JPRB, 0.96353E+00_JPRB ,&
&  0.95403E+00_JPRB, 0.94529E+00_JPRB, 0.93873E+00_JPRB, 0.92354E+00_JPRB, 0.90766E+00_JPRB, 0.89740E+00_JPRB, 0.88517E+00_JPRB ,&
&  0.88490E+00_JPRB, 0.88439E+00_JPRB, 0.87560E+00_JPRB, 0.86615E+00_JPRB, 0.86123E+00_JPRB, 0.85725E+00_JPRB, 0.86014E+00_JPRB ,&
&  0.85068E+00_JPRB, 0.83846E+00_JPRB, 0.83674E+00_JPRB, 0.82778E+00_JPRB, 0.81353E+00_JPRB, 0.80218E+00_JPRB, 0.77270E+00_JPRB ,&
&  0.81301E+00_JPRB, 0.10367E+01_JPRB, 0.15406E+01_JPRB, 0.21089E+01_JPRB, 0.20747E+01_JPRB, 0.19854E+01_JPRB, 0.18138E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.15249E+01_JPRB, 0.13798E+01_JPRB, 0.12829E+01_JPRB, 0.12505E+01_JPRB, 0.12153E+01_JPRB, 0.11740E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.10886E+01_JPRB, 0.10682E+01_JPRB, 0.10749E+01_JPRB, 0.10700E+01_JPRB, 0.10712E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10628E+01_JPRB, 0.10374E+01_JPRB, 0.10150E+01_JPRB, 0.10018E+01_JPRB, 0.10227E+01_JPRB, 0.12148E+01_JPRB ,&
&  0.14995E+01_JPRB, 0.16490E+01_JPRB, 0.17298E+01_JPRB, 0.18253E+01_JPRB, 0.18749E+01_JPRB, 0.18173E+01_JPRB, 0.16461E+01_JPRB ,&
&  0.15042E+01_JPRB, 0.14147E+01_JPRB, 0.13327E+01_JPRB, 0.13254E+01_JPRB, 0.13003E+01_JPRB, 0.12657E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.11923E+01_JPRB, 0.11788E+01_JPRB, 0.11539E+01_JPRB, 0.11355E+01_JPRB, 0.11158E+01_JPRB, 0.10893E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.10479E+01_JPRB, 0.10390E+01_JPRB, 0.10496E+01_JPRB, 0.10838E+01_JPRB, 0.11276E+01_JPRB, 0.11601E+01_JPRB, 0.11941E+01_JPRB ,&
&  0.12213E+01_JPRB, 0.12495E+01_JPRB, 0.12168E+01_JPRB, 0.12032E+01_JPRB, 0.13120E+01_JPRB, 0.13062E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.12329E+01_JPRB, 0.12052E+01_JPRB, 0.11858E+01_JPRB, 0.11215E+01_JPRB, 0.11024E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.99841E+00_JPRB, 0.10202E+01_JPRB, 0.10093E+01_JPRB, 0.97847E+00_JPRB, 0.96119E+00_JPRB, 0.95478E+00_JPRB, 0.95418E+00_JPRB ,&
&  0.95113E+00_JPRB, 0.94137E+00_JPRB, 0.94634E+00_JPRB, 0.95193E+00_JPRB, 0.94764E+00_JPRB, 0.93339E+00_JPRB, 0.92237E+00_JPRB ,&
&  0.91807E+00_JPRB, 0.92029E+00_JPRB, 0.92394E+00_JPRB, 0.91260E+00_JPRB, 0.88988E+00_JPRB, 0.87434E+00_JPRB, 0.86098E+00_JPRB ,&
&  0.85907E+00_JPRB, 0.85848E+00_JPRB, 0.85572E+00_JPRB, 0.85630E+00_JPRB, 0.83398E+00_JPRB, 0.80887E+00_JPRB, 0.78875E+00_JPRB ,&
&  0.78096E+00_JPRB, 0.75924E+00_JPRB, 0.10070E+01_JPRB, 0.13681E+01_JPRB, 0.15399E+01_JPRB, 0.15245E+01_JPRB, 0.14031E+01_JPRB ,&
&  0.13248E+01_JPRB, 0.12680E+01_JPRB, 0.12155E+01_JPRB, 0.11565E+01_JPRB, 0.11200E+01_JPRB, 0.10973E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10604E+01_JPRB, 0.10401E+01_JPRB, 0.10256E+01_JPRB, 0.10041E+01_JPRB, 0.98585E+00_JPRB, 0.98060E+00_JPRB, 0.97917E+00_JPRB ,&
&  0.98220E+00_JPRB, 0.97615E+00_JPRB, 0.97206E+00_JPRB, 0.96555E+00_JPRB, 0.93814E+00_JPRB, 0.93039E+00_JPRB, 0.95169E+00_JPRB ,&
&  0.10388E+01_JPRB, 0.11112E+01_JPRB, 0.11616E+01_JPRB, 0.12689E+01_JPRB, 0.13686E+01_JPRB, 0.14520E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.13995E+01_JPRB, 0.13554E+01_JPRB, 0.13076E+01_JPRB, 0.12857E+01_JPRB, 0.12603E+01_JPRB, 0.12120E+01_JPRB, 0.11610E+01_JPRB ,&
&  0.11307E+01_JPRB, 0.11079E+01_JPRB, 0.10778E+01_JPRB, 0.10529E+01_JPRB, 0.10253E+01_JPRB, 0.99172E+00_JPRB, 0.96251E+00_JPRB ,&
&  0.94201E+00_JPRB, 0.93607E+00_JPRB, 0.94166E+00_JPRB, 0.96475E+00_JPRB, 0.99892E+00_JPRB, 0.10396E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.11458E+01_JPRB, 0.11786E+01_JPRB, 0.12276E+01_JPRB, 0.12277E+01_JPRB, 0.12175E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11555E+01_JPRB, 0.11118E+01_JPRB, 0.10411E+01_JPRB, 0.94943E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.94696E+00_JPRB, 0.96175E+00_JPRB, 0.95885E+00_JPRB, 0.94697E+00_JPRB, 0.93749E+00_JPRB, 0.93228E+00_JPRB, 0.93639E+00_JPRB ,&
&  0.93596E+00_JPRB, 0.93107E+00_JPRB, 0.93392E+00_JPRB, 0.93519E+00_JPRB, 0.93549E+00_JPRB, 0.93066E+00_JPRB, 0.93322E+00_JPRB ,&
&  0.93208E+00_JPRB, 0.93226E+00_JPRB, 0.93430E+00_JPRB, 0.92693E+00_JPRB, 0.90591E+00_JPRB, 0.89077E+00_JPRB, 0.87612E+00_JPRB ,&
&  0.87479E+00_JPRB, 0.87414E+00_JPRB, 0.87437E+00_JPRB, 0.86755E+00_JPRB, 0.84634E+00_JPRB, 0.82763E+00_JPRB, 0.81572E+00_JPRB ,&
&  0.81604E+00_JPRB, 0.68868E+00_JPRB, 0.79867E+00_JPRB, 0.10770E+01_JPRB, 0.12109E+01_JPRB, 0.11542E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11232E+01_JPRB, 0.11183E+01_JPRB, 0.10987E+01_JPRB, 0.10627E+01_JPRB, 0.10086E+01_JPRB, 0.98754E+00_JPRB, 0.96101E+00_JPRB ,&
&  0.93967E+00_JPRB, 0.93065E+00_JPRB, 0.92179E+00_JPRB, 0.91853E+00_JPRB, 0.90496E+00_JPRB, 0.89024E+00_JPRB, 0.88842E+00_JPRB ,&
&  0.89503E+00_JPRB, 0.89524E+00_JPRB, 0.90059E+00_JPRB, 0.90454E+00_JPRB, 0.89645E+00_JPRB, 0.90768E+00_JPRB, 0.91507E+00_JPRB ,&
&  0.93418E+00_JPRB, 0.95329E+00_JPRB, 0.97011E+00_JPRB, 0.10053E+01_JPRB, 0.10649E+01_JPRB, 0.11656E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.12282E+01_JPRB, 0.12103E+01_JPRB, 0.12047E+01_JPRB, 0.11966E+01_JPRB, 0.11776E+01_JPRB, 0.11406E+01_JPRB, 0.11004E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.10403E+01_JPRB, 0.10114E+01_JPRB, 0.98060E+00_JPRB, 0.94973E+00_JPRB, 0.92085E+00_JPRB, 0.89663E+00_JPRB ,&
&  0.88049E+00_JPRB, 0.87363E+00_JPRB, 0.88253E+00_JPRB, 0.90591E+00_JPRB, 0.93740E+00_JPRB, 0.97360E+00_JPRB, 0.10114E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.10947E+01_JPRB, 0.11358E+01_JPRB, 0.10933E+01_JPRB, 0.11565E+01_JPRB, 0.11657E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11038E+01_JPRB, 0.10365E+01_JPRB, 0.83025E+00_JPRB, 0.91035E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.92488E+00_JPRB, 0.92997E+00_JPRB, 0.93275E+00_JPRB, 0.91997E+00_JPRB, 0.90364E+00_JPRB, 0.89758E+00_JPRB, 0.90448E+00_JPRB ,&
&  0.90047E+00_JPRB, 0.88740E+00_JPRB, 0.89198E+00_JPRB, 0.89458E+00_JPRB, 0.90023E+00_JPRB, 0.90195E+00_JPRB, 0.90390E+00_JPRB ,&
&  0.90607E+00_JPRB, 0.90859E+00_JPRB, 0.90607E+00_JPRB, 0.91390E+00_JPRB, 0.90606E+00_JPRB, 0.90008E+00_JPRB, 0.89692E+00_JPRB ,&
&  0.88843E+00_JPRB, 0.87984E+00_JPRB, 0.87246E+00_JPRB, 0.86191E+00_JPRB, 0.85328E+00_JPRB, 0.84272E+00_JPRB, 0.82635E+00_JPRB ,&
&  0.83723E+00_JPRB, 0.69203E+00_JPRB, 0.79534E+00_JPRB, 0.97760E+00_JPRB, 0.97810E+00_JPRB, 0.95796E+00_JPRB, 0.97102E+00_JPRB ,&
&  0.99269E+00_JPRB, 0.10024E+01_JPRB, 0.99441E+00_JPRB, 0.96710E+00_JPRB, 0.91136E+00_JPRB, 0.88497E+00_JPRB, 0.87053E+00_JPRB ,&
&  0.85446E+00_JPRB, 0.84198E+00_JPRB, 0.83401E+00_JPRB, 0.83515E+00_JPRB, 0.83548E+00_JPRB, 0.83238E+00_JPRB, 0.82792E+00_JPRB ,&
&  0.82991E+00_JPRB, 0.83838E+00_JPRB, 0.84314E+00_JPRB, 0.84843E+00_JPRB, 0.85322E+00_JPRB, 0.84741E+00_JPRB, 0.84410E+00_JPRB ,&
&  0.84250E+00_JPRB, 0.84631E+00_JPRB, 0.84980E+00_JPRB, 0.86477E+00_JPRB, 0.89394E+00_JPRB, 0.95792E+00_JPRB, 0.10174E+01_JPRB ,&
&  0.10422E+01_JPRB, 0.10482E+01_JPRB, 0.10499E+01_JPRB, 0.10495E+01_JPRB, 0.10512E+01_JPRB, 0.10440E+01_JPRB, 0.10228E+01_JPRB ,&
&  0.99718E+00_JPRB, 0.96635E+00_JPRB, 0.94193E+00_JPRB, 0.92276E+00_JPRB, 0.90270E+00_JPRB, 0.88779E+00_JPRB, 0.88148E+00_JPRB ,&
&  0.87167E+00_JPRB, 0.85881E+00_JPRB, 0.86206E+00_JPRB, 0.87881E+00_JPRB, 0.89664E+00_JPRB, 0.91228E+00_JPRB, 0.93812E+00_JPRB ,&
&  0.97929E+00_JPRB, 0.10210E+01_JPRB, 0.10549E+01_JPRB, 0.10876E+01_JPRB, 0.11011E+01_JPRB, 0.11037E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.10480E+01_JPRB, 0.78504E+00_JPRB, 0.88880E+00_JPRB, 0.91350E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.92265E+00_JPRB, 0.91303E+00_JPRB, 0.90149E+00_JPRB, 0.88485E+00_JPRB, 0.86780E+00_JPRB, 0.85750E+00_JPRB, 0.86272E+00_JPRB ,&
&  0.86105E+00_JPRB, 0.84885E+00_JPRB, 0.85888E+00_JPRB, 0.87158E+00_JPRB, 0.88143E+00_JPRB, 0.89010E+00_JPRB, 0.89209E+00_JPRB ,&
&  0.89450E+00_JPRB, 0.89066E+00_JPRB, 0.89350E+00_JPRB, 0.90568E+00_JPRB, 0.92452E+00_JPRB, 0.92628E+00_JPRB, 0.92323E+00_JPRB ,&
&  0.90098E+00_JPRB, 0.88984E+00_JPRB, 0.87960E+00_JPRB, 0.86708E+00_JPRB, 0.85537E+00_JPRB, 0.85046E+00_JPRB, 0.85164E+00_JPRB ,&
&  0.73705E+00_JPRB, 0.64607E+00_JPRB, 0.77548E+00_JPRB, 0.87759E+00_JPRB, 0.87234E+00_JPRB, 0.86403E+00_JPRB, 0.88103E+00_JPRB ,&
&  0.88737E+00_JPRB, 0.88477E+00_JPRB, 0.88419E+00_JPRB, 0.87687E+00_JPRB, 0.84645E+00_JPRB, 0.81121E+00_JPRB, 0.79359E+00_JPRB ,&
&  0.78343E+00_JPRB, 0.78853E+00_JPRB, 0.79075E+00_JPRB, 0.79836E+00_JPRB, 0.79856E+00_JPRB, 0.79143E+00_JPRB, 0.78924E+00_JPRB ,&
&  0.78871E+00_JPRB, 0.79719E+00_JPRB, 0.80482E+00_JPRB, 0.81012E+00_JPRB, 0.81143E+00_JPRB, 0.81089E+00_JPRB, 0.80637E+00_JPRB ,&
&  0.80294E+00_JPRB, 0.80106E+00_JPRB, 0.79922E+00_JPRB, 0.80528E+00_JPRB, 0.82104E+00_JPRB, 0.85185E+00_JPRB, 0.89636E+00_JPRB ,&
&  0.92569E+00_JPRB, 0.93781E+00_JPRB, 0.94399E+00_JPRB, 0.94222E+00_JPRB, 0.94958E+00_JPRB, 0.95111E+00_JPRB, 0.93566E+00_JPRB ,&
&  0.91578E+00_JPRB, 0.89966E+00_JPRB, 0.89359E+00_JPRB, 0.88691E+00_JPRB, 0.88262E+00_JPRB, 0.87691E+00_JPRB, 0.86767E+00_JPRB ,&
&  0.85778E+00_JPRB, 0.84433E+00_JPRB, 0.83464E+00_JPRB, 0.83594E+00_JPRB, 0.84884E+00_JPRB, 0.86061E+00_JPRB, 0.88332E+00_JPRB ,&
&  0.92094E+00_JPRB, 0.96457E+00_JPRB, 0.10003E+01_JPRB, 0.10364E+01_JPRB, 0.10560E+01_JPRB, 0.10632E+01_JPRB, 0.10533E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.99046E+00_JPRB, 0.97333E+00_JPRB, 0.94738E+00_JPRB, 0.93381E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.87651E+00_JPRB, 0.86929E+00_JPRB, 0.85512E+00_JPRB, 0.82964E+00_JPRB, 0.79633E+00_JPRB, 0.80420E+00_JPRB, 0.81486E+00_JPRB ,&
&  0.80774E+00_JPRB, 0.79791E+00_JPRB, 0.80850E+00_JPRB, 0.83379E+00_JPRB, 0.85807E+00_JPRB, 0.87162E+00_JPRB, 0.87311E+00_JPRB ,&
&  0.86725E+00_JPRB, 0.86850E+00_JPRB, 0.87653E+00_JPRB, 0.89383E+00_JPRB, 0.89327E+00_JPRB, 0.87550E+00_JPRB, 0.86459E+00_JPRB ,&
&  0.86178E+00_JPRB, 0.86004E+00_JPRB, 0.85762E+00_JPRB, 0.84091E+00_JPRB, 0.83001E+00_JPRB, 0.82697E+00_JPRB, 0.82710E+00_JPRB ,&
&  0.82386E+00_JPRB, 0.68617E+00_JPRB, 0.80314E+00_JPRB, 0.80981E+00_JPRB, 0.81487E+00_JPRB, 0.80211E+00_JPRB, 0.80025E+00_JPRB ,&
&  0.80357E+00_JPRB, 0.81865E+00_JPRB, 0.81257E+00_JPRB, 0.79107E+00_JPRB, 0.76821E+00_JPRB, 0.74174E+00_JPRB, 0.72369E+00_JPRB ,&
&  0.71452E+00_JPRB, 0.71784E+00_JPRB, 0.72830E+00_JPRB, 0.73973E+00_JPRB, 0.75156E+00_JPRB, 0.75354E+00_JPRB, 0.75735E+00_JPRB ,&
&  0.75929E+00_JPRB, 0.76317E+00_JPRB, 0.76421E+00_JPRB, 0.76823E+00_JPRB, 0.76896E+00_JPRB, 0.76487E+00_JPRB, 0.75655E+00_JPRB ,&
&  0.74471E+00_JPRB, 0.73808E+00_JPRB, 0.73833E+00_JPRB, 0.74223E+00_JPRB, 0.75523E+00_JPRB, 0.77967E+00_JPRB, 0.80953E+00_JPRB ,&
&  0.83995E+00_JPRB, 0.85978E+00_JPRB, 0.87085E+00_JPRB, 0.87329E+00_JPRB, 0.87969E+00_JPRB, 0.88572E+00_JPRB, 0.88000E+00_JPRB ,&
&  0.86580E+00_JPRB, 0.85034E+00_JPRB, 0.84051E+00_JPRB, 0.83019E+00_JPRB, 0.82465E+00_JPRB, 0.82026E+00_JPRB, 0.81298E+00_JPRB ,&
&  0.80415E+00_JPRB, 0.80316E+00_JPRB, 0.80551E+00_JPRB, 0.79598E+00_JPRB, 0.79921E+00_JPRB, 0.81149E+00_JPRB, 0.83310E+00_JPRB ,&
&  0.86000E+00_JPRB, 0.89484E+00_JPRB, 0.92858E+00_JPRB, 0.95519E+00_JPRB, 0.97025E+00_JPRB, 0.97372E+00_JPRB, 0.96638E+00_JPRB ,&
&  0.94748E+00_JPRB, 0.92208E+00_JPRB, 0.90290E+00_JPRB, 0.89224E+00_JPRB, 0.89218E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.82900E+00_JPRB, 0.82940E+00_JPRB, 0.81916E+00_JPRB, 0.78130E+00_JPRB, 0.75240E+00_JPRB, 0.76137E+00_JPRB, 0.76942E+00_JPRB ,&
&  0.76879E+00_JPRB, 0.78402E+00_JPRB, 0.80918E+00_JPRB, 0.83451E+00_JPRB, 0.85138E+00_JPRB, 0.85085E+00_JPRB, 0.85430E+00_JPRB ,&
&  0.85117E+00_JPRB, 0.84869E+00_JPRB, 0.85844E+00_JPRB, 0.86337E+00_JPRB, 0.85267E+00_JPRB, 0.83180E+00_JPRB, 0.82981E+00_JPRB ,&
&  0.83617E+00_JPRB, 0.83187E+00_JPRB, 0.81371E+00_JPRB, 0.80146E+00_JPRB, 0.79161E+00_JPRB, 0.79723E+00_JPRB, 0.80486E+00_JPRB ,&
&  0.80016E+00_JPRB, 0.75797E+00_JPRB, 0.79689E+00_JPRB, 0.80481E+00_JPRB, 0.80664E+00_JPRB, 0.78886E+00_JPRB, 0.79132E+00_JPRB ,&
&  0.80088E+00_JPRB, 0.80661E+00_JPRB, 0.78707E+00_JPRB, 0.76515E+00_JPRB, 0.74608E+00_JPRB, 0.72976E+00_JPRB, 0.70284E+00_JPRB ,&
&  0.69299E+00_JPRB, 0.69021E+00_JPRB, 0.70404E+00_JPRB, 0.71889E+00_JPRB, 0.73609E+00_JPRB, 0.74338E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.75575E+00_JPRB, 0.74936E+00_JPRB, 0.74317E+00_JPRB, 0.73786E+00_JPRB, 0.73596E+00_JPRB, 0.72944E+00_JPRB, 0.71972E+00_JPRB ,&
&  0.71136E+00_JPRB, 0.70994E+00_JPRB, 0.71535E+00_JPRB, 0.72548E+00_JPRB, 0.74013E+00_JPRB, 0.76576E+00_JPRB, 0.78862E+00_JPRB ,&
&  0.81505E+00_JPRB, 0.82644E+00_JPRB, 0.83513E+00_JPRB, 0.84887E+00_JPRB, 0.85560E+00_JPRB, 0.85256E+00_JPRB, 0.84774E+00_JPRB ,&
&  0.84024E+00_JPRB, 0.82945E+00_JPRB, 0.82970E+00_JPRB, 0.82461E+00_JPRB, 0.82142E+00_JPRB, 0.80986E+00_JPRB, 0.79238E+00_JPRB ,&
&  0.78551E+00_JPRB, 0.78104E+00_JPRB, 0.77326E+00_JPRB, 0.77098E+00_JPRB, 0.76880E+00_JPRB, 0.78767E+00_JPRB, 0.82080E+00_JPRB ,&
&  0.83973E+00_JPRB, 0.86480E+00_JPRB, 0.89983E+00_JPRB, 0.92195E+00_JPRB, 0.92129E+00_JPRB, 0.91904E+00_JPRB, 0.90606E+00_JPRB ,&
&  0.88620E+00_JPRB, 0.87206E+00_JPRB, 0.85903E+00_JPRB, 0.85408E+00_JPRB, 0.84977E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.78142E+00_JPRB, 0.80337E+00_JPRB, 0.77116E+00_JPRB, 0.75366E+00_JPRB, 0.73281E+00_JPRB, 0.72636E+00_JPRB, 0.71682E+00_JPRB ,&
&  0.72805E+00_JPRB, 0.74573E+00_JPRB, 0.76982E+00_JPRB, 0.77777E+00_JPRB, 0.78441E+00_JPRB, 0.78697E+00_JPRB, 0.80330E+00_JPRB ,&
&  0.79805E+00_JPRB, 0.78946E+00_JPRB, 0.79233E+00_JPRB, 0.78185E+00_JPRB, 0.77308E+00_JPRB, 0.77837E+00_JPRB, 0.78802E+00_JPRB ,&
&  0.79858E+00_JPRB, 0.78983E+00_JPRB, 0.77228E+00_JPRB, 0.74576E+00_JPRB, 0.73327E+00_JPRB, 0.73058E+00_JPRB, 0.73839E+00_JPRB ,&
&  0.74449E+00_JPRB, 0.77366E+00_JPRB, 0.70791E+00_JPRB, 0.76446E+00_JPRB, 0.76910E+00_JPRB, 0.76342E+00_JPRB, 0.76013E+00_JPRB ,&
&  0.76358E+00_JPRB, 0.75260E+00_JPRB, 0.74400E+00_JPRB, 0.73620E+00_JPRB, 0.71347E+00_JPRB, 0.69536E+00_JPRB, 0.67654E+00_JPRB ,&
&  0.65999E+00_JPRB, 0.64982E+00_JPRB, 0.65550E+00_JPRB, 0.67389E+00_JPRB, 0.69257E+00_JPRB, 0.70288E+00_JPRB, 0.70898E+00_JPRB ,&
&  0.71214E+00_JPRB, 0.70466E+00_JPRB, 0.69431E+00_JPRB, 0.69382E+00_JPRB, 0.69950E+00_JPRB, 0.70364E+00_JPRB, 0.70572E+00_JPRB ,&
&  0.70512E+00_JPRB, 0.68302E+00_JPRB, 0.67101E+00_JPRB, 0.68832E+00_JPRB, 0.70844E+00_JPRB, 0.73223E+00_JPRB, 0.76306E+00_JPRB ,&
&  0.78722E+00_JPRB, 0.79720E+00_JPRB, 0.79818E+00_JPRB, 0.80484E+00_JPRB, 0.81010E+00_JPRB, 0.81772E+00_JPRB, 0.82330E+00_JPRB ,&
&  0.80959E+00_JPRB, 0.80481E+00_JPRB, 0.80436E+00_JPRB, 0.80304E+00_JPRB, 0.79202E+00_JPRB, 0.76684E+00_JPRB, 0.73291E+00_JPRB ,&
&  0.71711E+00_JPRB, 0.70194E+00_JPRB, 0.69113E+00_JPRB, 0.68736E+00_JPRB, 0.68803E+00_JPRB, 0.71481E+00_JPRB, 0.75399E+00_JPRB ,&
&  0.79316E+00_JPRB, 0.82459E+00_JPRB, 0.85170E+00_JPRB, 0.87632E+00_JPRB, 0.87513E+00_JPRB, 0.85292E+00_JPRB, 0.84267E+00_JPRB ,&
&  0.82689E+00_JPRB, 0.80971E+00_JPRB, 0.79781E+00_JPRB, 0.78872E+00_JPRB, 0.77946E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.78069E+00_JPRB, 0.78261E+00_JPRB, 0.76844E+00_JPRB, 0.75865E+00_JPRB, 0.74105E+00_JPRB, 0.71778E+00_JPRB, 0.69791E+00_JPRB ,&
&  0.70004E+00_JPRB, 0.71615E+00_JPRB, 0.73149E+00_JPRB, 0.74244E+00_JPRB, 0.75778E+00_JPRB, 0.76068E+00_JPRB, 0.76659E+00_JPRB ,&
&  0.76156E+00_JPRB, 0.72408E+00_JPRB, 0.71803E+00_JPRB, 0.72554E+00_JPRB, 0.72740E+00_JPRB, 0.73385E+00_JPRB, 0.74524E+00_JPRB ,&
&  0.75563E+00_JPRB, 0.75112E+00_JPRB, 0.74283E+00_JPRB, 0.71302E+00_JPRB, 0.70620E+00_JPRB, 0.71105E+00_JPRB, 0.71598E+00_JPRB ,&
&  0.72617E+00_JPRB, 0.74412E+00_JPRB, 0.74599E+00_JPRB, 0.75457E+00_JPRB, 0.76159E+00_JPRB, 0.76407E+00_JPRB, 0.75981E+00_JPRB ,&
&  0.73286E+00_JPRB, 0.70614E+00_JPRB, 0.69509E+00_JPRB, 0.67788E+00_JPRB, 0.65527E+00_JPRB, 0.63943E+00_JPRB, 0.63214E+00_JPRB ,&
&  0.62865E+00_JPRB, 0.63419E+00_JPRB, 0.64057E+00_JPRB, 0.65467E+00_JPRB, 0.65999E+00_JPRB, 0.66419E+00_JPRB, 0.66972E+00_JPRB ,&
&  0.66974E+00_JPRB, 0.66618E+00_JPRB, 0.66665E+00_JPRB, 0.67660E+00_JPRB, 0.68208E+00_JPRB, 0.67909E+00_JPRB, 0.67387E+00_JPRB ,&
&  0.66062E+00_JPRB, 0.64810E+00_JPRB, 0.65632E+00_JPRB, 0.67268E+00_JPRB, 0.68150E+00_JPRB, 0.70973E+00_JPRB, 0.74588E+00_JPRB ,&
&  0.77679E+00_JPRB, 0.79632E+00_JPRB, 0.79649E+00_JPRB, 0.79125E+00_JPRB, 0.78777E+00_JPRB, 0.79420E+00_JPRB, 0.80552E+00_JPRB ,&
&  0.78855E+00_JPRB, 0.77899E+00_JPRB, 0.77661E+00_JPRB, 0.75663E+00_JPRB, 0.71905E+00_JPRB, 0.68574E+00_JPRB, 0.66147E+00_JPRB ,&
&  0.64241E+00_JPRB, 0.63172E+00_JPRB, 0.63552E+00_JPRB, 0.63842E+00_JPRB, 0.65029E+00_JPRB, 0.68613E+00_JPRB, 0.72426E+00_JPRB ,&
&  0.76586E+00_JPRB, 0.78752E+00_JPRB, 0.79496E+00_JPRB, 0.80368E+00_JPRB, 0.80995E+00_JPRB, 0.79878E+00_JPRB, 0.78917E+00_JPRB ,&
&  0.78610E+00_JPRB, 0.77728E+00_JPRB, 0.76788E+00_JPRB, 0.75860E+00_JPRB, 0.76669E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.77328E+00_JPRB, 0.76655E+00_JPRB, 0.75378E+00_JPRB, 0.72699E+00_JPRB, 0.70864E+00_JPRB, 0.68130E+00_JPRB, 0.66604E+00_JPRB ,&
&  0.66864E+00_JPRB, 0.67048E+00_JPRB, 0.67575E+00_JPRB, 0.68478E+00_JPRB, 0.68227E+00_JPRB, 0.67720E+00_JPRB, 0.67206E+00_JPRB ,&
&  0.67461E+00_JPRB, 0.67497E+00_JPRB, 0.68346E+00_JPRB, 0.69876E+00_JPRB, 0.71489E+00_JPRB, 0.72434E+00_JPRB, 0.73475E+00_JPRB ,&
&  0.74274E+00_JPRB, 0.73681E+00_JPRB, 0.73220E+00_JPRB, 0.70112E+00_JPRB, 0.68302E+00_JPRB, 0.68794E+00_JPRB, 0.69540E+00_JPRB ,&
&  0.70546E+00_JPRB, 0.71616E+00_JPRB, 0.72491E+00_JPRB, 0.73124E+00_JPRB, 0.71717E+00_JPRB, 0.71258E+00_JPRB, 0.70119E+00_JPRB ,&
&  0.65696E+00_JPRB, 0.61387E+00_JPRB, 0.59048E+00_JPRB, 0.58208E+00_JPRB, 0.57593E+00_JPRB, 0.58163E+00_JPRB, 0.58987E+00_JPRB ,&
&  0.60400E+00_JPRB, 0.61672E+00_JPRB, 0.61983E+00_JPRB, 0.62258E+00_JPRB, 0.62394E+00_JPRB, 0.62325E+00_JPRB, 0.62429E+00_JPRB ,&
&  0.62249E+00_JPRB, 0.62190E+00_JPRB, 0.62957E+00_JPRB, 0.63019E+00_JPRB, 0.62101E+00_JPRB, 0.61783E+00_JPRB, 0.61531E+00_JPRB ,&
&  0.61324E+00_JPRB, 0.61149E+00_JPRB, 0.60868E+00_JPRB, 0.61945E+00_JPRB, 0.64594E+00_JPRB, 0.67620E+00_JPRB, 0.70227E+00_JPRB ,&
&  0.71785E+00_JPRB, 0.73076E+00_JPRB, 0.73399E+00_JPRB, 0.73543E+00_JPRB, 0.73470E+00_JPRB, 0.72549E+00_JPRB, 0.72173E+00_JPRB ,&
&  0.71395E+00_JPRB, 0.69466E+00_JPRB, 0.66860E+00_JPRB, 0.62831E+00_JPRB, 0.58884E+00_JPRB, 0.56583E+00_JPRB, 0.56477E+00_JPRB ,&
&  0.57660E+00_JPRB, 0.58906E+00_JPRB, 0.59533E+00_JPRB, 0.60303E+00_JPRB, 0.61712E+00_JPRB, 0.63425E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.66442E+00_JPRB, 0.66477E+00_JPRB, 0.66291E+00_JPRB, 0.67149E+00_JPRB, 0.68290E+00_JPRB, 0.70130E+00_JPRB, 0.72143E+00_JPRB ,&
&  0.73893E+00_JPRB, 0.73894E+00_JPRB, 0.73965E+00_JPRB, 0.74139E+00_JPRB, 0.76186E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.76575E+00_JPRB, 0.76497E+00_JPRB, 0.73655E+00_JPRB, 0.70469E+00_JPRB, 0.68618E+00_JPRB, 0.66583E+00_JPRB, 0.64951E+00_JPRB ,&
&  0.64787E+00_JPRB, 0.64483E+00_JPRB, 0.64205E+00_JPRB, 0.63860E+00_JPRB, 0.63625E+00_JPRB, 0.63297E+00_JPRB, 0.62088E+00_JPRB ,&
&  0.61633E+00_JPRB, 0.62263E+00_JPRB, 0.64474E+00_JPRB, 0.66950E+00_JPRB, 0.68935E+00_JPRB, 0.71555E+00_JPRB, 0.72711E+00_JPRB ,&
&  0.73954E+00_JPRB, 0.75247E+00_JPRB, 0.75449E+00_JPRB, 0.72075E+00_JPRB, 0.69304E+00_JPRB, 0.69402E+00_JPRB, 0.70534E+00_JPRB ,&
&  0.70587E+00_JPRB, 0.70899E+00_JPRB, 0.71148E+00_JPRB, 0.70827E+00_JPRB, 0.68746E+00_JPRB, 0.64911E+00_JPRB, 0.61829E+00_JPRB ,&
&  0.59208E+00_JPRB, 0.57157E+00_JPRB, 0.56441E+00_JPRB, 0.55165E+00_JPRB, 0.54885E+00_JPRB, 0.55886E+00_JPRB, 0.57330E+00_JPRB ,&
&  0.58849E+00_JPRB, 0.59570E+00_JPRB, 0.60280E+00_JPRB, 0.59884E+00_JPRB, 0.59460E+00_JPRB, 0.59776E+00_JPRB, 0.59795E+00_JPRB ,&
&  0.59785E+00_JPRB, 0.58886E+00_JPRB, 0.58592E+00_JPRB, 0.57885E+00_JPRB, 0.57488E+00_JPRB, 0.57496E+00_JPRB, 0.57775E+00_JPRB ,&
&  0.57922E+00_JPRB, 0.58524E+00_JPRB, 0.58728E+00_JPRB, 0.58982E+00_JPRB, 0.59949E+00_JPRB, 0.61628E+00_JPRB, 0.63297E+00_JPRB ,&
&  0.64553E+00_JPRB, 0.65548E+00_JPRB, 0.66302E+00_JPRB, 0.66900E+00_JPRB, 0.66901E+00_JPRB, 0.66432E+00_JPRB, 0.65037E+00_JPRB ,&
&  0.62005E+00_JPRB, 0.58751E+00_JPRB, 0.56257E+00_JPRB, 0.54264E+00_JPRB, 0.53927E+00_JPRB, 0.55041E+00_JPRB, 0.56810E+00_JPRB ,&
&  0.58141E+00_JPRB, 0.58479E+00_JPRB, 0.57801E+00_JPRB, 0.57547E+00_JPRB, 0.57929E+00_JPRB, 0.58809E+00_JPRB, 0.59500E+00_JPRB ,&
&  0.59391E+00_JPRB, 0.59091E+00_JPRB, 0.59538E+00_JPRB, 0.60346E+00_JPRB, 0.61872E+00_JPRB, 0.63757E+00_JPRB, 0.65772E+00_JPRB ,&
&  0.67754E+00_JPRB, 0.69278E+00_JPRB, 0.71220E+00_JPRB, 0.73187E+00_JPRB, 0.75732E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.74642E+00_JPRB, 0.71886E+00_JPRB, 0.68966E+00_JPRB, 0.66479E+00_JPRB, 0.65330E+00_JPRB, 0.63610E+00_JPRB, 0.62149E+00_JPRB ,&
&  0.62053E+00_JPRB, 0.62805E+00_JPRB, 0.62273E+00_JPRB, 0.61604E+00_JPRB, 0.60643E+00_JPRB, 0.59877E+00_JPRB, 0.58987E+00_JPRB ,&
&  0.58576E+00_JPRB, 0.59674E+00_JPRB, 0.62202E+00_JPRB, 0.63430E+00_JPRB, 0.64017E+00_JPRB, 0.67048E+00_JPRB, 0.70207E+00_JPRB ,&
&  0.71184E+00_JPRB, 0.72629E+00_JPRB, 0.73028E+00_JPRB, 0.70418E+00_JPRB, 0.68711E+00_JPRB, 0.68052E+00_JPRB, 0.68427E+00_JPRB ,&
&  0.67968E+00_JPRB, 0.68380E+00_JPRB, 0.67507E+00_JPRB, 0.60627E+00_JPRB, 0.60495E+00_JPRB, 0.57442E+00_JPRB, 0.56341E+00_JPRB ,&
&  0.55721E+00_JPRB, 0.56142E+00_JPRB, 0.55642E+00_JPRB, 0.54616E+00_JPRB, 0.54207E+00_JPRB, 0.54552E+00_JPRB, 0.54783E+00_JPRB ,&
&  0.54450E+00_JPRB, 0.53455E+00_JPRB, 0.53929E+00_JPRB, 0.54568E+00_JPRB, 0.54930E+00_JPRB, 0.55653E+00_JPRB, 0.56721E+00_JPRB ,&
&  0.55703E+00_JPRB, 0.54732E+00_JPRB, 0.53874E+00_JPRB, 0.53949E+00_JPRB, 0.54653E+00_JPRB, 0.56081E+00_JPRB, 0.56531E+00_JPRB ,&
&  0.56320E+00_JPRB, 0.56227E+00_JPRB, 0.56380E+00_JPRB, 0.57049E+00_JPRB, 0.58504E+00_JPRB, 0.59946E+00_JPRB, 0.60060E+00_JPRB ,&
&  0.60194E+00_JPRB, 0.60691E+00_JPRB, 0.61264E+00_JPRB, 0.61393E+00_JPRB, 0.60947E+00_JPRB, 0.59411E+00_JPRB, 0.57642E+00_JPRB ,&
&  0.55977E+00_JPRB, 0.54191E+00_JPRB, 0.52717E+00_JPRB, 0.52309E+00_JPRB, 0.53724E+00_JPRB, 0.56736E+00_JPRB, 0.58434E+00_JPRB ,&
&  0.58864E+00_JPRB, 0.59071E+00_JPRB, 0.59352E+00_JPRB, 0.59373E+00_JPRB, 0.59491E+00_JPRB, 0.59228E+00_JPRB, 0.58487E+00_JPRB ,&
&  0.57308E+00_JPRB, 0.56235E+00_JPRB, 0.56079E+00_JPRB, 0.56665E+00_JPRB, 0.57326E+00_JPRB, 0.58701E+00_JPRB, 0.60493E+00_JPRB ,&
&  0.62777E+00_JPRB, 0.66633E+00_JPRB, 0.69851E+00_JPRB, 0.72331E+00_JPRB, 0.74105E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.72310E+00_JPRB, 0.69662E+00_JPRB, 0.67888E+00_JPRB, 0.66516E+00_JPRB, 0.64692E+00_JPRB, 0.63296E+00_JPRB, 0.62727E+00_JPRB ,&
&  0.63531E+00_JPRB, 0.63359E+00_JPRB, 0.62348E+00_JPRB, 0.61507E+00_JPRB, 0.59911E+00_JPRB, 0.58491E+00_JPRB, 0.57915E+00_JPRB ,&
&  0.58742E+00_JPRB, 0.60328E+00_JPRB, 0.62289E+00_JPRB, 0.63036E+00_JPRB, 0.62926E+00_JPRB, 0.63767E+00_JPRB, 0.65706E+00_JPRB ,&
&  0.68591E+00_JPRB, 0.70613E+00_JPRB, 0.71277E+00_JPRB, 0.69882E+00_JPRB, 0.68068E+00_JPRB, 0.67226E+00_JPRB, 0.65805E+00_JPRB ,&
&  0.64134E+00_JPRB, 0.64409E+00_JPRB, 0.57735E+00_JPRB, 0.56963E+00_JPRB, 0.54674E+00_JPRB, 0.53193E+00_JPRB, 0.52772E+00_JPRB ,&
&  0.54027E+00_JPRB, 0.54347E+00_JPRB, 0.53429E+00_JPRB, 0.52143E+00_JPRB, 0.51308E+00_JPRB, 0.50222E+00_JPRB, 0.49251E+00_JPRB ,&
&  0.48621E+00_JPRB, 0.48918E+00_JPRB, 0.49827E+00_JPRB, 0.50763E+00_JPRB, 0.51845E+00_JPRB, 0.52893E+00_JPRB, 0.52557E+00_JPRB ,&
&  0.51390E+00_JPRB, 0.51205E+00_JPRB, 0.51964E+00_JPRB, 0.53514E+00_JPRB, 0.55203E+00_JPRB, 0.55972E+00_JPRB, 0.56266E+00_JPRB ,&
&  0.55368E+00_JPRB, 0.53629E+00_JPRB, 0.52151E+00_JPRB, 0.50944E+00_JPRB, 0.48204E+00_JPRB, 0.41962E+00_JPRB, 0.39726E+00_JPRB ,&
&  0.40881E+00_JPRB, 0.46688E+00_JPRB, 0.53563E+00_JPRB, 0.52840E+00_JPRB, 0.52559E+00_JPRB, 0.51104E+00_JPRB, 0.47347E+00_JPRB ,&
&  0.42863E+00_JPRB, 0.36772E+00_JPRB, 0.34772E+00_JPRB, 0.34481E+00_JPRB, 0.34469E+00_JPRB, 0.35185E+00_JPRB, 0.36473E+00_JPRB ,&
&  0.38159E+00_JPRB, 0.40713E+00_JPRB, 0.42385E+00_JPRB, 0.42983E+00_JPRB, 0.42757E+00_JPRB, 0.41130E+00_JPRB, 0.39705E+00_JPRB ,&
&  0.38813E+00_JPRB, 0.41393E+00_JPRB, 0.47399E+00_JPRB, 0.55631E+00_JPRB, 0.55762E+00_JPRB, 0.58437E+00_JPRB, 0.61042E+00_JPRB ,&
&  0.63507E+00_JPRB, 0.67564E+00_JPRB, 0.70171E+00_JPRB, 0.70719E+00_JPRB, 0.72538E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.68217E+00_JPRB, 0.66975E+00_JPRB, 0.66116E+00_JPRB, 0.65507E+00_JPRB, 0.64797E+00_JPRB, 0.64648E+00_JPRB, 0.64559E+00_JPRB ,&
&  0.63962E+00_JPRB, 0.63106E+00_JPRB, 0.63198E+00_JPRB, 0.61494E+00_JPRB, 0.59057E+00_JPRB, 0.57426E+00_JPRB, 0.57198E+00_JPRB ,&
&  0.57569E+00_JPRB, 0.58274E+00_JPRB, 0.59161E+00_JPRB, 0.59968E+00_JPRB, 0.60439E+00_JPRB, 0.61607E+00_JPRB, 0.63689E+00_JPRB ,&
&  0.66103E+00_JPRB, 0.67457E+00_JPRB, 0.67844E+00_JPRB, 0.67001E+00_JPRB, 0.65749E+00_JPRB, 0.64467E+00_JPRB, 0.63342E+00_JPRB ,&
&  0.62286E+00_JPRB, 0.52125E+00_JPRB, 0.45485E+00_JPRB, 0.46232E+00_JPRB, 0.52874E+00_JPRB, 0.51229E+00_JPRB, 0.51119E+00_JPRB ,&
&  0.52357E+00_JPRB, 0.52280E+00_JPRB, 0.50902E+00_JPRB, 0.49166E+00_JPRB, 0.47561E+00_JPRB, 0.46419E+00_JPRB, 0.45631E+00_JPRB ,&
&  0.45299E+00_JPRB, 0.46006E+00_JPRB, 0.46784E+00_JPRB, 0.47984E+00_JPRB, 0.48239E+00_JPRB, 0.46912E+00_JPRB, 0.45314E+00_JPRB ,&
&  0.43795E+00_JPRB, 0.43245E+00_JPRB, 0.43509E+00_JPRB, 0.44239E+00_JPRB, 0.45197E+00_JPRB, 0.45944E+00_JPRB, 0.45705E+00_JPRB ,&
&  0.42892E+00_JPRB, 0.40558E+00_JPRB, 0.37731E+00_JPRB, 0.35059E+00_JPRB, 0.32747E+00_JPRB, 0.31206E+00_JPRB, 0.30396E+00_JPRB ,&
&  0.30234E+00_JPRB, 0.31078E+00_JPRB, 0.33285E+00_JPRB, 0.38211E+00_JPRB, 0.44566E+00_JPRB, 0.36622E+00_JPRB, 0.32759E+00_JPRB ,&
&  0.30664E+00_JPRB, 0.29331E+00_JPRB, 0.28819E+00_JPRB, 0.28922E+00_JPRB, 0.29391E+00_JPRB, 0.30061E+00_JPRB, 0.30589E+00_JPRB ,&
&  0.31016E+00_JPRB, 0.31355E+00_JPRB, 0.31486E+00_JPRB, 0.31535E+00_JPRB, 0.31658E+00_JPRB, 0.31765E+00_JPRB, 0.31648E+00_JPRB ,&
&  0.31531E+00_JPRB, 0.32244E+00_JPRB, 0.34122E+00_JPRB, 0.37048E+00_JPRB, 0.40407E+00_JPRB, 0.44084E+00_JPRB, 0.49359E+00_JPRB ,&
&  0.55904E+00_JPRB, 0.63188E+00_JPRB, 0.65118E+00_JPRB, 0.66537E+00_JPRB, 0.67617E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.62271E+00_JPRB, 0.62619E+00_JPRB, 0.62991E+00_JPRB, 0.63188E+00_JPRB, 0.62628E+00_JPRB, 0.61875E+00_JPRB, 0.61408E+00_JPRB ,&
&  0.61926E+00_JPRB, 0.61943E+00_JPRB, 0.59627E+00_JPRB, 0.57835E+00_JPRB, 0.56818E+00_JPRB, 0.56454E+00_JPRB, 0.56168E+00_JPRB ,&
&  0.55538E+00_JPRB, 0.55308E+00_JPRB, 0.55398E+00_JPRB, 0.56480E+00_JPRB, 0.57383E+00_JPRB, 0.58168E+00_JPRB, 0.58892E+00_JPRB ,&
&  0.58365E+00_JPRB, 0.59548E+00_JPRB, 0.61676E+00_JPRB, 0.61551E+00_JPRB, 0.61242E+00_JPRB, 0.60491E+00_JPRB, 0.59014E+00_JPRB ,&
&  0.54102E+00_JPRB, 0.47684E+00_JPRB, 0.41753E+00_JPRB, 0.43005E+00_JPRB, 0.50484E+00_JPRB, 0.51206E+00_JPRB, 0.49817E+00_JPRB ,&
&  0.50889E+00_JPRB, 0.50395E+00_JPRB, 0.48587E+00_JPRB, 0.47227E+00_JPRB, 0.45663E+00_JPRB, 0.44648E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.43598E+00_JPRB, 0.43533E+00_JPRB, 0.41939E+00_JPRB, 0.36957E+00_JPRB, 0.34163E+00_JPRB, 0.32409E+00_JPRB, 0.30592E+00_JPRB ,&
&  0.29139E+00_JPRB, 0.28267E+00_JPRB, 0.28157E+00_JPRB, 0.28190E+00_JPRB, 0.28733E+00_JPRB, 0.29025E+00_JPRB, 0.29293E+00_JPRB ,&
&  0.29327E+00_JPRB, 0.29233E+00_JPRB, 0.28836E+00_JPRB, 0.28389E+00_JPRB, 0.28071E+00_JPRB, 0.28044E+00_JPRB, 0.28005E+00_JPRB ,&
&  0.28286E+00_JPRB, 0.28675E+00_JPRB, 0.30385E+00_JPRB, 0.32183E+00_JPRB, 0.31885E+00_JPRB, 0.30370E+00_JPRB, 0.28960E+00_JPRB ,&
&  0.27914E+00_JPRB, 0.27392E+00_JPRB, 0.27047E+00_JPRB, 0.26992E+00_JPRB, 0.27320E+00_JPRB, 0.27795E+00_JPRB, 0.27912E+00_JPRB ,&
&  0.27883E+00_JPRB, 0.27943E+00_JPRB, 0.27941E+00_JPRB, 0.27981E+00_JPRB, 0.28037E+00_JPRB, 0.28310E+00_JPRB, 0.28833E+00_JPRB ,&
&  0.29442E+00_JPRB, 0.30543E+00_JPRB, 0.31779E+00_JPRB, 0.33434E+00_JPRB, 0.34648E+00_JPRB, 0.35579E+00_JPRB, 0.36873E+00_JPRB ,&
&  0.38716E+00_JPRB, 0.42691E+00_JPRB, 0.49591E+00_JPRB, 0.62027E+00_JPRB, 0.61401E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.58988E+00_JPRB, 0.60116E+00_JPRB, 0.61310E+00_JPRB, 0.60473E+00_JPRB, 0.59131E+00_JPRB, 0.58375E+00_JPRB, 0.58362E+00_JPRB ,&
&  0.58356E+00_JPRB, 0.58067E+00_JPRB, 0.56871E+00_JPRB, 0.55504E+00_JPRB, 0.53679E+00_JPRB, 0.48910E+00_JPRB, 0.47181E+00_JPRB ,&
&  0.45438E+00_JPRB, 0.45704E+00_JPRB, 0.46109E+00_JPRB, 0.46716E+00_JPRB, 0.48049E+00_JPRB, 0.49093E+00_JPRB, 0.49232E+00_JPRB ,&
&  0.48177E+00_JPRB, 0.46321E+00_JPRB, 0.44515E+00_JPRB, 0.43138E+00_JPRB, 0.43094E+00_JPRB, 0.43065E+00_JPRB, 0.42336E+00_JPRB ,&
&  0.40991E+00_JPRB, 0.40423E+00_JPRB, 0.40418E+00_JPRB, 0.42392E+00_JPRB, 0.47465E+00_JPRB, 0.49300E+00_JPRB, 0.46890E+00_JPRB ,&
&  0.47344E+00_JPRB, 0.46822E+00_JPRB, 0.46404E+00_JPRB, 0.46503E+00_JPRB, 0.45445E+00_JPRB, 0.43317E+00_JPRB, 0.39920E+00_JPRB ,&
&  0.35671E+00_JPRB, 0.33469E+00_JPRB, 0.31346E+00_JPRB, 0.29283E+00_JPRB, 0.27630E+00_JPRB, 0.26679E+00_JPRB, 0.25870E+00_JPRB ,&
&  0.25346E+00_JPRB, 0.24897E+00_JPRB, 0.24608E+00_JPRB, 0.24618E+00_JPRB, 0.24810E+00_JPRB, 0.24914E+00_JPRB, 0.25038E+00_JPRB ,&
&  0.25215E+00_JPRB, 0.25326E+00_JPRB, 0.25394E+00_JPRB, 0.25557E+00_JPRB, 0.25725E+00_JPRB, 0.26025E+00_JPRB, 0.26416E+00_JPRB ,&
&  0.26904E+00_JPRB, 0.27282E+00_JPRB, 0.27641E+00_JPRB, 0.28307E+00_JPRB, 0.28371E+00_JPRB, 0.28233E+00_JPRB, 0.27419E+00_JPRB ,&
&  0.26817E+00_JPRB, 0.26558E+00_JPRB, 0.26285E+00_JPRB, 0.26214E+00_JPRB, 0.26343E+00_JPRB, 0.26563E+00_JPRB, 0.26784E+00_JPRB ,&
&  0.26940E+00_JPRB, 0.26944E+00_JPRB, 0.26848E+00_JPRB, 0.26826E+00_JPRB, 0.26876E+00_JPRB, 0.26887E+00_JPRB, 0.27278E+00_JPRB ,&
&  0.28136E+00_JPRB, 0.29385E+00_JPRB, 0.30772E+00_JPRB, 0.32644E+00_JPRB, 0.34417E+00_JPRB, 0.36094E+00_JPRB, 0.38890E+00_JPRB ,&
&  0.42752E+00_JPRB, 0.53377E+00_JPRB, 0.60220E+00_JPRB, 0.54443E+00_JPRB, 0.56943E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.56245E+00_JPRB, 0.57002E+00_JPRB, 0.58064E+00_JPRB, 0.58519E+00_JPRB, 0.59304E+00_JPRB, 0.58139E+00_JPRB, 0.57406E+00_JPRB ,&
&  0.54376E+00_JPRB, 0.52248E+00_JPRB, 0.49175E+00_JPRB, 0.45405E+00_JPRB, 0.42854E+00_JPRB, 0.40902E+00_JPRB, 0.40152E+00_JPRB ,&
&  0.39166E+00_JPRB, 0.38577E+00_JPRB, 0.37935E+00_JPRB, 0.37665E+00_JPRB, 0.37433E+00_JPRB, 0.37483E+00_JPRB, 0.37401E+00_JPRB ,&
&  0.37000E+00_JPRB, 0.36430E+00_JPRB, 0.35746E+00_JPRB, 0.35994E+00_JPRB, 0.37275E+00_JPRB, 0.38744E+00_JPRB, 0.40497E+00_JPRB ,&
&  0.42535E+00_JPRB, 0.43574E+00_JPRB, 0.44606E+00_JPRB, 0.45001E+00_JPRB, 0.43452E+00_JPRB, 0.43716E+00_JPRB, 0.44079E+00_JPRB ,&
&  0.44875E+00_JPRB, 0.45459E+00_JPRB, 0.45110E+00_JPRB, 0.44571E+00_JPRB, 0.39411E+00_JPRB, 0.36165E+00_JPRB, 0.33291E+00_JPRB ,&
&  0.31151E+00_JPRB, 0.29789E+00_JPRB, 0.28570E+00_JPRB, 0.27522E+00_JPRB, 0.26402E+00_JPRB, 0.25695E+00_JPRB, 0.25167E+00_JPRB ,&
&  0.24748E+00_JPRB, 0.24579E+00_JPRB, 0.24425E+00_JPRB, 0.24392E+00_JPRB, 0.24460E+00_JPRB, 0.24408E+00_JPRB, 0.24421E+00_JPRB ,&
&  0.24423E+00_JPRB, 0.24377E+00_JPRB, 0.24415E+00_JPRB, 0.24573E+00_JPRB, 0.24795E+00_JPRB, 0.25149E+00_JPRB, 0.25245E+00_JPRB ,&
&  0.25356E+00_JPRB, 0.25443E+00_JPRB, 0.25455E+00_JPRB, 0.25669E+00_JPRB, 0.25789E+00_JPRB, 0.25803E+00_JPRB, 0.25644E+00_JPRB ,&
&  0.25472E+00_JPRB, 0.25339E+00_JPRB, 0.25345E+00_JPRB, 0.25413E+00_JPRB, 0.25586E+00_JPRB, 0.25846E+00_JPRB, 0.25896E+00_JPRB ,&
&  0.25989E+00_JPRB, 0.25935E+00_JPRB, 0.25877E+00_JPRB, 0.26101E+00_JPRB, 0.26359E+00_JPRB, 0.26636E+00_JPRB, 0.27002E+00_JPRB ,&
&  0.27167E+00_JPRB, 0.27925E+00_JPRB, 0.29337E+00_JPRB, 0.31525E+00_JPRB, 0.35078E+00_JPRB, 0.36768E+00_JPRB, 0.40732E+00_JPRB ,&
&  0.46466E+00_JPRB, 0.60298E+00_JPRB, 0.66736E+00_JPRB, 0.52999E+00_JPRB, 0.54676E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.50360E+00_JPRB, 0.51596E+00_JPRB, 0.53263E+00_JPRB, 0.54302E+00_JPRB, 0.55654E+00_JPRB, 0.55888E+00_JPRB, 0.55224E+00_JPRB ,&
&  0.53439E+00_JPRB, 0.50616E+00_JPRB, 0.48977E+00_JPRB, 0.46857E+00_JPRB, 0.44734E+00_JPRB, 0.42557E+00_JPRB, 0.41577E+00_JPRB ,&
&  0.40391E+00_JPRB, 0.39012E+00_JPRB, 0.37619E+00_JPRB, 0.36400E+00_JPRB, 0.35225E+00_JPRB, 0.34150E+00_JPRB, 0.33543E+00_JPRB ,&
&  0.32920E+00_JPRB, 0.32890E+00_JPRB, 0.32834E+00_JPRB, 0.33238E+00_JPRB, 0.34391E+00_JPRB, 0.36254E+00_JPRB, 0.38848E+00_JPRB ,&
&  0.42701E+00_JPRB, 0.43111E+00_JPRB, 0.41657E+00_JPRB, 0.41039E+00_JPRB, 0.40293E+00_JPRB, 0.39680E+00_JPRB, 0.38548E+00_JPRB ,&
&  0.39298E+00_JPRB, 0.40284E+00_JPRB, 0.41798E+00_JPRB, 0.40492E+00_JPRB, 0.35873E+00_JPRB, 0.33921E+00_JPRB, 0.31547E+00_JPRB ,&
&  0.30416E+00_JPRB, 0.29417E+00_JPRB, 0.28604E+00_JPRB, 0.27783E+00_JPRB, 0.26835E+00_JPRB, 0.26279E+00_JPRB, 0.25670E+00_JPRB ,&
&  0.25269E+00_JPRB, 0.25137E+00_JPRB, 0.24903E+00_JPRB, 0.24941E+00_JPRB, 0.24954E+00_JPRB, 0.24894E+00_JPRB, 0.24849E+00_JPRB ,&
&  0.24748E+00_JPRB, 0.24638E+00_JPRB, 0.24598E+00_JPRB, 0.24671E+00_JPRB, 0.24752E+00_JPRB, 0.24830E+00_JPRB, 0.24593E+00_JPRB ,&
&  0.24468E+00_JPRB, 0.24345E+00_JPRB, 0.24338E+00_JPRB, 0.24505E+00_JPRB, 0.24562E+00_JPRB, 0.24674E+00_JPRB, 0.24703E+00_JPRB ,&
&  0.24728E+00_JPRB, 0.24799E+00_JPRB, 0.24764E+00_JPRB, 0.24850E+00_JPRB, 0.24879E+00_JPRB, 0.25002E+00_JPRB, 0.25074E+00_JPRB ,&
&  0.25153E+00_JPRB, 0.25374E+00_JPRB, 0.25605E+00_JPRB, 0.25953E+00_JPRB, 0.26422E+00_JPRB, 0.26703E+00_JPRB, 0.27171E+00_JPRB ,&
&  0.27703E+00_JPRB, 0.28642E+00_JPRB, 0.30505E+00_JPRB, 0.32587E+00_JPRB, 0.35211E+00_JPRB, 0.36313E+00_JPRB, 0.36980E+00_JPRB ,&
&  0.38064E+00_JPRB, 0.41156E+00_JPRB, 0.46230E+00_JPRB, 0.48770E+00_JPRB, 0.49521E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.49563E+00_JPRB, 0.49812E+00_JPRB, 0.49997E+00_JPRB, 0.49743E+00_JPRB, 0.49725E+00_JPRB, 0.49747E+00_JPRB, 0.49479E+00_JPRB ,&
&  0.48669E+00_JPRB, 0.48462E+00_JPRB, 0.47947E+00_JPRB, 0.47167E+00_JPRB, 0.46384E+00_JPRB, 0.45092E+00_JPRB, 0.43434E+00_JPRB ,&
&  0.41220E+00_JPRB, 0.39246E+00_JPRB, 0.37789E+00_JPRB, 0.36395E+00_JPRB, 0.35037E+00_JPRB, 0.33712E+00_JPRB, 0.33246E+00_JPRB ,&
&  0.32809E+00_JPRB, 0.32641E+00_JPRB, 0.32612E+00_JPRB, 0.32935E+00_JPRB, 0.33680E+00_JPRB, 0.34614E+00_JPRB, 0.35646E+00_JPRB ,&
&  0.36773E+00_JPRB, 0.37835E+00_JPRB, 0.37940E+00_JPRB, 0.37165E+00_JPRB, 0.36470E+00_JPRB, 0.36051E+00_JPRB, 0.35721E+00_JPRB ,&
&  0.35331E+00_JPRB, 0.34520E+00_JPRB, 0.33560E+00_JPRB, 0.32534E+00_JPRB, 0.31502E+00_JPRB, 0.30750E+00_JPRB, 0.29866E+00_JPRB ,&
&  0.29214E+00_JPRB, 0.28604E+00_JPRB, 0.27935E+00_JPRB, 0.27354E+00_JPRB, 0.26958E+00_JPRB, 0.26637E+00_JPRB, 0.26304E+00_JPRB ,&
&  0.26157E+00_JPRB, 0.26011E+00_JPRB, 0.25851E+00_JPRB, 0.25704E+00_JPRB, 0.25509E+00_JPRB, 0.25368E+00_JPRB, 0.25241E+00_JPRB ,&
&  0.25148E+00_JPRB, 0.25063E+00_JPRB, 0.25013E+00_JPRB, 0.24958E+00_JPRB, 0.24908E+00_JPRB, 0.24795E+00_JPRB, 0.24664E+00_JPRB ,&
&  0.24590E+00_JPRB, 0.24584E+00_JPRB, 0.24506E+00_JPRB, 0.24470E+00_JPRB, 0.24440E+00_JPRB, 0.24356E+00_JPRB, 0.24414E+00_JPRB ,&
&  0.24552E+00_JPRB, 0.24704E+00_JPRB, 0.24719E+00_JPRB, 0.24827E+00_JPRB, 0.24954E+00_JPRB, 0.25118E+00_JPRB, 0.25390E+00_JPRB ,&
&  0.25593E+00_JPRB, 0.25807E+00_JPRB, 0.26080E+00_JPRB, 0.26307E+00_JPRB, 0.26674E+00_JPRB, 0.27129E+00_JPRB, 0.27693E+00_JPRB ,&
&  0.28466E+00_JPRB, 0.29401E+00_JPRB, 0.30747E+00_JPRB, 0.32403E+00_JPRB, 0.34830E+00_JPRB, 0.35614E+00_JPRB, 0.36321E+00_JPRB ,&
&  0.36831E+00_JPRB, 0.37797E+00_JPRB, 0.46077E+00_JPRB, 0.47512E+00_JPRB, 0.49009E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.37204E+00_JPRB, 0.38186E+00_JPRB, 0.38976E+00_JPRB, 0.39704E+00_JPRB, 0.39422E+00_JPRB, 0.45899E+00_JPRB, 0.43813E+00_JPRB ,&
&  0.42812E+00_JPRB, 0.40140E+00_JPRB, 0.40351E+00_JPRB, 0.39990E+00_JPRB, 0.39392E+00_JPRB, 0.38763E+00_JPRB, 0.38220E+00_JPRB ,&
&  0.37664E+00_JPRB, 0.36918E+00_JPRB, 0.36491E+00_JPRB, 0.35928E+00_JPRB, 0.35380E+00_JPRB, 0.34818E+00_JPRB, 0.34478E+00_JPRB ,&
&  0.34224E+00_JPRB, 0.34078E+00_JPRB, 0.33966E+00_JPRB, 0.33758E+00_JPRB, 0.33943E+00_JPRB, 0.34015E+00_JPRB, 0.33784E+00_JPRB ,&
&  0.33347E+00_JPRB, 0.32950E+00_JPRB, 0.32481E+00_JPRB, 0.32005E+00_JPRB, 0.31765E+00_JPRB, 0.31537E+00_JPRB, 0.31293E+00_JPRB ,&
&  0.30949E+00_JPRB, 0.30561E+00_JPRB, 0.30144E+00_JPRB, 0.29803E+00_JPRB, 0.29466E+00_JPRB, 0.29110E+00_JPRB, 0.28766E+00_JPRB ,&
&  0.28399E+00_JPRB, 0.28033E+00_JPRB, 0.27665E+00_JPRB, 0.27432E+00_JPRB, 0.27237E+00_JPRB, 0.27059E+00_JPRB, 0.26896E+00_JPRB ,&
&  0.26836E+00_JPRB, 0.26761E+00_JPRB, 0.26666E+00_JPRB, 0.26546E+00_JPRB, 0.26440E+00_JPRB, 0.26324E+00_JPRB, 0.26234E+00_JPRB ,&
&  0.26227E+00_JPRB, 0.26168E+00_JPRB, 0.26107E+00_JPRB, 0.26025E+00_JPRB, 0.25989E+00_JPRB, 0.25899E+00_JPRB, 0.25787E+00_JPRB ,&
&  0.25688E+00_JPRB, 0.25597E+00_JPRB, 0.25580E+00_JPRB, 0.25559E+00_JPRB, 0.25536E+00_JPRB, 0.25515E+00_JPRB, 0.25560E+00_JPRB ,&
&  0.25617E+00_JPRB, 0.25688E+00_JPRB, 0.25804E+00_JPRB, 0.25936E+00_JPRB, 0.26054E+00_JPRB, 0.26171E+00_JPRB, 0.26336E+00_JPRB ,&
&  0.26544E+00_JPRB, 0.26755E+00_JPRB, 0.26992E+00_JPRB, 0.27196E+00_JPRB, 0.27562E+00_JPRB, 0.27961E+00_JPRB, 0.28404E+00_JPRB ,&
&  0.28938E+00_JPRB, 0.29679E+00_JPRB, 0.30550E+00_JPRB, 0.31347E+00_JPRB, 0.32226E+00_JPRB, 0.33080E+00_JPRB, 0.33780E+00_JPRB ,&
&  0.34383E+00_JPRB, 0.34994E+00_JPRB, 0.35474E+00_JPRB, 0.36024E+00_JPRB, 0.36620E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.33032E+00_JPRB, 0.33339E+00_JPRB, 0.33580E+00_JPRB, 0.33737E+00_JPRB, 0.33705E+00_JPRB, 0.33835E+00_JPRB, 0.33833E+00_JPRB ,&
&  0.33807E+00_JPRB, 0.33760E+00_JPRB, 0.33831E+00_JPRB, 0.33910E+00_JPRB, 0.33957E+00_JPRB, 0.33989E+00_JPRB, 0.33995E+00_JPRB ,&
&  0.33937E+00_JPRB, 0.33830E+00_JPRB, 0.33363E+00_JPRB, 0.33102E+00_JPRB, 0.32939E+00_JPRB, 0.32851E+00_JPRB, 0.32797E+00_JPRB ,&
&  0.32691E+00_JPRB, 0.32571E+00_JPRB, 0.32391E+00_JPRB, 0.32372E+00_JPRB, 0.32290E+00_JPRB, 0.32181E+00_JPRB, 0.32025E+00_JPRB ,&
&  0.31870E+00_JPRB, 0.31748E+00_JPRB, 0.31601E+00_JPRB, 0.31452E+00_JPRB, 0.31274E+00_JPRB, 0.31056E+00_JPRB, 0.30770E+00_JPRB ,&
&  0.30478E+00_JPRB, 0.30184E+00_JPRB, 0.29938E+00_JPRB, 0.29706E+00_JPRB, 0.29506E+00_JPRB, 0.29341E+00_JPRB, 0.29217E+00_JPRB ,&
&  0.29079E+00_JPRB, 0.28920E+00_JPRB, 0.28758E+00_JPRB, 0.28621E+00_JPRB, 0.28487E+00_JPRB, 0.28371E+00_JPRB, 0.28308E+00_JPRB ,&
&  0.28272E+00_JPRB, 0.28228E+00_JPRB, 0.28175E+00_JPRB, 0.28124E+00_JPRB, 0.28062E+00_JPRB, 0.28005E+00_JPRB, 0.27947E+00_JPRB ,&
&  0.27898E+00_JPRB, 0.27872E+00_JPRB, 0.27831E+00_JPRB, 0.27790E+00_JPRB, 0.27764E+00_JPRB, 0.27720E+00_JPRB, 0.27672E+00_JPRB ,&
&  0.27618E+00_JPRB, 0.27592E+00_JPRB, 0.27562E+00_JPRB, 0.27522E+00_JPRB, 0.27485E+00_JPRB, 0.27460E+00_JPRB, 0.27462E+00_JPRB ,&
&  0.27459E+00_JPRB, 0.27449E+00_JPRB, 0.27456E+00_JPRB, 0.27538E+00_JPRB, 0.27625E+00_JPRB, 0.27731E+00_JPRB, 0.27849E+00_JPRB ,&
&  0.28001E+00_JPRB, 0.28161E+00_JPRB, 0.28341E+00_JPRB, 0.28515E+00_JPRB, 0.28750E+00_JPRB, 0.28983E+00_JPRB, 0.29217E+00_JPRB ,&
&  0.29443E+00_JPRB, 0.29678E+00_JPRB, 0.29918E+00_JPRB, 0.30166E+00_JPRB, 0.30502E+00_JPRB, 0.30768E+00_JPRB, 0.31068E+00_JPRB ,&
&  0.31357E+00_JPRB, 0.31684E+00_JPRB, 0.32009E+00_JPRB, 0.32298E+00_JPRB, 0.32583E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB ,&
&  0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB, 0.29970E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB, 0.45333E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.46067E+00_JPRB, 0.46068E+00_JPRB, 0.46072E+00_JPRB, 0.46088E+00_JPRB, 0.46072E+00_JPRB, 0.46035E+00_JPRB, 0.45974E+00_JPRB ,&
&  0.45906E+00_JPRB, 0.45832E+00_JPRB, 0.45731E+00_JPRB, 0.45628E+00_JPRB, 0.45527E+00_JPRB, 0.45425E+00_JPRB, 0.45287E+00_JPRB ,&
&  0.45134E+00_JPRB, 0.44959E+00_JPRB, 0.44779E+00_JPRB, 0.44611E+00_JPRB, 0.44503E+00_JPRB, 0.44422E+00_JPRB, 0.44367E+00_JPRB ,&
&  0.44309E+00_JPRB, 0.44262E+00_JPRB, 0.44219E+00_JPRB, 0.44191E+00_JPRB, 0.44184E+00_JPRB, 0.44163E+00_JPRB, 0.44147E+00_JPRB ,&
&  0.44132E+00_JPRB, 0.44132E+00_JPRB, 0.44114E+00_JPRB, 0.44096E+00_JPRB, 0.44074E+00_JPRB, 0.44090E+00_JPRB, 0.44095E+00_JPRB ,&
&  0.44117E+00_JPRB, 0.44131E+00_JPRB, 0.44142E+00_JPRB, 0.44146E+00_JPRB, 0.44158E+00_JPRB, 0.44162E+00_JPRB, 0.44162E+00_JPRB ,&
&  0.44168E+00_JPRB, 0.44196E+00_JPRB, 0.44273E+00_JPRB, 0.44382E+00_JPRB, 0.44490E+00_JPRB, 0.44609E+00_JPRB, 0.44727E+00_JPRB ,&
&  0.44847E+00_JPRB, 0.44949E+00_JPRB, 0.45077E+00_JPRB, 0.45201E+00_JPRB, 0.45342E+00_JPRB, 0.45472E+00_JPRB, 0.45613E+00_JPRB ,&
&  0.45745E+00_JPRB, 0.45897E+00_JPRB, 0.46075E+00_JPRB, 0.46262E+00_JPRB, 0.46441E+00_JPRB, 0.46599E+00_JPRB, 0.46768E+00_JPRB ,&
&  0.46945E+00_JPRB, 0.47114E+00_JPRB, 0.47241E+00_JPRB, 0.47380E+00_JPRB, 0.47525E+00_JPRB, 0.47668E+00_JPRB, 0.47770E+00_JPRB ,&
&  0.47877E+00_JPRB, 0.47996E+00_JPRB, 0.48115E+00_JPRB, 0.48150E+00_JPRB, 0.48175E+00_JPRB, 0.48185E+00_JPRB, 0.48164E+00_JPRB ,&
&  0.48081E+00_JPRB, 0.47990E+00_JPRB, 0.47898E+00_JPRB, 0.47795E+00_JPRB, 0.47658E+00_JPRB, 0.47504E+00_JPRB, 0.47326E+00_JPRB ,&
&  0.47142E+00_JPRB, 0.46920E+00_JPRB, 0.46709E+00_JPRB, 0.46556E+00_JPRB, 0.46458E+00_JPRB, 0.46350E+00_JPRB, 0.46242E+00_JPRB ,&
&  0.46146E+00_JPRB, 0.46115E+00_JPRB, 0.46102E+00_JPRB, 0.46088E+00_JPRB, 0.46083E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.46432E+00_JPRB, 0.46461E+00_JPRB, 0.46510E+00_JPRB, 0.46526E+00_JPRB, 0.46434E+00_JPRB, 0.46348E+00_JPRB, 0.46253E+00_JPRB ,&
&  0.46193E+00_JPRB, 0.46132E+00_JPRB, 0.46026E+00_JPRB, 0.45889E+00_JPRB, 0.45753E+00_JPRB, 0.45618E+00_JPRB, 0.45474E+00_JPRB ,&
&  0.45342E+00_JPRB, 0.45200E+00_JPRB, 0.45029E+00_JPRB, 0.44716E+00_JPRB, 0.44433E+00_JPRB, 0.44226E+00_JPRB, 0.44159E+00_JPRB ,&
&  0.44067E+00_JPRB, 0.43957E+00_JPRB, 0.43781E+00_JPRB, 0.43579E+00_JPRB, 0.43368E+00_JPRB, 0.43170E+00_JPRB, 0.42964E+00_JPRB ,&
&  0.42791E+00_JPRB, 0.42639E+00_JPRB, 0.42524E+00_JPRB, 0.42395E+00_JPRB, 0.42434E+00_JPRB, 0.42480E+00_JPRB, 0.42522E+00_JPRB ,&
&  0.42563E+00_JPRB, 0.42574E+00_JPRB, 0.42679E+00_JPRB, 0.42786E+00_JPRB, 0.42892E+00_JPRB, 0.43048E+00_JPRB, 0.43131E+00_JPRB ,&
&  0.43135E+00_JPRB, 0.43163E+00_JPRB, 0.43384E+00_JPRB, 0.43552E+00_JPRB, 0.43682E+00_JPRB, 0.43797E+00_JPRB, 0.44017E+00_JPRB ,&
&  0.44231E+00_JPRB, 0.44442E+00_JPRB, 0.44678E+00_JPRB, 0.44989E+00_JPRB, 0.45274E+00_JPRB, 0.45585E+00_JPRB, 0.45917E+00_JPRB ,&
&  0.46238E+00_JPRB, 0.46577E+00_JPRB, 0.46922E+00_JPRB, 0.47262E+00_JPRB, 0.47573E+00_JPRB, 0.47840E+00_JPRB, 0.48149E+00_JPRB ,&
&  0.48476E+00_JPRB, 0.48847E+00_JPRB, 0.49078E+00_JPRB, 0.49327E+00_JPRB, 0.49602E+00_JPRB, 0.49988E+00_JPRB, 0.50141E+00_JPRB ,&
&  0.50284E+00_JPRB, 0.50417E+00_JPRB, 0.50566E+00_JPRB, 0.50630E+00_JPRB, 0.50695E+00_JPRB, 0.50761E+00_JPRB, 0.50807E+00_JPRB ,&
&  0.50792E+00_JPRB, 0.50705E+00_JPRB, 0.50544E+00_JPRB, 0.50304E+00_JPRB, 0.50019E+00_JPRB, 0.49732E+00_JPRB, 0.49381E+00_JPRB ,&
&  0.49065E+00_JPRB, 0.48665E+00_JPRB, 0.48279E+00_JPRB, 0.48012E+00_JPRB, 0.47842E+00_JPRB, 0.47545E+00_JPRB, 0.47226E+00_JPRB ,&
&  0.46946E+00_JPRB, 0.46731E+00_JPRB, 0.46626E+00_JPRB, 0.46531E+00_JPRB, 0.46482E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.46856E+00_JPRB, 0.46883E+00_JPRB, 0.46900E+00_JPRB, 0.46878E+00_JPRB, 0.46739E+00_JPRB, 0.46619E+00_JPRB, 0.46490E+00_JPRB ,&
&  0.46385E+00_JPRB, 0.46246E+00_JPRB, 0.46133E+00_JPRB, 0.46037E+00_JPRB, 0.45951E+00_JPRB, 0.45896E+00_JPRB, 0.45838E+00_JPRB ,&
&  0.45809E+00_JPRB, 0.45741E+00_JPRB, 0.45683E+00_JPRB, 0.45525E+00_JPRB, 0.45233E+00_JPRB, 0.44797E+00_JPRB, 0.44367E+00_JPRB ,&
&  0.43940E+00_JPRB, 0.43534E+00_JPRB, 0.43089E+00_JPRB, 0.42636E+00_JPRB, 0.42235E+00_JPRB, 0.39776E+00_JPRB, 0.39036E+00_JPRB ,&
&  0.39079E+00_JPRB, 0.39162E+00_JPRB, 0.39479E+00_JPRB, 0.39907E+00_JPRB, 0.40225E+00_JPRB, 0.39669E+00_JPRB, 0.39436E+00_JPRB ,&
&  0.37799E+00_JPRB, 0.37969E+00_JPRB, 0.38151E+00_JPRB, 0.38374E+00_JPRB, 0.38858E+00_JPRB, 0.39298E+00_JPRB, 0.40055E+00_JPRB ,&
&  0.41734E+00_JPRB, 0.42036E+00_JPRB, 0.42351E+00_JPRB, 0.42721E+00_JPRB, 0.43037E+00_JPRB, 0.43322E+00_JPRB, 0.43651E+00_JPRB ,&
&  0.43902E+00_JPRB, 0.44180E+00_JPRB, 0.44478E+00_JPRB, 0.44831E+00_JPRB, 0.45248E+00_JPRB, 0.45703E+00_JPRB, 0.46178E+00_JPRB ,&
&  0.46632E+00_JPRB, 0.47162E+00_JPRB, 0.47712E+00_JPRB, 0.48267E+00_JPRB, 0.48817E+00_JPRB, 0.49359E+00_JPRB, 0.49937E+00_JPRB ,&
&  0.50476E+00_JPRB, 0.51051E+00_JPRB, 0.51453E+00_JPRB, 0.51890E+00_JPRB, 0.52186E+00_JPRB, 0.52535E+00_JPRB, 0.52623E+00_JPRB ,&
&  0.52744E+00_JPRB, 0.52597E+00_JPRB, 0.52579E+00_JPRB, 0.52476E+00_JPRB, 0.52386E+00_JPRB, 0.52344E+00_JPRB, 0.52261E+00_JPRB ,&
&  0.52227E+00_JPRB, 0.52120E+00_JPRB, 0.52011E+00_JPRB, 0.51791E+00_JPRB, 0.51441E+00_JPRB, 0.51008E+00_JPRB, 0.50543E+00_JPRB ,&
&  0.50213E+00_JPRB, 0.49981E+00_JPRB, 0.49787E+00_JPRB, 0.49644E+00_JPRB, 0.49371E+00_JPRB, 0.48983E+00_JPRB, 0.48527E+00_JPRB ,&
&  0.48134E+00_JPRB, 0.47732E+00_JPRB, 0.47396E+00_JPRB, 0.47115E+00_JPRB, 0.46949E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.47474E+00_JPRB, 0.47560E+00_JPRB, 0.47608E+00_JPRB, 0.47631E+00_JPRB, 0.47521E+00_JPRB, 0.47418E+00_JPRB, 0.47305E+00_JPRB ,&
&  0.47180E+00_JPRB, 0.47060E+00_JPRB, 0.46931E+00_JPRB, 0.46832E+00_JPRB, 0.46705E+00_JPRB, 0.46615E+00_JPRB, 0.46456E+00_JPRB ,&
&  0.46390E+00_JPRB, 0.46237E+00_JPRB, 0.46059E+00_JPRB, 0.45683E+00_JPRB, 0.45309E+00_JPRB, 0.44862E+00_JPRB, 0.44129E+00_JPRB ,&
&  0.43334E+00_JPRB, 0.42610E+00_JPRB, 0.40651E+00_JPRB, 0.40094E+00_JPRB, 0.39069E+00_JPRB, 0.38490E+00_JPRB, 0.38173E+00_JPRB ,&
&  0.37765E+00_JPRB, 0.37497E+00_JPRB, 0.36740E+00_JPRB, 0.36062E+00_JPRB, 0.35520E+00_JPRB, 0.34623E+00_JPRB, 0.34135E+00_JPRB ,&
&  0.33563E+00_JPRB, 0.33297E+00_JPRB, 0.33403E+00_JPRB, 0.33746E+00_JPRB, 0.34608E+00_JPRB, 0.35642E+00_JPRB, 0.37021E+00_JPRB ,&
&  0.38924E+00_JPRB, 0.40628E+00_JPRB, 0.42272E+00_JPRB, 0.42417E+00_JPRB, 0.42914E+00_JPRB, 0.43360E+00_JPRB, 0.43666E+00_JPRB ,&
&  0.43369E+00_JPRB, 0.43336E+00_JPRB, 0.44193E+00_JPRB, 0.44554E+00_JPRB, 0.44989E+00_JPRB, 0.45489E+00_JPRB, 0.46006E+00_JPRB ,&
&  0.46565E+00_JPRB, 0.47199E+00_JPRB, 0.47870E+00_JPRB, 0.48666E+00_JPRB, 0.49639E+00_JPRB, 0.50590E+00_JPRB, 0.51398E+00_JPRB ,&
&  0.52100E+00_JPRB, 0.52965E+00_JPRB, 0.53587E+00_JPRB, 0.54187E+00_JPRB, 0.54757E+00_JPRB, 0.55590E+00_JPRB, 0.56247E+00_JPRB ,&
&  0.56642E+00_JPRB, 0.56440E+00_JPRB, 0.55905E+00_JPRB, 0.54836E+00_JPRB, 0.54299E+00_JPRB, 0.54728E+00_JPRB, 0.54507E+00_JPRB ,&
&  0.54402E+00_JPRB, 0.54109E+00_JPRB, 0.53875E+00_JPRB, 0.53512E+00_JPRB, 0.53124E+00_JPRB, 0.52564E+00_JPRB, 0.51903E+00_JPRB ,&
&  0.51372E+00_JPRB, 0.51144E+00_JPRB, 0.50944E+00_JPRB, 0.50758E+00_JPRB, 0.50318E+00_JPRB, 0.49860E+00_JPRB, 0.49389E+00_JPRB ,&
&  0.48946E+00_JPRB, 0.48499E+00_JPRB, 0.48055E+00_JPRB, 0.47722E+00_JPRB, 0.47529E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.48757E+00_JPRB, 0.48840E+00_JPRB, 0.48779E+00_JPRB, 0.48740E+00_JPRB, 0.48592E+00_JPRB, 0.48471E+00_JPRB, 0.48284E+00_JPRB ,&
&  0.48097E+00_JPRB, 0.47908E+00_JPRB, 0.47718E+00_JPRB, 0.47538E+00_JPRB, 0.47355E+00_JPRB, 0.47175E+00_JPRB, 0.47057E+00_JPRB ,&
&  0.46935E+00_JPRB, 0.46715E+00_JPRB, 0.46362E+00_JPRB, 0.45916E+00_JPRB, 0.45524E+00_JPRB, 0.45072E+00_JPRB, 0.44548E+00_JPRB ,&
&  0.43973E+00_JPRB, 0.43201E+00_JPRB, 0.41930E+00_JPRB, 0.41338E+00_JPRB, 0.40509E+00_JPRB, 0.40155E+00_JPRB, 0.39634E+00_JPRB ,&
&  0.39467E+00_JPRB, 0.36754E+00_JPRB, 0.36642E+00_JPRB, 0.35573E+00_JPRB, 0.34780E+00_JPRB, 0.34176E+00_JPRB, 0.32986E+00_JPRB ,&
&  0.32465E+00_JPRB, 0.31803E+00_JPRB, 0.31546E+00_JPRB, 0.31851E+00_JPRB, 0.32436E+00_JPRB, 0.33649E+00_JPRB, 0.35243E+00_JPRB ,&
&  0.37251E+00_JPRB, 0.41299E+00_JPRB, 0.42306E+00_JPRB, 0.42995E+00_JPRB, 0.43660E+00_JPRB, 0.44147E+00_JPRB, 0.43516E+00_JPRB ,&
&  0.43360E+00_JPRB, 0.43181E+00_JPRB, 0.43397E+00_JPRB, 0.43929E+00_JPRB, 0.44720E+00_JPRB, 0.45041E+00_JPRB, 0.45528E+00_JPRB ,&
&  0.46008E+00_JPRB, 0.46788E+00_JPRB, 0.47622E+00_JPRB, 0.48791E+00_JPRB, 0.50424E+00_JPRB, 0.52077E+00_JPRB, 0.53539E+00_JPRB ,&
&  0.54669E+00_JPRB, 0.55312E+00_JPRB, 0.55551E+00_JPRB, 0.56217E+00_JPRB, 0.57188E+00_JPRB, 0.58653E+00_JPRB, 0.59594E+00_JPRB ,&
&  0.60196E+00_JPRB, 0.60721E+00_JPRB, 0.60588E+00_JPRB, 0.60100E+00_JPRB, 0.58972E+00_JPRB, 0.57865E+00_JPRB, 0.57300E+00_JPRB ,&
&  0.57477E+00_JPRB, 0.57041E+00_JPRB, 0.56372E+00_JPRB, 0.55407E+00_JPRB, 0.54776E+00_JPRB, 0.53971E+00_JPRB, 0.53176E+00_JPRB ,&
&  0.52523E+00_JPRB, 0.52173E+00_JPRB, 0.51716E+00_JPRB, 0.51382E+00_JPRB, 0.50808E+00_JPRB, 0.50385E+00_JPRB, 0.50041E+00_JPRB ,&
&  0.49636E+00_JPRB, 0.49319E+00_JPRB, 0.48916E+00_JPRB, 0.48714E+00_JPRB, 0.48742E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.50142E+00_JPRB, 0.50135E+00_JPRB, 0.50074E+00_JPRB, 0.49993E+00_JPRB, 0.49875E+00_JPRB, 0.49832E+00_JPRB, 0.49844E+00_JPRB ,&
&  0.49744E+00_JPRB, 0.49667E+00_JPRB, 0.49556E+00_JPRB, 0.49431E+00_JPRB, 0.49278E+00_JPRB, 0.49157E+00_JPRB, 0.49023E+00_JPRB ,&
&  0.48760E+00_JPRB, 0.48361E+00_JPRB, 0.47809E+00_JPRB, 0.47257E+00_JPRB, 0.46735E+00_JPRB, 0.46395E+00_JPRB, 0.45895E+00_JPRB ,&
&  0.45386E+00_JPRB, 0.44697E+00_JPRB, 0.43643E+00_JPRB, 0.42866E+00_JPRB, 0.42618E+00_JPRB, 0.42004E+00_JPRB, 0.41998E+00_JPRB ,&
&  0.41722E+00_JPRB, 0.41260E+00_JPRB, 0.40029E+00_JPRB, 0.39555E+00_JPRB, 0.38762E+00_JPRB, 0.34947E+00_JPRB, 0.34066E+00_JPRB ,&
&  0.32870E+00_JPRB, 0.31557E+00_JPRB, 0.31061E+00_JPRB, 0.30922E+00_JPRB, 0.31226E+00_JPRB, 0.32504E+00_JPRB, 0.34240E+00_JPRB ,&
&  0.36189E+00_JPRB, 0.41342E+00_JPRB, 0.42738E+00_JPRB, 0.43912E+00_JPRB, 0.44772E+00_JPRB, 0.45237E+00_JPRB, 0.45025E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44809E+00_JPRB, 0.45227E+00_JPRB, 0.45932E+00_JPRB, 0.46374E+00_JPRB, 0.46144E+00_JPRB, 0.46161E+00_JPRB ,&
&  0.45779E+00_JPRB, 0.46261E+00_JPRB, 0.47310E+00_JPRB, 0.48781E+00_JPRB, 0.50700E+00_JPRB, 0.52125E+00_JPRB, 0.54119E+00_JPRB ,&
&  0.55807E+00_JPRB, 0.56843E+00_JPRB, 0.58640E+00_JPRB, 0.60515E+00_JPRB, 0.61773E+00_JPRB, 0.62558E+00_JPRB, 0.62756E+00_JPRB ,&
&  0.62608E+00_JPRB, 0.64172E+00_JPRB, 0.65046E+00_JPRB, 0.64679E+00_JPRB, 0.63829E+00_JPRB, 0.62292E+00_JPRB, 0.60746E+00_JPRB ,&
&  0.59999E+00_JPRB, 0.60220E+00_JPRB, 0.59531E+00_JPRB, 0.57759E+00_JPRB, 0.56372E+00_JPRB, 0.55229E+00_JPRB, 0.54312E+00_JPRB ,&
&  0.53629E+00_JPRB, 0.53223E+00_JPRB, 0.52633E+00_JPRB, 0.52158E+00_JPRB, 0.51657E+00_JPRB, 0.51256E+00_JPRB, 0.51015E+00_JPRB ,&
&  0.50798E+00_JPRB, 0.50736E+00_JPRB, 0.50571E+00_JPRB, 0.50366E+00_JPRB, 0.50315E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.51663E+00_JPRB, 0.51807E+00_JPRB, 0.51903E+00_JPRB, 0.51897E+00_JPRB, 0.51937E+00_JPRB, 0.52061E+00_JPRB, 0.51929E+00_JPRB ,&
&  0.51584E+00_JPRB, 0.51306E+00_JPRB, 0.50922E+00_JPRB, 0.50807E+00_JPRB, 0.50673E+00_JPRB, 0.50542E+00_JPRB, 0.50428E+00_JPRB ,&
&  0.50152E+00_JPRB, 0.49148E+00_JPRB, 0.48485E+00_JPRB, 0.48192E+00_JPRB, 0.47873E+00_JPRB, 0.47697E+00_JPRB, 0.47515E+00_JPRB ,&
&  0.46981E+00_JPRB, 0.46233E+00_JPRB, 0.45491E+00_JPRB, 0.44266E+00_JPRB, 0.43383E+00_JPRB, 0.43420E+00_JPRB, 0.43814E+00_JPRB ,&
&  0.43346E+00_JPRB, 0.42041E+00_JPRB, 0.42026E+00_JPRB, 0.41702E+00_JPRB, 0.40005E+00_JPRB, 0.39396E+00_JPRB, 0.36329E+00_JPRB ,&
&  0.34128E+00_JPRB, 0.32246E+00_JPRB, 0.31580E+00_JPRB, 0.31157E+00_JPRB, 0.31605E+00_JPRB, 0.32753E+00_JPRB, 0.35431E+00_JPRB ,&
&  0.38339E+00_JPRB, 0.41496E+00_JPRB, 0.42651E+00_JPRB, 0.44011E+00_JPRB, 0.45006E+00_JPRB, 0.46363E+00_JPRB, 0.47539E+00_JPRB ,&
&  0.48190E+00_JPRB, 0.48662E+00_JPRB, 0.49342E+00_JPRB, 0.49806E+00_JPRB, 0.50862E+00_JPRB, 0.51701E+00_JPRB, 0.51997E+00_JPRB ,&
&  0.52683E+00_JPRB, 0.52750E+00_JPRB, 0.53506E+00_JPRB, 0.54422E+00_JPRB, 0.55632E+00_JPRB, 0.56213E+00_JPRB, 0.56846E+00_JPRB ,&
&  0.57699E+00_JPRB, 0.59952E+00_JPRB, 0.63259E+00_JPRB, 0.67415E+00_JPRB, 0.68553E+00_JPRB, 0.68175E+00_JPRB, 0.66447E+00_JPRB ,&
&  0.64401E+00_JPRB, 0.66443E+00_JPRB, 0.75701E+00_JPRB, 0.74250E+00_JPRB, 0.73006E+00_JPRB, 0.71366E+00_JPRB, 0.67852E+00_JPRB ,&
&  0.63807E+00_JPRB, 0.62144E+00_JPRB, 0.60691E+00_JPRB, 0.58242E+00_JPRB, 0.56284E+00_JPRB, 0.54910E+00_JPRB, 0.54125E+00_JPRB ,&
&  0.53642E+00_JPRB, 0.53568E+00_JPRB, 0.53155E+00_JPRB, 0.52897E+00_JPRB, 0.52435E+00_JPRB, 0.51969E+00_JPRB, 0.51814E+00_JPRB ,&
&  0.51668E+00_JPRB, 0.51721E+00_JPRB, 0.51817E+00_JPRB, 0.51854E+00_JPRB, 0.51804E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.53222E+00_JPRB, 0.53860E+00_JPRB, 0.54357E+00_JPRB, 0.55229E+00_JPRB, 0.56021E+00_JPRB, 0.56089E+00_JPRB, 0.55574E+00_JPRB ,&
&  0.54385E+00_JPRB, 0.52933E+00_JPRB, 0.51942E+00_JPRB, 0.51180E+00_JPRB, 0.50755E+00_JPRB, 0.50532E+00_JPRB, 0.50317E+00_JPRB ,&
&  0.50146E+00_JPRB, 0.49942E+00_JPRB, 0.49405E+00_JPRB, 0.48608E+00_JPRB, 0.48114E+00_JPRB, 0.48027E+00_JPRB, 0.47968E+00_JPRB ,&
&  0.47719E+00_JPRB, 0.47181E+00_JPRB, 0.46472E+00_JPRB, 0.45295E+00_JPRB, 0.44331E+00_JPRB, 0.44532E+00_JPRB, 0.44059E+00_JPRB ,&
&  0.43558E+00_JPRB, 0.43257E+00_JPRB, 0.42968E+00_JPRB, 0.42516E+00_JPRB, 0.42008E+00_JPRB, 0.40719E+00_JPRB, 0.38234E+00_JPRB ,&
&  0.35609E+00_JPRB, 0.33520E+00_JPRB, 0.32272E+00_JPRB, 0.31919E+00_JPRB, 0.32665E+00_JPRB, 0.34634E+00_JPRB, 0.37510E+00_JPRB ,&
&  0.41073E+00_JPRB, 0.42680E+00_JPRB, 0.44266E+00_JPRB, 0.46071E+00_JPRB, 0.47592E+00_JPRB, 0.49387E+00_JPRB, 0.50967E+00_JPRB ,&
&  0.52536E+00_JPRB, 0.54671E+00_JPRB, 0.57420E+00_JPRB, 0.60894E+00_JPRB, 0.65204E+00_JPRB, 0.63814E+00_JPRB, 0.66897E+00_JPRB ,&
&  0.76692E+00_JPRB, 0.76760E+00_JPRB, 0.73341E+00_JPRB, 0.69444E+00_JPRB, 0.66903E+00_JPRB, 0.65941E+00_JPRB, 0.66054E+00_JPRB ,&
&  0.66592E+00_JPRB, 0.67931E+00_JPRB, 0.70375E+00_JPRB, 0.73386E+00_JPRB, 0.74212E+00_JPRB, 0.73635E+00_JPRB, 0.71137E+00_JPRB ,&
&  0.68922E+00_JPRB, 0.71257E+00_JPRB, 0.90684E+00_JPRB, 0.82613E+00_JPRB, 0.75280E+00_JPRB, 0.71732E+00_JPRB, 0.67457E+00_JPRB ,&
&  0.64337E+00_JPRB, 0.61949E+00_JPRB, 0.59577E+00_JPRB, 0.57786E+00_JPRB, 0.55503E+00_JPRB, 0.53360E+00_JPRB, 0.51519E+00_JPRB ,&
&  0.50984E+00_JPRB, 0.50947E+00_JPRB, 0.51050E+00_JPRB, 0.51198E+00_JPRB, 0.51039E+00_JPRB, 0.51106E+00_JPRB, 0.51340E+00_JPRB ,&
&  0.51160E+00_JPRB, 0.51198E+00_JPRB, 0.51684E+00_JPRB, 0.52195E+00_JPRB, 0.52683E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.51841E+00_JPRB, 0.53775E+00_JPRB, 0.56621E+00_JPRB, 0.58455E+00_JPRB, 0.59271E+00_JPRB, 0.57863E+00_JPRB, 0.56175E+00_JPRB ,&
&  0.54336E+00_JPRB, 0.51867E+00_JPRB, 0.50513E+00_JPRB, 0.49673E+00_JPRB, 0.49174E+00_JPRB, 0.48811E+00_JPRB, 0.48524E+00_JPRB ,&
&  0.48293E+00_JPRB, 0.48068E+00_JPRB, 0.48012E+00_JPRB, 0.48198E+00_JPRB, 0.48335E+00_JPRB, 0.48125E+00_JPRB, 0.47778E+00_JPRB ,&
&  0.47475E+00_JPRB, 0.47044E+00_JPRB, 0.46119E+00_JPRB, 0.44898E+00_JPRB, 0.44507E+00_JPRB, 0.45178E+00_JPRB, 0.45004E+00_JPRB ,&
&  0.44459E+00_JPRB, 0.43794E+00_JPRB, 0.43673E+00_JPRB, 0.43692E+00_JPRB, 0.43243E+00_JPRB, 0.42192E+00_JPRB, 0.39966E+00_JPRB ,&
&  0.37098E+00_JPRB, 0.34825E+00_JPRB, 0.34125E+00_JPRB, 0.35339E+00_JPRB, 0.37800E+00_JPRB, 0.41465E+00_JPRB, 0.42950E+00_JPRB ,&
&  0.44193E+00_JPRB, 0.45491E+00_JPRB, 0.46818E+00_JPRB, 0.48810E+00_JPRB, 0.51071E+00_JPRB, 0.53047E+00_JPRB, 0.54876E+00_JPRB ,&
&  0.57643E+00_JPRB, 0.60633E+00_JPRB, 0.64438E+00_JPRB, 0.61485E+00_JPRB, 0.63340E+00_JPRB, 0.68741E+00_JPRB, 0.71604E+00_JPRB ,&
&  0.73007E+00_JPRB, 0.76102E+00_JPRB, 0.77887E+00_JPRB, 0.80358E+00_JPRB, 0.79042E+00_JPRB, 0.77209E+00_JPRB, 0.75283E+00_JPRB ,&
&  0.74848E+00_JPRB, 0.74415E+00_JPRB, 0.74896E+00_JPRB, 0.77882E+00_JPRB, 0.80680E+00_JPRB, 0.80800E+00_JPRB, 0.78535E+00_JPRB ,&
&  0.74942E+00_JPRB, 0.69445E+00_JPRB, 0.64432E+00_JPRB, 0.61178E+00_JPRB, 0.61101E+00_JPRB, 0.62469E+00_JPRB, 0.62987E+00_JPRB ,&
&  0.62626E+00_JPRB, 0.62063E+00_JPRB, 0.60676E+00_JPRB, 0.59561E+00_JPRB, 0.55340E+00_JPRB, 0.52345E+00_JPRB, 0.50017E+00_JPRB ,&
&  0.49202E+00_JPRB, 0.48809E+00_JPRB, 0.48756E+00_JPRB, 0.49526E+00_JPRB, 0.49426E+00_JPRB, 0.49635E+00_JPRB, 0.49824E+00_JPRB ,&
&  0.49682E+00_JPRB, 0.49470E+00_JPRB, 0.49773E+00_JPRB, 0.50198E+00_JPRB, 0.50832E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.57917E+00_JPRB, 0.59467E+00_JPRB, 0.60815E+00_JPRB, 0.61551E+00_JPRB, 0.60642E+00_JPRB, 0.59976E+00_JPRB, 0.59299E+00_JPRB ,&
&  0.59106E+00_JPRB, 0.54389E+00_JPRB, 0.51444E+00_JPRB, 0.49203E+00_JPRB, 0.47601E+00_JPRB, 0.46144E+00_JPRB, 0.45694E+00_JPRB ,&
&  0.45811E+00_JPRB, 0.46131E+00_JPRB, 0.46373E+00_JPRB, 0.46577E+00_JPRB, 0.46778E+00_JPRB, 0.46836E+00_JPRB, 0.47008E+00_JPRB ,&
&  0.47294E+00_JPRB, 0.47297E+00_JPRB, 0.46461E+00_JPRB, 0.45743E+00_JPRB, 0.45604E+00_JPRB, 0.45788E+00_JPRB, 0.45417E+00_JPRB ,&
&  0.44949E+00_JPRB, 0.44493E+00_JPRB, 0.44325E+00_JPRB, 0.44410E+00_JPRB, 0.44131E+00_JPRB, 0.43170E+00_JPRB, 0.40715E+00_JPRB ,&
&  0.37719E+00_JPRB, 0.36686E+00_JPRB, 0.38301E+00_JPRB, 0.42820E+00_JPRB, 0.44426E+00_JPRB, 0.45536E+00_JPRB, 0.46880E+00_JPRB ,&
&  0.46502E+00_JPRB, 0.46659E+00_JPRB, 0.48478E+00_JPRB, 0.52804E+00_JPRB, 0.54796E+00_JPRB, 0.56530E+00_JPRB, 0.58503E+00_JPRB ,&
&  0.61945E+00_JPRB, 0.64671E+00_JPRB, 0.67842E+00_JPRB, 0.64688E+00_JPRB, 0.70410E+00_JPRB, 0.84680E+00_JPRB, 0.81976E+00_JPRB ,&
&  0.81865E+00_JPRB, 0.82525E+00_JPRB, 0.86460E+00_JPRB, 0.88723E+00_JPRB, 0.88149E+00_JPRB, 0.86319E+00_JPRB, 0.85303E+00_JPRB ,&
&  0.83797E+00_JPRB, 0.83217E+00_JPRB, 0.84146E+00_JPRB, 0.88465E+00_JPRB, 0.88340E+00_JPRB, 0.86776E+00_JPRB, 0.84401E+00_JPRB ,&
&  0.80287E+00_JPRB, 0.74509E+00_JPRB, 0.69194E+00_JPRB, 0.65300E+00_JPRB, 0.64373E+00_JPRB, 0.64677E+00_JPRB, 0.64358E+00_JPRB ,&
&  0.63091E+00_JPRB, 0.62927E+00_JPRB, 0.63143E+00_JPRB, 0.63009E+00_JPRB, 0.58702E+00_JPRB, 0.53842E+00_JPRB, 0.50247E+00_JPRB ,&
&  0.47691E+00_JPRB, 0.46246E+00_JPRB, 0.45182E+00_JPRB, 0.45706E+00_JPRB, 0.46864E+00_JPRB, 0.48249E+00_JPRB, 0.48352E+00_JPRB ,&
&  0.48692E+00_JPRB, 0.49012E+00_JPRB, 0.50260E+00_JPRB, 0.52890E+00_JPRB, 0.55585E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.61109E+00_JPRB, 0.62968E+00_JPRB, 0.64491E+00_JPRB, 0.63987E+00_JPRB, 0.62709E+00_JPRB, 0.61636E+00_JPRB, 0.60599E+00_JPRB ,&
&  0.60370E+00_JPRB, 0.53123E+00_JPRB, 0.50187E+00_JPRB, 0.48493E+00_JPRB, 0.47452E+00_JPRB, 0.45539E+00_JPRB, 0.43932E+00_JPRB ,&
&  0.43691E+00_JPRB, 0.44447E+00_JPRB, 0.45580E+00_JPRB, 0.46309E+00_JPRB, 0.46948E+00_JPRB, 0.47380E+00_JPRB, 0.47588E+00_JPRB ,&
&  0.47842E+00_JPRB, 0.48089E+00_JPRB, 0.47688E+00_JPRB, 0.47113E+00_JPRB, 0.46733E+00_JPRB, 0.46602E+00_JPRB, 0.46477E+00_JPRB ,&
&  0.46214E+00_JPRB, 0.45822E+00_JPRB, 0.45113E+00_JPRB, 0.44800E+00_JPRB, 0.44541E+00_JPRB, 0.44659E+00_JPRB, 0.44267E+00_JPRB ,&
&  0.39428E+00_JPRB, 0.39918E+00_JPRB, 0.44233E+00_JPRB, 0.45068E+00_JPRB, 0.46329E+00_JPRB, 0.47029E+00_JPRB, 0.47649E+00_JPRB ,&
&  0.48725E+00_JPRB, 0.50746E+00_JPRB, 0.53585E+00_JPRB, 0.56856E+00_JPRB, 0.59490E+00_JPRB, 0.61370E+00_JPRB, 0.62979E+00_JPRB ,&
&  0.67111E+00_JPRB, 0.62178E+00_JPRB, 0.60495E+00_JPRB, 0.65804E+00_JPRB, 0.78427E+00_JPRB, 0.85236E+00_JPRB, 0.90817E+00_JPRB ,&
&  0.96337E+00_JPRB, 0.95873E+00_JPRB, 0.95748E+00_JPRB, 0.97185E+00_JPRB, 0.97287E+00_JPRB, 0.96581E+00_JPRB, 0.98191E+00_JPRB ,&
&  0.97790E+00_JPRB, 0.99091E+00_JPRB, 0.10133E+01_JPRB, 0.10295E+01_JPRB, 0.98902E+00_JPRB, 0.95823E+00_JPRB, 0.92252E+00_JPRB ,&
&  0.87923E+00_JPRB, 0.82858E+00_JPRB, 0.77138E+00_JPRB, 0.72618E+00_JPRB, 0.69507E+00_JPRB, 0.69347E+00_JPRB, 0.68949E+00_JPRB ,&
&  0.67583E+00_JPRB, 0.66817E+00_JPRB, 0.65763E+00_JPRB, 0.64697E+00_JPRB, 0.61766E+00_JPRB, 0.58933E+00_JPRB, 0.57455E+00_JPRB ,&
&  0.52998E+00_JPRB, 0.50142E+00_JPRB, 0.46641E+00_JPRB, 0.46551E+00_JPRB, 0.48666E+00_JPRB, 0.50409E+00_JPRB, 0.51355E+00_JPRB ,&
&  0.53350E+00_JPRB, 0.54591E+00_JPRB, 0.55156E+00_JPRB, 0.56495E+00_JPRB, 0.58625E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.62838E+00_JPRB, 0.63871E+00_JPRB, 0.64393E+00_JPRB, 0.64093E+00_JPRB, 0.64292E+00_JPRB, 0.63802E+00_JPRB, 0.64127E+00_JPRB ,&
&  0.66991E+00_JPRB, 0.62224E+00_JPRB, 0.65625E+00_JPRB, 0.54919E+00_JPRB, 0.49770E+00_JPRB, 0.47081E+00_JPRB, 0.45046E+00_JPRB ,&
&  0.44587E+00_JPRB, 0.45172E+00_JPRB, 0.45751E+00_JPRB, 0.46521E+00_JPRB, 0.47754E+00_JPRB, 0.47866E+00_JPRB, 0.47933E+00_JPRB ,&
&  0.48419E+00_JPRB, 0.48883E+00_JPRB, 0.49427E+00_JPRB, 0.48410E+00_JPRB, 0.47868E+00_JPRB, 0.47592E+00_JPRB, 0.47849E+00_JPRB ,&
&  0.46405E+00_JPRB, 0.46059E+00_JPRB, 0.46646E+00_JPRB, 0.46316E+00_JPRB, 0.46138E+00_JPRB, 0.45989E+00_JPRB, 0.46273E+00_JPRB ,&
&  0.47014E+00_JPRB, 0.45109E+00_JPRB, 0.47450E+00_JPRB, 0.48391E+00_JPRB, 0.49083E+00_JPRB, 0.49847E+00_JPRB, 0.50470E+00_JPRB ,&
&  0.51596E+00_JPRB, 0.53500E+00_JPRB, 0.56271E+00_JPRB, 0.59134E+00_JPRB, 0.62430E+00_JPRB, 0.65269E+00_JPRB, 0.67450E+00_JPRB ,&
&  0.72514E+00_JPRB, 0.64293E+00_JPRB, 0.74204E+00_JPRB, 0.79492E+00_JPRB, 0.90960E+00_JPRB, 0.10577E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.12238E+01_JPRB, 0.12079E+01_JPRB, 0.11808E+01_JPRB, 0.11460E+01_JPRB, 0.11275E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.11619E+01_JPRB, 0.11827E+01_JPRB, 0.12665E+01_JPRB, 0.12289E+01_JPRB, 0.11335E+01_JPRB, 0.10745E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.96306E+00_JPRB, 0.93089E+00_JPRB, 0.89740E+00_JPRB, 0.81649E+00_JPRB, 0.79673E+00_JPRB, 0.78502E+00_JPRB, 0.73773E+00_JPRB ,&
&  0.71495E+00_JPRB, 0.67935E+00_JPRB, 0.63798E+00_JPRB, 0.64470E+00_JPRB, 0.61327E+00_JPRB, 0.58969E+00_JPRB, 0.58452E+00_JPRB ,&
&  0.56232E+00_JPRB, 0.52267E+00_JPRB, 0.52396E+00_JPRB, 0.53549E+00_JPRB, 0.53259E+00_JPRB, 0.53271E+00_JPRB, 0.55848E+00_JPRB ,&
&  0.56663E+00_JPRB, 0.57885E+00_JPRB, 0.60824E+00_JPRB, 0.61365E+00_JPRB, 0.62378E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.63854E+00_JPRB, 0.63269E+00_JPRB, 0.63128E+00_JPRB, 0.63535E+00_JPRB, 0.65172E+00_JPRB, 0.67928E+00_JPRB, 0.66835E+00_JPRB ,&
&  0.68991E+00_JPRB, 0.69249E+00_JPRB, 0.68068E+00_JPRB, 0.66438E+00_JPRB, 0.63558E+00_JPRB, 0.55762E+00_JPRB, 0.46916E+00_JPRB ,&
&  0.44330E+00_JPRB, 0.44831E+00_JPRB, 0.46035E+00_JPRB, 0.46656E+00_JPRB, 0.47511E+00_JPRB, 0.47594E+00_JPRB, 0.47845E+00_JPRB ,&
&  0.48538E+00_JPRB, 0.49232E+00_JPRB, 0.49719E+00_JPRB, 0.49641E+00_JPRB, 0.48740E+00_JPRB, 0.48997E+00_JPRB, 0.49998E+00_JPRB ,&
&  0.47791E+00_JPRB, 0.46615E+00_JPRB, 0.46210E+00_JPRB, 0.45439E+00_JPRB, 0.46973E+00_JPRB, 0.47674E+00_JPRB, 0.48455E+00_JPRB ,&
&  0.49439E+00_JPRB, 0.50405E+00_JPRB, 0.50761E+00_JPRB, 0.51450E+00_JPRB, 0.52480E+00_JPRB, 0.54262E+00_JPRB, 0.56092E+00_JPRB ,&
&  0.58264E+00_JPRB, 0.60449E+00_JPRB, 0.63150E+00_JPRB, 0.66709E+00_JPRB, 0.71237E+00_JPRB, 0.71431E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.81550E+00_JPRB, 0.91102E+00_JPRB, 0.10230E+01_JPRB, 0.99950E+00_JPRB, 0.11278E+01_JPRB, 0.12454E+01_JPRB, 0.12603E+01_JPRB ,&
&  0.13365E+01_JPRB, 0.12982E+01_JPRB, 0.13507E+01_JPRB, 0.14012E+01_JPRB, 0.14397E+01_JPRB, 0.14291E+01_JPRB, 0.13713E+01_JPRB ,&
&  0.13589E+01_JPRB, 0.15138E+01_JPRB, 0.15313E+01_JPRB, 0.14465E+01_JPRB, 0.13075E+01_JPRB, 0.12210E+01_JPRB, 0.11253E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.11013E+01_JPRB, 0.10639E+01_JPRB, 0.10228E+01_JPRB, 0.95096E+00_JPRB, 0.87481E+00_JPRB, 0.81515E+00_JPRB ,&
&  0.74827E+00_JPRB, 0.66018E+00_JPRB, 0.61839E+00_JPRB, 0.60987E+00_JPRB, 0.60507E+00_JPRB, 0.57417E+00_JPRB, 0.56848E+00_JPRB ,&
&  0.57018E+00_JPRB, 0.56382E+00_JPRB, 0.58774E+00_JPRB, 0.59018E+00_JPRB, 0.58535E+00_JPRB, 0.57667E+00_JPRB, 0.56002E+00_JPRB ,&
&  0.55076E+00_JPRB, 0.58628E+00_JPRB, 0.61495E+00_JPRB, 0.63165E+00_JPRB, 0.63773E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.64216E+00_JPRB, 0.63374E+00_JPRB, 0.62007E+00_JPRB, 0.62730E+00_JPRB, 0.65819E+00_JPRB, 0.68779E+00_JPRB, 0.70453E+00_JPRB ,&
&  0.70100E+00_JPRB, 0.69929E+00_JPRB, 0.69080E+00_JPRB, 0.67872E+00_JPRB, 0.65720E+00_JPRB, 0.63235E+00_JPRB, 0.56746E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.47482E+00_JPRB, 0.45644E+00_JPRB, 0.46029E+00_JPRB, 0.47655E+00_JPRB, 0.47831E+00_JPRB, 0.48146E+00_JPRB ,&
&  0.48954E+00_JPRB, 0.49605E+00_JPRB, 0.50034E+00_JPRB, 0.49753E+00_JPRB, 0.49376E+00_JPRB, 0.51469E+00_JPRB, 0.50984E+00_JPRB ,&
&  0.47960E+00_JPRB, 0.46182E+00_JPRB, 0.45817E+00_JPRB, 0.46210E+00_JPRB, 0.47565E+00_JPRB, 0.49265E+00_JPRB, 0.50227E+00_JPRB ,&
&  0.51123E+00_JPRB, 0.52108E+00_JPRB, 0.52766E+00_JPRB, 0.54215E+00_JPRB, 0.56369E+00_JPRB, 0.59139E+00_JPRB, 0.61691E+00_JPRB ,&
&  0.63760E+00_JPRB, 0.66387E+00_JPRB, 0.70005E+00_JPRB, 0.74872E+00_JPRB, 0.78385E+00_JPRB, 0.82277E+00_JPRB, 0.89213E+00_JPRB ,&
&  0.95983E+00_JPRB, 0.10600E+01_JPRB, 0.11245E+01_JPRB, 0.12248E+01_JPRB, 0.13820E+01_JPRB, 0.14354E+01_JPRB, 0.14461E+01_JPRB ,&
&  0.14838E+01_JPRB, 0.14496E+01_JPRB, 0.16930E+01_JPRB, 0.17403E+01_JPRB, 0.17110E+01_JPRB, 0.17189E+01_JPRB, 0.16171E+01_JPRB ,&
&  0.15456E+01_JPRB, 0.15167E+01_JPRB, 0.14871E+01_JPRB, 0.14431E+01_JPRB, 0.13891E+01_JPRB, 0.13929E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.12792E+01_JPRB, 0.10910E+01_JPRB, 0.95182E+00_JPRB, 0.82542E+00_JPRB, 0.83627E+00_JPRB, 0.81570E+00_JPRB ,&
&  0.74551E+00_JPRB, 0.64252E+00_JPRB, 0.63594E+00_JPRB, 0.62426E+00_JPRB, 0.62927E+00_JPRB, 0.63608E+00_JPRB, 0.61856E+00_JPRB ,&
&  0.62564E+00_JPRB, 0.65150E+00_JPRB, 0.65453E+00_JPRB, 0.67269E+00_JPRB, 0.66131E+00_JPRB, 0.63556E+00_JPRB, 0.56898E+00_JPRB ,&
&  0.55567E+00_JPRB, 0.58313E+00_JPRB, 0.60532E+00_JPRB, 0.62671E+00_JPRB, 0.64012E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.66030E+00_JPRB, 0.65565E+00_JPRB, 0.64362E+00_JPRB, 0.64818E+00_JPRB, 0.67601E+00_JPRB, 0.70536E+00_JPRB, 0.72129E+00_JPRB ,&
&  0.71794E+00_JPRB, 0.71851E+00_JPRB, 0.70503E+00_JPRB, 0.69167E+00_JPRB, 0.67642E+00_JPRB, 0.65109E+00_JPRB, 0.64521E+00_JPRB ,&
&  0.53383E+00_JPRB, 0.49374E+00_JPRB, 0.44825E+00_JPRB, 0.44343E+00_JPRB, 0.48237E+00_JPRB, 0.48530E+00_JPRB, 0.49222E+00_JPRB ,&
&  0.49498E+00_JPRB, 0.50490E+00_JPRB, 0.50352E+00_JPRB, 0.50085E+00_JPRB, 0.51161E+00_JPRB, 0.53273E+00_JPRB, 0.51889E+00_JPRB ,&
&  0.49503E+00_JPRB, 0.47826E+00_JPRB, 0.47724E+00_JPRB, 0.47964E+00_JPRB, 0.48968E+00_JPRB, 0.50691E+00_JPRB, 0.52013E+00_JPRB ,&
&  0.53233E+00_JPRB, 0.55727E+00_JPRB, 0.57901E+00_JPRB, 0.60649E+00_JPRB, 0.63866E+00_JPRB, 0.68299E+00_JPRB, 0.71917E+00_JPRB ,&
&  0.75745E+00_JPRB, 0.80345E+00_JPRB, 0.85261E+00_JPRB, 0.91040E+00_JPRB, 0.93837E+00_JPRB, 0.10071E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.12355E+01_JPRB, 0.12879E+01_JPRB, 0.13165E+01_JPRB, 0.16213E+01_JPRB, 0.17762E+01_JPRB, 0.17815E+01_JPRB, 0.16759E+01_JPRB ,&
&  0.16383E+01_JPRB, 0.16063E+01_JPRB, 0.16576E+01_JPRB, 0.17351E+01_JPRB, 0.17386E+01_JPRB, 0.17956E+01_JPRB, 0.17142E+01_JPRB ,&
&  0.15477E+01_JPRB, 0.14735E+01_JPRB, 0.14359E+01_JPRB, 0.14482E+01_JPRB, 0.13894E+01_JPRB, 0.14142E+01_JPRB, 0.13654E+01_JPRB ,&
&  0.13843E+01_JPRB, 0.12064E+01_JPRB, 0.91492E+00_JPRB, 0.83896E+00_JPRB, 0.69354E+00_JPRB, 0.70953E+00_JPRB, 0.85858E+00_JPRB ,&
&  0.83209E+00_JPRB, 0.75146E+00_JPRB, 0.69514E+00_JPRB, 0.66516E+00_JPRB, 0.66452E+00_JPRB, 0.76182E+00_JPRB, 0.76223E+00_JPRB ,&
&  0.70085E+00_JPRB, 0.73406E+00_JPRB, 0.77685E+00_JPRB, 0.80459E+00_JPRB, 0.78813E+00_JPRB, 0.75548E+00_JPRB, 0.68690E+00_JPRB ,&
&  0.66904E+00_JPRB, 0.65792E+00_JPRB, 0.65808E+00_JPRB, 0.66165E+00_JPRB, 0.66228E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.71092E+00_JPRB, 0.70293E+00_JPRB, 0.69225E+00_JPRB, 0.69544E+00_JPRB, 0.71417E+00_JPRB, 0.73901E+00_JPRB, 0.75351E+00_JPRB ,&
&  0.74643E+00_JPRB, 0.74191E+00_JPRB, 0.72534E+00_JPRB, 0.70931E+00_JPRB, 0.69343E+00_JPRB, 0.67083E+00_JPRB, 0.65641E+00_JPRB ,&
&  0.65297E+00_JPRB, 0.55122E+00_JPRB, 0.47963E+00_JPRB, 0.44191E+00_JPRB, 0.46487E+00_JPRB, 0.47580E+00_JPRB, 0.49251E+00_JPRB ,&
&  0.49450E+00_JPRB, 0.51471E+00_JPRB, 0.51503E+00_JPRB, 0.51550E+00_JPRB, 0.54528E+00_JPRB, 0.55215E+00_JPRB, 0.54107E+00_JPRB ,&
&  0.53598E+00_JPRB, 0.52013E+00_JPRB, 0.51645E+00_JPRB, 0.52551E+00_JPRB, 0.54304E+00_JPRB, 0.53337E+00_JPRB, 0.56048E+00_JPRB ,&
&  0.59296E+00_JPRB, 0.62725E+00_JPRB, 0.65755E+00_JPRB, 0.70343E+00_JPRB, 0.76231E+00_JPRB, 0.81257E+00_JPRB, 0.86675E+00_JPRB ,&
&  0.91946E+00_JPRB, 0.95501E+00_JPRB, 0.97039E+00_JPRB, 0.10153E+01_JPRB, 0.10925E+01_JPRB, 0.12315E+01_JPRB, 0.12429E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.13398E+01_JPRB, 0.12934E+01_JPRB, 0.16627E+01_JPRB, 0.19267E+01_JPRB, 0.19374E+01_JPRB, 0.18788E+01_JPRB ,&
&  0.18498E+01_JPRB, 0.18190E+01_JPRB, 0.19093E+01_JPRB, 0.19345E+01_JPRB, 0.19427E+01_JPRB, 0.19120E+01_JPRB, 0.17714E+01_JPRB ,&
&  0.16021E+01_JPRB, 0.14900E+01_JPRB, 0.14347E+01_JPRB, 0.13820E+01_JPRB, 0.13345E+01_JPRB, 0.12927E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.14495E+01_JPRB, 0.86214E+00_JPRB, 0.70533E+00_JPRB, 0.79704E+00_JPRB, 0.68939E+00_JPRB, 0.70681E+00_JPRB, 0.78298E+00_JPRB ,&
&  0.76813E+00_JPRB, 0.75386E+00_JPRB, 0.77054E+00_JPRB, 0.75589E+00_JPRB, 0.78591E+00_JPRB, 0.95948E+00_JPRB, 0.98008E+00_JPRB ,&
&  0.92320E+00_JPRB, 0.87680E+00_JPRB, 0.93404E+00_JPRB, 0.93772E+00_JPRB, 0.92893E+00_JPRB, 0.90213E+00_JPRB, 0.85075E+00_JPRB ,&
&  0.80581E+00_JPRB, 0.77873E+00_JPRB, 0.75291E+00_JPRB, 0.73301E+00_JPRB, 0.71987E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.80268E+00_JPRB, 0.79371E+00_JPRB, 0.78604E+00_JPRB, 0.77982E+00_JPRB, 0.78471E+00_JPRB, 0.80267E+00_JPRB, 0.80966E+00_JPRB ,&
&  0.79691E+00_JPRB, 0.78402E+00_JPRB, 0.75961E+00_JPRB, 0.73779E+00_JPRB, 0.71784E+00_JPRB, 0.69950E+00_JPRB, 0.68390E+00_JPRB ,&
&  0.66723E+00_JPRB, 0.61561E+00_JPRB, 0.51500E+00_JPRB, 0.46753E+00_JPRB, 0.44118E+00_JPRB, 0.45910E+00_JPRB, 0.47960E+00_JPRB ,&
&  0.49443E+00_JPRB, 0.51232E+00_JPRB, 0.52310E+00_JPRB, 0.56121E+00_JPRB, 0.61315E+00_JPRB, 0.59515E+00_JPRB, 0.60852E+00_JPRB ,&
&  0.60899E+00_JPRB, 0.59319E+00_JPRB, 0.56823E+00_JPRB, 0.59043E+00_JPRB, 0.58956E+00_JPRB, 0.58822E+00_JPRB, 0.66133E+00_JPRB ,&
&  0.71404E+00_JPRB, 0.75336E+00_JPRB, 0.79517E+00_JPRB, 0.84012E+00_JPRB, 0.88658E+00_JPRB, 0.90399E+00_JPRB, 0.94249E+00_JPRB ,&
&  0.99056E+00_JPRB, 0.10283E+01_JPRB, 0.10677E+01_JPRB, 0.10886E+01_JPRB, 0.11453E+01_JPRB, 0.12240E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.12713E+01_JPRB, 0.11521E+01_JPRB, 0.92749E+00_JPRB, 0.12552E+01_JPRB, 0.18821E+01_JPRB, 0.20243E+01_JPRB, 0.19035E+01_JPRB ,&
&  0.20348E+01_JPRB, 0.20702E+01_JPRB, 0.22196E+01_JPRB, 0.23424E+01_JPRB, 0.22720E+01_JPRB, 0.21360E+01_JPRB, 0.19673E+01_JPRB ,&
&  0.17639E+01_JPRB, 0.16241E+01_JPRB, 0.15629E+01_JPRB, 0.16507E+01_JPRB, 0.16081E+01_JPRB, 0.16209E+01_JPRB, 0.16244E+01_JPRB ,&
&  0.14309E+01_JPRB, 0.12005E+01_JPRB, 0.87182E+00_JPRB, 0.89534E+00_JPRB, 0.72478E+00_JPRB, 0.73281E+00_JPRB, 0.80509E+00_JPRB ,&
&  0.78069E+00_JPRB, 0.82530E+00_JPRB, 0.85707E+00_JPRB, 0.86119E+00_JPRB, 0.10322E+01_JPRB, 0.11978E+01_JPRB, 0.13421E+01_JPRB ,&
&  0.13110E+01_JPRB, 0.11237E+01_JPRB, 0.11711E+01_JPRB, 0.11369E+01_JPRB, 0.11366E+01_JPRB, 0.11051E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.10001E+01_JPRB, 0.93687E+00_JPRB, 0.88362E+00_JPRB, 0.84218E+00_JPRB, 0.81709E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.86937E+00_JPRB, 0.84749E+00_JPRB, 0.83354E+00_JPRB, 0.82740E+00_JPRB, 0.83266E+00_JPRB, 0.84758E+00_JPRB, 0.84627E+00_JPRB ,&
&  0.83219E+00_JPRB, 0.81036E+00_JPRB, 0.78257E+00_JPRB, 0.75666E+00_JPRB, 0.73498E+00_JPRB, 0.71905E+00_JPRB, 0.70566E+00_JPRB ,&
&  0.68266E+00_JPRB, 0.60094E+00_JPRB, 0.49469E+00_JPRB, 0.44613E+00_JPRB, 0.42562E+00_JPRB, 0.43200E+00_JPRB, 0.46411E+00_JPRB ,&
&  0.49710E+00_JPRB, 0.52393E+00_JPRB, 0.56126E+00_JPRB, 0.62241E+00_JPRB, 0.67682E+00_JPRB, 0.68666E+00_JPRB, 0.70493E+00_JPRB ,&
&  0.77403E+00_JPRB, 0.66191E+00_JPRB, 0.66467E+00_JPRB, 0.66937E+00_JPRB, 0.73303E+00_JPRB, 0.78313E+00_JPRB, 0.77866E+00_JPRB ,&
&  0.77056E+00_JPRB, 0.80015E+00_JPRB, 0.83527E+00_JPRB, 0.88106E+00_JPRB, 0.92103E+00_JPRB, 0.98063E+00_JPRB, 0.10943E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.12066E+01_JPRB, 0.11523E+01_JPRB, 0.11191E+01_JPRB, 0.11957E+01_JPRB, 0.12582E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.10608E+01_JPRB, 0.12110E+01_JPRB, 0.15054E+01_JPRB, 0.18676E+01_JPRB, 0.21483E+01_JPRB, 0.22032E+01_JPRB ,&
&  0.22923E+01_JPRB, 0.22733E+01_JPRB, 0.23414E+01_JPRB, 0.21506E+01_JPRB, 0.23327E+01_JPRB, 0.22727E+01_JPRB, 0.21808E+01_JPRB ,&
&  0.21311E+01_JPRB, 0.21096E+01_JPRB, 0.20406E+01_JPRB, 0.21018E+01_JPRB, 0.22498E+01_JPRB, 0.23144E+01_JPRB, 0.18428E+01_JPRB ,&
&  0.14543E+01_JPRB, 0.14685E+01_JPRB, 0.12636E+01_JPRB, 0.10944E+01_JPRB, 0.95451E+00_JPRB, 0.96060E+00_JPRB, 0.95494E+00_JPRB ,&
&  0.94976E+00_JPRB, 0.96717E+00_JPRB, 0.95933E+00_JPRB, 0.10547E+01_JPRB, 0.12380E+01_JPRB, 0.14378E+01_JPRB, 0.15878E+01_JPRB ,&
&  0.14616E+01_JPRB, 0.15369E+01_JPRB, 0.14841E+01_JPRB, 0.14149E+01_JPRB, 0.14197E+01_JPRB, 0.13840E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.11069E+01_JPRB, 0.10277E+01_JPRB, 0.95962E+00_JPRB, 0.90610E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.10164E+01_JPRB, 0.97707E+00_JPRB, 0.94904E+00_JPRB, 0.93446E+00_JPRB, 0.93535E+00_JPRB, 0.94355E+00_JPRB, 0.92682E+00_JPRB ,&
&  0.88594E+00_JPRB, 0.84745E+00_JPRB, 0.81644E+00_JPRB, 0.79021E+00_JPRB, 0.76308E+00_JPRB, 0.74138E+00_JPRB, 0.72631E+00_JPRB ,&
&  0.71265E+00_JPRB, 0.59009E+00_JPRB, 0.47573E+00_JPRB, 0.45188E+00_JPRB, 0.43842E+00_JPRB, 0.42356E+00_JPRB, 0.44730E+00_JPRB ,&
&  0.49389E+00_JPRB, 0.55154E+00_JPRB, 0.60998E+00_JPRB, 0.71260E+00_JPRB, 0.77157E+00_JPRB, 0.81061E+00_JPRB, 0.83215E+00_JPRB ,&
&  0.80223E+00_JPRB, 0.79635E+00_JPRB, 0.83792E+00_JPRB, 0.83977E+00_JPRB, 0.82900E+00_JPRB, 0.83651E+00_JPRB, 0.82248E+00_JPRB ,&
&  0.82413E+00_JPRB, 0.84786E+00_JPRB, 0.89503E+00_JPRB, 0.96465E+00_JPRB, 0.10492E+01_JPRB, 0.11583E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.13078E+01_JPRB, 0.12546E+01_JPRB, 0.11303E+01_JPRB, 0.11272E+01_JPRB, 0.10678E+01_JPRB, 0.10794E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.13011E+01_JPRB, 0.13672E+01_JPRB, 0.13343E+01_JPRB, 0.16353E+01_JPRB, 0.16452E+01_JPRB, 0.18537E+01_JPRB, 0.23341E+01_JPRB ,&
&  0.24399E+01_JPRB, 0.24447E+01_JPRB, 0.25036E+01_JPRB, 0.20060E+01_JPRB, 0.17885E+01_JPRB, 0.25092E+01_JPRB, 0.26835E+01_JPRB ,&
&  0.28063E+01_JPRB, 0.29653E+01_JPRB, 0.29806E+01_JPRB, 0.31365E+01_JPRB, 0.26197E+01_JPRB, 0.21330E+01_JPRB, 0.15831E+01_JPRB ,&
&  0.14727E+01_JPRB, 0.13547E+01_JPRB, 0.14547E+01_JPRB, 0.12977E+01_JPRB, 0.12689E+01_JPRB, 0.11721E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.11202E+01_JPRB, 0.11874E+01_JPRB, 0.13410E+01_JPRB, 0.15584E+01_JPRB, 0.15723E+01_JPRB, 0.17909E+01_JPRB ,&
&  0.18646E+01_JPRB, 0.18128E+01_JPRB, 0.16060E+01_JPRB, 0.16089E+01_JPRB, 0.15479E+01_JPRB, 0.14902E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.12503E+01_JPRB, 0.11837E+01_JPRB, 0.11206E+01_JPRB, 0.10654E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.11551E+01_JPRB, 0.11158E+01_JPRB, 0.10838E+01_JPRB, 0.10648E+01_JPRB, 0.10528E+01_JPRB, 0.10310E+01_JPRB, 0.98112E+00_JPRB ,&
&  0.91973E+00_JPRB, 0.87261E+00_JPRB, 0.83006E+00_JPRB, 0.79864E+00_JPRB, 0.76457E+00_JPRB, 0.75079E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.73869E+00_JPRB, 0.65248E+00_JPRB, 0.51149E+00_JPRB, 0.46849E+00_JPRB, 0.46434E+00_JPRB, 0.44484E+00_JPRB, 0.44802E+00_JPRB ,&
&  0.52021E+00_JPRB, 0.61059E+00_JPRB, 0.69990E+00_JPRB, 0.83477E+00_JPRB, 0.89711E+00_JPRB, 0.95584E+00_JPRB, 0.97521E+00_JPRB ,&
&  0.98967E+00_JPRB, 0.90950E+00_JPRB, 0.86108E+00_JPRB, 0.82051E+00_JPRB, 0.79172E+00_JPRB, 0.77763E+00_JPRB, 0.79859E+00_JPRB ,&
&  0.82809E+00_JPRB, 0.89584E+00_JPRB, 0.97835E+00_JPRB, 0.10258E+01_JPRB, 0.11161E+01_JPRB, 0.12025E+01_JPRB, 0.12770E+01_JPRB ,&
&  0.12748E+01_JPRB, 0.12305E+01_JPRB, 0.11963E+01_JPRB, 0.12675E+01_JPRB, 0.12727E+01_JPRB, 0.11854E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.13384E+01_JPRB, 0.13517E+01_JPRB, 0.14121E+01_JPRB, 0.16069E+01_JPRB, 0.18962E+01_JPRB, 0.19962E+01_JPRB, 0.23090E+01_JPRB ,&
&  0.19182E+01_JPRB, 0.16959E+01_JPRB, 0.14803E+01_JPRB, 0.13206E+01_JPRB, 0.18019E+01_JPRB, 0.30970E+01_JPRB, 0.33561E+01_JPRB ,&
&  0.34121E+01_JPRB, 0.36531E+01_JPRB, 0.37443E+01_JPRB, 0.32491E+01_JPRB, 0.25976E+01_JPRB, 0.20476E+01_JPRB, 0.24412E+01_JPRB ,&
&  0.24244E+01_JPRB, 0.26127E+01_JPRB, 0.23154E+01_JPRB, 0.17856E+01_JPRB, 0.14266E+01_JPRB, 0.15754E+01_JPRB, 0.15207E+01_JPRB ,&
&  0.14778E+01_JPRB, 0.14800E+01_JPRB, 0.18524E+01_JPRB, 0.18817E+01_JPRB, 0.18748E+01_JPRB, 0.20108E+01_JPRB, 0.21851E+01_JPRB ,&
&  0.20163E+01_JPRB, 0.19431E+01_JPRB, 0.18508E+01_JPRB, 0.17327E+01_JPRB, 0.16428E+01_JPRB, 0.15529E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.14041E+01_JPRB, 0.13458E+01_JPRB, 0.12838E+01_JPRB, 0.12386E+01_JPRB, 0.11950E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.12528E+01_JPRB, 0.12148E+01_JPRB, 0.11843E+01_JPRB, 0.11520E+01_JPRB, 0.11158E+01_JPRB, 0.10655E+01_JPRB, 0.10016E+01_JPRB ,&
&  0.93904E+00_JPRB, 0.88579E+00_JPRB, 0.84416E+00_JPRB, 0.81226E+00_JPRB, 0.77803E+00_JPRB, 0.76054E+00_JPRB, 0.75754E+00_JPRB ,&
&  0.75252E+00_JPRB, 0.74864E+00_JPRB, 0.58416E+00_JPRB, 0.51121E+00_JPRB, 0.50122E+00_JPRB, 0.49723E+00_JPRB, 0.48602E+00_JPRB ,&
&  0.58519E+00_JPRB, 0.70151E+00_JPRB, 0.78026E+00_JPRB, 0.92545E+00_JPRB, 0.10782E+01_JPRB, 0.10168E+01_JPRB, 0.97998E+00_JPRB ,&
&  0.99406E+00_JPRB, 0.93722E+00_JPRB, 0.87479E+00_JPRB, 0.84329E+00_JPRB, 0.82696E+00_JPRB, 0.83857E+00_JPRB, 0.87271E+00_JPRB ,&
&  0.94143E+00_JPRB, 0.10295E+01_JPRB, 0.10563E+01_JPRB, 0.11040E+01_JPRB, 0.11628E+01_JPRB, 0.12626E+01_JPRB, 0.13347E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.12535E+01_JPRB, 0.12865E+01_JPRB, 0.14265E+01_JPRB, 0.14471E+01_JPRB, 0.13727E+01_JPRB, 0.15149E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.16506E+01_JPRB, 0.16171E+01_JPRB, 0.17796E+01_JPRB, 0.19200E+01_JPRB, 0.19231E+01_JPRB, 0.20640E+01_JPRB ,&
&  0.16285E+01_JPRB, 0.14573E+01_JPRB, 0.15472E+01_JPRB, 0.17882E+01_JPRB, 0.21136E+01_JPRB, 0.27477E+01_JPRB, 0.42420E+01_JPRB ,&
&  0.35546E+01_JPRB, 0.37958E+01_JPRB, 0.37922E+01_JPRB, 0.35745E+01_JPRB, 0.19376E+01_JPRB, 0.15639E+01_JPRB, 0.25676E+01_JPRB ,&
&  0.28654E+01_JPRB, 0.17915E+01_JPRB, 0.12258E+01_JPRB, 0.14319E+01_JPRB, 0.10971E+01_JPRB, 0.13153E+01_JPRB, 0.17585E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.19993E+01_JPRB, 0.25225E+01_JPRB, 0.22627E+01_JPRB, 0.21686E+01_JPRB, 0.28445E+01_JPRB, 0.24877E+01_JPRB ,&
&  0.22953E+01_JPRB, 0.20081E+01_JPRB, 0.19776E+01_JPRB, 0.17635E+01_JPRB, 0.16313E+01_JPRB, 0.15516E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.14446E+01_JPRB, 0.14036E+01_JPRB, 0.13522E+01_JPRB, 0.12966E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.12916E+01_JPRB, 0.12628E+01_JPRB, 0.12382E+01_JPRB, 0.12034E+01_JPRB, 0.11539E+01_JPRB, 0.10899E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.96885E+00_JPRB, 0.91108E+00_JPRB, 0.86653E+00_JPRB, 0.83192E+00_JPRB, 0.79882E+00_JPRB, 0.77565E+00_JPRB, 0.76382E+00_JPRB ,&
&  0.74861E+00_JPRB, 0.73714E+00_JPRB, 0.67946E+00_JPRB, 0.61451E+00_JPRB, 0.60322E+00_JPRB, 0.61827E+00_JPRB, 0.62921E+00_JPRB ,&
&  0.73545E+00_JPRB, 0.84714E+00_JPRB, 0.93769E+00_JPRB, 0.10413E+01_JPRB, 0.10826E+01_JPRB, 0.10134E+01_JPRB, 0.10315E+01_JPRB ,&
&  0.10221E+01_JPRB, 0.98172E+00_JPRB, 0.93277E+00_JPRB, 0.92476E+00_JPRB, 0.93657E+00_JPRB, 0.96224E+00_JPRB, 0.99927E+00_JPRB ,&
&  0.10497E+01_JPRB, 0.10956E+01_JPRB, 0.10986E+01_JPRB, 0.11085E+01_JPRB, 0.11992E+01_JPRB, 0.13191E+01_JPRB, 0.13786E+01_JPRB ,&
&  0.13398E+01_JPRB, 0.13452E+01_JPRB, 0.13945E+01_JPRB, 0.15318E+01_JPRB, 0.17348E+01_JPRB, 0.17370E+01_JPRB, 0.14802E+01_JPRB ,&
&  0.13129E+01_JPRB, 0.14223E+01_JPRB, 0.17915E+01_JPRB, 0.17433E+01_JPRB, 0.17763E+01_JPRB, 0.18447E+01_JPRB, 0.19097E+01_JPRB ,&
&  0.18593E+01_JPRB, 0.19016E+01_JPRB, 0.20071E+01_JPRB, 0.23809E+01_JPRB, 0.28241E+01_JPRB, 0.29177E+01_JPRB, 0.41859E+01_JPRB ,&
&  0.42201E+01_JPRB, 0.38272E+01_JPRB, 0.28844E+01_JPRB, 0.21426E+01_JPRB, 0.21486E+01_JPRB, 0.17394E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.10694E+01_JPRB, 0.94003E+00_JPRB, 0.92312E+00_JPRB, 0.86708E+00_JPRB, 0.11598E+01_JPRB, 0.17528E+01_JPRB ,&
&  0.24364E+01_JPRB, 0.28316E+01_JPRB, 0.29495E+01_JPRB, 0.27040E+01_JPRB, 0.24584E+01_JPRB, 0.31316E+01_JPRB, 0.27781E+01_JPRB ,&
&  0.21417E+01_JPRB, 0.19488E+01_JPRB, 0.18759E+01_JPRB, 0.17302E+01_JPRB, 0.16528E+01_JPRB, 0.15878E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.14955E+01_JPRB, 0.14596E+01_JPRB, 0.14209E+01_JPRB, 0.13728E+01_JPRB, 0.13301E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.13134E+01_JPRB, 0.12912E+01_JPRB, 0.12620E+01_JPRB, 0.12221E+01_JPRB, 0.11667E+01_JPRB, 0.11073E+01_JPRB, 0.10519E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.96399E+00_JPRB, 0.91238E+00_JPRB, 0.86877E+00_JPRB, 0.82925E+00_JPRB, 0.79798E+00_JPRB, 0.78136E+00_JPRB ,&
&  0.77055E+00_JPRB, 0.75441E+00_JPRB, 0.76148E+00_JPRB, 0.74311E+00_JPRB, 0.80562E+00_JPRB, 0.88538E+00_JPRB, 0.94731E+00_JPRB ,&
&  0.10493E+01_JPRB, 0.11302E+01_JPRB, 0.12499E+01_JPRB, 0.12778E+01_JPRB, 0.12827E+01_JPRB, 0.12484E+01_JPRB, 0.11904E+01_JPRB ,&
&  0.11401E+01_JPRB, 0.11334E+01_JPRB, 0.11099E+01_JPRB, 0.10963E+01_JPRB, 0.10818E+01_JPRB, 0.10731E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.11408E+01_JPRB, 0.11150E+01_JPRB, 0.11492E+01_JPRB, 0.12502E+01_JPRB, 0.13550E+01_JPRB, 0.14264E+01_JPRB ,&
&  0.14464E+01_JPRB, 0.14590E+01_JPRB, 0.15266E+01_JPRB, 0.16459E+01_JPRB, 0.15573E+01_JPRB, 0.13116E+01_JPRB, 0.14305E+01_JPRB ,&
&  0.15311E+01_JPRB, 0.17508E+01_JPRB, 0.17145E+01_JPRB, 0.19718E+01_JPRB, 0.19086E+01_JPRB, 0.18123E+01_JPRB, 0.19835E+01_JPRB ,&
&  0.21901E+01_JPRB, 0.28179E+01_JPRB, 0.24848E+01_JPRB, 0.25228E+01_JPRB, 0.37316E+01_JPRB, 0.35813E+01_JPRB, 0.38853E+01_JPRB ,&
&  0.42063E+01_JPRB, 0.34488E+01_JPRB, 0.27683E+01_JPRB, 0.24303E+01_JPRB, 0.33179E+01_JPRB, 0.32696E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.12494E+01_JPRB, 0.10900E+01_JPRB, 0.98373E+00_JPRB, 0.89096E+00_JPRB, 0.84937E+00_JPRB, 0.12250E+01_JPRB, 0.23630E+01_JPRB ,&
&  0.25197E+01_JPRB, 0.36571E+01_JPRB, 0.36686E+01_JPRB, 0.32935E+01_JPRB, 0.28192E+01_JPRB, 0.29949E+01_JPRB, 0.24878E+01_JPRB ,&
&  0.21324E+01_JPRB, 0.20015E+01_JPRB, 0.18307E+01_JPRB, 0.16945E+01_JPRB, 0.15705E+01_JPRB, 0.14993E+01_JPRB, 0.14698E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.14175E+01_JPRB, 0.13947E+01_JPRB, 0.13660E+01_JPRB, 0.13394E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.12879E+01_JPRB, 0.12565E+01_JPRB, 0.12283E+01_JPRB, 0.11891E+01_JPRB, 0.11502E+01_JPRB, 0.11142E+01_JPRB, 0.10760E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10030E+01_JPRB, 0.95105E+00_JPRB, 0.89851E+00_JPRB, 0.85187E+00_JPRB, 0.81605E+00_JPRB, 0.79880E+00_JPRB ,&
&  0.78705E+00_JPRB, 0.78488E+00_JPRB, 0.79553E+00_JPRB, 0.88463E+00_JPRB, 0.10214E+01_JPRB, 0.11150E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.14976E+01_JPRB, 0.16142E+01_JPRB, 0.17616E+01_JPRB, 0.17478E+01_JPRB, 0.16776E+01_JPRB, 0.15793E+01_JPRB, 0.14761E+01_JPRB ,&
&  0.14014E+01_JPRB, 0.13792E+01_JPRB, 0.13157E+01_JPRB, 0.12536E+01_JPRB, 0.11988E+01_JPRB, 0.11669E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11692E+01_JPRB, 0.11473E+01_JPRB, 0.11897E+01_JPRB, 0.12802E+01_JPRB, 0.14036E+01_JPRB, 0.14918E+01_JPRB ,&
&  0.15157E+01_JPRB, 0.15110E+01_JPRB, 0.15649E+01_JPRB, 0.16516E+01_JPRB, 0.13649E+01_JPRB, 0.15453E+01_JPRB, 0.15485E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.16340E+01_JPRB, 0.16065E+01_JPRB, 0.18418E+01_JPRB, 0.19266E+01_JPRB, 0.18335E+01_JPRB, 0.19833E+01_JPRB ,&
&  0.25984E+01_JPRB, 0.25253E+01_JPRB, 0.24531E+01_JPRB, 0.29245E+01_JPRB, 0.37335E+01_JPRB, 0.53563E+01_JPRB, 0.34632E+01_JPRB ,&
&  0.29124E+01_JPRB, 0.32417E+01_JPRB, 0.28279E+01_JPRB, 0.25715E+01_JPRB, 0.32217E+01_JPRB, 0.37962E+01_JPRB, 0.31768E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.10539E+01_JPRB, 0.10993E+01_JPRB, 0.11365E+01_JPRB, 0.10897E+01_JPRB, 0.15145E+01_JPRB, 0.36720E+01_JPRB ,&
&  0.32041E+01_JPRB, 0.41258E+01_JPRB, 0.38692E+01_JPRB, 0.33361E+01_JPRB, 0.29009E+01_JPRB, 0.27995E+01_JPRB, 0.24939E+01_JPRB ,&
&  0.20702E+01_JPRB, 0.18444E+01_JPRB, 0.16611E+01_JPRB, 0.15389E+01_JPRB, 0.14736E+01_JPRB, 0.14253E+01_JPRB, 0.13976E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.13550E+01_JPRB, 0.13437E+01_JPRB, 0.13342E+01_JPRB, 0.13167E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.12338E+01_JPRB, 0.12030E+01_JPRB, 0.11719E+01_JPRB, 0.11519E+01_JPRB, 0.11334E+01_JPRB, 0.10986E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10405E+01_JPRB, 0.99355E+00_JPRB, 0.95010E+00_JPRB, 0.90007E+00_JPRB, 0.85634E+00_JPRB, 0.82583E+00_JPRB, 0.80992E+00_JPRB ,&
&  0.81012E+00_JPRB, 0.81041E+00_JPRB, 0.82311E+00_JPRB, 0.94383E+00_JPRB, 0.11627E+01_JPRB, 0.12349E+01_JPRB, 0.14539E+01_JPRB ,&
&  0.21896E+01_JPRB, 0.24270E+01_JPRB, 0.22840E+01_JPRB, 0.21864E+01_JPRB, 0.21046E+01_JPRB, 0.20150E+01_JPRB, 0.18785E+01_JPRB ,&
&  0.17039E+01_JPRB, 0.15716E+01_JPRB, 0.14281E+01_JPRB, 0.13270E+01_JPRB, 0.12599E+01_JPRB, 0.12248E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.11921E+01_JPRB, 0.11736E+01_JPRB, 0.12056E+01_JPRB, 0.13119E+01_JPRB, 0.14295E+01_JPRB, 0.15241E+01_JPRB ,&
&  0.15433E+01_JPRB, 0.15366E+01_JPRB, 0.15480E+01_JPRB, 0.15141E+01_JPRB, 0.15285E+01_JPRB, 0.15535E+01_JPRB, 0.15985E+01_JPRB ,&
&  0.15909E+01_JPRB, 0.16833E+01_JPRB, 0.17856E+01_JPRB, 0.18201E+01_JPRB, 0.18608E+01_JPRB, 0.19162E+01_JPRB, 0.21179E+01_JPRB ,&
&  0.25382E+01_JPRB, 0.26073E+01_JPRB, 0.27781E+01_JPRB, 0.35617E+01_JPRB, 0.40647E+01_JPRB, 0.46326E+01_JPRB, 0.42538E+01_JPRB ,&
&  0.34447E+01_JPRB, 0.26023E+01_JPRB, 0.24063E+01_JPRB, 0.26759E+01_JPRB, 0.30374E+01_JPRB, 0.32723E+01_JPRB, 0.46053E+01_JPRB ,&
&  0.45484E+01_JPRB, 0.30519E+01_JPRB, 0.25503E+01_JPRB, 0.32607E+01_JPRB, 0.27629E+01_JPRB, 0.24844E+01_JPRB, 0.30704E+01_JPRB ,&
&  0.35421E+01_JPRB, 0.41850E+01_JPRB, 0.38135E+01_JPRB, 0.31267E+01_JPRB, 0.29257E+01_JPRB, 0.26529E+01_JPRB, 0.22805E+01_JPRB ,&
&  0.19092E+01_JPRB, 0.17300E+01_JPRB, 0.16054E+01_JPRB, 0.14875E+01_JPRB, 0.13969E+01_JPRB, 0.13476E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.12991E+01_JPRB, 0.12894E+01_JPRB, 0.12873E+01_JPRB, 0.12826E+01_JPRB, 0.12592E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.11612E+01_JPRB, 0.11413E+01_JPRB, 0.11319E+01_JPRB, 0.10985E+01_JPRB, 0.10589E+01_JPRB, 0.10247E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.99346E+00_JPRB, 0.95710E+00_JPRB, 0.91822E+00_JPRB, 0.88008E+00_JPRB, 0.84150E+00_JPRB, 0.82029E+00_JPRB, 0.81289E+00_JPRB ,&
&  0.81904E+00_JPRB, 0.83339E+00_JPRB, 0.89522E+00_JPRB, 0.10616E+01_JPRB, 0.13052E+01_JPRB, 0.17051E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.23144E+01_JPRB, 0.29933E+01_JPRB, 0.28294E+01_JPRB, 0.26651E+01_JPRB, 0.25014E+01_JPRB, 0.23890E+01_JPRB, 0.21032E+01_JPRB ,&
&  0.18309E+01_JPRB, 0.16574E+01_JPRB, 0.14816E+01_JPRB, 0.13648E+01_JPRB, 0.13055E+01_JPRB, 0.12783E+01_JPRB, 0.12450E+01_JPRB ,&
&  0.12255E+01_JPRB, 0.11942E+01_JPRB, 0.11767E+01_JPRB, 0.12103E+01_JPRB, 0.12911E+01_JPRB, 0.14234E+01_JPRB, 0.15115E+01_JPRB ,&
&  0.15513E+01_JPRB, 0.15488E+01_JPRB, 0.15350E+01_JPRB, 0.14495E+01_JPRB, 0.14925E+01_JPRB, 0.15385E+01_JPRB, 0.15991E+01_JPRB ,&
&  0.15653E+01_JPRB, 0.14988E+01_JPRB, 0.16729E+01_JPRB, 0.18451E+01_JPRB, 0.19178E+01_JPRB, 0.19169E+01_JPRB, 0.20944E+01_JPRB ,&
&  0.26233E+01_JPRB, 0.31111E+01_JPRB, 0.39182E+01_JPRB, 0.41844E+01_JPRB, 0.43269E+01_JPRB, 0.43846E+01_JPRB, 0.42798E+01_JPRB ,&
&  0.41847E+01_JPRB, 0.33293E+01_JPRB, 0.29388E+01_JPRB, 0.28439E+01_JPRB, 0.28302E+01_JPRB, 0.30582E+01_JPRB, 0.40812E+01_JPRB ,&
&  0.50484E+01_JPRB, 0.56120E+01_JPRB, 0.57312E+01_JPRB, 0.48490E+01_JPRB, 0.41316E+01_JPRB, 0.35043E+01_JPRB, 0.32708E+01_JPRB ,&
&  0.38905E+01_JPRB, 0.39809E+01_JPRB, 0.35507E+01_JPRB, 0.37203E+01_JPRB, 0.28773E+01_JPRB, 0.25967E+01_JPRB, 0.22079E+01_JPRB ,&
&  0.19169E+01_JPRB, 0.17091E+01_JPRB, 0.15478E+01_JPRB, 0.14320E+01_JPRB, 0.13593E+01_JPRB, 0.12860E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.12117E+01_JPRB, 0.12122E+01_JPRB, 0.12155E+01_JPRB, 0.12095E+01_JPRB, 0.11842E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.11275E+01_JPRB, 0.11161E+01_JPRB, 0.10960E+01_JPRB, 0.10525E+01_JPRB, 0.10190E+01_JPRB, 0.99760E+00_JPRB, 0.96403E+00_JPRB ,&
&  0.94216E+00_JPRB, 0.91895E+00_JPRB, 0.88582E+00_JPRB, 0.85202E+00_JPRB, 0.82802E+00_JPRB, 0.81012E+00_JPRB, 0.81417E+00_JPRB ,&
&  0.85858E+00_JPRB, 0.94404E+00_JPRB, 0.10952E+01_JPRB, 0.12619E+01_JPRB, 0.15251E+01_JPRB, 0.19551E+01_JPRB, 0.22240E+01_JPRB ,&
&  0.24775E+01_JPRB, 0.36485E+01_JPRB, 0.30885E+01_JPRB, 0.28309E+01_JPRB, 0.26418E+01_JPRB, 0.24557E+01_JPRB, 0.21041E+01_JPRB ,&
&  0.17382E+01_JPRB, 0.15906E+01_JPRB, 0.14955E+01_JPRB, 0.14105E+01_JPRB, 0.13608E+01_JPRB, 0.12929E+01_JPRB, 0.12791E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.12337E+01_JPRB, 0.12124E+01_JPRB, 0.12507E+01_JPRB, 0.13439E+01_JPRB, 0.14481E+01_JPRB, 0.15413E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.15645E+01_JPRB, 0.14983E+01_JPRB, 0.14660E+01_JPRB, 0.14612E+01_JPRB, 0.15254E+01_JPRB, 0.16074E+01_JPRB ,&
&  0.16271E+01_JPRB, 0.15730E+01_JPRB, 0.16737E+01_JPRB, 0.17320E+01_JPRB, 0.17884E+01_JPRB, 0.19984E+01_JPRB, 0.21682E+01_JPRB ,&
&  0.27275E+01_JPRB, 0.28559E+01_JPRB, 0.41593E+01_JPRB, 0.38919E+01_JPRB, 0.38429E+01_JPRB, 0.39884E+01_JPRB, 0.36983E+01_JPRB ,&
&  0.31349E+01_JPRB, 0.29158E+01_JPRB, 0.26665E+01_JPRB, 0.26636E+01_JPRB, 0.28639E+01_JPRB, 0.33139E+01_JPRB, 0.41479E+01_JPRB ,&
&  0.44143E+01_JPRB, 0.42669E+01_JPRB, 0.43007E+01_JPRB, 0.36146E+01_JPRB, 0.34624E+01_JPRB, 0.32203E+01_JPRB, 0.31368E+01_JPRB ,&
&  0.37631E+01_JPRB, 0.37709E+01_JPRB, 0.36040E+01_JPRB, 0.33572E+01_JPRB, 0.25603E+01_JPRB, 0.22472E+01_JPRB, 0.19160E+01_JPRB ,&
&  0.17010E+01_JPRB, 0.15278E+01_JPRB, 0.14309E+01_JPRB, 0.13504E+01_JPRB, 0.12829E+01_JPRB, 0.12128E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11582E+01_JPRB, 0.11553E+01_JPRB, 0.11591E+01_JPRB, 0.11502E+01_JPRB, 0.11364E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.11447E+01_JPRB, 0.11488E+01_JPRB, 0.11494E+01_JPRB, 0.11738E+01_JPRB, 0.12162E+01_JPRB, 0.12820E+01_JPRB, 0.13144E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.88313E+00_JPRB, 0.86143E+00_JPRB, 0.83997E+00_JPRB, 0.81946E+00_JPRB, 0.82350E+00_JPRB, 0.87794E+00_JPRB ,&
&  0.96750E+00_JPRB, 0.10830E+01_JPRB, 0.12171E+01_JPRB, 0.14323E+01_JPRB, 0.17541E+01_JPRB, 0.23227E+01_JPRB, 0.28915E+01_JPRB ,&
&  0.29587E+01_JPRB, 0.31638E+01_JPRB, 0.31414E+01_JPRB, 0.27084E+01_JPRB, 0.24634E+01_JPRB, 0.23201E+01_JPRB, 0.21678E+01_JPRB ,&
&  0.18289E+01_JPRB, 0.16503E+01_JPRB, 0.16103E+01_JPRB, 0.16010E+01_JPRB, 0.13491E+01_JPRB, 0.13247E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.13172E+01_JPRB, 0.13274E+01_JPRB, 0.13381E+01_JPRB, 0.13896E+01_JPRB, 0.14708E+01_JPRB, 0.15616E+01_JPRB, 0.16318E+01_JPRB ,&
&  0.16587E+01_JPRB, 0.16450E+01_JPRB, 0.15997E+01_JPRB, 0.14960E+01_JPRB, 0.14891E+01_JPRB, 0.15455E+01_JPRB, 0.16207E+01_JPRB ,&
&  0.16679E+01_JPRB, 0.17273E+01_JPRB, 0.17694E+01_JPRB, 0.18079E+01_JPRB, 0.18169E+01_JPRB, 0.21528E+01_JPRB, 0.23841E+01_JPRB ,&
&  0.27466E+01_JPRB, 0.31940E+01_JPRB, 0.44805E+01_JPRB, 0.32915E+01_JPRB, 0.34340E+01_JPRB, 0.35134E+01_JPRB, 0.32681E+01_JPRB ,&
&  0.27973E+01_JPRB, 0.26261E+01_JPRB, 0.24302E+01_JPRB, 0.24647E+01_JPRB, 0.28782E+01_JPRB, 0.33977E+01_JPRB, 0.43870E+01_JPRB ,&
&  0.36898E+01_JPRB, 0.37131E+01_JPRB, 0.32090E+01_JPRB, 0.32050E+01_JPRB, 0.31209E+01_JPRB, 0.29385E+01_JPRB, 0.27453E+01_JPRB ,&
&  0.32423E+01_JPRB, 0.30554E+01_JPRB, 0.27368E+01_JPRB, 0.23995E+01_JPRB, 0.18621E+01_JPRB, 0.16291E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.13296E+01_JPRB, 0.12880E+01_JPRB, 0.12794E+01_JPRB, 0.12367E+01_JPRB, 0.12070E+01_JPRB, 0.11749E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.11474E+01_JPRB, 0.11418E+01_JPRB, 0.11384E+01_JPRB, 0.11381E+01_JPRB, 0.11380E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.11963E+01_JPRB, 0.11815E+01_JPRB, 0.11685E+01_JPRB, 0.11592E+01_JPRB, 0.11483E+01_JPRB, 0.11165E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.95835E+00_JPRB, 0.88827E+00_JPRB, 0.87073E+00_JPRB, 0.86003E+00_JPRB, 0.86397E+00_JPRB, 0.89989E+00_JPRB, 0.96784E+00_JPRB ,&
&  0.10441E+01_JPRB, 0.11769E+01_JPRB, 0.13599E+01_JPRB, 0.16070E+01_JPRB, 0.19440E+01_JPRB, 0.24813E+01_JPRB, 0.30731E+01_JPRB ,&
&  0.31238E+01_JPRB, 0.23123E+01_JPRB, 0.26009E+01_JPRB, 0.25247E+01_JPRB, 0.23680E+01_JPRB, 0.21969E+01_JPRB, 0.19451E+01_JPRB ,&
&  0.16843E+01_JPRB, 0.16090E+01_JPRB, 0.15807E+01_JPRB, 0.15135E+01_JPRB, 0.13793E+01_JPRB, 0.14132E+01_JPRB, 0.14536E+01_JPRB ,&
&  0.14956E+01_JPRB, 0.15289E+01_JPRB, 0.15359E+01_JPRB, 0.15901E+01_JPRB, 0.16442E+01_JPRB, 0.17279E+01_JPRB, 0.18200E+01_JPRB ,&
&  0.18573E+01_JPRB, 0.18652E+01_JPRB, 0.18472E+01_JPRB, 0.17220E+01_JPRB, 0.16866E+01_JPRB, 0.17598E+01_JPRB, 0.18299E+01_JPRB ,&
&  0.18734E+01_JPRB, 0.18586E+01_JPRB, 0.19511E+01_JPRB, 0.21851E+01_JPRB, 0.24746E+01_JPRB, 0.23734E+01_JPRB, 0.27766E+01_JPRB ,&
&  0.30616E+01_JPRB, 0.36109E+01_JPRB, 0.35225E+01_JPRB, 0.40419E+01_JPRB, 0.26599E+01_JPRB, 0.28733E+01_JPRB, 0.25540E+01_JPRB ,&
&  0.25909E+01_JPRB, 0.23485E+01_JPRB, 0.23209E+01_JPRB, 0.25808E+01_JPRB, 0.31527E+01_JPRB, 0.38609E+01_JPRB, 0.39060E+01_JPRB ,&
&  0.28692E+01_JPRB, 0.26418E+01_JPRB, 0.27455E+01_JPRB, 0.30459E+01_JPRB, 0.30292E+01_JPRB, 0.28932E+01_JPRB, 0.24645E+01_JPRB ,&
&  0.24765E+01_JPRB, 0.22364E+01_JPRB, 0.20338E+01_JPRB, 0.17724E+01_JPRB, 0.13387E+01_JPRB, 0.11885E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.12155E+01_JPRB, 0.12231E+01_JPRB, 0.12241E+01_JPRB, 0.12033E+01_JPRB, 0.11815E+01_JPRB, 0.11669E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.11654E+01_JPRB, 0.11722E+01_JPRB, 0.11768E+01_JPRB, 0.11882E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.11608E+01_JPRB, 0.11319E+01_JPRB, 0.11044E+01_JPRB, 0.10793E+01_JPRB, 0.10598E+01_JPRB, 0.10140E+01_JPRB, 0.95986E+00_JPRB ,&
&  0.92505E+00_JPRB, 0.90662E+00_JPRB, 0.90536E+00_JPRB, 0.91473E+00_JPRB, 0.94395E+00_JPRB, 0.97775E+00_JPRB, 0.10400E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.12925E+01_JPRB, 0.14964E+01_JPRB, 0.17497E+01_JPRB, 0.20761E+01_JPRB, 0.24597E+01_JPRB, 0.27303E+01_JPRB ,&
&  0.29925E+01_JPRB, 0.30455E+01_JPRB, 0.25903E+01_JPRB, 0.18915E+01_JPRB, 0.17864E+01_JPRB, 0.18429E+01_JPRB, 0.16783E+01_JPRB ,&
&  0.16469E+01_JPRB, 0.15929E+01_JPRB, 0.15389E+01_JPRB, 0.15405E+01_JPRB, 0.15525E+01_JPRB, 0.16147E+01_JPRB, 0.16678E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.17381E+01_JPRB, 0.17824E+01_JPRB, 0.18704E+01_JPRB, 0.19514E+01_JPRB, 0.20472E+01_JPRB, 0.21214E+01_JPRB ,&
&  0.21687E+01_JPRB, 0.22398E+01_JPRB, 0.22368E+01_JPRB, 0.21643E+01_JPRB, 0.20869E+01_JPRB, 0.21592E+01_JPRB, 0.22520E+01_JPRB ,&
&  0.23300E+01_JPRB, 0.23605E+01_JPRB, 0.24325E+01_JPRB, 0.27348E+01_JPRB, 0.30022E+01_JPRB, 0.27895E+01_JPRB, 0.29155E+01_JPRB ,&
&  0.34106E+01_JPRB, 0.37688E+01_JPRB, 0.28652E+01_JPRB, 0.33977E+01_JPRB, 0.20036E+01_JPRB, 0.24595E+01_JPRB, 0.25884E+01_JPRB ,&
&  0.23625E+01_JPRB, 0.23557E+01_JPRB, 0.24390E+01_JPRB, 0.28122E+01_JPRB, 0.33689E+01_JPRB, 0.36978E+01_JPRB, 0.31840E+01_JPRB ,&
&  0.23505E+01_JPRB, 0.22720E+01_JPRB, 0.24142E+01_JPRB, 0.25792E+01_JPRB, 0.25990E+01_JPRB, 0.24018E+01_JPRB, 0.18142E+01_JPRB ,&
&  0.14297E+01_JPRB, 0.16940E+01_JPRB, 0.17008E+01_JPRB, 0.16594E+01_JPRB, 0.12429E+01_JPRB, 0.10767E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.12430E+01_JPRB, 0.12582E+01_JPRB, 0.12481E+01_JPRB, 0.12233E+01_JPRB, 0.12015E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.11869E+01_JPRB, 0.11879E+01_JPRB, 0.11914E+01_JPRB, 0.11843E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.10634E+01_JPRB, 0.10278E+01_JPRB, 0.10045E+01_JPRB, 0.98665E+00_JPRB, 0.97839E+00_JPRB, 0.95282E+00_JPRB, 0.91714E+00_JPRB ,&
&  0.89631E+00_JPRB, 0.88959E+00_JPRB, 0.90324E+00_JPRB, 0.93855E+00_JPRB, 0.98785E+00_JPRB, 0.10362E+01_JPRB, 0.11282E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.14119E+01_JPRB, 0.16298E+01_JPRB, 0.19507E+01_JPRB, 0.22224E+01_JPRB, 0.24827E+01_JPRB, 0.25557E+01_JPRB ,&
&  0.26732E+01_JPRB, 0.26953E+01_JPRB, 0.25509E+01_JPRB, 0.23912E+01_JPRB, 0.19213E+01_JPRB, 0.19473E+01_JPRB, 0.21193E+01_JPRB ,&
&  0.22177E+01_JPRB, 0.19263E+01_JPRB, 0.17105E+01_JPRB, 0.17393E+01_JPRB, 0.17605E+01_JPRB, 0.18344E+01_JPRB, 0.19024E+01_JPRB ,&
&  0.19595E+01_JPRB, 0.20211E+01_JPRB, 0.20856E+01_JPRB, 0.21800E+01_JPRB, 0.22829E+01_JPRB, 0.23457E+01_JPRB, 0.24067E+01_JPRB ,&
&  0.24460E+01_JPRB, 0.24950E+01_JPRB, 0.25387E+01_JPRB, 0.25466E+01_JPRB, 0.26179E+01_JPRB, 0.27292E+01_JPRB, 0.29243E+01_JPRB ,&
&  0.31245E+01_JPRB, 0.31197E+01_JPRB, 0.33455E+01_JPRB, 0.37198E+01_JPRB, 0.37763E+01_JPRB, 0.36374E+01_JPRB, 0.36165E+01_JPRB ,&
&  0.39806E+01_JPRB, 0.43021E+01_JPRB, 0.23632E+01_JPRB, 0.25952E+01_JPRB, 0.29903E+01_JPRB, 0.25627E+01_JPRB, 0.23126E+01_JPRB ,&
&  0.23221E+01_JPRB, 0.24024E+01_JPRB, 0.26058E+01_JPRB, 0.28749E+01_JPRB, 0.33288E+01_JPRB, 0.41213E+01_JPRB, 0.28305E+01_JPRB ,&
&  0.19614E+01_JPRB, 0.18527E+01_JPRB, 0.19369E+01_JPRB, 0.20217E+01_JPRB, 0.20235E+01_JPRB, 0.17616E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.15912E+01_JPRB, 0.17070E+01_JPRB, 0.17963E+01_JPRB, 0.15752E+01_JPRB, 0.10548E+01_JPRB, 0.11604E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.12342E+01_JPRB, 0.12144E+01_JPRB, 0.12076E+01_JPRB, 0.12019E+01_JPRB, 0.11917E+01_JPRB, 0.11900E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.11854E+01_JPRB, 0.11852E+01_JPRB, 0.11542E+01_JPRB, 0.11032E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.96226E+00_JPRB, 0.93461E+00_JPRB, 0.91923E+00_JPRB, 0.89808E+00_JPRB, 0.88521E+00_JPRB, 0.86948E+00_JPRB, 0.86219E+00_JPRB ,&
&  0.86133E+00_JPRB, 0.86747E+00_JPRB, 0.89055E+00_JPRB, 0.92731E+00_JPRB, 0.97951E+00_JPRB, 0.10507E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.15034E+01_JPRB, 0.17487E+01_JPRB, 0.20396E+01_JPRB, 0.22674E+01_JPRB, 0.23627E+01_JPRB, 0.25012E+01_JPRB ,&
&  0.25982E+01_JPRB, 0.26366E+01_JPRB, 0.26092E+01_JPRB, 0.25588E+01_JPRB, 0.24786E+01_JPRB, 0.25239E+01_JPRB, 0.26663E+01_JPRB ,&
&  0.24700E+01_JPRB, 0.19619E+01_JPRB, 0.18631E+01_JPRB, 0.18940E+01_JPRB, 0.19647E+01_JPRB, 0.20183E+01_JPRB, 0.21085E+01_JPRB ,&
&  0.21854E+01_JPRB, 0.22470E+01_JPRB, 0.23186E+01_JPRB, 0.24000E+01_JPRB, 0.25234E+01_JPRB, 0.26089E+01_JPRB, 0.27133E+01_JPRB ,&
&  0.28089E+01_JPRB, 0.29090E+01_JPRB, 0.30744E+01_JPRB, 0.29475E+01_JPRB, 0.32624E+01_JPRB, 0.35013E+01_JPRB, 0.37295E+01_JPRB ,&
&  0.38243E+01_JPRB, 0.38169E+01_JPRB, 0.41408E+01_JPRB, 0.43943E+01_JPRB, 0.45731E+01_JPRB, 0.46224E+01_JPRB, 0.44901E+01_JPRB ,&
&  0.46680E+01_JPRB, 0.43314E+01_JPRB, 0.23507E+01_JPRB, 0.20363E+01_JPRB, 0.21778E+01_JPRB, 0.19389E+01_JPRB, 0.22543E+01_JPRB ,&
&  0.22819E+01_JPRB, 0.24038E+01_JPRB, 0.25692E+01_JPRB, 0.28217E+01_JPRB, 0.31121E+01_JPRB, 0.37525E+01_JPRB, 0.25470E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.14103E+01_JPRB, 0.14262E+01_JPRB, 0.14668E+01_JPRB, 0.15180E+01_JPRB, 0.14055E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.13609E+01_JPRB, 0.14189E+01_JPRB, 0.13046E+01_JPRB, 0.10750E+01_JPRB, 0.10006E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.11235E+01_JPRB, 0.11197E+01_JPRB, 0.11268E+01_JPRB, 0.11199E+01_JPRB, 0.11143E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10915E+01_JPRB, 0.10712E+01_JPRB, 0.10251E+01_JPRB, 0.98789E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.83343E+00_JPRB, 0.83399E+00_JPRB, 0.82197E+00_JPRB, 0.80887E+00_JPRB, 0.80288E+00_JPRB, 0.78425E+00_JPRB, 0.80739E+00_JPRB ,&
&  0.80559E+00_JPRB, 0.83069E+00_JPRB, 0.85686E+00_JPRB, 0.88543E+00_JPRB, 0.93827E+00_JPRB, 0.99425E+00_JPRB, 0.10748E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.14358E+01_JPRB, 0.17223E+01_JPRB, 0.20802E+01_JPRB, 0.22241E+01_JPRB, 0.23260E+01_JPRB, 0.24742E+01_JPRB ,&
&  0.26516E+01_JPRB, 0.28007E+01_JPRB, 0.28948E+01_JPRB, 0.30097E+01_JPRB, 0.31360E+01_JPRB, 0.31136E+01_JPRB, 0.30002E+01_JPRB ,&
&  0.25055E+01_JPRB, 0.18996E+01_JPRB, 0.20536E+01_JPRB, 0.19406E+01_JPRB, 0.20718E+01_JPRB, 0.21895E+01_JPRB, 0.22838E+01_JPRB ,&
&  0.23829E+01_JPRB, 0.24566E+01_JPRB, 0.25572E+01_JPRB, 0.26750E+01_JPRB, 0.28297E+01_JPRB, 0.29819E+01_JPRB, 0.31442E+01_JPRB ,&
&  0.32743E+01_JPRB, 0.34011E+01_JPRB, 0.34950E+01_JPRB, 0.34563E+01_JPRB, 0.37080E+01_JPRB, 0.40426E+01_JPRB, 0.43548E+01_JPRB ,&
&  0.46058E+01_JPRB, 0.46002E+01_JPRB, 0.41427E+01_JPRB, 0.43522E+01_JPRB, 0.46751E+01_JPRB, 0.44780E+01_JPRB, 0.46524E+01_JPRB ,&
&  0.51414E+01_JPRB, 0.45142E+01_JPRB, 0.21836E+01_JPRB, 0.18678E+01_JPRB, 0.19793E+01_JPRB, 0.20462E+01_JPRB, 0.21072E+01_JPRB ,&
&  0.21498E+01_JPRB, 0.22142E+01_JPRB, 0.23216E+01_JPRB, 0.24470E+01_JPRB, 0.24857E+01_JPRB, 0.25405E+01_JPRB, 0.20728E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12064E+01_JPRB, 0.12727E+01_JPRB, 0.13528E+01_JPRB, 0.15347E+01_JPRB, 0.14968E+01_JPRB, 0.12335E+01_JPRB ,&
&  0.13283E+01_JPRB, 0.15381E+01_JPRB, 0.18334E+01_JPRB, 0.15595E+01_JPRB, 0.10193E+01_JPRB, 0.91889E+00_JPRB, 0.91898E+00_JPRB ,&
&  0.95552E+00_JPRB, 0.95923E+00_JPRB, 0.97804E+00_JPRB, 0.99048E+00_JPRB, 0.10074E+01_JPRB, 0.99314E+00_JPRB, 0.97584E+00_JPRB ,&
&  0.95315E+00_JPRB, 0.93249E+00_JPRB, 0.90871E+00_JPRB, 0.87353E+00_JPRB, 0.84782E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.72097E+00_JPRB, 0.68733E+00_JPRB, 0.65305E+00_JPRB, 0.65061E+00_JPRB, 0.65439E+00_JPRB, 0.63964E+00_JPRB, 0.67456E+00_JPRB ,&
&  0.64964E+00_JPRB, 0.67832E+00_JPRB, 0.69453E+00_JPRB, 0.69832E+00_JPRB, 0.74707E+00_JPRB, 0.79717E+00_JPRB, 0.86143E+00_JPRB ,&
&  0.94961E+00_JPRB, 0.10871E+01_JPRB, 0.12619E+01_JPRB, 0.14824E+01_JPRB, 0.17520E+01_JPRB, 0.18910E+01_JPRB, 0.20299E+01_JPRB ,&
&  0.22218E+01_JPRB, 0.24740E+01_JPRB, 0.28301E+01_JPRB, 0.32392E+01_JPRB, 0.37255E+01_JPRB, 0.35946E+01_JPRB, 0.30850E+01_JPRB ,&
&  0.23196E+01_JPRB, 0.19217E+01_JPRB, 0.18298E+01_JPRB, 0.17275E+01_JPRB, 0.18850E+01_JPRB, 0.21858E+01_JPRB, 0.23493E+01_JPRB ,&
&  0.24644E+01_JPRB, 0.25950E+01_JPRB, 0.27386E+01_JPRB, 0.28969E+01_JPRB, 0.30689E+01_JPRB, 0.32643E+01_JPRB, 0.33784E+01_JPRB ,&
&  0.34560E+01_JPRB, 0.35367E+01_JPRB, 0.36107E+01_JPRB, 0.38319E+01_JPRB, 0.41347E+01_JPRB, 0.45320E+01_JPRB, 0.46544E+01_JPRB ,&
&  0.46910E+01_JPRB, 0.45978E+01_JPRB, 0.40018E+01_JPRB, 0.39755E+01_JPRB, 0.40926E+01_JPRB, 0.41342E+01_JPRB, 0.42115E+01_JPRB ,&
&  0.46114E+01_JPRB, 0.36396E+01_JPRB, 0.21794E+01_JPRB, 0.20867E+01_JPRB, 0.20901E+01_JPRB, 0.21389E+01_JPRB, 0.19096E+01_JPRB ,&
&  0.19140E+01_JPRB, 0.19262E+01_JPRB, 0.19634E+01_JPRB, 0.19378E+01_JPRB, 0.19348E+01_JPRB, 0.18842E+01_JPRB, 0.16822E+01_JPRB ,&
&  0.13396E+01_JPRB, 0.12462E+01_JPRB, 0.12055E+01_JPRB, 0.12267E+01_JPRB, 0.15171E+01_JPRB, 0.14886E+01_JPRB, 0.13827E+01_JPRB ,&
&  0.12667E+01_JPRB, 0.14047E+01_JPRB, 0.12441E+01_JPRB, 0.14964E+01_JPRB, 0.91734E+00_JPRB, 0.80437E+00_JPRB, 0.78700E+00_JPRB ,&
&  0.81210E+00_JPRB, 0.82108E+00_JPRB, 0.83644E+00_JPRB, 0.82515E+00_JPRB, 0.83955E+00_JPRB, 0.87151E+00_JPRB, 0.87588E+00_JPRB ,&
&  0.82948E+00_JPRB, 0.81362E+00_JPRB, 0.78086E+00_JPRB, 0.74283E+00_JPRB, 0.73397E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.61063E+00_JPRB, 0.60177E+00_JPRB, 0.59810E+00_JPRB, 0.59347E+00_JPRB, 0.60298E+00_JPRB, 0.60632E+00_JPRB, 0.61033E+00_JPRB ,&
&  0.61300E+00_JPRB, 0.62928E+00_JPRB, 0.64287E+00_JPRB, 0.66198E+00_JPRB, 0.68281E+00_JPRB, 0.72343E+00_JPRB, 0.76902E+00_JPRB ,&
&  0.82239E+00_JPRB, 0.88239E+00_JPRB, 0.94953E+00_JPRB, 0.10475E+01_JPRB, 0.11859E+01_JPRB, 0.12886E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.15516E+01_JPRB, 0.17781E+01_JPRB, 0.20368E+01_JPRB, 0.23877E+01_JPRB, 0.27958E+01_JPRB, 0.29247E+01_JPRB, 0.27454E+01_JPRB ,&
&  0.20950E+01_JPRB, 0.20667E+01_JPRB, 0.19493E+01_JPRB, 0.17945E+01_JPRB, 0.18576E+01_JPRB, 0.18374E+01_JPRB, 0.19213E+01_JPRB ,&
&  0.20361E+01_JPRB, 0.20846E+01_JPRB, 0.21141E+01_JPRB, 0.21776E+01_JPRB, 0.22321E+01_JPRB, 0.22812E+01_JPRB, 0.22744E+01_JPRB ,&
&  0.22875E+01_JPRB, 0.24067E+01_JPRB, 0.28276E+01_JPRB, 0.33519E+01_JPRB, 0.37360E+01_JPRB, 0.39318E+01_JPRB, 0.37526E+01_JPRB ,&
&  0.34771E+01_JPRB, 0.33077E+01_JPRB, 0.31219E+01_JPRB, 0.30860E+01_JPRB, 0.31797E+01_JPRB, 0.32272E+01_JPRB, 0.34490E+01_JPRB ,&
&  0.32642E+01_JPRB, 0.26634E+01_JPRB, 0.23620E+01_JPRB, 0.21720E+01_JPRB, 0.20975E+01_JPRB, 0.19926E+01_JPRB, 0.18375E+01_JPRB ,&
&  0.18563E+01_JPRB, 0.17858E+01_JPRB, 0.17358E+01_JPRB, 0.16135E+01_JPRB, 0.14845E+01_JPRB, 0.13845E+01_JPRB, 0.13237E+01_JPRB ,&
&  0.11994E+01_JPRB, 0.11067E+01_JPRB, 0.11024E+01_JPRB, 0.12284E+01_JPRB, 0.14757E+01_JPRB, 0.23417E+01_JPRB, 0.20814E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.10425E+01_JPRB, 0.74665E+00_JPRB, 0.97850E+00_JPRB, 0.75268E+00_JPRB, 0.70374E+00_JPRB, 0.70110E+00_JPRB ,&
&  0.70623E+00_JPRB, 0.70201E+00_JPRB, 0.68213E+00_JPRB, 0.67831E+00_JPRB, 0.68880E+00_JPRB, 0.69856E+00_JPRB, 0.68930E+00_JPRB ,&
&  0.65195E+00_JPRB, 0.63512E+00_JPRB, 0.66373E+00_JPRB, 0.62112E+00_JPRB, 0.61229E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.56081E+00_JPRB, 0.56168E+00_JPRB, 0.57746E+00_JPRB, 0.59247E+00_JPRB, 0.60187E+00_JPRB, 0.61088E+00_JPRB, 0.62448E+00_JPRB ,&
&  0.63448E+00_JPRB, 0.64725E+00_JPRB, 0.66122E+00_JPRB, 0.68390E+00_JPRB, 0.71531E+00_JPRB, 0.76156E+00_JPRB, 0.81447E+00_JPRB ,&
&  0.87984E+00_JPRB, 0.95064E+00_JPRB, 0.10313E+01_JPRB, 0.11199E+01_JPRB, 0.12300E+01_JPRB, 0.13838E+01_JPRB, 0.15942E+01_JPRB ,&
&  0.17178E+01_JPRB, 0.18766E+01_JPRB, 0.20756E+01_JPRB, 0.22564E+01_JPRB, 0.23031E+01_JPRB, 0.22614E+01_JPRB, 0.18815E+01_JPRB ,&
&  0.22359E+01_JPRB, 0.20741E+01_JPRB, 0.19053E+01_JPRB, 0.17423E+01_JPRB, 0.16286E+01_JPRB, 0.15380E+01_JPRB, 0.14651E+01_JPRB ,&
&  0.14472E+01_JPRB, 0.13954E+01_JPRB, 0.13262E+01_JPRB, 0.12800E+01_JPRB, 0.12599E+01_JPRB, 0.12826E+01_JPRB, 0.12776E+01_JPRB ,&
&  0.14196E+01_JPRB, 0.17472E+01_JPRB, 0.21784E+01_JPRB, 0.27073E+01_JPRB, 0.30127E+01_JPRB, 0.29904E+01_JPRB, 0.24949E+01_JPRB ,&
&  0.22055E+01_JPRB, 0.21011E+01_JPRB, 0.20850E+01_JPRB, 0.22132E+01_JPRB, 0.22268E+01_JPRB, 0.23378E+01_JPRB, 0.24081E+01_JPRB ,&
&  0.23851E+01_JPRB, 0.20781E+01_JPRB, 0.20110E+01_JPRB, 0.23333E+01_JPRB, 0.21455E+01_JPRB, 0.19287E+01_JPRB, 0.18761E+01_JPRB ,&
&  0.17659E+01_JPRB, 0.16921E+01_JPRB, 0.16424E+01_JPRB, 0.15273E+01_JPRB, 0.13846E+01_JPRB, 0.12544E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10045E+01_JPRB, 0.96215E+00_JPRB, 0.10442E+01_JPRB, 0.13097E+01_JPRB, 0.16138E+01_JPRB, 0.22780E+01_JPRB ,&
&  0.12522E+01_JPRB, 0.75793E+00_JPRB, 0.76057E+00_JPRB, 0.75873E+00_JPRB, 0.68226E+00_JPRB, 0.63789E+00_JPRB, 0.68970E+00_JPRB ,&
&  0.75547E+00_JPRB, 0.82292E+00_JPRB, 0.81659E+00_JPRB, 0.70735E+00_JPRB, 0.66492E+00_JPRB, 0.64520E+00_JPRB, 0.62080E+00_JPRB ,&
&  0.60109E+00_JPRB, 0.56445E+00_JPRB, 0.57953E+00_JPRB, 0.56357E+00_JPRB, 0.56263E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.52769E+00_JPRB, 0.52706E+00_JPRB, 0.53667E+00_JPRB, 0.54596E+00_JPRB, 0.54894E+00_JPRB, 0.55243E+00_JPRB, 0.55610E+00_JPRB ,&
&  0.56072E+00_JPRB, 0.56207E+00_JPRB, 0.56550E+00_JPRB, 0.57369E+00_JPRB, 0.59100E+00_JPRB, 0.62888E+00_JPRB, 0.67393E+00_JPRB ,&
&  0.71478E+00_JPRB, 0.75458E+00_JPRB, 0.79552E+00_JPRB, 0.83214E+00_JPRB, 0.87131E+00_JPRB, 0.90159E+00_JPRB, 0.97584E+00_JPRB ,&
&  0.10713E+01_JPRB, 0.11862E+01_JPRB, 0.13074E+01_JPRB, 0.14061E+01_JPRB, 0.15037E+01_JPRB, 0.15012E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.19422E+01_JPRB, 0.18959E+01_JPRB, 0.17451E+01_JPRB, 0.15803E+01_JPRB, 0.14591E+01_JPRB, 0.13492E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.11341E+01_JPRB, 0.11967E+01_JPRB, 0.11503E+01_JPRB, 0.12157E+01_JPRB, 0.10898E+01_JPRB, 0.11560E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.15940E+01_JPRB, 0.18203E+01_JPRB, 0.21014E+01_JPRB, 0.23802E+01_JPRB, 0.25457E+01_JPRB, 0.25030E+01_JPRB, 0.22397E+01_JPRB ,&
&  0.20907E+01_JPRB, 0.20500E+01_JPRB, 0.19844E+01_JPRB, 0.21744E+01_JPRB, 0.20858E+01_JPRB, 0.19457E+01_JPRB, 0.19768E+01_JPRB ,&
&  0.17451E+01_JPRB, 0.17403E+01_JPRB, 0.19496E+01_JPRB, 0.23221E+01_JPRB, 0.21331E+01_JPRB, 0.19932E+01_JPRB, 0.19090E+01_JPRB ,&
&  0.17799E+01_JPRB, 0.16710E+01_JPRB, 0.15886E+01_JPRB, 0.14704E+01_JPRB, 0.13376E+01_JPRB, 0.12061E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.93890E+00_JPRB, 0.88035E+00_JPRB, 0.88026E+00_JPRB, 0.97241E+00_JPRB, 0.11137E+01_JPRB, 0.18969E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.97734E+00_JPRB, 0.10557E+01_JPRB, 0.48632E+00_JPRB, 0.56583E+00_JPRB, 0.58807E+00_JPRB, 0.69743E+00_JPRB ,&
&  0.85554E+00_JPRB, 0.13526E+01_JPRB, 0.20607E+01_JPRB, 0.18482E+01_JPRB, 0.14153E+01_JPRB, 0.91912E+00_JPRB, 0.56306E+00_JPRB ,&
&  0.54354E+00_JPRB, 0.53930E+00_JPRB, 0.54500E+00_JPRB, 0.53791E+00_JPRB, 0.53510E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.47920E+00_JPRB, 0.47066E+00_JPRB, 0.48033E+00_JPRB, 0.49096E+00_JPRB, 0.48748E+00_JPRB, 0.48751E+00_JPRB, 0.48244E+00_JPRB ,&
&  0.47304E+00_JPRB, 0.46642E+00_JPRB, 0.48939E+00_JPRB, 0.53269E+00_JPRB, 0.59355E+00_JPRB, 0.67304E+00_JPRB, 0.75336E+00_JPRB ,&
&  0.81014E+00_JPRB, 0.87780E+00_JPRB, 0.93929E+00_JPRB, 0.99041E+00_JPRB, 0.10026E+01_JPRB, 0.10114E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.10010E+01_JPRB, 0.10316E+01_JPRB, 0.11070E+01_JPRB, 0.11884E+01_JPRB, 0.13268E+01_JPRB, 0.15200E+01_JPRB, 0.10163E+01_JPRB ,&
&  0.15860E+01_JPRB, 0.15953E+01_JPRB, 0.15385E+01_JPRB, 0.14114E+01_JPRB, 0.13420E+01_JPRB, 0.12291E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11645E+01_JPRB, 0.12516E+01_JPRB, 0.12801E+01_JPRB, 0.14341E+01_JPRB, 0.15948E+01_JPRB, 0.17105E+01_JPRB, 0.19976E+01_JPRB ,&
&  0.22572E+01_JPRB, 0.25883E+01_JPRB, 0.29488E+01_JPRB, 0.31933E+01_JPRB, 0.33931E+01_JPRB, 0.34862E+01_JPRB, 0.34878E+01_JPRB ,&
&  0.32690E+01_JPRB, 0.31224E+01_JPRB, 0.32500E+01_JPRB, 0.35240E+01_JPRB, 0.30630E+01_JPRB, 0.28573E+01_JPRB, 0.23801E+01_JPRB ,&
&  0.17459E+01_JPRB, 0.15672E+01_JPRB, 0.17763E+01_JPRB, 0.21172E+01_JPRB, 0.20986E+01_JPRB, 0.19748E+01_JPRB, 0.17892E+01_JPRB ,&
&  0.16500E+01_JPRB, 0.15064E+01_JPRB, 0.14023E+01_JPRB, 0.12996E+01_JPRB, 0.11969E+01_JPRB, 0.10892E+01_JPRB, 0.98007E+00_JPRB ,&
&  0.92465E+00_JPRB, 0.86720E+00_JPRB, 0.80469E+00_JPRB, 0.81363E+00_JPRB, 0.88727E+00_JPRB, 0.11065E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.13428E+01_JPRB, 0.95455E+00_JPRB, 0.80128E+00_JPRB, 0.74686E+00_JPRB, 0.56796E+00_JPRB, 0.58528E+00_JPRB, 0.74848E+00_JPRB ,&
&  0.11623E+01_JPRB, 0.16937E+01_JPRB, 0.28306E+01_JPRB, 0.50560E+01_JPRB, 0.28032E+01_JPRB, 0.21679E+01_JPRB, 0.54523E+00_JPRB ,&
&  0.52412E+00_JPRB, 0.52561E+00_JPRB, 0.52348E+00_JPRB, 0.50772E+00_JPRB, 0.50278E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.46384E+00_JPRB, 0.45616E+00_JPRB, 0.46628E+00_JPRB, 0.46447E+00_JPRB, 0.46785E+00_JPRB, 0.47493E+00_JPRB, 0.47896E+00_JPRB ,&
&  0.48318E+00_JPRB, 0.50013E+00_JPRB, 0.59353E+00_JPRB, 0.72338E+00_JPRB, 0.84817E+00_JPRB, 0.94486E+00_JPRB, 0.99273E+00_JPRB ,&
&  0.10445E+01_JPRB, 0.10790E+01_JPRB, 0.10887E+01_JPRB, 0.10921E+01_JPRB, 0.11268E+01_JPRB, 0.11929E+01_JPRB, 0.12485E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.13587E+01_JPRB, 0.14231E+01_JPRB, 0.15220E+01_JPRB, 0.16197E+01_JPRB, 0.15885E+01_JPRB, 0.10036E+01_JPRB ,&
&  0.13611E+01_JPRB, 0.14448E+01_JPRB, 0.14711E+01_JPRB, 0.13784E+01_JPRB, 0.12831E+01_JPRB, 0.12367E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.14916E+01_JPRB, 0.15866E+01_JPRB, 0.16199E+01_JPRB, 0.17088E+01_JPRB, 0.23783E+01_JPRB, 0.26214E+01_JPRB, 0.31515E+01_JPRB ,&
&  0.35338E+01_JPRB, 0.39409E+01_JPRB, 0.42370E+01_JPRB, 0.45581E+01_JPRB, 0.48266E+01_JPRB, 0.50216E+01_JPRB, 0.52946E+01_JPRB ,&
&  0.53980E+01_JPRB, 0.55285E+01_JPRB, 0.55587E+01_JPRB, 0.54261E+01_JPRB, 0.43916E+01_JPRB, 0.46333E+01_JPRB, 0.30944E+01_JPRB ,&
&  0.21426E+01_JPRB, 0.18122E+01_JPRB, 0.19854E+01_JPRB, 0.20101E+01_JPRB, 0.18526E+01_JPRB, 0.16508E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.13115E+01_JPRB, 0.12573E+01_JPRB, 0.12100E+01_JPRB, 0.11348E+01_JPRB, 0.10860E+01_JPRB, 0.99772E+00_JPRB, 0.93053E+00_JPRB ,&
&  0.90127E+00_JPRB, 0.84941E+00_JPRB, 0.79008E+00_JPRB, 0.81090E+00_JPRB, 0.92050E+00_JPRB, 0.12290E+01_JPRB, 0.17679E+01_JPRB ,&
&  0.19195E+01_JPRB, 0.10359E+01_JPRB, 0.87218E+00_JPRB, 0.75248E+00_JPRB, 0.69193E+00_JPRB, 0.70194E+00_JPRB, 0.77596E+00_JPRB ,&
&  0.99511E+00_JPRB, 0.17544E+01_JPRB, 0.26284E+01_JPRB, 0.19719E+01_JPRB, 0.22328E+01_JPRB, 0.15419E+01_JPRB, 0.64437E+00_JPRB ,&
&  0.51044E+00_JPRB, 0.49589E+00_JPRB, 0.50112E+00_JPRB, 0.49235E+00_JPRB, 0.47961E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.51410E+00_JPRB, 0.50310E+00_JPRB, 0.50176E+00_JPRB, 0.51179E+00_JPRB, 0.51809E+00_JPRB, 0.54327E+00_JPRB, 0.57437E+00_JPRB ,&
&  0.59152E+00_JPRB, 0.66481E+00_JPRB, 0.82953E+00_JPRB, 0.97184E+00_JPRB, 0.10365E+01_JPRB, 0.10844E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.10426E+01_JPRB, 0.10142E+01_JPRB, 0.10314E+01_JPRB, 0.10753E+01_JPRB, 0.11440E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.13346E+01_JPRB, 0.14355E+01_JPRB, 0.15257E+01_JPRB, 0.16110E+01_JPRB, 0.17276E+01_JPRB, 0.17389E+01_JPRB, 0.16926E+01_JPRB ,&
&  0.84742E+00_JPRB, 0.13849E+01_JPRB, 0.14203E+01_JPRB, 0.13570E+01_JPRB, 0.12913E+01_JPRB, 0.13291E+01_JPRB, 0.15735E+01_JPRB ,&
&  0.18210E+01_JPRB, 0.19146E+01_JPRB, 0.22330E+01_JPRB, 0.23169E+01_JPRB, 0.30135E+01_JPRB, 0.34462E+01_JPRB, 0.38639E+01_JPRB ,&
&  0.42617E+01_JPRB, 0.45607E+01_JPRB, 0.48808E+01_JPRB, 0.51943E+01_JPRB, 0.55030E+01_JPRB, 0.57747E+01_JPRB, 0.60819E+01_JPRB ,&
&  0.64447E+01_JPRB, 0.69372E+01_JPRB, 0.74610E+01_JPRB, 0.72020E+01_JPRB, 0.60743E+01_JPRB, 0.56445E+01_JPRB, 0.43728E+01_JPRB ,&
&  0.28666E+01_JPRB, 0.17539E+01_JPRB, 0.17247E+01_JPRB, 0.17899E+01_JPRB, 0.16045E+01_JPRB, 0.14272E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.11183E+01_JPRB, 0.10421E+01_JPRB, 0.10403E+01_JPRB, 0.10511E+01_JPRB, 0.10053E+01_JPRB, 0.95856E+00_JPRB ,&
&  0.94326E+00_JPRB, 0.90651E+00_JPRB, 0.86316E+00_JPRB, 0.84646E+00_JPRB, 0.88818E+00_JPRB, 0.99388E+00_JPRB, 0.10866E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.98263E+00_JPRB, 0.88193E+00_JPRB, 0.86061E+00_JPRB, 0.83395E+00_JPRB, 0.81666E+00_JPRB, 0.79999E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.10068E+01_JPRB, 0.14040E+01_JPRB, 0.17982E+01_JPRB, 0.14323E+01_JPRB, 0.11332E+01_JPRB, 0.65885E+00_JPRB ,&
&  0.53299E+00_JPRB, 0.51517E+00_JPRB, 0.53289E+00_JPRB, 0.53700E+00_JPRB, 0.53705E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.62483E+00_JPRB, 0.61173E+00_JPRB, 0.59265E+00_JPRB, 0.60806E+00_JPRB, 0.64113E+00_JPRB, 0.68458E+00_JPRB, 0.70407E+00_JPRB ,&
&  0.73017E+00_JPRB, 0.83621E+00_JPRB, 0.97661E+00_JPRB, 0.11020E+01_JPRB, 0.11290E+01_JPRB, 0.11034E+01_JPRB, 0.10151E+01_JPRB ,&
&  0.99992E+00_JPRB, 0.98440E+00_JPRB, 0.98046E+00_JPRB, 0.98015E+00_JPRB, 0.10076E+01_JPRB, 0.10777E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.13194E+01_JPRB, 0.14689E+01_JPRB, 0.16367E+01_JPRB, 0.18193E+01_JPRB, 0.19909E+01_JPRB, 0.20767E+01_JPRB, 0.19648E+01_JPRB ,&
&  0.81895E+00_JPRB, 0.98182E+00_JPRB, 0.14166E+01_JPRB, 0.14325E+01_JPRB, 0.14073E+01_JPRB, 0.14636E+01_JPRB, 0.16634E+01_JPRB ,&
&  0.18691E+01_JPRB, 0.19691E+01_JPRB, 0.22816E+01_JPRB, 0.28507E+01_JPRB, 0.31734E+01_JPRB, 0.36540E+01_JPRB, 0.39454E+01_JPRB ,&
&  0.42440E+01_JPRB, 0.45421E+01_JPRB, 0.48659E+01_JPRB, 0.51987E+01_JPRB, 0.55302E+01_JPRB, 0.58410E+01_JPRB, 0.62283E+01_JPRB ,&
&  0.67441E+01_JPRB, 0.73186E+01_JPRB, 0.78010E+01_JPRB, 0.70372E+01_JPRB, 0.74370E+01_JPRB, 0.64776E+01_JPRB, 0.55546E+01_JPRB ,&
&  0.31850E+01_JPRB, 0.17985E+01_JPRB, 0.17641E+01_JPRB, 0.18262E+01_JPRB, 0.15641E+01_JPRB, 0.13446E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.95408E+00_JPRB, 0.98246E+00_JPRB, 0.10240E+01_JPRB, 0.10065E+01_JPRB, 0.10397E+01_JPRB, 0.98063E+00_JPRB ,&
&  0.98106E+00_JPRB, 0.98624E+00_JPRB, 0.96695E+00_JPRB, 0.96905E+00_JPRB, 0.97932E+00_JPRB, 0.99901E+00_JPRB, 0.99489E+00_JPRB ,&
&  0.99803E+00_JPRB, 0.98027E+00_JPRB, 0.96907E+00_JPRB, 0.96478E+00_JPRB, 0.94879E+00_JPRB, 0.92523E+00_JPRB, 0.87461E+00_JPRB ,&
&  0.82135E+00_JPRB, 0.82756E+00_JPRB, 0.92531E+00_JPRB, 0.10552E+01_JPRB, 0.99642E+00_JPRB, 0.86217E+00_JPRB, 0.68971E+00_JPRB ,&
&  0.67581E+00_JPRB, 0.69665E+00_JPRB, 0.69702E+00_JPRB, 0.67761E+00_JPRB, 0.65450E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.74573E+00_JPRB, 0.71832E+00_JPRB, 0.67905E+00_JPRB, 0.66860E+00_JPRB, 0.72029E+00_JPRB, 0.76923E+00_JPRB, 0.77200E+00_JPRB ,&
&  0.78276E+00_JPRB, 0.85133E+00_JPRB, 0.97483E+00_JPRB, 0.10729E+01_JPRB, 0.11068E+01_JPRB, 0.11017E+01_JPRB, 0.10171E+01_JPRB ,&
&  0.98871E+00_JPRB, 0.97269E+00_JPRB, 0.95148E+00_JPRB, 0.94661E+00_JPRB, 0.95559E+00_JPRB, 0.98275E+00_JPRB, 0.10372E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.12611E+01_JPRB, 0.14691E+01_JPRB, 0.17655E+01_JPRB, 0.21927E+01_JPRB, 0.26954E+01_JPRB, 0.31330E+01_JPRB ,&
&  0.23700E+01_JPRB, 0.97317E+00_JPRB, 0.11568E+01_JPRB, 0.15705E+01_JPRB, 0.15071E+01_JPRB, 0.15939E+01_JPRB, 0.16188E+01_JPRB ,&
&  0.17429E+01_JPRB, 0.19772E+01_JPRB, 0.22316E+01_JPRB, 0.27473E+01_JPRB, 0.31239E+01_JPRB, 0.35265E+01_JPRB, 0.37254E+01_JPRB ,&
&  0.39156E+01_JPRB, 0.41439E+01_JPRB, 0.43793E+01_JPRB, 0.46239E+01_JPRB, 0.48954E+01_JPRB, 0.51958E+01_JPRB, 0.56366E+01_JPRB ,&
&  0.61954E+01_JPRB, 0.67488E+01_JPRB, 0.71307E+01_JPRB, 0.61375E+01_JPRB, 0.64678E+01_JPRB, 0.50848E+01_JPRB, 0.37565E+01_JPRB ,&
&  0.28303E+01_JPRB, 0.21847E+01_JPRB, 0.19509E+01_JPRB, 0.19222E+01_JPRB, 0.15989E+01_JPRB, 0.11986E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.98083E+00_JPRB, 0.94742E+00_JPRB, 0.97724E+00_JPRB, 0.93810E+00_JPRB, 0.99942E+00_JPRB, 0.10226E+01_JPRB, 0.10375E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.10411E+01_JPRB, 0.10649E+01_JPRB, 0.11059E+01_JPRB, 0.10944E+01_JPRB, 0.10928E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10742E+01_JPRB, 0.10800E+01_JPRB, 0.10899E+01_JPRB, 0.10865E+01_JPRB, 0.10016E+01_JPRB, 0.94565E+00_JPRB ,&
&  0.91248E+00_JPRB, 0.89584E+00_JPRB, 0.85697E+00_JPRB, 0.89252E+00_JPRB, 0.89040E+00_JPRB, 0.89442E+00_JPRB, 0.92631E+00_JPRB ,&
&  0.93707E+00_JPRB, 0.93760E+00_JPRB, 0.88289E+00_JPRB, 0.82384E+00_JPRB, 0.78734E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.82139E+00_JPRB, 0.76776E+00_JPRB, 0.73356E+00_JPRB, 0.74225E+00_JPRB, 0.79203E+00_JPRB, 0.84290E+00_JPRB, 0.83942E+00_JPRB ,&
&  0.82421E+00_JPRB, 0.84694E+00_JPRB, 0.92103E+00_JPRB, 0.98675E+00_JPRB, 0.99779E+00_JPRB, 0.10062E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.98939E+00_JPRB, 0.96169E+00_JPRB, 0.94959E+00_JPRB, 0.93085E+00_JPRB, 0.93247E+00_JPRB, 0.93116E+00_JPRB, 0.94507E+00_JPRB ,&
&  0.98759E+00_JPRB, 0.10598E+01_JPRB, 0.11962E+01_JPRB, 0.14029E+01_JPRB, 0.17182E+01_JPRB, 0.23251E+01_JPRB, 0.32634E+01_JPRB ,&
&  0.47383E+01_JPRB, 0.31205E+01_JPRB, 0.11035E+01_JPRB, 0.14907E+01_JPRB, 0.16399E+01_JPRB, 0.17307E+01_JPRB, 0.15438E+01_JPRB ,&
&  0.16572E+01_JPRB, 0.18056E+01_JPRB, 0.19461E+01_JPRB, 0.24744E+01_JPRB, 0.28582E+01_JPRB, 0.31445E+01_JPRB, 0.33331E+01_JPRB ,&
&  0.34096E+01_JPRB, 0.35388E+01_JPRB, 0.36581E+01_JPRB, 0.37658E+01_JPRB, 0.38973E+01_JPRB, 0.40987E+01_JPRB, 0.44293E+01_JPRB ,&
&  0.50447E+01_JPRB, 0.58122E+01_JPRB, 0.64564E+01_JPRB, 0.56346E+01_JPRB, 0.56393E+01_JPRB, 0.42897E+01_JPRB, 0.34886E+01_JPRB ,&
&  0.29571E+01_JPRB, 0.28281E+01_JPRB, 0.26091E+01_JPRB, 0.21081E+01_JPRB, 0.16576E+01_JPRB, 0.13793E+01_JPRB, 0.13240E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.99703E+00_JPRB, 0.98981E+00_JPRB, 0.10311E+01_JPRB, 0.10438E+01_JPRB, 0.10545E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.10907E+01_JPRB, 0.11399E+01_JPRB, 0.11717E+01_JPRB, 0.11944E+01_JPRB, 0.12334E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.11914E+01_JPRB, 0.11912E+01_JPRB, 0.11969E+01_JPRB, 0.11360E+01_JPRB, 0.10487E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.10179E+01_JPRB, 0.97482E+00_JPRB, 0.97805E+00_JPRB, 0.98093E+00_JPRB, 0.10365E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.10994E+01_JPRB, 0.10226E+01_JPRB, 0.88896E+00_JPRB, 0.87266E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.89059E+00_JPRB, 0.87058E+00_JPRB, 0.83817E+00_JPRB, 0.81558E+00_JPRB, 0.83896E+00_JPRB, 0.87504E+00_JPRB, 0.87420E+00_JPRB ,&
&  0.84789E+00_JPRB, 0.83151E+00_JPRB, 0.86922E+00_JPRB, 0.91049E+00_JPRB, 0.90928E+00_JPRB, 0.93044E+00_JPRB, 0.94248E+00_JPRB ,&
&  0.97966E+00_JPRB, 0.97641E+00_JPRB, 0.95768E+00_JPRB, 0.93251E+00_JPRB, 0.93037E+00_JPRB, 0.91727E+00_JPRB, 0.91871E+00_JPRB ,&
&  0.93605E+00_JPRB, 0.97091E+00_JPRB, 0.10437E+01_JPRB, 0.11581E+01_JPRB, 0.12987E+01_JPRB, 0.15742E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.27945E+01_JPRB, 0.35970E+01_JPRB, 0.12330E+01_JPRB, 0.16460E+01_JPRB, 0.20321E+01_JPRB, 0.19770E+01_JPRB, 0.17971E+01_JPRB ,&
&  0.17549E+01_JPRB, 0.16989E+01_JPRB, 0.17286E+01_JPRB, 0.19827E+01_JPRB, 0.23263E+01_JPRB, 0.25731E+01_JPRB, 0.25724E+01_JPRB ,&
&  0.26454E+01_JPRB, 0.27960E+01_JPRB, 0.30429E+01_JPRB, 0.31098E+01_JPRB, 0.31133E+01_JPRB, 0.30948E+01_JPRB, 0.32323E+01_JPRB ,&
&  0.36604E+01_JPRB, 0.44133E+01_JPRB, 0.55248E+01_JPRB, 0.51786E+01_JPRB, 0.51735E+01_JPRB, 0.44750E+01_JPRB, 0.39049E+01_JPRB ,&
&  0.34471E+01_JPRB, 0.33634E+01_JPRB, 0.30987E+01_JPRB, 0.24527E+01_JPRB, 0.18659E+01_JPRB, 0.16080E+01_JPRB, 0.15256E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.11567E+01_JPRB, 0.11423E+01_JPRB, 0.11401E+01_JPRB, 0.11721E+01_JPRB, 0.11528E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11243E+01_JPRB, 0.11509E+01_JPRB, 0.11948E+01_JPRB, 0.12386E+01_JPRB, 0.12978E+01_JPRB, 0.13226E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.13046E+01_JPRB, 0.12833E+01_JPRB, 0.12218E+01_JPRB, 0.11410E+01_JPRB, 0.10997E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.11078E+01_JPRB, 0.10702E+01_JPRB, 0.10192E+01_JPRB, 0.10525E+01_JPRB, 0.10606E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.13080E+01_JPRB, 0.12506E+01_JPRB, 0.97087E+00_JPRB, 0.95185E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.10039E+01_JPRB, 0.95974E+00_JPRB, 0.93055E+00_JPRB, 0.90144E+00_JPRB, 0.87736E+00_JPRB, 0.91018E+00_JPRB, 0.90405E+00_JPRB ,&
&  0.88132E+00_JPRB, 0.84727E+00_JPRB, 0.86365E+00_JPRB, 0.91456E+00_JPRB, 0.91986E+00_JPRB, 0.90280E+00_JPRB, 0.89099E+00_JPRB ,&
&  0.89093E+00_JPRB, 0.90718E+00_JPRB, 0.91054E+00_JPRB, 0.92533E+00_JPRB, 0.92907E+00_JPRB, 0.92070E+00_JPRB, 0.91671E+00_JPRB ,&
&  0.93464E+00_JPRB, 0.95320E+00_JPRB, 0.98363E+00_JPRB, 0.10194E+01_JPRB, 0.10722E+01_JPRB, 0.11453E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.17951E+01_JPRB, 0.29223E+01_JPRB, 0.23670E+01_JPRB, 0.28230E+01_JPRB, 0.26771E+01_JPRB, 0.24548E+01_JPRB, 0.21665E+01_JPRB ,&
&  0.22530E+01_JPRB, 0.20714E+01_JPRB, 0.20238E+01_JPRB, 0.17227E+01_JPRB, 0.17364E+01_JPRB, 0.18654E+01_JPRB, 0.18412E+01_JPRB ,&
&  0.17737E+01_JPRB, 0.17580E+01_JPRB, 0.20744E+01_JPRB, 0.22372E+01_JPRB, 0.23112E+01_JPRB, 0.23361E+01_JPRB, 0.23772E+01_JPRB ,&
&  0.26021E+01_JPRB, 0.30133E+01_JPRB, 0.40718E+01_JPRB, 0.48895E+01_JPRB, 0.47683E+01_JPRB, 0.46459E+01_JPRB, 0.45255E+01_JPRB ,&
&  0.45496E+01_JPRB, 0.40338E+01_JPRB, 0.35239E+01_JPRB, 0.27978E+01_JPRB, 0.21353E+01_JPRB, 0.20153E+01_JPRB, 0.16969E+01_JPRB ,&
&  0.14643E+01_JPRB, 0.13116E+01_JPRB, 0.12844E+01_JPRB, 0.12722E+01_JPRB, 0.12821E+01_JPRB, 0.12412E+01_JPRB, 0.11861E+01_JPRB ,&
&  0.11702E+01_JPRB, 0.11770E+01_JPRB, 0.11601E+01_JPRB, 0.11906E+01_JPRB, 0.12185E+01_JPRB, 0.12589E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13458E+01_JPRB, 0.12904E+01_JPRB, 0.12359E+01_JPRB, 0.11728E+01_JPRB, 0.11353E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.11482E+01_JPRB, 0.11415E+01_JPRB, 0.10944E+01_JPRB, 0.10659E+01_JPRB, 0.10523E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.11236E+01_JPRB, 0.11068E+01_JPRB, 0.10510E+01_JPRB, 0.10339E+01_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.10784E+01_JPRB, 0.10541E+01_JPRB, 0.10012E+01_JPRB, 0.98440E+00_JPRB, 0.94348E+00_JPRB, 0.96469E+00_JPRB, 0.96991E+00_JPRB ,&
&  0.93795E+00_JPRB, 0.90557E+00_JPRB, 0.88077E+00_JPRB, 0.90181E+00_JPRB, 0.92586E+00_JPRB, 0.91575E+00_JPRB, 0.87900E+00_JPRB ,&
&  0.86629E+00_JPRB, 0.86228E+00_JPRB, 0.85989E+00_JPRB, 0.86484E+00_JPRB, 0.88318E+00_JPRB, 0.91009E+00_JPRB, 0.91067E+00_JPRB ,&
&  0.90848E+00_JPRB, 0.94487E+00_JPRB, 0.98426E+00_JPRB, 0.99355E+00_JPRB, 0.10015E+01_JPRB, 0.10212E+01_JPRB, 0.11492E+01_JPRB ,&
&  0.13503E+01_JPRB, 0.29674E+01_JPRB, 0.20165E+01_JPRB, 0.36345E+01_JPRB, 0.29035E+01_JPRB, 0.25909E+01_JPRB, 0.21168E+01_JPRB ,&
&  0.19541E+01_JPRB, 0.20641E+01_JPRB, 0.17471E+01_JPRB, 0.15271E+01_JPRB, 0.14038E+01_JPRB, 0.13838E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.11970E+01_JPRB, 0.11935E+01_JPRB, 0.13067E+01_JPRB, 0.14671E+01_JPRB, 0.15818E+01_JPRB, 0.16492E+01_JPRB, 0.17458E+01_JPRB ,&
&  0.19005E+01_JPRB, 0.21007E+01_JPRB, 0.27994E+01_JPRB, 0.37311E+01_JPRB, 0.45883E+01_JPRB, 0.48629E+01_JPRB, 0.52621E+01_JPRB ,&
&  0.54015E+01_JPRB, 0.48947E+01_JPRB, 0.39996E+01_JPRB, 0.31292E+01_JPRB, 0.25234E+01_JPRB, 0.23042E+01_JPRB, 0.20338E+01_JPRB ,&
&  0.17802E+01_JPRB, 0.15486E+01_JPRB, 0.14869E+01_JPRB, 0.14282E+01_JPRB, 0.14225E+01_JPRB, 0.13721E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12029E+01_JPRB, 0.11906E+01_JPRB, 0.12072E+01_JPRB, 0.12257E+01_JPRB, 0.12534E+01_JPRB, 0.12580E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.12935E+01_JPRB, 0.13310E+01_JPRB, 0.12982E+01_JPRB, 0.12839E+01_JPRB, 0.12331E+01_JPRB, 0.11926E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.12031E+01_JPRB, 0.11580E+01_JPRB, 0.11020E+01_JPRB, 0.10713E+01_JPRB, 0.10766E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10703E+01_JPRB, 0.11144E+01_JPRB, 0.10794E+01_JPRB, 0.10768E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.11087E+01_JPRB, 0.11035E+01_JPRB, 0.10369E+01_JPRB, 0.10128E+01_JPRB, 0.97692E+00_JPRB, 0.96268E+00_JPRB, 0.97114E+00_JPRB ,&
&  0.96214E+00_JPRB, 0.93794E+00_JPRB, 0.90719E+00_JPRB, 0.89353E+00_JPRB, 0.90714E+00_JPRB, 0.92153E+00_JPRB, 0.88969E+00_JPRB ,&
&  0.85294E+00_JPRB, 0.83844E+00_JPRB, 0.82890E+00_JPRB, 0.83102E+00_JPRB, 0.84406E+00_JPRB, 0.86162E+00_JPRB, 0.87341E+00_JPRB ,&
&  0.87081E+00_JPRB, 0.91388E+00_JPRB, 0.96723E+00_JPRB, 0.99406E+00_JPRB, 0.99028E+00_JPRB, 0.10092E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.39432E+01_JPRB, 0.22369E+01_JPRB, 0.37738E+01_JPRB, 0.29879E+01_JPRB, 0.25669E+01_JPRB, 0.20527E+01_JPRB ,&
&  0.18842E+01_JPRB, 0.18062E+01_JPRB, 0.16094E+01_JPRB, 0.14476E+01_JPRB, 0.13354E+01_JPRB, 0.12549E+01_JPRB, 0.11875E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.10757E+01_JPRB, 0.10860E+01_JPRB, 0.11724E+01_JPRB, 0.12380E+01_JPRB, 0.12824E+01_JPRB, 0.13411E+01_JPRB ,&
&  0.14400E+01_JPRB, 0.15765E+01_JPRB, 0.19931E+01_JPRB, 0.31550E+01_JPRB, 0.40874E+01_JPRB, 0.49155E+01_JPRB, 0.71438E+01_JPRB ,&
&  0.68153E+01_JPRB, 0.62017E+01_JPRB, 0.45077E+01_JPRB, 0.35100E+01_JPRB, 0.28598E+01_JPRB, 0.24927E+01_JPRB, 0.23192E+01_JPRB ,&
&  0.21397E+01_JPRB, 0.18847E+01_JPRB, 0.17675E+01_JPRB, 0.16845E+01_JPRB, 0.15931E+01_JPRB, 0.14961E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.13230E+01_JPRB, 0.12736E+01_JPRB, 0.12828E+01_JPRB, 0.12927E+01_JPRB, 0.13003E+01_JPRB, 0.12835E+01_JPRB, 0.12716E+01_JPRB ,&
&  0.12812E+01_JPRB, 0.13100E+01_JPRB, 0.12907E+01_JPRB, 0.12952E+01_JPRB, 0.12770E+01_JPRB, 0.12434E+01_JPRB, 0.12225E+01_JPRB ,&
&  0.12693E+01_JPRB, 0.12669E+01_JPRB, 0.11958E+01_JPRB, 0.11497E+01_JPRB, 0.10963E+01_JPRB, 0.11142E+01_JPRB, 0.10587E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.10231E+01_JPRB, 0.10851E+01_JPRB, 0.10919E+01_JPRB, 0.11100E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.11219E+01_JPRB, 0.11058E+01_JPRB, 0.10709E+01_JPRB, 0.10166E+01_JPRB, 0.99354E+00_JPRB, 0.96920E+00_JPRB, 0.95548E+00_JPRB ,&
&  0.95054E+00_JPRB, 0.93679E+00_JPRB, 0.92291E+00_JPRB, 0.90198E+00_JPRB, 0.89349E+00_JPRB, 0.90570E+00_JPRB, 0.90029E+00_JPRB ,&
&  0.88206E+00_JPRB, 0.85796E+00_JPRB, 0.84871E+00_JPRB, 0.83860E+00_JPRB, 0.82776E+00_JPRB, 0.82791E+00_JPRB, 0.83946E+00_JPRB ,&
&  0.84734E+00_JPRB, 0.86775E+00_JPRB, 0.90229E+00_JPRB, 0.93480E+00_JPRB, 0.93970E+00_JPRB, 0.95347E+00_JPRB, 0.10687E+01_JPRB ,&
&  0.14356E+01_JPRB, 0.15040E+01_JPRB, 0.31646E+01_JPRB, 0.36541E+01_JPRB, 0.30352E+01_JPRB, 0.26324E+01_JPRB, 0.22220E+01_JPRB ,&
&  0.19875E+01_JPRB, 0.17024E+01_JPRB, 0.14746E+01_JPRB, 0.13251E+01_JPRB, 0.12469E+01_JPRB, 0.11564E+01_JPRB, 0.11162E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.10631E+01_JPRB, 0.10554E+01_JPRB, 0.10761E+01_JPRB, 0.10873E+01_JPRB, 0.11014E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.12912E+01_JPRB, 0.15039E+01_JPRB, 0.22541E+01_JPRB, 0.37128E+01_JPRB, 0.51245E+01_JPRB, 0.71052E+01_JPRB ,&
&  0.80320E+01_JPRB, 0.72442E+01_JPRB, 0.50276E+01_JPRB, 0.38108E+01_JPRB, 0.31571E+01_JPRB, 0.27256E+01_JPRB, 0.25154E+01_JPRB ,&
&  0.23712E+01_JPRB, 0.21750E+01_JPRB, 0.20183E+01_JPRB, 0.19785E+01_JPRB, 0.18275E+01_JPRB, 0.16602E+01_JPRB, 0.15209E+01_JPRB ,&
&  0.14486E+01_JPRB, 0.13958E+01_JPRB, 0.13412E+01_JPRB, 0.13027E+01_JPRB, 0.13028E+01_JPRB, 0.12908E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.12679E+01_JPRB, 0.12793E+01_JPRB, 0.12677E+01_JPRB, 0.12639E+01_JPRB, 0.12857E+01_JPRB, 0.12885E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.12797E+01_JPRB, 0.12851E+01_JPRB, 0.12388E+01_JPRB, 0.12015E+01_JPRB, 0.11214E+01_JPRB, 0.11553E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.10516E+01_JPRB, 0.10190E+01_JPRB, 0.10272E+01_JPRB, 0.10771E+01_JPRB, 0.11176E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.10969E+01_JPRB, 0.10853E+01_JPRB, 0.10636E+01_JPRB, 0.10317E+01_JPRB, 0.10044E+01_JPRB, 0.98922E+00_JPRB, 0.97379E+00_JPRB ,&
&  0.97220E+00_JPRB, 0.97239E+00_JPRB, 0.96252E+00_JPRB, 0.93576E+00_JPRB, 0.91303E+00_JPRB, 0.90753E+00_JPRB, 0.91125E+00_JPRB ,&
&  0.90430E+00_JPRB, 0.89290E+00_JPRB, 0.89536E+00_JPRB, 0.87761E+00_JPRB, 0.85570E+00_JPRB, 0.84165E+00_JPRB, 0.83754E+00_JPRB ,&
&  0.84355E+00_JPRB, 0.85494E+00_JPRB, 0.88242E+00_JPRB, 0.90964E+00_JPRB, 0.91407E+00_JPRB, 0.91107E+00_JPRB, 0.92835E+00_JPRB ,&
&  0.12794E+01_JPRB, 0.22720E+01_JPRB, 0.35367E+01_JPRB, 0.33277E+01_JPRB, 0.28767E+01_JPRB, 0.24314E+01_JPRB, 0.22026E+01_JPRB ,&
&  0.18708E+01_JPRB, 0.16666E+01_JPRB, 0.14637E+01_JPRB, 0.12986E+01_JPRB, 0.11856E+01_JPRB, 0.11190E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.10261E+01_JPRB, 0.10216E+01_JPRB, 0.10212E+01_JPRB, 0.10205E+01_JPRB, 0.10128E+01_JPRB, 0.10218E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.11459E+01_JPRB, 0.12609E+01_JPRB, 0.16504E+01_JPRB, 0.30605E+01_JPRB, 0.38728E+01_JPRB, 0.60636E+01_JPRB ,&
&  0.82921E+01_JPRB, 0.68408E+01_JPRB, 0.51676E+01_JPRB, 0.40544E+01_JPRB, 0.33766E+01_JPRB, 0.29510E+01_JPRB, 0.26976E+01_JPRB ,&
&  0.24443E+01_JPRB, 0.22876E+01_JPRB, 0.21407E+01_JPRB, 0.19914E+01_JPRB, 0.18842E+01_JPRB, 0.17391E+01_JPRB, 0.16071E+01_JPRB ,&
&  0.15541E+01_JPRB, 0.14862E+01_JPRB, 0.14122E+01_JPRB, 0.13309E+01_JPRB, 0.12995E+01_JPRB, 0.12814E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.12546E+01_JPRB, 0.12419E+01_JPRB, 0.12411E+01_JPRB, 0.12771E+01_JPRB, 0.13093E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.12558E+01_JPRB, 0.12445E+01_JPRB, 0.12624E+01_JPRB, 0.12132E+01_JPRB, 0.11632E+01_JPRB, 0.12255E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.10392E+01_JPRB, 0.10140E+01_JPRB, 0.10233E+01_JPRB, 0.10884E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.10862E+01_JPRB, 0.10801E+01_JPRB, 0.10613E+01_JPRB, 0.10436E+01_JPRB, 0.10351E+01_JPRB, 0.10184E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.99960E+00_JPRB, 0.98746E+00_JPRB, 0.97833E+00_JPRB, 0.96358E+00_JPRB, 0.94382E+00_JPRB, 0.92604E+00_JPRB, 0.91876E+00_JPRB ,&
&  0.91739E+00_JPRB, 0.90747E+00_JPRB, 0.91098E+00_JPRB, 0.91199E+00_JPRB, 0.90856E+00_JPRB, 0.89681E+00_JPRB, 0.88509E+00_JPRB ,&
&  0.87749E+00_JPRB, 0.86298E+00_JPRB, 0.85976E+00_JPRB, 0.88207E+00_JPRB, 0.90196E+00_JPRB, 0.90692E+00_JPRB, 0.90587E+00_JPRB ,&
&  0.10676E+01_JPRB, 0.18214E+01_JPRB, 0.26129E+01_JPRB, 0.27955E+01_JPRB, 0.26189E+01_JPRB, 0.23132E+01_JPRB, 0.20118E+01_JPRB ,&
&  0.17496E+01_JPRB, 0.15436E+01_JPRB, 0.14144E+01_JPRB, 0.13120E+01_JPRB, 0.12177E+01_JPRB, 0.11406E+01_JPRB, 0.10849E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.10088E+01_JPRB, 0.99294E+00_JPRB, 0.98529E+00_JPRB, 0.97930E+00_JPRB, 0.97418E+00_JPRB, 0.98321E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.10605E+01_JPRB, 0.11250E+01_JPRB, 0.12515E+01_JPRB, 0.18769E+01_JPRB, 0.29553E+01_JPRB, 0.43822E+01_JPRB ,&
&  0.48087E+01_JPRB, 0.48956E+01_JPRB, 0.44212E+01_JPRB, 0.37362E+01_JPRB, 0.31775E+01_JPRB, 0.27823E+01_JPRB, 0.25280E+01_JPRB ,&
&  0.22877E+01_JPRB, 0.21165E+01_JPRB, 0.19924E+01_JPRB, 0.18343E+01_JPRB, 0.16786E+01_JPRB, 0.15826E+01_JPRB, 0.15774E+01_JPRB ,&
&  0.15958E+01_JPRB, 0.15529E+01_JPRB, 0.14677E+01_JPRB, 0.13647E+01_JPRB, 0.12663E+01_JPRB, 0.12220E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.11906E+01_JPRB, 0.11947E+01_JPRB, 0.12158E+01_JPRB, 0.12413E+01_JPRB, 0.12572E+01_JPRB, 0.12622E+01_JPRB, 0.12583E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.12575E+01_JPRB, 0.12424E+01_JPRB, 0.12309E+01_JPRB, 0.11909E+01_JPRB, 0.12452E+01_JPRB, 0.11900E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.10617E+01_JPRB, 0.10345E+01_JPRB, 0.10196E+01_JPRB, 0.10662E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.10911E+01_JPRB, 0.10839E+01_JPRB, 0.10663E+01_JPRB, 0.10520E+01_JPRB, 0.10327E+01_JPRB, 0.10145E+01_JPRB, 0.99880E+00_JPRB ,&
&  0.99079E+00_JPRB, 0.97397E+00_JPRB, 0.96509E+00_JPRB, 0.95623E+00_JPRB, 0.95149E+00_JPRB, 0.94066E+00_JPRB, 0.92454E+00_JPRB ,&
&  0.90791E+00_JPRB, 0.89841E+00_JPRB, 0.89510E+00_JPRB, 0.89868E+00_JPRB, 0.90837E+00_JPRB, 0.91491E+00_JPRB, 0.91003E+00_JPRB ,&
&  0.90028E+00_JPRB, 0.88453E+00_JPRB, 0.86591E+00_JPRB, 0.88147E+00_JPRB, 0.89910E+00_JPRB, 0.90523E+00_JPRB, 0.89638E+00_JPRB ,&
&  0.98642E+00_JPRB, 0.13031E+01_JPRB, 0.18016E+01_JPRB, 0.19511E+01_JPRB, 0.19539E+01_JPRB, 0.19677E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.14710E+01_JPRB, 0.13181E+01_JPRB, 0.12359E+01_JPRB, 0.11830E+01_JPRB, 0.11224E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.10149E+01_JPRB, 0.98383E+00_JPRB, 0.96346E+00_JPRB, 0.95522E+00_JPRB, 0.95137E+00_JPRB, 0.95180E+00_JPRB, 0.95848E+00_JPRB ,&
&  0.96954E+00_JPRB, 0.98961E+00_JPRB, 0.10263E+01_JPRB, 0.10845E+01_JPRB, 0.12208E+01_JPRB, 0.16621E+01_JPRB, 0.23592E+01_JPRB ,&
&  0.28382E+01_JPRB, 0.32013E+01_JPRB, 0.34450E+01_JPRB, 0.34132E+01_JPRB, 0.30921E+01_JPRB, 0.27155E+01_JPRB, 0.24342E+01_JPRB ,&
&  0.22008E+01_JPRB, 0.19760E+01_JPRB, 0.17879E+01_JPRB, 0.16290E+01_JPRB, 0.14894E+01_JPRB, 0.14297E+01_JPRB, 0.14800E+01_JPRB ,&
&  0.15110E+01_JPRB, 0.14885E+01_JPRB, 0.14160E+01_JPRB, 0.13296E+01_JPRB, 0.12314E+01_JPRB, 0.11737E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.11525E+01_JPRB, 0.11608E+01_JPRB, 0.11761E+01_JPRB, 0.11972E+01_JPRB, 0.12168E+01_JPRB, 0.12279E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.12390E+01_JPRB, 0.11788E+01_JPRB, 0.11558E+01_JPRB, 0.12368E+01_JPRB, 0.12165E+01_JPRB, 0.11906E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.10762E+01_JPRB, 0.10309E+01_JPRB, 0.99877E+00_JPRB, 0.10487E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.95146E+00_JPRB, 0.97726E+00_JPRB, 0.98662E+00_JPRB, 0.98811E+00_JPRB, 0.97744E+00_JPRB, 0.95988E+00_JPRB, 0.94513E+00_JPRB ,&
&  0.93898E+00_JPRB, 0.92467E+00_JPRB, 0.91831E+00_JPRB, 0.91531E+00_JPRB, 0.91792E+00_JPRB, 0.92421E+00_JPRB, 0.92280E+00_JPRB ,&
&  0.91194E+00_JPRB, 0.89645E+00_JPRB, 0.88603E+00_JPRB, 0.88498E+00_JPRB, 0.89445E+00_JPRB, 0.90510E+00_JPRB, 0.91192E+00_JPRB ,&
&  0.91420E+00_JPRB, 0.90665E+00_JPRB, 0.90200E+00_JPRB, 0.91201E+00_JPRB, 0.92142E+00_JPRB, 0.91452E+00_JPRB, 0.89677E+00_JPRB ,&
&  0.92899E+00_JPRB, 0.97606E+00_JPRB, 0.11651E+01_JPRB, 0.13408E+01_JPRB, 0.14899E+01_JPRB, 0.15453E+01_JPRB, 0.15160E+01_JPRB ,&
&  0.14308E+01_JPRB, 0.13457E+01_JPRB, 0.12457E+01_JPRB, 0.11594E+01_JPRB, 0.10960E+01_JPRB, 0.10570E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.99719E+00_JPRB, 0.96281E+00_JPRB, 0.93355E+00_JPRB, 0.91684E+00_JPRB, 0.91054E+00_JPRB, 0.90501E+00_JPRB, 0.89962E+00_JPRB ,&
&  0.89371E+00_JPRB, 0.89482E+00_JPRB, 0.91569E+00_JPRB, 0.95657E+00_JPRB, 0.10199E+01_JPRB, 0.11834E+01_JPRB, 0.14918E+01_JPRB ,&
&  0.17861E+01_JPRB, 0.20968E+01_JPRB, 0.24029E+01_JPRB, 0.26409E+01_JPRB, 0.26428E+01_JPRB, 0.24976E+01_JPRB, 0.22961E+01_JPRB ,&
&  0.20599E+01_JPRB, 0.17784E+01_JPRB, 0.15254E+01_JPRB, 0.13784E+01_JPRB, 0.13285E+01_JPRB, 0.13391E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.14118E+01_JPRB, 0.13861E+01_JPRB, 0.13079E+01_JPRB, 0.12289E+01_JPRB, 0.11695E+01_JPRB, 0.11208E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.10994E+01_JPRB, 0.11134E+01_JPRB, 0.11287E+01_JPRB, 0.11508E+01_JPRB, 0.11653E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.11530E+01_JPRB, 0.11449E+01_JPRB, 0.11579E+01_JPRB, 0.11398E+01_JPRB, 0.11276E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10416E+01_JPRB, 0.99379E+00_JPRB, 0.96388E+00_JPRB, 0.89694E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.88677E+00_JPRB, 0.91080E+00_JPRB, 0.92158E+00_JPRB, 0.92374E+00_JPRB, 0.92210E+00_JPRB, 0.91000E+00_JPRB, 0.89502E+00_JPRB ,&
&  0.87758E+00_JPRB, 0.87095E+00_JPRB, 0.86901E+00_JPRB, 0.87375E+00_JPRB, 0.87940E+00_JPRB, 0.88763E+00_JPRB, 0.89922E+00_JPRB ,&
&  0.91022E+00_JPRB, 0.91486E+00_JPRB, 0.91830E+00_JPRB, 0.91579E+00_JPRB, 0.91351E+00_JPRB, 0.91531E+00_JPRB, 0.91896E+00_JPRB ,&
&  0.92718E+00_JPRB, 0.92918E+00_JPRB, 0.92445E+00_JPRB, 0.93281E+00_JPRB, 0.94741E+00_JPRB, 0.93790E+00_JPRB, 0.92015E+00_JPRB ,&
&  0.92273E+00_JPRB, 0.83076E+00_JPRB, 0.87453E+00_JPRB, 0.10327E+01_JPRB, 0.11754E+01_JPRB, 0.12980E+01_JPRB, 0.13238E+01_JPRB ,&
&  0.12859E+01_JPRB, 0.12188E+01_JPRB, 0.11531E+01_JPRB, 0.10896E+01_JPRB, 0.10445E+01_JPRB, 0.10007E+01_JPRB, 0.97065E+00_JPRB ,&
&  0.95108E+00_JPRB, 0.92969E+00_JPRB, 0.89974E+00_JPRB, 0.87411E+00_JPRB, 0.85276E+00_JPRB, 0.83210E+00_JPRB, 0.81998E+00_JPRB ,&
&  0.81821E+00_JPRB, 0.82897E+00_JPRB, 0.84716E+00_JPRB, 0.87747E+00_JPRB, 0.93090E+00_JPRB, 0.10104E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.13075E+01_JPRB, 0.14863E+01_JPRB, 0.16596E+01_JPRB, 0.18140E+01_JPRB, 0.19444E+01_JPRB, 0.19643E+01_JPRB, 0.18634E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.15202E+01_JPRB, 0.13952E+01_JPRB, 0.13111E+01_JPRB, 0.12922E+01_JPRB, 0.13105E+01_JPRB, 0.13231E+01_JPRB ,&
&  0.13021E+01_JPRB, 0.12465E+01_JPRB, 0.11734E+01_JPRB, 0.11063E+01_JPRB, 0.10651E+01_JPRB, 0.10371E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.99189E+00_JPRB, 0.98326E+00_JPRB, 0.98690E+00_JPRB, 0.10048E+01_JPRB, 0.10307E+01_JPRB, 0.10582E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10798E+01_JPRB, 0.10914E+01_JPRB, 0.10308E+01_JPRB, 0.10636E+01_JPRB, 0.10670E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.10103E+01_JPRB, 0.96777E+00_JPRB, 0.81145E+00_JPRB, 0.86183E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.85895E+00_JPRB, 0.86537E+00_JPRB, 0.87069E+00_JPRB, 0.86759E+00_JPRB, 0.86424E+00_JPRB, 0.85727E+00_JPRB, 0.84234E+00_JPRB ,&
&  0.82811E+00_JPRB, 0.82312E+00_JPRB, 0.82408E+00_JPRB, 0.82616E+00_JPRB, 0.82966E+00_JPRB, 0.83598E+00_JPRB, 0.84718E+00_JPRB ,&
&  0.86065E+00_JPRB, 0.87449E+00_JPRB, 0.89065E+00_JPRB, 0.90518E+00_JPRB, 0.90748E+00_JPRB, 0.90302E+00_JPRB, 0.90183E+00_JPRB ,&
&  0.90681E+00_JPRB, 0.90494E+00_JPRB, 0.89703E+00_JPRB, 0.89838E+00_JPRB, 0.90795E+00_JPRB, 0.90049E+00_JPRB, 0.88329E+00_JPRB ,&
&  0.87787E+00_JPRB, 0.74904E+00_JPRB, 0.79657E+00_JPRB, 0.87986E+00_JPRB, 0.98686E+00_JPRB, 0.10816E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.10632E+01_JPRB, 0.10198E+01_JPRB, 0.96919E+00_JPRB, 0.94206E+00_JPRB, 0.92394E+00_JPRB, 0.90427E+00_JPRB ,&
&  0.87564E+00_JPRB, 0.84563E+00_JPRB, 0.81233E+00_JPRB, 0.78214E+00_JPRB, 0.75827E+00_JPRB, 0.74575E+00_JPRB, 0.74073E+00_JPRB ,&
&  0.73969E+00_JPRB, 0.75028E+00_JPRB, 0.75770E+00_JPRB, 0.77280E+00_JPRB, 0.80056E+00_JPRB, 0.84675E+00_JPRB, 0.91451E+00_JPRB ,&
&  0.10109E+01_JPRB, 0.11277E+01_JPRB, 0.12424E+01_JPRB, 0.13243E+01_JPRB, 0.13766E+01_JPRB, 0.14159E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.13571E+01_JPRB, 0.12962E+01_JPRB, 0.12802E+01_JPRB, 0.12711E+01_JPRB, 0.12481E+01_JPRB, 0.12215E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.11246E+01_JPRB, 0.10618E+01_JPRB, 0.10052E+01_JPRB, 0.95460E+00_JPRB, 0.92183E+00_JPRB, 0.90273E+00_JPRB, 0.87623E+00_JPRB ,&
&  0.85888E+00_JPRB, 0.85489E+00_JPRB, 0.86329E+00_JPRB, 0.88549E+00_JPRB, 0.91588E+00_JPRB, 0.94435E+00_JPRB, 0.95992E+00_JPRB ,&
&  0.96411E+00_JPRB, 0.96847E+00_JPRB, 0.97136E+00_JPRB, 0.99196E+00_JPRB, 0.99449E+00_JPRB, 0.99348E+00_JPRB, 0.98880E+00_JPRB ,&
&  0.98129E+00_JPRB, 0.97225E+00_JPRB, 0.76447E+00_JPRB, 0.82881E+00_JPRB, 0.84155E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.87178E+00_JPRB, 0.85302E+00_JPRB, 0.84313E+00_JPRB, 0.83736E+00_JPRB, 0.83311E+00_JPRB, 0.82657E+00_JPRB, 0.81343E+00_JPRB ,&
&  0.79981E+00_JPRB, 0.79284E+00_JPRB, 0.78675E+00_JPRB, 0.77900E+00_JPRB, 0.78127E+00_JPRB, 0.78805E+00_JPRB, 0.79969E+00_JPRB ,&
&  0.81214E+00_JPRB, 0.82767E+00_JPRB, 0.84233E+00_JPRB, 0.86042E+00_JPRB, 0.87159E+00_JPRB, 0.87879E+00_JPRB, 0.88195E+00_JPRB ,&
&  0.88380E+00_JPRB, 0.88344E+00_JPRB, 0.87486E+00_JPRB, 0.86374E+00_JPRB, 0.86111E+00_JPRB, 0.86425E+00_JPRB, 0.85991E+00_JPRB ,&
&  0.76194E+00_JPRB, 0.68153E+00_JPRB, 0.74379E+00_JPRB, 0.81623E+00_JPRB, 0.89097E+00_JPRB, 0.95524E+00_JPRB, 0.98485E+00_JPRB ,&
&  0.97067E+00_JPRB, 0.93496E+00_JPRB, 0.90150E+00_JPRB, 0.86253E+00_JPRB, 0.84468E+00_JPRB, 0.83656E+00_JPRB, 0.82599E+00_JPRB ,&
&  0.81527E+00_JPRB, 0.80219E+00_JPRB, 0.78438E+00_JPRB, 0.75660E+00_JPRB, 0.73616E+00_JPRB, 0.72136E+00_JPRB, 0.71732E+00_JPRB ,&
&  0.71833E+00_JPRB, 0.72163E+00_JPRB, 0.72523E+00_JPRB, 0.72908E+00_JPRB, 0.73885E+00_JPRB, 0.75722E+00_JPRB, 0.78331E+00_JPRB ,&
&  0.82556E+00_JPRB, 0.87951E+00_JPRB, 0.94657E+00_JPRB, 0.99441E+00_JPRB, 0.10225E+01_JPRB, 0.10480E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.11057E+01_JPRB, 0.11011E+01_JPRB, 0.10698E+01_JPRB, 0.10529E+01_JPRB, 0.10458E+01_JPRB, 0.10334E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.96450E+00_JPRB, 0.91300E+00_JPRB, 0.86005E+00_JPRB, 0.82648E+00_JPRB, 0.81598E+00_JPRB, 0.81163E+00_JPRB, 0.80319E+00_JPRB ,&
&  0.79540E+00_JPRB, 0.79345E+00_JPRB, 0.79915E+00_JPRB, 0.81150E+00_JPRB, 0.82814E+00_JPRB, 0.84789E+00_JPRB, 0.86318E+00_JPRB ,&
&  0.87675E+00_JPRB, 0.88922E+00_JPRB, 0.89831E+00_JPRB, 0.91326E+00_JPRB, 0.92877E+00_JPRB, 0.93514E+00_JPRB, 0.92924E+00_JPRB ,&
&  0.92578E+00_JPRB, 0.91325E+00_JPRB, 0.90220E+00_JPRB, 0.89645E+00_JPRB, 0.88742E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.83521E+00_JPRB, 0.81656E+00_JPRB, 0.80577E+00_JPRB, 0.80133E+00_JPRB, 0.79247E+00_JPRB, 0.77331E+00_JPRB, 0.75740E+00_JPRB ,&
&  0.74256E+00_JPRB, 0.73417E+00_JPRB, 0.73047E+00_JPRB, 0.72974E+00_JPRB, 0.73598E+00_JPRB, 0.74216E+00_JPRB, 0.74980E+00_JPRB ,&
&  0.76186E+00_JPRB, 0.78082E+00_JPRB, 0.79671E+00_JPRB, 0.81077E+00_JPRB, 0.82291E+00_JPRB, 0.83465E+00_JPRB, 0.83696E+00_JPRB ,&
&  0.83864E+00_JPRB, 0.84537E+00_JPRB, 0.84596E+00_JPRB, 0.83207E+00_JPRB, 0.81796E+00_JPRB, 0.82167E+00_JPRB, 0.81962E+00_JPRB ,&
&  0.81069E+00_JPRB, 0.69682E+00_JPRB, 0.75536E+00_JPRB, 0.78536E+00_JPRB, 0.82014E+00_JPRB, 0.85472E+00_JPRB, 0.87334E+00_JPRB ,&
&  0.86830E+00_JPRB, 0.84682E+00_JPRB, 0.82275E+00_JPRB, 0.79292E+00_JPRB, 0.78393E+00_JPRB, 0.77618E+00_JPRB, 0.76586E+00_JPRB ,&
&  0.75890E+00_JPRB, 0.75220E+00_JPRB, 0.74251E+00_JPRB, 0.72949E+00_JPRB, 0.71313E+00_JPRB, 0.70090E+00_JPRB, 0.68996E+00_JPRB ,&
&  0.68625E+00_JPRB, 0.67792E+00_JPRB, 0.67066E+00_JPRB, 0.67143E+00_JPRB, 0.68401E+00_JPRB, 0.69231E+00_JPRB, 0.69786E+00_JPRB ,&
&  0.70728E+00_JPRB, 0.72283E+00_JPRB, 0.74660E+00_JPRB, 0.77553E+00_JPRB, 0.79920E+00_JPRB, 0.81721E+00_JPRB, 0.84148E+00_JPRB ,&
&  0.87783E+00_JPRB, 0.90305E+00_JPRB, 0.90439E+00_JPRB, 0.88566E+00_JPRB, 0.86995E+00_JPRB, 0.86211E+00_JPRB, 0.84167E+00_JPRB ,&
&  0.80935E+00_JPRB, 0.78319E+00_JPRB, 0.76939E+00_JPRB, 0.76146E+00_JPRB, 0.75355E+00_JPRB, 0.74601E+00_JPRB, 0.74484E+00_JPRB ,&
&  0.74583E+00_JPRB, 0.74515E+00_JPRB, 0.74383E+00_JPRB, 0.74664E+00_JPRB, 0.75291E+00_JPRB, 0.76097E+00_JPRB, 0.77188E+00_JPRB ,&
&  0.78776E+00_JPRB, 0.80207E+00_JPRB, 0.80937E+00_JPRB, 0.81877E+00_JPRB, 0.83079E+00_JPRB, 0.83576E+00_JPRB, 0.84902E+00_JPRB ,&
&  0.84821E+00_JPRB, 0.84322E+00_JPRB, 0.84109E+00_JPRB, 0.84588E+00_JPRB, 0.84735E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.79679E+00_JPRB, 0.77651E+00_JPRB, 0.76431E+00_JPRB, 0.75265E+00_JPRB, 0.74454E+00_JPRB, 0.72665E+00_JPRB, 0.71642E+00_JPRB ,&
&  0.70961E+00_JPRB, 0.70514E+00_JPRB, 0.70390E+00_JPRB, 0.70401E+00_JPRB, 0.71447E+00_JPRB, 0.72145E+00_JPRB, 0.72443E+00_JPRB ,&
&  0.73253E+00_JPRB, 0.75074E+00_JPRB, 0.76769E+00_JPRB, 0.78167E+00_JPRB, 0.79231E+00_JPRB, 0.79960E+00_JPRB, 0.79878E+00_JPRB ,&
&  0.80180E+00_JPRB, 0.80918E+00_JPRB, 0.81252E+00_JPRB, 0.80137E+00_JPRB, 0.78744E+00_JPRB, 0.78010E+00_JPRB, 0.78110E+00_JPRB ,&
&  0.77042E+00_JPRB, 0.73144E+00_JPRB, 0.74812E+00_JPRB, 0.76077E+00_JPRB, 0.77604E+00_JPRB, 0.78699E+00_JPRB, 0.79831E+00_JPRB ,&
&  0.80176E+00_JPRB, 0.78387E+00_JPRB, 0.75958E+00_JPRB, 0.74204E+00_JPRB, 0.74557E+00_JPRB, 0.74258E+00_JPRB, 0.73190E+00_JPRB ,&
&  0.72611E+00_JPRB, 0.72104E+00_JPRB, 0.71260E+00_JPRB, 0.69894E+00_JPRB, 0.68670E+00_JPRB, 0.67793E+00_JPRB, 0.67166E+00_JPRB ,&
&  0.65929E+00_JPRB, 0.64920E+00_JPRB, 0.64379E+00_JPRB, 0.63790E+00_JPRB, 0.63337E+00_JPRB, 0.63869E+00_JPRB, 0.64451E+00_JPRB ,&
&  0.64823E+00_JPRB, 0.65481E+00_JPRB, 0.66822E+00_JPRB, 0.68825E+00_JPRB, 0.70656E+00_JPRB, 0.72584E+00_JPRB, 0.74701E+00_JPRB ,&
&  0.76937E+00_JPRB, 0.79024E+00_JPRB, 0.79922E+00_JPRB, 0.80441E+00_JPRB, 0.79748E+00_JPRB, 0.78498E+00_JPRB, 0.76101E+00_JPRB ,&
&  0.73407E+00_JPRB, 0.72435E+00_JPRB, 0.72115E+00_JPRB, 0.72346E+00_JPRB, 0.71925E+00_JPRB, 0.71549E+00_JPRB, 0.71425E+00_JPRB ,&
&  0.71080E+00_JPRB, 0.70727E+00_JPRB, 0.70191E+00_JPRB, 0.69704E+00_JPRB, 0.70019E+00_JPRB, 0.70579E+00_JPRB, 0.71893E+00_JPRB ,&
&  0.73694E+00_JPRB, 0.75627E+00_JPRB, 0.77567E+00_JPRB, 0.78786E+00_JPRB, 0.79114E+00_JPRB, 0.79296E+00_JPRB, 0.79553E+00_JPRB ,&
&  0.79600E+00_JPRB, 0.79349E+00_JPRB, 0.79880E+00_JPRB, 0.80278E+00_JPRB, 0.80718E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.73908E+00_JPRB, 0.71384E+00_JPRB, 0.69562E+00_JPRB, 0.68897E+00_JPRB, 0.68545E+00_JPRB, 0.67321E+00_JPRB, 0.66768E+00_JPRB ,&
&  0.67338E+00_JPRB, 0.67493E+00_JPRB, 0.68177E+00_JPRB, 0.68121E+00_JPRB, 0.68047E+00_JPRB, 0.68319E+00_JPRB, 0.67564E+00_JPRB ,&
&  0.67582E+00_JPRB, 0.68519E+00_JPRB, 0.69632E+00_JPRB, 0.71134E+00_JPRB, 0.72398E+00_JPRB, 0.73348E+00_JPRB, 0.74133E+00_JPRB ,&
&  0.74434E+00_JPRB, 0.75070E+00_JPRB, 0.75408E+00_JPRB, 0.74941E+00_JPRB, 0.73757E+00_JPRB, 0.72809E+00_JPRB, 0.72610E+00_JPRB ,&
&  0.72427E+00_JPRB, 0.72130E+00_JPRB, 0.67706E+00_JPRB, 0.70190E+00_JPRB, 0.70935E+00_JPRB, 0.71442E+00_JPRB, 0.71868E+00_JPRB ,&
&  0.71527E+00_JPRB, 0.70149E+00_JPRB, 0.69405E+00_JPRB, 0.68010E+00_JPRB, 0.67829E+00_JPRB, 0.68418E+00_JPRB, 0.68153E+00_JPRB ,&
&  0.67558E+00_JPRB, 0.66363E+00_JPRB, 0.64863E+00_JPRB, 0.63735E+00_JPRB, 0.63428E+00_JPRB, 0.63142E+00_JPRB, 0.62251E+00_JPRB ,&
&  0.61676E+00_JPRB, 0.61606E+00_JPRB, 0.61447E+00_JPRB, 0.60830E+00_JPRB, 0.59312E+00_JPRB, 0.57971E+00_JPRB, 0.57845E+00_JPRB ,&
&  0.58455E+00_JPRB, 0.59499E+00_JPRB, 0.61313E+00_JPRB, 0.63526E+00_JPRB, 0.65764E+00_JPRB, 0.67808E+00_JPRB, 0.69590E+00_JPRB ,&
&  0.70291E+00_JPRB, 0.70566E+00_JPRB, 0.70944E+00_JPRB, 0.71630E+00_JPRB, 0.71881E+00_JPRB, 0.71079E+00_JPRB, 0.68855E+00_JPRB ,&
&  0.66449E+00_JPRB, 0.65660E+00_JPRB, 0.65898E+00_JPRB, 0.66669E+00_JPRB, 0.67014E+00_JPRB, 0.66938E+00_JPRB, 0.66336E+00_JPRB ,&
&  0.66011E+00_JPRB, 0.65215E+00_JPRB, 0.64296E+00_JPRB, 0.63821E+00_JPRB, 0.63752E+00_JPRB, 0.64325E+00_JPRB, 0.66069E+00_JPRB ,&
&  0.68216E+00_JPRB, 0.70612E+00_JPRB, 0.72688E+00_JPRB, 0.73381E+00_JPRB, 0.73715E+00_JPRB, 0.73519E+00_JPRB, 0.72526E+00_JPRB ,&
&  0.72900E+00_JPRB, 0.74772E+00_JPRB, 0.75699E+00_JPRB, 0.75805E+00_JPRB, 0.75502E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.68485E+00_JPRB, 0.65618E+00_JPRB, 0.64495E+00_JPRB, 0.64701E+00_JPRB, 0.65041E+00_JPRB, 0.64594E+00_JPRB, 0.65301E+00_JPRB ,&
&  0.66398E+00_JPRB, 0.67236E+00_JPRB, 0.67697E+00_JPRB, 0.67307E+00_JPRB, 0.67047E+00_JPRB, 0.66735E+00_JPRB, 0.65227E+00_JPRB ,&
&  0.64714E+00_JPRB, 0.64538E+00_JPRB, 0.64981E+00_JPRB, 0.65749E+00_JPRB, 0.66603E+00_JPRB, 0.67896E+00_JPRB, 0.68855E+00_JPRB ,&
&  0.69473E+00_JPRB, 0.69850E+00_JPRB, 0.70267E+00_JPRB, 0.70273E+00_JPRB, 0.69883E+00_JPRB, 0.68907E+00_JPRB, 0.68151E+00_JPRB ,&
&  0.67687E+00_JPRB, 0.66584E+00_JPRB, 0.65502E+00_JPRB, 0.65356E+00_JPRB, 0.65652E+00_JPRB, 0.65951E+00_JPRB, 0.65966E+00_JPRB ,&
&  0.64821E+00_JPRB, 0.63261E+00_JPRB, 0.63176E+00_JPRB, 0.63099E+00_JPRB, 0.63551E+00_JPRB, 0.64740E+00_JPRB, 0.65046E+00_JPRB ,&
&  0.64634E+00_JPRB, 0.63119E+00_JPRB, 0.61119E+00_JPRB, 0.60708E+00_JPRB, 0.60877E+00_JPRB, 0.60066E+00_JPRB, 0.59307E+00_JPRB ,&
&  0.59188E+00_JPRB, 0.58918E+00_JPRB, 0.58332E+00_JPRB, 0.57510E+00_JPRB, 0.56196E+00_JPRB, 0.54512E+00_JPRB, 0.53712E+00_JPRB ,&
&  0.53839E+00_JPRB, 0.55732E+00_JPRB, 0.58907E+00_JPRB, 0.62333E+00_JPRB, 0.65436E+00_JPRB, 0.66861E+00_JPRB, 0.66690E+00_JPRB ,&
&  0.65196E+00_JPRB, 0.64623E+00_JPRB, 0.65016E+00_JPRB, 0.65513E+00_JPRB, 0.65754E+00_JPRB, 0.64752E+00_JPRB, 0.62923E+00_JPRB ,&
&  0.61540E+00_JPRB, 0.60572E+00_JPRB, 0.60460E+00_JPRB, 0.60917E+00_JPRB, 0.61366E+00_JPRB, 0.61474E+00_JPRB, 0.60932E+00_JPRB ,&
&  0.60425E+00_JPRB, 0.59834E+00_JPRB, 0.59415E+00_JPRB, 0.59595E+00_JPRB, 0.59999E+00_JPRB, 0.60451E+00_JPRB, 0.61829E+00_JPRB ,&
&  0.63850E+00_JPRB, 0.65828E+00_JPRB, 0.66264E+00_JPRB, 0.65797E+00_JPRB, 0.65684E+00_JPRB, 0.65344E+00_JPRB, 0.65021E+00_JPRB ,&
&  0.65988E+00_JPRB, 0.68206E+00_JPRB, 0.69785E+00_JPRB, 0.70619E+00_JPRB, 0.70583E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.60659E+00_JPRB, 0.58988E+00_JPRB, 0.59077E+00_JPRB, 0.59851E+00_JPRB, 0.60286E+00_JPRB, 0.61067E+00_JPRB, 0.63008E+00_JPRB ,&
&  0.64104E+00_JPRB, 0.64788E+00_JPRB, 0.65049E+00_JPRB, 0.64540E+00_JPRB, 0.64460E+00_JPRB, 0.63739E+00_JPRB, 0.62448E+00_JPRB ,&
&  0.61901E+00_JPRB, 0.61792E+00_JPRB, 0.61961E+00_JPRB, 0.62368E+00_JPRB, 0.62629E+00_JPRB, 0.62969E+00_JPRB, 0.63351E+00_JPRB ,&
&  0.63564E+00_JPRB, 0.63572E+00_JPRB, 0.63478E+00_JPRB, 0.63489E+00_JPRB, 0.63371E+00_JPRB, 0.62569E+00_JPRB, 0.61651E+00_JPRB ,&
&  0.60673E+00_JPRB, 0.59749E+00_JPRB, 0.59449E+00_JPRB, 0.59840E+00_JPRB, 0.60296E+00_JPRB, 0.60554E+00_JPRB, 0.60303E+00_JPRB ,&
&  0.59877E+00_JPRB, 0.59743E+00_JPRB, 0.60105E+00_JPRB, 0.60797E+00_JPRB, 0.61618E+00_JPRB, 0.62553E+00_JPRB, 0.63043E+00_JPRB ,&
&  0.62442E+00_JPRB, 0.60617E+00_JPRB, 0.59494E+00_JPRB, 0.59401E+00_JPRB, 0.58914E+00_JPRB, 0.57295E+00_JPRB, 0.56197E+00_JPRB ,&
&  0.55713E+00_JPRB, 0.54811E+00_JPRB, 0.53861E+00_JPRB, 0.52595E+00_JPRB, 0.51448E+00_JPRB, 0.51007E+00_JPRB, 0.50925E+00_JPRB ,&
&  0.51391E+00_JPRB, 0.53577E+00_JPRB, 0.57332E+00_JPRB, 0.60813E+00_JPRB, 0.62227E+00_JPRB, 0.61462E+00_JPRB, 0.59566E+00_JPRB ,&
&  0.58547E+00_JPRB, 0.58019E+00_JPRB, 0.58057E+00_JPRB, 0.58071E+00_JPRB, 0.57819E+00_JPRB, 0.56977E+00_JPRB, 0.56209E+00_JPRB ,&
&  0.55138E+00_JPRB, 0.53987E+00_JPRB, 0.53542E+00_JPRB, 0.53274E+00_JPRB, 0.52986E+00_JPRB, 0.53022E+00_JPRB, 0.52998E+00_JPRB ,&
&  0.53132E+00_JPRB, 0.53557E+00_JPRB, 0.54684E+00_JPRB, 0.55214E+00_JPRB, 0.55166E+00_JPRB, 0.54125E+00_JPRB, 0.54318E+00_JPRB ,&
&  0.54706E+00_JPRB, 0.54528E+00_JPRB, 0.54060E+00_JPRB, 0.54035E+00_JPRB, 0.54025E+00_JPRB, 0.54218E+00_JPRB, 0.55425E+00_JPRB ,&
&  0.58132E+00_JPRB, 0.60462E+00_JPRB, 0.62358E+00_JPRB, 0.62859E+00_JPRB, 0.62418E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.54894E+00_JPRB, 0.55436E+00_JPRB, 0.56779E+00_JPRB, 0.57881E+00_JPRB, 0.58239E+00_JPRB, 0.59220E+00_JPRB, 0.60334E+00_JPRB ,&
&  0.61188E+00_JPRB, 0.61312E+00_JPRB, 0.60921E+00_JPRB, 0.59927E+00_JPRB, 0.58760E+00_JPRB, 0.57787E+00_JPRB, 0.57416E+00_JPRB ,&
&  0.57641E+00_JPRB, 0.57939E+00_JPRB, 0.58489E+00_JPRB, 0.58804E+00_JPRB, 0.59200E+00_JPRB, 0.59118E+00_JPRB, 0.59208E+00_JPRB ,&
&  0.59433E+00_JPRB, 0.59564E+00_JPRB, 0.59620E+00_JPRB, 0.59233E+00_JPRB, 0.58912E+00_JPRB, 0.58620E+00_JPRB, 0.57696E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.56033E+00_JPRB, 0.56411E+00_JPRB, 0.57515E+00_JPRB, 0.58201E+00_JPRB, 0.57864E+00_JPRB, 0.57680E+00_JPRB ,&
&  0.57702E+00_JPRB, 0.58251E+00_JPRB, 0.59140E+00_JPRB, 0.60392E+00_JPRB, 0.61203E+00_JPRB, 0.62260E+00_JPRB, 0.62307E+00_JPRB ,&
&  0.61652E+00_JPRB, 0.60711E+00_JPRB, 0.60395E+00_JPRB, 0.60217E+00_JPRB, 0.57152E+00_JPRB, 0.53807E+00_JPRB, 0.52178E+00_JPRB ,&
&  0.51299E+00_JPRB, 0.50220E+00_JPRB, 0.48632E+00_JPRB, 0.47539E+00_JPRB, 0.47455E+00_JPRB, 0.47981E+00_JPRB, 0.49169E+00_JPRB ,&
&  0.50158E+00_JPRB, 0.51597E+00_JPRB, 0.53261E+00_JPRB, 0.54346E+00_JPRB, 0.54341E+00_JPRB, 0.53031E+00_JPRB, 0.51761E+00_JPRB ,&
&  0.50959E+00_JPRB, 0.50676E+00_JPRB, 0.50476E+00_JPRB, 0.50270E+00_JPRB, 0.50109E+00_JPRB, 0.50059E+00_JPRB, 0.49970E+00_JPRB ,&
&  0.49558E+00_JPRB, 0.48862E+00_JPRB, 0.48018E+00_JPRB, 0.47564E+00_JPRB, 0.47538E+00_JPRB, 0.47606E+00_JPRB, 0.47824E+00_JPRB ,&
&  0.48724E+00_JPRB, 0.49869E+00_JPRB, 0.50747E+00_JPRB, 0.51048E+00_JPRB, 0.50645E+00_JPRB, 0.49784E+00_JPRB, 0.49290E+00_JPRB ,&
&  0.48952E+00_JPRB, 0.48721E+00_JPRB, 0.48394E+00_JPRB, 0.48261E+00_JPRB, 0.48278E+00_JPRB, 0.49084E+00_JPRB, 0.50746E+00_JPRB ,&
&  0.52750E+00_JPRB, 0.55313E+00_JPRB, 0.56438E+00_JPRB, 0.56254E+00_JPRB, 0.55522E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.51770E+00_JPRB, 0.53085E+00_JPRB, 0.54728E+00_JPRB, 0.55864E+00_JPRB, 0.56338E+00_JPRB, 0.56854E+00_JPRB, 0.57080E+00_JPRB ,&
&  0.56493E+00_JPRB, 0.56229E+00_JPRB, 0.55672E+00_JPRB, 0.54903E+00_JPRB, 0.54239E+00_JPRB, 0.53918E+00_JPRB, 0.53974E+00_JPRB ,&
&  0.54191E+00_JPRB, 0.54376E+00_JPRB, 0.54041E+00_JPRB, 0.53887E+00_JPRB, 0.54320E+00_JPRB, 0.54904E+00_JPRB, 0.55293E+00_JPRB ,&
&  0.55481E+00_JPRB, 0.55668E+00_JPRB, 0.56171E+00_JPRB, 0.56483E+00_JPRB, 0.56482E+00_JPRB, 0.55791E+00_JPRB, 0.55254E+00_JPRB ,&
&  0.54712E+00_JPRB, 0.54505E+00_JPRB, 0.55076E+00_JPRB, 0.54319E+00_JPRB, 0.55749E+00_JPRB, 0.55887E+00_JPRB, 0.56382E+00_JPRB ,&
&  0.57159E+00_JPRB, 0.58531E+00_JPRB, 0.59647E+00_JPRB, 0.60506E+00_JPRB, 0.60742E+00_JPRB, 0.61025E+00_JPRB, 0.60002E+00_JPRB ,&
&  0.58609E+00_JPRB, 0.56742E+00_JPRB, 0.54845E+00_JPRB, 0.52728E+00_JPRB, 0.50615E+00_JPRB, 0.48995E+00_JPRB, 0.48242E+00_JPRB ,&
&  0.47320E+00_JPRB, 0.45910E+00_JPRB, 0.44568E+00_JPRB, 0.44063E+00_JPRB, 0.44541E+00_JPRB, 0.45562E+00_JPRB, 0.47135E+00_JPRB ,&
&  0.48176E+00_JPRB, 0.48507E+00_JPRB, 0.48223E+00_JPRB, 0.47838E+00_JPRB, 0.47418E+00_JPRB, 0.46617E+00_JPRB, 0.46128E+00_JPRB ,&
&  0.45741E+00_JPRB, 0.45172E+00_JPRB, 0.44883E+00_JPRB, 0.45019E+00_JPRB, 0.45304E+00_JPRB, 0.45514E+00_JPRB, 0.45416E+00_JPRB ,&
&  0.45186E+00_JPRB, 0.44714E+00_JPRB, 0.43913E+00_JPRB, 0.43551E+00_JPRB, 0.43669E+00_JPRB, 0.44145E+00_JPRB, 0.44888E+00_JPRB ,&
&  0.45126E+00_JPRB, 0.45765E+00_JPRB, 0.46364E+00_JPRB, 0.46551E+00_JPRB, 0.46779E+00_JPRB, 0.46935E+00_JPRB, 0.46913E+00_JPRB ,&
&  0.46711E+00_JPRB, 0.46197E+00_JPRB, 0.45527E+00_JPRB, 0.45307E+00_JPRB, 0.45498E+00_JPRB, 0.46258E+00_JPRB, 0.47226E+00_JPRB ,&
&  0.48158E+00_JPRB, 0.49611E+00_JPRB, 0.50686E+00_JPRB, 0.50916E+00_JPRB, 0.51133E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.49489E+00_JPRB, 0.50692E+00_JPRB, 0.52089E+00_JPRB, 0.53027E+00_JPRB, 0.52897E+00_JPRB, 0.52858E+00_JPRB, 0.53432E+00_JPRB ,&
&  0.52728E+00_JPRB, 0.51950E+00_JPRB, 0.51349E+00_JPRB, 0.50767E+00_JPRB, 0.50599E+00_JPRB, 0.50510E+00_JPRB, 0.50844E+00_JPRB ,&
&  0.51091E+00_JPRB, 0.51011E+00_JPRB, 0.50885E+00_JPRB, 0.51009E+00_JPRB, 0.51726E+00_JPRB, 0.52605E+00_JPRB, 0.53340E+00_JPRB ,&
&  0.53514E+00_JPRB, 0.53911E+00_JPRB, 0.54530E+00_JPRB, 0.55053E+00_JPRB, 0.55305E+00_JPRB, 0.55436E+00_JPRB, 0.55282E+00_JPRB ,&
&  0.54978E+00_JPRB, 0.54616E+00_JPRB, 0.52895E+00_JPRB, 0.54121E+00_JPRB, 0.54575E+00_JPRB, 0.54625E+00_JPRB, 0.55070E+00_JPRB ,&
&  0.55812E+00_JPRB, 0.56733E+00_JPRB, 0.57200E+00_JPRB, 0.57357E+00_JPRB, 0.56861E+00_JPRB, 0.55883E+00_JPRB, 0.53397E+00_JPRB ,&
&  0.51313E+00_JPRB, 0.49168E+00_JPRB, 0.47821E+00_JPRB, 0.46976E+00_JPRB, 0.46285E+00_JPRB, 0.45710E+00_JPRB, 0.45216E+00_JPRB ,&
&  0.44528E+00_JPRB, 0.43920E+00_JPRB, 0.43541E+00_JPRB, 0.43516E+00_JPRB, 0.43898E+00_JPRB, 0.44968E+00_JPRB, 0.46291E+00_JPRB ,&
&  0.46872E+00_JPRB, 0.46068E+00_JPRB, 0.45245E+00_JPRB, 0.44305E+00_JPRB, 0.42704E+00_JPRB, 0.40086E+00_JPRB, 0.38410E+00_JPRB ,&
&  0.38049E+00_JPRB, 0.39311E+00_JPRB, 0.41367E+00_JPRB, 0.42016E+00_JPRB, 0.42604E+00_JPRB, 0.42338E+00_JPRB, 0.41768E+00_JPRB ,&
&  0.40746E+00_JPRB, 0.38489E+00_JPRB, 0.37363E+00_JPRB, 0.37070E+00_JPRB, 0.36895E+00_JPRB, 0.37014E+00_JPRB, 0.37308E+00_JPRB ,&
&  0.37793E+00_JPRB, 0.38469E+00_JPRB, 0.38942E+00_JPRB, 0.39125E+00_JPRB, 0.39407E+00_JPRB, 0.39438E+00_JPRB, 0.39330E+00_JPRB ,&
&  0.39007E+00_JPRB, 0.39766E+00_JPRB, 0.41702E+00_JPRB, 0.44434E+00_JPRB, 0.45104E+00_JPRB, 0.45933E+00_JPRB, 0.46815E+00_JPRB ,&
&  0.47373E+00_JPRB, 0.47952E+00_JPRB, 0.48042E+00_JPRB, 0.48104E+00_JPRB, 0.48599E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.46828E+00_JPRB, 0.47544E+00_JPRB, 0.48435E+00_JPRB, 0.49582E+00_JPRB, 0.49822E+00_JPRB, 0.49973E+00_JPRB, 0.50857E+00_JPRB ,&
&  0.50836E+00_JPRB, 0.50502E+00_JPRB, 0.50190E+00_JPRB, 0.50255E+00_JPRB, 0.50528E+00_JPRB, 0.50824E+00_JPRB, 0.50847E+00_JPRB ,&
&  0.49980E+00_JPRB, 0.49496E+00_JPRB, 0.49821E+00_JPRB, 0.50175E+00_JPRB, 0.50670E+00_JPRB, 0.51321E+00_JPRB, 0.51904E+00_JPRB ,&
&  0.52240E+00_JPRB, 0.52640E+00_JPRB, 0.52942E+00_JPRB, 0.53602E+00_JPRB, 0.54578E+00_JPRB, 0.55258E+00_JPRB, 0.55823E+00_JPRB ,&
&  0.54161E+00_JPRB, 0.49335E+00_JPRB, 0.46344E+00_JPRB, 0.46918E+00_JPRB, 0.51554E+00_JPRB, 0.52642E+00_JPRB, 0.52722E+00_JPRB ,&
&  0.53002E+00_JPRB, 0.53744E+00_JPRB, 0.53712E+00_JPRB, 0.53153E+00_JPRB, 0.51939E+00_JPRB, 0.49987E+00_JPRB, 0.47626E+00_JPRB ,&
&  0.46070E+00_JPRB, 0.45072E+00_JPRB, 0.44383E+00_JPRB, 0.43670E+00_JPRB, 0.43306E+00_JPRB, 0.42805E+00_JPRB, 0.42015E+00_JPRB ,&
&  0.41691E+00_JPRB, 0.41597E+00_JPRB, 0.41733E+00_JPRB, 0.41842E+00_JPRB, 0.41985E+00_JPRB, 0.42038E+00_JPRB, 0.41512E+00_JPRB ,&
&  0.40326E+00_JPRB, 0.38694E+00_JPRB, 0.37160E+00_JPRB, 0.35487E+00_JPRB, 0.34088E+00_JPRB, 0.33265E+00_JPRB, 0.32580E+00_JPRB ,&
&  0.32345E+00_JPRB, 0.33054E+00_JPRB, 0.34977E+00_JPRB, 0.37100E+00_JPRB, 0.39966E+00_JPRB, 0.37351E+00_JPRB, 0.35003E+00_JPRB ,&
&  0.33931E+00_JPRB, 0.33391E+00_JPRB, 0.33087E+00_JPRB, 0.33107E+00_JPRB, 0.33287E+00_JPRB, 0.33337E+00_JPRB, 0.33355E+00_JPRB ,&
&  0.33528E+00_JPRB, 0.33821E+00_JPRB, 0.34020E+00_JPRB, 0.34202E+00_JPRB, 0.34477E+00_JPRB, 0.34785E+00_JPRB, 0.34945E+00_JPRB ,&
&  0.35046E+00_JPRB, 0.35597E+00_JPRB, 0.36641E+00_JPRB, 0.37859E+00_JPRB, 0.39033E+00_JPRB, 0.40249E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.44035E+00_JPRB, 0.46399E+00_JPRB, 0.46453E+00_JPRB, 0.46589E+00_JPRB, 0.46586E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.45300E+00_JPRB, 0.46436E+00_JPRB, 0.46999E+00_JPRB, 0.47568E+00_JPRB, 0.47824E+00_JPRB, 0.47982E+00_JPRB, 0.48792E+00_JPRB ,&
&  0.49283E+00_JPRB, 0.49685E+00_JPRB, 0.49951E+00_JPRB, 0.50137E+00_JPRB, 0.49806E+00_JPRB, 0.49199E+00_JPRB, 0.49069E+00_JPRB ,&
&  0.48971E+00_JPRB, 0.49026E+00_JPRB, 0.48945E+00_JPRB, 0.48952E+00_JPRB, 0.49007E+00_JPRB, 0.49480E+00_JPRB, 0.50189E+00_JPRB ,&
&  0.50354E+00_JPRB, 0.50771E+00_JPRB, 0.52108E+00_JPRB, 0.52882E+00_JPRB, 0.53367E+00_JPRB, 0.53267E+00_JPRB, 0.52622E+00_JPRB ,&
&  0.49668E+00_JPRB, 0.46806E+00_JPRB, 0.44191E+00_JPRB, 0.44305E+00_JPRB, 0.49402E+00_JPRB, 0.49954E+00_JPRB, 0.49952E+00_JPRB ,&
&  0.49807E+00_JPRB, 0.50207E+00_JPRB, 0.50230E+00_JPRB, 0.48749E+00_JPRB, 0.47674E+00_JPRB, 0.46239E+00_JPRB, 0.44964E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.43060E+00_JPRB, 0.41677E+00_JPRB, 0.38852E+00_JPRB, 0.37188E+00_JPRB, 0.36330E+00_JPRB, 0.34936E+00_JPRB ,&
&  0.33862E+00_JPRB, 0.33564E+00_JPRB, 0.33493E+00_JPRB, 0.33347E+00_JPRB, 0.33738E+00_JPRB, 0.33950E+00_JPRB, 0.33741E+00_JPRB ,&
&  0.33301E+00_JPRB, 0.32942E+00_JPRB, 0.32523E+00_JPRB, 0.31976E+00_JPRB, 0.31700E+00_JPRB, 0.31645E+00_JPRB, 0.31740E+00_JPRB ,&
&  0.32227E+00_JPRB, 0.32911E+00_JPRB, 0.34381E+00_JPRB, 0.35391E+00_JPRB, 0.34815E+00_JPRB, 0.33571E+00_JPRB, 0.32549E+00_JPRB ,&
&  0.31827E+00_JPRB, 0.31538E+00_JPRB, 0.31314E+00_JPRB, 0.31072E+00_JPRB, 0.31138E+00_JPRB, 0.31370E+00_JPRB, 0.31512E+00_JPRB ,&
&  0.31528E+00_JPRB, 0.31744E+00_JPRB, 0.31881E+00_JPRB, 0.32033E+00_JPRB, 0.32164E+00_JPRB, 0.32372E+00_JPRB, 0.32652E+00_JPRB ,&
&  0.32896E+00_JPRB, 0.33459E+00_JPRB, 0.34062E+00_JPRB, 0.34755E+00_JPRB, 0.35198E+00_JPRB, 0.35536E+00_JPRB, 0.36259E+00_JPRB ,&
&  0.36874E+00_JPRB, 0.38442E+00_JPRB, 0.41656E+00_JPRB, 0.45117E+00_JPRB, 0.44251E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.44031E+00_JPRB, 0.44934E+00_JPRB, 0.45717E+00_JPRB, 0.46334E+00_JPRB, 0.46750E+00_JPRB, 0.46858E+00_JPRB, 0.47162E+00_JPRB ,&
&  0.47950E+00_JPRB, 0.48426E+00_JPRB, 0.48820E+00_JPRB, 0.48719E+00_JPRB, 0.48670E+00_JPRB, 0.46884E+00_JPRB, 0.46515E+00_JPRB ,&
&  0.45877E+00_JPRB, 0.45428E+00_JPRB, 0.44783E+00_JPRB, 0.44757E+00_JPRB, 0.44720E+00_JPRB, 0.44945E+00_JPRB, 0.45035E+00_JPRB ,&
&  0.45123E+00_JPRB, 0.45027E+00_JPRB, 0.44746E+00_JPRB, 0.44580E+00_JPRB, 0.44688E+00_JPRB, 0.44789E+00_JPRB, 0.44462E+00_JPRB ,&
&  0.43999E+00_JPRB, 0.43872E+00_JPRB, 0.44169E+00_JPRB, 0.44801E+00_JPRB, 0.48709E+00_JPRB, 0.48103E+00_JPRB, 0.47930E+00_JPRB ,&
&  0.47595E+00_JPRB, 0.47160E+00_JPRB, 0.47077E+00_JPRB, 0.46373E+00_JPRB, 0.45381E+00_JPRB, 0.43046E+00_JPRB, 0.41764E+00_JPRB ,&
&  0.38372E+00_JPRB, 0.36548E+00_JPRB, 0.34954E+00_JPRB, 0.33924E+00_JPRB, 0.32815E+00_JPRB, 0.32168E+00_JPRB, 0.31607E+00_JPRB ,&
&  0.31178E+00_JPRB, 0.30830E+00_JPRB, 0.30519E+00_JPRB, 0.30595E+00_JPRB, 0.30791E+00_JPRB, 0.30880E+00_JPRB, 0.30975E+00_JPRB ,&
&  0.31032E+00_JPRB, 0.31088E+00_JPRB, 0.31154E+00_JPRB, 0.31312E+00_JPRB, 0.31521E+00_JPRB, 0.31799E+00_JPRB, 0.32152E+00_JPRB ,&
&  0.32718E+00_JPRB, 0.33148E+00_JPRB, 0.33288E+00_JPRB, 0.33589E+00_JPRB, 0.33552E+00_JPRB, 0.32888E+00_JPRB, 0.32169E+00_JPRB ,&
&  0.31403E+00_JPRB, 0.31004E+00_JPRB, 0.30605E+00_JPRB, 0.30319E+00_JPRB, 0.30247E+00_JPRB, 0.30234E+00_JPRB, 0.30443E+00_JPRB ,&
&  0.30677E+00_JPRB, 0.30893E+00_JPRB, 0.31048E+00_JPRB, 0.31212E+00_JPRB, 0.31423E+00_JPRB, 0.31574E+00_JPRB, 0.31770E+00_JPRB ,&
&  0.32025E+00_JPRB, 0.32366E+00_JPRB, 0.32878E+00_JPRB, 0.33476E+00_JPRB, 0.34282E+00_JPRB, 0.35426E+00_JPRB, 0.37365E+00_JPRB ,&
&  0.39812E+00_JPRB, 0.45187E+00_JPRB, 0.45509E+00_JPRB, 0.42702E+00_JPRB, 0.43357E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.41660E+00_JPRB, 0.42400E+00_JPRB, 0.43109E+00_JPRB, 0.43880E+00_JPRB, 0.44846E+00_JPRB, 0.45280E+00_JPRB, 0.45479E+00_JPRB ,&
&  0.45046E+00_JPRB, 0.44640E+00_JPRB, 0.43593E+00_JPRB, 0.42711E+00_JPRB, 0.41791E+00_JPRB, 0.41066E+00_JPRB, 0.40461E+00_JPRB ,&
&  0.40057E+00_JPRB, 0.39752E+00_JPRB, 0.39241E+00_JPRB, 0.38885E+00_JPRB, 0.38546E+00_JPRB, 0.38577E+00_JPRB, 0.38646E+00_JPRB ,&
&  0.38632E+00_JPRB, 0.38701E+00_JPRB, 0.38544E+00_JPRB, 0.39063E+00_JPRB, 0.40002E+00_JPRB, 0.41116E+00_JPRB, 0.42574E+00_JPRB ,&
&  0.44606E+00_JPRB, 0.45683E+00_JPRB, 0.47236E+00_JPRB, 0.47227E+00_JPRB, 0.47055E+00_JPRB, 0.46691E+00_JPRB, 0.46378E+00_JPRB ,&
&  0.45899E+00_JPRB, 0.45202E+00_JPRB, 0.44668E+00_JPRB, 0.43975E+00_JPRB, 0.40920E+00_JPRB, 0.38592E+00_JPRB, 0.36789E+00_JPRB ,&
&  0.35294E+00_JPRB, 0.33881E+00_JPRB, 0.32603E+00_JPRB, 0.31972E+00_JPRB, 0.31415E+00_JPRB, 0.31084E+00_JPRB, 0.30735E+00_JPRB ,&
&  0.30360E+00_JPRB, 0.30255E+00_JPRB, 0.30065E+00_JPRB, 0.29990E+00_JPRB, 0.30112E+00_JPRB, 0.30093E+00_JPRB, 0.30171E+00_JPRB ,&
&  0.30252E+00_JPRB, 0.30399E+00_JPRB, 0.30722E+00_JPRB, 0.30961E+00_JPRB, 0.31201E+00_JPRB, 0.31421E+00_JPRB, 0.31521E+00_JPRB ,&
&  0.31731E+00_JPRB, 0.31917E+00_JPRB, 0.31909E+00_JPRB, 0.31824E+00_JPRB, 0.31620E+00_JPRB, 0.31143E+00_JPRB, 0.30677E+00_JPRB ,&
&  0.30174E+00_JPRB, 0.29812E+00_JPRB, 0.29690E+00_JPRB, 0.29575E+00_JPRB, 0.29668E+00_JPRB, 0.29740E+00_JPRB, 0.29815E+00_JPRB ,&
&  0.30011E+00_JPRB, 0.30203E+00_JPRB, 0.30437E+00_JPRB, 0.30783E+00_JPRB, 0.31033E+00_JPRB, 0.31239E+00_JPRB, 0.31493E+00_JPRB ,&
&  0.31469E+00_JPRB, 0.31926E+00_JPRB, 0.32511E+00_JPRB, 0.33333E+00_JPRB, 0.34850E+00_JPRB, 0.35903E+00_JPRB, 0.38249E+00_JPRB ,&
&  0.41533E+00_JPRB, 0.47709E+00_JPRB, 0.46929E+00_JPRB, 0.40447E+00_JPRB, 0.40977E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.40355E+00_JPRB, 0.40722E+00_JPRB, 0.41059E+00_JPRB, 0.41485E+00_JPRB, 0.42001E+00_JPRB, 0.42481E+00_JPRB, 0.42880E+00_JPRB ,&
&  0.43153E+00_JPRB, 0.42947E+00_JPRB, 0.43073E+00_JPRB, 0.42564E+00_JPRB, 0.42060E+00_JPRB, 0.41398E+00_JPRB, 0.40766E+00_JPRB ,&
&  0.40136E+00_JPRB, 0.39221E+00_JPRB, 0.38248E+00_JPRB, 0.37398E+00_JPRB, 0.36670E+00_JPRB, 0.36125E+00_JPRB, 0.35712E+00_JPRB ,&
&  0.35422E+00_JPRB, 0.35507E+00_JPRB, 0.35715E+00_JPRB, 0.36326E+00_JPRB, 0.37352E+00_JPRB, 0.38584E+00_JPRB, 0.40485E+00_JPRB ,&
&  0.43129E+00_JPRB, 0.43342E+00_JPRB, 0.43973E+00_JPRB, 0.43896E+00_JPRB, 0.43489E+00_JPRB, 0.43336E+00_JPRB, 0.43037E+00_JPRB ,&
&  0.43106E+00_JPRB, 0.42171E+00_JPRB, 0.43128E+00_JPRB, 0.41293E+00_JPRB, 0.38421E+00_JPRB, 0.36878E+00_JPRB, 0.35014E+00_JPRB ,&
&  0.33726E+00_JPRB, 0.32625E+00_JPRB, 0.31810E+00_JPRB, 0.31514E+00_JPRB, 0.31091E+00_JPRB, 0.30901E+00_JPRB, 0.30477E+00_JPRB ,&
&  0.30053E+00_JPRB, 0.29874E+00_JPRB, 0.29545E+00_JPRB, 0.29547E+00_JPRB, 0.29574E+00_JPRB, 0.29512E+00_JPRB, 0.29525E+00_JPRB ,&
&  0.29574E+00_JPRB, 0.29712E+00_JPRB, 0.29959E+00_JPRB, 0.30179E+00_JPRB, 0.30312E+00_JPRB, 0.30388E+00_JPRB, 0.30371E+00_JPRB ,&
&  0.30430E+00_JPRB, 0.30398E+00_JPRB, 0.30303E+00_JPRB, 0.30157E+00_JPRB, 0.29924E+00_JPRB, 0.29678E+00_JPRB, 0.29437E+00_JPRB ,&
&  0.29236E+00_JPRB, 0.29149E+00_JPRB, 0.29067E+00_JPRB, 0.29072E+00_JPRB, 0.29118E+00_JPRB, 0.29303E+00_JPRB, 0.29475E+00_JPRB ,&
&  0.29676E+00_JPRB, 0.29970E+00_JPRB, 0.30224E+00_JPRB, 0.30538E+00_JPRB, 0.30814E+00_JPRB, 0.30949E+00_JPRB, 0.31385E+00_JPRB ,&
&  0.31955E+00_JPRB, 0.32756E+00_JPRB, 0.33694E+00_JPRB, 0.34565E+00_JPRB, 0.35657E+00_JPRB, 0.37185E+00_JPRB, 0.39214E+00_JPRB ,&
&  0.41215E+00_JPRB, 0.43184E+00_JPRB, 0.42437E+00_JPRB, 0.39914E+00_JPRB, 0.39946E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.40082E+00_JPRB, 0.39980E+00_JPRB, 0.40098E+00_JPRB, 0.40271E+00_JPRB, 0.40749E+00_JPRB, 0.41367E+00_JPRB, 0.42032E+00_JPRB ,&
&  0.42018E+00_JPRB, 0.42372E+00_JPRB, 0.42252E+00_JPRB, 0.41918E+00_JPRB, 0.41383E+00_JPRB, 0.40812E+00_JPRB, 0.40339E+00_JPRB ,&
&  0.39673E+00_JPRB, 0.38814E+00_JPRB, 0.37979E+00_JPRB, 0.37112E+00_JPRB, 0.36299E+00_JPRB, 0.35455E+00_JPRB, 0.35014E+00_JPRB ,&
&  0.34669E+00_JPRB, 0.34586E+00_JPRB, 0.34789E+00_JPRB, 0.35034E+00_JPRB, 0.35564E+00_JPRB, 0.36292E+00_JPRB, 0.37266E+00_JPRB ,&
&  0.38192E+00_JPRB, 0.39296E+00_JPRB, 0.39881E+00_JPRB, 0.39866E+00_JPRB, 0.39860E+00_JPRB, 0.39664E+00_JPRB, 0.39327E+00_JPRB ,&
&  0.38639E+00_JPRB, 0.37883E+00_JPRB, 0.36984E+00_JPRB, 0.36034E+00_JPRB, 0.35062E+00_JPRB, 0.34190E+00_JPRB, 0.33378E+00_JPRB ,&
&  0.32722E+00_JPRB, 0.32165E+00_JPRB, 0.31643E+00_JPRB, 0.31203E+00_JPRB, 0.30864E+00_JPRB, 0.30617E+00_JPRB, 0.30290E+00_JPRB ,&
&  0.30027E+00_JPRB, 0.29791E+00_JPRB, 0.29507E+00_JPRB, 0.29318E+00_JPRB, 0.29146E+00_JPRB, 0.29035E+00_JPRB, 0.28942E+00_JPRB ,&
&  0.28949E+00_JPRB, 0.28959E+00_JPRB, 0.29055E+00_JPRB, 0.29152E+00_JPRB, 0.29260E+00_JPRB, 0.29272E+00_JPRB, 0.29262E+00_JPRB ,&
&  0.29224E+00_JPRB, 0.29185E+00_JPRB, 0.29018E+00_JPRB, 0.28885E+00_JPRB, 0.28790E+00_JPRB, 0.28686E+00_JPRB, 0.28670E+00_JPRB ,&
&  0.28748E+00_JPRB, 0.28830E+00_JPRB, 0.28889E+00_JPRB, 0.29060E+00_JPRB, 0.29227E+00_JPRB, 0.29479E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.30008E+00_JPRB, 0.30270E+00_JPRB, 0.30560E+00_JPRB, 0.30896E+00_JPRB, 0.31304E+00_JPRB, 0.31720E+00_JPRB, 0.32218E+00_JPRB ,&
&  0.32782E+00_JPRB, 0.33348E+00_JPRB, 0.33984E+00_JPRB, 0.34689E+00_JPRB, 0.35536E+00_JPRB, 0.36083E+00_JPRB, 0.36974E+00_JPRB ,&
&  0.37617E+00_JPRB, 0.37144E+00_JPRB, 0.41182E+00_JPRB, 0.40538E+00_JPRB, 0.40156E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.34468E+00_JPRB, 0.35110E+00_JPRB, 0.35519E+00_JPRB, 0.35909E+00_JPRB, 0.35030E+00_JPRB, 0.38765E+00_JPRB, 0.38577E+00_JPRB ,&
&  0.38701E+00_JPRB, 0.37615E+00_JPRB, 0.37832E+00_JPRB, 0.37794E+00_JPRB, 0.37567E+00_JPRB, 0.37212E+00_JPRB, 0.36875E+00_JPRB ,&
&  0.36575E+00_JPRB, 0.36131E+00_JPRB, 0.35597E+00_JPRB, 0.35130E+00_JPRB, 0.34767E+00_JPRB, 0.34457E+00_JPRB, 0.34189E+00_JPRB ,&
&  0.34124E+00_JPRB, 0.34258E+00_JPRB, 0.34508E+00_JPRB, 0.34305E+00_JPRB, 0.34350E+00_JPRB, 0.34497E+00_JPRB, 0.34696E+00_JPRB ,&
&  0.34873E+00_JPRB, 0.35042E+00_JPRB, 0.35040E+00_JPRB, 0.34855E+00_JPRB, 0.34790E+00_JPRB, 0.34585E+00_JPRB, 0.34387E+00_JPRB ,&
&  0.34130E+00_JPRB, 0.33849E+00_JPRB, 0.33512E+00_JPRB, 0.33244E+00_JPRB, 0.32943E+00_JPRB, 0.32576E+00_JPRB, 0.32225E+00_JPRB ,&
&  0.31925E+00_JPRB, 0.31653E+00_JPRB, 0.31338E+00_JPRB, 0.31047E+00_JPRB, 0.30767E+00_JPRB, 0.30483E+00_JPRB, 0.30222E+00_JPRB ,&
&  0.30021E+00_JPRB, 0.29853E+00_JPRB, 0.29687E+00_JPRB, 0.29513E+00_JPRB, 0.29345E+00_JPRB, 0.29183E+00_JPRB, 0.29057E+00_JPRB ,&
&  0.29053E+00_JPRB, 0.28985E+00_JPRB, 0.28920E+00_JPRB, 0.28894E+00_JPRB, 0.28917E+00_JPRB, 0.28871E+00_JPRB, 0.28856E+00_JPRB ,&
&  0.28856E+00_JPRB, 0.28864E+00_JPRB, 0.28889E+00_JPRB, 0.28921E+00_JPRB, 0.28966E+00_JPRB, 0.29024E+00_JPRB, 0.29120E+00_JPRB ,&
&  0.29233E+00_JPRB, 0.29372E+00_JPRB, 0.29521E+00_JPRB, 0.29673E+00_JPRB, 0.29829E+00_JPRB, 0.29994E+00_JPRB, 0.30181E+00_JPRB ,&
&  0.30367E+00_JPRB, 0.30547E+00_JPRB, 0.30720E+00_JPRB, 0.30891E+00_JPRB, 0.31184E+00_JPRB, 0.31463E+00_JPRB, 0.31760E+00_JPRB ,&
&  0.32082E+00_JPRB, 0.32460E+00_JPRB, 0.32857E+00_JPRB, 0.33293E+00_JPRB, 0.33790E+00_JPRB, 0.33882E+00_JPRB, 0.33948E+00_JPRB ,&
&  0.33965E+00_JPRB, 0.34268E+00_JPRB, 0.34385E+00_JPRB, 0.34278E+00_JPRB, 0.34191E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.31831E+00_JPRB, 0.31829E+00_JPRB, 0.31819E+00_JPRB, 0.31803E+00_JPRB, 0.31616E+00_JPRB, 0.31642E+00_JPRB, 0.31676E+00_JPRB ,&
&  0.31752E+00_JPRB, 0.31932E+00_JPRB, 0.32133E+00_JPRB, 0.32282E+00_JPRB, 0.32384E+00_JPRB, 0.32449E+00_JPRB, 0.32499E+00_JPRB ,&
&  0.32509E+00_JPRB, 0.32544E+00_JPRB, 0.32151E+00_JPRB, 0.32009E+00_JPRB, 0.31956E+00_JPRB, 0.31950E+00_JPRB, 0.31968E+00_JPRB ,&
&  0.31977E+00_JPRB, 0.31970E+00_JPRB, 0.31904E+00_JPRB, 0.31938E+00_JPRB, 0.31975E+00_JPRB, 0.31999E+00_JPRB, 0.32024E+00_JPRB ,&
&  0.32049E+00_JPRB, 0.32072E+00_JPRB, 0.32066E+00_JPRB, 0.32041E+00_JPRB, 0.32031E+00_JPRB, 0.31987E+00_JPRB, 0.31902E+00_JPRB ,&
&  0.31797E+00_JPRB, 0.31663E+00_JPRB, 0.31531E+00_JPRB, 0.31409E+00_JPRB, 0.31304E+00_JPRB, 0.31215E+00_JPRB, 0.31129E+00_JPRB ,&
&  0.31021E+00_JPRB, 0.30878E+00_JPRB, 0.30725E+00_JPRB, 0.30576E+00_JPRB, 0.30433E+00_JPRB, 0.30303E+00_JPRB, 0.30164E+00_JPRB ,&
&  0.30058E+00_JPRB, 0.29970E+00_JPRB, 0.29883E+00_JPRB, 0.29809E+00_JPRB, 0.29735E+00_JPRB, 0.29665E+00_JPRB, 0.29597E+00_JPRB ,&
&  0.29504E+00_JPRB, 0.29456E+00_JPRB, 0.29402E+00_JPRB, 0.29380E+00_JPRB, 0.29394E+00_JPRB, 0.29404E+00_JPRB, 0.29424E+00_JPRB ,&
&  0.29452E+00_JPRB, 0.29485E+00_JPRB, 0.29501E+00_JPRB, 0.29526E+00_JPRB, 0.29563E+00_JPRB, 0.29610E+00_JPRB, 0.29666E+00_JPRB ,&
&  0.29725E+00_JPRB, 0.29791E+00_JPRB, 0.29862E+00_JPRB, 0.29942E+00_JPRB, 0.30025E+00_JPRB, 0.30116E+00_JPRB, 0.30202E+00_JPRB ,&
&  0.30289E+00_JPRB, 0.30370E+00_JPRB, 0.30452E+00_JPRB, 0.30515E+00_JPRB, 0.30609E+00_JPRB, 0.30699E+00_JPRB, 0.30807E+00_JPRB ,&
&  0.30915E+00_JPRB, 0.31031E+00_JPRB, 0.31133E+00_JPRB, 0.31244E+00_JPRB, 0.31369E+00_JPRB, 0.31337E+00_JPRB, 0.31319E+00_JPRB ,&
&  0.31305E+00_JPRB, 0.31328E+00_JPRB, 0.31310E+00_JPRB, 0.31338E+00_JPRB, 0.31428E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB, 0.30154E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB, 0.56084E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.56668E+00_JPRB, 0.56696E+00_JPRB, 0.56711E+00_JPRB, 0.56716E+00_JPRB, 0.56716E+00_JPRB, 0.56737E+00_JPRB, 0.56752E+00_JPRB ,&
&  0.56769E+00_JPRB, 0.56789E+00_JPRB, 0.56843E+00_JPRB, 0.56861E+00_JPRB, 0.56879E+00_JPRB, 0.56878E+00_JPRB, 0.56880E+00_JPRB ,&
&  0.56873E+00_JPRB, 0.56865E+00_JPRB, 0.56846E+00_JPRB, 0.56838E+00_JPRB, 0.56836E+00_JPRB, 0.56849E+00_JPRB, 0.56857E+00_JPRB ,&
&  0.56833E+00_JPRB, 0.56776E+00_JPRB, 0.56733E+00_JPRB, 0.56670E+00_JPRB, 0.56591E+00_JPRB, 0.56469E+00_JPRB, 0.56322E+00_JPRB ,&
&  0.56156E+00_JPRB, 0.56000E+00_JPRB, 0.55870E+00_JPRB, 0.55744E+00_JPRB, 0.55648E+00_JPRB, 0.55571E+00_JPRB, 0.55519E+00_JPRB ,&
&  0.55451E+00_JPRB, 0.55414E+00_JPRB, 0.55379E+00_JPRB, 0.55355E+00_JPRB, 0.55291E+00_JPRB, 0.55242E+00_JPRB, 0.55180E+00_JPRB ,&
&  0.55117E+00_JPRB, 0.55030E+00_JPRB, 0.54968E+00_JPRB, 0.54895E+00_JPRB, 0.54816E+00_JPRB, 0.54679E+00_JPRB, 0.54563E+00_JPRB ,&
&  0.54450E+00_JPRB, 0.54336E+00_JPRB, 0.54207E+00_JPRB, 0.54097E+00_JPRB, 0.54019E+00_JPRB, 0.53945E+00_JPRB, 0.53854E+00_JPRB ,&
&  0.53831E+00_JPRB, 0.53876E+00_JPRB, 0.53916E+00_JPRB, 0.53932E+00_JPRB, 0.53990E+00_JPRB, 0.54060E+00_JPRB, 0.54138E+00_JPRB ,&
&  0.54209E+00_JPRB, 0.54309E+00_JPRB, 0.54453E+00_JPRB, 0.54586E+00_JPRB, 0.54724E+00_JPRB, 0.54867E+00_JPRB, 0.55030E+00_JPRB ,&
&  0.55180E+00_JPRB, 0.55326E+00_JPRB, 0.55458E+00_JPRB, 0.55601E+00_JPRB, 0.55695E+00_JPRB, 0.55784E+00_JPRB, 0.55831E+00_JPRB ,&
&  0.55884E+00_JPRB, 0.55919E+00_JPRB, 0.55959E+00_JPRB, 0.55991E+00_JPRB, 0.56044E+00_JPRB, 0.56087E+00_JPRB, 0.56134E+00_JPRB ,&
&  0.56180E+00_JPRB, 0.56243E+00_JPRB, 0.56290E+00_JPRB, 0.56333E+00_JPRB, 0.56371E+00_JPRB, 0.56433E+00_JPRB, 0.56479E+00_JPRB ,&
&  0.56518E+00_JPRB, 0.56560E+00_JPRB, 0.56603E+00_JPRB, 0.56638E+00_JPRB, 0.56657E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.56130E+00_JPRB, 0.56325E+00_JPRB, 0.56434E+00_JPRB, 0.56538E+00_JPRB, 0.56635E+00_JPRB, 0.56791E+00_JPRB, 0.56913E+00_JPRB ,&
&  0.57034E+00_JPRB, 0.57202E+00_JPRB, 0.57437E+00_JPRB, 0.57582E+00_JPRB, 0.57693E+00_JPRB, 0.57757E+00_JPRB, 0.57809E+00_JPRB ,&
&  0.57807E+00_JPRB, 0.57763E+00_JPRB, 0.57689E+00_JPRB, 0.57627E+00_JPRB, 0.57563E+00_JPRB, 0.57503E+00_JPRB, 0.57417E+00_JPRB ,&
&  0.57255E+00_JPRB, 0.57046E+00_JPRB, 0.56817E+00_JPRB, 0.56477E+00_JPRB, 0.56096E+00_JPRB, 0.55690E+00_JPRB, 0.55256E+00_JPRB ,&
&  0.54803E+00_JPRB, 0.54445E+00_JPRB, 0.54100E+00_JPRB, 0.53773E+00_JPRB, 0.53642E+00_JPRB, 0.53519E+00_JPRB, 0.53392E+00_JPRB ,&
&  0.53263E+00_JPRB, 0.53069E+00_JPRB, 0.53017E+00_JPRB, 0.52998E+00_JPRB, 0.53018E+00_JPRB, 0.53132E+00_JPRB, 0.53228E+00_JPRB ,&
&  0.53288E+00_JPRB, 0.53279E+00_JPRB, 0.53243E+00_JPRB, 0.53248E+00_JPRB, 0.53275E+00_JPRB, 0.53252E+00_JPRB, 0.53150E+00_JPRB ,&
&  0.53075E+00_JPRB, 0.52950E+00_JPRB, 0.52801E+00_JPRB, 0.52628E+00_JPRB, 0.52538E+00_JPRB, 0.52420E+00_JPRB, 0.52290E+00_JPRB ,&
&  0.52212E+00_JPRB, 0.52200E+00_JPRB, 0.52207E+00_JPRB, 0.52225E+00_JPRB, 0.52309E+00_JPRB, 0.52422E+00_JPRB, 0.52533E+00_JPRB ,&
&  0.52653E+00_JPRB, 0.52864E+00_JPRB, 0.53108E+00_JPRB, 0.53321E+00_JPRB, 0.53533E+00_JPRB, 0.53739E+00_JPRB, 0.53983E+00_JPRB ,&
&  0.54167E+00_JPRB, 0.54307E+00_JPRB, 0.54355E+00_JPRB, 0.54479E+00_JPRB, 0.54573E+00_JPRB, 0.54711E+00_JPRB, 0.54789E+00_JPRB ,&
&  0.54933E+00_JPRB, 0.55007E+00_JPRB, 0.55044E+00_JPRB, 0.55089E+00_JPRB, 0.55146E+00_JPRB, 0.55185E+00_JPRB, 0.55191E+00_JPRB ,&
&  0.55189E+00_JPRB, 0.55202E+00_JPRB, 0.55206E+00_JPRB, 0.55204E+00_JPRB, 0.55238E+00_JPRB, 0.55289E+00_JPRB, 0.55353E+00_JPRB ,&
&  0.55431E+00_JPRB, 0.55572E+00_JPRB, 0.55742E+00_JPRB, 0.55865E+00_JPRB, 0.55983E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.55383E+00_JPRB, 0.55537E+00_JPRB, 0.55650E+00_JPRB, 0.55833E+00_JPRB, 0.56176E+00_JPRB, 0.56558E+00_JPRB, 0.56797E+00_JPRB ,&
&  0.57118E+00_JPRB, 0.57496E+00_JPRB, 0.57916E+00_JPRB, 0.58180E+00_JPRB, 0.58384E+00_JPRB, 0.58493E+00_JPRB, 0.58498E+00_JPRB ,&
&  0.58387E+00_JPRB, 0.58221E+00_JPRB, 0.58084E+00_JPRB, 0.57925E+00_JPRB, 0.57750E+00_JPRB, 0.57558E+00_JPRB, 0.57127E+00_JPRB ,&
&  0.56750E+00_JPRB, 0.56507E+00_JPRB, 0.55967E+00_JPRB, 0.55341E+00_JPRB, 0.54827E+00_JPRB, 0.51186E+00_JPRB, 0.49944E+00_JPRB ,&
&  0.49778E+00_JPRB, 0.49470E+00_JPRB, 0.49556E+00_JPRB, 0.49701E+00_JPRB, 0.50225E+00_JPRB, 0.49697E+00_JPRB, 0.49355E+00_JPRB ,&
&  0.47187E+00_JPRB, 0.47188E+00_JPRB, 0.46835E+00_JPRB, 0.46656E+00_JPRB, 0.46891E+00_JPRB, 0.47100E+00_JPRB, 0.47878E+00_JPRB ,&
&  0.49904E+00_JPRB, 0.50505E+00_JPRB, 0.50801E+00_JPRB, 0.51019E+00_JPRB, 0.51252E+00_JPRB, 0.51391E+00_JPRB, 0.51515E+00_JPRB ,&
&  0.51572E+00_JPRB, 0.51565E+00_JPRB, 0.51583E+00_JPRB, 0.51544E+00_JPRB, 0.51494E+00_JPRB, 0.51384E+00_JPRB, 0.51307E+00_JPRB ,&
&  0.51256E+00_JPRB, 0.51312E+00_JPRB, 0.51433E+00_JPRB, 0.51546E+00_JPRB, 0.51720E+00_JPRB, 0.51834E+00_JPRB, 0.51941E+00_JPRB ,&
&  0.52035E+00_JPRB, 0.52127E+00_JPRB, 0.52265E+00_JPRB, 0.52418E+00_JPRB, 0.52613E+00_JPRB, 0.52707E+00_JPRB, 0.52875E+00_JPRB ,&
&  0.53003E+00_JPRB, 0.53185E+00_JPRB, 0.53318E+00_JPRB, 0.53611E+00_JPRB, 0.53805E+00_JPRB, 0.54123E+00_JPRB, 0.54386E+00_JPRB ,&
&  0.54735E+00_JPRB, 0.54949E+00_JPRB, 0.55145E+00_JPRB, 0.55270E+00_JPRB, 0.55408E+00_JPRB, 0.55502E+00_JPRB, 0.55518E+00_JPRB ,&
&  0.55510E+00_JPRB, 0.55413E+00_JPRB, 0.55307E+00_JPRB, 0.55158E+00_JPRB, 0.55048E+00_JPRB, 0.54906E+00_JPRB, 0.54801E+00_JPRB ,&
&  0.54665E+00_JPRB, 0.54686E+00_JPRB, 0.54822E+00_JPRB, 0.55031E+00_JPRB, 0.55212E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.56408E+00_JPRB, 0.56403E+00_JPRB, 0.56346E+00_JPRB, 0.56530E+00_JPRB, 0.56777E+00_JPRB, 0.57265E+00_JPRB, 0.57371E+00_JPRB ,&
&  0.57820E+00_JPRB, 0.58270E+00_JPRB, 0.58767E+00_JPRB, 0.58987E+00_JPRB, 0.59174E+00_JPRB, 0.59167E+00_JPRB, 0.59007E+00_JPRB ,&
&  0.58793E+00_JPRB, 0.58417E+00_JPRB, 0.58124E+00_JPRB, 0.57724E+00_JPRB, 0.57272E+00_JPRB, 0.56981E+00_JPRB, 0.56539E+00_JPRB ,&
&  0.56285E+00_JPRB, 0.55891E+00_JPRB, 0.52474E+00_JPRB, 0.51386E+00_JPRB, 0.49643E+00_JPRB, 0.48613E+00_JPRB, 0.48060E+00_JPRB ,&
&  0.47389E+00_JPRB, 0.46736E+00_JPRB, 0.45858E+00_JPRB, 0.44748E+00_JPRB, 0.43508E+00_JPRB, 0.41762E+00_JPRB, 0.40984E+00_JPRB ,&
&  0.40263E+00_JPRB, 0.40115E+00_JPRB, 0.40272E+00_JPRB, 0.40724E+00_JPRB, 0.41684E+00_JPRB, 0.42703E+00_JPRB, 0.44008E+00_JPRB ,&
&  0.45846E+00_JPRB, 0.47690E+00_JPRB, 0.48668E+00_JPRB, 0.48511E+00_JPRB, 0.49830E+00_JPRB, 0.50483E+00_JPRB, 0.50846E+00_JPRB ,&
&  0.51000E+00_JPRB, 0.50835E+00_JPRB, 0.51280E+00_JPRB, 0.51305E+00_JPRB, 0.51334E+00_JPRB, 0.51396E+00_JPRB, 0.51515E+00_JPRB ,&
&  0.51768E+00_JPRB, 0.52024E+00_JPRB, 0.52207E+00_JPRB, 0.52363E+00_JPRB, 0.52439E+00_JPRB, 0.52439E+00_JPRB, 0.52422E+00_JPRB ,&
&  0.52419E+00_JPRB, 0.52428E+00_JPRB, 0.52462E+00_JPRB, 0.52521E+00_JPRB, 0.52666E+00_JPRB, 0.52750E+00_JPRB, 0.52787E+00_JPRB ,&
&  0.52916E+00_JPRB, 0.53121E+00_JPRB, 0.53348E+00_JPRB, 0.53277E+00_JPRB, 0.53553E+00_JPRB, 0.54327E+00_JPRB, 0.54734E+00_JPRB ,&
&  0.55410E+00_JPRB, 0.55928E+00_JPRB, 0.56524E+00_JPRB, 0.56984E+00_JPRB, 0.57375E+00_JPRB, 0.57533E+00_JPRB, 0.57480E+00_JPRB ,&
&  0.57385E+00_JPRB, 0.57199E+00_JPRB, 0.57066E+00_JPRB, 0.56800E+00_JPRB, 0.56579E+00_JPRB, 0.56266E+00_JPRB, 0.56080E+00_JPRB ,&
&  0.55874E+00_JPRB, 0.55851E+00_JPRB, 0.55698E+00_JPRB, 0.55937E+00_JPRB, 0.56155E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.57952E+00_JPRB, 0.57892E+00_JPRB, 0.57965E+00_JPRB, 0.58409E+00_JPRB, 0.59238E+00_JPRB, 0.60188E+00_JPRB, 0.60096E+00_JPRB ,&
&  0.60111E+00_JPRB, 0.59877E+00_JPRB, 0.60060E+00_JPRB, 0.59998E+00_JPRB, 0.59871E+00_JPRB, 0.59596E+00_JPRB, 0.59362E+00_JPRB ,&
&  0.59125E+00_JPRB, 0.58469E+00_JPRB, 0.57649E+00_JPRB, 0.57325E+00_JPRB, 0.57296E+00_JPRB, 0.57240E+00_JPRB, 0.56796E+00_JPRB ,&
&  0.56187E+00_JPRB, 0.54881E+00_JPRB, 0.52245E+00_JPRB, 0.50433E+00_JPRB, 0.48580E+00_JPRB, 0.47889E+00_JPRB, 0.46710E+00_JPRB ,&
&  0.46959E+00_JPRB, 0.43261E+00_JPRB, 0.43400E+00_JPRB, 0.41840E+00_JPRB, 0.40717E+00_JPRB, 0.39471E+00_JPRB, 0.38117E+00_JPRB ,&
&  0.37720E+00_JPRB, 0.37235E+00_JPRB, 0.37170E+00_JPRB, 0.37872E+00_JPRB, 0.38753E+00_JPRB, 0.40251E+00_JPRB, 0.41895E+00_JPRB ,&
&  0.43132E+00_JPRB, 0.47315E+00_JPRB, 0.48562E+00_JPRB, 0.49050E+00_JPRB, 0.50283E+00_JPRB, 0.51106E+00_JPRB, 0.51225E+00_JPRB ,&
&  0.51283E+00_JPRB, 0.51208E+00_JPRB, 0.51757E+00_JPRB, 0.52102E+00_JPRB, 0.52634E+00_JPRB, 0.52630E+00_JPRB, 0.52659E+00_JPRB ,&
&  0.52928E+00_JPRB, 0.53246E+00_JPRB, 0.53613E+00_JPRB, 0.53929E+00_JPRB, 0.54211E+00_JPRB, 0.54390E+00_JPRB, 0.54472E+00_JPRB ,&
&  0.54484E+00_JPRB, 0.54414E+00_JPRB, 0.54364E+00_JPRB, 0.54436E+00_JPRB, 0.54508E+00_JPRB, 0.54761E+00_JPRB, 0.54885E+00_JPRB ,&
&  0.55014E+00_JPRB, 0.55039E+00_JPRB, 0.54966E+00_JPRB, 0.55046E+00_JPRB, 0.55390E+00_JPRB, 0.55746E+00_JPRB, 0.56369E+00_JPRB ,&
&  0.57285E+00_JPRB, 0.57887E+00_JPRB, 0.58572E+00_JPRB, 0.59165E+00_JPRB, 0.59660E+00_JPRB, 0.59717E+00_JPRB, 0.59522E+00_JPRB ,&
&  0.59358E+00_JPRB, 0.59154E+00_JPRB, 0.58934E+00_JPRB, 0.58563E+00_JPRB, 0.58184E+00_JPRB, 0.57708E+00_JPRB, 0.57354E+00_JPRB ,&
&  0.57074E+00_JPRB, 0.57124E+00_JPRB, 0.56949E+00_JPRB, 0.57219E+00_JPRB, 0.57590E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.59417E+00_JPRB, 0.59568E+00_JPRB, 0.60064E+00_JPRB, 0.60713E+00_JPRB, 0.61425E+00_JPRB, 0.62226E+00_JPRB, 0.62559E+00_JPRB ,&
&  0.62787E+00_JPRB, 0.62548E+00_JPRB, 0.62142E+00_JPRB, 0.60948E+00_JPRB, 0.60152E+00_JPRB, 0.59765E+00_JPRB, 0.59299E+00_JPRB ,&
&  0.58742E+00_JPRB, 0.58368E+00_JPRB, 0.58093E+00_JPRB, 0.57957E+00_JPRB, 0.57596E+00_JPRB, 0.57354E+00_JPRB, 0.56475E+00_JPRB ,&
&  0.54830E+00_JPRB, 0.53321E+00_JPRB, 0.51067E+00_JPRB, 0.49143E+00_JPRB, 0.47833E+00_JPRB, 0.46637E+00_JPRB, 0.46551E+00_JPRB ,&
&  0.46284E+00_JPRB, 0.46091E+00_JPRB, 0.45834E+00_JPRB, 0.45585E+00_JPRB, 0.45737E+00_JPRB, 0.39358E+00_JPRB, 0.37842E+00_JPRB ,&
&  0.36854E+00_JPRB, 0.36088E+00_JPRB, 0.35915E+00_JPRB, 0.35960E+00_JPRB, 0.36569E+00_JPRB, 0.38476E+00_JPRB, 0.41032E+00_JPRB ,&
&  0.42820E+00_JPRB, 0.48483E+00_JPRB, 0.49559E+00_JPRB, 0.50561E+00_JPRB, 0.52215E+00_JPRB, 0.53296E+00_JPRB, 0.53692E+00_JPRB ,&
&  0.54423E+00_JPRB, 0.55669E+00_JPRB, 0.57144E+00_JPRB, 0.58524E+00_JPRB, 0.58580E+00_JPRB, 0.58935E+00_JPRB, 0.58821E+00_JPRB ,&
&  0.58285E+00_JPRB, 0.57482E+00_JPRB, 0.57219E+00_JPRB, 0.57113E+00_JPRB, 0.56993E+00_JPRB, 0.57018E+00_JPRB, 0.56830E+00_JPRB ,&
&  0.57023E+00_JPRB, 0.56940E+00_JPRB, 0.57449E+00_JPRB, 0.57663E+00_JPRB, 0.58113E+00_JPRB, 0.58617E+00_JPRB, 0.58864E+00_JPRB ,&
&  0.58864E+00_JPRB, 0.58896E+00_JPRB, 0.58489E+00_JPRB, 0.57810E+00_JPRB, 0.57589E+00_JPRB, 0.57320E+00_JPRB, 0.57559E+00_JPRB ,&
&  0.58027E+00_JPRB, 0.58970E+00_JPRB, 0.59474E+00_JPRB, 0.60024E+00_JPRB, 0.60504E+00_JPRB, 0.60537E+00_JPRB, 0.60638E+00_JPRB ,&
&  0.60646E+00_JPRB, 0.60709E+00_JPRB, 0.60543E+00_JPRB, 0.60195E+00_JPRB, 0.59626E+00_JPRB, 0.58952E+00_JPRB, 0.58306E+00_JPRB ,&
&  0.58091E+00_JPRB, 0.58247E+00_JPRB, 0.58185E+00_JPRB, 0.58434E+00_JPRB, 0.58918E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.60827E+00_JPRB, 0.61094E+00_JPRB, 0.61847E+00_JPRB, 0.62499E+00_JPRB, 0.63393E+00_JPRB, 0.64173E+00_JPRB, 0.64479E+00_JPRB ,&
&  0.65347E+00_JPRB, 0.64979E+00_JPRB, 0.63630E+00_JPRB, 0.61925E+00_JPRB, 0.60150E+00_JPRB, 0.58973E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.58370E+00_JPRB, 0.57737E+00_JPRB, 0.57308E+00_JPRB, 0.57077E+00_JPRB, 0.56661E+00_JPRB, 0.56284E+00_JPRB, 0.55446E+00_JPRB ,&
&  0.53578E+00_JPRB, 0.52293E+00_JPRB, 0.50920E+00_JPRB, 0.49037E+00_JPRB, 0.47443E+00_JPRB, 0.46592E+00_JPRB, 0.46817E+00_JPRB ,&
&  0.46846E+00_JPRB, 0.46056E+00_JPRB, 0.45988E+00_JPRB, 0.45655E+00_JPRB, 0.44922E+00_JPRB, 0.44540E+00_JPRB, 0.39559E+00_JPRB ,&
&  0.37919E+00_JPRB, 0.36281E+00_JPRB, 0.35698E+00_JPRB, 0.35711E+00_JPRB, 0.36991E+00_JPRB, 0.39077E+00_JPRB, 0.42853E+00_JPRB ,&
&  0.46429E+00_JPRB, 0.49669E+00_JPRB, 0.51898E+00_JPRB, 0.53589E+00_JPRB, 0.55361E+00_JPRB, 0.57024E+00_JPRB, 0.58185E+00_JPRB ,&
&  0.60083E+00_JPRB, 0.62577E+00_JPRB, 0.64895E+00_JPRB, 0.67052E+00_JPRB, 0.68475E+00_JPRB, 0.68912E+00_JPRB, 0.69218E+00_JPRB ,&
&  0.68512E+00_JPRB, 0.62682E+00_JPRB, 0.60328E+00_JPRB, 0.58929E+00_JPRB, 0.58012E+00_JPRB, 0.57963E+00_JPRB, 0.57563E+00_JPRB ,&
&  0.57865E+00_JPRB, 0.58394E+00_JPRB, 0.58616E+00_JPRB, 0.58907E+00_JPRB, 0.59283E+00_JPRB, 0.59450E+00_JPRB, 0.59995E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.62425E+00_JPRB, 0.64996E+00_JPRB, 0.63228E+00_JPRB, 0.61097E+00_JPRB, 0.59031E+00_JPRB, 0.57940E+00_JPRB ,&
&  0.57734E+00_JPRB, 0.58137E+00_JPRB, 0.58920E+00_JPRB, 0.59278E+00_JPRB, 0.59671E+00_JPRB, 0.60075E+00_JPRB, 0.61112E+00_JPRB ,&
&  0.61437E+00_JPRB, 0.61760E+00_JPRB, 0.61825E+00_JPRB, 0.61677E+00_JPRB, 0.61063E+00_JPRB, 0.60360E+00_JPRB, 0.59778E+00_JPRB ,&
&  0.59650E+00_JPRB, 0.59823E+00_JPRB, 0.59927E+00_JPRB, 0.60148E+00_JPRB, 0.60503E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.61382E+00_JPRB, 0.62272E+00_JPRB, 0.63778E+00_JPRB, 0.64974E+00_JPRB, 0.66031E+00_JPRB, 0.65173E+00_JPRB, 0.64155E+00_JPRB ,&
&  0.63939E+00_JPRB, 0.63671E+00_JPRB, 0.62584E+00_JPRB, 0.61270E+00_JPRB, 0.59434E+00_JPRB, 0.58001E+00_JPRB, 0.57516E+00_JPRB ,&
&  0.57450E+00_JPRB, 0.57007E+00_JPRB, 0.56368E+00_JPRB, 0.55375E+00_JPRB, 0.54456E+00_JPRB, 0.54233E+00_JPRB, 0.53680E+00_JPRB ,&
&  0.52507E+00_JPRB, 0.51733E+00_JPRB, 0.50876E+00_JPRB, 0.49554E+00_JPRB, 0.48272E+00_JPRB, 0.47640E+00_JPRB, 0.47399E+00_JPRB ,&
&  0.47225E+00_JPRB, 0.47169E+00_JPRB, 0.47251E+00_JPRB, 0.46701E+00_JPRB, 0.46609E+00_JPRB, 0.46595E+00_JPRB, 0.42938E+00_JPRB ,&
&  0.39484E+00_JPRB, 0.37198E+00_JPRB, 0.35934E+00_JPRB, 0.35947E+00_JPRB, 0.37744E+00_JPRB, 0.41401E+00_JPRB, 0.45636E+00_JPRB ,&
&  0.49624E+00_JPRB, 0.53320E+00_JPRB, 0.56246E+00_JPRB, 0.58052E+00_JPRB, 0.59477E+00_JPRB, 0.61241E+00_JPRB, 0.63265E+00_JPRB ,&
&  0.66019E+00_JPRB, 0.69547E+00_JPRB, 0.72715E+00_JPRB, 0.76014E+00_JPRB, 0.78373E+00_JPRB, 0.75625E+00_JPRB, 0.80716E+00_JPRB ,&
&  0.97980E+00_JPRB, 0.80574E+00_JPRB, 0.67162E+00_JPRB, 0.62185E+00_JPRB, 0.61241E+00_JPRB, 0.61656E+00_JPRB, 0.62241E+00_JPRB ,&
&  0.63076E+00_JPRB, 0.63180E+00_JPRB, 0.62712E+00_JPRB, 0.62320E+00_JPRB, 0.62610E+00_JPRB, 0.63367E+00_JPRB, 0.65286E+00_JPRB ,&
&  0.68455E+00_JPRB, 0.82373E+00_JPRB, 0.10483E+01_JPRB, 0.79523E+00_JPRB, 0.70097E+00_JPRB, 0.63517E+00_JPRB, 0.58314E+00_JPRB ,&
&  0.56236E+00_JPRB, 0.55791E+00_JPRB, 0.56356E+00_JPRB, 0.56820E+00_JPRB, 0.56860E+00_JPRB, 0.57253E+00_JPRB, 0.57955E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.60547E+00_JPRB, 0.61385E+00_JPRB, 0.61929E+00_JPRB, 0.61786E+00_JPRB, 0.61107E+00_JPRB, 0.60201E+00_JPRB ,&
&  0.60366E+00_JPRB, 0.60956E+00_JPRB, 0.61359E+00_JPRB, 0.61516E+00_JPRB, 0.61353E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.61465E+00_JPRB, 0.64307E+00_JPRB, 0.67707E+00_JPRB, 0.68042E+00_JPRB, 0.67747E+00_JPRB, 0.64248E+00_JPRB, 0.60634E+00_JPRB ,&
&  0.58412E+00_JPRB, 0.57391E+00_JPRB, 0.57023E+00_JPRB, 0.56171E+00_JPRB, 0.55378E+00_JPRB, 0.54995E+00_JPRB, 0.54701E+00_JPRB ,&
&  0.54464E+00_JPRB, 0.54188E+00_JPRB, 0.53805E+00_JPRB, 0.52979E+00_JPRB, 0.52513E+00_JPRB, 0.52739E+00_JPRB, 0.52551E+00_JPRB ,&
&  0.51915E+00_JPRB, 0.51620E+00_JPRB, 0.50475E+00_JPRB, 0.48965E+00_JPRB, 0.48230E+00_JPRB, 0.48266E+00_JPRB, 0.48697E+00_JPRB ,&
&  0.48144E+00_JPRB, 0.47586E+00_JPRB, 0.48232E+00_JPRB, 0.49143E+00_JPRB, 0.48420E+00_JPRB, 0.48050E+00_JPRB, 0.44022E+00_JPRB ,&
&  0.40162E+00_JPRB, 0.37845E+00_JPRB, 0.37255E+00_JPRB, 0.39767E+00_JPRB, 0.44478E+00_JPRB, 0.50536E+00_JPRB, 0.54229E+00_JPRB ,&
&  0.56553E+00_JPRB, 0.59281E+00_JPRB, 0.61736E+00_JPRB, 0.64265E+00_JPRB, 0.67527E+00_JPRB, 0.70930E+00_JPRB, 0.74917E+00_JPRB ,&
&  0.77991E+00_JPRB, 0.79760E+00_JPRB, 0.79578E+00_JPRB, 0.72376E+00_JPRB, 0.71087E+00_JPRB, 0.71498E+00_JPRB, 0.78168E+00_JPRB ,&
&  0.84911E+00_JPRB, 0.78028E+00_JPRB, 0.73171E+00_JPRB, 0.72964E+00_JPRB, 0.72276E+00_JPRB, 0.73219E+00_JPRB, 0.73396E+00_JPRB ,&
&  0.72063E+00_JPRB, 0.69750E+00_JPRB, 0.67350E+00_JPRB, 0.65511E+00_JPRB, 0.65764E+00_JPRB, 0.67734E+00_JPRB, 0.69907E+00_JPRB ,&
&  0.74095E+00_JPRB, 0.83366E+00_JPRB, 0.82028E+00_JPRB, 0.63901E+00_JPRB, 0.61185E+00_JPRB, 0.59915E+00_JPRB, 0.58510E+00_JPRB ,&
&  0.56300E+00_JPRB, 0.54994E+00_JPRB, 0.54916E+00_JPRB, 0.55009E+00_JPRB, 0.54449E+00_JPRB, 0.53941E+00_JPRB, 0.53882E+00_JPRB ,&
&  0.54377E+00_JPRB, 0.55445E+00_JPRB, 0.56843E+00_JPRB, 0.59329E+00_JPRB, 0.60636E+00_JPRB, 0.60824E+00_JPRB, 0.59941E+00_JPRB ,&
&  0.59468E+00_JPRB, 0.59746E+00_JPRB, 0.59821E+00_JPRB, 0.59577E+00_JPRB, 0.60023E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.70803E+00_JPRB, 0.71306E+00_JPRB, 0.70545E+00_JPRB, 0.70207E+00_JPRB, 0.67387E+00_JPRB, 0.65665E+00_JPRB, 0.62500E+00_JPRB ,&
&  0.61627E+00_JPRB, 0.59047E+00_JPRB, 0.56090E+00_JPRB, 0.52880E+00_JPRB, 0.51226E+00_JPRB, 0.50592E+00_JPRB, 0.50798E+00_JPRB ,&
&  0.51394E+00_JPRB, 0.52091E+00_JPRB, 0.52382E+00_JPRB, 0.51566E+00_JPRB, 0.51776E+00_JPRB, 0.52060E+00_JPRB, 0.51949E+00_JPRB ,&
&  0.51699E+00_JPRB, 0.51715E+00_JPRB, 0.51051E+00_JPRB, 0.49961E+00_JPRB, 0.49314E+00_JPRB, 0.49081E+00_JPRB, 0.48616E+00_JPRB ,&
&  0.47744E+00_JPRB, 0.47566E+00_JPRB, 0.48756E+00_JPRB, 0.51147E+00_JPRB, 0.49371E+00_JPRB, 0.48744E+00_JPRB, 0.43974E+00_JPRB ,&
&  0.40450E+00_JPRB, 0.39942E+00_JPRB, 0.42777E+00_JPRB, 0.49412E+00_JPRB, 0.52322E+00_JPRB, 0.55685E+00_JPRB, 0.57703E+00_JPRB ,&
&  0.57673E+00_JPRB, 0.59339E+00_JPRB, 0.63716E+00_JPRB, 0.71858E+00_JPRB, 0.75256E+00_JPRB, 0.79714E+00_JPRB, 0.84100E+00_JPRB ,&
&  0.87374E+00_JPRB, 0.89580E+00_JPRB, 0.90392E+00_JPRB, 0.85895E+00_JPRB, 0.87867E+00_JPRB, 0.95675E+00_JPRB, 0.85714E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.81687E+00_JPRB, 0.82013E+00_JPRB, 0.86373E+00_JPRB, 0.82591E+00_JPRB, 0.83823E+00_JPRB, 0.81986E+00_JPRB ,&
&  0.76364E+00_JPRB, 0.73197E+00_JPRB, 0.70941E+00_JPRB, 0.70352E+00_JPRB, 0.70622E+00_JPRB, 0.72422E+00_JPRB, 0.75389E+00_JPRB ,&
&  0.79582E+00_JPRB, 0.81275E+00_JPRB, 0.76031E+00_JPRB, 0.65128E+00_JPRB, 0.61083E+00_JPRB, 0.59279E+00_JPRB, 0.57921E+00_JPRB ,&
&  0.56395E+00_JPRB, 0.55415E+00_JPRB, 0.55450E+00_JPRB, 0.55371E+00_JPRB, 0.54690E+00_JPRB, 0.53327E+00_JPRB, 0.52167E+00_JPRB ,&
&  0.51679E+00_JPRB, 0.51875E+00_JPRB, 0.51981E+00_JPRB, 0.54343E+00_JPRB, 0.57067E+00_JPRB, 0.59267E+00_JPRB, 0.59384E+00_JPRB ,&
&  0.59448E+00_JPRB, 0.60574E+00_JPRB, 0.62070E+00_JPRB, 0.65265E+00_JPRB, 0.67786E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.72953E+00_JPRB, 0.73748E+00_JPRB, 0.73661E+00_JPRB, 0.72395E+00_JPRB, 0.70580E+00_JPRB, 0.67146E+00_JPRB, 0.59849E+00_JPRB ,&
&  0.59659E+00_JPRB, 0.55074E+00_JPRB, 0.52598E+00_JPRB, 0.50938E+00_JPRB, 0.50549E+00_JPRB, 0.49721E+00_JPRB, 0.49352E+00_JPRB ,&
&  0.49404E+00_JPRB, 0.50597E+00_JPRB, 0.51426E+00_JPRB, 0.50982E+00_JPRB, 0.51348E+00_JPRB, 0.51754E+00_JPRB, 0.51661E+00_JPRB ,&
&  0.52022E+00_JPRB, 0.52393E+00_JPRB, 0.52399E+00_JPRB, 0.51770E+00_JPRB, 0.50471E+00_JPRB, 0.49568E+00_JPRB, 0.48594E+00_JPRB ,&
&  0.47954E+00_JPRB, 0.47947E+00_JPRB, 0.48922E+00_JPRB, 0.51118E+00_JPRB, 0.50141E+00_JPRB, 0.48975E+00_JPRB, 0.46968E+00_JPRB ,&
&  0.42483E+00_JPRB, 0.44292E+00_JPRB, 0.50662E+00_JPRB, 0.53474E+00_JPRB, 0.56359E+00_JPRB, 0.59657E+00_JPRB, 0.64278E+00_JPRB ,&
&  0.68249E+00_JPRB, 0.72435E+00_JPRB, 0.77078E+00_JPRB, 0.82459E+00_JPRB, 0.88502E+00_JPRB, 0.93872E+00_JPRB, 0.97357E+00_JPRB ,&
&  0.10109E+01_JPRB, 0.84623E+00_JPRB, 0.83537E+00_JPRB, 0.97123E+00_JPRB, 0.11852E+01_JPRB, 0.10137E+01_JPRB, 0.96267E+00_JPRB ,&
&  0.93358E+00_JPRB, 0.88910E+00_JPRB, 0.87059E+00_JPRB, 0.92048E+00_JPRB, 0.92706E+00_JPRB, 0.96175E+00_JPRB, 0.98693E+00_JPRB ,&
&  0.90594E+00_JPRB, 0.85819E+00_JPRB, 0.81629E+00_JPRB, 0.81243E+00_JPRB, 0.82432E+00_JPRB, 0.84187E+00_JPRB, 0.84885E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.82322E+00_JPRB, 0.77786E+00_JPRB, 0.71395E+00_JPRB, 0.66788E+00_JPRB, 0.64066E+00_JPRB, 0.61954E+00_JPRB ,&
&  0.60069E+00_JPRB, 0.58123E+00_JPRB, 0.56547E+00_JPRB, 0.55650E+00_JPRB, 0.55329E+00_JPRB, 0.55660E+00_JPRB, 0.57331E+00_JPRB ,&
&  0.55771E+00_JPRB, 0.54910E+00_JPRB, 0.53634E+00_JPRB, 0.55069E+00_JPRB, 0.58200E+00_JPRB, 0.61061E+00_JPRB, 0.62466E+00_JPRB ,&
&  0.64783E+00_JPRB, 0.65853E+00_JPRB, 0.65285E+00_JPRB, 0.67249E+00_JPRB, 0.69337E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.74701E+00_JPRB, 0.75984E+00_JPRB, 0.75796E+00_JPRB, 0.75723E+00_JPRB, 0.73588E+00_JPRB, 0.67799E+00_JPRB, 0.62552E+00_JPRB ,&
&  0.68436E+00_JPRB, 0.61290E+00_JPRB, 0.62753E+00_JPRB, 0.54212E+00_JPRB, 0.51621E+00_JPRB, 0.50695E+00_JPRB, 0.48891E+00_JPRB ,&
&  0.46894E+00_JPRB, 0.47763E+00_JPRB, 0.49037E+00_JPRB, 0.49502E+00_JPRB, 0.50076E+00_JPRB, 0.50851E+00_JPRB, 0.50802E+00_JPRB ,&
&  0.51249E+00_JPRB, 0.52190E+00_JPRB, 0.53338E+00_JPRB, 0.52439E+00_JPRB, 0.51096E+00_JPRB, 0.50443E+00_JPRB, 0.49587E+00_JPRB ,&
&  0.48290E+00_JPRB, 0.47731E+00_JPRB, 0.48652E+00_JPRB, 0.50518E+00_JPRB, 0.51382E+00_JPRB, 0.49662E+00_JPRB, 0.48637E+00_JPRB ,&
&  0.48513E+00_JPRB, 0.49128E+00_JPRB, 0.54030E+00_JPRB, 0.57012E+00_JPRB, 0.60901E+00_JPRB, 0.65777E+00_JPRB, 0.70578E+00_JPRB ,&
&  0.76061E+00_JPRB, 0.81827E+00_JPRB, 0.88089E+00_JPRB, 0.95403E+00_JPRB, 0.10214E+01_JPRB, 0.10873E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.93295E+00_JPRB, 0.11328E+01_JPRB, 0.12294E+01_JPRB, 0.14147E+01_JPRB, 0.13899E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.13403E+01_JPRB, 0.11878E+01_JPRB, 0.10916E+01_JPRB, 0.10153E+01_JPRB, 0.10060E+01_JPRB, 0.10665E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.11998E+01_JPRB, 0.11656E+01_JPRB, 0.10963E+01_JPRB, 0.10809E+01_JPRB, 0.10790E+01_JPRB, 0.10734E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.99431E+00_JPRB, 0.93487E+00_JPRB, 0.87420E+00_JPRB, 0.79679E+00_JPRB, 0.75687E+00_JPRB, 0.72192E+00_JPRB, 0.68322E+00_JPRB ,&
&  0.66004E+00_JPRB, 0.61934E+00_JPRB, 0.57817E+00_JPRB, 0.55336E+00_JPRB, 0.53491E+00_JPRB, 0.53020E+00_JPRB, 0.54367E+00_JPRB ,&
&  0.55089E+00_JPRB, 0.55038E+00_JPRB, 0.58974E+00_JPRB, 0.62115E+00_JPRB, 0.63954E+00_JPRB, 0.65838E+00_JPRB, 0.70623E+00_JPRB ,&
&  0.67407E+00_JPRB, 0.68797E+00_JPRB, 0.72899E+00_JPRB, 0.73499E+00_JPRB, 0.73936E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.77958E+00_JPRB, 0.79995E+00_JPRB, 0.78945E+00_JPRB, 0.78829E+00_JPRB, 0.74692E+00_JPRB, 0.72579E+00_JPRB, 0.65901E+00_JPRB ,&
&  0.67892E+00_JPRB, 0.65524E+00_JPRB, 0.61926E+00_JPRB, 0.60784E+00_JPRB, 0.61648E+00_JPRB, 0.54848E+00_JPRB, 0.49260E+00_JPRB ,&
&  0.46209E+00_JPRB, 0.45476E+00_JPRB, 0.47126E+00_JPRB, 0.48258E+00_JPRB, 0.48636E+00_JPRB, 0.49220E+00_JPRB, 0.48742E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.51451E+00_JPRB, 0.52583E+00_JPRB, 0.52706E+00_JPRB, 0.52078E+00_JPRB, 0.51905E+00_JPRB, 0.51959E+00_JPRB ,&
&  0.50589E+00_JPRB, 0.50055E+00_JPRB, 0.48970E+00_JPRB, 0.48783E+00_JPRB, 0.53110E+00_JPRB, 0.52377E+00_JPRB, 0.51468E+00_JPRB ,&
&  0.52125E+00_JPRB, 0.54146E+00_JPRB, 0.57198E+00_JPRB, 0.61327E+00_JPRB, 0.66260E+00_JPRB, 0.71326E+00_JPRB, 0.76775E+00_JPRB ,&
&  0.81800E+00_JPRB, 0.88244E+00_JPRB, 0.93011E+00_JPRB, 0.10204E+01_JPRB, 0.10849E+01_JPRB, 0.10288E+01_JPRB, 0.12017E+01_JPRB ,&
&  0.13013E+01_JPRB, 0.14411E+01_JPRB, 0.15800E+01_JPRB, 0.14770E+01_JPRB, 0.16743E+01_JPRB, 0.16122E+01_JPRB, 0.14590E+01_JPRB ,&
&  0.14800E+01_JPRB, 0.13761E+01_JPRB, 0.13499E+01_JPRB, 0.13394E+01_JPRB, 0.14076E+01_JPRB, 0.13648E+01_JPRB, 0.13452E+01_JPRB ,&
&  0.14209E+01_JPRB, 0.16421E+01_JPRB, 0.15684E+01_JPRB, 0.15997E+01_JPRB, 0.15592E+01_JPRB, 0.15052E+01_JPRB, 0.13864E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.11731E+01_JPRB, 0.10655E+01_JPRB, 0.97896E+00_JPRB, 0.88000E+00_JPRB, 0.80588E+00_JPRB, 0.74800E+00_JPRB ,&
&  0.68695E+00_JPRB, 0.61185E+00_JPRB, 0.57970E+00_JPRB, 0.55752E+00_JPRB, 0.54518E+00_JPRB, 0.52063E+00_JPRB, 0.52117E+00_JPRB ,&
&  0.54411E+00_JPRB, 0.58964E+00_JPRB, 0.66007E+00_JPRB, 0.69348E+00_JPRB, 0.74065E+00_JPRB, 0.79100E+00_JPRB, 0.73933E+00_JPRB ,&
&  0.69625E+00_JPRB, 0.76495E+00_JPRB, 0.77693E+00_JPRB, 0.78082E+00_JPRB, 0.77668E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.86197E+00_JPRB, 0.87325E+00_JPRB, 0.85543E+00_JPRB, 0.82980E+00_JPRB, 0.77061E+00_JPRB, 0.73809E+00_JPRB, 0.69187E+00_JPRB ,&
&  0.69200E+00_JPRB, 0.66807E+00_JPRB, 0.63430E+00_JPRB, 0.62167E+00_JPRB, 0.62294E+00_JPRB, 0.60781E+00_JPRB, 0.54197E+00_JPRB ,&
&  0.49192E+00_JPRB, 0.47496E+00_JPRB, 0.46138E+00_JPRB, 0.46903E+00_JPRB, 0.48187E+00_JPRB, 0.48364E+00_JPRB, 0.49375E+00_JPRB ,&
&  0.51416E+00_JPRB, 0.52154E+00_JPRB, 0.52244E+00_JPRB, 0.52184E+00_JPRB, 0.52210E+00_JPRB, 0.52958E+00_JPRB, 0.53201E+00_JPRB ,&
&  0.51601E+00_JPRB, 0.50243E+00_JPRB, 0.48973E+00_JPRB, 0.49159E+00_JPRB, 0.53501E+00_JPRB, 0.55190E+00_JPRB, 0.55117E+00_JPRB ,&
&  0.55964E+00_JPRB, 0.57470E+00_JPRB, 0.59551E+00_JPRB, 0.63569E+00_JPRB, 0.69039E+00_JPRB, 0.75378E+00_JPRB, 0.82011E+00_JPRB ,&
&  0.88533E+00_JPRB, 0.95714E+00_JPRB, 0.10150E+01_JPRB, 0.10849E+01_JPRB, 0.10624E+01_JPRB, 0.11180E+01_JPRB, 0.12815E+01_JPRB ,&
&  0.13931E+01_JPRB, 0.15324E+01_JPRB, 0.16233E+01_JPRB, 0.18255E+01_JPRB, 0.19866E+01_JPRB, 0.18802E+01_JPRB, 0.17485E+01_JPRB ,&
&  0.17058E+01_JPRB, 0.16498E+01_JPRB, 0.20807E+01_JPRB, 0.18967E+01_JPRB, 0.17383E+01_JPRB, 0.17075E+01_JPRB, 0.16537E+01_JPRB ,&
&  0.17162E+01_JPRB, 0.18493E+01_JPRB, 0.19243E+01_JPRB, 0.19866E+01_JPRB, 0.19964E+01_JPRB, 0.19985E+01_JPRB, 0.18775E+01_JPRB ,&
&  0.17927E+01_JPRB, 0.16287E+01_JPRB, 0.12879E+01_JPRB, 0.10363E+01_JPRB, 0.84951E+00_JPRB, 0.83541E+00_JPRB, 0.77431E+00_JPRB ,&
&  0.69552E+00_JPRB, 0.61435E+00_JPRB, 0.59822E+00_JPRB, 0.57712E+00_JPRB, 0.57203E+00_JPRB, 0.55852E+00_JPRB, 0.53934E+00_JPRB ,&
&  0.55625E+00_JPRB, 0.62329E+00_JPRB, 0.66526E+00_JPRB, 0.74034E+00_JPRB, 0.83197E+00_JPRB, 0.92470E+00_JPRB, 0.79447E+00_JPRB ,&
&  0.80882E+00_JPRB, 0.88719E+00_JPRB, 0.90397E+00_JPRB, 0.90208E+00_JPRB, 0.87837E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.10225E+01_JPRB, 0.99733E+00_JPRB, 0.95582E+00_JPRB, 0.88924E+00_JPRB, 0.80843E+00_JPRB, 0.75127E+00_JPRB, 0.70000E+00_JPRB ,&
&  0.69158E+00_JPRB, 0.66866E+00_JPRB, 0.64006E+00_JPRB, 0.63156E+00_JPRB, 0.62878E+00_JPRB, 0.62317E+00_JPRB, 0.61720E+00_JPRB ,&
&  0.53980E+00_JPRB, 0.51753E+00_JPRB, 0.48590E+00_JPRB, 0.48732E+00_JPRB, 0.52789E+00_JPRB, 0.50260E+00_JPRB, 0.50955E+00_JPRB ,&
&  0.51730E+00_JPRB, 0.52989E+00_JPRB, 0.52446E+00_JPRB, 0.52462E+00_JPRB, 0.53499E+00_JPRB, 0.54749E+00_JPRB, 0.54959E+00_JPRB ,&
&  0.53270E+00_JPRB, 0.51377E+00_JPRB, 0.50888E+00_JPRB, 0.51476E+00_JPRB, 0.55709E+00_JPRB, 0.59061E+00_JPRB, 0.60991E+00_JPRB ,&
&  0.62608E+00_JPRB, 0.63885E+00_JPRB, 0.65029E+00_JPRB, 0.67978E+00_JPRB, 0.72730E+00_JPRB, 0.78541E+00_JPRB, 0.86957E+00_JPRB ,&
&  0.95639E+00_JPRB, 0.10367E+01_JPRB, 0.11214E+01_JPRB, 0.12011E+01_JPRB, 0.12015E+01_JPRB, 0.13220E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.15705E+01_JPRB, 0.15234E+01_JPRB, 0.15342E+01_JPRB, 0.19702E+01_JPRB, 0.23055E+01_JPRB, 0.23082E+01_JPRB, 0.21564E+01_JPRB ,&
&  0.20043E+01_JPRB, 0.18920E+01_JPRB, 0.19986E+01_JPRB, 0.19745E+01_JPRB, 0.18321E+01_JPRB, 0.18999E+01_JPRB, 0.19073E+01_JPRB ,&
&  0.19624E+01_JPRB, 0.20902E+01_JPRB, 0.22483E+01_JPRB, 0.23612E+01_JPRB, 0.23013E+01_JPRB, 0.23373E+01_JPRB, 0.22609E+01_JPRB ,&
&  0.21619E+01_JPRB, 0.18261E+01_JPRB, 0.13086E+01_JPRB, 0.10380E+01_JPRB, 0.77512E+00_JPRB, 0.73790E+00_JPRB, 0.83919E+00_JPRB ,&
&  0.81050E+00_JPRB, 0.76161E+00_JPRB, 0.70343E+00_JPRB, 0.66083E+00_JPRB, 0.63541E+00_JPRB, 0.65082E+00_JPRB, 0.64346E+00_JPRB ,&
&  0.63279E+00_JPRB, 0.70564E+00_JPRB, 0.83420E+00_JPRB, 0.98764E+00_JPRB, 0.10861E+01_JPRB, 0.11323E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.11114E+01_JPRB, 0.10977E+01_JPRB, 0.10955E+01_JPRB, 0.11003E+01_JPRB, 0.10727E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.11890E+01_JPRB, 0.11388E+01_JPRB, 0.10558E+01_JPRB, 0.94669E+00_JPRB, 0.83082E+00_JPRB, 0.74998E+00_JPRB, 0.71208E+00_JPRB ,&
&  0.70179E+00_JPRB, 0.68094E+00_JPRB, 0.65104E+00_JPRB, 0.63931E+00_JPRB, 0.63446E+00_JPRB, 0.62687E+00_JPRB, 0.62909E+00_JPRB ,&
&  0.64999E+00_JPRB, 0.60375E+00_JPRB, 0.53673E+00_JPRB, 0.49414E+00_JPRB, 0.51357E+00_JPRB, 0.51130E+00_JPRB, 0.51982E+00_JPRB ,&
&  0.52344E+00_JPRB, 0.54406E+00_JPRB, 0.52905E+00_JPRB, 0.53652E+00_JPRB, 0.56597E+00_JPRB, 0.58751E+00_JPRB, 0.59867E+00_JPRB ,&
&  0.59498E+00_JPRB, 0.58411E+00_JPRB, 0.58788E+00_JPRB, 0.61433E+00_JPRB, 0.66436E+00_JPRB, 0.65453E+00_JPRB, 0.68908E+00_JPRB ,&
&  0.70217E+00_JPRB, 0.72027E+00_JPRB, 0.72308E+00_JPRB, 0.75574E+00_JPRB, 0.81645E+00_JPRB, 0.91591E+00_JPRB, 0.99224E+00_JPRB ,&
&  0.10572E+01_JPRB, 0.11128E+01_JPRB, 0.11446E+01_JPRB, 0.12319E+01_JPRB, 0.13568E+01_JPRB, 0.15144E+01_JPRB, 0.14893E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.14527E+01_JPRB, 0.13964E+01_JPRB, 0.17803E+01_JPRB, 0.22058E+01_JPRB, 0.23382E+01_JPRB, 0.23530E+01_JPRB ,&
&  0.23140E+01_JPRB, 0.22517E+01_JPRB, 0.23441E+01_JPRB, 0.22644E+01_JPRB, 0.22011E+01_JPRB, 0.22432E+01_JPRB, 0.23232E+01_JPRB ,&
&  0.23231E+01_JPRB, 0.24415E+01_JPRB, 0.26143E+01_JPRB, 0.26256E+01_JPRB, 0.25930E+01_JPRB, 0.24939E+01_JPRB, 0.25001E+01_JPRB ,&
&  0.24311E+01_JPRB, 0.14814E+01_JPRB, 0.11542E+01_JPRB, 0.11874E+01_JPRB, 0.88751E+00_JPRB, 0.76409E+00_JPRB, 0.80121E+00_JPRB ,&
&  0.76674E+00_JPRB, 0.75503E+00_JPRB, 0.78011E+00_JPRB, 0.77030E+00_JPRB, 0.76292E+00_JPRB, 0.82265E+00_JPRB, 0.86319E+00_JPRB ,&
&  0.92735E+00_JPRB, 0.96906E+00_JPRB, 0.12007E+01_JPRB, 0.13515E+01_JPRB, 0.13926E+01_JPRB, 0.13876E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.12861E+01_JPRB, 0.12623E+01_JPRB, 0.12527E+01_JPRB, 0.12363E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.13626E+01_JPRB, 0.12971E+01_JPRB, 0.11840E+01_JPRB, 0.10426E+01_JPRB, 0.89151E+00_JPRB, 0.79127E+00_JPRB, 0.76629E+00_JPRB ,&
&  0.75066E+00_JPRB, 0.73354E+00_JPRB, 0.70285E+00_JPRB, 0.68088E+00_JPRB, 0.67026E+00_JPRB, 0.65885E+00_JPRB, 0.65843E+00_JPRB ,&
&  0.65844E+00_JPRB, 0.64803E+00_JPRB, 0.55773E+00_JPRB, 0.49413E+00_JPRB, 0.45995E+00_JPRB, 0.47803E+00_JPRB, 0.49671E+00_JPRB ,&
&  0.52239E+00_JPRB, 0.54927E+00_JPRB, 0.55682E+00_JPRB, 0.60730E+00_JPRB, 0.65762E+00_JPRB, 0.68270E+00_JPRB, 0.71953E+00_JPRB ,&
&  0.72437E+00_JPRB, 0.71383E+00_JPRB, 0.70764E+00_JPRB, 0.75691E+00_JPRB, 0.76963E+00_JPRB, 0.73431E+00_JPRB, 0.79144E+00_JPRB ,&
&  0.81748E+00_JPRB, 0.84134E+00_JPRB, 0.86106E+00_JPRB, 0.91166E+00_JPRB, 0.97430E+00_JPRB, 0.10049E+01_JPRB, 0.99443E+00_JPRB ,&
&  0.10209E+01_JPRB, 0.10752E+01_JPRB, 0.11551E+01_JPRB, 0.12289E+01_JPRB, 0.13228E+01_JPRB, 0.14179E+01_JPRB, 0.13918E+01_JPRB ,&
&  0.14261E+01_JPRB, 0.12673E+01_JPRB, 0.11276E+01_JPRB, 0.14383E+01_JPRB, 0.20714E+01_JPRB, 0.23795E+01_JPRB, 0.23052E+01_JPRB ,&
&  0.24987E+01_JPRB, 0.25744E+01_JPRB, 0.26930E+01_JPRB, 0.26520E+01_JPRB, 0.25650E+01_JPRB, 0.25828E+01_JPRB, 0.27333E+01_JPRB ,&
&  0.27956E+01_JPRB, 0.29513E+01_JPRB, 0.30383E+01_JPRB, 0.30210E+01_JPRB, 0.29608E+01_JPRB, 0.29818E+01_JPRB, 0.30120E+01_JPRB ,&
&  0.27492E+01_JPRB, 0.22952E+01_JPRB, 0.15955E+01_JPRB, 0.15101E+01_JPRB, 0.10543E+01_JPRB, 0.85710E+00_JPRB, 0.85280E+00_JPRB ,&
&  0.77701E+00_JPRB, 0.79571E+00_JPRB, 0.84840E+00_JPRB, 0.86598E+00_JPRB, 0.10004E+01_JPRB, 0.11250E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.14208E+01_JPRB, 0.14017E+01_JPRB, 0.16426E+01_JPRB, 0.16847E+01_JPRB, 0.16889E+01_JPRB, 0.16981E+01_JPRB, 0.16891E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.15658E+01_JPRB, 0.15092E+01_JPRB, 0.14535E+01_JPRB, 0.14090E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.14436E+01_JPRB, 0.13551E+01_JPRB, 0.12141E+01_JPRB, 0.10696E+01_JPRB, 0.92351E+00_JPRB, 0.81542E+00_JPRB, 0.79926E+00_JPRB ,&
&  0.82808E+00_JPRB, 0.82624E+00_JPRB, 0.80830E+00_JPRB, 0.77092E+00_JPRB, 0.74688E+00_JPRB, 0.72966E+00_JPRB, 0.71082E+00_JPRB ,&
&  0.69262E+00_JPRB, 0.62707E+00_JPRB, 0.52127E+00_JPRB, 0.46301E+00_JPRB, 0.44498E+00_JPRB, 0.44199E+00_JPRB, 0.47290E+00_JPRB ,&
&  0.51459E+00_JPRB, 0.58506E+00_JPRB, 0.65330E+00_JPRB, 0.73400E+00_JPRB, 0.83657E+00_JPRB, 0.93370E+00_JPRB, 0.96137E+00_JPRB ,&
&  0.97383E+00_JPRB, 0.85356E+00_JPRB, 0.88466E+00_JPRB, 0.87855E+00_JPRB, 0.91541E+00_JPRB, 0.91465E+00_JPRB, 0.91960E+00_JPRB ,&
&  0.90661E+00_JPRB, 0.92826E+00_JPRB, 0.95241E+00_JPRB, 0.97685E+00_JPRB, 0.99043E+00_JPRB, 0.99834E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.11421E+01_JPRB, 0.11979E+01_JPRB, 0.12438E+01_JPRB, 0.13217E+01_JPRB, 0.14240E+01_JPRB, 0.14560E+01_JPRB ,&
&  0.13985E+01_JPRB, 0.13471E+01_JPRB, 0.15252E+01_JPRB, 0.17394E+01_JPRB, 0.19971E+01_JPRB, 0.23612E+01_JPRB, 0.25559E+01_JPRB ,&
&  0.26784E+01_JPRB, 0.26129E+01_JPRB, 0.26599E+01_JPRB, 0.25189E+01_JPRB, 0.28631E+01_JPRB, 0.29261E+01_JPRB, 0.30771E+01_JPRB ,&
&  0.32253E+01_JPRB, 0.34632E+01_JPRB, 0.33774E+01_JPRB, 0.32888E+01_JPRB, 0.33745E+01_JPRB, 0.36003E+01_JPRB, 0.32387E+01_JPRB ,&
&  0.27211E+01_JPRB, 0.29263E+01_JPRB, 0.25646E+01_JPRB, 0.20984E+01_JPRB, 0.15840E+01_JPRB, 0.13452E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.10056E+01_JPRB, 0.94317E+00_JPRB, 0.96924E+00_JPRB, 0.11481E+01_JPRB, 0.14425E+01_JPRB, 0.17006E+01_JPRB, 0.18487E+01_JPRB ,&
&  0.18911E+01_JPRB, 0.22704E+01_JPRB, 0.22351E+01_JPRB, 0.21315E+01_JPRB, 0.20942E+01_JPRB, 0.20785E+01_JPRB, 0.20161E+01_JPRB ,&
&  0.18759E+01_JPRB, 0.17650E+01_JPRB, 0.16747E+01_JPRB, 0.15932E+01_JPRB, 0.15117E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.14827E+01_JPRB, 0.14015E+01_JPRB, 0.12600E+01_JPRB, 0.11037E+01_JPRB, 0.96445E+00_JPRB, 0.86353E+00_JPRB, 0.84155E+00_JPRB ,&
&  0.86062E+00_JPRB, 0.87927E+00_JPRB, 0.89103E+00_JPRB, 0.89118E+00_JPRB, 0.87828E+00_JPRB, 0.84457E+00_JPRB, 0.80320E+00_JPRB ,&
&  0.76999E+00_JPRB, 0.63095E+00_JPRB, 0.51449E+00_JPRB, 0.47637E+00_JPRB, 0.44848E+00_JPRB, 0.43640E+00_JPRB, 0.45611E+00_JPRB ,&
&  0.52548E+00_JPRB, 0.67871E+00_JPRB, 0.84518E+00_JPRB, 0.10224E+01_JPRB, 0.11695E+01_JPRB, 0.12579E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11055E+01_JPRB, 0.11031E+01_JPRB, 0.10640E+01_JPRB, 0.10529E+01_JPRB, 0.10209E+01_JPRB, 0.99921E+00_JPRB ,&
&  0.98711E+00_JPRB, 0.98263E+00_JPRB, 0.10221E+01_JPRB, 0.10519E+01_JPRB, 0.10760E+01_JPRB, 0.10985E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11917E+01_JPRB, 0.12233E+01_JPRB, 0.13155E+01_JPRB, 0.11669E+01_JPRB, 0.12299E+01_JPRB, 0.13764E+01_JPRB ,&
&  0.16764E+01_JPRB, 0.17141E+01_JPRB, 0.16600E+01_JPRB, 0.18253E+01_JPRB, 0.17443E+01_JPRB, 0.19326E+01_JPRB, 0.23913E+01_JPRB ,&
&  0.25537E+01_JPRB, 0.25810E+01_JPRB, 0.27409E+01_JPRB, 0.23995E+01_JPRB, 0.24043E+01_JPRB, 0.34651E+01_JPRB, 0.37680E+01_JPRB ,&
&  0.38570E+01_JPRB, 0.38980E+01_JPRB, 0.37515E+01_JPRB, 0.38002E+01_JPRB, 0.33324E+01_JPRB, 0.29280E+01_JPRB, 0.22448E+01_JPRB ,&
&  0.23972E+01_JPRB, 0.23174E+01_JPRB, 0.25622E+01_JPRB, 0.23260E+01_JPRB, 0.21493E+01_JPRB, 0.18887E+01_JPRB, 0.15351E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.12117E+01_JPRB, 0.12468E+01_JPRB, 0.16496E+01_JPRB, 0.21641E+01_JPRB, 0.22419E+01_JPRB, 0.26791E+01_JPRB ,&
&  0.28834E+01_JPRB, 0.29267E+01_JPRB, 0.25583E+01_JPRB, 0.24821E+01_JPRB, 0.23366E+01_JPRB, 0.21984E+01_JPRB, 0.20410E+01_JPRB ,&
&  0.19128E+01_JPRB, 0.18123E+01_JPRB, 0.17182E+01_JPRB, 0.16259E+01_JPRB, 0.15418E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.15121E+01_JPRB, 0.14426E+01_JPRB, 0.13043E+01_JPRB, 0.11464E+01_JPRB, 0.10116E+01_JPRB, 0.92562E+00_JPRB, 0.90872E+00_JPRB ,&
&  0.92861E+00_JPRB, 0.94875E+00_JPRB, 0.95712E+00_JPRB, 0.95037E+00_JPRB, 0.93481E+00_JPRB, 0.92105E+00_JPRB, 0.89850E+00_JPRB ,&
&  0.85361E+00_JPRB, 0.71200E+00_JPRB, 0.55166E+00_JPRB, 0.49026E+00_JPRB, 0.48573E+00_JPRB, 0.46570E+00_JPRB, 0.47439E+00_JPRB ,&
&  0.61274E+00_JPRB, 0.86571E+00_JPRB, 0.11866E+01_JPRB, 0.14334E+01_JPRB, 0.15791E+01_JPRB, 0.15456E+01_JPRB, 0.14203E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.13231E+01_JPRB, 0.11937E+01_JPRB, 0.10898E+01_JPRB, 0.10376E+01_JPRB, 0.10135E+01_JPRB, 0.98486E+00_JPRB ,&
&  0.96239E+00_JPRB, 0.96549E+00_JPRB, 0.98984E+00_JPRB, 0.10076E+01_JPRB, 0.10388E+01_JPRB, 0.10507E+01_JPRB, 0.10562E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.11550E+01_JPRB, 0.12339E+01_JPRB, 0.13522E+01_JPRB, 0.12796E+01_JPRB, 0.13472E+01_JPRB, 0.15628E+01_JPRB ,&
&  0.17521E+01_JPRB, 0.17927E+01_JPRB, 0.17755E+01_JPRB, 0.17977E+01_JPRB, 0.19145E+01_JPRB, 0.19550E+01_JPRB, 0.23113E+01_JPRB ,&
&  0.22381E+01_JPRB, 0.21308E+01_JPRB, 0.21348E+01_JPRB, 0.20761E+01_JPRB, 0.26847E+01_JPRB, 0.38353E+01_JPRB, 0.44440E+01_JPRB ,&
&  0.44232E+01_JPRB, 0.43515E+01_JPRB, 0.42624E+01_JPRB, 0.37973E+01_JPRB, 0.30214E+01_JPRB, 0.23013E+01_JPRB, 0.27398E+01_JPRB ,&
&  0.29718E+01_JPRB, 0.30841E+01_JPRB, 0.29280E+01_JPRB, 0.24377E+01_JPRB, 0.21065E+01_JPRB, 0.23351E+01_JPRB, 0.20947E+01_JPRB ,&
&  0.17280E+01_JPRB, 0.15949E+01_JPRB, 0.19790E+01_JPRB, 0.25119E+01_JPRB, 0.28025E+01_JPRB, 0.29508E+01_JPRB, 0.33315E+01_JPRB ,&
&  0.32039E+01_JPRB, 0.30942E+01_JPRB, 0.28504E+01_JPRB, 0.25597E+01_JPRB, 0.23143E+01_JPRB, 0.21252E+01_JPRB, 0.19776E+01_JPRB ,&
&  0.18946E+01_JPRB, 0.18328E+01_JPRB, 0.17487E+01_JPRB, 0.16646E+01_JPRB, 0.15757E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.15453E+01_JPRB, 0.14748E+01_JPRB, 0.13300E+01_JPRB, 0.11670E+01_JPRB, 0.10591E+01_JPRB, 0.99439E+00_JPRB, 0.96908E+00_JPRB ,&
&  0.97289E+00_JPRB, 0.97640E+00_JPRB, 0.97701E+00_JPRB, 0.96886E+00_JPRB, 0.95747E+00_JPRB, 0.94384E+00_JPRB, 0.93986E+00_JPRB ,&
&  0.90896E+00_JPRB, 0.85970E+00_JPRB, 0.64971E+00_JPRB, 0.56164E+00_JPRB, 0.54218E+00_JPRB, 0.54603E+00_JPRB, 0.55703E+00_JPRB ,&
&  0.79000E+00_JPRB, 0.12348E+01_JPRB, 0.16168E+01_JPRB, 0.18510E+01_JPRB, 0.19054E+01_JPRB, 0.16211E+01_JPRB, 0.15219E+01_JPRB ,&
&  0.15313E+01_JPRB, 0.14574E+01_JPRB, 0.13130E+01_JPRB, 0.11830E+01_JPRB, 0.10769E+01_JPRB, 0.10258E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.98208E+00_JPRB, 0.97954E+00_JPRB, 0.99259E+00_JPRB, 0.10258E+01_JPRB, 0.10443E+01_JPRB, 0.10557E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.11567E+01_JPRB, 0.12252E+01_JPRB, 0.13951E+01_JPRB, 0.14372E+01_JPRB, 0.16055E+01_JPRB, 0.19698E+01_JPRB ,&
&  0.20045E+01_JPRB, 0.20750E+01_JPRB, 0.19768E+01_JPRB, 0.19388E+01_JPRB, 0.19566E+01_JPRB, 0.19997E+01_JPRB, 0.23208E+01_JPRB ,&
&  0.22940E+01_JPRB, 0.23143E+01_JPRB, 0.25366E+01_JPRB, 0.27163E+01_JPRB, 0.29655E+01_JPRB, 0.35961E+01_JPRB, 0.51242E+01_JPRB ,&
&  0.44949E+01_JPRB, 0.44866E+01_JPRB, 0.44386E+01_JPRB, 0.40878E+01_JPRB, 0.21867E+01_JPRB, 0.16501E+01_JPRB, 0.25166E+01_JPRB ,&
&  0.26642E+01_JPRB, 0.16632E+01_JPRB, 0.12307E+01_JPRB, 0.16800E+01_JPRB, 0.13294E+01_JPRB, 0.15837E+01_JPRB, 0.21708E+01_JPRB ,&
&  0.22311E+01_JPRB, 0.20476E+01_JPRB, 0.25175E+01_JPRB, 0.27749E+01_JPRB, 0.28685E+01_JPRB, 0.35373E+01_JPRB, 0.36083E+01_JPRB ,&
&  0.34605E+01_JPRB, 0.29241E+01_JPRB, 0.28402E+01_JPRB, 0.25170E+01_JPRB, 0.22801E+01_JPRB, 0.21413E+01_JPRB, 0.20348E+01_JPRB ,&
&  0.19095E+01_JPRB, 0.17870E+01_JPRB, 0.17021E+01_JPRB, 0.16552E+01_JPRB, 0.15991E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.15469E+01_JPRB, 0.14744E+01_JPRB, 0.13240E+01_JPRB, 0.11649E+01_JPRB, 0.10796E+01_JPRB, 0.10225E+01_JPRB, 0.99015E+00_JPRB ,&
&  0.98640E+00_JPRB, 0.98981E+00_JPRB, 0.98746E+00_JPRB, 0.98165E+00_JPRB, 0.97433E+00_JPRB, 0.97008E+00_JPRB, 0.96679E+00_JPRB ,&
&  0.94273E+00_JPRB, 0.87913E+00_JPRB, 0.76203E+00_JPRB, 0.67948E+00_JPRB, 0.65873E+00_JPRB, 0.65564E+00_JPRB, 0.72695E+00_JPRB ,&
&  0.99349E+00_JPRB, 0.15746E+01_JPRB, 0.20003E+01_JPRB, 0.21142E+01_JPRB, 0.18771E+01_JPRB, 0.16418E+01_JPRB, 0.16824E+01_JPRB ,&
&  0.16764E+01_JPRB, 0.15635E+01_JPRB, 0.13843E+01_JPRB, 0.12617E+01_JPRB, 0.11619E+01_JPRB, 0.10992E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10358E+01_JPRB, 0.10240E+01_JPRB, 0.10459E+01_JPRB, 0.10597E+01_JPRB, 0.10669E+01_JPRB, 0.10764E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11975E+01_JPRB, 0.12999E+01_JPRB, 0.14754E+01_JPRB, 0.18476E+01_JPRB, 0.21056E+01_JPRB, 0.20443E+01_JPRB ,&
&  0.19168E+01_JPRB, 0.19609E+01_JPRB, 0.21783E+01_JPRB, 0.20069E+01_JPRB, 0.19348E+01_JPRB, 0.19874E+01_JPRB, 0.22489E+01_JPRB ,&
&  0.26707E+01_JPRB, 0.29092E+01_JPRB, 0.27605E+01_JPRB, 0.29351E+01_JPRB, 0.32458E+01_JPRB, 0.31265E+01_JPRB, 0.43392E+01_JPRB ,&
&  0.47550E+01_JPRB, 0.41237E+01_JPRB, 0.31842E+01_JPRB, 0.24005E+01_JPRB, 0.24139E+01_JPRB, 0.18796E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.95024E+00_JPRB, 0.87059E+00_JPRB, 0.96520E+00_JPRB, 0.97209E+00_JPRB, 0.13231E+01_JPRB, 0.20332E+01_JPRB ,&
&  0.26140E+01_JPRB, 0.26766E+01_JPRB, 0.28740E+01_JPRB, 0.29008E+01_JPRB, 0.28510E+01_JPRB, 0.35330E+01_JPRB, 0.36841E+01_JPRB ,&
&  0.30294E+01_JPRB, 0.26676E+01_JPRB, 0.25961E+01_JPRB, 0.23276E+01_JPRB, 0.22052E+01_JPRB, 0.21076E+01_JPRB, 0.19709E+01_JPRB ,&
&  0.18042E+01_JPRB, 0.16532E+01_JPRB, 0.15893E+01_JPRB, 0.15845E+01_JPRB, 0.15888E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.15345E+01_JPRB, 0.14573E+01_JPRB, 0.13263E+01_JPRB, 0.11913E+01_JPRB, 0.11262E+01_JPRB, 0.10809E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.10316E+01_JPRB, 0.10215E+01_JPRB, 0.10198E+01_JPRB, 0.10215E+01_JPRB, 0.10195E+01_JPRB, 0.10155E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.98304E+00_JPRB, 0.91347E+00_JPRB, 0.84803E+00_JPRB, 0.78689E+00_JPRB, 0.78842E+00_JPRB, 0.78598E+00_JPRB, 0.90705E+00_JPRB ,&
&  0.12729E+01_JPRB, 0.19246E+01_JPRB, 0.23175E+01_JPRB, 0.22870E+01_JPRB, 0.20689E+01_JPRB, 0.19040E+01_JPRB, 0.18371E+01_JPRB ,&
&  0.17203E+01_JPRB, 0.16089E+01_JPRB, 0.14564E+01_JPRB, 0.13363E+01_JPRB, 0.12426E+01_JPRB, 0.11756E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.10924E+01_JPRB, 0.11027E+01_JPRB, 0.10971E+01_JPRB, 0.11002E+01_JPRB, 0.10993E+01_JPRB, 0.11022E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.11849E+01_JPRB, 0.12935E+01_JPRB, 0.14868E+01_JPRB, 0.18065E+01_JPRB, 0.19461E+01_JPRB, 0.17808E+01_JPRB, 0.19140E+01_JPRB ,&
&  0.19846E+01_JPRB, 0.21259E+01_JPRB, 0.20647E+01_JPRB, 0.21292E+01_JPRB, 0.19969E+01_JPRB, 0.19413E+01_JPRB, 0.22284E+01_JPRB ,&
&  0.27317E+01_JPRB, 0.33328E+01_JPRB, 0.28780E+01_JPRB, 0.27738E+01_JPRB, 0.36211E+01_JPRB, 0.35210E+01_JPRB, 0.39750E+01_JPRB ,&
&  0.44982E+01_JPRB, 0.38252E+01_JPRB, 0.31542E+01_JPRB, 0.26387E+01_JPRB, 0.34088E+01_JPRB, 0.34306E+01_JPRB, 0.17308E+01_JPRB ,&
&  0.12318E+01_JPRB, 0.10199E+01_JPRB, 0.91127E+00_JPRB, 0.87536E+00_JPRB, 0.89554E+00_JPRB, 0.12961E+01_JPRB, 0.23316E+01_JPRB ,&
&  0.26199E+01_JPRB, 0.30869E+01_JPRB, 0.32329E+01_JPRB, 0.31783E+01_JPRB, 0.30812E+01_JPRB, 0.31358E+01_JPRB, 0.29418E+01_JPRB ,&
&  0.29654E+01_JPRB, 0.27222E+01_JPRB, 0.24166E+01_JPRB, 0.21767E+01_JPRB, 0.20029E+01_JPRB, 0.18079E+01_JPRB, 0.16377E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.14830E+01_JPRB, 0.14808E+01_JPRB, 0.15172E+01_JPRB, 0.15704E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.15202E+01_JPRB, 0.14799E+01_JPRB, 0.13669E+01_JPRB, 0.12453E+01_JPRB, 0.11817E+01_JPRB, 0.11377E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.10824E+01_JPRB, 0.10634E+01_JPRB, 0.10482E+01_JPRB, 0.10386E+01_JPRB, 0.10405E+01_JPRB, 0.10312E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.99548E+00_JPRB, 0.93653E+00_JPRB, 0.87428E+00_JPRB, 0.89450E+00_JPRB, 0.90720E+00_JPRB, 0.95349E+00_JPRB, 0.11413E+01_JPRB ,&
&  0.16683E+01_JPRB, 0.23959E+01_JPRB, 0.25818E+01_JPRB, 0.24902E+01_JPRB, 0.22747E+01_JPRB, 0.20737E+01_JPRB, 0.17762E+01_JPRB ,&
&  0.15633E+01_JPRB, 0.15202E+01_JPRB, 0.14031E+01_JPRB, 0.13543E+01_JPRB, 0.13065E+01_JPRB, 0.12479E+01_JPRB, 0.11982E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.11402E+01_JPRB, 0.11301E+01_JPRB, 0.11274E+01_JPRB, 0.11258E+01_JPRB, 0.11334E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.13113E+01_JPRB, 0.14611E+01_JPRB, 0.17261E+01_JPRB, 0.19762E+01_JPRB, 0.16735E+01_JPRB, 0.18352E+01_JPRB, 0.18835E+01_JPRB ,&
&  0.18959E+01_JPRB, 0.19251E+01_JPRB, 0.18215E+01_JPRB, 0.18601E+01_JPRB, 0.19540E+01_JPRB, 0.19528E+01_JPRB, 0.21656E+01_JPRB ,&
&  0.29734E+01_JPRB, 0.27833E+01_JPRB, 0.25644E+01_JPRB, 0.26776E+01_JPRB, 0.33637E+01_JPRB, 0.51861E+01_JPRB, 0.36668E+01_JPRB ,&
&  0.33487E+01_JPRB, 0.37955E+01_JPRB, 0.32796E+01_JPRB, 0.29955E+01_JPRB, 0.36615E+01_JPRB, 0.45380E+01_JPRB, 0.41943E+01_JPRB ,&
&  0.17516E+01_JPRB, 0.11624E+01_JPRB, 0.11378E+01_JPRB, 0.11254E+01_JPRB, 0.10608E+01_JPRB, 0.13467E+01_JPRB, 0.34575E+01_JPRB ,&
&  0.30015E+01_JPRB, 0.34346E+01_JPRB, 0.34289E+01_JPRB, 0.31864E+01_JPRB, 0.31489E+01_JPRB, 0.31191E+01_JPRB, 0.29275E+01_JPRB ,&
&  0.25936E+01_JPRB, 0.22992E+01_JPRB, 0.20359E+01_JPRB, 0.17728E+01_JPRB, 0.15667E+01_JPRB, 0.14302E+01_JPRB, 0.13701E+01_JPRB ,&
&  0.13543E+01_JPRB, 0.13693E+01_JPRB, 0.14048E+01_JPRB, 0.14581E+01_JPRB, 0.15132E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.15073E+01_JPRB, 0.14954E+01_JPRB, 0.13908E+01_JPRB, 0.12830E+01_JPRB, 0.12182E+01_JPRB, 0.11805E+01_JPRB, 0.11335E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.11074E+01_JPRB, 0.10566E+01_JPRB, 0.10244E+01_JPRB, 0.10244E+01_JPRB, 0.10143E+01_JPRB, 0.10057E+01_JPRB ,&
&  0.98389E+00_JPRB, 0.94753E+00_JPRB, 0.90161E+00_JPRB, 0.89771E+00_JPRB, 0.97275E+00_JPRB, 0.94081E+00_JPRB, 0.11388E+01_JPRB ,&
&  0.19880E+01_JPRB, 0.29866E+01_JPRB, 0.27330E+01_JPRB, 0.25498E+01_JPRB, 0.22931E+01_JPRB, 0.20925E+01_JPRB, 0.16337E+01_JPRB ,&
&  0.14692E+01_JPRB, 0.14512E+01_JPRB, 0.13793E+01_JPRB, 0.13653E+01_JPRB, 0.13351E+01_JPRB, 0.12938E+01_JPRB, 0.12465E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.11520E+01_JPRB, 0.11653E+01_JPRB, 0.11727E+01_JPRB, 0.11716E+01_JPRB, 0.12224E+01_JPRB, 0.13229E+01_JPRB ,&
&  0.14377E+01_JPRB, 0.16214E+01_JPRB, 0.18992E+01_JPRB, 0.18969E+01_JPRB, 0.18084E+01_JPRB, 0.17980E+01_JPRB, 0.18404E+01_JPRB ,&
&  0.18100E+01_JPRB, 0.18250E+01_JPRB, 0.18147E+01_JPRB, 0.17676E+01_JPRB, 0.18512E+01_JPRB, 0.19976E+01_JPRB, 0.21887E+01_JPRB ,&
&  0.25948E+01_JPRB, 0.24389E+01_JPRB, 0.22949E+01_JPRB, 0.25987E+01_JPRB, 0.32933E+01_JPRB, 0.46778E+01_JPRB, 0.48201E+01_JPRB ,&
&  0.42624E+01_JPRB, 0.31596E+01_JPRB, 0.29295E+01_JPRB, 0.35266E+01_JPRB, 0.40004E+01_JPRB, 0.44391E+01_JPRB, 0.60026E+01_JPRB ,&
&  0.59070E+01_JPRB, 0.32659E+01_JPRB, 0.25864E+01_JPRB, 0.32263E+01_JPRB, 0.22019E+01_JPRB, 0.15979E+01_JPRB, 0.23775E+01_JPRB ,&
&  0.28955E+01_JPRB, 0.33012E+01_JPRB, 0.33857E+01_JPRB, 0.32410E+01_JPRB, 0.32415E+01_JPRB, 0.29927E+01_JPRB, 0.25021E+01_JPRB ,&
&  0.21187E+01_JPRB, 0.17897E+01_JPRB, 0.15614E+01_JPRB, 0.14345E+01_JPRB, 0.13841E+01_JPRB, 0.13550E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.13255E+01_JPRB, 0.13384E+01_JPRB, 0.13805E+01_JPRB, 0.14499E+01_JPRB, 0.15016E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.14927E+01_JPRB, 0.14644E+01_JPRB, 0.13740E+01_JPRB, 0.12877E+01_JPRB, 0.12308E+01_JPRB, 0.11884E+01_JPRB, 0.11439E+01_JPRB ,&
&  0.12880E+01_JPRB, 0.11462E+01_JPRB, 0.10491E+01_JPRB, 0.10090E+01_JPRB, 0.99305E+00_JPRB, 0.98929E+00_JPRB, 0.98268E+00_JPRB ,&
&  0.97123E+00_JPRB, 0.95059E+00_JPRB, 0.93420E+00_JPRB, 0.94907E+00_JPRB, 0.10327E+01_JPRB, 0.13338E+01_JPRB, 0.13105E+01_JPRB ,&
&  0.21122E+01_JPRB, 0.34609E+01_JPRB, 0.28466E+01_JPRB, 0.25903E+01_JPRB, 0.23520E+01_JPRB, 0.22193E+01_JPRB, 0.16518E+01_JPRB ,&
&  0.14707E+01_JPRB, 0.14270E+01_JPRB, 0.13832E+01_JPRB, 0.13573E+01_JPRB, 0.13488E+01_JPRB, 0.13188E+01_JPRB, 0.12251E+01_JPRB ,&
&  0.11705E+01_JPRB, 0.11678E+01_JPRB, 0.11995E+01_JPRB, 0.12220E+01_JPRB, 0.12590E+01_JPRB, 0.13511E+01_JPRB, 0.14274E+01_JPRB ,&
&  0.15593E+01_JPRB, 0.18555E+01_JPRB, 0.20389E+01_JPRB, 0.18433E+01_JPRB, 0.17981E+01_JPRB, 0.18179E+01_JPRB, 0.18455E+01_JPRB ,&
&  0.17280E+01_JPRB, 0.16029E+01_JPRB, 0.17061E+01_JPRB, 0.18001E+01_JPRB, 0.18776E+01_JPRB, 0.19411E+01_JPRB, 0.20359E+01_JPRB ,&
&  0.23840E+01_JPRB, 0.25578E+01_JPRB, 0.28971E+01_JPRB, 0.29566E+01_JPRB, 0.33910E+01_JPRB, 0.43850E+01_JPRB, 0.48067E+01_JPRB ,&
&  0.50539E+01_JPRB, 0.41930E+01_JPRB, 0.41118E+01_JPRB, 0.43371E+01_JPRB, 0.43349E+01_JPRB, 0.42699E+01_JPRB, 0.48009E+01_JPRB ,&
&  0.53938E+01_JPRB, 0.63317E+01_JPRB, 0.55784E+01_JPRB, 0.43034E+01_JPRB, 0.31472E+01_JPRB, 0.21800E+01_JPRB, 0.24127E+01_JPRB ,&
&  0.29908E+01_JPRB, 0.30935E+01_JPRB, 0.30957E+01_JPRB, 0.39922E+01_JPRB, 0.31009E+01_JPRB, 0.24066E+01_JPRB, 0.18560E+01_JPRB ,&
&  0.15485E+01_JPRB, 0.14442E+01_JPRB, 0.14198E+01_JPRB, 0.13833E+01_JPRB, 0.13625E+01_JPRB, 0.13431E+01_JPRB, 0.13373E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.13792E+01_JPRB, 0.14262E+01_JPRB, 0.14773E+01_JPRB, 0.15070E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.14317E+01_JPRB, 0.13791E+01_JPRB, 0.13148E+01_JPRB, 0.12540E+01_JPRB, 0.11992E+01_JPRB, 0.11605E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.13989E+01_JPRB, 0.11192E+01_JPRB, 0.10161E+01_JPRB, 0.98656E+00_JPRB, 0.97723E+00_JPRB, 0.96744E+00_JPRB, 0.97100E+00_JPRB ,&
&  0.97475E+00_JPRB, 0.97875E+00_JPRB, 0.10050E+01_JPRB, 0.10776E+01_JPRB, 0.12675E+01_JPRB, 0.17359E+01_JPRB, 0.21098E+01_JPRB ,&
&  0.23854E+01_JPRB, 0.40593E+01_JPRB, 0.29525E+01_JPRB, 0.26191E+01_JPRB, 0.23832E+01_JPRB, 0.22579E+01_JPRB, 0.18253E+01_JPRB ,&
&  0.15295E+01_JPRB, 0.14345E+01_JPRB, 0.13913E+01_JPRB, 0.13452E+01_JPRB, 0.13127E+01_JPRB, 0.12506E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.12346E+01_JPRB, 0.12686E+01_JPRB, 0.13072E+01_JPRB, 0.13598E+01_JPRB, 0.14672E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.18503E+01_JPRB, 0.21207E+01_JPRB, 0.19997E+01_JPRB, 0.18610E+01_JPRB, 0.17839E+01_JPRB, 0.18254E+01_JPRB, 0.18486E+01_JPRB ,&
&  0.17556E+01_JPRB, 0.16487E+01_JPRB, 0.17297E+01_JPRB, 0.17620E+01_JPRB, 0.17729E+01_JPRB, 0.19197E+01_JPRB, 0.19852E+01_JPRB ,&
&  0.24170E+01_JPRB, 0.26963E+01_JPRB, 0.37705E+01_JPRB, 0.37097E+01_JPRB, 0.33604E+01_JPRB, 0.41598E+01_JPRB, 0.42004E+01_JPRB ,&
&  0.36676E+01_JPRB, 0.37383E+01_JPRB, 0.40587E+01_JPRB, 0.45727E+01_JPRB, 0.46667E+01_JPRB, 0.46544E+01_JPRB, 0.44395E+01_JPRB ,&
&  0.39755E+01_JPRB, 0.41215E+01_JPRB, 0.34329E+01_JPRB, 0.28291E+01_JPRB, 0.27717E+01_JPRB, 0.23646E+01_JPRB, 0.26872E+01_JPRB ,&
&  0.32233E+01_JPRB, 0.32714E+01_JPRB, 0.34880E+01_JPRB, 0.35360E+01_JPRB, 0.23447E+01_JPRB, 0.17394E+01_JPRB, 0.14414E+01_JPRB ,&
&  0.13766E+01_JPRB, 0.13947E+01_JPRB, 0.14017E+01_JPRB, 0.13805E+01_JPRB, 0.13709E+01_JPRB, 0.13701E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.14135E+01_JPRB, 0.14489E+01_JPRB, 0.14695E+01_JPRB, 0.14748E+01_JPRB, 0.14715E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.13844E+01_JPRB, 0.13490E+01_JPRB, 0.13063E+01_JPRB, 0.12724E+01_JPRB, 0.12673E+01_JPRB, 0.12459E+01_JPRB, 0.13208E+01_JPRB ,&
&  0.15100E+01_JPRB, 0.10832E+01_JPRB, 0.98278E+00_JPRB, 0.96351E+00_JPRB, 0.96420E+00_JPRB, 0.97558E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10813E+01_JPRB, 0.11603E+01_JPRB, 0.13577E+01_JPRB, 0.17615E+01_JPRB, 0.24509E+01_JPRB, 0.29721E+01_JPRB ,&
&  0.34258E+01_JPRB, 0.34054E+01_JPRB, 0.30552E+01_JPRB, 0.24750E+01_JPRB, 0.23063E+01_JPRB, 0.22150E+01_JPRB, 0.21271E+01_JPRB ,&
&  0.17291E+01_JPRB, 0.15484E+01_JPRB, 0.14644E+01_JPRB, 0.13777E+01_JPRB, 0.12837E+01_JPRB, 0.12670E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.13241E+01_JPRB, 0.13657E+01_JPRB, 0.14173E+01_JPRB, 0.14805E+01_JPRB, 0.15940E+01_JPRB, 0.17741E+01_JPRB, 0.19789E+01_JPRB ,&
&  0.21939E+01_JPRB, 0.22675E+01_JPRB, 0.20973E+01_JPRB, 0.19236E+01_JPRB, 0.18320E+01_JPRB, 0.18663E+01_JPRB, 0.18710E+01_JPRB ,&
&  0.18287E+01_JPRB, 0.18234E+01_JPRB, 0.18450E+01_JPRB, 0.18459E+01_JPRB, 0.17798E+01_JPRB, 0.20079E+01_JPRB, 0.21602E+01_JPRB ,&
&  0.26156E+01_JPRB, 0.31692E+01_JPRB, 0.43632E+01_JPRB, 0.35981E+01_JPRB, 0.37660E+01_JPRB, 0.39614E+01_JPRB, 0.39358E+01_JPRB ,&
&  0.34380E+01_JPRB, 0.37427E+01_JPRB, 0.42569E+01_JPRB, 0.45544E+01_JPRB, 0.49398E+01_JPRB, 0.46404E+01_JPRB, 0.46447E+01_JPRB ,&
&  0.34422E+01_JPRB, 0.32308E+01_JPRB, 0.24160E+01_JPRB, 0.25134E+01_JPRB, 0.25542E+01_JPRB, 0.24711E+01_JPRB, 0.26226E+01_JPRB ,&
&  0.30566E+01_JPRB, 0.30586E+01_JPRB, 0.27923E+01_JPRB, 0.23393E+01_JPRB, 0.15737E+01_JPRB, 0.13501E+01_JPRB, 0.12869E+01_JPRB ,&
&  0.13004E+01_JPRB, 0.13846E+01_JPRB, 0.13993E+01_JPRB, 0.14085E+01_JPRB, 0.14005E+01_JPRB, 0.14038E+01_JPRB, 0.14210E+01_JPRB ,&
&  0.14477E+01_JPRB, 0.14607E+01_JPRB, 0.14614E+01_JPRB, 0.14414E+01_JPRB, 0.14181E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.13297E+01_JPRB, 0.12984E+01_JPRB, 0.12668E+01_JPRB, 0.12186E+01_JPRB, 0.11467E+01_JPRB, 0.11099E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.99508E+00_JPRB, 0.10001E+01_JPRB, 0.10083E+01_JPRB, 0.10253E+01_JPRB, 0.10583E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.12752E+01_JPRB, 0.14905E+01_JPRB, 0.18428E+01_JPRB, 0.22668E+01_JPRB, 0.28171E+01_JPRB, 0.35301E+01_JPRB ,&
&  0.35875E+01_JPRB, 0.23793E+01_JPRB, 0.26440E+01_JPRB, 0.24782E+01_JPRB, 0.23199E+01_JPRB, 0.21041E+01_JPRB, 0.19523E+01_JPRB ,&
&  0.18137E+01_JPRB, 0.17174E+01_JPRB, 0.16563E+01_JPRB, 0.15649E+01_JPRB, 0.14495E+01_JPRB, 0.15000E+01_JPRB, 0.15556E+01_JPRB ,&
&  0.16079E+01_JPRB, 0.16804E+01_JPRB, 0.17699E+01_JPRB, 0.19022E+01_JPRB, 0.20849E+01_JPRB, 0.22589E+01_JPRB, 0.23925E+01_JPRB ,&
&  0.24457E+01_JPRB, 0.24746E+01_JPRB, 0.24150E+01_JPRB, 0.22788E+01_JPRB, 0.21178E+01_JPRB, 0.20849E+01_JPRB, 0.20172E+01_JPRB ,&
&  0.19695E+01_JPRB, 0.19275E+01_JPRB, 0.19969E+01_JPRB, 0.21474E+01_JPRB, 0.22565E+01_JPRB, 0.21943E+01_JPRB, 0.25536E+01_JPRB ,&
&  0.29746E+01_JPRB, 0.35902E+01_JPRB, 0.39318E+01_JPRB, 0.49238E+01_JPRB, 0.33873E+01_JPRB, 0.35609E+01_JPRB, 0.31930E+01_JPRB ,&
&  0.36022E+01_JPRB, 0.37573E+01_JPRB, 0.41107E+01_JPRB, 0.44082E+01_JPRB, 0.49427E+01_JPRB, 0.44873E+01_JPRB, 0.40256E+01_JPRB ,&
&  0.31487E+01_JPRB, 0.24491E+01_JPRB, 0.20382E+01_JPRB, 0.22197E+01_JPRB, 0.24430E+01_JPRB, 0.27390E+01_JPRB, 0.25689E+01_JPRB ,&
&  0.25971E+01_JPRB, 0.23352E+01_JPRB, 0.19855E+01_JPRB, 0.18411E+01_JPRB, 0.14253E+01_JPRB, 0.12972E+01_JPRB, 0.13053E+01_JPRB ,&
&  0.13413E+01_JPRB, 0.14064E+01_JPRB, 0.14123E+01_JPRB, 0.13998E+01_JPRB, 0.13903E+01_JPRB, 0.13871E+01_JPRB, 0.13961E+01_JPRB ,&
&  0.14004E+01_JPRB, 0.14041E+01_JPRB, 0.14013E+01_JPRB, 0.13798E+01_JPRB, 0.13527E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.12611E+01_JPRB, 0.12202E+01_JPRB, 0.11465E+01_JPRB, 0.10700E+01_JPRB, 0.10253E+01_JPRB, 0.99964E+00_JPRB, 0.98007E+00_JPRB ,&
&  0.98977E+00_JPRB, 0.10122E+01_JPRB, 0.10404E+01_JPRB, 0.10786E+01_JPRB, 0.11161E+01_JPRB, 0.11559E+01_JPRB, 0.12417E+01_JPRB ,&
&  0.13461E+01_JPRB, 0.15321E+01_JPRB, 0.18131E+01_JPRB, 0.21162E+01_JPRB, 0.23931E+01_JPRB, 0.26938E+01_JPRB, 0.29894E+01_JPRB ,&
&  0.33248E+01_JPRB, 0.33594E+01_JPRB, 0.29671E+01_JPRB, 0.20761E+01_JPRB, 0.18757E+01_JPRB, 0.20382E+01_JPRB, 0.19915E+01_JPRB ,&
&  0.19466E+01_JPRB, 0.18679E+01_JPRB, 0.18102E+01_JPRB, 0.18049E+01_JPRB, 0.18364E+01_JPRB, 0.19148E+01_JPRB, 0.19852E+01_JPRB ,&
&  0.20842E+01_JPRB, 0.21858E+01_JPRB, 0.22997E+01_JPRB, 0.24212E+01_JPRB, 0.25113E+01_JPRB, 0.25958E+01_JPRB, 0.26863E+01_JPRB ,&
&  0.27617E+01_JPRB, 0.28876E+01_JPRB, 0.29928E+01_JPRB, 0.29844E+01_JPRB, 0.27616E+01_JPRB, 0.26456E+01_JPRB, 0.24980E+01_JPRB ,&
&  0.24322E+01_JPRB, 0.23854E+01_JPRB, 0.24208E+01_JPRB, 0.26153E+01_JPRB, 0.27649E+01_JPRB, 0.25810E+01_JPRB, 0.27537E+01_JPRB ,&
&  0.33904E+01_JPRB, 0.40353E+01_JPRB, 0.36829E+01_JPRB, 0.46592E+01_JPRB, 0.27602E+01_JPRB, 0.31759E+01_JPRB, 0.34046E+01_JPRB ,&
&  0.31849E+01_JPRB, 0.34090E+01_JPRB, 0.37824E+01_JPRB, 0.40715E+01_JPRB, 0.41342E+01_JPRB, 0.37741E+01_JPRB, 0.34206E+01_JPRB ,&
&  0.28000E+01_JPRB, 0.21353E+01_JPRB, 0.18335E+01_JPRB, 0.18961E+01_JPRB, 0.21756E+01_JPRB, 0.24824E+01_JPRB, 0.19987E+01_JPRB ,&
&  0.15711E+01_JPRB, 0.17875E+01_JPRB, 0.17727E+01_JPRB, 0.18853E+01_JPRB, 0.15507E+01_JPRB, 0.12820E+01_JPRB, 0.13333E+01_JPRB ,&
&  0.13950E+01_JPRB, 0.13953E+01_JPRB, 0.13997E+01_JPRB, 0.13765E+01_JPRB, 0.13698E+01_JPRB, 0.13603E+01_JPRB, 0.13520E+01_JPRB ,&
&  0.13519E+01_JPRB, 0.13567E+01_JPRB, 0.13528E+01_JPRB, 0.13213E+01_JPRB, 0.12873E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.11505E+01_JPRB, 0.10697E+01_JPRB, 0.99092E+00_JPRB, 0.94140E+00_JPRB, 0.92269E+00_JPRB, 0.92048E+00_JPRB, 0.93322E+00_JPRB ,&
&  0.96240E+00_JPRB, 0.99935E+00_JPRB, 0.10456E+01_JPRB, 0.11058E+01_JPRB, 0.11641E+01_JPRB, 0.12291E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.14432E+01_JPRB, 0.16076E+01_JPRB, 0.18281E+01_JPRB, 0.20721E+01_JPRB, 0.22840E+01_JPRB, 0.25036E+01_JPRB, 0.27230E+01_JPRB ,&
&  0.29491E+01_JPRB, 0.30417E+01_JPRB, 0.29706E+01_JPRB, 0.28252E+01_JPRB, 0.23009E+01_JPRB, 0.24316E+01_JPRB, 0.25777E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.22835E+01_JPRB, 0.21518E+01_JPRB, 0.21469E+01_JPRB, 0.21985E+01_JPRB, 0.22995E+01_JPRB, 0.23857E+01_JPRB ,&
&  0.24742E+01_JPRB, 0.25511E+01_JPRB, 0.26237E+01_JPRB, 0.26832E+01_JPRB, 0.27423E+01_JPRB, 0.28477E+01_JPRB, 0.30271E+01_JPRB ,&
&  0.31219E+01_JPRB, 0.32217E+01_JPRB, 0.33022E+01_JPRB, 0.33637E+01_JPRB, 0.35648E+01_JPRB, 0.36935E+01_JPRB, 0.35883E+01_JPRB ,&
&  0.34696E+01_JPRB, 0.32351E+01_JPRB, 0.33282E+01_JPRB, 0.36172E+01_JPRB, 0.36726E+01_JPRB, 0.34673E+01_JPRB, 0.35139E+01_JPRB ,&
&  0.40518E+01_JPRB, 0.46265E+01_JPRB, 0.32410E+01_JPRB, 0.36074E+01_JPRB, 0.41191E+01_JPRB, 0.32925E+01_JPRB, 0.27443E+01_JPRB ,&
&  0.24138E+01_JPRB, 0.25716E+01_JPRB, 0.30763E+01_JPRB, 0.32957E+01_JPRB, 0.32653E+01_JPRB, 0.37317E+01_JPRB, 0.29847E+01_JPRB ,&
&  0.24317E+01_JPRB, 0.18443E+01_JPRB, 0.16148E+01_JPRB, 0.15960E+01_JPRB, 0.17441E+01_JPRB, 0.17859E+01_JPRB, 0.13181E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.15777E+01_JPRB, 0.16777E+01_JPRB, 0.19038E+01_JPRB, 0.18417E+01_JPRB, 0.12357E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.13433E+01_JPRB, 0.13388E+01_JPRB, 0.13264E+01_JPRB, 0.13151E+01_JPRB, 0.13046E+01_JPRB, 0.13008E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.12963E+01_JPRB, 0.12860E+01_JPRB, 0.12543E+01_JPRB, 0.12076E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.96823E+00_JPRB, 0.90075E+00_JPRB, 0.85592E+00_JPRB, 0.84791E+00_JPRB, 0.86227E+00_JPRB, 0.87751E+00_JPRB, 0.88530E+00_JPRB ,&
&  0.90809E+00_JPRB, 0.94174E+00_JPRB, 0.99205E+00_JPRB, 0.10376E+01_JPRB, 0.10935E+01_JPRB, 0.11487E+01_JPRB, 0.12357E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.14315E+01_JPRB, 0.15978E+01_JPRB, 0.18414E+01_JPRB, 0.20387E+01_JPRB, 0.22230E+01_JPRB, 0.24829E+01_JPRB ,&
&  0.27431E+01_JPRB, 0.29158E+01_JPRB, 0.30978E+01_JPRB, 0.32103E+01_JPRB, 0.31363E+01_JPRB, 0.30101E+01_JPRB, 0.30877E+01_JPRB ,&
&  0.26378E+01_JPRB, 0.22008E+01_JPRB, 0.22498E+01_JPRB, 0.23333E+01_JPRB, 0.24581E+01_JPRB, 0.25242E+01_JPRB, 0.25919E+01_JPRB ,&
&  0.26611E+01_JPRB, 0.27098E+01_JPRB, 0.27561E+01_JPRB, 0.28128E+01_JPRB, 0.29091E+01_JPRB, 0.30660E+01_JPRB, 0.32753E+01_JPRB ,&
&  0.33816E+01_JPRB, 0.35121E+01_JPRB, 0.36804E+01_JPRB, 0.34371E+01_JPRB, 0.38184E+01_JPRB, 0.43377E+01_JPRB, 0.46410E+01_JPRB ,&
&  0.46224E+01_JPRB, 0.43596E+01_JPRB, 0.44680E+01_JPRB, 0.46208E+01_JPRB, 0.46768E+01_JPRB, 0.45900E+01_JPRB, 0.45343E+01_JPRB ,&
&  0.48149E+01_JPRB, 0.46068E+01_JPRB, 0.32588E+01_JPRB, 0.28761E+01_JPRB, 0.29034E+01_JPRB, 0.22801E+01_JPRB, 0.21727E+01_JPRB ,&
&  0.18328E+01_JPRB, 0.19259E+01_JPRB, 0.23630E+01_JPRB, 0.27143E+01_JPRB, 0.29205E+01_JPRB, 0.33522E+01_JPRB, 0.27825E+01_JPRB ,&
&  0.19439E+01_JPRB, 0.14737E+01_JPRB, 0.12493E+01_JPRB, 0.11748E+01_JPRB, 0.12461E+01_JPRB, 0.12752E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.12301E+01_JPRB, 0.12660E+01_JPRB, 0.13105E+01_JPRB, 0.12815E+01_JPRB, 0.11629E+01_JPRB, 0.11399E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.12063E+01_JPRB, 0.12137E+01_JPRB, 0.12040E+01_JPRB, 0.12029E+01_JPRB, 0.11972E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.11892E+01_JPRB, 0.11628E+01_JPRB, 0.11234E+01_JPRB, 0.10539E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.81072E+00_JPRB, 0.76805E+00_JPRB, 0.76176E+00_JPRB, 0.77577E+00_JPRB, 0.78748E+00_JPRB, 0.79983E+00_JPRB, 0.81353E+00_JPRB ,&
&  0.82234E+00_JPRB, 0.82918E+00_JPRB, 0.84192E+00_JPRB, 0.85081E+00_JPRB, 0.89589E+00_JPRB, 0.94878E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.12089E+01_JPRB, 0.12927E+01_JPRB, 0.14323E+01_JPRB, 0.15357E+01_JPRB, 0.16415E+01_JPRB, 0.18068E+01_JPRB ,&
&  0.20117E+01_JPRB, 0.22549E+01_JPRB, 0.24846E+01_JPRB, 0.27318E+01_JPRB, 0.30702E+01_JPRB, 0.34148E+01_JPRB, 0.34894E+01_JPRB ,&
&  0.27356E+01_JPRB, 0.20958E+01_JPRB, 0.23405E+01_JPRB, 0.22493E+01_JPRB, 0.23976E+01_JPRB, 0.25182E+01_JPRB, 0.25806E+01_JPRB ,&
&  0.26585E+01_JPRB, 0.27169E+01_JPRB, 0.27719E+01_JPRB, 0.28339E+01_JPRB, 0.29559E+01_JPRB, 0.31053E+01_JPRB, 0.32970E+01_JPRB ,&
&  0.34459E+01_JPRB, 0.35671E+01_JPRB, 0.36198E+01_JPRB, 0.36542E+01_JPRB, 0.40301E+01_JPRB, 0.45528E+01_JPRB, 0.49385E+01_JPRB ,&
&  0.51760E+01_JPRB, 0.54153E+01_JPRB, 0.49826E+01_JPRB, 0.50328E+01_JPRB, 0.49521E+01_JPRB, 0.44532E+01_JPRB, 0.45192E+01_JPRB ,&
&  0.48296E+01_JPRB, 0.44756E+01_JPRB, 0.28138E+01_JPRB, 0.25440E+01_JPRB, 0.23823E+01_JPRB, 0.20748E+01_JPRB, 0.18586E+01_JPRB ,&
&  0.14890E+01_JPRB, 0.14996E+01_JPRB, 0.17044E+01_JPRB, 0.21782E+01_JPRB, 0.24610E+01_JPRB, 0.26098E+01_JPRB, 0.22845E+01_JPRB ,&
&  0.15998E+01_JPRB, 0.12499E+01_JPRB, 0.10699E+01_JPRB, 0.10842E+01_JPRB, 0.12299E+01_JPRB, 0.14252E+01_JPRB, 0.11972E+01_JPRB ,&
&  0.12483E+01_JPRB, 0.14472E+01_JPRB, 0.18032E+01_JPRB, 0.14522E+01_JPRB, 0.10488E+01_JPRB, 0.10251E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.10327E+01_JPRB, 0.10449E+01_JPRB, 0.10474E+01_JPRB, 0.10462E+01_JPRB, 0.10475E+01_JPRB, 0.10601E+01_JPRB, 0.10577E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.10199E+01_JPRB, 0.99786E+00_JPRB, 0.93121E+00_JPRB, 0.86604E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.70011E+00_JPRB, 0.69776E+00_JPRB, 0.70364E+00_JPRB, 0.72055E+00_JPRB, 0.72010E+00_JPRB, 0.72016E+00_JPRB, 0.75112E+00_JPRB ,&
&  0.73823E+00_JPRB, 0.72824E+00_JPRB, 0.73219E+00_JPRB, 0.71092E+00_JPRB, 0.73931E+00_JPRB, 0.76871E+00_JPRB, 0.82013E+00_JPRB ,&
&  0.87310E+00_JPRB, 0.90863E+00_JPRB, 0.93915E+00_JPRB, 0.98211E+00_JPRB, 0.10625E+01_JPRB, 0.11257E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.14246E+01_JPRB, 0.15834E+01_JPRB, 0.19376E+01_JPRB, 0.27517E+01_JPRB, 0.34198E+01_JPRB, 0.34367E+01_JPRB ,&
&  0.26462E+01_JPRB, 0.20595E+01_JPRB, 0.19751E+01_JPRB, 0.18282E+01_JPRB, 0.19588E+01_JPRB, 0.22067E+01_JPRB, 0.23161E+01_JPRB ,&
&  0.23334E+01_JPRB, 0.23879E+01_JPRB, 0.24194E+01_JPRB, 0.24594E+01_JPRB, 0.25095E+01_JPRB, 0.25452E+01_JPRB, 0.25194E+01_JPRB ,&
&  0.24324E+01_JPRB, 0.23231E+01_JPRB, 0.24267E+01_JPRB, 0.29579E+01_JPRB, 0.35483E+01_JPRB, 0.43454E+01_JPRB, 0.42759E+01_JPRB ,&
&  0.41062E+01_JPRB, 0.41056E+01_JPRB, 0.41406E+01_JPRB, 0.40766E+01_JPRB, 0.39142E+01_JPRB, 0.37307E+01_JPRB, 0.38085E+01_JPRB ,&
&  0.40586E+01_JPRB, 0.37790E+01_JPRB, 0.24773E+01_JPRB, 0.23793E+01_JPRB, 0.21488E+01_JPRB, 0.20322E+01_JPRB, 0.16379E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12316E+01_JPRB, 0.13005E+01_JPRB, 0.14900E+01_JPRB, 0.18408E+01_JPRB, 0.19773E+01_JPRB, 0.17454E+01_JPRB ,&
&  0.13751E+01_JPRB, 0.11254E+01_JPRB, 0.99060E+00_JPRB, 0.99281E+00_JPRB, 0.12478E+01_JPRB, 0.14401E+01_JPRB, 0.15956E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.12807E+01_JPRB, 0.11522E+01_JPRB, 0.14448E+01_JPRB, 0.91247E+00_JPRB, 0.82919E+00_JPRB, 0.81949E+00_JPRB ,&
&  0.82566E+00_JPRB, 0.81430E+00_JPRB, 0.80233E+00_JPRB, 0.78254E+00_JPRB, 0.76362E+00_JPRB, 0.81182E+00_JPRB, 0.86124E+00_JPRB ,&
&  0.81847E+00_JPRB, 0.84162E+00_JPRB, 0.80939E+00_JPRB, 0.75508E+00_JPRB, 0.73262E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.67257E+00_JPRB, 0.68500E+00_JPRB, 0.68871E+00_JPRB, 0.69865E+00_JPRB, 0.71797E+00_JPRB, 0.73350E+00_JPRB, 0.75421E+00_JPRB ,&
&  0.77753E+00_JPRB, 0.79178E+00_JPRB, 0.80699E+00_JPRB, 0.82583E+00_JPRB, 0.84126E+00_JPRB, 0.87210E+00_JPRB, 0.90207E+00_JPRB ,&
&  0.93951E+00_JPRB, 0.96946E+00_JPRB, 0.10003E+01_JPRB, 0.10469E+01_JPRB, 0.11051E+01_JPRB, 0.11938E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.15678E+01_JPRB, 0.17468E+01_JPRB, 0.18967E+01_JPRB, 0.19813E+01_JPRB, 0.20447E+01_JPRB, 0.23132E+01_JPRB ,&
&  0.20559E+01_JPRB, 0.21449E+01_JPRB, 0.20562E+01_JPRB, 0.18534E+01_JPRB, 0.18482E+01_JPRB, 0.17124E+01_JPRB, 0.16897E+01_JPRB ,&
&  0.17222E+01_JPRB, 0.16746E+01_JPRB, 0.16493E+01_JPRB, 0.16262E+01_JPRB, 0.15923E+01_JPRB, 0.15080E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.13924E+01_JPRB, 0.15143E+01_JPRB, 0.17636E+01_JPRB, 0.22658E+01_JPRB, 0.27750E+01_JPRB, 0.29898E+01_JPRB, 0.27767E+01_JPRB ,&
&  0.26761E+01_JPRB, 0.26997E+01_JPRB, 0.26289E+01_JPRB, 0.25113E+01_JPRB, 0.26266E+01_JPRB, 0.27226E+01_JPRB, 0.28332E+01_JPRB ,&
&  0.27509E+01_JPRB, 0.24404E+01_JPRB, 0.22407E+01_JPRB, 0.19864E+01_JPRB, 0.18722E+01_JPRB, 0.17586E+01_JPRB, 0.15295E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.11249E+01_JPRB, 0.10707E+01_JPRB, 0.11260E+01_JPRB, 0.13358E+01_JPRB, 0.13686E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.10383E+01_JPRB, 0.93090E+00_JPRB, 0.89540E+00_JPRB, 0.92647E+00_JPRB, 0.11110E+01_JPRB, 0.23358E+01_JPRB, 0.31447E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.97495E+00_JPRB, 0.65615E+00_JPRB, 0.91894E+00_JPRB, 0.71127E+00_JPRB, 0.68534E+00_JPRB, 0.68463E+00_JPRB ,&
&  0.67675E+00_JPRB, 0.67563E+00_JPRB, 0.67476E+00_JPRB, 0.66083E+00_JPRB, 0.65246E+00_JPRB, 0.65813E+00_JPRB, 0.69481E+00_JPRB ,&
&  0.69345E+00_JPRB, 0.69748E+00_JPRB, 0.68364E+00_JPRB, 0.68710E+00_JPRB, 0.67813E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.67776E+00_JPRB, 0.69572E+00_JPRB, 0.71808E+00_JPRB, 0.74574E+00_JPRB, 0.76657E+00_JPRB, 0.79311E+00_JPRB, 0.81634E+00_JPRB ,&
&  0.84619E+00_JPRB, 0.87213E+00_JPRB, 0.89577E+00_JPRB, 0.92083E+00_JPRB, 0.95021E+00_JPRB, 0.98342E+00_JPRB, 0.10246E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.11137E+01_JPRB, 0.11629E+01_JPRB, 0.12192E+01_JPRB, 0.12853E+01_JPRB, 0.13664E+01_JPRB, 0.14450E+01_JPRB ,&
&  0.15204E+01_JPRB, 0.15991E+01_JPRB, 0.16836E+01_JPRB, 0.17504E+01_JPRB, 0.17809E+01_JPRB, 0.18419E+01_JPRB, 0.16946E+01_JPRB ,&
&  0.20882E+01_JPRB, 0.20121E+01_JPRB, 0.18935E+01_JPRB, 0.17303E+01_JPRB, 0.15718E+01_JPRB, 0.14265E+01_JPRB, 0.13137E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.13003E+01_JPRB, 0.12870E+01_JPRB, 0.12933E+01_JPRB, 0.12009E+01_JPRB, 0.11974E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.14480E+01_JPRB, 0.16678E+01_JPRB, 0.18167E+01_JPRB, 0.19920E+01_JPRB, 0.22047E+01_JPRB, 0.21350E+01_JPRB, 0.20083E+01_JPRB ,&
&  0.18745E+01_JPRB, 0.17986E+01_JPRB, 0.14587E+01_JPRB, 0.16644E+01_JPRB, 0.17313E+01_JPRB, 0.18124E+01_JPRB, 0.19511E+01_JPRB ,&
&  0.18299E+01_JPRB, 0.14807E+01_JPRB, 0.15162E+01_JPRB, 0.17453E+01_JPRB, 0.16548E+01_JPRB, 0.15185E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.11443E+01_JPRB, 0.10481E+01_JPRB, 0.10140E+01_JPRB, 0.10787E+01_JPRB, 0.10682E+01_JPRB, 0.96675E+00_JPRB ,&
&  0.85545E+00_JPRB, 0.82794E+00_JPRB, 0.83628E+00_JPRB, 0.87696E+00_JPRB, 0.10803E+01_JPRB, 0.13293E+01_JPRB, 0.21210E+01_JPRB ,&
&  0.12306E+01_JPRB, 0.68402E+00_JPRB, 0.73921E+00_JPRB, 0.75469E+00_JPRB, 0.67525E+00_JPRB, 0.64598E+00_JPRB, 0.67217E+00_JPRB ,&
&  0.70142E+00_JPRB, 0.71299E+00_JPRB, 0.67069E+00_JPRB, 0.67004E+00_JPRB, 0.66602E+00_JPRB, 0.66259E+00_JPRB, 0.66213E+00_JPRB ,&
&  0.65802E+00_JPRB, 0.66016E+00_JPRB, 0.67131E+00_JPRB, 0.66904E+00_JPRB, 0.66975E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.59770E+00_JPRB, 0.60969E+00_JPRB, 0.62613E+00_JPRB, 0.64082E+00_JPRB, 0.65072E+00_JPRB, 0.66641E+00_JPRB, 0.67895E+00_JPRB ,&
&  0.69357E+00_JPRB, 0.70192E+00_JPRB, 0.71439E+00_JPRB, 0.72601E+00_JPRB, 0.73918E+00_JPRB, 0.75751E+00_JPRB, 0.77638E+00_JPRB ,&
&  0.79237E+00_JPRB, 0.81741E+00_JPRB, 0.84497E+00_JPRB, 0.86621E+00_JPRB, 0.91485E+00_JPRB, 0.10044E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.13174E+01_JPRB, 0.14323E+01_JPRB, 0.14982E+01_JPRB, 0.15195E+01_JPRB, 0.16000E+01_JPRB, 0.14148E+01_JPRB ,&
&  0.18857E+01_JPRB, 0.18668E+01_JPRB, 0.17074E+01_JPRB, 0.16136E+01_JPRB, 0.15165E+01_JPRB, 0.14226E+01_JPRB, 0.13749E+01_JPRB ,&
&  0.14191E+01_JPRB, 0.17113E+01_JPRB, 0.17340E+01_JPRB, 0.18003E+01_JPRB, 0.16391E+01_JPRB, 0.16278E+01_JPRB, 0.18278E+01_JPRB ,&
&  0.20615E+01_JPRB, 0.21601E+01_JPRB, 0.22045E+01_JPRB, 0.23302E+01_JPRB, 0.24705E+01_JPRB, 0.25450E+01_JPRB, 0.25755E+01_JPRB ,&
&  0.24805E+01_JPRB, 0.23926E+01_JPRB, 0.22478E+01_JPRB, 0.26693E+01_JPRB, 0.23478E+01_JPRB, 0.20306E+01_JPRB, 0.18419E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.12821E+01_JPRB, 0.13479E+01_JPRB, 0.13919E+01_JPRB, 0.13466E+01_JPRB, 0.13346E+01_JPRB, 0.12880E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.11591E+01_JPRB, 0.10865E+01_JPRB, 0.10619E+01_JPRB, 0.10660E+01_JPRB, 0.10085E+01_JPRB, 0.89044E+00_JPRB ,&
&  0.81271E+00_JPRB, 0.81398E+00_JPRB, 0.80686E+00_JPRB, 0.82433E+00_JPRB, 0.92763E+00_JPRB, 0.95737E+00_JPRB, 0.14233E+01_JPRB ,&
&  0.10400E+01_JPRB, 0.89362E+00_JPRB, 0.11715E+01_JPRB, 0.50747E+00_JPRB, 0.60137E+00_JPRB, 0.70494E+00_JPRB, 0.89657E+00_JPRB ,&
&  0.94027E+00_JPRB, 0.12046E+01_JPRB, 0.15402E+01_JPRB, 0.12240E+01_JPRB, 0.94251E+00_JPRB, 0.65460E+00_JPRB, 0.62281E+00_JPRB ,&
&  0.61500E+00_JPRB, 0.60602E+00_JPRB, 0.60860E+00_JPRB, 0.60117E+00_JPRB, 0.59859E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.51523E+00_JPRB, 0.52022E+00_JPRB, 0.53158E+00_JPRB, 0.53962E+00_JPRB, 0.54746E+00_JPRB, 0.55412E+00_JPRB, 0.55998E+00_JPRB ,&
&  0.57625E+00_JPRB, 0.58674E+00_JPRB, 0.61288E+00_JPRB, 0.63600E+00_JPRB, 0.65725E+00_JPRB, 0.69881E+00_JPRB, 0.73568E+00_JPRB ,&
&  0.79201E+00_JPRB, 0.86497E+00_JPRB, 0.90144E+00_JPRB, 0.94824E+00_JPRB, 0.97926E+00_JPRB, 0.10166E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.11572E+01_JPRB, 0.13672E+01_JPRB, 0.15213E+01_JPRB, 0.16838E+01_JPRB, 0.17095E+01_JPRB, 0.18375E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.18955E+01_JPRB, 0.19477E+01_JPRB, 0.18562E+01_JPRB, 0.18284E+01_JPRB, 0.18257E+01_JPRB, 0.18387E+01_JPRB, 0.18485E+01_JPRB ,&
&  0.19620E+01_JPRB, 0.21279E+01_JPRB, 0.21261E+01_JPRB, 0.22481E+01_JPRB, 0.23544E+01_JPRB, 0.23933E+01_JPRB, 0.26899E+01_JPRB ,&
&  0.29642E+01_JPRB, 0.31869E+01_JPRB, 0.34623E+01_JPRB, 0.37963E+01_JPRB, 0.40314E+01_JPRB, 0.41784E+01_JPRB, 0.42411E+01_JPRB ,&
&  0.42699E+01_JPRB, 0.44121E+01_JPRB, 0.47955E+01_JPRB, 0.47689E+01_JPRB, 0.37390E+01_JPRB, 0.30105E+01_JPRB, 0.22594E+01_JPRB ,&
&  0.14663E+01_JPRB, 0.11544E+01_JPRB, 0.11219E+01_JPRB, 0.11425E+01_JPRB, 0.11130E+01_JPRB, 0.11037E+01_JPRB, 0.10867E+01_JPRB ,&
&  0.10780E+01_JPRB, 0.10969E+01_JPRB, 0.10956E+01_JPRB, 0.10887E+01_JPRB, 0.10491E+01_JPRB, 0.94098E+00_JPRB, 0.81923E+00_JPRB ,&
&  0.78432E+00_JPRB, 0.77165E+00_JPRB, 0.78036E+00_JPRB, 0.82181E+00_JPRB, 0.90903E+00_JPRB, 0.10858E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.11471E+01_JPRB, 0.77797E+00_JPRB, 0.73542E+00_JPRB, 0.76383E+00_JPRB, 0.66411E+00_JPRB, 0.90756E+00_JPRB, 0.13144E+01_JPRB ,&
&  0.19266E+01_JPRB, 0.21664E+01_JPRB, 0.28344E+01_JPRB, 0.41388E+01_JPRB, 0.26134E+01_JPRB, 0.16881E+01_JPRB, 0.58356E+00_JPRB ,&
&  0.57426E+00_JPRB, 0.55072E+00_JPRB, 0.54371E+00_JPRB, 0.52786E+00_JPRB, 0.52129E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.42263E+00_JPRB, 0.41675E+00_JPRB, 0.42890E+00_JPRB, 0.44774E+00_JPRB, 0.46241E+00_JPRB, 0.49492E+00_JPRB, 0.53444E+00_JPRB ,&
&  0.57814E+00_JPRB, 0.63390E+00_JPRB, 0.68487E+00_JPRB, 0.72635E+00_JPRB, 0.79286E+00_JPRB, 0.88209E+00_JPRB, 0.93146E+00_JPRB ,&
&  0.10174E+01_JPRB, 0.10809E+01_JPRB, 0.11240E+01_JPRB, 0.11576E+01_JPRB, 0.12092E+01_JPRB, 0.12842E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.14809E+01_JPRB, 0.15933E+01_JPRB, 0.17400E+01_JPRB, 0.18422E+01_JPRB, 0.19023E+01_JPRB, 0.19389E+01_JPRB, 0.15391E+01_JPRB ,&
&  0.19617E+01_JPRB, 0.21239E+01_JPRB, 0.22055E+01_JPRB, 0.22014E+01_JPRB, 0.21840E+01_JPRB, 0.21915E+01_JPRB, 0.22725E+01_JPRB ,&
&  0.24059E+01_JPRB, 0.24589E+01_JPRB, 0.24688E+01_JPRB, 0.25010E+01_JPRB, 0.32133E+01_JPRB, 0.33594E+01_JPRB, 0.36923E+01_JPRB ,&
&  0.40102E+01_JPRB, 0.42916E+01_JPRB, 0.46593E+01_JPRB, 0.50483E+01_JPRB, 0.54517E+01_JPRB, 0.57951E+01_JPRB, 0.61062E+01_JPRB ,&
&  0.63975E+01_JPRB, 0.67094E+01_JPRB, 0.69997E+01_JPRB, 0.60006E+01_JPRB, 0.47141E+01_JPRB, 0.38043E+01_JPRB, 0.22754E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.11469E+01_JPRB, 0.11338E+01_JPRB, 0.10741E+01_JPRB, 0.96588E+00_JPRB, 0.92687E+00_JPRB, 0.89867E+00_JPRB ,&
&  0.92032E+00_JPRB, 0.97334E+00_JPRB, 0.10085E+01_JPRB, 0.10110E+01_JPRB, 0.95691E+00_JPRB, 0.85511E+00_JPRB, 0.75902E+00_JPRB ,&
&  0.76856E+00_JPRB, 0.76971E+00_JPRB, 0.77869E+00_JPRB, 0.85598E+00_JPRB, 0.96697E+00_JPRB, 0.12225E+01_JPRB, 0.15288E+01_JPRB ,&
&  0.14092E+01_JPRB, 0.79899E+00_JPRB, 0.72119E+00_JPRB, 0.69812E+00_JPRB, 0.69030E+00_JPRB, 0.75833E+00_JPRB, 0.91670E+00_JPRB ,&
&  0.12826E+01_JPRB, 0.20760E+01_JPRB, 0.23135E+01_JPRB, 0.15264E+01_JPRB, 0.20339E+01_JPRB, 0.13400E+01_JPRB, 0.56522E+00_JPRB ,&
&  0.53890E+00_JPRB, 0.52117E+00_JPRB, 0.49338E+00_JPRB, 0.46177E+00_JPRB, 0.43880E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.42505E+00_JPRB, 0.40671E+00_JPRB, 0.41870E+00_JPRB, 0.44443E+00_JPRB, 0.47460E+00_JPRB, 0.52151E+00_JPRB, 0.58063E+00_JPRB ,&
&  0.63553E+00_JPRB, 0.69214E+00_JPRB, 0.78758E+00_JPRB, 0.81944E+00_JPRB, 0.85362E+00_JPRB, 0.91646E+00_JPRB, 0.96976E+00_JPRB ,&
&  0.10344E+01_JPRB, 0.10971E+01_JPRB, 0.11621E+01_JPRB, 0.12241E+01_JPRB, 0.13047E+01_JPRB, 0.13940E+01_JPRB, 0.14713E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.16770E+01_JPRB, 0.18217E+01_JPRB, 0.19758E+01_JPRB, 0.21227E+01_JPRB, 0.21685E+01_JPRB, 0.22643E+01_JPRB ,&
&  0.15480E+01_JPRB, 0.21715E+01_JPRB, 0.22578E+01_JPRB, 0.23007E+01_JPRB, 0.22859E+01_JPRB, 0.22717E+01_JPRB, 0.23623E+01_JPRB ,&
&  0.24977E+01_JPRB, 0.24762E+01_JPRB, 0.26441E+01_JPRB, 0.29105E+01_JPRB, 0.37060E+01_JPRB, 0.39152E+01_JPRB, 0.41798E+01_JPRB ,&
&  0.44613E+01_JPRB, 0.47334E+01_JPRB, 0.50521E+01_JPRB, 0.54372E+01_JPRB, 0.58257E+01_JPRB, 0.61915E+01_JPRB, 0.65643E+01_JPRB ,&
&  0.69089E+01_JPRB, 0.72269E+01_JPRB, 0.75523E+01_JPRB, 0.62316E+01_JPRB, 0.51229E+01_JPRB, 0.41113E+01_JPRB, 0.25947E+01_JPRB ,&
&  0.15477E+01_JPRB, 0.11275E+01_JPRB, 0.11193E+01_JPRB, 0.11195E+01_JPRB, 0.96567E+00_JPRB, 0.88808E+00_JPRB, 0.79320E+00_JPRB ,&
&  0.78840E+00_JPRB, 0.81846E+00_JPRB, 0.84484E+00_JPRB, 0.85191E+00_JPRB, 0.85248E+00_JPRB, 0.77373E+00_JPRB, 0.71853E+00_JPRB ,&
&  0.73597E+00_JPRB, 0.73459E+00_JPRB, 0.75043E+00_JPRB, 0.79139E+00_JPRB, 0.85879E+00_JPRB, 0.91393E+00_JPRB, 0.91381E+00_JPRB ,&
&  0.90978E+00_JPRB, 0.80678E+00_JPRB, 0.76436E+00_JPRB, 0.77463E+00_JPRB, 0.74968E+00_JPRB, 0.72399E+00_JPRB, 0.73829E+00_JPRB ,&
&  0.81997E+00_JPRB, 0.92674E+00_JPRB, 0.10151E+01_JPRB, 0.99541E+00_JPRB, 0.10234E+01_JPRB, 0.81559E+00_JPRB, 0.62057E+00_JPRB ,&
&  0.58298E+00_JPRB, 0.57323E+00_JPRB, 0.52745E+00_JPRB, 0.49447E+00_JPRB, 0.45723E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.52792E+00_JPRB, 0.46497E+00_JPRB, 0.46069E+00_JPRB, 0.50522E+00_JPRB, 0.55020E+00_JPRB, 0.59745E+00_JPRB, 0.64314E+00_JPRB ,&
&  0.69383E+00_JPRB, 0.72454E+00_JPRB, 0.78222E+00_JPRB, 0.81650E+00_JPRB, 0.84688E+00_JPRB, 0.87037E+00_JPRB, 0.90158E+00_JPRB ,&
&  0.95810E+00_JPRB, 0.10262E+01_JPRB, 0.10905E+01_JPRB, 0.11767E+01_JPRB, 0.12671E+01_JPRB, 0.13502E+01_JPRB, 0.14438E+01_JPRB ,&
&  0.15370E+01_JPRB, 0.16422E+01_JPRB, 0.17525E+01_JPRB, 0.18840E+01_JPRB, 0.20012E+01_JPRB, 0.20543E+01_JPRB, 0.20101E+01_JPRB ,&
&  0.10379E+01_JPRB, 0.16865E+01_JPRB, 0.21954E+01_JPRB, 0.22070E+01_JPRB, 0.22196E+01_JPRB, 0.21946E+01_JPRB, 0.22847E+01_JPRB ,&
&  0.22601E+01_JPRB, 0.21705E+01_JPRB, 0.23903E+01_JPRB, 0.33253E+01_JPRB, 0.38199E+01_JPRB, 0.40738E+01_JPRB, 0.42322E+01_JPRB ,&
&  0.44300E+01_JPRB, 0.45847E+01_JPRB, 0.47743E+01_JPRB, 0.49808E+01_JPRB, 0.52110E+01_JPRB, 0.54333E+01_JPRB, 0.56421E+01_JPRB ,&
&  0.58292E+01_JPRB, 0.59451E+01_JPRB, 0.59239E+01_JPRB, 0.46667E+01_JPRB, 0.45791E+01_JPRB, 0.38389E+01_JPRB, 0.32228E+01_JPRB ,&
&  0.18749E+01_JPRB, 0.12118E+01_JPRB, 0.11591E+01_JPRB, 0.11515E+01_JPRB, 0.99057E+00_JPRB, 0.89804E+00_JPRB, 0.78069E+00_JPRB ,&
&  0.76594E+00_JPRB, 0.76420E+00_JPRB, 0.78156E+00_JPRB, 0.78273E+00_JPRB, 0.78438E+00_JPRB, 0.71540E+00_JPRB, 0.71736E+00_JPRB ,&
&  0.73696E+00_JPRB, 0.73859E+00_JPRB, 0.75652E+00_JPRB, 0.79417E+00_JPRB, 0.84402E+00_JPRB, 0.89002E+00_JPRB, 0.90559E+00_JPRB ,&
&  0.90058E+00_JPRB, 0.87872E+00_JPRB, 0.87072E+00_JPRB, 0.86368E+00_JPRB, 0.85421E+00_JPRB, 0.84197E+00_JPRB, 0.82229E+00_JPRB ,&
&  0.79644E+00_JPRB, 0.80063E+00_JPRB, 0.82572E+00_JPRB, 0.75866E+00_JPRB, 0.79801E+00_JPRB, 0.72578E+00_JPRB, 0.71056E+00_JPRB ,&
&  0.69316E+00_JPRB, 0.67701E+00_JPRB, 0.63091E+00_JPRB, 0.60694E+00_JPRB, 0.57194E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.64527E+00_JPRB, 0.56737E+00_JPRB, 0.50633E+00_JPRB, 0.52153E+00_JPRB, 0.58880E+00_JPRB, 0.64576E+00_JPRB, 0.67781E+00_JPRB ,&
&  0.71691E+00_JPRB, 0.73093E+00_JPRB, 0.74206E+00_JPRB, 0.75678E+00_JPRB, 0.79035E+00_JPRB, 0.81246E+00_JPRB, 0.85692E+00_JPRB ,&
&  0.89376E+00_JPRB, 0.92989E+00_JPRB, 0.96946E+00_JPRB, 0.10400E+01_JPRB, 0.11258E+01_JPRB, 0.12116E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.13819E+01_JPRB, 0.14897E+01_JPRB, 0.16382E+01_JPRB, 0.18185E+01_JPRB, 0.20677E+01_JPRB, 0.23436E+01_JPRB, 0.25562E+01_JPRB ,&
&  0.17994E+01_JPRB, 0.85982E+00_JPRB, 0.17327E+01_JPRB, 0.21333E+01_JPRB, 0.20738E+01_JPRB, 0.20585E+01_JPRB, 0.20210E+01_JPRB ,&
&  0.19910E+01_JPRB, 0.20130E+01_JPRB, 0.21081E+01_JPRB, 0.28139E+01_JPRB, 0.36019E+01_JPRB, 0.38863E+01_JPRB, 0.39886E+01_JPRB ,&
&  0.40916E+01_JPRB, 0.41324E+01_JPRB, 0.41989E+01_JPRB, 0.43005E+01_JPRB, 0.44286E+01_JPRB, 0.45080E+01_JPRB, 0.45708E+01_JPRB ,&
&  0.46214E+01_JPRB, 0.46123E+01_JPRB, 0.44371E+01_JPRB, 0.31798E+01_JPRB, 0.28276E+01_JPRB, 0.24167E+01_JPRB, 0.21798E+01_JPRB ,&
&  0.18756E+01_JPRB, 0.13920E+01_JPRB, 0.12456E+01_JPRB, 0.11879E+01_JPRB, 0.10471E+01_JPRB, 0.89222E+00_JPRB, 0.82335E+00_JPRB ,&
&  0.80331E+00_JPRB, 0.77646E+00_JPRB, 0.75669E+00_JPRB, 0.73597E+00_JPRB, 0.76751E+00_JPRB, 0.76295E+00_JPRB, 0.75067E+00_JPRB ,&
&  0.74447E+00_JPRB, 0.78522E+00_JPRB, 0.82860E+00_JPRB, 0.86617E+00_JPRB, 0.90471E+00_JPRB, 0.94868E+00_JPRB, 0.95612E+00_JPRB ,&
&  0.97191E+00_JPRB, 0.96230E+00_JPRB, 0.95451E+00_JPRB, 0.93063E+00_JPRB, 0.91124E+00_JPRB, 0.87797E+00_JPRB, 0.87441E+00_JPRB ,&
&  0.87388E+00_JPRB, 0.87747E+00_JPRB, 0.86195E+00_JPRB, 0.84077E+00_JPRB, 0.85672E+00_JPRB, 0.84158E+00_JPRB, 0.84365E+00_JPRB ,&
&  0.83686E+00_JPRB, 0.81528E+00_JPRB, 0.75927E+00_JPRB, 0.71214E+00_JPRB, 0.69026E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.71639E+00_JPRB, 0.66440E+00_JPRB, 0.59523E+00_JPRB, 0.54703E+00_JPRB, 0.56549E+00_JPRB, 0.63398E+00_JPRB, 0.69033E+00_JPRB ,&
&  0.71282E+00_JPRB, 0.71530E+00_JPRB, 0.70869E+00_JPRB, 0.71110E+00_JPRB, 0.73103E+00_JPRB, 0.73594E+00_JPRB, 0.76039E+00_JPRB ,&
&  0.79725E+00_JPRB, 0.82797E+00_JPRB, 0.85584E+00_JPRB, 0.90203E+00_JPRB, 0.95053E+00_JPRB, 0.10289E+01_JPRB, 0.11008E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.12198E+01_JPRB, 0.13109E+01_JPRB, 0.14381E+01_JPRB, 0.16249E+01_JPRB, 0.19623E+01_JPRB, 0.25503E+01_JPRB ,&
&  0.36270E+01_JPRB, 0.24473E+01_JPRB, 0.11170E+01_JPRB, 0.19527E+01_JPRB, 0.21158E+01_JPRB, 0.20592E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.18443E+01_JPRB, 0.17817E+01_JPRB, 0.18033E+01_JPRB, 0.21277E+01_JPRB, 0.25861E+01_JPRB, 0.31332E+01_JPRB, 0.34597E+01_JPRB ,&
&  0.35448E+01_JPRB, 0.35847E+01_JPRB, 0.36165E+01_JPRB, 0.36934E+01_JPRB, 0.38118E+01_JPRB, 0.39179E+01_JPRB, 0.39405E+01_JPRB ,&
&  0.39787E+01_JPRB, 0.40626E+01_JPRB, 0.39652E+01_JPRB, 0.29054E+01_JPRB, 0.26502E+01_JPRB, 0.25367E+01_JPRB, 0.23935E+01_JPRB ,&
&  0.20658E+01_JPRB, 0.18001E+01_JPRB, 0.15821E+01_JPRB, 0.13101E+01_JPRB, 0.11165E+01_JPRB, 0.10060E+01_JPRB, 0.97402E+00_JPRB ,&
&  0.92589E+00_JPRB, 0.85158E+00_JPRB, 0.82340E+00_JPRB, 0.79478E+00_JPRB, 0.81042E+00_JPRB, 0.84368E+00_JPRB, 0.84814E+00_JPRB ,&
&  0.87041E+00_JPRB, 0.90705E+00_JPRB, 0.93422E+00_JPRB, 0.96002E+00_JPRB, 0.99225E+00_JPRB, 0.10183E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.10502E+01_JPRB, 0.10445E+01_JPRB, 0.10298E+01_JPRB, 0.10083E+01_JPRB, 0.96295E+00_JPRB, 0.92308E+00_JPRB, 0.93325E+00_JPRB ,&
&  0.92575E+00_JPRB, 0.92898E+00_JPRB, 0.91228E+00_JPRB, 0.91754E+00_JPRB, 0.91084E+00_JPRB, 0.92515E+00_JPRB, 0.95031E+00_JPRB ,&
&  0.97205E+00_JPRB, 0.94756E+00_JPRB, 0.86052E+00_JPRB, 0.76619E+00_JPRB, 0.73986E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.75821E+00_JPRB, 0.73940E+00_JPRB, 0.69735E+00_JPRB, 0.61470E+00_JPRB, 0.58002E+00_JPRB, 0.59231E+00_JPRB, 0.65740E+00_JPRB ,&
&  0.68735E+00_JPRB, 0.69862E+00_JPRB, 0.68648E+00_JPRB, 0.67461E+00_JPRB, 0.67195E+00_JPRB, 0.68105E+00_JPRB, 0.70701E+00_JPRB ,&
&  0.73619E+00_JPRB, 0.73672E+00_JPRB, 0.73291E+00_JPRB, 0.74312E+00_JPRB, 0.78037E+00_JPRB, 0.83273E+00_JPRB, 0.91308E+00_JPRB ,&
&  0.97795E+00_JPRB, 0.10134E+01_JPRB, 0.10461E+01_JPRB, 0.11269E+01_JPRB, 0.12276E+01_JPRB, 0.12764E+01_JPRB, 0.14374E+01_JPRB ,&
&  0.21565E+01_JPRB, 0.32200E+01_JPRB, 0.13169E+01_JPRB, 0.20090E+01_JPRB, 0.23584E+01_JPRB, 0.21396E+01_JPRB, 0.19615E+01_JPRB ,&
&  0.18836E+01_JPRB, 0.16160E+01_JPRB, 0.14313E+01_JPRB, 0.15999E+01_JPRB, 0.18499E+01_JPRB, 0.20564E+01_JPRB, 0.23863E+01_JPRB ,&
&  0.26822E+01_JPRB, 0.28688E+01_JPRB, 0.30150E+01_JPRB, 0.31337E+01_JPRB, 0.32999E+01_JPRB, 0.35265E+01_JPRB, 0.37307E+01_JPRB ,&
&  0.38854E+01_JPRB, 0.41079E+01_JPRB, 0.42645E+01_JPRB, 0.33684E+01_JPRB, 0.32005E+01_JPRB, 0.34279E+01_JPRB, 0.32187E+01_JPRB ,&
&  0.27370E+01_JPRB, 0.23537E+01_JPRB, 0.19372E+01_JPRB, 0.15263E+01_JPRB, 0.12603E+01_JPRB, 0.11531E+01_JPRB, 0.11018E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.94694E+00_JPRB, 0.93099E+00_JPRB, 0.92449E+00_JPRB, 0.92576E+00_JPRB, 0.93745E+00_JPRB, 0.96283E+00_JPRB ,&
&  0.10062E+01_JPRB, 0.10464E+01_JPRB, 0.10759E+01_JPRB, 0.10911E+01_JPRB, 0.11019E+01_JPRB, 0.11100E+01_JPRB, 0.10974E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.10861E+01_JPRB, 0.10681E+01_JPRB, 0.10438E+01_JPRB, 0.10151E+01_JPRB, 0.10002E+01_JPRB, 0.99296E+00_JPRB ,&
&  0.98117E+00_JPRB, 0.96312E+00_JPRB, 0.93020E+00_JPRB, 0.90920E+00_JPRB, 0.93602E+00_JPRB, 0.91204E+00_JPRB, 0.95599E+00_JPRB ,&
&  0.10215E+01_JPRB, 0.10556E+01_JPRB, 0.99604E+00_JPRB, 0.79532E+00_JPRB, 0.76807E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.79440E+00_JPRB, 0.78337E+00_JPRB, 0.76797E+00_JPRB, 0.70970E+00_JPRB, 0.65118E+00_JPRB, 0.62134E+00_JPRB, 0.63445E+00_JPRB ,&
&  0.67143E+00_JPRB, 0.69764E+00_JPRB, 0.70081E+00_JPRB, 0.69392E+00_JPRB, 0.69593E+00_JPRB, 0.70879E+00_JPRB, 0.72596E+00_JPRB ,&
&  0.73448E+00_JPRB, 0.72853E+00_JPRB, 0.71559E+00_JPRB, 0.71876E+00_JPRB, 0.72364E+00_JPRB, 0.73868E+00_JPRB, 0.78263E+00_JPRB ,&
&  0.84192E+00_JPRB, 0.87508E+00_JPRB, 0.88266E+00_JPRB, 0.90750E+00_JPRB, 0.98427E+00_JPRB, 0.10099E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.13187E+01_JPRB, 0.25325E+01_JPRB, 0.27437E+01_JPRB, 0.31966E+01_JPRB, 0.28802E+01_JPRB, 0.24013E+01_JPRB, 0.22704E+01_JPRB ,&
&  0.22368E+01_JPRB, 0.18743E+01_JPRB, 0.15931E+01_JPRB, 0.13963E+01_JPRB, 0.14842E+01_JPRB, 0.16084E+01_JPRB, 0.16582E+01_JPRB ,&
&  0.18077E+01_JPRB, 0.21158E+01_JPRB, 0.23060E+01_JPRB, 0.24906E+01_JPRB, 0.26640E+01_JPRB, 0.28714E+01_JPRB, 0.31709E+01_JPRB ,&
&  0.35313E+01_JPRB, 0.39962E+01_JPRB, 0.45691E+01_JPRB, 0.43025E+01_JPRB, 0.37611E+01_JPRB, 0.43536E+01_JPRB, 0.45225E+01_JPRB ,&
&  0.42660E+01_JPRB, 0.32095E+01_JPRB, 0.24124E+01_JPRB, 0.18397E+01_JPRB, 0.14708E+01_JPRB, 0.14718E+01_JPRB, 0.12600E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.10615E+01_JPRB, 0.10348E+01_JPRB, 0.10411E+01_JPRB, 0.10483E+01_JPRB, 0.10380E+01_JPRB, 0.10560E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.11251E+01_JPRB, 0.11455E+01_JPRB, 0.11592E+01_JPRB, 0.11751E+01_JPRB, 0.11846E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.11361E+01_JPRB, 0.11112E+01_JPRB, 0.10430E+01_JPRB, 0.10331E+01_JPRB, 0.10304E+01_JPRB, 0.10357E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.99658E+00_JPRB, 0.99762E+00_JPRB, 0.96721E+00_JPRB, 0.94335E+00_JPRB, 0.94981E+00_JPRB, 0.93513E+00_JPRB, 0.90226E+00_JPRB ,&
&  0.90491E+00_JPRB, 0.88579E+00_JPRB, 0.85905E+00_JPRB, 0.82918E+00_JPRB, 0.80842E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.84468E+00_JPRB, 0.83383E+00_JPRB, 0.82785E+00_JPRB, 0.79836E+00_JPRB, 0.76019E+00_JPRB, 0.72569E+00_JPRB, 0.70614E+00_JPRB ,&
&  0.70661E+00_JPRB, 0.71901E+00_JPRB, 0.73168E+00_JPRB, 0.72400E+00_JPRB, 0.72348E+00_JPRB, 0.73984E+00_JPRB, 0.75013E+00_JPRB ,&
&  0.75671E+00_JPRB, 0.75547E+00_JPRB, 0.75076E+00_JPRB, 0.74947E+00_JPRB, 0.74471E+00_JPRB, 0.73086E+00_JPRB, 0.73080E+00_JPRB ,&
&  0.75531E+00_JPRB, 0.77975E+00_JPRB, 0.79120E+00_JPRB, 0.79571E+00_JPRB, 0.83983E+00_JPRB, 0.87210E+00_JPRB, 0.91474E+00_JPRB ,&
&  0.10477E+01_JPRB, 0.22054E+01_JPRB, 0.19867E+01_JPRB, 0.36794E+01_JPRB, 0.31326E+01_JPRB, 0.26312E+01_JPRB, 0.22359E+01_JPRB ,&
&  0.21369E+01_JPRB, 0.22184E+01_JPRB, 0.17926E+01_JPRB, 0.14978E+01_JPRB, 0.13708E+01_JPRB, 0.13873E+01_JPRB, 0.13750E+01_JPRB ,&
&  0.13547E+01_JPRB, 0.14135E+01_JPRB, 0.15336E+01_JPRB, 0.17181E+01_JPRB, 0.19378E+01_JPRB, 0.21303E+01_JPRB, 0.23558E+01_JPRB ,&
&  0.26302E+01_JPRB, 0.30979E+01_JPRB, 0.38505E+01_JPRB, 0.40467E+01_JPRB, 0.41740E+01_JPRB, 0.49208E+01_JPRB, 0.57264E+01_JPRB ,&
&  0.56479E+01_JPRB, 0.44488E+01_JPRB, 0.30734E+01_JPRB, 0.22002E+01_JPRB, 0.17899E+01_JPRB, 0.16675E+01_JPRB, 0.14376E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.11729E+01_JPRB, 0.11395E+01_JPRB, 0.11259E+01_JPRB, 0.11332E+01_JPRB, 0.11351E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.11930E+01_JPRB, 0.11946E+01_JPRB, 0.11874E+01_JPRB, 0.11890E+01_JPRB, 0.11935E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.11589E+01_JPRB, 0.10721E+01_JPRB, 0.10300E+01_JPRB, 0.10295E+01_JPRB, 0.10282E+01_JPRB, 0.10204E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.10317E+01_JPRB, 0.99599E+00_JPRB, 0.94734E+00_JPRB, 0.95107E+00_JPRB, 0.96422E+00_JPRB, 0.91029E+00_JPRB ,&
&  0.89375E+00_JPRB, 0.88154E+00_JPRB, 0.87542E+00_JPRB, 0.86874E+00_JPRB, 0.85662E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.85558E+00_JPRB, 0.84512E+00_JPRB, 0.84270E+00_JPRB, 0.82917E+00_JPRB, 0.80269E+00_JPRB, 0.77490E+00_JPRB, 0.75945E+00_JPRB ,&
&  0.74795E+00_JPRB, 0.74805E+00_JPRB, 0.75734E+00_JPRB, 0.75053E+00_JPRB, 0.75503E+00_JPRB, 0.76693E+00_JPRB, 0.77330E+00_JPRB ,&
&  0.76923E+00_JPRB, 0.76609E+00_JPRB, 0.75830E+00_JPRB, 0.75316E+00_JPRB, 0.75014E+00_JPRB, 0.73728E+00_JPRB, 0.72954E+00_JPRB ,&
&  0.73029E+00_JPRB, 0.74011E+00_JPRB, 0.74959E+00_JPRB, 0.75631E+00_JPRB, 0.78078E+00_JPRB, 0.80355E+00_JPRB, 0.83634E+00_JPRB ,&
&  0.97222E+00_JPRB, 0.23891E+01_JPRB, 0.18484E+01_JPRB, 0.33864E+01_JPRB, 0.29733E+01_JPRB, 0.24577E+01_JPRB, 0.20402E+01_JPRB ,&
&  0.19909E+01_JPRB, 0.19900E+01_JPRB, 0.17904E+01_JPRB, 0.16263E+01_JPRB, 0.14865E+01_JPRB, 0.13676E+01_JPRB, 0.12900E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.11810E+01_JPRB, 0.11829E+01_JPRB, 0.12293E+01_JPRB, 0.13560E+01_JPRB, 0.15187E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.18443E+01_JPRB, 0.21194E+01_JPRB, 0.26443E+01_JPRB, 0.35093E+01_JPRB, 0.38016E+01_JPRB, 0.44488E+01_JPRB, 0.67677E+01_JPRB ,&
&  0.69877E+01_JPRB, 0.59297E+01_JPRB, 0.39744E+01_JPRB, 0.26068E+01_JPRB, 0.20645E+01_JPRB, 0.18465E+01_JPRB, 0.16389E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.12959E+01_JPRB, 0.12446E+01_JPRB, 0.11879E+01_JPRB, 0.11610E+01_JPRB, 0.11878E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.12462E+01_JPRB, 0.12440E+01_JPRB, 0.12067E+01_JPRB, 0.11780E+01_JPRB, 0.11554E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11695E+01_JPRB, 0.11729E+01_JPRB, 0.11149E+01_JPRB, 0.10586E+01_JPRB, 0.10450E+01_JPRB, 0.10465E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10621E+01_JPRB, 0.10498E+01_JPRB, 0.10104E+01_JPRB, 0.97795E+00_JPRB, 0.94779E+00_JPRB, 0.96267E+00_JPRB, 0.90139E+00_JPRB ,&
&  0.87512E+00_JPRB, 0.86505E+00_JPRB, 0.86492E+00_JPRB, 0.87194E+00_JPRB, 0.87153E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.86551E+00_JPRB, 0.85143E+00_JPRB, 0.84872E+00_JPRB, 0.84416E+00_JPRB, 0.83124E+00_JPRB, 0.81530E+00_JPRB, 0.81161E+00_JPRB ,&
&  0.80717E+00_JPRB, 0.80273E+00_JPRB, 0.79167E+00_JPRB, 0.77464E+00_JPRB, 0.77382E+00_JPRB, 0.77374E+00_JPRB, 0.77184E+00_JPRB ,&
&  0.76802E+00_JPRB, 0.76598E+00_JPRB, 0.75415E+00_JPRB, 0.74778E+00_JPRB, 0.74926E+00_JPRB, 0.74475E+00_JPRB, 0.73360E+00_JPRB ,&
&  0.72250E+00_JPRB, 0.71895E+00_JPRB, 0.72815E+00_JPRB, 0.73868E+00_JPRB, 0.76118E+00_JPRB, 0.77192E+00_JPRB, 0.79884E+00_JPRB ,&
&  0.96834E+00_JPRB, 0.94973E+00_JPRB, 0.22120E+01_JPRB, 0.29188E+01_JPRB, 0.28227E+01_JPRB, 0.24094E+01_JPRB, 0.21210E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.18597E+01_JPRB, 0.17516E+01_JPRB, 0.16040E+01_JPRB, 0.14493E+01_JPRB, 0.12872E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.11305E+01_JPRB, 0.11149E+01_JPRB, 0.11194E+01_JPRB, 0.11439E+01_JPRB, 0.11924E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.14798E+01_JPRB, 0.17149E+01_JPRB, 0.23719E+01_JPRB, 0.34154E+01_JPRB, 0.42484E+01_JPRB, 0.59929E+01_JPRB ,&
&  0.75331E+01_JPRB, 0.69184E+01_JPRB, 0.47802E+01_JPRB, 0.30972E+01_JPRB, 0.23492E+01_JPRB, 0.20505E+01_JPRB, 0.18234E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.14551E+01_JPRB, 0.13316E+01_JPRB, 0.12443E+01_JPRB, 0.11876E+01_JPRB, 0.11705E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.13058E+01_JPRB, 0.12707E+01_JPRB, 0.12253E+01_JPRB, 0.11884E+01_JPRB, 0.11603E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11781E+01_JPRB, 0.11396E+01_JPRB, 0.10957E+01_JPRB, 0.10826E+01_JPRB, 0.10927E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10485E+01_JPRB, 0.10049E+01_JPRB, 0.98276E+00_JPRB, 0.92807E+00_JPRB, 0.94914E+00_JPRB, 0.89283E+00_JPRB ,&
&  0.86310E+00_JPRB, 0.85632E+00_JPRB, 0.86292E+00_JPRB, 0.86929E+00_JPRB, 0.88544E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.87402E+00_JPRB, 0.85071E+00_JPRB, 0.84434E+00_JPRB, 0.84978E+00_JPRB, 0.84820E+00_JPRB, 0.85299E+00_JPRB, 0.85562E+00_JPRB ,&
&  0.84757E+00_JPRB, 0.82605E+00_JPRB, 0.80778E+00_JPRB, 0.78881E+00_JPRB, 0.78244E+00_JPRB, 0.77512E+00_JPRB, 0.77386E+00_JPRB ,&
&  0.76583E+00_JPRB, 0.76014E+00_JPRB, 0.75058E+00_JPRB, 0.74777E+00_JPRB, 0.74753E+00_JPRB, 0.74134E+00_JPRB, 0.71905E+00_JPRB ,&
&  0.71141E+00_JPRB, 0.71132E+00_JPRB, 0.72327E+00_JPRB, 0.73815E+00_JPRB, 0.75707E+00_JPRB, 0.76610E+00_JPRB, 0.78302E+00_JPRB ,&
&  0.93918E+00_JPRB, 0.14245E+01_JPRB, 0.22368E+01_JPRB, 0.23757E+01_JPRB, 0.23646E+01_JPRB, 0.21464E+01_JPRB, 0.19622E+01_JPRB ,&
&  0.17961E+01_JPRB, 0.17244E+01_JPRB, 0.17120E+01_JPRB, 0.15605E+01_JPRB, 0.13385E+01_JPRB, 0.11651E+01_JPRB, 0.11018E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10481E+01_JPRB, 0.10381E+01_JPRB, 0.10424E+01_JPRB, 0.10456E+01_JPRB, 0.10531E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.11648E+01_JPRB, 0.12556E+01_JPRB, 0.15854E+01_JPRB, 0.27840E+01_JPRB, 0.32356E+01_JPRB, 0.51181E+01_JPRB ,&
&  0.75450E+01_JPRB, 0.66840E+01_JPRB, 0.50517E+01_JPRB, 0.35858E+01_JPRB, 0.27640E+01_JPRB, 0.23643E+01_JPRB, 0.20629E+01_JPRB ,&
&  0.18261E+01_JPRB, 0.16280E+01_JPRB, 0.14600E+01_JPRB, 0.13549E+01_JPRB, 0.12666E+01_JPRB, 0.12051E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.12538E+01_JPRB, 0.12827E+01_JPRB, 0.12634E+01_JPRB, 0.12262E+01_JPRB, 0.11947E+01_JPRB, 0.11875E+01_JPRB, 0.11830E+01_JPRB ,&
&  0.11876E+01_JPRB, 0.11858E+01_JPRB, 0.11549E+01_JPRB, 0.11046E+01_JPRB, 0.10938E+01_JPRB, 0.11123E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.10099E+01_JPRB, 0.10060E+01_JPRB, 0.96886E+00_JPRB, 0.93402E+00_JPRB, 0.95220E+00_JPRB, 0.88948E+00_JPRB ,&
&  0.85316E+00_JPRB, 0.84502E+00_JPRB, 0.86268E+00_JPRB, 0.88352E+00_JPRB, 0.91530E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.89821E+00_JPRB, 0.85783E+00_JPRB, 0.83703E+00_JPRB, 0.83938E+00_JPRB, 0.84911E+00_JPRB, 0.84947E+00_JPRB, 0.85484E+00_JPRB ,&
&  0.84624E+00_JPRB, 0.82765E+00_JPRB, 0.80679E+00_JPRB, 0.78863E+00_JPRB, 0.77902E+00_JPRB, 0.77233E+00_JPRB, 0.76721E+00_JPRB ,&
&  0.76245E+00_JPRB, 0.75723E+00_JPRB, 0.74915E+00_JPRB, 0.75173E+00_JPRB, 0.75374E+00_JPRB, 0.75138E+00_JPRB, 0.72725E+00_JPRB ,&
&  0.71383E+00_JPRB, 0.71561E+00_JPRB, 0.72457E+00_JPRB, 0.73226E+00_JPRB, 0.74435E+00_JPRB, 0.75815E+00_JPRB, 0.77770E+00_JPRB ,&
&  0.86261E+00_JPRB, 0.13063E+01_JPRB, 0.17612E+01_JPRB, 0.20531E+01_JPRB, 0.21365E+01_JPRB, 0.20435E+01_JPRB, 0.18628E+01_JPRB ,&
&  0.17218E+01_JPRB, 0.16408E+01_JPRB, 0.15653E+01_JPRB, 0.13809E+01_JPRB, 0.11884E+01_JPRB, 0.10831E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.10020E+01_JPRB, 0.99507E+00_JPRB, 0.99055E+00_JPRB, 0.98550E+00_JPRB, 0.98826E+00_JPRB, 0.99723E+00_JPRB ,&
&  0.10105E+01_JPRB, 0.10255E+01_JPRB, 0.10582E+01_JPRB, 0.11237E+01_JPRB, 0.16463E+01_JPRB, 0.24533E+01_JPRB, 0.37117E+01_JPRB ,&
&  0.43337E+01_JPRB, 0.44262E+01_JPRB, 0.39916E+01_JPRB, 0.34238E+01_JPRB, 0.29363E+01_JPRB, 0.25564E+01_JPRB, 0.22285E+01_JPRB ,&
&  0.19333E+01_JPRB, 0.16754E+01_JPRB, 0.14674E+01_JPRB, 0.13356E+01_JPRB, 0.12544E+01_JPRB, 0.11907E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11755E+01_JPRB, 0.11978E+01_JPRB, 0.12059E+01_JPRB, 0.12045E+01_JPRB, 0.11870E+01_JPRB, 0.11675E+01_JPRB, 0.11553E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.11502E+01_JPRB, 0.11481E+01_JPRB, 0.11207E+01_JPRB, 0.10914E+01_JPRB, 0.10702E+01_JPRB, 0.10579E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10088E+01_JPRB, 0.98518E+00_JPRB, 0.96495E+00_JPRB, 0.93192E+00_JPRB, 0.94711E+00_JPRB, 0.88553E+00_JPRB ,&
&  0.85260E+00_JPRB, 0.84461E+00_JPRB, 0.84842E+00_JPRB, 0.86416E+00_JPRB, 0.90371E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.91580E+00_JPRB, 0.88626E+00_JPRB, 0.85704E+00_JPRB, 0.84090E+00_JPRB, 0.83625E+00_JPRB, 0.83602E+00_JPRB, 0.83658E+00_JPRB ,&
&  0.83344E+00_JPRB, 0.82568E+00_JPRB, 0.80443E+00_JPRB, 0.78740E+00_JPRB, 0.77500E+00_JPRB, 0.77021E+00_JPRB, 0.76597E+00_JPRB ,&
&  0.76369E+00_JPRB, 0.76046E+00_JPRB, 0.75389E+00_JPRB, 0.75214E+00_JPRB, 0.75342E+00_JPRB, 0.74764E+00_JPRB, 0.72864E+00_JPRB ,&
&  0.70851E+00_JPRB, 0.70896E+00_JPRB, 0.71688E+00_JPRB, 0.72413E+00_JPRB, 0.73563E+00_JPRB, 0.74765E+00_JPRB, 0.76919E+00_JPRB ,&
&  0.84642E+00_JPRB, 0.10712E+01_JPRB, 0.14993E+01_JPRB, 0.17381E+01_JPRB, 0.18425E+01_JPRB, 0.18572E+01_JPRB, 0.17464E+01_JPRB ,&
&  0.16060E+01_JPRB, 0.14548E+01_JPRB, 0.13322E+01_JPRB, 0.12022E+01_JPRB, 0.11110E+01_JPRB, 0.10428E+01_JPRB, 0.99698E+00_JPRB ,&
&  0.97326E+00_JPRB, 0.96534E+00_JPRB, 0.95832E+00_JPRB, 0.94927E+00_JPRB, 0.94324E+00_JPRB, 0.94079E+00_JPRB, 0.94136E+00_JPRB ,&
&  0.94640E+00_JPRB, 0.95816E+00_JPRB, 0.97508E+00_JPRB, 0.99731E+00_JPRB, 0.10737E+01_JPRB, 0.13732E+01_JPRB, 0.20453E+01_JPRB ,&
&  0.26507E+01_JPRB, 0.30001E+01_JPRB, 0.29521E+01_JPRB, 0.28307E+01_JPRB, 0.27184E+01_JPRB, 0.24774E+01_JPRB, 0.22205E+01_JPRB ,&
&  0.19661E+01_JPRB, 0.17312E+01_JPRB, 0.14940E+01_JPRB, 0.13062E+01_JPRB, 0.11875E+01_JPRB, 0.11202E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.11123E+01_JPRB, 0.11295E+01_JPRB, 0.11471E+01_JPRB, 0.11443E+01_JPRB, 0.11257E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10472E+01_JPRB, 0.10343E+01_JPRB, 0.10169E+01_JPRB, 0.99844E+00_JPRB, 0.98961E+00_JPRB, 0.98319E+00_JPRB ,&
&  0.97841E+00_JPRB, 0.98264E+00_JPRB, 0.94165E+00_JPRB, 0.92420E+00_JPRB, 0.98211E+00_JPRB, 0.94044E+00_JPRB, 0.89673E+00_JPRB ,&
&  0.86353E+00_JPRB, 0.84940E+00_JPRB, 0.84626E+00_JPRB, 0.86064E+00_JPRB, 0.90332E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.82174E+00_JPRB, 0.82139E+00_JPRB, 0.81973E+00_JPRB, 0.82242E+00_JPRB, 0.82364E+00_JPRB, 0.81600E+00_JPRB, 0.80993E+00_JPRB ,&
&  0.80870E+00_JPRB, 0.80446E+00_JPRB, 0.79817E+00_JPRB, 0.78751E+00_JPRB, 0.77334E+00_JPRB, 0.76241E+00_JPRB, 0.75890E+00_JPRB ,&
&  0.75320E+00_JPRB, 0.75043E+00_JPRB, 0.74907E+00_JPRB, 0.74393E+00_JPRB, 0.73759E+00_JPRB, 0.72800E+00_JPRB, 0.71164E+00_JPRB ,&
&  0.69046E+00_JPRB, 0.68303E+00_JPRB, 0.69266E+00_JPRB, 0.70248E+00_JPRB, 0.71030E+00_JPRB, 0.71500E+00_JPRB, 0.73226E+00_JPRB ,&
&  0.81721E+00_JPRB, 0.94815E+00_JPRB, 0.11291E+01_JPRB, 0.13588E+01_JPRB, 0.14938E+01_JPRB, 0.14608E+01_JPRB, 0.14308E+01_JPRB ,&
&  0.13639E+01_JPRB, 0.12913E+01_JPRB, 0.12175E+01_JPRB, 0.11345E+01_JPRB, 0.10627E+01_JPRB, 0.10083E+01_JPRB, 0.97711E+00_JPRB ,&
&  0.95588E+00_JPRB, 0.93761E+00_JPRB, 0.92493E+00_JPRB, 0.90875E+00_JPRB, 0.89630E+00_JPRB, 0.88763E+00_JPRB, 0.88853E+00_JPRB ,&
&  0.89288E+00_JPRB, 0.89999E+00_JPRB, 0.90878E+00_JPRB, 0.92020E+00_JPRB, 0.94278E+00_JPRB, 0.10261E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.17120E+01_JPRB, 0.20809E+01_JPRB, 0.22558E+01_JPRB, 0.22225E+01_JPRB, 0.21706E+01_JPRB, 0.20933E+01_JPRB, 0.19083E+01_JPRB ,&
&  0.16811E+01_JPRB, 0.15304E+01_JPRB, 0.14153E+01_JPRB, 0.13153E+01_JPRB, 0.12403E+01_JPRB, 0.11773E+01_JPRB, 0.11288E+01_JPRB ,&
&  0.11003E+01_JPRB, 0.10936E+01_JPRB, 0.10920E+01_JPRB, 0.10905E+01_JPRB, 0.10805E+01_JPRB, 0.10518E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.95624E+00_JPRB, 0.92917E+00_JPRB, 0.92318E+00_JPRB, 0.90930E+00_JPRB, 0.89703E+00_JPRB, 0.89812E+00_JPRB, 0.90828E+00_JPRB ,&
&  0.91119E+00_JPRB, 0.91914E+00_JPRB, 0.92522E+00_JPRB, 0.94920E+00_JPRB, 0.94182E+00_JPRB, 0.91700E+00_JPRB, 0.90113E+00_JPRB ,&
&  0.87527E+00_JPRB, 0.84979E+00_JPRB, 0.83138E+00_JPRB, 0.82930E+00_JPRB, 0.78376E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.80774E+00_JPRB, 0.80503E+00_JPRB, 0.79942E+00_JPRB, 0.79982E+00_JPRB, 0.80258E+00_JPRB, 0.79093E+00_JPRB, 0.77952E+00_JPRB ,&
&  0.77347E+00_JPRB, 0.77429E+00_JPRB, 0.77728E+00_JPRB, 0.77746E+00_JPRB, 0.76992E+00_JPRB, 0.75662E+00_JPRB, 0.74803E+00_JPRB ,&
&  0.73687E+00_JPRB, 0.72749E+00_JPRB, 0.72472E+00_JPRB, 0.72322E+00_JPRB, 0.71657E+00_JPRB, 0.70849E+00_JPRB, 0.70140E+00_JPRB ,&
&  0.68613E+00_JPRB, 0.67514E+00_JPRB, 0.67284E+00_JPRB, 0.67703E+00_JPRB, 0.68227E+00_JPRB, 0.68291E+00_JPRB, 0.70796E+00_JPRB ,&
&  0.80658E+00_JPRB, 0.80981E+00_JPRB, 0.82124E+00_JPRB, 0.99529E+00_JPRB, 0.11331E+01_JPRB, 0.11555E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.12114E+01_JPRB, 0.12027E+01_JPRB, 0.11656E+01_JPRB, 0.10885E+01_JPRB, 0.10248E+01_JPRB, 0.97547E+00_JPRB, 0.93104E+00_JPRB ,&
&  0.89401E+00_JPRB, 0.86210E+00_JPRB, 0.84281E+00_JPRB, 0.82559E+00_JPRB, 0.81701E+00_JPRB, 0.80371E+00_JPRB, 0.79864E+00_JPRB ,&
&  0.80029E+00_JPRB, 0.80973E+00_JPRB, 0.82110E+00_JPRB, 0.83118E+00_JPRB, 0.84970E+00_JPRB, 0.88626E+00_JPRB, 0.99551E+00_JPRB ,&
&  0.12034E+01_JPRB, 0.14944E+01_JPRB, 0.17074E+01_JPRB, 0.17784E+01_JPRB, 0.17532E+01_JPRB, 0.16715E+01_JPRB, 0.15506E+01_JPRB ,&
&  0.14192E+01_JPRB, 0.13135E+01_JPRB, 0.12377E+01_JPRB, 0.11965E+01_JPRB, 0.11649E+01_JPRB, 0.11383E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.10696E+01_JPRB, 0.10563E+01_JPRB, 0.10398E+01_JPRB, 0.10175E+01_JPRB, 0.97522E+00_JPRB, 0.91620E+00_JPRB ,&
&  0.86966E+00_JPRB, 0.83814E+00_JPRB, 0.82572E+00_JPRB, 0.81611E+00_JPRB, 0.81175E+00_JPRB, 0.82370E+00_JPRB, 0.84096E+00_JPRB ,&
&  0.85019E+00_JPRB, 0.86095E+00_JPRB, 0.88261E+00_JPRB, 0.85714E+00_JPRB, 0.89337E+00_JPRB, 0.89221E+00_JPRB, 0.89250E+00_JPRB ,&
&  0.88039E+00_JPRB, 0.86156E+00_JPRB, 0.84348E+00_JPRB, 0.72932E+00_JPRB, 0.78982E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.79482E+00_JPRB, 0.79094E+00_JPRB, 0.77812E+00_JPRB, 0.76972E+00_JPRB, 0.76397E+00_JPRB, 0.75680E+00_JPRB, 0.74493E+00_JPRB ,&
&  0.73805E+00_JPRB, 0.73684E+00_JPRB, 0.73619E+00_JPRB, 0.73667E+00_JPRB, 0.73627E+00_JPRB, 0.72943E+00_JPRB, 0.71908E+00_JPRB ,&
&  0.70891E+00_JPRB, 0.70011E+00_JPRB, 0.70131E+00_JPRB, 0.69993E+00_JPRB, 0.69376E+00_JPRB, 0.68865E+00_JPRB, 0.68231E+00_JPRB ,&
&  0.66718E+00_JPRB, 0.65116E+00_JPRB, 0.63887E+00_JPRB, 0.64220E+00_JPRB, 0.64927E+00_JPRB, 0.64521E+00_JPRB, 0.66730E+00_JPRB ,&
&  0.76286E+00_JPRB, 0.71370E+00_JPRB, 0.74640E+00_JPRB, 0.84503E+00_JPRB, 0.90335E+00_JPRB, 0.95836E+00_JPRB, 0.99851E+00_JPRB ,&
&  0.10304E+01_JPRB, 0.10512E+01_JPRB, 0.99773E+00_JPRB, 0.93229E+00_JPRB, 0.86964E+00_JPRB, 0.81672E+00_JPRB, 0.77875E+00_JPRB ,&
&  0.75513E+00_JPRB, 0.74526E+00_JPRB, 0.73882E+00_JPRB, 0.72857E+00_JPRB, 0.72046E+00_JPRB, 0.71215E+00_JPRB, 0.70757E+00_JPRB ,&
&  0.70584E+00_JPRB, 0.70901E+00_JPRB, 0.71649E+00_JPRB, 0.73273E+00_JPRB, 0.75079E+00_JPRB, 0.77159E+00_JPRB, 0.80894E+00_JPRB ,&
&  0.88874E+00_JPRB, 0.10416E+01_JPRB, 0.12142E+01_JPRB, 0.13118E+01_JPRB, 0.13208E+01_JPRB, 0.12732E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.11177E+01_JPRB, 0.10619E+01_JPRB, 0.10281E+01_JPRB, 0.10061E+01_JPRB, 0.98490E+00_JPRB, 0.96683E+00_JPRB, 0.95246E+00_JPRB ,&
&  0.94030E+00_JPRB, 0.92510E+00_JPRB, 0.91503E+00_JPRB, 0.89972E+00_JPRB, 0.87488E+00_JPRB, 0.84014E+00_JPRB, 0.80554E+00_JPRB ,&
&  0.77262E+00_JPRB, 0.74673E+00_JPRB, 0.73178E+00_JPRB, 0.73011E+00_JPRB, 0.73307E+00_JPRB, 0.74594E+00_JPRB, 0.76601E+00_JPRB ,&
&  0.78177E+00_JPRB, 0.79354E+00_JPRB, 0.80567E+00_JPRB, 0.82939E+00_JPRB, 0.84504E+00_JPRB, 0.86126E+00_JPRB, 0.87032E+00_JPRB ,&
&  0.87130E+00_JPRB, 0.87044E+00_JPRB, 0.70403E+00_JPRB, 0.77028E+00_JPRB, 0.78672E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.79131E+00_JPRB, 0.77234E+00_JPRB, 0.74985E+00_JPRB, 0.73414E+00_JPRB, 0.72583E+00_JPRB, 0.72229E+00_JPRB, 0.71755E+00_JPRB ,&
&  0.71094E+00_JPRB, 0.71033E+00_JPRB, 0.71112E+00_JPRB, 0.71173E+00_JPRB, 0.71151E+00_JPRB, 0.70699E+00_JPRB, 0.69754E+00_JPRB ,&
&  0.68687E+00_JPRB, 0.67429E+00_JPRB, 0.67210E+00_JPRB, 0.67105E+00_JPRB, 0.66427E+00_JPRB, 0.65932E+00_JPRB, 0.65828E+00_JPRB ,&
&  0.65142E+00_JPRB, 0.64054E+00_JPRB, 0.62128E+00_JPRB, 0.61550E+00_JPRB, 0.61874E+00_JPRB, 0.61921E+00_JPRB, 0.62862E+00_JPRB ,&
&  0.63054E+00_JPRB, 0.62636E+00_JPRB, 0.69055E+00_JPRB, 0.73509E+00_JPRB, 0.76081E+00_JPRB, 0.78623E+00_JPRB, 0.81326E+00_JPRB ,&
&  0.80798E+00_JPRB, 0.80847E+00_JPRB, 0.78131E+00_JPRB, 0.73887E+00_JPRB, 0.70879E+00_JPRB, 0.68983E+00_JPRB, 0.67880E+00_JPRB ,&
&  0.67746E+00_JPRB, 0.67646E+00_JPRB, 0.67347E+00_JPRB, 0.67005E+00_JPRB, 0.66876E+00_JPRB, 0.66381E+00_JPRB, 0.66094E+00_JPRB ,&
&  0.66235E+00_JPRB, 0.66636E+00_JPRB, 0.66890E+00_JPRB, 0.67424E+00_JPRB, 0.68536E+00_JPRB, 0.70232E+00_JPRB, 0.72361E+00_JPRB ,&
&  0.75090E+00_JPRB, 0.79976E+00_JPRB, 0.88355E+00_JPRB, 0.95660E+00_JPRB, 0.98243E+00_JPRB, 0.96148E+00_JPRB, 0.91972E+00_JPRB ,&
&  0.88486E+00_JPRB, 0.86426E+00_JPRB, 0.85052E+00_JPRB, 0.83759E+00_JPRB, 0.81970E+00_JPRB, 0.80293E+00_JPRB, 0.78741E+00_JPRB ,&
&  0.78073E+00_JPRB, 0.77000E+00_JPRB, 0.75744E+00_JPRB, 0.74163E+00_JPRB, 0.72647E+00_JPRB, 0.71625E+00_JPRB, 0.70879E+00_JPRB ,&
&  0.70077E+00_JPRB, 0.68939E+00_JPRB, 0.68207E+00_JPRB, 0.68297E+00_JPRB, 0.68550E+00_JPRB, 0.69261E+00_JPRB, 0.70457E+00_JPRB ,&
&  0.71978E+00_JPRB, 0.73375E+00_JPRB, 0.74850E+00_JPRB, 0.77528E+00_JPRB, 0.80229E+00_JPRB, 0.82638E+00_JPRB, 0.84156E+00_JPRB ,&
&  0.85125E+00_JPRB, 0.84493E+00_JPRB, 0.83978E+00_JPRB, 0.82494E+00_JPRB, 0.80670E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.72908E+00_JPRB, 0.70663E+00_JPRB, 0.68886E+00_JPRB, 0.67699E+00_JPRB, 0.67008E+00_JPRB, 0.66842E+00_JPRB, 0.66511E+00_JPRB ,&
&  0.66246E+00_JPRB, 0.65833E+00_JPRB, 0.65580E+00_JPRB, 0.65836E+00_JPRB, 0.65963E+00_JPRB, 0.65990E+00_JPRB, 0.66031E+00_JPRB ,&
&  0.65369E+00_JPRB, 0.63976E+00_JPRB, 0.62878E+00_JPRB, 0.62547E+00_JPRB, 0.62589E+00_JPRB, 0.62136E+00_JPRB, 0.61958E+00_JPRB ,&
&  0.62260E+00_JPRB, 0.61445E+00_JPRB, 0.60067E+00_JPRB, 0.59175E+00_JPRB, 0.59249E+00_JPRB, 0.59022E+00_JPRB, 0.58911E+00_JPRB ,&
&  0.62165E+00_JPRB, 0.57734E+00_JPRB, 0.63270E+00_JPRB, 0.64334E+00_JPRB, 0.65292E+00_JPRB, 0.65973E+00_JPRB, 0.66169E+00_JPRB ,&
&  0.65781E+00_JPRB, 0.64948E+00_JPRB, 0.64056E+00_JPRB, 0.63091E+00_JPRB, 0.62309E+00_JPRB, 0.61464E+00_JPRB, 0.60784E+00_JPRB ,&
&  0.60648E+00_JPRB, 0.60539E+00_JPRB, 0.60808E+00_JPRB, 0.61264E+00_JPRB, 0.61983E+00_JPRB, 0.62216E+00_JPRB, 0.62423E+00_JPRB ,&
&  0.62687E+00_JPRB, 0.62849E+00_JPRB, 0.62830E+00_JPRB, 0.62709E+00_JPRB, 0.62568E+00_JPRB, 0.63028E+00_JPRB, 0.63792E+00_JPRB ,&
&  0.64767E+00_JPRB, 0.66897E+00_JPRB, 0.70442E+00_JPRB, 0.73835E+00_JPRB, 0.75342E+00_JPRB, 0.74854E+00_JPRB, 0.73206E+00_JPRB ,&
&  0.72375E+00_JPRB, 0.72575E+00_JPRB, 0.72901E+00_JPRB, 0.72466E+00_JPRB, 0.71622E+00_JPRB, 0.70809E+00_JPRB, 0.69435E+00_JPRB ,&
&  0.67633E+00_JPRB, 0.66028E+00_JPRB, 0.64881E+00_JPRB, 0.64148E+00_JPRB, 0.63790E+00_JPRB, 0.63362E+00_JPRB, 0.62917E+00_JPRB ,&
&  0.62746E+00_JPRB, 0.62494E+00_JPRB, 0.62488E+00_JPRB, 0.62953E+00_JPRB, 0.63398E+00_JPRB, 0.64018E+00_JPRB, 0.64801E+00_JPRB ,&
&  0.65894E+00_JPRB, 0.67088E+00_JPRB, 0.68268E+00_JPRB, 0.70168E+00_JPRB, 0.72880E+00_JPRB, 0.74829E+00_JPRB, 0.76122E+00_JPRB ,&
&  0.77324E+00_JPRB, 0.77653E+00_JPRB, 0.77043E+00_JPRB, 0.76044E+00_JPRB, 0.74629E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.67744E+00_JPRB, 0.66381E+00_JPRB, 0.65312E+00_JPRB, 0.64479E+00_JPRB, 0.63918E+00_JPRB, 0.63766E+00_JPRB, 0.63517E+00_JPRB ,&
&  0.63266E+00_JPRB, 0.63023E+00_JPRB, 0.62307E+00_JPRB, 0.61628E+00_JPRB, 0.61606E+00_JPRB, 0.62212E+00_JPRB, 0.62663E+00_JPRB ,&
&  0.62318E+00_JPRB, 0.61184E+00_JPRB, 0.60128E+00_JPRB, 0.59874E+00_JPRB, 0.60123E+00_JPRB, 0.60332E+00_JPRB, 0.60273E+00_JPRB ,&
&  0.60435E+00_JPRB, 0.60553E+00_JPRB, 0.59725E+00_JPRB, 0.57988E+00_JPRB, 0.57207E+00_JPRB, 0.56769E+00_JPRB, 0.57351E+00_JPRB ,&
&  0.58676E+00_JPRB, 0.59194E+00_JPRB, 0.60239E+00_JPRB, 0.59877E+00_JPRB, 0.59780E+00_JPRB, 0.59466E+00_JPRB, 0.58807E+00_JPRB ,&
&  0.58233E+00_JPRB, 0.57611E+00_JPRB, 0.57161E+00_JPRB, 0.56883E+00_JPRB, 0.56786E+00_JPRB, 0.56791E+00_JPRB, 0.57082E+00_JPRB ,&
&  0.57027E+00_JPRB, 0.56724E+00_JPRB, 0.56917E+00_JPRB, 0.57594E+00_JPRB, 0.58474E+00_JPRB, 0.58996E+00_JPRB, 0.59475E+00_JPRB ,&
&  0.59867E+00_JPRB, 0.60083E+00_JPRB, 0.60361E+00_JPRB, 0.60411E+00_JPRB, 0.60006E+00_JPRB, 0.59904E+00_JPRB, 0.59875E+00_JPRB ,&
&  0.60153E+00_JPRB, 0.60895E+00_JPRB, 0.62568E+00_JPRB, 0.64654E+00_JPRB, 0.65520E+00_JPRB, 0.64889E+00_JPRB, 0.63983E+00_JPRB ,&
&  0.63929E+00_JPRB, 0.64700E+00_JPRB, 0.65580E+00_JPRB, 0.66149E+00_JPRB, 0.66190E+00_JPRB, 0.65388E+00_JPRB, 0.64279E+00_JPRB ,&
&  0.63042E+00_JPRB, 0.61770E+00_JPRB, 0.60605E+00_JPRB, 0.59823E+00_JPRB, 0.59178E+00_JPRB, 0.58660E+00_JPRB, 0.58324E+00_JPRB ,&
&  0.58168E+00_JPRB, 0.58142E+00_JPRB, 0.58459E+00_JPRB, 0.59151E+00_JPRB, 0.59626E+00_JPRB, 0.60557E+00_JPRB, 0.61668E+00_JPRB ,&
&  0.62782E+00_JPRB, 0.63800E+00_JPRB, 0.64993E+00_JPRB, 0.66700E+00_JPRB, 0.68641E+00_JPRB, 0.70054E+00_JPRB, 0.71170E+00_JPRB ,&
&  0.71882E+00_JPRB, 0.71770E+00_JPRB, 0.70950E+00_JPRB, 0.70017E+00_JPRB, 0.69015E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.62896E+00_JPRB, 0.62106E+00_JPRB, 0.61460E+00_JPRB, 0.60892E+00_JPRB, 0.60409E+00_JPRB, 0.60221E+00_JPRB, 0.60026E+00_JPRB ,&
&  0.59834E+00_JPRB, 0.59556E+00_JPRB, 0.59367E+00_JPRB, 0.58975E+00_JPRB, 0.58668E+00_JPRB, 0.58977E+00_JPRB, 0.59559E+00_JPRB ,&
&  0.59409E+00_JPRB, 0.58683E+00_JPRB, 0.58113E+00_JPRB, 0.58090E+00_JPRB, 0.58416E+00_JPRB, 0.58654E+00_JPRB, 0.58914E+00_JPRB ,&
&  0.58871E+00_JPRB, 0.58782E+00_JPRB, 0.58576E+00_JPRB, 0.56956E+00_JPRB, 0.55453E+00_JPRB, 0.54721E+00_JPRB, 0.54327E+00_JPRB ,&
&  0.54462E+00_JPRB, 0.55572E+00_JPRB, 0.53792E+00_JPRB, 0.54738E+00_JPRB, 0.54305E+00_JPRB, 0.53645E+00_JPRB, 0.53250E+00_JPRB ,&
&  0.53027E+00_JPRB, 0.52800E+00_JPRB, 0.52616E+00_JPRB, 0.52780E+00_JPRB, 0.53004E+00_JPRB, 0.53041E+00_JPRB, 0.53168E+00_JPRB ,&
&  0.53105E+00_JPRB, 0.53330E+00_JPRB, 0.53547E+00_JPRB, 0.54063E+00_JPRB, 0.54450E+00_JPRB, 0.54696E+00_JPRB, 0.55025E+00_JPRB ,&
&  0.55488E+00_JPRB, 0.55920E+00_JPRB, 0.56330E+00_JPRB, 0.56675E+00_JPRB, 0.56812E+00_JPRB, 0.56777E+00_JPRB, 0.56344E+00_JPRB ,&
&  0.55999E+00_JPRB, 0.55993E+00_JPRB, 0.56535E+00_JPRB, 0.57811E+00_JPRB, 0.58898E+00_JPRB, 0.58928E+00_JPRB, 0.58584E+00_JPRB ,&
&  0.58577E+00_JPRB, 0.59082E+00_JPRB, 0.59356E+00_JPRB, 0.59414E+00_JPRB, 0.59337E+00_JPRB, 0.59132E+00_JPRB, 0.58679E+00_JPRB ,&
&  0.58001E+00_JPRB, 0.57382E+00_JPRB, 0.56726E+00_JPRB, 0.55497E+00_JPRB, 0.54582E+00_JPRB, 0.54474E+00_JPRB, 0.54630E+00_JPRB ,&
&  0.54572E+00_JPRB, 0.54415E+00_JPRB, 0.54540E+00_JPRB, 0.55128E+00_JPRB, 0.55225E+00_JPRB, 0.56048E+00_JPRB, 0.57557E+00_JPRB ,&
&  0.59057E+00_JPRB, 0.60343E+00_JPRB, 0.61343E+00_JPRB, 0.62367E+00_JPRB, 0.63372E+00_JPRB, 0.64125E+00_JPRB, 0.64755E+00_JPRB ,&
&  0.65030E+00_JPRB, 0.64825E+00_JPRB, 0.64259E+00_JPRB, 0.63925E+00_JPRB, 0.63638E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.60044E+00_JPRB, 0.59868E+00_JPRB, 0.59668E+00_JPRB, 0.59407E+00_JPRB, 0.59041E+00_JPRB, 0.58717E+00_JPRB, 0.58224E+00_JPRB ,&
&  0.57733E+00_JPRB, 0.57351E+00_JPRB, 0.57189E+00_JPRB, 0.57126E+00_JPRB, 0.57257E+00_JPRB, 0.57690E+00_JPRB, 0.58402E+00_JPRB ,&
&  0.58406E+00_JPRB, 0.57690E+00_JPRB, 0.57338E+00_JPRB, 0.57331E+00_JPRB, 0.57563E+00_JPRB, 0.57727E+00_JPRB, 0.57975E+00_JPRB ,&
&  0.58122E+00_JPRB, 0.58150E+00_JPRB, 0.58018E+00_JPRB, 0.57234E+00_JPRB, 0.55502E+00_JPRB, 0.54476E+00_JPRB, 0.53598E+00_JPRB ,&
&  0.53112E+00_JPRB, 0.52807E+00_JPRB, 0.52450E+00_JPRB, 0.52337E+00_JPRB, 0.52209E+00_JPRB, 0.52020E+00_JPRB, 0.52017E+00_JPRB ,&
&  0.51836E+00_JPRB, 0.51601E+00_JPRB, 0.51489E+00_JPRB, 0.51559E+00_JPRB, 0.51745E+00_JPRB, 0.51902E+00_JPRB, 0.51875E+00_JPRB ,&
&  0.51951E+00_JPRB, 0.52299E+00_JPRB, 0.52369E+00_JPRB, 0.52351E+00_JPRB, 0.52300E+00_JPRB, 0.52227E+00_JPRB, 0.52216E+00_JPRB ,&
&  0.52449E+00_JPRB, 0.52979E+00_JPRB, 0.53449E+00_JPRB, 0.53935E+00_JPRB, 0.54162E+00_JPRB, 0.54271E+00_JPRB, 0.54263E+00_JPRB ,&
&  0.53923E+00_JPRB, 0.53491E+00_JPRB, 0.53332E+00_JPRB, 0.53661E+00_JPRB, 0.54342E+00_JPRB, 0.54952E+00_JPRB, 0.55181E+00_JPRB ,&
&  0.55355E+00_JPRB, 0.55471E+00_JPRB, 0.55313E+00_JPRB, 0.55279E+00_JPRB, 0.55096E+00_JPRB, 0.54709E+00_JPRB, 0.54494E+00_JPRB ,&
&  0.54268E+00_JPRB, 0.54031E+00_JPRB, 0.53779E+00_JPRB, 0.52851E+00_JPRB, 0.52104E+00_JPRB, 0.51695E+00_JPRB, 0.51444E+00_JPRB ,&
&  0.51380E+00_JPRB, 0.51314E+00_JPRB, 0.51306E+00_JPRB, 0.51425E+00_JPRB, 0.51655E+00_JPRB, 0.52428E+00_JPRB, 0.53907E+00_JPRB ,&
&  0.56011E+00_JPRB, 0.57934E+00_JPRB, 0.58955E+00_JPRB, 0.59652E+00_JPRB, 0.60071E+00_JPRB, 0.60263E+00_JPRB, 0.60459E+00_JPRB ,&
&  0.60787E+00_JPRB, 0.60890E+00_JPRB, 0.60670E+00_JPRB, 0.60406E+00_JPRB, 0.60259E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.57064E+00_JPRB, 0.57068E+00_JPRB, 0.56895E+00_JPRB, 0.56721E+00_JPRB, 0.56537E+00_JPRB, 0.56376E+00_JPRB, 0.56157E+00_JPRB ,&
&  0.55668E+00_JPRB, 0.55034E+00_JPRB, 0.54666E+00_JPRB, 0.54523E+00_JPRB, 0.55121E+00_JPRB, 0.55980E+00_JPRB, 0.56741E+00_JPRB ,&
&  0.56631E+00_JPRB, 0.55673E+00_JPRB, 0.55367E+00_JPRB, 0.55553E+00_JPRB, 0.55802E+00_JPRB, 0.56074E+00_JPRB, 0.56435E+00_JPRB ,&
&  0.56835E+00_JPRB, 0.57235E+00_JPRB, 0.57411E+00_JPRB, 0.57109E+00_JPRB, 0.56395E+00_JPRB, 0.54979E+00_JPRB, 0.53766E+00_JPRB ,&
&  0.52975E+00_JPRB, 0.52367E+00_JPRB, 0.51955E+00_JPRB, 0.51831E+00_JPRB, 0.51595E+00_JPRB, 0.51216E+00_JPRB, 0.50697E+00_JPRB ,&
&  0.50265E+00_JPRB, 0.50083E+00_JPRB, 0.50350E+00_JPRB, 0.50564E+00_JPRB, 0.50666E+00_JPRB, 0.50617E+00_JPRB, 0.50282E+00_JPRB ,&
&  0.50069E+00_JPRB, 0.49968E+00_JPRB, 0.49881E+00_JPRB, 0.49976E+00_JPRB, 0.49676E+00_JPRB, 0.49430E+00_JPRB, 0.49601E+00_JPRB ,&
&  0.50041E+00_JPRB, 0.50450E+00_JPRB, 0.50842E+00_JPRB, 0.51174E+00_JPRB, 0.51341E+00_JPRB, 0.51585E+00_JPRB, 0.51718E+00_JPRB ,&
&  0.51595E+00_JPRB, 0.51187E+00_JPRB, 0.50806E+00_JPRB, 0.50822E+00_JPRB, 0.51027E+00_JPRB, 0.51269E+00_JPRB, 0.51595E+00_JPRB ,&
&  0.51633E+00_JPRB, 0.51115E+00_JPRB, 0.50608E+00_JPRB, 0.50173E+00_JPRB, 0.49785E+00_JPRB, 0.49506E+00_JPRB, 0.49388E+00_JPRB ,&
&  0.49381E+00_JPRB, 0.49072E+00_JPRB, 0.48467E+00_JPRB, 0.47977E+00_JPRB, 0.47958E+00_JPRB, 0.47972E+00_JPRB, 0.47664E+00_JPRB ,&
&  0.47778E+00_JPRB, 0.48089E+00_JPRB, 0.48001E+00_JPRB, 0.47811E+00_JPRB, 0.48009E+00_JPRB, 0.48595E+00_JPRB, 0.49824E+00_JPRB ,&
&  0.51453E+00_JPRB, 0.53297E+00_JPRB, 0.54982E+00_JPRB, 0.56385E+00_JPRB, 0.57187E+00_JPRB, 0.57360E+00_JPRB, 0.57169E+00_JPRB ,&
&  0.57104E+00_JPRB, 0.57199E+00_JPRB, 0.57126E+00_JPRB, 0.56977E+00_JPRB, 0.56986E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.54052E+00_JPRB, 0.54023E+00_JPRB, 0.54020E+00_JPRB, 0.54117E+00_JPRB, 0.54181E+00_JPRB, 0.54065E+00_JPRB, 0.54024E+00_JPRB ,&
&  0.53698E+00_JPRB, 0.53231E+00_JPRB, 0.52967E+00_JPRB, 0.53166E+00_JPRB, 0.54142E+00_JPRB, 0.55343E+00_JPRB, 0.56040E+00_JPRB ,&
&  0.55542E+00_JPRB, 0.54237E+00_JPRB, 0.53916E+00_JPRB, 0.54145E+00_JPRB, 0.54372E+00_JPRB, 0.54783E+00_JPRB, 0.55386E+00_JPRB ,&
&  0.55939E+00_JPRB, 0.56313E+00_JPRB, 0.56728E+00_JPRB, 0.56895E+00_JPRB, 0.56890E+00_JPRB, 0.56146E+00_JPRB, 0.55003E+00_JPRB ,&
&  0.53910E+00_JPRB, 0.53106E+00_JPRB, 0.52551E+00_JPRB, 0.52169E+00_JPRB, 0.51649E+00_JPRB, 0.51147E+00_JPRB, 0.50503E+00_JPRB ,&
&  0.50214E+00_JPRB, 0.50221E+00_JPRB, 0.50348E+00_JPRB, 0.50170E+00_JPRB, 0.49898E+00_JPRB, 0.49329E+00_JPRB, 0.48829E+00_JPRB ,&
&  0.48592E+00_JPRB, 0.48649E+00_JPRB, 0.48701E+00_JPRB, 0.48532E+00_JPRB, 0.48109E+00_JPRB, 0.47791E+00_JPRB, 0.47877E+00_JPRB ,&
&  0.48439E+00_JPRB, 0.48406E+00_JPRB, 0.48305E+00_JPRB, 0.48422E+00_JPRB, 0.48682E+00_JPRB, 0.48900E+00_JPRB, 0.48832E+00_JPRB ,&
&  0.48739E+00_JPRB, 0.48690E+00_JPRB, 0.48668E+00_JPRB, 0.48813E+00_JPRB, 0.48792E+00_JPRB, 0.48548E+00_JPRB, 0.47963E+00_JPRB ,&
&  0.47091E+00_JPRB, 0.46014E+00_JPRB, 0.45577E+00_JPRB, 0.45265E+00_JPRB, 0.44971E+00_JPRB, 0.44935E+00_JPRB, 0.45111E+00_JPRB ,&
&  0.44995E+00_JPRB, 0.44383E+00_JPRB, 0.44119E+00_JPRB, 0.44220E+00_JPRB, 0.44218E+00_JPRB, 0.44543E+00_JPRB, 0.45033E+00_JPRB ,&
&  0.46181E+00_JPRB, 0.46846E+00_JPRB, 0.47047E+00_JPRB, 0.46826E+00_JPRB, 0.46845E+00_JPRB, 0.47024E+00_JPRB, 0.47492E+00_JPRB ,&
&  0.48191E+00_JPRB, 0.48957E+00_JPRB, 0.49762E+00_JPRB, 0.50652E+00_JPRB, 0.51726E+00_JPRB, 0.52746E+00_JPRB, 0.53150E+00_JPRB ,&
&  0.53200E+00_JPRB, 0.53272E+00_JPRB, 0.53435E+00_JPRB, 0.53620E+00_JPRB, 0.53928E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.50391E+00_JPRB, 0.50614E+00_JPRB, 0.50847E+00_JPRB, 0.51011E+00_JPRB, 0.51292E+00_JPRB, 0.51797E+00_JPRB, 0.51999E+00_JPRB ,&
&  0.51791E+00_JPRB, 0.51556E+00_JPRB, 0.51547E+00_JPRB, 0.51925E+00_JPRB, 0.52661E+00_JPRB, 0.53408E+00_JPRB, 0.54189E+00_JPRB ,&
&  0.53728E+00_JPRB, 0.52399E+00_JPRB, 0.51924E+00_JPRB, 0.52136E+00_JPRB, 0.52634E+00_JPRB, 0.53182E+00_JPRB, 0.53846E+00_JPRB ,&
&  0.54476E+00_JPRB, 0.54878E+00_JPRB, 0.55306E+00_JPRB, 0.55702E+00_JPRB, 0.55955E+00_JPRB, 0.55757E+00_JPRB, 0.55297E+00_JPRB ,&
&  0.54743E+00_JPRB, 0.54149E+00_JPRB, 0.53588E+00_JPRB, 0.52028E+00_JPRB, 0.51775E+00_JPRB, 0.50886E+00_JPRB, 0.50161E+00_JPRB ,&
&  0.49674E+00_JPRB, 0.49267E+00_JPRB, 0.48999E+00_JPRB, 0.48666E+00_JPRB, 0.48184E+00_JPRB, 0.47377E+00_JPRB, 0.47086E+00_JPRB ,&
&  0.46909E+00_JPRB, 0.46937E+00_JPRB, 0.46387E+00_JPRB, 0.46064E+00_JPRB, 0.45703E+00_JPRB, 0.45408E+00_JPRB, 0.45765E+00_JPRB ,&
&  0.46238E+00_JPRB, 0.45948E+00_JPRB, 0.45528E+00_JPRB, 0.45508E+00_JPRB, 0.45956E+00_JPRB, 0.46126E+00_JPRB, 0.46211E+00_JPRB ,&
&  0.46362E+00_JPRB, 0.46365E+00_JPRB, 0.45837E+00_JPRB, 0.45278E+00_JPRB, 0.44420E+00_JPRB, 0.43273E+00_JPRB, 0.42919E+00_JPRB ,&
&  0.42593E+00_JPRB, 0.42042E+00_JPRB, 0.41765E+00_JPRB, 0.41679E+00_JPRB, 0.41660E+00_JPRB, 0.41778E+00_JPRB, 0.41828E+00_JPRB ,&
&  0.41874E+00_JPRB, 0.41536E+00_JPRB, 0.41042E+00_JPRB, 0.40644E+00_JPRB, 0.40097E+00_JPRB, 0.40470E+00_JPRB, 0.41585E+00_JPRB ,&
&  0.42913E+00_JPRB, 0.43737E+00_JPRB, 0.44270E+00_JPRB, 0.44616E+00_JPRB, 0.44923E+00_JPRB, 0.45053E+00_JPRB, 0.45288E+00_JPRB ,&
&  0.45599E+00_JPRB, 0.45744E+00_JPRB, 0.45636E+00_JPRB, 0.45740E+00_JPRB, 0.46152E+00_JPRB, 0.46754E+00_JPRB, 0.47450E+00_JPRB ,&
&  0.48117E+00_JPRB, 0.48674E+00_JPRB, 0.49133E+00_JPRB, 0.49698E+00_JPRB, 0.50064E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.47358E+00_JPRB, 0.47811E+00_JPRB, 0.48251E+00_JPRB, 0.48729E+00_JPRB, 0.49399E+00_JPRB, 0.49766E+00_JPRB, 0.49893E+00_JPRB ,&
&  0.50021E+00_JPRB, 0.50058E+00_JPRB, 0.50185E+00_JPRB, 0.50615E+00_JPRB, 0.51270E+00_JPRB, 0.51770E+00_JPRB, 0.52390E+00_JPRB ,&
&  0.52393E+00_JPRB, 0.51458E+00_JPRB, 0.50638E+00_JPRB, 0.50614E+00_JPRB, 0.51188E+00_JPRB, 0.51780E+00_JPRB, 0.52508E+00_JPRB ,&
&  0.53220E+00_JPRB, 0.53724E+00_JPRB, 0.54000E+00_JPRB, 0.54137E+00_JPRB, 0.54318E+00_JPRB, 0.54250E+00_JPRB, 0.53926E+00_JPRB ,&
&  0.53462E+00_JPRB, 0.52470E+00_JPRB, 0.50511E+00_JPRB, 0.50257E+00_JPRB, 0.49381E+00_JPRB, 0.48627E+00_JPRB, 0.48108E+00_JPRB ,&
&  0.47696E+00_JPRB, 0.47339E+00_JPRB, 0.46915E+00_JPRB, 0.46517E+00_JPRB, 0.46052E+00_JPRB, 0.45259E+00_JPRB, 0.44814E+00_JPRB ,&
&  0.44405E+00_JPRB, 0.44073E+00_JPRB, 0.43730E+00_JPRB, 0.43678E+00_JPRB, 0.43643E+00_JPRB, 0.43854E+00_JPRB, 0.43937E+00_JPRB ,&
&  0.43566E+00_JPRB, 0.43141E+00_JPRB, 0.42822E+00_JPRB, 0.42978E+00_JPRB, 0.43222E+00_JPRB, 0.43390E+00_JPRB, 0.43447E+00_JPRB ,&
&  0.43152E+00_JPRB, 0.42612E+00_JPRB, 0.42043E+00_JPRB, 0.41411E+00_JPRB, 0.40685E+00_JPRB, 0.38116E+00_JPRB, 0.37038E+00_JPRB ,&
&  0.36817E+00_JPRB, 0.37776E+00_JPRB, 0.38935E+00_JPRB, 0.38893E+00_JPRB, 0.39252E+00_JPRB, 0.39309E+00_JPRB, 0.39053E+00_JPRB ,&
&  0.38066E+00_JPRB, 0.36751E+00_JPRB, 0.35953E+00_JPRB, 0.35620E+00_JPRB, 0.35391E+00_JPRB, 0.35665E+00_JPRB, 0.36392E+00_JPRB ,&
&  0.37517E+00_JPRB, 0.38717E+00_JPRB, 0.39610E+00_JPRB, 0.40009E+00_JPRB, 0.39870E+00_JPRB, 0.39560E+00_JPRB, 0.39150E+00_JPRB ,&
&  0.38758E+00_JPRB, 0.39440E+00_JPRB, 0.40994E+00_JPRB, 0.42971E+00_JPRB, 0.43247E+00_JPRB, 0.43830E+00_JPRB, 0.44232E+00_JPRB ,&
&  0.44807E+00_JPRB, 0.45530E+00_JPRB, 0.46125E+00_JPRB, 0.46721E+00_JPRB, 0.47073E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.44968E+00_JPRB, 0.45483E+00_JPRB, 0.46180E+00_JPRB, 0.46956E+00_JPRB, 0.47523E+00_JPRB, 0.47812E+00_JPRB, 0.48123E+00_JPRB ,&
&  0.48494E+00_JPRB, 0.48655E+00_JPRB, 0.48762E+00_JPRB, 0.49032E+00_JPRB, 0.49604E+00_JPRB, 0.49949E+00_JPRB, 0.50482E+00_JPRB ,&
&  0.50710E+00_JPRB, 0.50477E+00_JPRB, 0.49769E+00_JPRB, 0.49580E+00_JPRB, 0.49964E+00_JPRB, 0.50462E+00_JPRB, 0.50981E+00_JPRB ,&
&  0.51503E+00_JPRB, 0.51975E+00_JPRB, 0.52188E+00_JPRB, 0.52223E+00_JPRB, 0.52214E+00_JPRB, 0.52174E+00_JPRB, 0.51887E+00_JPRB ,&
&  0.51238E+00_JPRB, 0.47155E+00_JPRB, 0.44522E+00_JPRB, 0.45083E+00_JPRB, 0.46173E+00_JPRB, 0.45493E+00_JPRB, 0.45334E+00_JPRB ,&
&  0.45180E+00_JPRB, 0.44786E+00_JPRB, 0.44207E+00_JPRB, 0.43714E+00_JPRB, 0.43123E+00_JPRB, 0.42351E+00_JPRB, 0.41748E+00_JPRB ,&
&  0.41567E+00_JPRB, 0.42103E+00_JPRB, 0.42228E+00_JPRB, 0.42337E+00_JPRB, 0.42072E+00_JPRB, 0.42145E+00_JPRB, 0.41913E+00_JPRB ,&
&  0.41254E+00_JPRB, 0.40517E+00_JPRB, 0.40233E+00_JPRB, 0.40160E+00_JPRB, 0.40277E+00_JPRB, 0.40554E+00_JPRB, 0.40460E+00_JPRB ,&
&  0.39187E+00_JPRB, 0.38008E+00_JPRB, 0.36109E+00_JPRB, 0.34264E+00_JPRB, 0.32939E+00_JPRB, 0.32351E+00_JPRB, 0.31931E+00_JPRB ,&
&  0.31892E+00_JPRB, 0.32596E+00_JPRB, 0.34487E+00_JPRB, 0.36718E+00_JPRB, 0.37838E+00_JPRB, 0.35972E+00_JPRB, 0.34484E+00_JPRB ,&
&  0.33601E+00_JPRB, 0.33119E+00_JPRB, 0.32695E+00_JPRB, 0.32577E+00_JPRB, 0.32668E+00_JPRB, 0.32953E+00_JPRB, 0.33344E+00_JPRB ,&
&  0.33885E+00_JPRB, 0.34375E+00_JPRB, 0.34656E+00_JPRB, 0.34791E+00_JPRB, 0.34873E+00_JPRB, 0.34965E+00_JPRB, 0.34911E+00_JPRB ,&
&  0.34929E+00_JPRB, 0.35358E+00_JPRB, 0.36144E+00_JPRB, 0.37105E+00_JPRB, 0.37962E+00_JPRB, 0.38942E+00_JPRB, 0.40310E+00_JPRB ,&
&  0.41780E+00_JPRB, 0.42864E+00_JPRB, 0.43271E+00_JPRB, 0.43932E+00_JPRB, 0.44486E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.43846E+00_JPRB, 0.44587E+00_JPRB, 0.45068E+00_JPRB, 0.46036E+00_JPRB, 0.46348E+00_JPRB, 0.46697E+00_JPRB, 0.47013E+00_JPRB ,&
&  0.47706E+00_JPRB, 0.48088E+00_JPRB, 0.47888E+00_JPRB, 0.47852E+00_JPRB, 0.47772E+00_JPRB, 0.47615E+00_JPRB, 0.47750E+00_JPRB ,&
&  0.47798E+00_JPRB, 0.48039E+00_JPRB, 0.47784E+00_JPRB, 0.47519E+00_JPRB, 0.47727E+00_JPRB, 0.48230E+00_JPRB, 0.48561E+00_JPRB ,&
&  0.48002E+00_JPRB, 0.47804E+00_JPRB, 0.48634E+00_JPRB, 0.48791E+00_JPRB, 0.48861E+00_JPRB, 0.49020E+00_JPRB, 0.48963E+00_JPRB ,&
&  0.47501E+00_JPRB, 0.44834E+00_JPRB, 0.42075E+00_JPRB, 0.42414E+00_JPRB, 0.43904E+00_JPRB, 0.43141E+00_JPRB, 0.42952E+00_JPRB ,&
&  0.42663E+00_JPRB, 0.42184E+00_JPRB, 0.41739E+00_JPRB, 0.41496E+00_JPRB, 0.41216E+00_JPRB, 0.40702E+00_JPRB, 0.40552E+00_JPRB ,&
&  0.40402E+00_JPRB, 0.41373E+00_JPRB, 0.40734E+00_JPRB, 0.38284E+00_JPRB, 0.36570E+00_JPRB, 0.35612E+00_JPRB, 0.34369E+00_JPRB ,&
&  0.32895E+00_JPRB, 0.32109E+00_JPRB, 0.32026E+00_JPRB, 0.31783E+00_JPRB, 0.31888E+00_JPRB, 0.32005E+00_JPRB, 0.32116E+00_JPRB ,&
&  0.32166E+00_JPRB, 0.32111E+00_JPRB, 0.31616E+00_JPRB, 0.30981E+00_JPRB, 0.30640E+00_JPRB, 0.30651E+00_JPRB, 0.30979E+00_JPRB ,&
&  0.31661E+00_JPRB, 0.32336E+00_JPRB, 0.33885E+00_JPRB, 0.34675E+00_JPRB, 0.33709E+00_JPRB, 0.33280E+00_JPRB, 0.32793E+00_JPRB ,&
&  0.31625E+00_JPRB, 0.31199E+00_JPRB, 0.30844E+00_JPRB, 0.30652E+00_JPRB, 0.30757E+00_JPRB, 0.31142E+00_JPRB, 0.31318E+00_JPRB ,&
&  0.31607E+00_JPRB, 0.31936E+00_JPRB, 0.32073E+00_JPRB, 0.32178E+00_JPRB, 0.32266E+00_JPRB, 0.32492E+00_JPRB, 0.32723E+00_JPRB ,&
&  0.32900E+00_JPRB, 0.33230E+00_JPRB, 0.33626E+00_JPRB, 0.34143E+00_JPRB, 0.34597E+00_JPRB, 0.34854E+00_JPRB, 0.35697E+00_JPRB ,&
&  0.36640E+00_JPRB, 0.38556E+00_JPRB, 0.41631E+00_JPRB, 0.44003E+00_JPRB, 0.43309E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.43916E+00_JPRB, 0.44332E+00_JPRB, 0.44351E+00_JPRB, 0.44677E+00_JPRB, 0.45229E+00_JPRB, 0.45861E+00_JPRB, 0.46409E+00_JPRB ,&
&  0.46963E+00_JPRB, 0.46935E+00_JPRB, 0.46432E+00_JPRB, 0.45639E+00_JPRB, 0.44987E+00_JPRB, 0.43318E+00_JPRB, 0.42480E+00_JPRB ,&
&  0.41779E+00_JPRB, 0.41799E+00_JPRB, 0.41839E+00_JPRB, 0.42025E+00_JPRB, 0.42116E+00_JPRB, 0.42630E+00_JPRB, 0.42758E+00_JPRB ,&
&  0.42453E+00_JPRB, 0.41896E+00_JPRB, 0.41321E+00_JPRB, 0.41202E+00_JPRB, 0.41464E+00_JPRB, 0.41636E+00_JPRB, 0.41299E+00_JPRB ,&
&  0.40922E+00_JPRB, 0.40907E+00_JPRB, 0.40954E+00_JPRB, 0.41096E+00_JPRB, 0.42038E+00_JPRB, 0.41405E+00_JPRB, 0.40927E+00_JPRB ,&
&  0.40218E+00_JPRB, 0.39622E+00_JPRB, 0.39229E+00_JPRB, 0.39274E+00_JPRB, 0.39646E+00_JPRB, 0.39475E+00_JPRB, 0.39775E+00_JPRB ,&
&  0.38715E+00_JPRB, 0.37823E+00_JPRB, 0.36261E+00_JPRB, 0.34988E+00_JPRB, 0.33576E+00_JPRB, 0.32615E+00_JPRB, 0.31744E+00_JPRB ,&
&  0.31008E+00_JPRB, 0.30372E+00_JPRB, 0.29782E+00_JPRB, 0.29606E+00_JPRB, 0.29707E+00_JPRB, 0.29688E+00_JPRB, 0.29605E+00_JPRB ,&
&  0.29640E+00_JPRB, 0.29731E+00_JPRB, 0.29800E+00_JPRB, 0.29910E+00_JPRB, 0.30100E+00_JPRB, 0.30403E+00_JPRB, 0.30820E+00_JPRB ,&
&  0.31458E+00_JPRB, 0.31954E+00_JPRB, 0.32214E+00_JPRB, 0.32377E+00_JPRB, 0.32054E+00_JPRB, 0.31477E+00_JPRB, 0.30922E+00_JPRB ,&
&  0.30327E+00_JPRB, 0.30054E+00_JPRB, 0.29794E+00_JPRB, 0.29669E+00_JPRB, 0.29638E+00_JPRB, 0.29731E+00_JPRB, 0.29939E+00_JPRB ,&
&  0.30151E+00_JPRB, 0.30434E+00_JPRB, 0.30619E+00_JPRB, 0.30836E+00_JPRB, 0.31090E+00_JPRB, 0.31307E+00_JPRB, 0.31529E+00_JPRB ,&
&  0.31778E+00_JPRB, 0.31975E+00_JPRB, 0.32318E+00_JPRB, 0.32746E+00_JPRB, 0.33578E+00_JPRB, 0.34501E+00_JPRB, 0.36130E+00_JPRB ,&
&  0.37977E+00_JPRB, 0.41084E+00_JPRB, 0.41878E+00_JPRB, 0.42940E+00_JPRB, 0.43795E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.43400E+00_JPRB, 0.43654E+00_JPRB, 0.44022E+00_JPRB, 0.44596E+00_JPRB, 0.45322E+00_JPRB, 0.46010E+00_JPRB, 0.46193E+00_JPRB ,&
&  0.45203E+00_JPRB, 0.43920E+00_JPRB, 0.42067E+00_JPRB, 0.40629E+00_JPRB, 0.39198E+00_JPRB, 0.38150E+00_JPRB, 0.37316E+00_JPRB ,&
&  0.36741E+00_JPRB, 0.36722E+00_JPRB, 0.36732E+00_JPRB, 0.36853E+00_JPRB, 0.36709E+00_JPRB, 0.36792E+00_JPRB, 0.36787E+00_JPRB ,&
&  0.36623E+00_JPRB, 0.36600E+00_JPRB, 0.36592E+00_JPRB, 0.36928E+00_JPRB, 0.37616E+00_JPRB, 0.38235E+00_JPRB, 0.38846E+00_JPRB ,&
&  0.39572E+00_JPRB, 0.39726E+00_JPRB, 0.40124E+00_JPRB, 0.39953E+00_JPRB, 0.39604E+00_JPRB, 0.39159E+00_JPRB, 0.38528E+00_JPRB ,&
&  0.38133E+00_JPRB, 0.37769E+00_JPRB, 0.37684E+00_JPRB, 0.37726E+00_JPRB, 0.37731E+00_JPRB, 0.38120E+00_JPRB, 0.37619E+00_JPRB ,&
&  0.36916E+00_JPRB, 0.35812E+00_JPRB, 0.34458E+00_JPRB, 0.33434E+00_JPRB, 0.32406E+00_JPRB, 0.31673E+00_JPRB, 0.31020E+00_JPRB ,&
&  0.30423E+00_JPRB, 0.30025E+00_JPRB, 0.29605E+00_JPRB, 0.29293E+00_JPRB, 0.29191E+00_JPRB, 0.28971E+00_JPRB, 0.28840E+00_JPRB ,&
&  0.28783E+00_JPRB, 0.28824E+00_JPRB, 0.29025E+00_JPRB, 0.29202E+00_JPRB, 0.29450E+00_JPRB, 0.29718E+00_JPRB, 0.29941E+00_JPRB ,&
&  0.30243E+00_JPRB, 0.30489E+00_JPRB, 0.30521E+00_JPRB, 0.30462E+00_JPRB, 0.30232E+00_JPRB, 0.29919E+00_JPRB, 0.29659E+00_JPRB ,&
&  0.29378E+00_JPRB, 0.29185E+00_JPRB, 0.29120E+00_JPRB, 0.29019E+00_JPRB, 0.29071E+00_JPRB, 0.29201E+00_JPRB, 0.29292E+00_JPRB ,&
&  0.29490E+00_JPRB, 0.29652E+00_JPRB, 0.29848E+00_JPRB, 0.30192E+00_JPRB, 0.30498E+00_JPRB, 0.30766E+00_JPRB, 0.31030E+00_JPRB ,&
&  0.31021E+00_JPRB, 0.31204E+00_JPRB, 0.31465E+00_JPRB, 0.31969E+00_JPRB, 0.33470E+00_JPRB, 0.34436E+00_JPRB, 0.36664E+00_JPRB ,&
&  0.38389E+00_JPRB, 0.41690E+00_JPRB, 0.41454E+00_JPRB, 0.42258E+00_JPRB, 0.43005E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.42817E+00_JPRB, 0.43523E+00_JPRB, 0.44186E+00_JPRB, 0.44721E+00_JPRB, 0.45075E+00_JPRB, 0.45408E+00_JPRB, 0.45155E+00_JPRB ,&
&  0.44551E+00_JPRB, 0.43117E+00_JPRB, 0.42001E+00_JPRB, 0.40856E+00_JPRB, 0.39912E+00_JPRB, 0.38993E+00_JPRB, 0.38145E+00_JPRB ,&
&  0.37311E+00_JPRB, 0.36607E+00_JPRB, 0.36006E+00_JPRB, 0.35472E+00_JPRB, 0.34751E+00_JPRB, 0.34111E+00_JPRB, 0.33596E+00_JPRB ,&
&  0.33170E+00_JPRB, 0.33160E+00_JPRB, 0.33415E+00_JPRB, 0.33879E+00_JPRB, 0.34752E+00_JPRB, 0.35590E+00_JPRB, 0.36521E+00_JPRB ,&
&  0.37928E+00_JPRB, 0.38155E+00_JPRB, 0.38038E+00_JPRB, 0.37978E+00_JPRB, 0.37475E+00_JPRB, 0.36837E+00_JPRB, 0.35943E+00_JPRB ,&
&  0.36198E+00_JPRB, 0.36001E+00_JPRB, 0.36838E+00_JPRB, 0.36836E+00_JPRB, 0.36705E+00_JPRB, 0.36835E+00_JPRB, 0.36122E+00_JPRB ,&
&  0.35666E+00_JPRB, 0.34696E+00_JPRB, 0.33941E+00_JPRB, 0.33091E+00_JPRB, 0.32200E+00_JPRB, 0.31677E+00_JPRB, 0.30986E+00_JPRB ,&
&  0.30490E+00_JPRB, 0.30136E+00_JPRB, 0.29667E+00_JPRB, 0.29383E+00_JPRB, 0.29127E+00_JPRB, 0.28855E+00_JPRB, 0.28711E+00_JPRB ,&
&  0.28602E+00_JPRB, 0.28563E+00_JPRB, 0.28585E+00_JPRB, 0.28609E+00_JPRB, 0.28667E+00_JPRB, 0.28770E+00_JPRB, 0.28846E+00_JPRB ,&
&  0.28932E+00_JPRB, 0.28968E+00_JPRB, 0.28933E+00_JPRB, 0.28896E+00_JPRB, 0.28768E+00_JPRB, 0.28688E+00_JPRB, 0.28574E+00_JPRB ,&
&  0.28501E+00_JPRB, 0.28536E+00_JPRB, 0.28564E+00_JPRB, 0.28638E+00_JPRB, 0.28710E+00_JPRB, 0.28883E+00_JPRB, 0.29040E+00_JPRB ,&
&  0.29209E+00_JPRB, 0.29431E+00_JPRB, 0.29605E+00_JPRB, 0.29837E+00_JPRB, 0.30108E+00_JPRB, 0.30277E+00_JPRB, 0.30594E+00_JPRB ,&
&  0.30898E+00_JPRB, 0.31286E+00_JPRB, 0.31982E+00_JPRB, 0.32733E+00_JPRB, 0.33745E+00_JPRB, 0.34901E+00_JPRB, 0.36327E+00_JPRB ,&
&  0.38126E+00_JPRB, 0.40545E+00_JPRB, 0.40631E+00_JPRB, 0.41619E+00_JPRB, 0.42225E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.41544E+00_JPRB, 0.42102E+00_JPRB, 0.42573E+00_JPRB, 0.42996E+00_JPRB, 0.43384E+00_JPRB, 0.43515E+00_JPRB, 0.43425E+00_JPRB ,&
&  0.42489E+00_JPRB, 0.41821E+00_JPRB, 0.40987E+00_JPRB, 0.40217E+00_JPRB, 0.39502E+00_JPRB, 0.38580E+00_JPRB, 0.37839E+00_JPRB ,&
&  0.37079E+00_JPRB, 0.36312E+00_JPRB, 0.35602E+00_JPRB, 0.34760E+00_JPRB, 0.33887E+00_JPRB, 0.33048E+00_JPRB, 0.32558E+00_JPRB ,&
&  0.32283E+00_JPRB, 0.32286E+00_JPRB, 0.32625E+00_JPRB, 0.33025E+00_JPRB, 0.33537E+00_JPRB, 0.34190E+00_JPRB, 0.34896E+00_JPRB ,&
&  0.35335E+00_JPRB, 0.35565E+00_JPRB, 0.35497E+00_JPRB, 0.35294E+00_JPRB, 0.35060E+00_JPRB, 0.34885E+00_JPRB, 0.34610E+00_JPRB ,&
&  0.34256E+00_JPRB, 0.34057E+00_JPRB, 0.34072E+00_JPRB, 0.34295E+00_JPRB, 0.34359E+00_JPRB, 0.34227E+00_JPRB, 0.33832E+00_JPRB ,&
&  0.33359E+00_JPRB, 0.32870E+00_JPRB, 0.32347E+00_JPRB, 0.31851E+00_JPRB, 0.31445E+00_JPRB, 0.31007E+00_JPRB, 0.30528E+00_JPRB ,&
&  0.30230E+00_JPRB, 0.29920E+00_JPRB, 0.29579E+00_JPRB, 0.29263E+00_JPRB, 0.28997E+00_JPRB, 0.28792E+00_JPRB, 0.28618E+00_JPRB ,&
&  0.28492E+00_JPRB, 0.28396E+00_JPRB, 0.28322E+00_JPRB, 0.28267E+00_JPRB, 0.28215E+00_JPRB, 0.28125E+00_JPRB, 0.28036E+00_JPRB ,&
&  0.27949E+00_JPRB, 0.27919E+00_JPRB, 0.27865E+00_JPRB, 0.27869E+00_JPRB, 0.27900E+00_JPRB, 0.27917E+00_JPRB, 0.27974E+00_JPRB ,&
&  0.28096E+00_JPRB, 0.28224E+00_JPRB, 0.28279E+00_JPRB, 0.28398E+00_JPRB, 0.28512E+00_JPRB, 0.28664E+00_JPRB, 0.28870E+00_JPRB ,&
&  0.28992E+00_JPRB, 0.29128E+00_JPRB, 0.29291E+00_JPRB, 0.29467E+00_JPRB, 0.29774E+00_JPRB, 0.30107E+00_JPRB, 0.30476E+00_JPRB ,&
&  0.30950E+00_JPRB, 0.31402E+00_JPRB, 0.31922E+00_JPRB, 0.32541E+00_JPRB, 0.33693E+00_JPRB, 0.34374E+00_JPRB, 0.35208E+00_JPRB ,&
&  0.35991E+00_JPRB, 0.36218E+00_JPRB, 0.39826E+00_JPRB, 0.40134E+00_JPRB, 0.40739E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.34027E+00_JPRB, 0.34709E+00_JPRB, 0.35123E+00_JPRB, 0.35533E+00_JPRB, 0.35002E+00_JPRB, 0.38962E+00_JPRB, 0.38877E+00_JPRB ,&
&  0.38819E+00_JPRB, 0.37530E+00_JPRB, 0.37402E+00_JPRB, 0.36967E+00_JPRB, 0.36424E+00_JPRB, 0.35808E+00_JPRB, 0.35275E+00_JPRB ,&
&  0.34780E+00_JPRB, 0.34218E+00_JPRB, 0.33769E+00_JPRB, 0.33352E+00_JPRB, 0.33011E+00_JPRB, 0.32711E+00_JPRB, 0.32500E+00_JPRB ,&
&  0.32455E+00_JPRB, 0.32625E+00_JPRB, 0.32976E+00_JPRB, 0.33137E+00_JPRB, 0.33180E+00_JPRB, 0.33215E+00_JPRB, 0.33255E+00_JPRB ,&
&  0.33234E+00_JPRB, 0.33197E+00_JPRB, 0.33041E+00_JPRB, 0.32764E+00_JPRB, 0.32561E+00_JPRB, 0.32430E+00_JPRB, 0.32336E+00_JPRB ,&
&  0.32252E+00_JPRB, 0.32183E+00_JPRB, 0.32107E+00_JPRB, 0.32073E+00_JPRB, 0.31995E+00_JPRB, 0.31863E+00_JPRB, 0.31646E+00_JPRB ,&
&  0.31418E+00_JPRB, 0.31175E+00_JPRB, 0.30873E+00_JPRB, 0.30631E+00_JPRB, 0.30394E+00_JPRB, 0.30154E+00_JPRB, 0.29912E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29644E+00_JPRB, 0.29475E+00_JPRB, 0.29264E+00_JPRB, 0.29106E+00_JPRB, 0.28940E+00_JPRB, 0.28797E+00_JPRB ,&
&  0.28705E+00_JPRB, 0.28606E+00_JPRB, 0.28506E+00_JPRB, 0.28403E+00_JPRB, 0.28330E+00_JPRB, 0.28243E+00_JPRB, 0.28160E+00_JPRB ,&
&  0.28085E+00_JPRB, 0.27997E+00_JPRB, 0.27976E+00_JPRB, 0.27965E+00_JPRB, 0.27973E+00_JPRB, 0.27975E+00_JPRB, 0.28008E+00_JPRB ,&
&  0.28060E+00_JPRB, 0.28138E+00_JPRB, 0.28236E+00_JPRB, 0.28326E+00_JPRB, 0.28412E+00_JPRB, 0.28501E+00_JPRB, 0.28636E+00_JPRB ,&
&  0.28779E+00_JPRB, 0.28928E+00_JPRB, 0.29082E+00_JPRB, 0.29201E+00_JPRB, 0.29474E+00_JPRB, 0.29730E+00_JPRB, 0.30000E+00_JPRB ,&
&  0.30311E+00_JPRB, 0.30682E+00_JPRB, 0.31086E+00_JPRB, 0.31524E+00_JPRB, 0.32112E+00_JPRB, 0.32448E+00_JPRB, 0.32682E+00_JPRB ,&
&  0.32942E+00_JPRB, 0.33400E+00_JPRB, 0.33513E+00_JPRB, 0.33620E+00_JPRB, 0.33794E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.31307E+00_JPRB, 0.31333E+00_JPRB, 0.31330E+00_JPRB, 0.31304E+00_JPRB, 0.31011E+00_JPRB, 0.31002E+00_JPRB, 0.30977E+00_JPRB ,&
&  0.30979E+00_JPRB, 0.30995E+00_JPRB, 0.31069E+00_JPRB, 0.31158E+00_JPRB, 0.31248E+00_JPRB, 0.31334E+00_JPRB, 0.31414E+00_JPRB ,&
&  0.31465E+00_JPRB, 0.31529E+00_JPRB, 0.31375E+00_JPRB, 0.31278E+00_JPRB, 0.31262E+00_JPRB, 0.31267E+00_JPRB, 0.31261E+00_JPRB ,&
&  0.31247E+00_JPRB, 0.31228E+00_JPRB, 0.31168E+00_JPRB, 0.31128E+00_JPRB, 0.31110E+00_JPRB, 0.31086E+00_JPRB, 0.31049E+00_JPRB ,&
&  0.31006E+00_JPRB, 0.30973E+00_JPRB, 0.30934E+00_JPRB, 0.30886E+00_JPRB, 0.30879E+00_JPRB, 0.30873E+00_JPRB, 0.30819E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30650E+00_JPRB, 0.30557E+00_JPRB, 0.30470E+00_JPRB, 0.30401E+00_JPRB, 0.30370E+00_JPRB, 0.30335E+00_JPRB ,&
&  0.30285E+00_JPRB, 0.30216E+00_JPRB, 0.30150E+00_JPRB, 0.30065E+00_JPRB, 0.29988E+00_JPRB, 0.29918E+00_JPRB, 0.29835E+00_JPRB ,&
&  0.29744E+00_JPRB, 0.29652E+00_JPRB, 0.29555E+00_JPRB, 0.29446E+00_JPRB, 0.29339E+00_JPRB, 0.29229E+00_JPRB, 0.29108E+00_JPRB ,&
&  0.28990E+00_JPRB, 0.28941E+00_JPRB, 0.28885E+00_JPRB, 0.28837E+00_JPRB, 0.28801E+00_JPRB, 0.28774E+00_JPRB, 0.28746E+00_JPRB ,&
&  0.28724E+00_JPRB, 0.28700E+00_JPRB, 0.28690E+00_JPRB, 0.28680E+00_JPRB, 0.28680E+00_JPRB, 0.28687E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.28731E+00_JPRB, 0.28760E+00_JPRB, 0.28800E+00_JPRB, 0.28869E+00_JPRB, 0.28936E+00_JPRB, 0.29015E+00_JPRB, 0.29092E+00_JPRB ,&
&  0.29177E+00_JPRB, 0.29261E+00_JPRB, 0.29347E+00_JPRB, 0.29429E+00_JPRB, 0.29541E+00_JPRB, 0.29648E+00_JPRB, 0.29766E+00_JPRB ,&
&  0.29884E+00_JPRB, 0.30011E+00_JPRB, 0.30136E+00_JPRB, 0.30270E+00_JPRB, 0.30425E+00_JPRB, 0.30467E+00_JPRB, 0.30512E+00_JPRB ,&
&  0.30567E+00_JPRB, 0.30649E+00_JPRB, 0.30722E+00_JPRB, 0.30824E+00_JPRB, 0.30997E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB ,&
&  0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB, 0.29776E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB ,&
&  0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB, 0.55255E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.55093E+00_JPRB, 0.55176E+00_JPRB, 0.55253E+00_JPRB, 0.55304E+00_JPRB, 0.55335E+00_JPRB, 0.55356E+00_JPRB, 0.55364E+00_JPRB ,&
&  0.55352E+00_JPRB, 0.55370E+00_JPRB, 0.55403E+00_JPRB, 0.55437E+00_JPRB, 0.55467E+00_JPRB, 0.55490E+00_JPRB, 0.55488E+00_JPRB ,&
&  0.55531E+00_JPRB, 0.55713E+00_JPRB, 0.55823E+00_JPRB, 0.55879E+00_JPRB, 0.55926E+00_JPRB, 0.55964E+00_JPRB, 0.55990E+00_JPRB ,&
&  0.56021E+00_JPRB, 0.56055E+00_JPRB, 0.56088E+00_JPRB, 0.56123E+00_JPRB, 0.56168E+00_JPRB, 0.56211E+00_JPRB, 0.56256E+00_JPRB ,&
&  0.56324E+00_JPRB, 0.56444E+00_JPRB, 0.56597E+00_JPRB, 0.56770E+00_JPRB, 0.56987E+00_JPRB, 0.57249E+00_JPRB, 0.57468E+00_JPRB ,&
&  0.57641E+00_JPRB, 0.57742E+00_JPRB, 0.57692E+00_JPRB, 0.57532E+00_JPRB, 0.57286E+00_JPRB, 0.57008E+00_JPRB, 0.56711E+00_JPRB ,&
&  0.56367E+00_JPRB, 0.56059E+00_JPRB, 0.55784E+00_JPRB, 0.55535E+00_JPRB, 0.55299E+00_JPRB, 0.55095E+00_JPRB, 0.54899E+00_JPRB ,&
&  0.54740E+00_JPRB, 0.54583E+00_JPRB, 0.54438E+00_JPRB, 0.54297E+00_JPRB, 0.54148E+00_JPRB, 0.53973E+00_JPRB, 0.53796E+00_JPRB ,&
&  0.53658E+00_JPRB, 0.53554E+00_JPRB, 0.53479E+00_JPRB, 0.53407E+00_JPRB, 0.53364E+00_JPRB, 0.53384E+00_JPRB, 0.53430E+00_JPRB ,&
&  0.53467E+00_JPRB, 0.53526E+00_JPRB, 0.53605E+00_JPRB, 0.53655E+00_JPRB, 0.53692E+00_JPRB, 0.53708E+00_JPRB, 0.53713E+00_JPRB ,&
&  0.53709E+00_JPRB, 0.53740E+00_JPRB, 0.53776E+00_JPRB, 0.53847E+00_JPRB, 0.53914E+00_JPRB, 0.53994E+00_JPRB, 0.54081E+00_JPRB ,&
&  0.54145E+00_JPRB, 0.54221E+00_JPRB, 0.54287E+00_JPRB, 0.54376E+00_JPRB, 0.54454E+00_JPRB, 0.54514E+00_JPRB, 0.54554E+00_JPRB ,&
&  0.54590E+00_JPRB, 0.54610E+00_JPRB, 0.54634E+00_JPRB, 0.54643E+00_JPRB, 0.54670E+00_JPRB, 0.54702E+00_JPRB, 0.54737E+00_JPRB ,&
&  0.54772E+00_JPRB, 0.54823E+00_JPRB, 0.54867E+00_JPRB, 0.54943E+00_JPRB, 0.55015E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.55429E+00_JPRB, 0.55549E+00_JPRB, 0.55626E+00_JPRB, 0.55658E+00_JPRB, 0.55661E+00_JPRB, 0.55601E+00_JPRB, 0.55481E+00_JPRB ,&
&  0.55298E+00_JPRB, 0.55152E+00_JPRB, 0.55052E+00_JPRB, 0.54932E+00_JPRB, 0.54754E+00_JPRB, 0.54776E+00_JPRB, 0.54868E+00_JPRB ,&
&  0.54760E+00_JPRB, 0.54609E+00_JPRB, 0.54438E+00_JPRB, 0.54303E+00_JPRB, 0.54150E+00_JPRB, 0.53977E+00_JPRB, 0.53863E+00_JPRB ,&
&  0.53813E+00_JPRB, 0.53726E+00_JPRB, 0.53587E+00_JPRB, 0.53380E+00_JPRB, 0.53330E+00_JPRB, 0.53278E+00_JPRB, 0.53212E+00_JPRB ,&
&  0.53116E+00_JPRB, 0.52877E+00_JPRB, 0.52666E+00_JPRB, 0.52548E+00_JPRB, 0.52872E+00_JPRB, 0.53256E+00_JPRB, 0.53726E+00_JPRB ,&
&  0.54280E+00_JPRB, 0.54710E+00_JPRB, 0.55228E+00_JPRB, 0.55667E+00_JPRB, 0.55908E+00_JPRB, 0.56317E+00_JPRB, 0.56586E+00_JPRB ,&
&  0.56633E+00_JPRB, 0.56450E+00_JPRB, 0.56301E+00_JPRB, 0.56272E+00_JPRB, 0.56269E+00_JPRB, 0.56223E+00_JPRB, 0.56142E+00_JPRB ,&
&  0.56070E+00_JPRB, 0.55969E+00_JPRB, 0.55822E+00_JPRB, 0.55607E+00_JPRB, 0.55316E+00_JPRB, 0.55007E+00_JPRB, 0.54666E+00_JPRB ,&
&  0.54380E+00_JPRB, 0.54050E+00_JPRB, 0.53732E+00_JPRB, 0.53412E+00_JPRB, 0.53140E+00_JPRB, 0.52823E+00_JPRB, 0.52517E+00_JPRB ,&
&  0.52336E+00_JPRB, 0.52361E+00_JPRB, 0.52461E+00_JPRB, 0.52438E+00_JPRB, 0.52442E+00_JPRB, 0.52437E+00_JPRB, 0.52540E+00_JPRB ,&
&  0.52659E+00_JPRB, 0.52901E+00_JPRB, 0.53164E+00_JPRB, 0.53533E+00_JPRB, 0.53874E+00_JPRB, 0.54202E+00_JPRB, 0.54449E+00_JPRB ,&
&  0.54670E+00_JPRB, 0.54821E+00_JPRB, 0.54916E+00_JPRB, 0.54977E+00_JPRB, 0.55048E+00_JPRB, 0.55088E+00_JPRB, 0.55080E+00_JPRB ,&
&  0.55050E+00_JPRB, 0.55024E+00_JPRB, 0.54979E+00_JPRB, 0.54905E+00_JPRB, 0.54866E+00_JPRB, 0.54793E+00_JPRB, 0.54774E+00_JPRB ,&
&  0.54769E+00_JPRB, 0.54840E+00_JPRB, 0.54929E+00_JPRB, 0.55083E+00_JPRB, 0.55257E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.54942E+00_JPRB, 0.55212E+00_JPRB, 0.55407E+00_JPRB, 0.55476E+00_JPRB, 0.55505E+00_JPRB, 0.55517E+00_JPRB, 0.55480E+00_JPRB ,&
&  0.55426E+00_JPRB, 0.55342E+00_JPRB, 0.55311E+00_JPRB, 0.55241E+00_JPRB, 0.55148E+00_JPRB, 0.55366E+00_JPRB, 0.55345E+00_JPRB ,&
&  0.55182E+00_JPRB, 0.54943E+00_JPRB, 0.54721E+00_JPRB, 0.54427E+00_JPRB, 0.54052E+00_JPRB, 0.53568E+00_JPRB, 0.52731E+00_JPRB ,&
&  0.52214E+00_JPRB, 0.51660E+00_JPRB, 0.51032E+00_JPRB, 0.50876E+00_JPRB, 0.50717E+00_JPRB, 0.48815E+00_JPRB, 0.48818E+00_JPRB ,&
&  0.50626E+00_JPRB, 0.51348E+00_JPRB, 0.51589E+00_JPRB, 0.51910E+00_JPRB, 0.49758E+00_JPRB, 0.47307E+00_JPRB, 0.45741E+00_JPRB ,&
&  0.43309E+00_JPRB, 0.45590E+00_JPRB, 0.46993E+00_JPRB, 0.47494E+00_JPRB, 0.47510E+00_JPRB, 0.47405E+00_JPRB, 0.49450E+00_JPRB ,&
&  0.54008E+00_JPRB, 0.54921E+00_JPRB, 0.56017E+00_JPRB, 0.57223E+00_JPRB, 0.58398E+00_JPRB, 0.59187E+00_JPRB, 0.60058E+00_JPRB ,&
&  0.60480E+00_JPRB, 0.60735E+00_JPRB, 0.60503E+00_JPRB, 0.60060E+00_JPRB, 0.59093E+00_JPRB, 0.58297E+00_JPRB, 0.57553E+00_JPRB ,&
&  0.57111E+00_JPRB, 0.56639E+00_JPRB, 0.56194E+00_JPRB, 0.55605E+00_JPRB, 0.54986E+00_JPRB, 0.54287E+00_JPRB, 0.53576E+00_JPRB ,&
&  0.52841E+00_JPRB, 0.52060E+00_JPRB, 0.51530E+00_JPRB, 0.51120E+00_JPRB, 0.51133E+00_JPRB, 0.51140E+00_JPRB, 0.51384E+00_JPRB ,&
&  0.51611E+00_JPRB, 0.51975E+00_JPRB, 0.52376E+00_JPRB, 0.52945E+00_JPRB, 0.53420E+00_JPRB, 0.53789E+00_JPRB, 0.54048E+00_JPRB ,&
&  0.54252E+00_JPRB, 0.54370E+00_JPRB, 0.54432E+00_JPRB, 0.54431E+00_JPRB, 0.54473E+00_JPRB, 0.54536E+00_JPRB, 0.54550E+00_JPRB ,&
&  0.54601E+00_JPRB, 0.54651E+00_JPRB, 0.54716E+00_JPRB, 0.54724E+00_JPRB, 0.54688E+00_JPRB, 0.54485E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.54060E+00_JPRB, 0.54044E+00_JPRB, 0.54104E+00_JPRB, 0.54324E+00_JPRB, 0.54583E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.55930E+00_JPRB, 0.56048E+00_JPRB, 0.56229E+00_JPRB, 0.56334E+00_JPRB, 0.56364E+00_JPRB, 0.56340E+00_JPRB, 0.56282E+00_JPRB ,&
&  0.56236E+00_JPRB, 0.56229E+00_JPRB, 0.56163E+00_JPRB, 0.56007E+00_JPRB, 0.55943E+00_JPRB, 0.55900E+00_JPRB, 0.55789E+00_JPRB ,&
&  0.55471E+00_JPRB, 0.55104E+00_JPRB, 0.54788E+00_JPRB, 0.54383E+00_JPRB, 0.53995E+00_JPRB, 0.53664E+00_JPRB, 0.52992E+00_JPRB ,&
&  0.52537E+00_JPRB, 0.51757E+00_JPRB, 0.48937E+00_JPRB, 0.49128E+00_JPRB, 0.48598E+00_JPRB, 0.48520E+00_JPRB, 0.48846E+00_JPRB ,&
&  0.48065E+00_JPRB, 0.47955E+00_JPRB, 0.46921E+00_JPRB, 0.45371E+00_JPRB, 0.44572E+00_JPRB, 0.43212E+00_JPRB, 0.43248E+00_JPRB ,&
&  0.42995E+00_JPRB, 0.43453E+00_JPRB, 0.44515E+00_JPRB, 0.44944E+00_JPRB, 0.44884E+00_JPRB, 0.44886E+00_JPRB, 0.45859E+00_JPRB ,&
&  0.48545E+00_JPRB, 0.52563E+00_JPRB, 0.57864E+00_JPRB, 0.60461E+00_JPRB, 0.62784E+00_JPRB, 0.64912E+00_JPRB, 0.66864E+00_JPRB ,&
&  0.68245E+00_JPRB, 0.69355E+00_JPRB, 0.69309E+00_JPRB, 0.69032E+00_JPRB, 0.67779E+00_JPRB, 0.65975E+00_JPRB, 0.63794E+00_JPRB ,&
&  0.61700E+00_JPRB, 0.60178E+00_JPRB, 0.59035E+00_JPRB, 0.58375E+00_JPRB, 0.58024E+00_JPRB, 0.57731E+00_JPRB, 0.57562E+00_JPRB ,&
&  0.56960E+00_JPRB, 0.55542E+00_JPRB, 0.54315E+00_JPRB, 0.53147E+00_JPRB, 0.52768E+00_JPRB, 0.52603E+00_JPRB, 0.52892E+00_JPRB ,&
&  0.53253E+00_JPRB, 0.53998E+00_JPRB, 0.54901E+00_JPRB, 0.55252E+00_JPRB, 0.55781E+00_JPRB, 0.56423E+00_JPRB, 0.56403E+00_JPRB ,&
&  0.56373E+00_JPRB, 0.56246E+00_JPRB, 0.55973E+00_JPRB, 0.55622E+00_JPRB, 0.55214E+00_JPRB, 0.54829E+00_JPRB, 0.54522E+00_JPRB ,&
&  0.54415E+00_JPRB, 0.54496E+00_JPRB, 0.54905E+00_JPRB, 0.55375E+00_JPRB, 0.55946E+00_JPRB, 0.56238E+00_JPRB, 0.56165E+00_JPRB ,&
&  0.55730E+00_JPRB, 0.55513E+00_JPRB, 0.55354E+00_JPRB, 0.55557E+00_JPRB, 0.55751E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.57299E+00_JPRB, 0.57646E+00_JPRB, 0.58299E+00_JPRB, 0.58918E+00_JPRB, 0.59413E+00_JPRB, 0.59484E+00_JPRB, 0.58837E+00_JPRB ,&
&  0.58221E+00_JPRB, 0.57621E+00_JPRB, 0.57133E+00_JPRB, 0.56756E+00_JPRB, 0.56540E+00_JPRB, 0.56177E+00_JPRB, 0.55765E+00_JPRB ,&
&  0.55403E+00_JPRB, 0.55178E+00_JPRB, 0.55072E+00_JPRB, 0.54991E+00_JPRB, 0.55074E+00_JPRB, 0.54989E+00_JPRB, 0.54846E+00_JPRB ,&
&  0.54725E+00_JPRB, 0.53661E+00_JPRB, 0.51402E+00_JPRB, 0.50529E+00_JPRB, 0.49653E+00_JPRB, 0.49665E+00_JPRB, 0.49682E+00_JPRB ,&
&  0.49286E+00_JPRB, 0.44389E+00_JPRB, 0.45089E+00_JPRB, 0.43919E+00_JPRB, 0.43219E+00_JPRB, 0.43243E+00_JPRB, 0.42174E+00_JPRB ,&
&  0.41475E+00_JPRB, 0.40629E+00_JPRB, 0.40698E+00_JPRB, 0.41361E+00_JPRB, 0.41390E+00_JPRB, 0.42326E+00_JPRB, 0.43642E+00_JPRB ,&
&  0.46425E+00_JPRB, 0.56932E+00_JPRB, 0.61556E+00_JPRB, 0.64326E+00_JPRB, 0.66595E+00_JPRB, 0.69332E+00_JPRB, 0.72021E+00_JPRB ,&
&  0.74014E+00_JPRB, 0.76508E+00_JPRB, 0.77436E+00_JPRB, 0.76820E+00_JPRB, 0.77189E+00_JPRB, 0.76163E+00_JPRB, 0.73819E+00_JPRB ,&
&  0.70530E+00_JPRB, 0.66366E+00_JPRB, 0.63577E+00_JPRB, 0.61860E+00_JPRB, 0.60549E+00_JPRB, 0.59464E+00_JPRB, 0.58746E+00_JPRB ,&
&  0.59034E+00_JPRB, 0.58419E+00_JPRB, 0.57646E+00_JPRB, 0.56257E+00_JPRB, 0.55508E+00_JPRB, 0.55110E+00_JPRB, 0.55199E+00_JPRB ,&
&  0.55536E+00_JPRB, 0.56320E+00_JPRB, 0.57380E+00_JPRB, 0.58256E+00_JPRB, 0.58708E+00_JPRB, 0.58674E+00_JPRB, 0.59040E+00_JPRB ,&
&  0.59024E+00_JPRB, 0.58745E+00_JPRB, 0.58399E+00_JPRB, 0.58006E+00_JPRB, 0.57593E+00_JPRB, 0.57317E+00_JPRB, 0.57435E+00_JPRB ,&
&  0.57654E+00_JPRB, 0.57785E+00_JPRB, 0.57837E+00_JPRB, 0.58041E+00_JPRB, 0.58449E+00_JPRB, 0.58804E+00_JPRB, 0.58929E+00_JPRB ,&
&  0.58322E+00_JPRB, 0.57696E+00_JPRB, 0.57143E+00_JPRB, 0.57101E+00_JPRB, 0.57120E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.56479E+00_JPRB, 0.57645E+00_JPRB, 0.59566E+00_JPRB, 0.61130E+00_JPRB, 0.62264E+00_JPRB, 0.62156E+00_JPRB, 0.61130E+00_JPRB ,&
&  0.60084E+00_JPRB, 0.59285E+00_JPRB, 0.58534E+00_JPRB, 0.57831E+00_JPRB, 0.57002E+00_JPRB, 0.56352E+00_JPRB, 0.55847E+00_JPRB ,&
&  0.55557E+00_JPRB, 0.55468E+00_JPRB, 0.55421E+00_JPRB, 0.55369E+00_JPRB, 0.55414E+00_JPRB, 0.55566E+00_JPRB, 0.55584E+00_JPRB ,&
&  0.55621E+00_JPRB, 0.54838E+00_JPRB, 0.52929E+00_JPRB, 0.51304E+00_JPRB, 0.50640E+00_JPRB, 0.49690E+00_JPRB, 0.49985E+00_JPRB ,&
&  0.49923E+00_JPRB, 0.49785E+00_JPRB, 0.49142E+00_JPRB, 0.48865E+00_JPRB, 0.47219E+00_JPRB, 0.41071E+00_JPRB, 0.41426E+00_JPRB ,&
&  0.40322E+00_JPRB, 0.38962E+00_JPRB, 0.38753E+00_JPRB, 0.38791E+00_JPRB, 0.39021E+00_JPRB, 0.41063E+00_JPRB, 0.43399E+00_JPRB ,&
&  0.46206E+00_JPRB, 0.58019E+00_JPRB, 0.61222E+00_JPRB, 0.64640E+00_JPRB, 0.67696E+00_JPRB, 0.71572E+00_JPRB, 0.75716E+00_JPRB ,&
&  0.78222E+00_JPRB, 0.80927E+00_JPRB, 0.82845E+00_JPRB, 0.84122E+00_JPRB, 0.84262E+00_JPRB, 0.84235E+00_JPRB, 0.84041E+00_JPRB ,&
&  0.82546E+00_JPRB, 0.75443E+00_JPRB, 0.70712E+00_JPRB, 0.67946E+00_JPRB, 0.65427E+00_JPRB, 0.63672E+00_JPRB, 0.62264E+00_JPRB ,&
&  0.61730E+00_JPRB, 0.59981E+00_JPRB, 0.59589E+00_JPRB, 0.58692E+00_JPRB, 0.58208E+00_JPRB, 0.57864E+00_JPRB, 0.57430E+00_JPRB ,&
&  0.56764E+00_JPRB, 0.56977E+00_JPRB, 0.58255E+00_JPRB, 0.58901E+00_JPRB, 0.59342E+00_JPRB, 0.58998E+00_JPRB, 0.58351E+00_JPRB ,&
&  0.57248E+00_JPRB, 0.57155E+00_JPRB, 0.56662E+00_JPRB, 0.56496E+00_JPRB, 0.56521E+00_JPRB, 0.57267E+00_JPRB, 0.58268E+00_JPRB ,&
&  0.58968E+00_JPRB, 0.59181E+00_JPRB, 0.59120E+00_JPRB, 0.59140E+00_JPRB, 0.59011E+00_JPRB, 0.58753E+00_JPRB, 0.58233E+00_JPRB ,&
&  0.57723E+00_JPRB, 0.57409E+00_JPRB, 0.56801E+00_JPRB, 0.56300E+00_JPRB, 0.56113E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.55867E+00_JPRB, 0.58265E+00_JPRB, 0.61096E+00_JPRB, 0.63055E+00_JPRB, 0.63977E+00_JPRB, 0.62782E+00_JPRB, 0.61178E+00_JPRB ,&
&  0.59879E+00_JPRB, 0.58923E+00_JPRB, 0.58222E+00_JPRB, 0.57722E+00_JPRB, 0.57111E+00_JPRB, 0.56518E+00_JPRB, 0.55992E+00_JPRB ,&
&  0.55600E+00_JPRB, 0.54617E+00_JPRB, 0.54237E+00_JPRB, 0.54408E+00_JPRB, 0.54535E+00_JPRB, 0.54924E+00_JPRB, 0.55275E+00_JPRB ,&
&  0.55073E+00_JPRB, 0.54270E+00_JPRB, 0.53541E+00_JPRB, 0.51983E+00_JPRB, 0.50429E+00_JPRB, 0.49859E+00_JPRB, 0.50240E+00_JPRB ,&
&  0.49122E+00_JPRB, 0.47235E+00_JPRB, 0.48935E+00_JPRB, 0.50307E+00_JPRB, 0.47998E+00_JPRB, 0.46047E+00_JPRB, 0.42183E+00_JPRB ,&
&  0.40844E+00_JPRB, 0.38690E+00_JPRB, 0.38187E+00_JPRB, 0.38214E+00_JPRB, 0.39093E+00_JPRB, 0.40922E+00_JPRB, 0.44803E+00_JPRB ,&
&  0.50094E+00_JPRB, 0.58540E+00_JPRB, 0.61057E+00_JPRB, 0.65274E+00_JPRB, 0.69009E+00_JPRB, 0.72331E+00_JPRB, 0.74716E+00_JPRB ,&
&  0.78457E+00_JPRB, 0.83610E+00_JPRB, 0.86680E+00_JPRB, 0.87131E+00_JPRB, 0.88104E+00_JPRB, 0.90045E+00_JPRB, 0.93920E+00_JPRB ,&
&  0.99692E+00_JPRB, 0.86841E+00_JPRB, 0.79044E+00_JPRB, 0.73951E+00_JPRB, 0.70748E+00_JPRB, 0.69485E+00_JPRB, 0.65305E+00_JPRB ,&
&  0.63831E+00_JPRB, 0.63386E+00_JPRB, 0.62404E+00_JPRB, 0.61895E+00_JPRB, 0.61552E+00_JPRB, 0.61253E+00_JPRB, 0.60548E+00_JPRB ,&
&  0.59578E+00_JPRB, 0.60047E+00_JPRB, 0.65690E+00_JPRB, 0.66990E+00_JPRB, 0.65717E+00_JPRB, 0.61937E+00_JPRB, 0.58019E+00_JPRB ,&
&  0.55392E+00_JPRB, 0.54411E+00_JPRB, 0.54051E+00_JPRB, 0.53840E+00_JPRB, 0.54291E+00_JPRB, 0.55094E+00_JPRB, 0.56469E+00_JPRB ,&
&  0.56596E+00_JPRB, 0.57467E+00_JPRB, 0.58302E+00_JPRB, 0.58697E+00_JPRB, 0.58560E+00_JPRB, 0.58069E+00_JPRB, 0.57697E+00_JPRB ,&
&  0.57351E+00_JPRB, 0.56979E+00_JPRB, 0.56394E+00_JPRB, 0.55673E+00_JPRB, 0.55250E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.54414E+00_JPRB, 0.57956E+00_JPRB, 0.62493E+00_JPRB, 0.65531E+00_JPRB, 0.65618E+00_JPRB, 0.62968E+00_JPRB, 0.60312E+00_JPRB ,&
&  0.58647E+00_JPRB, 0.57876E+00_JPRB, 0.57538E+00_JPRB, 0.57068E+00_JPRB, 0.56587E+00_JPRB, 0.56076E+00_JPRB, 0.55565E+00_JPRB ,&
&  0.55147E+00_JPRB, 0.54762E+00_JPRB, 0.54183E+00_JPRB, 0.53416E+00_JPRB, 0.52960E+00_JPRB, 0.52977E+00_JPRB, 0.53257E+00_JPRB ,&
&  0.53613E+00_JPRB, 0.53404E+00_JPRB, 0.53199E+00_JPRB, 0.52314E+00_JPRB, 0.51150E+00_JPRB, 0.49841E+00_JPRB, 0.48882E+00_JPRB ,&
&  0.48472E+00_JPRB, 0.48683E+00_JPRB, 0.48283E+00_JPRB, 0.50102E+00_JPRB, 0.49685E+00_JPRB, 0.47635E+00_JPRB, 0.44545E+00_JPRB ,&
&  0.42236E+00_JPRB, 0.39604E+00_JPRB, 0.38627E+00_JPRB, 0.39069E+00_JPRB, 0.41611E+00_JPRB, 0.45018E+00_JPRB, 0.50437E+00_JPRB ,&
&  0.60000E+00_JPRB, 0.65591E+00_JPRB, 0.69675E+00_JPRB, 0.73364E+00_JPRB, 0.76271E+00_JPRB, 0.79399E+00_JPRB, 0.83038E+00_JPRB ,&
&  0.87684E+00_JPRB, 0.91689E+00_JPRB, 0.93660E+00_JPRB, 0.91128E+00_JPRB, 0.89905E+00_JPRB, 0.84222E+00_JPRB, 0.94007E+00_JPRB ,&
&  0.12421E+01_JPRB, 0.10436E+01_JPRB, 0.90991E+00_JPRB, 0.84110E+00_JPRB, 0.79672E+00_JPRB, 0.76084E+00_JPRB, 0.72821E+00_JPRB ,&
&  0.69524E+00_JPRB, 0.67605E+00_JPRB, 0.66649E+00_JPRB, 0.63690E+00_JPRB, 0.60855E+00_JPRB, 0.60821E+00_JPRB, 0.64434E+00_JPRB ,&
&  0.73455E+00_JPRB, 0.87153E+00_JPRB, 0.10523E+01_JPRB, 0.74557E+00_JPRB, 0.64930E+00_JPRB, 0.58247E+00_JPRB, 0.55279E+00_JPRB ,&
&  0.54489E+00_JPRB, 0.54432E+00_JPRB, 0.54568E+00_JPRB, 0.54168E+00_JPRB, 0.53753E+00_JPRB, 0.53847E+00_JPRB, 0.53943E+00_JPRB ,&
&  0.53855E+00_JPRB, 0.54692E+00_JPRB, 0.55929E+00_JPRB, 0.56916E+00_JPRB, 0.57664E+00_JPRB, 0.57849E+00_JPRB, 0.57961E+00_JPRB ,&
&  0.57795E+00_JPRB, 0.57110E+00_JPRB, 0.56234E+00_JPRB, 0.54670E+00_JPRB, 0.53680E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.53911E+00_JPRB, 0.58041E+00_JPRB, 0.63987E+00_JPRB, 0.67453E+00_JPRB, 0.67331E+00_JPRB, 0.63207E+00_JPRB, 0.58498E+00_JPRB ,&
&  0.55562E+00_JPRB, 0.53778E+00_JPRB, 0.53346E+00_JPRB, 0.53084E+00_JPRB, 0.52990E+00_JPRB, 0.52942E+00_JPRB, 0.52813E+00_JPRB ,&
&  0.52556E+00_JPRB, 0.52280E+00_JPRB, 0.51953E+00_JPRB, 0.51671E+00_JPRB, 0.51285E+00_JPRB, 0.50973E+00_JPRB, 0.50934E+00_JPRB ,&
&  0.51116E+00_JPRB, 0.51090E+00_JPRB, 0.50974E+00_JPRB, 0.50790E+00_JPRB, 0.50887E+00_JPRB, 0.51252E+00_JPRB, 0.51076E+00_JPRB ,&
&  0.49694E+00_JPRB, 0.48642E+00_JPRB, 0.49323E+00_JPRB, 0.50913E+00_JPRB, 0.50732E+00_JPRB, 0.49256E+00_JPRB, 0.45853E+00_JPRB ,&
&  0.42974E+00_JPRB, 0.41025E+00_JPRB, 0.42366E+00_JPRB, 0.46479E+00_JPRB, 0.52466E+00_JPRB, 0.64605E+00_JPRB, 0.72726E+00_JPRB ,&
&  0.79744E+00_JPRB, 0.83808E+00_JPRB, 0.86310E+00_JPRB, 0.88856E+00_JPRB, 0.93815E+00_JPRB, 0.98886E+00_JPRB, 0.10246E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10576E+01_JPRB, 0.10572E+01_JPRB, 0.82527E+00_JPRB, 0.79798E+00_JPRB, 0.86971E+00_JPRB, 0.95969E+00_JPRB ,&
&  0.10158E+01_JPRB, 0.95722E+00_JPRB, 0.92638E+00_JPRB, 0.94061E+00_JPRB, 0.88327E+00_JPRB, 0.84342E+00_JPRB, 0.81240E+00_JPRB ,&
&  0.78110E+00_JPRB, 0.77005E+00_JPRB, 0.72072E+00_JPRB, 0.66817E+00_JPRB, 0.66558E+00_JPRB, 0.66713E+00_JPRB, 0.67566E+00_JPRB ,&
&  0.70572E+00_JPRB, 0.72730E+00_JPRB, 0.67340E+00_JPRB, 0.54710E+00_JPRB, 0.51544E+00_JPRB, 0.50392E+00_JPRB, 0.50589E+00_JPRB ,&
&  0.51145E+00_JPRB, 0.51711E+00_JPRB, 0.52437E+00_JPRB, 0.52536E+00_JPRB, 0.52318E+00_JPRB, 0.52034E+00_JPRB, 0.51590E+00_JPRB ,&
&  0.50985E+00_JPRB, 0.51078E+00_JPRB, 0.51898E+00_JPRB, 0.52339E+00_JPRB, 0.53823E+00_JPRB, 0.55207E+00_JPRB, 0.56493E+00_JPRB ,&
&  0.56221E+00_JPRB, 0.55675E+00_JPRB, 0.54987E+00_JPRB, 0.53461E+00_JPRB, 0.52388E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.58047E+00_JPRB, 0.62123E+00_JPRB, 0.65705E+00_JPRB, 0.68035E+00_JPRB, 0.67318E+00_JPRB, 0.64520E+00_JPRB, 0.58502E+00_JPRB ,&
&  0.56687E+00_JPRB, 0.53402E+00_JPRB, 0.52340E+00_JPRB, 0.51091E+00_JPRB, 0.50048E+00_JPRB, 0.49217E+00_JPRB, 0.48931E+00_JPRB ,&
&  0.49199E+00_JPRB, 0.49552E+00_JPRB, 0.49756E+00_JPRB, 0.49890E+00_JPRB, 0.49697E+00_JPRB, 0.49560E+00_JPRB, 0.49511E+00_JPRB ,&
&  0.49484E+00_JPRB, 0.49503E+00_JPRB, 0.49688E+00_JPRB, 0.49966E+00_JPRB, 0.50396E+00_JPRB, 0.50764E+00_JPRB, 0.50547E+00_JPRB ,&
&  0.49658E+00_JPRB, 0.48913E+00_JPRB, 0.50036E+00_JPRB, 0.51732E+00_JPRB, 0.51564E+00_JPRB, 0.50874E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.44130E+00_JPRB, 0.45327E+00_JPRB, 0.53354E+00_JPRB, 0.68453E+00_JPRB, 0.74663E+00_JPRB, 0.80846E+00_JPRB, 0.85041E+00_JPRB ,&
&  0.81062E+00_JPRB, 0.82232E+00_JPRB, 0.88349E+00_JPRB, 0.10591E+01_JPRB, 0.10913E+01_JPRB, 0.11312E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.12165E+01_JPRB, 0.11827E+01_JPRB, 0.99635E+00_JPRB, 0.11170E+01_JPRB, 0.13450E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.11296E+01_JPRB, 0.11546E+01_JPRB, 0.10834E+01_JPRB, 0.97846E+00_JPRB, 0.92052E+00_JPRB, 0.90604E+00_JPRB ,&
&  0.87439E+00_JPRB, 0.85026E+00_JPRB, 0.76302E+00_JPRB, 0.73612E+00_JPRB, 0.74308E+00_JPRB, 0.71638E+00_JPRB, 0.69316E+00_JPRB ,&
&  0.67064E+00_JPRB, 0.62585E+00_JPRB, 0.58870E+00_JPRB, 0.54826E+00_JPRB, 0.52200E+00_JPRB, 0.51598E+00_JPRB, 0.51789E+00_JPRB ,&
&  0.52003E+00_JPRB, 0.52011E+00_JPRB, 0.52377E+00_JPRB, 0.52745E+00_JPRB, 0.52452E+00_JPRB, 0.51706E+00_JPRB, 0.51663E+00_JPRB ,&
&  0.51296E+00_JPRB, 0.50428E+00_JPRB, 0.49792E+00_JPRB, 0.50487E+00_JPRB, 0.51429E+00_JPRB, 0.52914E+00_JPRB, 0.54505E+00_JPRB ,&
&  0.55466E+00_JPRB, 0.56157E+00_JPRB, 0.55793E+00_JPRB, 0.56018E+00_JPRB, 0.55861E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.61992E+00_JPRB, 0.63962E+00_JPRB, 0.66549E+00_JPRB, 0.68522E+00_JPRB, 0.69779E+00_JPRB, 0.64366E+00_JPRB, 0.59360E+00_JPRB ,&
&  0.61462E+00_JPRB, 0.54969E+00_JPRB, 0.52067E+00_JPRB, 0.50753E+00_JPRB, 0.50268E+00_JPRB, 0.48862E+00_JPRB, 0.47745E+00_JPRB ,&
&  0.47339E+00_JPRB, 0.47738E+00_JPRB, 0.48447E+00_JPRB, 0.48797E+00_JPRB, 0.48856E+00_JPRB, 0.48863E+00_JPRB, 0.48811E+00_JPRB ,&
&  0.48853E+00_JPRB, 0.49023E+00_JPRB, 0.49267E+00_JPRB, 0.49546E+00_JPRB, 0.49674E+00_JPRB, 0.49889E+00_JPRB, 0.49842E+00_JPRB ,&
&  0.49275E+00_JPRB, 0.49163E+00_JPRB, 0.50446E+00_JPRB, 0.52555E+00_JPRB, 0.53415E+00_JPRB, 0.54033E+00_JPRB, 0.53201E+00_JPRB ,&
&  0.47633E+00_JPRB, 0.53105E+00_JPRB, 0.73048E+00_JPRB, 0.79877E+00_JPRB, 0.84698E+00_JPRB, 0.91077E+00_JPRB, 0.96643E+00_JPRB ,&
&  0.10094E+01_JPRB, 0.10455E+01_JPRB, 0.10759E+01_JPRB, 0.11175E+01_JPRB, 0.11718E+01_JPRB, 0.12180E+01_JPRB, 0.12774E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.11165E+01_JPRB, 0.10296E+01_JPRB, 0.10872E+01_JPRB, 0.15123E+01_JPRB, 0.15581E+01_JPRB, 0.13665E+01_JPRB ,&
&  0.13363E+01_JPRB, 0.12820E+01_JPRB, 0.12321E+01_JPRB, 0.11386E+01_JPRB, 0.10650E+01_JPRB, 0.10384E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.96378E+00_JPRB, 0.93386E+00_JPRB, 0.83459E+00_JPRB, 0.81610E+00_JPRB, 0.81466E+00_JPRB, 0.76990E+00_JPRB, 0.70713E+00_JPRB ,&
&  0.65834E+00_JPRB, 0.61980E+00_JPRB, 0.59209E+00_JPRB, 0.57274E+00_JPRB, 0.55784E+00_JPRB, 0.54899E+00_JPRB, 0.54272E+00_JPRB ,&
&  0.53955E+00_JPRB, 0.53716E+00_JPRB, 0.52662E+00_JPRB, 0.52700E+00_JPRB, 0.52897E+00_JPRB, 0.52969E+00_JPRB, 0.53926E+00_JPRB ,&
&  0.53976E+00_JPRB, 0.53408E+00_JPRB, 0.52179E+00_JPRB, 0.52823E+00_JPRB, 0.54420E+00_JPRB, 0.57168E+00_JPRB, 0.59811E+00_JPRB ,&
&  0.62973E+00_JPRB, 0.62770E+00_JPRB, 0.59931E+00_JPRB, 0.59273E+00_JPRB, 0.59598E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.64728E+00_JPRB, 0.66451E+00_JPRB, 0.67714E+00_JPRB, 0.69493E+00_JPRB, 0.70988E+00_JPRB, 0.66308E+00_JPRB, 0.63061E+00_JPRB ,&
&  0.69149E+00_JPRB, 0.61054E+00_JPRB, 0.65712E+00_JPRB, 0.56874E+00_JPRB, 0.52685E+00_JPRB, 0.50665E+00_JPRB, 0.49134E+00_JPRB ,&
&  0.47737E+00_JPRB, 0.48178E+00_JPRB, 0.48803E+00_JPRB, 0.49086E+00_JPRB, 0.49665E+00_JPRB, 0.49293E+00_JPRB, 0.48970E+00_JPRB ,&
&  0.48771E+00_JPRB, 0.48856E+00_JPRB, 0.49468E+00_JPRB, 0.49507E+00_JPRB, 0.49399E+00_JPRB, 0.49279E+00_JPRB, 0.48698E+00_JPRB ,&
&  0.47525E+00_JPRB, 0.47896E+00_JPRB, 0.50219E+00_JPRB, 0.52424E+00_JPRB, 0.54578E+00_JPRB, 0.56153E+00_JPRB, 0.58660E+00_JPRB ,&
&  0.61144E+00_JPRB, 0.63566E+00_JPRB, 0.77480E+00_JPRB, 0.81794E+00_JPRB, 0.86709E+00_JPRB, 0.90290E+00_JPRB, 0.94612E+00_JPRB ,&
&  0.99011E+00_JPRB, 0.10269E+01_JPRB, 0.10496E+01_JPRB, 0.10937E+01_JPRB, 0.11795E+01_JPRB, 0.12362E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.11383E+01_JPRB, 0.13335E+01_JPRB, 0.13250E+01_JPRB, 0.15543E+01_JPRB, 0.17214E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.18593E+01_JPRB, 0.16272E+01_JPRB, 0.14606E+01_JPRB, 0.12880E+01_JPRB, 0.11559E+01_JPRB, 0.11245E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11389E+01_JPRB, 0.11130E+01_JPRB, 0.95882E+00_JPRB, 0.88995E+00_JPRB, 0.86597E+00_JPRB, 0.84629E+00_JPRB, 0.77725E+00_JPRB ,&
&  0.73979E+00_JPRB, 0.71640E+00_JPRB, 0.68432E+00_JPRB, 0.65047E+00_JPRB, 0.62459E+00_JPRB, 0.60993E+00_JPRB, 0.58889E+00_JPRB ,&
&  0.58404E+00_JPRB, 0.57059E+00_JPRB, 0.53318E+00_JPRB, 0.51830E+00_JPRB, 0.51413E+00_JPRB, 0.50554E+00_JPRB, 0.51735E+00_JPRB ,&
&  0.53402E+00_JPRB, 0.53814E+00_JPRB, 0.57298E+00_JPRB, 0.62008E+00_JPRB, 0.63553E+00_JPRB, 0.64527E+00_JPRB, 0.71132E+00_JPRB ,&
&  0.66753E+00_JPRB, 0.67485E+00_JPRB, 0.67514E+00_JPRB, 0.64709E+00_JPRB, 0.63692E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.68222E+00_JPRB, 0.68551E+00_JPRB, 0.68926E+00_JPRB, 0.70567E+00_JPRB, 0.71927E+00_JPRB, 0.72248E+00_JPRB, 0.68740E+00_JPRB ,&
&  0.73101E+00_JPRB, 0.70661E+00_JPRB, 0.68836E+00_JPRB, 0.68267E+00_JPRB, 0.66920E+00_JPRB, 0.58805E+00_JPRB, 0.50473E+00_JPRB ,&
&  0.48132E+00_JPRB, 0.47643E+00_JPRB, 0.49150E+00_JPRB, 0.49266E+00_JPRB, 0.49707E+00_JPRB, 0.49746E+00_JPRB, 0.49219E+00_JPRB ,&
&  0.49130E+00_JPRB, 0.49225E+00_JPRB, 0.50056E+00_JPRB, 0.50216E+00_JPRB, 0.49657E+00_JPRB, 0.49459E+00_JPRB, 0.49119E+00_JPRB ,&
&  0.47317E+00_JPRB, 0.46837E+00_JPRB, 0.48161E+00_JPRB, 0.48882E+00_JPRB, 0.54464E+00_JPRB, 0.57268E+00_JPRB, 0.60569E+00_JPRB ,&
&  0.64016E+00_JPRB, 0.68136E+00_JPRB, 0.75030E+00_JPRB, 0.80530E+00_JPRB, 0.84270E+00_JPRB, 0.87137E+00_JPRB, 0.91354E+00_JPRB ,&
&  0.94694E+00_JPRB, 0.99215E+00_JPRB, 0.10443E+01_JPRB, 0.11294E+01_JPRB, 0.12557E+01_JPRB, 0.11522E+01_JPRB, 0.13536E+01_JPRB ,&
&  0.14584E+01_JPRB, 0.16217E+01_JPRB, 0.17648E+01_JPRB, 0.16523E+01_JPRB, 0.18357E+01_JPRB, 0.19450E+01_JPRB, 0.18802E+01_JPRB ,&
&  0.18209E+01_JPRB, 0.16158E+01_JPRB, 0.15779E+01_JPRB, 0.15447E+01_JPRB, 0.15384E+01_JPRB, 0.14290E+01_JPRB, 0.13250E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.16037E+01_JPRB, 0.12647E+01_JPRB, 0.11541E+01_JPRB, 0.10589E+01_JPRB, 0.10265E+01_JPRB, 0.96712E+00_JPRB ,&
&  0.94170E+00_JPRB, 0.94208E+00_JPRB, 0.90880E+00_JPRB, 0.86139E+00_JPRB, 0.76799E+00_JPRB, 0.72512E+00_JPRB, 0.67403E+00_JPRB ,&
&  0.64638E+00_JPRB, 0.60655E+00_JPRB, 0.55031E+00_JPRB, 0.52043E+00_JPRB, 0.50037E+00_JPRB, 0.49203E+00_JPRB, 0.51010E+00_JPRB ,&
&  0.56022E+00_JPRB, 0.61358E+00_JPRB, 0.72727E+00_JPRB, 0.76261E+00_JPRB, 0.78426E+00_JPRB, 0.82000E+00_JPRB, 0.72359E+00_JPRB ,&
&  0.66810E+00_JPRB, 0.73797E+00_JPRB, 0.71602E+00_JPRB, 0.69255E+00_JPRB, 0.68147E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.74246E+00_JPRB, 0.73642E+00_JPRB, 0.72935E+00_JPRB, 0.73342E+00_JPRB, 0.74261E+00_JPRB, 0.73676E+00_JPRB, 0.73778E+00_JPRB ,&
&  0.74785E+00_JPRB, 0.74130E+00_JPRB, 0.71986E+00_JPRB, 0.70634E+00_JPRB, 0.68896E+00_JPRB, 0.68451E+00_JPRB, 0.60367E+00_JPRB ,&
&  0.52416E+00_JPRB, 0.50010E+00_JPRB, 0.48365E+00_JPRB, 0.48612E+00_JPRB, 0.50340E+00_JPRB, 0.50275E+00_JPRB, 0.50084E+00_JPRB ,&
&  0.50121E+00_JPRB, 0.50285E+00_JPRB, 0.50647E+00_JPRB, 0.50649E+00_JPRB, 0.50831E+00_JPRB, 0.51979E+00_JPRB, 0.52276E+00_JPRB ,&
&  0.49373E+00_JPRB, 0.46405E+00_JPRB, 0.45835E+00_JPRB, 0.47392E+00_JPRB, 0.52510E+00_JPRB, 0.57828E+00_JPRB, 0.61758E+00_JPRB ,&
&  0.65275E+00_JPRB, 0.69071E+00_JPRB, 0.73696E+00_JPRB, 0.79362E+00_JPRB, 0.82921E+00_JPRB, 0.86383E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.94583E+00_JPRB, 0.10079E+01_JPRB, 0.10891E+01_JPRB, 0.12100E+01_JPRB, 0.12096E+01_JPRB, 0.12408E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.14958E+01_JPRB, 0.16255E+01_JPRB, 0.17398E+01_JPRB, 0.20301E+01_JPRB, 0.22161E+01_JPRB, 0.21522E+01_JPRB, 0.20140E+01_JPRB ,&
&  0.19150E+01_JPRB, 0.17105E+01_JPRB, 0.19537E+01_JPRB, 0.17981E+01_JPRB, 0.16771E+01_JPRB, 0.16756E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.15782E+01_JPRB, 0.16485E+01_JPRB, 0.15948E+01_JPRB, 0.15724E+01_JPRB, 0.14559E+01_JPRB, 0.14314E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.13355E+01_JPRB, 0.13255E+01_JPRB, 0.11413E+01_JPRB, 0.97956E+00_JPRB, 0.82987E+00_JPRB, 0.79739E+00_JPRB, 0.73222E+00_JPRB ,&
&  0.68299E+00_JPRB, 0.62545E+00_JPRB, 0.58539E+00_JPRB, 0.54091E+00_JPRB, 0.53029E+00_JPRB, 0.54437E+00_JPRB, 0.56241E+00_JPRB ,&
&  0.62697E+00_JPRB, 0.75407E+00_JPRB, 0.81823E+00_JPRB, 0.85852E+00_JPRB, 0.88772E+00_JPRB, 0.92178E+00_JPRB, 0.73620E+00_JPRB ,&
&  0.71575E+00_JPRB, 0.77474E+00_JPRB, 0.76534E+00_JPRB, 0.74978E+00_JPRB, 0.74134E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.83688E+00_JPRB, 0.81940E+00_JPRB, 0.80096E+00_JPRB, 0.78773E+00_JPRB, 0.77632E+00_JPRB, 0.76658E+00_JPRB, 0.77219E+00_JPRB ,&
&  0.77918E+00_JPRB, 0.76739E+00_JPRB, 0.74921E+00_JPRB, 0.73438E+00_JPRB, 0.71847E+00_JPRB, 0.71457E+00_JPRB, 0.72445E+00_JPRB ,&
&  0.60389E+00_JPRB, 0.54787E+00_JPRB, 0.49226E+00_JPRB, 0.48314E+00_JPRB, 0.53573E+00_JPRB, 0.51784E+00_JPRB, 0.50928E+00_JPRB ,&
&  0.50451E+00_JPRB, 0.51241E+00_JPRB, 0.51490E+00_JPRB, 0.53629E+00_JPRB, 0.56054E+00_JPRB, 0.58815E+00_JPRB, 0.58944E+00_JPRB ,&
&  0.54672E+00_JPRB, 0.49856E+00_JPRB, 0.48258E+00_JPRB, 0.48690E+00_JPRB, 0.52755E+00_JPRB, 0.56825E+00_JPRB, 0.63855E+00_JPRB ,&
&  0.68577E+00_JPRB, 0.72715E+00_JPRB, 0.76714E+00_JPRB, 0.81968E+00_JPRB, 0.85973E+00_JPRB, 0.90523E+00_JPRB, 0.96216E+00_JPRB ,&
&  0.10263E+01_JPRB, 0.11041E+01_JPRB, 0.11860E+01_JPRB, 0.12892E+01_JPRB, 0.13123E+01_JPRB, 0.13660E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.15601E+01_JPRB, 0.16851E+01_JPRB, 0.23109E+01_JPRB, 0.27407E+01_JPRB, 0.27821E+01_JPRB, 0.24976E+01_JPRB ,&
&  0.21895E+01_JPRB, 0.18885E+01_JPRB, 0.18665E+01_JPRB, 0.17647E+01_JPRB, 0.16578E+01_JPRB, 0.17488E+01_JPRB, 0.17046E+01_JPRB ,&
&  0.16408E+01_JPRB, 0.16707E+01_JPRB, 0.16855E+01_JPRB, 0.17495E+01_JPRB, 0.16802E+01_JPRB, 0.17000E+01_JPRB, 0.16545E+01_JPRB ,&
&  0.16772E+01_JPRB, 0.15506E+01_JPRB, 0.11884E+01_JPRB, 0.10126E+01_JPRB, 0.84451E+00_JPRB, 0.79062E+00_JPRB, 0.90283E+00_JPRB ,&
&  0.80760E+00_JPRB, 0.75023E+00_JPRB, 0.70450E+00_JPRB, 0.66640E+00_JPRB, 0.68555E+00_JPRB, 0.77422E+00_JPRB, 0.71952E+00_JPRB ,&
&  0.70380E+00_JPRB, 0.81708E+00_JPRB, 0.98056E+00_JPRB, 0.10969E+01_JPRB, 0.11451E+01_JPRB, 0.11238E+01_JPRB, 0.93768E+00_JPRB ,&
&  0.92230E+00_JPRB, 0.89093E+00_JPRB, 0.87789E+00_JPRB, 0.85389E+00_JPRB, 0.84194E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.97825E+00_JPRB, 0.95125E+00_JPRB, 0.91638E+00_JPRB, 0.86929E+00_JPRB, 0.83973E+00_JPRB, 0.82830E+00_JPRB, 0.82956E+00_JPRB ,&
&  0.82197E+00_JPRB, 0.79730E+00_JPRB, 0.77911E+00_JPRB, 0.76216E+00_JPRB, 0.75548E+00_JPRB, 0.75814E+00_JPRB, 0.76139E+00_JPRB ,&
&  0.77566E+00_JPRB, 0.66322E+00_JPRB, 0.56267E+00_JPRB, 0.50212E+00_JPRB, 0.52012E+00_JPRB, 0.51153E+00_JPRB, 0.51275E+00_JPRB ,&
&  0.51333E+00_JPRB, 0.54441E+00_JPRB, 0.56424E+00_JPRB, 0.61245E+00_JPRB, 0.66913E+00_JPRB, 0.71666E+00_JPRB, 0.71385E+00_JPRB ,&
&  0.66012E+00_JPRB, 0.59087E+00_JPRB, 0.55874E+00_JPRB, 0.56099E+00_JPRB, 0.60699E+00_JPRB, 0.60246E+00_JPRB, 0.66857E+00_JPRB ,&
&  0.71517E+00_JPRB, 0.76343E+00_JPRB, 0.81554E+00_JPRB, 0.86207E+00_JPRB, 0.92287E+00_JPRB, 0.99517E+00_JPRB, 0.10715E+01_JPRB ,&
&  0.11512E+01_JPRB, 0.12133E+01_JPRB, 0.12385E+01_JPRB, 0.12839E+01_JPRB, 0.13937E+01_JPRB, 0.15084E+01_JPRB, 0.14497E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.15219E+01_JPRB, 0.16338E+01_JPRB, 0.22456E+01_JPRB, 0.28081E+01_JPRB, 0.29604E+01_JPRB, 0.28411E+01_JPRB ,&
&  0.25918E+01_JPRB, 0.23364E+01_JPRB, 0.22957E+01_JPRB, 0.20824E+01_JPRB, 0.19670E+01_JPRB, 0.19789E+01_JPRB, 0.19612E+01_JPRB ,&
&  0.18940E+01_JPRB, 0.18987E+01_JPRB, 0.19386E+01_JPRB, 0.18971E+01_JPRB, 0.19065E+01_JPRB, 0.19063E+01_JPRB, 0.19742E+01_JPRB ,&
&  0.21785E+01_JPRB, 0.15149E+01_JPRB, 0.12104E+01_JPRB, 0.12505E+01_JPRB, 0.10339E+01_JPRB, 0.90869E+00_JPRB, 0.92363E+00_JPRB ,&
&  0.81300E+00_JPRB, 0.81844E+00_JPRB, 0.89466E+00_JPRB, 0.87098E+00_JPRB, 0.98566E+00_JPRB, 0.11569E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10198E+01_JPRB, 0.10436E+01_JPRB, 0.12999E+01_JPRB, 0.14723E+01_JPRB, 0.15064E+01_JPRB, 0.14081E+01_JPRB, 0.12683E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.10778E+01_JPRB, 0.10525E+01_JPRB, 0.10271E+01_JPRB, 0.10070E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.11411E+01_JPRB, 0.10990E+01_JPRB, 0.10600E+01_JPRB, 0.10151E+01_JPRB, 0.98185E+00_JPRB, 0.95300E+00_JPRB, 0.91509E+00_JPRB ,&
&  0.87176E+00_JPRB, 0.83618E+00_JPRB, 0.81372E+00_JPRB, 0.80425E+00_JPRB, 0.80563E+00_JPRB, 0.81492E+00_JPRB, 0.81889E+00_JPRB ,&
&  0.82990E+00_JPRB, 0.77894E+00_JPRB, 0.64546E+00_JPRB, 0.55520E+00_JPRB, 0.49633E+00_JPRB, 0.50146E+00_JPRB, 0.50773E+00_JPRB ,&
&  0.53317E+00_JPRB, 0.58641E+00_JPRB, 0.68586E+00_JPRB, 0.80560E+00_JPRB, 0.87361E+00_JPRB, 0.88351E+00_JPRB, 0.87243E+00_JPRB ,&
&  0.80066E+00_JPRB, 0.73301E+00_JPRB, 0.66846E+00_JPRB, 0.68480E+00_JPRB, 0.68957E+00_JPRB, 0.68020E+00_JPRB, 0.74121E+00_JPRB ,&
&  0.78873E+00_JPRB, 0.84539E+00_JPRB, 0.89398E+00_JPRB, 0.96197E+00_JPRB, 0.10195E+01_JPRB, 0.10639E+01_JPRB, 0.11044E+01_JPRB ,&
&  0.11387E+01_JPRB, 0.11815E+01_JPRB, 0.12150E+01_JPRB, 0.12398E+01_JPRB, 0.13074E+01_JPRB, 0.13966E+01_JPRB, 0.14013E+01_JPRB ,&
&  0.15209E+01_JPRB, 0.14981E+01_JPRB, 0.15355E+01_JPRB, 0.22449E+01_JPRB, 0.33082E+01_JPRB, 0.33491E+01_JPRB, 0.30415E+01_JPRB ,&
&  0.29681E+01_JPRB, 0.29117E+01_JPRB, 0.29952E+01_JPRB, 0.29468E+01_JPRB, 0.27210E+01_JPRB, 0.26552E+01_JPRB, 0.25515E+01_JPRB ,&
&  0.24654E+01_JPRB, 0.24108E+01_JPRB, 0.23438E+01_JPRB, 0.23998E+01_JPRB, 0.24443E+01_JPRB, 0.25453E+01_JPRB, 0.26532E+01_JPRB ,&
&  0.26523E+01_JPRB, 0.23551E+01_JPRB, 0.16564E+01_JPRB, 0.15646E+01_JPRB, 0.12455E+01_JPRB, 0.11105E+01_JPRB, 0.11014E+01_JPRB ,&
&  0.97031E+00_JPRB, 0.10161E+01_JPRB, 0.10993E+01_JPRB, 0.10844E+01_JPRB, 0.14134E+01_JPRB, 0.15966E+01_JPRB, 0.16048E+01_JPRB ,&
&  0.15794E+01_JPRB, 0.15233E+01_JPRB, 0.18214E+01_JPRB, 0.19190E+01_JPRB, 0.18912E+01_JPRB, 0.17677E+01_JPRB, 0.16072E+01_JPRB ,&
&  0.14315E+01_JPRB, 0.13257E+01_JPRB, 0.12753E+01_JPRB, 0.12380E+01_JPRB, 0.11870E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.12897E+01_JPRB, 0.12228E+01_JPRB, 0.11594E+01_JPRB, 0.11125E+01_JPRB, 0.10747E+01_JPRB, 0.10292E+01_JPRB, 0.96922E+00_JPRB ,&
&  0.91594E+00_JPRB, 0.88845E+00_JPRB, 0.87444E+00_JPRB, 0.86070E+00_JPRB, 0.85594E+00_JPRB, 0.86014E+00_JPRB, 0.86663E+00_JPRB ,&
&  0.86612E+00_JPRB, 0.78744E+00_JPRB, 0.64298E+00_JPRB, 0.54756E+00_JPRB, 0.48753E+00_JPRB, 0.47104E+00_JPRB, 0.49357E+00_JPRB ,&
&  0.54795E+00_JPRB, 0.70596E+00_JPRB, 0.92896E+00_JPRB, 0.10535E+01_JPRB, 0.11253E+01_JPRB, 0.11379E+01_JPRB, 0.10923E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.91292E+00_JPRB, 0.89040E+00_JPRB, 0.82492E+00_JPRB, 0.83079E+00_JPRB, 0.86312E+00_JPRB, 0.86735E+00_JPRB ,&
&  0.85242E+00_JPRB, 0.88356E+00_JPRB, 0.91480E+00_JPRB, 0.96719E+00_JPRB, 0.10009E+01_JPRB, 0.10476E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.12596E+01_JPRB, 0.12257E+01_JPRB, 0.11949E+01_JPRB, 0.13851E+01_JPRB, 0.16247E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.19160E+01_JPRB, 0.27686E+01_JPRB, 0.32940E+01_JPRB, 0.35417E+01_JPRB, 0.38709E+01_JPRB, 0.38043E+01_JPRB ,&
&  0.34697E+01_JPRB, 0.31697E+01_JPRB, 0.30252E+01_JPRB, 0.27104E+01_JPRB, 0.31094E+01_JPRB, 0.32402E+01_JPRB, 0.32602E+01_JPRB ,&
&  0.32674E+01_JPRB, 0.31385E+01_JPRB, 0.30492E+01_JPRB, 0.30951E+01_JPRB, 0.32134E+01_JPRB, 0.33531E+01_JPRB, 0.32445E+01_JPRB ,&
&  0.28799E+01_JPRB, 0.29881E+01_JPRB, 0.25500E+01_JPRB, 0.20654E+01_JPRB, 0.16810E+01_JPRB, 0.15500E+01_JPRB, 0.14273E+01_JPRB ,&
&  0.12929E+01_JPRB, 0.12513E+01_JPRB, 0.13868E+01_JPRB, 0.15920E+01_JPRB, 0.19198E+01_JPRB, 0.22313E+01_JPRB, 0.22512E+01_JPRB ,&
&  0.21433E+01_JPRB, 0.23558E+01_JPRB, 0.22893E+01_JPRB, 0.21415E+01_JPRB, 0.20452E+01_JPRB, 0.18934E+01_JPRB, 0.17660E+01_JPRB ,&
&  0.16356E+01_JPRB, 0.15543E+01_JPRB, 0.14982E+01_JPRB, 0.14393E+01_JPRB, 0.13661E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.13832E+01_JPRB, 0.13101E+01_JPRB, 0.12401E+01_JPRB, 0.11808E+01_JPRB, 0.11202E+01_JPRB, 0.10655E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.96197E+00_JPRB, 0.93759E+00_JPRB, 0.91478E+00_JPRB, 0.90068E+00_JPRB, 0.89681E+00_JPRB, 0.88757E+00_JPRB, 0.88533E+00_JPRB ,&
&  0.87337E+00_JPRB, 0.75617E+00_JPRB, 0.63201E+00_JPRB, 0.58191E+00_JPRB, 0.53198E+00_JPRB, 0.49109E+00_JPRB, 0.50411E+00_JPRB ,&
&  0.61441E+00_JPRB, 0.91756E+00_JPRB, 0.11911E+01_JPRB, 0.13391E+01_JPRB, 0.13826E+01_JPRB, 0.13532E+01_JPRB, 0.12772E+01_JPRB ,&
&  0.11586E+01_JPRB, 0.11726E+01_JPRB, 0.11697E+01_JPRB, 0.11167E+01_JPRB, 0.10468E+01_JPRB, 0.10125E+01_JPRB, 0.98814E+00_JPRB ,&
&  0.96636E+00_JPRB, 0.98531E+00_JPRB, 0.10187E+01_JPRB, 0.10567E+01_JPRB, 0.11204E+01_JPRB, 0.11969E+01_JPRB, 0.12761E+01_JPRB ,&
&  0.12969E+01_JPRB, 0.13027E+01_JPRB, 0.12493E+01_JPRB, 0.12641E+01_JPRB, 0.13244E+01_JPRB, 0.18285E+01_JPRB, 0.21195E+01_JPRB ,&
&  0.26557E+01_JPRB, 0.31060E+01_JPRB, 0.32250E+01_JPRB, 0.37858E+01_JPRB, 0.32602E+01_JPRB, 0.36575E+01_JPRB, 0.40538E+01_JPRB ,&
&  0.35865E+01_JPRB, 0.32074E+01_JPRB, 0.30865E+01_JPRB, 0.26490E+01_JPRB, 0.26272E+01_JPRB, 0.38643E+01_JPRB, 0.41684E+01_JPRB ,&
&  0.42953E+01_JPRB, 0.45370E+01_JPRB, 0.42865E+01_JPRB, 0.40574E+01_JPRB, 0.38369E+01_JPRB, 0.35355E+01_JPRB, 0.27001E+01_JPRB ,&
&  0.26973E+01_JPRB, 0.24498E+01_JPRB, 0.25205E+01_JPRB, 0.22299E+01_JPRB, 0.20384E+01_JPRB, 0.18339E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.14800E+01_JPRB, 0.19426E+01_JPRB, 0.25883E+01_JPRB, 0.29998E+01_JPRB, 0.27585E+01_JPRB, 0.31755E+01_JPRB ,&
&  0.30464E+01_JPRB, 0.26220E+01_JPRB, 0.21470E+01_JPRB, 0.19883E+01_JPRB, 0.18629E+01_JPRB, 0.17583E+01_JPRB, 0.16850E+01_JPRB ,&
&  0.16162E+01_JPRB, 0.15516E+01_JPRB, 0.15059E+01_JPRB, 0.14814E+01_JPRB, 0.14402E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.14128E+01_JPRB, 0.13423E+01_JPRB, 0.12675E+01_JPRB, 0.12069E+01_JPRB, 0.11493E+01_JPRB, 0.10956E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.10216E+01_JPRB, 0.98895E+00_JPRB, 0.94348E+00_JPRB, 0.91371E+00_JPRB, 0.89851E+00_JPRB, 0.90497E+00_JPRB, 0.89325E+00_JPRB ,&
&  0.87249E+00_JPRB, 0.78750E+00_JPRB, 0.66493E+00_JPRB, 0.61411E+00_JPRB, 0.60624E+00_JPRB, 0.56673E+00_JPRB, 0.56463E+00_JPRB ,&
&  0.79884E+00_JPRB, 0.12967E+01_JPRB, 0.15416E+01_JPRB, 0.15686E+01_JPRB, 0.15731E+01_JPRB, 0.15260E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.13972E+01_JPRB, 0.13366E+01_JPRB, 0.12770E+01_JPRB, 0.12095E+01_JPRB, 0.11514E+01_JPRB, 0.10917E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10398E+01_JPRB, 0.10714E+01_JPRB, 0.10803E+01_JPRB, 0.11559E+01_JPRB, 0.12075E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.12480E+01_JPRB, 0.12791E+01_JPRB, 0.12760E+01_JPRB, 0.13146E+01_JPRB, 0.15842E+01_JPRB, 0.22224E+01_JPRB, 0.27722E+01_JPRB ,&
&  0.31207E+01_JPRB, 0.34161E+01_JPRB, 0.37529E+01_JPRB, 0.39432E+01_JPRB, 0.44958E+01_JPRB, 0.43726E+01_JPRB, 0.43675E+01_JPRB ,&
&  0.33445E+01_JPRB, 0.27157E+01_JPRB, 0.22509E+01_JPRB, 0.18894E+01_JPRB, 0.30865E+01_JPRB, 0.51298E+01_JPRB, 0.58791E+01_JPRB ,&
&  0.59255E+01_JPRB, 0.61703E+01_JPRB, 0.59206E+01_JPRB, 0.49620E+01_JPRB, 0.41656E+01_JPRB, 0.33482E+01_JPRB, 0.37629E+01_JPRB ,&
&  0.36118E+01_JPRB, 0.34716E+01_JPRB, 0.30482E+01_JPRB, 0.24409E+01_JPRB, 0.19856E+01_JPRB, 0.20730E+01_JPRB, 0.18807E+01_JPRB ,&
&  0.17295E+01_JPRB, 0.18555E+01_JPRB, 0.30952E+01_JPRB, 0.37589E+01_JPRB, 0.37092E+01_JPRB, 0.35304E+01_JPRB, 0.39105E+01_JPRB ,&
&  0.31405E+01_JPRB, 0.25203E+01_JPRB, 0.21049E+01_JPRB, 0.18400E+01_JPRB, 0.17133E+01_JPRB, 0.16385E+01_JPRB, 0.16053E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.15665E+01_JPRB, 0.15323E+01_JPRB, 0.14921E+01_JPRB, 0.14536E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.14468E+01_JPRB, 0.13767E+01_JPRB, 0.13143E+01_JPRB, 0.12496E+01_JPRB, 0.11867E+01_JPRB, 0.11399E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.10642E+01_JPRB, 0.10250E+01_JPRB, 0.98531E+00_JPRB, 0.95400E+00_JPRB, 0.93892E+00_JPRB, 0.93336E+00_JPRB, 0.91652E+00_JPRB ,&
&  0.90027E+00_JPRB, 0.86231E+00_JPRB, 0.72952E+00_JPRB, 0.67619E+00_JPRB, 0.66811E+00_JPRB, 0.66156E+00_JPRB, 0.71260E+00_JPRB ,&
&  0.11902E+01_JPRB, 0.17767E+01_JPRB, 0.18240E+01_JPRB, 0.16925E+01_JPRB, 0.16615E+01_JPRB, 0.14027E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.14186E+01_JPRB, 0.13758E+01_JPRB, 0.13197E+01_JPRB, 0.12617E+01_JPRB, 0.12122E+01_JPRB, 0.11727E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.11079E+01_JPRB, 0.11396E+01_JPRB, 0.12189E+01_JPRB, 0.12557E+01_JPRB, 0.12647E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.12607E+01_JPRB, 0.12833E+01_JPRB, 0.14597E+01_JPRB, 0.18439E+01_JPRB, 0.26305E+01_JPRB, 0.38901E+01_JPRB ,&
&  0.39361E+01_JPRB, 0.43541E+01_JPRB, 0.42423E+01_JPRB, 0.50127E+01_JPRB, 0.54246E+01_JPRB, 0.50103E+01_JPRB, 0.45572E+01_JPRB ,&
&  0.33102E+01_JPRB, 0.26702E+01_JPRB, 0.24677E+01_JPRB, 0.28284E+01_JPRB, 0.39193E+01_JPRB, 0.51989E+01_JPRB, 0.66254E+01_JPRB ,&
&  0.58985E+01_JPRB, 0.58315E+01_JPRB, 0.57790E+01_JPRB, 0.52873E+01_JPRB, 0.32201E+01_JPRB, 0.24749E+01_JPRB, 0.35587E+01_JPRB ,&
&  0.35713E+01_JPRB, 0.21303E+01_JPRB, 0.14961E+01_JPRB, 0.17620E+01_JPRB, 0.13159E+01_JPRB, 0.14865E+01_JPRB, 0.19709E+01_JPRB ,&
&  0.21044E+01_JPRB, 0.25232E+01_JPRB, 0.38035E+01_JPRB, 0.37157E+01_JPRB, 0.37767E+01_JPRB, 0.43864E+01_JPRB, 0.36384E+01_JPRB ,&
&  0.28677E+01_JPRB, 0.22754E+01_JPRB, 0.21291E+01_JPRB, 0.18856E+01_JPRB, 0.17673E+01_JPRB, 0.16946E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.16975E+01_JPRB, 0.16895E+01_JPRB, 0.16426E+01_JPRB, 0.15745E+01_JPRB, 0.15097E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.14972E+01_JPRB, 0.14317E+01_JPRB, 0.13659E+01_JPRB, 0.13064E+01_JPRB, 0.12543E+01_JPRB, 0.12067E+01_JPRB, 0.11432E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.10232E+01_JPRB, 0.10036E+01_JPRB, 0.98121E+00_JPRB, 0.97038E+00_JPRB, 0.97207E+00_JPRB, 0.95639E+00_JPRB ,&
&  0.92510E+00_JPRB, 0.88730E+00_JPRB, 0.82717E+00_JPRB, 0.77150E+00_JPRB, 0.74728E+00_JPRB, 0.72955E+00_JPRB, 0.95577E+00_JPRB ,&
&  0.17322E+01_JPRB, 0.22835E+01_JPRB, 0.20944E+01_JPRB, 0.17669E+01_JPRB, 0.15035E+01_JPRB, 0.13043E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.14322E+01_JPRB, 0.14219E+01_JPRB, 0.13506E+01_JPRB, 0.12958E+01_JPRB, 0.12494E+01_JPRB, 0.12050E+01_JPRB, 0.11827E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.11632E+01_JPRB, 0.12237E+01_JPRB, 0.12476E+01_JPRB, 0.12639E+01_JPRB, 0.12567E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.12864E+01_JPRB, 0.13029E+01_JPRB, 0.13766E+01_JPRB, 0.16307E+01_JPRB, 0.23511E+01_JPRB, 0.34523E+01_JPRB, 0.38291E+01_JPRB ,&
&  0.39182E+01_JPRB, 0.42121E+01_JPRB, 0.50360E+01_JPRB, 0.52405E+01_JPRB, 0.54208E+01_JPRB, 0.52130E+01_JPRB, 0.43437E+01_JPRB ,&
&  0.37629E+01_JPRB, 0.33628E+01_JPRB, 0.29345E+01_JPRB, 0.33550E+01_JPRB, 0.48584E+01_JPRB, 0.52337E+01_JPRB, 0.62117E+01_JPRB ,&
&  0.61565E+01_JPRB, 0.55135E+01_JPRB, 0.43007E+01_JPRB, 0.33065E+01_JPRB, 0.34264E+01_JPRB, 0.25582E+01_JPRB, 0.16350E+01_JPRB ,&
&  0.14490E+01_JPRB, 0.12883E+01_JPRB, 0.11316E+01_JPRB, 0.11322E+01_JPRB, 0.10718E+01_JPRB, 0.13356E+01_JPRB, 0.19719E+01_JPRB ,&
&  0.26366E+01_JPRB, 0.31636E+01_JPRB, 0.36042E+01_JPRB, 0.34197E+01_JPRB, 0.33034E+01_JPRB, 0.39793E+01_JPRB, 0.37276E+01_JPRB ,&
&  0.24005E+01_JPRB, 0.20270E+01_JPRB, 0.20061E+01_JPRB, 0.18795E+01_JPRB, 0.17896E+01_JPRB, 0.17619E+01_JPRB, 0.17974E+01_JPRB ,&
&  0.18402E+01_JPRB, 0.18194E+01_JPRB, 0.17480E+01_JPRB, 0.16497E+01_JPRB, 0.15633E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.14946E+01_JPRB, 0.14107E+01_JPRB, 0.13565E+01_JPRB, 0.13262E+01_JPRB, 0.12909E+01_JPRB, 0.12224E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.10110E+01_JPRB, 0.10024E+01_JPRB, 0.99122E+00_JPRB, 0.98152E+00_JPRB, 0.97539E+00_JPRB, 0.95822E+00_JPRB ,&
&  0.91829E+00_JPRB, 0.89280E+00_JPRB, 0.88714E+00_JPRB, 0.86510E+00_JPRB, 0.87464E+00_JPRB, 0.88274E+00_JPRB, 0.12917E+01_JPRB ,&
&  0.23432E+01_JPRB, 0.27237E+01_JPRB, 0.22544E+01_JPRB, 0.17386E+01_JPRB, 0.15231E+01_JPRB, 0.14573E+01_JPRB, 0.14984E+01_JPRB ,&
&  0.14540E+01_JPRB, 0.13879E+01_JPRB, 0.12696E+01_JPRB, 0.12239E+01_JPRB, 0.12134E+01_JPRB, 0.11964E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.11805E+01_JPRB, 0.12082E+01_JPRB, 0.12163E+01_JPRB, 0.12284E+01_JPRB, 0.12555E+01_JPRB, 0.12910E+01_JPRB, 0.13483E+01_JPRB ,&
&  0.13926E+01_JPRB, 0.14438E+01_JPRB, 0.16056E+01_JPRB, 0.21104E+01_JPRB, 0.29015E+01_JPRB, 0.33065E+01_JPRB, 0.39340E+01_JPRB ,&
&  0.45793E+01_JPRB, 0.50715E+01_JPRB, 0.48165E+01_JPRB, 0.56459E+01_JPRB, 0.54620E+01_JPRB, 0.48000E+01_JPRB, 0.42188E+01_JPRB ,&
&  0.38463E+01_JPRB, 0.42733E+01_JPRB, 0.35084E+01_JPRB, 0.32391E+01_JPRB, 0.52970E+01_JPRB, 0.58787E+01_JPRB, 0.63090E+01_JPRB ,&
&  0.67060E+01_JPRB, 0.57066E+01_JPRB, 0.45331E+01_JPRB, 0.39032E+01_JPRB, 0.51008E+01_JPRB, 0.47675E+01_JPRB, 0.19825E+01_JPRB ,&
&  0.14567E+01_JPRB, 0.12889E+01_JPRB, 0.10999E+01_JPRB, 0.10106E+01_JPRB, 0.10072E+01_JPRB, 0.12918E+01_JPRB, 0.22913E+01_JPRB ,&
&  0.25407E+01_JPRB, 0.33731E+01_JPRB, 0.34466E+01_JPRB, 0.33453E+01_JPRB, 0.30363E+01_JPRB, 0.29653E+01_JPRB, 0.28483E+01_JPRB ,&
&  0.24773E+01_JPRB, 0.21662E+01_JPRB, 0.18669E+01_JPRB, 0.17062E+01_JPRB, 0.16441E+01_JPRB, 0.16763E+01_JPRB, 0.17825E+01_JPRB ,&
&  0.18366E+01_JPRB, 0.18333E+01_JPRB, 0.17693E+01_JPRB, 0.16714E+01_JPRB, 0.15731E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.14360E+01_JPRB, 0.13985E+01_JPRB, 0.13727E+01_JPRB, 0.13219E+01_JPRB, 0.12673E+01_JPRB, 0.11953E+01_JPRB, 0.11052E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.10311E+01_JPRB, 0.10237E+01_JPRB, 0.10027E+01_JPRB, 0.97621E+00_JPRB, 0.95711E+00_JPRB, 0.94745E+00_JPRB ,&
&  0.91998E+00_JPRB, 0.88778E+00_JPRB, 0.88299E+00_JPRB, 0.94904E+00_JPRB, 0.10384E+01_JPRB, 0.12362E+01_JPRB, 0.17523E+01_JPRB ,&
&  0.28498E+01_JPRB, 0.30364E+01_JPRB, 0.24336E+01_JPRB, 0.19857E+01_JPRB, 0.17599E+01_JPRB, 0.16930E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.14212E+01_JPRB, 0.13573E+01_JPRB, 0.12347E+01_JPRB, 0.12333E+01_JPRB, 0.12362E+01_JPRB, 0.12469E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.12448E+01_JPRB, 0.12526E+01_JPRB, 0.12751E+01_JPRB, 0.13138E+01_JPRB, 0.13940E+01_JPRB, 0.14720E+01_JPRB ,&
&  0.15563E+01_JPRB, 0.16735E+01_JPRB, 0.19951E+01_JPRB, 0.25643E+01_JPRB, 0.27945E+01_JPRB, 0.35779E+01_JPRB, 0.40146E+01_JPRB ,&
&  0.44017E+01_JPRB, 0.45213E+01_JPRB, 0.43667E+01_JPRB, 0.49541E+01_JPRB, 0.52388E+01_JPRB, 0.47009E+01_JPRB, 0.42284E+01_JPRB ,&
&  0.44811E+01_JPRB, 0.39402E+01_JPRB, 0.35468E+01_JPRB, 0.32413E+01_JPRB, 0.43631E+01_JPRB, 0.74571E+01_JPRB, 0.58542E+01_JPRB ,&
&  0.56531E+01_JPRB, 0.63909E+01_JPRB, 0.53497E+01_JPRB, 0.47909E+01_JPRB, 0.52928E+01_JPRB, 0.60101E+01_JPRB, 0.45730E+01_JPRB ,&
&  0.17898E+01_JPRB, 0.12586E+01_JPRB, 0.12574E+01_JPRB, 0.12254E+01_JPRB, 0.10792E+01_JPRB, 0.13470E+01_JPRB, 0.35145E+01_JPRB ,&
&  0.29465E+01_JPRB, 0.37511E+01_JPRB, 0.36409E+01_JPRB, 0.33265E+01_JPRB, 0.30714E+01_JPRB, 0.26357E+01_JPRB, 0.24661E+01_JPRB ,&
&  0.20371E+01_JPRB, 0.17744E+01_JPRB, 0.16471E+01_JPRB, 0.15767E+01_JPRB, 0.15719E+01_JPRB, 0.16004E+01_JPRB, 0.16817E+01_JPRB ,&
&  0.17355E+01_JPRB, 0.17453E+01_JPRB, 0.16865E+01_JPRB, 0.15941E+01_JPRB, 0.15053E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.13923E+01_JPRB, 0.13733E+01_JPRB, 0.13372E+01_JPRB, 0.12791E+01_JPRB, 0.12268E+01_JPRB, 0.11518E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.10548E+01_JPRB, 0.10325E+01_JPRB, 0.10047E+01_JPRB, 0.97351E+00_JPRB, 0.94645E+00_JPRB, 0.93488E+00_JPRB ,&
&  0.92485E+00_JPRB, 0.89156E+00_JPRB, 0.88373E+00_JPRB, 0.96902E+00_JPRB, 0.12494E+01_JPRB, 0.16094E+01_JPRB, 0.19602E+01_JPRB ,&
&  0.32604E+01_JPRB, 0.33439E+01_JPRB, 0.23445E+01_JPRB, 0.20626E+01_JPRB, 0.19705E+01_JPRB, 0.18703E+01_JPRB, 0.16454E+01_JPRB ,&
&  0.14809E+01_JPRB, 0.14056E+01_JPRB, 0.12819E+01_JPRB, 0.12535E+01_JPRB, 0.12720E+01_JPRB, 0.13241E+01_JPRB, 0.13645E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.13214E+01_JPRB, 0.13767E+01_JPRB, 0.14050E+01_JPRB, 0.14520E+01_JPRB, 0.15581E+01_JPRB, 0.16548E+01_JPRB ,&
&  0.17882E+01_JPRB, 0.20105E+01_JPRB, 0.24138E+01_JPRB, 0.27454E+01_JPRB, 0.31985E+01_JPRB, 0.35332E+01_JPRB, 0.39772E+01_JPRB ,&
&  0.40082E+01_JPRB, 0.41221E+01_JPRB, 0.42904E+01_JPRB, 0.44926E+01_JPRB, 0.46130E+01_JPRB, 0.44973E+01_JPRB, 0.43869E+01_JPRB ,&
&  0.45622E+01_JPRB, 0.39232E+01_JPRB, 0.34234E+01_JPRB, 0.32971E+01_JPRB, 0.41048E+01_JPRB, 0.66493E+01_JPRB, 0.74643E+01_JPRB ,&
&  0.75372E+01_JPRB, 0.54048E+01_JPRB, 0.48007E+01_JPRB, 0.54458E+01_JPRB, 0.51841E+01_JPRB, 0.53778E+01_JPRB, 0.68566E+01_JPRB ,&
&  0.60008E+01_JPRB, 0.31060E+01_JPRB, 0.24827E+01_JPRB, 0.31066E+01_JPRB, 0.18783E+01_JPRB, 0.14497E+01_JPRB, 0.23176E+01_JPRB ,&
&  0.28443E+01_JPRB, 0.33724E+01_JPRB, 0.33557E+01_JPRB, 0.32308E+01_JPRB, 0.29927E+01_JPRB, 0.23737E+01_JPRB, 0.20175E+01_JPRB ,&
&  0.16581E+01_JPRB, 0.15483E+01_JPRB, 0.14951E+01_JPRB, 0.14228E+01_JPRB, 0.14023E+01_JPRB, 0.14573E+01_JPRB, 0.15253E+01_JPRB ,&
&  0.15751E+01_JPRB, 0.15876E+01_JPRB, 0.15656E+01_JPRB, 0.15123E+01_JPRB, 0.14394E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.13403E+01_JPRB, 0.13296E+01_JPRB, 0.13149E+01_JPRB, 0.12533E+01_JPRB, 0.11922E+01_JPRB, 0.11327E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.10605E+01_JPRB, 0.10333E+01_JPRB, 0.99437E+00_JPRB, 0.97724E+00_JPRB, 0.95349E+00_JPRB, 0.95874E+00_JPRB ,&
&  0.97045E+00_JPRB, 0.94853E+00_JPRB, 0.98850E+00_JPRB, 0.12384E+01_JPRB, 0.17142E+01_JPRB, 0.26936E+01_JPRB, 0.22763E+01_JPRB ,&
&  0.32426E+01_JPRB, 0.36157E+01_JPRB, 0.24102E+01_JPRB, 0.22420E+01_JPRB, 0.21783E+01_JPRB, 0.21478E+01_JPRB, 0.18364E+01_JPRB ,&
&  0.16139E+01_JPRB, 0.15142E+01_JPRB, 0.14340E+01_JPRB, 0.13609E+01_JPRB, 0.13719E+01_JPRB, 0.14037E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.14758E+01_JPRB, 0.15181E+01_JPRB, 0.15490E+01_JPRB, 0.16359E+01_JPRB, 0.17497E+01_JPRB, 0.18837E+01_JPRB ,&
&  0.21026E+01_JPRB, 0.23884E+01_JPRB, 0.28042E+01_JPRB, 0.29581E+01_JPRB, 0.32244E+01_JPRB, 0.35711E+01_JPRB, 0.37575E+01_JPRB ,&
&  0.36216E+01_JPRB, 0.34624E+01_JPRB, 0.38462E+01_JPRB, 0.41861E+01_JPRB, 0.42427E+01_JPRB, 0.42282E+01_JPRB, 0.44034E+01_JPRB ,&
&  0.48047E+01_JPRB, 0.46453E+01_JPRB, 0.47449E+01_JPRB, 0.41704E+01_JPRB, 0.50041E+01_JPRB, 0.67278E+01_JPRB, 0.68245E+01_JPRB ,&
&  0.82279E+01_JPRB, 0.70260E+01_JPRB, 0.65048E+01_JPRB, 0.58030E+01_JPRB, 0.51019E+01_JPRB, 0.50049E+01_JPRB, 0.53475E+01_JPRB ,&
&  0.55204E+01_JPRB, 0.59949E+01_JPRB, 0.55949E+01_JPRB, 0.33830E+01_JPRB, 0.21716E+01_JPRB, 0.16981E+01_JPRB, 0.19397E+01_JPRB ,&
&  0.25566E+01_JPRB, 0.27180E+01_JPRB, 0.27199E+01_JPRB, 0.39148E+01_JPRB, 0.25847E+01_JPRB, 0.19454E+01_JPRB, 0.15895E+01_JPRB ,&
&  0.14035E+01_JPRB, 0.13645E+01_JPRB, 0.13218E+01_JPRB, 0.12532E+01_JPRB, 0.12500E+01_JPRB, 0.12902E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.14053E+01_JPRB, 0.14472E+01_JPRB, 0.14711E+01_JPRB, 0.14390E+01_JPRB, 0.13810E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.12685E+01_JPRB, 0.12694E+01_JPRB, 0.12368E+01_JPRB, 0.11921E+01_JPRB, 0.11583E+01_JPRB, 0.11323E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10338E+01_JPRB, 0.10121E+01_JPRB, 0.99206E+00_JPRB, 0.10058E+01_JPRB, 0.10259E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.11955E+01_JPRB, 0.14117E+01_JPRB, 0.19097E+01_JPRB, 0.25768E+01_JPRB, 0.34849E+01_JPRB, 0.35005E+01_JPRB ,&
&  0.31845E+01_JPRB, 0.39635E+01_JPRB, 0.24463E+01_JPRB, 0.24816E+01_JPRB, 0.24932E+01_JPRB, 0.27537E+01_JPRB, 0.23953E+01_JPRB ,&
&  0.19330E+01_JPRB, 0.17710E+01_JPRB, 0.16702E+01_JPRB, 0.15134E+01_JPRB, 0.14760E+01_JPRB, 0.14566E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.15653E+01_JPRB, 0.16638E+01_JPRB, 0.17275E+01_JPRB, 0.18003E+01_JPRB, 0.19018E+01_JPRB, 0.20134E+01_JPRB, 0.22117E+01_JPRB ,&
&  0.25024E+01_JPRB, 0.28247E+01_JPRB, 0.29957E+01_JPRB, 0.31069E+01_JPRB, 0.31996E+01_JPRB, 0.34544E+01_JPRB, 0.35922E+01_JPRB ,&
&  0.35310E+01_JPRB, 0.33896E+01_JPRB, 0.36503E+01_JPRB, 0.37456E+01_JPRB, 0.38062E+01_JPRB, 0.42420E+01_JPRB, 0.43898E+01_JPRB ,&
&  0.49765E+01_JPRB, 0.47354E+01_JPRB, 0.55141E+01_JPRB, 0.52268E+01_JPRB, 0.55013E+01_JPRB, 0.61574E+01_JPRB, 0.53975E+01_JPRB ,&
&  0.53085E+01_JPRB, 0.57167E+01_JPRB, 0.57175E+01_JPRB, 0.55375E+01_JPRB, 0.52064E+01_JPRB, 0.49347E+01_JPRB, 0.45657E+01_JPRB ,&
&  0.39611E+01_JPRB, 0.38262E+01_JPRB, 0.33788E+01_JPRB, 0.22269E+01_JPRB, 0.17557E+01_JPRB, 0.15838E+01_JPRB, 0.17610E+01_JPRB ,&
&  0.22210E+01_JPRB, 0.24079E+01_JPRB, 0.27981E+01_JPRB, 0.28095E+01_JPRB, 0.18529E+01_JPRB, 0.14358E+01_JPRB, 0.13104E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.12138E+01_JPRB, 0.11442E+01_JPRB, 0.10934E+01_JPRB, 0.11086E+01_JPRB, 0.11634E+01_JPRB, 0.12136E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.13171E+01_JPRB, 0.13134E+01_JPRB, 0.12849E+01_JPRB, 0.12738E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.12810E+01_JPRB, 0.12903E+01_JPRB, 0.12964E+01_JPRB, 0.13254E+01_JPRB, 0.13700E+01_JPRB, 0.14314E+01_JPRB, 0.14219E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.10099E+01_JPRB, 0.10271E+01_JPRB, 0.10698E+01_JPRB, 0.11602E+01_JPRB, 0.12475E+01_JPRB, 0.13611E+01_JPRB ,&
&  0.15330E+01_JPRB, 0.17683E+01_JPRB, 0.21737E+01_JPRB, 0.27504E+01_JPRB, 0.34373E+01_JPRB, 0.42754E+01_JPRB, 0.44416E+01_JPRB ,&
&  0.40910E+01_JPRB, 0.29814E+01_JPRB, 0.27185E+01_JPRB, 0.26978E+01_JPRB, 0.25470E+01_JPRB, 0.28197E+01_JPRB, 0.27324E+01_JPRB ,&
&  0.22805E+01_JPRB, 0.20434E+01_JPRB, 0.19302E+01_JPRB, 0.17858E+01_JPRB, 0.16873E+01_JPRB, 0.17430E+01_JPRB, 0.18327E+01_JPRB ,&
&  0.19587E+01_JPRB, 0.20406E+01_JPRB, 0.20733E+01_JPRB, 0.21114E+01_JPRB, 0.22015E+01_JPRB, 0.23334E+01_JPRB, 0.25957E+01_JPRB ,&
&  0.28866E+01_JPRB, 0.31167E+01_JPRB, 0.32093E+01_JPRB, 0.31418E+01_JPRB, 0.32323E+01_JPRB, 0.33734E+01_JPRB, 0.34355E+01_JPRB ,&
&  0.34459E+01_JPRB, 0.35496E+01_JPRB, 0.36639E+01_JPRB, 0.38010E+01_JPRB, 0.37276E+01_JPRB, 0.42696E+01_JPRB, 0.43791E+01_JPRB ,&
&  0.47036E+01_JPRB, 0.46128E+01_JPRB, 0.53738E+01_JPRB, 0.48679E+01_JPRB, 0.54227E+01_JPRB, 0.53248E+01_JPRB, 0.48045E+01_JPRB ,&
&  0.47091E+01_JPRB, 0.52449E+01_JPRB, 0.54044E+01_JPRB, 0.52580E+01_JPRB, 0.50924E+01_JPRB, 0.43553E+01_JPRB, 0.44203E+01_JPRB ,&
&  0.34166E+01_JPRB, 0.33896E+01_JPRB, 0.22939E+01_JPRB, 0.17019E+01_JPRB, 0.13941E+01_JPRB, 0.15803E+01_JPRB, 0.18265E+01_JPRB ,&
&  0.21890E+01_JPRB, 0.21711E+01_JPRB, 0.19568E+01_JPRB, 0.18892E+01_JPRB, 0.15289E+01_JPRB, 0.12674E+01_JPRB, 0.12097E+01_JPRB ,&
&  0.11931E+01_JPRB, 0.11332E+01_JPRB, 0.10680E+01_JPRB, 0.10586E+01_JPRB, 0.10804E+01_JPRB, 0.11517E+01_JPRB, 0.11894E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.12475E+01_JPRB, 0.12431E+01_JPRB, 0.12470E+01_JPRB, 0.12578E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.12894E+01_JPRB, 0.12734E+01_JPRB, 0.12746E+01_JPRB, 0.12701E+01_JPRB, 0.12577E+01_JPRB, 0.11959E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10536E+01_JPRB, 0.11465E+01_JPRB, 0.12736E+01_JPRB, 0.14314E+01_JPRB, 0.16032E+01_JPRB, 0.17974E+01_JPRB ,&
&  0.20495E+01_JPRB, 0.23704E+01_JPRB, 0.27728E+01_JPRB, 0.31278E+01_JPRB, 0.34877E+01_JPRB, 0.39736E+01_JPRB, 0.43217E+01_JPRB ,&
&  0.37765E+01_JPRB, 0.24160E+01_JPRB, 0.27317E+01_JPRB, 0.29040E+01_JPRB, 0.27761E+01_JPRB, 0.27972E+01_JPRB, 0.26604E+01_JPRB ,&
&  0.24112E+01_JPRB, 0.22940E+01_JPRB, 0.22397E+01_JPRB, 0.21832E+01_JPRB, 0.21012E+01_JPRB, 0.21806E+01_JPRB, 0.22543E+01_JPRB ,&
&  0.22899E+01_JPRB, 0.22725E+01_JPRB, 0.22655E+01_JPRB, 0.23690E+01_JPRB, 0.25741E+01_JPRB, 0.27176E+01_JPRB, 0.29228E+01_JPRB ,&
&  0.31009E+01_JPRB, 0.33170E+01_JPRB, 0.34215E+01_JPRB, 0.34692E+01_JPRB, 0.36144E+01_JPRB, 0.37145E+01_JPRB, 0.37084E+01_JPRB ,&
&  0.36544E+01_JPRB, 0.36387E+01_JPRB, 0.37886E+01_JPRB, 0.42469E+01_JPRB, 0.45262E+01_JPRB, 0.41561E+01_JPRB, 0.44118E+01_JPRB ,&
&  0.46079E+01_JPRB, 0.48286E+01_JPRB, 0.47243E+01_JPRB, 0.61723E+01_JPRB, 0.46890E+01_JPRB, 0.46468E+01_JPRB, 0.40032E+01_JPRB ,&
&  0.44838E+01_JPRB, 0.46826E+01_JPRB, 0.49694E+01_JPRB, 0.49977E+01_JPRB, 0.47270E+01_JPRB, 0.41141E+01_JPRB, 0.42228E+01_JPRB ,&
&  0.36257E+01_JPRB, 0.30702E+01_JPRB, 0.21599E+01_JPRB, 0.15800E+01_JPRB, 0.15441E+01_JPRB, 0.24478E+01_JPRB, 0.24142E+01_JPRB ,&
&  0.21567E+01_JPRB, 0.17901E+01_JPRB, 0.17842E+01_JPRB, 0.19938E+01_JPRB, 0.15591E+01_JPRB, 0.12033E+01_JPRB, 0.11181E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.11107E+01_JPRB, 0.10693E+01_JPRB, 0.10740E+01_JPRB, 0.11199E+01_JPRB, 0.11923E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.12551E+01_JPRB, 0.12413E+01_JPRB, 0.12444E+01_JPRB, 0.12566E+01_JPRB, 0.12912E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.12202E+01_JPRB, 0.11838E+01_JPRB, 0.11516E+01_JPRB, 0.11166E+01_JPRB, 0.10772E+01_JPRB, 0.10542E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.11538E+01_JPRB, 0.12603E+01_JPRB, 0.13927E+01_JPRB, 0.15659E+01_JPRB, 0.17719E+01_JPRB, 0.19957E+01_JPRB ,&
&  0.22858E+01_JPRB, 0.25695E+01_JPRB, 0.27350E+01_JPRB, 0.28556E+01_JPRB, 0.30100E+01_JPRB, 0.32439E+01_JPRB, 0.34775E+01_JPRB ,&
&  0.36697E+01_JPRB, 0.35669E+01_JPRB, 0.32229E+01_JPRB, 0.24160E+01_JPRB, 0.21981E+01_JPRB, 0.25918E+01_JPRB, 0.25720E+01_JPRB ,&
&  0.25349E+01_JPRB, 0.24523E+01_JPRB, 0.23922E+01_JPRB, 0.23437E+01_JPRB, 0.23528E+01_JPRB, 0.24088E+01_JPRB, 0.24450E+01_JPRB ,&
&  0.24423E+01_JPRB, 0.24592E+01_JPRB, 0.25207E+01_JPRB, 0.26641E+01_JPRB, 0.28822E+01_JPRB, 0.30315E+01_JPRB, 0.31852E+01_JPRB ,&
&  0.33186E+01_JPRB, 0.35459E+01_JPRB, 0.36857E+01_JPRB, 0.38560E+01_JPRB, 0.39396E+01_JPRB, 0.40853E+01_JPRB, 0.42900E+01_JPRB ,&
&  0.43408E+01_JPRB, 0.44360E+01_JPRB, 0.46492E+01_JPRB, 0.50192E+01_JPRB, 0.52002E+01_JPRB, 0.46152E+01_JPRB, 0.45182E+01_JPRB ,&
&  0.49058E+01_JPRB, 0.53531E+01_JPRB, 0.50066E+01_JPRB, 0.64302E+01_JPRB, 0.40743E+01_JPRB, 0.43291E+01_JPRB, 0.41708E+01_JPRB ,&
&  0.39801E+01_JPRB, 0.40174E+01_JPRB, 0.42539E+01_JPRB, 0.43961E+01_JPRB, 0.41308E+01_JPRB, 0.35981E+01_JPRB, 0.40726E+01_JPRB ,&
&  0.37352E+01_JPRB, 0.27384E+01_JPRB, 0.20076E+01_JPRB, 0.14118E+01_JPRB, 0.17211E+01_JPRB, 0.28369E+01_JPRB, 0.21979E+01_JPRB ,&
&  0.15276E+01_JPRB, 0.16491E+01_JPRB, 0.18293E+01_JPRB, 0.20173E+01_JPRB, 0.16386E+01_JPRB, 0.10970E+01_JPRB, 0.10769E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11177E+01_JPRB, 0.11408E+01_JPRB, 0.11572E+01_JPRB, 0.11888E+01_JPRB, 0.12379E+01_JPRB, 0.12804E+01_JPRB ,&
&  0.12856E+01_JPRB, 0.12765E+01_JPRB, 0.12917E+01_JPRB, 0.12842E+01_JPRB, 0.12572E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.10597E+01_JPRB, 0.10091E+01_JPRB, 0.96894E+00_JPRB, 0.95455E+00_JPRB, 0.95938E+00_JPRB, 0.97871E+00_JPRB, 0.10155E+01_JPRB ,&
&  0.10821E+01_JPRB, 0.11669E+01_JPRB, 0.12549E+01_JPRB, 0.13428E+01_JPRB, 0.14582E+01_JPRB, 0.15876E+01_JPRB, 0.17485E+01_JPRB ,&
&  0.19261E+01_JPRB, 0.21112E+01_JPRB, 0.21975E+01_JPRB, 0.22135E+01_JPRB, 0.22696E+01_JPRB, 0.23511E+01_JPRB, 0.25963E+01_JPRB ,&
&  0.30073E+01_JPRB, 0.33585E+01_JPRB, 0.35520E+01_JPRB, 0.33575E+01_JPRB, 0.26748E+01_JPRB, 0.27041E+01_JPRB, 0.29364E+01_JPRB ,&
&  0.30166E+01_JPRB, 0.27474E+01_JPRB, 0.25398E+01_JPRB, 0.25113E+01_JPRB, 0.25158E+01_JPRB, 0.25648E+01_JPRB, 0.25973E+01_JPRB ,&
&  0.26050E+01_JPRB, 0.26399E+01_JPRB, 0.27332E+01_JPRB, 0.28504E+01_JPRB, 0.30203E+01_JPRB, 0.32025E+01_JPRB, 0.33981E+01_JPRB ,&
&  0.35133E+01_JPRB, 0.36495E+01_JPRB, 0.36671E+01_JPRB, 0.36070E+01_JPRB, 0.37332E+01_JPRB, 0.41475E+01_JPRB, 0.47261E+01_JPRB ,&
&  0.50884E+01_JPRB, 0.50983E+01_JPRB, 0.54917E+01_JPRB, 0.59468E+01_JPRB, 0.59014E+01_JPRB, 0.54388E+01_JPRB, 0.53722E+01_JPRB ,&
&  0.56133E+01_JPRB, 0.62758E+01_JPRB, 0.46286E+01_JPRB, 0.53855E+01_JPRB, 0.54992E+01_JPRB, 0.46664E+01_JPRB, 0.38304E+01_JPRB ,&
&  0.34022E+01_JPRB, 0.30330E+01_JPRB, 0.26022E+01_JPRB, 0.27159E+01_JPRB, 0.28517E+01_JPRB, 0.30240E+01_JPRB, 0.34273E+01_JPRB ,&
&  0.32488E+01_JPRB, 0.24016E+01_JPRB, 0.16963E+01_JPRB, 0.13452E+01_JPRB, 0.15579E+01_JPRB, 0.20924E+01_JPRB, 0.16284E+01_JPRB ,&
&  0.13869E+01_JPRB, 0.15536E+01_JPRB, 0.16430E+01_JPRB, 0.19229E+01_JPRB, 0.18690E+01_JPRB, 0.10405E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10817E+01_JPRB, 0.11257E+01_JPRB, 0.11412E+01_JPRB, 0.11536E+01_JPRB, 0.11628E+01_JPRB, 0.11933E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12291E+01_JPRB, 0.12249E+01_JPRB, 0.12065E+01_JPRB, 0.11672E+01_JPRB, 0.11091E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.87887E+00_JPRB, 0.87258E+00_JPRB, 0.86164E+00_JPRB, 0.88028E+00_JPRB, 0.90197E+00_JPRB, 0.90796E+00_JPRB, 0.93258E+00_JPRB ,&
&  0.96502E+00_JPRB, 0.99925E+00_JPRB, 0.10283E+01_JPRB, 0.10533E+01_JPRB, 0.11388E+01_JPRB, 0.12351E+01_JPRB, 0.13492E+01_JPRB ,&
&  0.14115E+01_JPRB, 0.14633E+01_JPRB, 0.14615E+01_JPRB, 0.14216E+01_JPRB, 0.13865E+01_JPRB, 0.12988E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.12823E+01_JPRB, 0.16542E+01_JPRB, 0.22550E+01_JPRB, 0.28449E+01_JPRB, 0.33957E+01_JPRB, 0.30195E+01_JPRB, 0.33084E+01_JPRB ,&
&  0.29274E+01_JPRB, 0.23793E+01_JPRB, 0.24122E+01_JPRB, 0.25311E+01_JPRB, 0.26531E+01_JPRB, 0.26523E+01_JPRB, 0.27017E+01_JPRB ,&
&  0.27389E+01_JPRB, 0.27852E+01_JPRB, 0.28598E+01_JPRB, 0.30052E+01_JPRB, 0.31337E+01_JPRB, 0.32380E+01_JPRB, 0.33630E+01_JPRB ,&
&  0.34716E+01_JPRB, 0.35340E+01_JPRB, 0.36237E+01_JPRB, 0.34093E+01_JPRB, 0.37419E+01_JPRB, 0.43919E+01_JPRB, 0.49622E+01_JPRB ,&
&  0.52484E+01_JPRB, 0.54249E+01_JPRB, 0.59712E+01_JPRB, 0.62344E+01_JPRB, 0.64150E+01_JPRB, 0.63537E+01_JPRB, 0.63467E+01_JPRB ,&
&  0.63909E+01_JPRB, 0.58645E+01_JPRB, 0.42978E+01_JPRB, 0.45790E+01_JPRB, 0.40464E+01_JPRB, 0.32216E+01_JPRB, 0.31732E+01_JPRB ,&
&  0.27907E+01_JPRB, 0.24092E+01_JPRB, 0.19169E+01_JPRB, 0.18563E+01_JPRB, 0.20393E+01_JPRB, 0.22599E+01_JPRB, 0.27691E+01_JPRB ,&
&  0.23449E+01_JPRB, 0.18402E+01_JPRB, 0.13949E+01_JPRB, 0.13142E+01_JPRB, 0.13644E+01_JPRB, 0.14886E+01_JPRB, 0.14726E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.13933E+01_JPRB, 0.14697E+01_JPRB, 0.16590E+01_JPRB, 0.11355E+01_JPRB, 0.93108E+00_JPRB, 0.95321E+00_JPRB ,&
&  0.97472E+00_JPRB, 0.98445E+00_JPRB, 0.98946E+00_JPRB, 0.98546E+00_JPRB, 0.99769E+00_JPRB, 0.10214E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.97074E+00_JPRB, 0.93054E+00_JPRB, 0.90383E+00_JPRB, 0.88374E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.69857E+00_JPRB, 0.70100E+00_JPRB, 0.70594E+00_JPRB, 0.71735E+00_JPRB, 0.72129E+00_JPRB, 0.72180E+00_JPRB, 0.71235E+00_JPRB ,&
&  0.70673E+00_JPRB, 0.69517E+00_JPRB, 0.69080E+00_JPRB, 0.69150E+00_JPRB, 0.71806E+00_JPRB, 0.76829E+00_JPRB, 0.83678E+00_JPRB ,&
&  0.85728E+00_JPRB, 0.85288E+00_JPRB, 0.81337E+00_JPRB, 0.78347E+00_JPRB, 0.79922E+00_JPRB, 0.79328E+00_JPRB, 0.76183E+00_JPRB ,&
&  0.74003E+00_JPRB, 0.81945E+00_JPRB, 0.10332E+01_JPRB, 0.13653E+01_JPRB, 0.19580E+01_JPRB, 0.23342E+01_JPRB, 0.30566E+01_JPRB ,&
&  0.30378E+01_JPRB, 0.21397E+01_JPRB, 0.22720E+01_JPRB, 0.21902E+01_JPRB, 0.23462E+01_JPRB, 0.24347E+01_JPRB, 0.24701E+01_JPRB ,&
&  0.25516E+01_JPRB, 0.26388E+01_JPRB, 0.27219E+01_JPRB, 0.28118E+01_JPRB, 0.28834E+01_JPRB, 0.29019E+01_JPRB, 0.29226E+01_JPRB ,&
&  0.29492E+01_JPRB, 0.29504E+01_JPRB, 0.30351E+01_JPRB, 0.32763E+01_JPRB, 0.38875E+01_JPRB, 0.45689E+01_JPRB, 0.47021E+01_JPRB ,&
&  0.47964E+01_JPRB, 0.52097E+01_JPRB, 0.51731E+01_JPRB, 0.55513E+01_JPRB, 0.57189E+01_JPRB, 0.55633E+01_JPRB, 0.59433E+01_JPRB ,&
&  0.61800E+01_JPRB, 0.51721E+01_JPRB, 0.34652E+01_JPRB, 0.35628E+01_JPRB, 0.32200E+01_JPRB, 0.28503E+01_JPRB, 0.28590E+01_JPRB ,&
&  0.26028E+01_JPRB, 0.23361E+01_JPRB, 0.20529E+01_JPRB, 0.18673E+01_JPRB, 0.18203E+01_JPRB, 0.17231E+01_JPRB, 0.17571E+01_JPRB ,&
&  0.16179E+01_JPRB, 0.15650E+01_JPRB, 0.14799E+01_JPRB, 0.14450E+01_JPRB, 0.15701E+01_JPRB, 0.17945E+01_JPRB, 0.15368E+01_JPRB ,&
&  0.14959E+01_JPRB, 0.14737E+01_JPRB, 0.16959E+01_JPRB, 0.18237E+01_JPRB, 0.10172E+01_JPRB, 0.78700E+00_JPRB, 0.74095E+00_JPRB ,&
&  0.74785E+00_JPRB, 0.74215E+00_JPRB, 0.73618E+00_JPRB, 0.72425E+00_JPRB, 0.75410E+00_JPRB, 0.80390E+00_JPRB, 0.78975E+00_JPRB ,&
&  0.78796E+00_JPRB, 0.74350E+00_JPRB, 0.72735E+00_JPRB, 0.72377E+00_JPRB, 0.72021E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.65406E+00_JPRB, 0.64504E+00_JPRB, 0.63646E+00_JPRB, 0.62756E+00_JPRB, 0.63721E+00_JPRB, 0.64283E+00_JPRB, 0.65838E+00_JPRB ,&
&  0.67566E+00_JPRB, 0.68148E+00_JPRB, 0.69250E+00_JPRB, 0.69836E+00_JPRB, 0.72117E+00_JPRB, 0.74078E+00_JPRB, 0.77850E+00_JPRB ,&
&  0.81601E+00_JPRB, 0.83732E+00_JPRB, 0.86238E+00_JPRB, 0.88759E+00_JPRB, 0.90300E+00_JPRB, 0.92029E+00_JPRB, 0.90751E+00_JPRB ,&
&  0.88833E+00_JPRB, 0.88817E+00_JPRB, 0.92447E+00_JPRB, 0.10767E+01_JPRB, 0.13689E+01_JPRB, 0.17473E+01_JPRB, 0.25159E+01_JPRB ,&
&  0.26251E+01_JPRB, 0.19171E+01_JPRB, 0.16897E+01_JPRB, 0.15025E+01_JPRB, 0.15876E+01_JPRB, 0.17109E+01_JPRB, 0.17277E+01_JPRB ,&
&  0.17565E+01_JPRB, 0.17747E+01_JPRB, 0.17721E+01_JPRB, 0.17458E+01_JPRB, 0.16618E+01_JPRB, 0.15534E+01_JPRB, 0.14872E+01_JPRB ,&
&  0.15117E+01_JPRB, 0.15237E+01_JPRB, 0.17665E+01_JPRB, 0.23765E+01_JPRB, 0.29128E+01_JPRB, 0.37707E+01_JPRB, 0.35935E+01_JPRB ,&
&  0.36603E+01_JPRB, 0.38733E+01_JPRB, 0.38321E+01_JPRB, 0.41062E+01_JPRB, 0.42206E+01_JPRB, 0.43918E+01_JPRB, 0.44039E+01_JPRB ,&
&  0.46960E+01_JPRB, 0.40910E+01_JPRB, 0.28722E+01_JPRB, 0.30438E+01_JPRB, 0.28167E+01_JPRB, 0.27696E+01_JPRB, 0.25849E+01_JPRB ,&
&  0.24078E+01_JPRB, 0.22025E+01_JPRB, 0.20263E+01_JPRB, 0.18835E+01_JPRB, 0.18424E+01_JPRB, 0.17734E+01_JPRB, 0.16163E+01_JPRB ,&
&  0.15869E+01_JPRB, 0.15855E+01_JPRB, 0.15522E+01_JPRB, 0.14893E+01_JPRB, 0.17022E+01_JPRB, 0.17442E+01_JPRB, 0.21858E+01_JPRB ,&
&  0.14866E+01_JPRB, 0.13455E+01_JPRB, 0.10422E+01_JPRB, 0.13198E+01_JPRB, 0.83649E+00_JPRB, 0.64991E+00_JPRB, 0.64435E+00_JPRB ,&
&  0.62669E+00_JPRB, 0.60501E+00_JPRB, 0.58771E+00_JPRB, 0.58577E+00_JPRB, 0.59959E+00_JPRB, 0.63157E+00_JPRB, 0.62878E+00_JPRB ,&
&  0.63424E+00_JPRB, 0.64141E+00_JPRB, 0.65682E+00_JPRB, 0.67708E+00_JPRB, 0.67175E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.69026E+00_JPRB, 0.69390E+00_JPRB, 0.70671E+00_JPRB, 0.72752E+00_JPRB, 0.74047E+00_JPRB, 0.75187E+00_JPRB, 0.76216E+00_JPRB ,&
&  0.78731E+00_JPRB, 0.81533E+00_JPRB, 0.84121E+00_JPRB, 0.86659E+00_JPRB, 0.89203E+00_JPRB, 0.91306E+00_JPRB, 0.95854E+00_JPRB ,&
&  0.98610E+00_JPRB, 0.10221E+01_JPRB, 0.10464E+01_JPRB, 0.10746E+01_JPRB, 0.10995E+01_JPRB, 0.10910E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.11560E+01_JPRB, 0.11924E+01_JPRB, 0.12617E+01_JPRB, 0.13515E+01_JPRB, 0.13882E+01_JPRB, 0.16182E+01_JPRB ,&
&  0.16681E+01_JPRB, 0.18348E+01_JPRB, 0.15878E+01_JPRB, 0.13620E+01_JPRB, 0.13070E+01_JPRB, 0.11719E+01_JPRB, 0.11042E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.11458E+01_JPRB, 0.11373E+01_JPRB, 0.11121E+01_JPRB, 0.10679E+01_JPRB, 0.11485E+01_JPRB, 0.13321E+01_JPRB ,&
&  0.16217E+01_JPRB, 0.17944E+01_JPRB, 0.20149E+01_JPRB, 0.22006E+01_JPRB, 0.24572E+01_JPRB, 0.26274E+01_JPRB, 0.26057E+01_JPRB ,&
&  0.26204E+01_JPRB, 0.25843E+01_JPRB, 0.24508E+01_JPRB, 0.28309E+01_JPRB, 0.31297E+01_JPRB, 0.32639E+01_JPRB, 0.32944E+01_JPRB ,&
&  0.32895E+01_JPRB, 0.29072E+01_JPRB, 0.27846E+01_JPRB, 0.26075E+01_JPRB, 0.24251E+01_JPRB, 0.24120E+01_JPRB, 0.22668E+01_JPRB ,&
&  0.20904E+01_JPRB, 0.19720E+01_JPRB, 0.19327E+01_JPRB, 0.19294E+01_JPRB, 0.19574E+01_JPRB, 0.19500E+01_JPRB, 0.17975E+01_JPRB ,&
&  0.16844E+01_JPRB, 0.16075E+01_JPRB, 0.15459E+01_JPRB, 0.15031E+01_JPRB, 0.15067E+01_JPRB, 0.28839E+01_JPRB, 0.37474E+01_JPRB ,&
&  0.14118E+01_JPRB, 0.99669E+00_JPRB, 0.61404E+00_JPRB, 0.75923E+00_JPRB, 0.66264E+00_JPRB, 0.64567E+00_JPRB, 0.71614E+00_JPRB ,&
&  0.73052E+00_JPRB, 0.71696E+00_JPRB, 0.65158E+00_JPRB, 0.58870E+00_JPRB, 0.60834E+00_JPRB, 0.61848E+00_JPRB, 0.62034E+00_JPRB ,&
&  0.63750E+00_JPRB, 0.64664E+00_JPRB, 0.66019E+00_JPRB, 0.68422E+00_JPRB, 0.68192E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.73421E+00_JPRB, 0.75321E+00_JPRB, 0.77734E+00_JPRB, 0.80877E+00_JPRB, 0.83957E+00_JPRB, 0.86492E+00_JPRB, 0.88336E+00_JPRB ,&
&  0.91089E+00_JPRB, 0.94639E+00_JPRB, 0.96927E+00_JPRB, 0.98147E+00_JPRB, 0.10080E+01_JPRB, 0.10311E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.11549E+01_JPRB, 0.12026E+01_JPRB, 0.12476E+01_JPRB, 0.12854E+01_JPRB, 0.13268E+01_JPRB, 0.13659E+01_JPRB ,&
&  0.13977E+01_JPRB, 0.14611E+01_JPRB, 0.15106E+01_JPRB, 0.15415E+01_JPRB, 0.16063E+01_JPRB, 0.17073E+01_JPRB, 0.14727E+01_JPRB ,&
&  0.18475E+01_JPRB, 0.17139E+01_JPRB, 0.15220E+01_JPRB, 0.13776E+01_JPRB, 0.12741E+01_JPRB, 0.12179E+01_JPRB, 0.12000E+01_JPRB ,&
&  0.12488E+01_JPRB, 0.13998E+01_JPRB, 0.15249E+01_JPRB, 0.15604E+01_JPRB, 0.15345E+01_JPRB, 0.17219E+01_JPRB, 0.21098E+01_JPRB ,&
&  0.23199E+01_JPRB, 0.24431E+01_JPRB, 0.25327E+01_JPRB, 0.25626E+01_JPRB, 0.26834E+01_JPRB, 0.28601E+01_JPRB, 0.29602E+01_JPRB ,&
&  0.29549E+01_JPRB, 0.28600E+01_JPRB, 0.25323E+01_JPRB, 0.30830E+01_JPRB, 0.30425E+01_JPRB, 0.28762E+01_JPRB, 0.25753E+01_JPRB ,&
&  0.22823E+01_JPRB, 0.19997E+01_JPRB, 0.20740E+01_JPRB, 0.21799E+01_JPRB, 0.20797E+01_JPRB, 0.20020E+01_JPRB, 0.18510E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.16709E+01_JPRB, 0.17091E+01_JPRB, 0.17926E+01_JPRB, 0.18620E+01_JPRB, 0.18514E+01_JPRB, 0.17186E+01_JPRB ,&
&  0.15849E+01_JPRB, 0.14475E+01_JPRB, 0.13620E+01_JPRB, 0.13103E+01_JPRB, 0.13251E+01_JPRB, 0.15107E+01_JPRB, 0.21954E+01_JPRB ,&
&  0.11824E+01_JPRB, 0.69149E+00_JPRB, 0.69894E+00_JPRB, 0.65480E+00_JPRB, 0.67552E+00_JPRB, 0.66453E+00_JPRB, 0.82067E+00_JPRB ,&
&  0.10303E+01_JPRB, 0.11782E+01_JPRB, 0.11534E+01_JPRB, 0.89032E+00_JPRB, 0.72420E+00_JPRB, 0.67580E+00_JPRB, 0.67330E+00_JPRB ,&
&  0.68627E+00_JPRB, 0.68874E+00_JPRB, 0.69591E+00_JPRB, 0.70629E+00_JPRB, 0.72063E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.71644E+00_JPRB, 0.73059E+00_JPRB, 0.74830E+00_JPRB, 0.76728E+00_JPRB, 0.79041E+00_JPRB, 0.81449E+00_JPRB, 0.83231E+00_JPRB ,&
&  0.85222E+00_JPRB, 0.87481E+00_JPRB, 0.89573E+00_JPRB, 0.91068E+00_JPRB, 0.92968E+00_JPRB, 0.96394E+00_JPRB, 0.10046E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.10927E+01_JPRB, 0.11327E+01_JPRB, 0.11984E+01_JPRB, 0.12622E+01_JPRB, 0.13364E+01_JPRB, 0.13943E+01_JPRB ,&
&  0.14574E+01_JPRB, 0.15477E+01_JPRB, 0.15914E+01_JPRB, 0.15899E+01_JPRB, 0.16312E+01_JPRB, 0.17922E+01_JPRB, 0.15127E+01_JPRB ,&
&  0.18374E+01_JPRB, 0.17873E+01_JPRB, 0.16390E+01_JPRB, 0.15863E+01_JPRB, 0.15521E+01_JPRB, 0.15933E+01_JPRB, 0.16685E+01_JPRB ,&
&  0.17844E+01_JPRB, 0.20360E+01_JPRB, 0.20602E+01_JPRB, 0.21380E+01_JPRB, 0.22753E+01_JPRB, 0.26116E+01_JPRB, 0.29270E+01_JPRB ,&
&  0.30635E+01_JPRB, 0.33061E+01_JPRB, 0.34344E+01_JPRB, 0.35519E+01_JPRB, 0.37356E+01_JPRB, 0.39322E+01_JPRB, 0.40737E+01_JPRB ,&
&  0.40244E+01_JPRB, 0.39375E+01_JPRB, 0.36233E+01_JPRB, 0.38874E+01_JPRB, 0.36436E+01_JPRB, 0.28942E+01_JPRB, 0.22088E+01_JPRB ,&
&  0.15929E+01_JPRB, 0.16322E+01_JPRB, 0.17235E+01_JPRB, 0.17944E+01_JPRB, 0.16682E+01_JPRB, 0.15611E+01_JPRB, 0.13329E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.12258E+01_JPRB, 0.13079E+01_JPRB, 0.14285E+01_JPRB, 0.14950E+01_JPRB, 0.14764E+01_JPRB, 0.13701E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.11570E+01_JPRB, 0.11138E+01_JPRB, 0.11081E+01_JPRB, 0.10743E+01_JPRB, 0.10216E+01_JPRB, 0.13525E+01_JPRB ,&
&  0.96963E+00_JPRB, 0.84824E+00_JPRB, 0.98201E+00_JPRB, 0.50217E+00_JPRB, 0.67307E+00_JPRB, 0.84754E+00_JPRB, 0.11157E+01_JPRB ,&
&  0.13168E+01_JPRB, 0.18339E+01_JPRB, 0.22689E+01_JPRB, 0.17277E+01_JPRB, 0.12138E+01_JPRB, 0.73249E+00_JPRB, 0.64352E+00_JPRB ,&
&  0.65756E+00_JPRB, 0.66585E+00_JPRB, 0.67786E+00_JPRB, 0.68181E+00_JPRB, 0.69882E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.65539E+00_JPRB, 0.65170E+00_JPRB, 0.65295E+00_JPRB, 0.64421E+00_JPRB, 0.64070E+00_JPRB, 0.64242E+00_JPRB, 0.63250E+00_JPRB ,&
&  0.63998E+00_JPRB, 0.65768E+00_JPRB, 0.66417E+00_JPRB, 0.68720E+00_JPRB, 0.70436E+00_JPRB, 0.75621E+00_JPRB, 0.83348E+00_JPRB ,&
&  0.89597E+00_JPRB, 0.10205E+01_JPRB, 0.10961E+01_JPRB, 0.11839E+01_JPRB, 0.12618E+01_JPRB, 0.13464E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.15168E+01_JPRB, 0.16390E+01_JPRB, 0.17622E+01_JPRB, 0.18538E+01_JPRB, 0.18898E+01_JPRB, 0.21315E+01_JPRB, 0.17066E+01_JPRB ,&
&  0.19491E+01_JPRB, 0.19856E+01_JPRB, 0.19031E+01_JPRB, 0.18821E+01_JPRB, 0.18717E+01_JPRB, 0.18905E+01_JPRB, 0.19218E+01_JPRB ,&
&  0.20171E+01_JPRB, 0.21033E+01_JPRB, 0.21815E+01_JPRB, 0.24751E+01_JPRB, 0.27981E+01_JPRB, 0.30843E+01_JPRB, 0.33589E+01_JPRB ,&
&  0.36191E+01_JPRB, 0.38860E+01_JPRB, 0.42093E+01_JPRB, 0.45415E+01_JPRB, 0.48766E+01_JPRB, 0.50827E+01_JPRB, 0.53308E+01_JPRB ,&
&  0.54577E+01_JPRB, 0.55410E+01_JPRB, 0.51600E+01_JPRB, 0.44699E+01_JPRB, 0.37327E+01_JPRB, 0.28824E+01_JPRB, 0.20539E+01_JPRB ,&
&  0.15031E+01_JPRB, 0.13990E+01_JPRB, 0.13818E+01_JPRB, 0.13520E+01_JPRB, 0.11920E+01_JPRB, 0.10886E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.95668E+00_JPRB, 0.88170E+00_JPRB, 0.93112E+00_JPRB, 0.10106E+01_JPRB, 0.10363E+01_JPRB, 0.10262E+01_JPRB, 0.10007E+01_JPRB ,&
&  0.95875E+00_JPRB, 0.90058E+00_JPRB, 0.90919E+00_JPRB, 0.92564E+00_JPRB, 0.97059E+00_JPRB, 0.10712E+01_JPRB, 0.94834E+00_JPRB ,&
&  0.10112E+01_JPRB, 0.73024E+00_JPRB, 0.70510E+00_JPRB, 0.70867E+00_JPRB, 0.71338E+00_JPRB, 0.90828E+00_JPRB, 0.12476E+01_JPRB ,&
&  0.18575E+01_JPRB, 0.20868E+01_JPRB, 0.23127E+01_JPRB, 0.28565E+01_JPRB, 0.24868E+01_JPRB, 0.18544E+01_JPRB, 0.58925E+00_JPRB ,&
&  0.59148E+00_JPRB, 0.59872E+00_JPRB, 0.62760E+00_JPRB, 0.63854E+00_JPRB, 0.64498E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.55446E+00_JPRB, 0.53772E+00_JPRB, 0.52989E+00_JPRB, 0.52593E+00_JPRB, 0.51049E+00_JPRB, 0.51252E+00_JPRB, 0.51156E+00_JPRB ,&
&  0.53120E+00_JPRB, 0.55812E+00_JPRB, 0.60430E+00_JPRB, 0.66222E+00_JPRB, 0.74101E+00_JPRB, 0.80718E+00_JPRB, 0.89032E+00_JPRB ,&
&  0.95075E+00_JPRB, 0.10259E+01_JPRB, 0.11025E+01_JPRB, 0.11795E+01_JPRB, 0.12666E+01_JPRB, 0.13436E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.15435E+01_JPRB, 0.16559E+01_JPRB, 0.17954E+01_JPRB, 0.19205E+01_JPRB, 0.20210E+01_JPRB, 0.20980E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.20456E+01_JPRB, 0.21426E+01_JPRB, 0.21714E+01_JPRB, 0.21235E+01_JPRB, 0.20511E+01_JPRB, 0.19786E+01_JPRB, 0.19990E+01_JPRB ,&
&  0.20438E+01_JPRB, 0.20740E+01_JPRB, 0.20974E+01_JPRB, 0.22071E+01_JPRB, 0.28249E+01_JPRB, 0.31014E+01_JPRB, 0.33908E+01_JPRB ,&
&  0.36977E+01_JPRB, 0.40412E+01_JPRB, 0.44000E+01_JPRB, 0.47620E+01_JPRB, 0.50051E+01_JPRB, 0.52662E+01_JPRB, 0.55383E+01_JPRB ,&
&  0.57533E+01_JPRB, 0.59347E+01_JPRB, 0.56667E+01_JPRB, 0.43768E+01_JPRB, 0.35459E+01_JPRB, 0.28208E+01_JPRB, 0.17546E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.11810E+01_JPRB, 0.11637E+01_JPRB, 0.11046E+01_JPRB, 0.96481E+00_JPRB, 0.86804E+00_JPRB, 0.80140E+00_JPRB ,&
&  0.70895E+00_JPRB, 0.68573E+00_JPRB, 0.68694E+00_JPRB, 0.74819E+00_JPRB, 0.78616E+00_JPRB, 0.75227E+00_JPRB, 0.73130E+00_JPRB ,&
&  0.68950E+00_JPRB, 0.65796E+00_JPRB, 0.71209E+00_JPRB, 0.79566E+00_JPRB, 0.97481E+00_JPRB, 0.11683E+01_JPRB, 0.13012E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.74495E+00_JPRB, 0.69037E+00_JPRB, 0.67442E+00_JPRB, 0.69026E+00_JPRB, 0.74082E+00_JPRB, 0.83434E+00_JPRB ,&
&  0.10441E+01_JPRB, 0.14163E+01_JPRB, 0.12422E+01_JPRB, 0.95257E+00_JPRB, 0.10040E+01_JPRB, 0.92878E+00_JPRB, 0.53723E+00_JPRB ,&
&  0.54118E+00_JPRB, 0.53953E+00_JPRB, 0.55686E+00_JPRB, 0.56271E+00_JPRB, 0.56646E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.49021E+00_JPRB, 0.47052E+00_JPRB, 0.45055E+00_JPRB, 0.44177E+00_JPRB, 0.43519E+00_JPRB, 0.42820E+00_JPRB, 0.44273E+00_JPRB ,&
&  0.47808E+00_JPRB, 0.51124E+00_JPRB, 0.56610E+00_JPRB, 0.64944E+00_JPRB, 0.73625E+00_JPRB, 0.79593E+00_JPRB, 0.85979E+00_JPRB ,&
&  0.91769E+00_JPRB, 0.97825E+00_JPRB, 0.10300E+01_JPRB, 0.10821E+01_JPRB, 0.11487E+01_JPRB, 0.12314E+01_JPRB, 0.13437E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.16113E+01_JPRB, 0.17754E+01_JPRB, 0.19715E+01_JPRB, 0.21626E+01_JPRB, 0.22597E+01_JPRB, 0.24216E+01_JPRB ,&
&  0.17379E+01_JPRB, 0.22050E+01_JPRB, 0.22339E+01_JPRB, 0.21932E+01_JPRB, 0.21270E+01_JPRB, 0.20614E+01_JPRB, 0.20429E+01_JPRB ,&
&  0.20428E+01_JPRB, 0.19372E+01_JPRB, 0.19639E+01_JPRB, 0.20920E+01_JPRB, 0.24770E+01_JPRB, 0.27704E+01_JPRB, 0.30612E+01_JPRB ,&
&  0.33703E+01_JPRB, 0.37152E+01_JPRB, 0.40755E+01_JPRB, 0.44196E+01_JPRB, 0.46909E+01_JPRB, 0.49828E+01_JPRB, 0.52064E+01_JPRB ,&
&  0.54811E+01_JPRB, 0.57126E+01_JPRB, 0.57805E+01_JPRB, 0.46703E+01_JPRB, 0.36364E+01_JPRB, 0.30601E+01_JPRB, 0.21141E+01_JPRB ,&
&  0.13295E+01_JPRB, 0.10211E+01_JPRB, 0.10272E+01_JPRB, 0.10458E+01_JPRB, 0.94429E+00_JPRB, 0.86346E+00_JPRB, 0.77302E+00_JPRB ,&
&  0.70356E+00_JPRB, 0.62575E+00_JPRB, 0.60114E+00_JPRB, 0.62577E+00_JPRB, 0.67989E+00_JPRB, 0.64286E+00_JPRB, 0.58919E+00_JPRB ,&
&  0.55285E+00_JPRB, 0.54227E+00_JPRB, 0.60277E+00_JPRB, 0.67358E+00_JPRB, 0.77539E+00_JPRB, 0.83191E+00_JPRB, 0.86068E+00_JPRB ,&
&  0.89220E+00_JPRB, 0.73902E+00_JPRB, 0.71412E+00_JPRB, 0.71794E+00_JPRB, 0.71143E+00_JPRB, 0.71488E+00_JPRB, 0.73129E+00_JPRB ,&
&  0.76712E+00_JPRB, 0.83294E+00_JPRB, 0.84815E+00_JPRB, 0.78878E+00_JPRB, 0.71295E+00_JPRB, 0.66030E+00_JPRB, 0.61449E+00_JPRB ,&
&  0.61781E+00_JPRB, 0.60453E+00_JPRB, 0.58902E+00_JPRB, 0.56117E+00_JPRB, 0.52713E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.50425E+00_JPRB, 0.47117E+00_JPRB, 0.42685E+00_JPRB, 0.39028E+00_JPRB, 0.41708E+00_JPRB, 0.42117E+00_JPRB, 0.44177E+00_JPRB ,&
&  0.45236E+00_JPRB, 0.48455E+00_JPRB, 0.51958E+00_JPRB, 0.57106E+00_JPRB, 0.64157E+00_JPRB, 0.71455E+00_JPRB, 0.77853E+00_JPRB ,&
&  0.83705E+00_JPRB, 0.89398E+00_JPRB, 0.93775E+00_JPRB, 0.95871E+00_JPRB, 0.99073E+00_JPRB, 0.10427E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.13121E+01_JPRB, 0.14667E+01_JPRB, 0.16635E+01_JPRB, 0.18924E+01_JPRB, 0.20347E+01_JPRB, 0.21484E+01_JPRB ,&
&  0.11928E+01_JPRB, 0.18864E+01_JPRB, 0.22985E+01_JPRB, 0.22003E+01_JPRB, 0.21444E+01_JPRB, 0.20859E+01_JPRB, 0.20496E+01_JPRB ,&
&  0.19086E+01_JPRB, 0.17576E+01_JPRB, 0.17634E+01_JPRB, 0.20593E+01_JPRB, 0.20678E+01_JPRB, 0.23272E+01_JPRB, 0.25437E+01_JPRB ,&
&  0.27783E+01_JPRB, 0.30289E+01_JPRB, 0.33192E+01_JPRB, 0.36785E+01_JPRB, 0.40438E+01_JPRB, 0.43451E+01_JPRB, 0.46057E+01_JPRB ,&
&  0.49581E+01_JPRB, 0.52890E+01_JPRB, 0.54075E+01_JPRB, 0.41268E+01_JPRB, 0.36918E+01_JPRB, 0.34178E+01_JPRB, 0.29831E+01_JPRB ,&
&  0.16432E+01_JPRB, 0.10414E+01_JPRB, 0.10403E+01_JPRB, 0.10773E+01_JPRB, 0.10001E+01_JPRB, 0.92717E+00_JPRB, 0.85232E+00_JPRB ,&
&  0.75568E+00_JPRB, 0.68451E+00_JPRB, 0.63402E+00_JPRB, 0.61534E+00_JPRB, 0.64356E+00_JPRB, 0.64735E+00_JPRB, 0.61195E+00_JPRB ,&
&  0.57296E+00_JPRB, 0.57541E+00_JPRB, 0.61735E+00_JPRB, 0.65436E+00_JPRB, 0.71254E+00_JPRB, 0.74951E+00_JPRB, 0.79313E+00_JPRB ,&
&  0.79768E+00_JPRB, 0.73309E+00_JPRB, 0.72945E+00_JPRB, 0.74292E+00_JPRB, 0.74894E+00_JPRB, 0.75395E+00_JPRB, 0.75506E+00_JPRB ,&
&  0.75388E+00_JPRB, 0.77363E+00_JPRB, 0.78055E+00_JPRB, 0.78570E+00_JPRB, 0.76247E+00_JPRB, 0.73635E+00_JPRB, 0.71992E+00_JPRB ,&
&  0.69653E+00_JPRB, 0.66697E+00_JPRB, 0.63953E+00_JPRB, 0.59257E+00_JPRB, 0.54511E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.53102E+00_JPRB, 0.49255E+00_JPRB, 0.44909E+00_JPRB, 0.43361E+00_JPRB, 0.46276E+00_JPRB, 0.48898E+00_JPRB, 0.48699E+00_JPRB ,&
&  0.48252E+00_JPRB, 0.47329E+00_JPRB, 0.47078E+00_JPRB, 0.48570E+00_JPRB, 0.53485E+00_JPRB, 0.60881E+00_JPRB, 0.67215E+00_JPRB ,&
&  0.72903E+00_JPRB, 0.77350E+00_JPRB, 0.80803E+00_JPRB, 0.81965E+00_JPRB, 0.81293E+00_JPRB, 0.83240E+00_JPRB, 0.86724E+00_JPRB ,&
&  0.92726E+00_JPRB, 0.10078E+01_JPRB, 0.11133E+01_JPRB, 0.12667E+01_JPRB, 0.14911E+01_JPRB, 0.17676E+01_JPRB, 0.20526E+01_JPRB ,&
&  0.14801E+01_JPRB, 0.67747E+00_JPRB, 0.19505E+01_JPRB, 0.24172E+01_JPRB, 0.22671E+01_JPRB, 0.22089E+01_JPRB, 0.20570E+01_JPRB ,&
&  0.18670E+01_JPRB, 0.18439E+01_JPRB, 0.17431E+01_JPRB, 0.18907E+01_JPRB, 0.18464E+01_JPRB, 0.19250E+01_JPRB, 0.20010E+01_JPRB ,&
&  0.21146E+01_JPRB, 0.22500E+01_JPRB, 0.24608E+01_JPRB, 0.27177E+01_JPRB, 0.30028E+01_JPRB, 0.34323E+01_JPRB, 0.37467E+01_JPRB ,&
&  0.40223E+01_JPRB, 0.43075E+01_JPRB, 0.43908E+01_JPRB, 0.31360E+01_JPRB, 0.24642E+01_JPRB, 0.21165E+01_JPRB, 0.18716E+01_JPRB ,&
&  0.15366E+01_JPRB, 0.11521E+01_JPRB, 0.11042E+01_JPRB, 0.11373E+01_JPRB, 0.10883E+01_JPRB, 0.97812E+00_JPRB, 0.96276E+00_JPRB ,&
&  0.84175E+00_JPRB, 0.76519E+00_JPRB, 0.70513E+00_JPRB, 0.65724E+00_JPRB, 0.65969E+00_JPRB, 0.67345E+00_JPRB, 0.67724E+00_JPRB ,&
&  0.65366E+00_JPRB, 0.65016E+00_JPRB, 0.66891E+00_JPRB, 0.67067E+00_JPRB, 0.67485E+00_JPRB, 0.68237E+00_JPRB, 0.69284E+00_JPRB ,&
&  0.72909E+00_JPRB, 0.72343E+00_JPRB, 0.73100E+00_JPRB, 0.75140E+00_JPRB, 0.76368E+00_JPRB, 0.75327E+00_JPRB, 0.76294E+00_JPRB ,&
&  0.78077E+00_JPRB, 0.79739E+00_JPRB, 0.79019E+00_JPRB, 0.80104E+00_JPRB, 0.79141E+00_JPRB, 0.78352E+00_JPRB, 0.78518E+00_JPRB ,&
&  0.78670E+00_JPRB, 0.77325E+00_JPRB, 0.73002E+00_JPRB, 0.63835E+00_JPRB, 0.58175E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.57276E+00_JPRB, 0.51238E+00_JPRB, 0.47345E+00_JPRB, 0.47773E+00_JPRB, 0.50502E+00_JPRB, 0.51562E+00_JPRB, 0.51824E+00_JPRB ,&
&  0.50929E+00_JPRB, 0.49678E+00_JPRB, 0.47670E+00_JPRB, 0.46771E+00_JPRB, 0.47539E+00_JPRB, 0.49537E+00_JPRB, 0.54636E+00_JPRB ,&
&  0.59709E+00_JPRB, 0.64606E+00_JPRB, 0.68488E+00_JPRB, 0.68632E+00_JPRB, 0.65825E+00_JPRB, 0.65353E+00_JPRB, 0.66982E+00_JPRB ,&
&  0.69477E+00_JPRB, 0.74297E+00_JPRB, 0.80854E+00_JPRB, 0.89906E+00_JPRB, 0.10416E+01_JPRB, 0.12845E+01_JPRB, 0.16633E+01_JPRB ,&
&  0.23139E+01_JPRB, 0.15771E+01_JPRB, 0.10185E+01_JPRB, 0.20957E+01_JPRB, 0.25547E+01_JPRB, 0.26206E+01_JPRB, 0.21706E+01_JPRB ,&
&  0.20881E+01_JPRB, 0.19660E+01_JPRB, 0.18368E+01_JPRB, 0.18784E+01_JPRB, 0.17540E+01_JPRB, 0.17242E+01_JPRB, 0.16975E+01_JPRB ,&
&  0.16688E+01_JPRB, 0.17659E+01_JPRB, 0.19020E+01_JPRB, 0.20345E+01_JPRB, 0.21714E+01_JPRB, 0.24637E+01_JPRB, 0.28401E+01_JPRB ,&
&  0.32071E+01_JPRB, 0.34442E+01_JPRB, 0.35303E+01_JPRB, 0.26028E+01_JPRB, 0.21091E+01_JPRB, 0.18726E+01_JPRB, 0.17519E+01_JPRB ,&
&  0.15141E+01_JPRB, 0.13586E+01_JPRB, 0.13100E+01_JPRB, 0.12370E+01_JPRB, 0.11610E+01_JPRB, 0.10792E+01_JPRB, 0.10786E+01_JPRB ,&
&  0.96660E+00_JPRB, 0.87172E+00_JPRB, 0.81057E+00_JPRB, 0.77044E+00_JPRB, 0.75249E+00_JPRB, 0.75035E+00_JPRB, 0.74315E+00_JPRB ,&
&  0.74493E+00_JPRB, 0.74673E+00_JPRB, 0.74953E+00_JPRB, 0.72744E+00_JPRB, 0.70402E+00_JPRB, 0.68824E+00_JPRB, 0.66924E+00_JPRB ,&
&  0.68752E+00_JPRB, 0.71330E+00_JPRB, 0.72872E+00_JPRB, 0.75355E+00_JPRB, 0.75499E+00_JPRB, 0.75027E+00_JPRB, 0.77476E+00_JPRB ,&
&  0.78594E+00_JPRB, 0.80554E+00_JPRB, 0.79983E+00_JPRB, 0.80561E+00_JPRB, 0.79271E+00_JPRB, 0.78323E+00_JPRB, 0.78820E+00_JPRB ,&
&  0.82501E+00_JPRB, 0.86502E+00_JPRB, 0.84134E+00_JPRB, 0.67497E+00_JPRB, 0.61068E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.55999E+00_JPRB, 0.53464E+00_JPRB, 0.49479E+00_JPRB, 0.50282E+00_JPRB, 0.50998E+00_JPRB, 0.52100E+00_JPRB, 0.52511E+00_JPRB ,&
&  0.52803E+00_JPRB, 0.52997E+00_JPRB, 0.53256E+00_JPRB, 0.52131E+00_JPRB, 0.49198E+00_JPRB, 0.47947E+00_JPRB, 0.48131E+00_JPRB ,&
&  0.49599E+00_JPRB, 0.53113E+00_JPRB, 0.57233E+00_JPRB, 0.58007E+00_JPRB, 0.57526E+00_JPRB, 0.56701E+00_JPRB, 0.56351E+00_JPRB ,&
&  0.57069E+00_JPRB, 0.58608E+00_JPRB, 0.60757E+00_JPRB, 0.65378E+00_JPRB, 0.73696E+00_JPRB, 0.86920E+00_JPRB, 0.10967E+01_JPRB ,&
&  0.16656E+01_JPRB, 0.17755E+01_JPRB, 0.80750E+00_JPRB, 0.16438E+01_JPRB, 0.25564E+01_JPRB, 0.27191E+01_JPRB, 0.25784E+01_JPRB ,&
&  0.24245E+01_JPRB, 0.21553E+01_JPRB, 0.19745E+01_JPRB, 0.18178E+01_JPRB, 0.16247E+01_JPRB, 0.15218E+01_JPRB, 0.14266E+01_JPRB ,&
&  0.13689E+01_JPRB, 0.14240E+01_JPRB, 0.15641E+01_JPRB, 0.16687E+01_JPRB, 0.17069E+01_JPRB, 0.17962E+01_JPRB, 0.20720E+01_JPRB ,&
&  0.24840E+01_JPRB, 0.29016E+01_JPRB, 0.30982E+01_JPRB, 0.24304E+01_JPRB, 0.21064E+01_JPRB, 0.21001E+01_JPRB, 0.20085E+01_JPRB ,&
&  0.18325E+01_JPRB, 0.15951E+01_JPRB, 0.14390E+01_JPRB, 0.13420E+01_JPRB, 0.12401E+01_JPRB, 0.11934E+01_JPRB, 0.11992E+01_JPRB ,&
&  0.11096E+01_JPRB, 0.99582E+00_JPRB, 0.92260E+00_JPRB, 0.87445E+00_JPRB, 0.84119E+00_JPRB, 0.82293E+00_JPRB, 0.80970E+00_JPRB ,&
&  0.80066E+00_JPRB, 0.80896E+00_JPRB, 0.82281E+00_JPRB, 0.80973E+00_JPRB, 0.77873E+00_JPRB, 0.74291E+00_JPRB, 0.70798E+00_JPRB ,&
&  0.69796E+00_JPRB, 0.70359E+00_JPRB, 0.72890E+00_JPRB, 0.74412E+00_JPRB, 0.75370E+00_JPRB, 0.76495E+00_JPRB, 0.78125E+00_JPRB ,&
&  0.80067E+00_JPRB, 0.80973E+00_JPRB, 0.79443E+00_JPRB, 0.77456E+00_JPRB, 0.78792E+00_JPRB, 0.77169E+00_JPRB, 0.75756E+00_JPRB ,&
&  0.79182E+00_JPRB, 0.85902E+00_JPRB, 0.95770E+00_JPRB, 0.68068E+00_JPRB, 0.60947E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.55822E+00_JPRB, 0.52398E+00_JPRB, 0.51848E+00_JPRB, 0.52000E+00_JPRB, 0.52736E+00_JPRB, 0.54009E+00_JPRB, 0.55385E+00_JPRB ,&
&  0.55843E+00_JPRB, 0.55948E+00_JPRB, 0.55942E+00_JPRB, 0.55410E+00_JPRB, 0.53832E+00_JPRB, 0.52141E+00_JPRB, 0.50461E+00_JPRB ,&
&  0.49823E+00_JPRB, 0.50074E+00_JPRB, 0.51727E+00_JPRB, 0.52606E+00_JPRB, 0.52680E+00_JPRB, 0.52365E+00_JPRB, 0.52500E+00_JPRB ,&
&  0.53047E+00_JPRB, 0.53207E+00_JPRB, 0.53215E+00_JPRB, 0.54995E+00_JPRB, 0.58928E+00_JPRB, 0.66738E+00_JPRB, 0.80902E+00_JPRB ,&
&  0.12082E+01_JPRB, 0.15362E+01_JPRB, 0.15174E+01_JPRB, 0.24465E+01_JPRB, 0.30446E+01_JPRB, 0.29855E+01_JPRB, 0.29007E+01_JPRB ,&
&  0.27044E+01_JPRB, 0.24031E+01_JPRB, 0.22578E+01_JPRB, 0.18672E+01_JPRB, 0.15898E+01_JPRB, 0.13824E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.10865E+01_JPRB, 0.11746E+01_JPRB, 0.13301E+01_JPRB, 0.14340E+01_JPRB, 0.14736E+01_JPRB, 0.15812E+01_JPRB ,&
&  0.18663E+01_JPRB, 0.23882E+01_JPRB, 0.28192E+01_JPRB, 0.26140E+01_JPRB, 0.21094E+01_JPRB, 0.23337E+01_JPRB, 0.25831E+01_JPRB ,&
&  0.29267E+01_JPRB, 0.20810E+01_JPRB, 0.16233E+01_JPRB, 0.14485E+01_JPRB, 0.13162E+01_JPRB, 0.14074E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.12281E+01_JPRB, 0.11239E+01_JPRB, 0.10326E+01_JPRB, 0.97272E+00_JPRB, 0.92748E+00_JPRB, 0.89340E+00_JPRB, 0.87158E+00_JPRB ,&
&  0.85859E+00_JPRB, 0.85234E+00_JPRB, 0.86183E+00_JPRB, 0.86337E+00_JPRB, 0.84907E+00_JPRB, 0.81718E+00_JPRB, 0.77347E+00_JPRB ,&
&  0.73677E+00_JPRB, 0.72245E+00_JPRB, 0.70656E+00_JPRB, 0.72180E+00_JPRB, 0.74476E+00_JPRB, 0.75868E+00_JPRB, 0.76365E+00_JPRB ,&
&  0.78635E+00_JPRB, 0.80368E+00_JPRB, 0.79399E+00_JPRB, 0.77395E+00_JPRB, 0.76721E+00_JPRB, 0.76459E+00_JPRB, 0.73501E+00_JPRB ,&
&  0.71904E+00_JPRB, 0.70555E+00_JPRB, 0.67584E+00_JPRB, 0.64371E+00_JPRB, 0.59594E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.56660E+00_JPRB, 0.54506E+00_JPRB, 0.53989E+00_JPRB, 0.54575E+00_JPRB, 0.55366E+00_JPRB, 0.56278E+00_JPRB, 0.57711E+00_JPRB ,&
&  0.58433E+00_JPRB, 0.58506E+00_JPRB, 0.58306E+00_JPRB, 0.57921E+00_JPRB, 0.56772E+00_JPRB, 0.55430E+00_JPRB, 0.53933E+00_JPRB ,&
&  0.52589E+00_JPRB, 0.50098E+00_JPRB, 0.48810E+00_JPRB, 0.49460E+00_JPRB, 0.49771E+00_JPRB, 0.49839E+00_JPRB, 0.50051E+00_JPRB ,&
&  0.50139E+00_JPRB, 0.50701E+00_JPRB, 0.50729E+00_JPRB, 0.51570E+00_JPRB, 0.53457E+00_JPRB, 0.58502E+00_JPRB, 0.67264E+00_JPRB ,&
&  0.94316E+00_JPRB, 0.14067E+01_JPRB, 0.13231E+01_JPRB, 0.28052E+01_JPRB, 0.31603E+01_JPRB, 0.30964E+01_JPRB, 0.27701E+01_JPRB ,&
&  0.25528E+01_JPRB, 0.26054E+01_JPRB, 0.20545E+01_JPRB, 0.17033E+01_JPRB, 0.15133E+01_JPRB, 0.13284E+01_JPRB, 0.11236E+01_JPRB ,&
&  0.96994E+00_JPRB, 0.89955E+00_JPRB, 0.88594E+00_JPRB, 0.95665E+00_JPRB, 0.10889E+01_JPRB, 0.12090E+01_JPRB, 0.13011E+01_JPRB ,&
&  0.14650E+01_JPRB, 0.18482E+01_JPRB, 0.24073E+01_JPRB, 0.23919E+01_JPRB, 0.22493E+01_JPRB, 0.25956E+01_JPRB, 0.32724E+01_JPRB ,&
&  0.39753E+01_JPRB, 0.30916E+01_JPRB, 0.20124E+01_JPRB, 0.16138E+01_JPRB, 0.15040E+01_JPRB, 0.15431E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.13861E+01_JPRB, 0.12835E+01_JPRB, 0.11633E+01_JPRB, 0.10830E+01_JPRB, 0.10262E+01_JPRB, 0.96482E+00_JPRB, 0.92170E+00_JPRB ,&
&  0.89842E+00_JPRB, 0.88378E+00_JPRB, 0.87690E+00_JPRB, 0.86672E+00_JPRB, 0.86303E+00_JPRB, 0.85630E+00_JPRB, 0.83723E+00_JPRB ,&
&  0.80116E+00_JPRB, 0.75815E+00_JPRB, 0.71130E+00_JPRB, 0.70163E+00_JPRB, 0.72204E+00_JPRB, 0.74119E+00_JPRB, 0.75321E+00_JPRB ,&
&  0.79133E+00_JPRB, 0.80178E+00_JPRB, 0.78112E+00_JPRB, 0.74854E+00_JPRB, 0.74417E+00_JPRB, 0.76771E+00_JPRB, 0.73684E+00_JPRB ,&
&  0.72693E+00_JPRB, 0.71090E+00_JPRB, 0.68716E+00_JPRB, 0.65919E+00_JPRB, 0.61333E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.60336E+00_JPRB, 0.58598E+00_JPRB, 0.57604E+00_JPRB, 0.57518E+00_JPRB, 0.57278E+00_JPRB, 0.57655E+00_JPRB, 0.58847E+00_JPRB ,&
&  0.60479E+00_JPRB, 0.60894E+00_JPRB, 0.60576E+00_JPRB, 0.60152E+00_JPRB, 0.59337E+00_JPRB, 0.58063E+00_JPRB, 0.56589E+00_JPRB ,&
&  0.55280E+00_JPRB, 0.53368E+00_JPRB, 0.50800E+00_JPRB, 0.48855E+00_JPRB, 0.49027E+00_JPRB, 0.49522E+00_JPRB, 0.49707E+00_JPRB ,&
&  0.49707E+00_JPRB, 0.50410E+00_JPRB, 0.50708E+00_JPRB, 0.51760E+00_JPRB, 0.52643E+00_JPRB, 0.55524E+00_JPRB, 0.63524E+00_JPRB ,&
&  0.84829E+00_JPRB, 0.16259E+01_JPRB, 0.14102E+01_JPRB, 0.25831E+01_JPRB, 0.29367E+01_JPRB, 0.28459E+01_JPRB, 0.25082E+01_JPRB ,&
&  0.23184E+01_JPRB, 0.21720E+01_JPRB, 0.18109E+01_JPRB, 0.15162E+01_JPRB, 0.13326E+01_JPRB, 0.11886E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.95536E+00_JPRB, 0.89530E+00_JPRB, 0.85879E+00_JPRB, 0.84096E+00_JPRB, 0.88269E+00_JPRB, 0.97100E+00_JPRB, 0.10728E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.13972E+01_JPRB, 0.18727E+01_JPRB, 0.23558E+01_JPRB, 0.22277E+01_JPRB, 0.26112E+01_JPRB, 0.39373E+01_JPRB ,&
&  0.46299E+01_JPRB, 0.44827E+01_JPRB, 0.28529E+01_JPRB, 0.19730E+01_JPRB, 0.16974E+01_JPRB, 0.17132E+01_JPRB, 0.16956E+01_JPRB ,&
&  0.15835E+01_JPRB, 0.14962E+01_JPRB, 0.13598E+01_JPRB, 0.12126E+01_JPRB, 0.11048E+01_JPRB, 0.10298E+01_JPRB, 0.96091E+00_JPRB ,&
&  0.91139E+00_JPRB, 0.88886E+00_JPRB, 0.86977E+00_JPRB, 0.84990E+00_JPRB, 0.83985E+00_JPRB, 0.84174E+00_JPRB, 0.84509E+00_JPRB ,&
&  0.83932E+00_JPRB, 0.81190E+00_JPRB, 0.75236E+00_JPRB, 0.71416E+00_JPRB, 0.71449E+00_JPRB, 0.73008E+00_JPRB, 0.74128E+00_JPRB ,&
&  0.78758E+00_JPRB, 0.79613E+00_JPRB, 0.76525E+00_JPRB, 0.74538E+00_JPRB, 0.73187E+00_JPRB, 0.77117E+00_JPRB, 0.73978E+00_JPRB ,&
&  0.72583E+00_JPRB, 0.71180E+00_JPRB, 0.70411E+00_JPRB, 0.68560E+00_JPRB, 0.64473E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.63618E+00_JPRB, 0.61494E+00_JPRB, 0.60453E+00_JPRB, 0.59220E+00_JPRB, 0.58363E+00_JPRB, 0.58257E+00_JPRB, 0.59249E+00_JPRB ,&
&  0.61170E+00_JPRB, 0.62502E+00_JPRB, 0.61743E+00_JPRB, 0.61253E+00_JPRB, 0.60910E+00_JPRB, 0.59859E+00_JPRB, 0.58477E+00_JPRB ,&
&  0.57078E+00_JPRB, 0.55739E+00_JPRB, 0.53996E+00_JPRB, 0.51945E+00_JPRB, 0.50517E+00_JPRB, 0.50415E+00_JPRB, 0.50170E+00_JPRB ,&
&  0.50007E+00_JPRB, 0.50469E+00_JPRB, 0.50901E+00_JPRB, 0.51678E+00_JPRB, 0.52535E+00_JPRB, 0.54400E+00_JPRB, 0.61619E+00_JPRB ,&
&  0.86496E+00_JPRB, 0.67931E+00_JPRB, 0.17139E+01_JPRB, 0.22773E+01_JPRB, 0.23697E+01_JPRB, 0.22129E+01_JPRB, 0.21234E+01_JPRB ,&
&  0.20160E+01_JPRB, 0.18071E+01_JPRB, 0.16149E+01_JPRB, 0.14481E+01_JPRB, 0.12430E+01_JPRB, 0.11074E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.96082E+00_JPRB, 0.90626E+00_JPRB, 0.86286E+00_JPRB, 0.82183E+00_JPRB, 0.79826E+00_JPRB, 0.81688E+00_JPRB, 0.87694E+00_JPRB ,&
&  0.95175E+00_JPRB, 0.10687E+01_JPRB, 0.13984E+01_JPRB, 0.18722E+01_JPRB, 0.21832E+01_JPRB, 0.25301E+01_JPRB, 0.32144E+01_JPRB ,&
&  0.42543E+01_JPRB, 0.46920E+01_JPRB, 0.35279E+01_JPRB, 0.25665E+01_JPRB, 0.20406E+01_JPRB, 0.19148E+01_JPRB, 0.19126E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.16771E+01_JPRB, 0.15196E+01_JPRB, 0.13430E+01_JPRB, 0.11924E+01_JPRB, 0.10904E+01_JPRB, 0.10160E+01_JPRB ,&
&  0.95300E+00_JPRB, 0.90326E+00_JPRB, 0.87078E+00_JPRB, 0.84781E+00_JPRB, 0.83256E+00_JPRB, 0.82739E+00_JPRB, 0.83216E+00_JPRB ,&
&  0.84341E+00_JPRB, 0.84079E+00_JPRB, 0.80463E+00_JPRB, 0.76882E+00_JPRB, 0.76224E+00_JPRB, 0.77252E+00_JPRB, 0.78710E+00_JPRB ,&
&  0.81128E+00_JPRB, 0.80901E+00_JPRB, 0.77298E+00_JPRB, 0.75230E+00_JPRB, 0.71703E+00_JPRB, 0.74510E+00_JPRB, 0.73077E+00_JPRB ,&
&  0.71822E+00_JPRB, 0.71152E+00_JPRB, 0.71365E+00_JPRB, 0.70156E+00_JPRB, 0.67087E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.66095E+00_JPRB, 0.63935E+00_JPRB, 0.62389E+00_JPRB, 0.60297E+00_JPRB, 0.59284E+00_JPRB, 0.59198E+00_JPRB, 0.60171E+00_JPRB ,&
&  0.61118E+00_JPRB, 0.61743E+00_JPRB, 0.61102E+00_JPRB, 0.60423E+00_JPRB, 0.59569E+00_JPRB, 0.58914E+00_JPRB, 0.57991E+00_JPRB ,&
&  0.57439E+00_JPRB, 0.56934E+00_JPRB, 0.56029E+00_JPRB, 0.54774E+00_JPRB, 0.53150E+00_JPRB, 0.52297E+00_JPRB, 0.51421E+00_JPRB ,&
&  0.50654E+00_JPRB, 0.50326E+00_JPRB, 0.50411E+00_JPRB, 0.51288E+00_JPRB, 0.52519E+00_JPRB, 0.54256E+00_JPRB, 0.59110E+00_JPRB ,&
&  0.87975E+00_JPRB, 0.12005E+01_JPRB, 0.19333E+01_JPRB, 0.19919E+01_JPRB, 0.17924E+01_JPRB, 0.15687E+01_JPRB, 0.15448E+01_JPRB ,&
&  0.14960E+01_JPRB, 0.14164E+01_JPRB, 0.13196E+01_JPRB, 0.12519E+01_JPRB, 0.11790E+01_JPRB, 0.11288E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.98463E+00_JPRB, 0.91160E+00_JPRB, 0.83468E+00_JPRB, 0.77552E+00_JPRB, 0.73760E+00_JPRB, 0.72611E+00_JPRB, 0.74585E+00_JPRB ,&
&  0.78111E+00_JPRB, 0.83157E+00_JPRB, 0.10179E+01_JPRB, 0.14151E+01_JPRB, 0.20398E+01_JPRB, 0.21882E+01_JPRB, 0.30062E+01_JPRB ,&
&  0.40914E+01_JPRB, 0.40552E+01_JPRB, 0.35859E+01_JPRB, 0.30800E+01_JPRB, 0.25571E+01_JPRB, 0.23529E+01_JPRB, 0.23035E+01_JPRB ,&
&  0.21330E+01_JPRB, 0.18597E+01_JPRB, 0.15876E+01_JPRB, 0.13842E+01_JPRB, 0.12448E+01_JPRB, 0.11580E+01_JPRB, 0.11060E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.99739E+00_JPRB, 0.94373E+00_JPRB, 0.90134E+00_JPRB, 0.87433E+00_JPRB, 0.86177E+00_JPRB, 0.85747E+00_JPRB ,&
&  0.85983E+00_JPRB, 0.85771E+00_JPRB, 0.84417E+00_JPRB, 0.82605E+00_JPRB, 0.84270E+00_JPRB, 0.87709E+00_JPRB, 0.89120E+00_JPRB ,&
&  0.87393E+00_JPRB, 0.84527E+00_JPRB, 0.80176E+00_JPRB, 0.76050E+00_JPRB, 0.72421E+00_JPRB, 0.74128E+00_JPRB, 0.72330E+00_JPRB ,&
&  0.70112E+00_JPRB, 0.69822E+00_JPRB, 0.71304E+00_JPRB, 0.71084E+00_JPRB, 0.69781E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.68200E+00_JPRB, 0.65803E+00_JPRB, 0.63685E+00_JPRB, 0.61781E+00_JPRB, 0.60279E+00_JPRB, 0.59864E+00_JPRB, 0.60240E+00_JPRB ,&
&  0.60657E+00_JPRB, 0.60436E+00_JPRB, 0.59746E+00_JPRB, 0.58773E+00_JPRB, 0.58058E+00_JPRB, 0.57395E+00_JPRB, 0.56830E+00_JPRB ,&
&  0.56051E+00_JPRB, 0.56121E+00_JPRB, 0.55713E+00_JPRB, 0.54895E+00_JPRB, 0.53918E+00_JPRB, 0.52732E+00_JPRB, 0.51600E+00_JPRB ,&
&  0.50814E+00_JPRB, 0.50461E+00_JPRB, 0.50420E+00_JPRB, 0.50852E+00_JPRB, 0.52180E+00_JPRB, 0.53711E+00_JPRB, 0.56380E+00_JPRB ,&
&  0.77908E+00_JPRB, 0.11951E+01_JPRB, 0.14824E+01_JPRB, 0.16571E+01_JPRB, 0.14701E+01_JPRB, 0.12680E+01_JPRB, 0.11615E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.11066E+01_JPRB, 0.10604E+01_JPRB, 0.10032E+01_JPRB, 0.93512E+00_JPRB, 0.90121E+00_JPRB, 0.86310E+00_JPRB ,&
&  0.82113E+00_JPRB, 0.78195E+00_JPRB, 0.74603E+00_JPRB, 0.72657E+00_JPRB, 0.70994E+00_JPRB, 0.69222E+00_JPRB, 0.68327E+00_JPRB ,&
&  0.68978E+00_JPRB, 0.70578E+00_JPRB, 0.76770E+00_JPRB, 0.10048E+01_JPRB, 0.15223E+01_JPRB, 0.19352E+01_JPRB, 0.24603E+01_JPRB ,&
&  0.26133E+01_JPRB, 0.27584E+01_JPRB, 0.28551E+01_JPRB, 0.27886E+01_JPRB, 0.25806E+01_JPRB, 0.24535E+01_JPRB, 0.24354E+01_JPRB ,&
&  0.22862E+01_JPRB, 0.19203E+01_JPRB, 0.15750E+01_JPRB, 0.13643E+01_JPRB, 0.12598E+01_JPRB, 0.12075E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.10969E+01_JPRB, 0.10503E+01_JPRB, 0.99758E+00_JPRB, 0.95017E+00_JPRB, 0.91008E+00_JPRB, 0.88537E+00_JPRB ,&
&  0.86559E+00_JPRB, 0.85493E+00_JPRB, 0.86161E+00_JPRB, 0.88281E+00_JPRB, 0.90621E+00_JPRB, 0.92619E+00_JPRB, 0.93554E+00_JPRB ,&
&  0.93874E+00_JPRB, 0.90499E+00_JPRB, 0.82948E+00_JPRB, 0.77611E+00_JPRB, 0.73295E+00_JPRB, 0.73032E+00_JPRB, 0.70238E+00_JPRB ,&
&  0.68366E+00_JPRB, 0.67743E+00_JPRB, 0.68924E+00_JPRB, 0.69730E+00_JPRB, 0.70134E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.75565E+00_JPRB, 0.70565E+00_JPRB, 0.66330E+00_JPRB, 0.63103E+00_JPRB, 0.61371E+00_JPRB, 0.60525E+00_JPRB, 0.59878E+00_JPRB ,&
&  0.59430E+00_JPRB, 0.58726E+00_JPRB, 0.57509E+00_JPRB, 0.56560E+00_JPRB, 0.55872E+00_JPRB, 0.55471E+00_JPRB, 0.55081E+00_JPRB ,&
&  0.53999E+00_JPRB, 0.53786E+00_JPRB, 0.53774E+00_JPRB, 0.53277E+00_JPRB, 0.52591E+00_JPRB, 0.51810E+00_JPRB, 0.50790E+00_JPRB ,&
&  0.50044E+00_JPRB, 0.49770E+00_JPRB, 0.49915E+00_JPRB, 0.50324E+00_JPRB, 0.50807E+00_JPRB, 0.52092E+00_JPRB, 0.55233E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.96051E+00_JPRB, 0.11361E+01_JPRB, 0.12141E+01_JPRB, 0.10570E+01_JPRB, 0.97289E+00_JPRB, 0.93249E+00_JPRB ,&
&  0.93116E+00_JPRB, 0.94067E+00_JPRB, 0.89495E+00_JPRB, 0.82018E+00_JPRB, 0.75228E+00_JPRB, 0.71521E+00_JPRB, 0.69088E+00_JPRB ,&
&  0.66913E+00_JPRB, 0.65198E+00_JPRB, 0.63638E+00_JPRB, 0.63473E+00_JPRB, 0.63375E+00_JPRB, 0.62851E+00_JPRB, 0.62428E+00_JPRB ,&
&  0.62414E+00_JPRB, 0.62997E+00_JPRB, 0.64679E+00_JPRB, 0.73378E+00_JPRB, 0.98708E+00_JPRB, 0.12729E+01_JPRB, 0.15610E+01_JPRB ,&
&  0.16990E+01_JPRB, 0.18353E+01_JPRB, 0.19970E+01_JPRB, 0.21109E+01_JPRB, 0.21229E+01_JPRB, 0.21204E+01_JPRB, 0.21170E+01_JPRB ,&
&  0.20579E+01_JPRB, 0.18410E+01_JPRB, 0.15546E+01_JPRB, 0.13467E+01_JPRB, 0.12343E+01_JPRB, 0.11768E+01_JPRB, 0.11390E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.10771E+01_JPRB, 0.10380E+01_JPRB, 0.98537E+00_JPRB, 0.92845E+00_JPRB, 0.88763E+00_JPRB, 0.85761E+00_JPRB ,&
&  0.83825E+00_JPRB, 0.84064E+00_JPRB, 0.86288E+00_JPRB, 0.90164E+00_JPRB, 0.92803E+00_JPRB, 0.93898E+00_JPRB, 0.93302E+00_JPRB ,&
&  0.92050E+00_JPRB, 0.91085E+00_JPRB, 0.83127E+00_JPRB, 0.77499E+00_JPRB, 0.78615E+00_JPRB, 0.72969E+00_JPRB, 0.68248E+00_JPRB ,&
&  0.66605E+00_JPRB, 0.65317E+00_JPRB, 0.65143E+00_JPRB, 0.66763E+00_JPRB, 0.77915E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.69556E+00_JPRB, 0.68041E+00_JPRB, 0.65375E+00_JPRB, 0.62874E+00_JPRB, 0.61359E+00_JPRB, 0.60509E+00_JPRB, 0.59969E+00_JPRB ,&
&  0.59192E+00_JPRB, 0.58072E+00_JPRB, 0.56767E+00_JPRB, 0.55189E+00_JPRB, 0.54164E+00_JPRB, 0.53610E+00_JPRB, 0.53193E+00_JPRB ,&
&  0.52565E+00_JPRB, 0.51508E+00_JPRB, 0.51147E+00_JPRB, 0.50821E+00_JPRB, 0.50656E+00_JPRB, 0.50433E+00_JPRB, 0.49880E+00_JPRB ,&
&  0.49203E+00_JPRB, 0.48873E+00_JPRB, 0.49176E+00_JPRB, 0.49345E+00_JPRB, 0.49584E+00_JPRB, 0.50254E+00_JPRB, 0.52379E+00_JPRB ,&
&  0.64314E+00_JPRB, 0.75210E+00_JPRB, 0.87950E+00_JPRB, 0.97872E+00_JPRB, 0.92160E+00_JPRB, 0.86365E+00_JPRB, 0.83201E+00_JPRB ,&
&  0.79212E+00_JPRB, 0.76539E+00_JPRB, 0.72056E+00_JPRB, 0.66719E+00_JPRB, 0.62299E+00_JPRB, 0.60754E+00_JPRB, 0.59779E+00_JPRB ,&
&  0.58968E+00_JPRB, 0.58393E+00_JPRB, 0.57953E+00_JPRB, 0.57724E+00_JPRB, 0.57717E+00_JPRB, 0.57542E+00_JPRB, 0.57370E+00_JPRB ,&
&  0.57479E+00_JPRB, 0.58009E+00_JPRB, 0.58835E+00_JPRB, 0.61022E+00_JPRB, 0.70493E+00_JPRB, 0.88408E+00_JPRB, 0.10772E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.12925E+01_JPRB, 0.14379E+01_JPRB, 0.15597E+01_JPRB, 0.16379E+01_JPRB, 0.16679E+01_JPRB, 0.16668E+01_JPRB ,&
&  0.16530E+01_JPRB, 0.15798E+01_JPRB, 0.14161E+01_JPRB, 0.12424E+01_JPRB, 0.11309E+01_JPRB, 0.10596E+01_JPRB, 0.10080E+01_JPRB ,&
&  0.96905E+00_JPRB, 0.92405E+00_JPRB, 0.88083E+00_JPRB, 0.84378E+00_JPRB, 0.81853E+00_JPRB, 0.79524E+00_JPRB, 0.77823E+00_JPRB ,&
&  0.78508E+00_JPRB, 0.81529E+00_JPRB, 0.85833E+00_JPRB, 0.88557E+00_JPRB, 0.89167E+00_JPRB, 0.88950E+00_JPRB, 0.87652E+00_JPRB ,&
&  0.86383E+00_JPRB, 0.85582E+00_JPRB, 0.83359E+00_JPRB, 0.80796E+00_JPRB, 0.76668E+00_JPRB, 0.72738E+00_JPRB, 0.67368E+00_JPRB ,&
&  0.64697E+00_JPRB, 0.63278E+00_JPRB, 0.61978E+00_JPRB, 0.63041E+00_JPRB, 0.67968E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.62943E+00_JPRB, 0.63024E+00_JPRB, 0.62683E+00_JPRB, 0.61910E+00_JPRB, 0.61511E+00_JPRB, 0.60994E+00_JPRB, 0.60020E+00_JPRB ,&
&  0.58831E+00_JPRB, 0.57740E+00_JPRB, 0.56624E+00_JPRB, 0.55180E+00_JPRB, 0.53630E+00_JPRB, 0.52948E+00_JPRB, 0.52674E+00_JPRB ,&
&  0.52160E+00_JPRB, 0.51265E+00_JPRB, 0.50430E+00_JPRB, 0.50197E+00_JPRB, 0.50080E+00_JPRB, 0.49987E+00_JPRB, 0.49623E+00_JPRB ,&
&  0.49046E+00_JPRB, 0.48940E+00_JPRB, 0.49247E+00_JPRB, 0.49297E+00_JPRB, 0.48883E+00_JPRB, 0.49081E+00_JPRB, 0.50831E+00_JPRB ,&
&  0.59737E+00_JPRB, 0.58158E+00_JPRB, 0.66361E+00_JPRB, 0.79560E+00_JPRB, 0.75511E+00_JPRB, 0.73394E+00_JPRB, 0.68951E+00_JPRB ,&
&  0.64457E+00_JPRB, 0.61997E+00_JPRB, 0.60204E+00_JPRB, 0.58017E+00_JPRB, 0.55868E+00_JPRB, 0.54712E+00_JPRB, 0.54671E+00_JPRB ,&
&  0.55243E+00_JPRB, 0.55497E+00_JPRB, 0.55499E+00_JPRB, 0.54863E+00_JPRB, 0.54439E+00_JPRB, 0.54098E+00_JPRB, 0.53892E+00_JPRB ,&
&  0.53773E+00_JPRB, 0.54087E+00_JPRB, 0.54688E+00_JPRB, 0.55454E+00_JPRB, 0.58166E+00_JPRB, 0.66758E+00_JPRB, 0.80831E+00_JPRB ,&
&  0.93353E+00_JPRB, 0.10347E+01_JPRB, 0.11324E+01_JPRB, 0.12335E+01_JPRB, 0.13102E+01_JPRB, 0.13788E+01_JPRB, 0.14149E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.13298E+01_JPRB, 0.12386E+01_JPRB, 0.11312E+01_JPRB, 0.10304E+01_JPRB, 0.94915E+00_JPRB, 0.88678E+00_JPRB ,&
&  0.83977E+00_JPRB, 0.80901E+00_JPRB, 0.78463E+00_JPRB, 0.75924E+00_JPRB, 0.74490E+00_JPRB, 0.74313E+00_JPRB, 0.75385E+00_JPRB ,&
&  0.77649E+00_JPRB, 0.80183E+00_JPRB, 0.81799E+00_JPRB, 0.82077E+00_JPRB, 0.81095E+00_JPRB, 0.80013E+00_JPRB, 0.78933E+00_JPRB ,&
&  0.78828E+00_JPRB, 0.79433E+00_JPRB, 0.79873E+00_JPRB, 0.76385E+00_JPRB, 0.76417E+00_JPRB, 0.74414E+00_JPRB, 0.70219E+00_JPRB ,&
&  0.66176E+00_JPRB, 0.63427E+00_JPRB, 0.61726E+00_JPRB, 0.55176E+00_JPRB, 0.62439E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.59738E+00_JPRB, 0.60031E+00_JPRB, 0.60190E+00_JPRB, 0.60246E+00_JPRB, 0.60121E+00_JPRB, 0.59539E+00_JPRB, 0.58449E+00_JPRB ,&
&  0.57289E+00_JPRB, 0.56145E+00_JPRB, 0.55238E+00_JPRB, 0.54278E+00_JPRB, 0.52855E+00_JPRB, 0.51863E+00_JPRB, 0.51574E+00_JPRB ,&
&  0.51296E+00_JPRB, 0.50681E+00_JPRB, 0.49996E+00_JPRB, 0.49853E+00_JPRB, 0.49608E+00_JPRB, 0.49262E+00_JPRB, 0.48790E+00_JPRB ,&
&  0.48316E+00_JPRB, 0.48156E+00_JPRB, 0.48177E+00_JPRB, 0.48256E+00_JPRB, 0.47923E+00_JPRB, 0.48104E+00_JPRB, 0.49892E+00_JPRB ,&
&  0.56899E+00_JPRB, 0.51152E+00_JPRB, 0.58125E+00_JPRB, 0.65507E+00_JPRB, 0.61450E+00_JPRB, 0.58256E+00_JPRB, 0.55592E+00_JPRB ,&
&  0.53598E+00_JPRB, 0.52581E+00_JPRB, 0.52030E+00_JPRB, 0.51411E+00_JPRB, 0.50881E+00_JPRB, 0.50294E+00_JPRB, 0.50200E+00_JPRB ,&
&  0.50405E+00_JPRB, 0.51144E+00_JPRB, 0.51577E+00_JPRB, 0.51581E+00_JPRB, 0.51153E+00_JPRB, 0.50680E+00_JPRB, 0.50257E+00_JPRB ,&
&  0.50026E+00_JPRB, 0.50102E+00_JPRB, 0.50384E+00_JPRB, 0.50975E+00_JPRB, 0.52348E+00_JPRB, 0.56005E+00_JPRB, 0.63769E+00_JPRB ,&
&  0.73721E+00_JPRB, 0.81792E+00_JPRB, 0.89555E+00_JPRB, 0.99272E+00_JPRB, 0.10886E+01_JPRB, 0.11581E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11627E+01_JPRB, 0.11048E+01_JPRB, 0.10503E+01_JPRB, 0.99145E+00_JPRB, 0.93278E+00_JPRB, 0.88169E+00_JPRB, 0.84516E+00_JPRB ,&
&  0.82051E+00_JPRB, 0.80437E+00_JPRB, 0.79572E+00_JPRB, 0.79094E+00_JPRB, 0.79234E+00_JPRB, 0.79604E+00_JPRB, 0.80181E+00_JPRB ,&
&  0.79861E+00_JPRB, 0.79041E+00_JPRB, 0.76728E+00_JPRB, 0.73382E+00_JPRB, 0.70948E+00_JPRB, 0.69637E+00_JPRB, 0.69025E+00_JPRB ,&
&  0.69151E+00_JPRB, 0.70035E+00_JPRB, 0.71297E+00_JPRB, 0.73239E+00_JPRB, 0.73330E+00_JPRB, 0.73267E+00_JPRB, 0.71522E+00_JPRB ,&
&  0.68068E+00_JPRB, 0.64507E+00_JPRB, 0.53522E+00_JPRB, 0.58511E+00_JPRB, 0.59871E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.60638E+00_JPRB, 0.60049E+00_JPRB, 0.59525E+00_JPRB, 0.58981E+00_JPRB, 0.58248E+00_JPRB, 0.57313E+00_JPRB, 0.56205E+00_JPRB ,&
&  0.55145E+00_JPRB, 0.54121E+00_JPRB, 0.53094E+00_JPRB, 0.52180E+00_JPRB, 0.51303E+00_JPRB, 0.50840E+00_JPRB, 0.50669E+00_JPRB ,&
&  0.50652E+00_JPRB, 0.50515E+00_JPRB, 0.50274E+00_JPRB, 0.50211E+00_JPRB, 0.50129E+00_JPRB, 0.49825E+00_JPRB, 0.49404E+00_JPRB ,&
&  0.48754E+00_JPRB, 0.48194E+00_JPRB, 0.47969E+00_JPRB, 0.48180E+00_JPRB, 0.48054E+00_JPRB, 0.48316E+00_JPRB, 0.49408E+00_JPRB ,&
&  0.48637E+00_JPRB, 0.46877E+00_JPRB, 0.51426E+00_JPRB, 0.56898E+00_JPRB, 0.53944E+00_JPRB, 0.51233E+00_JPRB, 0.50148E+00_JPRB ,&
&  0.49172E+00_JPRB, 0.48586E+00_JPRB, 0.48185E+00_JPRB, 0.48017E+00_JPRB, 0.48118E+00_JPRB, 0.48094E+00_JPRB, 0.47999E+00_JPRB ,&
&  0.47993E+00_JPRB, 0.48364E+00_JPRB, 0.48779E+00_JPRB, 0.49040E+00_JPRB, 0.49010E+00_JPRB, 0.48796E+00_JPRB, 0.48328E+00_JPRB ,&
&  0.47895E+00_JPRB, 0.47631E+00_JPRB, 0.47624E+00_JPRB, 0.48037E+00_JPRB, 0.48851E+00_JPRB, 0.50558E+00_JPRB, 0.54245E+00_JPRB ,&
&  0.60164E+00_JPRB, 0.66397E+00_JPRB, 0.72209E+00_JPRB, 0.79137E+00_JPRB, 0.87923E+00_JPRB, 0.96241E+00_JPRB, 0.10166E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.97128E+00_JPRB, 0.91285E+00_JPRB, 0.86150E+00_JPRB, 0.82226E+00_JPRB, 0.79709E+00_JPRB, 0.78234E+00_JPRB ,&
&  0.77701E+00_JPRB, 0.77460E+00_JPRB, 0.77424E+00_JPRB, 0.77479E+00_JPRB, 0.77431E+00_JPRB, 0.76873E+00_JPRB, 0.75776E+00_JPRB ,&
&  0.73811E+00_JPRB, 0.70706E+00_JPRB, 0.67147E+00_JPRB, 0.64246E+00_JPRB, 0.62239E+00_JPRB, 0.61242E+00_JPRB, 0.61034E+00_JPRB ,&
&  0.61525E+00_JPRB, 0.62836E+00_JPRB, 0.64668E+00_JPRB, 0.67754E+00_JPRB, 0.69961E+00_JPRB, 0.71362E+00_JPRB, 0.71731E+00_JPRB ,&
&  0.70042E+00_JPRB, 0.67037E+00_JPRB, 0.64073E+00_JPRB, 0.62411E+00_JPRB, 0.61635E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.60692E+00_JPRB, 0.59028E+00_JPRB, 0.57416E+00_JPRB, 0.56293E+00_JPRB, 0.55413E+00_JPRB, 0.54557E+00_JPRB, 0.53599E+00_JPRB ,&
&  0.52469E+00_JPRB, 0.51340E+00_JPRB, 0.50308E+00_JPRB, 0.49359E+00_JPRB, 0.48870E+00_JPRB, 0.48623E+00_JPRB, 0.48632E+00_JPRB ,&
&  0.48882E+00_JPRB, 0.49334E+00_JPRB, 0.49462E+00_JPRB, 0.49365E+00_JPRB, 0.49405E+00_JPRB, 0.49359E+00_JPRB, 0.49132E+00_JPRB ,&
&  0.48640E+00_JPRB, 0.48059E+00_JPRB, 0.47719E+00_JPRB, 0.47755E+00_JPRB, 0.47761E+00_JPRB, 0.47880E+00_JPRB, 0.48039E+00_JPRB ,&
&  0.48777E+00_JPRB, 0.45422E+00_JPRB, 0.48705E+00_JPRB, 0.51983E+00_JPRB, 0.50935E+00_JPRB, 0.49703E+00_JPRB, 0.48617E+00_JPRB ,&
&  0.47579E+00_JPRB, 0.47007E+00_JPRB, 0.46712E+00_JPRB, 0.46837E+00_JPRB, 0.47177E+00_JPRB, 0.47363E+00_JPRB, 0.47418E+00_JPRB ,&
&  0.47158E+00_JPRB, 0.47172E+00_JPRB, 0.47088E+00_JPRB, 0.47064E+00_JPRB, 0.47138E+00_JPRB, 0.47251E+00_JPRB, 0.47249E+00_JPRB ,&
&  0.47056E+00_JPRB, 0.46692E+00_JPRB, 0.46337E+00_JPRB, 0.46281E+00_JPRB, 0.46444E+00_JPRB, 0.46953E+00_JPRB, 0.48229E+00_JPRB ,&
&  0.50698E+00_JPRB, 0.53881E+00_JPRB, 0.57801E+00_JPRB, 0.62968E+00_JPRB, 0.70085E+00_JPRB, 0.77496E+00_JPRB, 0.83332E+00_JPRB ,&
&  0.85501E+00_JPRB, 0.84443E+00_JPRB, 0.81575E+00_JPRB, 0.77941E+00_JPRB, 0.74511E+00_JPRB, 0.71286E+00_JPRB, 0.68517E+00_JPRB ,&
&  0.66504E+00_JPRB, 0.64984E+00_JPRB, 0.63873E+00_JPRB, 0.63143E+00_JPRB, 0.62411E+00_JPRB, 0.61374E+00_JPRB, 0.60315E+00_JPRB ,&
&  0.59089E+00_JPRB, 0.57898E+00_JPRB, 0.56909E+00_JPRB, 0.56028E+00_JPRB, 0.55198E+00_JPRB, 0.54536E+00_JPRB, 0.54346E+00_JPRB ,&
&  0.54791E+00_JPRB, 0.55775E+00_JPRB, 0.57520E+00_JPRB, 0.59943E+00_JPRB, 0.62748E+00_JPRB, 0.65012E+00_JPRB, 0.66396E+00_JPRB ,&
&  0.66616E+00_JPRB, 0.66162E+00_JPRB, 0.64863E+00_JPRB, 0.63430E+00_JPRB, 0.62194E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.58251E+00_JPRB, 0.56869E+00_JPRB, 0.55485E+00_JPRB, 0.54359E+00_JPRB, 0.53439E+00_JPRB, 0.52439E+00_JPRB, 0.51319E+00_JPRB ,&
&  0.50379E+00_JPRB, 0.49410E+00_JPRB, 0.48701E+00_JPRB, 0.48112E+00_JPRB, 0.47693E+00_JPRB, 0.47552E+00_JPRB, 0.47614E+00_JPRB ,&
&  0.47877E+00_JPRB, 0.48445E+00_JPRB, 0.48692E+00_JPRB, 0.48487E+00_JPRB, 0.48637E+00_JPRB, 0.48854E+00_JPRB, 0.48873E+00_JPRB ,&
&  0.48576E+00_JPRB, 0.48196E+00_JPRB, 0.47789E+00_JPRB, 0.47537E+00_JPRB, 0.47510E+00_JPRB, 0.47636E+00_JPRB, 0.47381E+00_JPRB ,&
&  0.46974E+00_JPRB, 0.46797E+00_JPRB, 0.48264E+00_JPRB, 0.49446E+00_JPRB, 0.49407E+00_JPRB, 0.48670E+00_JPRB, 0.47786E+00_JPRB ,&
&  0.46786E+00_JPRB, 0.46255E+00_JPRB, 0.46277E+00_JPRB, 0.46522E+00_JPRB, 0.46815E+00_JPRB, 0.46985E+00_JPRB, 0.47168E+00_JPRB ,&
&  0.46869E+00_JPRB, 0.46698E+00_JPRB, 0.46451E+00_JPRB, 0.46262E+00_JPRB, 0.46317E+00_JPRB, 0.46437E+00_JPRB, 0.46530E+00_JPRB ,&
&  0.46455E+00_JPRB, 0.46525E+00_JPRB, 0.46587E+00_JPRB, 0.46477E+00_JPRB, 0.46599E+00_JPRB, 0.46822E+00_JPRB, 0.47215E+00_JPRB ,&
&  0.48016E+00_JPRB, 0.49348E+00_JPRB, 0.51077E+00_JPRB, 0.53737E+00_JPRB, 0.58143E+00_JPRB, 0.63271E+00_JPRB, 0.67440E+00_JPRB ,&
&  0.69647E+00_JPRB, 0.69999E+00_JPRB, 0.69135E+00_JPRB, 0.67489E+00_JPRB, 0.65804E+00_JPRB, 0.64113E+00_JPRB, 0.62206E+00_JPRB ,&
&  0.60166E+00_JPRB, 0.58576E+00_JPRB, 0.57228E+00_JPRB, 0.56182E+00_JPRB, 0.55271E+00_JPRB, 0.54472E+00_JPRB, 0.53904E+00_JPRB ,&
&  0.53560E+00_JPRB, 0.53263E+00_JPRB, 0.52814E+00_JPRB, 0.52322E+00_JPRB, 0.51991E+00_JPRB, 0.51547E+00_JPRB, 0.51299E+00_JPRB ,&
&  0.51456E+00_JPRB, 0.52037E+00_JPRB, 0.53081E+00_JPRB, 0.54745E+00_JPRB, 0.56570E+00_JPRB, 0.58229E+00_JPRB, 0.59603E+00_JPRB ,&
&  0.60642E+00_JPRB, 0.61151E+00_JPRB, 0.61033E+00_JPRB, 0.60504E+00_JPRB, 0.59574E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.53308E+00_JPRB, 0.52628E+00_JPRB, 0.51783E+00_JPRB, 0.50914E+00_JPRB, 0.50074E+00_JPRB, 0.49234E+00_JPRB, 0.48470E+00_JPRB ,&
&  0.47829E+00_JPRB, 0.47436E+00_JPRB, 0.47196E+00_JPRB, 0.47016E+00_JPRB, 0.46661E+00_JPRB, 0.46503E+00_JPRB, 0.46578E+00_JPRB ,&
&  0.46837E+00_JPRB, 0.47099E+00_JPRB, 0.47248E+00_JPRB, 0.47147E+00_JPRB, 0.47351E+00_JPRB, 0.47540E+00_JPRB, 0.47528E+00_JPRB ,&
&  0.47397E+00_JPRB, 0.47254E+00_JPRB, 0.46979E+00_JPRB, 0.46667E+00_JPRB, 0.46635E+00_JPRB, 0.46669E+00_JPRB, 0.46305E+00_JPRB ,&
&  0.45570E+00_JPRB, 0.46093E+00_JPRB, 0.45337E+00_JPRB, 0.47090E+00_JPRB, 0.47558E+00_JPRB, 0.46947E+00_JPRB, 0.46143E+00_JPRB ,&
&  0.45746E+00_JPRB, 0.45734E+00_JPRB, 0.46054E+00_JPRB, 0.46324E+00_JPRB, 0.46458E+00_JPRB, 0.46525E+00_JPRB, 0.46515E+00_JPRB ,&
&  0.45707E+00_JPRB, 0.45204E+00_JPRB, 0.45111E+00_JPRB, 0.45164E+00_JPRB, 0.45156E+00_JPRB, 0.45210E+00_JPRB, 0.45395E+00_JPRB ,&
&  0.45551E+00_JPRB, 0.45654E+00_JPRB, 0.45762E+00_JPRB, 0.46003E+00_JPRB, 0.46160E+00_JPRB, 0.46226E+00_JPRB, 0.46412E+00_JPRB ,&
&  0.46743E+00_JPRB, 0.47286E+00_JPRB, 0.47980E+00_JPRB, 0.48927E+00_JPRB, 0.50917E+00_JPRB, 0.53861E+00_JPRB, 0.56550E+00_JPRB ,&
&  0.58109E+00_JPRB, 0.58700E+00_JPRB, 0.58689E+00_JPRB, 0.58011E+00_JPRB, 0.56899E+00_JPRB, 0.55862E+00_JPRB, 0.54887E+00_JPRB ,&
&  0.53655E+00_JPRB, 0.52741E+00_JPRB, 0.51898E+00_JPRB, 0.50904E+00_JPRB, 0.50009E+00_JPRB, 0.49342E+00_JPRB, 0.48913E+00_JPRB ,&
&  0.48738E+00_JPRB, 0.48785E+00_JPRB, 0.48709E+00_JPRB, 0.48704E+00_JPRB, 0.48908E+00_JPRB, 0.49184E+00_JPRB, 0.49139E+00_JPRB ,&
&  0.48987E+00_JPRB, 0.48965E+00_JPRB, 0.49222E+00_JPRB, 0.49758E+00_JPRB, 0.50393E+00_JPRB, 0.51223E+00_JPRB, 0.52135E+00_JPRB ,&
&  0.53004E+00_JPRB, 0.53652E+00_JPRB, 0.53990E+00_JPRB, 0.54104E+00_JPRB, 0.53835E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.48241E+00_JPRB, 0.48038E+00_JPRB, 0.47758E+00_JPRB, 0.47326E+00_JPRB, 0.46853E+00_JPRB, 0.46434E+00_JPRB, 0.46100E+00_JPRB ,&
&  0.45887E+00_JPRB, 0.45823E+00_JPRB, 0.45869E+00_JPRB, 0.45887E+00_JPRB, 0.45877E+00_JPRB, 0.45971E+00_JPRB, 0.46020E+00_JPRB ,&
&  0.46190E+00_JPRB, 0.46223E+00_JPRB, 0.45955E+00_JPRB, 0.45869E+00_JPRB, 0.46102E+00_JPRB, 0.46564E+00_JPRB, 0.46708E+00_JPRB ,&
&  0.46437E+00_JPRB, 0.46364E+00_JPRB, 0.46290E+00_JPRB, 0.46027E+00_JPRB, 0.45828E+00_JPRB, 0.45714E+00_JPRB, 0.45325E+00_JPRB ,&
&  0.44762E+00_JPRB, 0.44921E+00_JPRB, 0.45082E+00_JPRB, 0.45635E+00_JPRB, 0.45803E+00_JPRB, 0.45768E+00_JPRB, 0.45841E+00_JPRB ,&
&  0.45661E+00_JPRB, 0.45614E+00_JPRB, 0.45578E+00_JPRB, 0.45617E+00_JPRB, 0.45423E+00_JPRB, 0.45108E+00_JPRB, 0.44510E+00_JPRB ,&
&  0.43444E+00_JPRB, 0.43051E+00_JPRB, 0.43234E+00_JPRB, 0.43675E+00_JPRB, 0.43607E+00_JPRB, 0.43651E+00_JPRB, 0.43958E+00_JPRB ,&
&  0.44356E+00_JPRB, 0.44649E+00_JPRB, 0.44829E+00_JPRB, 0.44939E+00_JPRB, 0.45096E+00_JPRB, 0.45244E+00_JPRB, 0.45321E+00_JPRB ,&
&  0.45426E+00_JPRB, 0.45629E+00_JPRB, 0.45957E+00_JPRB, 0.46437E+00_JPRB, 0.47194E+00_JPRB, 0.48503E+00_JPRB, 0.49895E+00_JPRB ,&
&  0.50826E+00_JPRB, 0.51141E+00_JPRB, 0.51150E+00_JPRB, 0.50797E+00_JPRB, 0.50089E+00_JPRB, 0.49358E+00_JPRB, 0.48633E+00_JPRB ,&
&  0.47669E+00_JPRB, 0.46827E+00_JPRB, 0.46150E+00_JPRB, 0.45732E+00_JPRB, 0.45481E+00_JPRB, 0.45378E+00_JPRB, 0.45415E+00_JPRB ,&
&  0.45480E+00_JPRB, 0.45730E+00_JPRB, 0.46329E+00_JPRB, 0.46661E+00_JPRB, 0.46794E+00_JPRB, 0.47098E+00_JPRB, 0.47264E+00_JPRB ,&
&  0.47101E+00_JPRB, 0.46962E+00_JPRB, 0.46895E+00_JPRB, 0.47149E+00_JPRB, 0.47329E+00_JPRB, 0.47322E+00_JPRB, 0.47385E+00_JPRB ,&
&  0.47719E+00_JPRB, 0.48097E+00_JPRB, 0.48230E+00_JPRB, 0.48295E+00_JPRB, 0.48362E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.44206E+00_JPRB, 0.44215E+00_JPRB, 0.44148E+00_JPRB, 0.44029E+00_JPRB, 0.43910E+00_JPRB, 0.43819E+00_JPRB, 0.43833E+00_JPRB ,&
&  0.43865E+00_JPRB, 0.43951E+00_JPRB, 0.44067E+00_JPRB, 0.44162E+00_JPRB, 0.44350E+00_JPRB, 0.44568E+00_JPRB, 0.44755E+00_JPRB ,&
&  0.44948E+00_JPRB, 0.44889E+00_JPRB, 0.44570E+00_JPRB, 0.44254E+00_JPRB, 0.44322E+00_JPRB, 0.44870E+00_JPRB, 0.45465E+00_JPRB ,&
&  0.45582E+00_JPRB, 0.45649E+00_JPRB, 0.45651E+00_JPRB, 0.45485E+00_JPRB, 0.45272E+00_JPRB, 0.45205E+00_JPRB, 0.44838E+00_JPRB ,&
&  0.44336E+00_JPRB, 0.44220E+00_JPRB, 0.44147E+00_JPRB, 0.43989E+00_JPRB, 0.44100E+00_JPRB, 0.44523E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.44598E+00_JPRB, 0.44437E+00_JPRB, 0.44087E+00_JPRB, 0.43731E+00_JPRB, 0.43290E+00_JPRB, 0.42507E+00_JPRB, 0.41611E+00_JPRB ,&
&  0.40910E+00_JPRB, 0.40894E+00_JPRB, 0.41448E+00_JPRB, 0.42012E+00_JPRB, 0.41859E+00_JPRB, 0.41699E+00_JPRB, 0.41832E+00_JPRB ,&
&  0.42197E+00_JPRB, 0.42677E+00_JPRB, 0.42987E+00_JPRB, 0.42918E+00_JPRB, 0.42807E+00_JPRB, 0.42998E+00_JPRB, 0.43271E+00_JPRB ,&
&  0.43392E+00_JPRB, 0.43525E+00_JPRB, 0.43627E+00_JPRB, 0.43826E+00_JPRB, 0.44031E+00_JPRB, 0.44281E+00_JPRB, 0.44773E+00_JPRB ,&
&  0.45103E+00_JPRB, 0.45193E+00_JPRB, 0.44945E+00_JPRB, 0.44406E+00_JPRB, 0.43852E+00_JPRB, 0.43348E+00_JPRB, 0.43087E+00_JPRB ,&
&  0.42779E+00_JPRB, 0.42729E+00_JPRB, 0.42626E+00_JPRB, 0.42316E+00_JPRB, 0.41970E+00_JPRB, 0.41937E+00_JPRB, 0.42286E+00_JPRB ,&
&  0.42793E+00_JPRB, 0.43207E+00_JPRB, 0.43820E+00_JPRB, 0.44079E+00_JPRB, 0.44243E+00_JPRB, 0.44494E+00_JPRB, 0.44585E+00_JPRB ,&
&  0.44334E+00_JPRB, 0.44119E+00_JPRB, 0.44073E+00_JPRB, 0.44375E+00_JPRB, 0.44472E+00_JPRB, 0.44355E+00_JPRB, 0.44015E+00_JPRB ,&
&  0.43876E+00_JPRB, 0.44025E+00_JPRB, 0.44068E+00_JPRB, 0.44016E+00_JPRB, 0.44070E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.41534E+00_JPRB, 0.41754E+00_JPRB, 0.41992E+00_JPRB, 0.42208E+00_JPRB, 0.42345E+00_JPRB, 0.42444E+00_JPRB, 0.42363E+00_JPRB ,&
&  0.42358E+00_JPRB, 0.42538E+00_JPRB, 0.42765E+00_JPRB, 0.42956E+00_JPRB, 0.43077E+00_JPRB, 0.43299E+00_JPRB, 0.43535E+00_JPRB ,&
&  0.43698E+00_JPRB, 0.43609E+00_JPRB, 0.43218E+00_JPRB, 0.43000E+00_JPRB, 0.42876E+00_JPRB, 0.43404E+00_JPRB, 0.44284E+00_JPRB ,&
&  0.44665E+00_JPRB, 0.44884E+00_JPRB, 0.45148E+00_JPRB, 0.45172E+00_JPRB, 0.45154E+00_JPRB, 0.45110E+00_JPRB, 0.44649E+00_JPRB ,&
&  0.44119E+00_JPRB, 0.43646E+00_JPRB, 0.43305E+00_JPRB, 0.42982E+00_JPRB, 0.43040E+00_JPRB, 0.42912E+00_JPRB, 0.42848E+00_JPRB ,&
&  0.42790E+00_JPRB, 0.42539E+00_JPRB, 0.42148E+00_JPRB, 0.41511E+00_JPRB, 0.40736E+00_JPRB, 0.39732E+00_JPRB, 0.39023E+00_JPRB ,&
&  0.38740E+00_JPRB, 0.39130E+00_JPRB, 0.40016E+00_JPRB, 0.40376E+00_JPRB, 0.39976E+00_JPRB, 0.39830E+00_JPRB, 0.40126E+00_JPRB ,&
&  0.40622E+00_JPRB, 0.41194E+00_JPRB, 0.41426E+00_JPRB, 0.41356E+00_JPRB, 0.41149E+00_JPRB, 0.41310E+00_JPRB, 0.41752E+00_JPRB ,&
&  0.42130E+00_JPRB, 0.42315E+00_JPRB, 0.42219E+00_JPRB, 0.42124E+00_JPRB, 0.41909E+00_JPRB, 0.41461E+00_JPRB, 0.40874E+00_JPRB ,&
&  0.40336E+00_JPRB, 0.39997E+00_JPRB, 0.39730E+00_JPRB, 0.39429E+00_JPRB, 0.39290E+00_JPRB, 0.39290E+00_JPRB, 0.39482E+00_JPRB ,&
&  0.39539E+00_JPRB, 0.39698E+00_JPRB, 0.39693E+00_JPRB, 0.39269E+00_JPRB, 0.38939E+00_JPRB, 0.39117E+00_JPRB, 0.39325E+00_JPRB ,&
&  0.39635E+00_JPRB, 0.39938E+00_JPRB, 0.40229E+00_JPRB, 0.40326E+00_JPRB, 0.40473E+00_JPRB, 0.40758E+00_JPRB, 0.40938E+00_JPRB ,&
&  0.41030E+00_JPRB, 0.41091E+00_JPRB, 0.41281E+00_JPRB, 0.41614E+00_JPRB, 0.41940E+00_JPRB, 0.42207E+00_JPRB, 0.42151E+00_JPRB ,&
&  0.41989E+00_JPRB, 0.41776E+00_JPRB, 0.41760E+00_JPRB, 0.41736E+00_JPRB, 0.41548E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.40904E+00_JPRB, 0.40862E+00_JPRB, 0.41178E+00_JPRB, 0.41520E+00_JPRB, 0.41580E+00_JPRB, 0.41521E+00_JPRB, 0.41238E+00_JPRB ,&
&  0.41050E+00_JPRB, 0.41163E+00_JPRB, 0.41347E+00_JPRB, 0.41489E+00_JPRB, 0.41734E+00_JPRB, 0.42072E+00_JPRB, 0.42345E+00_JPRB ,&
&  0.42365E+00_JPRB, 0.42166E+00_JPRB, 0.41827E+00_JPRB, 0.41840E+00_JPRB, 0.41820E+00_JPRB, 0.42051E+00_JPRB, 0.42837E+00_JPRB ,&
&  0.43401E+00_JPRB, 0.43777E+00_JPRB, 0.44089E+00_JPRB, 0.44113E+00_JPRB, 0.44027E+00_JPRB, 0.44163E+00_JPRB, 0.43798E+00_JPRB ,&
&  0.43145E+00_JPRB, 0.42555E+00_JPRB, 0.41960E+00_JPRB, 0.41260E+00_JPRB, 0.41540E+00_JPRB, 0.41309E+00_JPRB, 0.41018E+00_JPRB ,&
&  0.40586E+00_JPRB, 0.39994E+00_JPRB, 0.39288E+00_JPRB, 0.38618E+00_JPRB, 0.38114E+00_JPRB, 0.37647E+00_JPRB, 0.37076E+00_JPRB ,&
&  0.36897E+00_JPRB, 0.37153E+00_JPRB, 0.37730E+00_JPRB, 0.37911E+00_JPRB, 0.37564E+00_JPRB, 0.37596E+00_JPRB, 0.38264E+00_JPRB ,&
&  0.38995E+00_JPRB, 0.39363E+00_JPRB, 0.39367E+00_JPRB, 0.39430E+00_JPRB, 0.39492E+00_JPRB, 0.39635E+00_JPRB, 0.39706E+00_JPRB ,&
&  0.39592E+00_JPRB, 0.39411E+00_JPRB, 0.39141E+00_JPRB, 0.38551E+00_JPRB, 0.37876E+00_JPRB, 0.37286E+00_JPRB, 0.37092E+00_JPRB ,&
&  0.37405E+00_JPRB, 0.37465E+00_JPRB, 0.37583E+00_JPRB, 0.37813E+00_JPRB, 0.37889E+00_JPRB, 0.37824E+00_JPRB, 0.37737E+00_JPRB ,&
&  0.37885E+00_JPRB, 0.37741E+00_JPRB, 0.37819E+00_JPRB, 0.37530E+00_JPRB, 0.37089E+00_JPRB, 0.36656E+00_JPRB, 0.36235E+00_JPRB ,&
&  0.36018E+00_JPRB, 0.35987E+00_JPRB, 0.36209E+00_JPRB, 0.36289E+00_JPRB, 0.36479E+00_JPRB, 0.36711E+00_JPRB, 0.36910E+00_JPRB ,&
&  0.37033E+00_JPRB, 0.37211E+00_JPRB, 0.37539E+00_JPRB, 0.38161E+00_JPRB, 0.38813E+00_JPRB, 0.39388E+00_JPRB, 0.39950E+00_JPRB ,&
&  0.40453E+00_JPRB, 0.40790E+00_JPRB, 0.41362E+00_JPRB, 0.41433E+00_JPRB, 0.41285E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.40730E+00_JPRB, 0.40423E+00_JPRB, 0.40385E+00_JPRB, 0.40428E+00_JPRB, 0.40315E+00_JPRB, 0.40168E+00_JPRB, 0.39854E+00_JPRB ,&
&  0.39586E+00_JPRB, 0.39614E+00_JPRB, 0.39757E+00_JPRB, 0.40030E+00_JPRB, 0.40514E+00_JPRB, 0.41028E+00_JPRB, 0.41000E+00_JPRB ,&
&  0.40705E+00_JPRB, 0.40594E+00_JPRB, 0.40833E+00_JPRB, 0.41020E+00_JPRB, 0.41028E+00_JPRB, 0.41095E+00_JPRB, 0.41481E+00_JPRB ,&
&  0.41957E+00_JPRB, 0.42174E+00_JPRB, 0.42474E+00_JPRB, 0.42423E+00_JPRB, 0.42018E+00_JPRB, 0.42027E+00_JPRB, 0.41832E+00_JPRB ,&
&  0.41327E+00_JPRB, 0.40680E+00_JPRB, 0.39731E+00_JPRB, 0.39793E+00_JPRB, 0.39418E+00_JPRB, 0.39102E+00_JPRB, 0.38618E+00_JPRB ,&
&  0.38075E+00_JPRB, 0.37594E+00_JPRB, 0.37065E+00_JPRB, 0.36643E+00_JPRB, 0.36445E+00_JPRB, 0.36436E+00_JPRB, 0.36222E+00_JPRB ,&
&  0.36110E+00_JPRB, 0.36351E+00_JPRB, 0.36471E+00_JPRB, 0.36048E+00_JPRB, 0.35923E+00_JPRB, 0.36516E+00_JPRB, 0.37052E+00_JPRB ,&
&  0.37297E+00_JPRB, 0.37316E+00_JPRB, 0.37041E+00_JPRB, 0.36969E+00_JPRB, 0.36864E+00_JPRB, 0.36664E+00_JPRB, 0.36309E+00_JPRB ,&
&  0.35760E+00_JPRB, 0.35489E+00_JPRB, 0.35308E+00_JPRB, 0.34946E+00_JPRB, 0.34266E+00_JPRB, 0.32661E+00_JPRB, 0.31782E+00_JPRB ,&
&  0.31903E+00_JPRB, 0.32839E+00_JPRB, 0.34269E+00_JPRB, 0.35056E+00_JPRB, 0.35804E+00_JPRB, 0.36356E+00_JPRB, 0.36394E+00_JPRB ,&
&  0.36356E+00_JPRB, 0.34381E+00_JPRB, 0.33283E+00_JPRB, 0.32335E+00_JPRB, 0.31503E+00_JPRB, 0.31022E+00_JPRB, 0.30860E+00_JPRB ,&
&  0.31170E+00_JPRB, 0.31758E+00_JPRB, 0.32345E+00_JPRB, 0.32656E+00_JPRB, 0.32656E+00_JPRB, 0.32395E+00_JPRB, 0.32077E+00_JPRB ,&
&  0.32126E+00_JPRB, 0.32916E+00_JPRB, 0.34360E+00_JPRB, 0.35636E+00_JPRB, 0.36609E+00_JPRB, 0.37663E+00_JPRB, 0.38532E+00_JPRB ,&
&  0.39145E+00_JPRB, 0.39724E+00_JPRB, 0.40409E+00_JPRB, 0.40654E+00_JPRB, 0.40742E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.39284E+00_JPRB, 0.38851E+00_JPRB, 0.38821E+00_JPRB, 0.38833E+00_JPRB, 0.38684E+00_JPRB, 0.38635E+00_JPRB, 0.38527E+00_JPRB ,&
&  0.38453E+00_JPRB, 0.38246E+00_JPRB, 0.38067E+00_JPRB, 0.38133E+00_JPRB, 0.38373E+00_JPRB, 0.38642E+00_JPRB, 0.38729E+00_JPRB ,&
&  0.38739E+00_JPRB, 0.38914E+00_JPRB, 0.39287E+00_JPRB, 0.39645E+00_JPRB, 0.39695E+00_JPRB, 0.39714E+00_JPRB, 0.39899E+00_JPRB ,&
&  0.40130E+00_JPRB, 0.40212E+00_JPRB, 0.40498E+00_JPRB, 0.40260E+00_JPRB, 0.39783E+00_JPRB, 0.39601E+00_JPRB, 0.39440E+00_JPRB ,&
&  0.38711E+00_JPRB, 0.36285E+00_JPRB, 0.34763E+00_JPRB, 0.35314E+00_JPRB, 0.36816E+00_JPRB, 0.36684E+00_JPRB, 0.36441E+00_JPRB ,&
&  0.36186E+00_JPRB, 0.35836E+00_JPRB, 0.35543E+00_JPRB, 0.35434E+00_JPRB, 0.35470E+00_JPRB, 0.35497E+00_JPRB, 0.35389E+00_JPRB ,&
&  0.35473E+00_JPRB, 0.36050E+00_JPRB, 0.36657E+00_JPRB, 0.36223E+00_JPRB, 0.35642E+00_JPRB, 0.34852E+00_JPRB, 0.34356E+00_JPRB ,&
&  0.33895E+00_JPRB, 0.33450E+00_JPRB, 0.33149E+00_JPRB, 0.33062E+00_JPRB, 0.33039E+00_JPRB, 0.32816E+00_JPRB, 0.32433E+00_JPRB ,&
&  0.31774E+00_JPRB, 0.30818E+00_JPRB, 0.29869E+00_JPRB, 0.28932E+00_JPRB, 0.28292E+00_JPRB, 0.27825E+00_JPRB, 0.27443E+00_JPRB ,&
&  0.27689E+00_JPRB, 0.29015E+00_JPRB, 0.31050E+00_JPRB, 0.33120E+00_JPRB, 0.34769E+00_JPRB, 0.32649E+00_JPRB, 0.31645E+00_JPRB ,&
&  0.30458E+00_JPRB, 0.29424E+00_JPRB, 0.28698E+00_JPRB, 0.28179E+00_JPRB, 0.27800E+00_JPRB, 0.27582E+00_JPRB, 0.27566E+00_JPRB ,&
&  0.27899E+00_JPRB, 0.28348E+00_JPRB, 0.28767E+00_JPRB, 0.28918E+00_JPRB, 0.28890E+00_JPRB, 0.28720E+00_JPRB, 0.28626E+00_JPRB ,&
&  0.28789E+00_JPRB, 0.29428E+00_JPRB, 0.30440E+00_JPRB, 0.31921E+00_JPRB, 0.33221E+00_JPRB, 0.34548E+00_JPRB, 0.36028E+00_JPRB ,&
&  0.37557E+00_JPRB, 0.38861E+00_JPRB, 0.39219E+00_JPRB, 0.39568E+00_JPRB, 0.39608E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.38384E+00_JPRB, 0.38109E+00_JPRB, 0.37905E+00_JPRB, 0.37933E+00_JPRB, 0.37946E+00_JPRB, 0.38177E+00_JPRB, 0.38037E+00_JPRB ,&
&  0.37561E+00_JPRB, 0.37094E+00_JPRB, 0.36970E+00_JPRB, 0.36989E+00_JPRB, 0.36888E+00_JPRB, 0.36747E+00_JPRB, 0.36845E+00_JPRB ,&
&  0.36700E+00_JPRB, 0.36938E+00_JPRB, 0.37272E+00_JPRB, 0.37450E+00_JPRB, 0.37514E+00_JPRB, 0.37520E+00_JPRB, 0.37549E+00_JPRB ,&
&  0.36913E+00_JPRB, 0.36703E+00_JPRB, 0.37281E+00_JPRB, 0.36973E+00_JPRB, 0.36548E+00_JPRB, 0.36178E+00_JPRB, 0.36038E+00_JPRB ,&
&  0.35106E+00_JPRB, 0.33837E+00_JPRB, 0.32598E+00_JPRB, 0.33326E+00_JPRB, 0.35008E+00_JPRB, 0.34622E+00_JPRB, 0.34385E+00_JPRB ,&
&  0.34363E+00_JPRB, 0.34288E+00_JPRB, 0.34383E+00_JPRB, 0.34449E+00_JPRB, 0.34594E+00_JPRB, 0.34649E+00_JPRB, 0.34829E+00_JPRB ,&
&  0.35018E+00_JPRB, 0.35732E+00_JPRB, 0.36036E+00_JPRB, 0.33882E+00_JPRB, 0.32108E+00_JPRB, 0.30755E+00_JPRB, 0.29445E+00_JPRB ,&
&  0.28280E+00_JPRB, 0.28126E+00_JPRB, 0.27736E+00_JPRB, 0.26966E+00_JPRB, 0.27007E+00_JPRB, 0.27092E+00_JPRB, 0.26998E+00_JPRB ,&
&  0.26752E+00_JPRB, 0.26607E+00_JPRB, 0.26465E+00_JPRB, 0.26219E+00_JPRB, 0.26138E+00_JPRB, 0.26295E+00_JPRB, 0.26722E+00_JPRB ,&
&  0.27372E+00_JPRB, 0.28059E+00_JPRB, 0.29619E+00_JPRB, 0.30917E+00_JPRB, 0.30434E+00_JPRB, 0.29786E+00_JPRB, 0.29083E+00_JPRB ,&
&  0.27866E+00_JPRB, 0.27291E+00_JPRB, 0.26712E+00_JPRB, 0.26240E+00_JPRB, 0.26023E+00_JPRB, 0.26040E+00_JPRB, 0.26117E+00_JPRB ,&
&  0.26240E+00_JPRB, 0.26468E+00_JPRB, 0.26561E+00_JPRB, 0.26533E+00_JPRB, 0.26461E+00_JPRB, 0.26542E+00_JPRB, 0.26749E+00_JPRB ,&
&  0.27105E+00_JPRB, 0.27703E+00_JPRB, 0.28413E+00_JPRB, 0.29299E+00_JPRB, 0.30091E+00_JPRB, 0.30834E+00_JPRB, 0.32008E+00_JPRB ,&
&  0.32849E+00_JPRB, 0.34863E+00_JPRB, 0.37521E+00_JPRB, 0.39721E+00_JPRB, 0.38912E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.36325E+00_JPRB, 0.36100E+00_JPRB, 0.35837E+00_JPRB, 0.35801E+00_JPRB, 0.35886E+00_JPRB, 0.36127E+00_JPRB, 0.35853E+00_JPRB ,&
&  0.35357E+00_JPRB, 0.35156E+00_JPRB, 0.35279E+00_JPRB, 0.35219E+00_JPRB, 0.35203E+00_JPRB, 0.34224E+00_JPRB, 0.33669E+00_JPRB ,&
&  0.32968E+00_JPRB, 0.32778E+00_JPRB, 0.32615E+00_JPRB, 0.32722E+00_JPRB, 0.32924E+00_JPRB, 0.33042E+00_JPRB, 0.32593E+00_JPRB ,&
&  0.32136E+00_JPRB, 0.31719E+00_JPRB, 0.31264E+00_JPRB, 0.31068E+00_JPRB, 0.31198E+00_JPRB, 0.31315E+00_JPRB, 0.31306E+00_JPRB ,&
&  0.31380E+00_JPRB, 0.31680E+00_JPRB, 0.31979E+00_JPRB, 0.32367E+00_JPRB, 0.33645E+00_JPRB, 0.33378E+00_JPRB, 0.33247E+00_JPRB ,&
&  0.33302E+00_JPRB, 0.33373E+00_JPRB, 0.33521E+00_JPRB, 0.33638E+00_JPRB, 0.34136E+00_JPRB, 0.34406E+00_JPRB, 0.34623E+00_JPRB ,&
&  0.33288E+00_JPRB, 0.32615E+00_JPRB, 0.31878E+00_JPRB, 0.30919E+00_JPRB, 0.29651E+00_JPRB, 0.28661E+00_JPRB, 0.27754E+00_JPRB ,&
&  0.26996E+00_JPRB, 0.26286E+00_JPRB, 0.25618E+00_JPRB, 0.25361E+00_JPRB, 0.25264E+00_JPRB, 0.25110E+00_JPRB, 0.25089E+00_JPRB ,&
&  0.25148E+00_JPRB, 0.25234E+00_JPRB, 0.25284E+00_JPRB, 0.25454E+00_JPRB, 0.25729E+00_JPRB, 0.26166E+00_JPRB, 0.26631E+00_JPRB ,&
&  0.27316E+00_JPRB, 0.27792E+00_JPRB, 0.28363E+00_JPRB, 0.28883E+00_JPRB, 0.28734E+00_JPRB, 0.28177E+00_JPRB, 0.27595E+00_JPRB ,&
&  0.26844E+00_JPRB, 0.26420E+00_JPRB, 0.25983E+00_JPRB, 0.25667E+00_JPRB, 0.25486E+00_JPRB, 0.25374E+00_JPRB, 0.25391E+00_JPRB ,&
&  0.25388E+00_JPRB, 0.25430E+00_JPRB, 0.25371E+00_JPRB, 0.25399E+00_JPRB, 0.25497E+00_JPRB, 0.25708E+00_JPRB, 0.25988E+00_JPRB ,&
&  0.26422E+00_JPRB, 0.26924E+00_JPRB, 0.27513E+00_JPRB, 0.28156E+00_JPRB, 0.29127E+00_JPRB, 0.30354E+00_JPRB, 0.32046E+00_JPRB ,&
&  0.33650E+00_JPRB, 0.36399E+00_JPRB, 0.37459E+00_JPRB, 0.36926E+00_JPRB, 0.36704E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.34664E+00_JPRB, 0.34816E+00_JPRB, 0.34921E+00_JPRB, 0.35144E+00_JPRB, 0.35409E+00_JPRB, 0.35483E+00_JPRB, 0.35488E+00_JPRB ,&
&  0.34533E+00_JPRB, 0.33801E+00_JPRB, 0.33113E+00_JPRB, 0.32094E+00_JPRB, 0.31281E+00_JPRB, 0.30562E+00_JPRB, 0.30046E+00_JPRB ,&
&  0.29498E+00_JPRB, 0.29249E+00_JPRB, 0.28930E+00_JPRB, 0.28841E+00_JPRB, 0.28778E+00_JPRB, 0.28816E+00_JPRB, 0.28637E+00_JPRB ,&
&  0.28455E+00_JPRB, 0.28594E+00_JPRB, 0.28676E+00_JPRB, 0.29126E+00_JPRB, 0.29802E+00_JPRB, 0.30560E+00_JPRB, 0.31302E+00_JPRB ,&
&  0.32254E+00_JPRB, 0.32638E+00_JPRB, 0.32960E+00_JPRB, 0.33038E+00_JPRB, 0.32937E+00_JPRB, 0.32915E+00_JPRB, 0.32736E+00_JPRB ,&
&  0.32752E+00_JPRB, 0.32744E+00_JPRB, 0.32936E+00_JPRB, 0.33071E+00_JPRB, 0.32906E+00_JPRB, 0.32731E+00_JPRB, 0.32231E+00_JPRB ,&
&  0.31439E+00_JPRB, 0.30809E+00_JPRB, 0.30142E+00_JPRB, 0.29613E+00_JPRB, 0.28810E+00_JPRB, 0.28115E+00_JPRB, 0.27438E+00_JPRB ,&
&  0.26864E+00_JPRB, 0.26443E+00_JPRB, 0.25937E+00_JPRB, 0.25537E+00_JPRB, 0.25344E+00_JPRB, 0.25107E+00_JPRB, 0.25005E+00_JPRB ,&
&  0.25004E+00_JPRB, 0.25054E+00_JPRB, 0.25303E+00_JPRB, 0.25502E+00_JPRB, 0.25811E+00_JPRB, 0.26174E+00_JPRB, 0.26383E+00_JPRB ,&
&  0.26762E+00_JPRB, 0.27090E+00_JPRB, 0.27270E+00_JPRB, 0.27409E+00_JPRB, 0.27310E+00_JPRB, 0.26969E+00_JPRB, 0.26635E+00_JPRB ,&
&  0.26184E+00_JPRB, 0.25875E+00_JPRB, 0.25678E+00_JPRB, 0.25462E+00_JPRB, 0.25350E+00_JPRB, 0.25249E+00_JPRB, 0.25133E+00_JPRB ,&
&  0.25045E+00_JPRB, 0.25004E+00_JPRB, 0.24966E+00_JPRB, 0.25092E+00_JPRB, 0.25222E+00_JPRB, 0.25461E+00_JPRB, 0.25749E+00_JPRB ,&
&  0.26010E+00_JPRB, 0.26415E+00_JPRB, 0.26857E+00_JPRB, 0.27462E+00_JPRB, 0.28592E+00_JPRB, 0.29460E+00_JPRB, 0.31359E+00_JPRB ,&
&  0.32476E+00_JPRB, 0.34632E+00_JPRB, 0.35566E+00_JPRB, 0.34666E+00_JPRB, 0.34637E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.33541E+00_JPRB, 0.33773E+00_JPRB, 0.33840E+00_JPRB, 0.34030E+00_JPRB, 0.34196E+00_JPRB, 0.34240E+00_JPRB, 0.34201E+00_JPRB ,&
&  0.33983E+00_JPRB, 0.33431E+00_JPRB, 0.32980E+00_JPRB, 0.32311E+00_JPRB, 0.31695E+00_JPRB, 0.31012E+00_JPRB, 0.30415E+00_JPRB ,&
&  0.29763E+00_JPRB, 0.29166E+00_JPRB, 0.28549E+00_JPRB, 0.28023E+00_JPRB, 0.27630E+00_JPRB, 0.27305E+00_JPRB, 0.26980E+00_JPRB ,&
&  0.26703E+00_JPRB, 0.26850E+00_JPRB, 0.27147E+00_JPRB, 0.27696E+00_JPRB, 0.28540E+00_JPRB, 0.29292E+00_JPRB, 0.30117E+00_JPRB ,&
&  0.31496E+00_JPRB, 0.31767E+00_JPRB, 0.31783E+00_JPRB, 0.32025E+00_JPRB, 0.31990E+00_JPRB, 0.32053E+00_JPRB, 0.31095E+00_JPRB ,&
&  0.31466E+00_JPRB, 0.31694E+00_JPRB, 0.32553E+00_JPRB, 0.32840E+00_JPRB, 0.32257E+00_JPRB, 0.31853E+00_JPRB, 0.31067E+00_JPRB ,&
&  0.30481E+00_JPRB, 0.29995E+00_JPRB, 0.29671E+00_JPRB, 0.29480E+00_JPRB, 0.28930E+00_JPRB, 0.28523E+00_JPRB, 0.27840E+00_JPRB ,&
&  0.27291E+00_JPRB, 0.26942E+00_JPRB, 0.26501E+00_JPRB, 0.26238E+00_JPRB, 0.26020E+00_JPRB, 0.25802E+00_JPRB, 0.25634E+00_JPRB ,&
&  0.25543E+00_JPRB, 0.25466E+00_JPRB, 0.25544E+00_JPRB, 0.25632E+00_JPRB, 0.25802E+00_JPRB, 0.25940E+00_JPRB, 0.25963E+00_JPRB ,&
&  0.25989E+00_JPRB, 0.26041E+00_JPRB, 0.26063E+00_JPRB, 0.26144E+00_JPRB, 0.26086E+00_JPRB, 0.25974E+00_JPRB, 0.25840E+00_JPRB ,&
&  0.25688E+00_JPRB, 0.25623E+00_JPRB, 0.25511E+00_JPRB, 0.25438E+00_JPRB, 0.25222E+00_JPRB, 0.25079E+00_JPRB, 0.25006E+00_JPRB ,&
&  0.24940E+00_JPRB, 0.24997E+00_JPRB, 0.25019E+00_JPRB, 0.25122E+00_JPRB, 0.25278E+00_JPRB, 0.25413E+00_JPRB, 0.25652E+00_JPRB ,&
&  0.25947E+00_JPRB, 0.26288E+00_JPRB, 0.26882E+00_JPRB, 0.27486E+00_JPRB, 0.28151E+00_JPRB, 0.28911E+00_JPRB, 0.29505E+00_JPRB ,&
&  0.30385E+00_JPRB, 0.31768E+00_JPRB, 0.31970E+00_JPRB, 0.32594E+00_JPRB, 0.33176E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.31844E+00_JPRB, 0.32298E+00_JPRB, 0.32589E+00_JPRB, 0.32780E+00_JPRB, 0.32974E+00_JPRB, 0.32896E+00_JPRB, 0.32721E+00_JPRB ,&
&  0.32271E+00_JPRB, 0.32121E+00_JPRB, 0.31831E+00_JPRB, 0.31504E+00_JPRB, 0.31129E+00_JPRB, 0.30653E+00_JPRB, 0.30245E+00_JPRB ,&
&  0.29780E+00_JPRB, 0.29266E+00_JPRB, 0.28669E+00_JPRB, 0.27993E+00_JPRB, 0.27376E+00_JPRB, 0.26793E+00_JPRB, 0.26494E+00_JPRB ,&
&  0.26363E+00_JPRB, 0.26473E+00_JPRB, 0.26807E+00_JPRB, 0.27193E+00_JPRB, 0.27657E+00_JPRB, 0.28179E+00_JPRB, 0.28827E+00_JPRB ,&
&  0.29368E+00_JPRB, 0.30006E+00_JPRB, 0.30388E+00_JPRB, 0.30541E+00_JPRB, 0.30400E+00_JPRB, 0.30406E+00_JPRB, 0.30321E+00_JPRB ,&
&  0.30207E+00_JPRB, 0.30045E+00_JPRB, 0.30013E+00_JPRB, 0.30073E+00_JPRB, 0.30033E+00_JPRB, 0.29880E+00_JPRB, 0.29630E+00_JPRB ,&
&  0.29380E+00_JPRB, 0.29167E+00_JPRB, 0.28872E+00_JPRB, 0.28620E+00_JPRB, 0.28433E+00_JPRB, 0.28187E+00_JPRB, 0.27848E+00_JPRB ,&
&  0.27549E+00_JPRB, 0.27255E+00_JPRB, 0.26911E+00_JPRB, 0.26629E+00_JPRB, 0.26352E+00_JPRB, 0.26153E+00_JPRB, 0.25974E+00_JPRB ,&
&  0.25904E+00_JPRB, 0.25825E+00_JPRB, 0.25806E+00_JPRB, 0.25798E+00_JPRB, 0.25851E+00_JPRB, 0.25818E+00_JPRB, 0.25766E+00_JPRB ,&
&  0.25704E+00_JPRB, 0.25687E+00_JPRB, 0.25622E+00_JPRB, 0.25598E+00_JPRB, 0.25572E+00_JPRB, 0.25484E+00_JPRB, 0.25435E+00_JPRB ,&
&  0.25428E+00_JPRB, 0.25403E+00_JPRB, 0.25324E+00_JPRB, 0.25306E+00_JPRB, 0.25254E+00_JPRB, 0.25184E+00_JPRB, 0.25214E+00_JPRB ,&
&  0.25194E+00_JPRB, 0.25220E+00_JPRB, 0.25261E+00_JPRB, 0.25320E+00_JPRB, 0.25448E+00_JPRB, 0.25606E+00_JPRB, 0.25815E+00_JPRB ,&
&  0.26138E+00_JPRB, 0.26426E+00_JPRB, 0.26712E+00_JPRB, 0.27038E+00_JPRB, 0.27686E+00_JPRB, 0.27918E+00_JPRB, 0.28360E+00_JPRB ,&
&  0.28736E+00_JPRB, 0.28343E+00_JPRB, 0.30720E+00_JPRB, 0.30853E+00_JPRB, 0.31160E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.27359E+00_JPRB, 0.27878E+00_JPRB, 0.28223E+00_JPRB, 0.28486E+00_JPRB, 0.27968E+00_JPRB, 0.30034E+00_JPRB, 0.29861E+00_JPRB ,&
&  0.29921E+00_JPRB, 0.29447E+00_JPRB, 0.29652E+00_JPRB, 0.29563E+00_JPRB, 0.29341E+00_JPRB, 0.29041E+00_JPRB, 0.28728E+00_JPRB ,&
&  0.28399E+00_JPRB, 0.28000E+00_JPRB, 0.27642E+00_JPRB, 0.27291E+00_JPRB, 0.27001E+00_JPRB, 0.26743E+00_JPRB, 0.26608E+00_JPRB ,&
&  0.26622E+00_JPRB, 0.26792E+00_JPRB, 0.27081E+00_JPRB, 0.27218E+00_JPRB, 0.27368E+00_JPRB, 0.27528E+00_JPRB, 0.27702E+00_JPRB ,&
&  0.27837E+00_JPRB, 0.28030E+00_JPRB, 0.28127E+00_JPRB, 0.28078E+00_JPRB, 0.27996E+00_JPRB, 0.27992E+00_JPRB, 0.28016E+00_JPRB ,&
&  0.28109E+00_JPRB, 0.28171E+00_JPRB, 0.28210E+00_JPRB, 0.28260E+00_JPRB, 0.28310E+00_JPRB, 0.28274E+00_JPRB, 0.28152E+00_JPRB ,&
&  0.28049E+00_JPRB, 0.27964E+00_JPRB, 0.27777E+00_JPRB, 0.27600E+00_JPRB, 0.27433E+00_JPRB, 0.27259E+00_JPRB, 0.27074E+00_JPRB ,&
&  0.26944E+00_JPRB, 0.26814E+00_JPRB, 0.26649E+00_JPRB, 0.26459E+00_JPRB, 0.26285E+00_JPRB, 0.26132E+00_JPRB, 0.26015E+00_JPRB ,&
&  0.25996E+00_JPRB, 0.25934E+00_JPRB, 0.25879E+00_JPRB, 0.25825E+00_JPRB, 0.25813E+00_JPRB, 0.25761E+00_JPRB, 0.25721E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25650E+00_JPRB, 0.25604E+00_JPRB, 0.25557E+00_JPRB, 0.25525E+00_JPRB, 0.25483E+00_JPRB, 0.25440E+00_JPRB ,&
&  0.25411E+00_JPRB, 0.25389E+00_JPRB, 0.25364E+00_JPRB, 0.25314E+00_JPRB, 0.25265E+00_JPRB, 0.25207E+00_JPRB, 0.25203E+00_JPRB ,&
&  0.25217E+00_JPRB, 0.25246E+00_JPRB, 0.25299E+00_JPRB, 0.25316E+00_JPRB, 0.25415E+00_JPRB, 0.25510E+00_JPRB, 0.25623E+00_JPRB ,&
&  0.25741E+00_JPRB, 0.25923E+00_JPRB, 0.26125E+00_JPRB, 0.26345E+00_JPRB, 0.26703E+00_JPRB, 0.26800E+00_JPRB, 0.26874E+00_JPRB ,&
&  0.26970E+00_JPRB, 0.27369E+00_JPRB, 0.27429E+00_JPRB, 0.27367E+00_JPRB, 0.27397E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.26325E+00_JPRB, 0.26350E+00_JPRB, 0.26343E+00_JPRB, 0.26316E+00_JPRB, 0.26075E+00_JPRB, 0.25970E+00_JPRB, 0.25878E+00_JPRB ,&
&  0.25810E+00_JPRB, 0.25776E+00_JPRB, 0.25823E+00_JPRB, 0.25885E+00_JPRB, 0.25949E+00_JPRB, 0.26016E+00_JPRB, 0.26076E+00_JPRB ,&
&  0.26107E+00_JPRB, 0.26142E+00_JPRB, 0.26018E+00_JPRB, 0.25958E+00_JPRB, 0.25939E+00_JPRB, 0.25944E+00_JPRB, 0.25942E+00_JPRB ,&
&  0.25952E+00_JPRB, 0.25959E+00_JPRB, 0.25955E+00_JPRB, 0.25996E+00_JPRB, 0.26064E+00_JPRB, 0.26120E+00_JPRB, 0.26178E+00_JPRB ,&
&  0.26236E+00_JPRB, 0.26320E+00_JPRB, 0.26398E+00_JPRB, 0.26472E+00_JPRB, 0.26573E+00_JPRB, 0.26675E+00_JPRB, 0.26738E+00_JPRB ,&
&  0.26788E+00_JPRB, 0.26808E+00_JPRB, 0.26834E+00_JPRB, 0.26854E+00_JPRB, 0.26887E+00_JPRB, 0.26927E+00_JPRB, 0.26937E+00_JPRB ,&
&  0.26932E+00_JPRB, 0.26918E+00_JPRB, 0.26894E+00_JPRB, 0.26833E+00_JPRB, 0.26768E+00_JPRB, 0.26705E+00_JPRB, 0.26642E+00_JPRB ,&
&  0.26588E+00_JPRB, 0.26525E+00_JPRB, 0.26456E+00_JPRB, 0.26382E+00_JPRB, 0.26305E+00_JPRB, 0.26231E+00_JPRB, 0.26155E+00_JPRB ,&
&  0.26077E+00_JPRB, 0.26028E+00_JPRB, 0.25974E+00_JPRB, 0.25927E+00_JPRB, 0.25888E+00_JPRB, 0.25849E+00_JPRB, 0.25811E+00_JPRB ,&
&  0.25774E+00_JPRB, 0.25728E+00_JPRB, 0.25687E+00_JPRB, 0.25642E+00_JPRB, 0.25602E+00_JPRB, 0.25557E+00_JPRB, 0.25520E+00_JPRB ,&
&  0.25484E+00_JPRB, 0.25453E+00_JPRB, 0.25427E+00_JPRB, 0.25415E+00_JPRB, 0.25408E+00_JPRB, 0.25411E+00_JPRB, 0.25415E+00_JPRB ,&
&  0.25422E+00_JPRB, 0.25425E+00_JPRB, 0.25428E+00_JPRB, 0.25413E+00_JPRB, 0.25437E+00_JPRB, 0.25463E+00_JPRB, 0.25500E+00_JPRB ,&
&  0.25535E+00_JPRB, 0.25592E+00_JPRB, 0.25651E+00_JPRB, 0.25718E+00_JPRB, 0.25823E+00_JPRB, 0.25861E+00_JPRB, 0.25893E+00_JPRB ,&
&  0.25925E+00_JPRB, 0.25994E+00_JPRB, 0.26050E+00_JPRB, 0.26080E+00_JPRB, 0.26147E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB, 0.25847E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB, 0.70714E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.68526E+00_JPRB, 0.68192E+00_JPRB, 0.67857E+00_JPRB, 0.67505E+00_JPRB, 0.67128E+00_JPRB, 0.66696E+00_JPRB, 0.66223E+00_JPRB ,&
&  0.65795E+00_JPRB, 0.65404E+00_JPRB, 0.65052E+00_JPRB, 0.64734E+00_JPRB, 0.64463E+00_JPRB, 0.64249E+00_JPRB, 0.64164E+00_JPRB ,&
&  0.64149E+00_JPRB, 0.64144E+00_JPRB, 0.64163E+00_JPRB, 0.64183E+00_JPRB, 0.64169E+00_JPRB, 0.64134E+00_JPRB, 0.64087E+00_JPRB ,&
&  0.64063E+00_JPRB, 0.64049E+00_JPRB, 0.64041E+00_JPRB, 0.64076E+00_JPRB, 0.64128E+00_JPRB, 0.64182E+00_JPRB, 0.64228E+00_JPRB ,&
&  0.64325E+00_JPRB, 0.64445E+00_JPRB, 0.64610E+00_JPRB, 0.64816E+00_JPRB, 0.65233E+00_JPRB, 0.65680E+00_JPRB, 0.66141E+00_JPRB ,&
&  0.66575E+00_JPRB, 0.67008E+00_JPRB, 0.67373E+00_JPRB, 0.67690E+00_JPRB, 0.67982E+00_JPRB, 0.68350E+00_JPRB, 0.68763E+00_JPRB ,&
&  0.69211E+00_JPRB, 0.69705E+00_JPRB, 0.70280E+00_JPRB, 0.70847E+00_JPRB, 0.71364E+00_JPRB, 0.71892E+00_JPRB, 0.72458E+00_JPRB ,&
&  0.73007E+00_JPRB, 0.73559E+00_JPRB, 0.74110E+00_JPRB, 0.74686E+00_JPRB, 0.75241E+00_JPRB, 0.75896E+00_JPRB, 0.76671E+00_JPRB ,&
&  0.77555E+00_JPRB, 0.78243E+00_JPRB, 0.79038E+00_JPRB, 0.79892E+00_JPRB, 0.81177E+00_JPRB, 0.81812E+00_JPRB, 0.82422E+00_JPRB ,&
&  0.82955E+00_JPRB, 0.83823E+00_JPRB, 0.84366E+00_JPRB, 0.84905E+00_JPRB, 0.85408E+00_JPRB, 0.85855E+00_JPRB, 0.86155E+00_JPRB ,&
&  0.86417E+00_JPRB, 0.86651E+00_JPRB, 0.86701E+00_JPRB, 0.86659E+00_JPRB, 0.86501E+00_JPRB, 0.86298E+00_JPRB, 0.85775E+00_JPRB ,&
&  0.85295E+00_JPRB, 0.84835E+00_JPRB, 0.84411E+00_JPRB, 0.83795E+00_JPRB, 0.83162E+00_JPRB, 0.82509E+00_JPRB, 0.81836E+00_JPRB ,&
&  0.80969E+00_JPRB, 0.80078E+00_JPRB, 0.79123E+00_JPRB, 0.78102E+00_JPRB, 0.77112E+00_JPRB, 0.76076E+00_JPRB, 0.74946E+00_JPRB ,&
&  0.73685E+00_JPRB, 0.72500E+00_JPRB, 0.71171E+00_JPRB, 0.69855E+00_JPRB, 0.68907E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.66854E+00_JPRB, 0.65662E+00_JPRB, 0.65163E+00_JPRB, 0.64855E+00_JPRB, 0.64616E+00_JPRB, 0.64308E+00_JPRB, 0.63941E+00_JPRB ,&
&  0.63661E+00_JPRB, 0.63333E+00_JPRB, 0.63078E+00_JPRB, 0.63024E+00_JPRB, 0.62884E+00_JPRB, 0.62805E+00_JPRB, 0.62724E+00_JPRB ,&
&  0.62678E+00_JPRB, 0.62617E+00_JPRB, 0.62618E+00_JPRB, 0.62506E+00_JPRB, 0.62335E+00_JPRB, 0.62091E+00_JPRB, 0.61902E+00_JPRB ,&
&  0.61727E+00_JPRB, 0.61606E+00_JPRB, 0.61425E+00_JPRB, 0.61285E+00_JPRB, 0.61341E+00_JPRB, 0.61340E+00_JPRB, 0.61269E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61623E+00_JPRB, 0.61562E+00_JPRB, 0.61336E+00_JPRB, 0.61087E+00_JPRB, 0.61002E+00_JPRB, 0.61236E+00_JPRB ,&
&  0.61634E+00_JPRB, 0.62715E+00_JPRB, 0.63568E+00_JPRB, 0.64378E+00_JPRB, 0.65170E+00_JPRB, 0.66140E+00_JPRB, 0.67294E+00_JPRB ,&
&  0.68482E+00_JPRB, 0.69372E+00_JPRB, 0.70357E+00_JPRB, 0.71583E+00_JPRB, 0.72942E+00_JPRB, 0.74397E+00_JPRB, 0.76023E+00_JPRB ,&
&  0.77368E+00_JPRB, 0.78755E+00_JPRB, 0.80372E+00_JPRB, 0.82166E+00_JPRB, 0.83677E+00_JPRB, 0.85722E+00_JPRB, 0.88216E+00_JPRB ,&
&  0.91434E+00_JPRB, 0.93015E+00_JPRB, 0.94771E+00_JPRB, 0.96508E+00_JPRB, 0.98764E+00_JPRB, 0.10004E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.10274E+01_JPRB, 0.10249E+01_JPRB, 0.10205E+01_JPRB, 0.10172E+01_JPRB, 0.10127E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.10028E+01_JPRB, 0.99590E+00_JPRB, 0.98912E+00_JPRB, 0.98043E+00_JPRB, 0.97182E+00_JPRB, 0.95488E+00_JPRB ,&
&  0.93947E+00_JPRB, 0.92780E+00_JPRB, 0.92020E+00_JPRB, 0.90950E+00_JPRB, 0.89891E+00_JPRB, 0.88922E+00_JPRB, 0.87998E+00_JPRB ,&
&  0.87064E+00_JPRB, 0.85964E+00_JPRB, 0.84786E+00_JPRB, 0.83516E+00_JPRB, 0.82459E+00_JPRB, 0.80951E+00_JPRB, 0.79230E+00_JPRB ,&
&  0.77207E+00_JPRB, 0.75438E+00_JPRB, 0.73328E+00_JPRB, 0.70833E+00_JPRB, 0.68371E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.68655E+00_JPRB, 0.65833E+00_JPRB, 0.64370E+00_JPRB, 0.63987E+00_JPRB, 0.63801E+00_JPRB, 0.63613E+00_JPRB, 0.63429E+00_JPRB ,&
&  0.63138E+00_JPRB, 0.62958E+00_JPRB, 0.63065E+00_JPRB, 0.63199E+00_JPRB, 0.63209E+00_JPRB, 0.63193E+00_JPRB, 0.63052E+00_JPRB ,&
&  0.62860E+00_JPRB, 0.62498E+00_JPRB, 0.62138E+00_JPRB, 0.61532E+00_JPRB, 0.60898E+00_JPRB, 0.60104E+00_JPRB, 0.59037E+00_JPRB ,&
&  0.58139E+00_JPRB, 0.57328E+00_JPRB, 0.56264E+00_JPRB, 0.55931E+00_JPRB, 0.55797E+00_JPRB, 0.53448E+00_JPRB, 0.52959E+00_JPRB ,&
&  0.53732E+00_JPRB, 0.53899E+00_JPRB, 0.53975E+00_JPRB, 0.54050E+00_JPRB, 0.53794E+00_JPRB, 0.52808E+00_JPRB, 0.52526E+00_JPRB ,&
&  0.50799E+00_JPRB, 0.52329E+00_JPRB, 0.53413E+00_JPRB, 0.54376E+00_JPRB, 0.55364E+00_JPRB, 0.56786E+00_JPRB, 0.59557E+00_JPRB ,&
&  0.64684E+00_JPRB, 0.66941E+00_JPRB, 0.69503E+00_JPRB, 0.71938E+00_JPRB, 0.75152E+00_JPRB, 0.78554E+00_JPRB, 0.81703E+00_JPRB ,&
&  0.84407E+00_JPRB, 0.87316E+00_JPRB, 0.90683E+00_JPRB, 0.95109E+00_JPRB, 0.98624E+00_JPRB, 0.10187E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.10691E+01_JPRB, 0.10889E+01_JPRB, 0.11030E+01_JPRB, 0.11115E+01_JPRB, 0.11209E+01_JPRB, 0.11248E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.11315E+01_JPRB, 0.11219E+01_JPRB, 0.11046E+01_JPRB, 0.10925E+01_JPRB, 0.10874E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.10591E+01_JPRB, 0.10462E+01_JPRB, 0.10304E+01_JPRB, 0.10110E+01_JPRB, 0.99413E+00_JPRB, 0.97353E+00_JPRB ,&
&  0.95632E+00_JPRB, 0.94263E+00_JPRB, 0.93149E+00_JPRB, 0.91782E+00_JPRB, 0.90362E+00_JPRB, 0.88966E+00_JPRB, 0.87405E+00_JPRB ,&
&  0.85979E+00_JPRB, 0.84724E+00_JPRB, 0.83704E+00_JPRB, 0.82591E+00_JPRB, 0.81831E+00_JPRB, 0.80758E+00_JPRB, 0.79788E+00_JPRB ,&
&  0.78672E+00_JPRB, 0.77695E+00_JPRB, 0.76131E+00_JPRB, 0.73965E+00_JPRB, 0.71107E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.72131E+00_JPRB, 0.68979E+00_JPRB, 0.65732E+00_JPRB, 0.63929E+00_JPRB, 0.63547E+00_JPRB, 0.63257E+00_JPRB, 0.63048E+00_JPRB ,&
&  0.62957E+00_JPRB, 0.63237E+00_JPRB, 0.63529E+00_JPRB, 0.63718E+00_JPRB, 0.63672E+00_JPRB, 0.63431E+00_JPRB, 0.63035E+00_JPRB ,&
&  0.62438E+00_JPRB, 0.61701E+00_JPRB, 0.61045E+00_JPRB, 0.60406E+00_JPRB, 0.59708E+00_JPRB, 0.58958E+00_JPRB, 0.57856E+00_JPRB ,&
&  0.56734E+00_JPRB, 0.55681E+00_JPRB, 0.52123E+00_JPRB, 0.51943E+00_JPRB, 0.51273E+00_JPRB, 0.51127E+00_JPRB, 0.51507E+00_JPRB ,&
&  0.51463E+00_JPRB, 0.50534E+00_JPRB, 0.49449E+00_JPRB, 0.47859E+00_JPRB, 0.46636E+00_JPRB, 0.45323E+00_JPRB, 0.44681E+00_JPRB ,&
&  0.44495E+00_JPRB, 0.44844E+00_JPRB, 0.45762E+00_JPRB, 0.46641E+00_JPRB, 0.47682E+00_JPRB, 0.48726E+00_JPRB, 0.50545E+00_JPRB ,&
&  0.55216E+00_JPRB, 0.61694E+00_JPRB, 0.69537E+00_JPRB, 0.73635E+00_JPRB, 0.78498E+00_JPRB, 0.83482E+00_JPRB, 0.87249E+00_JPRB ,&
&  0.90705E+00_JPRB, 0.94337E+00_JPRB, 0.98143E+00_JPRB, 0.10237E+01_JPRB, 0.10633E+01_JPRB, 0.10946E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.11946E+01_JPRB, 0.12040E+01_JPRB, 0.12084E+01_JPRB, 0.12070E+01_JPRB, 0.11952E+01_JPRB, 0.11830E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.11614E+01_JPRB, 0.11455E+01_JPRB, 0.11124E+01_JPRB, 0.10942E+01_JPRB, 0.10875E+01_JPRB, 0.10682E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.10218E+01_JPRB, 0.99822E+00_JPRB, 0.96571E+00_JPRB, 0.94032E+00_JPRB, 0.93186E+00_JPRB, 0.92633E+00_JPRB ,&
&  0.92165E+00_JPRB, 0.91544E+00_JPRB, 0.90890E+00_JPRB, 0.89645E+00_JPRB, 0.87569E+00_JPRB, 0.85436E+00_JPRB, 0.83616E+00_JPRB ,&
&  0.81943E+00_JPRB, 0.80244E+00_JPRB, 0.78699E+00_JPRB, 0.77143E+00_JPRB, 0.76208E+00_JPRB, 0.75506E+00_JPRB, 0.75390E+00_JPRB ,&
&  0.75586E+00_JPRB, 0.75954E+00_JPRB, 0.76034E+00_JPRB, 0.75712E+00_JPRB, 0.74326E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.79875E+00_JPRB, 0.78604E+00_JPRB, 0.75985E+00_JPRB, 0.72403E+00_JPRB, 0.69653E+00_JPRB, 0.67757E+00_JPRB, 0.66832E+00_JPRB ,&
&  0.66271E+00_JPRB, 0.66007E+00_JPRB, 0.65734E+00_JPRB, 0.65408E+00_JPRB, 0.64876E+00_JPRB, 0.64183E+00_JPRB, 0.63483E+00_JPRB ,&
&  0.62291E+00_JPRB, 0.61183E+00_JPRB, 0.60027E+00_JPRB, 0.59327E+00_JPRB, 0.58746E+00_JPRB, 0.58279E+00_JPRB, 0.57873E+00_JPRB ,&
&  0.57403E+00_JPRB, 0.56791E+00_JPRB, 0.54837E+00_JPRB, 0.54084E+00_JPRB, 0.53301E+00_JPRB, 0.53321E+00_JPRB, 0.53047E+00_JPRB ,&
&  0.54351E+00_JPRB, 0.49097E+00_JPRB, 0.48300E+00_JPRB, 0.46781E+00_JPRB, 0.45670E+00_JPRB, 0.44373E+00_JPRB, 0.43090E+00_JPRB ,&
&  0.42368E+00_JPRB, 0.41635E+00_JPRB, 0.41523E+00_JPRB, 0.42429E+00_JPRB, 0.43251E+00_JPRB, 0.44771E+00_JPRB, 0.46434E+00_JPRB ,&
&  0.49848E+00_JPRB, 0.62901E+00_JPRB, 0.69254E+00_JPRB, 0.74020E+00_JPRB, 0.78798E+00_JPRB, 0.83442E+00_JPRB, 0.86108E+00_JPRB ,&
&  0.88870E+00_JPRB, 0.92445E+00_JPRB, 0.95251E+00_JPRB, 0.96228E+00_JPRB, 0.10366E+01_JPRB, 0.10906E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.11956E+01_JPRB, 0.12401E+01_JPRB, 0.12771E+01_JPRB, 0.13037E+01_JPRB, 0.13268E+01_JPRB, 0.13227E+01_JPRB, 0.13030E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.12747E+01_JPRB, 0.12656E+01_JPRB, 0.12214E+01_JPRB, 0.11912E+01_JPRB, 0.11728E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.10821E+01_JPRB, 0.10400E+01_JPRB, 0.99817E+00_JPRB, 0.95484E+00_JPRB, 0.91662E+00_JPRB, 0.89541E+00_JPRB, 0.88867E+00_JPRB ,&
&  0.88957E+00_JPRB, 0.88646E+00_JPRB, 0.87480E+00_JPRB, 0.85418E+00_JPRB, 0.83058E+00_JPRB, 0.81495E+00_JPRB, 0.79500E+00_JPRB ,&
&  0.77430E+00_JPRB, 0.74963E+00_JPRB, 0.73369E+00_JPRB, 0.72256E+00_JPRB, 0.71901E+00_JPRB, 0.72151E+00_JPRB, 0.72871E+00_JPRB ,&
&  0.73835E+00_JPRB, 0.75148E+00_JPRB, 0.76814E+00_JPRB, 0.78419E+00_JPRB, 0.79559E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.78705E+00_JPRB, 0.79601E+00_JPRB, 0.79805E+00_JPRB, 0.79114E+00_JPRB, 0.77723E+00_JPRB, 0.75859E+00_JPRB, 0.73946E+00_JPRB ,&
&  0.72192E+00_JPRB, 0.70815E+00_JPRB, 0.69517E+00_JPRB, 0.68525E+00_JPRB, 0.67351E+00_JPRB, 0.66561E+00_JPRB, 0.65650E+00_JPRB ,&
&  0.65121E+00_JPRB, 0.64899E+00_JPRB, 0.64146E+00_JPRB, 0.63256E+00_JPRB, 0.61629E+00_JPRB, 0.60170E+00_JPRB, 0.59037E+00_JPRB ,&
&  0.58019E+00_JPRB, 0.57173E+00_JPRB, 0.55279E+00_JPRB, 0.53909E+00_JPRB, 0.53204E+00_JPRB, 0.52670E+00_JPRB, 0.53337E+00_JPRB ,&
&  0.53283E+00_JPRB, 0.53863E+00_JPRB, 0.54365E+00_JPRB, 0.53723E+00_JPRB, 0.52980E+00_JPRB, 0.43417E+00_JPRB, 0.40696E+00_JPRB ,&
&  0.39919E+00_JPRB, 0.39273E+00_JPRB, 0.39202E+00_JPRB, 0.39859E+00_JPRB, 0.40909E+00_JPRB, 0.43640E+00_JPRB, 0.45666E+00_JPRB ,&
&  0.47451E+00_JPRB, 0.61192E+00_JPRB, 0.66821E+00_JPRB, 0.71899E+00_JPRB, 0.75620E+00_JPRB, 0.78750E+00_JPRB, 0.80864E+00_JPRB ,&
&  0.82842E+00_JPRB, 0.85630E+00_JPRB, 0.87180E+00_JPRB, 0.90475E+00_JPRB, 0.96874E+00_JPRB, 0.10781E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.13235E+01_JPRB, 0.13595E+01_JPRB, 0.13666E+01_JPRB, 0.13951E+01_JPRB, 0.14117E+01_JPRB, 0.14115E+01_JPRB ,&
&  0.14186E+01_JPRB, 0.13954E+01_JPRB, 0.14005E+01_JPRB, 0.13766E+01_JPRB, 0.13557E+01_JPRB, 0.13332E+01_JPRB, 0.12536E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.10999E+01_JPRB, 0.10438E+01_JPRB, 0.97295E+00_JPRB, 0.92055E+00_JPRB, 0.88301E+00_JPRB, 0.86982E+00_JPRB ,&
&  0.86276E+00_JPRB, 0.86548E+00_JPRB, 0.84658E+00_JPRB, 0.82129E+00_JPRB, 0.80285E+00_JPRB, 0.79032E+00_JPRB, 0.76725E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.72724E+00_JPRB, 0.71641E+00_JPRB, 0.70594E+00_JPRB, 0.69920E+00_JPRB, 0.69763E+00_JPRB, 0.70061E+00_JPRB ,&
&  0.70895E+00_JPRB, 0.72108E+00_JPRB, 0.73861E+00_JPRB, 0.75778E+00_JPRB, 0.77821E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.77016E+00_JPRB, 0.78575E+00_JPRB, 0.79345E+00_JPRB, 0.79073E+00_JPRB, 0.77605E+00_JPRB, 0.75653E+00_JPRB, 0.73931E+00_JPRB ,&
&  0.72311E+00_JPRB, 0.70792E+00_JPRB, 0.69375E+00_JPRB, 0.68301E+00_JPRB, 0.67355E+00_JPRB, 0.66708E+00_JPRB, 0.66231E+00_JPRB ,&
&  0.65963E+00_JPRB, 0.64411E+00_JPRB, 0.63815E+00_JPRB, 0.63793E+00_JPRB, 0.63983E+00_JPRB, 0.63757E+00_JPRB, 0.63150E+00_JPRB ,&
&  0.61083E+00_JPRB, 0.58251E+00_JPRB, 0.55600E+00_JPRB, 0.53343E+00_JPRB, 0.51570E+00_JPRB, 0.50887E+00_JPRB, 0.51379E+00_JPRB ,&
&  0.51979E+00_JPRB, 0.51571E+00_JPRB, 0.51764E+00_JPRB, 0.53646E+00_JPRB, 0.54596E+00_JPRB, 0.48579E+00_JPRB, 0.40655E+00_JPRB ,&
&  0.39546E+00_JPRB, 0.38013E+00_JPRB, 0.37977E+00_JPRB, 0.38649E+00_JPRB, 0.39923E+00_JPRB, 0.41859E+00_JPRB, 0.45161E+00_JPRB ,&
&  0.49649E+00_JPRB, 0.59551E+00_JPRB, 0.64701E+00_JPRB, 0.68748E+00_JPRB, 0.72490E+00_JPRB, 0.75752E+00_JPRB, 0.77442E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.80411E+00_JPRB, 0.84170E+00_JPRB, 0.88582E+00_JPRB, 0.96218E+00_JPRB, 0.11180E+01_JPRB, 0.12471E+01_JPRB ,&
&  0.14237E+01_JPRB, 0.15076E+01_JPRB, 0.14808E+01_JPRB, 0.13931E+01_JPRB, 0.14092E+01_JPRB, 0.14691E+01_JPRB, 0.14757E+01_JPRB ,&
&  0.15285E+01_JPRB, 0.15989E+01_JPRB, 0.15855E+01_JPRB, 0.15162E+01_JPRB, 0.14723E+01_JPRB, 0.14487E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.11747E+01_JPRB, 0.11850E+01_JPRB, 0.10652E+01_JPRB, 0.98346E+00_JPRB, 0.90636E+00_JPRB, 0.86345E+00_JPRB ,&
&  0.83649E+00_JPRB, 0.81438E+00_JPRB, 0.78459E+00_JPRB, 0.76058E+00_JPRB, 0.74747E+00_JPRB, 0.73889E+00_JPRB, 0.72853E+00_JPRB ,&
&  0.71755E+00_JPRB, 0.70782E+00_JPRB, 0.69820E+00_JPRB, 0.69131E+00_JPRB, 0.68438E+00_JPRB, 0.68116E+00_JPRB, 0.68162E+00_JPRB ,&
&  0.68730E+00_JPRB, 0.69861E+00_JPRB, 0.71497E+00_JPRB, 0.73214E+00_JPRB, 0.75317E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.77356E+00_JPRB, 0.78497E+00_JPRB, 0.77667E+00_JPRB, 0.75120E+00_JPRB, 0.71582E+00_JPRB, 0.67279E+00_JPRB, 0.64355E+00_JPRB ,&
&  0.62775E+00_JPRB, 0.62126E+00_JPRB, 0.61645E+00_JPRB, 0.60972E+00_JPRB, 0.60727E+00_JPRB, 0.60683E+00_JPRB, 0.60827E+00_JPRB ,&
&  0.61193E+00_JPRB, 0.61636E+00_JPRB, 0.61859E+00_JPRB, 0.61383E+00_JPRB, 0.61561E+00_JPRB, 0.61797E+00_JPRB, 0.62122E+00_JPRB ,&
&  0.61581E+00_JPRB, 0.59372E+00_JPRB, 0.56561E+00_JPRB, 0.54567E+00_JPRB, 0.52540E+00_JPRB, 0.52111E+00_JPRB, 0.50999E+00_JPRB ,&
&  0.48638E+00_JPRB, 0.48178E+00_JPRB, 0.50609E+00_JPRB, 0.53607E+00_JPRB, 0.53268E+00_JPRB, 0.55372E+00_JPRB, 0.44013E+00_JPRB ,&
&  0.40399E+00_JPRB, 0.38544E+00_JPRB, 0.38167E+00_JPRB, 0.39204E+00_JPRB, 0.41921E+00_JPRB, 0.45348E+00_JPRB, 0.48653E+00_JPRB ,&
&  0.53564E+00_JPRB, 0.59051E+00_JPRB, 0.62141E+00_JPRB, 0.65253E+00_JPRB, 0.68992E+00_JPRB, 0.72832E+00_JPRB, 0.75696E+00_JPRB ,&
&  0.77581E+00_JPRB, 0.80858E+00_JPRB, 0.86559E+00_JPRB, 0.95549E+00_JPRB, 0.11539E+01_JPRB, 0.11347E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.15454E+01_JPRB, 0.17322E+01_JPRB, 0.15923E+01_JPRB, 0.14653E+01_JPRB, 0.14178E+01_JPRB, 0.14496E+01_JPRB, 0.15116E+01_JPRB ,&
&  0.16528E+01_JPRB, 0.17829E+01_JPRB, 0.17865E+01_JPRB, 0.16264E+01_JPRB, 0.15456E+01_JPRB, 0.15022E+01_JPRB, 0.13558E+01_JPRB ,&
&  0.11968E+01_JPRB, 0.12305E+01_JPRB, 0.12982E+01_JPRB, 0.10958E+01_JPRB, 0.95703E+00_JPRB, 0.86964E+00_JPRB, 0.83332E+00_JPRB ,&
&  0.81215E+00_JPRB, 0.79329E+00_JPRB, 0.77019E+00_JPRB, 0.74787E+00_JPRB, 0.71927E+00_JPRB, 0.69505E+00_JPRB, 0.67405E+00_JPRB ,&
&  0.66023E+00_JPRB, 0.65053E+00_JPRB, 0.64911E+00_JPRB, 0.65202E+00_JPRB, 0.65348E+00_JPRB, 0.65491E+00_JPRB, 0.66359E+00_JPRB ,&
&  0.67766E+00_JPRB, 0.68495E+00_JPRB, 0.69966E+00_JPRB, 0.71908E+00_JPRB, 0.74611E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.71819E+00_JPRB, 0.71983E+00_JPRB, 0.70506E+00_JPRB, 0.67530E+00_JPRB, 0.64291E+00_JPRB, 0.60002E+00_JPRB, 0.56782E+00_JPRB ,&
&  0.54967E+00_JPRB, 0.54052E+00_JPRB, 0.53679E+00_JPRB, 0.53586E+00_JPRB, 0.53810E+00_JPRB, 0.54367E+00_JPRB, 0.54438E+00_JPRB ,&
&  0.54643E+00_JPRB, 0.54763E+00_JPRB, 0.55135E+00_JPRB, 0.55833E+00_JPRB, 0.56478E+00_JPRB, 0.57119E+00_JPRB, 0.57798E+00_JPRB ,&
&  0.58450E+00_JPRB, 0.57891E+00_JPRB, 0.55813E+00_JPRB, 0.54215E+00_JPRB, 0.54553E+00_JPRB, 0.56322E+00_JPRB, 0.54842E+00_JPRB ,&
&  0.50695E+00_JPRB, 0.48356E+00_JPRB, 0.48379E+00_JPRB, 0.54233E+00_JPRB, 0.58017E+00_JPRB, 0.59724E+00_JPRB, 0.48371E+00_JPRB ,&
&  0.42389E+00_JPRB, 0.40109E+00_JPRB, 0.40361E+00_JPRB, 0.43116E+00_JPRB, 0.48026E+00_JPRB, 0.54199E+00_JPRB, 0.60312E+00_JPRB ,&
&  0.64931E+00_JPRB, 0.68864E+00_JPRB, 0.69985E+00_JPRB, 0.72273E+00_JPRB, 0.76109E+00_JPRB, 0.79635E+00_JPRB, 0.83496E+00_JPRB ,&
&  0.85963E+00_JPRB, 0.89966E+00_JPRB, 0.10176E+01_JPRB, 0.89519E+00_JPRB, 0.94877E+00_JPRB, 0.10151E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.13686E+01_JPRB, 0.14450E+01_JPRB, 0.14557E+01_JPRB, 0.15588E+01_JPRB, 0.14660E+01_JPRB, 0.14006E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.16588E+01_JPRB, 0.18747E+01_JPRB, 0.18455E+01_JPRB, 0.16910E+01_JPRB, 0.16380E+01_JPRB, 0.16093E+01_JPRB, 0.14331E+01_JPRB ,&
&  0.12118E+01_JPRB, 0.10889E+01_JPRB, 0.98373E+00_JPRB, 0.85518E+00_JPRB, 0.80417E+00_JPRB, 0.77483E+00_JPRB, 0.75758E+00_JPRB ,&
&  0.74634E+00_JPRB, 0.73010E+00_JPRB, 0.71769E+00_JPRB, 0.67243E+00_JPRB, 0.62927E+00_JPRB, 0.58687E+00_JPRB, 0.55933E+00_JPRB ,&
&  0.54332E+00_JPRB, 0.54233E+00_JPRB, 0.55139E+00_JPRB, 0.57251E+00_JPRB, 0.61119E+00_JPRB, 0.64624E+00_JPRB, 0.66974E+00_JPRB ,&
&  0.67508E+00_JPRB, 0.66055E+00_JPRB, 0.65794E+00_JPRB, 0.66593E+00_JPRB, 0.69840E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.69069E+00_JPRB, 0.65492E+00_JPRB, 0.63275E+00_JPRB, 0.63696E+00_JPRB, 0.59630E+00_JPRB, 0.55651E+00_JPRB, 0.53413E+00_JPRB ,&
&  0.53480E+00_JPRB, 0.53135E+00_JPRB, 0.53252E+00_JPRB, 0.51126E+00_JPRB, 0.49237E+00_JPRB, 0.48189E+00_JPRB, 0.47773E+00_JPRB ,&
&  0.48765E+00_JPRB, 0.50028E+00_JPRB, 0.51027E+00_JPRB, 0.50933E+00_JPRB, 0.50757E+00_JPRB, 0.51128E+00_JPRB, 0.52182E+00_JPRB ,&
&  0.53642E+00_JPRB, 0.54762E+00_JPRB, 0.54573E+00_JPRB, 0.54260E+00_JPRB, 0.56499E+00_JPRB, 0.59179E+00_JPRB, 0.56625E+00_JPRB ,&
&  0.53523E+00_JPRB, 0.51060E+00_JPRB, 0.50571E+00_JPRB, 0.54843E+00_JPRB, 0.61579E+00_JPRB, 0.64538E+00_JPRB, 0.52981E+00_JPRB ,&
&  0.45104E+00_JPRB, 0.44021E+00_JPRB, 0.48161E+00_JPRB, 0.60063E+00_JPRB, 0.64314E+00_JPRB, 0.69322E+00_JPRB, 0.76249E+00_JPRB ,&
&  0.71434E+00_JPRB, 0.71799E+00_JPRB, 0.74389E+00_JPRB, 0.88445E+00_JPRB, 0.92137E+00_JPRB, 0.93693E+00_JPRB, 0.94943E+00_JPRB ,&
&  0.95798E+00_JPRB, 0.99374E+00_JPRB, 0.11350E+01_JPRB, 0.97015E+00_JPRB, 0.10239E+01_JPRB, 0.12431E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.13490E+01_JPRB, 0.13892E+01_JPRB, 0.15574E+01_JPRB, 0.15235E+01_JPRB, 0.14075E+01_JPRB, 0.13727E+01_JPRB, 0.14875E+01_JPRB ,&
&  0.17829E+01_JPRB, 0.20357E+01_JPRB, 0.19915E+01_JPRB, 0.19192E+01_JPRB, 0.18753E+01_JPRB, 0.17928E+01_JPRB, 0.15947E+01_JPRB ,&
&  0.13095E+01_JPRB, 0.10953E+01_JPRB, 0.93710E+00_JPRB, 0.80001E+00_JPRB, 0.73536E+00_JPRB, 0.71090E+00_JPRB, 0.68508E+00_JPRB ,&
&  0.67733E+00_JPRB, 0.66783E+00_JPRB, 0.65710E+00_JPRB, 0.60884E+00_JPRB, 0.56711E+00_JPRB, 0.53892E+00_JPRB, 0.51869E+00_JPRB ,&
&  0.51052E+00_JPRB, 0.49697E+00_JPRB, 0.48763E+00_JPRB, 0.50367E+00_JPRB, 0.55297E+00_JPRB, 0.61335E+00_JPRB, 0.64105E+00_JPRB ,&
&  0.63014E+00_JPRB, 0.61786E+00_JPRB, 0.64353E+00_JPRB, 0.68570E+00_JPRB, 0.70468E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.70745E+00_JPRB, 0.69590E+00_JPRB, 0.68872E+00_JPRB, 0.67732E+00_JPRB, 0.65838E+00_JPRB, 0.60787E+00_JPRB, 0.57301E+00_JPRB ,&
&  0.58133E+00_JPRB, 0.55632E+00_JPRB, 0.54840E+00_JPRB, 0.51905E+00_JPRB, 0.49598E+00_JPRB, 0.47138E+00_JPRB, 0.45771E+00_JPRB ,&
&  0.44682E+00_JPRB, 0.45624E+00_JPRB, 0.47881E+00_JPRB, 0.48544E+00_JPRB, 0.48607E+00_JPRB, 0.48598E+00_JPRB, 0.49603E+00_JPRB ,&
&  0.50963E+00_JPRB, 0.52884E+00_JPRB, 0.54180E+00_JPRB, 0.55211E+00_JPRB, 0.57256E+00_JPRB, 0.59983E+00_JPRB, 0.59971E+00_JPRB ,&
&  0.59618E+00_JPRB, 0.57433E+00_JPRB, 0.55412E+00_JPRB, 0.58384E+00_JPRB, 0.64818E+00_JPRB, 0.70958E+00_JPRB, 0.68376E+00_JPRB ,&
&  0.52168E+00_JPRB, 0.52693E+00_JPRB, 0.69508E+00_JPRB, 0.75007E+00_JPRB, 0.80661E+00_JPRB, 0.84340E+00_JPRB, 0.89083E+00_JPRB ,&
&  0.93341E+00_JPRB, 0.97667E+00_JPRB, 0.10079E+01_JPRB, 0.10332E+01_JPRB, 0.10506E+01_JPRB, 0.10709E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.11286E+01_JPRB, 0.92981E+00_JPRB, 0.88800E+00_JPRB, 0.94862E+00_JPRB, 0.12403E+01_JPRB, 0.13085E+01_JPRB, 0.12868E+01_JPRB ,&
&  0.13567E+01_JPRB, 0.13610E+01_JPRB, 0.14381E+01_JPRB, 0.15595E+01_JPRB, 0.15933E+01_JPRB, 0.16807E+01_JPRB, 0.18151E+01_JPRB ,&
&  0.20406E+01_JPRB, 0.22961E+01_JPRB, 0.23231E+01_JPRB, 0.22877E+01_JPRB, 0.21810E+01_JPRB, 0.19985E+01_JPRB, 0.17062E+01_JPRB ,&
&  0.13476E+01_JPRB, 0.11004E+01_JPRB, 0.90236E+00_JPRB, 0.76495E+00_JPRB, 0.68694E+00_JPRB, 0.65194E+00_JPRB, 0.62546E+00_JPRB ,&
&  0.62243E+00_JPRB, 0.61641E+00_JPRB, 0.60206E+00_JPRB, 0.56478E+00_JPRB, 0.54159E+00_JPRB, 0.55021E+00_JPRB, 0.55932E+00_JPRB ,&
&  0.56176E+00_JPRB, 0.54884E+00_JPRB, 0.51725E+00_JPRB, 0.53058E+00_JPRB, 0.57778E+00_JPRB, 0.62732E+00_JPRB, 0.64983E+00_JPRB ,&
&  0.64908E+00_JPRB, 0.64230E+00_JPRB, 0.65530E+00_JPRB, 0.68363E+00_JPRB, 0.69540E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.73461E+00_JPRB, 0.73803E+00_JPRB, 0.72251E+00_JPRB, 0.70870E+00_JPRB, 0.69008E+00_JPRB, 0.62749E+00_JPRB, 0.61144E+00_JPRB ,&
&  0.67757E+00_JPRB, 0.62149E+00_JPRB, 0.67469E+00_JPRB, 0.57054E+00_JPRB, 0.54602E+00_JPRB, 0.51936E+00_JPRB, 0.49596E+00_JPRB ,&
&  0.46718E+00_JPRB, 0.45700E+00_JPRB, 0.46660E+00_JPRB, 0.47700E+00_JPRB, 0.47775E+00_JPRB, 0.47465E+00_JPRB, 0.47812E+00_JPRB ,&
&  0.48620E+00_JPRB, 0.51020E+00_JPRB, 0.54260E+00_JPRB, 0.55963E+00_JPRB, 0.57707E+00_JPRB, 0.60666E+00_JPRB, 0.62351E+00_JPRB ,&
&  0.60576E+00_JPRB, 0.60650E+00_JPRB, 0.61233E+00_JPRB, 0.63430E+00_JPRB, 0.67638E+00_JPRB, 0.73878E+00_JPRB, 0.77975E+00_JPRB ,&
&  0.79630E+00_JPRB, 0.68263E+00_JPRB, 0.79243E+00_JPRB, 0.81912E+00_JPRB, 0.87892E+00_JPRB, 0.91296E+00_JPRB, 0.93709E+00_JPRB ,&
&  0.97079E+00_JPRB, 0.10092E+01_JPRB, 0.10522E+01_JPRB, 0.10910E+01_JPRB, 0.11305E+01_JPRB, 0.11660E+01_JPRB, 0.11652E+01_JPRB ,&
&  0.12437E+01_JPRB, 0.90809E+00_JPRB, 0.96751E+00_JPRB, 0.10114E+01_JPRB, 0.13506E+01_JPRB, 0.15117E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.17215E+01_JPRB, 0.16466E+01_JPRB, 0.17810E+01_JPRB, 0.19061E+01_JPRB, 0.20548E+01_JPRB, 0.22473E+01_JPRB, 0.23610E+01_JPRB ,&
&  0.24135E+01_JPRB, 0.25328E+01_JPRB, 0.26777E+01_JPRB, 0.25100E+01_JPRB, 0.22378E+01_JPRB, 0.19236E+01_JPRB, 0.15569E+01_JPRB ,&
&  0.13057E+01_JPRB, 0.10909E+01_JPRB, 0.91115E+00_JPRB, 0.78856E+00_JPRB, 0.69399E+00_JPRB, 0.63225E+00_JPRB, 0.59782E+00_JPRB ,&
&  0.57947E+00_JPRB, 0.55864E+00_JPRB, 0.54599E+00_JPRB, 0.52606E+00_JPRB, 0.52170E+00_JPRB, 0.53393E+00_JPRB, 0.56937E+00_JPRB ,&
&  0.58197E+00_JPRB, 0.58466E+00_JPRB, 0.63155E+00_JPRB, 0.69760E+00_JPRB, 0.70777E+00_JPRB, 0.72683E+00_JPRB, 0.80385E+00_JPRB ,&
&  0.71053E+00_JPRB, 0.72556E+00_JPRB, 0.77430E+00_JPRB, 0.76519E+00_JPRB, 0.73873E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.79364E+00_JPRB, 0.79007E+00_JPRB, 0.76182E+00_JPRB, 0.73779E+00_JPRB, 0.72526E+00_JPRB, 0.69260E+00_JPRB, 0.66156E+00_JPRB ,&
&  0.71339E+00_JPRB, 0.70325E+00_JPRB, 0.70470E+00_JPRB, 0.70249E+00_JPRB, 0.69318E+00_JPRB, 0.62759E+00_JPRB, 0.57188E+00_JPRB ,&
&  0.52724E+00_JPRB, 0.50156E+00_JPRB, 0.50451E+00_JPRB, 0.50298E+00_JPRB, 0.49939E+00_JPRB, 0.48933E+00_JPRB, 0.48455E+00_JPRB ,&
&  0.48184E+00_JPRB, 0.49646E+00_JPRB, 0.52949E+00_JPRB, 0.55767E+00_JPRB, 0.56886E+00_JPRB, 0.58389E+00_JPRB, 0.61172E+00_JPRB ,&
&  0.60133E+00_JPRB, 0.61033E+00_JPRB, 0.62770E+00_JPRB, 0.59904E+00_JPRB, 0.69536E+00_JPRB, 0.73962E+00_JPRB, 0.76451E+00_JPRB ,&
&  0.81254E+00_JPRB, 0.85360E+00_JPRB, 0.87145E+00_JPRB, 0.87578E+00_JPRB, 0.89436E+00_JPRB, 0.90513E+00_JPRB, 0.92842E+00_JPRB ,&
&  0.95168E+00_JPRB, 0.98726E+00_JPRB, 0.10367E+01_JPRB, 0.10959E+01_JPRB, 0.12123E+01_JPRB, 0.11159E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.12983E+01_JPRB, 0.13687E+01_JPRB, 0.12366E+01_JPRB, 0.15637E+01_JPRB, 0.17662E+01_JPRB, 0.16569E+01_JPRB ,&
&  0.17208E+01_JPRB, 0.17018E+01_JPRB, 0.20801E+01_JPRB, 0.23712E+01_JPRB, 0.26073E+01_JPRB, 0.28163E+01_JPRB, 0.29017E+01_JPRB ,&
&  0.28944E+01_JPRB, 0.30942E+01_JPRB, 0.30616E+01_JPRB, 0.27735E+01_JPRB, 0.23726E+01_JPRB, 0.20446E+01_JPRB, 0.16451E+01_JPRB ,&
&  0.13178E+01_JPRB, 0.10568E+01_JPRB, 0.89837E+00_JPRB, 0.82030E+00_JPRB, 0.68990E+00_JPRB, 0.64482E+00_JPRB, 0.59675E+00_JPRB ,&
&  0.55160E+00_JPRB, 0.51949E+00_JPRB, 0.51453E+00_JPRB, 0.51373E+00_JPRB, 0.51279E+00_JPRB, 0.52988E+00_JPRB, 0.55077E+00_JPRB ,&
&  0.61264E+00_JPRB, 0.72878E+00_JPRB, 0.95574E+00_JPRB, 0.95763E+00_JPRB, 0.93889E+00_JPRB, 0.91891E+00_JPRB, 0.81415E+00_JPRB ,&
&  0.75972E+00_JPRB, 0.85285E+00_JPRB, 0.82846E+00_JPRB, 0.80398E+00_JPRB, 0.78437E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.84302E+00_JPRB, 0.82777E+00_JPRB, 0.80767E+00_JPRB, 0.78355E+00_JPRB, 0.75853E+00_JPRB, 0.73510E+00_JPRB, 0.72320E+00_JPRB ,&
&  0.72800E+00_JPRB, 0.72523E+00_JPRB, 0.72583E+00_JPRB, 0.72098E+00_JPRB, 0.70697E+00_JPRB, 0.72095E+00_JPRB, 0.64781E+00_JPRB ,&
&  0.57169E+00_JPRB, 0.53968E+00_JPRB, 0.51815E+00_JPRB, 0.51988E+00_JPRB, 0.53206E+00_JPRB, 0.52578E+00_JPRB, 0.51458E+00_JPRB ,&
&  0.50301E+00_JPRB, 0.49976E+00_JPRB, 0.51219E+00_JPRB, 0.53432E+00_JPRB, 0.55017E+00_JPRB, 0.56567E+00_JPRB, 0.58524E+00_JPRB ,&
&  0.57289E+00_JPRB, 0.55506E+00_JPRB, 0.55176E+00_JPRB, 0.56393E+00_JPRB, 0.63506E+00_JPRB, 0.72943E+00_JPRB, 0.78218E+00_JPRB ,&
&  0.80328E+00_JPRB, 0.81926E+00_JPRB, 0.85478E+00_JPRB, 0.86822E+00_JPRB, 0.87494E+00_JPRB, 0.89483E+00_JPRB, 0.91973E+00_JPRB ,&
&  0.94662E+00_JPRB, 0.98404E+00_JPRB, 0.10355E+01_JPRB, 0.11195E+01_JPRB, 0.11390E+01_JPRB, 0.11258E+01_JPRB, 0.12314E+01_JPRB ,&
&  0.13192E+01_JPRB, 0.14519E+01_JPRB, 0.14870E+01_JPRB, 0.15930E+01_JPRB, 0.18099E+01_JPRB, 0.18158E+01_JPRB, 0.18080E+01_JPRB ,&
&  0.19555E+01_JPRB, 0.20318E+01_JPRB, 0.25961E+01_JPRB, 0.28242E+01_JPRB, 0.29182E+01_JPRB, 0.31262E+01_JPRB, 0.31505E+01_JPRB ,&
&  0.32194E+01_JPRB, 0.32519E+01_JPRB, 0.32135E+01_JPRB, 0.29274E+01_JPRB, 0.24739E+01_JPRB, 0.21124E+01_JPRB, 0.16349E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.10692E+01_JPRB, 0.82138E+00_JPRB, 0.72016E+00_JPRB, 0.62897E+00_JPRB, 0.60879E+00_JPRB, 0.57554E+00_JPRB ,&
&  0.53521E+00_JPRB, 0.52603E+00_JPRB, 0.52500E+00_JPRB, 0.56112E+00_JPRB, 0.57534E+00_JPRB, 0.55633E+00_JPRB, 0.60865E+00_JPRB ,&
&  0.71192E+00_JPRB, 0.98561E+00_JPRB, 0.10249E+01_JPRB, 0.11086E+01_JPRB, 0.10808E+01_JPRB, 0.10553E+01_JPRB, 0.84870E+00_JPRB ,&
&  0.87554E+00_JPRB, 0.95051E+00_JPRB, 0.89028E+00_JPRB, 0.86169E+00_JPRB, 0.85195E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.87424E+00_JPRB, 0.85163E+00_JPRB, 0.83784E+00_JPRB, 0.82731E+00_JPRB, 0.80905E+00_JPRB, 0.77943E+00_JPRB, 0.77845E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.76033E+00_JPRB, 0.75738E+00_JPRB, 0.75407E+00_JPRB, 0.74291E+00_JPRB, 0.74649E+00_JPRB, 0.76590E+00_JPRB ,&
&  0.66245E+00_JPRB, 0.61571E+00_JPRB, 0.55424E+00_JPRB, 0.53462E+00_JPRB, 0.58016E+00_JPRB, 0.56298E+00_JPRB, 0.55354E+00_JPRB ,&
&  0.53192E+00_JPRB, 0.51845E+00_JPRB, 0.50396E+00_JPRB, 0.51203E+00_JPRB, 0.53070E+00_JPRB, 0.54912E+00_JPRB, 0.56152E+00_JPRB ,&
&  0.54091E+00_JPRB, 0.51806E+00_JPRB, 0.51705E+00_JPRB, 0.53389E+00_JPRB, 0.58652E+00_JPRB, 0.66648E+00_JPRB, 0.75782E+00_JPRB ,&
&  0.79798E+00_JPRB, 0.81069E+00_JPRB, 0.83270E+00_JPRB, 0.85763E+00_JPRB, 0.88986E+00_JPRB, 0.94057E+00_JPRB, 0.97353E+00_JPRB ,&
&  0.99731E+00_JPRB, 0.10188E+01_JPRB, 0.10663E+01_JPRB, 0.11367E+01_JPRB, 0.10995E+01_JPRB, 0.11013E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.13912E+01_JPRB, 0.14584E+01_JPRB, 0.20179E+01_JPRB, 0.23766E+01_JPRB, 0.24607E+01_JPRB, 0.23967E+01_JPRB ,&
&  0.24785E+01_JPRB, 0.26135E+01_JPRB, 0.27924E+01_JPRB, 0.30154E+01_JPRB, 0.31569E+01_JPRB, 0.33191E+01_JPRB, 0.33302E+01_JPRB ,&
&  0.33401E+01_JPRB, 0.33524E+01_JPRB, 0.32431E+01_JPRB, 0.29761E+01_JPRB, 0.24282E+01_JPRB, 0.20070E+01_JPRB, 0.15477E+01_JPRB ,&
&  0.12913E+01_JPRB, 0.98956E+00_JPRB, 0.69707E+00_JPRB, 0.58764E+00_JPRB, 0.54771E+00_JPRB, 0.59102E+00_JPRB, 0.76182E+00_JPRB ,&
&  0.58573E+00_JPRB, 0.60316E+00_JPRB, 0.62789E+00_JPRB, 0.69867E+00_JPRB, 0.76488E+00_JPRB, 0.78360E+00_JPRB, 0.69962E+00_JPRB ,&
&  0.71814E+00_JPRB, 0.91752E+00_JPRB, 0.11772E+01_JPRB, 0.12767E+01_JPRB, 0.12200E+01_JPRB, 0.11998E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.11695E+01_JPRB, 0.10451E+01_JPRB, 0.95713E+00_JPRB, 0.90882E+00_JPRB, 0.90248E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.87806E+00_JPRB, 0.87330E+00_JPRB, 0.87440E+00_JPRB, 0.87367E+00_JPRB, 0.87063E+00_JPRB, 0.83404E+00_JPRB, 0.82772E+00_JPRB ,&
&  0.81733E+00_JPRB, 0.80404E+00_JPRB, 0.79070E+00_JPRB, 0.78380E+00_JPRB, 0.77529E+00_JPRB, 0.76796E+00_JPRB, 0.76313E+00_JPRB ,&
&  0.78281E+00_JPRB, 0.70871E+00_JPRB, 0.63162E+00_JPRB, 0.55144E+00_JPRB, 0.56475E+00_JPRB, 0.55820E+00_JPRB, 0.55591E+00_JPRB ,&
&  0.54180E+00_JPRB, 0.53867E+00_JPRB, 0.52118E+00_JPRB, 0.51076E+00_JPRB, 0.52449E+00_JPRB, 0.54530E+00_JPRB, 0.55604E+00_JPRB ,&
&  0.54467E+00_JPRB, 0.52333E+00_JPRB, 0.52115E+00_JPRB, 0.55031E+00_JPRB, 0.63586E+00_JPRB, 0.64537E+00_JPRB, 0.74357E+00_JPRB ,&
&  0.79699E+00_JPRB, 0.83023E+00_JPRB, 0.85974E+00_JPRB, 0.90734E+00_JPRB, 0.96245E+00_JPRB, 0.10346E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.11017E+01_JPRB, 0.10679E+01_JPRB, 0.11342E+01_JPRB, 0.11976E+01_JPRB, 0.12369E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.14033E+01_JPRB, 0.16152E+01_JPRB, 0.22659E+01_JPRB, 0.26641E+01_JPRB, 0.26994E+01_JPRB, 0.28655E+01_JPRB ,&
&  0.30770E+01_JPRB, 0.32833E+01_JPRB, 0.34922E+01_JPRB, 0.35345E+01_JPRB, 0.35827E+01_JPRB, 0.34779E+01_JPRB, 0.34425E+01_JPRB ,&
&  0.33699E+01_JPRB, 0.33959E+01_JPRB, 0.33545E+01_JPRB, 0.29211E+01_JPRB, 0.23558E+01_JPRB, 0.18855E+01_JPRB, 0.16247E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.81871E+00_JPRB, 0.58214E+00_JPRB, 0.48290E+00_JPRB, 0.46844E+00_JPRB, 0.49629E+00_JPRB, 0.61176E+00_JPRB ,&
&  0.55882E+00_JPRB, 0.64893E+00_JPRB, 0.82259E+00_JPRB, 0.92444E+00_JPRB, 0.11199E+01_JPRB, 0.13413E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.92422E+00_JPRB, 0.10167E+01_JPRB, 0.13991E+01_JPRB, 0.14781E+01_JPRB, 0.13742E+01_JPRB, 0.13394E+01_JPRB, 0.13474E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.11901E+01_JPRB, 0.10523E+01_JPRB, 0.94000E+00_JPRB, 0.89935E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.94483E+00_JPRB, 0.93121E+00_JPRB, 0.93907E+00_JPRB, 0.94342E+00_JPRB, 0.93367E+00_JPRB, 0.90485E+00_JPRB, 0.89394E+00_JPRB ,&
&  0.86709E+00_JPRB, 0.84049E+00_JPRB, 0.81713E+00_JPRB, 0.80360E+00_JPRB, 0.79445E+00_JPRB, 0.78574E+00_JPRB, 0.76860E+00_JPRB ,&
&  0.74879E+00_JPRB, 0.74347E+00_JPRB, 0.65169E+00_JPRB, 0.58034E+00_JPRB, 0.51746E+00_JPRB, 0.52741E+00_JPRB, 0.54344E+00_JPRB ,&
&  0.56425E+00_JPRB, 0.57499E+00_JPRB, 0.58584E+00_JPRB, 0.59551E+00_JPRB, 0.60672E+00_JPRB, 0.58968E+00_JPRB, 0.60543E+00_JPRB ,&
&  0.59493E+00_JPRB, 0.57600E+00_JPRB, 0.55490E+00_JPRB, 0.61307E+00_JPRB, 0.67951E+00_JPRB, 0.68517E+00_JPRB, 0.81557E+00_JPRB ,&
&  0.88489E+00_JPRB, 0.93401E+00_JPRB, 0.97310E+00_JPRB, 0.10214E+01_JPRB, 0.10728E+01_JPRB, 0.10760E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.99414E+00_JPRB, 0.99483E+00_JPRB, 0.10501E+01_JPRB, 0.10845E+01_JPRB, 0.12276E+01_JPRB, 0.12978E+01_JPRB, 0.13840E+01_JPRB ,&
&  0.15156E+01_JPRB, 0.15404E+01_JPRB, 0.16579E+01_JPRB, 0.23661E+01_JPRB, 0.32293E+01_JPRB, 0.32883E+01_JPRB, 0.34051E+01_JPRB ,&
&  0.36572E+01_JPRB, 0.37380E+01_JPRB, 0.37024E+01_JPRB, 0.35446E+01_JPRB, 0.35432E+01_JPRB, 0.35775E+01_JPRB, 0.36296E+01_JPRB ,&
&  0.34830E+01_JPRB, 0.34071E+01_JPRB, 0.32840E+01_JPRB, 0.29508E+01_JPRB, 0.24440E+01_JPRB, 0.21518E+01_JPRB, 0.18743E+01_JPRB ,&
&  0.16129E+01_JPRB, 0.10412E+01_JPRB, 0.67354E+00_JPRB, 0.55786E+00_JPRB, 0.52558E+00_JPRB, 0.53147E+00_JPRB, 0.62614E+00_JPRB ,&
&  0.64411E+00_JPRB, 0.79721E+00_JPRB, 0.10386E+01_JPRB, 0.11595E+01_JPRB, 0.17258E+01_JPRB, 0.21323E+01_JPRB, 0.19811E+01_JPRB ,&
&  0.16064E+01_JPRB, 0.13976E+01_JPRB, 0.17206E+01_JPRB, 0.17230E+01_JPRB, 0.16081E+01_JPRB, 0.14708E+01_JPRB, 0.14358E+01_JPRB ,&
&  0.13525E+01_JPRB, 0.12901E+01_JPRB, 0.11959E+01_JPRB, 0.10600E+01_JPRB, 0.97867E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.10498E+01_JPRB, 0.10023E+01_JPRB, 0.97667E+00_JPRB, 0.96380E+00_JPRB, 0.92383E+00_JPRB, 0.89646E+00_JPRB, 0.90575E+00_JPRB ,&
&  0.90482E+00_JPRB, 0.87216E+00_JPRB, 0.84610E+00_JPRB, 0.83729E+00_JPRB, 0.83397E+00_JPRB, 0.82678E+00_JPRB, 0.78735E+00_JPRB ,&
&  0.73857E+00_JPRB, 0.70508E+00_JPRB, 0.60661E+00_JPRB, 0.53931E+00_JPRB, 0.50896E+00_JPRB, 0.53050E+00_JPRB, 0.57787E+00_JPRB ,&
&  0.64515E+00_JPRB, 0.71018E+00_JPRB, 0.76862E+00_JPRB, 0.77378E+00_JPRB, 0.77960E+00_JPRB, 0.76165E+00_JPRB, 0.75016E+00_JPRB ,&
&  0.77775E+00_JPRB, 0.67366E+00_JPRB, 0.69002E+00_JPRB, 0.71209E+00_JPRB, 0.78912E+00_JPRB, 0.84381E+00_JPRB, 0.89633E+00_JPRB ,&
&  0.90852E+00_JPRB, 0.94362E+00_JPRB, 0.97321E+00_JPRB, 0.10023E+01_JPRB, 0.10256E+01_JPRB, 0.10293E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.10418E+01_JPRB, 0.10259E+01_JPRB, 0.10270E+01_JPRB, 0.12453E+01_JPRB, 0.17286E+01_JPRB, 0.19696E+01_JPRB, 0.20137E+01_JPRB ,&
&  0.19341E+01_JPRB, 0.20381E+01_JPRB, 0.29635E+01_JPRB, 0.33922E+01_JPRB, 0.36077E+01_JPRB, 0.41147E+01_JPRB, 0.42997E+01_JPRB ,&
&  0.41084E+01_JPRB, 0.37305E+01_JPRB, 0.34018E+01_JPRB, 0.30937E+01_JPRB, 0.37548E+01_JPRB, 0.40006E+01_JPRB, 0.39660E+01_JPRB ,&
&  0.35371E+01_JPRB, 0.34132E+01_JPRB, 0.30761E+01_JPRB, 0.27642E+01_JPRB, 0.25433E+01_JPRB, 0.25550E+01_JPRB, 0.20315E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.14154E+01_JPRB, 0.10605E+01_JPRB, 0.88526E+00_JPRB, 0.81709E+00_JPRB, 0.81141E+00_JPRB, 0.85324E+00_JPRB ,&
&  0.93712E+00_JPRB, 0.10979E+01_JPRB, 0.13485E+01_JPRB, 0.16207E+01_JPRB, 0.22790E+01_JPRB, 0.28184E+01_JPRB, 0.28528E+01_JPRB ,&
&  0.23102E+01_JPRB, 0.22837E+01_JPRB, 0.20916E+01_JPRB, 0.19778E+01_JPRB, 0.18520E+01_JPRB, 0.16540E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.13387E+01_JPRB, 0.12825E+01_JPRB, 0.11886E+01_JPRB, 0.11233E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.12072E+01_JPRB, 0.11412E+01_JPRB, 0.10503E+01_JPRB, 0.98593E+00_JPRB, 0.93103E+00_JPRB, 0.89959E+00_JPRB, 0.89055E+00_JPRB ,&
&  0.88282E+00_JPRB, 0.86919E+00_JPRB, 0.87640E+00_JPRB, 0.88872E+00_JPRB, 0.88910E+00_JPRB, 0.86460E+00_JPRB, 0.79721E+00_JPRB ,&
&  0.73617E+00_JPRB, 0.65086E+00_JPRB, 0.57537E+00_JPRB, 0.56761E+00_JPRB, 0.60427E+00_JPRB, 0.63503E+00_JPRB, 0.68399E+00_JPRB ,&
&  0.79139E+00_JPRB, 0.96122E+00_JPRB, 0.10302E+01_JPRB, 0.10727E+01_JPRB, 0.10737E+01_JPRB, 0.10349E+01_JPRB, 0.97612E+00_JPRB ,&
&  0.89173E+00_JPRB, 0.89417E+00_JPRB, 0.94395E+00_JPRB, 0.93885E+00_JPRB, 0.92013E+00_JPRB, 0.92443E+00_JPRB, 0.93626E+00_JPRB ,&
&  0.93859E+00_JPRB, 0.94214E+00_JPRB, 0.98529E+00_JPRB, 0.10117E+01_JPRB, 0.10692E+01_JPRB, 0.11066E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.10986E+01_JPRB, 0.12588E+01_JPRB, 0.18738E+01_JPRB, 0.24632E+01_JPRB, 0.29141E+01_JPRB, 0.28490E+01_JPRB ,&
&  0.31592E+01_JPRB, 0.34094E+01_JPRB, 0.35700E+01_JPRB, 0.40006E+01_JPRB, 0.32975E+01_JPRB, 0.36936E+01_JPRB, 0.42399E+01_JPRB ,&
&  0.40609E+01_JPRB, 0.36791E+01_JPRB, 0.34945E+01_JPRB, 0.34019E+01_JPRB, 0.38588E+01_JPRB, 0.51855E+01_JPRB, 0.47717E+01_JPRB ,&
&  0.39818E+01_JPRB, 0.35494E+01_JPRB, 0.30686E+01_JPRB, 0.29641E+01_JPRB, 0.25594E+01_JPRB, 0.22347E+01_JPRB, 0.15775E+01_JPRB ,&
&  0.15421E+01_JPRB, 0.13628E+01_JPRB, 0.13541E+01_JPRB, 0.12427E+01_JPRB, 0.12499E+01_JPRB, 0.11951E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.15030E+01_JPRB, 0.20354E+01_JPRB, 0.25022E+01_JPRB, 0.36387E+01_JPRB, 0.35632E+01_JPRB, 0.35626E+01_JPRB ,&
&  0.31663E+01_JPRB, 0.29544E+01_JPRB, 0.24223E+01_JPRB, 0.23463E+01_JPRB, 0.20897E+01_JPRB, 0.18781E+01_JPRB, 0.16967E+01_JPRB ,&
&  0.15030E+01_JPRB, 0.13713E+01_JPRB, 0.12887E+01_JPRB, 0.12356E+01_JPRB, 0.12323E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.11688E+01_JPRB, 0.11115E+01_JPRB, 0.10473E+01_JPRB, 0.98238E+00_JPRB, 0.91544E+00_JPRB, 0.88135E+00_JPRB, 0.86581E+00_JPRB ,&
&  0.86182E+00_JPRB, 0.87644E+00_JPRB, 0.88552E+00_JPRB, 0.88735E+00_JPRB, 0.88683E+00_JPRB, 0.88592E+00_JPRB, 0.81208E+00_JPRB ,&
&  0.73956E+00_JPRB, 0.66508E+00_JPRB, 0.58097E+00_JPRB, 0.57294E+00_JPRB, 0.68573E+00_JPRB, 0.76856E+00_JPRB, 0.76174E+00_JPRB ,&
&  0.10595E+01_JPRB, 0.13163E+01_JPRB, 0.13529E+01_JPRB, 0.13768E+01_JPRB, 0.14028E+01_JPRB, 0.13844E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.12014E+01_JPRB, 0.11612E+01_JPRB, 0.10658E+01_JPRB, 0.99097E+00_JPRB, 0.94392E+00_JPRB, 0.92120E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.88622E+00_JPRB, 0.89970E+00_JPRB, 0.92142E+00_JPRB, 0.95700E+00_JPRB, 0.10286E+01_JPRB, 0.10180E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.12510E+01_JPRB, 0.16226E+01_JPRB, 0.24076E+01_JPRB, 0.34198E+01_JPRB, 0.40801E+01_JPRB, 0.41897E+01_JPRB ,&
&  0.39370E+01_JPRB, 0.39244E+01_JPRB, 0.40704E+01_JPRB, 0.40117E+01_JPRB, 0.39353E+01_JPRB, 0.39947E+01_JPRB, 0.43004E+01_JPRB ,&
&  0.33693E+01_JPRB, 0.27916E+01_JPRB, 0.24850E+01_JPRB, 0.30000E+01_JPRB, 0.48715E+01_JPRB, 0.63214E+01_JPRB, 0.62335E+01_JPRB ,&
&  0.52324E+01_JPRB, 0.43570E+01_JPRB, 0.36643E+01_JPRB, 0.31352E+01_JPRB, 0.30459E+01_JPRB, 0.20870E+01_JPRB, 0.21934E+01_JPRB ,&
&  0.21893E+01_JPRB, 0.21862E+01_JPRB, 0.19840E+01_JPRB, 0.16532E+01_JPRB, 0.14938E+01_JPRB, 0.16539E+01_JPRB, 0.17733E+01_JPRB ,&
&  0.20071E+01_JPRB, 0.22600E+01_JPRB, 0.35697E+01_JPRB, 0.42308E+01_JPRB, 0.48726E+01_JPRB, 0.43427E+01_JPRB, 0.38603E+01_JPRB ,&
&  0.31581E+01_JPRB, 0.29852E+01_JPRB, 0.28779E+01_JPRB, 0.26314E+01_JPRB, 0.23408E+01_JPRB, 0.21106E+01_JPRB, 0.18514E+01_JPRB ,&
&  0.16380E+01_JPRB, 0.14565E+01_JPRB, 0.13159E+01_JPRB, 0.12266E+01_JPRB, 0.11993E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.11699E+01_JPRB, 0.11212E+01_JPRB, 0.10499E+01_JPRB, 0.94767E+00_JPRB, 0.89343E+00_JPRB, 0.88102E+00_JPRB, 0.89516E+00_JPRB ,&
&  0.90917E+00_JPRB, 0.91412E+00_JPRB, 0.90107E+00_JPRB, 0.88318E+00_JPRB, 0.88560E+00_JPRB, 0.87119E+00_JPRB, 0.81838E+00_JPRB ,&
&  0.77142E+00_JPRB, 0.72354E+00_JPRB, 0.62104E+00_JPRB, 0.64003E+00_JPRB, 0.80549E+00_JPRB, 0.92597E+00_JPRB, 0.97122E+00_JPRB ,&
&  0.14983E+01_JPRB, 0.15856E+01_JPRB, 0.14143E+01_JPRB, 0.14175E+01_JPRB, 0.16337E+01_JPRB, 0.14908E+01_JPRB, 0.13464E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.12308E+01_JPRB, 0.11022E+01_JPRB, 0.10041E+01_JPRB, 0.94334E+00_JPRB, 0.90868E+00_JPRB, 0.88040E+00_JPRB ,&
&  0.87591E+00_JPRB, 0.88907E+00_JPRB, 0.94448E+00_JPRB, 0.10096E+01_JPRB, 0.10069E+01_JPRB, 0.10107E+01_JPRB, 0.11213E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.13718E+01_JPRB, 0.17850E+01_JPRB, 0.25703E+01_JPRB, 0.37430E+01_JPRB, 0.47762E+01_JPRB, 0.59116E+01_JPRB ,&
&  0.52106E+01_JPRB, 0.52840E+01_JPRB, 0.46930E+01_JPRB, 0.50915E+01_JPRB, 0.45756E+01_JPRB, 0.43485E+01_JPRB, 0.42588E+01_JPRB ,&
&  0.30924E+01_JPRB, 0.26412E+01_JPRB, 0.30647E+01_JPRB, 0.46797E+01_JPRB, 0.52410E+01_JPRB, 0.59385E+01_JPRB, 0.72409E+01_JPRB ,&
&  0.57178E+01_JPRB, 0.47231E+01_JPRB, 0.41961E+01_JPRB, 0.40915E+01_JPRB, 0.26059E+01_JPRB, 0.19910E+01_JPRB, 0.29356E+01_JPRB ,&
&  0.27365E+01_JPRB, 0.16173E+01_JPRB, 0.12289E+01_JPRB, 0.14145E+01_JPRB, 0.11239E+01_JPRB, 0.13590E+01_JPRB, 0.20658E+01_JPRB ,&
&  0.24528E+01_JPRB, 0.29127E+01_JPRB, 0.46358E+01_JPRB, 0.44685E+01_JPRB, 0.47051E+01_JPRB, 0.48815E+01_JPRB, 0.38810E+01_JPRB ,&
&  0.34335E+01_JPRB, 0.27633E+01_JPRB, 0.27787E+01_JPRB, 0.25076E+01_JPRB, 0.23204E+01_JPRB, 0.21859E+01_JPRB, 0.19549E+01_JPRB ,&
&  0.17245E+01_JPRB, 0.15170E+01_JPRB, 0.13512E+01_JPRB, 0.12463E+01_JPRB, 0.12038E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.11308E+01_JPRB, 0.10753E+01_JPRB, 0.10427E+01_JPRB, 0.98301E+00_JPRB, 0.93728E+00_JPRB, 0.92590E+00_JPRB, 0.95025E+00_JPRB ,&
&  0.93894E+00_JPRB, 0.89841E+00_JPRB, 0.85691E+00_JPRB, 0.84521E+00_JPRB, 0.85780E+00_JPRB, 0.85528E+00_JPRB, 0.81193E+00_JPRB ,&
&  0.77301E+00_JPRB, 0.73624E+00_JPRB, 0.72444E+00_JPRB, 0.79507E+00_JPRB, 0.10820E+01_JPRB, 0.11180E+01_JPRB, 0.14483E+01_JPRB ,&
&  0.19112E+01_JPRB, 0.16953E+01_JPRB, 0.14755E+01_JPRB, 0.14833E+01_JPRB, 0.14138E+01_JPRB, 0.12877E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.12599E+01_JPRB, 0.11903E+01_JPRB, 0.10946E+01_JPRB, 0.10226E+01_JPRB, 0.94779E+00_JPRB, 0.89319E+00_JPRB, 0.87942E+00_JPRB ,&
&  0.89179E+00_JPRB, 0.94397E+00_JPRB, 0.10418E+01_JPRB, 0.10095E+01_JPRB, 0.10173E+01_JPRB, 0.10506E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.14945E+01_JPRB, 0.18779E+01_JPRB, 0.27572E+01_JPRB, 0.46555E+01_JPRB, 0.62045E+01_JPRB, 0.61347E+01_JPRB ,&
&  0.54861E+01_JPRB, 0.52612E+01_JPRB, 0.54359E+01_JPRB, 0.53532E+01_JPRB, 0.45828E+01_JPRB, 0.44309E+01_JPRB, 0.41202E+01_JPRB ,&
&  0.35801E+01_JPRB, 0.33533E+01_JPRB, 0.35179E+01_JPRB, 0.50038E+01_JPRB, 0.61408E+01_JPRB, 0.62883E+01_JPRB, 0.71541E+01_JPRB ,&
&  0.63654E+01_JPRB, 0.51348E+01_JPRB, 0.38460E+01_JPRB, 0.30539E+01_JPRB, 0.30325E+01_JPRB, 0.27096E+01_JPRB, 0.19854E+01_JPRB ,&
&  0.16651E+01_JPRB, 0.13714E+01_JPRB, 0.11528E+01_JPRB, 0.11396E+01_JPRB, 0.99720E+00_JPRB, 0.12074E+01_JPRB, 0.18612E+01_JPRB ,&
&  0.27932E+01_JPRB, 0.33850E+01_JPRB, 0.43040E+01_JPRB, 0.43243E+01_JPRB, 0.44350E+01_JPRB, 0.52004E+01_JPRB, 0.48304E+01_JPRB ,&
&  0.30362E+01_JPRB, 0.25682E+01_JPRB, 0.25895E+01_JPRB, 0.24344E+01_JPRB, 0.22792E+01_JPRB, 0.21260E+01_JPRB, 0.18981E+01_JPRB ,&
&  0.16509E+01_JPRB, 0.14299E+01_JPRB, 0.13093E+01_JPRB, 0.12653E+01_JPRB, 0.12137E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.99597E+00_JPRB, 0.98311E+00_JPRB, 0.99008E+00_JPRB, 0.97520E+00_JPRB, 0.95225E+00_JPRB, 0.95951E+00_JPRB, 0.94154E+00_JPRB ,&
&  0.89141E+00_JPRB, 0.83551E+00_JPRB, 0.81387E+00_JPRB, 0.82541E+00_JPRB, 0.83716E+00_JPRB, 0.84140E+00_JPRB, 0.81239E+00_JPRB ,&
&  0.77329E+00_JPRB, 0.76027E+00_JPRB, 0.80247E+00_JPRB, 0.99451E+00_JPRB, 0.15062E+01_JPRB, 0.14635E+01_JPRB, 0.20367E+01_JPRB ,&
&  0.22198E+01_JPRB, 0.17282E+01_JPRB, 0.15229E+01_JPRB, 0.14271E+01_JPRB, 0.13659E+01_JPRB, 0.12415E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.11196E+01_JPRB, 0.10227E+01_JPRB, 0.96276E+00_JPRB, 0.90929E+00_JPRB, 0.89175E+00_JPRB, 0.92227E+00_JPRB ,&
&  0.98709E+00_JPRB, 0.10654E+01_JPRB, 0.10512E+01_JPRB, 0.10453E+01_JPRB, 0.10965E+01_JPRB, 0.11516E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.14315E+01_JPRB, 0.16450E+01_JPRB, 0.20907E+01_JPRB, 0.35309E+01_JPRB, 0.49784E+01_JPRB, 0.53429E+01_JPRB, 0.59334E+01_JPRB ,&
&  0.60831E+01_JPRB, 0.61587E+01_JPRB, 0.52995E+01_JPRB, 0.56810E+01_JPRB, 0.46576E+01_JPRB, 0.42406E+01_JPRB, 0.40298E+01_JPRB ,&
&  0.36612E+01_JPRB, 0.40749E+01_JPRB, 0.40623E+01_JPRB, 0.44163E+01_JPRB, 0.61594E+01_JPRB, 0.71988E+01_JPRB, 0.71072E+01_JPRB ,&
&  0.67585E+01_JPRB, 0.55651E+01_JPRB, 0.43511E+01_JPRB, 0.37895E+01_JPRB, 0.47897E+01_JPRB, 0.52598E+01_JPRB, 0.24639E+01_JPRB ,&
&  0.16790E+01_JPRB, 0.15324E+01_JPRB, 0.11682E+01_JPRB, 0.10021E+01_JPRB, 0.97258E+00_JPRB, 0.12139E+01_JPRB, 0.21470E+01_JPRB ,&
&  0.24471E+01_JPRB, 0.39290E+01_JPRB, 0.42858E+01_JPRB, 0.45567E+01_JPRB, 0.43600E+01_JPRB, 0.43321E+01_JPRB, 0.38571E+01_JPRB ,&
&  0.38624E+01_JPRB, 0.33649E+01_JPRB, 0.27943E+01_JPRB, 0.24773E+01_JPRB, 0.21728E+01_JPRB, 0.18974E+01_JPRB, 0.16982E+01_JPRB ,&
&  0.15537E+01_JPRB, 0.13882E+01_JPRB, 0.12869E+01_JPRB, 0.12134E+01_JPRB, 0.10873E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.86633E+00_JPRB, 0.89257E+00_JPRB, 0.93206E+00_JPRB, 0.93303E+00_JPRB, 0.91516E+00_JPRB, 0.92348E+00_JPRB, 0.87788E+00_JPRB ,&
&  0.82696E+00_JPRB, 0.80604E+00_JPRB, 0.80633E+00_JPRB, 0.82775E+00_JPRB, 0.86464E+00_JPRB, 0.89108E+00_JPRB, 0.86274E+00_JPRB ,&
&  0.81841E+00_JPRB, 0.81207E+00_JPRB, 0.88847E+00_JPRB, 0.13850E+01_JPRB, 0.18675E+01_JPRB, 0.22899E+01_JPRB, 0.26131E+01_JPRB ,&
&  0.24318E+01_JPRB, 0.18669E+01_JPRB, 0.15783E+01_JPRB, 0.15261E+01_JPRB, 0.14771E+01_JPRB, 0.14097E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.11082E+01_JPRB, 0.10918E+01_JPRB, 0.97052E+00_JPRB, 0.93398E+00_JPRB, 0.91906E+00_JPRB, 0.93962E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.10885E+01_JPRB, 0.11214E+01_JPRB, 0.11798E+01_JPRB, 0.12613E+01_JPRB, 0.13157E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.16065E+01_JPRB, 0.19264E+01_JPRB, 0.26095E+01_JPRB, 0.42176E+01_JPRB, 0.44946E+01_JPRB, 0.57752E+01_JPRB, 0.59419E+01_JPRB ,&
&  0.58855E+01_JPRB, 0.57265E+01_JPRB, 0.50769E+01_JPRB, 0.51559E+01_JPRB, 0.46199E+01_JPRB, 0.42196E+01_JPRB, 0.39074E+01_JPRB ,&
&  0.42251E+01_JPRB, 0.41532E+01_JPRB, 0.51657E+01_JPRB, 0.50090E+01_JPRB, 0.56859E+01_JPRB, 0.92781E+01_JPRB, 0.72309E+01_JPRB ,&
&  0.60127E+01_JPRB, 0.61683E+01_JPRB, 0.49900E+01_JPRB, 0.45368E+01_JPRB, 0.54761E+01_JPRB, 0.65912E+01_JPRB, 0.54178E+01_JPRB ,&
&  0.21067E+01_JPRB, 0.14241E+01_JPRB, 0.12521E+01_JPRB, 0.11107E+01_JPRB, 0.96065E+00_JPRB, 0.11939E+01_JPRB, 0.38640E+01_JPRB ,&
&  0.28435E+01_JPRB, 0.39590E+01_JPRB, 0.40724E+01_JPRB, 0.41097E+01_JPRB, 0.42992E+01_JPRB, 0.38648E+01_JPRB, 0.36437E+01_JPRB ,&
&  0.30383E+01_JPRB, 0.24702E+01_JPRB, 0.20532E+01_JPRB, 0.17582E+01_JPRB, 0.15823E+01_JPRB, 0.14911E+01_JPRB, 0.14602E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.12854E+01_JPRB, 0.11673E+01_JPRB, 0.10133E+01_JPRB, 0.88347E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.87463E+00_JPRB, 0.88437E+00_JPRB, 0.90196E+00_JPRB, 0.93550E+00_JPRB, 0.95923E+00_JPRB, 0.94023E+00_JPRB, 0.89482E+00_JPRB ,&
&  0.89850E+00_JPRB, 0.90928E+00_JPRB, 0.95460E+00_JPRB, 0.98672E+00_JPRB, 0.10263E+01_JPRB, 0.10158E+01_JPRB, 0.97165E+00_JPRB ,&
&  0.91397E+00_JPRB, 0.93178E+00_JPRB, 0.12010E+01_JPRB, 0.17865E+01_JPRB, 0.21455E+01_JPRB, 0.24427E+01_JPRB, 0.29044E+01_JPRB ,&
&  0.28974E+01_JPRB, 0.20436E+01_JPRB, 0.16034E+01_JPRB, 0.16350E+01_JPRB, 0.16345E+01_JPRB, 0.16477E+01_JPRB, 0.13710E+01_JPRB ,&
&  0.11802E+01_JPRB, 0.11083E+01_JPRB, 0.99561E+00_JPRB, 0.98565E+00_JPRB, 0.10006E+01_JPRB, 0.10365E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.12426E+01_JPRB, 0.13525E+01_JPRB, 0.14442E+01_JPRB, 0.15100E+01_JPRB, 0.15581E+01_JPRB, 0.16647E+01_JPRB ,&
&  0.19130E+01_JPRB, 0.23410E+01_JPRB, 0.31031E+01_JPRB, 0.41538E+01_JPRB, 0.50490E+01_JPRB, 0.53160E+01_JPRB, 0.56393E+01_JPRB ,&
&  0.55897E+01_JPRB, 0.55161E+01_JPRB, 0.51949E+01_JPRB, 0.47666E+01_JPRB, 0.43014E+01_JPRB, 0.42131E+01_JPRB, 0.40099E+01_JPRB ,&
&  0.45447E+01_JPRB, 0.51366E+01_JPRB, 0.60270E+01_JPRB, 0.58446E+01_JPRB, 0.61304E+01_JPRB, 0.87383E+01_JPRB, 0.92877E+01_JPRB ,&
&  0.74895E+01_JPRB, 0.54377E+01_JPRB, 0.46580E+01_JPRB, 0.49445E+01_JPRB, 0.54349E+01_JPRB, 0.59129E+01_JPRB, 0.81342E+01_JPRB ,&
&  0.65046E+01_JPRB, 0.28639E+01_JPRB, 0.16295E+01_JPRB, 0.16300E+01_JPRB, 0.96040E+00_JPRB, 0.87732E+00_JPRB, 0.16962E+01_JPRB ,&
&  0.23510E+01_JPRB, 0.29975E+01_JPRB, 0.30895E+01_JPRB, 0.34424E+01_JPRB, 0.35117E+01_JPRB, 0.29137E+01_JPRB, 0.22252E+01_JPRB ,&
&  0.16755E+01_JPRB, 0.13821E+01_JPRB, 0.12305E+01_JPRB, 0.11576E+01_JPRB, 0.11889E+01_JPRB, 0.12501E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.11585E+01_JPRB, 0.98992E+00_JPRB, 0.86549E+00_JPRB, 0.84277E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.99354E+00_JPRB, 0.98556E+00_JPRB, 0.99558E+00_JPRB, 0.10355E+01_JPRB, 0.10542E+01_JPRB, 0.10662E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.12363E+01_JPRB, 0.12634E+01_JPRB, 0.12754E+01_JPRB, 0.12533E+01_JPRB, 0.11785E+01_JPRB, 0.11194E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.13467E+01_JPRB, 0.18189E+01_JPRB, 0.24276E+01_JPRB, 0.26854E+01_JPRB, 0.34208E+01_JPRB, 0.28068E+01_JPRB ,&
&  0.27880E+01_JPRB, 0.25464E+01_JPRB, 0.18100E+01_JPRB, 0.18702E+01_JPRB, 0.18709E+01_JPRB, 0.19465E+01_JPRB, 0.14842E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.12056E+01_JPRB, 0.10986E+01_JPRB, 0.10865E+01_JPRB, 0.11198E+01_JPRB, 0.11933E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.13966E+01_JPRB, 0.15698E+01_JPRB, 0.17210E+01_JPRB, 0.17957E+01_JPRB, 0.18386E+01_JPRB, 0.18940E+01_JPRB, 0.20542E+01_JPRB ,&
&  0.23293E+01_JPRB, 0.27943E+01_JPRB, 0.35574E+01_JPRB, 0.43763E+01_JPRB, 0.49106E+01_JPRB, 0.48941E+01_JPRB, 0.51313E+01_JPRB ,&
&  0.49047E+01_JPRB, 0.45742E+01_JPRB, 0.46335E+01_JPRB, 0.44751E+01_JPRB, 0.41692E+01_JPRB, 0.40083E+01_JPRB, 0.39147E+01_JPRB ,&
&  0.50037E+01_JPRB, 0.66635E+01_JPRB, 0.85643E+01_JPRB, 0.74426E+01_JPRB, 0.73435E+01_JPRB, 0.90166E+01_JPRB, 0.93406E+01_JPRB ,&
&  0.80819E+01_JPRB, 0.59579E+01_JPRB, 0.53021E+01_JPRB, 0.47830E+01_JPRB, 0.47224E+01_JPRB, 0.49366E+01_JPRB, 0.60038E+01_JPRB ,&
&  0.63282E+01_JPRB, 0.63727E+01_JPRB, 0.32846E+01_JPRB, 0.14077E+01_JPRB, 0.97911E+00_JPRB, 0.10718E+01_JPRB, 0.13515E+01_JPRB ,&
&  0.18326E+01_JPRB, 0.21149E+01_JPRB, 0.22224E+01_JPRB, 0.28998E+01_JPRB, 0.25905E+01_JPRB, 0.20321E+01_JPRB, 0.15807E+01_JPRB ,&
&  0.12958E+01_JPRB, 0.10887E+01_JPRB, 0.98901E+00_JPRB, 0.98714E+00_JPRB, 0.10719E+01_JPRB, 0.11614E+01_JPRB, 0.12014E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.98470E+00_JPRB, 0.92307E+00_JPRB, 0.92142E+00_JPRB, 0.95589E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.11559E+01_JPRB, 0.11852E+01_JPRB, 0.12417E+01_JPRB, 0.13248E+01_JPRB, 0.14050E+01_JPRB, 0.14995E+01_JPRB, 0.15903E+01_JPRB ,&
&  0.16374E+01_JPRB, 0.16279E+01_JPRB, 0.16099E+01_JPRB, 0.15553E+01_JPRB, 0.15235E+01_JPRB, 0.15140E+01_JPRB, 0.16111E+01_JPRB ,&
&  0.19040E+01_JPRB, 0.23533E+01_JPRB, 0.27976E+01_JPRB, 0.30696E+01_JPRB, 0.31535E+01_JPRB, 0.35480E+01_JPRB, 0.35689E+01_JPRB ,&
&  0.26455E+01_JPRB, 0.33195E+01_JPRB, 0.20711E+01_JPRB, 0.21611E+01_JPRB, 0.22458E+01_JPRB, 0.24656E+01_JPRB, 0.18858E+01_JPRB ,&
&  0.15066E+01_JPRB, 0.14245E+01_JPRB, 0.13682E+01_JPRB, 0.13629E+01_JPRB, 0.13927E+01_JPRB, 0.14987E+01_JPRB, 0.16569E+01_JPRB ,&
&  0.18204E+01_JPRB, 0.19560E+01_JPRB, 0.20579E+01_JPRB, 0.21197E+01_JPRB, 0.21925E+01_JPRB, 0.22872E+01_JPRB, 0.25198E+01_JPRB ,&
&  0.28432E+01_JPRB, 0.33541E+01_JPRB, 0.39725E+01_JPRB, 0.46151E+01_JPRB, 0.46114E+01_JPRB, 0.45372E+01_JPRB, 0.46643E+01_JPRB ,&
&  0.45066E+01_JPRB, 0.42021E+01_JPRB, 0.42316E+01_JPRB, 0.39843E+01_JPRB, 0.37381E+01_JPRB, 0.39540E+01_JPRB, 0.41417E+01_JPRB ,&
&  0.58787E+01_JPRB, 0.71601E+01_JPRB, 0.97942E+01_JPRB, 0.91119E+01_JPRB, 0.79593E+01_JPRB, 0.83613E+01_JPRB, 0.72512E+01_JPRB ,&
&  0.56450E+01_JPRB, 0.50484E+01_JPRB, 0.46481E+01_JPRB, 0.42381E+01_JPRB, 0.40859E+01_JPRB, 0.41796E+01_JPRB, 0.46799E+01_JPRB ,&
&  0.48712E+01_JPRB, 0.46134E+01_JPRB, 0.31939E+01_JPRB, 0.15077E+01_JPRB, 0.11158E+01_JPRB, 0.10767E+01_JPRB, 0.12850E+01_JPRB ,&
&  0.17413E+01_JPRB, 0.19207E+01_JPRB, 0.19384E+01_JPRB, 0.18626E+01_JPRB, 0.15228E+01_JPRB, 0.12369E+01_JPRB, 0.10367E+01_JPRB ,&
&  0.97329E+00_JPRB, 0.94857E+00_JPRB, 0.94838E+00_JPRB, 0.10221E+01_JPRB, 0.10986E+01_JPRB, 0.11321E+01_JPRB, 0.10352E+01_JPRB ,&
&  0.96822E+00_JPRB, 0.96329E+00_JPRB, 0.10195E+01_JPRB, 0.10762E+01_JPRB, 0.11245E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.13556E+01_JPRB, 0.14363E+01_JPRB, 0.15483E+01_JPRB, 0.17146E+01_JPRB, 0.19361E+01_JPRB, 0.21741E+01_JPRB, 0.21526E+01_JPRB ,&
&  0.20869E+01_JPRB, 0.19833E+01_JPRB, 0.20574E+01_JPRB, 0.21342E+01_JPRB, 0.22214E+01_JPRB, 0.23668E+01_JPRB, 0.25842E+01_JPRB ,&
&  0.28506E+01_JPRB, 0.31155E+01_JPRB, 0.33582E+01_JPRB, 0.35973E+01_JPRB, 0.38462E+01_JPRB, 0.44055E+01_JPRB, 0.46131E+01_JPRB ,&
&  0.41292E+01_JPRB, 0.26471E+01_JPRB, 0.25577E+01_JPRB, 0.22712E+01_JPRB, 0.23221E+01_JPRB, 0.25466E+01_JPRB, 0.24113E+01_JPRB ,&
&  0.19888E+01_JPRB, 0.18250E+01_JPRB, 0.17719E+01_JPRB, 0.17706E+01_JPRB, 0.17647E+01_JPRB, 0.19004E+01_JPRB, 0.20371E+01_JPRB ,&
&  0.21492E+01_JPRB, 0.22300E+01_JPRB, 0.23191E+01_JPRB, 0.23973E+01_JPRB, 0.24908E+01_JPRB, 0.26456E+01_JPRB, 0.29350E+01_JPRB ,&
&  0.32641E+01_JPRB, 0.36840E+01_JPRB, 0.41210E+01_JPRB, 0.43575E+01_JPRB, 0.42979E+01_JPRB, 0.42304E+01_JPRB, 0.42481E+01_JPRB ,&
&  0.41932E+01_JPRB, 0.41381E+01_JPRB, 0.40340E+01_JPRB, 0.38778E+01_JPRB, 0.37033E+01_JPRB, 0.43118E+01_JPRB, 0.50946E+01_JPRB ,&
&  0.66333E+01_JPRB, 0.77101E+01_JPRB, 0.99181E+01_JPRB, 0.87758E+01_JPRB, 0.76062E+01_JPRB, 0.68498E+01_JPRB, 0.58204E+01_JPRB ,&
&  0.45949E+01_JPRB, 0.43441E+01_JPRB, 0.40039E+01_JPRB, 0.33590E+01_JPRB, 0.29917E+01_JPRB, 0.27947E+01_JPRB, 0.34707E+01_JPRB ,&
&  0.37764E+01_JPRB, 0.37515E+01_JPRB, 0.23083E+01_JPRB, 0.15153E+01_JPRB, 0.11960E+01_JPRB, 0.11325E+01_JPRB, 0.13632E+01_JPRB ,&
&  0.17814E+01_JPRB, 0.17486E+01_JPRB, 0.15178E+01_JPRB, 0.14475E+01_JPRB, 0.12258E+01_JPRB, 0.10066E+01_JPRB, 0.85089E+00_JPRB ,&
&  0.87495E+00_JPRB, 0.88218E+00_JPRB, 0.91459E+00_JPRB, 0.98312E+00_JPRB, 0.10106E+01_JPRB, 0.10551E+01_JPRB, 0.10485E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.11519E+01_JPRB, 0.11974E+01_JPRB, 0.12436E+01_JPRB, 0.12898E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.14378E+01_JPRB, 0.15273E+01_JPRB, 0.16398E+01_JPRB, 0.17765E+01_JPRB, 0.19214E+01_JPRB, 0.20275E+01_JPRB, 0.20949E+01_JPRB ,&
&  0.22113E+01_JPRB, 0.23508E+01_JPRB, 0.25010E+01_JPRB, 0.26215E+01_JPRB, 0.27039E+01_JPRB, 0.28211E+01_JPRB, 0.29531E+01_JPRB ,&
&  0.30819E+01_JPRB, 0.32260E+01_JPRB, 0.34079E+01_JPRB, 0.36328E+01_JPRB, 0.39408E+01_JPRB, 0.43450E+01_JPRB, 0.45717E+01_JPRB ,&
&  0.36726E+01_JPRB, 0.23222E+01_JPRB, 0.26850E+01_JPRB, 0.25335E+01_JPRB, 0.24466E+01_JPRB, 0.24805E+01_JPRB, 0.23662E+01_JPRB ,&
&  0.21779E+01_JPRB, 0.21342E+01_JPRB, 0.21740E+01_JPRB, 0.21719E+01_JPRB, 0.21193E+01_JPRB, 0.22067E+01_JPRB, 0.22943E+01_JPRB ,&
&  0.23721E+01_JPRB, 0.24463E+01_JPRB, 0.25229E+01_JPRB, 0.25844E+01_JPRB, 0.26869E+01_JPRB, 0.28586E+01_JPRB, 0.31285E+01_JPRB ,&
&  0.34522E+01_JPRB, 0.37409E+01_JPRB, 0.40054E+01_JPRB, 0.41746E+01_JPRB, 0.41356E+01_JPRB, 0.41024E+01_JPRB, 0.41576E+01_JPRB ,&
&  0.42253E+01_JPRB, 0.41566E+01_JPRB, 0.42490E+01_JPRB, 0.45520E+01_JPRB, 0.49673E+01_JPRB, 0.51252E+01_JPRB, 0.63460E+01_JPRB ,&
&  0.74368E+01_JPRB, 0.79682E+01_JPRB, 0.86147E+01_JPRB, 0.10172E+02_JPRB, 0.64889E+01_JPRB, 0.56911E+01_JPRB, 0.46069E+01_JPRB ,&
&  0.45555E+01_JPRB, 0.41789E+01_JPRB, 0.35491E+01_JPRB, 0.26966E+01_JPRB, 0.22514E+01_JPRB, 0.19114E+01_JPRB, 0.23267E+01_JPRB ,&
&  0.29132E+01_JPRB, 0.28662E+01_JPRB, 0.20304E+01_JPRB, 0.16456E+01_JPRB, 0.14315E+01_JPRB, 0.15637E+01_JPRB, 0.17755E+01_JPRB ,&
&  0.18506E+01_JPRB, 0.15600E+01_JPRB, 0.14343E+01_JPRB, 0.15741E+01_JPRB, 0.12404E+01_JPRB, 0.85488E+00_JPRB, 0.70367E+00_JPRB ,&
&  0.71007E+00_JPRB, 0.75504E+00_JPRB, 0.82499E+00_JPRB, 0.91137E+00_JPRB, 0.95663E+00_JPRB, 0.10100E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.12285E+01_JPRB, 0.12862E+01_JPRB, 0.13162E+01_JPRB, 0.13752E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.13769E+01_JPRB, 0.14599E+01_JPRB, 0.15283E+01_JPRB, 0.16112E+01_JPRB, 0.16908E+01_JPRB, 0.17704E+01_JPRB, 0.18779E+01_JPRB ,&
&  0.20110E+01_JPRB, 0.21600E+01_JPRB, 0.23025E+01_JPRB, 0.24121E+01_JPRB, 0.25226E+01_JPRB, 0.26010E+01_JPRB, 0.26716E+01_JPRB ,&
&  0.27153E+01_JPRB, 0.27884E+01_JPRB, 0.28574E+01_JPRB, 0.29595E+01_JPRB, 0.31074E+01_JPRB, 0.32847E+01_JPRB, 0.34559E+01_JPRB ,&
&  0.35526E+01_JPRB, 0.33585E+01_JPRB, 0.28435E+01_JPRB, 0.20597E+01_JPRB, 0.18600E+01_JPRB, 0.21667E+01_JPRB, 0.21952E+01_JPRB ,&
&  0.22095E+01_JPRB, 0.21883E+01_JPRB, 0.21685E+01_JPRB, 0.22478E+01_JPRB, 0.22989E+01_JPRB, 0.23483E+01_JPRB, 0.23781E+01_JPRB ,&
&  0.24198E+01_JPRB, 0.24924E+01_JPRB, 0.25409E+01_JPRB, 0.25900E+01_JPRB, 0.26758E+01_JPRB, 0.28428E+01_JPRB, 0.30899E+01_JPRB ,&
&  0.33884E+01_JPRB, 0.36726E+01_JPRB, 0.38564E+01_JPRB, 0.39907E+01_JPRB, 0.39713E+01_JPRB, 0.40631E+01_JPRB, 0.42910E+01_JPRB ,&
&  0.45963E+01_JPRB, 0.47668E+01_JPRB, 0.50786E+01_JPRB, 0.56477E+01_JPRB, 0.60661E+01_JPRB, 0.59226E+01_JPRB, 0.66902E+01_JPRB ,&
&  0.75449E+01_JPRB, 0.78348E+01_JPRB, 0.77344E+01_JPRB, 0.98389E+01_JPRB, 0.61352E+01_JPRB, 0.59230E+01_JPRB, 0.53487E+01_JPRB ,&
&  0.47363E+01_JPRB, 0.45405E+01_JPRB, 0.33183E+01_JPRB, 0.22616E+01_JPRB, 0.18664E+01_JPRB, 0.15881E+01_JPRB, 0.19487E+01_JPRB ,&
&  0.24228E+01_JPRB, 0.21126E+01_JPRB, 0.16848E+01_JPRB, 0.15541E+01_JPRB, 0.15597E+01_JPRB, 0.19048E+01_JPRB, 0.20675E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.14319E+01_JPRB, 0.14025E+01_JPRB, 0.17981E+01_JPRB, 0.15481E+01_JPRB, 0.79082E+00_JPRB, 0.63452E+00_JPRB ,&
&  0.64600E+00_JPRB, 0.69822E+00_JPRB, 0.75959E+00_JPRB, 0.81114E+00_JPRB, 0.85671E+00_JPRB, 0.91367E+00_JPRB, 0.97675E+00_JPRB ,&
&  0.10496E+01_JPRB, 0.11405E+01_JPRB, 0.12081E+01_JPRB, 0.12658E+01_JPRB, 0.13146E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.10771E+01_JPRB, 0.11531E+01_JPRB, 0.12248E+01_JPRB, 0.12736E+01_JPRB, 0.12942E+01_JPRB, 0.13135E+01_JPRB, 0.13189E+01_JPRB ,&
&  0.13330E+01_JPRB, 0.13322E+01_JPRB, 0.13772E+01_JPRB, 0.14803E+01_JPRB, 0.16706E+01_JPRB, 0.17844E+01_JPRB, 0.19158E+01_JPRB ,&
&  0.19260E+01_JPRB, 0.19409E+01_JPRB, 0.19587E+01_JPRB, 0.20059E+01_JPRB, 0.20901E+01_JPRB, 0.22113E+01_JPRB, 0.24875E+01_JPRB ,&
&  0.30228E+01_JPRB, 0.33605E+01_JPRB, 0.33030E+01_JPRB, 0.29283E+01_JPRB, 0.23520E+01_JPRB, 0.22040E+01_JPRB, 0.25121E+01_JPRB ,&
&  0.25674E+01_JPRB, 0.23756E+01_JPRB, 0.22545E+01_JPRB, 0.22473E+01_JPRB, 0.22646E+01_JPRB, 0.22866E+01_JPRB, 0.23018E+01_JPRB ,&
&  0.23381E+01_JPRB, 0.23906E+01_JPRB, 0.24334E+01_JPRB, 0.24812E+01_JPRB, 0.25640E+01_JPRB, 0.26916E+01_JPRB, 0.28590E+01_JPRB ,&
&  0.30822E+01_JPRB, 0.32791E+01_JPRB, 0.34014E+01_JPRB, 0.34681E+01_JPRB, 0.36141E+01_JPRB, 0.39671E+01_JPRB, 0.44951E+01_JPRB ,&
&  0.49814E+01_JPRB, 0.52272E+01_JPRB, 0.57613E+01_JPRB, 0.63282E+01_JPRB, 0.64101E+01_JPRB, 0.63733E+01_JPRB, 0.67393E+01_JPRB ,&
&  0.69955E+01_JPRB, 0.70644E+01_JPRB, 0.61234E+01_JPRB, 0.84520E+01_JPRB, 0.86116E+01_JPRB, 0.71657E+01_JPRB, 0.55993E+01_JPRB ,&
&  0.49155E+01_JPRB, 0.44407E+01_JPRB, 0.35349E+01_JPRB, 0.25488E+01_JPRB, 0.20742E+01_JPRB, 0.20736E+01_JPRB, 0.21380E+01_JPRB ,&
&  0.21931E+01_JPRB, 0.17410E+01_JPRB, 0.15811E+01_JPRB, 0.15971E+01_JPRB, 0.15155E+01_JPRB, 0.19004E+01_JPRB, 0.17345E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.12175E+01_JPRB, 0.13069E+01_JPRB, 0.17938E+01_JPRB, 0.24874E+01_JPRB, 0.74527E+00_JPRB, 0.56874E+00_JPRB ,&
&  0.58539E+00_JPRB, 0.61960E+00_JPRB, 0.66176E+00_JPRB, 0.68167E+00_JPRB, 0.68669E+00_JPRB, 0.72585E+00_JPRB, 0.78548E+00_JPRB ,&
&  0.82019E+00_JPRB, 0.84862E+00_JPRB, 0.88393E+00_JPRB, 0.96167E+00_JPRB, 0.10136E+01_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.71107E+00_JPRB, 0.75274E+00_JPRB, 0.78184E+00_JPRB, 0.81166E+00_JPRB, 0.82445E+00_JPRB, 0.81605E+00_JPRB, 0.77793E+00_JPRB ,&
&  0.74721E+00_JPRB, 0.71903E+00_JPRB, 0.71371E+00_JPRB, 0.70190E+00_JPRB, 0.73622E+00_JPRB, 0.80114E+00_JPRB, 0.84529E+00_JPRB ,&
&  0.96189E+00_JPRB, 0.89574E+00_JPRB, 0.89017E+00_JPRB, 0.92005E+00_JPRB, 0.92320E+00_JPRB, 0.96706E+00_JPRB, 0.11297E+01_JPRB ,&
&  0.13829E+01_JPRB, 0.16660E+01_JPRB, 0.20845E+01_JPRB, 0.27687E+01_JPRB, 0.30030E+01_JPRB, 0.24732E+01_JPRB, 0.26937E+01_JPRB ,&
&  0.24306E+01_JPRB, 0.20485E+01_JPRB, 0.20375E+01_JPRB, 0.20059E+01_JPRB, 0.20585E+01_JPRB, 0.20751E+01_JPRB, 0.21283E+01_JPRB ,&
&  0.21898E+01_JPRB, 0.22614E+01_JPRB, 0.23291E+01_JPRB, 0.23885E+01_JPRB, 0.24559E+01_JPRB, 0.25370E+01_JPRB, 0.26482E+01_JPRB ,&
&  0.27332E+01_JPRB, 0.28071E+01_JPRB, 0.29396E+01_JPRB, 0.30143E+01_JPRB, 0.34104E+01_JPRB, 0.39858E+01_JPRB, 0.45662E+01_JPRB ,&
&  0.49685E+01_JPRB, 0.52502E+01_JPRB, 0.57418E+01_JPRB, 0.59043E+01_JPRB, 0.59887E+01_JPRB, 0.60786E+01_JPRB, 0.60225E+01_JPRB ,&
&  0.57511E+01_JPRB, 0.47278E+01_JPRB, 0.43178E+01_JPRB, 0.66602E+01_JPRB, 0.64012E+01_JPRB, 0.50180E+01_JPRB, 0.49630E+01_JPRB ,&
&  0.45455E+01_JPRB, 0.42568E+01_JPRB, 0.36215E+01_JPRB, 0.28627E+01_JPRB, 0.23116E+01_JPRB, 0.19814E+01_JPRB, 0.20391E+01_JPRB ,&
&  0.17728E+01_JPRB, 0.14703E+01_JPRB, 0.15460E+01_JPRB, 0.16487E+01_JPRB, 0.15750E+01_JPRB, 0.16793E+01_JPRB, 0.14863E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.10816E+01_JPRB, 0.12558E+01_JPRB, 0.16761E+01_JPRB, 0.11929E+01_JPRB, 0.61741E+00_JPRB, 0.50926E+00_JPRB ,&
&  0.51997E+00_JPRB, 0.53337E+00_JPRB, 0.54574E+00_JPRB, 0.55459E+00_JPRB, 0.56190E+00_JPRB, 0.58170E+00_JPRB, 0.59689E+00_JPRB ,&
&  0.61234E+00_JPRB, 0.64245E+00_JPRB, 0.65549E+00_JPRB, 0.67250E+00_JPRB, 0.69701E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.61947E+00_JPRB, 0.62005E+00_JPRB, 0.61441E+00_JPRB, 0.61710E+00_JPRB, 0.62418E+00_JPRB, 0.64182E+00_JPRB, 0.65325E+00_JPRB ,&
&  0.65421E+00_JPRB, 0.66172E+00_JPRB, 0.64516E+00_JPRB, 0.64953E+00_JPRB, 0.67323E+00_JPRB, 0.69176E+00_JPRB, 0.71129E+00_JPRB ,&
&  0.76401E+00_JPRB, 0.77919E+00_JPRB, 0.77682E+00_JPRB, 0.78350E+00_JPRB, 0.77603E+00_JPRB, 0.83231E+00_JPRB, 0.83647E+00_JPRB ,&
&  0.81219E+00_JPRB, 0.83774E+00_JPRB, 0.99047E+00_JPRB, 0.13889E+01_JPRB, 0.17781E+01_JPRB, 0.18352E+01_JPRB, 0.21826E+01_JPRB ,&
&  0.21667E+01_JPRB, 0.16303E+01_JPRB, 0.16972E+01_JPRB, 0.14906E+01_JPRB, 0.15226E+01_JPRB, 0.16000E+01_JPRB, 0.16278E+01_JPRB ,&
&  0.16704E+01_JPRB, 0.16742E+01_JPRB, 0.17053E+01_JPRB, 0.17536E+01_JPRB, 0.17842E+01_JPRB, 0.18253E+01_JPRB, 0.18752E+01_JPRB ,&
&  0.18189E+01_JPRB, 0.19039E+01_JPRB, 0.21614E+01_JPRB, 0.25957E+01_JPRB, 0.30959E+01_JPRB, 0.37996E+01_JPRB, 0.41286E+01_JPRB ,&
&  0.43666E+01_JPRB, 0.46361E+01_JPRB, 0.43896E+01_JPRB, 0.45357E+01_JPRB, 0.49286E+01_JPRB, 0.46293E+01_JPRB, 0.44368E+01_JPRB ,&
&  0.42292E+01_JPRB, 0.33250E+01_JPRB, 0.31832E+01_JPRB, 0.46136E+01_JPRB, 0.47704E+01_JPRB, 0.43950E+01_JPRB, 0.43821E+01_JPRB ,&
&  0.41528E+01_JPRB, 0.37762E+01_JPRB, 0.31595E+01_JPRB, 0.26669E+01_JPRB, 0.22307E+01_JPRB, 0.18157E+01_JPRB, 0.17256E+01_JPRB ,&
&  0.14105E+01_JPRB, 0.14099E+01_JPRB, 0.16615E+01_JPRB, 0.17870E+01_JPRB, 0.18247E+01_JPRB, 0.20342E+01_JPRB, 0.15460E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.11018E+01_JPRB, 0.15851E+01_JPRB, 0.21432E+01_JPRB, 0.10010E+01_JPRB, 0.62560E+00_JPRB, 0.46324E+00_JPRB ,&
&  0.47985E+00_JPRB, 0.49859E+00_JPRB, 0.50438E+00_JPRB, 0.51171E+00_JPRB, 0.52775E+00_JPRB, 0.54124E+00_JPRB, 0.56414E+00_JPRB ,&
&  0.57774E+00_JPRB, 0.58865E+00_JPRB, 0.60004E+00_JPRB, 0.61637E+00_JPRB, 0.61401E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.67508E+00_JPRB, 0.68616E+00_JPRB, 0.69485E+00_JPRB, 0.71563E+00_JPRB, 0.74605E+00_JPRB, 0.75773E+00_JPRB, 0.77907E+00_JPRB ,&
&  0.79309E+00_JPRB, 0.81603E+00_JPRB, 0.83948E+00_JPRB, 0.86708E+00_JPRB, 0.88842E+00_JPRB, 0.90013E+00_JPRB, 0.92937E+00_JPRB ,&
&  0.95646E+00_JPRB, 0.95447E+00_JPRB, 0.96951E+00_JPRB, 0.99991E+00_JPRB, 0.10069E+01_JPRB, 0.10465E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10596E+01_JPRB, 0.10845E+01_JPRB, 0.11817E+01_JPRB, 0.12960E+01_JPRB, 0.15169E+01_JPRB, 0.19533E+01_JPRB ,&
&  0.17778E+01_JPRB, 0.13787E+01_JPRB, 0.12334E+01_JPRB, 0.10545E+01_JPRB, 0.10507E+01_JPRB, 0.10796E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.10768E+01_JPRB, 0.10468E+01_JPRB, 0.10521E+01_JPRB, 0.10795E+01_JPRB, 0.11726E+01_JPRB, 0.13550E+01_JPRB, 0.15009E+01_JPRB ,&
&  0.15561E+01_JPRB, 0.18171E+01_JPRB, 0.21339E+01_JPRB, 0.25535E+01_JPRB, 0.28458E+01_JPRB, 0.36215E+01_JPRB, 0.37013E+01_JPRB ,&
&  0.36217E+01_JPRB, 0.35316E+01_JPRB, 0.30129E+01_JPRB, 0.31701E+01_JPRB, 0.35278E+01_JPRB, 0.34070E+01_JPRB, 0.30273E+01_JPRB ,&
&  0.30063E+01_JPRB, 0.28210E+01_JPRB, 0.28949E+01_JPRB, 0.37809E+01_JPRB, 0.40246E+01_JPRB, 0.40424E+01_JPRB, 0.38105E+01_JPRB ,&
&  0.35112E+01_JPRB, 0.30668E+01_JPRB, 0.26604E+01_JPRB, 0.24415E+01_JPRB, 0.22953E+01_JPRB, 0.19471E+01_JPRB, 0.16166E+01_JPRB ,&
&  0.14192E+01_JPRB, 0.15589E+01_JPRB, 0.17705E+01_JPRB, 0.18526E+01_JPRB, 0.21471E+01_JPRB, 0.18518E+01_JPRB, 0.20284E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.10667E+01_JPRB, 0.85264E+00_JPRB, 0.11409E+01_JPRB, 0.73763E+00_JPRB, 0.53140E+00_JPRB, 0.51074E+00_JPRB ,&
&  0.52502E+00_JPRB, 0.53457E+00_JPRB, 0.54951E+00_JPRB, 0.56096E+00_JPRB, 0.57516E+00_JPRB, 0.59993E+00_JPRB, 0.62654E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.64282E+00_JPRB, 0.64558E+00_JPRB, 0.65326E+00_JPRB, 0.66984E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.72590E+00_JPRB, 0.74996E+00_JPRB, 0.77432E+00_JPRB, 0.79904E+00_JPRB, 0.82108E+00_JPRB, 0.84232E+00_JPRB, 0.86422E+00_JPRB ,&
&  0.88582E+00_JPRB, 0.91054E+00_JPRB, 0.93668E+00_JPRB, 0.96531E+00_JPRB, 0.99092E+00_JPRB, 0.10060E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.10584E+01_JPRB, 0.10568E+01_JPRB, 0.10922E+01_JPRB, 0.11280E+01_JPRB, 0.11503E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.11951E+01_JPRB, 0.11866E+01_JPRB, 0.12198E+01_JPRB, 0.12473E+01_JPRB, 0.12027E+01_JPRB, 0.13623E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.12154E+01_JPRB, 0.11080E+01_JPRB, 0.97116E+00_JPRB, 0.93884E+00_JPRB, 0.83594E+00_JPRB, 0.78918E+00_JPRB ,&
&  0.86531E+00_JPRB, 0.90713E+00_JPRB, 0.96040E+00_JPRB, 0.10567E+01_JPRB, 0.12413E+01_JPRB, 0.15337E+01_JPRB, 0.17726E+01_JPRB ,&
&  0.20656E+01_JPRB, 0.23556E+01_JPRB, 0.26873E+01_JPRB, 0.30561E+01_JPRB, 0.34697E+01_JPRB, 0.37086E+01_JPRB, 0.36787E+01_JPRB ,&
&  0.35132E+01_JPRB, 0.32953E+01_JPRB, 0.24801E+01_JPRB, 0.28077E+01_JPRB, 0.33318E+01_JPRB, 0.32288E+01_JPRB, 0.27171E+01_JPRB ,&
&  0.24351E+01_JPRB, 0.23889E+01_JPRB, 0.26716E+01_JPRB, 0.29823E+01_JPRB, 0.31813E+01_JPRB, 0.32435E+01_JPRB, 0.30615E+01_JPRB ,&
&  0.28407E+01_JPRB, 0.24355E+01_JPRB, 0.22449E+01_JPRB, 0.22733E+01_JPRB, 0.22102E+01_JPRB, 0.20116E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.15515E+01_JPRB, 0.16510E+01_JPRB, 0.17776E+01_JPRB, 0.18322E+01_JPRB, 0.19011E+01_JPRB, 0.27003E+01_JPRB, 0.27219E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.89743E+00_JPRB, 0.55406E+00_JPRB, 0.64865E+00_JPRB, 0.67475E+00_JPRB, 0.62835E+00_JPRB, 0.67485E+00_JPRB ,&
&  0.70922E+00_JPRB, 0.67545E+00_JPRB, 0.66732E+00_JPRB, 0.64516E+00_JPRB, 0.64668E+00_JPRB, 0.64417E+00_JPRB, 0.65981E+00_JPRB ,&
&  0.66998E+00_JPRB, 0.67297E+00_JPRB, 0.68374E+00_JPRB, 0.69783E+00_JPRB, 0.70879E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.75651E+00_JPRB, 0.78354E+00_JPRB, 0.81061E+00_JPRB, 0.83535E+00_JPRB, 0.85934E+00_JPRB, 0.88205E+00_JPRB, 0.90592E+00_JPRB ,&
&  0.93032E+00_JPRB, 0.95637E+00_JPRB, 0.98163E+00_JPRB, 0.10131E+01_JPRB, 0.10436E+01_JPRB, 0.10618E+01_JPRB, 0.10937E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.11599E+01_JPRB, 0.11883E+01_JPRB, 0.12395E+01_JPRB, 0.12835E+01_JPRB, 0.13147E+01_JPRB, 0.13650E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.13977E+01_JPRB, 0.14089E+01_JPRB, 0.14492E+01_JPRB, 0.14761E+01_JPRB, 0.16068E+01_JPRB, 0.13639E+01_JPRB ,&
&  0.13672E+01_JPRB, 0.12387E+01_JPRB, 0.11463E+01_JPRB, 0.10758E+01_JPRB, 0.10172E+01_JPRB, 0.96468E+00_JPRB, 0.94471E+00_JPRB ,&
&  0.98780E+00_JPRB, 0.10330E+01_JPRB, 0.11157E+01_JPRB, 0.12404E+01_JPRB, 0.14519E+01_JPRB, 0.17721E+01_JPRB, 0.20369E+01_JPRB ,&
&  0.23343E+01_JPRB, 0.26805E+01_JPRB, 0.30099E+01_JPRB, 0.33596E+01_JPRB, 0.37521E+01_JPRB, 0.41559E+01_JPRB, 0.44915E+01_JPRB ,&
&  0.45421E+01_JPRB, 0.44568E+01_JPRB, 0.37693E+01_JPRB, 0.41822E+01_JPRB, 0.44582E+01_JPRB, 0.39570E+01_JPRB, 0.29525E+01_JPRB ,&
&  0.22897E+01_JPRB, 0.18837E+01_JPRB, 0.19580E+01_JPRB, 0.22658E+01_JPRB, 0.23870E+01_JPRB, 0.24060E+01_JPRB, 0.22806E+01_JPRB ,&
&  0.21134E+01_JPRB, 0.18738E+01_JPRB, 0.17812E+01_JPRB, 0.18237E+01_JPRB, 0.19158E+01_JPRB, 0.19401E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.17142E+01_JPRB, 0.16647E+01_JPRB, 0.16007E+01_JPRB, 0.14927E+01_JPRB, 0.14972E+01_JPRB, 0.13984E+01_JPRB, 0.15214E+01_JPRB ,&
&  0.89420E+00_JPRB, 0.70583E+00_JPRB, 0.70168E+00_JPRB, 0.65450E+00_JPRB, 0.72699E+00_JPRB, 0.75682E+00_JPRB, 0.92456E+00_JPRB ,&
&  0.10605E+01_JPRB, 0.11120E+01_JPRB, 0.10436E+01_JPRB, 0.86570E+00_JPRB, 0.72856E+00_JPRB, 0.67583E+00_JPRB, 0.67775E+00_JPRB ,&
&  0.68223E+00_JPRB, 0.68322E+00_JPRB, 0.69568E+00_JPRB, 0.71560E+00_JPRB, 0.73357E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.72599E+00_JPRB, 0.75113E+00_JPRB, 0.77895E+00_JPRB, 0.80173E+00_JPRB, 0.83047E+00_JPRB, 0.87133E+00_JPRB, 0.91179E+00_JPRB ,&
&  0.94692E+00_JPRB, 0.97355E+00_JPRB, 0.99467E+00_JPRB, 0.10243E+01_JPRB, 0.10627E+01_JPRB, 0.11165E+01_JPRB, 0.11847E+01_JPRB ,&
&  0.12225E+01_JPRB, 0.12832E+01_JPRB, 0.13259E+01_JPRB, 0.13985E+01_JPRB, 0.14422E+01_JPRB, 0.15252E+01_JPRB, 0.15897E+01_JPRB ,&
&  0.16244E+01_JPRB, 0.16422E+01_JPRB, 0.16645E+01_JPRB, 0.17082E+01_JPRB, 0.17036E+01_JPRB, 0.17831E+01_JPRB, 0.14388E+01_JPRB ,&
&  0.15053E+01_JPRB, 0.14411E+01_JPRB, 0.13325E+01_JPRB, 0.12869E+01_JPRB, 0.12488E+01_JPRB, 0.12265E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.12402E+01_JPRB, 0.12586E+01_JPRB, 0.12334E+01_JPRB, 0.13106E+01_JPRB, 0.14997E+01_JPRB, 0.18205E+01_JPRB, 0.21012E+01_JPRB ,&
&  0.24190E+01_JPRB, 0.27464E+01_JPRB, 0.30921E+01_JPRB, 0.35354E+01_JPRB, 0.41107E+01_JPRB, 0.47209E+01_JPRB, 0.53285E+01_JPRB ,&
&  0.55692E+01_JPRB, 0.56806E+01_JPRB, 0.51872E+01_JPRB, 0.52478E+01_JPRB, 0.53392E+01_JPRB, 0.43018E+01_JPRB, 0.29246E+01_JPRB ,&
&  0.17680E+01_JPRB, 0.16017E+01_JPRB, 0.17479E+01_JPRB, 0.18478E+01_JPRB, 0.17101E+01_JPRB, 0.16054E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.11974E+01_JPRB, 0.12112E+01_JPRB, 0.13300E+01_JPRB, 0.14738E+01_JPRB, 0.16402E+01_JPRB, 0.16822E+01_JPRB ,&
&  0.16384E+01_JPRB, 0.14985E+01_JPRB, 0.13617E+01_JPRB, 0.12550E+01_JPRB, 0.11529E+01_JPRB, 0.10290E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.85189E+00_JPRB, 0.80666E+00_JPRB, 0.80878E+00_JPRB, 0.58616E+00_JPRB, 0.75837E+00_JPRB, 0.94090E+00_JPRB, 0.11975E+01_JPRB ,&
&  0.13213E+01_JPRB, 0.17659E+01_JPRB, 0.21669E+01_JPRB, 0.15524E+01_JPRB, 0.11608E+01_JPRB, 0.74124E+00_JPRB, 0.67259E+00_JPRB ,&
&  0.67644E+00_JPRB, 0.66814E+00_JPRB, 0.67325E+00_JPRB, 0.68855E+00_JPRB, 0.70488E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.67347E+00_JPRB, 0.69165E+00_JPRB, 0.71717E+00_JPRB, 0.72419E+00_JPRB, 0.73556E+00_JPRB, 0.75661E+00_JPRB, 0.77841E+00_JPRB ,&
&  0.79957E+00_JPRB, 0.81697E+00_JPRB, 0.83356E+00_JPRB, 0.85582E+00_JPRB, 0.88439E+00_JPRB, 0.95179E+00_JPRB, 0.10493E+01_JPRB ,&
&  0.11381E+01_JPRB, 0.12245E+01_JPRB, 0.13322E+01_JPRB, 0.13897E+01_JPRB, 0.14772E+01_JPRB, 0.15787E+01_JPRB, 0.16514E+01_JPRB ,&
&  0.17391E+01_JPRB, 0.17847E+01_JPRB, 0.18294E+01_JPRB, 0.19329E+01_JPRB, 0.18956E+01_JPRB, 0.19457E+01_JPRB, 0.15015E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.15572E+01_JPRB, 0.14937E+01_JPRB, 0.14655E+01_JPRB, 0.14243E+01_JPRB, 0.13605E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.12927E+01_JPRB, 0.12218E+01_JPRB, 0.11469E+01_JPRB, 0.12213E+01_JPRB, 0.13592E+01_JPRB, 0.16159E+01_JPRB, 0.19498E+01_JPRB ,&
&  0.22970E+01_JPRB, 0.26208E+01_JPRB, 0.29643E+01_JPRB, 0.34584E+01_JPRB, 0.41084E+01_JPRB, 0.47980E+01_JPRB, 0.55254E+01_JPRB ,&
&  0.60296E+01_JPRB, 0.64111E+01_JPRB, 0.65069E+01_JPRB, 0.54938E+01_JPRB, 0.51688E+01_JPRB, 0.38607E+01_JPRB, 0.24412E+01_JPRB ,&
&  0.17389E+01_JPRB, 0.15331E+01_JPRB, 0.15409E+01_JPRB, 0.15046E+01_JPRB, 0.12795E+01_JPRB, 0.11443E+01_JPRB, 0.99755E+00_JPRB ,&
&  0.88558E+00_JPRB, 0.85988E+00_JPRB, 0.90627E+00_JPRB, 0.10356E+01_JPRB, 0.11729E+01_JPRB, 0.13401E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.12087E+01_JPRB, 0.11111E+01_JPRB, 0.10378E+01_JPRB, 0.10216E+01_JPRB, 0.10621E+01_JPRB, 0.94043E+00_JPRB ,&
&  0.85139E+00_JPRB, 0.71245E+00_JPRB, 0.69048E+00_JPRB, 0.74739E+00_JPRB, 0.80163E+00_JPRB, 0.10401E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.20728E+01_JPRB, 0.21908E+01_JPRB, 0.26493E+01_JPRB, 0.35097E+01_JPRB, 0.25814E+01_JPRB, 0.18367E+01_JPRB, 0.60629E+00_JPRB ,&
&  0.62454E+00_JPRB, 0.62448E+00_JPRB, 0.62871E+00_JPRB, 0.63324E+00_JPRB, 0.64750E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.56298E+00_JPRB, 0.59018E+00_JPRB, 0.58246E+00_JPRB, 0.58705E+00_JPRB, 0.59475E+00_JPRB, 0.60792E+00_JPRB, 0.63499E+00_JPRB ,&
&  0.64404E+00_JPRB, 0.68527E+00_JPRB, 0.73133E+00_JPRB, 0.77069E+00_JPRB, 0.82171E+00_JPRB, 0.89282E+00_JPRB, 0.98498E+00_JPRB ,&
&  0.10869E+01_JPRB, 0.11513E+01_JPRB, 0.12149E+01_JPRB, 0.12664E+01_JPRB, 0.13496E+01_JPRB, 0.14550E+01_JPRB, 0.15272E+01_JPRB ,&
&  0.16445E+01_JPRB, 0.17306E+01_JPRB, 0.18294E+01_JPRB, 0.19676E+01_JPRB, 0.18899E+01_JPRB, 0.18716E+01_JPRB, 0.13384E+01_JPRB ,&
&  0.15376E+01_JPRB, 0.15674E+01_JPRB, 0.15525E+01_JPRB, 0.15254E+01_JPRB, 0.14606E+01_JPRB, 0.13538E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.11703E+01_JPRB, 0.10965E+01_JPRB, 0.10644E+01_JPRB, 0.12072E+01_JPRB, 0.13558E+01_JPRB, 0.16590E+01_JPRB ,&
&  0.20186E+01_JPRB, 0.24184E+01_JPRB, 0.28079E+01_JPRB, 0.32830E+01_JPRB, 0.39058E+01_JPRB, 0.46211E+01_JPRB, 0.53491E+01_JPRB ,&
&  0.58973E+01_JPRB, 0.63553E+01_JPRB, 0.62918E+01_JPRB, 0.51985E+01_JPRB, 0.45601E+01_JPRB, 0.33468E+01_JPRB, 0.20849E+01_JPRB ,&
&  0.16161E+01_JPRB, 0.14822E+01_JPRB, 0.14795E+01_JPRB, 0.13630E+01_JPRB, 0.11103E+01_JPRB, 0.94726E+00_JPRB, 0.80774E+00_JPRB ,&
&  0.70207E+00_JPRB, 0.69774E+00_JPRB, 0.78228E+00_JPRB, 0.89948E+00_JPRB, 0.10039E+01_JPRB, 0.11119E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.10424E+01_JPRB, 0.93895E+00_JPRB, 0.87593E+00_JPRB, 0.89336E+00_JPRB, 0.10254E+01_JPRB, 0.11760E+01_JPRB, 0.12739E+01_JPRB ,&
&  0.91053E+00_JPRB, 0.70902E+00_JPRB, 0.69255E+00_JPRB, 0.71093E+00_JPRB, 0.75182E+00_JPRB, 0.82760E+00_JPRB, 0.94965E+00_JPRB ,&
&  0.11296E+01_JPRB, 0.15102E+01_JPRB, 0.14555E+01_JPRB, 0.11896E+01_JPRB, 0.16833E+01_JPRB, 0.14097E+01_JPRB, 0.55209E+00_JPRB ,&
&  0.53380E+00_JPRB, 0.52397E+00_JPRB, 0.53332E+00_JPRB, 0.54004E+00_JPRB, 0.55646E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.52705E+00_JPRB, 0.51584E+00_JPRB, 0.49789E+00_JPRB, 0.50830E+00_JPRB, 0.51059E+00_JPRB, 0.53103E+00_JPRB, 0.55465E+00_JPRB ,&
&  0.58786E+00_JPRB, 0.65116E+00_JPRB, 0.71672E+00_JPRB, 0.77352E+00_JPRB, 0.81399E+00_JPRB, 0.86133E+00_JPRB, 0.90582E+00_JPRB ,&
&  0.94248E+00_JPRB, 0.97571E+00_JPRB, 0.10229E+01_JPRB, 0.10862E+01_JPRB, 0.11679E+01_JPRB, 0.12628E+01_JPRB, 0.13772E+01_JPRB ,&
&  0.15044E+01_JPRB, 0.16396E+01_JPRB, 0.18087E+01_JPRB, 0.18861E+01_JPRB, 0.19436E+01_JPRB, 0.20017E+01_JPRB, 0.19924E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.16399E+01_JPRB, 0.15900E+01_JPRB, 0.15229E+01_JPRB, 0.14654E+01_JPRB, 0.13704E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.11380E+01_JPRB, 0.10948E+01_JPRB, 0.10069E+01_JPRB, 0.10542E+01_JPRB, 0.11503E+01_JPRB, 0.13431E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.20732E+01_JPRB, 0.25917E+01_JPRB, 0.30860E+01_JPRB, 0.35781E+01_JPRB, 0.41258E+01_JPRB, 0.48975E+01_JPRB ,&
&  0.57383E+01_JPRB, 0.63235E+01_JPRB, 0.66794E+01_JPRB, 0.57591E+01_JPRB, 0.45080E+01_JPRB, 0.38328E+01_JPRB, 0.27439E+01_JPRB ,&
&  0.16953E+01_JPRB, 0.13400E+01_JPRB, 0.13621E+01_JPRB, 0.13269E+01_JPRB, 0.10995E+01_JPRB, 0.94665E+00_JPRB, 0.80126E+00_JPRB ,&
&  0.68193E+00_JPRB, 0.62246E+00_JPRB, 0.65067E+00_JPRB, 0.77459E+00_JPRB, 0.84464E+00_JPRB, 0.87846E+00_JPRB, 0.85239E+00_JPRB ,&
&  0.82053E+00_JPRB, 0.75294E+00_JPRB, 0.68314E+00_JPRB, 0.67991E+00_JPRB, 0.72421E+00_JPRB, 0.76502E+00_JPRB, 0.76021E+00_JPRB ,&
&  0.73493E+00_JPRB, 0.72429E+00_JPRB, 0.72477E+00_JPRB, 0.73928E+00_JPRB, 0.74845E+00_JPRB, 0.76514E+00_JPRB, 0.78643E+00_JPRB ,&
&  0.81270E+00_JPRB, 0.85123E+00_JPRB, 0.82914E+00_JPRB, 0.80505E+00_JPRB, 0.86963E+00_JPRB, 0.91914E+00_JPRB, 0.60975E+00_JPRB ,&
&  0.52758E+00_JPRB, 0.49609E+00_JPRB, 0.50099E+00_JPRB, 0.49624E+00_JPRB, 0.53697E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.53840E+00_JPRB, 0.51821E+00_JPRB, 0.50720E+00_JPRB, 0.49905E+00_JPRB, 0.48463E+00_JPRB, 0.51141E+00_JPRB, 0.51966E+00_JPRB ,&
&  0.54886E+00_JPRB, 0.59685E+00_JPRB, 0.65713E+00_JPRB, 0.69810E+00_JPRB, 0.71714E+00_JPRB, 0.75464E+00_JPRB, 0.78682E+00_JPRB ,&
&  0.80917E+00_JPRB, 0.82596E+00_JPRB, 0.84965E+00_JPRB, 0.88035E+00_JPRB, 0.93319E+00_JPRB, 0.10136E+01_JPRB, 0.11336E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.14231E+01_JPRB, 0.15861E+01_JPRB, 0.17412E+01_JPRB, 0.18534E+01_JPRB, 0.19423E+01_JPRB, 0.19032E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.15078E+01_JPRB, 0.17348E+01_JPRB, 0.16071E+01_JPRB, 0.14978E+01_JPRB, 0.13938E+01_JPRB, 0.12884E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.10939E+01_JPRB, 0.10557E+01_JPRB, 0.10562E+01_JPRB, 0.96440E+00_JPRB, 0.10014E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.16088E+01_JPRB, 0.21841E+01_JPRB, 0.28039E+01_JPRB, 0.31503E+01_JPRB, 0.34953E+01_JPRB, 0.41311E+01_JPRB ,&
&  0.51099E+01_JPRB, 0.58738E+01_JPRB, 0.64278E+01_JPRB, 0.48124E+01_JPRB, 0.40420E+01_JPRB, 0.38948E+01_JPRB, 0.34770E+01_JPRB ,&
&  0.20124E+01_JPRB, 0.13609E+01_JPRB, 0.13752E+01_JPRB, 0.13703E+01_JPRB, 0.11565E+01_JPRB, 0.10334E+01_JPRB, 0.89759E+00_JPRB ,&
&  0.74570E+00_JPRB, 0.65086E+00_JPRB, 0.58844E+00_JPRB, 0.62511E+00_JPRB, 0.66573E+00_JPRB, 0.68767E+00_JPRB, 0.65022E+00_JPRB ,&
&  0.67880E+00_JPRB, 0.65302E+00_JPRB, 0.63439E+00_JPRB, 0.63838E+00_JPRB, 0.70063E+00_JPRB, 0.72943E+00_JPRB, 0.74219E+00_JPRB ,&
&  0.76064E+00_JPRB, 0.77553E+00_JPRB, 0.77713E+00_JPRB, 0.78502E+00_JPRB, 0.79107E+00_JPRB, 0.78788E+00_JPRB, 0.78236E+00_JPRB ,&
&  0.76802E+00_JPRB, 0.76082E+00_JPRB, 0.73250E+00_JPRB, 0.70054E+00_JPRB, 0.65564E+00_JPRB, 0.66248E+00_JPRB, 0.59904E+00_JPRB ,&
&  0.55641E+00_JPRB, 0.52824E+00_JPRB, 0.50991E+00_JPRB, 0.50665E+00_JPRB, 0.51681E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.55797E+00_JPRB, 0.55275E+00_JPRB, 0.54576E+00_JPRB, 0.53440E+00_JPRB, 0.52018E+00_JPRB, 0.51912E+00_JPRB, 0.50524E+00_JPRB ,&
&  0.50421E+00_JPRB, 0.53009E+00_JPRB, 0.57147E+00_JPRB, 0.60476E+00_JPRB, 0.63537E+00_JPRB, 0.65694E+00_JPRB, 0.68489E+00_JPRB ,&
&  0.69470E+00_JPRB, 0.69607E+00_JPRB, 0.69333E+00_JPRB, 0.69305E+00_JPRB, 0.70599E+00_JPRB, 0.75024E+00_JPRB, 0.82706E+00_JPRB ,&
&  0.93661E+00_JPRB, 0.10811E+01_JPRB, 0.12613E+01_JPRB, 0.14611E+01_JPRB, 0.16647E+01_JPRB, 0.18664E+01_JPRB, 0.19680E+01_JPRB ,&
&  0.12957E+01_JPRB, 0.68097E+00_JPRB, 0.15976E+01_JPRB, 0.17754E+01_JPRB, 0.16270E+01_JPRB, 0.15360E+01_JPRB, 0.13705E+01_JPRB ,&
&  0.12129E+01_JPRB, 0.11529E+01_JPRB, 0.10721E+01_JPRB, 0.10179E+01_JPRB, 0.93200E+00_JPRB, 0.92904E+00_JPRB, 0.95329E+00_JPRB ,&
&  0.10473E+01_JPRB, 0.12512E+01_JPRB, 0.16587E+01_JPRB, 0.23391E+01_JPRB, 0.27973E+01_JPRB, 0.29803E+01_JPRB, 0.33883E+01_JPRB ,&
&  0.42157E+01_JPRB, 0.49426E+01_JPRB, 0.52455E+01_JPRB, 0.34917E+01_JPRB, 0.23618E+01_JPRB, 0.20735E+01_JPRB, 0.19079E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.14406E+01_JPRB, 0.14098E+01_JPRB, 0.14327E+01_JPRB, 0.12761E+01_JPRB, 0.10983E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.90939E+00_JPRB, 0.78946E+00_JPRB, 0.69146E+00_JPRB, 0.64304E+00_JPRB, 0.61764E+00_JPRB, 0.62498E+00_JPRB, 0.61861E+00_JPRB ,&
&  0.61558E+00_JPRB, 0.63001E+00_JPRB, 0.63321E+00_JPRB, 0.64701E+00_JPRB, 0.68915E+00_JPRB, 0.73051E+00_JPRB, 0.75694E+00_JPRB ,&
&  0.77757E+00_JPRB, 0.80053E+00_JPRB, 0.81408E+00_JPRB, 0.81896E+00_JPRB, 0.81940E+00_JPRB, 0.80274E+00_JPRB, 0.80699E+00_JPRB ,&
&  0.81216E+00_JPRB, 0.81349E+00_JPRB, 0.78498E+00_JPRB, 0.76210E+00_JPRB, 0.70759E+00_JPRB, 0.66871E+00_JPRB, 0.60405E+00_JPRB ,&
&  0.57842E+00_JPRB, 0.53961E+00_JPRB, 0.51336E+00_JPRB, 0.51097E+00_JPRB, 0.52551E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.56892E+00_JPRB, 0.56939E+00_JPRB, 0.57113E+00_JPRB, 0.57455E+00_JPRB, 0.57102E+00_JPRB, 0.57186E+00_JPRB, 0.55074E+00_JPRB ,&
&  0.51989E+00_JPRB, 0.50878E+00_JPRB, 0.52917E+00_JPRB, 0.53430E+00_JPRB, 0.53656E+00_JPRB, 0.55988E+00_JPRB, 0.58519E+00_JPRB ,&
&  0.59917E+00_JPRB, 0.60603E+00_JPRB, 0.60552E+00_JPRB, 0.60447E+00_JPRB, 0.59474E+00_JPRB, 0.59756E+00_JPRB, 0.62530E+00_JPRB ,&
&  0.67392E+00_JPRB, 0.76258E+00_JPRB, 0.89580E+00_JPRB, 0.10636E+01_JPRB, 0.12582E+01_JPRB, 0.15337E+01_JPRB, 0.17943E+01_JPRB ,&
&  0.22760E+01_JPRB, 0.15413E+01_JPRB, 0.91627E+00_JPRB, 0.16075E+01_JPRB, 0.18081E+01_JPRB, 0.17931E+01_JPRB, 0.15064E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.13419E+01_JPRB, 0.12657E+01_JPRB, 0.11531E+01_JPRB, 0.99216E+00_JPRB, 0.92976E+00_JPRB, 0.88049E+00_JPRB ,&
&  0.87958E+00_JPRB, 0.97062E+00_JPRB, 0.11979E+01_JPRB, 0.16676E+01_JPRB, 0.22679E+01_JPRB, 0.25824E+01_JPRB, 0.27639E+01_JPRB ,&
&  0.33253E+01_JPRB, 0.40637E+01_JPRB, 0.42449E+01_JPRB, 0.27770E+01_JPRB, 0.19811E+01_JPRB, 0.17747E+01_JPRB, 0.17273E+01_JPRB ,&
&  0.16406E+01_JPRB, 0.16351E+01_JPRB, 0.16600E+01_JPRB, 0.15128E+01_JPRB, 0.13375E+01_JPRB, 0.12071E+01_JPRB, 0.12005E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.93767E+00_JPRB, 0.83582E+00_JPRB, 0.73519E+00_JPRB, 0.65418E+00_JPRB, 0.63659E+00_JPRB, 0.62596E+00_JPRB ,&
&  0.63018E+00_JPRB, 0.65053E+00_JPRB, 0.65296E+00_JPRB, 0.67427E+00_JPRB, 0.69668E+00_JPRB, 0.71990E+00_JPRB, 0.74225E+00_JPRB ,&
&  0.76234E+00_JPRB, 0.78598E+00_JPRB, 0.80889E+00_JPRB, 0.81412E+00_JPRB, 0.80665E+00_JPRB, 0.79465E+00_JPRB, 0.80906E+00_JPRB ,&
&  0.81826E+00_JPRB, 0.83418E+00_JPRB, 0.82719E+00_JPRB, 0.82409E+00_JPRB, 0.78066E+00_JPRB, 0.73941E+00_JPRB, 0.68673E+00_JPRB ,&
&  0.67290E+00_JPRB, 0.63086E+00_JPRB, 0.59700E+00_JPRB, 0.55566E+00_JPRB, 0.55946E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.59119E+00_JPRB, 0.58348E+00_JPRB, 0.58394E+00_JPRB, 0.58771E+00_JPRB, 0.58859E+00_JPRB, 0.59183E+00_JPRB, 0.59242E+00_JPRB ,&
&  0.56173E+00_JPRB, 0.53236E+00_JPRB, 0.51459E+00_JPRB, 0.50377E+00_JPRB, 0.49905E+00_JPRB, 0.51328E+00_JPRB, 0.52071E+00_JPRB ,&
&  0.51932E+00_JPRB, 0.52080E+00_JPRB, 0.54413E+00_JPRB, 0.55090E+00_JPRB, 0.54916E+00_JPRB, 0.54297E+00_JPRB, 0.54240E+00_JPRB ,&
&  0.55679E+00_JPRB, 0.59142E+00_JPRB, 0.66110E+00_JPRB, 0.75834E+00_JPRB, 0.86516E+00_JPRB, 0.10501E+01_JPRB, 0.12834E+01_JPRB ,&
&  0.16828E+01_JPRB, 0.17259E+01_JPRB, 0.81493E+00_JPRB, 0.13976E+01_JPRB, 0.18865E+01_JPRB, 0.19400E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.16675E+01_JPRB, 0.15529E+01_JPRB, 0.15961E+01_JPRB, 0.14649E+01_JPRB, 0.11697E+01_JPRB, 0.99046E+00_JPRB, 0.86812E+00_JPRB ,&
&  0.80433E+00_JPRB, 0.80584E+00_JPRB, 0.89172E+00_JPRB, 0.11232E+01_JPRB, 0.15909E+01_JPRB, 0.20268E+01_JPRB, 0.22445E+01_JPRB ,&
&  0.25104E+01_JPRB, 0.31980E+01_JPRB, 0.36809E+01_JPRB, 0.25109E+01_JPRB, 0.19819E+01_JPRB, 0.20141E+01_JPRB, 0.19850E+01_JPRB ,&
&  0.18772E+01_JPRB, 0.18159E+01_JPRB, 0.17608E+01_JPRB, 0.16109E+01_JPRB, 0.14211E+01_JPRB, 0.13340E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.10776E+01_JPRB, 0.96839E+00_JPRB, 0.86545E+00_JPRB, 0.75753E+00_JPRB, 0.68046E+00_JPRB, 0.67052E+00_JPRB ,&
&  0.67026E+00_JPRB, 0.69000E+00_JPRB, 0.69120E+00_JPRB, 0.70407E+00_JPRB, 0.71791E+00_JPRB, 0.73145E+00_JPRB, 0.74089E+00_JPRB ,&
&  0.75222E+00_JPRB, 0.76895E+00_JPRB, 0.78906E+00_JPRB, 0.79198E+00_JPRB, 0.79080E+00_JPRB, 0.78720E+00_JPRB, 0.79087E+00_JPRB ,&
&  0.80112E+00_JPRB, 0.81342E+00_JPRB, 0.81609E+00_JPRB, 0.81709E+00_JPRB, 0.83163E+00_JPRB, 0.81623E+00_JPRB, 0.78866E+00_JPRB ,&
&  0.82250E+00_JPRB, 0.85382E+00_JPRB, 0.78943E+00_JPRB, 0.64142E+00_JPRB, 0.60917E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.63212E+00_JPRB, 0.61846E+00_JPRB, 0.60912E+00_JPRB, 0.60813E+00_JPRB, 0.60605E+00_JPRB, 0.60660E+00_JPRB, 0.61580E+00_JPRB ,&
&  0.60505E+00_JPRB, 0.58442E+00_JPRB, 0.55868E+00_JPRB, 0.53130E+00_JPRB, 0.51051E+00_JPRB, 0.51140E+00_JPRB, 0.51747E+00_JPRB ,&
&  0.51858E+00_JPRB, 0.51491E+00_JPRB, 0.51577E+00_JPRB, 0.52288E+00_JPRB, 0.52184E+00_JPRB, 0.51714E+00_JPRB, 0.51196E+00_JPRB ,&
&  0.51083E+00_JPRB, 0.51872E+00_JPRB, 0.54598E+00_JPRB, 0.59400E+00_JPRB, 0.65285E+00_JPRB, 0.74801E+00_JPRB, 0.88721E+00_JPRB ,&
&  0.12015E+01_JPRB, 0.14031E+01_JPRB, 0.15048E+01_JPRB, 0.22542E+01_JPRB, 0.23709E+01_JPRB, 0.22204E+01_JPRB, 0.21677E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.19567E+01_JPRB, 0.20790E+01_JPRB, 0.16819E+01_JPRB, 0.13538E+01_JPRB, 0.10837E+01_JPRB, 0.90853E+00_JPRB ,&
&  0.81138E+00_JPRB, 0.75349E+00_JPRB, 0.75169E+00_JPRB, 0.81382E+00_JPRB, 0.10464E+01_JPRB, 0.14548E+01_JPRB, 0.17794E+01_JPRB ,&
&  0.20155E+01_JPRB, 0.24708E+01_JPRB, 0.31514E+01_JPRB, 0.26436E+01_JPRB, 0.21200E+01_JPRB, 0.24422E+01_JPRB, 0.27662E+01_JPRB ,&
&  0.31305E+01_JPRB, 0.22775E+01_JPRB, 0.19489E+01_JPRB, 0.17486E+01_JPRB, 0.15081E+01_JPRB, 0.16291E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.13086E+01_JPRB, 0.12071E+01_JPRB, 0.11004E+01_JPRB, 0.99765E+00_JPRB, 0.89825E+00_JPRB, 0.80663E+00_JPRB, 0.75711E+00_JPRB ,&
&  0.74753E+00_JPRB, 0.75091E+00_JPRB, 0.74645E+00_JPRB, 0.75337E+00_JPRB, 0.75402E+00_JPRB, 0.76247E+00_JPRB, 0.76869E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.77608E+00_JPRB, 0.76155E+00_JPRB, 0.76710E+00_JPRB, 0.77107E+00_JPRB, 0.76821E+00_JPRB, 0.76145E+00_JPRB ,&
&  0.76831E+00_JPRB, 0.78332E+00_JPRB, 0.79256E+00_JPRB, 0.79977E+00_JPRB, 0.81208E+00_JPRB, 0.80624E+00_JPRB, 0.78150E+00_JPRB ,&
&  0.76750E+00_JPRB, 0.76928E+00_JPRB, 0.75817E+00_JPRB, 0.68983E+00_JPRB, 0.65649E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.65555E+00_JPRB, 0.64264E+00_JPRB, 0.63386E+00_JPRB, 0.62470E+00_JPRB, 0.61427E+00_JPRB, 0.61312E+00_JPRB, 0.62137E+00_JPRB ,&
&  0.62082E+00_JPRB, 0.60814E+00_JPRB, 0.58901E+00_JPRB, 0.56955E+00_JPRB, 0.54497E+00_JPRB, 0.53126E+00_JPRB, 0.52274E+00_JPRB ,&
&  0.51684E+00_JPRB, 0.51500E+00_JPRB, 0.51013E+00_JPRB, 0.50901E+00_JPRB, 0.50763E+00_JPRB, 0.49712E+00_JPRB, 0.48990E+00_JPRB ,&
&  0.48580E+00_JPRB, 0.48991E+00_JPRB, 0.50179E+00_JPRB, 0.52723E+00_JPRB, 0.56542E+00_JPRB, 0.62793E+00_JPRB, 0.70789E+00_JPRB ,&
&  0.90899E+00_JPRB, 0.12172E+01_JPRB, 0.11520E+01_JPRB, 0.23901E+01_JPRB, 0.23626E+01_JPRB, 0.23060E+01_JPRB, 0.21656E+01_JPRB ,&
&  0.21492E+01_JPRB, 0.23688E+01_JPRB, 0.21019E+01_JPRB, 0.17626E+01_JPRB, 0.14063E+01_JPRB, 0.11488E+01_JPRB, 0.95677E+00_JPRB ,&
&  0.83765E+00_JPRB, 0.76713E+00_JPRB, 0.73455E+00_JPRB, 0.72265E+00_JPRB, 0.75298E+00_JPRB, 0.98234E+00_JPRB, 0.13465E+01_JPRB ,&
&  0.16208E+01_JPRB, 0.19500E+01_JPRB, 0.25744E+01_JPRB, 0.24665E+01_JPRB, 0.23001E+01_JPRB, 0.27524E+01_JPRB, 0.36262E+01_JPRB ,&
&  0.43850E+01_JPRB, 0.32822E+01_JPRB, 0.23699E+01_JPRB, 0.19933E+01_JPRB, 0.17900E+01_JPRB, 0.17372E+01_JPRB, 0.15707E+01_JPRB ,&
&  0.13914E+01_JPRB, 0.12932E+01_JPRB, 0.12088E+01_JPRB, 0.11244E+01_JPRB, 0.10328E+01_JPRB, 0.94979E+00_JPRB, 0.87435E+00_JPRB ,&
&  0.83398E+00_JPRB, 0.81811E+00_JPRB, 0.80969E+00_JPRB, 0.81617E+00_JPRB, 0.82017E+00_JPRB, 0.83084E+00_JPRB, 0.83066E+00_JPRB ,&
&  0.83140E+00_JPRB, 0.81497E+00_JPRB, 0.76692E+00_JPRB, 0.75443E+00_JPRB, 0.75570E+00_JPRB, 0.75079E+00_JPRB, 0.74052E+00_JPRB ,&
&  0.75113E+00_JPRB, 0.76242E+00_JPRB, 0.76293E+00_JPRB, 0.76307E+00_JPRB, 0.78312E+00_JPRB, 0.79342E+00_JPRB, 0.78312E+00_JPRB ,&
&  0.75796E+00_JPRB, 0.73563E+00_JPRB, 0.71454E+00_JPRB, 0.68535E+00_JPRB, 0.67201E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.66433E+00_JPRB, 0.65468E+00_JPRB, 0.64538E+00_JPRB, 0.63323E+00_JPRB, 0.61688E+00_JPRB, 0.60974E+00_JPRB, 0.61513E+00_JPRB ,&
&  0.62192E+00_JPRB, 0.61461E+00_JPRB, 0.60076E+00_JPRB, 0.58166E+00_JPRB, 0.55748E+00_JPRB, 0.54414E+00_JPRB, 0.53486E+00_JPRB ,&
&  0.52120E+00_JPRB, 0.51028E+00_JPRB, 0.49633E+00_JPRB, 0.49097E+00_JPRB, 0.49075E+00_JPRB, 0.48349E+00_JPRB, 0.46920E+00_JPRB ,&
&  0.47045E+00_JPRB, 0.47929E+00_JPRB, 0.49043E+00_JPRB, 0.50385E+00_JPRB, 0.53183E+00_JPRB, 0.58003E+00_JPRB, 0.65313E+00_JPRB ,&
&  0.80118E+00_JPRB, 0.13944E+01_JPRB, 0.10727E+01_JPRB, 0.18653E+01_JPRB, 0.19132E+01_JPRB, 0.20441E+01_JPRB, 0.20819E+01_JPRB ,&
&  0.21305E+01_JPRB, 0.21687E+01_JPRB, 0.20180E+01_JPRB, 0.16678E+01_JPRB, 0.13265E+01_JPRB, 0.11082E+01_JPRB, 0.97062E+00_JPRB ,&
&  0.87975E+00_JPRB, 0.81025E+00_JPRB, 0.76085E+00_JPRB, 0.72790E+00_JPRB, 0.70639E+00_JPRB, 0.73586E+00_JPRB, 0.92553E+00_JPRB ,&
&  0.12437E+01_JPRB, 0.15334E+01_JPRB, 0.20029E+01_JPRB, 0.25060E+01_JPRB, 0.23103E+01_JPRB, 0.26281E+01_JPRB, 0.39311E+01_JPRB ,&
&  0.47536E+01_JPRB, 0.47353E+01_JPRB, 0.31244E+01_JPRB, 0.24111E+01_JPRB, 0.21122E+01_JPRB, 0.19380E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.14430E+01_JPRB, 0.13262E+01_JPRB, 0.12634E+01_JPRB, 0.12082E+01_JPRB, 0.11380E+01_JPRB, 0.10415E+01_JPRB, 0.94916E+00_JPRB ,&
&  0.87338E+00_JPRB, 0.84949E+00_JPRB, 0.84522E+00_JPRB, 0.85158E+00_JPRB, 0.86726E+00_JPRB, 0.88421E+00_JPRB, 0.89487E+00_JPRB ,&
&  0.88980E+00_JPRB, 0.87314E+00_JPRB, 0.80478E+00_JPRB, 0.77126E+00_JPRB, 0.76200E+00_JPRB, 0.75311E+00_JPRB, 0.74104E+00_JPRB ,&
&  0.74778E+00_JPRB, 0.75165E+00_JPRB, 0.74723E+00_JPRB, 0.75167E+00_JPRB, 0.76585E+00_JPRB, 0.78724E+00_JPRB, 0.75776E+00_JPRB ,&
&  0.75519E+00_JPRB, 0.73715E+00_JPRB, 0.71557E+00_JPRB, 0.69025E+00_JPRB, 0.67786E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.66340E+00_JPRB, 0.65515E+00_JPRB, 0.64589E+00_JPRB, 0.63516E+00_JPRB, 0.62270E+00_JPRB, 0.61408E+00_JPRB, 0.61214E+00_JPRB ,&
&  0.61226E+00_JPRB, 0.61308E+00_JPRB, 0.60469E+00_JPRB, 0.58440E+00_JPRB, 0.56299E+00_JPRB, 0.54380E+00_JPRB, 0.53272E+00_JPRB ,&
&  0.52345E+00_JPRB, 0.51178E+00_JPRB, 0.49785E+00_JPRB, 0.48452E+00_JPRB, 0.48055E+00_JPRB, 0.48074E+00_JPRB, 0.47262E+00_JPRB ,&
&  0.47797E+00_JPRB, 0.48617E+00_JPRB, 0.49348E+00_JPRB, 0.50090E+00_JPRB, 0.52353E+00_JPRB, 0.56550E+00_JPRB, 0.63364E+00_JPRB ,&
&  0.76478E+00_JPRB, 0.66477E+00_JPRB, 0.11948E+01_JPRB, 0.14295E+01_JPRB, 0.15534E+01_JPRB, 0.16666E+01_JPRB, 0.19078E+01_JPRB ,&
&  0.20961E+01_JPRB, 0.20263E+01_JPRB, 0.18209E+01_JPRB, 0.15359E+01_JPRB, 0.12790E+01_JPRB, 0.10913E+01_JPRB, 0.96813E+00_JPRB ,&
&  0.89089E+00_JPRB, 0.83188E+00_JPRB, 0.77967E+00_JPRB, 0.73519E+00_JPRB, 0.70546E+00_JPRB, 0.69101E+00_JPRB, 0.71719E+00_JPRB ,&
&  0.84533E+00_JPRB, 0.10812E+01_JPRB, 0.14039E+01_JPRB, 0.19739E+01_JPRB, 0.22833E+01_JPRB, 0.24956E+01_JPRB, 0.29924E+01_JPRB ,&
&  0.40384E+01_JPRB, 0.46751E+01_JPRB, 0.36604E+01_JPRB, 0.28505E+01_JPRB, 0.24127E+01_JPRB, 0.20485E+01_JPRB, 0.17357E+01_JPRB ,&
&  0.14927E+01_JPRB, 0.13454E+01_JPRB, 0.12592E+01_JPRB, 0.12079E+01_JPRB, 0.11732E+01_JPRB, 0.11217E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.98427E+00_JPRB, 0.93948E+00_JPRB, 0.92111E+00_JPRB, 0.91881E+00_JPRB, 0.92195E+00_JPRB, 0.93534E+00_JPRB, 0.93641E+00_JPRB ,&
&  0.91947E+00_JPRB, 0.88975E+00_JPRB, 0.84006E+00_JPRB, 0.80005E+00_JPRB, 0.78545E+00_JPRB, 0.76975E+00_JPRB, 0.75068E+00_JPRB ,&
&  0.74199E+00_JPRB, 0.74255E+00_JPRB, 0.74364E+00_JPRB, 0.75284E+00_JPRB, 0.75207E+00_JPRB, 0.77451E+00_JPRB, 0.74264E+00_JPRB ,&
&  0.73463E+00_JPRB, 0.71694E+00_JPRB, 0.69940E+00_JPRB, 0.68185E+00_JPRB, 0.67553E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.66490E+00_JPRB, 0.65618E+00_JPRB, 0.64499E+00_JPRB, 0.63291E+00_JPRB, 0.61965E+00_JPRB, 0.60951E+00_JPRB, 0.60493E+00_JPRB ,&
&  0.60215E+00_JPRB, 0.60140E+00_JPRB, 0.60093E+00_JPRB, 0.58655E+00_JPRB, 0.56805E+00_JPRB, 0.55100E+00_JPRB, 0.54067E+00_JPRB ,&
&  0.53361E+00_JPRB, 0.52556E+00_JPRB, 0.51508E+00_JPRB, 0.50174E+00_JPRB, 0.49478E+00_JPRB, 0.49003E+00_JPRB, 0.48647E+00_JPRB ,&
&  0.49133E+00_JPRB, 0.49535E+00_JPRB, 0.50069E+00_JPRB, 0.50912E+00_JPRB, 0.52869E+00_JPRB, 0.56660E+00_JPRB, 0.62964E+00_JPRB ,&
&  0.76409E+00_JPRB, 0.10733E+01_JPRB, 0.13602E+01_JPRB, 0.12671E+01_JPRB, 0.12713E+01_JPRB, 0.13135E+01_JPRB, 0.15339E+01_JPRB ,&
&  0.16953E+01_JPRB, 0.17447E+01_JPRB, 0.15917E+01_JPRB, 0.13892E+01_JPRB, 0.12108E+01_JPRB, 0.10714E+01_JPRB, 0.97064E+00_JPRB ,&
&  0.87875E+00_JPRB, 0.79819E+00_JPRB, 0.74013E+00_JPRB, 0.69949E+00_JPRB, 0.67313E+00_JPRB, 0.65703E+00_JPRB, 0.65203E+00_JPRB ,&
&  0.67222E+00_JPRB, 0.75173E+00_JPRB, 0.93774E+00_JPRB, 0.13218E+01_JPRB, 0.21630E+01_JPRB, 0.21868E+01_JPRB, 0.27559E+01_JPRB ,&
&  0.38043E+01_JPRB, 0.37146E+01_JPRB, 0.33184E+01_JPRB, 0.28198E+01_JPRB, 0.24038E+01_JPRB, 0.20621E+01_JPRB, 0.17845E+01_JPRB ,&
&  0.15812E+01_JPRB, 0.14344E+01_JPRB, 0.13106E+01_JPRB, 0.12187E+01_JPRB, 0.11582E+01_JPRB, 0.11306E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.10342E+01_JPRB, 0.10063E+01_JPRB, 0.99224E+00_JPRB, 0.97867E+00_JPRB, 0.96789E+00_JPRB, 0.93757E+00_JPRB ,&
&  0.90244E+00_JPRB, 0.86579E+00_JPRB, 0.83195E+00_JPRB, 0.79817E+00_JPRB, 0.78872E+00_JPRB, 0.77614E+00_JPRB, 0.75260E+00_JPRB ,&
&  0.73042E+00_JPRB, 0.72492E+00_JPRB, 0.73794E+00_JPRB, 0.74182E+00_JPRB, 0.74990E+00_JPRB, 0.77175E+00_JPRB, 0.73910E+00_JPRB ,&
&  0.72402E+00_JPRB, 0.69908E+00_JPRB, 0.67815E+00_JPRB, 0.66989E+00_JPRB, 0.67514E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.65642E+00_JPRB, 0.64461E+00_JPRB, 0.63310E+00_JPRB, 0.62123E+00_JPRB, 0.60727E+00_JPRB, 0.59737E+00_JPRB, 0.58913E+00_JPRB ,&
&  0.58341E+00_JPRB, 0.58028E+00_JPRB, 0.58101E+00_JPRB, 0.58130E+00_JPRB, 0.57756E+00_JPRB, 0.56777E+00_JPRB, 0.55170E+00_JPRB ,&
&  0.54132E+00_JPRB, 0.53186E+00_JPRB, 0.51991E+00_JPRB, 0.50534E+00_JPRB, 0.49127E+00_JPRB, 0.48421E+00_JPRB, 0.48276E+00_JPRB ,&
&  0.48672E+00_JPRB, 0.49152E+00_JPRB, 0.49481E+00_JPRB, 0.50633E+00_JPRB, 0.52623E+00_JPRB, 0.56332E+00_JPRB, 0.61631E+00_JPRB ,&
&  0.71343E+00_JPRB, 0.10350E+01_JPRB, 0.11153E+01_JPRB, 0.11029E+01_JPRB, 0.11175E+01_JPRB, 0.11330E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.14225E+01_JPRB, 0.13798E+01_JPRB, 0.12304E+01_JPRB, 0.11038E+01_JPRB, 0.10153E+01_JPRB, 0.92886E+00_JPRB ,&
&  0.84566E+00_JPRB, 0.77929E+00_JPRB, 0.72670E+00_JPRB, 0.69087E+00_JPRB, 0.66370E+00_JPRB, 0.64339E+00_JPRB, 0.63195E+00_JPRB ,&
&  0.63004E+00_JPRB, 0.64125E+00_JPRB, 0.69304E+00_JPRB, 0.82241E+00_JPRB, 0.13549E+01_JPRB, 0.18379E+01_JPRB, 0.22612E+01_JPRB ,&
&  0.24338E+01_JPRB, 0.25120E+01_JPRB, 0.25406E+01_JPRB, 0.23659E+01_JPRB, 0.21401E+01_JPRB, 0.18826E+01_JPRB, 0.16974E+01_JPRB ,&
&  0.15715E+01_JPRB, 0.14344E+01_JPRB, 0.13099E+01_JPRB, 0.11964E+01_JPRB, 0.11145E+01_JPRB, 0.10674E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.98481E+00_JPRB, 0.96428E+00_JPRB, 0.94684E+00_JPRB, 0.92770E+00_JPRB, 0.90405E+00_JPRB, 0.87378E+00_JPRB ,&
&  0.83847E+00_JPRB, 0.80356E+00_JPRB, 0.77901E+00_JPRB, 0.76333E+00_JPRB, 0.75208E+00_JPRB, 0.73642E+00_JPRB, 0.71800E+00_JPRB ,&
&  0.71277E+00_JPRB, 0.71463E+00_JPRB, 0.72107E+00_JPRB, 0.72495E+00_JPRB, 0.72588E+00_JPRB, 0.74547E+00_JPRB, 0.72078E+00_JPRB ,&
&  0.69796E+00_JPRB, 0.67429E+00_JPRB, 0.65698E+00_JPRB, 0.64806E+00_JPRB, 0.66313E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.64977E+00_JPRB, 0.63354E+00_JPRB, 0.61606E+00_JPRB, 0.59609E+00_JPRB, 0.57986E+00_JPRB, 0.57044E+00_JPRB, 0.56409E+00_JPRB ,&
&  0.56096E+00_JPRB, 0.55999E+00_JPRB, 0.55896E+00_JPRB, 0.55698E+00_JPRB, 0.55371E+00_JPRB, 0.54940E+00_JPRB, 0.54093E+00_JPRB ,&
&  0.53054E+00_JPRB, 0.51788E+00_JPRB, 0.50720E+00_JPRB, 0.49798E+00_JPRB, 0.48219E+00_JPRB, 0.47201E+00_JPRB, 0.47254E+00_JPRB ,&
&  0.47692E+00_JPRB, 0.48190E+00_JPRB, 0.48591E+00_JPRB, 0.49641E+00_JPRB, 0.51983E+00_JPRB, 0.55687E+00_JPRB, 0.60031E+00_JPRB ,&
&  0.68755E+00_JPRB, 0.77657E+00_JPRB, 0.87013E+00_JPRB, 0.89124E+00_JPRB, 0.90188E+00_JPRB, 0.94726E+00_JPRB, 0.10150E+01_JPRB ,&
&  0.10565E+01_JPRB, 0.11389E+01_JPRB, 0.11269E+01_JPRB, 0.10270E+01_JPRB, 0.92340E+00_JPRB, 0.84969E+00_JPRB, 0.80318E+00_JPRB ,&
&  0.76817E+00_JPRB, 0.73673E+00_JPRB, 0.71007E+00_JPRB, 0.69125E+00_JPRB, 0.67337E+00_JPRB, 0.65535E+00_JPRB, 0.63784E+00_JPRB ,&
&  0.62652E+00_JPRB, 0.61951E+00_JPRB, 0.62185E+00_JPRB, 0.65023E+00_JPRB, 0.77071E+00_JPRB, 0.10119E+01_JPRB, 0.13450E+01_JPRB ,&
&  0.16179E+01_JPRB, 0.17808E+01_JPRB, 0.19131E+01_JPRB, 0.19165E+01_JPRB, 0.18348E+01_JPRB, 0.17226E+01_JPRB, 0.16052E+01_JPRB ,&
&  0.14560E+01_JPRB, 0.12826E+01_JPRB, 0.11514E+01_JPRB, 0.10560E+01_JPRB, 0.98777E+00_JPRB, 0.94262E+00_JPRB, 0.91446E+00_JPRB ,&
&  0.89269E+00_JPRB, 0.88636E+00_JPRB, 0.90588E+00_JPRB, 0.92529E+00_JPRB, 0.92324E+00_JPRB, 0.88840E+00_JPRB, 0.83831E+00_JPRB ,&
&  0.79307E+00_JPRB, 0.75833E+00_JPRB, 0.72992E+00_JPRB, 0.70819E+00_JPRB, 0.69114E+00_JPRB, 0.67826E+00_JPRB, 0.66488E+00_JPRB ,&
&  0.65740E+00_JPRB, 0.66628E+00_JPRB, 0.66354E+00_JPRB, 0.67415E+00_JPRB, 0.70650E+00_JPRB, 0.69159E+00_JPRB, 0.66741E+00_JPRB ,&
&  0.64871E+00_JPRB, 0.63741E+00_JPRB, 0.62657E+00_JPRB, 0.61772E+00_JPRB, 0.66553E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.57074E+00_JPRB, 0.56782E+00_JPRB, 0.55759E+00_JPRB, 0.54697E+00_JPRB, 0.53896E+00_JPRB, 0.53553E+00_JPRB, 0.53270E+00_JPRB ,&
&  0.53122E+00_JPRB, 0.53064E+00_JPRB, 0.53023E+00_JPRB, 0.52767E+00_JPRB, 0.52254E+00_JPRB, 0.51486E+00_JPRB, 0.50535E+00_JPRB ,&
&  0.49665E+00_JPRB, 0.48788E+00_JPRB, 0.48009E+00_JPRB, 0.47191E+00_JPRB, 0.46171E+00_JPRB, 0.45580E+00_JPRB, 0.45786E+00_JPRB ,&
&  0.46373E+00_JPRB, 0.47057E+00_JPRB, 0.47656E+00_JPRB, 0.48634E+00_JPRB, 0.51335E+00_JPRB, 0.54823E+00_JPRB, 0.58070E+00_JPRB ,&
&  0.65581E+00_JPRB, 0.66019E+00_JPRB, 0.70246E+00_JPRB, 0.75219E+00_JPRB, 0.80099E+00_JPRB, 0.82052E+00_JPRB, 0.84268E+00_JPRB ,&
&  0.86841E+00_JPRB, 0.89678E+00_JPRB, 0.89667E+00_JPRB, 0.85601E+00_JPRB, 0.77901E+00_JPRB, 0.70716E+00_JPRB, 0.66469E+00_JPRB ,&
&  0.64364E+00_JPRB, 0.63397E+00_JPRB, 0.62511E+00_JPRB, 0.61690E+00_JPRB, 0.61039E+00_JPRB, 0.60406E+00_JPRB, 0.59738E+00_JPRB ,&
&  0.59107E+00_JPRB, 0.58603E+00_JPRB, 0.58329E+00_JPRB, 0.58470E+00_JPRB, 0.60579E+00_JPRB, 0.67952E+00_JPRB, 0.84808E+00_JPRB ,&
&  0.10848E+01_JPRB, 0.13074E+01_JPRB, 0.14328E+01_JPRB, 0.15084E+01_JPRB, 0.15480E+01_JPRB, 0.14822E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.10382E+01_JPRB, 0.93816E+00_JPRB, 0.86396E+00_JPRB, 0.81098E+00_JPRB, 0.77611E+00_JPRB, 0.75658E+00_JPRB ,&
&  0.74959E+00_JPRB, 0.75054E+00_JPRB, 0.75924E+00_JPRB, 0.76820E+00_JPRB, 0.77211E+00_JPRB, 0.76434E+00_JPRB, 0.74612E+00_JPRB ,&
&  0.71981E+00_JPRB, 0.70014E+00_JPRB, 0.68041E+00_JPRB, 0.65838E+00_JPRB, 0.63891E+00_JPRB, 0.62086E+00_JPRB, 0.60685E+00_JPRB ,&
&  0.59872E+00_JPRB, 0.60146E+00_JPRB, 0.61177E+00_JPRB, 0.63342E+00_JPRB, 0.62307E+00_JPRB, 0.60697E+00_JPRB, 0.59284E+00_JPRB ,&
&  0.58691E+00_JPRB, 0.58582E+00_JPRB, 0.57719E+00_JPRB, 0.56924E+00_JPRB, 0.56819E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.52843E+00_JPRB, 0.52516E+00_JPRB, 0.51697E+00_JPRB, 0.50940E+00_JPRB, 0.50671E+00_JPRB, 0.50684E+00_JPRB, 0.50806E+00_JPRB ,&
&  0.50721E+00_JPRB, 0.50413E+00_JPRB, 0.49965E+00_JPRB, 0.49628E+00_JPRB, 0.49151E+00_JPRB, 0.48485E+00_JPRB, 0.47665E+00_JPRB ,&
&  0.46873E+00_JPRB, 0.46146E+00_JPRB, 0.45578E+00_JPRB, 0.44979E+00_JPRB, 0.44522E+00_JPRB, 0.44484E+00_JPRB, 0.44642E+00_JPRB ,&
&  0.45268E+00_JPRB, 0.46080E+00_JPRB, 0.46735E+00_JPRB, 0.48257E+00_JPRB, 0.50163E+00_JPRB, 0.53113E+00_JPRB, 0.56266E+00_JPRB ,&
&  0.62606E+00_JPRB, 0.56626E+00_JPRB, 0.60031E+00_JPRB, 0.67329E+00_JPRB, 0.71992E+00_JPRB, 0.74073E+00_JPRB, 0.72702E+00_JPRB ,&
&  0.71869E+00_JPRB, 0.72109E+00_JPRB, 0.72105E+00_JPRB, 0.71363E+00_JPRB, 0.68685E+00_JPRB, 0.63538E+00_JPRB, 0.58511E+00_JPRB ,&
&  0.55119E+00_JPRB, 0.53931E+00_JPRB, 0.53394E+00_JPRB, 0.52626E+00_JPRB, 0.51845E+00_JPRB, 0.51444E+00_JPRB, 0.51424E+00_JPRB ,&
&  0.51394E+00_JPRB, 0.51298E+00_JPRB, 0.51102E+00_JPRB, 0.50887E+00_JPRB, 0.51054E+00_JPRB, 0.52853E+00_JPRB, 0.59043E+00_JPRB ,&
&  0.71614E+00_JPRB, 0.86526E+00_JPRB, 0.10060E+01_JPRB, 0.11154E+01_JPRB, 0.11413E+01_JPRB, 0.11116E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.97452E+00_JPRB, 0.89626E+00_JPRB, 0.83223E+00_JPRB, 0.78013E+00_JPRB, 0.73819E+00_JPRB, 0.70520E+00_JPRB, 0.68090E+00_JPRB ,&
&  0.66804E+00_JPRB, 0.65959E+00_JPRB, 0.65362E+00_JPRB, 0.64543E+00_JPRB, 0.63691E+00_JPRB, 0.63117E+00_JPRB, 0.62854E+00_JPRB ,&
&  0.62267E+00_JPRB, 0.61537E+00_JPRB, 0.60866E+00_JPRB, 0.60091E+00_JPRB, 0.59247E+00_JPRB, 0.58417E+00_JPRB, 0.57686E+00_JPRB ,&
&  0.57265E+00_JPRB, 0.57024E+00_JPRB, 0.57086E+00_JPRB, 0.56739E+00_JPRB, 0.57504E+00_JPRB, 0.56235E+00_JPRB, 0.55258E+00_JPRB ,&
&  0.54837E+00_JPRB, 0.54326E+00_JPRB, 0.54005E+00_JPRB, 0.50053E+00_JPRB, 0.52554E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.49063E+00_JPRB, 0.49212E+00_JPRB, 0.49056E+00_JPRB, 0.48499E+00_JPRB, 0.48130E+00_JPRB, 0.47917E+00_JPRB, 0.47858E+00_JPRB ,&
&  0.47860E+00_JPRB, 0.47755E+00_JPRB, 0.47454E+00_JPRB, 0.46886E+00_JPRB, 0.46384E+00_JPRB, 0.45753E+00_JPRB, 0.45093E+00_JPRB ,&
&  0.44529E+00_JPRB, 0.44037E+00_JPRB, 0.43750E+00_JPRB, 0.43401E+00_JPRB, 0.43276E+00_JPRB, 0.43428E+00_JPRB, 0.43637E+00_JPRB ,&
&  0.43956E+00_JPRB, 0.44236E+00_JPRB, 0.45112E+00_JPRB, 0.46360E+00_JPRB, 0.48666E+00_JPRB, 0.50315E+00_JPRB, 0.53033E+00_JPRB ,&
&  0.58987E+00_JPRB, 0.50910E+00_JPRB, 0.55305E+00_JPRB, 0.60829E+00_JPRB, 0.64492E+00_JPRB, 0.66820E+00_JPRB, 0.63807E+00_JPRB ,&
&  0.61448E+00_JPRB, 0.59986E+00_JPRB, 0.59172E+00_JPRB, 0.58676E+00_JPRB, 0.58307E+00_JPRB, 0.56846E+00_JPRB, 0.54946E+00_JPRB ,&
&  0.52876E+00_JPRB, 0.51267E+00_JPRB, 0.49936E+00_JPRB, 0.48591E+00_JPRB, 0.47407E+00_JPRB, 0.46649E+00_JPRB, 0.46284E+00_JPRB ,&
&  0.46206E+00_JPRB, 0.46212E+00_JPRB, 0.46182E+00_JPRB, 0.46225E+00_JPRB, 0.46417E+00_JPRB, 0.46875E+00_JPRB, 0.47968E+00_JPRB ,&
&  0.51569E+00_JPRB, 0.58269E+00_JPRB, 0.66155E+00_JPRB, 0.73877E+00_JPRB, 0.79415E+00_JPRB, 0.82127E+00_JPRB, 0.83347E+00_JPRB ,&
&  0.82278E+00_JPRB, 0.79446E+00_JPRB, 0.75778E+00_JPRB, 0.72041E+00_JPRB, 0.68669E+00_JPRB, 0.65979E+00_JPRB, 0.63592E+00_JPRB ,&
&  0.61433E+00_JPRB, 0.59290E+00_JPRB, 0.57467E+00_JPRB, 0.56065E+00_JPRB, 0.54788E+00_JPRB, 0.53668E+00_JPRB, 0.53039E+00_JPRB ,&
&  0.52787E+00_JPRB, 0.52750E+00_JPRB, 0.52681E+00_JPRB, 0.52764E+00_JPRB, 0.52896E+00_JPRB, 0.52995E+00_JPRB, 0.52919E+00_JPRB ,&
&  0.52578E+00_JPRB, 0.52389E+00_JPRB, 0.52164E+00_JPRB, 0.52731E+00_JPRB, 0.52423E+00_JPRB, 0.52063E+00_JPRB, 0.51554E+00_JPRB ,&
&  0.51195E+00_JPRB, 0.50509E+00_JPRB, 0.45308E+00_JPRB, 0.47650E+00_JPRB, 0.48516E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.46647E+00_JPRB, 0.46638E+00_JPRB, 0.46756E+00_JPRB, 0.46743E+00_JPRB, 0.46655E+00_JPRB, 0.46482E+00_JPRB, 0.46170E+00_JPRB ,&
&  0.45706E+00_JPRB, 0.45379E+00_JPRB, 0.45097E+00_JPRB, 0.44725E+00_JPRB, 0.44309E+00_JPRB, 0.43956E+00_JPRB, 0.43632E+00_JPRB ,&
&  0.43297E+00_JPRB, 0.42939E+00_JPRB, 0.42691E+00_JPRB, 0.42522E+00_JPRB, 0.42339E+00_JPRB, 0.42276E+00_JPRB, 0.42383E+00_JPRB ,&
&  0.42380E+00_JPRB, 0.42661E+00_JPRB, 0.43746E+00_JPRB, 0.45092E+00_JPRB, 0.46909E+00_JPRB, 0.48628E+00_JPRB, 0.50463E+00_JPRB ,&
&  0.49584E+00_JPRB, 0.44871E+00_JPRB, 0.48285E+00_JPRB, 0.52571E+00_JPRB, 0.56906E+00_JPRB, 0.59953E+00_JPRB, 0.57832E+00_JPRB ,&
&  0.55015E+00_JPRB, 0.52561E+00_JPRB, 0.50955E+00_JPRB, 0.50001E+00_JPRB, 0.49929E+00_JPRB, 0.49846E+00_JPRB, 0.49669E+00_JPRB ,&
&  0.49171E+00_JPRB, 0.48175E+00_JPRB, 0.47066E+00_JPRB, 0.46025E+00_JPRB, 0.45274E+00_JPRB, 0.44680E+00_JPRB, 0.44184E+00_JPRB ,&
&  0.43938E+00_JPRB, 0.43872E+00_JPRB, 0.43849E+00_JPRB, 0.43861E+00_JPRB, 0.43943E+00_JPRB, 0.44236E+00_JPRB, 0.44597E+00_JPRB ,&
&  0.45100E+00_JPRB, 0.46600E+00_JPRB, 0.49898E+00_JPRB, 0.54120E+00_JPRB, 0.58494E+00_JPRB, 0.61926E+00_JPRB, 0.64434E+00_JPRB ,&
&  0.66317E+00_JPRB, 0.67282E+00_JPRB, 0.66916E+00_JPRB, 0.65191E+00_JPRB, 0.62846E+00_JPRB, 0.60556E+00_JPRB, 0.58329E+00_JPRB ,&
&  0.56077E+00_JPRB, 0.53524E+00_JPRB, 0.51014E+00_JPRB, 0.49368E+00_JPRB, 0.48401E+00_JPRB, 0.47529E+00_JPRB, 0.46862E+00_JPRB ,&
&  0.46598E+00_JPRB, 0.46642E+00_JPRB, 0.46850E+00_JPRB, 0.47042E+00_JPRB, 0.47230E+00_JPRB, 0.47515E+00_JPRB, 0.47734E+00_JPRB ,&
&  0.47861E+00_JPRB, 0.48111E+00_JPRB, 0.48544E+00_JPRB, 0.49468E+00_JPRB, 0.50045E+00_JPRB, 0.50223E+00_JPRB, 0.49956E+00_JPRB ,&
&  0.49150E+00_JPRB, 0.48141E+00_JPRB, 0.47152E+00_JPRB, 0.46558E+00_JPRB, 0.46503E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.44929E+00_JPRB, 0.44860E+00_JPRB, 0.44833E+00_JPRB, 0.44689E+00_JPRB, 0.44543E+00_JPRB, 0.44378E+00_JPRB, 0.43972E+00_JPRB ,&
&  0.43784E+00_JPRB, 0.43371E+00_JPRB, 0.42984E+00_JPRB, 0.42787E+00_JPRB, 0.42549E+00_JPRB, 0.42266E+00_JPRB, 0.42060E+00_JPRB ,&
&  0.41836E+00_JPRB, 0.41570E+00_JPRB, 0.41373E+00_JPRB, 0.41113E+00_JPRB, 0.40733E+00_JPRB, 0.40462E+00_JPRB, 0.40509E+00_JPRB ,&
&  0.40576E+00_JPRB, 0.40840E+00_JPRB, 0.41774E+00_JPRB, 0.42710E+00_JPRB, 0.43847E+00_JPRB, 0.45012E+00_JPRB, 0.46025E+00_JPRB ,&
&  0.47717E+00_JPRB, 0.42656E+00_JPRB, 0.45767E+00_JPRB, 0.48221E+00_JPRB, 0.51870E+00_JPRB, 0.54637E+00_JPRB, 0.53514E+00_JPRB ,&
&  0.50863E+00_JPRB, 0.48053E+00_JPRB, 0.46207E+00_JPRB, 0.45247E+00_JPRB, 0.45424E+00_JPRB, 0.45756E+00_JPRB, 0.45648E+00_JPRB ,&
&  0.45223E+00_JPRB, 0.44286E+00_JPRB, 0.43231E+00_JPRB, 0.42560E+00_JPRB, 0.42198E+00_JPRB, 0.41916E+00_JPRB, 0.41631E+00_JPRB ,&
&  0.41440E+00_JPRB, 0.41377E+00_JPRB, 0.41344E+00_JPRB, 0.41324E+00_JPRB, 0.41384E+00_JPRB, 0.41552E+00_JPRB, 0.41776E+00_JPRB ,&
&  0.41936E+00_JPRB, 0.41996E+00_JPRB, 0.42463E+00_JPRB, 0.43865E+00_JPRB, 0.45875E+00_JPRB, 0.47927E+00_JPRB, 0.49663E+00_JPRB ,&
&  0.51145E+00_JPRB, 0.52502E+00_JPRB, 0.53427E+00_JPRB, 0.53520E+00_JPRB, 0.52791E+00_JPRB, 0.51567E+00_JPRB, 0.50175E+00_JPRB ,&
&  0.48651E+00_JPRB, 0.47030E+00_JPRB, 0.45226E+00_JPRB, 0.44067E+00_JPRB, 0.43470E+00_JPRB, 0.43113E+00_JPRB, 0.42869E+00_JPRB ,&
&  0.42729E+00_JPRB, 0.42681E+00_JPRB, 0.42811E+00_JPRB, 0.43002E+00_JPRB, 0.43113E+00_JPRB, 0.43211E+00_JPRB, 0.43400E+00_JPRB ,&
&  0.43755E+00_JPRB, 0.44328E+00_JPRB, 0.45068E+00_JPRB, 0.46082E+00_JPRB, 0.46952E+00_JPRB, 0.47491E+00_JPRB, 0.47737E+00_JPRB ,&
&  0.47450E+00_JPRB, 0.46800E+00_JPRB, 0.45938E+00_JPRB, 0.45194E+00_JPRB, 0.45021E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.43639E+00_JPRB, 0.43236E+00_JPRB, 0.42950E+00_JPRB, 0.42503E+00_JPRB, 0.42136E+00_JPRB, 0.41848E+00_JPRB, 0.41552E+00_JPRB ,&
&  0.41343E+00_JPRB, 0.41336E+00_JPRB, 0.41179E+00_JPRB, 0.41079E+00_JPRB, 0.40942E+00_JPRB, 0.40629E+00_JPRB, 0.40265E+00_JPRB ,&
&  0.40070E+00_JPRB, 0.39897E+00_JPRB, 0.39742E+00_JPRB, 0.39559E+00_JPRB, 0.39360E+00_JPRB, 0.39084E+00_JPRB, 0.39066E+00_JPRB ,&
&  0.39136E+00_JPRB, 0.39399E+00_JPRB, 0.39790E+00_JPRB, 0.40246E+00_JPRB, 0.40991E+00_JPRB, 0.42023E+00_JPRB, 0.43017E+00_JPRB ,&
&  0.44160E+00_JPRB, 0.42922E+00_JPRB, 0.44842E+00_JPRB, 0.45985E+00_JPRB, 0.48064E+00_JPRB, 0.49802E+00_JPRB, 0.49778E+00_JPRB ,&
&  0.48682E+00_JPRB, 0.46727E+00_JPRB, 0.45111E+00_JPRB, 0.44180E+00_JPRB, 0.44039E+00_JPRB, 0.43872E+00_JPRB, 0.43172E+00_JPRB ,&
&  0.42178E+00_JPRB, 0.41017E+00_JPRB, 0.40140E+00_JPRB, 0.39669E+00_JPRB, 0.39492E+00_JPRB, 0.39435E+00_JPRB, 0.39256E+00_JPRB ,&
&  0.39058E+00_JPRB, 0.38862E+00_JPRB, 0.38759E+00_JPRB, 0.38707E+00_JPRB, 0.38654E+00_JPRB, 0.38664E+00_JPRB, 0.38738E+00_JPRB ,&
&  0.38815E+00_JPRB, 0.38850E+00_JPRB, 0.38854E+00_JPRB, 0.39048E+00_JPRB, 0.39644E+00_JPRB, 0.40547E+00_JPRB, 0.41499E+00_JPRB ,&
&  0.42211E+00_JPRB, 0.42904E+00_JPRB, 0.43561E+00_JPRB, 0.44037E+00_JPRB, 0.44036E+00_JPRB, 0.43491E+00_JPRB, 0.42677E+00_JPRB ,&
&  0.41940E+00_JPRB, 0.41246E+00_JPRB, 0.40544E+00_JPRB, 0.39976E+00_JPRB, 0.39644E+00_JPRB, 0.39550E+00_JPRB, 0.39548E+00_JPRB ,&
&  0.39574E+00_JPRB, 0.39628E+00_JPRB, 0.39750E+00_JPRB, 0.39966E+00_JPRB, 0.40120E+00_JPRB, 0.40265E+00_JPRB, 0.40407E+00_JPRB ,&
&  0.40791E+00_JPRB, 0.41271E+00_JPRB, 0.41957E+00_JPRB, 0.42969E+00_JPRB, 0.43894E+00_JPRB, 0.44562E+00_JPRB, 0.45116E+00_JPRB ,&
&  0.45343E+00_JPRB, 0.45241E+00_JPRB, 0.44931E+00_JPRB, 0.44555E+00_JPRB, 0.44140E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.41058E+00_JPRB, 0.40474E+00_JPRB, 0.40041E+00_JPRB, 0.39707E+00_JPRB, 0.39627E+00_JPRB, 0.39433E+00_JPRB, 0.39053E+00_JPRB ,&
&  0.38751E+00_JPRB, 0.38848E+00_JPRB, 0.39112E+00_JPRB, 0.39218E+00_JPRB, 0.39127E+00_JPRB, 0.38951E+00_JPRB, 0.38632E+00_JPRB ,&
&  0.38208E+00_JPRB, 0.38002E+00_JPRB, 0.37990E+00_JPRB, 0.38095E+00_JPRB, 0.38070E+00_JPRB, 0.37856E+00_JPRB, 0.37671E+00_JPRB ,&
&  0.37714E+00_JPRB, 0.37862E+00_JPRB, 0.37928E+00_JPRB, 0.38140E+00_JPRB, 0.38668E+00_JPRB, 0.39663E+00_JPRB, 0.40563E+00_JPRB ,&
&  0.41053E+00_JPRB, 0.41287E+00_JPRB, 0.40696E+00_JPRB, 0.42206E+00_JPRB, 0.43337E+00_JPRB, 0.44838E+00_JPRB, 0.45834E+00_JPRB ,&
&  0.46038E+00_JPRB, 0.45423E+00_JPRB, 0.44092E+00_JPRB, 0.43025E+00_JPRB, 0.42305E+00_JPRB, 0.41448E+00_JPRB, 0.40159E+00_JPRB ,&
&  0.38848E+00_JPRB, 0.37777E+00_JPRB, 0.37143E+00_JPRB, 0.36849E+00_JPRB, 0.36838E+00_JPRB, 0.36884E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.36458E+00_JPRB, 0.36232E+00_JPRB, 0.36135E+00_JPRB, 0.36044E+00_JPRB, 0.35919E+00_JPRB, 0.35816E+00_JPRB, 0.35780E+00_JPRB ,&
&  0.35786E+00_JPRB, 0.35791E+00_JPRB, 0.35821E+00_JPRB, 0.35887E+00_JPRB, 0.35945E+00_JPRB, 0.36120E+00_JPRB, 0.36498E+00_JPRB ,&
&  0.36886E+00_JPRB, 0.37222E+00_JPRB, 0.37515E+00_JPRB, 0.37737E+00_JPRB, 0.37840E+00_JPRB, 0.37777E+00_JPRB, 0.37481E+00_JPRB ,&
&  0.37119E+00_JPRB, 0.36929E+00_JPRB, 0.36795E+00_JPRB, 0.36573E+00_JPRB, 0.36347E+00_JPRB, 0.36291E+00_JPRB, 0.36383E+00_JPRB ,&
&  0.36508E+00_JPRB, 0.36699E+00_JPRB, 0.36875E+00_JPRB, 0.37057E+00_JPRB, 0.37165E+00_JPRB, 0.37323E+00_JPRB, 0.37492E+00_JPRB ,&
&  0.37723E+00_JPRB, 0.38058E+00_JPRB, 0.38526E+00_JPRB, 0.39137E+00_JPRB, 0.39780E+00_JPRB, 0.40299E+00_JPRB, 0.40865E+00_JPRB ,&
&  0.41390E+00_JPRB, 0.41688E+00_JPRB, 0.41766E+00_JPRB, 0.41792E+00_JPRB, 0.41582E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.37937E+00_JPRB, 0.37546E+00_JPRB, 0.37247E+00_JPRB, 0.37145E+00_JPRB, 0.37258E+00_JPRB, 0.37195E+00_JPRB, 0.36906E+00_JPRB ,&
&  0.36646E+00_JPRB, 0.36743E+00_JPRB, 0.37081E+00_JPRB, 0.37293E+00_JPRB, 0.37393E+00_JPRB, 0.37481E+00_JPRB, 0.37337E+00_JPRB ,&
&  0.36939E+00_JPRB, 0.36563E+00_JPRB, 0.36619E+00_JPRB, 0.36819E+00_JPRB, 0.36916E+00_JPRB, 0.36727E+00_JPRB, 0.36509E+00_JPRB ,&
&  0.36522E+00_JPRB, 0.36530E+00_JPRB, 0.36710E+00_JPRB, 0.36885E+00_JPRB, 0.37339E+00_JPRB, 0.38154E+00_JPRB, 0.38806E+00_JPRB ,&
&  0.39135E+00_JPRB, 0.38986E+00_JPRB, 0.38884E+00_JPRB, 0.39264E+00_JPRB, 0.39744E+00_JPRB, 0.40812E+00_JPRB, 0.41973E+00_JPRB ,&
&  0.42507E+00_JPRB, 0.42250E+00_JPRB, 0.41043E+00_JPRB, 0.39867E+00_JPRB, 0.38959E+00_JPRB, 0.37986E+00_JPRB, 0.37022E+00_JPRB ,&
&  0.36096E+00_JPRB, 0.35442E+00_JPRB, 0.35071E+00_JPRB, 0.34913E+00_JPRB, 0.34862E+00_JPRB, 0.34777E+00_JPRB, 0.34514E+00_JPRB ,&
&  0.34181E+00_JPRB, 0.33922E+00_JPRB, 0.33729E+00_JPRB, 0.33585E+00_JPRB, 0.33454E+00_JPRB, 0.33388E+00_JPRB, 0.33309E+00_JPRB ,&
&  0.33270E+00_JPRB, 0.33306E+00_JPRB, 0.33381E+00_JPRB, 0.33370E+00_JPRB, 0.33336E+00_JPRB, 0.33262E+00_JPRB, 0.33295E+00_JPRB ,&
&  0.33488E+00_JPRB, 0.33563E+00_JPRB, 0.33622E+00_JPRB, 0.33786E+00_JPRB, 0.33910E+00_JPRB, 0.34012E+00_JPRB, 0.34007E+00_JPRB ,&
&  0.33965E+00_JPRB, 0.33968E+00_JPRB, 0.33987E+00_JPRB, 0.33938E+00_JPRB, 0.33864E+00_JPRB, 0.33821E+00_JPRB, 0.33833E+00_JPRB ,&
&  0.33982E+00_JPRB, 0.34180E+00_JPRB, 0.34336E+00_JPRB, 0.34490E+00_JPRB, 0.34667E+00_JPRB, 0.34930E+00_JPRB, 0.35053E+00_JPRB ,&
&  0.35132E+00_JPRB, 0.35293E+00_JPRB, 0.35622E+00_JPRB, 0.36049E+00_JPRB, 0.36459E+00_JPRB, 0.36754E+00_JPRB, 0.37088E+00_JPRB ,&
&  0.37415E+00_JPRB, 0.37703E+00_JPRB, 0.37894E+00_JPRB, 0.37969E+00_JPRB, 0.38016E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.35006E+00_JPRB, 0.34868E+00_JPRB, 0.34811E+00_JPRB, 0.34832E+00_JPRB, 0.34982E+00_JPRB, 0.34933E+00_JPRB, 0.34839E+00_JPRB ,&
&  0.34763E+00_JPRB, 0.34954E+00_JPRB, 0.35346E+00_JPRB, 0.35602E+00_JPRB, 0.35674E+00_JPRB, 0.35694E+00_JPRB, 0.35650E+00_JPRB ,&
&  0.35592E+00_JPRB, 0.35544E+00_JPRB, 0.35450E+00_JPRB, 0.35525E+00_JPRB, 0.35503E+00_JPRB, 0.35360E+00_JPRB, 0.35414E+00_JPRB ,&
&  0.35402E+00_JPRB, 0.35450E+00_JPRB, 0.35687E+00_JPRB, 0.35891E+00_JPRB, 0.36352E+00_JPRB, 0.37148E+00_JPRB, 0.37673E+00_JPRB ,&
&  0.37790E+00_JPRB, 0.37316E+00_JPRB, 0.36864E+00_JPRB, 0.36903E+00_JPRB, 0.37143E+00_JPRB, 0.37669E+00_JPRB, 0.38285E+00_JPRB ,&
&  0.38547E+00_JPRB, 0.37964E+00_JPRB, 0.36898E+00_JPRB, 0.36013E+00_JPRB, 0.35355E+00_JPRB, 0.34830E+00_JPRB, 0.34360E+00_JPRB ,&
&  0.33911E+00_JPRB, 0.33615E+00_JPRB, 0.33502E+00_JPRB, 0.33500E+00_JPRB, 0.33451E+00_JPRB, 0.33127E+00_JPRB, 0.32739E+00_JPRB ,&
&  0.32261E+00_JPRB, 0.31923E+00_JPRB, 0.31650E+00_JPRB, 0.31551E+00_JPRB, 0.31553E+00_JPRB, 0.31513E+00_JPRB, 0.31388E+00_JPRB ,&
&  0.31235E+00_JPRB, 0.31177E+00_JPRB, 0.31127E+00_JPRB, 0.30993E+00_JPRB, 0.30824E+00_JPRB, 0.30754E+00_JPRB, 0.30671E+00_JPRB ,&
&  0.30575E+00_JPRB, 0.30631E+00_JPRB, 0.30790E+00_JPRB, 0.31014E+00_JPRB, 0.31080E+00_JPRB, 0.31236E+00_JPRB, 0.31450E+00_JPRB ,&
&  0.31416E+00_JPRB, 0.31171E+00_JPRB, 0.30993E+00_JPRB, 0.30981E+00_JPRB, 0.31125E+00_JPRB, 0.31358E+00_JPRB, 0.31462E+00_JPRB ,&
&  0.31554E+00_JPRB, 0.31553E+00_JPRB, 0.31654E+00_JPRB, 0.31815E+00_JPRB, 0.32053E+00_JPRB, 0.32286E+00_JPRB, 0.32437E+00_JPRB ,&
&  0.32530E+00_JPRB, 0.32604E+00_JPRB, 0.32842E+00_JPRB, 0.33197E+00_JPRB, 0.33503E+00_JPRB, 0.33844E+00_JPRB, 0.34108E+00_JPRB ,&
&  0.34296E+00_JPRB, 0.34497E+00_JPRB, 0.34636E+00_JPRB, 0.34740E+00_JPRB, 0.34880E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.33097E+00_JPRB, 0.33166E+00_JPRB, 0.33296E+00_JPRB, 0.33340E+00_JPRB, 0.33431E+00_JPRB, 0.33455E+00_JPRB, 0.33545E+00_JPRB ,&
&  0.33584E+00_JPRB, 0.33749E+00_JPRB, 0.33989E+00_JPRB, 0.34001E+00_JPRB, 0.33979E+00_JPRB, 0.33975E+00_JPRB, 0.34059E+00_JPRB ,&
&  0.34310E+00_JPRB, 0.34422E+00_JPRB, 0.34390E+00_JPRB, 0.34415E+00_JPRB, 0.34296E+00_JPRB, 0.34172E+00_JPRB, 0.34345E+00_JPRB ,&
&  0.34383E+00_JPRB, 0.34456E+00_JPRB, 0.34670E+00_JPRB, 0.34872E+00_JPRB, 0.35324E+00_JPRB, 0.35909E+00_JPRB, 0.36430E+00_JPRB ,&
&  0.36611E+00_JPRB, 0.36289E+00_JPRB, 0.35631E+00_JPRB, 0.35131E+00_JPRB, 0.35014E+00_JPRB, 0.35110E+00_JPRB, 0.35359E+00_JPRB ,&
&  0.35284E+00_JPRB, 0.34677E+00_JPRB, 0.33880E+00_JPRB, 0.33422E+00_JPRB, 0.33211E+00_JPRB, 0.32897E+00_JPRB, 0.32449E+00_JPRB ,&
&  0.32107E+00_JPRB, 0.32011E+00_JPRB, 0.32048E+00_JPRB, 0.31981E+00_JPRB, 0.31786E+00_JPRB, 0.31432E+00_JPRB, 0.30984E+00_JPRB ,&
&  0.30578E+00_JPRB, 0.30182E+00_JPRB, 0.29930E+00_JPRB, 0.29768E+00_JPRB, 0.29751E+00_JPRB, 0.29751E+00_JPRB, 0.29635E+00_JPRB ,&
&  0.29403E+00_JPRB, 0.29245E+00_JPRB, 0.29050E+00_JPRB, 0.28713E+00_JPRB, 0.28476E+00_JPRB, 0.28407E+00_JPRB, 0.28318E+00_JPRB ,&
&  0.28205E+00_JPRB, 0.28222E+00_JPRB, 0.28270E+00_JPRB, 0.28285E+00_JPRB, 0.28444E+00_JPRB, 0.28684E+00_JPRB, 0.28467E+00_JPRB ,&
&  0.28256E+00_JPRB, 0.28113E+00_JPRB, 0.28118E+00_JPRB, 0.28292E+00_JPRB, 0.28584E+00_JPRB, 0.28895E+00_JPRB, 0.29148E+00_JPRB ,&
&  0.29253E+00_JPRB, 0.29274E+00_JPRB, 0.29307E+00_JPRB, 0.29343E+00_JPRB, 0.29519E+00_JPRB, 0.29845E+00_JPRB, 0.30209E+00_JPRB ,&
&  0.30365E+00_JPRB, 0.30630E+00_JPRB, 0.30932E+00_JPRB, 0.31318E+00_JPRB, 0.31553E+00_JPRB, 0.31778E+00_JPRB, 0.31980E+00_JPRB ,&
&  0.32124E+00_JPRB, 0.32307E+00_JPRB, 0.32547E+00_JPRB, 0.32737E+00_JPRB, 0.32932E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.31692E+00_JPRB, 0.31847E+00_JPRB, 0.31936E+00_JPRB, 0.32094E+00_JPRB, 0.32370E+00_JPRB, 0.32511E+00_JPRB, 0.32487E+00_JPRB ,&
&  0.32490E+00_JPRB, 0.32641E+00_JPRB, 0.32767E+00_JPRB, 0.32829E+00_JPRB, 0.32606E+00_JPRB, 0.32578E+00_JPRB, 0.32772E+00_JPRB ,&
&  0.33027E+00_JPRB, 0.33244E+00_JPRB, 0.33277E+00_JPRB, 0.33299E+00_JPRB, 0.33205E+00_JPRB, 0.33052E+00_JPRB, 0.33189E+00_JPRB ,&
&  0.33313E+00_JPRB, 0.33367E+00_JPRB, 0.33511E+00_JPRB, 0.33678E+00_JPRB, 0.34031E+00_JPRB, 0.34451E+00_JPRB, 0.34783E+00_JPRB ,&
&  0.34972E+00_JPRB, 0.34846E+00_JPRB, 0.34088E+00_JPRB, 0.33319E+00_JPRB, 0.33295E+00_JPRB, 0.33364E+00_JPRB, 0.33331E+00_JPRB ,&
&  0.33103E+00_JPRB, 0.32796E+00_JPRB, 0.32593E+00_JPRB, 0.32431E+00_JPRB, 0.32214E+00_JPRB, 0.31788E+00_JPRB, 0.31321E+00_JPRB ,&
&  0.30933E+00_JPRB, 0.30806E+00_JPRB, 0.30683E+00_JPRB, 0.30432E+00_JPRB, 0.30041E+00_JPRB, 0.29708E+00_JPRB, 0.29358E+00_JPRB ,&
&  0.28770E+00_JPRB, 0.28208E+00_JPRB, 0.27899E+00_JPRB, 0.27810E+00_JPRB, 0.27827E+00_JPRB, 0.27760E+00_JPRB, 0.27546E+00_JPRB ,&
&  0.27300E+00_JPRB, 0.27013E+00_JPRB, 0.26841E+00_JPRB, 0.26802E+00_JPRB, 0.26937E+00_JPRB, 0.26784E+00_JPRB, 0.26313E+00_JPRB ,&
&  0.26072E+00_JPRB, 0.25971E+00_JPRB, 0.26062E+00_JPRB, 0.26001E+00_JPRB, 0.25952E+00_JPRB, 0.25857E+00_JPRB, 0.25907E+00_JPRB ,&
&  0.26033E+00_JPRB, 0.26044E+00_JPRB, 0.26323E+00_JPRB, 0.26826E+00_JPRB, 0.27266E+00_JPRB, 0.27525E+00_JPRB, 0.27688E+00_JPRB ,&
&  0.27756E+00_JPRB, 0.27727E+00_JPRB, 0.27693E+00_JPRB, 0.27777E+00_JPRB, 0.28040E+00_JPRB, 0.28401E+00_JPRB, 0.28716E+00_JPRB ,&
&  0.28763E+00_JPRB, 0.28882E+00_JPRB, 0.29202E+00_JPRB, 0.29684E+00_JPRB, 0.29965E+00_JPRB, 0.30183E+00_JPRB, 0.30352E+00_JPRB ,&
&  0.30416E+00_JPRB, 0.30563E+00_JPRB, 0.30927E+00_JPRB, 0.31157E+00_JPRB, 0.31399E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.30255E+00_JPRB, 0.30342E+00_JPRB, 0.30539E+00_JPRB, 0.30781E+00_JPRB, 0.30898E+00_JPRB, 0.30844E+00_JPRB, 0.30739E+00_JPRB ,&
&  0.30777E+00_JPRB, 0.31019E+00_JPRB, 0.31278E+00_JPRB, 0.31368E+00_JPRB, 0.31421E+00_JPRB, 0.31479E+00_JPRB, 0.31677E+00_JPRB ,&
&  0.31956E+00_JPRB, 0.32282E+00_JPRB, 0.32296E+00_JPRB, 0.32294E+00_JPRB, 0.32235E+00_JPRB, 0.32115E+00_JPRB, 0.32233E+00_JPRB ,&
&  0.32543E+00_JPRB, 0.32554E+00_JPRB, 0.32521E+00_JPRB, 0.32445E+00_JPRB, 0.32632E+00_JPRB, 0.32994E+00_JPRB, 0.33237E+00_JPRB ,&
&  0.33179E+00_JPRB, 0.32492E+00_JPRB, 0.31641E+00_JPRB, 0.31584E+00_JPRB, 0.31748E+00_JPRB, 0.31906E+00_JPRB, 0.31905E+00_JPRB ,&
&  0.31852E+00_JPRB, 0.31758E+00_JPRB, 0.31644E+00_JPRB, 0.31523E+00_JPRB, 0.31354E+00_JPRB, 0.30895E+00_JPRB, 0.30361E+00_JPRB ,&
&  0.29935E+00_JPRB, 0.29583E+00_JPRB, 0.29243E+00_JPRB, 0.28748E+00_JPRB, 0.28247E+00_JPRB, 0.27881E+00_JPRB, 0.27444E+00_JPRB ,&
&  0.26911E+00_JPRB, 0.26549E+00_JPRB, 0.26432E+00_JPRB, 0.26409E+00_JPRB, 0.26401E+00_JPRB, 0.26301E+00_JPRB, 0.26171E+00_JPRB ,&
&  0.25950E+00_JPRB, 0.25784E+00_JPRB, 0.25620E+00_JPRB, 0.25515E+00_JPRB, 0.24878E+00_JPRB, 0.24098E+00_JPRB, 0.23522E+00_JPRB ,&
&  0.23490E+00_JPRB, 0.23763E+00_JPRB, 0.24090E+00_JPRB, 0.24222E+00_JPRB, 0.24499E+00_JPRB, 0.24615E+00_JPRB, 0.24563E+00_JPRB ,&
&  0.24222E+00_JPRB, 0.23960E+00_JPRB, 0.24084E+00_JPRB, 0.24228E+00_JPRB, 0.24291E+00_JPRB, 0.24440E+00_JPRB, 0.24585E+00_JPRB ,&
&  0.24890E+00_JPRB, 0.25111E+00_JPRB, 0.25361E+00_JPRB, 0.25637E+00_JPRB, 0.25852E+00_JPRB, 0.25907E+00_JPRB, 0.25684E+00_JPRB ,&
&  0.25698E+00_JPRB, 0.26314E+00_JPRB, 0.27359E+00_JPRB, 0.28336E+00_JPRB, 0.28636E+00_JPRB, 0.28901E+00_JPRB, 0.29127E+00_JPRB ,&
&  0.29246E+00_JPRB, 0.29616E+00_JPRB, 0.29918E+00_JPRB, 0.30068E+00_JPRB, 0.30154E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.29259E+00_JPRB, 0.29188E+00_JPRB, 0.29038E+00_JPRB, 0.28914E+00_JPRB, 0.28748E+00_JPRB, 0.28597E+00_JPRB, 0.28669E+00_JPRB ,&
&  0.28953E+00_JPRB, 0.29328E+00_JPRB, 0.29691E+00_JPRB, 0.29968E+00_JPRB, 0.30181E+00_JPRB, 0.30290E+00_JPRB, 0.30455E+00_JPRB ,&
&  0.30643E+00_JPRB, 0.30985E+00_JPRB, 0.31143E+00_JPRB, 0.31279E+00_JPRB, 0.31207E+00_JPRB, 0.31127E+00_JPRB, 0.31236E+00_JPRB ,&
&  0.31563E+00_JPRB, 0.31699E+00_JPRB, 0.31595E+00_JPRB, 0.31361E+00_JPRB, 0.31413E+00_JPRB, 0.31640E+00_JPRB, 0.31618E+00_JPRB ,&
&  0.30907E+00_JPRB, 0.29485E+00_JPRB, 0.28560E+00_JPRB, 0.28933E+00_JPRB, 0.29775E+00_JPRB, 0.30159E+00_JPRB, 0.30462E+00_JPRB ,&
&  0.30655E+00_JPRB, 0.30671E+00_JPRB, 0.30574E+00_JPRB, 0.30393E+00_JPRB, 0.30091E+00_JPRB, 0.29567E+00_JPRB, 0.28955E+00_JPRB ,&
&  0.28420E+00_JPRB, 0.28173E+00_JPRB, 0.27840E+00_JPRB, 0.27230E+00_JPRB, 0.26651E+00_JPRB, 0.26298E+00_JPRB, 0.25832E+00_JPRB ,&
&  0.25346E+00_JPRB, 0.25009E+00_JPRB, 0.24937E+00_JPRB, 0.24880E+00_JPRB, 0.24846E+00_JPRB, 0.24748E+00_JPRB, 0.24454E+00_JPRB ,&
&  0.24145E+00_JPRB, 0.23684E+00_JPRB, 0.23110E+00_JPRB, 0.22580E+00_JPRB, 0.22222E+00_JPRB, 0.21843E+00_JPRB, 0.21363E+00_JPRB ,&
&  0.21129E+00_JPRB, 0.21421E+00_JPRB, 0.22408E+00_JPRB, 0.23447E+00_JPRB, 0.23720E+00_JPRB, 0.22561E+00_JPRB, 0.22231E+00_JPRB ,&
&  0.22224E+00_JPRB, 0.22227E+00_JPRB, 0.22276E+00_JPRB, 0.22437E+00_JPRB, 0.22511E+00_JPRB, 0.22467E+00_JPRB, 0.22531E+00_JPRB ,&
&  0.22775E+00_JPRB, 0.23025E+00_JPRB, 0.23152E+00_JPRB, 0.23307E+00_JPRB, 0.23405E+00_JPRB, 0.23390E+00_JPRB, 0.23346E+00_JPRB ,&
&  0.23427E+00_JPRB, 0.23785E+00_JPRB, 0.24351E+00_JPRB, 0.25141E+00_JPRB, 0.25676E+00_JPRB, 0.26162E+00_JPRB, 0.26760E+00_JPRB ,&
&  0.27526E+00_JPRB, 0.28210E+00_JPRB, 0.28678E+00_JPRB, 0.29080E+00_JPRB, 0.29222E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.28313E+00_JPRB, 0.28385E+00_JPRB, 0.27951E+00_JPRB, 0.27581E+00_JPRB, 0.27393E+00_JPRB, 0.27347E+00_JPRB, 0.27505E+00_JPRB ,&
&  0.27862E+00_JPRB, 0.28154E+00_JPRB, 0.28434E+00_JPRB, 0.28681E+00_JPRB, 0.28786E+00_JPRB, 0.28760E+00_JPRB, 0.28872E+00_JPRB ,&
&  0.28987E+00_JPRB, 0.29292E+00_JPRB, 0.29513E+00_JPRB, 0.29846E+00_JPRB, 0.29879E+00_JPRB, 0.29958E+00_JPRB, 0.30089E+00_JPRB ,&
&  0.30038E+00_JPRB, 0.30058E+00_JPRB, 0.30189E+00_JPRB, 0.29927E+00_JPRB, 0.29761E+00_JPRB, 0.29518E+00_JPRB, 0.29337E+00_JPRB ,&
&  0.28630E+00_JPRB, 0.27899E+00_JPRB, 0.27262E+00_JPRB, 0.27700E+00_JPRB, 0.28464E+00_JPRB, 0.28576E+00_JPRB, 0.29016E+00_JPRB ,&
&  0.29166E+00_JPRB, 0.28988E+00_JPRB, 0.28822E+00_JPRB, 0.28478E+00_JPRB, 0.28154E+00_JPRB, 0.27779E+00_JPRB, 0.27475E+00_JPRB ,&
&  0.27125E+00_JPRB, 0.27205E+00_JPRB, 0.26728E+00_JPRB, 0.25269E+00_JPRB, 0.24572E+00_JPRB, 0.24129E+00_JPRB, 0.23363E+00_JPRB ,&
&  0.22569E+00_JPRB, 0.22386E+00_JPRB, 0.22355E+00_JPRB, 0.22066E+00_JPRB, 0.22034E+00_JPRB, 0.22068E+00_JPRB, 0.22053E+00_JPRB ,&
&  0.21894E+00_JPRB, 0.21863E+00_JPRB, 0.21720E+00_JPRB, 0.21501E+00_JPRB, 0.21282E+00_JPRB, 0.21188E+00_JPRB, 0.21228E+00_JPRB ,&
&  0.21388E+00_JPRB, 0.21543E+00_JPRB, 0.22140E+00_JPRB, 0.22491E+00_JPRB, 0.21905E+00_JPRB, 0.21695E+00_JPRB, 0.21825E+00_JPRB ,&
&  0.21377E+00_JPRB, 0.21301E+00_JPRB, 0.21302E+00_JPRB, 0.21302E+00_JPRB, 0.21351E+00_JPRB, 0.21451E+00_JPRB, 0.21429E+00_JPRB ,&
&  0.21504E+00_JPRB, 0.21668E+00_JPRB, 0.21716E+00_JPRB, 0.21720E+00_JPRB, 0.21684E+00_JPRB, 0.21732E+00_JPRB, 0.21870E+00_JPRB ,&
&  0.22026E+00_JPRB, 0.22334E+00_JPRB, 0.22652E+00_JPRB, 0.23124E+00_JPRB, 0.23504E+00_JPRB, 0.23811E+00_JPRB, 0.24379E+00_JPRB ,&
&  0.24833E+00_JPRB, 0.25865E+00_JPRB, 0.27560E+00_JPRB, 0.28421E+00_JPRB, 0.27844E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.27248E+00_JPRB, 0.27105E+00_JPRB, 0.26921E+00_JPRB, 0.26758E+00_JPRB, 0.26855E+00_JPRB, 0.26983E+00_JPRB, 0.27169E+00_JPRB ,&
&  0.27373E+00_JPRB, 0.27437E+00_JPRB, 0.27435E+00_JPRB, 0.27180E+00_JPRB, 0.27053E+00_JPRB, 0.26487E+00_JPRB, 0.26296E+00_JPRB ,&
&  0.26160E+00_JPRB, 0.26404E+00_JPRB, 0.26718E+00_JPRB, 0.27182E+00_JPRB, 0.27361E+00_JPRB, 0.27550E+00_JPRB, 0.27434E+00_JPRB ,&
&  0.27197E+00_JPRB, 0.26918E+00_JPRB, 0.26638E+00_JPRB, 0.26418E+00_JPRB, 0.26349E+00_JPRB, 0.26145E+00_JPRB, 0.25939E+00_JPRB ,&
&  0.25940E+00_JPRB, 0.26098E+00_JPRB, 0.26166E+00_JPRB, 0.26237E+00_JPRB, 0.26812E+00_JPRB, 0.26827E+00_JPRB, 0.27069E+00_JPRB ,&
&  0.27157E+00_JPRB, 0.27258E+00_JPRB, 0.27229E+00_JPRB, 0.27105E+00_JPRB, 0.26991E+00_JPRB, 0.26354E+00_JPRB, 0.26106E+00_JPRB ,&
&  0.25414E+00_JPRB, 0.25019E+00_JPRB, 0.24310E+00_JPRB, 0.23647E+00_JPRB, 0.22878E+00_JPRB, 0.22302E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21418E+00_JPRB, 0.21122E+00_JPRB, 0.20886E+00_JPRB, 0.20856E+00_JPRB, 0.20860E+00_JPRB, 0.20807E+00_JPRB, 0.20825E+00_JPRB ,&
&  0.20928E+00_JPRB, 0.21039E+00_JPRB, 0.21027E+00_JPRB, 0.21089E+00_JPRB, 0.21141E+00_JPRB, 0.21218E+00_JPRB, 0.21377E+00_JPRB ,&
&  0.21620E+00_JPRB, 0.21816E+00_JPRB, 0.21867E+00_JPRB, 0.22093E+00_JPRB, 0.22026E+00_JPRB, 0.21743E+00_JPRB, 0.21470E+00_JPRB ,&
&  0.21143E+00_JPRB, 0.21003E+00_JPRB, 0.20837E+00_JPRB, 0.20743E+00_JPRB, 0.20728E+00_JPRB, 0.20744E+00_JPRB, 0.20829E+00_JPRB ,&
&  0.20922E+00_JPRB, 0.21020E+00_JPRB, 0.21054E+00_JPRB, 0.21098E+00_JPRB, 0.21132E+00_JPRB, 0.21225E+00_JPRB, 0.21409E+00_JPRB ,&
&  0.21666E+00_JPRB, 0.21963E+00_JPRB, 0.22254E+00_JPRB, 0.22604E+00_JPRB, 0.23177E+00_JPRB, 0.23921E+00_JPRB, 0.25042E+00_JPRB ,&
&  0.25866E+00_JPRB, 0.26984E+00_JPRB, 0.26823E+00_JPRB, 0.27185E+00_JPRB, 0.27320E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.26442E+00_JPRB, 0.26427E+00_JPRB, 0.26324E+00_JPRB, 0.26265E+00_JPRB, 0.26312E+00_JPRB, 0.26398E+00_JPRB, 0.26527E+00_JPRB ,&
&  0.26187E+00_JPRB, 0.25965E+00_JPRB, 0.25531E+00_JPRB, 0.25054E+00_JPRB, 0.24626E+00_JPRB, 0.24318E+00_JPRB, 0.24147E+00_JPRB ,&
&  0.24137E+00_JPRB, 0.24387E+00_JPRB, 0.24559E+00_JPRB, 0.24792E+00_JPRB, 0.24800E+00_JPRB, 0.24818E+00_JPRB, 0.24649E+00_JPRB ,&
&  0.24470E+00_JPRB, 0.24402E+00_JPRB, 0.24425E+00_JPRB, 0.24570E+00_JPRB, 0.24832E+00_JPRB, 0.24984E+00_JPRB, 0.25180E+00_JPRB ,&
&  0.25449E+00_JPRB, 0.25340E+00_JPRB, 0.25406E+00_JPRB, 0.25335E+00_JPRB, 0.25398E+00_JPRB, 0.25532E+00_JPRB, 0.25610E+00_JPRB ,&
&  0.25760E+00_JPRB, 0.25899E+00_JPRB, 0.25924E+00_JPRB, 0.25849E+00_JPRB, 0.25400E+00_JPRB, 0.25187E+00_JPRB, 0.24727E+00_JPRB ,&
&  0.24158E+00_JPRB, 0.23635E+00_JPRB, 0.22996E+00_JPRB, 0.22542E+00_JPRB, 0.21984E+00_JPRB, 0.21658E+00_JPRB, 0.21405E+00_JPRB ,&
&  0.21167E+00_JPRB, 0.21082E+00_JPRB, 0.20956E+00_JPRB, 0.20814E+00_JPRB, 0.20777E+00_JPRB, 0.20631E+00_JPRB, 0.20622E+00_JPRB ,&
&  0.20662E+00_JPRB, 0.20698E+00_JPRB, 0.20816E+00_JPRB, 0.20863E+00_JPRB, 0.20947E+00_JPRB, 0.21019E+00_JPRB, 0.21065E+00_JPRB ,&
&  0.21182E+00_JPRB, 0.21348E+00_JPRB, 0.21343E+00_JPRB, 0.21390E+00_JPRB, 0.21305E+00_JPRB, 0.21100E+00_JPRB, 0.20882E+00_JPRB ,&
&  0.20642E+00_JPRB, 0.20457E+00_JPRB, 0.20402E+00_JPRB, 0.20319E+00_JPRB, 0.20332E+00_JPRB, 0.20372E+00_JPRB, 0.20383E+00_JPRB ,&
&  0.20452E+00_JPRB, 0.20514E+00_JPRB, 0.20608E+00_JPRB, 0.20827E+00_JPRB, 0.20972E+00_JPRB, 0.21165E+00_JPRB, 0.21371E+00_JPRB ,&
&  0.21533E+00_JPRB, 0.21834E+00_JPRB, 0.22099E+00_JPRB, 0.22462E+00_JPRB, 0.23309E+00_JPRB, 0.24002E+00_JPRB, 0.25247E+00_JPRB ,&
&  0.25660E+00_JPRB, 0.26537E+00_JPRB, 0.26200E+00_JPRB, 0.26367E+00_JPRB, 0.26491E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.25897E+00_JPRB, 0.25987E+00_JPRB, 0.25964E+00_JPRB, 0.26064E+00_JPRB, 0.26136E+00_JPRB, 0.26246E+00_JPRB, 0.26230E+00_JPRB ,&
&  0.26195E+00_JPRB, 0.25816E+00_JPRB, 0.25610E+00_JPRB, 0.25263E+00_JPRB, 0.24949E+00_JPRB, 0.24604E+00_JPRB, 0.24327E+00_JPRB ,&
&  0.24152E+00_JPRB, 0.24015E+00_JPRB, 0.23891E+00_JPRB, 0.23716E+00_JPRB, 0.23492E+00_JPRB, 0.23264E+00_JPRB, 0.23058E+00_JPRB ,&
&  0.22868E+00_JPRB, 0.22886E+00_JPRB, 0.23032E+00_JPRB, 0.23268E+00_JPRB, 0.23705E+00_JPRB, 0.24006E+00_JPRB, 0.24396E+00_JPRB ,&
&  0.24965E+00_JPRB, 0.24539E+00_JPRB, 0.24565E+00_JPRB, 0.24622E+00_JPRB, 0.24618E+00_JPRB, 0.24623E+00_JPRB, 0.24411E+00_JPRB ,&
&  0.24915E+00_JPRB, 0.24948E+00_JPRB, 0.25352E+00_JPRB, 0.25221E+00_JPRB, 0.24705E+00_JPRB, 0.24354E+00_JPRB, 0.23834E+00_JPRB ,&
&  0.23492E+00_JPRB, 0.23004E+00_JPRB, 0.22676E+00_JPRB, 0.22362E+00_JPRB, 0.21963E+00_JPRB, 0.21801E+00_JPRB, 0.21523E+00_JPRB ,&
&  0.21361E+00_JPRB, 0.21328E+00_JPRB, 0.21216E+00_JPRB, 0.21216E+00_JPRB, 0.21116E+00_JPRB, 0.20977E+00_JPRB, 0.20889E+00_JPRB ,&
&  0.20814E+00_JPRB, 0.20734E+00_JPRB, 0.20709E+00_JPRB, 0.20678E+00_JPRB, 0.20692E+00_JPRB, 0.20689E+00_JPRB, 0.20688E+00_JPRB ,&
&  0.20705E+00_JPRB, 0.20721E+00_JPRB, 0.20671E+00_JPRB, 0.20661E+00_JPRB, 0.20559E+00_JPRB, 0.20467E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.20230E+00_JPRB, 0.20171E+00_JPRB, 0.20074E+00_JPRB, 0.20011E+00_JPRB, 0.19985E+00_JPRB, 0.20035E+00_JPRB, 0.20100E+00_JPRB ,&
&  0.20178E+00_JPRB, 0.20331E+00_JPRB, 0.20456E+00_JPRB, 0.20624E+00_JPRB, 0.20809E+00_JPRB, 0.20951E+00_JPRB, 0.21159E+00_JPRB ,&
&  0.21363E+00_JPRB, 0.21575E+00_JPRB, 0.21944E+00_JPRB, 0.22344E+00_JPRB, 0.22871E+00_JPRB, 0.23395E+00_JPRB, 0.23609E+00_JPRB ,&
&  0.23833E+00_JPRB, 0.24423E+00_JPRB, 0.24696E+00_JPRB, 0.25651E+00_JPRB, 0.25803E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.24798E+00_JPRB, 0.24976E+00_JPRB, 0.25150E+00_JPRB, 0.25317E+00_JPRB, 0.25521E+00_JPRB, 0.25607E+00_JPRB, 0.25613E+00_JPRB ,&
&  0.25369E+00_JPRB, 0.25328E+00_JPRB, 0.25178E+00_JPRB, 0.24975E+00_JPRB, 0.24809E+00_JPRB, 0.24445E+00_JPRB, 0.24136E+00_JPRB ,&
&  0.23832E+00_JPRB, 0.23542E+00_JPRB, 0.23294E+00_JPRB, 0.22948E+00_JPRB, 0.22621E+00_JPRB, 0.22291E+00_JPRB, 0.22224E+00_JPRB ,&
&  0.22208E+00_JPRB, 0.22288E+00_JPRB, 0.22499E+00_JPRB, 0.22652E+00_JPRB, 0.22877E+00_JPRB, 0.23113E+00_JPRB, 0.23464E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.24010E+00_JPRB, 0.24032E+00_JPRB, 0.23897E+00_JPRB, 0.23811E+00_JPRB, 0.23804E+00_JPRB, 0.23787E+00_JPRB ,&
&  0.23736E+00_JPRB, 0.23733E+00_JPRB, 0.23699E+00_JPRB, 0.23676E+00_JPRB, 0.23544E+00_JPRB, 0.23280E+00_JPRB, 0.23003E+00_JPRB ,&
&  0.22743E+00_JPRB, 0.22509E+00_JPRB, 0.22278E+00_JPRB, 0.22074E+00_JPRB, 0.21954E+00_JPRB, 0.21831E+00_JPRB, 0.21668E+00_JPRB ,&
&  0.21574E+00_JPRB, 0.21475E+00_JPRB, 0.21366E+00_JPRB, 0.21286E+00_JPRB, 0.21200E+00_JPRB, 0.21141E+00_JPRB, 0.21073E+00_JPRB ,&
&  0.21025E+00_JPRB, 0.20944E+00_JPRB, 0.20876E+00_JPRB, 0.20801E+00_JPRB, 0.20770E+00_JPRB, 0.20680E+00_JPRB, 0.20593E+00_JPRB ,&
&  0.20483E+00_JPRB, 0.20416E+00_JPRB, 0.20288E+00_JPRB, 0.20212E+00_JPRB, 0.20122E+00_JPRB, 0.20037E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19967E+00_JPRB, 0.19951E+00_JPRB, 0.19878E+00_JPRB, 0.19850E+00_JPRB, 0.19840E+00_JPRB, 0.19882E+00_JPRB, 0.20033E+00_JPRB ,&
&  0.20129E+00_JPRB, 0.20234E+00_JPRB, 0.20334E+00_JPRB, 0.20408E+00_JPRB, 0.20541E+00_JPRB, 0.20668E+00_JPRB, 0.20797E+00_JPRB ,&
&  0.20953E+00_JPRB, 0.21083E+00_JPRB, 0.21235E+00_JPRB, 0.21411E+00_JPRB, 0.21869E+00_JPRB, 0.22152E+00_JPRB, 0.22433E+00_JPRB ,&
&  0.22600E+00_JPRB, 0.22348E+00_JPRB, 0.23861E+00_JPRB, 0.24141E+00_JPRB, 0.24360E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.22252E+00_JPRB, 0.22552E+00_JPRB, 0.22714E+00_JPRB, 0.22821E+00_JPRB, 0.21975E+00_JPRB, 0.23147E+00_JPRB, 0.22804E+00_JPRB ,&
&  0.22763E+00_JPRB, 0.22729E+00_JPRB, 0.22965E+00_JPRB, 0.23012E+00_JPRB, 0.22976E+00_JPRB, 0.22907E+00_JPRB, 0.22805E+00_JPRB ,&
&  0.22703E+00_JPRB, 0.22566E+00_JPRB, 0.22488E+00_JPRB, 0.22400E+00_JPRB, 0.22330E+00_JPRB, 0.22259E+00_JPRB, 0.22210E+00_JPRB ,&
&  0.22222E+00_JPRB, 0.22324E+00_JPRB, 0.22499E+00_JPRB, 0.22422E+00_JPRB, 0.22413E+00_JPRB, 0.22448E+00_JPRB, 0.22509E+00_JPRB ,&
&  0.22530E+00_JPRB, 0.22533E+00_JPRB, 0.22495E+00_JPRB, 0.22368E+00_JPRB, 0.22390E+00_JPRB, 0.22418E+00_JPRB, 0.22454E+00_JPRB ,&
&  0.22467E+00_JPRB, 0.22454E+00_JPRB, 0.22370E+00_JPRB, 0.22291E+00_JPRB, 0.22209E+00_JPRB, 0.22106E+00_JPRB, 0.21977E+00_JPRB ,&
&  0.21871E+00_JPRB, 0.21790E+00_JPRB, 0.21687E+00_JPRB, 0.21583E+00_JPRB, 0.21494E+00_JPRB, 0.21406E+00_JPRB, 0.21320E+00_JPRB ,&
&  0.21265E+00_JPRB, 0.21225E+00_JPRB, 0.21180E+00_JPRB, 0.21115E+00_JPRB, 0.21043E+00_JPRB, 0.20975E+00_JPRB, 0.20915E+00_JPRB ,&
&  0.20876E+00_JPRB, 0.20809E+00_JPRB, 0.20742E+00_JPRB, 0.20669E+00_JPRB, 0.20614E+00_JPRB, 0.20521E+00_JPRB, 0.20440E+00_JPRB ,&
&  0.20360E+00_JPRB, 0.20288E+00_JPRB, 0.20231E+00_JPRB, 0.20188E+00_JPRB, 0.20148E+00_JPRB, 0.20117E+00_JPRB, 0.20091E+00_JPRB ,&
&  0.20070E+00_JPRB, 0.20051E+00_JPRB, 0.20041E+00_JPRB, 0.20024E+00_JPRB, 0.20004E+00_JPRB, 0.19994E+00_JPRB, 0.20021E+00_JPRB ,&
&  0.20045E+00_JPRB, 0.20059E+00_JPRB, 0.20065E+00_JPRB, 0.20023E+00_JPRB, 0.20066E+00_JPRB, 0.20104E+00_JPRB, 0.20150E+00_JPRB ,&
&  0.20190E+00_JPRB, 0.20280E+00_JPRB, 0.20388E+00_JPRB, 0.20521E+00_JPRB, 0.20783E+00_JPRB, 0.21008E+00_JPRB, 0.21249E+00_JPRB ,&
&  0.21518E+00_JPRB, 0.21914E+00_JPRB, 0.22062E+00_JPRB, 0.22151E+00_JPRB, 0.22210E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.21110E+00_JPRB, 0.21095E+00_JPRB, 0.21062E+00_JPRB, 0.20959E+00_JPRB, 0.20670E+00_JPRB, 0.20615E+00_JPRB, 0.20610E+00_JPRB ,&
&  0.20628E+00_JPRB, 0.20640E+00_JPRB, 0.20748E+00_JPRB, 0.20863E+00_JPRB, 0.20978E+00_JPRB, 0.21067E+00_JPRB, 0.21144E+00_JPRB ,&
&  0.21187E+00_JPRB, 0.21230E+00_JPRB, 0.21092E+00_JPRB, 0.21025E+00_JPRB, 0.21004E+00_JPRB, 0.21003E+00_JPRB, 0.21002E+00_JPRB ,&
&  0.20996E+00_JPRB, 0.20994E+00_JPRB, 0.20981E+00_JPRB, 0.21008E+00_JPRB, 0.21048E+00_JPRB, 0.21079E+00_JPRB, 0.21117E+00_JPRB ,&
&  0.21146E+00_JPRB, 0.21172E+00_JPRB, 0.21190E+00_JPRB, 0.21204E+00_JPRB, 0.21246E+00_JPRB, 0.21271E+00_JPRB, 0.21268E+00_JPRB ,&
&  0.21253E+00_JPRB, 0.21218E+00_JPRB, 0.21178E+00_JPRB, 0.21135E+00_JPRB, 0.21106E+00_JPRB, 0.21077E+00_JPRB, 0.21047E+00_JPRB ,&
&  0.21016E+00_JPRB, 0.20985E+00_JPRB, 0.20950E+00_JPRB, 0.20910E+00_JPRB, 0.20871E+00_JPRB, 0.20844E+00_JPRB, 0.20813E+00_JPRB ,&
&  0.20787E+00_JPRB, 0.20761E+00_JPRB, 0.20734E+00_JPRB, 0.20698E+00_JPRB, 0.20660E+00_JPRB, 0.20621E+00_JPRB, 0.20578E+00_JPRB ,&
&  0.20523E+00_JPRB, 0.20483E+00_JPRB, 0.20444E+00_JPRB, 0.20410E+00_JPRB, 0.20380E+00_JPRB, 0.20348E+00_JPRB, 0.20319E+00_JPRB ,&
&  0.20291E+00_JPRB, 0.20262E+00_JPRB, 0.20229E+00_JPRB, 0.20198E+00_JPRB, 0.20170E+00_JPRB, 0.20144E+00_JPRB, 0.20122E+00_JPRB ,&
&  0.20101E+00_JPRB, 0.20079E+00_JPRB, 0.20060E+00_JPRB, 0.20057E+00_JPRB, 0.20051E+00_JPRB, 0.20051E+00_JPRB, 0.20051E+00_JPRB ,&
&  0.20052E+00_JPRB, 0.20052E+00_JPRB, 0.20050E+00_JPRB, 0.20034E+00_JPRB, 0.20049E+00_JPRB, 0.20072E+00_JPRB, 0.20108E+00_JPRB ,&
&  0.20150E+00_JPRB, 0.20206E+00_JPRB, 0.20265E+00_JPRB, 0.20329E+00_JPRB, 0.20404E+00_JPRB, 0.20466E+00_JPRB, 0.20535E+00_JPRB ,&
&  0.20612E+00_JPRB, 0.20692E+00_JPRB, 0.20749E+00_JPRB, 0.20816E+00_JPRB, 0.20919E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB, 0.20421E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB, 0.69233E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.70354E+00_JPRB, 0.70246E+00_JPRB, 0.70157E+00_JPRB, 0.70108E+00_JPRB, 0.70086E+00_JPRB, 0.70064E+00_JPRB, 0.69947E+00_JPRB ,&
&  0.69805E+00_JPRB, 0.69626E+00_JPRB, 0.69437E+00_JPRB, 0.69209E+00_JPRB, 0.68928E+00_JPRB, 0.68595E+00_JPRB, 0.68292E+00_JPRB ,&
&  0.68008E+00_JPRB, 0.67774E+00_JPRB, 0.67529E+00_JPRB, 0.67356E+00_JPRB, 0.67158E+00_JPRB, 0.66955E+00_JPRB, 0.66743E+00_JPRB ,&
&  0.66573E+00_JPRB, 0.66415E+00_JPRB, 0.66265E+00_JPRB, 0.66132E+00_JPRB, 0.66072E+00_JPRB, 0.65996E+00_JPRB, 0.65917E+00_JPRB ,&
&  0.65846E+00_JPRB, 0.65803E+00_JPRB, 0.65767E+00_JPRB, 0.65730E+00_JPRB, 0.65701E+00_JPRB, 0.65714E+00_JPRB, 0.65727E+00_JPRB ,&
&  0.65761E+00_JPRB, 0.65855E+00_JPRB, 0.65999E+00_JPRB, 0.66131E+00_JPRB, 0.66278E+00_JPRB, 0.66486E+00_JPRB, 0.66740E+00_JPRB ,&
&  0.66964E+00_JPRB, 0.67177E+00_JPRB, 0.67424E+00_JPRB, 0.67709E+00_JPRB, 0.67983E+00_JPRB, 0.68279E+00_JPRB, 0.68561E+00_JPRB ,&
&  0.68843E+00_JPRB, 0.69071E+00_JPRB, 0.69297E+00_JPRB, 0.69406E+00_JPRB, 0.69562E+00_JPRB, 0.69692E+00_JPRB, 0.69844E+00_JPRB ,&
&  0.69884E+00_JPRB, 0.69907E+00_JPRB, 0.69942E+00_JPRB, 0.70004E+00_JPRB, 0.70042E+00_JPRB, 0.70093E+00_JPRB, 0.70156E+00_JPRB ,&
&  0.70245E+00_JPRB, 0.70293E+00_JPRB, 0.70368E+00_JPRB, 0.70420E+00_JPRB, 0.70490E+00_JPRB, 0.70506E+00_JPRB, 0.70554E+00_JPRB ,&
&  0.70606E+00_JPRB, 0.70687E+00_JPRB, 0.70701E+00_JPRB, 0.70724E+00_JPRB, 0.70739E+00_JPRB, 0.70769E+00_JPRB, 0.70796E+00_JPRB ,&
&  0.70834E+00_JPRB, 0.70862E+00_JPRB, 0.70887E+00_JPRB, 0.70866E+00_JPRB, 0.70809E+00_JPRB, 0.70758E+00_JPRB, 0.70719E+00_JPRB ,&
&  0.70703E+00_JPRB, 0.70681E+00_JPRB, 0.70698E+00_JPRB, 0.70735E+00_JPRB, 0.70808E+00_JPRB, 0.70813E+00_JPRB, 0.70816E+00_JPRB ,&
&  0.70802E+00_JPRB, 0.70780E+00_JPRB, 0.70695E+00_JPRB, 0.70588E+00_JPRB, 0.70470E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.70659E+00_JPRB, 0.70456E+00_JPRB, 0.70318E+00_JPRB, 0.70266E+00_JPRB, 0.70252E+00_JPRB, 0.70223E+00_JPRB, 0.70177E+00_JPRB ,&
&  0.69996E+00_JPRB, 0.69596E+00_JPRB, 0.69135E+00_JPRB, 0.68626E+00_JPRB, 0.68005E+00_JPRB, 0.67436E+00_JPRB, 0.67056E+00_JPRB ,&
&  0.66660E+00_JPRB, 0.66265E+00_JPRB, 0.65880E+00_JPRB, 0.65563E+00_JPRB, 0.65204E+00_JPRB, 0.64825E+00_JPRB, 0.64458E+00_JPRB ,&
&  0.64164E+00_JPRB, 0.63828E+00_JPRB, 0.63451E+00_JPRB, 0.62977E+00_JPRB, 0.62661E+00_JPRB, 0.62325E+00_JPRB, 0.62028E+00_JPRB ,&
&  0.61644E+00_JPRB, 0.61462E+00_JPRB, 0.61367E+00_JPRB, 0.61374E+00_JPRB, 0.61688E+00_JPRB, 0.61924E+00_JPRB, 0.61956E+00_JPRB ,&
&  0.61872E+00_JPRB, 0.61503E+00_JPRB, 0.61540E+00_JPRB, 0.61643E+00_JPRB, 0.61864E+00_JPRB, 0.62334E+00_JPRB, 0.62942E+00_JPRB ,&
&  0.63812E+00_JPRB, 0.64864E+00_JPRB, 0.65950E+00_JPRB, 0.66784E+00_JPRB, 0.67602E+00_JPRB, 0.68358E+00_JPRB, 0.68920E+00_JPRB ,&
&  0.69657E+00_JPRB, 0.70291E+00_JPRB, 0.70930E+00_JPRB, 0.71495E+00_JPRB, 0.72141E+00_JPRB, 0.72651E+00_JPRB, 0.73029E+00_JPRB ,&
&  0.73129E+00_JPRB, 0.73097E+00_JPRB, 0.73064E+00_JPRB, 0.73055E+00_JPRB, 0.72916E+00_JPRB, 0.72841E+00_JPRB, 0.72854E+00_JPRB ,&
&  0.72936E+00_JPRB, 0.72962E+00_JPRB, 0.73105E+00_JPRB, 0.73263E+00_JPRB, 0.73447E+00_JPRB, 0.73421E+00_JPRB, 0.73448E+00_JPRB ,&
&  0.73445E+00_JPRB, 0.73470E+00_JPRB, 0.73357E+00_JPRB, 0.73123E+00_JPRB, 0.72935E+00_JPRB, 0.72867E+00_JPRB, 0.72726E+00_JPRB ,&
&  0.72583E+00_JPRB, 0.72477E+00_JPRB, 0.72443E+00_JPRB, 0.72341E+00_JPRB, 0.72165E+00_JPRB, 0.72007E+00_JPRB, 0.71942E+00_JPRB ,&
&  0.71862E+00_JPRB, 0.71740E+00_JPRB, 0.71734E+00_JPRB, 0.71749E+00_JPRB, 0.71948E+00_JPRB, 0.71967E+00_JPRB, 0.71953E+00_JPRB ,&
&  0.71839E+00_JPRB, 0.71757E+00_JPRB, 0.71551E+00_JPRB, 0.71260E+00_JPRB, 0.70916E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.71967E+00_JPRB, 0.71873E+00_JPRB, 0.71961E+00_JPRB, 0.71838E+00_JPRB, 0.71616E+00_JPRB, 0.71239E+00_JPRB, 0.71002E+00_JPRB ,&
&  0.70567E+00_JPRB, 0.69876E+00_JPRB, 0.68835E+00_JPRB, 0.67947E+00_JPRB, 0.67070E+00_JPRB, 0.66498E+00_JPRB, 0.65962E+00_JPRB ,&
&  0.65425E+00_JPRB, 0.64981E+00_JPRB, 0.64507E+00_JPRB, 0.63920E+00_JPRB, 0.63189E+00_JPRB, 0.62328E+00_JPRB, 0.61197E+00_JPRB ,&
&  0.60360E+00_JPRB, 0.59557E+00_JPRB, 0.58881E+00_JPRB, 0.58432E+00_JPRB, 0.58346E+00_JPRB, 0.56379E+00_JPRB, 0.56063E+00_JPRB ,&
&  0.57047E+00_JPRB, 0.55952E+00_JPRB, 0.54899E+00_JPRB, 0.53189E+00_JPRB, 0.52688E+00_JPRB, 0.53615E+00_JPRB, 0.54317E+00_JPRB ,&
&  0.53621E+00_JPRB, 0.56433E+00_JPRB, 0.56678E+00_JPRB, 0.56780E+00_JPRB, 0.56559E+00_JPRB, 0.56047E+00_JPRB, 0.56972E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.60195E+00_JPRB, 0.62289E+00_JPRB, 0.64582E+00_JPRB, 0.66131E+00_JPRB, 0.67761E+00_JPRB, 0.69417E+00_JPRB ,&
&  0.71057E+00_JPRB, 0.72317E+00_JPRB, 0.73354E+00_JPRB, 0.74305E+00_JPRB, 0.75669E+00_JPRB, 0.77014E+00_JPRB, 0.78191E+00_JPRB ,&
&  0.79009E+00_JPRB, 0.79514E+00_JPRB, 0.80115E+00_JPRB, 0.80705E+00_JPRB, 0.81533E+00_JPRB, 0.82256E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.83575E+00_JPRB, 0.84053E+00_JPRB, 0.84355E+00_JPRB, 0.84481E+00_JPRB, 0.84505E+00_JPRB, 0.84217E+00_JPRB, 0.84147E+00_JPRB ,&
&  0.83619E+00_JPRB, 0.83014E+00_JPRB, 0.82222E+00_JPRB, 0.81191E+00_JPRB, 0.80432E+00_JPRB, 0.79841E+00_JPRB, 0.79167E+00_JPRB ,&
&  0.78465E+00_JPRB, 0.77759E+00_JPRB, 0.77130E+00_JPRB, 0.76352E+00_JPRB, 0.75624E+00_JPRB, 0.74881E+00_JPRB, 0.74379E+00_JPRB ,&
&  0.74023E+00_JPRB, 0.73745E+00_JPRB, 0.73661E+00_JPRB, 0.73540E+00_JPRB, 0.73608E+00_JPRB, 0.73567E+00_JPRB, 0.73668E+00_JPRB ,&
&  0.73628E+00_JPRB, 0.73283E+00_JPRB, 0.72755E+00_JPRB, 0.72350E+00_JPRB, 0.72022E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.72515E+00_JPRB, 0.72657E+00_JPRB, 0.73147E+00_JPRB, 0.72984E+00_JPRB, 0.72662E+00_JPRB, 0.71832E+00_JPRB, 0.71398E+00_JPRB ,&
&  0.70568E+00_JPRB, 0.69767E+00_JPRB, 0.68137E+00_JPRB, 0.67089E+00_JPRB, 0.66126E+00_JPRB, 0.65540E+00_JPRB, 0.65067E+00_JPRB ,&
&  0.64695E+00_JPRB, 0.64205E+00_JPRB, 0.63599E+00_JPRB, 0.62688E+00_JPRB, 0.61543E+00_JPRB, 0.60234E+00_JPRB, 0.58394E+00_JPRB ,&
&  0.56823E+00_JPRB, 0.55715E+00_JPRB, 0.54238E+00_JPRB, 0.54782E+00_JPRB, 0.54086E+00_JPRB, 0.53552E+00_JPRB, 0.53207E+00_JPRB ,&
&  0.52257E+00_JPRB, 0.51058E+00_JPRB, 0.49365E+00_JPRB, 0.48422E+00_JPRB, 0.47739E+00_JPRB, 0.46937E+00_JPRB, 0.48118E+00_JPRB ,&
&  0.49009E+00_JPRB, 0.51487E+00_JPRB, 0.52510E+00_JPRB, 0.52766E+00_JPRB, 0.53137E+00_JPRB, 0.53088E+00_JPRB, 0.53859E+00_JPRB ,&
&  0.54875E+00_JPRB, 0.56141E+00_JPRB, 0.60336E+00_JPRB, 0.62833E+00_JPRB, 0.64422E+00_JPRB, 0.66093E+00_JPRB, 0.67382E+00_JPRB ,&
&  0.69020E+00_JPRB, 0.70184E+00_JPRB, 0.71294E+00_JPRB, 0.72679E+00_JPRB, 0.74705E+00_JPRB, 0.77062E+00_JPRB, 0.79153E+00_JPRB ,&
&  0.81025E+00_JPRB, 0.82204E+00_JPRB, 0.83716E+00_JPRB, 0.85451E+00_JPRB, 0.87814E+00_JPRB, 0.90423E+00_JPRB, 0.93190E+00_JPRB ,&
&  0.96053E+00_JPRB, 0.98949E+00_JPRB, 0.10131E+01_JPRB, 0.10341E+01_JPRB, 0.10451E+01_JPRB, 0.10474E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.10121E+01_JPRB, 0.98824E+00_JPRB, 0.95149E+00_JPRB, 0.92371E+00_JPRB, 0.90896E+00_JPRB, 0.88119E+00_JPRB ,&
&  0.85812E+00_JPRB, 0.83677E+00_JPRB, 0.81750E+00_JPRB, 0.79667E+00_JPRB, 0.77798E+00_JPRB, 0.76053E+00_JPRB, 0.74866E+00_JPRB ,&
&  0.73932E+00_JPRB, 0.73365E+00_JPRB, 0.73045E+00_JPRB, 0.72889E+00_JPRB, 0.73009E+00_JPRB, 0.73073E+00_JPRB, 0.73469E+00_JPRB ,&
&  0.73578E+00_JPRB, 0.73392E+00_JPRB, 0.72863E+00_JPRB, 0.72566E+00_JPRB, 0.72352E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.73240E+00_JPRB, 0.73256E+00_JPRB, 0.73081E+00_JPRB, 0.72184E+00_JPRB, 0.71580E+00_JPRB, 0.70858E+00_JPRB, 0.70592E+00_JPRB ,&
&  0.69729E+00_JPRB, 0.68969E+00_JPRB, 0.67344E+00_JPRB, 0.66352E+00_JPRB, 0.65234E+00_JPRB, 0.64611E+00_JPRB, 0.64006E+00_JPRB ,&
&  0.63589E+00_JPRB, 0.63044E+00_JPRB, 0.62664E+00_JPRB, 0.61774E+00_JPRB, 0.60913E+00_JPRB, 0.59556E+00_JPRB, 0.58090E+00_JPRB ,&
&  0.56521E+00_JPRB, 0.54943E+00_JPRB, 0.53652E+00_JPRB, 0.53859E+00_JPRB, 0.53184E+00_JPRB, 0.52691E+00_JPRB, 0.52211E+00_JPRB ,&
&  0.52079E+00_JPRB, 0.48536E+00_JPRB, 0.49278E+00_JPRB, 0.48016E+00_JPRB, 0.46614E+00_JPRB, 0.44876E+00_JPRB, 0.43485E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.44046E+00_JPRB, 0.45102E+00_JPRB, 0.46741E+00_JPRB, 0.47476E+00_JPRB, 0.48884E+00_JPRB, 0.49890E+00_JPRB ,&
&  0.49622E+00_JPRB, 0.56529E+00_JPRB, 0.59999E+00_JPRB, 0.61962E+00_JPRB, 0.63952E+00_JPRB, 0.65692E+00_JPRB, 0.67029E+00_JPRB ,&
&  0.69155E+00_JPRB, 0.71583E+00_JPRB, 0.74066E+00_JPRB, 0.76519E+00_JPRB, 0.80135E+00_JPRB, 0.82692E+00_JPRB, 0.85014E+00_JPRB ,&
&  0.86964E+00_JPRB, 0.87841E+00_JPRB, 0.89404E+00_JPRB, 0.91774E+00_JPRB, 0.94874E+00_JPRB, 0.98363E+00_JPRB, 0.10228E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.11096E+01_JPRB, 0.11545E+01_JPRB, 0.11932E+01_JPRB, 0.12233E+01_JPRB, 0.12424E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.12271E+01_JPRB, 0.11931E+01_JPRB, 0.11455E+01_JPRB, 0.10870E+01_JPRB, 0.10217E+01_JPRB, 0.95594E+00_JPRB ,&
&  0.92162E+00_JPRB, 0.88500E+00_JPRB, 0.85105E+00_JPRB, 0.81896E+00_JPRB, 0.79045E+00_JPRB, 0.76685E+00_JPRB, 0.75288E+00_JPRB ,&
&  0.74070E+00_JPRB, 0.73262E+00_JPRB, 0.72765E+00_JPRB, 0.72380E+00_JPRB, 0.72223E+00_JPRB, 0.71871E+00_JPRB, 0.71891E+00_JPRB ,&
&  0.72045E+00_JPRB, 0.72446E+00_JPRB, 0.72520E+00_JPRB, 0.72718E+00_JPRB, 0.72939E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.74180E+00_JPRB, 0.74477E+00_JPRB, 0.74148E+00_JPRB, 0.73568E+00_JPRB, 0.73108E+00_JPRB, 0.71989E+00_JPRB, 0.71259E+00_JPRB ,&
&  0.70676E+00_JPRB, 0.70095E+00_JPRB, 0.69012E+00_JPRB, 0.67893E+00_JPRB, 0.66449E+00_JPRB, 0.65387E+00_JPRB, 0.64223E+00_JPRB ,&
&  0.63279E+00_JPRB, 0.62157E+00_JPRB, 0.60915E+00_JPRB, 0.59751E+00_JPRB, 0.58852E+00_JPRB, 0.57743E+00_JPRB, 0.56949E+00_JPRB ,&
&  0.56044E+00_JPRB, 0.54863E+00_JPRB, 0.54020E+00_JPRB, 0.54018E+00_JPRB, 0.53260E+00_JPRB, 0.52606E+00_JPRB, 0.52749E+00_JPRB ,&
&  0.53283E+00_JPRB, 0.52788E+00_JPRB, 0.52140E+00_JPRB, 0.50653E+00_JPRB, 0.48052E+00_JPRB, 0.42505E+00_JPRB, 0.43477E+00_JPRB ,&
&  0.43011E+00_JPRB, 0.42167E+00_JPRB, 0.42498E+00_JPRB, 0.43335E+00_JPRB, 0.44233E+00_JPRB, 0.46704E+00_JPRB, 0.46987E+00_JPRB ,&
&  0.46223E+00_JPRB, 0.55961E+00_JPRB, 0.58113E+00_JPRB, 0.60663E+00_JPRB, 0.63175E+00_JPRB, 0.65435E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.69450E+00_JPRB, 0.72057E+00_JPRB, 0.74482E+00_JPRB, 0.78225E+00_JPRB, 0.81750E+00_JPRB, 0.85946E+00_JPRB, 0.89625E+00_JPRB ,&
&  0.91682E+00_JPRB, 0.92926E+00_JPRB, 0.94708E+00_JPRB, 0.96074E+00_JPRB, 0.97893E+00_JPRB, 0.10126E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.11624E+01_JPRB, 0.12479E+01_JPRB, 0.13114E+01_JPRB, 0.13440E+01_JPRB, 0.13909E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14334E+01_JPRB, 0.14280E+01_JPRB, 0.14032E+01_JPRB, 0.13012E+01_JPRB, 0.11922E+01_JPRB, 0.11179E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.99653E+00_JPRB, 0.94386E+00_JPRB, 0.88810E+00_JPRB, 0.84567E+00_JPRB, 0.81279E+00_JPRB, 0.78887E+00_JPRB, 0.77594E+00_JPRB ,&
&  0.76133E+00_JPRB, 0.74909E+00_JPRB, 0.73779E+00_JPRB, 0.73336E+00_JPRB, 0.72315E+00_JPRB, 0.71456E+00_JPRB, 0.70576E+00_JPRB ,&
&  0.70473E+00_JPRB, 0.70652E+00_JPRB, 0.71514E+00_JPRB, 0.72443E+00_JPRB, 0.73070E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.74758E+00_JPRB, 0.76452E+00_JPRB, 0.77265E+00_JPRB, 0.77593E+00_JPRB, 0.77934E+00_JPRB, 0.76296E+00_JPRB, 0.74748E+00_JPRB ,&
&  0.73548E+00_JPRB, 0.72335E+00_JPRB, 0.71155E+00_JPRB, 0.69973E+00_JPRB, 0.68726E+00_JPRB, 0.67766E+00_JPRB, 0.66874E+00_JPRB ,&
&  0.66080E+00_JPRB, 0.62292E+00_JPRB, 0.60392E+00_JPRB, 0.59015E+00_JPRB, 0.57730E+00_JPRB, 0.57073E+00_JPRB, 0.56783E+00_JPRB ,&
&  0.56044E+00_JPRB, 0.55582E+00_JPRB, 0.55374E+00_JPRB, 0.54820E+00_JPRB, 0.53962E+00_JPRB, 0.52897E+00_JPRB, 0.52716E+00_JPRB ,&
&  0.53432E+00_JPRB, 0.52295E+00_JPRB, 0.51250E+00_JPRB, 0.49901E+00_JPRB, 0.47409E+00_JPRB, 0.45230E+00_JPRB, 0.44415E+00_JPRB ,&
&  0.43888E+00_JPRB, 0.41832E+00_JPRB, 0.41986E+00_JPRB, 0.42299E+00_JPRB, 0.42471E+00_JPRB, 0.43605E+00_JPRB, 0.45917E+00_JPRB ,&
&  0.48426E+00_JPRB, 0.56190E+00_JPRB, 0.59233E+00_JPRB, 0.62997E+00_JPRB, 0.66243E+00_JPRB, 0.68933E+00_JPRB, 0.71354E+00_JPRB ,&
&  0.73319E+00_JPRB, 0.76193E+00_JPRB, 0.79505E+00_JPRB, 0.83759E+00_JPRB, 0.87954E+00_JPRB, 0.91131E+00_JPRB, 0.94830E+00_JPRB ,&
&  0.99633E+00_JPRB, 0.98295E+00_JPRB, 0.99581E+00_JPRB, 0.99246E+00_JPRB, 0.10054E+01_JPRB, 0.10492E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.11906E+01_JPRB, 0.13256E+01_JPRB, 0.14628E+01_JPRB, 0.15274E+01_JPRB, 0.16006E+01_JPRB, 0.16351E+01_JPRB ,&
&  0.15617E+01_JPRB, 0.15205E+01_JPRB, 0.15660E+01_JPRB, 0.14439E+01_JPRB, 0.13123E+01_JPRB, 0.12014E+01_JPRB, 0.11242E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.96870E+00_JPRB, 0.91908E+00_JPRB, 0.88111E+00_JPRB, 0.84332E+00_JPRB, 0.81097E+00_JPRB, 0.80566E+00_JPRB ,&
&  0.80086E+00_JPRB, 0.79458E+00_JPRB, 0.78426E+00_JPRB, 0.77212E+00_JPRB, 0.74881E+00_JPRB, 0.72398E+00_JPRB, 0.70247E+00_JPRB ,&
&  0.69667E+00_JPRB, 0.69662E+00_JPRB, 0.70679E+00_JPRB, 0.71662E+00_JPRB, 0.72990E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.74461E+00_JPRB, 0.75866E+00_JPRB, 0.76728E+00_JPRB, 0.77691E+00_JPRB, 0.79407E+00_JPRB, 0.75297E+00_JPRB, 0.72398E+00_JPRB ,&
&  0.70777E+00_JPRB, 0.70853E+00_JPRB, 0.70812E+00_JPRB, 0.70189E+00_JPRB, 0.69098E+00_JPRB, 0.67971E+00_JPRB, 0.67083E+00_JPRB ,&
&  0.66319E+00_JPRB, 0.64491E+00_JPRB, 0.61408E+00_JPRB, 0.57031E+00_JPRB, 0.55841E+00_JPRB, 0.55524E+00_JPRB, 0.55777E+00_JPRB ,&
&  0.55833E+00_JPRB, 0.55394E+00_JPRB, 0.55448E+00_JPRB, 0.54545E+00_JPRB, 0.53846E+00_JPRB, 0.52020E+00_JPRB, 0.50537E+00_JPRB ,&
&  0.50681E+00_JPRB, 0.50502E+00_JPRB, 0.51676E+00_JPRB, 0.50629E+00_JPRB, 0.49819E+00_JPRB, 0.49417E+00_JPRB, 0.43717E+00_JPRB ,&
&  0.42704E+00_JPRB, 0.41422E+00_JPRB, 0.41034E+00_JPRB, 0.41017E+00_JPRB, 0.42609E+00_JPRB, 0.44113E+00_JPRB, 0.46217E+00_JPRB ,&
&  0.48625E+00_JPRB, 0.56917E+00_JPRB, 0.59647E+00_JPRB, 0.62555E+00_JPRB, 0.65432E+00_JPRB, 0.68760E+00_JPRB, 0.73120E+00_JPRB ,&
&  0.77229E+00_JPRB, 0.81258E+00_JPRB, 0.84526E+00_JPRB, 0.88571E+00_JPRB, 0.92727E+00_JPRB, 0.90476E+00_JPRB, 0.97210E+00_JPRB ,&
&  0.12627E+01_JPRB, 0.11914E+01_JPRB, 0.11220E+01_JPRB, 0.10855E+01_JPRB, 0.10850E+01_JPRB, 0.10959E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.12044E+01_JPRB, 0.13578E+01_JPRB, 0.15955E+01_JPRB, 0.17698E+01_JPRB, 0.17814E+01_JPRB, 0.18146E+01_JPRB, 0.17993E+01_JPRB ,&
&  0.16793E+01_JPRB, 0.16057E+01_JPRB, 0.16946E+01_JPRB, 0.14014E+01_JPRB, 0.11978E+01_JPRB, 0.10939E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.99943E+00_JPRB, 0.94365E+00_JPRB, 0.91143E+00_JPRB, 0.87341E+00_JPRB, 0.82535E+00_JPRB, 0.79757E+00_JPRB, 0.79328E+00_JPRB ,&
&  0.78977E+00_JPRB, 0.78250E+00_JPRB, 0.77735E+00_JPRB, 0.76853E+00_JPRB, 0.75117E+00_JPRB, 0.73113E+00_JPRB, 0.72406E+00_JPRB ,&
&  0.72311E+00_JPRB, 0.72085E+00_JPRB, 0.71943E+00_JPRB, 0.72056E+00_JPRB, 0.72866E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.73513E+00_JPRB, 0.74932E+00_JPRB, 0.76099E+00_JPRB, 0.75525E+00_JPRB, 0.74794E+00_JPRB, 0.68804E+00_JPRB, 0.62428E+00_JPRB ,&
&  0.58008E+00_JPRB, 0.58398E+00_JPRB, 0.60361E+00_JPRB, 0.61933E+00_JPRB, 0.61897E+00_JPRB, 0.60637E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60776E+00_JPRB, 0.60577E+00_JPRB, 0.58345E+00_JPRB, 0.56957E+00_JPRB, 0.54640E+00_JPRB, 0.54253E+00_JPRB, 0.54473E+00_JPRB ,&
&  0.54781E+00_JPRB, 0.54638E+00_JPRB, 0.54546E+00_JPRB, 0.54245E+00_JPRB, 0.54122E+00_JPRB, 0.53712E+00_JPRB, 0.52803E+00_JPRB ,&
&  0.51373E+00_JPRB, 0.50930E+00_JPRB, 0.51485E+00_JPRB, 0.52820E+00_JPRB, 0.52462E+00_JPRB, 0.52056E+00_JPRB, 0.45347E+00_JPRB ,&
&  0.42711E+00_JPRB, 0.40991E+00_JPRB, 0.41083E+00_JPRB, 0.42405E+00_JPRB, 0.45484E+00_JPRB, 0.47122E+00_JPRB, 0.53588E+00_JPRB ,&
&  0.58869E+00_JPRB, 0.61813E+00_JPRB, 0.63225E+00_JPRB, 0.64440E+00_JPRB, 0.67237E+00_JPRB, 0.73074E+00_JPRB, 0.80220E+00_JPRB ,&
&  0.84788E+00_JPRB, 0.87840E+00_JPRB, 0.87794E+00_JPRB, 0.74761E+00_JPRB, 0.74636E+00_JPRB, 0.81325E+00_JPRB, 0.91124E+00_JPRB ,&
&  0.10396E+01_JPRB, 0.10657E+01_JPRB, 0.10718E+01_JPRB, 0.11225E+01_JPRB, 0.11182E+01_JPRB, 0.11150E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.14331E+01_JPRB, 0.17514E+01_JPRB, 0.19231E+01_JPRB, 0.18828E+01_JPRB, 0.19143E+01_JPRB, 0.18505E+01_JPRB ,&
&  0.17813E+01_JPRB, 0.15992E+01_JPRB, 0.14408E+01_JPRB, 0.12426E+01_JPRB, 0.10878E+01_JPRB, 0.10018E+01_JPRB, 0.95297E+00_JPRB ,&
&  0.90650E+00_JPRB, 0.86250E+00_JPRB, 0.85559E+00_JPRB, 0.83171E+00_JPRB, 0.78460E+00_JPRB, 0.74264E+00_JPRB, 0.71072E+00_JPRB ,&
&  0.68869E+00_JPRB, 0.67730E+00_JPRB, 0.67607E+00_JPRB, 0.66293E+00_JPRB, 0.66475E+00_JPRB, 0.67606E+00_JPRB, 0.68147E+00_JPRB ,&
&  0.68078E+00_JPRB, 0.67959E+00_JPRB, 0.68753E+00_JPRB, 0.69889E+00_JPRB, 0.71215E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.78959E+00_JPRB, 0.76727E+00_JPRB, 0.73498E+00_JPRB, 0.73070E+00_JPRB, 0.70611E+00_JPRB, 0.68881E+00_JPRB, 0.62827E+00_JPRB ,&
&  0.55287E+00_JPRB, 0.53019E+00_JPRB, 0.55155E+00_JPRB, 0.55757E+00_JPRB, 0.54180E+00_JPRB, 0.53635E+00_JPRB, 0.54157E+00_JPRB ,&
&  0.54870E+00_JPRB, 0.57147E+00_JPRB, 0.56972E+00_JPRB, 0.55449E+00_JPRB, 0.54512E+00_JPRB, 0.54116E+00_JPRB, 0.54471E+00_JPRB ,&
&  0.54643E+00_JPRB, 0.54691E+00_JPRB, 0.54097E+00_JPRB, 0.53053E+00_JPRB, 0.52899E+00_JPRB, 0.52901E+00_JPRB, 0.52804E+00_JPRB ,&
&  0.52922E+00_JPRB, 0.52734E+00_JPRB, 0.53769E+00_JPRB, 0.54667E+00_JPRB, 0.54011E+00_JPRB, 0.52275E+00_JPRB, 0.46473E+00_JPRB ,&
&  0.43035E+00_JPRB, 0.41921E+00_JPRB, 0.44713E+00_JPRB, 0.51950E+00_JPRB, 0.56119E+00_JPRB, 0.60402E+00_JPRB, 0.63715E+00_JPRB ,&
&  0.61040E+00_JPRB, 0.61619E+00_JPRB, 0.65245E+00_JPRB, 0.76794E+00_JPRB, 0.81412E+00_JPRB, 0.87541E+00_JPRB, 0.92150E+00_JPRB ,&
&  0.95863E+00_JPRB, 0.98698E+00_JPRB, 0.99270E+00_JPRB, 0.84227E+00_JPRB, 0.85188E+00_JPRB, 0.10425E+01_JPRB, 0.99518E+00_JPRB ,&
&  0.10749E+01_JPRB, 0.11520E+01_JPRB, 0.12191E+01_JPRB, 0.11989E+01_JPRB, 0.11234E+01_JPRB, 0.11286E+01_JPRB, 0.12068E+01_JPRB ,&
&  0.13437E+01_JPRB, 0.15688E+01_JPRB, 0.19999E+01_JPRB, 0.22450E+01_JPRB, 0.22131E+01_JPRB, 0.21977E+01_JPRB, 0.21551E+01_JPRB ,&
&  0.21144E+01_JPRB, 0.19154E+01_JPRB, 0.16559E+01_JPRB, 0.14130E+01_JPRB, 0.12053E+01_JPRB, 0.10436E+01_JPRB, 0.94873E+00_JPRB ,&
&  0.90961E+00_JPRB, 0.88055E+00_JPRB, 0.87491E+00_JPRB, 0.84899E+00_JPRB, 0.79264E+00_JPRB, 0.73142E+00_JPRB, 0.68216E+00_JPRB ,&
&  0.65062E+00_JPRB, 0.61952E+00_JPRB, 0.58894E+00_JPRB, 0.57950E+00_JPRB, 0.61231E+00_JPRB, 0.65889E+00_JPRB, 0.67556E+00_JPRB ,&
&  0.66288E+00_JPRB, 0.66290E+00_JPRB, 0.69790E+00_JPRB, 0.73642E+00_JPRB, 0.75366E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.80793E+00_JPRB, 0.77495E+00_JPRB, 0.74647E+00_JPRB, 0.71862E+00_JPRB, 0.69948E+00_JPRB, 0.66956E+00_JPRB, 0.60526E+00_JPRB ,&
&  0.54056E+00_JPRB, 0.50696E+00_JPRB, 0.51626E+00_JPRB, 0.50445E+00_JPRB, 0.48789E+00_JPRB, 0.47816E+00_JPRB, 0.47939E+00_JPRB ,&
&  0.48857E+00_JPRB, 0.51478E+00_JPRB, 0.53383E+00_JPRB, 0.53860E+00_JPRB, 0.53534E+00_JPRB, 0.53169E+00_JPRB, 0.53837E+00_JPRB ,&
&  0.54818E+00_JPRB, 0.55918E+00_JPRB, 0.56471E+00_JPRB, 0.56168E+00_JPRB, 0.55525E+00_JPRB, 0.55331E+00_JPRB, 0.55529E+00_JPRB ,&
&  0.56223E+00_JPRB, 0.56586E+00_JPRB, 0.55510E+00_JPRB, 0.55830E+00_JPRB, 0.53841E+00_JPRB, 0.53795E+00_JPRB, 0.51568E+00_JPRB ,&
&  0.45990E+00_JPRB, 0.47060E+00_JPRB, 0.59147E+00_JPRB, 0.63089E+00_JPRB, 0.64657E+00_JPRB, 0.68167E+00_JPRB, 0.72952E+00_JPRB ,&
&  0.77390E+00_JPRB, 0.81603E+00_JPRB, 0.85423E+00_JPRB, 0.89619E+00_JPRB, 0.93510E+00_JPRB, 0.98689E+00_JPRB, 0.10353E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.93645E+00_JPRB, 0.88751E+00_JPRB, 0.93969E+00_JPRB, 0.12221E+01_JPRB, 0.12739E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.13155E+01_JPRB, 0.12995E+01_JPRB, 0.13126E+01_JPRB, 0.13704E+01_JPRB, 0.14708E+01_JPRB, 0.16359E+01_JPRB ,&
&  0.17906E+01_JPRB, 0.20916E+01_JPRB, 0.25451E+01_JPRB, 0.26631E+01_JPRB, 0.26339E+01_JPRB, 0.26220E+01_JPRB, 0.25400E+01_JPRB ,&
&  0.24304E+01_JPRB, 0.22154E+01_JPRB, 0.18601E+01_JPRB, 0.15620E+01_JPRB, 0.13157E+01_JPRB, 0.11294E+01_JPRB, 0.98252E+00_JPRB ,&
&  0.90839E+00_JPRB, 0.88421E+00_JPRB, 0.86944E+00_JPRB, 0.84530E+00_JPRB, 0.80811E+00_JPRB, 0.76331E+00_JPRB, 0.71363E+00_JPRB ,&
&  0.65263E+00_JPRB, 0.61567E+00_JPRB, 0.58116E+00_JPRB, 0.59698E+00_JPRB, 0.64680E+00_JPRB, 0.69948E+00_JPRB, 0.71777E+00_JPRB ,&
&  0.72964E+00_JPRB, 0.72891E+00_JPRB, 0.73114E+00_JPRB, 0.75327E+00_JPRB, 0.78533E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.80992E+00_JPRB, 0.76935E+00_JPRB, 0.72800E+00_JPRB, 0.68836E+00_JPRB, 0.66066E+00_JPRB, 0.63670E+00_JPRB, 0.60971E+00_JPRB ,&
&  0.65111E+00_JPRB, 0.56946E+00_JPRB, 0.64636E+00_JPRB, 0.54250E+00_JPRB, 0.51129E+00_JPRB, 0.50412E+00_JPRB, 0.48680E+00_JPRB ,&
&  0.47601E+00_JPRB, 0.47823E+00_JPRB, 0.48597E+00_JPRB, 0.49588E+00_JPRB, 0.49594E+00_JPRB, 0.49498E+00_JPRB, 0.50291E+00_JPRB ,&
&  0.51887E+00_JPRB, 0.54227E+00_JPRB, 0.56909E+00_JPRB, 0.58306E+00_JPRB, 0.58811E+00_JPRB, 0.58625E+00_JPRB, 0.59269E+00_JPRB ,&
&  0.56340E+00_JPRB, 0.55598E+00_JPRB, 0.56241E+00_JPRB, 0.56235E+00_JPRB, 0.55789E+00_JPRB, 0.54605E+00_JPRB, 0.56217E+00_JPRB ,&
&  0.55665E+00_JPRB, 0.53498E+00_JPRB, 0.64273E+00_JPRB, 0.65915E+00_JPRB, 0.68858E+00_JPRB, 0.72127E+00_JPRB, 0.75747E+00_JPRB ,&
&  0.79571E+00_JPRB, 0.84189E+00_JPRB, 0.89440E+00_JPRB, 0.93298E+00_JPRB, 0.99257E+00_JPRB, 0.10601E+01_JPRB, 0.11194E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.95424E+00_JPRB, 0.10459E+01_JPRB, 0.11021E+01_JPRB, 0.13342E+01_JPRB, 0.16006E+01_JPRB, 0.16216E+01_JPRB ,&
&  0.17983E+01_JPRB, 0.17721E+01_JPRB, 0.17375E+01_JPRB, 0.17071E+01_JPRB, 0.18148E+01_JPRB, 0.19670E+01_JPRB, 0.21417E+01_JPRB ,&
&  0.23817E+01_JPRB, 0.27149E+01_JPRB, 0.31981E+01_JPRB, 0.32264E+01_JPRB, 0.31724E+01_JPRB, 0.30852E+01_JPRB, 0.29192E+01_JPRB ,&
&  0.26889E+01_JPRB, 0.24066E+01_JPRB, 0.20528E+01_JPRB, 0.16704E+01_JPRB, 0.14114E+01_JPRB, 0.11946E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.89263E+00_JPRB, 0.82278E+00_JPRB, 0.81191E+00_JPRB, 0.78897E+00_JPRB, 0.77221E+00_JPRB, 0.74139E+00_JPRB, 0.68962E+00_JPRB ,&
&  0.63863E+00_JPRB, 0.65129E+00_JPRB, 0.69920E+00_JPRB, 0.77510E+00_JPRB, 0.80583E+00_JPRB, 0.81682E+00_JPRB, 0.86900E+00_JPRB ,&
&  0.79785E+00_JPRB, 0.83210E+00_JPRB, 0.88522E+00_JPRB, 0.86230E+00_JPRB, 0.84341E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.82860E+00_JPRB, 0.76954E+00_JPRB, 0.69411E+00_JPRB, 0.65593E+00_JPRB, 0.65740E+00_JPRB, 0.66019E+00_JPRB, 0.63937E+00_JPRB ,&
&  0.67160E+00_JPRB, 0.65650E+00_JPRB, 0.68219E+00_JPRB, 0.71910E+00_JPRB, 0.75463E+00_JPRB, 0.66833E+00_JPRB, 0.54058E+00_JPRB ,&
&  0.49981E+00_JPRB, 0.50230E+00_JPRB, 0.50714E+00_JPRB, 0.50447E+00_JPRB, 0.50410E+00_JPRB, 0.50028E+00_JPRB, 0.50045E+00_JPRB ,&
&  0.50629E+00_JPRB, 0.52021E+00_JPRB, 0.55493E+00_JPRB, 0.57034E+00_JPRB, 0.57822E+00_JPRB, 0.58131E+00_JPRB, 0.59337E+00_JPRB ,&
&  0.56626E+00_JPRB, 0.55716E+00_JPRB, 0.55393E+00_JPRB, 0.53014E+00_JPRB, 0.56446E+00_JPRB, 0.57761E+00_JPRB, 0.58816E+00_JPRB ,&
&  0.60145E+00_JPRB, 0.62707E+00_JPRB, 0.65938E+00_JPRB, 0.68884E+00_JPRB, 0.71649E+00_JPRB, 0.74546E+00_JPRB, 0.77686E+00_JPRB ,&
&  0.80983E+00_JPRB, 0.85146E+00_JPRB, 0.90055E+00_JPRB, 0.95853E+00_JPRB, 0.10844E+01_JPRB, 0.10606E+01_JPRB, 0.11555E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.13372E+01_JPRB, 0.14726E+01_JPRB, 0.14099E+01_JPRB, 0.15359E+01_JPRB, 0.17298E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.17547E+01_JPRB, 0.17336E+01_JPRB, 0.18452E+01_JPRB, 0.20071E+01_JPRB, 0.22332E+01_JPRB, 0.24262E+01_JPRB, 0.26154E+01_JPRB ,&
&  0.29364E+01_JPRB, 0.35344E+01_JPRB, 0.37816E+01_JPRB, 0.36814E+01_JPRB, 0.35437E+01_JPRB, 0.33786E+01_JPRB, 0.31202E+01_JPRB ,&
&  0.28295E+01_JPRB, 0.24950E+01_JPRB, 0.21227E+01_JPRB, 0.17478E+01_JPRB, 0.14262E+01_JPRB, 0.12498E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.87552E+00_JPRB, 0.75721E+00_JPRB, 0.75679E+00_JPRB, 0.75876E+00_JPRB, 0.74560E+00_JPRB, 0.70145E+00_JPRB, 0.66252E+00_JPRB ,&
&  0.69476E+00_JPRB, 0.78732E+00_JPRB, 0.10273E+01_JPRB, 0.10629E+01_JPRB, 0.10692E+01_JPRB, 0.10309E+01_JPRB, 0.85232E+00_JPRB ,&
&  0.78141E+00_JPRB, 0.91762E+00_JPRB, 0.88870E+00_JPRB, 0.88725E+00_JPRB, 0.87010E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.86057E+00_JPRB, 0.77636E+00_JPRB, 0.67519E+00_JPRB, 0.64551E+00_JPRB, 0.66198E+00_JPRB, 0.66652E+00_JPRB, 0.66790E+00_JPRB ,&
&  0.68254E+00_JPRB, 0.68957E+00_JPRB, 0.70050E+00_JPRB, 0.71595E+00_JPRB, 0.72408E+00_JPRB, 0.75520E+00_JPRB, 0.68388E+00_JPRB ,&
&  0.58839E+00_JPRB, 0.55790E+00_JPRB, 0.53383E+00_JPRB, 0.52793E+00_JPRB, 0.54681E+00_JPRB, 0.54885E+00_JPRB, 0.54789E+00_JPRB ,&
&  0.55483E+00_JPRB, 0.55599E+00_JPRB, 0.56879E+00_JPRB, 0.56886E+00_JPRB, 0.56747E+00_JPRB, 0.57402E+00_JPRB, 0.58392E+00_JPRB ,&
&  0.55273E+00_JPRB, 0.52805E+00_JPRB, 0.51305E+00_JPRB, 0.50532E+00_JPRB, 0.53486E+00_JPRB, 0.57847E+00_JPRB, 0.60563E+00_JPRB ,&
&  0.62408E+00_JPRB, 0.64429E+00_JPRB, 0.66761E+00_JPRB, 0.69666E+00_JPRB, 0.72862E+00_JPRB, 0.76119E+00_JPRB, 0.78760E+00_JPRB ,&
&  0.81620E+00_JPRB, 0.84954E+00_JPRB, 0.89644E+00_JPRB, 0.97554E+00_JPRB, 0.10201E+01_JPRB, 0.10800E+01_JPRB, 0.12138E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.14526E+01_JPRB, 0.15865E+01_JPRB, 0.18131E+01_JPRB, 0.19854E+01_JPRB, 0.19574E+01_JPRB, 0.19163E+01_JPRB ,&
&  0.19496E+01_JPRB, 0.19653E+01_JPRB, 0.22885E+01_JPRB, 0.25147E+01_JPRB, 0.26749E+01_JPRB, 0.28997E+01_JPRB, 0.31002E+01_JPRB ,&
&  0.34500E+01_JPRB, 0.38216E+01_JPRB, 0.39146E+01_JPRB, 0.38023E+01_JPRB, 0.36389E+01_JPRB, 0.34830E+01_JPRB, 0.31942E+01_JPRB ,&
&  0.28687E+01_JPRB, 0.24812E+01_JPRB, 0.20330E+01_JPRB, 0.16614E+01_JPRB, 0.13307E+01_JPRB, 0.12327E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.87801E+00_JPRB, 0.77501E+00_JPRB, 0.77466E+00_JPRB, 0.82070E+00_JPRB, 0.81193E+00_JPRB, 0.75165E+00_JPRB, 0.71691E+00_JPRB ,&
&  0.77195E+00_JPRB, 0.96324E+00_JPRB, 0.10799E+01_JPRB, 0.11627E+01_JPRB, 0.11735E+01_JPRB, 0.11159E+01_JPRB, 0.82091E+00_JPRB ,&
&  0.82052E+00_JPRB, 0.93894E+00_JPRB, 0.92932E+00_JPRB, 0.91892E+00_JPRB, 0.90236E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.90463E+00_JPRB, 0.80432E+00_JPRB, 0.71099E+00_JPRB, 0.66521E+00_JPRB, 0.67720E+00_JPRB, 0.69045E+00_JPRB, 0.69777E+00_JPRB ,&
&  0.71385E+00_JPRB, 0.72661E+00_JPRB, 0.73732E+00_JPRB, 0.74683E+00_JPRB, 0.75110E+00_JPRB, 0.74600E+00_JPRB, 0.79019E+00_JPRB ,&
&  0.64906E+00_JPRB, 0.59506E+00_JPRB, 0.55740E+00_JPRB, 0.55512E+00_JPRB, 0.62513E+00_JPRB, 0.64433E+00_JPRB, 0.66649E+00_JPRB ,&
&  0.66314E+00_JPRB, 0.66844E+00_JPRB, 0.65930E+00_JPRB, 0.64078E+00_JPRB, 0.62190E+00_JPRB, 0.61422E+00_JPRB, 0.59922E+00_JPRB ,&
&  0.55780E+00_JPRB, 0.52729E+00_JPRB, 0.52003E+00_JPRB, 0.52309E+00_JPRB, 0.55587E+00_JPRB, 0.58474E+00_JPRB, 0.63474E+00_JPRB ,&
&  0.66337E+00_JPRB, 0.68751E+00_JPRB, 0.71267E+00_JPRB, 0.74012E+00_JPRB, 0.76438E+00_JPRB, 0.79585E+00_JPRB, 0.83218E+00_JPRB ,&
&  0.86990E+00_JPRB, 0.90896E+00_JPRB, 0.97941E+00_JPRB, 0.10631E+01_JPRB, 0.10650E+01_JPRB, 0.11314E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.15638E+01_JPRB, 0.18302E+01_JPRB, 0.26145E+01_JPRB, 0.28488E+01_JPRB, 0.27721E+01_JPRB, 0.26479E+01_JPRB ,&
&  0.25949E+01_JPRB, 0.25548E+01_JPRB, 0.26769E+01_JPRB, 0.28523E+01_JPRB, 0.30237E+01_JPRB, 0.33192E+01_JPRB, 0.35958E+01_JPRB ,&
&  0.37740E+01_JPRB, 0.38443E+01_JPRB, 0.38031E+01_JPRB, 0.36795E+01_JPRB, 0.34429E+01_JPRB, 0.32641E+01_JPRB, 0.30573E+01_JPRB ,&
&  0.27854E+01_JPRB, 0.24435E+01_JPRB, 0.19179E+01_JPRB, 0.15823E+01_JPRB, 0.12005E+01_JPRB, 0.10920E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.95316E+00_JPRB, 0.84071E+00_JPRB, 0.86699E+00_JPRB, 0.94818E+00_JPRB, 0.96529E+00_JPRB, 0.94497E+00_JPRB, 0.83576E+00_JPRB ,&
&  0.75052E+00_JPRB, 0.89389E+00_JPRB, 0.12306E+01_JPRB, 0.13122E+01_JPRB, 0.12946E+01_JPRB, 0.12062E+01_JPRB, 0.96929E+00_JPRB ,&
&  0.10311E+01_JPRB, 0.98149E+00_JPRB, 0.97474E+00_JPRB, 0.96642E+00_JPRB, 0.95348E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.93784E+00_JPRB, 0.83750E+00_JPRB, 0.75652E+00_JPRB, 0.70667E+00_JPRB, 0.70953E+00_JPRB, 0.72923E+00_JPRB, 0.74238E+00_JPRB ,&
&  0.75426E+00_JPRB, 0.76308E+00_JPRB, 0.77023E+00_JPRB, 0.77708E+00_JPRB, 0.77966E+00_JPRB, 0.75809E+00_JPRB, 0.73998E+00_JPRB ,&
&  0.73045E+00_JPRB, 0.63142E+00_JPRB, 0.59867E+00_JPRB, 0.58597E+00_JPRB, 0.65057E+00_JPRB, 0.70912E+00_JPRB, 0.77551E+00_JPRB ,&
&  0.83182E+00_JPRB, 0.87305E+00_JPRB, 0.85624E+00_JPRB, 0.80793E+00_JPRB, 0.75769E+00_JPRB, 0.71638E+00_JPRB, 0.65859E+00_JPRB ,&
&  0.59826E+00_JPRB, 0.56881E+00_JPRB, 0.56908E+00_JPRB, 0.60154E+00_JPRB, 0.66077E+00_JPRB, 0.62670E+00_JPRB, 0.68933E+00_JPRB ,&
&  0.72533E+00_JPRB, 0.76017E+00_JPRB, 0.77528E+00_JPRB, 0.79803E+00_JPRB, 0.85782E+00_JPRB, 0.93299E+00_JPRB, 0.10008E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10807E+01_JPRB, 0.10714E+01_JPRB, 0.11428E+01_JPRB, 0.12158E+01_JPRB, 0.12702E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.15932E+01_JPRB, 0.17423E+01_JPRB, 0.20881E+01_JPRB, 0.30469E+01_JPRB, 0.33790E+01_JPRB, 0.33085E+01_JPRB, 0.33527E+01_JPRB ,&
&  0.33659E+01_JPRB, 0.32991E+01_JPRB, 0.32697E+01_JPRB, 0.32251E+01_JPRB, 0.34417E+01_JPRB, 0.37953E+01_JPRB, 0.40779E+01_JPRB ,&
&  0.39625E+01_JPRB, 0.36898E+01_JPRB, 0.35753E+01_JPRB, 0.33364E+01_JPRB, 0.31003E+01_JPRB, 0.28347E+01_JPRB, 0.28596E+01_JPRB ,&
&  0.28834E+01_JPRB, 0.20838E+01_JPRB, 0.17018E+01_JPRB, 0.15963E+01_JPRB, 0.11936E+01_JPRB, 0.10221E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.87967E+00_JPRB, 0.81140E+00_JPRB, 0.91597E+00_JPRB, 0.10272E+01_JPRB, 0.11598E+01_JPRB, 0.12272E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.11424E+01_JPRB, 0.15453E+01_JPRB, 0.15857E+01_JPRB, 0.14773E+01_JPRB, 0.13243E+01_JPRB, 0.11874E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.10850E+01_JPRB, 0.10664E+01_JPRB, 0.10472E+01_JPRB, 0.10224E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.10260E+01_JPRB, 0.92260E+00_JPRB, 0.82955E+00_JPRB, 0.78328E+00_JPRB, 0.78876E+00_JPRB, 0.80118E+00_JPRB, 0.81293E+00_JPRB ,&
&  0.79776E+00_JPRB, 0.78719E+00_JPRB, 0.79827E+00_JPRB, 0.79963E+00_JPRB, 0.79656E+00_JPRB, 0.77342E+00_JPRB, 0.73052E+00_JPRB ,&
&  0.70720E+00_JPRB, 0.68101E+00_JPRB, 0.63228E+00_JPRB, 0.63890E+00_JPRB, 0.67214E+00_JPRB, 0.78436E+00_JPRB, 0.88539E+00_JPRB ,&
&  0.99200E+00_JPRB, 0.10751E+01_JPRB, 0.10934E+01_JPRB, 0.10768E+01_JPRB, 0.10162E+01_JPRB, 0.92284E+00_JPRB, 0.83482E+00_JPRB ,&
&  0.75825E+00_JPRB, 0.72228E+00_JPRB, 0.70045E+00_JPRB, 0.77514E+00_JPRB, 0.80588E+00_JPRB, 0.76908E+00_JPRB, 0.85892E+00_JPRB ,&
&  0.89576E+00_JPRB, 0.92959E+00_JPRB, 0.94656E+00_JPRB, 0.10108E+01_JPRB, 0.10809E+01_JPRB, 0.11064E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.10999E+01_JPRB, 0.11428E+01_JPRB, 0.11329E+01_JPRB, 0.11957E+01_JPRB, 0.13684E+01_JPRB, 0.16706E+01_JPRB ,&
&  0.20171E+01_JPRB, 0.20210E+01_JPRB, 0.20880E+01_JPRB, 0.30393E+01_JPRB, 0.39332E+01_JPRB, 0.39575E+01_JPRB, 0.38867E+01_JPRB ,&
&  0.39209E+01_JPRB, 0.37141E+01_JPRB, 0.35171E+01_JPRB, 0.34748E+01_JPRB, 0.40088E+01_JPRB, 0.43944E+01_JPRB, 0.45172E+01_JPRB ,&
&  0.42263E+01_JPRB, 0.37977E+01_JPRB, 0.35833E+01_JPRB, 0.33306E+01_JPRB, 0.29689E+01_JPRB, 0.28423E+01_JPRB, 0.29137E+01_JPRB ,&
&  0.28641E+01_JPRB, 0.25398E+01_JPRB, 0.19461E+01_JPRB, 0.17442E+01_JPRB, 0.13867E+01_JPRB, 0.12077E+01_JPRB, 0.12041E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.98854E+00_JPRB, 0.10276E+01_JPRB, 0.10926E+01_JPRB, 0.16199E+01_JPRB, 0.17664E+01_JPRB, 0.19032E+01_JPRB ,&
&  0.18884E+01_JPRB, 0.17922E+01_JPRB, 0.21996E+01_JPRB, 0.21102E+01_JPRB, 0.18286E+01_JPRB, 0.15271E+01_JPRB, 0.12960E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.12106E+01_JPRB, 0.11797E+01_JPRB, 0.11477E+01_JPRB, 0.11168E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.10768E+01_JPRB, 0.96877E+00_JPRB, 0.87657E+00_JPRB, 0.82799E+00_JPRB, 0.81948E+00_JPRB, 0.82763E+00_JPRB, 0.82795E+00_JPRB ,&
&  0.81746E+00_JPRB, 0.81639E+00_JPRB, 0.81721E+00_JPRB, 0.80073E+00_JPRB, 0.78041E+00_JPRB, 0.75598E+00_JPRB, 0.71966E+00_JPRB ,&
&  0.69135E+00_JPRB, 0.67688E+00_JPRB, 0.64203E+00_JPRB, 0.65984E+00_JPRB, 0.72599E+00_JPRB, 0.88621E+00_JPRB, 0.10819E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.13638E+01_JPRB, 0.13993E+01_JPRB, 0.13880E+01_JPRB, 0.13729E+01_JPRB, 0.13010E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.98011E+00_JPRB, 0.10486E+01_JPRB, 0.10439E+01_JPRB, 0.11070E+01_JPRB, 0.11082E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10605E+01_JPRB, 0.10625E+01_JPRB, 0.10920E+01_JPRB, 0.11306E+01_JPRB, 0.11522E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.12044E+01_JPRB, 0.11667E+01_JPRB, 0.10974E+01_JPRB, 0.11021E+01_JPRB, 0.14793E+01_JPRB, 0.20682E+01_JPRB, 0.26258E+01_JPRB ,&
&  0.26134E+01_JPRB, 0.23868E+01_JPRB, 0.33968E+01_JPRB, 0.39842E+01_JPRB, 0.40011E+01_JPRB, 0.44475E+01_JPRB, 0.44043E+01_JPRB ,&
&  0.39951E+01_JPRB, 0.35207E+01_JPRB, 0.31736E+01_JPRB, 0.33923E+01_JPRB, 0.48433E+01_JPRB, 0.52561E+01_JPRB, 0.52677E+01_JPRB ,&
&  0.47336E+01_JPRB, 0.41528E+01_JPRB, 0.36358E+01_JPRB, 0.32883E+01_JPRB, 0.31009E+01_JPRB, 0.32426E+01_JPRB, 0.30618E+01_JPRB ,&
&  0.28497E+01_JPRB, 0.30031E+01_JPRB, 0.25122E+01_JPRB, 0.19813E+01_JPRB, 0.16632E+01_JPRB, 0.15259E+01_JPRB, 0.14433E+01_JPRB ,&
&  0.13021E+01_JPRB, 0.12462E+01_JPRB, 0.12687E+01_JPRB, 0.15614E+01_JPRB, 0.23470E+01_JPRB, 0.24368E+01_JPRB, 0.24807E+01_JPRB ,&
&  0.25904E+01_JPRB, 0.31649E+01_JPRB, 0.28330E+01_JPRB, 0.24539E+01_JPRB, 0.20777E+01_JPRB, 0.17407E+01_JPRB, 0.14915E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.13291E+01_JPRB, 0.13028E+01_JPRB, 0.12440E+01_JPRB, 0.11850E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.11665E+01_JPRB, 0.10387E+01_JPRB, 0.97330E+00_JPRB, 0.91806E+00_JPRB, 0.89439E+00_JPRB, 0.87410E+00_JPRB, 0.82902E+00_JPRB ,&
&  0.79842E+00_JPRB, 0.80738E+00_JPRB, 0.79345E+00_JPRB, 0.78321E+00_JPRB, 0.77198E+00_JPRB, 0.73997E+00_JPRB, 0.70662E+00_JPRB ,&
&  0.69396E+00_JPRB, 0.67446E+00_JPRB, 0.67973E+00_JPRB, 0.75946E+00_JPRB, 0.85692E+00_JPRB, 0.99360E+00_JPRB, 0.12361E+01_JPRB ,&
&  0.14727E+01_JPRB, 0.16342E+01_JPRB, 0.16807E+01_JPRB, 0.17584E+01_JPRB, 0.17902E+01_JPRB, 0.17718E+01_JPRB, 0.16447E+01_JPRB ,&
&  0.14521E+01_JPRB, 0.14351E+01_JPRB, 0.15119E+01_JPRB, 0.14566E+01_JPRB, 0.14179E+01_JPRB, 0.13583E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.12288E+01_JPRB, 0.11777E+01_JPRB, 0.11717E+01_JPRB, 0.11939E+01_JPRB, 0.12289E+01_JPRB, 0.12938E+01_JPRB, 0.12876E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.11646E+01_JPRB, 0.11246E+01_JPRB, 0.14105E+01_JPRB, 0.21287E+01_JPRB, 0.33249E+01_JPRB, 0.37561E+01_JPRB ,&
&  0.39371E+01_JPRB, 0.34941E+01_JPRB, 0.35650E+01_JPRB, 0.41584E+01_JPRB, 0.35238E+01_JPRB, 0.38789E+01_JPRB, 0.40205E+01_JPRB ,&
&  0.35963E+01_JPRB, 0.34336E+01_JPRB, 0.39160E+01_JPRB, 0.48038E+01_JPRB, 0.59799E+01_JPRB, 0.72827E+01_JPRB, 0.71518E+01_JPRB ,&
&  0.58992E+01_JPRB, 0.48130E+01_JPRB, 0.38115E+01_JPRB, 0.35061E+01_JPRB, 0.31273E+01_JPRB, 0.30639E+01_JPRB, 0.24717E+01_JPRB ,&
&  0.25081E+01_JPRB, 0.21605E+01_JPRB, 0.23007E+01_JPRB, 0.20292E+01_JPRB, 0.18846E+01_JPRB, 0.17109E+01_JPRB, 0.15783E+01_JPRB ,&
&  0.15298E+01_JPRB, 0.15370E+01_JPRB, 0.17333E+01_JPRB, 0.24296E+01_JPRB, 0.33513E+01_JPRB, 0.29104E+01_JPRB, 0.35417E+01_JPRB ,&
&  0.45692E+01_JPRB, 0.43320E+01_JPRB, 0.30852E+01_JPRB, 0.26430E+01_JPRB, 0.21907E+01_JPRB, 0.18737E+01_JPRB, 0.17041E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.14658E+01_JPRB, 0.14147E+01_JPRB, 0.13545E+01_JPRB, 0.12843E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.12344E+01_JPRB, 0.11037E+01_JPRB, 0.10469E+01_JPRB, 0.10250E+01_JPRB, 0.98400E+00_JPRB, 0.88878E+00_JPRB, 0.83858E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.82447E+00_JPRB, 0.77451E+00_JPRB, 0.74524E+00_JPRB, 0.74206E+00_JPRB, 0.74425E+00_JPRB, 0.71784E+00_JPRB ,&
&  0.72771E+00_JPRB, 0.77450E+00_JPRB, 0.82912E+00_JPRB, 0.90625E+00_JPRB, 0.10280E+01_JPRB, 0.11401E+01_JPRB, 0.12592E+01_JPRB ,&
&  0.17833E+01_JPRB, 0.19523E+01_JPRB, 0.18979E+01_JPRB, 0.19688E+01_JPRB, 0.20696E+01_JPRB, 0.21136E+01_JPRB, 0.20091E+01_JPRB ,&
&  0.19971E+01_JPRB, 0.20001E+01_JPRB, 0.17254E+01_JPRB, 0.15330E+01_JPRB, 0.14186E+01_JPRB, 0.13574E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.11367E+01_JPRB, 0.11340E+01_JPRB, 0.11376E+01_JPRB, 0.11804E+01_JPRB, 0.11230E+01_JPRB, 0.11068E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11708E+01_JPRB, 0.12937E+01_JPRB, 0.18315E+01_JPRB, 0.30081E+01_JPRB, 0.45598E+01_JPRB, 0.49749E+01_JPRB ,&
&  0.43410E+01_JPRB, 0.36447E+01_JPRB, 0.37756E+01_JPRB, 0.42000E+01_JPRB, 0.39632E+01_JPRB, 0.37280E+01_JPRB, 0.34938E+01_JPRB ,&
&  0.29558E+01_JPRB, 0.31065E+01_JPRB, 0.39684E+01_JPRB, 0.54451E+01_JPRB, 0.75584E+01_JPRB, 0.96028E+01_JPRB, 0.93276E+01_JPRB ,&
&  0.73699E+01_JPRB, 0.57616E+01_JPRB, 0.45896E+01_JPRB, 0.36403E+01_JPRB, 0.35306E+01_JPRB, 0.26951E+01_JPRB, 0.30020E+01_JPRB ,&
&  0.30196E+01_JPRB, 0.28891E+01_JPRB, 0.23920E+01_JPRB, 0.19615E+01_JPRB, 0.16156E+01_JPRB, 0.17646E+01_JPRB, 0.17196E+01_JPRB ,&
&  0.18196E+01_JPRB, 0.19630E+01_JPRB, 0.32753E+01_JPRB, 0.38699E+01_JPRB, 0.41927E+01_JPRB, 0.41954E+01_JPRB, 0.54868E+01_JPRB ,&
&  0.51394E+01_JPRB, 0.44236E+01_JPRB, 0.34765E+01_JPRB, 0.28456E+01_JPRB, 0.23679E+01_JPRB, 0.20882E+01_JPRB, 0.19185E+01_JPRB ,&
&  0.17325E+01_JPRB, 0.16339E+01_JPRB, 0.15579E+01_JPRB, 0.14742E+01_JPRB, 0.13735E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.12349E+01_JPRB, 0.11669E+01_JPRB, 0.11440E+01_JPRB, 0.10655E+01_JPRB, 0.94165E+00_JPRB, 0.86453E+00_JPRB, 0.85685E+00_JPRB ,&
&  0.84410E+00_JPRB, 0.80986E+00_JPRB, 0.76873E+00_JPRB, 0.73972E+00_JPRB, 0.74406E+00_JPRB, 0.75530E+00_JPRB, 0.76660E+00_JPRB ,&
&  0.83963E+00_JPRB, 0.99470E+00_JPRB, 0.10685E+01_JPRB, 0.11590E+01_JPRB, 0.12317E+01_JPRB, 0.12936E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.21014E+01_JPRB, 0.20466E+01_JPRB, 0.17811E+01_JPRB, 0.18596E+01_JPRB, 0.21508E+01_JPRB, 0.20701E+01_JPRB, 0.20195E+01_JPRB ,&
&  0.20441E+01_JPRB, 0.19382E+01_JPRB, 0.16818E+01_JPRB, 0.14668E+01_JPRB, 0.13285E+01_JPRB, 0.12501E+01_JPRB, 0.11632E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.11238E+01_JPRB, 0.11564E+01_JPRB, 0.11930E+01_JPRB, 0.11676E+01_JPRB, 0.11395E+01_JPRB, 0.11474E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.12513E+01_JPRB, 0.14810E+01_JPRB, 0.22381E+01_JPRB, 0.35605E+01_JPRB, 0.53211E+01_JPRB, 0.60781E+01_JPRB ,&
&  0.47931E+01_JPRB, 0.43681E+01_JPRB, 0.43326E+01_JPRB, 0.53501E+01_JPRB, 0.38974E+01_JPRB, 0.35816E+01_JPRB, 0.34971E+01_JPRB ,&
&  0.33430E+01_JPRB, 0.40214E+01_JPRB, 0.55773E+01_JPRB, 0.74067E+01_JPRB, 0.79715E+01_JPRB, 0.93600E+01_JPRB, 0.10763E+02_JPRB ,&
&  0.78690E+01_JPRB, 0.60105E+01_JPRB, 0.50329E+01_JPRB, 0.46440E+01_JPRB, 0.30325E+01_JPRB, 0.20846E+01_JPRB, 0.29931E+01_JPRB ,&
&  0.29406E+01_JPRB, 0.16831E+01_JPRB, 0.12405E+01_JPRB, 0.14009E+01_JPRB, 0.10408E+01_JPRB, 0.11981E+01_JPRB, 0.17324E+01_JPRB ,&
&  0.21405E+01_JPRB, 0.25888E+01_JPRB, 0.50837E+01_JPRB, 0.48135E+01_JPRB, 0.50861E+01_JPRB, 0.62597E+01_JPRB, 0.66525E+01_JPRB ,&
&  0.57970E+01_JPRB, 0.36365E+01_JPRB, 0.33123E+01_JPRB, 0.29145E+01_JPRB, 0.24107E+01_JPRB, 0.20388E+01_JPRB, 0.17760E+01_JPRB ,&
&  0.15646E+01_JPRB, 0.14318E+01_JPRB, 0.13536E+01_JPRB, 0.13144E+01_JPRB, 0.13050E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.10769E+01_JPRB, 0.10636E+01_JPRB, 0.10286E+01_JPRB, 0.92871E+00_JPRB, 0.85918E+00_JPRB, 0.84335E+00_JPRB, 0.83296E+00_JPRB ,&
&  0.81532E+00_JPRB, 0.77002E+00_JPRB, 0.74586E+00_JPRB, 0.75380E+00_JPRB, 0.76820E+00_JPRB, 0.78763E+00_JPRB, 0.82933E+00_JPRB ,&
&  0.90331E+00_JPRB, 0.10481E+01_JPRB, 0.13181E+01_JPRB, 0.15060E+01_JPRB, 0.16000E+01_JPRB, 0.13837E+01_JPRB, 0.17334E+01_JPRB ,&
&  0.21919E+01_JPRB, 0.18909E+01_JPRB, 0.16590E+01_JPRB, 0.18377E+01_JPRB, 0.19399E+01_JPRB, 0.17783E+01_JPRB, 0.17645E+01_JPRB ,&
&  0.17951E+01_JPRB, 0.17112E+01_JPRB, 0.15617E+01_JPRB, 0.14685E+01_JPRB, 0.13354E+01_JPRB, 0.12042E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.12740E+01_JPRB, 0.13437E+01_JPRB, 0.13067E+01_JPRB, 0.12957E+01_JPRB, 0.12737E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.13252E+01_JPRB, 0.14637E+01_JPRB, 0.18481E+01_JPRB, 0.29332E+01_JPRB, 0.48499E+01_JPRB, 0.71762E+01_JPRB, 0.59069E+01_JPRB ,&
&  0.43475E+01_JPRB, 0.38677E+01_JPRB, 0.48365E+01_JPRB, 0.50641E+01_JPRB, 0.37625E+01_JPRB, 0.36950E+01_JPRB, 0.36578E+01_JPRB ,&
&  0.42696E+01_JPRB, 0.51840E+01_JPRB, 0.57859E+01_JPRB, 0.73512E+01_JPRB, 0.92949E+01_JPRB, 0.96289E+01_JPRB, 0.97968E+01_JPRB ,&
&  0.79085E+01_JPRB, 0.59557E+01_JPRB, 0.42459E+01_JPRB, 0.33413E+01_JPRB, 0.37309E+01_JPRB, 0.27381E+01_JPRB, 0.17328E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.10514E+01_JPRB, 0.84365E+00_JPRB, 0.82277E+00_JPRB, 0.72836E+00_JPRB, 0.98947E+00_JPRB, 0.16185E+01_JPRB ,&
&  0.30331E+01_JPRB, 0.42790E+01_JPRB, 0.55961E+01_JPRB, 0.57353E+01_JPRB, 0.53126E+01_JPRB, 0.66088E+01_JPRB, 0.74007E+01_JPRB ,&
&  0.45626E+01_JPRB, 0.32890E+01_JPRB, 0.31198E+01_JPRB, 0.25470E+01_JPRB, 0.19412E+01_JPRB, 0.15818E+01_JPRB, 0.13492E+01_JPRB ,&
&  0.12149E+01_JPRB, 0.11359E+01_JPRB, 0.10923E+01_JPRB, 0.10881E+01_JPRB, 0.11007E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.95229E+00_JPRB, 0.93616E+00_JPRB, 0.91586E+00_JPRB, 0.86339E+00_JPRB, 0.82413E+00_JPRB, 0.82469E+00_JPRB, 0.82411E+00_JPRB ,&
&  0.79559E+00_JPRB, 0.76172E+00_JPRB, 0.76243E+00_JPRB, 0.78634E+00_JPRB, 0.81863E+00_JPRB, 0.87119E+00_JPRB, 0.92950E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.12000E+01_JPRB, 0.15783E+01_JPRB, 0.19282E+01_JPRB, 0.21377E+01_JPRB, 0.16992E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.21590E+01_JPRB, 0.18453E+01_JPRB, 0.17408E+01_JPRB, 0.18716E+01_JPRB, 0.18790E+01_JPRB, 0.16251E+01_JPRB, 0.16683E+01_JPRB ,&
&  0.16612E+01_JPRB, 0.16094E+01_JPRB, 0.15231E+01_JPRB, 0.14745E+01_JPRB, 0.13726E+01_JPRB, 0.14009E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.15528E+01_JPRB, 0.16269E+01_JPRB, 0.15890E+01_JPRB, 0.15443E+01_JPRB, 0.15212E+01_JPRB, 0.15092E+01_JPRB, 0.15414E+01_JPRB ,&
&  0.16468E+01_JPRB, 0.19314E+01_JPRB, 0.26873E+01_JPRB, 0.43953E+01_JPRB, 0.59512E+01_JPRB, 0.60150E+01_JPRB, 0.53435E+01_JPRB ,&
&  0.47582E+01_JPRB, 0.47445E+01_JPRB, 0.47628E+01_JPRB, 0.52840E+01_JPRB, 0.39497E+01_JPRB, 0.37485E+01_JPRB, 0.38212E+01_JPRB ,&
&  0.45192E+01_JPRB, 0.62171E+01_JPRB, 0.68737E+01_JPRB, 0.78655E+01_JPRB, 0.11414E+02_JPRB, 0.11868E+02_JPRB, 0.91748E+01_JPRB ,&
&  0.77650E+01_JPRB, 0.57803E+01_JPRB, 0.45424E+01_JPRB, 0.38599E+01_JPRB, 0.54028E+01_JPRB, 0.55019E+01_JPRB, 0.19389E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.88762E+00_JPRB, 0.65173E+00_JPRB, 0.62906E+00_JPRB, 0.67000E+00_JPRB, 0.98865E+00_JPRB, 0.24058E+01_JPRB ,&
&  0.28321E+01_JPRB, 0.45741E+01_JPRB, 0.51326E+01_JPRB, 0.55702E+01_JPRB, 0.52555E+01_JPRB, 0.52940E+01_JPRB, 0.48460E+01_JPRB ,&
&  0.41314E+01_JPRB, 0.32143E+01_JPRB, 0.24075E+01_JPRB, 0.18594E+01_JPRB, 0.14646E+01_JPRB, 0.12577E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.10545E+01_JPRB, 0.10219E+01_JPRB, 0.99058E+00_JPRB, 0.95506E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.84380E+00_JPRB, 0.85592E+00_JPRB, 0.87679E+00_JPRB, 0.87127E+00_JPRB, 0.86393E+00_JPRB, 0.87709E+00_JPRB, 0.86439E+00_JPRB ,&
&  0.83211E+00_JPRB, 0.80892E+00_JPRB, 0.82423E+00_JPRB, 0.85903E+00_JPRB, 0.91462E+00_JPRB, 0.98681E+00_JPRB, 0.10591E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.15002E+01_JPRB, 0.19320E+01_JPRB, 0.24840E+01_JPRB, 0.26380E+01_JPRB, 0.23388E+01_JPRB, 0.23611E+01_JPRB ,&
&  0.22249E+01_JPRB, 0.19702E+01_JPRB, 0.18263E+01_JPRB, 0.19394E+01_JPRB, 0.19326E+01_JPRB, 0.17995E+01_JPRB, 0.17393E+01_JPRB ,&
&  0.17419E+01_JPRB, 0.17634E+01_JPRB, 0.17345E+01_JPRB, 0.17769E+01_JPRB, 0.18677E+01_JPRB, 0.19748E+01_JPRB, 0.20663E+01_JPRB ,&
&  0.21074E+01_JPRB, 0.20464E+01_JPRB, 0.19364E+01_JPRB, 0.18652E+01_JPRB, 0.18468E+01_JPRB, 0.18820E+01_JPRB, 0.19881E+01_JPRB ,&
&  0.22177E+01_JPRB, 0.27831E+01_JPRB, 0.40014E+01_JPRB, 0.55572E+01_JPRB, 0.54448E+01_JPRB, 0.58169E+01_JPRB, 0.52810E+01_JPRB ,&
&  0.47539E+01_JPRB, 0.46869E+01_JPRB, 0.44166E+01_JPRB, 0.47166E+01_JPRB, 0.42513E+01_JPRB, 0.41493E+01_JPRB, 0.42730E+01_JPRB ,&
&  0.60385E+01_JPRB, 0.73251E+01_JPRB, 0.89314E+01_JPRB, 0.95913E+01_JPRB, 0.11815E+02_JPRB, 0.15212E+02_JPRB, 0.94513E+01_JPRB ,&
&  0.67082E+01_JPRB, 0.60874E+01_JPRB, 0.48550E+01_JPRB, 0.42140E+01_JPRB, 0.50273E+01_JPRB, 0.66036E+01_JPRB, 0.48535E+01_JPRB ,&
&  0.15273E+01_JPRB, 0.94114E+00_JPRB, 0.80609E+00_JPRB, 0.80437E+00_JPRB, 0.66982E+00_JPRB, 0.10170E+01_JPRB, 0.34005E+01_JPRB ,&
&  0.30270E+01_JPRB, 0.40129E+01_JPRB, 0.44585E+01_JPRB, 0.48629E+01_JPRB, 0.48619E+01_JPRB, 0.36761E+01_JPRB, 0.29647E+01_JPRB ,&
&  0.20149E+01_JPRB, 0.14995E+01_JPRB, 0.12998E+01_JPRB, 0.11759E+01_JPRB, 0.10278E+01_JPRB, 0.95207E+00_JPRB, 0.90545E+00_JPRB ,&
&  0.89243E+00_JPRB, 0.87504E+00_JPRB, 0.85923E+00_JPRB, 0.83868E+00_JPRB, 0.83962E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.81778E+00_JPRB, 0.86164E+00_JPRB, 0.90460E+00_JPRB, 0.94753E+00_JPRB, 0.97309E+00_JPRB, 0.96691E+00_JPRB, 0.94526E+00_JPRB ,&
&  0.93274E+00_JPRB, 0.92197E+00_JPRB, 0.94453E+00_JPRB, 0.98656E+00_JPRB, 0.10720E+01_JPRB, 0.11843E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.16219E+01_JPRB, 0.19967E+01_JPRB, 0.24431E+01_JPRB, 0.28722E+01_JPRB, 0.29003E+01_JPRB, 0.24801E+01_JPRB, 0.24214E+01_JPRB ,&
&  0.23604E+01_JPRB, 0.21986E+01_JPRB, 0.19843E+01_JPRB, 0.21609E+01_JPRB, 0.22516E+01_JPRB, 0.22148E+01_JPRB, 0.21449E+01_JPRB ,&
&  0.21554E+01_JPRB, 0.21914E+01_JPRB, 0.22376E+01_JPRB, 0.23479E+01_JPRB, 0.24570E+01_JPRB, 0.25339E+01_JPRB, 0.25617E+01_JPRB ,&
&  0.24785E+01_JPRB, 0.23514E+01_JPRB, 0.22813E+01_JPRB, 0.22807E+01_JPRB, 0.23341E+01_JPRB, 0.24209E+01_JPRB, 0.26289E+01_JPRB ,&
&  0.30878E+01_JPRB, 0.39135E+01_JPRB, 0.50137E+01_JPRB, 0.54870E+01_JPRB, 0.53552E+01_JPRB, 0.51707E+01_JPRB, 0.48348E+01_JPRB ,&
&  0.45583E+01_JPRB, 0.46047E+01_JPRB, 0.45395E+01_JPRB, 0.45331E+01_JPRB, 0.44514E+01_JPRB, 0.47295E+01_JPRB, 0.52877E+01_JPRB ,&
&  0.74951E+01_JPRB, 0.92568E+01_JPRB, 0.96620E+01_JPRB, 0.96823E+01_JPRB, 0.10211E+02_JPRB, 0.11806E+02_JPRB, 0.10817E+02_JPRB ,&
&  0.78852E+01_JPRB, 0.55797E+01_JPRB, 0.45646E+01_JPRB, 0.45522E+01_JPRB, 0.45995E+01_JPRB, 0.51084E+01_JPRB, 0.60930E+01_JPRB ,&
&  0.52855E+01_JPRB, 0.20407E+01_JPRB, 0.12782E+01_JPRB, 0.17759E+01_JPRB, 0.10606E+01_JPRB, 0.10704E+01_JPRB, 0.18308E+01_JPRB ,&
&  0.23649E+01_JPRB, 0.29174E+01_JPRB, 0.32968E+01_JPRB, 0.38961E+01_JPRB, 0.33735E+01_JPRB, 0.24693E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.10726E+01_JPRB, 0.10380E+01_JPRB, 0.93834E+00_JPRB, 0.85510E+00_JPRB, 0.82999E+00_JPRB, 0.79940E+00_JPRB ,&
&  0.77446E+00_JPRB, 0.78596E+00_JPRB, 0.80063E+00_JPRB, 0.79751E+00_JPRB, 0.79032E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.88215E+00_JPRB, 0.93783E+00_JPRB, 0.10209E+01_JPRB, 0.10632E+01_JPRB, 0.10674E+01_JPRB, 0.10708E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.11436E+01_JPRB, 0.12029E+01_JPRB, 0.13018E+01_JPRB, 0.14682E+01_JPRB, 0.16652E+01_JPRB, 0.19156E+01_JPRB ,&
&  0.22149E+01_JPRB, 0.25390E+01_JPRB, 0.28761E+01_JPRB, 0.31933E+01_JPRB, 0.33884E+01_JPRB, 0.35485E+01_JPRB, 0.23603E+01_JPRB ,&
&  0.20658E+01_JPRB, 0.25167E+01_JPRB, 0.21801E+01_JPRB, 0.24211E+01_JPRB, 0.25658E+01_JPRB, 0.26271E+01_JPRB, 0.24929E+01_JPRB ,&
&  0.25051E+01_JPRB, 0.25753E+01_JPRB, 0.26611E+01_JPRB, 0.27085E+01_JPRB, 0.27331E+01_JPRB, 0.27522E+01_JPRB, 0.27259E+01_JPRB ,&
&  0.26766E+01_JPRB, 0.26550E+01_JPRB, 0.26904E+01_JPRB, 0.27641E+01_JPRB, 0.28841E+01_JPRB, 0.30854E+01_JPRB, 0.34308E+01_JPRB ,&
&  0.39375E+01_JPRB, 0.45498E+01_JPRB, 0.51012E+01_JPRB, 0.49417E+01_JPRB, 0.47874E+01_JPRB, 0.46213E+01_JPRB, 0.45767E+01_JPRB ,&
&  0.43886E+01_JPRB, 0.41257E+01_JPRB, 0.44332E+01_JPRB, 0.47617E+01_JPRB, 0.48603E+01_JPRB, 0.52224E+01_JPRB, 0.61858E+01_JPRB ,&
&  0.84850E+01_JPRB, 0.98236E+01_JPRB, 0.10328E+02_JPRB, 0.88065E+01_JPRB, 0.82355E+01_JPRB, 0.90773E+01_JPRB, 0.93162E+01_JPRB ,&
&  0.78518E+01_JPRB, 0.59295E+01_JPRB, 0.48848E+01_JPRB, 0.42681E+01_JPRB, 0.38904E+01_JPRB, 0.39957E+01_JPRB, 0.43306E+01_JPRB ,&
&  0.45244E+01_JPRB, 0.50158E+01_JPRB, 0.39222E+01_JPRB, 0.22202E+01_JPRB, 0.15256E+01_JPRB, 0.14515E+01_JPRB, 0.17675E+01_JPRB ,&
&  0.20573E+01_JPRB, 0.22059E+01_JPRB, 0.22006E+01_JPRB, 0.32581E+01_JPRB, 0.19281E+01_JPRB, 0.13820E+01_JPRB, 0.10365E+01_JPRB ,&
&  0.92741E+00_JPRB, 0.92227E+00_JPRB, 0.92225E+00_JPRB, 0.86251E+00_JPRB, 0.80865E+00_JPRB, 0.81809E+00_JPRB, 0.79494E+00_JPRB ,&
&  0.81170E+00_JPRB, 0.83040E+00_JPRB, 0.83081E+00_JPRB, 0.83687E+00_JPRB, 0.86115E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.10290E+01_JPRB, 0.10871E+01_JPRB, 0.11522E+01_JPRB, 0.11878E+01_JPRB, 0.12314E+01_JPRB, 0.12780E+01_JPRB, 0.13556E+01_JPRB ,&
&  0.14445E+01_JPRB, 0.15472E+01_JPRB, 0.16697E+01_JPRB, 0.18257E+01_JPRB, 0.20297E+01_JPRB, 0.22390E+01_JPRB, 0.24695E+01_JPRB ,&
&  0.26817E+01_JPRB, 0.28646E+01_JPRB, 0.30411E+01_JPRB, 0.32200E+01_JPRB, 0.33568E+01_JPRB, 0.35480E+01_JPRB, 0.30646E+01_JPRB ,&
&  0.19470E+01_JPRB, 0.29635E+01_JPRB, 0.23278E+01_JPRB, 0.26165E+01_JPRB, 0.26965E+01_JPRB, 0.28596E+01_JPRB, 0.27076E+01_JPRB ,&
&  0.26665E+01_JPRB, 0.27125E+01_JPRB, 0.27396E+01_JPRB, 0.27773E+01_JPRB, 0.27990E+01_JPRB, 0.28254E+01_JPRB, 0.28497E+01_JPRB ,&
&  0.28907E+01_JPRB, 0.29716E+01_JPRB, 0.30850E+01_JPRB, 0.32150E+01_JPRB, 0.34229E+01_JPRB, 0.37102E+01_JPRB, 0.39956E+01_JPRB ,&
&  0.42924E+01_JPRB, 0.45906E+01_JPRB, 0.46682E+01_JPRB, 0.45008E+01_JPRB, 0.44020E+01_JPRB, 0.44931E+01_JPRB, 0.47098E+01_JPRB ,&
&  0.46052E+01_JPRB, 0.44115E+01_JPRB, 0.46851E+01_JPRB, 0.48491E+01_JPRB, 0.49853E+01_JPRB, 0.58381E+01_JPRB, 0.66995E+01_JPRB ,&
&  0.83604E+01_JPRB, 0.86896E+01_JPRB, 0.10044E+02_JPRB, 0.83333E+01_JPRB, 0.68515E+01_JPRB, 0.71260E+01_JPRB, 0.64729E+01_JPRB ,&
&  0.52690E+01_JPRB, 0.47838E+01_JPRB, 0.40916E+01_JPRB, 0.33443E+01_JPRB, 0.32239E+01_JPRB, 0.32023E+01_JPRB, 0.34770E+01_JPRB ,&
&  0.37204E+01_JPRB, 0.37525E+01_JPRB, 0.27575E+01_JPRB, 0.16821E+01_JPRB, 0.14923E+01_JPRB, 0.15308E+01_JPRB, 0.17836E+01_JPRB ,&
&  0.18304E+01_JPRB, 0.18686E+01_JPRB, 0.19989E+01_JPRB, 0.18827E+01_JPRB, 0.11602E+01_JPRB, 0.90404E+00_JPRB, 0.81752E+00_JPRB ,&
&  0.84238E+00_JPRB, 0.88153E+00_JPRB, 0.88162E+00_JPRB, 0.86316E+00_JPRB, 0.81917E+00_JPRB, 0.85540E+00_JPRB, 0.86733E+00_JPRB ,&
&  0.91350E+00_JPRB, 0.92301E+00_JPRB, 0.95198E+00_JPRB, 0.97862E+00_JPRB, 0.99903E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.12445E+01_JPRB, 0.13348E+01_JPRB, 0.14602E+01_JPRB, 0.15752E+01_JPRB, 0.16613E+01_JPRB, 0.17329E+01_JPRB, 0.17872E+01_JPRB ,&
&  0.18164E+01_JPRB, 0.18565E+01_JPRB, 0.20068E+01_JPRB, 0.21590E+01_JPRB, 0.23489E+01_JPRB, 0.25479E+01_JPRB, 0.27374E+01_JPRB ,&
&  0.28931E+01_JPRB, 0.30380E+01_JPRB, 0.31899E+01_JPRB, 0.34001E+01_JPRB, 0.36506E+01_JPRB, 0.40830E+01_JPRB, 0.39149E+01_JPRB ,&
&  0.28457E+01_JPRB, 0.23141E+01_JPRB, 0.24831E+01_JPRB, 0.25506E+01_JPRB, 0.25033E+01_JPRB, 0.27393E+01_JPRB, 0.28806E+01_JPRB ,&
&  0.27586E+01_JPRB, 0.27541E+01_JPRB, 0.27741E+01_JPRB, 0.28098E+01_JPRB, 0.28003E+01_JPRB, 0.28735E+01_JPRB, 0.29639E+01_JPRB ,&
&  0.30927E+01_JPRB, 0.32414E+01_JPRB, 0.33992E+01_JPRB, 0.35893E+01_JPRB, 0.37972E+01_JPRB, 0.40252E+01_JPRB, 0.42249E+01_JPRB ,&
&  0.43876E+01_JPRB, 0.45251E+01_JPRB, 0.45068E+01_JPRB, 0.43429E+01_JPRB, 0.44219E+01_JPRB, 0.46298E+01_JPRB, 0.47332E+01_JPRB ,&
&  0.47401E+01_JPRB, 0.48610E+01_JPRB, 0.51611E+01_JPRB, 0.53464E+01_JPRB, 0.53583E+01_JPRB, 0.62093E+01_JPRB, 0.68334E+01_JPRB ,&
&  0.75867E+01_JPRB, 0.77495E+01_JPRB, 0.91101E+01_JPRB, 0.76759E+01_JPRB, 0.64449E+01_JPRB, 0.59618E+01_JPRB, 0.51410E+01_JPRB ,&
&  0.43516E+01_JPRB, 0.40854E+01_JPRB, 0.36723E+01_JPRB, 0.26912E+01_JPRB, 0.26817E+01_JPRB, 0.24058E+01_JPRB, 0.30559E+01_JPRB ,&
&  0.34597E+01_JPRB, 0.37723E+01_JPRB, 0.22828E+01_JPRB, 0.16996E+01_JPRB, 0.14772E+01_JPRB, 0.16106E+01_JPRB, 0.16607E+01_JPRB ,&
&  0.15780E+01_JPRB, 0.14569E+01_JPRB, 0.13612E+01_JPRB, 0.13003E+01_JPRB, 0.88354E+00_JPRB, 0.77528E+00_JPRB, 0.75272E+00_JPRB ,&
&  0.80716E+00_JPRB, 0.85767E+00_JPRB, 0.86698E+00_JPRB, 0.86606E+00_JPRB, 0.88380E+00_JPRB, 0.94355E+00_JPRB, 0.98317E+00_JPRB ,&
&  0.10346E+01_JPRB, 0.10685E+01_JPRB, 0.11108E+01_JPRB, 0.11360E+01_JPRB, 0.11725E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.12859E+01_JPRB, 0.13704E+01_JPRB, 0.15004E+01_JPRB, 0.16172E+01_JPRB, 0.17062E+01_JPRB, 0.17532E+01_JPRB, 0.17684E+01_JPRB ,&
&  0.18129E+01_JPRB, 0.18938E+01_JPRB, 0.20246E+01_JPRB, 0.21425E+01_JPRB, 0.22795E+01_JPRB, 0.24175E+01_JPRB, 0.25544E+01_JPRB ,&
&  0.26807E+01_JPRB, 0.27962E+01_JPRB, 0.29501E+01_JPRB, 0.31480E+01_JPRB, 0.33846E+01_JPRB, 0.37528E+01_JPRB, 0.38159E+01_JPRB ,&
&  0.30294E+01_JPRB, 0.19402E+01_JPRB, 0.22117E+01_JPRB, 0.22844E+01_JPRB, 0.23068E+01_JPRB, 0.25827E+01_JPRB, 0.27205E+01_JPRB ,&
&  0.25300E+01_JPRB, 0.25675E+01_JPRB, 0.26835E+01_JPRB, 0.27411E+01_JPRB, 0.27753E+01_JPRB, 0.29415E+01_JPRB, 0.30886E+01_JPRB ,&
&  0.32525E+01_JPRB, 0.33980E+01_JPRB, 0.35375E+01_JPRB, 0.36885E+01_JPRB, 0.38280E+01_JPRB, 0.39851E+01_JPRB, 0.41686E+01_JPRB ,&
&  0.43644E+01_JPRB, 0.45122E+01_JPRB, 0.44847E+01_JPRB, 0.44551E+01_JPRB, 0.45431E+01_JPRB, 0.47843E+01_JPRB, 0.50116E+01_JPRB ,&
&  0.51524E+01_JPRB, 0.51675E+01_JPRB, 0.55854E+01_JPRB, 0.61468E+01_JPRB, 0.64130E+01_JPRB, 0.62704E+01_JPRB, 0.67362E+01_JPRB ,&
&  0.70644E+01_JPRB, 0.70051E+01_JPRB, 0.74750E+01_JPRB, 0.83083E+01_JPRB, 0.56848E+01_JPRB, 0.53830E+01_JPRB, 0.42526E+01_JPRB ,&
&  0.43411E+01_JPRB, 0.39831E+01_JPRB, 0.34911E+01_JPRB, 0.28661E+01_JPRB, 0.22816E+01_JPRB, 0.20005E+01_JPRB, 0.24742E+01_JPRB ,&
&  0.30744E+01_JPRB, 0.27195E+01_JPRB, 0.18930E+01_JPRB, 0.15799E+01_JPRB, 0.15320E+01_JPRB, 0.18766E+01_JPRB, 0.18468E+01_JPRB ,&
&  0.15735E+01_JPRB, 0.13969E+01_JPRB, 0.13409E+01_JPRB, 0.16444E+01_JPRB, 0.10856E+01_JPRB, 0.76462E+00_JPRB, 0.70665E+00_JPRB ,&
&  0.76563E+00_JPRB, 0.82573E+00_JPRB, 0.87953E+00_JPRB, 0.92040E+00_JPRB, 0.95872E+00_JPRB, 0.10079E+01_JPRB, 0.10355E+01_JPRB ,&
&  0.11001E+01_JPRB, 0.11089E+01_JPRB, 0.11674E+01_JPRB, 0.11996E+01_JPRB, 0.12418E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.12733E+01_JPRB, 0.13306E+01_JPRB, 0.13906E+01_JPRB, 0.14655E+01_JPRB, 0.15409E+01_JPRB, 0.15901E+01_JPRB, 0.16162E+01_JPRB ,&
&  0.16605E+01_JPRB, 0.17324E+01_JPRB, 0.18156E+01_JPRB, 0.18871E+01_JPRB, 0.19915E+01_JPRB, 0.20872E+01_JPRB, 0.21854E+01_JPRB ,&
&  0.22852E+01_JPRB, 0.24063E+01_JPRB, 0.25716E+01_JPRB, 0.27317E+01_JPRB, 0.28498E+01_JPRB, 0.29845E+01_JPRB, 0.31256E+01_JPRB ,&
&  0.32575E+01_JPRB, 0.28443E+01_JPRB, 0.25667E+01_JPRB, 0.20153E+01_JPRB, 0.16052E+01_JPRB, 0.20463E+01_JPRB, 0.20881E+01_JPRB ,&
&  0.21525E+01_JPRB, 0.22001E+01_JPRB, 0.22685E+01_JPRB, 0.23851E+01_JPRB, 0.25830E+01_JPRB, 0.27707E+01_JPRB, 0.29221E+01_JPRB ,&
&  0.30482E+01_JPRB, 0.31516E+01_JPRB, 0.32371E+01_JPRB, 0.33341E+01_JPRB, 0.34498E+01_JPRB, 0.35606E+01_JPRB, 0.36804E+01_JPRB ,&
&  0.38426E+01_JPRB, 0.40525E+01_JPRB, 0.41960E+01_JPRB, 0.43657E+01_JPRB, 0.45001E+01_JPRB, 0.48414E+01_JPRB, 0.52134E+01_JPRB ,&
&  0.54524E+01_JPRB, 0.55844E+01_JPRB, 0.59935E+01_JPRB, 0.64766E+01_JPRB, 0.65488E+01_JPRB, 0.58399E+01_JPRB, 0.61178E+01_JPRB ,&
&  0.65009E+01_JPRB, 0.63244E+01_JPRB, 0.61740E+01_JPRB, 0.79487E+01_JPRB, 0.48742E+01_JPRB, 0.50990E+01_JPRB, 0.49062E+01_JPRB ,&
&  0.41764E+01_JPRB, 0.39015E+01_JPRB, 0.33429E+01_JPRB, 0.25070E+01_JPRB, 0.20228E+01_JPRB, 0.17396E+01_JPRB, 0.21334E+01_JPRB ,&
&  0.24322E+01_JPRB, 0.19845E+01_JPRB, 0.14648E+01_JPRB, 0.12659E+01_JPRB, 0.13887E+01_JPRB, 0.18997E+01_JPRB, 0.19430E+01_JPRB ,&
&  0.14883E+01_JPRB, 0.14110E+01_JPRB, 0.12961E+01_JPRB, 0.20269E+01_JPRB, 0.16232E+01_JPRB, 0.76085E+00_JPRB, 0.62444E+00_JPRB ,&
&  0.65658E+00_JPRB, 0.72831E+00_JPRB, 0.82031E+00_JPRB, 0.88401E+00_JPRB, 0.91177E+00_JPRB, 0.94734E+00_JPRB, 0.98208E+00_JPRB ,&
&  0.10186E+01_JPRB, 0.10780E+01_JPRB, 0.11499E+01_JPRB, 0.11933E+01_JPRB, 0.12337E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.10377E+01_JPRB, 0.10825E+01_JPRB, 0.11186E+01_JPRB, 0.11366E+01_JPRB, 0.11409E+01_JPRB, 0.11342E+01_JPRB, 0.11067E+01_JPRB ,&
&  0.10757E+01_JPRB, 0.11014E+01_JPRB, 0.11785E+01_JPRB, 0.13477E+01_JPRB, 0.14864E+01_JPRB, 0.15756E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.17693E+01_JPRB, 0.18107E+01_JPRB, 0.17817E+01_JPRB, 0.17931E+01_JPRB, 0.17709E+01_JPRB, 0.17394E+01_JPRB, 0.19361E+01_JPRB ,&
&  0.22587E+01_JPRB, 0.26304E+01_JPRB, 0.29133E+01_JPRB, 0.31489E+01_JPRB, 0.25683E+01_JPRB, 0.20075E+01_JPRB, 0.22056E+01_JPRB ,&
&  0.24860E+01_JPRB, 0.22914E+01_JPRB, 0.19715E+01_JPRB, 0.19915E+01_JPRB, 0.20520E+01_JPRB, 0.21784E+01_JPRB, 0.23513E+01_JPRB ,&
&  0.24727E+01_JPRB, 0.25585E+01_JPRB, 0.26373E+01_JPRB, 0.27067E+01_JPRB, 0.27757E+01_JPRB, 0.28452E+01_JPRB, 0.29323E+01_JPRB ,&
&  0.30055E+01_JPRB, 0.30798E+01_JPRB, 0.32980E+01_JPRB, 0.36059E+01_JPRB, 0.39631E+01_JPRB, 0.44467E+01_JPRB, 0.48682E+01_JPRB ,&
&  0.51293E+01_JPRB, 0.53356E+01_JPRB, 0.59755E+01_JPRB, 0.63927E+01_JPRB, 0.60567E+01_JPRB, 0.54983E+01_JPRB, 0.57435E+01_JPRB ,&
&  0.57978E+01_JPRB, 0.54316E+01_JPRB, 0.43050E+01_JPRB, 0.62286E+01_JPRB, 0.62395E+01_JPRB, 0.52889E+01_JPRB, 0.42697E+01_JPRB ,&
&  0.37494E+01_JPRB, 0.35257E+01_JPRB, 0.31009E+01_JPRB, 0.25201E+01_JPRB, 0.19896E+01_JPRB, 0.21356E+01_JPRB, 0.21419E+01_JPRB ,&
&  0.21277E+01_JPRB, 0.16590E+01_JPRB, 0.12172E+01_JPRB, 0.10675E+01_JPRB, 0.11053E+01_JPRB, 0.16398E+01_JPRB, 0.16022E+01_JPRB ,&
&  0.12453E+01_JPRB, 0.11814E+01_JPRB, 0.11172E+01_JPRB, 0.17149E+01_JPRB, 0.28356E+01_JPRB, 0.80571E+00_JPRB, 0.55731E+00_JPRB ,&
&  0.57161E+00_JPRB, 0.61194E+00_JPRB, 0.65287E+00_JPRB, 0.66802E+00_JPRB, 0.67183E+00_JPRB, 0.70384E+00_JPRB, 0.75691E+00_JPRB ,&
&  0.81890E+00_JPRB, 0.87029E+00_JPRB, 0.90230E+00_JPRB, 0.95465E+00_JPRB, 0.99692E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.75610E+00_JPRB, 0.77953E+00_JPRB, 0.78189E+00_JPRB, 0.79082E+00_JPRB, 0.78533E+00_JPRB, 0.78546E+00_JPRB, 0.76245E+00_JPRB ,&
&  0.74652E+00_JPRB, 0.76399E+00_JPRB, 0.81823E+00_JPRB, 0.91992E+00_JPRB, 0.95356E+00_JPRB, 0.98327E+00_JPRB, 0.10002E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.99666E+00_JPRB, 0.97212E+00_JPRB, 0.92201E+00_JPRB, 0.90938E+00_JPRB, 0.94085E+00_JPRB, 0.10386E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.13844E+01_JPRB, 0.16286E+01_JPRB, 0.19153E+01_JPRB, 0.25628E+01_JPRB, 0.22863E+01_JPRB, 0.21647E+01_JPRB ,&
&  0.25021E+01_JPRB, 0.20082E+01_JPRB, 0.17691E+01_JPRB, 0.16241E+01_JPRB, 0.15390E+01_JPRB, 0.14982E+01_JPRB, 0.15665E+01_JPRB ,&
&  0.16084E+01_JPRB, 0.16800E+01_JPRB, 0.17649E+01_JPRB, 0.18026E+01_JPRB, 0.18434E+01_JPRB, 0.18257E+01_JPRB, 0.17951E+01_JPRB ,&
&  0.17559E+01_JPRB, 0.18772E+01_JPRB, 0.21399E+01_JPRB, 0.23261E+01_JPRB, 0.28092E+01_JPRB, 0.32462E+01_JPRB, 0.36225E+01_JPRB ,&
&  0.38396E+01_JPRB, 0.41257E+01_JPRB, 0.47002E+01_JPRB, 0.48619E+01_JPRB, 0.49631E+01_JPRB, 0.47386E+01_JPRB, 0.46105E+01_JPRB ,&
&  0.43519E+01_JPRB, 0.34998E+01_JPRB, 0.28882E+01_JPRB, 0.43568E+01_JPRB, 0.44583E+01_JPRB, 0.34924E+01_JPRB, 0.35959E+01_JPRB ,&
&  0.32942E+01_JPRB, 0.30846E+01_JPRB, 0.29253E+01_JPRB, 0.24208E+01_JPRB, 0.18648E+01_JPRB, 0.18737E+01_JPRB, 0.20715E+01_JPRB ,&
&  0.17905E+01_JPRB, 0.12945E+01_JPRB, 0.98955E+00_JPRB, 0.92753E+00_JPRB, 0.99293E+00_JPRB, 0.13507E+01_JPRB, 0.14259E+01_JPRB ,&
&  0.13356E+01_JPRB, 0.10198E+01_JPRB, 0.11464E+01_JPRB, 0.17883E+01_JPRB, 0.13498E+01_JPRB, 0.70237E+00_JPRB, 0.49417E+00_JPRB ,&
&  0.48998E+00_JPRB, 0.50919E+00_JPRB, 0.53059E+00_JPRB, 0.54118E+00_JPRB, 0.54594E+00_JPRB, 0.55596E+00_JPRB, 0.59326E+00_JPRB ,&
&  0.62548E+00_JPRB, 0.65738E+00_JPRB, 0.68379E+00_JPRB, 0.72005E+00_JPRB, 0.74197E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.73471E+00_JPRB, 0.75125E+00_JPRB, 0.77308E+00_JPRB, 0.78740E+00_JPRB, 0.80489E+00_JPRB, 0.81844E+00_JPRB, 0.85628E+00_JPRB ,&
&  0.88912E+00_JPRB, 0.91810E+00_JPRB, 0.95755E+00_JPRB, 0.99970E+00_JPRB, 0.10152E+01_JPRB, 0.10306E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.10756E+01_JPRB, 0.10652E+01_JPRB, 0.10645E+01_JPRB, 0.10632E+01_JPRB, 0.11382E+01_JPRB, 0.11321E+01_JPRB ,&
&  0.11190E+01_JPRB, 0.11375E+01_JPRB, 0.11504E+01_JPRB, 0.12145E+01_JPRB, 0.14100E+01_JPRB, 0.15950E+01_JPRB, 0.16556E+01_JPRB ,&
&  0.19444E+01_JPRB, 0.16235E+01_JPRB, 0.15344E+01_JPRB, 0.12048E+01_JPRB, 0.11386E+01_JPRB, 0.11596E+01_JPRB, 0.11310E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.11833E+01_JPRB, 0.12704E+01_JPRB, 0.12866E+01_JPRB, 0.13654E+01_JPRB, 0.14312E+01_JPRB, 0.14778E+01_JPRB ,&
&  0.15603E+01_JPRB, 0.16615E+01_JPRB, 0.17698E+01_JPRB, 0.19364E+01_JPRB, 0.20890E+01_JPRB, 0.25045E+01_JPRB, 0.26904E+01_JPRB ,&
&  0.27690E+01_JPRB, 0.29190E+01_JPRB, 0.27239E+01_JPRB, 0.27712E+01_JPRB, 0.31655E+01_JPRB, 0.31665E+01_JPRB, 0.30870E+01_JPRB ,&
&  0.31797E+01_JPRB, 0.28209E+01_JPRB, 0.20768E+01_JPRB, 0.30959E+01_JPRB, 0.34241E+01_JPRB, 0.31030E+01_JPRB, 0.30295E+01_JPRB ,&
&  0.28068E+01_JPRB, 0.24821E+01_JPRB, 0.22386E+01_JPRB, 0.20861E+01_JPRB, 0.17528E+01_JPRB, 0.15583E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.13279E+01_JPRB, 0.10683E+01_JPRB, 0.93128E+00_JPRB, 0.98194E+00_JPRB, 0.12618E+01_JPRB, 0.19879E+01_JPRB, 0.17914E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.10105E+01_JPRB, 0.14531E+01_JPRB, 0.21923E+01_JPRB, 0.11714E+01_JPRB, 0.73223E+00_JPRB, 0.49375E+00_JPRB ,&
&  0.47184E+00_JPRB, 0.49658E+00_JPRB, 0.52356E+00_JPRB, 0.54528E+00_JPRB, 0.57196E+00_JPRB, 0.59606E+00_JPRB, 0.61546E+00_JPRB ,&
&  0.63435E+00_JPRB, 0.65829E+00_JPRB, 0.67948E+00_JPRB, 0.70475E+00_JPRB, 0.72679E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.87135E+00_JPRB, 0.91214E+00_JPRB, 0.94183E+00_JPRB, 0.96656E+00_JPRB, 0.98592E+00_JPRB, 0.10054E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10822E+01_JPRB, 0.11066E+01_JPRB, 0.11382E+01_JPRB, 0.11782E+01_JPRB, 0.12132E+01_JPRB, 0.12289E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.12771E+01_JPRB, 0.13002E+01_JPRB, 0.13530E+01_JPRB, 0.13728E+01_JPRB, 0.14099E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.14102E+01_JPRB, 0.14103E+01_JPRB, 0.14163E+01_JPRB, 0.14330E+01_JPRB, 0.14448E+01_JPRB, 0.15668E+01_JPRB, 0.17495E+01_JPRB ,&
&  0.16502E+01_JPRB, 0.16187E+01_JPRB, 0.12595E+01_JPRB, 0.10477E+01_JPRB, 0.10512E+01_JPRB, 0.10869E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.11925E+01_JPRB, 0.12589E+01_JPRB, 0.12954E+01_JPRB, 0.14086E+01_JPRB, 0.16024E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.18223E+01_JPRB, 0.19358E+01_JPRB, 0.20017E+01_JPRB, 0.21915E+01_JPRB, 0.22141E+01_JPRB, 0.27015E+01_JPRB, 0.29438E+01_JPRB ,&
&  0.30066E+01_JPRB, 0.28547E+01_JPRB, 0.21899E+01_JPRB, 0.21894E+01_JPRB, 0.24130E+01_JPRB, 0.23448E+01_JPRB, 0.22644E+01_JPRB ,&
&  0.26413E+01_JPRB, 0.25483E+01_JPRB, 0.22080E+01_JPRB, 0.27745E+01_JPRB, 0.29247E+01_JPRB, 0.28311E+01_JPRB, 0.24767E+01_JPRB ,&
&  0.21492E+01_JPRB, 0.16552E+01_JPRB, 0.16140E+01_JPRB, 0.16149E+01_JPRB, 0.14724E+01_JPRB, 0.13153E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.95371E+00_JPRB, 0.94726E+00_JPRB, 0.10683E+01_JPRB, 0.16476E+01_JPRB, 0.19105E+01_JPRB, 0.23143E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10173E+01_JPRB, 0.81996E+00_JPRB, 0.10539E+01_JPRB, 0.85217E+00_JPRB, 0.57308E+00_JPRB, 0.50353E+00_JPRB ,&
&  0.54179E+00_JPRB, 0.58916E+00_JPRB, 0.62687E+00_JPRB, 0.65532E+00_JPRB, 0.68130E+00_JPRB, 0.71000E+00_JPRB, 0.72539E+00_JPRB ,&
&  0.74571E+00_JPRB, 0.76844E+00_JPRB, 0.79053E+00_JPRB, 0.82248E+00_JPRB, 0.84258E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.93926E+00_JPRB, 0.97298E+00_JPRB, 0.10028E+01_JPRB, 0.10312E+01_JPRB, 0.10621E+01_JPRB, 0.10943E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.11488E+01_JPRB, 0.11765E+01_JPRB, 0.12001E+01_JPRB, 0.12232E+01_JPRB, 0.12546E+01_JPRB, 0.12846E+01_JPRB, 0.13042E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.13660E+01_JPRB, 0.14069E+01_JPRB, 0.14449E+01_JPRB, 0.14652E+01_JPRB, 0.14821E+01_JPRB, 0.14827E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.14662E+01_JPRB, 0.14788E+01_JPRB, 0.15034E+01_JPRB, 0.15441E+01_JPRB, 0.15390E+01_JPRB, 0.16581E+01_JPRB ,&
&  0.14684E+01_JPRB, 0.17038E+01_JPRB, 0.13976E+01_JPRB, 0.12419E+01_JPRB, 0.12653E+01_JPRB, 0.12138E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.13424E+01_JPRB, 0.13986E+01_JPRB, 0.14470E+01_JPRB, 0.15558E+01_JPRB, 0.17156E+01_JPRB, 0.18363E+01_JPRB ,&
&  0.20216E+01_JPRB, 0.22287E+01_JPRB, 0.23934E+01_JPRB, 0.25823E+01_JPRB, 0.29897E+01_JPRB, 0.33423E+01_JPRB, 0.35185E+01_JPRB ,&
&  0.34808E+01_JPRB, 0.33086E+01_JPRB, 0.24861E+01_JPRB, 0.25666E+01_JPRB, 0.27201E+01_JPRB, 0.25073E+01_JPRB, 0.22752E+01_JPRB ,&
&  0.23676E+01_JPRB, 0.19602E+01_JPRB, 0.22428E+01_JPRB, 0.22771E+01_JPRB, 0.21679E+01_JPRB, 0.19064E+01_JPRB, 0.15598E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.10195E+01_JPRB, 0.10384E+01_JPRB, 0.11944E+01_JPRB, 0.11378E+01_JPRB, 0.93200E+00_JPRB, 0.86756E+00_JPRB ,&
&  0.80981E+00_JPRB, 0.76009E+00_JPRB, 0.78659E+00_JPRB, 0.97335E+00_JPRB, 0.13708E+01_JPRB, 0.23571E+01_JPRB, 0.22549E+01_JPRB ,&
&  0.99795E+00_JPRB, 0.89531E+00_JPRB, 0.56490E+00_JPRB, 0.61360E+00_JPRB, 0.63085E+00_JPRB, 0.54502E+00_JPRB, 0.56605E+00_JPRB ,&
&  0.67502E+00_JPRB, 0.77699E+00_JPRB, 0.79410E+00_JPRB, 0.74133E+00_JPRB, 0.72801E+00_JPRB, 0.74313E+00_JPRB, 0.76914E+00_JPRB ,&
&  0.79601E+00_JPRB, 0.82154E+00_JPRB, 0.84785E+00_JPRB, 0.87640E+00_JPRB, 0.90621E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.97346E+00_JPRB, 0.10083E+01_JPRB, 0.10388E+01_JPRB, 0.10658E+01_JPRB, 0.10987E+01_JPRB, 0.11340E+01_JPRB, 0.11666E+01_JPRB ,&
&  0.11963E+01_JPRB, 0.12234E+01_JPRB, 0.12530E+01_JPRB, 0.12813E+01_JPRB, 0.13107E+01_JPRB, 0.13430E+01_JPRB, 0.13804E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14422E+01_JPRB, 0.14885E+01_JPRB, 0.15438E+01_JPRB, 0.15823E+01_JPRB, 0.16042E+01_JPRB, 0.16152E+01_JPRB ,&
&  0.16272E+01_JPRB, 0.16658E+01_JPRB, 0.16922E+01_JPRB, 0.17169E+01_JPRB, 0.17584E+01_JPRB, 0.18927E+01_JPRB, 0.16833E+01_JPRB ,&
&  0.18709E+01_JPRB, 0.17015E+01_JPRB, 0.14768E+01_JPRB, 0.14067E+01_JPRB, 0.13972E+01_JPRB, 0.14006E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.14701E+01_JPRB, 0.14658E+01_JPRB, 0.14943E+01_JPRB, 0.15364E+01_JPRB, 0.16376E+01_JPRB, 0.17814E+01_JPRB, 0.19342E+01_JPRB ,&
&  0.21759E+01_JPRB, 0.24809E+01_JPRB, 0.28353E+01_JPRB, 0.32206E+01_JPRB, 0.36777E+01_JPRB, 0.40671E+01_JPRB, 0.43119E+01_JPRB ,&
&  0.43626E+01_JPRB, 0.43099E+01_JPRB, 0.36389E+01_JPRB, 0.38460E+01_JPRB, 0.37830E+01_JPRB, 0.31014E+01_JPRB, 0.25176E+01_JPRB ,&
&  0.20595E+01_JPRB, 0.15688E+01_JPRB, 0.16415E+01_JPRB, 0.16482E+01_JPRB, 0.14870E+01_JPRB, 0.12417E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.76226E+00_JPRB, 0.67528E+00_JPRB, 0.67525E+00_JPRB, 0.80977E+00_JPRB, 0.80712E+00_JPRB, 0.69808E+00_JPRB, 0.68713E+00_JPRB ,&
&  0.67775E+00_JPRB, 0.65856E+00_JPRB, 0.70519E+00_JPRB, 0.87721E+00_JPRB, 0.11531E+01_JPRB, 0.11635E+01_JPRB, 0.12176E+01_JPRB ,&
&  0.85646E+00_JPRB, 0.73516E+00_JPRB, 0.70151E+00_JPRB, 0.63334E+00_JPRB, 0.66995E+00_JPRB, 0.68380E+00_JPRB, 0.78247E+00_JPRB ,&
&  0.94367E+00_JPRB, 0.11571E+01_JPRB, 0.11294E+01_JPRB, 0.92559E+00_JPRB, 0.77541E+00_JPRB, 0.75172E+00_JPRB, 0.77661E+00_JPRB ,&
&  0.80612E+00_JPRB, 0.83252E+00_JPRB, 0.86426E+00_JPRB, 0.89886E+00_JPRB, 0.93523E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.98092E+00_JPRB, 0.10137E+01_JPRB, 0.10407E+01_JPRB, 0.10637E+01_JPRB, 0.10953E+01_JPRB, 0.11283E+01_JPRB, 0.11615E+01_JPRB ,&
&  0.11929E+01_JPRB, 0.12228E+01_JPRB, 0.12563E+01_JPRB, 0.12879E+01_JPRB, 0.13238E+01_JPRB, 0.13620E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.14435E+01_JPRB, 0.14831E+01_JPRB, 0.15151E+01_JPRB, 0.15823E+01_JPRB, 0.16613E+01_JPRB, 0.17157E+01_JPRB, 0.17526E+01_JPRB ,&
&  0.17676E+01_JPRB, 0.18108E+01_JPRB, 0.18616E+01_JPRB, 0.19096E+01_JPRB, 0.19281E+01_JPRB, 0.20382E+01_JPRB, 0.18313E+01_JPRB ,&
&  0.18599E+01_JPRB, 0.17196E+01_JPRB, 0.15567E+01_JPRB, 0.15354E+01_JPRB, 0.15223E+01_JPRB, 0.15300E+01_JPRB, 0.15353E+01_JPRB ,&
&  0.15392E+01_JPRB, 0.15648E+01_JPRB, 0.15656E+01_JPRB, 0.15956E+01_JPRB, 0.16700E+01_JPRB, 0.18132E+01_JPRB, 0.19919E+01_JPRB ,&
&  0.22370E+01_JPRB, 0.25678E+01_JPRB, 0.29689E+01_JPRB, 0.34098E+01_JPRB, 0.39118E+01_JPRB, 0.43667E+01_JPRB, 0.47943E+01_JPRB ,&
&  0.51152E+01_JPRB, 0.52995E+01_JPRB, 0.48518E+01_JPRB, 0.48124E+01_JPRB, 0.45215E+01_JPRB, 0.30742E+01_JPRB, 0.21873E+01_JPRB ,&
&  0.15235E+01_JPRB, 0.14290E+01_JPRB, 0.14602E+01_JPRB, 0.13350E+01_JPRB, 0.10495E+01_JPRB, 0.88886E+00_JPRB, 0.71368E+00_JPRB ,&
&  0.57567E+00_JPRB, 0.52805E+00_JPRB, 0.53941E+00_JPRB, 0.60945E+00_JPRB, 0.60013E+00_JPRB, 0.55698E+00_JPRB, 0.58364E+00_JPRB ,&
&  0.59363E+00_JPRB, 0.63218E+00_JPRB, 0.70328E+00_JPRB, 0.82260E+00_JPRB, 0.94471E+00_JPRB, 0.94696E+00_JPRB, 0.99054E+00_JPRB ,&
&  0.86110E+00_JPRB, 0.84112E+00_JPRB, 0.76987E+00_JPRB, 0.59753E+00_JPRB, 0.76615E+00_JPRB, 0.87128E+00_JPRB, 0.10044E+01_JPRB ,&
&  0.11390E+01_JPRB, 0.15376E+01_JPRB, 0.17704E+01_JPRB, 0.13896E+01_JPRB, 0.10874E+01_JPRB, 0.80863E+00_JPRB, 0.78397E+00_JPRB ,&
&  0.81416E+00_JPRB, 0.82885E+00_JPRB, 0.86046E+00_JPRB, 0.89943E+00_JPRB, 0.94152E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.94741E+00_JPRB, 0.98362E+00_JPRB, 0.10057E+01_JPRB, 0.10133E+01_JPRB, 0.10397E+01_JPRB, 0.10819E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11594E+01_JPRB, 0.11929E+01_JPRB, 0.12248E+01_JPRB, 0.12585E+01_JPRB, 0.12935E+01_JPRB, 0.13333E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.14324E+01_JPRB, 0.14737E+01_JPRB, 0.15477E+01_JPRB, 0.16276E+01_JPRB, 0.17291E+01_JPRB, 0.17907E+01_JPRB ,&
&  0.18098E+01_JPRB, 0.18658E+01_JPRB, 0.19309E+01_JPRB, 0.20071E+01_JPRB, 0.20098E+01_JPRB, 0.20315E+01_JPRB, 0.16013E+01_JPRB ,&
&  0.17414E+01_JPRB, 0.16909E+01_JPRB, 0.16212E+01_JPRB, 0.16015E+01_JPRB, 0.15752E+01_JPRB, 0.15458E+01_JPRB, 0.15145E+01_JPRB ,&
&  0.15220E+01_JPRB, 0.15165E+01_JPRB, 0.14759E+01_JPRB, 0.15781E+01_JPRB, 0.16577E+01_JPRB, 0.17772E+01_JPRB, 0.19562E+01_JPRB ,&
&  0.21575E+01_JPRB, 0.24485E+01_JPRB, 0.28274E+01_JPRB, 0.32828E+01_JPRB, 0.37840E+01_JPRB, 0.42872E+01_JPRB, 0.47855E+01_JPRB ,&
&  0.52495E+01_JPRB, 0.56171E+01_JPRB, 0.56775E+01_JPRB, 0.46411E+01_JPRB, 0.38466E+01_JPRB, 0.26554E+01_JPRB, 0.19623E+01_JPRB ,&
&  0.15854E+01_JPRB, 0.14184E+01_JPRB, 0.13425E+01_JPRB, 0.11886E+01_JPRB, 0.89613E+00_JPRB, 0.73931E+00_JPRB, 0.59653E+00_JPRB ,&
&  0.54069E+00_JPRB, 0.50312E+00_JPRB, 0.50471E+00_JPRB, 0.50770E+00_JPRB, 0.51074E+00_JPRB, 0.50462E+00_JPRB, 0.52977E+00_JPRB ,&
&  0.55358E+00_JPRB, 0.60680E+00_JPRB, 0.71026E+00_JPRB, 0.85261E+00_JPRB, 0.98507E+00_JPRB, 0.10500E+01_JPRB, 0.93022E+00_JPRB ,&
&  0.88361E+00_JPRB, 0.78643E+00_JPRB, 0.75435E+00_JPRB, 0.78935E+00_JPRB, 0.84638E+00_JPRB, 0.10225E+01_JPRB, 0.12314E+01_JPRB ,&
&  0.14885E+01_JPRB, 0.15233E+01_JPRB, 0.18115E+01_JPRB, 0.23656E+01_JPRB, 0.21332E+01_JPRB, 0.18352E+01_JPRB, 0.72755E+00_JPRB ,&
&  0.77442E+00_JPRB, 0.79196E+00_JPRB, 0.83224E+00_JPRB, 0.87244E+00_JPRB, 0.92228E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.76776E+00_JPRB, 0.80253E+00_JPRB, 0.83176E+00_JPRB, 0.84256E+00_JPRB, 0.88345E+00_JPRB, 0.92374E+00_JPRB, 0.95165E+00_JPRB ,&
&  0.97836E+00_JPRB, 0.10172E+01_JPRB, 0.10477E+01_JPRB, 0.10898E+01_JPRB, 0.11241E+01_JPRB, 0.11580E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.12776E+01_JPRB, 0.13329E+01_JPRB, 0.14116E+01_JPRB, 0.14927E+01_JPRB, 0.15936E+01_JPRB, 0.16864E+01_JPRB ,&
&  0.17569E+01_JPRB, 0.18322E+01_JPRB, 0.19205E+01_JPRB, 0.19794E+01_JPRB, 0.19614E+01_JPRB, 0.19257E+01_JPRB, 0.14338E+01_JPRB ,&
&  0.16291E+01_JPRB, 0.16563E+01_JPRB, 0.16507E+01_JPRB, 0.16213E+01_JPRB, 0.15726E+01_JPRB, 0.15120E+01_JPRB, 0.14835E+01_JPRB ,&
&  0.14666E+01_JPRB, 0.14326E+01_JPRB, 0.13967E+01_JPRB, 0.13937E+01_JPRB, 0.16006E+01_JPRB, 0.17138E+01_JPRB, 0.18710E+01_JPRB ,&
&  0.20793E+01_JPRB, 0.23348E+01_JPRB, 0.26473E+01_JPRB, 0.30965E+01_JPRB, 0.35928E+01_JPRB, 0.40869E+01_JPRB, 0.45980E+01_JPRB ,&
&  0.49995E+01_JPRB, 0.52075E+01_JPRB, 0.52598E+01_JPRB, 0.41690E+01_JPRB, 0.34037E+01_JPRB, 0.26185E+01_JPRB, 0.18193E+01_JPRB ,&
&  0.14869E+01_JPRB, 0.13612E+01_JPRB, 0.12971E+01_JPRB, 0.11203E+01_JPRB, 0.86776E+00_JPRB, 0.68974E+00_JPRB, 0.56682E+00_JPRB ,&
&  0.51919E+00_JPRB, 0.48032E+00_JPRB, 0.47973E+00_JPRB, 0.48575E+00_JPRB, 0.50041E+00_JPRB, 0.51106E+00_JPRB, 0.53256E+00_JPRB ,&
&  0.56344E+00_JPRB, 0.61198E+00_JPRB, 0.69284E+00_JPRB, 0.82288E+00_JPRB, 0.10052E+01_JPRB, 0.12058E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.91923E+00_JPRB, 0.78021E+00_JPRB, 0.73369E+00_JPRB, 0.71563E+00_JPRB, 0.73504E+00_JPRB, 0.79237E+00_JPRB, 0.86491E+00_JPRB ,&
&  0.99476E+00_JPRB, 0.10684E+01_JPRB, 0.96197E+00_JPRB, 0.80443E+00_JPRB, 0.10669E+01_JPRB, 0.10572E+01_JPRB, 0.59152E+00_JPRB ,&
&  0.64644E+00_JPRB, 0.63592E+00_JPRB, 0.66603E+00_JPRB, 0.70215E+00_JPRB, 0.74617E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.58715E+00_JPRB, 0.64204E+00_JPRB, 0.66798E+00_JPRB, 0.70074E+00_JPRB, 0.72791E+00_JPRB, 0.76651E+00_JPRB, 0.80321E+00_JPRB ,&
&  0.83958E+00_JPRB, 0.88161E+00_JPRB, 0.91196E+00_JPRB, 0.95352E+00_JPRB, 0.98846E+00_JPRB, 0.10194E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.11241E+01_JPRB, 0.11713E+01_JPRB, 0.12268E+01_JPRB, 0.12854E+01_JPRB, 0.13629E+01_JPRB, 0.14714E+01_JPRB ,&
&  0.16031E+01_JPRB, 0.17308E+01_JPRB, 0.18434E+01_JPRB, 0.19364E+01_JPRB, 0.19857E+01_JPRB, 0.20639E+01_JPRB, 0.20671E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.17278E+01_JPRB, 0.16693E+01_JPRB, 0.15945E+01_JPRB, 0.15379E+01_JPRB, 0.14860E+01_JPRB, 0.14738E+01_JPRB ,&
&  0.14552E+01_JPRB, 0.13600E+01_JPRB, 0.13488E+01_JPRB, 0.13498E+01_JPRB, 0.14679E+01_JPRB, 0.15859E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.19580E+01_JPRB, 0.22267E+01_JPRB, 0.25180E+01_JPRB, 0.28323E+01_JPRB, 0.32555E+01_JPRB, 0.37775E+01_JPRB, 0.42561E+01_JPRB ,&
&  0.47594E+01_JPRB, 0.52707E+01_JPRB, 0.55812E+01_JPRB, 0.46795E+01_JPRB, 0.37990E+01_JPRB, 0.33468E+01_JPRB, 0.23851E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.12174E+01_JPRB, 0.11942E+01_JPRB, 0.11141E+01_JPRB, 0.86708E+00_JPRB, 0.70832E+00_JPRB, 0.60124E+00_JPRB ,&
&  0.55207E+00_JPRB, 0.50740E+00_JPRB, 0.49451E+00_JPRB, 0.50043E+00_JPRB, 0.51400E+00_JPRB, 0.53075E+00_JPRB, 0.56629E+00_JPRB ,&
&  0.59273E+00_JPRB, 0.61771E+00_JPRB, 0.65036E+00_JPRB, 0.68354E+00_JPRB, 0.74586E+00_JPRB, 0.84646E+00_JPRB, 0.85782E+00_JPRB ,&
&  0.81556E+00_JPRB, 0.77375E+00_JPRB, 0.73417E+00_JPRB, 0.72303E+00_JPRB, 0.70237E+00_JPRB, 0.69285E+00_JPRB, 0.70503E+00_JPRB ,&
&  0.72647E+00_JPRB, 0.73973E+00_JPRB, 0.69834E+00_JPRB, 0.66631E+00_JPRB, 0.65719E+00_JPRB, 0.62604E+00_JPRB, 0.57239E+00_JPRB ,&
&  0.55260E+00_JPRB, 0.53533E+00_JPRB, 0.51710E+00_JPRB, 0.52617E+00_JPRB, 0.55288E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.49894E+00_JPRB, 0.51985E+00_JPRB, 0.54518E+00_JPRB, 0.57118E+00_JPRB, 0.59573E+00_JPRB, 0.61968E+00_JPRB, 0.65604E+00_JPRB ,&
&  0.69763E+00_JPRB, 0.72920E+00_JPRB, 0.74676E+00_JPRB, 0.77694E+00_JPRB, 0.81255E+00_JPRB, 0.85560E+00_JPRB, 0.89423E+00_JPRB ,&
&  0.93450E+00_JPRB, 0.98691E+00_JPRB, 0.10307E+01_JPRB, 0.10768E+01_JPRB, 0.11244E+01_JPRB, 0.11750E+01_JPRB, 0.12509E+01_JPRB ,&
&  0.13492E+01_JPRB, 0.14784E+01_JPRB, 0.16361E+01_JPRB, 0.17880E+01_JPRB, 0.19251E+01_JPRB, 0.20638E+01_JPRB, 0.21031E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.16321E+01_JPRB, 0.18529E+01_JPRB, 0.16667E+01_JPRB, 0.15241E+01_JPRB, 0.14432E+01_JPRB, 0.14086E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.12549E+01_JPRB, 0.12381E+01_JPRB, 0.13436E+01_JPRB, 0.13358E+01_JPRB, 0.14197E+01_JPRB, 0.15311E+01_JPRB ,&
&  0.17007E+01_JPRB, 0.19144E+01_JPRB, 0.21130E+01_JPRB, 0.23478E+01_JPRB, 0.26866E+01_JPRB, 0.31126E+01_JPRB, 0.36344E+01_JPRB ,&
&  0.42937E+01_JPRB, 0.50617E+01_JPRB, 0.57764E+01_JPRB, 0.46069E+01_JPRB, 0.41543E+01_JPRB, 0.38956E+01_JPRB, 0.32935E+01_JPRB ,&
&  0.18732E+01_JPRB, 0.12257E+01_JPRB, 0.11940E+01_JPRB, 0.11272E+01_JPRB, 0.93081E+00_JPRB, 0.79884E+00_JPRB, 0.70865E+00_JPRB ,&
&  0.64496E+00_JPRB, 0.59568E+00_JPRB, 0.57032E+00_JPRB, 0.56295E+00_JPRB, 0.56714E+00_JPRB, 0.59137E+00_JPRB, 0.62006E+00_JPRB ,&
&  0.64449E+00_JPRB, 0.64879E+00_JPRB, 0.67910E+00_JPRB, 0.69313E+00_JPRB, 0.73500E+00_JPRB, 0.76991E+00_JPRB, 0.79249E+00_JPRB ,&
&  0.80011E+00_JPRB, 0.78948E+00_JPRB, 0.78089E+00_JPRB, 0.77375E+00_JPRB, 0.76922E+00_JPRB, 0.76243E+00_JPRB, 0.75333E+00_JPRB ,&
&  0.74081E+00_JPRB, 0.74737E+00_JPRB, 0.73070E+00_JPRB, 0.71595E+00_JPRB, 0.71496E+00_JPRB, 0.68077E+00_JPRB, 0.66198E+00_JPRB ,&
&  0.64067E+00_JPRB, 0.58993E+00_JPRB, 0.55430E+00_JPRB, 0.52674E+00_JPRB, 0.50552E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.53060E+00_JPRB, 0.49962E+00_JPRB, 0.50243E+00_JPRB, 0.50825E+00_JPRB, 0.52750E+00_JPRB, 0.54733E+00_JPRB, 0.54339E+00_JPRB ,&
&  0.55344E+00_JPRB, 0.56840E+00_JPRB, 0.58943E+00_JPRB, 0.61265E+00_JPRB, 0.64078E+00_JPRB, 0.67188E+00_JPRB, 0.70323E+00_JPRB ,&
&  0.74416E+00_JPRB, 0.79310E+00_JPRB, 0.84630E+00_JPRB, 0.89645E+00_JPRB, 0.94842E+00_JPRB, 0.99043E+00_JPRB, 0.10319E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.11715E+01_JPRB, 0.12987E+01_JPRB, 0.14573E+01_JPRB, 0.16614E+01_JPRB, 0.19040E+01_JPRB, 0.20660E+01_JPRB ,&
&  0.14774E+01_JPRB, 0.78419E+00_JPRB, 0.17051E+01_JPRB, 0.18876E+01_JPRB, 0.16344E+01_JPRB, 0.15205E+01_JPRB, 0.13898E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.12923E+01_JPRB, 0.12229E+01_JPRB, 0.12468E+01_JPRB, 0.12274E+01_JPRB, 0.12633E+01_JPRB, 0.13159E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.15177E+01_JPRB, 0.17193E+01_JPRB, 0.18440E+01_JPRB, 0.20289E+01_JPRB, 0.23493E+01_JPRB, 0.28259E+01_JPRB ,&
&  0.34944E+01_JPRB, 0.43156E+01_JPRB, 0.51441E+01_JPRB, 0.39678E+01_JPRB, 0.30434E+01_JPRB, 0.23839E+01_JPRB, 0.19270E+01_JPRB ,&
&  0.16439E+01_JPRB, 0.12649E+01_JPRB, 0.11842E+01_JPRB, 0.11306E+01_JPRB, 0.10028E+01_JPRB, 0.85175E+00_JPRB, 0.82134E+00_JPRB ,&
&  0.75710E+00_JPRB, 0.70976E+00_JPRB, 0.68213E+00_JPRB, 0.66299E+00_JPRB, 0.65893E+00_JPRB, 0.67467E+00_JPRB, 0.69065E+00_JPRB ,&
&  0.71055E+00_JPRB, 0.72322E+00_JPRB, 0.74666E+00_JPRB, 0.77460E+00_JPRB, 0.78178E+00_JPRB, 0.78326E+00_JPRB, 0.79542E+00_JPRB ,&
&  0.79402E+00_JPRB, 0.80294E+00_JPRB, 0.81363E+00_JPRB, 0.81686E+00_JPRB, 0.81895E+00_JPRB, 0.80821E+00_JPRB, 0.81366E+00_JPRB ,&
&  0.81173E+00_JPRB, 0.80957E+00_JPRB, 0.79490E+00_JPRB, 0.79701E+00_JPRB, 0.79986E+00_JPRB, 0.79333E+00_JPRB, 0.80552E+00_JPRB ,&
&  0.79383E+00_JPRB, 0.75958E+00_JPRB, 0.69244E+00_JPRB, 0.61833E+00_JPRB, 0.57755E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.64988E+00_JPRB, 0.59983E+00_JPRB, 0.56031E+00_JPRB, 0.53922E+00_JPRB, 0.52554E+00_JPRB, 0.52040E+00_JPRB, 0.51897E+00_JPRB ,&
&  0.52340E+00_JPRB, 0.52862E+00_JPRB, 0.51791E+00_JPRB, 0.52496E+00_JPRB, 0.54165E+00_JPRB, 0.56227E+00_JPRB, 0.58897E+00_JPRB ,&
&  0.60855E+00_JPRB, 0.62845E+00_JPRB, 0.65037E+00_JPRB, 0.68301E+00_JPRB, 0.72466E+00_JPRB, 0.77137E+00_JPRB, 0.81701E+00_JPRB ,&
&  0.86315E+00_JPRB, 0.91489E+00_JPRB, 0.98906E+00_JPRB, 0.10945E+01_JPRB, 0.12439E+01_JPRB, 0.14650E+01_JPRB, 0.17138E+01_JPRB ,&
&  0.20515E+01_JPRB, 0.14947E+01_JPRB, 0.10359E+01_JPRB, 0.17120E+01_JPRB, 0.18060E+01_JPRB, 0.17064E+01_JPRB, 0.14462E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.14359E+01_JPRB, 0.13135E+01_JPRB, 0.12401E+01_JPRB, 0.11393E+01_JPRB, 0.11388E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11753E+01_JPRB, 0.13082E+01_JPRB, 0.14062E+01_JPRB, 0.14878E+01_JPRB, 0.16924E+01_JPRB, 0.21143E+01_JPRB ,&
&  0.26942E+01_JPRB, 0.34532E+01_JPRB, 0.43131E+01_JPRB, 0.36043E+01_JPRB, 0.26536E+01_JPRB, 0.19555E+01_JPRB, 0.16886E+01_JPRB ,&
&  0.15316E+01_JPRB, 0.14180E+01_JPRB, 0.12753E+01_JPRB, 0.11494E+01_JPRB, 0.10251E+01_JPRB, 0.90968E+00_JPRB, 0.88945E+00_JPRB ,&
&  0.85343E+00_JPRB, 0.79777E+00_JPRB, 0.75911E+00_JPRB, 0.72634E+00_JPRB, 0.71900E+00_JPRB, 0.72894E+00_JPRB, 0.74002E+00_JPRB ,&
&  0.75243E+00_JPRB, 0.76803E+00_JPRB, 0.79431E+00_JPRB, 0.82642E+00_JPRB, 0.83422E+00_JPRB, 0.82502E+00_JPRB, 0.81957E+00_JPRB ,&
&  0.81223E+00_JPRB, 0.80944E+00_JPRB, 0.82373E+00_JPRB, 0.83004E+00_JPRB, 0.82624E+00_JPRB, 0.81171E+00_JPRB, 0.82483E+00_JPRB ,&
&  0.82557E+00_JPRB, 0.83504E+00_JPRB, 0.82908E+00_JPRB, 0.83844E+00_JPRB, 0.84186E+00_JPRB, 0.86395E+00_JPRB, 0.88762E+00_JPRB ,&
&  0.90613E+00_JPRB, 0.89684E+00_JPRB, 0.85529E+00_JPRB, 0.73922E+00_JPRB, 0.69112E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.70540E+00_JPRB, 0.67271E+00_JPRB, 0.64758E+00_JPRB, 0.62469E+00_JPRB, 0.60120E+00_JPRB, 0.57833E+00_JPRB, 0.56011E+00_JPRB ,&
&  0.53919E+00_JPRB, 0.53632E+00_JPRB, 0.53286E+00_JPRB, 0.52729E+00_JPRB, 0.52032E+00_JPRB, 0.52296E+00_JPRB, 0.53306E+00_JPRB ,&
&  0.54081E+00_JPRB, 0.54676E+00_JPRB, 0.55541E+00_JPRB, 0.56010E+00_JPRB, 0.56777E+00_JPRB, 0.58485E+00_JPRB, 0.61250E+00_JPRB ,&
&  0.65201E+00_JPRB, 0.69536E+00_JPRB, 0.75122E+00_JPRB, 0.81911E+00_JPRB, 0.88883E+00_JPRB, 0.99870E+00_JPRB, 0.11367E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.14933E+01_JPRB, 0.87956E+00_JPRB, 0.15145E+01_JPRB, 0.19116E+01_JPRB, 0.18131E+01_JPRB, 0.16905E+01_JPRB ,&
&  0.16844E+01_JPRB, 0.15826E+01_JPRB, 0.14484E+01_JPRB, 0.12635E+01_JPRB, 0.10994E+01_JPRB, 0.10598E+01_JPRB, 0.10553E+01_JPRB ,&
&  0.10169E+01_JPRB, 0.99184E+00_JPRB, 0.10209E+01_JPRB, 0.10556E+01_JPRB, 0.10956E+01_JPRB, 0.11847E+01_JPRB, 0.14733E+01_JPRB ,&
&  0.19464E+01_JPRB, 0.26633E+01_JPRB, 0.34760E+01_JPRB, 0.32887E+01_JPRB, 0.24423E+01_JPRB, 0.20436E+01_JPRB, 0.18500E+01_JPRB ,&
&  0.17793E+01_JPRB, 0.16338E+01_JPRB, 0.13382E+01_JPRB, 0.11601E+01_JPRB, 0.10305E+01_JPRB, 0.93086E+00_JPRB, 0.92800E+00_JPRB ,&
&  0.90236E+00_JPRB, 0.85264E+00_JPRB, 0.82357E+00_JPRB, 0.78349E+00_JPRB, 0.76257E+00_JPRB, 0.75523E+00_JPRB, 0.76095E+00_JPRB ,&
&  0.77082E+00_JPRB, 0.78076E+00_JPRB, 0.79687E+00_JPRB, 0.81884E+00_JPRB, 0.83423E+00_JPRB, 0.83959E+00_JPRB, 0.83440E+00_JPRB ,&
&  0.82927E+00_JPRB, 0.82891E+00_JPRB, 0.83354E+00_JPRB, 0.83440E+00_JPRB, 0.82966E+00_JPRB, 0.82310E+00_JPRB, 0.81405E+00_JPRB ,&
&  0.80991E+00_JPRB, 0.81019E+00_JPRB, 0.80389E+00_JPRB, 0.79573E+00_JPRB, 0.82826E+00_JPRB, 0.83889E+00_JPRB, 0.84770E+00_JPRB ,&
&  0.88729E+00_JPRB, 0.91774E+00_JPRB, 0.90829E+00_JPRB, 0.79211E+00_JPRB, 0.74275E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.72786E+00_JPRB, 0.70094E+00_JPRB, 0.68005E+00_JPRB, 0.66401E+00_JPRB, 0.64443E+00_JPRB, 0.62510E+00_JPRB, 0.60868E+00_JPRB ,&
&  0.58677E+00_JPRB, 0.57350E+00_JPRB, 0.56465E+00_JPRB, 0.55526E+00_JPRB, 0.54595E+00_JPRB, 0.53725E+00_JPRB, 0.53007E+00_JPRB ,&
&  0.52321E+00_JPRB, 0.51761E+00_JPRB, 0.51456E+00_JPRB, 0.50837E+00_JPRB, 0.50290E+00_JPRB, 0.50208E+00_JPRB, 0.50811E+00_JPRB ,&
&  0.53000E+00_JPRB, 0.56291E+00_JPRB, 0.60292E+00_JPRB, 0.64574E+00_JPRB, 0.68314E+00_JPRB, 0.73373E+00_JPRB, 0.79575E+00_JPRB ,&
&  0.10507E+01_JPRB, 0.11975E+01_JPRB, 0.14070E+01_JPRB, 0.22469E+01_JPRB, 0.22959E+01_JPRB, 0.20557E+01_JPRB, 0.20350E+01_JPRB ,&
&  0.20525E+01_JPRB, 0.19060E+01_JPRB, 0.17549E+01_JPRB, 0.14139E+01_JPRB, 0.11634E+01_JPRB, 0.10384E+01_JPRB, 0.97731E+00_JPRB ,&
&  0.91882E+00_JPRB, 0.87195E+00_JPRB, 0.86953E+00_JPRB, 0.86274E+00_JPRB, 0.87832E+00_JPRB, 0.98809E+00_JPRB, 0.11363E+01_JPRB ,&
&  0.14404E+01_JPRB, 0.19689E+01_JPRB, 0.27042E+01_JPRB, 0.31881E+01_JPRB, 0.24610E+01_JPRB, 0.22614E+01_JPRB, 0.22862E+01_JPRB ,&
&  0.27565E+01_JPRB, 0.21047E+01_JPRB, 0.15221E+01_JPRB, 0.12056E+01_JPRB, 0.10321E+01_JPRB, 0.96471E+00_JPRB, 0.91653E+00_JPRB ,&
&  0.89041E+00_JPRB, 0.86265E+00_JPRB, 0.84173E+00_JPRB, 0.81739E+00_JPRB, 0.80048E+00_JPRB, 0.78461E+00_JPRB, 0.77685E+00_JPRB ,&
&  0.77591E+00_JPRB, 0.77880E+00_JPRB, 0.78326E+00_JPRB, 0.79355E+00_JPRB, 0.81245E+00_JPRB, 0.82715E+00_JPRB, 0.83152E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83656E+00_JPRB, 0.81500E+00_JPRB, 0.81147E+00_JPRB, 0.81713E+00_JPRB, 0.81114E+00_JPRB, 0.78348E+00_JPRB ,&
&  0.77235E+00_JPRB, 0.78527E+00_JPRB, 0.78699E+00_JPRB, 0.77954E+00_JPRB, 0.79019E+00_JPRB, 0.80072E+00_JPRB, 0.78181E+00_JPRB ,&
&  0.78717E+00_JPRB, 0.79679E+00_JPRB, 0.79775E+00_JPRB, 0.77843E+00_JPRB, 0.75603E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.73963E+00_JPRB, 0.71763E+00_JPRB, 0.69554E+00_JPRB, 0.67439E+00_JPRB, 0.65133E+00_JPRB, 0.63302E+00_JPRB, 0.62252E+00_JPRB ,&
&  0.60663E+00_JPRB, 0.59299E+00_JPRB, 0.58321E+00_JPRB, 0.57844E+00_JPRB, 0.57464E+00_JPRB, 0.56429E+00_JPRB, 0.54482E+00_JPRB ,&
&  0.52500E+00_JPRB, 0.51143E+00_JPRB, 0.50358E+00_JPRB, 0.49559E+00_JPRB, 0.48724E+00_JPRB, 0.48265E+00_JPRB, 0.48451E+00_JPRB ,&
&  0.49283E+00_JPRB, 0.50701E+00_JPRB, 0.53506E+00_JPRB, 0.56683E+00_JPRB, 0.59402E+00_JPRB, 0.61955E+00_JPRB, 0.63885E+00_JPRB ,&
&  0.78496E+00_JPRB, 0.10387E+01_JPRB, 0.10807E+01_JPRB, 0.19342E+01_JPRB, 0.20609E+01_JPRB, 0.20158E+01_JPRB, 0.20245E+01_JPRB ,&
&  0.20273E+01_JPRB, 0.21934E+01_JPRB, 0.18906E+01_JPRB, 0.15707E+01_JPRB, 0.12929E+01_JPRB, 0.11092E+01_JPRB, 0.98190E+00_JPRB ,&
&  0.90112E+00_JPRB, 0.83795E+00_JPRB, 0.80416E+00_JPRB, 0.78986E+00_JPRB, 0.78823E+00_JPRB, 0.84262E+00_JPRB, 0.98645E+00_JPRB ,&
&  0.11998E+01_JPRB, 0.15701E+01_JPRB, 0.21045E+01_JPRB, 0.24834E+01_JPRB, 0.23792E+01_JPRB, 0.24070E+01_JPRB, 0.26914E+01_JPRB ,&
&  0.34142E+01_JPRB, 0.28923E+01_JPRB, 0.18591E+01_JPRB, 0.13139E+01_JPRB, 0.10762E+01_JPRB, 0.97782E+00_JPRB, 0.90993E+00_JPRB ,&
&  0.87956E+00_JPRB, 0.85263E+00_JPRB, 0.83221E+00_JPRB, 0.81785E+00_JPRB, 0.80770E+00_JPRB, 0.80444E+00_JPRB, 0.79831E+00_JPRB ,&
&  0.79040E+00_JPRB, 0.78491E+00_JPRB, 0.78100E+00_JPRB, 0.77922E+00_JPRB, 0.78482E+00_JPRB, 0.79801E+00_JPRB, 0.81182E+00_JPRB ,&
&  0.82158E+00_JPRB, 0.82902E+00_JPRB, 0.80936E+00_JPRB, 0.79225E+00_JPRB, 0.79240E+00_JPRB, 0.78505E+00_JPRB, 0.76294E+00_JPRB ,&
&  0.76649E+00_JPRB, 0.77864E+00_JPRB, 0.77653E+00_JPRB, 0.75874E+00_JPRB, 0.76466E+00_JPRB, 0.78475E+00_JPRB, 0.76062E+00_JPRB ,&
&  0.75730E+00_JPRB, 0.76319E+00_JPRB, 0.77086E+00_JPRB, 0.76325E+00_JPRB, 0.75279E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.72534E+00_JPRB, 0.71220E+00_JPRB, 0.69350E+00_JPRB, 0.67255E+00_JPRB, 0.65139E+00_JPRB, 0.63327E+00_JPRB, 0.62329E+00_JPRB ,&
&  0.61144E+00_JPRB, 0.59937E+00_JPRB, 0.58874E+00_JPRB, 0.58260E+00_JPRB, 0.57990E+00_JPRB, 0.56981E+00_JPRB, 0.55127E+00_JPRB ,&
&  0.53166E+00_JPRB, 0.51410E+00_JPRB, 0.49965E+00_JPRB, 0.49095E+00_JPRB, 0.48169E+00_JPRB, 0.47738E+00_JPRB, 0.47711E+00_JPRB ,&
&  0.47882E+00_JPRB, 0.48648E+00_JPRB, 0.50357E+00_JPRB, 0.53039E+00_JPRB, 0.55514E+00_JPRB, 0.57337E+00_JPRB, 0.58559E+00_JPRB ,&
&  0.67109E+00_JPRB, 0.11816E+01_JPRB, 0.10082E+01_JPRB, 0.15084E+01_JPRB, 0.16273E+01_JPRB, 0.17139E+01_JPRB, 0.18338E+01_JPRB ,&
&  0.18841E+01_JPRB, 0.19529E+01_JPRB, 0.18364E+01_JPRB, 0.16259E+01_JPRB, 0.13836E+01_JPRB, 0.12106E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.97581E+00_JPRB, 0.90650E+00_JPRB, 0.85328E+00_JPRB, 0.81094E+00_JPRB, 0.78357E+00_JPRB, 0.79164E+00_JPRB, 0.90187E+00_JPRB ,&
&  0.11468E+01_JPRB, 0.14526E+01_JPRB, 0.17996E+01_JPRB, 0.21823E+01_JPRB, 0.21044E+01_JPRB, 0.23843E+01_JPRB, 0.32138E+01_JPRB ,&
&  0.37502E+01_JPRB, 0.36960E+01_JPRB, 0.24122E+01_JPRB, 0.15551E+01_JPRB, 0.11855E+01_JPRB, 0.10092E+01_JPRB, 0.90928E+00_JPRB ,&
&  0.85240E+00_JPRB, 0.82198E+00_JPRB, 0.80530E+00_JPRB, 0.79942E+00_JPRB, 0.80126E+00_JPRB, 0.81023E+00_JPRB, 0.81078E+00_JPRB ,&
&  0.80996E+00_JPRB, 0.80698E+00_JPRB, 0.80343E+00_JPRB, 0.79467E+00_JPRB, 0.78248E+00_JPRB, 0.77968E+00_JPRB, 0.78550E+00_JPRB ,&
&  0.79692E+00_JPRB, 0.80839E+00_JPRB, 0.79773E+00_JPRB, 0.78516E+00_JPRB, 0.77867E+00_JPRB, 0.77073E+00_JPRB, 0.75779E+00_JPRB ,&
&  0.76582E+00_JPRB, 0.77026E+00_JPRB, 0.76710E+00_JPRB, 0.76550E+00_JPRB, 0.75798E+00_JPRB, 0.77899E+00_JPRB, 0.75031E+00_JPRB ,&
&  0.74379E+00_JPRB, 0.74825E+00_JPRB, 0.75141E+00_JPRB, 0.74069E+00_JPRB, 0.73266E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.69385E+00_JPRB, 0.68279E+00_JPRB, 0.66567E+00_JPRB, 0.64772E+00_JPRB, 0.63529E+00_JPRB, 0.62161E+00_JPRB, 0.61092E+00_JPRB ,&
&  0.60178E+00_JPRB, 0.59365E+00_JPRB, 0.58727E+00_JPRB, 0.58010E+00_JPRB, 0.57063E+00_JPRB, 0.55743E+00_JPRB, 0.54416E+00_JPRB ,&
&  0.53046E+00_JPRB, 0.51734E+00_JPRB, 0.50394E+00_JPRB, 0.49011E+00_JPRB, 0.47900E+00_JPRB, 0.47160E+00_JPRB, 0.46657E+00_JPRB ,&
&  0.46578E+00_JPRB, 0.46891E+00_JPRB, 0.47913E+00_JPRB, 0.49993E+00_JPRB, 0.52552E+00_JPRB, 0.54727E+00_JPRB, 0.56867E+00_JPRB ,&
&  0.66226E+00_JPRB, 0.60704E+00_JPRB, 0.11309E+01_JPRB, 0.13061E+01_JPRB, 0.12754E+01_JPRB, 0.13012E+01_JPRB, 0.15334E+01_JPRB ,&
&  0.16827E+01_JPRB, 0.17377E+01_JPRB, 0.17353E+01_JPRB, 0.16362E+01_JPRB, 0.14504E+01_JPRB, 0.13114E+01_JPRB, 0.12045E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.10615E+01_JPRB, 0.98510E+00_JPRB, 0.90864E+00_JPRB, 0.85146E+00_JPRB, 0.83942E+00_JPRB, 0.89822E+00_JPRB ,&
&  0.11612E+01_JPRB, 0.14051E+01_JPRB, 0.16021E+01_JPRB, 0.18443E+01_JPRB, 0.19218E+01_JPRB, 0.22846E+01_JPRB, 0.27310E+01_JPRB ,&
&  0.36621E+01_JPRB, 0.37633E+01_JPRB, 0.27459E+01_JPRB, 0.19212E+01_JPRB, 0.14269E+01_JPRB, 0.11647E+01_JPRB, 0.98738E+00_JPRB ,&
&  0.88535E+00_JPRB, 0.82520E+00_JPRB, 0.78762E+00_JPRB, 0.77800E+00_JPRB, 0.79083E+00_JPRB, 0.81459E+00_JPRB, 0.83523E+00_JPRB ,&
&  0.85902E+00_JPRB, 0.87692E+00_JPRB, 0.87990E+00_JPRB, 0.87086E+00_JPRB, 0.83740E+00_JPRB, 0.79754E+00_JPRB, 0.77272E+00_JPRB ,&
&  0.76881E+00_JPRB, 0.77599E+00_JPRB, 0.77437E+00_JPRB, 0.76670E+00_JPRB, 0.76358E+00_JPRB, 0.75958E+00_JPRB, 0.75549E+00_JPRB ,&
&  0.75882E+00_JPRB, 0.76490E+00_JPRB, 0.76423E+00_JPRB, 0.76992E+00_JPRB, 0.74931E+00_JPRB, 0.77407E+00_JPRB, 0.75167E+00_JPRB ,&
&  0.73916E+00_JPRB, 0.73288E+00_JPRB, 0.72690E+00_JPRB, 0.71255E+00_JPRB, 0.69929E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.65872E+00_JPRB, 0.64765E+00_JPRB, 0.63093E+00_JPRB, 0.61443E+00_JPRB, 0.60462E+00_JPRB, 0.59748E+00_JPRB, 0.59235E+00_JPRB ,&
&  0.58914E+00_JPRB, 0.58094E+00_JPRB, 0.57267E+00_JPRB, 0.56097E+00_JPRB, 0.54580E+00_JPRB, 0.53273E+00_JPRB, 0.52445E+00_JPRB ,&
&  0.51461E+00_JPRB, 0.50953E+00_JPRB, 0.50299E+00_JPRB, 0.49048E+00_JPRB, 0.47669E+00_JPRB, 0.46519E+00_JPRB, 0.45773E+00_JPRB ,&
&  0.45408E+00_JPRB, 0.45631E+00_JPRB, 0.46303E+00_JPRB, 0.47857E+00_JPRB, 0.49452E+00_JPRB, 0.51289E+00_JPRB, 0.54408E+00_JPRB ,&
&  0.66250E+00_JPRB, 0.10674E+01_JPRB, 0.13410E+01_JPRB, 0.11902E+01_JPRB, 0.10935E+01_JPRB, 0.10252E+01_JPRB, 0.11729E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.14369E+01_JPRB, 0.15123E+01_JPRB, 0.15336E+01_JPRB, 0.14386E+01_JPRB, 0.13681E+01_JPRB, 0.12972E+01_JPRB ,&
&  0.12240E+01_JPRB, 0.11426E+01_JPRB, 0.10347E+01_JPRB, 0.92908E+00_JPRB, 0.87187E+00_JPRB, 0.87321E+00_JPRB, 0.94868E+00_JPRB ,&
&  0.11353E+01_JPRB, 0.13314E+01_JPRB, 0.14374E+01_JPRB, 0.16397E+01_JPRB, 0.19628E+01_JPRB, 0.20052E+01_JPRB, 0.26502E+01_JPRB ,&
&  0.34040E+01_JPRB, 0.30571E+01_JPRB, 0.25310E+01_JPRB, 0.20043E+01_JPRB, 0.16022E+01_JPRB, 0.12760E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.95568E+00_JPRB, 0.86859E+00_JPRB, 0.80635E+00_JPRB, 0.77392E+00_JPRB, 0.78586E+00_JPRB, 0.83086E+00_JPRB, 0.87076E+00_JPRB ,&
&  0.92621E+00_JPRB, 0.98753E+00_JPRB, 0.10265E+01_JPRB, 0.10280E+01_JPRB, 0.98633E+00_JPRB, 0.91472E+00_JPRB, 0.84482E+00_JPRB ,&
&  0.79847E+00_JPRB, 0.76972E+00_JPRB, 0.76003E+00_JPRB, 0.75135E+00_JPRB, 0.75602E+00_JPRB, 0.76750E+00_JPRB, 0.76452E+00_JPRB ,&
&  0.76103E+00_JPRB, 0.76508E+00_JPRB, 0.78004E+00_JPRB, 0.76992E+00_JPRB, 0.74778E+00_JPRB, 0.77833E+00_JPRB, 0.75604E+00_JPRB ,&
&  0.73540E+00_JPRB, 0.71610E+00_JPRB, 0.70103E+00_JPRB, 0.67651E+00_JPRB, 0.66360E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.63956E+00_JPRB, 0.62381E+00_JPRB, 0.60034E+00_JPRB, 0.58033E+00_JPRB, 0.56898E+00_JPRB, 0.56233E+00_JPRB, 0.56034E+00_JPRB ,&
&  0.55994E+00_JPRB, 0.55272E+00_JPRB, 0.54348E+00_JPRB, 0.52932E+00_JPRB, 0.51672E+00_JPRB, 0.50524E+00_JPRB, 0.49544E+00_JPRB ,&
&  0.48748E+00_JPRB, 0.48317E+00_JPRB, 0.47884E+00_JPRB, 0.47386E+00_JPRB, 0.46634E+00_JPRB, 0.45544E+00_JPRB, 0.44820E+00_JPRB ,&
&  0.44382E+00_JPRB, 0.44342E+00_JPRB, 0.44725E+00_JPRB, 0.45739E+00_JPRB, 0.46986E+00_JPRB, 0.48591E+00_JPRB, 0.51065E+00_JPRB ,&
&  0.58804E+00_JPRB, 0.90262E+00_JPRB, 0.95861E+00_JPRB, 0.93626E+00_JPRB, 0.93912E+00_JPRB, 0.92094E+00_JPRB, 0.97235E+00_JPRB ,&
&  0.10725E+01_JPRB, 0.11637E+01_JPRB, 0.12420E+01_JPRB, 0.12666E+01_JPRB, 0.12193E+01_JPRB, 0.11715E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.97399E+00_JPRB, 0.90463E+00_JPRB, 0.84204E+00_JPRB, 0.79986E+00_JPRB, 0.81441E+00_JPRB, 0.87327E+00_JPRB ,&
&  0.98676E+00_JPRB, 0.10852E+01_JPRB, 0.11662E+01_JPRB, 0.13294E+01_JPRB, 0.16659E+01_JPRB, 0.18764E+01_JPRB, 0.21560E+01_JPRB ,&
&  0.22247E+01_JPRB, 0.21531E+01_JPRB, 0.20191E+01_JPRB, 0.18326E+01_JPRB, 0.15798E+01_JPRB, 0.13374E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.99213E+00_JPRB, 0.89085E+00_JPRB, 0.82688E+00_JPRB, 0.79785E+00_JPRB, 0.80706E+00_JPRB, 0.84968E+00_JPRB, 0.90822E+00_JPRB ,&
&  0.98366E+00_JPRB, 0.10628E+01_JPRB, 0.11175E+01_JPRB, 0.11428E+01_JPRB, 0.11288E+01_JPRB, 0.10841E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.94137E+00_JPRB, 0.87824E+00_JPRB, 0.83046E+00_JPRB, 0.81088E+00_JPRB, 0.80839E+00_JPRB, 0.80837E+00_JPRB, 0.80643E+00_JPRB ,&
&  0.80955E+00_JPRB, 0.80914E+00_JPRB, 0.80448E+00_JPRB, 0.78115E+00_JPRB, 0.75066E+00_JPRB, 0.76770E+00_JPRB, 0.74443E+00_JPRB ,&
&  0.72106E+00_JPRB, 0.69475E+00_JPRB, 0.67218E+00_JPRB, 0.64453E+00_JPRB, 0.64476E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.66574E+00_JPRB, 0.63445E+00_JPRB, 0.59566E+00_JPRB, 0.56666E+00_JPRB, 0.54729E+00_JPRB, 0.53400E+00_JPRB, 0.52765E+00_JPRB ,&
&  0.52629E+00_JPRB, 0.51664E+00_JPRB, 0.50051E+00_JPRB, 0.48650E+00_JPRB, 0.47912E+00_JPRB, 0.47335E+00_JPRB, 0.46593E+00_JPRB ,&
&  0.46019E+00_JPRB, 0.45502E+00_JPRB, 0.45108E+00_JPRB, 0.44963E+00_JPRB, 0.44888E+00_JPRB, 0.44681E+00_JPRB, 0.44113E+00_JPRB ,&
&  0.43299E+00_JPRB, 0.42986E+00_JPRB, 0.43041E+00_JPRB, 0.43754E+00_JPRB, 0.44864E+00_JPRB, 0.46276E+00_JPRB, 0.48579E+00_JPRB ,&
&  0.56778E+00_JPRB, 0.61992E+00_JPRB, 0.68027E+00_JPRB, 0.69389E+00_JPRB, 0.71697E+00_JPRB, 0.77330E+00_JPRB, 0.82429E+00_JPRB ,&
&  0.87341E+00_JPRB, 0.93977E+00_JPRB, 0.10214E+01_JPRB, 0.10546E+01_JPRB, 0.10196E+01_JPRB, 0.95915E+00_JPRB, 0.90631E+00_JPRB ,&
&  0.85726E+00_JPRB, 0.82255E+00_JPRB, 0.79017E+00_JPRB, 0.76286E+00_JPRB, 0.75660E+00_JPRB, 0.77605E+00_JPRB, 0.81044E+00_JPRB ,&
&  0.83696E+00_JPRB, 0.84207E+00_JPRB, 0.93233E+00_JPRB, 0.10653E+01_JPRB, 0.12546E+01_JPRB, 0.13923E+01_JPRB, 0.15295E+01_JPRB ,&
&  0.16430E+01_JPRB, 0.16982E+01_JPRB, 0.17191E+01_JPRB, 0.16809E+01_JPRB, 0.15637E+01_JPRB, 0.14199E+01_JPRB, 0.12647E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.10062E+01_JPRB, 0.94559E+00_JPRB, 0.91432E+00_JPRB, 0.90768E+00_JPRB, 0.91707E+00_JPRB, 0.94416E+00_JPRB ,&
&  0.98697E+00_JPRB, 0.10290E+01_JPRB, 0.10646E+01_JPRB, 0.10819E+01_JPRB, 0.10787E+01_JPRB, 0.10726E+01_JPRB, 0.10617E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10196E+01_JPRB, 0.98123E+00_JPRB, 0.93826E+00_JPRB, 0.89977E+00_JPRB, 0.87204E+00_JPRB, 0.84975E+00_JPRB ,&
&  0.82720E+00_JPRB, 0.80996E+00_JPRB, 0.77575E+00_JPRB, 0.75135E+00_JPRB, 0.76613E+00_JPRB, 0.74470E+00_JPRB, 0.71857E+00_JPRB ,&
&  0.68754E+00_JPRB, 0.66016E+00_JPRB, 0.63462E+00_JPRB, 0.60761E+00_JPRB, 0.65405E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.56976E+00_JPRB, 0.56576E+00_JPRB, 0.55096E+00_JPRB, 0.53129E+00_JPRB, 0.51721E+00_JPRB, 0.50456E+00_JPRB, 0.49426E+00_JPRB ,&
&  0.48761E+00_JPRB, 0.47874E+00_JPRB, 0.46667E+00_JPRB, 0.45725E+00_JPRB, 0.45565E+00_JPRB, 0.45283E+00_JPRB, 0.44801E+00_JPRB ,&
&  0.43961E+00_JPRB, 0.43147E+00_JPRB, 0.42593E+00_JPRB, 0.42426E+00_JPRB, 0.42531E+00_JPRB, 0.42756E+00_JPRB, 0.42813E+00_JPRB ,&
&  0.42477E+00_JPRB, 0.41911E+00_JPRB, 0.41773E+00_JPRB, 0.41952E+00_JPRB, 0.42529E+00_JPRB, 0.43526E+00_JPRB, 0.45711E+00_JPRB ,&
&  0.51867E+00_JPRB, 0.51325E+00_JPRB, 0.54136E+00_JPRB, 0.55982E+00_JPRB, 0.58776E+00_JPRB, 0.64045E+00_JPRB, 0.69972E+00_JPRB ,&
&  0.73408E+00_JPRB, 0.76451E+00_JPRB, 0.81916E+00_JPRB, 0.86388E+00_JPRB, 0.86187E+00_JPRB, 0.82909E+00_JPRB, 0.78387E+00_JPRB ,&
&  0.73888E+00_JPRB, 0.70539E+00_JPRB, 0.67823E+00_JPRB, 0.65924E+00_JPRB, 0.65706E+00_JPRB, 0.66776E+00_JPRB, 0.67546E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.67025E+00_JPRB, 0.73198E+00_JPRB, 0.82705E+00_JPRB, 0.95094E+00_JPRB, 0.10768E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.13123E+01_JPRB, 0.13787E+01_JPRB, 0.14307E+01_JPRB, 0.14697E+01_JPRB, 0.14568E+01_JPRB, 0.14297E+01_JPRB, 0.13427E+01_JPRB ,&
&  0.12333E+01_JPRB, 0.11393E+01_JPRB, 0.10739E+01_JPRB, 0.10215E+01_JPRB, 0.98250E+00_JPRB, 0.95697E+00_JPRB, 0.94331E+00_JPRB ,&
&  0.94241E+00_JPRB, 0.94788E+00_JPRB, 0.94837E+00_JPRB, 0.94266E+00_JPRB, 0.93555E+00_JPRB, 0.93153E+00_JPRB, 0.92844E+00_JPRB ,&
&  0.92673E+00_JPRB, 0.92505E+00_JPRB, 0.91442E+00_JPRB, 0.89065E+00_JPRB, 0.85615E+00_JPRB, 0.82133E+00_JPRB, 0.78729E+00_JPRB ,&
&  0.75897E+00_JPRB, 0.74395E+00_JPRB, 0.72627E+00_JPRB, 0.71728E+00_JPRB, 0.69558E+00_JPRB, 0.67529E+00_JPRB, 0.65504E+00_JPRB ,&
&  0.63369E+00_JPRB, 0.61199E+00_JPRB, 0.58505E+00_JPRB, 0.55884E+00_JPRB, 0.55127E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.50931E+00_JPRB, 0.51443E+00_JPRB, 0.51345E+00_JPRB, 0.50115E+00_JPRB, 0.48755E+00_JPRB, 0.47966E+00_JPRB, 0.47052E+00_JPRB ,&
&  0.46026E+00_JPRB, 0.45204E+00_JPRB, 0.44110E+00_JPRB, 0.43585E+00_JPRB, 0.43352E+00_JPRB, 0.43276E+00_JPRB, 0.43040E+00_JPRB ,&
&  0.42448E+00_JPRB, 0.41552E+00_JPRB, 0.40876E+00_JPRB, 0.40603E+00_JPRB, 0.40703E+00_JPRB, 0.40928E+00_JPRB, 0.41118E+00_JPRB ,&
&  0.40947E+00_JPRB, 0.40774E+00_JPRB, 0.40568E+00_JPRB, 0.40439E+00_JPRB, 0.40520E+00_JPRB, 0.41338E+00_JPRB, 0.43011E+00_JPRB ,&
&  0.47832E+00_JPRB, 0.44526E+00_JPRB, 0.46028E+00_JPRB, 0.49005E+00_JPRB, 0.52181E+00_JPRB, 0.56783E+00_JPRB, 0.61686E+00_JPRB ,&
&  0.64578E+00_JPRB, 0.65017E+00_JPRB, 0.66851E+00_JPRB, 0.70317E+00_JPRB, 0.70697E+00_JPRB, 0.69969E+00_JPRB, 0.68017E+00_JPRB ,&
&  0.64961E+00_JPRB, 0.61350E+00_JPRB, 0.57906E+00_JPRB, 0.55748E+00_JPRB, 0.55576E+00_JPRB, 0.56328E+00_JPRB, 0.56365E+00_JPRB ,&
&  0.54365E+00_JPRB, 0.53748E+00_JPRB, 0.56508E+00_JPRB, 0.60760E+00_JPRB, 0.66098E+00_JPRB, 0.74386E+00_JPRB, 0.84794E+00_JPRB ,&
&  0.93914E+00_JPRB, 0.10309E+01_JPRB, 0.11259E+01_JPRB, 0.11999E+01_JPRB, 0.12596E+01_JPRB, 0.12792E+01_JPRB, 0.12473E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11277E+01_JPRB, 0.10629E+01_JPRB, 0.10003E+01_JPRB, 0.94681E+00_JPRB, 0.91165E+00_JPRB, 0.88847E+00_JPRB ,&
&  0.87212E+00_JPRB, 0.85865E+00_JPRB, 0.83917E+00_JPRB, 0.81260E+00_JPRB, 0.78508E+00_JPRB, 0.76181E+00_JPRB, 0.74155E+00_JPRB ,&
&  0.72436E+00_JPRB, 0.70645E+00_JPRB, 0.69223E+00_JPRB, 0.68162E+00_JPRB, 0.67036E+00_JPRB, 0.66009E+00_JPRB, 0.65081E+00_JPRB ,&
&  0.64073E+00_JPRB, 0.63622E+00_JPRB, 0.62575E+00_JPRB, 0.61139E+00_JPRB, 0.61572E+00_JPRB, 0.60500E+00_JPRB, 0.59540E+00_JPRB ,&
&  0.57975E+00_JPRB, 0.56182E+00_JPRB, 0.54175E+00_JPRB, 0.49186E+00_JPRB, 0.50625E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.46716E+00_JPRB, 0.47183E+00_JPRB, 0.47157E+00_JPRB, 0.46598E+00_JPRB, 0.45718E+00_JPRB, 0.45103E+00_JPRB, 0.44179E+00_JPRB ,&
&  0.43163E+00_JPRB, 0.42639E+00_JPRB, 0.42018E+00_JPRB, 0.41743E+00_JPRB, 0.41781E+00_JPRB, 0.41405E+00_JPRB, 0.41049E+00_JPRB ,&
&  0.40713E+00_JPRB, 0.40112E+00_JPRB, 0.39716E+00_JPRB, 0.39390E+00_JPRB, 0.39365E+00_JPRB, 0.39402E+00_JPRB, 0.39389E+00_JPRB ,&
&  0.39165E+00_JPRB, 0.38795E+00_JPRB, 0.38476E+00_JPRB, 0.38251E+00_JPRB, 0.38386E+00_JPRB, 0.39052E+00_JPRB, 0.40399E+00_JPRB ,&
&  0.43827E+00_JPRB, 0.39834E+00_JPRB, 0.41629E+00_JPRB, 0.43723E+00_JPRB, 0.46969E+00_JPRB, 0.49841E+00_JPRB, 0.51417E+00_JPRB ,&
&  0.53251E+00_JPRB, 0.55127E+00_JPRB, 0.56765E+00_JPRB, 0.58420E+00_JPRB, 0.58332E+00_JPRB, 0.57474E+00_JPRB, 0.56588E+00_JPRB ,&
&  0.55238E+00_JPRB, 0.53531E+00_JPRB, 0.51808E+00_JPRB, 0.50154E+00_JPRB, 0.49518E+00_JPRB, 0.49736E+00_JPRB, 0.49278E+00_JPRB ,&
&  0.47786E+00_JPRB, 0.46581E+00_JPRB, 0.46077E+00_JPRB, 0.46647E+00_JPRB, 0.48342E+00_JPRB, 0.51358E+00_JPRB, 0.57264E+00_JPRB ,&
&  0.64289E+00_JPRB, 0.72589E+00_JPRB, 0.81853E+00_JPRB, 0.89702E+00_JPRB, 0.95775E+00_JPRB, 0.99078E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.99535E+00_JPRB, 0.97535E+00_JPRB, 0.95076E+00_JPRB, 0.91726E+00_JPRB, 0.87197E+00_JPRB, 0.82743E+00_JPRB, 0.79071E+00_JPRB ,&
&  0.76186E+00_JPRB, 0.73256E+00_JPRB, 0.70158E+00_JPRB, 0.66796E+00_JPRB, 0.63221E+00_JPRB, 0.59884E+00_JPRB, 0.57098E+00_JPRB ,&
&  0.54760E+00_JPRB, 0.52783E+00_JPRB, 0.51555E+00_JPRB, 0.51024E+00_JPRB, 0.51339E+00_JPRB, 0.51931E+00_JPRB, 0.52316E+00_JPRB ,&
&  0.52654E+00_JPRB, 0.53201E+00_JPRB, 0.53476E+00_JPRB, 0.54105E+00_JPRB, 0.53859E+00_JPRB, 0.53443E+00_JPRB, 0.53094E+00_JPRB ,&
&  0.52322E+00_JPRB, 0.50863E+00_JPRB, 0.44749E+00_JPRB, 0.46486E+00_JPRB, 0.46500E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.43597E+00_JPRB, 0.43176E+00_JPRB, 0.42814E+00_JPRB, 0.42988E+00_JPRB, 0.42893E+00_JPRB, 0.42529E+00_JPRB, 0.41897E+00_JPRB ,&
&  0.40878E+00_JPRB, 0.40211E+00_JPRB, 0.39753E+00_JPRB, 0.39666E+00_JPRB, 0.39987E+00_JPRB, 0.39824E+00_JPRB, 0.39352E+00_JPRB ,&
&  0.38982E+00_JPRB, 0.38562E+00_JPRB, 0.38242E+00_JPRB, 0.37875E+00_JPRB, 0.37561E+00_JPRB, 0.37406E+00_JPRB, 0.37309E+00_JPRB ,&
&  0.37114E+00_JPRB, 0.36764E+00_JPRB, 0.36436E+00_JPRB, 0.36239E+00_JPRB, 0.36432E+00_JPRB, 0.37022E+00_JPRB, 0.37950E+00_JPRB ,&
&  0.37830E+00_JPRB, 0.36144E+00_JPRB, 0.38124E+00_JPRB, 0.40600E+00_JPRB, 0.42298E+00_JPRB, 0.44076E+00_JPRB, 0.45540E+00_JPRB ,&
&  0.46911E+00_JPRB, 0.47407E+00_JPRB, 0.48001E+00_JPRB, 0.48566E+00_JPRB, 0.48849E+00_JPRB, 0.48549E+00_JPRB, 0.48136E+00_JPRB ,&
&  0.47736E+00_JPRB, 0.47325E+00_JPRB, 0.46842E+00_JPRB, 0.46067E+00_JPRB, 0.45337E+00_JPRB, 0.44727E+00_JPRB, 0.44116E+00_JPRB ,&
&  0.43373E+00_JPRB, 0.42645E+00_JPRB, 0.42176E+00_JPRB, 0.42104E+00_JPRB, 0.42513E+00_JPRB, 0.43287E+00_JPRB, 0.44700E+00_JPRB ,&
&  0.46472E+00_JPRB, 0.49728E+00_JPRB, 0.55790E+00_JPRB, 0.62421E+00_JPRB, 0.67908E+00_JPRB, 0.71833E+00_JPRB, 0.75005E+00_JPRB ,&
&  0.76283E+00_JPRB, 0.76332E+00_JPRB, 0.75875E+00_JPRB, 0.74698E+00_JPRB, 0.72769E+00_JPRB, 0.70038E+00_JPRB, 0.66704E+00_JPRB ,&
&  0.63656E+00_JPRB, 0.60948E+00_JPRB, 0.58288E+00_JPRB, 0.55655E+00_JPRB, 0.53032E+00_JPRB, 0.50592E+00_JPRB, 0.48497E+00_JPRB ,&
&  0.46690E+00_JPRB, 0.45097E+00_JPRB, 0.43900E+00_JPRB, 0.43231E+00_JPRB, 0.43263E+00_JPRB, 0.43791E+00_JPRB, 0.44582E+00_JPRB ,&
&  0.45258E+00_JPRB, 0.46156E+00_JPRB, 0.46948E+00_JPRB, 0.47655E+00_JPRB, 0.48117E+00_JPRB, 0.47959E+00_JPRB, 0.47744E+00_JPRB ,&
&  0.47329E+00_JPRB, 0.46400E+00_JPRB, 0.45543E+00_JPRB, 0.44581E+00_JPRB, 0.43868E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.40244E+00_JPRB, 0.39655E+00_JPRB, 0.39599E+00_JPRB, 0.39967E+00_JPRB, 0.40387E+00_JPRB, 0.40379E+00_JPRB, 0.39981E+00_JPRB ,&
&  0.39021E+00_JPRB, 0.38182E+00_JPRB, 0.37535E+00_JPRB, 0.37289E+00_JPRB, 0.37542E+00_JPRB, 0.37622E+00_JPRB, 0.37360E+00_JPRB ,&
&  0.37043E+00_JPRB, 0.36450E+00_JPRB, 0.35932E+00_JPRB, 0.35421E+00_JPRB, 0.35048E+00_JPRB, 0.34871E+00_JPRB, 0.34788E+00_JPRB ,&
&  0.34627E+00_JPRB, 0.34429E+00_JPRB, 0.34356E+00_JPRB, 0.34316E+00_JPRB, 0.34455E+00_JPRB, 0.34849E+00_JPRB, 0.35391E+00_JPRB ,&
&  0.36416E+00_JPRB, 0.35020E+00_JPRB, 0.36991E+00_JPRB, 0.37535E+00_JPRB, 0.38196E+00_JPRB, 0.39191E+00_JPRB, 0.40452E+00_JPRB ,&
&  0.41599E+00_JPRB, 0.42143E+00_JPRB, 0.42420E+00_JPRB, 0.42546E+00_JPRB, 0.42474E+00_JPRB, 0.42513E+00_JPRB, 0.42272E+00_JPRB ,&
&  0.41816E+00_JPRB, 0.41604E+00_JPRB, 0.41497E+00_JPRB, 0.41271E+00_JPRB, 0.41060E+00_JPRB, 0.40736E+00_JPRB, 0.40387E+00_JPRB ,&
&  0.40081E+00_JPRB, 0.39736E+00_JPRB, 0.39427E+00_JPRB, 0.39088E+00_JPRB, 0.38861E+00_JPRB, 0.38937E+00_JPRB, 0.39356E+00_JPRB ,&
&  0.39845E+00_JPRB, 0.40665E+00_JPRB, 0.42434E+00_JPRB, 0.45165E+00_JPRB, 0.48498E+00_JPRB, 0.51794E+00_JPRB, 0.54460E+00_JPRB ,&
&  0.56134E+00_JPRB, 0.57005E+00_JPRB, 0.57171E+00_JPRB, 0.56898E+00_JPRB, 0.56104E+00_JPRB, 0.55006E+00_JPRB, 0.53475E+00_JPRB ,&
&  0.51918E+00_JPRB, 0.50407E+00_JPRB, 0.48856E+00_JPRB, 0.47299E+00_JPRB, 0.45621E+00_JPRB, 0.43880E+00_JPRB, 0.42289E+00_JPRB ,&
&  0.40998E+00_JPRB, 0.40052E+00_JPRB, 0.39365E+00_JPRB, 0.38941E+00_JPRB, 0.38707E+00_JPRB, 0.38806E+00_JPRB, 0.39108E+00_JPRB ,&
&  0.39598E+00_JPRB, 0.40235E+00_JPRB, 0.40959E+00_JPRB, 0.41580E+00_JPRB, 0.42161E+00_JPRB, 0.42360E+00_JPRB, 0.42482E+00_JPRB ,&
&  0.42347E+00_JPRB, 0.41877E+00_JPRB, 0.41501E+00_JPRB, 0.41102E+00_JPRB, 0.40750E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.37092E+00_JPRB, 0.36842E+00_JPRB, 0.37090E+00_JPRB, 0.37543E+00_JPRB, 0.38226E+00_JPRB, 0.38195E+00_JPRB, 0.37994E+00_JPRB ,&
&  0.37441E+00_JPRB, 0.36693E+00_JPRB, 0.35974E+00_JPRB, 0.35315E+00_JPRB, 0.35074E+00_JPRB, 0.35170E+00_JPRB, 0.35176E+00_JPRB ,&
&  0.35040E+00_JPRB, 0.34606E+00_JPRB, 0.34019E+00_JPRB, 0.33459E+00_JPRB, 0.33054E+00_JPRB, 0.32826E+00_JPRB, 0.32640E+00_JPRB ,&
&  0.32539E+00_JPRB, 0.32375E+00_JPRB, 0.32457E+00_JPRB, 0.32456E+00_JPRB, 0.32520E+00_JPRB, 0.32870E+00_JPRB, 0.33205E+00_JPRB ,&
&  0.33429E+00_JPRB, 0.34477E+00_JPRB, 0.34220E+00_JPRB, 0.34167E+00_JPRB, 0.34557E+00_JPRB, 0.35425E+00_JPRB, 0.36624E+00_JPRB ,&
&  0.37741E+00_JPRB, 0.38367E+00_JPRB, 0.38583E+00_JPRB, 0.38388E+00_JPRB, 0.38091E+00_JPRB, 0.37910E+00_JPRB, 0.37879E+00_JPRB ,&
&  0.37775E+00_JPRB, 0.37418E+00_JPRB, 0.36978E+00_JPRB, 0.36598E+00_JPRB, 0.36455E+00_JPRB, 0.36370E+00_JPRB, 0.36251E+00_JPRB ,&
&  0.36112E+00_JPRB, 0.35947E+00_JPRB, 0.35627E+00_JPRB, 0.35195E+00_JPRB, 0.34787E+00_JPRB, 0.34648E+00_JPRB, 0.34833E+00_JPRB ,&
&  0.35255E+00_JPRB, 0.35852E+00_JPRB, 0.36739E+00_JPRB, 0.37872E+00_JPRB, 0.39095E+00_JPRB, 0.40612E+00_JPRB, 0.42053E+00_JPRB ,&
&  0.43131E+00_JPRB, 0.43670E+00_JPRB, 0.43781E+00_JPRB, 0.43650E+00_JPRB, 0.43320E+00_JPRB, 0.42855E+00_JPRB, 0.42357E+00_JPRB ,&
&  0.41863E+00_JPRB, 0.41302E+00_JPRB, 0.40666E+00_JPRB, 0.39927E+00_JPRB, 0.38976E+00_JPRB, 0.37975E+00_JPRB, 0.37148E+00_JPRB ,&
&  0.36570E+00_JPRB, 0.36159E+00_JPRB, 0.35863E+00_JPRB, 0.35614E+00_JPRB, 0.35468E+00_JPRB, 0.35457E+00_JPRB, 0.35615E+00_JPRB ,&
&  0.35930E+00_JPRB, 0.36361E+00_JPRB, 0.36892E+00_JPRB, 0.37416E+00_JPRB, 0.37767E+00_JPRB, 0.38037E+00_JPRB, 0.38294E+00_JPRB ,&
&  0.38344E+00_JPRB, 0.38099E+00_JPRB, 0.37914E+00_JPRB, 0.37788E+00_JPRB, 0.37502E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.34409E+00_JPRB, 0.34409E+00_JPRB, 0.34878E+00_JPRB, 0.35378E+00_JPRB, 0.35928E+00_JPRB, 0.36130E+00_JPRB, 0.35916E+00_JPRB ,&
&  0.35636E+00_JPRB, 0.35249E+00_JPRB, 0.34660E+00_JPRB, 0.33917E+00_JPRB, 0.33245E+00_JPRB, 0.33079E+00_JPRB, 0.33114E+00_JPRB ,&
&  0.33144E+00_JPRB, 0.32925E+00_JPRB, 0.32518E+00_JPRB, 0.32074E+00_JPRB, 0.31576E+00_JPRB, 0.31139E+00_JPRB, 0.30809E+00_JPRB ,&
&  0.30712E+00_JPRB, 0.30662E+00_JPRB, 0.30733E+00_JPRB, 0.30805E+00_JPRB, 0.30906E+00_JPRB, 0.31166E+00_JPRB, 0.31737E+00_JPRB ,&
&  0.31575E+00_JPRB, 0.31980E+00_JPRB, 0.31153E+00_JPRB, 0.31447E+00_JPRB, 0.31759E+00_JPRB, 0.32281E+00_JPRB, 0.32938E+00_JPRB ,&
&  0.33664E+00_JPRB, 0.34221E+00_JPRB, 0.34559E+00_JPRB, 0.34561E+00_JPRB, 0.34384E+00_JPRB, 0.34380E+00_JPRB, 0.34416E+00_JPRB ,&
&  0.34391E+00_JPRB, 0.34321E+00_JPRB, 0.34119E+00_JPRB, 0.33688E+00_JPRB, 0.33094E+00_JPRB, 0.32596E+00_JPRB, 0.32262E+00_JPRB ,&
&  0.32230E+00_JPRB, 0.32242E+00_JPRB, 0.32128E+00_JPRB, 0.31951E+00_JPRB, 0.31748E+00_JPRB, 0.31668E+00_JPRB, 0.31817E+00_JPRB ,&
&  0.32114E+00_JPRB, 0.32429E+00_JPRB, 0.32725E+00_JPRB, 0.33236E+00_JPRB, 0.33946E+00_JPRB, 0.34606E+00_JPRB, 0.35218E+00_JPRB ,&
&  0.35742E+00_JPRB, 0.35906E+00_JPRB, 0.35808E+00_JPRB, 0.35577E+00_JPRB, 0.35400E+00_JPRB, 0.35267E+00_JPRB, 0.35231E+00_JPRB ,&
&  0.35157E+00_JPRB, 0.35011E+00_JPRB, 0.34743E+00_JPRB, 0.34363E+00_JPRB, 0.33973E+00_JPRB, 0.33587E+00_JPRB, 0.33295E+00_JPRB ,&
&  0.33119E+00_JPRB, 0.32994E+00_JPRB, 0.32855E+00_JPRB, 0.32721E+00_JPRB, 0.32706E+00_JPRB, 0.32730E+00_JPRB, 0.32824E+00_JPRB ,&
&  0.32988E+00_JPRB, 0.33236E+00_JPRB, 0.33585E+00_JPRB, 0.33957E+00_JPRB, 0.33996E+00_JPRB, 0.34023E+00_JPRB, 0.34231E+00_JPRB ,&
&  0.34442E+00_JPRB, 0.34513E+00_JPRB, 0.34495E+00_JPRB, 0.34482E+00_JPRB, 0.34516E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.32167E+00_JPRB, 0.32369E+00_JPRB, 0.32777E+00_JPRB, 0.33392E+00_JPRB, 0.33734E+00_JPRB, 0.34122E+00_JPRB, 0.34120E+00_JPRB ,&
&  0.33864E+00_JPRB, 0.33511E+00_JPRB, 0.33084E+00_JPRB, 0.32536E+00_JPRB, 0.31943E+00_JPRB, 0.31496E+00_JPRB, 0.31346E+00_JPRB ,&
&  0.31337E+00_JPRB, 0.31210E+00_JPRB, 0.30904E+00_JPRB, 0.30504E+00_JPRB, 0.30065E+00_JPRB, 0.29708E+00_JPRB, 0.29437E+00_JPRB ,&
&  0.29366E+00_JPRB, 0.29420E+00_JPRB, 0.29545E+00_JPRB, 0.29624E+00_JPRB, 0.29646E+00_JPRB, 0.29867E+00_JPRB, 0.30290E+00_JPRB ,&
&  0.30118E+00_JPRB, 0.29927E+00_JPRB, 0.29287E+00_JPRB, 0.29234E+00_JPRB, 0.29399E+00_JPRB, 0.29646E+00_JPRB, 0.29928E+00_JPRB ,&
&  0.30225E+00_JPRB, 0.30546E+00_JPRB, 0.30904E+00_JPRB, 0.31195E+00_JPRB, 0.31380E+00_JPRB, 0.31536E+00_JPRB, 0.31634E+00_JPRB ,&
&  0.31676E+00_JPRB, 0.31582E+00_JPRB, 0.31378E+00_JPRB, 0.31006E+00_JPRB, 0.30528E+00_JPRB, 0.30201E+00_JPRB, 0.30071E+00_JPRB ,&
&  0.29958E+00_JPRB, 0.29979E+00_JPRB, 0.29997E+00_JPRB, 0.29935E+00_JPRB, 0.29826E+00_JPRB, 0.29815E+00_JPRB, 0.29922E+00_JPRB ,&
&  0.30042E+00_JPRB, 0.30083E+00_JPRB, 0.30063E+00_JPRB, 0.30166E+00_JPRB, 0.30487E+00_JPRB, 0.30873E+00_JPRB, 0.31171E+00_JPRB ,&
&  0.31363E+00_JPRB, 0.31333E+00_JPRB, 0.31162E+00_JPRB, 0.30878E+00_JPRB, 0.30656E+00_JPRB, 0.30631E+00_JPRB, 0.30783E+00_JPRB ,&
&  0.30916E+00_JPRB, 0.30943E+00_JPRB, 0.30961E+00_JPRB, 0.30795E+00_JPRB, 0.30602E+00_JPRB, 0.30514E+00_JPRB, 0.30454E+00_JPRB ,&
&  0.30349E+00_JPRB, 0.30226E+00_JPRB, 0.30119E+00_JPRB, 0.30067E+00_JPRB, 0.30091E+00_JPRB, 0.30067E+00_JPRB, 0.30116E+00_JPRB ,&
&  0.30355E+00_JPRB, 0.30662E+00_JPRB, 0.30950E+00_JPRB, 0.31148E+00_JPRB, 0.31228E+00_JPRB, 0.31190E+00_JPRB, 0.31357E+00_JPRB ,&
&  0.31658E+00_JPRB, 0.31885E+00_JPRB, 0.31886E+00_JPRB, 0.31838E+00_JPRB, 0.31987E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.30235E+00_JPRB, 0.30489E+00_JPRB, 0.30922E+00_JPRB, 0.31423E+00_JPRB, 0.31745E+00_JPRB, 0.32198E+00_JPRB, 0.32281E+00_JPRB ,&
&  0.32216E+00_JPRB, 0.32039E+00_JPRB, 0.31617E+00_JPRB, 0.31098E+00_JPRB, 0.30593E+00_JPRB, 0.30164E+00_JPRB, 0.29868E+00_JPRB ,&
&  0.29709E+00_JPRB, 0.29568E+00_JPRB, 0.29397E+00_JPRB, 0.29132E+00_JPRB, 0.28818E+00_JPRB, 0.28538E+00_JPRB, 0.28326E+00_JPRB ,&
&  0.28261E+00_JPRB, 0.28267E+00_JPRB, 0.28313E+00_JPRB, 0.28312E+00_JPRB, 0.28278E+00_JPRB, 0.28383E+00_JPRB, 0.28556E+00_JPRB ,&
&  0.28465E+00_JPRB, 0.28040E+00_JPRB, 0.27737E+00_JPRB, 0.27709E+00_JPRB, 0.27707E+00_JPRB, 0.27749E+00_JPRB, 0.27821E+00_JPRB ,&
&  0.27855E+00_JPRB, 0.27914E+00_JPRB, 0.28087E+00_JPRB, 0.28296E+00_JPRB, 0.28476E+00_JPRB, 0.28761E+00_JPRB, 0.28906E+00_JPRB ,&
&  0.28925E+00_JPRB, 0.28851E+00_JPRB, 0.28709E+00_JPRB, 0.28455E+00_JPRB, 0.28290E+00_JPRB, 0.28261E+00_JPRB, 0.28212E+00_JPRB ,&
&  0.28301E+00_JPRB, 0.28393E+00_JPRB, 0.28478E+00_JPRB, 0.28388E+00_JPRB, 0.28205E+00_JPRB, 0.28189E+00_JPRB, 0.28253E+00_JPRB ,&
&  0.28284E+00_JPRB, 0.28207E+00_JPRB, 0.28050E+00_JPRB, 0.27981E+00_JPRB, 0.27983E+00_JPRB, 0.28080E+00_JPRB, 0.28123E+00_JPRB ,&
&  0.28078E+00_JPRB, 0.27913E+00_JPRB, 0.27786E+00_JPRB, 0.27717E+00_JPRB, 0.27679E+00_JPRB, 0.27710E+00_JPRB, 0.27704E+00_JPRB ,&
&  0.27816E+00_JPRB, 0.27924E+00_JPRB, 0.27850E+00_JPRB, 0.27779E+00_JPRB, 0.27714E+00_JPRB, 0.27698E+00_JPRB, 0.27657E+00_JPRB ,&
&  0.27570E+00_JPRB, 0.27474E+00_JPRB, 0.27415E+00_JPRB, 0.27409E+00_JPRB, 0.27306E+00_JPRB, 0.27183E+00_JPRB, 0.27245E+00_JPRB ,&
&  0.27571E+00_JPRB, 0.27933E+00_JPRB, 0.28247E+00_JPRB, 0.28454E+00_JPRB, 0.28521E+00_JPRB, 0.28661E+00_JPRB, 0.28934E+00_JPRB ,&
&  0.29161E+00_JPRB, 0.29251E+00_JPRB, 0.29475E+00_JPRB, 0.29681E+00_JPRB, 0.29904E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.28561E+00_JPRB, 0.28917E+00_JPRB, 0.29170E+00_JPRB, 0.29508E+00_JPRB, 0.29806E+00_JPRB, 0.30311E+00_JPRB, 0.30513E+00_JPRB ,&
&  0.30502E+00_JPRB, 0.30483E+00_JPRB, 0.30344E+00_JPRB, 0.30086E+00_JPRB, 0.29676E+00_JPRB, 0.29248E+00_JPRB, 0.28844E+00_JPRB ,&
&  0.28532E+00_JPRB, 0.28282E+00_JPRB, 0.28090E+00_JPRB, 0.27918E+00_JPRB, 0.27677E+00_JPRB, 0.27452E+00_JPRB, 0.27285E+00_JPRB ,&
&  0.27214E+00_JPRB, 0.27153E+00_JPRB, 0.27128E+00_JPRB, 0.27126E+00_JPRB, 0.27088E+00_JPRB, 0.27026E+00_JPRB, 0.27014E+00_JPRB ,&
&  0.26888E+00_JPRB, 0.26510E+00_JPRB, 0.26287E+00_JPRB, 0.26209E+00_JPRB, 0.26132E+00_JPRB, 0.26171E+00_JPRB, 0.26121E+00_JPRB ,&
&  0.26087E+00_JPRB, 0.26116E+00_JPRB, 0.26270E+00_JPRB, 0.26473E+00_JPRB, 0.26520E+00_JPRB, 0.26615E+00_JPRB, 0.26771E+00_JPRB ,&
&  0.26834E+00_JPRB, 0.26759E+00_JPRB, 0.26577E+00_JPRB, 0.26541E+00_JPRB, 0.26643E+00_JPRB, 0.26680E+00_JPRB, 0.26740E+00_JPRB ,&
&  0.26817E+00_JPRB, 0.26944E+00_JPRB, 0.27016E+00_JPRB, 0.26841E+00_JPRB, 0.26661E+00_JPRB, 0.26598E+00_JPRB, 0.26602E+00_JPRB ,&
&  0.26585E+00_JPRB, 0.26499E+00_JPRB, 0.26296E+00_JPRB, 0.26109E+00_JPRB, 0.25839E+00_JPRB, 0.25627E+00_JPRB, 0.25521E+00_JPRB ,&
&  0.25566E+00_JPRB, 0.25611E+00_JPRB, 0.25754E+00_JPRB, 0.25882E+00_JPRB, 0.25942E+00_JPRB, 0.25944E+00_JPRB, 0.25795E+00_JPRB ,&
&  0.25633E+00_JPRB, 0.25579E+00_JPRB, 0.25464E+00_JPRB, 0.25345E+00_JPRB, 0.25364E+00_JPRB, 0.25362E+00_JPRB, 0.25307E+00_JPRB ,&
&  0.25246E+00_JPRB, 0.25283E+00_JPRB, 0.25309E+00_JPRB, 0.25221E+00_JPRB, 0.25112E+00_JPRB, 0.24988E+00_JPRB, 0.25031E+00_JPRB ,&
&  0.25187E+00_JPRB, 0.25421E+00_JPRB, 0.25691E+00_JPRB, 0.25986E+00_JPRB, 0.26288E+00_JPRB, 0.26555E+00_JPRB, 0.26823E+00_JPRB ,&
&  0.27149E+00_JPRB, 0.27247E+00_JPRB, 0.27457E+00_JPRB, 0.27790E+00_JPRB, 0.28113E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.26596E+00_JPRB, 0.27114E+00_JPRB, 0.27383E+00_JPRB, 0.27596E+00_JPRB, 0.27846E+00_JPRB, 0.28318E+00_JPRB, 0.28655E+00_JPRB ,&
&  0.28733E+00_JPRB, 0.28751E+00_JPRB, 0.28731E+00_JPRB, 0.28639E+00_JPRB, 0.28473E+00_JPRB, 0.28310E+00_JPRB, 0.28028E+00_JPRB ,&
&  0.27671E+00_JPRB, 0.27344E+00_JPRB, 0.27050E+00_JPRB, 0.26774E+00_JPRB, 0.26516E+00_JPRB, 0.26279E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.26065E+00_JPRB, 0.26069E+00_JPRB, 0.25997E+00_JPRB, 0.25985E+00_JPRB, 0.25975E+00_JPRB, 0.25867E+00_JPRB, 0.25694E+00_JPRB ,&
&  0.25479E+00_JPRB, 0.25129E+00_JPRB, 0.24775E+00_JPRB, 0.24683E+00_JPRB, 0.24663E+00_JPRB, 0.24567E+00_JPRB, 0.24427E+00_JPRB ,&
&  0.24540E+00_JPRB, 0.24745E+00_JPRB, 0.24983E+00_JPRB, 0.25110E+00_JPRB, 0.25202E+00_JPRB, 0.25353E+00_JPRB, 0.25536E+00_JPRB ,&
&  0.25567E+00_JPRB, 0.25303E+00_JPRB, 0.25068E+00_JPRB, 0.25040E+00_JPRB, 0.25056E+00_JPRB, 0.25042E+00_JPRB, 0.25103E+00_JPRB ,&
&  0.25182E+00_JPRB, 0.25352E+00_JPRB, 0.25463E+00_JPRB, 0.25349E+00_JPRB, 0.25134E+00_JPRB, 0.24978E+00_JPRB, 0.24882E+00_JPRB ,&
&  0.24841E+00_JPRB, 0.24798E+00_JPRB, 0.24712E+00_JPRB, 0.24562E+00_JPRB, 0.24389E+00_JPRB, 0.24220E+00_JPRB, 0.24044E+00_JPRB ,&
&  0.23891E+00_JPRB, 0.23925E+00_JPRB, 0.24200E+00_JPRB, 0.24351E+00_JPRB, 0.24292E+00_JPRB, 0.24145E+00_JPRB, 0.23994E+00_JPRB ,&
&  0.23911E+00_JPRB, 0.23655E+00_JPRB, 0.23423E+00_JPRB, 0.23252E+00_JPRB, 0.23143E+00_JPRB, 0.22988E+00_JPRB, 0.22938E+00_JPRB ,&
&  0.22989E+00_JPRB, 0.23170E+00_JPRB, 0.23200E+00_JPRB, 0.23168E+00_JPRB, 0.23102E+00_JPRB, 0.23051E+00_JPRB, 0.23145E+00_JPRB ,&
&  0.23281E+00_JPRB, 0.23511E+00_JPRB, 0.23772E+00_JPRB, 0.24115E+00_JPRB, 0.24423E+00_JPRB, 0.24792E+00_JPRB, 0.25089E+00_JPRB ,&
&  0.25281E+00_JPRB, 0.25338E+00_JPRB, 0.25464E+00_JPRB, 0.25671E+00_JPRB, 0.26026E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.24511E+00_JPRB, 0.25094E+00_JPRB, 0.25584E+00_JPRB, 0.25829E+00_JPRB, 0.25969E+00_JPRB, 0.26202E+00_JPRB, 0.26460E+00_JPRB ,&
&  0.26658E+00_JPRB, 0.26787E+00_JPRB, 0.26861E+00_JPRB, 0.26872E+00_JPRB, 0.26800E+00_JPRB, 0.26663E+00_JPRB, 0.26495E+00_JPRB ,&
&  0.26304E+00_JPRB, 0.26023E+00_JPRB, 0.25761E+00_JPRB, 0.25567E+00_JPRB, 0.25410E+00_JPRB, 0.25228E+00_JPRB, 0.25086E+00_JPRB ,&
&  0.24946E+00_JPRB, 0.24919E+00_JPRB, 0.24863E+00_JPRB, 0.24782E+00_JPRB, 0.24700E+00_JPRB, 0.24577E+00_JPRB, 0.24476E+00_JPRB ,&
&  0.24193E+00_JPRB, 0.23738E+00_JPRB, 0.23573E+00_JPRB, 0.23451E+00_JPRB, 0.23467E+00_JPRB, 0.23386E+00_JPRB, 0.23278E+00_JPRB ,&
&  0.23429E+00_JPRB, 0.23723E+00_JPRB, 0.23948E+00_JPRB, 0.24068E+00_JPRB, 0.24153E+00_JPRB, 0.24287E+00_JPRB, 0.24433E+00_JPRB ,&
&  0.24203E+00_JPRB, 0.23913E+00_JPRB, 0.23709E+00_JPRB, 0.23621E+00_JPRB, 0.23634E+00_JPRB, 0.23744E+00_JPRB, 0.23940E+00_JPRB ,&
&  0.24115E+00_JPRB, 0.24224E+00_JPRB, 0.24200E+00_JPRB, 0.24053E+00_JPRB, 0.23862E+00_JPRB, 0.23772E+00_JPRB, 0.23727E+00_JPRB ,&
&  0.23709E+00_JPRB, 0.23671E+00_JPRB, 0.23625E+00_JPRB, 0.23496E+00_JPRB, 0.22836E+00_JPRB, 0.22508E+00_JPRB, 0.22243E+00_JPRB ,&
&  0.22328E+00_JPRB, 0.22259E+00_JPRB, 0.21977E+00_JPRB, 0.22648E+00_JPRB, 0.23338E+00_JPRB, 0.22795E+00_JPRB, 0.22180E+00_JPRB ,&
&  0.21339E+00_JPRB, 0.21582E+00_JPRB, 0.21302E+00_JPRB, 0.20767E+00_JPRB, 0.20649E+00_JPRB, 0.20683E+00_JPRB, 0.20624E+00_JPRB ,&
&  0.20693E+00_JPRB, 0.20651E+00_JPRB, 0.20753E+00_JPRB, 0.20797E+00_JPRB, 0.20823E+00_JPRB, 0.20929E+00_JPRB, 0.21036E+00_JPRB ,&
&  0.21307E+00_JPRB, 0.21756E+00_JPRB, 0.22229E+00_JPRB, 0.22303E+00_JPRB, 0.22627E+00_JPRB, 0.22895E+00_JPRB, 0.23060E+00_JPRB ,&
&  0.23195E+00_JPRB, 0.23405E+00_JPRB, 0.23602E+00_JPRB, 0.23770E+00_JPRB, 0.24044E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.23078E+00_JPRB, 0.23335E+00_JPRB, 0.23645E+00_JPRB, 0.23950E+00_JPRB, 0.24163E+00_JPRB, 0.24375E+00_JPRB, 0.24565E+00_JPRB ,&
&  0.24733E+00_JPRB, 0.24853E+00_JPRB, 0.24936E+00_JPRB, 0.24989E+00_JPRB, 0.25029E+00_JPRB, 0.25013E+00_JPRB, 0.24953E+00_JPRB ,&
&  0.24891E+00_JPRB, 0.24771E+00_JPRB, 0.24605E+00_JPRB, 0.24474E+00_JPRB, 0.24394E+00_JPRB, 0.24283E+00_JPRB, 0.24116E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.23767E+00_JPRB, 0.23621E+00_JPRB, 0.23468E+00_JPRB, 0.23446E+00_JPRB, 0.23505E+00_JPRB, 0.23556E+00_JPRB ,&
&  0.23013E+00_JPRB, 0.22471E+00_JPRB, 0.22656E+00_JPRB, 0.22760E+00_JPRB, 0.22416E+00_JPRB, 0.22544E+00_JPRB, 0.22610E+00_JPRB ,&
&  0.22701E+00_JPRB, 0.22836E+00_JPRB, 0.22978E+00_JPRB, 0.23042E+00_JPRB, 0.23084E+00_JPRB, 0.23158E+00_JPRB, 0.23099E+00_JPRB ,&
&  0.22891E+00_JPRB, 0.22721E+00_JPRB, 0.22574E+00_JPRB, 0.22423E+00_JPRB, 0.22397E+00_JPRB, 0.22599E+00_JPRB, 0.22730E+00_JPRB ,&
&  0.22641E+00_JPRB, 0.22489E+00_JPRB, 0.22345E+00_JPRB, 0.22397E+00_JPRB, 0.22492E+00_JPRB, 0.22545E+00_JPRB, 0.22304E+00_JPRB ,&
&  0.22213E+00_JPRB, 0.21913E+00_JPRB, 0.21646E+00_JPRB, 0.21380E+00_JPRB, 0.21267E+00_JPRB, 0.20876E+00_JPRB, 0.20451E+00_JPRB ,&
&  0.20374E+00_JPRB, 0.20836E+00_JPRB, 0.21509E+00_JPRB, 0.21663E+00_JPRB, 0.20250E+00_JPRB, 0.20522E+00_JPRB, 0.20634E+00_JPRB ,&
&  0.20163E+00_JPRB, 0.19580E+00_JPRB, 0.19148E+00_JPRB, 0.18963E+00_JPRB, 0.18891E+00_JPRB, 0.18818E+00_JPRB, 0.18860E+00_JPRB ,&
&  0.18947E+00_JPRB, 0.18983E+00_JPRB, 0.19039E+00_JPRB, 0.19174E+00_JPRB, 0.19310E+00_JPRB, 0.19450E+00_JPRB, 0.19578E+00_JPRB ,&
&  0.19870E+00_JPRB, 0.20219E+00_JPRB, 0.20473E+00_JPRB, 0.20794E+00_JPRB, 0.21136E+00_JPRB, 0.21467E+00_JPRB, 0.21768E+00_JPRB ,&
&  0.22097E+00_JPRB, 0.22245E+00_JPRB, 0.22425E+00_JPRB, 0.22758E+00_JPRB, 0.22921E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.22394E+00_JPRB, 0.22695E+00_JPRB, 0.22896E+00_JPRB, 0.23090E+00_JPRB, 0.23241E+00_JPRB, 0.23397E+00_JPRB, 0.23468E+00_JPRB ,&
&  0.23532E+00_JPRB, 0.23531E+00_JPRB, 0.23580E+00_JPRB, 0.23580E+00_JPRB, 0.23603E+00_JPRB, 0.23483E+00_JPRB, 0.23435E+00_JPRB ,&
&  0.23392E+00_JPRB, 0.23445E+00_JPRB, 0.23458E+00_JPRB, 0.23457E+00_JPRB, 0.23387E+00_JPRB, 0.23289E+00_JPRB, 0.23122E+00_JPRB ,&
&  0.22771E+00_JPRB, 0.22500E+00_JPRB, 0.22502E+00_JPRB, 0.22427E+00_JPRB, 0.22463E+00_JPRB, 0.22453E+00_JPRB, 0.22501E+00_JPRB ,&
&  0.22008E+00_JPRB, 0.21863E+00_JPRB, 0.21955E+00_JPRB, 0.21825E+00_JPRB, 0.21965E+00_JPRB, 0.22031E+00_JPRB, 0.22219E+00_JPRB ,&
&  0.22248E+00_JPRB, 0.22262E+00_JPRB, 0.22340E+00_JPRB, 0.22273E+00_JPRB, 0.22297E+00_JPRB, 0.22349E+00_JPRB, 0.22227E+00_JPRB ,&
&  0.21886E+00_JPRB, 0.21642E+00_JPRB, 0.21283E+00_JPRB, 0.20426E+00_JPRB, 0.20284E+00_JPRB, 0.20376E+00_JPRB, 0.20266E+00_JPRB ,&
&  0.19985E+00_JPRB, 0.20075E+00_JPRB, 0.20031E+00_JPRB, 0.19853E+00_JPRB, 0.20089E+00_JPRB, 0.20423E+00_JPRB, 0.20439E+00_JPRB ,&
&  0.20289E+00_JPRB, 0.20252E+00_JPRB, 0.20178E+00_JPRB, 0.20031E+00_JPRB, 0.19857E+00_JPRB, 0.19790E+00_JPRB, 0.19725E+00_JPRB ,&
&  0.19684E+00_JPRB, 0.19511E+00_JPRB, 0.19878E+00_JPRB, 0.19929E+00_JPRB, 0.19356E+00_JPRB, 0.19693E+00_JPRB, 0.19463E+00_JPRB ,&
&  0.18654E+00_JPRB, 0.18302E+00_JPRB, 0.18092E+00_JPRB, 0.17911E+00_JPRB, 0.17868E+00_JPRB, 0.17864E+00_JPRB, 0.17829E+00_JPRB ,&
&  0.17915E+00_JPRB, 0.18034E+00_JPRB, 0.18087E+00_JPRB, 0.18168E+00_JPRB, 0.18281E+00_JPRB, 0.18450E+00_JPRB, 0.18676E+00_JPRB ,&
&  0.18877E+00_JPRB, 0.19150E+00_JPRB, 0.19282E+00_JPRB, 0.19498E+00_JPRB, 0.19657E+00_JPRB, 0.19840E+00_JPRB, 0.20288E+00_JPRB ,&
&  0.20635E+00_JPRB, 0.21316E+00_JPRB, 0.22226E+00_JPRB, 0.22449E+00_JPRB, 0.22097E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.21876E+00_JPRB, 0.22104E+00_JPRB, 0.22315E+00_JPRB, 0.22489E+00_JPRB, 0.22670E+00_JPRB, 0.22819E+00_JPRB, 0.22896E+00_JPRB ,&
&  0.22908E+00_JPRB, 0.22831E+00_JPRB, 0.22828E+00_JPRB, 0.22659E+00_JPRB, 0.22496E+00_JPRB, 0.22060E+00_JPRB, 0.22033E+00_JPRB ,&
&  0.21939E+00_JPRB, 0.22026E+00_JPRB, 0.21979E+00_JPRB, 0.22014E+00_JPRB, 0.21928E+00_JPRB, 0.21879E+00_JPRB, 0.21671E+00_JPRB ,&
&  0.21370E+00_JPRB, 0.21154E+00_JPRB, 0.20952E+00_JPRB, 0.20905E+00_JPRB, 0.20997E+00_JPRB, 0.21057E+00_JPRB, 0.21013E+00_JPRB ,&
&  0.21017E+00_JPRB, 0.21040E+00_JPRB, 0.21079E+00_JPRB, 0.21060E+00_JPRB, 0.21617E+00_JPRB, 0.21857E+00_JPRB, 0.21975E+00_JPRB ,&
&  0.21936E+00_JPRB, 0.21943E+00_JPRB, 0.21936E+00_JPRB, 0.21815E+00_JPRB, 0.21697E+00_JPRB, 0.21193E+00_JPRB, 0.20886E+00_JPRB ,&
&  0.20084E+00_JPRB, 0.19501E+00_JPRB, 0.19063E+00_JPRB, 0.18794E+00_JPRB, 0.18692E+00_JPRB, 0.18712E+00_JPRB, 0.18681E+00_JPRB ,&
&  0.18516E+00_JPRB, 0.18360E+00_JPRB, 0.18263E+00_JPRB, 0.18354E+00_JPRB, 0.18499E+00_JPRB, 0.18655E+00_JPRB, 0.18831E+00_JPRB ,&
&  0.18982E+00_JPRB, 0.19024E+00_JPRB, 0.19021E+00_JPRB, 0.19077E+00_JPRB, 0.19122E+00_JPRB, 0.19188E+00_JPRB, 0.19233E+00_JPRB ,&
&  0.19352E+00_JPRB, 0.19242E+00_JPRB, 0.19316E+00_JPRB, 0.19460E+00_JPRB, 0.19181E+00_JPRB, 0.18744E+00_JPRB, 0.18317E+00_JPRB ,&
&  0.17899E+00_JPRB, 0.17666E+00_JPRB, 0.17471E+00_JPRB, 0.17341E+00_JPRB, 0.17306E+00_JPRB, 0.17303E+00_JPRB, 0.17353E+00_JPRB ,&
&  0.17422E+00_JPRB, 0.17528E+00_JPRB, 0.17595E+00_JPRB, 0.17693E+00_JPRB, 0.17814E+00_JPRB, 0.17951E+00_JPRB, 0.18115E+00_JPRB ,&
&  0.18319E+00_JPRB, 0.18509E+00_JPRB, 0.18646E+00_JPRB, 0.18835E+00_JPRB, 0.19186E+00_JPRB, 0.19642E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.21053E+00_JPRB, 0.21566E+00_JPRB, 0.21154E+00_JPRB, 0.21496E+00_JPRB, 0.21756E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.21874E+00_JPRB, 0.22064E+00_JPRB, 0.22223E+00_JPRB, 0.22388E+00_JPRB, 0.22520E+00_JPRB, 0.22641E+00_JPRB, 0.22718E+00_JPRB ,&
&  0.22424E+00_JPRB, 0.22269E+00_JPRB, 0.21978E+00_JPRB, 0.21739E+00_JPRB, 0.21442E+00_JPRB, 0.21106E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.20823E+00_JPRB, 0.20791E+00_JPRB, 0.20632E+00_JPRB, 0.20563E+00_JPRB, 0.20427E+00_JPRB, 0.20337E+00_JPRB, 0.20182E+00_JPRB ,&
&  0.19909E+00_JPRB, 0.19777E+00_JPRB, 0.19653E+00_JPRB, 0.19683E+00_JPRB, 0.19846E+00_JPRB, 0.20014E+00_JPRB, 0.20195E+00_JPRB ,&
&  0.20461E+00_JPRB, 0.20560E+00_JPRB, 0.20815E+00_JPRB, 0.21010E+00_JPRB, 0.21368E+00_JPRB, 0.21669E+00_JPRB, 0.21739E+00_JPRB ,&
&  0.21729E+00_JPRB, 0.21471E+00_JPRB, 0.21368E+00_JPRB, 0.21086E+00_JPRB, 0.20417E+00_JPRB, 0.20069E+00_JPRB, 0.19552E+00_JPRB ,&
&  0.18981E+00_JPRB, 0.18556E+00_JPRB, 0.18259E+00_JPRB, 0.18176E+00_JPRB, 0.18078E+00_JPRB, 0.18060E+00_JPRB, 0.18054E+00_JPRB ,&
&  0.17975E+00_JPRB, 0.17880E+00_JPRB, 0.17766E+00_JPRB, 0.17709E+00_JPRB, 0.17767E+00_JPRB, 0.17869E+00_JPRB, 0.18080E+00_JPRB ,&
&  0.18285E+00_JPRB, 0.18405E+00_JPRB, 0.18532E+00_JPRB, 0.18576E+00_JPRB, 0.18591E+00_JPRB, 0.18620E+00_JPRB, 0.18628E+00_JPRB ,&
&  0.18711E+00_JPRB, 0.18731E+00_JPRB, 0.18672E+00_JPRB, 0.18573E+00_JPRB, 0.18311E+00_JPRB, 0.17964E+00_JPRB, 0.17697E+00_JPRB ,&
&  0.17466E+00_JPRB, 0.17314E+00_JPRB, 0.17260E+00_JPRB, 0.17201E+00_JPRB, 0.17227E+00_JPRB, 0.17266E+00_JPRB, 0.17300E+00_JPRB ,&
&  0.17360E+00_JPRB, 0.17415E+00_JPRB, 0.17469E+00_JPRB, 0.17607E+00_JPRB, 0.17720E+00_JPRB, 0.17893E+00_JPRB, 0.18073E+00_JPRB ,&
&  0.18198E+00_JPRB, 0.18386E+00_JPRB, 0.18548E+00_JPRB, 0.18724E+00_JPRB, 0.19261E+00_JPRB, 0.19674E+00_JPRB, 0.20517E+00_JPRB ,&
&  0.21020E+00_JPRB, 0.21091E+00_JPRB, 0.20997E+00_JPRB, 0.21665E+00_JPRB, 0.21770E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.21880E+00_JPRB, 0.21965E+00_JPRB, 0.21969E+00_JPRB, 0.22077E+00_JPRB, 0.22147E+00_JPRB, 0.22223E+00_JPRB, 0.22110E+00_JPRB ,&
&  0.22018E+00_JPRB, 0.21883E+00_JPRB, 0.21774E+00_JPRB, 0.21443E+00_JPRB, 0.21185E+00_JPRB, 0.20863E+00_JPRB, 0.20585E+00_JPRB ,&
&  0.20336E+00_JPRB, 0.20097E+00_JPRB, 0.19872E+00_JPRB, 0.19677E+00_JPRB, 0.19492E+00_JPRB, 0.19325E+00_JPRB, 0.19158E+00_JPRB ,&
&  0.18911E+00_JPRB, 0.18844E+00_JPRB, 0.18842E+00_JPRB, 0.18983E+00_JPRB, 0.19278E+00_JPRB, 0.19457E+00_JPRB, 0.19673E+00_JPRB ,&
&  0.19889E+00_JPRB, 0.19781E+00_JPRB, 0.20130E+00_JPRB, 0.20463E+00_JPRB, 0.20739E+00_JPRB, 0.20975E+00_JPRB, 0.21169E+00_JPRB ,&
&  0.21225E+00_JPRB, 0.20595E+00_JPRB, 0.20686E+00_JPRB, 0.20155E+00_JPRB, 0.19636E+00_JPRB, 0.19437E+00_JPRB, 0.19018E+00_JPRB ,&
&  0.18866E+00_JPRB, 0.18536E+00_JPRB, 0.18325E+00_JPRB, 0.18137E+00_JPRB, 0.17902E+00_JPRB, 0.17855E+00_JPRB, 0.17769E+00_JPRB ,&
&  0.17728E+00_JPRB, 0.17751E+00_JPRB, 0.17671E+00_JPRB, 0.17654E+00_JPRB, 0.17613E+00_JPRB, 0.17579E+00_JPRB, 0.17643E+00_JPRB ,&
&  0.17733E+00_JPRB, 0.17833E+00_JPRB, 0.17925E+00_JPRB, 0.17939E+00_JPRB, 0.17929E+00_JPRB, 0.17927E+00_JPRB, 0.17918E+00_JPRB ,&
&  0.17953E+00_JPRB, 0.17959E+00_JPRB, 0.17914E+00_JPRB, 0.17863E+00_JPRB, 0.17739E+00_JPRB, 0.17613E+00_JPRB, 0.17488E+00_JPRB ,&
&  0.17375E+00_JPRB, 0.17314E+00_JPRB, 0.17261E+00_JPRB, 0.17243E+00_JPRB, 0.17241E+00_JPRB, 0.17275E+00_JPRB, 0.17310E+00_JPRB ,&
&  0.17343E+00_JPRB, 0.17428E+00_JPRB, 0.17489E+00_JPRB, 0.17592E+00_JPRB, 0.17716E+00_JPRB, 0.17827E+00_JPRB, 0.17997E+00_JPRB ,&
&  0.18181E+00_JPRB, 0.18381E+00_JPRB, 0.18671E+00_JPRB, 0.18918E+00_JPRB, 0.19234E+00_JPRB, 0.19633E+00_JPRB, 0.19918E+00_JPRB ,&
&  0.20220E+00_JPRB, 0.20479E+00_JPRB, 0.20451E+00_JPRB, 0.21265E+00_JPRB, 0.21716E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.20975E+00_JPRB, 0.21070E+00_JPRB, 0.21117E+00_JPRB, 0.21158E+00_JPRB, 0.21276E+00_JPRB, 0.21272E+00_JPRB, 0.21246E+00_JPRB ,&
&  0.21006E+00_JPRB, 0.20922E+00_JPRB, 0.20779E+00_JPRB, 0.20574E+00_JPRB, 0.20361E+00_JPRB, 0.19996E+00_JPRB, 0.19780E+00_JPRB ,&
&  0.19574E+00_JPRB, 0.19354E+00_JPRB, 0.19216E+00_JPRB, 0.19027E+00_JPRB, 0.18864E+00_JPRB, 0.18674E+00_JPRB, 0.18613E+00_JPRB ,&
&  0.18515E+00_JPRB, 0.18506E+00_JPRB, 0.18596E+00_JPRB, 0.18661E+00_JPRB, 0.18791E+00_JPRB, 0.18926E+00_JPRB, 0.19103E+00_JPRB ,&
&  0.19129E+00_JPRB, 0.19202E+00_JPRB, 0.19279E+00_JPRB, 0.19334E+00_JPRB, 0.19542E+00_JPRB, 0.19688E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19692E+00_JPRB, 0.19589E+00_JPRB, 0.19390E+00_JPRB, 0.19160E+00_JPRB, 0.18906E+00_JPRB, 0.18635E+00_JPRB, 0.18375E+00_JPRB ,&
&  0.18179E+00_JPRB, 0.17996E+00_JPRB, 0.17857E+00_JPRB, 0.17745E+00_JPRB, 0.17680E+00_JPRB, 0.17630E+00_JPRB, 0.17578E+00_JPRB ,&
&  0.17554E+00_JPRB, 0.17535E+00_JPRB, 0.17502E+00_JPRB, 0.17511E+00_JPRB, 0.17507E+00_JPRB, 0.17521E+00_JPRB, 0.17537E+00_JPRB ,&
&  0.17566E+00_JPRB, 0.17575E+00_JPRB, 0.17573E+00_JPRB, 0.17544E+00_JPRB, 0.17515E+00_JPRB, 0.17481E+00_JPRB, 0.17447E+00_JPRB ,&
&  0.17411E+00_JPRB, 0.17412E+00_JPRB, 0.17382E+00_JPRB, 0.17361E+00_JPRB, 0.17333E+00_JPRB, 0.17279E+00_JPRB, 0.17263E+00_JPRB ,&
&  0.17270E+00_JPRB, 0.17284E+00_JPRB, 0.17263E+00_JPRB, 0.17271E+00_JPRB, 0.17290E+00_JPRB, 0.17320E+00_JPRB, 0.17412E+00_JPRB ,&
&  0.17468E+00_JPRB, 0.17525E+00_JPRB, 0.17577E+00_JPRB, 0.17654E+00_JPRB, 0.17788E+00_JPRB, 0.17923E+00_JPRB, 0.18074E+00_JPRB ,&
&  0.18256E+00_JPRB, 0.18425E+00_JPRB, 0.18594E+00_JPRB, 0.18761E+00_JPRB, 0.19041E+00_JPRB, 0.19233E+00_JPRB, 0.19486E+00_JPRB ,&
&  0.19702E+00_JPRB, 0.19529E+00_JPRB, 0.20422E+00_JPRB, 0.20483E+00_JPRB, 0.20607E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.19265E+00_JPRB, 0.19256E+00_JPRB, 0.19143E+00_JPRB, 0.19018E+00_JPRB, 0.18225E+00_JPRB, 0.18893E+00_JPRB, 0.18829E+00_JPRB ,&
&  0.18901E+00_JPRB, 0.18960E+00_JPRB, 0.19135E+00_JPRB, 0.19174E+00_JPRB, 0.19157E+00_JPRB, 0.19156E+00_JPRB, 0.19138E+00_JPRB ,&
&  0.19111E+00_JPRB, 0.19031E+00_JPRB, 0.18995E+00_JPRB, 0.18936E+00_JPRB, 0.18873E+00_JPRB, 0.18764E+00_JPRB, 0.18646E+00_JPRB ,&
&  0.18592E+00_JPRB, 0.18598E+00_JPRB, 0.18639E+00_JPRB, 0.18438E+00_JPRB, 0.18343E+00_JPRB, 0.18280E+00_JPRB, 0.18246E+00_JPRB ,&
&  0.18192E+00_JPRB, 0.18174E+00_JPRB, 0.18173E+00_JPRB, 0.18166E+00_JPRB, 0.18321E+00_JPRB, 0.18370E+00_JPRB, 0.18408E+00_JPRB ,&
&  0.18417E+00_JPRB, 0.18404E+00_JPRB, 0.18332E+00_JPRB, 0.18264E+00_JPRB, 0.18195E+00_JPRB, 0.18071E+00_JPRB, 0.17943E+00_JPRB ,&
&  0.17834E+00_JPRB, 0.17752E+00_JPRB, 0.17656E+00_JPRB, 0.17584E+00_JPRB, 0.17526E+00_JPRB, 0.17471E+00_JPRB, 0.17436E+00_JPRB ,&
&  0.17429E+00_JPRB, 0.17432E+00_JPRB, 0.17435E+00_JPRB, 0.17436E+00_JPRB, 0.17435E+00_JPRB, 0.17436E+00_JPRB, 0.17441E+00_JPRB ,&
&  0.17486E+00_JPRB, 0.17489E+00_JPRB, 0.17473E+00_JPRB, 0.17443E+00_JPRB, 0.17435E+00_JPRB, 0.17402E+00_JPRB, 0.17381E+00_JPRB ,&
&  0.17363E+00_JPRB, 0.17341E+00_JPRB, 0.17331E+00_JPRB, 0.17330E+00_JPRB, 0.17332E+00_JPRB, 0.17332E+00_JPRB, 0.17343E+00_JPRB ,&
&  0.17361E+00_JPRB, 0.17383E+00_JPRB, 0.17425E+00_JPRB, 0.17456E+00_JPRB, 0.17485E+00_JPRB, 0.17510E+00_JPRB, 0.17565E+00_JPRB ,&
&  0.17623E+00_JPRB, 0.17679E+00_JPRB, 0.17735E+00_JPRB, 0.17771E+00_JPRB, 0.17854E+00_JPRB, 0.17938E+00_JPRB, 0.18030E+00_JPRB ,&
&  0.18130E+00_JPRB, 0.18255E+00_JPRB, 0.18386E+00_JPRB, 0.18518E+00_JPRB, 0.18695E+00_JPRB, 0.18835E+00_JPRB, 0.18973E+00_JPRB ,&
&  0.19127E+00_JPRB, 0.19414E+00_JPRB, 0.19481E+00_JPRB, 0.19482E+00_JPRB, 0.19403E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.18771E+00_JPRB, 0.18765E+00_JPRB, 0.18746E+00_JPRB, 0.18698E+00_JPRB, 0.18610E+00_JPRB, 0.18589E+00_JPRB, 0.18561E+00_JPRB ,&
&  0.18523E+00_JPRB, 0.18444E+00_JPRB, 0.18444E+00_JPRB, 0.18434E+00_JPRB, 0.18422E+00_JPRB, 0.18408E+00_JPRB, 0.18378E+00_JPRB ,&
&  0.18318E+00_JPRB, 0.18227E+00_JPRB, 0.17961E+00_JPRB, 0.17833E+00_JPRB, 0.17749E+00_JPRB, 0.17702E+00_JPRB, 0.17663E+00_JPRB ,&
&  0.17639E+00_JPRB, 0.17626E+00_JPRB, 0.17612E+00_JPRB, 0.17660E+00_JPRB, 0.17681E+00_JPRB, 0.17688E+00_JPRB, 0.17693E+00_JPRB ,&
&  0.17692E+00_JPRB, 0.17702E+00_JPRB, 0.17703E+00_JPRB, 0.17701E+00_JPRB, 0.17715E+00_JPRB, 0.17715E+00_JPRB, 0.17695E+00_JPRB ,&
&  0.17671E+00_JPRB, 0.17642E+00_JPRB, 0.17615E+00_JPRB, 0.17581E+00_JPRB, 0.17556E+00_JPRB, 0.17536E+00_JPRB, 0.17529E+00_JPRB ,&
&  0.17511E+00_JPRB, 0.17498E+00_JPRB, 0.17484E+00_JPRB, 0.17479E+00_JPRB, 0.17468E+00_JPRB, 0.17464E+00_JPRB, 0.17450E+00_JPRB ,&
&  0.17460E+00_JPRB, 0.17464E+00_JPRB, 0.17473E+00_JPRB, 0.17478E+00_JPRB, 0.17483E+00_JPRB, 0.17482E+00_JPRB, 0.17476E+00_JPRB ,&
&  0.17460E+00_JPRB, 0.17450E+00_JPRB, 0.17438E+00_JPRB, 0.17430E+00_JPRB, 0.17428E+00_JPRB, 0.17427E+00_JPRB, 0.17429E+00_JPRB ,&
&  0.17431E+00_JPRB, 0.17443E+00_JPRB, 0.17448E+00_JPRB, 0.17455E+00_JPRB, 0.17463E+00_JPRB, 0.17472E+00_JPRB, 0.17478E+00_JPRB ,&
&  0.17488E+00_JPRB, 0.17498E+00_JPRB, 0.17510E+00_JPRB, 0.17526E+00_JPRB, 0.17545E+00_JPRB, 0.17573E+00_JPRB, 0.17605E+00_JPRB ,&
&  0.17638E+00_JPRB, 0.17672E+00_JPRB, 0.17706E+00_JPRB, 0.17738E+00_JPRB, 0.17791E+00_JPRB, 0.17844E+00_JPRB, 0.17912E+00_JPRB ,&
&  0.17975E+00_JPRB, 0.18037E+00_JPRB, 0.18094E+00_JPRB, 0.18156E+00_JPRB, 0.18245E+00_JPRB, 0.18287E+00_JPRB, 0.18323E+00_JPRB ,&
&  0.18381E+00_JPRB, 0.18442E+00_JPRB, 0.18500E+00_JPRB, 0.18551E+00_JPRB, 0.18632E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB, 0.17660E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB ,&
&  0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB, 0.64408E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.64800E+00_JPRB, 0.64623E+00_JPRB, 0.64470E+00_JPRB, 0.64295E+00_JPRB, 0.64103E+00_JPRB, 0.63948E+00_JPRB, 0.63803E+00_JPRB ,&
&  0.63678E+00_JPRB, 0.63609E+00_JPRB, 0.63572E+00_JPRB, 0.63551E+00_JPRB, 0.63553E+00_JPRB, 0.63549E+00_JPRB, 0.63552E+00_JPRB ,&
&  0.63545E+00_JPRB, 0.63551E+00_JPRB, 0.63552E+00_JPRB, 0.63520E+00_JPRB, 0.63463E+00_JPRB, 0.63391E+00_JPRB, 0.63296E+00_JPRB ,&
&  0.63158E+00_JPRB, 0.63015E+00_JPRB, 0.62882E+00_JPRB, 0.62738E+00_JPRB, 0.62609E+00_JPRB, 0.62465E+00_JPRB, 0.62311E+00_JPRB ,&
&  0.62176E+00_JPRB, 0.62100E+00_JPRB, 0.62042E+00_JPRB, 0.61982E+00_JPRB, 0.61959E+00_JPRB, 0.62043E+00_JPRB, 0.62117E+00_JPRB ,&
&  0.62199E+00_JPRB, 0.62301E+00_JPRB, 0.62496E+00_JPRB, 0.62689E+00_JPRB, 0.62871E+00_JPRB, 0.63048E+00_JPRB, 0.63207E+00_JPRB ,&
&  0.63270E+00_JPRB, 0.63284E+00_JPRB, 0.63268E+00_JPRB, 0.63250E+00_JPRB, 0.63183E+00_JPRB, 0.63135E+00_JPRB, 0.63096E+00_JPRB ,&
&  0.63043E+00_JPRB, 0.62997E+00_JPRB, 0.62943E+00_JPRB, 0.62832E+00_JPRB, 0.62576E+00_JPRB, 0.62391E+00_JPRB, 0.62304E+00_JPRB ,&
&  0.62298E+00_JPRB, 0.62305E+00_JPRB, 0.62352E+00_JPRB, 0.62434E+00_JPRB, 0.62579E+00_JPRB, 0.62740E+00_JPRB, 0.62932E+00_JPRB ,&
&  0.63144E+00_JPRB, 0.63406E+00_JPRB, 0.63750E+00_JPRB, 0.64111E+00_JPRB, 0.64510E+00_JPRB, 0.64974E+00_JPRB, 0.65397E+00_JPRB ,&
&  0.65830E+00_JPRB, 0.66285E+00_JPRB, 0.66675E+00_JPRB, 0.67054E+00_JPRB, 0.67432E+00_JPRB, 0.67824E+00_JPRB, 0.68126E+00_JPRB ,&
&  0.68282E+00_JPRB, 0.68400E+00_JPRB, 0.68489E+00_JPRB, 0.68452E+00_JPRB, 0.68324E+00_JPRB, 0.68224E+00_JPRB, 0.68116E+00_JPRB ,&
&  0.67975E+00_JPRB, 0.67786E+00_JPRB, 0.67608E+00_JPRB, 0.67406E+00_JPRB, 0.67122E+00_JPRB, 0.66808E+00_JPRB, 0.66510E+00_JPRB ,&
&  0.66214E+00_JPRB, 0.65890E+00_JPRB, 0.65593E+00_JPRB, 0.65312E+00_JPRB, 0.65050E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.66940E+00_JPRB, 0.66230E+00_JPRB, 0.65591E+00_JPRB, 0.65010E+00_JPRB, 0.64541E+00_JPRB, 0.64185E+00_JPRB, 0.63891E+00_JPRB ,&
&  0.63644E+00_JPRB, 0.63529E+00_JPRB, 0.63407E+00_JPRB, 0.63291E+00_JPRB, 0.63173E+00_JPRB, 0.62994E+00_JPRB, 0.62773E+00_JPRB ,&
&  0.62570E+00_JPRB, 0.62434E+00_JPRB, 0.62295E+00_JPRB, 0.62125E+00_JPRB, 0.61847E+00_JPRB, 0.61442E+00_JPRB, 0.60972E+00_JPRB ,&
&  0.60446E+00_JPRB, 0.59809E+00_JPRB, 0.59079E+00_JPRB, 0.58325E+00_JPRB, 0.57939E+00_JPRB, 0.57488E+00_JPRB, 0.57058E+00_JPRB ,&
&  0.56690E+00_JPRB, 0.56589E+00_JPRB, 0.56512E+00_JPRB, 0.56447E+00_JPRB, 0.56439E+00_JPRB, 0.56609E+00_JPRB, 0.56718E+00_JPRB ,&
&  0.56827E+00_JPRB, 0.57072E+00_JPRB, 0.57664E+00_JPRB, 0.58303E+00_JPRB, 0.59041E+00_JPRB, 0.59930E+00_JPRB, 0.60949E+00_JPRB ,&
&  0.61704E+00_JPRB, 0.62115E+00_JPRB, 0.62462E+00_JPRB, 0.62779E+00_JPRB, 0.62942E+00_JPRB, 0.63017E+00_JPRB, 0.63018E+00_JPRB ,&
&  0.63149E+00_JPRB, 0.63218E+00_JPRB, 0.63291E+00_JPRB, 0.63349E+00_JPRB, 0.63215E+00_JPRB, 0.63087E+00_JPRB, 0.62951E+00_JPRB ,&
&  0.62734E+00_JPRB, 0.62651E+00_JPRB, 0.62715E+00_JPRB, 0.62797E+00_JPRB, 0.63026E+00_JPRB, 0.63408E+00_JPRB, 0.63917E+00_JPRB ,&
&  0.64650E+00_JPRB, 0.65671E+00_JPRB, 0.66932E+00_JPRB, 0.68338E+00_JPRB, 0.69955E+00_JPRB, 0.71974E+00_JPRB, 0.73593E+00_JPRB ,&
&  0.75062E+00_JPRB, 0.76323E+00_JPRB, 0.77466E+00_JPRB, 0.78304E+00_JPRB, 0.78929E+00_JPRB, 0.79538E+00_JPRB, 0.79588E+00_JPRB ,&
&  0.79507E+00_JPRB, 0.79354E+00_JPRB, 0.79242E+00_JPRB, 0.78853E+00_JPRB, 0.78362E+00_JPRB, 0.77848E+00_JPRB, 0.77364E+00_JPRB ,&
&  0.76623E+00_JPRB, 0.75859E+00_JPRB, 0.75104E+00_JPRB, 0.74367E+00_JPRB, 0.73528E+00_JPRB, 0.72712E+00_JPRB, 0.71906E+00_JPRB ,&
&  0.71093E+00_JPRB, 0.70222E+00_JPRB, 0.69388E+00_JPRB, 0.68578E+00_JPRB, 0.67768E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.69409E+00_JPRB, 0.68188E+00_JPRB, 0.67080E+00_JPRB, 0.66108E+00_JPRB, 0.65270E+00_JPRB, 0.64623E+00_JPRB, 0.64118E+00_JPRB ,&
&  0.63729E+00_JPRB, 0.63530E+00_JPRB, 0.63325E+00_JPRB, 0.63075E+00_JPRB, 0.62799E+00_JPRB, 0.62733E+00_JPRB, 0.62506E+00_JPRB ,&
&  0.62163E+00_JPRB, 0.61714E+00_JPRB, 0.61261E+00_JPRB, 0.60661E+00_JPRB, 0.59843E+00_JPRB, 0.58904E+00_JPRB, 0.57759E+00_JPRB ,&
&  0.57004E+00_JPRB, 0.56001E+00_JPRB, 0.54916E+00_JPRB, 0.53759E+00_JPRB, 0.52948E+00_JPRB, 0.51134E+00_JPRB, 0.50457E+00_JPRB ,&
&  0.49997E+00_JPRB, 0.48941E+00_JPRB, 0.48440E+00_JPRB, 0.48309E+00_JPRB, 0.49755E+00_JPRB, 0.51182E+00_JPRB, 0.51913E+00_JPRB ,&
&  0.50766E+00_JPRB, 0.51170E+00_JPRB, 0.50899E+00_JPRB, 0.50696E+00_JPRB, 0.50881E+00_JPRB, 0.51447E+00_JPRB, 0.53757E+00_JPRB ,&
&  0.57884E+00_JPRB, 0.59481E+00_JPRB, 0.60349E+00_JPRB, 0.61119E+00_JPRB, 0.61518E+00_JPRB, 0.61907E+00_JPRB, 0.62089E+00_JPRB ,&
&  0.62331E+00_JPRB, 0.62503E+00_JPRB, 0.62883E+00_JPRB, 0.63098E+00_JPRB, 0.63438E+00_JPRB, 0.63781E+00_JPRB, 0.64104E+00_JPRB ,&
&  0.63841E+00_JPRB, 0.63709E+00_JPRB, 0.63713E+00_JPRB, 0.63961E+00_JPRB, 0.64432E+00_JPRB, 0.65462E+00_JPRB, 0.66753E+00_JPRB ,&
&  0.68188E+00_JPRB, 0.69359E+00_JPRB, 0.71231E+00_JPRB, 0.73723E+00_JPRB, 0.76701E+00_JPRB, 0.80355E+00_JPRB, 0.84035E+00_JPRB ,&
&  0.87689E+00_JPRB, 0.90728E+00_JPRB, 0.93690E+00_JPRB, 0.95248E+00_JPRB, 0.96154E+00_JPRB, 0.96794E+00_JPRB, 0.96865E+00_JPRB ,&
&  0.96754E+00_JPRB, 0.96273E+00_JPRB, 0.95742E+00_JPRB, 0.94961E+00_JPRB, 0.93884E+00_JPRB, 0.92427E+00_JPRB, 0.90764E+00_JPRB ,&
&  0.88792E+00_JPRB, 0.86857E+00_JPRB, 0.84943E+00_JPRB, 0.83063E+00_JPRB, 0.81190E+00_JPRB, 0.79463E+00_JPRB, 0.77918E+00_JPRB ,&
&  0.76448E+00_JPRB, 0.74948E+00_JPRB, 0.73545E+00_JPRB, 0.72180E+00_JPRB, 0.70805E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.71821E+00_JPRB, 0.70468E+00_JPRB, 0.69030E+00_JPRB, 0.67726E+00_JPRB, 0.66632E+00_JPRB, 0.65799E+00_JPRB, 0.65217E+00_JPRB ,&
&  0.64745E+00_JPRB, 0.64186E+00_JPRB, 0.63607E+00_JPRB, 0.62993E+00_JPRB, 0.62428E+00_JPRB, 0.62201E+00_JPRB, 0.61999E+00_JPRB ,&
&  0.61823E+00_JPRB, 0.61284E+00_JPRB, 0.60540E+00_JPRB, 0.59465E+00_JPRB, 0.58332E+00_JPRB, 0.57342E+00_JPRB, 0.56235E+00_JPRB ,&
&  0.55197E+00_JPRB, 0.54082E+00_JPRB, 0.52296E+00_JPRB, 0.51140E+00_JPRB, 0.49066E+00_JPRB, 0.47738E+00_JPRB, 0.46956E+00_JPRB ,&
&  0.46452E+00_JPRB, 0.46617E+00_JPRB, 0.47083E+00_JPRB, 0.47689E+00_JPRB, 0.47687E+00_JPRB, 0.47261E+00_JPRB, 0.47085E+00_JPRB ,&
&  0.46451E+00_JPRB, 0.46029E+00_JPRB, 0.45732E+00_JPRB, 0.45663E+00_JPRB, 0.45900E+00_JPRB, 0.46608E+00_JPRB, 0.48092E+00_JPRB ,&
&  0.51437E+00_JPRB, 0.55760E+00_JPRB, 0.59758E+00_JPRB, 0.61636E+00_JPRB, 0.61235E+00_JPRB, 0.61617E+00_JPRB, 0.61422E+00_JPRB ,&
&  0.61454E+00_JPRB, 0.61311E+00_JPRB, 0.61533E+00_JPRB, 0.62168E+00_JPRB, 0.62944E+00_JPRB, 0.63679E+00_JPRB, 0.64478E+00_JPRB ,&
&  0.64801E+00_JPRB, 0.65160E+00_JPRB, 0.64878E+00_JPRB, 0.65268E+00_JPRB, 0.66371E+00_JPRB, 0.67771E+00_JPRB, 0.68512E+00_JPRB ,&
&  0.69734E+00_JPRB, 0.71392E+00_JPRB, 0.74451E+00_JPRB, 0.79024E+00_JPRB, 0.83924E+00_JPRB, 0.88449E+00_JPRB, 0.92706E+00_JPRB ,&
&  0.96831E+00_JPRB, 0.10190E+01_JPRB, 0.10684E+01_JPRB, 0.10895E+01_JPRB, 0.10907E+01_JPRB, 0.10969E+01_JPRB, 0.10900E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10764E+01_JPRB, 0.10725E+01_JPRB, 0.10685E+01_JPRB, 0.10538E+01_JPRB, 0.10300E+01_JPRB, 0.10046E+01_JPRB ,&
&  0.97694E+00_JPRB, 0.94652E+00_JPRB, 0.91759E+00_JPRB, 0.88815E+00_JPRB, 0.86048E+00_JPRB, 0.83347E+00_JPRB, 0.81076E+00_JPRB ,&
&  0.78883E+00_JPRB, 0.77173E+00_JPRB, 0.75591E+00_JPRB, 0.74301E+00_JPRB, 0.73079E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.74101E+00_JPRB, 0.73216E+00_JPRB, 0.72311E+00_JPRB, 0.71116E+00_JPRB, 0.69895E+00_JPRB, 0.68769E+00_JPRB, 0.67885E+00_JPRB ,&
&  0.67151E+00_JPRB, 0.66379E+00_JPRB, 0.65635E+00_JPRB, 0.64738E+00_JPRB, 0.63892E+00_JPRB, 0.63133E+00_JPRB, 0.62499E+00_JPRB ,&
&  0.62001E+00_JPRB, 0.61128E+00_JPRB, 0.60255E+00_JPRB, 0.59174E+00_JPRB, 0.58165E+00_JPRB, 0.57039E+00_JPRB, 0.55807E+00_JPRB ,&
&  0.54829E+00_JPRB, 0.53251E+00_JPRB, 0.51628E+00_JPRB, 0.50632E+00_JPRB, 0.48682E+00_JPRB, 0.47777E+00_JPRB, 0.47442E+00_JPRB ,&
&  0.49044E+00_JPRB, 0.48543E+00_JPRB, 0.49375E+00_JPRB, 0.47929E+00_JPRB, 0.46752E+00_JPRB, 0.45660E+00_JPRB, 0.44710E+00_JPRB ,&
&  0.44034E+00_JPRB, 0.43405E+00_JPRB, 0.43400E+00_JPRB, 0.43725E+00_JPRB, 0.43961E+00_JPRB, 0.44905E+00_JPRB, 0.46314E+00_JPRB ,&
&  0.49883E+00_JPRB, 0.59141E+00_JPRB, 0.60658E+00_JPRB, 0.62406E+00_JPRB, 0.62366E+00_JPRB, 0.62571E+00_JPRB, 0.62729E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.62767E+00_JPRB, 0.62542E+00_JPRB, 0.62446E+00_JPRB, 0.63630E+00_JPRB, 0.64549E+00_JPRB, 0.65730E+00_JPRB ,&
&  0.66233E+00_JPRB, 0.66877E+00_JPRB, 0.67058E+00_JPRB, 0.67620E+00_JPRB, 0.69253E+00_JPRB, 0.70227E+00_JPRB, 0.70422E+00_JPRB ,&
&  0.72243E+00_JPRB, 0.74869E+00_JPRB, 0.78935E+00_JPRB, 0.83902E+00_JPRB, 0.87623E+00_JPRB, 0.90775E+00_JPRB, 0.95556E+00_JPRB ,&
&  0.98931E+00_JPRB, 0.10453E+01_JPRB, 0.11057E+01_JPRB, 0.11338E+01_JPRB, 0.11397E+01_JPRB, 0.11339E+01_JPRB, 0.11249E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.11076E+01_JPRB, 0.10991E+01_JPRB, 0.10947E+01_JPRB, 0.10762E+01_JPRB, 0.10523E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.99565E+00_JPRB, 0.96783E+00_JPRB, 0.94447E+00_JPRB, 0.91767E+00_JPRB, 0.88976E+00_JPRB, 0.86249E+00_JPRB, 0.83599E+00_JPRB ,&
&  0.81058E+00_JPRB, 0.78999E+00_JPRB, 0.77088E+00_JPRB, 0.75867E+00_JPRB, 0.74775E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.75998E+00_JPRB, 0.75157E+00_JPRB, 0.74373E+00_JPRB, 0.73601E+00_JPRB, 0.72716E+00_JPRB, 0.71794E+00_JPRB, 0.70844E+00_JPRB ,&
&  0.70009E+00_JPRB, 0.69234E+00_JPRB, 0.68420E+00_JPRB, 0.67375E+00_JPRB, 0.66293E+00_JPRB, 0.65091E+00_JPRB, 0.64138E+00_JPRB ,&
&  0.63140E+00_JPRB, 0.62020E+00_JPRB, 0.60402E+00_JPRB, 0.58916E+00_JPRB, 0.57872E+00_JPRB, 0.56959E+00_JPRB, 0.55824E+00_JPRB ,&
&  0.54840E+00_JPRB, 0.53479E+00_JPRB, 0.51520E+00_JPRB, 0.49774E+00_JPRB, 0.47997E+00_JPRB, 0.47772E+00_JPRB, 0.50688E+00_JPRB ,&
&  0.52136E+00_JPRB, 0.52314E+00_JPRB, 0.52255E+00_JPRB, 0.51589E+00_JPRB, 0.50833E+00_JPRB, 0.46130E+00_JPRB, 0.46175E+00_JPRB ,&
&  0.44549E+00_JPRB, 0.43276E+00_JPRB, 0.43037E+00_JPRB, 0.43067E+00_JPRB, 0.43593E+00_JPRB, 0.44922E+00_JPRB, 0.46652E+00_JPRB ,&
&  0.50099E+00_JPRB, 0.60499E+00_JPRB, 0.62237E+00_JPRB, 0.63895E+00_JPRB, 0.64194E+00_JPRB, 0.64425E+00_JPRB, 0.64853E+00_JPRB ,&
&  0.64976E+00_JPRB, 0.65351E+00_JPRB, 0.65734E+00_JPRB, 0.66364E+00_JPRB, 0.67123E+00_JPRB, 0.67334E+00_JPRB, 0.68527E+00_JPRB ,&
&  0.69422E+00_JPRB, 0.70287E+00_JPRB, 0.70743E+00_JPRB, 0.70571E+00_JPRB, 0.71443E+00_JPRB, 0.71136E+00_JPRB, 0.71134E+00_JPRB ,&
&  0.74634E+00_JPRB, 0.79865E+00_JPRB, 0.84524E+00_JPRB, 0.88860E+00_JPRB, 0.92671E+00_JPRB, 0.96929E+00_JPRB, 0.10216E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.11108E+01_JPRB, 0.11744E+01_JPRB, 0.11962E+01_JPRB, 0.11961E+01_JPRB, 0.11936E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.12030E+01_JPRB, 0.12108E+01_JPRB, 0.11777E+01_JPRB, 0.11580E+01_JPRB, 0.11158E+01_JPRB, 0.10606E+01_JPRB, 0.10154E+01_JPRB ,&
&  0.95960E+00_JPRB, 0.91166E+00_JPRB, 0.88250E+00_JPRB, 0.86163E+00_JPRB, 0.84607E+00_JPRB, 0.83532E+00_JPRB, 0.82555E+00_JPRB ,&
&  0.81647E+00_JPRB, 0.80473E+00_JPRB, 0.79208E+00_JPRB, 0.77948E+00_JPRB, 0.76870E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.76655E+00_JPRB, 0.76097E+00_JPRB, 0.75295E+00_JPRB, 0.74580E+00_JPRB, 0.73615E+00_JPRB, 0.72879E+00_JPRB, 0.72160E+00_JPRB ,&
&  0.71679E+00_JPRB, 0.71380E+00_JPRB, 0.71037E+00_JPRB, 0.70662E+00_JPRB, 0.70055E+00_JPRB, 0.69354E+00_JPRB, 0.68381E+00_JPRB ,&
&  0.66911E+00_JPRB, 0.63262E+00_JPRB, 0.60583E+00_JPRB, 0.59383E+00_JPRB, 0.58252E+00_JPRB, 0.57350E+00_JPRB, 0.56537E+00_JPRB ,&
&  0.55219E+00_JPRB, 0.53961E+00_JPRB, 0.52371E+00_JPRB, 0.50118E+00_JPRB, 0.49056E+00_JPRB, 0.49750E+00_JPRB, 0.50471E+00_JPRB ,&
&  0.51614E+00_JPRB, 0.52898E+00_JPRB, 0.52494E+00_JPRB, 0.52023E+00_JPRB, 0.52965E+00_JPRB, 0.52685E+00_JPRB, 0.48145E+00_JPRB ,&
&  0.45426E+00_JPRB, 0.43511E+00_JPRB, 0.43028E+00_JPRB, 0.42956E+00_JPRB, 0.43552E+00_JPRB, 0.45162E+00_JPRB, 0.49337E+00_JPRB ,&
&  0.56998E+00_JPRB, 0.65779E+00_JPRB, 0.65181E+00_JPRB, 0.65110E+00_JPRB, 0.65756E+00_JPRB, 0.66277E+00_JPRB, 0.66877E+00_JPRB ,&
&  0.67470E+00_JPRB, 0.68557E+00_JPRB, 0.70006E+00_JPRB, 0.71404E+00_JPRB, 0.73158E+00_JPRB, 0.74539E+00_JPRB, 0.75400E+00_JPRB ,&
&  0.77041E+00_JPRB, 0.75596E+00_JPRB, 0.75197E+00_JPRB, 0.74276E+00_JPRB, 0.73843E+00_JPRB, 0.72509E+00_JPRB, 0.74028E+00_JPRB ,&
&  0.78426E+00_JPRB, 0.83228E+00_JPRB, 0.88388E+00_JPRB, 0.92585E+00_JPRB, 0.97057E+00_JPRB, 0.10309E+01_JPRB, 0.10974E+01_JPRB ,&
&  0.11568E+01_JPRB, 0.12366E+01_JPRB, 0.13511E+01_JPRB, 0.13354E+01_JPRB, 0.13134E+01_JPRB, 0.13018E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.13230E+01_JPRB, 0.12647E+01_JPRB, 0.11774E+01_JPRB, 0.10995E+01_JPRB, 0.10079E+01_JPRB, 0.95303E+00_JPRB ,&
&  0.91488E+00_JPRB, 0.85710E+00_JPRB, 0.82848E+00_JPRB, 0.81275E+00_JPRB, 0.80298E+00_JPRB, 0.79144E+00_JPRB, 0.78141E+00_JPRB ,&
&  0.77727E+00_JPRB, 0.77547E+00_JPRB, 0.77596E+00_JPRB, 0.77339E+00_JPRB, 0.77053E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.75210E+00_JPRB, 0.75560E+00_JPRB, 0.75600E+00_JPRB, 0.75059E+00_JPRB, 0.73181E+00_JPRB, 0.70016E+00_JPRB, 0.68129E+00_JPRB ,&
&  0.66985E+00_JPRB, 0.66828E+00_JPRB, 0.66591E+00_JPRB, 0.66211E+00_JPRB, 0.66286E+00_JPRB, 0.66914E+00_JPRB, 0.67547E+00_JPRB ,&
&  0.67680E+00_JPRB, 0.66813E+00_JPRB, 0.65133E+00_JPRB, 0.62346E+00_JPRB, 0.60057E+00_JPRB, 0.58306E+00_JPRB, 0.57342E+00_JPRB ,&
&  0.56792E+00_JPRB, 0.55868E+00_JPRB, 0.54536E+00_JPRB, 0.52531E+00_JPRB, 0.51065E+00_JPRB, 0.49860E+00_JPRB, 0.50282E+00_JPRB ,&
&  0.51736E+00_JPRB, 0.52304E+00_JPRB, 0.52553E+00_JPRB, 0.53469E+00_JPRB, 0.54093E+00_JPRB, 0.55741E+00_JPRB, 0.50645E+00_JPRB ,&
&  0.47055E+00_JPRB, 0.45471E+00_JPRB, 0.44278E+00_JPRB, 0.43952E+00_JPRB, 0.44835E+00_JPRB, 0.47160E+00_JPRB, 0.52099E+00_JPRB ,&
&  0.63562E+00_JPRB, 0.65677E+00_JPRB, 0.66444E+00_JPRB, 0.67767E+00_JPRB, 0.68802E+00_JPRB, 0.69197E+00_JPRB, 0.70040E+00_JPRB ,&
&  0.71744E+00_JPRB, 0.74688E+00_JPRB, 0.77710E+00_JPRB, 0.78026E+00_JPRB, 0.78105E+00_JPRB, 0.77695E+00_JPRB, 0.86762E+00_JPRB ,&
&  0.11204E+01_JPRB, 0.10201E+01_JPRB, 0.89035E+00_JPRB, 0.81060E+00_JPRB, 0.77371E+00_JPRB, 0.75937E+00_JPRB, 0.78423E+00_JPRB ,&
&  0.82482E+00_JPRB, 0.87205E+00_JPRB, 0.93870E+00_JPRB, 0.99090E+00_JPRB, 0.10440E+01_JPRB, 0.11179E+01_JPRB, 0.12209E+01_JPRB ,&
&  0.13257E+01_JPRB, 0.14684E+01_JPRB, 0.17847E+01_JPRB, 0.15713E+01_JPRB, 0.14498E+01_JPRB, 0.14092E+01_JPRB, 0.13958E+01_JPRB ,&
&  0.13531E+01_JPRB, 0.13563E+01_JPRB, 0.12655E+01_JPRB, 0.11402E+01_JPRB, 0.10732E+01_JPRB, 0.99962E+00_JPRB, 0.94539E+00_JPRB ,&
&  0.89389E+00_JPRB, 0.84645E+00_JPRB, 0.82088E+00_JPRB, 0.80027E+00_JPRB, 0.79382E+00_JPRB, 0.78691E+00_JPRB, 0.77785E+00_JPRB ,&
&  0.76664E+00_JPRB, 0.75718E+00_JPRB, 0.75065E+00_JPRB, 0.74652E+00_JPRB, 0.74630E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.72006E+00_JPRB, 0.74389E+00_JPRB, 0.76546E+00_JPRB, 0.74773E+00_JPRB, 0.72028E+00_JPRB, 0.66331E+00_JPRB, 0.62078E+00_JPRB ,&
&  0.59204E+00_JPRB, 0.58529E+00_JPRB, 0.58306E+00_JPRB, 0.58531E+00_JPRB, 0.59239E+00_JPRB, 0.60437E+00_JPRB, 0.61236E+00_JPRB ,&
&  0.62039E+00_JPRB, 0.61914E+00_JPRB, 0.61835E+00_JPRB, 0.61011E+00_JPRB, 0.59502E+00_JPRB, 0.57674E+00_JPRB, 0.56390E+00_JPRB ,&
&  0.55691E+00_JPRB, 0.55319E+00_JPRB, 0.54679E+00_JPRB, 0.53650E+00_JPRB, 0.52898E+00_JPRB, 0.51849E+00_JPRB, 0.52191E+00_JPRB ,&
&  0.51498E+00_JPRB, 0.51652E+00_JPRB, 0.53085E+00_JPRB, 0.54239E+00_JPRB, 0.55480E+00_JPRB, 0.57201E+00_JPRB, 0.53479E+00_JPRB ,&
&  0.49332E+00_JPRB, 0.47375E+00_JPRB, 0.46706E+00_JPRB, 0.47127E+00_JPRB, 0.51247E+00_JPRB, 0.61764E+00_JPRB, 0.65741E+00_JPRB ,&
&  0.67800E+00_JPRB, 0.69276E+00_JPRB, 0.70929E+00_JPRB, 0.73067E+00_JPRB, 0.73627E+00_JPRB, 0.74142E+00_JPRB, 0.75898E+00_JPRB ,&
&  0.78841E+00_JPRB, 0.83628E+00_JPRB, 0.87894E+00_JPRB, 0.77755E+00_JPRB, 0.73742E+00_JPRB, 0.76838E+00_JPRB, 0.82146E+00_JPRB ,&
&  0.89431E+00_JPRB, 0.88329E+00_JPRB, 0.86251E+00_JPRB, 0.88570E+00_JPRB, 0.88959E+00_JPRB, 0.89453E+00_JPRB, 0.89316E+00_JPRB ,&
&  0.90458E+00_JPRB, 0.95098E+00_JPRB, 0.10546E+01_JPRB, 0.11102E+01_JPRB, 0.11961E+01_JPRB, 0.13224E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.16530E+01_JPRB, 0.16979E+01_JPRB, 0.16358E+01_JPRB, 0.15888E+01_JPRB, 0.15878E+01_JPRB, 0.15596E+01_JPRB ,&
&  0.14454E+01_JPRB, 0.13633E+01_JPRB, 0.12939E+01_JPRB, 0.12227E+01_JPRB, 0.11192E+01_JPRB, 0.10340E+01_JPRB, 0.94644E+00_JPRB ,&
&  0.86229E+00_JPRB, 0.80084E+00_JPRB, 0.76922E+00_JPRB, 0.74900E+00_JPRB, 0.74274E+00_JPRB, 0.73949E+00_JPRB, 0.72817E+00_JPRB ,&
&  0.71113E+00_JPRB, 0.69306E+00_JPRB, 0.68525E+00_JPRB, 0.69100E+00_JPRB, 0.71111E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.77105E+00_JPRB, 0.76583E+00_JPRB, 0.73158E+00_JPRB, 0.73895E+00_JPRB, 0.70991E+00_JPRB, 0.68456E+00_JPRB, 0.64635E+00_JPRB ,&
&  0.62340E+00_JPRB, 0.59078E+00_JPRB, 0.56894E+00_JPRB, 0.53939E+00_JPRB, 0.52199E+00_JPRB, 0.51309E+00_JPRB, 0.52820E+00_JPRB ,&
&  0.54943E+00_JPRB, 0.56469E+00_JPRB, 0.57341E+00_JPRB, 0.57447E+00_JPRB, 0.57169E+00_JPRB, 0.56339E+00_JPRB, 0.55459E+00_JPRB ,&
&  0.54720E+00_JPRB, 0.54749E+00_JPRB, 0.54382E+00_JPRB, 0.53950E+00_JPRB, 0.53412E+00_JPRB, 0.52997E+00_JPRB, 0.52946E+00_JPRB ,&
&  0.52097E+00_JPRB, 0.51544E+00_JPRB, 0.52111E+00_JPRB, 0.54109E+00_JPRB, 0.56003E+00_JPRB, 0.57690E+00_JPRB, 0.54725E+00_JPRB ,&
&  0.51107E+00_JPRB, 0.50329E+00_JPRB, 0.51672E+00_JPRB, 0.61356E+00_JPRB, 0.63882E+00_JPRB, 0.64954E+00_JPRB, 0.67599E+00_JPRB ,&
&  0.65691E+00_JPRB, 0.65450E+00_JPRB, 0.67269E+00_JPRB, 0.74317E+00_JPRB, 0.76645E+00_JPRB, 0.77972E+00_JPRB, 0.80425E+00_JPRB ,&
&  0.83969E+00_JPRB, 0.89380E+00_JPRB, 0.95265E+00_JPRB, 0.86365E+00_JPRB, 0.90865E+00_JPRB, 0.10659E+01_JPRB, 0.97737E+00_JPRB ,&
&  0.95695E+00_JPRB, 0.93718E+00_JPRB, 0.96934E+00_JPRB, 0.99511E+00_JPRB, 0.98948E+00_JPRB, 0.10197E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10346E+01_JPRB, 0.10905E+01_JPRB, 0.12166E+01_JPRB, 0.13432E+01_JPRB, 0.14593E+01_JPRB, 0.16112E+01_JPRB, 0.17689E+01_JPRB ,&
&  0.19617E+01_JPRB, 0.21341E+01_JPRB, 0.22587E+01_JPRB, 0.22420E+01_JPRB, 0.21870E+01_JPRB, 0.20813E+01_JPRB, 0.19280E+01_JPRB ,&
&  0.17136E+01_JPRB, 0.15579E+01_JPRB, 0.14519E+01_JPRB, 0.13673E+01_JPRB, 0.12614E+01_JPRB, 0.11233E+01_JPRB, 0.94795E+00_JPRB ,&
&  0.81688E+00_JPRB, 0.73260E+00_JPRB, 0.69820E+00_JPRB, 0.68858E+00_JPRB, 0.69247E+00_JPRB, 0.69627E+00_JPRB, 0.68953E+00_JPRB ,&
&  0.67874E+00_JPRB, 0.67162E+00_JPRB, 0.66924E+00_JPRB, 0.68467E+00_JPRB, 0.73933E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.79501E+00_JPRB, 0.77579E+00_JPRB, 0.75491E+00_JPRB, 0.74822E+00_JPRB, 0.74575E+00_JPRB, 0.70738E+00_JPRB, 0.64014E+00_JPRB ,&
&  0.65097E+00_JPRB, 0.60352E+00_JPRB, 0.59075E+00_JPRB, 0.57680E+00_JPRB, 0.56037E+00_JPRB, 0.52793E+00_JPRB, 0.50200E+00_JPRB ,&
&  0.50284E+00_JPRB, 0.51935E+00_JPRB, 0.53749E+00_JPRB, 0.55290E+00_JPRB, 0.55833E+00_JPRB, 0.55462E+00_JPRB, 0.54854E+00_JPRB ,&
&  0.54366E+00_JPRB, 0.54376E+00_JPRB, 0.54680E+00_JPRB, 0.55092E+00_JPRB, 0.55032E+00_JPRB, 0.54884E+00_JPRB, 0.55105E+00_JPRB ,&
&  0.55131E+00_JPRB, 0.54945E+00_JPRB, 0.54252E+00_JPRB, 0.55187E+00_JPRB, 0.55985E+00_JPRB, 0.57803E+00_JPRB, 0.60037E+00_JPRB ,&
&  0.53828E+00_JPRB, 0.54658E+00_JPRB, 0.63563E+00_JPRB, 0.64473E+00_JPRB, 0.65926E+00_JPRB, 0.67365E+00_JPRB, 0.68503E+00_JPRB ,&
&  0.70083E+00_JPRB, 0.72579E+00_JPRB, 0.75670E+00_JPRB, 0.78460E+00_JPRB, 0.80821E+00_JPRB, 0.83745E+00_JPRB, 0.86174E+00_JPRB ,&
&  0.92841E+00_JPRB, 0.85063E+00_JPRB, 0.81546E+00_JPRB, 0.87374E+00_JPRB, 0.11650E+01_JPRB, 0.11809E+01_JPRB, 0.11690E+01_JPRB ,&
&  0.11917E+01_JPRB, 0.11498E+01_JPRB, 0.11448E+01_JPRB, 0.11821E+01_JPRB, 0.12033E+01_JPRB, 0.12664E+01_JPRB, 0.13588E+01_JPRB ,&
&  0.13662E+01_JPRB, 0.14648E+01_JPRB, 0.15923E+01_JPRB, 0.17066E+01_JPRB, 0.18334E+01_JPRB, 0.20216E+01_JPRB, 0.22319E+01_JPRB ,&
&  0.24541E+01_JPRB, 0.26458E+01_JPRB, 0.27341E+01_JPRB, 0.26970E+01_JPRB, 0.25741E+01_JPRB, 0.24121E+01_JPRB, 0.22098E+01_JPRB ,&
&  0.19479E+01_JPRB, 0.17600E+01_JPRB, 0.16321E+01_JPRB, 0.15069E+01_JPRB, 0.13924E+01_JPRB, 0.12416E+01_JPRB, 0.10266E+01_JPRB ,&
&  0.80479E+00_JPRB, 0.71046E+00_JPRB, 0.69153E+00_JPRB, 0.68852E+00_JPRB, 0.69391E+00_JPRB, 0.70548E+00_JPRB, 0.71556E+00_JPRB ,&
&  0.72037E+00_JPRB, 0.70797E+00_JPRB, 0.70651E+00_JPRB, 0.73479E+00_JPRB, 0.76904E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.76148E+00_JPRB, 0.75145E+00_JPRB, 0.75031E+00_JPRB, 0.76623E+00_JPRB, 0.75927E+00_JPRB, 0.71976E+00_JPRB, 0.67446E+00_JPRB ,&
&  0.78288E+00_JPRB, 0.74853E+00_JPRB, 0.80395E+00_JPRB, 0.65939E+00_JPRB, 0.60470E+00_JPRB, 0.58392E+00_JPRB, 0.55590E+00_JPRB ,&
&  0.53380E+00_JPRB, 0.52242E+00_JPRB, 0.51926E+00_JPRB, 0.53123E+00_JPRB, 0.53784E+00_JPRB, 0.53571E+00_JPRB, 0.53277E+00_JPRB ,&
&  0.53102E+00_JPRB, 0.53456E+00_JPRB, 0.54442E+00_JPRB, 0.54997E+00_JPRB, 0.55418E+00_JPRB, 0.55846E+00_JPRB, 0.56618E+00_JPRB ,&
&  0.55183E+00_JPRB, 0.54859E+00_JPRB, 0.55409E+00_JPRB, 0.55696E+00_JPRB, 0.56802E+00_JPRB, 0.57522E+00_JPRB, 0.59377E+00_JPRB ,&
&  0.63792E+00_JPRB, 0.60978E+00_JPRB, 0.68658E+00_JPRB, 0.69224E+00_JPRB, 0.70186E+00_JPRB, 0.71170E+00_JPRB, 0.72017E+00_JPRB ,&
&  0.72826E+00_JPRB, 0.74544E+00_JPRB, 0.77245E+00_JPRB, 0.79652E+00_JPRB, 0.83240E+00_JPRB, 0.87463E+00_JPRB, 0.90558E+00_JPRB ,&
&  0.99841E+00_JPRB, 0.84585E+00_JPRB, 0.91920E+00_JPRB, 0.96481E+00_JPRB, 0.11797E+01_JPRB, 0.14289E+01_JPRB, 0.13936E+01_JPRB ,&
&  0.16278E+01_JPRB, 0.15285E+01_JPRB, 0.14552E+01_JPRB, 0.13933E+01_JPRB, 0.14352E+01_JPRB, 0.15167E+01_JPRB, 0.16196E+01_JPRB ,&
&  0.17174E+01_JPRB, 0.18226E+01_JPRB, 0.19942E+01_JPRB, 0.21463E+01_JPRB, 0.23047E+01_JPRB, 0.24867E+01_JPRB, 0.26586E+01_JPRB ,&
&  0.28115E+01_JPRB, 0.29078E+01_JPRB, 0.29637E+01_JPRB, 0.28019E+01_JPRB, 0.26565E+01_JPRB, 0.25009E+01_JPRB, 0.21962E+01_JPRB ,&
&  0.19619E+01_JPRB, 0.17819E+01_JPRB, 0.16320E+01_JPRB, 0.15215E+01_JPRB, 0.13258E+01_JPRB, 0.11630E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.81171E+00_JPRB, 0.74053E+00_JPRB, 0.74895E+00_JPRB, 0.75756E+00_JPRB, 0.75320E+00_JPRB, 0.75801E+00_JPRB, 0.76552E+00_JPRB ,&
&  0.75199E+00_JPRB, 0.74989E+00_JPRB, 0.77689E+00_JPRB, 0.77744E+00_JPRB, 0.77037E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.79502E+00_JPRB, 0.80368E+00_JPRB, 0.80865E+00_JPRB, 0.80593E+00_JPRB, 0.78532E+00_JPRB, 0.80039E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.81534E+00_JPRB, 0.82679E+00_JPRB, 0.81248E+00_JPRB, 0.82180E+00_JPRB, 0.85310E+00_JPRB, 0.79385E+00_JPRB, 0.65054E+00_JPRB ,&
&  0.58470E+00_JPRB, 0.57541E+00_JPRB, 0.57309E+00_JPRB, 0.56998E+00_JPRB, 0.56948E+00_JPRB, 0.56586E+00_JPRB, 0.56382E+00_JPRB ,&
&  0.56251E+00_JPRB, 0.56059E+00_JPRB, 0.56578E+00_JPRB, 0.57106E+00_JPRB, 0.57221E+00_JPRB, 0.57413E+00_JPRB, 0.58300E+00_JPRB ,&
&  0.56421E+00_JPRB, 0.55229E+00_JPRB, 0.54546E+00_JPRB, 0.53491E+00_JPRB, 0.55558E+00_JPRB, 0.57264E+00_JPRB, 0.60122E+00_JPRB ,&
&  0.64241E+00_JPRB, 0.68399E+00_JPRB, 0.70585E+00_JPRB, 0.72381E+00_JPRB, 0.73853E+00_JPRB, 0.75331E+00_JPRB, 0.76838E+00_JPRB ,&
&  0.78237E+00_JPRB, 0.79997E+00_JPRB, 0.82606E+00_JPRB, 0.84598E+00_JPRB, 0.90507E+00_JPRB, 0.89325E+00_JPRB, 0.99090E+00_JPRB ,&
&  0.10606E+01_JPRB, 0.11948E+01_JPRB, 0.12841E+01_JPRB, 0.11654E+01_JPRB, 0.13382E+01_JPRB, 0.15476E+01_JPRB, 0.15615E+01_JPRB ,&
&  0.16279E+01_JPRB, 0.16283E+01_JPRB, 0.16869E+01_JPRB, 0.17680E+01_JPRB, 0.19759E+01_JPRB, 0.21309E+01_JPRB, 0.22082E+01_JPRB ,&
&  0.22850E+01_JPRB, 0.25543E+01_JPRB, 0.27373E+01_JPRB, 0.28551E+01_JPRB, 0.29085E+01_JPRB, 0.29542E+01_JPRB, 0.30264E+01_JPRB ,&
&  0.30790E+01_JPRB, 0.31275E+01_JPRB, 0.31146E+01_JPRB, 0.30125E+01_JPRB, 0.28555E+01_JPRB, 0.26392E+01_JPRB, 0.23467E+01_JPRB ,&
&  0.20850E+01_JPRB, 0.17595E+01_JPRB, 0.14938E+01_JPRB, 0.13294E+01_JPRB, 0.11753E+01_JPRB, 0.10231E+01_JPRB, 0.91793E+00_JPRB ,&
&  0.81543E+00_JPRB, 0.80509E+00_JPRB, 0.86875E+00_JPRB, 0.86718E+00_JPRB, 0.86474E+00_JPRB, 0.85277E+00_JPRB, 0.79134E+00_JPRB ,&
&  0.77677E+00_JPRB, 0.80504E+00_JPRB, 0.81479E+00_JPRB, 0.80734E+00_JPRB, 0.79756E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.89156E+00_JPRB, 0.88695E+00_JPRB, 0.88140E+00_JPRB, 0.86604E+00_JPRB, 0.85629E+00_JPRB, 0.85561E+00_JPRB, 0.85563E+00_JPRB ,&
&  0.85744E+00_JPRB, 0.86720E+00_JPRB, 0.86411E+00_JPRB, 0.84896E+00_JPRB, 0.83763E+00_JPRB, 0.84503E+00_JPRB, 0.76742E+00_JPRB ,&
&  0.66110E+00_JPRB, 0.62672E+00_JPRB, 0.61552E+00_JPRB, 0.61908E+00_JPRB, 0.63095E+00_JPRB, 0.63643E+00_JPRB, 0.63840E+00_JPRB ,&
&  0.64228E+00_JPRB, 0.63524E+00_JPRB, 0.63905E+00_JPRB, 0.63930E+00_JPRB, 0.63370E+00_JPRB, 0.63129E+00_JPRB, 0.62906E+00_JPRB ,&
&  0.59481E+00_JPRB, 0.57061E+00_JPRB, 0.55750E+00_JPRB, 0.54975E+00_JPRB, 0.56443E+00_JPRB, 0.59190E+00_JPRB, 0.63263E+00_JPRB ,&
&  0.68136E+00_JPRB, 0.71793E+00_JPRB, 0.75002E+00_JPRB, 0.78099E+00_JPRB, 0.80420E+00_JPRB, 0.82850E+00_JPRB, 0.85071E+00_JPRB ,&
&  0.86837E+00_JPRB, 0.88409E+00_JPRB, 0.89426E+00_JPRB, 0.91150E+00_JPRB, 0.91235E+00_JPRB, 0.99869E+00_JPRB, 0.11895E+01_JPRB ,&
&  0.12714E+01_JPRB, 0.13868E+01_JPRB, 0.14661E+01_JPRB, 0.16683E+01_JPRB, 0.17979E+01_JPRB, 0.17756E+01_JPRB, 0.18247E+01_JPRB ,&
&  0.19980E+01_JPRB, 0.21949E+01_JPRB, 0.26099E+01_JPRB, 0.26446E+01_JPRB, 0.27242E+01_JPRB, 0.28321E+01_JPRB, 0.28241E+01_JPRB ,&
&  0.29198E+01_JPRB, 0.30406E+01_JPRB, 0.32118E+01_JPRB, 0.32272E+01_JPRB, 0.31801E+01_JPRB, 0.32078E+01_JPRB, 0.32021E+01_JPRB ,&
&  0.32373E+01_JPRB, 0.32756E+01_JPRB, 0.31345E+01_JPRB, 0.30027E+01_JPRB, 0.27392E+01_JPRB, 0.27326E+01_JPRB, 0.25319E+01_JPRB ,&
&  0.21228E+01_JPRB, 0.15801E+01_JPRB, 0.14269E+01_JPRB, 0.12558E+01_JPRB, 0.11387E+01_JPRB, 0.10057E+01_JPRB, 0.90248E+00_JPRB ,&
&  0.86045E+00_JPRB, 0.91719E+00_JPRB, 0.92636E+00_JPRB, 0.93641E+00_JPRB, 0.94711E+00_JPRB, 0.94132E+00_JPRB, 0.84880E+00_JPRB ,&
&  0.84295E+00_JPRB, 0.89332E+00_JPRB, 0.89921E+00_JPRB, 0.89810E+00_JPRB, 0.89406E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.10266E+01_JPRB, 0.10040E+01_JPRB, 0.98442E+00_JPRB, 0.96986E+00_JPRB, 0.95178E+00_JPRB, 0.93027E+00_JPRB, 0.90875E+00_JPRB ,&
&  0.90752E+00_JPRB, 0.91981E+00_JPRB, 0.92188E+00_JPRB, 0.89552E+00_JPRB, 0.86859E+00_JPRB, 0.85328E+00_JPRB, 0.87418E+00_JPRB ,&
&  0.74753E+00_JPRB, 0.72070E+00_JPRB, 0.71704E+00_JPRB, 0.73973E+00_JPRB, 0.80677E+00_JPRB, 0.82439E+00_JPRB, 0.84998E+00_JPRB ,&
&  0.83936E+00_JPRB, 0.82295E+00_JPRB, 0.81360E+00_JPRB, 0.79788E+00_JPRB, 0.77148E+00_JPRB, 0.75258E+00_JPRB, 0.72690E+00_JPRB ,&
&  0.66748E+00_JPRB, 0.62191E+00_JPRB, 0.60535E+00_JPRB, 0.59805E+00_JPRB, 0.62191E+00_JPRB, 0.65039E+00_JPRB, 0.70420E+00_JPRB ,&
&  0.75410E+00_JPRB, 0.80049E+00_JPRB, 0.84392E+00_JPRB, 0.89443E+00_JPRB, 0.93405E+00_JPRB, 0.97208E+00_JPRB, 0.99620E+00_JPRB ,&
&  0.10075E+01_JPRB, 0.10089E+01_JPRB, 0.10217E+01_JPRB, 0.10045E+01_JPRB, 0.10069E+01_JPRB, 0.12346E+01_JPRB, 0.15393E+01_JPRB ,&
&  0.16657E+01_JPRB, 0.18016E+01_JPRB, 0.23059E+01_JPRB, 0.30458E+01_JPRB, 0.29006E+01_JPRB, 0.26399E+01_JPRB, 0.25701E+01_JPRB ,&
&  0.27603E+01_JPRB, 0.30263E+01_JPRB, 0.33138E+01_JPRB, 0.34250E+01_JPRB, 0.34212E+01_JPRB, 0.35284E+01_JPRB, 0.35627E+01_JPRB ,&
&  0.35529E+01_JPRB, 0.35987E+01_JPRB, 0.35521E+01_JPRB, 0.34500E+01_JPRB, 0.33546E+01_JPRB, 0.34204E+01_JPRB, 0.34148E+01_JPRB ,&
&  0.33763E+01_JPRB, 0.32511E+01_JPRB, 0.30126E+01_JPRB, 0.28799E+01_JPRB, 0.24190E+01_JPRB, 0.23419E+01_JPRB, 0.26163E+01_JPRB ,&
&  0.21587E+01_JPRB, 0.16980E+01_JPRB, 0.14476E+01_JPRB, 0.12901E+01_JPRB, 0.12185E+01_JPRB, 0.11898E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.98579E+00_JPRB, 0.10128E+01_JPRB, 0.10997E+01_JPRB, 0.11468E+01_JPRB, 0.11598E+01_JPRB, 0.11611E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.10592E+01_JPRB, 0.10548E+01_JPRB, 0.10541E+01_JPRB, 0.10441E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.11652E+01_JPRB, 0.11171E+01_JPRB, 0.10881E+01_JPRB, 0.10462E+01_JPRB, 0.10023E+01_JPRB, 0.96702E+00_JPRB, 0.94707E+00_JPRB ,&
&  0.95770E+00_JPRB, 0.96258E+00_JPRB, 0.95348E+00_JPRB, 0.92219E+00_JPRB, 0.89877E+00_JPRB, 0.88268E+00_JPRB, 0.89765E+00_JPRB ,&
&  0.89552E+00_JPRB, 0.82687E+00_JPRB, 0.81239E+00_JPRB, 0.81619E+00_JPRB, 0.94832E+00_JPRB, 0.10726E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.12100E+01_JPRB, 0.12247E+01_JPRB, 0.12035E+01_JPRB, 0.11298E+01_JPRB, 0.10525E+01_JPRB, 0.98250E+00_JPRB, 0.90802E+00_JPRB ,&
&  0.82686E+00_JPRB, 0.75298E+00_JPRB, 0.71707E+00_JPRB, 0.71835E+00_JPRB, 0.76633E+00_JPRB, 0.74571E+00_JPRB, 0.80419E+00_JPRB ,&
&  0.86207E+00_JPRB, 0.92630E+00_JPRB, 0.97925E+00_JPRB, 0.10470E+01_JPRB, 0.10930E+01_JPRB, 0.11279E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11533E+01_JPRB, 0.11280E+01_JPRB, 0.10226E+01_JPRB, 0.10633E+01_JPRB, 0.11872E+01_JPRB, 0.16641E+01_JPRB, 0.21827E+01_JPRB ,&
&  0.25436E+01_JPRB, 0.31044E+01_JPRB, 0.38044E+01_JPRB, 0.43034E+01_JPRB, 0.39043E+01_JPRB, 0.35844E+01_JPRB, 0.36216E+01_JPRB ,&
&  0.38565E+01_JPRB, 0.41169E+01_JPRB, 0.42247E+01_JPRB, 0.40561E+01_JPRB, 0.41160E+01_JPRB, 0.42270E+01_JPRB, 0.43174E+01_JPRB ,&
&  0.42979E+01_JPRB, 0.41434E+01_JPRB, 0.39278E+01_JPRB, 0.37231E+01_JPRB, 0.35901E+01_JPRB, 0.33339E+01_JPRB, 0.32674E+01_JPRB ,&
&  0.33382E+01_JPRB, 0.28434E+01_JPRB, 0.25645E+01_JPRB, 0.26685E+01_JPRB, 0.21994E+01_JPRB, 0.21488E+01_JPRB, 0.23100E+01_JPRB ,&
&  0.19679E+01_JPRB, 0.16437E+01_JPRB, 0.14758E+01_JPRB, 0.13214E+01_JPRB, 0.12875E+01_JPRB, 0.14208E+01_JPRB, 0.14952E+01_JPRB ,&
&  0.14123E+01_JPRB, 0.13508E+01_JPRB, 0.15453E+01_JPRB, 0.16623E+01_JPRB, 0.17262E+01_JPRB, 0.16612E+01_JPRB, 0.15530E+01_JPRB ,&
&  0.14593E+01_JPRB, 0.13935E+01_JPRB, 0.13430E+01_JPRB, 0.12888E+01_JPRB, 0.12318E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.12743E+01_JPRB, 0.12504E+01_JPRB, 0.11991E+01_JPRB, 0.11258E+01_JPRB, 0.10672E+01_JPRB, 0.10299E+01_JPRB, 0.99331E+00_JPRB ,&
&  0.96766E+00_JPRB, 0.94420E+00_JPRB, 0.93283E+00_JPRB, 0.91033E+00_JPRB, 0.90304E+00_JPRB, 0.90580E+00_JPRB, 0.90967E+00_JPRB ,&
&  0.90484E+00_JPRB, 0.90021E+00_JPRB, 0.86087E+00_JPRB, 0.87386E+00_JPRB, 0.10097E+01_JPRB, 0.13986E+01_JPRB, 0.16788E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.17877E+01_JPRB, 0.17228E+01_JPRB, 0.16549E+01_JPRB, 0.15369E+01_JPRB, 0.13660E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.10345E+01_JPRB, 0.92815E+00_JPRB, 0.93577E+00_JPRB, 0.95549E+00_JPRB, 0.91497E+00_JPRB, 0.10024E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.11143E+01_JPRB, 0.11585E+01_JPRB, 0.11996E+01_JPRB, 0.12186E+01_JPRB, 0.11539E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10798E+01_JPRB, 0.11189E+01_JPRB, 0.12880E+01_JPRB, 0.17561E+01_JPRB, 0.25873E+01_JPRB, 0.33275E+01_JPRB ,&
&  0.37982E+01_JPRB, 0.40286E+01_JPRB, 0.37977E+01_JPRB, 0.41978E+01_JPRB, 0.48456E+01_JPRB, 0.47125E+01_JPRB, 0.44883E+01_JPRB ,&
&  0.45034E+01_JPRB, 0.45002E+01_JPRB, 0.44331E+01_JPRB, 0.45248E+01_JPRB, 0.49149E+01_JPRB, 0.51290E+01_JPRB, 0.51429E+01_JPRB ,&
&  0.44973E+01_JPRB, 0.42509E+01_JPRB, 0.41240E+01_JPRB, 0.39469E+01_JPRB, 0.35131E+01_JPRB, 0.33456E+01_JPRB, 0.34184E+01_JPRB ,&
&  0.33312E+01_JPRB, 0.32703E+01_JPRB, 0.25942E+01_JPRB, 0.24348E+01_JPRB, 0.18846E+01_JPRB, 0.18189E+01_JPRB, 0.20416E+01_JPRB ,&
&  0.18681E+01_JPRB, 0.17057E+01_JPRB, 0.14931E+01_JPRB, 0.14122E+01_JPRB, 0.16689E+01_JPRB, 0.19586E+01_JPRB, 0.23611E+01_JPRB ,&
&  0.23382E+01_JPRB, 0.20121E+01_JPRB, 0.22028E+01_JPRB, 0.22840E+01_JPRB, 0.23834E+01_JPRB, 0.22518E+01_JPRB, 0.20112E+01_JPRB ,&
&  0.18174E+01_JPRB, 0.16706E+01_JPRB, 0.15280E+01_JPRB, 0.13979E+01_JPRB, 0.13175E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.13105E+01_JPRB, 0.12750E+01_JPRB, 0.12089E+01_JPRB, 0.11298E+01_JPRB, 0.10740E+01_JPRB, 0.10311E+01_JPRB, 0.99697E+00_JPRB ,&
&  0.96340E+00_JPRB, 0.93282E+00_JPRB, 0.91585E+00_JPRB, 0.90354E+00_JPRB, 0.89641E+00_JPRB, 0.89248E+00_JPRB, 0.89095E+00_JPRB ,&
&  0.89313E+00_JPRB, 0.91981E+00_JPRB, 0.89564E+00_JPRB, 0.92365E+00_JPRB, 0.11417E+01_JPRB, 0.16198E+01_JPRB, 0.20788E+01_JPRB ,&
&  0.22820E+01_JPRB, 0.22375E+01_JPRB, 0.21574E+01_JPRB, 0.20691E+01_JPRB, 0.19972E+01_JPRB, 0.18805E+01_JPRB, 0.17256E+01_JPRB ,&
&  0.15651E+01_JPRB, 0.12771E+01_JPRB, 0.12156E+01_JPRB, 0.11553E+01_JPRB, 0.11993E+01_JPRB, 0.12259E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.11827E+01_JPRB, 0.11852E+01_JPRB, 0.11472E+01_JPRB, 0.11214E+01_JPRB, 0.10654E+01_JPRB, 0.10733E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.12468E+01_JPRB, 0.15482E+01_JPRB, 0.20834E+01_JPRB, 0.30224E+01_JPRB, 0.43254E+01_JPRB, 0.45219E+01_JPRB ,&
&  0.37372E+01_JPRB, 0.35961E+01_JPRB, 0.49798E+01_JPRB, 0.48119E+01_JPRB, 0.50163E+01_JPRB, 0.56349E+01_JPRB, 0.53001E+01_JPRB ,&
&  0.47839E+01_JPRB, 0.44507E+01_JPRB, 0.43656E+01_JPRB, 0.46971E+01_JPRB, 0.62893E+01_JPRB, 0.65741E+01_JPRB, 0.61419E+01_JPRB ,&
&  0.51151E+01_JPRB, 0.48390E+01_JPRB, 0.44259E+01_JPRB, 0.39710E+01_JPRB, 0.37054E+01_JPRB, 0.38846E+01_JPRB, 0.36728E+01_JPRB ,&
&  0.33993E+01_JPRB, 0.36394E+01_JPRB, 0.28851E+01_JPRB, 0.22066E+01_JPRB, 0.18363E+01_JPRB, 0.18746E+01_JPRB, 0.19950E+01_JPRB ,&
&  0.19619E+01_JPRB, 0.18516E+01_JPRB, 0.17606E+01_JPRB, 0.20499E+01_JPRB, 0.27693E+01_JPRB, 0.33212E+01_JPRB, 0.32839E+01_JPRB ,&
&  0.31987E+01_JPRB, 0.34611E+01_JPRB, 0.32529E+01_JPRB, 0.32229E+01_JPRB, 0.32057E+01_JPRB, 0.28249E+01_JPRB, 0.23248E+01_JPRB ,&
&  0.19045E+01_JPRB, 0.16399E+01_JPRB, 0.14906E+01_JPRB, 0.14001E+01_JPRB, 0.13431E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.13713E+01_JPRB, 0.12991E+01_JPRB, 0.12259E+01_JPRB, 0.11548E+01_JPRB, 0.10873E+01_JPRB, 0.10335E+01_JPRB, 0.98562E+00_JPRB ,&
&  0.93799E+00_JPRB, 0.90565E+00_JPRB, 0.90330E+00_JPRB, 0.91439E+00_JPRB, 0.91893E+00_JPRB, 0.89574E+00_JPRB, 0.89480E+00_JPRB ,&
&  0.92794E+00_JPRB, 0.94816E+00_JPRB, 0.93576E+00_JPRB, 0.10292E+01_JPRB, 0.12495E+01_JPRB, 0.15841E+01_JPRB, 0.20737E+01_JPRB ,&
&  0.24167E+01_JPRB, 0.23255E+01_JPRB, 0.21667E+01_JPRB, 0.22280E+01_JPRB, 0.23279E+01_JPRB, 0.23266E+01_JPRB, 0.21910E+01_JPRB ,&
&  0.18828E+01_JPRB, 0.16880E+01_JPRB, 0.16365E+01_JPRB, 0.14940E+01_JPRB, 0.14286E+01_JPRB, 0.13444E+01_JPRB, 0.12376E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.11219E+01_JPRB, 0.11053E+01_JPRB, 0.10968E+01_JPRB, 0.11375E+01_JPRB, 0.11940E+01_JPRB, 0.12649E+01_JPRB ,&
&  0.14065E+01_JPRB, 0.16846E+01_JPRB, 0.21788E+01_JPRB, 0.29701E+01_JPRB, 0.36387E+01_JPRB, 0.54235E+01_JPRB, 0.50828E+01_JPRB ,&
&  0.46812E+01_JPRB, 0.46448E+01_JPRB, 0.46365E+01_JPRB, 0.51161E+01_JPRB, 0.45150E+01_JPRB, 0.49613E+01_JPRB, 0.51697E+01_JPRB ,&
&  0.49419E+01_JPRB, 0.48344E+01_JPRB, 0.51467E+01_JPRB, 0.53825E+01_JPRB, 0.66377E+01_JPRB, 0.84662E+01_JPRB, 0.77305E+01_JPRB ,&
&  0.62977E+01_JPRB, 0.55362E+01_JPRB, 0.47371E+01_JPRB, 0.42804E+01_JPRB, 0.36103E+01_JPRB, 0.35758E+01_JPRB, 0.31168E+01_JPRB ,&
&  0.28395E+01_JPRB, 0.22918E+01_JPRB, 0.24635E+01_JPRB, 0.20921E+01_JPRB, 0.19658E+01_JPRB, 0.19272E+01_JPRB, 0.19370E+01_JPRB ,&
&  0.20314E+01_JPRB, 0.21258E+01_JPRB, 0.25127E+01_JPRB, 0.33197E+01_JPRB, 0.44162E+01_JPRB, 0.39913E+01_JPRB, 0.42605E+01_JPRB ,&
&  0.44371E+01_JPRB, 0.43595E+01_JPRB, 0.36890E+01_JPRB, 0.35040E+01_JPRB, 0.31578E+01_JPRB, 0.28382E+01_JPRB, 0.24831E+01_JPRB ,&
&  0.21133E+01_JPRB, 0.18020E+01_JPRB, 0.16282E+01_JPRB, 0.15225E+01_JPRB, 0.14421E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.13273E+01_JPRB, 0.12454E+01_JPRB, 0.11825E+01_JPRB, 0.11245E+01_JPRB, 0.10693E+01_JPRB, 0.10038E+01_JPRB, 0.93376E+00_JPRB ,&
&  0.88197E+00_JPRB, 0.86213E+00_JPRB, 0.86921E+00_JPRB, 0.88631E+00_JPRB, 0.89597E+00_JPRB, 0.91677E+00_JPRB, 0.93512E+00_JPRB ,&
&  0.98559E+00_JPRB, 0.10255E+01_JPRB, 0.10167E+01_JPRB, 0.10747E+01_JPRB, 0.12789E+01_JPRB, 0.15341E+01_JPRB, 0.18157E+01_JPRB ,&
&  0.23928E+01_JPRB, 0.20817E+01_JPRB, 0.18748E+01_JPRB, 0.20553E+01_JPRB, 0.24979E+01_JPRB, 0.27357E+01_JPRB, 0.25729E+01_JPRB ,&
&  0.24895E+01_JPRB, 0.22491E+01_JPRB, 0.18552E+01_JPRB, 0.15575E+01_JPRB, 0.13335E+01_JPRB, 0.11761E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.98998E+00_JPRB, 0.97939E+00_JPRB, 0.99403E+00_JPRB, 0.10160E+01_JPRB, 0.10888E+01_JPRB, 0.11291E+01_JPRB, 0.13555E+01_JPRB ,&
&  0.16060E+01_JPRB, 0.19231E+01_JPRB, 0.23638E+01_JPRB, 0.28527E+01_JPRB, 0.36603E+01_JPRB, 0.59696E+01_JPRB, 0.68179E+01_JPRB ,&
&  0.51632E+01_JPRB, 0.47668E+01_JPRB, 0.50983E+01_JPRB, 0.53262E+01_JPRB, 0.53563E+01_JPRB, 0.49303E+01_JPRB, 0.48165E+01_JPRB ,&
&  0.38544E+01_JPRB, 0.35002E+01_JPRB, 0.37606E+01_JPRB, 0.48872E+01_JPRB, 0.80429E+01_JPRB, 0.10700E+02_JPRB, 0.98261E+01_JPRB ,&
&  0.80931E+01_JPRB, 0.67589E+01_JPRB, 0.56443E+01_JPRB, 0.42506E+01_JPRB, 0.37060E+01_JPRB, 0.31770E+01_JPRB, 0.36631E+01_JPRB ,&
&  0.34214E+01_JPRB, 0.30700E+01_JPRB, 0.25169E+01_JPRB, 0.19329E+01_JPRB, 0.15580E+01_JPRB, 0.18284E+01_JPRB, 0.19817E+01_JPRB ,&
&  0.22438E+01_JPRB, 0.25619E+01_JPRB, 0.41180E+01_JPRB, 0.47299E+01_JPRB, 0.50084E+01_JPRB, 0.45020E+01_JPRB, 0.53798E+01_JPRB ,&
&  0.49578E+01_JPRB, 0.47798E+01_JPRB, 0.40976E+01_JPRB, 0.31970E+01_JPRB, 0.25792E+01_JPRB, 0.22019E+01_JPRB, 0.20013E+01_JPRB ,&
&  0.19156E+01_JPRB, 0.18334E+01_JPRB, 0.17039E+01_JPRB, 0.15681E+01_JPRB, 0.14515E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.11502E+01_JPRB, 0.10758E+01_JPRB, 0.10292E+01_JPRB, 0.99016E+00_JPRB, 0.94521E+00_JPRB, 0.89675E+00_JPRB, 0.84923E+00_JPRB ,&
&  0.82472E+00_JPRB, 0.82275E+00_JPRB, 0.83832E+00_JPRB, 0.87478E+00_JPRB, 0.90979E+00_JPRB, 0.95205E+00_JPRB, 0.98361E+00_JPRB ,&
&  0.10418E+01_JPRB, 0.11379E+01_JPRB, 0.11483E+01_JPRB, 0.12117E+01_JPRB, 0.13873E+01_JPRB, 0.15123E+01_JPRB, 0.17887E+01_JPRB ,&
&  0.22883E+01_JPRB, 0.18849E+01_JPRB, 0.17235E+01_JPRB, 0.19493E+01_JPRB, 0.26289E+01_JPRB, 0.26998E+01_JPRB, 0.24845E+01_JPRB ,&
&  0.25534E+01_JPRB, 0.21958E+01_JPRB, 0.16496E+01_JPRB, 0.13515E+01_JPRB, 0.11648E+01_JPRB, 0.10195E+01_JPRB, 0.94957E+00_JPRB ,&
&  0.94983E+00_JPRB, 0.98585E+00_JPRB, 0.10340E+01_JPRB, 0.10771E+01_JPRB, 0.11191E+01_JPRB, 0.12534E+01_JPRB, 0.15214E+01_JPRB ,&
&  0.17750E+01_JPRB, 0.20462E+01_JPRB, 0.22852E+01_JPRB, 0.26279E+01_JPRB, 0.35735E+01_JPRB, 0.60074E+01_JPRB, 0.86356E+01_JPRB ,&
&  0.70225E+01_JPRB, 0.64925E+01_JPRB, 0.62199E+01_JPRB, 0.62631E+01_JPRB, 0.53089E+01_JPRB, 0.48086E+01_JPRB, 0.45681E+01_JPRB ,&
&  0.36932E+01_JPRB, 0.37128E+01_JPRB, 0.48977E+01_JPRB, 0.69581E+01_JPRB, 0.86616E+01_JPRB, 0.10499E+02_JPRB, 0.12210E+02_JPRB ,&
&  0.90432E+01_JPRB, 0.73066E+01_JPRB, 0.62236E+01_JPRB, 0.53051E+01_JPRB, 0.30581E+01_JPRB, 0.21663E+01_JPRB, 0.29859E+01_JPRB ,&
&  0.27224E+01_JPRB, 0.15525E+01_JPRB, 0.12123E+01_JPRB, 0.15111E+01_JPRB, 0.11040E+01_JPRB, 0.12954E+01_JPRB, 0.20388E+01_JPRB ,&
&  0.25659E+01_JPRB, 0.31842E+01_JPRB, 0.56365E+01_JPRB, 0.52074E+01_JPRB, 0.54035E+01_JPRB, 0.67570E+01_JPRB, 0.66334E+01_JPRB ,&
&  0.60447E+01_JPRB, 0.38672E+01_JPRB, 0.28162E+01_JPRB, 0.21422E+01_JPRB, 0.17737E+01_JPRB, 0.16875E+01_JPRB, 0.16349E+01_JPRB ,&
&  0.15900E+01_JPRB, 0.15096E+01_JPRB, 0.14318E+01_JPRB, 0.13221E+01_JPRB, 0.12408E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.93628E+00_JPRB, 0.88553E+00_JPRB, 0.87289E+00_JPRB, 0.85833E+00_JPRB, 0.81908E+00_JPRB, 0.79499E+00_JPRB, 0.77834E+00_JPRB ,&
&  0.78516E+00_JPRB, 0.80849E+00_JPRB, 0.84406E+00_JPRB, 0.89843E+00_JPRB, 0.96440E+00_JPRB, 0.10275E+01_JPRB, 0.10570E+01_JPRB ,&
&  0.11003E+01_JPRB, 0.12101E+01_JPRB, 0.13455E+01_JPRB, 0.15041E+01_JPRB, 0.17239E+01_JPRB, 0.16019E+01_JPRB, 0.21441E+01_JPRB ,&
&  0.22115E+01_JPRB, 0.17784E+01_JPRB, 0.17431E+01_JPRB, 0.20266E+01_JPRB, 0.23049E+01_JPRB, 0.21779E+01_JPRB, 0.20411E+01_JPRB ,&
&  0.19487E+01_JPRB, 0.16481E+01_JPRB, 0.13563E+01_JPRB, 0.11696E+01_JPRB, 0.10437E+01_JPRB, 0.98180E+00_JPRB, 0.10056E+01_JPRB ,&
&  0.10453E+01_JPRB, 0.11408E+01_JPRB, 0.12196E+01_JPRB, 0.11823E+01_JPRB, 0.12853E+01_JPRB, 0.14345E+01_JPRB, 0.16588E+01_JPRB ,&
&  0.18632E+01_JPRB, 0.20643E+01_JPRB, 0.22936E+01_JPRB, 0.28520E+01_JPRB, 0.46193E+01_JPRB, 0.71916E+01_JPRB, 0.78775E+01_JPRB ,&
&  0.70511E+01_JPRB, 0.64859E+01_JPRB, 0.71157E+01_JPRB, 0.64329E+01_JPRB, 0.54810E+01_JPRB, 0.53745E+01_JPRB, 0.49669E+01_JPRB ,&
&  0.50385E+01_JPRB, 0.56403E+01_JPRB, 0.65113E+01_JPRB, 0.82173E+01_JPRB, 0.10650E+02_JPRB, 0.11148E+02_JPRB, 0.11347E+02_JPRB ,&
&  0.99068E+01_JPRB, 0.75226E+01_JPRB, 0.50832E+01_JPRB, 0.35273E+01_JPRB, 0.33089E+01_JPRB, 0.27028E+01_JPRB, 0.17696E+01_JPRB ,&
&  0.14269E+01_JPRB, 0.12018E+01_JPRB, 0.88370E+00_JPRB, 0.96608E+00_JPRB, 0.87780E+00_JPRB, 0.10893E+01_JPRB, 0.19068E+01_JPRB ,&
&  0.33558E+01_JPRB, 0.43982E+01_JPRB, 0.56212E+01_JPRB, 0.52280E+01_JPRB, 0.53098E+01_JPRB, 0.69868E+01_JPRB, 0.74575E+01_JPRB ,&
&  0.35865E+01_JPRB, 0.23143E+01_JPRB, 0.18091E+01_JPRB, 0.15822E+01_JPRB, 0.14937E+01_JPRB, 0.14348E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.12975E+01_JPRB, 0.12237E+01_JPRB, 0.11873E+01_JPRB, 0.11271E+01_JPRB, 0.10270E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.83536E+00_JPRB, 0.82852E+00_JPRB, 0.83117E+00_JPRB, 0.81491E+00_JPRB, 0.76951E+00_JPRB, 0.77137E+00_JPRB, 0.78748E+00_JPRB ,&
&  0.81682E+00_JPRB, 0.85797E+00_JPRB, 0.91697E+00_JPRB, 0.99826E+00_JPRB, 0.11081E+01_JPRB, 0.11684E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.13789E+01_JPRB, 0.16173E+01_JPRB, 0.19310E+01_JPRB, 0.21902E+01_JPRB, 0.18616E+01_JPRB, 0.25225E+01_JPRB ,&
&  0.21366E+01_JPRB, 0.17950E+01_JPRB, 0.17810E+01_JPRB, 0.19299E+01_JPRB, 0.19500E+01_JPRB, 0.18208E+01_JPRB, 0.16962E+01_JPRB ,&
&  0.15165E+01_JPRB, 0.14056E+01_JPRB, 0.12047E+01_JPRB, 0.10979E+01_JPRB, 0.10228E+01_JPRB, 0.10515E+01_JPRB, 0.11204E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.13185E+01_JPRB, 0.13214E+01_JPRB, 0.14184E+01_JPRB, 0.16313E+01_JPRB, 0.18060E+01_JPRB, 0.19637E+01_JPRB ,&
&  0.20738E+01_JPRB, 0.23067E+01_JPRB, 0.27998E+01_JPRB, 0.41795E+01_JPRB, 0.56268E+01_JPRB, 0.61277E+01_JPRB, 0.70560E+01_JPRB ,&
&  0.75826E+01_JPRB, 0.76499E+01_JPRB, 0.67449E+01_JPRB, 0.71288E+01_JPRB, 0.62182E+01_JPRB, 0.55863E+01_JPRB, 0.56049E+01_JPRB ,&
&  0.61276E+01_JPRB, 0.78403E+01_JPRB, 0.86248E+01_JPRB, 0.93938E+01_JPRB, 0.13374E+02_JPRB, 0.14612E+02_JPRB, 0.11666E+02_JPRB ,&
&  0.10247E+02_JPRB, 0.73033E+01_JPRB, 0.50701E+01_JPRB, 0.36747E+01_JPRB, 0.45097E+01_JPRB, 0.48907E+01_JPRB, 0.24359E+01_JPRB ,&
&  0.15972E+01_JPRB, 0.12134E+01_JPRB, 0.87977E+00_JPRB, 0.85654E+00_JPRB, 0.86260E+00_JPRB, 0.11107E+01_JPRB, 0.25105E+01_JPRB ,&
&  0.30958E+01_JPRB, 0.50064E+01_JPRB, 0.57462E+01_JPRB, 0.55878E+01_JPRB, 0.54647E+01_JPRB, 0.50508E+01_JPRB, 0.36624E+01_JPRB ,&
&  0.23173E+01_JPRB, 0.15423E+01_JPRB, 0.13377E+01_JPRB, 0.13990E+01_JPRB, 0.14173E+01_JPRB, 0.13370E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.11303E+01_JPRB, 0.10767E+01_JPRB, 0.10200E+01_JPRB, 0.92532E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.84215E+00_JPRB, 0.86540E+00_JPRB, 0.88641E+00_JPRB, 0.83684E+00_JPRB, 0.82245E+00_JPRB, 0.83650E+00_JPRB, 0.83586E+00_JPRB ,&
&  0.88797E+00_JPRB, 0.96121E+00_JPRB, 0.10411E+01_JPRB, 0.11764E+01_JPRB, 0.13086E+01_JPRB, 0.13383E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.16457E+01_JPRB, 0.20041E+01_JPRB, 0.24552E+01_JPRB, 0.25174E+01_JPRB, 0.23374E+01_JPRB, 0.26956E+01_JPRB ,&
&  0.21898E+01_JPRB, 0.19144E+01_JPRB, 0.17788E+01_JPRB, 0.18911E+01_JPRB, 0.18993E+01_JPRB, 0.18338E+01_JPRB, 0.16704E+01_JPRB ,&
&  0.14528E+01_JPRB, 0.14357E+01_JPRB, 0.12030E+01_JPRB, 0.12398E+01_JPRB, 0.12081E+01_JPRB, 0.12844E+01_JPRB, 0.13460E+01_JPRB ,&
&  0.14488E+01_JPRB, 0.14998E+01_JPRB, 0.15705E+01_JPRB, 0.17579E+01_JPRB, 0.20087E+01_JPRB, 0.22288E+01_JPRB, 0.24327E+01_JPRB ,&
&  0.26046E+01_JPRB, 0.30625E+01_JPRB, 0.39631E+01_JPRB, 0.53566E+01_JPRB, 0.52417E+01_JPRB, 0.68472E+01_JPRB, 0.70894E+01_JPRB ,&
&  0.73859E+01_JPRB, 0.72638E+01_JPRB, 0.64260E+01_JPRB, 0.65437E+01_JPRB, 0.65489E+01_JPRB, 0.62584E+01_JPRB, 0.64564E+01_JPRB ,&
&  0.83377E+01_JPRB, 0.94573E+01_JPRB, 0.10688E+02_JPRB, 0.11505E+02_JPRB, 0.15103E+02_JPRB, 0.20577E+02_JPRB, 0.12228E+02_JPRB ,&
&  0.83962E+01_JPRB, 0.73759E+01_JPRB, 0.51820E+01_JPRB, 0.40434E+01_JPRB, 0.50300E+01_JPRB, 0.59948E+01_JPRB, 0.54609E+01_JPRB ,&
&  0.22449E+01_JPRB, 0.12599E+01_JPRB, 0.11052E+01_JPRB, 0.11107E+01_JPRB, 0.93228E+00_JPRB, 0.12189E+01_JPRB, 0.40852E+01_JPRB ,&
&  0.32701E+01_JPRB, 0.50818E+01_JPRB, 0.54344E+01_JPRB, 0.52501E+01_JPRB, 0.54543E+01_JPRB, 0.39290E+01_JPRB, 0.24859E+01_JPRB ,&
&  0.12200E+01_JPRB, 0.11517E+01_JPRB, 0.12602E+01_JPRB, 0.12258E+01_JPRB, 0.12094E+01_JPRB, 0.11380E+01_JPRB, 0.10219E+01_JPRB ,&
&  0.10083E+01_JPRB, 0.10014E+01_JPRB, 0.93801E+00_JPRB, 0.88380E+00_JPRB, 0.85092E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.88907E+00_JPRB, 0.91884E+00_JPRB, 0.92061E+00_JPRB, 0.91833E+00_JPRB, 0.92934E+00_JPRB, 0.94375E+00_JPRB, 0.94438E+00_JPRB ,&
&  0.10083E+01_JPRB, 0.11185E+01_JPRB, 0.12691E+01_JPRB, 0.14578E+01_JPRB, 0.15769E+01_JPRB, 0.15808E+01_JPRB, 0.16157E+01_JPRB ,&
&  0.17697E+01_JPRB, 0.20750E+01_JPRB, 0.24544E+01_JPRB, 0.27222E+01_JPRB, 0.26343E+01_JPRB, 0.23938E+01_JPRB, 0.25717E+01_JPRB ,&
&  0.23836E+01_JPRB, 0.20985E+01_JPRB, 0.17858E+01_JPRB, 0.19074E+01_JPRB, 0.20618E+01_JPRB, 0.21173E+01_JPRB, 0.19179E+01_JPRB ,&
&  0.17616E+01_JPRB, 0.17318E+01_JPRB, 0.16009E+01_JPRB, 0.16115E+01_JPRB, 0.16498E+01_JPRB, 0.17423E+01_JPRB, 0.18149E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.18493E+01_JPRB, 0.20110E+01_JPRB, 0.22569E+01_JPRB, 0.25454E+01_JPRB, 0.28184E+01_JPRB, 0.31306E+01_JPRB ,&
&  0.35458E+01_JPRB, 0.42404E+01_JPRB, 0.51324E+01_JPRB, 0.54771E+01_JPRB, 0.59159E+01_JPRB, 0.64695E+01_JPRB, 0.69006E+01_JPRB ,&
&  0.71357E+01_JPRB, 0.72107E+01_JPRB, 0.69007E+01_JPRB, 0.65974E+01_JPRB, 0.67543E+01_JPRB, 0.71571E+01_JPRB, 0.81981E+01_JPRB ,&
&  0.10364E+02_JPRB, 0.11727E+02_JPRB, 0.12015E+02_JPRB, 0.12680E+02_JPRB, 0.14039E+02_JPRB, 0.15062E+02_JPRB, 0.12464E+02_JPRB ,&
&  0.90428E+01_JPRB, 0.62698E+01_JPRB, 0.48431E+01_JPRB, 0.49887E+01_JPRB, 0.59490E+01_JPRB, 0.62036E+01_JPRB, 0.79248E+01_JPRB ,&
&  0.75679E+01_JPRB, 0.29450E+01_JPRB, 0.19576E+01_JPRB, 0.25587E+01_JPRB, 0.12988E+01_JPRB, 0.11605E+01_JPRB, 0.21360E+01_JPRB ,&
&  0.34287E+01_JPRB, 0.46406E+01_JPRB, 0.46490E+01_JPRB, 0.45459E+01_JPRB, 0.47718E+01_JPRB, 0.28217E+01_JPRB, 0.14123E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.11072E+01_JPRB, 0.11081E+01_JPRB, 0.10113E+01_JPRB, 0.97771E+00_JPRB, 0.10097E+01_JPRB, 0.91143E+00_JPRB ,&
&  0.91935E+00_JPRB, 0.92314E+00_JPRB, 0.92689E+00_JPRB, 0.90784E+00_JPRB, 0.88081E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.99614E+00_JPRB, 0.10240E+01_JPRB, 0.10729E+01_JPRB, 0.11074E+01_JPRB, 0.11132E+01_JPRB, 0.11385E+01_JPRB, 0.11759E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.14442E+01_JPRB, 0.16253E+01_JPRB, 0.17997E+01_JPRB, 0.18594E+01_JPRB, 0.18792E+01_JPRB, 0.19971E+01_JPRB ,&
&  0.22074E+01_JPRB, 0.25043E+01_JPRB, 0.27882E+01_JPRB, 0.29990E+01_JPRB, 0.30182E+01_JPRB, 0.31066E+01_JPRB, 0.25641E+01_JPRB ,&
&  0.22551E+01_JPRB, 0.23624E+01_JPRB, 0.19255E+01_JPRB, 0.20832E+01_JPRB, 0.22668E+01_JPRB, 0.24024E+01_JPRB, 0.22833E+01_JPRB ,&
&  0.21939E+01_JPRB, 0.21579E+01_JPRB, 0.20848E+01_JPRB, 0.20856E+01_JPRB, 0.21762E+01_JPRB, 0.22667E+01_JPRB, 0.23351E+01_JPRB ,&
&  0.24132E+01_JPRB, 0.24892E+01_JPRB, 0.26751E+01_JPRB, 0.29263E+01_JPRB, 0.32018E+01_JPRB, 0.35238E+01_JPRB, 0.39715E+01_JPRB ,&
&  0.44739E+01_JPRB, 0.49354E+01_JPRB, 0.53892E+01_JPRB, 0.53991E+01_JPRB, 0.57746E+01_JPRB, 0.62323E+01_JPRB, 0.65859E+01_JPRB ,&
&  0.64298E+01_JPRB, 0.61978E+01_JPRB, 0.67000E+01_JPRB, 0.74186E+01_JPRB, 0.79727E+01_JPRB, 0.84009E+01_JPRB, 0.94582E+01_JPRB ,&
&  0.11443E+02_JPRB, 0.12731E+02_JPRB, 0.13129E+02_JPRB, 0.11445E+02_JPRB, 0.10503E+02_JPRB, 0.98494E+01_JPRB, 0.96446E+01_JPRB ,&
&  0.81413E+01_JPRB, 0.63535E+01_JPRB, 0.55402E+01_JPRB, 0.56806E+01_JPRB, 0.60594E+01_JPRB, 0.62207E+01_JPRB, 0.70743E+01_JPRB ,&
&  0.77361E+01_JPRB, 0.68921E+01_JPRB, 0.52960E+01_JPRB, 0.32298E+01_JPRB, 0.19580E+01_JPRB, 0.16484E+01_JPRB, 0.21146E+01_JPRB ,&
&  0.32832E+01_JPRB, 0.35943E+01_JPRB, 0.35458E+01_JPRB, 0.45761E+01_JPRB, 0.34473E+01_JPRB, 0.19314E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.92809E+00_JPRB, 0.96419E+00_JPRB, 0.97156E+00_JPRB, 0.93331E+00_JPRB, 0.95574E+00_JPRB, 0.98408E+00_JPRB, 0.95077E+00_JPRB ,&
&  0.97288E+00_JPRB, 0.10027E+01_JPRB, 0.98108E+00_JPRB, 0.95918E+00_JPRB, 0.97542E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.12082E+01_JPRB, 0.12683E+01_JPRB, 0.13186E+01_JPRB, 0.13499E+01_JPRB, 0.13934E+01_JPRB, 0.14036E+01_JPRB, 0.14706E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.17839E+01_JPRB, 0.19346E+01_JPRB, 0.20268E+01_JPRB, 0.20692E+01_JPRB, 0.21492E+01_JPRB, 0.23121E+01_JPRB ,&
&  0.24847E+01_JPRB, 0.26931E+01_JPRB, 0.28729E+01_JPRB, 0.30712E+01_JPRB, 0.32141E+01_JPRB, 0.35856E+01_JPRB, 0.32009E+01_JPRB ,&
&  0.21740E+01_JPRB, 0.29538E+01_JPRB, 0.21330E+01_JPRB, 0.23462E+01_JPRB, 0.24749E+01_JPRB, 0.26597E+01_JPRB, 0.25225E+01_JPRB ,&
&  0.24451E+01_JPRB, 0.24854E+01_JPRB, 0.25014E+01_JPRB, 0.24777E+01_JPRB, 0.24971E+01_JPRB, 0.26538E+01_JPRB, 0.28095E+01_JPRB ,&
&  0.29479E+01_JPRB, 0.30675E+01_JPRB, 0.32426E+01_JPRB, 0.34663E+01_JPRB, 0.37241E+01_JPRB, 0.40412E+01_JPRB, 0.44145E+01_JPRB ,&
&  0.47351E+01_JPRB, 0.50406E+01_JPRB, 0.51467E+01_JPRB, 0.52454E+01_JPRB, 0.54120E+01_JPRB, 0.57043E+01_JPRB, 0.60397E+01_JPRB ,&
&  0.61552E+01_JPRB, 0.62344E+01_JPRB, 0.68783E+01_JPRB, 0.73260E+01_JPRB, 0.76522E+01_JPRB, 0.84744E+01_JPRB, 0.90329E+01_JPRB ,&
&  0.10638E+02_JPRB, 0.10663E+02_JPRB, 0.10831E+02_JPRB, 0.90032E+01_JPRB, 0.71079E+01_JPRB, 0.62369E+01_JPRB, 0.59655E+01_JPRB ,&
&  0.52396E+01_JPRB, 0.51006E+01_JPRB, 0.49010E+01_JPRB, 0.50892E+01_JPRB, 0.55974E+01_JPRB, 0.61089E+01_JPRB, 0.59331E+01_JPRB ,&
&  0.56575E+01_JPRB, 0.40286E+01_JPRB, 0.30057E+01_JPRB, 0.25445E+01_JPRB, 0.18800E+01_JPRB, 0.16862E+01_JPRB, 0.22836E+01_JPRB ,&
&  0.30822E+01_JPRB, 0.32605E+01_JPRB, 0.35290E+01_JPRB, 0.34852E+01_JPRB, 0.20683E+01_JPRB, 0.12551E+01_JPRB, 0.91402E+00_JPRB ,&
&  0.84984E+00_JPRB, 0.89570E+00_JPRB, 0.91138E+00_JPRB, 0.92127E+00_JPRB, 0.96699E+00_JPRB, 0.94733E+00_JPRB, 0.10024E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.10562E+01_JPRB, 0.10875E+01_JPRB, 0.10979E+01_JPRB, 0.11408E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.14750E+01_JPRB, 0.16176E+01_JPRB, 0.17580E+01_JPRB, 0.18579E+01_JPRB, 0.18352E+01_JPRB, 0.18145E+01_JPRB, 0.18271E+01_JPRB ,&
&  0.18560E+01_JPRB, 0.19221E+01_JPRB, 0.20321E+01_JPRB, 0.21287E+01_JPRB, 0.22128E+01_JPRB, 0.23493E+01_JPRB, 0.25471E+01_JPRB ,&
&  0.27637E+01_JPRB, 0.29845E+01_JPRB, 0.32060E+01_JPRB, 0.34764E+01_JPRB, 0.38201E+01_JPRB, 0.43629E+01_JPRB, 0.42790E+01_JPRB ,&
&  0.30695E+01_JPRB, 0.24455E+01_JPRB, 0.25229E+01_JPRB, 0.24360E+01_JPRB, 0.24080E+01_JPRB, 0.26299E+01_JPRB, 0.27083E+01_JPRB ,&
&  0.26444E+01_JPRB, 0.26528E+01_JPRB, 0.27048E+01_JPRB, 0.27053E+01_JPRB, 0.27036E+01_JPRB, 0.28872E+01_JPRB, 0.30633E+01_JPRB ,&
&  0.32403E+01_JPRB, 0.33649E+01_JPRB, 0.35045E+01_JPRB, 0.36782E+01_JPRB, 0.38781E+01_JPRB, 0.41043E+01_JPRB, 0.43576E+01_JPRB ,&
&  0.45323E+01_JPRB, 0.47290E+01_JPRB, 0.48379E+01_JPRB, 0.49234E+01_JPRB, 0.50534E+01_JPRB, 0.51536E+01_JPRB, 0.52956E+01_JPRB ,&
&  0.55991E+01_JPRB, 0.60581E+01_JPRB, 0.65359E+01_JPRB, 0.68328E+01_JPRB, 0.68898E+01_JPRB, 0.76458E+01_JPRB, 0.80584E+01_JPRB ,&
&  0.84146E+01_JPRB, 0.78821E+01_JPRB, 0.78946E+01_JPRB, 0.66739E+01_JPRB, 0.55969E+01_JPRB, 0.46011E+01_JPRB, 0.42567E+01_JPRB ,&
&  0.39659E+01_JPRB, 0.37947E+01_JPRB, 0.35578E+01_JPRB, 0.36961E+01_JPRB, 0.43720E+01_JPRB, 0.47515E+01_JPRB, 0.48533E+01_JPRB ,&
&  0.40214E+01_JPRB, 0.30304E+01_JPRB, 0.21471E+01_JPRB, 0.21870E+01_JPRB, 0.17923E+01_JPRB, 0.20128E+01_JPRB, 0.23750E+01_JPRB ,&
&  0.26462E+01_JPRB, 0.23352E+01_JPRB, 0.24046E+01_JPRB, 0.22931E+01_JPRB, 0.13956E+01_JPRB, 0.94993E+00_JPRB, 0.77021E+00_JPRB ,&
&  0.72248E+00_JPRB, 0.76997E+00_JPRB, 0.80064E+00_JPRB, 0.81375E+00_JPRB, 0.87563E+00_JPRB, 0.93280E+00_JPRB, 0.10114E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.11018E+01_JPRB, 0.11673E+01_JPRB, 0.12351E+01_JPRB, 0.13408E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.14837E+01_JPRB, 0.15830E+01_JPRB, 0.16844E+01_JPRB, 0.16902E+01_JPRB, 0.17051E+01_JPRB, 0.17086E+01_JPRB, 0.17236E+01_JPRB ,&
&  0.18129E+01_JPRB, 0.19043E+01_JPRB, 0.20177E+01_JPRB, 0.21167E+01_JPRB, 0.22056E+01_JPRB, 0.23501E+01_JPRB, 0.25249E+01_JPRB ,&
&  0.26631E+01_JPRB, 0.27966E+01_JPRB, 0.29373E+01_JPRB, 0.31277E+01_JPRB, 0.33336E+01_JPRB, 0.36231E+01_JPRB, 0.37734E+01_JPRB ,&
&  0.29794E+01_JPRB, 0.18791E+01_JPRB, 0.20813E+01_JPRB, 0.22080E+01_JPRB, 0.22877E+01_JPRB, 0.25507E+01_JPRB, 0.26810E+01_JPRB ,&
&  0.25211E+01_JPRB, 0.25460E+01_JPRB, 0.26592E+01_JPRB, 0.26636E+01_JPRB, 0.25996E+01_JPRB, 0.28009E+01_JPRB, 0.30012E+01_JPRB ,&
&  0.31964E+01_JPRB, 0.33645E+01_JPRB, 0.34997E+01_JPRB, 0.36496E+01_JPRB, 0.37749E+01_JPRB, 0.38948E+01_JPRB, 0.40054E+01_JPRB ,&
&  0.41204E+01_JPRB, 0.41919E+01_JPRB, 0.43026E+01_JPRB, 0.45279E+01_JPRB, 0.45962E+01_JPRB, 0.46260E+01_JPRB, 0.46448E+01_JPRB ,&
&  0.48437E+01_JPRB, 0.51473E+01_JPRB, 0.56571E+01_JPRB, 0.61156E+01_JPRB, 0.64619E+01_JPRB, 0.63929E+01_JPRB, 0.64921E+01_JPRB ,&
&  0.61464E+01_JPRB, 0.54804E+01_JPRB, 0.55709E+01_JPRB, 0.58270E+01_JPRB, 0.41334E+01_JPRB, 0.35704E+01_JPRB, 0.30739E+01_JPRB ,&
&  0.31237E+01_JPRB, 0.27509E+01_JPRB, 0.24718E+01_JPRB, 0.25444E+01_JPRB, 0.32423E+01_JPRB, 0.33819E+01_JPRB, 0.34825E+01_JPRB ,&
&  0.29341E+01_JPRB, 0.19491E+01_JPRB, 0.17434E+01_JPRB, 0.20573E+01_JPRB, 0.20872E+01_JPRB, 0.29506E+01_JPRB, 0.24947E+01_JPRB ,&
&  0.18288E+01_JPRB, 0.16119E+01_JPRB, 0.18140E+01_JPRB, 0.18387E+01_JPRB, 0.12023E+01_JPRB, 0.85091E+00_JPRB, 0.72698E+00_JPRB ,&
&  0.68077E+00_JPRB, 0.68117E+00_JPRB, 0.71895E+00_JPRB, 0.72554E+00_JPRB, 0.77934E+00_JPRB, 0.83267E+00_JPRB, 0.93992E+00_JPRB ,&
&  0.10274E+01_JPRB, 0.11069E+01_JPRB, 0.11741E+01_JPRB, 0.12613E+01_JPRB, 0.13618E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.10827E+01_JPRB, 0.11404E+01_JPRB, 0.11577E+01_JPRB, 0.12714E+01_JPRB, 0.13973E+01_JPRB, 0.14825E+01_JPRB, 0.15569E+01_JPRB ,&
&  0.16691E+01_JPRB, 0.18196E+01_JPRB, 0.19432E+01_JPRB, 0.20343E+01_JPRB, 0.21326E+01_JPRB, 0.22633E+01_JPRB, 0.23073E+01_JPRB ,&
&  0.23039E+01_JPRB, 0.23032E+01_JPRB, 0.23248E+01_JPRB, 0.23473E+01_JPRB, 0.23954E+01_JPRB, 0.24418E+01_JPRB, 0.26699E+01_JPRB ,&
&  0.27205E+01_JPRB, 0.25663E+01_JPRB, 0.23843E+01_JPRB, 0.18476E+01_JPRB, 0.14526E+01_JPRB, 0.18664E+01_JPRB, 0.18883E+01_JPRB ,&
&  0.20954E+01_JPRB, 0.21867E+01_JPRB, 0.21977E+01_JPRB, 0.23369E+01_JPRB, 0.24554E+01_JPRB, 0.26002E+01_JPRB, 0.27814E+01_JPRB ,&
&  0.29229E+01_JPRB, 0.30763E+01_JPRB, 0.32176E+01_JPRB, 0.33469E+01_JPRB, 0.34413E+01_JPRB, 0.34981E+01_JPRB, 0.35161E+01_JPRB ,&
&  0.35455E+01_JPRB, 0.36282E+01_JPRB, 0.37517E+01_JPRB, 0.40406E+01_JPRB, 0.41392E+01_JPRB, 0.42440E+01_JPRB, 0.41877E+01_JPRB ,&
&  0.42567E+01_JPRB, 0.43963E+01_JPRB, 0.47475E+01_JPRB, 0.50346E+01_JPRB, 0.50922E+01_JPRB, 0.47638E+01_JPRB, 0.45870E+01_JPRB ,&
&  0.43230E+01_JPRB, 0.39716E+01_JPRB, 0.40102E+01_JPRB, 0.44142E+01_JPRB, 0.29533E+01_JPRB, 0.29716E+01_JPRB, 0.29706E+01_JPRB ,&
&  0.25004E+01_JPRB, 0.21948E+01_JPRB, 0.19865E+01_JPRB, 0.17969E+01_JPRB, 0.21256E+01_JPRB, 0.24213E+01_JPRB, 0.28386E+01_JPRB ,&
&  0.23047E+01_JPRB, 0.15473E+01_JPRB, 0.14846E+01_JPRB, 0.18877E+01_JPRB, 0.22229E+01_JPRB, 0.29533E+01_JPRB, 0.21107E+01_JPRB ,&
&  0.13744E+01_JPRB, 0.15336E+01_JPRB, 0.17539E+01_JPRB, 0.20625E+01_JPRB, 0.14898E+01_JPRB, 0.82743E+00_JPRB, 0.67687E+00_JPRB ,&
&  0.64644E+00_JPRB, 0.65221E+00_JPRB, 0.66731E+00_JPRB, 0.67459E+00_JPRB, 0.70622E+00_JPRB, 0.74456E+00_JPRB, 0.76955E+00_JPRB ,&
&  0.79398E+00_JPRB, 0.81148E+00_JPRB, 0.85348E+00_JPRB, 0.91841E+00_JPRB, 0.10113E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.73832E+00_JPRB, 0.74455E+00_JPRB, 0.73382E+00_JPRB, 0.75991E+00_JPRB, 0.80830E+00_JPRB, 0.91035E+00_JPRB, 0.10124E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.12738E+01_JPRB, 0.14824E+01_JPRB, 0.16808E+01_JPRB, 0.17611E+01_JPRB, 0.17469E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.13886E+01_JPRB, 0.11702E+01_JPRB, 0.10398E+01_JPRB, 0.99235E+00_JPRB, 0.10460E+01_JPRB, 0.10728E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.12920E+01_JPRB, 0.15229E+01_JPRB, 0.17446E+01_JPRB, 0.22486E+01_JPRB, 0.24830E+01_JPRB, 0.18510E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.22233E+01_JPRB, 0.22955E+01_JPRB, 0.17715E+01_JPRB, 0.17532E+01_JPRB, 0.19388E+01_JPRB, 0.20938E+01_JPRB, 0.22969E+01_JPRB ,&
&  0.24006E+01_JPRB, 0.25100E+01_JPRB, 0.26073E+01_JPRB, 0.26626E+01_JPRB, 0.26837E+01_JPRB, 0.26353E+01_JPRB, 0.25892E+01_JPRB ,&
&  0.25656E+01_JPRB, 0.25531E+01_JPRB, 0.27373E+01_JPRB, 0.29825E+01_JPRB, 0.32454E+01_JPRB, 0.34559E+01_JPRB, 0.35236E+01_JPRB ,&
&  0.35761E+01_JPRB, 0.36599E+01_JPRB, 0.40400E+01_JPRB, 0.41391E+01_JPRB, 0.38947E+01_JPRB, 0.36367E+01_JPRB, 0.33025E+01_JPRB ,&
&  0.29553E+01_JPRB, 0.24926E+01_JPRB, 0.19338E+01_JPRB, 0.29058E+01_JPRB, 0.33120E+01_JPRB, 0.28106E+01_JPRB, 0.22626E+01_JPRB ,&
&  0.20208E+01_JPRB, 0.18377E+01_JPRB, 0.16496E+01_JPRB, 0.15565E+01_JPRB, 0.16748E+01_JPRB, 0.23637E+01_JPRB, 0.23903E+01_JPRB ,&
&  0.19731E+01_JPRB, 0.12724E+01_JPRB, 0.11091E+01_JPRB, 0.13021E+01_JPRB, 0.15606E+01_JPRB, 0.19792E+01_JPRB, 0.13987E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.14674E+01_JPRB, 0.16834E+01_JPRB, 0.21464E+01_JPRB, 0.23158E+01_JPRB, 0.69043E+00_JPRB, 0.63767E+00_JPRB ,&
&  0.62405E+00_JPRB, 0.62471E+00_JPRB, 0.64550E+00_JPRB, 0.64161E+00_JPRB, 0.64590E+00_JPRB, 0.64230E+00_JPRB, 0.63321E+00_JPRB ,&
&  0.63590E+00_JPRB, 0.65499E+00_JPRB, 0.67763E+00_JPRB, 0.69084E+00_JPRB, 0.71836E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.69558E+00_JPRB, 0.69664E+00_JPRB, 0.70161E+00_JPRB, 0.71024E+00_JPRB, 0.71378E+00_JPRB, 0.71640E+00_JPRB, 0.73786E+00_JPRB ,&
&  0.77587E+00_JPRB, 0.83916E+00_JPRB, 0.93102E+00_JPRB, 0.99343E+00_JPRB, 0.97638E+00_JPRB, 0.90621E+00_JPRB, 0.86920E+00_JPRB ,&
&  0.85538E+00_JPRB, 0.84494E+00_JPRB, 0.87659E+00_JPRB, 0.93036E+00_JPRB, 0.99857E+00_JPRB, 0.10176E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.11199E+01_JPRB, 0.12243E+01_JPRB, 0.13173E+01_JPRB, 0.19795E+01_JPRB, 0.18995E+01_JPRB, 0.16027E+01_JPRB ,&
&  0.17769E+01_JPRB, 0.18237E+01_JPRB, 0.16884E+01_JPRB, 0.14384E+01_JPRB, 0.13864E+01_JPRB, 0.13652E+01_JPRB, 0.14080E+01_JPRB ,&
&  0.14054E+01_JPRB, 0.15208E+01_JPRB, 0.16206E+01_JPRB, 0.17318E+01_JPRB, 0.18122E+01_JPRB, 0.18410E+01_JPRB, 0.19143E+01_JPRB ,&
&  0.19835E+01_JPRB, 0.20586E+01_JPRB, 0.21565E+01_JPRB, 0.19776E+01_JPRB, 0.21851E+01_JPRB, 0.24344E+01_JPRB, 0.26191E+01_JPRB ,&
&  0.27103E+01_JPRB, 0.27644E+01_JPRB, 0.30180E+01_JPRB, 0.29487E+01_JPRB, 0.28207E+01_JPRB, 0.25676E+01_JPRB, 0.23364E+01_JPRB ,&
&  0.21523E+01_JPRB, 0.15809E+01_JPRB, 0.10259E+01_JPRB, 0.15027E+01_JPRB, 0.18109E+01_JPRB, 0.15487E+01_JPRB, 0.16333E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.14442E+01_JPRB, 0.13502E+01_JPRB, 0.13342E+01_JPRB, 0.13013E+01_JPRB, 0.16279E+01_JPRB, 0.18351E+01_JPRB ,&
&  0.14997E+01_JPRB, 0.10490E+01_JPRB, 0.88889E+00_JPRB, 0.92725E+00_JPRB, 0.10670E+01_JPRB, 0.12767E+01_JPRB, 0.13696E+01_JPRB ,&
&  0.13547E+01_JPRB, 0.11515E+01_JPRB, 0.13999E+01_JPRB, 0.17512E+01_JPRB, 0.10025E+01_JPRB, 0.57565E+00_JPRB, 0.54654E+00_JPRB ,&
&  0.54912E+00_JPRB, 0.56229E+00_JPRB, 0.58567E+00_JPRB, 0.60116E+00_JPRB, 0.61246E+00_JPRB, 0.62196E+00_JPRB, 0.63824E+00_JPRB ,&
&  0.65043E+00_JPRB, 0.66505E+00_JPRB, 0.67257E+00_JPRB, 0.68129E+00_JPRB, 0.69048E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.89292E+00_JPRB, 0.91099E+00_JPRB, 0.93670E+00_JPRB, 0.96281E+00_JPRB, 0.97971E+00_JPRB, 0.10048E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.10666E+01_JPRB, 0.11163E+01_JPRB, 0.11506E+01_JPRB, 0.11637E+01_JPRB, 0.11784E+01_JPRB, 0.11977E+01_JPRB, 0.12026E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.12471E+01_JPRB, 0.13161E+01_JPRB, 0.13475E+01_JPRB, 0.13890E+01_JPRB, 0.14419E+01_JPRB, 0.14495E+01_JPRB ,&
&  0.14358E+01_JPRB, 0.14232E+01_JPRB, 0.14357E+01_JPRB, 0.14294E+01_JPRB, 0.15145E+01_JPRB, 0.15341E+01_JPRB, 0.14770E+01_JPRB ,&
&  0.14296E+01_JPRB, 0.14165E+01_JPRB, 0.15827E+01_JPRB, 0.12817E+01_JPRB, 0.12580E+01_JPRB, 0.12803E+01_JPRB, 0.11864E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.12635E+01_JPRB, 0.13524E+01_JPRB, 0.14436E+01_JPRB, 0.15682E+01_JPRB, 0.16373E+01_JPRB, 0.16933E+01_JPRB ,&
&  0.17600E+01_JPRB, 0.18320E+01_JPRB, 0.18744E+01_JPRB, 0.18481E+01_JPRB, 0.18636E+01_JPRB, 0.22023E+01_JPRB, 0.23544E+01_JPRB ,&
&  0.24222E+01_JPRB, 0.25001E+01_JPRB, 0.21082E+01_JPRB, 0.19675E+01_JPRB, 0.20766E+01_JPRB, 0.18469E+01_JPRB, 0.17549E+01_JPRB ,&
&  0.19163E+01_JPRB, 0.16409E+01_JPRB, 0.85569E+00_JPRB, 0.11136E+01_JPRB, 0.12877E+01_JPRB, 0.13290E+01_JPRB, 0.13382E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.10965E+01_JPRB, 0.10344E+01_JPRB, 0.11039E+01_JPRB, 0.11541E+01_JPRB, 0.11154E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.10351E+01_JPRB, 0.90347E+00_JPRB, 0.83713E+00_JPRB, 0.97523E+00_JPRB, 0.13788E+01_JPRB, 0.21375E+01_JPRB, 0.19334E+01_JPRB ,&
&  0.13994E+01_JPRB, 0.12239E+01_JPRB, 0.19351E+01_JPRB, 0.26614E+01_JPRB, 0.10311E+01_JPRB, 0.68693E+00_JPRB, 0.55508E+00_JPRB ,&
&  0.58815E+00_JPRB, 0.60445E+00_JPRB, 0.61655E+00_JPRB, 0.64217E+00_JPRB, 0.68099E+00_JPRB, 0.69976E+00_JPRB, 0.70762E+00_JPRB ,&
&  0.73241E+00_JPRB, 0.76945E+00_JPRB, 0.79754E+00_JPRB, 0.83188E+00_JPRB, 0.86269E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.10633E+01_JPRB, 0.10907E+01_JPRB, 0.11108E+01_JPRB, 0.11322E+01_JPRB, 0.11591E+01_JPRB, 0.11859E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.12298E+01_JPRB, 0.12517E+01_JPRB, 0.12626E+01_JPRB, 0.12846E+01_JPRB, 0.12926E+01_JPRB, 0.13550E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.14376E+01_JPRB, 0.14720E+01_JPRB, 0.14856E+01_JPRB, 0.14919E+01_JPRB, 0.15096E+01_JPRB, 0.15032E+01_JPRB, 0.14940E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.15725E+01_JPRB, 0.16280E+01_JPRB, 0.16796E+01_JPRB, 0.17028E+01_JPRB, 0.17952E+01_JPRB, 0.19199E+01_JPRB ,&
&  0.15082E+01_JPRB, 0.16079E+01_JPRB, 0.15955E+01_JPRB, 0.13851E+01_JPRB, 0.14104E+01_JPRB, 0.14495E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.14368E+01_JPRB, 0.14787E+01_JPRB, 0.15239E+01_JPRB, 0.15995E+01_JPRB, 0.16841E+01_JPRB, 0.17469E+01_JPRB ,&
&  0.18449E+01_JPRB, 0.19222E+01_JPRB, 0.19837E+01_JPRB, 0.20841E+01_JPRB, 0.20471E+01_JPRB, 0.24257E+01_JPRB, 0.25550E+01_JPRB ,&
&  0.25835E+01_JPRB, 0.25092E+01_JPRB, 0.19337E+01_JPRB, 0.18488E+01_JPRB, 0.19272E+01_JPRB, 0.18380E+01_JPRB, 0.17319E+01_JPRB ,&
&  0.18918E+01_JPRB, 0.17754E+01_JPRB, 0.12096E+01_JPRB, 0.12746E+01_JPRB, 0.12736E+01_JPRB, 0.12551E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.88992E+00_JPRB, 0.84465E+00_JPRB, 0.87341E+00_JPRB, 0.88420E+00_JPRB, 0.85775E+00_JPRB, 0.87679E+00_JPRB ,&
&  0.87861E+00_JPRB, 0.86340E+00_JPRB, 0.89263E+00_JPRB, 0.94135E+00_JPRB, 0.14848E+01_JPRB, 0.18380E+01_JPRB, 0.25763E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.11551E+01_JPRB, 0.10356E+01_JPRB, 0.14382E+01_JPRB, 0.90230E+00_JPRB, 0.60501E+00_JPRB, 0.60225E+00_JPRB ,&
&  0.66971E+00_JPRB, 0.70154E+00_JPRB, 0.71269E+00_JPRB, 0.73291E+00_JPRB, 0.78231E+00_JPRB, 0.81252E+00_JPRB, 0.84109E+00_JPRB ,&
&  0.85342E+00_JPRB, 0.89859E+00_JPRB, 0.95791E+00_JPRB, 0.10029E+01_JPRB, 0.10258E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.11435E+01_JPRB, 0.11783E+01_JPRB, 0.12086E+01_JPRB, 0.12370E+01_JPRB, 0.12641E+01_JPRB, 0.12932E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.13425E+01_JPRB, 0.13598E+01_JPRB, 0.13701E+01_JPRB, 0.13865E+01_JPRB, 0.14000E+01_JPRB, 0.14488E+01_JPRB, 0.14669E+01_JPRB ,&
&  0.15179E+01_JPRB, 0.15404E+01_JPRB, 0.15385E+01_JPRB, 0.15692E+01_JPRB, 0.15556E+01_JPRB, 0.15286E+01_JPRB, 0.15239E+01_JPRB ,&
&  0.15343E+01_JPRB, 0.15364E+01_JPRB, 0.15403E+01_JPRB, 0.15519E+01_JPRB, 0.15634E+01_JPRB, 0.15379E+01_JPRB, 0.16552E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.18065E+01_JPRB, 0.18554E+01_JPRB, 0.16701E+01_JPRB, 0.16053E+01_JPRB, 0.14958E+01_JPRB, 0.14883E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.15091E+01_JPRB, 0.15341E+01_JPRB, 0.15845E+01_JPRB, 0.16624E+01_JPRB, 0.17585E+01_JPRB, 0.18440E+01_JPRB ,&
&  0.19586E+01_JPRB, 0.20653E+01_JPRB, 0.21523E+01_JPRB, 0.22344E+01_JPRB, 0.23885E+01_JPRB, 0.25040E+01_JPRB, 0.25495E+01_JPRB ,&
&  0.25542E+01_JPRB, 0.25818E+01_JPRB, 0.20300E+01_JPRB, 0.20008E+01_JPRB, 0.21699E+01_JPRB, 0.20806E+01_JPRB, 0.18520E+01_JPRB ,&
&  0.18731E+01_JPRB, 0.17055E+01_JPRB, 0.15241E+01_JPRB, 0.13535E+01_JPRB, 0.12418E+01_JPRB, 0.11408E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.84095E+00_JPRB, 0.70853E+00_JPRB, 0.67792E+00_JPRB, 0.71979E+00_JPRB, 0.73403E+00_JPRB, 0.73092E+00_JPRB, 0.70725E+00_JPRB ,&
&  0.71790E+00_JPRB, 0.75126E+00_JPRB, 0.82617E+00_JPRB, 0.98983E+00_JPRB, 0.12281E+01_JPRB, 0.23395E+01_JPRB, 0.25303E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.10490E+01_JPRB, 0.69008E+00_JPRB, 0.82414E+00_JPRB, 0.83192E+00_JPRB, 0.71332E+00_JPRB, 0.73296E+00_JPRB ,&
&  0.84183E+00_JPRB, 0.90227E+00_JPRB, 0.92797E+00_JPRB, 0.86940E+00_JPRB, 0.89123E+00_JPRB, 0.88199E+00_JPRB, 0.89246E+00_JPRB ,&
&  0.92048E+00_JPRB, 0.96277E+00_JPRB, 0.10139E+01_JPRB, 0.10622E+01_JPRB, 0.11042E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.11775E+01_JPRB, 0.12135E+01_JPRB, 0.12507E+01_JPRB, 0.12914E+01_JPRB, 0.13326E+01_JPRB, 0.13646E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.14303E+01_JPRB, 0.14549E+01_JPRB, 0.14649E+01_JPRB, 0.14737E+01_JPRB, 0.14856E+01_JPRB, 0.15306E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.15611E+01_JPRB, 0.15842E+01_JPRB, 0.15993E+01_JPRB, 0.16161E+01_JPRB, 0.16067E+01_JPRB, 0.15702E+01_JPRB, 0.15848E+01_JPRB ,&
&  0.15824E+01_JPRB, 0.15906E+01_JPRB, 0.16004E+01_JPRB, 0.16413E+01_JPRB, 0.16992E+01_JPRB, 0.18411E+01_JPRB, 0.16063E+01_JPRB ,&
&  0.17868E+01_JPRB, 0.19450E+01_JPRB, 0.19400E+01_JPRB, 0.18014E+01_JPRB, 0.16484E+01_JPRB, 0.16101E+01_JPRB, 0.16027E+01_JPRB ,&
&  0.16191E+01_JPRB, 0.15923E+01_JPRB, 0.16038E+01_JPRB, 0.16269E+01_JPRB, 0.16776E+01_JPRB, 0.17729E+01_JPRB, 0.18486E+01_JPRB ,&
&  0.19608E+01_JPRB, 0.21343E+01_JPRB, 0.23043E+01_JPRB, 0.24578E+01_JPRB, 0.26533E+01_JPRB, 0.28538E+01_JPRB, 0.29737E+01_JPRB ,&
&  0.30300E+01_JPRB, 0.30972E+01_JPRB, 0.28259E+01_JPRB, 0.28712E+01_JPRB, 0.30184E+01_JPRB, 0.27227E+01_JPRB, 0.22710E+01_JPRB ,&
&  0.20610E+01_JPRB, 0.16032E+01_JPRB, 0.13475E+01_JPRB, 0.14291E+01_JPRB, 0.12574E+01_JPRB, 0.10872E+01_JPRB, 0.91273E+00_JPRB ,&
&  0.72472E+00_JPRB, 0.63117E+00_JPRB, 0.62853E+00_JPRB, 0.67342E+00_JPRB, 0.67088E+00_JPRB, 0.68140E+00_JPRB, 0.71190E+00_JPRB ,&
&  0.76412E+00_JPRB, 0.82349E+00_JPRB, 0.91451E+00_JPRB, 0.10776E+01_JPRB, 0.12922E+01_JPRB, 0.13102E+01_JPRB, 0.14613E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.88494E+00_JPRB, 0.91429E+00_JPRB, 0.91807E+00_JPRB, 0.94416E+00_JPRB, 0.91508E+00_JPRB, 0.10141E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.13007E+01_JPRB, 0.13132E+01_JPRB, 0.11016E+01_JPRB, 0.99794E+00_JPRB, 0.95630E+00_JPRB, 0.95763E+00_JPRB ,&
&  0.97555E+00_JPRB, 0.10038E+01_JPRB, 0.10473E+01_JPRB, 0.10968E+01_JPRB, 0.11403E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.11772E+01_JPRB, 0.12155E+01_JPRB, 0.12552E+01_JPRB, 0.12939E+01_JPRB, 0.13252E+01_JPRB, 0.13511E+01_JPRB, 0.13720E+01_JPRB ,&
&  0.14024E+01_JPRB, 0.14324E+01_JPRB, 0.14554E+01_JPRB, 0.14688E+01_JPRB, 0.14828E+01_JPRB, 0.15122E+01_JPRB, 0.15484E+01_JPRB ,&
&  0.15920E+01_JPRB, 0.16228E+01_JPRB, 0.16775E+01_JPRB, 0.16768E+01_JPRB, 0.16759E+01_JPRB, 0.16946E+01_JPRB, 0.17453E+01_JPRB ,&
&  0.17706E+01_JPRB, 0.17724E+01_JPRB, 0.17817E+01_JPRB, 0.18367E+01_JPRB, 0.18953E+01_JPRB, 0.19959E+01_JPRB, 0.17859E+01_JPRB ,&
&  0.19608E+01_JPRB, 0.20889E+01_JPRB, 0.20311E+01_JPRB, 0.19422E+01_JPRB, 0.17881E+01_JPRB, 0.17393E+01_JPRB, 0.17239E+01_JPRB ,&
&  0.17089E+01_JPRB, 0.16994E+01_JPRB, 0.16837E+01_JPRB, 0.16909E+01_JPRB, 0.17205E+01_JPRB, 0.17769E+01_JPRB, 0.18655E+01_JPRB ,&
&  0.19593E+01_JPRB, 0.20948E+01_JPRB, 0.23046E+01_JPRB, 0.25151E+01_JPRB, 0.27966E+01_JPRB, 0.30735E+01_JPRB, 0.32843E+01_JPRB ,&
&  0.34890E+01_JPRB, 0.36941E+01_JPRB, 0.34923E+01_JPRB, 0.36180E+01_JPRB, 0.38380E+01_JPRB, 0.33794E+01_JPRB, 0.25826E+01_JPRB ,&
&  0.17731E+01_JPRB, 0.14850E+01_JPRB, 0.14631E+01_JPRB, 0.14775E+01_JPRB, 0.12539E+01_JPRB, 0.10655E+01_JPRB, 0.92743E+00_JPRB ,&
&  0.76903E+00_JPRB, 0.65831E+00_JPRB, 0.66615E+00_JPRB, 0.66139E+00_JPRB, 0.65565E+00_JPRB, 0.69237E+00_JPRB, 0.74243E+00_JPRB ,&
&  0.80718E+00_JPRB, 0.91317E+00_JPRB, 0.10296E+01_JPRB, 0.11607E+01_JPRB, 0.12530E+01_JPRB, 0.12150E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.11531E+01_JPRB, 0.11149E+01_JPRB, 0.11089E+01_JPRB, 0.86929E+00_JPRB, 0.11304E+01_JPRB, 0.12025E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.17060E+01_JPRB, 0.20710E+01_JPRB, 0.16280E+01_JPRB, 0.13126E+01_JPRB, 0.10075E+01_JPRB, 0.96351E+00_JPRB ,&
&  0.97982E+00_JPRB, 0.10042E+01_JPRB, 0.10429E+01_JPRB, 0.10906E+01_JPRB, 0.11350E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.11497E+01_JPRB, 0.11931E+01_JPRB, 0.12394E+01_JPRB, 0.12748E+01_JPRB, 0.12993E+01_JPRB, 0.12955E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.13007E+01_JPRB, 0.13200E+01_JPRB, 0.13466E+01_JPRB, 0.13752E+01_JPRB, 0.13968E+01_JPRB, 0.14186E+01_JPRB, 0.14515E+01_JPRB ,&
&  0.14913E+01_JPRB, 0.15404E+01_JPRB, 0.16057E+01_JPRB, 0.16646E+01_JPRB, 0.17325E+01_JPRB, 0.17952E+01_JPRB, 0.18691E+01_JPRB ,&
&  0.19357E+01_JPRB, 0.20162E+01_JPRB, 0.20485E+01_JPRB, 0.21054E+01_JPRB, 0.21411E+01_JPRB, 0.22238E+01_JPRB, 0.18037E+01_JPRB ,&
&  0.20303E+01_JPRB, 0.21131E+01_JPRB, 0.20941E+01_JPRB, 0.19921E+01_JPRB, 0.18396E+01_JPRB, 0.17656E+01_JPRB, 0.17139E+01_JPRB ,&
&  0.16951E+01_JPRB, 0.16521E+01_JPRB, 0.15703E+01_JPRB, 0.16466E+01_JPRB, 0.17221E+01_JPRB, 0.17312E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.19307E+01_JPRB, 0.20758E+01_JPRB, 0.22542E+01_JPRB, 0.25147E+01_JPRB, 0.28401E+01_JPRB, 0.31499E+01_JPRB, 0.34641E+01_JPRB ,&
&  0.37895E+01_JPRB, 0.41337E+01_JPRB, 0.44009E+01_JPRB, 0.40216E+01_JPRB, 0.40192E+01_JPRB, 0.35324E+01_JPRB, 0.25820E+01_JPRB ,&
&  0.16881E+01_JPRB, 0.14421E+01_JPRB, 0.14621E+01_JPRB, 0.14899E+01_JPRB, 0.12811E+01_JPRB, 0.10699E+01_JPRB, 0.92234E+00_JPRB ,&
&  0.78172E+00_JPRB, 0.71036E+00_JPRB, 0.69445E+00_JPRB, 0.66881E+00_JPRB, 0.66475E+00_JPRB, 0.70644E+00_JPRB, 0.76146E+00_JPRB ,&
&  0.83116E+00_JPRB, 0.92956E+00_JPRB, 0.10507E+01_JPRB, 0.11823E+01_JPRB, 0.13134E+01_JPRB, 0.13599E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.11391E+01_JPRB, 0.11740E+01_JPRB, 0.12683E+01_JPRB, 0.12989E+01_JPRB, 0.15260E+01_JPRB, 0.17629E+01_JPRB ,&
&  0.19857E+01_JPRB, 0.18384E+01_JPRB, 0.21488E+01_JPRB, 0.29617E+01_JPRB, 0.27240E+01_JPRB, 0.22382E+01_JPRB, 0.91978E+00_JPRB ,&
&  0.92511E+00_JPRB, 0.93225E+00_JPRB, 0.97791E+00_JPRB, 0.10366E+01_JPRB, 0.11039E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.94055E+00_JPRB, 0.10033E+01_JPRB, 0.10677E+01_JPRB, 0.11519E+01_JPRB, 0.12257E+01_JPRB, 0.12354E+01_JPRB, 0.12190E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.12105E+01_JPRB, 0.12289E+01_JPRB, 0.12568E+01_JPRB, 0.12817E+01_JPRB, 0.13014E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.13628E+01_JPRB, 0.14059E+01_JPRB, 0.14566E+01_JPRB, 0.15129E+01_JPRB, 0.15692E+01_JPRB, 0.16502E+01_JPRB, 0.17461E+01_JPRB ,&
&  0.18505E+01_JPRB, 0.19592E+01_JPRB, 0.20481E+01_JPRB, 0.21381E+01_JPRB, 0.22172E+01_JPRB, 0.22159E+01_JPRB, 0.16222E+01_JPRB ,&
&  0.20005E+01_JPRB, 0.20522E+01_JPRB, 0.20943E+01_JPRB, 0.19976E+01_JPRB, 0.18239E+01_JPRB, 0.17203E+01_JPRB, 0.16831E+01_JPRB ,&
&  0.16444E+01_JPRB, 0.15851E+01_JPRB, 0.15141E+01_JPRB, 0.14750E+01_JPRB, 0.16234E+01_JPRB, 0.16360E+01_JPRB, 0.17531E+01_JPRB ,&
&  0.18731E+01_JPRB, 0.20136E+01_JPRB, 0.21800E+01_JPRB, 0.24392E+01_JPRB, 0.27786E+01_JPRB, 0.31282E+01_JPRB, 0.35100E+01_JPRB ,&
&  0.38954E+01_JPRB, 0.42412E+01_JPRB, 0.44413E+01_JPRB, 0.38963E+01_JPRB, 0.36962E+01_JPRB, 0.33956E+01_JPRB, 0.22126E+01_JPRB ,&
&  0.16937E+01_JPRB, 0.15856E+01_JPRB, 0.16444E+01_JPRB, 0.15608E+01_JPRB, 0.13540E+01_JPRB, 0.11770E+01_JPRB, 0.10045E+01_JPRB ,&
&  0.80828E+00_JPRB, 0.71591E+00_JPRB, 0.69886E+00_JPRB, 0.68748E+00_JPRB, 0.69090E+00_JPRB, 0.73623E+00_JPRB, 0.79386E+00_JPRB ,&
&  0.85668E+00_JPRB, 0.93394E+00_JPRB, 0.10019E+01_JPRB, 0.10654E+01_JPRB, 0.11374E+01_JPRB, 0.12614E+01_JPRB, 0.13391E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.10959E+01_JPRB, 0.11001E+01_JPRB, 0.11185E+01_JPRB, 0.11485E+01_JPRB, 0.12175E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.14181E+01_JPRB, 0.14890E+01_JPRB, 0.12592E+01_JPRB, 0.10527E+01_JPRB, 0.13540E+01_JPRB, 0.14501E+01_JPRB, 0.79205E+00_JPRB ,&
&  0.76394E+00_JPRB, 0.78567E+00_JPRB, 0.80590E+00_JPRB, 0.84890E+00_JPRB, 0.89314E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.75120E+00_JPRB, 0.77310E+00_JPRB, 0.85858E+00_JPRB, 0.88872E+00_JPRB, 0.97629E+00_JPRB, 0.10188E+01_JPRB, 0.10666E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.10935E+01_JPRB, 0.11023E+01_JPRB, 0.11156E+01_JPRB, 0.11323E+01_JPRB, 0.11558E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.12053E+01_JPRB, 0.12400E+01_JPRB, 0.12805E+01_JPRB, 0.13235E+01_JPRB, 0.13801E+01_JPRB, 0.14482E+01_JPRB, 0.15409E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.17978E+01_JPRB, 0.19454E+01_JPRB, 0.20993E+01_JPRB, 0.22623E+01_JPRB, 0.23870E+01_JPRB, 0.24335E+01_JPRB ,&
&  0.16777E+01_JPRB, 0.20540E+01_JPRB, 0.20284E+01_JPRB, 0.19318E+01_JPRB, 0.18038E+01_JPRB, 0.17203E+01_JPRB, 0.16891E+01_JPRB ,&
&  0.16547E+01_JPRB, 0.15445E+01_JPRB, 0.15117E+01_JPRB, 0.14761E+01_JPRB, 0.14942E+01_JPRB, 0.15104E+01_JPRB, 0.16227E+01_JPRB ,&
&  0.17354E+01_JPRB, 0.18886E+01_JPRB, 0.20858E+01_JPRB, 0.22895E+01_JPRB, 0.26102E+01_JPRB, 0.30358E+01_JPRB, 0.34785E+01_JPRB ,&
&  0.39451E+01_JPRB, 0.44345E+01_JPRB, 0.47272E+01_JPRB, 0.41975E+01_JPRB, 0.36578E+01_JPRB, 0.34316E+01_JPRB, 0.24527E+01_JPRB ,&
&  0.19381E+01_JPRB, 0.16590E+01_JPRB, 0.17669E+01_JPRB, 0.17456E+01_JPRB, 0.14976E+01_JPRB, 0.13393E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.97405E+00_JPRB, 0.79996E+00_JPRB, 0.74186E+00_JPRB, 0.73696E+00_JPRB, 0.74936E+00_JPRB, 0.78429E+00_JPRB, 0.81542E+00_JPRB ,&
&  0.85352E+00_JPRB, 0.89561E+00_JPRB, 0.93174E+00_JPRB, 0.97015E+00_JPRB, 0.98985E+00_JPRB, 0.10091E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.10541E+01_JPRB, 0.10514E+01_JPRB, 0.10619E+01_JPRB, 0.10480E+01_JPRB, 0.10542E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.11064E+01_JPRB, 0.10908E+01_JPRB, 0.10533E+01_JPRB, 0.97926E+00_JPRB, 0.89717E+00_JPRB, 0.73840E+00_JPRB ,&
&  0.71434E+00_JPRB, 0.69642E+00_JPRB, 0.70866E+00_JPRB, 0.71873E+00_JPRB, 0.73999E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.65074E+00_JPRB, 0.67120E+00_JPRB, 0.69245E+00_JPRB, 0.71815E+00_JPRB, 0.75841E+00_JPRB, 0.80707E+00_JPRB, 0.87074E+00_JPRB ,&
&  0.93570E+00_JPRB, 0.95919E+00_JPRB, 0.96629E+00_JPRB, 0.97104E+00_JPRB, 0.98079E+00_JPRB, 0.10030E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.10837E+01_JPRB, 0.11077E+01_JPRB, 0.11356E+01_JPRB, 0.11706E+01_JPRB, 0.12231E+01_JPRB, 0.12894E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.15078E+01_JPRB, 0.16894E+01_JPRB, 0.18988E+01_JPRB, 0.21260E+01_JPRB, 0.23110E+01_JPRB, 0.23922E+01_JPRB ,&
&  0.13672E+01_JPRB, 0.17524E+01_JPRB, 0.20928E+01_JPRB, 0.19733E+01_JPRB, 0.18367E+01_JPRB, 0.17649E+01_JPRB, 0.17365E+01_JPRB ,&
&  0.16451E+01_JPRB, 0.15194E+01_JPRB, 0.14724E+01_JPRB, 0.15186E+01_JPRB, 0.14050E+01_JPRB, 0.13986E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.15455E+01_JPRB, 0.16404E+01_JPRB, 0.17951E+01_JPRB, 0.20041E+01_JPRB, 0.22999E+01_JPRB, 0.27428E+01_JPRB, 0.32485E+01_JPRB ,&
&  0.37987E+01_JPRB, 0.44364E+01_JPRB, 0.50125E+01_JPRB, 0.45625E+01_JPRB, 0.45066E+01_JPRB, 0.46373E+01_JPRB, 0.41805E+01_JPRB ,&
&  0.26975E+01_JPRB, 0.19874E+01_JPRB, 0.20473E+01_JPRB, 0.20266E+01_JPRB, 0.17016E+01_JPRB, 0.15400E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.12240E+01_JPRB, 0.10458E+01_JPRB, 0.85944E+00_JPRB, 0.80920E+00_JPRB, 0.81527E+00_JPRB, 0.84207E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.88314E+00_JPRB, 0.88902E+00_JPRB, 0.90441E+00_JPRB, 0.92453E+00_JPRB, 0.96085E+00_JPRB, 0.97707E+00_JPRB, 0.98015E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.10151E+01_JPRB, 0.10337E+01_JPRB, 0.10606E+01_JPRB, 0.10855E+01_JPRB, 0.10964E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.10722E+01_JPRB, 0.10890E+01_JPRB, 0.10775E+01_JPRB, 0.10307E+01_JPRB, 0.92989E+00_JPRB, 0.84571E+00_JPRB ,&
&  0.75833E+00_JPRB, 0.74086E+00_JPRB, 0.70847E+00_JPRB, 0.68119E+00_JPRB, 0.65856E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.64175E+00_JPRB, 0.63688E+00_JPRB, 0.64293E+00_JPRB, 0.64901E+00_JPRB, 0.66326E+00_JPRB, 0.69352E+00_JPRB, 0.73020E+00_JPRB ,&
&  0.76116E+00_JPRB, 0.81426E+00_JPRB, 0.85169E+00_JPRB, 0.85369E+00_JPRB, 0.84870E+00_JPRB, 0.85933E+00_JPRB, 0.88263E+00_JPRB ,&
&  0.91399E+00_JPRB, 0.94127E+00_JPRB, 0.96025E+00_JPRB, 0.97724E+00_JPRB, 0.99990E+00_JPRB, 0.10236E+01_JPRB, 0.10547E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.11855E+01_JPRB, 0.13157E+01_JPRB, 0.15186E+01_JPRB, 0.17967E+01_JPRB, 0.21159E+01_JPRB, 0.23252E+01_JPRB ,&
&  0.16844E+01_JPRB, 0.86181E+00_JPRB, 0.18109E+01_JPRB, 0.21460E+01_JPRB, 0.19498E+01_JPRB, 0.18936E+01_JPRB, 0.17942E+01_JPRB ,&
&  0.16835E+01_JPRB, 0.16438E+01_JPRB, 0.15560E+01_JPRB, 0.15091E+01_JPRB, 0.13524E+01_JPRB, 0.13079E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.13364E+01_JPRB, 0.13908E+01_JPRB, 0.14969E+01_JPRB, 0.16555E+01_JPRB, 0.18820E+01_JPRB, 0.22459E+01_JPRB, 0.28007E+01_JPRB ,&
&  0.34020E+01_JPRB, 0.41261E+01_JPRB, 0.50722E+01_JPRB, 0.49837E+01_JPRB, 0.52176E+01_JPRB, 0.47560E+01_JPRB, 0.39498E+01_JPRB ,&
&  0.30358E+01_JPRB, 0.23928E+01_JPRB, 0.23889E+01_JPRB, 0.23954E+01_JPRB, 0.20031E+01_JPRB, 0.16756E+01_JPRB, 0.16524E+01_JPRB ,&
&  0.14146E+01_JPRB, 0.13060E+01_JPRB, 0.11006E+01_JPRB, 0.95360E+00_JPRB, 0.90506E+00_JPRB, 0.91942E+00_JPRB, 0.93875E+00_JPRB ,&
&  0.93908E+00_JPRB, 0.93423E+00_JPRB, 0.93840E+00_JPRB, 0.95092E+00_JPRB, 0.97275E+00_JPRB, 0.98739E+00_JPRB, 0.99191E+00_JPRB ,&
&  0.10137E+01_JPRB, 0.10376E+01_JPRB, 0.10613E+01_JPRB, 0.10850E+01_JPRB, 0.11025E+01_JPRB, 0.10856E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.10926E+01_JPRB, 0.10913E+01_JPRB, 0.10833E+01_JPRB, 0.10888E+01_JPRB, 0.10682E+01_JPRB, 0.10082E+01_JPRB, 0.98264E+00_JPRB ,&
&  0.91140E+00_JPRB, 0.83780E+00_JPRB, 0.74852E+00_JPRB, 0.69274E+00_JPRB, 0.67066E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.74849E+00_JPRB, 0.71194E+00_JPRB, 0.66806E+00_JPRB, 0.65579E+00_JPRB, 0.65398E+00_JPRB, 0.65096E+00_JPRB, 0.66929E+00_JPRB ,&
&  0.68581E+00_JPRB, 0.71111E+00_JPRB, 0.71886E+00_JPRB, 0.74360E+00_JPRB, 0.75576E+00_JPRB, 0.75956E+00_JPRB, 0.77433E+00_JPRB ,&
&  0.78689E+00_JPRB, 0.80162E+00_JPRB, 0.81733E+00_JPRB, 0.83660E+00_JPRB, 0.85663E+00_JPRB, 0.86548E+00_JPRB, 0.87717E+00_JPRB ,&
&  0.90361E+00_JPRB, 0.93864E+00_JPRB, 0.99373E+00_JPRB, 0.11143E+01_JPRB, 0.13246E+01_JPRB, 0.16152E+01_JPRB, 0.19518E+01_JPRB ,&
&  0.22547E+01_JPRB, 0.16162E+01_JPRB, 0.11389E+01_JPRB, 0.19696E+01_JPRB, 0.21794E+01_JPRB, 0.21475E+01_JPRB, 0.18570E+01_JPRB ,&
&  0.18717E+01_JPRB, 0.17985E+01_JPRB, 0.17073E+01_JPRB, 0.15875E+01_JPRB, 0.13813E+01_JPRB, 0.12794E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.11712E+01_JPRB, 0.12295E+01_JPRB, 0.13726E+01_JPRB, 0.15417E+01_JPRB, 0.17840E+01_JPRB, 0.22426E+01_JPRB ,&
&  0.28292E+01_JPRB, 0.35547E+01_JPRB, 0.46871E+01_JPRB, 0.52917E+01_JPRB, 0.61029E+01_JPRB, 0.51963E+01_JPRB, 0.42136E+01_JPRB ,&
&  0.32532E+01_JPRB, 0.30689E+01_JPRB, 0.31530E+01_JPRB, 0.27970E+01_JPRB, 0.22469E+01_JPRB, 0.19913E+01_JPRB, 0.19683E+01_JPRB ,&
&  0.16766E+01_JPRB, 0.15263E+01_JPRB, 0.13960E+01_JPRB, 0.12097E+01_JPRB, 0.10867E+01_JPRB, 0.10409E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.99264E+00_JPRB, 0.98045E+00_JPRB, 0.98556E+00_JPRB, 0.99497E+00_JPRB, 0.10004E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10463E+01_JPRB, 0.10680E+01_JPRB, 0.10802E+01_JPRB, 0.10688E+01_JPRB, 0.10493E+01_JPRB, 0.10723E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.11045E+01_JPRB, 0.10936E+01_JPRB, 0.10963E+01_JPRB, 0.10842E+01_JPRB, 0.10787E+01_JPRB, 0.10692E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10257E+01_JPRB, 0.93448E+00_JPRB, 0.82003E+00_JPRB, 0.78727E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.85012E+00_JPRB, 0.81901E+00_JPRB, 0.77976E+00_JPRB, 0.73976E+00_JPRB, 0.71779E+00_JPRB, 0.69592E+00_JPRB, 0.66956E+00_JPRB ,&
&  0.65511E+00_JPRB, 0.64791E+00_JPRB, 0.63544E+00_JPRB, 0.64582E+00_JPRB, 0.67225E+00_JPRB, 0.68674E+00_JPRB, 0.69818E+00_JPRB ,&
&  0.71087E+00_JPRB, 0.72321E+00_JPRB, 0.71922E+00_JPRB, 0.72117E+00_JPRB, 0.73396E+00_JPRB, 0.74393E+00_JPRB, 0.74537E+00_JPRB ,&
&  0.75204E+00_JPRB, 0.76967E+00_JPRB, 0.79478E+00_JPRB, 0.84491E+00_JPRB, 0.95056E+00_JPRB, 0.11198E+01_JPRB, 0.13576E+01_JPRB ,&
&  0.17152E+01_JPRB, 0.17279E+01_JPRB, 0.95298E+00_JPRB, 0.16703E+01_JPRB, 0.22906E+01_JPRB, 0.23205E+01_JPRB, 0.21571E+01_JPRB ,&
&  0.21400E+01_JPRB, 0.20702E+01_JPRB, 0.19809E+01_JPRB, 0.17182E+01_JPRB, 0.15154E+01_JPRB, 0.13588E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.10513E+01_JPRB, 0.10350E+01_JPRB, 0.11073E+01_JPRB, 0.12441E+01_JPRB, 0.13824E+01_JPRB, 0.16464E+01_JPRB ,&
&  0.21135E+01_JPRB, 0.27165E+01_JPRB, 0.37003E+01_JPRB, 0.47333E+01_JPRB, 0.62933E+01_JPRB, 0.63095E+01_JPRB, 0.50843E+01_JPRB ,&
&  0.39504E+01_JPRB, 0.38593E+01_JPRB, 0.38066E+01_JPRB, 0.33158E+01_JPRB, 0.25367E+01_JPRB, 0.22976E+01_JPRB, 0.22486E+01_JPRB ,&
&  0.19574E+01_JPRB, 0.17295E+01_JPRB, 0.16248E+01_JPRB, 0.15055E+01_JPRB, 0.13358E+01_JPRB, 0.11899E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10681E+01_JPRB, 0.10510E+01_JPRB, 0.10351E+01_JPRB, 0.10261E+01_JPRB, 0.10227E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.10528E+01_JPRB, 0.10678E+01_JPRB, 0.10636E+01_JPRB, 0.10577E+01_JPRB, 0.10543E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10555E+01_JPRB, 0.10552E+01_JPRB, 0.10257E+01_JPRB, 0.10678E+01_JPRB, 0.10676E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.11161E+01_JPRB, 0.11134E+01_JPRB, 0.10469E+01_JPRB, 0.91219E+00_JPRB, 0.88300E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.88582E+00_JPRB, 0.86133E+00_JPRB, 0.83655E+00_JPRB, 0.81537E+00_JPRB, 0.79705E+00_JPRB, 0.77795E+00_JPRB, 0.74851E+00_JPRB ,&
&  0.71225E+00_JPRB, 0.68855E+00_JPRB, 0.65368E+00_JPRB, 0.64115E+00_JPRB, 0.64404E+00_JPRB, 0.64793E+00_JPRB, 0.65926E+00_JPRB ,&
&  0.66770E+00_JPRB, 0.67658E+00_JPRB, 0.68165E+00_JPRB, 0.68326E+00_JPRB, 0.68555E+00_JPRB, 0.68630E+00_JPRB, 0.68479E+00_JPRB ,&
&  0.67884E+00_JPRB, 0.68123E+00_JPRB, 0.69241E+00_JPRB, 0.71113E+00_JPRB, 0.75626E+00_JPRB, 0.84045E+00_JPRB, 0.96415E+00_JPRB ,&
&  0.12579E+01_JPRB, 0.14273E+01_JPRB, 0.15524E+01_JPRB, 0.23012E+01_JPRB, 0.27406E+01_JPRB, 0.26331E+01_JPRB, 0.26143E+01_JPRB ,&
&  0.26673E+01_JPRB, 0.26290E+01_JPRB, 0.24208E+01_JPRB, 0.18953E+01_JPRB, 0.15527E+01_JPRB, 0.13763E+01_JPRB, 0.12431E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.10312E+01_JPRB, 0.96333E+00_JPRB, 0.94473E+00_JPRB, 0.99413E+00_JPRB, 0.11030E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.15000E+01_JPRB, 0.19541E+01_JPRB, 0.26240E+01_JPRB, 0.35003E+01_JPRB, 0.50606E+01_JPRB, 0.62058E+01_JPRB, 0.59068E+01_JPRB ,&
&  0.54971E+01_JPRB, 0.49762E+01_JPRB, 0.43641E+01_JPRB, 0.38646E+01_JPRB, 0.28698E+01_JPRB, 0.27673E+01_JPRB, 0.24342E+01_JPRB ,&
&  0.21083E+01_JPRB, 0.18733E+01_JPRB, 0.17405E+01_JPRB, 0.16604E+01_JPRB, 0.15429E+01_JPRB, 0.13771E+01_JPRB, 0.12397E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.11310E+01_JPRB, 0.11171E+01_JPRB, 0.10992E+01_JPRB, 0.10832E+01_JPRB, 0.10729E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.10589E+01_JPRB, 0.10556E+01_JPRB, 0.10365E+01_JPRB, 0.10415E+01_JPRB, 0.10330E+01_JPRB, 0.10199E+01_JPRB, 0.10045E+01_JPRB ,&
&  0.10120E+01_JPRB, 0.10328E+01_JPRB, 0.10389E+01_JPRB, 0.10322E+01_JPRB, 0.10271E+01_JPRB, 0.10239E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.97874E+00_JPRB, 0.93276E+00_JPRB, 0.91560E+00_JPRB, 0.90249E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.89153E+00_JPRB, 0.87067E+00_JPRB, 0.85565E+00_JPRB, 0.83762E+00_JPRB, 0.81941E+00_JPRB, 0.80757E+00_JPRB, 0.79643E+00_JPRB ,&
&  0.77781E+00_JPRB, 0.73853E+00_JPRB, 0.69793E+00_JPRB, 0.67597E+00_JPRB, 0.66619E+00_JPRB, 0.65000E+00_JPRB, 0.64923E+00_JPRB ,&
&  0.65181E+00_JPRB, 0.65958E+00_JPRB, 0.65845E+00_JPRB, 0.65838E+00_JPRB, 0.65879E+00_JPRB, 0.65919E+00_JPRB, 0.65559E+00_JPRB ,&
&  0.64603E+00_JPRB, 0.64083E+00_JPRB, 0.64453E+00_JPRB, 0.65211E+00_JPRB, 0.66983E+00_JPRB, 0.70971E+00_JPRB, 0.76285E+00_JPRB ,&
&  0.94470E+00_JPRB, 0.11713E+01_JPRB, 0.12562E+01_JPRB, 0.22779E+01_JPRB, 0.26819E+01_JPRB, 0.28536E+01_JPRB, 0.27108E+01_JPRB ,&
&  0.27614E+01_JPRB, 0.30703E+01_JPRB, 0.25912E+01_JPRB, 0.19290E+01_JPRB, 0.14834E+01_JPRB, 0.12790E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.10591E+01_JPRB, 0.97247E+00_JPRB, 0.91427E+00_JPRB, 0.88071E+00_JPRB, 0.88660E+00_JPRB, 0.91749E+00_JPRB, 0.98445E+00_JPRB ,&
&  0.11148E+01_JPRB, 0.14103E+01_JPRB, 0.18688E+01_JPRB, 0.23212E+01_JPRB, 0.34663E+01_JPRB, 0.47728E+01_JPRB, 0.55305E+01_JPRB ,&
&  0.58704E+01_JPRB, 0.57866E+01_JPRB, 0.47343E+01_JPRB, 0.40793E+01_JPRB, 0.33049E+01_JPRB, 0.28896E+01_JPRB, 0.24754E+01_JPRB ,&
&  0.21879E+01_JPRB, 0.19628E+01_JPRB, 0.17999E+01_JPRB, 0.17048E+01_JPRB, 0.16111E+01_JPRB, 0.14964E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.12306E+01_JPRB, 0.11687E+01_JPRB, 0.11406E+01_JPRB, 0.11242E+01_JPRB, 0.11187E+01_JPRB, 0.11094E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10662E+01_JPRB, 0.10333E+01_JPRB, 0.10292E+01_JPRB, 0.10063E+01_JPRB, 0.96856E+00_JPRB, 0.95967E+00_JPRB ,&
&  0.99617E+00_JPRB, 0.10242E+01_JPRB, 0.10249E+01_JPRB, 0.10002E+01_JPRB, 0.10105E+01_JPRB, 0.10258E+01_JPRB, 0.98372E+00_JPRB ,&
&  0.96603E+00_JPRB, 0.94906E+00_JPRB, 0.92543E+00_JPRB, 0.91236E+00_JPRB, 0.90468E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.88339E+00_JPRB, 0.85542E+00_JPRB, 0.83526E+00_JPRB, 0.81802E+00_JPRB, 0.80147E+00_JPRB, 0.78626E+00_JPRB, 0.78078E+00_JPRB ,&
&  0.77400E+00_JPRB, 0.76158E+00_JPRB, 0.73407E+00_JPRB, 0.70239E+00_JPRB, 0.68537E+00_JPRB, 0.67668E+00_JPRB, 0.66002E+00_JPRB ,&
&  0.65228E+00_JPRB, 0.65235E+00_JPRB, 0.65166E+00_JPRB, 0.64563E+00_JPRB, 0.63952E+00_JPRB, 0.63443E+00_JPRB, 0.62949E+00_JPRB ,&
&  0.61517E+00_JPRB, 0.60805E+00_JPRB, 0.60781E+00_JPRB, 0.60808E+00_JPRB, 0.61303E+00_JPRB, 0.64460E+00_JPRB, 0.70036E+00_JPRB ,&
&  0.80988E+00_JPRB, 0.12743E+01_JPRB, 0.12442E+01_JPRB, 0.18712E+01_JPRB, 0.21275E+01_JPRB, 0.24269E+01_JPRB, 0.25940E+01_JPRB ,&
&  0.27916E+01_JPRB, 0.30108E+01_JPRB, 0.24571E+01_JPRB, 0.18528E+01_JPRB, 0.13608E+01_JPRB, 0.11013E+01_JPRB, 0.98590E+00_JPRB ,&
&  0.92820E+00_JPRB, 0.87832E+00_JPRB, 0.84041E+00_JPRB, 0.82692E+00_JPRB, 0.83068E+00_JPRB, 0.84194E+00_JPRB, 0.85982E+00_JPRB ,&
&  0.92573E+00_JPRB, 0.10955E+01_JPRB, 0.14887E+01_JPRB, 0.20366E+01_JPRB, 0.23904E+01_JPRB, 0.32741E+01_JPRB, 0.45894E+01_JPRB ,&
&  0.54638E+01_JPRB, 0.60020E+01_JPRB, 0.49749E+01_JPRB, 0.41243E+01_JPRB, 0.34781E+01_JPRB, 0.29830E+01_JPRB, 0.25627E+01_JPRB ,&
&  0.22399E+01_JPRB, 0.20016E+01_JPRB, 0.18293E+01_JPRB, 0.17078E+01_JPRB, 0.16221E+01_JPRB, 0.15555E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.12617E+01_JPRB, 0.12051E+01_JPRB, 0.11734E+01_JPRB, 0.11629E+01_JPRB, 0.11445E+01_JPRB, 0.11099E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.10667E+01_JPRB, 0.10359E+01_JPRB, 0.10280E+01_JPRB, 0.10101E+01_JPRB, 0.96755E+00_JPRB, 0.95018E+00_JPRB ,&
&  0.98779E+00_JPRB, 0.10057E+01_JPRB, 0.10095E+01_JPRB, 0.10084E+01_JPRB, 0.10106E+01_JPRB, 0.10508E+01_JPRB, 0.99723E+00_JPRB ,&
&  0.95588E+00_JPRB, 0.93378E+00_JPRB, 0.91675E+00_JPRB, 0.90990E+00_JPRB, 0.90003E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.86426E+00_JPRB, 0.83516E+00_JPRB, 0.81109E+00_JPRB, 0.78711E+00_JPRB, 0.76868E+00_JPRB, 0.74915E+00_JPRB, 0.73604E+00_JPRB ,&
&  0.72942E+00_JPRB, 0.72832E+00_JPRB, 0.72443E+00_JPRB, 0.71124E+00_JPRB, 0.68985E+00_JPRB, 0.67273E+00_JPRB, 0.66044E+00_JPRB ,&
&  0.64843E+00_JPRB, 0.64469E+00_JPRB, 0.64274E+00_JPRB, 0.63327E+00_JPRB, 0.62278E+00_JPRB, 0.61273E+00_JPRB, 0.60182E+00_JPRB ,&
&  0.59145E+00_JPRB, 0.58579E+00_JPRB, 0.58181E+00_JPRB, 0.57813E+00_JPRB, 0.57595E+00_JPRB, 0.59271E+00_JPRB, 0.67660E+00_JPRB ,&
&  0.85641E+00_JPRB, 0.74825E+00_JPRB, 0.15231E+01_JPRB, 0.17205E+01_JPRB, 0.17893E+01_JPRB, 0.19146E+01_JPRB, 0.22393E+01_JPRB ,&
&  0.25782E+01_JPRB, 0.25486E+01_JPRB, 0.22048E+01_JPRB, 0.17447E+01_JPRB, 0.13408E+01_JPRB, 0.10777E+01_JPRB, 0.94143E+00_JPRB ,&
&  0.84927E+00_JPRB, 0.79483E+00_JPRB, 0.76991E+00_JPRB, 0.75596E+00_JPRB, 0.75785E+00_JPRB, 0.77232E+00_JPRB, 0.78675E+00_JPRB ,&
&  0.82362E+00_JPRB, 0.92460E+00_JPRB, 0.11968E+01_JPRB, 0.17289E+01_JPRB, 0.23064E+01_JPRB, 0.27421E+01_JPRB, 0.35801E+01_JPRB ,&
&  0.47428E+01_JPRB, 0.51399E+01_JPRB, 0.42698E+01_JPRB, 0.36130E+01_JPRB, 0.31905E+01_JPRB, 0.28831E+01_JPRB, 0.25753E+01_JPRB ,&
&  0.22772E+01_JPRB, 0.19896E+01_JPRB, 0.17810E+01_JPRB, 0.16348E+01_JPRB, 0.15518E+01_JPRB, 0.15206E+01_JPRB, 0.14923E+01_JPRB ,&
&  0.14521E+01_JPRB, 0.13919E+01_JPRB, 0.13306E+01_JPRB, 0.12740E+01_JPRB, 0.12308E+01_JPRB, 0.11902E+01_JPRB, 0.11325E+01_JPRB ,&
&  0.10825E+01_JPRB, 0.10568E+01_JPRB, 0.10247E+01_JPRB, 0.10007E+01_JPRB, 0.99153E+00_JPRB, 0.97788E+00_JPRB, 0.97121E+00_JPRB ,&
&  0.98021E+00_JPRB, 0.98257E+00_JPRB, 0.98227E+00_JPRB, 0.98875E+00_JPRB, 0.97265E+00_JPRB, 0.10145E+01_JPRB, 0.98805E+00_JPRB ,&
&  0.97193E+00_JPRB, 0.94415E+00_JPRB, 0.91396E+00_JPRB, 0.89729E+00_JPRB, 0.88504E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.84182E+00_JPRB, 0.81857E+00_JPRB, 0.78652E+00_JPRB, 0.75821E+00_JPRB, 0.73584E+00_JPRB, 0.71227E+00_JPRB, 0.69546E+00_JPRB ,&
&  0.68306E+00_JPRB, 0.68384E+00_JPRB, 0.68860E+00_JPRB, 0.69049E+00_JPRB, 0.68284E+00_JPRB, 0.66446E+00_JPRB, 0.64598E+00_JPRB ,&
&  0.63252E+00_JPRB, 0.62751E+00_JPRB, 0.62572E+00_JPRB, 0.62013E+00_JPRB, 0.61155E+00_JPRB, 0.60033E+00_JPRB, 0.58965E+00_JPRB ,&
&  0.57931E+00_JPRB, 0.57173E+00_JPRB, 0.56446E+00_JPRB, 0.55884E+00_JPRB, 0.55253E+00_JPRB, 0.56066E+00_JPRB, 0.64268E+00_JPRB ,&
&  0.89222E+00_JPRB, 0.11824E+01_JPRB, 0.14723E+01_JPRB, 0.13861E+01_JPRB, 0.13828E+01_JPRB, 0.14549E+01_JPRB, 0.16983E+01_JPRB ,&
&  0.18393E+01_JPRB, 0.18930E+01_JPRB, 0.17883E+01_JPRB, 0.15524E+01_JPRB, 0.12748E+01_JPRB, 0.10781E+01_JPRB, 0.93562E+00_JPRB ,&
&  0.81971E+00_JPRB, 0.74803E+00_JPRB, 0.70844E+00_JPRB, 0.69680E+00_JPRB, 0.70196E+00_JPRB, 0.71583E+00_JPRB, 0.72602E+00_JPRB ,&
&  0.74639E+00_JPRB, 0.80187E+00_JPRB, 0.10035E+01_JPRB, 0.14804E+01_JPRB, 0.23032E+01_JPRB, 0.24179E+01_JPRB, 0.33215E+01_JPRB ,&
&  0.42450E+01_JPRB, 0.38497E+01_JPRB, 0.34826E+01_JPRB, 0.31708E+01_JPRB, 0.29258E+01_JPRB, 0.27061E+01_JPRB, 0.23577E+01_JPRB ,&
&  0.20766E+01_JPRB, 0.18153E+01_JPRB, 0.16187E+01_JPRB, 0.15043E+01_JPRB, 0.14334E+01_JPRB, 0.14051E+01_JPRB, 0.14004E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.13745E+01_JPRB, 0.13560E+01_JPRB, 0.13159E+01_JPRB, 0.12599E+01_JPRB, 0.12034E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.10451E+01_JPRB, 0.99863E+00_JPRB, 0.95946E+00_JPRB, 0.95257E+00_JPRB, 0.95545E+00_JPRB, 0.94996E+00_JPRB ,&
&  0.93532E+00_JPRB, 0.92663E+00_JPRB, 0.93958E+00_JPRB, 0.93687E+00_JPRB, 0.93187E+00_JPRB, 0.97880E+00_JPRB, 0.96958E+00_JPRB ,&
&  0.95753E+00_JPRB, 0.92650E+00_JPRB, 0.89780E+00_JPRB, 0.87352E+00_JPRB, 0.85908E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.82366E+00_JPRB, 0.79453E+00_JPRB, 0.76206E+00_JPRB, 0.73205E+00_JPRB, 0.70487E+00_JPRB, 0.68386E+00_JPRB, 0.67050E+00_JPRB ,&
&  0.65892E+00_JPRB, 0.65222E+00_JPRB, 0.65493E+00_JPRB, 0.66091E+00_JPRB, 0.66015E+00_JPRB, 0.64939E+00_JPRB, 0.63336E+00_JPRB ,&
&  0.61689E+00_JPRB, 0.60421E+00_JPRB, 0.59516E+00_JPRB, 0.59132E+00_JPRB, 0.58582E+00_JPRB, 0.57954E+00_JPRB, 0.56920E+00_JPRB ,&
&  0.56069E+00_JPRB, 0.55368E+00_JPRB, 0.54382E+00_JPRB, 0.53359E+00_JPRB, 0.52603E+00_JPRB, 0.52890E+00_JPRB, 0.59241E+00_JPRB ,&
&  0.78633E+00_JPRB, 0.10368E+01_JPRB, 0.10501E+01_JPRB, 0.10809E+01_JPRB, 0.11653E+01_JPRB, 0.11958E+01_JPRB, 0.12921E+01_JPRB ,&
&  0.13436E+01_JPRB, 0.13723E+01_JPRB, 0.13815E+01_JPRB, 0.13026E+01_JPRB, 0.11501E+01_JPRB, 0.10306E+01_JPRB, 0.91990E+00_JPRB ,&
&  0.82139E+00_JPRB, 0.74461E+00_JPRB, 0.69609E+00_JPRB, 0.67606E+00_JPRB, 0.67690E+00_JPRB, 0.68610E+00_JPRB, 0.69045E+00_JPRB ,&
&  0.69700E+00_JPRB, 0.70842E+00_JPRB, 0.78234E+00_JPRB, 0.96851E+00_JPRB, 0.14441E+01_JPRB, 0.17913E+01_JPRB, 0.23156E+01_JPRB ,&
&  0.24295E+01_JPRB, 0.24024E+01_JPRB, 0.23429E+01_JPRB, 0.22787E+01_JPRB, 0.21902E+01_JPRB, 0.20439E+01_JPRB, 0.19122E+01_JPRB ,&
&  0.17472E+01_JPRB, 0.15737E+01_JPRB, 0.14362E+01_JPRB, 0.13426E+01_JPRB, 0.12823E+01_JPRB, 0.12454E+01_JPRB, 0.12309E+01_JPRB ,&
&  0.12102E+01_JPRB, 0.12038E+01_JPRB, 0.11850E+01_JPRB, 0.11684E+01_JPRB, 0.11561E+01_JPRB, 0.11219E+01_JPRB, 0.10686E+01_JPRB ,&
&  0.10121E+01_JPRB, 0.96349E+00_JPRB, 0.92535E+00_JPRB, 0.90251E+00_JPRB, 0.88974E+00_JPRB, 0.87955E+00_JPRB, 0.86987E+00_JPRB ,&
&  0.87269E+00_JPRB, 0.87358E+00_JPRB, 0.87655E+00_JPRB, 0.87137E+00_JPRB, 0.86014E+00_JPRB, 0.89524E+00_JPRB, 0.88931E+00_JPRB ,&
&  0.88396E+00_JPRB, 0.87683E+00_JPRB, 0.86106E+00_JPRB, 0.83928E+00_JPRB, 0.84055E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.81781E+00_JPRB, 0.79200E+00_JPRB, 0.75752E+00_JPRB, 0.72371E+00_JPRB, 0.69224E+00_JPRB, 0.66740E+00_JPRB, 0.65246E+00_JPRB ,&
&  0.64205E+00_JPRB, 0.63542E+00_JPRB, 0.63225E+00_JPRB, 0.63047E+00_JPRB, 0.62754E+00_JPRB, 0.62386E+00_JPRB, 0.61715E+00_JPRB ,&
&  0.60803E+00_JPRB, 0.59670E+00_JPRB, 0.58635E+00_JPRB, 0.57897E+00_JPRB, 0.57322E+00_JPRB, 0.56926E+00_JPRB, 0.56355E+00_JPRB ,&
&  0.55337E+00_JPRB, 0.54158E+00_JPRB, 0.52716E+00_JPRB, 0.51274E+00_JPRB, 0.50330E+00_JPRB, 0.50317E+00_JPRB, 0.54361E+00_JPRB ,&
&  0.68273E+00_JPRB, 0.74820E+00_JPRB, 0.81292E+00_JPRB, 0.86522E+00_JPRB, 0.93574E+00_JPRB, 0.98323E+00_JPRB, 0.10138E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10561E+01_JPRB, 0.10975E+01_JPRB, 0.10893E+01_JPRB, 0.10095E+01_JPRB, 0.93304E+00_JPRB, 0.86182E+00_JPRB ,&
&  0.79749E+00_JPRB, 0.74123E+00_JPRB, 0.69493E+00_JPRB, 0.67443E+00_JPRB, 0.66888E+00_JPRB, 0.67137E+00_JPRB, 0.67063E+00_JPRB ,&
&  0.67139E+00_JPRB, 0.66625E+00_JPRB, 0.67131E+00_JPRB, 0.70033E+00_JPRB, 0.77600E+00_JPRB, 0.92975E+00_JPRB, 0.11840E+01_JPRB ,&
&  0.13017E+01_JPRB, 0.13455E+01_JPRB, 0.13870E+01_JPRB, 0.14286E+01_JPRB, 0.14078E+01_JPRB, 0.13708E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.12442E+01_JPRB, 0.12005E+01_JPRB, 0.11455E+01_JPRB, 0.10980E+01_JPRB, 0.10633E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10123E+01_JPRB, 0.99934E+00_JPRB, 0.97215E+00_JPRB, 0.95269E+00_JPRB, 0.93185E+00_JPRB, 0.89924E+00_JPRB ,&
&  0.85286E+00_JPRB, 0.82302E+00_JPRB, 0.80441E+00_JPRB, 0.79956E+00_JPRB, 0.80048E+00_JPRB, 0.80304E+00_JPRB, 0.79987E+00_JPRB ,&
&  0.80063E+00_JPRB, 0.81192E+00_JPRB, 0.80147E+00_JPRB, 0.79104E+00_JPRB, 0.82347E+00_JPRB, 0.82196E+00_JPRB, 0.82188E+00_JPRB ,&
&  0.82051E+00_JPRB, 0.81556E+00_JPRB, 0.80425E+00_JPRB, 0.78251E+00_JPRB, 0.81827E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.70546E+00_JPRB, 0.70581E+00_JPRB, 0.68852E+00_JPRB, 0.66811E+00_JPRB, 0.64935E+00_JPRB, 0.63519E+00_JPRB, 0.62372E+00_JPRB ,&
&  0.61556E+00_JPRB, 0.60931E+00_JPRB, 0.60548E+00_JPRB, 0.60314E+00_JPRB, 0.59954E+00_JPRB, 0.59386E+00_JPRB, 0.58752E+00_JPRB ,&
&  0.58182E+00_JPRB, 0.57477E+00_JPRB, 0.56405E+00_JPRB, 0.55637E+00_JPRB, 0.55179E+00_JPRB, 0.54570E+00_JPRB, 0.53908E+00_JPRB ,&
&  0.53093E+00_JPRB, 0.51591E+00_JPRB, 0.50089E+00_JPRB, 0.49054E+00_JPRB, 0.48231E+00_JPRB, 0.47960E+00_JPRB, 0.50581E+00_JPRB ,&
&  0.59389E+00_JPRB, 0.58842E+00_JPRB, 0.64643E+00_JPRB, 0.70685E+00_JPRB, 0.77972E+00_JPRB, 0.82177E+00_JPRB, 0.84120E+00_JPRB ,&
&  0.85227E+00_JPRB, 0.87768E+00_JPRB, 0.89847E+00_JPRB, 0.92698E+00_JPRB, 0.89383E+00_JPRB, 0.83803E+00_JPRB, 0.79102E+00_JPRB ,&
&  0.74743E+00_JPRB, 0.70999E+00_JPRB, 0.67906E+00_JPRB, 0.65811E+00_JPRB, 0.65041E+00_JPRB, 0.64729E+00_JPRB, 0.64016E+00_JPRB ,&
&  0.63766E+00_JPRB, 0.63110E+00_JPRB, 0.62703E+00_JPRB, 0.63178E+00_JPRB, 0.62811E+00_JPRB, 0.66411E+00_JPRB, 0.72581E+00_JPRB ,&
&  0.78482E+00_JPRB, 0.86508E+00_JPRB, 0.90985E+00_JPRB, 0.94060E+00_JPRB, 0.94350E+00_JPRB, 0.94410E+00_JPRB, 0.94264E+00_JPRB ,&
&  0.94081E+00_JPRB, 0.93618E+00_JPRB, 0.92597E+00_JPRB, 0.90673E+00_JPRB, 0.88280E+00_JPRB, 0.86084E+00_JPRB, 0.84175E+00_JPRB ,&
&  0.82589E+00_JPRB, 0.81342E+00_JPRB, 0.80517E+00_JPRB, 0.78558E+00_JPRB, 0.75447E+00_JPRB, 0.72743E+00_JPRB, 0.70639E+00_JPRB ,&
&  0.68789E+00_JPRB, 0.67964E+00_JPRB, 0.67825E+00_JPRB, 0.68581E+00_JPRB, 0.69349E+00_JPRB, 0.70118E+00_JPRB, 0.70499E+00_JPRB ,&
&  0.70784E+00_JPRB, 0.71770E+00_JPRB, 0.73443E+00_JPRB, 0.75383E+00_JPRB, 0.76174E+00_JPRB, 0.76579E+00_JPRB, 0.76602E+00_JPRB ,&
&  0.76310E+00_JPRB, 0.75832E+00_JPRB, 0.74449E+00_JPRB, 0.72656E+00_JPRB, 0.69497E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.63760E+00_JPRB, 0.63865E+00_JPRB, 0.63017E+00_JPRB, 0.61663E+00_JPRB, 0.60510E+00_JPRB, 0.59507E+00_JPRB, 0.58732E+00_JPRB ,&
&  0.57583E+00_JPRB, 0.56588E+00_JPRB, 0.56244E+00_JPRB, 0.56251E+00_JPRB, 0.56179E+00_JPRB, 0.55751E+00_JPRB, 0.55160E+00_JPRB ,&
&  0.54472E+00_JPRB, 0.53692E+00_JPRB, 0.52771E+00_JPRB, 0.51845E+00_JPRB, 0.51090E+00_JPRB, 0.50197E+00_JPRB, 0.49446E+00_JPRB ,&
&  0.48721E+00_JPRB, 0.48051E+00_JPRB, 0.47040E+00_JPRB, 0.45841E+00_JPRB, 0.45270E+00_JPRB, 0.44729E+00_JPRB, 0.47010E+00_JPRB ,&
&  0.54472E+00_JPRB, 0.47819E+00_JPRB, 0.51841E+00_JPRB, 0.58564E+00_JPRB, 0.65465E+00_JPRB, 0.72346E+00_JPRB, 0.74904E+00_JPRB ,&
&  0.76166E+00_JPRB, 0.77245E+00_JPRB, 0.77936E+00_JPRB, 0.79270E+00_JPRB, 0.79448E+00_JPRB, 0.76370E+00_JPRB, 0.72168E+00_JPRB ,&
&  0.68899E+00_JPRB, 0.65970E+00_JPRB, 0.64082E+00_JPRB, 0.62263E+00_JPRB, 0.61347E+00_JPRB, 0.60920E+00_JPRB, 0.59885E+00_JPRB ,&
&  0.59021E+00_JPRB, 0.59207E+00_JPRB, 0.59720E+00_JPRB, 0.60566E+00_JPRB, 0.61160E+00_JPRB, 0.61621E+00_JPRB, 0.60785E+00_JPRB ,&
&  0.60015E+00_JPRB, 0.62471E+00_JPRB, 0.66791E+00_JPRB, 0.69349E+00_JPRB, 0.70965E+00_JPRB, 0.71090E+00_JPRB, 0.70490E+00_JPRB ,&
&  0.70287E+00_JPRB, 0.70252E+00_JPRB, 0.70626E+00_JPRB, 0.70372E+00_JPRB, 0.69698E+00_JPRB, 0.68333E+00_JPRB, 0.66622E+00_JPRB ,&
&  0.64955E+00_JPRB, 0.63474E+00_JPRB, 0.62303E+00_JPRB, 0.61232E+00_JPRB, 0.59956E+00_JPRB, 0.58296E+00_JPRB, 0.57342E+00_JPRB ,&
&  0.56856E+00_JPRB, 0.56857E+00_JPRB, 0.57257E+00_JPRB, 0.58005E+00_JPRB, 0.59092E+00_JPRB, 0.60092E+00_JPRB, 0.61317E+00_JPRB ,&
&  0.62381E+00_JPRB, 0.63458E+00_JPRB, 0.64688E+00_JPRB, 0.64976E+00_JPRB, 0.68102E+00_JPRB, 0.69006E+00_JPRB, 0.69853E+00_JPRB ,&
&  0.70016E+00_JPRB, 0.69901E+00_JPRB, 0.69429E+00_JPRB, 0.61313E+00_JPRB, 0.63474E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.58039E+00_JPRB, 0.57892E+00_JPRB, 0.57254E+00_JPRB, 0.56495E+00_JPRB, 0.55871E+00_JPRB, 0.55222E+00_JPRB, 0.54694E+00_JPRB ,&
&  0.53906E+00_JPRB, 0.52914E+00_JPRB, 0.52205E+00_JPRB, 0.51936E+00_JPRB, 0.51828E+00_JPRB, 0.51530E+00_JPRB, 0.50963E+00_JPRB ,&
&  0.50227E+00_JPRB, 0.49419E+00_JPRB, 0.48637E+00_JPRB, 0.47689E+00_JPRB, 0.46815E+00_JPRB, 0.45996E+00_JPRB, 0.45290E+00_JPRB ,&
&  0.44578E+00_JPRB, 0.43814E+00_JPRB, 0.43061E+00_JPRB, 0.42513E+00_JPRB, 0.42094E+00_JPRB, 0.41940E+00_JPRB, 0.43793E+00_JPRB ,&
&  0.49618E+00_JPRB, 0.42724E+00_JPRB, 0.45613E+00_JPRB, 0.49716E+00_JPRB, 0.55996E+00_JPRB, 0.63707E+00_JPRB, 0.67463E+00_JPRB ,&
&  0.69076E+00_JPRB, 0.69892E+00_JPRB, 0.69833E+00_JPRB, 0.69744E+00_JPRB, 0.70254E+00_JPRB, 0.69560E+00_JPRB, 0.67225E+00_JPRB ,&
&  0.64605E+00_JPRB, 0.62122E+00_JPRB, 0.60335E+00_JPRB, 0.58992E+00_JPRB, 0.57771E+00_JPRB, 0.56767E+00_JPRB, 0.55439E+00_JPRB ,&
&  0.53988E+00_JPRB, 0.53701E+00_JPRB, 0.53913E+00_JPRB, 0.53971E+00_JPRB, 0.54017E+00_JPRB, 0.53610E+00_JPRB, 0.51924E+00_JPRB ,&
&  0.50369E+00_JPRB, 0.49476E+00_JPRB, 0.49464E+00_JPRB, 0.50407E+00_JPRB, 0.51521E+00_JPRB, 0.52268E+00_JPRB, 0.52254E+00_JPRB ,&
&  0.51878E+00_JPRB, 0.51177E+00_JPRB, 0.50912E+00_JPRB, 0.50910E+00_JPRB, 0.50904E+00_JPRB, 0.50913E+00_JPRB, 0.50609E+00_JPRB ,&
&  0.50103E+00_JPRB, 0.49862E+00_JPRB, 0.49968E+00_JPRB, 0.50127E+00_JPRB, 0.50103E+00_JPRB, 0.49862E+00_JPRB, 0.49560E+00_JPRB ,&
&  0.49128E+00_JPRB, 0.48836E+00_JPRB, 0.48957E+00_JPRB, 0.49570E+00_JPRB, 0.50456E+00_JPRB, 0.51432E+00_JPRB, 0.52505E+00_JPRB ,&
&  0.53679E+00_JPRB, 0.54877E+00_JPRB, 0.56137E+00_JPRB, 0.58255E+00_JPRB, 0.59309E+00_JPRB, 0.60587E+00_JPRB, 0.61843E+00_JPRB ,&
&  0.62049E+00_JPRB, 0.61707E+00_JPRB, 0.52986E+00_JPRB, 0.56584E+00_JPRB, 0.57450E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.54280E+00_JPRB, 0.53295E+00_JPRB, 0.52145E+00_JPRB, 0.51411E+00_JPRB, 0.51149E+00_JPRB, 0.50845E+00_JPRB, 0.50537E+00_JPRB ,&
&  0.50010E+00_JPRB, 0.49409E+00_JPRB, 0.48808E+00_JPRB, 0.48276E+00_JPRB, 0.47841E+00_JPRB, 0.47540E+00_JPRB, 0.47128E+00_JPRB ,&
&  0.46497E+00_JPRB, 0.45752E+00_JPRB, 0.44967E+00_JPRB, 0.44072E+00_JPRB, 0.42913E+00_JPRB, 0.41976E+00_JPRB, 0.41234E+00_JPRB ,&
&  0.40652E+00_JPRB, 0.40192E+00_JPRB, 0.39992E+00_JPRB, 0.39759E+00_JPRB, 0.39511E+00_JPRB, 0.39511E+00_JPRB, 0.40928E+00_JPRB ,&
&  0.41630E+00_JPRB, 0.38975E+00_JPRB, 0.41278E+00_JPRB, 0.44448E+00_JPRB, 0.49415E+00_JPRB, 0.57096E+00_JPRB, 0.62062E+00_JPRB ,&
&  0.64644E+00_JPRB, 0.65759E+00_JPRB, 0.65922E+00_JPRB, 0.65175E+00_JPRB, 0.64429E+00_JPRB, 0.64450E+00_JPRB, 0.63859E+00_JPRB ,&
&  0.62100E+00_JPRB, 0.59817E+00_JPRB, 0.57676E+00_JPRB, 0.56078E+00_JPRB, 0.54285E+00_JPRB, 0.52769E+00_JPRB, 0.51186E+00_JPRB ,&
&  0.49803E+00_JPRB, 0.49056E+00_JPRB, 0.48831E+00_JPRB, 0.48647E+00_JPRB, 0.48364E+00_JPRB, 0.47515E+00_JPRB, 0.46248E+00_JPRB ,&
&  0.45254E+00_JPRB, 0.44377E+00_JPRB, 0.43512E+00_JPRB, 0.42793E+00_JPRB, 0.42331E+00_JPRB, 0.42045E+00_JPRB, 0.41823E+00_JPRB ,&
&  0.41842E+00_JPRB, 0.42022E+00_JPRB, 0.42381E+00_JPRB, 0.42760E+00_JPRB, 0.42991E+00_JPRB, 0.42976E+00_JPRB, 0.42941E+00_JPRB ,&
&  0.42902E+00_JPRB, 0.42598E+00_JPRB, 0.42587E+00_JPRB, 0.43102E+00_JPRB, 0.43785E+00_JPRB, 0.44229E+00_JPRB, 0.44380E+00_JPRB ,&
&  0.44209E+00_JPRB, 0.44037E+00_JPRB, 0.44050E+00_JPRB, 0.44316E+00_JPRB, 0.44717E+00_JPRB, 0.45327E+00_JPRB, 0.46067E+00_JPRB ,&
&  0.46829E+00_JPRB, 0.47800E+00_JPRB, 0.49107E+00_JPRB, 0.50947E+00_JPRB, 0.52326E+00_JPRB, 0.53402E+00_JPRB, 0.54319E+00_JPRB ,&
&  0.54514E+00_JPRB, 0.54547E+00_JPRB, 0.54977E+00_JPRB, 0.54905E+00_JPRB, 0.54756E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.48263E+00_JPRB, 0.47682E+00_JPRB, 0.46972E+00_JPRB, 0.46378E+00_JPRB, 0.46087E+00_JPRB, 0.45871E+00_JPRB, 0.45486E+00_JPRB ,&
&  0.45157E+00_JPRB, 0.44867E+00_JPRB, 0.44558E+00_JPRB, 0.44298E+00_JPRB, 0.44072E+00_JPRB, 0.43893E+00_JPRB, 0.43553E+00_JPRB ,&
&  0.42961E+00_JPRB, 0.42199E+00_JPRB, 0.41333E+00_JPRB, 0.40591E+00_JPRB, 0.39685E+00_JPRB, 0.38748E+00_JPRB, 0.38044E+00_JPRB ,&
&  0.37464E+00_JPRB, 0.37067E+00_JPRB, 0.36909E+00_JPRB, 0.36703E+00_JPRB, 0.36767E+00_JPRB, 0.37067E+00_JPRB, 0.38050E+00_JPRB ,&
&  0.39943E+00_JPRB, 0.38156E+00_JPRB, 0.40384E+00_JPRB, 0.42444E+00_JPRB, 0.46593E+00_JPRB, 0.52635E+00_JPRB, 0.56608E+00_JPRB ,&
&  0.59068E+00_JPRB, 0.61031E+00_JPRB, 0.62401E+00_JPRB, 0.62528E+00_JPRB, 0.61507E+00_JPRB, 0.60953E+00_JPRB, 0.60618E+00_JPRB ,&
&  0.59549E+00_JPRB, 0.57782E+00_JPRB, 0.55368E+00_JPRB, 0.52814E+00_JPRB, 0.50524E+00_JPRB, 0.48809E+00_JPRB, 0.47491E+00_JPRB ,&
&  0.46470E+00_JPRB, 0.45645E+00_JPRB, 0.45029E+00_JPRB, 0.44054E+00_JPRB, 0.42922E+00_JPRB, 0.41776E+00_JPRB, 0.40661E+00_JPRB ,&
&  0.39556E+00_JPRB, 0.38485E+00_JPRB, 0.37559E+00_JPRB, 0.36802E+00_JPRB, 0.36365E+00_JPRB, 0.36129E+00_JPRB, 0.36013E+00_JPRB ,&
&  0.36019E+00_JPRB, 0.36206E+00_JPRB, 0.36509E+00_JPRB, 0.36886E+00_JPRB, 0.37228E+00_JPRB, 0.37349E+00_JPRB, 0.37163E+00_JPRB ,&
&  0.37081E+00_JPRB, 0.37038E+00_JPRB, 0.37120E+00_JPRB, 0.37426E+00_JPRB, 0.37659E+00_JPRB, 0.37932E+00_JPRB, 0.38340E+00_JPRB ,&
&  0.38671E+00_JPRB, 0.38825E+00_JPRB, 0.38921E+00_JPRB, 0.39047E+00_JPRB, 0.39222E+00_JPRB, 0.39537E+00_JPRB, 0.40034E+00_JPRB ,&
&  0.40631E+00_JPRB, 0.41309E+00_JPRB, 0.42308E+00_JPRB, 0.43627E+00_JPRB, 0.44869E+00_JPRB, 0.45912E+00_JPRB, 0.46777E+00_JPRB ,&
&  0.47144E+00_JPRB, 0.47441E+00_JPRB, 0.47989E+00_JPRB, 0.48288E+00_JPRB, 0.48506E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.43373E+00_JPRB, 0.43092E+00_JPRB, 0.42752E+00_JPRB, 0.42457E+00_JPRB, 0.42236E+00_JPRB, 0.41842E+00_JPRB, 0.41387E+00_JPRB ,&
&  0.41117E+00_JPRB, 0.40889E+00_JPRB, 0.40747E+00_JPRB, 0.40481E+00_JPRB, 0.40368E+00_JPRB, 0.40318E+00_JPRB, 0.39934E+00_JPRB ,&
&  0.39447E+00_JPRB, 0.38715E+00_JPRB, 0.37980E+00_JPRB, 0.37300E+00_JPRB, 0.36642E+00_JPRB, 0.36019E+00_JPRB, 0.35332E+00_JPRB ,&
&  0.34691E+00_JPRB, 0.34210E+00_JPRB, 0.33935E+00_JPRB, 0.33707E+00_JPRB, 0.33834E+00_JPRB, 0.34436E+00_JPRB, 0.35422E+00_JPRB ,&
&  0.36335E+00_JPRB, 0.37385E+00_JPRB, 0.38903E+00_JPRB, 0.40526E+00_JPRB, 0.43668E+00_JPRB, 0.48110E+00_JPRB, 0.51271E+00_JPRB ,&
&  0.53008E+00_JPRB, 0.54968E+00_JPRB, 0.57361E+00_JPRB, 0.58909E+00_JPRB, 0.59027E+00_JPRB, 0.58199E+00_JPRB, 0.57549E+00_JPRB ,&
&  0.56684E+00_JPRB, 0.55052E+00_JPRB, 0.53266E+00_JPRB, 0.51165E+00_JPRB, 0.49017E+00_JPRB, 0.47248E+00_JPRB, 0.45866E+00_JPRB ,&
&  0.44658E+00_JPRB, 0.43544E+00_JPRB, 0.42495E+00_JPRB, 0.41380E+00_JPRB, 0.40158E+00_JPRB, 0.38813E+00_JPRB, 0.37379E+00_JPRB ,&
&  0.36006E+00_JPRB, 0.34865E+00_JPRB, 0.34002E+00_JPRB, 0.33535E+00_JPRB, 0.33259E+00_JPRB, 0.33013E+00_JPRB, 0.32832E+00_JPRB ,&
&  0.32815E+00_JPRB, 0.32921E+00_JPRB, 0.33126E+00_JPRB, 0.33398E+00_JPRB, 0.33679E+00_JPRB, 0.33817E+00_JPRB, 0.33628E+00_JPRB ,&
&  0.33388E+00_JPRB, 0.33389E+00_JPRB, 0.33484E+00_JPRB, 0.33306E+00_JPRB, 0.33071E+00_JPRB, 0.33171E+00_JPRB, 0.33490E+00_JPRB ,&
&  0.34015E+00_JPRB, 0.34483E+00_JPRB, 0.34839E+00_JPRB, 0.35116E+00_JPRB, 0.35311E+00_JPRB, 0.35644E+00_JPRB, 0.36061E+00_JPRB ,&
&  0.36598E+00_JPRB, 0.37280E+00_JPRB, 0.38235E+00_JPRB, 0.39429E+00_JPRB, 0.40509E+00_JPRB, 0.41400E+00_JPRB, 0.41920E+00_JPRB ,&
&  0.42237E+00_JPRB, 0.42454E+00_JPRB, 0.42673E+00_JPRB, 0.43001E+00_JPRB, 0.43311E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.38617E+00_JPRB, 0.38618E+00_JPRB, 0.38679E+00_JPRB, 0.38673E+00_JPRB, 0.38347E+00_JPRB, 0.38020E+00_JPRB, 0.37655E+00_JPRB ,&
&  0.37493E+00_JPRB, 0.37390E+00_JPRB, 0.37379E+00_JPRB, 0.37115E+00_JPRB, 0.36933E+00_JPRB, 0.36854E+00_JPRB, 0.36519E+00_JPRB ,&
&  0.36153E+00_JPRB, 0.35325E+00_JPRB, 0.34438E+00_JPRB, 0.33787E+00_JPRB, 0.33375E+00_JPRB, 0.33075E+00_JPRB, 0.32488E+00_JPRB ,&
&  0.32043E+00_JPRB, 0.31687E+00_JPRB, 0.31505E+00_JPRB, 0.31498E+00_JPRB, 0.31719E+00_JPRB, 0.32333E+00_JPRB, 0.33340E+00_JPRB ,&
&  0.34107E+00_JPRB, 0.35462E+00_JPRB, 0.35677E+00_JPRB, 0.37453E+00_JPRB, 0.39424E+00_JPRB, 0.42358E+00_JPRB, 0.45079E+00_JPRB ,&
&  0.46693E+00_JPRB, 0.47808E+00_JPRB, 0.49506E+00_JPRB, 0.51460E+00_JPRB, 0.52391E+00_JPRB, 0.52616E+00_JPRB, 0.52315E+00_JPRB ,&
&  0.51756E+00_JPRB, 0.51035E+00_JPRB, 0.50191E+00_JPRB, 0.49095E+00_JPRB, 0.47602E+00_JPRB, 0.45942E+00_JPRB, 0.44334E+00_JPRB ,&
&  0.42882E+00_JPRB, 0.41468E+00_JPRB, 0.40064E+00_JPRB, 0.38530E+00_JPRB, 0.36966E+00_JPRB, 0.35484E+00_JPRB, 0.34030E+00_JPRB ,&
&  0.32847E+00_JPRB, 0.31951E+00_JPRB, 0.31298E+00_JPRB, 0.30818E+00_JPRB, 0.30449E+00_JPRB, 0.30094E+00_JPRB, 0.29909E+00_JPRB ,&
&  0.29880E+00_JPRB, 0.30023E+00_JPRB, 0.30286E+00_JPRB, 0.30516E+00_JPRB, 0.30799E+00_JPRB, 0.30854E+00_JPRB, 0.30530E+00_JPRB ,&
&  0.30205E+00_JPRB, 0.30135E+00_JPRB, 0.29765E+00_JPRB, 0.29554E+00_JPRB, 0.29526E+00_JPRB, 0.29599E+00_JPRB, 0.29831E+00_JPRB ,&
&  0.30269E+00_JPRB, 0.30853E+00_JPRB, 0.31398E+00_JPRB, 0.31882E+00_JPRB, 0.32318E+00_JPRB, 0.32743E+00_JPRB, 0.33221E+00_JPRB ,&
&  0.33721E+00_JPRB, 0.34202E+00_JPRB, 0.34937E+00_JPRB, 0.35763E+00_JPRB, 0.36470E+00_JPRB, 0.37235E+00_JPRB, 0.37694E+00_JPRB ,&
&  0.37870E+00_JPRB, 0.37923E+00_JPRB, 0.38111E+00_JPRB, 0.38413E+00_JPRB, 0.38601E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.34656E+00_JPRB, 0.34817E+00_JPRB, 0.35017E+00_JPRB, 0.35035E+00_JPRB, 0.34951E+00_JPRB, 0.34922E+00_JPRB, 0.34799E+00_JPRB ,&
&  0.34556E+00_JPRB, 0.34420E+00_JPRB, 0.34443E+00_JPRB, 0.34307E+00_JPRB, 0.34032E+00_JPRB, 0.33700E+00_JPRB, 0.33338E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.32702E+00_JPRB, 0.32127E+00_JPRB, 0.31382E+00_JPRB, 0.30741E+00_JPRB, 0.30320E+00_JPRB, 0.30083E+00_JPRB ,&
&  0.29965E+00_JPRB, 0.29935E+00_JPRB, 0.29878E+00_JPRB, 0.29929E+00_JPRB, 0.30143E+00_JPRB, 0.30647E+00_JPRB, 0.31398E+00_JPRB ,&
&  0.32143E+00_JPRB, 0.32917E+00_JPRB, 0.33712E+00_JPRB, 0.34603E+00_JPRB, 0.35696E+00_JPRB, 0.37180E+00_JPRB, 0.38744E+00_JPRB ,&
&  0.40324E+00_JPRB, 0.40895E+00_JPRB, 0.41103E+00_JPRB, 0.41801E+00_JPRB, 0.43047E+00_JPRB, 0.44005E+00_JPRB, 0.44643E+00_JPRB ,&
&  0.44992E+00_JPRB, 0.44931E+00_JPRB, 0.44676E+00_JPRB, 0.44025E+00_JPRB, 0.43040E+00_JPRB, 0.41776E+00_JPRB, 0.40408E+00_JPRB ,&
&  0.39067E+00_JPRB, 0.37677E+00_JPRB, 0.36195E+00_JPRB, 0.34754E+00_JPRB, 0.33470E+00_JPRB, 0.32308E+00_JPRB, 0.31201E+00_JPRB ,&
&  0.30229E+00_JPRB, 0.29473E+00_JPRB, 0.28880E+00_JPRB, 0.28459E+00_JPRB, 0.28108E+00_JPRB, 0.27828E+00_JPRB, 0.27719E+00_JPRB ,&
&  0.27724E+00_JPRB, 0.27837E+00_JPRB, 0.27899E+00_JPRB, 0.28060E+00_JPRB, 0.28338E+00_JPRB, 0.28433E+00_JPRB, 0.28274E+00_JPRB ,&
&  0.27836E+00_JPRB, 0.27423E+00_JPRB, 0.27035E+00_JPRB, 0.26998E+00_JPRB, 0.27026E+00_JPRB, 0.27201E+00_JPRB, 0.27439E+00_JPRB ,&
&  0.27866E+00_JPRB, 0.28425E+00_JPRB, 0.28942E+00_JPRB, 0.29341E+00_JPRB, 0.29802E+00_JPRB, 0.30213E+00_JPRB, 0.30611E+00_JPRB ,&
&  0.31035E+00_JPRB, 0.31471E+00_JPRB, 0.32035E+00_JPRB, 0.32605E+00_JPRB, 0.33125E+00_JPRB, 0.33534E+00_JPRB, 0.33824E+00_JPRB ,&
&  0.34009E+00_JPRB, 0.34054E+00_JPRB, 0.34082E+00_JPRB, 0.34243E+00_JPRB, 0.34479E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.31222E+00_JPRB, 0.31443E+00_JPRB, 0.31725E+00_JPRB, 0.31938E+00_JPRB, 0.32002E+00_JPRB, 0.31936E+00_JPRB, 0.31847E+00_JPRB ,&
&  0.31809E+00_JPRB, 0.31765E+00_JPRB, 0.31596E+00_JPRB, 0.31332E+00_JPRB, 0.31070E+00_JPRB, 0.30800E+00_JPRB, 0.30620E+00_JPRB ,&
&  0.30451E+00_JPRB, 0.30245E+00_JPRB, 0.29846E+00_JPRB, 0.29243E+00_JPRB, 0.28618E+00_JPRB, 0.28260E+00_JPRB, 0.28101E+00_JPRB ,&
&  0.28103E+00_JPRB, 0.28165E+00_JPRB, 0.28416E+00_JPRB, 0.28584E+00_JPRB, 0.28819E+00_JPRB, 0.29226E+00_JPRB, 0.29800E+00_JPRB ,&
&  0.30337E+00_JPRB, 0.30860E+00_JPRB, 0.31417E+00_JPRB, 0.31917E+00_JPRB, 0.32416E+00_JPRB, 0.33081E+00_JPRB, 0.33877E+00_JPRB ,&
&  0.34921E+00_JPRB, 0.35705E+00_JPRB, 0.35765E+00_JPRB, 0.35830E+00_JPRB, 0.36237E+00_JPRB, 0.36791E+00_JPRB, 0.37232E+00_JPRB ,&
&  0.37581E+00_JPRB, 0.37542E+00_JPRB, 0.37345E+00_JPRB, 0.36947E+00_JPRB, 0.36456E+00_JPRB, 0.35863E+00_JPRB, 0.35061E+00_JPRB ,&
&  0.34092E+00_JPRB, 0.33034E+00_JPRB, 0.32006E+00_JPRB, 0.31014E+00_JPRB, 0.30051E+00_JPRB, 0.29173E+00_JPRB, 0.28399E+00_JPRB ,&
&  0.27697E+00_JPRB, 0.27153E+00_JPRB, 0.26633E+00_JPRB, 0.26295E+00_JPRB, 0.26065E+00_JPRB, 0.25812E+00_JPRB, 0.25736E+00_JPRB ,&
&  0.25610E+00_JPRB, 0.25532E+00_JPRB, 0.25512E+00_JPRB, 0.25662E+00_JPRB, 0.25973E+00_JPRB, 0.26161E+00_JPRB, 0.26024E+00_JPRB ,&
&  0.25682E+00_JPRB, 0.25479E+00_JPRB, 0.25364E+00_JPRB, 0.25287E+00_JPRB, 0.25064E+00_JPRB, 0.25063E+00_JPRB, 0.25302E+00_JPRB ,&
&  0.25808E+00_JPRB, 0.26331E+00_JPRB, 0.26750E+00_JPRB, 0.27015E+00_JPRB, 0.27312E+00_JPRB, 0.27580E+00_JPRB, 0.27817E+00_JPRB ,&
&  0.28058E+00_JPRB, 0.28229E+00_JPRB, 0.28352E+00_JPRB, 0.28511E+00_JPRB, 0.28760E+00_JPRB, 0.29000E+00_JPRB, 0.29233E+00_JPRB ,&
&  0.29496E+00_JPRB, 0.29916E+00_JPRB, 0.30342E+00_JPRB, 0.30656E+00_JPRB, 0.31010E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.28832E+00_JPRB, 0.29104E+00_JPRB, 0.29338E+00_JPRB, 0.29317E+00_JPRB, 0.29094E+00_JPRB, 0.28973E+00_JPRB, 0.28976E+00_JPRB ,&
&  0.29001E+00_JPRB, 0.28968E+00_JPRB, 0.28804E+00_JPRB, 0.28776E+00_JPRB, 0.28773E+00_JPRB, 0.28689E+00_JPRB, 0.28649E+00_JPRB ,&
&  0.28546E+00_JPRB, 0.28248E+00_JPRB, 0.27797E+00_JPRB, 0.27242E+00_JPRB, 0.26838E+00_JPRB, 0.26647E+00_JPRB, 0.26649E+00_JPRB ,&
&  0.26761E+00_JPRB, 0.26841E+00_JPRB, 0.27100E+00_JPRB, 0.27302E+00_JPRB, 0.27541E+00_JPRB, 0.27876E+00_JPRB, 0.28241E+00_JPRB ,&
&  0.28498E+00_JPRB, 0.28778E+00_JPRB, 0.29079E+00_JPRB, 0.29375E+00_JPRB, 0.29767E+00_JPRB, 0.30367E+00_JPRB, 0.30902E+00_JPRB ,&
&  0.31624E+00_JPRB, 0.32128E+00_JPRB, 0.31851E+00_JPRB, 0.31469E+00_JPRB, 0.31449E+00_JPRB, 0.31694E+00_JPRB, 0.31995E+00_JPRB ,&
&  0.32307E+00_JPRB, 0.32613E+00_JPRB, 0.32858E+00_JPRB, 0.32884E+00_JPRB, 0.32633E+00_JPRB, 0.32126E+00_JPRB, 0.31410E+00_JPRB ,&
&  0.30603E+00_JPRB, 0.29770E+00_JPRB, 0.28954E+00_JPRB, 0.28125E+00_JPRB, 0.27412E+00_JPRB, 0.26745E+00_JPRB, 0.26148E+00_JPRB ,&
&  0.25595E+00_JPRB, 0.25071E+00_JPRB, 0.24667E+00_JPRB, 0.24396E+00_JPRB, 0.24260E+00_JPRB, 0.24086E+00_JPRB, 0.24047E+00_JPRB ,&
&  0.23955E+00_JPRB, 0.23911E+00_JPRB, 0.23905E+00_JPRB, 0.23973E+00_JPRB, 0.24110E+00_JPRB, 0.24186E+00_JPRB, 0.24116E+00_JPRB ,&
&  0.24078E+00_JPRB, 0.24224E+00_JPRB, 0.24208E+00_JPRB, 0.23954E+00_JPRB, 0.23637E+00_JPRB, 0.23572E+00_JPRB, 0.23739E+00_JPRB ,&
&  0.24053E+00_JPRB, 0.24395E+00_JPRB, 0.24698E+00_JPRB, 0.24955E+00_JPRB, 0.25172E+00_JPRB, 0.25437E+00_JPRB, 0.25675E+00_JPRB ,&
&  0.25882E+00_JPRB, 0.26006E+00_JPRB, 0.26193E+00_JPRB, 0.26392E+00_JPRB, 0.26592E+00_JPRB, 0.26730E+00_JPRB, 0.26713E+00_JPRB ,&
&  0.26656E+00_JPRB, 0.26823E+00_JPRB, 0.27333E+00_JPRB, 0.28073E+00_JPRB, 0.28535E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.26765E+00_JPRB, 0.26988E+00_JPRB, 0.27095E+00_JPRB, 0.27039E+00_JPRB, 0.26938E+00_JPRB, 0.26982E+00_JPRB, 0.27025E+00_JPRB ,&
&  0.27092E+00_JPRB, 0.27085E+00_JPRB, 0.26922E+00_JPRB, 0.26773E+00_JPRB, 0.26921E+00_JPRB, 0.26886E+00_JPRB, 0.26853E+00_JPRB ,&
&  0.26710E+00_JPRB, 0.26527E+00_JPRB, 0.26244E+00_JPRB, 0.25921E+00_JPRB, 0.25536E+00_JPRB, 0.25335E+00_JPRB, 0.25417E+00_JPRB ,&
&  0.25613E+00_JPRB, 0.25690E+00_JPRB, 0.25766E+00_JPRB, 0.25846E+00_JPRB, 0.26029E+00_JPRB, 0.26270E+00_JPRB, 0.26445E+00_JPRB ,&
&  0.26574E+00_JPRB, 0.26866E+00_JPRB, 0.27049E+00_JPRB, 0.26997E+00_JPRB, 0.27421E+00_JPRB, 0.27878E+00_JPRB, 0.28326E+00_JPRB ,&
&  0.28788E+00_JPRB, 0.29081E+00_JPRB, 0.29101E+00_JPRB, 0.28977E+00_JPRB, 0.28874E+00_JPRB, 0.28978E+00_JPRB, 0.29056E+00_JPRB ,&
&  0.29171E+00_JPRB, 0.29521E+00_JPRB, 0.29923E+00_JPRB, 0.30111E+00_JPRB, 0.30009E+00_JPRB, 0.29579E+00_JPRB, 0.28851E+00_JPRB ,&
&  0.27875E+00_JPRB, 0.26827E+00_JPRB, 0.25956E+00_JPRB, 0.25259E+00_JPRB, 0.24729E+00_JPRB, 0.24317E+00_JPRB, 0.23981E+00_JPRB ,&
&  0.23608E+00_JPRB, 0.23332E+00_JPRB, 0.23302E+00_JPRB, 0.23338E+00_JPRB, 0.23299E+00_JPRB, 0.23059E+00_JPRB, 0.22892E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22789E+00_JPRB, 0.22900E+00_JPRB, 0.22944E+00_JPRB, 0.22966E+00_JPRB, 0.22960E+00_JPRB, 0.22952E+00_JPRB ,&
&  0.23058E+00_JPRB, 0.23045E+00_JPRB, 0.22916E+00_JPRB, 0.22707E+00_JPRB, 0.22527E+00_JPRB, 0.22506E+00_JPRB, 0.22569E+00_JPRB ,&
&  0.22675E+00_JPRB, 0.22866E+00_JPRB, 0.23105E+00_JPRB, 0.23347E+00_JPRB, 0.23535E+00_JPRB, 0.23627E+00_JPRB, 0.23675E+00_JPRB ,&
&  0.23712E+00_JPRB, 0.23730E+00_JPRB, 0.23891E+00_JPRB, 0.24293E+00_JPRB, 0.24673E+00_JPRB, 0.24957E+00_JPRB, 0.25096E+00_JPRB ,&
&  0.25031E+00_JPRB, 0.25081E+00_JPRB, 0.25391E+00_JPRB, 0.25976E+00_JPRB, 0.26412E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.25176E+00_JPRB, 0.25390E+00_JPRB, 0.25435E+00_JPRB, 0.25364E+00_JPRB, 0.25283E+00_JPRB, 0.25357E+00_JPRB, 0.25478E+00_JPRB ,&
&  0.25540E+00_JPRB, 0.25545E+00_JPRB, 0.25428E+00_JPRB, 0.25202E+00_JPRB, 0.25021E+00_JPRB, 0.25020E+00_JPRB, 0.24867E+00_JPRB ,&
&  0.24759E+00_JPRB, 0.24846E+00_JPRB, 0.24788E+00_JPRB, 0.24696E+00_JPRB, 0.24517E+00_JPRB, 0.24436E+00_JPRB, 0.24542E+00_JPRB ,&
&  0.24776E+00_JPRB, 0.24829E+00_JPRB, 0.24881E+00_JPRB, 0.24947E+00_JPRB, 0.25083E+00_JPRB, 0.25260E+00_JPRB, 0.25355E+00_JPRB ,&
&  0.25398E+00_JPRB, 0.25247E+00_JPRB, 0.25108E+00_JPRB, 0.25345E+00_JPRB, 0.25716E+00_JPRB, 0.26215E+00_JPRB, 0.26564E+00_JPRB ,&
&  0.26869E+00_JPRB, 0.27086E+00_JPRB, 0.27179E+00_JPRB, 0.27162E+00_JPRB, 0.27181E+00_JPRB, 0.27381E+00_JPRB, 0.27547E+00_JPRB ,&
&  0.27625E+00_JPRB, 0.27639E+00_JPRB, 0.27528E+00_JPRB, 0.27429E+00_JPRB, 0.27109E+00_JPRB, 0.26589E+00_JPRB, 0.25763E+00_JPRB ,&
&  0.24879E+00_JPRB, 0.24150E+00_JPRB, 0.23600E+00_JPRB, 0.23168E+00_JPRB, 0.22878E+00_JPRB, 0.22865E+00_JPRB, 0.22862E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22743E+00_JPRB, 0.22867E+00_JPRB, 0.22900E+00_JPRB, 0.22368E+00_JPRB, 0.21894E+00_JPRB, 0.21404E+00_JPRB ,&
&  0.21276E+00_JPRB, 0.21309E+00_JPRB, 0.21347E+00_JPRB, 0.21626E+00_JPRB, 0.22047E+00_JPRB, 0.22140E+00_JPRB, 0.22006E+00_JPRB ,&
&  0.21380E+00_JPRB, 0.20903E+00_JPRB, 0.20598E+00_JPRB, 0.20330E+00_JPRB, 0.20183E+00_JPRB, 0.20227E+00_JPRB, 0.20322E+00_JPRB ,&
&  0.20575E+00_JPRB, 0.20740E+00_JPRB, 0.21016E+00_JPRB, 0.21176E+00_JPRB, 0.21173E+00_JPRB, 0.21183E+00_JPRB, 0.21120E+00_JPRB ,&
&  0.21314E+00_JPRB, 0.21831E+00_JPRB, 0.22571E+00_JPRB, 0.22807E+00_JPRB, 0.23294E+00_JPRB, 0.23634E+00_JPRB, 0.23778E+00_JPRB ,&
&  0.23858E+00_JPRB, 0.23877E+00_JPRB, 0.23862E+00_JPRB, 0.24250E+00_JPRB, 0.24819E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.23861E+00_JPRB, 0.24014E+00_JPRB, 0.24054E+00_JPRB, 0.24032E+00_JPRB, 0.23990E+00_JPRB, 0.24015E+00_JPRB, 0.24052E+00_JPRB ,&
&  0.24035E+00_JPRB, 0.23949E+00_JPRB, 0.23816E+00_JPRB, 0.23586E+00_JPRB, 0.23297E+00_JPRB, 0.23122E+00_JPRB, 0.23223E+00_JPRB ,&
&  0.23388E+00_JPRB, 0.23508E+00_JPRB, 0.23643E+00_JPRB, 0.23715E+00_JPRB, 0.23736E+00_JPRB, 0.23806E+00_JPRB, 0.23953E+00_JPRB ,&
&  0.24080E+00_JPRB, 0.24136E+00_JPRB, 0.24255E+00_JPRB, 0.24344E+00_JPRB, 0.24541E+00_JPRB, 0.24716E+00_JPRB, 0.24744E+00_JPRB ,&
&  0.24373E+00_JPRB, 0.23578E+00_JPRB, 0.23085E+00_JPRB, 0.23355E+00_JPRB, 0.24084E+00_JPRB, 0.24665E+00_JPRB, 0.25050E+00_JPRB ,&
&  0.25300E+00_JPRB, 0.25408E+00_JPRB, 0.25423E+00_JPRB, 0.25553E+00_JPRB, 0.25768E+00_JPRB, 0.25989E+00_JPRB, 0.25959E+00_JPRB ,&
&  0.25765E+00_JPRB, 0.25606E+00_JPRB, 0.25372E+00_JPRB, 0.24940E+00_JPRB, 0.24331E+00_JPRB, 0.23724E+00_JPRB, 0.23186E+00_JPRB ,&
&  0.22691E+00_JPRB, 0.22217E+00_JPRB, 0.21840E+00_JPRB, 0.21599E+00_JPRB, 0.21540E+00_JPRB, 0.21703E+00_JPRB, 0.21656E+00_JPRB ,&
&  0.21711E+00_JPRB, 0.21517E+00_JPRB, 0.21290E+00_JPRB, 0.21005E+00_JPRB, 0.20760E+00_JPRB, 0.20299E+00_JPRB, 0.19709E+00_JPRB ,&
&  0.19417E+00_JPRB, 0.19715E+00_JPRB, 0.20479E+00_JPRB, 0.21034E+00_JPRB, 0.20719E+00_JPRB, 0.20108E+00_JPRB, 0.19789E+00_JPRB ,&
&  0.19388E+00_JPRB, 0.18940E+00_JPRB, 0.18643E+00_JPRB, 0.18573E+00_JPRB, 0.18594E+00_JPRB, 0.18622E+00_JPRB, 0.18741E+00_JPRB ,&
&  0.18931E+00_JPRB, 0.19093E+00_JPRB, 0.19181E+00_JPRB, 0.19295E+00_JPRB, 0.19345E+00_JPRB, 0.19357E+00_JPRB, 0.19371E+00_JPRB ,&
&  0.19570E+00_JPRB, 0.19975E+00_JPRB, 0.20492E+00_JPRB, 0.21058E+00_JPRB, 0.21481E+00_JPRB, 0.21867E+00_JPRB, 0.22174E+00_JPRB ,&
&  0.22457E+00_JPRB, 0.22525E+00_JPRB, 0.22831E+00_JPRB, 0.23270E+00_JPRB, 0.23604E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.22382E+00_JPRB, 0.22770E+00_JPRB, 0.22904E+00_JPRB, 0.22893E+00_JPRB, 0.22770E+00_JPRB, 0.22656E+00_JPRB, 0.22616E+00_JPRB ,&
&  0.22583E+00_JPRB, 0.22461E+00_JPRB, 0.22372E+00_JPRB, 0.22310E+00_JPRB, 0.22244E+00_JPRB, 0.22154E+00_JPRB, 0.22183E+00_JPRB ,&
&  0.22332E+00_JPRB, 0.22567E+00_JPRB, 0.22765E+00_JPRB, 0.22897E+00_JPRB, 0.22892E+00_JPRB, 0.22954E+00_JPRB, 0.23056E+00_JPRB ,&
&  0.23060E+00_JPRB, 0.23048E+00_JPRB, 0.23358E+00_JPRB, 0.23352E+00_JPRB, 0.23429E+00_JPRB, 0.23405E+00_JPRB, 0.23398E+00_JPRB ,&
&  0.22849E+00_JPRB, 0.22288E+00_JPRB, 0.22074E+00_JPRB, 0.22323E+00_JPRB, 0.23102E+00_JPRB, 0.23232E+00_JPRB, 0.23630E+00_JPRB ,&
&  0.23855E+00_JPRB, 0.23941E+00_JPRB, 0.24107E+00_JPRB, 0.24242E+00_JPRB, 0.24401E+00_JPRB, 0.24341E+00_JPRB, 0.24123E+00_JPRB ,&
&  0.23814E+00_JPRB, 0.23714E+00_JPRB, 0.23379E+00_JPRB, 0.21885E+00_JPRB, 0.21002E+00_JPRB, 0.20496E+00_JPRB, 0.19956E+00_JPRB ,&
&  0.19457E+00_JPRB, 0.19214E+00_JPRB, 0.19175E+00_JPRB, 0.19018E+00_JPRB, 0.19114E+00_JPRB, 0.19372E+00_JPRB, 0.19482E+00_JPRB ,&
&  0.19454E+00_JPRB, 0.19507E+00_JPRB, 0.19477E+00_JPRB, 0.19367E+00_JPRB, 0.19153E+00_JPRB, 0.19022E+00_JPRB, 0.18951E+00_JPRB ,&
&  0.19016E+00_JPRB, 0.18974E+00_JPRB, 0.19404E+00_JPRB, 0.19503E+00_JPRB, 0.18886E+00_JPRB, 0.18818E+00_JPRB, 0.18684E+00_JPRB ,&
&  0.18096E+00_JPRB, 0.17771E+00_JPRB, 0.17585E+00_JPRB, 0.17513E+00_JPRB, 0.17628E+00_JPRB, 0.17792E+00_JPRB, 0.17868E+00_JPRB ,&
&  0.17891E+00_JPRB, 0.18018E+00_JPRB, 0.18076E+00_JPRB, 0.18139E+00_JPRB, 0.18180E+00_JPRB, 0.18278E+00_JPRB, 0.18423E+00_JPRB ,&
&  0.18600E+00_JPRB, 0.18928E+00_JPRB, 0.19245E+00_JPRB, 0.19598E+00_JPRB, 0.19798E+00_JPRB, 0.19950E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.20501E+00_JPRB, 0.21265E+00_JPRB, 0.22355E+00_JPRB, 0.22507E+00_JPRB, 0.21885E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.21582E+00_JPRB, 0.21757E+00_JPRB, 0.21885E+00_JPRB, 0.21782E+00_JPRB, 0.21799E+00_JPRB, 0.21780E+00_JPRB, 0.21779E+00_JPRB ,&
&  0.21762E+00_JPRB, 0.21691E+00_JPRB, 0.21722E+00_JPRB, 0.21503E+00_JPRB, 0.21384E+00_JPRB, 0.21072E+00_JPRB, 0.21014E+00_JPRB ,&
&  0.20861E+00_JPRB, 0.20871E+00_JPRB, 0.20926E+00_JPRB, 0.21089E+00_JPRB, 0.21137E+00_JPRB, 0.21293E+00_JPRB, 0.21201E+00_JPRB ,&
&  0.21243E+00_JPRB, 0.21273E+00_JPRB, 0.21132E+00_JPRB, 0.21087E+00_JPRB, 0.21112E+00_JPRB, 0.21125E+00_JPRB, 0.21148E+00_JPRB ,&
&  0.21376E+00_JPRB, 0.21613E+00_JPRB, 0.21838E+00_JPRB, 0.21950E+00_JPRB, 0.22414E+00_JPRB, 0.22544E+00_JPRB, 0.22782E+00_JPRB ,&
&  0.22889E+00_JPRB, 0.23014E+00_JPRB, 0.23124E+00_JPRB, 0.23048E+00_JPRB, 0.22917E+00_JPRB, 0.22489E+00_JPRB, 0.22340E+00_JPRB ,&
&  0.21339E+00_JPRB, 0.20640E+00_JPRB, 0.19951E+00_JPRB, 0.19317E+00_JPRB, 0.18831E+00_JPRB, 0.18497E+00_JPRB, 0.18231E+00_JPRB ,&
&  0.18015E+00_JPRB, 0.17800E+00_JPRB, 0.17559E+00_JPRB, 0.17580E+00_JPRB, 0.17672E+00_JPRB, 0.17816E+00_JPRB, 0.17964E+00_JPRB ,&
&  0.18136E+00_JPRB, 0.18235E+00_JPRB, 0.18277E+00_JPRB, 0.18363E+00_JPRB, 0.18379E+00_JPRB, 0.18416E+00_JPRB, 0.18442E+00_JPRB ,&
&  0.18595E+00_JPRB, 0.18641E+00_JPRB, 0.18643E+00_JPRB, 0.18824E+00_JPRB, 0.18616E+00_JPRB, 0.18307E+00_JPRB, 0.17988E+00_JPRB ,&
&  0.17649E+00_JPRB, 0.17438E+00_JPRB, 0.17256E+00_JPRB, 0.17187E+00_JPRB, 0.17234E+00_JPRB, 0.17322E+00_JPRB, 0.17431E+00_JPRB ,&
&  0.17450E+00_JPRB, 0.17521E+00_JPRB, 0.17573E+00_JPRB, 0.17670E+00_JPRB, 0.17773E+00_JPRB, 0.17893E+00_JPRB, 0.18028E+00_JPRB ,&
&  0.18234E+00_JPRB, 0.18471E+00_JPRB, 0.18705E+00_JPRB, 0.18944E+00_JPRB, 0.19328E+00_JPRB, 0.19772E+00_JPRB, 0.20466E+00_JPRB ,&
&  0.21016E+00_JPRB, 0.21595E+00_JPRB, 0.21313E+00_JPRB, 0.21707E+00_JPRB, 0.21751E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.21044E+00_JPRB, 0.21115E+00_JPRB, 0.20894E+00_JPRB, 0.20914E+00_JPRB, 0.20981E+00_JPRB, 0.21013E+00_JPRB, 0.21060E+00_JPRB ,&
&  0.20798E+00_JPRB, 0.20733E+00_JPRB, 0.20507E+00_JPRB, 0.20330E+00_JPRB, 0.20093E+00_JPRB, 0.19858E+00_JPRB, 0.19685E+00_JPRB ,&
&  0.19493E+00_JPRB, 0.19440E+00_JPRB, 0.19352E+00_JPRB, 0.19464E+00_JPRB, 0.19471E+00_JPRB, 0.19661E+00_JPRB, 0.19814E+00_JPRB ,&
&  0.19889E+00_JPRB, 0.20004E+00_JPRB, 0.19840E+00_JPRB, 0.19779E+00_JPRB, 0.19944E+00_JPRB, 0.20079E+00_JPRB, 0.20340E+00_JPRB ,&
&  0.20604E+00_JPRB, 0.20934E+00_JPRB, 0.21484E+00_JPRB, 0.21672E+00_JPRB, 0.21921E+00_JPRB, 0.22070E+00_JPRB, 0.22056E+00_JPRB ,&
&  0.22025E+00_JPRB, 0.21983E+00_JPRB, 0.22057E+00_JPRB, 0.22019E+00_JPRB, 0.21296E+00_JPRB, 0.20758E+00_JPRB, 0.20181E+00_JPRB ,&
&  0.19663E+00_JPRB, 0.19158E+00_JPRB, 0.18699E+00_JPRB, 0.18391E+00_JPRB, 0.18046E+00_JPRB, 0.17803E+00_JPRB, 0.17614E+00_JPRB ,&
&  0.17425E+00_JPRB, 0.17324E+00_JPRB, 0.17184E+00_JPRB, 0.17146E+00_JPRB, 0.17186E+00_JPRB, 0.17284E+00_JPRB, 0.17422E+00_JPRB ,&
&  0.17583E+00_JPRB, 0.17693E+00_JPRB, 0.17831E+00_JPRB, 0.17959E+00_JPRB, 0.18052E+00_JPRB, 0.18108E+00_JPRB, 0.18075E+00_JPRB ,&
&  0.18100E+00_JPRB, 0.18135E+00_JPRB, 0.18089E+00_JPRB, 0.18092E+00_JPRB, 0.17966E+00_JPRB, 0.17817E+00_JPRB, 0.17664E+00_JPRB ,&
&  0.17452E+00_JPRB, 0.17288E+00_JPRB, 0.17231E+00_JPRB, 0.17200E+00_JPRB, 0.17283E+00_JPRB, 0.17407E+00_JPRB, 0.17485E+00_JPRB ,&
&  0.17521E+00_JPRB, 0.17508E+00_JPRB, 0.17506E+00_JPRB, 0.17638E+00_JPRB, 0.17774E+00_JPRB, 0.17947E+00_JPRB, 0.18129E+00_JPRB ,&
&  0.18182E+00_JPRB, 0.18328E+00_JPRB, 0.18454E+00_JPRB, 0.18617E+00_JPRB, 0.19183E+00_JPRB, 0.19606E+00_JPRB, 0.20413E+00_JPRB ,&
&  0.20722E+00_JPRB, 0.20604E+00_JPRB, 0.20374E+00_JPRB, 0.20914E+00_JPRB, 0.21008E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.20874E+00_JPRB, 0.21129E+00_JPRB, 0.20970E+00_JPRB, 0.20938E+00_JPRB, 0.20853E+00_JPRB, 0.20820E+00_JPRB, 0.20680E+00_JPRB ,&
&  0.20657E+00_JPRB, 0.20482E+00_JPRB, 0.20458E+00_JPRB, 0.20292E+00_JPRB, 0.20122E+00_JPRB, 0.19879E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19402E+00_JPRB, 0.19198E+00_JPRB, 0.19063E+00_JPRB, 0.19016E+00_JPRB, 0.18907E+00_JPRB, 0.18931E+00_JPRB, 0.19185E+00_JPRB ,&
&  0.19241E+00_JPRB, 0.19366E+00_JPRB, 0.19370E+00_JPRB, 0.19494E+00_JPRB, 0.19658E+00_JPRB, 0.19693E+00_JPRB, 0.19914E+00_JPRB ,&
&  0.19973E+00_JPRB, 0.20129E+00_JPRB, 0.20804E+00_JPRB, 0.21156E+00_JPRB, 0.21335E+00_JPRB, 0.21631E+00_JPRB, 0.21691E+00_JPRB ,&
&  0.21522E+00_JPRB, 0.20986E+00_JPRB, 0.21284E+00_JPRB, 0.20965E+00_JPRB, 0.20278E+00_JPRB, 0.20004E+00_JPRB, 0.19452E+00_JPRB ,&
&  0.19169E+00_JPRB, 0.18794E+00_JPRB, 0.18515E+00_JPRB, 0.18299E+00_JPRB, 0.17961E+00_JPRB, 0.17748E+00_JPRB, 0.17507E+00_JPRB ,&
&  0.17287E+00_JPRB, 0.17202E+00_JPRB, 0.17055E+00_JPRB, 0.17050E+00_JPRB, 0.17077E+00_JPRB, 0.17134E+00_JPRB, 0.17226E+00_JPRB ,&
&  0.17342E+00_JPRB, 0.17450E+00_JPRB, 0.17550E+00_JPRB, 0.17635E+00_JPRB, 0.17706E+00_JPRB, 0.17746E+00_JPRB, 0.17730E+00_JPRB ,&
&  0.17743E+00_JPRB, 0.17735E+00_JPRB, 0.17684E+00_JPRB, 0.17665E+00_JPRB, 0.17562E+00_JPRB, 0.17519E+00_JPRB, 0.17474E+00_JPRB ,&
&  0.17455E+00_JPRB, 0.17457E+00_JPRB, 0.17447E+00_JPRB, 0.17469E+00_JPRB, 0.17498E+00_JPRB, 0.17575E+00_JPRB, 0.17634E+00_JPRB ,&
&  0.17682E+00_JPRB, 0.17781E+00_JPRB, 0.17841E+00_JPRB, 0.17925E+00_JPRB, 0.18043E+00_JPRB, 0.18090E+00_JPRB, 0.18222E+00_JPRB ,&
&  0.18332E+00_JPRB, 0.18466E+00_JPRB, 0.18720E+00_JPRB, 0.18972E+00_JPRB, 0.19340E+00_JPRB, 0.19784E+00_JPRB, 0.19921E+00_JPRB ,&
&  0.20019E+00_JPRB, 0.20026E+00_JPRB, 0.19793E+00_JPRB, 0.20350E+00_JPRB, 0.20613E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.20157E+00_JPRB, 0.20354E+00_JPRB, 0.20481E+00_JPRB, 0.20568E+00_JPRB, 0.20714E+00_JPRB, 0.20781E+00_JPRB, 0.20839E+00_JPRB ,&
&  0.20640E+00_JPRB, 0.20586E+00_JPRB, 0.20484E+00_JPRB, 0.20354E+00_JPRB, 0.20268E+00_JPRB, 0.20012E+00_JPRB, 0.19765E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19264E+00_JPRB, 0.19137E+00_JPRB, 0.19011E+00_JPRB, 0.18878E+00_JPRB, 0.18768E+00_JPRB, 0.19010E+00_JPRB ,&
&  0.19153E+00_JPRB, 0.19241E+00_JPRB, 0.19390E+00_JPRB, 0.19352E+00_JPRB, 0.19488E+00_JPRB, 0.19705E+00_JPRB, 0.19996E+00_JPRB ,&
&  0.20155E+00_JPRB, 0.20452E+00_JPRB, 0.20623E+00_JPRB, 0.20709E+00_JPRB, 0.20835E+00_JPRB, 0.20929E+00_JPRB, 0.20902E+00_JPRB ,&
&  0.20761E+00_JPRB, 0.20574E+00_JPRB, 0.20349E+00_JPRB, 0.20055E+00_JPRB, 0.19743E+00_JPRB, 0.19445E+00_JPRB, 0.19146E+00_JPRB ,&
&  0.18934E+00_JPRB, 0.18739E+00_JPRB, 0.18528E+00_JPRB, 0.18334E+00_JPRB, 0.18148E+00_JPRB, 0.17980E+00_JPRB, 0.17811E+00_JPRB ,&
&  0.17679E+00_JPRB, 0.17574E+00_JPRB, 0.17443E+00_JPRB, 0.17380E+00_JPRB, 0.17316E+00_JPRB, 0.17291E+00_JPRB, 0.17283E+00_JPRB ,&
&  0.17336E+00_JPRB, 0.17402E+00_JPRB, 0.17468E+00_JPRB, 0.17506E+00_JPRB, 0.17543E+00_JPRB, 0.17561E+00_JPRB, 0.17594E+00_JPRB ,&
&  0.17626E+00_JPRB, 0.17661E+00_JPRB, 0.17618E+00_JPRB, 0.17577E+00_JPRB, 0.17521E+00_JPRB, 0.17458E+00_JPRB, 0.17438E+00_JPRB ,&
&  0.17470E+00_JPRB, 0.17496E+00_JPRB, 0.17474E+00_JPRB, 0.17505E+00_JPRB, 0.17561E+00_JPRB, 0.17618E+00_JPRB, 0.17766E+00_JPRB ,&
&  0.17847E+00_JPRB, 0.17924E+00_JPRB, 0.17995E+00_JPRB, 0.18050E+00_JPRB, 0.18136E+00_JPRB, 0.18205E+00_JPRB, 0.18292E+00_JPRB ,&
&  0.18397E+00_JPRB, 0.18498E+00_JPRB, 0.18637E+00_JPRB, 0.18816E+00_JPRB, 0.19251E+00_JPRB, 0.19524E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19817E+00_JPRB, 0.19352E+00_JPRB, 0.20071E+00_JPRB, 0.19935E+00_JPRB, 0.19887E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.19432E+00_JPRB, 0.19608E+00_JPRB, 0.19738E+00_JPRB, 0.19920E+00_JPRB, 0.19375E+00_JPRB, 0.20061E+00_JPRB, 0.19730E+00_JPRB ,&
&  0.19390E+00_JPRB, 0.18899E+00_JPRB, 0.18938E+00_JPRB, 0.18872E+00_JPRB, 0.18769E+00_JPRB, 0.18733E+00_JPRB, 0.18744E+00_JPRB ,&
&  0.18773E+00_JPRB, 0.18772E+00_JPRB, 0.18844E+00_JPRB, 0.18898E+00_JPRB, 0.18940E+00_JPRB, 0.18974E+00_JPRB, 0.19020E+00_JPRB ,&
&  0.19089E+00_JPRB, 0.19198E+00_JPRB, 0.19312E+00_JPRB, 0.19093E+00_JPRB, 0.19063E+00_JPRB, 0.19083E+00_JPRB, 0.19141E+00_JPRB ,&
&  0.19171E+00_JPRB, 0.19285E+00_JPRB, 0.19376E+00_JPRB, 0.19445E+00_JPRB, 0.19645E+00_JPRB, 0.19661E+00_JPRB, 0.19650E+00_JPRB ,&
&  0.19609E+00_JPRB, 0.19519E+00_JPRB, 0.19393E+00_JPRB, 0.19276E+00_JPRB, 0.19154E+00_JPRB, 0.18993E+00_JPRB, 0.18838E+00_JPRB ,&
&  0.18711E+00_JPRB, 0.18608E+00_JPRB, 0.18473E+00_JPRB, 0.18342E+00_JPRB, 0.18223E+00_JPRB, 0.18110E+00_JPRB, 0.18013E+00_JPRB ,&
&  0.17943E+00_JPRB, 0.17883E+00_JPRB, 0.17821E+00_JPRB, 0.17774E+00_JPRB, 0.17724E+00_JPRB, 0.17679E+00_JPRB, 0.17636E+00_JPRB ,&
&  0.17658E+00_JPRB, 0.17676E+00_JPRB, 0.17694E+00_JPRB, 0.17713E+00_JPRB, 0.17765E+00_JPRB, 0.17778E+00_JPRB, 0.17794E+00_JPRB ,&
&  0.17805E+00_JPRB, 0.17798E+00_JPRB, 0.17786E+00_JPRB, 0.17762E+00_JPRB, 0.17735E+00_JPRB, 0.17700E+00_JPRB, 0.17684E+00_JPRB ,&
&  0.17681E+00_JPRB, 0.17693E+00_JPRB, 0.17730E+00_JPRB, 0.17757E+00_JPRB, 0.17783E+00_JPRB, 0.17806E+00_JPRB, 0.17858E+00_JPRB ,&
&  0.17912E+00_JPRB, 0.17965E+00_JPRB, 0.18016E+00_JPRB, 0.18002E+00_JPRB, 0.18029E+00_JPRB, 0.18053E+00_JPRB, 0.18077E+00_JPRB ,&
&  0.18092E+00_JPRB, 0.18144E+00_JPRB, 0.18228E+00_JPRB, 0.18344E+00_JPRB, 0.18558E+00_JPRB, 0.18736E+00_JPRB, 0.18901E+00_JPRB ,&
&  0.19077E+00_JPRB, 0.19435E+00_JPRB, 0.19560E+00_JPRB, 0.19586E+00_JPRB, 0.19562E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.18452E+00_JPRB, 0.18543E+00_JPRB, 0.18605E+00_JPRB, 0.18652E+00_JPRB, 0.18627E+00_JPRB, 0.18630E+00_JPRB, 0.18631E+00_JPRB ,&
&  0.18621E+00_JPRB, 0.18612E+00_JPRB, 0.18641E+00_JPRB, 0.18665E+00_JPRB, 0.18695E+00_JPRB, 0.18727E+00_JPRB, 0.18768E+00_JPRB ,&
&  0.18783E+00_JPRB, 0.18819E+00_JPRB, 0.18678E+00_JPRB, 0.18560E+00_JPRB, 0.18481E+00_JPRB, 0.18430E+00_JPRB, 0.18402E+00_JPRB ,&
&  0.18378E+00_JPRB, 0.18362E+00_JPRB, 0.18345E+00_JPRB, 0.18445E+00_JPRB, 0.18508E+00_JPRB, 0.18564E+00_JPRB, 0.18607E+00_JPRB ,&
&  0.18622E+00_JPRB, 0.18647E+00_JPRB, 0.18668E+00_JPRB, 0.18691E+00_JPRB, 0.18718E+00_JPRB, 0.18716E+00_JPRB, 0.18704E+00_JPRB ,&
&  0.18684E+00_JPRB, 0.18650E+00_JPRB, 0.18610E+00_JPRB, 0.18572E+00_JPRB, 0.18541E+00_JPRB, 0.18504E+00_JPRB, 0.18463E+00_JPRB ,&
&  0.18425E+00_JPRB, 0.18380E+00_JPRB, 0.18338E+00_JPRB, 0.18297E+00_JPRB, 0.18258E+00_JPRB, 0.18226E+00_JPRB, 0.18190E+00_JPRB ,&
&  0.18166E+00_JPRB, 0.18144E+00_JPRB, 0.18121E+00_JPRB, 0.18106E+00_JPRB, 0.18080E+00_JPRB, 0.18071E+00_JPRB, 0.18050E+00_JPRB ,&
&  0.18043E+00_JPRB, 0.18036E+00_JPRB, 0.18032E+00_JPRB, 0.18026E+00_JPRB, 0.18028E+00_JPRB, 0.18028E+00_JPRB, 0.18029E+00_JPRB ,&
&  0.18028E+00_JPRB, 0.18045E+00_JPRB, 0.18034E+00_JPRB, 0.18018E+00_JPRB, 0.18000E+00_JPRB, 0.17982E+00_JPRB, 0.17965E+00_JPRB ,&
&  0.17950E+00_JPRB, 0.17934E+00_JPRB, 0.17924E+00_JPRB, 0.17927E+00_JPRB, 0.17932E+00_JPRB, 0.17941E+00_JPRB, 0.17950E+00_JPRB ,&
&  0.17960E+00_JPRB, 0.17973E+00_JPRB, 0.17986E+00_JPRB, 0.17982E+00_JPRB, 0.17984E+00_JPRB, 0.17987E+00_JPRB, 0.17989E+00_JPRB ,&
&  0.17991E+00_JPRB, 0.17996E+00_JPRB, 0.18006E+00_JPRB, 0.18020E+00_JPRB, 0.18051E+00_JPRB, 0.18063E+00_JPRB, 0.18082E+00_JPRB ,&
&  0.18107E+00_JPRB, 0.18152E+00_JPRB, 0.18191E+00_JPRB, 0.18233E+00_JPRB, 0.18282E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB, 0.18149E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB, 0.61583E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.61267E+00_JPRB, 0.61181E+00_JPRB, 0.61104E+00_JPRB, 0.61028E+00_JPRB, 0.60940E+00_JPRB, 0.60880E+00_JPRB, 0.60830E+00_JPRB ,&
&  0.60782E+00_JPRB, 0.60734E+00_JPRB, 0.60682E+00_JPRB, 0.60642E+00_JPRB, 0.60617E+00_JPRB, 0.60582E+00_JPRB, 0.60546E+00_JPRB ,&
&  0.60502E+00_JPRB, 0.60464E+00_JPRB, 0.60441E+00_JPRB, 0.60386E+00_JPRB, 0.60334E+00_JPRB, 0.60287E+00_JPRB, 0.60207E+00_JPRB ,&
&  0.60098E+00_JPRB, 0.59996E+00_JPRB, 0.59885E+00_JPRB, 0.59793E+00_JPRB, 0.59723E+00_JPRB, 0.59618E+00_JPRB, 0.59492E+00_JPRB ,&
&  0.59383E+00_JPRB, 0.59312E+00_JPRB, 0.59272E+00_JPRB, 0.59253E+00_JPRB, 0.59253E+00_JPRB, 0.59255E+00_JPRB, 0.59253E+00_JPRB ,&
&  0.59259E+00_JPRB, 0.59294E+00_JPRB, 0.59356E+00_JPRB, 0.59447E+00_JPRB, 0.59562E+00_JPRB, 0.59704E+00_JPRB, 0.59906E+00_JPRB ,&
&  0.60132E+00_JPRB, 0.60363E+00_JPRB, 0.60601E+00_JPRB, 0.60814E+00_JPRB, 0.61005E+00_JPRB, 0.61171E+00_JPRB, 0.61289E+00_JPRB ,&
&  0.61420E+00_JPRB, 0.61560E+00_JPRB, 0.61704E+00_JPRB, 0.61808E+00_JPRB, 0.61899E+00_JPRB, 0.61984E+00_JPRB, 0.62077E+00_JPRB ,&
&  0.62122E+00_JPRB, 0.62156E+00_JPRB, 0.62199E+00_JPRB, 0.62260E+00_JPRB, 0.62296E+00_JPRB, 0.62319E+00_JPRB, 0.62344E+00_JPRB ,&
&  0.62389E+00_JPRB, 0.62423E+00_JPRB, 0.62437E+00_JPRB, 0.62455E+00_JPRB, 0.62492E+00_JPRB, 0.62549E+00_JPRB, 0.62608E+00_JPRB ,&
&  0.62659E+00_JPRB, 0.62706E+00_JPRB, 0.62737E+00_JPRB, 0.62752E+00_JPRB, 0.62746E+00_JPRB, 0.62712E+00_JPRB, 0.62627E+00_JPRB ,&
&  0.62523E+00_JPRB, 0.62407E+00_JPRB, 0.62310E+00_JPRB, 0.62248E+00_JPRB, 0.62180E+00_JPRB, 0.62123E+00_JPRB, 0.62081E+00_JPRB ,&
&  0.62066E+00_JPRB, 0.62059E+00_JPRB, 0.62049E+00_JPRB, 0.62047E+00_JPRB, 0.62032E+00_JPRB, 0.62000E+00_JPRB, 0.61950E+00_JPRB ,&
&  0.61880E+00_JPRB, 0.61781E+00_JPRB, 0.61662E+00_JPRB, 0.61530E+00_JPRB, 0.61394E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.60319E+00_JPRB, 0.60123E+00_JPRB, 0.59982E+00_JPRB, 0.59893E+00_JPRB, 0.59809E+00_JPRB, 0.59735E+00_JPRB, 0.59662E+00_JPRB ,&
&  0.59587E+00_JPRB, 0.59589E+00_JPRB, 0.59552E+00_JPRB, 0.59526E+00_JPRB, 0.59491E+00_JPRB, 0.59507E+00_JPRB, 0.59491E+00_JPRB ,&
&  0.59445E+00_JPRB, 0.59317E+00_JPRB, 0.59225E+00_JPRB, 0.59084E+00_JPRB, 0.59076E+00_JPRB, 0.59074E+00_JPRB, 0.59201E+00_JPRB ,&
&  0.59166E+00_JPRB, 0.59108E+00_JPRB, 0.58969E+00_JPRB, 0.58540E+00_JPRB, 0.58161E+00_JPRB, 0.57793E+00_JPRB, 0.57483E+00_JPRB ,&
&  0.57203E+00_JPRB, 0.57001E+00_JPRB, 0.56932E+00_JPRB, 0.56940E+00_JPRB, 0.57221E+00_JPRB, 0.57296E+00_JPRB, 0.57314E+00_JPRB ,&
&  0.57235E+00_JPRB, 0.56905E+00_JPRB, 0.56980E+00_JPRB, 0.57085E+00_JPRB, 0.57252E+00_JPRB, 0.57647E+00_JPRB, 0.57997E+00_JPRB ,&
&  0.58419E+00_JPRB, 0.58904E+00_JPRB, 0.59671E+00_JPRB, 0.60222E+00_JPRB, 0.60781E+00_JPRB, 0.61258E+00_JPRB, 0.61516E+00_JPRB ,&
&  0.61754E+00_JPRB, 0.61968E+00_JPRB, 0.62181E+00_JPRB, 0.62348E+00_JPRB, 0.62481E+00_JPRB, 0.62624E+00_JPRB, 0.62790E+00_JPRB ,&
&  0.62899E+00_JPRB, 0.63004E+00_JPRB, 0.63119E+00_JPRB, 0.63230E+00_JPRB, 0.63299E+00_JPRB, 0.63358E+00_JPRB, 0.63437E+00_JPRB ,&
&  0.63519E+00_JPRB, 0.63500E+00_JPRB, 0.63481E+00_JPRB, 0.63541E+00_JPRB, 0.63607E+00_JPRB, 0.63674E+00_JPRB, 0.63658E+00_JPRB ,&
&  0.63664E+00_JPRB, 0.63640E+00_JPRB, 0.63642E+00_JPRB, 0.63637E+00_JPRB, 0.63605E+00_JPRB, 0.63578E+00_JPRB, 0.63497E+00_JPRB ,&
&  0.63374E+00_JPRB, 0.63204E+00_JPRB, 0.63060E+00_JPRB, 0.62988E+00_JPRB, 0.62891E+00_JPRB, 0.62865E+00_JPRB, 0.62864E+00_JPRB ,&
&  0.62895E+00_JPRB, 0.62773E+00_JPRB, 0.62622E+00_JPRB, 0.62428E+00_JPRB, 0.62236E+00_JPRB, 0.62032E+00_JPRB, 0.61839E+00_JPRB ,&
&  0.61641E+00_JPRB, 0.61411E+00_JPRB, 0.61145E+00_JPRB, 0.60862E+00_JPRB, 0.60583E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.60441E+00_JPRB, 0.60073E+00_JPRB, 0.59823E+00_JPRB, 0.59668E+00_JPRB, 0.59512E+00_JPRB, 0.59358E+00_JPRB, 0.59272E+00_JPRB ,&
&  0.59197E+00_JPRB, 0.59040E+00_JPRB, 0.58952E+00_JPRB, 0.58884E+00_JPRB, 0.58774E+00_JPRB, 0.58807E+00_JPRB, 0.58788E+00_JPRB ,&
&  0.58817E+00_JPRB, 0.58741E+00_JPRB, 0.58871E+00_JPRB, 0.58732E+00_JPRB, 0.58634E+00_JPRB, 0.58525E+00_JPRB, 0.58256E+00_JPRB ,&
&  0.58215E+00_JPRB, 0.58130E+00_JPRB, 0.57737E+00_JPRB, 0.57248E+00_JPRB, 0.56938E+00_JPRB, 0.55394E+00_JPRB, 0.55494E+00_JPRB ,&
&  0.56138E+00_JPRB, 0.55217E+00_JPRB, 0.54798E+00_JPRB, 0.54355E+00_JPRB, 0.53803E+00_JPRB, 0.53814E+00_JPRB, 0.54195E+00_JPRB ,&
&  0.53658E+00_JPRB, 0.54968E+00_JPRB, 0.55135E+00_JPRB, 0.55239E+00_JPRB, 0.55392E+00_JPRB, 0.55201E+00_JPRB, 0.55655E+00_JPRB ,&
&  0.57473E+00_JPRB, 0.57815E+00_JPRB, 0.58603E+00_JPRB, 0.59317E+00_JPRB, 0.60202E+00_JPRB, 0.61018E+00_JPRB, 0.61891E+00_JPRB ,&
&  0.62544E+00_JPRB, 0.62991E+00_JPRB, 0.63294E+00_JPRB, 0.63353E+00_JPRB, 0.63417E+00_JPRB, 0.63414E+00_JPRB, 0.63508E+00_JPRB ,&
&  0.63484E+00_JPRB, 0.63594E+00_JPRB, 0.63793E+00_JPRB, 0.64045E+00_JPRB, 0.64378E+00_JPRB, 0.64587E+00_JPRB, 0.64779E+00_JPRB ,&
&  0.64899E+00_JPRB, 0.64940E+00_JPRB, 0.64896E+00_JPRB, 0.64885E+00_JPRB, 0.64774E+00_JPRB, 0.64759E+00_JPRB, 0.64707E+00_JPRB ,&
&  0.64765E+00_JPRB, 0.64735E+00_JPRB, 0.64786E+00_JPRB, 0.64784E+00_JPRB, 0.64849E+00_JPRB, 0.65004E+00_JPRB, 0.65125E+00_JPRB ,&
&  0.65300E+00_JPRB, 0.65371E+00_JPRB, 0.65385E+00_JPRB, 0.65362E+00_JPRB, 0.65286E+00_JPRB, 0.65292E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.65309E+00_JPRB, 0.64982E+00_JPRB, 0.64602E+00_JPRB, 0.64196E+00_JPRB, 0.63839E+00_JPRB, 0.63514E+00_JPRB, 0.63169E+00_JPRB ,&
&  0.62851E+00_JPRB, 0.62423E+00_JPRB, 0.61946E+00_JPRB, 0.61408E+00_JPRB, 0.60889E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.61746E+00_JPRB, 0.61196E+00_JPRB, 0.60713E+00_JPRB, 0.60372E+00_JPRB, 0.60057E+00_JPRB, 0.59800E+00_JPRB, 0.59592E+00_JPRB ,&
&  0.59409E+00_JPRB, 0.59263E+00_JPRB, 0.59181E+00_JPRB, 0.59100E+00_JPRB, 0.59032E+00_JPRB, 0.58935E+00_JPRB, 0.58853E+00_JPRB ,&
&  0.58977E+00_JPRB, 0.58919E+00_JPRB, 0.59052E+00_JPRB, 0.58841E+00_JPRB, 0.58668E+00_JPRB, 0.58318E+00_JPRB, 0.57549E+00_JPRB ,&
&  0.57199E+00_JPRB, 0.56854E+00_JPRB, 0.55483E+00_JPRB, 0.56347E+00_JPRB, 0.55845E+00_JPRB, 0.55566E+00_JPRB, 0.55164E+00_JPRB ,&
&  0.54161E+00_JPRB, 0.53660E+00_JPRB, 0.53010E+00_JPRB, 0.52959E+00_JPRB, 0.53307E+00_JPRB, 0.53059E+00_JPRB, 0.53328E+00_JPRB ,&
&  0.52936E+00_JPRB, 0.52801E+00_JPRB, 0.52864E+00_JPRB, 0.52877E+00_JPRB, 0.53278E+00_JPRB, 0.53355E+00_JPRB, 0.53726E+00_JPRB ,&
&  0.54600E+00_JPRB, 0.55796E+00_JPRB, 0.57625E+00_JPRB, 0.58436E+00_JPRB, 0.59772E+00_JPRB, 0.60846E+00_JPRB, 0.61873E+00_JPRB ,&
&  0.62534E+00_JPRB, 0.62962E+00_JPRB, 0.63263E+00_JPRB, 0.63405E+00_JPRB, 0.63387E+00_JPRB, 0.63074E+00_JPRB, 0.63028E+00_JPRB ,&
&  0.63034E+00_JPRB, 0.63160E+00_JPRB, 0.63349E+00_JPRB, 0.63535E+00_JPRB, 0.63848E+00_JPRB, 0.64148E+00_JPRB, 0.64550E+00_JPRB ,&
&  0.64928E+00_JPRB, 0.65335E+00_JPRB, 0.65421E+00_JPRB, 0.65624E+00_JPRB, 0.65713E+00_JPRB, 0.65823E+00_JPRB, 0.65906E+00_JPRB ,&
&  0.66015E+00_JPRB, 0.66156E+00_JPRB, 0.66414E+00_JPRB, 0.66092E+00_JPRB, 0.66201E+00_JPRB, 0.67141E+00_JPRB, 0.67359E+00_JPRB ,&
&  0.67655E+00_JPRB, 0.67661E+00_JPRB, 0.67473E+00_JPRB, 0.67276E+00_JPRB, 0.67086E+00_JPRB, 0.67013E+00_JPRB, 0.66968E+00_JPRB ,&
&  0.66904E+00_JPRB, 0.66439E+00_JPRB, 0.65964E+00_JPRB, 0.65537E+00_JPRB, 0.65072E+00_JPRB, 0.64717E+00_JPRB, 0.64358E+00_JPRB ,&
&  0.64103E+00_JPRB, 0.63883E+00_JPRB, 0.63613E+00_JPRB, 0.63118E+00_JPRB, 0.62475E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.63378E+00_JPRB, 0.62782E+00_JPRB, 0.62309E+00_JPRB, 0.61904E+00_JPRB, 0.61620E+00_JPRB, 0.61374E+00_JPRB, 0.61136E+00_JPRB ,&
&  0.60884E+00_JPRB, 0.60606E+00_JPRB, 0.60303E+00_JPRB, 0.60089E+00_JPRB, 0.59908E+00_JPRB, 0.59744E+00_JPRB, 0.59540E+00_JPRB ,&
&  0.59435E+00_JPRB, 0.59279E+00_JPRB, 0.59326E+00_JPRB, 0.59073E+00_JPRB, 0.59114E+00_JPRB, 0.58719E+00_JPRB, 0.57810E+00_JPRB ,&
&  0.56957E+00_JPRB, 0.56052E+00_JPRB, 0.54965E+00_JPRB, 0.55696E+00_JPRB, 0.55576E+00_JPRB, 0.55626E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.53643E+00_JPRB, 0.52430E+00_JPRB, 0.54011E+00_JPRB, 0.53349E+00_JPRB, 0.52935E+00_JPRB, 0.52222E+00_JPRB, 0.50899E+00_JPRB ,&
&  0.50626E+00_JPRB, 0.49851E+00_JPRB, 0.49798E+00_JPRB, 0.50160E+00_JPRB, 0.50645E+00_JPRB, 0.51349E+00_JPRB, 0.52073E+00_JPRB ,&
&  0.52159E+00_JPRB, 0.55160E+00_JPRB, 0.57077E+00_JPRB, 0.58501E+00_JPRB, 0.59819E+00_JPRB, 0.60890E+00_JPRB, 0.61731E+00_JPRB ,&
&  0.62484E+00_JPRB, 0.63091E+00_JPRB, 0.63676E+00_JPRB, 0.63845E+00_JPRB, 0.64655E+00_JPRB, 0.64846E+00_JPRB, 0.65112E+00_JPRB ,&
&  0.65417E+00_JPRB, 0.65386E+00_JPRB, 0.65269E+00_JPRB, 0.64975E+00_JPRB, 0.64850E+00_JPRB, 0.64621E+00_JPRB, 0.64752E+00_JPRB ,&
&  0.64773E+00_JPRB, 0.64933E+00_JPRB, 0.65339E+00_JPRB, 0.65729E+00_JPRB, 0.66072E+00_JPRB, 0.66406E+00_JPRB, 0.66754E+00_JPRB ,&
&  0.67082E+00_JPRB, 0.67697E+00_JPRB, 0.68298E+00_JPRB, 0.69377E+00_JPRB, 0.70281E+00_JPRB, 0.70674E+00_JPRB, 0.70960E+00_JPRB ,&
&  0.71337E+00_JPRB, 0.70833E+00_JPRB, 0.70345E+00_JPRB, 0.70011E+00_JPRB, 0.69807E+00_JPRB, 0.69700E+00_JPRB, 0.69717E+00_JPRB ,&
&  0.69623E+00_JPRB, 0.69183E+00_JPRB, 0.68651E+00_JPRB, 0.68023E+00_JPRB, 0.67399E+00_JPRB, 0.66925E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66164E+00_JPRB, 0.65826E+00_JPRB, 0.65297E+00_JPRB, 0.64643E+00_JPRB, 0.63988E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.65716E+00_JPRB, 0.65134E+00_JPRB, 0.64686E+00_JPRB, 0.64260E+00_JPRB, 0.63989E+00_JPRB, 0.63740E+00_JPRB, 0.63519E+00_JPRB ,&
&  0.63342E+00_JPRB, 0.62956E+00_JPRB, 0.62499E+00_JPRB, 0.61942E+00_JPRB, 0.61542E+00_JPRB, 0.61177E+00_JPRB, 0.60770E+00_JPRB ,&
&  0.60171E+00_JPRB, 0.59522E+00_JPRB, 0.58823E+00_JPRB, 0.58259E+00_JPRB, 0.58321E+00_JPRB, 0.58037E+00_JPRB, 0.57622E+00_JPRB ,&
&  0.56797E+00_JPRB, 0.55829E+00_JPRB, 0.54958E+00_JPRB, 0.55094E+00_JPRB, 0.55242E+00_JPRB, 0.54474E+00_JPRB, 0.54409E+00_JPRB ,&
&  0.55437E+00_JPRB, 0.55612E+00_JPRB, 0.55234E+00_JPRB, 0.55032E+00_JPRB, 0.52750E+00_JPRB, 0.49279E+00_JPRB, 0.50960E+00_JPRB ,&
&  0.50857E+00_JPRB, 0.49484E+00_JPRB, 0.49146E+00_JPRB, 0.48936E+00_JPRB, 0.49488E+00_JPRB, 0.50755E+00_JPRB, 0.51994E+00_JPRB ,&
&  0.52428E+00_JPRB, 0.57501E+00_JPRB, 0.59611E+00_JPRB, 0.61067E+00_JPRB, 0.61980E+00_JPRB, 0.62781E+00_JPRB, 0.63505E+00_JPRB ,&
&  0.64144E+00_JPRB, 0.65126E+00_JPRB, 0.66402E+00_JPRB, 0.67648E+00_JPRB, 0.68823E+00_JPRB, 0.69710E+00_JPRB, 0.70643E+00_JPRB ,&
&  0.71914E+00_JPRB, 0.72099E+00_JPRB, 0.71043E+00_JPRB, 0.69859E+00_JPRB, 0.68815E+00_JPRB, 0.67817E+00_JPRB, 0.66970E+00_JPRB ,&
&  0.66592E+00_JPRB, 0.66310E+00_JPRB, 0.66789E+00_JPRB, 0.66917E+00_JPRB, 0.67212E+00_JPRB, 0.67611E+00_JPRB, 0.68651E+00_JPRB ,&
&  0.69334E+00_JPRB, 0.70349E+00_JPRB, 0.71498E+00_JPRB, 0.73028E+00_JPRB, 0.74179E+00_JPRB, 0.74599E+00_JPRB, 0.74842E+00_JPRB ,&
&  0.74822E+00_JPRB, 0.75385E+00_JPRB, 0.75495E+00_JPRB, 0.75861E+00_JPRB, 0.76556E+00_JPRB, 0.76878E+00_JPRB, 0.77264E+00_JPRB ,&
&  0.77431E+00_JPRB, 0.77372E+00_JPRB, 0.76886E+00_JPRB, 0.76059E+00_JPRB, 0.74925E+00_JPRB, 0.73645E+00_JPRB, 0.72131E+00_JPRB ,&
&  0.70616E+00_JPRB, 0.69036E+00_JPRB, 0.67718E+00_JPRB, 0.66899E+00_JPRB, 0.66230E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.69770E+00_JPRB, 0.68597E+00_JPRB, 0.67739E+00_JPRB, 0.67239E+00_JPRB, 0.66831E+00_JPRB, 0.66436E+00_JPRB, 0.66077E+00_JPRB ,&
&  0.65726E+00_JPRB, 0.65348E+00_JPRB, 0.64828E+00_JPRB, 0.64258E+00_JPRB, 0.63768E+00_JPRB, 0.63317E+00_JPRB, 0.62949E+00_JPRB ,&
&  0.62225E+00_JPRB, 0.60438E+00_JPRB, 0.59928E+00_JPRB, 0.59056E+00_JPRB, 0.57741E+00_JPRB, 0.57002E+00_JPRB, 0.56454E+00_JPRB ,&
&  0.55778E+00_JPRB, 0.55475E+00_JPRB, 0.55511E+00_JPRB, 0.55001E+00_JPRB, 0.54746E+00_JPRB, 0.55522E+00_JPRB, 0.55787E+00_JPRB ,&
&  0.53930E+00_JPRB, 0.53956E+00_JPRB, 0.56534E+00_JPRB, 0.56611E+00_JPRB, 0.53792E+00_JPRB, 0.52938E+00_JPRB, 0.52772E+00_JPRB ,&
&  0.52419E+00_JPRB, 0.50312E+00_JPRB, 0.49838E+00_JPRB, 0.49616E+00_JPRB, 0.50522E+00_JPRB, 0.51588E+00_JPRB, 0.54205E+00_JPRB ,&
&  0.56528E+00_JPRB, 0.61210E+00_JPRB, 0.63159E+00_JPRB, 0.64463E+00_JPRB, 0.65580E+00_JPRB, 0.66791E+00_JPRB, 0.68080E+00_JPRB ,&
&  0.69837E+00_JPRB, 0.71874E+00_JPRB, 0.74550E+00_JPRB, 0.76956E+00_JPRB, 0.79317E+00_JPRB, 0.80452E+00_JPRB, 0.80628E+00_JPRB ,&
&  0.82650E+00_JPRB, 0.82919E+00_JPRB, 0.80254E+00_JPRB, 0.78075E+00_JPRB, 0.75799E+00_JPRB, 0.73610E+00_JPRB, 0.71194E+00_JPRB ,&
&  0.70551E+00_JPRB, 0.71310E+00_JPRB, 0.70695E+00_JPRB, 0.70203E+00_JPRB, 0.70051E+00_JPRB, 0.70127E+00_JPRB, 0.70625E+00_JPRB ,&
&  0.70891E+00_JPRB, 0.73658E+00_JPRB, 0.82865E+00_JPRB, 0.85088E+00_JPRB, 0.85696E+00_JPRB, 0.85660E+00_JPRB, 0.85482E+00_JPRB ,&
&  0.85892E+00_JPRB, 0.86862E+00_JPRB, 0.88090E+00_JPRB, 0.88660E+00_JPRB, 0.89273E+00_JPRB, 0.88639E+00_JPRB, 0.89193E+00_JPRB ,&
&  0.88579E+00_JPRB, 0.87807E+00_JPRB, 0.86739E+00_JPRB, 0.85919E+00_JPRB, 0.84933E+00_JPRB, 0.83955E+00_JPRB, 0.82511E+00_JPRB ,&
&  0.80672E+00_JPRB, 0.78476E+00_JPRB, 0.76220E+00_JPRB, 0.73854E+00_JPRB, 0.71601E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.75593E+00_JPRB, 0.72723E+00_JPRB, 0.70794E+00_JPRB, 0.69976E+00_JPRB, 0.69494E+00_JPRB, 0.67874E+00_JPRB, 0.66660E+00_JPRB ,&
&  0.65855E+00_JPRB, 0.65582E+00_JPRB, 0.65161E+00_JPRB, 0.64512E+00_JPRB, 0.64140E+00_JPRB, 0.63809E+00_JPRB, 0.63647E+00_JPRB ,&
&  0.63362E+00_JPRB, 0.62626E+00_JPRB, 0.61067E+00_JPRB, 0.59083E+00_JPRB, 0.57634E+00_JPRB, 0.56546E+00_JPRB, 0.55769E+00_JPRB ,&
&  0.55169E+00_JPRB, 0.55060E+00_JPRB, 0.55766E+00_JPRB, 0.55783E+00_JPRB, 0.55685E+00_JPRB, 0.55262E+00_JPRB, 0.54822E+00_JPRB ,&
&  0.55002E+00_JPRB, 0.56086E+00_JPRB, 0.54831E+00_JPRB, 0.55141E+00_JPRB, 0.56346E+00_JPRB, 0.55434E+00_JPRB, 0.54043E+00_JPRB ,&
&  0.52910E+00_JPRB, 0.51859E+00_JPRB, 0.51327E+00_JPRB, 0.51281E+00_JPRB, 0.52784E+00_JPRB, 0.54822E+00_JPRB, 0.58258E+00_JPRB ,&
&  0.62732E+00_JPRB, 0.65262E+00_JPRB, 0.66639E+00_JPRB, 0.67955E+00_JPRB, 0.69357E+00_JPRB, 0.70915E+00_JPRB, 0.73368E+00_JPRB ,&
&  0.77322E+00_JPRB, 0.81559E+00_JPRB, 0.85774E+00_JPRB, 0.88582E+00_JPRB, 0.91638E+00_JPRB, 0.89293E+00_JPRB, 0.89424E+00_JPRB ,&
&  0.99994E+00_JPRB, 0.10375E+01_JPRB, 0.98093E+00_JPRB, 0.93288E+00_JPRB, 0.88430E+00_JPRB, 0.84191E+00_JPRB, 0.80724E+00_JPRB ,&
&  0.78463E+00_JPRB, 0.77240E+00_JPRB, 0.76308E+00_JPRB, 0.74897E+00_JPRB, 0.73958E+00_JPRB, 0.74491E+00_JPRB, 0.76848E+00_JPRB ,&
&  0.81753E+00_JPRB, 0.95576E+00_JPRB, 0.12400E+01_JPRB, 0.10692E+01_JPRB, 0.10043E+01_JPRB, 0.99241E+00_JPRB, 0.99478E+00_JPRB ,&
&  0.10161E+01_JPRB, 0.10333E+01_JPRB, 0.10414E+01_JPRB, 0.10229E+01_JPRB, 0.98868E+00_JPRB, 0.94901E+00_JPRB, 0.92220E+00_JPRB ,&
&  0.90000E+00_JPRB, 0.87797E+00_JPRB, 0.86271E+00_JPRB, 0.85743E+00_JPRB, 0.84701E+00_JPRB, 0.83583E+00_JPRB, 0.82618E+00_JPRB ,&
&  0.81652E+00_JPRB, 0.81089E+00_JPRB, 0.80904E+00_JPRB, 0.80091E+00_JPRB, 0.78666E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.76579E+00_JPRB, 0.77183E+00_JPRB, 0.76613E+00_JPRB, 0.75267E+00_JPRB, 0.73800E+00_JPRB, 0.69826E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.65255E+00_JPRB, 0.64712E+00_JPRB, 0.64751E+00_JPRB, 0.64641E+00_JPRB, 0.64590E+00_JPRB, 0.64415E+00_JPRB, 0.63868E+00_JPRB ,&
&  0.62907E+00_JPRB, 0.62113E+00_JPRB, 0.61377E+00_JPRB, 0.60129E+00_JPRB, 0.58504E+00_JPRB, 0.56799E+00_JPRB, 0.55462E+00_JPRB ,&
&  0.54562E+00_JPRB, 0.54251E+00_JPRB, 0.54912E+00_JPRB, 0.55631E+00_JPRB, 0.56384E+00_JPRB, 0.56664E+00_JPRB, 0.56469E+00_JPRB ,&
&  0.55372E+00_JPRB, 0.54947E+00_JPRB, 0.55801E+00_JPRB, 0.55982E+00_JPRB, 0.56457E+00_JPRB, 0.56918E+00_JPRB, 0.55758E+00_JPRB ,&
&  0.54609E+00_JPRB, 0.53682E+00_JPRB, 0.53867E+00_JPRB, 0.55145E+00_JPRB, 0.58990E+00_JPRB, 0.64158E+00_JPRB, 0.66065E+00_JPRB ,&
&  0.67760E+00_JPRB, 0.68860E+00_JPRB, 0.69992E+00_JPRB, 0.71934E+00_JPRB, 0.74410E+00_JPRB, 0.77317E+00_JPRB, 0.82630E+00_JPRB ,&
&  0.90111E+00_JPRB, 0.95788E+00_JPRB, 0.10120E+01_JPRB, 0.94310E+00_JPRB, 0.91682E+00_JPRB, 0.94279E+00_JPRB, 0.98269E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.10654E+01_JPRB, 0.10598E+01_JPRB, 0.10677E+01_JPRB, 0.10255E+01_JPRB, 0.97172E+00_JPRB, 0.89747E+00_JPRB ,&
&  0.84264E+00_JPRB, 0.80847E+00_JPRB, 0.79643E+00_JPRB, 0.78743E+00_JPRB, 0.80148E+00_JPRB, 0.83118E+00_JPRB, 0.88152E+00_JPRB ,&
&  0.96118E+00_JPRB, 0.10575E+01_JPRB, 0.11106E+01_JPRB, 0.10759E+01_JPRB, 0.10983E+01_JPRB, 0.11483E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.11847E+01_JPRB, 0.11790E+01_JPRB, 0.11826E+01_JPRB, 0.11701E+01_JPRB, 0.10920E+01_JPRB, 0.10220E+01_JPRB, 0.94321E+00_JPRB ,&
&  0.86778E+00_JPRB, 0.80030E+00_JPRB, 0.76455E+00_JPRB, 0.75417E+00_JPRB, 0.74605E+00_JPRB, 0.73642E+00_JPRB, 0.72030E+00_JPRB ,&
&  0.70516E+00_JPRB, 0.70336E+00_JPRB, 0.71514E+00_JPRB, 0.73084E+00_JPRB, 0.75229E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.80472E+00_JPRB, 0.82046E+00_JPRB, 0.82491E+00_JPRB, 0.82636E+00_JPRB, 0.79036E+00_JPRB, 0.75570E+00_JPRB, 0.72190E+00_JPRB ,&
&  0.70520E+00_JPRB, 0.68340E+00_JPRB, 0.66942E+00_JPRB, 0.65135E+00_JPRB, 0.63529E+00_JPRB, 0.62661E+00_JPRB, 0.62643E+00_JPRB ,&
&  0.62798E+00_JPRB, 0.62248E+00_JPRB, 0.61339E+00_JPRB, 0.59851E+00_JPRB, 0.58525E+00_JPRB, 0.57319E+00_JPRB, 0.56254E+00_JPRB ,&
&  0.55440E+00_JPRB, 0.55317E+00_JPRB, 0.55666E+00_JPRB, 0.56488E+00_JPRB, 0.57117E+00_JPRB, 0.57520E+00_JPRB, 0.57597E+00_JPRB ,&
&  0.57079E+00_JPRB, 0.56257E+00_JPRB, 0.56554E+00_JPRB, 0.57473E+00_JPRB, 0.58005E+00_JPRB, 0.58903E+00_JPRB, 0.57416E+00_JPRB ,&
&  0.56347E+00_JPRB, 0.56642E+00_JPRB, 0.58651E+00_JPRB, 0.64633E+00_JPRB, 0.67212E+00_JPRB, 0.68554E+00_JPRB, 0.69741E+00_JPRB ,&
&  0.68791E+00_JPRB, 0.68755E+00_JPRB, 0.70649E+00_JPRB, 0.75237E+00_JPRB, 0.79929E+00_JPRB, 0.87698E+00_JPRB, 0.97480E+00_JPRB ,&
&  0.10504E+01_JPRB, 0.10834E+01_JPRB, 0.11049E+01_JPRB, 0.10327E+01_JPRB, 0.10334E+01_JPRB, 0.11732E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.12432E+01_JPRB, 0.12569E+01_JPRB, 0.12937E+01_JPRB, 0.12559E+01_JPRB, 0.11408E+01_JPRB, 0.10561E+01_JPRB, 0.99451E+00_JPRB ,&
&  0.93167E+00_JPRB, 0.89722E+00_JPRB, 0.89677E+00_JPRB, 0.92742E+00_JPRB, 0.98968E+00_JPRB, 0.11088E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.13990E+01_JPRB, 0.14831E+01_JPRB, 0.15350E+01_JPRB, 0.15243E+01_JPRB, 0.15033E+01_JPRB, 0.14723E+01_JPRB, 0.14492E+01_JPRB ,&
&  0.14204E+01_JPRB, 0.14109E+01_JPRB, 0.13971E+01_JPRB, 0.13810E+01_JPRB, 0.12842E+01_JPRB, 0.11218E+01_JPRB, 0.95917E+00_JPRB ,&
&  0.84472E+00_JPRB, 0.77794E+00_JPRB, 0.73218E+00_JPRB, 0.72128E+00_JPRB, 0.72334E+00_JPRB, 0.72480E+00_JPRB, 0.70551E+00_JPRB ,&
&  0.69736E+00_JPRB, 0.70834E+00_JPRB, 0.72641E+00_JPRB, 0.74556E+00_JPRB, 0.77386E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.86736E+00_JPRB, 0.88086E+00_JPRB, 0.88721E+00_JPRB, 0.87975E+00_JPRB, 0.85605E+00_JPRB, 0.81009E+00_JPRB, 0.75248E+00_JPRB ,&
&  0.73414E+00_JPRB, 0.70056E+00_JPRB, 0.68533E+00_JPRB, 0.66861E+00_JPRB, 0.65447E+00_JPRB, 0.63484E+00_JPRB, 0.61468E+00_JPRB ,&
&  0.61029E+00_JPRB, 0.61498E+00_JPRB, 0.61656E+00_JPRB, 0.61067E+00_JPRB, 0.60053E+00_JPRB, 0.58848E+00_JPRB, 0.57781E+00_JPRB ,&
&  0.57097E+00_JPRB, 0.57114E+00_JPRB, 0.57898E+00_JPRB, 0.58926E+00_JPRB, 0.59368E+00_JPRB, 0.59649E+00_JPRB, 0.60165E+00_JPRB ,&
&  0.60164E+00_JPRB, 0.59533E+00_JPRB, 0.59277E+00_JPRB, 0.59689E+00_JPRB, 0.59964E+00_JPRB, 0.61642E+00_JPRB, 0.62000E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.60967E+00_JPRB, 0.66142E+00_JPRB, 0.68404E+00_JPRB, 0.69997E+00_JPRB, 0.71238E+00_JPRB, 0.72371E+00_JPRB ,&
&  0.73478E+00_JPRB, 0.75143E+00_JPRB, 0.77034E+00_JPRB, 0.79003E+00_JPRB, 0.86600E+00_JPRB, 0.10395E+01_JPRB, 0.11191E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.10771E+01_JPRB, 0.10170E+01_JPRB, 0.10320E+01_JPRB, 0.11960E+01_JPRB, 0.12901E+01_JPRB, 0.13468E+01_JPRB ,&
&  0.14304E+01_JPRB, 0.14368E+01_JPRB, 0.14183E+01_JPRB, 0.13619E+01_JPRB, 0.12670E+01_JPRB, 0.11967E+01_JPRB, 0.11782E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.11511E+01_JPRB, 0.12224E+01_JPRB, 0.13711E+01_JPRB, 0.15416E+01_JPRB, 0.16844E+01_JPRB, 0.17866E+01_JPRB ,&
&  0.18829E+01_JPRB, 0.19279E+01_JPRB, 0.19060E+01_JPRB, 0.18630E+01_JPRB, 0.17858E+01_JPRB, 0.17407E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.16996E+01_JPRB, 0.16793E+01_JPRB, 0.16368E+01_JPRB, 0.15749E+01_JPRB, 0.14717E+01_JPRB, 0.13690E+01_JPRB, 0.12257E+01_JPRB ,&
&  0.10006E+01_JPRB, 0.87164E+00_JPRB, 0.77808E+00_JPRB, 0.75154E+00_JPRB, 0.75366E+00_JPRB, 0.76330E+00_JPRB, 0.76080E+00_JPRB ,&
&  0.77683E+00_JPRB, 0.78771E+00_JPRB, 0.79117E+00_JPRB, 0.81257E+00_JPRB, 0.84078E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.92940E+00_JPRB, 0.93434E+00_JPRB, 0.93022E+00_JPRB, 0.92460E+00_JPRB, 0.90923E+00_JPRB, 0.86687E+00_JPRB, 0.82820E+00_JPRB ,&
&  0.87355E+00_JPRB, 0.82980E+00_JPRB, 0.83898E+00_JPRB, 0.73536E+00_JPRB, 0.69478E+00_JPRB, 0.68110E+00_JPRB, 0.65486E+00_JPRB ,&
&  0.64048E+00_JPRB, 0.63699E+00_JPRB, 0.63011E+00_JPRB, 0.62743E+00_JPRB, 0.62216E+00_JPRB, 0.60931E+00_JPRB, 0.59916E+00_JPRB ,&
&  0.59513E+00_JPRB, 0.59944E+00_JPRB, 0.60956E+00_JPRB, 0.61765E+00_JPRB, 0.62477E+00_JPRB, 0.63099E+00_JPRB, 0.63581E+00_JPRB ,&
&  0.62503E+00_JPRB, 0.62058E+00_JPRB, 0.62043E+00_JPRB, 0.61668E+00_JPRB, 0.62104E+00_JPRB, 0.63302E+00_JPRB, 0.65441E+00_JPRB ,&
&  0.67182E+00_JPRB, 0.66195E+00_JPRB, 0.70157E+00_JPRB, 0.72073E+00_JPRB, 0.73510E+00_JPRB, 0.74646E+00_JPRB, 0.75508E+00_JPRB ,&
&  0.76428E+00_JPRB, 0.78003E+00_JPRB, 0.79953E+00_JPRB, 0.82111E+00_JPRB, 0.95781E+00_JPRB, 0.12317E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12700E+01_JPRB, 0.10906E+01_JPRB, 0.12097E+01_JPRB, 0.12706E+01_JPRB, 0.14030E+01_JPRB, 0.15288E+01_JPRB, 0.15354E+01_JPRB ,&
&  0.17206E+01_JPRB, 0.18036E+01_JPRB, 0.17443E+01_JPRB, 0.16181E+01_JPRB, 0.15023E+01_JPRB, 0.14487E+01_JPRB, 0.14331E+01_JPRB ,&
&  0.14649E+01_JPRB, 0.14535E+01_JPRB, 0.16446E+01_JPRB, 0.18049E+01_JPRB, 0.18878E+01_JPRB, 0.19662E+01_JPRB, 0.20594E+01_JPRB ,&
&  0.21233E+01_JPRB, 0.21328E+01_JPRB, 0.21284E+01_JPRB, 0.20179E+01_JPRB, 0.19618E+01_JPRB, 0.19184E+01_JPRB, 0.18375E+01_JPRB ,&
&  0.17854E+01_JPRB, 0.17430E+01_JPRB, 0.16820E+01_JPRB, 0.16912E+01_JPRB, 0.15420E+01_JPRB, 0.14350E+01_JPRB, 0.13658E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.98835E+00_JPRB, 0.91601E+00_JPRB, 0.88448E+00_JPRB, 0.86168E+00_JPRB, 0.85812E+00_JPRB, 0.86390E+00_JPRB ,&
&  0.84158E+00_JPRB, 0.85313E+00_JPRB, 0.89635E+00_JPRB, 0.90956E+00_JPRB, 0.91628E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.99097E+00_JPRB, 0.99818E+00_JPRB, 0.99530E+00_JPRB, 0.98945E+00_JPRB, 0.98023E+00_JPRB, 0.96143E+00_JPRB, 0.91479E+00_JPRB ,&
&  0.92747E+00_JPRB, 0.90070E+00_JPRB, 0.86453E+00_JPRB, 0.84271E+00_JPRB, 0.84192E+00_JPRB, 0.78743E+00_JPRB, 0.69321E+00_JPRB ,&
&  0.65865E+00_JPRB, 0.65847E+00_JPRB, 0.66551E+00_JPRB, 0.66470E+00_JPRB, 0.66086E+00_JPRB, 0.65333E+00_JPRB, 0.64735E+00_JPRB ,&
&  0.64579E+00_JPRB, 0.64837E+00_JPRB, 0.65848E+00_JPRB, 0.67157E+00_JPRB, 0.67478E+00_JPRB, 0.67866E+00_JPRB, 0.68461E+00_JPRB ,&
&  0.66629E+00_JPRB, 0.64938E+00_JPRB, 0.63853E+00_JPRB, 0.62443E+00_JPRB, 0.64807E+00_JPRB, 0.66608E+00_JPRB, 0.69062E+00_JPRB ,&
&  0.71511E+00_JPRB, 0.73803E+00_JPRB, 0.76070E+00_JPRB, 0.77763E+00_JPRB, 0.78673E+00_JPRB, 0.79502E+00_JPRB, 0.80251E+00_JPRB ,&
&  0.81218E+00_JPRB, 0.82845E+00_JPRB, 0.85058E+00_JPRB, 0.87334E+00_JPRB, 0.11385E+01_JPRB, 0.14240E+01_JPRB, 0.15296E+01_JPRB ,&
&  0.15010E+01_JPRB, 0.16399E+01_JPRB, 0.17372E+01_JPRB, 0.16733E+01_JPRB, 0.17885E+01_JPRB, 0.18637E+01_JPRB, 0.18261E+01_JPRB ,&
&  0.19298E+01_JPRB, 0.19432E+01_JPRB, 0.19208E+01_JPRB, 0.18921E+01_JPRB, 0.18796E+01_JPRB, 0.18388E+01_JPRB, 0.18117E+01_JPRB ,&
&  0.18632E+01_JPRB, 0.20439E+01_JPRB, 0.20805E+01_JPRB, 0.21362E+01_JPRB, 0.21482E+01_JPRB, 0.21835E+01_JPRB, 0.22208E+01_JPRB ,&
&  0.22511E+01_JPRB, 0.22926E+01_JPRB, 0.23037E+01_JPRB, 0.22464E+01_JPRB, 0.21634E+01_JPRB, 0.20743E+01_JPRB, 0.19640E+01_JPRB ,&
&  0.18772E+01_JPRB, 0.16932E+01_JPRB, 0.15469E+01_JPRB, 0.14983E+01_JPRB, 0.14509E+01_JPRB, 0.13332E+01_JPRB, 0.12929E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.11535E+01_JPRB, 0.11776E+01_JPRB, 0.11035E+01_JPRB, 0.10474E+01_JPRB, 0.98333E+00_JPRB, 0.89165E+00_JPRB ,&
&  0.86141E+00_JPRB, 0.91776E+00_JPRB, 0.95898E+00_JPRB, 0.98140E+00_JPRB, 0.98610E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.10562E+01_JPRB, 0.10675E+01_JPRB, 0.10739E+01_JPRB, 0.10695E+01_JPRB, 0.10480E+01_JPRB, 0.10175E+01_JPRB, 0.98571E+00_JPRB ,&
&  0.96037E+00_JPRB, 0.92929E+00_JPRB, 0.90091E+00_JPRB, 0.87831E+00_JPRB, 0.86842E+00_JPRB, 0.86031E+00_JPRB, 0.80303E+00_JPRB ,&
&  0.72625E+00_JPRB, 0.70462E+00_JPRB, 0.68673E+00_JPRB, 0.68465E+00_JPRB, 0.69430E+00_JPRB, 0.69926E+00_JPRB, 0.70455E+00_JPRB ,&
&  0.72058E+00_JPRB, 0.73105E+00_JPRB, 0.73560E+00_JPRB, 0.74004E+00_JPRB, 0.74254E+00_JPRB, 0.75065E+00_JPRB, 0.75818E+00_JPRB ,&
&  0.72953E+00_JPRB, 0.69410E+00_JPRB, 0.67521E+00_JPRB, 0.66853E+00_JPRB, 0.69283E+00_JPRB, 0.72482E+00_JPRB, 0.76007E+00_JPRB ,&
&  0.79066E+00_JPRB, 0.81520E+00_JPRB, 0.83777E+00_JPRB, 0.85404E+00_JPRB, 0.85961E+00_JPRB, 0.86528E+00_JPRB, 0.87561E+00_JPRB ,&
&  0.89598E+00_JPRB, 0.90775E+00_JPRB, 0.91910E+00_JPRB, 0.95312E+00_JPRB, 0.13725E+01_JPRB, 0.18802E+01_JPRB, 0.21384E+01_JPRB ,&
&  0.21445E+01_JPRB, 0.21891E+01_JPRB, 0.22491E+01_JPRB, 0.24110E+01_JPRB, 0.24212E+01_JPRB, 0.22864E+01_JPRB, 0.21655E+01_JPRB ,&
&  0.22014E+01_JPRB, 0.21166E+01_JPRB, 0.22905E+01_JPRB, 0.22888E+01_JPRB, 0.23192E+01_JPRB, 0.23740E+01_JPRB, 0.23597E+01_JPRB ,&
&  0.24215E+01_JPRB, 0.24597E+01_JPRB, 0.24363E+01_JPRB, 0.23583E+01_JPRB, 0.23212E+01_JPRB, 0.23516E+01_JPRB, 0.23796E+01_JPRB ,&
&  0.24650E+01_JPRB, 0.25334E+01_JPRB, 0.24298E+01_JPRB, 0.23447E+01_JPRB, 0.22331E+01_JPRB, 0.23381E+01_JPRB, 0.21545E+01_JPRB ,&
&  0.18529E+01_JPRB, 0.14908E+01_JPRB, 0.14738E+01_JPRB, 0.14591E+01_JPRB, 0.14693E+01_JPRB, 0.14243E+01_JPRB, 0.13287E+01_JPRB ,&
&  0.13233E+01_JPRB, 0.14173E+01_JPRB, 0.14074E+01_JPRB, 0.12694E+01_JPRB, 0.11944E+01_JPRB, 0.10999E+01_JPRB, 0.95601E+00_JPRB ,&
&  0.94404E+00_JPRB, 0.10165E+01_JPRB, 0.10350E+01_JPRB, 0.10353E+01_JPRB, 0.10379E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.11588E+01_JPRB, 0.11354E+01_JPRB, 0.11276E+01_JPRB, 0.11183E+01_JPRB, 0.10945E+01_JPRB, 0.10655E+01_JPRB, 0.10329E+01_JPRB ,&
&  0.10073E+01_JPRB, 0.98506E+00_JPRB, 0.96374E+00_JPRB, 0.94537E+00_JPRB, 0.93369E+00_JPRB, 0.91478E+00_JPRB, 0.90379E+00_JPRB ,&
&  0.80323E+00_JPRB, 0.76163E+00_JPRB, 0.73650E+00_JPRB, 0.73316E+00_JPRB, 0.77293E+00_JPRB, 0.77165E+00_JPRB, 0.77926E+00_JPRB ,&
&  0.79025E+00_JPRB, 0.80000E+00_JPRB, 0.79327E+00_JPRB, 0.79296E+00_JPRB, 0.80276E+00_JPRB, 0.83253E+00_JPRB, 0.86682E+00_JPRB ,&
&  0.84095E+00_JPRB, 0.78930E+00_JPRB, 0.76907E+00_JPRB, 0.76336E+00_JPRB, 0.79889E+00_JPRB, 0.82739E+00_JPRB, 0.88592E+00_JPRB ,&
&  0.91467E+00_JPRB, 0.93031E+00_JPRB, 0.93815E+00_JPRB, 0.94268E+00_JPRB, 0.95157E+00_JPRB, 0.97119E+00_JPRB, 0.99930E+00_JPRB ,&
&  0.10244E+01_JPRB, 0.10153E+01_JPRB, 0.10202E+01_JPRB, 0.11075E+01_JPRB, 0.17537E+01_JPRB, 0.24664E+01_JPRB, 0.28950E+01_JPRB ,&
&  0.31118E+01_JPRB, 0.30382E+01_JPRB, 0.30694E+01_JPRB, 0.35317E+01_JPRB, 0.34503E+01_JPRB, 0.31027E+01_JPRB, 0.27873E+01_JPRB ,&
&  0.26561E+01_JPRB, 0.25414E+01_JPRB, 0.26611E+01_JPRB, 0.27639E+01_JPRB, 0.27012E+01_JPRB, 0.27105E+01_JPRB, 0.27087E+01_JPRB ,&
&  0.27221E+01_JPRB, 0.27407E+01_JPRB, 0.26341E+01_JPRB, 0.25249E+01_JPRB, 0.24380E+01_JPRB, 0.25170E+01_JPRB, 0.25737E+01_JPRB ,&
&  0.27313E+01_JPRB, 0.27022E+01_JPRB, 0.24182E+01_JPRB, 0.24057E+01_JPRB, 0.19580E+01_JPRB, 0.18568E+01_JPRB, 0.20988E+01_JPRB ,&
&  0.17886E+01_JPRB, 0.15416E+01_JPRB, 0.14710E+01_JPRB, 0.15046E+01_JPRB, 0.15434E+01_JPRB, 0.16273E+01_JPRB, 0.15678E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.14576E+01_JPRB, 0.16268E+01_JPRB, 0.14729E+01_JPRB, 0.13682E+01_JPRB, 0.12837E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11821E+01_JPRB, 0.11637E+01_JPRB, 0.11517E+01_JPRB, 0.11443E+01_JPRB, 0.11529E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.12237E+01_JPRB, 0.11477E+01_JPRB, 0.11508E+01_JPRB, 0.11139E+01_JPRB, 0.10912E+01_JPRB, 0.10690E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.10472E+01_JPRB, 0.10364E+01_JPRB, 0.10213E+01_JPRB, 0.10039E+01_JPRB, 0.99028E+00_JPRB, 0.97019E+00_JPRB, 0.94895E+00_JPRB ,&
&  0.93239E+00_JPRB, 0.86291E+00_JPRB, 0.84200E+00_JPRB, 0.80767E+00_JPRB, 0.84161E+00_JPRB, 0.84294E+00_JPRB, 0.85598E+00_JPRB ,&
&  0.87451E+00_JPRB, 0.89746E+00_JPRB, 0.89479E+00_JPRB, 0.89608E+00_JPRB, 0.91438E+00_JPRB, 0.96761E+00_JPRB, 0.10303E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.98641E+00_JPRB, 0.96029E+00_JPRB, 0.97799E+00_JPRB, 0.10536E+01_JPRB, 0.99906E+00_JPRB, 0.10293E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10302E+01_JPRB, 0.10256E+01_JPRB, 0.10353E+01_JPRB, 0.10737E+01_JPRB, 0.11269E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.11579E+01_JPRB, 0.10984E+01_JPRB, 0.10754E+01_JPRB, 0.14125E+01_JPRB, 0.22524E+01_JPRB, 0.32425E+01_JPRB, 0.36827E+01_JPRB ,&
&  0.38114E+01_JPRB, 0.37658E+01_JPRB, 0.37378E+01_JPRB, 0.41418E+01_JPRB, 0.39786E+01_JPRB, 0.35870E+01_JPRB, 0.33718E+01_JPRB ,&
&  0.32304E+01_JPRB, 0.31542E+01_JPRB, 0.32134E+01_JPRB, 0.30446E+01_JPRB, 0.28589E+01_JPRB, 0.27505E+01_JPRB, 0.26696E+01_JPRB ,&
&  0.25591E+01_JPRB, 0.25381E+01_JPRB, 0.25157E+01_JPRB, 0.23750E+01_JPRB, 0.23142E+01_JPRB, 0.22809E+01_JPRB, 0.25166E+01_JPRB ,&
&  0.28364E+01_JPRB, 0.22145E+01_JPRB, 0.18626E+01_JPRB, 0.20366E+01_JPRB, 0.15779E+01_JPRB, 0.15189E+01_JPRB, 0.16788E+01_JPRB ,&
&  0.15624E+01_JPRB, 0.15222E+01_JPRB, 0.16017E+01_JPRB, 0.15634E+01_JPRB, 0.16351E+01_JPRB, 0.17994E+01_JPRB, 0.17781E+01_JPRB ,&
&  0.16726E+01_JPRB, 0.16039E+01_JPRB, 0.18993E+01_JPRB, 0.17320E+01_JPRB, 0.15941E+01_JPRB, 0.15118E+01_JPRB, 0.14363E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.13464E+01_JPRB, 0.12975E+01_JPRB, 0.12681E+01_JPRB, 0.12626E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.12363E+01_JPRB, 0.11569E+01_JPRB, 0.11901E+01_JPRB, 0.11695E+01_JPRB, 0.11568E+01_JPRB, 0.11448E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.11015E+01_JPRB, 0.10834E+01_JPRB, 0.10484E+01_JPRB, 0.10224E+01_JPRB, 0.10004E+01_JPRB, 0.98588E+00_JPRB ,&
&  0.96654E+00_JPRB, 0.96746E+00_JPRB, 0.91802E+00_JPRB, 0.90005E+00_JPRB, 0.87376E+00_JPRB, 0.90105E+00_JPRB, 0.92909E+00_JPRB ,&
&  0.97816E+00_JPRB, 0.10326E+01_JPRB, 0.10877E+01_JPRB, 0.11479E+01_JPRB, 0.12008E+01_JPRB, 0.12397E+01_JPRB, 0.13137E+01_JPRB ,&
&  0.13257E+01_JPRB, 0.12581E+01_JPRB, 0.11960E+01_JPRB, 0.12231E+01_JPRB, 0.12483E+01_JPRB, 0.11781E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11593E+01_JPRB, 0.11590E+01_JPRB, 0.11960E+01_JPRB, 0.12035E+01_JPRB, 0.11422E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.10532E+01_JPRB, 0.10880E+01_JPRB, 0.13471E+01_JPRB, 0.19609E+01_JPRB, 0.29892E+01_JPRB, 0.42039E+01_JPRB, 0.45387E+01_JPRB ,&
&  0.45574E+01_JPRB, 0.40995E+01_JPRB, 0.32202E+01_JPRB, 0.35700E+01_JPRB, 0.43796E+01_JPRB, 0.41216E+01_JPRB, 0.37200E+01_JPRB ,&
&  0.36941E+01_JPRB, 0.34529E+01_JPRB, 0.33574E+01_JPRB, 0.30650E+01_JPRB, 0.28467E+01_JPRB, 0.26339E+01_JPRB, 0.24466E+01_JPRB ,&
&  0.22955E+01_JPRB, 0.22425E+01_JPRB, 0.22349E+01_JPRB, 0.22657E+01_JPRB, 0.22677E+01_JPRB, 0.24520E+01_JPRB, 0.27860E+01_JPRB ,&
&  0.27999E+01_JPRB, 0.25889E+01_JPRB, 0.20119E+01_JPRB, 0.19363E+01_JPRB, 0.14788E+01_JPRB, 0.14187E+01_JPRB, 0.15519E+01_JPRB ,&
&  0.15161E+01_JPRB, 0.16859E+01_JPRB, 0.19329E+01_JPRB, 0.19552E+01_JPRB, 0.22328E+01_JPRB, 0.23135E+01_JPRB, 0.22695E+01_JPRB ,&
&  0.21595E+01_JPRB, 0.19543E+01_JPRB, 0.22728E+01_JPRB, 0.20223E+01_JPRB, 0.18113E+01_JPRB, 0.16986E+01_JPRB, 0.16094E+01_JPRB ,&
&  0.15520E+01_JPRB, 0.14768E+01_JPRB, 0.13979E+01_JPRB, 0.13515E+01_JPRB, 0.13114E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.12323E+01_JPRB, 0.11824E+01_JPRB, 0.12190E+01_JPRB, 0.11867E+01_JPRB, 0.11930E+01_JPRB, 0.11616E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.11173E+01_JPRB, 0.10813E+01_JPRB, 0.10468E+01_JPRB, 0.10219E+01_JPRB, 0.10009E+01_JPRB, 0.99416E+00_JPRB ,&
&  0.98307E+00_JPRB, 0.96132E+00_JPRB, 0.88639E+00_JPRB, 0.84805E+00_JPRB, 0.83271E+00_JPRB, 0.87608E+00_JPRB, 0.96101E+00_JPRB ,&
&  0.10738E+01_JPRB, 0.11875E+01_JPRB, 0.13394E+01_JPRB, 0.14366E+01_JPRB, 0.15108E+01_JPRB, 0.15672E+01_JPRB, 0.16337E+01_JPRB ,&
&  0.16850E+01_JPRB, 0.14791E+01_JPRB, 0.14390E+01_JPRB, 0.13476E+01_JPRB, 0.13261E+01_JPRB, 0.13016E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.12194E+01_JPRB, 0.12112E+01_JPRB, 0.12029E+01_JPRB, 0.11586E+01_JPRB, 0.11036E+01_JPRB, 0.10671E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.13091E+01_JPRB, 0.19171E+01_JPRB, 0.28176E+01_JPRB, 0.42251E+01_JPRB, 0.54903E+01_JPRB, 0.54762E+01_JPRB ,&
&  0.42345E+01_JPRB, 0.36763E+01_JPRB, 0.47522E+01_JPRB, 0.41917E+01_JPRB, 0.42657E+01_JPRB, 0.45438E+01_JPRB, 0.42169E+01_JPRB ,&
&  0.36934E+01_JPRB, 0.31187E+01_JPRB, 0.30487E+01_JPRB, 0.28982E+01_JPRB, 0.36715E+01_JPRB, 0.31339E+01_JPRB, 0.27452E+01_JPRB ,&
&  0.25085E+01_JPRB, 0.24684E+01_JPRB, 0.24189E+01_JPRB, 0.24931E+01_JPRB, 0.27664E+01_JPRB, 0.32817E+01_JPRB, 0.34145E+01_JPRB ,&
&  0.30665E+01_JPRB, 0.30521E+01_JPRB, 0.25375E+01_JPRB, 0.20597E+01_JPRB, 0.17378E+01_JPRB, 0.16979E+01_JPRB, 0.17134E+01_JPRB ,&
&  0.17889E+01_JPRB, 0.19134E+01_JPRB, 0.23385E+01_JPRB, 0.28101E+01_JPRB, 0.32284E+01_JPRB, 0.32015E+01_JPRB, 0.27394E+01_JPRB ,&
&  0.26351E+01_JPRB, 0.29578E+01_JPRB, 0.29135E+01_JPRB, 0.23287E+01_JPRB, 0.19990E+01_JPRB, 0.18583E+01_JPRB, 0.17493E+01_JPRB ,&
&  0.16208E+01_JPRB, 0.14992E+01_JPRB, 0.14111E+01_JPRB, 0.13570E+01_JPRB, 0.13200E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.12861E+01_JPRB, 0.12432E+01_JPRB, 0.11923E+01_JPRB, 0.11686E+01_JPRB, 0.11739E+01_JPRB, 0.11296E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.10327E+01_JPRB, 0.10329E+01_JPRB, 0.10370E+01_JPRB, 0.10276E+01_JPRB, 0.10076E+01_JPRB, 0.99993E+00_JPRB ,&
&  0.98629E+00_JPRB, 0.91945E+00_JPRB, 0.85302E+00_JPRB, 0.85810E+00_JPRB, 0.85761E+00_JPRB, 0.86612E+00_JPRB, 0.98154E+00_JPRB ,&
&  0.11723E+01_JPRB, 0.14107E+01_JPRB, 0.16051E+01_JPRB, 0.16990E+01_JPRB, 0.17200E+01_JPRB, 0.17437E+01_JPRB, 0.17729E+01_JPRB ,&
&  0.16626E+01_JPRB, 0.16504E+01_JPRB, 0.16325E+01_JPRB, 0.15114E+01_JPRB, 0.14182E+01_JPRB, 0.13272E+01_JPRB, 0.12777E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.12528E+01_JPRB, 0.12106E+01_JPRB, 0.11521E+01_JPRB, 0.11653E+01_JPRB, 0.11791E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.12295E+01_JPRB, 0.16490E+01_JPRB, 0.24324E+01_JPRB, 0.34761E+01_JPRB, 0.38574E+01_JPRB, 0.50631E+01_JPRB, 0.50378E+01_JPRB ,&
&  0.56099E+01_JPRB, 0.52291E+01_JPRB, 0.48815E+01_JPRB, 0.52971E+01_JPRB, 0.43724E+01_JPRB, 0.43765E+01_JPRB, 0.42340E+01_JPRB ,&
&  0.37437E+01_JPRB, 0.35395E+01_JPRB, 0.38880E+01_JPRB, 0.35868E+01_JPRB, 0.38996E+01_JPRB, 0.43423E+01_JPRB, 0.38584E+01_JPRB ,&
&  0.36390E+01_JPRB, 0.34454E+01_JPRB, 0.32221E+01_JPRB, 0.33889E+01_JPRB, 0.34125E+01_JPRB, 0.35171E+01_JPRB, 0.29976E+01_JPRB ,&
&  0.28275E+01_JPRB, 0.23754E+01_JPRB, 0.25262E+01_JPRB, 0.22498E+01_JPRB, 0.21571E+01_JPRB, 0.20543E+01_JPRB, 0.19960E+01_JPRB ,&
&  0.21698E+01_JPRB, 0.23097E+01_JPRB, 0.29975E+01_JPRB, 0.37110E+01_JPRB, 0.43969E+01_JPRB, 0.38953E+01_JPRB, 0.34391E+01_JPRB ,&
&  0.36649E+01_JPRB, 0.37650E+01_JPRB, 0.31305E+01_JPRB, 0.24999E+01_JPRB, 0.21348E+01_JPRB, 0.19826E+01_JPRB, 0.18713E+01_JPRB ,&
&  0.17433E+01_JPRB, 0.16167E+01_JPRB, 0.15212E+01_JPRB, 0.14496E+01_JPRB, 0.13871E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.12019E+01_JPRB, 0.11746E+01_JPRB, 0.11158E+01_JPRB, 0.11110E+01_JPRB, 0.11316E+01_JPRB, 0.10960E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10303E+01_JPRB, 0.10077E+01_JPRB, 0.10174E+01_JPRB, 0.10315E+01_JPRB, 0.10212E+01_JPRB, 0.10132E+01_JPRB, 0.99610E+00_JPRB ,&
&  0.97790E+00_JPRB, 0.93131E+00_JPRB, 0.87245E+00_JPRB, 0.86854E+00_JPRB, 0.90427E+00_JPRB, 0.93116E+00_JPRB, 0.10279E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.17108E+01_JPRB, 0.18250E+01_JPRB, 0.18305E+01_JPRB, 0.17793E+01_JPRB, 0.17413E+01_JPRB, 0.16831E+01_JPRB ,&
&  0.16452E+01_JPRB, 0.15614E+01_JPRB, 0.14907E+01_JPRB, 0.14318E+01_JPRB, 0.13745E+01_JPRB, 0.13179E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.12299E+01_JPRB, 0.11476E+01_JPRB, 0.10723E+01_JPRB, 0.10234E+01_JPRB, 0.10541E+01_JPRB, 0.10425E+01_JPRB, 0.11137E+01_JPRB ,&
&  0.13765E+01_JPRB, 0.18537E+01_JPRB, 0.24754E+01_JPRB, 0.32792E+01_JPRB, 0.39191E+01_JPRB, 0.51607E+01_JPRB, 0.57978E+01_JPRB ,&
&  0.55013E+01_JPRB, 0.48864E+01_JPRB, 0.53625E+01_JPRB, 0.59164E+01_JPRB, 0.56198E+01_JPRB, 0.48100E+01_JPRB, 0.44918E+01_JPRB ,&
&  0.34035E+01_JPRB, 0.27832E+01_JPRB, 0.26686E+01_JPRB, 0.31322E+01_JPRB, 0.48111E+01_JPRB, 0.62359E+01_JPRB, 0.59265E+01_JPRB ,&
&  0.57728E+01_JPRB, 0.53937E+01_JPRB, 0.49445E+01_JPRB, 0.42786E+01_JPRB, 0.38892E+01_JPRB, 0.35589E+01_JPRB, 0.43834E+01_JPRB ,&
&  0.42285E+01_JPRB, 0.41303E+01_JPRB, 0.34505E+01_JPRB, 0.27459E+01_JPRB, 0.22442E+01_JPRB, 0.24025E+01_JPRB, 0.24244E+01_JPRB ,&
&  0.26161E+01_JPRB, 0.29156E+01_JPRB, 0.48447E+01_JPRB, 0.56136E+01_JPRB, 0.54963E+01_JPRB, 0.50863E+01_JPRB, 0.49349E+01_JPRB ,&
&  0.45069E+01_JPRB, 0.43465E+01_JPRB, 0.35098E+01_JPRB, 0.26274E+01_JPRB, 0.21182E+01_JPRB, 0.18277E+01_JPRB, 0.16662E+01_JPRB ,&
&  0.16141E+01_JPRB, 0.15618E+01_JPRB, 0.14787E+01_JPRB, 0.13750E+01_JPRB, 0.12938E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.11219E+01_JPRB, 0.10944E+01_JPRB, 0.10311E+01_JPRB, 0.10241E+01_JPRB, 0.10488E+01_JPRB, 0.10662E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10242E+01_JPRB, 0.10357E+01_JPRB, 0.10480E+01_JPRB, 0.10383E+01_JPRB, 0.10184E+01_JPRB, 0.10177E+01_JPRB, 0.10061E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.10310E+01_JPRB, 0.95905E+00_JPRB, 0.97823E+00_JPRB, 0.10399E+01_JPRB, 0.11206E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.17446E+01_JPRB, 0.18997E+01_JPRB, 0.17972E+01_JPRB, 0.17735E+01_JPRB, 0.17755E+01_JPRB, 0.15759E+01_JPRB, 0.15298E+01_JPRB ,&
&  0.15481E+01_JPRB, 0.14667E+01_JPRB, 0.14041E+01_JPRB, 0.13529E+01_JPRB, 0.13285E+01_JPRB, 0.13105E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.11636E+01_JPRB, 0.10708E+01_JPRB, 0.10320E+01_JPRB, 0.10272E+01_JPRB, 0.97270E+00_JPRB, 0.98445E+00_JPRB, 0.10824E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.19029E+01_JPRB, 0.22734E+01_JPRB, 0.28177E+01_JPRB, 0.35503E+01_JPRB, 0.49201E+01_JPRB, 0.68322E+01_JPRB ,&
&  0.62216E+01_JPRB, 0.60474E+01_JPRB, 0.58648E+01_JPRB, 0.64762E+01_JPRB, 0.55826E+01_JPRB, 0.48117E+01_JPRB, 0.43631E+01_JPRB ,&
&  0.32464E+01_JPRB, 0.27874E+01_JPRB, 0.32393E+01_JPRB, 0.42969E+01_JPRB, 0.49078E+01_JPRB, 0.57528E+01_JPRB, 0.71897E+01_JPRB ,&
&  0.64627E+01_JPRB, 0.62077E+01_JPRB, 0.59255E+01_JPRB, 0.55631E+01_JPRB, 0.33319E+01_JPRB, 0.27157E+01_JPRB, 0.39195E+01_JPRB ,&
&  0.40516E+01_JPRB, 0.26178E+01_JPRB, 0.17115E+01_JPRB, 0.21163E+01_JPRB, 0.15393E+01_JPRB, 0.17046E+01_JPRB, 0.23718E+01_JPRB ,&
&  0.30517E+01_JPRB, 0.40668E+01_JPRB, 0.69312E+01_JPRB, 0.64841E+01_JPRB, 0.61466E+01_JPRB, 0.71906E+01_JPRB, 0.60584E+01_JPRB ,&
&  0.57563E+01_JPRB, 0.41383E+01_JPRB, 0.32763E+01_JPRB, 0.23223E+01_JPRB, 0.16958E+01_JPRB, 0.14083E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.12906E+01_JPRB, 0.12847E+01_JPRB, 0.12362E+01_JPRB, 0.11749E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.11525E+01_JPRB, 0.10663E+01_JPRB, 0.95922E+00_JPRB, 0.95137E+00_JPRB, 0.97429E+00_JPRB, 0.10090E+01_JPRB, 0.10017E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10509E+01_JPRB, 0.11015E+01_JPRB, 0.10963E+01_JPRB, 0.10566E+01_JPRB, 0.10522E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.10905E+01_JPRB, 0.11653E+01_JPRB, 0.12648E+01_JPRB, 0.13861E+01_JPRB, 0.14533E+01_JPRB, 0.17617E+01_JPRB ,&
&  0.20321E+01_JPRB, 0.18806E+01_JPRB, 0.17427E+01_JPRB, 0.17215E+01_JPRB, 0.16119E+01_JPRB, 0.13967E+01_JPRB, 0.14388E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.14096E+01_JPRB, 0.13482E+01_JPRB, 0.12975E+01_JPRB, 0.12863E+01_JPRB, 0.12787E+01_JPRB, 0.12304E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.11013E+01_JPRB, 0.10680E+01_JPRB, 0.97534E+00_JPRB, 0.93731E+00_JPRB, 0.97291E+00_JPRB, 0.11066E+01_JPRB ,&
&  0.14274E+01_JPRB, 0.18102E+01_JPRB, 0.20601E+01_JPRB, 0.24606E+01_JPRB, 0.39499E+01_JPRB, 0.51315E+01_JPRB, 0.52683E+01_JPRB ,&
&  0.50826E+01_JPRB, 0.51141E+01_JPRB, 0.64379E+01_JPRB, 0.62982E+01_JPRB, 0.57868E+01_JPRB, 0.55285E+01_JPRB, 0.46763E+01_JPRB ,&
&  0.40894E+01_JPRB, 0.37636E+01_JPRB, 0.41823E+01_JPRB, 0.50152E+01_JPRB, 0.60045E+01_JPRB, 0.60981E+01_JPRB, 0.74888E+01_JPRB ,&
&  0.75772E+01_JPRB, 0.71127E+01_JPRB, 0.55438E+01_JPRB, 0.41713E+01_JPRB, 0.37610E+01_JPRB, 0.28023E+01_JPRB, 0.19317E+01_JPRB ,&
&  0.16881E+01_JPRB, 0.16070E+01_JPRB, 0.14286E+01_JPRB, 0.14405E+01_JPRB, 0.12873E+01_JPRB, 0.14765E+01_JPRB, 0.21243E+01_JPRB ,&
&  0.38941E+01_JPRB, 0.58469E+01_JPRB, 0.69079E+01_JPRB, 0.60052E+01_JPRB, 0.56341E+01_JPRB, 0.70811E+01_JPRB, 0.74491E+01_JPRB ,&
&  0.43746E+01_JPRB, 0.32691E+01_JPRB, 0.22412E+01_JPRB, 0.15933E+01_JPRB, 0.13125E+01_JPRB, 0.11656E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.11139E+01_JPRB, 0.11292E+01_JPRB, 0.11528E+01_JPRB, 0.11426E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.11303E+01_JPRB, 0.10055E+01_JPRB, 0.89756E+00_JPRB, 0.87347E+00_JPRB, 0.91512E+00_JPRB, 0.93845E+00_JPRB, 0.96358E+00_JPRB ,&
&  0.10144E+01_JPRB, 0.10929E+01_JPRB, 0.11457E+01_JPRB, 0.11311E+01_JPRB, 0.11021E+01_JPRB, 0.10824E+01_JPRB, 0.10680E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.12429E+01_JPRB, 0.14860E+01_JPRB, 0.17317E+01_JPRB, 0.19778E+01_JPRB, 0.19291E+01_JPRB, 0.22022E+01_JPRB ,&
&  0.20354E+01_JPRB, 0.17872E+01_JPRB, 0.17770E+01_JPRB, 0.17062E+01_JPRB, 0.15838E+01_JPRB, 0.14364E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.13868E+01_JPRB, 0.13842E+01_JPRB, 0.13128E+01_JPRB, 0.13123E+01_JPRB, 0.13035E+01_JPRB, 0.12699E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.11420E+01_JPRB, 0.10176E+01_JPRB, 0.10010E+01_JPRB, 0.10096E+01_JPRB, 0.10858E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.15137E+01_JPRB, 0.18274E+01_JPRB, 0.21344E+01_JPRB, 0.29559E+01_JPRB, 0.37681E+01_JPRB, 0.36841E+01_JPRB, 0.46429E+01_JPRB ,&
&  0.54294E+01_JPRB, 0.59645E+01_JPRB, 0.59287E+01_JPRB, 0.66301E+01_JPRB, 0.63150E+01_JPRB, 0.56810E+01_JPRB, 0.51370E+01_JPRB ,&
&  0.44724E+01_JPRB, 0.48844E+01_JPRB, 0.62884E+01_JPRB, 0.69224E+01_JPRB, 0.85687E+01_JPRB, 0.82290E+01_JPRB, 0.87148E+01_JPRB ,&
&  0.89834E+01_JPRB, 0.81606E+01_JPRB, 0.65170E+01_JPRB, 0.51451E+01_JPRB, 0.55470E+01_JPRB, 0.50445E+01_JPRB, 0.27141E+01_JPRB ,&
&  0.22331E+01_JPRB, 0.19350E+01_JPRB, 0.15646E+01_JPRB, 0.13501E+01_JPRB, 0.12843E+01_JPRB, 0.15457E+01_JPRB, 0.26578E+01_JPRB ,&
&  0.34720E+01_JPRB, 0.61494E+01_JPRB, 0.71540E+01_JPRB, 0.62590E+01_JPRB, 0.54497E+01_JPRB, 0.65507E+01_JPRB, 0.50432E+01_JPRB ,&
&  0.39475E+01_JPRB, 0.27755E+01_JPRB, 0.16156E+01_JPRB, 0.12555E+01_JPRB, 0.11776E+01_JPRB, 0.11223E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10028E+01_JPRB, 0.10012E+01_JPRB, 0.10352E+01_JPRB, 0.10686E+01_JPRB, 0.10778E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.98816E+00_JPRB, 0.89312E+00_JPRB, 0.83949E+00_JPRB, 0.80655E+00_JPRB, 0.86370E+00_JPRB, 0.94128E+00_JPRB, 0.96616E+00_JPRB ,&
&  0.10513E+01_JPRB, 0.11482E+01_JPRB, 0.11987E+01_JPRB, 0.11909E+01_JPRB, 0.11419E+01_JPRB, 0.11350E+01_JPRB, 0.11889E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.15769E+01_JPRB, 0.19318E+01_JPRB, 0.23121E+01_JPRB, 0.24682E+01_JPRB, 0.25166E+01_JPRB, 0.23815E+01_JPRB ,&
&  0.19375E+01_JPRB, 0.17552E+01_JPRB, 0.18010E+01_JPRB, 0.17859E+01_JPRB, 0.16188E+01_JPRB, 0.15479E+01_JPRB, 0.14433E+01_JPRB ,&
&  0.13865E+01_JPRB, 0.13929E+01_JPRB, 0.13723E+01_JPRB, 0.13845E+01_JPRB, 0.13498E+01_JPRB, 0.12969E+01_JPRB, 0.12524E+01_JPRB ,&
&  0.12099E+01_JPRB, 0.10906E+01_JPRB, 0.10309E+01_JPRB, 0.10407E+01_JPRB, 0.11644E+01_JPRB, 0.13623E+01_JPRB, 0.15832E+01_JPRB ,&
&  0.18563E+01_JPRB, 0.21830E+01_JPRB, 0.26589E+01_JPRB, 0.35191E+01_JPRB, 0.32191E+01_JPRB, 0.40965E+01_JPRB, 0.47293E+01_JPRB ,&
&  0.50248E+01_JPRB, 0.52619E+01_JPRB, 0.51300E+01_JPRB, 0.57685E+01_JPRB, 0.61321E+01_JPRB, 0.56775E+01_JPRB, 0.50732E+01_JPRB ,&
&  0.53258E+01_JPRB, 0.58025E+01_JPRB, 0.84005E+01_JPRB, 0.96925E+01_JPRB, 0.10739E+02_JPRB, 0.13864E+02_JPRB, 0.10241E+02_JPRB ,&
&  0.86146E+01_JPRB, 0.88915E+01_JPRB, 0.74748E+01_JPRB, 0.59895E+01_JPRB, 0.59944E+01_JPRB, 0.67729E+01_JPRB, 0.69439E+01_JPRB ,&
&  0.31702E+01_JPRB, 0.19598E+01_JPRB, 0.18765E+01_JPRB, 0.16831E+01_JPRB, 0.14148E+01_JPRB, 0.16336E+01_JPRB, 0.40910E+01_JPRB ,&
&  0.34576E+01_JPRB, 0.57126E+01_JPRB, 0.65124E+01_JPRB, 0.56405E+01_JPRB, 0.52371E+01_JPRB, 0.51618E+01_JPRB, 0.44860E+01_JPRB ,&
&  0.29464E+01_JPRB, 0.20105E+01_JPRB, 0.13918E+01_JPRB, 0.11584E+01_JPRB, 0.11148E+01_JPRB, 0.10865E+01_JPRB, 0.10336E+01_JPRB ,&
&  0.98303E+00_JPRB, 0.95292E+00_JPRB, 0.97501E+00_JPRB, 0.96025E+00_JPRB, 0.10039E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.90143E+00_JPRB, 0.86294E+00_JPRB, 0.88422E+00_JPRB, 0.90140E+00_JPRB, 0.94165E+00_JPRB, 0.97914E+00_JPRB, 0.10386E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.12529E+01_JPRB, 0.13298E+01_JPRB, 0.13258E+01_JPRB, 0.12910E+01_JPRB, 0.13627E+01_JPRB, 0.15270E+01_JPRB ,&
&  0.17542E+01_JPRB, 0.20444E+01_JPRB, 0.23763E+01_JPRB, 0.26004E+01_JPRB, 0.27122E+01_JPRB, 0.24819E+01_JPRB, 0.22064E+01_JPRB ,&
&  0.18628E+01_JPRB, 0.17188E+01_JPRB, 0.16767E+01_JPRB, 0.17220E+01_JPRB, 0.17020E+01_JPRB, 0.15747E+01_JPRB, 0.14447E+01_JPRB ,&
&  0.14070E+01_JPRB, 0.14067E+01_JPRB, 0.13910E+01_JPRB, 0.14110E+01_JPRB, 0.14287E+01_JPRB, 0.14265E+01_JPRB, 0.13754E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.11904E+01_JPRB, 0.11666E+01_JPRB, 0.12742E+01_JPRB, 0.15511E+01_JPRB, 0.18663E+01_JPRB, 0.21708E+01_JPRB ,&
&  0.24903E+01_JPRB, 0.28198E+01_JPRB, 0.32452E+01_JPRB, 0.35297E+01_JPRB, 0.39607E+01_JPRB, 0.43254E+01_JPRB, 0.45907E+01_JPRB ,&
&  0.46176E+01_JPRB, 0.49219E+01_JPRB, 0.52124E+01_JPRB, 0.54525E+01_JPRB, 0.56767E+01_JPRB, 0.55177E+01_JPRB, 0.53082E+01_JPRB ,&
&  0.61687E+01_JPRB, 0.78647E+01_JPRB, 0.10001E+02_JPRB, 0.10939E+02_JPRB, 0.11491E+02_JPRB, 0.13989E+02_JPRB, 0.13790E+02_JPRB ,&
&  0.10746E+02_JPRB, 0.78568E+01_JPRB, 0.67397E+01_JPRB, 0.66707E+01_JPRB, 0.69181E+01_JPRB, 0.77494E+01_JPRB, 0.10850E+02_JPRB ,&
&  0.10565E+02_JPRB, 0.48109E+01_JPRB, 0.30503E+01_JPRB, 0.33064E+01_JPRB, 0.18398E+01_JPRB, 0.15522E+01_JPRB, 0.25283E+01_JPRB ,&
&  0.34515E+01_JPRB, 0.51082E+01_JPRB, 0.59836E+01_JPRB, 0.48039E+01_JPRB, 0.45190E+01_JPRB, 0.36733E+01_JPRB, 0.28944E+01_JPRB ,&
&  0.20763E+01_JPRB, 0.15127E+01_JPRB, 0.11743E+01_JPRB, 0.10776E+01_JPRB, 0.10218E+01_JPRB, 0.10297E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.98968E+00_JPRB, 0.91175E+00_JPRB, 0.91619E+00_JPRB, 0.92249E+00_JPRB, 0.90578E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.93455E+00_JPRB, 0.94589E+00_JPRB, 0.10032E+01_JPRB, 0.10622E+01_JPRB, 0.11094E+01_JPRB, 0.11543E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.14373E+01_JPRB, 0.15204E+01_JPRB, 0.15738E+01_JPRB, 0.16460E+01_JPRB, 0.17888E+01_JPRB, 0.19724E+01_JPRB ,&
&  0.21866E+01_JPRB, 0.24120E+01_JPRB, 0.26341E+01_JPRB, 0.28727E+01_JPRB, 0.31958E+01_JPRB, 0.33652E+01_JPRB, 0.20435E+01_JPRB ,&
&  0.17528E+01_JPRB, 0.18728E+01_JPRB, 0.16680E+01_JPRB, 0.17738E+01_JPRB, 0.18277E+01_JPRB, 0.18212E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.14109E+01_JPRB, 0.13928E+01_JPRB, 0.13941E+01_JPRB, 0.14491E+01_JPRB, 0.15325E+01_JPRB, 0.16465E+01_JPRB, 0.16528E+01_JPRB ,&
&  0.16418E+01_JPRB, 0.16835E+01_JPRB, 0.17616E+01_JPRB, 0.20310E+01_JPRB, 0.24152E+01_JPRB, 0.27278E+01_JPRB, 0.29509E+01_JPRB ,&
&  0.31373E+01_JPRB, 0.33583E+01_JPRB, 0.36732E+01_JPRB, 0.38936E+01_JPRB, 0.43173E+01_JPRB, 0.45199E+01_JPRB, 0.46663E+01_JPRB ,&
&  0.45166E+01_JPRB, 0.43211E+01_JPRB, 0.47777E+01_JPRB, 0.51429E+01_JPRB, 0.52726E+01_JPRB, 0.52875E+01_JPRB, 0.57228E+01_JPRB ,&
&  0.77033E+01_JPRB, 0.10091E+02_JPRB, 0.12695E+02_JPRB, 0.11391E+02_JPRB, 0.10912E+02_JPRB, 0.12134E+02_JPRB, 0.13163E+02_JPRB ,&
&  0.10884E+02_JPRB, 0.84779E+01_JPRB, 0.78383E+01_JPRB, 0.74131E+01_JPRB, 0.76357E+01_JPRB, 0.79136E+01_JPRB, 0.95367E+01_JPRB ,&
&  0.10579E+02_JPRB, 0.81007E+01_JPRB, 0.46187E+01_JPRB, 0.24859E+01_JPRB, 0.17289E+01_JPRB, 0.17739E+01_JPRB, 0.22598E+01_JPRB ,&
&  0.36125E+01_JPRB, 0.45418E+01_JPRB, 0.53656E+01_JPRB, 0.55275E+01_JPRB, 0.32044E+01_JPRB, 0.22584E+01_JPRB, 0.17735E+01_JPRB ,&
&  0.12471E+01_JPRB, 0.10231E+01_JPRB, 0.10044E+01_JPRB, 0.98106E+00_JPRB, 0.93957E+00_JPRB, 0.98201E+00_JPRB, 0.10603E+01_JPRB ,&
&  0.10048E+01_JPRB, 0.89469E+00_JPRB, 0.89928E+00_JPRB, 0.89961E+00_JPRB, 0.90660E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.99369E+00_JPRB, 0.10354E+01_JPRB, 0.11047E+01_JPRB, 0.12120E+01_JPRB, 0.13347E+01_JPRB, 0.14471E+01_JPRB, 0.14516E+01_JPRB ,&
&  0.14717E+01_JPRB, 0.15794E+01_JPRB, 0.16910E+01_JPRB, 0.18041E+01_JPRB, 0.19235E+01_JPRB, 0.20635E+01_JPRB, 0.22109E+01_JPRB ,&
&  0.23699E+01_JPRB, 0.25280E+01_JPRB, 0.26910E+01_JPRB, 0.29100E+01_JPRB, 0.31879E+01_JPRB, 0.33469E+01_JPRB, 0.24922E+01_JPRB ,&
&  0.17512E+01_JPRB, 0.25059E+01_JPRB, 0.18004E+01_JPRB, 0.19494E+01_JPRB, 0.20614E+01_JPRB, 0.22474E+01_JPRB, 0.18379E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.14454E+01_JPRB, 0.14754E+01_JPRB, 0.14920E+01_JPRB, 0.15448E+01_JPRB, 0.17729E+01_JPRB, 0.20049E+01_JPRB ,&
&  0.21111E+01_JPRB, 0.22616E+01_JPRB, 0.24886E+01_JPRB, 0.28155E+01_JPRB, 0.30767E+01_JPRB, 0.32926E+01_JPRB, 0.34639E+01_JPRB ,&
&  0.36276E+01_JPRB, 0.38580E+01_JPRB, 0.40479E+01_JPRB, 0.43224E+01_JPRB, 0.44518E+01_JPRB, 0.46761E+01_JPRB, 0.48100E+01_JPRB ,&
&  0.46658E+01_JPRB, 0.44737E+01_JPRB, 0.46925E+01_JPRB, 0.47342E+01_JPRB, 0.50593E+01_JPRB, 0.61548E+01_JPRB, 0.70306E+01_JPRB ,&
&  0.92312E+01_JPRB, 0.10459E+02_JPRB, 0.12692E+02_JPRB, 0.10201E+02_JPRB, 0.92334E+01_JPRB, 0.96761E+01_JPRB, 0.90038E+01_JPRB ,&
&  0.76465E+01_JPRB, 0.75426E+01_JPRB, 0.73824E+01_JPRB, 0.74940E+01_JPRB, 0.77065E+01_JPRB, 0.79029E+01_JPRB, 0.83166E+01_JPRB ,&
&  0.68724E+01_JPRB, 0.36214E+01_JPRB, 0.23685E+01_JPRB, 0.17967E+01_JPRB, 0.14980E+01_JPRB, 0.16559E+01_JPRB, 0.25763E+01_JPRB ,&
&  0.40717E+01_JPRB, 0.46141E+01_JPRB, 0.51603E+01_JPRB, 0.37616E+01_JPRB, 0.18975E+01_JPRB, 0.13419E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.81014E+00_JPRB, 0.77479E+00_JPRB, 0.84530E+00_JPRB, 0.86995E+00_JPRB, 0.84887E+00_JPRB, 0.88463E+00_JPRB, 0.92368E+00_JPRB ,&
&  0.90557E+00_JPRB, 0.89363E+00_JPRB, 0.94232E+00_JPRB, 0.93644E+00_JPRB, 0.95701E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.10405E+01_JPRB, 0.10999E+01_JPRB, 0.12200E+01_JPRB, 0.13847E+01_JPRB, 0.16160E+01_JPRB, 0.18593E+01_JPRB, 0.19627E+01_JPRB ,&
&  0.17804E+01_JPRB, 0.16010E+01_JPRB, 0.17742E+01_JPRB, 0.19363E+01_JPRB, 0.20651E+01_JPRB, 0.22165E+01_JPRB, 0.23673E+01_JPRB ,&
&  0.25529E+01_JPRB, 0.27477E+01_JPRB, 0.29648E+01_JPRB, 0.32355E+01_JPRB, 0.35454E+01_JPRB, 0.38403E+01_JPRB, 0.37433E+01_JPRB ,&
&  0.27268E+01_JPRB, 0.20784E+01_JPRB, 0.21812E+01_JPRB, 0.20134E+01_JPRB, 0.19551E+01_JPRB, 0.21399E+01_JPRB, 0.21508E+01_JPRB ,&
&  0.18199E+01_JPRB, 0.16000E+01_JPRB, 0.16399E+01_JPRB, 0.16395E+01_JPRB, 0.13822E+01_JPRB, 0.16298E+01_JPRB, 0.20131E+01_JPRB ,&
&  0.22486E+01_JPRB, 0.24609E+01_JPRB, 0.27402E+01_JPRB, 0.30527E+01_JPRB, 0.32863E+01_JPRB, 0.35675E+01_JPRB, 0.38203E+01_JPRB ,&
&  0.40188E+01_JPRB, 0.42470E+01_JPRB, 0.43652E+01_JPRB, 0.44008E+01_JPRB, 0.45812E+01_JPRB, 0.47989E+01_JPRB, 0.48720E+01_JPRB ,&
&  0.48841E+01_JPRB, 0.51006E+01_JPRB, 0.53642E+01_JPRB, 0.56784E+01_JPRB, 0.59447E+01_JPRB, 0.71971E+01_JPRB, 0.81710E+01_JPRB ,&
&  0.92471E+01_JPRB, 0.95167E+01_JPRB, 0.10471E+02_JPRB, 0.82190E+01_JPRB, 0.74621E+01_JPRB, 0.70644E+01_JPRB, 0.66783E+01_JPRB ,&
&  0.62997E+01_JPRB, 0.65617E+01_JPRB, 0.65545E+01_JPRB, 0.69246E+01_JPRB, 0.74071E+01_JPRB, 0.71272E+01_JPRB, 0.70909E+01_JPRB ,&
&  0.42371E+01_JPRB, 0.25490E+01_JPRB, 0.16350E+01_JPRB, 0.15826E+01_JPRB, 0.15913E+01_JPRB, 0.18602E+01_JPRB, 0.24302E+01_JPRB ,&
&  0.33170E+01_JPRB, 0.29044E+01_JPRB, 0.27307E+01_JPRB, 0.24205E+01_JPRB, 0.13185E+01_JPRB, 0.91820E+00_JPRB, 0.75851E+00_JPRB ,&
&  0.69314E+00_JPRB, 0.71041E+00_JPRB, 0.75870E+00_JPRB, 0.76822E+00_JPRB, 0.74860E+00_JPRB, 0.78131E+00_JPRB, 0.83913E+00_JPRB ,&
&  0.86503E+00_JPRB, 0.89142E+00_JPRB, 0.94756E+00_JPRB, 0.95911E+00_JPRB, 0.10085E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.98906E+00_JPRB, 0.10856E+01_JPRB, 0.11654E+01_JPRB, 0.12523E+01_JPRB, 0.13253E+01_JPRB, 0.14369E+01_JPRB, 0.14727E+01_JPRB ,&
&  0.14352E+01_JPRB, 0.14163E+01_JPRB, 0.15403E+01_JPRB, 0.16913E+01_JPRB, 0.18240E+01_JPRB, 0.19064E+01_JPRB, 0.19980E+01_JPRB ,&
&  0.20999E+01_JPRB, 0.21994E+01_JPRB, 0.23108E+01_JPRB, 0.24175E+01_JPRB, 0.25817E+01_JPRB, 0.27941E+01_JPRB, 0.29703E+01_JPRB ,&
&  0.26083E+01_JPRB, 0.17405E+01_JPRB, 0.19936E+01_JPRB, 0.20104E+01_JPRB, 0.19175E+01_JPRB, 0.20110E+01_JPRB, 0.21420E+01_JPRB ,&
&  0.18034E+01_JPRB, 0.16576E+01_JPRB, 0.16958E+01_JPRB, 0.17404E+01_JPRB, 0.13057E+01_JPRB, 0.14582E+01_JPRB, 0.17883E+01_JPRB ,&
&  0.20924E+01_JPRB, 0.23926E+01_JPRB, 0.26941E+01_JPRB, 0.29589E+01_JPRB, 0.32309E+01_JPRB, 0.35692E+01_JPRB, 0.38792E+01_JPRB ,&
&  0.41325E+01_JPRB, 0.43260E+01_JPRB, 0.44813E+01_JPRB, 0.46372E+01_JPRB, 0.47987E+01_JPRB, 0.50441E+01_JPRB, 0.51263E+01_JPRB ,&
&  0.52987E+01_JPRB, 0.54245E+01_JPRB, 0.60310E+01_JPRB, 0.67752E+01_JPRB, 0.73601E+01_JPRB, 0.73128E+01_JPRB, 0.78468E+01_JPRB ,&
&  0.79799E+01_JPRB, 0.77127E+01_JPRB, 0.78217E+01_JPRB, 0.77789E+01_JPRB, 0.53164E+01_JPRB, 0.49865E+01_JPRB, 0.50234E+01_JPRB ,&
&  0.56390E+01_JPRB, 0.55685E+01_JPRB, 0.56472E+01_JPRB, 0.58592E+01_JPRB, 0.58927E+01_JPRB, 0.57156E+01_JPRB, 0.50783E+01_JPRB ,&
&  0.33960E+01_JPRB, 0.20250E+01_JPRB, 0.15164E+01_JPRB, 0.15937E+01_JPRB, 0.18573E+01_JPRB, 0.25700E+01_JPRB, 0.21477E+01_JPRB ,&
&  0.24430E+01_JPRB, 0.18552E+01_JPRB, 0.17892E+01_JPRB, 0.20865E+01_JPRB, 0.13111E+01_JPRB, 0.89857E+00_JPRB, 0.73304E+00_JPRB ,&
&  0.67768E+00_JPRB, 0.69409E+00_JPRB, 0.69789E+00_JPRB, 0.68885E+00_JPRB, 0.68269E+00_JPRB, 0.71952E+00_JPRB, 0.76704E+00_JPRB ,&
&  0.79382E+00_JPRB, 0.82729E+00_JPRB, 0.86764E+00_JPRB, 0.91098E+00_JPRB, 0.96786E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.80312E+00_JPRB, 0.82736E+00_JPRB, 0.87272E+00_JPRB, 0.94422E+00_JPRB, 0.10051E+01_JPRB, 0.10623E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.12241E+01_JPRB, 0.13004E+01_JPRB, 0.13599E+01_JPRB, 0.14037E+01_JPRB, 0.15026E+01_JPRB, 0.15708E+01_JPRB ,&
&  0.15885E+01_JPRB, 0.15729E+01_JPRB, 0.16111E+01_JPRB, 0.16899E+01_JPRB, 0.18650E+01_JPRB, 0.21575E+01_JPRB, 0.24778E+01_JPRB ,&
&  0.27934E+01_JPRB, 0.26954E+01_JPRB, 0.24745E+01_JPRB, 0.16113E+01_JPRB, 0.12619E+01_JPRB, 0.14782E+01_JPRB, 0.15403E+01_JPRB ,&
&  0.16265E+01_JPRB, 0.16643E+01_JPRB, 0.15001E+01_JPRB, 0.14061E+01_JPRB, 0.13373E+01_JPRB, 0.13681E+01_JPRB, 0.15414E+01_JPRB ,&
&  0.17699E+01_JPRB, 0.20396E+01_JPRB, 0.23497E+01_JPRB, 0.26021E+01_JPRB, 0.28728E+01_JPRB, 0.32178E+01_JPRB, 0.35820E+01_JPRB ,&
&  0.38872E+01_JPRB, 0.41568E+01_JPRB, 0.43918E+01_JPRB, 0.47560E+01_JPRB, 0.50054E+01_JPRB, 0.53454E+01_JPRB, 0.55922E+01_JPRB ,&
&  0.58748E+01_JPRB, 0.60118E+01_JPRB, 0.65274E+01_JPRB, 0.71016E+01_JPRB, 0.72107E+01_JPRB, 0.65718E+01_JPRB, 0.64028E+01_JPRB ,&
&  0.63949E+01_JPRB, 0.59908E+01_JPRB, 0.58645E+01_JPRB, 0.61367E+01_JPRB, 0.41048E+01_JPRB, 0.42018E+01_JPRB, 0.47165E+01_JPRB ,&
&  0.46395E+01_JPRB, 0.46709E+01_JPRB, 0.44111E+01_JPRB, 0.38006E+01_JPRB, 0.35763E+01_JPRB, 0.36069E+01_JPRB, 0.34318E+01_JPRB ,&
&  0.27471E+01_JPRB, 0.18711E+01_JPRB, 0.14392E+01_JPRB, 0.12665E+01_JPRB, 0.16224E+01_JPRB, 0.26422E+01_JPRB, 0.19521E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.15928E+01_JPRB, 0.14930E+01_JPRB, 0.22304E+01_JPRB, 0.17524E+01_JPRB, 0.10957E+01_JPRB, 0.80470E+00_JPRB ,&
&  0.67690E+00_JPRB, 0.65904E+00_JPRB, 0.66597E+00_JPRB, 0.65401E+00_JPRB, 0.66444E+00_JPRB, 0.69837E+00_JPRB, 0.70552E+00_JPRB ,&
&  0.75320E+00_JPRB, 0.76798E+00_JPRB, 0.79376E+00_JPRB, 0.81501E+00_JPRB, 0.79548E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.59560E+00_JPRB, 0.59290E+00_JPRB, 0.61092E+00_JPRB, 0.67522E+00_JPRB, 0.71788E+00_JPRB, 0.74972E+00_JPRB, 0.80427E+00_JPRB ,&
&  0.87334E+00_JPRB, 0.97807E+00_JPRB, 0.10593E+01_JPRB, 0.10876E+01_JPRB, 0.10518E+01_JPRB, 0.10178E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.10081E+01_JPRB, 0.92695E+00_JPRB, 0.86624E+00_JPRB, 0.86903E+00_JPRB, 0.92925E+00_JPRB, 0.10595E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.14917E+01_JPRB, 0.17284E+01_JPRB, 0.19644E+01_JPRB, 0.25622E+01_JPRB, 0.26259E+01_JPRB, 0.14758E+01_JPRB, 0.16267E+01_JPRB ,&
&  0.21894E+01_JPRB, 0.22505E+01_JPRB, 0.16344E+01_JPRB, 0.14435E+01_JPRB, 0.13454E+01_JPRB, 0.12917E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.14958E+01_JPRB, 0.16620E+01_JPRB, 0.18895E+01_JPRB, 0.21790E+01_JPRB, 0.24847E+01_JPRB, 0.27600E+01_JPRB, 0.30927E+01_JPRB ,&
&  0.33831E+01_JPRB, 0.35640E+01_JPRB, 0.38021E+01_JPRB, 0.41497E+01_JPRB, 0.46106E+01_JPRB, 0.51742E+01_JPRB, 0.56879E+01_JPRB ,&
&  0.61047E+01_JPRB, 0.63217E+01_JPRB, 0.68054E+01_JPRB, 0.71010E+01_JPRB, 0.65285E+01_JPRB, 0.59165E+01_JPRB, 0.53875E+01_JPRB ,&
&  0.49031E+01_JPRB, 0.42127E+01_JPRB, 0.37085E+01_JPRB, 0.45107E+01_JPRB, 0.46402E+01_JPRB, 0.40348E+01_JPRB, 0.38740E+01_JPRB ,&
&  0.38830E+01_JPRB, 0.37981E+01_JPRB, 0.33726E+01_JPRB, 0.26316E+01_JPRB, 0.21098E+01_JPRB, 0.23445E+01_JPRB, 0.24373E+01_JPRB ,&
&  0.21046E+01_JPRB, 0.15338E+01_JPRB, 0.12426E+01_JPRB, 0.11419E+01_JPRB, 0.11863E+01_JPRB, 0.18556E+01_JPRB, 0.14296E+01_JPRB ,&
&  0.10871E+01_JPRB, 0.12767E+01_JPRB, 0.12400E+01_JPRB, 0.18538E+01_JPRB, 0.26542E+01_JPRB, 0.14036E+01_JPRB, 0.86338E+00_JPRB ,&
&  0.65796E+00_JPRB, 0.64462E+00_JPRB, 0.65356E+00_JPRB, 0.65514E+00_JPRB, 0.65482E+00_JPRB, 0.67008E+00_JPRB, 0.67190E+00_JPRB ,&
&  0.68277E+00_JPRB, 0.67191E+00_JPRB, 0.67392E+00_JPRB, 0.64458E+00_JPRB, 0.59922E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.64563E+00_JPRB, 0.64448E+00_JPRB, 0.63211E+00_JPRB, 0.62616E+00_JPRB, 0.63218E+00_JPRB, 0.64376E+00_JPRB, 0.66202E+00_JPRB ,&
&  0.69116E+00_JPRB, 0.72775E+00_JPRB, 0.76248E+00_JPRB, 0.80550E+00_JPRB, 0.81344E+00_JPRB, 0.76783E+00_JPRB, 0.76711E+00_JPRB ,&
&  0.76205E+00_JPRB, 0.77475E+00_JPRB, 0.76769E+00_JPRB, 0.85423E+00_JPRB, 0.87685E+00_JPRB, 0.88567E+00_JPRB, 0.99276E+00_JPRB ,&
&  0.99068E+00_JPRB, 0.11078E+01_JPRB, 0.11928E+01_JPRB, 0.13240E+01_JPRB, 0.21373E+01_JPRB, 0.20008E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.19547E+01_JPRB, 0.20047E+01_JPRB, 0.16049E+01_JPRB, 0.13349E+01_JPRB, 0.12540E+01_JPRB, 0.11632E+01_JPRB, 0.11366E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.13369E+01_JPRB, 0.15659E+01_JPRB, 0.18007E+01_JPRB, 0.19975E+01_JPRB, 0.21813E+01_JPRB, 0.23856E+01_JPRB ,&
&  0.25403E+01_JPRB, 0.27035E+01_JPRB, 0.27964E+01_JPRB, 0.29633E+01_JPRB, 0.35185E+01_JPRB, 0.40655E+01_JPRB, 0.45019E+01_JPRB ,&
&  0.48510E+01_JPRB, 0.51849E+01_JPRB, 0.57153E+01_JPRB, 0.55271E+01_JPRB, 0.50226E+01_JPRB, 0.47144E+01_JPRB, 0.39565E+01_JPRB ,&
&  0.32045E+01_JPRB, 0.24820E+01_JPRB, 0.22096E+01_JPRB, 0.27476E+01_JPRB, 0.28493E+01_JPRB, 0.24852E+01_JPRB, 0.28647E+01_JPRB ,&
&  0.29637E+01_JPRB, 0.30033E+01_JPRB, 0.25154E+01_JPRB, 0.21795E+01_JPRB, 0.17966E+01_JPRB, 0.17648E+01_JPRB, 0.18205E+01_JPRB ,&
&  0.15173E+01_JPRB, 0.11687E+01_JPRB, 0.10144E+01_JPRB, 0.95073E+00_JPRB, 0.95552E+00_JPRB, 0.12638E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.11898E+01_JPRB, 0.12034E+01_JPRB, 0.18161E+01_JPRB, 0.15628E+01_JPRB, 0.94715E+00_JPRB, 0.66069E+00_JPRB ,&
&  0.60148E+00_JPRB, 0.60840E+00_JPRB, 0.60814E+00_JPRB, 0.60707E+00_JPRB, 0.61868E+00_JPRB, 0.63474E+00_JPRB, 0.62296E+00_JPRB ,&
&  0.62223E+00_JPRB, 0.61210E+00_JPRB, 0.60472E+00_JPRB, 0.60417E+00_JPRB, 0.61619E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.79432E+00_JPRB, 0.80094E+00_JPRB, 0.80879E+00_JPRB, 0.81960E+00_JPRB, 0.84125E+00_JPRB, 0.86538E+00_JPRB, 0.88467E+00_JPRB ,&
&  0.92304E+00_JPRB, 0.95315E+00_JPRB, 0.98350E+00_JPRB, 0.99979E+00_JPRB, 0.10183E+01_JPRB, 0.10331E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10913E+01_JPRB, 0.10909E+01_JPRB, 0.11000E+01_JPRB, 0.11272E+01_JPRB, 0.11447E+01_JPRB, 0.11256E+01_JPRB ,&
&  0.11023E+01_JPRB, 0.10766E+01_JPRB, 0.11001E+01_JPRB, 0.11422E+01_JPRB, 0.13501E+01_JPRB, 0.14955E+01_JPRB, 0.14885E+01_JPRB ,&
&  0.15759E+01_JPRB, 0.16931E+01_JPRB, 0.17452E+01_JPRB, 0.13736E+01_JPRB, 0.12637E+01_JPRB, 0.12143E+01_JPRB, 0.11625E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.12365E+01_JPRB, 0.13103E+01_JPRB, 0.13906E+01_JPRB, 0.14895E+01_JPRB, 0.15298E+01_JPRB, 0.16005E+01_JPRB ,&
&  0.16263E+01_JPRB, 0.16881E+01_JPRB, 0.17851E+01_JPRB, 0.20097E+01_JPRB, 0.24217E+01_JPRB, 0.29361E+01_JPRB, 0.30208E+01_JPRB ,&
&  0.30778E+01_JPRB, 0.33832E+01_JPRB, 0.31706E+01_JPRB, 0.30540E+01_JPRB, 0.29457E+01_JPRB, 0.25250E+01_JPRB, 0.21004E+01_JPRB ,&
&  0.20226E+01_JPRB, 0.18308E+01_JPRB, 0.13921E+01_JPRB, 0.17158E+01_JPRB, 0.19298E+01_JPRB, 0.19749E+01_JPRB, 0.22280E+01_JPRB ,&
&  0.23699E+01_JPRB, 0.20784E+01_JPRB, 0.16821E+01_JPRB, 0.16422E+01_JPRB, 0.16051E+01_JPRB, 0.13416E+01_JPRB, 0.12967E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.10008E+01_JPRB, 0.95154E+00_JPRB, 0.96268E+00_JPRB, 0.11435E+01_JPRB, 0.18030E+01_JPRB, 0.19276E+01_JPRB ,&
&  0.15186E+01_JPRB, 0.12771E+01_JPRB, 0.14939E+01_JPRB, 0.23715E+01_JPRB, 0.12899E+01_JPRB, 0.87962E+00_JPRB, 0.65163E+00_JPRB ,&
&  0.64760E+00_JPRB, 0.65260E+00_JPRB, 0.65694E+00_JPRB, 0.66122E+00_JPRB, 0.66776E+00_JPRB, 0.68637E+00_JPRB, 0.68016E+00_JPRB ,&
&  0.67461E+00_JPRB, 0.67203E+00_JPRB, 0.68881E+00_JPRB, 0.71341E+00_JPRB, 0.75905E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.90719E+00_JPRB, 0.90551E+00_JPRB, 0.92102E+00_JPRB, 0.94183E+00_JPRB, 0.96176E+00_JPRB, 0.98493E+00_JPRB, 0.10216E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.10994E+01_JPRB, 0.11334E+01_JPRB, 0.11584E+01_JPRB, 0.11675E+01_JPRB, 0.11822E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.12147E+01_JPRB, 0.12082E+01_JPRB, 0.12289E+01_JPRB, 0.12638E+01_JPRB, 0.13056E+01_JPRB, 0.13235E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.13610E+01_JPRB, 0.14080E+01_JPRB, 0.14613E+01_JPRB, 0.15262E+01_JPRB, 0.16324E+01_JPRB, 0.18146E+01_JPRB, 0.19882E+01_JPRB ,&
&  0.16098E+01_JPRB, 0.18538E+01_JPRB, 0.17158E+01_JPRB, 0.14289E+01_JPRB, 0.13534E+01_JPRB, 0.14061E+01_JPRB, 0.13880E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.14112E+01_JPRB, 0.14526E+01_JPRB, 0.14926E+01_JPRB, 0.15320E+01_JPRB, 0.15852E+01_JPRB, 0.15912E+01_JPRB ,&
&  0.16360E+01_JPRB, 0.16698E+01_JPRB, 0.17104E+01_JPRB, 0.18099E+01_JPRB, 0.18310E+01_JPRB, 0.23145E+01_JPRB, 0.22476E+01_JPRB ,&
&  0.21036E+01_JPRB, 0.21469E+01_JPRB, 0.16768E+01_JPRB, 0.17210E+01_JPRB, 0.17318E+01_JPRB, 0.14919E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.17229E+01_JPRB, 0.18556E+01_JPRB, 0.14030E+01_JPRB, 0.16002E+01_JPRB, 0.17131E+01_JPRB, 0.17692E+01_JPRB, 0.18009E+01_JPRB ,&
&  0.17635E+01_JPRB, 0.15220E+01_JPRB, 0.13164E+01_JPRB, 0.12129E+01_JPRB, 0.11669E+01_JPRB, 0.10975E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.95482E+00_JPRB, 0.92191E+00_JPRB, 0.89805E+00_JPRB, 0.90717E+00_JPRB, 0.13242E+01_JPRB, 0.17001E+01_JPRB, 0.25445E+01_JPRB ,&
&  0.14134E+01_JPRB, 0.12242E+01_JPRB, 0.98973E+00_JPRB, 0.14672E+01_JPRB, 0.10611E+01_JPRB, 0.71932E+00_JPRB, 0.73233E+00_JPRB ,&
&  0.78769E+00_JPRB, 0.82548E+00_JPRB, 0.81819E+00_JPRB, 0.79552E+00_JPRB, 0.78919E+00_JPRB, 0.79219E+00_JPRB, 0.76965E+00_JPRB ,&
&  0.75968E+00_JPRB, 0.77876E+00_JPRB, 0.81529E+00_JPRB, 0.83154E+00_JPRB, 0.87059E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.96620E+00_JPRB, 0.99055E+00_JPRB, 0.10171E+01_JPRB, 0.10468E+01_JPRB, 0.10755E+01_JPRB, 0.11025E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.11985E+01_JPRB, 0.12261E+01_JPRB, 0.12427E+01_JPRB, 0.12512E+01_JPRB, 0.12686E+01_JPRB, 0.12751E+01_JPRB ,&
&  0.12790E+01_JPRB, 0.12832E+01_JPRB, 0.13051E+01_JPRB, 0.13200E+01_JPRB, 0.13281E+01_JPRB, 0.13305E+01_JPRB, 0.13545E+01_JPRB ,&
&  0.13600E+01_JPRB, 0.14052E+01_JPRB, 0.14415E+01_JPRB, 0.14879E+01_JPRB, 0.15272E+01_JPRB, 0.15483E+01_JPRB, 0.16659E+01_JPRB ,&
&  0.14187E+01_JPRB, 0.21231E+01_JPRB, 0.20569E+01_JPRB, 0.16702E+01_JPRB, 0.16136E+01_JPRB, 0.15362E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.15674E+01_JPRB, 0.16020E+01_JPRB, 0.16582E+01_JPRB, 0.17409E+01_JPRB, 0.18158E+01_JPRB, 0.18840E+01_JPRB, 0.19178E+01_JPRB ,&
&  0.19737E+01_JPRB, 0.20317E+01_JPRB, 0.20948E+01_JPRB, 0.21339E+01_JPRB, 0.22049E+01_JPRB, 0.22651E+01_JPRB, 0.22143E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.18933E+01_JPRB, 0.13784E+01_JPRB, 0.13522E+01_JPRB, 0.14500E+01_JPRB, 0.13780E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.16120E+01_JPRB, 0.17397E+01_JPRB, 0.15978E+01_JPRB, 0.15582E+01_JPRB, 0.15268E+01_JPRB, 0.14366E+01_JPRB, 0.13769E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.10674E+01_JPRB, 0.96150E+00_JPRB, 0.89662E+00_JPRB, 0.89946E+00_JPRB, 0.88400E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.80244E+00_JPRB, 0.81327E+00_JPRB, 0.86837E+00_JPRB, 0.95158E+00_JPRB, 0.11517E+01_JPRB, 0.22748E+01_JPRB, 0.25386E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.10380E+01_JPRB, 0.73294E+00_JPRB, 0.88831E+00_JPRB, 0.90622E+00_JPRB, 0.87103E+00_JPRB, 0.97867E+00_JPRB ,&
&  0.10778E+01_JPRB, 0.10952E+01_JPRB, 0.10130E+01_JPRB, 0.91272E+00_JPRB, 0.87348E+00_JPRB, 0.86003E+00_JPRB, 0.83628E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.86559E+00_JPRB, 0.89327E+00_JPRB, 0.91621E+00_JPRB, 0.94101E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.10242E+01_JPRB, 0.10471E+01_JPRB, 0.10701E+01_JPRB, 0.11055E+01_JPRB, 0.11437E+01_JPRB, 0.11795E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.12631E+01_JPRB, 0.12897E+01_JPRB, 0.13112E+01_JPRB, 0.13273E+01_JPRB, 0.13394E+01_JPRB, 0.13441E+01_JPRB ,&
&  0.13486E+01_JPRB, 0.13578E+01_JPRB, 0.13779E+01_JPRB, 0.13970E+01_JPRB, 0.14076E+01_JPRB, 0.14239E+01_JPRB, 0.14513E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.14864E+01_JPRB, 0.15276E+01_JPRB, 0.15880E+01_JPRB, 0.16695E+01_JPRB, 0.18573E+01_JPRB, 0.17486E+01_JPRB ,&
&  0.19816E+01_JPRB, 0.22527E+01_JPRB, 0.22539E+01_JPRB, 0.18541E+01_JPRB, 0.17084E+01_JPRB, 0.16706E+01_JPRB, 0.16884E+01_JPRB ,&
&  0.17203E+01_JPRB, 0.17415E+01_JPRB, 0.17857E+01_JPRB, 0.18650E+01_JPRB, 0.19830E+01_JPRB, 0.21033E+01_JPRB, 0.21695E+01_JPRB ,&
&  0.22588E+01_JPRB, 0.23648E+01_JPRB, 0.24784E+01_JPRB, 0.25853E+01_JPRB, 0.27467E+01_JPRB, 0.28465E+01_JPRB, 0.28136E+01_JPRB ,&
&  0.26747E+01_JPRB, 0.25672E+01_JPRB, 0.21835E+01_JPRB, 0.20974E+01_JPRB, 0.21785E+01_JPRB, 0.19158E+01_JPRB, 0.16975E+01_JPRB ,&
&  0.18601E+01_JPRB, 0.15670E+01_JPRB, 0.13515E+01_JPRB, 0.14549E+01_JPRB, 0.13677E+01_JPRB, 0.12136E+01_JPRB, 0.11398E+01_JPRB ,&
&  0.96737E+00_JPRB, 0.82388E+00_JPRB, 0.72646E+00_JPRB, 0.70622E+00_JPRB, 0.72114E+00_JPRB, 0.75708E+00_JPRB, 0.76288E+00_JPRB ,&
&  0.79168E+00_JPRB, 0.85233E+00_JPRB, 0.92034E+00_JPRB, 0.10294E+01_JPRB, 0.12963E+01_JPRB, 0.13065E+01_JPRB, 0.15325E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.92495E+00_JPRB, 0.98142E+00_JPRB, 0.10006E+01_JPRB, 0.10551E+01_JPRB, 0.10925E+01_JPRB, 0.12431E+01_JPRB ,&
&  0.14222E+01_JPRB, 0.15691E+01_JPRB, 0.14292E+01_JPRB, 0.12281E+01_JPRB, 0.10235E+01_JPRB, 0.93758E+00_JPRB, 0.90288E+00_JPRB ,&
&  0.90298E+00_JPRB, 0.93611E+00_JPRB, 0.96246E+00_JPRB, 0.97878E+00_JPRB, 0.10007E+01_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.10352E+01_JPRB, 0.10593E+01_JPRB, 0.10791E+01_JPRB, 0.11057E+01_JPRB, 0.11379E+01_JPRB, 0.11739E+01_JPRB, 0.12020E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.12573E+01_JPRB, 0.12818E+01_JPRB, 0.13100E+01_JPRB, 0.13328E+01_JPRB, 0.13628E+01_JPRB, 0.13784E+01_JPRB ,&
&  0.14067E+01_JPRB, 0.14126E+01_JPRB, 0.14319E+01_JPRB, 0.14566E+01_JPRB, 0.14829E+01_JPRB, 0.15119E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.15719E+01_JPRB, 0.15783E+01_JPRB, 0.15936E+01_JPRB, 0.16175E+01_JPRB, 0.16281E+01_JPRB, 0.17548E+01_JPRB, 0.16741E+01_JPRB ,&
&  0.22288E+01_JPRB, 0.24970E+01_JPRB, 0.24048E+01_JPRB, 0.20189E+01_JPRB, 0.18731E+01_JPRB, 0.18347E+01_JPRB, 0.18457E+01_JPRB ,&
&  0.18558E+01_JPRB, 0.18706E+01_JPRB, 0.18783E+01_JPRB, 0.19307E+01_JPRB, 0.20502E+01_JPRB, 0.22319E+01_JPRB, 0.23882E+01_JPRB ,&
&  0.25114E+01_JPRB, 0.26703E+01_JPRB, 0.28650E+01_JPRB, 0.30860E+01_JPRB, 0.33547E+01_JPRB, 0.35179E+01_JPRB, 0.35845E+01_JPRB ,&
&  0.35763E+01_JPRB, 0.35308E+01_JPRB, 0.32115E+01_JPRB, 0.33681E+01_JPRB, 0.34650E+01_JPRB, 0.27754E+01_JPRB, 0.22163E+01_JPRB ,&
&  0.17929E+01_JPRB, 0.14879E+01_JPRB, 0.14474E+01_JPRB, 0.14008E+01_JPRB, 0.12374E+01_JPRB, 0.11325E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.83869E+00_JPRB, 0.74171E+00_JPRB, 0.68679E+00_JPRB, 0.64412E+00_JPRB, 0.64920E+00_JPRB, 0.68019E+00_JPRB, 0.75621E+00_JPRB ,&
&  0.81950E+00_JPRB, 0.89082E+00_JPRB, 0.98542E+00_JPRB, 0.10800E+01_JPRB, 0.12256E+01_JPRB, 0.12293E+01_JPRB, 0.13923E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.11637E+01_JPRB, 0.11693E+01_JPRB, 0.90744E+00_JPRB, 0.11350E+01_JPRB, 0.12245E+01_JPRB, 0.13752E+01_JPRB ,&
&  0.14194E+01_JPRB, 0.18874E+01_JPRB, 0.22433E+01_JPRB, 0.18881E+01_JPRB, 0.13929E+01_JPRB, 0.10025E+01_JPRB, 0.91425E+00_JPRB ,&
&  0.91947E+00_JPRB, 0.94890E+00_JPRB, 0.97772E+00_JPRB, 0.99465E+00_JPRB, 0.10143E+01_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.99302E+00_JPRB, 0.10153E+01_JPRB, 0.10362E+01_JPRB, 0.10525E+01_JPRB, 0.10615E+01_JPRB, 0.10811E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.11705E+01_JPRB, 0.11998E+01_JPRB, 0.12281E+01_JPRB, 0.12577E+01_JPRB, 0.12855E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.13855E+01_JPRB, 0.14416E+01_JPRB, 0.14825E+01_JPRB, 0.15477E+01_JPRB, 0.16024E+01_JPRB, 0.16472E+01_JPRB, 0.17352E+01_JPRB ,&
&  0.18186E+01_JPRB, 0.19123E+01_JPRB, 0.19845E+01_JPRB, 0.20277E+01_JPRB, 0.20226E+01_JPRB, 0.21560E+01_JPRB, 0.19746E+01_JPRB ,&
&  0.24266E+01_JPRB, 0.26924E+01_JPRB, 0.25367E+01_JPRB, 0.21489E+01_JPRB, 0.20179E+01_JPRB, 0.19490E+01_JPRB, 0.18978E+01_JPRB ,&
&  0.18964E+01_JPRB, 0.18695E+01_JPRB, 0.17911E+01_JPRB, 0.19014E+01_JPRB, 0.20309E+01_JPRB, 0.22337E+01_JPRB, 0.24708E+01_JPRB ,&
&  0.26696E+01_JPRB, 0.28889E+01_JPRB, 0.31238E+01_JPRB, 0.33797E+01_JPRB, 0.37007E+01_JPRB, 0.39681E+01_JPRB, 0.41845E+01_JPRB ,&
&  0.43543E+01_JPRB, 0.44679E+01_JPRB, 0.45579E+01_JPRB, 0.43126E+01_JPRB, 0.42878E+01_JPRB, 0.37461E+01_JPRB, 0.27471E+01_JPRB ,&
&  0.17145E+01_JPRB, 0.14134E+01_JPRB, 0.14097E+01_JPRB, 0.13518E+01_JPRB, 0.12197E+01_JPRB, 0.10396E+01_JPRB, 0.95735E+00_JPRB ,&
&  0.79632E+00_JPRB, 0.70193E+00_JPRB, 0.66934E+00_JPRB, 0.64541E+00_JPRB, 0.64048E+00_JPRB, 0.68959E+00_JPRB, 0.73799E+00_JPRB ,&
&  0.81851E+00_JPRB, 0.92658E+00_JPRB, 0.10290E+01_JPRB, 0.11352E+01_JPRB, 0.13215E+01_JPRB, 0.14901E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.13148E+01_JPRB, 0.11976E+01_JPRB, 0.11933E+01_JPRB, 0.13042E+01_JPRB, 0.12361E+01_JPRB, 0.14402E+01_JPRB, 0.16607E+01_JPRB ,&
&  0.19788E+01_JPRB, 0.18936E+01_JPRB, 0.21724E+01_JPRB, 0.31940E+01_JPRB, 0.27195E+01_JPRB, 0.23148E+01_JPRB, 0.91629E+00_JPRB ,&
&  0.91656E+00_JPRB, 0.94013E+00_JPRB, 0.97384E+00_JPRB, 0.98312E+00_JPRB, 0.99237E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.87676E+00_JPRB, 0.91856E+00_JPRB, 0.93109E+00_JPRB, 0.96860E+00_JPRB, 0.97418E+00_JPRB, 0.97203E+00_JPRB, 0.99533E+00_JPRB ,&
&  0.10197E+01_JPRB, 0.10427E+01_JPRB, 0.10779E+01_JPRB, 0.11131E+01_JPRB, 0.11424E+01_JPRB, 0.11599E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.12978E+01_JPRB, 0.13743E+01_JPRB, 0.14759E+01_JPRB, 0.15548E+01_JPRB, 0.16313E+01_JPRB, 0.17338E+01_JPRB ,&
&  0.18246E+01_JPRB, 0.19698E+01_JPRB, 0.21185E+01_JPRB, 0.22601E+01_JPRB, 0.23537E+01_JPRB, 0.23910E+01_JPRB, 0.18874E+01_JPRB ,&
&  0.25802E+01_JPRB, 0.26223E+01_JPRB, 0.25297E+01_JPRB, 0.22257E+01_JPRB, 0.20801E+01_JPRB, 0.19769E+01_JPRB, 0.19111E+01_JPRB ,&
&  0.18722E+01_JPRB, 0.18145E+01_JPRB, 0.17411E+01_JPRB, 0.17060E+01_JPRB, 0.19773E+01_JPRB, 0.21178E+01_JPRB, 0.23452E+01_JPRB ,&
&  0.25842E+01_JPRB, 0.28090E+01_JPRB, 0.30415E+01_JPRB, 0.32965E+01_JPRB, 0.36335E+01_JPRB, 0.40150E+01_JPRB, 0.43833E+01_JPRB ,&
&  0.47176E+01_JPRB, 0.49120E+01_JPRB, 0.51801E+01_JPRB, 0.46496E+01_JPRB, 0.45152E+01_JPRB, 0.42289E+01_JPRB, 0.26189E+01_JPRB ,&
&  0.15615E+01_JPRB, 0.14260E+01_JPRB, 0.14770E+01_JPRB, 0.13981E+01_JPRB, 0.12222E+01_JPRB, 0.10332E+01_JPRB, 0.89593E+00_JPRB ,&
&  0.76667E+00_JPRB, 0.72364E+00_JPRB, 0.68998E+00_JPRB, 0.66954E+00_JPRB, 0.66572E+00_JPRB, 0.70482E+00_JPRB, 0.74539E+00_JPRB ,&
&  0.83071E+00_JPRB, 0.90295E+00_JPRB, 0.98759E+00_JPRB, 0.10738E+01_JPRB, 0.11958E+01_JPRB, 0.13435E+01_JPRB, 0.14852E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.11193E+01_JPRB, 0.11109E+01_JPRB, 0.11555E+01_JPRB, 0.11812E+01_JPRB, 0.12639E+01_JPRB, 0.13668E+01_JPRB ,&
&  0.15021E+01_JPRB, 0.15966E+01_JPRB, 0.14468E+01_JPRB, 0.12039E+01_JPRB, 0.15254E+01_JPRB, 0.15621E+01_JPRB, 0.76795E+00_JPRB ,&
&  0.76439E+00_JPRB, 0.80311E+00_JPRB, 0.82942E+00_JPRB, 0.85817E+00_JPRB, 0.88477E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.71844E+00_JPRB, 0.72990E+00_JPRB, 0.73358E+00_JPRB, 0.77978E+00_JPRB, 0.79600E+00_JPRB, 0.82312E+00_JPRB, 0.84496E+00_JPRB ,&
&  0.87028E+00_JPRB, 0.90607E+00_JPRB, 0.94669E+00_JPRB, 0.98717E+00_JPRB, 0.10223E+01_JPRB, 0.10500E+01_JPRB, 0.10756E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11338E+01_JPRB, 0.11754E+01_JPRB, 0.12336E+01_JPRB, 0.13259E+01_JPRB, 0.14320E+01_JPRB, 0.15474E+01_JPRB ,&
&  0.16645E+01_JPRB, 0.18153E+01_JPRB, 0.19890E+01_JPRB, 0.21855E+01_JPRB, 0.23880E+01_JPRB, 0.25347E+01_JPRB, 0.26344E+01_JPRB ,&
&  0.20331E+01_JPRB, 0.24665E+01_JPRB, 0.24275E+01_JPRB, 0.22400E+01_JPRB, 0.20943E+01_JPRB, 0.20127E+01_JPRB, 0.19420E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.17658E+01_JPRB, 0.17375E+01_JPRB, 0.17443E+01_JPRB, 0.18716E+01_JPRB, 0.19675E+01_JPRB, 0.21303E+01_JPRB ,&
&  0.23545E+01_JPRB, 0.25352E+01_JPRB, 0.27551E+01_JPRB, 0.29840E+01_JPRB, 0.32869E+01_JPRB, 0.37513E+01_JPRB, 0.42915E+01_JPRB ,&
&  0.48760E+01_JPRB, 0.53439E+01_JPRB, 0.56883E+01_JPRB, 0.50334E+01_JPRB, 0.46162E+01_JPRB, 0.41812E+01_JPRB, 0.25637E+01_JPRB ,&
&  0.16687E+01_JPRB, 0.13828E+01_JPRB, 0.14558E+01_JPRB, 0.14873E+01_JPRB, 0.12735E+01_JPRB, 0.11018E+01_JPRB, 0.93523E+00_JPRB ,&
&  0.81102E+00_JPRB, 0.74331E+00_JPRB, 0.71706E+00_JPRB, 0.69119E+00_JPRB, 0.69178E+00_JPRB, 0.71921E+00_JPRB, 0.74869E+00_JPRB ,&
&  0.80495E+00_JPRB, 0.83370E+00_JPRB, 0.89989E+00_JPRB, 0.95721E+00_JPRB, 0.10159E+01_JPRB, 0.10586E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.11103E+01_JPRB, 0.11178E+01_JPRB, 0.11181E+01_JPRB, 0.11402E+01_JPRB, 0.11433E+01_JPRB, 0.11551E+01_JPRB, 0.11799E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.12179E+01_JPRB, 0.11785E+01_JPRB, 0.11329E+01_JPRB, 0.10542E+01_JPRB, 0.98373E+00_JPRB, 0.76180E+00_JPRB ,&
&  0.72039E+00_JPRB, 0.67888E+00_JPRB, 0.70052E+00_JPRB, 0.72760E+00_JPRB, 0.72157E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.63622E+00_JPRB, 0.62522E+00_JPRB, 0.61316E+00_JPRB, 0.62207E+00_JPRB, 0.63256E+00_JPRB, 0.66259E+00_JPRB, 0.66689E+00_JPRB ,&
&  0.68740E+00_JPRB, 0.74520E+00_JPRB, 0.81780E+00_JPRB, 0.86424E+00_JPRB, 0.90184E+00_JPRB, 0.93870E+00_JPRB, 0.97275E+00_JPRB ,&
&  0.99276E+00_JPRB, 0.10078E+01_JPRB, 0.10218E+01_JPRB, 0.10411E+01_JPRB, 0.10796E+01_JPRB, 0.11422E+01_JPRB, 0.12413E+01_JPRB ,&
&  0.13467E+01_JPRB, 0.14659E+01_JPRB, 0.16145E+01_JPRB, 0.17996E+01_JPRB, 0.19992E+01_JPRB, 0.21249E+01_JPRB, 0.22247E+01_JPRB ,&
&  0.13262E+01_JPRB, 0.20437E+01_JPRB, 0.24588E+01_JPRB, 0.23190E+01_JPRB, 0.21391E+01_JPRB, 0.20415E+01_JPRB, 0.19439E+01_JPRB ,&
&  0.17934E+01_JPRB, 0.16405E+01_JPRB, 0.16048E+01_JPRB, 0.17767E+01_JPRB, 0.17474E+01_JPRB, 0.17984E+01_JPRB, 0.18581E+01_JPRB ,&
&  0.20241E+01_JPRB, 0.21480E+01_JPRB, 0.22219E+01_JPRB, 0.23268E+01_JPRB, 0.25012E+01_JPRB, 0.28903E+01_JPRB, 0.35570E+01_JPRB ,&
&  0.44607E+01_JPRB, 0.54241E+01_JPRB, 0.59729E+01_JPRB, 0.51384E+01_JPRB, 0.52893E+01_JPRB, 0.48234E+01_JPRB, 0.38943E+01_JPRB ,&
&  0.21746E+01_JPRB, 0.14986E+01_JPRB, 0.15283E+01_JPRB, 0.15510E+01_JPRB, 0.13608E+01_JPRB, 0.12245E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.92261E+00_JPRB, 0.84082E+00_JPRB, 0.78960E+00_JPRB, 0.75623E+00_JPRB, 0.74150E+00_JPRB, 0.74608E+00_JPRB, 0.76696E+00_JPRB ,&
&  0.78104E+00_JPRB, 0.80151E+00_JPRB, 0.83842E+00_JPRB, 0.88169E+00_JPRB, 0.93433E+00_JPRB, 0.98251E+00_JPRB, 0.10323E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10960E+01_JPRB, 0.11215E+01_JPRB, 0.11400E+01_JPRB, 0.11480E+01_JPRB, 0.11556E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11460E+01_JPRB, 0.11465E+01_JPRB, 0.11270E+01_JPRB, 0.10700E+01_JPRB, 0.10144E+01_JPRB, 0.96196E+00_JPRB ,&
&  0.87455E+00_JPRB, 0.76329E+00_JPRB, 0.67293E+00_JPRB, 0.65144E+00_JPRB, 0.63996E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.60175E+00_JPRB, 0.57598E+00_JPRB, 0.56985E+00_JPRB, 0.57546E+00_JPRB, 0.59202E+00_JPRB, 0.60257E+00_JPRB, 0.60301E+00_JPRB ,&
&  0.60804E+00_JPRB, 0.62313E+00_JPRB, 0.64179E+00_JPRB, 0.69967E+00_JPRB, 0.76507E+00_JPRB, 0.81697E+00_JPRB, 0.85159E+00_JPRB ,&
&  0.87885E+00_JPRB, 0.88626E+00_JPRB, 0.88131E+00_JPRB, 0.88255E+00_JPRB, 0.88669E+00_JPRB, 0.90643E+00_JPRB, 0.95213E+00_JPRB ,&
&  0.10344E+01_JPRB, 0.11288E+01_JPRB, 0.12502E+01_JPRB, 0.14167E+01_JPRB, 0.16380E+01_JPRB, 0.18682E+01_JPRB, 0.20441E+01_JPRB ,&
&  0.15262E+01_JPRB, 0.78911E+00_JPRB, 0.21527E+01_JPRB, 0.25753E+01_JPRB, 0.22832E+01_JPRB, 0.21696E+01_JPRB, 0.19672E+01_JPRB ,&
&  0.17023E+01_JPRB, 0.16002E+01_JPRB, 0.15180E+01_JPRB, 0.15945E+01_JPRB, 0.15999E+01_JPRB, 0.16512E+01_JPRB, 0.17014E+01_JPRB ,&
&  0.17940E+01_JPRB, 0.19278E+01_JPRB, 0.19539E+01_JPRB, 0.19281E+01_JPRB, 0.19381E+01_JPRB, 0.20797E+01_JPRB, 0.25166E+01_JPRB ,&
&  0.34063E+01_JPRB, 0.47361E+01_JPRB, 0.59845E+01_JPRB, 0.53902E+01_JPRB, 0.55841E+01_JPRB, 0.43536E+01_JPRB, 0.30725E+01_JPRB ,&
&  0.22047E+01_JPRB, 0.17036E+01_JPRB, 0.16263E+01_JPRB, 0.16341E+01_JPRB, 0.14821E+01_JPRB, 0.13008E+01_JPRB, 0.12539E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.95764E+00_JPRB, 0.87615E+00_JPRB, 0.82928E+00_JPRB, 0.80511E+00_JPRB, 0.79918E+00_JPRB, 0.80307E+00_JPRB ,&
&  0.80228E+00_JPRB, 0.80910E+00_JPRB, 0.82173E+00_JPRB, 0.85425E+00_JPRB, 0.90036E+00_JPRB, 0.96099E+00_JPRB, 0.10093E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.10559E+01_JPRB, 0.10763E+01_JPRB, 0.10869E+01_JPRB, 0.10944E+01_JPRB, 0.10714E+01_JPRB, 0.10684E+01_JPRB ,&
&  0.10805E+01_JPRB, 0.11014E+01_JPRB, 0.11222E+01_JPRB, 0.11520E+01_JPRB, 0.11119E+01_JPRB, 0.10734E+01_JPRB, 0.10807E+01_JPRB ,&
&  0.10503E+01_JPRB, 0.95530E+00_JPRB, 0.80943E+00_JPRB, 0.68061E+00_JPRB, 0.63110E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.66763E+00_JPRB, 0.63725E+00_JPRB, 0.60714E+00_JPRB, 0.60162E+00_JPRB, 0.61532E+00_JPRB, 0.61067E+00_JPRB, 0.60867E+00_JPRB ,&
&  0.60405E+00_JPRB, 0.59863E+00_JPRB, 0.58274E+00_JPRB, 0.59215E+00_JPRB, 0.62797E+00_JPRB, 0.68443E+00_JPRB, 0.72867E+00_JPRB ,&
&  0.76475E+00_JPRB, 0.77625E+00_JPRB, 0.77474E+00_JPRB, 0.77212E+00_JPRB, 0.76930E+00_JPRB, 0.76991E+00_JPRB, 0.77859E+00_JPRB ,&
&  0.80828E+00_JPRB, 0.85812E+00_JPRB, 0.92594E+00_JPRB, 0.10106E+01_JPRB, 0.11346E+01_JPRB, 0.13546E+01_JPRB, 0.17021E+01_JPRB ,&
&  0.22572E+01_JPRB, 0.16450E+01_JPRB, 0.10316E+01_JPRB, 0.22263E+01_JPRB, 0.24886E+01_JPRB, 0.24548E+01_JPRB, 0.20577E+01_JPRB ,&
&  0.18812E+01_JPRB, 0.16627E+01_JPRB, 0.15077E+01_JPRB, 0.15420E+01_JPRB, 0.14498E+01_JPRB, 0.14798E+01_JPRB, 0.15601E+01_JPRB ,&
&  0.16300E+01_JPRB, 0.17056E+01_JPRB, 0.17708E+01_JPRB, 0.17532E+01_JPRB, 0.16825E+01_JPRB, 0.16559E+01_JPRB, 0.18067E+01_JPRB ,&
&  0.23526E+01_JPRB, 0.34847E+01_JPRB, 0.53169E+01_JPRB, 0.55055E+01_JPRB, 0.57001E+01_JPRB, 0.39601E+01_JPRB, 0.28235E+01_JPRB ,&
&  0.22649E+01_JPRB, 0.21384E+01_JPRB, 0.19960E+01_JPRB, 0.17744E+01_JPRB, 0.15442E+01_JPRB, 0.14061E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.11180E+01_JPRB, 0.10129E+01_JPRB, 0.94705E+00_JPRB, 0.90120E+00_JPRB, 0.87117E+00_JPRB, 0.85215E+00_JPRB ,&
&  0.85261E+00_JPRB, 0.85057E+00_JPRB, 0.85873E+00_JPRB, 0.87182E+00_JPRB, 0.89295E+00_JPRB, 0.92878E+00_JPRB, 0.97401E+00_JPRB ,&
&  0.10077E+01_JPRB, 0.10214E+01_JPRB, 0.10262E+01_JPRB, 0.10233E+01_JPRB, 0.99823E+00_JPRB, 0.98208E+00_JPRB, 0.10067E+01_JPRB ,&
&  0.10078E+01_JPRB, 0.10276E+01_JPRB, 0.10356E+01_JPRB, 0.10708E+01_JPRB, 0.10922E+01_JPRB, 0.11005E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11076E+01_JPRB, 0.98006E+00_JPRB, 0.76455E+00_JPRB, 0.70005E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.75929E+00_JPRB, 0.73555E+00_JPRB, 0.72047E+00_JPRB, 0.72108E+00_JPRB, 0.72143E+00_JPRB, 0.70431E+00_JPRB, 0.68112E+00_JPRB ,&
&  0.65025E+00_JPRB, 0.62659E+00_JPRB, 0.59332E+00_JPRB, 0.56068E+00_JPRB, 0.55360E+00_JPRB, 0.56989E+00_JPRB, 0.60443E+00_JPRB ,&
&  0.63711E+00_JPRB, 0.65188E+00_JPRB, 0.66244E+00_JPRB, 0.67196E+00_JPRB, 0.68051E+00_JPRB, 0.68416E+00_JPRB, 0.68369E+00_JPRB ,&
&  0.69116E+00_JPRB, 0.70885E+00_JPRB, 0.73595E+00_JPRB, 0.76984E+00_JPRB, 0.82018E+00_JPRB, 0.91254E+00_JPRB, 0.10959E+01_JPRB ,&
&  0.14830E+01_JPRB, 0.17002E+01_JPRB, 0.84838E+00_JPRB, 0.17340E+01_JPRB, 0.24821E+01_JPRB, 0.24830E+01_JPRB, 0.23365E+01_JPRB ,&
&  0.21233E+01_JPRB, 0.17909E+01_JPRB, 0.16731E+01_JPRB, 0.15358E+01_JPRB, 0.13805E+01_JPRB, 0.13423E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.13897E+01_JPRB, 0.13694E+01_JPRB, 0.13912E+01_JPRB, 0.14552E+01_JPRB, 0.14449E+01_JPRB, 0.13895E+01_JPRB, 0.14144E+01_JPRB ,&
&  0.16790E+01_JPRB, 0.24543E+01_JPRB, 0.41656E+01_JPRB, 0.52443E+01_JPRB, 0.56122E+01_JPRB, 0.43309E+01_JPRB, 0.31797E+01_JPRB ,&
&  0.27210E+01_JPRB, 0.27506E+01_JPRB, 0.25044E+01_JPRB, 0.20531E+01_JPRB, 0.16861E+01_JPRB, 0.15689E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.13886E+01_JPRB, 0.12568E+01_JPRB, 0.11710E+01_JPRB, 0.11001E+01_JPRB, 0.10441E+01_JPRB, 0.98547E+00_JPRB, 0.93902E+00_JPRB ,&
&  0.92177E+00_JPRB, 0.91080E+00_JPRB, 0.90776E+00_JPRB, 0.90568E+00_JPRB, 0.90334E+00_JPRB, 0.91013E+00_JPRB, 0.93333E+00_JPRB ,&
&  0.96447E+00_JPRB, 0.98003E+00_JPRB, 0.98519E+00_JPRB, 0.96440E+00_JPRB, 0.94849E+00_JPRB, 0.94972E+00_JPRB, 0.96836E+00_JPRB ,&
&  0.97330E+00_JPRB, 0.98573E+00_JPRB, 0.98837E+00_JPRB, 0.99286E+00_JPRB, 0.10480E+01_JPRB, 0.10677E+01_JPRB, 0.11025E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11966E+01_JPRB, 0.11661E+01_JPRB, 0.86886E+00_JPRB, 0.80634E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.83597E+00_JPRB, 0.80675E+00_JPRB, 0.80312E+00_JPRB, 0.80507E+00_JPRB, 0.80384E+00_JPRB, 0.79447E+00_JPRB, 0.77271E+00_JPRB ,&
&  0.74466E+00_JPRB, 0.72172E+00_JPRB, 0.69446E+00_JPRB, 0.65887E+00_JPRB, 0.60796E+00_JPRB, 0.56287E+00_JPRB, 0.55630E+00_JPRB ,&
&  0.57816E+00_JPRB, 0.58548E+00_JPRB, 0.59200E+00_JPRB, 0.60145E+00_JPRB, 0.61276E+00_JPRB, 0.62578E+00_JPRB, 0.63496E+00_JPRB ,&
&  0.63950E+00_JPRB, 0.64732E+00_JPRB, 0.65959E+00_JPRB, 0.67342E+00_JPRB, 0.69539E+00_JPRB, 0.74050E+00_JPRB, 0.82852E+00_JPRB ,&
&  0.10683E+01_JPRB, 0.14218E+01_JPRB, 0.15020E+01_JPRB, 0.24960E+01_JPRB, 0.28857E+01_JPRB, 0.27367E+01_JPRB, 0.27279E+01_JPRB ,&
&  0.26480E+01_JPRB, 0.23370E+01_JPRB, 0.20180E+01_JPRB, 0.16213E+01_JPRB, 0.14037E+01_JPRB, 0.13040E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.11816E+01_JPRB, 0.11329E+01_JPRB, 0.11378E+01_JPRB, 0.11511E+01_JPRB, 0.11405E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.12808E+01_JPRB, 0.17449E+01_JPRB, 0.29608E+01_JPRB, 0.49140E+01_JPRB, 0.55610E+01_JPRB, 0.49699E+01_JPRB, 0.40472E+01_JPRB ,&
&  0.40148E+01_JPRB, 0.37429E+01_JPRB, 0.33113E+01_JPRB, 0.25769E+01_JPRB, 0.20105E+01_JPRB, 0.19515E+01_JPRB, 0.16638E+01_JPRB ,&
&  0.14846E+01_JPRB, 0.13625E+01_JPRB, 0.12870E+01_JPRB, 0.12312E+01_JPRB, 0.11834E+01_JPRB, 0.11201E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.99016E+00_JPRB, 0.96213E+00_JPRB, 0.94524E+00_JPRB, 0.93711E+00_JPRB, 0.92550E+00_JPRB, 0.91499E+00_JPRB, 0.91427E+00_JPRB ,&
&  0.93102E+00_JPRB, 0.94370E+00_JPRB, 0.91414E+00_JPRB, 0.90785E+00_JPRB, 0.91547E+00_JPRB, 0.91871E+00_JPRB, 0.92093E+00_JPRB ,&
&  0.92202E+00_JPRB, 0.94604E+00_JPRB, 0.97166E+00_JPRB, 0.97742E+00_JPRB, 0.10020E+01_JPRB, 0.10321E+01_JPRB, 0.10453E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10385E+01_JPRB, 0.10010E+01_JPRB, 0.91862E+00_JPRB, 0.87763E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.87552E+00_JPRB, 0.84911E+00_JPRB, 0.83503E+00_JPRB, 0.82656E+00_JPRB, 0.82224E+00_JPRB, 0.81840E+00_JPRB, 0.80140E+00_JPRB ,&
&  0.78166E+00_JPRB, 0.76880E+00_JPRB, 0.74730E+00_JPRB, 0.72457E+00_JPRB, 0.68026E+00_JPRB, 0.62313E+00_JPRB, 0.59555E+00_JPRB ,&
&  0.58063E+00_JPRB, 0.57665E+00_JPRB, 0.58477E+00_JPRB, 0.58964E+00_JPRB, 0.59260E+00_JPRB, 0.60002E+00_JPRB, 0.60720E+00_JPRB ,&
&  0.61542E+00_JPRB, 0.61924E+00_JPRB, 0.62702E+00_JPRB, 0.63467E+00_JPRB, 0.64429E+00_JPRB, 0.67495E+00_JPRB, 0.72867E+00_JPRB ,&
&  0.88306E+00_JPRB, 0.13213E+01_JPRB, 0.11543E+01_JPRB, 0.24247E+01_JPRB, 0.27582E+01_JPRB, 0.27744E+01_JPRB, 0.26014E+01_JPRB ,&
&  0.26061E+01_JPRB, 0.25897E+01_JPRB, 0.20228E+01_JPRB, 0.16607E+01_JPRB, 0.14176E+01_JPRB, 0.12934E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11125E+01_JPRB, 0.10463E+01_JPRB, 0.99718E+00_JPRB, 0.98098E+00_JPRB, 0.97215E+00_JPRB, 0.98407E+00_JPRB ,&
&  0.10358E+01_JPRB, 0.12818E+01_JPRB, 0.20673E+01_JPRB, 0.37778E+01_JPRB, 0.53179E+01_JPRB, 0.55104E+01_JPRB, 0.49176E+01_JPRB ,&
&  0.49447E+01_JPRB, 0.49866E+01_JPRB, 0.44671E+01_JPRB, 0.34016E+01_JPRB, 0.26539E+01_JPRB, 0.22827E+01_JPRB, 0.19053E+01_JPRB ,&
&  0.16630E+01_JPRB, 0.15124E+01_JPRB, 0.14193E+01_JPRB, 0.13408E+01_JPRB, 0.12770E+01_JPRB, 0.12092E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.99030E+00_JPRB, 0.96198E+00_JPRB, 0.94809E+00_JPRB, 0.94325E+00_JPRB, 0.93566E+00_JPRB, 0.92791E+00_JPRB ,&
&  0.92675E+00_JPRB, 0.93569E+00_JPRB, 0.88920E+00_JPRB, 0.87459E+00_JPRB, 0.88262E+00_JPRB, 0.88772E+00_JPRB, 0.89763E+00_JPRB ,&
&  0.92136E+00_JPRB, 0.93278E+00_JPRB, 0.94273E+00_JPRB, 0.93735E+00_JPRB, 0.95879E+00_JPRB, 0.10112E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.99366E+00_JPRB, 0.96774E+00_JPRB, 0.93107E+00_JPRB, 0.90213E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.88761E+00_JPRB, 0.86819E+00_JPRB, 0.84474E+00_JPRB, 0.82359E+00_JPRB, 0.82015E+00_JPRB, 0.81958E+00_JPRB, 0.80614E+00_JPRB ,&
&  0.79207E+00_JPRB, 0.77390E+00_JPRB, 0.74641E+00_JPRB, 0.71545E+00_JPRB, 0.68133E+00_JPRB, 0.64012E+00_JPRB, 0.60253E+00_JPRB ,&
&  0.58535E+00_JPRB, 0.57169E+00_JPRB, 0.57296E+00_JPRB, 0.57911E+00_JPRB, 0.58769E+00_JPRB, 0.59307E+00_JPRB, 0.59997E+00_JPRB ,&
&  0.60683E+00_JPRB, 0.61097E+00_JPRB, 0.61571E+00_JPRB, 0.61985E+00_JPRB, 0.62304E+00_JPRB, 0.63974E+00_JPRB, 0.69754E+00_JPRB ,&
&  0.82329E+00_JPRB, 0.15046E+01_JPRB, 0.11682E+01_JPRB, 0.20059E+01_JPRB, 0.23030E+01_JPRB, 0.24566E+01_JPRB, 0.23240E+01_JPRB ,&
&  0.23435E+01_JPRB, 0.22159E+01_JPRB, 0.18670E+01_JPRB, 0.16025E+01_JPRB, 0.14095E+01_JPRB, 0.12851E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.11375E+01_JPRB, 0.10681E+01_JPRB, 0.99375E+00_JPRB, 0.92894E+00_JPRB, 0.89797E+00_JPRB, 0.88335E+00_JPRB, 0.88324E+00_JPRB ,&
&  0.89943E+00_JPRB, 0.10116E+01_JPRB, 0.14407E+01_JPRB, 0.28086E+01_JPRB, 0.44717E+01_JPRB, 0.54555E+01_JPRB, 0.56531E+01_JPRB ,&
&  0.57676E+01_JPRB, 0.61562E+01_JPRB, 0.55901E+01_JPRB, 0.43947E+01_JPRB, 0.34555E+01_JPRB, 0.27784E+01_JPRB, 0.22611E+01_JPRB ,&
&  0.19447E+01_JPRB, 0.17549E+01_JPRB, 0.16224E+01_JPRB, 0.15120E+01_JPRB, 0.14063E+01_JPRB, 0.12885E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10070E+01_JPRB, 0.96833E+00_JPRB, 0.95742E+00_JPRB, 0.95696E+00_JPRB, 0.95558E+00_JPRB, 0.95401E+00_JPRB ,&
&  0.95193E+00_JPRB, 0.95210E+00_JPRB, 0.90547E+00_JPRB, 0.88476E+00_JPRB, 0.88172E+00_JPRB, 0.88855E+00_JPRB, 0.89297E+00_JPRB ,&
&  0.92537E+00_JPRB, 0.93206E+00_JPRB, 0.92158E+00_JPRB, 0.91926E+00_JPRB, 0.93715E+00_JPRB, 0.10026E+01_JPRB, 0.10103E+01_JPRB ,&
&  0.10011E+01_JPRB, 0.97170E+00_JPRB, 0.94703E+00_JPRB, 0.93042E+00_JPRB, 0.91059E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.89278E+00_JPRB, 0.87126E+00_JPRB, 0.85741E+00_JPRB, 0.84638E+00_JPRB, 0.84567E+00_JPRB, 0.84038E+00_JPRB, 0.82131E+00_JPRB ,&
&  0.79202E+00_JPRB, 0.75704E+00_JPRB, 0.72138E+00_JPRB, 0.69190E+00_JPRB, 0.66916E+00_JPRB, 0.64488E+00_JPRB, 0.62449E+00_JPRB ,&
&  0.60722E+00_JPRB, 0.58828E+00_JPRB, 0.57619E+00_JPRB, 0.57063E+00_JPRB, 0.57253E+00_JPRB, 0.57733E+00_JPRB, 0.58477E+00_JPRB ,&
&  0.59417E+00_JPRB, 0.60043E+00_JPRB, 0.60150E+00_JPRB, 0.60369E+00_JPRB, 0.60222E+00_JPRB, 0.61653E+00_JPRB, 0.67722E+00_JPRB ,&
&  0.82573E+00_JPRB, 0.78694E+00_JPRB, 0.13274E+01_JPRB, 0.16039E+01_JPRB, 0.18102E+01_JPRB, 0.20154E+01_JPRB, 0.21431E+01_JPRB ,&
&  0.22047E+01_JPRB, 0.19754E+01_JPRB, 0.16869E+01_JPRB, 0.14918E+01_JPRB, 0.13374E+01_JPRB, 0.12251E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.10541E+01_JPRB, 0.99059E+00_JPRB, 0.93964E+00_JPRB, 0.87740E+00_JPRB, 0.84053E+00_JPRB, 0.82733E+00_JPRB, 0.82420E+00_JPRB ,&
&  0.81891E+00_JPRB, 0.85738E+00_JPRB, 0.10643E+01_JPRB, 0.17831E+01_JPRB, 0.35893E+01_JPRB, 0.50114E+01_JPRB, 0.59327E+01_JPRB ,&
&  0.70049E+01_JPRB, 0.74775E+01_JPRB, 0.66648E+01_JPRB, 0.53538E+01_JPRB, 0.40816E+01_JPRB, 0.32244E+01_JPRB, 0.25228E+01_JPRB ,&
&  0.21165E+01_JPRB, 0.18896E+01_JPRB, 0.17573E+01_JPRB, 0.16518E+01_JPRB, 0.15417E+01_JPRB, 0.13901E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.10079E+01_JPRB, 0.96072E+00_JPRB, 0.93431E+00_JPRB, 0.92267E+00_JPRB, 0.92512E+00_JPRB, 0.93592E+00_JPRB ,&
&  0.95013E+00_JPRB, 0.96297E+00_JPRB, 0.92144E+00_JPRB, 0.89288E+00_JPRB, 0.88800E+00_JPRB, 0.90085E+00_JPRB, 0.91309E+00_JPRB ,&
&  0.91376E+00_JPRB, 0.90087E+00_JPRB, 0.88543E+00_JPRB, 0.88759E+00_JPRB, 0.90310E+00_JPRB, 0.99093E+00_JPRB, 0.10038E+01_JPRB ,&
&  0.99501E+00_JPRB, 0.96926E+00_JPRB, 0.93584E+00_JPRB, 0.92525E+00_JPRB, 0.91782E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.89345E+00_JPRB, 0.87531E+00_JPRB, 0.87426E+00_JPRB, 0.89047E+00_JPRB, 0.90550E+00_JPRB, 0.89148E+00_JPRB, 0.84043E+00_JPRB ,&
&  0.79171E+00_JPRB, 0.74961E+00_JPRB, 0.71099E+00_JPRB, 0.68077E+00_JPRB, 0.66062E+00_JPRB, 0.64585E+00_JPRB, 0.63127E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.61007E+00_JPRB, 0.59123E+00_JPRB, 0.57385E+00_JPRB, 0.56569E+00_JPRB, 0.56405E+00_JPRB, 0.56803E+00_JPRB ,&
&  0.57301E+00_JPRB, 0.57872E+00_JPRB, 0.58251E+00_JPRB, 0.58507E+00_JPRB, 0.58499E+00_JPRB, 0.59895E+00_JPRB, 0.65201E+00_JPRB ,&
&  0.82201E+00_JPRB, 0.11549E+01_JPRB, 0.13539E+01_JPRB, 0.13665E+01_JPRB, 0.14812E+01_JPRB, 0.16940E+01_JPRB, 0.19373E+01_JPRB ,&
&  0.19268E+01_JPRB, 0.17542E+01_JPRB, 0.15064E+01_JPRB, 0.13341E+01_JPRB, 0.12039E+01_JPRB, 0.11117E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.94908E+00_JPRB, 0.89929E+00_JPRB, 0.84995E+00_JPRB, 0.80662E+00_JPRB, 0.79061E+00_JPRB, 0.79157E+00_JPRB, 0.79375E+00_JPRB ,&
&  0.79083E+00_JPRB, 0.79110E+00_JPRB, 0.87698E+00_JPRB, 0.13417E+01_JPRB, 0.28099E+01_JPRB, 0.38032E+01_JPRB, 0.55065E+01_JPRB ,&
&  0.72412E+01_JPRB, 0.72240E+01_JPRB, 0.65394E+01_JPRB, 0.54543E+01_JPRB, 0.42525E+01_JPRB, 0.32405E+01_JPRB, 0.25979E+01_JPRB ,&
&  0.21952E+01_JPRB, 0.18810E+01_JPRB, 0.16920E+01_JPRB, 0.15718E+01_JPRB, 0.14732E+01_JPRB, 0.13792E+01_JPRB, 0.12649E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.10469E+01_JPRB, 0.97714E+00_JPRB, 0.92994E+00_JPRB, 0.90566E+00_JPRB, 0.90044E+00_JPRB, 0.90124E+00_JPRB ,&
&  0.89731E+00_JPRB, 0.90794E+00_JPRB, 0.91058E+00_JPRB, 0.89520E+00_JPRB, 0.89607E+00_JPRB, 0.90842E+00_JPRB, 0.91544E+00_JPRB ,&
&  0.88990E+00_JPRB, 0.86808E+00_JPRB, 0.86481E+00_JPRB, 0.86211E+00_JPRB, 0.89360E+00_JPRB, 0.98743E+00_JPRB, 0.98812E+00_JPRB ,&
&  0.97905E+00_JPRB, 0.95986E+00_JPRB, 0.93979E+00_JPRB, 0.91832E+00_JPRB, 0.91843E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.88290E+00_JPRB, 0.86325E+00_JPRB, 0.85690E+00_JPRB, 0.88096E+00_JPRB, 0.90916E+00_JPRB, 0.90304E+00_JPRB, 0.85558E+00_JPRB ,&
&  0.79035E+00_JPRB, 0.73799E+00_JPRB, 0.70156E+00_JPRB, 0.67587E+00_JPRB, 0.65604E+00_JPRB, 0.63883E+00_JPRB, 0.62035E+00_JPRB ,&
&  0.60550E+00_JPRB, 0.59679E+00_JPRB, 0.59047E+00_JPRB, 0.58248E+00_JPRB, 0.56900E+00_JPRB, 0.55852E+00_JPRB, 0.55482E+00_JPRB ,&
&  0.55283E+00_JPRB, 0.55263E+00_JPRB, 0.55787E+00_JPRB, 0.56411E+00_JPRB, 0.56663E+00_JPRB, 0.57496E+00_JPRB, 0.60981E+00_JPRB ,&
&  0.73317E+00_JPRB, 0.10254E+01_JPRB, 0.10262E+01_JPRB, 0.11175E+01_JPRB, 0.12971E+01_JPRB, 0.14828E+01_JPRB, 0.16469E+01_JPRB ,&
&  0.16481E+01_JPRB, 0.15570E+01_JPRB, 0.13846E+01_JPRB, 0.12123E+01_JPRB, 0.10733E+01_JPRB, 0.98954E+00_JPRB, 0.93020E+00_JPRB ,&
&  0.87765E+00_JPRB, 0.82591E+00_JPRB, 0.78066E+00_JPRB, 0.74675E+00_JPRB, 0.72417E+00_JPRB, 0.71623E+00_JPRB, 0.71936E+00_JPRB ,&
&  0.72252E+00_JPRB, 0.72287E+00_JPRB, 0.73987E+00_JPRB, 0.90414E+00_JPRB, 0.16607E+01_JPRB, 0.27387E+01_JPRB, 0.39290E+01_JPRB ,&
&  0.43483E+01_JPRB, 0.45678E+01_JPRB, 0.45338E+01_JPRB, 0.42544E+01_JPRB, 0.37310E+01_JPRB, 0.32076E+01_JPRB, 0.26422E+01_JPRB ,&
&  0.22241E+01_JPRB, 0.19430E+01_JPRB, 0.17393E+01_JPRB, 0.15663E+01_JPRB, 0.14107E+01_JPRB, 0.12856E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.10789E+01_JPRB, 0.10264E+01_JPRB, 0.97851E+00_JPRB, 0.94263E+00_JPRB, 0.92211E+00_JPRB, 0.91822E+00_JPRB ,&
&  0.91014E+00_JPRB, 0.88987E+00_JPRB, 0.88244E+00_JPRB, 0.88160E+00_JPRB, 0.88569E+00_JPRB, 0.88395E+00_JPRB, 0.87827E+00_JPRB ,&
&  0.87919E+00_JPRB, 0.87772E+00_JPRB, 0.87332E+00_JPRB, 0.86824E+00_JPRB, 0.89127E+00_JPRB, 0.95878E+00_JPRB, 0.97639E+00_JPRB ,&
&  0.97316E+00_JPRB, 0.95338E+00_JPRB, 0.93688E+00_JPRB, 0.92447E+00_JPRB, 0.91501E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.96379E+00_JPRB, 0.88271E+00_JPRB, 0.83676E+00_JPRB, 0.82577E+00_JPRB, 0.83338E+00_JPRB, 0.82233E+00_JPRB, 0.79497E+00_JPRB ,&
&  0.76411E+00_JPRB, 0.73604E+00_JPRB, 0.71088E+00_JPRB, 0.68295E+00_JPRB, 0.65550E+00_JPRB, 0.63166E+00_JPRB, 0.60936E+00_JPRB ,&
&  0.58850E+00_JPRB, 0.57606E+00_JPRB, 0.56758E+00_JPRB, 0.56178E+00_JPRB, 0.55754E+00_JPRB, 0.54992E+00_JPRB, 0.54192E+00_JPRB ,&
&  0.53579E+00_JPRB, 0.52700E+00_JPRB, 0.52401E+00_JPRB, 0.53067E+00_JPRB, 0.53780E+00_JPRB, 0.54565E+00_JPRB, 0.57307E+00_JPRB ,&
&  0.67081E+00_JPRB, 0.78598E+00_JPRB, 0.88111E+00_JPRB, 0.96666E+00_JPRB, 0.10529E+01_JPRB, 0.11587E+01_JPRB, 0.11966E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.11593E+01_JPRB, 0.11314E+01_JPRB, 0.10415E+01_JPRB, 0.94157E+00_JPRB, 0.87474E+00_JPRB, 0.82790E+00_JPRB ,&
&  0.78857E+00_JPRB, 0.74701E+00_JPRB, 0.70523E+00_JPRB, 0.67216E+00_JPRB, 0.65253E+00_JPRB, 0.64536E+00_JPRB, 0.64598E+00_JPRB ,&
&  0.64978E+00_JPRB, 0.65598E+00_JPRB, 0.66201E+00_JPRB, 0.68688E+00_JPRB, 0.86070E+00_JPRB, 0.13324E+01_JPRB, 0.20657E+01_JPRB ,&
&  0.24464E+01_JPRB, 0.25966E+01_JPRB, 0.27136E+01_JPRB, 0.27415E+01_JPRB, 0.25679E+01_JPRB, 0.23692E+01_JPRB, 0.21454E+01_JPRB ,&
&  0.19214E+01_JPRB, 0.16868E+01_JPRB, 0.14865E+01_JPRB, 0.13292E+01_JPRB, 0.12069E+01_JPRB, 0.11222E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10531E+01_JPRB, 0.10415E+01_JPRB, 0.10308E+01_JPRB, 0.10083E+01_JPRB, 0.97910E+00_JPRB, 0.95718E+00_JPRB, 0.94291E+00_JPRB ,&
&  0.92477E+00_JPRB, 0.90670E+00_JPRB, 0.89080E+00_JPRB, 0.87877E+00_JPRB, 0.87176E+00_JPRB, 0.86443E+00_JPRB, 0.86057E+00_JPRB ,&
&  0.86056E+00_JPRB, 0.86722E+00_JPRB, 0.85268E+00_JPRB, 0.84842E+00_JPRB, 0.90299E+00_JPRB, 0.92311E+00_JPRB, 0.94433E+00_JPRB ,&
&  0.96103E+00_JPRB, 0.95064E+00_JPRB, 0.92476E+00_JPRB, 0.91699E+00_JPRB, 0.10203E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.86619E+00_JPRB, 0.83146E+00_JPRB, 0.78523E+00_JPRB, 0.75901E+00_JPRB, 0.74201E+00_JPRB, 0.71836E+00_JPRB, 0.69980E+00_JPRB ,&
&  0.68446E+00_JPRB, 0.66922E+00_JPRB, 0.64998E+00_JPRB, 0.62729E+00_JPRB, 0.60539E+00_JPRB, 0.58951E+00_JPRB, 0.57639E+00_JPRB ,&
&  0.56423E+00_JPRB, 0.55265E+00_JPRB, 0.54201E+00_JPRB, 0.53394E+00_JPRB, 0.52536E+00_JPRB, 0.51874E+00_JPRB, 0.51319E+00_JPRB ,&
&  0.50676E+00_JPRB, 0.49940E+00_JPRB, 0.49000E+00_JPRB, 0.49340E+00_JPRB, 0.50360E+00_JPRB, 0.51568E+00_JPRB, 0.53171E+00_JPRB ,&
&  0.59223E+00_JPRB, 0.64039E+00_JPRB, 0.72483E+00_JPRB, 0.81662E+00_JPRB, 0.88203E+00_JPRB, 0.93171E+00_JPRB, 0.93290E+00_JPRB ,&
&  0.89490E+00_JPRB, 0.90401E+00_JPRB, 0.88578E+00_JPRB, 0.84612E+00_JPRB, 0.81621E+00_JPRB, 0.75705E+00_JPRB, 0.70417E+00_JPRB ,&
&  0.66722E+00_JPRB, 0.64339E+00_JPRB, 0.62305E+00_JPRB, 0.60218E+00_JPRB, 0.58980E+00_JPRB, 0.58741E+00_JPRB, 0.59228E+00_JPRB ,&
&  0.59749E+00_JPRB, 0.60478E+00_JPRB, 0.61190E+00_JPRB, 0.62163E+00_JPRB, 0.64903E+00_JPRB, 0.78741E+00_JPRB, 0.11482E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.16139E+01_JPRB, 0.17376E+01_JPRB, 0.17916E+01_JPRB, 0.17453E+01_JPRB, 0.16445E+01_JPRB, 0.15159E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.12310E+01_JPRB, 0.10982E+01_JPRB, 0.99520E+00_JPRB, 0.91653E+00_JPRB, 0.86528E+00_JPRB, 0.83994E+00_JPRB ,&
&  0.83947E+00_JPRB, 0.85646E+00_JPRB, 0.88060E+00_JPRB, 0.90306E+00_JPRB, 0.91452E+00_JPRB, 0.91309E+00_JPRB, 0.90525E+00_JPRB ,&
&  0.89127E+00_JPRB, 0.87276E+00_JPRB, 0.85198E+00_JPRB, 0.83404E+00_JPRB, 0.81965E+00_JPRB, 0.80902E+00_JPRB, 0.80220E+00_JPRB ,&
&  0.79831E+00_JPRB, 0.79821E+00_JPRB, 0.80379E+00_JPRB, 0.82042E+00_JPRB, 0.82843E+00_JPRB, 0.84444E+00_JPRB, 0.85431E+00_JPRB ,&
&  0.86515E+00_JPRB, 0.86611E+00_JPRB, 0.85280E+00_JPRB, 0.82272E+00_JPRB, 0.82946E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.75115E+00_JPRB, 0.74126E+00_JPRB, 0.71705E+00_JPRB, 0.69093E+00_JPRB, 0.66910E+00_JPRB, 0.64870E+00_JPRB, 0.63054E+00_JPRB ,&
&  0.61475E+00_JPRB, 0.59985E+00_JPRB, 0.58350E+00_JPRB, 0.56808E+00_JPRB, 0.55311E+00_JPRB, 0.54105E+00_JPRB, 0.52998E+00_JPRB ,&
&  0.52228E+00_JPRB, 0.51655E+00_JPRB, 0.51071E+00_JPRB, 0.50388E+00_JPRB, 0.49694E+00_JPRB, 0.49019E+00_JPRB, 0.48303E+00_JPRB ,&
&  0.47735E+00_JPRB, 0.47417E+00_JPRB, 0.46612E+00_JPRB, 0.46350E+00_JPRB, 0.47097E+00_JPRB, 0.48235E+00_JPRB, 0.49447E+00_JPRB ,&
&  0.53642E+00_JPRB, 0.52354E+00_JPRB, 0.54153E+00_JPRB, 0.63159E+00_JPRB, 0.70097E+00_JPRB, 0.77845E+00_JPRB, 0.81329E+00_JPRB ,&
&  0.82015E+00_JPRB, 0.82759E+00_JPRB, 0.80416E+00_JPRB, 0.77132E+00_JPRB, 0.72063E+00_JPRB, 0.67765E+00_JPRB, 0.64554E+00_JPRB ,&
&  0.61758E+00_JPRB, 0.59238E+00_JPRB, 0.57269E+00_JPRB, 0.55711E+00_JPRB, 0.54491E+00_JPRB, 0.54007E+00_JPRB, 0.54187E+00_JPRB ,&
&  0.54527E+00_JPRB, 0.54910E+00_JPRB, 0.55442E+00_JPRB, 0.56422E+00_JPRB, 0.57790E+00_JPRB, 0.60941E+00_JPRB, 0.72855E+00_JPRB ,&
&  0.92039E+00_JPRB, 0.10712E+01_JPRB, 0.12057E+01_JPRB, 0.12849E+01_JPRB, 0.12857E+01_JPRB, 0.12252E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.98221E+00_JPRB, 0.88391E+00_JPRB, 0.82336E+00_JPRB, 0.78688E+00_JPRB, 0.76770E+00_JPRB, 0.75479E+00_JPRB, 0.74435E+00_JPRB ,&
&  0.73519E+00_JPRB, 0.73097E+00_JPRB, 0.73545E+00_JPRB, 0.74787E+00_JPRB, 0.76560E+00_JPRB, 0.77952E+00_JPRB, 0.78437E+00_JPRB ,&
&  0.78193E+00_JPRB, 0.77470E+00_JPRB, 0.76673E+00_JPRB, 0.75676E+00_JPRB, 0.74525E+00_JPRB, 0.73270E+00_JPRB, 0.72457E+00_JPRB ,&
&  0.72239E+00_JPRB, 0.72526E+00_JPRB, 0.73226E+00_JPRB, 0.72745E+00_JPRB, 0.75301E+00_JPRB, 0.76114E+00_JPRB, 0.76529E+00_JPRB ,&
&  0.76747E+00_JPRB, 0.76313E+00_JPRB, 0.75497E+00_JPRB, 0.67227E+00_JPRB, 0.73411E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.64206E+00_JPRB, 0.63994E+00_JPRB, 0.62424E+00_JPRB, 0.60639E+00_JPRB, 0.58830E+00_JPRB, 0.57090E+00_JPRB, 0.55750E+00_JPRB ,&
&  0.54700E+00_JPRB, 0.53406E+00_JPRB, 0.52285E+00_JPRB, 0.51505E+00_JPRB, 0.50693E+00_JPRB, 0.49959E+00_JPRB, 0.49136E+00_JPRB ,&
&  0.48474E+00_JPRB, 0.48036E+00_JPRB, 0.47703E+00_JPRB, 0.47309E+00_JPRB, 0.46871E+00_JPRB, 0.46355E+00_JPRB, 0.45853E+00_JPRB ,&
&  0.45397E+00_JPRB, 0.45090E+00_JPRB, 0.44582E+00_JPRB, 0.44239E+00_JPRB, 0.44530E+00_JPRB, 0.45345E+00_JPRB, 0.46119E+00_JPRB ,&
&  0.49615E+00_JPRB, 0.44684E+00_JPRB, 0.46095E+00_JPRB, 0.50386E+00_JPRB, 0.55344E+00_JPRB, 0.61797E+00_JPRB, 0.67052E+00_JPRB ,&
&  0.69432E+00_JPRB, 0.71636E+00_JPRB, 0.70462E+00_JPRB, 0.68305E+00_JPRB, 0.64652E+00_JPRB, 0.61060E+00_JPRB, 0.58299E+00_JPRB ,&
&  0.56629E+00_JPRB, 0.55368E+00_JPRB, 0.53900E+00_JPRB, 0.52288E+00_JPRB, 0.50805E+00_JPRB, 0.49793E+00_JPRB, 0.49380E+00_JPRB ,&
&  0.49476E+00_JPRB, 0.49662E+00_JPRB, 0.49713E+00_JPRB, 0.50070E+00_JPRB, 0.50971E+00_JPRB, 0.52427E+00_JPRB, 0.55688E+00_JPRB ,&
&  0.62888E+00_JPRB, 0.72543E+00_JPRB, 0.81122E+00_JPRB, 0.89268E+00_JPRB, 0.94539E+00_JPRB, 0.96149E+00_JPRB, 0.91380E+00_JPRB ,&
&  0.84607E+00_JPRB, 0.78282E+00_JPRB, 0.74128E+00_JPRB, 0.71929E+00_JPRB, 0.70638E+00_JPRB, 0.69726E+00_JPRB, 0.68387E+00_JPRB ,&
&  0.66669E+00_JPRB, 0.64660E+00_JPRB, 0.62890E+00_JPRB, 0.61120E+00_JPRB, 0.59889E+00_JPRB, 0.59546E+00_JPRB, 0.59904E+00_JPRB ,&
&  0.60446E+00_JPRB, 0.60807E+00_JPRB, 0.61085E+00_JPRB, 0.61378E+00_JPRB, 0.61551E+00_JPRB, 0.61704E+00_JPRB, 0.61818E+00_JPRB ,&
&  0.62231E+00_JPRB, 0.62860E+00_JPRB, 0.63538E+00_JPRB, 0.64794E+00_JPRB, 0.65675E+00_JPRB, 0.66218E+00_JPRB, 0.66531E+00_JPRB ,&
&  0.66500E+00_JPRB, 0.66835E+00_JPRB, 0.57935E+00_JPRB, 0.61656E+00_JPRB, 0.63286E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.56037E+00_JPRB, 0.55923E+00_JPRB, 0.55392E+00_JPRB, 0.54094E+00_JPRB, 0.52589E+00_JPRB, 0.51272E+00_JPRB, 0.50406E+00_JPRB ,&
&  0.49835E+00_JPRB, 0.49084E+00_JPRB, 0.48445E+00_JPRB, 0.48017E+00_JPRB, 0.47499E+00_JPRB, 0.46885E+00_JPRB, 0.46012E+00_JPRB ,&
&  0.45259E+00_JPRB, 0.44666E+00_JPRB, 0.44237E+00_JPRB, 0.43984E+00_JPRB, 0.43834E+00_JPRB, 0.43671E+00_JPRB, 0.43415E+00_JPRB ,&
&  0.43164E+00_JPRB, 0.43033E+00_JPRB, 0.42892E+00_JPRB, 0.42774E+00_JPRB, 0.42805E+00_JPRB, 0.43297E+00_JPRB, 0.43990E+00_JPRB ,&
&  0.43223E+00_JPRB, 0.40771E+00_JPRB, 0.42023E+00_JPRB, 0.44148E+00_JPRB, 0.46891E+00_JPRB, 0.51435E+00_JPRB, 0.56189E+00_JPRB ,&
&  0.59019E+00_JPRB, 0.60620E+00_JPRB, 0.59434E+00_JPRB, 0.57764E+00_JPRB, 0.56002E+00_JPRB, 0.54056E+00_JPRB, 0.52486E+00_JPRB ,&
&  0.51145E+00_JPRB, 0.50214E+00_JPRB, 0.49421E+00_JPRB, 0.48319E+00_JPRB, 0.47011E+00_JPRB, 0.45950E+00_JPRB, 0.45314E+00_JPRB ,&
&  0.45103E+00_JPRB, 0.45233E+00_JPRB, 0.45427E+00_JPRB, 0.45637E+00_JPRB, 0.46047E+00_JPRB, 0.46872E+00_JPRB, 0.48163E+00_JPRB ,&
&  0.49889E+00_JPRB, 0.53230E+00_JPRB, 0.57888E+00_JPRB, 0.62485E+00_JPRB, 0.66929E+00_JPRB, 0.70188E+00_JPRB, 0.72159E+00_JPRB ,&
&  0.72331E+00_JPRB, 0.70682E+00_JPRB, 0.68290E+00_JPRB, 0.65760E+00_JPRB, 0.62759E+00_JPRB, 0.59147E+00_JPRB, 0.55288E+00_JPRB ,&
&  0.52051E+00_JPRB, 0.49604E+00_JPRB, 0.48023E+00_JPRB, 0.47285E+00_JPRB, 0.47003E+00_JPRB, 0.47208E+00_JPRB, 0.47736E+00_JPRB ,&
&  0.48468E+00_JPRB, 0.49282E+00_JPRB, 0.49912E+00_JPRB, 0.50575E+00_JPRB, 0.51158E+00_JPRB, 0.51780E+00_JPRB, 0.52440E+00_JPRB ,&
&  0.53180E+00_JPRB, 0.53881E+00_JPRB, 0.54585E+00_JPRB, 0.55643E+00_JPRB, 0.56440E+00_JPRB, 0.57181E+00_JPRB, 0.57529E+00_JPRB ,&
&  0.57193E+00_JPRB, 0.56936E+00_JPRB, 0.56590E+00_JPRB, 0.56108E+00_JPRB, 0.56214E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.50275E+00_JPRB, 0.50080E+00_JPRB, 0.49865E+00_JPRB, 0.49513E+00_JPRB, 0.48901E+00_JPRB, 0.48038E+00_JPRB, 0.47339E+00_JPRB ,&
&  0.46879E+00_JPRB, 0.46397E+00_JPRB, 0.45987E+00_JPRB, 0.45499E+00_JPRB, 0.44911E+00_JPRB, 0.44179E+00_JPRB, 0.43220E+00_JPRB ,&
&  0.42329E+00_JPRB, 0.41513E+00_JPRB, 0.40975E+00_JPRB, 0.40719E+00_JPRB, 0.40710E+00_JPRB, 0.40710E+00_JPRB, 0.40523E+00_JPRB ,&
&  0.40401E+00_JPRB, 0.40392E+00_JPRB, 0.40400E+00_JPRB, 0.40367E+00_JPRB, 0.40375E+00_JPRB, 0.40757E+00_JPRB, 0.41182E+00_JPRB ,&
&  0.42171E+00_JPRB, 0.39945E+00_JPRB, 0.41194E+00_JPRB, 0.41694E+00_JPRB, 0.42927E+00_JPRB, 0.45466E+00_JPRB, 0.48756E+00_JPRB ,&
&  0.51361E+00_JPRB, 0.51502E+00_JPRB, 0.50367E+00_JPRB, 0.49000E+00_JPRB, 0.48232E+00_JPRB, 0.47410E+00_JPRB, 0.46479E+00_JPRB ,&
&  0.45591E+00_JPRB, 0.44966E+00_JPRB, 0.44429E+00_JPRB, 0.43901E+00_JPRB, 0.43196E+00_JPRB, 0.42317E+00_JPRB, 0.41487E+00_JPRB ,&
&  0.40915E+00_JPRB, 0.40726E+00_JPRB, 0.40852E+00_JPRB, 0.41126E+00_JPRB, 0.41451E+00_JPRB, 0.42016E+00_JPRB, 0.42765E+00_JPRB ,&
&  0.43631E+00_JPRB, 0.44225E+00_JPRB, 0.44989E+00_JPRB, 0.46549E+00_JPRB, 0.48421E+00_JPRB, 0.49943E+00_JPRB, 0.50950E+00_JPRB ,&
&  0.51457E+00_JPRB, 0.51165E+00_JPRB, 0.50308E+00_JPRB, 0.49030E+00_JPRB, 0.47526E+00_JPRB, 0.45963E+00_JPRB, 0.44579E+00_JPRB ,&
&  0.43461E+00_JPRB, 0.42532E+00_JPRB, 0.41754E+00_JPRB, 0.41219E+00_JPRB, 0.40964E+00_JPRB, 0.40914E+00_JPRB, 0.41056E+00_JPRB ,&
&  0.41499E+00_JPRB, 0.42100E+00_JPRB, 0.42653E+00_JPRB, 0.43193E+00_JPRB, 0.43677E+00_JPRB, 0.44090E+00_JPRB, 0.44608E+00_JPRB ,&
&  0.45285E+00_JPRB, 0.45801E+00_JPRB, 0.46227E+00_JPRB, 0.46792E+00_JPRB, 0.47453E+00_JPRB, 0.48173E+00_JPRB, 0.48847E+00_JPRB ,&
&  0.49311E+00_JPRB, 0.49548E+00_JPRB, 0.49860E+00_JPRB, 0.50002E+00_JPRB, 0.50324E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.46345E+00_JPRB, 0.46244E+00_JPRB, 0.46032E+00_JPRB, 0.45883E+00_JPRB, 0.45622E+00_JPRB, 0.45206E+00_JPRB, 0.44719E+00_JPRB ,&
&  0.44318E+00_JPRB, 0.43952E+00_JPRB, 0.43562E+00_JPRB, 0.43038E+00_JPRB, 0.42323E+00_JPRB, 0.41613E+00_JPRB, 0.40792E+00_JPRB ,&
&  0.40010E+00_JPRB, 0.39326E+00_JPRB, 0.38844E+00_JPRB, 0.38555E+00_JPRB, 0.38321E+00_JPRB, 0.38116E+00_JPRB, 0.37878E+00_JPRB ,&
&  0.37732E+00_JPRB, 0.37677E+00_JPRB, 0.37694E+00_JPRB, 0.37761E+00_JPRB, 0.37831E+00_JPRB, 0.38127E+00_JPRB, 0.38329E+00_JPRB ,&
&  0.38698E+00_JPRB, 0.39411E+00_JPRB, 0.40262E+00_JPRB, 0.40669E+00_JPRB, 0.41346E+00_JPRB, 0.42231E+00_JPRB, 0.43297E+00_JPRB ,&
&  0.44279E+00_JPRB, 0.44704E+00_JPRB, 0.44109E+00_JPRB, 0.43158E+00_JPRB, 0.42551E+00_JPRB, 0.42261E+00_JPRB, 0.41632E+00_JPRB ,&
&  0.40985E+00_JPRB, 0.40494E+00_JPRB, 0.40101E+00_JPRB, 0.39814E+00_JPRB, 0.39526E+00_JPRB, 0.39109E+00_JPRB, 0.38648E+00_JPRB ,&
&  0.38074E+00_JPRB, 0.37637E+00_JPRB, 0.37310E+00_JPRB, 0.37193E+00_JPRB, 0.37226E+00_JPRB, 0.37388E+00_JPRB, 0.37662E+00_JPRB ,&
&  0.38123E+00_JPRB, 0.38610E+00_JPRB, 0.38864E+00_JPRB, 0.39007E+00_JPRB, 0.39238E+00_JPRB, 0.39642E+00_JPRB, 0.39994E+00_JPRB ,&
&  0.40122E+00_JPRB, 0.40051E+00_JPRB, 0.39786E+00_JPRB, 0.39354E+00_JPRB, 0.38814E+00_JPRB, 0.38336E+00_JPRB, 0.37936E+00_JPRB ,&
&  0.37568E+00_JPRB, 0.37198E+00_JPRB, 0.36839E+00_JPRB, 0.36638E+00_JPRB, 0.36531E+00_JPRB, 0.36506E+00_JPRB, 0.36638E+00_JPRB ,&
&  0.36928E+00_JPRB, 0.37291E+00_JPRB, 0.37716E+00_JPRB, 0.38135E+00_JPRB, 0.38534E+00_JPRB, 0.38861E+00_JPRB, 0.39242E+00_JPRB ,&
&  0.39826E+00_JPRB, 0.40469E+00_JPRB, 0.41105E+00_JPRB, 0.41685E+00_JPRB, 0.42315E+00_JPRB, 0.43059E+00_JPRB, 0.43855E+00_JPRB ,&
&  0.44401E+00_JPRB, 0.44770E+00_JPRB, 0.45229E+00_JPRB, 0.45737E+00_JPRB, 0.46188E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.41826E+00_JPRB, 0.42035E+00_JPRB, 0.42129E+00_JPRB, 0.42252E+00_JPRB, 0.42272E+00_JPRB, 0.42129E+00_JPRB, 0.41788E+00_JPRB ,&
&  0.41391E+00_JPRB, 0.40936E+00_JPRB, 0.40477E+00_JPRB, 0.39891E+00_JPRB, 0.39268E+00_JPRB, 0.38742E+00_JPRB, 0.38162E+00_JPRB ,&
&  0.37566E+00_JPRB, 0.37011E+00_JPRB, 0.36604E+00_JPRB, 0.36279E+00_JPRB, 0.35963E+00_JPRB, 0.35649E+00_JPRB, 0.35258E+00_JPRB ,&
&  0.34915E+00_JPRB, 0.34607E+00_JPRB, 0.34636E+00_JPRB, 0.34911E+00_JPRB, 0.35142E+00_JPRB, 0.35302E+00_JPRB, 0.35448E+00_JPRB ,&
&  0.35502E+00_JPRB, 0.36778E+00_JPRB, 0.36236E+00_JPRB, 0.37408E+00_JPRB, 0.37777E+00_JPRB, 0.37844E+00_JPRB, 0.37927E+00_JPRB ,&
&  0.38254E+00_JPRB, 0.38650E+00_JPRB, 0.38930E+00_JPRB, 0.38512E+00_JPRB, 0.37896E+00_JPRB, 0.37420E+00_JPRB, 0.36925E+00_JPRB ,&
&  0.36267E+00_JPRB, 0.35984E+00_JPRB, 0.35956E+00_JPRB, 0.35975E+00_JPRB, 0.35983E+00_JPRB, 0.35936E+00_JPRB, 0.35804E+00_JPRB ,&
&  0.35599E+00_JPRB, 0.35299E+00_JPRB, 0.34949E+00_JPRB, 0.34609E+00_JPRB, 0.34394E+00_JPRB, 0.34168E+00_JPRB, 0.33987E+00_JPRB ,&
&  0.33951E+00_JPRB, 0.34084E+00_JPRB, 0.34231E+00_JPRB, 0.34290E+00_JPRB, 0.34235E+00_JPRB, 0.34194E+00_JPRB, 0.34201E+00_JPRB ,&
&  0.34178E+00_JPRB, 0.34115E+00_JPRB, 0.34004E+00_JPRB, 0.33827E+00_JPRB, 0.33621E+00_JPRB, 0.33482E+00_JPRB, 0.33409E+00_JPRB ,&
&  0.33307E+00_JPRB, 0.33096E+00_JPRB, 0.32816E+00_JPRB, 0.32760E+00_JPRB, 0.32768E+00_JPRB, 0.32811E+00_JPRB, 0.32880E+00_JPRB ,&
&  0.33032E+00_JPRB, 0.33305E+00_JPRB, 0.33625E+00_JPRB, 0.33954E+00_JPRB, 0.34288E+00_JPRB, 0.34731E+00_JPRB, 0.35227E+00_JPRB ,&
&  0.35764E+00_JPRB, 0.36334E+00_JPRB, 0.36899E+00_JPRB, 0.37447E+00_JPRB, 0.37915E+00_JPRB, 0.38434E+00_JPRB, 0.39080E+00_JPRB ,&
&  0.39621E+00_JPRB, 0.40046E+00_JPRB, 0.40419E+00_JPRB, 0.40899E+00_JPRB, 0.41425E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.38104E+00_JPRB, 0.38629E+00_JPRB, 0.38945E+00_JPRB, 0.39123E+00_JPRB, 0.39198E+00_JPRB, 0.39068E+00_JPRB, 0.38794E+00_JPRB ,&
&  0.38350E+00_JPRB, 0.37891E+00_JPRB, 0.37398E+00_JPRB, 0.36860E+00_JPRB, 0.36372E+00_JPRB, 0.35948E+00_JPRB, 0.35564E+00_JPRB ,&
&  0.35168E+00_JPRB, 0.34800E+00_JPRB, 0.34496E+00_JPRB, 0.34192E+00_JPRB, 0.33838E+00_JPRB, 0.33392E+00_JPRB, 0.32835E+00_JPRB ,&
&  0.32486E+00_JPRB, 0.32210E+00_JPRB, 0.32284E+00_JPRB, 0.32592E+00_JPRB, 0.32839E+00_JPRB, 0.32981E+00_JPRB, 0.33108E+00_JPRB ,&
&  0.33285E+00_JPRB, 0.33618E+00_JPRB, 0.33892E+00_JPRB, 0.34281E+00_JPRB, 0.34587E+00_JPRB, 0.34565E+00_JPRB, 0.34360E+00_JPRB ,&
&  0.34219E+00_JPRB, 0.34302E+00_JPRB, 0.34460E+00_JPRB, 0.34542E+00_JPRB, 0.34425E+00_JPRB, 0.34116E+00_JPRB, 0.33625E+00_JPRB ,&
&  0.32836E+00_JPRB, 0.32262E+00_JPRB, 0.32138E+00_JPRB, 0.32270E+00_JPRB, 0.32484E+00_JPRB, 0.32649E+00_JPRB, 0.32745E+00_JPRB ,&
&  0.32772E+00_JPRB, 0.32698E+00_JPRB, 0.32563E+00_JPRB, 0.32396E+00_JPRB, 0.32183E+00_JPRB, 0.31955E+00_JPRB, 0.31820E+00_JPRB ,&
&  0.31683E+00_JPRB, 0.31578E+00_JPRB, 0.31536E+00_JPRB, 0.31517E+00_JPRB, 0.31408E+00_JPRB, 0.31224E+00_JPRB, 0.31106E+00_JPRB ,&
&  0.30954E+00_JPRB, 0.30863E+00_JPRB, 0.30735E+00_JPRB, 0.30601E+00_JPRB, 0.30435E+00_JPRB, 0.30275E+00_JPRB, 0.30203E+00_JPRB ,&
&  0.30194E+00_JPRB, 0.30066E+00_JPRB, 0.29919E+00_JPRB, 0.29921E+00_JPRB, 0.30004E+00_JPRB, 0.30116E+00_JPRB, 0.30201E+00_JPRB ,&
&  0.30283E+00_JPRB, 0.30391E+00_JPRB, 0.30634E+00_JPRB, 0.30920E+00_JPRB, 0.31255E+00_JPRB, 0.31654E+00_JPRB, 0.32101E+00_JPRB ,&
&  0.32611E+00_JPRB, 0.33184E+00_JPRB, 0.33813E+00_JPRB, 0.34407E+00_JPRB, 0.34921E+00_JPRB, 0.35279E+00_JPRB, 0.35720E+00_JPRB ,&
&  0.36235E+00_JPRB, 0.36580E+00_JPRB, 0.36864E+00_JPRB, 0.37204E+00_JPRB, 0.37587E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.34510E+00_JPRB, 0.35051E+00_JPRB, 0.35467E+00_JPRB, 0.35717E+00_JPRB, 0.35757E+00_JPRB, 0.35645E+00_JPRB, 0.35368E+00_JPRB ,&
&  0.34943E+00_JPRB, 0.34502E+00_JPRB, 0.34070E+00_JPRB, 0.33721E+00_JPRB, 0.33494E+00_JPRB, 0.33301E+00_JPRB, 0.33127E+00_JPRB ,&
&  0.32890E+00_JPRB, 0.32615E+00_JPRB, 0.32293E+00_JPRB, 0.32011E+00_JPRB, 0.31664E+00_JPRB, 0.31193E+00_JPRB, 0.30832E+00_JPRB ,&
&  0.30574E+00_JPRB, 0.30338E+00_JPRB, 0.30258E+00_JPRB, 0.30386E+00_JPRB, 0.30635E+00_JPRB, 0.30877E+00_JPRB, 0.31148E+00_JPRB ,&
&  0.31342E+00_JPRB, 0.31499E+00_JPRB, 0.31634E+00_JPRB, 0.31638E+00_JPRB, 0.31629E+00_JPRB, 0.31594E+00_JPRB, 0.31527E+00_JPRB ,&
&  0.31452E+00_JPRB, 0.31381E+00_JPRB, 0.31257E+00_JPRB, 0.31230E+00_JPRB, 0.31197E+00_JPRB, 0.31050E+00_JPRB, 0.30797E+00_JPRB ,&
&  0.30397E+00_JPRB, 0.29859E+00_JPRB, 0.29446E+00_JPRB, 0.29254E+00_JPRB, 0.29207E+00_JPRB, 0.29250E+00_JPRB, 0.29348E+00_JPRB ,&
&  0.29469E+00_JPRB, 0.29640E+00_JPRB, 0.29802E+00_JPRB, 0.29882E+00_JPRB, 0.29867E+00_JPRB, 0.29847E+00_JPRB, 0.29716E+00_JPRB ,&
&  0.29589E+00_JPRB, 0.29505E+00_JPRB, 0.29451E+00_JPRB, 0.29390E+00_JPRB, 0.29168E+00_JPRB, 0.28887E+00_JPRB, 0.28807E+00_JPRB ,&
&  0.28633E+00_JPRB, 0.28520E+00_JPRB, 0.28359E+00_JPRB, 0.28261E+00_JPRB, 0.28147E+00_JPRB, 0.27993E+00_JPRB, 0.27836E+00_JPRB ,&
&  0.27719E+00_JPRB, 0.27657E+00_JPRB, 0.27628E+00_JPRB, 0.27559E+00_JPRB, 0.27621E+00_JPRB, 0.27726E+00_JPRB, 0.27888E+00_JPRB ,&
&  0.28012E+00_JPRB, 0.28126E+00_JPRB, 0.28298E+00_JPRB, 0.28495E+00_JPRB, 0.28792E+00_JPRB, 0.29138E+00_JPRB, 0.29532E+00_JPRB ,&
&  0.29874E+00_JPRB, 0.30317E+00_JPRB, 0.30812E+00_JPRB, 0.31279E+00_JPRB, 0.31654E+00_JPRB, 0.31927E+00_JPRB, 0.32299E+00_JPRB ,&
&  0.32873E+00_JPRB, 0.33320E+00_JPRB, 0.33531E+00_JPRB, 0.33791E+00_JPRB, 0.34060E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.31767E+00_JPRB, 0.31965E+00_JPRB, 0.32164E+00_JPRB, 0.32379E+00_JPRB, 0.32533E+00_JPRB, 0.32596E+00_JPRB, 0.32462E+00_JPRB ,&
&  0.32188E+00_JPRB, 0.31863E+00_JPRB, 0.31532E+00_JPRB, 0.31251E+00_JPRB, 0.31080E+00_JPRB, 0.30970E+00_JPRB, 0.30888E+00_JPRB ,&
&  0.30813E+00_JPRB, 0.30733E+00_JPRB, 0.30510E+00_JPRB, 0.30192E+00_JPRB, 0.29774E+00_JPRB, 0.29408E+00_JPRB, 0.29278E+00_JPRB ,&
&  0.29092E+00_JPRB, 0.28960E+00_JPRB, 0.28855E+00_JPRB, 0.28750E+00_JPRB, 0.28701E+00_JPRB, 0.28791E+00_JPRB, 0.28982E+00_JPRB ,&
&  0.29179E+00_JPRB, 0.29354E+00_JPRB, 0.29498E+00_JPRB, 0.29575E+00_JPRB, 0.29513E+00_JPRB, 0.29412E+00_JPRB, 0.29285E+00_JPRB ,&
&  0.29124E+00_JPRB, 0.28960E+00_JPRB, 0.28810E+00_JPRB, 0.28614E+00_JPRB, 0.28333E+00_JPRB, 0.27993E+00_JPRB, 0.27748E+00_JPRB ,&
&  0.27563E+00_JPRB, 0.27389E+00_JPRB, 0.27213E+00_JPRB, 0.27122E+00_JPRB, 0.27053E+00_JPRB, 0.27023E+00_JPRB, 0.27030E+00_JPRB ,&
&  0.27078E+00_JPRB, 0.27212E+00_JPRB, 0.27403E+00_JPRB, 0.27615E+00_JPRB, 0.27690E+00_JPRB, 0.27715E+00_JPRB, 0.27737E+00_JPRB ,&
&  0.27658E+00_JPRB, 0.27510E+00_JPRB, 0.27312E+00_JPRB, 0.27087E+00_JPRB, 0.26929E+00_JPRB, 0.26812E+00_JPRB, 0.26682E+00_JPRB ,&
&  0.26467E+00_JPRB, 0.26332E+00_JPRB, 0.26264E+00_JPRB, 0.26249E+00_JPRB, 0.26265E+00_JPRB, 0.26261E+00_JPRB, 0.26177E+00_JPRB ,&
&  0.26077E+00_JPRB, 0.25996E+00_JPRB, 0.25905E+00_JPRB, 0.25816E+00_JPRB, 0.25653E+00_JPRB, 0.25726E+00_JPRB, 0.25850E+00_JPRB ,&
&  0.25892E+00_JPRB, 0.25893E+00_JPRB, 0.25917E+00_JPRB, 0.26071E+00_JPRB, 0.26248E+00_JPRB, 0.26472E+00_JPRB, 0.26760E+00_JPRB ,&
&  0.27011E+00_JPRB, 0.27343E+00_JPRB, 0.27809E+00_JPRB, 0.28248E+00_JPRB, 0.28570E+00_JPRB, 0.28734E+00_JPRB, 0.29066E+00_JPRB ,&
&  0.29739E+00_JPRB, 0.30419E+00_JPRB, 0.30824E+00_JPRB, 0.31219E+00_JPRB, 0.31590E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.29234E+00_JPRB, 0.29334E+00_JPRB, 0.29422E+00_JPRB, 0.29537E+00_JPRB, 0.29699E+00_JPRB, 0.29804E+00_JPRB, 0.29888E+00_JPRB ,&
&  0.29827E+00_JPRB, 0.29641E+00_JPRB, 0.29419E+00_JPRB, 0.29184E+00_JPRB, 0.29075E+00_JPRB, 0.29053E+00_JPRB, 0.29057E+00_JPRB ,&
&  0.29001E+00_JPRB, 0.28973E+00_JPRB, 0.28775E+00_JPRB, 0.28444E+00_JPRB, 0.28047E+00_JPRB, 0.27764E+00_JPRB, 0.27675E+00_JPRB ,&
&  0.27608E+00_JPRB, 0.27501E+00_JPRB, 0.27435E+00_JPRB, 0.27344E+00_JPRB, 0.27070E+00_JPRB, 0.26932E+00_JPRB, 0.26967E+00_JPRB ,&
&  0.27045E+00_JPRB, 0.27196E+00_JPRB, 0.27205E+00_JPRB, 0.26983E+00_JPRB, 0.27202E+00_JPRB, 0.27184E+00_JPRB, 0.26991E+00_JPRB ,&
&  0.26741E+00_JPRB, 0.26481E+00_JPRB, 0.26245E+00_JPRB, 0.26004E+00_JPRB, 0.25754E+00_JPRB, 0.25561E+00_JPRB, 0.25433E+00_JPRB ,&
&  0.25400E+00_JPRB, 0.25428E+00_JPRB, 0.25488E+00_JPRB, 0.25588E+00_JPRB, 0.25654E+00_JPRB, 0.25690E+00_JPRB, 0.25698E+00_JPRB ,&
&  0.25714E+00_JPRB, 0.25740E+00_JPRB, 0.25784E+00_JPRB, 0.25811E+00_JPRB, 0.25839E+00_JPRB, 0.25857E+00_JPRB, 0.25829E+00_JPRB ,&
&  0.25696E+00_JPRB, 0.25485E+00_JPRB, 0.25212E+00_JPRB, 0.25095E+00_JPRB, 0.25076E+00_JPRB, 0.24888E+00_JPRB, 0.24617E+00_JPRB ,&
&  0.24375E+00_JPRB, 0.24344E+00_JPRB, 0.24493E+00_JPRB, 0.24635E+00_JPRB, 0.24749E+00_JPRB, 0.24788E+00_JPRB, 0.24759E+00_JPRB ,&
&  0.24654E+00_JPRB, 0.24403E+00_JPRB, 0.24127E+00_JPRB, 0.23908E+00_JPRB, 0.23774E+00_JPRB, 0.23774E+00_JPRB, 0.23880E+00_JPRB ,&
&  0.23902E+00_JPRB, 0.23926E+00_JPRB, 0.24055E+00_JPRB, 0.24178E+00_JPRB, 0.24278E+00_JPRB, 0.24387E+00_JPRB, 0.24552E+00_JPRB ,&
&  0.24773E+00_JPRB, 0.25144E+00_JPRB, 0.25600E+00_JPRB, 0.26119E+00_JPRB, 0.26482E+00_JPRB, 0.26424E+00_JPRB, 0.26554E+00_JPRB ,&
&  0.27112E+00_JPRB, 0.27743E+00_JPRB, 0.28234E+00_JPRB, 0.28609E+00_JPRB, 0.28961E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.27154E+00_JPRB, 0.27283E+00_JPRB, 0.27382E+00_JPRB, 0.27458E+00_JPRB, 0.27605E+00_JPRB, 0.27726E+00_JPRB, 0.27884E+00_JPRB ,&
&  0.27997E+00_JPRB, 0.27864E+00_JPRB, 0.27691E+00_JPRB, 0.27542E+00_JPRB, 0.27477E+00_JPRB, 0.27391E+00_JPRB, 0.27315E+00_JPRB ,&
&  0.27367E+00_JPRB, 0.27288E+00_JPRB, 0.27001E+00_JPRB, 0.26768E+00_JPRB, 0.26398E+00_JPRB, 0.26157E+00_JPRB, 0.26116E+00_JPRB ,&
&  0.26281E+00_JPRB, 0.26230E+00_JPRB, 0.26118E+00_JPRB, 0.25970E+00_JPRB, 0.25790E+00_JPRB, 0.25687E+00_JPRB, 0.25562E+00_JPRB ,&
&  0.25483E+00_JPRB, 0.25344E+00_JPRB, 0.24883E+00_JPRB, 0.24924E+00_JPRB, 0.24892E+00_JPRB, 0.24906E+00_JPRB, 0.24859E+00_JPRB ,&
&  0.24798E+00_JPRB, 0.24694E+00_JPRB, 0.24508E+00_JPRB, 0.24299E+00_JPRB, 0.24123E+00_JPRB, 0.24027E+00_JPRB, 0.24023E+00_JPRB ,&
&  0.24079E+00_JPRB, 0.24212E+00_JPRB, 0.24411E+00_JPRB, 0.24636E+00_JPRB, 0.24732E+00_JPRB, 0.24806E+00_JPRB, 0.24923E+00_JPRB ,&
&  0.24974E+00_JPRB, 0.24981E+00_JPRB, 0.24970E+00_JPRB, 0.24938E+00_JPRB, 0.24871E+00_JPRB, 0.24748E+00_JPRB, 0.24648E+00_JPRB ,&
&  0.24436E+00_JPRB, 0.24153E+00_JPRB, 0.23952E+00_JPRB, 0.23712E+00_JPRB, 0.23063E+00_JPRB, 0.22041E+00_JPRB, 0.21474E+00_JPRB ,&
&  0.21530E+00_JPRB, 0.22021E+00_JPRB, 0.22535E+00_JPRB, 0.22904E+00_JPRB, 0.23524E+00_JPRB, 0.23536E+00_JPRB, 0.23104E+00_JPRB ,&
&  0.22127E+00_JPRB, 0.21715E+00_JPRB, 0.21390E+00_JPRB, 0.20921E+00_JPRB, 0.20646E+00_JPRB, 0.20621E+00_JPRB, 0.20654E+00_JPRB ,&
&  0.20875E+00_JPRB, 0.21153E+00_JPRB, 0.21466E+00_JPRB, 0.21654E+00_JPRB, 0.21686E+00_JPRB, 0.21582E+00_JPRB, 0.21438E+00_JPRB ,&
&  0.21458E+00_JPRB, 0.21924E+00_JPRB, 0.22815E+00_JPRB, 0.23666E+00_JPRB, 0.24165E+00_JPRB, 0.24383E+00_JPRB, 0.24609E+00_JPRB ,&
&  0.25060E+00_JPRB, 0.25638E+00_JPRB, 0.26197E+00_JPRB, 0.26585E+00_JPRB, 0.26902E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.25058E+00_JPRB, 0.25188E+00_JPRB, 0.25311E+00_JPRB, 0.25424E+00_JPRB, 0.25544E+00_JPRB, 0.25682E+00_JPRB, 0.25805E+00_JPRB ,&
&  0.25952E+00_JPRB, 0.26046E+00_JPRB, 0.26064E+00_JPRB, 0.25949E+00_JPRB, 0.25862E+00_JPRB, 0.25825E+00_JPRB, 0.25851E+00_JPRB ,&
&  0.25626E+00_JPRB, 0.25315E+00_JPRB, 0.25174E+00_JPRB, 0.25134E+00_JPRB, 0.25055E+00_JPRB, 0.25022E+00_JPRB, 0.25107E+00_JPRB ,&
&  0.25199E+00_JPRB, 0.25125E+00_JPRB, 0.25060E+00_JPRB, 0.24897E+00_JPRB, 0.24768E+00_JPRB, 0.24668E+00_JPRB, 0.24538E+00_JPRB ,&
&  0.24113E+00_JPRB, 0.23088E+00_JPRB, 0.22477E+00_JPRB, 0.22905E+00_JPRB, 0.23527E+00_JPRB, 0.23439E+00_JPRB, 0.23459E+00_JPRB ,&
&  0.23430E+00_JPRB, 0.23301E+00_JPRB, 0.23153E+00_JPRB, 0.23017E+00_JPRB, 0.22973E+00_JPRB, 0.23024E+00_JPRB, 0.23145E+00_JPRB ,&
&  0.23296E+00_JPRB, 0.23586E+00_JPRB, 0.23879E+00_JPRB, 0.23994E+00_JPRB, 0.24117E+00_JPRB, 0.24184E+00_JPRB, 0.24200E+00_JPRB ,&
&  0.24122E+00_JPRB, 0.23971E+00_JPRB, 0.23838E+00_JPRB, 0.23704E+00_JPRB, 0.23573E+00_JPRB, 0.23400E+00_JPRB, 0.22926E+00_JPRB ,&
&  0.22401E+00_JPRB, 0.21838E+00_JPRB, 0.21161E+00_JPRB, 0.20482E+00_JPRB, 0.19900E+00_JPRB, 0.19478E+00_JPRB, 0.19110E+00_JPRB ,&
&  0.19086E+00_JPRB, 0.19550E+00_JPRB, 0.20431E+00_JPRB, 0.21432E+00_JPRB, 0.21776E+00_JPRB, 0.20891E+00_JPRB, 0.20309E+00_JPRB ,&
&  0.19702E+00_JPRB, 0.19201E+00_JPRB, 0.18901E+00_JPRB, 0.18744E+00_JPRB, 0.18629E+00_JPRB, 0.18623E+00_JPRB, 0.18692E+00_JPRB ,&
&  0.18851E+00_JPRB, 0.19017E+00_JPRB, 0.19170E+00_JPRB, 0.19275E+00_JPRB, 0.19320E+00_JPRB, 0.19308E+00_JPRB, 0.19268E+00_JPRB ,&
&  0.19346E+00_JPRB, 0.19629E+00_JPRB, 0.20086E+00_JPRB, 0.20609E+00_JPRB, 0.21075E+00_JPRB, 0.21533E+00_JPRB, 0.22140E+00_JPRB ,&
&  0.22895E+00_JPRB, 0.23439E+00_JPRB, 0.23963E+00_JPRB, 0.24523E+00_JPRB, 0.24857E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.23397E+00_JPRB, 0.23645E+00_JPRB, 0.23721E+00_JPRB, 0.23857E+00_JPRB, 0.23885E+00_JPRB, 0.24001E+00_JPRB, 0.24077E+00_JPRB ,&
&  0.24303E+00_JPRB, 0.24513E+00_JPRB, 0.24557E+00_JPRB, 0.24359E+00_JPRB, 0.24209E+00_JPRB, 0.23848E+00_JPRB, 0.23692E+00_JPRB ,&
&  0.23626E+00_JPRB, 0.23714E+00_JPRB, 0.23725E+00_JPRB, 0.23726E+00_JPRB, 0.23713E+00_JPRB, 0.23703E+00_JPRB, 0.23761E+00_JPRB ,&
&  0.23536E+00_JPRB, 0.23392E+00_JPRB, 0.23588E+00_JPRB, 0.23467E+00_JPRB, 0.23367E+00_JPRB, 0.23199E+00_JPRB, 0.23104E+00_JPRB ,&
&  0.22651E+00_JPRB, 0.22032E+00_JPRB, 0.21412E+00_JPRB, 0.21794E+00_JPRB, 0.22510E+00_JPRB, 0.22209E+00_JPRB, 0.22154E+00_JPRB ,&
&  0.22127E+00_JPRB, 0.22034E+00_JPRB, 0.22020E+00_JPRB, 0.21999E+00_JPRB, 0.22107E+00_JPRB, 0.22216E+00_JPRB, 0.22453E+00_JPRB ,&
&  0.22771E+00_JPRB, 0.23200E+00_JPRB, 0.23197E+00_JPRB, 0.22499E+00_JPRB, 0.22209E+00_JPRB, 0.22118E+00_JPRB, 0.21754E+00_JPRB ,&
&  0.21310E+00_JPRB, 0.21184E+00_JPRB, 0.21046E+00_JPRB, 0.20692E+00_JPRB, 0.20556E+00_JPRB, 0.20496E+00_JPRB, 0.20189E+00_JPRB ,&
&  0.19804E+00_JPRB, 0.19519E+00_JPRB, 0.19142E+00_JPRB, 0.18763E+00_JPRB, 0.18507E+00_JPRB, 0.18392E+00_JPRB, 0.18416E+00_JPRB ,&
&  0.18639E+00_JPRB, 0.18843E+00_JPRB, 0.19474E+00_JPRB, 0.19852E+00_JPRB, 0.19479E+00_JPRB, 0.19210E+00_JPRB, 0.18866E+00_JPRB ,&
&  0.18252E+00_JPRB, 0.17983E+00_JPRB, 0.17765E+00_JPRB, 0.17603E+00_JPRB, 0.17591E+00_JPRB, 0.17669E+00_JPRB, 0.17717E+00_JPRB ,&
&  0.17795E+00_JPRB, 0.17921E+00_JPRB, 0.17982E+00_JPRB, 0.18016E+00_JPRB, 0.18027E+00_JPRB, 0.18065E+00_JPRB, 0.18146E+00_JPRB ,&
&  0.18242E+00_JPRB, 0.18466E+00_JPRB, 0.18747E+00_JPRB, 0.19036E+00_JPRB, 0.19264E+00_JPRB, 0.19453E+00_JPRB, 0.19855E+00_JPRB ,&
&  0.20289E+00_JPRB, 0.21297E+00_JPRB, 0.22694E+00_JPRB, 0.23601E+00_JPRB, 0.23190E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.23074E+00_JPRB, 0.23233E+00_JPRB, 0.23199E+00_JPRB, 0.23215E+00_JPRB, 0.23159E+00_JPRB, 0.23235E+00_JPRB, 0.23117E+00_JPRB ,&
&  0.23068E+00_JPRB, 0.22898E+00_JPRB, 0.22747E+00_JPRB, 0.22401E+00_JPRB, 0.22212E+00_JPRB, 0.21583E+00_JPRB, 0.21391E+00_JPRB ,&
&  0.21209E+00_JPRB, 0.21274E+00_JPRB, 0.21282E+00_JPRB, 0.21327E+00_JPRB, 0.21394E+00_JPRB, 0.21545E+00_JPRB, 0.21509E+00_JPRB ,&
&  0.21374E+00_JPRB, 0.21166E+00_JPRB, 0.20918E+00_JPRB, 0.20693E+00_JPRB, 0.20729E+00_JPRB, 0.20679E+00_JPRB, 0.20562E+00_JPRB ,&
&  0.20462E+00_JPRB, 0.20527E+00_JPRB, 0.20571E+00_JPRB, 0.20714E+00_JPRB, 0.21207E+00_JPRB, 0.21004E+00_JPRB, 0.20907E+00_JPRB ,&
&  0.20929E+00_JPRB, 0.20966E+00_JPRB, 0.21073E+00_JPRB, 0.21166E+00_JPRB, 0.21456E+00_JPRB, 0.21417E+00_JPRB, 0.21752E+00_JPRB ,&
&  0.21587E+00_JPRB, 0.21486E+00_JPRB, 0.21306E+00_JPRB, 0.20997E+00_JPRB, 0.20537E+00_JPRB, 0.20245E+00_JPRB, 0.19971E+00_JPRB ,&
&  0.19662E+00_JPRB, 0.19404E+00_JPRB, 0.19068E+00_JPRB, 0.18949E+00_JPRB, 0.18767E+00_JPRB, 0.18606E+00_JPRB, 0.18462E+00_JPRB ,&
&  0.18362E+00_JPRB, 0.18218E+00_JPRB, 0.18099E+00_JPRB, 0.18020E+00_JPRB, 0.18003E+00_JPRB, 0.18022E+00_JPRB, 0.18145E+00_JPRB ,&
&  0.18371E+00_JPRB, 0.18546E+00_JPRB, 0.18686E+00_JPRB, 0.18832E+00_JPRB, 0.18675E+00_JPRB, 0.18389E+00_JPRB, 0.18113E+00_JPRB ,&
&  0.17704E+00_JPRB, 0.17490E+00_JPRB, 0.17280E+00_JPRB, 0.17154E+00_JPRB, 0.17118E+00_JPRB, 0.17103E+00_JPRB, 0.17123E+00_JPRB ,&
&  0.17145E+00_JPRB, 0.17196E+00_JPRB, 0.17233E+00_JPRB, 0.17289E+00_JPRB, 0.17363E+00_JPRB, 0.17437E+00_JPRB, 0.17548E+00_JPRB ,&
&  0.17715E+00_JPRB, 0.17924E+00_JPRB, 0.18183E+00_JPRB, 0.18469E+00_JPRB, 0.18844E+00_JPRB, 0.19328E+00_JPRB, 0.20022E+00_JPRB ,&
&  0.20692E+00_JPRB, 0.21699E+00_JPRB, 0.21781E+00_JPRB, 0.22557E+00_JPRB, 0.22966E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.22808E+00_JPRB, 0.23023E+00_JPRB, 0.23086E+00_JPRB, 0.23159E+00_JPRB, 0.23073E+00_JPRB, 0.22786E+00_JPRB, 0.22483E+00_JPRB ,&
&  0.21859E+00_JPRB, 0.21368E+00_JPRB, 0.20776E+00_JPRB, 0.20233E+00_JPRB, 0.19713E+00_JPRB, 0.19390E+00_JPRB, 0.19190E+00_JPRB ,&
&  0.19158E+00_JPRB, 0.19285E+00_JPRB, 0.19372E+00_JPRB, 0.19425E+00_JPRB, 0.19441E+00_JPRB, 0.19527E+00_JPRB, 0.19505E+00_JPRB ,&
&  0.19441E+00_JPRB, 0.19332E+00_JPRB, 0.19254E+00_JPRB, 0.19327E+00_JPRB, 0.19535E+00_JPRB, 0.19746E+00_JPRB, 0.19923E+00_JPRB ,&
&  0.20066E+00_JPRB, 0.19969E+00_JPRB, 0.19964E+00_JPRB, 0.19879E+00_JPRB, 0.19854E+00_JPRB, 0.19824E+00_JPRB, 0.19851E+00_JPRB ,&
&  0.19993E+00_JPRB, 0.20104E+00_JPRB, 0.20354E+00_JPRB, 0.20477E+00_JPRB, 0.20379E+00_JPRB, 0.20678E+00_JPRB, 0.20782E+00_JPRB ,&
&  0.20750E+00_JPRB, 0.20419E+00_JPRB, 0.20047E+00_JPRB, 0.19744E+00_JPRB, 0.19415E+00_JPRB, 0.19139E+00_JPRB, 0.18913E+00_JPRB ,&
&  0.18708E+00_JPRB, 0.18603E+00_JPRB, 0.18379E+00_JPRB, 0.18192E+00_JPRB, 0.18072E+00_JPRB, 0.17945E+00_JPRB, 0.17868E+00_JPRB ,&
&  0.17824E+00_JPRB, 0.17739E+00_JPRB, 0.17727E+00_JPRB, 0.17680E+00_JPRB, 0.17691E+00_JPRB, 0.17703E+00_JPRB, 0.17737E+00_JPRB ,&
&  0.17824E+00_JPRB, 0.17937E+00_JPRB, 0.17928E+00_JPRB, 0.17918E+00_JPRB, 0.17748E+00_JPRB, 0.17562E+00_JPRB, 0.17406E+00_JPRB ,&
&  0.17193E+00_JPRB, 0.17041E+00_JPRB, 0.16947E+00_JPRB, 0.16853E+00_JPRB, 0.16835E+00_JPRB, 0.16834E+00_JPRB, 0.16824E+00_JPRB ,&
&  0.16845E+00_JPRB, 0.16881E+00_JPRB, 0.16939E+00_JPRB, 0.17060E+00_JPRB, 0.17168E+00_JPRB, 0.17320E+00_JPRB, 0.17487E+00_JPRB ,&
&  0.17617E+00_JPRB, 0.17853E+00_JPRB, 0.18094E+00_JPRB, 0.18362E+00_JPRB, 0.18944E+00_JPRB, 0.19437E+00_JPRB, 0.20308E+00_JPRB ,&
&  0.20925E+00_JPRB, 0.21793E+00_JPRB, 0.21541E+00_JPRB, 0.22108E+00_JPRB, 0.22562E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.22665E+00_JPRB, 0.22790E+00_JPRB, 0.22747E+00_JPRB, 0.22757E+00_JPRB, 0.22600E+00_JPRB, 0.22376E+00_JPRB, 0.22041E+00_JPRB ,&
&  0.21724E+00_JPRB, 0.21117E+00_JPRB, 0.20675E+00_JPRB, 0.20179E+00_JPRB, 0.19763E+00_JPRB, 0.19392E+00_JPRB, 0.19127E+00_JPRB ,&
&  0.19000E+00_JPRB, 0.18943E+00_JPRB, 0.18907E+00_JPRB, 0.18806E+00_JPRB, 0.18622E+00_JPRB, 0.18502E+00_JPRB, 0.18350E+00_JPRB ,&
&  0.18240E+00_JPRB, 0.18237E+00_JPRB, 0.18333E+00_JPRB, 0.18527E+00_JPRB, 0.18876E+00_JPRB, 0.19108E+00_JPRB, 0.19324E+00_JPRB ,&
&  0.19507E+00_JPRB, 0.19093E+00_JPRB, 0.19090E+00_JPRB, 0.19144E+00_JPRB, 0.19158E+00_JPRB, 0.19115E+00_JPRB, 0.19053E+00_JPRB ,&
&  0.19442E+00_JPRB, 0.19548E+00_JPRB, 0.20131E+00_JPRB, 0.20096E+00_JPRB, 0.20005E+00_JPRB, 0.20192E+00_JPRB, 0.20089E+00_JPRB ,&
&  0.20107E+00_JPRB, 0.19776E+00_JPRB, 0.19531E+00_JPRB, 0.19152E+00_JPRB, 0.18765E+00_JPRB, 0.18568E+00_JPRB, 0.18342E+00_JPRB ,&
&  0.18270E+00_JPRB, 0.18253E+00_JPRB, 0.18093E+00_JPRB, 0.17992E+00_JPRB, 0.17802E+00_JPRB, 0.17588E+00_JPRB, 0.17458E+00_JPRB ,&
&  0.17326E+00_JPRB, 0.17222E+00_JPRB, 0.17168E+00_JPRB, 0.17113E+00_JPRB, 0.17088E+00_JPRB, 0.17074E+00_JPRB, 0.17059E+00_JPRB ,&
&  0.17061E+00_JPRB, 0.17054E+00_JPRB, 0.16995E+00_JPRB, 0.16965E+00_JPRB, 0.16853E+00_JPRB, 0.16776E+00_JPRB, 0.16712E+00_JPRB ,&
&  0.16653E+00_JPRB, 0.16633E+00_JPRB, 0.16621E+00_JPRB, 0.16628E+00_JPRB, 0.16644E+00_JPRB, 0.16700E+00_JPRB, 0.16764E+00_JPRB ,&
&  0.16826E+00_JPRB, 0.16934E+00_JPRB, 0.17017E+00_JPRB, 0.17137E+00_JPRB, 0.17281E+00_JPRB, 0.17403E+00_JPRB, 0.17597E+00_JPRB ,&
&  0.17809E+00_JPRB, 0.18054E+00_JPRB, 0.18408E+00_JPRB, 0.18709E+00_JPRB, 0.19087E+00_JPRB, 0.19591E+00_JPRB, 0.20125E+00_JPRB ,&
&  0.20701E+00_JPRB, 0.21313E+00_JPRB, 0.21350E+00_JPRB, 0.22024E+00_JPRB, 0.22441E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.21705E+00_JPRB, 0.21778E+00_JPRB, 0.21850E+00_JPRB, 0.21874E+00_JPRB, 0.21881E+00_JPRB, 0.21786E+00_JPRB, 0.21661E+00_JPRB ,&
&  0.21235E+00_JPRB, 0.20932E+00_JPRB, 0.20585E+00_JPRB, 0.20239E+00_JPRB, 0.19932E+00_JPRB, 0.19599E+00_JPRB, 0.19312E+00_JPRB ,&
&  0.19050E+00_JPRB, 0.18850E+00_JPRB, 0.18675E+00_JPRB, 0.18446E+00_JPRB, 0.18159E+00_JPRB, 0.17897E+00_JPRB, 0.17742E+00_JPRB ,&
&  0.17704E+00_JPRB, 0.17745E+00_JPRB, 0.17904E+00_JPRB, 0.18061E+00_JPRB, 0.18228E+00_JPRB, 0.18352E+00_JPRB, 0.18479E+00_JPRB ,&
&  0.18423E+00_JPRB, 0.18390E+00_JPRB, 0.18357E+00_JPRB, 0.18293E+00_JPRB, 0.18372E+00_JPRB, 0.18480E+00_JPRB, 0.18552E+00_JPRB ,&
&  0.18639E+00_JPRB, 0.18838E+00_JPRB, 0.18989E+00_JPRB, 0.19126E+00_JPRB, 0.19138E+00_JPRB, 0.19082E+00_JPRB, 0.18940E+00_JPRB ,&
&  0.18753E+00_JPRB, 0.18544E+00_JPRB, 0.18353E+00_JPRB, 0.18188E+00_JPRB, 0.18061E+00_JPRB, 0.17912E+00_JPRB, 0.17782E+00_JPRB ,&
&  0.17718E+00_JPRB, 0.17661E+00_JPRB, 0.17579E+00_JPRB, 0.17499E+00_JPRB, 0.17375E+00_JPRB, 0.17266E+00_JPRB, 0.17154E+00_JPRB ,&
&  0.17054E+00_JPRB, 0.16959E+00_JPRB, 0.16871E+00_JPRB, 0.16781E+00_JPRB, 0.16718E+00_JPRB, 0.16663E+00_JPRB, 0.16609E+00_JPRB ,&
&  0.16552E+00_JPRB, 0.16534E+00_JPRB, 0.16493E+00_JPRB, 0.16479E+00_JPRB, 0.16453E+00_JPRB, 0.16435E+00_JPRB, 0.16455E+00_JPRB ,&
&  0.16512E+00_JPRB, 0.16571E+00_JPRB, 0.16599E+00_JPRB, 0.16639E+00_JPRB, 0.16680E+00_JPRB, 0.16733E+00_JPRB, 0.16819E+00_JPRB ,&
&  0.16886E+00_JPRB, 0.16963E+00_JPRB, 0.17024E+00_JPRB, 0.17098E+00_JPRB, 0.17241E+00_JPRB, 0.17388E+00_JPRB, 0.17555E+00_JPRB ,&
&  0.17765E+00_JPRB, 0.17982E+00_JPRB, 0.18234E+00_JPRB, 0.18474E+00_JPRB, 0.18802E+00_JPRB, 0.18975E+00_JPRB, 0.19321E+00_JPRB ,&
&  0.19703E+00_JPRB, 0.19786E+00_JPRB, 0.21098E+00_JPRB, 0.21224E+00_JPRB, 0.21404E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.18889E+00_JPRB, 0.19088E+00_JPRB, 0.19211E+00_JPRB, 0.19352E+00_JPRB, 0.19128E+00_JPRB, 0.20349E+00_JPRB, 0.20128E+00_JPRB ,&
&  0.20003E+00_JPRB, 0.19563E+00_JPRB, 0.19550E+00_JPRB, 0.19384E+00_JPRB, 0.19172E+00_JPRB, 0.18933E+00_JPRB, 0.18712E+00_JPRB ,&
&  0.18505E+00_JPRB, 0.18274E+00_JPRB, 0.18113E+00_JPRB, 0.17987E+00_JPRB, 0.17872E+00_JPRB, 0.17767E+00_JPRB, 0.17687E+00_JPRB ,&
&  0.17690E+00_JPRB, 0.17754E+00_JPRB, 0.17874E+00_JPRB, 0.17859E+00_JPRB, 0.17821E+00_JPRB, 0.17773E+00_JPRB, 0.17746E+00_JPRB ,&
&  0.17682E+00_JPRB, 0.17636E+00_JPRB, 0.17593E+00_JPRB, 0.17532E+00_JPRB, 0.17618E+00_JPRB, 0.17709E+00_JPRB, 0.17800E+00_JPRB ,&
&  0.17884E+00_JPRB, 0.17967E+00_JPRB, 0.17977E+00_JPRB, 0.17970E+00_JPRB, 0.17933E+00_JPRB, 0.17885E+00_JPRB, 0.17803E+00_JPRB ,&
&  0.17726E+00_JPRB, 0.17641E+00_JPRB, 0.17537E+00_JPRB, 0.17468E+00_JPRB, 0.17406E+00_JPRB, 0.17350E+00_JPRB, 0.17300E+00_JPRB ,&
&  0.17273E+00_JPRB, 0.17256E+00_JPRB, 0.17235E+00_JPRB, 0.17200E+00_JPRB, 0.17160E+00_JPRB, 0.17117E+00_JPRB, 0.17081E+00_JPRB ,&
&  0.17052E+00_JPRB, 0.16997E+00_JPRB, 0.16948E+00_JPRB, 0.16896E+00_JPRB, 0.16857E+00_JPRB, 0.16803E+00_JPRB, 0.16758E+00_JPRB ,&
&  0.16715E+00_JPRB, 0.16683E+00_JPRB, 0.16671E+00_JPRB, 0.16662E+00_JPRB, 0.16655E+00_JPRB, 0.16658E+00_JPRB, 0.16670E+00_JPRB ,&
&  0.16688E+00_JPRB, 0.16709E+00_JPRB, 0.16740E+00_JPRB, 0.16763E+00_JPRB, 0.16786E+00_JPRB, 0.16805E+00_JPRB, 0.16850E+00_JPRB ,&
&  0.16905E+00_JPRB, 0.16961E+00_JPRB, 0.17012E+00_JPRB, 0.17027E+00_JPRB, 0.17111E+00_JPRB, 0.17202E+00_JPRB, 0.17302E+00_JPRB ,&
&  0.17408E+00_JPRB, 0.17544E+00_JPRB, 0.17693E+00_JPRB, 0.17857E+00_JPRB, 0.18053E+00_JPRB, 0.18172E+00_JPRB, 0.18310E+00_JPRB ,&
&  0.18500E+00_JPRB, 0.18816E+00_JPRB, 0.18911E+00_JPRB, 0.18936E+00_JPRB, 0.18934E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.17733E+00_JPRB, 0.17707E+00_JPRB, 0.17677E+00_JPRB, 0.17642E+00_JPRB, 0.17536E+00_JPRB, 0.17482E+00_JPRB, 0.17437E+00_JPRB ,&
&  0.17409E+00_JPRB, 0.17391E+00_JPRB, 0.17398E+00_JPRB, 0.17413E+00_JPRB, 0.17430E+00_JPRB, 0.17445E+00_JPRB, 0.17458E+00_JPRB ,&
&  0.17455E+00_JPRB, 0.17455E+00_JPRB, 0.17377E+00_JPRB, 0.17313E+00_JPRB, 0.17273E+00_JPRB, 0.17242E+00_JPRB, 0.17203E+00_JPRB ,&
&  0.17170E+00_JPRB, 0.17132E+00_JPRB, 0.17087E+00_JPRB, 0.17059E+00_JPRB, 0.17062E+00_JPRB, 0.17048E+00_JPRB, 0.17051E+00_JPRB ,&
&  0.17052E+00_JPRB, 0.17076E+00_JPRB, 0.17088E+00_JPRB, 0.17102E+00_JPRB, 0.17149E+00_JPRB, 0.17192E+00_JPRB, 0.17206E+00_JPRB ,&
&  0.17215E+00_JPRB, 0.17200E+00_JPRB, 0.17183E+00_JPRB, 0.17171E+00_JPRB, 0.17167E+00_JPRB, 0.17168E+00_JPRB, 0.17167E+00_JPRB ,&
&  0.17156E+00_JPRB, 0.17139E+00_JPRB, 0.17116E+00_JPRB, 0.17088E+00_JPRB, 0.17063E+00_JPRB, 0.17055E+00_JPRB, 0.17056E+00_JPRB ,&
&  0.17059E+00_JPRB, 0.17048E+00_JPRB, 0.17028E+00_JPRB, 0.16994E+00_JPRB, 0.16961E+00_JPRB, 0.16927E+00_JPRB, 0.16896E+00_JPRB ,&
&  0.16866E+00_JPRB, 0.16851E+00_JPRB, 0.16836E+00_JPRB, 0.16825E+00_JPRB, 0.16818E+00_JPRB, 0.16806E+00_JPRB, 0.16794E+00_JPRB ,&
&  0.16784E+00_JPRB, 0.16775E+00_JPRB, 0.16764E+00_JPRB, 0.16756E+00_JPRB, 0.16750E+00_JPRB, 0.16745E+00_JPRB, 0.16743E+00_JPRB ,&
&  0.16744E+00_JPRB, 0.16746E+00_JPRB, 0.16748E+00_JPRB, 0.16763E+00_JPRB, 0.16780E+00_JPRB, 0.16803E+00_JPRB, 0.16828E+00_JPRB ,&
&  0.16856E+00_JPRB, 0.16887E+00_JPRB, 0.16920E+00_JPRB, 0.16942E+00_JPRB, 0.16979E+00_JPRB, 0.17017E+00_JPRB, 0.17057E+00_JPRB ,&
&  0.17096E+00_JPRB, 0.17144E+00_JPRB, 0.17199E+00_JPRB, 0.17261E+00_JPRB, 0.17334E+00_JPRB, 0.17372E+00_JPRB, 0.17422E+00_JPRB ,&
&  0.17480E+00_JPRB, 0.17562E+00_JPRB, 0.17604E+00_JPRB, 0.17634E+00_JPRB, 0.17666E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB, 0.16990E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB, 0.60473E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.60951E+00_JPRB, 0.60924E+00_JPRB, 0.60895E+00_JPRB, 0.60873E+00_JPRB, 0.60851E+00_JPRB, 0.60840E+00_JPRB, 0.60823E+00_JPRB ,&
&  0.60827E+00_JPRB, 0.60828E+00_JPRB, 0.60827E+00_JPRB, 0.60816E+00_JPRB, 0.60819E+00_JPRB, 0.60815E+00_JPRB, 0.60807E+00_JPRB ,&
&  0.60789E+00_JPRB, 0.60781E+00_JPRB, 0.60746E+00_JPRB, 0.60709E+00_JPRB, 0.60659E+00_JPRB, 0.60624E+00_JPRB, 0.60569E+00_JPRB ,&
&  0.60529E+00_JPRB, 0.60463E+00_JPRB, 0.60401E+00_JPRB, 0.60294E+00_JPRB, 0.60216E+00_JPRB, 0.60131E+00_JPRB, 0.60056E+00_JPRB ,&
&  0.59991E+00_JPRB, 0.59950E+00_JPRB, 0.59909E+00_JPRB, 0.59854E+00_JPRB, 0.59810E+00_JPRB, 0.59819E+00_JPRB, 0.59840E+00_JPRB ,&
&  0.59862E+00_JPRB, 0.59893E+00_JPRB, 0.59937E+00_JPRB, 0.59953E+00_JPRB, 0.59943E+00_JPRB, 0.59946E+00_JPRB, 0.59941E+00_JPRB ,&
&  0.59923E+00_JPRB, 0.59893E+00_JPRB, 0.59861E+00_JPRB, 0.59852E+00_JPRB, 0.59844E+00_JPRB, 0.59818E+00_JPRB, 0.59799E+00_JPRB ,&
&  0.59783E+00_JPRB, 0.59764E+00_JPRB, 0.59737E+00_JPRB, 0.59696E+00_JPRB, 0.59666E+00_JPRB, 0.59635E+00_JPRB, 0.59601E+00_JPRB ,&
&  0.59563E+00_JPRB, 0.59517E+00_JPRB, 0.59471E+00_JPRB, 0.59421E+00_JPRB, 0.59366E+00_JPRB, 0.59324E+00_JPRB, 0.59295E+00_JPRB ,&
&  0.59276E+00_JPRB, 0.59274E+00_JPRB, 0.59282E+00_JPRB, 0.59294E+00_JPRB, 0.59316E+00_JPRB, 0.59353E+00_JPRB, 0.59403E+00_JPRB ,&
&  0.59463E+00_JPRB, 0.59532E+00_JPRB, 0.59622E+00_JPRB, 0.59731E+00_JPRB, 0.59836E+00_JPRB, 0.59944E+00_JPRB, 0.60063E+00_JPRB ,&
&  0.60170E+00_JPRB, 0.60253E+00_JPRB, 0.60314E+00_JPRB, 0.60377E+00_JPRB, 0.60474E+00_JPRB, 0.60551E+00_JPRB, 0.60612E+00_JPRB ,&
&  0.60666E+00_JPRB, 0.60728E+00_JPRB, 0.60785E+00_JPRB, 0.60848E+00_JPRB, 0.60874E+00_JPRB, 0.60902E+00_JPRB, 0.60927E+00_JPRB ,&
&  0.60951E+00_JPRB, 0.60962E+00_JPRB, 0.60968E+00_JPRB, 0.60971E+00_JPRB, 0.60965E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.60743E+00_JPRB, 0.60677E+00_JPRB, 0.60639E+00_JPRB, 0.60627E+00_JPRB, 0.60626E+00_JPRB, 0.60640E+00_JPRB, 0.60665E+00_JPRB ,&
&  0.60705E+00_JPRB, 0.60718E+00_JPRB, 0.60693E+00_JPRB, 0.60707E+00_JPRB, 0.60769E+00_JPRB, 0.60844E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.60821E+00_JPRB, 0.60804E+00_JPRB, 0.60805E+00_JPRB, 0.60759E+00_JPRB, 0.60681E+00_JPRB, 0.60570E+00_JPRB, 0.60466E+00_JPRB ,&
&  0.60382E+00_JPRB, 0.60264E+00_JPRB, 0.60129E+00_JPRB, 0.59861E+00_JPRB, 0.59632E+00_JPRB, 0.59404E+00_JPRB, 0.59183E+00_JPRB ,&
&  0.58882E+00_JPRB, 0.58623E+00_JPRB, 0.58421E+00_JPRB, 0.58252E+00_JPRB, 0.58504E+00_JPRB, 0.58667E+00_JPRB, 0.58759E+00_JPRB ,&
&  0.58765E+00_JPRB, 0.58495E+00_JPRB, 0.58411E+00_JPRB, 0.58365E+00_JPRB, 0.58339E+00_JPRB, 0.58563E+00_JPRB, 0.58715E+00_JPRB ,&
&  0.58839E+00_JPRB, 0.58942E+00_JPRB, 0.59188E+00_JPRB, 0.59308E+00_JPRB, 0.59391E+00_JPRB, 0.59439E+00_JPRB, 0.59428E+00_JPRB ,&
&  0.59476E+00_JPRB, 0.59525E+00_JPRB, 0.59566E+00_JPRB, 0.59551E+00_JPRB, 0.59530E+00_JPRB, 0.59506E+00_JPRB, 0.59446E+00_JPRB ,&
&  0.59340E+00_JPRB, 0.59239E+00_JPRB, 0.59131E+00_JPRB, 0.59015E+00_JPRB, 0.58880E+00_JPRB, 0.58789E+00_JPRB, 0.58709E+00_JPRB ,&
&  0.58633E+00_JPRB, 0.58577E+00_JPRB, 0.58541E+00_JPRB, 0.58515E+00_JPRB, 0.58504E+00_JPRB, 0.58539E+00_JPRB, 0.58635E+00_JPRB ,&
&  0.58782E+00_JPRB, 0.58971E+00_JPRB, 0.59191E+00_JPRB, 0.59425E+00_JPRB, 0.59629E+00_JPRB, 0.59811E+00_JPRB, 0.60036E+00_JPRB ,&
&  0.60277E+00_JPRB, 0.60466E+00_JPRB, 0.60621E+00_JPRB, 0.60756E+00_JPRB, 0.60929E+00_JPRB, 0.61029E+00_JPRB, 0.61078E+00_JPRB ,&
&  0.61110E+00_JPRB, 0.61190E+00_JPRB, 0.61276E+00_JPRB, 0.61374E+00_JPRB, 0.61413E+00_JPRB, 0.61396E+00_JPRB, 0.61341E+00_JPRB ,&
&  0.61268E+00_JPRB, 0.61195E+00_JPRB, 0.61091E+00_JPRB, 0.60966E+00_JPRB, 0.60847E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.60373E+00_JPRB, 0.60481E+00_JPRB, 0.60647E+00_JPRB, 0.60848E+00_JPRB, 0.61085E+00_JPRB, 0.61251E+00_JPRB, 0.61435E+00_JPRB ,&
&  0.61559E+00_JPRB, 0.61628E+00_JPRB, 0.61610E+00_JPRB, 0.61677E+00_JPRB, 0.61698E+00_JPRB, 0.61746E+00_JPRB, 0.61682E+00_JPRB ,&
&  0.61669E+00_JPRB, 0.61574E+00_JPRB, 0.61646E+00_JPRB, 0.61675E+00_JPRB, 0.61682E+00_JPRB, 0.61692E+00_JPRB, 0.61230E+00_JPRB ,&
&  0.60796E+00_JPRB, 0.60488E+00_JPRB, 0.59994E+00_JPRB, 0.59521E+00_JPRB, 0.59199E+00_JPRB, 0.57070E+00_JPRB, 0.56424E+00_JPRB ,&
&  0.57303E+00_JPRB, 0.57870E+00_JPRB, 0.58360E+00_JPRB, 0.58705E+00_JPRB, 0.57649E+00_JPRB, 0.56374E+00_JPRB, 0.55616E+00_JPRB ,&
&  0.53667E+00_JPRB, 0.54188E+00_JPRB, 0.54559E+00_JPRB, 0.54859E+00_JPRB, 0.55380E+00_JPRB, 0.55556E+00_JPRB, 0.56078E+00_JPRB ,&
&  0.57358E+00_JPRB, 0.57702E+00_JPRB, 0.58179E+00_JPRB, 0.58575E+00_JPRB, 0.58813E+00_JPRB, 0.59103E+00_JPRB, 0.59705E+00_JPRB ,&
&  0.59979E+00_JPRB, 0.60232E+00_JPRB, 0.60401E+00_JPRB, 0.60396E+00_JPRB, 0.60426E+00_JPRB, 0.60362E+00_JPRB, 0.60274E+00_JPRB ,&
&  0.60027E+00_JPRB, 0.59900E+00_JPRB, 0.59801E+00_JPRB, 0.59683E+00_JPRB, 0.59602E+00_JPRB, 0.59344E+00_JPRB, 0.59155E+00_JPRB ,&
&  0.59037E+00_JPRB, 0.58983E+00_JPRB, 0.58974E+00_JPRB, 0.59041E+00_JPRB, 0.59204E+00_JPRB, 0.59426E+00_JPRB, 0.59639E+00_JPRB ,&
&  0.59919E+00_JPRB, 0.60170E+00_JPRB, 0.60471E+00_JPRB, 0.60772E+00_JPRB, 0.61011E+00_JPRB, 0.61205E+00_JPRB, 0.61391E+00_JPRB ,&
&  0.61631E+00_JPRB, 0.61894E+00_JPRB, 0.62033E+00_JPRB, 0.62016E+00_JPRB, 0.62167E+00_JPRB, 0.62282E+00_JPRB, 0.62316E+00_JPRB ,&
&  0.62320E+00_JPRB, 0.62355E+00_JPRB, 0.62261E+00_JPRB, 0.62092E+00_JPRB, 0.61873E+00_JPRB, 0.61665E+00_JPRB, 0.61439E+00_JPRB ,&
&  0.61197E+00_JPRB, 0.60965E+00_JPRB, 0.60741E+00_JPRB, 0.60528E+00_JPRB, 0.60392E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.60815E+00_JPRB, 0.61182E+00_JPRB, 0.61655E+00_JPRB, 0.61934E+00_JPRB, 0.62291E+00_JPRB, 0.62428E+00_JPRB, 0.62642E+00_JPRB ,&
&  0.62701E+00_JPRB, 0.62804E+00_JPRB, 0.62871E+00_JPRB, 0.62889E+00_JPRB, 0.62873E+00_JPRB, 0.62794E+00_JPRB, 0.62729E+00_JPRB ,&
&  0.62709E+00_JPRB, 0.62645E+00_JPRB, 0.62707E+00_JPRB, 0.62726E+00_JPRB, 0.62713E+00_JPRB, 0.62685E+00_JPRB, 0.62159E+00_JPRB ,&
&  0.61552E+00_JPRB, 0.60986E+00_JPRB, 0.58307E+00_JPRB, 0.58139E+00_JPRB, 0.57737E+00_JPRB, 0.57531E+00_JPRB, 0.57682E+00_JPRB ,&
&  0.57359E+00_JPRB, 0.56966E+00_JPRB, 0.55553E+00_JPRB, 0.54157E+00_JPRB, 0.53123E+00_JPRB, 0.52002E+00_JPRB, 0.51944E+00_JPRB ,&
&  0.51833E+00_JPRB, 0.52213E+00_JPRB, 0.52583E+00_JPRB, 0.52823E+00_JPRB, 0.53560E+00_JPRB, 0.53989E+00_JPRB, 0.54835E+00_JPRB ,&
&  0.55900E+00_JPRB, 0.56620E+00_JPRB, 0.56578E+00_JPRB, 0.56411E+00_JPRB, 0.57912E+00_JPRB, 0.58929E+00_JPRB, 0.59851E+00_JPRB ,&
&  0.60432E+00_JPRB, 0.60565E+00_JPRB, 0.60871E+00_JPRB, 0.60896E+00_JPRB, 0.61002E+00_JPRB, 0.61278E+00_JPRB, 0.61359E+00_JPRB ,&
&  0.61523E+00_JPRB, 0.61457E+00_JPRB, 0.61255E+00_JPRB, 0.60969E+00_JPRB, 0.60486E+00_JPRB, 0.60175E+00_JPRB, 0.59991E+00_JPRB ,&
&  0.60008E+00_JPRB, 0.60215E+00_JPRB, 0.60500E+00_JPRB, 0.60906E+00_JPRB, 0.61327E+00_JPRB, 0.61646E+00_JPRB, 0.61836E+00_JPRB ,&
&  0.61965E+00_JPRB, 0.62018E+00_JPRB, 0.62394E+00_JPRB, 0.62418E+00_JPRB, 0.62611E+00_JPRB, 0.63203E+00_JPRB, 0.63155E+00_JPRB ,&
&  0.63287E+00_JPRB, 0.63637E+00_JPRB, 0.63861E+00_JPRB, 0.64047E+00_JPRB, 0.64172E+00_JPRB, 0.64215E+00_JPRB, 0.64281E+00_JPRB ,&
&  0.64274E+00_JPRB, 0.64145E+00_JPRB, 0.63857E+00_JPRB, 0.63464E+00_JPRB, 0.63041E+00_JPRB, 0.62609E+00_JPRB, 0.62053E+00_JPRB ,&
&  0.61518E+00_JPRB, 0.61004E+00_JPRB, 0.60657E+00_JPRB, 0.60440E+00_JPRB, 0.60552E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.62104E+00_JPRB, 0.62485E+00_JPRB, 0.63060E+00_JPRB, 0.63259E+00_JPRB, 0.63585E+00_JPRB, 0.63700E+00_JPRB, 0.63817E+00_JPRB ,&
&  0.63869E+00_JPRB, 0.63927E+00_JPRB, 0.63908E+00_JPRB, 0.63907E+00_JPRB, 0.63853E+00_JPRB, 0.63826E+00_JPRB, 0.63769E+00_JPRB ,&
&  0.63701E+00_JPRB, 0.63621E+00_JPRB, 0.63566E+00_JPRB, 0.63506E+00_JPRB, 0.63529E+00_JPRB, 0.63530E+00_JPRB, 0.63265E+00_JPRB ,&
&  0.62792E+00_JPRB, 0.61767E+00_JPRB, 0.59726E+00_JPRB, 0.59392E+00_JPRB, 0.59055E+00_JPRB, 0.59132E+00_JPRB, 0.58999E+00_JPRB ,&
&  0.58306E+00_JPRB, 0.54399E+00_JPRB, 0.54689E+00_JPRB, 0.53819E+00_JPRB, 0.53472E+00_JPRB, 0.53102E+00_JPRB, 0.52082E+00_JPRB ,&
&  0.51729E+00_JPRB, 0.51328E+00_JPRB, 0.51258E+00_JPRB, 0.51807E+00_JPRB, 0.52331E+00_JPRB, 0.53140E+00_JPRB, 0.54111E+00_JPRB ,&
&  0.54043E+00_JPRB, 0.55506E+00_JPRB, 0.56404E+00_JPRB, 0.56969E+00_JPRB, 0.58466E+00_JPRB, 0.59704E+00_JPRB, 0.60034E+00_JPRB ,&
&  0.60434E+00_JPRB, 0.60340E+00_JPRB, 0.60656E+00_JPRB, 0.61286E+00_JPRB, 0.61979E+00_JPRB, 0.62001E+00_JPRB, 0.62076E+00_JPRB ,&
&  0.62045E+00_JPRB, 0.62003E+00_JPRB, 0.61939E+00_JPRB, 0.61795E+00_JPRB, 0.61733E+00_JPRB, 0.61919E+00_JPRB, 0.62199E+00_JPRB ,&
&  0.62742E+00_JPRB, 0.63286E+00_JPRB, 0.63883E+00_JPRB, 0.64414E+00_JPRB, 0.64834E+00_JPRB, 0.65208E+00_JPRB, 0.65570E+00_JPRB ,&
&  0.65931E+00_JPRB, 0.66180E+00_JPRB, 0.66198E+00_JPRB, 0.66202E+00_JPRB, 0.66059E+00_JPRB, 0.65774E+00_JPRB, 0.65762E+00_JPRB ,&
&  0.66022E+00_JPRB, 0.66164E+00_JPRB, 0.66219E+00_JPRB, 0.66285E+00_JPRB, 0.66436E+00_JPRB, 0.66489E+00_JPRB, 0.66494E+00_JPRB ,&
&  0.66432E+00_JPRB, 0.66234E+00_JPRB, 0.65912E+00_JPRB, 0.65237E+00_JPRB, 0.64502E+00_JPRB, 0.63998E+00_JPRB, 0.63196E+00_JPRB ,&
&  0.62436E+00_JPRB, 0.61647E+00_JPRB, 0.61328E+00_JPRB, 0.61169E+00_JPRB, 0.61567E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.63813E+00_JPRB, 0.64388E+00_JPRB, 0.65028E+00_JPRB, 0.65352E+00_JPRB, 0.65680E+00_JPRB, 0.65743E+00_JPRB, 0.65880E+00_JPRB ,&
&  0.65946E+00_JPRB, 0.66057E+00_JPRB, 0.66087E+00_JPRB, 0.66090E+00_JPRB, 0.66012E+00_JPRB, 0.65911E+00_JPRB, 0.65741E+00_JPRB ,&
&  0.65465E+00_JPRB, 0.65199E+00_JPRB, 0.64808E+00_JPRB, 0.64492E+00_JPRB, 0.64327E+00_JPRB, 0.64393E+00_JPRB, 0.64237E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.62863E+00_JPRB, 0.61295E+00_JPRB, 0.60658E+00_JPRB, 0.60601E+00_JPRB, 0.59712E+00_JPRB, 0.59517E+00_JPRB ,&
&  0.59209E+00_JPRB, 0.59083E+00_JPRB, 0.58359E+00_JPRB, 0.57927E+00_JPRB, 0.56722E+00_JPRB, 0.51682E+00_JPRB, 0.51305E+00_JPRB ,&
&  0.51009E+00_JPRB, 0.50349E+00_JPRB, 0.50288E+00_JPRB, 0.50466E+00_JPRB, 0.51030E+00_JPRB, 0.52883E+00_JPRB, 0.54481E+00_JPRB ,&
&  0.54340E+00_JPRB, 0.56065E+00_JPRB, 0.56777E+00_JPRB, 0.57485E+00_JPRB, 0.58775E+00_JPRB, 0.59940E+00_JPRB, 0.60156E+00_JPRB ,&
&  0.60548E+00_JPRB, 0.60921E+00_JPRB, 0.61640E+00_JPRB, 0.62384E+00_JPRB, 0.62941E+00_JPRB, 0.63245E+00_JPRB, 0.63648E+00_JPRB ,&
&  0.64098E+00_JPRB, 0.64485E+00_JPRB, 0.64969E+00_JPRB, 0.65167E+00_JPRB, 0.65208E+00_JPRB, 0.65246E+00_JPRB, 0.65494E+00_JPRB ,&
&  0.66123E+00_JPRB, 0.66686E+00_JPRB, 0.67613E+00_JPRB, 0.67595E+00_JPRB, 0.67872E+00_JPRB, 0.68072E+00_JPRB, 0.68672E+00_JPRB ,&
&  0.69052E+00_JPRB, 0.70201E+00_JPRB, 0.71049E+00_JPRB, 0.71468E+00_JPRB, 0.70723E+00_JPRB, 0.69975E+00_JPRB, 0.69442E+00_JPRB ,&
&  0.69199E+00_JPRB, 0.69226E+00_JPRB, 0.68637E+00_JPRB, 0.68453E+00_JPRB, 0.68431E+00_JPRB, 0.68476E+00_JPRB, 0.68530E+00_JPRB ,&
&  0.68458E+00_JPRB, 0.68076E+00_JPRB, 0.67464E+00_JPRB, 0.66576E+00_JPRB, 0.65788E+00_JPRB, 0.65064E+00_JPRB, 0.63759E+00_JPRB ,&
&  0.62764E+00_JPRB, 0.61936E+00_JPRB, 0.61857E+00_JPRB, 0.62083E+00_JPRB, 0.62867E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.65426E+00_JPRB, 0.66347E+00_JPRB, 0.67250E+00_JPRB, 0.67997E+00_JPRB, 0.68302E+00_JPRB, 0.68312E+00_JPRB, 0.68094E+00_JPRB ,&
&  0.67850E+00_JPRB, 0.67759E+00_JPRB, 0.67642E+00_JPRB, 0.67560E+00_JPRB, 0.67464E+00_JPRB, 0.67258E+00_JPRB, 0.66981E+00_JPRB ,&
&  0.66766E+00_JPRB, 0.65497E+00_JPRB, 0.64755E+00_JPRB, 0.64627E+00_JPRB, 0.64459E+00_JPRB, 0.64454E+00_JPRB, 0.64498E+00_JPRB ,&
&  0.64002E+00_JPRB, 0.63174E+00_JPRB, 0.62512E+00_JPRB, 0.61594E+00_JPRB, 0.60677E+00_JPRB, 0.60961E+00_JPRB, 0.60830E+00_JPRB ,&
&  0.59263E+00_JPRB, 0.58051E+00_JPRB, 0.58574E+00_JPRB, 0.58723E+00_JPRB, 0.56466E+00_JPRB, 0.54000E+00_JPRB, 0.52098E+00_JPRB ,&
&  0.52215E+00_JPRB, 0.51054E+00_JPRB, 0.50858E+00_JPRB, 0.51028E+00_JPRB, 0.51846E+00_JPRB, 0.52733E+00_JPRB, 0.54389E+00_JPRB ,&
&  0.54904E+00_JPRB, 0.55339E+00_JPRB, 0.56756E+00_JPRB, 0.58236E+00_JPRB, 0.59210E+00_JPRB, 0.60215E+00_JPRB, 0.60798E+00_JPRB ,&
&  0.61765E+00_JPRB, 0.62750E+00_JPRB, 0.63628E+00_JPRB, 0.64405E+00_JPRB, 0.65275E+00_JPRB, 0.65601E+00_JPRB, 0.66134E+00_JPRB ,&
&  0.69370E+00_JPRB, 0.71917E+00_JPRB, 0.73131E+00_JPRB, 0.73986E+00_JPRB, 0.74110E+00_JPRB, 0.73795E+00_JPRB, 0.72607E+00_JPRB ,&
&  0.72116E+00_JPRB, 0.72449E+00_JPRB, 0.72622E+00_JPRB, 0.72988E+00_JPRB, 0.73192E+00_JPRB, 0.73047E+00_JPRB, 0.72419E+00_JPRB ,&
&  0.71365E+00_JPRB, 0.71864E+00_JPRB, 0.79232E+00_JPRB, 0.80738E+00_JPRB, 0.79562E+00_JPRB, 0.77219E+00_JPRB, 0.74663E+00_JPRB ,&
&  0.72800E+00_JPRB, 0.71646E+00_JPRB, 0.70971E+00_JPRB, 0.70457E+00_JPRB, 0.70079E+00_JPRB, 0.69516E+00_JPRB, 0.69539E+00_JPRB ,&
&  0.69238E+00_JPRB, 0.68570E+00_JPRB, 0.67831E+00_JPRB, 0.66979E+00_JPRB, 0.66159E+00_JPRB, 0.65445E+00_JPRB, 0.64411E+00_JPRB ,&
&  0.63535E+00_JPRB, 0.62838E+00_JPRB, 0.62946E+00_JPRB, 0.63499E+00_JPRB, 0.64485E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.67161E+00_JPRB, 0.68295E+00_JPRB, 0.69516E+00_JPRB, 0.70392E+00_JPRB, 0.70437E+00_JPRB, 0.69096E+00_JPRB, 0.67903E+00_JPRB ,&
&  0.67131E+00_JPRB, 0.66823E+00_JPRB, 0.66655E+00_JPRB, 0.66401E+00_JPRB, 0.66257E+00_JPRB, 0.66117E+00_JPRB, 0.66019E+00_JPRB ,&
&  0.65970E+00_JPRB, 0.65707E+00_JPRB, 0.65075E+00_JPRB, 0.64230E+00_JPRB, 0.63758E+00_JPRB, 0.63704E+00_JPRB, 0.63820E+00_JPRB ,&
&  0.63670E+00_JPRB, 0.63367E+00_JPRB, 0.63305E+00_JPRB, 0.62271E+00_JPRB, 0.61295E+00_JPRB, 0.59939E+00_JPRB, 0.59130E+00_JPRB ,&
&  0.59220E+00_JPRB, 0.59369E+00_JPRB, 0.58205E+00_JPRB, 0.58204E+00_JPRB, 0.58433E+00_JPRB, 0.56075E+00_JPRB, 0.53917E+00_JPRB ,&
&  0.52730E+00_JPRB, 0.51903E+00_JPRB, 0.51537E+00_JPRB, 0.51698E+00_JPRB, 0.53637E+00_JPRB, 0.56414E+00_JPRB, 0.57199E+00_JPRB ,&
&  0.54617E+00_JPRB, 0.56142E+00_JPRB, 0.58140E+00_JPRB, 0.59289E+00_JPRB, 0.60300E+00_JPRB, 0.61688E+00_JPRB, 0.63084E+00_JPRB ,&
&  0.64305E+00_JPRB, 0.65318E+00_JPRB, 0.65910E+00_JPRB, 0.67245E+00_JPRB, 0.67823E+00_JPRB, 0.67294E+00_JPRB, 0.68499E+00_JPRB ,&
&  0.77018E+00_JPRB, 0.83947E+00_JPRB, 0.86442E+00_JPRB, 0.86757E+00_JPRB, 0.85987E+00_JPRB, 0.84265E+00_JPRB, 0.83015E+00_JPRB ,&
&  0.81889E+00_JPRB, 0.80815E+00_JPRB, 0.79927E+00_JPRB, 0.79193E+00_JPRB, 0.78496E+00_JPRB, 0.77594E+00_JPRB, 0.77023E+00_JPRB ,&
&  0.76438E+00_JPRB, 0.78158E+00_JPRB, 0.98670E+00_JPRB, 0.96295E+00_JPRB, 0.90775E+00_JPRB, 0.86766E+00_JPRB, 0.82683E+00_JPRB ,&
&  0.79620E+00_JPRB, 0.77559E+00_JPRB, 0.76318E+00_JPRB, 0.74944E+00_JPRB, 0.72043E+00_JPRB, 0.69871E+00_JPRB, 0.68836E+00_JPRB ,&
&  0.68289E+00_JPRB, 0.67372E+00_JPRB, 0.66589E+00_JPRB, 0.65888E+00_JPRB, 0.65312E+00_JPRB, 0.65013E+00_JPRB, 0.64504E+00_JPRB ,&
&  0.64076E+00_JPRB, 0.64239E+00_JPRB, 0.64736E+00_JPRB, 0.65648E+00_JPRB, 0.66418E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.68499E+00_JPRB, 0.70902E+00_JPRB, 0.72623E+00_JPRB, 0.73193E+00_JPRB, 0.72697E+00_JPRB, 0.69237E+00_JPRB, 0.66138E+00_JPRB ,&
&  0.64354E+00_JPRB, 0.63755E+00_JPRB, 0.63791E+00_JPRB, 0.63843E+00_JPRB, 0.64011E+00_JPRB, 0.64254E+00_JPRB, 0.64356E+00_JPRB ,&
&  0.64237E+00_JPRB, 0.64183E+00_JPRB, 0.64148E+00_JPRB, 0.63867E+00_JPRB, 0.63409E+00_JPRB, 0.63086E+00_JPRB, 0.63054E+00_JPRB ,&
&  0.63003E+00_JPRB, 0.62756E+00_JPRB, 0.62543E+00_JPRB, 0.61994E+00_JPRB, 0.61699E+00_JPRB, 0.61199E+00_JPRB, 0.60291E+00_JPRB ,&
&  0.59034E+00_JPRB, 0.59060E+00_JPRB, 0.59829E+00_JPRB, 0.59838E+00_JPRB, 0.60015E+00_JPRB, 0.58471E+00_JPRB, 0.55932E+00_JPRB ,&
&  0.54240E+00_JPRB, 0.53576E+00_JPRB, 0.54596E+00_JPRB, 0.57714E+00_JPRB, 0.59715E+00_JPRB, 0.57130E+00_JPRB, 0.58339E+00_JPRB ,&
&  0.58672E+00_JPRB, 0.58922E+00_JPRB, 0.59494E+00_JPRB, 0.61008E+00_JPRB, 0.62768E+00_JPRB, 0.64403E+00_JPRB, 0.65688E+00_JPRB ,&
&  0.66904E+00_JPRB, 0.67871E+00_JPRB, 0.68740E+00_JPRB, 0.67629E+00_JPRB, 0.67746E+00_JPRB, 0.71274E+00_JPRB, 0.75766E+00_JPRB ,&
&  0.81355E+00_JPRB, 0.85414E+00_JPRB, 0.86895E+00_JPRB, 0.90441E+00_JPRB, 0.92522E+00_JPRB, 0.92782E+00_JPRB, 0.91975E+00_JPRB ,&
&  0.90951E+00_JPRB, 0.88430E+00_JPRB, 0.86178E+00_JPRB, 0.84898E+00_JPRB, 0.84803E+00_JPRB, 0.85514E+00_JPRB, 0.85484E+00_JPRB ,&
&  0.85031E+00_JPRB, 0.82757E+00_JPRB, 0.80696E+00_JPRB, 0.76662E+00_JPRB, 0.76643E+00_JPRB, 0.78569E+00_JPRB, 0.80252E+00_JPRB ,&
&  0.79860E+00_JPRB, 0.78908E+00_JPRB, 0.78074E+00_JPRB, 0.76664E+00_JPRB, 0.71874E+00_JPRB, 0.68619E+00_JPRB, 0.66652E+00_JPRB ,&
&  0.65511E+00_JPRB, 0.64069E+00_JPRB, 0.63994E+00_JPRB, 0.63683E+00_JPRB, 0.64012E+00_JPRB, 0.64451E+00_JPRB, 0.64099E+00_JPRB ,&
&  0.63905E+00_JPRB, 0.63995E+00_JPRB, 0.64682E+00_JPRB, 0.65518E+00_JPRB, 0.66850E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.75302E+00_JPRB, 0.76281E+00_JPRB, 0.75973E+00_JPRB, 0.75992E+00_JPRB, 0.74321E+00_JPRB, 0.72204E+00_JPRB, 0.69872E+00_JPRB ,&
&  0.68594E+00_JPRB, 0.66529E+00_JPRB, 0.64474E+00_JPRB, 0.62560E+00_JPRB, 0.61522E+00_JPRB, 0.61136E+00_JPRB, 0.61446E+00_JPRB ,&
&  0.62088E+00_JPRB, 0.62836E+00_JPRB, 0.63326E+00_JPRB, 0.63137E+00_JPRB, 0.62703E+00_JPRB, 0.62526E+00_JPRB, 0.62618E+00_JPRB ,&
&  0.62529E+00_JPRB, 0.62358E+00_JPRB, 0.62102E+00_JPRB, 0.61764E+00_JPRB, 0.61397E+00_JPRB, 0.61000E+00_JPRB, 0.60882E+00_JPRB ,&
&  0.60879E+00_JPRB, 0.60880E+00_JPRB, 0.61318E+00_JPRB, 0.61384E+00_JPRB, 0.61777E+00_JPRB, 0.60466E+00_JPRB, 0.58284E+00_JPRB ,&
&  0.56472E+00_JPRB, 0.56470E+00_JPRB, 0.60313E+00_JPRB, 0.58798E+00_JPRB, 0.60472E+00_JPRB, 0.61709E+00_JPRB, 0.61556E+00_JPRB ,&
&  0.61457E+00_JPRB, 0.61805E+00_JPRB, 0.63258E+00_JPRB, 0.64898E+00_JPRB, 0.66316E+00_JPRB, 0.67913E+00_JPRB, 0.69279E+00_JPRB ,&
&  0.70935E+00_JPRB, 0.72525E+00_JPRB, 0.73205E+00_JPRB, 0.71886E+00_JPRB, 0.77007E+00_JPRB, 0.93973E+00_JPRB, 0.97614E+00_JPRB ,&
&  0.10052E+01_JPRB, 0.10269E+01_JPRB, 0.10643E+01_JPRB, 0.10944E+01_JPRB, 0.10730E+01_JPRB, 0.10551E+01_JPRB, 0.10438E+01_JPRB ,&
&  0.10139E+01_JPRB, 0.99517E+00_JPRB, 0.98178E+00_JPRB, 0.99562E+00_JPRB, 0.99838E+00_JPRB, 0.10027E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.99441E+00_JPRB, 0.96975E+00_JPRB, 0.92746E+00_JPRB, 0.90537E+00_JPRB, 0.89048E+00_JPRB, 0.88258E+00_JPRB ,&
&  0.86789E+00_JPRB, 0.85826E+00_JPRB, 0.84536E+00_JPRB, 0.82127E+00_JPRB, 0.76294E+00_JPRB, 0.70686E+00_JPRB, 0.67268E+00_JPRB ,&
&  0.64874E+00_JPRB, 0.62946E+00_JPRB, 0.62749E+00_JPRB, 0.63169E+00_JPRB, 0.64091E+00_JPRB, 0.65103E+00_JPRB, 0.64825E+00_JPRB ,&
&  0.65276E+00_JPRB, 0.66125E+00_JPRB, 0.68003E+00_JPRB, 0.70550E+00_JPRB, 0.73023E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.81688E+00_JPRB, 0.81897E+00_JPRB, 0.80766E+00_JPRB, 0.79743E+00_JPRB, 0.78264E+00_JPRB, 0.75331E+00_JPRB, 0.71152E+00_JPRB ,&
&  0.69693E+00_JPRB, 0.66161E+00_JPRB, 0.64039E+00_JPRB, 0.63249E+00_JPRB, 0.62688E+00_JPRB, 0.61706E+00_JPRB, 0.60558E+00_JPRB ,&
&  0.60587E+00_JPRB, 0.61624E+00_JPRB, 0.62705E+00_JPRB, 0.63108E+00_JPRB, 0.63284E+00_JPRB, 0.63055E+00_JPRB, 0.63014E+00_JPRB ,&
&  0.62968E+00_JPRB, 0.62958E+00_JPRB, 0.63056E+00_JPRB, 0.62490E+00_JPRB, 0.62076E+00_JPRB, 0.61906E+00_JPRB, 0.61734E+00_JPRB ,&
&  0.62111E+00_JPRB, 0.62201E+00_JPRB, 0.62668E+00_JPRB, 0.62552E+00_JPRB, 0.62316E+00_JPRB, 0.62444E+00_JPRB, 0.59386E+00_JPRB ,&
&  0.58241E+00_JPRB, 0.60215E+00_JPRB, 0.59750E+00_JPRB, 0.60694E+00_JPRB, 0.62202E+00_JPRB, 0.63215E+00_JPRB, 0.64066E+00_JPRB ,&
&  0.64659E+00_JPRB, 0.65458E+00_JPRB, 0.66522E+00_JPRB, 0.67909E+00_JPRB, 0.69694E+00_JPRB, 0.71465E+00_JPRB, 0.72995E+00_JPRB ,&
&  0.75831E+00_JPRB, 0.73554E+00_JPRB, 0.73855E+00_JPRB, 0.79830E+00_JPRB, 0.99809E+00_JPRB, 0.10709E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.12041E+01_JPRB, 0.12417E+01_JPRB, 0.12530E+01_JPRB, 0.12697E+01_JPRB, 0.12555E+01_JPRB, 0.12399E+01_JPRB, 0.12481E+01_JPRB ,&
&  0.12333E+01_JPRB, 0.12424E+01_JPRB, 0.12441E+01_JPRB, 0.12657E+01_JPRB, 0.12678E+01_JPRB, 0.12849E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.12554E+01_JPRB, 0.11900E+01_JPRB, 0.11227E+01_JPRB, 0.10603E+01_JPRB, 0.10244E+01_JPRB, 0.99466E+00_JPRB ,&
&  0.96185E+00_JPRB, 0.93792E+00_JPRB, 0.90878E+00_JPRB, 0.86701E+00_JPRB, 0.81217E+00_JPRB, 0.76743E+00_JPRB, 0.73191E+00_JPRB ,&
&  0.68284E+00_JPRB, 0.66305E+00_JPRB, 0.66168E+00_JPRB, 0.66423E+00_JPRB, 0.67381E+00_JPRB, 0.69359E+00_JPRB, 0.69220E+00_JPRB ,&
&  0.71295E+00_JPRB, 0.72968E+00_JPRB, 0.73884E+00_JPRB, 0.75790E+00_JPRB, 0.78485E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.87702E+00_JPRB, 0.87456E+00_JPRB, 0.86065E+00_JPRB, 0.84617E+00_JPRB, 0.82721E+00_JPRB, 0.78428E+00_JPRB, 0.75192E+00_JPRB ,&
&  0.77265E+00_JPRB, 0.73520E+00_JPRB, 0.72194E+00_JPRB, 0.67392E+00_JPRB, 0.64943E+00_JPRB, 0.63364E+00_JPRB, 0.61892E+00_JPRB ,&
&  0.61553E+00_JPRB, 0.62221E+00_JPRB, 0.62947E+00_JPRB, 0.63672E+00_JPRB, 0.63913E+00_JPRB, 0.63754E+00_JPRB, 0.63469E+00_JPRB ,&
&  0.63412E+00_JPRB, 0.63823E+00_JPRB, 0.64115E+00_JPRB, 0.64246E+00_JPRB, 0.64313E+00_JPRB, 0.64265E+00_JPRB, 0.63678E+00_JPRB ,&
&  0.63252E+00_JPRB, 0.63301E+00_JPRB, 0.63857E+00_JPRB, 0.63722E+00_JPRB, 0.63778E+00_JPRB, 0.63643E+00_JPRB, 0.63377E+00_JPRB ,&
&  0.61804E+00_JPRB, 0.61549E+00_JPRB, 0.62259E+00_JPRB, 0.63375E+00_JPRB, 0.64674E+00_JPRB, 0.65765E+00_JPRB, 0.66773E+00_JPRB ,&
&  0.68004E+00_JPRB, 0.69458E+00_JPRB, 0.71129E+00_JPRB, 0.72810E+00_JPRB, 0.74717E+00_JPRB, 0.76907E+00_JPRB, 0.78603E+00_JPRB ,&
&  0.84016E+00_JPRB, 0.80008E+00_JPRB, 0.91995E+00_JPRB, 0.10281E+01_JPRB, 0.12123E+01_JPRB, 0.13625E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.15632E+01_JPRB, 0.15824E+01_JPRB, 0.15612E+01_JPRB, 0.15336E+01_JPRB, 0.15245E+01_JPRB, 0.15394E+01_JPRB ,&
&  0.15684E+01_JPRB, 0.15526E+01_JPRB, 0.16559E+01_JPRB, 0.16548E+01_JPRB, 0.16447E+01_JPRB, 0.16550E+01_JPRB, 0.16391E+01_JPRB ,&
&  0.15964E+01_JPRB, 0.15220E+01_JPRB, 0.14227E+01_JPRB, 0.12647E+01_JPRB, 0.11950E+01_JPRB, 0.11362E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.99879E+00_JPRB, 0.94634E+00_JPRB, 0.89871E+00_JPRB, 0.88863E+00_JPRB, 0.82198E+00_JPRB, 0.77159E+00_JPRB, 0.74023E+00_JPRB ,&
&  0.70257E+00_JPRB, 0.69224E+00_JPRB, 0.70806E+00_JPRB, 0.72789E+00_JPRB, 0.74075E+00_JPRB, 0.74524E+00_JPRB, 0.77137E+00_JPRB ,&
&  0.75990E+00_JPRB, 0.78485E+00_JPRB, 0.83457E+00_JPRB, 0.85260E+00_JPRB, 0.86619E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.96400E+00_JPRB, 0.95404E+00_JPRB, 0.93584E+00_JPRB, 0.91431E+00_JPRB, 0.87560E+00_JPRB, 0.82700E+00_JPRB, 0.78520E+00_JPRB ,&
&  0.79157E+00_JPRB, 0.77921E+00_JPRB, 0.76277E+00_JPRB, 0.73889E+00_JPRB, 0.71197E+00_JPRB, 0.67577E+00_JPRB, 0.63428E+00_JPRB ,&
&  0.62424E+00_JPRB, 0.63619E+00_JPRB, 0.64780E+00_JPRB, 0.65665E+00_JPRB, 0.65606E+00_JPRB, 0.65421E+00_JPRB, 0.65407E+00_JPRB ,&
&  0.65601E+00_JPRB, 0.65999E+00_JPRB, 0.66534E+00_JPRB, 0.66939E+00_JPRB, 0.66982E+00_JPRB, 0.67031E+00_JPRB, 0.66846E+00_JPRB ,&
&  0.65196E+00_JPRB, 0.64190E+00_JPRB, 0.63828E+00_JPRB, 0.63792E+00_JPRB, 0.66047E+00_JPRB, 0.67386E+00_JPRB, 0.68470E+00_JPRB ,&
&  0.68702E+00_JPRB, 0.68487E+00_JPRB, 0.68652E+00_JPRB, 0.69892E+00_JPRB, 0.71114E+00_JPRB, 0.71719E+00_JPRB, 0.72702E+00_JPRB ,&
&  0.74202E+00_JPRB, 0.76014E+00_JPRB, 0.78038E+00_JPRB, 0.80670E+00_JPRB, 0.83985E+00_JPRB, 0.84006E+00_JPRB, 0.88659E+00_JPRB ,&
&  0.96793E+00_JPRB, 0.11112E+01_JPRB, 0.12679E+01_JPRB, 0.13095E+01_JPRB, 0.14915E+01_JPRB, 0.16330E+01_JPRB, 0.16698E+01_JPRB ,&
&  0.17727E+01_JPRB, 0.18027E+01_JPRB, 0.19046E+01_JPRB, 0.19320E+01_JPRB, 0.19649E+01_JPRB, 0.19889E+01_JPRB, 0.19961E+01_JPRB ,&
&  0.20157E+01_JPRB, 0.20777E+01_JPRB, 0.21378E+01_JPRB, 0.21514E+01_JPRB, 0.21049E+01_JPRB, 0.20298E+01_JPRB, 0.19238E+01_JPRB ,&
&  0.18196E+01_JPRB, 0.17009E+01_JPRB, 0.15828E+01_JPRB, 0.14889E+01_JPRB, 0.13770E+01_JPRB, 0.12560E+01_JPRB, 0.11388E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.90167E+00_JPRB, 0.84727E+00_JPRB, 0.84388E+00_JPRB, 0.82601E+00_JPRB, 0.76531E+00_JPRB, 0.73326E+00_JPRB ,&
&  0.72526E+00_JPRB, 0.74687E+00_JPRB, 0.78027E+00_JPRB, 0.79945E+00_JPRB, 0.83468E+00_JPRB, 0.86348E+00_JPRB, 0.82202E+00_JPRB ,&
&  0.79714E+00_JPRB, 0.87006E+00_JPRB, 0.91598E+00_JPRB, 0.94324E+00_JPRB, 0.95995E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.10440E+01_JPRB, 0.10255E+01_JPRB, 0.10061E+01_JPRB, 0.98508E+00_JPRB, 0.94424E+00_JPRB, 0.88845E+00_JPRB, 0.84902E+00_JPRB ,&
&  0.83314E+00_JPRB, 0.81027E+00_JPRB, 0.78692E+00_JPRB, 0.76466E+00_JPRB, 0.74493E+00_JPRB, 0.72163E+00_JPRB, 0.69525E+00_JPRB ,&
&  0.67597E+00_JPRB, 0.67997E+00_JPRB, 0.67482E+00_JPRB, 0.67452E+00_JPRB, 0.68123E+00_JPRB, 0.67775E+00_JPRB, 0.67783E+00_JPRB ,&
&  0.68605E+00_JPRB, 0.69027E+00_JPRB, 0.69094E+00_JPRB, 0.68924E+00_JPRB, 0.68900E+00_JPRB, 0.69232E+00_JPRB, 0.69077E+00_JPRB ,&
&  0.66886E+00_JPRB, 0.65158E+00_JPRB, 0.63991E+00_JPRB, 0.64086E+00_JPRB, 0.66701E+00_JPRB, 0.70616E+00_JPRB, 0.73761E+00_JPRB ,&
&  0.76200E+00_JPRB, 0.78028E+00_JPRB, 0.78763E+00_JPRB, 0.79377E+00_JPRB, 0.79470E+00_JPRB, 0.79325E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.80043E+00_JPRB, 0.81418E+00_JPRB, 0.83479E+00_JPRB, 0.87220E+00_JPRB, 0.89895E+00_JPRB, 0.92989E+00_JPRB, 0.10256E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.13780E+01_JPRB, 0.14969E+01_JPRB, 0.17114E+01_JPRB, 0.18917E+01_JPRB, 0.19640E+01_JPRB, 0.19927E+01_JPRB ,&
&  0.20770E+01_JPRB, 0.20672E+01_JPRB, 0.22384E+01_JPRB, 0.22343E+01_JPRB, 0.22572E+01_JPRB, 0.23606E+01_JPRB, 0.23628E+01_JPRB ,&
&  0.23432E+01_JPRB, 0.23162E+01_JPRB, 0.24184E+01_JPRB, 0.24248E+01_JPRB, 0.23867E+01_JPRB, 0.23152E+01_JPRB, 0.21787E+01_JPRB ,&
&  0.20422E+01_JPRB, 0.18944E+01_JPRB, 0.16212E+01_JPRB, 0.14163E+01_JPRB, 0.12062E+01_JPRB, 0.11850E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.97887E+00_JPRB, 0.85089E+00_JPRB, 0.84180E+00_JPRB, 0.84356E+00_JPRB, 0.86090E+00_JPRB, 0.83418E+00_JPRB, 0.78536E+00_JPRB ,&
&  0.78247E+00_JPRB, 0.82125E+00_JPRB, 0.83658E+00_JPRB, 0.86470E+00_JPRB, 0.91994E+00_JPRB, 0.98543E+00_JPRB, 0.87833E+00_JPRB ,&
&  0.89463E+00_JPRB, 0.98861E+00_JPRB, 0.10303E+01_JPRB, 0.10501E+01_JPRB, 0.10535E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.10755E+01_JPRB, 0.10476E+01_JPRB, 0.10216E+01_JPRB, 0.10002E+01_JPRB, 0.97001E+00_JPRB, 0.93825E+00_JPRB, 0.89900E+00_JPRB ,&
&  0.86335E+00_JPRB, 0.82999E+00_JPRB, 0.80702E+00_JPRB, 0.79348E+00_JPRB, 0.78177E+00_JPRB, 0.75837E+00_JPRB, 0.75629E+00_JPRB ,&
&  0.72499E+00_JPRB, 0.71492E+00_JPRB, 0.68362E+00_JPRB, 0.68636E+00_JPRB, 0.72031E+00_JPRB, 0.70250E+00_JPRB, 0.70250E+00_JPRB ,&
&  0.70563E+00_JPRB, 0.71298E+00_JPRB, 0.70699E+00_JPRB, 0.69969E+00_JPRB, 0.69661E+00_JPRB, 0.69774E+00_JPRB, 0.70112E+00_JPRB ,&
&  0.68468E+00_JPRB, 0.65366E+00_JPRB, 0.63792E+00_JPRB, 0.64250E+00_JPRB, 0.67860E+00_JPRB, 0.72498E+00_JPRB, 0.77692E+00_JPRB ,&
&  0.80779E+00_JPRB, 0.83397E+00_JPRB, 0.85225E+00_JPRB, 0.86310E+00_JPRB, 0.86470E+00_JPRB, 0.86564E+00_JPRB, 0.86958E+00_JPRB ,&
&  0.87412E+00_JPRB, 0.88966E+00_JPRB, 0.92144E+00_JPRB, 0.96137E+00_JPRB, 0.97183E+00_JPRB, 0.10497E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.16946E+01_JPRB, 0.18114E+01_JPRB, 0.22297E+01_JPRB, 0.24714E+01_JPRB, 0.25461E+01_JPRB, 0.25645E+01_JPRB ,&
&  0.25885E+01_JPRB, 0.25698E+01_JPRB, 0.25821E+01_JPRB, 0.26846E+01_JPRB, 0.26660E+01_JPRB, 0.26690E+01_JPRB, 0.25975E+01_JPRB ,&
&  0.24951E+01_JPRB, 0.25046E+01_JPRB, 0.25421E+01_JPRB, 0.24873E+01_JPRB, 0.23867E+01_JPRB, 0.23506E+01_JPRB, 0.22854E+01_JPRB ,&
&  0.21613E+01_JPRB, 0.19025E+01_JPRB, 0.14190E+01_JPRB, 0.12535E+01_JPRB, 0.98052E+00_JPRB, 0.97910E+00_JPRB, 0.11230E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.99837E+00_JPRB, 0.94820E+00_JPRB, 0.93215E+00_JPRB, 0.93754E+00_JPRB, 0.96889E+00_JPRB, 0.92808E+00_JPRB ,&
&  0.87148E+00_JPRB, 0.88626E+00_JPRB, 0.91903E+00_JPRB, 0.98448E+00_JPRB, 0.10797E+01_JPRB, 0.11370E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.11363E+01_JPRB, 0.11428E+01_JPRB, 0.11374E+01_JPRB, 0.11115E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.10496E+01_JPRB, 0.10210E+01_JPRB, 0.99132E+00_JPRB, 0.97844E+00_JPRB, 0.97375E+00_JPRB, 0.96519E+00_JPRB, 0.93473E+00_JPRB ,&
&  0.88761E+00_JPRB, 0.85040E+00_JPRB, 0.82790E+00_JPRB, 0.82364E+00_JPRB, 0.81950E+00_JPRB, 0.79640E+00_JPRB, 0.78706E+00_JPRB ,&
&  0.80604E+00_JPRB, 0.77980E+00_JPRB, 0.72556E+00_JPRB, 0.68859E+00_JPRB, 0.70976E+00_JPRB, 0.70570E+00_JPRB, 0.71118E+00_JPRB ,&
&  0.70553E+00_JPRB, 0.72454E+00_JPRB, 0.71934E+00_JPRB, 0.71639E+00_JPRB, 0.71734E+00_JPRB, 0.72655E+00_JPRB, 0.75333E+00_JPRB ,&
&  0.75202E+00_JPRB, 0.70803E+00_JPRB, 0.68520E+00_JPRB, 0.70148E+00_JPRB, 0.76522E+00_JPRB, 0.76241E+00_JPRB, 0.80824E+00_JPRB ,&
&  0.84370E+00_JPRB, 0.87814E+00_JPRB, 0.90664E+00_JPRB, 0.93187E+00_JPRB, 0.94960E+00_JPRB, 0.96201E+00_JPRB, 0.97662E+00_JPRB ,&
&  0.99766E+00_JPRB, 0.10165E+01_JPRB, 0.10171E+01_JPRB, 0.10498E+01_JPRB, 0.11479E+01_JPRB, 0.14062E+01_JPRB, 0.17223E+01_JPRB ,&
&  0.19348E+01_JPRB, 0.19645E+01_JPRB, 0.19972E+01_JPRB, 0.23936E+01_JPRB, 0.27411E+01_JPRB, 0.28873E+01_JPRB, 0.30434E+01_JPRB ,&
&  0.30895E+01_JPRB, 0.30942E+01_JPRB, 0.30697E+01_JPRB, 0.28557E+01_JPRB, 0.25398E+01_JPRB, 0.23395E+01_JPRB, 0.22666E+01_JPRB ,&
&  0.22057E+01_JPRB, 0.22358E+01_JPRB, 0.23397E+01_JPRB, 0.23012E+01_JPRB, 0.22860E+01_JPRB, 0.21916E+01_JPRB, 0.23016E+01_JPRB ,&
&  0.22428E+01_JPRB, 0.13857E+01_JPRB, 0.10776E+01_JPRB, 0.11324E+01_JPRB, 0.94946E+00_JPRB, 0.95674E+00_JPRB, 0.10577E+01_JPRB ,&
&  0.10261E+01_JPRB, 0.10167E+01_JPRB, 0.10551E+01_JPRB, 0.10693E+01_JPRB, 0.10751E+01_JPRB, 0.11462E+01_JPRB, 0.11608E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.10529E+01_JPRB, 0.11054E+01_JPRB, 0.12030E+01_JPRB, 0.13038E+01_JPRB, 0.13193E+01_JPRB, 0.12885E+01_JPRB ,&
&  0.12692E+01_JPRB, 0.12491E+01_JPRB, 0.12186E+01_JPRB, 0.11605E+01_JPRB, 0.10931E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.10698E+01_JPRB, 0.10429E+01_JPRB, 0.10094E+01_JPRB, 0.10063E+01_JPRB, 0.10149E+01_JPRB, 0.10053E+01_JPRB, 0.96643E+00_JPRB ,&
&  0.89684E+00_JPRB, 0.86011E+00_JPRB, 0.84398E+00_JPRB, 0.84132E+00_JPRB, 0.84518E+00_JPRB, 0.84019E+00_JPRB, 0.83015E+00_JPRB ,&
&  0.81071E+00_JPRB, 0.82395E+00_JPRB, 0.76276E+00_JPRB, 0.72639E+00_JPRB, 0.68301E+00_JPRB, 0.69654E+00_JPRB, 0.71265E+00_JPRB ,&
&  0.72716E+00_JPRB, 0.73939E+00_JPRB, 0.76720E+00_JPRB, 0.80534E+00_JPRB, 0.84090E+00_JPRB, 0.87257E+00_JPRB, 0.94174E+00_JPRB ,&
&  0.92713E+00_JPRB, 0.85987E+00_JPRB, 0.80109E+00_JPRB, 0.82491E+00_JPRB, 0.86446E+00_JPRB, 0.83070E+00_JPRB, 0.89517E+00_JPRB ,&
&  0.94041E+00_JPRB, 0.98054E+00_JPRB, 0.10175E+01_JPRB, 0.10382E+01_JPRB, 0.10434E+01_JPRB, 0.10181E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10425E+01_JPRB, 0.10833E+01_JPRB, 0.11301E+01_JPRB, 0.14060E+01_JPRB, 0.17942E+01_JPRB, 0.20908E+01_JPRB ,&
&  0.22950E+01_JPRB, 0.19726E+01_JPRB, 0.17596E+01_JPRB, 0.23053E+01_JPRB, 0.32854E+01_JPRB, 0.36015E+01_JPRB, 0.33963E+01_JPRB ,&
&  0.36046E+01_JPRB, 0.33037E+01_JPRB, 0.28887E+01_JPRB, 0.26067E+01_JPRB, 0.23993E+01_JPRB, 0.22509E+01_JPRB, 0.21898E+01_JPRB ,&
&  0.20679E+01_JPRB, 0.20747E+01_JPRB, 0.21940E+01_JPRB, 0.22667E+01_JPRB, 0.22567E+01_JPRB, 0.22429E+01_JPRB, 0.23123E+01_JPRB ,&
&  0.21662E+01_JPRB, 0.17572E+01_JPRB, 0.12445E+01_JPRB, 0.11459E+01_JPRB, 0.93814E+00_JPRB, 0.92297E+00_JPRB, 0.10259E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10763E+01_JPRB, 0.11111E+01_JPRB, 0.11105E+01_JPRB, 0.12273E+01_JPRB, 0.13039E+01_JPRB, 0.14323E+01_JPRB ,&
&  0.15160E+01_JPRB, 0.13241E+01_JPRB, 0.14225E+01_JPRB, 0.15288E+01_JPRB, 0.15642E+01_JPRB, 0.14964E+01_JPRB, 0.14103E+01_JPRB ,&
&  0.13552E+01_JPRB, 0.13101E+01_JPRB, 0.12490E+01_JPRB, 0.11632E+01_JPRB, 0.11029E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.10898E+01_JPRB, 0.10739E+01_JPRB, 0.10368E+01_JPRB, 0.10330E+01_JPRB, 0.10268E+01_JPRB, 0.10025E+01_JPRB, 0.97063E+00_JPRB ,&
&  0.93785E+00_JPRB, 0.92317E+00_JPRB, 0.90535E+00_JPRB, 0.89273E+00_JPRB, 0.88838E+00_JPRB, 0.87711E+00_JPRB, 0.85789E+00_JPRB ,&
&  0.82971E+00_JPRB, 0.81610E+00_JPRB, 0.74375E+00_JPRB, 0.69990E+00_JPRB, 0.67502E+00_JPRB, 0.68361E+00_JPRB, 0.71226E+00_JPRB ,&
&  0.75817E+00_JPRB, 0.82176E+00_JPRB, 0.92729E+00_JPRB, 0.10260E+01_JPRB, 0.11617E+01_JPRB, 0.12686E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.13368E+01_JPRB, 0.10830E+01_JPRB, 0.10592E+01_JPRB, 0.10049E+01_JPRB, 0.10170E+01_JPRB, 0.99849E+00_JPRB, 0.10072E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.10431E+01_JPRB, 0.10627E+01_JPRB, 0.10496E+01_JPRB, 0.10310E+01_JPRB, 0.10230E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.11152E+01_JPRB, 0.11297E+01_JPRB, 0.11798E+01_JPRB, 0.14046E+01_JPRB, 0.18804E+01_JPRB, 0.23119E+01_JPRB, 0.25936E+01_JPRB ,&
&  0.24565E+01_JPRB, 0.23194E+01_JPRB, 0.31268E+01_JPRB, 0.34741E+01_JPRB, 0.32826E+01_JPRB, 0.37596E+01_JPRB, 0.38263E+01_JPRB ,&
&  0.34602E+01_JPRB, 0.28103E+01_JPRB, 0.25108E+01_JPRB, 0.23024E+01_JPRB, 0.27028E+01_JPRB, 0.25473E+01_JPRB, 0.22631E+01_JPRB ,&
&  0.20222E+01_JPRB, 0.21504E+01_JPRB, 0.22114E+01_JPRB, 0.23201E+01_JPRB, 0.24756E+01_JPRB, 0.25564E+01_JPRB, 0.23305E+01_JPRB ,&
&  0.20396E+01_JPRB, 0.21399E+01_JPRB, 0.17050E+01_JPRB, 0.12893E+01_JPRB, 0.10953E+01_JPRB, 0.10870E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.11547E+01_JPRB, 0.11792E+01_JPRB, 0.11903E+01_JPRB, 0.12796E+01_JPRB, 0.14768E+01_JPRB, 0.16037E+01_JPRB, 0.16823E+01_JPRB ,&
&  0.16718E+01_JPRB, 0.18287E+01_JPRB, 0.18510E+01_JPRB, 0.18191E+01_JPRB, 0.16887E+01_JPRB, 0.15158E+01_JPRB, 0.14105E+01_JPRB ,&
&  0.13623E+01_JPRB, 0.13252E+01_JPRB, 0.12417E+01_JPRB, 0.11539E+01_JPRB, 0.10968E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.11061E+01_JPRB, 0.11028E+01_JPRB, 0.10893E+01_JPRB, 0.10688E+01_JPRB, 0.10452E+01_JPRB, 0.10227E+01_JPRB, 0.10110E+01_JPRB ,&
&  0.99214E+00_JPRB, 0.96164E+00_JPRB, 0.95802E+00_JPRB, 0.96387E+00_JPRB, 0.95855E+00_JPRB, 0.93240E+00_JPRB, 0.90016E+00_JPRB ,&
&  0.87639E+00_JPRB, 0.80927E+00_JPRB, 0.72582E+00_JPRB, 0.72347E+00_JPRB, 0.71945E+00_JPRB, 0.70320E+00_JPRB, 0.74073E+00_JPRB ,&
&  0.81833E+00_JPRB, 0.98288E+00_JPRB, 0.11736E+01_JPRB, 0.13817E+01_JPRB, 0.15088E+01_JPRB, 0.15518E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.14409E+01_JPRB, 0.13757E+01_JPRB, 0.13256E+01_JPRB, 0.12492E+01_JPRB, 0.12120E+01_JPRB, 0.11870E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.10913E+01_JPRB, 0.10896E+01_JPRB, 0.10615E+01_JPRB, 0.10657E+01_JPRB, 0.11011E+01_JPRB, 0.11589E+01_JPRB ,&
&  0.11766E+01_JPRB, 0.12021E+01_JPRB, 0.13463E+01_JPRB, 0.17281E+01_JPRB, 0.19197E+01_JPRB, 0.22681E+01_JPRB, 0.26053E+01_JPRB ,&
&  0.34860E+01_JPRB, 0.37709E+01_JPRB, 0.34617E+01_JPRB, 0.41408E+01_JPRB, 0.27060E+01_JPRB, 0.31034E+01_JPRB, 0.35504E+01_JPRB ,&
&  0.32699E+01_JPRB, 0.31149E+01_JPRB, 0.33028E+01_JPRB, 0.29601E+01_JPRB, 0.25468E+01_JPRB, 0.34452E+01_JPRB, 0.28254E+01_JPRB ,&
&  0.23963E+01_JPRB, 0.24012E+01_JPRB, 0.25503E+01_JPRB, 0.28727E+01_JPRB, 0.25183E+01_JPRB, 0.22438E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.17877E+01_JPRB, 0.16086E+01_JPRB, 0.17170E+01_JPRB, 0.14971E+01_JPRB, 0.14733E+01_JPRB, 0.13566E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.12919E+01_JPRB, 0.12946E+01_JPRB, 0.13290E+01_JPRB, 0.15615E+01_JPRB, 0.19497E+01_JPRB, 0.18352E+01_JPRB, 0.19860E+01_JPRB ,&
&  0.21159E+01_JPRB, 0.21559E+01_JPRB, 0.19791E+01_JPRB, 0.18411E+01_JPRB, 0.16057E+01_JPRB, 0.14651E+01_JPRB, 0.14034E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.12712E+01_JPRB, 0.11752E+01_JPRB, 0.11353E+01_JPRB, 0.10928E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.11108E+01_JPRB, 0.10938E+01_JPRB, 0.10877E+01_JPRB, 0.10696E+01_JPRB, 0.10651E+01_JPRB, 0.10504E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.99564E+00_JPRB, 0.99692E+00_JPRB, 0.99643E+00_JPRB, 0.98537E+00_JPRB, 0.96960E+00_JPRB, 0.93180E+00_JPRB ,&
&  0.91454E+00_JPRB, 0.86338E+00_JPRB, 0.75708E+00_JPRB, 0.72791E+00_JPRB, 0.74373E+00_JPRB, 0.74302E+00_JPRB, 0.77987E+00_JPRB ,&
&  0.99481E+00_JPRB, 0.13027E+01_JPRB, 0.15010E+01_JPRB, 0.16395E+01_JPRB, 0.16948E+01_JPRB, 0.16248E+01_JPRB, 0.15545E+01_JPRB ,&
&  0.15379E+01_JPRB, 0.13861E+01_JPRB, 0.12893E+01_JPRB, 0.12694E+01_JPRB, 0.12282E+01_JPRB, 0.11799E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.10556E+01_JPRB, 0.10302E+01_JPRB, 0.10110E+01_JPRB, 0.10391E+01_JPRB, 0.10918E+01_JPRB, 0.11592E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.13084E+01_JPRB, 0.15969E+01_JPRB, 0.19835E+01_JPRB, 0.22962E+01_JPRB, 0.26389E+01_JPRB, 0.34075E+01_JPRB ,&
&  0.38106E+01_JPRB, 0.37168E+01_JPRB, 0.38233E+01_JPRB, 0.40230E+01_JPRB, 0.37017E+01_JPRB, 0.31997E+01_JPRB, 0.34353E+01_JPRB ,&
&  0.26963E+01_JPRB, 0.24379E+01_JPRB, 0.24010E+01_JPRB, 0.24766E+01_JPRB, 0.31900E+01_JPRB, 0.47885E+01_JPRB, 0.42454E+01_JPRB ,&
&  0.37814E+01_JPRB, 0.32763E+01_JPRB, 0.33400E+01_JPRB, 0.31008E+01_JPRB, 0.27065E+01_JPRB, 0.20042E+01_JPRB, 0.24266E+01_JPRB ,&
&  0.24948E+01_JPRB, 0.25515E+01_JPRB, 0.22982E+01_JPRB, 0.18641E+01_JPRB, 0.15470E+01_JPRB, 0.16424E+01_JPRB, 0.15942E+01_JPRB ,&
&  0.15668E+01_JPRB, 0.15415E+01_JPRB, 0.19254E+01_JPRB, 0.21887E+01_JPRB, 0.23446E+01_JPRB, 0.23634E+01_JPRB, 0.25893E+01_JPRB ,&
&  0.24344E+01_JPRB, 0.23414E+01_JPRB, 0.20793E+01_JPRB, 0.17501E+01_JPRB, 0.15439E+01_JPRB, 0.14451E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.12923E+01_JPRB, 0.11834E+01_JPRB, 0.11298E+01_JPRB, 0.10949E+01_JPRB, 0.10808E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.11001E+01_JPRB, 0.10896E+01_JPRB, 0.10701E+01_JPRB, 0.10596E+01_JPRB, 0.10441E+01_JPRB, 0.10203E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.10096E+01_JPRB, 0.98359E+00_JPRB, 0.96557E+00_JPRB, 0.95216E+00_JPRB, 0.96387E+00_JPRB, 0.96844E+00_JPRB ,&
&  0.96697E+00_JPRB, 0.95897E+00_JPRB, 0.82500E+00_JPRB, 0.79193E+00_JPRB, 0.80199E+00_JPRB, 0.83480E+00_JPRB, 0.88954E+00_JPRB ,&
&  0.12492E+01_JPRB, 0.15966E+01_JPRB, 0.16696E+01_JPRB, 0.17089E+01_JPRB, 0.17040E+01_JPRB, 0.16101E+01_JPRB, 0.15753E+01_JPRB ,&
&  0.15646E+01_JPRB, 0.14615E+01_JPRB, 0.13426E+01_JPRB, 0.12689E+01_JPRB, 0.11955E+01_JPRB, 0.11479E+01_JPRB, 0.11117E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10343E+01_JPRB, 0.10162E+01_JPRB, 0.10266E+01_JPRB, 0.10356E+01_JPRB, 0.11330E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.13504E+01_JPRB, 0.15753E+01_JPRB, 0.18611E+01_JPRB, 0.22560E+01_JPRB, 0.26942E+01_JPRB, 0.31424E+01_JPRB, 0.43893E+01_JPRB ,&
&  0.42721E+01_JPRB, 0.42853E+01_JPRB, 0.42053E+01_JPRB, 0.48654E+01_JPRB, 0.36409E+01_JPRB, 0.30954E+01_JPRB, 0.32363E+01_JPRB ,&
&  0.25280E+01_JPRB, 0.23615E+01_JPRB, 0.28145E+01_JPRB, 0.36963E+01_JPRB, 0.38636E+01_JPRB, 0.42923E+01_JPRB, 0.56597E+01_JPRB ,&
&  0.50052E+01_JPRB, 0.43415E+01_JPRB, 0.40720E+01_JPRB, 0.39879E+01_JPRB, 0.20581E+01_JPRB, 0.16295E+01_JPRB, 0.24718E+01_JPRB ,&
&  0.25722E+01_JPRB, 0.17413E+01_JPRB, 0.13332E+01_JPRB, 0.15951E+01_JPRB, 0.12445E+01_JPRB, 0.14038E+01_JPRB, 0.18164E+01_JPRB ,&
&  0.18676E+01_JPRB, 0.20118E+01_JPRB, 0.28678E+01_JPRB, 0.28036E+01_JPRB, 0.27427E+01_JPRB, 0.35410E+01_JPRB, 0.31418E+01_JPRB ,&
&  0.27700E+01_JPRB, 0.21729E+01_JPRB, 0.19349E+01_JPRB, 0.16387E+01_JPRB, 0.14746E+01_JPRB, 0.13544E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11121E+01_JPRB, 0.11118E+01_JPRB, 0.11104E+01_JPRB, 0.10872E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.10941E+01_JPRB, 0.10649E+01_JPRB, 0.10303E+01_JPRB, 0.10286E+01_JPRB, 0.10072E+01_JPRB, 0.98101E+00_JPRB, 0.10296E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.99645E+00_JPRB, 0.94354E+00_JPRB, 0.92821E+00_JPRB, 0.93373E+00_JPRB, 0.99846E+00_JPRB, 0.10330E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10376E+01_JPRB, 0.99127E+00_JPRB, 0.97796E+00_JPRB, 0.10014E+01_JPRB, 0.10462E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.17750E+01_JPRB, 0.16826E+01_JPRB, 0.16365E+01_JPRB, 0.16248E+01_JPRB, 0.15676E+01_JPRB, 0.16211E+01_JPRB ,&
&  0.15655E+01_JPRB, 0.14364E+01_JPRB, 0.13350E+01_JPRB, 0.12543E+01_JPRB, 0.12135E+01_JPRB, 0.11833E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.10488E+01_JPRB, 0.10527E+01_JPRB, 0.10428E+01_JPRB, 0.10979E+01_JPRB, 0.12256E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.18299E+01_JPRB, 0.21189E+01_JPRB, 0.25972E+01_JPRB, 0.33888E+01_JPRB, 0.38139E+01_JPRB, 0.35595E+01_JPRB ,&
&  0.32786E+01_JPRB, 0.32476E+01_JPRB, 0.42393E+01_JPRB, 0.43547E+01_JPRB, 0.37624E+01_JPRB, 0.34388E+01_JPRB, 0.32266E+01_JPRB ,&
&  0.28922E+01_JPRB, 0.28262E+01_JPRB, 0.32736E+01_JPRB, 0.42944E+01_JPRB, 0.48768E+01_JPRB, 0.46193E+01_JPRB, 0.59024E+01_JPRB ,&
&  0.59604E+01_JPRB, 0.51790E+01_JPRB, 0.35688E+01_JPRB, 0.26393E+01_JPRB, 0.23425E+01_JPRB, 0.19844E+01_JPRB, 0.14249E+01_JPRB ,&
&  0.12896E+01_JPRB, 0.12780E+01_JPRB, 0.11971E+01_JPRB, 0.12084E+01_JPRB, 0.11665E+01_JPRB, 0.14380E+01_JPRB, 0.18178E+01_JPRB ,&
&  0.23804E+01_JPRB, 0.28930E+01_JPRB, 0.32507E+01_JPRB, 0.32074E+01_JPRB, 0.30618E+01_JPRB, 0.40970E+01_JPRB, 0.33458E+01_JPRB ,&
&  0.22867E+01_JPRB, 0.18903E+01_JPRB, 0.16945E+01_JPRB, 0.15046E+01_JPRB, 0.13656E+01_JPRB, 0.12396E+01_JPRB, 0.11439E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.10937E+01_JPRB, 0.11009E+01_JPRB, 0.11000E+01_JPRB, 0.10728E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.10864E+01_JPRB, 0.10552E+01_JPRB, 0.10197E+01_JPRB, 0.99189E+00_JPRB, 0.95078E+00_JPRB, 0.96385E+00_JPRB, 0.10480E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.97967E+00_JPRB, 0.91964E+00_JPRB, 0.92158E+00_JPRB, 0.98775E+00_JPRB, 0.10624E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11287E+01_JPRB, 0.11892E+01_JPRB, 0.12607E+01_JPRB, 0.14030E+01_JPRB, 0.14438E+01_JPRB, 0.17423E+01_JPRB ,&
&  0.20048E+01_JPRB, 0.18179E+01_JPRB, 0.16403E+01_JPRB, 0.16031E+01_JPRB, 0.16171E+01_JPRB, 0.16163E+01_JPRB, 0.16209E+01_JPRB ,&
&  0.14989E+01_JPRB, 0.13881E+01_JPRB, 0.12880E+01_JPRB, 0.12391E+01_JPRB, 0.12240E+01_JPRB, 0.11717E+01_JPRB, 0.11340E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.10815E+01_JPRB, 0.10679E+01_JPRB, 0.11033E+01_JPRB, 0.12317E+01_JPRB, 0.14054E+01_JPRB, 0.16013E+01_JPRB ,&
&  0.18227E+01_JPRB, 0.21416E+01_JPRB, 0.24975E+01_JPRB, 0.30685E+01_JPRB, 0.30920E+01_JPRB, 0.26855E+01_JPRB, 0.30751E+01_JPRB ,&
&  0.33865E+01_JPRB, 0.36842E+01_JPRB, 0.35947E+01_JPRB, 0.43368E+01_JPRB, 0.39947E+01_JPRB, 0.33589E+01_JPRB, 0.31248E+01_JPRB ,&
&  0.28535E+01_JPRB, 0.35046E+01_JPRB, 0.41702E+01_JPRB, 0.48056E+01_JPRB, 0.64024E+01_JPRB, 0.61302E+01_JPRB, 0.66392E+01_JPRB ,&
&  0.68819E+01_JPRB, 0.56619E+01_JPRB, 0.40769E+01_JPRB, 0.30505E+01_JPRB, 0.35353E+01_JPRB, 0.38142E+01_JPRB, 0.17329E+01_JPRB ,&
&  0.14205E+01_JPRB, 0.13207E+01_JPRB, 0.12845E+01_JPRB, 0.12000E+01_JPRB, 0.11861E+01_JPRB, 0.14880E+01_JPRB, 0.24809E+01_JPRB ,&
&  0.24231E+01_JPRB, 0.37337E+01_JPRB, 0.36105E+01_JPRB, 0.35059E+01_JPRB, 0.32270E+01_JPRB, 0.39648E+01_JPRB, 0.28917E+01_JPRB ,&
&  0.22974E+01_JPRB, 0.19441E+01_JPRB, 0.16489E+01_JPRB, 0.14811E+01_JPRB, 0.13145E+01_JPRB, 0.11738E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.10532E+01_JPRB, 0.10469E+01_JPRB, 0.10525E+01_JPRB, 0.10626E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.10738E+01_JPRB, 0.10407E+01_JPRB, 0.10174E+01_JPRB, 0.99888E+00_JPRB, 0.96956E+00_JPRB, 0.10249E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.10056E+01_JPRB, 0.96733E+00_JPRB, 0.99241E+00_JPRB, 0.10640E+01_JPRB, 0.11465E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.12653E+01_JPRB, 0.13328E+01_JPRB, 0.14522E+01_JPRB, 0.17434E+01_JPRB, 0.19494E+01_JPRB, 0.20564E+01_JPRB, 0.23204E+01_JPRB ,&
&  0.21167E+01_JPRB, 0.17114E+01_JPRB, 0.16181E+01_JPRB, 0.16478E+01_JPRB, 0.15972E+01_JPRB, 0.15510E+01_JPRB, 0.15099E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.13530E+01_JPRB, 0.12908E+01_JPRB, 0.12997E+01_JPRB, 0.13000E+01_JPRB, 0.12413E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.11657E+01_JPRB, 0.11878E+01_JPRB, 0.12374E+01_JPRB, 0.14176E+01_JPRB, 0.15977E+01_JPRB, 0.18286E+01_JPRB ,&
&  0.21242E+01_JPRB, 0.24200E+01_JPRB, 0.27553E+01_JPRB, 0.31578E+01_JPRB, 0.26896E+01_JPRB, 0.32276E+01_JPRB, 0.34130E+01_JPRB ,&
&  0.35797E+01_JPRB, 0.34491E+01_JPRB, 0.31061E+01_JPRB, 0.35908E+01_JPRB, 0.37064E+01_JPRB, 0.31817E+01_JPRB, 0.29211E+01_JPRB ,&
&  0.34195E+01_JPRB, 0.40064E+01_JPRB, 0.51112E+01_JPRB, 0.59957E+01_JPRB, 0.69038E+01_JPRB, 0.95925E+01_JPRB, 0.67778E+01_JPRB ,&
&  0.56630E+01_JPRB, 0.55504E+01_JPRB, 0.45917E+01_JPRB, 0.38237E+01_JPRB, 0.42212E+01_JPRB, 0.58500E+01_JPRB, 0.42317E+01_JPRB ,&
&  0.16428E+01_JPRB, 0.12627E+01_JPRB, 0.14061E+01_JPRB, 0.14218E+01_JPRB, 0.12564E+01_JPRB, 0.15226E+01_JPRB, 0.39419E+01_JPRB ,&
&  0.30376E+01_JPRB, 0.42067E+01_JPRB, 0.36668E+01_JPRB, 0.34103E+01_JPRB, 0.33671E+01_JPRB, 0.36434E+01_JPRB, 0.30993E+01_JPRB ,&
&  0.22502E+01_JPRB, 0.18781E+01_JPRB, 0.16032E+01_JPRB, 0.14142E+01_JPRB, 0.12446E+01_JPRB, 0.11405E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.10401E+01_JPRB, 0.10375E+01_JPRB, 0.10413E+01_JPRB, 0.10585E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.10780E+01_JPRB, 0.10227E+01_JPRB, 0.99943E+00_JPRB, 0.97989E+00_JPRB, 0.10314E+01_JPRB, 0.10972E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.11276E+01_JPRB, 0.11337E+01_JPRB, 0.11392E+01_JPRB, 0.11344E+01_JPRB, 0.11909E+01_JPRB, 0.13160E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.15519E+01_JPRB, 0.16955E+01_JPRB, 0.18885E+01_JPRB, 0.21955E+01_JPRB, 0.24969E+01_JPRB, 0.24675E+01_JPRB, 0.23855E+01_JPRB ,&
&  0.20524E+01_JPRB, 0.17058E+01_JPRB, 0.15655E+01_JPRB, 0.16616E+01_JPRB, 0.16350E+01_JPRB, 0.15831E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.15503E+01_JPRB, 0.15090E+01_JPRB, 0.14613E+01_JPRB, 0.14790E+01_JPRB, 0.14261E+01_JPRB, 0.14227E+01_JPRB, 0.15070E+01_JPRB ,&
&  0.14715E+01_JPRB, 0.14593E+01_JPRB, 0.14780E+01_JPRB, 0.15384E+01_JPRB, 0.17270E+01_JPRB, 0.19096E+01_JPRB, 0.21508E+01_JPRB ,&
&  0.24219E+01_JPRB, 0.26434E+01_JPRB, 0.29332E+01_JPRB, 0.29998E+01_JPRB, 0.30382E+01_JPRB, 0.31817E+01_JPRB, 0.34781E+01_JPRB ,&
&  0.35520E+01_JPRB, 0.34011E+01_JPRB, 0.32003E+01_JPRB, 0.32158E+01_JPRB, 0.32038E+01_JPRB, 0.30339E+01_JPRB, 0.31186E+01_JPRB ,&
&  0.40726E+01_JPRB, 0.53709E+01_JPRB, 0.63245E+01_JPRB, 0.71492E+01_JPRB, 0.76445E+01_JPRB, 0.96462E+01_JPRB, 0.87992E+01_JPRB ,&
&  0.66778E+01_JPRB, 0.51172E+01_JPRB, 0.46482E+01_JPRB, 0.50094E+01_JPRB, 0.53179E+01_JPRB, 0.57299E+01_JPRB, 0.64772E+01_JPRB ,&
&  0.63450E+01_JPRB, 0.34185E+01_JPRB, 0.25399E+01_JPRB, 0.35050E+01_JPRB, 0.23344E+01_JPRB, 0.16717E+01_JPRB, 0.23258E+01_JPRB ,&
&  0.32602E+01_JPRB, 0.39036E+01_JPRB, 0.37356E+01_JPRB, 0.33372E+01_JPRB, 0.34379E+01_JPRB, 0.32596E+01_JPRB, 0.26512E+01_JPRB ,&
&  0.20022E+01_JPRB, 0.16385E+01_JPRB, 0.14199E+01_JPRB, 0.12720E+01_JPRB, 0.11416E+01_JPRB, 0.10898E+01_JPRB, 0.10828E+01_JPRB ,&
&  0.10565E+01_JPRB, 0.10676E+01_JPRB, 0.10688E+01_JPRB, 0.10616E+01_JPRB, 0.10783E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.10484E+01_JPRB, 0.10040E+01_JPRB, 0.97799E+00_JPRB, 0.98833E+00_JPRB, 0.10798E+01_JPRB, 0.11624E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.12424E+01_JPRB, 0.12918E+01_JPRB, 0.12918E+01_JPRB, 0.14110E+01_JPRB, 0.16382E+01_JPRB, 0.18217E+01_JPRB ,&
&  0.19711E+01_JPRB, 0.21293E+01_JPRB, 0.23225E+01_JPRB, 0.26391E+01_JPRB, 0.30851E+01_JPRB, 0.36534E+01_JPRB, 0.21953E+01_JPRB ,&
&  0.18491E+01_JPRB, 0.17818E+01_JPRB, 0.16431E+01_JPRB, 0.18172E+01_JPRB, 0.17814E+01_JPRB, 0.17762E+01_JPRB, 0.17148E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.17585E+01_JPRB, 0.17640E+01_JPRB, 0.18262E+01_JPRB, 0.18164E+01_JPRB, 0.19830E+01_JPRB, 0.21116E+01_JPRB ,&
&  0.20665E+01_JPRB, 0.20454E+01_JPRB, 0.21206E+01_JPRB, 0.22025E+01_JPRB, 0.23171E+01_JPRB, 0.24302E+01_JPRB, 0.25794E+01_JPRB ,&
&  0.27405E+01_JPRB, 0.29303E+01_JPRB, 0.31950E+01_JPRB, 0.30362E+01_JPRB, 0.30083E+01_JPRB, 0.30818E+01_JPRB, 0.33016E+01_JPRB ,&
&  0.31514E+01_JPRB, 0.28108E+01_JPRB, 0.28816E+01_JPRB, 0.30449E+01_JPRB, 0.30401E+01_JPRB, 0.30544E+01_JPRB, 0.37851E+01_JPRB ,&
&  0.53700E+01_JPRB, 0.69532E+01_JPRB, 0.86875E+01_JPRB, 0.78639E+01_JPRB, 0.79249E+01_JPRB, 0.85646E+01_JPRB, 0.81459E+01_JPRB ,&
&  0.73774E+01_JPRB, 0.61014E+01_JPRB, 0.56051E+01_JPRB, 0.55983E+01_JPRB, 0.53051E+01_JPRB, 0.49023E+01_JPRB, 0.49086E+01_JPRB ,&
&  0.58936E+01_JPRB, 0.73301E+01_JPRB, 0.61341E+01_JPRB, 0.38622E+01_JPRB, 0.27207E+01_JPRB, 0.19471E+01_JPRB, 0.22217E+01_JPRB ,&
&  0.34517E+01_JPRB, 0.34309E+01_JPRB, 0.32824E+01_JPRB, 0.36878E+01_JPRB, 0.30217E+01_JPRB, 0.25810E+01_JPRB, 0.20065E+01_JPRB ,&
&  0.15564E+01_JPRB, 0.12543E+01_JPRB, 0.11455E+01_JPRB, 0.10518E+01_JPRB, 0.10366E+01_JPRB, 0.10477E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.11069E+01_JPRB, 0.11107E+01_JPRB, 0.10948E+01_JPRB, 0.10773E+01_JPRB, 0.10783E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.10512E+01_JPRB, 0.10162E+01_JPRB, 0.99067E+00_JPRB, 0.10271E+01_JPRB, 0.11266E+01_JPRB, 0.11988E+01_JPRB, 0.12301E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.12819E+01_JPRB, 0.13800E+01_JPRB, 0.14694E+01_JPRB, 0.16875E+01_JPRB, 0.19231E+01_JPRB, 0.21324E+01_JPRB ,&
&  0.22966E+01_JPRB, 0.24306E+01_JPRB, 0.26203E+01_JPRB, 0.28994E+01_JPRB, 0.32019E+01_JPRB, 0.34625E+01_JPRB, 0.26439E+01_JPRB ,&
&  0.16755E+01_JPRB, 0.22295E+01_JPRB, 0.18502E+01_JPRB, 0.20696E+01_JPRB, 0.21499E+01_JPRB, 0.23365E+01_JPRB, 0.22064E+01_JPRB ,&
&  0.22121E+01_JPRB, 0.21717E+01_JPRB, 0.22501E+01_JPRB, 0.23346E+01_JPRB, 0.23395E+01_JPRB, 0.25168E+01_JPRB, 0.26620E+01_JPRB ,&
&  0.27404E+01_JPRB, 0.28292E+01_JPRB, 0.28579E+01_JPRB, 0.28451E+01_JPRB, 0.28571E+01_JPRB, 0.28921E+01_JPRB, 0.29628E+01_JPRB ,&
&  0.31259E+01_JPRB, 0.33385E+01_JPRB, 0.33925E+01_JPRB, 0.33385E+01_JPRB, 0.31300E+01_JPRB, 0.31829E+01_JPRB, 0.32784E+01_JPRB ,&
&  0.30218E+01_JPRB, 0.26900E+01_JPRB, 0.28184E+01_JPRB, 0.29563E+01_JPRB, 0.31984E+01_JPRB, 0.41438E+01_JPRB, 0.51989E+01_JPRB ,&
&  0.68242E+01_JPRB, 0.75921E+01_JPRB, 0.96805E+01_JPRB, 0.71666E+01_JPRB, 0.68199E+01_JPRB, 0.71961E+01_JPRB, 0.63129E+01_JPRB ,&
&  0.54850E+01_JPRB, 0.54645E+01_JPRB, 0.52445E+01_JPRB, 0.54150E+01_JPRB, 0.52157E+01_JPRB, 0.46202E+01_JPRB, 0.40526E+01_JPRB ,&
&  0.41787E+01_JPRB, 0.44251E+01_JPRB, 0.37929E+01_JPRB, 0.25552E+01_JPRB, 0.21883E+01_JPRB, 0.21106E+01_JPRB, 0.28559E+01_JPRB ,&
&  0.33897E+01_JPRB, 0.33608E+01_JPRB, 0.33346E+01_JPRB, 0.31543E+01_JPRB, 0.21915E+01_JPRB, 0.17242E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.99682E+00_JPRB, 0.88918E+00_JPRB, 0.98089E+00_JPRB, 0.96462E+00_JPRB, 0.99190E+00_JPRB, 0.10328E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.10741E+01_JPRB, 0.10787E+01_JPRB, 0.10841E+01_JPRB, 0.10758E+01_JPRB, 0.10731E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.97525E+00_JPRB, 0.96333E+00_JPRB, 0.96262E+00_JPRB, 0.10558E+01_JPRB, 0.12080E+01_JPRB, 0.13362E+01_JPRB, 0.13600E+01_JPRB ,&
&  0.13823E+01_JPRB, 0.12602E+01_JPRB, 0.14018E+01_JPRB, 0.15335E+01_JPRB, 0.17447E+01_JPRB, 0.19848E+01_JPRB, 0.22528E+01_JPRB ,&
&  0.25133E+01_JPRB, 0.27584E+01_JPRB, 0.29744E+01_JPRB, 0.32198E+01_JPRB, 0.35405E+01_JPRB, 0.38674E+01_JPRB, 0.34664E+01_JPRB ,&
&  0.25207E+01_JPRB, 0.19445E+01_JPRB, 0.20720E+01_JPRB, 0.21609E+01_JPRB, 0.22382E+01_JPRB, 0.24506E+01_JPRB, 0.25615E+01_JPRB ,&
&  0.26367E+01_JPRB, 0.24715E+01_JPRB, 0.25073E+01_JPRB, 0.26226E+01_JPRB, 0.25668E+01_JPRB, 0.27473E+01_JPRB, 0.29560E+01_JPRB ,&
&  0.30905E+01_JPRB, 0.31956E+01_JPRB, 0.32207E+01_JPRB, 0.32054E+01_JPRB, 0.32185E+01_JPRB, 0.32899E+01_JPRB, 0.34079E+01_JPRB ,&
&  0.36242E+01_JPRB, 0.38866E+01_JPRB, 0.40278E+01_JPRB, 0.38125E+01_JPRB, 0.37046E+01_JPRB, 0.37717E+01_JPRB, 0.37103E+01_JPRB ,&
&  0.34962E+01_JPRB, 0.34651E+01_JPRB, 0.37272E+01_JPRB, 0.42364E+01_JPRB, 0.45615E+01_JPRB, 0.57307E+01_JPRB, 0.64506E+01_JPRB ,&
&  0.72673E+01_JPRB, 0.77179E+01_JPRB, 0.85793E+01_JPRB, 0.60591E+01_JPRB, 0.60039E+01_JPRB, 0.59149E+01_JPRB, 0.53648E+01_JPRB ,&
&  0.50067E+01_JPRB, 0.50828E+01_JPRB, 0.50323E+01_JPRB, 0.49324E+01_JPRB, 0.47912E+01_JPRB, 0.39842E+01_JPRB, 0.37408E+01_JPRB ,&
&  0.34322E+01_JPRB, 0.36626E+01_JPRB, 0.28915E+01_JPRB, 0.25022E+01_JPRB, 0.21824E+01_JPRB, 0.23197E+01_JPRB, 0.25987E+01_JPRB ,&
&  0.29902E+01_JPRB, 0.29339E+01_JPRB, 0.26182E+01_JPRB, 0.20271E+01_JPRB, 0.12611E+01_JPRB, 0.96001E+00_JPRB, 0.82461E+00_JPRB ,&
&  0.81762E+00_JPRB, 0.85195E+00_JPRB, 0.89606E+00_JPRB, 0.91834E+00_JPRB, 0.94047E+00_JPRB, 0.96991E+00_JPRB, 0.97260E+00_JPRB ,&
&  0.97043E+00_JPRB, 0.95775E+00_JPRB, 0.94834E+00_JPRB, 0.98532E+00_JPRB, 0.99922E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.84227E+00_JPRB, 0.81883E+00_JPRB, 0.82253E+00_JPRB, 0.92166E+00_JPRB, 0.10347E+01_JPRB, 0.11046E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.10709E+01_JPRB, 0.11818E+01_JPRB, 0.13204E+01_JPRB, 0.14918E+01_JPRB, 0.17201E+01_JPRB, 0.19614E+01_JPRB ,&
&  0.22192E+01_JPRB, 0.24181E+01_JPRB, 0.25653E+01_JPRB, 0.27424E+01_JPRB, 0.29755E+01_JPRB, 0.32274E+01_JPRB, 0.33403E+01_JPRB ,&
&  0.26705E+01_JPRB, 0.16951E+01_JPRB, 0.19254E+01_JPRB, 0.21489E+01_JPRB, 0.23772E+01_JPRB, 0.25664E+01_JPRB, 0.26257E+01_JPRB ,&
&  0.23888E+01_JPRB, 0.23590E+01_JPRB, 0.24801E+01_JPRB, 0.25936E+01_JPRB, 0.25271E+01_JPRB, 0.27262E+01_JPRB, 0.29381E+01_JPRB ,&
&  0.31555E+01_JPRB, 0.33157E+01_JPRB, 0.34091E+01_JPRB, 0.34960E+01_JPRB, 0.35908E+01_JPRB, 0.37362E+01_JPRB, 0.39736E+01_JPRB ,&
&  0.42861E+01_JPRB, 0.45697E+01_JPRB, 0.47952E+01_JPRB, 0.47957E+01_JPRB, 0.47621E+01_JPRB, 0.48912E+01_JPRB, 0.48646E+01_JPRB ,&
&  0.48696E+01_JPRB, 0.48749E+01_JPRB, 0.54378E+01_JPRB, 0.62337E+01_JPRB, 0.67084E+01_JPRB, 0.61534E+01_JPRB, 0.67599E+01_JPRB ,&
&  0.69346E+01_JPRB, 0.67879E+01_JPRB, 0.63107E+01_JPRB, 0.64867E+01_JPRB, 0.45128E+01_JPRB, 0.46546E+01_JPRB, 0.45044E+01_JPRB ,&
&  0.48690E+01_JPRB, 0.47706E+01_JPRB, 0.46458E+01_JPRB, 0.43682E+01_JPRB, 0.43879E+01_JPRB, 0.36098E+01_JPRB, 0.32237E+01_JPRB ,&
&  0.31007E+01_JPRB, 0.30848E+01_JPRB, 0.27586E+01_JPRB, 0.26369E+01_JPRB, 0.24016E+01_JPRB, 0.25253E+01_JPRB, 0.23805E+01_JPRB ,&
&  0.24943E+01_JPRB, 0.20811E+01_JPRB, 0.17426E+01_JPRB, 0.13342E+01_JPRB, 0.81807E+00_JPRB, 0.70745E+00_JPRB, 0.72985E+00_JPRB ,&
&  0.76331E+00_JPRB, 0.79899E+00_JPRB, 0.80506E+00_JPRB, 0.83799E+00_JPRB, 0.86047E+00_JPRB, 0.84975E+00_JPRB, 0.82494E+00_JPRB ,&
&  0.82383E+00_JPRB, 0.85082E+00_JPRB, 0.85191E+00_JPRB, 0.85564E+00_JPRB, 0.85440E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.69332E+00_JPRB, 0.69624E+00_JPRB, 0.71616E+00_JPRB, 0.76474E+00_JPRB, 0.84902E+00_JPRB, 0.90276E+00_JPRB, 0.92577E+00_JPRB ,&
&  0.92181E+00_JPRB, 0.94847E+00_JPRB, 0.10117E+01_JPRB, 0.10506E+01_JPRB, 0.11479E+01_JPRB, 0.13030E+01_JPRB, 0.14568E+01_JPRB ,&
&  0.16584E+01_JPRB, 0.18388E+01_JPRB, 0.19877E+01_JPRB, 0.22029E+01_JPRB, 0.24131E+01_JPRB, 0.26880E+01_JPRB, 0.28611E+01_JPRB ,&
&  0.28302E+01_JPRB, 0.23796E+01_JPRB, 0.22260E+01_JPRB, 0.18508E+01_JPRB, 0.17542E+01_JPRB, 0.21352E+01_JPRB, 0.20604E+01_JPRB ,&
&  0.20731E+01_JPRB, 0.21406E+01_JPRB, 0.21646E+01_JPRB, 0.22776E+01_JPRB, 0.24428E+01_JPRB, 0.26330E+01_JPRB, 0.28737E+01_JPRB ,&
&  0.31404E+01_JPRB, 0.33844E+01_JPRB, 0.36045E+01_JPRB, 0.38126E+01_JPRB, 0.39761E+01_JPRB, 0.41871E+01_JPRB, 0.44829E+01_JPRB ,&
&  0.48244E+01_JPRB, 0.51097E+01_JPRB, 0.54407E+01_JPRB, 0.58687E+01_JPRB, 0.58619E+01_JPRB, 0.60803E+01_JPRB, 0.61888E+01_JPRB ,&
&  0.62604E+01_JPRB, 0.62036E+01_JPRB, 0.62770E+01_JPRB, 0.65488E+01_JPRB, 0.65061E+01_JPRB, 0.57726E+01_JPRB, 0.57339E+01_JPRB ,&
&  0.59115E+01_JPRB, 0.54622E+01_JPRB, 0.46891E+01_JPRB, 0.52315E+01_JPRB, 0.35147E+01_JPRB, 0.39833E+01_JPRB, 0.43840E+01_JPRB ,&
&  0.42566E+01_JPRB, 0.40900E+01_JPRB, 0.38046E+01_JPRB, 0.36906E+01_JPRB, 0.36087E+01_JPRB, 0.32421E+01_JPRB, 0.30643E+01_JPRB ,&
&  0.28972E+01_JPRB, 0.23519E+01_JPRB, 0.21600E+01_JPRB, 0.23137E+01_JPRB, 0.24915E+01_JPRB, 0.22711E+01_JPRB, 0.18171E+01_JPRB ,&
&  0.15341E+01_JPRB, 0.16852E+01_JPRB, 0.15545E+01_JPRB, 0.16831E+01_JPRB, 0.11672E+01_JPRB, 0.68825E+00_JPRB, 0.68331E+00_JPRB ,&
&  0.70051E+00_JPRB, 0.71921E+00_JPRB, 0.71578E+00_JPRB, 0.75042E+00_JPRB, 0.75802E+00_JPRB, 0.74291E+00_JPRB, 0.74485E+00_JPRB ,&
&  0.75914E+00_JPRB, 0.75646E+00_JPRB, 0.75453E+00_JPRB, 0.73917E+00_JPRB, 0.72489E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.68426E+00_JPRB, 0.67498E+00_JPRB, 0.68069E+00_JPRB, 0.67900E+00_JPRB, 0.69152E+00_JPRB, 0.69414E+00_JPRB, 0.70821E+00_JPRB ,&
&  0.73420E+00_JPRB, 0.76598E+00_JPRB, 0.76409E+00_JPRB, 0.74593E+00_JPRB, 0.74848E+00_JPRB, 0.76920E+00_JPRB, 0.83148E+00_JPRB ,&
&  0.96471E+00_JPRB, 0.11095E+01_JPRB, 0.12136E+01_JPRB, 0.13398E+01_JPRB, 0.15842E+01_JPRB, 0.20217E+01_JPRB, 0.24444E+01_JPRB ,&
&  0.27632E+01_JPRB, 0.29754E+01_JPRB, 0.29465E+01_JPRB, 0.30522E+01_JPRB, 0.24400E+01_JPRB, 0.19878E+01_JPRB, 0.22285E+01_JPRB ,&
&  0.24503E+01_JPRB, 0.24202E+01_JPRB, 0.21205E+01_JPRB, 0.21201E+01_JPRB, 0.21830E+01_JPRB, 0.23472E+01_JPRB, 0.26788E+01_JPRB ,&
&  0.30066E+01_JPRB, 0.33537E+01_JPRB, 0.36465E+01_JPRB, 0.39057E+01_JPRB, 0.41289E+01_JPRB, 0.43851E+01_JPRB, 0.46356E+01_JPRB ,&
&  0.49053E+01_JPRB, 0.51199E+01_JPRB, 0.53453E+01_JPRB, 0.57844E+01_JPRB, 0.62115E+01_JPRB, 0.65846E+01_JPRB, 0.67841E+01_JPRB ,&
&  0.68203E+01_JPRB, 0.64274E+01_JPRB, 0.64523E+01_JPRB, 0.64966E+01_JPRB, 0.60762E+01_JPRB, 0.54916E+01_JPRB, 0.52418E+01_JPRB ,&
&  0.49657E+01_JPRB, 0.45242E+01_JPRB, 0.34117E+01_JPRB, 0.40634E+01_JPRB, 0.45434E+01_JPRB, 0.40216E+01_JPRB, 0.37957E+01_JPRB ,&
&  0.35150E+01_JPRB, 0.33284E+01_JPRB, 0.30894E+01_JPRB, 0.30877E+01_JPRB, 0.32515E+01_JPRB, 0.40360E+01_JPRB, 0.31357E+01_JPRB ,&
&  0.26521E+01_JPRB, 0.17451E+01_JPRB, 0.15032E+01_JPRB, 0.16823E+01_JPRB, 0.18763E+01_JPRB, 0.17344E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.14293E+01_JPRB, 0.15323E+01_JPRB, 0.20301E+01_JPRB, 0.22315E+01_JPRB, 0.72850E+00_JPRB, 0.63688E+00_JPRB ,&
&  0.63872E+00_JPRB, 0.64082E+00_JPRB, 0.63876E+00_JPRB, 0.65493E+00_JPRB, 0.65213E+00_JPRB, 0.65837E+00_JPRB, 0.67898E+00_JPRB ,&
&  0.69343E+00_JPRB, 0.69719E+00_JPRB, 0.69779E+00_JPRB, 0.68079E+00_JPRB, 0.68690E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.75911E+00_JPRB, 0.75882E+00_JPRB, 0.76398E+00_JPRB, 0.75737E+00_JPRB, 0.75068E+00_JPRB, 0.74924E+00_JPRB, 0.75268E+00_JPRB ,&
&  0.75431E+00_JPRB, 0.74355E+00_JPRB, 0.73077E+00_JPRB, 0.72185E+00_JPRB, 0.71182E+00_JPRB, 0.67889E+00_JPRB, 0.63314E+00_JPRB ,&
&  0.64517E+00_JPRB, 0.66395E+00_JPRB, 0.68214E+00_JPRB, 0.72376E+00_JPRB, 0.82360E+00_JPRB, 0.97666E+00_JPRB, 0.11916E+01_JPRB ,&
&  0.16033E+01_JPRB, 0.20561E+01_JPRB, 0.23816E+01_JPRB, 0.24739E+01_JPRB, 0.27596E+01_JPRB, 0.21702E+01_JPRB, 0.21918E+01_JPRB ,&
&  0.25317E+01_JPRB, 0.21686E+01_JPRB, 0.19989E+01_JPRB, 0.19212E+01_JPRB, 0.19724E+01_JPRB, 0.21157E+01_JPRB, 0.23479E+01_JPRB ,&
&  0.26859E+01_JPRB, 0.30396E+01_JPRB, 0.33510E+01_JPRB, 0.36659E+01_JPRB, 0.39686E+01_JPRB, 0.42785E+01_JPRB, 0.45431E+01_JPRB ,&
&  0.47228E+01_JPRB, 0.49332E+01_JPRB, 0.52165E+01_JPRB, 0.54029E+01_JPRB, 0.59557E+01_JPRB, 0.63082E+01_JPRB, 0.64744E+01_JPRB ,&
&  0.63186E+01_JPRB, 0.60012E+01_JPRB, 0.59904E+01_JPRB, 0.56457E+01_JPRB, 0.52638E+01_JPRB, 0.49114E+01_JPRB, 0.45149E+01_JPRB ,&
&  0.41315E+01_JPRB, 0.35974E+01_JPRB, 0.28341E+01_JPRB, 0.31288E+01_JPRB, 0.32554E+01_JPRB, 0.29086E+01_JPRB, 0.33200E+01_JPRB ,&
&  0.30540E+01_JPRB, 0.29203E+01_JPRB, 0.27869E+01_JPRB, 0.26458E+01_JPRB, 0.25302E+01_JPRB, 0.28983E+01_JPRB, 0.28142E+01_JPRB ,&
&  0.19549E+01_JPRB, 0.13354E+01_JPRB, 0.10923E+01_JPRB, 0.11083E+01_JPRB, 0.12466E+01_JPRB, 0.13288E+01_JPRB, 0.12408E+01_JPRB ,&
&  0.12727E+01_JPRB, 0.12130E+01_JPRB, 0.13431E+01_JPRB, 0.17170E+01_JPRB, 0.11704E+01_JPRB, 0.70917E+00_JPRB, 0.60012E+00_JPRB ,&
&  0.57778E+00_JPRB, 0.57754E+00_JPRB, 0.58544E+00_JPRB, 0.61216E+00_JPRB, 0.63653E+00_JPRB, 0.65997E+00_JPRB, 0.67974E+00_JPRB ,&
&  0.69098E+00_JPRB, 0.71190E+00_JPRB, 0.73045E+00_JPRB, 0.74850E+00_JPRB, 0.76051E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.86165E+00_JPRB, 0.87830E+00_JPRB, 0.87307E+00_JPRB, 0.88204E+00_JPRB, 0.88932E+00_JPRB, 0.90618E+00_JPRB, 0.90573E+00_JPRB ,&
&  0.90016E+00_JPRB, 0.90361E+00_JPRB, 0.88199E+00_JPRB, 0.88748E+00_JPRB, 0.88601E+00_JPRB, 0.88857E+00_JPRB, 0.87622E+00_JPRB ,&
&  0.88236E+00_JPRB, 0.90211E+00_JPRB, 0.90985E+00_JPRB, 0.91190E+00_JPRB, 0.89883E+00_JPRB, 0.87210E+00_JPRB, 0.85037E+00_JPRB ,&
&  0.90314E+00_JPRB, 0.10278E+01_JPRB, 0.11813E+01_JPRB, 0.12954E+01_JPRB, 0.15603E+01_JPRB, 0.16051E+01_JPRB, 0.17877E+01_JPRB ,&
&  0.21173E+01_JPRB, 0.19035E+01_JPRB, 0.19716E+01_JPRB, 0.17237E+01_JPRB, 0.17258E+01_JPRB, 0.17113E+01_JPRB, 0.17349E+01_JPRB ,&
&  0.18258E+01_JPRB, 0.19883E+01_JPRB, 0.21714E+01_JPRB, 0.23749E+01_JPRB, 0.25475E+01_JPRB, 0.26229E+01_JPRB, 0.26689E+01_JPRB ,&
&  0.27148E+01_JPRB, 0.29897E+01_JPRB, 0.37183E+01_JPRB, 0.43291E+01_JPRB, 0.50228E+01_JPRB, 0.53459E+01_JPRB, 0.53870E+01_JPRB ,&
&  0.53107E+01_JPRB, 0.53066E+01_JPRB, 0.45471E+01_JPRB, 0.39471E+01_JPRB, 0.37792E+01_JPRB, 0.33490E+01_JPRB, 0.31893E+01_JPRB ,&
&  0.32716E+01_JPRB, 0.30263E+01_JPRB, 0.23280E+01_JPRB, 0.24673E+01_JPRB, 0.26039E+01_JPRB, 0.24938E+01_JPRB, 0.28110E+01_JPRB ,&
&  0.24324E+01_JPRB, 0.21876E+01_JPRB, 0.20811E+01_JPRB, 0.20147E+01_JPRB, 0.19258E+01_JPRB, 0.18104E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.14186E+01_JPRB, 0.11825E+01_JPRB, 0.10841E+01_JPRB, 0.10829E+01_JPRB, 0.13003E+01_JPRB, 0.17247E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.13303E+01_JPRB, 0.12682E+01_JPRB, 0.16909E+01_JPRB, 0.24298E+01_JPRB, 0.11247E+01_JPRB, 0.75504E+00_JPRB, 0.58896E+00_JPRB ,&
&  0.59099E+00_JPRB, 0.62345E+00_JPRB, 0.65206E+00_JPRB, 0.68884E+00_JPRB, 0.70986E+00_JPRB, 0.71713E+00_JPRB, 0.73179E+00_JPRB ,&
&  0.76580E+00_JPRB, 0.79802E+00_JPRB, 0.84246E+00_JPRB, 0.84944E+00_JPRB, 0.84314E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.92663E+00_JPRB, 0.94915E+00_JPRB, 0.95354E+00_JPRB, 0.97355E+00_JPRB, 0.99914E+00_JPRB, 0.10092E+01_JPRB, 0.10133E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.10055E+01_JPRB, 0.10027E+01_JPRB, 0.10213E+01_JPRB, 0.10256E+01_JPRB, 0.10272E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10657E+01_JPRB, 0.11037E+01_JPRB, 0.11075E+01_JPRB, 0.11025E+01_JPRB, 0.11088E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.11626E+01_JPRB, 0.11912E+01_JPRB, 0.12317E+01_JPRB, 0.12684E+01_JPRB, 0.13973E+01_JPRB, 0.15717E+01_JPRB, 0.18789E+01_JPRB ,&
&  0.18711E+01_JPRB, 0.19657E+01_JPRB, 0.17892E+01_JPRB, 0.15393E+01_JPRB, 0.15290E+01_JPRB, 0.15568E+01_JPRB, 0.14904E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.14035E+01_JPRB, 0.13836E+01_JPRB, 0.13854E+01_JPRB, 0.14035E+01_JPRB, 0.13718E+01_JPRB, 0.13579E+01_JPRB ,&
&  0.13045E+01_JPRB, 0.13444E+01_JPRB, 0.17475E+01_JPRB, 0.22940E+01_JPRB, 0.28459E+01_JPRB, 0.33171E+01_JPRB, 0.29887E+01_JPRB ,&
&  0.28574E+01_JPRB, 0.30599E+01_JPRB, 0.28099E+01_JPRB, 0.25809E+01_JPRB, 0.23208E+01_JPRB, 0.20706E+01_JPRB, 0.19406E+01_JPRB ,&
&  0.21274E+01_JPRB, 0.22667E+01_JPRB, 0.20392E+01_JPRB, 0.22811E+01_JPRB, 0.23097E+01_JPRB, 0.22819E+01_JPRB, 0.20174E+01_JPRB ,&
&  0.17326E+01_JPRB, 0.15957E+01_JPRB, 0.15639E+01_JPRB, 0.15543E+01_JPRB, 0.15645E+01_JPRB, 0.14672E+01_JPRB, 0.13131E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.10934E+01_JPRB, 0.10604E+01_JPRB, 0.10144E+01_JPRB, 0.13130E+01_JPRB, 0.15403E+01_JPRB, 0.19859E+01_JPRB ,&
&  0.14105E+01_JPRB, 0.12907E+01_JPRB, 0.11266E+01_JPRB, 0.17442E+01_JPRB, 0.97809E+00_JPRB, 0.62449E+00_JPRB, 0.61221E+00_JPRB ,&
&  0.68549E+00_JPRB, 0.72916E+00_JPRB, 0.75034E+00_JPRB, 0.78263E+00_JPRB, 0.79621E+00_JPRB, 0.79546E+00_JPRB, 0.80842E+00_JPRB ,&
&  0.85092E+00_JPRB, 0.86434E+00_JPRB, 0.87726E+00_JPRB, 0.88196E+00_JPRB, 0.90310E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.95919E+00_JPRB, 0.99237E+00_JPRB, 0.10129E+01_JPRB, 0.10334E+01_JPRB, 0.10598E+01_JPRB, 0.10758E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10916E+01_JPRB, 0.11019E+01_JPRB, 0.11180E+01_JPRB, 0.11267E+01_JPRB, 0.11234E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11508E+01_JPRB, 0.11821E+01_JPRB, 0.12174E+01_JPRB, 0.12127E+01_JPRB, 0.12112E+01_JPRB, 0.12338E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.12616E+01_JPRB, 0.12724E+01_JPRB, 0.12898E+01_JPRB, 0.13219E+01_JPRB, 0.13376E+01_JPRB, 0.14557E+01_JPRB ,&
&  0.14904E+01_JPRB, 0.20999E+01_JPRB, 0.19667E+01_JPRB, 0.17061E+01_JPRB, 0.16906E+01_JPRB, 0.15707E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.15182E+01_JPRB, 0.14906E+01_JPRB, 0.14606E+01_JPRB, 0.14710E+01_JPRB, 0.14942E+01_JPRB, 0.14606E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.15195E+01_JPRB, 0.16551E+01_JPRB, 0.18347E+01_JPRB, 0.20117E+01_JPRB, 0.20765E+01_JPRB, 0.19225E+01_JPRB ,&
&  0.18147E+01_JPRB, 0.16390E+01_JPRB, 0.13606E+01_JPRB, 0.13845E+01_JPRB, 0.13520E+01_JPRB, 0.11136E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.13963E+01_JPRB, 0.15966E+01_JPRB, 0.19952E+01_JPRB, 0.19939E+01_JPRB, 0.19113E+01_JPRB, 0.19176E+01_JPRB, 0.16258E+01_JPRB ,&
&  0.14302E+01_JPRB, 0.13102E+01_JPRB, 0.13002E+01_JPRB, 0.12605E+01_JPRB, 0.12754E+01_JPRB, 0.12475E+01_JPRB, 0.11338E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.10097E+01_JPRB, 0.10182E+01_JPRB, 0.10751E+01_JPRB, 0.11687E+01_JPRB, 0.21050E+01_JPRB, 0.32791E+01_JPRB ,&
&  0.16360E+01_JPRB, 0.11656E+01_JPRB, 0.72667E+00_JPRB, 0.91531E+00_JPRB, 0.73668E+00_JPRB, 0.63410E+00_JPRB, 0.68002E+00_JPRB ,&
&  0.79905E+00_JPRB, 0.85227E+00_JPRB, 0.86423E+00_JPRB, 0.86018E+00_JPRB, 0.84102E+00_JPRB, 0.82423E+00_JPRB, 0.81876E+00_JPRB ,&
&  0.84001E+00_JPRB, 0.86175E+00_JPRB, 0.88622E+00_JPRB, 0.90284E+00_JPRB, 0.92600E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.97457E+00_JPRB, 0.10071E+01_JPRB, 0.10331E+01_JPRB, 0.10628E+01_JPRB, 0.10945E+01_JPRB, 0.11236E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11724E+01_JPRB, 0.11907E+01_JPRB, 0.12081E+01_JPRB, 0.12279E+01_JPRB, 0.12513E+01_JPRB, 0.12698E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.13305E+01_JPRB, 0.13620E+01_JPRB, 0.13991E+01_JPRB, 0.14025E+01_JPRB, 0.14077E+01_JPRB, 0.14429E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.14857E+01_JPRB, 0.15058E+01_JPRB, 0.15373E+01_JPRB, 0.15782E+01_JPRB, 0.16352E+01_JPRB, 0.17569E+01_JPRB, 0.17234E+01_JPRB ,&
&  0.19520E+01_JPRB, 0.21879E+01_JPRB, 0.21037E+01_JPRB, 0.19035E+01_JPRB, 0.18207E+01_JPRB, 0.18043E+01_JPRB, 0.18433E+01_JPRB ,&
&  0.19149E+01_JPRB, 0.19086E+01_JPRB, 0.19160E+01_JPRB, 0.19089E+01_JPRB, 0.19221E+01_JPRB, 0.19702E+01_JPRB, 0.20160E+01_JPRB ,&
&  0.20777E+01_JPRB, 0.21586E+01_JPRB, 0.22736E+01_JPRB, 0.24248E+01_JPRB, 0.25921E+01_JPRB, 0.25984E+01_JPRB, 0.25267E+01_JPRB ,&
&  0.23430E+01_JPRB, 0.21427E+01_JPRB, 0.16520E+01_JPRB, 0.15405E+01_JPRB, 0.14336E+01_JPRB, 0.12346E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.13901E+01_JPRB, 0.14547E+01_JPRB, 0.16040E+01_JPRB, 0.18223E+01_JPRB, 0.16985E+01_JPRB, 0.15678E+01_JPRB, 0.13840E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.11416E+01_JPRB, 0.11068E+01_JPRB, 0.10621E+01_JPRB, 0.10547E+01_JPRB, 0.10570E+01_JPRB, 0.97221E+00_JPRB ,&
&  0.93992E+00_JPRB, 0.94712E+00_JPRB, 0.97253E+00_JPRB, 0.10357E+01_JPRB, 0.12046E+01_JPRB, 0.13352E+01_JPRB, 0.20293E+01_JPRB ,&
&  0.14670E+01_JPRB, 0.88733E+00_JPRB, 0.87372E+00_JPRB, 0.81487E+00_JPRB, 0.74348E+00_JPRB, 0.68922E+00_JPRB, 0.75835E+00_JPRB ,&
&  0.91142E+00_JPRB, 0.10356E+01_JPRB, 0.10666E+01_JPRB, 0.10150E+01_JPRB, 0.92829E+00_JPRB, 0.88032E+00_JPRB, 0.85681E+00_JPRB ,&
&  0.87330E+00_JPRB, 0.88928E+00_JPRB, 0.90840E+00_JPRB, 0.91955E+00_JPRB, 0.94053E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.96445E+00_JPRB, 0.10006E+01_JPRB, 0.10315E+01_JPRB, 0.10624E+01_JPRB, 0.10927E+01_JPRB, 0.11227E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.11788E+01_JPRB, 0.12102E+01_JPRB, 0.12462E+01_JPRB, 0.12822E+01_JPRB, 0.13159E+01_JPRB, 0.13547E+01_JPRB, 0.13893E+01_JPRB ,&
&  0.14336E+01_JPRB, 0.14668E+01_JPRB, 0.15048E+01_JPRB, 0.15171E+01_JPRB, 0.15321E+01_JPRB, 0.15633E+01_JPRB, 0.15920E+01_JPRB ,&
&  0.16003E+01_JPRB, 0.16173E+01_JPRB, 0.16419E+01_JPRB, 0.16810E+01_JPRB, 0.17032E+01_JPRB, 0.18089E+01_JPRB, 0.16716E+01_JPRB ,&
&  0.21419E+01_JPRB, 0.23966E+01_JPRB, 0.23048E+01_JPRB, 0.21271E+01_JPRB, 0.20380E+01_JPRB, 0.20854E+01_JPRB, 0.21530E+01_JPRB ,&
&  0.22438E+01_JPRB, 0.23754E+01_JPRB, 0.23350E+01_JPRB, 0.23259E+01_JPRB, 0.23591E+01_JPRB, 0.24473E+01_JPRB, 0.25780E+01_JPRB ,&
&  0.27130E+01_JPRB, 0.28611E+01_JPRB, 0.30206E+01_JPRB, 0.31777E+01_JPRB, 0.33278E+01_JPRB, 0.34287E+01_JPRB, 0.34517E+01_JPRB ,&
&  0.33558E+01_JPRB, 0.32037E+01_JPRB, 0.28527E+01_JPRB, 0.27948E+01_JPRB, 0.26216E+01_JPRB, 0.21398E+01_JPRB, 0.16812E+01_JPRB ,&
&  0.15898E+01_JPRB, 0.15739E+01_JPRB, 0.15135E+01_JPRB, 0.16440E+01_JPRB, 0.15478E+01_JPRB, 0.14154E+01_JPRB, 0.12612E+01_JPRB ,&
&  0.11263E+01_JPRB, 0.10208E+01_JPRB, 0.99596E+00_JPRB, 0.93840E+00_JPRB, 0.89096E+00_JPRB, 0.86104E+00_JPRB, 0.83994E+00_JPRB ,&
&  0.85664E+00_JPRB, 0.88880E+00_JPRB, 0.93681E+00_JPRB, 0.99394E+00_JPRB, 0.10518E+01_JPRB, 0.10565E+01_JPRB, 0.16359E+01_JPRB ,&
&  0.12788E+01_JPRB, 0.10260E+01_JPRB, 0.11931E+01_JPRB, 0.67652E+00_JPRB, 0.74277E+00_JPRB, 0.80338E+00_JPRB, 0.93393E+00_JPRB ,&
&  0.10194E+01_JPRB, 0.13997E+01_JPRB, 0.18767E+01_JPRB, 0.15046E+01_JPRB, 0.12304E+01_JPRB, 0.96088E+00_JPRB, 0.87495E+00_JPRB ,&
&  0.87921E+00_JPRB, 0.88754E+00_JPRB, 0.90221E+00_JPRB, 0.91164E+00_JPRB, 0.93178E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.94265E+00_JPRB, 0.97155E+00_JPRB, 0.10050E+01_JPRB, 0.10359E+01_JPRB, 0.10595E+01_JPRB, 0.10813E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.11358E+01_JPRB, 0.11583E+01_JPRB, 0.11908E+01_JPRB, 0.12288E+01_JPRB, 0.12726E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.13847E+01_JPRB, 0.14454E+01_JPRB, 0.15230E+01_JPRB, 0.15889E+01_JPRB, 0.16414E+01_JPRB, 0.16967E+01_JPRB, 0.17542E+01_JPRB ,&
&  0.18292E+01_JPRB, 0.18805E+01_JPRB, 0.19249E+01_JPRB, 0.20058E+01_JPRB, 0.20781E+01_JPRB, 0.21789E+01_JPRB, 0.18446E+01_JPRB ,&
&  0.23527E+01_JPRB, 0.27268E+01_JPRB, 0.25646E+01_JPRB, 0.23281E+01_JPRB, 0.22161E+01_JPRB, 0.22396E+01_JPRB, 0.22690E+01_JPRB ,&
&  0.23298E+01_JPRB, 0.23732E+01_JPRB, 0.23261E+01_JPRB, 0.25290E+01_JPRB, 0.26884E+01_JPRB, 0.27819E+01_JPRB, 0.29870E+01_JPRB ,&
&  0.31970E+01_JPRB, 0.34082E+01_JPRB, 0.36016E+01_JPRB, 0.37817E+01_JPRB, 0.39424E+01_JPRB, 0.41131E+01_JPRB, 0.41841E+01_JPRB ,&
&  0.41984E+01_JPRB, 0.42005E+01_JPRB, 0.41419E+01_JPRB, 0.37539E+01_JPRB, 0.35676E+01_JPRB, 0.33253E+01_JPRB, 0.26456E+01_JPRB ,&
&  0.20614E+01_JPRB, 0.15925E+01_JPRB, 0.14317E+01_JPRB, 0.14363E+01_JPRB, 0.13247E+01_JPRB, 0.11859E+01_JPRB, 0.10666E+01_JPRB ,&
&  0.95918E+00_JPRB, 0.89369E+00_JPRB, 0.89321E+00_JPRB, 0.84480E+00_JPRB, 0.73871E+00_JPRB, 0.75666E+00_JPRB, 0.78632E+00_JPRB ,&
&  0.78818E+00_JPRB, 0.81934E+00_JPRB, 0.86236E+00_JPRB, 0.89150E+00_JPRB, 0.97585E+00_JPRB, 0.11643E+01_JPRB, 0.12852E+01_JPRB ,&
&  0.13367E+01_JPRB, 0.99790E+00_JPRB, 0.94147E+00_JPRB, 0.94598E+00_JPRB, 0.84398E+00_JPRB, 0.10663E+01_JPRB, 0.14234E+01_JPRB ,&
&  0.20364E+01_JPRB, 0.21579E+01_JPRB, 0.27478E+01_JPRB, 0.42393E+01_JPRB, 0.32291E+01_JPRB, 0.21777E+01_JPRB, 0.89281E+00_JPRB ,&
&  0.90028E+00_JPRB, 0.89881E+00_JPRB, 0.92054E+00_JPRB, 0.92415E+00_JPRB, 0.93592E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.80546E+00_JPRB, 0.80919E+00_JPRB, 0.84267E+00_JPRB, 0.87984E+00_JPRB, 0.91507E+00_JPRB, 0.94897E+00_JPRB, 0.97650E+00_JPRB ,&
&  0.10153E+01_JPRB, 0.10487E+01_JPRB, 0.10643E+01_JPRB, 0.10708E+01_JPRB, 0.10905E+01_JPRB, 0.11090E+01_JPRB, 0.11467E+01_JPRB ,&
&  0.12021E+01_JPRB, 0.12650E+01_JPRB, 0.13517E+01_JPRB, 0.14176E+01_JPRB, 0.14885E+01_JPRB, 0.15860E+01_JPRB, 0.16826E+01_JPRB ,&
&  0.17879E+01_JPRB, 0.19182E+01_JPRB, 0.20277E+01_JPRB, 0.21387E+01_JPRB, 0.22546E+01_JPRB, 0.22573E+01_JPRB, 0.17227E+01_JPRB ,&
&  0.24932E+01_JPRB, 0.28364E+01_JPRB, 0.27576E+01_JPRB, 0.24929E+01_JPRB, 0.23314E+01_JPRB, 0.22826E+01_JPRB, 0.23025E+01_JPRB ,&
&  0.23177E+01_JPRB, 0.22784E+01_JPRB, 0.22389E+01_JPRB, 0.22912E+01_JPRB, 0.27349E+01_JPRB, 0.28030E+01_JPRB, 0.30292E+01_JPRB ,&
&  0.32577E+01_JPRB, 0.34768E+01_JPRB, 0.36630E+01_JPRB, 0.38237E+01_JPRB, 0.39476E+01_JPRB, 0.40984E+01_JPRB, 0.42620E+01_JPRB ,&
&  0.44115E+01_JPRB, 0.44984E+01_JPRB, 0.46399E+01_JPRB, 0.42195E+01_JPRB, 0.40139E+01_JPRB, 0.43130E+01_JPRB, 0.38040E+01_JPRB ,&
&  0.24028E+01_JPRB, 0.16164E+01_JPRB, 0.15694E+01_JPRB, 0.14300E+01_JPRB, 0.12373E+01_JPRB, 0.10807E+01_JPRB, 0.97015E+00_JPRB ,&
&  0.89384E+00_JPRB, 0.83762E+00_JPRB, 0.80861E+00_JPRB, 0.76097E+00_JPRB, 0.74687E+00_JPRB, 0.76491E+00_JPRB, 0.76644E+00_JPRB ,&
&  0.79771E+00_JPRB, 0.77082E+00_JPRB, 0.76876E+00_JPRB, 0.79528E+00_JPRB, 0.89409E+00_JPRB, 0.11082E+01_JPRB, 0.15005E+01_JPRB ,&
&  0.16387E+01_JPRB, 0.97496E+00_JPRB, 0.93013E+00_JPRB, 0.91674E+00_JPRB, 0.95657E+00_JPRB, 0.10785E+01_JPRB, 0.12773E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.21286E+01_JPRB, 0.20253E+01_JPRB, 0.15972E+01_JPRB, 0.19710E+01_JPRB, 0.16720E+01_JPRB, 0.79363E+00_JPRB ,&
&  0.80022E+00_JPRB, 0.77714E+00_JPRB, 0.78699E+00_JPRB, 0.79844E+00_JPRB, 0.80492E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.65781E+00_JPRB, 0.64773E+00_JPRB, 0.65918E+00_JPRB, 0.68347E+00_JPRB, 0.70772E+00_JPRB, 0.75161E+00_JPRB, 0.80493E+00_JPRB ,&
&  0.87026E+00_JPRB, 0.91751E+00_JPRB, 0.93811E+00_JPRB, 0.94897E+00_JPRB, 0.96589E+00_JPRB, 0.98371E+00_JPRB, 0.10109E+01_JPRB ,&
&  0.10531E+01_JPRB, 0.10960E+01_JPRB, 0.11542E+01_JPRB, 0.12043E+01_JPRB, 0.12571E+01_JPRB, 0.13551E+01_JPRB, 0.14702E+01_JPRB ,&
&  0.16094E+01_JPRB, 0.17588E+01_JPRB, 0.19317E+01_JPRB, 0.21139E+01_JPRB, 0.23041E+01_JPRB, 0.23926E+01_JPRB, 0.24373E+01_JPRB ,&
&  0.18437E+01_JPRB, 0.26451E+01_JPRB, 0.27451E+01_JPRB, 0.25712E+01_JPRB, 0.24026E+01_JPRB, 0.23733E+01_JPRB, 0.23326E+01_JPRB ,&
&  0.22801E+01_JPRB, 0.21135E+01_JPRB, 0.21177E+01_JPRB, 0.22116E+01_JPRB, 0.24877E+01_JPRB, 0.26349E+01_JPRB, 0.28377E+01_JPRB ,&
&  0.30318E+01_JPRB, 0.31969E+01_JPRB, 0.33432E+01_JPRB, 0.34901E+01_JPRB, 0.36748E+01_JPRB, 0.39218E+01_JPRB, 0.42375E+01_JPRB ,&
&  0.45230E+01_JPRB, 0.47410E+01_JPRB, 0.48874E+01_JPRB, 0.45773E+01_JPRB, 0.42531E+01_JPRB, 0.47303E+01_JPRB, 0.44915E+01_JPRB ,&
&  0.27412E+01_JPRB, 0.17294E+01_JPRB, 0.17426E+01_JPRB, 0.16916E+01_JPRB, 0.13918E+01_JPRB, 0.12156E+01_JPRB, 0.10680E+01_JPRB ,&
&  0.89393E+00_JPRB, 0.79003E+00_JPRB, 0.79622E+00_JPRB, 0.78719E+00_JPRB, 0.79428E+00_JPRB, 0.79578E+00_JPRB, 0.81002E+00_JPRB ,&
&  0.84111E+00_JPRB, 0.80638E+00_JPRB, 0.79025E+00_JPRB, 0.78927E+00_JPRB, 0.82301E+00_JPRB, 0.87018E+00_JPRB, 0.91629E+00_JPRB ,&
&  0.98395E+00_JPRB, 0.91684E+00_JPRB, 0.90598E+00_JPRB, 0.93366E+00_JPRB, 0.95728E+00_JPRB, 0.99643E+00_JPRB, 0.10493E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.11328E+01_JPRB, 0.11600E+01_JPRB, 0.11472E+01_JPRB, 0.10432E+01_JPRB, 0.96069E+00_JPRB, 0.71856E+00_JPRB ,&
&  0.70204E+00_JPRB, 0.68153E+00_JPRB, 0.68028E+00_JPRB, 0.67736E+00_JPRB, 0.67668E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.57613E+00_JPRB, 0.57092E+00_JPRB, 0.58053E+00_JPRB, 0.59390E+00_JPRB, 0.61852E+00_JPRB, 0.65973E+00_JPRB, 0.71754E+00_JPRB ,&
&  0.78130E+00_JPRB, 0.81996E+00_JPRB, 0.85261E+00_JPRB, 0.84952E+00_JPRB, 0.85609E+00_JPRB, 0.88077E+00_JPRB, 0.90103E+00_JPRB ,&
&  0.94456E+00_JPRB, 0.98952E+00_JPRB, 0.10180E+01_JPRB, 0.10305E+01_JPRB, 0.10513E+01_JPRB, 0.11027E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.14041E+01_JPRB, 0.15890E+01_JPRB, 0.18383E+01_JPRB, 0.21130E+01_JPRB, 0.22379E+01_JPRB, 0.22630E+01_JPRB ,&
&  0.11181E+01_JPRB, 0.19349E+01_JPRB, 0.26397E+01_JPRB, 0.26433E+01_JPRB, 0.25675E+01_JPRB, 0.24992E+01_JPRB, 0.24548E+01_JPRB ,&
&  0.22018E+01_JPRB, 0.19092E+01_JPRB, 0.18304E+01_JPRB, 0.21435E+01_JPRB, 0.21794E+01_JPRB, 0.23711E+01_JPRB, 0.25414E+01_JPRB ,&
&  0.27262E+01_JPRB, 0.28849E+01_JPRB, 0.29892E+01_JPRB, 0.30844E+01_JPRB, 0.32420E+01_JPRB, 0.34787E+01_JPRB, 0.37740E+01_JPRB ,&
&  0.41546E+01_JPRB, 0.46207E+01_JPRB, 0.49503E+01_JPRB, 0.47608E+01_JPRB, 0.46438E+01_JPRB, 0.49783E+01_JPRB, 0.62119E+01_JPRB ,&
&  0.37827E+01_JPRB, 0.20951E+01_JPRB, 0.20597E+01_JPRB, 0.20473E+01_JPRB, 0.16429E+01_JPRB, 0.14447E+01_JPRB, 0.12785E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.92702E+00_JPRB, 0.87543E+00_JPRB, 0.84354E+00_JPRB, 0.85518E+00_JPRB, 0.88704E+00_JPRB, 0.94640E+00_JPRB ,&
&  0.97624E+00_JPRB, 0.95891E+00_JPRB, 0.93184E+00_JPRB, 0.92655E+00_JPRB, 0.91760E+00_JPRB, 0.92380E+00_JPRB, 0.91977E+00_JPRB ,&
&  0.93568E+00_JPRB, 0.91714E+00_JPRB, 0.91422E+00_JPRB, 0.94358E+00_JPRB, 0.98694E+00_JPRB, 0.99858E+00_JPRB, 0.99998E+00_JPRB ,&
&  0.96577E+00_JPRB, 0.98108E+00_JPRB, 0.98252E+00_JPRB, 0.95274E+00_JPRB, 0.90540E+00_JPRB, 0.84355E+00_JPRB, 0.74126E+00_JPRB ,&
&  0.69020E+00_JPRB, 0.63172E+00_JPRB, 0.59500E+00_JPRB, 0.57840E+00_JPRB, 0.58699E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.55668E+00_JPRB, 0.55174E+00_JPRB, 0.55423E+00_JPRB, 0.58123E+00_JPRB, 0.58042E+00_JPRB, 0.60164E+00_JPRB, 0.63167E+00_JPRB ,&
&  0.69475E+00_JPRB, 0.75105E+00_JPRB, 0.80368E+00_JPRB, 0.80423E+00_JPRB, 0.80241E+00_JPRB, 0.78596E+00_JPRB, 0.79177E+00_JPRB ,&
&  0.81117E+00_JPRB, 0.84257E+00_JPRB, 0.87782E+00_JPRB, 0.89709E+00_JPRB, 0.91288E+00_JPRB, 0.93112E+00_JPRB, 0.95558E+00_JPRB ,&
&  0.99263E+00_JPRB, 0.10501E+01_JPRB, 0.11629E+01_JPRB, 0.13791E+01_JPRB, 0.17338E+01_JPRB, 0.21226E+01_JPRB, 0.24233E+01_JPRB ,&
&  0.16713E+01_JPRB, 0.69957E+00_JPRB, 0.20131E+01_JPRB, 0.26726E+01_JPRB, 0.26612E+01_JPRB, 0.25699E+01_JPRB, 0.24604E+01_JPRB ,&
&  0.21304E+01_JPRB, 0.19170E+01_JPRB, 0.16635E+01_JPRB, 0.18198E+01_JPRB, 0.17516E+01_JPRB, 0.18791E+01_JPRB, 0.20779E+01_JPRB ,&
&  0.22927E+01_JPRB, 0.24925E+01_JPRB, 0.26785E+01_JPRB, 0.27572E+01_JPRB, 0.27968E+01_JPRB, 0.28968E+01_JPRB, 0.30982E+01_JPRB ,&
&  0.34541E+01_JPRB, 0.40488E+01_JPRB, 0.48285E+01_JPRB, 0.51836E+01_JPRB, 0.60522E+01_JPRB, 0.57114E+01_JPRB, 0.62076E+01_JPRB ,&
&  0.39837E+01_JPRB, 0.26067E+01_JPRB, 0.24153E+01_JPRB, 0.24122E+01_JPRB, 0.19563E+01_JPRB, 0.15980E+01_JPRB, 0.15652E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.10808E+01_JPRB, 0.10231E+01_JPRB, 0.97475E+00_JPRB, 0.94164E+00_JPRB, 0.97932E+00_JPRB, 0.10465E+01_JPRB ,&
&  0.10957E+01_JPRB, 0.11038E+01_JPRB, 0.10874E+01_JPRB, 0.10740E+01_JPRB, 0.10581E+01_JPRB, 0.10260E+01_JPRB, 0.99592E+00_JPRB ,&
&  0.99878E+00_JPRB, 0.10064E+01_JPRB, 0.10165E+01_JPRB, 0.10269E+01_JPRB, 0.10382E+01_JPRB, 0.10152E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.99976E+00_JPRB, 0.10141E+01_JPRB, 0.99108E+00_JPRB, 0.99013E+00_JPRB, 0.95476E+00_JPRB, 0.90178E+00_JPRB, 0.85966E+00_JPRB ,&
&  0.78990E+00_JPRB, 0.68940E+00_JPRB, 0.61458E+00_JPRB, 0.58623E+00_JPRB, 0.56351E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.60306E+00_JPRB, 0.58691E+00_JPRB, 0.58914E+00_JPRB, 0.60433E+00_JPRB, 0.61861E+00_JPRB, 0.63131E+00_JPRB, 0.61691E+00_JPRB ,&
&  0.63672E+00_JPRB, 0.68270E+00_JPRB, 0.71109E+00_JPRB, 0.74159E+00_JPRB, 0.73704E+00_JPRB, 0.72230E+00_JPRB, 0.71156E+00_JPRB ,&
&  0.70806E+00_JPRB, 0.71300E+00_JPRB, 0.73473E+00_JPRB, 0.74692E+00_JPRB, 0.75904E+00_JPRB, 0.77568E+00_JPRB, 0.78570E+00_JPRB ,&
&  0.79521E+00_JPRB, 0.81424E+00_JPRB, 0.85657E+00_JPRB, 0.94885E+00_JPRB, 0.11273E+01_JPRB, 0.14703E+01_JPRB, 0.19863E+01_JPRB ,&
&  0.29294E+01_JPRB, 0.21180E+01_JPRB, 0.94465E+00_JPRB, 0.21584E+01_JPRB, 0.27945E+01_JPRB, 0.28361E+01_JPRB, 0.24152E+01_JPRB ,&
&  0.22643E+01_JPRB, 0.19375E+01_JPRB, 0.16858E+01_JPRB, 0.17269E+01_JPRB, 0.14797E+01_JPRB, 0.14075E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.16259E+01_JPRB, 0.18725E+01_JPRB, 0.21432E+01_JPRB, 0.23324E+01_JPRB, 0.24348E+01_JPRB, 0.24830E+01_JPRB, 0.25604E+01_JPRB ,&
&  0.27929E+01_JPRB, 0.31696E+01_JPRB, 0.38510E+01_JPRB, 0.50491E+01_JPRB, 0.74384E+01_JPRB, 0.69569E+01_JPRB, 0.69256E+01_JPRB ,&
&  0.42574E+01_JPRB, 0.35067E+01_JPRB, 0.31964E+01_JPRB, 0.28740E+01_JPRB, 0.22539E+01_JPRB, 0.19506E+01_JPRB, 0.19478E+01_JPRB ,&
&  0.15245E+01_JPRB, 0.12570E+01_JPRB, 0.11587E+01_JPRB, 0.10840E+01_JPRB, 0.10463E+01_JPRB, 0.10593E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.11638E+01_JPRB, 0.11629E+01_JPRB, 0.11411E+01_JPRB, 0.11232E+01_JPRB, 0.11107E+01_JPRB, 0.10948E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10650E+01_JPRB, 0.10856E+01_JPRB, 0.10741E+01_JPRB, 0.10373E+01_JPRB, 0.10086E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10427E+01_JPRB, 0.10244E+01_JPRB, 0.10363E+01_JPRB, 0.10060E+01_JPRB, 0.97303E+00_JPRB, 0.94035E+00_JPRB ,&
&  0.89152E+00_JPRB, 0.79940E+00_JPRB, 0.70198E+00_JPRB, 0.61248E+00_JPRB, 0.60192E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.68372E+00_JPRB, 0.66624E+00_JPRB, 0.67174E+00_JPRB, 0.66610E+00_JPRB, 0.65663E+00_JPRB, 0.65826E+00_JPRB, 0.65374E+00_JPRB ,&
&  0.64229E+00_JPRB, 0.63569E+00_JPRB, 0.62620E+00_JPRB, 0.65924E+00_JPRB, 0.67796E+00_JPRB, 0.66586E+00_JPRB, 0.65375E+00_JPRB ,&
&  0.64325E+00_JPRB, 0.64385E+00_JPRB, 0.65250E+00_JPRB, 0.65945E+00_JPRB, 0.66466E+00_JPRB, 0.67567E+00_JPRB, 0.68483E+00_JPRB ,&
&  0.68192E+00_JPRB, 0.68342E+00_JPRB, 0.69666E+00_JPRB, 0.73000E+00_JPRB, 0.79988E+00_JPRB, 0.93449E+00_JPRB, 0.11955E+01_JPRB ,&
&  0.18546E+01_JPRB, 0.19263E+01_JPRB, 0.86644E+00_JPRB, 0.16920E+01_JPRB, 0.28179E+01_JPRB, 0.29825E+01_JPRB, 0.28306E+01_JPRB ,&
&  0.25894E+01_JPRB, 0.20428E+01_JPRB, 0.18100E+01_JPRB, 0.17662E+01_JPRB, 0.15013E+01_JPRB, 0.13070E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.11569E+01_JPRB, 0.12345E+01_JPRB, 0.14301E+01_JPRB, 0.16682E+01_JPRB, 0.18801E+01_JPRB, 0.20086E+01_JPRB, 0.20773E+01_JPRB ,&
&  0.21756E+01_JPRB, 0.23830E+01_JPRB, 0.28249E+01_JPRB, 0.34386E+01_JPRB, 0.60071E+01_JPRB, 0.74041E+01_JPRB, 0.78262E+01_JPRB ,&
&  0.57059E+01_JPRB, 0.48751E+01_JPRB, 0.41821E+01_JPRB, 0.35038E+01_JPRB, 0.26942E+01_JPRB, 0.24575E+01_JPRB, 0.24125E+01_JPRB ,&
&  0.19442E+01_JPRB, 0.15472E+01_JPRB, 0.13641E+01_JPRB, 0.12427E+01_JPRB, 0.11608E+01_JPRB, 0.11285E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.11436E+01_JPRB, 0.11566E+01_JPRB, 0.11349E+01_JPRB, 0.11183E+01_JPRB, 0.11185E+01_JPRB, 0.11179E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.10988E+01_JPRB, 0.11130E+01_JPRB, 0.11014E+01_JPRB, 0.11012E+01_JPRB, 0.11083E+01_JPRB, 0.10686E+01_JPRB ,&
&  0.10293E+01_JPRB, 0.10299E+01_JPRB, 0.10223E+01_JPRB, 0.99535E+00_JPRB, 0.10428E+01_JPRB, 0.10333E+01_JPRB, 0.10078E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.11253E+01_JPRB, 0.10648E+01_JPRB, 0.74924E+00_JPRB, 0.69617E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.75186E+00_JPRB, 0.72518E+00_JPRB, 0.72116E+00_JPRB, 0.71414E+00_JPRB, 0.69944E+00_JPRB, 0.68897E+00_JPRB, 0.68757E+00_JPRB ,&
&  0.66983E+00_JPRB, 0.64151E+00_JPRB, 0.61376E+00_JPRB, 0.61086E+00_JPRB, 0.62306E+00_JPRB, 0.63492E+00_JPRB, 0.62420E+00_JPRB ,&
&  0.60813E+00_JPRB, 0.60518E+00_JPRB, 0.61997E+00_JPRB, 0.62785E+00_JPRB, 0.62813E+00_JPRB, 0.63224E+00_JPRB, 0.63185E+00_JPRB ,&
&  0.62976E+00_JPRB, 0.62432E+00_JPRB, 0.62499E+00_JPRB, 0.63889E+00_JPRB, 0.66889E+00_JPRB, 0.73104E+00_JPRB, 0.83467E+00_JPRB ,&
&  0.12008E+01_JPRB, 0.15147E+01_JPRB, 0.19101E+01_JPRB, 0.29109E+01_JPRB, 0.36581E+01_JPRB, 0.35195E+01_JPRB, 0.33518E+01_JPRB ,&
&  0.31704E+01_JPRB, 0.26675E+01_JPRB, 0.22346E+01_JPRB, 0.19016E+01_JPRB, 0.15803E+01_JPRB, 0.13460E+01_JPRB, 0.11880E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.10528E+01_JPRB, 0.10674E+01_JPRB, 0.11514E+01_JPRB, 0.12847E+01_JPRB, 0.14290E+01_JPRB, 0.15329E+01_JPRB ,&
&  0.16213E+01_JPRB, 0.17685E+01_JPRB, 0.20609E+01_JPRB, 0.24507E+01_JPRB, 0.39045E+01_JPRB, 0.62903E+01_JPRB, 0.75464E+01_JPRB ,&
&  0.74988E+01_JPRB, 0.66283E+01_JPRB, 0.54152E+01_JPRB, 0.43256E+01_JPRB, 0.31444E+01_JPRB, 0.31836E+01_JPRB, 0.28136E+01_JPRB ,&
&  0.23494E+01_JPRB, 0.19671E+01_JPRB, 0.16779E+01_JPRB, 0.14775E+01_JPRB, 0.13306E+01_JPRB, 0.12209E+01_JPRB, 0.11630E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.11383E+01_JPRB, 0.11208E+01_JPRB, 0.10983E+01_JPRB, 0.11038E+01_JPRB, 0.11167E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.11081E+01_JPRB, 0.10766E+01_JPRB, 0.10941E+01_JPRB, 0.11196E+01_JPRB, 0.11033E+01_JPRB, 0.10627E+01_JPRB ,&
&  0.10155E+01_JPRB, 0.10193E+01_JPRB, 0.10220E+01_JPRB, 0.10133E+01_JPRB, 0.10266E+01_JPRB, 0.10417E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.99559E+00_JPRB, 0.92307E+00_JPRB, 0.82401E+00_JPRB, 0.78258E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.82218E+00_JPRB, 0.79369E+00_JPRB, 0.77453E+00_JPRB, 0.75676E+00_JPRB, 0.74837E+00_JPRB, 0.74850E+00_JPRB, 0.74366E+00_JPRB ,&
&  0.72886E+00_JPRB, 0.69941E+00_JPRB, 0.66065E+00_JPRB, 0.62583E+00_JPRB, 0.62111E+00_JPRB, 0.61635E+00_JPRB, 0.61517E+00_JPRB ,&
&  0.60154E+00_JPRB, 0.59693E+00_JPRB, 0.60398E+00_JPRB, 0.60597E+00_JPRB, 0.60523E+00_JPRB, 0.60255E+00_JPRB, 0.60041E+00_JPRB ,&
&  0.60406E+00_JPRB, 0.60593E+00_JPRB, 0.60477E+00_JPRB, 0.60682E+00_JPRB, 0.61899E+00_JPRB, 0.65638E+00_JPRB, 0.72918E+00_JPRB ,&
&  0.93990E+00_JPRB, 0.13973E+01_JPRB, 0.16029E+01_JPRB, 0.33704E+01_JPRB, 0.39335E+01_JPRB, 0.38654E+01_JPRB, 0.34524E+01_JPRB ,&
&  0.30428E+01_JPRB, 0.29756E+01_JPRB, 0.24238E+01_JPRB, 0.19366E+01_JPRB, 0.15464E+01_JPRB, 0.13300E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.10028E+01_JPRB, 0.98363E+00_JPRB, 0.98488E+00_JPRB, 0.99978E+00_JPRB, 0.10547E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.13446E+01_JPRB, 0.16381E+01_JPRB, 0.19221E+01_JPRB, 0.28048E+01_JPRB, 0.46674E+01_JPRB, 0.62920E+01_JPRB ,&
&  0.70484E+01_JPRB, 0.66515E+01_JPRB, 0.58414E+01_JPRB, 0.47474E+01_JPRB, 0.37893E+01_JPRB, 0.34235E+01_JPRB, 0.30271E+01_JPRB ,&
&  0.26276E+01_JPRB, 0.23559E+01_JPRB, 0.19931E+01_JPRB, 0.17269E+01_JPRB, 0.15799E+01_JPRB, 0.14175E+01_JPRB, 0.12776E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.11488E+01_JPRB, 0.11234E+01_JPRB, 0.10953E+01_JPRB, 0.10874E+01_JPRB, 0.11038E+01_JPRB, 0.11149E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.10990E+01_JPRB, 0.10457E+01_JPRB, 0.10395E+01_JPRB, 0.10695E+01_JPRB, 0.10634E+01_JPRB, 0.10288E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.10001E+01_JPRB, 0.99902E+00_JPRB, 0.98987E+00_JPRB, 0.97418E+00_JPRB, 0.10491E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.96309E+00_JPRB, 0.91884E+00_JPRB, 0.87437E+00_JPRB, 0.84987E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.86246E+00_JPRB, 0.84252E+00_JPRB, 0.82534E+00_JPRB, 0.80372E+00_JPRB, 0.79134E+00_JPRB, 0.78638E+00_JPRB, 0.77846E+00_JPRB ,&
&  0.75941E+00_JPRB, 0.73158E+00_JPRB, 0.69878E+00_JPRB, 0.65540E+00_JPRB, 0.62801E+00_JPRB, 0.62003E+00_JPRB, 0.62499E+00_JPRB ,&
&  0.61365E+00_JPRB, 0.59606E+00_JPRB, 0.59674E+00_JPRB, 0.60102E+00_JPRB, 0.60038E+00_JPRB, 0.59242E+00_JPRB, 0.58321E+00_JPRB ,&
&  0.58678E+00_JPRB, 0.59280E+00_JPRB, 0.59696E+00_JPRB, 0.59905E+00_JPRB, 0.60336E+00_JPRB, 0.61952E+00_JPRB, 0.67636E+00_JPRB ,&
&  0.86164E+00_JPRB, 0.18046E+01_JPRB, 0.17074E+01_JPRB, 0.31480E+01_JPRB, 0.34022E+01_JPRB, 0.33589E+01_JPRB, 0.30822E+01_JPRB ,&
&  0.27604E+01_JPRB, 0.25341E+01_JPRB, 0.21011E+01_JPRB, 0.17771E+01_JPRB, 0.14765E+01_JPRB, 0.12537E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.94841E+00_JPRB, 0.90799E+00_JPRB, 0.89552E+00_JPRB, 0.89243E+00_JPRB, 0.90089E+00_JPRB, 0.92375E+00_JPRB ,&
&  0.95929E+00_JPRB, 0.10667E+01_JPRB, 0.13467E+01_JPRB, 0.19715E+01_JPRB, 0.25189E+01_JPRB, 0.38276E+01_JPRB, 0.57583E+01_JPRB ,&
&  0.66986E+01_JPRB, 0.65818E+01_JPRB, 0.55087E+01_JPRB, 0.45418E+01_JPRB, 0.38065E+01_JPRB, 0.33073E+01_JPRB, 0.28915E+01_JPRB ,&
&  0.26096E+01_JPRB, 0.24037E+01_JPRB, 0.21888E+01_JPRB, 0.19260E+01_JPRB, 0.17316E+01_JPRB, 0.15888E+01_JPRB, 0.14492E+01_JPRB ,&
&  0.13103E+01_JPRB, 0.11988E+01_JPRB, 0.11354E+01_JPRB, 0.10874E+01_JPRB, 0.10595E+01_JPRB, 0.10671E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10639E+01_JPRB, 0.10122E+01_JPRB, 0.99825E+00_JPRB, 0.10248E+01_JPRB, 0.10247E+01_JPRB, 0.99280E+00_JPRB ,&
&  0.98754E+00_JPRB, 0.98951E+00_JPRB, 0.98384E+00_JPRB, 0.99390E+00_JPRB, 0.96178E+00_JPRB, 0.10459E+01_JPRB, 0.10364E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.97516E+00_JPRB, 0.93086E+00_JPRB, 0.91473E+00_JPRB, 0.89250E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.88128E+00_JPRB, 0.86105E+00_JPRB, 0.84628E+00_JPRB, 0.83509E+00_JPRB, 0.82660E+00_JPRB, 0.81180E+00_JPRB, 0.79453E+00_JPRB ,&
&  0.77829E+00_JPRB, 0.75904E+00_JPRB, 0.72684E+00_JPRB, 0.68081E+00_JPRB, 0.64267E+00_JPRB, 0.62307E+00_JPRB, 0.62362E+00_JPRB ,&
&  0.62333E+00_JPRB, 0.61216E+00_JPRB, 0.60340E+00_JPRB, 0.60193E+00_JPRB, 0.60021E+00_JPRB, 0.59117E+00_JPRB, 0.57987E+00_JPRB ,&
&  0.57221E+00_JPRB, 0.57380E+00_JPRB, 0.57855E+00_JPRB, 0.58485E+00_JPRB, 0.59030E+00_JPRB, 0.59856E+00_JPRB, 0.63752E+00_JPRB ,&
&  0.82311E+00_JPRB, 0.86840E+00_JPRB, 0.20229E+01_JPRB, 0.26990E+01_JPRB, 0.27341E+01_JPRB, 0.26392E+01_JPRB, 0.26263E+01_JPRB ,&
&  0.24227E+01_JPRB, 0.21233E+01_JPRB, 0.17721E+01_JPRB, 0.14859E+01_JPRB, 0.12723E+01_JPRB, 0.11312E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.94747E+00_JPRB, 0.88706E+00_JPRB, 0.84139E+00_JPRB, 0.81691E+00_JPRB, 0.82021E+00_JPRB, 0.83138E+00_JPRB, 0.84703E+00_JPRB ,&
&  0.86922E+00_JPRB, 0.91657E+00_JPRB, 0.10681E+01_JPRB, 0.15578E+01_JPRB, 0.26726E+01_JPRB, 0.36354E+01_JPRB, 0.51339E+01_JPRB ,&
&  0.62953E+01_JPRB, 0.58248E+01_JPRB, 0.47259E+01_JPRB, 0.39393E+01_JPRB, 0.33550E+01_JPRB, 0.29386E+01_JPRB, 0.26233E+01_JPRB ,&
&  0.23839E+01_JPRB, 0.22137E+01_JPRB, 0.20704E+01_JPRB, 0.19136E+01_JPRB, 0.17771E+01_JPRB, 0.16693E+01_JPRB, 0.15654E+01_JPRB ,&
&  0.14452E+01_JPRB, 0.13267E+01_JPRB, 0.12191E+01_JPRB, 0.11252E+01_JPRB, 0.10498E+01_JPRB, 0.10296E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.10166E+01_JPRB, 0.10083E+01_JPRB, 0.98268E+00_JPRB, 0.98184E+00_JPRB, 0.10135E+01_JPRB, 0.10130E+01_JPRB, 0.99543E+00_JPRB ,&
&  0.96692E+00_JPRB, 0.97635E+00_JPRB, 0.96590E+00_JPRB, 0.94915E+00_JPRB, 0.90058E+00_JPRB, 0.99674E+00_JPRB, 0.99471E+00_JPRB ,&
&  0.99107E+00_JPRB, 0.97721E+00_JPRB, 0.93740E+00_JPRB, 0.92571E+00_JPRB, 0.90904E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.88066E+00_JPRB, 0.85814E+00_JPRB, 0.84415E+00_JPRB, 0.83475E+00_JPRB, 0.82711E+00_JPRB, 0.81948E+00_JPRB, 0.80067E+00_JPRB ,&
&  0.77957E+00_JPRB, 0.76705E+00_JPRB, 0.74894E+00_JPRB, 0.72435E+00_JPRB, 0.68755E+00_JPRB, 0.65769E+00_JPRB, 0.64788E+00_JPRB ,&
&  0.64219E+00_JPRB, 0.63325E+00_JPRB, 0.61728E+00_JPRB, 0.60696E+00_JPRB, 0.59708E+00_JPRB, 0.58534E+00_JPRB, 0.57775E+00_JPRB ,&
&  0.56567E+00_JPRB, 0.56090E+00_JPRB, 0.56397E+00_JPRB, 0.57161E+00_JPRB, 0.57873E+00_JPRB, 0.58737E+00_JPRB, 0.60233E+00_JPRB ,&
&  0.75515E+00_JPRB, 0.14134E+01_JPRB, 0.19250E+01_JPRB, 0.22079E+01_JPRB, 0.22240E+01_JPRB, 0.20705E+01_JPRB, 0.20634E+01_JPRB ,&
&  0.18519E+01_JPRB, 0.16407E+01_JPRB, 0.14260E+01_JPRB, 0.12273E+01_JPRB, 0.10703E+01_JPRB, 0.98558E+00_JPRB, 0.93904E+00_JPRB ,&
&  0.88624E+00_JPRB, 0.83925E+00_JPRB, 0.79881E+00_JPRB, 0.76439E+00_JPRB, 0.75225E+00_JPRB, 0.76685E+00_JPRB, 0.79156E+00_JPRB ,&
&  0.80547E+00_JPRB, 0.82465E+00_JPRB, 0.89871E+00_JPRB, 0.11939E+01_JPRB, 0.21009E+01_JPRB, 0.25940E+01_JPRB, 0.39534E+01_JPRB ,&
&  0.47699E+01_JPRB, 0.43012E+01_JPRB, 0.35995E+01_JPRB, 0.30750E+01_JPRB, 0.26363E+01_JPRB, 0.22978E+01_JPRB, 0.20518E+01_JPRB ,&
&  0.19100E+01_JPRB, 0.18124E+01_JPRB, 0.17259E+01_JPRB, 0.16423E+01_JPRB, 0.15647E+01_JPRB, 0.15034E+01_JPRB, 0.14545E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.13360E+01_JPRB, 0.12595E+01_JPRB, 0.11731E+01_JPRB, 0.10871E+01_JPRB, 0.10320E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.97453E+00_JPRB, 0.96469E+00_JPRB, 0.95436E+00_JPRB, 0.96302E+00_JPRB, 0.99192E+00_JPRB, 0.10027E+01_JPRB, 0.99561E+00_JPRB ,&
&  0.95435E+00_JPRB, 0.93766E+00_JPRB, 0.92039E+00_JPRB, 0.87045E+00_JPRB, 0.85369E+00_JPRB, 0.93484E+00_JPRB, 0.92541E+00_JPRB ,&
&  0.91354E+00_JPRB, 0.90752E+00_JPRB, 0.89669E+00_JPRB, 0.89305E+00_JPRB, 0.90656E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.88947E+00_JPRB, 0.85920E+00_JPRB, 0.83800E+00_JPRB, 0.82405E+00_JPRB, 0.81609E+00_JPRB, 0.80715E+00_JPRB, 0.79121E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.75504E+00_JPRB, 0.73941E+00_JPRB, 0.72782E+00_JPRB, 0.70361E+00_JPRB, 0.66233E+00_JPRB, 0.62982E+00_JPRB ,&
&  0.61864E+00_JPRB, 0.60986E+00_JPRB, 0.60203E+00_JPRB, 0.58935E+00_JPRB, 0.57912E+00_JPRB, 0.57439E+00_JPRB, 0.56958E+00_JPRB ,&
&  0.56451E+00_JPRB, 0.55962E+00_JPRB, 0.56124E+00_JPRB, 0.56640E+00_JPRB, 0.57088E+00_JPRB, 0.57487E+00_JPRB, 0.58233E+00_JPRB ,&
&  0.65943E+00_JPRB, 0.10610E+01_JPRB, 0.12941E+01_JPRB, 0.16360E+01_JPRB, 0.17412E+01_JPRB, 0.16295E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.14434E+01_JPRB, 0.13141E+01_JPRB, 0.11951E+01_JPRB, 0.10826E+01_JPRB, 0.95856E+00_JPRB, 0.88468E+00_JPRB, 0.84931E+00_JPRB ,&
&  0.82449E+00_JPRB, 0.79209E+00_JPRB, 0.75905E+00_JPRB, 0.72565E+00_JPRB, 0.70372E+00_JPRB, 0.70868E+00_JPRB, 0.72371E+00_JPRB ,&
&  0.72470E+00_JPRB, 0.72727E+00_JPRB, 0.74691E+00_JPRB, 0.86224E+00_JPRB, 0.13098E+01_JPRB, 0.17655E+01_JPRB, 0.23900E+01_JPRB ,&
&  0.24526E+01_JPRB, 0.25971E+01_JPRB, 0.23891E+01_JPRB, 0.21385E+01_JPRB, 0.19137E+01_JPRB, 0.17386E+01_JPRB, 0.16060E+01_JPRB ,&
&  0.15144E+01_JPRB, 0.14382E+01_JPRB, 0.13806E+01_JPRB, 0.13364E+01_JPRB, 0.13016E+01_JPRB, 0.12600E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12235E+01_JPRB, 0.11967E+01_JPRB, 0.11613E+01_JPRB, 0.11194E+01_JPRB, 0.10695E+01_JPRB, 0.10271E+01_JPRB, 0.99502E+00_JPRB ,&
&  0.96639E+00_JPRB, 0.93468E+00_JPRB, 0.91788E+00_JPRB, 0.92337E+00_JPRB, 0.93316E+00_JPRB, 0.93988E+00_JPRB, 0.93106E+00_JPRB ,&
&  0.91629E+00_JPRB, 0.90392E+00_JPRB, 0.88451E+00_JPRB, 0.84075E+00_JPRB, 0.82314E+00_JPRB, 0.87557E+00_JPRB, 0.86874E+00_JPRB ,&
&  0.85274E+00_JPRB, 0.84066E+00_JPRB, 0.84421E+00_JPRB, 0.85667E+00_JPRB, 0.91447E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.90754E+00_JPRB, 0.87356E+00_JPRB, 0.83659E+00_JPRB, 0.81203E+00_JPRB, 0.79457E+00_JPRB, 0.77909E+00_JPRB, 0.75963E+00_JPRB ,&
&  0.74091E+00_JPRB, 0.72297E+00_JPRB, 0.70763E+00_JPRB, 0.69596E+00_JPRB, 0.68357E+00_JPRB, 0.65540E+00_JPRB, 0.61689E+00_JPRB ,&
&  0.59083E+00_JPRB, 0.58184E+00_JPRB, 0.57597E+00_JPRB, 0.56596E+00_JPRB, 0.55679E+00_JPRB, 0.55316E+00_JPRB, 0.55007E+00_JPRB ,&
&  0.55043E+00_JPRB, 0.55191E+00_JPRB, 0.55620E+00_JPRB, 0.55822E+00_JPRB, 0.55558E+00_JPRB, 0.56160E+00_JPRB, 0.56208E+00_JPRB ,&
&  0.61817E+00_JPRB, 0.75864E+00_JPRB, 0.92960E+00_JPRB, 0.11232E+01_JPRB, 0.12627E+01_JPRB, 0.13036E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.11656E+01_JPRB, 0.10883E+01_JPRB, 0.10131E+01_JPRB, 0.92863E+00_JPRB, 0.83924E+00_JPRB, 0.78705E+00_JPRB ,&
&  0.76867E+00_JPRB, 0.74802E+00_JPRB, 0.72798E+00_JPRB, 0.70552E+00_JPRB, 0.68524E+00_JPRB, 0.67790E+00_JPRB, 0.67861E+00_JPRB ,&
&  0.67351E+00_JPRB, 0.66695E+00_JPRB, 0.66113E+00_JPRB, 0.66969E+00_JPRB, 0.74329E+00_JPRB, 0.95976E+00_JPRB, 0.12458E+01_JPRB ,&
&  0.13676E+01_JPRB, 0.14595E+01_JPRB, 0.15192E+01_JPRB, 0.15168E+01_JPRB, 0.14503E+01_JPRB, 0.13506E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11138E+01_JPRB, 0.10848E+01_JPRB, 0.10754E+01_JPRB, 0.10723E+01_JPRB, 0.10612E+01_JPRB, 0.10392E+01_JPRB ,&
&  0.10203E+01_JPRB, 0.10114E+01_JPRB, 0.10110E+01_JPRB, 0.10039E+01_JPRB, 0.99269E+00_JPRB, 0.97885E+00_JPRB, 0.96952E+00_JPRB ,&
&  0.96938E+00_JPRB, 0.95961E+00_JPRB, 0.92879E+00_JPRB, 0.89809E+00_JPRB, 0.89304E+00_JPRB, 0.89021E+00_JPRB, 0.88113E+00_JPRB ,&
&  0.87198E+00_JPRB, 0.87122E+00_JPRB, 0.85430E+00_JPRB, 0.84931E+00_JPRB, 0.86622E+00_JPRB, 0.84333E+00_JPRB, 0.83054E+00_JPRB ,&
&  0.80722E+00_JPRB, 0.79390E+00_JPRB, 0.79996E+00_JPRB, 0.82081E+00_JPRB, 0.89215E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.77062E+00_JPRB, 0.76362E+00_JPRB, 0.75410E+00_JPRB, 0.74525E+00_JPRB, 0.73167E+00_JPRB, 0.71768E+00_JPRB, 0.70015E+00_JPRB ,&
&  0.68340E+00_JPRB, 0.66735E+00_JPRB, 0.65402E+00_JPRB, 0.64954E+00_JPRB, 0.64137E+00_JPRB, 0.62908E+00_JPRB, 0.60204E+00_JPRB ,&
&  0.57946E+00_JPRB, 0.56239E+00_JPRB, 0.55184E+00_JPRB, 0.54652E+00_JPRB, 0.54255E+00_JPRB, 0.53816E+00_JPRB, 0.53627E+00_JPRB ,&
&  0.53707E+00_JPRB, 0.53689E+00_JPRB, 0.53939E+00_JPRB, 0.53861E+00_JPRB, 0.54068E+00_JPRB, 0.53611E+00_JPRB, 0.53267E+00_JPRB ,&
&  0.56546E+00_JPRB, 0.61676E+00_JPRB, 0.71781E+00_JPRB, 0.82839E+00_JPRB, 0.94731E+00_JPRB, 0.10295E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.11142E+01_JPRB, 0.10746E+01_JPRB, 0.10113E+01_JPRB, 0.93503E+00_JPRB, 0.87537E+00_JPRB, 0.81336E+00_JPRB, 0.75174E+00_JPRB ,&
&  0.72072E+00_JPRB, 0.70355E+00_JPRB, 0.68966E+00_JPRB, 0.67827E+00_JPRB, 0.66784E+00_JPRB, 0.66137E+00_JPRB, 0.65694E+00_JPRB ,&
&  0.64680E+00_JPRB, 0.63473E+00_JPRB, 0.62069E+00_JPRB, 0.61519E+00_JPRB, 0.61936E+00_JPRB, 0.66910E+00_JPRB, 0.80522E+00_JPRB ,&
&  0.88148E+00_JPRB, 0.92302E+00_JPRB, 0.95804E+00_JPRB, 0.10126E+01_JPRB, 0.10516E+01_JPRB, 0.10521E+01_JPRB, 0.99685E+00_JPRB ,&
&  0.94257E+00_JPRB, 0.90106E+00_JPRB, 0.87758E+00_JPRB, 0.86873E+00_JPRB, 0.87074E+00_JPRB, 0.87419E+00_JPRB, 0.87629E+00_JPRB ,&
&  0.86623E+00_JPRB, 0.85563E+00_JPRB, 0.85263E+00_JPRB, 0.85459E+00_JPRB, 0.86192E+00_JPRB, 0.86858E+00_JPRB, 0.87911E+00_JPRB ,&
&  0.89934E+00_JPRB, 0.92545E+00_JPRB, 0.94514E+00_JPRB, 0.92605E+00_JPRB, 0.89049E+00_JPRB, 0.86395E+00_JPRB, 0.84554E+00_JPRB ,&
&  0.83328E+00_JPRB, 0.83202E+00_JPRB, 0.83570E+00_JPRB, 0.84690E+00_JPRB, 0.84320E+00_JPRB, 0.82266E+00_JPRB, 0.79702E+00_JPRB ,&
&  0.77401E+00_JPRB, 0.75882E+00_JPRB, 0.75752E+00_JPRB, 0.76319E+00_JPRB, 0.74600E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.68364E+00_JPRB, 0.67864E+00_JPRB, 0.67228E+00_JPRB, 0.66793E+00_JPRB, 0.65954E+00_JPRB, 0.65002E+00_JPRB, 0.63974E+00_JPRB ,&
&  0.62673E+00_JPRB, 0.61393E+00_JPRB, 0.60330E+00_JPRB, 0.60018E+00_JPRB, 0.59898E+00_JPRB, 0.59811E+00_JPRB, 0.59044E+00_JPRB ,&
&  0.57744E+00_JPRB, 0.56482E+00_JPRB, 0.55357E+00_JPRB, 0.54598E+00_JPRB, 0.54253E+00_JPRB, 0.53863E+00_JPRB, 0.53389E+00_JPRB ,&
&  0.53030E+00_JPRB, 0.52972E+00_JPRB, 0.52790E+00_JPRB, 0.52578E+00_JPRB, 0.52270E+00_JPRB, 0.51614E+00_JPRB, 0.51488E+00_JPRB ,&
&  0.52888E+00_JPRB, 0.51228E+00_JPRB, 0.56715E+00_JPRB, 0.61832E+00_JPRB, 0.67213E+00_JPRB, 0.75709E+00_JPRB, 0.85813E+00_JPRB ,&
&  0.94069E+00_JPRB, 0.94897E+00_JPRB, 0.92118E+00_JPRB, 0.86630E+00_JPRB, 0.82574E+00_JPRB, 0.79590E+00_JPRB, 0.75380E+00_JPRB ,&
&  0.71229E+00_JPRB, 0.68415E+00_JPRB, 0.66323E+00_JPRB, 0.64665E+00_JPRB, 0.63453E+00_JPRB, 0.62415E+00_JPRB, 0.61458E+00_JPRB ,&
&  0.60192E+00_JPRB, 0.58932E+00_JPRB, 0.57877E+00_JPRB, 0.57379E+00_JPRB, 0.57565E+00_JPRB, 0.58411E+00_JPRB, 0.62269E+00_JPRB ,&
&  0.66532E+00_JPRB, 0.69197E+00_JPRB, 0.69875E+00_JPRB, 0.72313E+00_JPRB, 0.76125E+00_JPRB, 0.79327E+00_JPRB, 0.80259E+00_JPRB ,&
&  0.78302E+00_JPRB, 0.75107E+00_JPRB, 0.72303E+00_JPRB, 0.70229E+00_JPRB, 0.69200E+00_JPRB, 0.69459E+00_JPRB, 0.70700E+00_JPRB ,&
&  0.71330E+00_JPRB, 0.71286E+00_JPRB, 0.71411E+00_JPRB, 0.71977E+00_JPRB, 0.72793E+00_JPRB, 0.74181E+00_JPRB, 0.76117E+00_JPRB ,&
&  0.78566E+00_JPRB, 0.81439E+00_JPRB, 0.84641E+00_JPRB, 0.87052E+00_JPRB, 0.87490E+00_JPRB, 0.86800E+00_JPRB, 0.85400E+00_JPRB ,&
&  0.83815E+00_JPRB, 0.82456E+00_JPRB, 0.81105E+00_JPRB, 0.78173E+00_JPRB, 0.80083E+00_JPRB, 0.79147E+00_JPRB, 0.76668E+00_JPRB ,&
&  0.74428E+00_JPRB, 0.72240E+00_JPRB, 0.70724E+00_JPRB, 0.64825E+00_JPRB, 0.68077E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.61244E+00_JPRB, 0.60763E+00_JPRB, 0.60060E+00_JPRB, 0.59479E+00_JPRB, 0.59095E+00_JPRB, 0.58555E+00_JPRB, 0.58133E+00_JPRB ,&
&  0.57539E+00_JPRB, 0.57068E+00_JPRB, 0.56274E+00_JPRB, 0.55768E+00_JPRB, 0.55573E+00_JPRB, 0.55638E+00_JPRB, 0.55690E+00_JPRB ,&
&  0.55324E+00_JPRB, 0.54605E+00_JPRB, 0.53830E+00_JPRB, 0.53199E+00_JPRB, 0.52668E+00_JPRB, 0.52066E+00_JPRB, 0.51461E+00_JPRB ,&
&  0.50908E+00_JPRB, 0.50430E+00_JPRB, 0.50006E+00_JPRB, 0.49882E+00_JPRB, 0.49767E+00_JPRB, 0.49548E+00_JPRB, 0.49201E+00_JPRB ,&
&  0.49744E+00_JPRB, 0.44640E+00_JPRB, 0.46520E+00_JPRB, 0.48655E+00_JPRB, 0.51244E+00_JPRB, 0.57368E+00_JPRB, 0.66046E+00_JPRB ,&
&  0.74060E+00_JPRB, 0.79431E+00_JPRB, 0.80744E+00_JPRB, 0.78306E+00_JPRB, 0.75442E+00_JPRB, 0.73628E+00_JPRB, 0.72115E+00_JPRB ,&
&  0.69269E+00_JPRB, 0.66719E+00_JPRB, 0.64228E+00_JPRB, 0.61744E+00_JPRB, 0.59430E+00_JPRB, 0.57400E+00_JPRB, 0.55726E+00_JPRB ,&
&  0.54634E+00_JPRB, 0.54108E+00_JPRB, 0.53805E+00_JPRB, 0.53640E+00_JPRB, 0.53734E+00_JPRB, 0.53972E+00_JPRB, 0.54265E+00_JPRB ,&
&  0.55166E+00_JPRB, 0.56972E+00_JPRB, 0.57820E+00_JPRB, 0.58572E+00_JPRB, 0.60209E+00_JPRB, 0.62513E+00_JPRB, 0.63779E+00_JPRB ,&
&  0.63912E+00_JPRB, 0.63172E+00_JPRB, 0.62012E+00_JPRB, 0.60721E+00_JPRB, 0.59572E+00_JPRB, 0.58628E+00_JPRB, 0.58466E+00_JPRB ,&
&  0.58528E+00_JPRB, 0.58865E+00_JPRB, 0.59338E+00_JPRB, 0.60253E+00_JPRB, 0.61383E+00_JPRB, 0.62532E+00_JPRB, 0.63666E+00_JPRB ,&
&  0.64809E+00_JPRB, 0.66129E+00_JPRB, 0.68046E+00_JPRB, 0.70254E+00_JPRB, 0.72074E+00_JPRB, 0.72980E+00_JPRB, 0.73187E+00_JPRB ,&
&  0.72867E+00_JPRB, 0.72342E+00_JPRB, 0.71333E+00_JPRB, 0.71036E+00_JPRB, 0.70451E+00_JPRB, 0.70128E+00_JPRB, 0.69040E+00_JPRB ,&
&  0.67722E+00_JPRB, 0.66339E+00_JPRB, 0.58079E+00_JPRB, 0.61067E+00_JPRB, 0.61254E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.56369E+00_JPRB, 0.55566E+00_JPRB, 0.54819E+00_JPRB, 0.54089E+00_JPRB, 0.53546E+00_JPRB, 0.53192E+00_JPRB, 0.53074E+00_JPRB ,&
&  0.53221E+00_JPRB, 0.53423E+00_JPRB, 0.53304E+00_JPRB, 0.52721E+00_JPRB, 0.52153E+00_JPRB, 0.51835E+00_JPRB, 0.51757E+00_JPRB ,&
&  0.51617E+00_JPRB, 0.51267E+00_JPRB, 0.50769E+00_JPRB, 0.50277E+00_JPRB, 0.49684E+00_JPRB, 0.49155E+00_JPRB, 0.48675E+00_JPRB ,&
&  0.48255E+00_JPRB, 0.48007E+00_JPRB, 0.47794E+00_JPRB, 0.47621E+00_JPRB, 0.47562E+00_JPRB, 0.47399E+00_JPRB, 0.47650E+00_JPRB ,&
&  0.44782E+00_JPRB, 0.41520E+00_JPRB, 0.42955E+00_JPRB, 0.44310E+00_JPRB, 0.45663E+00_JPRB, 0.48927E+00_JPRB, 0.53937E+00_JPRB ,&
&  0.60198E+00_JPRB, 0.65626E+00_JPRB, 0.68902E+00_JPRB, 0.68645E+00_JPRB, 0.66946E+00_JPRB, 0.65895E+00_JPRB, 0.65361E+00_JPRB ,&
&  0.63895E+00_JPRB, 0.62119E+00_JPRB, 0.60552E+00_JPRB, 0.58704E+00_JPRB, 0.56723E+00_JPRB, 0.54954E+00_JPRB, 0.53555E+00_JPRB ,&
&  0.52497E+00_JPRB, 0.51863E+00_JPRB, 0.51630E+00_JPRB, 0.51401E+00_JPRB, 0.51179E+00_JPRB, 0.51109E+00_JPRB, 0.50920E+00_JPRB ,&
&  0.50882E+00_JPRB, 0.50997E+00_JPRB, 0.51529E+00_JPRB, 0.51983E+00_JPRB, 0.52617E+00_JPRB, 0.53211E+00_JPRB, 0.53485E+00_JPRB ,&
&  0.53314E+00_JPRB, 0.53150E+00_JPRB, 0.53212E+00_JPRB, 0.52968E+00_JPRB, 0.52167E+00_JPRB, 0.51271E+00_JPRB, 0.50877E+00_JPRB ,&
&  0.50888E+00_JPRB, 0.51014E+00_JPRB, 0.51224E+00_JPRB, 0.51678E+00_JPRB, 0.52322E+00_JPRB, 0.53053E+00_JPRB, 0.53794E+00_JPRB ,&
&  0.54532E+00_JPRB, 0.55211E+00_JPRB, 0.55905E+00_JPRB, 0.56829E+00_JPRB, 0.58133E+00_JPRB, 0.59343E+00_JPRB, 0.60346E+00_JPRB ,&
&  0.61108E+00_JPRB, 0.61638E+00_JPRB, 0.62020E+00_JPRB, 0.62517E+00_JPRB, 0.62897E+00_JPRB, 0.63025E+00_JPRB, 0.62550E+00_JPRB ,&
&  0.61546E+00_JPRB, 0.60172E+00_JPRB, 0.59309E+00_JPRB, 0.58079E+00_JPRB, 0.57067E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.50433E+00_JPRB, 0.49933E+00_JPRB, 0.49353E+00_JPRB, 0.48768E+00_JPRB, 0.48213E+00_JPRB, 0.47965E+00_JPRB, 0.47944E+00_JPRB ,&
&  0.48242E+00_JPRB, 0.48443E+00_JPRB, 0.48756E+00_JPRB, 0.48645E+00_JPRB, 0.48384E+00_JPRB, 0.48154E+00_JPRB, 0.47924E+00_JPRB ,&
&  0.47664E+00_JPRB, 0.47306E+00_JPRB, 0.46943E+00_JPRB, 0.46662E+00_JPRB, 0.46340E+00_JPRB, 0.45889E+00_JPRB, 0.45455E+00_JPRB ,&
&  0.45097E+00_JPRB, 0.44891E+00_JPRB, 0.44734E+00_JPRB, 0.44588E+00_JPRB, 0.44433E+00_JPRB, 0.44410E+00_JPRB, 0.44318E+00_JPRB ,&
&  0.44474E+00_JPRB, 0.41410E+00_JPRB, 0.42593E+00_JPRB, 0.43025E+00_JPRB, 0.43829E+00_JPRB, 0.45573E+00_JPRB, 0.47821E+00_JPRB ,&
&  0.50750E+00_JPRB, 0.54137E+00_JPRB, 0.56899E+00_JPRB, 0.57877E+00_JPRB, 0.57361E+00_JPRB, 0.57043E+00_JPRB, 0.57309E+00_JPRB ,&
&  0.57134E+00_JPRB, 0.56540E+00_JPRB, 0.56028E+00_JPRB, 0.55340E+00_JPRB, 0.54292E+00_JPRB, 0.52861E+00_JPRB, 0.51538E+00_JPRB ,&
&  0.50741E+00_JPRB, 0.50268E+00_JPRB, 0.49832E+00_JPRB, 0.49349E+00_JPRB, 0.48636E+00_JPRB, 0.47926E+00_JPRB, 0.47289E+00_JPRB ,&
&  0.46708E+00_JPRB, 0.46015E+00_JPRB, 0.45632E+00_JPRB, 0.45547E+00_JPRB, 0.45487E+00_JPRB, 0.45623E+00_JPRB, 0.45718E+00_JPRB ,&
&  0.45564E+00_JPRB, 0.45230E+00_JPRB, 0.45072E+00_JPRB, 0.45131E+00_JPRB, 0.45102E+00_JPRB, 0.45033E+00_JPRB, 0.44926E+00_JPRB ,&
&  0.45018E+00_JPRB, 0.45137E+00_JPRB, 0.45149E+00_JPRB, 0.45215E+00_JPRB, 0.45430E+00_JPRB, 0.45848E+00_JPRB, 0.46324E+00_JPRB ,&
&  0.46843E+00_JPRB, 0.47407E+00_JPRB, 0.47926E+00_JPRB, 0.48433E+00_JPRB, 0.49163E+00_JPRB, 0.49926E+00_JPRB, 0.50855E+00_JPRB ,&
&  0.51874E+00_JPRB, 0.52783E+00_JPRB, 0.53572E+00_JPRB, 0.54242E+00_JPRB, 0.54627E+00_JPRB, 0.54649E+00_JPRB, 0.54445E+00_JPRB ,&
&  0.54074E+00_JPRB, 0.53176E+00_JPRB, 0.52392E+00_JPRB, 0.51555E+00_JPRB, 0.50830E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.45951E+00_JPRB, 0.45449E+00_JPRB, 0.44901E+00_JPRB, 0.44474E+00_JPRB, 0.44109E+00_JPRB, 0.43763E+00_JPRB, 0.43652E+00_JPRB ,&
&  0.43765E+00_JPRB, 0.44042E+00_JPRB, 0.44432E+00_JPRB, 0.44683E+00_JPRB, 0.44813E+00_JPRB, 0.44937E+00_JPRB, 0.44924E+00_JPRB ,&
&  0.44831E+00_JPRB, 0.44652E+00_JPRB, 0.44396E+00_JPRB, 0.44049E+00_JPRB, 0.43651E+00_JPRB, 0.43272E+00_JPRB, 0.42785E+00_JPRB ,&
&  0.42420E+00_JPRB, 0.42172E+00_JPRB, 0.42002E+00_JPRB, 0.41902E+00_JPRB, 0.41929E+00_JPRB, 0.41982E+00_JPRB, 0.42009E+00_JPRB ,&
&  0.41807E+00_JPRB, 0.41252E+00_JPRB, 0.41873E+00_JPRB, 0.42055E+00_JPRB, 0.42762E+00_JPRB, 0.44099E+00_JPRB, 0.45329E+00_JPRB ,&
&  0.46243E+00_JPRB, 0.47668E+00_JPRB, 0.49373E+00_JPRB, 0.50568E+00_JPRB, 0.50684E+00_JPRB, 0.50591E+00_JPRB, 0.51029E+00_JPRB ,&
&  0.51314E+00_JPRB, 0.51223E+00_JPRB, 0.51222E+00_JPRB, 0.51109E+00_JPRB, 0.50787E+00_JPRB, 0.50358E+00_JPRB, 0.49651E+00_JPRB ,&
&  0.48902E+00_JPRB, 0.48285E+00_JPRB, 0.47628E+00_JPRB, 0.46952E+00_JPRB, 0.46045E+00_JPRB, 0.45024E+00_JPRB, 0.43904E+00_JPRB ,&
&  0.42760E+00_JPRB, 0.41933E+00_JPRB, 0.41399E+00_JPRB, 0.40976E+00_JPRB, 0.40635E+00_JPRB, 0.40511E+00_JPRB, 0.40367E+00_JPRB ,&
&  0.40304E+00_JPRB, 0.40326E+00_JPRB, 0.40198E+00_JPRB, 0.40138E+00_JPRB, 0.40260E+00_JPRB, 0.40370E+00_JPRB, 0.40624E+00_JPRB ,&
&  0.40744E+00_JPRB, 0.40839E+00_JPRB, 0.40918E+00_JPRB, 0.40917E+00_JPRB, 0.40841E+00_JPRB, 0.40979E+00_JPRB, 0.41266E+00_JPRB ,&
&  0.41579E+00_JPRB, 0.42051E+00_JPRB, 0.42563E+00_JPRB, 0.43005E+00_JPRB, 0.43425E+00_JPRB, 0.44085E+00_JPRB, 0.44933E+00_JPRB ,&
&  0.45889E+00_JPRB, 0.46914E+00_JPRB, 0.47827E+00_JPRB, 0.48515E+00_JPRB, 0.48714E+00_JPRB, 0.48703E+00_JPRB, 0.48628E+00_JPRB ,&
&  0.48348E+00_JPRB, 0.47659E+00_JPRB, 0.47219E+00_JPRB, 0.46849E+00_JPRB, 0.46408E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.40677E+00_JPRB, 0.40480E+00_JPRB, 0.40317E+00_JPRB, 0.40240E+00_JPRB, 0.40204E+00_JPRB, 0.40213E+00_JPRB, 0.40310E+00_JPRB ,&
&  0.40522E+00_JPRB, 0.40732E+00_JPRB, 0.41220E+00_JPRB, 0.41522E+00_JPRB, 0.41688E+00_JPRB, 0.41885E+00_JPRB, 0.42011E+00_JPRB ,&
&  0.41923E+00_JPRB, 0.41830E+00_JPRB, 0.41692E+00_JPRB, 0.41416E+00_JPRB, 0.40979E+00_JPRB, 0.40699E+00_JPRB, 0.40425E+00_JPRB ,&
&  0.40034E+00_JPRB, 0.39577E+00_JPRB, 0.39163E+00_JPRB, 0.38897E+00_JPRB, 0.38814E+00_JPRB, 0.38687E+00_JPRB, 0.38663E+00_JPRB ,&
&  0.38561E+00_JPRB, 0.39018E+00_JPRB, 0.37855E+00_JPRB, 0.38817E+00_JPRB, 0.39683E+00_JPRB, 0.40549E+00_JPRB, 0.41475E+00_JPRB ,&
&  0.42132E+00_JPRB, 0.42846E+00_JPRB, 0.43922E+00_JPRB, 0.44675E+00_JPRB, 0.44901E+00_JPRB, 0.44689E+00_JPRB, 0.44749E+00_JPRB ,&
&  0.44801E+00_JPRB, 0.44514E+00_JPRB, 0.44431E+00_JPRB, 0.44627E+00_JPRB, 0.44837E+00_JPRB, 0.44826E+00_JPRB, 0.44702E+00_JPRB ,&
&  0.44591E+00_JPRB, 0.44207E+00_JPRB, 0.43521E+00_JPRB, 0.42755E+00_JPRB, 0.41993E+00_JPRB, 0.41136E+00_JPRB, 0.40071E+00_JPRB ,&
&  0.38903E+00_JPRB, 0.37968E+00_JPRB, 0.37195E+00_JPRB, 0.36690E+00_JPRB, 0.36436E+00_JPRB, 0.36248E+00_JPRB, 0.36101E+00_JPRB ,&
&  0.36093E+00_JPRB, 0.36127E+00_JPRB, 0.36103E+00_JPRB, 0.36049E+00_JPRB, 0.36126E+00_JPRB, 0.36239E+00_JPRB, 0.36604E+00_JPRB ,&
&  0.37033E+00_JPRB, 0.37316E+00_JPRB, 0.37507E+00_JPRB, 0.37655E+00_JPRB, 0.37560E+00_JPRB, 0.37654E+00_JPRB, 0.37871E+00_JPRB ,&
&  0.38100E+00_JPRB, 0.38379E+00_JPRB, 0.38571E+00_JPRB, 0.38775E+00_JPRB, 0.39060E+00_JPRB, 0.39478E+00_JPRB, 0.40118E+00_JPRB ,&
&  0.40758E+00_JPRB, 0.41445E+00_JPRB, 0.42170E+00_JPRB, 0.42736E+00_JPRB, 0.43062E+00_JPRB, 0.43138E+00_JPRB, 0.43114E+00_JPRB ,&
&  0.42862E+00_JPRB, 0.42321E+00_JPRB, 0.41765E+00_JPRB, 0.41335E+00_JPRB, 0.40970E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.36858E+00_JPRB, 0.36893E+00_JPRB, 0.37022E+00_JPRB, 0.37062E+00_JPRB, 0.37061E+00_JPRB, 0.37165E+00_JPRB, 0.37503E+00_JPRB ,&
&  0.37827E+00_JPRB, 0.38080E+00_JPRB, 0.38346E+00_JPRB, 0.38546E+00_JPRB, 0.38695E+00_JPRB, 0.38964E+00_JPRB, 0.39174E+00_JPRB ,&
&  0.39248E+00_JPRB, 0.39163E+00_JPRB, 0.39032E+00_JPRB, 0.38840E+00_JPRB, 0.38562E+00_JPRB, 0.38246E+00_JPRB, 0.37965E+00_JPRB ,&
&  0.37605E+00_JPRB, 0.37197E+00_JPRB, 0.36786E+00_JPRB, 0.36547E+00_JPRB, 0.36427E+00_JPRB, 0.36343E+00_JPRB, 0.36355E+00_JPRB ,&
&  0.36365E+00_JPRB, 0.36590E+00_JPRB, 0.36866E+00_JPRB, 0.37216E+00_JPRB, 0.37558E+00_JPRB, 0.38013E+00_JPRB, 0.38512E+00_JPRB ,&
&  0.38986E+00_JPRB, 0.39253E+00_JPRB, 0.39546E+00_JPRB, 0.39550E+00_JPRB, 0.39858E+00_JPRB, 0.40093E+00_JPRB, 0.40220E+00_JPRB ,&
&  0.40139E+00_JPRB, 0.39542E+00_JPRB, 0.39108E+00_JPRB, 0.39136E+00_JPRB, 0.39339E+00_JPRB, 0.39535E+00_JPRB, 0.39701E+00_JPRB ,&
&  0.39716E+00_JPRB, 0.39573E+00_JPRB, 0.39174E+00_JPRB, 0.38606E+00_JPRB, 0.37991E+00_JPRB, 0.37294E+00_JPRB, 0.36368E+00_JPRB ,&
&  0.35518E+00_JPRB, 0.34826E+00_JPRB, 0.34268E+00_JPRB, 0.33773E+00_JPRB, 0.33431E+00_JPRB, 0.33086E+00_JPRB, 0.33008E+00_JPRB ,&
&  0.33091E+00_JPRB, 0.33325E+00_JPRB, 0.33440E+00_JPRB, 0.33350E+00_JPRB, 0.33219E+00_JPRB, 0.33231E+00_JPRB, 0.33507E+00_JPRB ,&
&  0.34007E+00_JPRB, 0.34417E+00_JPRB, 0.34603E+00_JPRB, 0.34655E+00_JPRB, 0.34639E+00_JPRB, 0.34809E+00_JPRB, 0.35095E+00_JPRB ,&
&  0.35261E+00_JPRB, 0.35346E+00_JPRB, 0.35455E+00_JPRB, 0.35431E+00_JPRB, 0.35619E+00_JPRB, 0.35951E+00_JPRB, 0.36333E+00_JPRB ,&
&  0.36711E+00_JPRB, 0.37184E+00_JPRB, 0.37649E+00_JPRB, 0.37995E+00_JPRB, 0.38277E+00_JPRB, 0.38256E+00_JPRB, 0.38226E+00_JPRB ,&
&  0.38157E+00_JPRB, 0.37933E+00_JPRB, 0.37699E+00_JPRB, 0.37404E+00_JPRB, 0.37065E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.33691E+00_JPRB, 0.34021E+00_JPRB, 0.34423E+00_JPRB, 0.34500E+00_JPRB, 0.34522E+00_JPRB, 0.34670E+00_JPRB, 0.35006E+00_JPRB ,&
&  0.35276E+00_JPRB, 0.35409E+00_JPRB, 0.35495E+00_JPRB, 0.35571E+00_JPRB, 0.35710E+00_JPRB, 0.35972E+00_JPRB, 0.36123E+00_JPRB ,&
&  0.36105E+00_JPRB, 0.36054E+00_JPRB, 0.35963E+00_JPRB, 0.35867E+00_JPRB, 0.35668E+00_JPRB, 0.35376E+00_JPRB, 0.35109E+00_JPRB ,&
&  0.34752E+00_JPRB, 0.34412E+00_JPRB, 0.34136E+00_JPRB, 0.33901E+00_JPRB, 0.33735E+00_JPRB, 0.33665E+00_JPRB, 0.33764E+00_JPRB ,&
&  0.33911E+00_JPRB, 0.34122E+00_JPRB, 0.34404E+00_JPRB, 0.34743E+00_JPRB, 0.35079E+00_JPRB, 0.35463E+00_JPRB, 0.35805E+00_JPRB ,&
&  0.36079E+00_JPRB, 0.36331E+00_JPRB, 0.36531E+00_JPRB, 0.36329E+00_JPRB, 0.36113E+00_JPRB, 0.36268E+00_JPRB, 0.36481E+00_JPRB ,&
&  0.36314E+00_JPRB, 0.35890E+00_JPRB, 0.35344E+00_JPRB, 0.35063E+00_JPRB, 0.35004E+00_JPRB, 0.34914E+00_JPRB, 0.34785E+00_JPRB ,&
&  0.34700E+00_JPRB, 0.34606E+00_JPRB, 0.34426E+00_JPRB, 0.34131E+00_JPRB, 0.33701E+00_JPRB, 0.33153E+00_JPRB, 0.32629E+00_JPRB ,&
&  0.32302E+00_JPRB, 0.32069E+00_JPRB, 0.32034E+00_JPRB, 0.31779E+00_JPRB, 0.31294E+00_JPRB, 0.30824E+00_JPRB, 0.30745E+00_JPRB ,&
&  0.30815E+00_JPRB, 0.30966E+00_JPRB, 0.31025E+00_JPRB, 0.30925E+00_JPRB, 0.30923E+00_JPRB, 0.30951E+00_JPRB, 0.31093E+00_JPRB ,&
&  0.31585E+00_JPRB, 0.31964E+00_JPRB, 0.31996E+00_JPRB, 0.31913E+00_JPRB, 0.31729E+00_JPRB, 0.31706E+00_JPRB, 0.31836E+00_JPRB ,&
&  0.31985E+00_JPRB, 0.32076E+00_JPRB, 0.32131E+00_JPRB, 0.32082E+00_JPRB, 0.32085E+00_JPRB, 0.32228E+00_JPRB, 0.32417E+00_JPRB ,&
&  0.32655E+00_JPRB, 0.32950E+00_JPRB, 0.33205E+00_JPRB, 0.33455E+00_JPRB, 0.33681E+00_JPRB, 0.33777E+00_JPRB, 0.33867E+00_JPRB ,&
&  0.33964E+00_JPRB, 0.33957E+00_JPRB, 0.33892E+00_JPRB, 0.33834E+00_JPRB, 0.33763E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.31254E+00_JPRB, 0.31725E+00_JPRB, 0.32304E+00_JPRB, 0.32417E+00_JPRB, 0.32608E+00_JPRB, 0.32830E+00_JPRB, 0.33179E+00_JPRB ,&
&  0.33449E+00_JPRB, 0.33538E+00_JPRB, 0.33447E+00_JPRB, 0.33343E+00_JPRB, 0.33313E+00_JPRB, 0.33347E+00_JPRB, 0.33346E+00_JPRB ,&
&  0.33358E+00_JPRB, 0.33385E+00_JPRB, 0.33373E+00_JPRB, 0.33330E+00_JPRB, 0.33163E+00_JPRB, 0.32921E+00_JPRB, 0.32643E+00_JPRB ,&
&  0.32224E+00_JPRB, 0.31924E+00_JPRB, 0.31785E+00_JPRB, 0.31697E+00_JPRB, 0.31563E+00_JPRB, 0.31572E+00_JPRB, 0.31727E+00_JPRB ,&
&  0.32093E+00_JPRB, 0.32378E+00_JPRB, 0.32745E+00_JPRB, 0.33112E+00_JPRB, 0.33319E+00_JPRB, 0.33474E+00_JPRB, 0.33655E+00_JPRB ,&
&  0.33770E+00_JPRB, 0.33799E+00_JPRB, 0.33816E+00_JPRB, 0.33833E+00_JPRB, 0.33617E+00_JPRB, 0.33688E+00_JPRB, 0.33871E+00_JPRB ,&
&  0.33908E+00_JPRB, 0.33647E+00_JPRB, 0.33233E+00_JPRB, 0.32896E+00_JPRB, 0.32560E+00_JPRB, 0.32301E+00_JPRB, 0.32079E+00_JPRB ,&
&  0.31905E+00_JPRB, 0.31798E+00_JPRB, 0.31750E+00_JPRB, 0.31542E+00_JPRB, 0.30961E+00_JPRB, 0.30404E+00_JPRB, 0.30098E+00_JPRB ,&
&  0.29957E+00_JPRB, 0.29915E+00_JPRB, 0.30050E+00_JPRB, 0.29899E+00_JPRB, 0.29438E+00_JPRB, 0.29053E+00_JPRB, 0.28917E+00_JPRB ,&
&  0.28796E+00_JPRB, 0.28774E+00_JPRB, 0.28896E+00_JPRB, 0.28962E+00_JPRB, 0.28993E+00_JPRB, 0.28966E+00_JPRB, 0.29091E+00_JPRB ,&
&  0.29280E+00_JPRB, 0.29502E+00_JPRB, 0.29513E+00_JPRB, 0.29233E+00_JPRB, 0.28835E+00_JPRB, 0.28571E+00_JPRB, 0.28583E+00_JPRB ,&
&  0.28825E+00_JPRB, 0.29053E+00_JPRB, 0.29157E+00_JPRB, 0.29217E+00_JPRB, 0.29345E+00_JPRB, 0.29604E+00_JPRB, 0.29931E+00_JPRB ,&
&  0.30079E+00_JPRB, 0.30218E+00_JPRB, 0.30415E+00_JPRB, 0.30615E+00_JPRB, 0.30741E+00_JPRB, 0.30802E+00_JPRB, 0.30808E+00_JPRB ,&
&  0.30738E+00_JPRB, 0.30796E+00_JPRB, 0.30939E+00_JPRB, 0.31114E+00_JPRB, 0.31216E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.29530E+00_JPRB, 0.29854E+00_JPRB, 0.30164E+00_JPRB, 0.30319E+00_JPRB, 0.30481E+00_JPRB, 0.30674E+00_JPRB, 0.31028E+00_JPRB ,&
&  0.31463E+00_JPRB, 0.31638E+00_JPRB, 0.31489E+00_JPRB, 0.31213E+00_JPRB, 0.31050E+00_JPRB, 0.31003E+00_JPRB, 0.31016E+00_JPRB ,&
&  0.31025E+00_JPRB, 0.31024E+00_JPRB, 0.30937E+00_JPRB, 0.30727E+00_JPRB, 0.30425E+00_JPRB, 0.30109E+00_JPRB, 0.29848E+00_JPRB ,&
&  0.29756E+00_JPRB, 0.29850E+00_JPRB, 0.30031E+00_JPRB, 0.30008E+00_JPRB, 0.30003E+00_JPRB, 0.30134E+00_JPRB, 0.30361E+00_JPRB ,&
&  0.30767E+00_JPRB, 0.30941E+00_JPRB, 0.31083E+00_JPRB, 0.30705E+00_JPRB, 0.31086E+00_JPRB, 0.31226E+00_JPRB, 0.31250E+00_JPRB ,&
&  0.31353E+00_JPRB, 0.31444E+00_JPRB, 0.31562E+00_JPRB, 0.31512E+00_JPRB, 0.31117E+00_JPRB, 0.31039E+00_JPRB, 0.31281E+00_JPRB ,&
&  0.31477E+00_JPRB, 0.31333E+00_JPRB, 0.31019E+00_JPRB, 0.30723E+00_JPRB, 0.30466E+00_JPRB, 0.30269E+00_JPRB, 0.30104E+00_JPRB ,&
&  0.30027E+00_JPRB, 0.29929E+00_JPRB, 0.29695E+00_JPRB, 0.29126E+00_JPRB, 0.28495E+00_JPRB, 0.28068E+00_JPRB, 0.27755E+00_JPRB ,&
&  0.27726E+00_JPRB, 0.27913E+00_JPRB, 0.28100E+00_JPRB, 0.28110E+00_JPRB, 0.27938E+00_JPRB, 0.27622E+00_JPRB, 0.27215E+00_JPRB ,&
&  0.26714E+00_JPRB, 0.26555E+00_JPRB, 0.26559E+00_JPRB, 0.26501E+00_JPRB, 0.26548E+00_JPRB, 0.26736E+00_JPRB, 0.26949E+00_JPRB ,&
&  0.27220E+00_JPRB, 0.27197E+00_JPRB, 0.27030E+00_JPRB, 0.26762E+00_JPRB, 0.26491E+00_JPRB, 0.26478E+00_JPRB, 0.26588E+00_JPRB ,&
&  0.26710E+00_JPRB, 0.26831E+00_JPRB, 0.26988E+00_JPRB, 0.27176E+00_JPRB, 0.27392E+00_JPRB, 0.27664E+00_JPRB, 0.27871E+00_JPRB ,&
&  0.27859E+00_JPRB, 0.27735E+00_JPRB, 0.27855E+00_JPRB, 0.28076E+00_JPRB, 0.28194E+00_JPRB, 0.28257E+00_JPRB, 0.28294E+00_JPRB ,&
&  0.28238E+00_JPRB, 0.28209E+00_JPRB, 0.28385E+00_JPRB, 0.28835E+00_JPRB, 0.29249E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.27970E+00_JPRB, 0.28113E+00_JPRB, 0.28131E+00_JPRB, 0.28182E+00_JPRB, 0.28250E+00_JPRB, 0.28443E+00_JPRB, 0.28872E+00_JPRB ,&
&  0.29306E+00_JPRB, 0.29507E+00_JPRB, 0.29337E+00_JPRB, 0.29092E+00_JPRB, 0.28968E+00_JPRB, 0.28919E+00_JPRB, 0.28920E+00_JPRB ,&
&  0.28918E+00_JPRB, 0.28823E+00_JPRB, 0.28524E+00_JPRB, 0.28104E+00_JPRB, 0.27766E+00_JPRB, 0.27635E+00_JPRB, 0.27626E+00_JPRB ,&
&  0.27966E+00_JPRB, 0.28194E+00_JPRB, 0.28405E+00_JPRB, 0.28625E+00_JPRB, 0.28710E+00_JPRB, 0.28710E+00_JPRB, 0.28887E+00_JPRB ,&
&  0.28969E+00_JPRB, 0.28648E+00_JPRB, 0.28086E+00_JPRB, 0.28426E+00_JPRB, 0.28662E+00_JPRB, 0.28801E+00_JPRB, 0.28723E+00_JPRB ,&
&  0.28780E+00_JPRB, 0.28997E+00_JPRB, 0.29235E+00_JPRB, 0.29415E+00_JPRB, 0.29354E+00_JPRB, 0.29110E+00_JPRB, 0.29103E+00_JPRB ,&
&  0.29203E+00_JPRB, 0.29106E+00_JPRB, 0.28802E+00_JPRB, 0.28518E+00_JPRB, 0.28197E+00_JPRB, 0.28046E+00_JPRB, 0.28028E+00_JPRB ,&
&  0.27955E+00_JPRB, 0.27738E+00_JPRB, 0.27307E+00_JPRB, 0.26869E+00_JPRB, 0.26540E+00_JPRB, 0.26191E+00_JPRB, 0.25995E+00_JPRB ,&
&  0.26002E+00_JPRB, 0.26102E+00_JPRB, 0.26094E+00_JPRB, 0.25898E+00_JPRB, 0.25257E+00_JPRB, 0.24367E+00_JPRB, 0.23631E+00_JPRB ,&
&  0.23497E+00_JPRB, 0.23914E+00_JPRB, 0.24642E+00_JPRB, 0.24932E+00_JPRB, 0.25230E+00_JPRB, 0.25404E+00_JPRB, 0.25513E+00_JPRB ,&
&  0.25228E+00_JPRB, 0.24011E+00_JPRB, 0.23472E+00_JPRB, 0.23119E+00_JPRB, 0.22808E+00_JPRB, 0.22825E+00_JPRB, 0.23035E+00_JPRB ,&
&  0.23444E+00_JPRB, 0.23880E+00_JPRB, 0.24223E+00_JPRB, 0.24445E+00_JPRB, 0.24529E+00_JPRB, 0.24552E+00_JPRB, 0.24487E+00_JPRB ,&
&  0.24550E+00_JPRB, 0.24992E+00_JPRB, 0.25820E+00_JPRB, 0.26485E+00_JPRB, 0.26597E+00_JPRB, 0.26750E+00_JPRB, 0.26741E+00_JPRB ,&
&  0.26744E+00_JPRB, 0.26943E+00_JPRB, 0.27053E+00_JPRB, 0.27322E+00_JPRB, 0.27676E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.26693E+00_JPRB, 0.26821E+00_JPRB, 0.26837E+00_JPRB, 0.26731E+00_JPRB, 0.26591E+00_JPRB, 0.26589E+00_JPRB, 0.26718E+00_JPRB ,&
&  0.27008E+00_JPRB, 0.27211E+00_JPRB, 0.27244E+00_JPRB, 0.27235E+00_JPRB, 0.27141E+00_JPRB, 0.26946E+00_JPRB, 0.26795E+00_JPRB ,&
&  0.26686E+00_JPRB, 0.26634E+00_JPRB, 0.26589E+00_JPRB, 0.26578E+00_JPRB, 0.26566E+00_JPRB, 0.26626E+00_JPRB, 0.26671E+00_JPRB ,&
&  0.26999E+00_JPRB, 0.26963E+00_JPRB, 0.26886E+00_JPRB, 0.26864E+00_JPRB, 0.26851E+00_JPRB, 0.26866E+00_JPRB, 0.26847E+00_JPRB ,&
&  0.26485E+00_JPRB, 0.25730E+00_JPRB, 0.25128E+00_JPRB, 0.25789E+00_JPRB, 0.26819E+00_JPRB, 0.26840E+00_JPRB, 0.26882E+00_JPRB ,&
&  0.27031E+00_JPRB, 0.27314E+00_JPRB, 0.27542E+00_JPRB, 0.27639E+00_JPRB, 0.27527E+00_JPRB, 0.27379E+00_JPRB, 0.27351E+00_JPRB ,&
&  0.27399E+00_JPRB, 0.27230E+00_JPRB, 0.26664E+00_JPRB, 0.25992E+00_JPRB, 0.25316E+00_JPRB, 0.25068E+00_JPRB, 0.24928E+00_JPRB ,&
&  0.24664E+00_JPRB, 0.24502E+00_JPRB, 0.24455E+00_JPRB, 0.24416E+00_JPRB, 0.24466E+00_JPRB, 0.24211E+00_JPRB, 0.23796E+00_JPRB ,&
&  0.23528E+00_JPRB, 0.23089E+00_JPRB, 0.22604E+00_JPRB, 0.22092E+00_JPRB, 0.21667E+00_JPRB, 0.21160E+00_JPRB, 0.20809E+00_JPRB ,&
&  0.20770E+00_JPRB, 0.21256E+00_JPRB, 0.22325E+00_JPRB, 0.23300E+00_JPRB, 0.23688E+00_JPRB, 0.22862E+00_JPRB, 0.22278E+00_JPRB ,&
&  0.21521E+00_JPRB, 0.20833E+00_JPRB, 0.20439E+00_JPRB, 0.20364E+00_JPRB, 0.20387E+00_JPRB, 0.20480E+00_JPRB, 0.20686E+00_JPRB ,&
&  0.20960E+00_JPRB, 0.21291E+00_JPRB, 0.21537E+00_JPRB, 0.21736E+00_JPRB, 0.21878E+00_JPRB, 0.21917E+00_JPRB, 0.21888E+00_JPRB ,&
&  0.21863E+00_JPRB, 0.22008E+00_JPRB, 0.22255E+00_JPRB, 0.22718E+00_JPRB, 0.22989E+00_JPRB, 0.23364E+00_JPRB, 0.23837E+00_JPRB ,&
&  0.24562E+00_JPRB, 0.25379E+00_JPRB, 0.25806E+00_JPRB, 0.26103E+00_JPRB, 0.26432E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.25352E+00_JPRB, 0.25841E+00_JPRB, 0.26063E+00_JPRB, 0.25987E+00_JPRB, 0.25924E+00_JPRB, 0.25952E+00_JPRB, 0.25994E+00_JPRB ,&
&  0.26076E+00_JPRB, 0.26007E+00_JPRB, 0.25840E+00_JPRB, 0.25695E+00_JPRB, 0.25542E+00_JPRB, 0.25352E+00_JPRB, 0.25244E+00_JPRB ,&
&  0.25169E+00_JPRB, 0.25214E+00_JPRB, 0.25245E+00_JPRB, 0.25387E+00_JPRB, 0.25463E+00_JPRB, 0.25645E+00_JPRB, 0.25441E+00_JPRB ,&
&  0.25025E+00_JPRB, 0.24768E+00_JPRB, 0.25005E+00_JPRB, 0.25027E+00_JPRB, 0.25077E+00_JPRB, 0.25139E+00_JPRB, 0.25288E+00_JPRB ,&
&  0.24994E+00_JPRB, 0.24728E+00_JPRB, 0.24515E+00_JPRB, 0.25212E+00_JPRB, 0.25902E+00_JPRB, 0.25712E+00_JPRB, 0.25700E+00_JPRB ,&
&  0.25887E+00_JPRB, 0.26152E+00_JPRB, 0.26279E+00_JPRB, 0.26282E+00_JPRB, 0.26136E+00_JPRB, 0.26084E+00_JPRB, 0.25957E+00_JPRB ,&
&  0.25632E+00_JPRB, 0.25355E+00_JPRB, 0.24345E+00_JPRB, 0.22594E+00_JPRB, 0.22025E+00_JPRB, 0.21794E+00_JPRB, 0.21409E+00_JPRB ,&
&  0.20868E+00_JPRB, 0.20694E+00_JPRB, 0.20706E+00_JPRB, 0.20476E+00_JPRB, 0.20484E+00_JPRB, 0.20455E+00_JPRB, 0.20314E+00_JPRB ,&
&  0.20144E+00_JPRB, 0.20037E+00_JPRB, 0.19883E+00_JPRB, 0.19620E+00_JPRB, 0.19420E+00_JPRB, 0.19392E+00_JPRB, 0.19500E+00_JPRB ,&
&  0.19722E+00_JPRB, 0.20026E+00_JPRB, 0.20941E+00_JPRB, 0.21467E+00_JPRB, 0.21277E+00_JPRB, 0.20958E+00_JPRB, 0.20272E+00_JPRB ,&
&  0.19589E+00_JPRB, 0.19386E+00_JPRB, 0.19242E+00_JPRB, 0.19118E+00_JPRB, 0.19159E+00_JPRB, 0.19276E+00_JPRB, 0.19393E+00_JPRB ,&
&  0.19546E+00_JPRB, 0.19738E+00_JPRB, 0.19857E+00_JPRB, 0.19926E+00_JPRB, 0.19990E+00_JPRB, 0.20020E+00_JPRB, 0.20073E+00_JPRB ,&
&  0.20071E+00_JPRB, 0.20217E+00_JPRB, 0.20326E+00_JPRB, 0.20580E+00_JPRB, 0.20747E+00_JPRB, 0.20981E+00_JPRB, 0.21511E+00_JPRB ,&
&  0.21902E+00_JPRB, 0.23110E+00_JPRB, 0.24668E+00_JPRB, 0.25216E+00_JPRB, 0.24846E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.24429E+00_JPRB, 0.24662E+00_JPRB, 0.24823E+00_JPRB, 0.24650E+00_JPRB, 0.24638E+00_JPRB, 0.24652E+00_JPRB, 0.24758E+00_JPRB ,&
&  0.24799E+00_JPRB, 0.24730E+00_JPRB, 0.24688E+00_JPRB, 0.24489E+00_JPRB, 0.24328E+00_JPRB, 0.23686E+00_JPRB, 0.23444E+00_JPRB ,&
&  0.23243E+00_JPRB, 0.23338E+00_JPRB, 0.23397E+00_JPRB, 0.23605E+00_JPRB, 0.23580E+00_JPRB, 0.23432E+00_JPRB, 0.23064E+00_JPRB ,&
&  0.22817E+00_JPRB, 0.22707E+00_JPRB, 0.22564E+00_JPRB, 0.22627E+00_JPRB, 0.22873E+00_JPRB, 0.23111E+00_JPRB, 0.23275E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23669E+00_JPRB, 0.23915E+00_JPRB, 0.24229E+00_JPRB, 0.24838E+00_JPRB, 0.24828E+00_JPRB, 0.24949E+00_JPRB ,&
&  0.25048E+00_JPRB, 0.25144E+00_JPRB, 0.25185E+00_JPRB, 0.25109E+00_JPRB, 0.24903E+00_JPRB, 0.24287E+00_JPRB, 0.24116E+00_JPRB ,&
&  0.22917E+00_JPRB, 0.22206E+00_JPRB, 0.21576E+00_JPRB, 0.21126E+00_JPRB, 0.20495E+00_JPRB, 0.20161E+00_JPRB, 0.19783E+00_JPRB ,&
&  0.19461E+00_JPRB, 0.19110E+00_JPRB, 0.18855E+00_JPRB, 0.18755E+00_JPRB, 0.18722E+00_JPRB, 0.18692E+00_JPRB, 0.18608E+00_JPRB ,&
&  0.18626E+00_JPRB, 0.18578E+00_JPRB, 0.18545E+00_JPRB, 0.18511E+00_JPRB, 0.18488E+00_JPRB, 0.18504E+00_JPRB, 0.18596E+00_JPRB ,&
&  0.18823E+00_JPRB, 0.19018E+00_JPRB, 0.19233E+00_JPRB, 0.19461E+00_JPRB, 0.19426E+00_JPRB, 0.19230E+00_JPRB, 0.18935E+00_JPRB ,&
&  0.18669E+00_JPRB, 0.18497E+00_JPRB, 0.18312E+00_JPRB, 0.18261E+00_JPRB, 0.18332E+00_JPRB, 0.18417E+00_JPRB, 0.18572E+00_JPRB ,&
&  0.18709E+00_JPRB, 0.18823E+00_JPRB, 0.18899E+00_JPRB, 0.18945E+00_JPRB, 0.19020E+00_JPRB, 0.19086E+00_JPRB, 0.19203E+00_JPRB ,&
&  0.19333E+00_JPRB, 0.19491E+00_JPRB, 0.19630E+00_JPRB, 0.19827E+00_JPRB, 0.20137E+00_JPRB, 0.20580E+00_JPRB, 0.21356E+00_JPRB ,&
&  0.22048E+00_JPRB, 0.23844E+00_JPRB, 0.24080E+00_JPRB, 0.24273E+00_JPRB, 0.24469E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.23400E+00_JPRB, 0.23589E+00_JPRB, 0.23673E+00_JPRB, 0.23768E+00_JPRB, 0.23755E+00_JPRB, 0.23757E+00_JPRB, 0.23818E+00_JPRB ,&
&  0.23403E+00_JPRB, 0.23130E+00_JPRB, 0.22657E+00_JPRB, 0.22243E+00_JPRB, 0.21843E+00_JPRB, 0.21524E+00_JPRB, 0.21331E+00_JPRB ,&
&  0.21221E+00_JPRB, 0.21238E+00_JPRB, 0.21220E+00_JPRB, 0.21256E+00_JPRB, 0.21118E+00_JPRB, 0.20996E+00_JPRB, 0.20893E+00_JPRB ,&
&  0.20790E+00_JPRB, 0.20886E+00_JPRB, 0.20980E+00_JPRB, 0.21350E+00_JPRB, 0.21869E+00_JPRB, 0.22436E+00_JPRB, 0.22864E+00_JPRB ,&
&  0.23150E+00_JPRB, 0.23377E+00_JPRB, 0.23686E+00_JPRB, 0.23821E+00_JPRB, 0.23980E+00_JPRB, 0.24128E+00_JPRB, 0.24164E+00_JPRB ,&
&  0.24291E+00_JPRB, 0.24257E+00_JPRB, 0.24219E+00_JPRB, 0.24195E+00_JPRB, 0.23281E+00_JPRB, 0.22638E+00_JPRB, 0.21967E+00_JPRB ,&
&  0.21343E+00_JPRB, 0.20703E+00_JPRB, 0.20123E+00_JPRB, 0.19815E+00_JPRB, 0.19440E+00_JPRB, 0.19225E+00_JPRB, 0.19039E+00_JPRB ,&
&  0.18896E+00_JPRB, 0.18748E+00_JPRB, 0.18535E+00_JPRB, 0.18326E+00_JPRB, 0.18202E+00_JPRB, 0.18099E+00_JPRB, 0.17966E+00_JPRB ,&
&  0.17959E+00_JPRB, 0.17890E+00_JPRB, 0.17897E+00_JPRB, 0.17863E+00_JPRB, 0.17842E+00_JPRB, 0.17823E+00_JPRB, 0.17825E+00_JPRB ,&
&  0.17873E+00_JPRB, 0.17972E+00_JPRB, 0.18018E+00_JPRB, 0.18079E+00_JPRB, 0.18079E+00_JPRB, 0.18009E+00_JPRB, 0.17953E+00_JPRB ,&
&  0.17883E+00_JPRB, 0.17831E+00_JPRB, 0.17836E+00_JPRB, 0.17823E+00_JPRB, 0.17879E+00_JPRB, 0.17954E+00_JPRB, 0.18025E+00_JPRB ,&
&  0.18127E+00_JPRB, 0.18199E+00_JPRB, 0.18275E+00_JPRB, 0.18377E+00_JPRB, 0.18459E+00_JPRB, 0.18566E+00_JPRB, 0.18688E+00_JPRB ,&
&  0.18774E+00_JPRB, 0.18963E+00_JPRB, 0.19120E+00_JPRB, 0.19322E+00_JPRB, 0.19728E+00_JPRB, 0.20107E+00_JPRB, 0.21042E+00_JPRB ,&
&  0.21828E+00_JPRB, 0.23404E+00_JPRB, 0.23503E+00_JPRB, 0.23154E+00_JPRB, 0.23263E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.22791E+00_JPRB, 0.22929E+00_JPRB, 0.22963E+00_JPRB, 0.23039E+00_JPRB, 0.23049E+00_JPRB, 0.23183E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.23171E+00_JPRB, 0.22804E+00_JPRB, 0.22582E+00_JPRB, 0.22307E+00_JPRB, 0.22066E+00_JPRB, 0.21768E+00_JPRB, 0.21481E+00_JPRB ,&
&  0.21259E+00_JPRB, 0.21057E+00_JPRB, 0.20837E+00_JPRB, 0.20541E+00_JPRB, 0.20242E+00_JPRB, 0.20037E+00_JPRB, 0.19909E+00_JPRB ,&
&  0.19789E+00_JPRB, 0.19917E+00_JPRB, 0.20128E+00_JPRB, 0.20514E+00_JPRB, 0.21010E+00_JPRB, 0.21398E+00_JPRB, 0.21925E+00_JPRB ,&
&  0.22513E+00_JPRB, 0.22696E+00_JPRB, 0.23009E+00_JPRB, 0.23138E+00_JPRB, 0.23107E+00_JPRB, 0.23182E+00_JPRB, 0.22995E+00_JPRB ,&
&  0.23002E+00_JPRB, 0.22628E+00_JPRB, 0.23078E+00_JPRB, 0.22675E+00_JPRB, 0.21930E+00_JPRB, 0.21631E+00_JPRB, 0.20988E+00_JPRB ,&
&  0.20570E+00_JPRB, 0.20090E+00_JPRB, 0.19785E+00_JPRB, 0.19619E+00_JPRB, 0.19256E+00_JPRB, 0.19066E+00_JPRB, 0.18762E+00_JPRB ,&
&  0.18480E+00_JPRB, 0.18399E+00_JPRB, 0.18232E+00_JPRB, 0.18174E+00_JPRB, 0.18088E+00_JPRB, 0.17961E+00_JPRB, 0.17822E+00_JPRB ,&
&  0.17733E+00_JPRB, 0.17659E+00_JPRB, 0.17626E+00_JPRB, 0.17573E+00_JPRB, 0.17496E+00_JPRB, 0.17417E+00_JPRB, 0.17323E+00_JPRB ,&
&  0.17265E+00_JPRB, 0.17243E+00_JPRB, 0.17230E+00_JPRB, 0.17241E+00_JPRB, 0.17249E+00_JPRB, 0.17234E+00_JPRB, 0.17257E+00_JPRB ,&
&  0.17291E+00_JPRB, 0.17362E+00_JPRB, 0.17415E+00_JPRB, 0.17488E+00_JPRB, 0.17549E+00_JPRB, 0.17629E+00_JPRB, 0.17682E+00_JPRB ,&
&  0.17746E+00_JPRB, 0.17847E+00_JPRB, 0.17935E+00_JPRB, 0.18040E+00_JPRB, 0.18152E+00_JPRB, 0.18244E+00_JPRB, 0.18399E+00_JPRB ,&
&  0.18556E+00_JPRB, 0.18712E+00_JPRB, 0.18939E+00_JPRB, 0.19162E+00_JPRB, 0.19449E+00_JPRB, 0.19914E+00_JPRB, 0.20435E+00_JPRB ,&
&  0.21089E+00_JPRB, 0.22032E+00_JPRB, 0.22061E+00_JPRB, 0.22448E+00_JPRB, 0.22641E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.22699E+00_JPRB, 0.22832E+00_JPRB, 0.22941E+00_JPRB, 0.22993E+00_JPRB, 0.23066E+00_JPRB, 0.23066E+00_JPRB, 0.23048E+00_JPRB ,&
&  0.22733E+00_JPRB, 0.22606E+00_JPRB, 0.22411E+00_JPRB, 0.22190E+00_JPRB, 0.21996E+00_JPRB, 0.21696E+00_JPRB, 0.21434E+00_JPRB ,&
&  0.21194E+00_JPRB, 0.20918E+00_JPRB, 0.20659E+00_JPRB, 0.20350E+00_JPRB, 0.20065E+00_JPRB, 0.19799E+00_JPRB, 0.19684E+00_JPRB ,&
&  0.19640E+00_JPRB, 0.19712E+00_JPRB, 0.19919E+00_JPRB, 0.20071E+00_JPRB, 0.20284E+00_JPRB, 0.20551E+00_JPRB, 0.20955E+00_JPRB ,&
&  0.21240E+00_JPRB, 0.21607E+00_JPRB, 0.21840E+00_JPRB, 0.21900E+00_JPRB, 0.21832E+00_JPRB, 0.21799E+00_JPRB, 0.21704E+00_JPRB ,&
&  0.21490E+00_JPRB, 0.21273E+00_JPRB, 0.21090E+00_JPRB, 0.20890E+00_JPRB, 0.20678E+00_JPRB, 0.20436E+00_JPRB, 0.20125E+00_JPRB ,&
&  0.19850E+00_JPRB, 0.19608E+00_JPRB, 0.19372E+00_JPRB, 0.19185E+00_JPRB, 0.18998E+00_JPRB, 0.18826E+00_JPRB, 0.18630E+00_JPRB ,&
&  0.18478E+00_JPRB, 0.18348E+00_JPRB, 0.18184E+00_JPRB, 0.18058E+00_JPRB, 0.17912E+00_JPRB, 0.17790E+00_JPRB, 0.17658E+00_JPRB ,&
&  0.17562E+00_JPRB, 0.17480E+00_JPRB, 0.17416E+00_JPRB, 0.17355E+00_JPRB, 0.17323E+00_JPRB, 0.17292E+00_JPRB, 0.17257E+00_JPRB ,&
&  0.17227E+00_JPRB, 0.17241E+00_JPRB, 0.17255E+00_JPRB, 0.17270E+00_JPRB, 0.17289E+00_JPRB, 0.17257E+00_JPRB, 0.17266E+00_JPRB ,&
&  0.17299E+00_JPRB, 0.17332E+00_JPRB, 0.17327E+00_JPRB, 0.17366E+00_JPRB, 0.17435E+00_JPRB, 0.17518E+00_JPRB, 0.17648E+00_JPRB ,&
&  0.17749E+00_JPRB, 0.17854E+00_JPRB, 0.17954E+00_JPRB, 0.18056E+00_JPRB, 0.18192E+00_JPRB, 0.18339E+00_JPRB, 0.18492E+00_JPRB ,&
&  0.18673E+00_JPRB, 0.18840E+00_JPRB, 0.19014E+00_JPRB, 0.19193E+00_JPRB, 0.19483E+00_JPRB, 0.19745E+00_JPRB, 0.20153E+00_JPRB ,&
&  0.20560E+00_JPRB, 0.20697E+00_JPRB, 0.22116E+00_JPRB, 0.22220E+00_JPRB, 0.22384E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.20539E+00_JPRB, 0.20756E+00_JPRB, 0.20883E+00_JPRB, 0.21010E+00_JPRB, 0.20655E+00_JPRB, 0.21842E+00_JPRB, 0.21613E+00_JPRB ,&
&  0.21570E+00_JPRB, 0.21106E+00_JPRB, 0.21186E+00_JPRB, 0.21136E+00_JPRB, 0.21001E+00_JPRB, 0.20845E+00_JPRB, 0.20663E+00_JPRB ,&
&  0.20483E+00_JPRB, 0.20250E+00_JPRB, 0.20068E+00_JPRB, 0.19917E+00_JPRB, 0.19778E+00_JPRB, 0.19659E+00_JPRB, 0.19540E+00_JPRB ,&
&  0.19539E+00_JPRB, 0.19623E+00_JPRB, 0.19748E+00_JPRB, 0.19659E+00_JPRB, 0.19694E+00_JPRB, 0.19782E+00_JPRB, 0.19907E+00_JPRB ,&
&  0.20006E+00_JPRB, 0.20155E+00_JPRB, 0.20254E+00_JPRB, 0.20286E+00_JPRB, 0.20360E+00_JPRB, 0.20289E+00_JPRB, 0.20238E+00_JPRB ,&
&  0.20158E+00_JPRB, 0.20046E+00_JPRB, 0.19904E+00_JPRB, 0.19782E+00_JPRB, 0.19648E+00_JPRB, 0.19480E+00_JPRB, 0.19307E+00_JPRB ,&
&  0.19151E+00_JPRB, 0.19015E+00_JPRB, 0.18863E+00_JPRB, 0.18739E+00_JPRB, 0.18620E+00_JPRB, 0.18502E+00_JPRB, 0.18390E+00_JPRB ,&
&  0.18301E+00_JPRB, 0.18222E+00_JPRB, 0.18136E+00_JPRB, 0.18050E+00_JPRB, 0.17956E+00_JPRB, 0.17867E+00_JPRB, 0.17781E+00_JPRB ,&
&  0.17747E+00_JPRB, 0.17718E+00_JPRB, 0.17687E+00_JPRB, 0.17654E+00_JPRB, 0.17651E+00_JPRB, 0.17645E+00_JPRB, 0.17633E+00_JPRB ,&
&  0.17619E+00_JPRB, 0.17594E+00_JPRB, 0.17587E+00_JPRB, 0.17579E+00_JPRB, 0.17571E+00_JPRB, 0.17565E+00_JPRB, 0.17576E+00_JPRB ,&
&  0.17595E+00_JPRB, 0.17622E+00_JPRB, 0.17678E+00_JPRB, 0.17736E+00_JPRB, 0.17799E+00_JPRB, 0.17857E+00_JPRB, 0.17941E+00_JPRB ,&
&  0.18034E+00_JPRB, 0.18122E+00_JPRB, 0.18197E+00_JPRB, 0.18260E+00_JPRB, 0.18367E+00_JPRB, 0.18469E+00_JPRB, 0.18570E+00_JPRB ,&
&  0.18679E+00_JPRB, 0.18819E+00_JPRB, 0.18965E+00_JPRB, 0.19115E+00_JPRB, 0.19293E+00_JPRB, 0.19435E+00_JPRB, 0.19580E+00_JPRB ,&
&  0.19753E+00_JPRB, 0.20078E+00_JPRB, 0.20282E+00_JPRB, 0.20409E+00_JPRB, 0.20537E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.18995E+00_JPRB, 0.19009E+00_JPRB, 0.19011E+00_JPRB, 0.19004E+00_JPRB, 0.18974E+00_JPRB, 0.18965E+00_JPRB, 0.18957E+00_JPRB ,&
&  0.18949E+00_JPRB, 0.18924E+00_JPRB, 0.18945E+00_JPRB, 0.18953E+00_JPRB, 0.18955E+00_JPRB, 0.18954E+00_JPRB, 0.18936E+00_JPRB ,&
&  0.18899E+00_JPRB, 0.18853E+00_JPRB, 0.18736E+00_JPRB, 0.18715E+00_JPRB, 0.18714E+00_JPRB, 0.18735E+00_JPRB, 0.18750E+00_JPRB ,&
&  0.18777E+00_JPRB, 0.18805E+00_JPRB, 0.18829E+00_JPRB, 0.18902E+00_JPRB, 0.18933E+00_JPRB, 0.18968E+00_JPRB, 0.19002E+00_JPRB ,&
&  0.19035E+00_JPRB, 0.19073E+00_JPRB, 0.19104E+00_JPRB, 0.19135E+00_JPRB, 0.19165E+00_JPRB, 0.19161E+00_JPRB, 0.19137E+00_JPRB ,&
&  0.19100E+00_JPRB, 0.19042E+00_JPRB, 0.18981E+00_JPRB, 0.18925E+00_JPRB, 0.18874E+00_JPRB, 0.18821E+00_JPRB, 0.18771E+00_JPRB ,&
&  0.18717E+00_JPRB, 0.18657E+00_JPRB, 0.18595E+00_JPRB, 0.18538E+00_JPRB, 0.18480E+00_JPRB, 0.18429E+00_JPRB, 0.18371E+00_JPRB ,&
&  0.18327E+00_JPRB, 0.18282E+00_JPRB, 0.18240E+00_JPRB, 0.18195E+00_JPRB, 0.18157E+00_JPRB, 0.18117E+00_JPRB, 0.18079E+00_JPRB ,&
&  0.18038E+00_JPRB, 0.18019E+00_JPRB, 0.17992E+00_JPRB, 0.17968E+00_JPRB, 0.17946E+00_JPRB, 0.17930E+00_JPRB, 0.17908E+00_JPRB ,&
&  0.17887E+00_JPRB, 0.17867E+00_JPRB, 0.17860E+00_JPRB, 0.17846E+00_JPRB, 0.17842E+00_JPRB, 0.17840E+00_JPRB, 0.17848E+00_JPRB ,&
&  0.17854E+00_JPRB, 0.17865E+00_JPRB, 0.17884E+00_JPRB, 0.17915E+00_JPRB, 0.17943E+00_JPRB, 0.17973E+00_JPRB, 0.18004E+00_JPRB ,&
&  0.18039E+00_JPRB, 0.18073E+00_JPRB, 0.18111E+00_JPRB, 0.18146E+00_JPRB, 0.18191E+00_JPRB, 0.18237E+00_JPRB, 0.18289E+00_JPRB ,&
&  0.18336E+00_JPRB, 0.18389E+00_JPRB, 0.18434E+00_JPRB, 0.18489E+00_JPRB, 0.18535E+00_JPRB, 0.18564E+00_JPRB, 0.18590E+00_JPRB ,&
&  0.18637E+00_JPRB, 0.18681E+00_JPRB, 0.18734E+00_JPRB, 0.18781E+00_JPRB, 0.18865E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB, 0.18233E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB, 0.60202E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.59393E+00_JPRB, 0.59409E+00_JPRB, 0.59421E+00_JPRB, 0.59434E+00_JPRB, 0.59444E+00_JPRB, 0.59454E+00_JPRB, 0.59467E+00_JPRB ,&
&  0.59472E+00_JPRB, 0.59495E+00_JPRB, 0.59528E+00_JPRB, 0.59566E+00_JPRB, 0.59602E+00_JPRB, 0.59637E+00_JPRB, 0.59671E+00_JPRB ,&
&  0.59703E+00_JPRB, 0.59721E+00_JPRB, 0.59744E+00_JPRB, 0.59789E+00_JPRB, 0.59848E+00_JPRB, 0.59918E+00_JPRB, 0.60018E+00_JPRB ,&
&  0.60058E+00_JPRB, 0.60047E+00_JPRB, 0.60039E+00_JPRB, 0.60094E+00_JPRB, 0.60083E+00_JPRB, 0.60080E+00_JPRB, 0.60084E+00_JPRB ,&
&  0.60083E+00_JPRB, 0.60076E+00_JPRB, 0.60055E+00_JPRB, 0.60033E+00_JPRB, 0.60028E+00_JPRB, 0.60044E+00_JPRB, 0.60067E+00_JPRB ,&
&  0.60102E+00_JPRB, 0.60132E+00_JPRB, 0.60203E+00_JPRB, 0.60280E+00_JPRB, 0.60369E+00_JPRB, 0.60432E+00_JPRB, 0.60503E+00_JPRB ,&
&  0.60550E+00_JPRB, 0.60604E+00_JPRB, 0.60682E+00_JPRB, 0.60789E+00_JPRB, 0.60902E+00_JPRB, 0.61008E+00_JPRB, 0.61092E+00_JPRB ,&
&  0.61214E+00_JPRB, 0.61372E+00_JPRB, 0.61542E+00_JPRB, 0.61727E+00_JPRB, 0.61874E+00_JPRB, 0.62027E+00_JPRB, 0.62190E+00_JPRB ,&
&  0.62330E+00_JPRB, 0.62410E+00_JPRB, 0.62504E+00_JPRB, 0.62603E+00_JPRB, 0.62672E+00_JPRB, 0.62619E+00_JPRB, 0.62579E+00_JPRB ,&
&  0.62543E+00_JPRB, 0.62478E+00_JPRB, 0.62294E+00_JPRB, 0.62138E+00_JPRB, 0.61995E+00_JPRB, 0.61846E+00_JPRB, 0.61678E+00_JPRB ,&
&  0.61527E+00_JPRB, 0.61390E+00_JPRB, 0.61241E+00_JPRB, 0.61081E+00_JPRB, 0.60938E+00_JPRB, 0.60801E+00_JPRB, 0.60669E+00_JPRB ,&
&  0.60547E+00_JPRB, 0.60442E+00_JPRB, 0.60342E+00_JPRB, 0.60250E+00_JPRB, 0.60145E+00_JPRB, 0.60046E+00_JPRB, 0.59946E+00_JPRB ,&
&  0.59857E+00_JPRB, 0.59764E+00_JPRB, 0.59678E+00_JPRB, 0.59595E+00_JPRB, 0.59537E+00_JPRB, 0.59496E+00_JPRB, 0.59461E+00_JPRB ,&
&  0.59431E+00_JPRB, 0.59413E+00_JPRB, 0.59407E+00_JPRB, 0.59397E+00_JPRB, 0.59388E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.59073E+00_JPRB, 0.59127E+00_JPRB, 0.59175E+00_JPRB, 0.59214E+00_JPRB, 0.59266E+00_JPRB, 0.59326E+00_JPRB, 0.59377E+00_JPRB ,&
&  0.59416E+00_JPRB, 0.59482E+00_JPRB, 0.59558E+00_JPRB, 0.59635E+00_JPRB, 0.59706E+00_JPRB, 0.59784E+00_JPRB, 0.59856E+00_JPRB ,&
&  0.59927E+00_JPRB, 0.59994E+00_JPRB, 0.60064E+00_JPRB, 0.60128E+00_JPRB, 0.60194E+00_JPRB, 0.60236E+00_JPRB, 0.60416E+00_JPRB ,&
&  0.60569E+00_JPRB, 0.60633E+00_JPRB, 0.60565E+00_JPRB, 0.60063E+00_JPRB, 0.59699E+00_JPRB, 0.59371E+00_JPRB, 0.59099E+00_JPRB ,&
&  0.58817E+00_JPRB, 0.58820E+00_JPRB, 0.58867E+00_JPRB, 0.58920E+00_JPRB, 0.59164E+00_JPRB, 0.59126E+00_JPRB, 0.59064E+00_JPRB ,&
&  0.58902E+00_JPRB, 0.58490E+00_JPRB, 0.58555E+00_JPRB, 0.58636E+00_JPRB, 0.58722E+00_JPRB, 0.58954E+00_JPRB, 0.59135E+00_JPRB ,&
&  0.59354E+00_JPRB, 0.59597E+00_JPRB, 0.60034E+00_JPRB, 0.60522E+00_JPRB, 0.60965E+00_JPRB, 0.61333E+00_JPRB, 0.61690E+00_JPRB ,&
&  0.62001E+00_JPRB, 0.62323E+00_JPRB, 0.62674E+00_JPRB, 0.63086E+00_JPRB, 0.63408E+00_JPRB, 0.63759E+00_JPRB, 0.64042E+00_JPRB ,&
&  0.64203E+00_JPRB, 0.64269E+00_JPRB, 0.64464E+00_JPRB, 0.64749E+00_JPRB, 0.65046E+00_JPRB, 0.65077E+00_JPRB, 0.65000E+00_JPRB ,&
&  0.64825E+00_JPRB, 0.64645E+00_JPRB, 0.64330E+00_JPRB, 0.64041E+00_JPRB, 0.63728E+00_JPRB, 0.63406E+00_JPRB, 0.63026E+00_JPRB ,&
&  0.62646E+00_JPRB, 0.62280E+00_JPRB, 0.61995E+00_JPRB, 0.61725E+00_JPRB, 0.61465E+00_JPRB, 0.61223E+00_JPRB, 0.61066E+00_JPRB ,&
&  0.60901E+00_JPRB, 0.60760E+00_JPRB, 0.60611E+00_JPRB, 0.60461E+00_JPRB, 0.60219E+00_JPRB, 0.59983E+00_JPRB, 0.59732E+00_JPRB ,&
&  0.59528E+00_JPRB, 0.59325E+00_JPRB, 0.59160E+00_JPRB, 0.59019E+00_JPRB, 0.58955E+00_JPRB, 0.58921E+00_JPRB, 0.58904E+00_JPRB ,&
&  0.58896E+00_JPRB, 0.58923E+00_JPRB, 0.58962E+00_JPRB, 0.58994E+00_JPRB, 0.59028E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.59510E+00_JPRB, 0.59602E+00_JPRB, 0.59693E+00_JPRB, 0.59743E+00_JPRB, 0.59848E+00_JPRB, 0.59943E+00_JPRB, 0.60035E+00_JPRB ,&
&  0.60125E+00_JPRB, 0.60188E+00_JPRB, 0.60241E+00_JPRB, 0.60300E+00_JPRB, 0.60348E+00_JPRB, 0.60457E+00_JPRB, 0.60557E+00_JPRB ,&
&  0.60641E+00_JPRB, 0.60690E+00_JPRB, 0.60837E+00_JPRB, 0.60972E+00_JPRB, 0.61037E+00_JPRB, 0.61088E+00_JPRB, 0.60679E+00_JPRB ,&
&  0.60421E+00_JPRB, 0.60250E+00_JPRB, 0.60031E+00_JPRB, 0.59535E+00_JPRB, 0.59288E+00_JPRB, 0.57869E+00_JPRB, 0.58321E+00_JPRB ,&
&  0.60110E+00_JPRB, 0.60006E+00_JPRB, 0.59620E+00_JPRB, 0.58957E+00_JPRB, 0.56593E+00_JPRB, 0.55563E+00_JPRB, 0.55383E+00_JPRB ,&
&  0.53969E+00_JPRB, 0.55280E+00_JPRB, 0.55659E+00_JPRB, 0.55825E+00_JPRB, 0.56151E+00_JPRB, 0.55937E+00_JPRB, 0.55988E+00_JPRB ,&
&  0.56944E+00_JPRB, 0.57000E+00_JPRB, 0.57543E+00_JPRB, 0.58213E+00_JPRB, 0.58938E+00_JPRB, 0.59622E+00_JPRB, 0.60395E+00_JPRB ,&
&  0.61014E+00_JPRB, 0.61467E+00_JPRB, 0.61777E+00_JPRB, 0.61893E+00_JPRB, 0.62045E+00_JPRB, 0.62230E+00_JPRB, 0.62617E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.63897E+00_JPRB, 0.64244E+00_JPRB, 0.64154E+00_JPRB, 0.64272E+00_JPRB, 0.64355E+00_JPRB, 0.64434E+00_JPRB ,&
&  0.64420E+00_JPRB, 0.64337E+00_JPRB, 0.64045E+00_JPRB, 0.63974E+00_JPRB, 0.63916E+00_JPRB, 0.63742E+00_JPRB, 0.63464E+00_JPRB ,&
&  0.63112E+00_JPRB, 0.62621E+00_JPRB, 0.62220E+00_JPRB, 0.61942E+00_JPRB, 0.61694E+00_JPRB, 0.61487E+00_JPRB, 0.61416E+00_JPRB ,&
&  0.61253E+00_JPRB, 0.61128E+00_JPRB, 0.60887E+00_JPRB, 0.60551E+00_JPRB, 0.60185E+00_JPRB, 0.59886E+00_JPRB, 0.59564E+00_JPRB ,&
&  0.59380E+00_JPRB, 0.59264E+00_JPRB, 0.59199E+00_JPRB, 0.59085E+00_JPRB, 0.59068E+00_JPRB, 0.59063E+00_JPRB, 0.59107E+00_JPRB ,&
&  0.59154E+00_JPRB, 0.59236E+00_JPRB, 0.59291E+00_JPRB, 0.59366E+00_JPRB, 0.59403E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.59673E+00_JPRB, 0.59683E+00_JPRB, 0.59724E+00_JPRB, 0.59752E+00_JPRB, 0.59844E+00_JPRB, 0.59916E+00_JPRB, 0.60119E+00_JPRB ,&
&  0.60312E+00_JPRB, 0.60499E+00_JPRB, 0.60678E+00_JPRB, 0.60825E+00_JPRB, 0.60970E+00_JPRB, 0.61030E+00_JPRB, 0.61054E+00_JPRB ,&
&  0.61068E+00_JPRB, 0.61055E+00_JPRB, 0.61137E+00_JPRB, 0.61243E+00_JPRB, 0.61264E+00_JPRB, 0.61346E+00_JPRB, 0.60914E+00_JPRB ,&
&  0.60588E+00_JPRB, 0.60370E+00_JPRB, 0.58885E+00_JPRB, 0.59610E+00_JPRB, 0.59458E+00_JPRB, 0.59549E+00_JPRB, 0.59292E+00_JPRB ,&
&  0.57657E+00_JPRB, 0.56943E+00_JPRB, 0.55157E+00_JPRB, 0.53682E+00_JPRB, 0.53823E+00_JPRB, 0.52821E+00_JPRB, 0.52953E+00_JPRB ,&
&  0.52760E+00_JPRB, 0.52366E+00_JPRB, 0.52197E+00_JPRB, 0.52079E+00_JPRB, 0.52336E+00_JPRB, 0.52533E+00_JPRB, 0.52763E+00_JPRB ,&
&  0.53377E+00_JPRB, 0.53916E+00_JPRB, 0.54728E+00_JPRB, 0.54862E+00_JPRB, 0.55857E+00_JPRB, 0.56975E+00_JPRB, 0.58132E+00_JPRB ,&
&  0.58851E+00_JPRB, 0.58698E+00_JPRB, 0.59036E+00_JPRB, 0.59591E+00_JPRB, 0.60047E+00_JPRB, 0.60531E+00_JPRB, 0.61289E+00_JPRB ,&
&  0.62264E+00_JPRB, 0.63189E+00_JPRB, 0.63516E+00_JPRB, 0.63371E+00_JPRB, 0.63099E+00_JPRB, 0.62868E+00_JPRB, 0.62870E+00_JPRB ,&
&  0.62871E+00_JPRB, 0.63131E+00_JPRB, 0.63226E+00_JPRB, 0.63495E+00_JPRB, 0.63655E+00_JPRB, 0.64009E+00_JPRB, 0.64239E+00_JPRB ,&
&  0.64361E+00_JPRB, 0.64335E+00_JPRB, 0.64494E+00_JPRB, 0.63983E+00_JPRB, 0.63878E+00_JPRB, 0.63693E+00_JPRB, 0.63148E+00_JPRB ,&
&  0.62610E+00_JPRB, 0.62185E+00_JPRB, 0.61743E+00_JPRB, 0.61336E+00_JPRB, 0.60973E+00_JPRB, 0.60712E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.60424E+00_JPRB, 0.60276E+00_JPRB, 0.60149E+00_JPRB, 0.59831E+00_JPRB, 0.59758E+00_JPRB, 0.59555E+00_JPRB, 0.59442E+00_JPRB ,&
&  0.59362E+00_JPRB, 0.59424E+00_JPRB, 0.59426E+00_JPRB, 0.59530E+00_JPRB, 0.59638E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.59152E+00_JPRB, 0.59409E+00_JPRB, 0.59878E+00_JPRB, 0.60281E+00_JPRB, 0.60661E+00_JPRB, 0.60967E+00_JPRB, 0.61130E+00_JPRB ,&
&  0.61253E+00_JPRB, 0.61292E+00_JPRB, 0.61336E+00_JPRB, 0.61403E+00_JPRB, 0.61521E+00_JPRB, 0.61611E+00_JPRB, 0.61733E+00_JPRB ,&
&  0.61769E+00_JPRB, 0.61781E+00_JPRB, 0.61764E+00_JPRB, 0.61751E+00_JPRB, 0.61767E+00_JPRB, 0.61892E+00_JPRB, 0.61800E+00_JPRB ,&
&  0.61823E+00_JPRB, 0.61420E+00_JPRB, 0.60319E+00_JPRB, 0.60664E+00_JPRB, 0.60753E+00_JPRB, 0.60852E+00_JPRB, 0.60185E+00_JPRB ,&
&  0.58385E+00_JPRB, 0.55296E+00_JPRB, 0.56956E+00_JPRB, 0.55860E+00_JPRB, 0.54709E+00_JPRB, 0.53520E+00_JPRB, 0.51759E+00_JPRB ,&
&  0.50687E+00_JPRB, 0.49298E+00_JPRB, 0.48265E+00_JPRB, 0.47905E+00_JPRB, 0.47863E+00_JPRB, 0.48383E+00_JPRB, 0.49109E+00_JPRB ,&
&  0.49453E+00_JPRB, 0.50467E+00_JPRB, 0.51088E+00_JPRB, 0.51912E+00_JPRB, 0.52915E+00_JPRB, 0.54206E+00_JPRB, 0.54709E+00_JPRB ,&
&  0.55605E+00_JPRB, 0.55957E+00_JPRB, 0.56869E+00_JPRB, 0.58071E+00_JPRB, 0.59699E+00_JPRB, 0.60716E+00_JPRB, 0.61375E+00_JPRB ,&
&  0.62153E+00_JPRB, 0.62552E+00_JPRB, 0.62778E+00_JPRB, 0.62881E+00_JPRB, 0.63137E+00_JPRB, 0.63427E+00_JPRB, 0.63754E+00_JPRB ,&
&  0.64064E+00_JPRB, 0.64571E+00_JPRB, 0.65118E+00_JPRB, 0.65746E+00_JPRB, 0.66136E+00_JPRB, 0.66836E+00_JPRB, 0.67661E+00_JPRB ,&
&  0.68411E+00_JPRB, 0.68811E+00_JPRB, 0.69649E+00_JPRB, 0.68931E+00_JPRB, 0.68634E+00_JPRB, 0.67324E+00_JPRB, 0.66379E+00_JPRB ,&
&  0.65654E+00_JPRB, 0.64765E+00_JPRB, 0.64014E+00_JPRB, 0.63370E+00_JPRB, 0.62870E+00_JPRB, 0.62421E+00_JPRB, 0.61975E+00_JPRB ,&
&  0.61523E+00_JPRB, 0.60964E+00_JPRB, 0.60615E+00_JPRB, 0.60174E+00_JPRB, 0.59920E+00_JPRB, 0.59585E+00_JPRB, 0.59388E+00_JPRB ,&
&  0.59223E+00_JPRB, 0.59161E+00_JPRB, 0.59089E+00_JPRB, 0.59151E+00_JPRB, 0.59174E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.59477E+00_JPRB, 0.59943E+00_JPRB, 0.60512E+00_JPRB, 0.60987E+00_JPRB, 0.61214E+00_JPRB, 0.61401E+00_JPRB, 0.61256E+00_JPRB ,&
&  0.61167E+00_JPRB, 0.61018E+00_JPRB, 0.60912E+00_JPRB, 0.60920E+00_JPRB, 0.61024E+00_JPRB, 0.61215E+00_JPRB, 0.61452E+00_JPRB ,&
&  0.61665E+00_JPRB, 0.61931E+00_JPRB, 0.62033E+00_JPRB, 0.62166E+00_JPRB, 0.62330E+00_JPRB, 0.62698E+00_JPRB, 0.62851E+00_JPRB ,&
&  0.63036E+00_JPRB, 0.62557E+00_JPRB, 0.61476E+00_JPRB, 0.61135E+00_JPRB, 0.61621E+00_JPRB, 0.60538E+00_JPRB, 0.60608E+00_JPRB ,&
&  0.61313E+00_JPRB, 0.61412E+00_JPRB, 0.60299E+00_JPRB, 0.59475E+00_JPRB, 0.57427E+00_JPRB, 0.51330E+00_JPRB, 0.51201E+00_JPRB ,&
&  0.50063E+00_JPRB, 0.48057E+00_JPRB, 0.47010E+00_JPRB, 0.46099E+00_JPRB, 0.45681E+00_JPRB, 0.46441E+00_JPRB, 0.47653E+00_JPRB ,&
&  0.47964E+00_JPRB, 0.49410E+00_JPRB, 0.49988E+00_JPRB, 0.50924E+00_JPRB, 0.51923E+00_JPRB, 0.53340E+00_JPRB, 0.54125E+00_JPRB ,&
&  0.54985E+00_JPRB, 0.55865E+00_JPRB, 0.57232E+00_JPRB, 0.59021E+00_JPRB, 0.60501E+00_JPRB, 0.61275E+00_JPRB, 0.61167E+00_JPRB ,&
&  0.61987E+00_JPRB, 0.61480E+00_JPRB, 0.61865E+00_JPRB, 0.62362E+00_JPRB, 0.62707E+00_JPRB, 0.63314E+00_JPRB, 0.63556E+00_JPRB ,&
&  0.64446E+00_JPRB, 0.65933E+00_JPRB, 0.67534E+00_JPRB, 0.68418E+00_JPRB, 0.69163E+00_JPRB, 0.70007E+00_JPRB, 0.71377E+00_JPRB ,&
&  0.72779E+00_JPRB, 0.74412E+00_JPRB, 0.75843E+00_JPRB, 0.74352E+00_JPRB, 0.73618E+00_JPRB, 0.71872E+00_JPRB, 0.70606E+00_JPRB ,&
&  0.69052E+00_JPRB, 0.68025E+00_JPRB, 0.66320E+00_JPRB, 0.65344E+00_JPRB, 0.64354E+00_JPRB, 0.63554E+00_JPRB, 0.62760E+00_JPRB ,&
&  0.62079E+00_JPRB, 0.61444E+00_JPRB, 0.61047E+00_JPRB, 0.60804E+00_JPRB, 0.60532E+00_JPRB, 0.60146E+00_JPRB, 0.59844E+00_JPRB ,&
&  0.59358E+00_JPRB, 0.59044E+00_JPRB, 0.58826E+00_JPRB, 0.58847E+00_JPRB, 0.59013E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.59009E+00_JPRB, 0.59640E+00_JPRB, 0.60168E+00_JPRB, 0.60864E+00_JPRB, 0.60873E+00_JPRB, 0.61068E+00_JPRB, 0.60539E+00_JPRB ,&
&  0.60179E+00_JPRB, 0.59982E+00_JPRB, 0.59893E+00_JPRB, 0.60030E+00_JPRB, 0.60264E+00_JPRB, 0.60562E+00_JPRB, 0.60985E+00_JPRB ,&
&  0.61402E+00_JPRB, 0.61316E+00_JPRB, 0.61611E+00_JPRB, 0.62225E+00_JPRB, 0.62491E+00_JPRB, 0.62925E+00_JPRB, 0.63189E+00_JPRB ,&
&  0.63244E+00_JPRB, 0.63010E+00_JPRB, 0.62761E+00_JPRB, 0.61991E+00_JPRB, 0.61701E+00_JPRB, 0.62035E+00_JPRB, 0.62366E+00_JPRB ,&
&  0.60887E+00_JPRB, 0.60209E+00_JPRB, 0.61678E+00_JPRB, 0.62014E+00_JPRB, 0.58433E+00_JPRB, 0.55178E+00_JPRB, 0.53401E+00_JPRB ,&
&  0.51222E+00_JPRB, 0.48130E+00_JPRB, 0.47017E+00_JPRB, 0.46261E+00_JPRB, 0.46491E+00_JPRB, 0.46853E+00_JPRB, 0.47799E+00_JPRB ,&
&  0.48451E+00_JPRB, 0.49315E+00_JPRB, 0.50457E+00_JPRB, 0.51937E+00_JPRB, 0.52935E+00_JPRB, 0.54066E+00_JPRB, 0.54973E+00_JPRB ,&
&  0.55960E+00_JPRB, 0.57265E+00_JPRB, 0.59053E+00_JPRB, 0.61301E+00_JPRB, 0.62933E+00_JPRB, 0.64102E+00_JPRB, 0.64556E+00_JPRB ,&
&  0.66925E+00_JPRB, 0.65482E+00_JPRB, 0.65845E+00_JPRB, 0.66074E+00_JPRB, 0.66195E+00_JPRB, 0.66645E+00_JPRB, 0.66953E+00_JPRB ,&
&  0.68386E+00_JPRB, 0.70627E+00_JPRB, 0.71355E+00_JPRB, 0.71807E+00_JPRB, 0.72209E+00_JPRB, 0.72869E+00_JPRB, 0.74640E+00_JPRB ,&
&  0.76366E+00_JPRB, 0.79548E+00_JPRB, 0.87036E+00_JPRB, 0.83674E+00_JPRB, 0.81127E+00_JPRB, 0.78383E+00_JPRB, 0.75531E+00_JPRB ,&
&  0.72921E+00_JPRB, 0.70578E+00_JPRB, 0.68260E+00_JPRB, 0.66175E+00_JPRB, 0.64677E+00_JPRB, 0.63319E+00_JPRB, 0.62916E+00_JPRB ,&
&  0.62536E+00_JPRB, 0.62056E+00_JPRB, 0.61708E+00_JPRB, 0.61404E+00_JPRB, 0.60834E+00_JPRB, 0.60309E+00_JPRB, 0.59939E+00_JPRB ,&
&  0.59248E+00_JPRB, 0.58895E+00_JPRB, 0.58558E+00_JPRB, 0.58553E+00_JPRB, 0.58626E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.58639E+00_JPRB, 0.59951E+00_JPRB, 0.60867E+00_JPRB, 0.61434E+00_JPRB, 0.61114E+00_JPRB, 0.61188E+00_JPRB, 0.60508E+00_JPRB ,&
&  0.60139E+00_JPRB, 0.60182E+00_JPRB, 0.60236E+00_JPRB, 0.60293E+00_JPRB, 0.60371E+00_JPRB, 0.60427E+00_JPRB, 0.60584E+00_JPRB ,&
&  0.60867E+00_JPRB, 0.61237E+00_JPRB, 0.61515E+00_JPRB, 0.61907E+00_JPRB, 0.62417E+00_JPRB, 0.63027E+00_JPRB, 0.63558E+00_JPRB ,&
&  0.63723E+00_JPRB, 0.63509E+00_JPRB, 0.63745E+00_JPRB, 0.63358E+00_JPRB, 0.62941E+00_JPRB, 0.62424E+00_JPRB, 0.62166E+00_JPRB ,&
&  0.62809E+00_JPRB, 0.63651E+00_JPRB, 0.62778E+00_JPRB, 0.62190E+00_JPRB, 0.61809E+00_JPRB, 0.58820E+00_JPRB, 0.55649E+00_JPRB ,&
&  0.52416E+00_JPRB, 0.49643E+00_JPRB, 0.48353E+00_JPRB, 0.47716E+00_JPRB, 0.48241E+00_JPRB, 0.49449E+00_JPRB, 0.50587E+00_JPRB ,&
&  0.50454E+00_JPRB, 0.52097E+00_JPRB, 0.53775E+00_JPRB, 0.54841E+00_JPRB, 0.55706E+00_JPRB, 0.56574E+00_JPRB, 0.57417E+00_JPRB ,&
&  0.58757E+00_JPRB, 0.60811E+00_JPRB, 0.64445E+00_JPRB, 0.68473E+00_JPRB, 0.71621E+00_JPRB, 0.72723E+00_JPRB, 0.77116E+00_JPRB ,&
&  0.87993E+00_JPRB, 0.81280E+00_JPRB, 0.77671E+00_JPRB, 0.76331E+00_JPRB, 0.74897E+00_JPRB, 0.74445E+00_JPRB, 0.75084E+00_JPRB ,&
&  0.76501E+00_JPRB, 0.77893E+00_JPRB, 0.78153E+00_JPRB, 0.77929E+00_JPRB, 0.78141E+00_JPRB, 0.78899E+00_JPRB, 0.80668E+00_JPRB ,&
&  0.82663E+00_JPRB, 0.88238E+00_JPRB, 0.10172E+01_JPRB, 0.89963E+00_JPRB, 0.83461E+00_JPRB, 0.80238E+00_JPRB, 0.77188E+00_JPRB ,&
&  0.74528E+00_JPRB, 0.72216E+00_JPRB, 0.69840E+00_JPRB, 0.67014E+00_JPRB, 0.64734E+00_JPRB, 0.62746E+00_JPRB, 0.61998E+00_JPRB ,&
&  0.61489E+00_JPRB, 0.60982E+00_JPRB, 0.60905E+00_JPRB, 0.60561E+00_JPRB, 0.60090E+00_JPRB, 0.60042E+00_JPRB, 0.59847E+00_JPRB ,&
&  0.59143E+00_JPRB, 0.58489E+00_JPRB, 0.57976E+00_JPRB, 0.57809E+00_JPRB, 0.57895E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.60142E+00_JPRB, 0.60963E+00_JPRB, 0.61953E+00_JPRB, 0.63911E+00_JPRB, 0.64673E+00_JPRB, 0.64129E+00_JPRB, 0.62435E+00_JPRB ,&
&  0.61388E+00_JPRB, 0.60975E+00_JPRB, 0.61313E+00_JPRB, 0.61745E+00_JPRB, 0.61950E+00_JPRB, 0.61938E+00_JPRB, 0.61732E+00_JPRB ,&
&  0.61335E+00_JPRB, 0.61221E+00_JPRB, 0.61378E+00_JPRB, 0.61721E+00_JPRB, 0.62200E+00_JPRB, 0.62744E+00_JPRB, 0.63427E+00_JPRB ,&
&  0.63964E+00_JPRB, 0.64047E+00_JPRB, 0.64072E+00_JPRB, 0.63864E+00_JPRB, 0.63938E+00_JPRB, 0.64297E+00_JPRB, 0.64142E+00_JPRB ,&
&  0.63320E+00_JPRB, 0.63902E+00_JPRB, 0.64615E+00_JPRB, 0.63482E+00_JPRB, 0.62930E+00_JPRB, 0.60737E+00_JPRB, 0.57635E+00_JPRB ,&
&  0.54954E+00_JPRB, 0.52359E+00_JPRB, 0.51648E+00_JPRB, 0.51849E+00_JPRB, 0.52626E+00_JPRB, 0.52837E+00_JPRB, 0.54462E+00_JPRB ,&
&  0.55824E+00_JPRB, 0.56813E+00_JPRB, 0.57584E+00_JPRB, 0.58121E+00_JPRB, 0.58727E+00_JPRB, 0.59736E+00_JPRB, 0.61775E+00_JPRB ,&
&  0.65513E+00_JPRB, 0.70420E+00_JPRB, 0.75705E+00_JPRB, 0.74017E+00_JPRB, 0.74981E+00_JPRB, 0.77910E+00_JPRB, 0.82496E+00_JPRB ,&
&  0.86227E+00_JPRB, 0.85884E+00_JPRB, 0.85383E+00_JPRB, 0.88811E+00_JPRB, 0.90723E+00_JPRB, 0.92914E+00_JPRB, 0.92607E+00_JPRB ,&
&  0.91251E+00_JPRB, 0.89357E+00_JPRB, 0.88051E+00_JPRB, 0.88114E+00_JPRB, 0.89801E+00_JPRB, 0.91295E+00_JPRB, 0.91202E+00_JPRB ,&
&  0.88838E+00_JPRB, 0.84891E+00_JPRB, 0.80365E+00_JPRB, 0.75978E+00_JPRB, 0.75007E+00_JPRB, 0.75383E+00_JPRB, 0.75505E+00_JPRB ,&
&  0.74049E+00_JPRB, 0.72069E+00_JPRB, 0.69975E+00_JPRB, 0.67327E+00_JPRB, 0.64569E+00_JPRB, 0.61810E+00_JPRB, 0.60426E+00_JPRB ,&
&  0.59636E+00_JPRB, 0.59240E+00_JPRB, 0.59547E+00_JPRB, 0.58677E+00_JPRB, 0.58759E+00_JPRB, 0.59068E+00_JPRB, 0.58319E+00_JPRB ,&
&  0.57692E+00_JPRB, 0.57549E+00_JPRB, 0.57027E+00_JPRB, 0.57360E+00_JPRB, 0.58191E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.62812E+00_JPRB, 0.65424E+00_JPRB, 0.67804E+00_JPRB, 0.69173E+00_JPRB, 0.69663E+00_JPRB, 0.69555E+00_JPRB, 0.67936E+00_JPRB ,&
&  0.67583E+00_JPRB, 0.65107E+00_JPRB, 0.63417E+00_JPRB, 0.61634E+00_JPRB, 0.60289E+00_JPRB, 0.59535E+00_JPRB, 0.59756E+00_JPRB ,&
&  0.60282E+00_JPRB, 0.60811E+00_JPRB, 0.61153E+00_JPRB, 0.61321E+00_JPRB, 0.61679E+00_JPRB, 0.62257E+00_JPRB, 0.63071E+00_JPRB ,&
&  0.63939E+00_JPRB, 0.64512E+00_JPRB, 0.64945E+00_JPRB, 0.65223E+00_JPRB, 0.65391E+00_JPRB, 0.65392E+00_JPRB, 0.64886E+00_JPRB ,&
&  0.64917E+00_JPRB, 0.64793E+00_JPRB, 0.64926E+00_JPRB, 0.64271E+00_JPRB, 0.63808E+00_JPRB, 0.62127E+00_JPRB, 0.58788E+00_JPRB ,&
&  0.57035E+00_JPRB, 0.56110E+00_JPRB, 0.56547E+00_JPRB, 0.56875E+00_JPRB, 0.57803E+00_JPRB, 0.58982E+00_JPRB, 0.59410E+00_JPRB ,&
&  0.59245E+00_JPRB, 0.59326E+00_JPRB, 0.60337E+00_JPRB, 0.61106E+00_JPRB, 0.61987E+00_JPRB, 0.63413E+00_JPRB, 0.65053E+00_JPRB ,&
&  0.68152E+00_JPRB, 0.72997E+00_JPRB, 0.80770E+00_JPRB, 0.83372E+00_JPRB, 0.90326E+00_JPRB, 0.10616E+01_JPRB, 0.98893E+00_JPRB ,&
&  0.10152E+01_JPRB, 0.10392E+01_JPRB, 0.10832E+01_JPRB, 0.11554E+01_JPRB, 0.11364E+01_JPRB, 0.11229E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.10518E+01_JPRB, 0.10357E+01_JPRB, 0.10589E+01_JPRB, 0.10518E+01_JPRB, 0.10423E+01_JPRB, 0.10283E+01_JPRB ,&
&  0.98050E+00_JPRB, 0.92004E+00_JPRB, 0.86050E+00_JPRB, 0.80371E+00_JPRB, 0.77483E+00_JPRB, 0.75795E+00_JPRB, 0.74702E+00_JPRB ,&
&  0.73226E+00_JPRB, 0.72025E+00_JPRB, 0.70894E+00_JPRB, 0.69220E+00_JPRB, 0.66262E+00_JPRB, 0.62785E+00_JPRB, 0.61367E+00_JPRB ,&
&  0.60230E+00_JPRB, 0.59396E+00_JPRB, 0.58878E+00_JPRB, 0.58430E+00_JPRB, 0.58725E+00_JPRB, 0.58622E+00_JPRB, 0.58115E+00_JPRB ,&
&  0.58296E+00_JPRB, 0.57796E+00_JPRB, 0.57323E+00_JPRB, 0.57824E+00_JPRB, 0.60218E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.67537E+00_JPRB, 0.70920E+00_JPRB, 0.72320E+00_JPRB, 0.73224E+00_JPRB, 0.74042E+00_JPRB, 0.73928E+00_JPRB, 0.70813E+00_JPRB ,&
&  0.70060E+00_JPRB, 0.64812E+00_JPRB, 0.62640E+00_JPRB, 0.61014E+00_JPRB, 0.60236E+00_JPRB, 0.59365E+00_JPRB, 0.58422E+00_JPRB ,&
&  0.58703E+00_JPRB, 0.59834E+00_JPRB, 0.61012E+00_JPRB, 0.61520E+00_JPRB, 0.61983E+00_JPRB, 0.62359E+00_JPRB, 0.62830E+00_JPRB ,&
&  0.63606E+00_JPRB, 0.64540E+00_JPRB, 0.65544E+00_JPRB, 0.65951E+00_JPRB, 0.66039E+00_JPRB, 0.66022E+00_JPRB, 0.65857E+00_JPRB ,&
&  0.65730E+00_JPRB, 0.64914E+00_JPRB, 0.64988E+00_JPRB, 0.64744E+00_JPRB, 0.63979E+00_JPRB, 0.63678E+00_JPRB, 0.61593E+00_JPRB ,&
&  0.59340E+00_JPRB, 0.60667E+00_JPRB, 0.61173E+00_JPRB, 0.62328E+00_JPRB, 0.63169E+00_JPRB, 0.63706E+00_JPRB, 0.63905E+00_JPRB ,&
&  0.63765E+00_JPRB, 0.63559E+00_JPRB, 0.63720E+00_JPRB, 0.64131E+00_JPRB, 0.64712E+00_JPRB, 0.65555E+00_JPRB, 0.66360E+00_JPRB ,&
&  0.67276E+00_JPRB, 0.65959E+00_JPRB, 0.71220E+00_JPRB, 0.85773E+00_JPRB, 0.11589E+01_JPRB, 0.11609E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.12934E+01_JPRB, 0.12885E+01_JPRB, 0.13185E+01_JPRB, 0.13453E+01_JPRB, 0.13438E+01_JPRB, 0.13271E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.13059E+01_JPRB, 0.12820E+01_JPRB, 0.12703E+01_JPRB, 0.12330E+01_JPRB, 0.12121E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11254E+01_JPRB, 0.10785E+01_JPRB, 0.10035E+01_JPRB, 0.91695E+00_JPRB, 0.84644E+00_JPRB, 0.80883E+00_JPRB, 0.77622E+00_JPRB ,&
&  0.75284E+00_JPRB, 0.73959E+00_JPRB, 0.72423E+00_JPRB, 0.71511E+00_JPRB, 0.69638E+00_JPRB, 0.67461E+00_JPRB, 0.66808E+00_JPRB ,&
&  0.64289E+00_JPRB, 0.62805E+00_JPRB, 0.61447E+00_JPRB, 0.60783E+00_JPRB, 0.61501E+00_JPRB, 0.62204E+00_JPRB, 0.62499E+00_JPRB ,&
&  0.63383E+00_JPRB, 0.62728E+00_JPRB, 0.62645E+00_JPRB, 0.63444E+00_JPRB, 0.64958E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.71781E+00_JPRB, 0.74275E+00_JPRB, 0.75395E+00_JPRB, 0.76630E+00_JPRB, 0.78172E+00_JPRB, 0.78006E+00_JPRB, 0.77432E+00_JPRB ,&
&  0.79328E+00_JPRB, 0.75737E+00_JPRB, 0.79169E+00_JPRB, 0.67404E+00_JPRB, 0.62813E+00_JPRB, 0.61716E+00_JPRB, 0.60343E+00_JPRB ,&
&  0.60147E+00_JPRB, 0.60720E+00_JPRB, 0.61335E+00_JPRB, 0.61986E+00_JPRB, 0.62684E+00_JPRB, 0.62773E+00_JPRB, 0.62915E+00_JPRB ,&
&  0.63379E+00_JPRB, 0.64359E+00_JPRB, 0.65807E+00_JPRB, 0.66434E+00_JPRB, 0.66928E+00_JPRB, 0.67450E+00_JPRB, 0.67019E+00_JPRB ,&
&  0.66186E+00_JPRB, 0.66309E+00_JPRB, 0.66201E+00_JPRB, 0.65289E+00_JPRB, 0.64299E+00_JPRB, 0.64168E+00_JPRB, 0.65362E+00_JPRB ,&
&  0.65280E+00_JPRB, 0.64439E+00_JPRB, 0.65938E+00_JPRB, 0.66597E+00_JPRB, 0.66858E+00_JPRB, 0.66858E+00_JPRB, 0.66770E+00_JPRB ,&
&  0.66781E+00_JPRB, 0.66656E+00_JPRB, 0.66951E+00_JPRB, 0.67081E+00_JPRB, 0.67287E+00_JPRB, 0.67663E+00_JPRB, 0.67759E+00_JPRB ,&
&  0.68904E+00_JPRB, 0.68131E+00_JPRB, 0.79837E+00_JPRB, 0.93817E+00_JPRB, 0.12088E+01_JPRB, 0.13660E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.16272E+01_JPRB, 0.15664E+01_JPRB, 0.16507E+01_JPRB, 0.16374E+01_JPRB, 0.16102E+01_JPRB, 0.15648E+01_JPRB, 0.15054E+01_JPRB ,&
&  0.14937E+01_JPRB, 0.14803E+01_JPRB, 0.15404E+01_JPRB, 0.15028E+01_JPRB, 0.14203E+01_JPRB, 0.13851E+01_JPRB, 0.13587E+01_JPRB ,&
&  0.13428E+01_JPRB, 0.13199E+01_JPRB, 0.12561E+01_JPRB, 0.10817E+01_JPRB, 0.99479E+00_JPRB, 0.92360E+00_JPRB, 0.83512E+00_JPRB ,&
&  0.79197E+00_JPRB, 0.74943E+00_JPRB, 0.71759E+00_JPRB, 0.71270E+00_JPRB, 0.69836E+00_JPRB, 0.68496E+00_JPRB, 0.68220E+00_JPRB ,&
&  0.66303E+00_JPRB, 0.65353E+00_JPRB, 0.65027E+00_JPRB, 0.65450E+00_JPRB, 0.66058E+00_JPRB, 0.66736E+00_JPRB, 0.67550E+00_JPRB ,&
&  0.66986E+00_JPRB, 0.66231E+00_JPRB, 0.67261E+00_JPRB, 0.67731E+00_JPRB, 0.69367E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.74243E+00_JPRB, 0.76622E+00_JPRB, 0.78479E+00_JPRB, 0.80215E+00_JPRB, 0.82568E+00_JPRB, 0.84949E+00_JPRB, 0.84644E+00_JPRB ,&
&  0.85587E+00_JPRB, 0.85793E+00_JPRB, 0.85558E+00_JPRB, 0.84024E+00_JPRB, 0.80986E+00_JPRB, 0.72849E+00_JPRB, 0.63004E+00_JPRB ,&
&  0.60128E+00_JPRB, 0.60582E+00_JPRB, 0.61608E+00_JPRB, 0.62069E+00_JPRB, 0.62379E+00_JPRB, 0.62763E+00_JPRB, 0.62887E+00_JPRB ,&
&  0.63394E+00_JPRB, 0.64176E+00_JPRB, 0.65477E+00_JPRB, 0.67059E+00_JPRB, 0.68491E+00_JPRB, 0.69235E+00_JPRB, 0.68970E+00_JPRB ,&
&  0.67819E+00_JPRB, 0.67393E+00_JPRB, 0.66344E+00_JPRB, 0.65188E+00_JPRB, 0.65474E+00_JPRB, 0.66216E+00_JPRB, 0.67970E+00_JPRB ,&
&  0.69532E+00_JPRB, 0.70559E+00_JPRB, 0.70960E+00_JPRB, 0.71410E+00_JPRB, 0.71608E+00_JPRB, 0.71226E+00_JPRB, 0.70511E+00_JPRB ,&
&  0.69885E+00_JPRB, 0.69441E+00_JPRB, 0.69185E+00_JPRB, 0.69390E+00_JPRB, 0.69266E+00_JPRB, 0.70308E+00_JPRB, 0.72043E+00_JPRB ,&
&  0.74176E+00_JPRB, 0.83196E+00_JPRB, 0.97445E+00_JPRB, 0.10262E+01_JPRB, 0.12893E+01_JPRB, 0.14453E+01_JPRB, 0.15800E+01_JPRB ,&
&  0.16826E+01_JPRB, 0.16385E+01_JPRB, 0.18289E+01_JPRB, 0.18864E+01_JPRB, 0.19083E+01_JPRB, 0.18591E+01_JPRB, 0.17399E+01_JPRB ,&
&  0.16801E+01_JPRB, 0.17624E+01_JPRB, 0.18064E+01_JPRB, 0.17843E+01_JPRB, 0.17072E+01_JPRB, 0.16818E+01_JPRB, 0.16637E+01_JPRB ,&
&  0.16865E+01_JPRB, 0.17029E+01_JPRB, 0.16235E+01_JPRB, 0.14597E+01_JPRB, 0.12646E+01_JPRB, 0.10941E+01_JPRB, 0.95870E+00_JPRB ,&
&  0.85212E+00_JPRB, 0.75151E+00_JPRB, 0.71447E+00_JPRB, 0.70509E+00_JPRB, 0.70556E+00_JPRB, 0.68802E+00_JPRB, 0.68420E+00_JPRB ,&
&  0.68624E+00_JPRB, 0.68609E+00_JPRB, 0.69181E+00_JPRB, 0.69524E+00_JPRB, 0.71301E+00_JPRB, 0.72243E+00_JPRB, 0.69693E+00_JPRB ,&
&  0.67943E+00_JPRB, 0.68235E+00_JPRB, 0.68776E+00_JPRB, 0.69647E+00_JPRB, 0.71738E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.77665E+00_JPRB, 0.80397E+00_JPRB, 0.82380E+00_JPRB, 0.84024E+00_JPRB, 0.86663E+00_JPRB, 0.90585E+00_JPRB, 0.91909E+00_JPRB ,&
&  0.91581E+00_JPRB, 0.91064E+00_JPRB, 0.90598E+00_JPRB, 0.89502E+00_JPRB, 0.87113E+00_JPRB, 0.83077E+00_JPRB, 0.73821E+00_JPRB ,&
&  0.64065E+00_JPRB, 0.62935E+00_JPRB, 0.61575E+00_JPRB, 0.62003E+00_JPRB, 0.62601E+00_JPRB, 0.62588E+00_JPRB, 0.62383E+00_JPRB ,&
&  0.62976E+00_JPRB, 0.63758E+00_JPRB, 0.65117E+00_JPRB, 0.66790E+00_JPRB, 0.67533E+00_JPRB, 0.68346E+00_JPRB, 0.69191E+00_JPRB ,&
&  0.68117E+00_JPRB, 0.66743E+00_JPRB, 0.65734E+00_JPRB, 0.65193E+00_JPRB, 0.66682E+00_JPRB, 0.69045E+00_JPRB, 0.71200E+00_JPRB ,&
&  0.72647E+00_JPRB, 0.73507E+00_JPRB, 0.74185E+00_JPRB, 0.74740E+00_JPRB, 0.75028E+00_JPRB, 0.74653E+00_JPRB, 0.73613E+00_JPRB ,&
&  0.72344E+00_JPRB, 0.71581E+00_JPRB, 0.71177E+00_JPRB, 0.71609E+00_JPRB, 0.72231E+00_JPRB, 0.75581E+00_JPRB, 0.79877E+00_JPRB ,&
&  0.86119E+00_JPRB, 0.96828E+00_JPRB, 0.10525E+01_JPRB, 0.12162E+01_JPRB, 0.14522E+01_JPRB, 0.16424E+01_JPRB, 0.17668E+01_JPRB ,&
&  0.18946E+01_JPRB, 0.18837E+01_JPRB, 0.21687E+01_JPRB, 0.21912E+01_JPRB, 0.21038E+01_JPRB, 0.20785E+01_JPRB, 0.19416E+01_JPRB ,&
&  0.18312E+01_JPRB, 0.17439E+01_JPRB, 0.18496E+01_JPRB, 0.19124E+01_JPRB, 0.19531E+01_JPRB, 0.20093E+01_JPRB, 0.20180E+01_JPRB ,&
&  0.20363E+01_JPRB, 0.20207E+01_JPRB, 0.17528E+01_JPRB, 0.14779E+01_JPRB, 0.11962E+01_JPRB, 0.11501E+01_JPRB, 0.10183E+01_JPRB ,&
&  0.87943E+00_JPRB, 0.74939E+00_JPRB, 0.73300E+00_JPRB, 0.72271E+00_JPRB, 0.73619E+00_JPRB, 0.72978E+00_JPRB, 0.72378E+00_JPRB ,&
&  0.73467E+00_JPRB, 0.75166E+00_JPRB, 0.74929E+00_JPRB, 0.75799E+00_JPRB, 0.77941E+00_JPRB, 0.78154E+00_JPRB, 0.74561E+00_JPRB ,&
&  0.72736E+00_JPRB, 0.72318E+00_JPRB, 0.70967E+00_JPRB, 0.71680E+00_JPRB, 0.74176E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.82118E+00_JPRB, 0.84377E+00_JPRB, 0.86086E+00_JPRB, 0.88358E+00_JPRB, 0.91878E+00_JPRB, 0.95777E+00_JPRB, 0.96709E+00_JPRB ,&
&  0.96028E+00_JPRB, 0.95241E+00_JPRB, 0.94311E+00_JPRB, 0.92694E+00_JPRB, 0.90659E+00_JPRB, 0.86633E+00_JPRB, 0.81861E+00_JPRB ,&
&  0.69362E+00_JPRB, 0.65354E+00_JPRB, 0.60911E+00_JPRB, 0.60821E+00_JPRB, 0.64872E+00_JPRB, 0.64175E+00_JPRB, 0.64158E+00_JPRB ,&
&  0.63802E+00_JPRB, 0.64620E+00_JPRB, 0.65295E+00_JPRB, 0.66352E+00_JPRB, 0.66245E+00_JPRB, 0.67307E+00_JPRB, 0.68578E+00_JPRB ,&
&  0.67835E+00_JPRB, 0.66958E+00_JPRB, 0.66919E+00_JPRB, 0.67546E+00_JPRB, 0.69580E+00_JPRB, 0.72422E+00_JPRB, 0.75788E+00_JPRB ,&
&  0.77377E+00_JPRB, 0.78771E+00_JPRB, 0.79464E+00_JPRB, 0.79909E+00_JPRB, 0.79950E+00_JPRB, 0.79357E+00_JPRB, 0.78201E+00_JPRB ,&
&  0.76681E+00_JPRB, 0.75486E+00_JPRB, 0.75684E+00_JPRB, 0.76928E+00_JPRB, 0.77058E+00_JPRB, 0.82207E+00_JPRB, 0.92800E+00_JPRB ,&
&  0.10631E+01_JPRB, 0.11228E+01_JPRB, 0.12199E+01_JPRB, 0.15767E+01_JPRB, 0.18657E+01_JPRB, 0.20391E+01_JPRB, 0.20943E+01_JPRB ,&
&  0.21320E+01_JPRB, 0.21381E+01_JPRB, 0.21527E+01_JPRB, 0.22045E+01_JPRB, 0.21426E+01_JPRB, 0.21738E+01_JPRB, 0.20739E+01_JPRB ,&
&  0.19044E+01_JPRB, 0.18802E+01_JPRB, 0.19865E+01_JPRB, 0.20625E+01_JPRB, 0.20979E+01_JPRB, 0.21748E+01_JPRB, 0.22421E+01_JPRB ,&
&  0.22140E+01_JPRB, 0.19656E+01_JPRB, 0.14883E+01_JPRB, 0.13311E+01_JPRB, 0.10089E+01_JPRB, 0.97966E+00_JPRB, 0.10763E+01_JPRB ,&
&  0.10385E+01_JPRB, 0.91401E+00_JPRB, 0.82470E+00_JPRB, 0.78499E+00_JPRB, 0.78588E+00_JPRB, 0.86225E+00_JPRB, 0.85925E+00_JPRB ,&
&  0.81065E+00_JPRB, 0.82615E+00_JPRB, 0.84686E+00_JPRB, 0.87249E+00_JPRB, 0.89903E+00_JPRB, 0.89975E+00_JPRB, 0.85741E+00_JPRB ,&
&  0.84753E+00_JPRB, 0.81410E+00_JPRB, 0.78521E+00_JPRB, 0.77980E+00_JPRB, 0.79495E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.89817E+00_JPRB, 0.90545E+00_JPRB, 0.91741E+00_JPRB, 0.93807E+00_JPRB, 0.97783E+00_JPRB, 0.10076E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.99444E+00_JPRB, 0.98515E+00_JPRB, 0.97089E+00_JPRB, 0.95232E+00_JPRB, 0.91976E+00_JPRB, 0.86798E+00_JPRB, 0.82066E+00_JPRB ,&
&  0.80291E+00_JPRB, 0.71870E+00_JPRB, 0.65663E+00_JPRB, 0.61612E+00_JPRB, 0.64096E+00_JPRB, 0.64370E+00_JPRB, 0.65708E+00_JPRB ,&
&  0.65631E+00_JPRB, 0.67311E+00_JPRB, 0.67458E+00_JPRB, 0.67200E+00_JPRB, 0.66879E+00_JPRB, 0.68768E+00_JPRB, 0.71051E+00_JPRB ,&
&  0.72435E+00_JPRB, 0.72970E+00_JPRB, 0.73480E+00_JPRB, 0.75647E+00_JPRB, 0.79558E+00_JPRB, 0.77588E+00_JPRB, 0.81191E+00_JPRB ,&
&  0.82959E+00_JPRB, 0.84140E+00_JPRB, 0.84964E+00_JPRB, 0.86160E+00_JPRB, 0.86478E+00_JPRB, 0.86913E+00_JPRB, 0.87443E+00_JPRB ,&
&  0.88183E+00_JPRB, 0.88108E+00_JPRB, 0.85439E+00_JPRB, 0.84821E+00_JPRB, 0.86066E+00_JPRB, 0.96360E+00_JPRB, 0.10236E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.10959E+01_JPRB, 0.11551E+01_JPRB, 0.15674E+01_JPRB, 0.19227E+01_JPRB, 0.20594E+01_JPRB, 0.21800E+01_JPRB ,&
&  0.22467E+01_JPRB, 0.22784E+01_JPRB, 0.23510E+01_JPRB, 0.23779E+01_JPRB, 0.23940E+01_JPRB, 0.23818E+01_JPRB, 0.23340E+01_JPRB ,&
&  0.22001E+01_JPRB, 0.21614E+01_JPRB, 0.22431E+01_JPRB, 0.22081E+01_JPRB, 0.21866E+01_JPRB, 0.21000E+01_JPRB, 0.22615E+01_JPRB ,&
&  0.22697E+01_JPRB, 0.14004E+01_JPRB, 0.11300E+01_JPRB, 0.12764E+01_JPRB, 0.10620E+01_JPRB, 0.10174E+01_JPRB, 0.10700E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.94911E+00_JPRB, 0.92044E+00_JPRB, 0.87581E+00_JPRB, 0.88716E+00_JPRB, 0.10650E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.99880E+00_JPRB, 0.10448E+01_JPRB, 0.10543E+01_JPRB, 0.10513E+01_JPRB, 0.10605E+01_JPRB, 0.10318E+01_JPRB ,&
&  0.97374E+00_JPRB, 0.92776E+00_JPRB, 0.89364E+00_JPRB, 0.88350E+00_JPRB, 0.89011E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.10147E+01_JPRB, 0.10028E+01_JPRB, 0.99708E+00_JPRB, 0.10105E+01_JPRB, 0.10437E+01_JPRB, 0.10623E+01_JPRB, 0.10496E+01_JPRB ,&
&  0.10362E+01_JPRB, 0.10185E+01_JPRB, 0.99955E+00_JPRB, 0.97056E+00_JPRB, 0.92437E+00_JPRB, 0.87125E+00_JPRB, 0.82292E+00_JPRB ,&
&  0.79103E+00_JPRB, 0.75398E+00_JPRB, 0.67395E+00_JPRB, 0.63938E+00_JPRB, 0.61495E+00_JPRB, 0.63114E+00_JPRB, 0.65141E+00_JPRB ,&
&  0.67264E+00_JPRB, 0.68379E+00_JPRB, 0.68832E+00_JPRB, 0.69499E+00_JPRB, 0.70414E+00_JPRB, 0.73652E+00_JPRB, 0.78452E+00_JPRB ,&
&  0.81331E+00_JPRB, 0.82791E+00_JPRB, 0.81612E+00_JPRB, 0.86472E+00_JPRB, 0.87235E+00_JPRB, 0.84647E+00_JPRB, 0.89699E+00_JPRB ,&
&  0.92300E+00_JPRB, 0.94116E+00_JPRB, 0.94362E+00_JPRB, 0.95913E+00_JPRB, 0.96783E+00_JPRB, 0.94828E+00_JPRB, 0.92346E+00_JPRB ,&
&  0.89243E+00_JPRB, 0.87492E+00_JPRB, 0.87223E+00_JPRB, 0.86619E+00_JPRB, 0.87739E+00_JPRB, 0.93885E+00_JPRB, 0.94557E+00_JPRB ,&
&  0.93796E+00_JPRB, 0.82511E+00_JPRB, 0.80186E+00_JPRB, 0.11419E+01_JPRB, 0.16879E+01_JPRB, 0.20102E+01_JPRB, 0.20220E+01_JPRB ,&
&  0.22211E+01_JPRB, 0.23605E+01_JPRB, 0.25358E+01_JPRB, 0.26411E+01_JPRB, 0.26647E+01_JPRB, 0.25932E+01_JPRB, 0.25112E+01_JPRB ,&
&  0.24356E+01_JPRB, 0.24160E+01_JPRB, 0.24157E+01_JPRB, 0.24130E+01_JPRB, 0.23772E+01_JPRB, 0.24432E+01_JPRB, 0.26087E+01_JPRB ,&
&  0.24505E+01_JPRB, 0.21249E+01_JPRB, 0.14443E+01_JPRB, 0.14218E+01_JPRB, 0.11011E+01_JPRB, 0.10442E+01_JPRB, 0.11066E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.10482E+01_JPRB, 0.10217E+01_JPRB, 0.97948E+00_JPRB, 0.10781E+01_JPRB, 0.12812E+01_JPRB, 0.13320E+01_JPRB ,&
&  0.13717E+01_JPRB, 0.12812E+01_JPRB, 0.13717E+01_JPRB, 0.13794E+01_JPRB, 0.13604E+01_JPRB, 0.13338E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.11181E+01_JPRB, 0.10598E+01_JPRB, 0.10325E+01_JPRB, 0.10197E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.10967E+01_JPRB, 0.10633E+01_JPRB, 0.10451E+01_JPRB, 0.10511E+01_JPRB, 0.10748E+01_JPRB, 0.10842E+01_JPRB, 0.10653E+01_JPRB ,&
&  0.10497E+01_JPRB, 0.10294E+01_JPRB, 0.10124E+01_JPRB, 0.96732E+00_JPRB, 0.92376E+00_JPRB, 0.86797E+00_JPRB, 0.81656E+00_JPRB ,&
&  0.78361E+00_JPRB, 0.72842E+00_JPRB, 0.65761E+00_JPRB, 0.62716E+00_JPRB, 0.60370E+00_JPRB, 0.60707E+00_JPRB, 0.63304E+00_JPRB ,&
&  0.66602E+00_JPRB, 0.69215E+00_JPRB, 0.72992E+00_JPRB, 0.74805E+00_JPRB, 0.77964E+00_JPRB, 0.83540E+00_JPRB, 0.89901E+00_JPRB ,&
&  0.98888E+00_JPRB, 0.89945E+00_JPRB, 0.93121E+00_JPRB, 0.93132E+00_JPRB, 0.95619E+00_JPRB, 0.96042E+00_JPRB, 0.95858E+00_JPRB ,&
&  0.96875E+00_JPRB, 0.97824E+00_JPRB, 0.99744E+00_JPRB, 0.99578E+00_JPRB, 0.97931E+00_JPRB, 0.95814E+00_JPRB, 0.94117E+00_JPRB ,&
&  0.93573E+00_JPRB, 0.92322E+00_JPRB, 0.89607E+00_JPRB, 0.86571E+00_JPRB, 0.92963E+00_JPRB, 0.97624E+00_JPRB, 0.92833E+00_JPRB ,&
&  0.79604E+00_JPRB, 0.82396E+00_JPRB, 0.10790E+01_JPRB, 0.14265E+01_JPRB, 0.17989E+01_JPRB, 0.21576E+01_JPRB, 0.23766E+01_JPRB ,&
&  0.25638E+01_JPRB, 0.26701E+01_JPRB, 0.27641E+01_JPRB, 0.24872E+01_JPRB, 0.27108E+01_JPRB, 0.26709E+01_JPRB, 0.25746E+01_JPRB ,&
&  0.24789E+01_JPRB, 0.24990E+01_JPRB, 0.24525E+01_JPRB, 0.25580E+01_JPRB, 0.27544E+01_JPRB, 0.30442E+01_JPRB, 0.27151E+01_JPRB ,&
&  0.23060E+01_JPRB, 0.24499E+01_JPRB, 0.21040E+01_JPRB, 0.16681E+01_JPRB, 0.13127E+01_JPRB, 0.12616E+01_JPRB, 0.12282E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.11453E+01_JPRB, 0.10991E+01_JPRB, 0.11667E+01_JPRB, 0.13822E+01_JPRB, 0.15741E+01_JPRB, 0.15752E+01_JPRB ,&
&  0.15459E+01_JPRB, 0.16755E+01_JPRB, 0.17064E+01_JPRB, 0.16953E+01_JPRB, 0.16928E+01_JPRB, 0.16475E+01_JPRB, 0.15567E+01_JPRB ,&
&  0.14393E+01_JPRB, 0.13364E+01_JPRB, 0.12595E+01_JPRB, 0.11939E+01_JPRB, 0.11392E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.11510E+01_JPRB, 0.11105E+01_JPRB, 0.10932E+01_JPRB, 0.10983E+01_JPRB, 0.11175E+01_JPRB, 0.11119E+01_JPRB, 0.10890E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.10232E+01_JPRB, 0.98188E+00_JPRB, 0.94297E+00_JPRB, 0.91176E+00_JPRB, 0.86330E+00_JPRB, 0.82011E+00_JPRB ,&
&  0.78138E+00_JPRB, 0.71815E+00_JPRB, 0.66537E+00_JPRB, 0.65893E+00_JPRB, 0.64268E+00_JPRB, 0.61500E+00_JPRB, 0.62998E+00_JPRB ,&
&  0.66852E+00_JPRB, 0.72254E+00_JPRB, 0.78070E+00_JPRB, 0.84981E+00_JPRB, 0.91374E+00_JPRB, 0.99368E+00_JPRB, 0.10513E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10136E+01_JPRB, 0.10350E+01_JPRB, 0.10351E+01_JPRB, 0.10014E+01_JPRB, 0.99259E+00_JPRB, 0.10032E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.10369E+01_JPRB, 0.10446E+01_JPRB, 0.10349E+01_JPRB, 0.10309E+01_JPRB, 0.10244E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.97412E+00_JPRB, 0.93276E+00_JPRB, 0.88676E+00_JPRB, 0.90450E+00_JPRB, 0.83715E+00_JPRB, 0.80891E+00_JPRB, 0.80414E+00_JPRB ,&
&  0.89940E+00_JPRB, 0.11004E+01_JPRB, 0.12603E+01_JPRB, 0.18005E+01_JPRB, 0.18278E+01_JPRB, 0.19893E+01_JPRB, 0.24233E+01_JPRB ,&
&  0.26558E+01_JPRB, 0.27317E+01_JPRB, 0.27348E+01_JPRB, 0.22350E+01_JPRB, 0.20094E+01_JPRB, 0.26521E+01_JPRB, 0.27124E+01_JPRB ,&
&  0.26279E+01_JPRB, 0.26215E+01_JPRB, 0.27204E+01_JPRB, 0.31871E+01_JPRB, 0.26484E+01_JPRB, 0.23623E+01_JPRB, 0.19063E+01_JPRB ,&
&  0.19930E+01_JPRB, 0.17432E+01_JPRB, 0.19643E+01_JPRB, 0.17222E+01_JPRB, 0.16336E+01_JPRB, 0.15001E+01_JPRB, 0.13286E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.12097E+01_JPRB, 0.12271E+01_JPRB, 0.14759E+01_JPRB, 0.17920E+01_JPRB, 0.18486E+01_JPRB, 0.19438E+01_JPRB ,&
&  0.20182E+01_JPRB, 0.20592E+01_JPRB, 0.19618E+01_JPRB, 0.19642E+01_JPRB, 0.19029E+01_JPRB, 0.18216E+01_JPRB, 0.17229E+01_JPRB ,&
&  0.16185E+01_JPRB, 0.15166E+01_JPRB, 0.14064E+01_JPRB, 0.13012E+01_JPRB, 0.12136E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.12083E+01_JPRB, 0.11731E+01_JPRB, 0.11628E+01_JPRB, 0.11648E+01_JPRB, 0.11577E+01_JPRB, 0.11311E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.99729E+00_JPRB, 0.94479E+00_JPRB, 0.90554E+00_JPRB, 0.88391E+00_JPRB, 0.86000E+00_JPRB, 0.82818E+00_JPRB ,&
&  0.79653E+00_JPRB, 0.76733E+00_JPRB, 0.71194E+00_JPRB, 0.68565E+00_JPRB, 0.71291E+00_JPRB, 0.67296E+00_JPRB, 0.63983E+00_JPRB ,&
&  0.69625E+00_JPRB, 0.76967E+00_JPRB, 0.85637E+00_JPRB, 0.96466E+00_JPRB, 0.10599E+01_JPRB, 0.11485E+01_JPRB, 0.11964E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.10980E+01_JPRB, 0.10503E+01_JPRB, 0.10034E+01_JPRB, 0.96072E+00_JPRB, 0.95992E+00_JPRB, 0.97517E+00_JPRB ,&
&  0.10115E+01_JPRB, 0.10441E+01_JPRB, 0.10344E+01_JPRB, 0.99097E+00_JPRB, 0.97707E+00_JPRB, 0.92896E+00_JPRB, 0.89577E+00_JPRB ,&
&  0.89916E+00_JPRB, 0.89389E+00_JPRB, 0.88372E+00_JPRB, 0.93912E+00_JPRB, 0.85818E+00_JPRB, 0.80805E+00_JPRB, 0.83428E+00_JPRB ,&
&  0.94816E+00_JPRB, 0.11648E+01_JPRB, 0.13803E+01_JPRB, 0.17966E+01_JPRB, 0.23456E+01_JPRB, 0.24389E+01_JPRB, 0.27001E+01_JPRB ,&
&  0.23253E+01_JPRB, 0.19637E+01_JPRB, 0.17300E+01_JPRB, 0.15468E+01_JPRB, 0.19884E+01_JPRB, 0.29288E+01_JPRB, 0.32748E+01_JPRB ,&
&  0.32386E+01_JPRB, 0.32593E+01_JPRB, 0.35846E+01_JPRB, 0.33699E+01_JPRB, 0.24978E+01_JPRB, 0.18215E+01_JPRB, 0.23594E+01_JPRB ,&
&  0.25303E+01_JPRB, 0.25346E+01_JPRB, 0.22890E+01_JPRB, 0.18628E+01_JPRB, 0.14979E+01_JPRB, 0.16679E+01_JPRB, 0.15378E+01_JPRB ,&
&  0.13777E+01_JPRB, 0.13134E+01_JPRB, 0.15896E+01_JPRB, 0.20328E+01_JPRB, 0.22133E+01_JPRB, 0.24710E+01_JPRB, 0.25518E+01_JPRB ,&
&  0.24326E+01_JPRB, 0.23410E+01_JPRB, 0.22135E+01_JPRB, 0.20630E+01_JPRB, 0.19277E+01_JPRB, 0.17898E+01_JPRB, 0.16836E+01_JPRB ,&
&  0.15913E+01_JPRB, 0.15086E+01_JPRB, 0.14190E+01_JPRB, 0.13332E+01_JPRB, 0.12610E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.12511E+01_JPRB, 0.12318E+01_JPRB, 0.12147E+01_JPRB, 0.11953E+01_JPRB, 0.11671E+01_JPRB, 0.11305E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.96845E+00_JPRB, 0.91593E+00_JPRB, 0.88874E+00_JPRB, 0.86293E+00_JPRB, 0.83356E+00_JPRB, 0.81890E+00_JPRB ,&
&  0.82427E+00_JPRB, 0.84386E+00_JPRB, 0.77628E+00_JPRB, 0.75311E+00_JPRB, 0.77077E+00_JPRB, 0.76860E+00_JPRB, 0.67904E+00_JPRB ,&
&  0.75798E+00_JPRB, 0.83953E+00_JPRB, 0.91408E+00_JPRB, 0.10368E+01_JPRB, 0.12315E+01_JPRB, 0.12015E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.11209E+01_JPRB, 0.10776E+01_JPRB, 0.10156E+01_JPRB, 0.97303E+00_JPRB, 0.98473E+00_JPRB, 0.10053E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10619E+01_JPRB, 0.10311E+01_JPRB, 0.97661E+00_JPRB, 0.93103E+00_JPRB, 0.90090E+00_JPRB, 0.87937E+00_JPRB ,&
&  0.87833E+00_JPRB, 0.87444E+00_JPRB, 0.87597E+00_JPRB, 0.95155E+00_JPRB, 0.89597E+00_JPRB, 0.87966E+00_JPRB, 0.10379E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.15739E+01_JPRB, 0.17334E+01_JPRB, 0.20262E+01_JPRB, 0.23508E+01_JPRB, 0.24846E+01_JPRB, 0.26183E+01_JPRB ,&
&  0.20871E+01_JPRB, 0.18617E+01_JPRB, 0.21384E+01_JPRB, 0.25109E+01_JPRB, 0.25395E+01_JPRB, 0.28234E+01_JPRB, 0.46947E+01_JPRB ,&
&  0.33951E+01_JPRB, 0.36328E+01_JPRB, 0.40987E+01_JPRB, 0.40996E+01_JPRB, 0.19206E+01_JPRB, 0.13469E+01_JPRB, 0.21111E+01_JPRB ,&
&  0.21623E+01_JPRB, 0.13213E+01_JPRB, 0.10155E+01_JPRB, 0.12698E+01_JPRB, 0.10108E+01_JPRB, 0.11588E+01_JPRB, 0.14437E+01_JPRB ,&
&  0.15276E+01_JPRB, 0.15651E+01_JPRB, 0.19800E+01_JPRB, 0.21906E+01_JPRB, 0.23770E+01_JPRB, 0.34253E+01_JPRB, 0.31042E+01_JPRB ,&
&  0.28739E+01_JPRB, 0.24268E+01_JPRB, 0.22891E+01_JPRB, 0.20325E+01_JPRB, 0.18475E+01_JPRB, 0.16844E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.14609E+01_JPRB, 0.13881E+01_JPRB, 0.13370E+01_JPRB, 0.12955E+01_JPRB, 0.12744E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.12842E+01_JPRB, 0.12616E+01_JPRB, 0.12376E+01_JPRB, 0.12072E+01_JPRB, 0.11695E+01_JPRB, 0.11194E+01_JPRB, 0.10560E+01_JPRB ,&
&  0.98954E+00_JPRB, 0.93423E+00_JPRB, 0.89205E+00_JPRB, 0.86634E+00_JPRB, 0.83737E+00_JPRB, 0.81301E+00_JPRB, 0.81619E+00_JPRB ,&
&  0.83446E+00_JPRB, 0.86327E+00_JPRB, 0.86061E+00_JPRB, 0.85177E+00_JPRB, 0.87023E+00_JPRB, 0.87538E+00_JPRB, 0.82750E+00_JPRB ,&
&  0.91355E+00_JPRB, 0.95994E+00_JPRB, 0.10015E+01_JPRB, 0.10844E+01_JPRB, 0.11610E+01_JPRB, 0.11342E+01_JPRB, 0.11633E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.11383E+01_JPRB, 0.10905E+01_JPRB, 0.10417E+01_JPRB, 0.10263E+01_JPRB, 0.10315E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10756E+01_JPRB, 0.10836E+01_JPRB, 0.10393E+01_JPRB, 0.96763E+00_JPRB, 0.92988E+00_JPRB, 0.89719E+00_JPRB, 0.87784E+00_JPRB ,&
&  0.87228E+00_JPRB, 0.89228E+00_JPRB, 0.92979E+00_JPRB, 0.10588E+01_JPRB, 0.13075E+01_JPRB, 0.14564E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.14967E+01_JPRB, 0.19440E+01_JPRB, 0.20608E+01_JPRB, 0.22486E+01_JPRB, 0.24252E+01_JPRB, 0.24156E+01_JPRB ,&
&  0.23980E+01_JPRB, 0.27358E+01_JPRB, 0.28407E+01_JPRB, 0.33020E+01_JPRB, 0.36586E+01_JPRB, 0.29936E+01_JPRB, 0.38341E+01_JPRB ,&
&  0.39172E+01_JPRB, 0.37929E+01_JPRB, 0.30539E+01_JPRB, 0.22284E+01_JPRB, 0.24422E+01_JPRB, 0.17895E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.95035E+00_JPRB, 0.87821E+00_JPRB, 0.82878E+00_JPRB, 0.85960E+00_JPRB, 0.86453E+00_JPRB, 0.10802E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.18254E+01_JPRB, 0.20370E+01_JPRB, 0.21354E+01_JPRB, 0.22096E+01_JPRB, 0.22652E+01_JPRB, 0.31844E+01_JPRB, 0.30012E+01_JPRB ,&
&  0.25145E+01_JPRB, 0.22936E+01_JPRB, 0.20971E+01_JPRB, 0.18970E+01_JPRB, 0.17524E+01_JPRB, 0.16335E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.14692E+01_JPRB, 0.14162E+01_JPRB, 0.13774E+01_JPRB, 0.13443E+01_JPRB, 0.13084E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.12717E+01_JPRB, 0.12446E+01_JPRB, 0.12176E+01_JPRB, 0.11773E+01_JPRB, 0.11309E+01_JPRB, 0.10728E+01_JPRB, 0.10001E+01_JPRB ,&
&  0.92903E+00_JPRB, 0.87874E+00_JPRB, 0.85144E+00_JPRB, 0.82502E+00_JPRB, 0.80915E+00_JPRB, 0.80238E+00_JPRB, 0.81916E+00_JPRB ,&
&  0.84629E+00_JPRB, 0.88236E+00_JPRB, 0.93498E+00_JPRB, 0.97667E+00_JPRB, 0.11058E+01_JPRB, 0.11236E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.11838E+01_JPRB, 0.11695E+01_JPRB, 0.11736E+01_JPRB, 0.11854E+01_JPRB, 0.12137E+01_JPRB, 0.11875E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.11739E+01_JPRB, 0.11350E+01_JPRB, 0.10978E+01_JPRB, 0.10739E+01_JPRB, 0.10871E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.10974E+01_JPRB, 0.10594E+01_JPRB, 0.99059E+00_JPRB, 0.95206E+00_JPRB, 0.93120E+00_JPRB, 0.93583E+00_JPRB ,&
&  0.94754E+00_JPRB, 0.10119E+01_JPRB, 0.10977E+01_JPRB, 0.12881E+01_JPRB, 0.12792E+01_JPRB, 0.10879E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.13500E+01_JPRB, 0.17067E+01_JPRB, 0.18583E+01_JPRB, 0.21307E+01_JPRB, 0.22123E+01_JPRB, 0.21662E+01_JPRB, 0.23782E+01_JPRB ,&
&  0.27311E+01_JPRB, 0.39443E+01_JPRB, 0.31098E+01_JPRB, 0.32153E+01_JPRB, 0.51505E+01_JPRB, 0.41918E+01_JPRB, 0.39059E+01_JPRB ,&
&  0.44583E+01_JPRB, 0.38690E+01_JPRB, 0.36974E+01_JPRB, 0.31433E+01_JPRB, 0.50074E+01_JPRB, 0.53113E+01_JPRB, 0.16063E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.95500E+00_JPRB, 0.86884E+00_JPRB, 0.80032E+00_JPRB, 0.85356E+00_JPRB, 0.11330E+01_JPRB, 0.16734E+01_JPRB ,&
&  0.19479E+01_JPRB, 0.22771E+01_JPRB, 0.22840E+01_JPRB, 0.23779E+01_JPRB, 0.22692E+01_JPRB, 0.26109E+01_JPRB, 0.22974E+01_JPRB ,&
&  0.21754E+01_JPRB, 0.20767E+01_JPRB, 0.18717E+01_JPRB, 0.17124E+01_JPRB, 0.16189E+01_JPRB, 0.15489E+01_JPRB, 0.14811E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.13913E+01_JPRB, 0.13611E+01_JPRB, 0.13298E+01_JPRB, 0.13001E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.12172E+01_JPRB, 0.11932E+01_JPRB, 0.11604E+01_JPRB, 0.11099E+01_JPRB, 0.10710E+01_JPRB, 0.10204E+01_JPRB, 0.94931E+00_JPRB ,&
&  0.89651E+00_JPRB, 0.86685E+00_JPRB, 0.83520E+00_JPRB, 0.80607E+00_JPRB, 0.80150E+00_JPRB, 0.80863E+00_JPRB, 0.83743E+00_JPRB ,&
&  0.87766E+00_JPRB, 0.93093E+00_JPRB, 0.99957E+00_JPRB, 0.11842E+01_JPRB, 0.13916E+01_JPRB, 0.14401E+01_JPRB, 0.16892E+01_JPRB ,&
&  0.16146E+01_JPRB, 0.14943E+01_JPRB, 0.14265E+01_JPRB, 0.13634E+01_JPRB, 0.13205E+01_JPRB, 0.12506E+01_JPRB, 0.12207E+01_JPRB ,&
&  0.11876E+01_JPRB, 0.12263E+01_JPRB, 0.11860E+01_JPRB, 0.11593E+01_JPRB, 0.11541E+01_JPRB, 0.11440E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.11273E+01_JPRB, 0.10994E+01_JPRB, 0.10475E+01_JPRB, 0.10101E+01_JPRB, 0.99697E+00_JPRB, 0.10167E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.11853E+01_JPRB, 0.12964E+01_JPRB, 0.14235E+01_JPRB, 0.11729E+01_JPRB, 0.13418E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.16111E+01_JPRB, 0.17824E+01_JPRB, 0.17944E+01_JPRB, 0.19310E+01_JPRB, 0.21610E+01_JPRB, 0.22043E+01_JPRB, 0.24932E+01_JPRB ,&
&  0.33385E+01_JPRB, 0.33669E+01_JPRB, 0.30246E+01_JPRB, 0.33961E+01_JPRB, 0.46412E+01_JPRB, 0.68836E+01_JPRB, 0.35909E+01_JPRB ,&
&  0.28969E+01_JPRB, 0.39210E+01_JPRB, 0.40379E+01_JPRB, 0.39031E+01_JPRB, 0.56496E+01_JPRB, 0.74046E+01_JPRB, 0.46913E+01_JPRB ,&
&  0.13526E+01_JPRB, 0.96421E+00_JPRB, 0.10720E+01_JPRB, 0.96689E+00_JPRB, 0.91570E+00_JPRB, 0.12032E+01_JPRB, 0.26839E+01_JPRB ,&
&  0.22305E+01_JPRB, 0.24659E+01_JPRB, 0.24684E+01_JPRB, 0.24187E+01_JPRB, 0.22903E+01_JPRB, 0.23096E+01_JPRB, 0.21315E+01_JPRB ,&
&  0.18004E+01_JPRB, 0.16807E+01_JPRB, 0.15264E+01_JPRB, 0.14472E+01_JPRB, 0.14165E+01_JPRB, 0.13942E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.12933E+01_JPRB, 0.12744E+01_JPRB, 0.12592E+01_JPRB, 0.12423E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.11474E+01_JPRB, 0.11231E+01_JPRB, 0.10798E+01_JPRB, 0.10483E+01_JPRB, 0.10425E+01_JPRB, 0.99793E+00_JPRB, 0.95468E+00_JPRB ,&
&  0.90822E+00_JPRB, 0.88424E+00_JPRB, 0.85571E+00_JPRB, 0.84823E+00_JPRB, 0.83609E+00_JPRB, 0.85230E+00_JPRB, 0.88830E+00_JPRB ,&
&  0.94069E+00_JPRB, 0.10019E+01_JPRB, 0.10848E+01_JPRB, 0.13169E+01_JPRB, 0.16379E+01_JPRB, 0.16832E+01_JPRB, 0.20015E+01_JPRB ,&
&  0.22625E+01_JPRB, 0.20143E+01_JPRB, 0.17284E+01_JPRB, 0.15715E+01_JPRB, 0.14657E+01_JPRB, 0.13879E+01_JPRB, 0.13203E+01_JPRB ,&
&  0.12738E+01_JPRB, 0.13230E+01_JPRB, 0.12694E+01_JPRB, 0.12267E+01_JPRB, 0.11834E+01_JPRB, 0.11775E+01_JPRB, 0.11987E+01_JPRB ,&
&  0.11890E+01_JPRB, 0.12003E+01_JPRB, 0.11796E+01_JPRB, 0.11529E+01_JPRB, 0.11258E+01_JPRB, 0.11249E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.13225E+01_JPRB, 0.14544E+01_JPRB, 0.15338E+01_JPRB, 0.15126E+01_JPRB, 0.15256E+01_JPRB, 0.15501E+01_JPRB, 0.17250E+01_JPRB ,&
&  0.18793E+01_JPRB, 0.19696E+01_JPRB, 0.20113E+01_JPRB, 0.20328E+01_JPRB, 0.22347E+01_JPRB, 0.25060E+01_JPRB, 0.29019E+01_JPRB ,&
&  0.35420E+01_JPRB, 0.37498E+01_JPRB, 0.33592E+01_JPRB, 0.35966E+01_JPRB, 0.43774E+01_JPRB, 0.53893E+01_JPRB, 0.47591E+01_JPRB ,&
&  0.37987E+01_JPRB, 0.32498E+01_JPRB, 0.36746E+01_JPRB, 0.46797E+01_JPRB, 0.52911E+01_JPRB, 0.52606E+01_JPRB, 0.62984E+01_JPRB ,&
&  0.52764E+01_JPRB, 0.24762E+01_JPRB, 0.19245E+01_JPRB, 0.22370E+01_JPRB, 0.15988E+01_JPRB, 0.16120E+01_JPRB, 0.21476E+01_JPRB ,&
&  0.22882E+01_JPRB, 0.25345E+01_JPRB, 0.25735E+01_JPRB, 0.22222E+01_JPRB, 0.21745E+01_JPRB, 0.21020E+01_JPRB, 0.18245E+01_JPRB ,&
&  0.15635E+01_JPRB, 0.14207E+01_JPRB, 0.13266E+01_JPRB, 0.12682E+01_JPRB, 0.12349E+01_JPRB, 0.12008E+01_JPRB, 0.11803E+01_JPRB ,&
&  0.11575E+01_JPRB, 0.11473E+01_JPRB, 0.11468E+01_JPRB, 0.11486E+01_JPRB, 0.11548E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.10898E+01_JPRB, 0.10586E+01_JPRB, 0.10573E+01_JPRB, 0.10740E+01_JPRB, 0.10668E+01_JPRB, 0.10495E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.95801E+00_JPRB, 0.95201E+00_JPRB, 0.96925E+00_JPRB, 0.96198E+00_JPRB, 0.93581E+00_JPRB, 0.94508E+00_JPRB, 0.97975E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.11425E+01_JPRB, 0.13181E+01_JPRB, 0.16922E+01_JPRB, 0.20858E+01_JPRB, 0.26904E+01_JPRB, 0.20572E+01_JPRB ,&
&  0.22932E+01_JPRB, 0.22902E+01_JPRB, 0.18855E+01_JPRB, 0.17329E+01_JPRB, 0.16433E+01_JPRB, 0.16001E+01_JPRB, 0.14701E+01_JPRB ,&
&  0.14461E+01_JPRB, 0.14391E+01_JPRB, 0.13659E+01_JPRB, 0.12672E+01_JPRB, 0.12170E+01_JPRB, 0.12854E+01_JPRB, 0.12931E+01_JPRB ,&
&  0.13013E+01_JPRB, 0.13365E+01_JPRB, 0.13113E+01_JPRB, 0.13169E+01_JPRB, 0.13069E+01_JPRB, 0.13215E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.15671E+01_JPRB, 0.16852E+01_JPRB, 0.17598E+01_JPRB, 0.16896E+01_JPRB, 0.17718E+01_JPRB, 0.18903E+01_JPRB, 0.21207E+01_JPRB ,&
&  0.21703E+01_JPRB, 0.19786E+01_JPRB, 0.20476E+01_JPRB, 0.22814E+01_JPRB, 0.25727E+01_JPRB, 0.27126E+01_JPRB, 0.30826E+01_JPRB ,&
&  0.39322E+01_JPRB, 0.46022E+01_JPRB, 0.48241E+01_JPRB, 0.40701E+01_JPRB, 0.41158E+01_JPRB, 0.48514E+01_JPRB, 0.52386E+01_JPRB ,&
&  0.49702E+01_JPRB, 0.44629E+01_JPRB, 0.46303E+01_JPRB, 0.45029E+01_JPRB, 0.42960E+01_JPRB, 0.39874E+01_JPRB, 0.41812E+01_JPRB ,&
&  0.46978E+01_JPRB, 0.50529E+01_JPRB, 0.36405E+01_JPRB, 0.28106E+01_JPRB, 0.26238E+01_JPRB, 0.22942E+01_JPRB, 0.21805E+01_JPRB ,&
&  0.24463E+01_JPRB, 0.24719E+01_JPRB, 0.24384E+01_JPRB, 0.26453E+01_JPRB, 0.21890E+01_JPRB, 0.18402E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.11117E+01_JPRB, 0.10719E+01_JPRB, 0.10553E+01_JPRB, 0.10792E+01_JPRB, 0.10817E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.10812E+01_JPRB, 0.11010E+01_JPRB, 0.11173E+01_JPRB, 0.11143E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.10591E+01_JPRB, 0.10629E+01_JPRB, 0.10893E+01_JPRB, 0.11153E+01_JPRB, 0.11236E+01_JPRB, 0.11481E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.10200E+01_JPRB, 0.10443E+01_JPRB, 0.10273E+01_JPRB, 0.10122E+01_JPRB, 0.10282E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.11729E+01_JPRB, 0.13691E+01_JPRB, 0.16869E+01_JPRB, 0.20965E+01_JPRB, 0.23509E+01_JPRB, 0.26221E+01_JPRB, 0.27227E+01_JPRB ,&
&  0.20506E+01_JPRB, 0.24359E+01_JPRB, 0.18788E+01_JPRB, 0.18212E+01_JPRB, 0.18136E+01_JPRB, 0.19869E+01_JPRB, 0.18600E+01_JPRB ,&
&  0.16242E+01_JPRB, 0.14811E+01_JPRB, 0.14757E+01_JPRB, 0.14128E+01_JPRB, 0.13777E+01_JPRB, 0.13935E+01_JPRB, 0.14156E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.14123E+01_JPRB, 0.14585E+01_JPRB, 0.15016E+01_JPRB, 0.15023E+01_JPRB, 0.15269E+01_JPRB, 0.15901E+01_JPRB ,&
&  0.17554E+01_JPRB, 0.18978E+01_JPRB, 0.19444E+01_JPRB, 0.20055E+01_JPRB, 0.20570E+01_JPRB, 0.23238E+01_JPRB, 0.25822E+01_JPRB ,&
&  0.25348E+01_JPRB, 0.22674E+01_JPRB, 0.23075E+01_JPRB, 0.24807E+01_JPRB, 0.26390E+01_JPRB, 0.30696E+01_JPRB, 0.33803E+01_JPRB ,&
&  0.42959E+01_JPRB, 0.43245E+01_JPRB, 0.51723E+01_JPRB, 0.40670E+01_JPRB, 0.39057E+01_JPRB, 0.45265E+01_JPRB, 0.42326E+01_JPRB ,&
&  0.36830E+01_JPRB, 0.41032E+01_JPRB, 0.41506E+01_JPRB, 0.41878E+01_JPRB, 0.40018E+01_JPRB, 0.35665E+01_JPRB, 0.33238E+01_JPRB ,&
&  0.33921E+01_JPRB, 0.35712E+01_JPRB, 0.33412E+01_JPRB, 0.27533E+01_JPRB, 0.25421E+01_JPRB, 0.22945E+01_JPRB, 0.24019E+01_JPRB ,&
&  0.27171E+01_JPRB, 0.28991E+01_JPRB, 0.29206E+01_JPRB, 0.26928E+01_JPRB, 0.16825E+01_JPRB, 0.13035E+01_JPRB, 0.10855E+01_JPRB ,&
&  0.99758E+00_JPRB, 0.96370E+00_JPRB, 0.97748E+00_JPRB, 0.10168E+01_JPRB, 0.10513E+01_JPRB, 0.10671E+01_JPRB, 0.10592E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.10721E+01_JPRB, 0.10790E+01_JPRB, 0.10769E+01_JPRB, 0.10696E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.10881E+01_JPRB, 0.11151E+01_JPRB, 0.11509E+01_JPRB, 0.12062E+01_JPRB, 0.12528E+01_JPRB, 0.13433E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.97690E+00_JPRB, 0.10049E+01_JPRB, 0.10349E+01_JPRB, 0.10731E+01_JPRB, 0.11298E+01_JPRB, 0.12220E+01_JPRB ,&
&  0.13945E+01_JPRB, 0.16215E+01_JPRB, 0.18827E+01_JPRB, 0.21352E+01_JPRB, 0.24091E+01_JPRB, 0.26685E+01_JPRB, 0.26800E+01_JPRB ,&
&  0.21826E+01_JPRB, 0.19116E+01_JPRB, 0.20391E+01_JPRB, 0.20094E+01_JPRB, 0.20273E+01_JPRB, 0.20843E+01_JPRB, 0.21194E+01_JPRB ,&
&  0.18183E+01_JPRB, 0.16611E+01_JPRB, 0.16749E+01_JPRB, 0.17056E+01_JPRB, 0.15377E+01_JPRB, 0.15946E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.16469E+01_JPRB, 0.16560E+01_JPRB, 0.17270E+01_JPRB, 0.17967E+01_JPRB, 0.17833E+01_JPRB, 0.18038E+01_JPRB, 0.18964E+01_JPRB ,&
&  0.21120E+01_JPRB, 0.22746E+01_JPRB, 0.23694E+01_JPRB, 0.23844E+01_JPRB, 0.25456E+01_JPRB, 0.28306E+01_JPRB, 0.29302E+01_JPRB ,&
&  0.27873E+01_JPRB, 0.27389E+01_JPRB, 0.27916E+01_JPRB, 0.29240E+01_JPRB, 0.28780E+01_JPRB, 0.34700E+01_JPRB, 0.37234E+01_JPRB ,&
&  0.40881E+01_JPRB, 0.41806E+01_JPRB, 0.49239E+01_JPRB, 0.36336E+01_JPRB, 0.38946E+01_JPRB, 0.40823E+01_JPRB, 0.36868E+01_JPRB ,&
&  0.32980E+01_JPRB, 0.34767E+01_JPRB, 0.34219E+01_JPRB, 0.34687E+01_JPRB, 0.34332E+01_JPRB, 0.29775E+01_JPRB, 0.30887E+01_JPRB ,&
&  0.29260E+01_JPRB, 0.32136E+01_JPRB, 0.30122E+01_JPRB, 0.28777E+01_JPRB, 0.24576E+01_JPRB, 0.23354E+01_JPRB, 0.24286E+01_JPRB ,&
&  0.27961E+01_JPRB, 0.29487E+01_JPRB, 0.25681E+01_JPRB, 0.19884E+01_JPRB, 0.11736E+01_JPRB, 0.95985E+00_JPRB, 0.92490E+00_JPRB ,&
&  0.92557E+00_JPRB, 0.96049E+00_JPRB, 0.10004E+01_JPRB, 0.10384E+01_JPRB, 0.10595E+01_JPRB, 0.10625E+01_JPRB, 0.10612E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10754E+01_JPRB, 0.10718E+01_JPRB, 0.10733E+01_JPRB, 0.10781E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.11001E+01_JPRB, 0.11063E+01_JPRB, 0.11161E+01_JPRB, 0.11028E+01_JPRB, 0.10801E+01_JPRB, 0.10442E+01_JPRB, 0.97525E+00_JPRB ,&
&  0.91820E+00_JPRB, 0.90027E+00_JPRB, 0.95266E+00_JPRB, 0.10207E+01_JPRB, 0.10897E+01_JPRB, 0.11742E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.15923E+01_JPRB, 0.17341E+01_JPRB, 0.19664E+01_JPRB, 0.22351E+01_JPRB, 0.23110E+01_JPRB, 0.23613E+01_JPRB ,&
&  0.21055E+01_JPRB, 0.16106E+01_JPRB, 0.19851E+01_JPRB, 0.22380E+01_JPRB, 0.22851E+01_JPRB, 0.23240E+01_JPRB, 0.22331E+01_JPRB ,&
&  0.18898E+01_JPRB, 0.19150E+01_JPRB, 0.20316E+01_JPRB, 0.20813E+01_JPRB, 0.18458E+01_JPRB, 0.19006E+01_JPRB, 0.19625E+01_JPRB ,&
&  0.20251E+01_JPRB, 0.20709E+01_JPRB, 0.21860E+01_JPRB, 0.22579E+01_JPRB, 0.22801E+01_JPRB, 0.23612E+01_JPRB, 0.25028E+01_JPRB ,&
&  0.26526E+01_JPRB, 0.27687E+01_JPRB, 0.29310E+01_JPRB, 0.30375E+01_JPRB, 0.31541E+01_JPRB, 0.33150E+01_JPRB, 0.32780E+01_JPRB ,&
&  0.31392E+01_JPRB, 0.30094E+01_JPRB, 0.31828E+01_JPRB, 0.35952E+01_JPRB, 0.39199E+01_JPRB, 0.35470E+01_JPRB, 0.39165E+01_JPRB ,&
&  0.39252E+01_JPRB, 0.39633E+01_JPRB, 0.38899E+01_JPRB, 0.44548E+01_JPRB, 0.30715E+01_JPRB, 0.33641E+01_JPRB, 0.29083E+01_JPRB ,&
&  0.30217E+01_JPRB, 0.28884E+01_JPRB, 0.27766E+01_JPRB, 0.27760E+01_JPRB, 0.28060E+01_JPRB, 0.25163E+01_JPRB, 0.26284E+01_JPRB ,&
&  0.27625E+01_JPRB, 0.28602E+01_JPRB, 0.28821E+01_JPRB, 0.29326E+01_JPRB, 0.27487E+01_JPRB, 0.27271E+01_JPRB, 0.27203E+01_JPRB ,&
&  0.29413E+01_JPRB, 0.24262E+01_JPRB, 0.18902E+01_JPRB, 0.13898E+01_JPRB, 0.99156E+00_JPRB, 0.85490E+00_JPRB, 0.89845E+00_JPRB ,&
&  0.93723E+00_JPRB, 0.97987E+00_JPRB, 0.10069E+01_JPRB, 0.10303E+01_JPRB, 0.10448E+01_JPRB, 0.10532E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.10821E+01_JPRB, 0.10863E+01_JPRB, 0.10850E+01_JPRB, 0.10862E+01_JPRB, 0.10915E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.10806E+01_JPRB, 0.10630E+01_JPRB, 0.10249E+01_JPRB, 0.96139E+00_JPRB, 0.89230E+00_JPRB, 0.82322E+00_JPRB, 0.78184E+00_JPRB ,&
&  0.74901E+00_JPRB, 0.77492E+00_JPRB, 0.84234E+00_JPRB, 0.93444E+00_JPRB, 0.10422E+01_JPRB, 0.11532E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.13271E+01_JPRB, 0.13770E+01_JPRB, 0.14742E+01_JPRB, 0.17014E+01_JPRB, 0.19556E+01_JPRB, 0.21365E+01_JPRB, 0.22401E+01_JPRB ,&
&  0.23122E+01_JPRB, 0.23223E+01_JPRB, 0.22068E+01_JPRB, 0.18407E+01_JPRB, 0.17242E+01_JPRB, 0.21841E+01_JPRB, 0.20258E+01_JPRB ,&
&  0.20003E+01_JPRB, 0.19925E+01_JPRB, 0.20490E+01_JPRB, 0.21320E+01_JPRB, 0.21503E+01_JPRB, 0.21935E+01_JPRB, 0.23053E+01_JPRB ,&
&  0.24033E+01_JPRB, 0.25197E+01_JPRB, 0.26334E+01_JPRB, 0.27169E+01_JPRB, 0.28194E+01_JPRB, 0.29490E+01_JPRB, 0.30750E+01_JPRB ,&
&  0.31944E+01_JPRB, 0.33716E+01_JPRB, 0.35794E+01_JPRB, 0.37388E+01_JPRB, 0.37023E+01_JPRB, 0.37835E+01_JPRB, 0.37132E+01_JPRB ,&
&  0.35980E+01_JPRB, 0.35102E+01_JPRB, 0.35287E+01_JPRB, 0.37786E+01_JPRB, 0.38739E+01_JPRB, 0.33917E+01_JPRB, 0.34146E+01_JPRB ,&
&  0.36460E+01_JPRB, 0.36881E+01_JPRB, 0.32165E+01_JPRB, 0.41376E+01_JPRB, 0.24150E+01_JPRB, 0.28643E+01_JPRB, 0.29612E+01_JPRB ,&
&  0.27199E+01_JPRB, 0.24649E+01_JPRB, 0.23934E+01_JPRB, 0.24151E+01_JPRB, 0.23688E+01_JPRB, 0.22834E+01_JPRB, 0.24764E+01_JPRB ,&
&  0.25661E+01_JPRB, 0.26215E+01_JPRB, 0.27871E+01_JPRB, 0.28201E+01_JPRB, 0.28100E+01_JPRB, 0.26230E+01_JPRB, 0.23069E+01_JPRB ,&
&  0.19723E+01_JPRB, 0.19972E+01_JPRB, 0.17265E+01_JPRB, 0.15367E+01_JPRB, 0.11077E+01_JPRB, 0.84886E+00_JPRB, 0.90059E+00_JPRB ,&
&  0.93643E+00_JPRB, 0.95883E+00_JPRB, 0.99426E+00_JPRB, 0.10197E+01_JPRB, 0.10242E+01_JPRB, 0.10350E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.10602E+01_JPRB, 0.10648E+01_JPRB, 0.10680E+01_JPRB, 0.10776E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.90630E+00_JPRB, 0.88062E+00_JPRB, 0.83983E+00_JPRB, 0.80002E+00_JPRB, 0.76086E+00_JPRB, 0.71461E+00_JPRB, 0.70775E+00_JPRB ,&
&  0.68800E+00_JPRB, 0.69586E+00_JPRB, 0.72681E+00_JPRB, 0.76374E+00_JPRB, 0.82429E+00_JPRB, 0.91385E+00_JPRB, 0.99934E+00_JPRB ,&
&  0.10476E+01_JPRB, 0.10639E+01_JPRB, 0.11312E+01_JPRB, 0.12353E+01_JPRB, 0.14245E+01_JPRB, 0.17518E+01_JPRB, 0.22932E+01_JPRB ,&
&  0.25813E+01_JPRB, 0.27495E+01_JPRB, 0.27109E+01_JPRB, 0.25473E+01_JPRB, 0.22373E+01_JPRB, 0.23405E+01_JPRB, 0.25820E+01_JPRB ,&
&  0.27439E+01_JPRB, 0.25505E+01_JPRB, 0.22547E+01_JPRB, 0.22808E+01_JPRB, 0.23528E+01_JPRB, 0.24450E+01_JPRB, 0.25687E+01_JPRB ,&
&  0.26990E+01_JPRB, 0.28003E+01_JPRB, 0.29041E+01_JPRB, 0.30308E+01_JPRB, 0.31426E+01_JPRB, 0.32529E+01_JPRB, 0.33919E+01_JPRB ,&
&  0.35478E+01_JPRB, 0.36944E+01_JPRB, 0.38114E+01_JPRB, 0.39506E+01_JPRB, 0.41607E+01_JPRB, 0.43577E+01_JPRB, 0.43775E+01_JPRB ,&
&  0.42979E+01_JPRB, 0.40172E+01_JPRB, 0.40840E+01_JPRB, 0.42755E+01_JPRB, 0.40937E+01_JPRB, 0.36896E+01_JPRB, 0.35586E+01_JPRB ,&
&  0.36918E+01_JPRB, 0.37717E+01_JPRB, 0.25664E+01_JPRB, 0.32455E+01_JPRB, 0.35880E+01_JPRB, 0.30708E+01_JPRB, 0.26105E+01_JPRB ,&
&  0.23376E+01_JPRB, 0.21479E+01_JPRB, 0.21847E+01_JPRB, 0.22040E+01_JPRB, 0.23769E+01_JPRB, 0.23454E+01_JPRB, 0.20250E+01_JPRB ,&
&  0.20210E+01_JPRB, 0.20911E+01_JPRB, 0.23391E+01_JPRB, 0.24638E+01_JPRB, 0.23667E+01_JPRB, 0.21112E+01_JPRB, 0.17117E+01_JPRB ,&
&  0.14530E+01_JPRB, 0.17041E+01_JPRB, 0.15459E+01_JPRB, 0.15858E+01_JPRB, 0.13689E+01_JPRB, 0.83202E+00_JPRB, 0.88013E+00_JPRB ,&
&  0.90003E+00_JPRB, 0.92140E+00_JPRB, 0.94713E+00_JPRB, 0.95971E+00_JPRB, 0.97265E+00_JPRB, 0.98241E+00_JPRB, 0.99737E+00_JPRB ,&
&  0.99263E+00_JPRB, 0.97712E+00_JPRB, 0.96404E+00_JPRB, 0.95511E+00_JPRB, 0.93657E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.81015E+00_JPRB, 0.80097E+00_JPRB, 0.79093E+00_JPRB, 0.78858E+00_JPRB, 0.79478E+00_JPRB, 0.78961E+00_JPRB, 0.77283E+00_JPRB ,&
&  0.74639E+00_JPRB, 0.73412E+00_JPRB, 0.70849E+00_JPRB, 0.68294E+00_JPRB, 0.68859E+00_JPRB, 0.74707E+00_JPRB, 0.80867E+00_JPRB ,&
&  0.84599E+00_JPRB, 0.86174E+00_JPRB, 0.85893E+00_JPRB, 0.85702E+00_JPRB, 0.89447E+00_JPRB, 0.93295E+00_JPRB, 0.11861E+01_JPRB ,&
&  0.16485E+01_JPRB, 0.22965E+01_JPRB, 0.27792E+01_JPRB, 0.30353E+01_JPRB, 0.29425E+01_JPRB, 0.25990E+01_JPRB, 0.31041E+01_JPRB ,&
&  0.28636E+01_JPRB, 0.22767E+01_JPRB, 0.21250E+01_JPRB, 0.21179E+01_JPRB, 0.21844E+01_JPRB, 0.23018E+01_JPRB, 0.24489E+01_JPRB ,&
&  0.25868E+01_JPRB, 0.27259E+01_JPRB, 0.28710E+01_JPRB, 0.30379E+01_JPRB, 0.32153E+01_JPRB, 0.34056E+01_JPRB, 0.35987E+01_JPRB ,&
&  0.38322E+01_JPRB, 0.39851E+01_JPRB, 0.41668E+01_JPRB, 0.40481E+01_JPRB, 0.44628E+01_JPRB, 0.48068E+01_JPRB, 0.49781E+01_JPRB ,&
&  0.49220E+01_JPRB, 0.47489E+01_JPRB, 0.48662E+01_JPRB, 0.48532E+01_JPRB, 0.48045E+01_JPRB, 0.45507E+01_JPRB, 0.41978E+01_JPRB ,&
&  0.42820E+01_JPRB, 0.39714E+01_JPRB, 0.26385E+01_JPRB, 0.26445E+01_JPRB, 0.28573E+01_JPRB, 0.22576E+01_JPRB, 0.23902E+01_JPRB ,&
&  0.20777E+01_JPRB, 0.18691E+01_JPRB, 0.17027E+01_JPRB, 0.16857E+01_JPRB, 0.19061E+01_JPRB, 0.21304E+01_JPRB, 0.17936E+01_JPRB ,&
&  0.15045E+01_JPRB, 0.15465E+01_JPRB, 0.17061E+01_JPRB, 0.18161E+01_JPRB, 0.17626E+01_JPRB, 0.16095E+01_JPRB, 0.14401E+01_JPRB ,&
&  0.14262E+01_JPRB, 0.13798E+01_JPRB, 0.12701E+01_JPRB, 0.12021E+01_JPRB, 0.90169E+00_JPRB, 0.77778E+00_JPRB, 0.80464E+00_JPRB ,&
&  0.81773E+00_JPRB, 0.81194E+00_JPRB, 0.82108E+00_JPRB, 0.82885E+00_JPRB, 0.84550E+00_JPRB, 0.84337E+00_JPRB, 0.85829E+00_JPRB ,&
&  0.86496E+00_JPRB, 0.86052E+00_JPRB, 0.85463E+00_JPRB, 0.85540E+00_JPRB, 0.83376E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.89889E+00_JPRB, 0.90725E+00_JPRB, 0.92361E+00_JPRB, 0.93917E+00_JPRB, 0.96148E+00_JPRB, 0.94766E+00_JPRB, 0.92873E+00_JPRB ,&
&  0.89098E+00_JPRB, 0.88272E+00_JPRB, 0.87374E+00_JPRB, 0.86897E+00_JPRB, 0.87915E+00_JPRB, 0.90947E+00_JPRB, 0.96570E+00_JPRB ,&
&  0.98298E+00_JPRB, 0.98758E+00_JPRB, 0.97513E+00_JPRB, 0.95822E+00_JPRB, 0.91313E+00_JPRB, 0.87157E+00_JPRB, 0.84259E+00_JPRB ,&
&  0.87585E+00_JPRB, 0.11182E+01_JPRB, 0.14369E+01_JPRB, 0.17492E+01_JPRB, 0.21377E+01_JPRB, 0.20783E+01_JPRB, 0.24870E+01_JPRB ,&
&  0.28689E+01_JPRB, 0.21046E+01_JPRB, 0.21307E+01_JPRB, 0.18943E+01_JPRB, 0.19211E+01_JPRB, 0.19523E+01_JPRB, 0.20898E+01_JPRB ,&
&  0.22556E+01_JPRB, 0.23835E+01_JPRB, 0.24955E+01_JPRB, 0.26161E+01_JPRB, 0.27588E+01_JPRB, 0.28871E+01_JPRB, 0.29686E+01_JPRB ,&
&  0.30386E+01_JPRB, 0.29610E+01_JPRB, 0.30571E+01_JPRB, 0.36854E+01_JPRB, 0.42641E+01_JPRB, 0.45751E+01_JPRB, 0.48077E+01_JPRB ,&
&  0.51171E+01_JPRB, 0.53234E+01_JPRB, 0.48876E+01_JPRB, 0.48289E+01_JPRB, 0.50872E+01_JPRB, 0.45744E+01_JPRB, 0.45685E+01_JPRB ,&
&  0.47277E+01_JPRB, 0.40801E+01_JPRB, 0.24334E+01_JPRB, 0.25962E+01_JPRB, 0.26520E+01_JPRB, 0.23954E+01_JPRB, 0.23371E+01_JPRB ,&
&  0.19781E+01_JPRB, 0.17430E+01_JPRB, 0.14723E+01_JPRB, 0.13566E+01_JPRB, 0.13773E+01_JPRB, 0.15341E+01_JPRB, 0.15229E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.12418E+01_JPRB, 0.14132E+01_JPRB, 0.16224E+01_JPRB, 0.16864E+01_JPRB, 0.15510E+01_JPRB, 0.13280E+01_JPRB ,&
&  0.14061E+01_JPRB, 0.14277E+01_JPRB, 0.17313E+01_JPRB, 0.16611E+01_JPRB, 0.94124E+00_JPRB, 0.80298E+00_JPRB, 0.77976E+00_JPRB ,&
&  0.79436E+00_JPRB, 0.79136E+00_JPRB, 0.78324E+00_JPRB, 0.79322E+00_JPRB, 0.79625E+00_JPRB, 0.79517E+00_JPRB, 0.82467E+00_JPRB ,&
&  0.86161E+00_JPRB, 0.86480E+00_JPRB, 0.87061E+00_JPRB, 0.87524E+00_JPRB, 0.88553E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.10141E+01_JPRB, 0.10355E+01_JPRB, 0.10629E+01_JPRB, 0.10699E+01_JPRB, 0.10777E+01_JPRB, 0.10703E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10468E+01_JPRB, 0.10425E+01_JPRB, 0.10463E+01_JPRB, 0.10521E+01_JPRB, 0.10670E+01_JPRB, 0.10846E+01_JPRB, 0.11215E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.11314E+01_JPRB, 0.11410E+01_JPRB, 0.11606E+01_JPRB, 0.11769E+01_JPRB, 0.11689E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.11055E+01_JPRB, 0.11205E+01_JPRB, 0.11560E+01_JPRB, 0.13511E+01_JPRB, 0.17025E+01_JPRB, 0.22869E+01_JPRB ,&
&  0.24165E+01_JPRB, 0.21348E+01_JPRB, 0.18880E+01_JPRB, 0.16392E+01_JPRB, 0.16152E+01_JPRB, 0.16458E+01_JPRB, 0.15737E+01_JPRB ,&
&  0.15666E+01_JPRB, 0.15488E+01_JPRB, 0.15875E+01_JPRB, 0.16673E+01_JPRB, 0.17329E+01_JPRB, 0.17160E+01_JPRB, 0.16840E+01_JPRB ,&
&  0.15448E+01_JPRB, 0.15022E+01_JPRB, 0.16391E+01_JPRB, 0.20019E+01_JPRB, 0.27701E+01_JPRB, 0.32971E+01_JPRB, 0.31453E+01_JPRB ,&
&  0.35118E+01_JPRB, 0.41849E+01_JPRB, 0.44050E+01_JPRB, 0.44292E+01_JPRB, 0.42452E+01_JPRB, 0.39769E+01_JPRB, 0.38085E+01_JPRB ,&
&  0.37336E+01_JPRB, 0.33230E+01_JPRB, 0.24877E+01_JPRB, 0.25969E+01_JPRB, 0.24636E+01_JPRB, 0.24485E+01_JPRB, 0.21651E+01_JPRB ,&
&  0.18552E+01_JPRB, 0.16509E+01_JPRB, 0.14928E+01_JPRB, 0.13438E+01_JPRB, 0.12504E+01_JPRB, 0.12865E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.11900E+01_JPRB, 0.12424E+01_JPRB, 0.13452E+01_JPRB, 0.16795E+01_JPRB, 0.15359E+01_JPRB, 0.16189E+01_JPRB ,&
&  0.14463E+01_JPRB, 0.14728E+01_JPRB, 0.11992E+01_JPRB, 0.17468E+01_JPRB, 0.94768E+00_JPRB, 0.74475E+00_JPRB, 0.76155E+00_JPRB ,&
&  0.80426E+00_JPRB, 0.82519E+00_JPRB, 0.85551E+00_JPRB, 0.88380E+00_JPRB, 0.88847E+00_JPRB, 0.89278E+00_JPRB, 0.91726E+00_JPRB ,&
&  0.94403E+00_JPRB, 0.96020E+00_JPRB, 0.97579E+00_JPRB, 0.98166E+00_JPRB, 0.99748E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.10877E+01_JPRB, 0.11091E+01_JPRB, 0.11279E+01_JPRB, 0.11427E+01_JPRB, 0.11502E+01_JPRB, 0.11554E+01_JPRB, 0.11601E+01_JPRB ,&
&  0.11608E+01_JPRB, 0.11595E+01_JPRB, 0.11597E+01_JPRB, 0.11771E+01_JPRB, 0.12038E+01_JPRB, 0.12175E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12649E+01_JPRB, 0.12931E+01_JPRB, 0.13174E+01_JPRB, 0.13533E+01_JPRB, 0.13553E+01_JPRB, 0.13378E+01_JPRB ,&
&  0.13321E+01_JPRB, 0.13204E+01_JPRB, 0.13260E+01_JPRB, 0.13345E+01_JPRB, 0.13544E+01_JPRB, 0.13880E+01_JPRB, 0.16117E+01_JPRB ,&
&  0.18475E+01_JPRB, 0.22654E+01_JPRB, 0.20982E+01_JPRB, 0.18973E+01_JPRB, 0.19011E+01_JPRB, 0.17239E+01_JPRB, 0.16251E+01_JPRB ,&
&  0.14786E+01_JPRB, 0.13423E+01_JPRB, 0.13035E+01_JPRB, 0.13017E+01_JPRB, 0.13737E+01_JPRB, 0.14561E+01_JPRB, 0.15217E+01_JPRB ,&
&  0.16429E+01_JPRB, 0.17469E+01_JPRB, 0.18818E+01_JPRB, 0.19807E+01_JPRB, 0.20671E+01_JPRB, 0.23461E+01_JPRB, 0.22334E+01_JPRB ,&
&  0.23264E+01_JPRB, 0.26320E+01_JPRB, 0.28228E+01_JPRB, 0.29612E+01_JPRB, 0.27762E+01_JPRB, 0.25675E+01_JPRB, 0.22927E+01_JPRB ,&
&  0.20244E+01_JPRB, 0.19611E+01_JPRB, 0.21708E+01_JPRB, 0.21446E+01_JPRB, 0.21067E+01_JPRB, 0.21248E+01_JPRB, 0.19128E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.15672E+01_JPRB, 0.14248E+01_JPRB, 0.12968E+01_JPRB, 0.12231E+01_JPRB, 0.12046E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11146E+01_JPRB, 0.11517E+01_JPRB, 0.12312E+01_JPRB, 0.13424E+01_JPRB, 0.22003E+01_JPRB, 0.26903E+01_JPRB ,&
&  0.16853E+01_JPRB, 0.13622E+01_JPRB, 0.85081E+00_JPRB, 0.12351E+01_JPRB, 0.82077E+00_JPRB, 0.72395E+00_JPRB, 0.78466E+00_JPRB ,&
&  0.88589E+00_JPRB, 0.96434E+00_JPRB, 0.10018E+01_JPRB, 0.10450E+01_JPRB, 0.95967E+00_JPRB, 0.95039E+00_JPRB, 0.95508E+00_JPRB ,&
&  0.97296E+00_JPRB, 0.99751E+00_JPRB, 0.10203E+01_JPRB, 0.10409E+01_JPRB, 0.10651E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.11235E+01_JPRB, 0.11439E+01_JPRB, 0.11631E+01_JPRB, 0.11859E+01_JPRB, 0.12059E+01_JPRB, 0.12299E+01_JPRB, 0.12454E+01_JPRB ,&
&  0.12585E+01_JPRB, 0.12681E+01_JPRB, 0.12714E+01_JPRB, 0.12870E+01_JPRB, 0.13042E+01_JPRB, 0.13351E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.14149E+01_JPRB, 0.14623E+01_JPRB, 0.15163E+01_JPRB, 0.15531E+01_JPRB, 0.15851E+01_JPRB, 0.16167E+01_JPRB, 0.16457E+01_JPRB ,&
&  0.16645E+01_JPRB, 0.16657E+01_JPRB, 0.16926E+01_JPRB, 0.17458E+01_JPRB, 0.18061E+01_JPRB, 0.19113E+01_JPRB, 0.17351E+01_JPRB ,&
&  0.21526E+01_JPRB, 0.23789E+01_JPRB, 0.23892E+01_JPRB, 0.22523E+01_JPRB, 0.22401E+01_JPRB, 0.22388E+01_JPRB, 0.22810E+01_JPRB ,&
&  0.23653E+01_JPRB, 0.22838E+01_JPRB, 0.21756E+01_JPRB, 0.19985E+01_JPRB, 0.19044E+01_JPRB, 0.19496E+01_JPRB, 0.20720E+01_JPRB ,&
&  0.21474E+01_JPRB, 0.22358E+01_JPRB, 0.22634E+01_JPRB, 0.22711E+01_JPRB, 0.22899E+01_JPRB, 0.22934E+01_JPRB, 0.21644E+01_JPRB ,&
&  0.20620E+01_JPRB, 0.20317E+01_JPRB, 0.17671E+01_JPRB, 0.17920E+01_JPRB, 0.17911E+01_JPRB, 0.16376E+01_JPRB, 0.13994E+01_JPRB ,&
&  0.13413E+01_JPRB, 0.12623E+01_JPRB, 0.13406E+01_JPRB, 0.17166E+01_JPRB, 0.18298E+01_JPRB, 0.18059E+01_JPRB, 0.16973E+01_JPRB ,&
&  0.15290E+01_JPRB, 0.13641E+01_JPRB, 0.11880E+01_JPRB, 0.10876E+01_JPRB, 0.10652E+01_JPRB, 0.11043E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10920E+01_JPRB, 0.10960E+01_JPRB, 0.11222E+01_JPRB, 0.13054E+01_JPRB, 0.16362E+01_JPRB, 0.24665E+01_JPRB ,&
&  0.17561E+01_JPRB, 0.10369E+01_JPRB, 0.98412E+00_JPRB, 0.10291E+01_JPRB, 0.77843E+00_JPRB, 0.68358E+00_JPRB, 0.77703E+00_JPRB ,&
&  0.99142E+00_JPRB, 0.11214E+01_JPRB, 0.11190E+01_JPRB, 0.11029E+01_JPRB, 0.10567E+01_JPRB, 0.10643E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.10562E+01_JPRB, 0.10688E+01_JPRB, 0.10825E+01_JPRB, 0.11021E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.11403E+01_JPRB, 0.11653E+01_JPRB, 0.11883E+01_JPRB, 0.12108E+01_JPRB, 0.12290E+01_JPRB, 0.12555E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12837E+01_JPRB, 0.12946E+01_JPRB, 0.13160E+01_JPRB, 0.13413E+01_JPRB, 0.13701E+01_JPRB, 0.14061E+01_JPRB, 0.14498E+01_JPRB ,&
&  0.15051E+01_JPRB, 0.15517E+01_JPRB, 0.16051E+01_JPRB, 0.16445E+01_JPRB, 0.16597E+01_JPRB, 0.16886E+01_JPRB, 0.17172E+01_JPRB ,&
&  0.17235E+01_JPRB, 0.17212E+01_JPRB, 0.17542E+01_JPRB, 0.18059E+01_JPRB, 0.18297E+01_JPRB, 0.19063E+01_JPRB, 0.15738E+01_JPRB ,&
&  0.22875E+01_JPRB, 0.26064E+01_JPRB, 0.27405E+01_JPRB, 0.26175E+01_JPRB, 0.25705E+01_JPRB, 0.26666E+01_JPRB, 0.27842E+01_JPRB ,&
&  0.29335E+01_JPRB, 0.30691E+01_JPRB, 0.30080E+01_JPRB, 0.28475E+01_JPRB, 0.26765E+01_JPRB, 0.27070E+01_JPRB, 0.27705E+01_JPRB ,&
&  0.28268E+01_JPRB, 0.28700E+01_JPRB, 0.28632E+01_JPRB, 0.28476E+01_JPRB, 0.28403E+01_JPRB, 0.28060E+01_JPRB, 0.27180E+01_JPRB ,&
&  0.26266E+01_JPRB, 0.25146E+01_JPRB, 0.22062E+01_JPRB, 0.24083E+01_JPRB, 0.22485E+01_JPRB, 0.18706E+01_JPRB, 0.14984E+01_JPRB ,&
&  0.13059E+01_JPRB, 0.12401E+01_JPRB, 0.14451E+01_JPRB, 0.15232E+01_JPRB, 0.14905E+01_JPRB, 0.14574E+01_JPRB, 0.13815E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.10695E+01_JPRB, 0.97913E+00_JPRB, 0.95776E+00_JPRB, 0.97864E+00_JPRB, 0.10186E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.10435E+01_JPRB, 0.10349E+01_JPRB, 0.10241E+01_JPRB, 0.10971E+01_JPRB, 0.12444E+01_JPRB, 0.19992E+01_JPRB ,&
&  0.15731E+01_JPRB, 0.11402E+01_JPRB, 0.14860E+01_JPRB, 0.63251E+00_JPRB, 0.68475E+00_JPRB, 0.62995E+00_JPRB, 0.71478E+00_JPRB ,&
&  0.85391E+00_JPRB, 0.11932E+01_JPRB, 0.16073E+01_JPRB, 0.14137E+01_JPRB, 0.12979E+01_JPRB, 0.11987E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.10844E+01_JPRB, 0.10909E+01_JPRB, 0.10962E+01_JPRB, 0.11148E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.11642E+01_JPRB, 0.11929E+01_JPRB, 0.12037E+01_JPRB, 0.12288E+01_JPRB, 0.12154E+01_JPRB, 0.12386E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.12912E+01_JPRB, 0.13201E+01_JPRB, 0.13498E+01_JPRB, 0.13901E+01_JPRB, 0.14291E+01_JPRB, 0.14690E+01_JPRB ,&
&  0.15170E+01_JPRB, 0.15713E+01_JPRB, 0.16170E+01_JPRB, 0.16746E+01_JPRB, 0.16934E+01_JPRB, 0.17379E+01_JPRB, 0.17774E+01_JPRB ,&
&  0.18493E+01_JPRB, 0.18618E+01_JPRB, 0.18882E+01_JPRB, 0.19214E+01_JPRB, 0.19298E+01_JPRB, 0.20154E+01_JPRB, 0.15301E+01_JPRB ,&
&  0.24232E+01_JPRB, 0.27682E+01_JPRB, 0.30017E+01_JPRB, 0.29304E+01_JPRB, 0.28341E+01_JPRB, 0.28439E+01_JPRB, 0.29027E+01_JPRB ,&
&  0.30399E+01_JPRB, 0.31558E+01_JPRB, 0.31524E+01_JPRB, 0.33698E+01_JPRB, 0.34739E+01_JPRB, 0.34172E+01_JPRB, 0.35349E+01_JPRB ,&
&  0.36307E+01_JPRB, 0.37161E+01_JPRB, 0.37784E+01_JPRB, 0.38464E+01_JPRB, 0.39103E+01_JPRB, 0.40245E+01_JPRB, 0.41005E+01_JPRB ,&
&  0.40813E+01_JPRB, 0.40814E+01_JPRB, 0.40509E+01_JPRB, 0.38397E+01_JPRB, 0.36112E+01_JPRB, 0.30144E+01_JPRB, 0.24327E+01_JPRB ,&
&  0.16476E+01_JPRB, 0.13460E+01_JPRB, 0.13725E+01_JPRB, 0.13957E+01_JPRB, 0.13089E+01_JPRB, 0.12402E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.97389E+00_JPRB, 0.93000E+00_JPRB, 0.90907E+00_JPRB, 0.92389E+00_JPRB, 0.95722E+00_JPRB, 0.98133E+00_JPRB, 0.10015E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.10094E+01_JPRB, 0.97242E+00_JPRB, 0.94022E+00_JPRB, 0.99466E+00_JPRB, 0.11869E+01_JPRB, 0.12782E+01_JPRB ,&
&  0.14855E+01_JPRB, 0.11374E+01_JPRB, 0.99667E+00_JPRB, 0.90344E+00_JPRB, 0.70160E+00_JPRB, 0.67201E+00_JPRB, 0.82827E+00_JPRB ,&
&  0.13368E+01_JPRB, 0.16994E+01_JPRB, 0.23977E+01_JPRB, 0.40381E+01_JPRB, 0.28680E+01_JPRB, 0.21560E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.11001E+01_JPRB, 0.10984E+01_JPRB, 0.10983E+01_JPRB, 0.11287E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.10586E+01_JPRB, 0.10946E+01_JPRB, 0.11329E+01_JPRB, 0.11699E+01_JPRB, 0.11922E+01_JPRB, 0.12037E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.12399E+01_JPRB, 0.12304E+01_JPRB, 0.12613E+01_JPRB, 0.12509E+01_JPRB, 0.12798E+01_JPRB, 0.12858E+01_JPRB, 0.13159E+01_JPRB ,&
&  0.13595E+01_JPRB, 0.14155E+01_JPRB, 0.14807E+01_JPRB, 0.15309E+01_JPRB, 0.15756E+01_JPRB, 0.16606E+01_JPRB, 0.17677E+01_JPRB ,&
&  0.18933E+01_JPRB, 0.19741E+01_JPRB, 0.20502E+01_JPRB, 0.21067E+01_JPRB, 0.20929E+01_JPRB, 0.20757E+01_JPRB, 0.15410E+01_JPRB ,&
&  0.23393E+01_JPRB, 0.28341E+01_JPRB, 0.32170E+01_JPRB, 0.32548E+01_JPRB, 0.30993E+01_JPRB, 0.29482E+01_JPRB, 0.29906E+01_JPRB ,&
&  0.30640E+01_JPRB, 0.30761E+01_JPRB, 0.30757E+01_JPRB, 0.31884E+01_JPRB, 0.36402E+01_JPRB, 0.35881E+01_JPRB, 0.37122E+01_JPRB ,&
&  0.38465E+01_JPRB, 0.40021E+01_JPRB, 0.41999E+01_JPRB, 0.44320E+01_JPRB, 0.46632E+01_JPRB, 0.48850E+01_JPRB, 0.51199E+01_JPRB ,&
&  0.53082E+01_JPRB, 0.53851E+01_JPRB, 0.54378E+01_JPRB, 0.48304E+01_JPRB, 0.44376E+01_JPRB, 0.44661E+01_JPRB, 0.32727E+01_JPRB ,&
&  0.18487E+01_JPRB, 0.14542E+01_JPRB, 0.14388E+01_JPRB, 0.13968E+01_JPRB, 0.13083E+01_JPRB, 0.11591E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.94725E+00_JPRB, 0.91903E+00_JPRB, 0.91089E+00_JPRB, 0.94566E+00_JPRB, 0.97146E+00_JPRB, 0.99645E+00_JPRB, 0.10047E+01_JPRB ,&
&  0.10209E+01_JPRB, 0.10006E+01_JPRB, 0.95807E+00_JPRB, 0.94147E+00_JPRB, 0.10127E+01_JPRB, 0.12295E+01_JPRB, 0.15758E+01_JPRB ,&
&  0.17067E+01_JPRB, 0.10162E+01_JPRB, 0.96373E+00_JPRB, 0.84745E+00_JPRB, 0.79125E+00_JPRB, 0.82082E+00_JPRB, 0.99062E+00_JPRB ,&
&  0.14578E+01_JPRB, 0.22172E+01_JPRB, 0.24512E+01_JPRB, 0.20965E+01_JPRB, 0.25535E+01_JPRB, 0.18606E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10443E+01_JPRB, 0.10124E+01_JPRB, 0.10067E+01_JPRB, 0.10217E+01_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.88029E+00_JPRB, 0.90542E+00_JPRB, 0.93474E+00_JPRB, 0.97749E+00_JPRB, 0.10256E+01_JPRB, 0.10606E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.11392E+01_JPRB, 0.11663E+01_JPRB, 0.11459E+01_JPRB, 0.11704E+01_JPRB, 0.11742E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.12447E+01_JPRB, 0.12918E+01_JPRB, 0.13483E+01_JPRB, 0.14293E+01_JPRB, 0.15259E+01_JPRB, 0.16422E+01_JPRB ,&
&  0.17844E+01_JPRB, 0.19165E+01_JPRB, 0.20558E+01_JPRB, 0.21547E+01_JPRB, 0.22397E+01_JPRB, 0.22928E+01_JPRB, 0.23078E+01_JPRB ,&
&  0.14519E+01_JPRB, 0.25626E+01_JPRB, 0.29215E+01_JPRB, 0.31466E+01_JPRB, 0.33130E+01_JPRB, 0.31684E+01_JPRB, 0.31994E+01_JPRB ,&
&  0.30847E+01_JPRB, 0.28885E+01_JPRB, 0.29399E+01_JPRB, 0.30814E+01_JPRB, 0.33109E+01_JPRB, 0.32875E+01_JPRB, 0.33399E+01_JPRB ,&
&  0.34235E+01_JPRB, 0.35845E+01_JPRB, 0.38048E+01_JPRB, 0.40616E+01_JPRB, 0.43690E+01_JPRB, 0.47306E+01_JPRB, 0.51604E+01_JPRB ,&
&  0.56150E+01_JPRB, 0.60246E+01_JPRB, 0.63899E+01_JPRB, 0.56625E+01_JPRB, 0.50025E+01_JPRB, 0.51457E+01_JPRB, 0.42451E+01_JPRB ,&
&  0.23232E+01_JPRB, 0.14989E+01_JPRB, 0.15045E+01_JPRB, 0.15841E+01_JPRB, 0.14392E+01_JPRB, 0.13072E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.99003E+00_JPRB, 0.10236E+01_JPRB, 0.10068E+01_JPRB, 0.10492E+01_JPRB, 0.10563E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10833E+01_JPRB, 0.10630E+01_JPRB, 0.10519E+01_JPRB, 0.10339E+01_JPRB, 0.10202E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.97199E+00_JPRB, 0.93022E+00_JPRB, 0.93092E+00_JPRB, 0.92769E+00_JPRB, 0.93355E+00_JPRB, 0.10195E+01_JPRB ,&
&  0.11680E+01_JPRB, 0.13406E+01_JPRB, 0.16257E+01_JPRB, 0.17028E+01_JPRB, 0.16198E+01_JPRB, 0.12414E+01_JPRB, 0.93653E+00_JPRB ,&
&  0.93710E+00_JPRB, 0.89799E+00_JPRB, 0.86680E+00_JPRB, 0.85033E+00_JPRB, 0.85859E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.79504E+00_JPRB, 0.81545E+00_JPRB, 0.81124E+00_JPRB, 0.84109E+00_JPRB, 0.86349E+00_JPRB, 0.89611E+00_JPRB, 0.93351E+00_JPRB ,&
&  0.96477E+00_JPRB, 0.99304E+00_JPRB, 0.10351E+01_JPRB, 0.10321E+01_JPRB, 0.10612E+01_JPRB, 0.10735E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.11118E+01_JPRB, 0.11380E+01_JPRB, 0.11683E+01_JPRB, 0.12140E+01_JPRB, 0.12920E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.14908E+01_JPRB, 0.16280E+01_JPRB, 0.18091E+01_JPRB, 0.19860E+01_JPRB, 0.21505E+01_JPRB, 0.22272E+01_JPRB, 0.21938E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.14477E+01_JPRB, 0.24243E+01_JPRB, 0.28020E+01_JPRB, 0.30221E+01_JPRB, 0.31316E+01_JPRB, 0.31372E+01_JPRB ,&
&  0.29325E+01_JPRB, 0.26153E+01_JPRB, 0.26225E+01_JPRB, 0.30012E+01_JPRB, 0.29483E+01_JPRB, 0.29512E+01_JPRB, 0.29591E+01_JPRB ,&
&  0.29966E+01_JPRB, 0.30863E+01_JPRB, 0.32341E+01_JPRB, 0.34324E+01_JPRB, 0.37426E+01_JPRB, 0.41472E+01_JPRB, 0.46679E+01_JPRB ,&
&  0.53026E+01_JPRB, 0.59627E+01_JPRB, 0.65539E+01_JPRB, 0.57937E+01_JPRB, 0.56674E+01_JPRB, 0.56438E+01_JPRB, 0.58593E+01_JPRB ,&
&  0.30285E+01_JPRB, 0.16924E+01_JPRB, 0.17236E+01_JPRB, 0.18201E+01_JPRB, 0.16051E+01_JPRB, 0.15079E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.11935E+01_JPRB, 0.11713E+01_JPRB, 0.11401E+01_JPRB, 0.11334E+01_JPRB, 0.11552E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.12209E+01_JPRB, 0.12210E+01_JPRB, 0.12167E+01_JPRB, 0.11743E+01_JPRB, 0.11297E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.10724E+01_JPRB, 0.10701E+01_JPRB, 0.11005E+01_JPRB, 0.11256E+01_JPRB, 0.11311E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.11100E+01_JPRB, 0.12172E+01_JPRB, 0.12483E+01_JPRB, 0.12181E+01_JPRB, 0.10315E+01_JPRB, 0.86409E+00_JPRB ,&
&  0.84702E+00_JPRB, 0.82112E+00_JPRB, 0.79754E+00_JPRB, 0.79201E+00_JPRB, 0.78767E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.73214E+00_JPRB, 0.71709E+00_JPRB, 0.70126E+00_JPRB, 0.71091E+00_JPRB, 0.72271E+00_JPRB, 0.74176E+00_JPRB, 0.79774E+00_JPRB ,&
&  0.84596E+00_JPRB, 0.86385E+00_JPRB, 0.89018E+00_JPRB, 0.90068E+00_JPRB, 0.92379E+00_JPRB, 0.92949E+00_JPRB, 0.95021E+00_JPRB ,&
&  0.95752E+00_JPRB, 0.97816E+00_JPRB, 0.10091E+01_JPRB, 0.10377E+01_JPRB, 0.10599E+01_JPRB, 0.10902E+01_JPRB, 0.11355E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.12568E+01_JPRB, 0.13732E+01_JPRB, 0.15674E+01_JPRB, 0.18681E+01_JPRB, 0.22246E+01_JPRB, 0.24911E+01_JPRB ,&
&  0.19226E+01_JPRB, 0.82149E+00_JPRB, 0.16883E+01_JPRB, 0.24893E+01_JPRB, 0.26251E+01_JPRB, 0.27712E+01_JPRB, 0.27047E+01_JPRB ,&
&  0.25022E+01_JPRB, 0.23327E+01_JPRB, 0.22559E+01_JPRB, 0.26192E+01_JPRB, 0.26686E+01_JPRB, 0.26819E+01_JPRB, 0.26786E+01_JPRB ,&
&  0.26641E+01_JPRB, 0.26900E+01_JPRB, 0.27457E+01_JPRB, 0.28615E+01_JPRB, 0.30929E+01_JPRB, 0.34643E+01_JPRB, 0.39559E+01_JPRB ,&
&  0.46209E+01_JPRB, 0.55310E+01_JPRB, 0.64525E+01_JPRB, 0.59979E+01_JPRB, 0.61969E+01_JPRB, 0.54919E+01_JPRB, 0.47010E+01_JPRB ,&
&  0.27675E+01_JPRB, 0.18911E+01_JPRB, 0.18886E+01_JPRB, 0.20477E+01_JPRB, 0.18238E+01_JPRB, 0.15814E+01_JPRB, 0.16460E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.14120E+01_JPRB, 0.13509E+01_JPRB, 0.13018E+01_JPRB, 0.12832E+01_JPRB, 0.12767E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.12889E+01_JPRB, 0.13125E+01_JPRB, 0.13090E+01_JPRB, 0.12773E+01_JPRB, 0.12535E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.12574E+01_JPRB, 0.12447E+01_JPRB, 0.12553E+01_JPRB, 0.12698E+01_JPRB, 0.12723E+01_JPRB, 0.12326E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.11862E+01_JPRB, 0.11518E+01_JPRB, 0.10983E+01_JPRB, 0.10591E+01_JPRB, 0.97344E+00_JPRB, 0.87597E+00_JPRB ,&
&  0.84327E+00_JPRB, 0.80777E+00_JPRB, 0.78431E+00_JPRB, 0.76818E+00_JPRB, 0.75164E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.79392E+00_JPRB, 0.77621E+00_JPRB, 0.74200E+00_JPRB, 0.69347E+00_JPRB, 0.68632E+00_JPRB, 0.68273E+00_JPRB, 0.69391E+00_JPRB ,&
&  0.74121E+00_JPRB, 0.79858E+00_JPRB, 0.81317E+00_JPRB, 0.81350E+00_JPRB, 0.79191E+00_JPRB, 0.79794E+00_JPRB, 0.81112E+00_JPRB ,&
&  0.83388E+00_JPRB, 0.85531E+00_JPRB, 0.88669E+00_JPRB, 0.92580E+00_JPRB, 0.96037E+00_JPRB, 0.98769E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.10286E+01_JPRB, 0.10560E+01_JPRB, 0.11086E+01_JPRB, 0.12262E+01_JPRB, 0.14801E+01_JPRB, 0.19147E+01_JPRB ,&
&  0.29137E+01_JPRB, 0.24236E+01_JPRB, 0.11244E+01_JPRB, 0.20968E+01_JPRB, 0.24493E+01_JPRB, 0.26809E+01_JPRB, 0.23091E+01_JPRB ,&
&  0.22270E+01_JPRB, 0.19909E+01_JPRB, 0.18500E+01_JPRB, 0.20771E+01_JPRB, 0.21539E+01_JPRB, 0.22229E+01_JPRB, 0.22381E+01_JPRB ,&
&  0.21767E+01_JPRB, 0.21996E+01_JPRB, 0.22444E+01_JPRB, 0.23080E+01_JPRB, 0.24238E+01_JPRB, 0.26934E+01_JPRB, 0.31829E+01_JPRB ,&
&  0.38105E+01_JPRB, 0.46674E+01_JPRB, 0.59686E+01_JPRB, 0.65803E+01_JPRB, 0.72166E+01_JPRB, 0.53598E+01_JPRB, 0.40953E+01_JPRB ,&
&  0.26024E+01_JPRB, 0.22961E+01_JPRB, 0.23636E+01_JPRB, 0.23297E+01_JPRB, 0.20952E+01_JPRB, 0.18898E+01_JPRB, 0.19468E+01_JPRB ,&
&  0.17561E+01_JPRB, 0.16288E+01_JPRB, 0.15594E+01_JPRB, 0.14946E+01_JPRB, 0.14473E+01_JPRB, 0.14200E+01_JPRB, 0.13970E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.13791E+01_JPRB, 0.13922E+01_JPRB, 0.14039E+01_JPRB, 0.14018E+01_JPRB, 0.13811E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.13753E+01_JPRB, 0.13760E+01_JPRB, 0.13775E+01_JPRB, 0.13902E+01_JPRB, 0.13431E+01_JPRB, 0.12778E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.12682E+01_JPRB, 0.12311E+01_JPRB, 0.12168E+01_JPRB, 0.11276E+01_JPRB, 0.10242E+01_JPRB, 0.92944E+00_JPRB ,&
&  0.90889E+00_JPRB, 0.89195E+00_JPRB, 0.84692E+00_JPRB, 0.79951E+00_JPRB, 0.78899E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.87959E+00_JPRB, 0.88902E+00_JPRB, 0.88187E+00_JPRB, 0.83094E+00_JPRB, 0.78360E+00_JPRB, 0.73754E+00_JPRB, 0.70648E+00_JPRB ,&
&  0.69867E+00_JPRB, 0.70178E+00_JPRB, 0.73124E+00_JPRB, 0.75801E+00_JPRB, 0.75120E+00_JPRB, 0.72689E+00_JPRB, 0.71956E+00_JPRB ,&
&  0.72939E+00_JPRB, 0.75691E+00_JPRB, 0.78229E+00_JPRB, 0.82224E+00_JPRB, 0.85846E+00_JPRB, 0.88490E+00_JPRB, 0.91200E+00_JPRB ,&
&  0.90545E+00_JPRB, 0.90658E+00_JPRB, 0.91671E+00_JPRB, 0.93030E+00_JPRB, 0.96014E+00_JPRB, 0.10372E+01_JPRB, 0.11480E+01_JPRB ,&
&  0.15163E+01_JPRB, 0.25823E+01_JPRB, 0.14785E+01_JPRB, 0.21021E+01_JPRB, 0.25362E+01_JPRB, 0.26060E+01_JPRB, 0.23249E+01_JPRB ,&
&  0.21094E+01_JPRB, 0.18166E+01_JPRB, 0.17156E+01_JPRB, 0.17387E+01_JPRB, 0.16176E+01_JPRB, 0.16442E+01_JPRB, 0.16732E+01_JPRB ,&
&  0.16954E+01_JPRB, 0.17466E+01_JPRB, 0.17931E+01_JPRB, 0.18342E+01_JPRB, 0.18593E+01_JPRB, 0.19479E+01_JPRB, 0.22331E+01_JPRB ,&
&  0.27868E+01_JPRB, 0.35905E+01_JPRB, 0.49707E+01_JPRB, 0.62552E+01_JPRB, 0.77013E+01_JPRB, 0.59493E+01_JPRB, 0.43373E+01_JPRB ,&
&  0.31093E+01_JPRB, 0.29012E+01_JPRB, 0.28805E+01_JPRB, 0.28163E+01_JPRB, 0.25057E+01_JPRB, 0.24676E+01_JPRB, 0.24965E+01_JPRB ,&
&  0.21920E+01_JPRB, 0.19460E+01_JPRB, 0.18192E+01_JPRB, 0.17333E+01_JPRB, 0.16447E+01_JPRB, 0.15788E+01_JPRB, 0.15348E+01_JPRB ,&
&  0.14997E+01_JPRB, 0.14838E+01_JPRB, 0.14967E+01_JPRB, 0.15101E+01_JPRB, 0.15229E+01_JPRB, 0.15135E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.14575E+01_JPRB, 0.14343E+01_JPRB, 0.14222E+01_JPRB, 0.14104E+01_JPRB, 0.13702E+01_JPRB, 0.13191E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.13252E+01_JPRB, 0.13101E+01_JPRB, 0.12588E+01_JPRB, 0.12601E+01_JPRB, 0.11672E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.10709E+01_JPRB, 0.11998E+01_JPRB, 0.12379E+01_JPRB, 0.89036E+00_JPRB, 0.87549E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.94287E+00_JPRB, 0.94076E+00_JPRB, 0.92514E+00_JPRB, 0.91951E+00_JPRB, 0.88282E+00_JPRB, 0.83605E+00_JPRB, 0.79084E+00_JPRB ,&
&  0.75850E+00_JPRB, 0.73726E+00_JPRB, 0.73251E+00_JPRB, 0.72781E+00_JPRB, 0.72776E+00_JPRB, 0.71964E+00_JPRB, 0.71548E+00_JPRB ,&
&  0.70430E+00_JPRB, 0.71198E+00_JPRB, 0.73068E+00_JPRB, 0.76350E+00_JPRB, 0.80206E+00_JPRB, 0.81452E+00_JPRB, 0.82539E+00_JPRB ,&
&  0.83089E+00_JPRB, 0.82886E+00_JPRB, 0.82938E+00_JPRB, 0.83873E+00_JPRB, 0.85983E+00_JPRB, 0.89010E+00_JPRB, 0.94371E+00_JPRB ,&
&  0.10888E+01_JPRB, 0.21748E+01_JPRB, 0.32754E+01_JPRB, 0.37162E+01_JPRB, 0.32844E+01_JPRB, 0.27968E+01_JPRB, 0.25104E+01_JPRB ,&
&  0.24040E+01_JPRB, 0.21833E+01_JPRB, 0.20366E+01_JPRB, 0.17229E+01_JPRB, 0.14523E+01_JPRB, 0.13605E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.14558E+01_JPRB, 0.14974E+01_JPRB, 0.15159E+01_JPRB, 0.15080E+01_JPRB, 0.14719E+01_JPRB, 0.14663E+01_JPRB, 0.15444E+01_JPRB ,&
&  0.17984E+01_JPRB, 0.23525E+01_JPRB, 0.33785E+01_JPRB, 0.51663E+01_JPRB, 0.70935E+01_JPRB, 0.63344E+01_JPRB, 0.50640E+01_JPRB ,&
&  0.46183E+01_JPRB, 0.41515E+01_JPRB, 0.37407E+01_JPRB, 0.35157E+01_JPRB, 0.28011E+01_JPRB, 0.33727E+01_JPRB, 0.31540E+01_JPRB ,&
&  0.28820E+01_JPRB, 0.26944E+01_JPRB, 0.24336E+01_JPRB, 0.21559E+01_JPRB, 0.19524E+01_JPRB, 0.17694E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.15881E+01_JPRB, 0.15939E+01_JPRB, 0.16157E+01_JPRB, 0.16547E+01_JPRB, 0.16968E+01_JPRB, 0.17048E+01_JPRB, 0.16615E+01_JPRB ,&
&  0.15660E+01_JPRB, 0.14820E+01_JPRB, 0.13701E+01_JPRB, 0.13497E+01_JPRB, 0.13307E+01_JPRB, 0.12823E+01_JPRB, 0.12529E+01_JPRB ,&
&  0.12720E+01_JPRB, 0.13186E+01_JPRB, 0.13302E+01_JPRB, 0.13052E+01_JPRB, 0.12731E+01_JPRB, 0.12483E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.11306E+01_JPRB, 0.10688E+01_JPRB, 0.99366E+00_JPRB, 0.96720E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.99867E+00_JPRB, 0.96282E+00_JPRB, 0.94030E+00_JPRB, 0.93683E+00_JPRB, 0.91495E+00_JPRB, 0.89761E+00_JPRB, 0.88652E+00_JPRB ,&
&  0.83599E+00_JPRB, 0.79596E+00_JPRB, 0.75999E+00_JPRB, 0.74320E+00_JPRB, 0.73948E+00_JPRB, 0.73292E+00_JPRB, 0.73888E+00_JPRB ,&
&  0.73281E+00_JPRB, 0.71934E+00_JPRB, 0.71900E+00_JPRB, 0.73939E+00_JPRB, 0.76990E+00_JPRB, 0.77889E+00_JPRB, 0.79475E+00_JPRB ,&
&  0.81162E+00_JPRB, 0.80951E+00_JPRB, 0.79875E+00_JPRB, 0.79694E+00_JPRB, 0.80176E+00_JPRB, 0.80257E+00_JPRB, 0.83215E+00_JPRB ,&
&  0.97068E+00_JPRB, 0.22240E+01_JPRB, 0.23120E+01_JPRB, 0.41975E+01_JPRB, 0.36687E+01_JPRB, 0.31530E+01_JPRB, 0.24841E+01_JPRB ,&
&  0.24843E+01_JPRB, 0.27833E+01_JPRB, 0.22666E+01_JPRB, 0.18971E+01_JPRB, 0.15748E+01_JPRB, 0.13407E+01_JPRB, 0.12782E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.13773E+01_JPRB, 0.13983E+01_JPRB, 0.13433E+01_JPRB, 0.12771E+01_JPRB, 0.12505E+01_JPRB, 0.12679E+01_JPRB ,&
&  0.13312E+01_JPRB, 0.15142E+01_JPRB, 0.20353E+01_JPRB, 0.31163E+01_JPRB, 0.54887E+01_JPRB, 0.65766E+01_JPRB, 0.59627E+01_JPRB ,&
&  0.56796E+01_JPRB, 0.58081E+01_JPRB, 0.52457E+01_JPRB, 0.46176E+01_JPRB, 0.37833E+01_JPRB, 0.37715E+01_JPRB, 0.35494E+01_JPRB ,&
&  0.33074E+01_JPRB, 0.32326E+01_JPRB, 0.31102E+01_JPRB, 0.28534E+01_JPRB, 0.25388E+01_JPRB, 0.22167E+01_JPRB, 0.19241E+01_JPRB ,&
&  0.17211E+01_JPRB, 0.16311E+01_JPRB, 0.15990E+01_JPRB, 0.16524E+01_JPRB, 0.17510E+01_JPRB, 0.18175E+01_JPRB, 0.18245E+01_JPRB ,&
&  0.17447E+01_JPRB, 0.16185E+01_JPRB, 0.14106E+01_JPRB, 0.13076E+01_JPRB, 0.12846E+01_JPRB, 0.12362E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.13034E+01_JPRB, 0.13056E+01_JPRB, 0.12809E+01_JPRB, 0.12685E+01_JPRB, 0.12953E+01_JPRB, 0.12166E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.11681E+01_JPRB, 0.11635E+01_JPRB, 0.10968E+01_JPRB, 0.10515E+01_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.10664E+01_JPRB, 0.10102E+01_JPRB, 0.96686E+00_JPRB, 0.94662E+00_JPRB, 0.93730E+00_JPRB, 0.92654E+00_JPRB, 0.91504E+00_JPRB ,&
&  0.88412E+00_JPRB, 0.84568E+00_JPRB, 0.80854E+00_JPRB, 0.77754E+00_JPRB, 0.77033E+00_JPRB, 0.76240E+00_JPRB, 0.75877E+00_JPRB ,&
&  0.75024E+00_JPRB, 0.73754E+00_JPRB, 0.72881E+00_JPRB, 0.73495E+00_JPRB, 0.74717E+00_JPRB, 0.75852E+00_JPRB, 0.77598E+00_JPRB ,&
&  0.78276E+00_JPRB, 0.79163E+00_JPRB, 0.79321E+00_JPRB, 0.78828E+00_JPRB, 0.77918E+00_JPRB, 0.76761E+00_JPRB, 0.76333E+00_JPRB ,&
&  0.94610E+00_JPRB, 0.26816E+01_JPRB, 0.20661E+01_JPRB, 0.38208E+01_JPRB, 0.33466E+01_JPRB, 0.30355E+01_JPRB, 0.24232E+01_JPRB ,&
&  0.22918E+01_JPRB, 0.25280E+01_JPRB, 0.22586E+01_JPRB, 0.19866E+01_JPRB, 0.17622E+01_JPRB, 0.15457E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.13467E+01_JPRB, 0.13357E+01_JPRB, 0.13138E+01_JPRB, 0.12640E+01_JPRB, 0.12233E+01_JPRB, 0.11918E+01_JPRB, 0.11891E+01_JPRB ,&
&  0.12020E+01_JPRB, 0.12515E+01_JPRB, 0.14473E+01_JPRB, 0.21156E+01_JPRB, 0.36202E+01_JPRB, 0.55589E+01_JPRB, 0.68481E+01_JPRB ,&
&  0.69979E+01_JPRB, 0.75936E+01_JPRB, 0.67822E+01_JPRB, 0.57742E+01_JPRB, 0.49642E+01_JPRB, 0.44886E+01_JPRB, 0.40618E+01_JPRB ,&
&  0.36250E+01_JPRB, 0.33612E+01_JPRB, 0.32775E+01_JPRB, 0.32179E+01_JPRB, 0.30870E+01_JPRB, 0.28531E+01_JPRB, 0.25486E+01_JPRB ,&
&  0.22278E+01_JPRB, 0.19976E+01_JPRB, 0.18438E+01_JPRB, 0.17796E+01_JPRB, 0.17876E+01_JPRB, 0.18348E+01_JPRB, 0.18547E+01_JPRB ,&
&  0.17815E+01_JPRB, 0.16817E+01_JPRB, 0.14948E+01_JPRB, 0.13527E+01_JPRB, 0.13070E+01_JPRB, 0.12534E+01_JPRB, 0.12179E+01_JPRB ,&
&  0.12665E+01_JPRB, 0.13016E+01_JPRB, 0.12891E+01_JPRB, 0.12696E+01_JPRB, 0.12383E+01_JPRB, 0.13069E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11775E+01_JPRB, 0.11675E+01_JPRB, 0.11412E+01_JPRB, 0.11162E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.11106E+01_JPRB, 0.10692E+01_JPRB, 0.10261E+01_JPRB, 0.99260E+00_JPRB, 0.96560E+00_JPRB, 0.95082E+00_JPRB, 0.93779E+00_JPRB ,&
&  0.90910E+00_JPRB, 0.87751E+00_JPRB, 0.85126E+00_JPRB, 0.83036E+00_JPRB, 0.81383E+00_JPRB, 0.80327E+00_JPRB, 0.78798E+00_JPRB ,&
&  0.76985E+00_JPRB, 0.74302E+00_JPRB, 0.74470E+00_JPRB, 0.75498E+00_JPRB, 0.75846E+00_JPRB, 0.76284E+00_JPRB, 0.78483E+00_JPRB ,&
&  0.78014E+00_JPRB, 0.77893E+00_JPRB, 0.78145E+00_JPRB, 0.78277E+00_JPRB, 0.77158E+00_JPRB, 0.76969E+00_JPRB, 0.75269E+00_JPRB ,&
&  0.90950E+00_JPRB, 0.11301E+01_JPRB, 0.23780E+01_JPRB, 0.32907E+01_JPRB, 0.30164E+01_JPRB, 0.27116E+01_JPRB, 0.24441E+01_JPRB ,&
&  0.23832E+01_JPRB, 0.22931E+01_JPRB, 0.20572E+01_JPRB, 0.18937E+01_JPRB, 0.17364E+01_JPRB, 0.16098E+01_JPRB, 0.15088E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.13553E+01_JPRB, 0.12950E+01_JPRB, 0.12322E+01_JPRB, 0.11873E+01_JPRB, 0.11598E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.11495E+01_JPRB, 0.12066E+01_JPRB, 0.15514E+01_JPRB, 0.27961E+01_JPRB, 0.44388E+01_JPRB, 0.62744E+01_JPRB ,&
&  0.76134E+01_JPRB, 0.81780E+01_JPRB, 0.74686E+01_JPRB, 0.65752E+01_JPRB, 0.56853E+01_JPRB, 0.50001E+01_JPRB, 0.44700E+01_JPRB ,&
&  0.39904E+01_JPRB, 0.35744E+01_JPRB, 0.32514E+01_JPRB, 0.30791E+01_JPRB, 0.29783E+01_JPRB, 0.29005E+01_JPRB, 0.28023E+01_JPRB ,&
&  0.26545E+01_JPRB, 0.24406E+01_JPRB, 0.22164E+01_JPRB, 0.20371E+01_JPRB, 0.19306E+01_JPRB, 0.18903E+01_JPRB, 0.18575E+01_JPRB ,&
&  0.17626E+01_JPRB, 0.16622E+01_JPRB, 0.15154E+01_JPRB, 0.14165E+01_JPRB, 0.13941E+01_JPRB, 0.13544E+01_JPRB, 0.13163E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.13146E+01_JPRB, 0.12956E+01_JPRB, 0.12831E+01_JPRB, 0.12329E+01_JPRB, 0.13061E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.11892E+01_JPRB, 0.11619E+01_JPRB, 0.11428E+01_JPRB, 0.11335E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.11424E+01_JPRB, 0.11199E+01_JPRB, 0.10725E+01_JPRB, 0.10265E+01_JPRB, 0.99325E+00_JPRB, 0.96979E+00_JPRB, 0.94944E+00_JPRB ,&
&  0.92572E+00_JPRB, 0.89593E+00_JPRB, 0.87034E+00_JPRB, 0.85638E+00_JPRB, 0.85175E+00_JPRB, 0.84524E+00_JPRB, 0.82603E+00_JPRB ,&
&  0.80072E+00_JPRB, 0.77884E+00_JPRB, 0.77661E+00_JPRB, 0.78148E+00_JPRB, 0.78566E+00_JPRB, 0.77306E+00_JPRB, 0.78240E+00_JPRB ,&
&  0.78791E+00_JPRB, 0.78495E+00_JPRB, 0.78179E+00_JPRB, 0.77782E+00_JPRB, 0.77162E+00_JPRB, 0.76621E+00_JPRB, 0.75191E+00_JPRB ,&
&  0.85405E+00_JPRB, 0.14774E+01_JPRB, 0.23088E+01_JPRB, 0.28427E+01_JPRB, 0.28033E+01_JPRB, 0.24594E+01_JPRB, 0.22524E+01_JPRB ,&
&  0.20782E+01_JPRB, 0.19886E+01_JPRB, 0.18146E+01_JPRB, 0.16933E+01_JPRB, 0.15821E+01_JPRB, 0.14904E+01_JPRB, 0.14115E+01_JPRB ,&
&  0.13543E+01_JPRB, 0.13042E+01_JPRB, 0.12501E+01_JPRB, 0.12005E+01_JPRB, 0.11624E+01_JPRB, 0.11314E+01_JPRB, 0.11153E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10995E+01_JPRB, 0.11100E+01_JPRB, 0.12510E+01_JPRB, 0.21165E+01_JPRB, 0.27642E+01_JPRB, 0.48519E+01_JPRB ,&
&  0.65322E+01_JPRB, 0.62289E+01_JPRB, 0.59596E+01_JPRB, 0.54763E+01_JPRB, 0.48571E+01_JPRB, 0.43350E+01_JPRB, 0.37896E+01_JPRB ,&
&  0.33204E+01_JPRB, 0.29517E+01_JPRB, 0.26511E+01_JPRB, 0.24462E+01_JPRB, 0.23210E+01_JPRB, 0.22593E+01_JPRB, 0.22383E+01_JPRB ,&
&  0.22284E+01_JPRB, 0.22117E+01_JPRB, 0.21712E+01_JPRB, 0.20980E+01_JPRB, 0.19932E+01_JPRB, 0.18826E+01_JPRB, 0.17721E+01_JPRB ,&
&  0.16575E+01_JPRB, 0.15803E+01_JPRB, 0.14816E+01_JPRB, 0.14186E+01_JPRB, 0.14347E+01_JPRB, 0.14668E+01_JPRB, 0.14347E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.13310E+01_JPRB, 0.13197E+01_JPRB, 0.12697E+01_JPRB, 0.12137E+01_JPRB, 0.13058E+01_JPRB, 0.12787E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.12041E+01_JPRB, 0.11775E+01_JPRB, 0.11488E+01_JPRB, 0.11523E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.11667E+01_JPRB, 0.11246E+01_JPRB, 0.10828E+01_JPRB, 0.10396E+01_JPRB, 0.10082E+01_JPRB, 0.97500E+00_JPRB, 0.94301E+00_JPRB ,&
&  0.91665E+00_JPRB, 0.90111E+00_JPRB, 0.88714E+00_JPRB, 0.86915E+00_JPRB, 0.86657E+00_JPRB, 0.85925E+00_JPRB, 0.84166E+00_JPRB ,&
&  0.81468E+00_JPRB, 0.79454E+00_JPRB, 0.78570E+00_JPRB, 0.78332E+00_JPRB, 0.78940E+00_JPRB, 0.78947E+00_JPRB, 0.79456E+00_JPRB ,&
&  0.80026E+00_JPRB, 0.78357E+00_JPRB, 0.77153E+00_JPRB, 0.76943E+00_JPRB, 0.76692E+00_JPRB, 0.76710E+00_JPRB, 0.75924E+00_JPRB ,&
&  0.82486E+00_JPRB, 0.11756E+01_JPRB, 0.16580E+01_JPRB, 0.22837E+01_JPRB, 0.23684E+01_JPRB, 0.21838E+01_JPRB, 0.20059E+01_JPRB ,&
&  0.18572E+01_JPRB, 0.17431E+01_JPRB, 0.16016E+01_JPRB, 0.14400E+01_JPRB, 0.13141E+01_JPRB, 0.12327E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.11332E+01_JPRB, 0.11085E+01_JPRB, 0.10857E+01_JPRB, 0.10631E+01_JPRB, 0.10434E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10251E+01_JPRB, 0.10180E+01_JPRB, 0.10184E+01_JPRB, 0.10529E+01_JPRB, 0.14151E+01_JPRB, 0.18036E+01_JPRB, 0.27361E+01_JPRB ,&
&  0.31545E+01_JPRB, 0.33581E+01_JPRB, 0.35420E+01_JPRB, 0.35065E+01_JPRB, 0.32897E+01_JPRB, 0.30640E+01_JPRB, 0.28247E+01_JPRB ,&
&  0.25667E+01_JPRB, 0.23102E+01_JPRB, 0.21331E+01_JPRB, 0.20038E+01_JPRB, 0.18840E+01_JPRB, 0.17956E+01_JPRB, 0.17441E+01_JPRB ,&
&  0.17356E+01_JPRB, 0.17693E+01_JPRB, 0.18106E+01_JPRB, 0.18391E+01_JPRB, 0.18208E+01_JPRB, 0.17712E+01_JPRB, 0.16875E+01_JPRB ,&
&  0.15745E+01_JPRB, 0.14827E+01_JPRB, 0.14329E+01_JPRB, 0.14173E+01_JPRB, 0.14252E+01_JPRB, 0.14317E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.14296E+01_JPRB, 0.13934E+01_JPRB, 0.13568E+01_JPRB, 0.13061E+01_JPRB, 0.12548E+01_JPRB, 0.13205E+01_JPRB, 0.12995E+01_JPRB ,&
&  0.12648E+01_JPRB, 0.12278E+01_JPRB, 0.12007E+01_JPRB, 0.11707E+01_JPRB, 0.11968E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.12282E+01_JPRB, 0.11566E+01_JPRB, 0.11002E+01_JPRB, 0.10582E+01_JPRB, 0.10303E+01_JPRB, 0.99544E+00_JPRB, 0.95447E+00_JPRB ,&
&  0.92357E+00_JPRB, 0.91120E+00_JPRB, 0.89814E+00_JPRB, 0.87534E+00_JPRB, 0.85757E+00_JPRB, 0.84074E+00_JPRB, 0.82576E+00_JPRB ,&
&  0.80357E+00_JPRB, 0.79267E+00_JPRB, 0.78365E+00_JPRB, 0.78126E+00_JPRB, 0.77638E+00_JPRB, 0.77605E+00_JPRB, 0.78406E+00_JPRB ,&
&  0.78725E+00_JPRB, 0.76993E+00_JPRB, 0.76694E+00_JPRB, 0.76414E+00_JPRB, 0.75977E+00_JPRB, 0.75650E+00_JPRB, 0.74979E+00_JPRB ,&
&  0.82147E+00_JPRB, 0.95423E+00_JPRB, 0.12455E+01_JPRB, 0.15517E+01_JPRB, 0.16957E+01_JPRB, 0.17577E+01_JPRB, 0.17037E+01_JPRB ,&
&  0.16147E+01_JPRB, 0.15041E+01_JPRB, 0.13834E+01_JPRB, 0.12669E+01_JPRB, 0.11576E+01_JPRB, 0.10809E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10058E+01_JPRB, 0.99116E+00_JPRB, 0.97812E+00_JPRB, 0.96814E+00_JPRB, 0.95805E+00_JPRB, 0.94452E+00_JPRB, 0.93917E+00_JPRB ,&
&  0.94144E+00_JPRB, 0.93108E+00_JPRB, 0.92881E+00_JPRB, 0.94758E+00_JPRB, 0.10117E+01_JPRB, 0.11678E+01_JPRB, 0.15872E+01_JPRB ,&
&  0.18607E+01_JPRB, 0.20737E+01_JPRB, 0.22982E+01_JPRB, 0.23878E+01_JPRB, 0.23281E+01_JPRB, 0.22100E+01_JPRB, 0.20453E+01_JPRB ,&
&  0.19138E+01_JPRB, 0.18203E+01_JPRB, 0.17347E+01_JPRB, 0.16493E+01_JPRB, 0.15764E+01_JPRB, 0.15212E+01_JPRB, 0.14834E+01_JPRB ,&
&  0.14574E+01_JPRB, 0.14619E+01_JPRB, 0.14858E+01_JPRB, 0.15131E+01_JPRB, 0.15317E+01_JPRB, 0.15321E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.14562E+01_JPRB, 0.13798E+01_JPRB, 0.13328E+01_JPRB, 0.13336E+01_JPRB, 0.13644E+01_JPRB, 0.13825E+01_JPRB, 0.13904E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.13977E+01_JPRB, 0.13518E+01_JPRB, 0.13094E+01_JPRB, 0.13649E+01_JPRB, 0.13362E+01_JPRB, 0.12995E+01_JPRB ,&
&  0.12594E+01_JPRB, 0.12315E+01_JPRB, 0.12013E+01_JPRB, 0.11669E+01_JPRB, 0.12224E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.10643E+01_JPRB, 0.10550E+01_JPRB, 0.10273E+01_JPRB, 0.10038E+01_JPRB, 0.99120E+00_JPRB, 0.97878E+00_JPRB, 0.95722E+00_JPRB ,&
&  0.93261E+00_JPRB, 0.90902E+00_JPRB, 0.89324E+00_JPRB, 0.87088E+00_JPRB, 0.84760E+00_JPRB, 0.83177E+00_JPRB, 0.81653E+00_JPRB ,&
&  0.79695E+00_JPRB, 0.78733E+00_JPRB, 0.78302E+00_JPRB, 0.77748E+00_JPRB, 0.76943E+00_JPRB, 0.76313E+00_JPRB, 0.76636E+00_JPRB ,&
&  0.76577E+00_JPRB, 0.76560E+00_JPRB, 0.76711E+00_JPRB, 0.76078E+00_JPRB, 0.75138E+00_JPRB, 0.74649E+00_JPRB, 0.73317E+00_JPRB ,&
&  0.75855E+00_JPRB, 0.77851E+00_JPRB, 0.91954E+00_JPRB, 0.10949E+01_JPRB, 0.12459E+01_JPRB, 0.13540E+01_JPRB, 0.14170E+01_JPRB ,&
&  0.13880E+01_JPRB, 0.12976E+01_JPRB, 0.12046E+01_JPRB, 0.11426E+01_JPRB, 0.10817E+01_JPRB, 0.10316E+01_JPRB, 0.98889E+00_JPRB ,&
&  0.96002E+00_JPRB, 0.94236E+00_JPRB, 0.92788E+00_JPRB, 0.91422E+00_JPRB, 0.89991E+00_JPRB, 0.88752E+00_JPRB, 0.87922E+00_JPRB ,&
&  0.87100E+00_JPRB, 0.86910E+00_JPRB, 0.87107E+00_JPRB, 0.86997E+00_JPRB, 0.88602E+00_JPRB, 0.96818E+00_JPRB, 0.11492E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.14245E+01_JPRB, 0.16148E+01_JPRB, 0.17263E+01_JPRB, 0.17129E+01_JPRB, 0.16565E+01_JPRB, 0.15769E+01_JPRB ,&
&  0.15086E+01_JPRB, 0.14350E+01_JPRB, 0.13868E+01_JPRB, 0.13385E+01_JPRB, 0.12870E+01_JPRB, 0.12441E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12080E+01_JPRB, 0.12200E+01_JPRB, 0.12391E+01_JPRB, 0.12534E+01_JPRB, 0.12659E+01_JPRB, 0.12791E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.12659E+01_JPRB, 0.12311E+01_JPRB, 0.12311E+01_JPRB, 0.12622E+01_JPRB, 0.12888E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.13215E+01_JPRB, 0.13368E+01_JPRB, 0.13418E+01_JPRB, 0.13472E+01_JPRB, 0.13268E+01_JPRB, 0.12816E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.11648E+01_JPRB, 0.11408E+01_JPRB, 0.11082E+01_JPRB, 0.10187E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.99570E+00_JPRB, 0.97995E+00_JPRB, 0.95651E+00_JPRB, 0.93510E+00_JPRB, 0.92466E+00_JPRB, 0.92098E+00_JPRB, 0.91969E+00_JPRB ,&
&  0.91271E+00_JPRB, 0.90171E+00_JPRB, 0.88861E+00_JPRB, 0.87205E+00_JPRB, 0.85333E+00_JPRB, 0.83635E+00_JPRB, 0.82106E+00_JPRB ,&
&  0.79910E+00_JPRB, 0.78499E+00_JPRB, 0.78205E+00_JPRB, 0.77843E+00_JPRB, 0.77483E+00_JPRB, 0.76839E+00_JPRB, 0.76566E+00_JPRB ,&
&  0.76697E+00_JPRB, 0.76864E+00_JPRB, 0.76961E+00_JPRB, 0.76164E+00_JPRB, 0.75355E+00_JPRB, 0.75031E+00_JPRB, 0.74294E+00_JPRB ,&
&  0.74559E+00_JPRB, 0.66330E+00_JPRB, 0.76869E+00_JPRB, 0.89161E+00_JPRB, 0.96602E+00_JPRB, 0.10640E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.11658E+01_JPRB, 0.11072E+01_JPRB, 0.10569E+01_JPRB, 0.10089E+01_JPRB, 0.97069E+00_JPRB, 0.93982E+00_JPRB ,&
&  0.91894E+00_JPRB, 0.90444E+00_JPRB, 0.88997E+00_JPRB, 0.87224E+00_JPRB, 0.85363E+00_JPRB, 0.84167E+00_JPRB, 0.83307E+00_JPRB ,&
&  0.82829E+00_JPRB, 0.82836E+00_JPRB, 0.82799E+00_JPRB, 0.83396E+00_JPRB, 0.84029E+00_JPRB, 0.86922E+00_JPRB, 0.95426E+00_JPRB ,&
&  0.10189E+01_JPRB, 0.10985E+01_JPRB, 0.12130E+01_JPRB, 0.12692E+01_JPRB, 0.12711E+01_JPRB, 0.12560E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.11334E+01_JPRB, 0.10982E+01_JPRB, 0.10641E+01_JPRB, 0.10391E+01_JPRB, 0.10255E+01_JPRB, 0.10280E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.10314E+01_JPRB, 0.10337E+01_JPRB, 0.10407E+01_JPRB, 0.10455E+01_JPRB, 0.10498E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.11352E+01_JPRB, 0.11434E+01_JPRB, 0.11340E+01_JPRB, 0.11348E+01_JPRB, 0.11544E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.12063E+01_JPRB, 0.12227E+01_JPRB, 0.11784E+01_JPRB, 0.12209E+01_JPRB, 0.11964E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11279E+01_JPRB, 0.10983E+01_JPRB, 0.10749E+01_JPRB, 0.93133E+00_JPRB, 0.97700E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.90993E+00_JPRB, 0.89602E+00_JPRB, 0.87643E+00_JPRB, 0.85860E+00_JPRB, 0.84638E+00_JPRB, 0.84121E+00_JPRB, 0.83913E+00_JPRB ,&
&  0.84258E+00_JPRB, 0.84593E+00_JPRB, 0.84608E+00_JPRB, 0.84248E+00_JPRB, 0.83598E+00_JPRB, 0.82515E+00_JPRB, 0.81520E+00_JPRB ,&
&  0.80015E+00_JPRB, 0.78444E+00_JPRB, 0.77816E+00_JPRB, 0.77803E+00_JPRB, 0.77603E+00_JPRB, 0.76808E+00_JPRB, 0.76186E+00_JPRB ,&
&  0.75836E+00_JPRB, 0.75941E+00_JPRB, 0.75205E+00_JPRB, 0.74614E+00_JPRB, 0.73883E+00_JPRB, 0.73878E+00_JPRB, 0.73743E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.61643E+00_JPRB, 0.71727E+00_JPRB, 0.79647E+00_JPRB, 0.83193E+00_JPRB, 0.90088E+00_JPRB, 0.96855E+00_JPRB ,&
&  0.10131E+01_JPRB, 0.10303E+01_JPRB, 0.10184E+01_JPRB, 0.99316E+00_JPRB, 0.95698E+00_JPRB, 0.92546E+00_JPRB, 0.89944E+00_JPRB ,&
&  0.87456E+00_JPRB, 0.85331E+00_JPRB, 0.83236E+00_JPRB, 0.81190E+00_JPRB, 0.79745E+00_JPRB, 0.78307E+00_JPRB, 0.77077E+00_JPRB ,&
&  0.76630E+00_JPRB, 0.76928E+00_JPRB, 0.76765E+00_JPRB, 0.76958E+00_JPRB, 0.77454E+00_JPRB, 0.78033E+00_JPRB, 0.81360E+00_JPRB ,&
&  0.84859E+00_JPRB, 0.88228E+00_JPRB, 0.91999E+00_JPRB, 0.94708E+00_JPRB, 0.95584E+00_JPRB, 0.96572E+00_JPRB, 0.96402E+00_JPRB ,&
&  0.95886E+00_JPRB, 0.94251E+00_JPRB, 0.92286E+00_JPRB, 0.90270E+00_JPRB, 0.88552E+00_JPRB, 0.87335E+00_JPRB, 0.87110E+00_JPRB ,&
&  0.87389E+00_JPRB, 0.86536E+00_JPRB, 0.85608E+00_JPRB, 0.85352E+00_JPRB, 0.85541E+00_JPRB, 0.86379E+00_JPRB, 0.88329E+00_JPRB ,&
&  0.91471E+00_JPRB, 0.95427E+00_JPRB, 0.98555E+00_JPRB, 0.99844E+00_JPRB, 0.10006E+01_JPRB, 0.10085E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.10525E+01_JPRB, 0.10641E+01_JPRB, 0.10910E+01_JPRB, 0.10911E+01_JPRB, 0.10754E+01_JPRB, 0.10513E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.99874E+00_JPRB, 0.82461E+00_JPRB, 0.88976E+00_JPRB, 0.90153E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.83801E+00_JPRB, 0.82390E+00_JPRB, 0.80466E+00_JPRB, 0.78920E+00_JPRB, 0.77690E+00_JPRB, 0.77169E+00_JPRB, 0.76854E+00_JPRB ,&
&  0.77236E+00_JPRB, 0.78038E+00_JPRB, 0.78888E+00_JPRB, 0.79451E+00_JPRB, 0.79892E+00_JPRB, 0.79925E+00_JPRB, 0.79497E+00_JPRB ,&
&  0.79123E+00_JPRB, 0.77964E+00_JPRB, 0.77009E+00_JPRB, 0.76949E+00_JPRB, 0.76644E+00_JPRB, 0.76233E+00_JPRB, 0.75612E+00_JPRB ,&
&  0.74949E+00_JPRB, 0.74298E+00_JPRB, 0.73057E+00_JPRB, 0.72363E+00_JPRB, 0.72103E+00_JPRB, 0.72286E+00_JPRB, 0.73126E+00_JPRB ,&
&  0.65394E+00_JPRB, 0.57618E+00_JPRB, 0.65284E+00_JPRB, 0.73666E+00_JPRB, 0.77554E+00_JPRB, 0.81869E+00_JPRB, 0.85646E+00_JPRB ,&
&  0.90722E+00_JPRB, 0.94017E+00_JPRB, 0.95981E+00_JPRB, 0.95691E+00_JPRB, 0.93824E+00_JPRB, 0.91322E+00_JPRB, 0.88572E+00_JPRB ,&
&  0.85116E+00_JPRB, 0.81144E+00_JPRB, 0.78404E+00_JPRB, 0.76863E+00_JPRB, 0.75612E+00_JPRB, 0.74334E+00_JPRB, 0.73382E+00_JPRB ,&
&  0.72822E+00_JPRB, 0.72429E+00_JPRB, 0.72075E+00_JPRB, 0.71856E+00_JPRB, 0.72031E+00_JPRB, 0.72358E+00_JPRB, 0.73170E+00_JPRB ,&
&  0.74683E+00_JPRB, 0.76246E+00_JPRB, 0.77641E+00_JPRB, 0.78479E+00_JPRB, 0.79237E+00_JPRB, 0.80615E+00_JPRB, 0.81638E+00_JPRB ,&
&  0.82531E+00_JPRB, 0.82563E+00_JPRB, 0.82060E+00_JPRB, 0.81180E+00_JPRB, 0.79898E+00_JPRB, 0.78608E+00_JPRB, 0.77603E+00_JPRB ,&
&  0.76964E+00_JPRB, 0.76057E+00_JPRB, 0.75662E+00_JPRB, 0.75834E+00_JPRB, 0.76313E+00_JPRB, 0.76966E+00_JPRB, 0.77785E+00_JPRB ,&
&  0.79333E+00_JPRB, 0.81944E+00_JPRB, 0.84215E+00_JPRB, 0.85705E+00_JPRB, 0.86655E+00_JPRB, 0.87359E+00_JPRB, 0.88442E+00_JPRB ,&
&  0.89622E+00_JPRB, 0.90900E+00_JPRB, 0.92530E+00_JPRB, 0.94302E+00_JPRB, 0.94829E+00_JPRB, 0.94335E+00_JPRB, 0.93058E+00_JPRB ,&
&  0.91081E+00_JPRB, 0.88521E+00_JPRB, 0.87293E+00_JPRB, 0.85801E+00_JPRB, 0.84680E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.73658E+00_JPRB, 0.72778E+00_JPRB, 0.71492E+00_JPRB, 0.70689E+00_JPRB, 0.70307E+00_JPRB, 0.70513E+00_JPRB, 0.70873E+00_JPRB ,&
&  0.71278E+00_JPRB, 0.71414E+00_JPRB, 0.72193E+00_JPRB, 0.73074E+00_JPRB, 0.73688E+00_JPRB, 0.74148E+00_JPRB, 0.74130E+00_JPRB ,&
&  0.74139E+00_JPRB, 0.74159E+00_JPRB, 0.73737E+00_JPRB, 0.73326E+00_JPRB, 0.72753E+00_JPRB, 0.72801E+00_JPRB, 0.72722E+00_JPRB ,&
&  0.72416E+00_JPRB, 0.71838E+00_JPRB, 0.70903E+00_JPRB, 0.69780E+00_JPRB, 0.69203E+00_JPRB, 0.69000E+00_JPRB, 0.68676E+00_JPRB ,&
&  0.69236E+00_JPRB, 0.61141E+00_JPRB, 0.65380E+00_JPRB, 0.68210E+00_JPRB, 0.71880E+00_JPRB, 0.75324E+00_JPRB, 0.78691E+00_JPRB ,&
&  0.82118E+00_JPRB, 0.84964E+00_JPRB, 0.86186E+00_JPRB, 0.86396E+00_JPRB, 0.84825E+00_JPRB, 0.81957E+00_JPRB, 0.78708E+00_JPRB ,&
&  0.75977E+00_JPRB, 0.73935E+00_JPRB, 0.71773E+00_JPRB, 0.70444E+00_JPRB, 0.69360E+00_JPRB, 0.67999E+00_JPRB, 0.66760E+00_JPRB ,&
&  0.66043E+00_JPRB, 0.65545E+00_JPRB, 0.65362E+00_JPRB, 0.65455E+00_JPRB, 0.65707E+00_JPRB, 0.65920E+00_JPRB, 0.65944E+00_JPRB ,&
&  0.66183E+00_JPRB, 0.66728E+00_JPRB, 0.67286E+00_JPRB, 0.67960E+00_JPRB, 0.68795E+00_JPRB, 0.69856E+00_JPRB, 0.70269E+00_JPRB ,&
&  0.69949E+00_JPRB, 0.70141E+00_JPRB, 0.70314E+00_JPRB, 0.70123E+00_JPRB, 0.69835E+00_JPRB, 0.69465E+00_JPRB, 0.68893E+00_JPRB ,&
&  0.68413E+00_JPRB, 0.68395E+00_JPRB, 0.68498E+00_JPRB, 0.69004E+00_JPRB, 0.69397E+00_JPRB, 0.70345E+00_JPRB, 0.71270E+00_JPRB ,&
&  0.72337E+00_JPRB, 0.74058E+00_JPRB, 0.75998E+00_JPRB, 0.77581E+00_JPRB, 0.78589E+00_JPRB, 0.79255E+00_JPRB, 0.79877E+00_JPRB ,&
&  0.80428E+00_JPRB, 0.81010E+00_JPRB, 0.81714E+00_JPRB, 0.82139E+00_JPRB, 0.81957E+00_JPRB, 0.81441E+00_JPRB, 0.80787E+00_JPRB ,&
&  0.79325E+00_JPRB, 0.77360E+00_JPRB, 0.76155E+00_JPRB, 0.75076E+00_JPRB, 0.74186E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.66929E+00_JPRB, 0.66312E+00_JPRB, 0.65712E+00_JPRB, 0.65578E+00_JPRB, 0.65441E+00_JPRB, 0.65585E+00_JPRB, 0.65802E+00_JPRB ,&
&  0.66441E+00_JPRB, 0.66866E+00_JPRB, 0.67825E+00_JPRB, 0.68643E+00_JPRB, 0.69088E+00_JPRB, 0.69601E+00_JPRB, 0.69934E+00_JPRB ,&
&  0.70096E+00_JPRB, 0.70302E+00_JPRB, 0.70434E+00_JPRB, 0.70907E+00_JPRB, 0.70998E+00_JPRB, 0.70949E+00_JPRB, 0.70834E+00_JPRB ,&
&  0.70552E+00_JPRB, 0.70170E+00_JPRB, 0.69437E+00_JPRB, 0.68497E+00_JPRB, 0.67371E+00_JPRB, 0.66789E+00_JPRB, 0.66047E+00_JPRB ,&
&  0.66050E+00_JPRB, 0.63461E+00_JPRB, 0.64742E+00_JPRB, 0.64580E+00_JPRB, 0.66626E+00_JPRB, 0.69818E+00_JPRB, 0.73591E+00_JPRB ,&
&  0.75997E+00_JPRB, 0.77071E+00_JPRB, 0.76929E+00_JPRB, 0.76301E+00_JPRB, 0.74535E+00_JPRB, 0.72797E+00_JPRB, 0.70855E+00_JPRB ,&
&  0.68686E+00_JPRB, 0.67429E+00_JPRB, 0.65943E+00_JPRB, 0.64845E+00_JPRB, 0.64363E+00_JPRB, 0.63996E+00_JPRB, 0.63372E+00_JPRB ,&
&  0.62421E+00_JPRB, 0.61582E+00_JPRB, 0.61521E+00_JPRB, 0.61809E+00_JPRB, 0.62153E+00_JPRB, 0.62370E+00_JPRB, 0.62240E+00_JPRB ,&
&  0.61633E+00_JPRB, 0.61261E+00_JPRB, 0.61421E+00_JPRB, 0.61771E+00_JPRB, 0.61788E+00_JPRB, 0.61429E+00_JPRB, 0.61003E+00_JPRB ,&
&  0.61015E+00_JPRB, 0.61324E+00_JPRB, 0.61642E+00_JPRB, 0.62066E+00_JPRB, 0.62321E+00_JPRB, 0.62611E+00_JPRB, 0.62704E+00_JPRB ,&
&  0.63103E+00_JPRB, 0.63496E+00_JPRB, 0.64346E+00_JPRB, 0.65169E+00_JPRB, 0.65322E+00_JPRB, 0.66179E+00_JPRB, 0.67131E+00_JPRB ,&
&  0.68200E+00_JPRB, 0.69515E+00_JPRB, 0.70720E+00_JPRB, 0.71759E+00_JPRB, 0.72783E+00_JPRB, 0.73392E+00_JPRB, 0.74089E+00_JPRB ,&
&  0.74936E+00_JPRB, 0.75168E+00_JPRB, 0.75790E+00_JPRB, 0.75848E+00_JPRB, 0.75391E+00_JPRB, 0.74582E+00_JPRB, 0.73341E+00_JPRB ,&
&  0.71815E+00_JPRB, 0.69846E+00_JPRB, 0.68614E+00_JPRB, 0.67787E+00_JPRB, 0.67266E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.59412E+00_JPRB, 0.59120E+00_JPRB, 0.59084E+00_JPRB, 0.59360E+00_JPRB, 0.59696E+00_JPRB, 0.59943E+00_JPRB, 0.60480E+00_JPRB ,&
&  0.61455E+00_JPRB, 0.62240E+00_JPRB, 0.63221E+00_JPRB, 0.64245E+00_JPRB, 0.65044E+00_JPRB, 0.65588E+00_JPRB, 0.65889E+00_JPRB ,&
&  0.65755E+00_JPRB, 0.65541E+00_JPRB, 0.65905E+00_JPRB, 0.66422E+00_JPRB, 0.66586E+00_JPRB, 0.66555E+00_JPRB, 0.66559E+00_JPRB ,&
&  0.66476E+00_JPRB, 0.66174E+00_JPRB, 0.65750E+00_JPRB, 0.65136E+00_JPRB, 0.64433E+00_JPRB, 0.64006E+00_JPRB, 0.63632E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.62949E+00_JPRB, 0.57849E+00_JPRB, 0.59278E+00_JPRB, 0.60476E+00_JPRB, 0.62682E+00_JPRB, 0.64928E+00_JPRB ,&
&  0.66459E+00_JPRB, 0.67279E+00_JPRB, 0.67616E+00_JPRB, 0.67301E+00_JPRB, 0.65935E+00_JPRB, 0.65145E+00_JPRB, 0.64303E+00_JPRB ,&
&  0.63199E+00_JPRB, 0.62405E+00_JPRB, 0.61028E+00_JPRB, 0.59534E+00_JPRB, 0.59026E+00_JPRB, 0.59073E+00_JPRB, 0.59041E+00_JPRB ,&
&  0.58561E+00_JPRB, 0.58085E+00_JPRB, 0.58131E+00_JPRB, 0.58177E+00_JPRB, 0.58126E+00_JPRB, 0.57588E+00_JPRB, 0.56924E+00_JPRB ,&
&  0.55894E+00_JPRB, 0.54912E+00_JPRB, 0.54258E+00_JPRB, 0.53847E+00_JPRB, 0.53546E+00_JPRB, 0.53211E+00_JPRB, 0.52855E+00_JPRB ,&
&  0.53397E+00_JPRB, 0.54726E+00_JPRB, 0.55945E+00_JPRB, 0.56905E+00_JPRB, 0.57526E+00_JPRB, 0.57911E+00_JPRB, 0.58420E+00_JPRB ,&
&  0.59017E+00_JPRB, 0.60064E+00_JPRB, 0.60834E+00_JPRB, 0.61517E+00_JPRB, 0.61995E+00_JPRB, 0.62450E+00_JPRB, 0.62990E+00_JPRB ,&
&  0.63505E+00_JPRB, 0.64380E+00_JPRB, 0.65141E+00_JPRB, 0.66036E+00_JPRB, 0.66651E+00_JPRB, 0.67348E+00_JPRB, 0.68022E+00_JPRB ,&
&  0.68717E+00_JPRB, 0.69299E+00_JPRB, 0.69523E+00_JPRB, 0.69313E+00_JPRB, 0.68555E+00_JPRB, 0.67580E+00_JPRB, 0.66515E+00_JPRB ,&
&  0.65130E+00_JPRB, 0.63477E+00_JPRB, 0.61814E+00_JPRB, 0.60714E+00_JPRB, 0.59784E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.53744E+00_JPRB, 0.53743E+00_JPRB, 0.54153E+00_JPRB, 0.54556E+00_JPRB, 0.54865E+00_JPRB, 0.55334E+00_JPRB, 0.56293E+00_JPRB ,&
&  0.57268E+00_JPRB, 0.58213E+00_JPRB, 0.58862E+00_JPRB, 0.59649E+00_JPRB, 0.60774E+00_JPRB, 0.61550E+00_JPRB, 0.62169E+00_JPRB ,&
&  0.62446E+00_JPRB, 0.62524E+00_JPRB, 0.62682E+00_JPRB, 0.63036E+00_JPRB, 0.63018E+00_JPRB, 0.62669E+00_JPRB, 0.62547E+00_JPRB ,&
&  0.62529E+00_JPRB, 0.62330E+00_JPRB, 0.62254E+00_JPRB, 0.62274E+00_JPRB, 0.61922E+00_JPRB, 0.61407E+00_JPRB, 0.61224E+00_JPRB ,&
&  0.61090E+00_JPRB, 0.59647E+00_JPRB, 0.58801E+00_JPRB, 0.58028E+00_JPRB, 0.58318E+00_JPRB, 0.59398E+00_JPRB, 0.60614E+00_JPRB ,&
&  0.61342E+00_JPRB, 0.61826E+00_JPRB, 0.61815E+00_JPRB, 0.61727E+00_JPRB, 0.61239E+00_JPRB, 0.60103E+00_JPRB, 0.59186E+00_JPRB ,&
&  0.58581E+00_JPRB, 0.57659E+00_JPRB, 0.55356E+00_JPRB, 0.53974E+00_JPRB, 0.53584E+00_JPRB, 0.53868E+00_JPRB, 0.54328E+00_JPRB ,&
&  0.54496E+00_JPRB, 0.54189E+00_JPRB, 0.53512E+00_JPRB, 0.52753E+00_JPRB, 0.51762E+00_JPRB, 0.50640E+00_JPRB, 0.49662E+00_JPRB ,&
&  0.49131E+00_JPRB, 0.48756E+00_JPRB, 0.48467E+00_JPRB, 0.48778E+00_JPRB, 0.48736E+00_JPRB, 0.48391E+00_JPRB, 0.48691E+00_JPRB ,&
&  0.50008E+00_JPRB, 0.51677E+00_JPRB, 0.53222E+00_JPRB, 0.54065E+00_JPRB, 0.54423E+00_JPRB, 0.54826E+00_JPRB, 0.55266E+00_JPRB ,&
&  0.55843E+00_JPRB, 0.56375E+00_JPRB, 0.56910E+00_JPRB, 0.57655E+00_JPRB, 0.58565E+00_JPRB, 0.59370E+00_JPRB, 0.59713E+00_JPRB ,&
&  0.60011E+00_JPRB, 0.60255E+00_JPRB, 0.60523E+00_JPRB, 0.61219E+00_JPRB, 0.62143E+00_JPRB, 0.62800E+00_JPRB, 0.63282E+00_JPRB ,&
&  0.63742E+00_JPRB, 0.63939E+00_JPRB, 0.63767E+00_JPRB, 0.63297E+00_JPRB, 0.62698E+00_JPRB, 0.61571E+00_JPRB, 0.60149E+00_JPRB ,&
&  0.58505E+00_JPRB, 0.56960E+00_JPRB, 0.55777E+00_JPRB, 0.54752E+00_JPRB, 0.54015E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.49439E+00_JPRB, 0.49332E+00_JPRB, 0.49558E+00_JPRB, 0.50143E+00_JPRB, 0.50543E+00_JPRB, 0.50913E+00_JPRB, 0.51695E+00_JPRB ,&
&  0.52426E+00_JPRB, 0.53090E+00_JPRB, 0.53857E+00_JPRB, 0.54666E+00_JPRB, 0.55285E+00_JPRB, 0.56195E+00_JPRB, 0.57078E+00_JPRB ,&
&  0.57660E+00_JPRB, 0.57913E+00_JPRB, 0.58081E+00_JPRB, 0.58228E+00_JPRB, 0.58244E+00_JPRB, 0.58018E+00_JPRB, 0.57987E+00_JPRB ,&
&  0.57803E+00_JPRB, 0.57587E+00_JPRB, 0.57780E+00_JPRB, 0.57786E+00_JPRB, 0.57895E+00_JPRB, 0.58087E+00_JPRB, 0.57991E+00_JPRB ,&
&  0.57651E+00_JPRB, 0.56230E+00_JPRB, 0.55664E+00_JPRB, 0.55301E+00_JPRB, 0.55672E+00_JPRB, 0.55730E+00_JPRB, 0.55542E+00_JPRB ,&
&  0.54850E+00_JPRB, 0.54412E+00_JPRB, 0.54578E+00_JPRB, 0.54796E+00_JPRB, 0.54197E+00_JPRB, 0.52879E+00_JPRB, 0.51889E+00_JPRB ,&
&  0.51141E+00_JPRB, 0.49943E+00_JPRB, 0.48128E+00_JPRB, 0.47000E+00_JPRB, 0.46504E+00_JPRB, 0.46018E+00_JPRB, 0.45608E+00_JPRB ,&
&  0.45403E+00_JPRB, 0.45326E+00_JPRB, 0.45449E+00_JPRB, 0.45370E+00_JPRB, 0.45017E+00_JPRB, 0.44603E+00_JPRB, 0.44464E+00_JPRB ,&
&  0.44776E+00_JPRB, 0.45072E+00_JPRB, 0.45470E+00_JPRB, 0.45762E+00_JPRB, 0.45741E+00_JPRB, 0.45806E+00_JPRB, 0.46236E+00_JPRB ,&
&  0.46976E+00_JPRB, 0.47719E+00_JPRB, 0.48366E+00_JPRB, 0.48619E+00_JPRB, 0.49006E+00_JPRB, 0.49774E+00_JPRB, 0.49876E+00_JPRB ,&
&  0.49710E+00_JPRB, 0.49847E+00_JPRB, 0.50244E+00_JPRB, 0.51086E+00_JPRB, 0.52381E+00_JPRB, 0.53248E+00_JPRB, 0.53404E+00_JPRB ,&
&  0.53437E+00_JPRB, 0.53695E+00_JPRB, 0.54163E+00_JPRB, 0.55245E+00_JPRB, 0.56385E+00_JPRB, 0.57188E+00_JPRB, 0.57501E+00_JPRB ,&
&  0.57155E+00_JPRB, 0.56425E+00_JPRB, 0.55159E+00_JPRB, 0.53697E+00_JPRB, 0.52418E+00_JPRB, 0.51427E+00_JPRB, 0.50609E+00_JPRB ,&
&  0.49803E+00_JPRB, 0.49421E+00_JPRB, 0.49347E+00_JPRB, 0.49489E+00_JPRB, 0.49518E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.46040E+00_JPRB, 0.46180E+00_JPRB, 0.46382E+00_JPRB, 0.46660E+00_JPRB, 0.46952E+00_JPRB, 0.47120E+00_JPRB, 0.47476E+00_JPRB ,&
&  0.47698E+00_JPRB, 0.48380E+00_JPRB, 0.49487E+00_JPRB, 0.50579E+00_JPRB, 0.51359E+00_JPRB, 0.52013E+00_JPRB, 0.52673E+00_JPRB ,&
&  0.53223E+00_JPRB, 0.53945E+00_JPRB, 0.54057E+00_JPRB, 0.53839E+00_JPRB, 0.53595E+00_JPRB, 0.53414E+00_JPRB, 0.53247E+00_JPRB ,&
&  0.52996E+00_JPRB, 0.52976E+00_JPRB, 0.53338E+00_JPRB, 0.53740E+00_JPRB, 0.54141E+00_JPRB, 0.54211E+00_JPRB, 0.54011E+00_JPRB ,&
&  0.54283E+00_JPRB, 0.53231E+00_JPRB, 0.52517E+00_JPRB, 0.51712E+00_JPRB, 0.51010E+00_JPRB, 0.49822E+00_JPRB, 0.48935E+00_JPRB ,&
&  0.47877E+00_JPRB, 0.46934E+00_JPRB, 0.46521E+00_JPRB, 0.46089E+00_JPRB, 0.45095E+00_JPRB, 0.43969E+00_JPRB, 0.43167E+00_JPRB ,&
&  0.42644E+00_JPRB, 0.42109E+00_JPRB, 0.41443E+00_JPRB, 0.40876E+00_JPRB, 0.40411E+00_JPRB, 0.40244E+00_JPRB, 0.40406E+00_JPRB ,&
&  0.40844E+00_JPRB, 0.41286E+00_JPRB, 0.41574E+00_JPRB, 0.41670E+00_JPRB, 0.41523E+00_JPRB, 0.41277E+00_JPRB, 0.41151E+00_JPRB ,&
&  0.41260E+00_JPRB, 0.41471E+00_JPRB, 0.41684E+00_JPRB, 0.42201E+00_JPRB, 0.42313E+00_JPRB, 0.42170E+00_JPRB, 0.41645E+00_JPRB ,&
&  0.41480E+00_JPRB, 0.41439E+00_JPRB, 0.41671E+00_JPRB, 0.42225E+00_JPRB, 0.43111E+00_JPRB, 0.43972E+00_JPRB, 0.44572E+00_JPRB ,&
&  0.44840E+00_JPRB, 0.44970E+00_JPRB, 0.45251E+00_JPRB, 0.45627E+00_JPRB, 0.45813E+00_JPRB, 0.45792E+00_JPRB, 0.45680E+00_JPRB ,&
&  0.45988E+00_JPRB, 0.46784E+00_JPRB, 0.47644E+00_JPRB, 0.48259E+00_JPRB, 0.48600E+00_JPRB, 0.48553E+00_JPRB, 0.47884E+00_JPRB ,&
&  0.46616E+00_JPRB, 0.45143E+00_JPRB, 0.43719E+00_JPRB, 0.42532E+00_JPRB, 0.41718E+00_JPRB, 0.41326E+00_JPRB, 0.41607E+00_JPRB ,&
&  0.42375E+00_JPRB, 0.43363E+00_JPRB, 0.44252E+00_JPRB, 0.44890E+00_JPRB, 0.45584E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.42820E+00_JPRB, 0.42810E+00_JPRB, 0.42971E+00_JPRB, 0.42873E+00_JPRB, 0.42597E+00_JPRB, 0.42353E+00_JPRB, 0.42650E+00_JPRB ,&
&  0.43307E+00_JPRB, 0.44489E+00_JPRB, 0.46079E+00_JPRB, 0.47481E+00_JPRB, 0.48315E+00_JPRB, 0.48810E+00_JPRB, 0.49570E+00_JPRB ,&
&  0.50291E+00_JPRB, 0.51096E+00_JPRB, 0.51128E+00_JPRB, 0.50260E+00_JPRB, 0.49530E+00_JPRB, 0.49148E+00_JPRB, 0.49108E+00_JPRB ,&
&  0.49039E+00_JPRB, 0.49201E+00_JPRB, 0.49496E+00_JPRB, 0.49801E+00_JPRB, 0.49698E+00_JPRB, 0.49471E+00_JPRB, 0.49774E+00_JPRB ,&
&  0.49500E+00_JPRB, 0.48436E+00_JPRB, 0.47328E+00_JPRB, 0.44788E+00_JPRB, 0.44644E+00_JPRB, 0.43545E+00_JPRB, 0.42724E+00_JPRB ,&
&  0.41622E+00_JPRB, 0.40248E+00_JPRB, 0.39244E+00_JPRB, 0.38604E+00_JPRB, 0.38308E+00_JPRB, 0.38061E+00_JPRB, 0.37866E+00_JPRB ,&
&  0.37738E+00_JPRB, 0.37729E+00_JPRB, 0.37896E+00_JPRB, 0.37662E+00_JPRB, 0.37543E+00_JPRB, 0.37625E+00_JPRB, 0.37815E+00_JPRB ,&
&  0.37857E+00_JPRB, 0.37915E+00_JPRB, 0.37975E+00_JPRB, 0.37939E+00_JPRB, 0.37795E+00_JPRB, 0.37556E+00_JPRB, 0.37338E+00_JPRB ,&
&  0.37274E+00_JPRB, 0.37439E+00_JPRB, 0.37530E+00_JPRB, 0.37177E+00_JPRB, 0.36430E+00_JPRB, 0.35454E+00_JPRB, 0.34830E+00_JPRB ,&
&  0.34613E+00_JPRB, 0.34899E+00_JPRB, 0.35763E+00_JPRB, 0.36937E+00_JPRB, 0.37996E+00_JPRB, 0.38636E+00_JPRB, 0.39024E+00_JPRB ,&
&  0.39438E+00_JPRB, 0.39172E+00_JPRB, 0.38741E+00_JPRB, 0.38388E+00_JPRB, 0.38079E+00_JPRB, 0.37659E+00_JPRB, 0.37652E+00_JPRB ,&
&  0.37958E+00_JPRB, 0.38281E+00_JPRB, 0.38409E+00_JPRB, 0.38337E+00_JPRB, 0.38150E+00_JPRB, 0.37735E+00_JPRB, 0.37150E+00_JPRB ,&
&  0.36407E+00_JPRB, 0.35537E+00_JPRB, 0.35382E+00_JPRB, 0.35488E+00_JPRB, 0.35879E+00_JPRB, 0.36656E+00_JPRB, 0.37726E+00_JPRB ,&
&  0.38845E+00_JPRB, 0.39744E+00_JPRB, 0.40726E+00_JPRB, 0.41638E+00_JPRB, 0.42461E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.40757E+00_JPRB, 0.40899E+00_JPRB, 0.40638E+00_JPRB, 0.40368E+00_JPRB, 0.40350E+00_JPRB, 0.40491E+00_JPRB, 0.41023E+00_JPRB ,&
&  0.41991E+00_JPRB, 0.43293E+00_JPRB, 0.44548E+00_JPRB, 0.45580E+00_JPRB, 0.46492E+00_JPRB, 0.46887E+00_JPRB, 0.47470E+00_JPRB ,&
&  0.48107E+00_JPRB, 0.48692E+00_JPRB, 0.48765E+00_JPRB, 0.47730E+00_JPRB, 0.46732E+00_JPRB, 0.46201E+00_JPRB, 0.46197E+00_JPRB ,&
&  0.46360E+00_JPRB, 0.46597E+00_JPRB, 0.47182E+00_JPRB, 0.47373E+00_JPRB, 0.46743E+00_JPRB, 0.46553E+00_JPRB, 0.46295E+00_JPRB ,&
&  0.45677E+00_JPRB, 0.44399E+00_JPRB, 0.41094E+00_JPRB, 0.40430E+00_JPRB, 0.39549E+00_JPRB, 0.38615E+00_JPRB, 0.37719E+00_JPRB ,&
&  0.36683E+00_JPRB, 0.35624E+00_JPRB, 0.34904E+00_JPRB, 0.34470E+00_JPRB, 0.34518E+00_JPRB, 0.34641E+00_JPRB, 0.34600E+00_JPRB ,&
&  0.34814E+00_JPRB, 0.35049E+00_JPRB, 0.35272E+00_JPRB, 0.35319E+00_JPRB, 0.35300E+00_JPRB, 0.35240E+00_JPRB, 0.34852E+00_JPRB ,&
&  0.34503E+00_JPRB, 0.34346E+00_JPRB, 0.34233E+00_JPRB, 0.34140E+00_JPRB, 0.34161E+00_JPRB, 0.34233E+00_JPRB, 0.34264E+00_JPRB ,&
&  0.34034E+00_JPRB, 0.33690E+00_JPRB, 0.33303E+00_JPRB, 0.32588E+00_JPRB, 0.31257E+00_JPRB, 0.29843E+00_JPRB, 0.28911E+00_JPRB ,&
&  0.28807E+00_JPRB, 0.29873E+00_JPRB, 0.31943E+00_JPRB, 0.33070E+00_JPRB, 0.33830E+00_JPRB, 0.34194E+00_JPRB, 0.34572E+00_JPRB ,&
&  0.33888E+00_JPRB, 0.31633E+00_JPRB, 0.30336E+00_JPRB, 0.29575E+00_JPRB, 0.29054E+00_JPRB, 0.28701E+00_JPRB, 0.28663E+00_JPRB ,&
&  0.29005E+00_JPRB, 0.29590E+00_JPRB, 0.30239E+00_JPRB, 0.30553E+00_JPRB, 0.30467E+00_JPRB, 0.30170E+00_JPRB, 0.29832E+00_JPRB ,&
&  0.29622E+00_JPRB, 0.30148E+00_JPRB, 0.31637E+00_JPRB, 0.33624E+00_JPRB, 0.34474E+00_JPRB, 0.35145E+00_JPRB, 0.35766E+00_JPRB ,&
&  0.36441E+00_JPRB, 0.37452E+00_JPRB, 0.38645E+00_JPRB, 0.39572E+00_JPRB, 0.40383E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.37878E+00_JPRB, 0.38262E+00_JPRB, 0.38497E+00_JPRB, 0.38831E+00_JPRB, 0.39598E+00_JPRB, 0.40646E+00_JPRB, 0.41846E+00_JPRB ,&
&  0.43003E+00_JPRB, 0.43981E+00_JPRB, 0.44483E+00_JPRB, 0.45013E+00_JPRB, 0.45506E+00_JPRB, 0.45731E+00_JPRB, 0.46222E+00_JPRB ,&
&  0.46280E+00_JPRB, 0.45916E+00_JPRB, 0.45289E+00_JPRB, 0.44476E+00_JPRB, 0.44040E+00_JPRB, 0.44149E+00_JPRB, 0.44411E+00_JPRB ,&
&  0.44564E+00_JPRB, 0.44679E+00_JPRB, 0.44658E+00_JPRB, 0.44281E+00_JPRB, 0.43930E+00_JPRB, 0.43716E+00_JPRB, 0.43261E+00_JPRB ,&
&  0.42156E+00_JPRB, 0.37122E+00_JPRB, 0.33395E+00_JPRB, 0.33059E+00_JPRB, 0.34555E+00_JPRB, 0.34741E+00_JPRB, 0.34280E+00_JPRB ,&
&  0.33735E+00_JPRB, 0.33236E+00_JPRB, 0.32942E+00_JPRB, 0.32717E+00_JPRB, 0.32438E+00_JPRB, 0.32172E+00_JPRB, 0.32201E+00_JPRB ,&
&  0.32313E+00_JPRB, 0.32091E+00_JPRB, 0.31788E+00_JPRB, 0.31145E+00_JPRB, 0.30694E+00_JPRB, 0.30566E+00_JPRB, 0.30466E+00_JPRB ,&
&  0.30057E+00_JPRB, 0.29638E+00_JPRB, 0.29662E+00_JPRB, 0.29818E+00_JPRB, 0.30050E+00_JPRB, 0.30204E+00_JPRB, 0.29914E+00_JPRB ,&
&  0.29449E+00_JPRB, 0.28767E+00_JPRB, 0.27969E+00_JPRB, 0.26993E+00_JPRB, 0.26080E+00_JPRB, 0.25391E+00_JPRB, 0.24826E+00_JPRB ,&
&  0.24796E+00_JPRB, 0.25501E+00_JPRB, 0.26957E+00_JPRB, 0.28506E+00_JPRB, 0.29937E+00_JPRB, 0.29182E+00_JPRB, 0.28209E+00_JPRB ,&
&  0.27088E+00_JPRB, 0.26122E+00_JPRB, 0.25424E+00_JPRB, 0.25113E+00_JPRB, 0.24989E+00_JPRB, 0.24977E+00_JPRB, 0.25102E+00_JPRB ,&
&  0.25336E+00_JPRB, 0.25618E+00_JPRB, 0.25844E+00_JPRB, 0.25948E+00_JPRB, 0.25988E+00_JPRB, 0.26008E+00_JPRB, 0.25958E+00_JPRB ,&
&  0.25891E+00_JPRB, 0.26162E+00_JPRB, 0.26877E+00_JPRB, 0.27760E+00_JPRB, 0.28585E+00_JPRB, 0.29393E+00_JPRB, 0.30704E+00_JPRB ,&
&  0.32311E+00_JPRB, 0.34239E+00_JPRB, 0.35592E+00_JPRB, 0.36646E+00_JPRB, 0.37510E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.36885E+00_JPRB, 0.37796E+00_JPRB, 0.38960E+00_JPRB, 0.39992E+00_JPRB, 0.40930E+00_JPRB, 0.41802E+00_JPRB, 0.42600E+00_JPRB ,&
&  0.43066E+00_JPRB, 0.43336E+00_JPRB, 0.43448E+00_JPRB, 0.43818E+00_JPRB, 0.43497E+00_JPRB, 0.42747E+00_JPRB, 0.42171E+00_JPRB ,&
&  0.41683E+00_JPRB, 0.41307E+00_JPRB, 0.40768E+00_JPRB, 0.40480E+00_JPRB, 0.40443E+00_JPRB, 0.40433E+00_JPRB, 0.39849E+00_JPRB ,&
&  0.38752E+00_JPRB, 0.38580E+00_JPRB, 0.39010E+00_JPRB, 0.38416E+00_JPRB, 0.37811E+00_JPRB, 0.37216E+00_JPRB, 0.36596E+00_JPRB ,&
&  0.34909E+00_JPRB, 0.32839E+00_JPRB, 0.30204E+00_JPRB, 0.29956E+00_JPRB, 0.31525E+00_JPRB, 0.30791E+00_JPRB, 0.31179E+00_JPRB ,&
&  0.31285E+00_JPRB, 0.31287E+00_JPRB, 0.31140E+00_JPRB, 0.30964E+00_JPRB, 0.30612E+00_JPRB, 0.30510E+00_JPRB, 0.30319E+00_JPRB ,&
&  0.29659E+00_JPRB, 0.29134E+00_JPRB, 0.28280E+00_JPRB, 0.26435E+00_JPRB, 0.26021E+00_JPRB, 0.25962E+00_JPRB, 0.25537E+00_JPRB ,&
&  0.24786E+00_JPRB, 0.24318E+00_JPRB, 0.24184E+00_JPRB, 0.23721E+00_JPRB, 0.23947E+00_JPRB, 0.24184E+00_JPRB, 0.24171E+00_JPRB ,&
&  0.24160E+00_JPRB, 0.24110E+00_JPRB, 0.23878E+00_JPRB, 0.23481E+00_JPRB, 0.23159E+00_JPRB, 0.23073E+00_JPRB, 0.23202E+00_JPRB ,&
&  0.23472E+00_JPRB, 0.23923E+00_JPRB, 0.25084E+00_JPRB, 0.25970E+00_JPRB, 0.25829E+00_JPRB, 0.25442E+00_JPRB, 0.24509E+00_JPRB ,&
&  0.23598E+00_JPRB, 0.23270E+00_JPRB, 0.22976E+00_JPRB, 0.22808E+00_JPRB, 0.22827E+00_JPRB, 0.22973E+00_JPRB, 0.23036E+00_JPRB ,&
&  0.23153E+00_JPRB, 0.23281E+00_JPRB, 0.23371E+00_JPRB, 0.23413E+00_JPRB, 0.23487E+00_JPRB, 0.23574E+00_JPRB, 0.23748E+00_JPRB ,&
&  0.24005E+00_JPRB, 0.24459E+00_JPRB, 0.25049E+00_JPRB, 0.25688E+00_JPRB, 0.26162E+00_JPRB, 0.26405E+00_JPRB, 0.26899E+00_JPRB ,&
&  0.27521E+00_JPRB, 0.29144E+00_JPRB, 0.32062E+00_JPRB, 0.36438E+00_JPRB, 0.36733E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.36666E+00_JPRB, 0.37897E+00_JPRB, 0.39416E+00_JPRB, 0.40382E+00_JPRB, 0.40969E+00_JPRB, 0.41275E+00_JPRB, 0.41208E+00_JPRB ,&
&  0.40876E+00_JPRB, 0.40804E+00_JPRB, 0.41893E+00_JPRB, 0.40019E+00_JPRB, 0.38289E+00_JPRB, 0.35032E+00_JPRB, 0.33371E+00_JPRB ,&
&  0.32081E+00_JPRB, 0.31845E+00_JPRB, 0.31783E+00_JPRB, 0.32233E+00_JPRB, 0.32579E+00_JPRB, 0.32847E+00_JPRB, 0.32422E+00_JPRB ,&
&  0.31843E+00_JPRB, 0.30901E+00_JPRB, 0.30110E+00_JPRB, 0.29384E+00_JPRB, 0.29415E+00_JPRB, 0.29296E+00_JPRB, 0.29029E+00_JPRB ,&
&  0.28744E+00_JPRB, 0.28581E+00_JPRB, 0.28600E+00_JPRB, 0.28692E+00_JPRB, 0.29847E+00_JPRB, 0.29709E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.29781E+00_JPRB, 0.29777E+00_JPRB, 0.29672E+00_JPRB, 0.29493E+00_JPRB, 0.28993E+00_JPRB, 0.27425E+00_JPRB, 0.27266E+00_JPRB ,&
&  0.26440E+00_JPRB, 0.25403E+00_JPRB, 0.24376E+00_JPRB, 0.23663E+00_JPRB, 0.23229E+00_JPRB, 0.23080E+00_JPRB, 0.22612E+00_JPRB ,&
&  0.22117E+00_JPRB, 0.21514E+00_JPRB, 0.21251E+00_JPRB, 0.21262E+00_JPRB, 0.21343E+00_JPRB, 0.21408E+00_JPRB, 0.21460E+00_JPRB ,&
&  0.21606E+00_JPRB, 0.21674E+00_JPRB, 0.21718E+00_JPRB, 0.21805E+00_JPRB, 0.21890E+00_JPRB, 0.22009E+00_JPRB, 0.22206E+00_JPRB ,&
&  0.22554E+00_JPRB, 0.22917E+00_JPRB, 0.23263E+00_JPRB, 0.23600E+00_JPRB, 0.23502E+00_JPRB, 0.23101E+00_JPRB, 0.22586E+00_JPRB ,&
&  0.22200E+00_JPRB, 0.22059E+00_JPRB, 0.21925E+00_JPRB, 0.21897E+00_JPRB, 0.21897E+00_JPRB, 0.21930E+00_JPRB, 0.21987E+00_JPRB ,&
&  0.22075E+00_JPRB, 0.22149E+00_JPRB, 0.22239E+00_JPRB, 0.22357E+00_JPRB, 0.22517E+00_JPRB, 0.22662E+00_JPRB, 0.22865E+00_JPRB ,&
&  0.23167E+00_JPRB, 0.23503E+00_JPRB, 0.23976E+00_JPRB, 0.24409E+00_JPRB, 0.25042E+00_JPRB, 0.25894E+00_JPRB, 0.27283E+00_JPRB ,&
&  0.28737E+00_JPRB, 0.32055E+00_JPRB, 0.33520E+00_JPRB, 0.34826E+00_JPRB, 0.35645E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.36636E+00_JPRB, 0.37636E+00_JPRB, 0.39041E+00_JPRB, 0.39973E+00_JPRB, 0.40686E+00_JPRB, 0.40276E+00_JPRB, 0.39878E+00_JPRB ,&
&  0.39451E+00_JPRB, 0.38085E+00_JPRB, 0.36353E+00_JPRB, 0.33442E+00_JPRB, 0.30994E+00_JPRB, 0.29129E+00_JPRB, 0.28046E+00_JPRB ,&
&  0.27354E+00_JPRB, 0.27100E+00_JPRB, 0.26826E+00_JPRB, 0.26728E+00_JPRB, 0.26542E+00_JPRB, 0.26653E+00_JPRB, 0.26743E+00_JPRB ,&
&  0.26668E+00_JPRB, 0.26578E+00_JPRB, 0.26548E+00_JPRB, 0.26871E+00_JPRB, 0.27509E+00_JPRB, 0.28000E+00_JPRB, 0.28482E+00_JPRB ,&
&  0.28805E+00_JPRB, 0.28513E+00_JPRB, 0.28683E+00_JPRB, 0.28516E+00_JPRB, 0.28410E+00_JPRB, 0.28527E+00_JPRB, 0.28702E+00_JPRB ,&
&  0.28742E+00_JPRB, 0.28729E+00_JPRB, 0.28615E+00_JPRB, 0.28243E+00_JPRB, 0.26979E+00_JPRB, 0.26342E+00_JPRB, 0.25503E+00_JPRB ,&
&  0.24691E+00_JPRB, 0.23743E+00_JPRB, 0.22948E+00_JPRB, 0.22589E+00_JPRB, 0.22101E+00_JPRB, 0.21931E+00_JPRB, 0.21598E+00_JPRB ,&
&  0.21223E+00_JPRB, 0.20874E+00_JPRB, 0.20669E+00_JPRB, 0.20570E+00_JPRB, 0.20539E+00_JPRB, 0.20398E+00_JPRB, 0.20354E+00_JPRB ,&
&  0.20399E+00_JPRB, 0.20441E+00_JPRB, 0.20553E+00_JPRB, 0.20650E+00_JPRB, 0.20750E+00_JPRB, 0.20832E+00_JPRB, 0.20927E+00_JPRB ,&
&  0.21097E+00_JPRB, 0.21316E+00_JPRB, 0.21413E+00_JPRB, 0.21505E+00_JPRB, 0.21470E+00_JPRB, 0.21398E+00_JPRB, 0.21297E+00_JPRB ,&
&  0.21212E+00_JPRB, 0.21181E+00_JPRB, 0.21179E+00_JPRB, 0.21166E+00_JPRB, 0.21210E+00_JPRB, 0.21298E+00_JPRB, 0.21382E+00_JPRB ,&
&  0.21542E+00_JPRB, 0.21683E+00_JPRB, 0.21847E+00_JPRB, 0.22054E+00_JPRB, 0.22280E+00_JPRB, 0.22555E+00_JPRB, 0.22861E+00_JPRB ,&
&  0.23117E+00_JPRB, 0.23460E+00_JPRB, 0.23753E+00_JPRB, 0.23990E+00_JPRB, 0.24625E+00_JPRB, 0.25387E+00_JPRB, 0.26935E+00_JPRB ,&
&  0.28606E+00_JPRB, 0.32431E+00_JPRB, 0.36518E+00_JPRB, 0.35229E+00_JPRB, 0.35803E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.33889E+00_JPRB, 0.35016E+00_JPRB, 0.35884E+00_JPRB, 0.36430E+00_JPRB, 0.36959E+00_JPRB, 0.37145E+00_JPRB, 0.37885E+00_JPRB ,&
&  0.37341E+00_JPRB, 0.35566E+00_JPRB, 0.34454E+00_JPRB, 0.32791E+00_JPRB, 0.30759E+00_JPRB, 0.29076E+00_JPRB, 0.28197E+00_JPRB ,&
&  0.27663E+00_JPRB, 0.27219E+00_JPRB, 0.26698E+00_JPRB, 0.26134E+00_JPRB, 0.25575E+00_JPRB, 0.25154E+00_JPRB, 0.24901E+00_JPRB ,&
&  0.24668E+00_JPRB, 0.24777E+00_JPRB, 0.25093E+00_JPRB, 0.25577E+00_JPRB, 0.26273E+00_JPRB, 0.26696E+00_JPRB, 0.27043E+00_JPRB ,&
&  0.27162E+00_JPRB, 0.26151E+00_JPRB, 0.26451E+00_JPRB, 0.26746E+00_JPRB, 0.26660E+00_JPRB, 0.27104E+00_JPRB, 0.27833E+00_JPRB ,&
&  0.27484E+00_JPRB, 0.26538E+00_JPRB, 0.26737E+00_JPRB, 0.26068E+00_JPRB, 0.25193E+00_JPRB, 0.25081E+00_JPRB, 0.24488E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.23596E+00_JPRB, 0.23080E+00_JPRB, 0.22741E+00_JPRB, 0.22076E+00_JPRB, 0.21812E+00_JPRB, 0.21427E+00_JPRB ,&
&  0.21050E+00_JPRB, 0.20808E+00_JPRB, 0.20514E+00_JPRB, 0.20392E+00_JPRB, 0.20282E+00_JPRB, 0.20141E+00_JPRB, 0.20028E+00_JPRB ,&
&  0.19960E+00_JPRB, 0.19899E+00_JPRB, 0.19899E+00_JPRB, 0.19894E+00_JPRB, 0.19880E+00_JPRB, 0.19907E+00_JPRB, 0.19914E+00_JPRB ,&
&  0.19976E+00_JPRB, 0.20058E+00_JPRB, 0.20122E+00_JPRB, 0.20236E+00_JPRB, 0.20278E+00_JPRB, 0.20339E+00_JPRB, 0.20380E+00_JPRB ,&
&  0.20421E+00_JPRB, 0.20486E+00_JPRB, 0.20506E+00_JPRB, 0.20568E+00_JPRB, 0.20635E+00_JPRB, 0.20786E+00_JPRB, 0.20934E+00_JPRB ,&
&  0.21117E+00_JPRB, 0.21341E+00_JPRB, 0.21575E+00_JPRB, 0.21825E+00_JPRB, 0.22137E+00_JPRB, 0.22413E+00_JPRB, 0.22786E+00_JPRB ,&
&  0.23192E+00_JPRB, 0.23582E+00_JPRB, 0.24070E+00_JPRB, 0.24664E+00_JPRB, 0.25467E+00_JPRB, 0.26471E+00_JPRB, 0.27335E+00_JPRB ,&
&  0.28310E+00_JPRB, 0.30809E+00_JPRB, 0.31451E+00_JPRB, 0.32065E+00_JPRB, 0.32986E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.32884E+00_JPRB, 0.33408E+00_JPRB, 0.33812E+00_JPRB, 0.34074E+00_JPRB, 0.34337E+00_JPRB, 0.34508E+00_JPRB, 0.34573E+00_JPRB ,&
&  0.33915E+00_JPRB, 0.33425E+00_JPRB, 0.32712E+00_JPRB, 0.31842E+00_JPRB, 0.31042E+00_JPRB, 0.30222E+00_JPRB, 0.29362E+00_JPRB ,&
&  0.28652E+00_JPRB, 0.27984E+00_JPRB, 0.27316E+00_JPRB, 0.26605E+00_JPRB, 0.25868E+00_JPRB, 0.25126E+00_JPRB, 0.24836E+00_JPRB ,&
&  0.24793E+00_JPRB, 0.24971E+00_JPRB, 0.25250E+00_JPRB, 0.25405E+00_JPRB, 0.25493E+00_JPRB, 0.25536E+00_JPRB, 0.25413E+00_JPRB ,&
&  0.25016E+00_JPRB, 0.25118E+00_JPRB, 0.25263E+00_JPRB, 0.25359E+00_JPRB, 0.25550E+00_JPRB, 0.25714E+00_JPRB, 0.25705E+00_JPRB ,&
&  0.25498E+00_JPRB, 0.25231E+00_JPRB, 0.24867E+00_JPRB, 0.24525E+00_JPRB, 0.24419E+00_JPRB, 0.24213E+00_JPRB, 0.23979E+00_JPRB ,&
&  0.23833E+00_JPRB, 0.23447E+00_JPRB, 0.23016E+00_JPRB, 0.22712E+00_JPRB, 0.22283E+00_JPRB, 0.21993E+00_JPRB, 0.21610E+00_JPRB ,&
&  0.21310E+00_JPRB, 0.21048E+00_JPRB, 0.20758E+00_JPRB, 0.20540E+00_JPRB, 0.20340E+00_JPRB, 0.20192E+00_JPRB, 0.20042E+00_JPRB ,&
&  0.19956E+00_JPRB, 0.19869E+00_JPRB, 0.19812E+00_JPRB, 0.19745E+00_JPRB, 0.19700E+00_JPRB, 0.19656E+00_JPRB, 0.19622E+00_JPRB ,&
&  0.19602E+00_JPRB, 0.19637E+00_JPRB, 0.19659E+00_JPRB, 0.19701E+00_JPRB, 0.19749E+00_JPRB, 0.19772E+00_JPRB, 0.19825E+00_JPRB ,&
&  0.19934E+00_JPRB, 0.20053E+00_JPRB, 0.20121E+00_JPRB, 0.20216E+00_JPRB, 0.20324E+00_JPRB, 0.20454E+00_JPRB, 0.20640E+00_JPRB ,&
&  0.20833E+00_JPRB, 0.21051E+00_JPRB, 0.21298E+00_JPRB, 0.21539E+00_JPRB, 0.21859E+00_JPRB, 0.22188E+00_JPRB, 0.22537E+00_JPRB ,&
&  0.22963E+00_JPRB, 0.23412E+00_JPRB, 0.23882E+00_JPRB, 0.24400E+00_JPRB, 0.25151E+00_JPRB, 0.25743E+00_JPRB, 0.26472E+00_JPRB ,&
&  0.27152E+00_JPRB, 0.27546E+00_JPRB, 0.30855E+00_JPRB, 0.31335E+00_JPRB, 0.32029E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.25369E+00_JPRB, 0.26009E+00_JPRB, 0.26620E+00_JPRB, 0.27496E+00_JPRB, 0.27914E+00_JPRB, 0.31529E+00_JPRB, 0.31028E+00_JPRB ,&
&  0.30746E+00_JPRB, 0.29519E+00_JPRB, 0.29428E+00_JPRB, 0.29094E+00_JPRB, 0.28654E+00_JPRB, 0.28180E+00_JPRB, 0.27690E+00_JPRB ,&
&  0.27235E+00_JPRB, 0.26692E+00_JPRB, 0.26250E+00_JPRB, 0.25894E+00_JPRB, 0.25578E+00_JPRB, 0.25265E+00_JPRB, 0.24997E+00_JPRB ,&
&  0.24891E+00_JPRB, 0.24891E+00_JPRB, 0.24819E+00_JPRB, 0.24345E+00_JPRB, 0.23977E+00_JPRB, 0.23759E+00_JPRB, 0.23703E+00_JPRB ,&
&  0.23707E+00_JPRB, 0.23922E+00_JPRB, 0.24076E+00_JPRB, 0.24211E+00_JPRB, 0.24401E+00_JPRB, 0.24312E+00_JPRB, 0.24222E+00_JPRB ,&
&  0.24089E+00_JPRB, 0.23947E+00_JPRB, 0.23803E+00_JPRB, 0.23674E+00_JPRB, 0.23544E+00_JPRB, 0.23352E+00_JPRB, 0.23195E+00_JPRB ,&
&  0.23049E+00_JPRB, 0.22846E+00_JPRB, 0.22546E+00_JPRB, 0.22348E+00_JPRB, 0.22110E+00_JPRB, 0.21909E+00_JPRB, 0.21701E+00_JPRB ,&
&  0.21544E+00_JPRB, 0.21405E+00_JPRB, 0.21246E+00_JPRB, 0.21054E+00_JPRB, 0.20867E+00_JPRB, 0.20702E+00_JPRB, 0.20546E+00_JPRB ,&
&  0.20466E+00_JPRB, 0.20380E+00_JPRB, 0.20294E+00_JPRB, 0.20203E+00_JPRB, 0.20158E+00_JPRB, 0.20110E+00_JPRB, 0.20064E+00_JPRB ,&
&  0.20027E+00_JPRB, 0.20001E+00_JPRB, 0.19998E+00_JPRB, 0.20003E+00_JPRB, 0.20011E+00_JPRB, 0.20048E+00_JPRB, 0.20094E+00_JPRB ,&
&  0.20157E+00_JPRB, 0.20235E+00_JPRB, 0.20347E+00_JPRB, 0.20463E+00_JPRB, 0.20566E+00_JPRB, 0.20647E+00_JPRB, 0.20749E+00_JPRB ,&
&  0.20912E+00_JPRB, 0.21099E+00_JPRB, 0.21303E+00_JPRB, 0.21452E+00_JPRB, 0.21680E+00_JPRB, 0.21888E+00_JPRB, 0.22096E+00_JPRB ,&
&  0.22302E+00_JPRB, 0.22561E+00_JPRB, 0.22859E+00_JPRB, 0.23185E+00_JPRB, 0.23572E+00_JPRB, 0.23870E+00_JPRB, 0.24149E+00_JPRB ,&
&  0.24435E+00_JPRB, 0.24865E+00_JPRB, 0.25052E+00_JPRB, 0.25136E+00_JPRB, 0.25237E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.23309E+00_JPRB, 0.23307E+00_JPRB, 0.23336E+00_JPRB, 0.23366E+00_JPRB, 0.23389E+00_JPRB, 0.23443E+00_JPRB, 0.23447E+00_JPRB ,&
&  0.23433E+00_JPRB, 0.23410E+00_JPRB, 0.23445E+00_JPRB, 0.23484E+00_JPRB, 0.23507E+00_JPRB, 0.23508E+00_JPRB, 0.23468E+00_JPRB ,&
&  0.23394E+00_JPRB, 0.23303E+00_JPRB, 0.23147E+00_JPRB, 0.23080E+00_JPRB, 0.23032E+00_JPRB, 0.22990E+00_JPRB, 0.22910E+00_JPRB ,&
&  0.22839E+00_JPRB, 0.22717E+00_JPRB, 0.22551E+00_JPRB, 0.22441E+00_JPRB, 0.22436E+00_JPRB, 0.22485E+00_JPRB, 0.22567E+00_JPRB ,&
&  0.22632E+00_JPRB, 0.22677E+00_JPRB, 0.22698E+00_JPRB, 0.22726E+00_JPRB, 0.22764E+00_JPRB, 0.22752E+00_JPRB, 0.22728E+00_JPRB ,&
&  0.22695E+00_JPRB, 0.22638E+00_JPRB, 0.22583E+00_JPRB, 0.22534E+00_JPRB, 0.22490E+00_JPRB, 0.22436E+00_JPRB, 0.22372E+00_JPRB ,&
&  0.22298E+00_JPRB, 0.22207E+00_JPRB, 0.22096E+00_JPRB, 0.22007E+00_JPRB, 0.21913E+00_JPRB, 0.21838E+00_JPRB, 0.21743E+00_JPRB ,&
&  0.21664E+00_JPRB, 0.21591E+00_JPRB, 0.21518E+00_JPRB, 0.21450E+00_JPRB, 0.21386E+00_JPRB, 0.21326E+00_JPRB, 0.21268E+00_JPRB ,&
&  0.21211E+00_JPRB, 0.21165E+00_JPRB, 0.21114E+00_JPRB, 0.21065E+00_JPRB, 0.21032E+00_JPRB, 0.21004E+00_JPRB, 0.20981E+00_JPRB ,&
&  0.20961E+00_JPRB, 0.20956E+00_JPRB, 0.20955E+00_JPRB, 0.20953E+00_JPRB, 0.20953E+00_JPRB, 0.20965E+00_JPRB, 0.20977E+00_JPRB ,&
&  0.20994E+00_JPRB, 0.21005E+00_JPRB, 0.21017E+00_JPRB, 0.21036E+00_JPRB, 0.21056E+00_JPRB, 0.21079E+00_JPRB, 0.21112E+00_JPRB ,&
&  0.21151E+00_JPRB, 0.21196E+00_JPRB, 0.21246E+00_JPRB, 0.21272E+00_JPRB, 0.21341E+00_JPRB, 0.21420E+00_JPRB, 0.21514E+00_JPRB ,&
&  0.21617E+00_JPRB, 0.21742E+00_JPRB, 0.21885E+00_JPRB, 0.22056E+00_JPRB, 0.22237E+00_JPRB, 0.22346E+00_JPRB, 0.22457E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22708E+00_JPRB, 0.22837E+00_JPRB, 0.22968E+00_JPRB, 0.23123E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB, 0.21651E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB, 0.53329E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.54413E+00_JPRB, 0.54362E+00_JPRB, 0.54312E+00_JPRB, 0.54256E+00_JPRB, 0.54186E+00_JPRB, 0.54115E+00_JPRB, 0.54054E+00_JPRB ,&
&  0.53994E+00_JPRB, 0.53950E+00_JPRB, 0.53876E+00_JPRB, 0.53801E+00_JPRB, 0.53724E+00_JPRB, 0.53656E+00_JPRB, 0.53582E+00_JPRB ,&
&  0.53506E+00_JPRB, 0.53429E+00_JPRB, 0.53370E+00_JPRB, 0.53297E+00_JPRB, 0.53219E+00_JPRB, 0.53148E+00_JPRB, 0.53083E+00_JPRB ,&
&  0.53022E+00_JPRB, 0.52953E+00_JPRB, 0.52882E+00_JPRB, 0.52797E+00_JPRB, 0.52741E+00_JPRB, 0.52664E+00_JPRB, 0.52599E+00_JPRB ,&
&  0.52527E+00_JPRB, 0.52474E+00_JPRB, 0.52418E+00_JPRB, 0.52364E+00_JPRB, 0.52302E+00_JPRB, 0.52292E+00_JPRB, 0.52254E+00_JPRB ,&
&  0.52235E+00_JPRB, 0.52216E+00_JPRB, 0.52217E+00_JPRB, 0.52212E+00_JPRB, 0.52213E+00_JPRB, 0.52222E+00_JPRB, 0.52266E+00_JPRB ,&
&  0.52302E+00_JPRB, 0.52345E+00_JPRB, 0.52411E+00_JPRB, 0.52472E+00_JPRB, 0.52527E+00_JPRB, 0.52585E+00_JPRB, 0.52651E+00_JPRB ,&
&  0.52738E+00_JPRB, 0.52823E+00_JPRB, 0.52903E+00_JPRB, 0.52986E+00_JPRB, 0.53067E+00_JPRB, 0.53133E+00_JPRB, 0.53188E+00_JPRB ,&
&  0.53223E+00_JPRB, 0.53265E+00_JPRB, 0.53305E+00_JPRB, 0.53349E+00_JPRB, 0.53361E+00_JPRB, 0.53391E+00_JPRB, 0.53409E+00_JPRB ,&
&  0.53424E+00_JPRB, 0.53400E+00_JPRB, 0.53409E+00_JPRB, 0.53402E+00_JPRB, 0.53411E+00_JPRB, 0.53410E+00_JPRB, 0.53444E+00_JPRB ,&
&  0.53470E+00_JPRB, 0.53523E+00_JPRB, 0.53580E+00_JPRB, 0.53658E+00_JPRB, 0.53716E+00_JPRB, 0.53782E+00_JPRB, 0.53841E+00_JPRB ,&
&  0.53908E+00_JPRB, 0.53978E+00_JPRB, 0.54055E+00_JPRB, 0.54110E+00_JPRB, 0.54159E+00_JPRB, 0.54216E+00_JPRB, 0.54275E+00_JPRB ,&
&  0.54351E+00_JPRB, 0.54397E+00_JPRB, 0.54446E+00_JPRB, 0.54487E+00_JPRB, 0.54506E+00_JPRB, 0.54520E+00_JPRB, 0.54520E+00_JPRB ,&
&  0.54511E+00_JPRB, 0.54497E+00_JPRB, 0.54478E+00_JPRB, 0.54463E+00_JPRB, 0.54446E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.55279E+00_JPRB, 0.55247E+00_JPRB, 0.55193E+00_JPRB, 0.55111E+00_JPRB, 0.55014E+00_JPRB, 0.54895E+00_JPRB, 0.54780E+00_JPRB ,&
&  0.54650E+00_JPRB, 0.54524E+00_JPRB, 0.54369E+00_JPRB, 0.54215E+00_JPRB, 0.54057E+00_JPRB, 0.53907E+00_JPRB, 0.53747E+00_JPRB ,&
&  0.53596E+00_JPRB, 0.53446E+00_JPRB, 0.53340E+00_JPRB, 0.53226E+00_JPRB, 0.53102E+00_JPRB, 0.52953E+00_JPRB, 0.52827E+00_JPRB ,&
&  0.52694E+00_JPRB, 0.52537E+00_JPRB, 0.52367E+00_JPRB, 0.52073E+00_JPRB, 0.51819E+00_JPRB, 0.51585E+00_JPRB, 0.51383E+00_JPRB ,&
&  0.51200E+00_JPRB, 0.51028E+00_JPRB, 0.50895E+00_JPRB, 0.50807E+00_JPRB, 0.50890E+00_JPRB, 0.50904E+00_JPRB, 0.50875E+00_JPRB ,&
&  0.50830E+00_JPRB, 0.50656E+00_JPRB, 0.50602E+00_JPRB, 0.50587E+00_JPRB, 0.50593E+00_JPRB, 0.50719E+00_JPRB, 0.50824E+00_JPRB ,&
&  0.50937E+00_JPRB, 0.51030E+00_JPRB, 0.51209E+00_JPRB, 0.51382E+00_JPRB, 0.51542E+00_JPRB, 0.51714E+00_JPRB, 0.51817E+00_JPRB ,&
&  0.51933E+00_JPRB, 0.52102E+00_JPRB, 0.52262E+00_JPRB, 0.52431E+00_JPRB, 0.52535E+00_JPRB, 0.52630E+00_JPRB, 0.52700E+00_JPRB ,&
&  0.52760E+00_JPRB, 0.52840E+00_JPRB, 0.52927E+00_JPRB, 0.53009E+00_JPRB, 0.53064E+00_JPRB, 0.53116E+00_JPRB, 0.53150E+00_JPRB ,&
&  0.53162E+00_JPRB, 0.53122E+00_JPRB, 0.53109E+00_JPRB, 0.53086E+00_JPRB, 0.53043E+00_JPRB, 0.52989E+00_JPRB, 0.52974E+00_JPRB ,&
&  0.52955E+00_JPRB, 0.52918E+00_JPRB, 0.52860E+00_JPRB, 0.52890E+00_JPRB, 0.52932E+00_JPRB, 0.52955E+00_JPRB, 0.52960E+00_JPRB ,&
&  0.53150E+00_JPRB, 0.53414E+00_JPRB, 0.53720E+00_JPRB, 0.54134E+00_JPRB, 0.54441E+00_JPRB, 0.54636E+00_JPRB, 0.54770E+00_JPRB ,&
&  0.54829E+00_JPRB, 0.54886E+00_JPRB, 0.54935E+00_JPRB, 0.55011E+00_JPRB, 0.55064E+00_JPRB, 0.55158E+00_JPRB, 0.55285E+00_JPRB ,&
&  0.55371E+00_JPRB, 0.55396E+00_JPRB, 0.55387E+00_JPRB, 0.55355E+00_JPRB, 0.55316E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.55577E+00_JPRB, 0.55473E+00_JPRB, 0.55366E+00_JPRB, 0.55198E+00_JPRB, 0.55100E+00_JPRB, 0.54957E+00_JPRB, 0.54830E+00_JPRB ,&
&  0.54674E+00_JPRB, 0.54495E+00_JPRB, 0.54354E+00_JPRB, 0.54236E+00_JPRB, 0.54112E+00_JPRB, 0.53988E+00_JPRB, 0.53853E+00_JPRB ,&
&  0.53713E+00_JPRB, 0.53555E+00_JPRB, 0.53484E+00_JPRB, 0.53374E+00_JPRB, 0.53253E+00_JPRB, 0.53117E+00_JPRB, 0.52786E+00_JPRB ,&
&  0.52461E+00_JPRB, 0.52059E+00_JPRB, 0.51635E+00_JPRB, 0.51266E+00_JPRB, 0.51035E+00_JPRB, 0.49551E+00_JPRB, 0.49303E+00_JPRB ,&
&  0.49906E+00_JPRB, 0.50247E+00_JPRB, 0.50628E+00_JPRB, 0.51021E+00_JPRB, 0.50186E+00_JPRB, 0.49185E+00_JPRB, 0.48593E+00_JPRB ,&
&  0.47200E+00_JPRB, 0.47754E+00_JPRB, 0.48043E+00_JPRB, 0.48317E+00_JPRB, 0.48719E+00_JPRB, 0.48735E+00_JPRB, 0.48988E+00_JPRB ,&
&  0.49898E+00_JPRB, 0.49910E+00_JPRB, 0.50038E+00_JPRB, 0.50179E+00_JPRB, 0.50318E+00_JPRB, 0.50458E+00_JPRB, 0.50718E+00_JPRB ,&
&  0.50930E+00_JPRB, 0.51175E+00_JPRB, 0.51446E+00_JPRB, 0.51575E+00_JPRB, 0.51705E+00_JPRB, 0.51881E+00_JPRB, 0.52045E+00_JPRB ,&
&  0.52125E+00_JPRB, 0.52231E+00_JPRB, 0.52348E+00_JPRB, 0.52460E+00_JPRB, 0.52607E+00_JPRB, 0.52707E+00_JPRB, 0.52790E+00_JPRB ,&
&  0.52855E+00_JPRB, 0.52842E+00_JPRB, 0.52840E+00_JPRB, 0.52859E+00_JPRB, 0.52854E+00_JPRB, 0.52831E+00_JPRB, 0.52850E+00_JPRB ,&
&  0.52907E+00_JPRB, 0.52947E+00_JPRB, 0.53031E+00_JPRB, 0.53154E+00_JPRB, 0.53462E+00_JPRB, 0.53882E+00_JPRB, 0.54712E+00_JPRB ,&
&  0.55395E+00_JPRB, 0.55987E+00_JPRB, 0.56202E+00_JPRB, 0.56436E+00_JPRB, 0.56613E+00_JPRB, 0.56652E+00_JPRB, 0.56635E+00_JPRB ,&
&  0.56523E+00_JPRB, 0.56329E+00_JPRB, 0.56134E+00_JPRB, 0.55953E+00_JPRB, 0.55848E+00_JPRB, 0.55777E+00_JPRB, 0.55778E+00_JPRB ,&
&  0.55838E+00_JPRB, 0.55935E+00_JPRB, 0.55956E+00_JPRB, 0.55836E+00_JPRB, 0.55688E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.56344E+00_JPRB, 0.56097E+00_JPRB, 0.56004E+00_JPRB, 0.55788E+00_JPRB, 0.55581E+00_JPRB, 0.55312E+00_JPRB, 0.55089E+00_JPRB ,&
&  0.54834E+00_JPRB, 0.54604E+00_JPRB, 0.54403E+00_JPRB, 0.54217E+00_JPRB, 0.54116E+00_JPRB, 0.54004E+00_JPRB, 0.53930E+00_JPRB ,&
&  0.53835E+00_JPRB, 0.53710E+00_JPRB, 0.53638E+00_JPRB, 0.53535E+00_JPRB, 0.53433E+00_JPRB, 0.53317E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.52559E+00_JPRB, 0.51869E+00_JPRB, 0.50450E+00_JPRB, 0.50673E+00_JPRB, 0.50515E+00_JPRB, 0.50346E+00_JPRB, 0.50415E+00_JPRB ,&
&  0.49714E+00_JPRB, 0.49516E+00_JPRB, 0.48714E+00_JPRB, 0.48004E+00_JPRB, 0.47753E+00_JPRB, 0.47070E+00_JPRB, 0.47075E+00_JPRB ,&
&  0.46839E+00_JPRB, 0.46718E+00_JPRB, 0.46809E+00_JPRB, 0.46761E+00_JPRB, 0.47162E+00_JPRB, 0.47349E+00_JPRB, 0.47647E+00_JPRB ,&
&  0.48200E+00_JPRB, 0.48651E+00_JPRB, 0.48948E+00_JPRB, 0.48477E+00_JPRB, 0.48719E+00_JPRB, 0.48966E+00_JPRB, 0.49278E+00_JPRB ,&
&  0.49577E+00_JPRB, 0.49789E+00_JPRB, 0.50014E+00_JPRB, 0.50175E+00_JPRB, 0.50357E+00_JPRB, 0.50631E+00_JPRB, 0.50917E+00_JPRB ,&
&  0.51265E+00_JPRB, 0.51510E+00_JPRB, 0.51666E+00_JPRB, 0.51827E+00_JPRB, 0.51996E+00_JPRB, 0.52162E+00_JPRB, 0.52373E+00_JPRB ,&
&  0.52635E+00_JPRB, 0.52962E+00_JPRB, 0.53313E+00_JPRB, 0.53681E+00_JPRB, 0.54018E+00_JPRB, 0.54463E+00_JPRB, 0.54971E+00_JPRB ,&
&  0.55724E+00_JPRB, 0.56407E+00_JPRB, 0.57463E+00_JPRB, 0.58113E+00_JPRB, 0.59467E+00_JPRB, 0.60738E+00_JPRB, 0.61534E+00_JPRB ,&
&  0.61384E+00_JPRB, 0.61001E+00_JPRB, 0.60620E+00_JPRB, 0.60200E+00_JPRB, 0.59831E+00_JPRB, 0.59366E+00_JPRB, 0.59036E+00_JPRB ,&
&  0.58707E+00_JPRB, 0.58387E+00_JPRB, 0.58063E+00_JPRB, 0.57736E+00_JPRB, 0.57462E+00_JPRB, 0.57163E+00_JPRB, 0.56974E+00_JPRB ,&
&  0.56904E+00_JPRB, 0.56946E+00_JPRB, 0.56941E+00_JPRB, 0.56839E+00_JPRB, 0.56615E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.57255E+00_JPRB, 0.57214E+00_JPRB, 0.57321E+00_JPRB, 0.57213E+00_JPRB, 0.56957E+00_JPRB, 0.56699E+00_JPRB, 0.56192E+00_JPRB ,&
&  0.55689E+00_JPRB, 0.55188E+00_JPRB, 0.54743E+00_JPRB, 0.54409E+00_JPRB, 0.54133E+00_JPRB, 0.53906E+00_JPRB, 0.53811E+00_JPRB ,&
&  0.53671E+00_JPRB, 0.53646E+00_JPRB, 0.53630E+00_JPRB, 0.53635E+00_JPRB, 0.53705E+00_JPRB, 0.53724E+00_JPRB, 0.53731E+00_JPRB ,&
&  0.53427E+00_JPRB, 0.52615E+00_JPRB, 0.51519E+00_JPRB, 0.51686E+00_JPRB, 0.51801E+00_JPRB, 0.52005E+00_JPRB, 0.52246E+00_JPRB ,&
&  0.52002E+00_JPRB, 0.49976E+00_JPRB, 0.50626E+00_JPRB, 0.49715E+00_JPRB, 0.49245E+00_JPRB, 0.48710E+00_JPRB, 0.47229E+00_JPRB ,&
&  0.46712E+00_JPRB, 0.45822E+00_JPRB, 0.45322E+00_JPRB, 0.45323E+00_JPRB, 0.45526E+00_JPRB, 0.46071E+00_JPRB, 0.46758E+00_JPRB ,&
&  0.46843E+00_JPRB, 0.47850E+00_JPRB, 0.47973E+00_JPRB, 0.47933E+00_JPRB, 0.48184E+00_JPRB, 0.48486E+00_JPRB, 0.48340E+00_JPRB ,&
&  0.48435E+00_JPRB, 0.48395E+00_JPRB, 0.48354E+00_JPRB, 0.48650E+00_JPRB, 0.49230E+00_JPRB, 0.49525E+00_JPRB, 0.49828E+00_JPRB ,&
&  0.50263E+00_JPRB, 0.50518E+00_JPRB, 0.50676E+00_JPRB, 0.50889E+00_JPRB, 0.51178E+00_JPRB, 0.51473E+00_JPRB, 0.51925E+00_JPRB ,&
&  0.52437E+00_JPRB, 0.53328E+00_JPRB, 0.54164E+00_JPRB, 0.55423E+00_JPRB, 0.56603E+00_JPRB, 0.58187E+00_JPRB, 0.59573E+00_JPRB ,&
&  0.61039E+00_JPRB, 0.62231E+00_JPRB, 0.63681E+00_JPRB, 0.65015E+00_JPRB, 0.66068E+00_JPRB, 0.65728E+00_JPRB, 0.64491E+00_JPRB ,&
&  0.63549E+00_JPRB, 0.62209E+00_JPRB, 0.61436E+00_JPRB, 0.60817E+00_JPRB, 0.60380E+00_JPRB, 0.60174E+00_JPRB, 0.59952E+00_JPRB ,&
&  0.59837E+00_JPRB, 0.59621E+00_JPRB, 0.59339E+00_JPRB, 0.58956E+00_JPRB, 0.58633E+00_JPRB, 0.58287E+00_JPRB, 0.58054E+00_JPRB ,&
&  0.57747E+00_JPRB, 0.57639E+00_JPRB, 0.57482E+00_JPRB, 0.57391E+00_JPRB, 0.57321E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.58193E+00_JPRB, 0.58496E+00_JPRB, 0.58392E+00_JPRB, 0.58220E+00_JPRB, 0.57659E+00_JPRB, 0.57185E+00_JPRB, 0.56445E+00_JPRB ,&
&  0.55727E+00_JPRB, 0.55003E+00_JPRB, 0.54348E+00_JPRB, 0.53893E+00_JPRB, 0.53543E+00_JPRB, 0.53298E+00_JPRB, 0.53193E+00_JPRB ,&
&  0.53165E+00_JPRB, 0.53234E+00_JPRB, 0.53303E+00_JPRB, 0.53385E+00_JPRB, 0.53586E+00_JPRB, 0.53820E+00_JPRB, 0.54074E+00_JPRB ,&
&  0.54122E+00_JPRB, 0.53763E+00_JPRB, 0.52745E+00_JPRB, 0.52638E+00_JPRB, 0.53116E+00_JPRB, 0.52775E+00_JPRB, 0.53776E+00_JPRB ,&
&  0.54514E+00_JPRB, 0.54821E+00_JPRB, 0.54046E+00_JPRB, 0.53503E+00_JPRB, 0.52445E+00_JPRB, 0.48269E+00_JPRB, 0.47991E+00_JPRB ,&
&  0.47253E+00_JPRB, 0.46124E+00_JPRB, 0.45629E+00_JPRB, 0.45216E+00_JPRB, 0.45144E+00_JPRB, 0.45833E+00_JPRB, 0.46817E+00_JPRB ,&
&  0.47040E+00_JPRB, 0.48573E+00_JPRB, 0.48567E+00_JPRB, 0.48494E+00_JPRB, 0.48600E+00_JPRB, 0.48932E+00_JPRB, 0.48779E+00_JPRB ,&
&  0.48875E+00_JPRB, 0.48909E+00_JPRB, 0.48927E+00_JPRB, 0.49332E+00_JPRB, 0.49740E+00_JPRB, 0.50069E+00_JPRB, 0.50573E+00_JPRB ,&
&  0.51655E+00_JPRB, 0.52356E+00_JPRB, 0.52750E+00_JPRB, 0.53096E+00_JPRB, 0.53655E+00_JPRB, 0.54287E+00_JPRB, 0.55189E+00_JPRB ,&
&  0.55858E+00_JPRB, 0.56682E+00_JPRB, 0.57646E+00_JPRB, 0.58669E+00_JPRB, 0.59715E+00_JPRB, 0.60880E+00_JPRB, 0.62153E+00_JPRB ,&
&  0.62851E+00_JPRB, 0.63845E+00_JPRB, 0.64958E+00_JPRB, 0.66926E+00_JPRB, 0.66968E+00_JPRB, 0.65700E+00_JPRB, 0.64691E+00_JPRB ,&
&  0.63956E+00_JPRB, 0.64009E+00_JPRB, 0.63337E+00_JPRB, 0.62908E+00_JPRB, 0.62434E+00_JPRB, 0.62261E+00_JPRB, 0.61924E+00_JPRB ,&
&  0.61661E+00_JPRB, 0.61271E+00_JPRB, 0.60740E+00_JPRB, 0.60118E+00_JPRB, 0.59654E+00_JPRB, 0.59160E+00_JPRB, 0.58809E+00_JPRB ,&
&  0.58369E+00_JPRB, 0.58157E+00_JPRB, 0.58013E+00_JPRB, 0.57988E+00_JPRB, 0.57936E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.59024E+00_JPRB, 0.59193E+00_JPRB, 0.58631E+00_JPRB, 0.58351E+00_JPRB, 0.57478E+00_JPRB, 0.56602E+00_JPRB, 0.55512E+00_JPRB ,&
&  0.54658E+00_JPRB, 0.54012E+00_JPRB, 0.53397E+00_JPRB, 0.53114E+00_JPRB, 0.53005E+00_JPRB, 0.52980E+00_JPRB, 0.53134E+00_JPRB ,&
&  0.53320E+00_JPRB, 0.53155E+00_JPRB, 0.53376E+00_JPRB, 0.53765E+00_JPRB, 0.54060E+00_JPRB, 0.54316E+00_JPRB, 0.54578E+00_JPRB ,&
&  0.54487E+00_JPRB, 0.54347E+00_JPRB, 0.54128E+00_JPRB, 0.53527E+00_JPRB, 0.53285E+00_JPRB, 0.54061E+00_JPRB, 0.55184E+00_JPRB ,&
&  0.54662E+00_JPRB, 0.54751E+00_JPRB, 0.55281E+00_JPRB, 0.55278E+00_JPRB, 0.53739E+00_JPRB, 0.51516E+00_JPRB, 0.49417E+00_JPRB ,&
&  0.47910E+00_JPRB, 0.45960E+00_JPRB, 0.45128E+00_JPRB, 0.44701E+00_JPRB, 0.45083E+00_JPRB, 0.45848E+00_JPRB, 0.47284E+00_JPRB ,&
&  0.48172E+00_JPRB, 0.48563E+00_JPRB, 0.49066E+00_JPRB, 0.49541E+00_JPRB, 0.49668E+00_JPRB, 0.49997E+00_JPRB, 0.50105E+00_JPRB ,&
&  0.50566E+00_JPRB, 0.50867E+00_JPRB, 0.51169E+00_JPRB, 0.51820E+00_JPRB, 0.52821E+00_JPRB, 0.53870E+00_JPRB, 0.55466E+00_JPRB ,&
&  0.59401E+00_JPRB, 0.60715E+00_JPRB, 0.61529E+00_JPRB, 0.61924E+00_JPRB, 0.62507E+00_JPRB, 0.62691E+00_JPRB, 0.62001E+00_JPRB ,&
&  0.61992E+00_JPRB, 0.62952E+00_JPRB, 0.63138E+00_JPRB, 0.63423E+00_JPRB, 0.63856E+00_JPRB, 0.64291E+00_JPRB, 0.64853E+00_JPRB ,&
&  0.64406E+00_JPRB, 0.65676E+00_JPRB, 0.71773E+00_JPRB, 0.73636E+00_JPRB, 0.72478E+00_JPRB, 0.70737E+00_JPRB, 0.69061E+00_JPRB ,&
&  0.67768E+00_JPRB, 0.66622E+00_JPRB, 0.65635E+00_JPRB, 0.64823E+00_JPRB, 0.64224E+00_JPRB, 0.63893E+00_JPRB, 0.63800E+00_JPRB ,&
&  0.63200E+00_JPRB, 0.62489E+00_JPRB, 0.61696E+00_JPRB, 0.61098E+00_JPRB, 0.60597E+00_JPRB, 0.60022E+00_JPRB, 0.59663E+00_JPRB ,&
&  0.59304E+00_JPRB, 0.59249E+00_JPRB, 0.59221E+00_JPRB, 0.59246E+00_JPRB, 0.59179E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.59205E+00_JPRB, 0.59456E+00_JPRB, 0.59425E+00_JPRB, 0.58571E+00_JPRB, 0.56780E+00_JPRB, 0.55097E+00_JPRB, 0.53878E+00_JPRB ,&
&  0.53370E+00_JPRB, 0.53120E+00_JPRB, 0.52926E+00_JPRB, 0.52827E+00_JPRB, 0.52854E+00_JPRB, 0.52996E+00_JPRB, 0.53245E+00_JPRB ,&
&  0.53601E+00_JPRB, 0.53934E+00_JPRB, 0.54090E+00_JPRB, 0.54144E+00_JPRB, 0.54386E+00_JPRB, 0.54851E+00_JPRB, 0.55361E+00_JPRB ,&
&  0.55597E+00_JPRB, 0.55492E+00_JPRB, 0.55552E+00_JPRB, 0.55301E+00_JPRB, 0.55110E+00_JPRB, 0.54807E+00_JPRB, 0.54833E+00_JPRB ,&
&  0.55583E+00_JPRB, 0.56169E+00_JPRB, 0.55378E+00_JPRB, 0.55459E+00_JPRB, 0.55985E+00_JPRB, 0.54023E+00_JPRB, 0.51242E+00_JPRB ,&
&  0.48247E+00_JPRB, 0.46003E+00_JPRB, 0.45042E+00_JPRB, 0.44815E+00_JPRB, 0.45514E+00_JPRB, 0.47091E+00_JPRB, 0.48998E+00_JPRB ,&
&  0.49209E+00_JPRB, 0.49753E+00_JPRB, 0.50628E+00_JPRB, 0.51164E+00_JPRB, 0.51332E+00_JPRB, 0.51368E+00_JPRB, 0.51481E+00_JPRB ,&
&  0.51872E+00_JPRB, 0.52868E+00_JPRB, 0.54574E+00_JPRB, 0.57026E+00_JPRB, 0.60708E+00_JPRB, 0.62495E+00_JPRB, 0.66089E+00_JPRB ,&
&  0.71851E+00_JPRB, 0.71609E+00_JPRB, 0.71996E+00_JPRB, 0.71683E+00_JPRB, 0.71312E+00_JPRB, 0.70692E+00_JPRB, 0.70430E+00_JPRB ,&
&  0.70305E+00_JPRB, 0.69963E+00_JPRB, 0.69170E+00_JPRB, 0.68382E+00_JPRB, 0.68480E+00_JPRB, 0.69251E+00_JPRB, 0.71070E+00_JPRB ,&
&  0.74008E+00_JPRB, 0.78814E+00_JPRB, 0.90709E+00_JPRB, 0.83109E+00_JPRB, 0.78946E+00_JPRB, 0.77171E+00_JPRB, 0.74776E+00_JPRB ,&
&  0.72783E+00_JPRB, 0.71453E+00_JPRB, 0.70610E+00_JPRB, 0.69570E+00_JPRB, 0.67580E+00_JPRB, 0.66130E+00_JPRB, 0.65043E+00_JPRB ,&
&  0.64158E+00_JPRB, 0.63273E+00_JPRB, 0.62597E+00_JPRB, 0.62117E+00_JPRB, 0.61579E+00_JPRB, 0.61108E+00_JPRB, 0.60832E+00_JPRB ,&
&  0.60413E+00_JPRB, 0.60132E+00_JPRB, 0.59954E+00_JPRB, 0.59608E+00_JPRB, 0.59358E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.59555E+00_JPRB, 0.59831E+00_JPRB, 0.59191E+00_JPRB, 0.58204E+00_JPRB, 0.56153E+00_JPRB, 0.54569E+00_JPRB, 0.53533E+00_JPRB ,&
&  0.53219E+00_JPRB, 0.52839E+00_JPRB, 0.52647E+00_JPRB, 0.52626E+00_JPRB, 0.52771E+00_JPRB, 0.53100E+00_JPRB, 0.53344E+00_JPRB ,&
&  0.53540E+00_JPRB, 0.53893E+00_JPRB, 0.54323E+00_JPRB, 0.54641E+00_JPRB, 0.54912E+00_JPRB, 0.55264E+00_JPRB, 0.55723E+00_JPRB ,&
&  0.56220E+00_JPRB, 0.56444E+00_JPRB, 0.56537E+00_JPRB, 0.56589E+00_JPRB, 0.56917E+00_JPRB, 0.57398E+00_JPRB, 0.57457E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.56381E+00_JPRB, 0.56917E+00_JPRB, 0.56928E+00_JPRB, 0.57297E+00_JPRB, 0.55920E+00_JPRB, 0.52731E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.47076E+00_JPRB, 0.46761E+00_JPRB, 0.47685E+00_JPRB, 0.49396E+00_JPRB, 0.50146E+00_JPRB, 0.51134E+00_JPRB ,&
&  0.51914E+00_JPRB, 0.52190E+00_JPRB, 0.52430E+00_JPRB, 0.53011E+00_JPRB, 0.53367E+00_JPRB, 0.53420E+00_JPRB, 0.53682E+00_JPRB ,&
&  0.54478E+00_JPRB, 0.55850E+00_JPRB, 0.56535E+00_JPRB, 0.56511E+00_JPRB, 0.59330E+00_JPRB, 0.66826E+00_JPRB, 0.73369E+00_JPRB ,&
&  0.74866E+00_JPRB, 0.75611E+00_JPRB, 0.76319E+00_JPRB, 0.78931E+00_JPRB, 0.79966E+00_JPRB, 0.80930E+00_JPRB, 0.80834E+00_JPRB ,&
&  0.79656E+00_JPRB, 0.77772E+00_JPRB, 0.76213E+00_JPRB, 0.76159E+00_JPRB, 0.78329E+00_JPRB, 0.82530E+00_JPRB, 0.86794E+00_JPRB ,&
&  0.89467E+00_JPRB, 0.87607E+00_JPRB, 0.80658E+00_JPRB, 0.74365E+00_JPRB, 0.72452E+00_JPRB, 0.72975E+00_JPRB, 0.73969E+00_JPRB ,&
&  0.73580E+00_JPRB, 0.72914E+00_JPRB, 0.72144E+00_JPRB, 0.71364E+00_JPRB, 0.68090E+00_JPRB, 0.65988E+00_JPRB, 0.64291E+00_JPRB ,&
&  0.63151E+00_JPRB, 0.62349E+00_JPRB, 0.62406E+00_JPRB, 0.62125E+00_JPRB, 0.61740E+00_JPRB, 0.61235E+00_JPRB, 0.60474E+00_JPRB ,&
&  0.59970E+00_JPRB, 0.59604E+00_JPRB, 0.59219E+00_JPRB, 0.58990E+00_JPRB, 0.59024E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.60748E+00_JPRB, 0.60338E+00_JPRB, 0.59840E+00_JPRB, 0.58646E+00_JPRB, 0.57664E+00_JPRB, 0.57156E+00_JPRB, 0.56784E+00_JPRB ,&
&  0.57408E+00_JPRB, 0.55708E+00_JPRB, 0.54075E+00_JPRB, 0.52943E+00_JPRB, 0.52333E+00_JPRB, 0.52209E+00_JPRB, 0.52686E+00_JPRB ,&
&  0.53379E+00_JPRB, 0.54022E+00_JPRB, 0.54586E+00_JPRB, 0.55186E+00_JPRB, 0.55750E+00_JPRB, 0.56124E+00_JPRB, 0.56510E+00_JPRB ,&
&  0.57007E+00_JPRB, 0.57508E+00_JPRB, 0.57965E+00_JPRB, 0.58621E+00_JPRB, 0.59065E+00_JPRB, 0.59309E+00_JPRB, 0.59084E+00_JPRB ,&
&  0.58528E+00_JPRB, 0.58058E+00_JPRB, 0.58121E+00_JPRB, 0.58295E+00_JPRB, 0.58373E+00_JPRB, 0.57155E+00_JPRB, 0.53958E+00_JPRB ,&
&  0.50988E+00_JPRB, 0.49757E+00_JPRB, 0.50967E+00_JPRB, 0.52610E+00_JPRB, 0.53454E+00_JPRB, 0.54146E+00_JPRB, 0.54239E+00_JPRB ,&
&  0.54012E+00_JPRB, 0.53993E+00_JPRB, 0.54704E+00_JPRB, 0.55540E+00_JPRB, 0.55793E+00_JPRB, 0.56014E+00_JPRB, 0.56467E+00_JPRB ,&
&  0.57630E+00_JPRB, 0.58971E+00_JPRB, 0.59670E+00_JPRB, 0.59315E+00_JPRB, 0.65809E+00_JPRB, 0.87781E+00_JPRB, 0.88476E+00_JPRB ,&
&  0.89777E+00_JPRB, 0.90651E+00_JPRB, 0.92887E+00_JPRB, 0.95068E+00_JPRB, 0.93571E+00_JPRB, 0.92884E+00_JPRB, 0.92731E+00_JPRB ,&
&  0.90352E+00_JPRB, 0.89442E+00_JPRB, 0.89877E+00_JPRB, 0.93936E+00_JPRB, 0.97182E+00_JPRB, 0.99309E+00_JPRB, 0.10100E+01_JPRB ,&
&  0.10024E+01_JPRB, 0.94951E+00_JPRB, 0.88371E+00_JPRB, 0.82332E+00_JPRB, 0.79294E+00_JPRB, 0.77329E+00_JPRB, 0.76136E+00_JPRB ,&
&  0.74825E+00_JPRB, 0.74334E+00_JPRB, 0.74246E+00_JPRB, 0.73862E+00_JPRB, 0.70904E+00_JPRB, 0.67256E+00_JPRB, 0.65057E+00_JPRB ,&
&  0.63723E+00_JPRB, 0.62618E+00_JPRB, 0.61658E+00_JPRB, 0.61062E+00_JPRB, 0.61249E+00_JPRB, 0.61394E+00_JPRB, 0.60796E+00_JPRB ,&
&  0.60744E+00_JPRB, 0.60182E+00_JPRB, 0.60052E+00_JPRB, 0.60181E+00_JPRB, 0.60762E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.61235E+00_JPRB, 0.61096E+00_JPRB, 0.60689E+00_JPRB, 0.60052E+00_JPRB, 0.59884E+00_JPRB, 0.59636E+00_JPRB, 0.58646E+00_JPRB ,&
&  0.59782E+00_JPRB, 0.56328E+00_JPRB, 0.54629E+00_JPRB, 0.53878E+00_JPRB, 0.53767E+00_JPRB, 0.53327E+00_JPRB, 0.52759E+00_JPRB ,&
&  0.52992E+00_JPRB, 0.53947E+00_JPRB, 0.55019E+00_JPRB, 0.55811E+00_JPRB, 0.56530E+00_JPRB, 0.56970E+00_JPRB, 0.57363E+00_JPRB ,&
&  0.57879E+00_JPRB, 0.58609E+00_JPRB, 0.59626E+00_JPRB, 0.60302E+00_JPRB, 0.60605E+00_JPRB, 0.60829E+00_JPRB, 0.60640E+00_JPRB ,&
&  0.60300E+00_JPRB, 0.59605E+00_JPRB, 0.59207E+00_JPRB, 0.59029E+00_JPRB, 0.58791E+00_JPRB, 0.58803E+00_JPRB, 0.57327E+00_JPRB ,&
&  0.54447E+00_JPRB, 0.54474E+00_JPRB, 0.55819E+00_JPRB, 0.56764E+00_JPRB, 0.56825E+00_JPRB, 0.56518E+00_JPRB, 0.56515E+00_JPRB ,&
&  0.56708E+00_JPRB, 0.56913E+00_JPRB, 0.57430E+00_JPRB, 0.57640E+00_JPRB, 0.57978E+00_JPRB, 0.58302E+00_JPRB, 0.59469E+00_JPRB ,&
&  0.62001E+00_JPRB, 0.60260E+00_JPRB, 0.60951E+00_JPRB, 0.65269E+00_JPRB, 0.83393E+00_JPRB, 0.91673E+00_JPRB, 0.98018E+00_JPRB ,&
&  0.10885E+01_JPRB, 0.11083E+01_JPRB, 0.11041E+01_JPRB, 0.11052E+01_JPRB, 0.11119E+01_JPRB, 0.11151E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.11114E+01_JPRB, 0.11348E+01_JPRB, 0.11723E+01_JPRB, 0.11807E+01_JPRB, 0.11773E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.10725E+01_JPRB, 0.99978E+00_JPRB, 0.92908E+00_JPRB, 0.86995E+00_JPRB, 0.84004E+00_JPRB, 0.81944E+00_JPRB ,&
&  0.80151E+00_JPRB, 0.79147E+00_JPRB, 0.78107E+00_JPRB, 0.76765E+00_JPRB, 0.75094E+00_JPRB, 0.73358E+00_JPRB, 0.72789E+00_JPRB ,&
&  0.69404E+00_JPRB, 0.67228E+00_JPRB, 0.64829E+00_JPRB, 0.63552E+00_JPRB, 0.63434E+00_JPRB, 0.63594E+00_JPRB, 0.63577E+00_JPRB ,&
&  0.63934E+00_JPRB, 0.62786E+00_JPRB, 0.62450E+00_JPRB, 0.62192E+00_JPRB, 0.61955E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.61759E+00_JPRB, 0.62088E+00_JPRB, 0.61858E+00_JPRB, 0.61661E+00_JPRB, 0.62087E+00_JPRB, 0.62151E+00_JPRB, 0.62109E+00_JPRB ,&
&  0.66087E+00_JPRB, 0.63693E+00_JPRB, 0.64496E+00_JPRB, 0.58754E+00_JPRB, 0.56333E+00_JPRB, 0.55143E+00_JPRB, 0.53667E+00_JPRB ,&
&  0.53454E+00_JPRB, 0.53939E+00_JPRB, 0.54631E+00_JPRB, 0.55566E+00_JPRB, 0.56553E+00_JPRB, 0.57125E+00_JPRB, 0.57660E+00_JPRB ,&
&  0.58320E+00_JPRB, 0.59197E+00_JPRB, 0.60049E+00_JPRB, 0.60638E+00_JPRB, 0.61001E+00_JPRB, 0.61386E+00_JPRB, 0.61048E+00_JPRB ,&
&  0.59974E+00_JPRB, 0.59537E+00_JPRB, 0.59391E+00_JPRB, 0.59154E+00_JPRB, 0.59384E+00_JPRB, 0.59528E+00_JPRB, 0.60434E+00_JPRB ,&
&  0.60423E+00_JPRB, 0.59681E+00_JPRB, 0.60343E+00_JPRB, 0.60007E+00_JPRB, 0.59697E+00_JPRB, 0.59251E+00_JPRB, 0.58800E+00_JPRB ,&
&  0.58605E+00_JPRB, 0.58789E+00_JPRB, 0.59394E+00_JPRB, 0.59871E+00_JPRB, 0.60302E+00_JPRB, 0.61233E+00_JPRB, 0.63679E+00_JPRB ,&
&  0.67669E+00_JPRB, 0.64236E+00_JPRB, 0.71312E+00_JPRB, 0.75244E+00_JPRB, 0.94471E+00_JPRB, 0.11486E+01_JPRB, 0.11935E+01_JPRB ,&
&  0.13865E+01_JPRB, 0.13922E+01_JPRB, 0.13655E+01_JPRB, 0.13473E+01_JPRB, 0.13548E+01_JPRB, 0.13261E+01_JPRB, 0.12779E+01_JPRB ,&
&  0.12739E+01_JPRB, 0.12611E+01_JPRB, 0.13392E+01_JPRB, 0.13415E+01_JPRB, 0.13027E+01_JPRB, 0.12747E+01_JPRB, 0.12215E+01_JPRB ,&
&  0.11784E+01_JPRB, 0.11339E+01_JPRB, 0.10900E+01_JPRB, 0.99980E+00_JPRB, 0.96579E+00_JPRB, 0.93244E+00_JPRB, 0.87038E+00_JPRB ,&
&  0.84278E+00_JPRB, 0.80873E+00_JPRB, 0.77573E+00_JPRB, 0.78296E+00_JPRB, 0.75899E+00_JPRB, 0.74548E+00_JPRB, 0.73906E+00_JPRB ,&
&  0.71702E+00_JPRB, 0.69233E+00_JPRB, 0.69560E+00_JPRB, 0.69047E+00_JPRB, 0.67606E+00_JPRB, 0.66386E+00_JPRB, 0.66266E+00_JPRB ,&
&  0.65164E+00_JPRB, 0.64671E+00_JPRB, 0.65191E+00_JPRB, 0.63772E+00_JPRB, 0.62587E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.63618E+00_JPRB, 0.63691E+00_JPRB, 0.63211E+00_JPRB, 0.63452E+00_JPRB, 0.64406E+00_JPRB, 0.65277E+00_JPRB, 0.66099E+00_JPRB ,&
&  0.68221E+00_JPRB, 0.68975E+00_JPRB, 0.68805E+00_JPRB, 0.67619E+00_JPRB, 0.65593E+00_JPRB, 0.60698E+00_JPRB, 0.54800E+00_JPRB ,&
&  0.53438E+00_JPRB, 0.54293E+00_JPRB, 0.55352E+00_JPRB, 0.55929E+00_JPRB, 0.56705E+00_JPRB, 0.57526E+00_JPRB, 0.58265E+00_JPRB ,&
&  0.59078E+00_JPRB, 0.59713E+00_JPRB, 0.60403E+00_JPRB, 0.60970E+00_JPRB, 0.61272E+00_JPRB, 0.61755E+00_JPRB, 0.61901E+00_JPRB ,&
&  0.60287E+00_JPRB, 0.59451E+00_JPRB, 0.58984E+00_JPRB, 0.59348E+00_JPRB, 0.60774E+00_JPRB, 0.61700E+00_JPRB, 0.62807E+00_JPRB ,&
&  0.63769E+00_JPRB, 0.64267E+00_JPRB, 0.64077E+00_JPRB, 0.63779E+00_JPRB, 0.63289E+00_JPRB, 0.62972E+00_JPRB, 0.62493E+00_JPRB ,&
&  0.62184E+00_JPRB, 0.62159E+00_JPRB, 0.62682E+00_JPRB, 0.63102E+00_JPRB, 0.63777E+00_JPRB, 0.65620E+00_JPRB, 0.71525E+00_JPRB ,&
&  0.77837E+00_JPRB, 0.85375E+00_JPRB, 0.89455E+00_JPRB, 0.90305E+00_JPRB, 0.10751E+01_JPRB, 0.12315E+01_JPRB, 0.13019E+01_JPRB ,&
&  0.14111E+01_JPRB, 0.13839E+01_JPRB, 0.14673E+01_JPRB, 0.15558E+01_JPRB, 0.16221E+01_JPRB, 0.15920E+01_JPRB, 0.14985E+01_JPRB ,&
&  0.14845E+01_JPRB, 0.15623E+01_JPRB, 0.15643E+01_JPRB, 0.15577E+01_JPRB, 0.14920E+01_JPRB, 0.14430E+01_JPRB, 0.13779E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.13466E+01_JPRB, 0.12883E+01_JPRB, 0.12193E+01_JPRB, 0.11411E+01_JPRB, 0.10593E+01_JPRB, 0.97574E+00_JPRB ,&
&  0.89729E+00_JPRB, 0.80515E+00_JPRB, 0.76980E+00_JPRB, 0.76632E+00_JPRB, 0.77189E+00_JPRB, 0.74808E+00_JPRB, 0.74042E+00_JPRB ,&
&  0.73622E+00_JPRB, 0.73498E+00_JPRB, 0.75186E+00_JPRB, 0.73305E+00_JPRB, 0.70530E+00_JPRB, 0.67524E+00_JPRB, 0.65803E+00_JPRB ,&
&  0.64235E+00_JPRB, 0.65029E+00_JPRB, 0.65480E+00_JPRB, 0.64791E+00_JPRB, 0.63867E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.65216E+00_JPRB, 0.64465E+00_JPRB, 0.64164E+00_JPRB, 0.64995E+00_JPRB, 0.66368E+00_JPRB, 0.67981E+00_JPRB, 0.70134E+00_JPRB ,&
&  0.72177E+00_JPRB, 0.73080E+00_JPRB, 0.71950E+00_JPRB, 0.70007E+00_JPRB, 0.68360E+00_JPRB, 0.67265E+00_JPRB, 0.63600E+00_JPRB ,&
&  0.58344E+00_JPRB, 0.57716E+00_JPRB, 0.56797E+00_JPRB, 0.57050E+00_JPRB, 0.58162E+00_JPRB, 0.58997E+00_JPRB, 0.59848E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.61519E+00_JPRB, 0.61818E+00_JPRB, 0.62032E+00_JPRB, 0.62447E+00_JPRB, 0.63696E+00_JPRB, 0.64500E+00_JPRB ,&
&  0.62288E+00_JPRB, 0.60853E+00_JPRB, 0.60785E+00_JPRB, 0.61593E+00_JPRB, 0.63238E+00_JPRB, 0.65127E+00_JPRB, 0.66484E+00_JPRB ,&
&  0.67584E+00_JPRB, 0.68236E+00_JPRB, 0.68331E+00_JPRB, 0.68117E+00_JPRB, 0.67686E+00_JPRB, 0.67126E+00_JPRB, 0.66809E+00_JPRB ,&
&  0.66574E+00_JPRB, 0.66689E+00_JPRB, 0.67315E+00_JPRB, 0.67935E+00_JPRB, 0.68594E+00_JPRB, 0.72956E+00_JPRB, 0.83501E+00_JPRB ,&
&  0.96103E+00_JPRB, 0.10953E+01_JPRB, 0.11343E+01_JPRB, 0.12395E+01_JPRB, 0.13467E+01_JPRB, 0.14598E+01_JPRB, 0.15338E+01_JPRB ,&
&  0.15946E+01_JPRB, 0.15323E+01_JPRB, 0.17175E+01_JPRB, 0.18092E+01_JPRB, 0.18030E+01_JPRB, 0.18369E+01_JPRB, 0.17449E+01_JPRB ,&
&  0.16841E+01_JPRB, 0.16268E+01_JPRB, 0.16867E+01_JPRB, 0.16910E+01_JPRB, 0.16538E+01_JPRB, 0.16687E+01_JPRB, 0.16114E+01_JPRB ,&
&  0.15777E+01_JPRB, 0.15243E+01_JPRB, 0.13350E+01_JPRB, 0.11864E+01_JPRB, 0.10360E+01_JPRB, 0.10708E+01_JPRB, 0.10074E+01_JPRB ,&
&  0.92153E+00_JPRB, 0.81215E+00_JPRB, 0.81153E+00_JPRB, 0.80686E+00_JPRB, 0.82244E+00_JPRB, 0.82907E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.79927E+00_JPRB, 0.81600E+00_JPRB, 0.79448E+00_JPRB, 0.77143E+00_JPRB, 0.74366E+00_JPRB, 0.70758E+00_JPRB, 0.68598E+00_JPRB ,&
&  0.68630E+00_JPRB, 0.69086E+00_JPRB, 0.68298E+00_JPRB, 0.66851E+00_JPRB, 0.65907E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.68751E+00_JPRB, 0.68015E+00_JPRB, 0.67830E+00_JPRB, 0.68550E+00_JPRB, 0.69826E+00_JPRB, 0.71836E+00_JPRB, 0.74071E+00_JPRB ,&
&  0.75717E+00_JPRB, 0.75799E+00_JPRB, 0.73837E+00_JPRB, 0.72209E+00_JPRB, 0.71171E+00_JPRB, 0.70142E+00_JPRB, 0.69976E+00_JPRB ,&
&  0.63524E+00_JPRB, 0.60655E+00_JPRB, 0.57784E+00_JPRB, 0.57668E+00_JPRB, 0.59817E+00_JPRB, 0.60599E+00_JPRB, 0.61478E+00_JPRB ,&
&  0.62320E+00_JPRB, 0.63404E+00_JPRB, 0.63735E+00_JPRB, 0.64103E+00_JPRB, 0.64790E+00_JPRB, 0.66623E+00_JPRB, 0.68530E+00_JPRB ,&
&  0.66733E+00_JPRB, 0.65178E+00_JPRB, 0.65585E+00_JPRB, 0.66900E+00_JPRB, 0.68189E+00_JPRB, 0.70262E+00_JPRB, 0.72656E+00_JPRB ,&
&  0.73530E+00_JPRB, 0.73478E+00_JPRB, 0.72828E+00_JPRB, 0.72420E+00_JPRB, 0.72198E+00_JPRB, 0.71931E+00_JPRB, 0.72267E+00_JPRB ,&
&  0.72811E+00_JPRB, 0.73280E+00_JPRB, 0.73978E+00_JPRB, 0.74277E+00_JPRB, 0.74241E+00_JPRB, 0.81302E+00_JPRB, 0.94948E+00_JPRB ,&
&  0.11267E+01_JPRB, 0.11650E+01_JPRB, 0.12401E+01_JPRB, 0.16092E+01_JPRB, 0.18174E+01_JPRB, 0.19068E+01_JPRB, 0.18841E+01_JPRB ,&
&  0.18430E+01_JPRB, 0.17456E+01_JPRB, 0.17483E+01_JPRB, 0.17958E+01_JPRB, 0.17906E+01_JPRB, 0.18911E+01_JPRB, 0.18483E+01_JPRB ,&
&  0.17403E+01_JPRB, 0.17093E+01_JPRB, 0.17093E+01_JPRB, 0.16949E+01_JPRB, 0.16486E+01_JPRB, 0.16630E+01_JPRB, 0.16299E+01_JPRB ,&
&  0.15605E+01_JPRB, 0.13721E+01_JPRB, 0.10909E+01_JPRB, 0.10332E+01_JPRB, 0.87882E+00_JPRB, 0.91679E+00_JPRB, 0.10642E+01_JPRB ,&
&  0.10738E+01_JPRB, 0.10112E+01_JPRB, 0.95031E+00_JPRB, 0.92120E+00_JPRB, 0.91561E+00_JPRB, 0.97397E+00_JPRB, 0.93377E+00_JPRB ,&
&  0.87904E+00_JPRB, 0.89050E+00_JPRB, 0.88480E+00_JPRB, 0.85771E+00_JPRB, 0.81947E+00_JPRB, 0.78709E+00_JPRB, 0.76649E+00_JPRB ,&
&  0.76086E+00_JPRB, 0.75684E+00_JPRB, 0.74499E+00_JPRB, 0.71494E+00_JPRB, 0.69500E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.74776E+00_JPRB, 0.74107E+00_JPRB, 0.73406E+00_JPRB, 0.73886E+00_JPRB, 0.75037E+00_JPRB, 0.76745E+00_JPRB, 0.78454E+00_JPRB ,&
&  0.79067E+00_JPRB, 0.77903E+00_JPRB, 0.75544E+00_JPRB, 0.74611E+00_JPRB, 0.73739E+00_JPRB, 0.72358E+00_JPRB, 0.71935E+00_JPRB ,&
&  0.72929E+00_JPRB, 0.66477E+00_JPRB, 0.62121E+00_JPRB, 0.58957E+00_JPRB, 0.60148E+00_JPRB, 0.60838E+00_JPRB, 0.62198E+00_JPRB ,&
&  0.63236E+00_JPRB, 0.65278E+00_JPRB, 0.66123E+00_JPRB, 0.66729E+00_JPRB, 0.68021E+00_JPRB, 0.70861E+00_JPRB, 0.73571E+00_JPRB ,&
&  0.73749E+00_JPRB, 0.73960E+00_JPRB, 0.75211E+00_JPRB, 0.77598E+00_JPRB, 0.79669E+00_JPRB, 0.77502E+00_JPRB, 0.79809E+00_JPRB ,&
&  0.79931E+00_JPRB, 0.79332E+00_JPRB, 0.78066E+00_JPRB, 0.78209E+00_JPRB, 0.78831E+00_JPRB, 0.79947E+00_JPRB, 0.81342E+00_JPRB ,&
&  0.83005E+00_JPRB, 0.83184E+00_JPRB, 0.80999E+00_JPRB, 0.80403E+00_JPRB, 0.81851E+00_JPRB, 0.93678E+00_JPRB, 0.10161E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11528E+01_JPRB, 0.11633E+01_JPRB, 0.15123E+01_JPRB, 0.18621E+01_JPRB, 0.20115E+01_JPRB, 0.20592E+01_JPRB ,&
&  0.20298E+01_JPRB, 0.19683E+01_JPRB, 0.19352E+01_JPRB, 0.18616E+01_JPRB, 0.18744E+01_JPRB, 0.19298E+01_JPRB, 0.18922E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.16887E+01_JPRB, 0.16617E+01_JPRB, 0.16016E+01_JPRB, 0.15521E+01_JPRB, 0.14572E+01_JPRB, 0.14939E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.99817E+00_JPRB, 0.86283E+00_JPRB, 0.96680E+00_JPRB, 0.86509E+00_JPRB, 0.89410E+00_JPRB, 0.10024E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.10083E+01_JPRB, 0.10438E+01_JPRB, 0.10189E+01_JPRB, 0.10427E+01_JPRB, 0.10953E+01_JPRB, 0.10726E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.99323E+00_JPRB, 0.99554E+00_JPRB, 0.96395E+00_JPRB, 0.91911E+00_JPRB, 0.88786E+00_JPRB, 0.86307E+00_JPRB ,&
&  0.83821E+00_JPRB, 0.82168E+00_JPRB, 0.81212E+00_JPRB, 0.78194E+00_JPRB, 0.75649E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.83449E+00_JPRB, 0.82515E+00_JPRB, 0.81829E+00_JPRB, 0.81870E+00_JPRB, 0.82213E+00_JPRB, 0.83195E+00_JPRB, 0.83562E+00_JPRB ,&
&  0.82703E+00_JPRB, 0.79854E+00_JPRB, 0.77357E+00_JPRB, 0.77168E+00_JPRB, 0.75837E+00_JPRB, 0.74231E+00_JPRB, 0.74285E+00_JPRB ,&
&  0.75933E+00_JPRB, 0.74362E+00_JPRB, 0.66098E+00_JPRB, 0.62050E+00_JPRB, 0.58980E+00_JPRB, 0.60090E+00_JPRB, 0.61399E+00_JPRB ,&
&  0.63836E+00_JPRB, 0.65629E+00_JPRB, 0.66799E+00_JPRB, 0.68462E+00_JPRB, 0.71673E+00_JPRB, 0.75827E+00_JPRB, 0.80511E+00_JPRB ,&
&  0.82639E+00_JPRB, 0.84857E+00_JPRB, 0.84049E+00_JPRB, 0.88370E+00_JPRB, 0.89671E+00_JPRB, 0.86372E+00_JPRB, 0.88822E+00_JPRB ,&
&  0.88413E+00_JPRB, 0.87954E+00_JPRB, 0.86839E+00_JPRB, 0.87345E+00_JPRB, 0.87825E+00_JPRB, 0.86720E+00_JPRB, 0.85033E+00_JPRB ,&
&  0.83837E+00_JPRB, 0.83362E+00_JPRB, 0.82659E+00_JPRB, 0.81761E+00_JPRB, 0.84832E+00_JPRB, 0.92916E+00_JPRB, 0.96758E+00_JPRB ,&
&  0.10561E+01_JPRB, 0.94734E+00_JPRB, 0.87209E+00_JPRB, 0.11881E+01_JPRB, 0.16026E+01_JPRB, 0.17624E+01_JPRB, 0.17496E+01_JPRB ,&
&  0.18976E+01_JPRB, 0.19402E+01_JPRB, 0.19870E+01_JPRB, 0.20489E+01_JPRB, 0.20575E+01_JPRB, 0.20460E+01_JPRB, 0.19772E+01_JPRB ,&
&  0.18189E+01_JPRB, 0.17091E+01_JPRB, 0.16553E+01_JPRB, 0.16868E+01_JPRB, 0.16249E+01_JPRB, 0.15844E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.12530E+01_JPRB, 0.97568E+00_JPRB, 0.99025E+00_JPRB, 0.83836E+00_JPRB, 0.84686E+00_JPRB, 0.94730E+00_JPRB ,&
&  0.95434E+00_JPRB, 0.10165E+01_JPRB, 0.10446E+01_JPRB, 0.10301E+01_JPRB, 0.11737E+01_JPRB, 0.12344E+01_JPRB, 0.12286E+01_JPRB ,&
&  0.11995E+01_JPRB, 0.10881E+01_JPRB, 0.11100E+01_JPRB, 0.11167E+01_JPRB, 0.10908E+01_JPRB, 0.10629E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.98772E+00_JPRB, 0.94697E+00_JPRB, 0.91959E+00_JPRB, 0.88117E+00_JPRB, 0.85074E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.92697E+00_JPRB, 0.90077E+00_JPRB, 0.88437E+00_JPRB, 0.87865E+00_JPRB, 0.87805E+00_JPRB, 0.87748E+00_JPRB, 0.87160E+00_JPRB ,&
&  0.85495E+00_JPRB, 0.81826E+00_JPRB, 0.80275E+00_JPRB, 0.79637E+00_JPRB, 0.77447E+00_JPRB, 0.75775E+00_JPRB, 0.76418E+00_JPRB ,&
&  0.78518E+00_JPRB, 0.74516E+00_JPRB, 0.64815E+00_JPRB, 0.59704E+00_JPRB, 0.56922E+00_JPRB, 0.57342E+00_JPRB, 0.59358E+00_JPRB ,&
&  0.62018E+00_JPRB, 0.64126E+00_JPRB, 0.66545E+00_JPRB, 0.69771E+00_JPRB, 0.74843E+00_JPRB, 0.80266E+00_JPRB, 0.89228E+00_JPRB ,&
&  0.10008E+01_JPRB, 0.92795E+00_JPRB, 0.95206E+00_JPRB, 0.95062E+00_JPRB, 0.98337E+00_JPRB, 0.97247E+00_JPRB, 0.95181E+00_JPRB ,&
&  0.91945E+00_JPRB, 0.89913E+00_JPRB, 0.88144E+00_JPRB, 0.85992E+00_JPRB, 0.83987E+00_JPRB, 0.82923E+00_JPRB, 0.83421E+00_JPRB ,&
&  0.85997E+00_JPRB, 0.86852E+00_JPRB, 0.85377E+00_JPRB, 0.86311E+00_JPRB, 0.94284E+00_JPRB, 0.96308E+00_JPRB, 0.96076E+00_JPRB ,&
&  0.92713E+00_JPRB, 0.98223E+00_JPRB, 0.12595E+01_JPRB, 0.14800E+01_JPRB, 0.16393E+01_JPRB, 0.17820E+01_JPRB, 0.18297E+01_JPRB ,&
&  0.19306E+01_JPRB, 0.18847E+01_JPRB, 0.19271E+01_JPRB, 0.17601E+01_JPRB, 0.19799E+01_JPRB, 0.19872E+01_JPRB, 0.19494E+01_JPRB ,&
&  0.18639E+01_JPRB, 0.18377E+01_JPRB, 0.17748E+01_JPRB, 0.18246E+01_JPRB, 0.18177E+01_JPRB, 0.18279E+01_JPRB, 0.14748E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.13071E+01_JPRB, 0.11900E+01_JPRB, 0.10609E+01_JPRB, 0.94210E+00_JPRB, 0.95384E+00_JPRB, 0.97242E+00_JPRB ,&
&  0.99341E+00_JPRB, 0.10105E+01_JPRB, 0.10052E+01_JPRB, 0.10593E+01_JPRB, 0.12315E+01_JPRB, 0.13997E+01_JPRB, 0.14183E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.13408E+01_JPRB, 0.13532E+01_JPRB, 0.13321E+01_JPRB, 0.13517E+01_JPRB, 0.13192E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.11912E+01_JPRB, 0.11309E+01_JPRB, 0.10778E+01_JPRB, 0.10226E+01_JPRB, 0.96604E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.10240E+01_JPRB, 0.99333E+00_JPRB, 0.96671E+00_JPRB, 0.94835E+00_JPRB, 0.93192E+00_JPRB, 0.92057E+00_JPRB, 0.90611E+00_JPRB ,&
&  0.87802E+00_JPRB, 0.83215E+00_JPRB, 0.82836E+00_JPRB, 0.81653E+00_JPRB, 0.79725E+00_JPRB, 0.78278E+00_JPRB, 0.79191E+00_JPRB ,&
&  0.81268E+00_JPRB, 0.74075E+00_JPRB, 0.64406E+00_JPRB, 0.63094E+00_JPRB, 0.60620E+00_JPRB, 0.57584E+00_JPRB, 0.58215E+00_JPRB ,&
&  0.60485E+00_JPRB, 0.63096E+00_JPRB, 0.65767E+00_JPRB, 0.71642E+00_JPRB, 0.79532E+00_JPRB, 0.88681E+00_JPRB, 0.10040E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.10481E+01_JPRB, 0.11062E+01_JPRB, 0.10728E+01_JPRB, 0.10322E+01_JPRB, 0.98819E+00_JPRB, 0.94761E+00_JPRB ,&
&  0.91798E+00_JPRB, 0.89404E+00_JPRB, 0.87366E+00_JPRB, 0.85869E+00_JPRB, 0.85899E+00_JPRB, 0.86037E+00_JPRB, 0.86575E+00_JPRB ,&
&  0.87536E+00_JPRB, 0.86630E+00_JPRB, 0.84364E+00_JPRB, 0.92042E+00_JPRB, 0.85035E+00_JPRB, 0.82760E+00_JPRB, 0.84025E+00_JPRB ,&
&  0.10607E+01_JPRB, 0.12609E+01_JPRB, 0.13956E+01_JPRB, 0.16954E+01_JPRB, 0.14994E+01_JPRB, 0.16303E+01_JPRB, 0.19653E+01_JPRB ,&
&  0.20366E+01_JPRB, 0.20344E+01_JPRB, 0.21627E+01_JPRB, 0.17953E+01_JPRB, 0.16349E+01_JPRB, 0.21705E+01_JPRB, 0.21951E+01_JPRB ,&
&  0.21134E+01_JPRB, 0.20786E+01_JPRB, 0.20545E+01_JPRB, 0.22650E+01_JPRB, 0.16055E+01_JPRB, 0.13677E+01_JPRB, 0.11101E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.10626E+01_JPRB, 0.11903E+01_JPRB, 0.11331E+01_JPRB, 0.11254E+01_JPRB, 0.10865E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.10021E+01_JPRB, 0.99810E+00_JPRB, 0.10125E+01_JPRB, 0.11680E+01_JPRB, 0.14563E+01_JPRB, 0.14825E+01_JPRB, 0.16180E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.17464E+01_JPRB, 0.16569E+01_JPRB, 0.16597E+01_JPRB, 0.16248E+01_JPRB, 0.15528E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.13625E+01_JPRB, 0.12846E+01_JPRB, 0.12033E+01_JPRB, 0.11308E+01_JPRB, 0.10705E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.10977E+01_JPRB, 0.10541E+01_JPRB, 0.10154E+01_JPRB, 0.98251E+00_JPRB, 0.96392E+00_JPRB, 0.94725E+00_JPRB, 0.92296E+00_JPRB ,&
&  0.87749E+00_JPRB, 0.83542E+00_JPRB, 0.83621E+00_JPRB, 0.80653E+00_JPRB, 0.78740E+00_JPRB, 0.78819E+00_JPRB, 0.81484E+00_JPRB ,&
&  0.83364E+00_JPRB, 0.81112E+00_JPRB, 0.70157E+00_JPRB, 0.65026E+00_JPRB, 0.65069E+00_JPRB, 0.60915E+00_JPRB, 0.59117E+00_JPRB ,&
&  0.61747E+00_JPRB, 0.64659E+00_JPRB, 0.69966E+00_JPRB, 0.80452E+00_JPRB, 0.91991E+00_JPRB, 0.10353E+01_JPRB, 0.11593E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.11677E+01_JPRB, 0.11220E+01_JPRB, 0.10602E+01_JPRB, 0.99167E+00_JPRB, 0.94381E+00_JPRB, 0.90880E+00_JPRB ,&
&  0.88452E+00_JPRB, 0.87251E+00_JPRB, 0.87304E+00_JPRB, 0.86480E+00_JPRB, 0.87126E+00_JPRB, 0.84095E+00_JPRB, 0.83225E+00_JPRB ,&
&  0.84109E+00_JPRB, 0.85056E+00_JPRB, 0.86658E+00_JPRB, 0.98979E+00_JPRB, 0.94811E+00_JPRB, 0.88295E+00_JPRB, 0.91893E+00_JPRB ,&
&  0.10581E+01_JPRB, 0.11897E+01_JPRB, 0.13839E+01_JPRB, 0.16254E+01_JPRB, 0.19780E+01_JPRB, 0.21099E+01_JPRB, 0.22924E+01_JPRB ,&
&  0.18798E+01_JPRB, 0.16836E+01_JPRB, 0.17382E+01_JPRB, 0.16823E+01_JPRB, 0.21553E+01_JPRB, 0.29809E+01_JPRB, 0.30356E+01_JPRB ,&
&  0.28937E+01_JPRB, 0.27416E+01_JPRB, 0.27209E+01_JPRB, 0.22093E+01_JPRB, 0.15924E+01_JPRB, 0.12475E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.15021E+01_JPRB, 0.15701E+01_JPRB, 0.14360E+01_JPRB, 0.12352E+01_JPRB, 0.10891E+01_JPRB, 0.12495E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.11270E+01_JPRB, 0.10840E+01_JPRB, 0.13162E+01_JPRB, 0.16452E+01_JPRB, 0.18186E+01_JPRB, 0.18865E+01_JPRB, 0.21819E+01_JPRB ,&
&  0.22906E+01_JPRB, 0.23773E+01_JPRB, 0.22776E+01_JPRB, 0.21267E+01_JPRB, 0.19557E+01_JPRB, 0.17671E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.14919E+01_JPRB, 0.13810E+01_JPRB, 0.12846E+01_JPRB, 0.12045E+01_JPRB, 0.11453E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.11543E+01_JPRB, 0.10967E+01_JPRB, 0.10463E+01_JPRB, 0.10151E+01_JPRB, 0.98912E+00_JPRB, 0.96064E+00_JPRB, 0.91987E+00_JPRB ,&
&  0.85914E+00_JPRB, 0.84582E+00_JPRB, 0.83718E+00_JPRB, 0.80347E+00_JPRB, 0.79213E+00_JPRB, 0.79904E+00_JPRB, 0.82138E+00_JPRB ,&
&  0.84596E+00_JPRB, 0.89371E+00_JPRB, 0.77430E+00_JPRB, 0.72358E+00_JPRB, 0.71102E+00_JPRB, 0.68880E+00_JPRB, 0.63077E+00_JPRB ,&
&  0.65644E+00_JPRB, 0.69747E+00_JPRB, 0.77824E+00_JPRB, 0.91774E+00_JPRB, 0.10949E+01_JPRB, 0.11509E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.11673E+01_JPRB, 0.11064E+01_JPRB, 0.10448E+01_JPRB, 0.97557E+00_JPRB, 0.94117E+00_JPRB, 0.92383E+00_JPRB ,&
&  0.91640E+00_JPRB, 0.92904E+00_JPRB, 0.93091E+00_JPRB, 0.95066E+00_JPRB, 0.92015E+00_JPRB, 0.87689E+00_JPRB, 0.86025E+00_JPRB ,&
&  0.84936E+00_JPRB, 0.86131E+00_JPRB, 0.89763E+00_JPRB, 0.10362E+01_JPRB, 0.11059E+01_JPRB, 0.10560E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.14519E+01_JPRB, 0.15497E+01_JPRB, 0.17799E+01_JPRB, 0.20039E+01_JPRB, 0.21260E+01_JPRB, 0.23126E+01_JPRB ,&
&  0.21878E+01_JPRB, 0.23433E+01_JPRB, 0.29190E+01_JPRB, 0.36048E+01_JPRB, 0.31718E+01_JPRB, 0.31928E+01_JPRB, 0.43933E+01_JPRB ,&
&  0.32044E+01_JPRB, 0.31470E+01_JPRB, 0.32638E+01_JPRB, 0.31589E+01_JPRB, 0.13532E+01_JPRB, 0.10900E+01_JPRB, 0.15290E+01_JPRB ,&
&  0.15914E+01_JPRB, 0.10894E+01_JPRB, 0.87109E+00_JPRB, 0.99928E+00_JPRB, 0.84328E+00_JPRB, 0.98544E+00_JPRB, 0.13446E+01_JPRB ,&
&  0.13956E+01_JPRB, 0.14191E+01_JPRB, 0.18840E+01_JPRB, 0.21234E+01_JPRB, 0.21123E+01_JPRB, 0.27481E+01_JPRB, 0.29415E+01_JPRB ,&
&  0.30153E+01_JPRB, 0.27769E+01_JPRB, 0.27672E+01_JPRB, 0.25181E+01_JPRB, 0.22240E+01_JPRB, 0.19597E+01_JPRB, 0.17482E+01_JPRB ,&
&  0.15632E+01_JPRB, 0.14311E+01_JPRB, 0.13421E+01_JPRB, 0.12615E+01_JPRB, 0.12008E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.11345E+01_JPRB, 0.10797E+01_JPRB, 0.10270E+01_JPRB, 0.99360E+00_JPRB, 0.97315E+00_JPRB, 0.94559E+00_JPRB, 0.90235E+00_JPRB ,&
&  0.85549E+00_JPRB, 0.86686E+00_JPRB, 0.83581E+00_JPRB, 0.80950E+00_JPRB, 0.79174E+00_JPRB, 0.79453E+00_JPRB, 0.81873E+00_JPRB ,&
&  0.84026E+00_JPRB, 0.87330E+00_JPRB, 0.86008E+00_JPRB, 0.81215E+00_JPRB, 0.79422E+00_JPRB, 0.76161E+00_JPRB, 0.72083E+00_JPRB ,&
&  0.74695E+00_JPRB, 0.81395E+00_JPRB, 0.91158E+00_JPRB, 0.10510E+01_JPRB, 0.11268E+01_JPRB, 0.11483E+01_JPRB, 0.12155E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.11412E+01_JPRB, 0.10877E+01_JPRB, 0.10343E+01_JPRB, 0.99057E+00_JPRB, 0.96868E+00_JPRB, 0.95059E+00_JPRB ,&
&  0.94546E+00_JPRB, 0.94883E+00_JPRB, 0.95974E+00_JPRB, 0.92770E+00_JPRB, 0.89269E+00_JPRB, 0.86651E+00_JPRB, 0.84657E+00_JPRB ,&
&  0.84449E+00_JPRB, 0.89360E+00_JPRB, 0.97118E+00_JPRB, 0.10907E+01_JPRB, 0.13129E+01_JPRB, 0.13993E+01_JPRB, 0.12938E+01_JPRB ,&
&  0.12012E+01_JPRB, 0.13218E+01_JPRB, 0.17167E+01_JPRB, 0.17240E+01_JPRB, 0.16919E+01_JPRB, 0.18108E+01_JPRB, 0.18811E+01_JPRB ,&
&  0.21731E+01_JPRB, 0.27991E+01_JPRB, 0.30080E+01_JPRB, 0.37896E+01_JPRB, 0.40762E+01_JPRB, 0.30346E+01_JPRB, 0.34816E+01_JPRB ,&
&  0.34375E+01_JPRB, 0.29904E+01_JPRB, 0.22842E+01_JPRB, 0.16906E+01_JPRB, 0.18176E+01_JPRB, 0.15903E+01_JPRB, 0.10205E+01_JPRB ,&
&  0.91048E+00_JPRB, 0.86011E+00_JPRB, 0.82119E+00_JPRB, 0.82199E+00_JPRB, 0.81248E+00_JPRB, 0.10010E+01_JPRB, 0.13221E+01_JPRB ,&
&  0.17538E+01_JPRB, 0.20512E+01_JPRB, 0.23464E+01_JPRB, 0.24284E+01_JPRB, 0.22590E+01_JPRB, 0.29168E+01_JPRB, 0.33744E+01_JPRB ,&
&  0.28310E+01_JPRB, 0.25791E+01_JPRB, 0.25129E+01_JPRB, 0.22862E+01_JPRB, 0.20538E+01_JPRB, 0.18473E+01_JPRB, 0.16628E+01_JPRB ,&
&  0.15128E+01_JPRB, 0.14089E+01_JPRB, 0.13317E+01_JPRB, 0.12599E+01_JPRB, 0.11947E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.10873E+01_JPRB, 0.10490E+01_JPRB, 0.10224E+01_JPRB, 0.99850E+00_JPRB, 0.97074E+00_JPRB, 0.94755E+00_JPRB, 0.89423E+00_JPRB ,&
&  0.87723E+00_JPRB, 0.86967E+00_JPRB, 0.82204E+00_JPRB, 0.78800E+00_JPRB, 0.77087E+00_JPRB, 0.77805E+00_JPRB, 0.80292E+00_JPRB ,&
&  0.82222E+00_JPRB, 0.86017E+00_JPRB, 0.93996E+00_JPRB, 0.94076E+00_JPRB, 0.10072E+01_JPRB, 0.92947E+00_JPRB, 0.93038E+00_JPRB ,&
&  0.93569E+00_JPRB, 0.10089E+01_JPRB, 0.11121E+01_JPRB, 0.11762E+01_JPRB, 0.11827E+01_JPRB, 0.12336E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.11499E+01_JPRB, 0.10799E+01_JPRB, 0.10306E+01_JPRB, 0.10078E+01_JPRB, 0.97617E+00_JPRB, 0.95645E+00_JPRB ,&
&  0.95532E+00_JPRB, 0.96659E+00_JPRB, 0.93445E+00_JPRB, 0.92269E+00_JPRB, 0.91798E+00_JPRB, 0.90113E+00_JPRB, 0.88796E+00_JPRB ,&
&  0.91530E+00_JPRB, 0.98888E+00_JPRB, 0.10851E+01_JPRB, 0.11962E+01_JPRB, 0.12150E+01_JPRB, 0.10532E+01_JPRB, 0.11930E+01_JPRB ,&
&  0.13161E+01_JPRB, 0.15644E+01_JPRB, 0.16298E+01_JPRB, 0.18449E+01_JPRB, 0.17261E+01_JPRB, 0.16797E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.23571E+01_JPRB, 0.33692E+01_JPRB, 0.30378E+01_JPRB, 0.35645E+01_JPRB, 0.48002E+01_JPRB, 0.35952E+01_JPRB, 0.30246E+01_JPRB ,&
&  0.32645E+01_JPRB, 0.27309E+01_JPRB, 0.22653E+01_JPRB, 0.20006E+01_JPRB, 0.28355E+01_JPRB, 0.32653E+01_JPRB, 0.14297E+01_JPRB ,&
&  0.10825E+01_JPRB, 0.10203E+01_JPRB, 0.97266E+00_JPRB, 0.90361E+00_JPRB, 0.89956E+00_JPRB, 0.11452E+01_JPRB, 0.17049E+01_JPRB ,&
&  0.19765E+01_JPRB, 0.25406E+01_JPRB, 0.27676E+01_JPRB, 0.27003E+01_JPRB, 0.22908E+01_JPRB, 0.26608E+01_JPRB, 0.25703E+01_JPRB ,&
&  0.27698E+01_JPRB, 0.26427E+01_JPRB, 0.24069E+01_JPRB, 0.21742E+01_JPRB, 0.19450E+01_JPRB, 0.17387E+01_JPRB, 0.15679E+01_JPRB ,&
&  0.14252E+01_JPRB, 0.13246E+01_JPRB, 0.12503E+01_JPRB, 0.11928E+01_JPRB, 0.11347E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.10858E+01_JPRB, 0.10521E+01_JPRB, 0.10207E+01_JPRB, 0.98670E+00_JPRB, 0.96795E+00_JPRB, 0.93923E+00_JPRB, 0.89070E+00_JPRB ,&
&  0.90791E+00_JPRB, 0.85959E+00_JPRB, 0.80668E+00_JPRB, 0.77398E+00_JPRB, 0.76533E+00_JPRB, 0.77170E+00_JPRB, 0.78984E+00_JPRB ,&
&  0.82280E+00_JPRB, 0.85538E+00_JPRB, 0.94839E+00_JPRB, 0.11677E+01_JPRB, 0.12820E+01_JPRB, 0.12033E+01_JPRB, 0.12792E+01_JPRB ,&
&  0.12306E+01_JPRB, 0.12600E+01_JPRB, 0.13321E+01_JPRB, 0.13016E+01_JPRB, 0.12945E+01_JPRB, 0.13162E+01_JPRB, 0.12293E+01_JPRB ,&
&  0.11633E+01_JPRB, 0.11604E+01_JPRB, 0.10954E+01_JPRB, 0.10580E+01_JPRB, 0.10248E+01_JPRB, 0.10134E+01_JPRB, 0.99729E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.97655E+00_JPRB, 0.96380E+00_JPRB, 0.96724E+00_JPRB, 0.97497E+00_JPRB, 0.97342E+00_JPRB, 0.99261E+00_JPRB ,&
&  0.10620E+01_JPRB, 0.11336E+01_JPRB, 0.12305E+01_JPRB, 0.13090E+01_JPRB, 0.10826E+01_JPRB, 0.12432E+01_JPRB, 0.12854E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.14121E+01_JPRB, 0.13952E+01_JPRB, 0.15512E+01_JPRB, 0.17076E+01_JPRB, 0.17022E+01_JPRB, 0.18912E+01_JPRB ,&
&  0.27178E+01_JPRB, 0.28011E+01_JPRB, 0.29461E+01_JPRB, 0.35916E+01_JPRB, 0.40743E+01_JPRB, 0.54662E+01_JPRB, 0.27714E+01_JPRB ,&
&  0.20976E+01_JPRB, 0.25158E+01_JPRB, 0.24245E+01_JPRB, 0.24258E+01_JPRB, 0.30956E+01_JPRB, 0.43878E+01_JPRB, 0.37031E+01_JPRB ,&
&  0.13762E+01_JPRB, 0.10623E+01_JPRB, 0.12176E+01_JPRB, 0.12235E+01_JPRB, 0.11041E+01_JPRB, 0.14187E+01_JPRB, 0.28379E+01_JPRB ,&
&  0.26829E+01_JPRB, 0.30255E+01_JPRB, 0.29664E+01_JPRB, 0.25447E+01_JPRB, 0.22564E+01_JPRB, 0.24821E+01_JPRB, 0.27469E+01_JPRB ,&
&  0.26178E+01_JPRB, 0.23572E+01_JPRB, 0.21440E+01_JPRB, 0.19656E+01_JPRB, 0.17534E+01_JPRB, 0.15516E+01_JPRB, 0.13866E+01_JPRB ,&
&  0.12736E+01_JPRB, 0.12106E+01_JPRB, 0.11768E+01_JPRB, 0.11491E+01_JPRB, 0.11182E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.10395E+01_JPRB, 0.10131E+01_JPRB, 0.99229E+00_JPRB, 0.97783E+00_JPRB, 0.97674E+00_JPRB, 0.94832E+00_JPRB, 0.91367E+00_JPRB ,&
&  0.94234E+00_JPRB, 0.85988E+00_JPRB, 0.80928E+00_JPRB, 0.77200E+00_JPRB, 0.77029E+00_JPRB, 0.78495E+00_JPRB, 0.79945E+00_JPRB ,&
&  0.81618E+00_JPRB, 0.86068E+00_JPRB, 0.98511E+00_JPRB, 0.12716E+01_JPRB, 0.14674E+01_JPRB, 0.14072E+01_JPRB, 0.14821E+01_JPRB ,&
&  0.15968E+01_JPRB, 0.15321E+01_JPRB, 0.14105E+01_JPRB, 0.13375E+01_JPRB, 0.13900E+01_JPRB, 0.13728E+01_JPRB, 0.12739E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.12035E+01_JPRB, 0.11509E+01_JPRB, 0.11137E+01_JPRB, 0.10958E+01_JPRB, 0.10863E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10453E+01_JPRB, 0.10450E+01_JPRB, 0.10492E+01_JPRB, 0.10802E+01_JPRB, 0.10840E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.12996E+01_JPRB, 0.13625E+01_JPRB, 0.12953E+01_JPRB, 0.13031E+01_JPRB, 0.13079E+01_JPRB, 0.13622E+01_JPRB ,&
&  0.13645E+01_JPRB, 0.14099E+01_JPRB, 0.14901E+01_JPRB, 0.15773E+01_JPRB, 0.17602E+01_JPRB, 0.19217E+01_JPRB, 0.22130E+01_JPRB ,&
&  0.27829E+01_JPRB, 0.30655E+01_JPRB, 0.30468E+01_JPRB, 0.33110E+01_JPRB, 0.37767E+01_JPRB, 0.46417E+01_JPRB, 0.35600E+01_JPRB ,&
&  0.26414E+01_JPRB, 0.21966E+01_JPRB, 0.24452E+01_JPRB, 0.32091E+01_JPRB, 0.37641E+01_JPRB, 0.47105E+01_JPRB, 0.61916E+01_JPRB ,&
&  0.67900E+01_JPRB, 0.36584E+01_JPRB, 0.28873E+01_JPRB, 0.33983E+01_JPRB, 0.25508E+01_JPRB, 0.21411E+01_JPRB, 0.26702E+01_JPRB ,&
&  0.29806E+01_JPRB, 0.30978E+01_JPRB, 0.29129E+01_JPRB, 0.23661E+01_JPRB, 0.22939E+01_JPRB, 0.25637E+01_JPRB, 0.25622E+01_JPRB ,&
&  0.22432E+01_JPRB, 0.20292E+01_JPRB, 0.18108E+01_JPRB, 0.15835E+01_JPRB, 0.14005E+01_JPRB, 0.12720E+01_JPRB, 0.11966E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.11439E+01_JPRB, 0.11280E+01_JPRB, 0.11009E+01_JPRB, 0.10697E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.99876E+00_JPRB, 0.99465E+00_JPRB, 0.10039E+01_JPRB, 0.10076E+01_JPRB, 0.99578E+00_JPRB, 0.95888E+00_JPRB, 0.96305E+00_JPRB ,&
&  0.98895E+00_JPRB, 0.88561E+00_JPRB, 0.81919E+00_JPRB, 0.79020E+00_JPRB, 0.79760E+00_JPRB, 0.82087E+00_JPRB, 0.83374E+00_JPRB ,&
&  0.85756E+00_JPRB, 0.93484E+00_JPRB, 0.11587E+01_JPRB, 0.14216E+01_JPRB, 0.16961E+01_JPRB, 0.21507E+01_JPRB, 0.15575E+01_JPRB ,&
&  0.16472E+01_JPRB, 0.15771E+01_JPRB, 0.13961E+01_JPRB, 0.13897E+01_JPRB, 0.14411E+01_JPRB, 0.14478E+01_JPRB, 0.13532E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.12837E+01_JPRB, 0.12529E+01_JPRB, 0.12213E+01_JPRB, 0.12070E+01_JPRB, 0.12220E+01_JPRB, 0.11945E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.12011E+01_JPRB, 0.11749E+01_JPRB, 0.12027E+01_JPRB, 0.12368E+01_JPRB, 0.12565E+01_JPRB, 0.13361E+01_JPRB ,&
&  0.14027E+01_JPRB, 0.14354E+01_JPRB, 0.14456E+01_JPRB, 0.13705E+01_JPRB, 0.14115E+01_JPRB, 0.14432E+01_JPRB, 0.14991E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.13319E+01_JPRB, 0.15203E+01_JPRB, 0.17872E+01_JPRB, 0.20125E+01_JPRB, 0.21016E+01_JPRB, 0.24075E+01_JPRB ,&
&  0.30641E+01_JPRB, 0.35838E+01_JPRB, 0.38076E+01_JPRB, 0.31626E+01_JPRB, 0.34502E+01_JPRB, 0.40688E+01_JPRB, 0.41922E+01_JPRB ,&
&  0.39451E+01_JPRB, 0.33200E+01_JPRB, 0.35076E+01_JPRB, 0.37495E+01_JPRB, 0.39818E+01_JPRB, 0.47501E+01_JPRB, 0.48821E+01_JPRB ,&
&  0.57985E+01_JPRB, 0.72171E+01_JPRB, 0.59773E+01_JPRB, 0.45726E+01_JPRB, 0.37359E+01_JPRB, 0.31248E+01_JPRB, 0.30206E+01_JPRB ,&
&  0.32720E+01_JPRB, 0.30272E+01_JPRB, 0.26816E+01_JPRB, 0.27197E+01_JPRB, 0.23585E+01_JPRB, 0.23486E+01_JPRB, 0.21322E+01_JPRB ,&
&  0.17730E+01_JPRB, 0.15219E+01_JPRB, 0.12883E+01_JPRB, 0.11515E+01_JPRB, 0.10791E+01_JPRB, 0.10621E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.10582E+01_JPRB, 0.10499E+01_JPRB, 0.10393E+01_JPRB, 0.10184E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.10056E+01_JPRB, 0.10131E+01_JPRB, 0.10351E+01_JPRB, 0.10422E+01_JPRB, 0.10256E+01_JPRB, 0.10174E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.89132E+00_JPRB, 0.86395E+00_JPRB, 0.86486E+00_JPRB, 0.88328E+00_JPRB, 0.89231E+00_JPRB, 0.90143E+00_JPRB ,&
&  0.95562E+00_JPRB, 0.11046E+01_JPRB, 0.13477E+01_JPRB, 0.15593E+01_JPRB, 0.17707E+01_JPRB, 0.20461E+01_JPRB, 0.19912E+01_JPRB ,&
&  0.15452E+01_JPRB, 0.17135E+01_JPRB, 0.14447E+01_JPRB, 0.15410E+01_JPRB, 0.16003E+01_JPRB, 0.17184E+01_JPRB, 0.15877E+01_JPRB ,&
&  0.14637E+01_JPRB, 0.14457E+01_JPRB, 0.14988E+01_JPRB, 0.14449E+01_JPRB, 0.13777E+01_JPRB, 0.13361E+01_JPRB, 0.13723E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.13823E+01_JPRB, 0.13794E+01_JPRB, 0.14218E+01_JPRB, 0.14520E+01_JPRB, 0.14619E+01_JPRB, 0.14869E+01_JPRB ,&
&  0.15031E+01_JPRB, 0.15119E+01_JPRB, 0.14852E+01_JPRB, 0.15097E+01_JPRB, 0.15366E+01_JPRB, 0.16213E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.16594E+01_JPRB, 0.15813E+01_JPRB, 0.17247E+01_JPRB, 0.19096E+01_JPRB, 0.20824E+01_JPRB, 0.23986E+01_JPRB, 0.26379E+01_JPRB ,&
&  0.32719E+01_JPRB, 0.33919E+01_JPRB, 0.40429E+01_JPRB, 0.29504E+01_JPRB, 0.31090E+01_JPRB, 0.36179E+01_JPRB, 0.35117E+01_JPRB ,&
&  0.29267E+01_JPRB, 0.31322E+01_JPRB, 0.33595E+01_JPRB, 0.37532E+01_JPRB, 0.40414E+01_JPRB, 0.43308E+01_JPRB, 0.40171E+01_JPRB ,&
&  0.41169E+01_JPRB, 0.44744E+01_JPRB, 0.42567E+01_JPRB, 0.34790E+01_JPRB, 0.31762E+01_JPRB, 0.27935E+01_JPRB, 0.30329E+01_JPRB ,&
&  0.32325E+01_JPRB, 0.30565E+01_JPRB, 0.29977E+01_JPRB, 0.28467E+01_JPRB, 0.20838E+01_JPRB, 0.18443E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.11079E+01_JPRB, 0.97042E+00_JPRB, 0.95134E+00_JPRB, 0.96099E+00_JPRB, 0.97149E+00_JPRB, 0.10004E+01_JPRB ,&
&  0.10045E+01_JPRB, 0.10061E+01_JPRB, 0.10147E+01_JPRB, 0.10156E+01_JPRB, 0.10065E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.10328E+01_JPRB, 0.10402E+01_JPRB, 0.10654E+01_JPRB, 0.10958E+01_JPRB, 0.11229E+01_JPRB, 0.11636E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.93243E+00_JPRB, 0.95115E+00_JPRB, 0.96383E+00_JPRB, 0.97887E+00_JPRB, 0.97211E+00_JPRB, 0.10034E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.12875E+01_JPRB, 0.14981E+01_JPRB, 0.17302E+01_JPRB, 0.19616E+01_JPRB, 0.22402E+01_JPRB, 0.22823E+01_JPRB ,&
&  0.17965E+01_JPRB, 0.16396E+01_JPRB, 0.15965E+01_JPRB, 0.16918E+01_JPRB, 0.17207E+01_JPRB, 0.18735E+01_JPRB, 0.19430E+01_JPRB ,&
&  0.18720E+01_JPRB, 0.17895E+01_JPRB, 0.18641E+01_JPRB, 0.16911E+01_JPRB, 0.14493E+01_JPRB, 0.14867E+01_JPRB, 0.15460E+01_JPRB ,&
&  0.15823E+01_JPRB, 0.15980E+01_JPRB, 0.16241E+01_JPRB, 0.16715E+01_JPRB, 0.16952E+01_JPRB, 0.16711E+01_JPRB, 0.16405E+01_JPRB ,&
&  0.16377E+01_JPRB, 0.16621E+01_JPRB, 0.16644E+01_JPRB, 0.16451E+01_JPRB, 0.17205E+01_JPRB, 0.18132E+01_JPRB, 0.18806E+01_JPRB ,&
&  0.19305E+01_JPRB, 0.20216E+01_JPRB, 0.21225E+01_JPRB, 0.22707E+01_JPRB, 0.22986E+01_JPRB, 0.27090E+01_JPRB, 0.29512E+01_JPRB ,&
&  0.32717E+01_JPRB, 0.33774E+01_JPRB, 0.37350E+01_JPRB, 0.26702E+01_JPRB, 0.30728E+01_JPRB, 0.32606E+01_JPRB, 0.32165E+01_JPRB ,&
&  0.29476E+01_JPRB, 0.31498E+01_JPRB, 0.32596E+01_JPRB, 0.34284E+01_JPRB, 0.36230E+01_JPRB, 0.34086E+01_JPRB, 0.34487E+01_JPRB ,&
&  0.32242E+01_JPRB, 0.35812E+01_JPRB, 0.33983E+01_JPRB, 0.31644E+01_JPRB, 0.27602E+01_JPRB, 0.25849E+01_JPRB, 0.27103E+01_JPRB ,&
&  0.29349E+01_JPRB, 0.28063E+01_JPRB, 0.25997E+01_JPRB, 0.21899E+01_JPRB, 0.15560E+01_JPRB, 0.13191E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.93527E+00_JPRB, 0.90751E+00_JPRB, 0.93741E+00_JPRB, 0.95160E+00_JPRB, 0.10002E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.99432E+00_JPRB, 0.10101E+01_JPRB, 0.10286E+01_JPRB, 0.10364E+01_JPRB, 0.10356E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.10123E+01_JPRB, 0.10059E+01_JPRB, 0.10014E+01_JPRB, 0.10094E+01_JPRB, 0.10307E+01_JPRB, 0.10418E+01_JPRB, 0.10017E+01_JPRB ,&
&  0.99183E+00_JPRB, 0.10022E+01_JPRB, 0.10440E+01_JPRB, 0.10632E+01_JPRB, 0.10703E+01_JPRB, 0.10822E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.14340E+01_JPRB, 0.16290E+01_JPRB, 0.17690E+01_JPRB, 0.18924E+01_JPRB, 0.20476E+01_JPRB, 0.22042E+01_JPRB ,&
&  0.19171E+01_JPRB, 0.14968E+01_JPRB, 0.17068E+01_JPRB, 0.18318E+01_JPRB, 0.18526E+01_JPRB, 0.21135E+01_JPRB, 0.24443E+01_JPRB ,&
&  0.19951E+01_JPRB, 0.18755E+01_JPRB, 0.19060E+01_JPRB, 0.17988E+01_JPRB, 0.15840E+01_JPRB, 0.16459E+01_JPRB, 0.17243E+01_JPRB ,&
&  0.17819E+01_JPRB, 0.18177E+01_JPRB, 0.18835E+01_JPRB, 0.19445E+01_JPRB, 0.19626E+01_JPRB, 0.19302E+01_JPRB, 0.18854E+01_JPRB ,&
&  0.19100E+01_JPRB, 0.19572E+01_JPRB, 0.20128E+01_JPRB, 0.20146E+01_JPRB, 0.20656E+01_JPRB, 0.21656E+01_JPRB, 0.22595E+01_JPRB ,&
&  0.23468E+01_JPRB, 0.23745E+01_JPRB, 0.25255E+01_JPRB, 0.28165E+01_JPRB, 0.30778E+01_JPRB, 0.28040E+01_JPRB, 0.30912E+01_JPRB ,&
&  0.30661E+01_JPRB, 0.29900E+01_JPRB, 0.28603E+01_JPRB, 0.33734E+01_JPRB, 0.23588E+01_JPRB, 0.27296E+01_JPRB, 0.26006E+01_JPRB ,&
&  0.30380E+01_JPRB, 0.29961E+01_JPRB, 0.30412E+01_JPRB, 0.31545E+01_JPRB, 0.32827E+01_JPRB, 0.29726E+01_JPRB, 0.29179E+01_JPRB ,&
&  0.29771E+01_JPRB, 0.30173E+01_JPRB, 0.29843E+01_JPRB, 0.29642E+01_JPRB, 0.27303E+01_JPRB, 0.26620E+01_JPRB, 0.25767E+01_JPRB ,&
&  0.26218E+01_JPRB, 0.22879E+01_JPRB, 0.19345E+01_JPRB, 0.15266E+01_JPRB, 0.11337E+01_JPRB, 0.10190E+01_JPRB, 0.98210E+00_JPRB ,&
&  0.98135E+00_JPRB, 0.10025E+01_JPRB, 0.97637E+00_JPRB, 0.96835E+00_JPRB, 0.98471E+00_JPRB, 0.10245E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.10262E+01_JPRB, 0.10299E+01_JPRB, 0.10434E+01_JPRB, 0.10445E+01_JPRB, 0.10343E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.10046E+01_JPRB, 0.99313E+00_JPRB, 0.97554E+00_JPRB, 0.98599E+00_JPRB, 0.99185E+00_JPRB, 0.99035E+00_JPRB, 0.98331E+00_JPRB ,&
&  0.99976E+00_JPRB, 0.10444E+01_JPRB, 0.10944E+01_JPRB, 0.11259E+01_JPRB, 0.11361E+01_JPRB, 0.11552E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.14607E+01_JPRB, 0.16387E+01_JPRB, 0.17657E+01_JPRB, 0.18681E+01_JPRB, 0.19160E+01_JPRB, 0.20128E+01_JPRB ,&
&  0.19840E+01_JPRB, 0.19917E+01_JPRB, 0.19118E+01_JPRB, 0.16169E+01_JPRB, 0.15742E+01_JPRB, 0.19942E+01_JPRB, 0.19717E+01_JPRB ,&
&  0.18364E+01_JPRB, 0.17802E+01_JPRB, 0.17588E+01_JPRB, 0.17487E+01_JPRB, 0.17517E+01_JPRB, 0.18287E+01_JPRB, 0.19197E+01_JPRB ,&
&  0.19810E+01_JPRB, 0.20647E+01_JPRB, 0.21449E+01_JPRB, 0.22010E+01_JPRB, 0.22299E+01_JPRB, 0.22760E+01_JPRB, 0.22938E+01_JPRB ,&
&  0.23855E+01_JPRB, 0.24964E+01_JPRB, 0.26085E+01_JPRB, 0.26510E+01_JPRB, 0.26197E+01_JPRB, 0.27061E+01_JPRB, 0.27840E+01_JPRB ,&
&  0.28635E+01_JPRB, 0.28623E+01_JPRB, 0.28451E+01_JPRB, 0.29783E+01_JPRB, 0.30376E+01_JPRB, 0.26296E+01_JPRB, 0.26271E+01_JPRB ,&
&  0.27982E+01_JPRB, 0.27265E+01_JPRB, 0.24096E+01_JPRB, 0.30453E+01_JPRB, 0.18975E+01_JPRB, 0.24406E+01_JPRB, 0.28468E+01_JPRB ,&
&  0.27818E+01_JPRB, 0.27572E+01_JPRB, 0.28024E+01_JPRB, 0.28534E+01_JPRB, 0.28366E+01_JPRB, 0.26056E+01_JPRB, 0.27213E+01_JPRB ,&
&  0.28183E+01_JPRB, 0.27662E+01_JPRB, 0.27252E+01_JPRB, 0.26830E+01_JPRB, 0.25872E+01_JPRB, 0.23521E+01_JPRB, 0.20188E+01_JPRB ,&
&  0.16923E+01_JPRB, 0.18530E+01_JPRB, 0.15832E+01_JPRB, 0.13770E+01_JPRB, 0.10687E+01_JPRB, 0.92765E+00_JPRB, 0.95553E+00_JPRB ,&
&  0.98794E+00_JPRB, 0.99472E+00_JPRB, 0.98535E+00_JPRB, 0.10013E+01_JPRB, 0.10114E+01_JPRB, 0.10241E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.10342E+01_JPRB, 0.10400E+01_JPRB, 0.10326E+01_JPRB, 0.10199E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.96118E+00_JPRB, 0.94893E+00_JPRB, 0.93597E+00_JPRB, 0.96989E+00_JPRB, 0.96453E+00_JPRB, 0.95371E+00_JPRB, 0.96166E+00_JPRB ,&
&  0.97743E+00_JPRB, 0.10038E+01_JPRB, 0.10399E+01_JPRB, 0.10795E+01_JPRB, 0.11298E+01_JPRB, 0.12007E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.13447E+01_JPRB, 0.14249E+01_JPRB, 0.15601E+01_JPRB, 0.17309E+01_JPRB, 0.19503E+01_JPRB, 0.20643E+01_JPRB, 0.20689E+01_JPRB ,&
&  0.19526E+01_JPRB, 0.20121E+01_JPRB, 0.21057E+01_JPRB, 0.21911E+01_JPRB, 0.19414E+01_JPRB, 0.21513E+01_JPRB, 0.23307E+01_JPRB ,&
&  0.23005E+01_JPRB, 0.20505E+01_JPRB, 0.18609E+01_JPRB, 0.18615E+01_JPRB, 0.18850E+01_JPRB, 0.19860E+01_JPRB, 0.20976E+01_JPRB ,&
&  0.21910E+01_JPRB, 0.22937E+01_JPRB, 0.23665E+01_JPRB, 0.24416E+01_JPRB, 0.25467E+01_JPRB, 0.27048E+01_JPRB, 0.27869E+01_JPRB ,&
&  0.28718E+01_JPRB, 0.30346E+01_JPRB, 0.31200E+01_JPRB, 0.30853E+01_JPRB, 0.31770E+01_JPRB, 0.32396E+01_JPRB, 0.32888E+01_JPRB ,&
&  0.33758E+01_JPRB, 0.31981E+01_JPRB, 0.32381E+01_JPRB, 0.33203E+01_JPRB, 0.31600E+01_JPRB, 0.28038E+01_JPRB, 0.27076E+01_JPRB ,&
&  0.27685E+01_JPRB, 0.27416E+01_JPRB, 0.19636E+01_JPRB, 0.27029E+01_JPRB, 0.32745E+01_JPRB, 0.28153E+01_JPRB, 0.27094E+01_JPRB ,&
&  0.25840E+01_JPRB, 0.26107E+01_JPRB, 0.26932E+01_JPRB, 0.27772E+01_JPRB, 0.28104E+01_JPRB, 0.28001E+01_JPRB, 0.25920E+01_JPRB ,&
&  0.27852E+01_JPRB, 0.26910E+01_JPRB, 0.25358E+01_JPRB, 0.23792E+01_JPRB, 0.21738E+01_JPRB, 0.18500E+01_JPRB, 0.15294E+01_JPRB ,&
&  0.13409E+01_JPRB, 0.16173E+01_JPRB, 0.14747E+01_JPRB, 0.14582E+01_JPRB, 0.13331E+01_JPRB, 0.91881E+00_JPRB, 0.96927E+00_JPRB ,&
&  0.98537E+00_JPRB, 0.99304E+00_JPRB, 0.99076E+00_JPRB, 0.97467E+00_JPRB, 0.98026E+00_JPRB, 0.99026E+00_JPRB, 0.99925E+00_JPRB ,&
&  0.10005E+01_JPRB, 0.99381E+00_JPRB, 0.99368E+00_JPRB, 0.98063E+00_JPRB, 0.97312E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.95007E+00_JPRB, 0.94902E+00_JPRB, 0.94027E+00_JPRB, 0.95393E+00_JPRB, 0.94483E+00_JPRB, 0.96268E+00_JPRB, 0.94285E+00_JPRB ,&
&  0.92526E+00_JPRB, 0.92310E+00_JPRB, 0.93177E+00_JPRB, 0.94239E+00_JPRB, 0.97343E+00_JPRB, 0.10343E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.11913E+01_JPRB, 0.12297E+01_JPRB, 0.13453E+01_JPRB, 0.16076E+01_JPRB, 0.18758E+01_JPRB, 0.19613E+01_JPRB ,&
&  0.20833E+01_JPRB, 0.22657E+01_JPRB, 0.24264E+01_JPRB, 0.26140E+01_JPRB, 0.26698E+01_JPRB, 0.27384E+01_JPRB, 0.29373E+01_JPRB ,&
&  0.25203E+01_JPRB, 0.20226E+01_JPRB, 0.19073E+01_JPRB, 0.19082E+01_JPRB, 0.19777E+01_JPRB, 0.20823E+01_JPRB, 0.21819E+01_JPRB ,&
&  0.22799E+01_JPRB, 0.23924E+01_JPRB, 0.24968E+01_JPRB, 0.26366E+01_JPRB, 0.28307E+01_JPRB, 0.30521E+01_JPRB, 0.31975E+01_JPRB ,&
&  0.32924E+01_JPRB, 0.34193E+01_JPRB, 0.35725E+01_JPRB, 0.34613E+01_JPRB, 0.38948E+01_JPRB, 0.40369E+01_JPRB, 0.40967E+01_JPRB ,&
&  0.40252E+01_JPRB, 0.37928E+01_JPRB, 0.37965E+01_JPRB, 0.37770E+01_JPRB, 0.37305E+01_JPRB, 0.33643E+01_JPRB, 0.31002E+01_JPRB ,&
&  0.30445E+01_JPRB, 0.25708E+01_JPRB, 0.18421E+01_JPRB, 0.20221E+01_JPRB, 0.22784E+01_JPRB, 0.20396E+01_JPRB, 0.25676E+01_JPRB ,&
&  0.24792E+01_JPRB, 0.24330E+01_JPRB, 0.24644E+01_JPRB, 0.25331E+01_JPRB, 0.26680E+01_JPRB, 0.26112E+01_JPRB, 0.25052E+01_JPRB ,&
&  0.24977E+01_JPRB, 0.23511E+01_JPRB, 0.21455E+01_JPRB, 0.19008E+01_JPRB, 0.16532E+01_JPRB, 0.14440E+01_JPRB, 0.13298E+01_JPRB ,&
&  0.13002E+01_JPRB, 0.13208E+01_JPRB, 0.12744E+01_JPRB, 0.11595E+01_JPRB, 0.96319E+00_JPRB, 0.87339E+00_JPRB, 0.89289E+00_JPRB ,&
&  0.93828E+00_JPRB, 0.94361E+00_JPRB, 0.94361E+00_JPRB, 0.93075E+00_JPRB, 0.92664E+00_JPRB, 0.92332E+00_JPRB, 0.92540E+00_JPRB ,&
&  0.94119E+00_JPRB, 0.96315E+00_JPRB, 0.96400E+00_JPRB, 0.96153E+00_JPRB, 0.95810E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.92617E+00_JPRB, 0.92283E+00_JPRB, 0.92194E+00_JPRB, 0.93751E+00_JPRB, 0.94989E+00_JPRB, 0.93524E+00_JPRB, 0.94224E+00_JPRB ,&
&  0.91796E+00_JPRB, 0.89849E+00_JPRB, 0.90892E+00_JPRB, 0.91457E+00_JPRB, 0.93335E+00_JPRB, 0.95333E+00_JPRB, 0.96516E+00_JPRB ,&
&  0.96335E+00_JPRB, 0.95303E+00_JPRB, 0.90894E+00_JPRB, 0.91321E+00_JPRB, 0.10068E+01_JPRB, 0.11741E+01_JPRB, 0.14960E+01_JPRB ,&
&  0.18593E+01_JPRB, 0.20518E+01_JPRB, 0.21979E+01_JPRB, 0.22577E+01_JPRB, 0.22170E+01_JPRB, 0.21365E+01_JPRB, 0.28504E+01_JPRB ,&
&  0.29413E+01_JPRB, 0.19967E+01_JPRB, 0.20094E+01_JPRB, 0.18827E+01_JPRB, 0.19565E+01_JPRB, 0.20058E+01_JPRB, 0.20906E+01_JPRB ,&
&  0.21699E+01_JPRB, 0.22926E+01_JPRB, 0.24622E+01_JPRB, 0.26848E+01_JPRB, 0.29087E+01_JPRB, 0.31432E+01_JPRB, 0.33295E+01_JPRB ,&
&  0.34588E+01_JPRB, 0.35758E+01_JPRB, 0.37819E+01_JPRB, 0.38615E+01_JPRB, 0.39176E+01_JPRB, 0.41316E+01_JPRB, 0.43568E+01_JPRB ,&
&  0.45184E+01_JPRB, 0.48135E+01_JPRB, 0.42404E+01_JPRB, 0.41817E+01_JPRB, 0.43063E+01_JPRB, 0.36430E+01_JPRB, 0.34305E+01_JPRB ,&
&  0.33022E+01_JPRB, 0.25344E+01_JPRB, 0.16674E+01_JPRB, 0.17811E+01_JPRB, 0.19419E+01_JPRB, 0.20431E+01_JPRB, 0.22119E+01_JPRB ,&
&  0.21238E+01_JPRB, 0.20239E+01_JPRB, 0.20452E+01_JPRB, 0.21762E+01_JPRB, 0.22016E+01_JPRB, 0.21254E+01_JPRB, 0.20098E+01_JPRB ,&
&  0.19244E+01_JPRB, 0.17728E+01_JPRB, 0.16833E+01_JPRB, 0.15353E+01_JPRB, 0.14289E+01_JPRB, 0.13205E+01_JPRB, 0.11917E+01_JPRB ,&
&  0.12833E+01_JPRB, 0.14545E+01_JPRB, 0.17841E+01_JPRB, 0.15317E+01_JPRB, 0.98445E+00_JPRB, 0.88569E+00_JPRB, 0.86298E+00_JPRB ,&
&  0.87753E+00_JPRB, 0.88659E+00_JPRB, 0.90914E+00_JPRB, 0.90850E+00_JPRB, 0.89574E+00_JPRB, 0.88284E+00_JPRB, 0.87294E+00_JPRB ,&
&  0.89445E+00_JPRB, 0.92351E+00_JPRB, 0.93823E+00_JPRB, 0.93019E+00_JPRB, 0.92333E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.98400E+00_JPRB, 0.98910E+00_JPRB, 0.10116E+01_JPRB, 0.10184E+01_JPRB, 0.10175E+01_JPRB, 0.10243E+01_JPRB, 0.10242E+01_JPRB ,&
&  0.10160E+01_JPRB, 0.10221E+01_JPRB, 0.10311E+01_JPRB, 0.10427E+01_JPRB, 0.10591E+01_JPRB, 0.10749E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.10731E+01_JPRB, 0.11029E+01_JPRB, 0.11245E+01_JPRB, 0.11198E+01_JPRB, 0.11132E+01_JPRB, 0.11380E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.12426E+01_JPRB, 0.12577E+01_JPRB, 0.12473E+01_JPRB, 0.12975E+01_JPRB, 0.15296E+01_JPRB, 0.23613E+01_JPRB ,&
&  0.26147E+01_JPRB, 0.20170E+01_JPRB, 0.18057E+01_JPRB, 0.16470E+01_JPRB, 0.16973E+01_JPRB, 0.18216E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.18445E+01_JPRB, 0.19128E+01_JPRB, 0.19861E+01_JPRB, 0.20577E+01_JPRB, 0.21596E+01_JPRB, 0.22360E+01_JPRB, 0.22778E+01_JPRB ,&
&  0.22430E+01_JPRB, 0.22357E+01_JPRB, 0.23175E+01_JPRB, 0.24637E+01_JPRB, 0.28711E+01_JPRB, 0.34744E+01_JPRB, 0.33515E+01_JPRB ,&
&  0.33622E+01_JPRB, 0.37080E+01_JPRB, 0.39791E+01_JPRB, 0.41092E+01_JPRB, 0.39063E+01_JPRB, 0.34745E+01_JPRB, 0.31127E+01_JPRB ,&
&  0.27430E+01_JPRB, 0.22040E+01_JPRB, 0.17801E+01_JPRB, 0.20039E+01_JPRB, 0.20288E+01_JPRB, 0.20673E+01_JPRB, 0.18933E+01_JPRB ,&
&  0.17821E+01_JPRB, 0.15925E+01_JPRB, 0.15749E+01_JPRB, 0.16594E+01_JPRB, 0.17166E+01_JPRB, 0.16609E+01_JPRB, 0.15636E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.14186E+01_JPRB, 0.14640E+01_JPRB, 0.14508E+01_JPRB, 0.14596E+01_JPRB, 0.12238E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.14238E+01_JPRB, 0.12138E+01_JPRB, 0.15348E+01_JPRB, 0.92167E+00_JPRB, 0.80342E+00_JPRB, 0.80771E+00_JPRB ,&
&  0.83128E+00_JPRB, 0.86388E+00_JPRB, 0.87390E+00_JPRB, 0.96427E+00_JPRB, 0.95362E+00_JPRB, 0.94928E+00_JPRB, 0.96344E+00_JPRB ,&
&  0.94215E+00_JPRB, 0.94367E+00_JPRB, 0.94887E+00_JPRB, 0.96579E+00_JPRB, 0.97592E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.10522E+01_JPRB, 0.10582E+01_JPRB, 0.10684E+01_JPRB, 0.10839E+01_JPRB, 0.10959E+01_JPRB, 0.11061E+01_JPRB, 0.11093E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11044E+01_JPRB, 0.11165E+01_JPRB, 0.11349E+01_JPRB, 0.11534E+01_JPRB, 0.11693E+01_JPRB, 0.11900E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.12007E+01_JPRB, 0.12285E+01_JPRB, 0.12582E+01_JPRB, 0.12735E+01_JPRB, 0.12739E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.12733E+01_JPRB, 0.12859E+01_JPRB, 0.13238E+01_JPRB, 0.13524E+01_JPRB, 0.13904E+01_JPRB, 0.14872E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.19694E+01_JPRB, 0.21735E+01_JPRB, 0.19242E+01_JPRB, 0.17383E+01_JPRB, 0.17393E+01_JPRB, 0.16640E+01_JPRB, 0.16070E+01_JPRB ,&
&  0.15463E+01_JPRB, 0.15248E+01_JPRB, 0.14710E+01_JPRB, 0.14797E+01_JPRB, 0.14674E+01_JPRB, 0.15045E+01_JPRB, 0.15098E+01_JPRB ,&
&  0.15835E+01_JPRB, 0.16720E+01_JPRB, 0.18515E+01_JPRB, 0.20120E+01_JPRB, 0.22230E+01_JPRB, 0.23738E+01_JPRB, 0.21453E+01_JPRB ,&
&  0.21061E+01_JPRB, 0.23702E+01_JPRB, 0.27800E+01_JPRB, 0.30672E+01_JPRB, 0.28812E+01_JPRB, 0.25620E+01_JPRB, 0.22889E+01_JPRB ,&
&  0.20137E+01_JPRB, 0.18376E+01_JPRB, 0.20831E+01_JPRB, 0.20342E+01_JPRB, 0.20041E+01_JPRB, 0.18548E+01_JPRB, 0.16380E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.11874E+01_JPRB, 0.11594E+01_JPRB, 0.11547E+01_JPRB, 0.11724E+01_JPRB, 0.11401E+01_JPRB, 0.11254E+01_JPRB ,&
&  0.10785E+01_JPRB, 0.10635E+01_JPRB, 0.12294E+01_JPRB, 0.14374E+01_JPRB, 0.16011E+01_JPRB, 0.20010E+01_JPRB, 0.18015E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.11750E+01_JPRB, 0.75156E+00_JPRB, 0.10322E+01_JPRB, 0.76057E+00_JPRB, 0.74889E+00_JPRB, 0.82701E+00_JPRB ,&
&  0.93934E+00_JPRB, 0.10383E+01_JPRB, 0.11097E+01_JPRB, 0.10784E+01_JPRB, 0.10110E+01_JPRB, 0.10035E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10241E+01_JPRB, 0.10300E+01_JPRB, 0.10358E+01_JPRB, 0.10378E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.10840E+01_JPRB, 0.10990E+01_JPRB, 0.11183E+01_JPRB, 0.11415E+01_JPRB, 0.11628E+01_JPRB, 0.11829E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.12293E+01_JPRB, 0.12586E+01_JPRB, 0.12916E+01_JPRB, 0.13084E+01_JPRB, 0.13284E+01_JPRB, 0.13372E+01_JPRB ,&
&  0.13686E+01_JPRB, 0.14062E+01_JPRB, 0.14281E+01_JPRB, 0.14542E+01_JPRB, 0.14723E+01_JPRB, 0.14932E+01_JPRB, 0.15207E+01_JPRB ,&
&  0.15428E+01_JPRB, 0.15942E+01_JPRB, 0.16735E+01_JPRB, 0.17371E+01_JPRB, 0.18313E+01_JPRB, 0.20346E+01_JPRB, 0.18400E+01_JPRB ,&
&  0.21691E+01_JPRB, 0.21789E+01_JPRB, 0.20640E+01_JPRB, 0.18552E+01_JPRB, 0.17862E+01_JPRB, 0.17208E+01_JPRB, 0.16699E+01_JPRB ,&
&  0.16370E+01_JPRB, 0.15752E+01_JPRB, 0.15989E+01_JPRB, 0.15972E+01_JPRB, 0.15971E+01_JPRB, 0.17120E+01_JPRB, 0.18133E+01_JPRB ,&
&  0.18675E+01_JPRB, 0.19262E+01_JPRB, 0.20115E+01_JPRB, 0.20639E+01_JPRB, 0.21510E+01_JPRB, 0.21967E+01_JPRB, 0.20894E+01_JPRB ,&
&  0.19388E+01_JPRB, 0.18754E+01_JPRB, 0.18236E+01_JPRB, 0.18850E+01_JPRB, 0.18444E+01_JPRB, 0.16125E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.15338E+01_JPRB, 0.14852E+01_JPRB, 0.15481E+01_JPRB, 0.21086E+01_JPRB, 0.19582E+01_JPRB, 0.17512E+01_JPRB, 0.14386E+01_JPRB ,&
&  0.12272E+01_JPRB, 0.97293E+00_JPRB, 0.88948E+00_JPRB, 0.89657E+00_JPRB, 0.87495E+00_JPRB, 0.86966E+00_JPRB, 0.84828E+00_JPRB ,&
&  0.81401E+00_JPRB, 0.84243E+00_JPRB, 0.10072E+01_JPRB, 0.12631E+01_JPRB, 0.15860E+01_JPRB, 0.17155E+01_JPRB, 0.22341E+01_JPRB ,&
&  0.14454E+01_JPRB, 0.84798E+00_JPRB, 0.92649E+00_JPRB, 0.86181E+00_JPRB, 0.77330E+00_JPRB, 0.70763E+00_JPRB, 0.82851E+00_JPRB ,&
&  0.10395E+01_JPRB, 0.11062E+01_JPRB, 0.11303E+01_JPRB, 0.10854E+01_JPRB, 0.10810E+01_JPRB, 0.10862E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10773E+01_JPRB, 0.10802E+01_JPRB, 0.10759E+01_JPRB, 0.10754E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.11240E+01_JPRB, 0.11450E+01_JPRB, 0.11739E+01_JPRB, 0.12098E+01_JPRB, 0.12322E+01_JPRB, 0.12638E+01_JPRB, 0.12804E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.13380E+01_JPRB, 0.13745E+01_JPRB, 0.14139E+01_JPRB, 0.14532E+01_JPRB, 0.14899E+01_JPRB, 0.15266E+01_JPRB ,&
&  0.15574E+01_JPRB, 0.15908E+01_JPRB, 0.16103E+01_JPRB, 0.16225E+01_JPRB, 0.16182E+01_JPRB, 0.16189E+01_JPRB, 0.16080E+01_JPRB ,&
&  0.15959E+01_JPRB, 0.16057E+01_JPRB, 0.16304E+01_JPRB, 0.16518E+01_JPRB, 0.16942E+01_JPRB, 0.18676E+01_JPRB, 0.14731E+01_JPRB ,&
&  0.20872E+01_JPRB, 0.21594E+01_JPRB, 0.21598E+01_JPRB, 0.20170E+01_JPRB, 0.19498E+01_JPRB, 0.19792E+01_JPRB, 0.20540E+01_JPRB ,&
&  0.21674E+01_JPRB, 0.23189E+01_JPRB, 0.24631E+01_JPRB, 0.23750E+01_JPRB, 0.22415E+01_JPRB, 0.23034E+01_JPRB, 0.23991E+01_JPRB ,&
&  0.24532E+01_JPRB, 0.24747E+01_JPRB, 0.25036E+01_JPRB, 0.24832E+01_JPRB, 0.24679E+01_JPRB, 0.24414E+01_JPRB, 0.23686E+01_JPRB ,&
&  0.22496E+01_JPRB, 0.20868E+01_JPRB, 0.18770E+01_JPRB, 0.20412E+01_JPRB, 0.20551E+01_JPRB, 0.17130E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.13106E+01_JPRB, 0.15782E+01_JPRB, 0.21073E+01_JPRB, 0.18377E+01_JPRB, 0.15787E+01_JPRB, 0.13921E+01_JPRB ,&
&  0.11132E+01_JPRB, 0.91040E+00_JPRB, 0.80274E+00_JPRB, 0.79939E+00_JPRB, 0.79844E+00_JPRB, 0.81027E+00_JPRB, 0.79409E+00_JPRB ,&
&  0.77673E+00_JPRB, 0.78771E+00_JPRB, 0.89248E+00_JPRB, 0.10474E+01_JPRB, 0.12824E+01_JPRB, 0.13477E+01_JPRB, 0.21047E+01_JPRB ,&
&  0.13727E+01_JPRB, 0.10952E+01_JPRB, 0.14658E+01_JPRB, 0.68239E+00_JPRB, 0.79239E+00_JPRB, 0.86903E+00_JPRB, 0.97922E+00_JPRB ,&
&  0.10030E+01_JPRB, 0.13084E+01_JPRB, 0.15976E+01_JPRB, 0.14207E+01_JPRB, 0.12912E+01_JPRB, 0.11848E+01_JPRB, 0.11441E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.11295E+01_JPRB, 0.11307E+01_JPRB, 0.11269E+01_JPRB, 0.11195E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.11963E+01_JPRB, 0.12063E+01_JPRB, 0.12072E+01_JPRB, 0.12468E+01_JPRB, 0.12529E+01_JPRB, 0.12768E+01_JPRB, 0.12850E+01_JPRB ,&
&  0.13149E+01_JPRB, 0.13425E+01_JPRB, 0.13766E+01_JPRB, 0.14268E+01_JPRB, 0.14572E+01_JPRB, 0.14876E+01_JPRB, 0.15322E+01_JPRB ,&
&  0.15760E+01_JPRB, 0.16290E+01_JPRB, 0.16402E+01_JPRB, 0.16410E+01_JPRB, 0.16241E+01_JPRB, 0.16230E+01_JPRB, 0.16151E+01_JPRB ,&
&  0.16132E+01_JPRB, 0.16173E+01_JPRB, 0.16420E+01_JPRB, 0.16803E+01_JPRB, 0.17301E+01_JPRB, 0.18477E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.20267E+01_JPRB, 0.22464E+01_JPRB, 0.22205E+01_JPRB, 0.22193E+01_JPRB, 0.21372E+01_JPRB, 0.22115E+01_JPRB, 0.23465E+01_JPRB ,&
&  0.25314E+01_JPRB, 0.27179E+01_JPRB, 0.28013E+01_JPRB, 0.28308E+01_JPRB, 0.28401E+01_JPRB, 0.29587E+01_JPRB, 0.30774E+01_JPRB ,&
&  0.31586E+01_JPRB, 0.32125E+01_JPRB, 0.32751E+01_JPRB, 0.33136E+01_JPRB, 0.33647E+01_JPRB, 0.34277E+01_JPRB, 0.34306E+01_JPRB ,&
&  0.33692E+01_JPRB, 0.33118E+01_JPRB, 0.33521E+01_JPRB, 0.35647E+01_JPRB, 0.35357E+01_JPRB, 0.30198E+01_JPRB, 0.22806E+01_JPRB ,&
&  0.14189E+01_JPRB, 0.11133E+01_JPRB, 0.14997E+01_JPRB, 0.19939E+01_JPRB, 0.17090E+01_JPRB, 0.14623E+01_JPRB, 0.12484E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.92097E+00_JPRB, 0.79904E+00_JPRB, 0.79352E+00_JPRB, 0.82080E+00_JPRB, 0.88155E+00_JPRB, 0.86077E+00_JPRB ,&
&  0.88401E+00_JPRB, 0.91272E+00_JPRB, 0.94039E+00_JPRB, 0.10291E+01_JPRB, 0.12376E+01_JPRB, 0.13952E+01_JPRB, 0.13186E+01_JPRB ,&
&  0.15044E+01_JPRB, 0.11481E+01_JPRB, 0.10803E+01_JPRB, 0.11604E+01_JPRB, 0.10838E+01_JPRB, 0.12647E+01_JPRB, 0.15567E+01_JPRB ,&
&  0.20161E+01_JPRB, 0.19377E+01_JPRB, 0.25332E+01_JPRB, 0.38371E+01_JPRB, 0.30195E+01_JPRB, 0.21838E+01_JPRB, 0.12229E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11965E+01_JPRB, 0.12129E+01_JPRB, 0.12039E+01_JPRB, 0.11973E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.11336E+01_JPRB, 0.11231E+01_JPRB, 0.11305E+01_JPRB, 0.11362E+01_JPRB, 0.11362E+01_JPRB, 0.11506E+01_JPRB, 0.11777E+01_JPRB ,&
&  0.11956E+01_JPRB, 0.12399E+01_JPRB, 0.13097E+01_JPRB, 0.13628E+01_JPRB, 0.14331E+01_JPRB, 0.14607E+01_JPRB, 0.14498E+01_JPRB ,&
&  0.14613E+01_JPRB, 0.15162E+01_JPRB, 0.15678E+01_JPRB, 0.15988E+01_JPRB, 0.15789E+01_JPRB, 0.15838E+01_JPRB, 0.16371E+01_JPRB ,&
&  0.17184E+01_JPRB, 0.17833E+01_JPRB, 0.18444E+01_JPRB, 0.19282E+01_JPRB, 0.19641E+01_JPRB, 0.18987E+01_JPRB, 0.13568E+01_JPRB ,&
&  0.19170E+01_JPRB, 0.22321E+01_JPRB, 0.23647E+01_JPRB, 0.23663E+01_JPRB, 0.23993E+01_JPRB, 0.23399E+01_JPRB, 0.24660E+01_JPRB ,&
&  0.26577E+01_JPRB, 0.28489E+01_JPRB, 0.28592E+01_JPRB, 0.27712E+01_JPRB, 0.31965E+01_JPRB, 0.33302E+01_JPRB, 0.35249E+01_JPRB ,&
&  0.36904E+01_JPRB, 0.38387E+01_JPRB, 0.39816E+01_JPRB, 0.41484E+01_JPRB, 0.42976E+01_JPRB, 0.44321E+01_JPRB, 0.45400E+01_JPRB ,&
&  0.45815E+01_JPRB, 0.45968E+01_JPRB, 0.47159E+01_JPRB, 0.45374E+01_JPRB, 0.44282E+01_JPRB, 0.45953E+01_JPRB, 0.38750E+01_JPRB ,&
&  0.21503E+01_JPRB, 0.13410E+01_JPRB, 0.16527E+01_JPRB, 0.18345E+01_JPRB, 0.15453E+01_JPRB, 0.13397E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.91361E+00_JPRB, 0.87023E+00_JPRB, 0.91748E+00_JPRB, 0.10089E+01_JPRB, 0.10971E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.10790E+01_JPRB, 0.11011E+01_JPRB, 0.11612E+01_JPRB, 0.12969E+01_JPRB, 0.15635E+01_JPRB, 0.19302E+01_JPRB ,&
&  0.18353E+01_JPRB, 0.12498E+01_JPRB, 0.12665E+01_JPRB, 0.12900E+01_JPRB, 0.13600E+01_JPRB, 0.14969E+01_JPRB, 0.17932E+01_JPRB ,&
&  0.22652E+01_JPRB, 0.27621E+01_JPRB, 0.26634E+01_JPRB, 0.22240E+01_JPRB, 0.27722E+01_JPRB, 0.18872E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.10850E+01_JPRB, 0.11065E+01_JPRB, 0.11333E+01_JPRB, 0.11380E+01_JPRB, 0.11391E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.97741E+00_JPRB, 0.96227E+00_JPRB, 0.95353E+00_JPRB, 0.94604E+00_JPRB, 0.95941E+00_JPRB, 0.99905E+00_JPRB, 0.10462E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.11601E+01_JPRB, 0.12377E+01_JPRB, 0.12819E+01_JPRB, 0.13390E+01_JPRB, 0.13566E+01_JPRB, 0.13649E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.13977E+01_JPRB, 0.14477E+01_JPRB, 0.14979E+01_JPRB, 0.15361E+01_JPRB, 0.16005E+01_JPRB, 0.16831E+01_JPRB ,&
&  0.17870E+01_JPRB, 0.18843E+01_JPRB, 0.19976E+01_JPRB, 0.21638E+01_JPRB, 0.22473E+01_JPRB, 0.22967E+01_JPRB, 0.23691E+01_JPRB ,&
&  0.13154E+01_JPRB, 0.20339E+01_JPRB, 0.22443E+01_JPRB, 0.22850E+01_JPRB, 0.25196E+01_JPRB, 0.26072E+01_JPRB, 0.26902E+01_JPRB ,&
&  0.28564E+01_JPRB, 0.28385E+01_JPRB, 0.28411E+01_JPRB, 0.29337E+01_JPRB, 0.32270E+01_JPRB, 0.33818E+01_JPRB, 0.35559E+01_JPRB ,&
&  0.36532E+01_JPRB, 0.37431E+01_JPRB, 0.39206E+01_JPRB, 0.41263E+01_JPRB, 0.43464E+01_JPRB, 0.45665E+01_JPRB, 0.48387E+01_JPRB ,&
&  0.50495E+01_JPRB, 0.52272E+01_JPRB, 0.55317E+01_JPRB, 0.50653E+01_JPRB, 0.47367E+01_JPRB, 0.50702E+01_JPRB, 0.46376E+01_JPRB ,&
&  0.31198E+01_JPRB, 0.16987E+01_JPRB, 0.16580E+01_JPRB, 0.18448E+01_JPRB, 0.14912E+01_JPRB, 0.13187E+01_JPRB, 0.12105E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.94709E+00_JPRB, 0.10133E+01_JPRB, 0.11033E+01_JPRB, 0.12140E+01_JPRB, 0.13106E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.12264E+01_JPRB, 0.11683E+01_JPRB, 0.11586E+01_JPRB, 0.11811E+01_JPRB, 0.12154E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12269E+01_JPRB, 0.11998E+01_JPRB, 0.12235E+01_JPRB, 0.12542E+01_JPRB, 0.12734E+01_JPRB, 0.12720E+01_JPRB, 0.13987E+01_JPRB ,&
&  0.16232E+01_JPRB, 0.18295E+01_JPRB, 0.20712E+01_JPRB, 0.20031E+01_JPRB, 0.17969E+01_JPRB, 0.13125E+01_JPRB, 0.91132E+00_JPRB ,&
&  0.93740E+00_JPRB, 0.96340E+00_JPRB, 0.97228E+00_JPRB, 0.99098E+00_JPRB, 0.98515E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.87937E+00_JPRB, 0.87257E+00_JPRB, 0.86650E+00_JPRB, 0.87460E+00_JPRB, 0.88655E+00_JPRB, 0.94124E+00_JPRB, 0.10307E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11823E+01_JPRB, 0.12366E+01_JPRB, 0.12739E+01_JPRB, 0.12914E+01_JPRB, 0.12733E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.12992E+01_JPRB, 0.13151E+01_JPRB, 0.13586E+01_JPRB, 0.14202E+01_JPRB, 0.14815E+01_JPRB, 0.15496E+01_JPRB, 0.16304E+01_JPRB ,&
&  0.17306E+01_JPRB, 0.18698E+01_JPRB, 0.20219E+01_JPRB, 0.22346E+01_JPRB, 0.24237E+01_JPRB, 0.25744E+01_JPRB, 0.26082E+01_JPRB ,&
&  0.12257E+01_JPRB, 0.13892E+01_JPRB, 0.20772E+01_JPRB, 0.20835E+01_JPRB, 0.23005E+01_JPRB, 0.25787E+01_JPRB, 0.29205E+01_JPRB ,&
&  0.28546E+01_JPRB, 0.25460E+01_JPRB, 0.25945E+01_JPRB, 0.30095E+01_JPRB, 0.30437E+01_JPRB, 0.31872E+01_JPRB, 0.33182E+01_JPRB ,&
&  0.33696E+01_JPRB, 0.33796E+01_JPRB, 0.34440E+01_JPRB, 0.36140E+01_JPRB, 0.38529E+01_JPRB, 0.41109E+01_JPRB, 0.44169E+01_JPRB ,&
&  0.47374E+01_JPRB, 0.50949E+01_JPRB, 0.54467E+01_JPRB, 0.46420E+01_JPRB, 0.49917E+01_JPRB, 0.54570E+01_JPRB, 0.58045E+01_JPRB ,&
&  0.43632E+01_JPRB, 0.25221E+01_JPRB, 0.20132E+01_JPRB, 0.20656E+01_JPRB, 0.16370E+01_JPRB, 0.15419E+01_JPRB, 0.13176E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.11574E+01_JPRB, 0.12120E+01_JPRB, 0.13036E+01_JPRB, 0.14064E+01_JPRB, 0.14884E+01_JPRB, 0.15051E+01_JPRB ,&
&  0.14223E+01_JPRB, 0.13655E+01_JPRB, 0.13330E+01_JPRB, 0.13269E+01_JPRB, 0.13352E+01_JPRB, 0.13415E+01_JPRB, 0.13305E+01_JPRB ,&
&  0.13131E+01_JPRB, 0.13047E+01_JPRB, 0.13397E+01_JPRB, 0.13610E+01_JPRB, 0.13768E+01_JPRB, 0.13880E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.13307E+01_JPRB, 0.12789E+01_JPRB, 0.13444E+01_JPRB, 0.12282E+01_JPRB, 0.10805E+01_JPRB, 0.93191E+00_JPRB, 0.79876E+00_JPRB ,&
&  0.82361E+00_JPRB, 0.83730E+00_JPRB, 0.83965E+00_JPRB, 0.85266E+00_JPRB, 0.87276E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.78584E+00_JPRB, 0.79350E+00_JPRB, 0.78763E+00_JPRB, 0.80989E+00_JPRB, 0.81433E+00_JPRB, 0.89260E+00_JPRB, 0.96758E+00_JPRB ,&
&  0.10886E+01_JPRB, 0.11815E+01_JPRB, 0.12458E+01_JPRB, 0.12742E+01_JPRB, 0.12640E+01_JPRB, 0.12267E+01_JPRB, 0.11927E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.12197E+01_JPRB, 0.12414E+01_JPRB, 0.12759E+01_JPRB, 0.13161E+01_JPRB, 0.13727E+01_JPRB, 0.14336E+01_JPRB ,&
&  0.15214E+01_JPRB, 0.16534E+01_JPRB, 0.18260E+01_JPRB, 0.20741E+01_JPRB, 0.24042E+01_JPRB, 0.28524E+01_JPRB, 0.33214E+01_JPRB ,&
&  0.26612E+01_JPRB, 0.13217E+01_JPRB, 0.19648E+01_JPRB, 0.19305E+01_JPRB, 0.18242E+01_JPRB, 0.22048E+01_JPRB, 0.26655E+01_JPRB ,&
&  0.25647E+01_JPRB, 0.23974E+01_JPRB, 0.23544E+01_JPRB, 0.27575E+01_JPRB, 0.28225E+01_JPRB, 0.29418E+01_JPRB, 0.30376E+01_JPRB ,&
&  0.31102E+01_JPRB, 0.31288E+01_JPRB, 0.31426E+01_JPRB, 0.32826E+01_JPRB, 0.35153E+01_JPRB, 0.38070E+01_JPRB, 0.41151E+01_JPRB ,&
&  0.44654E+01_JPRB, 0.48604E+01_JPRB, 0.53334E+01_JPRB, 0.46209E+01_JPRB, 0.52156E+01_JPRB, 0.52552E+01_JPRB, 0.45900E+01_JPRB ,&
&  0.42484E+01_JPRB, 0.28694E+01_JPRB, 0.21859E+01_JPRB, 0.22427E+01_JPRB, 0.19074E+01_JPRB, 0.14729E+01_JPRB, 0.14517E+01_JPRB ,&
&  0.13735E+01_JPRB, 0.13662E+01_JPRB, 0.13622E+01_JPRB, 0.14645E+01_JPRB, 0.15283E+01_JPRB, 0.15740E+01_JPRB, 0.15598E+01_JPRB ,&
&  0.15454E+01_JPRB, 0.14902E+01_JPRB, 0.14650E+01_JPRB, 0.14628E+01_JPRB, 0.14613E+01_JPRB, 0.14826E+01_JPRB, 0.14780E+01_JPRB ,&
&  0.14634E+01_JPRB, 0.14728E+01_JPRB, 0.15181E+01_JPRB, 0.15679E+01_JPRB, 0.15798E+01_JPRB, 0.14855E+01_JPRB, 0.14018E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.12890E+01_JPRB, 0.11562E+01_JPRB, 0.10118E+01_JPRB, 0.94821E+00_JPRB, 0.85815E+00_JPRB, 0.78654E+00_JPRB ,&
&  0.79552E+00_JPRB, 0.79344E+00_JPRB, 0.78237E+00_JPRB, 0.78484E+00_JPRB, 0.78769E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.84833E+00_JPRB, 0.83983E+00_JPRB, 0.80670E+00_JPRB, 0.79047E+00_JPRB, 0.78928E+00_JPRB, 0.82376E+00_JPRB, 0.86751E+00_JPRB ,&
&  0.93541E+00_JPRB, 0.10323E+01_JPRB, 0.11173E+01_JPRB, 0.11669E+01_JPRB, 0.11666E+01_JPRB, 0.11794E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11311E+01_JPRB, 0.11476E+01_JPRB, 0.11569E+01_JPRB, 0.11766E+01_JPRB, 0.12051E+01_JPRB, 0.12363E+01_JPRB ,&
&  0.12541E+01_JPRB, 0.12774E+01_JPRB, 0.13144E+01_JPRB, 0.14527E+01_JPRB, 0.16937E+01_JPRB, 0.20711E+01_JPRB, 0.27583E+01_JPRB ,&
&  0.41952E+01_JPRB, 0.42778E+01_JPRB, 0.21428E+01_JPRB, 0.22992E+01_JPRB, 0.20148E+01_JPRB, 0.21919E+01_JPRB, 0.21665E+01_JPRB ,&
&  0.23356E+01_JPRB, 0.21702E+01_JPRB, 0.20934E+01_JPRB, 0.24034E+01_JPRB, 0.24900E+01_JPRB, 0.26095E+01_JPRB, 0.27056E+01_JPRB ,&
&  0.28358E+01_JPRB, 0.29027E+01_JPRB, 0.28752E+01_JPRB, 0.29209E+01_JPRB, 0.31229E+01_JPRB, 0.33990E+01_JPRB, 0.37196E+01_JPRB ,&
&  0.41031E+01_JPRB, 0.44686E+01_JPRB, 0.49522E+01_JPRB, 0.46144E+01_JPRB, 0.51664E+01_JPRB, 0.47618E+01_JPRB, 0.38037E+01_JPRB ,&
&  0.31782E+01_JPRB, 0.28025E+01_JPRB, 0.25523E+01_JPRB, 0.24556E+01_JPRB, 0.21764E+01_JPRB, 0.16647E+01_JPRB, 0.16114E+01_JPRB ,&
&  0.15317E+01_JPRB, 0.14880E+01_JPRB, 0.14730E+01_JPRB, 0.15044E+01_JPRB, 0.15680E+01_JPRB, 0.15942E+01_JPRB, 0.16061E+01_JPRB ,&
&  0.15927E+01_JPRB, 0.15538E+01_JPRB, 0.15344E+01_JPRB, 0.15288E+01_JPRB, 0.15293E+01_JPRB, 0.15529E+01_JPRB, 0.15613E+01_JPRB ,&
&  0.15689E+01_JPRB, 0.15950E+01_JPRB, 0.16424E+01_JPRB, 0.16893E+01_JPRB, 0.16487E+01_JPRB, 0.15413E+01_JPRB, 0.14263E+01_JPRB ,&
&  0.13831E+01_JPRB, 0.13783E+01_JPRB, 0.12800E+01_JPRB, 0.12000E+01_JPRB, 0.11114E+01_JPRB, 0.10670E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.10028E+01_JPRB, 0.99376E+00_JPRB, 0.95401E+00_JPRB, 0.84596E+00_JPRB, 0.84554E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.96810E+00_JPRB, 0.92193E+00_JPRB, 0.91903E+00_JPRB, 0.91708E+00_JPRB, 0.90778E+00_JPRB, 0.91280E+00_JPRB, 0.86389E+00_JPRB ,&
&  0.84776E+00_JPRB, 0.89105E+00_JPRB, 0.96044E+00_JPRB, 0.10062E+01_JPRB, 0.10360E+01_JPRB, 0.10428E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10220E+01_JPRB, 0.10464E+01_JPRB, 0.10681E+01_JPRB, 0.10815E+01_JPRB, 0.11027E+01_JPRB, 0.11207E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11654E+01_JPRB, 0.11392E+01_JPRB, 0.11020E+01_JPRB, 0.10968E+01_JPRB, 0.11877E+01_JPRB, 0.13427E+01_JPRB, 0.15790E+01_JPRB ,&
&  0.25271E+01_JPRB, 0.54267E+01_JPRB, 0.26586E+01_JPRB, 0.25396E+01_JPRB, 0.24386E+01_JPRB, 0.23153E+01_JPRB, 0.21886E+01_JPRB ,&
&  0.22520E+01_JPRB, 0.19521E+01_JPRB, 0.17959E+01_JPRB, 0.19724E+01_JPRB, 0.20627E+01_JPRB, 0.21515E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.23681E+01_JPRB, 0.24477E+01_JPRB, 0.24933E+01_JPRB, 0.24696E+01_JPRB, 0.26122E+01_JPRB, 0.28201E+01_JPRB, 0.31085E+01_JPRB ,&
&  0.34425E+01_JPRB, 0.38736E+01_JPRB, 0.44469E+01_JPRB, 0.44317E+01_JPRB, 0.48699E+01_JPRB, 0.45612E+01_JPRB, 0.37091E+01_JPRB ,&
&  0.32141E+01_JPRB, 0.30701E+01_JPRB, 0.27487E+01_JPRB, 0.25601E+01_JPRB, 0.23189E+01_JPRB, 0.19563E+01_JPRB, 0.19074E+01_JPRB ,&
&  0.16955E+01_JPRB, 0.16030E+01_JPRB, 0.15861E+01_JPRB, 0.15758E+01_JPRB, 0.15991E+01_JPRB, 0.16083E+01_JPRB, 0.16149E+01_JPRB ,&
&  0.16042E+01_JPRB, 0.15708E+01_JPRB, 0.15536E+01_JPRB, 0.15650E+01_JPRB, 0.15716E+01_JPRB, 0.15677E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.15942E+01_JPRB, 0.16314E+01_JPRB, 0.16960E+01_JPRB, 0.16958E+01_JPRB, 0.16382E+01_JPRB, 0.15251E+01_JPRB, 0.14127E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.13695E+01_JPRB, 0.13246E+01_JPRB, 0.12628E+01_JPRB, 0.12490E+01_JPRB, 0.12044E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12909E+01_JPRB, 0.13832E+01_JPRB, 0.14526E+01_JPRB, 0.99700E+00_JPRB, 0.99412E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.10413E+01_JPRB, 0.99278E+00_JPRB, 0.99444E+00_JPRB, 0.10285E+01_JPRB, 0.10334E+01_JPRB, 0.10124E+01_JPRB, 0.96043E+00_JPRB ,&
&  0.88866E+00_JPRB, 0.87652E+00_JPRB, 0.89422E+00_JPRB, 0.91997E+00_JPRB, 0.94191E+00_JPRB, 0.96154E+00_JPRB, 0.96261E+00_JPRB ,&
&  0.94916E+00_JPRB, 0.96529E+00_JPRB, 0.98369E+00_JPRB, 0.10134E+01_JPRB, 0.10386E+01_JPRB, 0.10626E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10985E+01_JPRB, 0.10866E+01_JPRB, 0.10646E+01_JPRB, 0.10512E+01_JPRB, 0.10580E+01_JPRB, 0.11043E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.16980E+01_JPRB, 0.46489E+01_JPRB, 0.43634E+01_JPRB, 0.32179E+01_JPRB, 0.25548E+01_JPRB, 0.22290E+01_JPRB, 0.21046E+01_JPRB ,&
&  0.23166E+01_JPRB, 0.23402E+01_JPRB, 0.21693E+01_JPRB, 0.18736E+01_JPRB, 0.17878E+01_JPRB, 0.18318E+01_JPRB, 0.19057E+01_JPRB ,&
&  0.19278E+01_JPRB, 0.19541E+01_JPRB, 0.19910E+01_JPRB, 0.19898E+01_JPRB, 0.20268E+01_JPRB, 0.22015E+01_JPRB, 0.24218E+01_JPRB ,&
&  0.26459E+01_JPRB, 0.30432E+01_JPRB, 0.37466E+01_JPRB, 0.44663E+01_JPRB, 0.45426E+01_JPRB, 0.46633E+01_JPRB, 0.44087E+01_JPRB ,&
&  0.45822E+01_JPRB, 0.39770E+01_JPRB, 0.31273E+01_JPRB, 0.25415E+01_JPRB, 0.21933E+01_JPRB, 0.22598E+01_JPRB, 0.21294E+01_JPRB ,&
&  0.18207E+01_JPRB, 0.16753E+01_JPRB, 0.16170E+01_JPRB, 0.16073E+01_JPRB, 0.16133E+01_JPRB, 0.16166E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.15511E+01_JPRB, 0.15519E+01_JPRB, 0.15695E+01_JPRB, 0.15725E+01_JPRB, 0.15713E+01_JPRB, 0.15819E+01_JPRB ,&
&  0.15759E+01_JPRB, 0.15930E+01_JPRB, 0.16036E+01_JPRB, 0.16476E+01_JPRB, 0.15967E+01_JPRB, 0.14775E+01_JPRB, 0.13704E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.13395E+01_JPRB, 0.13398E+01_JPRB, 0.13159E+01_JPRB, 0.12941E+01_JPRB, 0.12680E+01_JPRB, 0.12275E+01_JPRB ,&
&  0.12299E+01_JPRB, 0.12283E+01_JPRB, 0.11556E+01_JPRB, 0.10796E+01_JPRB, 0.10833E+01_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.11243E+01_JPRB, 0.10918E+01_JPRB, 0.10690E+01_JPRB, 0.10784E+01_JPRB, 0.10876E+01_JPRB, 0.10723E+01_JPRB, 0.10414E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.95884E+00_JPRB, 0.95981E+00_JPRB, 0.94988E+00_JPRB, 0.94243E+00_JPRB, 0.94131E+00_JPRB, 0.94566E+00_JPRB ,&
&  0.95463E+00_JPRB, 0.96939E+00_JPRB, 0.96192E+00_JPRB, 0.96441E+00_JPRB, 0.98593E+00_JPRB, 0.10121E+01_JPRB, 0.10236E+01_JPRB ,&
&  0.10319E+01_JPRB, 0.10385E+01_JPRB, 0.10314E+01_JPRB, 0.10262E+01_JPRB, 0.10141E+01_JPRB, 0.10304E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.13907E+01_JPRB, 0.36986E+01_JPRB, 0.25543E+01_JPRB, 0.32649E+01_JPRB, 0.25087E+01_JPRB, 0.21807E+01_JPRB, 0.18320E+01_JPRB ,&
&  0.19993E+01_JPRB, 0.24416E+01_JPRB, 0.20373E+01_JPRB, 0.17903E+01_JPRB, 0.16574E+01_JPRB, 0.16380E+01_JPRB, 0.16481E+01_JPRB ,&
&  0.16580E+01_JPRB, 0.16841E+01_JPRB, 0.16815E+01_JPRB, 0.17080E+01_JPRB, 0.16975E+01_JPRB, 0.17814E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.20470E+01_JPRB, 0.22537E+01_JPRB, 0.27392E+01_JPRB, 0.33620E+01_JPRB, 0.45454E+01_JPRB, 0.51788E+01_JPRB, 0.54861E+01_JPRB ,&
&  0.57109E+01_JPRB, 0.48582E+01_JPRB, 0.38068E+01_JPRB, 0.29262E+01_JPRB, 0.23906E+01_JPRB, 0.23100E+01_JPRB, 0.22739E+01_JPRB ,&
&  0.20102E+01_JPRB, 0.18176E+01_JPRB, 0.17077E+01_JPRB, 0.16559E+01_JPRB, 0.16584E+01_JPRB, 0.16591E+01_JPRB, 0.16003E+01_JPRB ,&
&  0.15534E+01_JPRB, 0.15466E+01_JPRB, 0.15602E+01_JPRB, 0.15777E+01_JPRB, 0.15697E+01_JPRB, 0.15421E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.15453E+01_JPRB, 0.15490E+01_JPRB, 0.15438E+01_JPRB, 0.16247E+01_JPRB, 0.15697E+01_JPRB, 0.14747E+01_JPRB, 0.13778E+01_JPRB ,&
&  0.13457E+01_JPRB, 0.13354E+01_JPRB, 0.13237E+01_JPRB, 0.12954E+01_JPRB, 0.13171E+01_JPRB, 0.13471E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.12263E+01_JPRB, 0.11975E+01_JPRB, 0.11488E+01_JPRB, 0.11493E+01_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.11527E+01_JPRB, 0.11294E+01_JPRB, 0.11309E+01_JPRB, 0.11302E+01_JPRB, 0.11199E+01_JPRB, 0.10929E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10510E+01_JPRB, 0.10312E+01_JPRB, 0.10022E+01_JPRB, 0.98622E+00_JPRB, 0.97014E+00_JPRB, 0.96539E+00_JPRB ,&
&  0.97324E+00_JPRB, 0.97826E+00_JPRB, 0.97057E+00_JPRB, 0.96877E+00_JPRB, 0.95944E+00_JPRB, 0.96888E+00_JPRB, 0.98263E+00_JPRB ,&
&  0.99538E+00_JPRB, 0.10020E+01_JPRB, 0.10123E+01_JPRB, 0.10128E+01_JPRB, 0.10025E+01_JPRB, 0.99492E+00_JPRB, 0.10291E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.37992E+01_JPRB, 0.22207E+01_JPRB, 0.32616E+01_JPRB, 0.24622E+01_JPRB, 0.21644E+01_JPRB, 0.17845E+01_JPRB ,&
&  0.19103E+01_JPRB, 0.22690E+01_JPRB, 0.19713E+01_JPRB, 0.16933E+01_JPRB, 0.16024E+01_JPRB, 0.15496E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.14926E+01_JPRB, 0.15262E+01_JPRB, 0.15393E+01_JPRB, 0.15644E+01_JPRB, 0.15813E+01_JPRB, 0.15713E+01_JPRB, 0.16177E+01_JPRB ,&
&  0.16639E+01_JPRB, 0.17431E+01_JPRB, 0.20414E+01_JPRB, 0.30540E+01_JPRB, 0.40082E+01_JPRB, 0.54911E+01_JPRB, 0.70526E+01_JPRB ,&
&  0.68118E+01_JPRB, 0.58209E+01_JPRB, 0.41304E+01_JPRB, 0.33347E+01_JPRB, 0.27563E+01_JPRB, 0.25078E+01_JPRB, 0.23686E+01_JPRB ,&
&  0.22205E+01_JPRB, 0.20777E+01_JPRB, 0.18916E+01_JPRB, 0.17993E+01_JPRB, 0.17712E+01_JPRB, 0.17476E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.16499E+01_JPRB, 0.16395E+01_JPRB, 0.16442E+01_JPRB, 0.16416E+01_JPRB, 0.16261E+01_JPRB, 0.15807E+01_JPRB, 0.15539E+01_JPRB ,&
&  0.15615E+01_JPRB, 0.15525E+01_JPRB, 0.15553E+01_JPRB, 0.16702E+01_JPRB, 0.16154E+01_JPRB, 0.14977E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.13994E+01_JPRB, 0.13567E+01_JPRB, 0.13388E+01_JPRB, 0.13284E+01_JPRB, 0.13405E+01_JPRB, 0.14237E+01_JPRB, 0.13414E+01_JPRB ,&
&  0.12836E+01_JPRB, 0.12351E+01_JPRB, 0.12029E+01_JPRB, 0.11779E+01_JPRB, 0.11672E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.12043E+01_JPRB, 0.11845E+01_JPRB, 0.11819E+01_JPRB, 0.11956E+01_JPRB, 0.11747E+01_JPRB, 0.11408E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.11020E+01_JPRB, 0.10810E+01_JPRB, 0.10410E+01_JPRB, 0.10226E+01_JPRB, 0.10106E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.10158E+01_JPRB, 0.10056E+01_JPRB, 0.98616E+00_JPRB, 0.97473E+00_JPRB, 0.96930E+00_JPRB, 0.96271E+00_JPRB, 0.97506E+00_JPRB ,&
&  0.96711E+00_JPRB, 0.97297E+00_JPRB, 0.99347E+00_JPRB, 0.99779E+00_JPRB, 0.10006E+01_JPRB, 0.98301E+00_JPRB, 0.99912E+00_JPRB ,&
&  0.11584E+01_JPRB, 0.14804E+01_JPRB, 0.25455E+01_JPRB, 0.29849E+01_JPRB, 0.24542E+01_JPRB, 0.21895E+01_JPRB, 0.20202E+01_JPRB ,&
&  0.20380E+01_JPRB, 0.20645E+01_JPRB, 0.18752E+01_JPRB, 0.16941E+01_JPRB, 0.15948E+01_JPRB, 0.15010E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.13539E+01_JPRB, 0.13819E+01_JPRB, 0.14340E+01_JPRB, 0.14686E+01_JPRB, 0.14715E+01_JPRB, 0.14406E+01_JPRB ,&
&  0.14238E+01_JPRB, 0.14269E+01_JPRB, 0.15536E+01_JPRB, 0.22387E+01_JPRB, 0.38215E+01_JPRB, 0.56702E+01_JPRB, 0.68973E+01_JPRB ,&
&  0.70330E+01_JPRB, 0.58155E+01_JPRB, 0.42032E+01_JPRB, 0.35306E+01_JPRB, 0.30557E+01_JPRB, 0.26933E+01_JPRB, 0.23990E+01_JPRB ,&
&  0.22292E+01_JPRB, 0.21619E+01_JPRB, 0.20725E+01_JPRB, 0.19557E+01_JPRB, 0.18490E+01_JPRB, 0.17867E+01_JPRB, 0.17412E+01_JPRB ,&
&  0.17030E+01_JPRB, 0.17111E+01_JPRB, 0.17358E+01_JPRB, 0.17013E+01_JPRB, 0.16558E+01_JPRB, 0.16027E+01_JPRB, 0.15740E+01_JPRB ,&
&  0.15824E+01_JPRB, 0.15953E+01_JPRB, 0.16041E+01_JPRB, 0.16704E+01_JPRB, 0.16297E+01_JPRB, 0.15648E+01_JPRB, 0.14871E+01_JPRB ,&
&  0.14188E+01_JPRB, 0.13625E+01_JPRB, 0.13338E+01_JPRB, 0.13661E+01_JPRB, 0.13520E+01_JPRB, 0.14462E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.13366E+01_JPRB, 0.12831E+01_JPRB, 0.12511E+01_JPRB, 0.12323E+01_JPRB, 0.12251E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.12570E+01_JPRB, 0.12450E+01_JPRB, 0.12518E+01_JPRB, 0.12625E+01_JPRB, 0.12498E+01_JPRB, 0.12137E+01_JPRB, 0.11764E+01_JPRB ,&
&  0.11487E+01_JPRB, 0.11194E+01_JPRB, 0.10856E+01_JPRB, 0.10455E+01_JPRB, 0.10297E+01_JPRB, 0.10352E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.10401E+01_JPRB, 0.10169E+01_JPRB, 0.99139E+00_JPRB, 0.98251E+00_JPRB, 0.97821E+00_JPRB, 0.98442E+00_JPRB ,&
&  0.97594E+00_JPRB, 0.97245E+00_JPRB, 0.97760E+00_JPRB, 0.98349E+00_JPRB, 0.98974E+00_JPRB, 0.97912E+00_JPRB, 0.97467E+00_JPRB ,&
&  0.10688E+01_JPRB, 0.18857E+01_JPRB, 0.23039E+01_JPRB, 0.24456E+01_JPRB, 0.23438E+01_JPRB, 0.21423E+01_JPRB, 0.20502E+01_JPRB ,&
&  0.19025E+01_JPRB, 0.18657E+01_JPRB, 0.18110E+01_JPRB, 0.17199E+01_JPRB, 0.16129E+01_JPRB, 0.15113E+01_JPRB, 0.14280E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.13176E+01_JPRB, 0.13001E+01_JPRB, 0.13178E+01_JPRB, 0.13502E+01_JPRB, 0.13803E+01_JPRB, 0.13632E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.12926E+01_JPRB, 0.13071E+01_JPRB, 0.16490E+01_JPRB, 0.28960E+01_JPRB, 0.37074E+01_JPRB, 0.54945E+01_JPRB ,&
&  0.60854E+01_JPRB, 0.45388E+01_JPRB, 0.35849E+01_JPRB, 0.31370E+01_JPRB, 0.28136E+01_JPRB, 0.24897E+01_JPRB, 0.22450E+01_JPRB ,&
&  0.21282E+01_JPRB, 0.20580E+01_JPRB, 0.20353E+01_JPRB, 0.19625E+01_JPRB, 0.18233E+01_JPRB, 0.16966E+01_JPRB, 0.16342E+01_JPRB ,&
&  0.15997E+01_JPRB, 0.15734E+01_JPRB, 0.15772E+01_JPRB, 0.15917E+01_JPRB, 0.15820E+01_JPRB, 0.15309E+01_JPRB, 0.15038E+01_JPRB ,&
&  0.15050E+01_JPRB, 0.15380E+01_JPRB, 0.15802E+01_JPRB, 0.15962E+01_JPRB, 0.16031E+01_JPRB, 0.16127E+01_JPRB, 0.15399E+01_JPRB ,&
&  0.14286E+01_JPRB, 0.13561E+01_JPRB, 0.13355E+01_JPRB, 0.13435E+01_JPRB, 0.13706E+01_JPRB, 0.14761E+01_JPRB, 0.14341E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.13110E+01_JPRB, 0.12837E+01_JPRB, 0.12641E+01_JPRB, 0.12763E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.12818E+01_JPRB, 0.12691E+01_JPRB, 0.12754E+01_JPRB, 0.12798E+01_JPRB, 0.12683E+01_JPRB, 0.12322E+01_JPRB, 0.12004E+01_JPRB ,&
&  0.11794E+01_JPRB, 0.11327E+01_JPRB, 0.10854E+01_JPRB, 0.10499E+01_JPRB, 0.10461E+01_JPRB, 0.10535E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.10476E+01_JPRB, 0.10286E+01_JPRB, 0.10085E+01_JPRB, 0.99673E+00_JPRB, 0.98530E+00_JPRB, 0.96687E+00_JPRB ,&
&  0.96431E+00_JPRB, 0.96161E+00_JPRB, 0.96566E+00_JPRB, 0.97474E+00_JPRB, 0.97283E+00_JPRB, 0.97593E+00_JPRB, 0.97492E+00_JPRB ,&
&  0.10018E+01_JPRB, 0.13452E+01_JPRB, 0.15580E+01_JPRB, 0.19233E+01_JPRB, 0.20659E+01_JPRB, 0.20013E+01_JPRB, 0.19104E+01_JPRB ,&
&  0.17933E+01_JPRB, 0.17215E+01_JPRB, 0.16899E+01_JPRB, 0.16371E+01_JPRB, 0.15543E+01_JPRB, 0.14716E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.13434E+01_JPRB, 0.13072E+01_JPRB, 0.12970E+01_JPRB, 0.12850E+01_JPRB, 0.12781E+01_JPRB, 0.12932E+01_JPRB, 0.13090E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.12289E+01_JPRB, 0.12029E+01_JPRB, 0.12737E+01_JPRB, 0.18031E+01_JPRB, 0.25221E+01_JPRB, 0.34828E+01_JPRB ,&
&  0.32760E+01_JPRB, 0.28693E+01_JPRB, 0.26111E+01_JPRB, 0.24136E+01_JPRB, 0.22685E+01_JPRB, 0.20784E+01_JPRB, 0.19091E+01_JPRB ,&
&  0.17608E+01_JPRB, 0.16898E+01_JPRB, 0.16319E+01_JPRB, 0.15986E+01_JPRB, 0.15559E+01_JPRB, 0.15253E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.14172E+01_JPRB, 0.13894E+01_JPRB, 0.13834E+01_JPRB, 0.14088E+01_JPRB, 0.14493E+01_JPRB, 0.14610E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.14938E+01_JPRB, 0.15362E+01_JPRB, 0.15851E+01_JPRB, 0.15584E+01_JPRB, 0.15139E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.13892E+01_JPRB, 0.13409E+01_JPRB, 0.13485E+01_JPRB, 0.13550E+01_JPRB, 0.14472E+01_JPRB, 0.14314E+01_JPRB ,&
&  0.13770E+01_JPRB, 0.13081E+01_JPRB, 0.12796E+01_JPRB, 0.12635E+01_JPRB, 0.12823E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.12907E+01_JPRB, 0.12969E+01_JPRB, 0.12918E+01_JPRB, 0.12857E+01_JPRB, 0.12728E+01_JPRB, 0.12412E+01_JPRB, 0.12053E+01_JPRB ,&
&  0.11772E+01_JPRB, 0.11435E+01_JPRB, 0.10955E+01_JPRB, 0.10661E+01_JPRB, 0.10550E+01_JPRB, 0.10517E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.10405E+01_JPRB, 0.10300E+01_JPRB, 0.10254E+01_JPRB, 0.10134E+01_JPRB, 0.10030E+01_JPRB, 0.97749E+00_JPRB ,&
&  0.95916E+00_JPRB, 0.94673E+00_JPRB, 0.94358E+00_JPRB, 0.95146E+00_JPRB, 0.95923E+00_JPRB, 0.96472E+00_JPRB, 0.97416E+00_JPRB ,&
&  0.10065E+01_JPRB, 0.10034E+01_JPRB, 0.11451E+01_JPRB, 0.13950E+01_JPRB, 0.15789E+01_JPRB, 0.16923E+01_JPRB, 0.16592E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.15420E+01_JPRB, 0.15144E+01_JPRB, 0.14664E+01_JPRB, 0.14028E+01_JPRB, 0.13387E+01_JPRB, 0.12925E+01_JPRB ,&
&  0.12734E+01_JPRB, 0.12544E+01_JPRB, 0.12386E+01_JPRB, 0.12257E+01_JPRB, 0.12185E+01_JPRB, 0.12192E+01_JPRB, 0.12359E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.11985E+01_JPRB, 0.11600E+01_JPRB, 0.11376E+01_JPRB, 0.12221E+01_JPRB, 0.14658E+01_JPRB, 0.18967E+01_JPRB ,&
&  0.21271E+01_JPRB, 0.19991E+01_JPRB, 0.18982E+01_JPRB, 0.18087E+01_JPRB, 0.17328E+01_JPRB, 0.16790E+01_JPRB, 0.16242E+01_JPRB ,&
&  0.15816E+01_JPRB, 0.15514E+01_JPRB, 0.15328E+01_JPRB, 0.15108E+01_JPRB, 0.14853E+01_JPRB, 0.14576E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13744E+01_JPRB, 0.13593E+01_JPRB, 0.13707E+01_JPRB, 0.13880E+01_JPRB, 0.14031E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.14387E+01_JPRB, 0.14790E+01_JPRB, 0.15069E+01_JPRB, 0.15305E+01_JPRB, 0.15116E+01_JPRB, 0.14732E+01_JPRB, 0.14468E+01_JPRB ,&
&  0.14148E+01_JPRB, 0.13855E+01_JPRB, 0.13171E+01_JPRB, 0.13100E+01_JPRB, 0.14129E+01_JPRB, 0.14010E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.13704E+01_JPRB, 0.13333E+01_JPRB, 0.12788E+01_JPRB, 0.12258E+01_JPRB, 0.12496E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.11652E+01_JPRB, 0.11993E+01_JPRB, 0.12003E+01_JPRB, 0.11997E+01_JPRB, 0.11976E+01_JPRB, 0.11837E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.11394E+01_JPRB, 0.11248E+01_JPRB, 0.11039E+01_JPRB, 0.10764E+01_JPRB, 0.10569E+01_JPRB, 0.10430E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.10129E+01_JPRB, 0.10055E+01_JPRB, 0.99736E+00_JPRB, 0.98403E+00_JPRB, 0.97273E+00_JPRB, 0.95578E+00_JPRB ,&
&  0.94362E+00_JPRB, 0.94047E+00_JPRB, 0.93983E+00_JPRB, 0.94395E+00_JPRB, 0.95253E+00_JPRB, 0.96260E+00_JPRB, 0.96981E+00_JPRB ,&
&  0.98915E+00_JPRB, 0.91026E+00_JPRB, 0.95867E+00_JPRB, 0.10927E+01_JPRB, 0.12599E+01_JPRB, 0.14055E+01_JPRB, 0.14496E+01_JPRB ,&
&  0.14353E+01_JPRB, 0.14058E+01_JPRB, 0.13653E+01_JPRB, 0.13229E+01_JPRB, 0.12832E+01_JPRB, 0.12545E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.11670E+01_JPRB, 0.11652E+01_JPRB, 0.11589E+01_JPRB, 0.11493E+01_JPRB, 0.11425E+01_JPRB, 0.11474E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.11332E+01_JPRB, 0.11094E+01_JPRB, 0.10944E+01_JPRB, 0.11111E+01_JPRB, 0.12128E+01_JPRB, 0.13912E+01_JPRB ,&
&  0.15603E+01_JPRB, 0.15477E+01_JPRB, 0.14981E+01_JPRB, 0.14433E+01_JPRB, 0.14143E+01_JPRB, 0.14008E+01_JPRB, 0.13939E+01_JPRB ,&
&  0.13815E+01_JPRB, 0.13720E+01_JPRB, 0.13847E+01_JPRB, 0.13905E+01_JPRB, 0.13978E+01_JPRB, 0.13962E+01_JPRB, 0.13920E+01_JPRB ,&
&  0.13788E+01_JPRB, 0.13607E+01_JPRB, 0.13522E+01_JPRB, 0.13524E+01_JPRB, 0.13643E+01_JPRB, 0.13708E+01_JPRB, 0.13657E+01_JPRB ,&
&  0.13578E+01_JPRB, 0.13715E+01_JPRB, 0.14155E+01_JPRB, 0.14513E+01_JPRB, 0.14374E+01_JPRB, 0.14119E+01_JPRB, 0.13868E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.13520E+01_JPRB, 0.13361E+01_JPRB, 0.13612E+01_JPRB, 0.13566E+01_JPRB, 0.13394E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.12816E+01_JPRB, 0.12557E+01_JPRB, 0.12088E+01_JPRB, 0.11057E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.10600E+01_JPRB, 0.10940E+01_JPRB, 0.11135E+01_JPRB, 0.11192E+01_JPRB, 0.11282E+01_JPRB, 0.11343E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.11258E+01_JPRB, 0.11115E+01_JPRB, 0.10884E+01_JPRB, 0.10644E+01_JPRB, 0.10376E+01_JPRB, 0.10153E+01_JPRB ,&
&  0.10051E+01_JPRB, 0.99700E+00_JPRB, 0.97934E+00_JPRB, 0.96321E+00_JPRB, 0.94852E+00_JPRB, 0.93680E+00_JPRB, 0.93120E+00_JPRB ,&
&  0.92477E+00_JPRB, 0.92157E+00_JPRB, 0.92623E+00_JPRB, 0.93292E+00_JPRB, 0.94251E+00_JPRB, 0.95343E+00_JPRB, 0.96782E+00_JPRB ,&
&  0.97696E+00_JPRB, 0.80362E+00_JPRB, 0.83975E+00_JPRB, 0.94476E+00_JPRB, 0.10415E+01_JPRB, 0.11526E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12957E+01_JPRB, 0.12742E+01_JPRB, 0.12456E+01_JPRB, 0.12304E+01_JPRB, 0.12005E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.11093E+01_JPRB, 0.11010E+01_JPRB, 0.11089E+01_JPRB, 0.11120E+01_JPRB, 0.11043E+01_JPRB, 0.10936E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10911E+01_JPRB, 0.10916E+01_JPRB, 0.10897E+01_JPRB, 0.11083E+01_JPRB, 0.11580E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12939E+01_JPRB, 0.12801E+01_JPRB, 0.12519E+01_JPRB, 0.12364E+01_JPRB, 0.12433E+01_JPRB, 0.12475E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12434E+01_JPRB, 0.12529E+01_JPRB, 0.12626E+01_JPRB, 0.12788E+01_JPRB, 0.12953E+01_JPRB, 0.13213E+01_JPRB ,&
&  0.13415E+01_JPRB, 0.13450E+01_JPRB, 0.13439E+01_JPRB, 0.13306E+01_JPRB, 0.13065E+01_JPRB, 0.12891E+01_JPRB, 0.12817E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.12860E+01_JPRB, 0.13103E+01_JPRB, 0.13357E+01_JPRB, 0.13367E+01_JPRB, 0.13320E+01_JPRB, 0.13274E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13228E+01_JPRB, 0.13226E+01_JPRB, 0.12517E+01_JPRB, 0.12866E+01_JPRB, 0.12744E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.11734E+01_JPRB, 0.11455E+01_JPRB, 0.96709E+00_JPRB, 0.10346E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.99068E+00_JPRB, 0.10107E+01_JPRB, 0.10163E+01_JPRB, 0.10316E+01_JPRB, 0.10430E+01_JPRB, 0.10517E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.10672E+01_JPRB, 0.10599E+01_JPRB, 0.10446E+01_JPRB, 0.10190E+01_JPRB, 0.99164E+00_JPRB, 0.96881E+00_JPRB ,&
&  0.95144E+00_JPRB, 0.94613E+00_JPRB, 0.93552E+00_JPRB, 0.92266E+00_JPRB, 0.91451E+00_JPRB, 0.90988E+00_JPRB, 0.91081E+00_JPRB ,&
&  0.91782E+00_JPRB, 0.92123E+00_JPRB, 0.91552E+00_JPRB, 0.91359E+00_JPRB, 0.92179E+00_JPRB, 0.93713E+00_JPRB, 0.94380E+00_JPRB ,&
&  0.94565E+00_JPRB, 0.75032E+00_JPRB, 0.80050E+00_JPRB, 0.86195E+00_JPRB, 0.92297E+00_JPRB, 0.10081E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.11798E+01_JPRB, 0.11759E+01_JPRB, 0.11593E+01_JPRB, 0.11359E+01_JPRB, 0.11143E+01_JPRB, 0.10873E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10502E+01_JPRB, 0.10508E+01_JPRB, 0.10534E+01_JPRB, 0.10539E+01_JPRB, 0.10570E+01_JPRB, 0.10508E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10479E+01_JPRB, 0.10489E+01_JPRB, 0.10523E+01_JPRB, 0.10559E+01_JPRB, 0.10657E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.11117E+01_JPRB, 0.11143E+01_JPRB, 0.11051E+01_JPRB, 0.11028E+01_JPRB, 0.11101E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.11496E+01_JPRB, 0.11578E+01_JPRB, 0.11690E+01_JPRB, 0.11858E+01_JPRB, 0.12058E+01_JPRB, 0.12295E+01_JPRB ,&
&  0.12451E+01_JPRB, 0.12481E+01_JPRB, 0.12500E+01_JPRB, 0.12417E+01_JPRB, 0.12380E+01_JPRB, 0.12161E+01_JPRB, 0.11915E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.11861E+01_JPRB, 0.11905E+01_JPRB, 0.12052E+01_JPRB, 0.12115E+01_JPRB, 0.12072E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.12153E+01_JPRB, 0.12156E+01_JPRB, 0.12026E+01_JPRB, 0.11909E+01_JPRB, 0.11680E+01_JPRB, 0.11484E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.10583E+01_JPRB, 0.85093E+00_JPRB, 0.94196E+00_JPRB, 0.96367E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.96743E+00_JPRB, 0.97321E+00_JPRB, 0.97656E+00_JPRB, 0.98188E+00_JPRB, 0.98427E+00_JPRB, 0.98477E+00_JPRB, 0.99122E+00_JPRB ,&
&  0.99813E+00_JPRB, 0.99487E+00_JPRB, 0.98714E+00_JPRB, 0.98395E+00_JPRB, 0.97354E+00_JPRB, 0.95828E+00_JPRB, 0.94524E+00_JPRB ,&
&  0.92915E+00_JPRB, 0.92180E+00_JPRB, 0.91701E+00_JPRB, 0.91203E+00_JPRB, 0.90415E+00_JPRB, 0.89487E+00_JPRB, 0.89794E+00_JPRB ,&
&  0.90461E+00_JPRB, 0.91187E+00_JPRB, 0.91348E+00_JPRB, 0.91684E+00_JPRB, 0.92226E+00_JPRB, 0.92837E+00_JPRB, 0.93927E+00_JPRB ,&
&  0.82592E+00_JPRB, 0.69949E+00_JPRB, 0.75548E+00_JPRB, 0.81360E+00_JPRB, 0.86835E+00_JPRB, 0.95275E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.11026E+01_JPRB, 0.11097E+01_JPRB, 0.10846E+01_JPRB, 0.10586E+01_JPRB, 0.10513E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.10422E+01_JPRB, 0.10432E+01_JPRB, 0.10458E+01_JPRB, 0.10496E+01_JPRB, 0.10557E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.10428E+01_JPRB, 0.10282E+01_JPRB, 0.10064E+01_JPRB, 0.98723E+00_JPRB, 0.98212E+00_JPRB, 0.98165E+00_JPRB ,&
&  0.97649E+00_JPRB, 0.98370E+00_JPRB, 0.99529E+00_JPRB, 0.10038E+01_JPRB, 0.10161E+01_JPRB, 0.10312E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.10777E+01_JPRB, 0.10995E+01_JPRB, 0.11169E+01_JPRB, 0.11346E+01_JPRB, 0.11427E+01_JPRB, 0.11517E+01_JPRB, 0.11681E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.11688E+01_JPRB, 0.11563E+01_JPRB, 0.11453E+01_JPRB, 0.11428E+01_JPRB, 0.11425E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.11240E+01_JPRB, 0.11171E+01_JPRB, 0.11140E+01_JPRB, 0.11190E+01_JPRB, 0.11161E+01_JPRB, 0.11068E+01_JPRB, 0.10991E+01_JPRB ,&
&  0.10910E+01_JPRB, 0.10750E+01_JPRB, 0.10575E+01_JPRB, 0.10452E+01_JPRB, 0.10288E+01_JPRB, 0.10158E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.99306E+00_JPRB, 0.97473E+00_JPRB, 0.97616E+00_JPRB, 0.96785E+00_JPRB, 0.96352E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.93099E+00_JPRB, 0.93457E+00_JPRB, 0.93890E+00_JPRB, 0.94028E+00_JPRB, 0.93779E+00_JPRB, 0.93690E+00_JPRB, 0.94043E+00_JPRB ,&
&  0.94366E+00_JPRB, 0.93746E+00_JPRB, 0.93073E+00_JPRB, 0.92529E+00_JPRB, 0.92274E+00_JPRB, 0.91305E+00_JPRB, 0.89828E+00_JPRB ,&
&  0.88748E+00_JPRB, 0.88544E+00_JPRB, 0.88988E+00_JPRB, 0.89202E+00_JPRB, 0.88268E+00_JPRB, 0.87300E+00_JPRB, 0.86961E+00_JPRB ,&
&  0.87061E+00_JPRB, 0.87656E+00_JPRB, 0.88655E+00_JPRB, 0.89198E+00_JPRB, 0.90020E+00_JPRB, 0.91393E+00_JPRB, 0.91412E+00_JPRB ,&
&  0.91162E+00_JPRB, 0.77324E+00_JPRB, 0.81346E+00_JPRB, 0.82991E+00_JPRB, 0.86271E+00_JPRB, 0.91267E+00_JPRB, 0.95651E+00_JPRB ,&
&  0.99941E+00_JPRB, 0.10235E+01_JPRB, 0.10342E+01_JPRB, 0.10271E+01_JPRB, 0.10100E+01_JPRB, 0.99417E+00_JPRB, 0.99117E+00_JPRB ,&
&  0.99287E+00_JPRB, 0.10016E+01_JPRB, 0.10159E+01_JPRB, 0.10253E+01_JPRB, 0.10286E+01_JPRB, 0.10297E+01_JPRB, 0.10325E+01_JPRB ,&
&  0.10292E+01_JPRB, 0.10119E+01_JPRB, 0.98636E+00_JPRB, 0.95457E+00_JPRB, 0.92838E+00_JPRB, 0.91468E+00_JPRB, 0.90287E+00_JPRB ,&
&  0.88797E+00_JPRB, 0.88410E+00_JPRB, 0.89168E+00_JPRB, 0.90215E+00_JPRB, 0.91738E+00_JPRB, 0.93441E+00_JPRB, 0.94880E+00_JPRB ,&
&  0.96372E+00_JPRB, 0.98487E+00_JPRB, 0.10076E+01_JPRB, 0.10285E+01_JPRB, 0.10460E+01_JPRB, 0.10591E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10622E+01_JPRB, 0.10479E+01_JPRB, 0.10379E+01_JPRB, 0.10350E+01_JPRB, 0.10412E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.10285E+01_JPRB, 0.10235E+01_JPRB, 0.10106E+01_JPRB, 0.99582E+00_JPRB, 0.98122E+00_JPRB, 0.96618E+00_JPRB ,&
&  0.95447E+00_JPRB, 0.94834E+00_JPRB, 0.94267E+00_JPRB, 0.93426E+00_JPRB, 0.92174E+00_JPRB, 0.91381E+00_JPRB, 0.90704E+00_JPRB ,&
&  0.90113E+00_JPRB, 0.89504E+00_JPRB, 0.89550E+00_JPRB, 0.90559E+00_JPRB, 0.92065E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.88311E+00_JPRB, 0.88916E+00_JPRB, 0.89878E+00_JPRB, 0.91011E+00_JPRB, 0.91477E+00_JPRB, 0.91558E+00_JPRB, 0.91452E+00_JPRB ,&
&  0.91653E+00_JPRB, 0.90950E+00_JPRB, 0.90435E+00_JPRB, 0.90115E+00_JPRB, 0.90037E+00_JPRB, 0.89631E+00_JPRB, 0.88605E+00_JPRB ,&
&  0.87727E+00_JPRB, 0.87330E+00_JPRB, 0.87693E+00_JPRB, 0.87761E+00_JPRB, 0.86161E+00_JPRB, 0.85614E+00_JPRB, 0.84784E+00_JPRB ,&
&  0.84634E+00_JPRB, 0.84579E+00_JPRB, 0.84801E+00_JPRB, 0.85334E+00_JPRB, 0.86899E+00_JPRB, 0.87999E+00_JPRB, 0.88557E+00_JPRB ,&
&  0.88277E+00_JPRB, 0.80767E+00_JPRB, 0.83614E+00_JPRB, 0.84740E+00_JPRB, 0.86955E+00_JPRB, 0.89283E+00_JPRB, 0.91524E+00_JPRB ,&
&  0.94497E+00_JPRB, 0.96233E+00_JPRB, 0.97060E+00_JPRB, 0.97324E+00_JPRB, 0.97513E+00_JPRB, 0.97428E+00_JPRB, 0.96645E+00_JPRB ,&
&  0.96247E+00_JPRB, 0.97107E+00_JPRB, 0.99004E+00_JPRB, 0.10005E+01_JPRB, 0.10062E+01_JPRB, 0.10094E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.99948E+00_JPRB, 0.98394E+00_JPRB, 0.95737E+00_JPRB, 0.92483E+00_JPRB, 0.88800E+00_JPRB, 0.86143E+00_JPRB, 0.84652E+00_JPRB ,&
&  0.83886E+00_JPRB, 0.84182E+00_JPRB, 0.84823E+00_JPRB, 0.85506E+00_JPRB, 0.86319E+00_JPRB, 0.87643E+00_JPRB, 0.88500E+00_JPRB ,&
&  0.89574E+00_JPRB, 0.90516E+00_JPRB, 0.91813E+00_JPRB, 0.93799E+00_JPRB, 0.95707E+00_JPRB, 0.96689E+00_JPRB, 0.96649E+00_JPRB ,&
&  0.97170E+00_JPRB, 0.97481E+00_JPRB, 0.97101E+00_JPRB, 0.97014E+00_JPRB, 0.96537E+00_JPRB, 0.96630E+00_JPRB, 0.96404E+00_JPRB ,&
&  0.95498E+00_JPRB, 0.94771E+00_JPRB, 0.93869E+00_JPRB, 0.92192E+00_JPRB, 0.90654E+00_JPRB, 0.89473E+00_JPRB, 0.88475E+00_JPRB ,&
&  0.88222E+00_JPRB, 0.88508E+00_JPRB, 0.88976E+00_JPRB, 0.88384E+00_JPRB, 0.86833E+00_JPRB, 0.85773E+00_JPRB, 0.84578E+00_JPRB ,&
&  0.83769E+00_JPRB, 0.83920E+00_JPRB, 0.83880E+00_JPRB, 0.84614E+00_JPRB, 0.86564E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.83236E+00_JPRB, 0.84533E+00_JPRB, 0.86172E+00_JPRB, 0.87889E+00_JPRB, 0.89025E+00_JPRB, 0.89758E+00_JPRB, 0.89644E+00_JPRB ,&
&  0.88644E+00_JPRB, 0.87651E+00_JPRB, 0.87902E+00_JPRB, 0.87948E+00_JPRB, 0.87855E+00_JPRB, 0.87645E+00_JPRB, 0.86921E+00_JPRB ,&
&  0.85878E+00_JPRB, 0.84969E+00_JPRB, 0.84554E+00_JPRB, 0.84206E+00_JPRB, 0.83167E+00_JPRB, 0.82354E+00_JPRB, 0.81467E+00_JPRB ,&
&  0.81381E+00_JPRB, 0.81570E+00_JPRB, 0.81775E+00_JPRB, 0.81705E+00_JPRB, 0.82409E+00_JPRB, 0.83342E+00_JPRB, 0.83706E+00_JPRB ,&
&  0.83073E+00_JPRB, 0.83968E+00_JPRB, 0.75528E+00_JPRB, 0.82486E+00_JPRB, 0.84937E+00_JPRB, 0.86404E+00_JPRB, 0.88063E+00_JPRB ,&
&  0.89281E+00_JPRB, 0.90575E+00_JPRB, 0.91531E+00_JPRB, 0.92159E+00_JPRB, 0.94069E+00_JPRB, 0.95496E+00_JPRB, 0.95765E+00_JPRB ,&
&  0.95925E+00_JPRB, 0.95810E+00_JPRB, 0.95978E+00_JPRB, 0.96120E+00_JPRB, 0.96722E+00_JPRB, 0.96880E+00_JPRB, 0.96390E+00_JPRB ,&
&  0.95631E+00_JPRB, 0.94413E+00_JPRB, 0.92547E+00_JPRB, 0.90320E+00_JPRB, 0.86859E+00_JPRB, 0.83835E+00_JPRB, 0.81890E+00_JPRB ,&
&  0.80385E+00_JPRB, 0.80015E+00_JPRB, 0.79635E+00_JPRB, 0.79295E+00_JPRB, 0.79329E+00_JPRB, 0.79746E+00_JPRB, 0.79871E+00_JPRB ,&
&  0.80686E+00_JPRB, 0.81619E+00_JPRB, 0.82050E+00_JPRB, 0.82833E+00_JPRB, 0.83892E+00_JPRB, 0.85406E+00_JPRB, 0.86856E+00_JPRB ,&
&  0.88215E+00_JPRB, 0.89191E+00_JPRB, 0.89412E+00_JPRB, 0.89452E+00_JPRB, 0.89162E+00_JPRB, 0.88968E+00_JPRB, 0.88212E+00_JPRB ,&
&  0.87616E+00_JPRB, 0.87292E+00_JPRB, 0.86607E+00_JPRB, 0.85323E+00_JPRB, 0.84467E+00_JPRB, 0.83730E+00_JPRB, 0.83493E+00_JPRB ,&
&  0.83512E+00_JPRB, 0.83613E+00_JPRB, 0.83358E+00_JPRB, 0.83412E+00_JPRB, 0.82722E+00_JPRB, 0.80573E+00_JPRB, 0.78868E+00_JPRB ,&
&  0.78708E+00_JPRB, 0.79190E+00_JPRB, 0.79635E+00_JPRB, 0.80497E+00_JPRB, 0.81997E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.84670E+00_JPRB, 0.85290E+00_JPRB, 0.86253E+00_JPRB, 0.87670E+00_JPRB, 0.89177E+00_JPRB, 0.90194E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.89203E+00_JPRB, 0.87918E+00_JPRB, 0.87520E+00_JPRB, 0.87495E+00_JPRB, 0.87221E+00_JPRB, 0.87080E+00_JPRB, 0.86598E+00_JPRB ,&
&  0.85418E+00_JPRB, 0.83638E+00_JPRB, 0.82770E+00_JPRB, 0.81327E+00_JPRB, 0.79965E+00_JPRB, 0.78832E+00_JPRB, 0.78773E+00_JPRB ,&
&  0.79589E+00_JPRB, 0.80553E+00_JPRB, 0.81749E+00_JPRB, 0.83086E+00_JPRB, 0.83737E+00_JPRB, 0.83497E+00_JPRB, 0.83096E+00_JPRB ,&
&  0.82758E+00_JPRB, 0.82448E+00_JPRB, 0.83449E+00_JPRB, 0.84292E+00_JPRB, 0.84851E+00_JPRB, 0.85946E+00_JPRB, 0.86908E+00_JPRB ,&
&  0.87785E+00_JPRB, 0.89000E+00_JPRB, 0.90945E+00_JPRB, 0.92399E+00_JPRB, 0.93507E+00_JPRB, 0.94647E+00_JPRB, 0.94947E+00_JPRB ,&
&  0.95513E+00_JPRB, 0.96623E+00_JPRB, 0.96856E+00_JPRB, 0.95967E+00_JPRB, 0.94581E+00_JPRB, 0.93754E+00_JPRB, 0.93345E+00_JPRB ,&
&  0.92500E+00_JPRB, 0.90580E+00_JPRB, 0.89092E+00_JPRB, 0.87854E+00_JPRB, 0.85841E+00_JPRB, 0.83699E+00_JPRB, 0.81219E+00_JPRB ,&
&  0.78434E+00_JPRB, 0.76363E+00_JPRB, 0.74767E+00_JPRB, 0.73327E+00_JPRB, 0.72967E+00_JPRB, 0.72105E+00_JPRB, 0.71511E+00_JPRB ,&
&  0.72055E+00_JPRB, 0.74028E+00_JPRB, 0.75547E+00_JPRB, 0.76179E+00_JPRB, 0.76554E+00_JPRB, 0.78253E+00_JPRB, 0.80296E+00_JPRB ,&
&  0.81910E+00_JPRB, 0.82646E+00_JPRB, 0.82337E+00_JPRB, 0.82635E+00_JPRB, 0.83182E+00_JPRB, 0.83604E+00_JPRB, 0.82941E+00_JPRB ,&
&  0.82018E+00_JPRB, 0.81424E+00_JPRB, 0.81267E+00_JPRB, 0.80857E+00_JPRB, 0.80974E+00_JPRB, 0.80939E+00_JPRB, 0.80648E+00_JPRB ,&
&  0.80078E+00_JPRB, 0.79177E+00_JPRB, 0.77984E+00_JPRB, 0.77190E+00_JPRB, 0.76027E+00_JPRB, 0.74294E+00_JPRB, 0.74260E+00_JPRB ,&
&  0.75351E+00_JPRB, 0.77159E+00_JPRB, 0.78925E+00_JPRB, 0.81043E+00_JPRB, 0.83525E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.87120E+00_JPRB, 0.87916E+00_JPRB, 0.88468E+00_JPRB, 0.88884E+00_JPRB, 0.89811E+00_JPRB, 0.90987E+00_JPRB, 0.91029E+00_JPRB ,&
&  0.89595E+00_JPRB, 0.88231E+00_JPRB, 0.87280E+00_JPRB, 0.87039E+00_JPRB, 0.87236E+00_JPRB, 0.87483E+00_JPRB, 0.86887E+00_JPRB ,&
&  0.84675E+00_JPRB, 0.82313E+00_JPRB, 0.80855E+00_JPRB, 0.79415E+00_JPRB, 0.78505E+00_JPRB, 0.77907E+00_JPRB, 0.78540E+00_JPRB ,&
&  0.79495E+00_JPRB, 0.81029E+00_JPRB, 0.82261E+00_JPRB, 0.83135E+00_JPRB, 0.83458E+00_JPRB, 0.82256E+00_JPRB, 0.82333E+00_JPRB ,&
&  0.82981E+00_JPRB, 0.83152E+00_JPRB, 0.84608E+00_JPRB, 0.84509E+00_JPRB, 0.83125E+00_JPRB, 0.81987E+00_JPRB, 0.80886E+00_JPRB ,&
&  0.80614E+00_JPRB, 0.81747E+00_JPRB, 0.84296E+00_JPRB, 0.88004E+00_JPRB, 0.90225E+00_JPRB, 0.91095E+00_JPRB, 0.90745E+00_JPRB ,&
&  0.90947E+00_JPRB, 0.92116E+00_JPRB, 0.92783E+00_JPRB, 0.91965E+00_JPRB, 0.90164E+00_JPRB, 0.88713E+00_JPRB, 0.87457E+00_JPRB ,&
&  0.86566E+00_JPRB, 0.85471E+00_JPRB, 0.84703E+00_JPRB, 0.83717E+00_JPRB, 0.81487E+00_JPRB, 0.78836E+00_JPRB, 0.76510E+00_JPRB ,&
&  0.74758E+00_JPRB, 0.73022E+00_JPRB, 0.70928E+00_JPRB, 0.70223E+00_JPRB, 0.70579E+00_JPRB, 0.68289E+00_JPRB, 0.65862E+00_JPRB ,&
&  0.66302E+00_JPRB, 0.69555E+00_JPRB, 0.72137E+00_JPRB, 0.73261E+00_JPRB, 0.73462E+00_JPRB, 0.73978E+00_JPRB, 0.74398E+00_JPRB ,&
&  0.75140E+00_JPRB, 0.76090E+00_JPRB, 0.76822E+00_JPRB, 0.77993E+00_JPRB, 0.79189E+00_JPRB, 0.79015E+00_JPRB, 0.77515E+00_JPRB ,&
&  0.75894E+00_JPRB, 0.74981E+00_JPRB, 0.74620E+00_JPRB, 0.74860E+00_JPRB, 0.75212E+00_JPRB, 0.75031E+00_JPRB, 0.74455E+00_JPRB ,&
&  0.73260E+00_JPRB, 0.71412E+00_JPRB, 0.69831E+00_JPRB, 0.68889E+00_JPRB, 0.68765E+00_JPRB, 0.69356E+00_JPRB, 0.70959E+00_JPRB ,&
&  0.74093E+00_JPRB, 0.78882E+00_JPRB, 0.82509E+00_JPRB, 0.84643E+00_JPRB, 0.85331E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.89732E+00_JPRB, 0.90559E+00_JPRB, 0.91537E+00_JPRB, 0.92875E+00_JPRB, 0.94128E+00_JPRB, 0.93842E+00_JPRB, 0.92586E+00_JPRB ,&
&  0.90221E+00_JPRB, 0.89302E+00_JPRB, 0.89752E+00_JPRB, 0.89990E+00_JPRB, 0.90166E+00_JPRB, 0.89946E+00_JPRB, 0.88302E+00_JPRB ,&
&  0.84432E+00_JPRB, 0.81343E+00_JPRB, 0.79855E+00_JPRB, 0.79065E+00_JPRB, 0.78792E+00_JPRB, 0.79436E+00_JPRB, 0.80184E+00_JPRB ,&
&  0.81743E+00_JPRB, 0.82636E+00_JPRB, 0.83783E+00_JPRB, 0.84712E+00_JPRB, 0.84987E+00_JPRB, 0.85369E+00_JPRB, 0.85438E+00_JPRB ,&
&  0.83958E+00_JPRB, 0.83945E+00_JPRB, 0.84014E+00_JPRB, 0.82983E+00_JPRB, 0.80739E+00_JPRB, 0.78484E+00_JPRB, 0.75753E+00_JPRB ,&
&  0.73921E+00_JPRB, 0.72993E+00_JPRB, 0.75550E+00_JPRB, 0.81790E+00_JPRB, 0.86216E+00_JPRB, 0.87036E+00_JPRB, 0.86179E+00_JPRB ,&
&  0.86302E+00_JPRB, 0.87016E+00_JPRB, 0.87382E+00_JPRB, 0.86254E+00_JPRB, 0.84298E+00_JPRB, 0.82583E+00_JPRB, 0.80966E+00_JPRB ,&
&  0.80473E+00_JPRB, 0.80467E+00_JPRB, 0.78801E+00_JPRB, 0.75615E+00_JPRB, 0.72832E+00_JPRB, 0.70853E+00_JPRB, 0.69795E+00_JPRB ,&
&  0.69121E+00_JPRB, 0.68098E+00_JPRB, 0.67421E+00_JPRB, 0.67838E+00_JPRB, 0.65814E+00_JPRB, 0.64040E+00_JPRB, 0.64461E+00_JPRB ,&
&  0.66158E+00_JPRB, 0.66805E+00_JPRB, 0.67821E+00_JPRB, 0.68238E+00_JPRB, 0.68045E+00_JPRB, 0.67774E+00_JPRB, 0.67971E+00_JPRB ,&
&  0.68925E+00_JPRB, 0.70013E+00_JPRB, 0.70814E+00_JPRB, 0.71332E+00_JPRB, 0.71833E+00_JPRB, 0.70580E+00_JPRB, 0.68410E+00_JPRB ,&
&  0.66655E+00_JPRB, 0.66340E+00_JPRB, 0.66792E+00_JPRB, 0.66734E+00_JPRB, 0.66581E+00_JPRB, 0.66617E+00_JPRB, 0.66523E+00_JPRB ,&
&  0.66279E+00_JPRB, 0.65586E+00_JPRB, 0.64737E+00_JPRB, 0.64715E+00_JPRB, 0.65796E+00_JPRB, 0.68094E+00_JPRB, 0.71669E+00_JPRB ,&
&  0.76904E+00_JPRB, 0.80842E+00_JPRB, 0.84732E+00_JPRB, 0.86420E+00_JPRB, 0.86904E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.93012E+00_JPRB, 0.95254E+00_JPRB, 0.97152E+00_JPRB, 0.98443E+00_JPRB, 0.98640E+00_JPRB, 0.97948E+00_JPRB, 0.96329E+00_JPRB ,&
&  0.93289E+00_JPRB, 0.91554E+00_JPRB, 0.92439E+00_JPRB, 0.93907E+00_JPRB, 0.94787E+00_JPRB, 0.94773E+00_JPRB, 0.91572E+00_JPRB ,&
&  0.86009E+00_JPRB, 0.83884E+00_JPRB, 0.83545E+00_JPRB, 0.84466E+00_JPRB, 0.86216E+00_JPRB, 0.86808E+00_JPRB, 0.85479E+00_JPRB ,&
&  0.84154E+00_JPRB, 0.84241E+00_JPRB, 0.84477E+00_JPRB, 0.85164E+00_JPRB, 0.84131E+00_JPRB, 0.82022E+00_JPRB, 0.81054E+00_JPRB ,&
&  0.80720E+00_JPRB, 0.79846E+00_JPRB, 0.78927E+00_JPRB, 0.74043E+00_JPRB, 0.76171E+00_JPRB, 0.74974E+00_JPRB, 0.73063E+00_JPRB ,&
&  0.71258E+00_JPRB, 0.71459E+00_JPRB, 0.75688E+00_JPRB, 0.80419E+00_JPRB, 0.81600E+00_JPRB, 0.81841E+00_JPRB, 0.81263E+00_JPRB ,&
&  0.80864E+00_JPRB, 0.80896E+00_JPRB, 0.81041E+00_JPRB, 0.80228E+00_JPRB, 0.78199E+00_JPRB, 0.75574E+00_JPRB, 0.72713E+00_JPRB ,&
&  0.71009E+00_JPRB, 0.68566E+00_JPRB, 0.64247E+00_JPRB, 0.62237E+00_JPRB, 0.62083E+00_JPRB, 0.62911E+00_JPRB, 0.63300E+00_JPRB ,&
&  0.62998E+00_JPRB, 0.62230E+00_JPRB, 0.62521E+00_JPRB, 0.62152E+00_JPRB, 0.59953E+00_JPRB, 0.58576E+00_JPRB, 0.57963E+00_JPRB ,&
&  0.56883E+00_JPRB, 0.56903E+00_JPRB, 0.57508E+00_JPRB, 0.58771E+00_JPRB, 0.59879E+00_JPRB, 0.61413E+00_JPRB, 0.62100E+00_JPRB ,&
&  0.62365E+00_JPRB, 0.61959E+00_JPRB, 0.61295E+00_JPRB, 0.60025E+00_JPRB, 0.58107E+00_JPRB, 0.57365E+00_JPRB, 0.57526E+00_JPRB ,&
&  0.58457E+00_JPRB, 0.59115E+00_JPRB, 0.59116E+00_JPRB, 0.59035E+00_JPRB, 0.57890E+00_JPRB, 0.56629E+00_JPRB, 0.55696E+00_JPRB ,&
&  0.55303E+00_JPRB, 0.55677E+00_JPRB, 0.57342E+00_JPRB, 0.59941E+00_JPRB, 0.63234E+00_JPRB, 0.67231E+00_JPRB, 0.71761E+00_JPRB ,&
&  0.74922E+00_JPRB, 0.76734E+00_JPRB, 0.79177E+00_JPRB, 0.84035E+00_JPRB, 0.88793E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.93002E+00_JPRB, 0.95754E+00_JPRB, 0.98710E+00_JPRB, 0.10041E+01_JPRB, 0.10049E+01_JPRB, 0.10018E+01_JPRB, 0.98668E+00_JPRB ,&
&  0.95576E+00_JPRB, 0.93066E+00_JPRB, 0.92664E+00_JPRB, 0.93068E+00_JPRB, 0.94233E+00_JPRB, 0.96074E+00_JPRB, 0.96139E+00_JPRB ,&
&  0.93342E+00_JPRB, 0.91317E+00_JPRB, 0.90285E+00_JPRB, 0.89494E+00_JPRB, 0.87696E+00_JPRB, 0.85146E+00_JPRB, 0.83471E+00_JPRB ,&
&  0.82965E+00_JPRB, 0.82909E+00_JPRB, 0.81997E+00_JPRB, 0.80614E+00_JPRB, 0.77456E+00_JPRB, 0.73545E+00_JPRB, 0.70408E+00_JPRB ,&
&  0.69230E+00_JPRB, 0.68110E+00_JPRB, 0.64116E+00_JPRB, 0.65495E+00_JPRB, 0.65909E+00_JPRB, 0.65949E+00_JPRB, 0.65624E+00_JPRB ,&
&  0.66719E+00_JPRB, 0.70041E+00_JPRB, 0.73390E+00_JPRB, 0.74362E+00_JPRB, 0.75275E+00_JPRB, 0.75428E+00_JPRB, 0.75542E+00_JPRB ,&
&  0.75973E+00_JPRB, 0.75175E+00_JPRB, 0.72212E+00_JPRB, 0.67806E+00_JPRB, 0.63466E+00_JPRB, 0.60092E+00_JPRB, 0.57229E+00_JPRB ,&
&  0.55284E+00_JPRB, 0.54068E+00_JPRB, 0.53108E+00_JPRB, 0.52785E+00_JPRB, 0.53595E+00_JPRB, 0.55141E+00_JPRB, 0.54638E+00_JPRB ,&
&  0.52769E+00_JPRB, 0.50995E+00_JPRB, 0.48704E+00_JPRB, 0.45598E+00_JPRB, 0.42368E+00_JPRB, 0.38635E+00_JPRB, 0.36660E+00_JPRB ,&
&  0.37174E+00_JPRB, 0.41174E+00_JPRB, 0.46393E+00_JPRB, 0.48525E+00_JPRB, 0.51338E+00_JPRB, 0.54114E+00_JPRB, 0.53049E+00_JPRB ,&
&  0.48058E+00_JPRB, 0.41466E+00_JPRB, 0.39177E+00_JPRB, 0.37796E+00_JPRB, 0.37089E+00_JPRB, 0.36626E+00_JPRB, 0.36473E+00_JPRB ,&
&  0.36929E+00_JPRB, 0.38035E+00_JPRB, 0.39053E+00_JPRB, 0.38801E+00_JPRB, 0.37803E+00_JPRB, 0.37289E+00_JPRB, 0.37441E+00_JPRB ,&
&  0.38603E+00_JPRB, 0.41571E+00_JPRB, 0.47853E+00_JPRB, 0.59130E+00_JPRB, 0.63041E+00_JPRB, 0.66521E+00_JPRB, 0.70471E+00_JPRB ,&
&  0.74275E+00_JPRB, 0.77721E+00_JPRB, 0.80618E+00_JPRB, 0.84159E+00_JPRB, 0.88930E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.86667E+00_JPRB, 0.88607E+00_JPRB, 0.90822E+00_JPRB, 0.91990E+00_JPRB, 0.92123E+00_JPRB, 0.91842E+00_JPRB, 0.90979E+00_JPRB ,&
&  0.89491E+00_JPRB, 0.87702E+00_JPRB, 0.86371E+00_JPRB, 0.85442E+00_JPRB, 0.84983E+00_JPRB, 0.85199E+00_JPRB, 0.85631E+00_JPRB ,&
&  0.85435E+00_JPRB, 0.84942E+00_JPRB, 0.83646E+00_JPRB, 0.81967E+00_JPRB, 0.80200E+00_JPRB, 0.78842E+00_JPRB, 0.77820E+00_JPRB ,&
&  0.76830E+00_JPRB, 0.75361E+00_JPRB, 0.73083E+00_JPRB, 0.69538E+00_JPRB, 0.65332E+00_JPRB, 0.61488E+00_JPRB, 0.58381E+00_JPRB ,&
&  0.55522E+00_JPRB, 0.48964E+00_JPRB, 0.45365E+00_JPRB, 0.46289E+00_JPRB, 0.55006E+00_JPRB, 0.58397E+00_JPRB, 0.60944E+00_JPRB ,&
&  0.64518E+00_JPRB, 0.68341E+00_JPRB, 0.70186E+00_JPRB, 0.70208E+00_JPRB, 0.69630E+00_JPRB, 0.68344E+00_JPRB, 0.66348E+00_JPRB ,&
&  0.63821E+00_JPRB, 0.60745E+00_JPRB, 0.58107E+00_JPRB, 0.55123E+00_JPRB, 0.52091E+00_JPRB, 0.49879E+00_JPRB, 0.48347E+00_JPRB ,&
&  0.47295E+00_JPRB, 0.46291E+00_JPRB, 0.45478E+00_JPRB, 0.44483E+00_JPRB, 0.43573E+00_JPRB, 0.41461E+00_JPRB, 0.39161E+00_JPRB ,&
&  0.36984E+00_JPRB, 0.35140E+00_JPRB, 0.33284E+00_JPRB, 0.31439E+00_JPRB, 0.29931E+00_JPRB, 0.28930E+00_JPRB, 0.28169E+00_JPRB ,&
&  0.27989E+00_JPRB, 0.28747E+00_JPRB, 0.30797E+00_JPRB, 0.34082E+00_JPRB, 0.37490E+00_JPRB, 0.35198E+00_JPRB, 0.33307E+00_JPRB ,&
&  0.31074E+00_JPRB, 0.29792E+00_JPRB, 0.29292E+00_JPRB, 0.29215E+00_JPRB, 0.29321E+00_JPRB, 0.29529E+00_JPRB, 0.29772E+00_JPRB ,&
&  0.30144E+00_JPRB, 0.30505E+00_JPRB, 0.30864E+00_JPRB, 0.31140E+00_JPRB, 0.31313E+00_JPRB, 0.31601E+00_JPRB, 0.32092E+00_JPRB ,&
&  0.32915E+00_JPRB, 0.34237E+00_JPRB, 0.36295E+00_JPRB, 0.38975E+00_JPRB, 0.41840E+00_JPRB, 0.44408E+00_JPRB, 0.49738E+00_JPRB ,&
&  0.58492E+00_JPRB, 0.69557E+00_JPRB, 0.75563E+00_JPRB, 0.81422E+00_JPRB, 0.84372E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.79912E+00_JPRB, 0.81294E+00_JPRB, 0.82415E+00_JPRB, 0.82712E+00_JPRB, 0.82656E+00_JPRB, 0.82155E+00_JPRB, 0.80932E+00_JPRB ,&
&  0.78953E+00_JPRB, 0.76315E+00_JPRB, 0.73735E+00_JPRB, 0.71680E+00_JPRB, 0.70376E+00_JPRB, 0.70603E+00_JPRB, 0.71393E+00_JPRB ,&
&  0.73044E+00_JPRB, 0.75517E+00_JPRB, 0.75968E+00_JPRB, 0.75223E+00_JPRB, 0.71605E+00_JPRB, 0.67833E+00_JPRB, 0.64400E+00_JPRB ,&
&  0.61311E+00_JPRB, 0.58330E+00_JPRB, 0.57225E+00_JPRB, 0.55290E+00_JPRB, 0.53926E+00_JPRB, 0.52223E+00_JPRB, 0.49734E+00_JPRB ,&
&  0.45682E+00_JPRB, 0.42691E+00_JPRB, 0.41232E+00_JPRB, 0.42175E+00_JPRB, 0.50592E+00_JPRB, 0.56717E+00_JPRB, 0.57962E+00_JPRB ,&
&  0.61334E+00_JPRB, 0.63689E+00_JPRB, 0.63147E+00_JPRB, 0.61102E+00_JPRB, 0.58900E+00_JPRB, 0.56540E+00_JPRB, 0.54527E+00_JPRB ,&
&  0.52527E+00_JPRB, 0.49879E+00_JPRB, 0.44842E+00_JPRB, 0.37702E+00_JPRB, 0.34200E+00_JPRB, 0.32878E+00_JPRB, 0.31442E+00_JPRB ,&
&  0.30368E+00_JPRB, 0.29585E+00_JPRB, 0.29255E+00_JPRB, 0.28835E+00_JPRB, 0.28784E+00_JPRB, 0.28411E+00_JPRB, 0.28054E+00_JPRB ,&
&  0.27523E+00_JPRB, 0.27126E+00_JPRB, 0.26606E+00_JPRB, 0.26102E+00_JPRB, 0.25689E+00_JPRB, 0.25630E+00_JPRB, 0.25628E+00_JPRB ,&
&  0.25758E+00_JPRB, 0.26199E+00_JPRB, 0.27499E+00_JPRB, 0.28199E+00_JPRB, 0.28261E+00_JPRB, 0.27906E+00_JPRB, 0.27176E+00_JPRB ,&
&  0.26433E+00_JPRB, 0.26291E+00_JPRB, 0.26150E+00_JPRB, 0.26220E+00_JPRB, 0.26536E+00_JPRB, 0.26933E+00_JPRB, 0.27229E+00_JPRB ,&
&  0.27465E+00_JPRB, 0.27741E+00_JPRB, 0.27984E+00_JPRB, 0.28252E+00_JPRB, 0.28590E+00_JPRB, 0.28989E+00_JPRB, 0.29538E+00_JPRB ,&
&  0.30104E+00_JPRB, 0.30914E+00_JPRB, 0.32006E+00_JPRB, 0.33604E+00_JPRB, 0.35090E+00_JPRB, 0.36084E+00_JPRB, 0.37501E+00_JPRB ,&
&  0.39187E+00_JPRB, 0.44073E+00_JPRB, 0.57656E+00_JPRB, 0.79800E+00_JPRB, 0.83052E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.71884E+00_JPRB, 0.73185E+00_JPRB, 0.74667E+00_JPRB, 0.75058E+00_JPRB, 0.76097E+00_JPRB, 0.75967E+00_JPRB, 0.74195E+00_JPRB ,&
&  0.71380E+00_JPRB, 0.68672E+00_JPRB, 0.67204E+00_JPRB, 0.65207E+00_JPRB, 0.62351E+00_JPRB, 0.56409E+00_JPRB, 0.54153E+00_JPRB ,&
&  0.52255E+00_JPRB, 0.52182E+00_JPRB, 0.51202E+00_JPRB, 0.51174E+00_JPRB, 0.50025E+00_JPRB, 0.48862E+00_JPRB, 0.46934E+00_JPRB ,&
&  0.45178E+00_JPRB, 0.43522E+00_JPRB, 0.42348E+00_JPRB, 0.41845E+00_JPRB, 0.41594E+00_JPRB, 0.41048E+00_JPRB, 0.39690E+00_JPRB ,&
&  0.38756E+00_JPRB, 0.38564E+00_JPRB, 0.38781E+00_JPRB, 0.39465E+00_JPRB, 0.43555E+00_JPRB, 0.49273E+00_JPRB, 0.49426E+00_JPRB ,&
&  0.51584E+00_JPRB, 0.52840E+00_JPRB, 0.54083E+00_JPRB, 0.52917E+00_JPRB, 0.49570E+00_JPRB, 0.45473E+00_JPRB, 0.40187E+00_JPRB ,&
&  0.35167E+00_JPRB, 0.32221E+00_JPRB, 0.29822E+00_JPRB, 0.28091E+00_JPRB, 0.26861E+00_JPRB, 0.26385E+00_JPRB, 0.25766E+00_JPRB ,&
&  0.25374E+00_JPRB, 0.25070E+00_JPRB, 0.24950E+00_JPRB, 0.24909E+00_JPRB, 0.24831E+00_JPRB, 0.24609E+00_JPRB, 0.24550E+00_JPRB ,&
&  0.24623E+00_JPRB, 0.24631E+00_JPRB, 0.24595E+00_JPRB, 0.24693E+00_JPRB, 0.24797E+00_JPRB, 0.24960E+00_JPRB, 0.25124E+00_JPRB ,&
&  0.25375E+00_JPRB, 0.25513E+00_JPRB, 0.25719E+00_JPRB, 0.25951E+00_JPRB, 0.25843E+00_JPRB, 0.25737E+00_JPRB, 0.25462E+00_JPRB ,&
&  0.25136E+00_JPRB, 0.25140E+00_JPRB, 0.25100E+00_JPRB, 0.25177E+00_JPRB, 0.25374E+00_JPRB, 0.25624E+00_JPRB, 0.25944E+00_JPRB ,&
&  0.26244E+00_JPRB, 0.26554E+00_JPRB, 0.26790E+00_JPRB, 0.27031E+00_JPRB, 0.27441E+00_JPRB, 0.27865E+00_JPRB, 0.28407E+00_JPRB ,&
&  0.28966E+00_JPRB, 0.29609E+00_JPRB, 0.30480E+00_JPRB, 0.31616E+00_JPRB, 0.33304E+00_JPRB, 0.35141E+00_JPRB, 0.38010E+00_JPRB ,&
&  0.40406E+00_JPRB, 0.50075E+00_JPRB, 0.73468E+00_JPRB, 0.66763E+00_JPRB, 0.68974E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.68127E+00_JPRB, 0.69893E+00_JPRB, 0.72332E+00_JPRB, 0.71640E+00_JPRB, 0.73614E+00_JPRB, 0.72276E+00_JPRB, 0.69918E+00_JPRB ,&
&  0.65180E+00_JPRB, 0.59613E+00_JPRB, 0.54909E+00_JPRB, 0.49706E+00_JPRB, 0.45573E+00_JPRB, 0.42623E+00_JPRB, 0.41139E+00_JPRB ,&
&  0.40334E+00_JPRB, 0.40064E+00_JPRB, 0.39813E+00_JPRB, 0.39677E+00_JPRB, 0.39088E+00_JPRB, 0.38689E+00_JPRB, 0.38058E+00_JPRB ,&
&  0.37471E+00_JPRB, 0.37139E+00_JPRB, 0.36827E+00_JPRB, 0.37325E+00_JPRB, 0.37908E+00_JPRB, 0.38492E+00_JPRB, 0.38747E+00_JPRB ,&
&  0.38936E+00_JPRB, 0.39086E+00_JPRB, 0.39396E+00_JPRB, 0.39164E+00_JPRB, 0.40312E+00_JPRB, 0.42385E+00_JPRB, 0.44065E+00_JPRB ,&
&  0.45698E+00_JPRB, 0.47035E+00_JPRB, 0.46565E+00_JPRB, 0.44115E+00_JPRB, 0.37474E+00_JPRB, 0.34071E+00_JPRB, 0.30995E+00_JPRB ,&
&  0.29016E+00_JPRB, 0.27540E+00_JPRB, 0.26522E+00_JPRB, 0.25950E+00_JPRB, 0.25452E+00_JPRB, 0.25206E+00_JPRB, 0.24942E+00_JPRB ,&
&  0.24696E+00_JPRB, 0.24566E+00_JPRB, 0.24397E+00_JPRB, 0.24311E+00_JPRB, 0.24201E+00_JPRB, 0.24053E+00_JPRB, 0.23962E+00_JPRB ,&
&  0.24004E+00_JPRB, 0.23999E+00_JPRB, 0.24137E+00_JPRB, 0.24247E+00_JPRB, 0.24454E+00_JPRB, 0.24621E+00_JPRB, 0.24864E+00_JPRB ,&
&  0.25088E+00_JPRB, 0.25278E+00_JPRB, 0.25269E+00_JPRB, 0.25238E+00_JPRB, 0.25153E+00_JPRB, 0.25114E+00_JPRB, 0.25040E+00_JPRB ,&
&  0.24927E+00_JPRB, 0.24966E+00_JPRB, 0.25024E+00_JPRB, 0.25081E+00_JPRB, 0.25228E+00_JPRB, 0.25419E+00_JPRB, 0.25689E+00_JPRB ,&
&  0.26048E+00_JPRB, 0.26416E+00_JPRB, 0.26732E+00_JPRB, 0.27078E+00_JPRB, 0.27513E+00_JPRB, 0.27995E+00_JPRB, 0.28583E+00_JPRB ,&
&  0.29059E+00_JPRB, 0.29944E+00_JPRB, 0.31203E+00_JPRB, 0.32540E+00_JPRB, 0.34764E+00_JPRB, 0.36667E+00_JPRB, 0.39849E+00_JPRB ,&
&  0.44500E+00_JPRB, 0.60991E+00_JPRB, 0.89146E+00_JPRB, 0.67191E+00_JPRB, 0.66482E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.59944E+00_JPRB, 0.60918E+00_JPRB, 0.62326E+00_JPRB, 0.63478E+00_JPRB, 0.65830E+00_JPRB, 0.67155E+00_JPRB, 0.71582E+00_JPRB ,&
&  0.67034E+00_JPRB, 0.59230E+00_JPRB, 0.56013E+00_JPRB, 0.52462E+00_JPRB, 0.48468E+00_JPRB, 0.45570E+00_JPRB, 0.43536E+00_JPRB ,&
&  0.42284E+00_JPRB, 0.41209E+00_JPRB, 0.40151E+00_JPRB, 0.38860E+00_JPRB, 0.37550E+00_JPRB, 0.36133E+00_JPRB, 0.35113E+00_JPRB ,&
&  0.34525E+00_JPRB, 0.34421E+00_JPRB, 0.34371E+00_JPRB, 0.34747E+00_JPRB, 0.35251E+00_JPRB, 0.35905E+00_JPRB, 0.36699E+00_JPRB ,&
&  0.38634E+00_JPRB, 0.39449E+00_JPRB, 0.39695E+00_JPRB, 0.39641E+00_JPRB, 0.39708E+00_JPRB, 0.39993E+00_JPRB, 0.39055E+00_JPRB ,&
&  0.39579E+00_JPRB, 0.41408E+00_JPRB, 0.41938E+00_JPRB, 0.39860E+00_JPRB, 0.34399E+00_JPRB, 0.31203E+00_JPRB, 0.28917E+00_JPRB ,&
&  0.28131E+00_JPRB, 0.27082E+00_JPRB, 0.26427E+00_JPRB, 0.25781E+00_JPRB, 0.25317E+00_JPRB, 0.25123E+00_JPRB, 0.24920E+00_JPRB ,&
&  0.24793E+00_JPRB, 0.24731E+00_JPRB, 0.24531E+00_JPRB, 0.24454E+00_JPRB, 0.24324E+00_JPRB, 0.24250E+00_JPRB, 0.24216E+00_JPRB ,&
&  0.24199E+00_JPRB, 0.24170E+00_JPRB, 0.24198E+00_JPRB, 0.24257E+00_JPRB, 0.24354E+00_JPRB, 0.24430E+00_JPRB, 0.24502E+00_JPRB ,&
&  0.24610E+00_JPRB, 0.24712E+00_JPRB, 0.24747E+00_JPRB, 0.24818E+00_JPRB, 0.24878E+00_JPRB, 0.25023E+00_JPRB, 0.25095E+00_JPRB ,&
&  0.25168E+00_JPRB, 0.25235E+00_JPRB, 0.25246E+00_JPRB, 0.25331E+00_JPRB, 0.25445E+00_JPRB, 0.25660E+00_JPRB, 0.25896E+00_JPRB ,&
&  0.26195E+00_JPRB, 0.26553E+00_JPRB, 0.26939E+00_JPRB, 0.27329E+00_JPRB, 0.27829E+00_JPRB, 0.28344E+00_JPRB, 0.29037E+00_JPRB ,&
&  0.29781E+00_JPRB, 0.30790E+00_JPRB, 0.32204E+00_JPRB, 0.33877E+00_JPRB, 0.35952E+00_JPRB, 0.37804E+00_JPRB, 0.39734E+00_JPRB ,&
&  0.43046E+00_JPRB, 0.52645E+00_JPRB, 0.58662E+00_JPRB, 0.57922E+00_JPRB, 0.59203E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.57774E+00_JPRB, 0.58084E+00_JPRB, 0.58406E+00_JPRB, 0.58873E+00_JPRB, 0.59380E+00_JPRB, 0.59846E+00_JPRB, 0.60216E+00_JPRB ,&
&  0.58353E+00_JPRB, 0.56041E+00_JPRB, 0.54121E+00_JPRB, 0.51829E+00_JPRB, 0.49732E+00_JPRB, 0.47921E+00_JPRB, 0.45888E+00_JPRB ,&
&  0.43843E+00_JPRB, 0.41811E+00_JPRB, 0.39532E+00_JPRB, 0.37656E+00_JPRB, 0.35817E+00_JPRB, 0.34329E+00_JPRB, 0.33477E+00_JPRB ,&
&  0.32980E+00_JPRB, 0.32830E+00_JPRB, 0.32947E+00_JPRB, 0.33149E+00_JPRB, 0.33365E+00_JPRB, 0.33796E+00_JPRB, 0.34420E+00_JPRB ,&
&  0.35288E+00_JPRB, 0.36843E+00_JPRB, 0.38223E+00_JPRB, 0.38691E+00_JPRB, 0.38272E+00_JPRB, 0.37369E+00_JPRB, 0.36579E+00_JPRB ,&
&  0.35644E+00_JPRB, 0.34441E+00_JPRB, 0.32928E+00_JPRB, 0.31421E+00_JPRB, 0.30017E+00_JPRB, 0.29000E+00_JPRB, 0.28122E+00_JPRB ,&
&  0.27365E+00_JPRB, 0.26690E+00_JPRB, 0.26158E+00_JPRB, 0.25804E+00_JPRB, 0.25500E+00_JPRB, 0.25263E+00_JPRB, 0.25028E+00_JPRB ,&
&  0.24925E+00_JPRB, 0.24819E+00_JPRB, 0.24756E+00_JPRB, 0.24723E+00_JPRB, 0.24667E+00_JPRB, 0.24648E+00_JPRB, 0.24635E+00_JPRB ,&
&  0.24598E+00_JPRB, 0.24532E+00_JPRB, 0.24496E+00_JPRB, 0.24473E+00_JPRB, 0.24526E+00_JPRB, 0.24572E+00_JPRB, 0.24644E+00_JPRB ,&
&  0.24750E+00_JPRB, 0.24957E+00_JPRB, 0.25082E+00_JPRB, 0.25201E+00_JPRB, 0.25263E+00_JPRB, 0.25323E+00_JPRB, 0.25400E+00_JPRB ,&
&  0.25524E+00_JPRB, 0.25669E+00_JPRB, 0.25794E+00_JPRB, 0.25964E+00_JPRB, 0.26136E+00_JPRB, 0.26373E+00_JPRB, 0.26632E+00_JPRB ,&
&  0.26916E+00_JPRB, 0.27267E+00_JPRB, 0.27695E+00_JPRB, 0.28076E+00_JPRB, 0.28692E+00_JPRB, 0.29315E+00_JPRB, 0.30022E+00_JPRB ,&
&  0.30766E+00_JPRB, 0.31573E+00_JPRB, 0.32652E+00_JPRB, 0.33944E+00_JPRB, 0.35584E+00_JPRB, 0.36344E+00_JPRB, 0.37352E+00_JPRB ,&
&  0.38922E+00_JPRB, 0.42213E+00_JPRB, 0.55916E+00_JPRB, 0.57284E+00_JPRB, 0.57807E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.35711E+00_JPRB, 0.36290E+00_JPRB, 0.36673E+00_JPRB, 0.37556E+00_JPRB, 0.38612E+00_JPRB, 0.48522E+00_JPRB, 0.48025E+00_JPRB ,&
&  0.47963E+00_JPRB, 0.43634E+00_JPRB, 0.42870E+00_JPRB, 0.41667E+00_JPRB, 0.40137E+00_JPRB, 0.38250E+00_JPRB, 0.37086E+00_JPRB ,&
&  0.36121E+00_JPRB, 0.34981E+00_JPRB, 0.33930E+00_JPRB, 0.33302E+00_JPRB, 0.32805E+00_JPRB, 0.32353E+00_JPRB, 0.32071E+00_JPRB ,&
&  0.31927E+00_JPRB, 0.31925E+00_JPRB, 0.31955E+00_JPRB, 0.31973E+00_JPRB, 0.32277E+00_JPRB, 0.32623E+00_JPRB, 0.32977E+00_JPRB ,&
&  0.33114E+00_JPRB, 0.33571E+00_JPRB, 0.33684E+00_JPRB, 0.33380E+00_JPRB, 0.32900E+00_JPRB, 0.32160E+00_JPRB, 0.31414E+00_JPRB ,&
&  0.30646E+00_JPRB, 0.29945E+00_JPRB, 0.29361E+00_JPRB, 0.28837E+00_JPRB, 0.28231E+00_JPRB, 0.27640E+00_JPRB, 0.27249E+00_JPRB ,&
&  0.26888E+00_JPRB, 0.26515E+00_JPRB, 0.26214E+00_JPRB, 0.26066E+00_JPRB, 0.25900E+00_JPRB, 0.25787E+00_JPRB, 0.25699E+00_JPRB ,&
&  0.25676E+00_JPRB, 0.25662E+00_JPRB, 0.25668E+00_JPRB, 0.25695E+00_JPRB, 0.25692E+00_JPRB, 0.25683E+00_JPRB, 0.25678E+00_JPRB ,&
&  0.25740E+00_JPRB, 0.25746E+00_JPRB, 0.25742E+00_JPRB, 0.25724E+00_JPRB, 0.25806E+00_JPRB, 0.25841E+00_JPRB, 0.25893E+00_JPRB ,&
&  0.25965E+00_JPRB, 0.26059E+00_JPRB, 0.26128E+00_JPRB, 0.26203E+00_JPRB, 0.26277E+00_JPRB, 0.26369E+00_JPRB, 0.26476E+00_JPRB ,&
&  0.26586E+00_JPRB, 0.26748E+00_JPRB, 0.26914E+00_JPRB, 0.27058E+00_JPRB, 0.27188E+00_JPRB, 0.27365E+00_JPRB, 0.27564E+00_JPRB ,&
&  0.27855E+00_JPRB, 0.28136E+00_JPRB, 0.28421E+00_JPRB, 0.28575E+00_JPRB, 0.28845E+00_JPRB, 0.29073E+00_JPRB, 0.29324E+00_JPRB ,&
&  0.29625E+00_JPRB, 0.30127E+00_JPRB, 0.30878E+00_JPRB, 0.31716E+00_JPRB, 0.32672E+00_JPRB, 0.33386E+00_JPRB, 0.33845E+00_JPRB ,&
&  0.34223E+00_JPRB, 0.34646E+00_JPRB, 0.35141E+00_JPRB, 0.35402E+00_JPRB, 0.35601E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.32266E+00_JPRB, 0.32259E+00_JPRB, 0.32265E+00_JPRB, 0.32226E+00_JPRB, 0.32054E+00_JPRB, 0.31913E+00_JPRB, 0.31783E+00_JPRB ,&
&  0.31702E+00_JPRB, 0.31597E+00_JPRB, 0.31545E+00_JPRB, 0.31515E+00_JPRB, 0.31463E+00_JPRB, 0.31393E+00_JPRB, 0.31301E+00_JPRB ,&
&  0.31180E+00_JPRB, 0.31024E+00_JPRB, 0.30848E+00_JPRB, 0.30794E+00_JPRB, 0.30776E+00_JPRB, 0.30759E+00_JPRB, 0.30734E+00_JPRB ,&
&  0.30696E+00_JPRB, 0.30663E+00_JPRB, 0.30591E+00_JPRB, 0.30600E+00_JPRB, 0.30594E+00_JPRB, 0.30589E+00_JPRB, 0.30595E+00_JPRB ,&
&  0.30609E+00_JPRB, 0.30634E+00_JPRB, 0.30617E+00_JPRB, 0.30573E+00_JPRB, 0.30489E+00_JPRB, 0.30319E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.29742E+00_JPRB, 0.29435E+00_JPRB, 0.29190E+00_JPRB, 0.28965E+00_JPRB, 0.28759E+00_JPRB, 0.28572E+00_JPRB, 0.28419E+00_JPRB ,&
&  0.28232E+00_JPRB, 0.28047E+00_JPRB, 0.27876E+00_JPRB, 0.27751E+00_JPRB, 0.27612E+00_JPRB, 0.27489E+00_JPRB, 0.27371E+00_JPRB ,&
&  0.27289E+00_JPRB, 0.27218E+00_JPRB, 0.27157E+00_JPRB, 0.27125E+00_JPRB, 0.27100E+00_JPRB, 0.27078E+00_JPRB, 0.27053E+00_JPRB ,&
&  0.27031E+00_JPRB, 0.27019E+00_JPRB, 0.27011E+00_JPRB, 0.27013E+00_JPRB, 0.27030E+00_JPRB, 0.27025E+00_JPRB, 0.27016E+00_JPRB ,&
&  0.27017E+00_JPRB, 0.27053E+00_JPRB, 0.27076E+00_JPRB, 0.27109E+00_JPRB, 0.27158E+00_JPRB, 0.27227E+00_JPRB, 0.27301E+00_JPRB ,&
&  0.27376E+00_JPRB, 0.27463E+00_JPRB, 0.27555E+00_JPRB, 0.27671E+00_JPRB, 0.27796E+00_JPRB, 0.27921E+00_JPRB, 0.28041E+00_JPRB ,&
&  0.28169E+00_JPRB, 0.28288E+00_JPRB, 0.28400E+00_JPRB, 0.28477E+00_JPRB, 0.28592E+00_JPRB, 0.28712E+00_JPRB, 0.28854E+00_JPRB ,&
&  0.29017E+00_JPRB, 0.29235E+00_JPRB, 0.29547E+00_JPRB, 0.29919E+00_JPRB, 0.30402E+00_JPRB, 0.30747E+00_JPRB, 0.31089E+00_JPRB ,&
&  0.31326E+00_JPRB, 0.31517E+00_JPRB, 0.31663E+00_JPRB, 0.31846E+00_JPRB, 0.32047E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB, 0.28860E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_A1B2060:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_A1B2060
